var h7 = Object.defineProperty;
var Cy = (O) => {
  throw TypeError(O);
};
var p7 = (O, _, H) => _ in O ? h7(O, _, { enumerable: !0, configurable: !0, writable: !0, value: H }) : O[_] = H;
var m7 = (O, _) => () => (_ || O((_ = { exports: {} }).exports, _), _.exports);
var mt = (O, _, H) => p7(O, typeof _ != "symbol" ? _ + "" : _, H), g7 = (O, _, H) => _.has(O) || Cy("Cannot " + H);
var Ly = (O, _, H) => _.has(O) ? Cy("Cannot add the same private member more than once") : _ instanceof WeakSet ? _.add(O) : _.set(O, H);
var ky = (O, _, H) => (g7(O, _, "access private method"), H);
var _7 = m7((exports, module) => {
  /**
  * @vue/shared v3.5.13
  * (c) 2018-present Yuxi (Evan) You and Vue contributors
  * @license MIT
  **/
  /*! #__NO_SIDE_EFFECTS__ */
  // @__NO_SIDE_EFFECTS__
  function makeMap(O) {
    const _ = /* @__PURE__ */ Object.create(null);
    for (const H of O.split(",")) _[H] = 1;
    return (H) => H in _;
  }
  const EMPTY_OBJ$1 = {}, EMPTY_ARR = [], NOOP = () => {
  }, NO = () => !1, isOn = (O) => O.charCodeAt(0) === 111 && O.charCodeAt(1) === 110 && // uppercase letter
  (O.charCodeAt(2) > 122 || O.charCodeAt(2) < 97), isModelListener = (O) => O.startsWith("onUpdate:"), extend$a = Object.assign, remove$4 = (O, _) => {
    const H = O.indexOf(_);
    H > -1 && O.splice(H, 1);
  }, hasOwnProperty$f = Object.prototype.hasOwnProperty, hasOwn = (O, _) => hasOwnProperty$f.call(O, _), isArray$6 = Array.isArray, isMap = (O) => toTypeString(O) === "[object Map]", isSet$1 = (O) => toTypeString(O) === "[object Set]", isFunction$6 = (O) => typeof O == "function", isString$5 = (O) => typeof O == "string", isSymbol$2 = (O) => typeof O == "symbol", isObject$7 = (O) => O !== null && typeof O == "object", isPromise$1 = (O) => (isObject$7(O) || isFunction$6(O)) && isFunction$6(O.then) && isFunction$6(O.catch), objectToString$1 = Object.prototype.toString, toTypeString = (O) => objectToString$1.call(O), toRawType = (O) => toTypeString(O).slice(8, -1), isPlainObject$3 = (O) => toTypeString(O) === "[object Object]", isIntegerKey = (O) => isString$5(O) && O !== "NaN" && O[0] !== "-" && "" + parseInt(O, 10) === O, isReservedProp = /* @__PURE__ */ makeMap(
    // the leading comma is intentional so empty string "" is also included
    ",key,ref,ref_for,ref_key,onVnodeBeforeMount,onVnodeMounted,onVnodeBeforeUpdate,onVnodeUpdated,onVnodeBeforeUnmount,onVnodeUnmounted"
  ), cacheStringFunction = (O) => {
    const _ = /* @__PURE__ */ Object.create(null);
    return (H) => _[H] || (_[H] = O(H));
  }, camelizeRE = /-(\w)/g, camelize = cacheStringFunction(
    (O) => O.replace(camelizeRE, (_, H) => H ? H.toUpperCase() : "")
  ), hyphenateRE = /\B([A-Z])/g, hyphenate$1 = cacheStringFunction(
    (O) => O.replace(hyphenateRE, "-$1").toLowerCase()
  ), capitalize$1 = cacheStringFunction((O) => O.charAt(0).toUpperCase() + O.slice(1)), toHandlerKey = cacheStringFunction(
    (O) => O ? `on${capitalize$1(O)}` : ""
  ), hasChanged = (O, _) => !Object.is(O, _), invokeArrayFns = (O, ..._) => {
    for (let H = 0; H < O.length; H++)
      O[H](..._);
  }, def$2 = (O, _, H, V = !1) => {
    Object.defineProperty(O, _, {
      configurable: !0,
      enumerable: !1,
      writable: V,
      value: H
    });
  }, looseToNumber = (O) => {
    const _ = parseFloat(O);
    return isNaN(_) ? O : _;
  }, toNumber$2 = (O) => {
    const _ = isString$5(O) ? Number(O) : NaN;
    return isNaN(_) ? O : _;
  };
  let _globalThis;
  const getGlobalThis = () => _globalThis || (_globalThis = typeof globalThis < "u" ? globalThis : typeof self < "u" ? self : typeof window < "u" ? window : typeof global < "u" ? global : {});
  function normalizeStyle(O) {
    if (isArray$6(O)) {
      const _ = {};
      for (let H = 0; H < O.length; H++) {
        const V = O[H], q = isString$5(V) ? parseStringStyle(V) : normalizeStyle(V);
        if (q)
          for (const G in q)
            _[G] = q[G];
      }
      return _;
    } else if (isString$5(O) || isObject$7(O))
      return O;
  }
  const listDelimiterRE = /;(?![^(]*\))/g, propertyDelimiterRE = /:([^]+)/, styleCommentRE = /\/\*[^]*?\*\//g;
  function parseStringStyle(O) {
    const _ = {};
    return O.replace(styleCommentRE, "").split(listDelimiterRE).forEach((H) => {
      if (H) {
        const V = H.split(propertyDelimiterRE);
        V.length > 1 && (_[V[0].trim()] = V[1].trim());
      }
    }), _;
  }
  function normalizeClass(O) {
    let _ = "";
    if (isString$5(O))
      _ = O;
    else if (isArray$6(O))
      for (let H = 0; H < O.length; H++) {
        const V = normalizeClass(O[H]);
        V && (_ += V + " ");
      }
    else if (isObject$7(O))
      for (const H in O)
        O[H] && (_ += H + " ");
    return _.trim();
  }
  const specialBooleanAttrs = "itemscope,allowfullscreen,formnovalidate,ismap,nomodule,novalidate,readonly", isSpecialBooleanAttr = /* @__PURE__ */ makeMap(specialBooleanAttrs);
  function includeBooleanAttr(O) {
    return !!O || O === "";
  }
  const isRef$1 = (O) => !!(O && O.__v_isRef === !0), toDisplayString = (O) => isString$5(O) ? O : O == null ? "" : isArray$6(O) || isObject$7(O) && (O.toString === objectToString$1 || !isFunction$6(O.toString)) ? isRef$1(O) ? toDisplayString(O.value) : JSON.stringify(O, replacer$1, 2) : String(O), replacer$1 = (O, _) => isRef$1(_) ? replacer$1(O, _.value) : isMap(_) ? {
    [`Map(${_.size})`]: [..._.entries()].reduce(
      (H, [V, q], G) => (H[stringifySymbol(V, G) + " =>"] = q, H),
      {}
    )
  } : isSet$1(_) ? {
    [`Set(${_.size})`]: [..._.values()].map((H) => stringifySymbol(H))
  } : isSymbol$2(_) ? stringifySymbol(_) : isObject$7(_) && !isArray$6(_) && !isPlainObject$3(_) ? String(_) : _, stringifySymbol = (O, _ = "") => {
    var H;
    return (
      // Symbol.description in es2019+ so we need to cast here to pass
      // the lib: es2016 check
      isSymbol$2(O) ? `Symbol(${(H = O.description) != null ? H : _})` : O
    );
  };
  /**
  * @vue/reactivity v3.5.13
  * (c) 2018-present Yuxi (Evan) You and Vue contributors
  * @license MIT
  **/
  let activeEffectScope;
  class EffectScope {
    constructor(_ = !1) {
      this.detached = _, this._active = !0, this.effects = [], this.cleanups = [], this._isPaused = !1, this.parent = activeEffectScope, !_ && activeEffectScope && (this.index = (activeEffectScope.scopes || (activeEffectScope.scopes = [])).push(
        this
      ) - 1);
    }
    get active() {
      return this._active;
    }
    pause() {
      if (this._active) {
        this._isPaused = !0;
        let _, H;
        if (this.scopes)
          for (_ = 0, H = this.scopes.length; _ < H; _++)
            this.scopes[_].pause();
        for (_ = 0, H = this.effects.length; _ < H; _++)
          this.effects[_].pause();
      }
    }
    /**
     * Resumes the effect scope, including all child scopes and effects.
     */
    resume() {
      if (this._active && this._isPaused) {
        this._isPaused = !1;
        let _, H;
        if (this.scopes)
          for (_ = 0, H = this.scopes.length; _ < H; _++)
            this.scopes[_].resume();
        for (_ = 0, H = this.effects.length; _ < H; _++)
          this.effects[_].resume();
      }
    }
    run(_) {
      if (this._active) {
        const H = activeEffectScope;
        try {
          return activeEffectScope = this, _();
        } finally {
          activeEffectScope = H;
        }
      }
    }
    /**
     * This should only be called on non-detached scopes
     * @internal
     */
    on() {
      activeEffectScope = this;
    }
    /**
     * This should only be called on non-detached scopes
     * @internal
     */
    off() {
      activeEffectScope = this.parent;
    }
    stop(_) {
      if (this._active) {
        this._active = !1;
        let H, V;
        for (H = 0, V = this.effects.length; H < V; H++)
          this.effects[H].stop();
        for (this.effects.length = 0, H = 0, V = this.cleanups.length; H < V; H++)
          this.cleanups[H]();
        if (this.cleanups.length = 0, this.scopes) {
          for (H = 0, V = this.scopes.length; H < V; H++)
            this.scopes[H].stop(!0);
          this.scopes.length = 0;
        }
        if (!this.detached && this.parent && !_) {
          const q = this.parent.scopes.pop();
          q && q !== this && (this.parent.scopes[this.index] = q, q.index = this.index);
        }
        this.parent = void 0;
      }
    }
  }
  function effectScope(O) {
    return new EffectScope(O);
  }
  function getCurrentScope() {
    return activeEffectScope;
  }
  function onScopeDispose(O, _ = !1) {
    activeEffectScope && activeEffectScope.cleanups.push(O);
  }
  let activeSub;
  const pausedQueueEffects = /* @__PURE__ */ new WeakSet();
  class ReactiveEffect {
    constructor(_) {
      this.fn = _, this.deps = void 0, this.depsTail = void 0, this.flags = 5, this.next = void 0, this.cleanup = void 0, this.scheduler = void 0, activeEffectScope && activeEffectScope.active && activeEffectScope.effects.push(this);
    }
    pause() {
      this.flags |= 64;
    }
    resume() {
      this.flags & 64 && (this.flags &= -65, pausedQueueEffects.has(this) && (pausedQueueEffects.delete(this), this.trigger()));
    }
    /**
     * @internal
     */
    notify() {
      this.flags & 2 && !(this.flags & 32) || this.flags & 8 || batch(this);
    }
    run() {
      if (!(this.flags & 1))
        return this.fn();
      this.flags |= 2, cleanupEffect(this), prepareDeps(this);
      const _ = activeSub, H = shouldTrack;
      activeSub = this, shouldTrack = !0;
      try {
        return this.fn();
      } finally {
        cleanupDeps(this), activeSub = _, shouldTrack = H, this.flags &= -3;
      }
    }
    stop() {
      if (this.flags & 1) {
        for (let _ = this.deps; _; _ = _.nextDep)
          removeSub(_);
        this.deps = this.depsTail = void 0, cleanupEffect(this), this.onStop && this.onStop(), this.flags &= -2;
      }
    }
    trigger() {
      this.flags & 64 ? pausedQueueEffects.add(this) : this.scheduler ? this.scheduler() : this.runIfDirty();
    }
    /**
     * @internal
     */
    runIfDirty() {
      isDirty(this) && this.run();
    }
    get dirty() {
      return isDirty(this);
    }
  }
  let batchDepth = 0, batchedSub, batchedComputed;
  function batch(O, _ = !1) {
    if (O.flags |= 8, _) {
      O.next = batchedComputed, batchedComputed = O;
      return;
    }
    O.next = batchedSub, batchedSub = O;
  }
  function startBatch() {
    batchDepth++;
  }
  function endBatch() {
    if (--batchDepth > 0)
      return;
    if (batchedComputed) {
      let _ = batchedComputed;
      for (batchedComputed = void 0; _; ) {
        const H = _.next;
        _.next = void 0, _.flags &= -9, _ = H;
      }
    }
    let O;
    for (; batchedSub; ) {
      let _ = batchedSub;
      for (batchedSub = void 0; _; ) {
        const H = _.next;
        if (_.next = void 0, _.flags &= -9, _.flags & 1)
          try {
            _.trigger();
          } catch (V) {
            O || (O = V);
          }
        _ = H;
      }
    }
    if (O) throw O;
  }
  function prepareDeps(O) {
    for (let _ = O.deps; _; _ = _.nextDep)
      _.version = -1, _.prevActiveLink = _.dep.activeLink, _.dep.activeLink = _;
  }
  function cleanupDeps(O) {
    let _, H = O.depsTail, V = H;
    for (; V; ) {
      const q = V.prevDep;
      V.version === -1 ? (V === H && (H = q), removeSub(V), removeDep(V)) : _ = V, V.dep.activeLink = V.prevActiveLink, V.prevActiveLink = void 0, V = q;
    }
    O.deps = _, O.depsTail = H;
  }
  function isDirty(O) {
    for (let _ = O.deps; _; _ = _.nextDep)
      if (_.dep.version !== _.version || _.dep.computed && (refreshComputed(_.dep.computed) || _.dep.version !== _.version))
        return !0;
    return !!O._dirty;
  }
  function refreshComputed(O) {
    if (O.flags & 4 && !(O.flags & 16) || (O.flags &= -17, O.globalVersion === globalVersion))
      return;
    O.globalVersion = globalVersion;
    const _ = O.dep;
    if (O.flags |= 2, _.version > 0 && !O.isSSR && O.deps && !isDirty(O)) {
      O.flags &= -3;
      return;
    }
    const H = activeSub, V = shouldTrack;
    activeSub = O, shouldTrack = !0;
    try {
      prepareDeps(O);
      const q = O.fn(O._value);
      (_.version === 0 || hasChanged(q, O._value)) && (O._value = q, _.version++);
    } catch (q) {
      throw _.version++, q;
    } finally {
      activeSub = H, shouldTrack = V, cleanupDeps(O), O.flags &= -3;
    }
  }
  function removeSub(O, _ = !1) {
    const { dep: H, prevSub: V, nextSub: q } = O;
    if (V && (V.nextSub = q, O.prevSub = void 0), q && (q.prevSub = V, O.nextSub = void 0), H.subs === O && (H.subs = V, !V && H.computed)) {
      H.computed.flags &= -5;
      for (let G = H.computed.deps; G; G = G.nextDep)
        removeSub(G, !0);
    }
    !_ && !--H.sc && H.map && H.map.delete(H.key);
  }
  function removeDep(O) {
    const { prevDep: _, nextDep: H } = O;
    _ && (_.nextDep = H, O.prevDep = void 0), H && (H.prevDep = _, O.nextDep = void 0);
  }
  let shouldTrack = !0;
  const trackStack = [];
  function pauseTracking() {
    trackStack.push(shouldTrack), shouldTrack = !1;
  }
  function resetTracking() {
    const O = trackStack.pop();
    shouldTrack = O === void 0 ? !0 : O;
  }
  function cleanupEffect(O) {
    const { cleanup: _ } = O;
    if (O.cleanup = void 0, _) {
      const H = activeSub;
      activeSub = void 0;
      try {
        _();
      } finally {
        activeSub = H;
      }
    }
  }
  let globalVersion = 0;
  class Link {
    constructor(_, H) {
      this.sub = _, this.dep = H, this.version = H.version, this.nextDep = this.prevDep = this.nextSub = this.prevSub = this.prevActiveLink = void 0;
    }
  }
  class Dep {
    constructor(_) {
      this.computed = _, this.version = 0, this.activeLink = void 0, this.subs = void 0, this.map = void 0, this.key = void 0, this.sc = 0;
    }
    track(_) {
      if (!activeSub || !shouldTrack || activeSub === this.computed)
        return;
      let H = this.activeLink;
      if (H === void 0 || H.sub !== activeSub)
        H = this.activeLink = new Link(activeSub, this), activeSub.deps ? (H.prevDep = activeSub.depsTail, activeSub.depsTail.nextDep = H, activeSub.depsTail = H) : activeSub.deps = activeSub.depsTail = H, addSub(H);
      else if (H.version === -1 && (H.version = this.version, H.nextDep)) {
        const V = H.nextDep;
        V.prevDep = H.prevDep, H.prevDep && (H.prevDep.nextDep = V), H.prevDep = activeSub.depsTail, H.nextDep = void 0, activeSub.depsTail.nextDep = H, activeSub.depsTail = H, activeSub.deps === H && (activeSub.deps = V);
      }
      return H;
    }
    trigger(_) {
      this.version++, globalVersion++, this.notify(_);
    }
    notify(_) {
      startBatch();
      try {
        for (let H = this.subs; H; H = H.prevSub)
          H.sub.notify() && H.sub.dep.notify();
      } finally {
        endBatch();
      }
    }
  }
  function addSub(O) {
    if (O.dep.sc++, O.sub.flags & 4) {
      const _ = O.dep.computed;
      if (_ && !O.dep.subs) {
        _.flags |= 20;
        for (let V = _.deps; V; V = V.nextDep)
          addSub(V);
      }
      const H = O.dep.subs;
      H !== O && (O.prevSub = H, H && (H.nextSub = O)), O.dep.subs = O;
    }
  }
  const targetMap = /* @__PURE__ */ new WeakMap(), ITERATE_KEY = Symbol(
    ""
  ), MAP_KEY_ITERATE_KEY = Symbol(
    ""
  ), ARRAY_ITERATE_KEY = Symbol(
    ""
  );
  function track(O, _, H) {
    if (shouldTrack && activeSub) {
      let V = targetMap.get(O);
      V || targetMap.set(O, V = /* @__PURE__ */ new Map());
      let q = V.get(H);
      q || (V.set(H, q = new Dep()), q.map = V, q.key = H), q.track();
    }
  }
  function trigger$1(O, _, H, V, q, G) {
    const J = targetMap.get(O);
    if (!J) {
      globalVersion++;
      return;
    }
    const Z = (te) => {
      te && te.trigger();
    };
    if (startBatch(), _ === "clear")
      J.forEach(Z);
    else {
      const te = isArray$6(O), ne = te && isIntegerKey(H);
      if (te && H === "length") {
        const ee = Number(V);
        J.forEach((ae, oe) => {
          (oe === "length" || oe === ARRAY_ITERATE_KEY || !isSymbol$2(oe) && oe >= ee) && Z(ae);
        });
      } else
        switch ((H !== void 0 || J.has(void 0)) && Z(J.get(H)), ne && Z(J.get(ARRAY_ITERATE_KEY)), _) {
          case "add":
            te ? ne && Z(J.get("length")) : (Z(J.get(ITERATE_KEY)), isMap(O) && Z(J.get(MAP_KEY_ITERATE_KEY)));
            break;
          case "delete":
            te || (Z(J.get(ITERATE_KEY)), isMap(O) && Z(J.get(MAP_KEY_ITERATE_KEY)));
            break;
          case "set":
            isMap(O) && Z(J.get(ITERATE_KEY));
            break;
        }
    }
    endBatch();
  }
  function getDepFromReactive(O, _) {
    const H = targetMap.get(O);
    return H && H.get(_);
  }
  function reactiveReadArray(O) {
    const _ = toRaw(O);
    return _ === O ? _ : (track(_, "iterate", ARRAY_ITERATE_KEY), isShallow(O) ? _ : _.map(toReactive));
  }
  function shallowReadArray(O) {
    return track(O = toRaw(O), "iterate", ARRAY_ITERATE_KEY), O;
  }
  const arrayInstrumentations = {
    __proto__: null,
    [Symbol.iterator]() {
      return iterator(this, Symbol.iterator, toReactive);
    },
    concat(...O) {
      return reactiveReadArray(this).concat(
        ...O.map((_) => isArray$6(_) ? reactiveReadArray(_) : _)
      );
    },
    entries() {
      return iterator(this, "entries", (O) => (O[1] = toReactive(O[1]), O));
    },
    every(O, _) {
      return apply$2(this, "every", O, _, void 0, arguments);
    },
    filter(O, _) {
      return apply$2(this, "filter", O, _, (H) => H.map(toReactive), arguments);
    },
    find(O, _) {
      return apply$2(this, "find", O, _, toReactive, arguments);
    },
    findIndex(O, _) {
      return apply$2(this, "findIndex", O, _, void 0, arguments);
    },
    findLast(O, _) {
      return apply$2(this, "findLast", O, _, toReactive, arguments);
    },
    findLastIndex(O, _) {
      return apply$2(this, "findLastIndex", O, _, void 0, arguments);
    },
    // flat, flatMap could benefit from ARRAY_ITERATE but are not straight-forward to implement
    forEach(O, _) {
      return apply$2(this, "forEach", O, _, void 0, arguments);
    },
    includes(...O) {
      return searchProxy(this, "includes", O);
    },
    indexOf(...O) {
      return searchProxy(this, "indexOf", O);
    },
    join(O) {
      return reactiveReadArray(this).join(O);
    },
    // keys() iterator only reads `length`, no optimisation required
    lastIndexOf(...O) {
      return searchProxy(this, "lastIndexOf", O);
    },
    map(O, _) {
      return apply$2(this, "map", O, _, void 0, arguments);
    },
    pop() {
      return noTracking(this, "pop");
    },
    push(...O) {
      return noTracking(this, "push", O);
    },
    reduce(O, ..._) {
      return reduce$4(this, "reduce", O, _);
    },
    reduceRight(O, ..._) {
      return reduce$4(this, "reduceRight", O, _);
    },
    shift() {
      return noTracking(this, "shift");
    },
    // slice could use ARRAY_ITERATE but also seems to beg for range tracking
    some(O, _) {
      return apply$2(this, "some", O, _, void 0, arguments);
    },
    splice(...O) {
      return noTracking(this, "splice", O);
    },
    toReversed() {
      return reactiveReadArray(this).toReversed();
    },
    toSorted(O) {
      return reactiveReadArray(this).toSorted(O);
    },
    toSpliced(...O) {
      return reactiveReadArray(this).toSpliced(...O);
    },
    unshift(...O) {
      return noTracking(this, "unshift", O);
    },
    values() {
      return iterator(this, "values", toReactive);
    }
  };
  function iterator(O, _, H) {
    const V = shallowReadArray(O), q = V[_]();
    return V !== O && !isShallow(O) && (q._next = q.next, q.next = () => {
      const G = q._next();
      return G.value && (G.value = H(G.value)), G;
    }), q;
  }
  const arrayProto$1 = Array.prototype;
  function apply$2(O, _, H, V, q, G) {
    const J = shallowReadArray(O), Z = J !== O && !isShallow(O), te = J[_];
    if (te !== arrayProto$1[_]) {
      const ae = te.apply(O, G);
      return Z ? toReactive(ae) : ae;
    }
    let ne = H;
    J !== O && (Z ? ne = function(ae, oe) {
      return H.call(this, toReactive(ae), oe, O);
    } : H.length > 2 && (ne = function(ae, oe) {
      return H.call(this, ae, oe, O);
    }));
    const ee = te.call(J, ne, V);
    return Z && q ? q(ee) : ee;
  }
  function reduce$4(O, _, H, V) {
    const q = shallowReadArray(O);
    let G = H;
    return q !== O && (isShallow(O) ? H.length > 3 && (G = function(J, Z, te) {
      return H.call(this, J, Z, te, O);
    }) : G = function(J, Z, te) {
      return H.call(this, J, toReactive(Z), te, O);
    }), q[_](G, ...V);
  }
  function searchProxy(O, _, H) {
    const V = toRaw(O);
    track(V, "iterate", ARRAY_ITERATE_KEY);
    const q = V[_](...H);
    return (q === -1 || q === !1) && isProxy(H[0]) ? (H[0] = toRaw(H[0]), V[_](...H)) : q;
  }
  function noTracking(O, _, H = []) {
    pauseTracking(), startBatch();
    const V = toRaw(O)[_].apply(O, H);
    return endBatch(), resetTracking(), V;
  }
  const isNonTrackableKeys = /* @__PURE__ */ makeMap("__proto__,__v_isRef,__isVue"), builtInSymbols = new Set(
    /* @__PURE__ */ Object.getOwnPropertyNames(Symbol).filter((O) => O !== "arguments" && O !== "caller").map((O) => Symbol[O]).filter(isSymbol$2)
  );
  function hasOwnProperty$e(O) {
    isSymbol$2(O) || (O = String(O));
    const _ = toRaw(this);
    return track(_, "has", O), _.hasOwnProperty(O);
  }
  class BaseReactiveHandler {
    constructor(_ = !1, H = !1) {
      this._isReadonly = _, this._isShallow = H;
    }
    get(_, H, V) {
      if (H === "__v_skip") return _.__v_skip;
      const q = this._isReadonly, G = this._isShallow;
      if (H === "__v_isReactive")
        return !q;
      if (H === "__v_isReadonly")
        return q;
      if (H === "__v_isShallow")
        return G;
      if (H === "__v_raw")
        return V === (q ? G ? shallowReadonlyMap : readonlyMap : G ? shallowReactiveMap : reactiveMap).get(_) || // receiver is not the reactive proxy, but has the same prototype
        // this means the receiver is a user proxy of the reactive proxy
        Object.getPrototypeOf(_) === Object.getPrototypeOf(V) ? _ : void 0;
      const J = isArray$6(_);
      if (!q) {
        let te;
        if (J && (te = arrayInstrumentations[H]))
          return te;
        if (H === "hasOwnProperty")
          return hasOwnProperty$e;
      }
      const Z = Reflect.get(
        _,
        H,
        // if this is a proxy wrapping a ref, return methods using the raw ref
        // as receiver so that we don't have to call `toRaw` on the ref in all
        // its class methods
        isRef(_) ? _ : V
      );
      return (isSymbol$2(H) ? builtInSymbols.has(H) : isNonTrackableKeys(H)) || (q || track(_, "get", H), G) ? Z : isRef(Z) ? J && isIntegerKey(H) ? Z : Z.value : isObject$7(Z) ? q ? readonly(Z) : reactive(Z) : Z;
    }
  }
  class MutableReactiveHandler extends BaseReactiveHandler {
    constructor(_ = !1) {
      super(!1, _);
    }
    set(_, H, V, q) {
      let G = _[H];
      if (!this._isShallow) {
        const te = isReadonly(G);
        if (!isShallow(V) && !isReadonly(V) && (G = toRaw(G), V = toRaw(V)), !isArray$6(_) && isRef(G) && !isRef(V))
          return te ? !1 : (G.value = V, !0);
      }
      const J = isArray$6(_) && isIntegerKey(H) ? Number(H) < _.length : hasOwn(_, H), Z = Reflect.set(
        _,
        H,
        V,
        isRef(_) ? _ : q
      );
      return _ === toRaw(q) && (J ? hasChanged(V, G) && trigger$1(_, "set", H, V) : trigger$1(_, "add", H, V)), Z;
    }
    deleteProperty(_, H) {
      const V = hasOwn(_, H);
      _[H];
      const q = Reflect.deleteProperty(_, H);
      return q && V && trigger$1(_, "delete", H, void 0), q;
    }
    has(_, H) {
      const V = Reflect.has(_, H);
      return (!isSymbol$2(H) || !builtInSymbols.has(H)) && track(_, "has", H), V;
    }
    ownKeys(_) {
      return track(
        _,
        "iterate",
        isArray$6(_) ? "length" : ITERATE_KEY
      ), Reflect.ownKeys(_);
    }
  }
  class ReadonlyReactiveHandler extends BaseReactiveHandler {
    constructor(_ = !1) {
      super(!0, _);
    }
    set(_, H) {
      return !0;
    }
    deleteProperty(_, H) {
      return !0;
    }
  }
  const mutableHandlers = /* @__PURE__ */ new MutableReactiveHandler(), readonlyHandlers = /* @__PURE__ */ new ReadonlyReactiveHandler(), shallowReactiveHandlers = /* @__PURE__ */ new MutableReactiveHandler(!0), shallowReadonlyHandlers = /* @__PURE__ */ new ReadonlyReactiveHandler(!0), toShallow = (O) => O, getProto = (O) => Reflect.getPrototypeOf(O);
  function createIterableMethod(O, _, H) {
    return function(...V) {
      const q = this.__v_raw, G = toRaw(q), J = isMap(G), Z = O === "entries" || O === Symbol.iterator && J, te = O === "keys" && J, ne = q[O](...V), ee = H ? toShallow : _ ? toReadonly : toReactive;
      return !_ && track(
        G,
        "iterate",
        te ? MAP_KEY_ITERATE_KEY : ITERATE_KEY
      ), {
        // iterator protocol
        next() {
          const { value: ae, done: oe } = ne.next();
          return oe ? { value: ae, done: oe } : {
            value: Z ? [ee(ae[0]), ee(ae[1])] : ee(ae),
            done: oe
          };
        },
        // iterable protocol
        [Symbol.iterator]() {
          return this;
        }
      };
    };
  }
  function createReadonlyMethod(O) {
    return function(..._) {
      return O === "delete" ? !1 : O === "clear" ? void 0 : this;
    };
  }
  function createInstrumentations(O, _) {
    const H = {
      get(q) {
        const G = this.__v_raw, J = toRaw(G), Z = toRaw(q);
        O || (hasChanged(q, Z) && track(J, "get", q), track(J, "get", Z));
        const { has: te } = getProto(J), ne = _ ? toShallow : O ? toReadonly : toReactive;
        if (te.call(J, q))
          return ne(G.get(q));
        if (te.call(J, Z))
          return ne(G.get(Z));
        G !== J && G.get(q);
      },
      get size() {
        const q = this.__v_raw;
        return !O && track(toRaw(q), "iterate", ITERATE_KEY), Reflect.get(q, "size", q);
      },
      has(q) {
        const G = this.__v_raw, J = toRaw(G), Z = toRaw(q);
        return O || (hasChanged(q, Z) && track(J, "has", q), track(J, "has", Z)), q === Z ? G.has(q) : G.has(q) || G.has(Z);
      },
      forEach(q, G) {
        const J = this, Z = J.__v_raw, te = toRaw(Z), ne = _ ? toShallow : O ? toReadonly : toReactive;
        return !O && track(te, "iterate", ITERATE_KEY), Z.forEach((ee, ae) => q.call(G, ne(ee), ne(ae), J));
      }
    };
    return extend$a(
      H,
      O ? {
        add: createReadonlyMethod("add"),
        set: createReadonlyMethod("set"),
        delete: createReadonlyMethod("delete"),
        clear: createReadonlyMethod("clear")
      } : {
        add(q) {
          !_ && !isShallow(q) && !isReadonly(q) && (q = toRaw(q));
          const G = toRaw(this);
          return getProto(G).has.call(G, q) || (G.add(q), trigger$1(G, "add", q, q)), this;
        },
        set(q, G) {
          !_ && !isShallow(G) && !isReadonly(G) && (G = toRaw(G));
          const J = toRaw(this), { has: Z, get: te } = getProto(J);
          let ne = Z.call(J, q);
          ne || (q = toRaw(q), ne = Z.call(J, q));
          const ee = te.call(J, q);
          return J.set(q, G), ne ? hasChanged(G, ee) && trigger$1(J, "set", q, G) : trigger$1(J, "add", q, G), this;
        },
        delete(q) {
          const G = toRaw(this), { has: J, get: Z } = getProto(G);
          let te = J.call(G, q);
          te || (q = toRaw(q), te = J.call(G, q)), Z && Z.call(G, q);
          const ne = G.delete(q);
          return te && trigger$1(G, "delete", q, void 0), ne;
        },
        clear() {
          const q = toRaw(this), G = q.size !== 0, J = q.clear();
          return G && trigger$1(
            q,
            "clear",
            void 0,
            void 0
          ), J;
        }
      }
    ), [
      "keys",
      "values",
      "entries",
      Symbol.iterator
    ].forEach((q) => {
      H[q] = createIterableMethod(q, O, _);
    }), H;
  }
  function createInstrumentationGetter(O, _) {
    const H = createInstrumentations(O, _);
    return (V, q, G) => q === "__v_isReactive" ? !O : q === "__v_isReadonly" ? O : q === "__v_raw" ? V : Reflect.get(
      hasOwn(H, q) && q in V ? H : V,
      q,
      G
    );
  }
  const mutableCollectionHandlers = {
    get: /* @__PURE__ */ createInstrumentationGetter(!1, !1)
  }, shallowCollectionHandlers = {
    get: /* @__PURE__ */ createInstrumentationGetter(!1, !0)
  }, readonlyCollectionHandlers = {
    get: /* @__PURE__ */ createInstrumentationGetter(!0, !1)
  }, shallowReadonlyCollectionHandlers = {
    get: /* @__PURE__ */ createInstrumentationGetter(!0, !0)
  }, reactiveMap = /* @__PURE__ */ new WeakMap(), shallowReactiveMap = /* @__PURE__ */ new WeakMap(), readonlyMap = /* @__PURE__ */ new WeakMap(), shallowReadonlyMap = /* @__PURE__ */ new WeakMap();
  function targetTypeMap(O) {
    switch (O) {
      case "Object":
      case "Array":
        return 1;
      case "Map":
      case "Set":
      case "WeakMap":
      case "WeakSet":
        return 2;
      default:
        return 0;
    }
  }
  function getTargetType(O) {
    return O.__v_skip || !Object.isExtensible(O) ? 0 : targetTypeMap(toRawType(O));
  }
  function reactive(O) {
    return isReadonly(O) ? O : createReactiveObject(
      O,
      !1,
      mutableHandlers,
      mutableCollectionHandlers,
      reactiveMap
    );
  }
  function shallowReactive(O) {
    return createReactiveObject(
      O,
      !1,
      shallowReactiveHandlers,
      shallowCollectionHandlers,
      shallowReactiveMap
    );
  }
  function readonly(O) {
    return createReactiveObject(
      O,
      !0,
      readonlyHandlers,
      readonlyCollectionHandlers,
      readonlyMap
    );
  }
  function shallowReadonly(O) {
    return createReactiveObject(
      O,
      !0,
      shallowReadonlyHandlers,
      shallowReadonlyCollectionHandlers,
      shallowReadonlyMap
    );
  }
  function createReactiveObject(O, _, H, V, q) {
    if (!isObject$7(O) || O.__v_raw && !(_ && O.__v_isReactive))
      return O;
    const G = q.get(O);
    if (G)
      return G;
    const J = getTargetType(O);
    if (J === 0)
      return O;
    const Z = new Proxy(
      O,
      J === 2 ? V : H
    );
    return q.set(O, Z), Z;
  }
  function isReactive(O) {
    return isReadonly(O) ? isReactive(O.__v_raw) : !!(O && O.__v_isReactive);
  }
  function isReadonly(O) {
    return !!(O && O.__v_isReadonly);
  }
  function isShallow(O) {
    return !!(O && O.__v_isShallow);
  }
  function isProxy(O) {
    return O ? !!O.__v_raw : !1;
  }
  function toRaw(O) {
    const _ = O && O.__v_raw;
    return _ ? toRaw(_) : O;
  }
  function markRaw(O) {
    return !hasOwn(O, "__v_skip") && Object.isExtensible(O) && def$2(O, "__v_skip", !0), O;
  }
  const toReactive = (O) => isObject$7(O) ? reactive(O) : O, toReadonly = (O) => isObject$7(O) ? readonly(O) : O;
  function isRef(O) {
    return O ? O.__v_isRef === !0 : !1;
  }
  function ref$1(O) {
    return createRef(O, !1);
  }
  function createRef(O, _) {
    return isRef(O) ? O : new RefImpl(O, _);
  }
  class RefImpl {
    constructor(_, H) {
      this.dep = new Dep(), this.__v_isRef = !0, this.__v_isShallow = !1, this._rawValue = H ? _ : toRaw(_), this._value = H ? _ : toReactive(_), this.__v_isShallow = H;
    }
    get value() {
      return this.dep.track(), this._value;
    }
    set value(_) {
      const H = this._rawValue, V = this.__v_isShallow || isShallow(_) || isReadonly(_);
      _ = V ? _ : toRaw(_), hasChanged(_, H) && (this._rawValue = _, this._value = V ? _ : toReactive(_), this.dep.trigger());
    }
  }
  function triggerRef(O) {
    O.dep && O.dep.trigger();
  }
  function unref(O) {
    return isRef(O) ? O.value : O;
  }
  const shallowUnwrapHandlers = {
    get: (O, _, H) => _ === "__v_raw" ? O : unref(Reflect.get(O, _, H)),
    set: (O, _, H, V) => {
      const q = O[_];
      return isRef(q) && !isRef(H) ? (q.value = H, !0) : Reflect.set(O, _, H, V);
    }
  };
  function proxyRefs(O) {
    return isReactive(O) ? O : new Proxy(O, shallowUnwrapHandlers);
  }
  function toRefs(O) {
    const _ = isArray$6(O) ? new Array(O.length) : {};
    for (const H in O)
      _[H] = propertyToRef(O, H);
    return _;
  }
  class ObjectRefImpl {
    constructor(_, H, V) {
      this._object = _, this._key = H, this._defaultValue = V, this.__v_isRef = !0, this._value = void 0;
    }
    get value() {
      const _ = this._object[this._key];
      return this._value = _ === void 0 ? this._defaultValue : _;
    }
    set value(_) {
      this._object[this._key] = _;
    }
    get dep() {
      return getDepFromReactive(toRaw(this._object), this._key);
    }
  }
  class GetterRefImpl {
    constructor(_) {
      this._getter = _, this.__v_isRef = !0, this.__v_isReadonly = !0, this._value = void 0;
    }
    get value() {
      return this._value = this._getter();
    }
  }
  function toRef(O, _, H) {
    return isRef(O) ? O : isFunction$6(O) ? new GetterRefImpl(O) : isObject$7(O) && arguments.length > 1 ? propertyToRef(O, _, H) : ref$1(O);
  }
  function propertyToRef(O, _, H) {
    const V = O[_];
    return isRef(V) ? V : new ObjectRefImpl(O, _, H);
  }
  class ComputedRefImpl {
    constructor(_, H, V) {
      this.fn = _, this.setter = H, this._value = void 0, this.dep = new Dep(this), this.__v_isRef = !0, this.deps = void 0, this.depsTail = void 0, this.flags = 16, this.globalVersion = globalVersion - 1, this.next = void 0, this.effect = this, this.__v_isReadonly = !H, this.isSSR = V;
    }
    /**
     * @internal
     */
    notify() {
      if (this.flags |= 16, !(this.flags & 8) && // avoid infinite self recursion
      activeSub !== this)
        return batch(this, !0), !0;
    }
    get value() {
      const _ = this.dep.track();
      return refreshComputed(this), _ && (_.version = this.dep.version), this._value;
    }
    set value(_) {
      this.setter && this.setter(_);
    }
  }
  function computed$1(O, _, H = !1) {
    let V, q;
    return isFunction$6(O) ? V = O : (V = O.get, q = O.set), new ComputedRefImpl(V, q, H);
  }
  const INITIAL_WATCHER_VALUE = {}, cleanupMap = /* @__PURE__ */ new WeakMap();
  let activeWatcher;
  function onWatcherCleanup(O, _ = !1, H = activeWatcher) {
    if (H) {
      let V = cleanupMap.get(H);
      V || cleanupMap.set(H, V = []), V.push(O);
    }
  }
  function watch$2(O, _, H = EMPTY_OBJ$1) {
    const { immediate: V, deep: q, once: G, scheduler: J, augmentJob: Z, call: te } = H, ne = (me) => q ? me : isShallow(me) || q === !1 || q === 0 ? traverse(me, 1) : traverse(me);
    let ee, ae, oe, se, ce = !1, le = !1;
    if (isRef(O) ? (ae = () => O.value, ce = isShallow(O)) : isReactive(O) ? (ae = () => ne(O), ce = !0) : isArray$6(O) ? (le = !0, ce = O.some((me) => isReactive(me) || isShallow(me)), ae = () => O.map((me) => {
      if (isRef(me))
        return me.value;
      if (isReactive(me))
        return ne(me);
      if (isFunction$6(me))
        return te ? te(me, 2) : me();
    })) : isFunction$6(O) ? _ ? ae = te ? () => te(O, 2) : O : ae = () => {
      if (oe) {
        pauseTracking();
        try {
          oe();
        } finally {
          resetTracking();
        }
      }
      const me = activeWatcher;
      activeWatcher = ee;
      try {
        return te ? te(O, 3, [se]) : O(se);
      } finally {
        activeWatcher = me;
      }
    } : ae = NOOP, _ && q) {
      const me = ae, pe = q === !0 ? 1 / 0 : q;
      ae = () => traverse(me(), pe);
    }
    const ue = getCurrentScope(), de = () => {
      ee.stop(), ue && ue.active && remove$4(ue.effects, ee);
    };
    if (G && _) {
      const me = _;
      _ = (...pe) => {
        me(...pe), de();
      };
    }
    let fe = le ? new Array(O.length).fill(INITIAL_WATCHER_VALUE) : INITIAL_WATCHER_VALUE;
    const he = (me) => {
      if (!(!(ee.flags & 1) || !ee.dirty && !me))
        if (_) {
          const pe = ee.run();
          if (q || ce || (le ? pe.some((ye, ge) => hasChanged(ye, fe[ge])) : hasChanged(pe, fe))) {
            oe && oe();
            const ye = activeWatcher;
            activeWatcher = ee;
            try {
              const ge = [
                pe,
                // pass undefined as the old value when it's changed for the first time
                fe === INITIAL_WATCHER_VALUE ? void 0 : le && fe[0] === INITIAL_WATCHER_VALUE ? [] : fe,
                se
              ];
              te ? te(_, 3, ge) : (
                // @ts-expect-error
                _(...ge)
              ), fe = pe;
            } finally {
              activeWatcher = ye;
            }
          }
        } else
          ee.run();
    };
    return Z && Z(he), ee = new ReactiveEffect(ae), ee.scheduler = J ? () => J(he, !1) : he, se = (me) => onWatcherCleanup(me, !1, ee), oe = ee.onStop = () => {
      const me = cleanupMap.get(ee);
      if (me) {
        if (te)
          te(me, 4);
        else
          for (const pe of me) pe();
        cleanupMap.delete(ee);
      }
    }, _ ? V ? he(!0) : fe = ee.run() : J ? J(he.bind(null, !0), !0) : ee.run(), de.pause = ee.pause.bind(ee), de.resume = ee.resume.bind(ee), de.stop = de, de;
  }
  function traverse(O, _ = 1 / 0, H) {
    if (_ <= 0 || !isObject$7(O) || O.__v_skip || (H = H || /* @__PURE__ */ new Set(), H.has(O)))
      return O;
    if (H.add(O), _--, isRef(O))
      traverse(O.value, _, H);
    else if (isArray$6(O))
      for (let V = 0; V < O.length; V++)
        traverse(O[V], _, H);
    else if (isSet$1(O) || isMap(O))
      O.forEach((V) => {
        traverse(V, _, H);
      });
    else if (isPlainObject$3(O)) {
      for (const V in O)
        traverse(O[V], _, H);
      for (const V of Object.getOwnPropertySymbols(O))
        Object.prototype.propertyIsEnumerable.call(O, V) && traverse(O[V], _, H);
    }
    return O;
  }
  /**
  * @vue/runtime-core v3.5.13
  * (c) 2018-present Yuxi (Evan) You and Vue contributors
  * @license MIT
  **/
  const stack$1 = [];
  let isWarning = !1;
  function warn$1$1(O, ..._) {
    if (isWarning) return;
    isWarning = !0, pauseTracking();
    const H = stack$1.length ? stack$1[stack$1.length - 1].component : null, V = H && H.appContext.config.warnHandler, q = getComponentTrace();
    if (V)
      callWithErrorHandling(
        V,
        H,
        11,
        [
          // eslint-disable-next-line no-restricted-syntax
          O + _.map((G) => {
            var J, Z;
            return (Z = (J = G.toString) == null ? void 0 : J.call(G)) != null ? Z : JSON.stringify(G);
          }).join(""),
          H && H.proxy,
          q.map(
            ({ vnode: G }) => `at <${formatComponentName(H, G.type)}>`
          ).join(`
`),
          q
        ]
      );
    else {
      const G = [`[Vue warn]: ${O}`, ..._];
      q.length && G.push(`
`, ...formatTrace(q)), console.warn(...G);
    }
    resetTracking(), isWarning = !1;
  }
  function getComponentTrace() {
    let O = stack$1[stack$1.length - 1];
    if (!O)
      return [];
    const _ = [];
    for (; O; ) {
      const H = _[0];
      H && H.vnode === O ? H.recurseCount++ : _.push({
        vnode: O,
        recurseCount: 0
      });
      const V = O.component && O.component.parent;
      O = V && V.vnode;
    }
    return _;
  }
  function formatTrace(O) {
    const _ = [];
    return O.forEach((H, V) => {
      _.push(...V === 0 ? [] : [`
`], ...formatTraceEntry(H));
    }), _;
  }
  function formatTraceEntry({ vnode: O, recurseCount: _ }) {
    const H = _ > 0 ? `... (${_} recursive calls)` : "", V = O.component ? O.component.parent == null : !1, q = ` at <${formatComponentName(
      O.component,
      O.type,
      V
    )}`, G = ">" + H;
    return O.props ? [q, ...formatProps(O.props), G] : [q + G];
  }
  function formatProps(O) {
    const _ = [], H = Object.keys(O);
    return H.slice(0, 3).forEach((V) => {
      _.push(...formatProp(V, O[V]));
    }), H.length > 3 && _.push(" ..."), _;
  }
  function formatProp(O, _, H) {
    return isString$5(_) ? (_ = JSON.stringify(_), H ? _ : [`${O}=${_}`]) : typeof _ == "number" || typeof _ == "boolean" || _ == null ? H ? _ : [`${O}=${_}`] : isRef(_) ? (_ = formatProp(O, toRaw(_.value), !0), H ? _ : [`${O}=Ref<`, _, ">"]) : isFunction$6(_) ? [`${O}=fn${_.name ? `<${_.name}>` : ""}`] : (_ = toRaw(_), H ? _ : [`${O}=`, _]);
  }
  function callWithErrorHandling(O, _, H, V) {
    try {
      return V ? O(...V) : O();
    } catch (q) {
      handleError(q, _, H);
    }
  }
  function callWithAsyncErrorHandling(O, _, H, V) {
    if (isFunction$6(O)) {
      const q = callWithErrorHandling(O, _, H, V);
      return q && isPromise$1(q) && q.catch((G) => {
        handleError(G, _, H);
      }), q;
    }
    if (isArray$6(O)) {
      const q = [];
      for (let G = 0; G < O.length; G++)
        q.push(callWithAsyncErrorHandling(O[G], _, H, V));
      return q;
    }
  }
  function handleError(O, _, H, V = !0) {
    const q = _ ? _.vnode : null, { errorHandler: G, throwUnhandledErrorInProduction: J } = _ && _.appContext.config || EMPTY_OBJ$1;
    if (_) {
      let Z = _.parent;
      const te = _.proxy, ne = `https://vuejs.org/error-reference/#runtime-${H}`;
      for (; Z; ) {
        const ee = Z.ec;
        if (ee) {
          for (let ae = 0; ae < ee.length; ae++)
            if (ee[ae](O, te, ne) === !1)
              return;
        }
        Z = Z.parent;
      }
      if (G) {
        pauseTracking(), callWithErrorHandling(G, null, 10, [
          O,
          te,
          ne
        ]), resetTracking();
        return;
      }
    }
    logError(O, H, q, V, J);
  }
  function logError(O, _, H, V = !0, q = !1) {
    if (q)
      throw O;
    console.error(O);
  }
  const queue = [];
  let flushIndex = -1;
  const pendingPostFlushCbs = [];
  let activePostFlushCbs = null, postFlushIndex = 0;
  const resolvedPromise = /* @__PURE__ */ Promise.resolve();
  let currentFlushPromise = null;
  function nextTick$1(O) {
    const _ = currentFlushPromise || resolvedPromise;
    return O ? _.then(this ? O.bind(this) : O) : _;
  }
  function findInsertionIndex(O) {
    let _ = flushIndex + 1, H = queue.length;
    for (; _ < H; ) {
      const V = _ + H >>> 1, q = queue[V], G = getId$1(q);
      G < O || G === O && q.flags & 2 ? _ = V + 1 : H = V;
    }
    return _;
  }
  function queueJob(O) {
    if (!(O.flags & 1)) {
      const _ = getId$1(O), H = queue[queue.length - 1];
      !H || // fast path when the job id is larger than the tail
      !(O.flags & 2) && _ >= getId$1(H) ? queue.push(O) : queue.splice(findInsertionIndex(_), 0, O), O.flags |= 1, queueFlush();
    }
  }
  function queueFlush() {
    currentFlushPromise || (currentFlushPromise = resolvedPromise.then(flushJobs));
  }
  function queuePostFlushCb(O) {
    isArray$6(O) ? pendingPostFlushCbs.push(...O) : activePostFlushCbs && O.id === -1 ? activePostFlushCbs.splice(postFlushIndex + 1, 0, O) : O.flags & 1 || (pendingPostFlushCbs.push(O), O.flags |= 1), queueFlush();
  }
  function flushPreFlushCbs(O, _, H = flushIndex + 1) {
    for (; H < queue.length; H++) {
      const V = queue[H];
      if (V && V.flags & 2) {
        if (O && V.id !== O.uid)
          continue;
        queue.splice(H, 1), H--, V.flags & 4 && (V.flags &= -2), V(), V.flags & 4 || (V.flags &= -2);
      }
    }
  }
  function flushPostFlushCbs(O) {
    if (pendingPostFlushCbs.length) {
      const _ = [...new Set(pendingPostFlushCbs)].sort(
        (H, V) => getId$1(H) - getId$1(V)
      );
      if (pendingPostFlushCbs.length = 0, activePostFlushCbs) {
        activePostFlushCbs.push(..._);
        return;
      }
      for (activePostFlushCbs = _, postFlushIndex = 0; postFlushIndex < activePostFlushCbs.length; postFlushIndex++) {
        const H = activePostFlushCbs[postFlushIndex];
        H.flags & 4 && (H.flags &= -2), H.flags & 8 || H(), H.flags &= -2;
      }
      activePostFlushCbs = null, postFlushIndex = 0;
    }
  }
  const getId$1 = (O) => O.id == null ? O.flags & 2 ? -1 : 1 / 0 : O.id;
  function flushJobs(O) {
    try {
      for (flushIndex = 0; flushIndex < queue.length; flushIndex++) {
        const _ = queue[flushIndex];
        _ && !(_.flags & 8) && (_.flags & 4 && (_.flags &= -2), callWithErrorHandling(
          _,
          _.i,
          _.i ? 15 : 14
        ), _.flags & 4 || (_.flags &= -2));
      }
    } finally {
      for (; flushIndex < queue.length; flushIndex++) {
        const _ = queue[flushIndex];
        _ && (_.flags &= -2);
      }
      flushIndex = -1, queue.length = 0, flushPostFlushCbs(), currentFlushPromise = null, (queue.length || pendingPostFlushCbs.length) && flushJobs();
    }
  }
  let currentRenderingInstance = null, currentScopeId = null;
  function setCurrentRenderingInstance(O) {
    const _ = currentRenderingInstance;
    return currentRenderingInstance = O, currentScopeId = O && O.type.__scopeId || null, _;
  }
  function withCtx(O, _ = currentRenderingInstance, H) {
    if (!_ || O._n)
      return O;
    const V = (...q) => {
      V._d && setBlockTracking(-1);
      const G = setCurrentRenderingInstance(_);
      let J;
      try {
        J = O(...q);
      } finally {
        setCurrentRenderingInstance(G), V._d && setBlockTracking(1);
      }
      return J;
    };
    return V._n = !0, V._c = !0, V._d = !0, V;
  }
  function withDirectives(O, _) {
    if (currentRenderingInstance === null)
      return O;
    const H = getComponentPublicInstance(currentRenderingInstance), V = O.dirs || (O.dirs = []);
    for (let q = 0; q < _.length; q++) {
      let [G, J, Z, te = EMPTY_OBJ$1] = _[q];
      G && (isFunction$6(G) && (G = {
        mounted: G,
        updated: G
      }), G.deep && traverse(J), V.push({
        dir: G,
        instance: H,
        value: J,
        oldValue: void 0,
        arg: Z,
        modifiers: te
      }));
    }
    return O;
  }
  function invokeDirectiveHook(O, _, H, V) {
    const q = O.dirs, G = _ && _.dirs;
    for (let J = 0; J < q.length; J++) {
      const Z = q[J];
      G && (Z.oldValue = G[J].value);
      let te = Z.dir[V];
      te && (pauseTracking(), callWithAsyncErrorHandling(te, H, 8, [
        O.el,
        Z,
        O,
        _
      ]), resetTracking());
    }
  }
  const TeleportEndKey = Symbol("_vte"), isTeleport = (O) => O.__isTeleport, leaveCbKey = Symbol("_leaveCb"), enterCbKey = Symbol("_enterCb");
  function useTransitionState() {
    const O = {
      isMounted: !1,
      isLeaving: !1,
      isUnmounting: !1,
      leavingVNodes: /* @__PURE__ */ new Map()
    };
    return onMounted(() => {
      O.isMounted = !0;
    }), onBeforeUnmount(() => {
      O.isUnmounting = !0;
    }), O;
  }
  const TransitionHookValidator = [Function, Array], BaseTransitionPropsValidators = {
    mode: String,
    appear: Boolean,
    persisted: Boolean,
    // enter
    onBeforeEnter: TransitionHookValidator,
    onEnter: TransitionHookValidator,
    onAfterEnter: TransitionHookValidator,
    onEnterCancelled: TransitionHookValidator,
    // leave
    onBeforeLeave: TransitionHookValidator,
    onLeave: TransitionHookValidator,
    onAfterLeave: TransitionHookValidator,
    onLeaveCancelled: TransitionHookValidator,
    // appear
    onBeforeAppear: TransitionHookValidator,
    onAppear: TransitionHookValidator,
    onAfterAppear: TransitionHookValidator,
    onAppearCancelled: TransitionHookValidator
  }, recursiveGetSubtree = (O) => {
    const _ = O.subTree;
    return _.component ? recursiveGetSubtree(_.component) : _;
  }, BaseTransitionImpl = {
    name: "BaseTransition",
    props: BaseTransitionPropsValidators,
    setup(O, { slots: _ }) {
      const H = getCurrentInstance(), V = useTransitionState();
      return () => {
        const q = _.default && getTransitionRawChildren(_.default(), !0);
        if (!q || !q.length)
          return;
        const G = findNonCommentChild(q), J = toRaw(O), { mode: Z } = J;
        if (V.isLeaving)
          return emptyPlaceholder(G);
        const te = getInnerChild$1(G);
        if (!te)
          return emptyPlaceholder(G);
        let ne = resolveTransitionHooks(
          te,
          J,
          V,
          H,
          // #11061, ensure enterHooks is fresh after clone
          (ae) => ne = ae
        );
        te.type !== Comment && setTransitionHooks(te, ne);
        let ee = H.subTree && getInnerChild$1(H.subTree);
        if (ee && ee.type !== Comment && !isSameVNodeType(te, ee) && recursiveGetSubtree(H).type !== Comment) {
          let ae = resolveTransitionHooks(
            ee,
            J,
            V,
            H
          );
          if (setTransitionHooks(ee, ae), Z === "out-in" && te.type !== Comment)
            return V.isLeaving = !0, ae.afterLeave = () => {
              V.isLeaving = !1, H.job.flags & 8 || H.update(), delete ae.afterLeave, ee = void 0;
            }, emptyPlaceholder(G);
          Z === "in-out" && te.type !== Comment ? ae.delayLeave = (oe, se, ce) => {
            const le = getLeavingNodesForType(
              V,
              ee
            );
            le[String(ee.key)] = ee, oe[leaveCbKey] = () => {
              se(), oe[leaveCbKey] = void 0, delete ne.delayedLeave, ee = void 0;
            }, ne.delayedLeave = () => {
              ce(), delete ne.delayedLeave, ee = void 0;
            };
          } : ee = void 0;
        } else ee && (ee = void 0);
        return G;
      };
    }
  };
  function findNonCommentChild(O) {
    let _ = O[0];
    if (O.length > 1) {
      for (const H of O)
        if (H.type !== Comment) {
          _ = H;
          break;
        }
    }
    return _;
  }
  const BaseTransition = BaseTransitionImpl;
  function getLeavingNodesForType(O, _) {
    const { leavingVNodes: H } = O;
    let V = H.get(_.type);
    return V || (V = /* @__PURE__ */ Object.create(null), H.set(_.type, V)), V;
  }
  function resolveTransitionHooks(O, _, H, V, q) {
    const {
      appear: G,
      mode: J,
      persisted: Z = !1,
      onBeforeEnter: te,
      onEnter: ne,
      onAfterEnter: ee,
      onEnterCancelled: ae,
      onBeforeLeave: oe,
      onLeave: se,
      onAfterLeave: ce,
      onLeaveCancelled: le,
      onBeforeAppear: ue,
      onAppear: de,
      onAfterAppear: fe,
      onAppearCancelled: he
    } = _, me = String(O.key), pe = getLeavingNodesForType(H, O), ye = (be, Ce) => {
      be && callWithAsyncErrorHandling(
        be,
        V,
        9,
        Ce
      );
    }, ge = (be, Ce) => {
      const Se = Ce[1];
      ye(be, Ce), isArray$6(be) ? be.every((we) => we.length <= 1) && Se() : be.length <= 1 && Se();
    }, ve = {
      mode: J,
      persisted: Z,
      beforeEnter(be) {
        let Ce = te;
        if (!H.isMounted)
          if (G)
            Ce = ue || te;
          else
            return;
        be[leaveCbKey] && be[leaveCbKey](
          !0
          /* cancelled */
        );
        const Se = pe[me];
        Se && isSameVNodeType(O, Se) && Se.el[leaveCbKey] && Se.el[leaveCbKey](), ye(Ce, [be]);
      },
      enter(be) {
        let Ce = ne, Se = ee, we = ae;
        if (!H.isMounted)
          if (G)
            Ce = de || ne, Se = fe || ee, we = he || ae;
          else
            return;
        let Ee = !1;
        const xe = be[enterCbKey] = (Le) => {
          Ee || (Ee = !0, Le ? ye(we, [be]) : ye(Se, [be]), ve.delayedLeave && ve.delayedLeave(), be[enterCbKey] = void 0);
        };
        Ce ? ge(Ce, [be, xe]) : xe();
      },
      leave(be, Ce) {
        const Se = String(O.key);
        if (be[enterCbKey] && be[enterCbKey](
          !0
          /* cancelled */
        ), H.isUnmounting)
          return Ce();
        ye(oe, [be]);
        let we = !1;
        const Ee = be[leaveCbKey] = (xe) => {
          we || (we = !0, Ce(), xe ? ye(le, [be]) : ye(ce, [be]), be[leaveCbKey] = void 0, pe[Se] === O && delete pe[Se]);
        };
        pe[Se] = O, se ? ge(se, [be, Ee]) : Ee();
      },
      clone(be) {
        const Ce = resolveTransitionHooks(
          be,
          _,
          H,
          V,
          q
        );
        return q && q(Ce), Ce;
      }
    };
    return ve;
  }
  function emptyPlaceholder(O) {
    if (isKeepAlive(O))
      return O = cloneVNode(O), O.children = null, O;
  }
  function getInnerChild$1(O) {
    if (!isKeepAlive(O))
      return isTeleport(O.type) && O.children ? findNonCommentChild(O.children) : O;
    const { shapeFlag: _, children: H } = O;
    if (H) {
      if (_ & 16)
        return H[0];
      if (_ & 32 && isFunction$6(H.default))
        return H.default();
    }
  }
  function setTransitionHooks(O, _) {
    O.shapeFlag & 6 && O.component ? (O.transition = _, setTransitionHooks(O.component.subTree, _)) : O.shapeFlag & 128 ? (O.ssContent.transition = _.clone(O.ssContent), O.ssFallback.transition = _.clone(O.ssFallback)) : O.transition = _;
  }
  function getTransitionRawChildren(O, _ = !1, H) {
    let V = [], q = 0;
    for (let G = 0; G < O.length; G++) {
      let J = O[G];
      const Z = H == null ? J.key : String(H) + String(J.key != null ? J.key : G);
      J.type === Fragment ? (J.patchFlag & 128 && q++, V = V.concat(
        getTransitionRawChildren(J.children, _, Z)
      )) : (_ || J.type !== Comment) && V.push(Z != null ? cloneVNode(J, { key: Z }) : J);
    }
    if (q > 1)
      for (let G = 0; G < V.length; G++)
        V[G].patchFlag = -2;
    return V;
  }
  /*! #__NO_SIDE_EFFECTS__ */
  // @__NO_SIDE_EFFECTS__
  function defineComponent(O, _) {
    return isFunction$6(O) ? (
      // #8236: extend call and options.name access are considered side-effects
      // by Rollup, so we have to wrap it in a pure-annotated IIFE.
      extend$a({ name: O.name }, _, { setup: O })
    ) : O;
  }
  function markAsyncBoundary(O) {
    O.ids = [O.ids[0] + O.ids[2]++ + "-", 0, 0];
  }
  function setRef(O, _, H, V, q = !1) {
    if (isArray$6(O)) {
      O.forEach(
        (ce, le) => setRef(
          ce,
          _ && (isArray$6(_) ? _[le] : _),
          H,
          V,
          q
        )
      );
      return;
    }
    if (isAsyncWrapper(V) && !q) {
      V.shapeFlag & 512 && V.type.__asyncResolved && V.component.subTree.component && setRef(O, _, H, V.component.subTree);
      return;
    }
    const G = V.shapeFlag & 4 ? getComponentPublicInstance(V.component) : V.el, J = q ? null : G, { i: Z, r: te } = O, ne = _ && _.r, ee = Z.refs === EMPTY_OBJ$1 ? Z.refs = {} : Z.refs, ae = Z.setupState, oe = toRaw(ae), se = ae === EMPTY_OBJ$1 ? () => !1 : (ce) => hasOwn(oe, ce);
    if (ne != null && ne !== te && (isString$5(ne) ? (ee[ne] = null, se(ne) && (ae[ne] = null)) : isRef(ne) && (ne.value = null)), isFunction$6(te))
      callWithErrorHandling(te, Z, 12, [J, ee]);
    else {
      const ce = isString$5(te), le = isRef(te);
      if (ce || le) {
        const ue = () => {
          if (O.f) {
            const de = ce ? se(te) ? ae[te] : ee[te] : te.value;
            q ? isArray$6(de) && remove$4(de, G) : isArray$6(de) ? de.includes(G) || de.push(G) : ce ? (ee[te] = [G], se(te) && (ae[te] = ee[te])) : (te.value = [G], O.k && (ee[O.k] = te.value));
          } else ce ? (ee[te] = J, se(te) && (ae[te] = J)) : le && (te.value = J, O.k && (ee[O.k] = J));
        };
        J ? (ue.id = -1, queuePostRenderEffect(ue, H)) : ue();
      }
    }
  }
  const isComment = (O) => O.nodeType === 8;
  getGlobalThis().requestIdleCallback;
  getGlobalThis().cancelIdleCallback;
  function forEachElement(O, _) {
    if (isComment(O) && O.data === "[") {
      let H = 1, V = O.nextSibling;
      for (; V; ) {
        if (V.nodeType === 1) {
          if (_(V) === !1)
            break;
        } else if (isComment(V))
          if (V.data === "]") {
            if (--H === 0) break;
          } else V.data === "[" && H++;
        V = V.nextSibling;
      }
    } else
      _(O);
  }
  const isAsyncWrapper = (O) => !!O.type.__asyncLoader;
  /*! #__NO_SIDE_EFFECTS__ */
  // @__NO_SIDE_EFFECTS__
  function defineAsyncComponent(O) {
    isFunction$6(O) && (O = { loader: O });
    const {
      loader: _,
      loadingComponent: H,
      errorComponent: V,
      delay: q = 200,
      hydrate: G,
      timeout: J,
      // undefined = never times out
      suspensible: Z = !0,
      onError: te
    } = O;
    let ne = null, ee, ae = 0;
    const oe = () => (ae++, ne = null, se()), se = () => {
      let ce;
      return ne || (ce = ne = _().catch((le) => {
        if (le = le instanceof Error ? le : new Error(String(le)), te)
          return new Promise((ue, de) => {
            te(le, () => ue(oe()), () => de(le), ae + 1);
          });
        throw le;
      }).then((le) => ce !== ne && ne ? ne : (le && (le.__esModule || le[Symbol.toStringTag] === "Module") && (le = le.default), ee = le, le)));
    };
    return /* @__PURE__ */ defineComponent({
      name: "AsyncComponentWrapper",
      __asyncLoader: se,
      __asyncHydrate(ce, le, ue) {
        const de = G ? () => {
          const fe = G(
            ue,
            (he) => forEachElement(ce, he)
          );
          fe && (le.bum || (le.bum = [])).push(fe);
        } : ue;
        ee ? de() : se().then(() => !le.isUnmounted && de());
      },
      get __asyncResolved() {
        return ee;
      },
      setup() {
        const ce = currentInstance;
        if (markAsyncBoundary(ce), ee)
          return () => createInnerComp(ee, ce);
        const le = (he) => {
          ne = null, handleError(
            he,
            ce,
            13,
            !V
          );
        };
        if (Z && ce.suspense || isInSSRComponentSetup)
          return se().then((he) => () => createInnerComp(he, ce)).catch((he) => (le(he), () => V ? createVNode(V, {
            error: he
          }) : null));
        const ue = ref$1(!1), de = ref$1(), fe = ref$1(!!q);
        return q && setTimeout(() => {
          fe.value = !1;
        }, q), J != null && setTimeout(() => {
          if (!ue.value && !de.value) {
            const he = new Error(
              `Async component timed out after ${J}ms.`
            );
            le(he), de.value = he;
          }
        }, J), se().then(() => {
          ue.value = !0, ce.parent && isKeepAlive(ce.parent.vnode) && ce.parent.update();
        }).catch((he) => {
          le(he), de.value = he;
        }), () => {
          if (ue.value && ee)
            return createInnerComp(ee, ce);
          if (de.value && V)
            return createVNode(V, {
              error: de.value
            });
          if (H && !fe.value)
            return createVNode(H);
        };
      }
    });
  }
  function createInnerComp(O, _) {
    const { ref: H, props: V, children: q, ce: G } = _.vnode, J = createVNode(O, V, q);
    return J.ref = H, J.ce = G, delete _.vnode.ce, J;
  }
  const isKeepAlive = (O) => O.type.__isKeepAlive;
  function onActivated(O, _) {
    registerKeepAliveHook(O, "a", _);
  }
  function onDeactivated(O, _) {
    registerKeepAliveHook(O, "da", _);
  }
  function registerKeepAliveHook(O, _, H = currentInstance) {
    const V = O.__wdc || (O.__wdc = () => {
      let q = H;
      for (; q; ) {
        if (q.isDeactivated)
          return;
        q = q.parent;
      }
      return O();
    });
    if (injectHook(_, V, H), H) {
      let q = H.parent;
      for (; q && q.parent; )
        isKeepAlive(q.parent.vnode) && injectToKeepAliveRoot(V, _, H, q), q = q.parent;
    }
  }
  function injectToKeepAliveRoot(O, _, H, V) {
    const q = injectHook(
      _,
      O,
      V,
      !0
      /* prepend */
    );
    onUnmounted(() => {
      remove$4(V[_], q);
    }, H);
  }
  function injectHook(O, _, H = currentInstance, V = !1) {
    if (H) {
      const q = H[O] || (H[O] = []), G = _.__weh || (_.__weh = (...J) => {
        pauseTracking();
        const Z = setCurrentInstance(H), te = callWithAsyncErrorHandling(_, H, O, J);
        return Z(), resetTracking(), te;
      });
      return V ? q.unshift(G) : q.push(G), G;
    }
  }
  const createHook = (O) => (_, H = currentInstance) => {
    (!isInSSRComponentSetup || O === "sp") && injectHook(O, (...V) => _(...V), H);
  }, onBeforeMount = createHook("bm"), onMounted = createHook("m"), onBeforeUpdate = createHook(
    "bu"
  ), onUpdated = createHook("u"), onBeforeUnmount = createHook(
    "bum"
  ), onUnmounted = createHook("um"), onServerPrefetch = createHook(
    "sp"
  ), onRenderTriggered = createHook("rtg"), onRenderTracked = createHook("rtc");
  function onErrorCaptured(O, _ = currentInstance) {
    injectHook("ec", O, _);
  }
  const COMPONENTS = "components";
  function resolveComponent(O, _) {
    return resolveAsset(COMPONENTS, O, !0, _) || O;
  }
  const NULL_DYNAMIC_COMPONENT = Symbol.for("v-ndc");
  function resolveDynamicComponent(O) {
    return isString$5(O) ? resolveAsset(COMPONENTS, O, !1) || O : O || NULL_DYNAMIC_COMPONENT;
  }
  function resolveAsset(O, _, H = !0, V = !1) {
    const q = currentRenderingInstance || currentInstance;
    if (q) {
      const G = q.type;
      {
        const Z = getComponentName(
          G,
          !1
        );
        if (Z && (Z === _ || Z === camelize(_) || Z === capitalize$1(camelize(_))))
          return G;
      }
      const J = (
        // local registration
        // check instance[type] first which is resolved for options API
        resolve$1(q[O] || G[O], _) || // global registration
        resolve$1(q.appContext[O], _)
      );
      return !J && V ? G : J;
    }
  }
  function resolve$1(O, _) {
    return O && (O[_] || O[camelize(_)] || O[capitalize$1(camelize(_))]);
  }
  function renderList(O, _, H, V) {
    let q;
    const G = H, J = isArray$6(O);
    if (J || isString$5(O)) {
      const Z = J && isReactive(O);
      let te = !1;
      Z && (te = !isShallow(O), O = shallowReadArray(O)), q = new Array(O.length);
      for (let ne = 0, ee = O.length; ne < ee; ne++)
        q[ne] = _(
          te ? toReactive(O[ne]) : O[ne],
          ne,
          void 0,
          G
        );
    } else if (typeof O == "number") {
      q = new Array(O);
      for (let Z = 0; Z < O; Z++)
        q[Z] = _(Z + 1, Z, void 0, G);
    } else if (isObject$7(O))
      if (O[Symbol.iterator])
        q = Array.from(
          O,
          (Z, te) => _(Z, te, void 0, G)
        );
      else {
        const Z = Object.keys(O);
        q = new Array(Z.length);
        for (let te = 0, ne = Z.length; te < ne; te++) {
          const ee = Z[te];
          q[te] = _(O[ee], ee, te, G);
        }
      }
    else
      q = [];
    return q;
  }
  function renderSlot(O, _, H = {}, V, q) {
    if (currentRenderingInstance.ce || currentRenderingInstance.parent && isAsyncWrapper(currentRenderingInstance.parent) && currentRenderingInstance.parent.ce)
      return _ !== "default" && (H.name = _), openBlock(), createBlock(
        Fragment,
        null,
        [createVNode("slot", H, V && V())],
        64
      );
    let G = O[_];
    G && G._c && (G._d = !1), openBlock();
    const J = G && ensureValidVNode(G(H)), Z = H.key || // slot content array of a dynamic conditional slot may have a branch
    // key attached in the `createSlots` helper, respect that
    J && J.key, te = createBlock(
      Fragment,
      {
        key: (Z && !isSymbol$2(Z) ? Z : `_${_}`) + // #7256 force differentiate fallback content from actual content
        (!J && V ? "_fb" : "")
      },
      J || (V ? V() : []),
      J && O._ === 1 ? 64 : -2
    );
    return te.scopeId && (te.slotScopeIds = [te.scopeId + "-s"]), G && G._c && (G._d = !0), te;
  }
  function ensureValidVNode(O) {
    return O.some((_) => isVNode(_) ? !(_.type === Comment || _.type === Fragment && !ensureValidVNode(_.children)) : !0) ? O : null;
  }
  const getPublicInstance = (O) => O ? isStatefulComponent(O) ? getComponentPublicInstance(O) : getPublicInstance(O.parent) : null, publicPropertiesMap = (
    // Move PURE marker to new line to workaround compiler discarding it
    // due to type annotation
    /* @__PURE__ */ extend$a(/* @__PURE__ */ Object.create(null), {
      $: (O) => O,
      $el: (O) => O.vnode.el,
      $data: (O) => O.data,
      $props: (O) => O.props,
      $attrs: (O) => O.attrs,
      $slots: (O) => O.slots,
      $refs: (O) => O.refs,
      $parent: (O) => getPublicInstance(O.parent),
      $root: (O) => getPublicInstance(O.root),
      $host: (O) => O.ce,
      $emit: (O) => O.emit,
      $options: (O) => resolveMergedOptions(O),
      $forceUpdate: (O) => O.f || (O.f = () => {
        queueJob(O.update);
      }),
      $nextTick: (O) => O.n || (O.n = nextTick$1.bind(O.proxy)),
      $watch: (O) => instanceWatch.bind(O)
    })
  ), hasSetupBinding = (O, _) => O !== EMPTY_OBJ$1 && !O.__isScriptSetup && hasOwn(O, _), PublicInstanceProxyHandlers = {
    get({ _: O }, _) {
      if (_ === "__v_skip")
        return !0;
      const { ctx: H, setupState: V, data: q, props: G, accessCache: J, type: Z, appContext: te } = O;
      let ne;
      if (_[0] !== "$") {
        const se = J[_];
        if (se !== void 0)
          switch (se) {
            case 1:
              return V[_];
            case 2:
              return q[_];
            case 4:
              return H[_];
            case 3:
              return G[_];
          }
        else {
          if (hasSetupBinding(V, _))
            return J[_] = 1, V[_];
          if (q !== EMPTY_OBJ$1 && hasOwn(q, _))
            return J[_] = 2, q[_];
          if (
            // only cache other properties when instance has declared (thus stable)
            // props
            (ne = O.propsOptions[0]) && hasOwn(ne, _)
          )
            return J[_] = 3, G[_];
          if (H !== EMPTY_OBJ$1 && hasOwn(H, _))
            return J[_] = 4, H[_];
          shouldCacheAccess && (J[_] = 0);
        }
      }
      const ee = publicPropertiesMap[_];
      let ae, oe;
      if (ee)
        return _ === "$attrs" && track(O.attrs, "get", ""), ee(O);
      if (
        // css module (injected by vue-loader)
        (ae = Z.__cssModules) && (ae = ae[_])
      )
        return ae;
      if (H !== EMPTY_OBJ$1 && hasOwn(H, _))
        return J[_] = 4, H[_];
      if (
        // global properties
        oe = te.config.globalProperties, hasOwn(oe, _)
      )
        return oe[_];
    },
    set({ _: O }, _, H) {
      const { data: V, setupState: q, ctx: G } = O;
      return hasSetupBinding(q, _) ? (q[_] = H, !0) : V !== EMPTY_OBJ$1 && hasOwn(V, _) ? (V[_] = H, !0) : hasOwn(O.props, _) || _[0] === "$" && _.slice(1) in O ? !1 : (G[_] = H, !0);
    },
    has({
      _: { data: O, setupState: _, accessCache: H, ctx: V, appContext: q, propsOptions: G }
    }, J) {
      let Z;
      return !!H[J] || O !== EMPTY_OBJ$1 && hasOwn(O, J) || hasSetupBinding(_, J) || (Z = G[0]) && hasOwn(Z, J) || hasOwn(V, J) || hasOwn(publicPropertiesMap, J) || hasOwn(q.config.globalProperties, J);
    },
    defineProperty(O, _, H) {
      return H.get != null ? O._.accessCache[_] = 0 : hasOwn(H, "value") && this.set(O, _, H.value, null), Reflect.defineProperty(O, _, H);
    }
  };
  function normalizePropsOrEmits(O) {
    return isArray$6(O) ? O.reduce(
      (_, H) => (_[H] = null, _),
      {}
    ) : O;
  }
  let shouldCacheAccess = !0;
  function applyOptions(O) {
    const _ = resolveMergedOptions(O), H = O.proxy, V = O.ctx;
    shouldCacheAccess = !1, _.beforeCreate && callHook$1(_.beforeCreate, O, "bc");
    const {
      // state
      data: q,
      computed: G,
      methods: J,
      watch: Z,
      provide: te,
      inject: ne,
      // lifecycle
      created: ee,
      beforeMount: ae,
      mounted: oe,
      beforeUpdate: se,
      updated: ce,
      activated: le,
      deactivated: ue,
      beforeDestroy: de,
      beforeUnmount: fe,
      destroyed: he,
      unmounted: me,
      render: pe,
      renderTracked: ye,
      renderTriggered: ge,
      errorCaptured: ve,
      serverPrefetch: be,
      // public API
      expose: Ce,
      inheritAttrs: Se,
      // assets
      components: we,
      directives: Ee,
      filters: xe
    } = _;
    if (ne && resolveInjections(ne, V, null), J)
      for (const ke in J) {
        const Me = J[ke];
        isFunction$6(Me) && (V[ke] = Me.bind(H));
      }
    if (q) {
      const ke = q.call(H, H);
      isObject$7(ke) && (O.data = reactive(ke));
    }
    if (shouldCacheAccess = !0, G)
      for (const ke in G) {
        const Me = G[ke], Re = isFunction$6(Me) ? Me.bind(H, H) : isFunction$6(Me.get) ? Me.get.bind(H, H) : NOOP, De = !isFunction$6(Me) && isFunction$6(Me.set) ? Me.set.bind(H) : NOOP, Pe = computed({
          get: Re,
          set: De
        });
        Object.defineProperty(V, ke, {
          enumerable: !0,
          configurable: !0,
          get: () => Pe.value,
          set: ($e) => Pe.value = $e
        });
      }
    if (Z)
      for (const ke in Z)
        createWatcher(Z[ke], V, H, ke);
    if (te) {
      const ke = isFunction$6(te) ? te.call(H) : te;
      Reflect.ownKeys(ke).forEach((Me) => {
        provide(Me, ke[Me]);
      });
    }
    ee && callHook$1(ee, O, "c");
    function Qe(ke, Me) {
      isArray$6(Me) ? Me.forEach((Re) => ke(Re.bind(H))) : Me && ke(Me.bind(H));
    }
    if (Qe(onBeforeMount, ae), Qe(onMounted, oe), Qe(onBeforeUpdate, se), Qe(onUpdated, ce), Qe(onActivated, le), Qe(onDeactivated, ue), Qe(onErrorCaptured, ve), Qe(onRenderTracked, ye), Qe(onRenderTriggered, ge), Qe(onBeforeUnmount, fe), Qe(onUnmounted, me), Qe(onServerPrefetch, be), isArray$6(Ce))
      if (Ce.length) {
        const ke = O.exposed || (O.exposed = {});
        Ce.forEach((Me) => {
          Object.defineProperty(ke, Me, {
            get: () => H[Me],
            set: (Re) => H[Me] = Re
          });
        });
      } else O.exposed || (O.exposed = {});
    pe && O.render === NOOP && (O.render = pe), Se != null && (O.inheritAttrs = Se), we && (O.components = we), Ee && (O.directives = Ee), be && markAsyncBoundary(O);
  }
  function resolveInjections(O, _, H = NOOP) {
    isArray$6(O) && (O = normalizeInject(O));
    for (const V in O) {
      const q = O[V];
      let G;
      isObject$7(q) ? "default" in q ? G = inject(
        q.from || V,
        q.default,
        !0
      ) : G = inject(q.from || V) : G = inject(q), isRef(G) ? Object.defineProperty(_, V, {
        enumerable: !0,
        configurable: !0,
        get: () => G.value,
        set: (J) => G.value = J
      }) : _[V] = G;
    }
  }
  function callHook$1(O, _, H) {
    callWithAsyncErrorHandling(
      isArray$6(O) ? O.map((V) => V.bind(_.proxy)) : O.bind(_.proxy),
      _,
      H
    );
  }
  function createWatcher(O, _, H, V) {
    let q = V.includes(".") ? createPathGetter(H, V) : () => H[V];
    if (isString$5(O)) {
      const G = _[O];
      isFunction$6(G) && watch$1(q, G);
    } else if (isFunction$6(O))
      watch$1(q, O.bind(H));
    else if (isObject$7(O))
      if (isArray$6(O))
        O.forEach((G) => createWatcher(G, _, H, V));
      else {
        const G = isFunction$6(O.handler) ? O.handler.bind(H) : _[O.handler];
        isFunction$6(G) && watch$1(q, G, O);
      }
  }
  function resolveMergedOptions(O) {
    const _ = O.type, { mixins: H, extends: V } = _, {
      mixins: q,
      optionsCache: G,
      config: { optionMergeStrategies: J }
    } = O.appContext, Z = G.get(_);
    let te;
    return Z ? te = Z : !q.length && !H && !V ? te = _ : (te = {}, q.length && q.forEach(
      (ne) => mergeOptions(te, ne, J, !0)
    ), mergeOptions(te, _, J)), isObject$7(_) && G.set(_, te), te;
  }
  function mergeOptions(O, _, H, V = !1) {
    const { mixins: q, extends: G } = _;
    G && mergeOptions(O, G, H, !0), q && q.forEach(
      (J) => mergeOptions(O, J, H, !0)
    );
    for (const J in _)
      if (!(V && J === "expose")) {
        const Z = internalOptionMergeStrats[J] || H && H[J];
        O[J] = Z ? Z(O[J], _[J]) : _[J];
      }
    return O;
  }
  const internalOptionMergeStrats = {
    data: mergeDataFn,
    props: mergeEmitsOrPropsOptions,
    emits: mergeEmitsOrPropsOptions,
    // objects
    methods: mergeObjectOptions,
    computed: mergeObjectOptions,
    // lifecycle
    beforeCreate: mergeAsArray,
    created: mergeAsArray,
    beforeMount: mergeAsArray,
    mounted: mergeAsArray,
    beforeUpdate: mergeAsArray,
    updated: mergeAsArray,
    beforeDestroy: mergeAsArray,
    beforeUnmount: mergeAsArray,
    destroyed: mergeAsArray,
    unmounted: mergeAsArray,
    activated: mergeAsArray,
    deactivated: mergeAsArray,
    errorCaptured: mergeAsArray,
    serverPrefetch: mergeAsArray,
    // assets
    components: mergeObjectOptions,
    directives: mergeObjectOptions,
    // watch
    watch: mergeWatchOptions,
    // provide / inject
    provide: mergeDataFn,
    inject: mergeInject
  };
  function mergeDataFn(O, _) {
    return _ ? O ? function() {
      return extend$a(
        isFunction$6(O) ? O.call(this, this) : O,
        isFunction$6(_) ? _.call(this, this) : _
      );
    } : _ : O;
  }
  function mergeInject(O, _) {
    return mergeObjectOptions(normalizeInject(O), normalizeInject(_));
  }
  function normalizeInject(O) {
    if (isArray$6(O)) {
      const _ = {};
      for (let H = 0; H < O.length; H++)
        _[O[H]] = O[H];
      return _;
    }
    return O;
  }
  function mergeAsArray(O, _) {
    return O ? [...new Set([].concat(O, _))] : _;
  }
  function mergeObjectOptions(O, _) {
    return O ? extend$a(/* @__PURE__ */ Object.create(null), O, _) : _;
  }
  function mergeEmitsOrPropsOptions(O, _) {
    return O ? isArray$6(O) && isArray$6(_) ? [.../* @__PURE__ */ new Set([...O, ..._])] : extend$a(
      /* @__PURE__ */ Object.create(null),
      normalizePropsOrEmits(O),
      normalizePropsOrEmits(_ ?? {})
    ) : _;
  }
  function mergeWatchOptions(O, _) {
    if (!O) return _;
    if (!_) return O;
    const H = extend$a(/* @__PURE__ */ Object.create(null), O);
    for (const V in _)
      H[V] = mergeAsArray(O[V], _[V]);
    return H;
  }
  function createAppContext() {
    return {
      app: null,
      config: {
        isNativeTag: NO,
        performance: !1,
        globalProperties: {},
        optionMergeStrategies: {},
        errorHandler: void 0,
        warnHandler: void 0,
        compilerOptions: {}
      },
      mixins: [],
      components: {},
      directives: {},
      provides: /* @__PURE__ */ Object.create(null),
      optionsCache: /* @__PURE__ */ new WeakMap(),
      propsCache: /* @__PURE__ */ new WeakMap(),
      emitsCache: /* @__PURE__ */ new WeakMap()
    };
  }
  let uid$1 = 0;
  function createAppAPI(O, _) {
    return function(V, q = null) {
      isFunction$6(V) || (V = extend$a({}, V)), q != null && !isObject$7(q) && (q = null);
      const G = createAppContext(), J = /* @__PURE__ */ new WeakSet(), Z = [];
      let te = !1;
      const ne = G.app = {
        _uid: uid$1++,
        _component: V,
        _props: q,
        _container: null,
        _context: G,
        _instance: null,
        version: version$8,
        get config() {
          return G.config;
        },
        set config(ee) {
        },
        use(ee, ...ae) {
          return J.has(ee) || (ee && isFunction$6(ee.install) ? (J.add(ee), ee.install(ne, ...ae)) : isFunction$6(ee) && (J.add(ee), ee(ne, ...ae))), ne;
        },
        mixin(ee) {
          return G.mixins.includes(ee) || G.mixins.push(ee), ne;
        },
        component(ee, ae) {
          return ae ? (G.components[ee] = ae, ne) : G.components[ee];
        },
        directive(ee, ae) {
          return ae ? (G.directives[ee] = ae, ne) : G.directives[ee];
        },
        mount(ee, ae, oe) {
          if (!te) {
            const se = ne._ceVNode || createVNode(V, q);
            return se.appContext = G, oe === !0 ? oe = "svg" : oe === !1 && (oe = void 0), O(se, ee, oe), te = !0, ne._container = ee, ee.__vue_app__ = ne, getComponentPublicInstance(se.component);
          }
        },
        onUnmount(ee) {
          Z.push(ee);
        },
        unmount() {
          te && (callWithAsyncErrorHandling(
            Z,
            ne._instance,
            16
          ), O(null, ne._container), delete ne._container.__vue_app__);
        },
        provide(ee, ae) {
          return G.provides[ee] = ae, ne;
        },
        runWithContext(ee) {
          const ae = currentApp;
          currentApp = ne;
          try {
            return ee();
          } finally {
            currentApp = ae;
          }
        }
      };
      return ne;
    };
  }
  let currentApp = null;
  function provide(O, _) {
    if (currentInstance) {
      let H = currentInstance.provides;
      const V = currentInstance.parent && currentInstance.parent.provides;
      V === H && (H = currentInstance.provides = Object.create(V)), H[O] = _;
    }
  }
  function inject(O, _, H = !1) {
    const V = currentInstance || currentRenderingInstance;
    if (V || currentApp) {
      const q = currentApp ? currentApp._context.provides : V ? V.parent == null ? V.vnode.appContext && V.vnode.appContext.provides : V.parent.provides : void 0;
      if (q && O in q)
        return q[O];
      if (arguments.length > 1)
        return H && isFunction$6(_) ? _.call(V && V.proxy) : _;
    }
  }
  function hasInjectionContext() {
    return !!(currentInstance || currentRenderingInstance || currentApp);
  }
  const internalObjectProto = {}, createInternalObject = () => Object.create(internalObjectProto), isInternalObject = (O) => Object.getPrototypeOf(O) === internalObjectProto;
  function initProps(O, _, H, V = !1) {
    const q = {}, G = createInternalObject();
    O.propsDefaults = /* @__PURE__ */ Object.create(null), setFullProps(O, _, q, G);
    for (const J in O.propsOptions[0])
      J in q || (q[J] = void 0);
    H ? O.props = V ? q : shallowReactive(q) : O.type.props ? O.props = q : O.props = G, O.attrs = G;
  }
  function updateProps(O, _, H, V) {
    const {
      props: q,
      attrs: G,
      vnode: { patchFlag: J }
    } = O, Z = toRaw(q), [te] = O.propsOptions;
    let ne = !1;
    if (
      // always force full diff in dev
      // - #1942 if hmr is enabled with sfc component
      // - vite#872 non-sfc component used by sfc component
      (V || J > 0) && !(J & 16)
    ) {
      if (J & 8) {
        const ee = O.vnode.dynamicProps;
        for (let ae = 0; ae < ee.length; ae++) {
          let oe = ee[ae];
          if (isEmitListener(O.emitsOptions, oe))
            continue;
          const se = _[oe];
          if (te)
            if (hasOwn(G, oe))
              se !== G[oe] && (G[oe] = se, ne = !0);
            else {
              const ce = camelize(oe);
              q[ce] = resolvePropValue(
                te,
                Z,
                ce,
                se,
                O,
                !1
              );
            }
          else
            se !== G[oe] && (G[oe] = se, ne = !0);
        }
      }
    } else {
      setFullProps(O, _, q, G) && (ne = !0);
      let ee;
      for (const ae in Z)
        (!_ || // for camelCase
        !hasOwn(_, ae) && // it's possible the original props was passed in as kebab-case
        // and converted to camelCase (#955)
        ((ee = hyphenate$1(ae)) === ae || !hasOwn(_, ee))) && (te ? H && // for camelCase
        (H[ae] !== void 0 || // for kebab-case
        H[ee] !== void 0) && (q[ae] = resolvePropValue(
          te,
          Z,
          ae,
          void 0,
          O,
          !0
        )) : delete q[ae]);
      if (G !== Z)
        for (const ae in G)
          (!_ || !hasOwn(_, ae)) && (delete G[ae], ne = !0);
    }
    ne && trigger$1(O.attrs, "set", "");
  }
  function setFullProps(O, _, H, V) {
    const [q, G] = O.propsOptions;
    let J = !1, Z;
    if (_)
      for (let te in _) {
        if (isReservedProp(te))
          continue;
        const ne = _[te];
        let ee;
        q && hasOwn(q, ee = camelize(te)) ? !G || !G.includes(ee) ? H[ee] = ne : (Z || (Z = {}))[ee] = ne : isEmitListener(O.emitsOptions, te) || (!(te in V) || ne !== V[te]) && (V[te] = ne, J = !0);
      }
    if (G) {
      const te = toRaw(H), ne = Z || EMPTY_OBJ$1;
      for (let ee = 0; ee < G.length; ee++) {
        const ae = G[ee];
        H[ae] = resolvePropValue(
          q,
          te,
          ae,
          ne[ae],
          O,
          !hasOwn(ne, ae)
        );
      }
    }
    return J;
  }
  function resolvePropValue(O, _, H, V, q, G) {
    const J = O[H];
    if (J != null) {
      const Z = hasOwn(J, "default");
      if (Z && V === void 0) {
        const te = J.default;
        if (J.type !== Function && !J.skipFactory && isFunction$6(te)) {
          const { propsDefaults: ne } = q;
          if (H in ne)
            V = ne[H];
          else {
            const ee = setCurrentInstance(q);
            V = ne[H] = te.call(
              null,
              _
            ), ee();
          }
        } else
          V = te;
        q.ce && q.ce._setProp(H, V);
      }
      J[
        0
        /* shouldCast */
      ] && (G && !Z ? V = !1 : J[
        1
        /* shouldCastTrue */
      ] && (V === "" || V === hyphenate$1(H)) && (V = !0));
    }
    return V;
  }
  const mixinPropsCache = /* @__PURE__ */ new WeakMap();
  function normalizePropsOptions(O, _, H = !1) {
    const V = H ? mixinPropsCache : _.propsCache, q = V.get(O);
    if (q)
      return q;
    const G = O.props, J = {}, Z = [];
    let te = !1;
    if (!isFunction$6(O)) {
      const ee = (ae) => {
        te = !0;
        const [oe, se] = normalizePropsOptions(ae, _, !0);
        extend$a(J, oe), se && Z.push(...se);
      };
      !H && _.mixins.length && _.mixins.forEach(ee), O.extends && ee(O.extends), O.mixins && O.mixins.forEach(ee);
    }
    if (!G && !te)
      return isObject$7(O) && V.set(O, EMPTY_ARR), EMPTY_ARR;
    if (isArray$6(G))
      for (let ee = 0; ee < G.length; ee++) {
        const ae = camelize(G[ee]);
        validatePropName(ae) && (J[ae] = EMPTY_OBJ$1);
      }
    else if (G)
      for (const ee in G) {
        const ae = camelize(ee);
        if (validatePropName(ae)) {
          const oe = G[ee], se = J[ae] = isArray$6(oe) || isFunction$6(oe) ? { type: oe } : extend$a({}, oe), ce = se.type;
          let le = !1, ue = !0;
          if (isArray$6(ce))
            for (let de = 0; de < ce.length; ++de) {
              const fe = ce[de], he = isFunction$6(fe) && fe.name;
              if (he === "Boolean") {
                le = !0;
                break;
              } else he === "String" && (ue = !1);
            }
          else
            le = isFunction$6(ce) && ce.name === "Boolean";
          se[
            0
            /* shouldCast */
          ] = le, se[
            1
            /* shouldCastTrue */
          ] = ue, (le || hasOwn(se, "default")) && Z.push(ae);
        }
      }
    const ne = [J, Z];
    return isObject$7(O) && V.set(O, ne), ne;
  }
  function validatePropName(O) {
    return O[0] !== "$" && !isReservedProp(O);
  }
  const isInternalKey = (O) => O[0] === "_" || O === "$stable", normalizeSlotValue = (O) => isArray$6(O) ? O.map(normalizeVNode) : [normalizeVNode(O)], normalizeSlot = (O, _, H) => {
    if (_._n)
      return _;
    const V = withCtx((...q) => normalizeSlotValue(_(...q)), H);
    return V._c = !1, V;
  }, normalizeObjectSlots = (O, _, H) => {
    const V = O._ctx;
    for (const q in O) {
      if (isInternalKey(q)) continue;
      const G = O[q];
      if (isFunction$6(G))
        _[q] = normalizeSlot(q, G, V);
      else if (G != null) {
        const J = normalizeSlotValue(G);
        _[q] = () => J;
      }
    }
  }, normalizeVNodeSlots = (O, _) => {
    const H = normalizeSlotValue(_);
    O.slots.default = () => H;
  }, assignSlots = (O, _, H) => {
    for (const V in _)
      (H || V !== "_") && (O[V] = _[V]);
  }, initSlots = (O, _, H) => {
    const V = O.slots = createInternalObject();
    if (O.vnode.shapeFlag & 32) {
      const q = _._;
      q ? (assignSlots(V, _, H), H && def$2(V, "_", q, !0)) : normalizeObjectSlots(_, V);
    } else _ && normalizeVNodeSlots(O, _);
  }, updateSlots = (O, _, H) => {
    const { vnode: V, slots: q } = O;
    let G = !0, J = EMPTY_OBJ$1;
    if (V.shapeFlag & 32) {
      const Z = _._;
      Z ? H && Z === 1 ? G = !1 : assignSlots(q, _, H) : (G = !_.$stable, normalizeObjectSlots(_, q)), J = _;
    } else _ && (normalizeVNodeSlots(O, _), J = { default: 1 });
    if (G)
      for (const Z in q)
        !isInternalKey(Z) && J[Z] == null && delete q[Z];
  }, queuePostRenderEffect = queueEffectWithSuspense;
  function createRenderer(O) {
    return baseCreateRenderer(O);
  }
  function baseCreateRenderer(O, _) {
    const H = getGlobalThis();
    H.__VUE__ = !0;
    const {
      insert: V,
      remove: q,
      patchProp: G,
      createElement: J,
      createText: Z,
      createComment: te,
      setText: ne,
      setElementText: ee,
      parentNode: ae,
      nextSibling: oe,
      setScopeId: se = NOOP,
      insertStaticContent: ce
    } = O, le = (et, dt, Je, it = null, st = null, gt = null, bt = void 0, yt = null, St = !!dt.dynamicChildren) => {
      if (et === dt)
        return;
      et && !isSameVNodeType(et, dt) && (it = Ge(et), $e(et, st, gt, !0), et = null), dt.patchFlag === -2 && (St = !1, dt.dynamicChildren = null);
      const { type: Lt, ref: Rt, shapeFlag: vt } = dt;
      switch (Lt) {
        case Text$5:
          ue(et, dt, Je, it);
          break;
        case Comment:
          de(et, dt, Je, it);
          break;
        case Static:
          et == null && fe(dt, Je, it, bt);
          break;
        case Fragment:
          we(
            et,
            dt,
            Je,
            it,
            st,
            gt,
            bt,
            yt,
            St
          );
          break;
        default:
          vt & 1 ? pe(
            et,
            dt,
            Je,
            it,
            st,
            gt,
            bt,
            yt,
            St
          ) : vt & 6 ? Ee(
            et,
            dt,
            Je,
            it,
            st,
            gt,
            bt,
            yt,
            St
          ) : (vt & 64 || vt & 128) && Lt.process(
            et,
            dt,
            Je,
            it,
            st,
            gt,
            bt,
            yt,
            St,
            at
          );
      }
      Rt != null && st && setRef(Rt, et && et.ref, gt, dt || et, !dt);
    }, ue = (et, dt, Je, it) => {
      if (et == null)
        V(
          dt.el = Z(dt.children),
          Je,
          it
        );
      else {
        const st = dt.el = et.el;
        dt.children !== et.children && ne(st, dt.children);
      }
    }, de = (et, dt, Je, it) => {
      et == null ? V(
        dt.el = te(dt.children || ""),
        Je,
        it
      ) : dt.el = et.el;
    }, fe = (et, dt, Je, it) => {
      [et.el, et.anchor] = ce(
        et.children,
        dt,
        Je,
        it,
        et.el,
        et.anchor
      );
    }, he = ({ el: et, anchor: dt }, Je, it) => {
      let st;
      for (; et && et !== dt; )
        st = oe(et), V(et, Je, it), et = st;
      V(dt, Je, it);
    }, me = ({ el: et, anchor: dt }) => {
      let Je;
      for (; et && et !== dt; )
        Je = oe(et), q(et), et = Je;
      q(dt);
    }, pe = (et, dt, Je, it, st, gt, bt, yt, St) => {
      dt.type === "svg" ? bt = "svg" : dt.type === "math" && (bt = "mathml"), et == null ? ye(
        dt,
        Je,
        it,
        st,
        gt,
        bt,
        yt,
        St
      ) : be(
        et,
        dt,
        st,
        gt,
        bt,
        yt,
        St
      );
    }, ye = (et, dt, Je, it, st, gt, bt, yt) => {
      let St, Lt;
      const { props: Rt, shapeFlag: vt, transition: $t, dirs: Pt } = et;
      if (St = et.el = J(
        et.type,
        gt,
        Rt && Rt.is,
        Rt
      ), vt & 8 ? ee(St, et.children) : vt & 16 && ve(
        et.children,
        St,
        null,
        it,
        st,
        resolveChildrenNamespace(et, gt),
        bt,
        yt
      ), Pt && invokeDirectiveHook(et, null, it, "created"), ge(St, et, et.scopeId, bt, it), Rt) {
        for (const Nt in Rt)
          Nt !== "value" && !isReservedProp(Nt) && G(St, Nt, null, Rt[Nt], gt, it);
        "value" in Rt && G(St, "value", null, Rt.value, gt), (Lt = Rt.onVnodeBeforeMount) && invokeVNodeHook(Lt, it, et);
      }
      Pt && invokeDirectiveHook(et, null, it, "beforeMount");
      const Xt = needTransition(st, $t);
      Xt && $t.beforeEnter(St), V(St, dt, Je), ((Lt = Rt && Rt.onVnodeMounted) || Xt || Pt) && queuePostRenderEffect(() => {
        Lt && invokeVNodeHook(Lt, it, et), Xt && $t.enter(St), Pt && invokeDirectiveHook(et, null, it, "mounted");
      }, st);
    }, ge = (et, dt, Je, it, st) => {
      if (Je && se(et, Je), it)
        for (let gt = 0; gt < it.length; gt++)
          se(et, it[gt]);
      if (st) {
        let gt = st.subTree;
        if (dt === gt || isSuspense(gt.type) && (gt.ssContent === dt || gt.ssFallback === dt)) {
          const bt = st.vnode;
          ge(
            et,
            bt,
            bt.scopeId,
            bt.slotScopeIds,
            st.parent
          );
        }
      }
    }, ve = (et, dt, Je, it, st, gt, bt, yt, St = 0) => {
      for (let Lt = St; Lt < et.length; Lt++) {
        const Rt = et[Lt] = yt ? cloneIfMounted(et[Lt]) : normalizeVNode(et[Lt]);
        le(
          null,
          Rt,
          dt,
          Je,
          it,
          st,
          gt,
          bt,
          yt
        );
      }
    }, be = (et, dt, Je, it, st, gt, bt) => {
      const yt = dt.el = et.el;
      let { patchFlag: St, dynamicChildren: Lt, dirs: Rt } = dt;
      St |= et.patchFlag & 16;
      const vt = et.props || EMPTY_OBJ$1, $t = dt.props || EMPTY_OBJ$1;
      let Pt;
      if (Je && toggleRecurse(Je, !1), (Pt = $t.onVnodeBeforeUpdate) && invokeVNodeHook(Pt, Je, dt, et), Rt && invokeDirectiveHook(dt, et, Je, "beforeUpdate"), Je && toggleRecurse(Je, !0), (vt.innerHTML && $t.innerHTML == null || vt.textContent && $t.textContent == null) && ee(yt, ""), Lt ? Ce(
        et.dynamicChildren,
        Lt,
        yt,
        Je,
        it,
        resolveChildrenNamespace(dt, st),
        gt
      ) : bt || Me(
        et,
        dt,
        yt,
        null,
        Je,
        it,
        resolveChildrenNamespace(dt, st),
        gt,
        !1
      ), St > 0) {
        if (St & 16)
          Se(yt, vt, $t, Je, st);
        else if (St & 2 && vt.class !== $t.class && G(yt, "class", null, $t.class, st), St & 4 && G(yt, "style", vt.style, $t.style, st), St & 8) {
          const Xt = dt.dynamicProps;
          for (let Nt = 0; Nt < Xt.length; Nt++) {
            const Gt = Xt[Nt], Ft = vt[Gt], Tt = $t[Gt];
            (Tt !== Ft || Gt === "value") && G(yt, Gt, Ft, Tt, st, Je);
          }
        }
        St & 1 && et.children !== dt.children && ee(yt, dt.children);
      } else !bt && Lt == null && Se(yt, vt, $t, Je, st);
      ((Pt = $t.onVnodeUpdated) || Rt) && queuePostRenderEffect(() => {
        Pt && invokeVNodeHook(Pt, Je, dt, et), Rt && invokeDirectiveHook(dt, et, Je, "updated");
      }, it);
    }, Ce = (et, dt, Je, it, st, gt, bt) => {
      for (let yt = 0; yt < dt.length; yt++) {
        const St = et[yt], Lt = dt[yt], Rt = (
          // oldVNode may be an errored async setup() component inside Suspense
          // which will not have a mounted element
          St.el && // - In the case of a Fragment, we need to provide the actual parent
          // of the Fragment itself so it can move its children.
          (St.type === Fragment || // - In the case of different nodes, there is going to be a replacement
          // which also requires the correct parent container
          !isSameVNodeType(St, Lt) || // - In the case of a component, it could contain anything.
          St.shapeFlag & 70) ? ae(St.el) : (
            // In other cases, the parent container is not actually used so we
            // just pass the block element here to avoid a DOM parentNode call.
            Je
          )
        );
        le(
          St,
          Lt,
          Rt,
          null,
          it,
          st,
          gt,
          bt,
          !0
        );
      }
    }, Se = (et, dt, Je, it, st) => {
      if (dt !== Je) {
        if (dt !== EMPTY_OBJ$1)
          for (const gt in dt)
            !isReservedProp(gt) && !(gt in Je) && G(
              et,
              gt,
              dt[gt],
              null,
              st,
              it
            );
        for (const gt in Je) {
          if (isReservedProp(gt)) continue;
          const bt = Je[gt], yt = dt[gt];
          bt !== yt && gt !== "value" && G(et, gt, yt, bt, st, it);
        }
        "value" in Je && G(et, "value", dt.value, Je.value, st);
      }
    }, we = (et, dt, Je, it, st, gt, bt, yt, St) => {
      const Lt = dt.el = et ? et.el : Z(""), Rt = dt.anchor = et ? et.anchor : Z("");
      let { patchFlag: vt, dynamicChildren: $t, slotScopeIds: Pt } = dt;
      Pt && (yt = yt ? yt.concat(Pt) : Pt), et == null ? (V(Lt, Je, it), V(Rt, Je, it), ve(
        // #10007
        // such fragment like `<></>` will be compiled into
        // a fragment which doesn't have a children.
        // In this case fallback to an empty array
        dt.children || [],
        Je,
        Rt,
        st,
        gt,
        bt,
        yt,
        St
      )) : vt > 0 && vt & 64 && $t && // #2715 the previous fragment could've been a BAILed one as a result
      // of renderSlot() with no valid children
      et.dynamicChildren ? (Ce(
        et.dynamicChildren,
        $t,
        Je,
        st,
        gt,
        bt,
        yt
      ), // #2080 if the stable fragment has a key, it's a <template v-for> that may
      //  get moved around. Make sure all root level vnodes inherit el.
      // #2134 or if it's a component root, it may also get moved around
      // as the component is being moved.
      (dt.key != null || st && dt === st.subTree) && traverseStaticChildren(
        et,
        dt,
        !0
        /* shallow */
      )) : Me(
        et,
        dt,
        Je,
        Rt,
        st,
        gt,
        bt,
        yt,
        St
      );
    }, Ee = (et, dt, Je, it, st, gt, bt, yt, St) => {
      dt.slotScopeIds = yt, et == null ? dt.shapeFlag & 512 ? st.ctx.activate(
        dt,
        Je,
        it,
        bt,
        St
      ) : xe(
        dt,
        Je,
        it,
        st,
        gt,
        bt,
        St
      ) : Le(et, dt, St);
    }, xe = (et, dt, Je, it, st, gt, bt) => {
      const yt = et.component = createComponentInstance(
        et,
        it,
        st
      );
      if (isKeepAlive(et) && (yt.ctx.renderer = at), setupComponent(yt, !1, bt), yt.asyncDep) {
        if (st && st.registerDep(yt, Qe, bt), !et.el) {
          const St = yt.subTree = createVNode(Comment);
          de(null, St, dt, Je);
        }
      } else
        Qe(
          yt,
          et,
          dt,
          Je,
          st,
          gt,
          bt
        );
    }, Le = (et, dt, Je) => {
      const it = dt.component = et.component;
      if (shouldUpdateComponent(et, dt, Je))
        if (it.asyncDep && !it.asyncResolved) {
          ke(it, dt, Je);
          return;
        } else
          it.next = dt, it.update();
      else
        dt.el = et.el, it.vnode = dt;
    }, Qe = (et, dt, Je, it, st, gt, bt) => {
      const yt = () => {
        if (et.isMounted) {
          let { next: vt, bu: $t, u: Pt, parent: Xt, vnode: Nt } = et;
          {
            const Ct = locateNonHydratedAsyncRoot(et);
            if (Ct) {
              vt && (vt.el = Nt.el, ke(et, vt, bt)), Ct.asyncDep.then(() => {
                et.isUnmounted || yt();
              });
              return;
            }
          }
          let Gt = vt, Ft;
          toggleRecurse(et, !1), vt ? (vt.el = Nt.el, ke(et, vt, bt)) : vt = Nt, $t && invokeArrayFns($t), (Ft = vt.props && vt.props.onVnodeBeforeUpdate) && invokeVNodeHook(Ft, Xt, vt, Nt), toggleRecurse(et, !0);
          const Tt = renderComponentRoot(et), It = et.subTree;
          et.subTree = Tt, le(
            It,
            Tt,
            // parent may have changed if it's in a teleport
            ae(It.el),
            // anchor may have changed if it's in a fragment
            Ge(It),
            et,
            st,
            gt
          ), vt.el = Tt.el, Gt === null && updateHOCHostEl(et, Tt.el), Pt && queuePostRenderEffect(Pt, st), (Ft = vt.props && vt.props.onVnodeUpdated) && queuePostRenderEffect(
            () => invokeVNodeHook(Ft, Xt, vt, Nt),
            st
          );
        } else {
          let vt;
          const { el: $t, props: Pt } = dt, { bm: Xt, m: Nt, parent: Gt, root: Ft, type: Tt } = et, It = isAsyncWrapper(dt);
          toggleRecurse(et, !1), Xt && invokeArrayFns(Xt), !It && (vt = Pt && Pt.onVnodeBeforeMount) && invokeVNodeHook(vt, Gt, dt), toggleRecurse(et, !0);
          {
            Ft.ce && Ft.ce._injectChildStyle(Tt);
            const Ct = et.subTree = renderComponentRoot(et);
            le(
              null,
              Ct,
              Je,
              it,
              et,
              st,
              gt
            ), dt.el = Ct.el;
          }
          if (Nt && queuePostRenderEffect(Nt, st), !It && (vt = Pt && Pt.onVnodeMounted)) {
            const Ct = dt;
            queuePostRenderEffect(
              () => invokeVNodeHook(vt, Gt, Ct),
              st
            );
          }
          (dt.shapeFlag & 256 || Gt && isAsyncWrapper(Gt.vnode) && Gt.vnode.shapeFlag & 256) && et.a && queuePostRenderEffect(et.a, st), et.isMounted = !0, dt = Je = it = null;
        }
      };
      et.scope.on();
      const St = et.effect = new ReactiveEffect(yt);
      et.scope.off();
      const Lt = et.update = St.run.bind(St), Rt = et.job = St.runIfDirty.bind(St);
      Rt.i = et, Rt.id = et.uid, St.scheduler = () => queueJob(Rt), toggleRecurse(et, !0), Lt();
    }, ke = (et, dt, Je) => {
      dt.component = et;
      const it = et.vnode.props;
      et.vnode = dt, et.next = null, updateProps(et, dt.props, it, Je), updateSlots(et, dt.children, Je), pauseTracking(), flushPreFlushCbs(et), resetTracking();
    }, Me = (et, dt, Je, it, st, gt, bt, yt, St = !1) => {
      const Lt = et && et.children, Rt = et ? et.shapeFlag : 0, vt = dt.children, { patchFlag: $t, shapeFlag: Pt } = dt;
      if ($t > 0) {
        if ($t & 128) {
          De(
            Lt,
            vt,
            Je,
            it,
            st,
            gt,
            bt,
            yt,
            St
          );
          return;
        } else if ($t & 256) {
          Re(
            Lt,
            vt,
            Je,
            it,
            st,
            gt,
            bt,
            yt,
            St
          );
          return;
        }
      }
      Pt & 8 ? (Rt & 16 && Ze(Lt, st, gt), vt !== Lt && ee(Je, vt)) : Rt & 16 ? Pt & 16 ? De(
        Lt,
        vt,
        Je,
        it,
        st,
        gt,
        bt,
        yt,
        St
      ) : Ze(Lt, st, gt, !0) : (Rt & 8 && ee(Je, ""), Pt & 16 && ve(
        vt,
        Je,
        it,
        st,
        gt,
        bt,
        yt,
        St
      ));
    }, Re = (et, dt, Je, it, st, gt, bt, yt, St) => {
      et = et || EMPTY_ARR, dt = dt || EMPTY_ARR;
      const Lt = et.length, Rt = dt.length, vt = Math.min(Lt, Rt);
      let $t;
      for ($t = 0; $t < vt; $t++) {
        const Pt = dt[$t] = St ? cloneIfMounted(dt[$t]) : normalizeVNode(dt[$t]);
        le(
          et[$t],
          Pt,
          Je,
          null,
          st,
          gt,
          bt,
          yt,
          St
        );
      }
      Lt > Rt ? Ze(
        et,
        st,
        gt,
        !0,
        !1,
        vt
      ) : ve(
        dt,
        Je,
        it,
        st,
        gt,
        bt,
        yt,
        St,
        vt
      );
    }, De = (et, dt, Je, it, st, gt, bt, yt, St) => {
      let Lt = 0;
      const Rt = dt.length;
      let vt = et.length - 1, $t = Rt - 1;
      for (; Lt <= vt && Lt <= $t; ) {
        const Pt = et[Lt], Xt = dt[Lt] = St ? cloneIfMounted(dt[Lt]) : normalizeVNode(dt[Lt]);
        if (isSameVNodeType(Pt, Xt))
          le(
            Pt,
            Xt,
            Je,
            null,
            st,
            gt,
            bt,
            yt,
            St
          );
        else
          break;
        Lt++;
      }
      for (; Lt <= vt && Lt <= $t; ) {
        const Pt = et[vt], Xt = dt[$t] = St ? cloneIfMounted(dt[$t]) : normalizeVNode(dt[$t]);
        if (isSameVNodeType(Pt, Xt))
          le(
            Pt,
            Xt,
            Je,
            null,
            st,
            gt,
            bt,
            yt,
            St
          );
        else
          break;
        vt--, $t--;
      }
      if (Lt > vt) {
        if (Lt <= $t) {
          const Pt = $t + 1, Xt = Pt < Rt ? dt[Pt].el : it;
          for (; Lt <= $t; )
            le(
              null,
              dt[Lt] = St ? cloneIfMounted(dt[Lt]) : normalizeVNode(dt[Lt]),
              Je,
              Xt,
              st,
              gt,
              bt,
              yt,
              St
            ), Lt++;
        }
      } else if (Lt > $t)
        for (; Lt <= vt; )
          $e(et[Lt], st, gt, !0), Lt++;
      else {
        const Pt = Lt, Xt = Lt, Nt = /* @__PURE__ */ new Map();
        for (Lt = Xt; Lt <= $t; Lt++) {
          const Yt = dt[Lt] = St ? cloneIfMounted(dt[Lt]) : normalizeVNode(dt[Lt]);
          Yt.key != null && Nt.set(Yt.key, Lt);
        }
        let Gt, Ft = 0;
        const Tt = $t - Xt + 1;
        let It = !1, Ct = 0;
        const Kt = new Array(Tt);
        for (Lt = 0; Lt < Tt; Lt++) Kt[Lt] = 0;
        for (Lt = Pt; Lt <= vt; Lt++) {
          const Yt = et[Lt];
          if (Ft >= Tt) {
            $e(Yt, st, gt, !0);
            continue;
          }
          let sn;
          if (Yt.key != null)
            sn = Nt.get(Yt.key);
          else
            for (Gt = Xt; Gt <= $t; Gt++)
              if (Kt[Gt - Xt] === 0 && isSameVNodeType(Yt, dt[Gt])) {
                sn = Gt;
                break;
              }
          sn === void 0 ? $e(Yt, st, gt, !0) : (Kt[sn - Xt] = Lt + 1, sn >= Ct ? Ct = sn : It = !0, le(
            Yt,
            dt[sn],
            Je,
            null,
            st,
            gt,
            bt,
            yt,
            St
          ), Ft++);
        }
        const Vt = It ? getSequence(Kt) : EMPTY_ARR;
        for (Gt = Vt.length - 1, Lt = Tt - 1; Lt >= 0; Lt--) {
          const Yt = Xt + Lt, sn = dt[Yt], hn = Yt + 1 < Rt ? dt[Yt + 1].el : it;
          Kt[Lt] === 0 ? le(
            null,
            sn,
            Je,
            hn,
            st,
            gt,
            bt,
            yt,
            St
          ) : It && (Gt < 0 || Lt !== Vt[Gt] ? Pe(sn, Je, hn, 2) : Gt--);
        }
      }
    }, Pe = (et, dt, Je, it, st = null) => {
      const { el: gt, type: bt, transition: yt, children: St, shapeFlag: Lt } = et;
      if (Lt & 6) {
        Pe(et.component.subTree, dt, Je, it);
        return;
      }
      if (Lt & 128) {
        et.suspense.move(dt, Je, it);
        return;
      }
      if (Lt & 64) {
        bt.move(et, dt, Je, at);
        return;
      }
      if (bt === Fragment) {
        V(gt, dt, Je);
        for (let vt = 0; vt < St.length; vt++)
          Pe(St[vt], dt, Je, it);
        V(et.anchor, dt, Je);
        return;
      }
      if (bt === Static) {
        he(et, dt, Je);
        return;
      }
      if (it !== 2 && Lt & 1 && yt)
        if (it === 0)
          yt.beforeEnter(gt), V(gt, dt, Je), queuePostRenderEffect(() => yt.enter(gt), st);
        else {
          const { leave: vt, delayLeave: $t, afterLeave: Pt } = yt, Xt = () => V(gt, dt, Je), Nt = () => {
            vt(gt, () => {
              Xt(), Pt && Pt();
            });
          };
          $t ? $t(gt, Xt, Nt) : Nt();
        }
      else
        V(gt, dt, Je);
    }, $e = (et, dt, Je, it = !1, st = !1) => {
      const {
        type: gt,
        props: bt,
        ref: yt,
        children: St,
        dynamicChildren: Lt,
        shapeFlag: Rt,
        patchFlag: vt,
        dirs: $t,
        cacheIndex: Pt
      } = et;
      if (vt === -2 && (st = !1), yt != null && setRef(yt, null, Je, et, !0), Pt != null && (dt.renderCache[Pt] = void 0), Rt & 256) {
        dt.ctx.deactivate(et);
        return;
      }
      const Xt = Rt & 1 && $t, Nt = !isAsyncWrapper(et);
      let Gt;
      if (Nt && (Gt = bt && bt.onVnodeBeforeUnmount) && invokeVNodeHook(Gt, dt, et), Rt & 6)
        Ve(et.component, Je, it);
      else {
        if (Rt & 128) {
          et.suspense.unmount(Je, it);
          return;
        }
        Xt && invokeDirectiveHook(et, null, dt, "beforeUnmount"), Rt & 64 ? et.type.remove(
          et,
          dt,
          Je,
          at,
          it
        ) : Lt && // #5154
        // when v-once is used inside a block, setBlockTracking(-1) marks the
        // parent block with hasOnce: true
        // so that it doesn't take the fast path during unmount - otherwise
        // components nested in v-once are never unmounted.
        !Lt.hasOnce && // #1153: fast path should not be taken for non-stable (v-for) fragments
        (gt !== Fragment || vt > 0 && vt & 64) ? Ze(
          Lt,
          dt,
          Je,
          !1,
          !0
        ) : (gt === Fragment && vt & 384 || !st && Rt & 16) && Ze(St, dt, Je), it && qe(et);
      }
      (Nt && (Gt = bt && bt.onVnodeUnmounted) || Xt) && queuePostRenderEffect(() => {
        Gt && invokeVNodeHook(Gt, dt, et), Xt && invokeDirectiveHook(et, null, dt, "unmounted");
      }, Je);
    }, qe = (et) => {
      const { type: dt, el: Je, anchor: it, transition: st } = et;
      if (dt === Fragment) {
        Xe(Je, it);
        return;
      }
      if (dt === Static) {
        me(et);
        return;
      }
      const gt = () => {
        q(Je), st && !st.persisted && st.afterLeave && st.afterLeave();
      };
      if (et.shapeFlag & 1 && st && !st.persisted) {
        const { leave: bt, delayLeave: yt } = st, St = () => bt(Je, gt);
        yt ? yt(et.el, gt, St) : St();
      } else
        gt();
    }, Xe = (et, dt) => {
      let Je;
      for (; et !== dt; )
        Je = oe(et), q(et), et = Je;
      q(dt);
    }, Ve = (et, dt, Je) => {
      const { bum: it, scope: st, job: gt, subTree: bt, um: yt, m: St, a: Lt } = et;
      invalidateMount(St), invalidateMount(Lt), it && invokeArrayFns(it), st.stop(), gt && (gt.flags |= 8, $e(bt, et, dt, Je)), yt && queuePostRenderEffect(yt, dt), queuePostRenderEffect(() => {
        et.isUnmounted = !0;
      }, dt), dt && dt.pendingBranch && !dt.isUnmounted && et.asyncDep && !et.asyncResolved && et.suspenseId === dt.pendingId && (dt.deps--, dt.deps === 0 && dt.resolve());
    }, Ze = (et, dt, Je, it = !1, st = !1, gt = 0) => {
      for (let bt = gt; bt < et.length; bt++)
        $e(et[bt], dt, Je, it, st);
    }, Ge = (et) => {
      if (et.shapeFlag & 6)
        return Ge(et.component.subTree);
      if (et.shapeFlag & 128)
        return et.suspense.next();
      const dt = oe(et.anchor || et.el), Je = dt && dt[TeleportEndKey];
      return Je ? oe(Je) : dt;
    };
    let ze = !1;
    const We = (et, dt, Je) => {
      et == null ? dt._vnode && $e(dt._vnode, null, null, !0) : le(
        dt._vnode || null,
        et,
        dt,
        null,
        null,
        null,
        Je
      ), dt._vnode = et, ze || (ze = !0, flushPreFlushCbs(), flushPostFlushCbs(), ze = !1);
    }, at = {
      p: le,
      um: $e,
      m: Pe,
      r: qe,
      mt: xe,
      mc: ve,
      pc: Me,
      pbc: Ce,
      n: Ge,
      o: O
    };
    return {
      render: We,
      hydrate: void 0,
      createApp: createAppAPI(We)
    };
  }
  function resolveChildrenNamespace({ type: O, props: _ }, H) {
    return H === "svg" && O === "foreignObject" || H === "mathml" && O === "annotation-xml" && _ && _.encoding && _.encoding.includes("html") ? void 0 : H;
  }
  function toggleRecurse({ effect: O, job: _ }, H) {
    H ? (O.flags |= 32, _.flags |= 4) : (O.flags &= -33, _.flags &= -5);
  }
  function needTransition(O, _) {
    return (!O || O && !O.pendingBranch) && _ && !_.persisted;
  }
  function traverseStaticChildren(O, _, H = !1) {
    const V = O.children, q = _.children;
    if (isArray$6(V) && isArray$6(q))
      for (let G = 0; G < V.length; G++) {
        const J = V[G];
        let Z = q[G];
        Z.shapeFlag & 1 && !Z.dynamicChildren && ((Z.patchFlag <= 0 || Z.patchFlag === 32) && (Z = q[G] = cloneIfMounted(q[G]), Z.el = J.el), !H && Z.patchFlag !== -2 && traverseStaticChildren(J, Z)), Z.type === Text$5 && (Z.el = J.el);
      }
  }
  function getSequence(O) {
    const _ = O.slice(), H = [0];
    let V, q, G, J, Z;
    const te = O.length;
    for (V = 0; V < te; V++) {
      const ne = O[V];
      if (ne !== 0) {
        if (q = H[H.length - 1], O[q] < ne) {
          _[V] = q, H.push(V);
          continue;
        }
        for (G = 0, J = H.length - 1; G < J; )
          Z = G + J >> 1, O[H[Z]] < ne ? G = Z + 1 : J = Z;
        ne < O[H[G]] && (G > 0 && (_[V] = H[G - 1]), H[G] = V);
      }
    }
    for (G = H.length, J = H[G - 1]; G-- > 0; )
      H[G] = J, J = _[J];
    return H;
  }
  function locateNonHydratedAsyncRoot(O) {
    const _ = O.subTree.component;
    if (_)
      return _.asyncDep && !_.asyncResolved ? _ : locateNonHydratedAsyncRoot(_);
  }
  function invalidateMount(O) {
    if (O)
      for (let _ = 0; _ < O.length; _++)
        O[_].flags |= 8;
  }
  const ssrContextKey = Symbol.for("v-scx"), useSSRContext = () => inject(ssrContextKey);
  function watchEffect(O, _) {
    return doWatch(O, null, _);
  }
  function watch$1(O, _, H) {
    return doWatch(O, _, H);
  }
  function doWatch(O, _, H = EMPTY_OBJ$1) {
    const { immediate: V, deep: q, flush: G, once: J } = H, Z = extend$a({}, H), te = _ && V || !_ && G !== "post";
    let ne;
    if (isInSSRComponentSetup) {
      if (G === "sync") {
        const se = useSSRContext();
        ne = se.__watcherHandles || (se.__watcherHandles = []);
      } else if (!te) {
        const se = () => {
        };
        return se.stop = NOOP, se.resume = NOOP, se.pause = NOOP, se;
      }
    }
    const ee = currentInstance;
    Z.call = (se, ce, le) => callWithAsyncErrorHandling(se, ee, ce, le);
    let ae = !1;
    G === "post" ? Z.scheduler = (se) => {
      queuePostRenderEffect(se, ee && ee.suspense);
    } : G !== "sync" && (ae = !0, Z.scheduler = (se, ce) => {
      ce ? se() : queueJob(se);
    }), Z.augmentJob = (se) => {
      _ && (se.flags |= 4), ae && (se.flags |= 2, ee && (se.id = ee.uid, se.i = ee));
    };
    const oe = watch$2(O, _, Z);
    return isInSSRComponentSetup && (ne ? ne.push(oe) : te && oe()), oe;
  }
  function instanceWatch(O, _, H) {
    const V = this.proxy, q = isString$5(O) ? O.includes(".") ? createPathGetter(V, O) : () => V[O] : O.bind(V, V);
    let G;
    isFunction$6(_) ? G = _ : (G = _.handler, H = _);
    const J = setCurrentInstance(this), Z = doWatch(q, G.bind(V), H);
    return J(), Z;
  }
  function createPathGetter(O, _) {
    const H = _.split(".");
    return () => {
      let V = O;
      for (let q = 0; q < H.length && V; q++)
        V = V[H[q]];
      return V;
    };
  }
  const getModelModifiers = (O, _) => _ === "modelValue" || _ === "model-value" ? O.modelModifiers : O[`${_}Modifiers`] || O[`${camelize(_)}Modifiers`] || O[`${hyphenate$1(_)}Modifiers`];
  function emit$2(O, _, ...H) {
    if (O.isUnmounted) return;
    const V = O.vnode.props || EMPTY_OBJ$1;
    let q = H;
    const G = _.startsWith("update:"), J = G && getModelModifiers(V, _.slice(7));
    J && (J.trim && (q = H.map((ee) => isString$5(ee) ? ee.trim() : ee)), J.number && (q = H.map(looseToNumber)));
    let Z, te = V[Z = toHandlerKey(_)] || // also try camelCase event handler (#2249)
    V[Z = toHandlerKey(camelize(_))];
    !te && G && (te = V[Z = toHandlerKey(hyphenate$1(_))]), te && callWithAsyncErrorHandling(
      te,
      O,
      6,
      q
    );
    const ne = V[Z + "Once"];
    if (ne) {
      if (!O.emitted)
        O.emitted = {};
      else if (O.emitted[Z])
        return;
      O.emitted[Z] = !0, callWithAsyncErrorHandling(
        ne,
        O,
        6,
        q
      );
    }
  }
  function normalizeEmitsOptions(O, _, H = !1) {
    const V = _.emitsCache, q = V.get(O);
    if (q !== void 0)
      return q;
    const G = O.emits;
    let J = {}, Z = !1;
    if (!isFunction$6(O)) {
      const te = (ne) => {
        const ee = normalizeEmitsOptions(ne, _, !0);
        ee && (Z = !0, extend$a(J, ee));
      };
      !H && _.mixins.length && _.mixins.forEach(te), O.extends && te(O.extends), O.mixins && O.mixins.forEach(te);
    }
    return !G && !Z ? (isObject$7(O) && V.set(O, null), null) : (isArray$6(G) ? G.forEach((te) => J[te] = null) : extend$a(J, G), isObject$7(O) && V.set(O, J), J);
  }
  function isEmitListener(O, _) {
    return !O || !isOn(_) ? !1 : (_ = _.slice(2).replace(/Once$/, ""), hasOwn(O, _[0].toLowerCase() + _.slice(1)) || hasOwn(O, hyphenate$1(_)) || hasOwn(O, _));
  }
  function markAttrsAccessed() {
  }
  function renderComponentRoot(O) {
    const {
      type: _,
      vnode: H,
      proxy: V,
      withProxy: q,
      propsOptions: [G],
      slots: J,
      attrs: Z,
      emit: te,
      render: ne,
      renderCache: ee,
      props: ae,
      data: oe,
      setupState: se,
      ctx: ce,
      inheritAttrs: le
    } = O, ue = setCurrentRenderingInstance(O);
    let de, fe;
    try {
      if (H.shapeFlag & 4) {
        const me = q || V, pe = me;
        de = normalizeVNode(
          ne.call(
            pe,
            me,
            ee,
            ae,
            se,
            oe,
            ce
          )
        ), fe = Z;
      } else {
        const me = _;
        de = normalizeVNode(
          me.length > 1 ? me(
            ae,
            { attrs: Z, slots: J, emit: te }
          ) : me(
            ae,
            null
          )
        ), fe = _.props ? Z : getFunctionalFallthrough(Z);
      }
    } catch (me) {
      blockStack.length = 0, handleError(me, O, 1), de = createVNode(Comment);
    }
    let he = de;
    if (fe && le !== !1) {
      const me = Object.keys(fe), { shapeFlag: pe } = he;
      me.length && pe & 7 && (G && me.some(isModelListener) && (fe = filterModelListeners(
        fe,
        G
      )), he = cloneVNode(he, fe, !1, !0));
    }
    return H.dirs && (he = cloneVNode(he, null, !1, !0), he.dirs = he.dirs ? he.dirs.concat(H.dirs) : H.dirs), H.transition && setTransitionHooks(he, H.transition), de = he, setCurrentRenderingInstance(ue), de;
  }
  const getFunctionalFallthrough = (O) => {
    let _;
    for (const H in O)
      (H === "class" || H === "style" || isOn(H)) && ((_ || (_ = {}))[H] = O[H]);
    return _;
  }, filterModelListeners = (O, _) => {
    const H = {};
    for (const V in O)
      (!isModelListener(V) || !(V.slice(9) in _)) && (H[V] = O[V]);
    return H;
  };
  function shouldUpdateComponent(O, _, H) {
    const { props: V, children: q, component: G } = O, { props: J, children: Z, patchFlag: te } = _, ne = G.emitsOptions;
    if (_.dirs || _.transition)
      return !0;
    if (H && te >= 0) {
      if (te & 1024)
        return !0;
      if (te & 16)
        return V ? hasPropsChanged(V, J, ne) : !!J;
      if (te & 8) {
        const ee = _.dynamicProps;
        for (let ae = 0; ae < ee.length; ae++) {
          const oe = ee[ae];
          if (J[oe] !== V[oe] && !isEmitListener(ne, oe))
            return !0;
        }
      }
    } else
      return (q || Z) && (!Z || !Z.$stable) ? !0 : V === J ? !1 : V ? J ? hasPropsChanged(V, J, ne) : !0 : !!J;
    return !1;
  }
  function hasPropsChanged(O, _, H) {
    const V = Object.keys(_);
    if (V.length !== Object.keys(O).length)
      return !0;
    for (let q = 0; q < V.length; q++) {
      const G = V[q];
      if (_[G] !== O[G] && !isEmitListener(H, G))
        return !0;
    }
    return !1;
  }
  function updateHOCHostEl({ vnode: O, parent: _ }, H) {
    for (; _; ) {
      const V = _.subTree;
      if (V.suspense && V.suspense.activeBranch === O && (V.el = O.el), V === O)
        (O = _.vnode).el = H, _ = _.parent;
      else
        break;
    }
  }
  const isSuspense = (O) => O.__isSuspense;
  function queueEffectWithSuspense(O, _) {
    _ && _.pendingBranch ? isArray$6(O) ? _.effects.push(...O) : _.effects.push(O) : queuePostFlushCb(O);
  }
  const Fragment = Symbol.for("v-fgt"), Text$5 = Symbol.for("v-txt"), Comment = Symbol.for("v-cmt"), Static = Symbol.for("v-stc"), blockStack = [];
  let currentBlock = null;
  function openBlock(O = !1) {
    blockStack.push(currentBlock = O ? null : []);
  }
  function closeBlock() {
    blockStack.pop(), currentBlock = blockStack[blockStack.length - 1] || null;
  }
  let isBlockTreeEnabled = 1;
  function setBlockTracking(O, _ = !1) {
    isBlockTreeEnabled += O, O < 0 && currentBlock && _ && (currentBlock.hasOnce = !0);
  }
  function setupBlock(O) {
    return O.dynamicChildren = isBlockTreeEnabled > 0 ? currentBlock || EMPTY_ARR : null, closeBlock(), isBlockTreeEnabled > 0 && currentBlock && currentBlock.push(O), O;
  }
  function createElementBlock(O, _, H, V, q, G) {
    return setupBlock(
      createBaseVNode(
        O,
        _,
        H,
        V,
        q,
        G,
        !0
      )
    );
  }
  function createBlock(O, _, H, V, q) {
    return setupBlock(
      createVNode(
        O,
        _,
        H,
        V,
        q,
        !0
      )
    );
  }
  function isVNode(O) {
    return O ? O.__v_isVNode === !0 : !1;
  }
  function isSameVNodeType(O, _) {
    return O.type === _.type && O.key === _.key;
  }
  const normalizeKey = ({ key: O }) => O ?? null, normalizeRef = ({
    ref: O,
    ref_key: _,
    ref_for: H
  }) => (typeof O == "number" && (O = "" + O), O != null ? isString$5(O) || isRef(O) || isFunction$6(O) ? { i: currentRenderingInstance, r: O, k: _, f: !!H } : O : null);
  function createBaseVNode(O, _ = null, H = null, V = 0, q = null, G = O === Fragment ? 0 : 1, J = !1, Z = !1) {
    const te = {
      __v_isVNode: !0,
      __v_skip: !0,
      type: O,
      props: _,
      key: _ && normalizeKey(_),
      ref: _ && normalizeRef(_),
      scopeId: currentScopeId,
      slotScopeIds: null,
      children: H,
      component: null,
      suspense: null,
      ssContent: null,
      ssFallback: null,
      dirs: null,
      transition: null,
      el: null,
      anchor: null,
      target: null,
      targetStart: null,
      targetAnchor: null,
      staticCount: 0,
      shapeFlag: G,
      patchFlag: V,
      dynamicProps: q,
      dynamicChildren: null,
      appContext: null,
      ctx: currentRenderingInstance
    };
    return Z ? (normalizeChildren(te, H), G & 128 && O.normalize(te)) : H && (te.shapeFlag |= isString$5(H) ? 8 : 16), isBlockTreeEnabled > 0 && // avoid a block node from tracking itself
    !J && // has current parent block
    currentBlock && // presence of a patch flag indicates this node needs patching on updates.
    // component nodes also should always be patched, because even if the
    // component doesn't need to update, it needs to persist the instance on to
    // the next vnode so that it can be properly unmounted later.
    (te.patchFlag > 0 || G & 6) && // the EVENTS flag is only for hydration and if it is the only flag, the
    // vnode should not be considered dynamic due to handler caching.
    te.patchFlag !== 32 && currentBlock.push(te), te;
  }
  const createVNode = _createVNode;
  function _createVNode(O, _ = null, H = null, V = 0, q = null, G = !1) {
    if ((!O || O === NULL_DYNAMIC_COMPONENT) && (O = Comment), isVNode(O)) {
      const Z = cloneVNode(
        O,
        _,
        !0
        /* mergeRef: true */
      );
      return H && normalizeChildren(Z, H), isBlockTreeEnabled > 0 && !G && currentBlock && (Z.shapeFlag & 6 ? currentBlock[currentBlock.indexOf(O)] = Z : currentBlock.push(Z)), Z.patchFlag = -2, Z;
    }
    if (isClassComponent(O) && (O = O.__vccOpts), _) {
      _ = guardReactiveProps(_);
      let { class: Z, style: te } = _;
      Z && !isString$5(Z) && (_.class = normalizeClass(Z)), isObject$7(te) && (isProxy(te) && !isArray$6(te) && (te = extend$a({}, te)), _.style = normalizeStyle(te));
    }
    const J = isString$5(O) ? 1 : isSuspense(O) ? 128 : isTeleport(O) ? 64 : isObject$7(O) ? 4 : isFunction$6(O) ? 2 : 0;
    return createBaseVNode(
      O,
      _,
      H,
      V,
      q,
      J,
      G,
      !0
    );
  }
  function guardReactiveProps(O) {
    return O ? isProxy(O) || isInternalObject(O) ? extend$a({}, O) : O : null;
  }
  function cloneVNode(O, _, H = !1, V = !1) {
    const { props: q, ref: G, patchFlag: J, children: Z, transition: te } = O, ne = _ ? mergeProps$1(q || {}, _) : q, ee = {
      __v_isVNode: !0,
      __v_skip: !0,
      type: O.type,
      props: ne,
      key: ne && normalizeKey(ne),
      ref: _ && _.ref ? (
        // #2078 in the case of <component :is="vnode" ref="extra"/>
        // if the vnode itself already has a ref, cloneVNode will need to merge
        // the refs so the single vnode can be set on multiple refs
        H && G ? isArray$6(G) ? G.concat(normalizeRef(_)) : [G, normalizeRef(_)] : normalizeRef(_)
      ) : G,
      scopeId: O.scopeId,
      slotScopeIds: O.slotScopeIds,
      children: Z,
      target: O.target,
      targetStart: O.targetStart,
      targetAnchor: O.targetAnchor,
      staticCount: O.staticCount,
      shapeFlag: O.shapeFlag,
      // if the vnode is cloned with extra props, we can no longer assume its
      // existing patch flag to be reliable and need to add the FULL_PROPS flag.
      // note: preserve flag for fragments since they use the flag for children
      // fast paths only.
      patchFlag: _ && O.type !== Fragment ? J === -1 ? 16 : J | 16 : J,
      dynamicProps: O.dynamicProps,
      dynamicChildren: O.dynamicChildren,
      appContext: O.appContext,
      dirs: O.dirs,
      transition: te,
      // These should technically only be non-null on mounted VNodes. However,
      // they *should* be copied for kept-alive vnodes. So we just always copy
      // them since them being non-null during a mount doesn't affect the logic as
      // they will simply be overwritten.
      component: O.component,
      suspense: O.suspense,
      ssContent: O.ssContent && cloneVNode(O.ssContent),
      ssFallback: O.ssFallback && cloneVNode(O.ssFallback),
      el: O.el,
      anchor: O.anchor,
      ctx: O.ctx,
      ce: O.ce
    };
    return te && V && setTransitionHooks(
      ee,
      te.clone(ee)
    ), ee;
  }
  function createTextVNode(O = " ", _ = 0) {
    return createVNode(Text$5, null, O, _);
  }
  function createCommentVNode(O = "", _ = !1) {
    return _ ? (openBlock(), createBlock(Comment, null, O)) : createVNode(Comment, null, O);
  }
  function normalizeVNode(O) {
    return O == null || typeof O == "boolean" ? createVNode(Comment) : isArray$6(O) ? createVNode(
      Fragment,
      null,
      // #3666, avoid reference pollution when reusing vnode
      O.slice()
    ) : isVNode(O) ? cloneIfMounted(O) : createVNode(Text$5, null, String(O));
  }
  function cloneIfMounted(O) {
    return O.el === null && O.patchFlag !== -1 || O.memo ? O : cloneVNode(O);
  }
  function normalizeChildren(O, _) {
    let H = 0;
    const { shapeFlag: V } = O;
    if (_ == null)
      _ = null;
    else if (isArray$6(_))
      H = 16;
    else if (typeof _ == "object")
      if (V & 65) {
        const q = _.default;
        q && (q._c && (q._d = !1), normalizeChildren(O, q()), q._c && (q._d = !0));
        return;
      } else {
        H = 32;
        const q = _._;
        !q && !isInternalObject(_) ? _._ctx = currentRenderingInstance : q === 3 && currentRenderingInstance && (currentRenderingInstance.slots._ === 1 ? _._ = 1 : (_._ = 2, O.patchFlag |= 1024));
      }
    else isFunction$6(_) ? (_ = { default: _, _ctx: currentRenderingInstance }, H = 32) : (_ = String(_), V & 64 ? (H = 16, _ = [createTextVNode(_)]) : H = 8);
    O.children = _, O.shapeFlag |= H;
  }
  function mergeProps$1(...O) {
    const _ = {};
    for (let H = 0; H < O.length; H++) {
      const V = O[H];
      for (const q in V)
        if (q === "class")
          _.class !== V.class && (_.class = normalizeClass([_.class, V.class]));
        else if (q === "style")
          _.style = normalizeStyle([_.style, V.style]);
        else if (isOn(q)) {
          const G = _[q], J = V[q];
          J && G !== J && !(isArray$6(G) && G.includes(J)) && (_[q] = G ? [].concat(G, J) : J);
        } else q !== "" && (_[q] = V[q]);
    }
    return _;
  }
  function invokeVNodeHook(O, _, H, V = null) {
    callWithAsyncErrorHandling(O, _, 7, [
      H,
      V
    ]);
  }
  const emptyAppContext = createAppContext();
  let uid = 0;
  function createComponentInstance(O, _, H) {
    const V = O.type, q = (_ ? _.appContext : O.appContext) || emptyAppContext, G = {
      uid: uid++,
      vnode: O,
      type: V,
      parent: _,
      appContext: q,
      root: null,
      // to be immediately set
      next: null,
      subTree: null,
      // will be set synchronously right after creation
      effect: null,
      update: null,
      // will be set synchronously right after creation
      job: null,
      scope: new EffectScope(
        !0
        /* detached */
      ),
      render: null,
      proxy: null,
      exposed: null,
      exposeProxy: null,
      withProxy: null,
      provides: _ ? _.provides : Object.create(q.provides),
      ids: _ ? _.ids : ["", 0, 0],
      accessCache: null,
      renderCache: [],
      // local resolved assets
      components: null,
      directives: null,
      // resolved props and emits options
      propsOptions: normalizePropsOptions(V, q),
      emitsOptions: normalizeEmitsOptions(V, q),
      // emit
      emit: null,
      // to be set immediately
      emitted: null,
      // props default value
      propsDefaults: EMPTY_OBJ$1,
      // inheritAttrs
      inheritAttrs: V.inheritAttrs,
      // state
      ctx: EMPTY_OBJ$1,
      data: EMPTY_OBJ$1,
      props: EMPTY_OBJ$1,
      attrs: EMPTY_OBJ$1,
      slots: EMPTY_OBJ$1,
      refs: EMPTY_OBJ$1,
      setupState: EMPTY_OBJ$1,
      setupContext: null,
      // suspense related
      suspense: H,
      suspenseId: H ? H.pendingId : 0,
      asyncDep: null,
      asyncResolved: !1,
      // lifecycle hooks
      // not using enums here because it results in computed properties
      isMounted: !1,
      isUnmounted: !1,
      isDeactivated: !1,
      bc: null,
      c: null,
      bm: null,
      m: null,
      bu: null,
      u: null,
      um: null,
      bum: null,
      da: null,
      a: null,
      rtg: null,
      rtc: null,
      ec: null,
      sp: null
    };
    return G.ctx = { _: G }, G.root = _ ? _.root : G, G.emit = emit$2.bind(null, G), O.ce && O.ce(G), G;
  }
  let currentInstance = null;
  const getCurrentInstance = () => currentInstance || currentRenderingInstance;
  let internalSetCurrentInstance, setInSSRSetupState;
  {
    const O = getGlobalThis(), _ = (H, V) => {
      let q;
      return (q = O[H]) || (q = O[H] = []), q.push(V), (G) => {
        q.length > 1 ? q.forEach((J) => J(G)) : q[0](G);
      };
    };
    internalSetCurrentInstance = _(
      "__VUE_INSTANCE_SETTERS__",
      (H) => currentInstance = H
    ), setInSSRSetupState = _(
      "__VUE_SSR_SETTERS__",
      (H) => isInSSRComponentSetup = H
    );
  }
  const setCurrentInstance = (O) => {
    const _ = currentInstance;
    return internalSetCurrentInstance(O), O.scope.on(), () => {
      O.scope.off(), internalSetCurrentInstance(_);
    };
  }, unsetCurrentInstance = () => {
    currentInstance && currentInstance.scope.off(), internalSetCurrentInstance(null);
  };
  function isStatefulComponent(O) {
    return O.vnode.shapeFlag & 4;
  }
  let isInSSRComponentSetup = !1;
  function setupComponent(O, _ = !1, H = !1) {
    _ && setInSSRSetupState(_);
    const { props: V, children: q } = O.vnode, G = isStatefulComponent(O);
    initProps(O, V, G, _), initSlots(O, q, H);
    const J = G ? setupStatefulComponent(O, _) : void 0;
    return _ && setInSSRSetupState(!1), J;
  }
  function setupStatefulComponent(O, _) {
    const H = O.type;
    O.accessCache = /* @__PURE__ */ Object.create(null), O.proxy = new Proxy(O.ctx, PublicInstanceProxyHandlers);
    const { setup: V } = H;
    if (V) {
      pauseTracking();
      const q = O.setupContext = V.length > 1 ? createSetupContext(O) : null, G = setCurrentInstance(O), J = callWithErrorHandling(
        V,
        O,
        0,
        [
          O.props,
          q
        ]
      ), Z = isPromise$1(J);
      if (resetTracking(), G(), (Z || O.sp) && !isAsyncWrapper(O) && markAsyncBoundary(O), Z) {
        if (J.then(unsetCurrentInstance, unsetCurrentInstance), _)
          return J.then((te) => {
            handleSetupResult(O, te);
          }).catch((te) => {
            handleError(te, O, 0);
          });
        O.asyncDep = J;
      } else
        handleSetupResult(O, J);
    } else
      finishComponentSetup(O);
  }
  function handleSetupResult(O, _, H) {
    isFunction$6(_) ? O.type.__ssrInlineRender ? O.ssrRender = _ : O.render = _ : isObject$7(_) && (O.setupState = proxyRefs(_)), finishComponentSetup(O);
  }
  function finishComponentSetup(O, _, H) {
    const V = O.type;
    O.render || (O.render = V.render || NOOP);
    {
      const q = setCurrentInstance(O);
      pauseTracking();
      try {
        applyOptions(O);
      } finally {
        resetTracking(), q();
      }
    }
  }
  const attrsProxyHandlers = {
    get(O, _) {
      return track(O, "get", ""), O[_];
    }
  };
  function createSetupContext(O) {
    const _ = (H) => {
      O.exposed = H || {};
    };
    return {
      attrs: new Proxy(O.attrs, attrsProxyHandlers),
      slots: O.slots,
      emit: O.emit,
      expose: _
    };
  }
  function getComponentPublicInstance(O) {
    return O.exposed ? O.exposeProxy || (O.exposeProxy = new Proxy(proxyRefs(markRaw(O.exposed)), {
      get(_, H) {
        if (H in _)
          return _[H];
        if (H in publicPropertiesMap)
          return publicPropertiesMap[H](O);
      },
      has(_, H) {
        return H in _ || H in publicPropertiesMap;
      }
    })) : O.proxy;
  }
  const classifyRE = /(?:^|[-_])(\w)/g, classify = (O) => O.replace(classifyRE, (_) => _.toUpperCase()).replace(/[-_]/g, "");
  function getComponentName(O, _ = !0) {
    return isFunction$6(O) ? O.displayName || O.name : O.name || _ && O.__name;
  }
  function formatComponentName(O, _, H = !1) {
    let V = getComponentName(_);
    if (!V && _.__file) {
      const q = _.__file.match(/([^/\\]+)\.\w+$/);
      q && (V = q[1]);
    }
    if (!V && O && O.parent) {
      const q = (G) => {
        for (const J in G)
          if (G[J] === _)
            return J;
      };
      V = q(
        O.components || O.parent.type.components
      ) || q(O.appContext.components);
    }
    return V ? classify(V) : H ? "App" : "Anonymous";
  }
  function isClassComponent(O) {
    return isFunction$6(O) && "__vccOpts" in O;
  }
  const computed = (O, _) => computed$1(O, _, isInSSRComponentSetup);
  function h$3(O, _, H) {
    const V = arguments.length;
    return V === 2 ? isObject$7(_) && !isArray$6(_) ? isVNode(_) ? createVNode(O, null, [_]) : createVNode(O, _) : createVNode(O, null, _) : (V > 3 ? H = Array.prototype.slice.call(arguments, 2) : V === 3 && isVNode(H) && (H = [H]), createVNode(O, _, H));
  }
  const version$8 = "3.5.13";
  /**
  * @vue/runtime-dom v3.5.13
  * (c) 2018-present Yuxi (Evan) You and Vue contributors
  * @license MIT
  **/
  let policy;
  const tt = typeof window < "u" && window.trustedTypes;
  if (tt)
    try {
      policy = /* @__PURE__ */ tt.createPolicy("vue", {
        createHTML: (O) => O
      });
    } catch {
    }
  const unsafeToTrustedHTML = policy ? (O) => policy.createHTML(O) : (O) => O, svgNS = "http://www.w3.org/2000/svg", mathmlNS = "http://www.w3.org/1998/Math/MathML", doc$2 = typeof document < "u" ? document : null, templateContainer = doc$2 && /* @__PURE__ */ doc$2.createElement("template"), nodeOps = {
    insert: (O, _, H) => {
      _.insertBefore(O, H || null);
    },
    remove: (O) => {
      const _ = O.parentNode;
      _ && _.removeChild(O);
    },
    createElement: (O, _, H, V) => {
      const q = _ === "svg" ? doc$2.createElementNS(svgNS, O) : _ === "mathml" ? doc$2.createElementNS(mathmlNS, O) : H ? doc$2.createElement(O, { is: H }) : doc$2.createElement(O);
      return O === "select" && V && V.multiple != null && q.setAttribute("multiple", V.multiple), q;
    },
    createText: (O) => doc$2.createTextNode(O),
    createComment: (O) => doc$2.createComment(O),
    setText: (O, _) => {
      O.nodeValue = _;
    },
    setElementText: (O, _) => {
      O.textContent = _;
    },
    parentNode: (O) => O.parentNode,
    nextSibling: (O) => O.nextSibling,
    querySelector: (O) => doc$2.querySelector(O),
    setScopeId(O, _) {
      O.setAttribute(_, "");
    },
    // __UNSAFE__
    // Reason: innerHTML.
    // Static content here can only come from compiled templates.
    // As long as the user only uses trusted templates, this is safe.
    insertStaticContent(O, _, H, V, q, G) {
      const J = H ? H.previousSibling : _.lastChild;
      if (q && (q === G || q.nextSibling))
        for (; _.insertBefore(q.cloneNode(!0), H), !(q === G || !(q = q.nextSibling)); )
          ;
      else {
        templateContainer.innerHTML = unsafeToTrustedHTML(
          V === "svg" ? `<svg>${O}</svg>` : V === "mathml" ? `<math>${O}</math>` : O
        );
        const Z = templateContainer.content;
        if (V === "svg" || V === "mathml") {
          const te = Z.firstChild;
          for (; te.firstChild; )
            Z.appendChild(te.firstChild);
          Z.removeChild(te);
        }
        _.insertBefore(Z, H);
      }
      return [
        // first
        J ? J.nextSibling : _.firstChild,
        // last
        H ? H.previousSibling : _.lastChild
      ];
    }
  }, TRANSITION = "transition", ANIMATION = "animation", vtcKey = Symbol("_vtc"), DOMTransitionPropsValidators = {
    name: String,
    type: String,
    css: {
      type: Boolean,
      default: !0
    },
    duration: [String, Number, Object],
    enterFromClass: String,
    enterActiveClass: String,
    enterToClass: String,
    appearFromClass: String,
    appearActiveClass: String,
    appearToClass: String,
    leaveFromClass: String,
    leaveActiveClass: String,
    leaveToClass: String
  }, TransitionPropsValidators = /* @__PURE__ */ extend$a(
    {},
    BaseTransitionPropsValidators,
    DOMTransitionPropsValidators
  ), decorate$1 = (O) => (O.displayName = "Transition", O.props = TransitionPropsValidators, O), Transition = /* @__PURE__ */ decorate$1(
    (O, { slots: _ }) => h$3(BaseTransition, resolveTransitionProps(O), _)
  ), callHook = (O, _ = []) => {
    isArray$6(O) ? O.forEach((H) => H(..._)) : O && O(..._);
  }, hasExplicitCallback = (O) => O ? isArray$6(O) ? O.some((_) => _.length > 1) : O.length > 1 : !1;
  function resolveTransitionProps(O) {
    const _ = {};
    for (const we in O)
      we in DOMTransitionPropsValidators || (_[we] = O[we]);
    if (O.css === !1)
      return _;
    const {
      name: H = "v",
      type: V,
      duration: q,
      enterFromClass: G = `${H}-enter-from`,
      enterActiveClass: J = `${H}-enter-active`,
      enterToClass: Z = `${H}-enter-to`,
      appearFromClass: te = G,
      appearActiveClass: ne = J,
      appearToClass: ee = Z,
      leaveFromClass: ae = `${H}-leave-from`,
      leaveActiveClass: oe = `${H}-leave-active`,
      leaveToClass: se = `${H}-leave-to`
    } = O, ce = normalizeDuration(q), le = ce && ce[0], ue = ce && ce[1], {
      onBeforeEnter: de,
      onEnter: fe,
      onEnterCancelled: he,
      onLeave: me,
      onLeaveCancelled: pe,
      onBeforeAppear: ye = de,
      onAppear: ge = fe,
      onAppearCancelled: ve = he
    } = _, be = (we, Ee, xe, Le) => {
      we._enterCancelled = Le, removeTransitionClass(we, Ee ? ee : Z), removeTransitionClass(we, Ee ? ne : J), xe && xe();
    }, Ce = (we, Ee) => {
      we._isLeaving = !1, removeTransitionClass(we, ae), removeTransitionClass(we, se), removeTransitionClass(we, oe), Ee && Ee();
    }, Se = (we) => (Ee, xe) => {
      const Le = we ? ge : fe, Qe = () => be(Ee, we, xe);
      callHook(Le, [Ee, Qe]), nextFrame(() => {
        removeTransitionClass(Ee, we ? te : G), addTransitionClass(Ee, we ? ee : Z), hasExplicitCallback(Le) || whenTransitionEnds(Ee, V, le, Qe);
      });
    };
    return extend$a(_, {
      onBeforeEnter(we) {
        callHook(de, [we]), addTransitionClass(we, G), addTransitionClass(we, J);
      },
      onBeforeAppear(we) {
        callHook(ye, [we]), addTransitionClass(we, te), addTransitionClass(we, ne);
      },
      onEnter: Se(!1),
      onAppear: Se(!0),
      onLeave(we, Ee) {
        we._isLeaving = !0;
        const xe = () => Ce(we, Ee);
        addTransitionClass(we, ae), we._enterCancelled ? (addTransitionClass(we, oe), forceReflow()) : (forceReflow(), addTransitionClass(we, oe)), nextFrame(() => {
          we._isLeaving && (removeTransitionClass(we, ae), addTransitionClass(we, se), hasExplicitCallback(me) || whenTransitionEnds(we, V, ue, xe));
        }), callHook(me, [we, xe]);
      },
      onEnterCancelled(we) {
        be(we, !1, void 0, !0), callHook(he, [we]);
      },
      onAppearCancelled(we) {
        be(we, !0, void 0, !0), callHook(ve, [we]);
      },
      onLeaveCancelled(we) {
        Ce(we), callHook(pe, [we]);
      }
    });
  }
  function normalizeDuration(O) {
    if (O == null)
      return null;
    if (isObject$7(O))
      return [NumberOf(O.enter), NumberOf(O.leave)];
    {
      const _ = NumberOf(O);
      return [_, _];
    }
  }
  function NumberOf(O) {
    return toNumber$2(O);
  }
  function addTransitionClass(O, _) {
    _.split(/\s+/).forEach((H) => H && O.classList.add(H)), (O[vtcKey] || (O[vtcKey] = /* @__PURE__ */ new Set())).add(_);
  }
  function removeTransitionClass(O, _) {
    _.split(/\s+/).forEach((V) => V && O.classList.remove(V));
    const H = O[vtcKey];
    H && (H.delete(_), H.size || (O[vtcKey] = void 0));
  }
  function nextFrame(O) {
    requestAnimationFrame(() => {
      requestAnimationFrame(O);
    });
  }
  let endId = 0;
  function whenTransitionEnds(O, _, H, V) {
    const q = O._endId = ++endId, G = () => {
      q === O._endId && V();
    };
    if (H != null)
      return setTimeout(G, H);
    const { type: J, timeout: Z, propCount: te } = getTransitionInfo(O, _);
    if (!J)
      return V();
    const ne = J + "end";
    let ee = 0;
    const ae = () => {
      O.removeEventListener(ne, oe), G();
    }, oe = (se) => {
      se.target === O && ++ee >= te && ae();
    };
    setTimeout(() => {
      ee < te && ae();
    }, Z + 1), O.addEventListener(ne, oe);
  }
  function getTransitionInfo(O, _) {
    const H = window.getComputedStyle(O), V = (ce) => (H[ce] || "").split(", "), q = V(`${TRANSITION}Delay`), G = V(`${TRANSITION}Duration`), J = getTimeout(q, G), Z = V(`${ANIMATION}Delay`), te = V(`${ANIMATION}Duration`), ne = getTimeout(Z, te);
    let ee = null, ae = 0, oe = 0;
    _ === TRANSITION ? J > 0 && (ee = TRANSITION, ae = J, oe = G.length) : _ === ANIMATION ? ne > 0 && (ee = ANIMATION, ae = ne, oe = te.length) : (ae = Math.max(J, ne), ee = ae > 0 ? J > ne ? TRANSITION : ANIMATION : null, oe = ee ? ee === TRANSITION ? G.length : te.length : 0);
    const se = ee === TRANSITION && /\b(transform|all)(,|$)/.test(
      V(`${TRANSITION}Property`).toString()
    );
    return {
      type: ee,
      timeout: ae,
      propCount: oe,
      hasTransform: se
    };
  }
  function getTimeout(O, _) {
    for (; O.length < _.length; )
      O = O.concat(O);
    return Math.max(..._.map((H, V) => toMs(H) + toMs(O[V])));
  }
  function toMs(O) {
    return O === "auto" ? 0 : Number(O.slice(0, -1).replace(",", ".")) * 1e3;
  }
  function forceReflow() {
    return document.body.offsetHeight;
  }
  function patchClass(O, _, H) {
    const V = O[vtcKey];
    V && (_ = (_ ? [_, ...V] : [...V]).join(" ")), _ == null ? O.removeAttribute("class") : H ? O.setAttribute("class", _) : O.className = _;
  }
  const vShowOriginalDisplay = Symbol("_vod"), vShowHidden = Symbol("_vsh"), vShow = {
    beforeMount(O, { value: _ }, { transition: H }) {
      O[vShowOriginalDisplay] = O.style.display === "none" ? "" : O.style.display, H && _ ? H.beforeEnter(O) : setDisplay(O, _);
    },
    mounted(O, { value: _ }, { transition: H }) {
      H && _ && H.enter(O);
    },
    updated(O, { value: _, oldValue: H }, { transition: V }) {
      !_ != !H && (V ? _ ? (V.beforeEnter(O), setDisplay(O, !0), V.enter(O)) : V.leave(O, () => {
        setDisplay(O, !1);
      }) : setDisplay(O, _));
    },
    beforeUnmount(O, { value: _ }) {
      setDisplay(O, _);
    }
  };
  function setDisplay(O, _) {
    O.style.display = _ ? O[vShowOriginalDisplay] : "none", O[vShowHidden] = !_;
  }
  const CSS_VAR_TEXT = Symbol(""), displayRE = /(^|;)\s*display\s*:/;
  function patchStyle(O, _, H) {
    const V = O.style, q = isString$5(H);
    let G = !1;
    if (H && !q) {
      if (_)
        if (isString$5(_))
          for (const J of _.split(";")) {
            const Z = J.slice(0, J.indexOf(":")).trim();
            H[Z] == null && setStyle$1(V, Z, "");
          }
        else
          for (const J in _)
            H[J] == null && setStyle$1(V, J, "");
      for (const J in H)
        J === "display" && (G = !0), setStyle$1(V, J, H[J]);
    } else if (q) {
      if (_ !== H) {
        const J = V[CSS_VAR_TEXT];
        J && (H += ";" + J), V.cssText = H, G = displayRE.test(H);
      }
    } else _ && O.removeAttribute("style");
    vShowOriginalDisplay in O && (O[vShowOriginalDisplay] = G ? V.display : "", O[vShowHidden] && (V.display = "none"));
  }
  const importantRE = /\s*!important$/;
  function setStyle$1(O, _, H) {
    if (isArray$6(H))
      H.forEach((V) => setStyle$1(O, _, V));
    else if (H == null && (H = ""), _.startsWith("--"))
      O.setProperty(_, H);
    else {
      const V = autoPrefix(O, _);
      importantRE.test(H) ? O.setProperty(
        hyphenate$1(V),
        H.replace(importantRE, ""),
        "important"
      ) : O[V] = H;
    }
  }
  const prefixes$1 = ["Webkit", "Moz", "ms"], prefixCache = {};
  function autoPrefix(O, _) {
    const H = prefixCache[_];
    if (H)
      return H;
    let V = camelize(_);
    if (V !== "filter" && V in O)
      return prefixCache[_] = V;
    V = capitalize$1(V);
    for (let q = 0; q < prefixes$1.length; q++) {
      const G = prefixes$1[q] + V;
      if (G in O)
        return prefixCache[_] = G;
    }
    return _;
  }
  const xlinkNS = "http://www.w3.org/1999/xlink";
  function patchAttr(O, _, H, V, q, G = isSpecialBooleanAttr(_)) {
    V && _.startsWith("xlink:") ? H == null ? O.removeAttributeNS(xlinkNS, _.slice(6, _.length)) : O.setAttributeNS(xlinkNS, _, H) : H == null || G && !includeBooleanAttr(H) ? O.removeAttribute(_) : O.setAttribute(
      _,
      G ? "" : isSymbol$2(H) ? String(H) : H
    );
  }
  function patchDOMProp(O, _, H, V, q) {
    if (_ === "innerHTML" || _ === "textContent") {
      H != null && (O[_] = _ === "innerHTML" ? unsafeToTrustedHTML(H) : H);
      return;
    }
    const G = O.tagName;
    if (_ === "value" && G !== "PROGRESS" && // custom elements may use _value internally
    !G.includes("-")) {
      const Z = G === "OPTION" ? O.getAttribute("value") || "" : O.value, te = H == null ? (
        // #11647: value should be set as empty string for null and undefined,
        // but <input type="checkbox"> should be set as 'on'.
        O.type === "checkbox" ? "on" : ""
      ) : String(H);
      (Z !== te || !("_value" in O)) && (O.value = te), H == null && O.removeAttribute(_), O._value = H;
      return;
    }
    let J = !1;
    if (H === "" || H == null) {
      const Z = typeof O[_];
      Z === "boolean" ? H = includeBooleanAttr(H) : H == null && Z === "string" ? (H = "", J = !0) : Z === "number" && (H = 0, J = !0);
    }
    try {
      O[_] = H;
    } catch {
    }
    J && O.removeAttribute(q || _);
  }
  function addEventListener$2(O, _, H, V) {
    O.addEventListener(_, H, V);
  }
  function removeEventListener$1(O, _, H, V) {
    O.removeEventListener(_, H, V);
  }
  const veiKey = Symbol("_vei");
  function patchEvent(O, _, H, V, q = null) {
    const G = O[veiKey] || (O[veiKey] = {}), J = G[_];
    if (V && J)
      J.value = V;
    else {
      const [Z, te] = parseName(_);
      if (V) {
        const ne = G[_] = createInvoker(
          V,
          q
        );
        addEventListener$2(O, Z, ne, te);
      } else J && (removeEventListener$1(O, Z, J, te), G[_] = void 0);
    }
  }
  const optionsModifierRE = /(?:Once|Passive|Capture)$/;
  function parseName(O) {
    let _;
    if (optionsModifierRE.test(O)) {
      _ = {};
      let V;
      for (; V = O.match(optionsModifierRE); )
        O = O.slice(0, O.length - V[0].length), _[V[0].toLowerCase()] = !0;
    }
    return [O[2] === ":" ? O.slice(3) : hyphenate$1(O.slice(2)), _];
  }
  let cachedNow = 0;
  const p$3 = /* @__PURE__ */ Promise.resolve(), getNow = () => cachedNow || (p$3.then(() => cachedNow = 0), cachedNow = Date.now());
  function createInvoker(O, _) {
    const H = (V) => {
      if (!V._vts)
        V._vts = Date.now();
      else if (V._vts <= H.attached)
        return;
      callWithAsyncErrorHandling(
        patchStopImmediatePropagation(V, H.value),
        _,
        5,
        [V]
      );
    };
    return H.value = O, H.attached = getNow(), H;
  }
  function patchStopImmediatePropagation(O, _) {
    if (isArray$6(_)) {
      const H = O.stopImmediatePropagation;
      return O.stopImmediatePropagation = () => {
        H.call(O), O._stopped = !0;
      }, _.map(
        (V) => (q) => !q._stopped && V && V(q)
      );
    } else
      return _;
  }
  const isNativeOn = (O) => O.charCodeAt(0) === 111 && O.charCodeAt(1) === 110 && // lowercase letter
  O.charCodeAt(2) > 96 && O.charCodeAt(2) < 123, patchProp = (O, _, H, V, q, G) => {
    const J = q === "svg";
    _ === "class" ? patchClass(O, V, J) : _ === "style" ? patchStyle(O, H, V) : isOn(_) ? isModelListener(_) || patchEvent(O, _, H, V, G) : (_[0] === "." ? (_ = _.slice(1), !0) : _[0] === "^" ? (_ = _.slice(1), !1) : shouldSetAsProp(O, _, V, J)) ? (patchDOMProp(O, _, V), !O.tagName.includes("-") && (_ === "value" || _ === "checked" || _ === "selected") && patchAttr(O, _, V, J, G, _ !== "value")) : /* #11081 force set props for possible async custom element */ O._isVueCE && (/[A-Z]/.test(_) || !isString$5(V)) ? patchDOMProp(O, camelize(_), V, G, _) : (_ === "true-value" ? O._trueValue = V : _ === "false-value" && (O._falseValue = V), patchAttr(O, _, V, J));
  };
  function shouldSetAsProp(O, _, H, V) {
    if (V)
      return !!(_ === "innerHTML" || _ === "textContent" || _ in O && isNativeOn(_) && isFunction$6(H));
    if (_ === "spellcheck" || _ === "draggable" || _ === "translate" || _ === "form" || _ === "list" && O.tagName === "INPUT" || _ === "type" && O.tagName === "TEXTAREA")
      return !1;
    if (_ === "width" || _ === "height") {
      const q = O.tagName;
      if (q === "IMG" || q === "VIDEO" || q === "CANVAS" || q === "SOURCE")
        return !1;
    }
    return isNativeOn(_) && isString$5(H) ? !1 : _ in O;
  }
  const REMOVAL = {};
  /*! #__NO_SIDE_EFFECTS__ */
  // @__NO_SIDE_EFFECTS__
  function defineCustomElement(O, _, H) {
    const V = /* @__PURE__ */ defineComponent(O, _);
    isPlainObject$3(V) && extend$a(V, _);
    class q extends VueElement {
      constructor(J) {
        super(V, J, H);
      }
    }
    return q.def = V, q;
  }
  const BaseClass = typeof HTMLElement < "u" ? HTMLElement : class {
  };
  class VueElement extends BaseClass {
    constructor(_, H = {}, V = createApp) {
      super(), this._def = _, this._props = H, this._createApp = V, this._isVueCE = !0, this._instance = null, this._app = null, this._nonce = this._def.nonce, this._connected = !1, this._resolved = !1, this._numberProps = null, this._styleChildren = /* @__PURE__ */ new WeakSet(), this._ob = null, this.shadowRoot && V !== createApp ? this._root = this.shadowRoot : _.shadowRoot !== !1 ? (this.attachShadow({ mode: "open" }), this._root = this.shadowRoot) : this._root = this, this._def.__asyncLoader || this._resolveProps(this._def);
    }
    connectedCallback() {
      if (!this.isConnected) return;
      this.shadowRoot || this._parseSlots(), this._connected = !0;
      let _ = this;
      for (; _ = _ && (_.parentNode || _.host); )
        if (_ instanceof VueElement) {
          this._parent = _;
          break;
        }
      this._instance || (this._resolved ? (this._setParent(), this._update()) : _ && _._pendingResolve ? this._pendingResolve = _._pendingResolve.then(() => {
        this._pendingResolve = void 0, this._resolveDef();
      }) : this._resolveDef());
    }
    _setParent(_ = this._parent) {
      _ && (this._instance.parent = _._instance, this._instance.provides = _._instance.provides);
    }
    disconnectedCallback() {
      this._connected = !1, nextTick$1(() => {
        this._connected || (this._ob && (this._ob.disconnect(), this._ob = null), this._app && this._app.unmount(), this._instance && (this._instance.ce = void 0), this._app = this._instance = null);
      });
    }
    /**
     * resolve inner component definition (handle possible async component)
     */
    _resolveDef() {
      if (this._pendingResolve)
        return;
      for (let V = 0; V < this.attributes.length; V++)
        this._setAttr(this.attributes[V].name);
      this._ob = new MutationObserver((V) => {
        for (const q of V)
          this._setAttr(q.attributeName);
      }), this._ob.observe(this, { attributes: !0 });
      const _ = (V, q = !1) => {
        this._resolved = !0, this._pendingResolve = void 0;
        const { props: G, styles: J } = V;
        let Z;
        if (G && !isArray$6(G))
          for (const te in G) {
            const ne = G[te];
            (ne === Number || ne && ne.type === Number) && (te in this._props && (this._props[te] = toNumber$2(this._props[te])), (Z || (Z = /* @__PURE__ */ Object.create(null)))[camelize(te)] = !0);
          }
        this._numberProps = Z, q && this._resolveProps(V), this.shadowRoot && this._applyStyles(J), this._mount(V);
      }, H = this._def.__asyncLoader;
      H ? this._pendingResolve = H().then(
        (V) => _(this._def = V, !0)
      ) : _(this._def);
    }
    _mount(_) {
      this._app = this._createApp(_), _.configureApp && _.configureApp(this._app), this._app._ceVNode = this._createVNode(), this._app.mount(this._root);
      const H = this._instance && this._instance.exposed;
      if (H)
        for (const V in H)
          hasOwn(this, V) || Object.defineProperty(this, V, {
            // unwrap ref to be consistent with public instance behavior
            get: () => unref(H[V])
          });
    }
    _resolveProps(_) {
      const { props: H } = _, V = isArray$6(H) ? H : Object.keys(H || {});
      for (const q of Object.keys(this))
        q[0] !== "_" && V.includes(q) && this._setProp(q, this[q]);
      for (const q of V.map(camelize))
        Object.defineProperty(this, q, {
          get() {
            return this._getProp(q);
          },
          set(G) {
            this._setProp(q, G, !0, !0);
          }
        });
    }
    _setAttr(_) {
      if (_.startsWith("data-v-")) return;
      const H = this.hasAttribute(_);
      let V = H ? this.getAttribute(_) : REMOVAL;
      const q = camelize(_);
      H && this._numberProps && this._numberProps[q] && (V = toNumber$2(V)), this._setProp(q, V, !1, !0);
    }
    /**
     * @internal
     */
    _getProp(_) {
      return this._props[_];
    }
    /**
     * @internal
     */
    _setProp(_, H, V = !0, q = !1) {
      if (H !== this._props[_] && (H === REMOVAL ? delete this._props[_] : (this._props[_] = H, _ === "key" && this._app && (this._app._ceVNode.key = H)), q && this._instance && this._update(), V)) {
        const G = this._ob;
        G && G.disconnect(), H === !0 ? this.setAttribute(hyphenate$1(_), "") : typeof H == "string" || typeof H == "number" ? this.setAttribute(hyphenate$1(_), H + "") : H || this.removeAttribute(hyphenate$1(_)), G && G.observe(this, { attributes: !0 });
      }
    }
    _update() {
      render$5(this._createVNode(), this._root);
    }
    _createVNode() {
      const _ = {};
      this.shadowRoot || (_.onVnodeMounted = _.onVnodeUpdated = this._renderSlots.bind(this));
      const H = createVNode(this._def, extend$a(_, this._props));
      return this._instance || (H.ce = (V) => {
        this._instance = V, V.ce = this, V.isCE = !0;
        const q = (G, J) => {
          this.dispatchEvent(
            new CustomEvent(
              G,
              isPlainObject$3(J[0]) ? extend$a({ detail: J }, J[0]) : { detail: J }
            )
          );
        };
        V.emit = (G, ...J) => {
          q(G, J), hyphenate$1(G) !== G && q(hyphenate$1(G), J);
        }, this._setParent();
      }), H;
    }
    _applyStyles(_, H) {
      if (!_) return;
      if (H) {
        if (H === this._def || this._styleChildren.has(H))
          return;
        this._styleChildren.add(H);
      }
      const V = this._nonce;
      for (let q = _.length - 1; q >= 0; q--) {
        const G = document.createElement("style");
        V && G.setAttribute("nonce", V), G.textContent = _[q], this.shadowRoot.prepend(G);
      }
    }
    /**
     * Only called when shadowRoot is false
     */
    _parseSlots() {
      const _ = this._slots = {};
      let H;
      for (; H = this.firstChild; ) {
        const V = H.nodeType === 1 && H.getAttribute("slot") || "default";
        (_[V] || (_[V] = [])).push(H), this.removeChild(H);
      }
    }
    /**
     * Only called when shadowRoot is false
     */
    _renderSlots() {
      const _ = (this._teleportTarget || this).querySelectorAll("slot"), H = this._instance.type.__scopeId;
      for (let V = 0; V < _.length; V++) {
        const q = _[V], G = q.getAttribute("name") || "default", J = this._slots[G], Z = q.parentNode;
        if (J)
          for (const te of J) {
            if (H && te.nodeType === 1) {
              const ne = H + "-s", ee = document.createTreeWalker(te, 1);
              te.setAttribute(ne, "");
              let ae;
              for (; ae = ee.nextNode(); )
                ae.setAttribute(ne, "");
            }
            Z.insertBefore(te, q);
          }
        else
          for (; q.firstChild; ) Z.insertBefore(q.firstChild, q);
        Z.removeChild(q);
      }
    }
    /**
     * @internal
     */
    _injectChildStyle(_) {
      this._applyStyles(_.styles, _);
    }
    /**
     * @internal
     */
    _removeChildStyle(_) {
    }
  }
  const systemModifiers = ["ctrl", "shift", "alt", "meta"], modifierGuards = {
    stop: (O) => O.stopPropagation(),
    prevent: (O) => O.preventDefault(),
    self: (O) => O.target !== O.currentTarget,
    ctrl: (O) => !O.ctrlKey,
    shift: (O) => !O.shiftKey,
    alt: (O) => !O.altKey,
    meta: (O) => !O.metaKey,
    left: (O) => "button" in O && O.button !== 0,
    middle: (O) => "button" in O && O.button !== 1,
    right: (O) => "button" in O && O.button !== 2,
    exact: (O, _) => systemModifiers.some((H) => O[`${H}Key`] && !_.includes(H))
  }, withModifiers = (O, _) => {
    const H = O._withMods || (O._withMods = {}), V = _.join(".");
    return H[V] || (H[V] = (q, ...G) => {
      for (let J = 0; J < _.length; J++) {
        const Z = modifierGuards[_[J]];
        if (Z && Z(q, _)) return;
      }
      return O(q, ...G);
    });
  }, keyNames = {
    esc: "escape",
    space: " ",
    up: "arrow-up",
    left: "arrow-left",
    right: "arrow-right",
    down: "arrow-down",
    delete: "backspace"
  }, withKeys = (O, _) => {
    const H = O._withKeys || (O._withKeys = {}), V = _.join(".");
    return H[V] || (H[V] = (q) => {
      if (!("key" in q))
        return;
      const G = hyphenate$1(q.key);
      if (_.some(
        (J) => J === G || keyNames[J] === G
      ))
        return O(q);
    });
  }, rendererOptions = /* @__PURE__ */ extend$a({ patchProp }, nodeOps);
  let renderer;
  function ensureRenderer() {
    return renderer || (renderer = createRenderer(rendererOptions));
  }
  const render$5 = (...O) => {
    ensureRenderer().render(...O);
  }, createApp = (...O) => {
    const _ = ensureRenderer().createApp(...O), { mount: H } = _;
    return _.mount = (V) => {
      const q = normalizeContainer(V);
      if (!q) return;
      const G = _._component;
      !isFunction$6(G) && !G.render && !G.template && (G.template = q.innerHTML), q.nodeType === 1 && (q.textContent = "");
      const J = H(q, !1, resolveRootNamespace(q));
      return q instanceof Element && (q.removeAttribute("v-cloak"), q.setAttribute("data-v-app", "")), J;
    }, _;
  };
  function resolveRootNamespace(O) {
    if (O instanceof SVGElement)
      return "svg";
    if (typeof MathMLElement == "function" && O instanceof MathMLElement)
      return "mathml";
  }
  function normalizeContainer(O) {
    return isString$5(O) ? document.querySelector(O) : O;
  }
  var explicitKeys = [
    "__key",
    "__init",
    "__shim",
    "__original",
    "__index",
    "__prevKey"
  ];
  function token$2() {
    return Math.random().toString(36).substring(2, 15);
  }
  function dedupe(O, _) {
    return [...O instanceof Set ? O : new Set(O)];
  }
  function has$2(O, _) {
    return Object.prototype.hasOwnProperty.call(O, _);
  }
  function eq$2(O, _, H = !0, V = ["__key"]) {
    if (O === _)
      return !0;
    if (typeof _ == "object" && typeof O == "object") {
      if (O instanceof Map || O instanceof Set)
        return !1;
      if (O instanceof Date && _ instanceof Date)
        return O.getTime() === _.getTime();
      if (O instanceof RegExp && _ instanceof RegExp)
        return eqRegExp(O, _);
      if (O === null || _ === null || Object.keys(O).length !== Object.keys(_).length)
        return !1;
      for (const q of V)
        if ((q in O || q in _) && O[q] !== _[q])
          return !1;
      for (const q in O)
        if (!(q in _) || O[q] !== _[q] && !H || H && !eq$2(O[q], _[q], H, V))
          return !1;
      return !0;
    }
    return !1;
  }
  function eqRegExp(O, _) {
    return O.source === _.source && O.flags.split("").sort().join("") === _.flags.split("").sort().join("");
  }
  function empty$3(O) {
    const _ = typeof O;
    if (_ === "number")
      return !1;
    if (O === void 0)
      return !0;
    if (_ === "string")
      return O === "";
    if (_ === "object") {
      if (O === null)
        return !0;
      for (const H in O)
        return !1;
      return !(O instanceof RegExp || O instanceof Date);
    }
    return !1;
  }
  function escapeExp(O) {
    return O.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
  }
  function regexForFormat(O) {
    const _ = `^${escapeExp(O)}$`, H = {
      MM: "(0[1-9]|1[012])",
      M: "([1-9]|1[012])",
      DD: "([012][0-9]|3[01])",
      D: "([012]?[0-9]|3[01])",
      YYYY: "\\d{4}",
      YY: "\\d{2}"
    }, V = Object.keys(H);
    return new RegExp(
      V.reduce((q, G) => q.replace(G, H[G]), _)
    );
  }
  function isRecord(O) {
    return Object.prototype.toString.call(O) === "[object Object]";
  }
  function isObject$6(O) {
    return isRecord(O) || Array.isArray(O);
  }
  function isPojo(O) {
    if (isRecord(O) === !1 || O.__FKNode__ || O.__POJO__ === !1)
      return !1;
    const _ = O.constructor;
    if (_ === void 0)
      return !0;
    const H = _.prototype;
    return !(isRecord(H) === !1 || H.hasOwnProperty("isPrototypeOf") === !1);
  }
  var extend$9 = /* @__NO_SIDE_EFFECTS__ */ (O, _, H = !1, V = !1) => {
    if (_ === null)
      return null;
    const q = {};
    if (typeof _ == "string")
      return _;
    for (const G in O)
      if (has$2(_, G) && (_[G] !== void 0 || !V)) {
        if (H && Array.isArray(O[G]) && Array.isArray(_[G])) {
          q[G] = O[G].concat(_[G]);
          continue;
        }
        if (_[G] === void 0)
          continue;
        isPojo(O[G]) && isPojo(_[G]) ? q[G] = /* @__PURE__ */ extend$9(
          O[G],
          _[G],
          H,
          V
        ) : q[G] = _[G];
      } else
        q[G] = O[G];
    for (const G in _)
      !has$2(q, G) && _[G] !== void 0 && (q[G] = _[G]);
    return q;
  };
  function isQuotedString(O) {
    if (O[0] !== '"' && O[0] !== "'" || O[0] !== O[O.length - 1])
      return !1;
    const _ = O[0];
    for (let H = 1; H < O.length; H++)
      if (O[H] === _ && (H === 1 || O[H - 1] !== "\\") && H !== O.length - 1)
        return !1;
    return !0;
  }
  function rmEscapes(O) {
    if (!O.length)
      return "";
    let _ = "", H = "";
    for (let V = 0; V < O.length; V++) {
      const q = O.charAt(V);
      (q !== "\\" || H === "\\") && (_ += q), H = q;
    }
    return _;
  }
  function nodeProps(...O) {
    return O.reduce((_, H) => {
      const { value: V, name: q, modelValue: G, config: J, plugins: Z, ...te } = H;
      return Object.assign(_, te);
    }, {});
  }
  function parseArgs(O) {
    const _ = [];
    let H = "", V = 0, q = "", G = "";
    for (let J = 0; J < O.length; J++) {
      const Z = O.charAt(J);
      Z === q && G !== "\\" ? q = "" : (Z === "'" || Z === '"') && !q && G !== "\\" ? q = Z : Z === "(" && !q ? V++ : Z === ")" && !q && V--, Z === "," && !q && V === 0 ? (_.push(H), H = "") : (Z !== " " || q) && (H += Z), G = Z;
    }
    return H && _.push(H), _;
  }
  function except(O, _) {
    const H = {}, V = _.filter((G) => G instanceof RegExp), q = new Set(_);
    for (const G in O)
      !q.has(G) && !V.some((J) => J.test(G)) && (H[G] = O[G]);
    return H;
  }
  function only(O, _) {
    const H = {}, V = _.filter((q) => q instanceof RegExp);
    return _.forEach((q) => {
      q instanceof RegExp || (H[q] = O[q]);
    }), Object.keys(O).forEach((q) => {
      V.some((G) => G.test(q)) && (H[q] = O[q]);
    }), H;
  }
  function camel(O) {
    return O.replace(
      /-([a-z0-9])/gi,
      (_, H) => H.toUpperCase()
    );
  }
  function kebab(O) {
    return O.replace(
      /([a-z0-9])([A-Z])/g,
      (_, H, V) => H + "-" + V.toLowerCase()
    ).replace(" ", "-").toLowerCase();
  }
  function shallowClone(O, _ = explicitKeys) {
    if (O !== null && typeof O == "object") {
      let H;
      if (Array.isArray(O) ? H = [...O] : isPojo(O) && (H = { ...O }), H)
        return applyExplicit(O, H, _), H;
    }
    return O;
  }
  function clone$3(O, _ = explicitKeys) {
    if (O === null || O instanceof RegExp || O instanceof Date || O instanceof Map || O instanceof Set || typeof File == "function" && O instanceof File)
      return O;
    let H;
    Array.isArray(O) ? H = O.map((V) => typeof V == "object" ? clone$3(V, _) : V) : H = Object.keys(O).reduce((V, q) => (V[q] = typeof O[q] == "object" ? clone$3(O[q], _) : O[q], V), {});
    for (const V of _)
      V in O && Object.defineProperty(H, V, {
        enumerable: !1,
        value: O[V]
      });
    return H;
  }
  function cloneAny(O) {
    return typeof O == "object" ? clone$3(O) : O;
  }
  function getAt(O, _) {
    if (!O || typeof O != "object")
      return null;
    const H = _.split(".");
    let V = O;
    for (const q in H) {
      const G = H[q];
      if (has$2(V, G) && (V = V[G]), +q === H.length - 1)
        return V;
      if (!V || typeof V != "object")
        return null;
    }
    return null;
  }
  function undefine(O) {
    return O !== void 0 && O !== "false" && O !== !1 ? !0 : void 0;
  }
  function init$3(O) {
    return Object.isFrozen(O) ? O : Object.defineProperty(O, "__init", {
      enumerable: !1,
      value: !0
    });
  }
  function slugify(O) {
    return O.normalize("NFD").replace(/[\u0300-\u036f]/g, "").toLowerCase().replace(/[^a-z0-9]/g, " ").trim().replace(/\s+/g, "-");
  }
  function applyExplicit(O, _, H) {
    for (const V of H)
      V in O && Object.defineProperty(_, V, {
        enumerable: !1,
        value: O[V]
      });
    return _;
  }
  function oncePerTick(O) {
    let _ = !1;
    return (...H) => {
      if (!_)
        return _ = !0, queueMicrotask(() => _ = !1), O(...H);
    };
  }
  function boolGetter(O) {
    if (!(O === "false" || O === !1))
      return !0;
  }
  function createDispatcher() {
    const O = [];
    let _ = 0;
    const H = (q) => O.push(q), V = (q) => {
      const G = O[_];
      return typeof G == "function" ? G(q, (J) => (_++, V(J))) : (_ = 0, q);
    };
    return H.dispatch = V, H.unshift = (q) => O.unshift(q), H.remove = (q) => {
      const G = O.indexOf(q);
      G > -1 && O.splice(G, 1);
    }, H;
  }
  function createEmitter() {
    const O = /* @__PURE__ */ new Map(), _ = /* @__PURE__ */ new Map();
    let H;
    const V = (q, G) => {
      if (H) {
        H.set(G.name, [q, G]);
        return;
      }
      O.has(G.name) && O.get(G.name).forEach((J) => {
        (G.origin === q || J.modifiers.includes("deep")) && J.listener(G);
      }), G.bubble && q.bubble(G);
    };
    return V.flush = () => {
      O.clear(), _.clear(), H == null || H.clear();
    }, V.on = (q, G, J = "push") => {
      const [Z, ...te] = q.split("."), ne = G.receipt || token$2(), ee = {
        modifiers: te,
        event: Z,
        listener: G,
        receipt: ne
      };
      return O.has(Z) ? O.get(Z)[J](ee) : O.set(Z, [ee]), _.has(ne) ? _.get(ne)[J](Z) : _.set(ne, [Z]), ne;
    }, V.off = (q) => {
      var G;
      _.has(q) && ((G = _.get(q)) == null || G.forEach((J) => {
        const Z = O.get(J);
        Array.isArray(Z) && O.set(
          J,
          Z.filter((te) => te.receipt !== q)
        );
      }), _.delete(q));
    }, V.pause = (q) => {
      H || (H = /* @__PURE__ */ new Map()), q && q.walk((G) => G._e.pause());
    }, V.play = (q) => {
      if (!H)
        return;
      const G = H;
      H = void 0, G.forEach(([J, Z]) => V(J, Z)), q && q.walk((J) => J._e.play());
    }, V;
  }
  function emit$1(O, _, H, V, q = !0, G) {
    return _._e(O, {
      payload: V,
      name: H,
      bubble: q,
      origin: O,
      meta: G
    }), O;
  }
  function bubble$1(O, _, H) {
    return isNode(O.parent) && O.parent._e(O.parent, H), O;
  }
  function on$1(O, _, H, V, q) {
    return _._e.on(H, V, q);
  }
  function off(O, _, H) {
    return _._e.off(H), O;
  }
  var errorHandler = createDispatcher();
  errorHandler((O, _) => (O.message || (O.message = `E${O.code}`), _(O)));
  var warningHandler = createDispatcher();
  warningHandler((O, _) => {
    O.message || (O.message = `W${O.code}`);
    const H = _(O);
    return console && typeof console.warn == "function" && console.warn(H.message), H;
  });
  function warn$3(O, _ = {}) {
    warningHandler.dispatch({ code: O, data: _ });
  }
  function error$2(O, _ = {}) {
    throw Error(errorHandler.dispatch({ code: O, data: _ }).message);
  }
  function createMessage(O, _) {
    return {
      blocking: !1,
      key: token$2(),
      meta: {},
      type: "state",
      visible: !0,
      ...O
    };
  }
  var storeTraps = {
    apply: applyMessages,
    set: setMessage,
    remove: removeMessage$1,
    filter: filterMessages,
    reduce: reduceMessages,
    release: releaseBuffer,
    touch: touchMessages
  };
  function createStore$1(O = !1) {
    const _ = {};
    let H, V = O, q = [];
    const G = /* @__PURE__ */ new Map();
    let J;
    const Z = new Proxy(_, {
      get(...te) {
        const [ne, ee] = te;
        return ee === "buffer" ? V : ee === "_b" ? q : ee === "_m" ? G : ee === "_r" ? J : has$2(storeTraps, ee) ? storeTraps[ee].bind(
          null,
          _,
          Z,
          H
        ) : Reflect.get(...te);
      },
      set(te, ne, ee) {
        return ne === "_n" ? (H = ee, J === "__n" && releaseMissed(H, Z), !0) : ne === "_b" ? (q = ee, !0) : ne === "buffer" ? (V = ee, !0) : ne === "_r" ? (J = ee, !0) : (error$2(101, H), !1);
      }
    });
    return Z;
  }
  function setMessage(O, _, H, V) {
    if (_.buffer)
      return _._b.push([[V]]), _;
    if (O[V.key] !== V) {
      if (typeof V.value == "string" && V.meta.localize !== !1) {
        const G = V.value;
        V.value = H.t(V), V.value !== G && (V.meta.locale = H.props.locale);
      }
      const q = `message-${has$2(O, V.key) ? "updated" : "added"}`;
      O[V.key] = Object.freeze(
        H.hook.message.dispatch(V)
      ), H.emit(q, V);
    }
    return _;
  }
  function touchMessages(O, _) {
    for (const H in O) {
      const V = { ...O[H] };
      _.set(V);
    }
  }
  function removeMessage$1(O, _, H, V) {
    if (has$2(O, V)) {
      const q = O[V];
      delete O[V], H.emit("message-removed", q);
    }
    return _.buffer === !0 && (_._b = _._b.filter((q) => (q[0] = q[0].filter((G) => G.key !== V), q[1] || q[0].length))), _;
  }
  function filterMessages(O, _, H, V, q) {
    for (const G in O) {
      const J = O[G];
      (!q || J.type === q) && !V(J) && removeMessage$1(O, _, H, G);
    }
  }
  function reduceMessages(O, _, H, V, q) {
    for (const G in O) {
      const J = O[G];
      q = V(q, J);
    }
    return q;
  }
  function applyMessages(O, _, H, V, q) {
    if (Array.isArray(V)) {
      if (_.buffer) {
        _._b.push([V, q]);
        return;
      }
      const G = new Set(
        V.map((J) => (_.set(J), J.key))
      );
      typeof q == "string" ? _.filter(
        (J) => J.type !== q || G.has(J.key)
      ) : typeof q == "function" && _.filter((J) => !q(J) || G.has(J.key));
    } else
      for (const G in V) {
        const J = H.at(G);
        J ? J.store.apply(V[G], q) : missed(H, _, G, V[G], q);
      }
  }
  function createMessages(O, ..._) {
    const H = `${O.name}-set`, V = (q) => /* @__PURE__ */ createMessage({
      key: slugify(q),
      type: "error",
      value: q,
      meta: { source: H, autoClear: !0 }
    });
    return _.filter((q) => !!q).map((q) => {
      if (typeof q == "string" && (q = [q]), Array.isArray(q))
        return q.map((G) => V(G));
      {
        const G = {};
        for (const J in q)
          Array.isArray(q[J]) ? G[J] = q[J].map(
            (Z) => V(Z)
          ) : G[J] = [V(q[J])];
        return G;
      }
    });
  }
  function missed(O, _, H, V, q) {
    var J;
    const G = _._m;
    G.has(H) || G.set(H, []), _._r || (_._r = releaseMissed(O, _)), (J = G.get(H)) == null || J.push([V, q]);
  }
  function releaseMissed(O, _) {
    return O.on(
      "child.deep",
      ({ payload: H }) => {
        _._m.forEach((V, q) => {
          O.at(q) === H && (V.forEach(([G, J]) => {
            H.store.apply(G, J);
          }), _._m.delete(q));
        }), _._m.size === 0 && _._r && (O.off(_._r), _._r = void 0);
      }
    );
  }
  function releaseBuffer(O, _) {
    _.buffer = !1, _._b.forEach(([H, V]) => _.apply(H, V)), _._b = [];
  }
  function createLedger() {
    const O = {};
    let _;
    return {
      count: (...H) => createCounter(_, O, ...H),
      init(H) {
        _ = H, H.on("message-added.deep", add$8(O, 1)), H.on("message-removed.deep", add$8(O, -1));
      },
      merge: (H) => merge$6(_, O, H),
      settled(H) {
        return has$2(O, H) ? O[H].promise : Promise.resolve();
      },
      unmerge: (H) => merge$6(_, O, H, !0),
      value(H) {
        return has$2(O, H) ? O[H].count : 0;
      }
    };
  }
  function createCounter(O, _, H, V, q = 0) {
    if (V = parseCondition(V || H), !has$2(_, H)) {
      const G = {
        condition: V,
        count: 0,
        name: H,
        node: O,
        promise: Promise.resolve(),
        resolve: () => {
        }
        // eslint-disable-line @typescript-eslint/no-empty-function
      };
      _[H] = G, q = O.store.reduce(
        (J, Z) => J + G.condition(Z) * 1,
        q
      ), O.each((J) => {
        J.ledger.count(G.name, G.condition), q += J.ledger.value(G.name);
      });
    }
    return count$1(_[H], q).promise;
  }
  function parseCondition(O) {
    return typeof O == "function" ? O : (_) => _.type === O;
  }
  function count$1(O, _) {
    const H = O.count, V = O.count + _;
    return O.count = V, H === 0 && V !== 0 ? (O.node.emit(`unsettled:${O.name}`, O.count, !1), O.promise = new Promise((q) => O.resolve = q)) : H !== 0 && V === 0 && (O.node.emit(`settled:${O.name}`, O.count, !1), O.resolve()), O.node.emit(`count:${O.name}`, O.count, !1), O;
  }
  function add$8(O, _) {
    return (H) => {
      for (const V in O) {
        const q = O[V];
        q.condition(H.payload) && count$1(q, _);
      }
    };
  }
  function merge$6(O, _, H, V = !1) {
    const q = O;
    for (const G in _) {
      const J = _[G].condition;
      V || H.ledger.count(G, J);
      const Z = H.ledger.value(G) * (V ? -1 : 1);
      if (O) {
        do
          O.ledger.count(G, J, Z), O = O.parent;
        while (O);
        O = q;
      }
    }
  }
  var registry = /* @__PURE__ */ new Map(), reflected = /* @__PURE__ */ new Map(), emit2 = createEmitter();
  function register$1(O) {
    O.props.id && (registry.set(O.props.id, O), reflected.set(O, O.props.id), emit2(O, {
      payload: O,
      name: O.props.id,
      bubble: !1,
      origin: O
    }));
  }
  function deregister(O) {
    if (reflected.has(O)) {
      const _ = reflected.get(O);
      reflected.delete(O), registry.delete(_), emit2(O, {
        payload: null,
        name: _,
        bubble: !1,
        origin: O
      });
    }
  }
  function getNode(O) {
    return registry.get(O);
  }
  function watchRegistry(O, _) {
    return emit2.on(O, _);
  }
  function configChange(O, _, H) {
    let V = !0;
    return _ in O.config._t ? V = !1 : O.emit(`config:${_}`, H, !1), _ in O.props || (O.emit("prop", { prop: _, value: H }), O.emit(`prop:${_}`, H)), V;
  }
  function createConfig(O = {}) {
    const _ = /* @__PURE__ */ new Set(), H = {
      ...O,
      _add: (q) => _.add(q),
      _rm: (q) => _.delete(q)
    };
    return new Proxy(H, {
      set(q, G, J, Z) {
        return typeof G == "string" && _.forEach((te) => configChange(te, G, J)), Reflect.set(q, G, J, Z);
      }
    });
  }
  function submitForm(O, _) {
    const H = (_ || document).getElementById(O);
    if (H instanceof HTMLFormElement) {
      const V = new Event("submit", { cancelable: !0, bubbles: !0 });
      H.dispatchEvent(V);
      return;
    }
    warn$3(151, O);
  }
  function clearState(O) {
    const _ = (H) => {
      for (const V in H.store) {
        const q = H.store[V];
        q.type === "error" || q.type === "ui" && V === "incomplete" ? H.store.remove(V) : q.type === "state" && H.store.set({ ...q, value: !1 });
      }
    };
    _(O), O.walk(_);
  }
  function reset$2(O, _) {
    const H = typeof O == "string" ? getNode(O) : O;
    if (H) {
      const V = (J) => {
        const Z = cloneAny(J.props.initial);
        return Z !== void 0 ? Z : J.type === "group" ? {} : J.type === "list" ? [] : void 0;
      };
      H._e.pause(H);
      const q = cloneAny(_);
      return _ && !empty$3(_) && (H.props.initial = isObject$6(q) ? init$3(q) : q, H.props._init = H.props.initial), H.input(V(H), !1), H.walk((J) => {
        J.type === "list" && J.sync || J.input(V(J), !1);
      }), H.input(
        empty$3(q) && q ? q : V(H),
        !1
      ), H.type !== "input" && _ && !empty$3(_) && isObject$6(_) && H.walk((J) => {
        J.props.initial = isObject$6(J.value) ? init$3(J.value) : J.value, J.props._init = J.props.initial;
      }), H._e.play(H), clearState(H), H.emit("reset", H), H;
    }
    warn$3(152, O);
  }
  var defaultConfig$4 = {
    delimiter: ".",
    delay: 0,
    locale: "en",
    rootClasses: (O) => ({ [`formkit-${kebab(O)}`]: !0 })
  }, useIndex = Symbol("index"), valueRemoved = Symbol("removed"), valueMoved = Symbol("moved"), valueInserted = Symbol("inserted");
  function isList(O) {
    return O.type === "list" && Array.isArray(O._value);
  }
  function isNode(O) {
    return O && typeof O == "object" && O.__FKNode__ === !0;
  }
  var invalidSetter = (O, _, H) => {
    error$2(102, [O, H]);
  }, traps = {
    _c: trap$1(getContext$1, invalidSetter, !1),
    add: trap$1(addChild),
    addProps: trap$1(addProps),
    address: trap$1(getAddress, invalidSetter, !1),
    at: trap$1(getNode2),
    bubble: trap$1(bubble$1),
    clearErrors: trap$1(clearErrors),
    calm: trap$1(calm),
    config: trap$1(!1),
    define: trap$1(define$1),
    disturb: trap$1(disturb),
    destroy: trap$1(destroy),
    extend: trap$1(extend$8),
    hydrate: trap$1(hydrate),
    index: trap$1(getIndex, setIndex, !1),
    input: trap$1(input$3),
    each: trap$1(eachChild),
    emit: trap$1(emit$1),
    find: trap$1(find$5),
    on: trap$1(on$1),
    off: trap$1(off),
    parent: trap$1(!1, setParent),
    plugins: trap$1(!1),
    remove: trap$1(removeChild),
    root: trap$1(getRoot$2, invalidSetter, !1),
    reset: trap$1(resetValue),
    resetConfig: trap$1(resetConfig),
    setErrors: trap$1(setErrors),
    submit: trap$1(submit),
    t: trap$1(text$6),
    use: trap$1(use),
    name: trap$1(getName$1, !1, !1),
    walk: trap$1(walkTree)
  };
  function createTraps() {
    return new Map(Object.entries(traps));
  }
  function trap$1(O, _, H = !0) {
    return {
      get: O ? (V, q) => H ? (...G) => O(V, q, ...G) : O(V, q) : !1,
      set: _ !== void 0 ? _ : invalidSetter.bind(null)
    };
  }
  function createHooks() {
    const O = /* @__PURE__ */ new Map();
    return new Proxy(O, {
      get(_, H) {
        return O.has(H) || O.set(H, createDispatcher()), O.get(H);
      }
    });
  }
  var nameCount = 0, idCount = 0;
  function createName(O) {
    var _, H;
    return ((_ = O.parent) == null ? void 0 : _.type) === "list" ? useIndex : O.name || `${((H = O.props) == null ? void 0 : H.type) || "input"}_${++nameCount}`;
  }
  function createValue(O) {
    return O.type === "group" ? init$3(
      O.value && typeof O.value == "object" && !Array.isArray(O.value) ? O.value : {}
    ) : O.type === "list" ? init$3(Array.isArray(O.value) ? O.value : []) : O.value;
  }
  function input$3(O, _, H, V = !0) {
    return _._value = validateInput(O, O.hook.input.dispatch(H)), O.emit("input", _._value), O.isCreated && O.type === "input" && eq$2(_._value, _.value) && !O.props.mergeStrategy ? (O.emit("commitRaw", _.value), _.settled) : (_.isSettled && O.disturb(), V ? (_._tmo && clearTimeout(_._tmo), _._tmo = setTimeout(
      commit,
      O.props.delay,
      O,
      _
    )) : commit(O, _), _.settled);
  }
  function validateInput(O, _) {
    switch (O.type) {
      case "input":
        break;
      case "group":
        (!_ || typeof _ != "object") && error$2(107, [O, _]);
        break;
      case "list":
        Array.isArray(_) || error$2(108, [O, _]);
        break;
    }
    return _;
  }
  function commit(O, _, H = !0, V = !0) {
    _._value = _.value = O.hook.commit.dispatch(_._value), O.type !== "input" && V && O.hydrate(), O.emit("commitRaw", _.value), O.emit("commit", _.value), H && O.calm();
  }
  function partial(O, { name: _, value: H, from: V }) {
    if (!Object.isFrozen(O._value)) {
      if (isList(O)) {
        const q = H === valueRemoved ? [] : H === valueMoved && typeof V == "number" ? O._value.splice(V, 1) : [H];
        O._value.splice(
          _,
          H === valueMoved || V === valueInserted ? 0 : 1,
          ...q
        );
        return;
      }
      H !== valueRemoved ? O._value[_] = H : delete O._value[_];
    }
  }
  function hydrate(O, _) {
    const H = _._value;
    return O.type === "list" && O.sync && syncListNodes(O, _), _.children.forEach((V) => {
      if (typeof H == "object")
        if (V.name in H) {
          const q = V.type !== "input" || H[V.name] && typeof H[V.name] == "object" ? init$3(H[V.name]) : H[V.name];
          if (!V.isSettled || (!isObject$6(q) || V.props.mergeStrategy) && eq$2(q, V._value))
            return;
          V.input(q, !1);
        } else
          (O.type !== "list" || typeof V.name == "number") && partial(_, { name: V.name, value: V.value }), H.__init || (V.type === "group" ? V.input({}, !1) : V.type === "list" ? V.input([], !1) : V.input(void 0, !1));
    }), O;
  }
  function syncListNodes(O, _) {
    const H = O._value;
    if (!Array.isArray(H))
      return;
    const V = [], q = new Set(_.children), G = /* @__PURE__ */ new Map();
    H.forEach((Z, te) => {
      if (_.children[te] && _.children[te]._value === Z)
        V.push(_.children[te]), q.delete(_.children[te]);
      else {
        V.push(null);
        const ne = G.get(Z) || [];
        ne.push(te), G.set(Z, ne);
      }
    }), q.size && G.size && q.forEach((Z) => {
      if (G.has(Z._value)) {
        const te = G.get(Z._value), ne = te.shift();
        V[ne] = Z, q.delete(Z), te.length || G.delete(Z._value);
      }
    });
    const J = [];
    for (G.forEach((Z) => {
      J.push(...Z);
    }); q.size && J.length; ) {
      const Z = q.values().next().value, te = J.shift();
      if (te === void 0)
        break;
      V[te] = Z, q.delete(Z);
    }
    J.forEach((Z, te) => {
      V[Z] = createPlaceholder({ value: te });
    }), q.size && q.forEach((Z) => {
      if (!("__FKP" in Z)) {
        const te = Z._c.parent;
        if (!te || isPlaceholder(te))
          return;
        te.ledger.unmerge(Z), Z._c.parent = null, Z.destroy();
      }
    }), _.children = V;
  }
  function disturb(O, _) {
    var H;
    return _._d <= 0 && (_.isSettled = !1, O.emit("settled", !1, !1), _.settled = new Promise((V) => {
      _._resolve = V;
    }), O.parent && ((H = O.parent) == null || H.disturb())), _._d++, O;
  }
  function calm(O, _, H) {
    var V;
    if (H !== void 0 && O.type !== "input") {
      partial(_, H);
      const q = !!(O.config.mergeStrategy && O.config.mergeStrategy[H.name]);
      return commit(O, _, !0, q);
    }
    _._d > 0 && _._d--, _._d === 0 && (_.isSettled = !0, O.emit("settled", !0, !1), O.parent && ((V = O.parent) == null || V.calm({ name: O.name, value: _.value })), _._resolve && _._resolve(_.value));
  }
  function destroy(O, _) {
    O.emit("destroying", O), O.store.filter(() => !1), O.parent && O.parent.remove(O), deregister(O), O.emit("destroyed", O), _._e.flush(), _._value = _.value = void 0;
    for (const H in _.context)
      delete _.context[H];
    _.plugins.clear(), _.context = null;
  }
  function define$1(O, _, H) {
    _.type = H.type;
    const V = clone$3(H);
    O.props.__propDefs = mergeProps(
      O.props.__propDefs ?? [],
      (V == null ? void 0 : V.props) || []
    ), V.props = O.props.__propDefs, _.props.definition = V, _.value = _._value = createValue({
      type: O.type,
      value: _.value
    }), H.forceTypeProp && (O.props.type && (O.props.originalType = O.props.type), _.props.type = H.forceTypeProp), H.family && (_.props.family = H.family), H.features && H.features.forEach((q) => q(O)), H.props && O.addProps(H.props), O.emit("defined", H);
  }
  function addProps(O, _, H) {
    const V = Array.isArray(H) ? H : Object.keys(H), q = Array.isArray(H) ? {} : V.reduce((J, Z) => ("default" in H[Z] && (J[Z] = H[Z].default), J), {});
    if (O.props.attrs) {
      const J = { ...O.props.attrs };
      O.props._emit = !1;
      for (const te in J) {
        const ne = camel(te);
        V.includes(ne) && (O.props[ne] = J[te], delete J[te]);
      }
      Array.isArray(H) || V.forEach((te) => {
        "default" in H[te] && O.props[te] === void 0 && (O.props[te] = q[te]);
      });
      const Z = cloneAny(_._value);
      O.props.initial = O.type !== "input" ? init$3(Z) : Z, O.props._emit = !0, O.props.attrs = J;
    }
    const G = mergeProps(O.props.__propDefs ?? [], H);
    return O.props.definition && (O.props.definition.props = G), O.props.__propDefs = G, O.emit("added-props", H), O;
  }
  function toPropsObj(O) {
    return Array.isArray(O) ? O.reduce((_, H) => (_[H] = {}, _), {}) : O;
  }
  function mergeProps(O, _) {
    return Array.isArray(O) && Array.isArray(_) ? O.concat(_) : extend$9(toPropsObj(O), toPropsObj(_));
  }
  function addChild(O, _, H, V) {
    if (O.type === "input" && error$2(100, O), H.parent && H.parent !== O && H.parent.remove(H), !_.children.includes(H)) {
      if (V !== void 0 && O.type === "list") {
        const q = _.children[V];
        q && "__FKP" in q ? (H._c.uid = q.uid, _.children.splice(V, 1, H)) : _.children.splice(V, 0, H), Array.isArray(O.value) && O.value.length < _.children.length && O.disturb().calm({
          name: V,
          value: H.value,
          from: valueInserted
        });
      } else
        _.children.push(H);
      H.isSettled || O.disturb();
    }
    if (H.parent !== O) {
      if (H.parent = O, H.parent !== O)
        return O.remove(H), H.parent.add(H), O;
    } else
      H.use(O.plugins);
    return commit(O, _, !1), O.ledger.merge(H), O.emit("child", H), O;
  }
  function setParent(O, _, H, V) {
    return isNode(V) ? (O.parent && O.parent !== V && O.parent.remove(O), _.parent = V, O.resetConfig(), V.children.includes(O) ? O.use(V.plugins) : V.add(O), !0) : V === null ? (_.parent = null, !0) : !1;
  }
  function removeChild(O, _, H) {
    const V = _.children.indexOf(H);
    if (V !== -1) {
      H.isSettled && O.disturb(), _.children.splice(V, 1);
      let q = undefine(H.props.preserve), G = H.parent;
      for (; q === void 0 && G; )
        q = undefine(G.props.preserve), G = G.parent;
      q ? O.calm() : O.calm({
        name: O.type === "list" ? V : H.name,
        value: valueRemoved
      }), H.parent = null, H.config._rmn = H;
    }
    return O.ledger.unmerge(H), O.emit("childRemoved", H), O;
  }
  function eachChild(O, _, H) {
    _.children.forEach((V) => !("__FKP" in V) && H(V));
  }
  function walkTree(O, _, H, V = !1, q = !1) {
    _.children.some((G) => {
      if ("__FKP" in G)
        return !1;
      const J = H(G);
      return V && J === !1 ? !0 : q && J === !1 ? !1 : G.walk(H, V, q);
    });
  }
  function resetConfig(O, _) {
    const H = O.parent || void 0;
    _.config = createConfig2(O.config._t, H), O.walk((V) => V.resetConfig());
  }
  function use(O, _, H, V = !0, q = !0) {
    return Array.isArray(H) || H instanceof Set ? (H.forEach((G) => use(O, _, G)), O) : (_.plugins.has(H) || (q && typeof H.library == "function" && H.library(O), V && H(O) !== !1 && (_.plugins.add(H), O.children.forEach((G) => G.use(H)))), O);
  }
  function setIndex(O, _, H, V) {
    if (isNode(O.parent)) {
      const q = O.parent.children, G = V >= q.length ? q.length - 1 : V < 0 ? 0 : V, J = q.indexOf(O);
      return J === -1 ? !1 : (q.splice(J, 1), q.splice(G, 0, O), O.parent.children = q, O.parent.type === "list" && O.parent.disturb().calm({ name: G, value: valueMoved, from: J }), !0);
    }
    return !1;
  }
  function getIndex(O) {
    if (O.parent) {
      const _ = [...O.parent.children].indexOf(O);
      return _ === -1 ? O.parent.children.length : _;
    }
    return -1;
  }
  function getContext$1(O, _) {
    return _;
  }
  function getName$1(O, _) {
    var H;
    return ((H = O.parent) == null ? void 0 : H.type) === "list" ? O.index : _.name !== useIndex ? _.name : O.index;
  }
  function getAddress(O, _) {
    return _.parent ? _.parent.address.concat([O.name]) : [O.name];
  }
  function getNode2(O, _, H) {
    const V = typeof H == "string" ? H.split(O.config.delimiter) : H;
    if (!V.length)
      return;
    const q = V[0];
    let G = O.parent;
    for (G || (String(V[0]) === String(O.name) && V.shift(), G = O), q === "$parent" && V.shift(); G && V.length; ) {
      const J = V.shift();
      switch (J) {
        case "$root":
          G = O.root;
          break;
        case "$parent":
          G = G.parent;
          break;
        case "$self":
          G = O;
          break;
        default:
          G = G.children.find(
            (Z) => !("__FKP" in Z) && String(Z.name) === String(J)
          ) || select$3(G, J);
      }
    }
    return G || void 0;
  }
  function select$3(O, _) {
    const H = String(_).match(/^(find)\((.*)\)$/);
    if (H) {
      const [, V, q] = H, G = q.split(",").map((J) => J.trim());
      switch (V) {
        case "find":
          return O.find(G[0], G[1]);
        default:
          return;
      }
    }
  }
  function find$5(O, _, H, V) {
    return bfs(O, H, V);
  }
  function bfs(O, _, H = "name") {
    const V = typeof H == "string" ? (G) => G[H] == _ : H, q = [O];
    for (; q.length; ) {
      const G = q.shift();
      if (!("__FKP" in G)) {
        if (V(G, _))
          return G;
        q.push(...G.children);
      }
    }
  }
  function getRoot$2(O) {
    let _ = O;
    for (; _.parent; )
      _ = _.parent;
    return _;
  }
  function createConfig2(O = {}, _) {
    let H;
    return new Proxy(O, {
      get(...V) {
        const q = V[1];
        if (q === "_t")
          return O;
        const G = Reflect.get(...V);
        if (G !== void 0)
          return G;
        if (_) {
          const J = _.config[q];
          if (J !== void 0)
            return J;
        }
        if (O.rootConfig && typeof q == "string") {
          const J = O.rootConfig[q];
          if (J !== void 0)
            return J;
        }
        return q === "delay" && (H == null ? void 0 : H.type) === "input" ? 20 : defaultConfig$4[q];
      },
      set(...V) {
        const q = V[1], G = V[2];
        if (q === "_n")
          return H = G, O.rootConfig && O.rootConfig._add(H), !0;
        if (q === "_rmn")
          return O.rootConfig && O.rootConfig._rm(H), H = void 0, !0;
        if (!eq$2(O[q], G, !1)) {
          const J = Reflect.set(...V);
          return H && (H.emit(`config:${q}`, G, !1), configChange(H, q, G), H.walk((Z) => configChange(Z, q, G), !1, !0)), J;
        }
        return !0;
      }
    });
  }
  function text$6(O, _, H, V = "ui") {
    const q = typeof H == "string" ? { key: H, value: H, type: V } : H, G = O.hook.text.dispatch(q);
    return O.emit("text", G, !1), G.value;
  }
  function submit(O) {
    const _ = O.name;
    do {
      if (O.props.isForm === !0)
        break;
      O.parent || error$2(106, _), O = O.parent;
    } while (O);
    O.props.id && submitForm(O.props.id, O.props.__root);
  }
  function resetValue(O, _, H) {
    return reset$2(O, H);
  }
  function setErrors(O, _, H, V) {
    const q = `${O.name}-set`, G = O.hook.setErrors.dispatch({ localErrors: H, childErrors: V });
    return createMessages(O, G.localErrors, G.childErrors).forEach(
      (J) => {
        O.store.apply(J, (Z) => Z.meta.source === q);
      }
    ), O;
  }
  function clearErrors(O, _, H = !0, V) {
    return O.store.filter((q) => !(V === void 0 || q.meta.source === V), "error"), H && (V = V || `${O.name}-set`, O.walk((q) => {
      q.store.filter((G) => !(G.type === "error" && G.meta && G.meta.source === V));
    })), O;
  }
  function createProps(O) {
    const _ = {
      initial: typeof O == "object" ? cloneAny(O) : O
    };
    let H, V = !0, q = {};
    return new Proxy(_, {
      get(...G) {
        var ee, ae, oe, se;
        const [J, Z] = G;
        let te;
        has$2(_, Z) ? (te = Reflect.get(...G), (ee = q[Z]) != null && ee.boolean && (te = boolGetter(te))) : H && typeof Z == "string" && H.config[Z] !== void 0 ? (te = H.config[Z], Z === "mergeStrategy" && (H == null ? void 0 : H.type) === "input" && isRecord(te) && H.name in te && (te = te[H.name])) : te = (ae = q[Z]) == null ? void 0 : ae.default;
        const ne = (oe = q[Z]) == null ? void 0 : oe.getter;
        return (se = q[Z]) != null && se.boolean && (te = !!te), ne ? ne(te, H) : te;
      },
      set(G, J, Z, te) {
        var oe;
        if (J === "_n")
          return H = Z, !0;
        if (J === "_emit")
          return V = Z, !0;
        let { prop: ne, value: ee } = H.hook.prop.dispatch({
          prop: J,
          value: Z
        });
        const ae = (oe = q[ne]) == null ? void 0 : oe.setter;
        if (ee = ae ? ae(ee, H) : ee, !eq$2(_[ne], ee, !1) || typeof ee == "object") {
          const se = Reflect.set(G, ne, ee, te);
          return ne === "__propDefs" && (q = toPropsObj(ee)), V && (H.emit("prop", { prop: ne, value: ee }), typeof ne == "string" && H.emit(`prop:${ne}`, ee)), se;
        }
        return !0;
      }
    });
  }
  function extend$8(O, _, H, V) {
    return _.traps.set(H, V), O;
  }
  function findDefinition(O, _) {
    if (O.props.definition)
      return O.define(O.props.definition);
    for (const H of _) {
      if (O.props.definition)
        return;
      typeof H.library == "function" && H.library(O);
    }
  }
  function createContext(O) {
    const _ = createValue(O), H = createConfig2(O.config || {}, O.parent);
    return {
      _d: 0,
      _e: createEmitter(),
      uid: Symbol(),
      _resolve: !1,
      _tmo: !1,
      _value: _,
      children: dedupe(O.children || []),
      config: H,
      hook: createHooks(),
      isCreated: !1,
      isSettled: !0,
      ledger: createLedger(),
      name: createName(O),
      parent: O.parent || null,
      plugins: /* @__PURE__ */ new Set(),
      props: createProps(_),
      settled: Promise.resolve(_),
      store: createStore$1(!0),
      sync: O.sync || !1,
      traps: createTraps(),
      type: O.type || "input",
      value: _
    };
  }
  function nodeInit(O, _) {
    var V, q;
    const H = (V = _.props) == null ? void 0 : V.id;
    if (H || (q = _.props) == null || delete q.id, O.ledger.init(O.store._n = O.props._n = O.config._n = O), O.props._emit = !1, Object.assign(
      O.props,
      H ? {} : { id: `input_${idCount++}` },
      _.props ?? {}
    ), O.props._emit = !0, findDefinition(
      O,
      /* @__PURE__ */ new Set([
        ..._.plugins || [],
        ...O.parent ? O.parent.plugins : []
      ])
    ), _.plugins)
      for (const G of _.plugins)
        use(O, O._c, G, !0, !1);
    return O.each((G) => O.add(G)), O.parent && O.parent.add(O, _.index), O.type === "input" && O.children.length && error$2(100, O), input$3(O, O._c, O._value, !1), O.store.release(), H && register$1(O), O.emit("created", O), O.isCreated = !0, O;
  }
  function createPlaceholder(O) {
    return {
      __FKP: !0,
      uid: Symbol(),
      name: (O == null ? void 0 : O.name) ?? `p_${nameCount++}`,
      value: (O == null ? void 0 : O.value) ?? null,
      _value: (O == null ? void 0 : O.value) ?? null,
      type: (O == null ? void 0 : O.type) ?? "input",
      props: {},
      use: () => {
      },
      input(_) {
        return this._value = _, this.value = _, Promise.resolve();
      },
      isSettled: !0
    };
  }
  function isPlaceholder(O) {
    return "__FKP" in O;
  }
  function createNode(O) {
    const _ = O || {}, H = createContext(_), V = new Proxy(H, {
      get(...q) {
        const [, G] = q;
        if (G === "__FKNode__")
          return !0;
        const J = H.traps.get(G);
        return J && J.get ? J.get(V, H) : Reflect.get(...q);
      },
      set(...q) {
        const [, G, J] = q, Z = H.traps.get(G);
        return Z && Z.set ? Z.set(V, H, G, J) : Reflect.set(...q);
      }
    });
    return nodeInit(V, _);
  }
  function isDOM(O) {
    return typeof O != "string" && has$2(O, "$el");
  }
  function isComponent(O) {
    return typeof O != "string" && has$2(O, "$cmp");
  }
  function isConditional(O) {
    return !O || typeof O == "string" ? !1 : has$2(O, "if") && has$2(O, "then");
  }
  function isSugar(O) {
    return typeof O != "string" && "$formkit" in O;
  }
  function sugar(O) {
    if (typeof O == "string")
      return {
        $el: "text",
        children: O
      };
    if (isSugar(O)) {
      const {
        $formkit: _,
        for: H,
        if: V,
        children: q,
        bind: G,
        ...J
      } = O;
      return Object.assign(
        {
          $cmp: "FormKit",
          props: { ...J, type: _ }
        },
        V ? { if: V } : {},
        H ? { for: H } : {},
        q ? { children: q } : {},
        G ? { bind: G } : {}
      );
    }
    return O;
  }
  function compile$1(O) {
    let _;
    const H = /* @__PURE__ */ new Set(), V = function(ue, de) {
      return typeof ue == "function" ? ue(de) : ue;
    }, q = [
      {
        "&&": (le, ue, de) => V(le, de) && V(ue, de),
        "||": (le, ue, de) => V(le, de) || V(ue, de)
      },
      {
        "===": (le, ue, de) => V(le, de) === V(ue, de),
        "!==": (le, ue, de) => V(le, de) !== V(ue, de),
        "==": (le, ue, de) => V(le, de) == V(ue, de),
        "!=": (le, ue, de) => V(le, de) != V(ue, de),
        ">=": (le, ue, de) => V(le, de) >= V(ue, de),
        "<=": (le, ue, de) => V(le, de) <= V(ue, de),
        ">": (le, ue, de) => V(le, de) > V(ue, de),
        "<": (le, ue, de) => V(le, de) < V(ue, de)
      },
      {
        "+": (le, ue, de) => V(le, de) + V(ue, de),
        "-": (le, ue, de) => V(le, de) - V(ue, de)
      },
      {
        "*": (le, ue, de) => V(le, de) * V(ue, de),
        "/": (le, ue, de) => V(le, de) / V(ue, de),
        "%": (le, ue, de) => V(le, de) % V(ue, de)
      }
    ], G = q.reduce((le, ue) => le.concat(Object.keys(ue)), []), J = new Set(G.map((le) => le.charAt(0)));
    function Z(le, ue, de, fe) {
      const he = le.filter((me) => me.startsWith(ue));
      return he.length ? he.find((me) => fe.length >= de + me.length && fe.substring(de, de + me.length) === me ? me : !1) : !1;
    }
    function te(le, ue, de = 1) {
      let fe = de ? ue.substring(le + 1).trim() : ue.substring(0, le).trim();
      if (!fe.length)
        return -1;
      if (!de) {
        const me = fe.split("").reverse(), pe = me.findIndex((ye) => J.has(ye));
        fe = me.slice(pe).join("");
      }
      const he = fe[0];
      return q.findIndex((me) => {
        const pe = Object.keys(me);
        return !!Z(pe, he, 0, fe);
      });
    }
    function ne(le, ue) {
      let de = "";
      const fe = ue.length;
      let he = 0;
      for (let me = le; me < fe; me++) {
        const pe = ue.charAt(me);
        if (pe === "(")
          he++;
        else if (pe === ")")
          he--;
        else if (he === 0 && pe === " ")
          continue;
        if (he === 0 && Z(G, pe, me, ue))
          return [de, me - 1];
        de += pe;
      }
      return [de, ue.length - 1];
    }
    function ee(le, ue = 0) {
      const de = q[ue], fe = le.length, he = Object.keys(de);
      let me = 0, pe = !1, ye = null, ge = "", ve = null, be, Ce = "", Se = "", we = "", Ee = "", xe = 0;
      const Le = (Qe, ke) => {
        Qe ? we += ke : ge += ke;
      };
      for (let Qe = 0; Qe < fe; Qe++)
        if (Ce = Se, Se = le.charAt(Qe), (Se === "'" || Se === '"') && Ce !== "\\" && (me === 0 && !pe || me && !Ee)) {
          me ? Ee = Se : pe = Se, Le(me, Se);
          continue;
        } else if (pe && (Se !== pe || Ce === "\\") || Ee && (Se !== Ee || Ce === "\\")) {
          Le(me, Se);
          continue;
        } else if (pe === Se) {
          pe = !1, Le(me, Se);
          continue;
        } else if (Ee === Se) {
          Ee = !1, Le(me, Se);
          continue;
        } else {
          if (Se === " ")
            continue;
          if (Se === "(")
            me === 0 ? xe = Qe : we += Se, me++;
          else if (Se === ")")
            if (me--, me === 0) {
              const ke = typeof ge == "string" && ge.startsWith("$") ? ge : void 0, Me = ke && le.charAt(Qe + 1) === ".";
              let Re = "";
              Me && ([Re, Qe] = ne(Qe + 2, le));
              const De = ye ? ue : te(xe, le, 0), Pe = te(Qe, le);
              De === -1 && Pe === -1 ? (ge = ae(we, -1, ke, Re), typeof ge == "string" && (ge = we)) : ye && (De >= Pe || Pe === -1) && ue === De ? (ve = ye.bind(null, ae(we, -1, ke, Re)), ye = null, ge = "") : Pe > De && ue === Pe ? ge = ae(we, -1, ke, Re) : ge += `(${we})${Me ? `.${Re}` : ""}`, we = "";
            } else
              we += Se;
          else if (me === 0 && (be = Z(he, Se, Qe, le))) {
            Qe === 0 && error$2(103, [be, le]), Qe += be.length - 1, Qe === le.length - 1 && error$2(104, [be, le]), ye ? ge && (ve = ye.bind(null, ae(ge, ue)), ye = de[be].bind(null, ve), ge = "") : ve ? (ye = de[be].bind(null, ae(ve, ue)), ve = null) : (ye = de[be].bind(null, ae(ge, ue)), ge = "");
            continue;
          } else
            Le(me, Se);
        }
      return ge && ye && (ye = ye.bind(null, ae(ge, ue))), ye = !ye && ve ? ve : ye, !ye && ge && (ye = (Qe, ke) => typeof Qe == "function" ? Qe(ke) : Qe, ye = ye.bind(null, ae(ge, ue))), !ye && !ge && error$2(105, le), ye;
    }
    function ae(le, ue, de, fe) {
      if (de) {
        const he = ae(de, q.length);
        let me, pe = fe ? compile$1(`$${fe}`) : !1;
        if (typeof he == "function") {
          const ye = parseArgs(String(le)).map(
            (ge) => ae(ge, -1)
          );
          return (ge) => {
            const ve = he(ge);
            return typeof ve != "function" ? (warn$3(150, de), ve) : (me = ve(
              ...ye.map(
                (be) => typeof be == "function" ? be(ge) : be
              )
            ), pe && (pe = pe.provide((be) => {
              const Ce = _(be);
              return be.reduce(
                (we, Ee) => {
                  if (Ee === fe || (fe == null ? void 0 : fe.startsWith(`${Ee}(`))) {
                    const Le = getAt(me, Ee);
                    we[Ee] = () => Le;
                  } else
                    we[Ee] = Ce[Ee];
                  return we;
                },
                {}
              );
            })), pe ? pe() : me);
          };
        }
      } else if (typeof le == "string") {
        if (le === "true")
          return !0;
        if (le === "false")
          return !1;
        if (le === "undefined")
          return;
        if (isQuotedString(le))
          return rmEscapes(le.substring(1, le.length - 1));
        if (!isNaN(+le))
          return Number(le);
        if (ue < q.length - 1)
          return ee(le, ue + 1);
        if (le.startsWith("$")) {
          const he = le.substring(1);
          return H.add(he), function(pe) {
            return he in pe ? pe[he]() : void 0;
          };
        }
        return le;
      }
      return le;
    }
    const oe = ee(
      O.startsWith("$:") ? O.substring(2) : O
    ), se = Array.from(H);
    function ce(le) {
      return _ = le, Object.assign(
        // @ts-ignore - @rollup/plugin-typescript doesn't like this
        oe.bind(null, le(se)),
        { provide: ce }
      );
    }
    return Object.assign(oe, {
      provide: ce
    });
  }
  function createClasses(O, _, H) {
    return H ? typeof H == "string" ? H.split(" ").reduce(
      (q, G) => Object.assign(q, { [G]: !0 }),
      {}
    ) : typeof H == "function" ? createClasses(
      O,
      _,
      H(_, O)
    ) : H : {};
  }
  function generateClassList(O, _, ...H) {
    const V = H.reduce((q, G) => {
      if (!G)
        return handleNegativeClasses(q);
      const { $reset: J, ...Z } = G;
      return handleNegativeClasses(J ? Z : Object.assign(q, Z));
    }, {});
    return Object.keys(
      O.hook.classes.dispatch({ property: _, classes: V }).classes
    ).filter((q) => V[q]).join(" ") || null;
  }
  function handleNegativeClasses(O) {
    const _ = "$remove:";
    let H = !1;
    const V = Object.keys(O).filter((q) => (O[q] && q.startsWith(_) && (H = !0), O[q]));
    return V.length > 1 && H && V.filter((G) => G.startsWith(_)).map((G) => {
      const J = G.substring(_.length);
      O[J] = !1, O[G] = !1;
    }), O;
  }
  function setErrors2(O, _, H) {
    const V = getNode(O);
    V ? V.setErrors(_, H) : warn$3(651, O);
  }
  function clearErrors2(O, _ = !0) {
    const H = getNode(O);
    H ? H.clearErrors(_) : warn$3(652, O);
  }
  var FORMKIT_VERSION = "1.6.9", revokedObservers = /* @__PURE__ */ new WeakSet();
  function createObserver(O, _) {
    const H = _ || Object.assign(/* @__PURE__ */ new Map(), { active: !1 }), V = /* @__PURE__ */ new Map(), q = function(ee) {
      var ae;
      H.active && (H.has(O) || H.set(O, /* @__PURE__ */ new Set()), (ae = H.get(O)) == null || ae.add(ee));
    }, G = function(ee) {
      return new Proxy(ee, {
        get(...ae) {
          return typeof ae[1] == "string" && q(`prop:${ae[1]}`), Reflect.get(...ae);
        }
      });
    }, J = function(ee) {
      return new Proxy(ee, {
        get(...ae) {
          return ae[1] === "value" ? (oe) => (q(`count:${oe}`), ee.value(oe)) : Reflect.get(...ae);
        }
      });
    }, Z = function(ee, ae) {
      return isNode(ee) ? createObserver(ee, H) : (ae === "value" && q("commit"), ae === "_value" && q("input"), ae === "props" ? G(ee) : ae === "ledger" ? J(ee) : (ae === "children" && (q("child"), q("childRemoved")), ee));
    }, {
      proxy: te,
      revoke: ne
    } = Proxy.revocable(O, {
      get(...ee) {
        switch (ee[1]) {
          case "_node":
            return O;
          case "deps":
            return H;
          case "watch":
            return (oe, se, ce) => watch(te, oe, se, ce);
          case "observe":
            return () => {
              const oe = new Map(H);
              return H.clear(), H.active = !0, oe;
            };
          case "stopObserve":
            return () => {
              const oe = new Map(H);
              return H.active = !1, oe;
            };
          case "receipts":
            return V;
          case "kill":
            return () => {
              removeListeners(V), revokedObservers.add(ee[2]), ne();
            };
        }
        const ae = Reflect.get(...ee);
        return typeof ae == "function" ? (...oe) => {
          const se = ae(...oe);
          return Z(se, ee[1]);
        } : Z(ae, ee[1]);
      }
    });
    return te;
  }
  function applyListeners(O, [_, H], V, q) {
    _.forEach((G, J) => {
      G.forEach((Z) => {
        O.receipts.has(J) || O.receipts.set(J, {});
        const te = O.receipts.get(J) ?? {};
        te[Z] = te[Z] ?? [], te[Z].push(J.on(Z, V, q)), O.receipts.set(J, te);
      });
    }), H.forEach((G, J) => {
      G.forEach((Z) => {
        if (O.receipts.has(J)) {
          const te = O.receipts.get(J);
          te && has$2(te, Z) && (te[Z].map(J.off), delete te[Z], O.receipts.set(J, te));
        }
      });
    });
  }
  function removeListeners(O) {
    O.forEach((_, H) => {
      for (const V in _)
        _[V].map(H.off);
    }), O.clear();
  }
  function watch(O, _, H, V) {
    const q = (Z) => {
      const te = O.stopObserve();
      applyListeners(
        O,
        diffDeps(G, te),
        () => watch(O, _, H, V),
        V
      ), H && H(Z);
    }, G = new Map(O.deps);
    O.observe();
    const J = _(O);
    J instanceof Promise ? J.then((Z) => q(Z)) : q(J);
  }
  function diffDeps(O, _) {
    const H = /* @__PURE__ */ new Map(), V = /* @__PURE__ */ new Map();
    return _.forEach((q, G) => {
      if (!O.has(G))
        H.set(G, q);
      else {
        const J = /* @__PURE__ */ new Set(), Z = O.get(G);
        q.forEach(
          (te) => !(Z != null && Z.has(te)) && J.add(te)
        ), H.set(G, J);
      }
    }), O.forEach((q, G) => {
      if (!_.has(G))
        V.set(G, q);
      else {
        const J = /* @__PURE__ */ new Set(), Z = _.get(G);
        q.forEach(
          (te) => !(Z != null && Z.has(te)) && J.add(te)
        ), V.set(G, J);
      }
    }), [H, V];
  }
  function isKilled(O) {
    return revokedObservers.has(O);
  }
  var accepted = function({ value: _ }) {
    return ["yes", "on", "1", 1, !0, "true"].includes(_);
  };
  accepted.skipEmpty = !1;
  var accepted_default = accepted, date_after = function({ value: O }, _ = !1) {
    const H = Date.parse(_ || /* @__PURE__ */ new Date()), V = Date.parse(String(O));
    return isNaN(V) ? !1 : V > H;
  }, date_after_default = date_after, date_after_or_equal = function({ value: O }, _ = !1) {
    const H = Date.parse(_ || /* @__PURE__ */ new Date()), V = Date.parse(String(O));
    return isNaN(V) ? !1 : V > H || V === H;
  }, date_after_or_equal_default = date_after_or_equal, date_after_node = function(O, _) {
    var q;
    if (!_)
      return !1;
    const H = Date.parse(String(O.value)), V = Date.parse(String((q = O.at(_)) == null ? void 0 : q.value));
    return isNaN(V) ? !0 : isNaN(H) ? !1 : H > V;
  }, date_after_node_default = date_after_node, alpha = function({ value: O }, _ = "default") {
    const H = {
      default: new RegExp("^\\p{L}+$", "u"),
      latin: /^[a-z]+$/i
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, alpha_default = alpha, alpha_spaces = function({ value: O }, _ = "default") {
    const H = {
      default: /^[\p{L} ]+$/u,
      latin: /^[a-z ]+$/i
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, alpha_spaces_default = alpha_spaces, alphanumeric = function({ value: O }, _ = "default") {
    const H = {
      default: /^[0-9\p{L}]+$/u,
      latin: /^[0-9a-z]+$/i
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, alphanumeric_default = alphanumeric, date_before = function({ value: O }, _ = !1) {
    const H = Date.parse(_ || /* @__PURE__ */ new Date()), V = Date.parse(String(O));
    return isNaN(V) ? !1 : V < H;
  }, date_before_default = date_before, date_before_node = function(O, _) {
    var q;
    if (!_)
      return !1;
    const H = Date.parse(String(O.value)), V = Date.parse(String((q = O.at(_)) == null ? void 0 : q.value));
    return isNaN(V) ? !0 : isNaN(H) ? !1 : H < V;
  }, date_before_node_default = date_before_node, date_before_or_equal = function({ value: O }, _ = !1) {
    const H = Date.parse(_ || /* @__PURE__ */ new Date()), V = Date.parse(String(O));
    return isNaN(V) ? !1 : V < H || V === H;
  }, date_before_or_equal_default = date_before_or_equal, between = function({ value: _ }, H, V) {
    if (!isNaN(_) && !isNaN(H) && !isNaN(V)) {
      const q = 1 * _;
      H = Number(H), V = Number(V);
      const [G, J] = H <= V ? [H, V] : [V, H];
      return q >= 1 * G && q <= 1 * J;
    }
    return !1;
  }, between_default = between, hasConfirm = /(_confirm(?:ed)?)$/, confirm = function(_, H, V = "loose") {
    var G;
    H || (H = hasConfirm.test(_.name) ? _.name.replace(hasConfirm, "") : `${_.name}_confirm`);
    const q = (G = _.at(H)) == null ? void 0 : G.value;
    return V === "strict" ? _.value === q : _.value == q;
  }, confirm_default = confirm, contains_alpha = function({ value: O }, _ = "default") {
    const H = {
      default: new RegExp("\\p{L}", "u"),
      latin: /[a-z]/i
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, contains_alpha_default = contains_alpha, contains_alpha_spaces = function({ value: O }, _ = "default") {
    const H = {
      default: /[\p{L} ]/u,
      latin: /[a-z ]/i
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, contains_alpha_spaces_default = contains_alpha_spaces, contains_alphanumeric = function({ value: O }, _ = "default") {
    const H = {
      default: /[0-9\p{L}]/u,
      latin: /[0-9a-z]/i
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, contains_alphanumeric_default = contains_alphanumeric, contains_lowercase = function({ value: O }, _ = "default") {
    const H = {
      default: new RegExp("\\p{Ll}", "u"),
      latin: /[a-z]/
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, contains_lowercase_default = contains_lowercase, contains_numeric = function({ value: _ }) {
    return /[0-9]/.test(String(_));
  }, contains_numeric_default = contains_numeric, contains_symbol = function({ value: O }) {
    return /[!-/:-@[-`{-~]/.test(String(O));
  }, contains_symbol_default = contains_symbol, contains_uppercase = function({ value: O }, _ = "default") {
    const H = {
      default: new RegExp("\\p{Lu}", "u"),
      latin: /[A-Z]/
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, contains_uppercase_default = contains_uppercase, date_between = function({ value: _ }, H, V) {
    H = H instanceof Date ? H.getTime() : Date.parse(H), V = V instanceof Date ? V.getTime() : Date.parse(V);
    const q = _ instanceof Date ? _.getTime() : Date.parse(String(_));
    if (H && !V)
      V = H, H = Date.now();
    else if (!H || !q)
      return !1;
    return q >= H && q <= V;
  }, date_between_default = date_between, date_format = function({ value: _ }, H) {
    return H && typeof H == "string" ? regexForFormat(H).test(String(_)) : !isNaN(Date.parse(String(_)));
  }, date_format_default = date_format, email = function({ value: _ }) {
    return /^(([^<>()\[\]\.,;:\s@\"]+(\.[^<>()\[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i.test(String(_));
  }, email_default = email, ends_with = function({ value: _ }, ...H) {
    return typeof _ == "string" && H.length ? H.some((V) => _.endsWith(V)) : typeof _ == "string" && H.length === 0;
  }, ends_with_default = ends_with, is = function({ value: _ }, ...H) {
    return H.some((V) => typeof V == "object" ? eq$2(V, _) : V == _);
  }, is_default = is, length$1 = function({ value: _ }, H = 0, V = 1 / 0) {
    H = parseInt(H), V = isNaN(parseInt(V)) ? 1 / 0 : parseInt(V);
    const q = H <= V ? H : V, G = V >= H ? V : H;
    if (typeof _ == "string" || Array.isArray(_))
      return _.length >= q && _.length <= G;
    if (_ && typeof _ == "object") {
      const J = Object.keys(_).length;
      return J >= q && J <= G;
    }
    return !1;
  }, length_default = length$1, lowercase = function({ value: O }, _ = "default") {
    const H = {
      default: new RegExp("^\\p{Ll}+$", "u"),
      allow_non_alpha: /^[0-9\p{Ll}!-/:-@[-`{-~]+$/u,
      allow_numeric: /^[0-9\p{Ll}]+$/u,
      allow_numeric_dashes: /^[0-9\p{Ll}-]+$/u,
      latin: /^[a-z]+$/
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, lowercase_default = lowercase, matches = function({ value: _ }, ...H) {
    return H.some((V) => (typeof V == "string" && V.substr(0, 1) === "/" && V.substr(-1) === "/" && (V = new RegExp(V.substr(1, V.length - 2))), V instanceof RegExp ? V.test(String(_)) : V === _));
  }, matches_default = matches, max$q = function({ value: _ }, H = 10) {
    return Array.isArray(_) ? _.length <= H : Number(_) <= Number(H);
  }, max_default = max$q, min$i = function({ value: _ }, H = 1) {
    return Array.isArray(_) ? _.length >= H : Number(_) >= Number(H);
  }, min_default = min$i, not$1 = function({ value: _ }, ...H) {
    return !H.some((V) => typeof V == "object" ? eq$2(V, _) : V === _);
  }, not_default = not$1, number2 = function({ value: _ }) {
    return !isNaN(_);
  }, number_default = number2, require_one = function(O, ..._) {
    return empty$3(O.value) ? _.map((V) => {
      var q;
      return (q = O.at(V)) == null ? void 0 : q.value;
    }).some((V) => !empty$3(V)) : !0;
  };
  require_one.skipEmpty = !1;
  var require_one_default = require_one, required$1 = function({ value: _ }, H = "default") {
    return H === "trim" && typeof _ == "string" ? !empty$3(_.trim()) : !empty$3(_);
  };
  required$1.skipEmpty = !1;
  var required_default = required$1, starts_with = function({ value: _ }, ...H) {
    return typeof _ == "string" && H.length ? H.some((V) => _.startsWith(V)) : typeof _ == "string" && H.length === 0;
  }, starts_with_default = starts_with, symbol$3 = function({ value: O }) {
    return /^[!-/:-@[-`{-~]+$/.test(String(O));
  }, symbol_default = symbol$3, uppercase$1 = function({ value: O }, _ = "default") {
    const H = {
      default: new RegExp("^\\p{Lu}+$", "u"),
      latin: /^[A-Z]+$/
    }, V = has$2(H, _) ? _ : "default";
    return H[V].test(String(O));
  }, uppercase_default = uppercase$1, url = function({ value: _ }, ...H) {
    try {
      const V = H.length ? H : ["http:", "https:"], q = new URL(String(_));
      return V.includes(q.protocol);
    } catch {
      return !1;
    }
  }, url_default = url;
  const defaultRules = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    accepted: accepted_default,
    alpha: alpha_default,
    alpha_spaces: alpha_spaces_default,
    alphanumeric: alphanumeric_default,
    between: between_default,
    confirm: confirm_default,
    contains_alpha: contains_alpha_default,
    contains_alpha_spaces: contains_alpha_spaces_default,
    contains_alphanumeric: contains_alphanumeric_default,
    contains_lowercase: contains_lowercase_default,
    contains_numeric: contains_numeric_default,
    contains_symbol: contains_symbol_default,
    contains_uppercase: contains_uppercase_default,
    date_after: date_after_default,
    date_after_node: date_after_node_default,
    date_after_or_equal: date_after_or_equal_default,
    date_before: date_before_default,
    date_before_node: date_before_node_default,
    date_before_or_equal: date_before_or_equal_default,
    date_between: date_between_default,
    date_format: date_format_default,
    email: email_default,
    ends_with: ends_with_default,
    is: is_default,
    length: length_default,
    lowercase: lowercase_default,
    matches: matches_default,
    max: max_default,
    min: min_default,
    not: not_default,
    number: number_default,
    require_one: require_one_default,
    required: required_default,
    starts_with: starts_with_default,
    symbol: symbol_default,
    uppercase: uppercase_default,
    url: url_default
  }, Symbol.toStringTag, { value: "Module" }));
  var validatingMessage = /* @__PURE__ */ createMessage({
    type: "state",
    blocking: !0,
    visible: !1,
    value: !0,
    key: "validating"
  });
  function createValidationPlugin(O = {}) {
    return function(H) {
      let V = cloneAny(H.props.validationRules || {}), q = { ...O, ...V };
      const G = { input: token$2(), rerun: null, isPassing: !0 };
      let J = cloneAny(H.props.validation);
      H.on("prop:validation", ({ payload: te }) => Z(te, V)), H.on(
        "prop:validationRules",
        ({ payload: te }) => Z(J, te)
      );
      function Z(te, ne) {
        var ee;
        eq$2(Object.keys(V || {}), Object.keys(ne || {})) && eq$2(J, te) || (V = cloneAny(ne), J = cloneAny(te), q = { ...O, ...V }, (ee = H.props.parsedRules) == null || ee.forEach((ae) => {
          removeMessage(ae), removeListeners(ae.observer.receipts), ae.observer.kill();
        }), H.store.filter(() => !1, "validation"), H.props.parsedRules = parseRules(te, q, H), G.isPassing = !0, validate$1(H, H.props.parsedRules, G));
      }
      H.props.parsedRules = parseRules(J, q, H), validate$1(H, H.props.parsedRules, G);
    };
  }
  function validate$1(O, _, H) {
    isKilled(O) || (H.input = token$2(), O.store.set(
      /* @__PURE__ */ createMessage({
        key: "failing",
        value: !H.isPassing,
        visible: !1
      })
    ), H.isPassing = !0, O.store.filter((V) => !V.meta.removeImmediately, "validation"), _.forEach(
      (V) => V.debounce && clearTimeout(V.timer)
    ), _.length && (O.store.set(validatingMessage), run$1(0, _, H, !1, () => {
      O.store.remove(validatingMessage.key), O.store.set(
        /* @__PURE__ */ createMessage({
          key: "failing",
          value: !H.isPassing,
          visible: !1
        })
      );
    })));
  }
  function run$1(O, _, H, V, q) {
    const G = _[O];
    if (!G)
      return q();
    const J = G.observer;
    if (isKilled(J))
      return;
    const Z = H.input;
    G.state = null;
    function te(ne, ee) {
      if (H.input !== Z)
        return;
      H.isPassing = H.isPassing && !!ee, G.queued = !1;
      const ae = J.stopObserve(), oe = diffDeps(G.deps, ae);
      if (applyListeners(
        J,
        oe,
        function() {
          try {
            J.store.set(validatingMessage);
          } catch {
          }
          G.queued = !0, H.rerun && clearTimeout(H.rerun), H.rerun = setTimeout(
            validate$1,
            0,
            J,
            _,
            H
          );
        },
        "unshift"
        // We want these listeners to run before other events are emitted so the 'state.validating' will be reliable.
      ), G.deps = ae, G.state = ee, ee === !1 ? createFailedMessage(G, V || ne) : removeMessage(G), _.length > O + 1) {
        const se = _[O + 1];
        (ee || se.force || !se.skipEmpty) && se.state === null && (se.queued = !0), run$1(O + 1, _, H, V || ne, q);
      } else
        q();
    }
    (!empty$3(J.value) || !G.skipEmpty) && (H.isPassing || G.force) ? G.queued ? runRule(G, J, (ne) => {
      ne instanceof Promise ? ne.then((ee) => te(!0, ee)) : te(!1, ne);
    }) : run$1(O + 1, _, H, V, q) : empty$3(J.value) && G.skipEmpty && H.isPassing ? (J.observe(), J.value, te(!1, H.isPassing)) : te(!1, null);
  }
  function runRule(O, _, H) {
    O.debounce ? O.timer = setTimeout(() => {
      _.observe(), H(O.rule(_, ...O.args));
    }, O.debounce) : (_.observe(), H(O.rule(_, ...O.args)));
  }
  function removeMessage(O) {
    const _ = `rule_${O.name}`;
    O.messageObserver && (O.messageObserver = O.messageObserver.kill()), has$2(O.observer.store, _) && O.observer.store.remove(_);
  }
  function createFailedMessage(O, _) {
    const H = O.observer;
    isKilled(H) || (O.messageObserver || (O.messageObserver = createObserver(H._node)), O.messageObserver.watch(
      (V) => createI18nArgs(
        V,
        O
      ),
      (V) => {
        const q = createCustomMessage(H, O, V), G = /* @__PURE__ */ createMessage({
          blocking: O.blocking,
          key: `rule_${O.name}`,
          meta: {
            /**
             * Use this key instead of the message root key to produce i18n validation
             * messages.
             */
            messageKey: O.name,
            /**
             * For messages that were created *by or after* a debounced or async
             * validation rule — we make note of it so we can immediately remove them
             * as soon as the next commit happens.
             */
            removeImmediately: _,
            /**
             * Determines if this message should be passed to localization.
             */
            localize: !q,
            /**
             * The arguments that will be passed to the validation rules
             */
            i18nArgs: V
          },
          type: "validation",
          value: q || "This field is not valid."
        });
        H.store.set(G);
      }
    ));
  }
  function createCustomMessage(O, _, H) {
    const V = O.props.validationMessages && has$2(O.props.validationMessages, _.name) ? O.props.validationMessages[_.name] : void 0;
    return typeof V == "function" ? V(...H) : V;
  }
  function createI18nArgs(O, _) {
    return [
      {
        node: O,
        name: createMessageName(O),
        args: _.args
      }
    ];
  }
  function createMessageName(O) {
    return typeof O.props.validationLabel == "function" ? O.props.validationLabel(O) : O.props.validationLabel || O.props.label || O.props.name || String(O.name);
  }
  var hintPattern = "(?:[\\*+?()0-9]+)", rulePattern = "[a-zA-Z][a-zA-Z0-9_]+", ruleExtractor = new RegExp(
    `^(${hintPattern}?${rulePattern})(?:\\:(.*)+)?$`,
    "i"
  ), hintExtractor = new RegExp(`^(${hintPattern})(${rulePattern})$`, "i"), debounceExtractor = /([\*+?]+)?(\(\d+\))([\*+?]+)?/, hasDebounce = /\(\d+\)/, defaultHints = {
    blocking: !0,
    debounce: 0,
    force: !1,
    skipEmpty: !0,
    name: ""
  };
  function parseRules(O, _, H) {
    return O ? (typeof O == "string" ? extractRules(O) : clone$3(O)).reduce((q, G) => {
      let J = G.shift();
      const Z = {};
      if (typeof J == "string") {
        const [te, ne] = parseHints(J);
        has$2(_, te) && (J = _[te], Object.assign(Z, ne));
      }
      return typeof J == "function" && q.push({
        observer: createObserver(H),
        rule: J,
        args: G,
        timer: 0,
        state: null,
        queued: !0,
        deps: /* @__PURE__ */ new Map(),
        ...defaultHints,
        ...fnHints(Z, J)
      }), q;
    }, []) : [];
  }
  function extractRules(O) {
    return O.split("|").reduce((_, H) => {
      const V = parseRule(H);
      return V && _.push(V), _;
    }, []);
  }
  function parseRule(O) {
    const _ = O.trim();
    if (_) {
      const H = _.match(ruleExtractor);
      if (H && typeof H[1] == "string") {
        const V = H[1].trim(), q = H[2] && typeof H[2] == "string" ? H[2].split(",").map((G) => G.trim()) : [];
        return [V, ...q];
      }
    }
    return !1;
  }
  function parseHints(O) {
    const _ = O.match(hintExtractor);
    if (!_)
      return [O, { name: O }];
    const H = {
      "*": { force: !0 },
      "+": { skipEmpty: !1 },
      "?": { blocking: !1 }
    }, [, V, q] = _, G = hasDebounce.test(V) ? V.match(debounceExtractor) || [] : [, V];
    return [
      q,
      [G[1], G[2], G[3]].reduce(
        (J, Z) => (Z && (hasDebounce.test(Z) ? J.debounce = parseInt(Z.substr(1, Z.length - 1)) : Z.split("").forEach(
          (te) => has$2(H, te) && Object.assign(J, H[te])
        )), J),
        { name: q }
      )
    ];
  }
  function fnHints(O, _) {
    return O.name || (O.name = _.ruleName || _.name), ["skipEmpty", "force", "debounce", "blocking"].reduce(
      (H, V) => (has$2(_, V) && !has$2(H, V) && Object.assign(H, {
        [V]: _[V]
      }), H),
      O
    );
  }
  function sentence(O) {
    return O[0].toUpperCase() + O.substr(1);
  }
  function list$2(O, _ = "or") {
    return O.reduce((H, V, q) => (H += V, q <= O.length - 2 && O.length > 2 && (H += ", "), q === O.length - 2 && (H += `${O.length === 2 ? " " : ""}${_} `), H), "");
  }
  function date$2(O) {
    const _ = typeof O == "string" ? new Date(Date.parse(O)) : O;
    return _ instanceof Date ? new Intl.DateTimeFormat(void 0, {
      dateStyle: "medium",
      timeZone: "UTC"
    }).format(_) : "(unknown)";
  }
  function order(O, _) {
    return Number(O) >= Number(_) ? [_, O] : [O, _];
  }
  var ui10 = {
    /**
     * Shown on a button for adding additional items.
     */
    add: "Add",
    /**
     * Shown when a button to remove items is visible.
     */
    remove: "Remove",
    /**
     * Shown when there are multiple items to remove at the same time.
     */
    removeAll: "Remove all",
    /**
     * Shown when all fields are not filled out correctly.
     */
    incomplete: "Sorry, not all fields are filled out correctly.",
    /**
     * Shown in a button inside a form to submit the form.
     */
    submit: "Submit",
    /**
     * Shown when no files are selected.
     */
    noFiles: "No file chosen",
    /**
     * Shown on buttons that move fields up in a list.
     */
    moveUp: "Move up",
    /**
     * Shown on buttons that move fields down in a list.
     */
    moveDown: "Move down",
    /**
     * Shown when something is actively loading.
     */
    isLoading: "Loading...",
    /**
     * Shown when there is more to load.
     */
    loadMore: "Load more",
    /**
     * Show on buttons that navigate state forward
     */
    next: "Next",
    /**
     * Show on buttons that navigate state backward
     */
    prev: "Previous",
    /**
     * Shown when adding all values.
     */
    addAllValues: "Add all values",
    /**
     * Shown when adding selected values.
     */
    addSelectedValues: "Add selected values",
    /**
     * Shown when removing all values.
     */
    removeAllValues: "Remove all values",
    /**
     * Shown when removing selected values.
     */
    removeSelectedValues: "Remove selected values",
    /**
     * Shown when there is a date to choose.
     */
    chooseDate: "Choose date",
    /**
     * Shown when there is a date to change.
     */
    changeDate: "Change date",
    /**
     * Shown above error summaries when someone attempts to submit a form with
     * errors and the developer has implemented `<FormKitSummary />`.
     */
    summaryHeader: "There were errors in your form.",
    /*
     * Shown when there is something to close
     */
    close: "Close",
    /**
     * Shown when there is something to open.
     */
    open: "Open"
  }, validation10 = {
    /**
     * The value is not an accepted value.
     * @see {@link https://formkit.com/essentials/validation#accepted}
     */
    accepted({ name: O }) {
      return `Please accept the ${O}.`;
    },
    /**
     * The date is not after
     * @see {@link https://formkit.com/essentials/validation#date-after}
     */
    date_after({ name: O, args: _ }) {
      return Array.isArray(_) && _.length ? `${sentence(O)} must be after ${date$2(_[0])}.` : `${sentence(O)} must be in the future.`;
    },
    /**
     * The value is not a letter.
     * @see {@link https://formkit.com/essentials/validation#alpha}
     */
    alpha({ name: O }) {
      return `${sentence(O)} can only contain alphabetical characters.`;
    },
    /**
     * The value is not alphanumeric
     * @see {@link https://formkit.com/essentials/validation#alphanumeric}
     */
    alphanumeric({ name: O }) {
      return `${sentence(O)} can only contain letters and numbers.`;
    },
    /**
     * The value is not letter and/or spaces
     * @see {@link https://formkit.com/essentials/validation#alpha-spaces}
     */
    alpha_spaces({ name: O }) {
      return `${sentence(O)} can only contain letters and spaces.`;
    },
    /**
     * The value have no letter.
     * @see {@link https://formkit.com/essentials/validation#contains_alpha}
     */
    contains_alpha({ name: O }) {
      return `${sentence(O)} must contain alphabetical characters.`;
    },
    /**
     * The value have no alphanumeric
     * @see {@link https://formkit.com/essentials/validation#contains_alphanumeric}
     */
    contains_alphanumeric({ name: O }) {
      return `${sentence(O)} must contain letters or numbers.`;
    },
    /**
     * The value have no letter and/or spaces
     * @see {@link https://formkit.com/essentials/validation#contains_alpha-spaces}
     */
    contains_alpha_spaces({ name: O }) {
      return `${sentence(O)} must contain letters or spaces.`;
    },
    /**
     * The value have no symbol
     * @see {@link https://formkit.com/essentials/validation#contains_symbol}
     */
    contains_symbol({ name: O }) {
      return `${sentence(O)} must contain a symbol.`;
    },
    /**
     * The value have no uppercase
     * @see {@link https://formkit.com/essentials/validation#contains_uppercase}
     */
    contains_uppercase({ name: O }) {
      return `${sentence(O)} must contain an uppercase letter.`;
    },
    /**
     * The value have no lowercase
     * @see {@link https://formkit.com/essentials/validation#contains_lowercase}
     */
    contains_lowercase({ name: O }) {
      return `${sentence(O)} must contain a lowercase letter.`;
    },
    /**
     *  The value have no numeric
     * @see {@link https://formkit.com/essentials/validation#contains_numeric}
     */
    contains_numeric({ name: O }) {
      return `${sentence(O)} must contain numbers.`;
    },
    /**
     * The value is not symbol
     * @see {@link https://formkit.com/essentials/validation#symbol}
     */
    symbol({ name: O }) {
      return `${sentence(O)} must be a symbol.`;
    },
    /**
     * The value is not uppercase
     * @see {@link https://formkit.com/essentials/validation#uppercase}
     */
    uppercase({ name: O }) {
      return `${sentence(O)} can only contain uppercase letters.`;
    },
    /**
     * The value is not lowercase
     * @see {@link https://formkit.com/essentials/validation#lowercase}
     */
    lowercase({ name: O, args: _ }) {
      let H = "";
      return Array.isArray(_) && _.length && (_[0] === "allow_non_alpha" && (H = ", numbers and symbols"), _[0] === "allow_numeric" && (H = " and numbers"), _[0] === "allow_numeric_dashes" && (H = ", numbers and dashes")), `${sentence(O)} can only contain lowercase letters${H}.`;
    },
    /**
     * The date is not before
     * @see {@link https://formkit.com/essentials/validation#date-before}
     */
    date_before({ name: O, args: _ }) {
      return Array.isArray(_) && _.length ? `${sentence(O)} must be before ${date$2(_[0])}.` : `${sentence(O)} must be in the past.`;
    },
    /**
     * The value is not between two numbers
     * @see {@link https://formkit.com/essentials/validation#between}
     */
    between({ name: O, args: _ }) {
      if (isNaN(_[0]) || isNaN(_[1]))
        return "This field was configured incorrectly and can’t be submitted.";
      const [H, V] = order(_[0], _[1]);
      return `${sentence(O)} must be between ${H} and ${V}.`;
    },
    /**
     * The confirmation field does not match
     * @see {@link https://formkit.com/essentials/validation#confirm}
     */
    confirm({ name: O }) {
      return `${sentence(O)} does not match.`;
    },
    /**
     * The value is not a valid date
     * @see {@link https://formkit.com/essentials/validation#date-format}
     */
    date_format({ name: O, args: _ }) {
      return Array.isArray(_) && _.length ? `${sentence(O)} is not a valid date, please use the format ${_[0]}` : "This field was configured incorrectly and can’t be submitted";
    },
    /**
     * Is not within expected date range
     * @see {@link https://formkit.com/essentials/validation#date-between}
     */
    date_between({ name: O, args: _ }) {
      return `${sentence(O)} must be between ${date$2(_[0])} and ${date$2(_[1])}`;
    },
    /**
     * Shown when the user-provided value is not a valid email address.
     * @see {@link https://formkit.com/essentials/validation#email}
     */
    email: "Please enter a valid email address.",
    /**
     * Does not end with the specified value
     * @see {@link https://formkit.com/essentials/validation#ends-with}
     */
    ends_with({ name: O, args: _ }) {
      return `${sentence(O)} doesn’t end with ${list$2(_)}.`;
    },
    /**
     * Is not an allowed value
     * @see {@link https://formkit.com/essentials/validation#is}
     */
    is({ name: O }) {
      return `${sentence(O)} is not an allowed value.`;
    },
    /**
     * Does not match specified length
     * @see {@link https://formkit.com/essentials/validation#length}
     */
    length({ name: O, args: [_ = 0, H = 1 / 0] }) {
      const V = Number(_) <= Number(H) ? _ : H, q = Number(H) >= Number(_) ? H : _;
      return V == 1 && q === 1 / 0 ? `${sentence(O)} must be at least one character.` : V == 0 && q ? `${sentence(O)} must be less than or equal to ${q} characters.` : V === q ? `${sentence(O)} should be ${q} characters long.` : V && q === 1 / 0 ? `${sentence(O)} must be greater than or equal to ${V} characters.` : `${sentence(O)} must be between ${V} and ${q} characters.`;
    },
    /**
     * Value is not a match
     * @see {@link https://formkit.com/essentials/validation#matches}
     */
    matches({ name: O }) {
      return `${sentence(O)} is not an allowed value.`;
    },
    /**
     * Exceeds maximum allowed value
     * @see {@link https://formkit.com/essentials/validation#max}
     */
    max({ name: O, node: { value: _ }, args: H }) {
      return Array.isArray(_) ? `Cannot have more than ${H[0]} ${O}.` : `${sentence(O)} must be no more than ${H[0]}.`;
    },
    /**
     * The (field-level) value does not match specified mime type
     * @see {@link https://formkit.com/essentials/validation#mime}
     */
    mime({ name: O, args: _ }) {
      return _[0] ? `${sentence(O)} must be of the type: ${_[0]}` : "No file formats allowed.";
    },
    /**
     * Does not fulfill minimum allowed value
     * @see {@link https://formkit.com/essentials/validation#min}
     */
    min({ name: O, node: { value: _ }, args: H }) {
      return Array.isArray(_) ? `Cannot have fewer than ${H[0]} ${O}.` : `${sentence(O)} must be at least ${H[0]}.`;
    },
    /**
     * Is not an allowed value
     * @see {@link https://formkit.com/essentials/validation#not}
     */
    not({ name: O, node: { value: _ } }) {
      return `“${_}” is not an allowed ${O}.`;
    },
    /**
     *  Is not a number
     * @see {@link https://formkit.com/essentials/validation#number}
     */
    number({ name: O }) {
      return `${sentence(O)} must be a number.`;
    },
    /**
     * Require one field.
     * @see {@link https://formkit.com/essentials/validation#require-one}
     */
    require_one: ({ name: O, node: _, args: H }) => {
      const V = H.map((q) => {
        const G = _.at(q);
        return G ? createMessageName(G) : !1;
      }).filter((q) => !!q);
      return V.unshift(O), `${V.join(" or ")} is required.`;
    },
    /**
     * Required field.
     * @see {@link https://formkit.com/essentials/validation#required}
     */
    required({ name: O }) {
      return `${sentence(O)} is required.`;
    },
    /**
     * Does not start with specified value
     * @see {@link https://formkit.com/essentials/validation#starts-with}
     */
    starts_with({ name: O, args: _ }) {
      return `${sentence(O)} doesn’t start with ${list$2(_)}.`;
    },
    /**
     * Is not a url
     * @see {@link https://formkit.com/essentials/validation#url}
     */
    url() {
      return "Please enter a valid URL.";
    },
    /**
     * Shown when the date is invalid.
     */
    invalidDate: "The selected date is invalid."
  }, en = { ui: ui10, validation: validation10 }, i18nNodes = /* @__PURE__ */ new Set();
  function createI18nPlugin(O) {
    return function(H) {
      i18nNodes.add(H), H.on("destroying", () => i18nNodes.delete(H));
      let V = parseLocale(H.config.locale, O), q = V ? O[V] : {};
      H.on("prop:locale", ({ payload: G }) => {
        V = parseLocale(G, O), q = V ? O[V] : {}, H.store.touch();
      }), H.on("prop:label", () => H.store.touch()), H.on("prop:validationLabel", () => H.store.touch()), H.hook.text((G, J) => {
        var te, ne;
        const Z = ((te = G.meta) == null ? void 0 : te.messageKey) || G.key;
        if (has$2(q, G.type) && has$2(q[G.type], Z)) {
          const ee = q[G.type][Z];
          typeof ee == "function" ? G.value = Array.isArray((ne = G.meta) == null ? void 0 : ne.i18nArgs) ? ee(...G.meta.i18nArgs) : ee(G) : G.value = ee;
        }
        return J(G);
      });
    };
  }
  function parseLocale(O, _) {
    if (has$2(_, O))
      return O;
    const [H] = O.split("-");
    if (has$2(_, H))
      return H;
    for (const V in _)
      return V;
    return !1;
  }
  function createLibraryPlugin(...O) {
    const _ = O.reduce(
      (V, q) => extend$9(V, q),
      {}
    ), H = () => {
    };
    return H.library = function(V) {
      const q = camel(V.props.type);
      has$2(_, q) && V.define(_[q]);
    }, H;
  }
  var runtimeProps = [
    "classes",
    "config",
    "delay",
    "errors",
    "id",
    "index",
    "inputErrors",
    "library",
    "modelValue",
    "onUpdate:modelValue",
    "name",
    "number",
    "parent",
    "plugins",
    "sectionsSchema",
    "type",
    "validation",
    "validationLabel",
    "validationMessages",
    "validationRules",
    // Runtime event props:
    "onInput",
    "onInputRaw",
    "onUpdate:modelValue",
    "onNode",
    "onSubmit",
    "onSubmitInvalid",
    "onSubmitRaw"
  ];
  function isGroupOption(O) {
    return O && typeof O == "object" && "group" in O && Array.isArray(O.options);
  }
  function normalizeOptions(O, _ = { count: 1 }) {
    return Array.isArray(O) ? O.map(
      (H) => {
        if (typeof H == "string" || typeof H == "number")
          return {
            label: String(H),
            value: String(H)
          };
        if (typeof H == "object") {
          if ("group" in H)
            return H.options = normalizeOptions(H.options || [], _), H;
          "value" in H && typeof H.value != "string" && Object.assign(H, {
            value: `__mask_${_.count++}`,
            __original: H.value
          });
        }
        return H;
      }
    ) : Object.keys(O).map((H) => ({
      label: O[H],
      value: H
    }));
  }
  function optionValue(O, _, H = !1) {
    if (Array.isArray(O)) {
      for (const V of O)
        if (!(typeof V != "object" && V)) {
          if (isGroupOption(V)) {
            const q = optionValue(V.options, _, !0);
            if (q !== void 0)
              return q;
          } else if (_ == V.value)
            return "__original" in V ? V.__original : V.value;
        }
    }
    return H ? void 0 : _;
  }
  function shouldSelect(O, _) {
    return O === null && _ === void 0 || O === void 0 && _ === null ? !1 : O == _ ? !0 : isPojo(O) && isPojo(_) ? eq$2(O, _) : !1;
  }
  function options(O) {
    O.hook.prop((_, H) => {
      var V;
      return _.prop === "options" && (typeof _.value == "function" ? (O.props.optionsLoader = _.value, _.value = []) : ((V = O.props)._normalizeCounter ?? (V._normalizeCounter = { count: 1 }), _.value = normalizeOptions(_.value, O.props._normalizeCounter))), H(_);
    });
  }
  // @__NO_SIDE_EFFECTS__
  function createSection(O, _, H = !1) {
    return (...V) => {
      const q = (G) => {
        const J = !_ || typeof _ == "string" ? { $el: _ } : _();
        return (isDOM(J) || isComponent(J)) && (J.meta ? J.meta.section = O : J.meta = { section: O }, V.length && !J.children && (J.children = [
          ...V.map(
            (Z) => typeof Z == "function" ? Z(G) : Z
          )
        ]), isDOM(J) && (J.attrs = {
          class: `$classes.${O}`,
          ...J.attrs || {}
        })), {
          if: `$slots.${O}`,
          then: `$slots.${O}`,
          else: O in G ? /* @__PURE__ */ extendSchema(J, G[O]) : J
        };
      };
      return q._s = O, H ? /* @__PURE__ */ createRoot(q) : q;
    };
  }
  // @__NO_SIDE_EFFECTS__
  function createRoot(O) {
    return (_) => [O(_)];
  }
  function isSchemaObject(O) {
    return !!(O && typeof O == "object" && ("$el" in O || "$cmp" in O || "$formkit" in O));
  }
  // @__NO_SIDE_EFFECTS__
  function extendSchema(O, _ = {}) {
    return typeof O == "string" ? isSchemaObject(_) || typeof _ == "string" ? _ : O : Array.isArray(O) ? isSchemaObject(_) ? _ : O : extend$9(O, _);
  }
  var actions = /* @__PURE__ */ createSection("actions", () => ({
    $el: "div",
    if: "$actions"
  })), box = /* @__PURE__ */ createSection("input", () => ({
    $el: "input",
    bind: "$attrs",
    attrs: {
      type: "$type",
      name: "$node.props.altName || $node.name",
      disabled: "$option.attrs.disabled || $disabled",
      onInput: "$handlers.toggleChecked",
      checked: "$fns.eq($_value, $onValue)",
      onBlur: "$handlers.blur",
      value: "$: true",
      id: "$id",
      "aria-describedby": {
        if: "$options.length",
        then: {
          if: "$option.help",
          then: '$: "help-" + $option.attrs.id',
          else: void 0
        },
        else: {
          if: "$help",
          then: '$: "help-" + $id',
          else: void 0
        }
      }
    }
  })), boxHelp = /* @__PURE__ */ createSection("optionHelp", () => ({
    $el: "div",
    if: "$option.help",
    attrs: {
      id: '$: "help-" + $option.attrs.id'
    }
  })), boxInner = /* @__PURE__ */ createSection("inner", "span"), boxLabel = /* @__PURE__ */ createSection("label", "span"), boxOption = /* @__PURE__ */ createSection("option", () => ({
    $el: "li",
    for: ["option", "$options"],
    attrs: {
      "data-disabled": "$option.attrs.disabled || $disabled || undefined"
    }
  })), boxOptions = /* @__PURE__ */ createSection("options", "ul"), boxWrapper = /* @__PURE__ */ createSection("wrapper", () => ({
    $el: "label",
    attrs: {
      "data-disabled": {
        if: "$options.length",
        then: void 0,
        else: "$disabled || undefined"
      },
      "data-checked": {
        if: "$options == undefined",
        then: "$fns.eq($_value, $onValue) || undefined",
        else: "$fns.isChecked($option.value) || undefined"
      }
    }
  })), buttonInput = /* @__PURE__ */ createSection("input", () => ({
    $el: "button",
    bind: "$attrs",
    attrs: {
      type: "$type",
      disabled: "$disabled",
      name: "$node.name",
      id: "$id"
    }
  })), buttonLabel = /* @__PURE__ */ createSection("default", null), decorator = /* @__PURE__ */ createSection("decorator", () => ({
    $el: "span",
    attrs: {
      "aria-hidden": "true"
    }
  })), fieldset$1 = /* @__PURE__ */ createSection("fieldset", () => ({
    $el: "fieldset",
    attrs: {
      id: "$id",
      "aria-describedby": {
        if: "$help",
        then: '$: "help-" + $id',
        else: void 0
      }
    }
  })), fileInput = /* @__PURE__ */ createSection("input", () => ({
    $el: "input",
    bind: "$attrs",
    attrs: {
      type: "file",
      disabled: "$disabled",
      name: "$node.name",
      onChange: "$handlers.files",
      onBlur: "$handlers.blur",
      id: "$id",
      "aria-describedby": "$describedBy",
      "aria-required": "$state.required || undefined"
    }
  })), fileItem = /* @__PURE__ */ createSection("fileItem", () => ({
    $el: "li",
    for: ["file", "$value"]
  })), fileList = /* @__PURE__ */ createSection("fileList", () => ({
    $el: "ul",
    if: "$value.length",
    attrs: {
      "data-has-multiple": "$_hasMultipleFiles"
    }
  })), fileName = /* @__PURE__ */ createSection("fileName", () => ({
    $el: "span",
    attrs: {
      class: "$classes.fileName"
    }
  })), fileRemove = /* @__PURE__ */ createSection("fileRemove", () => ({
    $el: "button",
    attrs: {
      type: "button",
      onClick: "$handlers.resetFiles"
    }
  })), formInput = /* @__PURE__ */ createSection("form", () => ({
    $el: "form",
    bind: "$attrs",
    meta: {
      autoAnimate: !0
    },
    attrs: {
      id: "$id",
      name: "$node.name",
      onSubmit: "$handlers.submit",
      "data-loading": "$state.loading || undefined"
    }
  })), fragment = /* @__PURE__ */ createSection("wrapper", null, !0), help = /* @__PURE__ */ createSection("help", () => ({
    $el: "div",
    if: "$help",
    attrs: {
      id: '$: "help-" + $id'
    }
  })), icon$2 = (O, _) => (/* @__PURE__ */ createSection(`${O}Icon`, () => {
    const H = `_raw${O.charAt(0).toUpperCase()}${O.slice(1)}Icon`;
    return {
      if: `$${O}Icon && $${H}`,
      $el: `${_ || "span"}`,
      attrs: {
        class: `$classes.${O}Icon + " " + $classes.icon`,
        innerHTML: `$${H}`,
        onClick: `$handlers.iconClick(${O})`,
        role: `$fns.iconRole(${O})`,
        tabindex: `$fns.iconRole(${O}) === "button" && "0" || undefined`,
        for: {
          if: `${_ === "label"}`,
          then: "$id"
        }
      }
    };
  }))(), inner$3 = /* @__PURE__ */ createSection("inner", "div"), label$3 = /* @__PURE__ */ createSection("label", () => ({
    $el: "label",
    if: "$label",
    attrs: {
      for: "$id"
    }
  })), legend$1 = /* @__PURE__ */ createSection("legend", () => ({
    $el: "legend",
    if: "$label"
  })), message$1 = /* @__PURE__ */ createSection("message", () => ({
    $el: "li",
    for: ["message", "$messages"],
    attrs: {
      key: "$message.key",
      id: "$id + '-' + $message.key",
      "data-message-type": "$message.type"
    }
  })), messages$1 = /* @__PURE__ */ createSection("messages", () => ({
    $el: "ul",
    if: "$defaultMessagePlacement && $fns.length($messages)"
  })), noFiles = /* @__PURE__ */ createSection("noFiles", () => ({
    $el: "span",
    if: "$value == null || $value.length == 0"
  })), optGroup = /* @__PURE__ */ createSection("optGroup", () => ({
    $el: "optgroup",
    bind: "$option.attrs",
    attrs: {
      label: "$option.group"
    }
  })), option$1 = /* @__PURE__ */ createSection("option", () => ({
    $el: "option",
    bind: "$option.attrs",
    attrs: {
      class: "$classes.option",
      value: "$option.value",
      selected: "$fns.isSelected($option)"
    }
  })), optionSlot = /* @__PURE__ */ createSection("options", () => ({
    $el: null,
    if: "$options.length",
    for: ["option", "$option.options || $options"]
  })), outer = /* @__PURE__ */ createSection("outer", () => ({
    $el: "div",
    meta: {
      autoAnimate: !0
    },
    attrs: {
      key: "$id",
      "data-family": "$family || undefined",
      "data-type": "$type",
      "data-multiple": '$attrs.multiple || ($type != "select" && $options != undefined) || undefined',
      "data-has-multiple": "$_hasMultipleFiles",
      "data-disabled": '$: ($disabled !== "false" && $disabled) || undefined',
      "data-empty": "$state.empty || undefined",
      "data-complete": "$state.complete || undefined",
      "data-invalid": "$state.invalid || undefined",
      "data-errors": "$state.errors || undefined",
      "data-submitted": "$state.submitted || undefined",
      "data-prefix-icon": "$_rawPrefixIcon !== undefined || undefined",
      "data-suffix-icon": "$_rawSuffixIcon !== undefined || undefined",
      "data-prefix-icon-click": "$onPrefixIconClick !== undefined || undefined",
      "data-suffix-icon-click": "$onSuffixIconClick !== undefined || undefined"
    }
  })), prefix = /* @__PURE__ */ createSection("prefix", null), selectInput = /* @__PURE__ */ createSection("input", () => ({
    $el: "select",
    bind: "$attrs",
    attrs: {
      id: "$id",
      "data-placeholder": "$fns.showPlaceholder($_value, $placeholder)",
      disabled: "$disabled",
      class: "$classes.input",
      name: "$node.name",
      onChange: "$handlers.onChange",
      onInput: "$handlers.selectInput",
      onBlur: "$handlers.blur",
      "aria-describedby": "$describedBy",
      "aria-required": "$state.required || undefined"
    }
  })), submitInput = /* @__PURE__ */ createSection("submit", () => ({
    $cmp: "FormKit",
    bind: "$submitAttrs",
    props: {
      type: "submit",
      label: "$submitLabel"
    }
  })), suffix = /* @__PURE__ */ createSection("suffix", null), textInput = /* @__PURE__ */ createSection("input", () => ({
    $el: "input",
    bind: "$attrs",
    attrs: {
      type: "$type",
      disabled: "$disabled",
      name: "$node.name",
      onInput: "$handlers.DOMInput",
      onBlur: "$handlers.blur",
      value: "$_value",
      id: "$id",
      "aria-describedby": "$describedBy",
      "aria-required": "$state.required || undefined"
    }
  })), textareaInput = /* @__PURE__ */ createSection("input", () => ({
    $el: "textarea",
    bind: "$attrs",
    attrs: {
      disabled: "$disabled",
      name: "$node.name",
      onInput: "$handlers.DOMInput",
      onBlur: "$handlers.blur",
      value: "$_value",
      id: "$id",
      "aria-describedby": "$describedBy",
      "aria-required": "$state.required || undefined"
    },
    children: "$initialValue"
  })), wrapper = /* @__PURE__ */ createSection("wrapper", "div"), radioInstance = 0;
  function renamesRadios(O) {
    (O.type === "group" || O.type === "list") && O.plugins.add(renamesRadiosPlugin);
  }
  function renamesRadiosPlugin(O) {
    O.props.type === "radio" && (O.addProps(["altName"]), O.props.altName = `${O.name}_${radioInstance++}`);
  }
  function normalizeBoxes(O) {
    return function(_, H) {
      return _.prop === "options" && Array.isArray(_.value) && (_.value = _.value.map((V) => {
        var q;
        return (q = V.attrs) != null && q.id ? V : extend$9(V, {
          attrs: {
            id: `${O.props.id}-option-${slugify(String(V.value))}`
          }
        });
      }), O.props.type === "checkbox" && !Array.isArray(O.value) && (O.isCreated ? O.input([], !1) : O.on("created", () => {
        Array.isArray(O.value) || O.input([], !1);
      }))), H(_);
    };
  }
  function toggleChecked(O, _) {
    const H = _.target;
    if (H instanceof HTMLInputElement) {
      const V = Array.isArray(O.props.options) ? optionValue(O.props.options, H.value) : H.value;
      Array.isArray(O.props.options) && O.props.options.length ? Array.isArray(O._value) ? O._value.some((q) => shouldSelect(V, q)) ? O.input(
        O._value.filter(
          (q) => !shouldSelect(V, q)
        )
      ) : O.input([...O._value, V]) : O.input([V]) : H.checked ? O.input(O.props.onValue) : O.input(O.props.offValue);
    }
  }
  function isChecked(O, _) {
    var H, V;
    return (H = O.context) == null || H.value, (V = O.context) == null || V._value, Array.isArray(O._value) ? O._value.some(
      (q) => shouldSelect(optionValue(O.props.options, _), q)
    ) : !1;
  }
  function checkboxes(O) {
    O.on("created", () => {
      var _, H;
      (_ = O.context) != null && _.handlers && (O.context.handlers.toggleChecked = toggleChecked.bind(null, O)), (H = O.context) != null && H.fns && (O.context.fns.isChecked = isChecked.bind(null, O)), has$2(O.props, "onValue") || (O.props.onValue = !0), has$2(O.props, "offValue") || (O.props.offValue = !1);
    }), O.hook.prop(normalizeBoxes(O));
  }
  function defaultIcon(O, _) {
    return (H) => {
      H.props[`${O}Icon`] === void 0 && (H.props[`${O}Icon`] = _.startsWith("<svg") ? _ : `default:${_}`);
    };
  }
  function disables(O) {
    O.on("created", () => {
      "disabled" in O.props && (O.props.disabled = undefine(O.props.disabled), O.config.disabled = undefine(O.props.disabled));
    }), O.hook.prop(({ prop: _, value: H }, V) => (H = _ === "disabled" ? undefine(H) : H, V({ prop: _, value: H }))), O.on("prop:disabled", ({ payload: _ }) => {
      O.config.disabled = undefine(_);
    });
  }
  function localize(O, _) {
    return (H) => {
      H.store.set(
        /* @__PURE__ */ createMessage({
          key: O,
          type: "ui",
          value: _ || O,
          meta: {
            localize: !0,
            i18nArgs: [H]
          }
        })
      );
    };
  }
  var isBrowser$1 = typeof window < "u";
  function removeHover(O) {
    O.target instanceof HTMLElement && O.target.hasAttribute("data-file-hover") && O.target.removeAttribute("data-file-hover");
  }
  function preventStrayDrop(O, _) {
    _.target instanceof HTMLInputElement ? O === "dragover" && _.target.setAttribute("data-file-hover", "true") : _.preventDefault(), O === "drop" && removeHover(_);
  }
  function files$2(O) {
    localize("noFiles", "Select file")(O), localize("removeAll", "Remove all")(O), localize("remove")(O), O.addProps(["_hasMultipleFiles"]), isBrowser$1 && (window._FormKit_File_Drop || (window.addEventListener(
      "dragover",
      preventStrayDrop.bind(null, "dragover")
    ), window.addEventListener("drop", preventStrayDrop.bind(null, "drop")), window.addEventListener("dragleave", removeHover), window._FormKit_File_Drop = !0)), O.hook.input((_, H) => H(Array.isArray(_) ? _ : [])), O.on("input", ({ payload: _ }) => {
      O.props._hasMultipleFiles = Array.isArray(_) && _.length > 1 ? !0 : void 0;
    }), O.on("reset", () => {
      if (O.props.id && isBrowser$1) {
        const _ = document.getElementById(O.props.id);
        _ && (_.value = "");
      }
    }), O.on("created", () => {
      Array.isArray(O.value) || O.input([], !1), O.context && (O.context.handlers.resetFiles = (_) => {
        if (_.preventDefault(), O.input([]), O.props.id && isBrowser$1) {
          const H = document.getElementById(O.props.id);
          H && (H.value = ""), H == null || H.focus();
        }
      }, O.context.handlers.files = (_) => {
        var V, q;
        const H = [];
        if (_.target instanceof HTMLInputElement && _.target.files) {
          for (let G = 0; G < _.target.files.length; G++) {
            let J;
            (J = _.target.files.item(G)) && H.push({ name: J.name, file: J });
          }
          O.input(H);
        }
        O.context && (O.context.files = H), typeof ((V = O.props.attrs) == null ? void 0 : V.onChange) == "function" && ((q = O.props.attrs) == null || q.onChange(_));
      });
    });
  }
  var loading = /* @__PURE__ */ createMessage({
    key: "loading",
    value: !0,
    visible: !1
  });
  async function handleSubmit(O, _) {
    const H = Math.random();
    if (O.props._submitNonce = H, _.preventDefault(), await O.settled, O.ledger.value("validating") && (O.store.set(loading), await O.ledger.settled("validating"), O.store.remove("loading"), O.props._submitNonce !== H))
      return;
    const V = (q) => q.store.set(
      /* @__PURE__ */ createMessage({
        key: "submitted",
        value: !0,
        visible: !1
      })
    );
    if (O.walk(V), V(O), O.emit("submit-raw"), typeof O.props.onSubmitRaw == "function" && O.props.onSubmitRaw(_, O), O.ledger.value("blocking"))
      typeof O.props.onSubmitInvalid == "function" && O.props.onSubmitInvalid(O), O.props.incompleteMessage !== !1 && setIncompleteMessage(O);
    else if (typeof O.props.onSubmit == "function") {
      const q = O.props.onSubmit(
        O.hook.submit.dispatch(clone$3(O.value)),
        O
      );
      if (q instanceof Promise) {
        const G = O.props.disabled === void 0 && O.props.submitBehavior !== "live";
        G && (O.props.disabled = !0), O.store.set(loading), await q, G && (O.props.disabled = !1), O.store.remove("loading");
      }
    } else
      _.target instanceof HTMLFormElement && _.target.submit();
  }
  function setIncompleteMessage(O) {
    O.store.set(
      /* @__PURE__ */ createMessage({
        blocking: !1,
        key: "incomplete",
        meta: {
          localize: O.props.incompleteMessage === void 0,
          i18nArgs: [{ node: O }],
          showAsMessage: !0
        },
        type: "ui",
        value: O.props.incompleteMessage || "Form incomplete."
      })
    );
  }
  function form$1(O) {
    var _;
    O.props.isForm = !0, O.ledger.count("validating", (H) => H.key === "validating"), (_ = O.props).submitAttrs ?? (_.submitAttrs = {
      disabled: O.props.disabled
    }), O.on("prop:disabled", ({ payload: H }) => {
      O.props.submitAttrs = { ...O.props.submitAttrs, disabled: H };
    }), O.on("created", () => {
      var H;
      (H = O.context) != null && H.handlers && (O.context.handlers.submit = handleSubmit.bind(null, O)), has$2(O.props, "actions") || (O.props.actions = !0);
    }), O.on("prop:incompleteMessage", () => {
      O.store.incomplete && setIncompleteMessage(O);
    }), O.on("settled:blocking", () => O.store.remove("incomplete"));
  }
  function ignore(O) {
    O.props.ignore === void 0 && (O.props.ignore = !0, O.parent = null);
  }
  function initialValue(O) {
    O.on("created", () => {
      O.context && (O.context.initialValue = O.value || "");
    });
  }
  function casts(O) {
    if (typeof O.props.number > "u")
      return;
    const _ = ["number", "range", "hidden"].includes(O.props.type);
    O.hook.input((H, V) => {
      if (H === "")
        return V(void 0);
      const q = O.props.number === "integer" ? parseInt(H) : parseFloat(H);
      return Number.isFinite(q) ? V(q) : V(_ ? void 0 : H);
    });
  }
  function toggleChecked2(O, _) {
    _.target instanceof HTMLInputElement && O.input(optionValue(O.props.options, _.target.value));
  }
  function isChecked2(O, _) {
    var H, V;
    return (H = O.context) == null || H.value, (V = O.context) == null || V._value, shouldSelect(optionValue(O.props.options, _), O._value);
  }
  function radios(O) {
    O.on("created", () => {
      var _, H;
      Array.isArray(O.props.options) || warn$3(350, {
        node: O,
        inputType: "radio"
      }), (_ = O.context) != null && _.handlers && (O.context.handlers.toggleChecked = toggleChecked2.bind(null, O)), (H = O.context) != null && H.fns && (O.context.fns.isChecked = isChecked2.bind(null, O));
    }), O.hook.prop(normalizeBoxes(O));
  }
  function isSelected(O, _) {
    if (isGroupOption(_))
      return !1;
    O.context && O.context.value;
    const H = "__original" in _ ? _.__original : _.value;
    return Array.isArray(O._value) ? O._value.some((V) => shouldSelect(V, H)) : (O._value === void 0 || O._value === null && !containsValue(O.props.options, null)) && _.attrs && _.attrs["data-is-placeholder"] ? !0 : shouldSelect(H, O._value);
  }
  function containsValue(O, _) {
    return O.some((H) => isGroupOption(H) ? containsValue(H.options, _) : ("__original" in H ? H.__original : H.value) === _);
  }
  async function deferChange(O, _) {
    var H;
    typeof ((H = O.props.attrs) == null ? void 0 : H.onChange) == "function" && (await new Promise((V) => setTimeout(V, 0)), await O.settled, O.props.attrs.onChange(_));
  }
  function selectInput2(O, _) {
    const H = _.target, V = H.hasAttribute("multiple") ? Array.from(H.selectedOptions).map(
      (q) => optionValue(O.props.options, q.value)
    ) : optionValue(O.props.options, H.value);
    O.input(V);
  }
  function applyPlaceholder(O, _) {
    return O.some(
      (H) => H.attrs && H.attrs["data-is-placeholder"]
    ) ? O : [
      {
        label: _,
        value: "",
        attrs: {
          hidden: !0,
          disabled: !0,
          "data-is-placeholder": "true"
        }
      },
      ...O
    ];
  }
  function firstValue(O) {
    const _ = O.length > 0 ? O[0] : void 0;
    if (_)
      return isGroupOption(_) ? firstValue(_.options) : "__original" in _ ? _.__original : _.value;
  }
  function select$2(O) {
    O.on("created", () => {
      var H, V, q;
      const _ = undefine((H = O.props.attrs) == null ? void 0 : H.multiple);
      !_ && O.props.placeholder && Array.isArray(O.props.options) && (O.hook.prop(({ prop: G, value: J }, Z) => (G === "options" && (J = applyPlaceholder(J, O.props.placeholder)), Z({ prop: G, value: J }))), O.props.options = applyPlaceholder(
        O.props.options,
        O.props.placeholder
      )), _ ? O.value === void 0 && O.input([], !1) : O.context && !O.context.options && (O.props.attrs = Object.assign({}, O.props.attrs, {
        value: O._value
      }), O.on("input", ({ payload: G }) => {
        O.props.attrs = Object.assign({}, O.props.attrs, {
          value: G
        });
      })), (V = O.context) != null && V.handlers && (O.context.handlers.selectInput = selectInput2.bind(null, O), O.context.handlers.onChange = deferChange.bind(null, O)), (q = O.context) != null && q.fns && (O.context.fns.isSelected = isSelected.bind(null, O), O.context.fns.showPlaceholder = (G, J) => {
        if (!Array.isArray(O.props.options))
          return !1;
        const Z = O.props.options.some(
          (te) => {
            if (te.attrs && "data-is-placeholder" in te.attrs)
              return !1;
            const ne = "__original" in te ? te.__original : te.value;
            return eq$2(G, ne);
          }
        );
        return J && !Z ? !0 : void 0;
      });
    }), O.hook.input((_, H) => {
      var V, q, G;
      return !O.props.placeholder && _ === void 0 && Array.isArray((V = O.props) == null ? void 0 : V.options) && O.props.options.length && !undefine((G = (q = O.props) == null ? void 0 : q.attrs) == null ? void 0 : G.multiple) && (_ = firstValue(O.props.options)), H(_);
    });
  }
  // @__NO_SIDE_EFFECTS__
  function isSlotCondition(O) {
    return !!(isConditional(O) && O.if && O.if.startsWith("$slots.") && typeof O.then == "string" && O.then.startsWith("$slots.") && "else" in O);
  }
  // @__NO_SIDE_EFFECTS__
  function useSchema(O, _ = {}) {
    const H = /* @__PURE__ */ outer(
      /* @__PURE__ */ wrapper(
        /* @__PURE__ */ label$3("$label"),
        /* @__PURE__ */ inner$3(/* @__PURE__ */ icon$2("prefix"), /* @__PURE__ */ prefix(), O(), /* @__PURE__ */ suffix(), /* @__PURE__ */ icon$2("suffix"))
      ),
      /* @__PURE__ */ help("$help"),
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value"))
    );
    return (V = {}) => H(extend$9(_, V));
  }
  // @__NO_SIDE_EFFECTS__
  function $if(O, _, H) {
    const V = (q) => {
      const G = _(q);
      if (H || isSchemaObject(G) && "if" in G || /* @__PURE__ */ isSlotCondition(G)) {
        const J = {
          if: O,
          then: G
        };
        return H && (J.else = H(q)), J;
      } else /* @__PURE__ */ isSlotCondition(G) ? Object.assign(G.else, { if: O }) : isSchemaObject(G) && Object.assign(G, { if: O });
      return G;
    };
    return V._s = token$2(), V;
  }
  // @__NO_SIDE_EFFECTS__
  function $extend(O, _) {
    const H = (V) => {
      const q = O({});
      return /* @__PURE__ */ isSlotCondition(q) ? (Array.isArray(q.else) || (q.else = /* @__PURE__ */ extendSchema(
        /* @__PURE__ */ extendSchema(q.else, _),
        O._s ? V[O._s] : {}
      )), q) : /* @__PURE__ */ extendSchema(
        /* @__PURE__ */ extendSchema(q, _),
        O._s ? V[O._s] : {}
      );
    };
    return H._s = O._s, H;
  }
  var button$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ outer(
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value")),
      /* @__PURE__ */ wrapper(
        /* @__PURE__ */ buttonInput(
          /* @__PURE__ */ icon$2("prefix"),
          /* @__PURE__ */ prefix(),
          /* @__PURE__ */ buttonLabel("$label || $ui.submit.value"),
          /* @__PURE__ */ suffix(),
          /* @__PURE__ */ icon$2("suffix")
        )
      ),
      /* @__PURE__ */ help("$help")
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * The family of inputs this one belongs too. For example "text" and "email"
     * are both part of the "text" family. This is primary used for styling.
     */
    family: "button",
    /**
     * An array of extra props to accept for this input.
     */
    props: [],
    /**
     * Additional features that should be added to your input
     */
    features: [localize("submit"), ignore],
    /**
     * A key to use for memoizing the schema. This is used to prevent the schema
     * from needing to be stringified when performing a memo lookup.
     */
    schemaMemoKey: "h6st4epl3j8"
  }, checkbox$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ outer(
      /* @__PURE__ */ $if(
        "$options == undefined",
        /**
         * Single checkbox structure.
         */
        /* @__PURE__ */ boxWrapper(
          /* @__PURE__ */ boxInner(/* @__PURE__ */ prefix(), /* @__PURE__ */ box(), /* @__PURE__ */ decorator(/* @__PURE__ */ icon$2("decorator")), /* @__PURE__ */ suffix()),
          /* @__PURE__ */ $extend(/* @__PURE__ */ boxLabel("$label"), {
            if: "$label"
          })
        ),
        /**
         * Multi checkbox structure.
         */
        /* @__PURE__ */ fieldset$1(
          /* @__PURE__ */ legend$1("$label"),
          /* @__PURE__ */ help("$help"),
          /* @__PURE__ */ boxOptions(
            /* @__PURE__ */ boxOption(
              /* @__PURE__ */ boxWrapper(
                /* @__PURE__ */ boxInner(
                  /* @__PURE__ */ prefix(),
                  /* @__PURE__ */ $extend(/* @__PURE__ */ box(), {
                    bind: "$option.attrs",
                    attrs: {
                      id: "$option.attrs.id",
                      value: "$option.value",
                      checked: "$fns.isChecked($option.value)"
                    }
                  }),
                  /* @__PURE__ */ decorator(/* @__PURE__ */ icon$2("decorator")),
                  /* @__PURE__ */ suffix()
                ),
                /* @__PURE__ */ $extend(/* @__PURE__ */ boxLabel("$option.label"), {
                  if: "$option.label"
                })
              ),
              /* @__PURE__ */ boxHelp("$option.help")
            )
          )
        )
      ),
      // Help text only goes under the input when it is a single.
      /* @__PURE__ */ $if("$options == undefined && $help", /* @__PURE__ */ help("$help")),
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value"))
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * The family of inputs this one belongs too. For example "text" and "email"
     * are both part of the "text" family. This is primary used for styling.
     */
    family: "box",
    /**
     * An array of extra props to accept for this input.
     */
    props: ["options", "onValue", "offValue", "optionsLoader"],
    /**
     * Additional features that should be added to your input
     */
    features: [
      options,
      checkboxes,
      defaultIcon("decorator", "checkboxDecorator")
    ],
    /**
     * The key used to memoize the schema.
     */
    schemaMemoKey: "qje02tb3gu8"
  }, file = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ outer(
      /* @__PURE__ */ wrapper(
        /* @__PURE__ */ label$3("$label"),
        /* @__PURE__ */ inner$3(
          /* @__PURE__ */ icon$2("prefix", "label"),
          /* @__PURE__ */ prefix(),
          /* @__PURE__ */ fileInput(),
          /* @__PURE__ */ fileList(
            /* @__PURE__ */ fileItem(
              /* @__PURE__ */ icon$2("fileItem"),
              /* @__PURE__ */ fileName("$file.name"),
              /* @__PURE__ */ $if(
                "$value.length === 1",
                /* @__PURE__ */ fileRemove(
                  /* @__PURE__ */ icon$2("fileRemove"),
                  '$ui.remove.value + " " + $file.name'
                )
              )
            )
          ),
          /* @__PURE__ */ $if("$value.length > 1", /* @__PURE__ */ fileRemove("$ui.removeAll.value")),
          /* @__PURE__ */ noFiles(/* @__PURE__ */ icon$2("noFiles"), "$ui.noFiles.value"),
          /* @__PURE__ */ suffix(),
          /* @__PURE__ */ icon$2("suffix")
        )
      ),
      /* @__PURE__ */ help("$help"),
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value"))
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * The family of inputs this one belongs too. For example "text" and "email"
     * are both part of the "text" family. This is primary used for styling.
     */
    family: "text",
    /**
     * An array of extra props to accept for this input.
     */
    props: [],
    /**
     * Additional features that should be added to your input
     */
    features: [
      files$2,
      defaultIcon("fileItem", "fileItem"),
      defaultIcon("fileRemove", "fileRemove"),
      defaultIcon("noFiles", "noFiles")
    ],
    /**
     * The key used to memoize the schema.
     */
    schemaMemoKey: "9kqc4852fv8"
  }, form2 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ formInput(
      "$slots.default",
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value")),
      /* @__PURE__ */ actions(/* @__PURE__ */ submitInput())
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "group",
    /**
     * An array of extra props to accept for this input.
     */
    props: [
      "actions",
      "submit",
      "submitLabel",
      "submitAttrs",
      "submitBehavior",
      "incompleteMessage"
    ],
    /**
     * Additional features that should be added to your input
     */
    features: [form$1, disables],
    /**
     * The key used to memoize the schema.
     */
    schemaMemoKey: "5bg016redjo"
  }, group$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ fragment("$slots.default"),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "group",
    /**
     * An array of extra props to accept for this input.
     */
    props: [],
    /**
     * Additional features that should be added to your input
     */
    features: [disables, renamesRadios]
  }, hidden$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ textInput(),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * An array of extra props to accept for this input.
     */
    props: [],
    /**
     * Additional features that should be added to your input
     */
    features: [casts]
  }, list$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ fragment("$slots.default"),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "list",
    /**
     * An array of extra props to accept for this input.
     */
    props: ["sync", "dynamic"],
    /**
     * Additional features that should be added to your input
     */
    features: [disables, renamesRadios]
  }, meta$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ fragment(),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * An array of extra props to accept for this input.
     */
    props: [],
    /**
     * Additional features that should be added to your input
     */
    features: []
  }, radio$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ outer(
      /* @__PURE__ */ $if(
        "$options == undefined",
        /**
         * Single radio structure.
         */
        /* @__PURE__ */ boxWrapper(
          /* @__PURE__ */ boxInner(/* @__PURE__ */ prefix(), /* @__PURE__ */ box(), /* @__PURE__ */ decorator(/* @__PURE__ */ icon$2("decorator")), /* @__PURE__ */ suffix()),
          /* @__PURE__ */ $extend(/* @__PURE__ */ boxLabel("$label"), {
            if: "$label"
          })
        ),
        /**
         * Multi radio structure.
         */
        /* @__PURE__ */ fieldset$1(
          /* @__PURE__ */ legend$1("$label"),
          /* @__PURE__ */ help("$help"),
          /* @__PURE__ */ boxOptions(
            /* @__PURE__ */ boxOption(
              /* @__PURE__ */ boxWrapper(
                /* @__PURE__ */ boxInner(
                  /* @__PURE__ */ prefix(),
                  /* @__PURE__ */ $extend(/* @__PURE__ */ box(), {
                    bind: "$option.attrs",
                    attrs: {
                      id: "$option.attrs.id",
                      value: "$option.value",
                      checked: "$fns.isChecked($option.value)"
                    }
                  }),
                  /* @__PURE__ */ decorator(/* @__PURE__ */ icon$2("decorator")),
                  /* @__PURE__ */ suffix()
                ),
                /* @__PURE__ */ $extend(/* @__PURE__ */ boxLabel("$option.label"), {
                  if: "$option.label"
                })
              ),
              /* @__PURE__ */ boxHelp("$option.help")
            )
          )
        )
      ),
      // Help text only goes under the input when it is a single.
      /* @__PURE__ */ $if("$options == undefined && $help", /* @__PURE__ */ help("$help")),
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value"))
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * The family of inputs this one belongs too. For example "text" and "email"
     * are both part of the "text" family. This is primary used for styling.
     */
    family: "box",
    /**
     * An array of extra props to accept for this input.
     */
    props: ["options", "onValue", "offValue", "optionsLoader"],
    /**
     * Additional features that should be added to your input
     */
    features: [options, radios, defaultIcon("decorator", "radioDecorator")],
    /**
     * The key used to memoize the schema.
     */
    schemaMemoKey: "qje02tb3gu8"
  }, select2 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ outer(
      /* @__PURE__ */ wrapper(
        /* @__PURE__ */ label$3("$label"),
        /* @__PURE__ */ inner$3(
          /* @__PURE__ */ icon$2("prefix"),
          /* @__PURE__ */ prefix(),
          /* @__PURE__ */ selectInput(
            /* @__PURE__ */ $if(
              "$slots.default",
              () => "$slots.default",
              /* @__PURE__ */ optionSlot(
                /* @__PURE__ */ $if(
                  "$option.group",
                  /* @__PURE__ */ optGroup(/* @__PURE__ */ optionSlot(/* @__PURE__ */ option$1("$option.label"))),
                  /* @__PURE__ */ option$1("$option.label")
                )
              )
            )
          ),
          /* @__PURE__ */ $if("$attrs.multiple !== undefined", () => "", /* @__PURE__ */ icon$2("select")),
          /* @__PURE__ */ suffix(),
          /* @__PURE__ */ icon$2("suffix")
        )
      ),
      /* @__PURE__ */ help("$help"),
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value"))
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * An array of extra props to accept for this input.
     */
    props: ["options", "placeholder", "optionsLoader"],
    /**
     * Additional features that should be added to your input
     */
    features: [options, select$2, defaultIcon("select", "select")],
    /**
     * The key used to memoize the schema.
     */
    schemaMemoKey: "cb119h43krg"
  }, textarea$1 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ outer(
      /* @__PURE__ */ wrapper(
        /* @__PURE__ */ label$3("$label"),
        /* @__PURE__ */ inner$3(
          /* @__PURE__ */ icon$2("prefix", "label"),
          /* @__PURE__ */ prefix(),
          /* @__PURE__ */ textareaInput(),
          /* @__PURE__ */ suffix(),
          /* @__PURE__ */ icon$2("suffix")
        )
      ),
      /* @__PURE__ */ help("$help"),
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value"))
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * An array of extra props to accept for this input.
     */
    props: [],
    /**
     * Additional features that should be added to your input
     */
    features: [initialValue],
    /**
     * The key used to memoize the schema.
     */
    schemaMemoKey: "b1n0td79m9g"
  }, text$5 = {
    /**
     * The actual schema of the input, or a function that returns the schema.
     */
    schema: /* @__PURE__ */ outer(
      /* @__PURE__ */ wrapper(
        /* @__PURE__ */ label$3("$label"),
        /* @__PURE__ */ inner$3(
          /* @__PURE__ */ icon$2("prefix", "label"),
          /* @__PURE__ */ prefix(),
          /* @__PURE__ */ textInput(),
          /* @__PURE__ */ suffix(),
          /* @__PURE__ */ icon$2("suffix")
        )
      ),
      /* @__PURE__ */ help("$help"),
      /* @__PURE__ */ messages$1(/* @__PURE__ */ message$1("$message.value"))
    ),
    /**
     * The type of node, can be a list, group, or input.
     */
    type: "input",
    /**
     * The family of inputs this one belongs too. For example "text" and "email"
     * are both part of the "text" family. This is primary used for styling.
     */
    family: "text",
    /**
     * An array of extra props to accept for this input.
     */
    props: [],
    /**
     * Additional features that should be added to your input
     */
    features: [casts],
    /**
     * The key used to memoize the schema.
     */
    schemaMemoKey: "c3cc4kflsg"
  }, inputs = {
    button: button$1,
    submit: button$1,
    checkbox: checkbox$1,
    file,
    form: form2,
    group: group$1,
    hidden: hidden$1,
    list: list$1,
    meta: meta$1,
    radio: radio$1,
    select: select2,
    textarea: textarea$1,
    text: text$5,
    color: text$5,
    date: text$5,
    datetimeLocal: text$5,
    email: text$5,
    month: text$5,
    number: text$5,
    password: text$5,
    search: text$5,
    tel: text$5,
    time: text$5,
    url: text$5,
    week: text$5,
    range: text$5
  };
  function generateClasses(O) {
    const _ = {};
    return Object.keys(O).forEach((H) => {
      Object.keys(O[H]).forEach((V) => {
        _[V] ? _[V][H] = O[H][V] : _[V] = {
          [H]: O[H][V]
        };
      });
    }), Object.keys(_).forEach((H) => {
      const V = _[H];
      _[H] = function(q, G) {
        return addClassesBySection(q, G, V);
      };
    }), _;
  }
  function addClassesBySection(O, _, H) {
    const V = O.props.type, q = O.props.family;
    let G = "";
    H.global && (G += H.global + " "), H[`family:${q}`] && (G += H[`family:${q}`] + " "), H[V] && (G += H[V]);
    const J = G.split("$reset");
    return J.length > 1 ? `$reset ${J[J.length - 1].trim()}` : J[0].trim();
  }
  var documentStyles = void 0, documentThemeLinkTag = null, themeDidLoad, themeHasLoaded = !1, themeWasRequested = !1, themeLoaded = /* @__PURE__ */ new Promise((O) => {
    themeDidLoad = () => {
      themeHasLoaded = !0, O();
    };
  }), isClient = typeof window < "u" && typeof fetch < "u";
  documentStyles = isClient ? /* @__PURE__ */ getComputedStyle(document.documentElement) : void 0;
  var iconRegistry = {}, iconRequests = {};
  function createThemePlugin(O, _, H, V) {
    _ && Object.assign(iconRegistry, _), isClient && !themeWasRequested && (documentStyles != null && documentStyles.getPropertyValue("--formkit-theme")) ? (themeDidLoad(), themeWasRequested = !0) : O && !themeWasRequested && isClient ? loadTheme(O) : !themeWasRequested && isClient && themeDidLoad();
    const q = function(J) {
      var Z, te;
      J.addProps(["iconLoader", "iconLoaderUrl"]), J.props.iconHandler = createIconHandler(
        (Z = J.props) != null && Z.iconLoader ? J.props.iconLoader : V,
        (te = J.props) != null && te.iconLoaderUrl ? J.props.iconLoaderUrl : H
      ), loadIconPropIcons(J, J.props.iconHandler), J.on("created", () => {
        var ne, ee;
        (ne = J == null ? void 0 : J.context) != null && ne.handlers && (J.context.handlers.iconClick = (ae) => {
          const oe = `on${ae.charAt(0).toUpperCase()}${ae.slice(1)}IconClick`, se = J.props[oe];
          if (se && typeof se == "function")
            return (ce) => se(J, ce);
        }), (ee = J == null ? void 0 : J.context) != null && ee.fns && (J.context.fns.iconRole = (ae) => {
          const oe = `on${ae.charAt(0).toUpperCase()}${ae.slice(1)}IconClick`;
          return typeof J.props[oe] == "function" ? "button" : null;
        });
      });
    };
    return q.iconHandler = createIconHandler(V, H), q;
  }
  function loadTheme(O) {
    if (!(!O || !isClient || typeof getComputedStyle != "function") && (themeWasRequested = !0, documentThemeLinkTag = document.getElementById("formkit-theme"), O && // if we have a window object
    isClient && // we don't have an existing theme OR the theme being set up is different
    (!(documentStyles != null && documentStyles.getPropertyValue("--formkit-theme")) && !documentThemeLinkTag || documentThemeLinkTag != null && documentThemeLinkTag.getAttribute("data-theme") && (documentThemeLinkTag == null ? void 0 : documentThemeLinkTag.getAttribute("data-theme")) !== O))) {
      const H = `https://cdn.jsdelivr.net/npm/@formkit/themes@${FORMKIT_VERSION.startsWith("__") ? "latest" : FORMKIT_VERSION}/dist/${O}/theme.css`, V = document.createElement("link");
      V.type = "text/css", V.rel = "stylesheet", V.id = "formkit-theme", V.setAttribute("data-theme", O), V.onload = () => {
        documentStyles = getComputedStyle(document.documentElement), themeDidLoad();
      }, document.head.appendChild(V), V.href = H, documentThemeLinkTag && documentThemeLinkTag.remove();
    }
  }
  function createIconHandler(O, _) {
    return (H) => {
      if (typeof H != "string")
        return;
      if (H.startsWith("<svg"))
        return H;
      const V = H.startsWith("default:");
      H = V ? H.split(":")[1] : H;
      const q = H in iconRegistry;
      let G;
      if (q)
        return iconRegistry[H];
      if (!iconRequests[H]) {
        if (G = getIconFromStylesheet(H), G = isClient && typeof G > "u" ? Promise.resolve(G) : G, G instanceof Promise)
          iconRequests[H] = G.then((J) => !J && typeof H == "string" && !V ? G = typeof O == "function" ? O(H) : getRemoteIcon(H, _) : J).then((J) => (typeof H == "string" && (iconRegistry[V ? `default:${H}` : H] = J), J));
        else if (typeof G == "string")
          return iconRegistry[V ? `default:${H}` : H] = G, G;
      }
      return iconRequests[H];
    };
  }
  function getIconFromStylesheet(O) {
    if (isClient)
      return themeHasLoaded ? loadStylesheetIcon(O) : themeLoaded.then(() => loadStylesheetIcon(O));
  }
  function loadStylesheetIcon(O) {
    const _ = documentStyles == null ? void 0 : documentStyles.getPropertyValue(`--fk-icon-${O}`);
    if (_) {
      const H = atob(_);
      if (H.startsWith("<svg"))
        return iconRegistry[O] = H, H;
    }
  }
  function getRemoteIcon(O, _) {
    const H = FORMKIT_VERSION.startsWith("__") ? "latest" : FORMKIT_VERSION, V = typeof _ == "function" ? _(O) : `https://cdn.jsdelivr.net/npm/@formkit/icons@${H}/dist/icons/${O}.svg`;
    if (isClient)
      return fetch(`${V}`).then(async (q) => {
        const G = await q.text();
        if (G.startsWith("<svg"))
          return G;
      }).catch((q) => {
        console.error(q);
      });
  }
  function loadIconPropIcons(O, _) {
    const H = /^[a-zA-Z-]+(?:-icon|Icon)$/;
    Object.keys(O.props).filter((q) => H.test(q)).forEach((q) => loadPropIcon(O, _, q));
  }
  function loadPropIcon(O, _, H) {
    const V = O.props[H], q = _(V), G = `_raw${H.charAt(0).toUpperCase()}${H.slice(1)}`, J = `on${H.charAt(0).toUpperCase()}${H.slice(1)}Click`;
    if (O.addProps([G, J]), O.on(`prop:${H}`, reloadIcon), q instanceof Promise)
      return q.then((Z) => {
        O.props[G] = Z;
      });
    O.props[G] = q;
  }
  function reloadIcon(O) {
    var J;
    const _ = O.origin, H = O.payload, V = (J = _ == null ? void 0 : _.props) == null ? void 0 : J.iconHandler, q = O.name.split(":")[1], G = `_raw${q.charAt(0).toUpperCase()}${q.slice(1)}`;
    if (V && typeof V == "function") {
      const Z = V(H);
      if (Z instanceof Promise)
        return Z.then((te) => {
          _.props[G] = te;
        });
      _.props[G] = Z;
    }
  }
  var errors = {
    /**
     * FormKit errors:
     */
    100: ({ data: O }) => `Only groups, lists, and forms can have children (${O.name}).`,
    101: ({ data: O }) => `You cannot directly modify the store (${O.name}). See: https://formkit.com/advanced/core#message-store`,
    102: ({
      data: [O, _]
    }) => `You cannot directly assign node.${_} (${O.name})`,
    103: ({ data: [O] }) => `Schema expressions cannot start with an operator (${O})`,
    104: ({ data: [O, _] }) => `Schema expressions cannot end with an operator (${O} in "${_}")`,
    105: ({ data: O }) => `Invalid schema expression: ${O}`,
    106: ({ data: O }) => `Cannot submit because (${O}) is not in a form.`,
    107: ({ data: [O, _] }) => `Cannot set ${O.name} to non object value: ${_}`,
    108: ({ data: [O, _] }) => `Cannot set ${O.name} to non array value: ${_}`,
    /**
     * Input specific errors:
     */
    300: ({ data: [O] }) => `Cannot set behavior prop to overscroll (on ${O.name} input) when options prop is a function.`,
    /**
     * FormKit vue errors:
     */
    600: ({ data: O }) => `Unknown input type${typeof O.props.type == "string" ? ' "' + O.props.type + '"' : ""} ("${O.name}")`,
    601: ({ data: O }) => `Input definition${typeof O.props.type == "string" ? ' "' + O.props.type + '"' : ""} is missing a schema or component property (${O.name}).`
  }, warnings = {
    /**
     * Core warnings:
     */
    150: ({ data: O }) => `Schema function "${O}()" is not a valid function.`,
    151: ({ data: O }) => `No form element with id: ${O}`,
    152: ({ data: O }) => `No input element with id: ${O}`,
    /**
     * Input specific warnings:
     */
    350: ({
      data: { node: O, inputType: _ }
    }) => `Invalid options prop for ${O.name} input (${_}). See https://formkit.com/inputs/${_}`,
    /**
     * Vue warnings:
     */
    650: 'Schema "$get()" must use the id of an input to access.',
    651: ({ data: O }) => `Cannot setErrors() on "${O}" because no such id exists.`,
    652: ({ data: O }) => `Cannot clearErrors() on "${O}" because no such id exists.`,
    /**
     * Deprecation warnings:
     */
    800: ({ data: O }) => `${O} is deprecated.`
  }, decodeErrors = (O, _) => {
    if (O.code in errors) {
      const H = errors[O.code];
      O.message = typeof H == "function" ? H(O) : H;
    }
    return _(O);
  }, registered = !1;
  function register() {
    registered || (errorHandler(decodeErrors), warningHandler(decodeWarnings), registered = !0);
  }
  var decodeWarnings = (O, _) => {
    if (O.code in warnings) {
      const H = warnings[O.code];
      O.message = typeof H == "function" ? H(O) : H;
    }
    return _(O);
  }, __defProp$1 = Object.defineProperty, __getOwnPropNames = Object.getOwnPropertyNames, __esm = (O, _) => function() {
    return O && (_ = (0, O[__getOwnPropNames(O)[0]])(O = 0)), _;
  }, __export$1 = (O, _) => {
    for (var H in _)
      __defProp$1(O, H, { get: _[H], enumerable: !0 });
  }, vueBindings, bindings_default, init_bindings = __esm({
    "packages/vue/src/bindings.ts"() {
      vueBindings = function(_) {
        _.ledger.count("blocking", (we) => we.blocking);
        const H = ref$1(!_.ledger.value("blocking"));
        _.ledger.count("errors", (we) => we.type === "error");
        const V = ref$1(!!_.ledger.value("errors"));
        let q = !1;
        nextTick$1(() => {
          q = !0;
        });
        const G = reactive(
          _.store.reduce((we, Ee) => (Ee.visible && (we[Ee.key] = Ee), we), {})
        ), J = ref$1(
          _.props.validationVisibility || (_.props.type === "checkbox" ? "dirty" : "blur")
        );
        _.on("prop:validationVisibility", ({ payload: we }) => {
          J.value = we;
        });
        const Z = ref$1(J.value === "live"), te = ref$1(!1), ne = (we) => {
          te.value = (we ?? []).some(
            (Ee) => Ee.name === "required"
          );
        };
        ne(_.props.parsedRules), _.on("prop:parsedRules", ({ payload: we }) => ne(we));
        const ee = ref$1(_.children.map((we) => we.uid)), ae = computed(() => {
          if (!ge.state)
            return !1;
          if (ge.state.submitted)
            return !0;
          if (!Z.value && !ge.state.settled)
            return !1;
          switch (J.value) {
            case "live":
              return !0;
            case "blur":
              return ge.state.blurred;
            case "dirty":
              return ge.state.dirty;
            default:
              return !1;
          }
        }), oe = computed(() => ge.state.failing && ae.value), se = computed(() => ge && ce.value ? H.value && !V.value : ge.state.dirty && !empty$3(ge.value)), ce = ref$1(
          Array.isArray(_.props.parsedRules) && _.props.parsedRules.length > 0
        );
        _.on("prop:parsedRules", ({ payload: we }) => {
          ce.value = Array.isArray(we) && we.length > 0;
        });
        const le = computed(() => {
          const we = {};
          for (const Ee in G) {
            const xe = G[Ee];
            (xe.type !== "validation" || ae.value) && (we[Ee] = xe);
          }
          return we;
        }), ue = reactive(
          _.store.reduce((we, Ee) => (Ee.type === "ui" && Ee.visible && (we[Ee.key] = Ee), we), {})
        ), de = computed(() => !ge.state.failing), fe = reactive({}), he = new Proxy(fe, {
          get(...we) {
            if (!_)
              return "";
            const [Ee, xe] = we;
            let Le = Reflect.get(...we);
            return !Le && typeof xe == "string" && !has$2(Ee, xe) && !xe.startsWith("__v") && createObserver(_).watch((ke) => {
              const Me = typeof ke.config.rootClasses == "function" ? ke.config.rootClasses(xe, ke) : {}, Re = ke.config.classes ? createClasses(xe, ke, ke.config.classes[xe]) : {}, De = createClasses(
                xe,
                ke,
                ke.props[`_${xe}Class`]
              ), Pe = createClasses(
                xe,
                ke,
                ke.props[`${xe}Class`]
              );
              Le = generateClassList(
                ke,
                xe,
                Me,
                Re,
                De,
                Pe
              ), Ee[xe] = Le ?? "";
            }), Le;
          }
        });
        _.on("prop:rootClasses", () => {
          const we = Object.keys(fe);
          for (const Ee of we)
            delete fe[Ee];
        });
        const me = computed(() => {
          if (!_)
            return;
          const we = [];
          ge.help && we.push(`help-${_.props.id}`);
          for (const Ee in le.value)
            we.push(`${_.props.id}-${Ee}`);
          return we.length ? we.join(" ") : void 0;
        }), pe = ref$1(_.value), ye = ref$1(_.value), ge = reactive({
          _value: ye,
          attrs: _.props.attrs,
          disabled: _.props.disabled,
          describedBy: me,
          fns: {
            length: (we) => Object.keys(we).length,
            number: (we) => Number(we),
            string: (we) => String(we),
            json: (we) => JSON.stringify(we),
            eq: eq$2
          },
          handlers: {
            blur: (we) => {
              _ && (_.store.set(
                /* @__PURE__ */ createMessage({ key: "blurred", visible: !1, value: !0 })
              ), typeof _.props.attrs.onBlur == "function" && _.props.attrs.onBlur(we));
            },
            touch: () => {
              var xe;
              const we = ge.dirtyBehavior === "compare";
              if ((xe = _.store.dirty) != null && xe.value && !we)
                return;
              const Ee = !eq$2(_.props._init, _._value);
              !Ee && !we || _.store.set(
                /* @__PURE__ */ createMessage({ key: "dirty", visible: !1, value: Ee })
              );
            },
            DOMInput: (we) => {
              _.input(we.target.value), _.emit("dom-input-event", we);
            }
          },
          help: _.props.help,
          id: _.props.id,
          items: ee,
          label: _.props.label,
          messages: le,
          didMount: !1,
          node: markRaw(_),
          options: _.props.options,
          defaultMessagePlacement: !0,
          slots: _.props.__slots,
          state: {
            blurred: !1,
            complete: se,
            dirty: !1,
            empty: empty$3(pe),
            submitted: !1,
            settled: _.isSettled,
            valid: H,
            invalid: oe,
            errors: V,
            rules: ce,
            validationVisible: ae,
            required: te,
            failing: !1,
            passing: de
          },
          type: _.props.type,
          family: _.props.family,
          ui: ue,
          value: pe,
          classes: he
        });
        _.on("created", () => {
          eq$2(ge.value, _.value) || (ye.value = _.value, pe.value = _.value, triggerRef(pe), triggerRef(ye)), (async () => (await _.settled, _ && (_.props._init = cloneAny(_.value))))();
        }), _.on("mounted", () => {
          ge.didMount = !0;
        }), _.on("settled", ({ payload: we }) => {
          ge.state.settled = we;
        });
        function ve(we) {
          (Array.isArray(we) ? we : Object.keys(we)).forEach((xe) => {
            xe = camel(xe), has$2(ge, xe) || (ge[xe] = _.props[xe]), _.on(`prop:${xe}`, ({ payload: Le }) => {
              ge[xe] = Le;
            });
          });
        }
        ve((() => {
          const we = [
            "__root",
            "help",
            "label",
            "disabled",
            "options",
            "type",
            "attrs",
            "preserve",
            "preserveErrors",
            "id",
            "dirtyBehavior"
          ], Ee = /^[a-zA-Z-]+(?:-icon|Icon)$/, xe = Object.keys(_.props).filter((Le) => Ee.test(Le));
          return we.concat(xe);
        })());
        function Ce(we) {
          we.props && ve(we.props);
        }
        _.props.definition && Ce(_.props.definition), _.on("added-props", ({ payload: we }) => ve(we)), _.on("input", ({ payload: we }) => {
          _.type !== "input" && !isRef(we) && !isReactive(we) ? ye.value = shallowClone(we) : (ye.value = we, triggerRef(ye));
        }), _.on("commitRaw", ({ payload: we }) => {
          _.type !== "input" && !isRef(we) && !isReactive(we) ? pe.value = ye.value = shallowClone(we) : (pe.value = ye.value = we, triggerRef(pe)), _.emit("modelUpdated");
        }), _.on("commit", ({ payload: we }) => {
          var Ee;
          if ((!ge.state.dirty || ge.dirtyBehavior === "compare") && _.isCreated && q)
            if (!((Ee = _.store.validating) != null && Ee.value))
              ge.handlers.touch();
            else {
              const xe = _.on("message-removed", ({ payload: Le }) => {
                Le.key === "validating" && (ge.handlers.touch(), _.off(xe));
              });
            }
          se && _.type === "input" && V.value && !undefine(_.props.preserveErrors) && _.store.filter(
            (xe) => {
              var Le;
              return !(xe.type === "error" && ((Le = xe.meta) == null ? void 0 : Le.autoClear) === !0);
            }
          ), _.type === "list" && _.sync && (ee.value = _.children.map((xe) => xe.uid)), ge.state.empty = empty$3(we);
        });
        const Se = async (we) => {
          we.type === "ui" && we.visible && !we.meta.showAsMessage ? ue[we.key] = we : we.visible ? G[we.key] = we : we.type === "state" && (ge.state[we.key] = !!we.value);
        };
        _.on("message-added", (we) => Se(we.payload)), _.on("message-updated", (we) => Se(we.payload)), _.on("message-removed", ({ payload: we }) => {
          delete ue[we.key], delete G[we.key], delete ge.state[we.key];
        }), _.on("settled:blocking", () => {
          H.value = !0;
        }), _.on("unsettled:blocking", () => {
          H.value = !1;
        }), _.on("settled:errors", () => {
          V.value = !1;
        }), _.on("unsettled:errors", () => {
          V.value = !0;
        }), watch$1(ae, (we) => {
          we && (Z.value = !0);
        }), _.context = ge, _.emit("context", _, !1), _.on("destroyed", () => {
          _.context = void 0, _ = null;
        });
      }, bindings_default = vueBindings;
    }
  }), defaultConfig_exports = {};
  __export$1(defaultConfig_exports, {
    defaultConfig: () => defaultConfig$3
  });
  var defaultConfig$3, init_defaultConfig = __esm({
    "packages/vue/src/defaultConfig.ts"() {
      init_bindings(), defaultConfig$3 = (O = {}) => {
        register();
        const {
          rules: _ = {},
          locales: H = {},
          inputs: V = {},
          messages: q = {},
          locale: G = void 0,
          theme: J = void 0,
          iconLoaderUrl: Z = void 0,
          iconLoader: te = void 0,
          icons: ne = {},
          ...ee
        } = O, ae = createValidationPlugin({
          ...defaultRules,
          ..._ || {}
        }), oe = createI18nPlugin(
          extend$9({ en, ...H || {} }, q)
        ), se = createLibraryPlugin(inputs, V), ce = createThemePlugin(J, ne, Z, te);
        return extend$9(
          {
            plugins: [se, ce, bindings_default, oe, ae],
            ...G ? { config: { locale: G } } : {}
          },
          ee || {},
          !0
        );
      };
    }
  }), isServer = typeof window > "u", ssrCompleteRegistry = /* @__PURE__ */ new Map();
  function onSSRComplete(O, _) {
    var H;
    !isServer || !O || (ssrCompleteRegistry.has(O) || ssrCompleteRegistry.set(O, /* @__PURE__ */ new Set()), (H = ssrCompleteRegistry.get(O)) == null || H.add(_));
  }
  var isServer2 = typeof window > "u", memo = {}, memoKeys = {}, instanceKey, instanceScopes = /* @__PURE__ */ new WeakMap(), raw = "__raw__", isClassProp = /[a-zA-Z0-9\-][cC]lass$/;
  function getRef(O, _) {
    const H = ref$1(null);
    if (O === "get") {
      const q = {};
      return H.value = get$8.bind(null, q), H;
    }
    const V = O.split(".");
    return watchEffect(() => {
      H.value = getValue$3(
        isRef(_) ? _.value : _,
        V
      );
    }), H;
  }
  function getValue$3(O, _) {
    if (Array.isArray(O)) {
      for (const q of O) {
        const G = q !== !1 && getValue$3(q, _);
        if (G !== void 0)
          return G;
      }
      return;
    }
    let H, V = O;
    for (const q in _) {
      const G = _[q];
      if (typeof V != "object" || V === null) {
        H = void 0;
        break;
      }
      const J = V[G];
      if (Number(q) === _.length - 1 && J !== void 0) {
        H = typeof J == "function" ? J.bind(V) : J;
        break;
      }
      V = J;
    }
    return H;
  }
  function get$8(O, _) {
    if (typeof _ != "string")
      return warn$3(650);
    if (_ in O || (O[_] = ref$1(void 0)), O[_].value === void 0) {
      O[_].value = null;
      const H = getNode(_);
      H && (O[_].value = H.context), watchRegistry(_, ({ payload: V }) => {
        O[_].value = isNode(V) ? V.context : V;
      });
    }
    return O[_].value;
  }
  function parseSchema(O, _, H) {
    function V(se, ce) {
      const le = ae(compile$1(ce.if), { if: !0 }), ue = ne(se, ce.then), de = ce.else ? ne(se, ce.else) : null;
      return [le, ue, de];
    }
    function q(se, ce) {
      var fe, he;
      const le = ae(compile$1(se.if));
      let ue = () => ce, de = () => ce;
      return typeof se.then == "object" ? de = G(se.then, void 0) : typeof se.then == "string" && ((fe = se.then) != null && fe.startsWith("$")) ? de = ae(compile$1(se.then)) : de = () => se.then, has$2(se, "else") && (typeof se.else == "object" ? ue = G(se.else) : typeof se.else == "string" && ((he = se.else) != null && he.startsWith("$")) ? ue = ae(compile$1(se.else)) : ue = () => se.else), () => le() ? de() : ue();
    }
    function G(se, ce, le = {}) {
      const ue = new Set(Object.keys(se || {})), de = ce ? ae(compile$1(ce)) : () => ({}), fe = [
        (he) => {
          const me = de();
          for (const pe in me)
            ue.has(pe) || (he[pe] = me[pe]);
        }
      ];
      if (se) {
        if (isConditional(se))
          return q(
            se,
            le
          );
        for (let he in se) {
          const me = se[he];
          let pe;
          const ye = typeof me == "string";
          he.startsWith(raw) ? (he = he.substring(7), pe = () => me) : ye && me.startsWith("$") && me.length > 1 && !(me.startsWith("$reset") && isClassProp.test(he)) ? pe = ae(compile$1(me)) : typeof me == "object" && isConditional(me) ? pe = q(me, void 0) : typeof me == "object" && isPojo(me) ? pe = G(me) : pe = () => me, fe.push((ge) => {
            ge[he] = pe();
          });
        }
      }
      return () => {
        const he = Array.isArray(se) ? [] : {};
        return fe.forEach((me) => me(he)), he;
      };
    }
    function J(se, ce) {
      let le = null, ue = () => null, de = !1, fe = null, he = null, me = null, pe = !1;
      const ye = sugar(ce);
      if (isDOM(ye) ? (le = ye.$el, ue = ye.$el !== "text" ? G(ye.attrs, ye.bind) : () => null) : isComponent(ye) ? (typeof ye.$cmp == "string" ? has$2(se, ye.$cmp) ? le = se[ye.$cmp] : (le = ye.$cmp, pe = !0) : le = ye.$cmp, ue = G(ye.props, ye.bind)) : isConditional(ye) && ([de, fe, he] = V(se, ye)), !isConditional(ye) && "if" in ye ? de = ae(compile$1(ye.if)) : !isConditional(ye) && le === null && (de = () => !0), "children" in ye && ye.children)
        if (typeof ye.children == "string")
          if (ye.children.startsWith("$slots."))
            le = le === "text" ? "slot" : le, fe = ae(compile$1(ye.children));
          else if (ye.children.startsWith("$") && ye.children.length > 1) {
            const ge = ae(compile$1(ye.children));
            fe = () => String(ge());
          } else
            fe = () => String(ye.children);
        else if (Array.isArray(ye.children))
          fe = ne(se, ye.children);
        else {
          const [ge, ve, be] = V(se, ye.children);
          fe = (Ce) => ge && ge() ? ve && ve(Ce) : be && be(Ce);
        }
      if (isComponent(ye))
        if (fe) {
          const ge = fe;
          fe = (ve) => ({
            default(be, Ce) {
              var Ee, xe, Le, Qe;
              const Se = instanceKey;
              Ce && (instanceKey = Ce), be && ((Ee = instanceScopes.get(instanceKey)) == null || Ee.unshift(be)), ve && ((xe = instanceScopes.get(instanceKey)) == null || xe.unshift(ve));
              const we = ge(ve);
              return be && ((Le = instanceScopes.get(instanceKey)) == null || Le.shift()), ve && ((Qe = instanceScopes.get(instanceKey)) == null || Qe.shift()), instanceKey = Se, we;
            }
          }), fe.slot = !0;
        } else
          fe = () => ({});
      if ("for" in ye && ye.for) {
        const ge = ye.for.length === 3 ? ye.for[2] : ye.for[1];
        me = [
          typeof ge == "string" && ge.startsWith("$") ? ae(compile$1(ge)) : () => ge,
          ye.for[0],
          ye.for.length === 3 ? String(ye.for[1]) : null
        ];
      }
      return [de, le, ue, fe, he, me, pe];
    }
    function Z(se, ce) {
      const le = se(ce), ue = instanceKey;
      return Object.keys(le).reduce((de, fe) => {
        const he = le && le[fe];
        return de[fe] = (me) => he && he(me, ue) || null, de;
      }, {});
    }
    function te(se, ce) {
      const [le, ue, de, fe, he, me, pe] = J(se, ce);
      let ye = (ge) => {
        if (le && ue === null && fe)
          return le() ? fe(ge) : he && he(ge);
        if (ue && (!le || le())) {
          if (ue === "text" && fe)
            return createTextVNode(String(fe()));
          if (ue === "slot" && fe)
            return fe(ge);
          const ve = pe ? resolveComponent(ue) : ue, be = fe != null && fe.slot ? Z(fe, ge) : null;
          return h$3(
            ve,
            de(),
            be || (fe ? fe(ge) : [])
          );
        }
        return typeof he == "function" ? he(ge) : he;
      };
      if (me) {
        const ge = ye, [ve, be, Ce] = me;
        ye = () => {
          const Se = ve(), we = Number.isFinite(Se) ? Array(Number(Se)).fill(0).map((Qe, ke) => ke) : Se, Ee = [];
          if (typeof we != "object")
            return null;
          const xe = instanceScopes.get(instanceKey) || [], Le = Array.isArray(we);
          for (const Qe in we) {
            if (Le && Qe in Array.prototype)
              continue;
            const ke = Object.defineProperty(
              {
                ...xe.reduce(
                  (Me, Re) => Me.__idata ? { ...Me, ...Re } : Re,
                  {}
                ),
                [be]: we[Qe],
                ...Ce !== null ? { [Ce]: Le ? Number(Qe) : Qe } : {}
              },
              "__idata",
              { enumerable: !1, value: !0 }
            );
            xe.unshift(ke), Ee.push(ge.bind(null, ke)()), xe.shift();
          }
          return Ee;
        };
      }
      return ye;
    }
    function ne(se, ce) {
      if (Array.isArray(ce)) {
        const ue = ce.map(te.bind(null, se));
        return (de) => ue.map((fe) => fe(de));
      }
      const le = te(se, ce);
      return (ue) => le(ue);
    }
    const ee = [];
    function ae(se, ce = {}) {
      const le = /* @__PURE__ */ new WeakMap();
      return ee.push((ue, de) => {
        le.set(
          de,
          se.provide((fe) => ue(fe, ce))
        );
      }), () => le.get(instanceKey)();
    }
    function oe(se, ce) {
      H ?? (H = toMemoKey(_));
      const [le, ue] = has$2(memo, H) ? memo[H] : [ne(O, _), ee];
      return isServer2 || (memoKeys[H] ?? (memoKeys[H] = 0), memoKeys[H]++, memo[H] = [le, ue]), ue.forEach((de) => {
        de(se, ce);
      }), () => (instanceKey = ce, le());
    }
    return oe;
  }
  function useScope(O, _) {
    const H = instanceScopes.get(instanceKey) || [];
    let V;
    return H.length && (V = getValue$3(H, O.split("."))), V === void 0 ? _ : V;
  }
  function slotData(O, _) {
    return new Proxy(O, {
      get(...H) {
        let V;
        const q = H[1];
        if (typeof q == "string") {
          const G = instanceKey;
          instanceKey = _, V = useScope(q, void 0), instanceKey = G;
        }
        return V !== void 0 ? V : Reflect.get(...H);
      }
    });
  }
  function createRenderFn(O, _, H) {
    return O(
      (V, q = {}) => V.reduce((G, J) => {
        if (J.startsWith("slots.")) {
          const Z = J.substring(6), te = () => _.slots && has$2(_.slots, Z) && typeof _.slots[Z] == "function";
          if (q.if)
            G[J] = te;
          else if (_.slots) {
            const ne = slotData(_, H);
            G[J] = () => te() ? _.slots[Z](ne) : null;
          }
        } else {
          const Z = getRef(J, _);
          G[J] = () => useScope(J, Z.value);
        }
        return G;
      }, {}),
      H
    );
  }
  function clean(O, _, H) {
    if (_ ?? (_ = toMemoKey(O)), memoKeys[_]--, memoKeys[_] === 0) {
      delete memoKeys[_];
      const [, V] = memo[_];
      delete memo[_], V.length = 0;
    }
    instanceScopes.delete(H);
  }
  function toMemoKey(O) {
    return JSON.stringify(O, (_, H) => typeof H == "function" ? H.toString() : H);
  }
  var FormKitSchema = /* @__PURE__ */ defineComponent({
    name: "FormKitSchema",
    props: {
      schema: {
        type: [Array, Object],
        required: !0
      },
      data: {
        type: Object,
        default: () => ({})
      },
      library: {
        type: Object,
        default: () => ({})
      },
      memoKey: {
        type: String,
        required: !1
      }
    },
    emits: ["mounted"],
    setup(O, _) {
      var ne;
      const H = getCurrentInstance();
      let V = {};
      instanceScopes.set(V, []);
      const q = { FormKit: markRaw(FormKit_default), ...O.library };
      let G = parseSchema(q, O.schema, O.memoKey), J, Z;
      isServer2 || watch$1(
        () => O.schema,
        (ee, ae) => {
          var se;
          const oe = V;
          V = {}, instanceScopes.set(V, []), G = parseSchema(q, O.schema, O.memoKey), J = createRenderFn(G, Z, V), ee === ae && ((se = H == null ? void 0 : H.proxy) == null ? void 0 : se.$forceUpdate).call(se), clean(O.schema, O.memoKey, oe);
        },
        { deep: !0 }
      ), watchEffect(() => {
        Z = Object.assign(reactive(O.data ?? {}), {
          slots: _.slots
        }), _.slots, J = createRenderFn(G, Z, V);
      });
      function te() {
        clean(O.schema, O.memoKey, V), Z && (Z.node && Z.node.destroy(), Z.slots = null, Z = null), J = null;
      }
      return onMounted(() => _.emit("mounted")), onUnmounted(te), onSSRComplete((ne = getCurrentInstance()) == null ? void 0 : ne.appContext.app, te), () => J ? J() : null;
    }
  }), FormKitSchema_default = FormKitSchema, isServer3 = typeof window > "u", parentSymbol = Symbol("FormKitParent"), componentSymbol = Symbol("FormKitComponentCallback");
  function FormKit(O, _) {
    const H = useInput(O, _);
    if (H.props.definition || error$2(600, H), H.props.definition.component)
      return () => {
        var ne;
        return h$3(
          (ne = H.props.definition) == null ? void 0 : ne.component,
          {
            context: H.context
          },
          { ..._.slots }
        );
      };
    const V = ref$1([]);
    let q = H.props.definition.schemaMemoKey;
    const G = () => {
      var ee, ae;
      const ne = (ae = (ee = H.props) == null ? void 0 : ee.definition) == null ? void 0 : ae.schema;
      ne || error$2(601, H), typeof ne == "function" ? (V.value = ne({ ...O.sectionsSchema || {} }), (q && O.sectionsSchema || "memoKey" in ne && typeof ne.memoKey == "string") && (q = (q ?? (ne == null ? void 0 : ne.memoKey)) + JSON.stringify(O.sectionsSchema))) : V.value = ne;
    };
    G(), isServer3 || H.on("schema", () => {
      q += "♻️", G();
    }), _.emit("node", H);
    const J = H.props.definition.library, Z = {
      FormKit: markRaw(formkitComponent),
      ...J,
      ...O.library ?? {}
    };
    function te() {
      H.emit("mounted");
    }
    return _.expose({ node: H }), () => h$3(
      FormKitSchema,
      {
        schema: V.value,
        data: H.context,
        onMounted: te,
        library: Z,
        memoKey: q
      },
      { ..._.slots }
    );
  }
  var formkitComponent = /* @__PURE__ */ defineComponent(
    FormKit,
    {
      props: runtimeProps,
      inheritAttrs: !1
    }
  ), FormKit_default = formkitComponent, rootSymbol = Symbol();
  function createPlugin(O, _) {
    return O.component(_.alias || "FormKit", FormKit_default).component(_.schemaAlias || "FormKitSchema", FormKitSchema_default), {
      get: getNode,
      setLocale: (H) => {
        var V;
        (V = _.config) != null && V.rootConfig && (_.config.rootConfig.locale = H);
      },
      clearErrors: clearErrors2,
      setErrors: setErrors2,
      submit: submitForm,
      reset: reset$2
    };
  }
  var optionsSymbol = Symbol.for("FormKitOptions"), configSymbol = Symbol.for("FormKitConfig"), plugin = {
    install(O, _) {
      const H = Object.assign(
        {
          alias: "FormKit",
          schemaAlias: "FormKitSchema"
        },
        typeof _ == "function" ? _() : _
      ), V = createConfig(H.config || {});
      H.config = { rootConfig: V }, O.config.globalProperties.$formkit = createPlugin(O, H), O.provide(optionsSymbol, H), O.provide(configSymbol, V), typeof window < "u" && (globalThis.__FORMKIT_CONFIGS__ = (globalThis.__FORMKIT_CONFIGS__ || []).concat([V]));
    }
  }, isBrowser = typeof window < "u", pseudoProps = [
    // Boolean props
    "ignore",
    "disabled",
    "preserve",
    // String props
    "help",
    "label",
    /^preserve(-e|E)rrors/,
    /^[a-z]+(?:-visibility|Visibility|-behavior|Behavior)$/,
    /^[a-zA-Z-]+(?:-class|Class)$/,
    "prefixIcon",
    "suffixIcon",
    /^[a-zA-Z-]+(?:-icon|Icon)$/
  ], boolProps = ["disabled", "ignore", "preserve"];
  function classesToNodeProps(O, _) {
    _.classes && Object.keys(_.classes).forEach(
      (H) => {
        typeof H == "string" && (O.props[`_${H}Class`] = _.classes[H], isObject$6(_.classes[H]) && H === "inner" && Object.values(_.classes[H]));
      }
    );
  }
  function onlyListeners(O) {
    return O ? ["Submit", "SubmitRaw", "SubmitInvalid"].reduce(
      (H, V) => {
        const q = `on${V}`;
        return q in O && typeof O[q] == "function" && (H[q] = O[q]), H;
      },
      {}
    ) : {};
  }
  function useInput(O, _, H = {}) {
    const V = Object.assign({}, inject(optionsSymbol) || {}, H), q = inject(rootSymbol, ref$1(isBrowser ? document : void 0)), G = inject(componentSymbol, () => {
    }), J = getCurrentInstance(), Z = onlyListeners(J == null ? void 0 : J.vnode.props), te = ["modelValue", "model-value"].some(
      (ye) => ye in ((J == null ? void 0 : J.vnode.props) ?? {})
    );
    let ne = !1;
    onMounted(() => {
      ne = !0;
    });
    const ee = O.modelValue !== void 0 ? O.modelValue : cloneAny(_.attrs.value);
    function ae() {
      const ye = {
        ...nodeProps(O),
        ...Z,
        type: O.type ?? "text",
        __root: q.value,
        __slots: _.slots
      }, ge = except(nodeProps(_.attrs), pseudoProps);
      ge.key || (ge.key = token$2()), ye.attrs = ge;
      const ve = only(nodeProps(_.attrs), pseudoProps);
      for (const Ce in ve)
        boolProps.includes(Ce) && ve[Ce] === "" && (ve[Ce] = !0), ye[camel(Ce)] = ve[Ce];
      const be = { props: {} };
      return classesToNodeProps(be, O), Object.assign(ye, be.props), typeof ye.type != "string" && (ye.definition = ye.type, delete ye.type), ye;
    }
    const oe = ae(), se = oe.ignore ? null : O.parent || inject(parentSymbol, null), ce = createNode(
      extend$9(
        V || {},
        {
          name: O.name || void 0,
          value: ee,
          parent: se,
          plugins: (V.plugins || []).concat(O.plugins ?? []),
          config: O.config || {},
          props: oe,
          index: O.index,
          sync: !!undefine(_.attrs.sync || _.attrs.dynamic)
        },
        !1,
        !0
      )
    );
    G(ce), ce.props.definition || error$2(600, ce);
    const le = ref$1(
      new Set(
        Array.isArray(ce.props.__propDefs) ? ce.props.__propDefs : Object.keys(ce.props.__propDefs ?? {})
      )
    );
    ce.on(
      "added-props",
      ({ payload: ye }) => {
        (Array.isArray(ye) ? ye : Object.keys(ye ?? {})).forEach((ve) => le.value.add(ve));
      }
    );
    const ue = computed(
      () => pseudoProps.concat([...le.value]).reduce((ye, ge) => (typeof ge == "string" ? (ye.push(camel(ge)), ye.push(kebab(ge))) : ye.push(ge), ye), [])
    );
    watchEffect(() => classesToNodeProps(ce, O));
    const de = nodeProps(O);
    for (const ye in de)
      watch$1(
        () => O[ye],
        () => {
          O[ye] !== void 0 && (ce.props[ye] = O[ye]);
        }
      );
    watchEffect(() => {
      ce.props.__root = q.value;
    });
    const fe = /* @__PURE__ */ new Set(), he = nodeProps(_.attrs);
    watchEffect(() => {
      me(only(he, ue.value));
    });
    function me(ye) {
      fe.forEach((ge) => {
        ge(), fe.delete(ge);
      });
      for (const ge in ye) {
        const ve = camel(ge);
        fe.add(
          watch$1(
            () => _.attrs[ge],
            () => {
              ce.props[ve] = _.attrs[ge];
            }
          )
        );
      }
    }
    if (watchEffect(() => {
      const ye = except(nodeProps(_.attrs), ue.value);
      "multiple" in ye && (ye.multiple = undefine(ye.multiple)), typeof ye.onBlur == "function" && (ye.onBlur = oncePerTick(ye.onBlur)), ce.props.attrs = Object.assign({}, ce.props.attrs || {}, ye);
    }), watchEffect(() => {
      const ye = (O.errors ?? []).map(
        (ge) => /* @__PURE__ */ createMessage({
          key: slugify(ge),
          type: "error",
          value: ge,
          meta: { source: "prop" }
        })
      );
      ce.store.apply(
        ye,
        (ge) => ge.type === "error" && ge.meta.source === "prop"
      );
    }), ce.type !== "input") {
      const ye = `${ce.name}-prop`;
      watchEffect(() => {
        const ge = O.inputErrors ?? {}, ve = Object.keys(ge);
        ve.length || ce.clearErrors(!0, ye);
        const be = ve.reduce((Ce, Se) => {
          let we = ge[Se];
          return typeof we == "string" && (we = [we]), Array.isArray(we) && (Ce[Se] = we.map(
            (Ee) => /* @__PURE__ */ createMessage({
              key: Ee,
              type: "error",
              value: Ee,
              meta: { source: ye }
            })
          )), Ce;
        }, {});
        ce.store.apply(
          be,
          (Ce) => Ce.type === "error" && Ce.meta.source === ye
        );
      });
    }
    watchEffect(() => Object.assign(ce.config, O.config)), ce.type !== "input" && provide(parentSymbol, ce);
    let pe;
    return ce.on("modelUpdated", () => {
      var ye, ge;
      _.emit("inputRaw", (ye = ce.context) == null ? void 0 : ye.value, ce), ne && _.emit("input", (ge = ce.context) == null ? void 0 : ge.value, ce), te && ce.context && (pe = cloneAny(ce.value), _.emit("update:modelValue", shallowClone(ce.value)));
    }), te && (watch$1(
      toRef(O, "modelValue"),
      (ye) => {
        eq$2(pe, ye) || ce.input(ye, !1);
      },
      { deep: !0 }
    ), ce.value !== ee && ce.emit("modelUpdated")), onBeforeUnmount(() => ce.destroy()), ce;
  }
  var totalCreated = 1;
  function isComponent2(O) {
    return typeof O == "function" && O.length === 2 || typeof O == "object" && !Array.isArray(O) && !("$el" in O) && !("$cmp" in O) && !("if" in O);
  }
  function createInput(O, _ = {}, H = {}) {
    const V = {
      type: "input",
      ..._
    };
    let q;
    if (isComponent2(O)) {
      const G = `SchemaComponent${totalCreated++}`;
      q = /* @__PURE__ */ createSection("input", () => ({
        $cmp: G,
        props: {
          context: "$node.context"
        }
      })), V.library = { [G]: markRaw(O) };
    } else typeof O == "function" ? q = O : q = /* @__PURE__ */ createSection("input", () => cloneAny(O));
    return V.schema = /* @__PURE__ */ useSchema(q || "Schema undefined", H), V.schemaMemoKey || (V.schemaMemoKey = `${Math.random()}`), V;
  }
  var messages = /* @__PURE__ */ createSection("messages", () => ({
    $el: "ul",
    if: "$fns.length($messages)"
  })), message = /* @__PURE__ */ createSection("message", () => ({
    $el: "li",
    for: ["message", "$messages"],
    attrs: {
      key: "$message.key",
      id: "$id + '-' + $message.key",
      "data-message-type": "$message.type"
    }
  })), definition$2 = messages(message("$message.value")), FormKitMessages = /* @__PURE__ */ defineComponent({
    props: {
      node: {
        type: Object,
        required: !1
      },
      sectionsSchema: {
        type: Object,
        default: {}
      },
      defaultPosition: {
        type: [String, Boolean],
        default: !1
      },
      library: {
        type: Object,
        default: () => ({})
      }
    },
    setup(O, _) {
      const H = computed(() => O.node || inject(parentSymbol, void 0));
      watch$1(
        H,
        () => {
          var G;
          (G = H.value) != null && G.context && !undefine(O.defaultPosition) && (H.value.context.defaultMessagePlacement = !1);
        },
        { immediate: !0 }
      );
      const V = definition$2(O.sectionsSchema || {}), q = computed(() => {
        var G, J, Z, te, ne, ee;
        return {
          messages: ((J = (G = H.value) == null ? void 0 : G.context) == null ? void 0 : J.messages) || {},
          fns: ((te = (Z = H.value) == null ? void 0 : Z.context) == null ? void 0 : te.fns) || {},
          classes: ((ee = (ne = H.value) == null ? void 0 : ne.context) == null ? void 0 : ee.classes) || {}
        };
      });
      return () => {
        var G;
        return (G = H.value) != null && G.context ? h$3(
          FormKitSchema_default,
          { schema: V, data: q.value, library: O.library },
          { ..._.slots }
        ) : null;
      };
    }
  }), summary = /* @__PURE__ */ createSection("summary", () => ({
    $el: "div",
    attrs: {
      "aria-live": "polite"
    }
  })), summaryInner = /* @__PURE__ */ createSection("summaryInner", () => ({
    $el: "div",
    if: "$summaries.length && $showSummaries"
  })), messages2 = /* @__PURE__ */ createSection("messages", () => ({
    $el: "ul",
    if: "$summaries.length && $showSummaries"
  })), message2 = /* @__PURE__ */ createSection("message", () => ({
    $el: "li",
    for: ["summary", "$summaries"],
    attrs: {
      key: "$summary.key",
      "data-message-type": "$summary.type"
    }
  })), summaryHeader = /* @__PURE__ */ createSection("summaryHeader", () => ({
    $el: "h2",
    attrs: {
      id: "$id"
    }
  })), messageLink = /* @__PURE__ */ createSection("messageLink", () => ({
    $el: "a",
    attrs: {
      id: "$summary.key",
      href: '$: "#" + $summary.id',
      onClick: "$jumpLink"
    }
  }));
  summary(
    summaryInner(
      summaryHeader("$summaryHeader"),
      messages2(message2(messageLink("$summary.message")))
    )
  );
  init_defaultConfig();
  init_bindings();
  /*!
   * pinia v2.3.0
   * (c) 2024 Eduardo San Martin Morote
   * @license MIT
   */
  let activePinia;
  const setActivePinia = (O) => activePinia = O, piniaSymbol = (
    /* istanbul ignore next */
    Symbol()
  );
  function isPlainObject$2(O) {
    return O && typeof O == "object" && Object.prototype.toString.call(O) === "[object Object]" && typeof O.toJSON != "function";
  }
  var MutationType;
  (function(O) {
    O.direct = "direct", O.patchObject = "patch object", O.patchFunction = "patch function";
  })(MutationType || (MutationType = {}));
  function createPinia() {
    const O = effectScope(!0), _ = O.run(() => ref$1({}));
    let H = [], V = [];
    const q = markRaw({
      install(G) {
        setActivePinia(q), q._a = G, G.provide(piniaSymbol, q), G.config.globalProperties.$pinia = q, V.forEach((J) => H.push(J)), V = [];
      },
      use(G) {
        return this._a ? H.push(G) : V.push(G), this;
      },
      _p: H,
      // it's actually undefined here
      // @ts-expect-error
      _a: null,
      _e: O,
      _s: /* @__PURE__ */ new Map(),
      state: _
    });
    return q;
  }
  const noop$6 = () => {
  };
  function addSubscription(O, _, H, V = noop$6) {
    O.push(_);
    const q = () => {
      const G = O.indexOf(_);
      G > -1 && (O.splice(G, 1), V());
    };
    return !H && getCurrentScope() && onScopeDispose(q), q;
  }
  function triggerSubscriptions(O, ..._) {
    O.slice().forEach((H) => {
      H(..._);
    });
  }
  const fallbackRunWithContext = (O) => O(), ACTION_MARKER = Symbol(), ACTION_NAME = Symbol();
  function mergeReactiveObjects(O, _) {
    O instanceof Map && _ instanceof Map ? _.forEach((H, V) => O.set(V, H)) : O instanceof Set && _ instanceof Set && _.forEach(O.add, O);
    for (const H in _) {
      if (!_.hasOwnProperty(H))
        continue;
      const V = _[H], q = O[H];
      isPlainObject$2(q) && isPlainObject$2(V) && O.hasOwnProperty(H) && !isRef(V) && !isReactive(V) ? O[H] = mergeReactiveObjects(q, V) : O[H] = V;
    }
    return O;
  }
  const skipHydrateSymbol = (
    /* istanbul ignore next */
    Symbol()
  );
  function shouldHydrate(O) {
    return !isPlainObject$2(O) || !O.hasOwnProperty(skipHydrateSymbol);
  }
  const { assign: assign$1 } = Object;
  function isComputed(O) {
    return !!(isRef(O) && O.effect);
  }
  function createOptionsStore(O, _, H, V) {
    const { state: q, actions: G, getters: J } = _, Z = H.state.value[O];
    let te;
    function ne() {
      Z || (H.state.value[O] = q ? q() : {});
      const ee = toRefs(H.state.value[O]);
      return assign$1(ee, G, Object.keys(J || {}).reduce((ae, oe) => (ae[oe] = markRaw(computed(() => {
        setActivePinia(H);
        const se = H._s.get(O);
        return J[oe].call(se, se);
      })), ae), {}));
    }
    return te = createSetupStore(O, ne, _, H, V, !0), te;
  }
  function createSetupStore(O, _, H = {}, V, q, G) {
    let J;
    const Z = assign$1({ actions: {} }, H), te = { deep: !0 };
    let ne, ee, ae = [], oe = [], se;
    const ce = V.state.value[O];
    !G && !ce && (V.state.value[O] = {}), ref$1({});
    let le;
    function ue(ve) {
      let be;
      ne = ee = !1, typeof ve == "function" ? (ve(V.state.value[O]), be = {
        type: MutationType.patchFunction,
        storeId: O,
        events: se
      }) : (mergeReactiveObjects(V.state.value[O], ve), be = {
        type: MutationType.patchObject,
        payload: ve,
        storeId: O,
        events: se
      });
      const Ce = le = Symbol();
      nextTick$1().then(() => {
        le === Ce && (ne = !0);
      }), ee = !0, triggerSubscriptions(ae, be, V.state.value[O]);
    }
    const de = G ? function() {
      const { state: be } = H, Ce = be ? be() : {};
      this.$patch((Se) => {
        assign$1(Se, Ce);
      });
    } : (
      /* istanbul ignore next */
      noop$6
    );
    function fe() {
      J.stop(), ae = [], oe = [], V._s.delete(O);
    }
    const he = (ve, be = "") => {
      if (ACTION_MARKER in ve)
        return ve[ACTION_NAME] = be, ve;
      const Ce = function() {
        setActivePinia(V);
        const Se = Array.from(arguments), we = [], Ee = [];
        function xe(ke) {
          we.push(ke);
        }
        function Le(ke) {
          Ee.push(ke);
        }
        triggerSubscriptions(oe, {
          args: Se,
          name: Ce[ACTION_NAME],
          store: pe,
          after: xe,
          onError: Le
        });
        let Qe;
        try {
          Qe = ve.apply(this && this.$id === O ? this : pe, Se);
        } catch (ke) {
          throw triggerSubscriptions(Ee, ke), ke;
        }
        return Qe instanceof Promise ? Qe.then((ke) => (triggerSubscriptions(we, ke), ke)).catch((ke) => (triggerSubscriptions(Ee, ke), Promise.reject(ke))) : (triggerSubscriptions(we, Qe), Qe);
      };
      return Ce[ACTION_MARKER] = !0, Ce[ACTION_NAME] = be, Ce;
    }, me = {
      _p: V,
      // _s: scope,
      $id: O,
      $onAction: addSubscription.bind(null, oe),
      $patch: ue,
      $reset: de,
      $subscribe(ve, be = {}) {
        const Ce = addSubscription(ae, ve, be.detached, () => Se()), Se = J.run(() => watch$1(() => V.state.value[O], (we) => {
          (be.flush === "sync" ? ee : ne) && ve({
            storeId: O,
            type: MutationType.direct,
            events: se
          }, we);
        }, assign$1({}, te, be)));
        return Ce;
      },
      $dispose: fe
    }, pe = reactive(me);
    V._s.set(O, pe);
    const ge = (V._a && V._a.runWithContext || fallbackRunWithContext)(() => V._e.run(() => (J = effectScope()).run(() => _({ action: he }))));
    for (const ve in ge) {
      const be = ge[ve];
      if (isRef(be) && !isComputed(be) || isReactive(be))
        G || (ce && shouldHydrate(be) && (isRef(be) ? be.value = ce[ve] : mergeReactiveObjects(be, ce[ve])), V.state.value[O][ve] = be);
      else if (typeof be == "function") {
        const Ce = he(be, ve);
        ge[ve] = Ce, Z.actions[ve] = be;
      }
    }
    return assign$1(pe, ge), assign$1(toRaw(pe), ge), Object.defineProperty(pe, "$state", {
      get: () => V.state.value[O],
      set: (ve) => {
        ue((be) => {
          assign$1(be, ve);
        });
      }
    }), V._p.forEach((ve) => {
      assign$1(pe, J.run(() => ve({
        store: pe,
        app: V._a,
        pinia: V,
        options: Z
      })));
    }), ce && G && H.hydrate && H.hydrate(pe.$state, ce), ne = !0, ee = !0, pe;
  }
  /*! #__NO_SIDE_EFFECTS__ */
  // @__NO_SIDE_EFFECTS__
  function defineStore(O, _, H) {
    let V, q;
    const G = typeof _ == "function";
    typeof O == "string" ? (V = O, q = G ? H : _) : (q = O, V = O.id);
    function J(Z, te) {
      const ne = hasInjectionContext();
      return Z = // in test mode, ignore the argument provided as we can always retrieve a
      // pinia instance with getActivePinia()
      Z || (ne ? inject(piniaSymbol, null) : null), Z && setActivePinia(Z), Z = activePinia, Z._s.has(V) || (G ? createSetupStore(V, _, q, Z) : createOptionsStore(V, q, Z)), Z._s.get(V);
    }
    return J.$id = V, J;
  }
  function storeToRefs(O) {
    {
      const _ = toRaw(O), H = {};
      for (const V in _) {
        const q = _[V];
        q.effect ? H[V] = // ...
        computed({
          get: () => O[V],
          set(G) {
            O[V] = G;
          }
        }) : (isRef(q) || isReactive(q)) && (H[V] = // ---
        toRef(O, V));
      }
      return H;
    }
  }
  const _hoisted_1$w = ["innerHTML"], _sfc_main$S = /* @__PURE__ */ defineComponent({
    __name: "Table.ce",
    props: {
      html: { type: String },
      singleBlockEmbed: { type: Boolean },
      class: { type: String }
    },
    setup(O) {
      const _ = O, H = (V) => {
        if (V !== null) {
          const q = V.querySelector(".dataframe");
          q && q.removeAttribute("border");
        }
      };
      return (V, q) => (openBlock(), createElementBlock("div", {
        ref: H,
        class: normalizeClass(["w-full", { "h-full absolute top-0": _.singleBlockEmbed }]),
        innerHTML: _.html,
        "data-cy": "block-shadow"
      }, null, 10, _hoisted_1$w));
    }
  }), _style_0 = "table{min-width:100%;border-collapse:collapse;table-layout:auto;text-align:center}thead{color:#111827;font-weight:600;border-bottom:1px solid rgb(209,213,219)}thead th{vertical-align:bottom}tr{text-align:center!important}tbody tr{border-bottom:1px solid rgb(229,231,235)}tbody tr:last-child{border-bottom:0}tbody td{vertical-align:top;padding:.5rem 1.5rem}", _export_sfc = (O, _) => {
    const H = O.__vccOpts || O;
    for (const [V, q] of _)
      H[V] = q;
    return H;
  }, TableBlock$1 = /* @__PURE__ */ _export_sfc(_sfc_main$S, [["styles", [_style_0]]]);
  function isEmpty$1(O) {
    return O === 0 ? !1 : Array.isArray(O) && O.length === 0 ? !0 : !O;
  }
  function not(O) {
    return (..._) => !O(..._);
  }
  function includes$1(O, _) {
    return O === void 0 && (O = "undefined"), O === null && (O = "null"), O === !1 && (O = "false"), O.toString().toLowerCase().indexOf(_.trim()) !== -1;
  }
  function filterOptions(O, _, H, V) {
    return _ ? O.filter((q) => includes$1(V(q, H), _)).sort((q, G) => V(q, H).length - V(G, H).length) : O;
  }
  function stripGroups(O) {
    return O.filter((_) => !_.$isLabel);
  }
  function flattenOptions(O, _) {
    return (H) => H.reduce((V, q) => q[O] && q[O].length ? (V.push({
      $groupLabel: q[_],
      $isLabel: !0
    }), V.concat(q[O])) : V, []);
  }
  function filterGroups(O, _, H, V, q) {
    return (G) => G.map((J) => {
      if (!J[H])
        return console.warn("Options passed to vue-multiselect do not contain groups, despite the config."), [];
      const Z = filterOptions(J[H], O, _, q);
      return Z.length ? {
        [V]: J[V],
        [H]: Z
      } : [];
    });
  }
  const flow = (...O) => (_) => O.reduce((H, V) => V(H), _);
  var multiselectMixin = {
    data() {
      return {
        search: "",
        isOpen: !1,
        preferredOpenDirection: "below",
        optimizedHeight: this.maxHeight
      };
    },
    props: {
      /**
       * Decide whether to filter the results based on search query.
       * Useful for async filtering, where we search through more complex data.
       * @type {Boolean}
       */
      internalSearch: {
        type: Boolean,
        default: !0
      },
      /**
       * Array of available options: Objects, Strings or Integers.
       * If array of objects, visible label will default to option.label.
       * If `labal` prop is passed, label will equal option['label']
       * @type {Array}
       */
      options: {
        type: Array,
        required: !0
      },
      /**
       * Equivalent to the `multiple` attribute on a `<select>` input.
       * @default false
       * @type {Boolean}
       */
      multiple: {
        type: Boolean,
        default: !1
      },
      /**
       * Key to compare objects
       * @default 'id'
       * @type {String}
       */
      trackBy: {
        type: String
      },
      /**
       * Label to look for in option Object
       * @default 'label'
       * @type {String}
       */
      label: {
        type: String
      },
      /**
       * Enable/disable search in options
       * @default true
       * @type {Boolean}
       */
      searchable: {
        type: Boolean,
        default: !0
      },
      /**
       * Clear the search input after `)
       * @default true
       * @type {Boolean}
       */
      clearOnSelect: {
        type: Boolean,
        default: !0
      },
      /**
       * Hide already selected options
       * @default false
       * @type {Boolean}
       */
      hideSelected: {
        type: Boolean,
        default: !1
      },
      /**
       * Equivalent to the `placeholder` attribute on a `<select>` input.
       * @default 'Select option'
       * @type {String}
       */
      placeholder: {
        type: String,
        default: "Select option"
      },
      /**
       * Allow to remove all selected values
       * @default true
       * @type {Boolean}
       */
      allowEmpty: {
        type: Boolean,
        default: !0
      },
      /**
       * Reset this.internalValue, this.search after this.internalValue changes.
       * Useful if want to create a stateless dropdown.
       * @default false
       * @type {Boolean}
       */
      resetAfter: {
        type: Boolean,
        default: !1
      },
      /**
       * Enable/disable closing after selecting an option
       * @default true
       * @type {Boolean}
       */
      closeOnSelect: {
        type: Boolean,
        default: !0
      },
      /**
       * Function to interpolate the custom label
       * @default false
       * @type {Function}
       */
      customLabel: {
        type: Function,
        default(O, _) {
          return isEmpty$1(O) ? "" : _ ? O[_] : O;
        }
      },
      /**
       * Disable / Enable tagging
       * @default false
       * @type {Boolean}
       */
      taggable: {
        type: Boolean,
        default: !1
      },
      /**
       * String to show when highlighting a potential tag
       * @default 'Press enter to create a tag'
       * @type {String}
      */
      tagPlaceholder: {
        type: String,
        default: "Press enter to create a tag"
      },
      /**
       * By default new tags will appear above the search results.
       * Changing to 'bottom' will revert this behaviour
       * and will proritize the search results
       * @default 'top'
       * @type {String}
      */
      tagPosition: {
        type: String,
        default: "top"
      },
      /**
       * Number of allowed selected options. No limit if 0.
       * @default 0
       * @type {Number}
      */
      max: {
        type: [Number, Boolean],
        default: !1
      },
      /**
       * Will be passed with all events as second param.
       * Useful for identifying events origin.
       * @default null
       * @type {String|Integer}
      */
      id: {
        default: null
      },
      /**
       * Limits the options displayed in the dropdown
       * to the first X options.
       * @default 1000
       * @type {Integer}
      */
      optionsLimit: {
        type: Number,
        default: 1e3
      },
      /**
       * Name of the property containing
       * the group values
       * @default 1000
       * @type {String}
      */
      groupValues: {
        type: String
      },
      /**
       * Name of the property containing
       * the group label
       * @default 1000
       * @type {String}
      */
      groupLabel: {
        type: String
      },
      /**
       * Allow to select all group values
       * by selecting the group label
       * @default false
       * @type {Boolean}
       */
      groupSelect: {
        type: Boolean,
        default: !1
      },
      /**
       * Array of keyboard keys to block
       * when selecting
       * @default 1000
       * @type {String}
      */
      blockKeys: {
        type: Array,
        default() {
          return [];
        }
      },
      /**
       * Prevent from wiping up the search value
       * @default false
       * @type {Boolean}
      */
      preserveSearch: {
        type: Boolean,
        default: !1
      },
      /**
       * Select 1st options if value is empty
       * @default false
       * @type {Boolean}
      */
      preselectFirst: {
        type: Boolean,
        default: !1
      },
      /**
       * Prevent autofocus
       * @default false
       * @type {Boolean}
      */
      preventAutofocus: {
        type: Boolean,
        default: !1
      }
    },
    mounted() {
      !this.multiple && this.max && console.warn("[Vue-Multiselect warn]: Max prop should not be used when prop Multiple equals false."), this.preselectFirst && !this.internalValue.length && this.options.length && this.select(this.filteredOptions[0]);
    },
    computed: {
      internalValue() {
        return this.modelValue || this.modelValue === 0 ? Array.isArray(this.modelValue) ? this.modelValue : [this.modelValue] : [];
      },
      filteredOptions() {
        const O = this.search || "", _ = O.toLowerCase().trim();
        let H = this.options.concat();
        return this.internalSearch ? H = this.groupValues ? this.filterAndFlat(H, _, this.label) : filterOptions(H, _, this.label, this.customLabel) : H = this.groupValues ? flattenOptions(this.groupValues, this.groupLabel)(H) : H, H = this.hideSelected ? H.filter(not(this.isSelected)) : H, this.taggable && _.length && !this.isExistingOption(_) && (this.tagPosition === "bottom" ? H.push({ isTag: !0, label: O }) : H.unshift({ isTag: !0, label: O })), H.slice(0, this.optionsLimit);
      },
      valueKeys() {
        return this.trackBy ? this.internalValue.map((O) => O[this.trackBy]) : this.internalValue;
      },
      optionKeys() {
        return (this.groupValues ? this.flatAndStrip(this.options) : this.options).map((_) => this.customLabel(_, this.label).toString().toLowerCase());
      },
      currentOptionLabel() {
        return this.multiple ? this.searchable ? "" : this.placeholder : this.internalValue.length ? this.getOptionLabel(this.internalValue[0]) : this.searchable ? "" : this.placeholder;
      }
    },
    watch: {
      internalValue: {
        handler() {
          this.resetAfter && this.internalValue.length && (this.search = "", this.$emit("update:modelValue", this.multiple ? [] : null));
        },
        deep: !0
      },
      search() {
        this.$emit("search-change", this.search);
      }
    },
    emits: ["open", "search-change", "close", "select", "update:modelValue", "remove", "tag"],
    methods: {
      /**
       * Returns the internalValue in a way it can be emited to the parent
       * @returns {Object||Array||String||Integer}
       */
      getValue() {
        return this.multiple ? this.internalValue : this.internalValue.length === 0 ? null : this.internalValue[0];
      },
      /**
       * Filters and then flattens the options list
       * @param  {Array}
       * @return {Array} returns a filtered and flat options list
       */
      filterAndFlat(O, _, H) {
        return flow(
          filterGroups(_, H, this.groupValues, this.groupLabel, this.customLabel),
          flattenOptions(this.groupValues, this.groupLabel)
        )(O);
      },
      /**
       * Flattens and then strips the group labels from the options list
       * @param  {Array}
       * @return {Array} returns a flat options list without group labels
       */
      flatAndStrip(O) {
        return flow(
          flattenOptions(this.groupValues, this.groupLabel),
          stripGroups
        )(O);
      },
      /**
       * Updates the search value
       * @param  {String}
       */
      updateSearch(O) {
        this.search = O;
      },
      /**
       * Finds out if the given query is already present
       * in the available options
       * @param  {String}
       * @return {Boolean} returns true if element is available
       */
      isExistingOption(O) {
        return this.options ? this.optionKeys.indexOf(O) > -1 : !1;
      },
      /**
       * Finds out if the given element is already present
       * in the result value
       * @param  {Object||String||Integer} option passed element to check
       * @returns {Boolean} returns true if element is selected
       */
      isSelected(O) {
        const _ = this.trackBy ? O[this.trackBy] : O;
        return this.valueKeys.indexOf(_) > -1;
      },
      /**
       * Finds out if the given option is disabled
       * @param  {Object||String||Integer} option passed element to check
       * @returns {Boolean} returns true if element is disabled
       */
      isOptionDisabled(O) {
        return !!O.$isDisabled;
      },
      /**
       * Returns empty string when options is null/undefined
       * Returns tag query if option is tag.
       * Returns the customLabel() results and casts it to string.
       *
       * @param  {Object||String||Integer} Passed option
       * @returns {Object||String}
       */
      getOptionLabel(O) {
        if (isEmpty$1(O)) return "";
        if (O.isTag) return O.label;
        if (O.$isLabel) return O.$groupLabel;
        const _ = this.customLabel(O, this.label);
        return isEmpty$1(_) ? "" : _;
      },
      /**
       * Add the given option to the list of selected options
       * or sets the option as the selected option.
       * If option is already selected -> remove it from the results.
       *
       * @param  {Object||String||Integer} option to select/deselect
       * @param  {Boolean} block removing
       */
      select(O, _) {
        if (O.$isLabel && this.groupSelect) {
          this.selectGroup(O);
          return;
        }
        if (!(this.blockKeys.indexOf(_) !== -1 || this.disabled || O.$isDisabled || O.$isLabel) && !(this.max && this.multiple && this.internalValue.length === this.max) && !(_ === "Tab" && !this.pointerDirty)) {
          if (O.isTag)
            this.$emit("tag", O.label, this.id), this.search = "", this.closeOnSelect && !this.multiple && this.deactivate();
          else {
            if (this.isSelected(O)) {
              _ !== "Tab" && this.removeElement(O);
              return;
            }
            this.multiple ? this.$emit("update:modelValue", this.internalValue.concat([O])) : this.$emit("update:modelValue", O), this.$emit("select", O, this.id), this.clearOnSelect && (this.search = "");
          }
          this.closeOnSelect && this.deactivate();
        }
      },
      /**
       * Add the given group options to the list of selected options
       * If all group optiona are already selected -> remove it from the results.
       *
       * @param  {Object||String||Integer} group to select/deselect
       */
      selectGroup(O) {
        const _ = this.options.find((H) => H[this.groupLabel] === O.$groupLabel);
        if (_) {
          if (this.wholeGroupSelected(_)) {
            this.$emit("remove", _[this.groupValues], this.id);
            const H = this.trackBy ? _[this.groupValues].map((q) => q[this.trackBy]) : _[this.groupValues], V = this.internalValue.filter(
              (q) => H.indexOf(this.trackBy ? q[this.trackBy] : q) === -1
            );
            this.$emit("update:modelValue", V);
          } else {
            let H = _[this.groupValues].filter(
              (V) => !(this.isOptionDisabled(V) || this.isSelected(V))
            );
            this.max && H.splice(this.max - this.internalValue.length), this.$emit("select", H, this.id), this.$emit(
              "update:modelValue",
              this.internalValue.concat(H)
            );
          }
          this.closeOnSelect && this.deactivate();
        }
      },
      /**
       * Helper to identify if all values in a group are selected
       *
       * @param {Object} group to validated selected values against
       */
      wholeGroupSelected(O) {
        return O[this.groupValues].every(
          (_) => this.isSelected(_) || this.isOptionDisabled(_)
        );
      },
      /**
       * Helper to identify if all values in a group are disabled
       *
       * @param {Object} group to check for disabled values
       */
      wholeGroupDisabled(O) {
        return O[this.groupValues].every(this.isOptionDisabled);
      },
      /**
       * Removes the given option from the selected options.
       * Additionally checks this.allowEmpty prop if option can be removed when
       * it is the last selected option.
       *
       * @param  {type} option description
       * @return {type}        description
       */
      removeElement(O, _ = !0) {
        if (this.disabled || O.$isDisabled) return;
        if (!this.allowEmpty && this.internalValue.length <= 1) {
          this.deactivate();
          return;
        }
        const H = typeof O == "object" ? this.valueKeys.indexOf(O[this.trackBy]) : this.valueKeys.indexOf(O);
        if (this.multiple) {
          const V = this.internalValue.slice(0, H).concat(this.internalValue.slice(H + 1));
          this.$emit("update:modelValue", V);
        } else
          this.$emit("update:modelValue", null);
        this.$emit("remove", O, this.id), this.closeOnSelect && _ && this.deactivate();
      },
      /**
       * Calls this.removeElement() with the last element
       * from this.internalValue (selected element Array)
       *
       * @fires this#removeElement
       */
      removeLastElement() {
        this.blockKeys.indexOf("Delete") === -1 && this.search.length === 0 && Array.isArray(this.internalValue) && this.internalValue.length && this.removeElement(this.internalValue[this.internalValue.length - 1], !1);
      },
      /**
       * Opens the multiselect’s dropdown.
       * Sets this.isOpen to TRUE
       */
      activate() {
        this.isOpen || this.disabled || (this.adjustPosition(), this.groupValues && this.pointer === 0 && this.filteredOptions.length && (this.pointer = 1), this.isOpen = !0, this.searchable ? (this.preserveSearch || (this.search = ""), this.preventAutofocus || this.$nextTick(() => this.$refs.search && this.$refs.search.focus())) : this.preventAutofocus || typeof this.$el < "u" && this.$el.focus(), this.$emit("open", this.id));
      },
      /**
       * Closes the multiselect’s dropdown.
       * Sets this.isOpen to FALSE
       */
      deactivate() {
        this.isOpen && (this.isOpen = !1, this.searchable ? this.$refs.search !== null && typeof this.$refs.search < "u" && this.$refs.search.blur() : typeof this.$el < "u" && this.$el.blur(), this.preserveSearch || (this.search = ""), this.$emit("close", this.getValue(), this.id));
      },
      /**
       * Call this.activate() or this.deactivate()
       * depending on this.isOpen value.
       *
       * @fires this#activate || this#deactivate
       * @property {Boolean} isOpen indicates if dropdown is open
       */
      toggle() {
        this.isOpen ? this.deactivate() : this.activate();
      },
      /**
       * Updates the hasEnoughSpace variable used for
       * detecting where to expand the dropdown
       */
      adjustPosition() {
        if (typeof window > "u") return;
        const O = this.$el.getBoundingClientRect().top, _ = window.innerHeight - this.$el.getBoundingClientRect().bottom;
        _ > this.maxHeight || _ > O || this.openDirection === "below" || this.openDirection === "bottom" ? (this.preferredOpenDirection = "below", this.optimizedHeight = Math.min(_ - 40, this.maxHeight)) : (this.preferredOpenDirection = "above", this.optimizedHeight = Math.min(O - 40, this.maxHeight));
      }
    }
  }, pointerMixin = {
    data() {
      return {
        pointer: 0,
        pointerDirty: !1
      };
    },
    props: {
      /**
       * Enable/disable highlighting of the pointed value.
       * @type {Boolean}
       * @default true
       */
      showPointer: {
        type: Boolean,
        default: !0
      },
      optionHeight: {
        type: Number,
        default: 40
      }
    },
    computed: {
      pointerPosition() {
        return this.pointer * this.optionHeight;
      },
      visibleElements() {
        return this.optimizedHeight / this.optionHeight;
      }
    },
    watch: {
      filteredOptions() {
        this.pointerAdjust();
      },
      isOpen() {
        this.pointerDirty = !1;
      },
      pointer() {
        this.$refs.search && this.$refs.search.setAttribute("aria-activedescendant", this.id + "-" + this.pointer.toString());
      }
    },
    methods: {
      optionHighlight(O, _) {
        return {
          "multiselect__option--highlight": O === this.pointer && this.showPointer,
          "multiselect__option--selected": this.isSelected(_)
        };
      },
      groupHighlight(O, _) {
        if (!this.groupSelect)
          return [
            "multiselect__option--disabled",
            { "multiselect__option--group": _.$isLabel }
          ];
        const H = this.options.find((V) => V[this.groupLabel] === _.$groupLabel);
        return H && !this.wholeGroupDisabled(H) ? [
          "multiselect__option--group",
          { "multiselect__option--highlight": O === this.pointer && this.showPointer },
          { "multiselect__option--group-selected": this.wholeGroupSelected(H) }
        ] : "multiselect__option--disabled";
      },
      addPointerElement({ key: O } = "Enter") {
        this.filteredOptions.length > 0 && this.select(this.filteredOptions[this.pointer], O), this.pointerReset();
      },
      pointerForward() {
        this.pointer < this.filteredOptions.length - 1 && (this.pointer++, this.$refs.list.scrollTop <= this.pointerPosition - (this.visibleElements - 1) * this.optionHeight && (this.$refs.list.scrollTop = this.pointerPosition - (this.visibleElements - 1) * this.optionHeight), this.filteredOptions[this.pointer] && this.filteredOptions[this.pointer].$isLabel && !this.groupSelect && this.pointerForward()), this.pointerDirty = !0;
      },
      pointerBackward() {
        this.pointer > 0 ? (this.pointer--, this.$refs.list.scrollTop >= this.pointerPosition && (this.$refs.list.scrollTop = this.pointerPosition), this.filteredOptions[this.pointer] && this.filteredOptions[this.pointer].$isLabel && !this.groupSelect && this.pointerBackward()) : this.filteredOptions[this.pointer] && this.filteredOptions[0].$isLabel && !this.groupSelect && this.pointerForward(), this.pointerDirty = !0;
      },
      pointerReset() {
        this.closeOnSelect && (this.pointer = 0, this.$refs.list && (this.$refs.list.scrollTop = 0));
      },
      pointerAdjust() {
        this.pointer >= this.filteredOptions.length - 1 && (this.pointer = this.filteredOptions.length ? this.filteredOptions.length - 1 : 0), this.filteredOptions.length > 0 && this.filteredOptions[this.pointer].$isLabel && !this.groupSelect && this.pointerForward();
      },
      pointerSet(O) {
        this.pointer = O, this.pointerDirty = !0;
      }
    }
  }, script$2 = {
    name: "vue-multiselect",
    mixins: [multiselectMixin, pointerMixin],
    compatConfig: {
      MODE: 3,
      ATTR_ENUMERATED_COERCION: !1
    },
    props: {
      /**
         * name attribute to match optional label element
         * @default ''
         * @type {String}
         */
      name: {
        type: String,
        default: ""
      },
      /**
         * Presets the selected options value.
         * @type {Object||Array||String||Integer}
         */
      modelValue: {
        type: null,
        default() {
          return [];
        }
      },
      /**
         * String to show when pointing to an option
         * @default 'Press enter to select'
         * @type {String}
         */
      selectLabel: {
        type: String,
        default: "Press enter to select"
      },
      /**
         * String to show when pointing to an option
         * @default 'Press enter to select'
         * @type {String}
         */
      selectGroupLabel: {
        type: String,
        default: "Press enter to select group"
      },
      /**
         * String to show next to selected option
         * @default 'Selected'
         * @type {String}
         */
      selectedLabel: {
        type: String,
        default: "Selected"
      },
      /**
         * String to show when pointing to an already selected option
         * @default 'Press enter to remove'
         * @type {String}
         */
      deselectLabel: {
        type: String,
        default: "Press enter to remove"
      },
      /**
         * String to show when pointing to an already selected option
         * @default 'Press enter to remove'
         * @type {String}
         */
      deselectGroupLabel: {
        type: String,
        default: "Press enter to deselect group"
      },
      /**
         * Decide whether to show pointer labels
         * @default true
         * @type {Boolean}
         */
      showLabels: {
        type: Boolean,
        default: !0
      },
      /**
         * Limit the display of selected options. The rest will be hidden within the limitText string.
         * @default 99999
         * @type {Integer}
         */
      limit: {
        type: Number,
        default: 99999
      },
      /**
         * Sets maxHeight style value of the dropdown
         * @default 300
         * @type {Integer}
         */
      maxHeight: {
        type: Number,
        default: 300
      },
      /**
         * Function that process the message shown when selected
         * elements pass the defined limit.
         * @default 'and * more'
         * @param {Int} count Number of elements more than limit
         * @type {Function}
         */
      limitText: {
        type: Function,
        default: (O) => `and ${O} more`
      },
      /**
         * Set true to trigger the loading spinner.
         * @default False
         * @type {Boolean}
         */
      loading: {
        type: Boolean,
        default: !1
      },
      /**
         * Disables the multiselect if true.
         * @default false
         * @type {Boolean}
         */
      disabled: {
        type: Boolean,
        default: !1
      },
      /**
       * Enables search input's spellcheck if true.
       * @default false
       * @type {Boolean}
       */
      spellcheck: {
        type: Boolean,
        default: !1
      },
      /**
         * Fixed opening direction
         * @default ''
         * @type {String}
         */
      openDirection: {
        type: String,
        default: ""
      },
      /**
         * Shows slot with message about empty options
         * @default true
         * @type {Boolean}
         */
      showNoOptions: {
        type: Boolean,
        default: !0
      },
      showNoResults: {
        type: Boolean,
        default: !0
      },
      tabindex: {
        type: Number,
        default: 0
      },
      required: {
        type: Boolean,
        default: !1
      }
    },
    computed: {
      hasOptionGroup() {
        return this.groupValues && this.groupLabel && this.groupSelect;
      },
      isSingleLabelVisible() {
        return (this.singleValue || this.singleValue === 0) && (!this.isOpen || !this.searchable) && !this.visibleValues.length;
      },
      isPlaceholderVisible() {
        return !this.internalValue.length && (!this.searchable || !this.isOpen);
      },
      visibleValues() {
        return this.multiple ? this.internalValue.slice(0, this.limit) : [];
      },
      singleValue() {
        return this.internalValue[0];
      },
      deselectLabelText() {
        return this.showLabels ? this.deselectLabel : "";
      },
      deselectGroupLabelText() {
        return this.showLabels ? this.deselectGroupLabel : "";
      },
      selectLabelText() {
        return this.showLabels ? this.selectLabel : "";
      },
      selectGroupLabelText() {
        return this.showLabels ? this.selectGroupLabel : "";
      },
      selectedLabelText() {
        return this.showLabels ? this.selectedLabel : "";
      },
      inputStyle() {
        return this.searchable || this.multiple && this.modelValue && this.modelValue.length ? this.isOpen ? { width: "100%" } : { width: "0", position: "absolute", padding: "0" } : "";
      },
      contentStyle() {
        return this.options.length ? { display: "inline-block" } : { display: "block" };
      },
      isAbove() {
        return this.openDirection === "above" || this.openDirection === "top" ? !0 : this.openDirection === "below" || this.openDirection === "bottom" ? !1 : this.preferredOpenDirection === "above";
      },
      showSearchInput() {
        return this.searchable && (this.hasSingleSelectedSlot && (this.visibleSingleValue || this.visibleSingleValue === 0) ? this.isOpen : !0);
      }
    }
  };
  const _hoisted_1$v = {
    ref: "tags",
    class: "multiselect__tags"
  }, _hoisted_2$e = { class: "multiselect__tags-wrap" }, _hoisted_3$c = { class: "multiselect__spinner" }, _hoisted_4$a = { key: 0 }, _hoisted_5$6 = { class: "multiselect__option" }, _hoisted_6$2 = { class: "multiselect__option" }, _hoisted_7$2 = /* @__PURE__ */ createTextVNode("No elements found. Consider changing the search query."), _hoisted_8$2 = { class: "multiselect__option" }, _hoisted_9$1 = /* @__PURE__ */ createTextVNode("List is empty.");
  function render$4(O, _, H, V, q, G) {
    return openBlock(), createBlock("div", {
      tabindex: O.searchable ? -1 : H.tabindex,
      class: [{ "multiselect--active": O.isOpen, "multiselect--disabled": H.disabled, "multiselect--above": G.isAbove, "multiselect--has-options-group": G.hasOptionGroup }, "multiselect"],
      onFocus: _[14] || (_[14] = (J) => O.activate()),
      onBlur: _[15] || (_[15] = (J) => O.searchable ? !1 : O.deactivate()),
      onKeydown: [
        _[16] || (_[16] = withKeys(withModifiers((J) => O.pointerForward(), ["self", "prevent"]), ["down"])),
        _[17] || (_[17] = withKeys(withModifiers((J) => O.pointerBackward(), ["self", "prevent"]), ["up"]))
      ],
      onKeypress: _[18] || (_[18] = withKeys(withModifiers((J) => O.addPointerElement(J), ["stop", "self"]), ["enter", "tab"])),
      onKeyup: _[19] || (_[19] = withKeys((J) => O.deactivate(), ["esc"])),
      role: "combobox",
      "aria-owns": "listbox-" + O.id
    }, [
      renderSlot(O.$slots, "caret", { toggle: O.toggle }, () => [
        createVNode(
          "div",
          {
            onMousedown: _[1] || (_[1] = withModifiers((J) => O.toggle(), ["prevent", "stop"])),
            class: "multiselect__select"
          },
          null,
          32
          /* HYDRATE_EVENTS */
        )
      ]),
      renderSlot(O.$slots, "clear", { search: O.search }),
      createVNode(
        "div",
        _hoisted_1$v,
        [
          renderSlot(O.$slots, "selection", {
            search: O.search,
            remove: O.removeElement,
            values: G.visibleValues,
            isOpen: O.isOpen
          }, () => [
            withDirectives(createVNode(
              "div",
              _hoisted_2$e,
              [
                (openBlock(!0), createBlock(
                  Fragment,
                  null,
                  renderList(G.visibleValues, (J, Z) => renderSlot(O.$slots, "tag", {
                    option: J,
                    search: O.search,
                    remove: O.removeElement
                  }, () => [
                    (openBlock(), createBlock("span", {
                      class: "multiselect__tag",
                      key: Z
                    }, [
                      createVNode("span", {
                        textContent: toDisplayString(O.getOptionLabel(J))
                      }, null, 8, ["textContent"]),
                      createVNode("i", {
                        tabindex: "1",
                        onKeypress: withKeys(withModifiers((te) => O.removeElement(J), ["prevent"]), ["enter"]),
                        onMousedown: withModifiers((te) => O.removeElement(J), ["prevent"]),
                        class: "multiselect__tag-icon"
                      }, null, 40, ["onKeypress", "onMousedown"])
                    ]))
                  ])),
                  256
                  /* UNKEYED_FRAGMENT */
                ))
              ],
              512
              /* NEED_PATCH */
            ), [
              [vShow, G.visibleValues.length > 0]
            ]),
            O.internalValue && O.internalValue.length > H.limit ? renderSlot(O.$slots, "limit", { key: 0 }, () => [
              createVNode("strong", {
                class: "multiselect__strong",
                textContent: toDisplayString(H.limitText(O.internalValue.length - H.limit))
              }, null, 8, ["textContent"])
            ]) : createCommentVNode("v-if", !0)
          ]),
          createVNode(Transition, { name: "multiselect__loading" }, {
            default: withCtx(() => [
              renderSlot(O.$slots, "loading", {}, () => [
                withDirectives(createVNode(
                  "div",
                  _hoisted_3$c,
                  null,
                  512
                  /* NEED_PATCH */
                ), [
                  [vShow, H.loading]
                ])
              ])
            ]),
            _: 3
            /* FORWARDED */
          }),
          O.searchable ? (openBlock(), createBlock("input", {
            key: 0,
            ref: "search",
            name: H.name,
            id: O.id,
            type: "text",
            autocomplete: "off",
            spellcheck: H.spellcheck,
            placeholder: O.placeholder,
            required: H.required,
            style: G.inputStyle,
            value: O.search,
            disabled: H.disabled,
            tabindex: H.tabindex,
            onInput: _[2] || (_[2] = (J) => O.updateSearch(J.target.value)),
            onFocus: _[3] || (_[3] = withModifiers((J) => O.activate(), ["prevent"])),
            onBlur: _[4] || (_[4] = withModifiers((J) => O.deactivate(), ["prevent"])),
            onKeyup: _[5] || (_[5] = withKeys((J) => O.deactivate(), ["esc"])),
            onKeydown: [
              _[6] || (_[6] = withKeys(withModifiers((J) => O.pointerForward(), ["prevent"]), ["down"])),
              _[7] || (_[7] = withKeys(withModifiers((J) => O.pointerBackward(), ["prevent"]), ["up"])),
              _[9] || (_[9] = withKeys(withModifiers((J) => O.removeLastElement(), ["stop"]), ["delete"]))
            ],
            onKeypress: _[8] || (_[8] = withKeys(withModifiers((J) => O.addPointerElement(J), ["prevent", "stop", "self"]), ["enter"])),
            class: "multiselect__input",
            "aria-controls": "listbox-" + O.id
          }, null, 44, ["name", "id", "spellcheck", "placeholder", "required", "value", "disabled", "tabindex", "aria-controls"])) : createCommentVNode("v-if", !0),
          G.isSingleLabelVisible ? (openBlock(), createBlock(
            "span",
            {
              key: 1,
              class: "multiselect__single",
              onMousedown: _[10] || (_[10] = withModifiers((...J) => O.toggle && O.toggle(...J), ["prevent"]))
            },
            [
              renderSlot(O.$slots, "singleLabel", { option: G.singleValue }, () => [
                createTextVNode(
                  toDisplayString(O.currentOptionLabel),
                  1
                  /* TEXT */
                )
              ])
            ],
            32
            /* HYDRATE_EVENTS */
          )) : createCommentVNode("v-if", !0),
          G.isPlaceholderVisible ? (openBlock(), createBlock(
            "span",
            {
              key: 2,
              class: "multiselect__placeholder",
              onMousedown: _[11] || (_[11] = withModifiers((...J) => O.toggle && O.toggle(...J), ["prevent"]))
            },
            [
              renderSlot(O.$slots, "placeholder", {}, () => [
                createTextVNode(
                  toDisplayString(O.placeholder),
                  1
                  /* TEXT */
                )
              ])
            ],
            32
            /* HYDRATE_EVENTS */
          )) : createCommentVNode("v-if", !0)
        ],
        512
        /* NEED_PATCH */
      ),
      createVNode(Transition, { name: "multiselect" }, {
        default: withCtx(() => [
          withDirectives(createVNode(
            "div",
            {
              class: "multiselect__content-wrapper",
              onFocus: _[12] || (_[12] = (...J) => O.activate && O.activate(...J)),
              tabindex: "-1",
              onMousedown: _[13] || (_[13] = withModifiers(() => {
              }, ["prevent"])),
              style: { maxHeight: O.optimizedHeight + "px" },
              ref: "list"
            },
            [
              createVNode("ul", {
                class: "multiselect__content",
                style: G.contentStyle,
                role: "listbox",
                id: "listbox-" + O.id,
                "aria-multiselectable": O.multiple
              }, [
                renderSlot(O.$slots, "beforeList"),
                O.multiple && O.max === O.internalValue.length ? (openBlock(), createBlock("li", _hoisted_4$a, [
                  createVNode("span", _hoisted_5$6, [
                    renderSlot(O.$slots, "maxElements", {}, () => [
                      createTextVNode(
                        "Maximum of " + toDisplayString(O.max) + " options selected. First remove a selected option to select another.",
                        1
                        /* TEXT */
                      )
                    ])
                  ])
                ])) : createCommentVNode("v-if", !0),
                !O.max || O.internalValue.length < O.max ? (openBlock(!0), createBlock(
                  Fragment,
                  { key: 1 },
                  renderList(O.filteredOptions, (J, Z) => (openBlock(), createBlock("li", {
                    class: "multiselect__element",
                    key: Z,
                    "aria-selected": O.isSelected(J),
                    id: O.id + "-" + Z,
                    role: J && (J.$isLabel || J.$isDisabled) ? null : "option"
                  }, [
                    J && (J.$isLabel || J.$isDisabled) ? createCommentVNode("v-if", !0) : (openBlock(), createBlock("span", {
                      key: 0,
                      class: [O.optionHighlight(Z, J), "multiselect__option"],
                      onClick: withModifiers((te) => O.select(J), ["stop"]),
                      onMouseenter: withModifiers((te) => O.pointerSet(Z), ["self"]),
                      "data-select": J && J.isTag ? O.tagPlaceholder : G.selectLabelText,
                      "data-selected": G.selectedLabelText,
                      "data-deselect": G.deselectLabelText
                    }, [
                      renderSlot(O.$slots, "option", {
                        option: J,
                        search: O.search,
                        index: Z
                      }, () => [
                        createVNode(
                          "span",
                          null,
                          toDisplayString(O.getOptionLabel(J)),
                          1
                          /* TEXT */
                        )
                      ])
                    ], 42, ["onClick", "onMouseenter", "data-select", "data-selected", "data-deselect"])),
                    J && (J.$isLabel || J.$isDisabled) ? (openBlock(), createBlock("span", {
                      key: 1,
                      "data-select": O.groupSelect && G.selectGroupLabelText,
                      "data-deselect": O.groupSelect && G.deselectGroupLabelText,
                      class: [O.groupHighlight(Z, J), "multiselect__option"],
                      onMouseenter: withModifiers((te) => O.groupSelect && O.pointerSet(Z), ["self"]),
                      onMousedown: withModifiers((te) => O.selectGroup(J), ["prevent"])
                    }, [
                      renderSlot(O.$slots, "option", {
                        option: J,
                        search: O.search,
                        index: Z
                      }, () => [
                        createVNode(
                          "span",
                          null,
                          toDisplayString(O.getOptionLabel(J)),
                          1
                          /* TEXT */
                        )
                      ])
                    ], 42, ["data-select", "data-deselect", "onMouseenter", "onMousedown"])) : createCommentVNode("v-if", !0)
                  ], 8, ["aria-selected", "id", "role"]))),
                  128
                  /* KEYED_FRAGMENT */
                )) : createCommentVNode("v-if", !0),
                withDirectives(createVNode(
                  "li",
                  null,
                  [
                    createVNode("span", _hoisted_6$2, [
                      renderSlot(O.$slots, "noResult", { search: O.search }, () => [
                        _hoisted_7$2
                      ])
                    ])
                  ],
                  512
                  /* NEED_PATCH */
                ), [
                  [vShow, H.showNoResults && O.filteredOptions.length === 0 && O.search && !H.loading]
                ]),
                withDirectives(createVNode(
                  "li",
                  null,
                  [
                    createVNode("span", _hoisted_8$2, [
                      renderSlot(O.$slots, "noOptions", {}, () => [
                        _hoisted_9$1
                      ])
                    ])
                  ],
                  512
                  /* NEED_PATCH */
                ), [
                  [vShow, H.showNoOptions && (O.options.length === 0 || G.hasOptionGroup === !0 && O.filteredOptions.length === 0) && !O.search && !H.loading]
                ]),
                renderSlot(O.$slots, "afterList")
              ], 12, ["id", "aria-multiselectable"])
            ],
            36
            /* STYLE, HYDRATE_EVENTS */
          ), [
            [vShow, O.isOpen]
          ])
        ]),
        _: 3
        /* FORWARDED */
      })
    ], 42, ["tabindex", "aria-owns"]);
  }
  script$2.render = render$4;
  const _hoisted_1$u = ["value"], _sfc_main$R = /* @__PURE__ */ defineComponent({
    __name: "MultiSelectBase",
    props: {
      context: {}
    },
    setup(O) {
      const _ = O, H = ref$1(_.context.tags.map((J) => ({ name: J }))), V = ref$1(_.context.options.map((J) => ({ name: J }))), q = ref$1();
      onMounted(() => {
        if (q.value)
          _.context.node.input(_.context.tags.join(","));
        else
          throw new Error("Tags field not properly initialized");
      }), watch$1(
        () => H,
        () => {
          const J = H.value.map((Z) => Z.name);
          _.context.node.emit("newTag", J), _.context.node.input(J.join(","));
        },
        { deep: !0 }
      );
      const G = (J) => {
        const Z = { name: J };
        H.value.push(Z), V.value.push(Z);
      };
      return (J, Z) => (openBlock(), createElementBlock(Fragment, null, [
        createBaseVNode("input", {
          type: "hidden",
          value: J.context._value,
          ref_key: "hiddenRef",
          ref: q
        }, null, 8, _hoisted_1$u),
        createVNode(unref(script$2), mergeProps$1({
          modelValue: H.value,
          "onUpdate:modelValue": Z[0] || (Z[0] = (te) => H.value = te),
          "tag-placeholder": "Add this as new tag",
          placeholder: "Search or add a tag",
          label: "name",
          "track-by": "name",
          multiple: !0,
          taggable: !0,
          options: V.value,
          onTag: G
        }, J.context.multiSelectProps), null, 16, ["modelValue", "options"])
      ], 64));
    }
  }), textClassification = {
    inner: "mt-1",
    outer: "w-full",
    input: `block w-full rounded-md border-gray-300 shadow-sm focus:border-indigo-500
  focus:ring-indigo-500 sm:text-sm formkit-invalid:border-red-300 formkit-invalid:text-red-900
  formkit-invalid:placeholder-red-300`
  }, boxClassification = {
    // fieldset: "w-full border border-gray-200 rounded-md px-2 pb-1",
    legend: "block text-sm font-medium text-gray-700 formkit-invalid:text-red-500",
    options: "space-y-2 mt-2",
    wrapper: "flex items-center mb-1 cursor-pointer",
    help: "text-sm text-gray-500 mt-0",
    input: "h-4 w-4 rounded border-gray-300 text-indigo-600 focus:ring-indigo-500 ",
    inner: "mr-3 flex h-5 items-center",
    outer: "relative flex",
    label: "font-medium"
  }, buttonClassification = {
    wrapper: "mb-1",
    input: "bg-blue-500 hover:bg-blue-700 text-white text-sm font-normal py-3 px-5 rounded"
  }, formkitConfig = {
    inputs: {
      multiSelectField: createInput(_sfc_main$R, {
        props: ["tags", "options", "multiSelectProps"]
      })
    },
    config: {
      classes: generateClasses({
        // the global key will apply to all inputs
        global: {
          label: "block text-sm font-medium text-gray-700 formkit-invalid:text-red-500",
          outer: "mb-6 formkit-disabled:opacity-50",
          help: "mt-2 text-sm text-gray-500",
          messages: "list-none p-0 mt-1 mb-0",
          message: "text-red-500 mb-1 text-xs"
        },
        button: buttonClassification,
        color: {
          label: "block mb-1 font-bold text-sm",
          input: "w-16 h-8 appearance-none cursor-pointer border border-gray-300 rounded-md mb-2 p-1"
        },
        date: textClassification,
        "datetime-local": textClassification,
        checkbox: boxClassification,
        email: textClassification,
        file: {
          label: "block mb-1 font-bold text-sm ",
          fileItem: "flex items-center text-gray-800 mb-1 last:mb-0 text-sm px-4 pt-3 pb-4",
          fileItemIcon: "w-4 mr-2 shrink-0",
          fileList: "shrink grow peer px-3 py-2 formkit-multiple:data-[has-multiple='true']:mb-6",
          fileName: "break-all grow text-ellipsis",
          fileRemove: "flex items-center z-[2] ml-auto  hover:text-red-500 pl-2 peer-data-[has-multiple=true]:text-sm peer-data-[has-multiple=true]:text-blue-500 peer-data-[has-multiple=true]:ml-3 peer-data-[has-multiple=true]:mb-2 formkit-multiple:bottom-[0.15em] formkit-multiple:pl-0 formkit-multiple:ml-0 formkit-multiple:left-[1em] formkit-multiple:formkit-prefix-icon:left-[3.75em]",
          fileRemoveIcon: "block text-base w-3 relative z-[2] mr-2",
          inner: "mt-1 flex relative justify-center rounded-md border-2 border-dashed border-gray-300 ",
          input: "hidden"
        },
        month: textClassification,
        number: textClassification,
        password: textClassification,
        tagsField: {
          ...textClassification,
          input: `${textClassification.input}`,
          outer: `${textClassification.outer} mb-1`
        },
        radio: {
          ...boxClassification,
          input: boxClassification.input.replace("rounded-sm", "rounded-full")
        },
        range: {
          inner: "w-full flex items-center",
          input: "w-full h-2 bg-gray-200 rounded-lg appearance-none cursor-pointer dark:bg-gray-700"
        },
        search: textClassification,
        select: textClassification,
        submit: buttonClassification,
        tel: textClassification,
        text: textClassification,
        textarea: {
          ...textClassification,
          input: "block w-full h-32 px-3 text-base text-gray-700 placeholder-gray-400 border-gray-300 focus:shadow-outline"
        },
        time: textClassification,
        url: textClassification,
        week: textClassification,
        tags: textClassification
      })
    }
  };
  function bind$3(O, _) {
    return function() {
      return O.apply(_, arguments);
    };
  }
  const { toString: toString$5 } = Object.prototype, { getPrototypeOf } = Object, kindOf = /* @__PURE__ */ ((O) => (_) => {
    const H = toString$5.call(_);
    return O[H] || (O[H] = H.slice(8, -1).toLowerCase());
  })(/* @__PURE__ */ Object.create(null)), kindOfTest = (O) => (O = O.toLowerCase(), (_) => kindOf(_) === O), typeOfTest = (O) => (_) => typeof _ === O, { isArray: isArray$5 } = Array, isUndefined$1 = typeOfTest("undefined");
  function isBuffer$3(O) {
    return O !== null && !isUndefined$1(O) && O.constructor !== null && !isUndefined$1(O.constructor) && isFunction$5(O.constructor.isBuffer) && O.constructor.isBuffer(O);
  }
  const isArrayBuffer = kindOfTest("ArrayBuffer");
  function isArrayBufferView(O) {
    let _;
    return typeof ArrayBuffer < "u" && ArrayBuffer.isView ? _ = ArrayBuffer.isView(O) : _ = O && O.buffer && isArrayBuffer(O.buffer), _;
  }
  const isString$4 = typeOfTest("string"), isFunction$5 = typeOfTest("function"), isNumber$5 = typeOfTest("number"), isObject$5 = (O) => O !== null && typeof O == "object", isBoolean$5 = (O) => O === !0 || O === !1, isPlainObject$1 = (O) => {
    if (kindOf(O) !== "object")
      return !1;
    const _ = getPrototypeOf(O);
    return (_ === null || _ === Object.prototype || Object.getPrototypeOf(_) === null) && !(Symbol.toStringTag in O) && !(Symbol.iterator in O);
  }, isDate$3 = kindOfTest("Date"), isFile = kindOfTest("File"), isBlob = kindOfTest("Blob"), isFileList = kindOfTest("FileList"), isStream = (O) => isObject$5(O) && isFunction$5(O.pipe), isFormData = (O) => {
    let _;
    return O && (typeof FormData == "function" && O instanceof FormData || isFunction$5(O.append) && ((_ = kindOf(O)) === "formdata" || // detect form-data instance
    _ === "object" && isFunction$5(O.toString) && O.toString() === "[object FormData]"));
  }, isURLSearchParams = kindOfTest("URLSearchParams"), [isReadableStream, isRequest, isResponse, isHeaders] = ["ReadableStream", "Request", "Response", "Headers"].map(kindOfTest), trim = (O) => O.trim ? O.trim() : O.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, "");
  function forEach$2(O, _, { allOwnKeys: H = !1 } = {}) {
    if (O === null || typeof O > "u")
      return;
    let V, q;
    if (typeof O != "object" && (O = [O]), isArray$5(O))
      for (V = 0, q = O.length; V < q; V++)
        _.call(null, O[V], V, O);
    else {
      const G = H ? Object.getOwnPropertyNames(O) : Object.keys(O), J = G.length;
      let Z;
      for (V = 0; V < J; V++)
        Z = G[V], _.call(null, O[Z], Z, O);
    }
  }
  function findKey(O, _) {
    _ = _.toLowerCase();
    const H = Object.keys(O);
    let V = H.length, q;
    for (; V-- > 0; )
      if (q = H[V], _ === q.toLowerCase())
        return q;
    return null;
  }
  const _global = typeof globalThis < "u" ? globalThis : typeof self < "u" ? self : typeof window < "u" ? window : global, isContextDefined = (O) => !isUndefined$1(O) && O !== _global;
  function merge$5() {
    const { caseless: O } = isContextDefined(this) && this || {}, _ = {}, H = (V, q) => {
      const G = O && findKey(_, q) || q;
      isPlainObject$1(_[G]) && isPlainObject$1(V) ? _[G] = merge$5(_[G], V) : isPlainObject$1(V) ? _[G] = merge$5({}, V) : isArray$5(V) ? _[G] = V.slice() : _[G] = V;
    };
    for (let V = 0, q = arguments.length; V < q; V++)
      arguments[V] && forEach$2(arguments[V], H);
    return _;
  }
  const extend$7 = (O, _, H, { allOwnKeys: V } = {}) => (forEach$2(_, (q, G) => {
    H && isFunction$5(q) ? O[G] = bind$3(q, H) : O[G] = q;
  }, { allOwnKeys: V }), O), stripBOM = (O) => (O.charCodeAt(0) === 65279 && (O = O.slice(1)), O), inherits$1 = (O, _, H, V) => {
    O.prototype = Object.create(_.prototype, V), O.prototype.constructor = O, Object.defineProperty(O, "super", {
      value: _.prototype
    }), H && Object.assign(O.prototype, H);
  }, toFlatObject = (O, _, H, V) => {
    let q, G, J;
    const Z = {};
    if (_ = _ || {}, O == null) return _;
    do {
      for (q = Object.getOwnPropertyNames(O), G = q.length; G-- > 0; )
        J = q[G], (!V || V(J, O, _)) && !Z[J] && (_[J] = O[J], Z[J] = !0);
      O = H !== !1 && getPrototypeOf(O);
    } while (O && (!H || H(O, _)) && O !== Object.prototype);
    return _;
  }, endsWith = (O, _, H) => {
    O = String(O), (H === void 0 || H > O.length) && (H = O.length), H -= _.length;
    const V = O.indexOf(_, H);
    return V !== -1 && V === H;
  }, toArray$1 = (O) => {
    if (!O) return null;
    if (isArray$5(O)) return O;
    let _ = O.length;
    if (!isNumber$5(_)) return null;
    const H = new Array(_);
    for (; _-- > 0; )
      H[_] = O[_];
    return H;
  }, isTypedArray$3 = /* @__PURE__ */ ((O) => (_) => O && _ instanceof O)(typeof Uint8Array < "u" && getPrototypeOf(Uint8Array)), forEachEntry = (O, _) => {
    const V = (O && O[Symbol.iterator]).call(O);
    let q;
    for (; (q = V.next()) && !q.done; ) {
      const G = q.value;
      _.call(O, G[0], G[1]);
    }
  }, matchAll = (O, _) => {
    let H;
    const V = [];
    for (; (H = O.exec(_)) !== null; )
      V.push(H);
    return V;
  }, isHTMLForm = kindOfTest("HTMLFormElement"), toCamelCase = (O) => O.toLowerCase().replace(
    /[-_\s]([a-z\d])(\w*)/g,
    function(H, V, q) {
      return V.toUpperCase() + q;
    }
  ), hasOwnProperty$d = (({ hasOwnProperty: O }) => (_, H) => O.call(_, H))(Object.prototype), isRegExp$1 = kindOfTest("RegExp"), reduceDescriptors = (O, _) => {
    const H = Object.getOwnPropertyDescriptors(O), V = {};
    forEach$2(H, (q, G) => {
      let J;
      (J = _(q, G, O)) !== !1 && (V[G] = J || q);
    }), Object.defineProperties(O, V);
  }, freezeMethods = (O) => {
    reduceDescriptors(O, (_, H) => {
      if (isFunction$5(O) && ["arguments", "caller", "callee"].indexOf(H) !== -1)
        return !1;
      const V = O[H];
      if (isFunction$5(V)) {
        if (_.enumerable = !1, "writable" in _) {
          _.writable = !1;
          return;
        }
        _.set || (_.set = () => {
          throw Error("Can not rewrite read-only method '" + H + "'");
        });
      }
    });
  }, toObjectSet = (O, _) => {
    const H = {}, V = (q) => {
      q.forEach((G) => {
        H[G] = !0;
      });
    };
    return isArray$5(O) ? V(O) : V(String(O).split(_)), H;
  }, noop$5 = () => {
  }, toFiniteNumber = (O, _) => O != null && Number.isFinite(O = +O) ? O : _, ALPHA$1 = "abcdefghijklmnopqrstuvwxyz", DIGIT = "0123456789", ALPHABET = {
    DIGIT,
    ALPHA: ALPHA$1,
    ALPHA_DIGIT: ALPHA$1 + ALPHA$1.toUpperCase() + DIGIT
  }, generateString = (O = 16, _ = ALPHABET.ALPHA_DIGIT) => {
    let H = "";
    const { length: V } = _;
    for (; O--; )
      H += _[Math.random() * V | 0];
    return H;
  };
  function isSpecCompliantForm(O) {
    return !!(O && isFunction$5(O.append) && O[Symbol.toStringTag] === "FormData" && O[Symbol.iterator]);
  }
  const toJSONObject = (O) => {
    const _ = new Array(10), H = (V, q) => {
      if (isObject$5(V)) {
        if (_.indexOf(V) >= 0)
          return;
        if (!("toJSON" in V)) {
          _[q] = V;
          const G = isArray$5(V) ? [] : {};
          return forEach$2(V, (J, Z) => {
            const te = H(J, q + 1);
            !isUndefined$1(te) && (G[Z] = te);
          }), _[q] = void 0, G;
        }
      }
      return V;
    };
    return H(O, 0);
  }, isAsyncFn = kindOfTest("AsyncFunction"), isThenable = (O) => O && (isObject$5(O) || isFunction$5(O)) && isFunction$5(O.then) && isFunction$5(O.catch), _setImmediate = ((O, _) => O ? setImmediate : _ ? ((H, V) => (_global.addEventListener("message", ({ source: q, data: G }) => {
    q === _global && G === H && V.length && V.shift()();
  }, !1), (q) => {
    V.push(q), _global.postMessage(H, "*");
  }))(`axios@${Math.random()}`, []) : (H) => setTimeout(H))(
    typeof setImmediate == "function",
    isFunction$5(_global.postMessage)
  ), asap = typeof queueMicrotask < "u" ? queueMicrotask.bind(_global) : typeof process < "u" && process.nextTick || _setImmediate, utils$2 = {
    isArray: isArray$5,
    isArrayBuffer,
    isBuffer: isBuffer$3,
    isFormData,
    isArrayBufferView,
    isString: isString$4,
    isNumber: isNumber$5,
    isBoolean: isBoolean$5,
    isObject: isObject$5,
    isPlainObject: isPlainObject$1,
    isReadableStream,
    isRequest,
    isResponse,
    isHeaders,
    isUndefined: isUndefined$1,
    isDate: isDate$3,
    isFile,
    isBlob,
    isRegExp: isRegExp$1,
    isFunction: isFunction$5,
    isStream,
    isURLSearchParams,
    isTypedArray: isTypedArray$3,
    isFileList,
    forEach: forEach$2,
    merge: merge$5,
    extend: extend$7,
    trim,
    stripBOM,
    inherits: inherits$1,
    toFlatObject,
    kindOf,
    kindOfTest,
    endsWith,
    toArray: toArray$1,
    forEachEntry,
    matchAll,
    isHTMLForm,
    hasOwnProperty: hasOwnProperty$d,
    hasOwnProp: hasOwnProperty$d,
    // an alias to avoid ESLint no-prototype-builtins detection
    reduceDescriptors,
    freezeMethods,
    toObjectSet,
    toCamelCase,
    noop: noop$5,
    toFiniteNumber,
    findKey,
    global: _global,
    isContextDefined,
    ALPHABET,
    generateString,
    isSpecCompliantForm,
    toJSONObject,
    isAsyncFn,
    isThenable,
    setImmediate: _setImmediate,
    asap
  };
  function AxiosError(O, _, H, V, q) {
    Error.call(this), Error.captureStackTrace ? Error.captureStackTrace(this, this.constructor) : this.stack = new Error().stack, this.message = O, this.name = "AxiosError", _ && (this.code = _), H && (this.config = H), V && (this.request = V), q && (this.response = q, this.status = q.status ? q.status : null);
  }
  utils$2.inherits(AxiosError, Error, {
    toJSON: function() {
      return {
        // Standard
        message: this.message,
        name: this.name,
        // Microsoft
        description: this.description,
        number: this.number,
        // Mozilla
        fileName: this.fileName,
        lineNumber: this.lineNumber,
        columnNumber: this.columnNumber,
        stack: this.stack,
        // Axios
        config: utils$2.toJSONObject(this.config),
        code: this.code,
        status: this.status
      };
    }
  });
  const prototype$3 = AxiosError.prototype, descriptors = {};
  [
    "ERR_BAD_OPTION_VALUE",
    "ERR_BAD_OPTION",
    "ECONNABORTED",
    "ETIMEDOUT",
    "ERR_NETWORK",
    "ERR_FR_TOO_MANY_REDIRECTS",
    "ERR_DEPRECATED",
    "ERR_BAD_RESPONSE",
    "ERR_BAD_REQUEST",
    "ERR_CANCELED",
    "ERR_NOT_SUPPORT",
    "ERR_INVALID_URL"
    // eslint-disable-next-line func-names
  ].forEach((O) => {
    descriptors[O] = { value: O };
  });
  Object.defineProperties(AxiosError, descriptors);
  Object.defineProperty(prototype$3, "isAxiosError", { value: !0 });
  AxiosError.from = (O, _, H, V, q, G) => {
    const J = Object.create(prototype$3);
    return utils$2.toFlatObject(O, J, function(te) {
      return te !== Error.prototype;
    }, (Z) => Z !== "isAxiosError"), AxiosError.call(J, O.message, _, H, V, q), J.cause = O, J.name = O.name, G && Object.assign(J, G), J;
  };
  const httpAdapter = null;
  function isVisitable(O) {
    return utils$2.isPlainObject(O) || utils$2.isArray(O);
  }
  function removeBrackets(O) {
    return utils$2.endsWith(O, "[]") ? O.slice(0, -2) : O;
  }
  function renderKey(O, _, H) {
    return O ? O.concat(_).map(function(q, G) {
      return q = removeBrackets(q), !H && G ? "[" + q + "]" : q;
    }).join(H ? "." : "") : _;
  }
  function isFlatArray(O) {
    return utils$2.isArray(O) && !O.some(isVisitable);
  }
  const predicates = utils$2.toFlatObject(utils$2, {}, null, function(_) {
    return /^is[A-Z]/.test(_);
  });
  function toFormData(O, _, H) {
    if (!utils$2.isObject(O))
      throw new TypeError("target must be an object");
    _ = _ || new FormData(), H = utils$2.toFlatObject(H, {
      metaTokens: !0,
      dots: !1,
      indexes: !1
    }, !1, function(le, ue) {
      return !utils$2.isUndefined(ue[le]);
    });
    const V = H.metaTokens, q = H.visitor || ee, G = H.dots, J = H.indexes, te = (H.Blob || typeof Blob < "u" && Blob) && utils$2.isSpecCompliantForm(_);
    if (!utils$2.isFunction(q))
      throw new TypeError("visitor must be a function");
    function ne(ce) {
      if (ce === null) return "";
      if (utils$2.isDate(ce))
        return ce.toISOString();
      if (!te && utils$2.isBlob(ce))
        throw new AxiosError("Blob is not supported. Use a Buffer instead.");
      return utils$2.isArrayBuffer(ce) || utils$2.isTypedArray(ce) ? te && typeof Blob == "function" ? new Blob([ce]) : Buffer.from(ce) : ce;
    }
    function ee(ce, le, ue) {
      let de = ce;
      if (ce && !ue && typeof ce == "object") {
        if (utils$2.endsWith(le, "{}"))
          le = V ? le : le.slice(0, -2), ce = JSON.stringify(ce);
        else if (utils$2.isArray(ce) && isFlatArray(ce) || (utils$2.isFileList(ce) || utils$2.endsWith(le, "[]")) && (de = utils$2.toArray(ce)))
          return le = removeBrackets(le), de.forEach(function(he, me) {
            !(utils$2.isUndefined(he) || he === null) && _.append(
              // eslint-disable-next-line no-nested-ternary
              J === !0 ? renderKey([le], me, G) : J === null ? le : le + "[]",
              ne(he)
            );
          }), !1;
      }
      return isVisitable(ce) ? !0 : (_.append(renderKey(ue, le, G), ne(ce)), !1);
    }
    const ae = [], oe = Object.assign(predicates, {
      defaultVisitor: ee,
      convertValue: ne,
      isVisitable
    });
    function se(ce, le) {
      if (!utils$2.isUndefined(ce)) {
        if (ae.indexOf(ce) !== -1)
          throw Error("Circular reference detected in " + le.join("."));
        ae.push(ce), utils$2.forEach(ce, function(de, fe) {
          (!(utils$2.isUndefined(de) || de === null) && q.call(
            _,
            de,
            utils$2.isString(fe) ? fe.trim() : fe,
            le,
            oe
          )) === !0 && se(de, le ? le.concat(fe) : [fe]);
        }), ae.pop();
      }
    }
    if (!utils$2.isObject(O))
      throw new TypeError("data must be an object");
    return se(O), _;
  }
  function encode$3(O) {
    const _ = {
      "!": "%21",
      "'": "%27",
      "(": "%28",
      ")": "%29",
      "~": "%7E",
      "%20": "+",
      "%00": "\0"
    };
    return encodeURIComponent(O).replace(/[!'()~]|%20|%00/g, function(V) {
      return _[V];
    });
  }
  function AxiosURLSearchParams(O, _) {
    this._pairs = [], O && toFormData(O, this, _);
  }
  const prototype$2 = AxiosURLSearchParams.prototype;
  prototype$2.append = function(_, H) {
    this._pairs.push([_, H]);
  };
  prototype$2.toString = function(_) {
    const H = _ ? function(V) {
      return _.call(this, V, encode$3);
    } : encode$3;
    return this._pairs.map(function(q) {
      return H(q[0]) + "=" + H(q[1]);
    }, "").join("&");
  };
  function encode$2(O) {
    return encodeURIComponent(O).replace(/%3A/gi, ":").replace(/%24/g, "$").replace(/%2C/gi, ",").replace(/%20/g, "+").replace(/%5B/gi, "[").replace(/%5D/gi, "]");
  }
  function buildURL(O, _, H) {
    if (!_)
      return O;
    const V = H && H.encode || encode$2;
    utils$2.isFunction(H) && (H = {
      serialize: H
    });
    const q = H && H.serialize;
    let G;
    if (q ? G = q(_, H) : G = utils$2.isURLSearchParams(_) ? _.toString() : new AxiosURLSearchParams(_, H).toString(V), G) {
      const J = O.indexOf("#");
      J !== -1 && (O = O.slice(0, J)), O += (O.indexOf("?") === -1 ? "?" : "&") + G;
    }
    return O;
  }
  class InterceptorManager {
    constructor() {
      this.handlers = [];
    }
    /**
     * Add a new interceptor to the stack
     *
     * @param {Function} fulfilled The function to handle `then` for a `Promise`
     * @param {Function} rejected The function to handle `reject` for a `Promise`
     *
     * @return {Number} An ID used to remove interceptor later
     */
    use(_, H, V) {
      return this.handlers.push({
        fulfilled: _,
        rejected: H,
        synchronous: V ? V.synchronous : !1,
        runWhen: V ? V.runWhen : null
      }), this.handlers.length - 1;
    }
    /**
     * Remove an interceptor from the stack
     *
     * @param {Number} id The ID that was returned by `use`
     *
     * @returns {Boolean} `true` if the interceptor was removed, `false` otherwise
     */
    eject(_) {
      this.handlers[_] && (this.handlers[_] = null);
    }
    /**
     * Clear all interceptors from the stack
     *
     * @returns {void}
     */
    clear() {
      this.handlers && (this.handlers = []);
    }
    /**
     * Iterate over all the registered interceptors
     *
     * This method is particularly useful for skipping over any
     * interceptors that may have become `null` calling `eject`.
     *
     * @param {Function} fn The function to call for each interceptor
     *
     * @returns {void}
     */
    forEach(_) {
      utils$2.forEach(this.handlers, function(V) {
        V !== null && _(V);
      });
    }
  }
  const transitionalDefaults = {
    silentJSONParsing: !0,
    forcedJSONParsing: !0,
    clarifyTimeoutError: !1
  }, URLSearchParams$1 = typeof URLSearchParams < "u" ? URLSearchParams : AxiosURLSearchParams, FormData$1 = typeof FormData < "u" ? FormData : null, Blob$1 = typeof Blob < "u" ? Blob : null, platform$1 = {
    isBrowser: !0,
    classes: {
      URLSearchParams: URLSearchParams$1,
      FormData: FormData$1,
      Blob: Blob$1
    },
    protocols: ["http", "https", "file", "blob", "url", "data"]
  }, hasBrowserEnv = typeof window < "u" && typeof document < "u", _navigator = typeof navigator == "object" && navigator || void 0, hasStandardBrowserEnv = hasBrowserEnv && (!_navigator || ["ReactNative", "NativeScript", "NS"].indexOf(_navigator.product) < 0), hasStandardBrowserWebWorkerEnv = typeof WorkerGlobalScope < "u" && // eslint-disable-next-line no-undef
  self instanceof WorkerGlobalScope && typeof self.importScripts == "function", origin = hasBrowserEnv && window.location.href || "http://localhost", utils$1 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    hasBrowserEnv,
    hasStandardBrowserEnv,
    hasStandardBrowserWebWorkerEnv,
    navigator: _navigator,
    origin
  }, Symbol.toStringTag, { value: "Module" })), platform = {
    ...utils$1,
    ...platform$1
  };
  function toURLEncodedForm(O, _) {
    return toFormData(O, new platform.classes.URLSearchParams(), Object.assign({
      visitor: function(H, V, q, G) {
        return platform.isNode && utils$2.isBuffer(H) ? (this.append(V, H.toString("base64")), !1) : G.defaultVisitor.apply(this, arguments);
      }
    }, _));
  }
  function parsePropPath(O) {
    return utils$2.matchAll(/\w+|\[(\w*)]/g, O).map((_) => _[0] === "[]" ? "" : _[1] || _[0]);
  }
  function arrayToObject(O) {
    const _ = {}, H = Object.keys(O);
    let V;
    const q = H.length;
    let G;
    for (V = 0; V < q; V++)
      G = H[V], _[G] = O[G];
    return _;
  }
  function formDataToJSON(O) {
    function _(H, V, q, G) {
      let J = H[G++];
      if (J === "__proto__") return !0;
      const Z = Number.isFinite(+J), te = G >= H.length;
      return J = !J && utils$2.isArray(q) ? q.length : J, te ? (utils$2.hasOwnProp(q, J) ? q[J] = [q[J], V] : q[J] = V, !Z) : ((!q[J] || !utils$2.isObject(q[J])) && (q[J] = []), _(H, V, q[J], G) && utils$2.isArray(q[J]) && (q[J] = arrayToObject(q[J])), !Z);
    }
    if (utils$2.isFormData(O) && utils$2.isFunction(O.entries)) {
      const H = {};
      return utils$2.forEachEntry(O, (V, q) => {
        _(parsePropPath(V), q, H, 0);
      }), H;
    }
    return null;
  }
  function stringifySafely(O, _, H) {
    if (utils$2.isString(O))
      try {
        return (_ || JSON.parse)(O), utils$2.trim(O);
      } catch (V) {
        if (V.name !== "SyntaxError")
          throw V;
      }
    return (0, JSON.stringify)(O);
  }
  const defaults$7 = {
    transitional: transitionalDefaults,
    adapter: ["xhr", "http", "fetch"],
    transformRequest: [function(_, H) {
      const V = H.getContentType() || "", q = V.indexOf("application/json") > -1, G = utils$2.isObject(_);
      if (G && utils$2.isHTMLForm(_) && (_ = new FormData(_)), utils$2.isFormData(_))
        return q ? JSON.stringify(formDataToJSON(_)) : _;
      if (utils$2.isArrayBuffer(_) || utils$2.isBuffer(_) || utils$2.isStream(_) || utils$2.isFile(_) || utils$2.isBlob(_) || utils$2.isReadableStream(_))
        return _;
      if (utils$2.isArrayBufferView(_))
        return _.buffer;
      if (utils$2.isURLSearchParams(_))
        return H.setContentType("application/x-www-form-urlencoded;charset=utf-8", !1), _.toString();
      let Z;
      if (G) {
        if (V.indexOf("application/x-www-form-urlencoded") > -1)
          return toURLEncodedForm(_, this.formSerializer).toString();
        if ((Z = utils$2.isFileList(_)) || V.indexOf("multipart/form-data") > -1) {
          const te = this.env && this.env.FormData;
          return toFormData(
            Z ? { "files[]": _ } : _,
            te && new te(),
            this.formSerializer
          );
        }
      }
      return G || q ? (H.setContentType("application/json", !1), stringifySafely(_)) : _;
    }],
    transformResponse: [function(_) {
      const H = this.transitional || defaults$7.transitional, V = H && H.forcedJSONParsing, q = this.responseType === "json";
      if (utils$2.isResponse(_) || utils$2.isReadableStream(_))
        return _;
      if (_ && utils$2.isString(_) && (V && !this.responseType || q)) {
        const J = !(H && H.silentJSONParsing) && q;
        try {
          return JSON.parse(_);
        } catch (Z) {
          if (J)
            throw Z.name === "SyntaxError" ? AxiosError.from(Z, AxiosError.ERR_BAD_RESPONSE, this, null, this.response) : Z;
        }
      }
      return _;
    }],
    /**
     * A timeout in milliseconds to abort a request. If set to 0 (default) a
     * timeout is not created.
     */
    timeout: 0,
    xsrfCookieName: "XSRF-TOKEN",
    xsrfHeaderName: "X-XSRF-TOKEN",
    maxContentLength: -1,
    maxBodyLength: -1,
    env: {
      FormData: platform.classes.FormData,
      Blob: platform.classes.Blob
    },
    validateStatus: function(_) {
      return _ >= 200 && _ < 300;
    },
    headers: {
      common: {
        Accept: "application/json, text/plain, */*",
        "Content-Type": void 0
      }
    }
  };
  utils$2.forEach(["delete", "get", "head", "post", "put", "patch"], (O) => {
    defaults$7.headers[O] = {};
  });
  const ignoreDuplicateOf = utils$2.toObjectSet([
    "age",
    "authorization",
    "content-length",
    "content-type",
    "etag",
    "expires",
    "from",
    "host",
    "if-modified-since",
    "if-unmodified-since",
    "last-modified",
    "location",
    "max-forwards",
    "proxy-authorization",
    "referer",
    "retry-after",
    "user-agent"
  ]), parseHeaders = (O) => {
    const _ = {};
    let H, V, q;
    return O && O.split(`
`).forEach(function(J) {
      q = J.indexOf(":"), H = J.substring(0, q).trim().toLowerCase(), V = J.substring(q + 1).trim(), !(!H || _[H] && ignoreDuplicateOf[H]) && (H === "set-cookie" ? _[H] ? _[H].push(V) : _[H] = [V] : _[H] = _[H] ? _[H] + ", " + V : V);
    }), _;
  }, $internals = Symbol("internals");
  function normalizeHeader(O) {
    return O && String(O).trim().toLowerCase();
  }
  function normalizeValue(O) {
    return O === !1 || O == null ? O : utils$2.isArray(O) ? O.map(normalizeValue) : String(O);
  }
  function parseTokens(O) {
    const _ = /* @__PURE__ */ Object.create(null), H = /([^\s,;=]+)\s*(?:=\s*([^,;]+))?/g;
    let V;
    for (; V = H.exec(O); )
      _[V[1]] = V[2];
    return _;
  }
  const isValidHeaderName = (O) => /^[-_a-zA-Z0-9^`|~,!#$%&'*+.]+$/.test(O.trim());
  function matchHeaderValue(O, _, H, V, q) {
    if (utils$2.isFunction(V))
      return V.call(this, _, H);
    if (q && (_ = H), !!utils$2.isString(_)) {
      if (utils$2.isString(V))
        return _.indexOf(V) !== -1;
      if (utils$2.isRegExp(V))
        return V.test(_);
    }
  }
  function formatHeader(O) {
    return O.trim().toLowerCase().replace(/([a-z\d])(\w*)/g, (_, H, V) => H.toUpperCase() + V);
  }
  function buildAccessors(O, _) {
    const H = utils$2.toCamelCase(" " + _);
    ["get", "set", "has"].forEach((V) => {
      Object.defineProperty(O, V + H, {
        value: function(q, G, J) {
          return this[V].call(this, _, q, G, J);
        },
        configurable: !0
      });
    });
  }
  class AxiosHeaders {
    constructor(_) {
      _ && this.set(_);
    }
    set(_, H, V) {
      const q = this;
      function G(Z, te, ne) {
        const ee = normalizeHeader(te);
        if (!ee)
          throw new Error("header name must be a non-empty string");
        const ae = utils$2.findKey(q, ee);
        (!ae || q[ae] === void 0 || ne === !0 || ne === void 0 && q[ae] !== !1) && (q[ae || te] = normalizeValue(Z));
      }
      const J = (Z, te) => utils$2.forEach(Z, (ne, ee) => G(ne, ee, te));
      if (utils$2.isPlainObject(_) || _ instanceof this.constructor)
        J(_, H);
      else if (utils$2.isString(_) && (_ = _.trim()) && !isValidHeaderName(_))
        J(parseHeaders(_), H);
      else if (utils$2.isHeaders(_))
        for (const [Z, te] of _.entries())
          G(te, Z, V);
      else
        _ != null && G(H, _, V);
      return this;
    }
    get(_, H) {
      if (_ = normalizeHeader(_), _) {
        const V = utils$2.findKey(this, _);
        if (V) {
          const q = this[V];
          if (!H)
            return q;
          if (H === !0)
            return parseTokens(q);
          if (utils$2.isFunction(H))
            return H.call(this, q, V);
          if (utils$2.isRegExp(H))
            return H.exec(q);
          throw new TypeError("parser must be boolean|regexp|function");
        }
      }
    }
    has(_, H) {
      if (_ = normalizeHeader(_), _) {
        const V = utils$2.findKey(this, _);
        return !!(V && this[V] !== void 0 && (!H || matchHeaderValue(this, this[V], V, H)));
      }
      return !1;
    }
    delete(_, H) {
      const V = this;
      let q = !1;
      function G(J) {
        if (J = normalizeHeader(J), J) {
          const Z = utils$2.findKey(V, J);
          Z && (!H || matchHeaderValue(V, V[Z], Z, H)) && (delete V[Z], q = !0);
        }
      }
      return utils$2.isArray(_) ? _.forEach(G) : G(_), q;
    }
    clear(_) {
      const H = Object.keys(this);
      let V = H.length, q = !1;
      for (; V--; ) {
        const G = H[V];
        (!_ || matchHeaderValue(this, this[G], G, _, !0)) && (delete this[G], q = !0);
      }
      return q;
    }
    normalize(_) {
      const H = this, V = {};
      return utils$2.forEach(this, (q, G) => {
        const J = utils$2.findKey(V, G);
        if (J) {
          H[J] = normalizeValue(q), delete H[G];
          return;
        }
        const Z = _ ? formatHeader(G) : String(G).trim();
        Z !== G && delete H[G], H[Z] = normalizeValue(q), V[Z] = !0;
      }), this;
    }
    concat(..._) {
      return this.constructor.concat(this, ..._);
    }
    toJSON(_) {
      const H = /* @__PURE__ */ Object.create(null);
      return utils$2.forEach(this, (V, q) => {
        V != null && V !== !1 && (H[q] = _ && utils$2.isArray(V) ? V.join(", ") : V);
      }), H;
    }
    [Symbol.iterator]() {
      return Object.entries(this.toJSON())[Symbol.iterator]();
    }
    toString() {
      return Object.entries(this.toJSON()).map(([_, H]) => _ + ": " + H).join(`
`);
    }
    get [Symbol.toStringTag]() {
      return "AxiosHeaders";
    }
    static from(_) {
      return _ instanceof this ? _ : new this(_);
    }
    static concat(_, ...H) {
      const V = new this(_);
      return H.forEach((q) => V.set(q)), V;
    }
    static accessor(_) {
      const V = (this[$internals] = this[$internals] = {
        accessors: {}
      }).accessors, q = this.prototype;
      function G(J) {
        const Z = normalizeHeader(J);
        V[Z] || (buildAccessors(q, J), V[Z] = !0);
      }
      return utils$2.isArray(_) ? _.forEach(G) : G(_), this;
    }
  }
  AxiosHeaders.accessor(["Content-Type", "Content-Length", "Accept", "Accept-Encoding", "User-Agent", "Authorization"]);
  utils$2.reduceDescriptors(AxiosHeaders.prototype, ({ value: O }, _) => {
    let H = _[0].toUpperCase() + _.slice(1);
    return {
      get: () => O,
      set(V) {
        this[H] = V;
      }
    };
  });
  utils$2.freezeMethods(AxiosHeaders);
  function transformData(O, _) {
    const H = this || defaults$7, V = _ || H, q = AxiosHeaders.from(V.headers);
    let G = V.data;
    return utils$2.forEach(O, function(Z) {
      G = Z.call(H, G, q.normalize(), _ ? _.status : void 0);
    }), q.normalize(), G;
  }
  function isCancel(O) {
    return !!(O && O.__CANCEL__);
  }
  function CanceledError(O, _, H) {
    AxiosError.call(this, O ?? "canceled", AxiosError.ERR_CANCELED, _, H), this.name = "CanceledError";
  }
  utils$2.inherits(CanceledError, AxiosError, {
    __CANCEL__: !0
  });
  function settle(O, _, H) {
    const V = H.config.validateStatus;
    !H.status || !V || V(H.status) ? O(H) : _(new AxiosError(
      "Request failed with status code " + H.status,
      [AxiosError.ERR_BAD_REQUEST, AxiosError.ERR_BAD_RESPONSE][Math.floor(H.status / 100) - 4],
      H.config,
      H.request,
      H
    ));
  }
  function parseProtocol(O) {
    const _ = /^([-+\w]{1,25})(:?\/\/|:)/.exec(O);
    return _ && _[1] || "";
  }
  function speedometer(O, _) {
    O = O || 10;
    const H = new Array(O), V = new Array(O);
    let q = 0, G = 0, J;
    return _ = _ !== void 0 ? _ : 1e3, function(te) {
      const ne = Date.now(), ee = V[G];
      J || (J = ne), H[q] = te, V[q] = ne;
      let ae = G, oe = 0;
      for (; ae !== q; )
        oe += H[ae++], ae = ae % O;
      if (q = (q + 1) % O, q === G && (G = (G + 1) % O), ne - J < _)
        return;
      const se = ee && ne - ee;
      return se ? Math.round(oe * 1e3 / se) : void 0;
    };
  }
  function throttle$2(O, _) {
    let H = 0, V = 1e3 / _, q, G;
    const J = (ne, ee = Date.now()) => {
      H = ee, q = null, G && (clearTimeout(G), G = null), O.apply(null, ne);
    };
    return [(...ne) => {
      const ee = Date.now(), ae = ee - H;
      ae >= V ? J(ne, ee) : (q = ne, G || (G = setTimeout(() => {
        G = null, J(q);
      }, V - ae)));
    }, () => q && J(q)];
  }
  const progressEventReducer = (O, _, H = 3) => {
    let V = 0;
    const q = speedometer(50, 250);
    return throttle$2((G) => {
      const J = G.loaded, Z = G.lengthComputable ? G.total : void 0, te = J - V, ne = q(te), ee = J <= Z;
      V = J;
      const ae = {
        loaded: J,
        total: Z,
        progress: Z ? J / Z : void 0,
        bytes: te,
        rate: ne || void 0,
        estimated: ne && Z && ee ? (Z - J) / ne : void 0,
        event: G,
        lengthComputable: Z != null,
        [_ ? "download" : "upload"]: !0
      };
      O(ae);
    }, H);
  }, progressEventDecorator = (O, _) => {
    const H = O != null;
    return [(V) => _[0]({
      lengthComputable: H,
      total: O,
      loaded: V
    }), _[1]];
  }, asyncDecorator = (O) => (..._) => utils$2.asap(() => O(..._)), isURLSameOrigin = platform.hasStandardBrowserEnv ? /* @__PURE__ */ ((O, _) => (H) => (H = new URL(H, platform.origin), O.protocol === H.protocol && O.host === H.host && (_ || O.port === H.port)))(
    new URL(platform.origin),
    platform.navigator && /(msie|trident)/i.test(platform.navigator.userAgent)
  ) : () => !0, cookies = platform.hasStandardBrowserEnv ? (
    // Standard browser envs support document.cookie
    {
      write(O, _, H, V, q, G) {
        const J = [O + "=" + encodeURIComponent(_)];
        utils$2.isNumber(H) && J.push("expires=" + new Date(H).toGMTString()), utils$2.isString(V) && J.push("path=" + V), utils$2.isString(q) && J.push("domain=" + q), G === !0 && J.push("secure"), document.cookie = J.join("; ");
      },
      read(O) {
        const _ = document.cookie.match(new RegExp("(^|;\\s*)(" + O + ")=([^;]*)"));
        return _ ? decodeURIComponent(_[3]) : null;
      },
      remove(O) {
        this.write(O, "", Date.now() - 864e5);
      }
    }
  ) : (
    // Non-standard browser env (web workers, react-native) lack needed support.
    {
      write() {
      },
      read() {
        return null;
      },
      remove() {
      }
    }
  );
  function isAbsoluteURL(O) {
    return /^([a-z][a-z\d+\-.]*:)?\/\//i.test(O);
  }
  function combineURLs(O, _) {
    return _ ? O.replace(/\/?\/$/, "") + "/" + _.replace(/^\/+/, "") : O;
  }
  function buildFullPath(O, _) {
    return O && !isAbsoluteURL(_) ? combineURLs(O, _) : _;
  }
  const headersToObject = (O) => O instanceof AxiosHeaders ? { ...O } : O;
  function mergeConfig$1(O, _) {
    _ = _ || {};
    const H = {};
    function V(ne, ee, ae, oe) {
      return utils$2.isPlainObject(ne) && utils$2.isPlainObject(ee) ? utils$2.merge.call({ caseless: oe }, ne, ee) : utils$2.isPlainObject(ee) ? utils$2.merge({}, ee) : utils$2.isArray(ee) ? ee.slice() : ee;
    }
    function q(ne, ee, ae, oe) {
      if (utils$2.isUndefined(ee)) {
        if (!utils$2.isUndefined(ne))
          return V(void 0, ne, ae, oe);
      } else return V(ne, ee, ae, oe);
    }
    function G(ne, ee) {
      if (!utils$2.isUndefined(ee))
        return V(void 0, ee);
    }
    function J(ne, ee) {
      if (utils$2.isUndefined(ee)) {
        if (!utils$2.isUndefined(ne))
          return V(void 0, ne);
      } else return V(void 0, ee);
    }
    function Z(ne, ee, ae) {
      if (ae in _)
        return V(ne, ee);
      if (ae in O)
        return V(void 0, ne);
    }
    const te = {
      url: G,
      method: G,
      data: G,
      baseURL: J,
      transformRequest: J,
      transformResponse: J,
      paramsSerializer: J,
      timeout: J,
      timeoutMessage: J,
      withCredentials: J,
      withXSRFToken: J,
      adapter: J,
      responseType: J,
      xsrfCookieName: J,
      xsrfHeaderName: J,
      onUploadProgress: J,
      onDownloadProgress: J,
      decompress: J,
      maxContentLength: J,
      maxBodyLength: J,
      beforeRedirect: J,
      transport: J,
      httpAgent: J,
      httpsAgent: J,
      cancelToken: J,
      socketPath: J,
      responseEncoding: J,
      validateStatus: Z,
      headers: (ne, ee, ae) => q(headersToObject(ne), headersToObject(ee), ae, !0)
    };
    return utils$2.forEach(Object.keys(Object.assign({}, O, _)), function(ee) {
      const ae = te[ee] || q, oe = ae(O[ee], _[ee], ee);
      utils$2.isUndefined(oe) && ae !== Z || (H[ee] = oe);
    }), H;
  }
  const resolveConfig = (O) => {
    const _ = mergeConfig$1({}, O);
    let { data: H, withXSRFToken: V, xsrfHeaderName: q, xsrfCookieName: G, headers: J, auth: Z } = _;
    _.headers = J = AxiosHeaders.from(J), _.url = buildURL(buildFullPath(_.baseURL, _.url), O.params, O.paramsSerializer), Z && J.set(
      "Authorization",
      "Basic " + btoa((Z.username || "") + ":" + (Z.password ? unescape(encodeURIComponent(Z.password)) : ""))
    );
    let te;
    if (utils$2.isFormData(H)) {
      if (platform.hasStandardBrowserEnv || platform.hasStandardBrowserWebWorkerEnv)
        J.setContentType(void 0);
      else if ((te = J.getContentType()) !== !1) {
        const [ne, ...ee] = te ? te.split(";").map((ae) => ae.trim()).filter(Boolean) : [];
        J.setContentType([ne || "multipart/form-data", ...ee].join("; "));
      }
    }
    if (platform.hasStandardBrowserEnv && (V && utils$2.isFunction(V) && (V = V(_)), V || V !== !1 && isURLSameOrigin(_.url))) {
      const ne = q && G && cookies.read(G);
      ne && J.set(q, ne);
    }
    return _;
  }, isXHRAdapterSupported = typeof XMLHttpRequest < "u", xhrAdapter = isXHRAdapterSupported && function(O) {
    return new Promise(function(H, V) {
      const q = resolveConfig(O);
      let G = q.data;
      const J = AxiosHeaders.from(q.headers).normalize();
      let { responseType: Z, onUploadProgress: te, onDownloadProgress: ne } = q, ee, ae, oe, se, ce;
      function le() {
        se && se(), ce && ce(), q.cancelToken && q.cancelToken.unsubscribe(ee), q.signal && q.signal.removeEventListener("abort", ee);
      }
      let ue = new XMLHttpRequest();
      ue.open(q.method.toUpperCase(), q.url, !0), ue.timeout = q.timeout;
      function de() {
        if (!ue)
          return;
        const he = AxiosHeaders.from(
          "getAllResponseHeaders" in ue && ue.getAllResponseHeaders()
        ), pe = {
          data: !Z || Z === "text" || Z === "json" ? ue.responseText : ue.response,
          status: ue.status,
          statusText: ue.statusText,
          headers: he,
          config: O,
          request: ue
        };
        settle(function(ge) {
          H(ge), le();
        }, function(ge) {
          V(ge), le();
        }, pe), ue = null;
      }
      "onloadend" in ue ? ue.onloadend = de : ue.onreadystatechange = function() {
        !ue || ue.readyState !== 4 || ue.status === 0 && !(ue.responseURL && ue.responseURL.indexOf("file:") === 0) || setTimeout(de);
      }, ue.onabort = function() {
        ue && (V(new AxiosError("Request aborted", AxiosError.ECONNABORTED, O, ue)), ue = null);
      }, ue.onerror = function() {
        V(new AxiosError("Network Error", AxiosError.ERR_NETWORK, O, ue)), ue = null;
      }, ue.ontimeout = function() {
        let me = q.timeout ? "timeout of " + q.timeout + "ms exceeded" : "timeout exceeded";
        const pe = q.transitional || transitionalDefaults;
        q.timeoutErrorMessage && (me = q.timeoutErrorMessage), V(new AxiosError(
          me,
          pe.clarifyTimeoutError ? AxiosError.ETIMEDOUT : AxiosError.ECONNABORTED,
          O,
          ue
        )), ue = null;
      }, G === void 0 && J.setContentType(null), "setRequestHeader" in ue && utils$2.forEach(J.toJSON(), function(me, pe) {
        ue.setRequestHeader(pe, me);
      }), utils$2.isUndefined(q.withCredentials) || (ue.withCredentials = !!q.withCredentials), Z && Z !== "json" && (ue.responseType = q.responseType), ne && ([oe, ce] = progressEventReducer(ne, !0), ue.addEventListener("progress", oe)), te && ue.upload && ([ae, se] = progressEventReducer(te), ue.upload.addEventListener("progress", ae), ue.upload.addEventListener("loadend", se)), (q.cancelToken || q.signal) && (ee = (he) => {
        ue && (V(!he || he.type ? new CanceledError(null, O, ue) : he), ue.abort(), ue = null);
      }, q.cancelToken && q.cancelToken.subscribe(ee), q.signal && (q.signal.aborted ? ee() : q.signal.addEventListener("abort", ee)));
      const fe = parseProtocol(q.url);
      if (fe && platform.protocols.indexOf(fe) === -1) {
        V(new AxiosError("Unsupported protocol " + fe + ":", AxiosError.ERR_BAD_REQUEST, O));
        return;
      }
      ue.send(G || null);
    });
  }, composeSignals = (O, _) => {
    const { length: H } = O = O ? O.filter(Boolean) : [];
    if (_ || H) {
      let V = new AbortController(), q;
      const G = function(ne) {
        if (!q) {
          q = !0, Z();
          const ee = ne instanceof Error ? ne : this.reason;
          V.abort(ee instanceof AxiosError ? ee : new CanceledError(ee instanceof Error ? ee.message : ee));
        }
      };
      let J = _ && setTimeout(() => {
        J = null, G(new AxiosError(`timeout ${_} of ms exceeded`, AxiosError.ETIMEDOUT));
      }, _);
      const Z = () => {
        O && (J && clearTimeout(J), J = null, O.forEach((ne) => {
          ne.unsubscribe ? ne.unsubscribe(G) : ne.removeEventListener("abort", G);
        }), O = null);
      };
      O.forEach((ne) => ne.addEventListener("abort", G));
      const { signal: te } = V;
      return te.unsubscribe = () => utils$2.asap(Z), te;
    }
  }, streamChunk = function* (O, _) {
    let H = O.byteLength;
    if (H < _) {
      yield O;
      return;
    }
    let V = 0, q;
    for (; V < H; )
      q = V + _, yield O.slice(V, q), V = q;
  }, readBytes = async function* (O, _) {
    for await (const H of readStream(O))
      yield* streamChunk(H, _);
  }, readStream = async function* (O) {
    if (O[Symbol.asyncIterator]) {
      yield* O;
      return;
    }
    const _ = O.getReader();
    try {
      for (; ; ) {
        const { done: H, value: V } = await _.read();
        if (H)
          break;
        yield V;
      }
    } finally {
      await _.cancel();
    }
  }, trackStream = (O, _, H, V) => {
    const q = readBytes(O, _);
    let G = 0, J, Z = (te) => {
      J || (J = !0, V && V(te));
    };
    return new ReadableStream({
      async pull(te) {
        try {
          const { done: ne, value: ee } = await q.next();
          if (ne) {
            Z(), te.close();
            return;
          }
          let ae = ee.byteLength;
          if (H) {
            let oe = G += ae;
            H(oe);
          }
          te.enqueue(new Uint8Array(ee));
        } catch (ne) {
          throw Z(ne), ne;
        }
      },
      cancel(te) {
        return Z(te), q.return();
      }
    }, {
      highWaterMark: 2
    });
  }, isFetchSupported = typeof fetch == "function" && typeof Request == "function" && typeof Response == "function", isReadableStreamSupported = isFetchSupported && typeof ReadableStream == "function", encodeText = isFetchSupported && (typeof TextEncoder == "function" ? /* @__PURE__ */ ((O) => (_) => O.encode(_))(new TextEncoder()) : async (O) => new Uint8Array(await new Response(O).arrayBuffer())), test$1 = (O, ..._) => {
    try {
      return !!O(..._);
    } catch {
      return !1;
    }
  }, supportsRequestStream = isReadableStreamSupported && test$1(() => {
    let O = !1;
    const _ = new Request(platform.origin, {
      body: new ReadableStream(),
      method: "POST",
      get duplex() {
        return O = !0, "half";
      }
    }).headers.has("Content-Type");
    return O && !_;
  }), DEFAULT_CHUNK_SIZE = 64 * 1024, supportsResponseStream = isReadableStreamSupported && test$1(() => utils$2.isReadableStream(new Response("").body)), resolvers = {
    stream: supportsResponseStream && ((O) => O.body)
  };
  isFetchSupported && ((O) => {
    ["text", "arrayBuffer", "blob", "formData", "stream"].forEach((_) => {
      !resolvers[_] && (resolvers[_] = utils$2.isFunction(O[_]) ? (H) => H[_]() : (H, V) => {
        throw new AxiosError(`Response type '${_}' is not supported`, AxiosError.ERR_NOT_SUPPORT, V);
      });
    });
  })(new Response());
  const getBodyLength = async (O) => {
    if (O == null)
      return 0;
    if (utils$2.isBlob(O))
      return O.size;
    if (utils$2.isSpecCompliantForm(O))
      return (await new Request(platform.origin, {
        method: "POST",
        body: O
      }).arrayBuffer()).byteLength;
    if (utils$2.isArrayBufferView(O) || utils$2.isArrayBuffer(O))
      return O.byteLength;
    if (utils$2.isURLSearchParams(O) && (O = O + ""), utils$2.isString(O))
      return (await encodeText(O)).byteLength;
  }, resolveBodyLength = async (O, _) => {
    const H = utils$2.toFiniteNumber(O.getContentLength());
    return H ?? getBodyLength(_);
  }, fetchAdapter = isFetchSupported && (async (O) => {
    let {
      url: _,
      method: H,
      data: V,
      signal: q,
      cancelToken: G,
      timeout: J,
      onDownloadProgress: Z,
      onUploadProgress: te,
      responseType: ne,
      headers: ee,
      withCredentials: ae = "same-origin",
      fetchOptions: oe
    } = resolveConfig(O);
    ne = ne ? (ne + "").toLowerCase() : "text";
    let se = composeSignals([q, G && G.toAbortSignal()], J), ce;
    const le = se && se.unsubscribe && (() => {
      se.unsubscribe();
    });
    let ue;
    try {
      if (te && supportsRequestStream && H !== "get" && H !== "head" && (ue = await resolveBodyLength(ee, V)) !== 0) {
        let pe = new Request(_, {
          method: "POST",
          body: V,
          duplex: "half"
        }), ye;
        if (utils$2.isFormData(V) && (ye = pe.headers.get("content-type")) && ee.setContentType(ye), pe.body) {
          const [ge, ve] = progressEventDecorator(
            ue,
            progressEventReducer(asyncDecorator(te))
          );
          V = trackStream(pe.body, DEFAULT_CHUNK_SIZE, ge, ve);
        }
      }
      utils$2.isString(ae) || (ae = ae ? "include" : "omit");
      const de = "credentials" in Request.prototype;
      ce = new Request(_, {
        ...oe,
        signal: se,
        method: H.toUpperCase(),
        headers: ee.normalize().toJSON(),
        body: V,
        duplex: "half",
        credentials: de ? ae : void 0
      });
      let fe = await fetch(ce);
      const he = supportsResponseStream && (ne === "stream" || ne === "response");
      if (supportsResponseStream && (Z || he && le)) {
        const pe = {};
        ["status", "statusText", "headers"].forEach((be) => {
          pe[be] = fe[be];
        });
        const ye = utils$2.toFiniteNumber(fe.headers.get("content-length")), [ge, ve] = Z && progressEventDecorator(
          ye,
          progressEventReducer(asyncDecorator(Z), !0)
        ) || [];
        fe = new Response(
          trackStream(fe.body, DEFAULT_CHUNK_SIZE, ge, () => {
            ve && ve(), le && le();
          }),
          pe
        );
      }
      ne = ne || "text";
      let me = await resolvers[utils$2.findKey(resolvers, ne) || "text"](fe, O);
      return !he && le && le(), await new Promise((pe, ye) => {
        settle(pe, ye, {
          data: me,
          headers: AxiosHeaders.from(fe.headers),
          status: fe.status,
          statusText: fe.statusText,
          config: O,
          request: ce
        });
      });
    } catch (de) {
      throw le && le(), de && de.name === "TypeError" && /fetch/i.test(de.message) ? Object.assign(
        new AxiosError("Network Error", AxiosError.ERR_NETWORK, O, ce),
        {
          cause: de.cause || de
        }
      ) : AxiosError.from(de, de && de.code, O, ce);
    }
  }), knownAdapters = {
    http: httpAdapter,
    xhr: xhrAdapter,
    fetch: fetchAdapter
  };
  utils$2.forEach(knownAdapters, (O, _) => {
    if (O) {
      try {
        Object.defineProperty(O, "name", { value: _ });
      } catch {
      }
      Object.defineProperty(O, "adapterName", { value: _ });
    }
  });
  const renderReason = (O) => `- ${O}`, isResolvedHandle = (O) => utils$2.isFunction(O) || O === null || O === !1, adapters = {
    getAdapter: (O) => {
      O = utils$2.isArray(O) ? O : [O];
      const { length: _ } = O;
      let H, V;
      const q = {};
      for (let G = 0; G < _; G++) {
        H = O[G];
        let J;
        if (V = H, !isResolvedHandle(H) && (V = knownAdapters[(J = String(H)).toLowerCase()], V === void 0))
          throw new AxiosError(`Unknown adapter '${J}'`);
        if (V)
          break;
        q[J || "#" + G] = V;
      }
      if (!V) {
        const G = Object.entries(q).map(
          ([Z, te]) => `adapter ${Z} ` + (te === !1 ? "is not supported by the environment" : "is not available in the build")
        );
        let J = _ ? G.length > 1 ? `since :
` + G.map(renderReason).join(`
`) : " " + renderReason(G[0]) : "as no adapter specified";
        throw new AxiosError(
          "There is no suitable adapter to dispatch the request " + J,
          "ERR_NOT_SUPPORT"
        );
      }
      return V;
    },
    adapters: knownAdapters
  };
  function throwIfCancellationRequested(O) {
    if (O.cancelToken && O.cancelToken.throwIfRequested(), O.signal && O.signal.aborted)
      throw new CanceledError(null, O);
  }
  function dispatchRequest(O) {
    return throwIfCancellationRequested(O), O.headers = AxiosHeaders.from(O.headers), O.data = transformData.call(
      O,
      O.transformRequest
    ), ["post", "put", "patch"].indexOf(O.method) !== -1 && O.headers.setContentType("application/x-www-form-urlencoded", !1), adapters.getAdapter(O.adapter || defaults$7.adapter)(O).then(function(V) {
      return throwIfCancellationRequested(O), V.data = transformData.call(
        O,
        O.transformResponse,
        V
      ), V.headers = AxiosHeaders.from(V.headers), V;
    }, function(V) {
      return isCancel(V) || (throwIfCancellationRequested(O), V && V.response && (V.response.data = transformData.call(
        O,
        O.transformResponse,
        V.response
      ), V.response.headers = AxiosHeaders.from(V.response.headers))), Promise.reject(V);
    });
  }
  const VERSION$2 = "1.7.9", validators$1 = {};
  ["object", "boolean", "number", "function", "string", "symbol"].forEach((O, _) => {
    validators$1[O] = function(V) {
      return typeof V === O || "a" + (_ < 1 ? "n " : " ") + O;
    };
  });
  const deprecatedWarnings = {};
  validators$1.transitional = function(_, H, V) {
    function q(G, J) {
      return "[Axios v" + VERSION$2 + "] Transitional option '" + G + "'" + J + (V ? ". " + V : "");
    }
    return (G, J, Z) => {
      if (_ === !1)
        throw new AxiosError(
          q(J, " has been removed" + (H ? " in " + H : "")),
          AxiosError.ERR_DEPRECATED
        );
      return H && !deprecatedWarnings[J] && (deprecatedWarnings[J] = !0, console.warn(
        q(
          J,
          " has been deprecated since v" + H + " and will be removed in the near future"
        )
      )), _ ? _(G, J, Z) : !0;
    };
  };
  validators$1.spelling = function(_) {
    return (H, V) => (console.warn(`${V} is likely a misspelling of ${_}`), !0);
  };
  function assertOptions(O, _, H) {
    if (typeof O != "object")
      throw new AxiosError("options must be an object", AxiosError.ERR_BAD_OPTION_VALUE);
    const V = Object.keys(O);
    let q = V.length;
    for (; q-- > 0; ) {
      const G = V[q], J = _[G];
      if (J) {
        const Z = O[G], te = Z === void 0 || J(Z, G, O);
        if (te !== !0)
          throw new AxiosError("option " + G + " must be " + te, AxiosError.ERR_BAD_OPTION_VALUE);
        continue;
      }
      if (H !== !0)
        throw new AxiosError("Unknown option " + G, AxiosError.ERR_BAD_OPTION);
    }
  }
  const validator$1 = {
    assertOptions,
    validators: validators$1
  }, validators = validator$1.validators;
  class Axios {
    constructor(_) {
      this.defaults = _, this.interceptors = {
        request: new InterceptorManager(),
        response: new InterceptorManager()
      };
    }
    /**
     * Dispatch a request
     *
     * @param {String|Object} configOrUrl The config specific for this request (merged with this.defaults)
     * @param {?Object} config
     *
     * @returns {Promise} The Promise to be fulfilled
     */
    async request(_, H) {
      try {
        return await this._request(_, H);
      } catch (V) {
        if (V instanceof Error) {
          let q = {};
          Error.captureStackTrace ? Error.captureStackTrace(q) : q = new Error();
          const G = q.stack ? q.stack.replace(/^.+\n/, "") : "";
          try {
            V.stack ? G && !String(V.stack).endsWith(G.replace(/^.+\n.+\n/, "")) && (V.stack += `
` + G) : V.stack = G;
          } catch {
          }
        }
        throw V;
      }
    }
    _request(_, H) {
      typeof _ == "string" ? (H = H || {}, H.url = _) : H = _ || {}, H = mergeConfig$1(this.defaults, H);
      const { transitional: V, paramsSerializer: q, headers: G } = H;
      V !== void 0 && validator$1.assertOptions(V, {
        silentJSONParsing: validators.transitional(validators.boolean),
        forcedJSONParsing: validators.transitional(validators.boolean),
        clarifyTimeoutError: validators.transitional(validators.boolean)
      }, !1), q != null && (utils$2.isFunction(q) ? H.paramsSerializer = {
        serialize: q
      } : validator$1.assertOptions(q, {
        encode: validators.function,
        serialize: validators.function
      }, !0)), validator$1.assertOptions(H, {
        baseUrl: validators.spelling("baseURL"),
        withXsrfToken: validators.spelling("withXSRFToken")
      }, !0), H.method = (H.method || this.defaults.method || "get").toLowerCase();
      let J = G && utils$2.merge(
        G.common,
        G[H.method]
      );
      G && utils$2.forEach(
        ["delete", "get", "head", "post", "put", "patch", "common"],
        (ce) => {
          delete G[ce];
        }
      ), H.headers = AxiosHeaders.concat(J, G);
      const Z = [];
      let te = !0;
      this.interceptors.request.forEach(function(le) {
        typeof le.runWhen == "function" && le.runWhen(H) === !1 || (te = te && le.synchronous, Z.unshift(le.fulfilled, le.rejected));
      });
      const ne = [];
      this.interceptors.response.forEach(function(le) {
        ne.push(le.fulfilled, le.rejected);
      });
      let ee, ae = 0, oe;
      if (!te) {
        const ce = [dispatchRequest.bind(this), void 0];
        for (ce.unshift.apply(ce, Z), ce.push.apply(ce, ne), oe = ce.length, ee = Promise.resolve(H); ae < oe; )
          ee = ee.then(ce[ae++], ce[ae++]);
        return ee;
      }
      oe = Z.length;
      let se = H;
      for (ae = 0; ae < oe; ) {
        const ce = Z[ae++], le = Z[ae++];
        try {
          se = ce(se);
        } catch (ue) {
          le.call(this, ue);
          break;
        }
      }
      try {
        ee = dispatchRequest.call(this, se);
      } catch (ce) {
        return Promise.reject(ce);
      }
      for (ae = 0, oe = ne.length; ae < oe; )
        ee = ee.then(ne[ae++], ne[ae++]);
      return ee;
    }
    getUri(_) {
      _ = mergeConfig$1(this.defaults, _);
      const H = buildFullPath(_.baseURL, _.url);
      return buildURL(H, _.params, _.paramsSerializer);
    }
  }
  utils$2.forEach(["delete", "get", "head", "options"], function(_) {
    Axios.prototype[_] = function(H, V) {
      return this.request(mergeConfig$1(V || {}, {
        method: _,
        url: H,
        data: (V || {}).data
      }));
    };
  });
  utils$2.forEach(["post", "put", "patch"], function(_) {
    function H(V) {
      return function(G, J, Z) {
        return this.request(mergeConfig$1(Z || {}, {
          method: _,
          headers: V ? {
            "Content-Type": "multipart/form-data"
          } : {},
          url: G,
          data: J
        }));
      };
    }
    Axios.prototype[_] = H(), Axios.prototype[_ + "Form"] = H(!0);
  });
  class CancelToken {
    constructor(_) {
      if (typeof _ != "function")
        throw new TypeError("executor must be a function.");
      let H;
      this.promise = new Promise(function(G) {
        H = G;
      });
      const V = this;
      this.promise.then((q) => {
        if (!V._listeners) return;
        let G = V._listeners.length;
        for (; G-- > 0; )
          V._listeners[G](q);
        V._listeners = null;
      }), this.promise.then = (q) => {
        let G;
        const J = new Promise((Z) => {
          V.subscribe(Z), G = Z;
        }).then(q);
        return J.cancel = function() {
          V.unsubscribe(G);
        }, J;
      }, _(function(G, J, Z) {
        V.reason || (V.reason = new CanceledError(G, J, Z), H(V.reason));
      });
    }
    /**
     * Throws a `CanceledError` if cancellation has been requested.
     */
    throwIfRequested() {
      if (this.reason)
        throw this.reason;
    }
    /**
     * Subscribe to the cancel signal
     */
    subscribe(_) {
      if (this.reason) {
        _(this.reason);
        return;
      }
      this._listeners ? this._listeners.push(_) : this._listeners = [_];
    }
    /**
     * Unsubscribe from the cancel signal
     */
    unsubscribe(_) {
      if (!this._listeners)
        return;
      const H = this._listeners.indexOf(_);
      H !== -1 && this._listeners.splice(H, 1);
    }
    toAbortSignal() {
      const _ = new AbortController(), H = (V) => {
        _.abort(V);
      };
      return this.subscribe(H), _.signal.unsubscribe = () => this.unsubscribe(H), _.signal;
    }
    /**
     * Returns an object that contains a new `CancelToken` and a function that, when called,
     * cancels the `CancelToken`.
     */
    static source() {
      let _;
      return {
        token: new CancelToken(function(q) {
          _ = q;
        }),
        cancel: _
      };
    }
  }
  function spread(O) {
    return function(H) {
      return O.apply(null, H);
    };
  }
  function isAxiosError(O) {
    return utils$2.isObject(O) && O.isAxiosError === !0;
  }
  const HttpStatusCode = {
    Continue: 100,
    SwitchingProtocols: 101,
    Processing: 102,
    EarlyHints: 103,
    Ok: 200,
    Created: 201,
    Accepted: 202,
    NonAuthoritativeInformation: 203,
    NoContent: 204,
    ResetContent: 205,
    PartialContent: 206,
    MultiStatus: 207,
    AlreadyReported: 208,
    ImUsed: 226,
    MultipleChoices: 300,
    MovedPermanently: 301,
    Found: 302,
    SeeOther: 303,
    NotModified: 304,
    UseProxy: 305,
    Unused: 306,
    TemporaryRedirect: 307,
    PermanentRedirect: 308,
    BadRequest: 400,
    Unauthorized: 401,
    PaymentRequired: 402,
    Forbidden: 403,
    NotFound: 404,
    MethodNotAllowed: 405,
    NotAcceptable: 406,
    ProxyAuthenticationRequired: 407,
    RequestTimeout: 408,
    Conflict: 409,
    Gone: 410,
    LengthRequired: 411,
    PreconditionFailed: 412,
    PayloadTooLarge: 413,
    UriTooLong: 414,
    UnsupportedMediaType: 415,
    RangeNotSatisfiable: 416,
    ExpectationFailed: 417,
    ImATeapot: 418,
    MisdirectedRequest: 421,
    UnprocessableEntity: 422,
    Locked: 423,
    FailedDependency: 424,
    TooEarly: 425,
    UpgradeRequired: 426,
    PreconditionRequired: 428,
    TooManyRequests: 429,
    RequestHeaderFieldsTooLarge: 431,
    UnavailableForLegalReasons: 451,
    InternalServerError: 500,
    NotImplemented: 501,
    BadGateway: 502,
    ServiceUnavailable: 503,
    GatewayTimeout: 504,
    HttpVersionNotSupported: 505,
    VariantAlsoNegotiates: 506,
    InsufficientStorage: 507,
    LoopDetected: 508,
    NotExtended: 510,
    NetworkAuthenticationRequired: 511
  };
  Object.entries(HttpStatusCode).forEach(([O, _]) => {
    HttpStatusCode[_] = O;
  });
  function createInstance(O) {
    const _ = new Axios(O), H = bind$3(Axios.prototype.request, _);
    return utils$2.extend(H, Axios.prototype, _, { allOwnKeys: !0 }), utils$2.extend(H, _, null, { allOwnKeys: !0 }), H.create = function(q) {
      return createInstance(mergeConfig$1(O, q));
    }, H;
  }
  const axios = createInstance(defaults$7);
  axios.Axios = Axios;
  axios.CanceledError = CanceledError;
  axios.CancelToken = CancelToken;
  axios.isCancel = isCancel;
  axios.VERSION = VERSION$2;
  axios.toFormData = toFormData;
  axios.AxiosError = AxiosError;
  axios.Cancel = axios.CanceledError;
  axios.all = function(_) {
    return Promise.all(_);
  };
  axios.spread = spread;
  axios.isAxiosError = isAxiosError;
  axios.mergeConfig = mergeConfig$1;
  axios.AxiosHeaders = AxiosHeaders;
  axios.formToJSON = (O) => formDataToJSON(utils$2.isHTMLForm(O) ? new FormData(O) : O);
  axios.getAdapter = adapters.getAdapter;
  axios.HttpStatusCode = HttpStatusCode;
  axios.default = axios;
  var commonjsGlobal = typeof globalThis < "u" ? globalThis : typeof window < "u" ? window : typeof global < "u" ? global : typeof self < "u" ? self : {};
  function getDefaultExportFromCjs$1(O) {
    return O && O.__esModule && Object.prototype.hasOwnProperty.call(O, "default") ? O.default : O;
  }
  function getAugmentedNamespace(O) {
    if (O.__esModule) return O;
    var _ = O.default;
    if (typeof _ == "function") {
      var H = function V() {
        return this instanceof V ? Reflect.construct(_, arguments, this.constructor) : _.apply(this, arguments);
      };
      H.prototype = _.prototype;
    } else H = {};
    return Object.defineProperty(H, "__esModule", { value: !0 }), Object.keys(O).forEach(function(V) {
      var q = Object.getOwnPropertyDescriptor(O, V);
      Object.defineProperty(H, V, q.get ? q : {
        enumerable: !0,
        get: function() {
          return O[V];
        }
      });
    }), H;
  }
  var FileSaver_min$1 = { exports: {} }, FileSaver_min = FileSaver_min$1.exports, hasRequiredFileSaver_min;
  function requireFileSaver_min() {
    return hasRequiredFileSaver_min || (hasRequiredFileSaver_min = 1, function(O, _) {
      (function(H, V) {
        V();
      })(FileSaver_min, function() {
        function H(ne, ee) {
          return typeof ee > "u" ? ee = { autoBom: !1 } : typeof ee != "object" && (console.warn("Deprecated: Expected third argument to be a object"), ee = { autoBom: !ee }), ee.autoBom && /^\s*(?:text\/\S*|application\/xml|\S*\/\S*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(ne.type) ? new Blob(["\uFEFF", ne], { type: ne.type }) : ne;
        }
        function V(ne, ee, ae) {
          var oe = new XMLHttpRequest();
          oe.open("GET", ne), oe.responseType = "blob", oe.onload = function() {
            te(oe.response, ee, ae);
          }, oe.onerror = function() {
            console.error("could not download file");
          }, oe.send();
        }
        function q(ne) {
          var ee = new XMLHttpRequest();
          ee.open("HEAD", ne, !1);
          try {
            ee.send();
          } catch {
          }
          return 200 <= ee.status && 299 >= ee.status;
        }
        function G(ne) {
          try {
            ne.dispatchEvent(new MouseEvent("click"));
          } catch {
            var ee = document.createEvent("MouseEvents");
            ee.initMouseEvent("click", !0, !0, window, 0, 0, 0, 80, 20, !1, !1, !1, !1, 0, null), ne.dispatchEvent(ee);
          }
        }
        var J = typeof window == "object" && window.window === window ? window : typeof self == "object" && self.self === self ? self : typeof commonjsGlobal == "object" && commonjsGlobal.global === commonjsGlobal ? commonjsGlobal : void 0, Z = J.navigator && /Macintosh/.test(navigator.userAgent) && /AppleWebKit/.test(navigator.userAgent) && !/Safari/.test(navigator.userAgent), te = J.saveAs || (typeof window != "object" || window !== J ? function() {
        } : "download" in HTMLAnchorElement.prototype && !Z ? function(ne, ee, ae) {
          var oe = J.URL || J.webkitURL, se = document.createElement("a");
          ee = ee || ne.name || "download", se.download = ee, se.rel = "noopener", typeof ne == "string" ? (se.href = ne, se.origin === location.origin ? G(se) : q(se.href) ? V(ne, ee, ae) : G(se, se.target = "_blank")) : (se.href = oe.createObjectURL(ne), setTimeout(function() {
            oe.revokeObjectURL(se.href);
          }, 4e4), setTimeout(function() {
            G(se);
          }, 0));
        } : "msSaveOrOpenBlob" in navigator ? function(ne, ee, ae) {
          if (ee = ee || ne.name || "download", typeof ne != "string") navigator.msSaveOrOpenBlob(H(ne, ae), ee);
          else if (q(ne)) V(ne, ee, ae);
          else {
            var oe = document.createElement("a");
            oe.href = ne, oe.target = "_blank", setTimeout(function() {
              G(oe);
            });
          }
        } : function(ne, ee, ae, oe) {
          if (oe = oe || open("", "_blank"), oe && (oe.document.title = oe.document.body.innerText = "downloading..."), typeof ne == "string") return V(ne, ee, ae);
          var se = ne.type === "application/octet-stream", ce = /constructor/i.test(J.HTMLElement) || J.safari, le = /CriOS\/[\d]+/.test(navigator.userAgent);
          if ((le || se && ce || Z) && typeof FileReader < "u") {
            var ue = new FileReader();
            ue.onloadend = function() {
              var he = ue.result;
              he = le ? he : he.replace(/^data:[^;]*;/, "data:attachment/file;"), oe ? oe.location.href = he : location = he, oe = null;
            }, ue.readAsDataURL(ne);
          } else {
            var de = J.URL || J.webkitURL, fe = de.createObjectURL(ne);
            oe ? oe.location = fe : location.href = fe, oe = null, setTimeout(function() {
              de.revokeObjectURL(fe);
            }, 4e4);
          }
        });
        J.saveAs = te.saveAs = te, O.exports = te;
      });
    }(FileSaver_min$1)), FileSaver_min$1.exports;
  }
  var FileSaver_minExports = requireFileSaver_min();
  const byteToHex = [];
  for (let O = 0; O < 256; ++O)
    byteToHex.push((O + 256).toString(16).slice(1));
  function unsafeStringify(O, _ = 0) {
    return (byteToHex[O[_ + 0]] + byteToHex[O[_ + 1]] + byteToHex[O[_ + 2]] + byteToHex[O[_ + 3]] + "-" + byteToHex[O[_ + 4]] + byteToHex[O[_ + 5]] + "-" + byteToHex[O[_ + 6]] + byteToHex[O[_ + 7]] + "-" + byteToHex[O[_ + 8]] + byteToHex[O[_ + 9]] + "-" + byteToHex[O[_ + 10]] + byteToHex[O[_ + 11]] + byteToHex[O[_ + 12]] + byteToHex[O[_ + 13]] + byteToHex[O[_ + 14]] + byteToHex[O[_ + 15]]).toLowerCase();
  }
  let getRandomValues;
  const rnds8 = new Uint8Array(16);
  function rng() {
    if (!getRandomValues) {
      if (typeof crypto > "u" || !crypto.getRandomValues)
        throw new Error("crypto.getRandomValues() not supported. See https://github.com/uuidjs/uuid#getrandomvalues-not-supported");
      getRandomValues = crypto.getRandomValues.bind(crypto);
    }
    return getRandomValues(rnds8);
  }
  const randomUUID = typeof crypto < "u" && crypto.randomUUID && crypto.randomUUID.bind(crypto), native = { randomUUID };
  function v4(O, _, H) {
    var q;
    if (native.randomUUID && !O)
      return native.randomUUID();
    O = O || {};
    const V = O.random ?? ((q = O.rng) == null ? void 0 : q.call(O)) ?? rng();
    if (V.length < 16)
      throw new Error("Random bytes length must be >= 16");
    return V[6] = V[6] & 15 | 64, V[8] = V[8] & 63 | 128, unsafeStringify(V);
  }
  const _hoisted_1$t = {
    key: 0,
    class: "text-sm text-ar-light-gray italic text-justify"
  }, _sfc_main$Q = /* @__PURE__ */ defineComponent({
    __name: "BlockWrapper",
    props: {
      figure: {},
      singleBlockEmbed: { type: Boolean },
      showOverflow: { type: Boolean }
    },
    setup(O) {
      const _ = O, { caption: H, count: V, captionType: q } = toRefs(_.figure);
      return (G, J) => (openBlock(), createElementBlock("div", {
        class: normalizeClass([
          "w-full relative flex flex-col justify-start items-center",
          { "h-iframe": G.singleBlockEmbed },
          // TODO - why does overflow-x-auto create auto-y overflow in `Compute` block?
          { "overflow-x-auto": !G.showOverflow },
          { "overflow-visible": G.showOverflow }
        ])
      }, [
        renderSlot(G.$slots, "default"),
        unref(H) ? (openBlock(), createElementBlock("div", _hoisted_1$t, [
          createBaseVNode("b", null, toDisplayString(unref(q)) + " " + toDisplayString(unref(V)), 1),
          createTextVNode(" " + toDisplayString(unref(H)), 1)
        ])) : createCommentVNode("", !0)
      ], 2));
    }
  }), _hoisted_1$s = ["data-cy"], _sfc_main$P = /* @__PURE__ */ defineComponent({
    __name: "ARButton",
    props: {
      icon: {},
      dataCy: {}
    },
    setup(O) {
      const _ = O;
      return (H, V) => (openBlock(), createElementBlock("button", {
        type: "button",
        class: normalizeClass(["ar-btn-sm disabled:opacity-50"]),
        "data-cy": _.dataCy
      }, [
        _.icon ? (openBlock(), createElementBlock("i", {
          key: 0,
          class: normalizeClass(`mr-2 ${_.icon}`)
        }, null, 2)) : createCommentVNode("", !0),
        renderSlot(H.$slots, "default")
      ], 8, _hoisted_1$s));
    }
  }), _hoisted_1$r = {
    class: "w-full p-4 bg-gray-50 shadow",
    "data-cy": "block-file"
  }, _hoisted_2$d = { class: "flex items-center" }, _hoisted_3$b = { class: "text-base" }, _hoisted_4$9 = { class: "ml-auto" }, _sfc_main$O = /* @__PURE__ */ defineComponent({
    __name: "Attachment",
    props: {
      downloadFile: { type: Function },
      filename: {},
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      return (_, H) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: _.figure,
        "single-block-embed": _.singleBlockEmbed
      }, {
        default: withCtx(() => [
          createBaseVNode("div", _hoisted_1$r, [
            createBaseVNode("div", _hoisted_2$d, [
              H[1] || (H[1] = createBaseVNode("svg", {
                class: "flex-shrink-0 h-5 w-5 mr-2 text-gray-400",
                "x-description": "Heroicon name: solid/paper-clip",
                xmlns: "http://www.w3.org/2000/svg",
                viewBox: "0 0 20 20",
                fill: "currentColor",
                "aria-hidden": "true"
              }, [
                createBaseVNode("path", {
                  "fill-rule": "evenodd",
                  "clip-rule": "evenodd",
                  d: "M8 4a3 3 0 00-3 3v4a5 5 0 0010 0V7a1 1 0 112 0v4a7 7 0 11-14 0V7a5 5 0 0110 0v4a3 3 0 11-6 0V7a1 1 0 012 0v4a1 1 0 102 0V7a3 3 0 00-3-3z"
                })
              ], -1)),
              createBaseVNode("span", _hoisted_3$b, toDisplayString(_.filename), 1),
              createBaseVNode("div", _hoisted_4$9, [
                createVNode(_sfc_main$P, {
                  icon: "fa fa-download",
                  onClick: _.downloadFile
                }, {
                  default: withCtx(() => H[0] || (H[0] = [
                    createTextVNode(" Download ")
                  ])),
                  _: 1
                }, 8, ["onClick"])
              ])
            ])
          ])
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), _hoisted_1$q = {
    key: 0,
    class: "flex justify-start align-center"
  }, _hoisted_2$c = {
    key: 1,
    class: "pl-1 text-gray-500"
  }, _sfc_main$N = /* @__PURE__ */ defineComponent({
    __name: "BigNumberArrow",
    props: {
      isPositiveIntent: { type: Boolean },
      isUpwardChange: { type: Boolean },
      change: {},
      prevValue: {}
    },
    setup(O) {
      const _ = O, H = computed(
        () => _.isPositiveIntent ? "bg-green-100" : "bg-red-100"
      ), V = computed(
        () => _.isPositiveIntent ? "text-green-800" : "text-red-800"
      );
      return (q, G) => q.change ? (openBlock(), createElementBlock("div", _hoisted_1$q, [
        q.change ? (openBlock(), createElementBlock("div", {
          key: 0,
          class: normalizeClass([
            `items-baseline px-2.5 py-0.5 rounded-lg text-sm font-medium leading-5 ${H.value} ${V.value}`
          ])
        }, toDisplayString(q.isUpwardChange ? "+" : "-") + toDisplayString(q.change), 3)) : createCommentVNode("", !0),
        q.prevValue ? (openBlock(), createElementBlock("div", _hoisted_2$c, "from " + toDisplayString(q.prevValue), 1)) : createCommentVNode("", !0)
      ])) : createCommentVNode("", !0);
    }
  }), _hoisted_1$p = {
    "data-cy": "block-bignumber",
    class: "rounded-md bg-white overflow-hidden border border-gray-300 w-full"
  }, _hoisted_2$b = { class: "px-4 py-5 sm:p-6" }, _hoisted_3$a = { class: "text-base leading-6 font-normal text-gray-900 truncate" }, _hoisted_4$8 = { class: "mt-1 flex justify-between items-baseline md:block lg:flex" }, _hoisted_5$5 = { class: "flex items-baseline text-4xl leading-8 font-semibold text-ar-accent" }, _sfc_main$M = /* @__PURE__ */ defineComponent({
    __name: "BigNumber",
    props: {
      heading: {},
      value: {},
      isPositiveIntent: { type: Boolean },
      isUpwardChange: { type: Boolean },
      figure: {},
      singleBlockEmbed: { type: Boolean },
      prevValue: {},
      change: {}
    },
    setup(O) {
      return (_, H) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: _.figure,
        "single-block-embed": _.singleBlockEmbed
      }, {
        default: withCtx(() => [
          createBaseVNode("div", _hoisted_1$p, [
            createBaseVNode("div", _hoisted_2$b, [
              createBaseVNode("dl", null, [
                createBaseVNode("dt", _hoisted_3$a, toDisplayString(_.heading), 1),
                createBaseVNode("dd", _hoisted_4$8, [
                  createBaseVNode("div", _hoisted_5$5, toDisplayString(_.value), 1),
                  createVNode(_sfc_main$N, {
                    class: "inline-flex lg:inline-flex sm:hidden",
                    "is-positive-intent": _.isPositiveIntent,
                    "is-upward-change": _.isUpwardChange,
                    "prev-value": _.prevValue,
                    change: _.change
                  }, null, 8, ["is-positive-intent", "is-upward-change", "prev-value", "change"])
                ])
              ])
            ])
          ])
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), _hoisted_1$o = {
    "data-cy": "block-bignumber",
    class: "bg-white overflow-hidden border border-gray-300 rounded-lg w-full"
  }, _hoisted_2$a = { class: "px-4 py-5 sm:p-6" }, _hoisted_3$9 = { class: "text-base leading-6 font-normal text-gray-900 truncate" }, _hoisted_4$7 = { class: "mt-1 text-4xl leading-9 font-semibold text-gray-900" }, _sfc_main$L = /* @__PURE__ */ defineComponent({
    __name: "BigNumberSimple",
    props: {
      heading: {},
      value: {},
      figure: {}
    },
    setup(O) {
      return (_, H) => (openBlock(), createBlock(_sfc_main$Q, { figure: _.figure }, {
        default: withCtx(() => [
          createBaseVNode("div", _hoisted_1$o, [
            createBaseVNode("div", _hoisted_2$a, [
              createBaseVNode("dl", null, [
                createBaseVNode("dt", _hoisted_3$9, toDisplayString(_.heading), 1),
                createBaseVNode("dd", _hoisted_4$7, toDisplayString(_.value), 1)
              ])
            ])
          ])
        ]),
        _: 1
      }, 8, ["figure"]));
    }
  }), _sfc_main$K = /* @__PURE__ */ defineComponent({
    __name: "BokehConnector",
    props: {
      fetchAssetData: {},
      responsive: { type: Boolean },
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = /* @__PURE__ */ defineAsyncComponent(() => Promise.resolve().then(() => Bokeh)), H = O, V = ref$1(null);
      return (async () => V.value = await H.fetchAssetData())(), (q, G) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: q.figure,
        "single-block-embed": q.singleBlockEmbed
      }, {
        default: withCtx(() => [
          V.value ? (openBlock(), createBlock(unref(_), {
            key: 0,
            "plot-json": V.value,
            responsive: q.responsive,
            "single-block-embed": q.singleBlockEmbed
          }, null, 8, ["plot-json", "responsive", "single-block-embed"])) : createCommentVNode("", !0)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  });
  var core$1, hasRequiredCore;
  function requireCore() {
    if (hasRequiredCore) return core$1;
    hasRequiredCore = 1;
    function O(Ht) {
      return Ht instanceof Map ? Ht.clear = Ht.delete = Ht.set = function() {
        throw new Error("map is read-only");
      } : Ht instanceof Set && (Ht.add = Ht.clear = Ht.delete = function() {
        throw new Error("set is read-only");
      }), Object.freeze(Ht), Object.getOwnPropertyNames(Ht).forEach((jt) => {
        const cn = Ht[jt], wn = typeof cn;
        (wn === "object" || wn === "function") && !Object.isFrozen(cn) && O(cn);
      }), Ht;
    }
    class _ {
      /**
       * @param {CompiledMode} mode
       */
      constructor(jt) {
        jt.data === void 0 && (jt.data = {}), this.data = jt.data, this.isMatchIgnored = !1;
      }
      ignoreMatch() {
        this.isMatchIgnored = !0;
      }
    }
    function H(Ht) {
      return Ht.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;").replace(/'/g, "&#x27;");
    }
    function V(Ht, ...jt) {
      const cn = /* @__PURE__ */ Object.create(null);
      for (const wn in Ht)
        cn[wn] = Ht[wn];
      return jt.forEach(function(wn) {
        for (const Rn in wn)
          cn[Rn] = wn[Rn];
      }), /** @type {T} */
      cn;
    }
    const q = "</span>", G = (Ht) => !!Ht.scope, J = (Ht, { prefix: jt }) => {
      if (Ht.startsWith("language:"))
        return Ht.replace("language:", "language-");
      if (Ht.includes(".")) {
        const cn = Ht.split(".");
        return [
          `${jt}${cn.shift()}`,
          ...cn.map((wn, Rn) => `${wn}${"_".repeat(Rn + 1)}`)
        ].join(" ");
      }
      return `${jt}${Ht}`;
    };
    class Z {
      /**
       * Creates a new HTMLRenderer
       *
       * @param {Tree} parseTree - the parse tree (must support `walk` API)
       * @param {{classPrefix: string}} options
       */
      constructor(jt, cn) {
        this.buffer = "", this.classPrefix = cn.classPrefix, jt.walk(this);
      }
      /**
       * Adds texts to the output stream
       *
       * @param {string} text */
      addText(jt) {
        this.buffer += H(jt);
      }
      /**
       * Adds a node open to the output stream (if needed)
       *
       * @param {Node} node */
      openNode(jt) {
        if (!G(jt)) return;
        const cn = J(
          jt.scope,
          { prefix: this.classPrefix }
        );
        this.span(cn);
      }
      /**
       * Adds a node close to the output stream (if needed)
       *
       * @param {Node} node */
      closeNode(jt) {
        G(jt) && (this.buffer += q);
      }
      /**
       * returns the accumulated buffer
      */
      value() {
        return this.buffer;
      }
      // helpers
      /**
       * Builds a span element
       *
       * @param {string} className */
      span(jt) {
        this.buffer += `<span class="${jt}">`;
      }
    }
    const te = (Ht = {}) => {
      const jt = { children: [] };
      return Object.assign(jt, Ht), jt;
    };
    class ne {
      constructor() {
        this.rootNode = te(), this.stack = [this.rootNode];
      }
      get top() {
        return this.stack[this.stack.length - 1];
      }
      get root() {
        return this.rootNode;
      }
      /** @param {Node} node */
      add(jt) {
        this.top.children.push(jt);
      }
      /** @param {string} scope */
      openNode(jt) {
        const cn = te({ scope: jt });
        this.add(cn), this.stack.push(cn);
      }
      closeNode() {
        if (this.stack.length > 1)
          return this.stack.pop();
      }
      closeAllNodes() {
        for (; this.closeNode(); ) ;
      }
      toJSON() {
        return JSON.stringify(this.rootNode, null, 4);
      }
      /**
       * @typedef { import("./html_renderer").Renderer } Renderer
       * @param {Renderer} builder
       */
      walk(jt) {
        return this.constructor._walk(jt, this.rootNode);
      }
      /**
       * @param {Renderer} builder
       * @param {Node} node
       */
      static _walk(jt, cn) {
        return typeof cn == "string" ? jt.addText(cn) : cn.children && (jt.openNode(cn), cn.children.forEach((wn) => this._walk(jt, wn)), jt.closeNode(cn)), jt;
      }
      /**
       * @param {Node} node
       */
      static _collapse(jt) {
        typeof jt != "string" && jt.children && (jt.children.every((cn) => typeof cn == "string") ? jt.children = [jt.children.join("")] : jt.children.forEach((cn) => {
          ne._collapse(cn);
        }));
      }
    }
    class ee extends ne {
      /**
       * @param {*} options
       */
      constructor(jt) {
        super(), this.options = jt;
      }
      /**
       * @param {string} text
       */
      addText(jt) {
        jt !== "" && this.add(jt);
      }
      /** @param {string} scope */
      startScope(jt) {
        this.openNode(jt);
      }
      endScope() {
        this.closeNode();
      }
      /**
       * @param {Emitter & {root: DataNode}} emitter
       * @param {string} name
       */
      __addSublanguage(jt, cn) {
        const wn = jt.root;
        cn && (wn.scope = `language:${cn}`), this.add(wn);
      }
      toHTML() {
        return new Z(this, this.options).value();
      }
      finalize() {
        return this.closeAllNodes(), !0;
      }
    }
    function ae(Ht) {
      return Ht ? typeof Ht == "string" ? Ht : Ht.source : null;
    }
    function oe(Ht) {
      return le("(?=", Ht, ")");
    }
    function se(Ht) {
      return le("(?:", Ht, ")*");
    }
    function ce(Ht) {
      return le("(?:", Ht, ")?");
    }
    function le(...Ht) {
      return Ht.map((cn) => ae(cn)).join("");
    }
    function ue(Ht) {
      const jt = Ht[Ht.length - 1];
      return typeof jt == "object" && jt.constructor === Object ? (Ht.splice(Ht.length - 1, 1), jt) : {};
    }
    function de(...Ht) {
      return "(" + (ue(Ht).capture ? "" : "?:") + Ht.map((wn) => ae(wn)).join("|") + ")";
    }
    function fe(Ht) {
      return new RegExp(Ht.toString() + "|").exec("").length - 1;
    }
    function he(Ht, jt) {
      const cn = Ht && Ht.exec(jt);
      return cn && cn.index === 0;
    }
    const me = /\[(?:[^\\\]]|\\.)*\]|\(\??|\\([1-9][0-9]*)|\\./;
    function pe(Ht, { joinWith: jt }) {
      let cn = 0;
      return Ht.map((wn) => {
        cn += 1;
        const Rn = cn;
        let Pn = ae(wn), Ie = "";
        for (; Pn.length > 0; ) {
          const rt = me.exec(Pn);
          if (!rt) {
            Ie += Pn;
            break;
          }
          Ie += Pn.substring(0, rt.index), Pn = Pn.substring(rt.index + rt[0].length), rt[0][0] === "\\" && rt[1] ? Ie += "\\" + String(Number(rt[1]) + Rn) : (Ie += rt[0], rt[0] === "(" && cn++);
        }
        return Ie;
      }).map((wn) => `(${wn})`).join(jt);
    }
    const ye = /\b\B/, ge = "[a-zA-Z]\\w*", ve = "[a-zA-Z_]\\w*", be = "\\b\\d+(\\.\\d+)?", Ce = "(-?)(\\b0[xX][a-fA-F0-9]+|(\\b\\d+(\\.\\d*)?|\\.\\d+)([eE][-+]?\\d+)?)", Se = "\\b(0b[01]+)", we = "!|!=|!==|%|%=|&|&&|&=|\\*|\\*=|\\+|\\+=|,|-|-=|/=|/|:|;|<<|<<=|<=|<|===|==|=|>>>=|>>=|>=|>>>|>>|>|\\?|\\[|\\{|\\(|\\^|\\^=|\\||\\|=|\\|\\||~", Ee = (Ht = {}) => {
      const jt = /^#![ ]*\//;
      return Ht.binary && (Ht.begin = le(
        jt,
        /.*\b/,
        Ht.binary,
        /\b.*/
      )), V({
        scope: "meta",
        begin: jt,
        end: /$/,
        relevance: 0,
        /** @type {ModeCallback} */
        "on:begin": (cn, wn) => {
          cn.index !== 0 && wn.ignoreMatch();
        }
      }, Ht);
    }, xe = {
      begin: "\\\\[\\s\\S]",
      relevance: 0
    }, Le = {
      scope: "string",
      begin: "'",
      end: "'",
      illegal: "\\n",
      contains: [xe]
    }, Qe = {
      scope: "string",
      begin: '"',
      end: '"',
      illegal: "\\n",
      contains: [xe]
    }, ke = {
      begin: /\b(a|an|the|are|I'm|isn't|don't|doesn't|won't|but|just|should|pretty|simply|enough|gonna|going|wtf|so|such|will|you|your|they|like|more)\b/
    }, Me = function(Ht, jt, cn = {}) {
      const wn = V(
        {
          scope: "comment",
          begin: Ht,
          end: jt,
          contains: []
        },
        cn
      );
      wn.contains.push({
        scope: "doctag",
        // hack to avoid the space from being included. the space is necessary to
        // match here to prevent the plain text rule below from gobbling up doctags
        begin: "[ ]*(?=(TODO|FIXME|NOTE|BUG|OPTIMIZE|HACK|XXX):)",
        end: /(TODO|FIXME|NOTE|BUG|OPTIMIZE|HACK|XXX):/,
        excludeBegin: !0,
        relevance: 0
      });
      const Rn = de(
        // list of common 1 and 2 letter words in English
        "I",
        "a",
        "is",
        "so",
        "us",
        "to",
        "at",
        "if",
        "in",
        "it",
        "on",
        // note: this is not an exhaustive list of contractions, just popular ones
        /[A-Za-z]+['](d|ve|re|ll|t|s|n)/,
        // contractions - can't we'd they're let's, etc
        /[A-Za-z]+[-][a-z]+/,
        // `no-way`, etc.
        /[A-Za-z][a-z]{2,}/
        // allow capitalized words at beginning of sentences
      );
      return wn.contains.push(
        {
          // TODO: how to include ", (, ) without breaking grammars that use these for
          // comment delimiters?
          // begin: /[ ]+([()"]?([A-Za-z'-]{3,}|is|a|I|so|us|[tT][oO]|at|if|in|it|on)[.]?[()":]?([.][ ]|[ ]|\))){3}/
          // ---
          // this tries to find sequences of 3 english words in a row (without any
          // "programming" type syntax) this gives us a strong signal that we've
          // TRULY found a comment - vs perhaps scanning with the wrong language.
          // It's possible to find something that LOOKS like the start of the
          // comment - but then if there is no readable text - good chance it is a
          // false match and not a comment.
          //
          // for a visual example please see:
          // https://github.com/highlightjs/highlight.js/issues/2827
          begin: le(
            /[ ]+/,
            // necessary to prevent us gobbling up doctags like /* @author Bob Mcgill */
            "(",
            Rn,
            /[.]?[:]?([.][ ]|[ ])/,
            "){3}"
          )
          // look for 3 words in a row
        }
      ), wn;
    }, Re = Me("//", "$"), De = Me("/\\*", "\\*/"), Pe = Me("#", "$"), $e = {
      scope: "number",
      begin: be,
      relevance: 0
    }, qe = {
      scope: "number",
      begin: Ce,
      relevance: 0
    }, Xe = {
      scope: "number",
      begin: Se,
      relevance: 0
    }, Ve = {
      scope: "regexp",
      begin: /\/(?=[^/\n]*\/)/,
      end: /\/[gimuy]*/,
      contains: [
        xe,
        {
          begin: /\[/,
          end: /\]/,
          relevance: 0,
          contains: [xe]
        }
      ]
    }, Ze = {
      scope: "title",
      begin: ge,
      relevance: 0
    }, Ge = {
      scope: "title",
      begin: ve,
      relevance: 0
    }, ze = {
      // excludes method names from keyword processing
      begin: "\\.\\s*" + ve,
      relevance: 0
    };
    var at = /* @__PURE__ */ Object.freeze({
      __proto__: null,
      APOS_STRING_MODE: Le,
      BACKSLASH_ESCAPE: xe,
      BINARY_NUMBER_MODE: Xe,
      BINARY_NUMBER_RE: Se,
      COMMENT: Me,
      C_BLOCK_COMMENT_MODE: De,
      C_LINE_COMMENT_MODE: Re,
      C_NUMBER_MODE: qe,
      C_NUMBER_RE: Ce,
      END_SAME_AS_BEGIN: function(Ht) {
        return Object.assign(
          Ht,
          {
            /** @type {ModeCallback} */
            "on:begin": (jt, cn) => {
              cn.data._beginMatch = jt[1];
            },
            /** @type {ModeCallback} */
            "on:end": (jt, cn) => {
              cn.data._beginMatch !== jt[1] && cn.ignoreMatch();
            }
          }
        );
      },
      HASH_COMMENT_MODE: Pe,
      IDENT_RE: ge,
      MATCH_NOTHING_RE: ye,
      METHOD_GUARD: ze,
      NUMBER_MODE: $e,
      NUMBER_RE: be,
      PHRASAL_WORDS_MODE: ke,
      QUOTE_STRING_MODE: Qe,
      REGEXP_MODE: Ve,
      RE_STARTERS_RE: we,
      SHEBANG: Ee,
      TITLE_MODE: Ze,
      UNDERSCORE_IDENT_RE: ve,
      UNDERSCORE_TITLE_MODE: Ge
    });
    function ct(Ht, jt) {
      Ht.input[Ht.index - 1] === "." && jt.ignoreMatch();
    }
    function et(Ht, jt) {
      Ht.className !== void 0 && (Ht.scope = Ht.className, delete Ht.className);
    }
    function dt(Ht, jt) {
      jt && Ht.beginKeywords && (Ht.begin = "\\b(" + Ht.beginKeywords.split(" ").join("|") + ")(?!\\.)(?=\\b|\\s)", Ht.__beforeBegin = ct, Ht.keywords = Ht.keywords || Ht.beginKeywords, delete Ht.beginKeywords, Ht.relevance === void 0 && (Ht.relevance = 0));
    }
    function Je(Ht, jt) {
      Array.isArray(Ht.illegal) && (Ht.illegal = de(...Ht.illegal));
    }
    function it(Ht, jt) {
      if (Ht.match) {
        if (Ht.begin || Ht.end) throw new Error("begin & end are not supported with match");
        Ht.begin = Ht.match, delete Ht.match;
      }
    }
    function st(Ht, jt) {
      Ht.relevance === void 0 && (Ht.relevance = 1);
    }
    const gt = (Ht, jt) => {
      if (!Ht.beforeMatch) return;
      if (Ht.starts) throw new Error("beforeMatch cannot be used with starts");
      const cn = Object.assign({}, Ht);
      Object.keys(Ht).forEach((wn) => {
        delete Ht[wn];
      }), Ht.keywords = cn.keywords, Ht.begin = le(cn.beforeMatch, oe(cn.begin)), Ht.starts = {
        relevance: 0,
        contains: [
          Object.assign(cn, { endsParent: !0 })
        ]
      }, Ht.relevance = 0, delete cn.beforeMatch;
    }, bt = [
      "of",
      "and",
      "for",
      "in",
      "not",
      "or",
      "if",
      "then",
      "parent",
      // common variable name
      "list",
      // common variable name
      "value"
      // common variable name
    ], yt = "keyword";
    function St(Ht, jt, cn = yt) {
      const wn = /* @__PURE__ */ Object.create(null);
      return typeof Ht == "string" ? Rn(cn, Ht.split(" ")) : Array.isArray(Ht) ? Rn(cn, Ht) : Object.keys(Ht).forEach(function(Pn) {
        Object.assign(
          wn,
          St(Ht[Pn], jt, Pn)
        );
      }), wn;
      function Rn(Pn, Ie) {
        jt && (Ie = Ie.map((rt) => rt.toLowerCase())), Ie.forEach(function(rt) {
          const ft = rt.split("|");
          wn[ft[0]] = [Pn, Lt(ft[0], ft[1])];
        });
      }
    }
    function Lt(Ht, jt) {
      return jt ? Number(jt) : Rt(Ht) ? 0 : 1;
    }
    function Rt(Ht) {
      return bt.includes(Ht.toLowerCase());
    }
    const vt = {}, $t = (Ht) => {
      console.error(Ht);
    }, Pt = (Ht, ...jt) => {
      console.log(`WARN: ${Ht}`, ...jt);
    }, Xt = (Ht, jt) => {
      vt[`${Ht}/${jt}`] || (console.log(`Deprecated as of ${Ht}. ${jt}`), vt[`${Ht}/${jt}`] = !0);
    }, Nt = new Error();
    function Gt(Ht, jt, { key: cn }) {
      let wn = 0;
      const Rn = Ht[cn], Pn = {}, Ie = {};
      for (let rt = 1; rt <= jt.length; rt++)
        Ie[rt + wn] = Rn[rt], Pn[rt + wn] = !0, wn += fe(jt[rt - 1]);
      Ht[cn] = Ie, Ht[cn]._emit = Pn, Ht[cn]._multi = !0;
    }
    function Ft(Ht) {
      if (Array.isArray(Ht.begin)) {
        if (Ht.skip || Ht.excludeBegin || Ht.returnBegin)
          throw $t("skip, excludeBegin, returnBegin not compatible with beginScope: {}"), Nt;
        if (typeof Ht.beginScope != "object" || Ht.beginScope === null)
          throw $t("beginScope must be object"), Nt;
        Gt(Ht, Ht.begin, { key: "beginScope" }), Ht.begin = pe(Ht.begin, { joinWith: "" });
      }
    }
    function Tt(Ht) {
      if (Array.isArray(Ht.end)) {
        if (Ht.skip || Ht.excludeEnd || Ht.returnEnd)
          throw $t("skip, excludeEnd, returnEnd not compatible with endScope: {}"), Nt;
        if (typeof Ht.endScope != "object" || Ht.endScope === null)
          throw $t("endScope must be object"), Nt;
        Gt(Ht, Ht.end, { key: "endScope" }), Ht.end = pe(Ht.end, { joinWith: "" });
      }
    }
    function It(Ht) {
      Ht.scope && typeof Ht.scope == "object" && Ht.scope !== null && (Ht.beginScope = Ht.scope, delete Ht.scope);
    }
    function Ct(Ht) {
      It(Ht), typeof Ht.beginScope == "string" && (Ht.beginScope = { _wrap: Ht.beginScope }), typeof Ht.endScope == "string" && (Ht.endScope = { _wrap: Ht.endScope }), Ft(Ht), Tt(Ht);
    }
    function Kt(Ht) {
      function jt(Ie, rt) {
        return new RegExp(
          ae(Ie),
          "m" + (Ht.case_insensitive ? "i" : "") + (Ht.unicodeRegex ? "u" : "") + (rt ? "g" : "")
        );
      }
      class cn {
        constructor() {
          this.matchIndexes = {}, this.regexes = [], this.matchAt = 1, this.position = 0;
        }
        // @ts-ignore
        addRule(rt, ft) {
          ft.position = this.position++, this.matchIndexes[this.matchAt] = ft, this.regexes.push([ft, rt]), this.matchAt += fe(rt) + 1;
        }
        compile() {
          this.regexes.length === 0 && (this.exec = () => null);
          const rt = this.regexes.map((ft) => ft[1]);
          this.matcherRe = jt(pe(rt, { joinWith: "|" }), !0), this.lastIndex = 0;
        }
        /** @param {string} s */
        exec(rt) {
          this.matcherRe.lastIndex = this.lastIndex;
          const ft = this.matcherRe.exec(rt);
          if (!ft)
            return null;
          const pt = ft.findIndex((Bt, At) => At > 0 && Bt !== void 0), wt = this.matchIndexes[pt];
          return ft.splice(0, pt), Object.assign(ft, wt);
        }
      }
      class wn {
        constructor() {
          this.rules = [], this.multiRegexes = [], this.count = 0, this.lastIndex = 0, this.regexIndex = 0;
        }
        // @ts-ignore
        getMatcher(rt) {
          if (this.multiRegexes[rt]) return this.multiRegexes[rt];
          const ft = new cn();
          return this.rules.slice(rt).forEach(([pt, wt]) => ft.addRule(pt, wt)), ft.compile(), this.multiRegexes[rt] = ft, ft;
        }
        resumingScanAtSamePosition() {
          return this.regexIndex !== 0;
        }
        considerAll() {
          this.regexIndex = 0;
        }
        // @ts-ignore
        addRule(rt, ft) {
          this.rules.push([rt, ft]), ft.type === "begin" && this.count++;
        }
        /** @param {string} s */
        exec(rt) {
          const ft = this.getMatcher(this.regexIndex);
          ft.lastIndex = this.lastIndex;
          let pt = ft.exec(rt);
          if (this.resumingScanAtSamePosition() && !(pt && pt.index === this.lastIndex)) {
            const wt = this.getMatcher(0);
            wt.lastIndex = this.lastIndex + 1, pt = wt.exec(rt);
          }
          return pt && (this.regexIndex += pt.position + 1, this.regexIndex === this.count && this.considerAll()), pt;
        }
      }
      function Rn(Ie) {
        const rt = new wn();
        return Ie.contains.forEach((ft) => rt.addRule(ft.begin, { rule: ft, type: "begin" })), Ie.terminatorEnd && rt.addRule(Ie.terminatorEnd, { type: "end" }), Ie.illegal && rt.addRule(Ie.illegal, { type: "illegal" }), rt;
      }
      function Pn(Ie, rt) {
        const ft = (
          /** @type CompiledMode */
          Ie
        );
        if (Ie.isCompiled) return ft;
        [
          et,
          // do this early so compiler extensions generally don't have to worry about
          // the distinction between match/begin
          it,
          Ct,
          gt
        ].forEach((wt) => wt(Ie, rt)), Ht.compilerExtensions.forEach((wt) => wt(Ie, rt)), Ie.__beforeBegin = null, [
          dt,
          // do this later so compiler extensions that come earlier have access to the
          // raw array if they wanted to perhaps manipulate it, etc.
          Je,
          // default to 1 relevance if not specified
          st
        ].forEach((wt) => wt(Ie, rt)), Ie.isCompiled = !0;
        let pt = null;
        return typeof Ie.keywords == "object" && Ie.keywords.$pattern && (Ie.keywords = Object.assign({}, Ie.keywords), pt = Ie.keywords.$pattern, delete Ie.keywords.$pattern), pt = pt || /\w+/, Ie.keywords && (Ie.keywords = St(Ie.keywords, Ht.case_insensitive)), ft.keywordPatternRe = jt(pt, !0), rt && (Ie.begin || (Ie.begin = /\B|\b/), ft.beginRe = jt(ft.begin), !Ie.end && !Ie.endsWithParent && (Ie.end = /\B|\b/), Ie.end && (ft.endRe = jt(ft.end)), ft.terminatorEnd = ae(ft.end) || "", Ie.endsWithParent && rt.terminatorEnd && (ft.terminatorEnd += (Ie.end ? "|" : "") + rt.terminatorEnd)), Ie.illegal && (ft.illegalRe = jt(
          /** @type {RegExp | string} */
          Ie.illegal
        )), Ie.contains || (Ie.contains = []), Ie.contains = [].concat(...Ie.contains.map(function(wt) {
          return Yt(wt === "self" ? Ie : wt);
        })), Ie.contains.forEach(function(wt) {
          Pn(
            /** @type Mode */
            wt,
            ft
          );
        }), Ie.starts && Pn(Ie.starts, rt), ft.matcher = Rn(ft), ft;
      }
      if (Ht.compilerExtensions || (Ht.compilerExtensions = []), Ht.contains && Ht.contains.includes("self"))
        throw new Error("ERR: contains `self` is not supported at the top-level of a language.  See documentation.");
      return Ht.classNameAliases = V(Ht.classNameAliases || {}), Pn(
        /** @type Mode */
        Ht
      );
    }
    function Vt(Ht) {
      return Ht ? Ht.endsWithParent || Vt(Ht.starts) : !1;
    }
    function Yt(Ht) {
      return Ht.variants && !Ht.cachedVariants && (Ht.cachedVariants = Ht.variants.map(function(jt) {
        return V(Ht, { variants: null }, jt);
      })), Ht.cachedVariants ? Ht.cachedVariants : Vt(Ht) ? V(Ht, { starts: Ht.starts ? V(Ht.starts) : null }) : Object.isFrozen(Ht) ? V(Ht) : Ht;
    }
    var sn = "11.11.1";
    class hn extends Error {
      constructor(jt, cn) {
        super(jt), this.name = "HTMLInjectionError", this.html = cn;
      }
    }
    const Qt = H, Ut = V, Wt = Symbol("nomatch"), rn = 7, vn = function(Ht) {
      const jt = /* @__PURE__ */ Object.create(null), cn = /* @__PURE__ */ Object.create(null), wn = [];
      let Rn = !0;
      const Pn = "Could not find the language '{}', did you forget to load/include a language module?", Ie = { disableAutodetect: !0, name: "Plain text", contains: [] };
      let rt = {
        ignoreUnescapedHTML: !1,
        throwUnescapedHTML: !1,
        noHighlightRe: /^(no-?highlight)$/i,
        languageDetectRe: /\blang(?:uage)?-([\w-]+)\b/i,
        classPrefix: "hljs-",
        cssSelector: "pre code",
        languages: null,
        // beta configuration options, subject to change, welcome to discuss
        // https://github.com/highlightjs/highlight.js/issues/1086
        __emitter: ee
      };
      function ft(mr) {
        return rt.noHighlightRe.test(mr);
      }
      function pt(mr) {
        let Ar = mr.className + " ";
        Ar += mr.parentNode ? mr.parentNode.className : "";
        const Yr = rt.languageDetectRe.exec(Ar);
        if (Yr) {
          const jr = In(Yr[1]);
          return jr || (Pt(Pn.replace("{}", Yr[1])), Pt("Falling back to no-highlight mode for this block.", mr)), jr ? Yr[1] : "no-highlight";
        }
        return Ar.split(/\s+/).find((jr) => ft(jr) || In(jr));
      }
      function wt(mr, Ar, Yr) {
        let jr = "", Mr = "";
        typeof Ar == "object" ? (jr = mr, Yr = Ar.ignoreIllegals, Mr = Ar.language) : (Xt("10.7.0", "highlight(lang, code, ...args) has been deprecated."), Xt("10.7.0", `Please use highlight(code, options) instead.
https://github.com/highlightjs/highlight.js/issues/2277`), Mr = mr, jr = Ar), Yr === void 0 && (Yr = !0);
        const Wr = {
          code: jr,
          language: Mr
        };
        Jn("before:highlight", Wr);
        const Bi = Wr.result ? Wr.result : Bt(Wr.language, Wr.code, Yr);
        return Bi.code = Wr.code, Jn("after:highlight", Bi), Bi;
      }
      function Bt(mr, Ar, Yr, jr) {
        const Mr = /* @__PURE__ */ Object.create(null);
        function Wr(la, wa) {
          return la.keywords[wa];
        }
        function Bi() {
          if (!Gr.keywords) {
            Sr.addText(Dr);
            return;
          }
          let la = 0;
          Gr.keywordPatternRe.lastIndex = 0;
          let wa = Gr.keywordPatternRe.exec(Dr), Wa = "";
          for (; wa; ) {
            Wa += Dr.substring(la, wa.index);
            const Qr = ka.case_insensitive ? wa[0].toLowerCase() : wa[0], Ur = Wr(Gr, Qr);
            if (Ur) {
              const [rr, vr] = Ur;
              if (Sr.addText(Wa), Wa = "", Mr[Qr] = (Mr[Qr] || 0) + 1, Mr[Qr] <= rn && (si += vr), rr.startsWith("_"))
                Wa += wa[0];
              else {
                const Rr = ka.classNameAliases[rr] || rr;
                fi(wa[0], Rr);
              }
            } else
              Wa += wa[0];
            la = Gr.keywordPatternRe.lastIndex, wa = Gr.keywordPatternRe.exec(Dr);
          }
          Wa += Dr.substring(la), Sr.addText(Wa);
        }
        function gi() {
          if (Dr === "") return;
          let la = null;
          if (typeof Gr.subLanguage == "string") {
            if (!jt[Gr.subLanguage]) {
              Sr.addText(Dr);
              return;
            }
            la = Bt(Gr.subLanguage, Dr, !0, Ca[Gr.subLanguage]), Ca[Gr.subLanguage] = /** @type {CompiledMode} */
            la._top;
          } else
            la = Dt(Dr, Gr.subLanguage.length ? Gr.subLanguage : null);
          Gr.relevance > 0 && (si += la.relevance), Sr.__addSublanguage(la._emitter, la.language);
        }
        function Zr() {
          Gr.subLanguage != null ? gi() : Bi(), Dr = "";
        }
        function fi(la, wa) {
          la !== "" && (Sr.startScope(wa), Sr.addText(la), Sr.endScope());
        }
        function ri(la, wa) {
          let Wa = 1;
          const Qr = wa.length - 1;
          for (; Wa <= Qr; ) {
            if (!la._emit[Wa]) {
              Wa++;
              continue;
            }
            const Ur = ka.classNameAliases[la[Wa]] || la[Wa], rr = wa[Wa];
            Ur ? fi(rr, Ur) : (Dr = rr, Bi(), Dr = ""), Wa++;
          }
        }
        function bi(la, wa) {
          return la.scope && typeof la.scope == "string" && Sr.openNode(ka.classNameAliases[la.scope] || la.scope), la.beginScope && (la.beginScope._wrap ? (fi(Dr, ka.classNameAliases[la.beginScope._wrap] || la.beginScope._wrap), Dr = "") : la.beginScope._multi && (ri(la.beginScope, wa), Dr = "")), Gr = Object.create(la, { parent: { value: Gr } }), Gr;
        }
        function Zi(la, wa, Wa) {
          let Qr = he(la.endRe, Wa);
          if (Qr) {
            if (la["on:end"]) {
              const Ur = new _(la);
              la["on:end"](wa, Ur), Ur.isMatchIgnored && (Qr = !1);
            }
            if (Qr) {
              for (; la.endsParent && la.parent; )
                la = la.parent;
              return la;
            }
          }
          if (la.endsWithParent)
            return Zi(la.parent, wa, Wa);
        }
        function $i(la) {
          return Gr.matcher.regexIndex === 0 ? (Dr += la[0], 1) : (Da = !0, 0);
        }
        function Br(la) {
          const wa = la[0], Wa = la.rule, Qr = new _(Wa), Ur = [Wa.__beforeBegin, Wa["on:begin"]];
          for (const rr of Ur)
            if (rr && (rr(la, Qr), Qr.isMatchIgnored))
              return $i(wa);
          return Wa.skip ? Dr += wa : (Wa.excludeBegin && (Dr += wa), Zr(), !Wa.returnBegin && !Wa.excludeBegin && (Dr = wa)), bi(Wa, la), Wa.returnBegin ? 0 : wa.length;
        }
        function Pr(la) {
          const wa = la[0], Wa = Ar.substring(la.index), Qr = Zi(Gr, la, Wa);
          if (!Qr)
            return Wt;
          const Ur = Gr;
          Gr.endScope && Gr.endScope._wrap ? (Zr(), fi(wa, Gr.endScope._wrap)) : Gr.endScope && Gr.endScope._multi ? (Zr(), ri(Gr.endScope, la)) : Ur.skip ? Dr += wa : (Ur.returnEnd || Ur.excludeEnd || (Dr += wa), Zr(), Ur.excludeEnd && (Dr = wa));
          do
            Gr.scope && Sr.closeNode(), !Gr.skip && !Gr.subLanguage && (si += Gr.relevance), Gr = Gr.parent;
          while (Gr !== Qr.parent);
          return Qr.starts && bi(Qr.starts, la), Ur.returnEnd ? 0 : wa.length;
        }
        function ei() {
          const la = [];
          for (let wa = Gr; wa !== ka; wa = wa.parent)
            wa.scope && la.unshift(wa.scope);
          la.forEach((wa) => Sr.openNode(wa));
        }
        let Ci = {};
        function oa(la, wa) {
          const Wa = wa && wa[0];
          if (Dr += la, Wa == null)
            return Zr(), 0;
          if (Ci.type === "begin" && wa.type === "end" && Ci.index === wa.index && Wa === "") {
            if (Dr += Ar.slice(wa.index, wa.index + 1), !Rn) {
              const Qr = new Error(`0 width match regex (${mr})`);
              throw Qr.languageName = mr, Qr.badRule = Ci.rule, Qr;
            }
            return 1;
          }
          if (Ci = wa, wa.type === "begin")
            return Br(wa);
          if (wa.type === "illegal" && !Yr) {
            const Qr = new Error('Illegal lexeme "' + Wa + '" for mode "' + (Gr.scope || "<unnamed>") + '"');
            throw Qr.mode = Gr, Qr;
          } else if (wa.type === "end") {
            const Qr = Pr(wa);
            if (Qr !== Wt)
              return Qr;
          }
          if (wa.type === "illegal" && Wa === "")
            return Dr += `
`, 1;
          if (di > 1e5 && di > wa.index * 3)
            throw new Error("potential infinite loop, way more iterations than matches");
          return Dr += Wa, Wa.length;
        }
        const ka = In(mr);
        if (!ka)
          throw $t(Pn.replace("{}", mr)), new Error('Unknown language: "' + mr + '"');
        const aa = Kt(ka);
        let hi = "", Gr = jr || aa;
        const Ca = {}, Sr = new rt.__emitter(rt);
        ei();
        let Dr = "", si = 0, zr = 0, di = 0, Da = !1;
        try {
          if (ka.__emitTokens)
            ka.__emitTokens(Ar, Sr);
          else {
            for (Gr.matcher.considerAll(); ; ) {
              di++, Da ? Da = !1 : Gr.matcher.considerAll(), Gr.matcher.lastIndex = zr;
              const la = Gr.matcher.exec(Ar);
              if (!la) break;
              const wa = Ar.substring(zr, la.index), Wa = oa(wa, la);
              zr = la.index + Wa;
            }
            oa(Ar.substring(zr));
          }
          return Sr.finalize(), hi = Sr.toHTML(), {
            language: mr,
            value: hi,
            relevance: si,
            illegal: !1,
            _emitter: Sr,
            _top: Gr
          };
        } catch (la) {
          if (la.message && la.message.includes("Illegal"))
            return {
              language: mr,
              value: Qt(Ar),
              illegal: !0,
              relevance: 0,
              _illegalBy: {
                message: la.message,
                index: zr,
                context: Ar.slice(zr - 100, zr + 100),
                mode: la.mode,
                resultSoFar: hi
              },
              _emitter: Sr
            };
          if (Rn)
            return {
              language: mr,
              value: Qt(Ar),
              illegal: !1,
              relevance: 0,
              errorRaised: la,
              _emitter: Sr,
              _top: Gr
            };
          throw la;
        }
      }
      function At(mr) {
        const Ar = {
          value: Qt(mr),
          illegal: !1,
          relevance: 0,
          _top: Ie,
          _emitter: new rt.__emitter(rt)
        };
        return Ar._emitter.addText(mr), Ar;
      }
      function Dt(mr, Ar) {
        Ar = Ar || rt.languages || Object.keys(jt);
        const Yr = At(mr), jr = Ar.filter(In).filter(er).map(
          (Zr) => Bt(Zr, mr, !1)
        );
        jr.unshift(Yr);
        const Mr = jr.sort((Zr, fi) => {
          if (Zr.relevance !== fi.relevance) return fi.relevance - Zr.relevance;
          if (Zr.language && fi.language) {
            if (In(Zr.language).supersetOf === fi.language)
              return 1;
            if (In(fi.language).supersetOf === Zr.language)
              return -1;
          }
          return 0;
        }), [Wr, Bi] = Mr, gi = Wr;
        return gi.secondBest = Bi, gi;
      }
      function Ot(mr, Ar, Yr) {
        const jr = Ar && cn[Ar] || Yr;
        mr.classList.add("hljs"), mr.classList.add(`language-${jr}`);
      }
      function un(mr) {
        let Ar = null;
        const Yr = pt(mr);
        if (ft(Yr)) return;
        if (Jn(
          "before:highlightElement",
          { el: mr, language: Yr }
        ), mr.dataset.highlighted) {
          console.log("Element previously highlighted. To highlight again, first unset `dataset.highlighted`.", mr);
          return;
        }
        if (mr.children.length > 0 && (rt.ignoreUnescapedHTML || (console.warn("One of your code blocks includes unescaped HTML. This is a potentially serious security risk."), console.warn("https://github.com/highlightjs/highlight.js/wiki/security"), console.warn("The element with unescaped HTML:"), console.warn(mr)), rt.throwUnescapedHTML))
          throw new hn(
            "One of your code blocks includes unescaped HTML.",
            mr.innerHTML
          );
        Ar = mr;
        const jr = Ar.textContent, Mr = Yr ? wt(jr, { language: Yr, ignoreIllegals: !0 }) : Dt(jr);
        mr.innerHTML = Mr.value, mr.dataset.highlighted = "yes", Ot(mr, Yr, Mr.language), mr.result = {
          language: Mr.language,
          // TODO: remove with version 11.0
          re: Mr.relevance,
          relevance: Mr.relevance
        }, Mr.secondBest && (mr.secondBest = {
          language: Mr.secondBest.language,
          relevance: Mr.secondBest.relevance
        }), Jn("after:highlightElement", { el: mr, result: Mr, text: jr });
      }
      function yn(mr) {
        rt = Ut(rt, mr);
      }
      const _n = () => {
        an(), Xt("10.6.0", "initHighlighting() deprecated.  Use highlightAll() now.");
      };
      function gn() {
        an(), Xt("10.6.0", "initHighlightingOnLoad() deprecated.  Use highlightAll() now.");
      }
      let bn = !1;
      function an() {
        function mr() {
          an();
        }
        if (document.readyState === "loading") {
          bn || window.addEventListener("DOMContentLoaded", mr, !1), bn = !0;
          return;
        }
        document.querySelectorAll(rt.cssSelector).forEach(un);
      }
      function Zt(mr, Ar) {
        let Yr = null;
        try {
          Yr = Ar(Ht);
        } catch (jr) {
          if ($t("Language definition for '{}' could not be registered.".replace("{}", mr)), Rn)
            $t(jr);
          else
            throw jr;
          Yr = Ie;
        }
        Yr.name || (Yr.name = mr), jt[mr] = Yr, Yr.rawDefinition = Ar.bind(null, Ht), Yr.aliases && kn(Yr.aliases, { languageName: mr });
      }
      function Mn(mr) {
        delete jt[mr];
        for (const Ar of Object.keys(cn))
          cn[Ar] === mr && delete cn[Ar];
      }
      function Ln() {
        return Object.keys(jt);
      }
      function In(mr) {
        return mr = (mr || "").toLowerCase(), jt[mr] || jt[cn[mr]];
      }
      function kn(mr, { languageName: Ar }) {
        typeof mr == "string" && (mr = [mr]), mr.forEach((Yr) => {
          cn[Yr.toLowerCase()] = Ar;
        });
      }
      function er(mr) {
        const Ar = In(mr);
        return Ar && !Ar.disableAutodetect;
      }
      function jn(mr) {
        mr["before:highlightBlock"] && !mr["before:highlightElement"] && (mr["before:highlightElement"] = (Ar) => {
          mr["before:highlightBlock"](
            Object.assign({ block: Ar.el }, Ar)
          );
        }), mr["after:highlightBlock"] && !mr["after:highlightElement"] && (mr["after:highlightElement"] = (Ar) => {
          mr["after:highlightBlock"](
            Object.assign({ block: Ar.el }, Ar)
          );
        });
      }
      function ur(mr) {
        jn(mr), wn.push(mr);
      }
      function Kn(mr) {
        const Ar = wn.indexOf(mr);
        Ar !== -1 && wn.splice(Ar, 1);
      }
      function Jn(mr, Ar) {
        const Yr = mr;
        wn.forEach(function(jr) {
          jr[Yr] && jr[Yr](Ar);
        });
      }
      function Lr(mr) {
        return Xt("10.7.0", "highlightBlock will be removed entirely in v12.0"), Xt("10.7.0", "Please use highlightElement now."), un(mr);
      }
      Object.assign(Ht, {
        highlight: wt,
        highlightAuto: Dt,
        highlightAll: an,
        highlightElement: un,
        // TODO: Remove with v12 API
        highlightBlock: Lr,
        configure: yn,
        initHighlighting: _n,
        initHighlightingOnLoad: gn,
        registerLanguage: Zt,
        unregisterLanguage: Mn,
        listLanguages: Ln,
        getLanguage: In,
        registerAliases: kn,
        autoDetection: er,
        inherit: Ut,
        addPlugin: ur,
        removePlugin: Kn
      }), Ht.debugMode = function() {
        Rn = !1;
      }, Ht.safeMode = function() {
        Rn = !0;
      }, Ht.versionString = sn, Ht.regex = {
        concat: le,
        lookahead: oe,
        either: de,
        optional: ce,
        anyNumberOfTimes: se
      };
      for (const mr in at)
        typeof at[mr] == "object" && O(at[mr]);
      return Object.assign(Ht, at), Ht;
    }, dn = vn({});
    return dn.newInstance = () => vn({}), core$1 = dn, dn.HighlightJS = dn, dn.default = dn, core$1;
  }
  var xml_1, hasRequiredXml;
  function requireXml() {
    if (hasRequiredXml) return xml_1;
    hasRequiredXml = 1;
    function O(_) {
      const H = _.regex, V = H.concat(/[\p{L}_]/u, H.optional(/[\p{L}0-9_.-]*:/u), /[\p{L}0-9_.-]*/u), q = /[\p{L}0-9._:-]+/u, G = {
        className: "symbol",
        begin: /&[a-z]+;|&#[0-9]+;|&#x[a-f0-9]+;/
      }, J = {
        begin: /\s/,
        contains: [
          {
            className: "keyword",
            begin: /#?[a-z_][a-z1-9_-]+/,
            illegal: /\n/
          }
        ]
      }, Z = _.inherit(J, {
        begin: /\(/,
        end: /\)/
      }), te = _.inherit(_.APOS_STRING_MODE, { className: "string" }), ne = _.inherit(_.QUOTE_STRING_MODE, { className: "string" }), ee = {
        endsWithParent: !0,
        illegal: /</,
        relevance: 0,
        contains: [
          {
            className: "attr",
            begin: q,
            relevance: 0
          },
          {
            begin: /=\s*/,
            relevance: 0,
            contains: [
              {
                className: "string",
                endsParent: !0,
                variants: [
                  {
                    begin: /"/,
                    end: /"/,
                    contains: [G]
                  },
                  {
                    begin: /'/,
                    end: /'/,
                    contains: [G]
                  },
                  { begin: /[^\s"'=<>`]+/ }
                ]
              }
            ]
          }
        ]
      };
      return {
        name: "HTML, XML",
        aliases: [
          "html",
          "xhtml",
          "rss",
          "atom",
          "xjb",
          "xsd",
          "xsl",
          "plist",
          "wsf",
          "svg"
        ],
        case_insensitive: !0,
        unicodeRegex: !0,
        contains: [
          {
            className: "meta",
            begin: /<![a-z]/,
            end: />/,
            relevance: 10,
            contains: [
              J,
              ne,
              te,
              Z,
              {
                begin: /\[/,
                end: /\]/,
                contains: [
                  {
                    className: "meta",
                    begin: /<![a-z]/,
                    end: />/,
                    contains: [
                      J,
                      Z,
                      ne,
                      te
                    ]
                  }
                ]
              }
            ]
          },
          _.COMMENT(
            /<!--/,
            /-->/,
            { relevance: 10 }
          ),
          {
            begin: /<!\[CDATA\[/,
            end: /\]\]>/,
            relevance: 10
          },
          G,
          // xml processing instructions
          {
            className: "meta",
            end: /\?>/,
            variants: [
              {
                begin: /<\?xml/,
                relevance: 10,
                contains: [
                  ne
                ]
              },
              {
                begin: /<\?[a-z][a-z0-9]+/
              }
            ]
          },
          {
            className: "tag",
            /*
            The lookahead pattern (?=...) ensures that 'begin' only matches
            '<style' as a single word, followed by a whitespace or an
            ending bracket.
            */
            begin: /<style(?=\s|>)/,
            end: />/,
            keywords: { name: "style" },
            contains: [ee],
            starts: {
              end: /<\/style>/,
              returnEnd: !0,
              subLanguage: [
                "css",
                "xml"
              ]
            }
          },
          {
            className: "tag",
            // See the comment in the <style tag about the lookahead pattern
            begin: /<script(?=\s|>)/,
            end: />/,
            keywords: { name: "script" },
            contains: [ee],
            starts: {
              end: /<\/script>/,
              returnEnd: !0,
              subLanguage: [
                "javascript",
                "handlebars",
                "xml"
              ]
            }
          },
          // we need this for now for jSX
          {
            className: "tag",
            begin: /<>|<\/>/
          },
          // open tag
          {
            className: "tag",
            begin: H.concat(
              /</,
              H.lookahead(H.concat(
                V,
                // <tag/>
                // <tag>
                // <tag ...
                H.either(/\/>/, />/, /\s/)
              ))
            ),
            end: /\/?>/,
            contains: [
              {
                className: "name",
                begin: V,
                relevance: 0,
                starts: ee
              }
            ]
          },
          // close tag
          {
            className: "tag",
            begin: H.concat(
              /<\//,
              H.lookahead(H.concat(
                V,
                />/
              ))
            ),
            contains: [
              {
                className: "name",
                begin: V,
                relevance: 0
              },
              {
                begin: />/,
                relevance: 0,
                endsParent: !0
              }
            ]
          }
        ]
      };
    }
    return xml_1 = O, xml_1;
  }
  var bash_1, hasRequiredBash;
  function requireBash() {
    if (hasRequiredBash) return bash_1;
    hasRequiredBash = 1;
    function O(_) {
      const H = _.regex, V = {}, q = {
        begin: /\$\{/,
        end: /\}/,
        contains: [
          "self",
          {
            begin: /:-/,
            contains: [V]
          }
          // default values
        ]
      };
      Object.assign(V, {
        className: "variable",
        variants: [
          { begin: H.concat(
            /\$[\w\d#@][\w\d_]*/,
            // negative look-ahead tries to avoid matching patterns that are not
            // Perl at all like $ident$, @ident@, etc.
            "(?![\\w\\d])(?![$])"
          ) },
          q
        ]
      });
      const G = {
        className: "subst",
        begin: /\$\(/,
        end: /\)/,
        contains: [_.BACKSLASH_ESCAPE]
      }, J = _.inherit(
        _.COMMENT(),
        {
          match: [
            /(^|\s)/,
            /#.*$/
          ],
          scope: {
            2: "comment"
          }
        }
      ), Z = {
        begin: /<<-?\s*(?=\w+)/,
        starts: { contains: [
          _.END_SAME_AS_BEGIN({
            begin: /(\w+)/,
            end: /(\w+)/,
            className: "string"
          })
        ] }
      }, te = {
        className: "string",
        begin: /"/,
        end: /"/,
        contains: [
          _.BACKSLASH_ESCAPE,
          V,
          G
        ]
      };
      G.contains.push(te);
      const ne = {
        match: /\\"/
      }, ee = {
        className: "string",
        begin: /'/,
        end: /'/
      }, ae = {
        match: /\\'/
      }, oe = {
        begin: /\$?\(\(/,
        end: /\)\)/,
        contains: [
          {
            begin: /\d+#[0-9a-f]+/,
            className: "number"
          },
          _.NUMBER_MODE,
          V
        ]
      }, se = [
        "fish",
        "bash",
        "zsh",
        "sh",
        "csh",
        "ksh",
        "tcsh",
        "dash",
        "scsh"
      ], ce = _.SHEBANG({
        binary: `(${se.join("|")})`,
        relevance: 10
      }), le = {
        className: "function",
        begin: /\w[\w\d_]*\s*\(\s*\)\s*\{/,
        returnBegin: !0,
        contains: [_.inherit(_.TITLE_MODE, { begin: /\w[\w\d_]*/ })],
        relevance: 0
      }, ue = [
        "if",
        "then",
        "else",
        "elif",
        "fi",
        "time",
        "for",
        "while",
        "until",
        "in",
        "do",
        "done",
        "case",
        "esac",
        "coproc",
        "function",
        "select"
      ], de = [
        "true",
        "false"
      ], fe = { match: /(\/[a-z._-]+)+/ }, he = [
        "break",
        "cd",
        "continue",
        "eval",
        "exec",
        "exit",
        "export",
        "getopts",
        "hash",
        "pwd",
        "readonly",
        "return",
        "shift",
        "test",
        "times",
        "trap",
        "umask",
        "unset"
      ], me = [
        "alias",
        "bind",
        "builtin",
        "caller",
        "command",
        "declare",
        "echo",
        "enable",
        "help",
        "let",
        "local",
        "logout",
        "mapfile",
        "printf",
        "read",
        "readarray",
        "source",
        "sudo",
        "type",
        "typeset",
        "ulimit",
        "unalias"
      ], pe = [
        "autoload",
        "bg",
        "bindkey",
        "bye",
        "cap",
        "chdir",
        "clone",
        "comparguments",
        "compcall",
        "compctl",
        "compdescribe",
        "compfiles",
        "compgroups",
        "compquote",
        "comptags",
        "comptry",
        "compvalues",
        "dirs",
        "disable",
        "disown",
        "echotc",
        "echoti",
        "emulate",
        "fc",
        "fg",
        "float",
        "functions",
        "getcap",
        "getln",
        "history",
        "integer",
        "jobs",
        "kill",
        "limit",
        "log",
        "noglob",
        "popd",
        "print",
        "pushd",
        "pushln",
        "rehash",
        "sched",
        "setcap",
        "setopt",
        "stat",
        "suspend",
        "ttyctl",
        "unfunction",
        "unhash",
        "unlimit",
        "unsetopt",
        "vared",
        "wait",
        "whence",
        "where",
        "which",
        "zcompile",
        "zformat",
        "zftp",
        "zle",
        "zmodload",
        "zparseopts",
        "zprof",
        "zpty",
        "zregexparse",
        "zsocket",
        "zstyle",
        "ztcp"
      ], ye = [
        "chcon",
        "chgrp",
        "chown",
        "chmod",
        "cp",
        "dd",
        "df",
        "dir",
        "dircolors",
        "ln",
        "ls",
        "mkdir",
        "mkfifo",
        "mknod",
        "mktemp",
        "mv",
        "realpath",
        "rm",
        "rmdir",
        "shred",
        "sync",
        "touch",
        "truncate",
        "vdir",
        "b2sum",
        "base32",
        "base64",
        "cat",
        "cksum",
        "comm",
        "csplit",
        "cut",
        "expand",
        "fmt",
        "fold",
        "head",
        "join",
        "md5sum",
        "nl",
        "numfmt",
        "od",
        "paste",
        "ptx",
        "pr",
        "sha1sum",
        "sha224sum",
        "sha256sum",
        "sha384sum",
        "sha512sum",
        "shuf",
        "sort",
        "split",
        "sum",
        "tac",
        "tail",
        "tr",
        "tsort",
        "unexpand",
        "uniq",
        "wc",
        "arch",
        "basename",
        "chroot",
        "date",
        "dirname",
        "du",
        "echo",
        "env",
        "expr",
        "factor",
        // "false", // keyword literal already
        "groups",
        "hostid",
        "id",
        "link",
        "logname",
        "nice",
        "nohup",
        "nproc",
        "pathchk",
        "pinky",
        "printenv",
        "printf",
        "pwd",
        "readlink",
        "runcon",
        "seq",
        "sleep",
        "stat",
        "stdbuf",
        "stty",
        "tee",
        "test",
        "timeout",
        // "true", // keyword literal already
        "tty",
        "uname",
        "unlink",
        "uptime",
        "users",
        "who",
        "whoami",
        "yes"
      ];
      return {
        name: "Bash",
        aliases: [
          "sh",
          "zsh"
        ],
        keywords: {
          $pattern: /\b[a-z][a-z0-9._-]+\b/,
          keyword: ue,
          literal: de,
          built_in: [
            ...he,
            ...me,
            // Shell modifiers
            "set",
            "shopt",
            ...pe,
            ...ye
          ]
        },
        contains: [
          ce,
          // to catch known shells and boost relevancy
          _.SHEBANG(),
          // to catch unknown shells but still highlight the shebang
          le,
          oe,
          J,
          Z,
          fe,
          te,
          ne,
          ee,
          ae,
          V
        ]
      };
    }
    return bash_1 = O, bash_1;
  }
  var c_1, hasRequiredC;
  function requireC() {
    if (hasRequiredC) return c_1;
    hasRequiredC = 1;
    function O(_) {
      const H = _.regex, V = _.COMMENT("//", "$", { contains: [{ begin: /\\\n/ }] }), q = "decltype\\(auto\\)", G = "[a-zA-Z_]\\w*::", Z = "(" + q + "|" + H.optional(G) + "[a-zA-Z_]\\w*" + H.optional("<[^<>]+>") + ")", te = {
        className: "type",
        variants: [
          { begin: "\\b[a-z\\d_]*_t\\b" },
          { match: /\batomic_[a-z]{3,6}\b/ }
        ]
      }, ee = {
        className: "string",
        variants: [
          {
            begin: '(u8?|U|L)?"',
            end: '"',
            illegal: "\\n",
            contains: [_.BACKSLASH_ESCAPE]
          },
          {
            begin: "(u8?|U|L)?'(" + "\\\\(x[0-9A-Fa-f]{2}|u[0-9A-Fa-f]{4,8}|[0-7]{3}|\\S)" + "|.)",
            end: "'",
            illegal: "."
          },
          _.END_SAME_AS_BEGIN({
            begin: /(?:u8?|U|L)?R"([^()\\ ]{0,16})\(/,
            end: /\)([^()\\ ]{0,16})"/
          })
        ]
      }, ae = {
        className: "number",
        variants: [
          { match: /\b(0b[01']+)/ },
          { match: /(-?)\b([\d']+(\.[\d']*)?|\.[\d']+)((ll|LL|l|L)(u|U)?|(u|U)(ll|LL|l|L)?|f|F|b|B)/ },
          { match: /(-?)\b(0[xX][a-fA-F0-9]+(?:'[a-fA-F0-9]+)*(?:\.[a-fA-F0-9]*(?:'[a-fA-F0-9]*)*)?(?:[pP][-+]?[0-9]+)?(l|L)?(u|U)?)/ },
          { match: /(-?)\b\d+(?:'\d+)*(?:\.\d*(?:'\d*)*)?(?:[eE][-+]?\d+)?/ }
        ],
        relevance: 0
      }, oe = {
        className: "meta",
        begin: /#\s*[a-z]+\b/,
        end: /$/,
        keywords: { keyword: "if else elif endif define undef warning error line pragma _Pragma ifdef ifndef elifdef elifndef include" },
        contains: [
          {
            begin: /\\\n/,
            relevance: 0
          },
          _.inherit(ee, { className: "string" }),
          {
            className: "string",
            begin: /<.*?>/
          },
          V,
          _.C_BLOCK_COMMENT_MODE
        ]
      }, se = {
        className: "title",
        begin: H.optional(G) + _.IDENT_RE,
        relevance: 0
      }, ce = H.optional(G) + _.IDENT_RE + "\\s*\\(", de = {
        keyword: [
          "asm",
          "auto",
          "break",
          "case",
          "continue",
          "default",
          "do",
          "else",
          "enum",
          "extern",
          "for",
          "fortran",
          "goto",
          "if",
          "inline",
          "register",
          "restrict",
          "return",
          "sizeof",
          "typeof",
          "typeof_unqual",
          "struct",
          "switch",
          "typedef",
          "union",
          "volatile",
          "while",
          "_Alignas",
          "_Alignof",
          "_Atomic",
          "_Generic",
          "_Noreturn",
          "_Static_assert",
          "_Thread_local",
          // aliases
          "alignas",
          "alignof",
          "noreturn",
          "static_assert",
          "thread_local",
          // not a C keyword but is, for all intents and purposes, treated exactly like one.
          "_Pragma"
        ],
        type: [
          "float",
          "double",
          "signed",
          "unsigned",
          "int",
          "short",
          "long",
          "char",
          "void",
          "_Bool",
          "_BitInt",
          "_Complex",
          "_Imaginary",
          "_Decimal32",
          "_Decimal64",
          "_Decimal96",
          "_Decimal128",
          "_Decimal64x",
          "_Decimal128x",
          "_Float16",
          "_Float32",
          "_Float64",
          "_Float128",
          "_Float32x",
          "_Float64x",
          "_Float128x",
          // modifiers
          "const",
          "static",
          "constexpr",
          // aliases
          "complex",
          "bool",
          "imaginary"
        ],
        literal: "true false NULL",
        // TODO: apply hinting work similar to what was done in cpp.js
        built_in: "std string wstring cin cout cerr clog stdin stdout stderr stringstream istringstream ostringstream auto_ptr deque list queue stack vector map set pair bitset multiset multimap unordered_set unordered_map unordered_multiset unordered_multimap priority_queue make_pair array shared_ptr abort terminate abs acos asin atan2 atan calloc ceil cosh cos exit exp fabs floor fmod fprintf fputs free frexp fscanf future isalnum isalpha iscntrl isdigit isgraph islower isprint ispunct isspace isupper isxdigit tolower toupper labs ldexp log10 log malloc realloc memchr memcmp memcpy memset modf pow printf putchar puts scanf sinh sin snprintf sprintf sqrt sscanf strcat strchr strcmp strcpy strcspn strlen strncat strncmp strncpy strpbrk strrchr strspn strstr tanh tan vfprintf vprintf vsprintf endl initializer_list unique_ptr"
      }, fe = [
        oe,
        te,
        V,
        _.C_BLOCK_COMMENT_MODE,
        ae,
        ee
      ], he = {
        // This mode covers expression context where we can't expect a function
        // definition and shouldn't highlight anything that looks like one:
        // `return some()`, `else if()`, `(x*sum(1, 2))`
        variants: [
          {
            begin: /=/,
            end: /;/
          },
          {
            begin: /\(/,
            end: /\)/
          },
          {
            beginKeywords: "new throw return else",
            end: /;/
          }
        ],
        keywords: de,
        contains: fe.concat([
          {
            begin: /\(/,
            end: /\)/,
            keywords: de,
            contains: fe.concat(["self"]),
            relevance: 0
          }
        ]),
        relevance: 0
      }, me = {
        begin: "(" + Z + "[\\*&\\s]+)+" + ce,
        returnBegin: !0,
        end: /[{;=]/,
        excludeEnd: !0,
        keywords: de,
        illegal: /[^\w\s\*&:<>.]/,
        contains: [
          {
            // to prevent it from being confused as the function title
            begin: q,
            keywords: de,
            relevance: 0
          },
          {
            begin: ce,
            returnBegin: !0,
            contains: [_.inherit(se, { className: "title.function" })],
            relevance: 0
          },
          // allow for multiple declarations, e.g.:
          // extern void f(int), g(char);
          {
            relevance: 0,
            match: /,/
          },
          {
            className: "params",
            begin: /\(/,
            end: /\)/,
            keywords: de,
            relevance: 0,
            contains: [
              V,
              _.C_BLOCK_COMMENT_MODE,
              ee,
              ae,
              te,
              // Count matching parentheses.
              {
                begin: /\(/,
                end: /\)/,
                keywords: de,
                relevance: 0,
                contains: [
                  "self",
                  V,
                  _.C_BLOCK_COMMENT_MODE,
                  ee,
                  ae,
                  te
                ]
              }
            ]
          },
          te,
          V,
          _.C_BLOCK_COMMENT_MODE,
          oe
        ]
      };
      return {
        name: "C",
        aliases: ["h"],
        keywords: de,
        // Until differentiations are added between `c` and `cpp`, `c` will
        // not be auto-detected to avoid auto-detect conflicts between C and C++
        disableAutodetect: !0,
        illegal: "</",
        contains: [].concat(
          he,
          me,
          fe,
          [
            oe,
            {
              begin: _.IDENT_RE + "::",
              keywords: de
            },
            {
              className: "class",
              beginKeywords: "enum class struct union",
              end: /[{;:<>=]/,
              contains: [
                { beginKeywords: "final class struct" },
                _.TITLE_MODE
              ]
            }
          ]
        ),
        exports: {
          preprocessor: oe,
          strings: ee,
          keywords: de
        }
      };
    }
    return c_1 = O, c_1;
  }
  var cpp_1, hasRequiredCpp;
  function requireCpp() {
    if (hasRequiredCpp) return cpp_1;
    hasRequiredCpp = 1;
    function O(_) {
      const H = _.regex, V = _.COMMENT("//", "$", { contains: [{ begin: /\\\n/ }] }), q = "decltype\\(auto\\)", G = "[a-zA-Z_]\\w*::", Z = "(?!struct)(" + q + "|" + H.optional(G) + "[a-zA-Z_]\\w*" + H.optional("<[^<>]+>") + ")", te = {
        className: "type",
        begin: "\\b[a-z\\d_]*_t\\b"
      }, ee = {
        className: "string",
        variants: [
          {
            begin: '(u8?|U|L)?"',
            end: '"',
            illegal: "\\n",
            contains: [_.BACKSLASH_ESCAPE]
          },
          {
            begin: "(u8?|U|L)?'(" + "\\\\(x[0-9A-Fa-f]{2}|u[0-9A-Fa-f]{4,8}|[0-7]{3}|\\S)" + "|.)",
            end: "'",
            illegal: "."
          },
          _.END_SAME_AS_BEGIN({
            begin: /(?:u8?|U|L)?R"([^()\\ ]{0,16})\(/,
            end: /\)([^()\\ ]{0,16})"/
          })
        ]
      }, ae = {
        className: "number",
        variants: [
          // Floating-point literal.
          {
            begin: "[+-]?(?:(?:[0-9](?:'?[0-9])*\\.(?:[0-9](?:'?[0-9])*)?|\\.[0-9](?:'?[0-9])*)(?:[Ee][+-]?[0-9](?:'?[0-9])*)?|[0-9](?:'?[0-9])*[Ee][+-]?[0-9](?:'?[0-9])*|0[Xx](?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*(?:\\.(?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)?)?|\\.[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)[Pp][+-]?[0-9](?:'?[0-9])*)(?:[Ff](?:16|32|64|128)?|(BF|bf)16|[Ll]|)"
          },
          // Integer literal.
          {
            begin: "[+-]?\\b(?:0[Bb][01](?:'?[01])*|0[Xx][0-9A-Fa-f](?:'?[0-9A-Fa-f])*|0(?:'?[0-7])*|[1-9](?:'?[0-9])*)(?:[Uu](?:LL?|ll?)|[Uu][Zz]?|(?:LL?|ll?)[Uu]?|[Zz][Uu]|)"
            // Note: there are user-defined literal suffixes too, but perhaps having the custom suffix not part of the
            // literal highlight actually makes it stand out more.
          }
        ],
        relevance: 0
      }, oe = {
        className: "meta",
        begin: /#\s*[a-z]+\b/,
        end: /$/,
        keywords: { keyword: "if else elif endif define undef warning error line pragma _Pragma ifdef ifndef include" },
        contains: [
          {
            begin: /\\\n/,
            relevance: 0
          },
          _.inherit(ee, { className: "string" }),
          {
            className: "string",
            begin: /<.*?>/
          },
          V,
          _.C_BLOCK_COMMENT_MODE
        ]
      }, se = {
        className: "title",
        begin: H.optional(G) + _.IDENT_RE,
        relevance: 0
      }, ce = H.optional(G) + _.IDENT_RE + "\\s*\\(", le = [
        "alignas",
        "alignof",
        "and",
        "and_eq",
        "asm",
        "atomic_cancel",
        "atomic_commit",
        "atomic_noexcept",
        "auto",
        "bitand",
        "bitor",
        "break",
        "case",
        "catch",
        "class",
        "co_await",
        "co_return",
        "co_yield",
        "compl",
        "concept",
        "const_cast|10",
        "consteval",
        "constexpr",
        "constinit",
        "continue",
        "decltype",
        "default",
        "delete",
        "do",
        "dynamic_cast|10",
        "else",
        "enum",
        "explicit",
        "export",
        "extern",
        "false",
        "final",
        "for",
        "friend",
        "goto",
        "if",
        "import",
        "inline",
        "module",
        "mutable",
        "namespace",
        "new",
        "noexcept",
        "not",
        "not_eq",
        "nullptr",
        "operator",
        "or",
        "or_eq",
        "override",
        "private",
        "protected",
        "public",
        "reflexpr",
        "register",
        "reinterpret_cast|10",
        "requires",
        "return",
        "sizeof",
        "static_assert",
        "static_cast|10",
        "struct",
        "switch",
        "synchronized",
        "template",
        "this",
        "thread_local",
        "throw",
        "transaction_safe",
        "transaction_safe_dynamic",
        "true",
        "try",
        "typedef",
        "typeid",
        "typename",
        "union",
        "using",
        "virtual",
        "volatile",
        "while",
        "xor",
        "xor_eq"
      ], ue = [
        "bool",
        "char",
        "char16_t",
        "char32_t",
        "char8_t",
        "double",
        "float",
        "int",
        "long",
        "short",
        "void",
        "wchar_t",
        "unsigned",
        "signed",
        "const",
        "static"
      ], de = [
        "any",
        "auto_ptr",
        "barrier",
        "binary_semaphore",
        "bitset",
        "complex",
        "condition_variable",
        "condition_variable_any",
        "counting_semaphore",
        "deque",
        "false_type",
        "flat_map",
        "flat_set",
        "future",
        "imaginary",
        "initializer_list",
        "istringstream",
        "jthread",
        "latch",
        "lock_guard",
        "multimap",
        "multiset",
        "mutex",
        "optional",
        "ostringstream",
        "packaged_task",
        "pair",
        "promise",
        "priority_queue",
        "queue",
        "recursive_mutex",
        "recursive_timed_mutex",
        "scoped_lock",
        "set",
        "shared_future",
        "shared_lock",
        "shared_mutex",
        "shared_timed_mutex",
        "shared_ptr",
        "stack",
        "string_view",
        "stringstream",
        "timed_mutex",
        "thread",
        "true_type",
        "tuple",
        "unique_lock",
        "unique_ptr",
        "unordered_map",
        "unordered_multimap",
        "unordered_multiset",
        "unordered_set",
        "variant",
        "vector",
        "weak_ptr",
        "wstring",
        "wstring_view"
      ], fe = [
        "abort",
        "abs",
        "acos",
        "apply",
        "as_const",
        "asin",
        "atan",
        "atan2",
        "calloc",
        "ceil",
        "cerr",
        "cin",
        "clog",
        "cos",
        "cosh",
        "cout",
        "declval",
        "endl",
        "exchange",
        "exit",
        "exp",
        "fabs",
        "floor",
        "fmod",
        "forward",
        "fprintf",
        "fputs",
        "free",
        "frexp",
        "fscanf",
        "future",
        "invoke",
        "isalnum",
        "isalpha",
        "iscntrl",
        "isdigit",
        "isgraph",
        "islower",
        "isprint",
        "ispunct",
        "isspace",
        "isupper",
        "isxdigit",
        "labs",
        "launder",
        "ldexp",
        "log",
        "log10",
        "make_pair",
        "make_shared",
        "make_shared_for_overwrite",
        "make_tuple",
        "make_unique",
        "malloc",
        "memchr",
        "memcmp",
        "memcpy",
        "memset",
        "modf",
        "move",
        "pow",
        "printf",
        "putchar",
        "puts",
        "realloc",
        "scanf",
        "sin",
        "sinh",
        "snprintf",
        "sprintf",
        "sqrt",
        "sscanf",
        "std",
        "stderr",
        "stdin",
        "stdout",
        "strcat",
        "strchr",
        "strcmp",
        "strcpy",
        "strcspn",
        "strlen",
        "strncat",
        "strncmp",
        "strncpy",
        "strpbrk",
        "strrchr",
        "strspn",
        "strstr",
        "swap",
        "tan",
        "tanh",
        "terminate",
        "to_underlying",
        "tolower",
        "toupper",
        "vfprintf",
        "visit",
        "vprintf",
        "vsprintf"
      ], pe = {
        type: ue,
        keyword: le,
        literal: [
          "NULL",
          "false",
          "nullopt",
          "nullptr",
          "true"
        ],
        built_in: ["_Pragma"],
        _type_hints: de
      }, ye = {
        className: "function.dispatch",
        relevance: 0,
        keywords: {
          // Only for relevance, not highlighting.
          _hint: fe
        },
        begin: H.concat(
          /\b/,
          /(?!decltype)/,
          /(?!if)/,
          /(?!for)/,
          /(?!switch)/,
          /(?!while)/,
          _.IDENT_RE,
          H.lookahead(/(<[^<>]+>|)\s*\(/)
        )
      }, ge = [
        ye,
        oe,
        te,
        V,
        _.C_BLOCK_COMMENT_MODE,
        ae,
        ee
      ], ve = {
        // This mode covers expression context where we can't expect a function
        // definition and shouldn't highlight anything that looks like one:
        // `return some()`, `else if()`, `(x*sum(1, 2))`
        variants: [
          {
            begin: /=/,
            end: /;/
          },
          {
            begin: /\(/,
            end: /\)/
          },
          {
            beginKeywords: "new throw return else",
            end: /;/
          }
        ],
        keywords: pe,
        contains: ge.concat([
          {
            begin: /\(/,
            end: /\)/,
            keywords: pe,
            contains: ge.concat(["self"]),
            relevance: 0
          }
        ]),
        relevance: 0
      }, be = {
        className: "function",
        begin: "(" + Z + "[\\*&\\s]+)+" + ce,
        returnBegin: !0,
        end: /[{;=]/,
        excludeEnd: !0,
        keywords: pe,
        illegal: /[^\w\s\*&:<>.]/,
        contains: [
          {
            // to prevent it from being confused as the function title
            begin: q,
            keywords: pe,
            relevance: 0
          },
          {
            begin: ce,
            returnBegin: !0,
            contains: [se],
            relevance: 0
          },
          // needed because we do not have look-behind on the below rule
          // to prevent it from grabbing the final : in a :: pair
          {
            begin: /::/,
            relevance: 0
          },
          // initializers
          {
            begin: /:/,
            endsWithParent: !0,
            contains: [
              ee,
              ae
            ]
          },
          // allow for multiple declarations, e.g.:
          // extern void f(int), g(char);
          {
            relevance: 0,
            match: /,/
          },
          {
            className: "params",
            begin: /\(/,
            end: /\)/,
            keywords: pe,
            relevance: 0,
            contains: [
              V,
              _.C_BLOCK_COMMENT_MODE,
              ee,
              ae,
              te,
              // Count matching parentheses.
              {
                begin: /\(/,
                end: /\)/,
                keywords: pe,
                relevance: 0,
                contains: [
                  "self",
                  V,
                  _.C_BLOCK_COMMENT_MODE,
                  ee,
                  ae,
                  te
                ]
              }
            ]
          },
          te,
          V,
          _.C_BLOCK_COMMENT_MODE,
          oe
        ]
      };
      return {
        name: "C++",
        aliases: [
          "cc",
          "c++",
          "h++",
          "hpp",
          "hh",
          "hxx",
          "cxx"
        ],
        keywords: pe,
        illegal: "</",
        classNameAliases: { "function.dispatch": "built_in" },
        contains: [].concat(
          ve,
          be,
          ye,
          ge,
          [
            oe,
            {
              // containers: ie, `vector <int> rooms (9);`
              begin: "\\b(deque|list|queue|priority_queue|pair|stack|vector|map|set|bitset|multiset|multimap|unordered_map|unordered_set|unordered_multiset|unordered_multimap|array|tuple|optional|variant|function|flat_map|flat_set)\\s*<(?!<)",
              end: ">",
              keywords: pe,
              contains: [
                "self",
                te
              ]
            },
            {
              begin: _.IDENT_RE + "::",
              keywords: pe
            },
            {
              match: [
                // extra complexity to deal with `enum class` and `enum struct`
                /\b(?:enum(?:\s+(?:class|struct))?|class|struct|union)/,
                /\s+/,
                /\w+/
              ],
              className: {
                1: "keyword",
                3: "title.class"
              }
            }
          ]
        )
      };
    }
    return cpp_1 = O, cpp_1;
  }
  var csharp_1, hasRequiredCsharp;
  function requireCsharp() {
    if (hasRequiredCsharp) return csharp_1;
    hasRequiredCsharp = 1;
    function O(_) {
      const H = [
        "bool",
        "byte",
        "char",
        "decimal",
        "delegate",
        "double",
        "dynamic",
        "enum",
        "float",
        "int",
        "long",
        "nint",
        "nuint",
        "object",
        "sbyte",
        "short",
        "string",
        "ulong",
        "uint",
        "ushort"
      ], V = [
        "public",
        "private",
        "protected",
        "static",
        "internal",
        "protected",
        "abstract",
        "async",
        "extern",
        "override",
        "unsafe",
        "virtual",
        "new",
        "sealed",
        "partial"
      ], q = [
        "default",
        "false",
        "null",
        "true"
      ], G = [
        "abstract",
        "as",
        "base",
        "break",
        "case",
        "catch",
        "class",
        "const",
        "continue",
        "do",
        "else",
        "event",
        "explicit",
        "extern",
        "finally",
        "fixed",
        "for",
        "foreach",
        "goto",
        "if",
        "implicit",
        "in",
        "interface",
        "internal",
        "is",
        "lock",
        "namespace",
        "new",
        "operator",
        "out",
        "override",
        "params",
        "private",
        "protected",
        "public",
        "readonly",
        "record",
        "ref",
        "return",
        "scoped",
        "sealed",
        "sizeof",
        "stackalloc",
        "static",
        "struct",
        "switch",
        "this",
        "throw",
        "try",
        "typeof",
        "unchecked",
        "unsafe",
        "using",
        "virtual",
        "void",
        "volatile",
        "while"
      ], J = [
        "add",
        "alias",
        "and",
        "ascending",
        "args",
        "async",
        "await",
        "by",
        "descending",
        "dynamic",
        "equals",
        "file",
        "from",
        "get",
        "global",
        "group",
        "init",
        "into",
        "join",
        "let",
        "nameof",
        "not",
        "notnull",
        "on",
        "or",
        "orderby",
        "partial",
        "record",
        "remove",
        "required",
        "scoped",
        "select",
        "set",
        "unmanaged",
        "value|0",
        "var",
        "when",
        "where",
        "with",
        "yield"
      ], Z = {
        keyword: G.concat(J),
        built_in: H,
        literal: q
      }, te = _.inherit(_.TITLE_MODE, { begin: "[a-zA-Z](\\.?\\w)*" }), ne = {
        className: "number",
        variants: [
          { begin: "\\b(0b[01']+)" },
          { begin: "(-?)\\b([\\d']+(\\.[\\d']*)?|\\.[\\d']+)(u|U|l|L|ul|UL|f|F|b|B)" },
          { begin: "(-?)(\\b0[xX][a-fA-F0-9']+|(\\b[\\d']+(\\.[\\d']*)?|\\.[\\d']+)([eE][-+]?[\\d']+)?)" }
        ],
        relevance: 0
      }, ee = {
        className: "string",
        begin: /"""("*)(?!")(.|\n)*?"""\1/,
        relevance: 1
      }, ae = {
        className: "string",
        begin: '@"',
        end: '"',
        contains: [{ begin: '""' }]
      }, oe = _.inherit(ae, { illegal: /\n/ }), se = {
        className: "subst",
        begin: /\{/,
        end: /\}/,
        keywords: Z
      }, ce = _.inherit(se, { illegal: /\n/ }), le = {
        className: "string",
        begin: /\$"/,
        end: '"',
        illegal: /\n/,
        contains: [
          { begin: /\{\{/ },
          { begin: /\}\}/ },
          _.BACKSLASH_ESCAPE,
          ce
        ]
      }, ue = {
        className: "string",
        begin: /\$@"/,
        end: '"',
        contains: [
          { begin: /\{\{/ },
          { begin: /\}\}/ },
          { begin: '""' },
          se
        ]
      }, de = _.inherit(ue, {
        illegal: /\n/,
        contains: [
          { begin: /\{\{/ },
          { begin: /\}\}/ },
          { begin: '""' },
          ce
        ]
      });
      se.contains = [
        ue,
        le,
        ae,
        _.APOS_STRING_MODE,
        _.QUOTE_STRING_MODE,
        ne,
        _.C_BLOCK_COMMENT_MODE
      ], ce.contains = [
        de,
        le,
        oe,
        _.APOS_STRING_MODE,
        _.QUOTE_STRING_MODE,
        ne,
        _.inherit(_.C_BLOCK_COMMENT_MODE, { illegal: /\n/ })
      ];
      const fe = { variants: [
        ee,
        ue,
        le,
        ae,
        _.APOS_STRING_MODE,
        _.QUOTE_STRING_MODE
      ] }, he = {
        begin: "<",
        end: ">",
        contains: [
          { beginKeywords: "in out" },
          te
        ]
      }, me = _.IDENT_RE + "(<" + _.IDENT_RE + "(\\s*,\\s*" + _.IDENT_RE + ")*>)?(\\[\\])?", pe = {
        // prevents expressions like `@class` from incorrect flagging
        // `class` as a keyword
        begin: "@" + _.IDENT_RE,
        relevance: 0
      };
      return {
        name: "C#",
        aliases: [
          "cs",
          "c#"
        ],
        keywords: Z,
        illegal: /::/,
        contains: [
          _.COMMENT(
            "///",
            "$",
            {
              returnBegin: !0,
              contains: [
                {
                  className: "doctag",
                  variants: [
                    {
                      begin: "///",
                      relevance: 0
                    },
                    { begin: "<!--|-->" },
                    {
                      begin: "</?",
                      end: ">"
                    }
                  ]
                }
              ]
            }
          ),
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "meta",
            begin: "#",
            end: "$",
            keywords: { keyword: "if else elif endif define undef warning error line region endregion pragma checksum" }
          },
          fe,
          ne,
          {
            beginKeywords: "class interface",
            relevance: 0,
            end: /[{;=]/,
            illegal: /[^\s:,]/,
            contains: [
              { beginKeywords: "where class" },
              te,
              he,
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          {
            beginKeywords: "namespace",
            relevance: 0,
            end: /[{;=]/,
            illegal: /[^\s:]/,
            contains: [
              te,
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          {
            beginKeywords: "record",
            relevance: 0,
            end: /[{;=]/,
            illegal: /[^\s:]/,
            contains: [
              te,
              he,
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          {
            // [Attributes("")]
            className: "meta",
            begin: "^\\s*\\[(?=[\\w])",
            excludeBegin: !0,
            end: "\\]",
            excludeEnd: !0,
            contains: [
              {
                className: "string",
                begin: /"/,
                end: /"/
              }
            ]
          },
          {
            // Expression keywords prevent 'keyword Name(...)' from being
            // recognized as a function definition
            beginKeywords: "new return throw await else",
            relevance: 0
          },
          {
            className: "function",
            begin: "(" + me + "\\s+)+" + _.IDENT_RE + "\\s*(<[^=]+>\\s*)?\\(",
            returnBegin: !0,
            end: /\s*[{;=]/,
            excludeEnd: !0,
            keywords: Z,
            contains: [
              // prevents these from being highlighted `title`
              {
                beginKeywords: V.join(" "),
                relevance: 0
              },
              {
                begin: _.IDENT_RE + "\\s*(<[^=]+>\\s*)?\\(",
                returnBegin: !0,
                contains: [
                  _.TITLE_MODE,
                  he
                ],
                relevance: 0
              },
              { match: /\(\)/ },
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                excludeBegin: !0,
                excludeEnd: !0,
                keywords: Z,
                relevance: 0,
                contains: [
                  fe,
                  ne,
                  _.C_BLOCK_COMMENT_MODE
                ]
              },
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          pe
        ]
      };
    }
    return csharp_1 = O, csharp_1;
  }
  var css_1, hasRequiredCss;
  function requireCss() {
    if (hasRequiredCss) return css_1;
    hasRequiredCss = 1;
    const O = (ne) => ({
      IMPORTANT: {
        scope: "meta",
        begin: "!important"
      },
      BLOCK_COMMENT: ne.C_BLOCK_COMMENT_MODE,
      HEXCOLOR: {
        scope: "number",
        begin: /#(([0-9a-fA-F]{3,4})|(([0-9a-fA-F]{2}){3,4}))\b/
      },
      FUNCTION_DISPATCH: {
        className: "built_in",
        begin: /[\w-]+(?=\()/
      },
      ATTRIBUTE_SELECTOR_MODE: {
        scope: "selector-attr",
        begin: /\[/,
        end: /\]/,
        illegal: "$",
        contains: [
          ne.APOS_STRING_MODE,
          ne.QUOTE_STRING_MODE
        ]
      },
      CSS_NUMBER_MODE: {
        scope: "number",
        begin: ne.NUMBER_RE + "(%|em|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc|px|deg|grad|rad|turn|s|ms|Hz|kHz|dpi|dpcm|dppx)?",
        relevance: 0
      },
      CSS_VARIABLE: {
        className: "attr",
        begin: /--[A-Za-z_][A-Za-z0-9_-]*/
      }
    }), _ = [
      "a",
      "abbr",
      "address",
      "article",
      "aside",
      "audio",
      "b",
      "blockquote",
      "body",
      "button",
      "canvas",
      "caption",
      "cite",
      "code",
      "dd",
      "del",
      "details",
      "dfn",
      "div",
      "dl",
      "dt",
      "em",
      "fieldset",
      "figcaption",
      "figure",
      "footer",
      "form",
      "h1",
      "h2",
      "h3",
      "h4",
      "h5",
      "h6",
      "header",
      "hgroup",
      "html",
      "i",
      "iframe",
      "img",
      "input",
      "ins",
      "kbd",
      "label",
      "legend",
      "li",
      "main",
      "mark",
      "menu",
      "nav",
      "object",
      "ol",
      "optgroup",
      "option",
      "p",
      "picture",
      "q",
      "quote",
      "samp",
      "section",
      "select",
      "source",
      "span",
      "strong",
      "summary",
      "sup",
      "table",
      "tbody",
      "td",
      "textarea",
      "tfoot",
      "th",
      "thead",
      "time",
      "tr",
      "ul",
      "var",
      "video"
    ], H = [
      "defs",
      "g",
      "marker",
      "mask",
      "pattern",
      "svg",
      "switch",
      "symbol",
      "feBlend",
      "feColorMatrix",
      "feComponentTransfer",
      "feComposite",
      "feConvolveMatrix",
      "feDiffuseLighting",
      "feDisplacementMap",
      "feFlood",
      "feGaussianBlur",
      "feImage",
      "feMerge",
      "feMorphology",
      "feOffset",
      "feSpecularLighting",
      "feTile",
      "feTurbulence",
      "linearGradient",
      "radialGradient",
      "stop",
      "circle",
      "ellipse",
      "image",
      "line",
      "path",
      "polygon",
      "polyline",
      "rect",
      "text",
      "use",
      "textPath",
      "tspan",
      "foreignObject",
      "clipPath"
    ], V = [
      ..._,
      ...H
    ], q = [
      "any-hover",
      "any-pointer",
      "aspect-ratio",
      "color",
      "color-gamut",
      "color-index",
      "device-aspect-ratio",
      "device-height",
      "device-width",
      "display-mode",
      "forced-colors",
      "grid",
      "height",
      "hover",
      "inverted-colors",
      "monochrome",
      "orientation",
      "overflow-block",
      "overflow-inline",
      "pointer",
      "prefers-color-scheme",
      "prefers-contrast",
      "prefers-reduced-motion",
      "prefers-reduced-transparency",
      "resolution",
      "scan",
      "scripting",
      "update",
      "width",
      // TODO: find a better solution?
      "min-width",
      "max-width",
      "min-height",
      "max-height"
    ].sort().reverse(), G = [
      "active",
      "any-link",
      "blank",
      "checked",
      "current",
      "default",
      "defined",
      "dir",
      // dir()
      "disabled",
      "drop",
      "empty",
      "enabled",
      "first",
      "first-child",
      "first-of-type",
      "fullscreen",
      "future",
      "focus",
      "focus-visible",
      "focus-within",
      "has",
      // has()
      "host",
      // host or host()
      "host-context",
      // host-context()
      "hover",
      "indeterminate",
      "in-range",
      "invalid",
      "is",
      // is()
      "lang",
      // lang()
      "last-child",
      "last-of-type",
      "left",
      "link",
      "local-link",
      "not",
      // not()
      "nth-child",
      // nth-child()
      "nth-col",
      // nth-col()
      "nth-last-child",
      // nth-last-child()
      "nth-last-col",
      // nth-last-col()
      "nth-last-of-type",
      //nth-last-of-type()
      "nth-of-type",
      //nth-of-type()
      "only-child",
      "only-of-type",
      "optional",
      "out-of-range",
      "past",
      "placeholder-shown",
      "read-only",
      "read-write",
      "required",
      "right",
      "root",
      "scope",
      "target",
      "target-within",
      "user-invalid",
      "valid",
      "visited",
      "where"
      // where()
    ].sort().reverse(), J = [
      "after",
      "backdrop",
      "before",
      "cue",
      "cue-region",
      "first-letter",
      "first-line",
      "grammar-error",
      "marker",
      "part",
      "placeholder",
      "selection",
      "slotted",
      "spelling-error"
    ].sort().reverse(), Z = [
      "accent-color",
      "align-content",
      "align-items",
      "align-self",
      "alignment-baseline",
      "all",
      "anchor-name",
      "animation",
      "animation-composition",
      "animation-delay",
      "animation-direction",
      "animation-duration",
      "animation-fill-mode",
      "animation-iteration-count",
      "animation-name",
      "animation-play-state",
      "animation-range",
      "animation-range-end",
      "animation-range-start",
      "animation-timeline",
      "animation-timing-function",
      "appearance",
      "aspect-ratio",
      "backdrop-filter",
      "backface-visibility",
      "background",
      "background-attachment",
      "background-blend-mode",
      "background-clip",
      "background-color",
      "background-image",
      "background-origin",
      "background-position",
      "background-position-x",
      "background-position-y",
      "background-repeat",
      "background-size",
      "baseline-shift",
      "block-size",
      "border",
      "border-block",
      "border-block-color",
      "border-block-end",
      "border-block-end-color",
      "border-block-end-style",
      "border-block-end-width",
      "border-block-start",
      "border-block-start-color",
      "border-block-start-style",
      "border-block-start-width",
      "border-block-style",
      "border-block-width",
      "border-bottom",
      "border-bottom-color",
      "border-bottom-left-radius",
      "border-bottom-right-radius",
      "border-bottom-style",
      "border-bottom-width",
      "border-collapse",
      "border-color",
      "border-end-end-radius",
      "border-end-start-radius",
      "border-image",
      "border-image-outset",
      "border-image-repeat",
      "border-image-slice",
      "border-image-source",
      "border-image-width",
      "border-inline",
      "border-inline-color",
      "border-inline-end",
      "border-inline-end-color",
      "border-inline-end-style",
      "border-inline-end-width",
      "border-inline-start",
      "border-inline-start-color",
      "border-inline-start-style",
      "border-inline-start-width",
      "border-inline-style",
      "border-inline-width",
      "border-left",
      "border-left-color",
      "border-left-style",
      "border-left-width",
      "border-radius",
      "border-right",
      "border-right-color",
      "border-right-style",
      "border-right-width",
      "border-spacing",
      "border-start-end-radius",
      "border-start-start-radius",
      "border-style",
      "border-top",
      "border-top-color",
      "border-top-left-radius",
      "border-top-right-radius",
      "border-top-style",
      "border-top-width",
      "border-width",
      "bottom",
      "box-align",
      "box-decoration-break",
      "box-direction",
      "box-flex",
      "box-flex-group",
      "box-lines",
      "box-ordinal-group",
      "box-orient",
      "box-pack",
      "box-shadow",
      "box-sizing",
      "break-after",
      "break-before",
      "break-inside",
      "caption-side",
      "caret-color",
      "clear",
      "clip",
      "clip-path",
      "clip-rule",
      "color",
      "color-interpolation",
      "color-interpolation-filters",
      "color-profile",
      "color-rendering",
      "color-scheme",
      "column-count",
      "column-fill",
      "column-gap",
      "column-rule",
      "column-rule-color",
      "column-rule-style",
      "column-rule-width",
      "column-span",
      "column-width",
      "columns",
      "contain",
      "contain-intrinsic-block-size",
      "contain-intrinsic-height",
      "contain-intrinsic-inline-size",
      "contain-intrinsic-size",
      "contain-intrinsic-width",
      "container",
      "container-name",
      "container-type",
      "content",
      "content-visibility",
      "counter-increment",
      "counter-reset",
      "counter-set",
      "cue",
      "cue-after",
      "cue-before",
      "cursor",
      "cx",
      "cy",
      "direction",
      "display",
      "dominant-baseline",
      "empty-cells",
      "enable-background",
      "field-sizing",
      "fill",
      "fill-opacity",
      "fill-rule",
      "filter",
      "flex",
      "flex-basis",
      "flex-direction",
      "flex-flow",
      "flex-grow",
      "flex-shrink",
      "flex-wrap",
      "float",
      "flood-color",
      "flood-opacity",
      "flow",
      "font",
      "font-display",
      "font-family",
      "font-feature-settings",
      "font-kerning",
      "font-language-override",
      "font-optical-sizing",
      "font-palette",
      "font-size",
      "font-size-adjust",
      "font-smooth",
      "font-smoothing",
      "font-stretch",
      "font-style",
      "font-synthesis",
      "font-synthesis-position",
      "font-synthesis-small-caps",
      "font-synthesis-style",
      "font-synthesis-weight",
      "font-variant",
      "font-variant-alternates",
      "font-variant-caps",
      "font-variant-east-asian",
      "font-variant-emoji",
      "font-variant-ligatures",
      "font-variant-numeric",
      "font-variant-position",
      "font-variation-settings",
      "font-weight",
      "forced-color-adjust",
      "gap",
      "glyph-orientation-horizontal",
      "glyph-orientation-vertical",
      "grid",
      "grid-area",
      "grid-auto-columns",
      "grid-auto-flow",
      "grid-auto-rows",
      "grid-column",
      "grid-column-end",
      "grid-column-start",
      "grid-gap",
      "grid-row",
      "grid-row-end",
      "grid-row-start",
      "grid-template",
      "grid-template-areas",
      "grid-template-columns",
      "grid-template-rows",
      "hanging-punctuation",
      "height",
      "hyphenate-character",
      "hyphenate-limit-chars",
      "hyphens",
      "icon",
      "image-orientation",
      "image-rendering",
      "image-resolution",
      "ime-mode",
      "initial-letter",
      "initial-letter-align",
      "inline-size",
      "inset",
      "inset-area",
      "inset-block",
      "inset-block-end",
      "inset-block-start",
      "inset-inline",
      "inset-inline-end",
      "inset-inline-start",
      "isolation",
      "justify-content",
      "justify-items",
      "justify-self",
      "kerning",
      "left",
      "letter-spacing",
      "lighting-color",
      "line-break",
      "line-height",
      "line-height-step",
      "list-style",
      "list-style-image",
      "list-style-position",
      "list-style-type",
      "margin",
      "margin-block",
      "margin-block-end",
      "margin-block-start",
      "margin-bottom",
      "margin-inline",
      "margin-inline-end",
      "margin-inline-start",
      "margin-left",
      "margin-right",
      "margin-top",
      "margin-trim",
      "marker",
      "marker-end",
      "marker-mid",
      "marker-start",
      "marks",
      "mask",
      "mask-border",
      "mask-border-mode",
      "mask-border-outset",
      "mask-border-repeat",
      "mask-border-slice",
      "mask-border-source",
      "mask-border-width",
      "mask-clip",
      "mask-composite",
      "mask-image",
      "mask-mode",
      "mask-origin",
      "mask-position",
      "mask-repeat",
      "mask-size",
      "mask-type",
      "masonry-auto-flow",
      "math-depth",
      "math-shift",
      "math-style",
      "max-block-size",
      "max-height",
      "max-inline-size",
      "max-width",
      "min-block-size",
      "min-height",
      "min-inline-size",
      "min-width",
      "mix-blend-mode",
      "nav-down",
      "nav-index",
      "nav-left",
      "nav-right",
      "nav-up",
      "none",
      "normal",
      "object-fit",
      "object-position",
      "offset",
      "offset-anchor",
      "offset-distance",
      "offset-path",
      "offset-position",
      "offset-rotate",
      "opacity",
      "order",
      "orphans",
      "outline",
      "outline-color",
      "outline-offset",
      "outline-style",
      "outline-width",
      "overflow",
      "overflow-anchor",
      "overflow-block",
      "overflow-clip-margin",
      "overflow-inline",
      "overflow-wrap",
      "overflow-x",
      "overflow-y",
      "overlay",
      "overscroll-behavior",
      "overscroll-behavior-block",
      "overscroll-behavior-inline",
      "overscroll-behavior-x",
      "overscroll-behavior-y",
      "padding",
      "padding-block",
      "padding-block-end",
      "padding-block-start",
      "padding-bottom",
      "padding-inline",
      "padding-inline-end",
      "padding-inline-start",
      "padding-left",
      "padding-right",
      "padding-top",
      "page",
      "page-break-after",
      "page-break-before",
      "page-break-inside",
      "paint-order",
      "pause",
      "pause-after",
      "pause-before",
      "perspective",
      "perspective-origin",
      "place-content",
      "place-items",
      "place-self",
      "pointer-events",
      "position",
      "position-anchor",
      "position-visibility",
      "print-color-adjust",
      "quotes",
      "r",
      "resize",
      "rest",
      "rest-after",
      "rest-before",
      "right",
      "rotate",
      "row-gap",
      "ruby-align",
      "ruby-position",
      "scale",
      "scroll-behavior",
      "scroll-margin",
      "scroll-margin-block",
      "scroll-margin-block-end",
      "scroll-margin-block-start",
      "scroll-margin-bottom",
      "scroll-margin-inline",
      "scroll-margin-inline-end",
      "scroll-margin-inline-start",
      "scroll-margin-left",
      "scroll-margin-right",
      "scroll-margin-top",
      "scroll-padding",
      "scroll-padding-block",
      "scroll-padding-block-end",
      "scroll-padding-block-start",
      "scroll-padding-bottom",
      "scroll-padding-inline",
      "scroll-padding-inline-end",
      "scroll-padding-inline-start",
      "scroll-padding-left",
      "scroll-padding-right",
      "scroll-padding-top",
      "scroll-snap-align",
      "scroll-snap-stop",
      "scroll-snap-type",
      "scroll-timeline",
      "scroll-timeline-axis",
      "scroll-timeline-name",
      "scrollbar-color",
      "scrollbar-gutter",
      "scrollbar-width",
      "shape-image-threshold",
      "shape-margin",
      "shape-outside",
      "shape-rendering",
      "speak",
      "speak-as",
      "src",
      // @font-face
      "stop-color",
      "stop-opacity",
      "stroke",
      "stroke-dasharray",
      "stroke-dashoffset",
      "stroke-linecap",
      "stroke-linejoin",
      "stroke-miterlimit",
      "stroke-opacity",
      "stroke-width",
      "tab-size",
      "table-layout",
      "text-align",
      "text-align-all",
      "text-align-last",
      "text-anchor",
      "text-combine-upright",
      "text-decoration",
      "text-decoration-color",
      "text-decoration-line",
      "text-decoration-skip",
      "text-decoration-skip-ink",
      "text-decoration-style",
      "text-decoration-thickness",
      "text-emphasis",
      "text-emphasis-color",
      "text-emphasis-position",
      "text-emphasis-style",
      "text-indent",
      "text-justify",
      "text-orientation",
      "text-overflow",
      "text-rendering",
      "text-shadow",
      "text-size-adjust",
      "text-transform",
      "text-underline-offset",
      "text-underline-position",
      "text-wrap",
      "text-wrap-mode",
      "text-wrap-style",
      "timeline-scope",
      "top",
      "touch-action",
      "transform",
      "transform-box",
      "transform-origin",
      "transform-style",
      "transition",
      "transition-behavior",
      "transition-delay",
      "transition-duration",
      "transition-property",
      "transition-timing-function",
      "translate",
      "unicode-bidi",
      "user-modify",
      "user-select",
      "vector-effect",
      "vertical-align",
      "view-timeline",
      "view-timeline-axis",
      "view-timeline-inset",
      "view-timeline-name",
      "view-transition-name",
      "visibility",
      "voice-balance",
      "voice-duration",
      "voice-family",
      "voice-pitch",
      "voice-range",
      "voice-rate",
      "voice-stress",
      "voice-volume",
      "white-space",
      "white-space-collapse",
      "widows",
      "width",
      "will-change",
      "word-break",
      "word-spacing",
      "word-wrap",
      "writing-mode",
      "x",
      "y",
      "z-index",
      "zoom"
    ].sort().reverse();
    function te(ne) {
      const ee = ne.regex, ae = O(ne), oe = { begin: /-(webkit|moz|ms|o)-(?=[a-z])/ }, se = "and or not only", ce = /@-?\w[\w]*(-\w+)*/, le = "[a-zA-Z-][a-zA-Z0-9_-]*", ue = [
        ne.APOS_STRING_MODE,
        ne.QUOTE_STRING_MODE
      ];
      return {
        name: "CSS",
        case_insensitive: !0,
        illegal: /[=|'\$]/,
        keywords: { keyframePosition: "from to" },
        classNameAliases: {
          // for visual continuity with `tag {}` and because we
          // don't have a great class for this?
          keyframePosition: "selector-tag"
        },
        contains: [
          ae.BLOCK_COMMENT,
          oe,
          // to recognize keyframe 40% etc which are outside the scope of our
          // attribute value mode
          ae.CSS_NUMBER_MODE,
          {
            className: "selector-id",
            begin: /#[A-Za-z0-9_-]+/,
            relevance: 0
          },
          {
            className: "selector-class",
            begin: "\\." + le,
            relevance: 0
          },
          ae.ATTRIBUTE_SELECTOR_MODE,
          {
            className: "selector-pseudo",
            variants: [
              { begin: ":(" + G.join("|") + ")" },
              { begin: ":(:)?(" + J.join("|") + ")" }
            ]
          },
          // we may actually need this (12/2020)
          // { // pseudo-selector params
          //   begin: /\(/,
          //   end: /\)/,
          //   contains: [ hljs.CSS_NUMBER_MODE ]
          // },
          ae.CSS_VARIABLE,
          {
            className: "attribute",
            begin: "\\b(" + Z.join("|") + ")\\b"
          },
          // attribute values
          {
            begin: /:/,
            end: /[;}{]/,
            contains: [
              ae.BLOCK_COMMENT,
              ae.HEXCOLOR,
              ae.IMPORTANT,
              ae.CSS_NUMBER_MODE,
              ...ue,
              // needed to highlight these as strings and to avoid issues with
              // illegal characters that might be inside urls that would tigger the
              // languages illegal stack
              {
                begin: /(url|data-uri)\(/,
                end: /\)/,
                relevance: 0,
                // from keywords
                keywords: { built_in: "url data-uri" },
                contains: [
                  ...ue,
                  {
                    className: "string",
                    // any character other than `)` as in `url()` will be the start
                    // of a string, which ends with `)` (from the parent mode)
                    begin: /[^)]/,
                    endsWithParent: !0,
                    excludeEnd: !0
                  }
                ]
              },
              ae.FUNCTION_DISPATCH
            ]
          },
          {
            begin: ee.lookahead(/@/),
            end: "[{;]",
            relevance: 0,
            illegal: /:/,
            // break on Less variables @var: ...
            contains: [
              {
                className: "keyword",
                begin: ce
              },
              {
                begin: /\s/,
                endsWithParent: !0,
                excludeEnd: !0,
                relevance: 0,
                keywords: {
                  $pattern: /[a-z-]+/,
                  keyword: se,
                  attribute: q.join(" ")
                },
                contains: [
                  {
                    begin: /[a-z-]+(?=:)/,
                    className: "attribute"
                  },
                  ...ue,
                  ae.CSS_NUMBER_MODE
                ]
              }
            ]
          },
          {
            className: "selector-tag",
            begin: "\\b(" + V.join("|") + ")\\b"
          }
        ]
      };
    }
    return css_1 = te, css_1;
  }
  var markdown_1, hasRequiredMarkdown;
  function requireMarkdown() {
    if (hasRequiredMarkdown) return markdown_1;
    hasRequiredMarkdown = 1;
    function O(_) {
      const H = _.regex, V = {
        begin: /<\/?[A-Za-z_]/,
        end: ">",
        subLanguage: "xml",
        relevance: 0
      }, q = {
        begin: "^[-\\*]{3,}",
        end: "$"
      }, G = {
        className: "code",
        variants: [
          // TODO: fix to allow these to work with sublanguage also
          { begin: "(`{3,})[^`](.|\\n)*?\\1`*[ ]*" },
          { begin: "(~{3,})[^~](.|\\n)*?\\1~*[ ]*" },
          // needed to allow markdown as a sublanguage to work
          {
            begin: "```",
            end: "```+[ ]*$"
          },
          {
            begin: "~~~",
            end: "~~~+[ ]*$"
          },
          { begin: "`.+?`" },
          {
            begin: "(?=^( {4}|\\t))",
            // use contains to gobble up multiple lines to allow the block to be whatever size
            // but only have a single open/close tag vs one per line
            contains: [
              {
                begin: "^( {4}|\\t)",
                end: "(\\n)$"
              }
            ],
            relevance: 0
          }
        ]
      }, J = {
        className: "bullet",
        begin: "^[ 	]*([*+-]|(\\d+\\.))(?=\\s+)",
        end: "\\s+",
        excludeEnd: !0
      }, Z = {
        begin: /^\[[^\n]+\]:/,
        returnBegin: !0,
        contains: [
          {
            className: "symbol",
            begin: /\[/,
            end: /\]/,
            excludeBegin: !0,
            excludeEnd: !0
          },
          {
            className: "link",
            begin: /:\s*/,
            end: /$/,
            excludeBegin: !0
          }
        ]
      }, te = /[A-Za-z][A-Za-z0-9+.-]*/, ne = {
        variants: [
          // too much like nested array access in so many languages
          // to have any real relevance
          {
            begin: /\[.+?\]\[.*?\]/,
            relevance: 0
          },
          // popular internet URLs
          {
            begin: /\[.+?\]\(((data|javascript|mailto):|(?:http|ftp)s?:\/\/).*?\)/,
            relevance: 2
          },
          {
            begin: H.concat(/\[.+?\]\(/, te, /:\/\/.*?\)/),
            relevance: 2
          },
          // relative urls
          {
            begin: /\[.+?\]\([./?&#].*?\)/,
            relevance: 1
          },
          // whatever else, lower relevance (might not be a link at all)
          {
            begin: /\[.*?\]\(.*?\)/,
            relevance: 0
          }
        ],
        returnBegin: !0,
        contains: [
          {
            // empty strings for alt or link text
            match: /\[(?=\])/
          },
          {
            className: "string",
            relevance: 0,
            begin: "\\[",
            end: "\\]",
            excludeBegin: !0,
            returnEnd: !0
          },
          {
            className: "link",
            relevance: 0,
            begin: "\\]\\(",
            end: "\\)",
            excludeBegin: !0,
            excludeEnd: !0
          },
          {
            className: "symbol",
            relevance: 0,
            begin: "\\]\\[",
            end: "\\]",
            excludeBegin: !0,
            excludeEnd: !0
          }
        ]
      }, ee = {
        className: "strong",
        contains: [],
        // defined later
        variants: [
          {
            begin: /_{2}(?!\s)/,
            end: /_{2}/
          },
          {
            begin: /\*{2}(?!\s)/,
            end: /\*{2}/
          }
        ]
      }, ae = {
        className: "emphasis",
        contains: [],
        // defined later
        variants: [
          {
            begin: /\*(?![*\s])/,
            end: /\*/
          },
          {
            begin: /_(?![_\s])/,
            end: /_/,
            relevance: 0
          }
        ]
      }, oe = _.inherit(ee, { contains: [] }), se = _.inherit(ae, { contains: [] });
      ee.contains.push(se), ae.contains.push(oe);
      let ce = [
        V,
        ne
      ];
      return [
        ee,
        ae,
        oe,
        se
      ].forEach((fe) => {
        fe.contains = fe.contains.concat(ce);
      }), ce = ce.concat(ee, ae), {
        name: "Markdown",
        aliases: [
          "md",
          "mkdown",
          "mkd"
        ],
        contains: [
          {
            className: "section",
            variants: [
              {
                begin: "^#{1,6}",
                end: "$",
                contains: ce
              },
              {
                begin: "(?=^.+?\\n[=-]{2,}$)",
                contains: [
                  { begin: "^[=-]*$" },
                  {
                    begin: "^",
                    end: "\\n",
                    contains: ce
                  }
                ]
              }
            ]
          },
          V,
          J,
          ee,
          ae,
          {
            className: "quote",
            begin: "^>\\s+",
            contains: ce,
            end: "$"
          },
          G,
          q,
          ne,
          Z,
          {
            //https://spec.commonmark.org/0.31.2/#entity-references
            scope: "literal",
            match: /&([a-zA-Z0-9]+|#[0-9]{1,7}|#[Xx][0-9a-fA-F]{1,6});/
          }
        ]
      };
    }
    return markdown_1 = O, markdown_1;
  }
  var diff_1, hasRequiredDiff;
  function requireDiff() {
    if (hasRequiredDiff) return diff_1;
    hasRequiredDiff = 1;
    function O(_) {
      const H = _.regex;
      return {
        name: "Diff",
        aliases: ["patch"],
        contains: [
          {
            className: "meta",
            relevance: 10,
            match: H.either(
              /^@@ +-\d+,\d+ +\+\d+,\d+ +@@/,
              /^\*\*\* +\d+,\d+ +\*\*\*\*$/,
              /^--- +\d+,\d+ +----$/
            )
          },
          {
            className: "comment",
            variants: [
              {
                begin: H.either(
                  /Index: /,
                  /^index/,
                  /={3,}/,
                  /^-{3}/,
                  /^\*{3} /,
                  /^\+{3}/,
                  /^diff --git/
                ),
                end: /$/
              },
              { match: /^\*{15}$/ }
            ]
          },
          {
            className: "addition",
            begin: /^\+/,
            end: /$/
          },
          {
            className: "deletion",
            begin: /^-/,
            end: /$/
          },
          {
            className: "addition",
            begin: /^!/,
            end: /$/
          }
        ]
      };
    }
    return diff_1 = O, diff_1;
  }
  var ruby_1, hasRequiredRuby;
  function requireRuby() {
    if (hasRequiredRuby) return ruby_1;
    hasRequiredRuby = 1;
    function O(_) {
      const H = _.regex, V = "([a-zA-Z_]\\w*[!?=]?|[-+~]@|<<|>>|=~|===?|<=>|[<>]=?|\\*\\*|[-/+%^&*~`|]|\\[\\]=?)", q = H.either(
        /\b([A-Z]+[a-z0-9]+)+/,
        // ends in caps
        /\b([A-Z]+[a-z0-9]+)+[A-Z]+/
      ), G = H.concat(q, /(::\w+)*/), Z = {
        "variable.constant": [
          "__FILE__",
          "__LINE__",
          "__ENCODING__"
        ],
        "variable.language": [
          "self",
          "super"
        ],
        keyword: [
          "alias",
          "and",
          "begin",
          "BEGIN",
          "break",
          "case",
          "class",
          "defined",
          "do",
          "else",
          "elsif",
          "end",
          "END",
          "ensure",
          "for",
          "if",
          "in",
          "module",
          "next",
          "not",
          "or",
          "redo",
          "require",
          "rescue",
          "retry",
          "return",
          "then",
          "undef",
          "unless",
          "until",
          "when",
          "while",
          "yield",
          ...[
            "include",
            "extend",
            "prepend",
            "public",
            "private",
            "protected",
            "raise",
            "throw"
          ]
        ],
        built_in: [
          "proc",
          "lambda",
          "attr_accessor",
          "attr_reader",
          "attr_writer",
          "define_method",
          "private_constant",
          "module_function"
        ],
        literal: [
          "true",
          "false",
          "nil"
        ]
      }, te = {
        className: "doctag",
        begin: "@[A-Za-z]+"
      }, ne = {
        begin: "#<",
        end: ">"
      }, ee = [
        _.COMMENT(
          "#",
          "$",
          { contains: [te] }
        ),
        _.COMMENT(
          "^=begin",
          "^=end",
          {
            contains: [te],
            relevance: 10
          }
        ),
        _.COMMENT("^__END__", _.MATCH_NOTHING_RE)
      ], ae = {
        className: "subst",
        begin: /#\{/,
        end: /\}/,
        keywords: Z
      }, oe = {
        className: "string",
        contains: [
          _.BACKSLASH_ESCAPE,
          ae
        ],
        variants: [
          {
            begin: /'/,
            end: /'/
          },
          {
            begin: /"/,
            end: /"/
          },
          {
            begin: /`/,
            end: /`/
          },
          {
            begin: /%[qQwWx]?\(/,
            end: /\)/
          },
          {
            begin: /%[qQwWx]?\[/,
            end: /\]/
          },
          {
            begin: /%[qQwWx]?\{/,
            end: /\}/
          },
          {
            begin: /%[qQwWx]?</,
            end: />/
          },
          {
            begin: /%[qQwWx]?\//,
            end: /\//
          },
          {
            begin: /%[qQwWx]?%/,
            end: /%/
          },
          {
            begin: /%[qQwWx]?-/,
            end: /-/
          },
          {
            begin: /%[qQwWx]?\|/,
            end: /\|/
          },
          // in the following expressions, \B in the beginning suppresses recognition of ?-sequences
          // where ? is the last character of a preceding identifier, as in: `func?4`
          { begin: /\B\?(\\\d{1,3})/ },
          { begin: /\B\?(\\x[A-Fa-f0-9]{1,2})/ },
          { begin: /\B\?(\\u\{?[A-Fa-f0-9]{1,6}\}?)/ },
          { begin: /\B\?(\\M-\\C-|\\M-\\c|\\c\\M-|\\M-|\\C-\\M-)[\x20-\x7e]/ },
          { begin: /\B\?\\(c|C-)[\x20-\x7e]/ },
          { begin: /\B\?\\?\S/ },
          // heredocs
          {
            // this guard makes sure that we have an entire heredoc and not a false
            // positive (auto-detect, etc.)
            begin: H.concat(
              /<<[-~]?'?/,
              H.lookahead(/(\w+)(?=\W)[^\n]*\n(?:[^\n]*\n)*?\s*\1\b/)
            ),
            contains: [
              _.END_SAME_AS_BEGIN({
                begin: /(\w+)/,
                end: /(\w+)/,
                contains: [
                  _.BACKSLASH_ESCAPE,
                  ae
                ]
              })
            ]
          }
        ]
      }, se = "[1-9](_?[0-9])*|0", ce = "[0-9](_?[0-9])*", le = {
        className: "number",
        relevance: 0,
        variants: [
          // decimal integer/float, optionally exponential or rational, optionally imaginary
          { begin: `\\b(${se})(\\.(${ce}))?([eE][+-]?(${ce})|r)?i?\\b` },
          // explicit decimal/binary/octal/hexadecimal integer,
          // optionally rational and/or imaginary
          { begin: "\\b0[dD][0-9](_?[0-9])*r?i?\\b" },
          { begin: "\\b0[bB][0-1](_?[0-1])*r?i?\\b" },
          { begin: "\\b0[oO][0-7](_?[0-7])*r?i?\\b" },
          { begin: "\\b0[xX][0-9a-fA-F](_?[0-9a-fA-F])*r?i?\\b" },
          // 0-prefixed implicit octal integer, optionally rational and/or imaginary
          { begin: "\\b0(_?[0-7])+r?i?\\b" }
        ]
      }, ue = {
        variants: [
          {
            match: /\(\)/
          },
          {
            className: "params",
            begin: /\(/,
            end: /(?=\))/,
            excludeBegin: !0,
            endsParent: !0,
            keywords: Z
          }
        ]
      }, ge = [
        oe,
        {
          variants: [
            {
              match: [
                /class\s+/,
                G,
                /\s+<\s+/,
                G
              ]
            },
            {
              match: [
                /\b(class|module)\s+/,
                G
              ]
            }
          ],
          scope: {
            2: "title.class",
            4: "title.class.inherited"
          },
          keywords: Z
        },
        {
          match: [
            /(include|extend)\s+/,
            G
          ],
          scope: {
            2: "title.class"
          },
          keywords: Z
        },
        {
          relevance: 0,
          match: [
            G,
            /\.new[. (]/
          ],
          scope: {
            1: "title.class"
          }
        },
        {
          relevance: 0,
          match: /\b[A-Z][A-Z_0-9]+\b/,
          className: "variable.constant"
        },
        {
          relevance: 0,
          match: q,
          scope: "title.class"
        },
        {
          match: [
            /def/,
            /\s+/,
            V
          ],
          scope: {
            1: "keyword",
            3: "title.function"
          },
          contains: [
            ue
          ]
        },
        {
          // swallow namespace qualifiers before symbols
          begin: _.IDENT_RE + "::"
        },
        {
          className: "symbol",
          begin: _.UNDERSCORE_IDENT_RE + "(!|\\?)?:",
          relevance: 0
        },
        {
          className: "symbol",
          begin: ":(?!\\s)",
          contains: [
            oe,
            { begin: V }
          ],
          relevance: 0
        },
        le,
        {
          // negative-look forward attempts to prevent false matches like:
          // @ident@ or $ident$ that might indicate this is not ruby at all
          className: "variable",
          begin: "(\\$\\W)|((\\$|@@?)(\\w+))(?=[^@$?])(?![A-Za-z])(?![@$?'])"
        },
        {
          className: "params",
          begin: /\|(?!=)/,
          end: /\|/,
          excludeBegin: !0,
          excludeEnd: !0,
          relevance: 0,
          // this could be a lot of things (in other languages) other than params
          keywords: Z
        },
        {
          // regexp container
          begin: "(" + _.RE_STARTERS_RE + "|unless)\\s*",
          keywords: "unless",
          contains: [
            {
              className: "regexp",
              contains: [
                _.BACKSLASH_ESCAPE,
                ae
              ],
              illegal: /\n/,
              variants: [
                {
                  begin: "/",
                  end: "/[a-z]*"
                },
                {
                  begin: /%r\{/,
                  end: /\}[a-z]*/
                },
                {
                  begin: "%r\\(",
                  end: "\\)[a-z]*"
                },
                {
                  begin: "%r!",
                  end: "![a-z]*"
                },
                {
                  begin: "%r\\[",
                  end: "\\][a-z]*"
                }
              ]
            }
          ].concat(ne, ee),
          relevance: 0
        }
      ].concat(ne, ee);
      ae.contains = ge, ue.contains = ge;
      const Se = [
        {
          begin: /^\s*=>/,
          starts: {
            end: "$",
            contains: ge
          }
        },
        {
          className: "meta.prompt",
          begin: "^(" + "[>?]>" + "|" + "[\\w#]+\\(\\w+\\):\\d+:\\d+[>*]" + "|" + "(\\w+-)?\\d+\\.\\d+\\.\\d+(p\\d+)?[^\\d][^>]+>" + ")(?=[ ])",
          starts: {
            end: "$",
            keywords: Z,
            contains: ge
          }
        }
      ];
      return ee.unshift(ne), {
        name: "Ruby",
        aliases: [
          "rb",
          "gemspec",
          "podspec",
          "thor",
          "irb"
        ],
        keywords: Z,
        illegal: /\/\*/,
        contains: [_.SHEBANG({ binary: "ruby" })].concat(Se).concat(ee).concat(ge)
      };
    }
    return ruby_1 = O, ruby_1;
  }
  var go_1, hasRequiredGo;
  function requireGo() {
    if (hasRequiredGo) return go_1;
    hasRequiredGo = 1;
    function O(_) {
      const J = {
        keyword: [
          "break",
          "case",
          "chan",
          "const",
          "continue",
          "default",
          "defer",
          "else",
          "fallthrough",
          "for",
          "func",
          "go",
          "goto",
          "if",
          "import",
          "interface",
          "map",
          "package",
          "range",
          "return",
          "select",
          "struct",
          "switch",
          "type",
          "var"
        ],
        type: [
          "bool",
          "byte",
          "complex64",
          "complex128",
          "error",
          "float32",
          "float64",
          "int8",
          "int16",
          "int32",
          "int64",
          "string",
          "uint8",
          "uint16",
          "uint32",
          "uint64",
          "int",
          "uint",
          "uintptr",
          "rune"
        ],
        literal: [
          "true",
          "false",
          "iota",
          "nil"
        ],
        built_in: [
          "append",
          "cap",
          "close",
          "complex",
          "copy",
          "imag",
          "len",
          "make",
          "new",
          "panic",
          "print",
          "println",
          "real",
          "recover",
          "delete"
        ]
      };
      return {
        name: "Go",
        aliases: ["golang"],
        keywords: J,
        illegal: "</",
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "string",
            variants: [
              _.QUOTE_STRING_MODE,
              _.APOS_STRING_MODE,
              {
                begin: "`",
                end: "`"
              }
            ]
          },
          {
            className: "number",
            variants: [
              {
                match: /-?\b0[xX]\.[a-fA-F0-9](_?[a-fA-F0-9])*[pP][+-]?\d(_?\d)*i?/,
                // hex without a present digit before . (making a digit afterwards required)
                relevance: 0
              },
              {
                match: /-?\b0[xX](_?[a-fA-F0-9])+((\.([a-fA-F0-9](_?[a-fA-F0-9])*)?)?[pP][+-]?\d(_?\d)*)?i?/,
                // hex with a present digit before . (making a digit afterwards optional)
                relevance: 0
              },
              {
                match: /-?\b0[oO](_?[0-7])*i?/,
                // leading 0o octal
                relevance: 0
              },
              {
                match: /-?\.\d(_?\d)*([eE][+-]?\d(_?\d)*)?i?/,
                // decimal without a present digit before . (making a digit afterwards required)
                relevance: 0
              },
              {
                match: /-?\b\d(_?\d)*(\.(\d(_?\d)*)?)?([eE][+-]?\d(_?\d)*)?i?/,
                // decimal with a present digit before . (making a digit afterwards optional)
                relevance: 0
              }
            ]
          },
          {
            begin: /:=/
            // relevance booster
          },
          {
            className: "function",
            beginKeywords: "func",
            end: "\\s*(\\{|$)",
            excludeEnd: !0,
            contains: [
              _.TITLE_MODE,
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                endsParent: !0,
                keywords: J,
                illegal: /["']/
              }
            ]
          }
        ]
      };
    }
    return go_1 = O, go_1;
  }
  var graphql_1, hasRequiredGraphql;
  function requireGraphql() {
    if (hasRequiredGraphql) return graphql_1;
    hasRequiredGraphql = 1;
    function O(_) {
      const H = _.regex, V = /[_A-Za-z][_0-9A-Za-z]*/;
      return {
        name: "GraphQL",
        aliases: ["gql"],
        case_insensitive: !0,
        disableAutodetect: !1,
        keywords: {
          keyword: [
            "query",
            "mutation",
            "subscription",
            "type",
            "input",
            "schema",
            "directive",
            "interface",
            "union",
            "scalar",
            "fragment",
            "enum",
            "on"
          ],
          literal: [
            "true",
            "false",
            "null"
          ]
        },
        contains: [
          _.HASH_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE,
          {
            scope: "punctuation",
            match: /[.]{3}/,
            relevance: 0
          },
          {
            scope: "punctuation",
            begin: /[\!\(\)\:\=\[\]\{\|\}]{1}/,
            relevance: 0
          },
          {
            scope: "variable",
            begin: /\$/,
            end: /\W/,
            excludeEnd: !0,
            relevance: 0
          },
          {
            scope: "meta",
            match: /@\w+/,
            excludeEnd: !0
          },
          {
            scope: "symbol",
            begin: H.concat(V, H.lookahead(/\s*:/)),
            relevance: 0
          }
        ],
        illegal: [
          /[;<']/,
          /BEGIN/
        ]
      };
    }
    return graphql_1 = O, graphql_1;
  }
  var ini_1, hasRequiredIni;
  function requireIni() {
    if (hasRequiredIni) return ini_1;
    hasRequiredIni = 1;
    function O(_) {
      const H = _.regex, V = {
        className: "number",
        relevance: 0,
        variants: [
          { begin: /([+-]+)?[\d]+_[\d_]+/ },
          { begin: _.NUMBER_RE }
        ]
      }, q = _.COMMENT();
      q.variants = [
        {
          begin: /;/,
          end: /$/
        },
        {
          begin: /#/,
          end: /$/
        }
      ];
      const G = {
        className: "variable",
        variants: [
          { begin: /\$[\w\d"][\w\d_]*/ },
          { begin: /\$\{(.*?)\}/ }
        ]
      }, J = {
        className: "literal",
        begin: /\bon|off|true|false|yes|no\b/
      }, Z = {
        className: "string",
        contains: [_.BACKSLASH_ESCAPE],
        variants: [
          {
            begin: "'''",
            end: "'''",
            relevance: 10
          },
          {
            begin: '"""',
            end: '"""',
            relevance: 10
          },
          {
            begin: '"',
            end: '"'
          },
          {
            begin: "'",
            end: "'"
          }
        ]
      }, te = {
        begin: /\[/,
        end: /\]/,
        contains: [
          q,
          J,
          G,
          Z,
          V,
          "self"
        ],
        relevance: 0
      }, ne = /[A-Za-z0-9_-]+/, ee = /"(\\"|[^"])*"/, ae = /'[^']*'/, oe = H.either(
        ne,
        ee,
        ae
      ), se = H.concat(
        oe,
        "(\\s*\\.\\s*",
        oe,
        ")*",
        H.lookahead(/\s*=\s*[^#\s]/)
      );
      return {
        name: "TOML, also INI",
        aliases: ["toml"],
        case_insensitive: !0,
        illegal: /\S/,
        contains: [
          q,
          {
            className: "section",
            begin: /\[+/,
            end: /\]+/
          },
          {
            begin: se,
            className: "attr",
            starts: {
              end: /$/,
              contains: [
                q,
                te,
                J,
                G,
                Z,
                V
              ]
            }
          }
        ]
      };
    }
    return ini_1 = O, ini_1;
  }
  var java_1, hasRequiredJava;
  function requireJava() {
    if (hasRequiredJava) return java_1;
    hasRequiredJava = 1;
    var O = "[0-9](_*[0-9])*", _ = `\\.(${O})`, H = "[0-9a-fA-F](_*[0-9a-fA-F])*", V = {
      className: "number",
      variants: [
        // DecimalFloatingPointLiteral
        // including ExponentPart
        { begin: `(\\b(${O})((${_})|\\.)?|(${_}))[eE][+-]?(${O})[fFdD]?\\b` },
        // excluding ExponentPart
        { begin: `\\b(${O})((${_})[fFdD]?\\b|\\.([fFdD]\\b)?)` },
        { begin: `(${_})[fFdD]?\\b` },
        { begin: `\\b(${O})[fFdD]\\b` },
        // HexadecimalFloatingPointLiteral
        { begin: `\\b0[xX]((${H})\\.?|(${H})?\\.(${H}))[pP][+-]?(${O})[fFdD]?\\b` },
        // DecimalIntegerLiteral
        { begin: "\\b(0|[1-9](_*[0-9])*)[lL]?\\b" },
        // HexIntegerLiteral
        { begin: `\\b0[xX](${H})[lL]?\\b` },
        // OctalIntegerLiteral
        { begin: "\\b0(_*[0-7])*[lL]?\\b" },
        // BinaryIntegerLiteral
        { begin: "\\b0[bB][01](_*[01])*[lL]?\\b" }
      ],
      relevance: 0
    };
    function q(J, Z, te) {
      return te === -1 ? "" : J.replace(Z, (ne) => q(J, Z, te - 1));
    }
    function G(J) {
      const Z = J.regex, te = "[À-ʸa-zA-Z_$][À-ʸa-zA-Z_$0-9]*", ne = te + q("(?:<" + te + "~~~(?:\\s*,\\s*" + te + "~~~)*>)?", /~~~/g, 2), ce = {
        keyword: [
          "synchronized",
          "abstract",
          "private",
          "var",
          "static",
          "if",
          "const ",
          "for",
          "while",
          "strictfp",
          "finally",
          "protected",
          "import",
          "native",
          "final",
          "void",
          "enum",
          "else",
          "break",
          "transient",
          "catch",
          "instanceof",
          "volatile",
          "case",
          "assert",
          "package",
          "default",
          "public",
          "try",
          "switch",
          "continue",
          "throws",
          "protected",
          "public",
          "private",
          "module",
          "requires",
          "exports",
          "do",
          "sealed",
          "yield",
          "permits",
          "goto",
          "when"
        ],
        literal: [
          "false",
          "true",
          "null"
        ],
        type: [
          "char",
          "boolean",
          "long",
          "float",
          "int",
          "byte",
          "short",
          "double"
        ],
        built_in: [
          "super",
          "this"
        ]
      }, le = {
        className: "meta",
        begin: "@" + te,
        contains: [
          {
            begin: /\(/,
            end: /\)/,
            contains: ["self"]
            // allow nested () inside our annotation
          }
        ]
      }, ue = {
        className: "params",
        begin: /\(/,
        end: /\)/,
        keywords: ce,
        relevance: 0,
        contains: [J.C_BLOCK_COMMENT_MODE],
        endsParent: !0
      };
      return {
        name: "Java",
        aliases: ["jsp"],
        keywords: ce,
        illegal: /<\/|#/,
        contains: [
          J.COMMENT(
            "/\\*\\*",
            "\\*/",
            {
              relevance: 0,
              contains: [
                {
                  // eat up @'s in emails to prevent them to be recognized as doctags
                  begin: /\w+@/,
                  relevance: 0
                },
                {
                  className: "doctag",
                  begin: "@[A-Za-z]+"
                }
              ]
            }
          ),
          // relevance boost
          {
            begin: /import java\.[a-z]+\./,
            keywords: "import",
            relevance: 2
          },
          J.C_LINE_COMMENT_MODE,
          J.C_BLOCK_COMMENT_MODE,
          {
            begin: /"""/,
            end: /"""/,
            className: "string",
            contains: [J.BACKSLASH_ESCAPE]
          },
          J.APOS_STRING_MODE,
          J.QUOTE_STRING_MODE,
          {
            match: [
              /\b(?:class|interface|enum|extends|implements|new)/,
              /\s+/,
              te
            ],
            className: {
              1: "keyword",
              3: "title.class"
            }
          },
          {
            // Exceptions for hyphenated keywords
            match: /non-sealed/,
            scope: "keyword"
          },
          {
            begin: [
              Z.concat(/(?!else)/, te),
              /\s+/,
              te,
              /\s+/,
              /=(?!=)/
            ],
            className: {
              1: "type",
              3: "variable",
              5: "operator"
            }
          },
          {
            begin: [
              /record/,
              /\s+/,
              te
            ],
            className: {
              1: "keyword",
              3: "title.class"
            },
            contains: [
              ue,
              J.C_LINE_COMMENT_MODE,
              J.C_BLOCK_COMMENT_MODE
            ]
          },
          {
            // Expression keywords prevent 'keyword Name(...)' from being
            // recognized as a function definition
            beginKeywords: "new throw return else",
            relevance: 0
          },
          {
            begin: [
              "(?:" + ne + "\\s+)",
              J.UNDERSCORE_IDENT_RE,
              /\s*(?=\()/
            ],
            className: { 2: "title.function" },
            keywords: ce,
            contains: [
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                keywords: ce,
                relevance: 0,
                contains: [
                  le,
                  J.APOS_STRING_MODE,
                  J.QUOTE_STRING_MODE,
                  V,
                  J.C_BLOCK_COMMENT_MODE
                ]
              },
              J.C_LINE_COMMENT_MODE,
              J.C_BLOCK_COMMENT_MODE
            ]
          },
          V,
          le
        ]
      };
    }
    return java_1 = G, java_1;
  }
  var javascript_1, hasRequiredJavascript;
  function requireJavascript() {
    if (hasRequiredJavascript) return javascript_1;
    hasRequiredJavascript = 1;
    const O = "[A-Za-z$_][0-9A-Za-z$_]*", _ = [
      "as",
      // for exports
      "in",
      "of",
      "if",
      "for",
      "while",
      "finally",
      "var",
      "new",
      "function",
      "do",
      "return",
      "void",
      "else",
      "break",
      "catch",
      "instanceof",
      "with",
      "throw",
      "case",
      "default",
      "try",
      "switch",
      "continue",
      "typeof",
      "delete",
      "let",
      "yield",
      "const",
      "class",
      // JS handles these with a special rule
      // "get",
      // "set",
      "debugger",
      "async",
      "await",
      "static",
      "import",
      "from",
      "export",
      "extends",
      // It's reached stage 3, which is "recommended for implementation":
      "using"
    ], H = [
      "true",
      "false",
      "null",
      "undefined",
      "NaN",
      "Infinity"
    ], V = [
      // Fundamental objects
      "Object",
      "Function",
      "Boolean",
      "Symbol",
      // numbers and dates
      "Math",
      "Date",
      "Number",
      "BigInt",
      // text
      "String",
      "RegExp",
      // Indexed collections
      "Array",
      "Float32Array",
      "Float64Array",
      "Int8Array",
      "Uint8Array",
      "Uint8ClampedArray",
      "Int16Array",
      "Int32Array",
      "Uint16Array",
      "Uint32Array",
      "BigInt64Array",
      "BigUint64Array",
      // Keyed collections
      "Set",
      "Map",
      "WeakSet",
      "WeakMap",
      // Structured data
      "ArrayBuffer",
      "SharedArrayBuffer",
      "Atomics",
      "DataView",
      "JSON",
      // Control abstraction objects
      "Promise",
      "Generator",
      "GeneratorFunction",
      "AsyncFunction",
      // Reflection
      "Reflect",
      "Proxy",
      // Internationalization
      "Intl",
      // WebAssembly
      "WebAssembly"
    ], q = [
      "Error",
      "EvalError",
      "InternalError",
      "RangeError",
      "ReferenceError",
      "SyntaxError",
      "TypeError",
      "URIError"
    ], G = [
      "setInterval",
      "setTimeout",
      "clearInterval",
      "clearTimeout",
      "require",
      "exports",
      "eval",
      "isFinite",
      "isNaN",
      "parseFloat",
      "parseInt",
      "decodeURI",
      "decodeURIComponent",
      "encodeURI",
      "encodeURIComponent",
      "escape",
      "unescape"
    ], J = [
      "arguments",
      "this",
      "super",
      "console",
      "window",
      "document",
      "localStorage",
      "sessionStorage",
      "module",
      "global"
      // Node.js
    ], Z = [].concat(
      G,
      V,
      q
    );
    function te(ne) {
      const ee = ne.regex, ae = (Ge, { after: ze }) => {
        const We = "</" + Ge[0].slice(1);
        return Ge.input.indexOf(We, ze) !== -1;
      }, oe = O, se = {
        begin: "<>",
        end: "</>"
      }, ce = /<[A-Za-z0-9\\._:-]+\s*\/>/, le = {
        begin: /<[A-Za-z0-9\\._:-]+/,
        end: /\/[A-Za-z0-9\\._:-]+>|\/>/,
        /**
         * @param {RegExpMatchArray} match
         * @param {CallbackResponse} response
         */
        isTrulyOpeningTag: (Ge, ze) => {
          const We = Ge[0].length + Ge.index, at = Ge.input[We];
          if (
            // HTML should not include another raw `<` inside a tag
            // nested type?
            // `<Array<Array<number>>`, etc.
            at === "<" || // the , gives away that this is not HTML
            // `<T, A extends keyof T, V>`
            at === ","
          ) {
            ze.ignoreMatch();
            return;
          }
          at === ">" && (ae(Ge, { after: We }) || ze.ignoreMatch());
          let ct;
          const et = Ge.input.substring(We);
          if (ct = et.match(/^\s*=/)) {
            ze.ignoreMatch();
            return;
          }
          if ((ct = et.match(/^\s+extends\s+/)) && ct.index === 0) {
            ze.ignoreMatch();
            return;
          }
        }
      }, ue = {
        $pattern: O,
        keyword: _,
        literal: H,
        built_in: Z,
        "variable.language": J
      }, de = "[0-9](_?[0-9])*", fe = `\\.(${de})`, he = "0|[1-9](_?[0-9])*|0[0-7]*[89][0-9]*", me = {
        className: "number",
        variants: [
          // DecimalLiteral
          { begin: `(\\b(${he})((${fe})|\\.)?|(${fe}))[eE][+-]?(${de})\\b` },
          { begin: `\\b(${he})\\b((${fe})\\b|\\.)?|(${fe})\\b` },
          // DecimalBigIntegerLiteral
          { begin: "\\b(0|[1-9](_?[0-9])*)n\\b" },
          // NonDecimalIntegerLiteral
          { begin: "\\b0[xX][0-9a-fA-F](_?[0-9a-fA-F])*n?\\b" },
          { begin: "\\b0[bB][0-1](_?[0-1])*n?\\b" },
          { begin: "\\b0[oO][0-7](_?[0-7])*n?\\b" },
          // LegacyOctalIntegerLiteral (does not include underscore separators)
          // https://tc39.es/ecma262/#sec-additional-syntax-numeric-literals
          { begin: "\\b0[0-7]+n?\\b" }
        ],
        relevance: 0
      }, pe = {
        className: "subst",
        begin: "\\$\\{",
        end: "\\}",
        keywords: ue,
        contains: []
        // defined later
      }, ye = {
        begin: ".?html`",
        end: "",
        starts: {
          end: "`",
          returnEnd: !1,
          contains: [
            ne.BACKSLASH_ESCAPE,
            pe
          ],
          subLanguage: "xml"
        }
      }, ge = {
        begin: ".?css`",
        end: "",
        starts: {
          end: "`",
          returnEnd: !1,
          contains: [
            ne.BACKSLASH_ESCAPE,
            pe
          ],
          subLanguage: "css"
        }
      }, ve = {
        begin: ".?gql`",
        end: "",
        starts: {
          end: "`",
          returnEnd: !1,
          contains: [
            ne.BACKSLASH_ESCAPE,
            pe
          ],
          subLanguage: "graphql"
        }
      }, be = {
        className: "string",
        begin: "`",
        end: "`",
        contains: [
          ne.BACKSLASH_ESCAPE,
          pe
        ]
      }, Se = {
        className: "comment",
        variants: [
          ne.COMMENT(
            /\/\*\*(?!\/)/,
            "\\*/",
            {
              relevance: 0,
              contains: [
                {
                  begin: "(?=@[A-Za-z]+)",
                  relevance: 0,
                  contains: [
                    {
                      className: "doctag",
                      begin: "@[A-Za-z]+"
                    },
                    {
                      className: "type",
                      begin: "\\{",
                      end: "\\}",
                      excludeEnd: !0,
                      excludeBegin: !0,
                      relevance: 0
                    },
                    {
                      className: "variable",
                      begin: oe + "(?=\\s*(-)|$)",
                      endsParent: !0,
                      relevance: 0
                    },
                    // eat spaces (not newlines) so we can find
                    // types or variables
                    {
                      begin: /(?=[^\n])\s/,
                      relevance: 0
                    }
                  ]
                }
              ]
            }
          ),
          ne.C_BLOCK_COMMENT_MODE,
          ne.C_LINE_COMMENT_MODE
        ]
      }, we = [
        ne.APOS_STRING_MODE,
        ne.QUOTE_STRING_MODE,
        ye,
        ge,
        ve,
        be,
        // Skip numbers when they are part of a variable name
        { match: /\$\d+/ },
        me
        // This is intentional:
        // See https://github.com/highlightjs/highlight.js/issues/3288
        // hljs.REGEXP_MODE
      ];
      pe.contains = we.concat({
        // we need to pair up {} inside our subst to prevent
        // it from ending too early by matching another }
        begin: /\{/,
        end: /\}/,
        keywords: ue,
        contains: [
          "self"
        ].concat(we)
      });
      const Ee = [].concat(Se, pe.contains), xe = Ee.concat([
        // eat recursive parens in sub expressions
        {
          begin: /(\s*)\(/,
          end: /\)/,
          keywords: ue,
          contains: ["self"].concat(Ee)
        }
      ]), Le = {
        className: "params",
        // convert this to negative lookbehind in v12
        begin: /(\s*)\(/,
        // to match the parms with
        end: /\)/,
        excludeBegin: !0,
        excludeEnd: !0,
        keywords: ue,
        contains: xe
      }, Qe = {
        variants: [
          // class Car extends vehicle
          {
            match: [
              /class/,
              /\s+/,
              oe,
              /\s+/,
              /extends/,
              /\s+/,
              ee.concat(oe, "(", ee.concat(/\./, oe), ")*")
            ],
            scope: {
              1: "keyword",
              3: "title.class",
              5: "keyword",
              7: "title.class.inherited"
            }
          },
          // class Car
          {
            match: [
              /class/,
              /\s+/,
              oe
            ],
            scope: {
              1: "keyword",
              3: "title.class"
            }
          }
        ]
      }, ke = {
        relevance: 0,
        match: ee.either(
          // Hard coded exceptions
          /\bJSON/,
          // Float32Array, OutT
          /\b[A-Z][a-z]+([A-Z][a-z]*|\d)*/,
          // CSSFactory, CSSFactoryT
          /\b[A-Z]{2,}([A-Z][a-z]+|\d)+([A-Z][a-z]*)*/,
          // FPs, FPsT
          /\b[A-Z]{2,}[a-z]+([A-Z][a-z]+|\d)*([A-Z][a-z]*)*/
          // P
          // single letters are not highlighted
          // BLAH
          // this will be flagged as a UPPER_CASE_CONSTANT instead
        ),
        className: "title.class",
        keywords: {
          _: [
            // se we still get relevance credit for JS library classes
            ...V,
            ...q
          ]
        }
      }, Me = {
        label: "use_strict",
        className: "meta",
        relevance: 10,
        begin: /^\s*['"]use (strict|asm)['"]/
      }, Re = {
        variants: [
          {
            match: [
              /function/,
              /\s+/,
              oe,
              /(?=\s*\()/
            ]
          },
          // anonymous function
          {
            match: [
              /function/,
              /\s*(?=\()/
            ]
          }
        ],
        className: {
          1: "keyword",
          3: "title.function"
        },
        label: "func.def",
        contains: [Le],
        illegal: /%/
      }, De = {
        relevance: 0,
        match: /\b[A-Z][A-Z_0-9]+\b/,
        className: "variable.constant"
      };
      function Pe(Ge) {
        return ee.concat("(?!", Ge.join("|"), ")");
      }
      const $e = {
        match: ee.concat(
          /\b/,
          Pe([
            ...G,
            "super",
            "import"
          ].map((Ge) => `${Ge}\\s*\\(`)),
          oe,
          ee.lookahead(/\s*\(/)
        ),
        className: "title.function",
        relevance: 0
      }, qe = {
        begin: ee.concat(/\./, ee.lookahead(
          ee.concat(oe, /(?![0-9A-Za-z$_(])/)
        )),
        end: oe,
        excludeBegin: !0,
        keywords: "prototype",
        className: "property",
        relevance: 0
      }, Xe = {
        match: [
          /get|set/,
          /\s+/,
          oe,
          /(?=\()/
        ],
        className: {
          1: "keyword",
          3: "title.function"
        },
        contains: [
          {
            // eat to avoid empty params
            begin: /\(\)/
          },
          Le
        ]
      }, Ve = "(\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)|" + ne.UNDERSCORE_IDENT_RE + ")\\s*=>", Ze = {
        match: [
          /const|var|let/,
          /\s+/,
          oe,
          /\s*/,
          /=\s*/,
          /(async\s*)?/,
          // async is optional
          ee.lookahead(Ve)
        ],
        keywords: "async",
        className: {
          1: "keyword",
          3: "title.function"
        },
        contains: [
          Le
        ]
      };
      return {
        name: "JavaScript",
        aliases: ["js", "jsx", "mjs", "cjs"],
        keywords: ue,
        // this will be extended by TypeScript
        exports: { PARAMS_CONTAINS: xe, CLASS_REFERENCE: ke },
        illegal: /#(?![$_A-z])/,
        contains: [
          ne.SHEBANG({
            label: "shebang",
            binary: "node",
            relevance: 5
          }),
          Me,
          ne.APOS_STRING_MODE,
          ne.QUOTE_STRING_MODE,
          ye,
          ge,
          ve,
          be,
          Se,
          // Skip numbers when they are part of a variable name
          { match: /\$\d+/ },
          me,
          ke,
          {
            scope: "attr",
            match: oe + ee.lookahead(":"),
            relevance: 0
          },
          Ze,
          {
            // "value" container
            begin: "(" + ne.RE_STARTERS_RE + "|\\b(case|return|throw)\\b)\\s*",
            keywords: "return throw case",
            relevance: 0,
            contains: [
              Se,
              ne.REGEXP_MODE,
              {
                className: "function",
                // we have to count the parens to make sure we actually have the
                // correct bounding ( ) before the =>.  There could be any number of
                // sub-expressions inside also surrounded by parens.
                begin: Ve,
                returnBegin: !0,
                end: "\\s*=>",
                contains: [
                  {
                    className: "params",
                    variants: [
                      {
                        begin: ne.UNDERSCORE_IDENT_RE,
                        relevance: 0
                      },
                      {
                        className: null,
                        begin: /\(\s*\)/,
                        skip: !0
                      },
                      {
                        begin: /(\s*)\(/,
                        end: /\)/,
                        excludeBegin: !0,
                        excludeEnd: !0,
                        keywords: ue,
                        contains: xe
                      }
                    ]
                  }
                ]
              },
              {
                // could be a comma delimited list of params to a function call
                begin: /,/,
                relevance: 0
              },
              {
                match: /\s+/,
                relevance: 0
              },
              {
                // JSX
                variants: [
                  { begin: se.begin, end: se.end },
                  { match: ce },
                  {
                    begin: le.begin,
                    // we carefully check the opening tag to see if it truly
                    // is a tag and not a false positive
                    "on:begin": le.isTrulyOpeningTag,
                    end: le.end
                  }
                ],
                subLanguage: "xml",
                contains: [
                  {
                    begin: le.begin,
                    end: le.end,
                    skip: !0,
                    contains: ["self"]
                  }
                ]
              }
            ]
          },
          Re,
          {
            // prevent this from getting swallowed up by function
            // since they appear "function like"
            beginKeywords: "while if switch catch for"
          },
          {
            // we have to count the parens to make sure we actually have the correct
            // bounding ( ).  There could be any number of sub-expressions inside
            // also surrounded by parens.
            begin: "\\b(?!function)" + ne.UNDERSCORE_IDENT_RE + "\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)\\s*\\{",
            // end parens
            returnBegin: !0,
            label: "func.def",
            contains: [
              Le,
              ne.inherit(ne.TITLE_MODE, { begin: oe, className: "title.function" })
            ]
          },
          // catch ... so it won't trigger the property rule below
          {
            match: /\.\.\./,
            relevance: 0
          },
          qe,
          // hack: prevents detection of keywords in some circumstances
          // .keyword()
          // $keyword = x
          {
            match: "\\$" + oe,
            relevance: 0
          },
          {
            match: [/\bconstructor(?=\s*\()/],
            className: { 1: "title.function" },
            contains: [Le]
          },
          $e,
          De,
          Qe,
          Xe,
          {
            match: /\$[(.]/
            // relevance booster for a pattern common to JS libs: `$(something)` and `$.something`
          }
        ]
      };
    }
    return javascript_1 = te, javascript_1;
  }
  var json_1, hasRequiredJson;
  function requireJson() {
    if (hasRequiredJson) return json_1;
    hasRequiredJson = 1;
    function O(_) {
      const H = {
        className: "attr",
        begin: /"(\\.|[^\\"\r\n])*"(?=\s*:)/,
        relevance: 1.01
      }, V = {
        match: /[{}[\],:]/,
        className: "punctuation",
        relevance: 0
      }, q = [
        "true",
        "false",
        "null"
      ], G = {
        scope: "literal",
        beginKeywords: q.join(" ")
      };
      return {
        name: "JSON",
        aliases: ["jsonc"],
        keywords: {
          literal: q
        },
        contains: [
          H,
          V,
          _.QUOTE_STRING_MODE,
          G,
          _.C_NUMBER_MODE,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE
        ],
        illegal: "\\S"
      };
    }
    return json_1 = O, json_1;
  }
  var kotlin_1, hasRequiredKotlin;
  function requireKotlin() {
    if (hasRequiredKotlin) return kotlin_1;
    hasRequiredKotlin = 1;
    var O = "[0-9](_*[0-9])*", _ = `\\.(${O})`, H = "[0-9a-fA-F](_*[0-9a-fA-F])*", V = {
      className: "number",
      variants: [
        // DecimalFloatingPointLiteral
        // including ExponentPart
        { begin: `(\\b(${O})((${_})|\\.)?|(${_}))[eE][+-]?(${O})[fFdD]?\\b` },
        // excluding ExponentPart
        { begin: `\\b(${O})((${_})[fFdD]?\\b|\\.([fFdD]\\b)?)` },
        { begin: `(${_})[fFdD]?\\b` },
        { begin: `\\b(${O})[fFdD]\\b` },
        // HexadecimalFloatingPointLiteral
        { begin: `\\b0[xX]((${H})\\.?|(${H})?\\.(${H}))[pP][+-]?(${O})[fFdD]?\\b` },
        // DecimalIntegerLiteral
        { begin: "\\b(0|[1-9](_*[0-9])*)[lL]?\\b" },
        // HexIntegerLiteral
        { begin: `\\b0[xX](${H})[lL]?\\b` },
        // OctalIntegerLiteral
        { begin: "\\b0(_*[0-7])*[lL]?\\b" },
        // BinaryIntegerLiteral
        { begin: "\\b0[bB][01](_*[01])*[lL]?\\b" }
      ],
      relevance: 0
    };
    function q(G) {
      const J = {
        keyword: "abstract as val var vararg get set class object open private protected public noinline crossinline dynamic final enum if else do while for when throw try catch finally import package is in fun override companion reified inline lateinit init interface annotation data sealed internal infix operator out by constructor super tailrec where const inner suspend typealias external expect actual",
        built_in: "Byte Short Char Int Long Boolean Float Double Void Unit Nothing",
        literal: "true false null"
      }, Z = {
        className: "keyword",
        begin: /\b(break|continue|return|this)\b/,
        starts: { contains: [
          {
            className: "symbol",
            begin: /@\w+/
          }
        ] }
      }, te = {
        className: "symbol",
        begin: G.UNDERSCORE_IDENT_RE + "@"
      }, ne = {
        className: "subst",
        begin: /\$\{/,
        end: /\}/,
        contains: [G.C_NUMBER_MODE]
      }, ee = {
        className: "variable",
        begin: "\\$" + G.UNDERSCORE_IDENT_RE
      }, ae = {
        className: "string",
        variants: [
          {
            begin: '"""',
            end: '"""(?=[^"])',
            contains: [
              ee,
              ne
            ]
          },
          // Can't use built-in modes easily, as we want to use STRING in the meta
          // context as 'meta-string' and there's no syntax to remove explicitly set
          // classNames in built-in modes.
          {
            begin: "'",
            end: "'",
            illegal: /\n/,
            contains: [G.BACKSLASH_ESCAPE]
          },
          {
            begin: '"',
            end: '"',
            illegal: /\n/,
            contains: [
              G.BACKSLASH_ESCAPE,
              ee,
              ne
            ]
          }
        ]
      };
      ne.contains.push(ae);
      const oe = {
        className: "meta",
        begin: "@(?:file|property|field|get|set|receiver|param|setparam|delegate)\\s*:(?:\\s*" + G.UNDERSCORE_IDENT_RE + ")?"
      }, se = {
        className: "meta",
        begin: "@" + G.UNDERSCORE_IDENT_RE,
        contains: [
          {
            begin: /\(/,
            end: /\)/,
            contains: [
              G.inherit(ae, { className: "string" }),
              "self"
            ]
          }
        ]
      }, ce = V, le = G.COMMENT(
        "/\\*",
        "\\*/",
        { contains: [G.C_BLOCK_COMMENT_MODE] }
      ), ue = { variants: [
        {
          className: "type",
          begin: G.UNDERSCORE_IDENT_RE
        },
        {
          begin: /\(/,
          end: /\)/,
          contains: []
          // defined later
        }
      ] }, de = ue;
      return de.variants[1].contains = [ue], ue.variants[1].contains = [de], {
        name: "Kotlin",
        aliases: [
          "kt",
          "kts"
        ],
        keywords: J,
        contains: [
          G.COMMENT(
            "/\\*\\*",
            "\\*/",
            {
              relevance: 0,
              contains: [
                {
                  className: "doctag",
                  begin: "@[A-Za-z]+"
                }
              ]
            }
          ),
          G.C_LINE_COMMENT_MODE,
          le,
          Z,
          te,
          oe,
          se,
          {
            className: "function",
            beginKeywords: "fun",
            end: "[(]|$",
            returnBegin: !0,
            excludeEnd: !0,
            keywords: J,
            relevance: 5,
            contains: [
              {
                begin: G.UNDERSCORE_IDENT_RE + "\\s*\\(",
                returnBegin: !0,
                relevance: 0,
                contains: [G.UNDERSCORE_TITLE_MODE]
              },
              {
                className: "type",
                begin: /</,
                end: />/,
                keywords: "reified",
                relevance: 0
              },
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                endsParent: !0,
                keywords: J,
                relevance: 0,
                contains: [
                  {
                    begin: /:/,
                    end: /[=,\/]/,
                    endsWithParent: !0,
                    contains: [
                      ue,
                      G.C_LINE_COMMENT_MODE,
                      le
                    ],
                    relevance: 0
                  },
                  G.C_LINE_COMMENT_MODE,
                  le,
                  oe,
                  se,
                  ae,
                  G.C_NUMBER_MODE
                ]
              },
              le
            ]
          },
          {
            begin: [
              /class|interface|trait/,
              /\s+/,
              G.UNDERSCORE_IDENT_RE
            ],
            beginScope: {
              3: "title.class"
            },
            keywords: "class interface trait",
            end: /[:\{(]|$/,
            excludeEnd: !0,
            illegal: "extends implements",
            contains: [
              { beginKeywords: "public protected internal private constructor" },
              G.UNDERSCORE_TITLE_MODE,
              {
                className: "type",
                begin: /</,
                end: />/,
                excludeBegin: !0,
                excludeEnd: !0,
                relevance: 0
              },
              {
                className: "type",
                begin: /[,:]\s*/,
                end: /[<\(,){\s]|$/,
                excludeBegin: !0,
                returnEnd: !0
              },
              oe,
              se
            ]
          },
          ae,
          {
            className: "meta",
            begin: "^#!/usr/bin/env",
            end: "$",
            illegal: `
`
          },
          ce
        ]
      };
    }
    return kotlin_1 = q, kotlin_1;
  }
  var less_1, hasRequiredLess;
  function requireLess() {
    if (hasRequiredLess) return less_1;
    hasRequiredLess = 1;
    const O = (ee) => ({
      IMPORTANT: {
        scope: "meta",
        begin: "!important"
      },
      BLOCK_COMMENT: ee.C_BLOCK_COMMENT_MODE,
      HEXCOLOR: {
        scope: "number",
        begin: /#(([0-9a-fA-F]{3,4})|(([0-9a-fA-F]{2}){3,4}))\b/
      },
      FUNCTION_DISPATCH: {
        className: "built_in",
        begin: /[\w-]+(?=\()/
      },
      ATTRIBUTE_SELECTOR_MODE: {
        scope: "selector-attr",
        begin: /\[/,
        end: /\]/,
        illegal: "$",
        contains: [
          ee.APOS_STRING_MODE,
          ee.QUOTE_STRING_MODE
        ]
      },
      CSS_NUMBER_MODE: {
        scope: "number",
        begin: ee.NUMBER_RE + "(%|em|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc|px|deg|grad|rad|turn|s|ms|Hz|kHz|dpi|dpcm|dppx)?",
        relevance: 0
      },
      CSS_VARIABLE: {
        className: "attr",
        begin: /--[A-Za-z_][A-Za-z0-9_-]*/
      }
    }), _ = [
      "a",
      "abbr",
      "address",
      "article",
      "aside",
      "audio",
      "b",
      "blockquote",
      "body",
      "button",
      "canvas",
      "caption",
      "cite",
      "code",
      "dd",
      "del",
      "details",
      "dfn",
      "div",
      "dl",
      "dt",
      "em",
      "fieldset",
      "figcaption",
      "figure",
      "footer",
      "form",
      "h1",
      "h2",
      "h3",
      "h4",
      "h5",
      "h6",
      "header",
      "hgroup",
      "html",
      "i",
      "iframe",
      "img",
      "input",
      "ins",
      "kbd",
      "label",
      "legend",
      "li",
      "main",
      "mark",
      "menu",
      "nav",
      "object",
      "ol",
      "optgroup",
      "option",
      "p",
      "picture",
      "q",
      "quote",
      "samp",
      "section",
      "select",
      "source",
      "span",
      "strong",
      "summary",
      "sup",
      "table",
      "tbody",
      "td",
      "textarea",
      "tfoot",
      "th",
      "thead",
      "time",
      "tr",
      "ul",
      "var",
      "video"
    ], H = [
      "defs",
      "g",
      "marker",
      "mask",
      "pattern",
      "svg",
      "switch",
      "symbol",
      "feBlend",
      "feColorMatrix",
      "feComponentTransfer",
      "feComposite",
      "feConvolveMatrix",
      "feDiffuseLighting",
      "feDisplacementMap",
      "feFlood",
      "feGaussianBlur",
      "feImage",
      "feMerge",
      "feMorphology",
      "feOffset",
      "feSpecularLighting",
      "feTile",
      "feTurbulence",
      "linearGradient",
      "radialGradient",
      "stop",
      "circle",
      "ellipse",
      "image",
      "line",
      "path",
      "polygon",
      "polyline",
      "rect",
      "text",
      "use",
      "textPath",
      "tspan",
      "foreignObject",
      "clipPath"
    ], V = [
      ..._,
      ...H
    ], q = [
      "any-hover",
      "any-pointer",
      "aspect-ratio",
      "color",
      "color-gamut",
      "color-index",
      "device-aspect-ratio",
      "device-height",
      "device-width",
      "display-mode",
      "forced-colors",
      "grid",
      "height",
      "hover",
      "inverted-colors",
      "monochrome",
      "orientation",
      "overflow-block",
      "overflow-inline",
      "pointer",
      "prefers-color-scheme",
      "prefers-contrast",
      "prefers-reduced-motion",
      "prefers-reduced-transparency",
      "resolution",
      "scan",
      "scripting",
      "update",
      "width",
      // TODO: find a better solution?
      "min-width",
      "max-width",
      "min-height",
      "max-height"
    ].sort().reverse(), G = [
      "active",
      "any-link",
      "blank",
      "checked",
      "current",
      "default",
      "defined",
      "dir",
      // dir()
      "disabled",
      "drop",
      "empty",
      "enabled",
      "first",
      "first-child",
      "first-of-type",
      "fullscreen",
      "future",
      "focus",
      "focus-visible",
      "focus-within",
      "has",
      // has()
      "host",
      // host or host()
      "host-context",
      // host-context()
      "hover",
      "indeterminate",
      "in-range",
      "invalid",
      "is",
      // is()
      "lang",
      // lang()
      "last-child",
      "last-of-type",
      "left",
      "link",
      "local-link",
      "not",
      // not()
      "nth-child",
      // nth-child()
      "nth-col",
      // nth-col()
      "nth-last-child",
      // nth-last-child()
      "nth-last-col",
      // nth-last-col()
      "nth-last-of-type",
      //nth-last-of-type()
      "nth-of-type",
      //nth-of-type()
      "only-child",
      "only-of-type",
      "optional",
      "out-of-range",
      "past",
      "placeholder-shown",
      "read-only",
      "read-write",
      "required",
      "right",
      "root",
      "scope",
      "target",
      "target-within",
      "user-invalid",
      "valid",
      "visited",
      "where"
      // where()
    ].sort().reverse(), J = [
      "after",
      "backdrop",
      "before",
      "cue",
      "cue-region",
      "first-letter",
      "first-line",
      "grammar-error",
      "marker",
      "part",
      "placeholder",
      "selection",
      "slotted",
      "spelling-error"
    ].sort().reverse(), Z = [
      "accent-color",
      "align-content",
      "align-items",
      "align-self",
      "alignment-baseline",
      "all",
      "anchor-name",
      "animation",
      "animation-composition",
      "animation-delay",
      "animation-direction",
      "animation-duration",
      "animation-fill-mode",
      "animation-iteration-count",
      "animation-name",
      "animation-play-state",
      "animation-range",
      "animation-range-end",
      "animation-range-start",
      "animation-timeline",
      "animation-timing-function",
      "appearance",
      "aspect-ratio",
      "backdrop-filter",
      "backface-visibility",
      "background",
      "background-attachment",
      "background-blend-mode",
      "background-clip",
      "background-color",
      "background-image",
      "background-origin",
      "background-position",
      "background-position-x",
      "background-position-y",
      "background-repeat",
      "background-size",
      "baseline-shift",
      "block-size",
      "border",
      "border-block",
      "border-block-color",
      "border-block-end",
      "border-block-end-color",
      "border-block-end-style",
      "border-block-end-width",
      "border-block-start",
      "border-block-start-color",
      "border-block-start-style",
      "border-block-start-width",
      "border-block-style",
      "border-block-width",
      "border-bottom",
      "border-bottom-color",
      "border-bottom-left-radius",
      "border-bottom-right-radius",
      "border-bottom-style",
      "border-bottom-width",
      "border-collapse",
      "border-color",
      "border-end-end-radius",
      "border-end-start-radius",
      "border-image",
      "border-image-outset",
      "border-image-repeat",
      "border-image-slice",
      "border-image-source",
      "border-image-width",
      "border-inline",
      "border-inline-color",
      "border-inline-end",
      "border-inline-end-color",
      "border-inline-end-style",
      "border-inline-end-width",
      "border-inline-start",
      "border-inline-start-color",
      "border-inline-start-style",
      "border-inline-start-width",
      "border-inline-style",
      "border-inline-width",
      "border-left",
      "border-left-color",
      "border-left-style",
      "border-left-width",
      "border-radius",
      "border-right",
      "border-right-color",
      "border-right-style",
      "border-right-width",
      "border-spacing",
      "border-start-end-radius",
      "border-start-start-radius",
      "border-style",
      "border-top",
      "border-top-color",
      "border-top-left-radius",
      "border-top-right-radius",
      "border-top-style",
      "border-top-width",
      "border-width",
      "bottom",
      "box-align",
      "box-decoration-break",
      "box-direction",
      "box-flex",
      "box-flex-group",
      "box-lines",
      "box-ordinal-group",
      "box-orient",
      "box-pack",
      "box-shadow",
      "box-sizing",
      "break-after",
      "break-before",
      "break-inside",
      "caption-side",
      "caret-color",
      "clear",
      "clip",
      "clip-path",
      "clip-rule",
      "color",
      "color-interpolation",
      "color-interpolation-filters",
      "color-profile",
      "color-rendering",
      "color-scheme",
      "column-count",
      "column-fill",
      "column-gap",
      "column-rule",
      "column-rule-color",
      "column-rule-style",
      "column-rule-width",
      "column-span",
      "column-width",
      "columns",
      "contain",
      "contain-intrinsic-block-size",
      "contain-intrinsic-height",
      "contain-intrinsic-inline-size",
      "contain-intrinsic-size",
      "contain-intrinsic-width",
      "container",
      "container-name",
      "container-type",
      "content",
      "content-visibility",
      "counter-increment",
      "counter-reset",
      "counter-set",
      "cue",
      "cue-after",
      "cue-before",
      "cursor",
      "cx",
      "cy",
      "direction",
      "display",
      "dominant-baseline",
      "empty-cells",
      "enable-background",
      "field-sizing",
      "fill",
      "fill-opacity",
      "fill-rule",
      "filter",
      "flex",
      "flex-basis",
      "flex-direction",
      "flex-flow",
      "flex-grow",
      "flex-shrink",
      "flex-wrap",
      "float",
      "flood-color",
      "flood-opacity",
      "flow",
      "font",
      "font-display",
      "font-family",
      "font-feature-settings",
      "font-kerning",
      "font-language-override",
      "font-optical-sizing",
      "font-palette",
      "font-size",
      "font-size-adjust",
      "font-smooth",
      "font-smoothing",
      "font-stretch",
      "font-style",
      "font-synthesis",
      "font-synthesis-position",
      "font-synthesis-small-caps",
      "font-synthesis-style",
      "font-synthesis-weight",
      "font-variant",
      "font-variant-alternates",
      "font-variant-caps",
      "font-variant-east-asian",
      "font-variant-emoji",
      "font-variant-ligatures",
      "font-variant-numeric",
      "font-variant-position",
      "font-variation-settings",
      "font-weight",
      "forced-color-adjust",
      "gap",
      "glyph-orientation-horizontal",
      "glyph-orientation-vertical",
      "grid",
      "grid-area",
      "grid-auto-columns",
      "grid-auto-flow",
      "grid-auto-rows",
      "grid-column",
      "grid-column-end",
      "grid-column-start",
      "grid-gap",
      "grid-row",
      "grid-row-end",
      "grid-row-start",
      "grid-template",
      "grid-template-areas",
      "grid-template-columns",
      "grid-template-rows",
      "hanging-punctuation",
      "height",
      "hyphenate-character",
      "hyphenate-limit-chars",
      "hyphens",
      "icon",
      "image-orientation",
      "image-rendering",
      "image-resolution",
      "ime-mode",
      "initial-letter",
      "initial-letter-align",
      "inline-size",
      "inset",
      "inset-area",
      "inset-block",
      "inset-block-end",
      "inset-block-start",
      "inset-inline",
      "inset-inline-end",
      "inset-inline-start",
      "isolation",
      "justify-content",
      "justify-items",
      "justify-self",
      "kerning",
      "left",
      "letter-spacing",
      "lighting-color",
      "line-break",
      "line-height",
      "line-height-step",
      "list-style",
      "list-style-image",
      "list-style-position",
      "list-style-type",
      "margin",
      "margin-block",
      "margin-block-end",
      "margin-block-start",
      "margin-bottom",
      "margin-inline",
      "margin-inline-end",
      "margin-inline-start",
      "margin-left",
      "margin-right",
      "margin-top",
      "margin-trim",
      "marker",
      "marker-end",
      "marker-mid",
      "marker-start",
      "marks",
      "mask",
      "mask-border",
      "mask-border-mode",
      "mask-border-outset",
      "mask-border-repeat",
      "mask-border-slice",
      "mask-border-source",
      "mask-border-width",
      "mask-clip",
      "mask-composite",
      "mask-image",
      "mask-mode",
      "mask-origin",
      "mask-position",
      "mask-repeat",
      "mask-size",
      "mask-type",
      "masonry-auto-flow",
      "math-depth",
      "math-shift",
      "math-style",
      "max-block-size",
      "max-height",
      "max-inline-size",
      "max-width",
      "min-block-size",
      "min-height",
      "min-inline-size",
      "min-width",
      "mix-blend-mode",
      "nav-down",
      "nav-index",
      "nav-left",
      "nav-right",
      "nav-up",
      "none",
      "normal",
      "object-fit",
      "object-position",
      "offset",
      "offset-anchor",
      "offset-distance",
      "offset-path",
      "offset-position",
      "offset-rotate",
      "opacity",
      "order",
      "orphans",
      "outline",
      "outline-color",
      "outline-offset",
      "outline-style",
      "outline-width",
      "overflow",
      "overflow-anchor",
      "overflow-block",
      "overflow-clip-margin",
      "overflow-inline",
      "overflow-wrap",
      "overflow-x",
      "overflow-y",
      "overlay",
      "overscroll-behavior",
      "overscroll-behavior-block",
      "overscroll-behavior-inline",
      "overscroll-behavior-x",
      "overscroll-behavior-y",
      "padding",
      "padding-block",
      "padding-block-end",
      "padding-block-start",
      "padding-bottom",
      "padding-inline",
      "padding-inline-end",
      "padding-inline-start",
      "padding-left",
      "padding-right",
      "padding-top",
      "page",
      "page-break-after",
      "page-break-before",
      "page-break-inside",
      "paint-order",
      "pause",
      "pause-after",
      "pause-before",
      "perspective",
      "perspective-origin",
      "place-content",
      "place-items",
      "place-self",
      "pointer-events",
      "position",
      "position-anchor",
      "position-visibility",
      "print-color-adjust",
      "quotes",
      "r",
      "resize",
      "rest",
      "rest-after",
      "rest-before",
      "right",
      "rotate",
      "row-gap",
      "ruby-align",
      "ruby-position",
      "scale",
      "scroll-behavior",
      "scroll-margin",
      "scroll-margin-block",
      "scroll-margin-block-end",
      "scroll-margin-block-start",
      "scroll-margin-bottom",
      "scroll-margin-inline",
      "scroll-margin-inline-end",
      "scroll-margin-inline-start",
      "scroll-margin-left",
      "scroll-margin-right",
      "scroll-margin-top",
      "scroll-padding",
      "scroll-padding-block",
      "scroll-padding-block-end",
      "scroll-padding-block-start",
      "scroll-padding-bottom",
      "scroll-padding-inline",
      "scroll-padding-inline-end",
      "scroll-padding-inline-start",
      "scroll-padding-left",
      "scroll-padding-right",
      "scroll-padding-top",
      "scroll-snap-align",
      "scroll-snap-stop",
      "scroll-snap-type",
      "scroll-timeline",
      "scroll-timeline-axis",
      "scroll-timeline-name",
      "scrollbar-color",
      "scrollbar-gutter",
      "scrollbar-width",
      "shape-image-threshold",
      "shape-margin",
      "shape-outside",
      "shape-rendering",
      "speak",
      "speak-as",
      "src",
      // @font-face
      "stop-color",
      "stop-opacity",
      "stroke",
      "stroke-dasharray",
      "stroke-dashoffset",
      "stroke-linecap",
      "stroke-linejoin",
      "stroke-miterlimit",
      "stroke-opacity",
      "stroke-width",
      "tab-size",
      "table-layout",
      "text-align",
      "text-align-all",
      "text-align-last",
      "text-anchor",
      "text-combine-upright",
      "text-decoration",
      "text-decoration-color",
      "text-decoration-line",
      "text-decoration-skip",
      "text-decoration-skip-ink",
      "text-decoration-style",
      "text-decoration-thickness",
      "text-emphasis",
      "text-emphasis-color",
      "text-emphasis-position",
      "text-emphasis-style",
      "text-indent",
      "text-justify",
      "text-orientation",
      "text-overflow",
      "text-rendering",
      "text-shadow",
      "text-size-adjust",
      "text-transform",
      "text-underline-offset",
      "text-underline-position",
      "text-wrap",
      "text-wrap-mode",
      "text-wrap-style",
      "timeline-scope",
      "top",
      "touch-action",
      "transform",
      "transform-box",
      "transform-origin",
      "transform-style",
      "transition",
      "transition-behavior",
      "transition-delay",
      "transition-duration",
      "transition-property",
      "transition-timing-function",
      "translate",
      "unicode-bidi",
      "user-modify",
      "user-select",
      "vector-effect",
      "vertical-align",
      "view-timeline",
      "view-timeline-axis",
      "view-timeline-inset",
      "view-timeline-name",
      "view-transition-name",
      "visibility",
      "voice-balance",
      "voice-duration",
      "voice-family",
      "voice-pitch",
      "voice-range",
      "voice-rate",
      "voice-stress",
      "voice-volume",
      "white-space",
      "white-space-collapse",
      "widows",
      "width",
      "will-change",
      "word-break",
      "word-spacing",
      "word-wrap",
      "writing-mode",
      "x",
      "y",
      "z-index",
      "zoom"
    ].sort().reverse(), te = G.concat(J).sort().reverse();
    function ne(ee) {
      const ae = O(ee), oe = te, se = "and or not only", ce = "[\\w-]+", le = "(" + ce + "|@\\{" + ce + "\\})", ue = [], de = [], fe = function(Ee) {
        return {
          // Less strings are not multiline (also include '~' for more consistent coloring of "escaped" strings)
          className: "string",
          begin: "~?" + Ee + ".*?" + Ee
        };
      }, he = function(Ee, xe, Le) {
        return {
          className: Ee,
          begin: xe,
          relevance: Le
        };
      }, me = {
        $pattern: /[a-z-]+/,
        keyword: se,
        attribute: q.join(" ")
      }, pe = {
        // used only to properly balance nested parens inside mixin call, def. arg list
        begin: "\\(",
        end: "\\)",
        contains: de,
        keywords: me,
        relevance: 0
      };
      de.push(
        ee.C_LINE_COMMENT_MODE,
        ee.C_BLOCK_COMMENT_MODE,
        fe("'"),
        fe('"'),
        ae.CSS_NUMBER_MODE,
        // fixme: it does not include dot for numbers like .5em :(
        {
          begin: "(url|data-uri)\\(",
          starts: {
            className: "string",
            end: "[\\)\\n]",
            excludeEnd: !0
          }
        },
        ae.HEXCOLOR,
        pe,
        he("variable", "@@?" + ce, 10),
        he("variable", "@\\{" + ce + "\\}"),
        he("built_in", "~?`[^`]*?`"),
        // inline javascript (or whatever host language) *multiline* string
        {
          // @media features (it’s here to not duplicate things in AT_RULE_MODE with extra PARENS_MODE overriding):
          className: "attribute",
          begin: ce + "\\s*:",
          end: ":",
          returnBegin: !0,
          excludeEnd: !0
        },
        ae.IMPORTANT,
        { beginKeywords: "and not" },
        ae.FUNCTION_DISPATCH
      );
      const ye = de.concat({
        begin: /\{/,
        end: /\}/,
        contains: ue
      }), ge = {
        beginKeywords: "when",
        endsWithParent: !0,
        contains: [{ beginKeywords: "and not" }].concat(de)
        // using this form to override VALUE’s 'function' match
      }, ve = {
        begin: le + "\\s*:",
        returnBegin: !0,
        end: /[;}]/,
        relevance: 0,
        contains: [
          { begin: /-(webkit|moz|ms|o)-/ },
          ae.CSS_VARIABLE,
          {
            className: "attribute",
            begin: "\\b(" + Z.join("|") + ")\\b",
            end: /(?=:)/,
            starts: {
              endsWithParent: !0,
              illegal: "[<=$]",
              relevance: 0,
              contains: de
            }
          }
        ]
      }, be = {
        className: "keyword",
        begin: "@(import|media|charset|font-face|(-[a-z]+-)?keyframes|supports|document|namespace|page|viewport|host)\\b",
        starts: {
          end: "[;{}]",
          keywords: me,
          returnEnd: !0,
          contains: de,
          relevance: 0
        }
      }, Ce = {
        className: "variable",
        variants: [
          // using more strict pattern for higher relevance to increase chances of Less detection.
          // this is *the only* Less specific statement used in most of the sources, so...
          // (we’ll still often loose to the css-parser unless there's '//' comment,
          // simply because 1 variable just can't beat 99 properties :)
          {
            begin: "@" + ce + "\\s*:",
            relevance: 15
          },
          { begin: "@" + ce }
        ],
        starts: {
          end: "[;}]",
          returnEnd: !0,
          contains: ye
        }
      }, Se = {
        // first parse unambiguous selectors (i.e. those not starting with tag)
        // then fall into the scary lookahead-discriminator variant.
        // this mode also handles mixin definitions and calls
        variants: [
          {
            begin: "[\\.#:&\\[>]",
            end: "[;{}]"
            // mixin calls end with ';'
          },
          {
            begin: le,
            end: /\{/
          }
        ],
        returnBegin: !0,
        returnEnd: !0,
        illegal: `[<='$"]`,
        relevance: 0,
        contains: [
          ee.C_LINE_COMMENT_MODE,
          ee.C_BLOCK_COMMENT_MODE,
          ge,
          he("keyword", "all\\b"),
          he("variable", "@\\{" + ce + "\\}"),
          // otherwise it’s identified as tag
          {
            begin: "\\b(" + V.join("|") + ")\\b",
            className: "selector-tag"
          },
          ae.CSS_NUMBER_MODE,
          he("selector-tag", le, 0),
          he("selector-id", "#" + le),
          he("selector-class", "\\." + le, 0),
          he("selector-tag", "&", 0),
          ae.ATTRIBUTE_SELECTOR_MODE,
          {
            className: "selector-pseudo",
            begin: ":(" + G.join("|") + ")"
          },
          {
            className: "selector-pseudo",
            begin: ":(:)?(" + J.join("|") + ")"
          },
          {
            begin: /\(/,
            end: /\)/,
            relevance: 0,
            contains: ye
          },
          // argument list of parametric mixins
          { begin: "!important" },
          // eat !important after mixin call or it will be colored as tag
          ae.FUNCTION_DISPATCH
        ]
      }, we = {
        begin: ce + `:(:)?(${oe.join("|")})`,
        returnBegin: !0,
        contains: [Se]
      };
      return ue.push(
        ee.C_LINE_COMMENT_MODE,
        ee.C_BLOCK_COMMENT_MODE,
        be,
        Ce,
        we,
        ve,
        Se,
        ge,
        ae.FUNCTION_DISPATCH
      ), {
        name: "Less",
        case_insensitive: !0,
        illegal: `[=>'/<($"]`,
        contains: ue
      };
    }
    return less_1 = ne, less_1;
  }
  var lua_1, hasRequiredLua;
  function requireLua() {
    if (hasRequiredLua) return lua_1;
    hasRequiredLua = 1;
    function O(_) {
      const H = "\\[=*\\[", V = "\\]=*\\]", q = {
        begin: H,
        end: V,
        contains: ["self"]
      }, G = [
        _.COMMENT("--(?!" + H + ")", "$"),
        _.COMMENT(
          "--" + H,
          V,
          {
            contains: [q],
            relevance: 10
          }
        )
      ];
      return {
        name: "Lua",
        aliases: ["pluto"],
        keywords: {
          $pattern: _.UNDERSCORE_IDENT_RE,
          literal: "true false nil",
          keyword: "and break do else elseif end for goto if in local not or repeat return then until while",
          built_in: (
            // Metatags and globals:
            "_G _ENV _VERSION __index __newindex __mode __call __metatable __tostring __len __gc __add __sub __mul __div __mod __pow __concat __unm __eq __lt __le assert collectgarbage dofile error getfenv getmetatable ipairs load loadfile loadstring module next pairs pcall print rawequal rawget rawset require select setfenv setmetatable tonumber tostring type unpack xpcall arg self coroutine resume yield status wrap create running debug getupvalue debug sethook getmetatable gethook setmetatable setlocal traceback setfenv getinfo setupvalue getlocal getregistry getfenv io lines write close flush open output type read stderr stdin input stdout popen tmpfile math log max acos huge ldexp pi cos tanh pow deg tan cosh sinh random randomseed frexp ceil floor rad abs sqrt modf asin min mod fmod log10 atan2 exp sin atan os exit setlocale date getenv difftime remove time clock tmpname rename execute package preload loadlib loaded loaders cpath config path seeall string sub upper len gfind rep find match char dump gmatch reverse byte format gsub lower table setn insert getn foreachi maxn foreach concat sort remove"
          )
        },
        contains: G.concat([
          {
            className: "function",
            beginKeywords: "function",
            end: "\\)",
            contains: [
              _.inherit(_.TITLE_MODE, { begin: "([_a-zA-Z]\\w*\\.)*([_a-zA-Z]\\w*:)?[_a-zA-Z]\\w*" }),
              {
                className: "params",
                begin: "\\(",
                endsWithParent: !0,
                contains: G
              }
            ].concat(G)
          },
          _.C_NUMBER_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            begin: H,
            end: V,
            contains: [q],
            relevance: 5
          }
        ])
      };
    }
    return lua_1 = O, lua_1;
  }
  var makefile_1, hasRequiredMakefile;
  function requireMakefile() {
    if (hasRequiredMakefile) return makefile_1;
    hasRequiredMakefile = 1;
    function O(_) {
      const H = {
        className: "variable",
        variants: [
          {
            begin: "\\$\\(" + _.UNDERSCORE_IDENT_RE + "\\)",
            contains: [_.BACKSLASH_ESCAPE]
          },
          { begin: /\$[@%<?\^\+\*]/ }
        ]
      }, V = {
        className: "string",
        begin: /"/,
        end: /"/,
        contains: [
          _.BACKSLASH_ESCAPE,
          H
        ]
      }, q = {
        className: "variable",
        begin: /\$\([\w-]+\s/,
        end: /\)/,
        keywords: { built_in: "subst patsubst strip findstring filter filter-out sort word wordlist firstword lastword dir notdir suffix basename addsuffix addprefix join wildcard realpath abspath error warning shell origin flavor foreach if or and call eval file value" },
        contains: [
          H,
          V
          // Added QUOTE_STRING as they can be a part of functions
        ]
      }, G = { begin: "^" + _.UNDERSCORE_IDENT_RE + "\\s*(?=[:+?]?=)" }, J = {
        className: "meta",
        begin: /^\.PHONY:/,
        end: /$/,
        keywords: {
          $pattern: /[\.\w]+/,
          keyword: ".PHONY"
        }
      }, Z = {
        className: "section",
        begin: /^[^\s]+:/,
        end: /$/,
        contains: [H]
      };
      return {
        name: "Makefile",
        aliases: [
          "mk",
          "mak",
          "make"
        ],
        keywords: {
          $pattern: /[\w-]+/,
          keyword: "define endef undefine ifdef ifndef ifeq ifneq else endif include -include sinclude override export unexport private vpath"
        },
        contains: [
          _.HASH_COMMENT_MODE,
          H,
          V,
          q,
          G,
          J,
          Z
        ]
      };
    }
    return makefile_1 = O, makefile_1;
  }
  var perl_1, hasRequiredPerl;
  function requirePerl() {
    if (hasRequiredPerl) return perl_1;
    hasRequiredPerl = 1;
    function O(_) {
      const H = _.regex, V = [
        "abs",
        "accept",
        "alarm",
        "and",
        "atan2",
        "bind",
        "binmode",
        "bless",
        "break",
        "caller",
        "chdir",
        "chmod",
        "chomp",
        "chop",
        "chown",
        "chr",
        "chroot",
        "class",
        "close",
        "closedir",
        "connect",
        "continue",
        "cos",
        "crypt",
        "dbmclose",
        "dbmopen",
        "defined",
        "delete",
        "die",
        "do",
        "dump",
        "each",
        "else",
        "elsif",
        "endgrent",
        "endhostent",
        "endnetent",
        "endprotoent",
        "endpwent",
        "endservent",
        "eof",
        "eval",
        "exec",
        "exists",
        "exit",
        "exp",
        "fcntl",
        "field",
        "fileno",
        "flock",
        "for",
        "foreach",
        "fork",
        "format",
        "formline",
        "getc",
        "getgrent",
        "getgrgid",
        "getgrnam",
        "gethostbyaddr",
        "gethostbyname",
        "gethostent",
        "getlogin",
        "getnetbyaddr",
        "getnetbyname",
        "getnetent",
        "getpeername",
        "getpgrp",
        "getpriority",
        "getprotobyname",
        "getprotobynumber",
        "getprotoent",
        "getpwent",
        "getpwnam",
        "getpwuid",
        "getservbyname",
        "getservbyport",
        "getservent",
        "getsockname",
        "getsockopt",
        "given",
        "glob",
        "gmtime",
        "goto",
        "grep",
        "gt",
        "hex",
        "if",
        "index",
        "int",
        "ioctl",
        "join",
        "keys",
        "kill",
        "last",
        "lc",
        "lcfirst",
        "length",
        "link",
        "listen",
        "local",
        "localtime",
        "log",
        "lstat",
        "lt",
        "ma",
        "map",
        "method",
        "mkdir",
        "msgctl",
        "msgget",
        "msgrcv",
        "msgsnd",
        "my",
        "ne",
        "next",
        "no",
        "not",
        "oct",
        "open",
        "opendir",
        "or",
        "ord",
        "our",
        "pack",
        "package",
        "pipe",
        "pop",
        "pos",
        "print",
        "printf",
        "prototype",
        "push",
        "q|0",
        "qq",
        "quotemeta",
        "qw",
        "qx",
        "rand",
        "read",
        "readdir",
        "readline",
        "readlink",
        "readpipe",
        "recv",
        "redo",
        "ref",
        "rename",
        "require",
        "reset",
        "return",
        "reverse",
        "rewinddir",
        "rindex",
        "rmdir",
        "say",
        "scalar",
        "seek",
        "seekdir",
        "select",
        "semctl",
        "semget",
        "semop",
        "send",
        "setgrent",
        "sethostent",
        "setnetent",
        "setpgrp",
        "setpriority",
        "setprotoent",
        "setpwent",
        "setservent",
        "setsockopt",
        "shift",
        "shmctl",
        "shmget",
        "shmread",
        "shmwrite",
        "shutdown",
        "sin",
        "sleep",
        "socket",
        "socketpair",
        "sort",
        "splice",
        "split",
        "sprintf",
        "sqrt",
        "srand",
        "stat",
        "state",
        "study",
        "sub",
        "substr",
        "symlink",
        "syscall",
        "sysopen",
        "sysread",
        "sysseek",
        "system",
        "syswrite",
        "tell",
        "telldir",
        "tie",
        "tied",
        "time",
        "times",
        "tr",
        "truncate",
        "uc",
        "ucfirst",
        "umask",
        "undef",
        "unless",
        "unlink",
        "unpack",
        "unshift",
        "untie",
        "until",
        "use",
        "utime",
        "values",
        "vec",
        "wait",
        "waitpid",
        "wantarray",
        "warn",
        "when",
        "while",
        "write",
        "x|0",
        "xor",
        "y|0"
      ], q = /[dualxmsipngr]{0,12}/, G = {
        $pattern: /[\w.]+/,
        keyword: V.join(" ")
      }, J = {
        className: "subst",
        begin: "[$@]\\{",
        end: "\\}",
        keywords: G
      }, Z = {
        begin: /->\{/,
        end: /\}/
        // contains defined later
      }, te = {
        scope: "attr",
        match: /\s+:\s*\w+(\s*\(.*?\))?/
      }, ne = {
        scope: "variable",
        variants: [
          { begin: /\$\d/ },
          {
            begin: H.concat(
              /[$%@](?!")(\^\w\b|#\w+(::\w+)*|\{\w+\}|\w+(::\w*)*)/,
              // negative look-ahead tries to avoid matching patterns that are not
              // Perl at all like $ident$, @ident@, etc.
              "(?![A-Za-z])(?![@$%])"
            )
          },
          {
            // Only $= is a special Perl variable and one can't declare @= or %=.
            begin: /[$%@](?!")[^\s\w{=]|\$=/,
            relevance: 0
          }
        ],
        contains: [te]
      }, ee = {
        className: "number",
        variants: [
          // decimal numbers:
          // include the case where a number starts with a dot (eg. .9), and
          // the leading 0? avoids mixing the first and second match on 0.x cases
          { match: /0?\.[0-9][0-9_]+\b/ },
          // include the special versioned number (eg. v5.38)
          { match: /\bv?(0|[1-9][0-9_]*(\.[0-9_]+)?|[1-9][0-9_]*)\b/ },
          // non-decimal numbers:
          { match: /\b0[0-7][0-7_]*\b/ },
          { match: /\b0x[0-9a-fA-F][0-9a-fA-F_]*\b/ },
          { match: /\b0b[0-1][0-1_]*\b/ }
        ],
        relevance: 0
      }, ae = [
        _.BACKSLASH_ESCAPE,
        J,
        ne
      ], oe = [
        /!/,
        /\//,
        /\|/,
        /\?/,
        /'/,
        /"/,
        // valid but infrequent and weird
        /#/
        // valid but infrequent and weird
      ], se = (ue, de, fe = "\\1") => {
        const he = fe === "\\1" ? fe : H.concat(fe, de);
        return H.concat(
          H.concat("(?:", ue, ")"),
          de,
          /(?:\\.|[^\\\/])*?/,
          he,
          /(?:\\.|[^\\\/])*?/,
          fe,
          q
        );
      }, ce = (ue, de, fe) => H.concat(
        H.concat("(?:", ue, ")"),
        de,
        /(?:\\.|[^\\\/])*?/,
        fe,
        q
      ), le = [
        ne,
        _.HASH_COMMENT_MODE,
        _.COMMENT(
          /^=\w/,
          /=cut/,
          { endsWithParent: !0 }
        ),
        Z,
        {
          className: "string",
          contains: ae,
          variants: [
            {
              begin: "q[qwxr]?\\s*\\(",
              end: "\\)",
              relevance: 5
            },
            {
              begin: "q[qwxr]?\\s*\\[",
              end: "\\]",
              relevance: 5
            },
            {
              begin: "q[qwxr]?\\s*\\{",
              end: "\\}",
              relevance: 5
            },
            {
              begin: "q[qwxr]?\\s*\\|",
              end: "\\|",
              relevance: 5
            },
            {
              begin: "q[qwxr]?\\s*<",
              end: ">",
              relevance: 5
            },
            {
              begin: "qw\\s+q",
              end: "q",
              relevance: 5
            },
            {
              begin: "'",
              end: "'",
              contains: [_.BACKSLASH_ESCAPE]
            },
            {
              begin: '"',
              end: '"'
            },
            {
              begin: "`",
              end: "`",
              contains: [_.BACKSLASH_ESCAPE]
            },
            {
              begin: /\{\w+\}/,
              relevance: 0
            },
            {
              begin: "-?\\w+\\s*=>",
              relevance: 0
            }
          ]
        },
        ee,
        {
          // regexp container
          begin: "(\\/\\/|" + _.RE_STARTERS_RE + "|\\b(split|return|print|reverse|grep)\\b)\\s*",
          keywords: "split return print reverse grep",
          relevance: 0,
          contains: [
            _.HASH_COMMENT_MODE,
            {
              className: "regexp",
              variants: [
                // allow matching common delimiters
                { begin: se("s|tr|y", H.either(...oe, { capture: !0 })) },
                // and then paired delmis
                { begin: se("s|tr|y", "\\(", "\\)") },
                { begin: se("s|tr|y", "\\[", "\\]") },
                { begin: se("s|tr|y", "\\{", "\\}") }
              ],
              relevance: 2
            },
            {
              className: "regexp",
              variants: [
                {
                  // could be a comment in many languages so do not count
                  // as relevant
                  begin: /(m|qr)\/\//,
                  relevance: 0
                },
                // prefix is optional with /regex/
                { begin: ce("(?:m|qr)?", /\//, /\//) },
                // allow matching common delimiters
                { begin: ce("m|qr", H.either(...oe, { capture: !0 }), /\1/) },
                // allow common paired delmins
                { begin: ce("m|qr", /\(/, /\)/) },
                { begin: ce("m|qr", /\[/, /\]/) },
                { begin: ce("m|qr", /\{/, /\}/) }
              ]
            }
          ]
        },
        {
          className: "function",
          beginKeywords: "sub method",
          end: "(\\s*\\(.*?\\))?[;{]",
          excludeEnd: !0,
          relevance: 5,
          contains: [_.TITLE_MODE, te]
        },
        {
          className: "class",
          beginKeywords: "class",
          end: "[;{]",
          excludeEnd: !0,
          relevance: 5,
          contains: [_.TITLE_MODE, te, ee]
        },
        {
          begin: "-\\w\\b",
          relevance: 0
        },
        {
          begin: "^__DATA__$",
          end: "^__END__$",
          subLanguage: "mojolicious",
          contains: [
            {
              begin: "^@@.*",
              end: "$",
              className: "comment"
            }
          ]
        }
      ];
      return J.contains = le, Z.contains = le, {
        name: "Perl",
        aliases: [
          "pl",
          "pm"
        ],
        keywords: G,
        contains: le
      };
    }
    return perl_1 = O, perl_1;
  }
  var objectivec_1, hasRequiredObjectivec;
  function requireObjectivec() {
    if (hasRequiredObjectivec) return objectivec_1;
    hasRequiredObjectivec = 1;
    function O(_) {
      const H = {
        className: "built_in",
        begin: "\\b(AV|CA|CF|CG|CI|CL|CM|CN|CT|MK|MP|MTK|MTL|NS|SCN|SK|UI|WK|XC)\\w+"
      }, V = /[a-zA-Z@][a-zA-Z0-9_]*/, te = {
        "variable.language": [
          "this",
          "super"
        ],
        $pattern: V,
        keyword: [
          "while",
          "export",
          "sizeof",
          "typedef",
          "const",
          "struct",
          "for",
          "union",
          "volatile",
          "static",
          "mutable",
          "if",
          "do",
          "return",
          "goto",
          "enum",
          "else",
          "break",
          "extern",
          "asm",
          "case",
          "default",
          "register",
          "explicit",
          "typename",
          "switch",
          "continue",
          "inline",
          "readonly",
          "assign",
          "readwrite",
          "self",
          "@synchronized",
          "id",
          "typeof",
          "nonatomic",
          "IBOutlet",
          "IBAction",
          "strong",
          "weak",
          "copy",
          "in",
          "out",
          "inout",
          "bycopy",
          "byref",
          "oneway",
          "__strong",
          "__weak",
          "__block",
          "__autoreleasing",
          "@private",
          "@protected",
          "@public",
          "@try",
          "@property",
          "@end",
          "@throw",
          "@catch",
          "@finally",
          "@autoreleasepool",
          "@synthesize",
          "@dynamic",
          "@selector",
          "@optional",
          "@required",
          "@encode",
          "@package",
          "@import",
          "@defs",
          "@compatibility_alias",
          "__bridge",
          "__bridge_transfer",
          "__bridge_retained",
          "__bridge_retain",
          "__covariant",
          "__contravariant",
          "__kindof",
          "_Nonnull",
          "_Nullable",
          "_Null_unspecified",
          "__FUNCTION__",
          "__PRETTY_FUNCTION__",
          "__attribute__",
          "getter",
          "setter",
          "retain",
          "unsafe_unretained",
          "nonnull",
          "nullable",
          "null_unspecified",
          "null_resettable",
          "class",
          "instancetype",
          "NS_DESIGNATED_INITIALIZER",
          "NS_UNAVAILABLE",
          "NS_REQUIRES_SUPER",
          "NS_RETURNS_INNER_POINTER",
          "NS_INLINE",
          "NS_AVAILABLE",
          "NS_DEPRECATED",
          "NS_ENUM",
          "NS_OPTIONS",
          "NS_SWIFT_UNAVAILABLE",
          "NS_ASSUME_NONNULL_BEGIN",
          "NS_ASSUME_NONNULL_END",
          "NS_REFINED_FOR_SWIFT",
          "NS_SWIFT_NAME",
          "NS_SWIFT_NOTHROW",
          "NS_DURING",
          "NS_HANDLER",
          "NS_ENDHANDLER",
          "NS_VALUERETURN",
          "NS_VOIDRETURN"
        ],
        literal: [
          "false",
          "true",
          "FALSE",
          "TRUE",
          "nil",
          "YES",
          "NO",
          "NULL"
        ],
        built_in: [
          "dispatch_once_t",
          "dispatch_queue_t",
          "dispatch_sync",
          "dispatch_async",
          "dispatch_once"
        ],
        type: [
          "int",
          "float",
          "char",
          "unsigned",
          "signed",
          "short",
          "long",
          "double",
          "wchar_t",
          "unichar",
          "void",
          "bool",
          "BOOL",
          "id|0",
          "_Bool"
        ]
      }, ne = {
        $pattern: V,
        keyword: [
          "@interface",
          "@class",
          "@protocol",
          "@implementation"
        ]
      };
      return {
        name: "Objective-C",
        aliases: [
          "mm",
          "objc",
          "obj-c",
          "obj-c++",
          "objective-c++"
        ],
        keywords: te,
        illegal: "</",
        contains: [
          H,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.C_NUMBER_MODE,
          _.QUOTE_STRING_MODE,
          _.APOS_STRING_MODE,
          {
            className: "string",
            variants: [
              {
                begin: '@"',
                end: '"',
                illegal: "\\n",
                contains: [_.BACKSLASH_ESCAPE]
              }
            ]
          },
          {
            className: "meta",
            begin: /#\s*[a-z]+\b/,
            end: /$/,
            keywords: { keyword: "if else elif endif define undef warning error line pragma ifdef ifndef include" },
            contains: [
              {
                begin: /\\\n/,
                relevance: 0
              },
              _.inherit(_.QUOTE_STRING_MODE, { className: "string" }),
              {
                className: "string",
                begin: /<.*?>/,
                end: /$/,
                illegal: "\\n"
              },
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          {
            className: "class",
            begin: "(" + ne.keyword.join("|") + ")\\b",
            end: /(\{|$)/,
            excludeEnd: !0,
            keywords: ne,
            contains: [_.UNDERSCORE_TITLE_MODE]
          },
          {
            begin: "\\." + _.UNDERSCORE_IDENT_RE,
            relevance: 0
          }
        ]
      };
    }
    return objectivec_1 = O, objectivec_1;
  }
  var php_1, hasRequiredPhp;
  function requirePhp() {
    if (hasRequiredPhp) return php_1;
    hasRequiredPhp = 1;
    function O(_) {
      const H = _.regex, V = /(?![A-Za-z0-9])(?![$])/, q = H.concat(
        /[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*/,
        V
      ), G = H.concat(
        /(\\?[A-Z][a-z0-9_\x7f-\xff]+|\\?[A-Z]+(?=[A-Z][a-z0-9_\x7f-\xff])){1,}/,
        V
      ), J = H.concat(
        /[A-Z]+/,
        V
      ), Z = {
        scope: "variable",
        match: "\\$+" + q
      }, te = {
        scope: "meta",
        variants: [
          { begin: /<\?php/, relevance: 10 },
          // boost for obvious PHP
          { begin: /<\?=/ },
          // less relevant per PSR-1 which says not to use short-tags
          { begin: /<\?/, relevance: 0.1 },
          { begin: /\?>/ }
          // end php tag
        ]
      }, ne = {
        scope: "subst",
        variants: [
          { begin: /\$\w+/ },
          {
            begin: /\{\$/,
            end: /\}/
          }
        ]
      }, ee = _.inherit(_.APOS_STRING_MODE, { illegal: null }), ae = _.inherit(_.QUOTE_STRING_MODE, {
        illegal: null,
        contains: _.QUOTE_STRING_MODE.contains.concat(ne)
      }), oe = {
        begin: /<<<[ \t]*(?:(\w+)|"(\w+)")\n/,
        end: /[ \t]*(\w+)\b/,
        contains: _.QUOTE_STRING_MODE.contains.concat(ne),
        "on:begin": (Le, Qe) => {
          Qe.data._beginMatch = Le[1] || Le[2];
        },
        "on:end": (Le, Qe) => {
          Qe.data._beginMatch !== Le[1] && Qe.ignoreMatch();
        }
      }, se = _.END_SAME_AS_BEGIN({
        begin: /<<<[ \t]*'(\w+)'\n/,
        end: /[ \t]*(\w+)\b/
      }), ce = `[ 	
]`, le = {
        scope: "string",
        variants: [
          ae,
          ee,
          oe,
          se
        ]
      }, ue = {
        scope: "number",
        variants: [
          { begin: "\\b0[bB][01]+(?:_[01]+)*\\b" },
          // Binary w/ underscore support
          { begin: "\\b0[oO][0-7]+(?:_[0-7]+)*\\b" },
          // Octals w/ underscore support
          { begin: "\\b0[xX][\\da-fA-F]+(?:_[\\da-fA-F]+)*\\b" },
          // Hex w/ underscore support
          // Decimals w/ underscore support, with optional fragments and scientific exponent (e) suffix.
          { begin: "(?:\\b\\d+(?:_\\d+)*(\\.(?:\\d+(?:_\\d+)*))?|\\B\\.\\d+)(?:[eE][+-]?\\d+)?" }
        ],
        relevance: 0
      }, de = [
        "false",
        "null",
        "true"
      ], fe = [
        // Magic constants:
        // <https://www.php.net/manual/en/language.constants.predefined.php>
        "__CLASS__",
        "__DIR__",
        "__FILE__",
        "__FUNCTION__",
        "__COMPILER_HALT_OFFSET__",
        "__LINE__",
        "__METHOD__",
        "__NAMESPACE__",
        "__TRAIT__",
        // Function that look like language construct or language construct that look like function:
        // List of keywords that may not require parenthesis
        "die",
        "echo",
        "exit",
        "include",
        "include_once",
        "print",
        "require",
        "require_once",
        // These are not language construct (function) but operate on the currently-executing function and can access the current symbol table
        // 'compact extract func_get_arg func_get_args func_num_args get_called_class get_parent_class ' +
        // Other keywords:
        // <https://www.php.net/manual/en/reserved.php>
        // <https://www.php.net/manual/en/language.types.type-juggling.php>
        "array",
        "abstract",
        "and",
        "as",
        "binary",
        "bool",
        "boolean",
        "break",
        "callable",
        "case",
        "catch",
        "class",
        "clone",
        "const",
        "continue",
        "declare",
        "default",
        "do",
        "double",
        "else",
        "elseif",
        "empty",
        "enddeclare",
        "endfor",
        "endforeach",
        "endif",
        "endswitch",
        "endwhile",
        "enum",
        "eval",
        "extends",
        "final",
        "finally",
        "float",
        "for",
        "foreach",
        "from",
        "global",
        "goto",
        "if",
        "implements",
        "instanceof",
        "insteadof",
        "int",
        "integer",
        "interface",
        "isset",
        "iterable",
        "list",
        "match|0",
        "mixed",
        "new",
        "never",
        "object",
        "or",
        "private",
        "protected",
        "public",
        "readonly",
        "real",
        "return",
        "string",
        "switch",
        "throw",
        "trait",
        "try",
        "unset",
        "use",
        "var",
        "void",
        "while",
        "xor",
        "yield"
      ], he = [
        // Standard PHP library:
        // <https://www.php.net/manual/en/book.spl.php>
        "Error|0",
        "AppendIterator",
        "ArgumentCountError",
        "ArithmeticError",
        "ArrayIterator",
        "ArrayObject",
        "AssertionError",
        "BadFunctionCallException",
        "BadMethodCallException",
        "CachingIterator",
        "CallbackFilterIterator",
        "CompileError",
        "Countable",
        "DirectoryIterator",
        "DivisionByZeroError",
        "DomainException",
        "EmptyIterator",
        "ErrorException",
        "Exception",
        "FilesystemIterator",
        "FilterIterator",
        "GlobIterator",
        "InfiniteIterator",
        "InvalidArgumentException",
        "IteratorIterator",
        "LengthException",
        "LimitIterator",
        "LogicException",
        "MultipleIterator",
        "NoRewindIterator",
        "OutOfBoundsException",
        "OutOfRangeException",
        "OuterIterator",
        "OverflowException",
        "ParentIterator",
        "ParseError",
        "RangeException",
        "RecursiveArrayIterator",
        "RecursiveCachingIterator",
        "RecursiveCallbackFilterIterator",
        "RecursiveDirectoryIterator",
        "RecursiveFilterIterator",
        "RecursiveIterator",
        "RecursiveIteratorIterator",
        "RecursiveRegexIterator",
        "RecursiveTreeIterator",
        "RegexIterator",
        "RuntimeException",
        "SeekableIterator",
        "SplDoublyLinkedList",
        "SplFileInfo",
        "SplFileObject",
        "SplFixedArray",
        "SplHeap",
        "SplMaxHeap",
        "SplMinHeap",
        "SplObjectStorage",
        "SplObserver",
        "SplPriorityQueue",
        "SplQueue",
        "SplStack",
        "SplSubject",
        "SplTempFileObject",
        "TypeError",
        "UnderflowException",
        "UnexpectedValueException",
        "UnhandledMatchError",
        // Reserved interfaces:
        // <https://www.php.net/manual/en/reserved.interfaces.php>
        "ArrayAccess",
        "BackedEnum",
        "Closure",
        "Fiber",
        "Generator",
        "Iterator",
        "IteratorAggregate",
        "Serializable",
        "Stringable",
        "Throwable",
        "Traversable",
        "UnitEnum",
        "WeakReference",
        "WeakMap",
        // Reserved classes:
        // <https://www.php.net/manual/en/reserved.classes.php>
        "Directory",
        "__PHP_Incomplete_Class",
        "parent",
        "php_user_filter",
        "self",
        "static",
        "stdClass"
      ], pe = {
        keyword: fe,
        literal: ((Le) => {
          const Qe = [];
          return Le.forEach((ke) => {
            Qe.push(ke), ke.toLowerCase() === ke ? Qe.push(ke.toUpperCase()) : Qe.push(ke.toLowerCase());
          }), Qe;
        })(de),
        built_in: he
      }, ye = (Le) => Le.map((Qe) => Qe.replace(/\|\d+$/, "")), ge = { variants: [
        {
          match: [
            /new/,
            H.concat(ce, "+"),
            // to prevent built ins from being confused as the class constructor call
            H.concat("(?!", ye(he).join("\\b|"), "\\b)"),
            G
          ],
          scope: {
            1: "keyword",
            4: "title.class"
          }
        }
      ] }, ve = H.concat(q, "\\b(?!\\()"), be = { variants: [
        {
          match: [
            H.concat(
              /::/,
              H.lookahead(/(?!class\b)/)
            ),
            ve
          ],
          scope: { 2: "variable.constant" }
        },
        {
          match: [
            /::/,
            /class/
          ],
          scope: { 2: "variable.language" }
        },
        {
          match: [
            G,
            H.concat(
              /::/,
              H.lookahead(/(?!class\b)/)
            ),
            ve
          ],
          scope: {
            1: "title.class",
            3: "variable.constant"
          }
        },
        {
          match: [
            G,
            H.concat(
              "::",
              H.lookahead(/(?!class\b)/)
            )
          ],
          scope: { 1: "title.class" }
        },
        {
          match: [
            G,
            /::/,
            /class/
          ],
          scope: {
            1: "title.class",
            3: "variable.language"
          }
        }
      ] }, Ce = {
        scope: "attr",
        match: H.concat(q, H.lookahead(":"), H.lookahead(/(?!::)/))
      }, Se = {
        relevance: 0,
        begin: /\(/,
        end: /\)/,
        keywords: pe,
        contains: [
          Ce,
          Z,
          be,
          _.C_BLOCK_COMMENT_MODE,
          le,
          ue,
          ge
        ]
      }, we = {
        relevance: 0,
        match: [
          /\b/,
          // to prevent keywords from being confused as the function title
          H.concat("(?!fn\\b|function\\b|", ye(fe).join("\\b|"), "|", ye(he).join("\\b|"), "\\b)"),
          q,
          H.concat(ce, "*"),
          H.lookahead(/(?=\()/)
        ],
        scope: { 3: "title.function.invoke" },
        contains: [Se]
      };
      Se.contains.push(we);
      const Ee = [
        Ce,
        be,
        _.C_BLOCK_COMMENT_MODE,
        le,
        ue,
        ge
      ], xe = {
        begin: H.concat(
          /#\[\s*\\?/,
          H.either(
            G,
            J
          )
        ),
        beginScope: "meta",
        end: /]/,
        endScope: "meta",
        keywords: {
          literal: de,
          keyword: [
            "new",
            "array"
          ]
        },
        contains: [
          {
            begin: /\[/,
            end: /]/,
            keywords: {
              literal: de,
              keyword: [
                "new",
                "array"
              ]
            },
            contains: [
              "self",
              ...Ee
            ]
          },
          ...Ee,
          {
            scope: "meta",
            variants: [
              { match: G },
              { match: J }
            ]
          }
        ]
      };
      return {
        case_insensitive: !1,
        keywords: pe,
        contains: [
          xe,
          _.HASH_COMMENT_MODE,
          _.COMMENT("//", "$"),
          _.COMMENT(
            "/\\*",
            "\\*/",
            { contains: [
              {
                scope: "doctag",
                match: "@[A-Za-z]+"
              }
            ] }
          ),
          {
            match: /__halt_compiler\(\);/,
            keywords: "__halt_compiler",
            starts: {
              scope: "comment",
              end: _.MATCH_NOTHING_RE,
              contains: [
                {
                  match: /\?>/,
                  scope: "meta",
                  endsParent: !0
                }
              ]
            }
          },
          te,
          {
            scope: "variable.language",
            match: /\$this\b/
          },
          Z,
          we,
          be,
          {
            match: [
              /const/,
              /\s/,
              q
            ],
            scope: {
              1: "keyword",
              3: "variable.constant"
            }
          },
          ge,
          {
            scope: "function",
            relevance: 0,
            beginKeywords: "fn function",
            end: /[;{]/,
            excludeEnd: !0,
            illegal: "[$%\\[]",
            contains: [
              { beginKeywords: "use" },
              _.UNDERSCORE_TITLE_MODE,
              {
                begin: "=>",
                // No markup, just a relevance booster
                endsParent: !0
              },
              {
                scope: "params",
                begin: "\\(",
                end: "\\)",
                excludeBegin: !0,
                excludeEnd: !0,
                keywords: pe,
                contains: [
                  "self",
                  xe,
                  Z,
                  be,
                  _.C_BLOCK_COMMENT_MODE,
                  le,
                  ue
                ]
              }
            ]
          },
          {
            scope: "class",
            variants: [
              {
                beginKeywords: "enum",
                illegal: /[($"]/
              },
              {
                beginKeywords: "class interface trait",
                illegal: /[:($"]/
              }
            ],
            relevance: 0,
            end: /\{/,
            excludeEnd: !0,
            contains: [
              { beginKeywords: "extends implements" },
              _.UNDERSCORE_TITLE_MODE
            ]
          },
          // both use and namespace still use "old style" rules (vs multi-match)
          // because the namespace name can include `\` and we still want each
          // element to be treated as its own *individual* title
          {
            beginKeywords: "namespace",
            relevance: 0,
            end: ";",
            illegal: /[.']/,
            contains: [_.inherit(_.UNDERSCORE_TITLE_MODE, { scope: "title.class" })]
          },
          {
            beginKeywords: "use",
            relevance: 0,
            end: ";",
            contains: [
              // TODO: title.function vs title.class
              {
                match: /\b(as|const|function)\b/,
                scope: "keyword"
              },
              // TODO: could be title.class or title.function
              _.UNDERSCORE_TITLE_MODE
            ]
          },
          le,
          ue
        ]
      };
    }
    return php_1 = O, php_1;
  }
  var phpTemplate_1, hasRequiredPhpTemplate;
  function requirePhpTemplate() {
    if (hasRequiredPhpTemplate) return phpTemplate_1;
    hasRequiredPhpTemplate = 1;
    function O(_) {
      return {
        name: "PHP template",
        subLanguage: "xml",
        contains: [
          {
            begin: /<\?(php|=)?/,
            end: /\?>/,
            subLanguage: "php",
            contains: [
              // We don't want the php closing tag ?> to close the PHP block when
              // inside any of the following blocks:
              {
                begin: "/\\*",
                end: "\\*/",
                skip: !0
              },
              {
                begin: 'b"',
                end: '"',
                skip: !0
              },
              {
                begin: "b'",
                end: "'",
                skip: !0
              },
              _.inherit(_.APOS_STRING_MODE, {
                illegal: null,
                className: null,
                contains: null,
                skip: !0
              }),
              _.inherit(_.QUOTE_STRING_MODE, {
                illegal: null,
                className: null,
                contains: null,
                skip: !0
              })
            ]
          }
        ]
      };
    }
    return phpTemplate_1 = O, phpTemplate_1;
  }
  var plaintext_1, hasRequiredPlaintext;
  function requirePlaintext() {
    if (hasRequiredPlaintext) return plaintext_1;
    hasRequiredPlaintext = 1;
    function O(_) {
      return {
        name: "Plain text",
        aliases: [
          "text",
          "txt"
        ],
        disableAutodetect: !0
      };
    }
    return plaintext_1 = O, plaintext_1;
  }
  var python_1, hasRequiredPython;
  function requirePython() {
    if (hasRequiredPython) return python_1;
    hasRequiredPython = 1;
    function O(_) {
      const H = _.regex, V = new RegExp("[\\p{XID_Start}_]\\p{XID_Continue}*", "u"), q = [
        "and",
        "as",
        "assert",
        "async",
        "await",
        "break",
        "case",
        "class",
        "continue",
        "def",
        "del",
        "elif",
        "else",
        "except",
        "finally",
        "for",
        "from",
        "global",
        "if",
        "import",
        "in",
        "is",
        "lambda",
        "match",
        "nonlocal|10",
        "not",
        "or",
        "pass",
        "raise",
        "return",
        "try",
        "while",
        "with",
        "yield"
      ], te = {
        $pattern: /[A-Za-z]\w+|__\w+__/,
        keyword: q,
        built_in: [
          "__import__",
          "abs",
          "all",
          "any",
          "ascii",
          "bin",
          "bool",
          "breakpoint",
          "bytearray",
          "bytes",
          "callable",
          "chr",
          "classmethod",
          "compile",
          "complex",
          "delattr",
          "dict",
          "dir",
          "divmod",
          "enumerate",
          "eval",
          "exec",
          "filter",
          "float",
          "format",
          "frozenset",
          "getattr",
          "globals",
          "hasattr",
          "hash",
          "help",
          "hex",
          "id",
          "input",
          "int",
          "isinstance",
          "issubclass",
          "iter",
          "len",
          "list",
          "locals",
          "map",
          "max",
          "memoryview",
          "min",
          "next",
          "object",
          "oct",
          "open",
          "ord",
          "pow",
          "print",
          "property",
          "range",
          "repr",
          "reversed",
          "round",
          "set",
          "setattr",
          "slice",
          "sorted",
          "staticmethod",
          "str",
          "sum",
          "super",
          "tuple",
          "type",
          "vars",
          "zip"
        ],
        literal: [
          "__debug__",
          "Ellipsis",
          "False",
          "None",
          "NotImplemented",
          "True"
        ],
        type: [
          "Any",
          "Callable",
          "Coroutine",
          "Dict",
          "List",
          "Literal",
          "Generic",
          "Optional",
          "Sequence",
          "Set",
          "Tuple",
          "Type",
          "Union"
        ]
      }, ne = {
        className: "meta",
        begin: /^(>>>|\.\.\.) /
      }, ee = {
        className: "subst",
        begin: /\{/,
        end: /\}/,
        keywords: te,
        illegal: /#/
      }, ae = {
        begin: /\{\{/,
        relevance: 0
      }, oe = {
        className: "string",
        contains: [_.BACKSLASH_ESCAPE],
        variants: [
          {
            begin: /([uU]|[bB]|[rR]|[bB][rR]|[rR][bB])?'''/,
            end: /'''/,
            contains: [
              _.BACKSLASH_ESCAPE,
              ne
            ],
            relevance: 10
          },
          {
            begin: /([uU]|[bB]|[rR]|[bB][rR]|[rR][bB])?"""/,
            end: /"""/,
            contains: [
              _.BACKSLASH_ESCAPE,
              ne
            ],
            relevance: 10
          },
          {
            begin: /([fF][rR]|[rR][fF]|[fF])'''/,
            end: /'''/,
            contains: [
              _.BACKSLASH_ESCAPE,
              ne,
              ae,
              ee
            ]
          },
          {
            begin: /([fF][rR]|[rR][fF]|[fF])"""/,
            end: /"""/,
            contains: [
              _.BACKSLASH_ESCAPE,
              ne,
              ae,
              ee
            ]
          },
          {
            begin: /([uU]|[rR])'/,
            end: /'/,
            relevance: 10
          },
          {
            begin: /([uU]|[rR])"/,
            end: /"/,
            relevance: 10
          },
          {
            begin: /([bB]|[bB][rR]|[rR][bB])'/,
            end: /'/
          },
          {
            begin: /([bB]|[bB][rR]|[rR][bB])"/,
            end: /"/
          },
          {
            begin: /([fF][rR]|[rR][fF]|[fF])'/,
            end: /'/,
            contains: [
              _.BACKSLASH_ESCAPE,
              ae,
              ee
            ]
          },
          {
            begin: /([fF][rR]|[rR][fF]|[fF])"/,
            end: /"/,
            contains: [
              _.BACKSLASH_ESCAPE,
              ae,
              ee
            ]
          },
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE
        ]
      }, se = "[0-9](_?[0-9])*", ce = `(\\b(${se}))?\\.(${se})|\\b(${se})\\.`, le = `\\b|${q.join("|")}`, ue = {
        className: "number",
        relevance: 0,
        variants: [
          // exponentfloat, pointfloat
          // https://docs.python.org/3.9/reference/lexical_analysis.html#floating-point-literals
          // optionally imaginary
          // https://docs.python.org/3.9/reference/lexical_analysis.html#imaginary-literals
          // Note: no leading \b because floats can start with a decimal point
          // and we don't want to mishandle e.g. `fn(.5)`,
          // no trailing \b for pointfloat because it can end with a decimal point
          // and we don't want to mishandle e.g. `0..hex()`; this should be safe
          // because both MUST contain a decimal point and so cannot be confused with
          // the interior part of an identifier
          {
            begin: `(\\b(${se})|(${ce}))[eE][+-]?(${se})[jJ]?(?=${le})`
          },
          {
            begin: `(${ce})[jJ]?`
          },
          // decinteger, bininteger, octinteger, hexinteger
          // https://docs.python.org/3.9/reference/lexical_analysis.html#integer-literals
          // optionally "long" in Python 2
          // https://docs.python.org/2.7/reference/lexical_analysis.html#integer-and-long-integer-literals
          // decinteger is optionally imaginary
          // https://docs.python.org/3.9/reference/lexical_analysis.html#imaginary-literals
          {
            begin: `\\b([1-9](_?[0-9])*|0+(_?0)*)[lLjJ]?(?=${le})`
          },
          {
            begin: `\\b0[bB](_?[01])+[lL]?(?=${le})`
          },
          {
            begin: `\\b0[oO](_?[0-7])+[lL]?(?=${le})`
          },
          {
            begin: `\\b0[xX](_?[0-9a-fA-F])+[lL]?(?=${le})`
          },
          // imagnumber (digitpart-based)
          // https://docs.python.org/3.9/reference/lexical_analysis.html#imaginary-literals
          {
            begin: `\\b(${se})[jJ](?=${le})`
          }
        ]
      }, de = {
        className: "comment",
        begin: H.lookahead(/# type:/),
        end: /$/,
        keywords: te,
        contains: [
          {
            // prevent keywords from coloring `type`
            begin: /# type:/
          },
          // comment within a datatype comment includes no keywords
          {
            begin: /#/,
            end: /\b\B/,
            endsWithParent: !0
          }
        ]
      }, fe = {
        className: "params",
        variants: [
          // Exclude params in functions without params
          {
            className: "",
            begin: /\(\s*\)/,
            skip: !0
          },
          {
            begin: /\(/,
            end: /\)/,
            excludeBegin: !0,
            excludeEnd: !0,
            keywords: te,
            contains: [
              "self",
              ne,
              ue,
              oe,
              _.HASH_COMMENT_MODE
            ]
          }
        ]
      };
      return ee.contains = [
        oe,
        ue,
        ne
      ], {
        name: "Python",
        aliases: [
          "py",
          "gyp",
          "ipython"
        ],
        unicodeRegex: !0,
        keywords: te,
        illegal: /(<\/|\?)|=>/,
        contains: [
          ne,
          ue,
          {
            // very common convention
            scope: "variable.language",
            match: /\bself\b/
          },
          {
            // eat "if" prior to string so that it won't accidentally be
            // labeled as an f-string
            beginKeywords: "if",
            relevance: 0
          },
          { match: /\bor\b/, scope: "keyword" },
          oe,
          de,
          _.HASH_COMMENT_MODE,
          {
            match: [
              /\bdef/,
              /\s+/,
              V
            ],
            scope: {
              1: "keyword",
              3: "title.function"
            },
            contains: [fe]
          },
          {
            variants: [
              {
                match: [
                  /\bclass/,
                  /\s+/,
                  V,
                  /\s*/,
                  /\(\s*/,
                  V,
                  /\s*\)/
                ]
              },
              {
                match: [
                  /\bclass/,
                  /\s+/,
                  V
                ]
              }
            ],
            scope: {
              1: "keyword",
              3: "title.class",
              6: "title.class.inherited"
            }
          },
          {
            className: "meta",
            begin: /^[\t ]*@/,
            end: /(?=#)|$/,
            contains: [
              ue,
              fe,
              oe
            ]
          }
        ]
      };
    }
    return python_1 = O, python_1;
  }
  var pythonRepl_1, hasRequiredPythonRepl;
  function requirePythonRepl() {
    if (hasRequiredPythonRepl) return pythonRepl_1;
    hasRequiredPythonRepl = 1;
    function O(_) {
      return {
        aliases: ["pycon"],
        contains: [
          {
            className: "meta.prompt",
            starts: {
              // a space separates the REPL prefix from the actual code
              // this is purely for cleaner HTML output
              end: / |$/,
              starts: {
                end: "$",
                subLanguage: "python"
              }
            },
            variants: [
              { begin: /^>>>(?=[ ]|$)/ },
              { begin: /^\.\.\.(?=[ ]|$)/ }
            ]
          }
        ]
      };
    }
    return pythonRepl_1 = O, pythonRepl_1;
  }
  var r_1, hasRequiredR;
  function requireR() {
    if (hasRequiredR) return r_1;
    hasRequiredR = 1;
    function O(_) {
      const H = _.regex, V = /(?:(?:[a-zA-Z]|\.[._a-zA-Z])[._a-zA-Z0-9]*)|\.(?!\d)/, q = H.either(
        // Special case: only hexadecimal binary powers can contain fractions
        /0[xX][0-9a-fA-F]+\.[0-9a-fA-F]*[pP][+-]?\d+i?/,
        // Hexadecimal numbers without fraction and optional binary power
        /0[xX][0-9a-fA-F]+(?:[pP][+-]?\d+)?[Li]?/,
        // Decimal numbers
        /(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][+-]?\d+)?[Li]?/
      ), G = /[=!<>:]=|\|\||&&|:::?|<-|<<-|->>|->|\|>|[-+*\/?!$&|:<=>@^~]|\*\*/, J = H.either(
        /[()]/,
        /[{}]/,
        /\[\[/,
        /[[\]]/,
        /\\/,
        /,/
      );
      return {
        name: "R",
        keywords: {
          $pattern: V,
          keyword: "function if in break next repeat else for while",
          literal: "NULL NA TRUE FALSE Inf NaN NA_integer_|10 NA_real_|10 NA_character_|10 NA_complex_|10",
          built_in: (
            // Builtin constants
            "LETTERS letters month.abb month.name pi T F abs acos acosh all any anyNA Arg as.call as.character as.complex as.double as.environment as.integer as.logical as.null.default as.numeric as.raw asin asinh atan atanh attr attributes baseenv browser c call ceiling class Conj cos cosh cospi cummax cummin cumprod cumsum digamma dim dimnames emptyenv exp expression floor forceAndCall gamma gc.time globalenv Im interactive invisible is.array is.atomic is.call is.character is.complex is.double is.environment is.expression is.finite is.function is.infinite is.integer is.language is.list is.logical is.matrix is.na is.name is.nan is.null is.numeric is.object is.pairlist is.raw is.recursive is.single is.symbol lazyLoadDBfetch length lgamma list log max min missing Mod names nargs nzchar oldClass on.exit pos.to.env proc.time prod quote range Re rep retracemem return round seq_along seq_len seq.int sign signif sin sinh sinpi sqrt standardGeneric substitute sum switch tan tanh tanpi tracemem trigamma trunc unclass untracemem UseMethod xtfrm"
          )
        },
        contains: [
          // Roxygen comments
          _.COMMENT(
            /#'/,
            /$/,
            { contains: [
              {
                // Handle `@examples` separately to cause all subsequent code
                // until the next `@`-tag on its own line to be kept as-is,
                // preventing highlighting. This code is example R code, so nested
                // doctags shouldn’t be treated as such. See
                // `test/markup/r/roxygen.txt` for an example.
                scope: "doctag",
                match: /@examples/,
                starts: {
                  end: H.lookahead(H.either(
                    // end if another doc comment
                    /\n^#'\s*(?=@[a-zA-Z]+)/,
                    // or a line with no comment
                    /\n^(?!#')/
                  )),
                  endsParent: !0
                }
              },
              {
                // Handle `@param` to highlight the parameter name following
                // after.
                scope: "doctag",
                begin: "@param",
                end: /$/,
                contains: [
                  {
                    scope: "variable",
                    variants: [
                      { match: V },
                      { match: /`(?:\\.|[^`\\])+`/ }
                    ],
                    endsParent: !0
                  }
                ]
              },
              {
                scope: "doctag",
                match: /@[a-zA-Z]+/
              },
              {
                scope: "keyword",
                match: /\\[a-zA-Z]+/
              }
            ] }
          ),
          _.HASH_COMMENT_MODE,
          {
            scope: "string",
            contains: [_.BACKSLASH_ESCAPE],
            variants: [
              _.END_SAME_AS_BEGIN({
                begin: /[rR]"(-*)\(/,
                end: /\)(-*)"/
              }),
              _.END_SAME_AS_BEGIN({
                begin: /[rR]"(-*)\{/,
                end: /\}(-*)"/
              }),
              _.END_SAME_AS_BEGIN({
                begin: /[rR]"(-*)\[/,
                end: /\](-*)"/
              }),
              _.END_SAME_AS_BEGIN({
                begin: /[rR]'(-*)\(/,
                end: /\)(-*)'/
              }),
              _.END_SAME_AS_BEGIN({
                begin: /[rR]'(-*)\{/,
                end: /\}(-*)'/
              }),
              _.END_SAME_AS_BEGIN({
                begin: /[rR]'(-*)\[/,
                end: /\](-*)'/
              }),
              {
                begin: '"',
                end: '"',
                relevance: 0
              },
              {
                begin: "'",
                end: "'",
                relevance: 0
              }
            ]
          },
          // Matching numbers immediately following punctuation and operators is
          // tricky since we need to look at the character ahead of a number to
          // ensure the number is not part of an identifier, and we cannot use
          // negative look-behind assertions. So instead we explicitly handle all
          // possible combinations of (operator|punctuation), number.
          // TODO: replace with negative look-behind when available
          // { begin: /(?<![a-zA-Z0-9._])0[xX][0-9a-fA-F]+\.[0-9a-fA-F]*[pP][+-]?\d+i?/ },
          // { begin: /(?<![a-zA-Z0-9._])0[xX][0-9a-fA-F]+([pP][+-]?\d+)?[Li]?/ },
          // { begin: /(?<![a-zA-Z0-9._])(\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?[Li]?/ }
          {
            relevance: 0,
            variants: [
              {
                scope: {
                  1: "operator",
                  2: "number"
                },
                match: [
                  G,
                  q
                ]
              },
              {
                scope: {
                  1: "operator",
                  2: "number"
                },
                match: [
                  /%[^%]*%/,
                  q
                ]
              },
              {
                scope: {
                  1: "punctuation",
                  2: "number"
                },
                match: [
                  J,
                  q
                ]
              },
              {
                scope: { 2: "number" },
                match: [
                  /[^a-zA-Z0-9._]|^/,
                  // not part of an identifier, or start of document
                  q
                ]
              }
            ]
          },
          // Operators/punctuation when they're not directly followed by numbers
          {
            // Relevance boost for the most common assignment form.
            scope: { 3: "operator" },
            match: [
              V,
              /\s+/,
              /<-/,
              /\s+/
            ]
          },
          {
            scope: "operator",
            relevance: 0,
            variants: [
              { match: G },
              { match: /%[^%]*%/ }
            ]
          },
          {
            scope: "punctuation",
            relevance: 0,
            match: J
          },
          {
            // Escaped identifier
            begin: "`",
            end: "`",
            contains: [{ begin: /\\./ }]
          }
        ]
      };
    }
    return r_1 = O, r_1;
  }
  var rust_1, hasRequiredRust;
  function requireRust() {
    if (hasRequiredRust) return rust_1;
    hasRequiredRust = 1;
    function O(_) {
      const H = _.regex, V = /(r#)?/, q = H.concat(V, _.UNDERSCORE_IDENT_RE), G = H.concat(V, _.IDENT_RE), J = {
        className: "title.function.invoke",
        relevance: 0,
        begin: H.concat(
          /\b/,
          /(?!let|for|while|if|else|match\b)/,
          G,
          H.lookahead(/\s*\(/)
        )
      }, Z = "([ui](8|16|32|64|128|size)|f(32|64))?", te = [
        "abstract",
        "as",
        "async",
        "await",
        "become",
        "box",
        "break",
        "const",
        "continue",
        "crate",
        "do",
        "dyn",
        "else",
        "enum",
        "extern",
        "false",
        "final",
        "fn",
        "for",
        "if",
        "impl",
        "in",
        "let",
        "loop",
        "macro",
        "match",
        "mod",
        "move",
        "mut",
        "override",
        "priv",
        "pub",
        "ref",
        "return",
        "self",
        "Self",
        "static",
        "struct",
        "super",
        "trait",
        "true",
        "try",
        "type",
        "typeof",
        "union",
        "unsafe",
        "unsized",
        "use",
        "virtual",
        "where",
        "while",
        "yield"
      ], ne = [
        "true",
        "false",
        "Some",
        "None",
        "Ok",
        "Err"
      ], ee = [
        // functions
        "drop ",
        // traits
        "Copy",
        "Send",
        "Sized",
        "Sync",
        "Drop",
        "Fn",
        "FnMut",
        "FnOnce",
        "ToOwned",
        "Clone",
        "Debug",
        "PartialEq",
        "PartialOrd",
        "Eq",
        "Ord",
        "AsRef",
        "AsMut",
        "Into",
        "From",
        "Default",
        "Iterator",
        "Extend",
        "IntoIterator",
        "DoubleEndedIterator",
        "ExactSizeIterator",
        "SliceConcatExt",
        "ToString",
        // macros
        "assert!",
        "assert_eq!",
        "bitflags!",
        "bytes!",
        "cfg!",
        "col!",
        "concat!",
        "concat_idents!",
        "debug_assert!",
        "debug_assert_eq!",
        "env!",
        "eprintln!",
        "panic!",
        "file!",
        "format!",
        "format_args!",
        "include_bytes!",
        "include_str!",
        "line!",
        "local_data_key!",
        "module_path!",
        "option_env!",
        "print!",
        "println!",
        "select!",
        "stringify!",
        "try!",
        "unimplemented!",
        "unreachable!",
        "vec!",
        "write!",
        "writeln!",
        "macro_rules!",
        "assert_ne!",
        "debug_assert_ne!"
      ], ae = [
        "i8",
        "i16",
        "i32",
        "i64",
        "i128",
        "isize",
        "u8",
        "u16",
        "u32",
        "u64",
        "u128",
        "usize",
        "f32",
        "f64",
        "str",
        "char",
        "bool",
        "Box",
        "Option",
        "Result",
        "String",
        "Vec"
      ];
      return {
        name: "Rust",
        aliases: ["rs"],
        keywords: {
          $pattern: _.IDENT_RE + "!?",
          type: ae,
          keyword: te,
          literal: ne,
          built_in: ee
        },
        illegal: "</",
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.COMMENT("/\\*", "\\*/", { contains: ["self"] }),
          _.inherit(_.QUOTE_STRING_MODE, {
            begin: /b?"/,
            illegal: null
          }),
          {
            className: "symbol",
            // negative lookahead to avoid matching `'`
            begin: /'[a-zA-Z_][a-zA-Z0-9_]*(?!')/
          },
          {
            scope: "string",
            variants: [
              { begin: /b?r(#*)"(.|\n)*?"\1(?!#)/ },
              {
                begin: /b?'/,
                end: /'/,
                contains: [
                  {
                    scope: "char.escape",
                    match: /\\('|\w|x\w{2}|u\w{4}|U\w{8})/
                  }
                ]
              }
            ]
          },
          {
            className: "number",
            variants: [
              { begin: "\\b0b([01_]+)" + Z },
              { begin: "\\b0o([0-7_]+)" + Z },
              { begin: "\\b0x([A-Fa-f0-9_]+)" + Z },
              { begin: "\\b(\\d[\\d_]*(\\.[0-9_]+)?([eE][+-]?[0-9_]+)?)" + Z }
            ],
            relevance: 0
          },
          {
            begin: [
              /fn/,
              /\s+/,
              q
            ],
            className: {
              1: "keyword",
              3: "title.function"
            }
          },
          {
            className: "meta",
            begin: "#!?\\[",
            end: "\\]",
            contains: [
              {
                className: "string",
                begin: /"/,
                end: /"/,
                contains: [
                  _.BACKSLASH_ESCAPE
                ]
              }
            ]
          },
          {
            begin: [
              /let/,
              /\s+/,
              /(?:mut\s+)?/,
              q
            ],
            className: {
              1: "keyword",
              3: "keyword",
              4: "variable"
            }
          },
          // must come before impl/for rule later
          {
            begin: [
              /for/,
              /\s+/,
              q,
              /\s+/,
              /in/
            ],
            className: {
              1: "keyword",
              3: "variable",
              5: "keyword"
            }
          },
          {
            begin: [
              /type/,
              /\s+/,
              q
            ],
            className: {
              1: "keyword",
              3: "title.class"
            }
          },
          {
            begin: [
              /(?:trait|enum|struct|union|impl|for)/,
              /\s+/,
              q
            ],
            className: {
              1: "keyword",
              3: "title.class"
            }
          },
          {
            begin: _.IDENT_RE + "::",
            keywords: {
              keyword: "Self",
              built_in: ee,
              type: ae
            }
          },
          {
            className: "punctuation",
            begin: "->"
          },
          J
        ]
      };
    }
    return rust_1 = O, rust_1;
  }
  var scss_1, hasRequiredScss;
  function requireScss() {
    if (hasRequiredScss) return scss_1;
    hasRequiredScss = 1;
    const O = (ne) => ({
      IMPORTANT: {
        scope: "meta",
        begin: "!important"
      },
      BLOCK_COMMENT: ne.C_BLOCK_COMMENT_MODE,
      HEXCOLOR: {
        scope: "number",
        begin: /#(([0-9a-fA-F]{3,4})|(([0-9a-fA-F]{2}){3,4}))\b/
      },
      FUNCTION_DISPATCH: {
        className: "built_in",
        begin: /[\w-]+(?=\()/
      },
      ATTRIBUTE_SELECTOR_MODE: {
        scope: "selector-attr",
        begin: /\[/,
        end: /\]/,
        illegal: "$",
        contains: [
          ne.APOS_STRING_MODE,
          ne.QUOTE_STRING_MODE
        ]
      },
      CSS_NUMBER_MODE: {
        scope: "number",
        begin: ne.NUMBER_RE + "(%|em|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc|px|deg|grad|rad|turn|s|ms|Hz|kHz|dpi|dpcm|dppx)?",
        relevance: 0
      },
      CSS_VARIABLE: {
        className: "attr",
        begin: /--[A-Za-z_][A-Za-z0-9_-]*/
      }
    }), _ = [
      "a",
      "abbr",
      "address",
      "article",
      "aside",
      "audio",
      "b",
      "blockquote",
      "body",
      "button",
      "canvas",
      "caption",
      "cite",
      "code",
      "dd",
      "del",
      "details",
      "dfn",
      "div",
      "dl",
      "dt",
      "em",
      "fieldset",
      "figcaption",
      "figure",
      "footer",
      "form",
      "h1",
      "h2",
      "h3",
      "h4",
      "h5",
      "h6",
      "header",
      "hgroup",
      "html",
      "i",
      "iframe",
      "img",
      "input",
      "ins",
      "kbd",
      "label",
      "legend",
      "li",
      "main",
      "mark",
      "menu",
      "nav",
      "object",
      "ol",
      "optgroup",
      "option",
      "p",
      "picture",
      "q",
      "quote",
      "samp",
      "section",
      "select",
      "source",
      "span",
      "strong",
      "summary",
      "sup",
      "table",
      "tbody",
      "td",
      "textarea",
      "tfoot",
      "th",
      "thead",
      "time",
      "tr",
      "ul",
      "var",
      "video"
    ], H = [
      "defs",
      "g",
      "marker",
      "mask",
      "pattern",
      "svg",
      "switch",
      "symbol",
      "feBlend",
      "feColorMatrix",
      "feComponentTransfer",
      "feComposite",
      "feConvolveMatrix",
      "feDiffuseLighting",
      "feDisplacementMap",
      "feFlood",
      "feGaussianBlur",
      "feImage",
      "feMerge",
      "feMorphology",
      "feOffset",
      "feSpecularLighting",
      "feTile",
      "feTurbulence",
      "linearGradient",
      "radialGradient",
      "stop",
      "circle",
      "ellipse",
      "image",
      "line",
      "path",
      "polygon",
      "polyline",
      "rect",
      "text",
      "use",
      "textPath",
      "tspan",
      "foreignObject",
      "clipPath"
    ], V = [
      ..._,
      ...H
    ], q = [
      "any-hover",
      "any-pointer",
      "aspect-ratio",
      "color",
      "color-gamut",
      "color-index",
      "device-aspect-ratio",
      "device-height",
      "device-width",
      "display-mode",
      "forced-colors",
      "grid",
      "height",
      "hover",
      "inverted-colors",
      "monochrome",
      "orientation",
      "overflow-block",
      "overflow-inline",
      "pointer",
      "prefers-color-scheme",
      "prefers-contrast",
      "prefers-reduced-motion",
      "prefers-reduced-transparency",
      "resolution",
      "scan",
      "scripting",
      "update",
      "width",
      // TODO: find a better solution?
      "min-width",
      "max-width",
      "min-height",
      "max-height"
    ].sort().reverse(), G = [
      "active",
      "any-link",
      "blank",
      "checked",
      "current",
      "default",
      "defined",
      "dir",
      // dir()
      "disabled",
      "drop",
      "empty",
      "enabled",
      "first",
      "first-child",
      "first-of-type",
      "fullscreen",
      "future",
      "focus",
      "focus-visible",
      "focus-within",
      "has",
      // has()
      "host",
      // host or host()
      "host-context",
      // host-context()
      "hover",
      "indeterminate",
      "in-range",
      "invalid",
      "is",
      // is()
      "lang",
      // lang()
      "last-child",
      "last-of-type",
      "left",
      "link",
      "local-link",
      "not",
      // not()
      "nth-child",
      // nth-child()
      "nth-col",
      // nth-col()
      "nth-last-child",
      // nth-last-child()
      "nth-last-col",
      // nth-last-col()
      "nth-last-of-type",
      //nth-last-of-type()
      "nth-of-type",
      //nth-of-type()
      "only-child",
      "only-of-type",
      "optional",
      "out-of-range",
      "past",
      "placeholder-shown",
      "read-only",
      "read-write",
      "required",
      "right",
      "root",
      "scope",
      "target",
      "target-within",
      "user-invalid",
      "valid",
      "visited",
      "where"
      // where()
    ].sort().reverse(), J = [
      "after",
      "backdrop",
      "before",
      "cue",
      "cue-region",
      "first-letter",
      "first-line",
      "grammar-error",
      "marker",
      "part",
      "placeholder",
      "selection",
      "slotted",
      "spelling-error"
    ].sort().reverse(), Z = [
      "accent-color",
      "align-content",
      "align-items",
      "align-self",
      "alignment-baseline",
      "all",
      "anchor-name",
      "animation",
      "animation-composition",
      "animation-delay",
      "animation-direction",
      "animation-duration",
      "animation-fill-mode",
      "animation-iteration-count",
      "animation-name",
      "animation-play-state",
      "animation-range",
      "animation-range-end",
      "animation-range-start",
      "animation-timeline",
      "animation-timing-function",
      "appearance",
      "aspect-ratio",
      "backdrop-filter",
      "backface-visibility",
      "background",
      "background-attachment",
      "background-blend-mode",
      "background-clip",
      "background-color",
      "background-image",
      "background-origin",
      "background-position",
      "background-position-x",
      "background-position-y",
      "background-repeat",
      "background-size",
      "baseline-shift",
      "block-size",
      "border",
      "border-block",
      "border-block-color",
      "border-block-end",
      "border-block-end-color",
      "border-block-end-style",
      "border-block-end-width",
      "border-block-start",
      "border-block-start-color",
      "border-block-start-style",
      "border-block-start-width",
      "border-block-style",
      "border-block-width",
      "border-bottom",
      "border-bottom-color",
      "border-bottom-left-radius",
      "border-bottom-right-radius",
      "border-bottom-style",
      "border-bottom-width",
      "border-collapse",
      "border-color",
      "border-end-end-radius",
      "border-end-start-radius",
      "border-image",
      "border-image-outset",
      "border-image-repeat",
      "border-image-slice",
      "border-image-source",
      "border-image-width",
      "border-inline",
      "border-inline-color",
      "border-inline-end",
      "border-inline-end-color",
      "border-inline-end-style",
      "border-inline-end-width",
      "border-inline-start",
      "border-inline-start-color",
      "border-inline-start-style",
      "border-inline-start-width",
      "border-inline-style",
      "border-inline-width",
      "border-left",
      "border-left-color",
      "border-left-style",
      "border-left-width",
      "border-radius",
      "border-right",
      "border-right-color",
      "border-right-style",
      "border-right-width",
      "border-spacing",
      "border-start-end-radius",
      "border-start-start-radius",
      "border-style",
      "border-top",
      "border-top-color",
      "border-top-left-radius",
      "border-top-right-radius",
      "border-top-style",
      "border-top-width",
      "border-width",
      "bottom",
      "box-align",
      "box-decoration-break",
      "box-direction",
      "box-flex",
      "box-flex-group",
      "box-lines",
      "box-ordinal-group",
      "box-orient",
      "box-pack",
      "box-shadow",
      "box-sizing",
      "break-after",
      "break-before",
      "break-inside",
      "caption-side",
      "caret-color",
      "clear",
      "clip",
      "clip-path",
      "clip-rule",
      "color",
      "color-interpolation",
      "color-interpolation-filters",
      "color-profile",
      "color-rendering",
      "color-scheme",
      "column-count",
      "column-fill",
      "column-gap",
      "column-rule",
      "column-rule-color",
      "column-rule-style",
      "column-rule-width",
      "column-span",
      "column-width",
      "columns",
      "contain",
      "contain-intrinsic-block-size",
      "contain-intrinsic-height",
      "contain-intrinsic-inline-size",
      "contain-intrinsic-size",
      "contain-intrinsic-width",
      "container",
      "container-name",
      "container-type",
      "content",
      "content-visibility",
      "counter-increment",
      "counter-reset",
      "counter-set",
      "cue",
      "cue-after",
      "cue-before",
      "cursor",
      "cx",
      "cy",
      "direction",
      "display",
      "dominant-baseline",
      "empty-cells",
      "enable-background",
      "field-sizing",
      "fill",
      "fill-opacity",
      "fill-rule",
      "filter",
      "flex",
      "flex-basis",
      "flex-direction",
      "flex-flow",
      "flex-grow",
      "flex-shrink",
      "flex-wrap",
      "float",
      "flood-color",
      "flood-opacity",
      "flow",
      "font",
      "font-display",
      "font-family",
      "font-feature-settings",
      "font-kerning",
      "font-language-override",
      "font-optical-sizing",
      "font-palette",
      "font-size",
      "font-size-adjust",
      "font-smooth",
      "font-smoothing",
      "font-stretch",
      "font-style",
      "font-synthesis",
      "font-synthesis-position",
      "font-synthesis-small-caps",
      "font-synthesis-style",
      "font-synthesis-weight",
      "font-variant",
      "font-variant-alternates",
      "font-variant-caps",
      "font-variant-east-asian",
      "font-variant-emoji",
      "font-variant-ligatures",
      "font-variant-numeric",
      "font-variant-position",
      "font-variation-settings",
      "font-weight",
      "forced-color-adjust",
      "gap",
      "glyph-orientation-horizontal",
      "glyph-orientation-vertical",
      "grid",
      "grid-area",
      "grid-auto-columns",
      "grid-auto-flow",
      "grid-auto-rows",
      "grid-column",
      "grid-column-end",
      "grid-column-start",
      "grid-gap",
      "grid-row",
      "grid-row-end",
      "grid-row-start",
      "grid-template",
      "grid-template-areas",
      "grid-template-columns",
      "grid-template-rows",
      "hanging-punctuation",
      "height",
      "hyphenate-character",
      "hyphenate-limit-chars",
      "hyphens",
      "icon",
      "image-orientation",
      "image-rendering",
      "image-resolution",
      "ime-mode",
      "initial-letter",
      "initial-letter-align",
      "inline-size",
      "inset",
      "inset-area",
      "inset-block",
      "inset-block-end",
      "inset-block-start",
      "inset-inline",
      "inset-inline-end",
      "inset-inline-start",
      "isolation",
      "justify-content",
      "justify-items",
      "justify-self",
      "kerning",
      "left",
      "letter-spacing",
      "lighting-color",
      "line-break",
      "line-height",
      "line-height-step",
      "list-style",
      "list-style-image",
      "list-style-position",
      "list-style-type",
      "margin",
      "margin-block",
      "margin-block-end",
      "margin-block-start",
      "margin-bottom",
      "margin-inline",
      "margin-inline-end",
      "margin-inline-start",
      "margin-left",
      "margin-right",
      "margin-top",
      "margin-trim",
      "marker",
      "marker-end",
      "marker-mid",
      "marker-start",
      "marks",
      "mask",
      "mask-border",
      "mask-border-mode",
      "mask-border-outset",
      "mask-border-repeat",
      "mask-border-slice",
      "mask-border-source",
      "mask-border-width",
      "mask-clip",
      "mask-composite",
      "mask-image",
      "mask-mode",
      "mask-origin",
      "mask-position",
      "mask-repeat",
      "mask-size",
      "mask-type",
      "masonry-auto-flow",
      "math-depth",
      "math-shift",
      "math-style",
      "max-block-size",
      "max-height",
      "max-inline-size",
      "max-width",
      "min-block-size",
      "min-height",
      "min-inline-size",
      "min-width",
      "mix-blend-mode",
      "nav-down",
      "nav-index",
      "nav-left",
      "nav-right",
      "nav-up",
      "none",
      "normal",
      "object-fit",
      "object-position",
      "offset",
      "offset-anchor",
      "offset-distance",
      "offset-path",
      "offset-position",
      "offset-rotate",
      "opacity",
      "order",
      "orphans",
      "outline",
      "outline-color",
      "outline-offset",
      "outline-style",
      "outline-width",
      "overflow",
      "overflow-anchor",
      "overflow-block",
      "overflow-clip-margin",
      "overflow-inline",
      "overflow-wrap",
      "overflow-x",
      "overflow-y",
      "overlay",
      "overscroll-behavior",
      "overscroll-behavior-block",
      "overscroll-behavior-inline",
      "overscroll-behavior-x",
      "overscroll-behavior-y",
      "padding",
      "padding-block",
      "padding-block-end",
      "padding-block-start",
      "padding-bottom",
      "padding-inline",
      "padding-inline-end",
      "padding-inline-start",
      "padding-left",
      "padding-right",
      "padding-top",
      "page",
      "page-break-after",
      "page-break-before",
      "page-break-inside",
      "paint-order",
      "pause",
      "pause-after",
      "pause-before",
      "perspective",
      "perspective-origin",
      "place-content",
      "place-items",
      "place-self",
      "pointer-events",
      "position",
      "position-anchor",
      "position-visibility",
      "print-color-adjust",
      "quotes",
      "r",
      "resize",
      "rest",
      "rest-after",
      "rest-before",
      "right",
      "rotate",
      "row-gap",
      "ruby-align",
      "ruby-position",
      "scale",
      "scroll-behavior",
      "scroll-margin",
      "scroll-margin-block",
      "scroll-margin-block-end",
      "scroll-margin-block-start",
      "scroll-margin-bottom",
      "scroll-margin-inline",
      "scroll-margin-inline-end",
      "scroll-margin-inline-start",
      "scroll-margin-left",
      "scroll-margin-right",
      "scroll-margin-top",
      "scroll-padding",
      "scroll-padding-block",
      "scroll-padding-block-end",
      "scroll-padding-block-start",
      "scroll-padding-bottom",
      "scroll-padding-inline",
      "scroll-padding-inline-end",
      "scroll-padding-inline-start",
      "scroll-padding-left",
      "scroll-padding-right",
      "scroll-padding-top",
      "scroll-snap-align",
      "scroll-snap-stop",
      "scroll-snap-type",
      "scroll-timeline",
      "scroll-timeline-axis",
      "scroll-timeline-name",
      "scrollbar-color",
      "scrollbar-gutter",
      "scrollbar-width",
      "shape-image-threshold",
      "shape-margin",
      "shape-outside",
      "shape-rendering",
      "speak",
      "speak-as",
      "src",
      // @font-face
      "stop-color",
      "stop-opacity",
      "stroke",
      "stroke-dasharray",
      "stroke-dashoffset",
      "stroke-linecap",
      "stroke-linejoin",
      "stroke-miterlimit",
      "stroke-opacity",
      "stroke-width",
      "tab-size",
      "table-layout",
      "text-align",
      "text-align-all",
      "text-align-last",
      "text-anchor",
      "text-combine-upright",
      "text-decoration",
      "text-decoration-color",
      "text-decoration-line",
      "text-decoration-skip",
      "text-decoration-skip-ink",
      "text-decoration-style",
      "text-decoration-thickness",
      "text-emphasis",
      "text-emphasis-color",
      "text-emphasis-position",
      "text-emphasis-style",
      "text-indent",
      "text-justify",
      "text-orientation",
      "text-overflow",
      "text-rendering",
      "text-shadow",
      "text-size-adjust",
      "text-transform",
      "text-underline-offset",
      "text-underline-position",
      "text-wrap",
      "text-wrap-mode",
      "text-wrap-style",
      "timeline-scope",
      "top",
      "touch-action",
      "transform",
      "transform-box",
      "transform-origin",
      "transform-style",
      "transition",
      "transition-behavior",
      "transition-delay",
      "transition-duration",
      "transition-property",
      "transition-timing-function",
      "translate",
      "unicode-bidi",
      "user-modify",
      "user-select",
      "vector-effect",
      "vertical-align",
      "view-timeline",
      "view-timeline-axis",
      "view-timeline-inset",
      "view-timeline-name",
      "view-transition-name",
      "visibility",
      "voice-balance",
      "voice-duration",
      "voice-family",
      "voice-pitch",
      "voice-range",
      "voice-rate",
      "voice-stress",
      "voice-volume",
      "white-space",
      "white-space-collapse",
      "widows",
      "width",
      "will-change",
      "word-break",
      "word-spacing",
      "word-wrap",
      "writing-mode",
      "x",
      "y",
      "z-index",
      "zoom"
    ].sort().reverse();
    function te(ne) {
      const ee = O(ne), ae = J, oe = G, se = "@[a-z-]+", ce = "and or not only", ue = {
        className: "variable",
        begin: "(\\$" + "[a-zA-Z-][a-zA-Z0-9_-]*" + ")\\b",
        relevance: 0
      };
      return {
        name: "SCSS",
        case_insensitive: !0,
        illegal: "[=/|']",
        contains: [
          ne.C_LINE_COMMENT_MODE,
          ne.C_BLOCK_COMMENT_MODE,
          // to recognize keyframe 40% etc which are outside the scope of our
          // attribute value mode
          ee.CSS_NUMBER_MODE,
          {
            className: "selector-id",
            begin: "#[A-Za-z0-9_-]+",
            relevance: 0
          },
          {
            className: "selector-class",
            begin: "\\.[A-Za-z0-9_-]+",
            relevance: 0
          },
          ee.ATTRIBUTE_SELECTOR_MODE,
          {
            className: "selector-tag",
            begin: "\\b(" + V.join("|") + ")\\b",
            // was there, before, but why?
            relevance: 0
          },
          {
            className: "selector-pseudo",
            begin: ":(" + oe.join("|") + ")"
          },
          {
            className: "selector-pseudo",
            begin: ":(:)?(" + ae.join("|") + ")"
          },
          ue,
          {
            // pseudo-selector params
            begin: /\(/,
            end: /\)/,
            contains: [ee.CSS_NUMBER_MODE]
          },
          ee.CSS_VARIABLE,
          {
            className: "attribute",
            begin: "\\b(" + Z.join("|") + ")\\b"
          },
          { begin: "\\b(whitespace|wait|w-resize|visible|vertical-text|vertical-ideographic|uppercase|upper-roman|upper-alpha|underline|transparent|top|thin|thick|text|text-top|text-bottom|tb-rl|table-header-group|table-footer-group|sw-resize|super|strict|static|square|solid|small-caps|separate|se-resize|scroll|s-resize|rtl|row-resize|ridge|right|repeat|repeat-y|repeat-x|relative|progress|pointer|overline|outside|outset|oblique|nowrap|not-allowed|normal|none|nw-resize|no-repeat|no-drop|newspaper|ne-resize|n-resize|move|middle|medium|ltr|lr-tb|lowercase|lower-roman|lower-alpha|loose|list-item|line|line-through|line-edge|lighter|left|keep-all|justify|italic|inter-word|inter-ideograph|inside|inset|inline|inline-block|inherit|inactive|ideograph-space|ideograph-parenthesis|ideograph-numeric|ideograph-alpha|horizontal|hidden|help|hand|groove|fixed|ellipsis|e-resize|double|dotted|distribute|distribute-space|distribute-letter|distribute-all-lines|disc|disabled|default|decimal|dashed|crosshair|collapse|col-resize|circle|char|center|capitalize|break-word|break-all|bottom|both|bolder|bold|block|bidi-override|below|baseline|auto|always|all-scroll|absolute|table|table-cell)\\b" },
          {
            begin: /:/,
            end: /[;}{]/,
            relevance: 0,
            contains: [
              ee.BLOCK_COMMENT,
              ue,
              ee.HEXCOLOR,
              ee.CSS_NUMBER_MODE,
              ne.QUOTE_STRING_MODE,
              ne.APOS_STRING_MODE,
              ee.IMPORTANT,
              ee.FUNCTION_DISPATCH
            ]
          },
          // matching these here allows us to treat them more like regular CSS
          // rules so everything between the {} gets regular rule highlighting,
          // which is what we want for page and font-face
          {
            begin: "@(page|font-face)",
            keywords: {
              $pattern: se,
              keyword: "@page @font-face"
            }
          },
          {
            begin: "@",
            end: "[{;]",
            returnBegin: !0,
            keywords: {
              $pattern: /[a-z-]+/,
              keyword: ce,
              attribute: q.join(" ")
            },
            contains: [
              {
                begin: se,
                className: "keyword"
              },
              {
                begin: /[a-z-]+(?=:)/,
                className: "attribute"
              },
              ue,
              ne.QUOTE_STRING_MODE,
              ne.APOS_STRING_MODE,
              ee.HEXCOLOR,
              ee.CSS_NUMBER_MODE
            ]
          },
          ee.FUNCTION_DISPATCH
        ]
      };
    }
    return scss_1 = te, scss_1;
  }
  var shell_1, hasRequiredShell;
  function requireShell() {
    if (hasRequiredShell) return shell_1;
    hasRequiredShell = 1;
    function O(_) {
      return {
        name: "Shell Session",
        aliases: [
          "console",
          "shellsession"
        ],
        contains: [
          {
            className: "meta.prompt",
            // We cannot add \s (spaces) in the regular expression otherwise it will be too broad and produce unexpected result.
            // For instance, in the following example, it would match "echo /path/to/home >" as a prompt:
            // echo /path/to/home > t.exe
            begin: /^\s{0,3}[/~\w\d[\]()@-]*[>%$#][ ]?/,
            starts: {
              end: /[^\\](?=\s*$)/,
              subLanguage: "bash"
            }
          }
        ]
      };
    }
    return shell_1 = O, shell_1;
  }
  var sql_1, hasRequiredSql;
  function requireSql() {
    if (hasRequiredSql) return sql_1;
    hasRequiredSql = 1;
    function O(_) {
      const H = _.regex, V = _.COMMENT("--", "$"), q = {
        scope: "string",
        variants: [
          {
            begin: /'/,
            end: /'/,
            contains: [{ match: /''/ }]
          }
        ]
      }, G = {
        begin: /"/,
        end: /"/,
        contains: [{ match: /""/ }]
      }, J = [
        "true",
        "false",
        // Not sure it's correct to call NULL literal, and clauses like IS [NOT] NULL look strange that way.
        // "null",
        "unknown"
      ], Z = [
        "double precision",
        "large object",
        "with timezone",
        "without timezone"
      ], te = [
        "bigint",
        "binary",
        "blob",
        "boolean",
        "char",
        "character",
        "clob",
        "date",
        "dec",
        "decfloat",
        "decimal",
        "float",
        "int",
        "integer",
        "interval",
        "nchar",
        "nclob",
        "national",
        "numeric",
        "real",
        "row",
        "smallint",
        "time",
        "timestamp",
        "varchar",
        "varying",
        // modifier (character varying)
        "varbinary"
      ], ne = [
        "add",
        "asc",
        "collation",
        "desc",
        "final",
        "first",
        "last",
        "view"
      ], ee = [
        "abs",
        "acos",
        "all",
        "allocate",
        "alter",
        "and",
        "any",
        "are",
        "array",
        "array_agg",
        "array_max_cardinality",
        "as",
        "asensitive",
        "asin",
        "asymmetric",
        "at",
        "atan",
        "atomic",
        "authorization",
        "avg",
        "begin",
        "begin_frame",
        "begin_partition",
        "between",
        "bigint",
        "binary",
        "blob",
        "boolean",
        "both",
        "by",
        "call",
        "called",
        "cardinality",
        "cascaded",
        "case",
        "cast",
        "ceil",
        "ceiling",
        "char",
        "char_length",
        "character",
        "character_length",
        "check",
        "classifier",
        "clob",
        "close",
        "coalesce",
        "collate",
        "collect",
        "column",
        "commit",
        "condition",
        "connect",
        "constraint",
        "contains",
        "convert",
        "copy",
        "corr",
        "corresponding",
        "cos",
        "cosh",
        "count",
        "covar_pop",
        "covar_samp",
        "create",
        "cross",
        "cube",
        "cume_dist",
        "current",
        "current_catalog",
        "current_date",
        "current_default_transform_group",
        "current_path",
        "current_role",
        "current_row",
        "current_schema",
        "current_time",
        "current_timestamp",
        "current_path",
        "current_role",
        "current_transform_group_for_type",
        "current_user",
        "cursor",
        "cycle",
        "date",
        "day",
        "deallocate",
        "dec",
        "decimal",
        "decfloat",
        "declare",
        "default",
        "define",
        "delete",
        "dense_rank",
        "deref",
        "describe",
        "deterministic",
        "disconnect",
        "distinct",
        "double",
        "drop",
        "dynamic",
        "each",
        "element",
        "else",
        "empty",
        "end",
        "end_frame",
        "end_partition",
        "end-exec",
        "equals",
        "escape",
        "every",
        "except",
        "exec",
        "execute",
        "exists",
        "exp",
        "external",
        "extract",
        "false",
        "fetch",
        "filter",
        "first_value",
        "float",
        "floor",
        "for",
        "foreign",
        "frame_row",
        "free",
        "from",
        "full",
        "function",
        "fusion",
        "get",
        "global",
        "grant",
        "group",
        "grouping",
        "groups",
        "having",
        "hold",
        "hour",
        "identity",
        "in",
        "indicator",
        "initial",
        "inner",
        "inout",
        "insensitive",
        "insert",
        "int",
        "integer",
        "intersect",
        "intersection",
        "interval",
        "into",
        "is",
        "join",
        "json_array",
        "json_arrayagg",
        "json_exists",
        "json_object",
        "json_objectagg",
        "json_query",
        "json_table",
        "json_table_primitive",
        "json_value",
        "lag",
        "language",
        "large",
        "last_value",
        "lateral",
        "lead",
        "leading",
        "left",
        "like",
        "like_regex",
        "listagg",
        "ln",
        "local",
        "localtime",
        "localtimestamp",
        "log",
        "log10",
        "lower",
        "match",
        "match_number",
        "match_recognize",
        "matches",
        "max",
        "member",
        "merge",
        "method",
        "min",
        "minute",
        "mod",
        "modifies",
        "module",
        "month",
        "multiset",
        "national",
        "natural",
        "nchar",
        "nclob",
        "new",
        "no",
        "none",
        "normalize",
        "not",
        "nth_value",
        "ntile",
        "null",
        "nullif",
        "numeric",
        "octet_length",
        "occurrences_regex",
        "of",
        "offset",
        "old",
        "omit",
        "on",
        "one",
        "only",
        "open",
        "or",
        "order",
        "out",
        "outer",
        "over",
        "overlaps",
        "overlay",
        "parameter",
        "partition",
        "pattern",
        "per",
        "percent",
        "percent_rank",
        "percentile_cont",
        "percentile_disc",
        "period",
        "portion",
        "position",
        "position_regex",
        "power",
        "precedes",
        "precision",
        "prepare",
        "primary",
        "procedure",
        "ptf",
        "range",
        "rank",
        "reads",
        "real",
        "recursive",
        "ref",
        "references",
        "referencing",
        "regr_avgx",
        "regr_avgy",
        "regr_count",
        "regr_intercept",
        "regr_r2",
        "regr_slope",
        "regr_sxx",
        "regr_sxy",
        "regr_syy",
        "release",
        "result",
        "return",
        "returns",
        "revoke",
        "right",
        "rollback",
        "rollup",
        "row",
        "row_number",
        "rows",
        "running",
        "savepoint",
        "scope",
        "scroll",
        "search",
        "second",
        "seek",
        "select",
        "sensitive",
        "session_user",
        "set",
        "show",
        "similar",
        "sin",
        "sinh",
        "skip",
        "smallint",
        "some",
        "specific",
        "specifictype",
        "sql",
        "sqlexception",
        "sqlstate",
        "sqlwarning",
        "sqrt",
        "start",
        "static",
        "stddev_pop",
        "stddev_samp",
        "submultiset",
        "subset",
        "substring",
        "substring_regex",
        "succeeds",
        "sum",
        "symmetric",
        "system",
        "system_time",
        "system_user",
        "table",
        "tablesample",
        "tan",
        "tanh",
        "then",
        "time",
        "timestamp",
        "timezone_hour",
        "timezone_minute",
        "to",
        "trailing",
        "translate",
        "translate_regex",
        "translation",
        "treat",
        "trigger",
        "trim",
        "trim_array",
        "true",
        "truncate",
        "uescape",
        "union",
        "unique",
        "unknown",
        "unnest",
        "update",
        "upper",
        "user",
        "using",
        "value",
        "values",
        "value_of",
        "var_pop",
        "var_samp",
        "varbinary",
        "varchar",
        "varying",
        "versioning",
        "when",
        "whenever",
        "where",
        "width_bucket",
        "window",
        "with",
        "within",
        "without",
        "year"
      ], ae = [
        "abs",
        "acos",
        "array_agg",
        "asin",
        "atan",
        "avg",
        "cast",
        "ceil",
        "ceiling",
        "coalesce",
        "corr",
        "cos",
        "cosh",
        "count",
        "covar_pop",
        "covar_samp",
        "cume_dist",
        "dense_rank",
        "deref",
        "element",
        "exp",
        "extract",
        "first_value",
        "floor",
        "json_array",
        "json_arrayagg",
        "json_exists",
        "json_object",
        "json_objectagg",
        "json_query",
        "json_table",
        "json_table_primitive",
        "json_value",
        "lag",
        "last_value",
        "lead",
        "listagg",
        "ln",
        "log",
        "log10",
        "lower",
        "max",
        "min",
        "mod",
        "nth_value",
        "ntile",
        "nullif",
        "percent_rank",
        "percentile_cont",
        "percentile_disc",
        "position",
        "position_regex",
        "power",
        "rank",
        "regr_avgx",
        "regr_avgy",
        "regr_count",
        "regr_intercept",
        "regr_r2",
        "regr_slope",
        "regr_sxx",
        "regr_sxy",
        "regr_syy",
        "row_number",
        "sin",
        "sinh",
        "sqrt",
        "stddev_pop",
        "stddev_samp",
        "substring",
        "substring_regex",
        "sum",
        "tan",
        "tanh",
        "translate",
        "translate_regex",
        "treat",
        "trim",
        "trim_array",
        "unnest",
        "upper",
        "value_of",
        "var_pop",
        "var_samp",
        "width_bucket"
      ], oe = [
        "current_catalog",
        "current_date",
        "current_default_transform_group",
        "current_path",
        "current_role",
        "current_schema",
        "current_transform_group_for_type",
        "current_user",
        "session_user",
        "system_time",
        "system_user",
        "current_time",
        "localtime",
        "current_timestamp",
        "localtimestamp"
      ], se = [
        "create table",
        "insert into",
        "primary key",
        "foreign key",
        "not null",
        "alter table",
        "add constraint",
        "grouping sets",
        "on overflow",
        "character set",
        "respect nulls",
        "ignore nulls",
        "nulls first",
        "nulls last",
        "depth first",
        "breadth first"
      ], ce = ae, le = [
        ...ee,
        ...ne
      ].filter((ye) => !ae.includes(ye)), ue = {
        scope: "variable",
        match: /@[a-z0-9][a-z0-9_]*/
      }, de = {
        scope: "operator",
        match: /[-+*/=%^~]|&&?|\|\|?|!=?|<(?:=>?|<|>)?|>[>=]?/,
        relevance: 0
      }, fe = {
        match: H.concat(/\b/, H.either(...ce), /\s*\(/),
        relevance: 0,
        keywords: { built_in: ce }
      };
      function he(ye) {
        return H.concat(
          /\b/,
          H.either(...ye.map((ge) => ge.replace(/\s+/, "\\s+"))),
          /\b/
        );
      }
      const me = {
        scope: "keyword",
        match: he(se),
        relevance: 0
      };
      function pe(ye, {
        exceptions: ge,
        when: ve
      } = {}) {
        const be = ve;
        return ge = ge || [], ye.map((Ce) => Ce.match(/\|\d+$/) || ge.includes(Ce) ? Ce : be(Ce) ? `${Ce}|0` : Ce);
      }
      return {
        name: "SQL",
        case_insensitive: !0,
        // does not include {} or HTML tags `</`
        illegal: /[{}]|<\//,
        keywords: {
          $pattern: /\b[\w\.]+/,
          keyword: pe(le, { when: (ye) => ye.length < 3 }),
          literal: J,
          type: te,
          built_in: oe
        },
        contains: [
          {
            scope: "type",
            match: he(Z)
          },
          me,
          fe,
          ue,
          q,
          G,
          _.C_NUMBER_MODE,
          _.C_BLOCK_COMMENT_MODE,
          V,
          de
        ]
      };
    }
    return sql_1 = O, sql_1;
  }
  var swift_1, hasRequiredSwift;
  function requireSwift() {
    if (hasRequiredSwift) return swift_1;
    hasRequiredSwift = 1;
    function O(ve) {
      return ve ? typeof ve == "string" ? ve : ve.source : null;
    }
    function _(ve) {
      return H("(?=", ve, ")");
    }
    function H(...ve) {
      return ve.map((Ce) => O(Ce)).join("");
    }
    function V(ve) {
      const be = ve[ve.length - 1];
      return typeof be == "object" && be.constructor === Object ? (ve.splice(ve.length - 1, 1), be) : {};
    }
    function q(...ve) {
      return "(" + (V(ve).capture ? "" : "?:") + ve.map((Se) => O(Se)).join("|") + ")";
    }
    const G = (ve) => H(
      /\b/,
      ve,
      /\w$/.test(ve) ? /\b/ : /\B/
    ), J = [
      "Protocol",
      // contextual
      "Type"
      // contextual
    ].map(G), Z = [
      "init",
      "self"
    ].map(G), te = [
      "Any",
      "Self"
    ], ne = [
      // strings below will be fed into the regular `keywords` engine while regex
      // will result in additional modes being created to scan for those keywords to
      // avoid conflicts with other rules
      "actor",
      "any",
      // contextual
      "associatedtype",
      "async",
      "await",
      /as\?/,
      // operator
      /as!/,
      // operator
      "as",
      // operator
      "borrowing",
      // contextual
      "break",
      "case",
      "catch",
      "class",
      "consume",
      // contextual
      "consuming",
      // contextual
      "continue",
      "convenience",
      // contextual
      "copy",
      // contextual
      "default",
      "defer",
      "deinit",
      "didSet",
      // contextual
      "distributed",
      "do",
      "dynamic",
      // contextual
      "each",
      "else",
      "enum",
      "extension",
      "fallthrough",
      /fileprivate\(set\)/,
      "fileprivate",
      "final",
      // contextual
      "for",
      "func",
      "get",
      // contextual
      "guard",
      "if",
      "import",
      "indirect",
      // contextual
      "infix",
      // contextual
      /init\?/,
      /init!/,
      "inout",
      /internal\(set\)/,
      "internal",
      "in",
      "is",
      // operator
      "isolated",
      // contextual
      "nonisolated",
      // contextual
      "lazy",
      // contextual
      "let",
      "macro",
      "mutating",
      // contextual
      "nonmutating",
      // contextual
      /open\(set\)/,
      // contextual
      "open",
      // contextual
      "operator",
      "optional",
      // contextual
      "override",
      // contextual
      "package",
      "postfix",
      // contextual
      "precedencegroup",
      "prefix",
      // contextual
      /private\(set\)/,
      "private",
      "protocol",
      /public\(set\)/,
      "public",
      "repeat",
      "required",
      // contextual
      "rethrows",
      "return",
      "set",
      // contextual
      "some",
      // contextual
      "static",
      "struct",
      "subscript",
      "super",
      "switch",
      "throws",
      "throw",
      /try\?/,
      // operator
      /try!/,
      // operator
      "try",
      // operator
      "typealias",
      /unowned\(safe\)/,
      // contextual
      /unowned\(unsafe\)/,
      // contextual
      "unowned",
      // contextual
      "var",
      "weak",
      // contextual
      "where",
      "while",
      "willSet"
      // contextual
    ], ee = [
      "false",
      "nil",
      "true"
    ], ae = [
      "assignment",
      "associativity",
      "higherThan",
      "left",
      "lowerThan",
      "none",
      "right"
    ], oe = [
      "#colorLiteral",
      "#column",
      "#dsohandle",
      "#else",
      "#elseif",
      "#endif",
      "#error",
      "#file",
      "#fileID",
      "#fileLiteral",
      "#filePath",
      "#function",
      "#if",
      "#imageLiteral",
      "#keyPath",
      "#line",
      "#selector",
      "#sourceLocation",
      "#warning"
    ], se = [
      "abs",
      "all",
      "any",
      "assert",
      "assertionFailure",
      "debugPrint",
      "dump",
      "fatalError",
      "getVaList",
      "isKnownUniquelyReferenced",
      "max",
      "min",
      "numericCast",
      "pointwiseMax",
      "pointwiseMin",
      "precondition",
      "preconditionFailure",
      "print",
      "readLine",
      "repeatElement",
      "sequence",
      "stride",
      "swap",
      "swift_unboxFromSwiftValueWithType",
      "transcode",
      "type",
      "unsafeBitCast",
      "unsafeDowncast",
      "withExtendedLifetime",
      "withUnsafeMutablePointer",
      "withUnsafePointer",
      "withVaList",
      "withoutActuallyEscaping",
      "zip"
    ], ce = q(
      /[/=\-+!*%<>&|^~?]/,
      /[\u00A1-\u00A7]/,
      /[\u00A9\u00AB]/,
      /[\u00AC\u00AE]/,
      /[\u00B0\u00B1]/,
      /[\u00B6\u00BB\u00BF\u00D7\u00F7]/,
      /[\u2016-\u2017]/,
      /[\u2020-\u2027]/,
      /[\u2030-\u203E]/,
      /[\u2041-\u2053]/,
      /[\u2055-\u205E]/,
      /[\u2190-\u23FF]/,
      /[\u2500-\u2775]/,
      /[\u2794-\u2BFF]/,
      /[\u2E00-\u2E7F]/,
      /[\u3001-\u3003]/,
      /[\u3008-\u3020]/,
      /[\u3030]/
    ), le = q(
      ce,
      /[\u0300-\u036F]/,
      /[\u1DC0-\u1DFF]/,
      /[\u20D0-\u20FF]/,
      /[\uFE00-\uFE0F]/,
      /[\uFE20-\uFE2F]/
      // TODO: The following characters are also allowed, but the regex isn't supported yet.
      // /[\u{E0100}-\u{E01EF}]/u
    ), ue = H(ce, le, "*"), de = q(
      /[a-zA-Z_]/,
      /[\u00A8\u00AA\u00AD\u00AF\u00B2-\u00B5\u00B7-\u00BA]/,
      /[\u00BC-\u00BE\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF]/,
      /[\u0100-\u02FF\u0370-\u167F\u1681-\u180D\u180F-\u1DBF]/,
      /[\u1E00-\u1FFF]/,
      /[\u200B-\u200D\u202A-\u202E\u203F-\u2040\u2054\u2060-\u206F]/,
      /[\u2070-\u20CF\u2100-\u218F\u2460-\u24FF\u2776-\u2793]/,
      /[\u2C00-\u2DFF\u2E80-\u2FFF]/,
      /[\u3004-\u3007\u3021-\u302F\u3031-\u303F\u3040-\uD7FF]/,
      /[\uF900-\uFD3D\uFD40-\uFDCF\uFDF0-\uFE1F\uFE30-\uFE44]/,
      /[\uFE47-\uFEFE\uFF00-\uFFFD]/
      // Should be /[\uFE47-\uFFFD]/, but we have to exclude FEFF.
      // The following characters are also allowed, but the regexes aren't supported yet.
      // /[\u{10000}-\u{1FFFD}\u{20000-\u{2FFFD}\u{30000}-\u{3FFFD}\u{40000}-\u{4FFFD}]/u,
      // /[\u{50000}-\u{5FFFD}\u{60000-\u{6FFFD}\u{70000}-\u{7FFFD}\u{80000}-\u{8FFFD}]/u,
      // /[\u{90000}-\u{9FFFD}\u{A0000-\u{AFFFD}\u{B0000}-\u{BFFFD}\u{C0000}-\u{CFFFD}]/u,
      // /[\u{D0000}-\u{DFFFD}\u{E0000-\u{EFFFD}]/u
    ), fe = q(
      de,
      /\d/,
      /[\u0300-\u036F\u1DC0-\u1DFF\u20D0-\u20FF\uFE20-\uFE2F]/
    ), he = H(de, fe, "*"), me = H(/[A-Z]/, fe, "*"), pe = [
      "attached",
      "autoclosure",
      H(/convention\(/, q("swift", "block", "c"), /\)/),
      "discardableResult",
      "dynamicCallable",
      "dynamicMemberLookup",
      "escaping",
      "freestanding",
      "frozen",
      "GKInspectable",
      "IBAction",
      "IBDesignable",
      "IBInspectable",
      "IBOutlet",
      "IBSegueAction",
      "inlinable",
      "main",
      "nonobjc",
      "NSApplicationMain",
      "NSCopying",
      "NSManaged",
      H(/objc\(/, he, /\)/),
      "objc",
      "objcMembers",
      "propertyWrapper",
      "requires_stored_property_inits",
      "resultBuilder",
      "Sendable",
      "testable",
      "UIApplicationMain",
      "unchecked",
      "unknown",
      "usableFromInline",
      "warn_unqualified_access"
    ], ye = [
      "iOS",
      "iOSApplicationExtension",
      "macOS",
      "macOSApplicationExtension",
      "macCatalyst",
      "macCatalystApplicationExtension",
      "watchOS",
      "watchOSApplicationExtension",
      "tvOS",
      "tvOSApplicationExtension",
      "swift"
    ];
    function ge(ve) {
      const be = {
        match: /\s+/,
        relevance: 0
      }, Ce = ve.COMMENT(
        "/\\*",
        "\\*/",
        { contains: ["self"] }
      ), Se = [
        ve.C_LINE_COMMENT_MODE,
        Ce
      ], we = {
        match: [
          /\./,
          q(...J, ...Z)
        ],
        className: { 2: "keyword" }
      }, Ee = {
        // Consume .keyword to prevent highlighting properties and methods as keywords.
        match: H(/\./, q(...ne)),
        relevance: 0
      }, xe = ne.filter((Wt) => typeof Wt == "string").concat(["_|0"]), Le = ne.filter((Wt) => typeof Wt != "string").concat(te).map(G), Qe = { variants: [
        {
          className: "keyword",
          match: q(...Le, ...Z)
        }
      ] }, ke = {
        $pattern: q(
          /\b\w+/,
          // regular keywords
          /#\w+/
          // number keywords
        ),
        keyword: xe.concat(oe),
        literal: ee
      }, Me = [
        we,
        Ee,
        Qe
      ], Re = {
        // Consume .built_in to prevent highlighting properties and methods.
        match: H(/\./, q(...se)),
        relevance: 0
      }, De = {
        className: "built_in",
        match: H(/\b/, q(...se), /(?=\()/)
      }, Pe = [
        Re,
        De
      ], $e = {
        // Prevent -> from being highlighting as an operator.
        match: /->/,
        relevance: 0
      }, qe = {
        className: "operator",
        relevance: 0,
        variants: [
          { match: ue },
          {
            // dot-operator: only operators that start with a dot are allowed to use dots as
            // characters (..., ...<, .*, etc). So there rule here is: a dot followed by one or more
            // characters that may also include dots.
            match: `\\.(\\.|${le})+`
          }
        ]
      }, Xe = [
        $e,
        qe
      ], Ve = "([0-9]_*)+", Ze = "([0-9a-fA-F]_*)+", Ge = {
        className: "number",
        relevance: 0,
        variants: [
          // decimal floating-point-literal (subsumes decimal-literal)
          { match: `\\b(${Ve})(\\.(${Ve}))?([eE][+-]?(${Ve}))?\\b` },
          // hexadecimal floating-point-literal (subsumes hexadecimal-literal)
          { match: `\\b0x(${Ze})(\\.(${Ze}))?([pP][+-]?(${Ve}))?\\b` },
          // octal-literal
          { match: /\b0o([0-7]_*)+\b/ },
          // binary-literal
          { match: /\b0b([01]_*)+\b/ }
        ]
      }, ze = (Wt = "") => ({
        className: "subst",
        variants: [
          { match: H(/\\/, Wt, /[0\\tnr"']/) },
          { match: H(/\\/, Wt, /u\{[0-9a-fA-F]{1,8}\}/) }
        ]
      }), We = (Wt = "") => ({
        className: "subst",
        match: H(/\\/, Wt, /[\t ]*(?:[\r\n]|\r\n)/)
      }), at = (Wt = "") => ({
        className: "subst",
        label: "interpol",
        begin: H(/\\/, Wt, /\(/),
        end: /\)/
      }), ct = (Wt = "") => ({
        begin: H(Wt, /"""/),
        end: H(/"""/, Wt),
        contains: [
          ze(Wt),
          We(Wt),
          at(Wt)
        ]
      }), et = (Wt = "") => ({
        begin: H(Wt, /"/),
        end: H(/"/, Wt),
        contains: [
          ze(Wt),
          at(Wt)
        ]
      }), dt = {
        className: "string",
        variants: [
          ct(),
          ct("#"),
          ct("##"),
          ct("###"),
          et(),
          et("#"),
          et("##"),
          et("###")
        ]
      }, Je = [
        ve.BACKSLASH_ESCAPE,
        {
          begin: /\[/,
          end: /\]/,
          relevance: 0,
          contains: [ve.BACKSLASH_ESCAPE]
        }
      ], it = {
        begin: /\/[^\s](?=[^/\n]*\/)/,
        end: /\//,
        contains: Je
      }, st = (Wt) => {
        const rn = H(Wt, /\//), vn = H(/\//, Wt);
        return {
          begin: rn,
          end: vn,
          contains: [
            ...Je,
            {
              scope: "comment",
              begin: `#(?!.*${vn})`,
              end: /$/
            }
          ]
        };
      }, gt = {
        scope: "regexp",
        variants: [
          st("###"),
          st("##"),
          st("#"),
          it
        ]
      }, bt = { match: H(/`/, he, /`/) }, yt = {
        className: "variable",
        match: /\$\d+/
      }, St = {
        className: "variable",
        match: `\\$${fe}+`
      }, Lt = [
        bt,
        yt,
        St
      ], Rt = {
        match: /(@|#(un)?)available/,
        scope: "keyword",
        starts: { contains: [
          {
            begin: /\(/,
            end: /\)/,
            keywords: ye,
            contains: [
              ...Xe,
              Ge,
              dt
            ]
          }
        ] }
      }, vt = {
        scope: "keyword",
        match: H(/@/, q(...pe), _(q(/\(/, /\s+/)))
      }, $t = {
        scope: "meta",
        match: H(/@/, he)
      }, Pt = [
        Rt,
        vt,
        $t
      ], Xt = {
        match: _(/\b[A-Z]/),
        relevance: 0,
        contains: [
          {
            // Common Apple frameworks, for relevance boost
            className: "type",
            match: H(/(AV|CA|CF|CG|CI|CL|CM|CN|CT|MK|MP|MTK|MTL|NS|SCN|SK|UI|WK|XC)/, fe, "+")
          },
          {
            // Type identifier
            className: "type",
            match: me,
            relevance: 0
          },
          {
            // Optional type
            match: /[?!]+/,
            relevance: 0
          },
          {
            // Variadic parameter
            match: /\.\.\./,
            relevance: 0
          },
          {
            // Protocol composition
            match: H(/\s+&\s+/, _(me)),
            relevance: 0
          }
        ]
      }, Nt = {
        begin: /</,
        end: />/,
        keywords: ke,
        contains: [
          ...Se,
          ...Me,
          ...Pt,
          $e,
          Xt
        ]
      };
      Xt.contains.push(Nt);
      const Gt = {
        match: H(he, /\s*:/),
        keywords: "_|0",
        relevance: 0
      }, Ft = {
        begin: /\(/,
        end: /\)/,
        relevance: 0,
        keywords: ke,
        contains: [
          "self",
          Gt,
          ...Se,
          gt,
          ...Me,
          ...Pe,
          ...Xe,
          Ge,
          dt,
          ...Lt,
          ...Pt,
          Xt
        ]
      }, Tt = {
        begin: /</,
        end: />/,
        keywords: "repeat each",
        contains: [
          ...Se,
          Xt
        ]
      }, It = {
        begin: q(
          _(H(he, /\s*:/)),
          _(H(he, /\s+/, he, /\s*:/))
        ),
        end: /:/,
        relevance: 0,
        contains: [
          {
            className: "keyword",
            match: /\b_\b/
          },
          {
            className: "params",
            match: he
          }
        ]
      }, Ct = {
        begin: /\(/,
        end: /\)/,
        keywords: ke,
        contains: [
          It,
          ...Se,
          ...Me,
          ...Xe,
          Ge,
          dt,
          ...Pt,
          Xt,
          Ft
        ],
        endsParent: !0,
        illegal: /["']/
      }, Kt = {
        match: [
          /(func|macro)/,
          /\s+/,
          q(bt.match, he, ue)
        ],
        className: {
          1: "keyword",
          3: "title.function"
        },
        contains: [
          Tt,
          Ct,
          be
        ],
        illegal: [
          /\[/,
          /%/
        ]
      }, Vt = {
        match: [
          /\b(?:subscript|init[?!]?)/,
          /\s*(?=[<(])/
        ],
        className: { 1: "keyword" },
        contains: [
          Tt,
          Ct,
          be
        ],
        illegal: /\[|%/
      }, Yt = {
        match: [
          /operator/,
          /\s+/,
          ue
        ],
        className: {
          1: "keyword",
          3: "title"
        }
      }, sn = {
        begin: [
          /precedencegroup/,
          /\s+/,
          me
        ],
        className: {
          1: "keyword",
          3: "title"
        },
        contains: [Xt],
        keywords: [
          ...ae,
          ...ee
        ],
        end: /}/
      }, hn = {
        match: [
          /class\b/,
          /\s+/,
          /func\b/,
          /\s+/,
          /\b[A-Za-z_][A-Za-z0-9_]*\b/
        ],
        scope: {
          1: "keyword",
          3: "keyword",
          5: "title.function"
        }
      }, Qt = {
        match: [
          /class\b/,
          /\s+/,
          /var\b/
        ],
        scope: {
          1: "keyword",
          3: "keyword"
        }
      }, Ut = {
        begin: [
          /(struct|protocol|class|extension|enum|actor)/,
          /\s+/,
          he,
          /\s*/
        ],
        beginScope: {
          1: "keyword",
          3: "title.class"
        },
        keywords: ke,
        contains: [
          Tt,
          ...Me,
          {
            begin: /:/,
            end: /\{/,
            keywords: ke,
            contains: [
              {
                scope: "title.class.inherited",
                match: me
              },
              ...Me
            ],
            relevance: 0
          }
        ]
      };
      for (const Wt of dt.variants) {
        const rn = Wt.contains.find((dn) => dn.label === "interpol");
        rn.keywords = ke;
        const vn = [
          ...Me,
          ...Pe,
          ...Xe,
          Ge,
          dt,
          ...Lt
        ];
        rn.contains = [
          ...vn,
          {
            begin: /\(/,
            end: /\)/,
            contains: [
              "self",
              ...vn
            ]
          }
        ];
      }
      return {
        name: "Swift",
        keywords: ke,
        contains: [
          ...Se,
          Kt,
          Vt,
          hn,
          Qt,
          Ut,
          Yt,
          sn,
          {
            beginKeywords: "import",
            end: /$/,
            contains: [...Se],
            relevance: 0
          },
          gt,
          ...Me,
          ...Pe,
          ...Xe,
          Ge,
          dt,
          ...Lt,
          ...Pt,
          Xt,
          Ft
        ]
      };
    }
    return swift_1 = ge, swift_1;
  }
  var yaml_1, hasRequiredYaml;
  function requireYaml() {
    if (hasRequiredYaml) return yaml_1;
    hasRequiredYaml = 1;
    function O(_) {
      const H = "true false yes no null", V = "[\\w#;/?:@&=+$,.~*'()[\\]]+", q = {
        className: "attr",
        variants: [
          // added brackets support and special char support
          { begin: /[\w*@][\w*@ :()\./-]*:(?=[ \t]|$)/ },
          {
            // double quoted keys - with brackets and special char support
            begin: /"[\w*@][\w*@ :()\./-]*":(?=[ \t]|$)/
          },
          {
            // single quoted keys - with brackets and special char support
            begin: /'[\w*@][\w*@ :()\./-]*':(?=[ \t]|$)/
          }
        ]
      }, G = {
        className: "template-variable",
        variants: [
          {
            // jinja templates Ansible
            begin: /\{\{/,
            end: /\}\}/
          },
          {
            // Ruby i18n
            begin: /%\{/,
            end: /\}/
          }
        ]
      }, J = {
        className: "string",
        relevance: 0,
        begin: /'/,
        end: /'/,
        contains: [
          {
            match: /''/,
            scope: "char.escape",
            relevance: 0
          }
        ]
      }, Z = {
        className: "string",
        relevance: 0,
        variants: [
          {
            begin: /"/,
            end: /"/
          },
          { begin: /\S+/ }
        ],
        contains: [
          _.BACKSLASH_ESCAPE,
          G
        ]
      }, te = _.inherit(Z, { variants: [
        {
          begin: /'/,
          end: /'/,
          contains: [
            {
              begin: /''/,
              relevance: 0
            }
          ]
        },
        {
          begin: /"/,
          end: /"/
        },
        { begin: /[^\s,{}[\]]+/ }
      ] }), se = {
        className: "number",
        begin: "\\b" + "[0-9]{4}(-[0-9][0-9]){0,2}" + "([Tt \\t][0-9][0-9]?(:[0-9][0-9]){2})?" + "(\\.[0-9]*)?" + "([ \\t])*(Z|[-+][0-9][0-9]?(:[0-9][0-9])?)?" + "\\b"
      }, ce = {
        end: ",",
        endsWithParent: !0,
        excludeEnd: !0,
        keywords: H,
        relevance: 0
      }, le = {
        begin: /\{/,
        end: /\}/,
        contains: [ce],
        illegal: "\\n",
        relevance: 0
      }, ue = {
        begin: "\\[",
        end: "\\]",
        contains: [ce],
        illegal: "\\n",
        relevance: 0
      }, de = [
        q,
        {
          className: "meta",
          begin: "^---\\s*$",
          relevance: 10
        },
        {
          // multi line string
          // Blocks start with a | or > followed by a newline
          //
          // Indentation of subsequent lines must be the same to
          // be considered part of the block
          className: "string",
          begin: "[\\|>]([1-9]?[+-])?[ ]*\\n( +)[^ ][^\\n]*\\n(\\2[^\\n]+\\n?)*"
        },
        {
          // Ruby/Rails erb
          begin: "<%[%=-]?",
          end: "[%-]?%>",
          subLanguage: "ruby",
          excludeBegin: !0,
          excludeEnd: !0,
          relevance: 0
        },
        {
          // named tags
          className: "type",
          begin: "!\\w+!" + V
        },
        // https://yaml.org/spec/1.2/spec.html#id2784064
        {
          // verbatim tags
          className: "type",
          begin: "!<" + V + ">"
        },
        {
          // primary tags
          className: "type",
          begin: "!" + V
        },
        {
          // secondary tags
          className: "type",
          begin: "!!" + V
        },
        {
          // fragment id &ref
          className: "meta",
          begin: "&" + _.UNDERSCORE_IDENT_RE + "$"
        },
        {
          // fragment reference *ref
          className: "meta",
          begin: "\\*" + _.UNDERSCORE_IDENT_RE + "$"
        },
        {
          // array listing
          className: "bullet",
          // TODO: remove |$ hack when we have proper look-ahead support
          begin: "-(?=[ ]|$)",
          relevance: 0
        },
        _.HASH_COMMENT_MODE,
        {
          beginKeywords: H,
          keywords: { literal: H }
        },
        se,
        // numbers are any valid C-style number that
        // sit isolated from other words
        {
          className: "number",
          begin: _.C_NUMBER_RE + "\\b",
          relevance: 0
        },
        le,
        ue,
        J,
        Z
      ], fe = [...de];
      return fe.pop(), fe.push(te), ce.contains = fe, {
        name: "YAML",
        case_insensitive: !0,
        aliases: ["yml"],
        contains: de
      };
    }
    return yaml_1 = O, yaml_1;
  }
  var typescript_1, hasRequiredTypescript;
  function requireTypescript() {
    if (hasRequiredTypescript) return typescript_1;
    hasRequiredTypescript = 1;
    const O = "[A-Za-z$_][0-9A-Za-z$_]*", _ = [
      "as",
      // for exports
      "in",
      "of",
      "if",
      "for",
      "while",
      "finally",
      "var",
      "new",
      "function",
      "do",
      "return",
      "void",
      "else",
      "break",
      "catch",
      "instanceof",
      "with",
      "throw",
      "case",
      "default",
      "try",
      "switch",
      "continue",
      "typeof",
      "delete",
      "let",
      "yield",
      "const",
      "class",
      // JS handles these with a special rule
      // "get",
      // "set",
      "debugger",
      "async",
      "await",
      "static",
      "import",
      "from",
      "export",
      "extends",
      // It's reached stage 3, which is "recommended for implementation":
      "using"
    ], H = [
      "true",
      "false",
      "null",
      "undefined",
      "NaN",
      "Infinity"
    ], V = [
      // Fundamental objects
      "Object",
      "Function",
      "Boolean",
      "Symbol",
      // numbers and dates
      "Math",
      "Date",
      "Number",
      "BigInt",
      // text
      "String",
      "RegExp",
      // Indexed collections
      "Array",
      "Float32Array",
      "Float64Array",
      "Int8Array",
      "Uint8Array",
      "Uint8ClampedArray",
      "Int16Array",
      "Int32Array",
      "Uint16Array",
      "Uint32Array",
      "BigInt64Array",
      "BigUint64Array",
      // Keyed collections
      "Set",
      "Map",
      "WeakSet",
      "WeakMap",
      // Structured data
      "ArrayBuffer",
      "SharedArrayBuffer",
      "Atomics",
      "DataView",
      "JSON",
      // Control abstraction objects
      "Promise",
      "Generator",
      "GeneratorFunction",
      "AsyncFunction",
      // Reflection
      "Reflect",
      "Proxy",
      // Internationalization
      "Intl",
      // WebAssembly
      "WebAssembly"
    ], q = [
      "Error",
      "EvalError",
      "InternalError",
      "RangeError",
      "ReferenceError",
      "SyntaxError",
      "TypeError",
      "URIError"
    ], G = [
      "setInterval",
      "setTimeout",
      "clearInterval",
      "clearTimeout",
      "require",
      "exports",
      "eval",
      "isFinite",
      "isNaN",
      "parseFloat",
      "parseInt",
      "decodeURI",
      "decodeURIComponent",
      "encodeURI",
      "encodeURIComponent",
      "escape",
      "unescape"
    ], J = [
      "arguments",
      "this",
      "super",
      "console",
      "window",
      "document",
      "localStorage",
      "sessionStorage",
      "module",
      "global"
      // Node.js
    ], Z = [].concat(
      G,
      V,
      q
    );
    function te(ee) {
      const ae = ee.regex, oe = (ze, { after: We }) => {
        const at = "</" + ze[0].slice(1);
        return ze.input.indexOf(at, We) !== -1;
      }, se = O, ce = {
        begin: "<>",
        end: "</>"
      }, le = /<[A-Za-z0-9\\._:-]+\s*\/>/, ue = {
        begin: /<[A-Za-z0-9\\._:-]+/,
        end: /\/[A-Za-z0-9\\._:-]+>|\/>/,
        /**
         * @param {RegExpMatchArray} match
         * @param {CallbackResponse} response
         */
        isTrulyOpeningTag: (ze, We) => {
          const at = ze[0].length + ze.index, ct = ze.input[at];
          if (
            // HTML should not include another raw `<` inside a tag
            // nested type?
            // `<Array<Array<number>>`, etc.
            ct === "<" || // the , gives away that this is not HTML
            // `<T, A extends keyof T, V>`
            ct === ","
          ) {
            We.ignoreMatch();
            return;
          }
          ct === ">" && (oe(ze, { after: at }) || We.ignoreMatch());
          let et;
          const dt = ze.input.substring(at);
          if (et = dt.match(/^\s*=/)) {
            We.ignoreMatch();
            return;
          }
          if ((et = dt.match(/^\s+extends\s+/)) && et.index === 0) {
            We.ignoreMatch();
            return;
          }
        }
      }, de = {
        $pattern: O,
        keyword: _,
        literal: H,
        built_in: Z,
        "variable.language": J
      }, fe = "[0-9](_?[0-9])*", he = `\\.(${fe})`, me = "0|[1-9](_?[0-9])*|0[0-7]*[89][0-9]*", pe = {
        className: "number",
        variants: [
          // DecimalLiteral
          { begin: `(\\b(${me})((${he})|\\.)?|(${he}))[eE][+-]?(${fe})\\b` },
          { begin: `\\b(${me})\\b((${he})\\b|\\.)?|(${he})\\b` },
          // DecimalBigIntegerLiteral
          { begin: "\\b(0|[1-9](_?[0-9])*)n\\b" },
          // NonDecimalIntegerLiteral
          { begin: "\\b0[xX][0-9a-fA-F](_?[0-9a-fA-F])*n?\\b" },
          { begin: "\\b0[bB][0-1](_?[0-1])*n?\\b" },
          { begin: "\\b0[oO][0-7](_?[0-7])*n?\\b" },
          // LegacyOctalIntegerLiteral (does not include underscore separators)
          // https://tc39.es/ecma262/#sec-additional-syntax-numeric-literals
          { begin: "\\b0[0-7]+n?\\b" }
        ],
        relevance: 0
      }, ye = {
        className: "subst",
        begin: "\\$\\{",
        end: "\\}",
        keywords: de,
        contains: []
        // defined later
      }, ge = {
        begin: ".?html`",
        end: "",
        starts: {
          end: "`",
          returnEnd: !1,
          contains: [
            ee.BACKSLASH_ESCAPE,
            ye
          ],
          subLanguage: "xml"
        }
      }, ve = {
        begin: ".?css`",
        end: "",
        starts: {
          end: "`",
          returnEnd: !1,
          contains: [
            ee.BACKSLASH_ESCAPE,
            ye
          ],
          subLanguage: "css"
        }
      }, be = {
        begin: ".?gql`",
        end: "",
        starts: {
          end: "`",
          returnEnd: !1,
          contains: [
            ee.BACKSLASH_ESCAPE,
            ye
          ],
          subLanguage: "graphql"
        }
      }, Ce = {
        className: "string",
        begin: "`",
        end: "`",
        contains: [
          ee.BACKSLASH_ESCAPE,
          ye
        ]
      }, we = {
        className: "comment",
        variants: [
          ee.COMMENT(
            /\/\*\*(?!\/)/,
            "\\*/",
            {
              relevance: 0,
              contains: [
                {
                  begin: "(?=@[A-Za-z]+)",
                  relevance: 0,
                  contains: [
                    {
                      className: "doctag",
                      begin: "@[A-Za-z]+"
                    },
                    {
                      className: "type",
                      begin: "\\{",
                      end: "\\}",
                      excludeEnd: !0,
                      excludeBegin: !0,
                      relevance: 0
                    },
                    {
                      className: "variable",
                      begin: se + "(?=\\s*(-)|$)",
                      endsParent: !0,
                      relevance: 0
                    },
                    // eat spaces (not newlines) so we can find
                    // types or variables
                    {
                      begin: /(?=[^\n])\s/,
                      relevance: 0
                    }
                  ]
                }
              ]
            }
          ),
          ee.C_BLOCK_COMMENT_MODE,
          ee.C_LINE_COMMENT_MODE
        ]
      }, Ee = [
        ee.APOS_STRING_MODE,
        ee.QUOTE_STRING_MODE,
        ge,
        ve,
        be,
        Ce,
        // Skip numbers when they are part of a variable name
        { match: /\$\d+/ },
        pe
        // This is intentional:
        // See https://github.com/highlightjs/highlight.js/issues/3288
        // hljs.REGEXP_MODE
      ];
      ye.contains = Ee.concat({
        // we need to pair up {} inside our subst to prevent
        // it from ending too early by matching another }
        begin: /\{/,
        end: /\}/,
        keywords: de,
        contains: [
          "self"
        ].concat(Ee)
      });
      const xe = [].concat(we, ye.contains), Le = xe.concat([
        // eat recursive parens in sub expressions
        {
          begin: /(\s*)\(/,
          end: /\)/,
          keywords: de,
          contains: ["self"].concat(xe)
        }
      ]), Qe = {
        className: "params",
        // convert this to negative lookbehind in v12
        begin: /(\s*)\(/,
        // to match the parms with
        end: /\)/,
        excludeBegin: !0,
        excludeEnd: !0,
        keywords: de,
        contains: Le
      }, ke = {
        variants: [
          // class Car extends vehicle
          {
            match: [
              /class/,
              /\s+/,
              se,
              /\s+/,
              /extends/,
              /\s+/,
              ae.concat(se, "(", ae.concat(/\./, se), ")*")
            ],
            scope: {
              1: "keyword",
              3: "title.class",
              5: "keyword",
              7: "title.class.inherited"
            }
          },
          // class Car
          {
            match: [
              /class/,
              /\s+/,
              se
            ],
            scope: {
              1: "keyword",
              3: "title.class"
            }
          }
        ]
      }, Me = {
        relevance: 0,
        match: ae.either(
          // Hard coded exceptions
          /\bJSON/,
          // Float32Array, OutT
          /\b[A-Z][a-z]+([A-Z][a-z]*|\d)*/,
          // CSSFactory, CSSFactoryT
          /\b[A-Z]{2,}([A-Z][a-z]+|\d)+([A-Z][a-z]*)*/,
          // FPs, FPsT
          /\b[A-Z]{2,}[a-z]+([A-Z][a-z]+|\d)*([A-Z][a-z]*)*/
          // P
          // single letters are not highlighted
          // BLAH
          // this will be flagged as a UPPER_CASE_CONSTANT instead
        ),
        className: "title.class",
        keywords: {
          _: [
            // se we still get relevance credit for JS library classes
            ...V,
            ...q
          ]
        }
      }, Re = {
        label: "use_strict",
        className: "meta",
        relevance: 10,
        begin: /^\s*['"]use (strict|asm)['"]/
      }, De = {
        variants: [
          {
            match: [
              /function/,
              /\s+/,
              se,
              /(?=\s*\()/
            ]
          },
          // anonymous function
          {
            match: [
              /function/,
              /\s*(?=\()/
            ]
          }
        ],
        className: {
          1: "keyword",
          3: "title.function"
        },
        label: "func.def",
        contains: [Qe],
        illegal: /%/
      }, Pe = {
        relevance: 0,
        match: /\b[A-Z][A-Z_0-9]+\b/,
        className: "variable.constant"
      };
      function $e(ze) {
        return ae.concat("(?!", ze.join("|"), ")");
      }
      const qe = {
        match: ae.concat(
          /\b/,
          $e([
            ...G,
            "super",
            "import"
          ].map((ze) => `${ze}\\s*\\(`)),
          se,
          ae.lookahead(/\s*\(/)
        ),
        className: "title.function",
        relevance: 0
      }, Xe = {
        begin: ae.concat(/\./, ae.lookahead(
          ae.concat(se, /(?![0-9A-Za-z$_(])/)
        )),
        end: se,
        excludeBegin: !0,
        keywords: "prototype",
        className: "property",
        relevance: 0
      }, Ve = {
        match: [
          /get|set/,
          /\s+/,
          se,
          /(?=\()/
        ],
        className: {
          1: "keyword",
          3: "title.function"
        },
        contains: [
          {
            // eat to avoid empty params
            begin: /\(\)/
          },
          Qe
        ]
      }, Ze = "(\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)|" + ee.UNDERSCORE_IDENT_RE + ")\\s*=>", Ge = {
        match: [
          /const|var|let/,
          /\s+/,
          se,
          /\s*/,
          /=\s*/,
          /(async\s*)?/,
          // async is optional
          ae.lookahead(Ze)
        ],
        keywords: "async",
        className: {
          1: "keyword",
          3: "title.function"
        },
        contains: [
          Qe
        ]
      };
      return {
        name: "JavaScript",
        aliases: ["js", "jsx", "mjs", "cjs"],
        keywords: de,
        // this will be extended by TypeScript
        exports: { PARAMS_CONTAINS: Le, CLASS_REFERENCE: Me },
        illegal: /#(?![$_A-z])/,
        contains: [
          ee.SHEBANG({
            label: "shebang",
            binary: "node",
            relevance: 5
          }),
          Re,
          ee.APOS_STRING_MODE,
          ee.QUOTE_STRING_MODE,
          ge,
          ve,
          be,
          Ce,
          we,
          // Skip numbers when they are part of a variable name
          { match: /\$\d+/ },
          pe,
          Me,
          {
            scope: "attr",
            match: se + ae.lookahead(":"),
            relevance: 0
          },
          Ge,
          {
            // "value" container
            begin: "(" + ee.RE_STARTERS_RE + "|\\b(case|return|throw)\\b)\\s*",
            keywords: "return throw case",
            relevance: 0,
            contains: [
              we,
              ee.REGEXP_MODE,
              {
                className: "function",
                // we have to count the parens to make sure we actually have the
                // correct bounding ( ) before the =>.  There could be any number of
                // sub-expressions inside also surrounded by parens.
                begin: Ze,
                returnBegin: !0,
                end: "\\s*=>",
                contains: [
                  {
                    className: "params",
                    variants: [
                      {
                        begin: ee.UNDERSCORE_IDENT_RE,
                        relevance: 0
                      },
                      {
                        className: null,
                        begin: /\(\s*\)/,
                        skip: !0
                      },
                      {
                        begin: /(\s*)\(/,
                        end: /\)/,
                        excludeBegin: !0,
                        excludeEnd: !0,
                        keywords: de,
                        contains: Le
                      }
                    ]
                  }
                ]
              },
              {
                // could be a comma delimited list of params to a function call
                begin: /,/,
                relevance: 0
              },
              {
                match: /\s+/,
                relevance: 0
              },
              {
                // JSX
                variants: [
                  { begin: ce.begin, end: ce.end },
                  { match: le },
                  {
                    begin: ue.begin,
                    // we carefully check the opening tag to see if it truly
                    // is a tag and not a false positive
                    "on:begin": ue.isTrulyOpeningTag,
                    end: ue.end
                  }
                ],
                subLanguage: "xml",
                contains: [
                  {
                    begin: ue.begin,
                    end: ue.end,
                    skip: !0,
                    contains: ["self"]
                  }
                ]
              }
            ]
          },
          De,
          {
            // prevent this from getting swallowed up by function
            // since they appear "function like"
            beginKeywords: "while if switch catch for"
          },
          {
            // we have to count the parens to make sure we actually have the correct
            // bounding ( ).  There could be any number of sub-expressions inside
            // also surrounded by parens.
            begin: "\\b(?!function)" + ee.UNDERSCORE_IDENT_RE + "\\([^()]*(\\([^()]*(\\([^()]*\\)[^()]*)*\\)[^()]*)*\\)\\s*\\{",
            // end parens
            returnBegin: !0,
            label: "func.def",
            contains: [
              Qe,
              ee.inherit(ee.TITLE_MODE, { begin: se, className: "title.function" })
            ]
          },
          // catch ... so it won't trigger the property rule below
          {
            match: /\.\.\./,
            relevance: 0
          },
          Xe,
          // hack: prevents detection of keywords in some circumstances
          // .keyword()
          // $keyword = x
          {
            match: "\\$" + se,
            relevance: 0
          },
          {
            match: [/\bconstructor(?=\s*\()/],
            className: { 1: "title.function" },
            contains: [Qe]
          },
          qe,
          Pe,
          ke,
          Ve,
          {
            match: /\$[(.]/
            // relevance booster for a pattern common to JS libs: `$(something)` and `$.something`
          }
        ]
      };
    }
    function ne(ee) {
      const ae = ee.regex, oe = te(ee), se = O, ce = [
        "any",
        "void",
        "number",
        "boolean",
        "string",
        "object",
        "never",
        "symbol",
        "bigint",
        "unknown"
      ], le = {
        begin: [
          /namespace/,
          /\s+/,
          ee.IDENT_RE
        ],
        beginScope: {
          1: "keyword",
          3: "title.class"
        }
      }, ue = {
        beginKeywords: "interface",
        end: /\{/,
        excludeEnd: !0,
        keywords: {
          keyword: "interface extends",
          built_in: ce
        },
        contains: [oe.exports.CLASS_REFERENCE]
      }, de = {
        className: "meta",
        relevance: 10,
        begin: /^\s*['"]use strict['"]/
      }, fe = [
        "type",
        // "namespace",
        "interface",
        "public",
        "private",
        "protected",
        "implements",
        "declare",
        "abstract",
        "readonly",
        "enum",
        "override",
        "satisfies"
      ], he = {
        $pattern: O,
        keyword: _.concat(fe),
        literal: H,
        built_in: Z.concat(ce),
        "variable.language": J
      }, me = {
        className: "meta",
        begin: "@" + se
      }, pe = (be, Ce, Se) => {
        const we = be.contains.findIndex((Ee) => Ee.label === Ce);
        if (we === -1)
          throw new Error("can not find mode to replace");
        be.contains.splice(we, 1, Se);
      };
      Object.assign(oe.keywords, he), oe.exports.PARAMS_CONTAINS.push(me);
      const ye = oe.contains.find((be) => be.scope === "attr"), ge = Object.assign(
        {},
        ye,
        { match: ae.concat(se, ae.lookahead(/\s*\?:/)) }
      );
      oe.exports.PARAMS_CONTAINS.push([
        oe.exports.CLASS_REFERENCE,
        // class reference for highlighting the params types
        ye,
        // highlight the params key
        ge
        // Added for optional property assignment highlighting
      ]), oe.contains = oe.contains.concat([
        me,
        le,
        ue,
        ge
        // Added for optional property assignment highlighting
      ]), pe(oe, "shebang", ee.SHEBANG()), pe(oe, "use_strict", de);
      const ve = oe.contains.find((be) => be.label === "func.def");
      return ve.relevance = 0, Object.assign(oe, {
        name: "TypeScript",
        aliases: [
          "ts",
          "tsx",
          "mts",
          "cts"
        ]
      }), oe;
    }
    return typescript_1 = ne, typescript_1;
  }
  var vbnet_1, hasRequiredVbnet;
  function requireVbnet() {
    if (hasRequiredVbnet) return vbnet_1;
    hasRequiredVbnet = 1;
    function O(_) {
      const H = _.regex, V = {
        className: "string",
        begin: /"(""|[^/n])"C\b/
      }, q = {
        className: "string",
        begin: /"/,
        end: /"/,
        illegal: /\n/,
        contains: [
          {
            // double quote escape
            begin: /""/
          }
        ]
      }, G = /\d{1,2}\/\d{1,2}\/\d{4}/, J = /\d{4}-\d{1,2}-\d{1,2}/, Z = /(\d|1[012])(:\d+){0,2} *(AM|PM)/, te = /\d{1,2}(:\d{1,2}){1,2}/, ne = {
        className: "literal",
        variants: [
          {
            // #YYYY-MM-DD# (ISO-Date) or #M/D/YYYY# (US-Date)
            begin: H.concat(/# */, H.either(J, G), / *#/)
          },
          {
            // #H:mm[:ss]# (24h Time)
            begin: H.concat(/# */, te, / *#/)
          },
          {
            // #h[:mm[:ss]] A# (12h Time)
            begin: H.concat(/# */, Z, / *#/)
          },
          {
            // date plus time
            begin: H.concat(
              /# */,
              H.either(J, G),
              / +/,
              H.either(Z, te),
              / *#/
            )
          }
        ]
      }, ee = {
        className: "number",
        relevance: 0,
        variants: [
          {
            // Float
            begin: /\b\d[\d_]*((\.[\d_]+(E[+-]?[\d_]+)?)|(E[+-]?[\d_]+))[RFD@!#]?/
          },
          {
            // Integer (base 10)
            begin: /\b\d[\d_]*((U?[SIL])|[%&])?/
          },
          {
            // Integer (base 16)
            begin: /&H[\dA-F_]+((U?[SIL])|[%&])?/
          },
          {
            // Integer (base 8)
            begin: /&O[0-7_]+((U?[SIL])|[%&])?/
          },
          {
            // Integer (base 2)
            begin: /&B[01_]+((U?[SIL])|[%&])?/
          }
        ]
      }, ae = {
        className: "label",
        begin: /^\w+:/
      }, oe = _.COMMENT(/'''/, /$/, { contains: [
        {
          className: "doctag",
          begin: /<\/?/,
          end: />/
        }
      ] }), se = _.COMMENT(null, /$/, { variants: [
        { begin: /'/ },
        {
          // TODO: Use multi-class for leading spaces
          begin: /([\t ]|^)REM(?=\s)/
        }
      ] });
      return {
        name: "Visual Basic .NET",
        aliases: ["vb"],
        case_insensitive: !0,
        classNameAliases: { label: "symbol" },
        keywords: {
          keyword: "addhandler alias aggregate ansi as async assembly auto binary by byref byval call case catch class compare const continue custom declare default delegate dim distinct do each equals else elseif end enum erase error event exit explicit finally for friend from function get global goto group handles if implements imports in inherits interface into iterator join key let lib loop me mid module mustinherit mustoverride mybase myclass namespace narrowing new next notinheritable notoverridable of off on operator option optional order overloads overridable overrides paramarray partial preserve private property protected public raiseevent readonly redim removehandler resume return select set shadows shared skip static step stop structure strict sub synclock take text then throw to try unicode until using when where while widening with withevents writeonly yield",
          built_in: (
            // Operators https://docs.microsoft.com/dotnet/visual-basic/language-reference/operators
            "addressof and andalso await directcast gettype getxmlnamespace is isfalse isnot istrue like mod nameof new not or orelse trycast typeof xor cbool cbyte cchar cdate cdbl cdec cint clng cobj csbyte cshort csng cstr cuint culng cushort"
          ),
          type: (
            // Data types https://docs.microsoft.com/dotnet/visual-basic/language-reference/data-types
            "boolean byte char date decimal double integer long object sbyte short single string uinteger ulong ushort"
          ),
          literal: "true false nothing"
        },
        illegal: "//|\\{|\\}|endif|gosub|variant|wend|^\\$ ",
        contains: [
          V,
          q,
          ne,
          ee,
          ae,
          oe,
          se,
          {
            className: "meta",
            // TODO: Use multi-class for indentation once available
            begin: /[\t ]*#(const|disable|else|elseif|enable|end|externalsource|if|region)\b/,
            end: /$/,
            keywords: { keyword: "const disable else elseif enable end externalsource if region then" },
            contains: [se]
          }
        ]
      };
    }
    return vbnet_1 = O, vbnet_1;
  }
  var wasm_1, hasRequiredWasm;
  function requireWasm() {
    if (hasRequiredWasm) return wasm_1;
    hasRequiredWasm = 1;
    function O(_) {
      _.regex;
      const H = _.COMMENT(/\(;/, /;\)/);
      H.contains.push("self");
      const V = _.COMMENT(/;;/, /$/), q = [
        "anyfunc",
        "block",
        "br",
        "br_if",
        "br_table",
        "call",
        "call_indirect",
        "data",
        "drop",
        "elem",
        "else",
        "end",
        "export",
        "func",
        "global.get",
        "global.set",
        "local.get",
        "local.set",
        "local.tee",
        "get_global",
        "get_local",
        "global",
        "if",
        "import",
        "local",
        "loop",
        "memory",
        "memory.grow",
        "memory.size",
        "module",
        "mut",
        "nop",
        "offset",
        "param",
        "result",
        "return",
        "select",
        "set_global",
        "set_local",
        "start",
        "table",
        "tee_local",
        "then",
        "type",
        "unreachable"
      ], G = {
        begin: [
          /(?:func|call|call_indirect)/,
          /\s+/,
          /\$[^\s)]+/
        ],
        className: {
          1: "keyword",
          3: "title.function"
        }
      }, J = {
        className: "variable",
        begin: /\$[\w_]+/
      }, Z = {
        match: /(\((?!;)|\))+/,
        className: "punctuation",
        relevance: 0
      }, te = {
        className: "number",
        relevance: 0,
        // borrowed from Prism, TODO: split out into variants
        match: /[+-]?\b(?:\d(?:_?\d)*(?:\.\d(?:_?\d)*)?(?:[eE][+-]?\d(?:_?\d)*)?|0x[\da-fA-F](?:_?[\da-fA-F])*(?:\.[\da-fA-F](?:_?[\da-fA-D])*)?(?:[pP][+-]?\d(?:_?\d)*)?)\b|\binf\b|\bnan(?::0x[\da-fA-F](?:_?[\da-fA-D])*)?\b/
      }, ne = {
        // look-ahead prevents us from gobbling up opcodes
        match: /(i32|i64|f32|f64)(?!\.)/,
        className: "type"
      }, ee = {
        className: "keyword",
        // borrowed from Prism, TODO: split out into variants
        match: /\b(f32|f64|i32|i64)(?:\.(?:abs|add|and|ceil|clz|const|convert_[su]\/i(?:32|64)|copysign|ctz|demote\/f64|div(?:_[su])?|eqz?|extend_[su]\/i32|floor|ge(?:_[su])?|gt(?:_[su])?|le(?:_[su])?|load(?:(?:8|16|32)_[su])?|lt(?:_[su])?|max|min|mul|nearest|neg?|or|popcnt|promote\/f32|reinterpret\/[fi](?:32|64)|rem_[su]|rot[lr]|shl|shr_[su]|store(?:8|16|32)?|sqrt|sub|trunc(?:_[su]\/f(?:32|64))?|wrap\/i64|xor))\b/
      };
      return {
        name: "WebAssembly",
        keywords: {
          $pattern: /[\w.]+/,
          keyword: q
        },
        contains: [
          V,
          H,
          {
            match: [
              /(?:offset|align)/,
              /\s*/,
              /=/
            ],
            className: {
              1: "keyword",
              3: "operator"
            }
          },
          J,
          Z,
          G,
          _.QUOTE_STRING_MODE,
          ne,
          ee,
          te
        ]
      };
    }
    return wasm_1 = O, wasm_1;
  }
  var common, hasRequiredCommon;
  function requireCommon() {
    if (hasRequiredCommon) return common;
    hasRequiredCommon = 1;
    var O = /* @__PURE__ */ requireCore();
    return O.registerLanguage("xml", /* @__PURE__ */ requireXml()), O.registerLanguage("bash", /* @__PURE__ */ requireBash()), O.registerLanguage("c", /* @__PURE__ */ requireC()), O.registerLanguage("cpp", /* @__PURE__ */ requireCpp()), O.registerLanguage("csharp", /* @__PURE__ */ requireCsharp()), O.registerLanguage("css", /* @__PURE__ */ requireCss()), O.registerLanguage("markdown", /* @__PURE__ */ requireMarkdown()), O.registerLanguage("diff", /* @__PURE__ */ requireDiff()), O.registerLanguage("ruby", /* @__PURE__ */ requireRuby()), O.registerLanguage("go", /* @__PURE__ */ requireGo()), O.registerLanguage("graphql", /* @__PURE__ */ requireGraphql()), O.registerLanguage("ini", /* @__PURE__ */ requireIni()), O.registerLanguage("java", /* @__PURE__ */ requireJava()), O.registerLanguage("javascript", /* @__PURE__ */ requireJavascript()), O.registerLanguage("json", /* @__PURE__ */ requireJson()), O.registerLanguage("kotlin", /* @__PURE__ */ requireKotlin()), O.registerLanguage("less", /* @__PURE__ */ requireLess()), O.registerLanguage("lua", /* @__PURE__ */ requireLua()), O.registerLanguage("makefile", /* @__PURE__ */ requireMakefile()), O.registerLanguage("perl", /* @__PURE__ */ requirePerl()), O.registerLanguage("objectivec", /* @__PURE__ */ requireObjectivec()), O.registerLanguage("php", /* @__PURE__ */ requirePhp()), O.registerLanguage("php-template", /* @__PURE__ */ requirePhpTemplate()), O.registerLanguage("plaintext", /* @__PURE__ */ requirePlaintext()), O.registerLanguage("python", /* @__PURE__ */ requirePython()), O.registerLanguage("python-repl", /* @__PURE__ */ requirePythonRepl()), O.registerLanguage("r", /* @__PURE__ */ requireR()), O.registerLanguage("rust", /* @__PURE__ */ requireRust()), O.registerLanguage("scss", /* @__PURE__ */ requireScss()), O.registerLanguage("shell", /* @__PURE__ */ requireShell()), O.registerLanguage("sql", /* @__PURE__ */ requireSql()), O.registerLanguage("swift", /* @__PURE__ */ requireSwift()), O.registerLanguage("yaml", /* @__PURE__ */ requireYaml()), O.registerLanguage("typescript", /* @__PURE__ */ requireTypescript()), O.registerLanguage("vbnet", /* @__PURE__ */ requireVbnet()), O.registerLanguage("wasm", /* @__PURE__ */ requireWasm()), O.HighlightJS = O, O.default = O, common = O, common;
  }
  requireCommon();
  var coreExports = /* @__PURE__ */ requireCore();
  const HighlightJS$1 = /* @__PURE__ */ getDefaultExportFromCjs$1(coreExports);
  var r$1 = /* @__PURE__ */ defineComponent({ props: { code: { type: String, required: !0 }, language: { type: String, default: "" }, autodetect: { type: Boolean, default: !0 }, ignoreIllegals: { type: Boolean, default: !0 } }, setup: function(O) {
    var _ = ref$1(O.language);
    watch$1(function() {
      return O.language;
    }, function(q) {
      _.value = q;
    });
    var H = computed(function() {
      return O.autodetect || !_.value;
    }), V = computed(function() {
      return !H.value && !HighlightJS$1.getLanguage(_.value);
    });
    return { className: computed(function() {
      return V.value ? "" : "hljs " + _.value;
    }), highlightedCode: computed(function() {
      var q;
      if (V.value) return console.warn('The language "' + _.value + '" you specified could not be found.'), O.code.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;").replace(/'/g, "&#x27;");
      if (H.value) {
        var G = HighlightJS$1.highlightAuto(O.code);
        return _.value = (q = G.language) !== null && q !== void 0 ? q : "", G.value;
      }
      return (G = HighlightJS$1.highlight(O.code, { language: _.value, ignoreIllegals: O.ignoreIllegals })).value;
    }) };
  }, render: function() {
    return h$3("pre", {}, [h$3("code", { class: this.className, innerHTML: this.highlightedCode })]);
  } }), o$1 = { install: function(O) {
    O.component("highlightjs", r$1);
  }, component: r$1 }, clipboard$1 = { exports: {} };
  /*!
   * clipboard.js v2.0.11
   * https://clipboardjs.com/
   *
   * Licensed MIT © Zeno Rocha
   */
  var clipboard = clipboard$1.exports, hasRequiredClipboard;
  function requireClipboard() {
    return hasRequiredClipboard || (hasRequiredClipboard = 1, function(O, _) {
      (function(V, q) {
        O.exports = q();
      })(clipboard, function() {
        return (
          /******/
          function() {
            var H = {
              /***/
              686: (
                /***/
                function(G, J, Z) {
                  Z.d(J, {
                    default: function() {
                      return (
                        /* binding */
                        Pe
                      );
                    }
                  });
                  var te = Z(279), ne = /* @__PURE__ */ Z.n(te), ee = Z(370), ae = /* @__PURE__ */ Z.n(ee), oe = Z(817), se = /* @__PURE__ */ Z.n(oe);
                  function ce($e) {
                    try {
                      return document.execCommand($e);
                    } catch {
                      return !1;
                    }
                  }
                  var le = function(qe) {
                    var Xe = se()(qe);
                    return ce("cut"), Xe;
                  }, ue = le;
                  function de($e) {
                    var qe = document.documentElement.getAttribute("dir") === "rtl", Xe = document.createElement("textarea");
                    Xe.style.fontSize = "12pt", Xe.style.border = "0", Xe.style.padding = "0", Xe.style.margin = "0", Xe.style.position = "absolute", Xe.style[qe ? "right" : "left"] = "-9999px";
                    var Ve = window.pageYOffset || document.documentElement.scrollTop;
                    return Xe.style.top = "".concat(Ve, "px"), Xe.setAttribute("readonly", ""), Xe.value = $e, Xe;
                  }
                  var fe = function(qe, Xe) {
                    var Ve = de(qe);
                    Xe.container.appendChild(Ve);
                    var Ze = se()(Ve);
                    return ce("copy"), Ve.remove(), Ze;
                  }, he = function(qe) {
                    var Xe = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {
                      container: document.body
                    }, Ve = "";
                    return typeof qe == "string" ? Ve = fe(qe, Xe) : qe instanceof HTMLInputElement && !["text", "search", "url", "tel", "password"].includes(qe == null ? void 0 : qe.type) ? Ve = fe(qe.value, Xe) : (Ve = se()(qe), ce("copy")), Ve;
                  }, me = he;
                  function pe($e) {
                    "@babel/helpers - typeof";
                    return typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? pe = function(Xe) {
                      return typeof Xe;
                    } : pe = function(Xe) {
                      return Xe && typeof Symbol == "function" && Xe.constructor === Symbol && Xe !== Symbol.prototype ? "symbol" : typeof Xe;
                    }, pe($e);
                  }
                  var ye = function() {
                    var qe = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : {}, Xe = qe.action, Ve = Xe === void 0 ? "copy" : Xe, Ze = qe.container, Ge = qe.target, ze = qe.text;
                    if (Ve !== "copy" && Ve !== "cut")
                      throw new Error('Invalid "action" value, use either "copy" or "cut"');
                    if (Ge !== void 0)
                      if (Ge && pe(Ge) === "object" && Ge.nodeType === 1) {
                        if (Ve === "copy" && Ge.hasAttribute("disabled"))
                          throw new Error('Invalid "target" attribute. Please use "readonly" instead of "disabled" attribute');
                        if (Ve === "cut" && (Ge.hasAttribute("readonly") || Ge.hasAttribute("disabled")))
                          throw new Error(`Invalid "target" attribute. You can't cut text from elements with "readonly" or "disabled" attributes`);
                      } else
                        throw new Error('Invalid "target" value, use a valid Element');
                    if (ze)
                      return me(ze, {
                        container: Ze
                      });
                    if (Ge)
                      return Ve === "cut" ? ue(Ge) : me(Ge, {
                        container: Ze
                      });
                  }, ge = ye;
                  function ve($e) {
                    "@babel/helpers - typeof";
                    return typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? ve = function(Xe) {
                      return typeof Xe;
                    } : ve = function(Xe) {
                      return Xe && typeof Symbol == "function" && Xe.constructor === Symbol && Xe !== Symbol.prototype ? "symbol" : typeof Xe;
                    }, ve($e);
                  }
                  function be($e, qe) {
                    if (!($e instanceof qe))
                      throw new TypeError("Cannot call a class as a function");
                  }
                  function Ce($e, qe) {
                    for (var Xe = 0; Xe < qe.length; Xe++) {
                      var Ve = qe[Xe];
                      Ve.enumerable = Ve.enumerable || !1, Ve.configurable = !0, "value" in Ve && (Ve.writable = !0), Object.defineProperty($e, Ve.key, Ve);
                    }
                  }
                  function Se($e, qe, Xe) {
                    return Ce($e.prototype, qe), Ce($e, Xe), $e;
                  }
                  function we($e, qe) {
                    if (typeof qe != "function" && qe !== null)
                      throw new TypeError("Super expression must either be null or a function");
                    $e.prototype = Object.create(qe && qe.prototype, { constructor: { value: $e, writable: !0, configurable: !0 } }), qe && Ee($e, qe);
                  }
                  function Ee($e, qe) {
                    return Ee = Object.setPrototypeOf || function(Ve, Ze) {
                      return Ve.__proto__ = Ze, Ve;
                    }, Ee($e, qe);
                  }
                  function xe($e) {
                    var qe = ke();
                    return function() {
                      var Ve = Me($e), Ze;
                      if (qe) {
                        var Ge = Me(this).constructor;
                        Ze = Reflect.construct(Ve, arguments, Ge);
                      } else
                        Ze = Ve.apply(this, arguments);
                      return Le(this, Ze);
                    };
                  }
                  function Le($e, qe) {
                    return qe && (ve(qe) === "object" || typeof qe == "function") ? qe : Qe($e);
                  }
                  function Qe($e) {
                    if ($e === void 0)
                      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
                    return $e;
                  }
                  function ke() {
                    if (typeof Reflect > "u" || !Reflect.construct || Reflect.construct.sham) return !1;
                    if (typeof Proxy == "function") return !0;
                    try {
                      return Date.prototype.toString.call(Reflect.construct(Date, [], function() {
                      })), !0;
                    } catch {
                      return !1;
                    }
                  }
                  function Me($e) {
                    return Me = Object.setPrototypeOf ? Object.getPrototypeOf : function(Xe) {
                      return Xe.__proto__ || Object.getPrototypeOf(Xe);
                    }, Me($e);
                  }
                  function Re($e, qe) {
                    var Xe = "data-clipboard-".concat($e);
                    if (qe.hasAttribute(Xe))
                      return qe.getAttribute(Xe);
                  }
                  var De = /* @__PURE__ */ function($e) {
                    we(Xe, $e);
                    var qe = xe(Xe);
                    function Xe(Ve, Ze) {
                      var Ge;
                      return be(this, Xe), Ge = qe.call(this), Ge.resolveOptions(Ze), Ge.listenClick(Ve), Ge;
                    }
                    return Se(Xe, [{
                      key: "resolveOptions",
                      value: function() {
                        var Ze = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : {};
                        this.action = typeof Ze.action == "function" ? Ze.action : this.defaultAction, this.target = typeof Ze.target == "function" ? Ze.target : this.defaultTarget, this.text = typeof Ze.text == "function" ? Ze.text : this.defaultText, this.container = ve(Ze.container) === "object" ? Ze.container : document.body;
                      }
                      /**
                       * Adds a click event listener to the passed trigger.
                       * @param {String|HTMLElement|HTMLCollection|NodeList} trigger
                       */
                    }, {
                      key: "listenClick",
                      value: function(Ze) {
                        var Ge = this;
                        this.listener = ae()(Ze, "click", function(ze) {
                          return Ge.onClick(ze);
                        });
                      }
                      /**
                       * Defines a new `ClipboardAction` on each click event.
                       * @param {Event} e
                       */
                    }, {
                      key: "onClick",
                      value: function(Ze) {
                        var Ge = Ze.delegateTarget || Ze.currentTarget, ze = this.action(Ge) || "copy", We = ge({
                          action: ze,
                          container: this.container,
                          target: this.target(Ge),
                          text: this.text(Ge)
                        });
                        this.emit(We ? "success" : "error", {
                          action: ze,
                          text: We,
                          trigger: Ge,
                          clearSelection: function() {
                            Ge && Ge.focus(), window.getSelection().removeAllRanges();
                          }
                        });
                      }
                      /**
                       * Default `action` lookup function.
                       * @param {Element} trigger
                       */
                    }, {
                      key: "defaultAction",
                      value: function(Ze) {
                        return Re("action", Ze);
                      }
                      /**
                       * Default `target` lookup function.
                       * @param {Element} trigger
                       */
                    }, {
                      key: "defaultTarget",
                      value: function(Ze) {
                        var Ge = Re("target", Ze);
                        if (Ge)
                          return document.querySelector(Ge);
                      }
                      /**
                       * Allow fire programmatically a copy action
                       * @param {String|HTMLElement} target
                       * @param {Object} options
                       * @returns Text copied.
                       */
                    }, {
                      key: "defaultText",
                      /**
                       * Default `text` lookup function.
                       * @param {Element} trigger
                       */
                      value: function(Ze) {
                        return Re("text", Ze);
                      }
                      /**
                       * Destroy lifecycle.
                       */
                    }, {
                      key: "destroy",
                      value: function() {
                        this.listener.destroy();
                      }
                    }], [{
                      key: "copy",
                      value: function(Ze) {
                        var Ge = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : {
                          container: document.body
                        };
                        return me(Ze, Ge);
                      }
                      /**
                       * Allow fire programmatically a cut action
                       * @param {String|HTMLElement} target
                       * @returns Text cutted.
                       */
                    }, {
                      key: "cut",
                      value: function(Ze) {
                        return ue(Ze);
                      }
                      /**
                       * Returns the support of the given action, or all actions if no action is
                       * given.
                       * @param {String} [action]
                       */
                    }, {
                      key: "isSupported",
                      value: function() {
                        var Ze = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : ["copy", "cut"], Ge = typeof Ze == "string" ? [Ze] : Ze, ze = !!document.queryCommandSupported;
                        return Ge.forEach(function(We) {
                          ze = ze && !!document.queryCommandSupported(We);
                        }), ze;
                      }
                    }]), Xe;
                  }(ne()), Pe = De;
                }
              ),
              /***/
              828: (
                /***/
                function(G) {
                  var J = 9;
                  if (typeof Element < "u" && !Element.prototype.matches) {
                    var Z = Element.prototype;
                    Z.matches = Z.matchesSelector || Z.mozMatchesSelector || Z.msMatchesSelector || Z.oMatchesSelector || Z.webkitMatchesSelector;
                  }
                  function te(ne, ee) {
                    for (; ne && ne.nodeType !== J; ) {
                      if (typeof ne.matches == "function" && ne.matches(ee))
                        return ne;
                      ne = ne.parentNode;
                    }
                  }
                  G.exports = te;
                }
              ),
              /***/
              438: (
                /***/
                function(G, J, Z) {
                  var te = Z(828);
                  function ne(oe, se, ce, le, ue) {
                    var de = ae.apply(this, arguments);
                    return oe.addEventListener(ce, de, ue), {
                      destroy: function() {
                        oe.removeEventListener(ce, de, ue);
                      }
                    };
                  }
                  function ee(oe, se, ce, le, ue) {
                    return typeof oe.addEventListener == "function" ? ne.apply(null, arguments) : typeof ce == "function" ? ne.bind(null, document).apply(null, arguments) : (typeof oe == "string" && (oe = document.querySelectorAll(oe)), Array.prototype.map.call(oe, function(de) {
                      return ne(de, se, ce, le, ue);
                    }));
                  }
                  function ae(oe, se, ce, le) {
                    return function(ue) {
                      ue.delegateTarget = te(ue.target, se), ue.delegateTarget && le.call(oe, ue);
                    };
                  }
                  G.exports = ee;
                }
              ),
              /***/
              879: (
                /***/
                function(G, J) {
                  J.node = function(Z) {
                    return Z !== void 0 && Z instanceof HTMLElement && Z.nodeType === 1;
                  }, J.nodeList = function(Z) {
                    var te = Object.prototype.toString.call(Z);
                    return Z !== void 0 && (te === "[object NodeList]" || te === "[object HTMLCollection]") && "length" in Z && (Z.length === 0 || J.node(Z[0]));
                  }, J.string = function(Z) {
                    return typeof Z == "string" || Z instanceof String;
                  }, J.fn = function(Z) {
                    var te = Object.prototype.toString.call(Z);
                    return te === "[object Function]";
                  };
                }
              ),
              /***/
              370: (
                /***/
                function(G, J, Z) {
                  var te = Z(879), ne = Z(438);
                  function ee(ce, le, ue) {
                    if (!ce && !le && !ue)
                      throw new Error("Missing required arguments");
                    if (!te.string(le))
                      throw new TypeError("Second argument must be a String");
                    if (!te.fn(ue))
                      throw new TypeError("Third argument must be a Function");
                    if (te.node(ce))
                      return ae(ce, le, ue);
                    if (te.nodeList(ce))
                      return oe(ce, le, ue);
                    if (te.string(ce))
                      return se(ce, le, ue);
                    throw new TypeError("First argument must be a String, HTMLElement, HTMLCollection, or NodeList");
                  }
                  function ae(ce, le, ue) {
                    return ce.addEventListener(le, ue), {
                      destroy: function() {
                        ce.removeEventListener(le, ue);
                      }
                    };
                  }
                  function oe(ce, le, ue) {
                    return Array.prototype.forEach.call(ce, function(de) {
                      de.addEventListener(le, ue);
                    }), {
                      destroy: function() {
                        Array.prototype.forEach.call(ce, function(de) {
                          de.removeEventListener(le, ue);
                        });
                      }
                    };
                  }
                  function se(ce, le, ue) {
                    return ne(document.body, ce, le, ue);
                  }
                  G.exports = ee;
                }
              ),
              /***/
              817: (
                /***/
                function(G) {
                  function J(Z) {
                    var te;
                    if (Z.nodeName === "SELECT")
                      Z.focus(), te = Z.value;
                    else if (Z.nodeName === "INPUT" || Z.nodeName === "TEXTAREA") {
                      var ne = Z.hasAttribute("readonly");
                      ne || Z.setAttribute("readonly", ""), Z.select(), Z.setSelectionRange(0, Z.value.length), ne || Z.removeAttribute("readonly"), te = Z.value;
                    } else {
                      Z.hasAttribute("contenteditable") && Z.focus();
                      var ee = window.getSelection(), ae = document.createRange();
                      ae.selectNodeContents(Z), ee.removeAllRanges(), ee.addRange(ae), te = ee.toString();
                    }
                    return te;
                  }
                  G.exports = J;
                }
              ),
              /***/
              279: (
                /***/
                function(G) {
                  function J() {
                  }
                  J.prototype = {
                    on: function(Z, te, ne) {
                      var ee = this.e || (this.e = {});
                      return (ee[Z] || (ee[Z] = [])).push({
                        fn: te,
                        ctx: ne
                      }), this;
                    },
                    once: function(Z, te, ne) {
                      var ee = this;
                      function ae() {
                        ee.off(Z, ae), te.apply(ne, arguments);
                      }
                      return ae._ = te, this.on(Z, ae, ne);
                    },
                    emit: function(Z) {
                      var te = [].slice.call(arguments, 1), ne = ((this.e || (this.e = {}))[Z] || []).slice(), ee = 0, ae = ne.length;
                      for (ee; ee < ae; ee++)
                        ne[ee].fn.apply(ne[ee].ctx, te);
                      return this;
                    },
                    off: function(Z, te) {
                      var ne = this.e || (this.e = {}), ee = ne[Z], ae = [];
                      if (ee && te)
                        for (var oe = 0, se = ee.length; oe < se; oe++)
                          ee[oe].fn !== te && ee[oe].fn._ !== te && ae.push(ee[oe]);
                      return ae.length ? ne[Z] = ae : delete ne[Z], this;
                    }
                  }, G.exports = J, G.exports.TinyEmitter = J;
                }
              )
              /******/
            }, V = {};
            function q(G) {
              if (V[G])
                return V[G].exports;
              var J = V[G] = {
                /******/
                // no module.id needed
                /******/
                // no module.loaded needed
                /******/
                exports: {}
                /******/
              };
              return H[G](J, J.exports, q), J.exports;
            }
            return function() {
              q.n = function(G) {
                var J = G && G.__esModule ? (
                  /******/
                  function() {
                    return G.default;
                  }
                ) : (
                  /******/
                  function() {
                    return G;
                  }
                );
                return q.d(J, { a: J }), J;
              };
            }(), function() {
              q.d = function(G, J) {
                for (var Z in J)
                  q.o(J, Z) && !q.o(G, Z) && Object.defineProperty(G, Z, { enumerable: !0, get: J[Z] });
              };
            }(), function() {
              q.o = function(G, J) {
                return Object.prototype.hasOwnProperty.call(G, J);
              };
            }(), q(686);
          }().default
        );
      });
    }(clipboard$1)), clipboard$1.exports;
  }
  var clipboardExports = requireClipboard();
  const ClipboardJS = /* @__PURE__ */ getDefaultExportFromCjs$1(clipboardExports), fp = class fp {
    constructor(_, H) {
      /**
       * Clipboard API that binds a clipboard write event to supplied button element,
       * and shows a notification on copy
       */
      mt(this, "clip");
      this.clip = new ClipboardJS(_, {
        text() {
          return H.text ? H.text : fp.getFieldValue(H.fieldId);
        }
      }), this.clip.on(
        "error",
        () => console.error("An error occurred while copying to clipboard")
      ), this.clip.on("success", fp.onSuccess);
    }
    destroy() {
      this.clip.destroy();
    }
    static getFieldValue(_) {
      const H = document.querySelector(`#${_}`);
      if (!H)
        throw `Field with ID ${_} not found`;
      return H.value;
    }
    static async copyOnce(_) {
      try {
        await navigator.clipboard.writeText(_), fp.onSuccess();
      } catch (H) {
        console.error("Error copying to clipboard: ", H);
      }
    }
  };
  mt(fp, "onSuccess", () => {
    const _ = document.getElementById("copy-notification");
    _ && (_.classList.toggle("ar-invisible"), setTimeout(() => _.classList.toggle("ar-invisible"), 2e3));
  });
  let ARClipboard = fp;
  const _hoisted_1$n = { class: "relative" }, _sfc_main$J = /* @__PURE__ */ defineComponent({
    __name: "Code",
    props: {
      language: {},
      code: {}
    },
    setup(O) {
      const _ = o$1.component, H = O, V = ref$1(), q = ref$1(null), G = computed(() => H.code.trim());
      return onMounted(() => {
        q.value && (V.value = new ARClipboard(q.value, {
          text: G.value
        }));
      }), onUnmounted(() => {
        var J;
        (J = V.value) == null || J.destroy();
      }), (J, Z) => (openBlock(), createElementBlock("div", _hoisted_1$n, [
        createBaseVNode("button", {
          class: "absolute top-2 right-2 text-gray-700 h-5 w-5 opacity-75",
          ref_key: "copyBtn",
          ref: q
        }, Z[0] || (Z[0] = [
          createBaseVNode("svg", {
            class: "h-5 w-5",
            fill: "currentColor",
            viewBox: "0 0 20 20"
          }, [
            createBaseVNode("title", null, "Copy"),
            createBaseVNode("path", { d: "M8 3a1 1 0 011-1h2a1 1 0 110 2H9a1 1 0 01-1-1z" }),
            createBaseVNode("path", { d: "M6 3a2 2 0 00-2 2v11a2 2 0 002 2h8a2 2 0 002-2V5a2 2 0 00-2-2 3 3 0 01-3 3H9a3 3 0 01-3-3z" })
          ], -1)
        ]), 512),
        createVNode(unref(_), {
          language: J.language,
          code: G.value,
          "data-cy": "block-code"
        }, null, 8, ["language", "code"])
      ]));
    }
  }), CodeBlock$1 = /* @__PURE__ */ _export_sfc(_sfc_main$J, [["__scopeId", "data-v-1d5be7a9"]]), _sfc_main$I = /* @__PURE__ */ defineComponent({
    __name: "CodeConnector",
    props: {
      code: {},
      language: {},
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = { code: _.code, language: _.language };
      return (V, q) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: V.figure,
        "single-block-embed": V.singleBlockEmbed
      }, {
        default: withCtx(() => [
          createVNode(CodeBlock$1, mergeProps$1(H, { class: "w-full" }), null, 16)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), childIframeResizerJs = '/*!\n *  @preserve\n *  \n *  @module      iframe-resizer/child 5.3.2 (umd) - 2024-10-22\n *\n *  @license     GPL-3.0 for non-commercial use only.\n *               For commercial use, you must purchase a license from\n *               https://iframe-resizer.com/pricing\n * \n *  @description Keep same and cross domain iFrames sized to their content \n *\n *  @author      David J. Bradshaw <info@iframe-resizer.com>\n * \n *  @see         {@link https://iframe-resizer.com}\n * \n *  @copyright  (c) 2013 - 2024, David J. Bradshaw. All rights reserved.\n */\n\n\n!function(e){"function"==typeof define&&define.amd?define(e):e()}((function(){"use strict";const e="5.3.2",t=10,n="data-iframe-size",o="data-overflowed",i="bottom",r="right",a="resizeParent",l=(e,t,n,o)=>e.addEventListener(t,n,o||!1),s=["<iy><yi>Puchspk Spjluzl Rlf</><iy><iy>","<iy><yi>Tpzzpun Spjluzl Rlf</><iy><iy>","Aopz spiyhyf pz hchpshisl dpao ivao Jvttlyjphs huk Vwlu-Zvbyjl spjluzlz.<iy><iy><i>Jvttlyjphs Spjluzl</><iy>Mvy jvttlyjphs bzl, <p>pmyhtl-ylzpgly</> ylxbpylz h svd jvza vul aptl spjluzl mll. Mvy tvyl pumvythapvu cpzpa <b>oaawz://pmyhtl-ylzpgly.jvt/wypjpun</>.<iy><iy><i>Vwlu Zvbyjl Spjluzl</><iy>Pm fvb hyl bzpun aopz spiyhyf pu h uvu-jvttlyjphs vwlu zvbyjl wyvqlja aolu fvb jhu bzl pa mvy myll bukly aol alytz vm aol NWS C3 Spjluzl. Av jvumpyt fvb hjjlwa aolzl alytz, wslhzl zla aol <i>spjluzl</> rlf pu <p>pmyhtl-ylzpgly</> vwapvuz av <i>NWSc3</>.<iy><iy>Mvy tvyl pumvythapvu wslhzl zll: <b>oaawz://pmyhtl-ylzpgly.jvt/nws</>","<i>NWSc3 Spjluzl Clyzpvu</><iy><iy>Aopz clyzpvu vm <p>pmyhtl-ylzpgly</> pz ilpun bzlk bukly aol alytz vm aol <i>NWS C3</> spjluzl. Aopz spjluzl hssvdz fvb av bzl <p>pmyhtl-ylzpgly</> pu Vwlu Zvbyjl wyvqljaz, iba pa ylxbpylz fvby wyvqlja av il wbispj, wyvcpkl haaypibapvu huk il spjluzlk bukly clyzpvu 3 vy shaly vm aol NUB Nlulyhs Wbispj Spjluzl.<iy><iy>Pm fvb hyl bzpun aopz spiyhyf pu h uvu-vwlu zvbyjl wyvqlja vy dlizpal, fvb dpss ullk av wbyjohzl h svd jvza vul aptl jvttlyjphs spjluzl.<iy><iy>Mvy tvyl pumvythapvu cpzpa <b>oaawz://pmyhtl-ylzpgly.jvt/wypjpun</>."];Object.fromEntries(["2cgs7fdf4xb","1c9ctcccr4z","1q2pc4eebgb","ueokt0969w","w2zxchhgqz","1umuxblj2e5"].map(((e,t)=>[e,Math.max(0,t-1)])));const c=e=>(e=>e.replaceAll(/[A-Za-z]/g,(e=>String.fromCodePoint((e<="Z"?90:122)>=(e=e.codePointAt(0)+19)?e:e-26))))(s[e]),d=e=>e,u=1e5,m=e=>Math.round(e*u)/u;let f="",p=!1;const h=e=>{f=e.id,p=e.logging},y=e=>""!=`${e}`&&void 0!==e;const g=(...e)=>[`[iframe-resizer][${f||"child"}]`,...e].join(" "),b=(...e)=>p&&console?.info(`%c[iframe-resizer][${f}]%c`,"font-weight: bold;","font-weight: normal;",...e),v=(...e)=>console?.warn(g(...e)),w=(...e)=>console?.warn((e=>t=>window.chrome?e(t.replaceAll("<br>","\\n").replaceAll("<rb>","\x1B[31;1m").replaceAll("</>","\x1B[m").replaceAll("<b>","\x1B[1m").replaceAll("<i>","\x1B[3m").replaceAll("<u>","\x1B[4m")):e(t.replaceAll("<br>","\\n").replaceAll(/<[/a-z]+>/gi,"")))(g)(...e)),z=e=>w(e),S=e=>{const t=e.side||i,n=e.onChange||d,r={root:e.root,rootMargin:"0px",threshold:1};function a(){const e=document.querySelectorAll(`[${o}]`);n(e)}const l=new IntersectionObserver((function(e){for(const n of e){const{boundingClientRect:e,rootBounds:i,target:r}=n,a=e[t],l=0===a||a>i[t];r.toggleAttribute(o,l)}requestAnimationFrame(a)}),r),s=new WeakSet;return function(e){for(const t of e)t.nodeType!==Node.ELEMENT_NODE||s.has(t)||(l.observe(t),s.add(t))}},E="--ifr-start",j="--ifr-end",$="--ifr-measure",C=[],T=new WeakSet,M=e=>"object"==typeof e&&T.add(e);let O=null,P={};const A=setInterval((()=>{if(C.length<10)return;if(P.hasTags&&P.len<25)return;C.sort();const e=Math.min(C.reduce(((e,t)=>e+t),0)/C.length,C[Math.floor(C.length/2)]);e<=4||(clearInterval(A),w(`<rb>Performance Warning</>\\n\\nCalculating the page size is taking an excessive amount of time (${m(e)}ms).\\n\\nTo improve performance add the <b>data-iframe-size</> attribute to the ${P.Side.toLowerCase()} most element on the page. For more details see: <u>https://iframe-resizer.com/perf</>.`))}),5e3);function N(e){e.getEntries().forEach((e=>{if(e.name===j){const{duration:t}=performance.measure($,E,j);!function(e,t){const{Side:n,len:o,hasTags:i,logging:r}=e;P=e,T.has(O)||i&&o<=1||(r||M(O),b(`\\n  ${n} position calculated from:`,O,`\\n  Parsed ${o} ${i?"tagged":"potentially overflowing"} elements in ${m(t)}ms`))}(e.detail,t),C.push(t),C.length>100&&C.shift()}}))}function I(){new PerformanceObserver(N).observe({entryTypes:["mark"]}),M(document.documentElement),M(document.body)}"undefined"!=typeof document&&"undefined"!=typeof PerformanceObserver&&("loading"===document.readyState?document.addEventListener("DOMContentLoaded",I):I()),"undefined"!=typeof window&&function(){const o={contentVisibilityAuto:!0,opacityProperty:!0,visibilityProperty:!0},s={height:()=>(v("Custom height calculation function not defined"),Le.auto()),width:()=>(v("Custom width calculation function not defined"),Be.auto())},u={bodyOffset:1,bodyScroll:1,offset:1,documentElementOffset:1,documentElementScroll:1,boundingClientRect:1,max:1,min:1,grow:1,lowestElement:1},f=128,p={},g="checkVisibility"in window,b="auto",$="[iFrameSizer]",C=$.length,T={max:1,min:1,bodyScroll:1,documentElementScroll:1},M="scroll";let P,A,N,I,k,R=!0,x="",q=0,L="",B="",D=!0,W=!1,U=!0,F=!1,V=!1,J=1,H=b,Z=!0,_="",Q={},X=!1,Y=0,G=!1,K="",ee=d,te=[],ne="child",oe=null,ie=!1,re="",ae=[],le=window.parent,se="*",ce=0,de=!1,ue="",me=1,fe=M,pe=window,he=()=>{v("onMessage function not defined")},ye=()=>{},ge=null,be=null;function ve(){var o,d,u;!function(){const e=e=>"true"===e,t=_.slice(C).split(":");K=t[0],q=void 0===t[1]?q:Number(t[1]),W=void 0===t[2]?W:e(t[2]),X=void 0===t[3]?X:e(t[3]),R=void 0===t[6]?R:e(t[6]),L=t[7],H=void 0===t[8]?H:t[8],x=t[9],B=t[10],ce=void 0===t[11]?ce:Number(t[11]),Q.enable=void 0!==t[12]&&e(t[12]),ne=void 0===t[13]?ne:t[13],fe=void 0===t[14]?fe:t[14],G=void 0===t[15]?G:e(t[15]),P=void 0===t[16]?P:Number(t[16]),A=void 0===t[17]?A:Number(t[17]),D=void 0===t[18]?D:e(t[18]),t[19],ue=t[20]||ue,Y=void 0===t[21]?Y:Number(t[21])}(),h({id:K,logging:X}),function(){function e(e){he=e?.onMessage||he,ye=e?.onReady||ye,"number"==typeof e?.offset&&(w("<rb>Deprecated option</>\\n\\n The <b>offset</> option has been renamed to <b>offsetSize</>. Use of the old name will be removed in a future version of <i>iframe-resizer</>."),D&&(P=e?.offset),W&&(A=e?.offset)),"number"==typeof e?.offsetSize&&(D&&(P=e?.offsetSize),W&&(A=e?.offsetSize)),Object.prototype.hasOwnProperty.call(e,"sizeSelector")&&(re=e.sizeSelector),se=e?.targetOrigin||se,H=e?.heightCalculationMethod||H,fe=e?.widthCalculationMethod||fe}function t(e,t){return"function"==typeof e&&(s[t]=e,e="custom"),e}if(1===Y)return;const n=window.iframeResizer||window.iFrameResizer;"object"==typeof n&&(e(n),H=t(H,"height"),fe=t(fe,"width"))}(),function(){try{ie="iframeParentListener"in window.parent}catch(e){}}(),Y<0?z(`${c(Y+2)}${c(2)}`):ue.codePointAt(0)>4||Y<2&&z(c(3)),ue&&""!==ue&&"false"!==ue?ue!==e&&w(`<b>Version mismatch</>\\n\\nThe parent and child pages are running different versions of <i>iframe resizer</>.\\n\\nParent page: ${ue} - Child page: ${e}.\\n`):w("<rb>Legacy version detected on parent page</>\\n\\nDetected legacy version of parent page script. It is recommended to update the parent page to use <b>@iframe-resizer/parent</>.\\n\\nSee <u>https://iframe-resizer.com/setup/</> for more details.\\n"),Te(),Me(),function(){let e=!1;const t=t=>document.querySelectorAll(`[${t}]`).forEach((o=>{e=!0,o.removeAttribute(t),o.toggleAttribute(n,!0)}));t("data-iframe-height"),t("data-iframe-width"),e&&w("<rb>Deprecated Attributes</>\\n          \\nThe <b>data-iframe-height</> and <b>data-iframe-width</> attributes have been deprecated and replaced with the single <b>data-iframe-size</> attribute. Use of the old attributes will be removed in a future version of <i>iframe-resizer</>.")}(),ze(),D!==W&&(ee=S({onChange:we,root:document.documentElement,side:D?i:r})),1!==Y&&(pe.parentIframe=Object.freeze({autoResize:e=>(!0===e&&!1===R?(R=!0,Ue("autoResizeEnabled","Auto Resize enabled")):!1===e&&!0===R&&(R=!1),He(0,0,"autoResize",JSON.stringify(R)),R),close(){He(0,0,"close")},getId:()=>K,getOrigin:()=>N,getPageInfo(e){if("function"==typeof e)return ge=e,He(0,0,"pageInfo"),void w("<rb>Deprecated Method</>\\n          \\nThe <b>getPageInfo()</> method has been deprecated and replaced with  <b>getParentProps()</>. Use of this method will be removed in a future version of <i>iframe-resizer</>.\\n");ge=null,He(0,0,"pageInfoStop")},getParentProps(e){if("function"!=typeof e)throw new TypeError("parentIframe.getParentProps(callback) callback not a function");return be=e,He(0,0,"parentInfo"),()=>{be=null,He(0,0,"parentInfoStop")}},getParentProperties(e){w("<rb>Renamed Method</>\\n          \\nThe <b>getParentProperties()</> method has been renamed <b>getParentProps()</>. Use of the old name will be removed in a future version of <i>iframe-resizer</>.\\n"),this.getParentProps(e)},moveToAnchor(e){Q.findTarget(e)},reset(){Je()},scrollBy(e,t){He(t,e,"scrollBy")},scrollTo(e,t){He(t,e,"scrollTo")},scrollToOffset(e,t){He(t,e,"scrollToOffset")},sendMessage(e,t){He(0,0,"message",JSON.stringify(e),t)},setHeightCalculationMethod(e){H=e,Te()},setWidthCalculationMethod(e){fe=e,Me()},setTargetOrigin(e){se=e},resize(e,t){Ue(a,`parentIframe.resize(${e||""}${t?`,${t}`:""})`,e,t)},size(e,t){w("<rb>Deprecated Method</>\\n          \\nThe <b>size()</> method has been deprecated and replaced with  <b>resize()</>. Use of this method will be removed in a future version of <i>iframe-resizer</>.\\n"),this.resize(e,t)}}),pe.parentIFrame=pe.parentIframe),function(){function e(e){He(0,0,e.type,`${e.screenY}:${e.screenX}`)}function t(t,n){l(window.document,t,e)}!0===G&&(t("mouseenter"),t("mouseleave"))}(),Q=function(){const e=()=>({x:document.documentElement.scrollLeft,y:document.documentElement.scrollTop});function n(n){const o=n.getBoundingClientRect(),i=e();return{x:parseInt(o.left,t)+parseInt(i.x,t),y:parseInt(o.top,t)+parseInt(i.y,t)}}function o(e){function t(e){const t=n(e);He(t.y,t.x,"scrollToOffset")}const o=e.split("#")[1]||e,i=decodeURIComponent(o),r=document.getElementById(i)||document.getElementsByName(i)[0];void 0===r?He(0,0,"inPageLink",`#${o}`):t(r)}function i(){const{hash:e,href:t}=window.location;""!==e&&"#"!==e&&o(t)}function r(){for(const e of document.querySelectorAll(\'a[href^="#"]\'))"#"!==e.getAttribute("href")&&l(e,"click",(t=>{t.preventDefault(),o(e.getAttribute("href"))}))}function a(){l(window,"hashchange",i)}function s(){setTimeout(i,f)}function c(){r(),a(),s()}return Q.enable&&(1===Y?w("In page linking requires a Professional or Business license. Please see https://iframe-resizer.com/pricing for more details."):c()),{findTarget:o}}(),Se(ke(document)()),void 0===L&&(L=`${q}px`),Ee("margin",(d="margin",(u=L).includes("-")&&(v(`Negative CSS value ignored for ${d}`),u=""),u)),Ee("background",x),Ee("padding",B),function(){const e=document.createElement("div");e.style.clear="both",e.style.display="block",e.style.height="0",document.body.append(e)}(),function(){const e=e=>e.style.setProperty("height","auto","important");e(document.documentElement),e(document.body)}(),je(),Ue("init","Init message from host page",void 0,void 0,e),document.title&&""!==document.title&&He(0,0,"title",document.title),$e({method:o="add",eventType:"After Print",eventName:"afterprint"}),$e({method:o,eventType:"Before Print",eventName:"beforeprint"}),$e({method:o,eventType:"Ready State Change",eventName:"readystatechange"}),function(){const e=new Set;let t=!1,n=0,o=[];const i=t=>{for(const n of t){const{addedNodes:t,removedNodes:o}=n;for(const n of t)e.add(n);for(const t of o)e.delete(t)}},r=16,a=2,l=200;let s=1;function c(){const d=performance.now(),u=d-n;if(u>r*s+++a&&u<l)return setTimeout(c,r*s),void(n=d);s=1,o.forEach(i),o=[],0!==e.size?(je(),ze(),Se(e),e.forEach(Ae),e.clear(),t=!1):t=!1}function d(e){o.push(e),t||(n=performance.now(),t=!0,requestAnimationFrame(c))}function u(){const e=new window.MutationObserver(d),t=document.querySelector("body"),n={attributes:!1,attributeOldValue:!1,characterData:!1,characterDataOldValue:!1,childList:!0,subtree:!0};return e.observe(t,n),e}u()}(),oe=new ResizeObserver(Oe),oe.observe(document.body),Pe.add(document.body),Ae(document.body),setTimeout(ye)}function we(e){te=e,F=te.length>0,Ue("overflowChanged","Overflow updated")}function ze(){ae=document.querySelectorAll(`[${n}]`),V=ae.length>0}function Se(e){V||ee(e)}function Ee(e,t){void 0!==t&&""!==t&&"null"!==t&&document.body.style.setProperty(e,t)}function je(){if(""!==re)for(const e of document.querySelectorAll(re))e.dataset.iframeSize=!0}function $e(e){({add(t){function n(){Ue(e.eventName,e.eventType)}p[t]=n,l(window,t,n,{passive:!0})},remove(e){const t=p[e];var n,o;delete p[e],n=e,o=t,window.removeEventListener(n,o,!1)}})[e.method](e.eventName)}function Ce(e,t,n,o){return t!==e&&(e in n||(v(`${e} is not a valid option for ${o}CalculationMethod.`),e=t),e in u&&w(`<rb>Deprecated ${o}CalculationMethod (${e})</>\\n\\nThis version of <i>iframe-resizer</> can auto detect the most suitable ${o} calculation method. It is recommended that you remove this option.`)),e}function Te(){H=Ce(H,b,Le,"height")}function Me(){fe=Ce(fe,M,Be,"width")}function Oe(e){Array.isArray(e)&&0!==e.length&&Ue("resizeObserver",`Resize Observed: ${function(e){switch(!0){case!y(e):return"";case y(e.id):return`${e.nodeName.toUpperCase()}#${e.id}`;case y(e.name):return`${e.nodeName.toUpperCase()} (${e.name})`;default:return e.nodeName.toUpperCase()+(y(e.className)?`.${e.className}`:"")}}(e[0].target)}`)}const Pe=new WeakSet;function Ae(e){if(e.nodeType!==Node.ELEMENT_NODE)return;if(!Pe.has(e)){const t=getComputedStyle(e)?.position;""!==t&&"static"!==t&&(oe.observe(e),Pe.add(e))}const t=ke(e)();for(const e of t){if(Pe.has(e)||e?.nodeType!==Node.ELEMENT_NODE)continue;const t=getComputedStyle(e)?.position;""!==t&&"static"!==t&&(oe.observe(e),Pe.add(e))}}function Ne(e){performance.mark(E);const t=(n=e).charAt(0).toUpperCase()+n.slice(1);var n;let i=0,r=document.documentElement,a=V?0:document.documentElement.getBoundingClientRect().bottom;performance.mark(E);const l=V?ae:F?te:ke(document)();let s=l.length;for(const t of l)V||!g||t.checkVisibility(o)?(i=t.getBoundingClientRect()[e]+parseFloat(getComputedStyle(t).getPropertyValue(`margin-${e}`)),i>a&&(a=i,r=t)):s-=1;return O=r,performance.mark(j,{detail:{Side:t,len:s,hasTags:V,logging:X}}),a}const Ie=e=>[e.bodyOffset(),e.bodyScroll(),e.documentElementOffset(),e.documentElementScroll(),e.boundingClientRect()],ke=e=>()=>e.querySelectorAll("* :not(head):not(meta):not(base):not(title):not(script):not(link):not(style):not(map):not(area):not(option):not(optgroup):not(template):not(track):not(wbr):not(nobr)"),Re={height:0,width:0},xe={height:0,width:0};function qe(e){function t(){return xe[o]=i,Re[o]=l,i}const n=e===Le,o=n?"height":"width",i=e.boundingClientRect(),r=Math.ceil(i),a=Math.floor(i),l=(e=>e.documentElementScroll()+Math.max(0,e.getOffset()))(e);switch(!0){case!e.enabled():return l;case V:return e.taggedElement();case!F&&0===xe[o]&&0===Re[o]:return t();case de&&i===xe[o]&&l===Re[o]:return Math.max(i,l);case 0===i:return l;case!F&&i!==xe[o]&&l<=Re[o]:return t();case!n:return e.taggedElement();case!F&&i<xe[o]:case l===a||l===r:case i>l:return t()}return Math.max(e.taggedElement(),t())}const Le={enabled:()=>D,getOffset:()=>P,auto:()=>qe(Le),bodyOffset:()=>{const{body:e}=document,n=getComputedStyle(e);return e.offsetHeight+parseInt(n.marginTop,t)+parseInt(n.marginBottom,t)},bodyScroll:()=>document.body.scrollHeight,offset:()=>Le.bodyOffset(),custom:()=>s.height(),documentElementOffset:()=>document.documentElement.offsetHeight,documentElementScroll:()=>document.documentElement.scrollHeight,boundingClientRect:()=>Math.max(document.documentElement.getBoundingClientRect().bottom,document.body.getBoundingClientRect().bottom),max:()=>Math.max(...Ie(Le)),min:()=>Math.min(...Ie(Le)),grow:()=>Le.max(),lowestElement:()=>Ne(i),taggedElement:()=>Ne(i)},Be={enabled:()=>W,getOffset:()=>A,auto:()=>qe(Be),bodyScroll:()=>document.body.scrollWidth,bodyOffset:()=>document.body.offsetWidth,custom:()=>s.width(),documentElementScroll:()=>document.documentElement.scrollWidth,documentElementOffset:()=>document.documentElement.offsetWidth,boundingClientRect:()=>Math.max(document.documentElement.getBoundingClientRect().right,document.body.getBoundingClientRect().right),max:()=>Math.max(...Ie(Be)),min:()=>Math.min(...Ie(Be)),rightMostElement:()=>Ne(r),scroll:()=>Math.max(Be.bodyScroll(),Be.documentElementScroll()),taggedElement:()=>Ne(r)},De=(e,t)=>!(Math.abs(e-t)<=ce);let We=!1;function Ue(e,t,n,o,i){k=performance.now(),(R||e===a)&&(document.hidden||(We||(I=!0,function(e,t,n,o,i){const r=void 0===n?Le[H]():n,l=void 0===o?Be[fe]():o;D&&De(J,r)||W&&De(me,l)||"init"===e?(Fe(),J=r,me=l,He(J,me,e,i)):!e!==a&&(D&&H in T||W&&fe in T)?Je():I=!1}(e,0,n,o,i),requestAnimationFrame((()=>{We=!1}))),We=!0))}function Fe(){de||(de=!0,requestAnimationFrame((()=>{de=!1})))}function Ve(e){J=Le[H](),me=Be[fe](),He(J,me,e)}function Je(e){const t=H;H=b,Fe(),Ve("reset"),H=t}function He(e,t,n,o,i){Y<-1||(void 0!==i||(i=se),function(){const r=`${K}:${e+(P||0)}:${t+(A||0)}:${n}${void 0===o?"":`:${o}`}`;X&&(console.group(`[iframe-resizer][${K}]`),console.info("Sending message to host page via "+(ie?"sameDomain":"postMessage")),console.info(`%c${r}`,"font-style: italic"),I&&console.info(function(){const e=m(performance.now()-k);return"init"===n?`Initialised iFrame in %c${e}ms`:`Content size recalculated in %c${e}ms`}(),"font-weight:bold;color:#777"),console.groupEnd()),I=!1,ie?window.parent.iframeParentListener($+r):le.postMessage($+r,i)}())}function Ze(e){const t={init:function(){_=e.data,le=e.source,N=e.origin,ve(),U=!1,setTimeout((()=>{Z=!1}),f)},reset(){Z||Ve("resetPage")},resize(){Ue(a)},moveToAnchor(){Q.findTarget(o())},inPageLink(){this.moveToAnchor()},pageInfo(){const e=o();ge?setTimeout((()=>ge(JSON.parse(e)))):He(0,0,"pageInfoStop")},parentInfo(){const e=o();be?setTimeout(be(Object.freeze(JSON.parse(e)))):He(0,0,"parentInfoStop")},message(){const e=o();he(JSON.parse(e))}},n=()=>e.data.split("]")[1].split(":")[0],o=()=>e.data.slice(e.data.indexOf(":")+1),i=()=>"iframeResize"in window||void 0!==window.jQuery&&""in window.jQuery.prototype,r=()=>e.data.split(":")[2]in{true:1,false:1};$===`${e.data}`.slice(0,C)&&(!1!==U?r()&&t.init():function(){const o=n();o in t?t[o]():i()||r()||v(`Unexpected message (${e.data})`)}())}function _e(){"loading"!==document.readyState&&window.parent.postMessage("[iFrameResizerChild]Ready","*")}"iframeChildListener"in window?v("Already setup"):(window.iframeChildListener=e=>setTimeout((()=>Ze({data:e,sameDomain:!0}))),l(window,"message",Ze),l(window,"readystatechange",_e),_e())}()}));\n';
  /*!
   *  @preserve
   *  
   *  @module      iframe-resizer/core 5.3.2 (esm) - 2024-10-22
   *
   *  @license     GPL-3.0 for non-commercial use only.
   *               For commercial use, you must purchase a license from
   *               https://iframe-resizer.com/pricing
   * 
   *  @description Keep same and cross domain iFrames sized to their content 
   *
   *  @author      David J. Bradshaw <info@iframe-resizer.com>
   * 
   *  @see         {@link https://iframe-resizer.com}
   * 
   *  @copyright  (c) 2013 - 2024, David J. Bradshaw. All rights reserved.
   */
  const e$1 = "5.3.2", t$1 = "[iFrameSizer]", i$3 = t$1.length, n = Object.freeze({ max: 1, scroll: 1, bodyScroll: 1, documentElementScroll: 1 }), o = (O, _, H, V) => O.addEventListener(_, H, V || !1), r = (O, _, H) => O.removeEventListener(_, H, !1), a$3 = (O) => {
    if (!O) return "";
    let _ = -559038744, H = 1103547984;
    for (let V, q = 0; q < O.length; q++) V = O.codePointAt(q), _ = Math.imul(_ ^ V, 2246822519), H = Math.imul(H ^ V, 3266489917);
    return _ ^= Math.imul(_ ^ H >>> 15, 1935289751), H ^= Math.imul(H ^ _ >>> 15, 3405138345), _ ^= H >>> 16, H ^= _ >>> 16, (2097152 * (H >>> 0) + (_ >>> 11)).toString(36);
  }, s = (O) => O.replaceAll(/[A-Za-z]/g, (_) => String.fromCodePoint((_ <= "Z" ? 90 : 122) >= (_ = _.codePointAt(0) + 19) ? _ : _ - 26)), l = ["<iy><yi>Puchspk Spjluzl Rlf</><iy><iy>", "<iy><yi>Tpzzpun Spjluzl Rlf</><iy><iy>", "Aopz spiyhyf pz hchpshisl dpao ivao Jvttlyjphs huk Vwlu-Zvbyjl spjluzlz.<iy><iy><i>Jvttlyjphs Spjluzl</><iy>Mvy jvttlyjphs bzl, <p>pmyhtl-ylzpgly</> ylxbpylz h svd jvza vul aptl spjluzl mll. Mvy tvyl pumvythapvu cpzpa <b>oaawz://pmyhtl-ylzpgly.jvt/wypjpun</>.<iy><iy><i>Vwlu Zvbyjl Spjluzl</><iy>Pm fvb hyl bzpun aopz spiyhyf pu h uvu-jvttlyjphs vwlu zvbyjl wyvqlja aolu fvb jhu bzl pa mvy myll bukly aol alytz vm aol NWS C3 Spjluzl. Av jvumpyt fvb hjjlwa aolzl alytz, wslhzl zla aol <i>spjluzl</> rlf pu <p>pmyhtl-ylzpgly</> vwapvuz av <i>NWSc3</>.<iy><iy>Mvy tvyl pumvythapvu wslhzl zll: <b>oaawz://pmyhtl-ylzpgly.jvt/nws</>", "<i>NWSc3 Spjluzl Clyzpvu</><iy><iy>Aopz clyzpvu vm <p>pmyhtl-ylzpgly</> pz ilpun bzlk bukly aol alytz vm aol <i>NWS C3</> spjluzl. Aopz spjluzl hssvdz fvb av bzl <p>pmyhtl-ylzpgly</> pu Vwlu Zvbyjl wyvqljaz, iba pa ylxbpylz fvby wyvqlja av il wbispj, wyvcpkl haaypibapvu huk il spjluzlk bukly clyzpvu 3 vy shaly vm aol NUB Nlulyhs Wbispj Spjluzl.<iy><iy>Pm fvb hyl bzpun aopz spiyhyf pu h uvu-vwlu zvbyjl wyvqlja vy dlizpal, fvb dpss ullk av wbyjohzl h svd jvza vul aptl jvttlyjphs spjluzl.<iy><iy>Mvy tvyl pumvythapvu cpzpa <b>oaawz://pmyhtl-ylzpgly.jvt/wypjpun</>."], c$2 = ["NWSc3", "zvsv", "wyv", "ibzpulzz", "vlt"], u$1 = Object.fromEntries(["2cgs7fdf4xb", "1c9ctcccr4z", "1q2pc4eebgb", "ueokt0969w", "w2zxchhgqz", "1umuxblj2e5"].map((O, _) => [O, Math.max(0, _ - 1)])), d = (O) => s(l[O]), f = (O) => {
    const _ = O[s("spjluzl")];
    if (!_) return -1;
    const H = _.split("-");
    let V = function(q = "") {
      let G = -2;
      const J = a$3(s(q));
      return J in u$1 && (G = u$1[J]), G;
    }(H[0]);
    return V === 0 || ((q) => q[2] === a$3(q[0] + q[1]))(H) || (V = -2), V;
  }, p$2 = "[iframe-resizer]", h$2 = (O) => `${p$2}[${function(_) {
    var H;
    return window.top === window.self ? `Parent page: ${_}` : (H = window == null ? void 0 : window.parentIFrame) != null && H.getId ? `${window.parentIFrame.getId()}: ${_}` : `Nested parent page: ${_}`;
  }(O)}]`, m$3 = (O, ..._) => ((H, V, ...q) => window == null ? void 0 : window.console[H](h$2(V), ...q))("warn", O, ..._), y$5 = (O, _) => console == null ? void 0 : console.warn((/* @__PURE__ */ ((H) => (V) => window.chrome ? H(V.replaceAll("<br>", `
`).replaceAll("<rb>", "\x1B[31;1m").replaceAll("</>", "\x1B[m").replaceAll("<b>", "\x1B[1m").replaceAll("<i>", "\x1B[3m").replaceAll("<u>", "\x1B[4m")) : H(V.replaceAll("<br>", `
`).replaceAll(/<[/a-z]+>/gi, "")))(/* @__PURE__ */ ((H) => (...V) => [`${p$2}[${H}]`, ...V].join(" "))(O)))(_)), g = {}, b$1 = Object.freeze({ autoResize: !0, bodyBackground: null, bodyMargin: null, bodyPadding: null, checkOrigin: !0, direction: "vertical", inPageLinks: !1, heightCalculationMethod: "auto", id: "iFrameResizer", log: !1, license: void 0, mouseEvents: !0, offsetHeight: null, offsetWidth: null, postMessageTarget: null, sameDomain: !1, scrolling: !1, sizeHeight: !0, sizeWidth: !1, warningTimeout: 5e3, tolerance: 0, waitForLoad: !1, widthCalculationMethod: "auto", onClose: () => !0, onClosed() {
  }, onInit: !1, onMessage: null, onMouseEnter() {
  }, onMouseLeave() {
  }, onReady: (O) => {
    typeof g[O.id].onInit == "function" && (y$5(O.id, `
\x1B[31;1mDeprecated Option\x1B[m

The \x1B[1monInit()\x1B[m function is deprecated and has been replaced with \x1B[1monReady()\x1B[m. It will be removed in a future version of iFrame Resizer.
      `), g[O.id].onInit(O));
  }, onResized() {
  }, onScroll: () => !0 }), w$2 = { position: null, version: e$1 };
  function v(O) {
    function _() {
      M$1(me), S$1(), fe("onResized", me);
    }
    function H(ye) {
      return ye.boxSizing !== "border-box" ? 0 : (ye.paddingTop ? parseInt(ye.paddingTop, 10) : 0) + (ye.paddingBottom ? parseInt(ye.paddingBottom, 10) : 0);
    }
    function V(ye) {
      return ye.boxSizing !== "border-box" ? 0 : (ye.borderTopWidth ? parseInt(ye.borderTopWidth, 10) : 0) + (ye.borderBottomWidth ? parseInt(ye.borderBottomWidth, 10) : 0);
    }
    const q = (ye) => he.slice(he.indexOf(":") + 7 + ye), G = (ye, ge) => (ve, be) => {
      const Ce = {};
      var Se, we;
      Se = function() {
        x$6(`Send ${ye} (${ve})`, `${ye}:${ge()}`, be);
      }, Ce[we = be] || (Se(), Ce[we] = requestAnimationFrame(() => {
        Ce[we] = null;
      }));
    }, J = (ye, ge) => () => {
      let ve = !1;
      const be = (ke) => () => {
        g[xe] ? ve && ve !== ke || (ye(ke, xe), ve = ke, requestAnimationFrame(() => {
          ve = !1;
        })) : Ee();
      }, Ce = be("scroll"), Se = be("resize window");
      function we(ke, Me) {
        Me(window, "scroll", Ce), Me(window, "resize", Se);
      }
      function Ee() {
        we(0, r), Le.disconnect(), Qe.disconnect();
      }
      const xe = pe, Le = new ResizeObserver(be("page observed")), Qe = new ResizeObserver(be("iframe observed"));
      we(0, o), Le.observe(document.body, { attributes: !0, childList: !0, subtree: !0 }), Qe.observe(g[xe].iframe, { attributes: !0, childList: !1, subtree: !1 }), g[xe] && (g[xe][`stop${ge}`] = Ee);
    }, Z = (ye) => () => {
      ye in g[pe] && (g[pe][ye](), delete g[pe][ye]);
    }, te = G("pageInfo", function() {
      const ye = document.body.getBoundingClientRect(), ge = me.iframe.getBoundingClientRect(), { scrollY: ve, scrollX: be, innerHeight: Ce, innerWidth: Se } = window, { clientHeight: we, clientWidth: Ee } = document.documentElement;
      return JSON.stringify({ iframeHeight: ge.height, iframeWidth: ge.width, clientHeight: Math.max(we, Ce || 0), clientWidth: Math.max(Ee, Se || 0), offsetTop: parseInt(ge.top - ye.top, 10), offsetLeft: parseInt(ge.left - ye.left, 10), scrollTop: ve, scrollLeft: be, documentHeight: we, documentWidth: Ee, windowHeight: Ce, windowWidth: Se });
    }), ne = G("parentInfo", function() {
      const { iframe: ye } = me, { scrollWidth: ge, scrollHeight: ve } = document.documentElement, { width: be, height: Ce, offsetLeft: Se, offsetTop: we, pageLeft: Ee, pageTop: xe, scale: Le } = window.visualViewport;
      return JSON.stringify({ iframe: ye.getBoundingClientRect(), document: { scrollWidth: ge, scrollHeight: ve }, viewport: { width: be, height: Ce, offsetLeft: Se, offsetTop: we, pageLeft: Ee, pageTop: xe, scale: Le } });
    }), ee = J(te, "PageInfo"), ae = J(ne, "ParentInfo"), oe = Z("stopPageInfo"), se = Z("stopParentInfo");
    function ce(ye) {
      const ge = ye.getBoundingClientRect();
      return k$1(), { x: Number(ge.left) + Number(w$2.position.x), y: Number(ge.top) + Number(w$2.position.y) };
    }
    function le(ye) {
      const ge = ye ? ce(me.iframe) : { x: 0, y: 0 }, ve = ((Ce, Se) => ({ x: Ce.width + Se.x, y: Ce.height + Se.y }))(me, ge), be = window.parentIframe || window.parentIFrame;
      be ? function(Ce, Se) {
        Ce["scrollTo" + (ye ? "Offset" : "")](Se.x, Se.y);
      }(be, ve) : function(Ce) {
        w$2.position = Ce, ue(pe);
      }(ve);
    }
    function ue(ye) {
      var Ce;
      const { x: ge, y: ve } = w$2.position, be = (Ce = g[ye]) == null ? void 0 : Ce.iframe;
      fe("onScroll", { iframe: be, top: ve, left: ge, x: ge, y: ve }) !== !1 ? S$1() : $$3();
    }
    function de(ye) {
      let ge = {};
      if (me.width === 0 && me.height === 0) {
        const ve = q(9).split(":");
        ge = { x: ve[1], y: ve[0] };
      } else ge = { x: me.width, y: me.height };
      fe(ye, { iframe: me.iframe, screenX: Number(ge.x), screenY: Number(ge.y), type: me.type });
    }
    const fe = (ye, ge) => z(pe, ye, ge);
    let he = O.data, me = {}, pe = null;
    he !== "[iFrameResizerChild]Ready" ? t$1 === `${he}`.slice(0, i$3) && he.slice(i$3).split(":")[0] in g && (me = function() {
      var Ce;
      const ye = he.slice(i$3).split(":"), ge = ye[1] ? Number(ye[1]) : 0, ve = (Ce = g[ye[0]]) == null ? void 0 : Ce.iframe, be = getComputedStyle(ve);
      return { iframe: ve, id: ye[0], height: ge + H(be) + V(be), width: Number(ye[2]), type: ye[3], msg: ye[4] };
    }(), pe = me.id, pe ? (function(ye) {
      if (!g[ye]) throw new Error(`${me.type} No settings for ${ye}. Message was: ${he}`);
    }(pe), me.type in { true: 1, false: 1, undefined: 1 } || (g[pe].loaded = !0, (me.iframe !== null || (m$3(pe, `The iframe (${me.id}) was not found.`), 0)) && function() {
      var be;
      const { origin: ye, sameDomain: ge } = O;
      if (ge) return !0;
      let ve = (be = g[pe]) == null ? void 0 : be.checkOrigin;
      if (ve && `${ye}` != "null" && !(ve.constructor === Array ? function() {
        let Ce = 0, Se = !1;
        for (; Ce < ve.length; Ce++) if (ve[Ce] === ye) {
          Se = !0;
          break;
        }
        return Se;
      }() : function() {
        var Se;
        const Ce = (Se = g[pe]) == null ? void 0 : Se.remoteHost;
        return ye === Ce;
      }())) throw new Error(`Unexpected message received from: ${ye} for ${me.iframe.id}. Message was: ${O.data}. This error can be disabled by setting the checkOrigin: false option or by providing of array of trusted domains.`);
      return !0;
    }() && function() {
      var ve;
      switch ((ve = g[pe]) != null && ve.firstRun && g[pe] && (g[pe].firstRun = !1), me.type) {
        case "close":
          T$1(me.iframe);
          break;
        case "message":
          ge = q(6), fe("onMessage", { iframe: me.iframe, message: JSON.parse(ge) });
          break;
        case "mouseenter":
          de("onMouseEnter");
          break;
        case "mouseleave":
          de("onMouseLeave");
          break;
        case "autoResize":
          g[pe].autoResize = JSON.parse(q(9));
          break;
        case "scrollBy":
          (function() {
            const be = me.width, Ce = me.height;
            (window.parentIframe || window).scrollBy(be, Ce);
          })();
          break;
        case "scrollTo":
          le(!1);
          break;
        case "scrollToOffset":
          le(!0);
          break;
        case "pageInfo":
          te("start", pe), ee();
          break;
        case "parentInfo":
          ne("start", pe), ae();
          break;
        case "pageInfoStop":
          oe();
          break;
        case "parentInfoStop":
          se();
          break;
        case "inPageLink":
          (function(be) {
            const Ce = be.split("#")[1] || "", Se = decodeURIComponent(Ce);
            let we = document.getElementById(Se) || document.getElementsByName(Se)[0];
            we ? function() {
              const Ee = ce(we);
              w$2.position = { x: Ee.x, y: Ee.y }, ue(pe), window.location.hash = Ce;
            }() : window.top !== window.self && window.parentIFrame && window.parentIFrame.moveToAnchor(Ce);
          })(q(9));
          break;
        case "title":
          (function(be, Ce) {
            var Se;
            (Se = g[Ce]) != null && Se.syncTitle && (g[Ce].iframe.title = be);
          })(me.msg, pe);
          break;
        case "reset":
          I(me);
          break;
        case "init":
          _(), function(be) {
            var Ce, Se, we;
            try {
              g[be].sameDomain = !!((we = (Se = (Ce = g[be]) == null ? void 0 : Ce.iframe) == null ? void 0 : Se.contentWindow) != null && we.iframeChildListener);
            } catch {
              g[be].sameDomain = !1;
            }
          }(pe), (ye = me.msg) !== e$1 && (ye !== void 0 || y$5(pe, `<rb>Legacy version detected in iframe</>

Detected legacy version of child page script. It is recommended to update the page in the iframe to use <b>@iframe-resizer/child</>.

See <u>https://iframe-resizer.com/setup/#child-page-setup</> for more details.
`)), F$1 = !0, fe("onReady", me.iframe);
          break;
        default:
          if (me.width === 0 && me.height === 0) return void m$3(pe, `Unsupported message received (${me.type}), this is likely due to the iframe containing a later version of iframe-resizer than the parent page`);
          if (me.width === 0 || me.height === 0 || document.hidden) return;
          _();
      }
      var ye, ge;
    }())) : m$3("", "iframeResizer received messageData without id, message was: ", he)) : Object.keys(g).forEach((ye) => {
      g[ye].mode >= 0 && x$6("iFrame requested init", R(ye), ye);
    });
  }
  function z(O, _, H) {
    let V = null, q = null;
    if (g[O]) {
      if (V = g[O][_], typeof V != "function") throw new TypeError(`${_} on iFrame[${O}] is not a function`);
      _ === "onClose" || _ === "onScroll" ? q = V(H) : setTimeout(() => V(H));
    }
    return q;
  }
  function j(O) {
    const { id: _ } = O;
    delete g[_];
  }
  function T$1(O) {
    const { id: _ } = O;
    if (z(_, "onClose", _) !== !1) {
      try {
        O.parentNode && O.remove();
      } catch (H) {
        m$3(_, H);
      }
      z(_, "onClosed", _), j(O);
    }
  }
  function k$1(O) {
    w$2.position === null && (w$2.position = { x: window.scrollX, y: window.scrollY });
  }
  function $$3() {
    w$2.position = null;
  }
  function S$1(O) {
    w$2.position !== null && (window.scrollTo(w$2.position.x, w$2.position.y), $$3());
  }
  function I(O) {
    k$1(O.id), M$1(O), x$6("reset", "reset", O.id);
  }
  function M$1(O) {
    function _(G) {
      const J = `${O[G]}px`;
      O.iframe.style[G] = J;
    }
    const { id: H } = O, { sizeHeight: V, sizeWidth: q } = g[H];
    V && _("height"), q && _("width");
  }
  function x$6(O, _, H, V) {
    var q, G;
    g[H] && ((q = g[H]) != null && q.postMessageTarget ? function() {
      const { iframe: J, postMessageTarget: Z, sameDomain: te, targetOrigin: ne } = g[H];
      if (te) try {
        return void J.contentWindow.iframeChildListener(t$1 + _);
      } catch {
      }
      Z.postMessage(t$1 + _, ne);
    }() : m$3(H, `[${O}] IFrame(${H}) not found`), V && ((G = g[H]) != null && G.warningTimeout) && (g[H].msgTimeout = setTimeout(function() {
      if (g[H] === void 0) return;
      const { iframe: J, loaded: Z, loadErrorShown: te, waitForLoad: ne } = g[H], { sandbox: ee } = J;
      Z || te || (g[H].loadErrorShown = !0, y$5(H, `
<rb>No response from iFrame</>
            
The iframe (<i>${H}</>) has not responded within ${g[H].warningTimeout / 1e3} seconds. Check <b>@iframe-resizer/child</> package has been loaded in the iframe.
${ne ? `
The <b>waitForLoad</> option is currently set to <i>'true'</>. If the iframe loads before the JavaScript runs, this option will prevent <i>iframe-resizer</> from initialising. To disable this, set the <b>waitForLoad</> option to <i>'false'</>.  
` : ""}
${!((ee == null ? void 0 : ee.length) > 0) || ee.contains("allow-scripts") && ee.contains("allow-same-origin") ? "" : "The iframe has the <b>sandbox</> attribute, please ensure it contains both the <i>'allow-same-origin'</> and <i>'allow-scripts'</> values."}

This message can be ignored if everything is working, or you can set the <b>warningTimeout</> option to a higher value or zero to suppress this warning.
`));
    }, g[H].warningTimeout)));
  }
  function R(O) {
    const _ = g[O];
    return [O, "8", _.sizeWidth, _.log, "32", !0, _.autoResize, _.bodyMargin, _.heightCalculationMethod, _.bodyBackground, _.bodyPadding, _.tolerance, _.inPageLinks, "child", _.widthCalculationMethod, _.mouseEvents, _.offsetHeight, _.offsetWidth, _.sizeHeight, _.license, w$2.version, _.mode].join(":");
  }
  let W$1 = 0, F$1 = !1, L = !1;
  const N$1 = (O) => (_) => {
    function H(G) {
      if (!G) return {};
      if (typeof G != "object") throw new TypeError("Options is not an object");
      return ("sizeWidth" in G || "sizeHeight" in G || "autoResize" in G) && y$5(q, `<rb>Deprecated Option</>

The <b>sizeWidth</>, <b>sizeHeight</> and <b>autoResize</> options have been replaced with new <b>direction</> option which expects values of <i>"vertical"</>, <i>"horizontal"</> or <i>"horizontal"</>.
`), G;
    }
    function V(G) {
      var Z, te;
      const J = (te = (Z = g[G]) == null ? void 0 : Z.iframe) == null ? void 0 : te.title;
      return J === "" || J === void 0;
    }
    const q = function(G) {
      if (G && typeof G != "string") throw new TypeError("Invalid id for iFrame. Expected String");
      return G !== "" && G || (G = function() {
        let J = (O == null ? void 0 : O.id) || b$1.id + W$1++;
        return document.getElementById(J) !== null && (J += W$1++), J;
      }(), _.id = G, O.log), G;
    }(_.id);
    return q in g && "iFrameResizer" in _ ? m$3(q, "Ignored iFrame, already setup.") : (function(G) {
      var J, Z;
      g[q] = { iframe: _, firstRun: !0, remoteHost: _ == null ? void 0 : _.src.split("/").slice(0, 3).join("/"), ...b$1, ...H(G), mode: f(G), syncTitle: V(q) }, function() {
        const { direction: te } = g[q];
        if (te === "horizontal") return g[q].sizeWidth = !0, void (g[q].sizeHeight = !1);
        if (te === "none") return g[q].sizeWidth = !1, g[q].sizeHeight = !1, void (g[q].autoResize = !1);
        if (te !== "vertical") throw new TypeError(q, `Direction value of "${te}" is not valid`);
      }(), (J = (G == null ? void 0 : G.offsetSize) || (G == null ? void 0 : G.offset)) && (g[q].direction === "vertical" ? g[q].offsetHeight = J : g[q].offsetWidth = J), G != null && G.offset && y$5(q, `<rb>Deprecated option</>

 The <b>offset</> option has been renamed to <b>offsetSize</>. Use of the old name will be removed in a future version of <i>iframe-resizer</>.`), g[q].postMessageTarget === null && (g[q].postMessageTarget = _.contentWindow), g[q].targetOrigin = g[q].checkOrigin === !0 ? (Z = g[q].remoteHost) === "" || Z.match(/^(about:blank|javascript:|file:\/\/)/) !== null ? "*" : Z : "*";
    }(O), function() {
      const { mode: G } = g[q];
      var J;
      G < 0 && y$5("Parent", `${d(G + 2)}${d(2)}`), L || G < 0 || (L = !0, J = `v${e$1} (${((Z) => s(c$2[Z]))(G)})`, console.info(`%c[iframe-resizer] ${J}`, "font-weight: bold;"), G < 1 && y$5("Parent", d(3)));
    }(), E$2(), function() {
      var G, J;
      switch (_.style.overflow = ((G = g[q]) == null ? void 0 : G.scrolling) === !1 ? "hidden" : "auto", (J = g[q]) == null ? void 0 : J.scrolling) {
        case "omit":
          break;
        case !0:
          _.scrolling = "yes";
          break;
        case !1:
          _.scrolling = "no";
          break;
        default:
          _.scrolling = g[q] ? g[q].scrolling : "no";
      }
    }(), function() {
      const { bodyMargin: G } = g[q];
      typeof G != "number" && G !== "0" || (g[q].bodyMargin = `${G}px`);
    }(), function(G) {
      const { id: J } = _, { mode: Z, waitForLoad: te } = g[J];
      Z !== -1 && Z !== -2 && (o(_, "load", function() {
        x$6("iFrame.onload", `${G}:${F$1}`, J, !0), function() {
          var ae, oe;
          const ne = (ae = g[q]) == null ? void 0 : ae.firstRun, ee = ((oe = g[q]) == null ? void 0 : oe.heightCalculationMethod) in n;
          !ne && ee && I({ iframe: _, height: 0, width: 0, type: "init" });
        }();
      }), te === !1 && x$6("init", `${G}:${F$1}`, J, !0));
    }(R(q)), function() {
      if (g[q]) {
        const { iframe: G } = g[q], J = { close: T$1.bind(null, G), disconnect: j.bind(null, G), removeListeners() {
          y$5(q, `
<rb>Deprecated Method Name</>

The \x1B[removeListeners()</> method has been renamed to \x1B[disconnect()</>.
`), this.disconnect();
        }, resize: x$6.bind(null, "Window resize", "resize", q), moveToAnchor(Z) {
          x$6("Move to anchor", `moveToAnchor:${Z}`, q);
        }, sendMessage(Z) {
          x$6("Send Message", `message:${Z = JSON.stringify(Z)}`, q);
        } };
        G.iframeResizer = J, G.iFrameResizer = J;
      }
    }()), _ == null ? void 0 : _.iFrameResizer;
  };
  function C$3() {
    document.hidden === !1 && function(O, _) {
      const H = (V) => {
        var q, G;
        return ((q = g[V]) == null ? void 0 : q.autoResize) && !((G = g[V]) != null && G.firstRun);
      };
      Object.keys(g).forEach(function(V) {
        H(V) && x$6(O, _, V);
      });
    }("Tab Visible", "resize");
  }
  const E$2 = /* @__PURE__ */ ((O) => {
    let _ = !1;
    return function() {
      return _ ? void 0 : (_ = !0, Reflect.apply(O, this, arguments));
    };
  })(() => {
    o(window, "message", v), o(document, "visibilitychange", C$3), window.iframeParentListener = (O) => setTimeout(() => v({ data: O, sameDomain: !0 }));
  }), _sfc_main$H = {
    name: "IframeResizer",
    props: {
      license: {
        type: String,
        required: !0
      },
      bodyBackground: {
        type: String
      },
      bodyMargin: {
        type: String
      },
      bodyPadding: {
        type: String
      },
      checkOrigin: {
        type: Boolean,
        default: !0
      },
      direction: {
        type: String
      },
      inPageLinks: {
        type: Boolean
      },
      offset: {
        type: Number
      },
      scrolling: {
        type: Boolean
      },
      tolerance: {
        type: Number
      },
      warningTimeout: {
        type: Number
      }
    },
    mounted() {
      const O = this, { iframe: _ } = this.$refs, H = {
        ...Object.fromEntries(
          Object.entries(this.$props).filter(([q, G]) => G !== void 0)
        ),
        waitForLoad: !0,
        onClosed: () => !1,
        // Disable close methods, use Vue to remove iframe
        onReady: (...q) => O.$emit("onReady", ...q),
        onMessage: (...q) => O.$emit("onMessage", ...q),
        onResized: (...q) => O.$emit("onResized", ...q)
      }, V = N$1(H);
      O.resizer = V(_);
    },
    beforeUnmount() {
      var O;
      (O = this.resizer) == null || O.disconnect();
    },
    methods: {
      moveToAnchor(O) {
        this.resizer.moveToAnchor(O);
      },
      resize() {
        this.resizer.resize();
      },
      sendMessage(O, _) {
        this.resizer.sendMessage(O, _);
      }
    }
  };
  function _sfc_render(O, _, H, V, q, G) {
    return openBlock(), createElementBlock("iframe", mergeProps$1({ ref: "iframe" }, O.$attrs), null, 16);
  }
  const IframeResizer = /* @__PURE__ */ _export_sfc(_sfc_main$H, [["render", _sfc_render]]), _hoisted_1$m = ["innerHTML"], _sfc_main$G = /* @__PURE__ */ defineComponent({
    __name: "Embed",
    props: {
      html: {},
      isIframe: { type: Boolean },
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = computed(() => _.html.replace("&lt;script&gt;", "<script>").replace("&lt;&sol;script&gt;", "<\/script>")), V = computed(() => `
        <!DOCTYPE html>
        <html>
        <body>
          <script>${childIframeResizerJs}<\/script>
          ${H.value}
        </body>
        </html>
    `);
      return (q, G) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: q.figure,
        "single-block-embed": q.singleBlockEmbed
      }, {
        default: withCtx(() => [
          q.isIframe ? (openBlock(), createElementBlock("div", {
            key: 0,
            innerHTML: H.value,
            class: "flex justify-center items-center",
            "data-cy": "block-embed"
          }, null, 8, _hoisted_1$m)) : (openBlock(), createBlock(unref(IframeResizer), {
            key: 1,
            license: "GPLv3",
            srcdoc: V.value,
            class: "flex justify-center items-center"
          }, null, 8, ["srcdoc"]))
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), VEmbedBlock = /* @__PURE__ */ _export_sfc(_sfc_main$G, [["__scopeId", "data-v-a75d58d0"]]), _hoisted_1$l = ["srcdoc"], IFRAME_HEIGHT = "35rem", _sfc_main$F = /* @__PURE__ */ defineComponent({
    __name: "Folium",
    props: {
      iframeContent: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      return (_, H) => (openBlock(), createElementBlock("div", {
        style: normalizeStyle({ height: IFRAME_HEIGHT }),
        class: normalizeClass(["flex justify-center w-full", { "h-iframe": _.singleBlockEmbed }]),
        "data-cy": "block-iframe"
      }, [
        createBaseVNode("iframe", {
          style: { border: "none !important" },
          class: normalizeClass(["w-full", { "h-iframe": _.singleBlockEmbed }]),
          srcdoc: _.iframeContent,
          sandbox: "allow-scripts"
        }, null, 10, _hoisted_1$l)
      ], 6));
    }
  }), _sfc_main$E = /* @__PURE__ */ defineComponent({
    __name: "FoliumConnector",
    props: {
      fetchAssetData: {},
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = ref$1(null);
      return (async () => H.value = await _.fetchAssetData())(), (V, q) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: V.figure,
        "single-block-embed": V.singleBlockEmbed
      }, {
        default: withCtx(() => [
          H.value ? (openBlock(), createBlock(_sfc_main$F, {
            key: 0,
            "iframe-content": H.value,
            "single-block-embed": V.singleBlockEmbed
          }, null, 8, ["iframe-content", "single-block-embed"])) : createCommentVNode("", !0)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), _sfc_main$D = /* @__PURE__ */ defineComponent({
    __name: "FormulaConnector",
    props: {
      content: {},
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = /* @__PURE__ */ defineAsyncComponent(() => Promise.resolve().then(() => Formula$1));
      return (H, V) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: H.figure,
        "single-block-embed": H.singleBlockEmbed
      }, {
        default: withCtx(() => [
          createVNode(unref(_), { content: H.content }, null, 8, ["content"])
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), userIframeCss = "body{font-family:Inter,system-ui,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,Noto Sans,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji}.doc-root{overflow-x:auto;width:100%}", _sfc_main$C = /* @__PURE__ */ defineComponent({
    __name: "HTML",
    props: {
      html: {},
      sandbox: {},
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = computed(() => `
        <!DOCTYPE html>
        <html>
        <body>
          <style>${userIframeCss}</style>
          <script>${childIframeResizerJs}<\/script>
          <div class="doc-root">${_.html}</div>
        </body>
        </html>
    `);
      return (V, q) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: V.figure,
        "single-block-embed": V.singleBlockEmbed
      }, {
        default: withCtx(() => [
          createVNode(unref(IframeResizer), {
            license: "GPLv3",
            srcdoc: H.value,
            sandbox: V.sandbox || "",
            width: "100%",
            "data-cy": "block-user-iframe"
          }, null, 8, ["srcdoc", "sandbox"])
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), VHTMLBlock = /* @__PURE__ */ _export_sfc(_sfc_main$C, [["__scopeId", "data-v-98078261"]]), _hoisted_1$k = {
    class: "w-full flex items-center justify-center",
    "data-cy": "block-media"
  }, _hoisted_2$9 = ["src"], _hoisted_3$8 = ["src"], _hoisted_4$6 = ["src"], _sfc_main$B = /* @__PURE__ */ defineComponent({
    __name: "Media",
    props: {
      src: {},
      type: {},
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      return (_, H) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: _.figure,
        "single-block-embed": _.singleBlockEmbed
      }, {
        default: withCtx(() => [
          createBaseVNode("div", _hoisted_1$k, [
            _.type.startsWith("video") ? (openBlock(), createElementBlock("video", {
              key: 0,
              controls: "",
              src: _.src,
              width: "800"
            }, null, 8, _hoisted_2$9)) : _.type.startsWith("audio") ? (openBlock(), createElementBlock("audio", {
              key: 1,
              controls: "",
              src: _.src,
              class: "w-full"
            }, null, 8, _hoisted_3$8)) : (openBlock(), createElementBlock("img", {
              key: 2,
              src: _.src,
              loading: "lazy",
              class: "mx-auto"
            }, null, 8, _hoisted_4$6))
          ])
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), _sfc_main$A = /* @__PURE__ */ defineComponent({
    __name: "PlotlyConnector",
    props: {
      fetchAssetData: {},
      responsive: { type: Boolean },
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = /* @__PURE__ */ defineAsyncComponent(() => Promise.resolve().then(() => Plotly)), H = O, V = ref$1(null);
      return (async () => V.value = await H.fetchAssetData())(), (q, G) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: q.figure,
        "single-block-embed": q.singleBlockEmbed
      }, {
        default: withCtx(() => [
          V.value ? (openBlock(), createBlock(unref(_), {
            key: 0,
            "plot-json": V.value,
            responsive: q.responsive,
            "single-block-embed": q.singleBlockEmbed
          }, null, 8, ["plot-json", "responsive", "single-block-embed"])) : createCommentVNode("", !0)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), _hoisted_1$j = ["src"], _sfc_main$z = /* @__PURE__ */ defineComponent({
    __name: "SVG",
    props: {
      src: {},
      responsive: { type: Boolean },
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      return (_, H) => (openBlock(), createElementBlock("img", {
        src: _.src,
        "data-cy": "block-svg",
        loading: "lazy",
        class: normalizeClass([
          "mx-auto",
          {
            "w-full": _.responsive,
            "h-full": _.responsive && _.singleBlockEmbed,
            "max-w-none": !_.responsive,
            // Don't scale non-responsive SVGs like a normal image -- overwriting default browser `max-width: 100%`
            "absolute top-0 left-0": !_.responsive && _.singleBlockEmbed
            // Prevent large embedded SVGs from being wrongly positioned in the iframe
          }
        ])
      }, null, 10, _hoisted_1$j));
    }
  }), _sfc_main$y = /* @__PURE__ */ defineComponent({
    __name: "SVGConnector",
    props: {
      fetchAssetData: {},
      responsive: { type: Boolean },
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = ref$1(null);
      return (async () => H.value = await _.fetchAssetData())(), (V, q) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: V.figure,
        "single-block-embed": V.singleBlockEmbed
      }, {
        default: withCtx(() => [
          H.value ? (openBlock(), createBlock(_sfc_main$z, {
            key: 0,
            src: H.value,
            responsive: V.responsive,
            "single-block-embed": V.singleBlockEmbed
          }, null, 8, ["src", "responsive", "single-block-embed"])) : createCommentVNode("", !0)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), _hoisted_1$i = ["html", "single-block-embed"], _sfc_main$x = /* @__PURE__ */ defineComponent({
    __name: "TableConnector",
    props: {
      fetchAssetData: {},
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = ref$1(null);
      return (async () => H.value = await _.fetchAssetData())(), (V, q) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: _.figure,
        "single-block-embed": V.singleBlockEmbed
      }, {
        default: withCtx(() => [
          H.value ? (openBlock(), createElementBlock("x-table-block", {
            key: 0,
            html: H.value,
            "single-block-embed": V.singleBlockEmbed,
            class: "w-full"
          }, null, 8, _hoisted_1$i)) : createCommentVNode("", !0)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  });
  var _1c_1, hasRequired_1c;
  function require_1c() {
    if (hasRequired_1c) return _1c_1;
    hasRequired_1c = 1;
    function O(_) {
      const H = "[A-Za-zА-Яа-яёЁ_][A-Za-zА-Яа-яёЁ_0-9]+", G = "далее " + "возврат вызватьисключение выполнить для если и из или иначе иначеесли исключение каждого конецесли конецпопытки конеццикла не новый перейти перем по пока попытка прервать продолжить тогда цикл экспорт ", te = "загрузитьизфайла " + "вебклиент вместо внешнеесоединение клиент конецобласти мобильноеприложениеклиент мобильноеприложениесервер наклиенте наклиентенасервере наклиентенасерверебезконтекста насервере насерверебезконтекста область перед после сервер толстыйклиентобычноеприложение толстыйклиентуправляемоеприложение тонкийклиент ", se = "разделительстраниц разделительстрок символтабуляции " + "ansitooem oemtoansi ввестивидсубконто ввестиперечисление ввестипериод ввестиплансчетов выбранныйплансчетов датагод датамесяц датачисло заголовоксистемы значениевстроку значениеизстроки каталогиб каталогпользователя кодсимв конгода конецпериодаби конецрассчитанногопериодаби конецстандартногоинтервала конквартала конмесяца коннедели лог лог10 максимальноеколичествосубконто названиеинтерфейса названиенабораправ назначитьвид назначитьсчет найтиссылки началопериодаби началостандартногоинтервала начгода начквартала начмесяца начнедели номерднягода номерднянедели номернеделигода обработкаожидания основнойжурналрасчетов основнойплансчетов основнойязык очиститьокносообщений периодстр получитьвремята получитьдатута получитьдокументта получитьзначенияотбора получитьпозициюта получитьпустоезначение получитьта префиксавтонумерации пропись пустоезначение разм разобратьпозициюдокумента рассчитатьрегистрына рассчитатьрегистрыпо симв создатьобъект статусвозврата стрколичествострок сформироватьпозициюдокумента счетпокоду текущеевремя типзначения типзначениястр установитьтана установитьтапо фиксшаблон шаблон " + "acos asin atan base64значение base64строка cos exp log log10 pow sin sqrt tan xmlзначение xmlстрока xmlтип xmlтипзнч активноеокно безопасныйрежим безопасныйрежимразделенияданных булево ввестидату ввестизначение ввестистроку ввестичисло возможностьчтенияxml вопрос восстановитьзначение врег выгрузитьжурналрегистрации выполнитьобработкуоповещения выполнитьпроверкуправдоступа вычислить год данныеформывзначение дата день деньгода деньнедели добавитьмесяц заблокироватьданныедляредактирования заблокироватьработупользователя завершитьработусистемы загрузитьвнешнююкомпоненту закрытьсправку записатьjson записатьxml записатьдатуjson записьжурналарегистрации заполнитьзначениясвойств запроситьразрешениепользователя запуститьприложение запуститьсистему зафиксироватьтранзакцию значениевданныеформы значениевстрокувнутр значениевфайл значениезаполнено значениеизстрокивнутр значениеизфайла изxmlтипа импортмоделиxdto имякомпьютера имяпользователя инициализироватьпредопределенныеданные информацияобошибке каталогбиблиотекимобильногоустройства каталогвременныхфайлов каталогдокументов каталогпрограммы кодироватьстроку кодлокализацииинформационнойбазы кодсимвола командасистемы конецгода конецдня конецквартала конецмесяца конецминуты конецнедели конецчаса конфигурациябазыданныхизмененадинамически конфигурацияизменена копироватьданныеформы копироватьфайл краткоепредставлениеошибки лев макс местноевремя месяц мин минута монопольныйрежим найти найтинедопустимыесимволыxml найтиокнопонавигационнойссылке найтипомеченныенаудаление найтипоссылкам найтифайлы началогода началодня началоквартала началомесяца началоминуты началонедели началочаса начатьзапросразрешенияпользователя начатьзапускприложения начатькопированиефайла начатьперемещениефайла начатьподключениевнешнейкомпоненты начатьподключениерасширенияработыскриптографией начатьподключениерасширенияработысфайлами начатьпоискфайлов начатьполучениекаталогавременныхфайлов начатьполучениекаталогадокументов начатьполучениерабочегокаталогаданныхпользователя начатьполучениефайлов начатьпомещениефайла начатьпомещениефайлов начатьсозданиедвоичныхданныхизфайла начатьсозданиекаталога начатьтранзакцию начатьудалениефайлов начатьустановкувнешнейкомпоненты начатьустановкурасширенияработыскриптографией начатьустановкурасширенияработысфайлами неделягода необходимостьзавершениясоединения номерсеансаинформационнойбазы номерсоединенияинформационнойбазы нрег нстр обновитьинтерфейс обновитьнумерациюобъектов обновитьповторноиспользуемыезначения обработкапрерыванияпользователя объединитьфайлы окр описаниеошибки оповестить оповеститьобизменении отключитьобработчикзапросанастроекклиенталицензирования отключитьобработчикожидания отключитьобработчикоповещения открытьзначение открытьиндекссправки открытьсодержаниесправки открытьсправку открытьформу открытьформумодально отменитьтранзакцию очиститьжурналрегистрации очиститьнастройкипользователя очиститьсообщения параметрыдоступа перейтипонавигационнойссылке переместитьфайл подключитьвнешнююкомпоненту подключитьобработчикзапросанастроекклиенталицензирования подключитьобработчикожидания подключитьобработчикоповещения подключитьрасширениеработыскриптографией подключитьрасширениеработысфайлами подробноепредставлениеошибки показатьвводдаты показатьвводзначения показатьвводстроки показатьвводчисла показатьвопрос показатьзначение показатьинформациюобошибке показатьнакарте показатьоповещениепользователя показатьпредупреждение полноеимяпользователя получитьcomобъект получитьxmlтип получитьадреспоместоположению получитьблокировкусеансов получитьвремязавершенияспящегосеанса получитьвремязасыпанияпассивногосеанса получитьвремяожиданияблокировкиданных получитьданныевыбора получитьдополнительныйпараметрклиенталицензирования получитьдопустимыекодылокализации получитьдопустимыечасовыепояса получитьзаголовокклиентскогоприложения получитьзаголовоксистемы получитьзначенияотборажурналарегистрации получитьидентификаторконфигурации получитьизвременногохранилища получитьимявременногофайла получитьимяклиенталицензирования получитьинформациюэкрановклиента получитьиспользованиежурналарегистрации получитьиспользованиесобытияжурналарегистрации получитькраткийзаголовокприложения получитьмакетоформления получитьмаскувсефайлы получитьмаскувсефайлыклиента получитьмаскувсефайлысервера получитьместоположениепоадресу получитьминимальнуюдлинупаролейпользователей получитьнавигационнуюссылку получитьнавигационнуюссылкуинформационнойбазы получитьобновлениеконфигурациибазыданных получитьобновлениепредопределенныхданныхинформационнойбазы получитьобщиймакет получитьобщуюформу получитьокна получитьоперативнуюотметкувремени получитьотключениебезопасногорежима получитьпараметрыфункциональныхопцийинтерфейса получитьполноеимяпредопределенногозначения получитьпредставлениянавигационныхссылок получитьпроверкусложностипаролейпользователей получитьразделительпути получитьразделительпутиклиента получитьразделительпутисервера получитьсеансыинформационнойбазы получитьскоростьклиентскогосоединения получитьсоединенияинформационнойбазы получитьсообщенияпользователю получитьсоответствиеобъектаиформы получитьсоставстандартногоинтерфейсаodata получитьструктурухранениябазыданных получитьтекущийсеансинформационнойбазы получитьфайл получитьфайлы получитьформу получитьфункциональнуюопцию получитьфункциональнуюопциюинтерфейса получитьчасовойпоясинформационнойбазы пользователиос поместитьвовременноехранилище поместитьфайл поместитьфайлы прав праводоступа предопределенноезначение представлениекодалокализации представлениепериода представлениеправа представлениеприложения представлениесобытияжурналарегистрации представлениечасовогопояса предупреждение прекратитьработусистемы привилегированныйрежим продолжитьвызов прочитатьjson прочитатьxml прочитатьдатуjson пустаястрока рабочийкаталогданныхпользователя разблокироватьданныедляредактирования разделитьфайл разорватьсоединениесвнешнимисточникомданных раскодироватьстроку рольдоступна секунда сигнал символ скопироватьжурналрегистрации смещениелетнеговремени смещениестандартноговремени соединитьбуферыдвоичныхданных создатькаталог создатьфабрикуxdto сокрл сокрлп сокрп сообщить состояние сохранитьзначение сохранитьнастройкипользователя сред стрдлина стрзаканчиваетсяна стрзаменить стрнайти стрначинаетсяс строка строкасоединенияинформационнойбазы стрполучитьстроку стрразделить стрсоединить стрсравнить стрчисловхождений стрчислострок стршаблон текущаядата текущаядатасеанса текущаяуниверсальнаядата текущаяуниверсальнаядатавмиллисекундах текущийвариантинтерфейсаклиентскогоприложения текущийвариантосновногошрифтаклиентскогоприложения текущийкодлокализации текущийрежимзапуска текущийязык текущийязыксистемы тип типзнч транзакцияактивна трег удалитьданныеинформационнойбазы удалитьизвременногохранилища удалитьобъекты удалитьфайлы универсальноевремя установитьбезопасныйрежим установитьбезопасныйрежимразделенияданных установитьблокировкусеансов установитьвнешнююкомпоненту установитьвремязавершенияспящегосеанса установитьвремязасыпанияпассивногосеанса установитьвремяожиданияблокировкиданных установитьзаголовокклиентскогоприложения установитьзаголовоксистемы установитьиспользованиежурналарегистрации установитьиспользованиесобытияжурналарегистрации установитькраткийзаголовокприложения установитьминимальнуюдлинупаролейпользователей установитьмонопольныйрежим установитьнастройкиклиенталицензирования установитьобновлениепредопределенныхданныхинформационнойбазы установитьотключениебезопасногорежима установитьпараметрыфункциональныхопцийинтерфейса установитьпривилегированныйрежим установитьпроверкусложностипаролейпользователей установитьрасширениеработыскриптографией установитьрасширениеработысфайлами установитьсоединениесвнешнимисточникомданных установитьсоответствиеобъектаиформы установитьсоставстандартногоинтерфейсаodata установитьчасовойпоясинформационнойбазы установитьчасовойпояссеанса формат цел час часовойпояс часовойпояссеанса число числопрописью этоадресвременногохранилища " + "wsссылки библиотекакартинок библиотекамакетовоформлениякомпоновкиданных библиотекастилей бизнеспроцессы внешниеисточникиданных внешниеобработки внешниеотчеты встроенныепокупки главныйинтерфейс главныйстиль документы доставляемыеуведомления журналыдокументов задачи информацияобинтернетсоединении использованиерабочейдаты историяработыпользователя константы критерииотбора метаданные обработки отображениерекламы отправкадоставляемыхуведомлений отчеты панельзадачос параметрзапуска параметрысеанса перечисления планывидоврасчета планывидовхарактеристик планыобмена планысчетов полнотекстовыйпоиск пользователиинформационнойбазы последовательности проверкавстроенныхпокупок рабочаядата расширенияконфигурации регистрыбухгалтерии регистрынакопления регистрырасчета регистрысведений регламентныезадания сериализаторxdto справочники средствагеопозиционирования средствакриптографии средствамультимедиа средстваотображениярекламы средствапочты средствателефонии фабрикаxdto файловыепотоки фоновыезадания хранилищанастроек хранилищевариантовотчетов хранилищенастроекданныхформ хранилищеобщихнастроек хранилищепользовательскихнастроекдинамическихсписков хранилищепользовательскихнастроекотчетов хранилищесистемныхнастроек ", De = "webцвета windowsцвета windowsшрифты библиотекакартинок рамкистиля символы цветастиля шрифтыстиля " + "автоматическоесохранениеданныхформывнастройках автонумерациявформе автораздвижениесерий анимациядиаграммы вариантвыравниванияэлементовизаголовков вариантуправлениявысотойтаблицы вертикальнаяпрокруткаформы вертикальноеположение вертикальноеположениеэлемента видгруппыформы виддекорацииформы виддополненияэлементаформы видизмененияданных видкнопкиформы видпереключателя видподписейкдиаграмме видполяформы видфлажка влияниеразмеранапузырекдиаграммы горизонтальноеположение горизонтальноеположениеэлемента группировкаколонок группировкаподчиненныхэлементовформы группыиэлементы действиеперетаскивания дополнительныйрежимотображения допустимыедействияперетаскивания интервалмеждуэлементамиформы использованиевывода использованиеполосыпрокрутки используемоезначениеточкибиржевойдиаграммы историявыборапривводе источникзначенийоситочекдиаграммы источникзначенияразмерапузырькадиаграммы категориягруппыкоманд максимумсерий начальноеотображениедерева начальноеотображениесписка обновлениетекстаредактирования ориентациядендрограммы ориентациядиаграммы ориентацияметокдиаграммы ориентацияметоксводнойдиаграммы ориентацияэлементаформы отображениевдиаграмме отображениевлегендедиаграммы отображениегруппыкнопок отображениезаголовкашкалыдиаграммы отображениезначенийсводнойдиаграммы отображениезначенияизмерительнойдиаграммы отображениеинтерваладиаграммыганта отображениекнопки отображениекнопкивыбора отображениеобсужденийформы отображениеобычнойгруппы отображениеотрицательныхзначенийпузырьковойдиаграммы отображениепанелипоиска отображениеподсказки отображениепредупрежденияприредактировании отображениеразметкиполосырегулирования отображениестраницформы отображениетаблицы отображениетекстазначениядиаграммыганта отображениеуправленияобычнойгруппы отображениефигурыкнопки палитрацветовдиаграммы поведениеобычнойгруппы поддержкамасштабадендрограммы поддержкамасштабадиаграммыганта поддержкамасштабасводнойдиаграммы поисквтаблицепривводе положениезаголовкаэлементаформы положениекартинкикнопкиформы положениекартинкиэлементаграфическойсхемы положениекоманднойпанелиформы положениекоманднойпанелиэлементаформы положениеопорнойточкиотрисовки положениеподписейкдиаграмме положениеподписейшкалызначенийизмерительнойдиаграммы положениесостоянияпросмотра положениестрокипоиска положениетекстасоединительнойлинии положениеуправленияпоиском положениешкалывремени порядокотображенияточекгоризонтальнойгистограммы порядоксерийвлегендедиаграммы размеркартинки расположениезаголовкашкалыдиаграммы растягиваниеповертикалидиаграммыганта режимавтоотображениясостояния режимвводастроктаблицы режимвыборанезаполненного режимвыделениядаты режимвыделениястрокитаблицы режимвыделениятаблицы режимизмененияразмера режимизменениясвязанногозначения режимиспользованиядиалогапечати режимиспользованияпараметракоманды режиммасштабированияпросмотра режимосновногоокнаклиентскогоприложения режимоткрытияокнаформы режимотображениявыделения режимотображениягеографическойсхемы режимотображениязначенийсерии режимотрисовкисеткиграфическойсхемы режимполупрозрачностидиаграммы режимпробеловдиаграммы режимразмещениянастранице режимредактированияколонки режимсглаживаниядиаграммы режимсглаживанияиндикатора режимсписказадач сквозноевыравнивание сохранениеданныхформывнастройках способзаполнениятекстазаголовкашкалыдиаграммы способопределенияограничивающегозначениядиаграммы стандартнаягруппакоманд стандартноеоформление статусоповещенияпользователя стильстрелки типаппроксимациилиниитрендадиаграммы типдиаграммы типединицышкалывремени типимпортасерийслоягеографическойсхемы типлиниигеографическойсхемы типлиниидиаграммы типмаркерагеографическойсхемы типмаркерадиаграммы типобластиоформления типорганизацииисточникаданныхгеографическойсхемы типотображениясериислоягеографическойсхемы типотображенияточечногообъектагеографическойсхемы типотображенияшкалыэлементалегендыгеографическойсхемы типпоискаобъектовгеографическойсхемы типпроекциигеографическойсхемы типразмещенияизмерений типразмещенияреквизитовизмерений типрамкиэлементауправления типсводнойдиаграммы типсвязидиаграммыганта типсоединениязначенийпосериямдиаграммы типсоединенияточекдиаграммы типсоединительнойлинии типстороныэлементаграфическойсхемы типформыотчета типшкалырадарнойдиаграммы факторлиниитрендадиаграммы фигуракнопки фигурыграфическойсхемы фиксациявтаблице форматдняшкалывремени форматкартинки ширинаподчиненныхэлементовформы " + "виддвижениябухгалтерии виддвижениянакопления видпериодарегистрарасчета видсчета видточкимаршрутабизнеспроцесса использованиеагрегатарегистранакопления использованиегруппиэлементов использованиережимапроведения использованиесреза периодичностьагрегатарегистранакопления режимавтовремя режимзаписидокумента режимпроведениядокумента " + "авторегистрацияизменений допустимыйномерсообщения отправкаэлементаданных получениеэлементаданных " + "использованиерасшифровкитабличногодокумента ориентациястраницы положениеитоговколоноксводнойтаблицы положениеитоговстроксводнойтаблицы положениетекстаотносительнокартинки расположениезаголовкагруппировкитабличногодокумента способчтениязначенийтабличногодокумента типдвустороннейпечати типзаполненияобластитабличногодокумента типкурсоровтабличногодокумента типлиниирисункатабличногодокумента типлинииячейкитабличногодокумента типнаправленияпереходатабличногодокумента типотображениявыделениятабличногодокумента типотображениялинийсводнойтаблицы типразмещениятекстатабличногодокумента типрисункатабличногодокумента типсмещениятабличногодокумента типузоратабличногодокумента типфайлатабличногодокумента точностьпечати чередованиерасположениястраниц " + "отображениевремениэлементовпланировщика " + "типфайлаформатированногодокумента " + "обходрезультатазапроса типзаписизапроса " + "видзаполнениярасшифровкипостроителяотчета типдобавленияпредставлений типизмеренияпостроителяотчета типразмещенияитогов " + "доступкфайлу режимдиалогавыборафайла режимоткрытияфайла " + "типизмеренияпостроителязапроса " + "видданныханализа методкластеризации типединицыинтервалавременианализаданных типзаполнениятаблицырезультатаанализаданных типиспользованиячисловыхзначенийанализаданных типисточникаданныхпоискаассоциаций типколонкианализаданныхдереворешений типколонкианализаданныхкластеризация типколонкианализаданныхобщаястатистика типколонкианализаданныхпоискассоциаций типколонкианализаданныхпоискпоследовательностей типколонкимоделипрогноза типмерырасстоянияанализаданных типотсеченияправилассоциации типполяанализаданных типстандартизациианализаданных типупорядочиванияправилассоциациианализаданных типупорядочиванияшаблоновпоследовательностейанализаданных типупрощениядереварешений " + "wsнаправлениепараметра вариантxpathxs вариантзаписидатыjson вариантпростоготипаxs видгруппымоделиxs видфасетаxdto действиепостроителяdom завершенностьпростоготипаxs завершенностьсоставноготипаxs завершенностьсхемыxs запрещенныеподстановкиxs исключениягруппподстановкиxs категорияиспользованияатрибутаxs категорияограниченияидентичностиxs категорияограниченияпространствименxs методнаследованияxs модельсодержимогоxs назначениетипаxml недопустимыеподстановкиxs обработкапробельныхсимволовxs обработкасодержимогоxs ограничениезначенияxs параметрыотбораузловdom переносстрокjson позициявдокументеdom пробельныесимволыxml типатрибутаxml типзначенияjson типканоническогоxml типкомпонентыxs типпроверкиxml типрезультатаdomxpath типузлаdom типузлаxml формаxml формапредставленияxs форматдатыjson экранированиесимволовjson " + "видсравнениякомпоновкиданных действиеобработкирасшифровкикомпоновкиданных направлениесортировкикомпоновкиданных расположениевложенныхэлементоврезультатакомпоновкиданных расположениеитоговкомпоновкиданных расположениегруппировкикомпоновкиданных расположениеполейгруппировкикомпоновкиданных расположениеполякомпоновкиданных расположениереквизитовкомпоновкиданных расположениересурсовкомпоновкиданных типбухгалтерскогоостаткакомпоновкиданных типвыводатекстакомпоновкиданных типгруппировкикомпоновкиданных типгруппыэлементовотборакомпоновкиданных типдополненияпериодакомпоновкиданных типзаголовкаполейкомпоновкиданных типмакетагруппировкикомпоновкиданных типмакетаобластикомпоновкиданных типостаткакомпоновкиданных типпериодакомпоновкиданных типразмещениятекстакомпоновкиданных типсвязинаборовданныхкомпоновкиданных типэлементарезультатакомпоновкиданных расположениелегендыдиаграммыкомпоновкиданных типпримененияотборакомпоновкиданных режимотображенияэлементанастройкикомпоновкиданных режимотображениянастроеккомпоновкиданных состояниеэлементанастройкикомпоновкиданных способвосстановлениянастроеккомпоновкиданных режимкомпоновкирезультата использованиепараметракомпоновкиданных автопозицияресурсовкомпоновкиданных вариантиспользованиягруппировкикомпоновкиданных расположениересурсоввдиаграммекомпоновкиданных фиксациякомпоновкиданных использованиеусловногооформлениякомпоновкиданных " + "важностьинтернетпочтовогосообщения обработкатекстаинтернетпочтовогосообщения способкодированияинтернетпочтовоговложения способкодированиянеasciiсимволовинтернетпочтовогосообщения типтекстапочтовогосообщения протоколинтернетпочты статусразборапочтовогосообщения " + "режимтранзакциизаписижурналарегистрации статустранзакциизаписижурналарегистрации уровеньжурналарегистрации " + "расположениехранилищасертификатовкриптографии режимвключениясертификатовкриптографии режимпроверкисертификатакриптографии типхранилищасертификатовкриптографии " + "кодировкаименфайловвzipфайле методсжатияzip методшифрованияzip режимвосстановленияпутейфайловzip режимобработкиподкаталоговzip режимсохраненияпутейzip уровеньсжатияzip " + "звуковоеоповещение направлениепереходакстроке позициявпотоке порядокбайтов режимблокировкиданных режимуправленияблокировкойданных сервисвстроенныхпокупок состояниефоновогозадания типподписчикадоставляемыхуведомлений уровеньиспользованиязащищенногосоединенияftp " + "направлениепорядкасхемызапроса типдополненияпериодамисхемызапроса типконтрольнойточкисхемызапроса типобъединениясхемызапроса типпараметрадоступнойтаблицысхемызапроса типсоединениясхемызапроса " + "httpметод автоиспользованиеобщегореквизита автопрефиксномеразадачи вариантвстроенногоязыка видиерархии видрегистранакопления видтаблицывнешнегоисточникаданных записьдвиженийприпроведении заполнениепоследовательностей индексирование использованиебазыпланавидоврасчета использованиебыстроговыбора использованиеобщегореквизита использованиеподчинения использованиеполнотекстовогопоиска использованиеразделяемыхданныхобщегореквизита использованиереквизита назначениеиспользованияприложения назначениерасширенияконфигурации направлениепередачи обновлениепредопределенныхданных оперативноепроведение основноепредставлениевидарасчета основноепредставлениевидахарактеристики основноепредставлениезадачи основноепредставлениепланаобмена основноепредставлениесправочника основноепредставлениесчета перемещениеграницыприпроведении периодичностьномерабизнеспроцесса периодичностьномерадокумента периодичностьрегистрарасчета периодичностьрегистрасведений повторноеиспользованиевозвращаемыхзначений полнотекстовыйпоискпривводепостроке принадлежностьобъекта проведение разделениеаутентификацииобщегореквизита разделениеданныхобщегореквизита разделениерасширенийконфигурацииобщегореквизита режимавтонумерацииобъектов режимзаписирегистра режимиспользованиямодальности режимиспользованиясинхронныхвызововрасширенийплатформыивнешнихкомпонент режимповторногоиспользованиясеансов режимполученияданныхвыборапривводепостроке режимсовместимости режимсовместимостиинтерфейса режимуправленияблокировкойданныхпоумолчанию сериикодовпланавидовхарактеристик сериикодовпланасчетов сериикодовсправочника созданиепривводе способвыбора способпоискастрокипривводепостроке способредактирования типданныхтаблицывнешнегоисточникаданных типкодапланавидоврасчета типкодасправочника типмакета типномерабизнеспроцесса типномерадокумента типномеразадачи типформы удалениедвижений " + "важностьпроблемыприменениярасширенияконфигурации вариантинтерфейсаклиентскогоприложения вариантмасштабаформклиентскогоприложения вариантосновногошрифтаклиентскогоприложения вариантстандартногопериода вариантстандартнойдатыначала видграницы видкартинки видотображенияполнотекстовогопоиска видрамки видсравнения видцвета видчисловогозначения видшрифта допустимаядлина допустимыйзнак использованиеbyteordermark использованиеметаданныхполнотекстовогопоиска источникрасширенийконфигурации клавиша кодвозвратадиалога кодировкаxbase кодировкатекста направлениепоиска направлениесортировки обновлениепредопределенныхданных обновлениеприизмененииданных отображениепанелиразделов проверказаполнения режимдиалогавопрос режимзапускаклиентскогоприложения режимокругления режимоткрытияформприложения режимполнотекстовогопоиска скоростьклиентскогосоединения состояниевнешнегоисточникаданных состояниеобновленияконфигурациибазыданных способвыборасертификатаwindows способкодированиястроки статуссообщения типвнешнейкомпоненты типплатформы типповеденияклавишиenter типэлементаинформацииовыполненииобновленияконфигурациибазыданных уровеньизоляциитранзакций хешфункция частидаты", qe = "comобъект ftpсоединение httpзапрос httpсервисответ httpсоединение wsопределения wsпрокси xbase анализданных аннотацияxs блокировкаданных буфердвоичныхданных включениеxs выражениекомпоновкиданных генераторслучайныхчисел географическаясхема географическиекоординаты графическаясхема группамоделиxs данныерасшифровкикомпоновкиданных двоичныеданные дендрограмма диаграмма диаграммаганта диалогвыборафайла диалогвыборацвета диалогвыборашрифта диалограсписаниярегламентногозадания диалогредактированиястандартногопериода диапазон документdom документhtml документацияxs доставляемоеуведомление записьdom записьfastinfoset записьhtml записьjson записьxml записьzipфайла записьданных записьтекста записьузловdom запрос защищенноесоединениеopenssl значенияполейрасшифровкикомпоновкиданных извлечениетекста импортxs интернетпочта интернетпочтовоесообщение интернетпочтовыйпрофиль интернетпрокси интернетсоединение информациядляприложенияxs использованиеатрибутаxs использованиесобытияжурналарегистрации источникдоступныхнастроеккомпоновкиданных итераторузловdom картинка квалификаторыдаты квалификаторыдвоичныхданных квалификаторыстроки квалификаторычисла компоновщикмакетакомпоновкиданных компоновщикнастроеккомпоновкиданных конструктормакетаоформлениякомпоновкиданных конструкторнастроеккомпоновкиданных конструкторформатнойстроки линия макеткомпоновкиданных макетобластикомпоновкиданных макетоформлениякомпоновкиданных маскаxs менеджеркриптографии наборсхемxml настройкикомпоновкиданных настройкисериализацииjson обработкакартинок обработкарасшифровкикомпоновкиданных обходдереваdom объявлениеатрибутаxs объявлениенотацииxs объявлениеэлементаxs описаниеиспользованиясобытиядоступжурналарегистрации описаниеиспользованиясобытияотказвдоступежурналарегистрации описаниеобработкирасшифровкикомпоновкиданных описаниепередаваемогофайла описаниетипов определениегруппыатрибутовxs определениегруппымоделиxs определениеограниченияидентичностиxs определениепростоготипаxs определениесоставноготипаxs определениетипадокументаdom определенияxpathxs отборкомпоновкиданных пакетотображаемыхдокументов параметрвыбора параметркомпоновкиданных параметрызаписиjson параметрызаписиxml параметрычтенияxml переопределениеxs планировщик полеанализаданных полекомпоновкиданных построительdom построительзапроса построительотчета построительотчетаанализаданных построительсхемxml поток потоквпамяти почта почтовоесообщение преобразованиеxsl преобразованиекканоническомуxml процессорвыводарезультатакомпоновкиданныхвколлекциюзначений процессорвыводарезультатакомпоновкиданныхвтабличныйдокумент процессоркомпоновкиданных разыменовательпространствименdom рамка расписаниерегламентногозадания расширенноеимяxml результатчтенияданных своднаядиаграмма связьпараметравыбора связьпотипу связьпотипукомпоновкиданных сериализаторxdto сертификатклиентаwindows сертификатклиентафайл сертификаткриптографии сертификатыудостоверяющихцентровwindows сертификатыудостоверяющихцентровфайл сжатиеданных системнаяинформация сообщениепользователю сочетаниеклавиш сравнениезначений стандартнаядатаначала стандартныйпериод схемаxml схемакомпоновкиданных табличныйдокумент текстовыйдокумент тестируемоеприложение типданныхxml уникальныйидентификатор фабрикаxdto файл файловыйпоток фасетдлиныxs фасетколичестваразрядовдробнойчастиxs фасетмаксимальноговключающегозначенияxs фасетмаксимальногоисключающегозначенияxs фасетмаксимальнойдлиныxs фасетминимальноговключающегозначенияxs фасетминимальногоисключающегозначенияxs фасетминимальнойдлиныxs фасетобразцаxs фасетобщегоколичестваразрядовxs фасетперечисленияxs фасетпробельныхсимволовxs фильтрузловdom форматированнаястрока форматированныйдокумент фрагментxs хешированиеданных хранилищезначения цвет чтениеfastinfoset чтениеhtml чтениеjson чтениеxml чтениеzipфайла чтениеданных чтениетекста чтениеузловdom шрифт элементрезультатакомпоновкиданных " + "comsafearray деревозначений массив соответствие списокзначений структура таблицазначений фиксированнаяструктура фиксированноесоответствие фиксированныймассив ", Xe = "null истина ложь неопределено", Ve = _.inherit(_.NUMBER_MODE), Ze = {
        className: "string",
        begin: '"|\\|',
        end: '"|$',
        contains: [{ begin: '""' }]
      }, Ge = {
        begin: "'",
        end: "'",
        excludeBegin: !0,
        excludeEnd: !0,
        contains: [
          {
            className: "number",
            begin: "\\d{4}([\\.\\\\/:-]?\\d{2}){0,5}"
          }
        ]
      }, ze = {
        match: /[;()+\-:=,]/,
        className: "punctuation",
        relevance: 0
      }, We = _.inherit(_.C_LINE_COMMENT_MODE), at = {
        className: "meta",
        begin: "#|&",
        end: "$",
        keywords: {
          $pattern: H,
          keyword: G + te
        },
        contains: [We]
      }, ct = {
        className: "symbol",
        begin: "~",
        end: ";|:",
        excludeEnd: !0
      }, et = {
        className: "function",
        variants: [
          {
            begin: "процедура|функция",
            end: "\\)",
            keywords: "процедура функция"
          },
          {
            begin: "конецпроцедуры|конецфункции",
            keywords: "конецпроцедуры конецфункции"
          }
        ],
        contains: [
          {
            begin: "\\(",
            end: "\\)",
            endsParent: !0,
            contains: [
              {
                className: "params",
                begin: H,
                end: ",",
                excludeEnd: !0,
                endsWithParent: !0,
                keywords: {
                  $pattern: H,
                  keyword: "знач",
                  literal: Xe
                },
                contains: [
                  Ve,
                  Ze,
                  Ge
                ]
              },
              We
            ]
          },
          _.inherit(_.TITLE_MODE, { begin: H })
        ]
      };
      return {
        name: "1C:Enterprise",
        case_insensitive: !0,
        keywords: {
          $pattern: H,
          keyword: G,
          built_in: se,
          class: De,
          type: qe,
          literal: Xe
        },
        contains: [
          at,
          et,
          We,
          ct,
          Ve,
          Ze,
          Ge,
          ze
        ]
      };
    }
    return _1c_1 = O, _1c_1;
  }
  var abnf_1, hasRequiredAbnf;
  function requireAbnf() {
    if (hasRequiredAbnf) return abnf_1;
    hasRequiredAbnf = 1;
    function O(_) {
      const H = _.regex, V = /^[a-zA-Z][a-zA-Z0-9-]*/, q = [
        "ALPHA",
        "BIT",
        "CHAR",
        "CR",
        "CRLF",
        "CTL",
        "DIGIT",
        "DQUOTE",
        "HEXDIG",
        "HTAB",
        "LF",
        "LWSP",
        "OCTET",
        "SP",
        "VCHAR",
        "WSP"
      ], G = _.COMMENT(/;/, /$/), J = {
        scope: "symbol",
        match: /%b[0-1]+(-[0-1]+|(\.[0-1]+)+)?/
      }, Z = {
        scope: "symbol",
        match: /%d[0-9]+(-[0-9]+|(\.[0-9]+)+)?/
      }, te = {
        scope: "symbol",
        match: /%x[0-9A-F]+(-[0-9A-F]+|(\.[0-9A-F]+)+)?/
      }, ne = {
        scope: "symbol",
        match: /%[si](?=".*")/
      }, ee = {
        scope: "attribute",
        match: H.concat(V, /(?=\s*=)/)
      };
      return {
        name: "Augmented Backus-Naur Form",
        illegal: /[!@#$^&',?+~`|:]/,
        keywords: q,
        contains: [
          {
            scope: "operator",
            match: /=\/?/
          },
          ee,
          G,
          J,
          Z,
          te,
          ne,
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE
        ]
      };
    }
    return abnf_1 = O, abnf_1;
  }
  var accesslog_1, hasRequiredAccesslog;
  function requireAccesslog() {
    if (hasRequiredAccesslog) return accesslog_1;
    hasRequiredAccesslog = 1;
    function O(_) {
      const H = _.regex, V = [
        "GET",
        "POST",
        "HEAD",
        "PUT",
        "DELETE",
        "CONNECT",
        "OPTIONS",
        "PATCH",
        "TRACE"
      ];
      return {
        name: "Apache Access Log",
        contains: [
          // IP
          {
            className: "number",
            begin: /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(:\d{1,5})?\b/,
            relevance: 5
          },
          // Other numbers
          {
            className: "number",
            begin: /\b\d+\b/,
            relevance: 0
          },
          // Requests
          {
            className: "string",
            begin: H.concat(/"/, H.either(...V)),
            end: /"/,
            keywords: V,
            illegal: /\n/,
            relevance: 5,
            contains: [
              {
                begin: /HTTP\/[12]\.\d'/,
                relevance: 5
              }
            ]
          },
          // Dates
          {
            className: "string",
            // dates must have a certain length, this prevents matching
            // simple array accesses a[123] and [] and other common patterns
            // found in other languages
            begin: /\[\d[^\]\n]{8,}\]/,
            illegal: /\n/,
            relevance: 1
          },
          {
            className: "string",
            begin: /\[/,
            end: /\]/,
            illegal: /\n/,
            relevance: 0
          },
          // User agent / relevance boost
          {
            className: "string",
            begin: /"Mozilla\/\d\.\d \(/,
            end: /"/,
            illegal: /\n/,
            relevance: 3
          },
          // Strings
          {
            className: "string",
            begin: /"/,
            end: /"/,
            illegal: /\n/,
            relevance: 0
          }
        ]
      };
    }
    return accesslog_1 = O, accesslog_1;
  }
  var actionscript_1, hasRequiredActionscript;
  function requireActionscript() {
    if (hasRequiredActionscript) return actionscript_1;
    hasRequiredActionscript = 1;
    function O(_) {
      const H = _.regex, V = /[a-zA-Z_$][a-zA-Z0-9_$]*/, q = H.concat(
        V,
        H.concat("(\\.", V, ")*")
      ), G = /([*]|[a-zA-Z_$][a-zA-Z0-9_$]*)/, J = {
        className: "rest_arg",
        begin: /[.]{3}/,
        end: V,
        relevance: 10
      };
      return {
        name: "ActionScript",
        aliases: ["as"],
        keywords: {
          keyword: [
            "as",
            "break",
            "case",
            "catch",
            "class",
            "const",
            "continue",
            "default",
            "delete",
            "do",
            "dynamic",
            "each",
            "else",
            "extends",
            "final",
            "finally",
            "for",
            "function",
            "get",
            "if",
            "implements",
            "import",
            "in",
            "include",
            "instanceof",
            "interface",
            "internal",
            "is",
            "namespace",
            "native",
            "new",
            "override",
            "package",
            "private",
            "protected",
            "public",
            "return",
            "set",
            "static",
            "super",
            "switch",
            "this",
            "throw",
            "try",
            "typeof",
            "use",
            "var",
            "void",
            "while",
            "with"
          ],
          literal: [
            "true",
            "false",
            "null",
            "undefined"
          ]
        },
        contains: [
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.C_NUMBER_MODE,
          {
            match: [
              /\bpackage/,
              /\s+/,
              q
            ],
            className: {
              1: "keyword",
              3: "title.class"
            }
          },
          {
            match: [
              /\b(?:class|interface|extends|implements)/,
              /\s+/,
              V
            ],
            className: {
              1: "keyword",
              3: "title.class"
            }
          },
          {
            className: "meta",
            beginKeywords: "import include",
            end: /;/,
            keywords: { keyword: "import include" }
          },
          {
            beginKeywords: "function",
            end: /[{;]/,
            excludeEnd: !0,
            illegal: /\S/,
            contains: [
              _.inherit(_.TITLE_MODE, { className: "title.function" }),
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                contains: [
                  _.APOS_STRING_MODE,
                  _.QUOTE_STRING_MODE,
                  _.C_LINE_COMMENT_MODE,
                  _.C_BLOCK_COMMENT_MODE,
                  J
                ]
              },
              { begin: H.concat(/:\s*/, G) }
            ]
          },
          _.METHOD_GUARD
        ],
        illegal: /#/
      };
    }
    return actionscript_1 = O, actionscript_1;
  }
  var ada_1, hasRequiredAda;
  function requireAda() {
    if (hasRequiredAda) return ada_1;
    hasRequiredAda = 1;
    function O(_) {
      const H = "\\d(_|\\d)*", V = "[eE][-+]?" + H, q = H + "(\\." + H + ")?(" + V + ")?", G = "\\w+", Z = "\\b(" + (H + "#" + G + "(\\." + G + ")?#(" + V + ")?") + "|" + q + ")", te = "[A-Za-z](_?[A-Za-z0-9.])*", ne = `[]\\{\\}%#'"`, ee = _.COMMENT("--", "$"), ae = {
        // TODO: These spaces are not required by the Ada syntax
        // however, I have yet to see handwritten Ada code where
        // someone does not put spaces around :
        begin: "\\s+:\\s+",
        end: "\\s*(:=|;|\\)|=>|$)",
        // endsWithParent: true,
        // returnBegin: true,
        illegal: ne,
        contains: [
          {
            // workaround to avoid highlighting
            // named loops and declare blocks
            beginKeywords: "loop for declare others",
            endsParent: !0
          },
          {
            // properly highlight all modifiers
            className: "keyword",
            beginKeywords: "not null constant access function procedure in out aliased exception"
          },
          {
            className: "type",
            begin: te,
            endsParent: !0,
            relevance: 0
          }
        ]
      };
      return {
        name: "Ada",
        case_insensitive: !0,
        keywords: {
          keyword: [
            "abort",
            "else",
            "new",
            "return",
            "abs",
            "elsif",
            "not",
            "reverse",
            "abstract",
            "end",
            "accept",
            "entry",
            "select",
            "access",
            "exception",
            "of",
            "separate",
            "aliased",
            "exit",
            "or",
            "some",
            "all",
            "others",
            "subtype",
            "and",
            "for",
            "out",
            "synchronized",
            "array",
            "function",
            "overriding",
            "at",
            "tagged",
            "generic",
            "package",
            "task",
            "begin",
            "goto",
            "pragma",
            "terminate",
            "body",
            "private",
            "then",
            "if",
            "procedure",
            "type",
            "case",
            "in",
            "protected",
            "constant",
            "interface",
            "is",
            "raise",
            "use",
            "declare",
            "range",
            "delay",
            "limited",
            "record",
            "when",
            "delta",
            "loop",
            "rem",
            "while",
            "digits",
            "renames",
            "with",
            "do",
            "mod",
            "requeue",
            "xor"
          ],
          literal: [
            "True",
            "False"
          ]
        },
        contains: [
          ee,
          // strings "foobar"
          {
            className: "string",
            begin: /"/,
            end: /"/,
            contains: [
              {
                begin: /""/,
                relevance: 0
              }
            ]
          },
          // characters ''
          {
            // character literals always contain one char
            className: "string",
            begin: /'.'/
          },
          {
            // number literals
            className: "number",
            begin: Z,
            relevance: 0
          },
          {
            // Attributes
            className: "symbol",
            begin: "'" + te
          },
          {
            // package definition, maybe inside generic
            className: "title",
            begin: "(\\bwith\\s+)?(\\bprivate\\s+)?\\bpackage\\s+(\\bbody\\s+)?",
            end: "(is|$)",
            keywords: "package body",
            excludeBegin: !0,
            excludeEnd: !0,
            illegal: ne
          },
          {
            // function/procedure declaration/definition
            // maybe inside generic
            begin: "(\\b(with|overriding)\\s+)?\\b(function|procedure)\\s+",
            end: "(\\bis|\\bwith|\\brenames|\\)\\s*;)",
            keywords: "overriding function procedure with is renames return",
            // we need to re-match the 'function' keyword, so that
            // the title mode below matches only exactly once
            returnBegin: !0,
            contains: [
              ee,
              {
                // name of the function/procedure
                className: "title",
                begin: "(\\bwith\\s+)?\\b(function|procedure)\\s+",
                end: "(\\(|\\s+|$)",
                excludeBegin: !0,
                excludeEnd: !0,
                illegal: ne
              },
              // 'self'
              // // parameter types
              ae,
              {
                // return type
                className: "type",
                begin: "\\breturn\\s+",
                end: "(\\s+|;|$)",
                keywords: "return",
                excludeBegin: !0,
                excludeEnd: !0,
                // we are done with functions
                endsParent: !0,
                illegal: ne
              }
            ]
          },
          {
            // new type declarations
            // maybe inside generic
            className: "type",
            begin: "\\b(sub)?type\\s+",
            end: "\\s+",
            keywords: "type",
            excludeBegin: !0,
            illegal: ne
          },
          // see comment above the definition
          ae
          // no markup
          // relevance boosters for small snippets
          // {begin: '\\s*=>\\s*'},
          // {begin: '\\s*:=\\s*'},
          // {begin: '\\s+:=\\s+'},
        ]
      };
    }
    return ada_1 = O, ada_1;
  }
  var angelscript_1, hasRequiredAngelscript;
  function requireAngelscript() {
    if (hasRequiredAngelscript) return angelscript_1;
    hasRequiredAngelscript = 1;
    function O(_) {
      const H = {
        className: "built_in",
        begin: "\\b(void|bool|int8|int16|int32|int64|int|uint8|uint16|uint32|uint64|uint|string|ref|array|double|float|auto|dictionary)"
      }, V = {
        className: "symbol",
        begin: "[a-zA-Z0-9_]+@"
      }, q = {
        className: "keyword",
        begin: "<",
        end: ">",
        contains: [
          H,
          V
        ]
      };
      return H.contains = [q], V.contains = [q], {
        name: "AngelScript",
        aliases: ["asc"],
        keywords: [
          "for",
          "in|0",
          "break",
          "continue",
          "while",
          "do|0",
          "return",
          "if",
          "else",
          "case",
          "switch",
          "namespace",
          "is",
          "cast",
          "or",
          "and",
          "xor",
          "not",
          "get|0",
          "in",
          "inout|10",
          "out",
          "override",
          "set|0",
          "private",
          "public",
          "const",
          "default|0",
          "final",
          "shared",
          "external",
          "mixin|10",
          "enum",
          "typedef",
          "funcdef",
          "this",
          "super",
          "import",
          "from",
          "interface",
          "abstract|0",
          "try",
          "catch",
          "protected",
          "explicit",
          "property"
        ],
        // avoid close detection with C# and JS
        illegal: "(^using\\s+[A-Za-z0-9_\\.]+;$|\\bfunction\\s*[^\\(])",
        contains: [
          {
            // 'strings'
            className: "string",
            begin: "'",
            end: "'",
            illegal: "\\n",
            contains: [_.BACKSLASH_ESCAPE],
            relevance: 0
          },
          // """heredoc strings"""
          {
            className: "string",
            begin: '"""',
            end: '"""'
          },
          {
            // "strings"
            className: "string",
            begin: '"',
            end: '"',
            illegal: "\\n",
            contains: [_.BACKSLASH_ESCAPE],
            relevance: 0
          },
          _.C_LINE_COMMENT_MODE,
          // single-line comments
          _.C_BLOCK_COMMENT_MODE,
          // comment blocks
          {
            // metadata
            className: "string",
            begin: "^\\s*\\[",
            end: "\\]"
          },
          {
            // interface or namespace declaration
            beginKeywords: "interface namespace",
            end: /\{/,
            illegal: "[;.\\-]",
            contains: [
              {
                // interface or namespace name
                className: "symbol",
                begin: "[a-zA-Z0-9_]+"
              }
            ]
          },
          {
            // class declaration
            beginKeywords: "class",
            end: /\{/,
            illegal: "[;.\\-]",
            contains: [
              {
                // class name
                className: "symbol",
                begin: "[a-zA-Z0-9_]+",
                contains: [
                  {
                    begin: "[:,]\\s*",
                    contains: [
                      {
                        className: "symbol",
                        begin: "[a-zA-Z0-9_]+"
                      }
                    ]
                  }
                ]
              }
            ]
          },
          H,
          // built-in types
          V,
          // object handles
          {
            // literals
            className: "literal",
            begin: "\\b(null|true|false)"
          },
          {
            // numbers
            className: "number",
            relevance: 0,
            begin: "(-?)(\\b0[xXbBoOdD][a-fA-F0-9]+|(\\b\\d+(\\.\\d*)?f?|\\.\\d+f?)([eE][-+]?\\d+f?)?)"
          }
        ]
      };
    }
    return angelscript_1 = O, angelscript_1;
  }
  var apache_1, hasRequiredApache;
  function requireApache() {
    if (hasRequiredApache) return apache_1;
    hasRequiredApache = 1;
    function O(_) {
      const H = {
        className: "number",
        begin: /[$%]\d+/
      }, V = {
        className: "number",
        begin: /\b\d+/
      }, q = {
        className: "number",
        begin: /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(:\d{1,5})?/
      }, G = {
        className: "number",
        begin: /:\d{1,5}/
      };
      return {
        name: "Apache config",
        aliases: ["apacheconf"],
        case_insensitive: !0,
        contains: [
          _.HASH_COMMENT_MODE,
          {
            className: "section",
            begin: /<\/?/,
            end: />/,
            contains: [
              q,
              G,
              // low relevance prevents us from claming XML/HTML where this rule would
              // match strings inside of XML tags
              _.inherit(_.QUOTE_STRING_MODE, { relevance: 0 })
            ]
          },
          {
            className: "attribute",
            begin: /\w+/,
            relevance: 0,
            // keywords aren’t needed for highlighting per se, they only boost relevance
            // for a very generally defined mode (starts with a word, ends with line-end
            keywords: { _: [
              "order",
              "deny",
              "allow",
              "setenv",
              "rewriterule",
              "rewriteengine",
              "rewritecond",
              "documentroot",
              "sethandler",
              "errordocument",
              "loadmodule",
              "options",
              "header",
              "listen",
              "serverroot",
              "servername"
            ] },
            starts: {
              end: /$/,
              relevance: 0,
              keywords: { literal: "on off all deny allow" },
              contains: [
                {
                  scope: "punctuation",
                  match: /\\\n/
                },
                {
                  className: "meta",
                  begin: /\s\[/,
                  end: /\]$/
                },
                {
                  className: "variable",
                  begin: /[\$%]\{/,
                  end: /\}/,
                  contains: [
                    "self",
                    H
                  ]
                },
                q,
                V,
                _.QUOTE_STRING_MODE
              ]
            }
          }
        ],
        illegal: /\S/
      };
    }
    return apache_1 = O, apache_1;
  }
  var applescript_1, hasRequiredApplescript;
  function requireApplescript() {
    if (hasRequiredApplescript) return applescript_1;
    hasRequiredApplescript = 1;
    function O(_) {
      const H = _.regex, V = _.inherit(
        _.QUOTE_STRING_MODE,
        { illegal: null }
      ), q = {
        className: "params",
        begin: /\(/,
        end: /\)/,
        contains: [
          "self",
          _.C_NUMBER_MODE,
          V
        ]
      }, G = _.COMMENT(/--/, /$/), J = _.COMMENT(
        /\(\*/,
        /\*\)/,
        { contains: [
          "self",
          // allow nesting
          G
        ] }
      ), Z = [
        G,
        J,
        _.HASH_COMMENT_MODE
      ], te = [
        /apart from/,
        /aside from/,
        /instead of/,
        /out of/,
        /greater than/,
        /isn't|(doesn't|does not) (equal|come before|come after|contain)/,
        /(greater|less) than( or equal)?/,
        /(starts?|ends|begins?) with/,
        /contained by/,
        /comes (before|after)/,
        /a (ref|reference)/,
        /POSIX (file|path)/,
        /(date|time) string/,
        /quoted form/
      ], ne = [
        /clipboard info/,
        /the clipboard/,
        /info for/,
        /list (disks|folder)/,
        /mount volume/,
        /path to/,
        /(close|open for) access/,
        /(get|set) eof/,
        /current date/,
        /do shell script/,
        /get volume settings/,
        /random number/,
        /set volume/,
        /system attribute/,
        /system info/,
        /time to GMT/,
        /(load|run|store) script/,
        /scripting components/,
        /ASCII (character|number)/,
        /localized string/,
        /choose (application|color|file|file name|folder|from list|remote application|URL)/,
        /display (alert|dialog)/
      ];
      return {
        name: "AppleScript",
        aliases: ["osascript"],
        keywords: {
          keyword: "about above after against and around as at back before beginning behind below beneath beside between but by considering contain contains continue copy div does eighth else end equal equals error every exit fifth first for fourth from front get given global if ignoring in into is it its last local me middle mod my ninth not of on onto or over prop property put ref reference repeat returning script second set seventh since sixth some tell tenth that the|0 then third through thru timeout times to transaction try until where while whose with without",
          literal: "AppleScript false linefeed return pi quote result space tab true",
          built_in: "alias application boolean class constant date file integer list number real record string text activate beep count delay launch log offset read round run say summarize write character characters contents day frontmost id item length month name|0 paragraph paragraphs rest reverse running time version weekday word words year"
        },
        contains: [
          V,
          _.C_NUMBER_MODE,
          {
            className: "built_in",
            begin: H.concat(
              /\b/,
              H.either(...ne),
              /\b/
            )
          },
          {
            className: "built_in",
            begin: /^\s*return\b/
          },
          {
            className: "literal",
            begin: /\b(text item delimiters|current application|missing value)\b/
          },
          {
            className: "keyword",
            begin: H.concat(
              /\b/,
              H.either(...te),
              /\b/
            )
          },
          {
            beginKeywords: "on",
            illegal: /[${=;\n]/,
            contains: [
              _.UNDERSCORE_TITLE_MODE,
              q
            ]
          },
          ...Z
        ],
        illegal: /\/\/|->|=>|\[\[/
      };
    }
    return applescript_1 = O, applescript_1;
  }
  var arcade_1, hasRequiredArcade;
  function requireArcade() {
    if (hasRequiredArcade) return arcade_1;
    hasRequiredArcade = 1;
    function O(_) {
      const H = _.regex, V = "[A-Za-z_][0-9A-Za-z_]*", q = {
        keyword: [
          "break",
          "case",
          "catch",
          "continue",
          "debugger",
          "do",
          "else",
          "export",
          "for",
          "function",
          "if",
          "import",
          "in",
          "new",
          "of",
          "return",
          "switch",
          "try",
          "var",
          "void",
          "while"
        ],
        literal: [
          "BackSlash",
          "DoubleQuote",
          "ForwardSlash",
          "Infinity",
          "NaN",
          "NewLine",
          "PI",
          "SingleQuote",
          "Tab",
          "TextFormatting",
          "false",
          "null",
          "true",
          "undefined"
        ],
        built_in: [
          "Abs",
          "Acos",
          "All",
          "Angle",
          "Any",
          "Area",
          "AreaGeodetic",
          "Array",
          "Asin",
          "Atan",
          "Atan2",
          "Attachments",
          "Average",
          "Back",
          "Bearing",
          "Boolean",
          "Buffer",
          "BufferGeodetic",
          "Ceil",
          "Centroid",
          "ChangeTimeZone",
          "Clip",
          "Concatenate",
          "Console",
          "Constrain",
          "Contains",
          "ConvertDirection",
          "ConvexHull",
          "Cos",
          "Count",
          "Crosses",
          "Cut",
          "Date|0",
          "DateAdd",
          "DateDiff",
          "DateOnly",
          "Day",
          "Decode",
          "DefaultValue",
          "Densify",
          "DensifyGeodetic",
          "Dictionary",
          "Difference",
          "Disjoint",
          "Distance",
          "DistanceGeodetic",
          "DistanceToCoordinate",
          "Distinct",
          "Domain",
          "DomainCode",
          "DomainName",
          "EnvelopeIntersects",
          "Equals",
          "Erase",
          "Exp",
          "Expects",
          "Extent",
          "Feature",
          "FeatureInFilter",
          "FeatureSet",
          "FeatureSetByAssociation",
          "FeatureSetById",
          "FeatureSetByName",
          "FeatureSetByPortalItem",
          "FeatureSetByRelationshipClass",
          "FeatureSetByRelationshipName",
          "Filter",
          "FilterBySubtypeCode",
          "Find",
          "First|0",
          "Floor",
          "FromCharCode",
          "FromCodePoint",
          "FromJSON",
          "Front",
          "GdbVersion",
          "Generalize",
          "Geometry",
          "GetEnvironment",
          "GetFeatureSet",
          "GetFeatureSetInfo",
          "GetUser",
          "GroupBy",
          "Guid",
          "HasKey",
          "HasValue",
          "Hash",
          "Hour",
          "IIf",
          "ISOMonth",
          "ISOWeek",
          "ISOWeekday",
          "ISOYear",
          "Includes",
          "IndexOf",
          "Insert",
          "Intersection",
          "Intersects",
          "IsEmpty",
          "IsNan",
          "IsSelfIntersecting",
          "IsSimple",
          "KnowledgeGraphByPortalItem",
          "Left|0",
          "Length",
          "Length3D",
          "LengthGeodetic",
          "Log",
          "Lower",
          "Map",
          "Max",
          "Mean",
          "MeasureToCoordinate",
          "Mid",
          "Millisecond",
          "Min",
          "Minute",
          "Month",
          "MultiPartToSinglePart",
          "Multipoint",
          "NearestCoordinate",
          "NearestVertex",
          "NextSequenceValue",
          "None",
          "Now",
          "Number",
          "Offset",
          "OrderBy",
          "Overlaps",
          "Point",
          "PointToCoordinate",
          "Polygon",
          "Polyline",
          "Pop",
          "Portal",
          "Pow",
          "Proper",
          "Push",
          "QueryGraph",
          "Random",
          "Reduce",
          "Relate",
          "Replace",
          "Resize",
          "Reverse",
          "Right|0",
          "RingIsClockwise",
          "Rotate",
          "Round",
          "Schema",
          "Second",
          "SetGeometry",
          "Simplify",
          "Sin",
          "Slice",
          "Sort",
          "Splice",
          "Split",
          "Sqrt",
          "StandardizeFilename",
          "StandardizeGuid",
          "Stdev",
          "SubtypeCode",
          "SubtypeName",
          "Subtypes",
          "Sum",
          "SymmetricDifference",
          "Tan",
          "Text",
          "Time",
          "TimeZone",
          "TimeZoneOffset",
          "Timestamp",
          "ToCharCode",
          "ToCodePoint",
          "ToHex",
          "ToLocal",
          "ToUTC",
          "Today",
          "Top|0",
          "Touches",
          "TrackAccelerationAt",
          "TrackAccelerationWindow",
          "TrackCurrentAcceleration",
          "TrackCurrentDistance",
          "TrackCurrentSpeed",
          "TrackCurrentTime",
          "TrackDistanceAt",
          "TrackDistanceWindow",
          "TrackDuration",
          "TrackFieldWindow",
          "TrackGeometryWindow",
          "TrackIndex",
          "TrackSpeedAt",
          "TrackSpeedWindow",
          "TrackStartTime",
          "TrackWindow",
          "Trim",
          "TypeOf",
          "Union",
          "Upper",
          "UrlEncode",
          "Variance",
          "Week",
          "Weekday",
          "When|0",
          "Within",
          "Year|0"
        ]
      }, G = [
        "aggregatedFeatures",
        "analytic",
        "config",
        "datapoint",
        "datastore",
        "editcontext",
        "feature",
        "featureSet",
        "feedfeature",
        "fencefeature",
        "fencenotificationtype",
        "graph",
        "join",
        "layer",
        "locationupdate",
        "map",
        "measure",
        "measure",
        "originalFeature",
        "record",
        "reference",
        "rowindex",
        "sourcedatastore",
        "sourcefeature",
        "sourcelayer",
        "target",
        "targetdatastore",
        "targetfeature",
        "targetlayer",
        "userInput",
        "value",
        "variables",
        "view"
      ], J = {
        className: "symbol",
        begin: "\\$" + H.either(...G)
      }, Z = {
        className: "number",
        variants: [
          { begin: "\\b(0[bB][01]+)" },
          { begin: "\\b(0[oO][0-7]+)" },
          { begin: _.C_NUMBER_RE }
        ],
        relevance: 0
      }, te = {
        className: "subst",
        begin: "\\$\\{",
        end: "\\}",
        keywords: q,
        contains: []
        // defined later
      }, ne = {
        className: "string",
        begin: "`",
        end: "`",
        contains: [
          _.BACKSLASH_ESCAPE,
          te
        ]
      };
      te.contains = [
        _.APOS_STRING_MODE,
        _.QUOTE_STRING_MODE,
        ne,
        Z,
        _.REGEXP_MODE
      ];
      const ee = te.contains.concat([
        _.C_BLOCK_COMMENT_MODE,
        _.C_LINE_COMMENT_MODE
      ]);
      return {
        name: "ArcGIS Arcade",
        case_insensitive: !0,
        keywords: q,
        contains: [
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          ne,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          J,
          Z,
          {
            // object attr container
            begin: /[{,]\s*/,
            relevance: 0,
            contains: [
              {
                begin: V + "\\s*:",
                returnBegin: !0,
                relevance: 0,
                contains: [
                  {
                    className: "attr",
                    begin: V,
                    relevance: 0
                  }
                ]
              }
            ]
          },
          {
            // "value" container
            begin: "(" + _.RE_STARTERS_RE + "|\\b(return)\\b)\\s*",
            keywords: "return",
            contains: [
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE,
              _.REGEXP_MODE,
              {
                className: "function",
                begin: "(\\(.*?\\)|" + V + ")\\s*=>",
                returnBegin: !0,
                end: "\\s*=>",
                contains: [
                  {
                    className: "params",
                    variants: [
                      { begin: V },
                      { begin: /\(\s*\)/ },
                      {
                        begin: /\(/,
                        end: /\)/,
                        excludeBegin: !0,
                        excludeEnd: !0,
                        keywords: q,
                        contains: ee
                      }
                    ]
                  }
                ]
              }
            ],
            relevance: 0
          },
          {
            beginKeywords: "function",
            end: /\{/,
            excludeEnd: !0,
            contains: [
              _.inherit(_.TITLE_MODE, {
                className: "title.function",
                begin: V
              }),
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                excludeBegin: !0,
                excludeEnd: !0,
                contains: ee
              }
            ],
            illegal: /\[|%/
          },
          { begin: /\$[(.]/ }
        ],
        illegal: /#(?!!)/
      };
    }
    return arcade_1 = O, arcade_1;
  }
  var arduino_1, hasRequiredArduino;
  function requireArduino() {
    if (hasRequiredArduino) return arduino_1;
    hasRequiredArduino = 1;
    function O(H) {
      const V = H.regex, q = H.COMMENT("//", "$", { contains: [{ begin: /\\\n/ }] }), G = "decltype\\(auto\\)", J = "[a-zA-Z_]\\w*::", te = "(?!struct)(" + G + "|" + V.optional(J) + "[a-zA-Z_]\\w*" + V.optional("<[^<>]+>") + ")", ne = {
        className: "type",
        begin: "\\b[a-z\\d_]*_t\\b"
      }, ae = {
        className: "string",
        variants: [
          {
            begin: '(u8?|U|L)?"',
            end: '"',
            illegal: "\\n",
            contains: [H.BACKSLASH_ESCAPE]
          },
          {
            begin: "(u8?|U|L)?'(" + "\\\\(x[0-9A-Fa-f]{2}|u[0-9A-Fa-f]{4,8}|[0-7]{3}|\\S)" + "|.)",
            end: "'",
            illegal: "."
          },
          H.END_SAME_AS_BEGIN({
            begin: /(?:u8?|U|L)?R"([^()\\ ]{0,16})\(/,
            end: /\)([^()\\ ]{0,16})"/
          })
        ]
      }, oe = {
        className: "number",
        variants: [
          // Floating-point literal.
          {
            begin: "[+-]?(?:(?:[0-9](?:'?[0-9])*\\.(?:[0-9](?:'?[0-9])*)?|\\.[0-9](?:'?[0-9])*)(?:[Ee][+-]?[0-9](?:'?[0-9])*)?|[0-9](?:'?[0-9])*[Ee][+-]?[0-9](?:'?[0-9])*|0[Xx](?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*(?:\\.(?:[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)?)?|\\.[0-9A-Fa-f](?:'?[0-9A-Fa-f])*)[Pp][+-]?[0-9](?:'?[0-9])*)(?:[Ff](?:16|32|64|128)?|(BF|bf)16|[Ll]|)"
          },
          // Integer literal.
          {
            begin: "[+-]?\\b(?:0[Bb][01](?:'?[01])*|0[Xx][0-9A-Fa-f](?:'?[0-9A-Fa-f])*|0(?:'?[0-7])*|[1-9](?:'?[0-9])*)(?:[Uu](?:LL?|ll?)|[Uu][Zz]?|(?:LL?|ll?)[Uu]?|[Zz][Uu]|)"
            // Note: there are user-defined literal suffixes too, but perhaps having the custom suffix not part of the
            // literal highlight actually makes it stand out more.
          }
        ],
        relevance: 0
      }, se = {
        className: "meta",
        begin: /#\s*[a-z]+\b/,
        end: /$/,
        keywords: { keyword: "if else elif endif define undef warning error line pragma _Pragma ifdef ifndef include" },
        contains: [
          {
            begin: /\\\n/,
            relevance: 0
          },
          H.inherit(ae, { className: "string" }),
          {
            className: "string",
            begin: /<.*?>/
          },
          q,
          H.C_BLOCK_COMMENT_MODE
        ]
      }, ce = {
        className: "title",
        begin: V.optional(J) + H.IDENT_RE,
        relevance: 0
      }, le = V.optional(J) + H.IDENT_RE + "\\s*\\(", ue = [
        "alignas",
        "alignof",
        "and",
        "and_eq",
        "asm",
        "atomic_cancel",
        "atomic_commit",
        "atomic_noexcept",
        "auto",
        "bitand",
        "bitor",
        "break",
        "case",
        "catch",
        "class",
        "co_await",
        "co_return",
        "co_yield",
        "compl",
        "concept",
        "const_cast|10",
        "consteval",
        "constexpr",
        "constinit",
        "continue",
        "decltype",
        "default",
        "delete",
        "do",
        "dynamic_cast|10",
        "else",
        "enum",
        "explicit",
        "export",
        "extern",
        "false",
        "final",
        "for",
        "friend",
        "goto",
        "if",
        "import",
        "inline",
        "module",
        "mutable",
        "namespace",
        "new",
        "noexcept",
        "not",
        "not_eq",
        "nullptr",
        "operator",
        "or",
        "or_eq",
        "override",
        "private",
        "protected",
        "public",
        "reflexpr",
        "register",
        "reinterpret_cast|10",
        "requires",
        "return",
        "sizeof",
        "static_assert",
        "static_cast|10",
        "struct",
        "switch",
        "synchronized",
        "template",
        "this",
        "thread_local",
        "throw",
        "transaction_safe",
        "transaction_safe_dynamic",
        "true",
        "try",
        "typedef",
        "typeid",
        "typename",
        "union",
        "using",
        "virtual",
        "volatile",
        "while",
        "xor",
        "xor_eq"
      ], de = [
        "bool",
        "char",
        "char16_t",
        "char32_t",
        "char8_t",
        "double",
        "float",
        "int",
        "long",
        "short",
        "void",
        "wchar_t",
        "unsigned",
        "signed",
        "const",
        "static"
      ], fe = [
        "any",
        "auto_ptr",
        "barrier",
        "binary_semaphore",
        "bitset",
        "complex",
        "condition_variable",
        "condition_variable_any",
        "counting_semaphore",
        "deque",
        "false_type",
        "flat_map",
        "flat_set",
        "future",
        "imaginary",
        "initializer_list",
        "istringstream",
        "jthread",
        "latch",
        "lock_guard",
        "multimap",
        "multiset",
        "mutex",
        "optional",
        "ostringstream",
        "packaged_task",
        "pair",
        "promise",
        "priority_queue",
        "queue",
        "recursive_mutex",
        "recursive_timed_mutex",
        "scoped_lock",
        "set",
        "shared_future",
        "shared_lock",
        "shared_mutex",
        "shared_timed_mutex",
        "shared_ptr",
        "stack",
        "string_view",
        "stringstream",
        "timed_mutex",
        "thread",
        "true_type",
        "tuple",
        "unique_lock",
        "unique_ptr",
        "unordered_map",
        "unordered_multimap",
        "unordered_multiset",
        "unordered_set",
        "variant",
        "vector",
        "weak_ptr",
        "wstring",
        "wstring_view"
      ], he = [
        "abort",
        "abs",
        "acos",
        "apply",
        "as_const",
        "asin",
        "atan",
        "atan2",
        "calloc",
        "ceil",
        "cerr",
        "cin",
        "clog",
        "cos",
        "cosh",
        "cout",
        "declval",
        "endl",
        "exchange",
        "exit",
        "exp",
        "fabs",
        "floor",
        "fmod",
        "forward",
        "fprintf",
        "fputs",
        "free",
        "frexp",
        "fscanf",
        "future",
        "invoke",
        "isalnum",
        "isalpha",
        "iscntrl",
        "isdigit",
        "isgraph",
        "islower",
        "isprint",
        "ispunct",
        "isspace",
        "isupper",
        "isxdigit",
        "labs",
        "launder",
        "ldexp",
        "log",
        "log10",
        "make_pair",
        "make_shared",
        "make_shared_for_overwrite",
        "make_tuple",
        "make_unique",
        "malloc",
        "memchr",
        "memcmp",
        "memcpy",
        "memset",
        "modf",
        "move",
        "pow",
        "printf",
        "putchar",
        "puts",
        "realloc",
        "scanf",
        "sin",
        "sinh",
        "snprintf",
        "sprintf",
        "sqrt",
        "sscanf",
        "std",
        "stderr",
        "stdin",
        "stdout",
        "strcat",
        "strchr",
        "strcmp",
        "strcpy",
        "strcspn",
        "strlen",
        "strncat",
        "strncmp",
        "strncpy",
        "strpbrk",
        "strrchr",
        "strspn",
        "strstr",
        "swap",
        "tan",
        "tanh",
        "terminate",
        "to_underlying",
        "tolower",
        "toupper",
        "vfprintf",
        "visit",
        "vprintf",
        "vsprintf"
      ], ye = {
        type: de,
        keyword: ue,
        literal: [
          "NULL",
          "false",
          "nullopt",
          "nullptr",
          "true"
        ],
        built_in: ["_Pragma"],
        _type_hints: fe
      }, ge = {
        className: "function.dispatch",
        relevance: 0,
        keywords: {
          // Only for relevance, not highlighting.
          _hint: he
        },
        begin: V.concat(
          /\b/,
          /(?!decltype)/,
          /(?!if)/,
          /(?!for)/,
          /(?!switch)/,
          /(?!while)/,
          H.IDENT_RE,
          V.lookahead(/(<[^<>]+>|)\s*\(/)
        )
      }, ve = [
        ge,
        se,
        ne,
        q,
        H.C_BLOCK_COMMENT_MODE,
        oe,
        ae
      ], be = {
        // This mode covers expression context where we can't expect a function
        // definition and shouldn't highlight anything that looks like one:
        // `return some()`, `else if()`, `(x*sum(1, 2))`
        variants: [
          {
            begin: /=/,
            end: /;/
          },
          {
            begin: /\(/,
            end: /\)/
          },
          {
            beginKeywords: "new throw return else",
            end: /;/
          }
        ],
        keywords: ye,
        contains: ve.concat([
          {
            begin: /\(/,
            end: /\)/,
            keywords: ye,
            contains: ve.concat(["self"]),
            relevance: 0
          }
        ]),
        relevance: 0
      }, Ce = {
        className: "function",
        begin: "(" + te + "[\\*&\\s]+)+" + le,
        returnBegin: !0,
        end: /[{;=]/,
        excludeEnd: !0,
        keywords: ye,
        illegal: /[^\w\s\*&:<>.]/,
        contains: [
          {
            // to prevent it from being confused as the function title
            begin: G,
            keywords: ye,
            relevance: 0
          },
          {
            begin: le,
            returnBegin: !0,
            contains: [ce],
            relevance: 0
          },
          // needed because we do not have look-behind on the below rule
          // to prevent it from grabbing the final : in a :: pair
          {
            begin: /::/,
            relevance: 0
          },
          // initializers
          {
            begin: /:/,
            endsWithParent: !0,
            contains: [
              ae,
              oe
            ]
          },
          // allow for multiple declarations, e.g.:
          // extern void f(int), g(char);
          {
            relevance: 0,
            match: /,/
          },
          {
            className: "params",
            begin: /\(/,
            end: /\)/,
            keywords: ye,
            relevance: 0,
            contains: [
              q,
              H.C_BLOCK_COMMENT_MODE,
              ae,
              oe,
              ne,
              // Count matching parentheses.
              {
                begin: /\(/,
                end: /\)/,
                keywords: ye,
                relevance: 0,
                contains: [
                  "self",
                  q,
                  H.C_BLOCK_COMMENT_MODE,
                  ae,
                  oe,
                  ne
                ]
              }
            ]
          },
          ne,
          q,
          H.C_BLOCK_COMMENT_MODE,
          se
        ]
      };
      return {
        name: "C++",
        aliases: [
          "cc",
          "c++",
          "h++",
          "hpp",
          "hh",
          "hxx",
          "cxx"
        ],
        keywords: ye,
        illegal: "</",
        classNameAliases: { "function.dispatch": "built_in" },
        contains: [].concat(
          be,
          Ce,
          ge,
          ve,
          [
            se,
            {
              // containers: ie, `vector <int> rooms (9);`
              begin: "\\b(deque|list|queue|priority_queue|pair|stack|vector|map|set|bitset|multiset|multimap|unordered_map|unordered_set|unordered_multiset|unordered_multimap|array|tuple|optional|variant|function|flat_map|flat_set)\\s*<(?!<)",
              end: ">",
              keywords: ye,
              contains: [
                "self",
                ne
              ]
            },
            {
              begin: H.IDENT_RE + "::",
              keywords: ye
            },
            {
              match: [
                // extra complexity to deal with `enum class` and `enum struct`
                /\b(?:enum(?:\s+(?:class|struct))?|class|struct|union)/,
                /\s+/,
                /\w+/
              ],
              className: {
                1: "keyword",
                3: "title.class"
              }
            }
          ]
        )
      };
    }
    function _(H) {
      const V = {
        type: [
          "boolean",
          "byte",
          "word",
          "String"
        ],
        built_in: [
          "KeyboardController",
          "MouseController",
          "SoftwareSerial",
          "EthernetServer",
          "EthernetClient",
          "LiquidCrystal",
          "RobotControl",
          "GSMVoiceCall",
          "EthernetUDP",
          "EsploraTFT",
          "HttpClient",
          "RobotMotor",
          "WiFiClient",
          "GSMScanner",
          "FileSystem",
          "Scheduler",
          "GSMServer",
          "YunClient",
          "YunServer",
          "IPAddress",
          "GSMClient",
          "GSMModem",
          "Keyboard",
          "Ethernet",
          "Console",
          "GSMBand",
          "Esplora",
          "Stepper",
          "Process",
          "WiFiUDP",
          "GSM_SMS",
          "Mailbox",
          "USBHost",
          "Firmata",
          "PImage",
          "Client",
          "Server",
          "GSMPIN",
          "FileIO",
          "Bridge",
          "Serial",
          "EEPROM",
          "Stream",
          "Mouse",
          "Audio",
          "Servo",
          "File",
          "Task",
          "GPRS",
          "WiFi",
          "Wire",
          "TFT",
          "GSM",
          "SPI",
          "SD"
        ],
        _hints: [
          "setup",
          "loop",
          "runShellCommandAsynchronously",
          "analogWriteResolution",
          "retrieveCallingNumber",
          "printFirmwareVersion",
          "analogReadResolution",
          "sendDigitalPortPair",
          "noListenOnLocalhost",
          "readJoystickButton",
          "setFirmwareVersion",
          "readJoystickSwitch",
          "scrollDisplayRight",
          "getVoiceCallStatus",
          "scrollDisplayLeft",
          "writeMicroseconds",
          "delayMicroseconds",
          "beginTransmission",
          "getSignalStrength",
          "runAsynchronously",
          "getAsynchronously",
          "listenOnLocalhost",
          "getCurrentCarrier",
          "readAccelerometer",
          "messageAvailable",
          "sendDigitalPorts",
          "lineFollowConfig",
          "countryNameWrite",
          "runShellCommand",
          "readStringUntil",
          "rewindDirectory",
          "readTemperature",
          "setClockDivider",
          "readLightSensor",
          "endTransmission",
          "analogReference",
          "detachInterrupt",
          "countryNameRead",
          "attachInterrupt",
          "encryptionType",
          "readBytesUntil",
          "robotNameWrite",
          "readMicrophone",
          "robotNameRead",
          "cityNameWrite",
          "userNameWrite",
          "readJoystickY",
          "readJoystickX",
          "mouseReleased",
          "openNextFile",
          "scanNetworks",
          "noInterrupts",
          "digitalWrite",
          "beginSpeaker",
          "mousePressed",
          "isActionDone",
          "mouseDragged",
          "displayLogos",
          "noAutoscroll",
          "addParameter",
          "remoteNumber",
          "getModifiers",
          "keyboardRead",
          "userNameRead",
          "waitContinue",
          "processInput",
          "parseCommand",
          "printVersion",
          "readNetworks",
          "writeMessage",
          "blinkVersion",
          "cityNameRead",
          "readMessage",
          "setDataMode",
          "parsePacket",
          "isListening",
          "setBitOrder",
          "beginPacket",
          "isDirectory",
          "motorsWrite",
          "drawCompass",
          "digitalRead",
          "clearScreen",
          "serialEvent",
          "rightToLeft",
          "setTextSize",
          "leftToRight",
          "requestFrom",
          "keyReleased",
          "compassRead",
          "analogWrite",
          "interrupts",
          "WiFiServer",
          "disconnect",
          "playMelody",
          "parseFloat",
          "autoscroll",
          "getPINUsed",
          "setPINUsed",
          "setTimeout",
          "sendAnalog",
          "readSlider",
          "analogRead",
          "beginWrite",
          "createChar",
          "motorsStop",
          "keyPressed",
          "tempoWrite",
          "readButton",
          "subnetMask",
          "debugPrint",
          "macAddress",
          "writeGreen",
          "randomSeed",
          "attachGPRS",
          "readString",
          "sendString",
          "remotePort",
          "releaseAll",
          "mouseMoved",
          "background",
          "getXChange",
          "getYChange",
          "answerCall",
          "getResult",
          "voiceCall",
          "endPacket",
          "constrain",
          "getSocket",
          "writeJSON",
          "getButton",
          "available",
          "connected",
          "findUntil",
          "readBytes",
          "exitValue",
          "readGreen",
          "writeBlue",
          "startLoop",
          "IPAddress",
          "isPressed",
          "sendSysex",
          "pauseMode",
          "gatewayIP",
          "setCursor",
          "getOemKey",
          "tuneWrite",
          "noDisplay",
          "loadImage",
          "switchPIN",
          "onRequest",
          "onReceive",
          "changePIN",
          "playFile",
          "noBuffer",
          "parseInt",
          "overflow",
          "checkPIN",
          "knobRead",
          "beginTFT",
          "bitClear",
          "updateIR",
          "bitWrite",
          "position",
          "writeRGB",
          "highByte",
          "writeRed",
          "setSpeed",
          "readBlue",
          "noStroke",
          "remoteIP",
          "transfer",
          "shutdown",
          "hangCall",
          "beginSMS",
          "endWrite",
          "attached",
          "maintain",
          "noCursor",
          "checkReg",
          "checkPUK",
          "shiftOut",
          "isValid",
          "shiftIn",
          "pulseIn",
          "connect",
          "println",
          "localIP",
          "pinMode",
          "getIMEI",
          "display",
          "noBlink",
          "process",
          "getBand",
          "running",
          "beginSD",
          "drawBMP",
          "lowByte",
          "setBand",
          "release",
          "bitRead",
          "prepare",
          "pointTo",
          "readRed",
          "setMode",
          "noFill",
          "remove",
          "listen",
          "stroke",
          "detach",
          "attach",
          "noTone",
          "exists",
          "buffer",
          "height",
          "bitSet",
          "circle",
          "config",
          "cursor",
          "random",
          "IRread",
          "setDNS",
          "endSMS",
          "getKey",
          "micros",
          "millis",
          "begin",
          "print",
          "write",
          "ready",
          "flush",
          "width",
          "isPIN",
          "blink",
          "clear",
          "press",
          "mkdir",
          "rmdir",
          "close",
          "point",
          "yield",
          "image",
          "BSSID",
          "click",
          "delay",
          "read",
          "text",
          "move",
          "peek",
          "beep",
          "rect",
          "line",
          "open",
          "seek",
          "fill",
          "size",
          "turn",
          "stop",
          "home",
          "find",
          "step",
          "tone",
          "sqrt",
          "RSSI",
          "SSID",
          "end",
          "bit",
          "tan",
          "cos",
          "sin",
          "pow",
          "map",
          "abs",
          "max",
          "min",
          "get",
          "run",
          "put"
        ],
        literal: [
          "DIGITAL_MESSAGE",
          "FIRMATA_STRING",
          "ANALOG_MESSAGE",
          "REPORT_DIGITAL",
          "REPORT_ANALOG",
          "INPUT_PULLUP",
          "SET_PIN_MODE",
          "INTERNAL2V56",
          "SYSTEM_RESET",
          "LED_BUILTIN",
          "INTERNAL1V1",
          "SYSEX_START",
          "INTERNAL",
          "EXTERNAL",
          "DEFAULT",
          "OUTPUT",
          "INPUT",
          "HIGH",
          "LOW"
        ]
      }, q = O(H), G = (
        /** @type {Record<string,any>} */
        q.keywords
      );
      return G.type = [
        ...G.type,
        ...V.type
      ], G.literal = [
        ...G.literal,
        ...V.literal
      ], G.built_in = [
        ...G.built_in,
        ...V.built_in
      ], G._hints = V._hints, q.name = "Arduino", q.aliases = ["ino"], q.supersetOf = "cpp", q;
    }
    return arduino_1 = _, arduino_1;
  }
  var armasm_1, hasRequiredArmasm;
  function requireArmasm() {
    if (hasRequiredArmasm) return armasm_1;
    hasRequiredArmasm = 1;
    function O(_) {
      const H = { variants: [
        _.COMMENT("^[ \\t]*(?=#)", "$", {
          relevance: 0,
          excludeBegin: !0
        }),
        _.COMMENT("[;@]", "$", { relevance: 0 }),
        _.C_LINE_COMMENT_MODE,
        _.C_BLOCK_COMMENT_MODE
      ] };
      return {
        name: "ARM Assembly",
        case_insensitive: !0,
        aliases: ["arm"],
        keywords: {
          $pattern: "\\.?" + _.IDENT_RE,
          meta: (
            // GNU preprocs
            ".2byte .4byte .align .ascii .asciz .balign .byte .code .data .else .end .endif .endm .endr .equ .err .exitm .extern .global .hword .if .ifdef .ifndef .include .irp .long .macro .rept .req .section .set .skip .space .text .word .arm .thumb .code16 .code32 .force_thumb .thumb_func .ltorg ALIAS ALIGN ARM AREA ASSERT ATTR CN CODE CODE16 CODE32 COMMON CP DATA DCB DCD DCDU DCDO DCFD DCFDU DCI DCQ DCQU DCW DCWU DN ELIF ELSE END ENDFUNC ENDIF ENDP ENTRY EQU EXPORT EXPORTAS EXTERN FIELD FILL FUNCTION GBLA GBLL GBLS GET GLOBAL IF IMPORT INCBIN INCLUDE INFO KEEP LCLA LCLL LCLS LTORG MACRO MAP MEND MEXIT NOFP OPT PRESERVE8 PROC QN READONLY RELOC REQUIRE REQUIRE8 RLIST FN ROUT SETA SETL SETS SN SPACE SUBT THUMB THUMBX TTL WHILE WEND "
          ),
          built_in: "r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 r10 r11 r12 r13 r14 r15 w0 w1 w2 w3 w4 w5 w6 w7 w8 w9 w10 w11 w12 w13 w14 w15 w16 w17 w18 w19 w20 w21 w22 w23 w24 w25 w26 w27 w28 w29 w30 x0 x1 x2 x3 x4 x5 x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18 x19 x20 x21 x22 x23 x24 x25 x26 x27 x28 x29 x30 pc lr sp ip sl sb fp a1 a2 a3 a4 v1 v2 v3 v4 v5 v6 v7 v8 f0 f1 f2 f3 f4 f5 f6 f7 p0 p1 p2 p3 p4 p5 p6 p7 p8 p9 p10 p11 p12 p13 p14 p15 c0 c1 c2 c3 c4 c5 c6 c7 c8 c9 c10 c11 c12 c13 c14 c15 q0 q1 q2 q3 q4 q5 q6 q7 q8 q9 q10 q11 q12 q13 q14 q15 cpsr_c cpsr_x cpsr_s cpsr_f cpsr_cx cpsr_cxs cpsr_xs cpsr_xsf cpsr_sf cpsr_cxsf spsr_c spsr_x spsr_s spsr_f spsr_cx spsr_cxs spsr_xs spsr_xsf spsr_sf spsr_cxsf s0 s1 s2 s3 s4 s5 s6 s7 s8 s9 s10 s11 s12 s13 s14 s15 s16 s17 s18 s19 s20 s21 s22 s23 s24 s25 s26 s27 s28 s29 s30 s31 d0 d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 d15 d16 d17 d18 d19 d20 d21 d22 d23 d24 d25 d26 d27 d28 d29 d30 d31 {PC} {VAR} {TRUE} {FALSE} {OPT} {CONFIG} {ENDIAN} {CODESIZE} {CPU} {FPU} {ARCHITECTURE} {PCSTOREOFFSET} {ARMASM_VERSION} {INTER} {ROPI} {RWPI} {SWST} {NOSWST} . @"
        },
        contains: [
          {
            className: "keyword",
            begin: "\\b(adc|(qd?|sh?|u[qh]?)?add(8|16)?|usada?8|(q|sh?|u[qh]?)?(as|sa)x|and|adrl?|sbc|rs[bc]|asr|b[lx]?|blx|bxj|cbn?z|tb[bh]|bic|bfc|bfi|[su]bfx|bkpt|cdp2?|clz|clrex|cmp|cmn|cpsi[ed]|cps|setend|dbg|dmb|dsb|eor|isb|it[te]{0,3}|lsl|lsr|ror|rrx|ldm(([id][ab])|f[ds])?|ldr((s|ex)?[bhd])?|movt?|mvn|mra|mar|mul|[us]mull|smul[bwt][bt]|smu[as]d|smmul|smmla|mla|umlaal|smlal?([wbt][bt]|d)|mls|smlsl?[ds]|smc|svc|sev|mia([bt]{2}|ph)?|mrr?c2?|mcrr2?|mrs|msr|orr|orn|pkh(tb|bt)|rbit|rev(16|sh)?|sel|[su]sat(16)?|nop|pop|push|rfe([id][ab])?|stm([id][ab])?|str(ex)?[bhd]?|(qd?)?sub|(sh?|q|u[qh]?)?sub(8|16)|[su]xt(a?h|a?b(16)?)|srs([id][ab])?|swpb?|swi|smi|tst|teq|wfe|wfi|yield)(eq|ne|cs|cc|mi|pl|vs|vc|hi|ls|ge|lt|gt|le|al|hs|lo)?[sptrx]?(?=\\s)"
            // followed by space
          },
          H,
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            begin: "'",
            end: "[^\\\\]'",
            relevance: 0
          },
          {
            className: "title",
            begin: "\\|",
            end: "\\|",
            illegal: "\\n",
            relevance: 0
          },
          {
            className: "number",
            variants: [
              {
                // hex
                begin: "[#$=]?0x[0-9a-f]+"
              },
              {
                // bin
                begin: "[#$=]?0b[01]+"
              },
              {
                // literal
                begin: "[#$=]\\d+"
              },
              {
                // bare number
                begin: "\\b\\d+"
              }
            ],
            relevance: 0
          },
          {
            className: "symbol",
            variants: [
              {
                // GNU ARM syntax
                begin: "^[ \\t]*[a-z_\\.\\$][a-z0-9_\\.\\$]+:"
              },
              {
                // ARM syntax
                begin: "^[a-z_\\.\\$][a-z0-9_\\.\\$]+"
              },
              {
                // label reference
                begin: "[=#]\\w+"
              }
            ],
            relevance: 0
          }
        ]
      };
    }
    return armasm_1 = O, armasm_1;
  }
  var asciidoc_1, hasRequiredAsciidoc;
  function requireAsciidoc() {
    if (hasRequiredAsciidoc) return asciidoc_1;
    hasRequiredAsciidoc = 1;
    function O(_) {
      const H = _.regex, V = {
        begin: "^'{3,}[ \\t]*$",
        relevance: 10
      }, q = [
        // escaped constrained formatting marks (i.e., \* \_ or \`)
        { begin: /\\[*_`]/ },
        // escaped unconstrained formatting marks (i.e., \\** \\__ or \\``)
        // must ignore until the next formatting marks
        // this rule might not be 100% compliant with Asciidoctor 2.0 but we are entering undefined behavior territory...
        { begin: /\\\\\*{2}[^\n]*?\*{2}/ },
        { begin: /\\\\_{2}[^\n]*_{2}/ },
        { begin: /\\\\`{2}[^\n]*`{2}/ },
        // guard: constrained formatting mark may not be preceded by ":", ";" or
        // "}". match these so the constrained rule doesn't see them
        { begin: /[:;}][*_`](?![*_`])/ }
      ], G = [
        // inline unconstrained strong (single line)
        {
          className: "strong",
          begin: /\*{2}([^\n]+?)\*{2}/
        },
        // inline unconstrained strong (multi-line)
        {
          className: "strong",
          begin: H.concat(
            /\*\*/,
            /((\*(?!\*)|\\[^\n]|[^*\n\\])+\n)+/,
            /(\*(?!\*)|\\[^\n]|[^*\n\\])*/,
            /\*\*/
          ),
          relevance: 0
        },
        // inline constrained strong (single line)
        {
          className: "strong",
          // must not precede or follow a word character
          begin: /\B\*(\S|\S[^\n]*?\S)\*(?!\w)/
        },
        // inline constrained strong (multi-line)
        {
          className: "strong",
          // must not precede or follow a word character
          begin: /\*[^\s]([^\n]+\n)+([^\n]+)\*/
        }
      ], J = [
        // inline unconstrained emphasis (single line)
        {
          className: "emphasis",
          begin: /_{2}([^\n]+?)_{2}/
        },
        // inline unconstrained emphasis (multi-line)
        {
          className: "emphasis",
          begin: H.concat(
            /__/,
            /((_(?!_)|\\[^\n]|[^_\n\\])+\n)+/,
            /(_(?!_)|\\[^\n]|[^_\n\\])*/,
            /__/
          ),
          relevance: 0
        },
        // inline constrained emphasis (single line)
        {
          className: "emphasis",
          // must not precede or follow a word character
          begin: /\b_(\S|\S[^\n]*?\S)_(?!\w)/
        },
        // inline constrained emphasis (multi-line)
        {
          className: "emphasis",
          // must not precede or follow a word character
          begin: /_[^\s]([^\n]+\n)+([^\n]+)_/
        },
        // inline constrained emphasis using single quote (legacy)
        {
          className: "emphasis",
          // must not follow a word character or be followed by a single quote or space
          begin: "\\B'(?!['\\s])",
          end: "(\\n{2}|')",
          // allow escaped single quote followed by word char
          contains: [
            {
              begin: "\\\\'\\w",
              relevance: 0
            }
          ],
          relevance: 0
        }
      ], Z = {
        className: "symbol",
        begin: "^(NOTE|TIP|IMPORTANT|WARNING|CAUTION):\\s+",
        relevance: 10
      }, te = {
        className: "bullet",
        begin: "^(\\*+|-+|\\.+|[^\\n]+?::)\\s+"
      };
      return {
        name: "AsciiDoc",
        aliases: ["adoc"],
        contains: [
          // block comment
          _.COMMENT(
            "^/{4,}\\n",
            "\\n/{4,}$",
            // can also be done as...
            // '^/{4,}$',
            // '^/{4,}$',
            { relevance: 10 }
          ),
          // line comment
          _.COMMENT(
            "^//",
            "$",
            { relevance: 0 }
          ),
          // title
          {
            className: "title",
            begin: "^\\.\\w.*$"
          },
          // example, admonition & sidebar blocks
          {
            begin: "^[=\\*]{4,}\\n",
            end: "\\n^[=\\*]{4,}$",
            relevance: 10
          },
          // headings
          {
            className: "section",
            relevance: 10,
            variants: [
              { begin: "^(={1,6})[ 	].+?([ 	]\\1)?$" },
              { begin: "^[^\\[\\]\\n]+?\\n[=\\-~\\^\\+]{2,}$" }
            ]
          },
          // document attributes
          {
            className: "meta",
            begin: "^:.+?:",
            end: "\\s",
            excludeEnd: !0,
            relevance: 10
          },
          // block attributes
          {
            className: "meta",
            begin: "^\\[.+?\\]$",
            relevance: 0
          },
          // quoteblocks
          {
            className: "quote",
            begin: "^_{4,}\\n",
            end: "\\n_{4,}$",
            relevance: 10
          },
          // listing and literal blocks
          {
            className: "code",
            begin: "^[\\-\\.]{4,}\\n",
            end: "\\n[\\-\\.]{4,}$",
            relevance: 10
          },
          // passthrough blocks
          {
            begin: "^\\+{4,}\\n",
            end: "\\n\\+{4,}$",
            contains: [
              {
                begin: "<",
                end: ">",
                subLanguage: "xml",
                relevance: 0
              }
            ],
            relevance: 10
          },
          te,
          Z,
          ...q,
          ...G,
          ...J,
          // inline smart quotes
          {
            className: "string",
            variants: [
              { begin: "``.+?''" },
              { begin: "`.+?'" }
            ]
          },
          // inline unconstrained emphasis
          {
            className: "code",
            begin: /`{2}/,
            end: /(\n{2}|`{2})/
          },
          // inline code snippets (TODO should get same treatment as strong and emphasis)
          {
            className: "code",
            begin: "(`.+?`|\\+.+?\\+)",
            relevance: 0
          },
          // indented literal block
          {
            className: "code",
            begin: "^[ \\t]",
            end: "$",
            relevance: 0
          },
          V,
          // images and links
          {
            begin: "(link:)?(http|https|ftp|file|irc|image:?):\\S+?\\[[^[]*?\\]",
            returnBegin: !0,
            contains: [
              {
                begin: "(link|image:?):",
                relevance: 0
              },
              {
                className: "link",
                begin: "\\w",
                end: "[^\\[]+",
                relevance: 0
              },
              {
                className: "string",
                begin: "\\[",
                end: "\\]",
                excludeBegin: !0,
                excludeEnd: !0,
                relevance: 0
              }
            ],
            relevance: 10
          }
        ]
      };
    }
    return asciidoc_1 = O, asciidoc_1;
  }
  var aspectj_1, hasRequiredAspectj;
  function requireAspectj() {
    if (hasRequiredAspectj) return aspectj_1;
    hasRequiredAspectj = 1;
    function O(_) {
      const H = _.regex, V = [
        "false",
        "synchronized",
        "int",
        "abstract",
        "float",
        "private",
        "char",
        "boolean",
        "static",
        "null",
        "if",
        "const",
        "for",
        "true",
        "while",
        "long",
        "throw",
        "strictfp",
        "finally",
        "protected",
        "import",
        "native",
        "final",
        "return",
        "void",
        "enum",
        "else",
        "extends",
        "implements",
        "break",
        "transient",
        "new",
        "catch",
        "instanceof",
        "byte",
        "super",
        "volatile",
        "case",
        "assert",
        "short",
        "package",
        "default",
        "double",
        "public",
        "try",
        "this",
        "switch",
        "continue",
        "throws",
        "privileged",
        "aspectOf",
        "adviceexecution",
        "proceed",
        "cflowbelow",
        "cflow",
        "initialization",
        "preinitialization",
        "staticinitialization",
        "withincode",
        "target",
        "within",
        "execution",
        "getWithinTypeName",
        "handler",
        "thisJoinPoint",
        "thisJoinPointStaticPart",
        "thisEnclosingJoinPointStaticPart",
        "declare",
        "parents",
        "warning",
        "error",
        "soft",
        "precedence",
        "thisAspectInstance"
      ], q = [
        "get",
        "set",
        "args",
        "call"
      ];
      return {
        name: "AspectJ",
        keywords: V,
        illegal: /<\/|#/,
        contains: [
          _.COMMENT(
            /\/\*\*/,
            /\*\//,
            {
              relevance: 0,
              contains: [
                {
                  // eat up @'s in emails to prevent them to be recognized as doctags
                  begin: /\w+@/,
                  relevance: 0
                },
                {
                  className: "doctag",
                  begin: /@[A-Za-z]+/
                }
              ]
            }
          ),
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          {
            className: "class",
            beginKeywords: "aspect",
            end: /[{;=]/,
            excludeEnd: !0,
            illegal: /[:;"\[\]]/,
            contains: [
              { beginKeywords: "extends implements pertypewithin perthis pertarget percflowbelow percflow issingleton" },
              _.UNDERSCORE_TITLE_MODE,
              {
                begin: /\([^\)]*/,
                end: /[)]+/,
                keywords: V.concat(q),
                excludeEnd: !1
              }
            ]
          },
          {
            className: "class",
            beginKeywords: "class interface",
            end: /[{;=]/,
            excludeEnd: !0,
            relevance: 0,
            keywords: "class interface",
            illegal: /[:"\[\]]/,
            contains: [
              { beginKeywords: "extends implements" },
              _.UNDERSCORE_TITLE_MODE
            ]
          },
          {
            // AspectJ Constructs
            beginKeywords: "pointcut after before around throwing returning",
            end: /[)]/,
            excludeEnd: !1,
            illegal: /["\[\]]/,
            contains: [
              {
                begin: H.concat(_.UNDERSCORE_IDENT_RE, /\s*\(/),
                returnBegin: !0,
                contains: [_.UNDERSCORE_TITLE_MODE]
              }
            ]
          },
          {
            begin: /[:]/,
            returnBegin: !0,
            end: /[{;]/,
            relevance: 0,
            excludeEnd: !1,
            keywords: V,
            illegal: /["\[\]]/,
            contains: [
              {
                begin: H.concat(_.UNDERSCORE_IDENT_RE, /\s*\(/),
                keywords: V.concat(q),
                relevance: 0
              },
              _.QUOTE_STRING_MODE
            ]
          },
          {
            // this prevents 'new Name(...), or throw ...' from being recognized as a function definition
            beginKeywords: "new throw",
            relevance: 0
          },
          {
            // the function class is a bit different for AspectJ compared to the Java language
            className: "function",
            begin: /\w+ +\w+(\.\w+)?\s*\([^\)]*\)\s*((throws)[\w\s,]+)?[\{;]/,
            returnBegin: !0,
            end: /[{;=]/,
            keywords: V,
            excludeEnd: !0,
            contains: [
              {
                begin: H.concat(_.UNDERSCORE_IDENT_RE, /\s*\(/),
                returnBegin: !0,
                relevance: 0,
                contains: [_.UNDERSCORE_TITLE_MODE]
              },
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                relevance: 0,
                keywords: V,
                contains: [
                  _.APOS_STRING_MODE,
                  _.QUOTE_STRING_MODE,
                  _.C_NUMBER_MODE,
                  _.C_BLOCK_COMMENT_MODE
                ]
              },
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          _.C_NUMBER_MODE,
          {
            // annotation is also used in this language
            className: "meta",
            begin: /@[A-Za-z]+/
          }
        ]
      };
    }
    return aspectj_1 = O, aspectj_1;
  }
  var autohotkey_1, hasRequiredAutohotkey;
  function requireAutohotkey() {
    if (hasRequiredAutohotkey) return autohotkey_1;
    hasRequiredAutohotkey = 1;
    function O(_) {
      const H = { begin: "`[\\s\\S]" };
      return {
        name: "AutoHotkey",
        case_insensitive: !0,
        aliases: ["ahk"],
        keywords: {
          keyword: "Break Continue Critical Exit ExitApp Gosub Goto New OnExit Pause return SetBatchLines SetTimer Suspend Thread Throw Until ahk_id ahk_class ahk_pid ahk_exe ahk_group",
          literal: "true false NOT AND OR",
          built_in: "ComSpec Clipboard ClipboardAll ErrorLevel"
        },
        contains: [
          H,
          _.inherit(_.QUOTE_STRING_MODE, { contains: [H] }),
          _.COMMENT(";", "$", { relevance: 0 }),
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "number",
            begin: _.NUMBER_RE,
            relevance: 0
          },
          {
            // subst would be the most accurate however fails the point of
            // highlighting. variable is comparably the most accurate that actually
            // has some effect
            className: "variable",
            begin: "%[a-zA-Z0-9#_$@]+%"
          },
          {
            className: "built_in",
            begin: "^\\s*\\w+\\s*(,|%)"
            // I don't really know if this is totally relevant
          },
          {
            // symbol would be most accurate however is highlighted just like
            // built_in and that makes up a lot of AutoHotkey code meaning that it
            // would fail to highlight anything
            className: "title",
            variants: [
              { begin: '^[^\\n";]+::(?!=)' },
              {
                begin: '^[^\\n";]+:(?!=)',
                // zero relevance as it catches a lot of things
                // followed by a single ':' in many languages
                relevance: 0
              }
            ]
          },
          {
            className: "meta",
            begin: "^\\s*#\\w+",
            end: "$",
            relevance: 0
          },
          {
            className: "built_in",
            begin: "A_[a-zA-Z0-9]+"
          },
          {
            // consecutive commas, not for highlighting but just for relevance
            begin: ",\\s*,"
          }
        ]
      };
    }
    return autohotkey_1 = O, autohotkey_1;
  }
  var autoit_1, hasRequiredAutoit;
  function requireAutoit() {
    if (hasRequiredAutoit) return autoit_1;
    hasRequiredAutoit = 1;
    function O(_) {
      const H = "ByRef Case Const ContinueCase ContinueLoop Dim Do Else ElseIf EndFunc EndIf EndSelect EndSwitch EndWith Enum Exit ExitLoop For Func Global If In Local Next ReDim Return Select Static Step Switch Then To Until Volatile WEnd While With", V = [
        "EndRegion",
        "forcedef",
        "forceref",
        "ignorefunc",
        "include",
        "include-once",
        "NoTrayIcon",
        "OnAutoItStartRegister",
        "pragma",
        "Region",
        "RequireAdmin",
        "Tidy_Off",
        "Tidy_On",
        "Tidy_Parameters"
      ], q = "True False And Null Not Or Default", G = "Abs ACos AdlibRegister AdlibUnRegister Asc AscW ASin Assign ATan AutoItSetOption AutoItWinGetTitle AutoItWinSetTitle Beep Binary BinaryLen BinaryMid BinaryToString BitAND BitNOT BitOR BitRotate BitShift BitXOR BlockInput Break Call CDTray Ceiling Chr ChrW ClipGet ClipPut ConsoleRead ConsoleWrite ConsoleWriteError ControlClick ControlCommand ControlDisable ControlEnable ControlFocus ControlGetFocus ControlGetHandle ControlGetPos ControlGetText ControlHide ControlListView ControlMove ControlSend ControlSetText ControlShow ControlTreeView Cos Dec DirCopy DirCreate DirGetSize DirMove DirRemove DllCall DllCallAddress DllCallbackFree DllCallbackGetPtr DllCallbackRegister DllClose DllOpen DllStructCreate DllStructGetData DllStructGetPtr DllStructGetSize DllStructSetData DriveGetDrive DriveGetFileSystem DriveGetLabel DriveGetSerial DriveGetType DriveMapAdd DriveMapDel DriveMapGet DriveSetLabel DriveSpaceFree DriveSpaceTotal DriveStatus EnvGet EnvSet EnvUpdate Eval Execute Exp FileChangeDir FileClose FileCopy FileCreateNTFSLink FileCreateShortcut FileDelete FileExists FileFindFirstFile FileFindNextFile FileFlush FileGetAttrib FileGetEncoding FileGetLongName FileGetPos FileGetShortcut FileGetShortName FileGetSize FileGetTime FileGetVersion FileInstall FileMove FileOpen FileOpenDialog FileRead FileReadLine FileReadToArray FileRecycle FileRecycleEmpty FileSaveDialog FileSelectFolder FileSetAttrib FileSetEnd FileSetPos FileSetTime FileWrite FileWriteLine Floor FtpSetProxy FuncName GUICreate GUICtrlCreateAvi GUICtrlCreateButton GUICtrlCreateCheckbox GUICtrlCreateCombo GUICtrlCreateContextMenu GUICtrlCreateDate GUICtrlCreateDummy GUICtrlCreateEdit GUICtrlCreateGraphic GUICtrlCreateGroup GUICtrlCreateIcon GUICtrlCreateInput GUICtrlCreateLabel GUICtrlCreateList GUICtrlCreateListView GUICtrlCreateListViewItem GUICtrlCreateMenu GUICtrlCreateMenuItem GUICtrlCreateMonthCal GUICtrlCreateObj GUICtrlCreatePic GUICtrlCreateProgress GUICtrlCreateRadio GUICtrlCreateSlider GUICtrlCreateTab GUICtrlCreateTabItem GUICtrlCreateTreeView GUICtrlCreateTreeViewItem GUICtrlCreateUpdown GUICtrlDelete GUICtrlGetHandle GUICtrlGetState GUICtrlRead GUICtrlRecvMsg GUICtrlRegisterListViewSort GUICtrlSendMsg GUICtrlSendToDummy GUICtrlSetBkColor GUICtrlSetColor GUICtrlSetCursor GUICtrlSetData GUICtrlSetDefBkColor GUICtrlSetDefColor GUICtrlSetFont GUICtrlSetGraphic GUICtrlSetImage GUICtrlSetLimit GUICtrlSetOnEvent GUICtrlSetPos GUICtrlSetResizing GUICtrlSetState GUICtrlSetStyle GUICtrlSetTip GUIDelete GUIGetCursorInfo GUIGetMsg GUIGetStyle GUIRegisterMsg GUISetAccelerators GUISetBkColor GUISetCoord GUISetCursor GUISetFont GUISetHelp GUISetIcon GUISetOnEvent GUISetState GUISetStyle GUIStartGroup GUISwitch Hex HotKeySet HttpSetProxy HttpSetUserAgent HWnd InetClose InetGet InetGetInfo InetGetSize InetRead IniDelete IniRead IniReadSection IniReadSectionNames IniRenameSection IniWrite IniWriteSection InputBox Int IsAdmin IsArray IsBinary IsBool IsDeclared IsDllStruct IsFloat IsFunc IsHWnd IsInt IsKeyword IsNumber IsObj IsPtr IsString Log MemGetStats Mod MouseClick MouseClickDrag MouseDown MouseGetCursor MouseGetPos MouseMove MouseUp MouseWheel MsgBox Number ObjCreate ObjCreateInterface ObjEvent ObjGet ObjName OnAutoItExitRegister OnAutoItExitUnRegister Ping PixelChecksum PixelGetColor PixelSearch ProcessClose ProcessExists ProcessGetStats ProcessList ProcessSetPriority ProcessWait ProcessWaitClose ProgressOff ProgressOn ProgressSet Ptr Random RegDelete RegEnumKey RegEnumVal RegRead RegWrite Round Run RunAs RunAsWait RunWait Send SendKeepActive SetError SetExtended ShellExecute ShellExecuteWait Shutdown Sin Sleep SoundPlay SoundSetWaveVolume SplashImageOn SplashOff SplashTextOn Sqrt SRandom StatusbarGetText StderrRead StdinWrite StdioClose StdoutRead String StringAddCR StringCompare StringFormat StringFromASCIIArray StringInStr StringIsAlNum StringIsAlpha StringIsASCII StringIsDigit StringIsFloat StringIsInt StringIsLower StringIsSpace StringIsUpper StringIsXDigit StringLeft StringLen StringLower StringMid StringRegExp StringRegExpReplace StringReplace StringReverse StringRight StringSplit StringStripCR StringStripWS StringToASCIIArray StringToBinary StringTrimLeft StringTrimRight StringUpper Tan TCPAccept TCPCloseSocket TCPConnect TCPListen TCPNameToIP TCPRecv TCPSend TCPShutdown, UDPShutdown TCPStartup, UDPStartup TimerDiff TimerInit ToolTip TrayCreateItem TrayCreateMenu TrayGetMsg TrayItemDelete TrayItemGetHandle TrayItemGetState TrayItemGetText TrayItemSetOnEvent TrayItemSetState TrayItemSetText TraySetClick TraySetIcon TraySetOnEvent TraySetPauseIcon TraySetState TraySetToolTip TrayTip UBound UDPBind UDPCloseSocket UDPOpen UDPRecv UDPSend VarGetType WinActivate WinActive WinClose WinExists WinFlash WinGetCaretPos WinGetClassList WinGetClientSize WinGetHandle WinGetPos WinGetProcess WinGetState WinGetText WinGetTitle WinKill WinList WinMenuSelectItem WinMinimizeAll WinMinimizeAllUndo WinMove WinSetOnTop WinSetState WinSetTitle WinSetTrans WinWait WinWaitActive WinWaitClose WinWaitNotActive", J = { variants: [
        _.COMMENT(";", "$", { relevance: 0 }),
        _.COMMENT("#cs", "#ce"),
        _.COMMENT("#comments-start", "#comments-end")
      ] }, Z = { begin: "\\$[A-z0-9_]+" }, te = {
        className: "string",
        variants: [
          {
            begin: /"/,
            end: /"/,
            contains: [
              {
                begin: /""/,
                relevance: 0
              }
            ]
          },
          {
            begin: /'/,
            end: /'/,
            contains: [
              {
                begin: /''/,
                relevance: 0
              }
            ]
          }
        ]
      }, ne = { variants: [
        _.BINARY_NUMBER_MODE,
        _.C_NUMBER_MODE
      ] }, ee = {
        className: "meta",
        begin: "#",
        end: "$",
        keywords: { keyword: V },
        contains: [
          {
            begin: /\\\n/,
            relevance: 0
          },
          {
            beginKeywords: "include",
            keywords: { keyword: "include" },
            end: "$",
            contains: [
              te,
              {
                className: "string",
                variants: [
                  {
                    begin: "<",
                    end: ">"
                  },
                  {
                    begin: /"/,
                    end: /"/,
                    contains: [
                      {
                        begin: /""/,
                        relevance: 0
                      }
                    ]
                  },
                  {
                    begin: /'/,
                    end: /'/,
                    contains: [
                      {
                        begin: /''/,
                        relevance: 0
                      }
                    ]
                  }
                ]
              }
            ]
          },
          te,
          J
        ]
      }, ae = {
        className: "symbol",
        // begin: '@',
        // end: '$',
        // keywords: 'AppDataCommonDir AppDataDir AutoItExe AutoItPID AutoItVersion AutoItX64 COM_EventObj CommonFilesDir Compiled ComputerName ComSpec CPUArch CR CRLF DesktopCommonDir DesktopDepth DesktopDir DesktopHeight DesktopRefresh DesktopWidth DocumentsCommonDir error exitCode exitMethod extended FavoritesCommonDir FavoritesDir GUI_CtrlHandle GUI_CtrlId GUI_DragFile GUI_DragId GUI_DropId GUI_WinHandle HomeDrive HomePath HomeShare HotKeyPressed HOUR IPAddress1 IPAddress2 IPAddress3 IPAddress4 KBLayout LF LocalAppDataDir LogonDNSDomain LogonDomain LogonServer MDAY MIN MON MSEC MUILang MyDocumentsDir NumParams OSArch OSBuild OSLang OSServicePack OSType OSVersion ProgramFilesDir ProgramsCommonDir ProgramsDir ScriptDir ScriptFullPath ScriptLineNumber ScriptName SEC StartMenuCommonDir StartMenuDir StartupCommonDir StartupDir SW_DISABLE SW_ENABLE SW_HIDE SW_LOCK SW_MAXIMIZE SW_MINIMIZE SW_RESTORE SW_SHOW SW_SHOWDEFAULT SW_SHOWMAXIMIZED SW_SHOWMINIMIZED SW_SHOWMINNOACTIVE SW_SHOWNA SW_SHOWNOACTIVATE SW_SHOWNORMAL SW_UNLOCK SystemDir TAB TempDir TRAY_ID TrayIconFlashing TrayIconVisible UserName UserProfileDir WDAY WindowsDir WorkingDir YDAY YEAR',
        // relevance: 5
        begin: "@[A-z0-9_]+"
      }, oe = {
        beginKeywords: "Func",
        end: "$",
        illegal: "\\$|\\[|%",
        contains: [
          _.inherit(_.UNDERSCORE_TITLE_MODE, { className: "title.function" }),
          {
            className: "params",
            begin: "\\(",
            end: "\\)",
            contains: [
              Z,
              te,
              ne
            ]
          }
        ]
      };
      return {
        name: "AutoIt",
        case_insensitive: !0,
        illegal: /\/\*/,
        keywords: {
          keyword: H,
          built_in: G,
          literal: q
        },
        contains: [
          J,
          Z,
          te,
          ne,
          ee,
          ae,
          oe
        ]
      };
    }
    return autoit_1 = O, autoit_1;
  }
  var avrasm_1, hasRequiredAvrasm;
  function requireAvrasm() {
    if (hasRequiredAvrasm) return avrasm_1;
    hasRequiredAvrasm = 1;
    function O(_) {
      return {
        name: "AVR Assembly",
        case_insensitive: !0,
        keywords: {
          $pattern: "\\.?" + _.IDENT_RE,
          keyword: (
            /* mnemonic */
            "adc add adiw and andi asr bclr bld brbc brbs brcc brcs break breq brge brhc brhs brid brie brlo brlt brmi brne brpl brsh brtc brts brvc brvs bset bst call cbi cbr clc clh cli cln clr cls clt clv clz com cp cpc cpi cpse dec eicall eijmp elpm eor fmul fmuls fmulsu icall ijmp in inc jmp ld ldd ldi lds lpm lsl lsr mov movw mul muls mulsu neg nop or ori out pop push rcall ret reti rjmp rol ror sbc sbr sbrc sbrs sec seh sbi sbci sbic sbis sbiw sei sen ser ses set sev sez sleep spm st std sts sub subi swap tst wdr"
          ),
          built_in: (
            /* general purpose registers */
            "r0 r1 r2 r3 r4 r5 r6 r7 r8 r9 r10 r11 r12 r13 r14 r15 r16 r17 r18 r19 r20 r21 r22 r23 r24 r25 r26 r27 r28 r29 r30 r31 x|0 xh xl y|0 yh yl z|0 zh zl ucsr1c udr1 ucsr1a ucsr1b ubrr1l ubrr1h ucsr0c ubrr0h tccr3c tccr3a tccr3b tcnt3h tcnt3l ocr3ah ocr3al ocr3bh ocr3bl ocr3ch ocr3cl icr3h icr3l etimsk etifr tccr1c ocr1ch ocr1cl twcr twdr twar twsr twbr osccal xmcra xmcrb eicra spmcsr spmcr portg ddrg ping portf ddrf sreg sph spl xdiv rampz eicrb eimsk gimsk gicr eifr gifr timsk tifr mcucr mcucsr tccr0 tcnt0 ocr0 assr tccr1a tccr1b tcnt1h tcnt1l ocr1ah ocr1al ocr1bh ocr1bl icr1h icr1l tccr2 tcnt2 ocr2 ocdr wdtcr sfior eearh eearl eedr eecr porta ddra pina portb ddrb pinb portc ddrc pinc portd ddrd pind spdr spsr spcr udr0 ucsr0a ucsr0b ubrr0l acsr admux adcsr adch adcl porte ddre pine pinf"
          ),
          meta: ".byte .cseg .db .def .device .dseg .dw .endmacro .equ .eseg .exit .include .list .listmac .macro .nolist .org .set"
        },
        contains: [
          _.C_BLOCK_COMMENT_MODE,
          _.COMMENT(
            ";",
            "$",
            { relevance: 0 }
          ),
          _.C_NUMBER_MODE,
          // 0x..., decimal, float
          _.BINARY_NUMBER_MODE,
          // 0b...
          {
            className: "number",
            begin: "\\b(\\$[a-zA-Z0-9]+|0o[0-7]+)"
            // $..., 0o...
          },
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            begin: "'",
            end: "[^\\\\]'",
            illegal: "[^\\\\][^']"
          },
          {
            className: "symbol",
            begin: "^[A-Za-z0-9_.$]+:"
          },
          {
            className: "meta",
            begin: "#",
            end: "$"
          },
          {
            // substitution within a macro
            className: "subst",
            begin: "@[0-9]+"
          }
        ]
      };
    }
    return avrasm_1 = O, avrasm_1;
  }
  var awk_1, hasRequiredAwk;
  function requireAwk() {
    if (hasRequiredAwk) return awk_1;
    hasRequiredAwk = 1;
    function O(_) {
      const H = {
        className: "variable",
        variants: [
          { begin: /\$[\w\d#@][\w\d_]*/ },
          { begin: /\$\{(.*?)\}/ }
        ]
      }, V = "BEGIN END if else while do for in break continue delete next nextfile function func exit|10", q = {
        className: "string",
        contains: [_.BACKSLASH_ESCAPE],
        variants: [
          {
            begin: /(u|b)?r?'''/,
            end: /'''/,
            relevance: 10
          },
          {
            begin: /(u|b)?r?"""/,
            end: /"""/,
            relevance: 10
          },
          {
            begin: /(u|r|ur)'/,
            end: /'/,
            relevance: 10
          },
          {
            begin: /(u|r|ur)"/,
            end: /"/,
            relevance: 10
          },
          {
            begin: /(b|br)'/,
            end: /'/
          },
          {
            begin: /(b|br)"/,
            end: /"/
          },
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE
        ]
      };
      return {
        name: "Awk",
        keywords: { keyword: V },
        contains: [
          H,
          q,
          _.REGEXP_MODE,
          _.HASH_COMMENT_MODE,
          _.NUMBER_MODE
        ]
      };
    }
    return awk_1 = O, awk_1;
  }
  var axapta_1, hasRequiredAxapta;
  function requireAxapta() {
    if (hasRequiredAxapta) return axapta_1;
    hasRequiredAxapta = 1;
    function O(_) {
      const H = _.UNDERSCORE_IDENT_RE, J = {
        keyword: [
          "abstract",
          "as",
          "asc",
          "avg",
          "break",
          "breakpoint",
          "by",
          "byref",
          "case",
          "catch",
          "changecompany",
          "class",
          "client",
          "client",
          "common",
          "const",
          "continue",
          "count",
          "crosscompany",
          "delegate",
          "delete_from",
          "desc",
          "display",
          "div",
          "do",
          "edit",
          "else",
          "eventhandler",
          "exists",
          "extends",
          "final",
          "finally",
          "firstfast",
          "firstonly",
          "firstonly1",
          "firstonly10",
          "firstonly100",
          "firstonly1000",
          "flush",
          "for",
          "forceliterals",
          "forcenestedloop",
          "forceplaceholders",
          "forceselectorder",
          "forupdate",
          "from",
          "generateonly",
          "group",
          "hint",
          "if",
          "implements",
          "in",
          "index",
          "insert_recordset",
          "interface",
          "internal",
          "is",
          "join",
          "like",
          "maxof",
          "minof",
          "mod",
          "namespace",
          "new",
          "next",
          "nofetch",
          "notexists",
          "optimisticlock",
          "order",
          "outer",
          "pessimisticlock",
          "print",
          "private",
          "protected",
          "public",
          "readonly",
          "repeatableread",
          "retry",
          "return",
          "reverse",
          "select",
          "server",
          "setting",
          "static",
          "sum",
          "super",
          "switch",
          "this",
          "throw",
          "try",
          "ttsabort",
          "ttsbegin",
          "ttscommit",
          "unchecked",
          "update_recordset",
          "using",
          "validtimestate",
          "void",
          "where",
          "while"
        ],
        built_in: [
          "anytype",
          "boolean",
          "byte",
          "char",
          "container",
          "date",
          "double",
          "enum",
          "guid",
          "int",
          "int64",
          "long",
          "real",
          "short",
          "str",
          "utcdatetime",
          "var"
        ],
        literal: [
          "default",
          "false",
          "null",
          "true"
        ]
      }, Z = {
        variants: [
          { match: [
            /(class|interface)\s+/,
            H,
            /\s+(extends|implements)\s+/,
            H
          ] },
          { match: [
            /class\s+/,
            H
          ] }
        ],
        scope: {
          2: "title.class",
          4: "title.class.inherited"
        },
        keywords: J
      };
      return {
        name: "X++",
        aliases: ["x++"],
        keywords: J,
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE,
          {
            className: "meta",
            begin: "#",
            end: "$"
          },
          Z
        ]
      };
    }
    return axapta_1 = O, axapta_1;
  }
  var basic_1, hasRequiredBasic;
  function requireBasic() {
    if (hasRequiredBasic) return basic_1;
    hasRequiredBasic = 1;
    function O(_) {
      return {
        name: "BASIC",
        case_insensitive: !0,
        illegal: "^.",
        // Support explicitly typed variables that end with $%! or #.
        keywords: {
          $pattern: "[a-zA-Z][a-zA-Z0-9_$%!#]*",
          keyword: [
            "ABS",
            "ASC",
            "AND",
            "ATN",
            "AUTO|0",
            "BEEP",
            "BLOAD|10",
            "BSAVE|10",
            "CALL",
            "CALLS",
            "CDBL",
            "CHAIN",
            "CHDIR",
            "CHR$|10",
            "CINT",
            "CIRCLE",
            "CLEAR",
            "CLOSE",
            "CLS",
            "COLOR",
            "COM",
            "COMMON",
            "CONT",
            "COS",
            "CSNG",
            "CSRLIN",
            "CVD",
            "CVI",
            "CVS",
            "DATA",
            "DATE$",
            "DEFDBL",
            "DEFINT",
            "DEFSNG",
            "DEFSTR",
            "DEF|0",
            "SEG",
            "USR",
            "DELETE",
            "DIM",
            "DRAW",
            "EDIT",
            "END",
            "ENVIRON",
            "ENVIRON$",
            "EOF",
            "EQV",
            "ERASE",
            "ERDEV",
            "ERDEV$",
            "ERL",
            "ERR",
            "ERROR",
            "EXP",
            "FIELD",
            "FILES",
            "FIX",
            "FOR|0",
            "FRE",
            "GET",
            "GOSUB|10",
            "GOTO",
            "HEX$",
            "IF",
            "THEN",
            "ELSE|0",
            "INKEY$",
            "INP",
            "INPUT",
            "INPUT#",
            "INPUT$",
            "INSTR",
            "IMP",
            "INT",
            "IOCTL",
            "IOCTL$",
            "KEY",
            "ON",
            "OFF",
            "LIST",
            "KILL",
            "LEFT$",
            "LEN",
            "LET",
            "LINE",
            "LLIST",
            "LOAD",
            "LOC",
            "LOCATE",
            "LOF",
            "LOG",
            "LPRINT",
            "USING",
            "LSET",
            "MERGE",
            "MID$",
            "MKDIR",
            "MKD$",
            "MKI$",
            "MKS$",
            "MOD",
            "NAME",
            "NEW",
            "NEXT",
            "NOISE",
            "NOT",
            "OCT$",
            "ON",
            "OR",
            "PEN",
            "PLAY",
            "STRIG",
            "OPEN",
            "OPTION",
            "BASE",
            "OUT",
            "PAINT",
            "PALETTE",
            "PCOPY",
            "PEEK",
            "PMAP",
            "POINT",
            "POKE",
            "POS",
            "PRINT",
            "PRINT]",
            "PSET",
            "PRESET",
            "PUT",
            "RANDOMIZE",
            "READ",
            "REM",
            "RENUM",
            "RESET|0",
            "RESTORE",
            "RESUME",
            "RETURN|0",
            "RIGHT$",
            "RMDIR",
            "RND",
            "RSET",
            "RUN",
            "SAVE",
            "SCREEN",
            "SGN",
            "SHELL",
            "SIN",
            "SOUND",
            "SPACE$",
            "SPC",
            "SQR",
            "STEP",
            "STICK",
            "STOP",
            "STR$",
            "STRING$",
            "SWAP",
            "SYSTEM",
            "TAB",
            "TAN",
            "TIME$",
            "TIMER",
            "TROFF",
            "TRON",
            "TO",
            "USR",
            "VAL",
            "VARPTR",
            "VARPTR$",
            "VIEW",
            "WAIT",
            "WHILE",
            "WEND",
            "WIDTH",
            "WINDOW",
            "WRITE",
            "XOR"
          ]
        },
        contains: [
          {
            // Match strings that start with " and end with " or a line break
            scope: "string",
            begin: /"/,
            end: /"|$/,
            contains: [_.BACKSLASH_ESCAPE]
          },
          _.COMMENT("REM", "$", { relevance: 10 }),
          _.COMMENT("'", "$", { relevance: 0 }),
          {
            // Match line numbers
            className: "symbol",
            begin: "^[0-9]+ ",
            relevance: 10
          },
          {
            // Match typed numeric constants (1000, 12.34!, 1.2e5, 1.5#, 1.2D2)
            className: "number",
            begin: "\\b\\d+(\\.\\d+)?([edED]\\d+)?[#!]?",
            relevance: 0
          },
          {
            // Match hexadecimal numbers (&Hxxxx)
            className: "number",
            begin: "(&[hH][0-9a-fA-F]{1,4})"
          },
          {
            // Match octal numbers (&Oxxxxxx)
            className: "number",
            begin: "(&[oO][0-7]{1,6})"
          }
        ]
      };
    }
    return basic_1 = O, basic_1;
  }
  var bnf_1, hasRequiredBnf;
  function requireBnf() {
    if (hasRequiredBnf) return bnf_1;
    hasRequiredBnf = 1;
    function O(_) {
      return {
        name: "Backus–Naur Form",
        contains: [
          // Attribute
          {
            className: "attribute",
            begin: /</,
            end: />/
          },
          // Specific
          {
            begin: /::=/,
            end: /$/,
            contains: [
              {
                begin: /</,
                end: />/
              },
              // Common
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE,
              _.APOS_STRING_MODE,
              _.QUOTE_STRING_MODE
            ]
          }
        ]
      };
    }
    return bnf_1 = O, bnf_1;
  }
  var brainfuck_1, hasRequiredBrainfuck;
  function requireBrainfuck() {
    if (hasRequiredBrainfuck) return brainfuck_1;
    hasRequiredBrainfuck = 1;
    function O(_) {
      const H = {
        className: "literal",
        begin: /[+-]+/,
        relevance: 0
      };
      return {
        name: "Brainfuck",
        aliases: ["bf"],
        contains: [
          _.COMMENT(
            /[^\[\]\.,\+\-<> \r\n]/,
            /[\[\]\.,\+\-<> \r\n]/,
            {
              contains: [
                {
                  match: /[ ]+[^\[\]\.,\+\-<> \r\n]/,
                  relevance: 0
                }
              ],
              returnEnd: !0,
              relevance: 0
            }
          ),
          {
            className: "title",
            begin: "[\\[\\]]",
            relevance: 0
          },
          {
            className: "string",
            begin: "[\\.,]",
            relevance: 0
          },
          {
            // this mode works as the only relevance counter
            // it looks ahead to find the start of a run of literals
            // so only the runs are counted as relevant
            begin: /(?=\+\+|--)/,
            contains: [H]
          },
          H
        ]
      };
    }
    return brainfuck_1 = O, brainfuck_1;
  }
  var cal_1, hasRequiredCal;
  function requireCal() {
    if (hasRequiredCal) return cal_1;
    hasRequiredCal = 1;
    function O(_) {
      const H = _.regex, V = [
        "div",
        "mod",
        "in",
        "and",
        "or",
        "not",
        "xor",
        "asserterror",
        "begin",
        "case",
        "do",
        "downto",
        "else",
        "end",
        "exit",
        "for",
        "local",
        "if",
        "of",
        "repeat",
        "then",
        "to",
        "until",
        "while",
        "with",
        "var"
      ], q = "false true", G = [
        _.C_LINE_COMMENT_MODE,
        _.COMMENT(
          /\{/,
          /\}/,
          { relevance: 0 }
        ),
        _.COMMENT(
          /\(\*/,
          /\*\)/,
          { relevance: 10 }
        )
      ], J = {
        className: "string",
        begin: /'/,
        end: /'/,
        contains: [{ begin: /''/ }]
      }, Z = {
        className: "string",
        begin: /(#\d+)+/
      }, te = {
        className: "number",
        begin: "\\b\\d+(\\.\\d+)?(DT|D|T)",
        relevance: 0
      }, ne = {
        className: "string",
        // not a string technically but makes sense to be highlighted in the same style
        begin: '"',
        end: '"'
      }, ee = {
        match: [
          /procedure/,
          /\s+/,
          /[a-zA-Z_][\w@]*/,
          /\s*/
        ],
        scope: {
          1: "keyword",
          3: "title.function"
        },
        contains: [
          {
            className: "params",
            begin: /\(/,
            end: /\)/,
            keywords: V,
            contains: [
              J,
              Z,
              _.NUMBER_MODE
            ]
          },
          ...G
        ]
      }, ae = [
        "Table",
        "Form",
        "Report",
        "Dataport",
        "Codeunit",
        "XMLport",
        "MenuSuite",
        "Page",
        "Query"
      ], oe = {
        match: [
          /OBJECT/,
          /\s+/,
          H.either(...ae),
          /\s+/,
          /\d+/,
          /\s+(?=[^\s])/,
          /.*/,
          /$/
        ],
        relevance: 3,
        scope: {
          1: "keyword",
          3: "type",
          5: "number",
          7: "title"
        }
      };
      return {
        name: "C/AL",
        case_insensitive: !0,
        keywords: {
          keyword: V,
          literal: q
        },
        illegal: /\/\*/,
        contains: [
          {
            match: /[\w]+(?=\=)/,
            scope: "attribute",
            relevance: 0
          },
          J,
          Z,
          te,
          ne,
          _.NUMBER_MODE,
          oe,
          ee
        ]
      };
    }
    return cal_1 = O, cal_1;
  }
  var capnproto_1, hasRequiredCapnproto;
  function requireCapnproto() {
    if (hasRequiredCapnproto) return capnproto_1;
    hasRequiredCapnproto = 1;
    function O(_) {
      const H = [
        "struct",
        "enum",
        "interface",
        "union",
        "group",
        "import",
        "using",
        "const",
        "annotation",
        "extends",
        "in",
        "of",
        "on",
        "as",
        "with",
        "from",
        "fixed"
      ], V = [
        "Void",
        "Bool",
        "Int8",
        "Int16",
        "Int32",
        "Int64",
        "UInt8",
        "UInt16",
        "UInt32",
        "UInt64",
        "Float32",
        "Float64",
        "Text",
        "Data",
        "AnyPointer",
        "AnyStruct",
        "Capability",
        "List"
      ], q = [
        "true",
        "false"
      ], G = {
        variants: [
          { match: [
            /(struct|enum|interface)/,
            /\s+/,
            _.IDENT_RE
          ] },
          { match: [
            /extends/,
            /\s*\(/,
            _.IDENT_RE,
            /\s*\)/
          ] }
        ],
        scope: {
          1: "keyword",
          3: "title.class"
        }
      };
      return {
        name: "Cap’n Proto",
        aliases: ["capnp"],
        keywords: {
          keyword: H,
          type: V,
          literal: q
        },
        contains: [
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE,
          _.HASH_COMMENT_MODE,
          {
            className: "meta",
            begin: /@0x[\w\d]{16};/,
            illegal: /\n/
          },
          {
            className: "symbol",
            begin: /@\d+\b/
          },
          G
        ]
      };
    }
    return capnproto_1 = O, capnproto_1;
  }
  var ceylon_1, hasRequiredCeylon;
  function requireCeylon() {
    if (hasRequiredCeylon) return ceylon_1;
    hasRequiredCeylon = 1;
    function O(_) {
      const H = [
        "assembly",
        "module",
        "package",
        "import",
        "alias",
        "class",
        "interface",
        "object",
        "given",
        "value",
        "assign",
        "void",
        "function",
        "new",
        "of",
        "extends",
        "satisfies",
        "abstracts",
        "in",
        "out",
        "return",
        "break",
        "continue",
        "throw",
        "assert",
        "dynamic",
        "if",
        "else",
        "switch",
        "case",
        "for",
        "while",
        "try",
        "catch",
        "finally",
        "then",
        "let",
        "this",
        "outer",
        "super",
        "is",
        "exists",
        "nonempty"
      ], V = [
        "shared",
        "abstract",
        "formal",
        "default",
        "actual",
        "variable",
        "late",
        "native",
        "deprecated",
        "final",
        "sealed",
        "annotation",
        "suppressWarnings",
        "small"
      ], q = [
        "doc",
        "by",
        "license",
        "see",
        "throws",
        "tagged"
      ], G = {
        className: "subst",
        excludeBegin: !0,
        excludeEnd: !0,
        begin: /``/,
        end: /``/,
        keywords: H,
        relevance: 10
      }, J = [
        {
          // verbatim string
          className: "string",
          begin: '"""',
          end: '"""',
          relevance: 10
        },
        {
          // string literal or template
          className: "string",
          begin: '"',
          end: '"',
          contains: [G]
        },
        {
          // character literal
          className: "string",
          begin: "'",
          end: "'"
        },
        {
          // numeric literal
          className: "number",
          begin: "#[0-9a-fA-F_]+|\\$[01_]+|[0-9_]+(?:\\.[0-9_](?:[eE][+-]?\\d+)?)?[kMGTPmunpf]?",
          relevance: 0
        }
      ];
      return G.contains = J, {
        name: "Ceylon",
        keywords: {
          keyword: H.concat(V),
          meta: q
        },
        illegal: "\\$[^01]|#[^0-9a-fA-F]",
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.COMMENT("/\\*", "\\*/", { contains: ["self"] }),
          {
            // compiler annotation
            className: "meta",
            begin: '@[a-z]\\w*(?::"[^"]*")?'
          }
        ].concat(J)
      };
    }
    return ceylon_1 = O, ceylon_1;
  }
  var clean_1, hasRequiredClean;
  function requireClean() {
    if (hasRequiredClean) return clean_1;
    hasRequiredClean = 1;
    function O(_) {
      return {
        name: "Clean",
        aliases: [
          "icl",
          "dcl"
        ],
        keywords: {
          keyword: [
            "if",
            "let",
            "in",
            "with",
            "where",
            "case",
            "of",
            "class",
            "instance",
            "otherwise",
            "implementation",
            "definition",
            "system",
            "module",
            "from",
            "import",
            "qualified",
            "as",
            "special",
            "code",
            "inline",
            "foreign",
            "export",
            "ccall",
            "stdcall",
            "generic",
            "derive",
            "infix",
            "infixl",
            "infixr"
          ],
          built_in: "Int Real Char Bool",
          literal: "True False"
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE,
          {
            // relevance booster
            begin: "->|<-[|:]?|#!?|>>=|\\{\\||\\|\\}|:==|=:|<>"
          }
        ]
      };
    }
    return clean_1 = O, clean_1;
  }
  var clojure_1, hasRequiredClojure;
  function requireClojure() {
    if (hasRequiredClojure) return clojure_1;
    hasRequiredClojure = 1;
    function O(_) {
      const H = "a-zA-Z_\\-!.?+*=<>&'", V = "[#]?[" + H + "][" + H + "0-9/;:$#]*", q = "def defonce defprotocol defstruct defmulti defmethod defn- defn defmacro deftype defrecord", G = {
        $pattern: V,
        built_in: (
          // Clojure keywords
          q + " cond apply if-not if-let if not not= =|0 <|0 >|0 <=|0 >=|0 ==|0 +|0 /|0 *|0 -|0 rem quot neg? pos? delay? symbol? keyword? true? false? integer? empty? coll? list? set? ifn? fn? associative? sequential? sorted? counted? reversible? number? decimal? class? distinct? isa? float? rational? reduced? ratio? odd? even? char? seq? vector? string? map? nil? contains? zero? instance? not-every? not-any? libspec? -> ->> .. . inc compare do dotimes mapcat take remove take-while drop letfn drop-last take-last drop-while while intern condp case reduced cycle split-at split-with repeat replicate iterate range merge zipmap declare line-seq sort comparator sort-by dorun doall nthnext nthrest partition eval doseq await await-for let agent atom send send-off release-pending-sends add-watch mapv filterv remove-watch agent-error restart-agent set-error-handler error-handler set-error-mode! error-mode shutdown-agents quote var fn loop recur throw try monitor-enter monitor-exit macroexpand macroexpand-1 for dosync and or when when-not when-let comp juxt partial sequence memoize constantly complement identity assert peek pop doto proxy first rest cons cast coll last butlast sigs reify second ffirst fnext nfirst nnext meta with-meta ns in-ns create-ns import refer keys select-keys vals key val rseq name namespace promise into transient persistent! conj! assoc! dissoc! pop! disj! use class type num float double short byte boolean bigint biginteger bigdec print-method print-dup throw-if printf format load compile get-in update-in pr pr-on newline flush read slurp read-line subvec with-open memfn time re-find re-groups rand-int rand mod locking assert-valid-fdecl alias resolve ref deref refset swap! reset! set-validator! compare-and-set! alter-meta! reset-meta! commute get-validator alter ref-set ref-history-count ref-min-history ref-max-history ensure sync io! new next conj set! to-array future future-call into-array aset gen-class reduce map filter find empty hash-map hash-set sorted-map sorted-map-by sorted-set sorted-set-by vec vector seq flatten reverse assoc dissoc list disj get union difference intersection extend extend-type extend-protocol int nth delay count concat chunk chunk-buffer chunk-append chunk-first chunk-rest max min dec unchecked-inc-int unchecked-inc unchecked-dec-inc unchecked-dec unchecked-negate unchecked-add-int unchecked-add unchecked-subtract-int unchecked-subtract chunk-next chunk-cons chunked-seq? prn vary-meta lazy-seq spread list* str find-keyword keyword symbol gensym force rationalize"
        )
      }, J = {
        begin: V,
        relevance: 0
      }, Z = {
        scope: "number",
        relevance: 0,
        variants: [
          { match: /[-+]?0[xX][0-9a-fA-F]+N?/ },
          // hexadecimal                 // 0x2a
          { match: /[-+]?0[0-7]+N?/ },
          // octal                       // 052
          { match: /[-+]?[1-9][0-9]?[rR][0-9a-zA-Z]+N?/ },
          // variable radix from 2 to 36 // 2r101010, 8r52, 36r16
          { match: /[-+]?[0-9]+\/[0-9]+N?/ },
          // ratio                       // 1/2
          { match: /[-+]?[0-9]+((\.[0-9]*([eE][+-]?[0-9]+)?M?)|([eE][+-]?[0-9]+M?|M))/ },
          // float        // 0.42 4.2E-1M 42E1 42M
          { match: /[-+]?([1-9][0-9]*|0)N?/ }
          // int (don't match leading 0) // 42 42N
        ]
      }, te = {
        scope: "character",
        variants: [
          { match: /\\o[0-3]?[0-7]{1,2}/ },
          // Unicode Octal 0 - 377
          { match: /\\u[0-9a-fA-F]{4}/ },
          // Unicode Hex 0000 - FFFF
          { match: /\\(newline|space|tab|formfeed|backspace|return)/ },
          // special characters
          {
            match: /\\\S/,
            relevance: 0
          }
          // any non-whitespace char
        ]
      }, ne = {
        scope: "regex",
        begin: /#"/,
        end: /"/,
        contains: [_.BACKSLASH_ESCAPE]
      }, ee = _.inherit(_.QUOTE_STRING_MODE, { illegal: null }), ae = {
        scope: "punctuation",
        match: /,/,
        relevance: 0
      }, oe = _.COMMENT(
        ";",
        "$",
        { relevance: 0 }
      ), se = {
        className: "literal",
        begin: /\b(true|false|nil)\b/
      }, ce = {
        begin: "\\[|(#::?" + V + ")?\\{",
        end: "[\\]\\}]",
        relevance: 0
      }, le = {
        className: "symbol",
        begin: "[:]{1,2}" + V
      }, ue = {
        begin: "\\(",
        end: "\\)"
      }, de = {
        endsWithParent: !0,
        relevance: 0
      }, fe = {
        keywords: G,
        className: "name",
        begin: V,
        relevance: 0,
        starts: de
      }, he = [
        ae,
        ue,
        te,
        ne,
        ee,
        oe,
        le,
        ce,
        Z,
        se,
        J
      ], me = {
        beginKeywords: q,
        keywords: {
          $pattern: V,
          keyword: q
        },
        end: '(\\[|#|\\d|"|:|\\{|\\)|\\(|$)',
        contains: [
          {
            className: "title",
            begin: V,
            relevance: 0,
            excludeEnd: !0,
            // we can only have a single title
            endsParent: !0
          }
        ].concat(he)
      };
      return ue.contains = [
        me,
        fe,
        de
      ], de.contains = he, ce.contains = he, {
        name: "Clojure",
        aliases: [
          "clj",
          "edn"
        ],
        illegal: /\S/,
        contains: [
          ae,
          ue,
          te,
          ne,
          ee,
          oe,
          le,
          ce,
          Z,
          se
        ]
      };
    }
    return clojure_1 = O, clojure_1;
  }
  var clojureRepl_1, hasRequiredClojureRepl;
  function requireClojureRepl() {
    if (hasRequiredClojureRepl) return clojureRepl_1;
    hasRequiredClojureRepl = 1;
    function O(_) {
      return {
        name: "Clojure REPL",
        contains: [
          {
            className: "meta.prompt",
            begin: /^([\w.-]+|\s*#_)?=>/,
            starts: {
              end: /$/,
              subLanguage: "clojure"
            }
          }
        ]
      };
    }
    return clojureRepl_1 = O, clojureRepl_1;
  }
  var cmake_1, hasRequiredCmake;
  function requireCmake() {
    if (hasRequiredCmake) return cmake_1;
    hasRequiredCmake = 1;
    function O(_) {
      return {
        name: "CMake",
        aliases: ["cmake.in"],
        case_insensitive: !0,
        keywords: { keyword: (
          // scripting commands
          "break cmake_host_system_information cmake_minimum_required cmake_parse_arguments cmake_policy configure_file continue elseif else endforeach endfunction endif endmacro endwhile execute_process file find_file find_library find_package find_path find_program foreach function get_cmake_property get_directory_property get_filename_component get_property if include include_guard list macro mark_as_advanced math message option return separate_arguments set_directory_properties set_property set site_name string unset variable_watch while add_compile_definitions add_compile_options add_custom_command add_custom_target add_definitions add_dependencies add_executable add_library add_link_options add_subdirectory add_test aux_source_directory build_command create_test_sourcelist define_property enable_language enable_testing export fltk_wrap_ui get_source_file_property get_target_property get_test_property include_directories include_external_msproject include_regular_expression install link_directories link_libraries load_cache project qt_wrap_cpp qt_wrap_ui remove_definitions set_source_files_properties set_target_properties set_tests_properties source_group target_compile_definitions target_compile_features target_compile_options target_include_directories target_link_directories target_link_libraries target_link_options target_sources try_compile try_run ctest_build ctest_configure ctest_coverage ctest_empty_binary_directory ctest_memcheck ctest_read_custom_files ctest_run_script ctest_sleep ctest_start ctest_submit ctest_test ctest_update ctest_upload build_name exec_program export_library_dependencies install_files install_programs install_targets load_command make_directory output_required_files remove subdir_depends subdirs use_mangled_mesa utility_source variable_requires write_file qt5_use_modules qt5_use_package qt5_wrap_cpp on off true false and or not command policy target test exists is_newer_than is_directory is_symlink is_absolute matches less greater equal less_equal greater_equal strless strgreater strequal strless_equal strgreater_equal version_less version_greater version_equal version_less_equal version_greater_equal in_list defined"
        ) },
        contains: [
          {
            className: "variable",
            begin: /\$\{/,
            end: /\}/
          },
          _.COMMENT(/#\[\[/, /]]/),
          _.HASH_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE
        ]
      };
    }
    return cmake_1 = O, cmake_1;
  }
  var coffeescript_1, hasRequiredCoffeescript;
  function requireCoffeescript() {
    if (hasRequiredCoffeescript) return coffeescript_1;
    hasRequiredCoffeescript = 1;
    const O = [
      "as",
      // for exports
      "in",
      "of",
      "if",
      "for",
      "while",
      "finally",
      "var",
      "new",
      "function",
      "do",
      "return",
      "void",
      "else",
      "break",
      "catch",
      "instanceof",
      "with",
      "throw",
      "case",
      "default",
      "try",
      "switch",
      "continue",
      "typeof",
      "delete",
      "let",
      "yield",
      "const",
      "class",
      // JS handles these with a special rule
      // "get",
      // "set",
      "debugger",
      "async",
      "await",
      "static",
      "import",
      "from",
      "export",
      "extends",
      // It's reached stage 3, which is "recommended for implementation":
      "using"
    ], _ = [
      "true",
      "false",
      "null",
      "undefined",
      "NaN",
      "Infinity"
    ], H = [
      // Fundamental objects
      "Object",
      "Function",
      "Boolean",
      "Symbol",
      // numbers and dates
      "Math",
      "Date",
      "Number",
      "BigInt",
      // text
      "String",
      "RegExp",
      // Indexed collections
      "Array",
      "Float32Array",
      "Float64Array",
      "Int8Array",
      "Uint8Array",
      "Uint8ClampedArray",
      "Int16Array",
      "Int32Array",
      "Uint16Array",
      "Uint32Array",
      "BigInt64Array",
      "BigUint64Array",
      // Keyed collections
      "Set",
      "Map",
      "WeakSet",
      "WeakMap",
      // Structured data
      "ArrayBuffer",
      "SharedArrayBuffer",
      "Atomics",
      "DataView",
      "JSON",
      // Control abstraction objects
      "Promise",
      "Generator",
      "GeneratorFunction",
      "AsyncFunction",
      // Reflection
      "Reflect",
      "Proxy",
      // Internationalization
      "Intl",
      // WebAssembly
      "WebAssembly"
    ], V = [
      "Error",
      "EvalError",
      "InternalError",
      "RangeError",
      "ReferenceError",
      "SyntaxError",
      "TypeError",
      "URIError"
    ], q = [
      "setInterval",
      "setTimeout",
      "clearInterval",
      "clearTimeout",
      "require",
      "exports",
      "eval",
      "isFinite",
      "isNaN",
      "parseFloat",
      "parseInt",
      "decodeURI",
      "decodeURIComponent",
      "encodeURI",
      "encodeURIComponent",
      "escape",
      "unescape"
    ], G = [].concat(
      q,
      H,
      V
    );
    function J(Z) {
      const te = [
        "npm",
        "print"
      ], ne = [
        "yes",
        "no",
        "on",
        "off"
      ], ee = [
        "then",
        "unless",
        "until",
        "loop",
        "by",
        "when",
        "and",
        "or",
        "is",
        "isnt",
        "not"
      ], ae = [
        "var",
        "const",
        "let",
        "function",
        "static"
      ], oe = (pe) => (ye) => !pe.includes(ye), se = {
        keyword: O.concat(ee).filter(oe(ae)),
        literal: _.concat(ne),
        built_in: G.concat(te)
      }, ce = "[A-Za-z$_][0-9A-Za-z$_]*", le = {
        className: "subst",
        begin: /#\{/,
        end: /\}/,
        keywords: se
      }, ue = [
        Z.BINARY_NUMBER_MODE,
        Z.inherit(Z.C_NUMBER_MODE, { starts: {
          end: "(\\s*/)?",
          relevance: 0
        } }),
        // a number tries to eat the following slash to prevent treating it as a regexp
        {
          className: "string",
          variants: [
            {
              begin: /'''/,
              end: /'''/,
              contains: [Z.BACKSLASH_ESCAPE]
            },
            {
              begin: /'/,
              end: /'/,
              contains: [Z.BACKSLASH_ESCAPE]
            },
            {
              begin: /"""/,
              end: /"""/,
              contains: [
                Z.BACKSLASH_ESCAPE,
                le
              ]
            },
            {
              begin: /"/,
              end: /"/,
              contains: [
                Z.BACKSLASH_ESCAPE,
                le
              ]
            }
          ]
        },
        {
          className: "regexp",
          variants: [
            {
              begin: "///",
              end: "///",
              contains: [
                le,
                Z.HASH_COMMENT_MODE
              ]
            },
            {
              begin: "//[gim]{0,3}(?=\\W)",
              relevance: 0
            },
            {
              // regex can't start with space to parse x / 2 / 3 as two divisions
              // regex can't start with *, and it supports an "illegal" in the main mode
              begin: /\/(?![ *]).*?(?![\\]).\/[gim]{0,3}(?=\W)/
            }
          ]
        },
        {
          begin: "@" + ce
          // relevance booster
        },
        {
          subLanguage: "javascript",
          excludeBegin: !0,
          excludeEnd: !0,
          variants: [
            {
              begin: "```",
              end: "```"
            },
            {
              begin: "`",
              end: "`"
            }
          ]
        }
      ];
      le.contains = ue;
      const de = Z.inherit(Z.TITLE_MODE, { begin: ce }), fe = "(\\(.*\\)\\s*)?\\B[-=]>", he = {
        className: "params",
        begin: "\\([^\\(]",
        returnBegin: !0,
        /* We need another contained nameless mode to not have every nested
        pair of parens to be called "params" */
        contains: [
          {
            begin: /\(/,
            end: /\)/,
            keywords: se,
            contains: ["self"].concat(ue)
          }
        ]
      }, me = {
        variants: [
          { match: [
            /class\s+/,
            ce,
            /\s+extends\s+/,
            ce
          ] },
          { match: [
            /class\s+/,
            ce
          ] }
        ],
        scope: {
          2: "title.class",
          4: "title.class.inherited"
        },
        keywords: se
      };
      return {
        name: "CoffeeScript",
        aliases: [
          "coffee",
          "cson",
          "iced"
        ],
        keywords: se,
        illegal: /\/\*/,
        contains: [
          ...ue,
          Z.COMMENT("###", "###"),
          Z.HASH_COMMENT_MODE,
          {
            className: "function",
            begin: "^\\s*" + ce + "\\s*=\\s*" + fe,
            end: "[-=]>",
            returnBegin: !0,
            contains: [
              de,
              he
            ]
          },
          {
            // anonymous function start
            begin: /[:\(,=]\s*/,
            relevance: 0,
            contains: [
              {
                className: "function",
                begin: fe,
                end: "[-=]>",
                returnBegin: !0,
                contains: [he]
              }
            ]
          },
          me,
          {
            begin: ce + ":",
            end: ":",
            returnBegin: !0,
            returnEnd: !0,
            relevance: 0
          }
        ]
      };
    }
    return coffeescript_1 = J, coffeescript_1;
  }
  var coq_1, hasRequiredCoq;
  function requireCoq() {
    if (hasRequiredCoq) return coq_1;
    hasRequiredCoq = 1;
    function O(_) {
      return {
        name: "Coq",
        keywords: {
          keyword: [
            "_|0",
            "as",
            "at",
            "cofix",
            "else",
            "end",
            "exists",
            "exists2",
            "fix",
            "for",
            "forall",
            "fun",
            "if",
            "IF",
            "in",
            "let",
            "match",
            "mod",
            "Prop",
            "return",
            "Set",
            "then",
            "Type",
            "using",
            "where",
            "with",
            "Abort",
            "About",
            "Add",
            "Admit",
            "Admitted",
            "All",
            "Arguments",
            "Assumptions",
            "Axiom",
            "Back",
            "BackTo",
            "Backtrack",
            "Bind",
            "Blacklist",
            "Canonical",
            "Cd",
            "Check",
            "Class",
            "Classes",
            "Close",
            "Coercion",
            "Coercions",
            "CoFixpoint",
            "CoInductive",
            "Collection",
            "Combined",
            "Compute",
            "Conjecture",
            "Conjectures",
            "Constant",
            "constr",
            "Constraint",
            "Constructors",
            "Context",
            "Corollary",
            "CreateHintDb",
            "Cut",
            "Declare",
            "Defined",
            "Definition",
            "Delimit",
            "Dependencies",
            "Dependent",
            "Derive",
            "Drop",
            "eauto",
            "End",
            "Equality",
            "Eval",
            "Example",
            "Existential",
            "Existentials",
            "Existing",
            "Export",
            "exporting",
            "Extern",
            "Extract",
            "Extraction",
            "Fact",
            "Field",
            "Fields",
            "File",
            "Fixpoint",
            "Focus",
            "for",
            "From",
            "Function",
            "Functional",
            "Generalizable",
            "Global",
            "Goal",
            "Grab",
            "Grammar",
            "Graph",
            "Guarded",
            "Heap",
            "Hint",
            "HintDb",
            "Hints",
            "Hypotheses",
            "Hypothesis",
            "ident",
            "Identity",
            "If",
            "Immediate",
            "Implicit",
            "Import",
            "Include",
            "Inductive",
            "Infix",
            "Info",
            "Initial",
            "Inline",
            "Inspect",
            "Instance",
            "Instances",
            "Intro",
            "Intros",
            "Inversion",
            "Inversion_clear",
            "Language",
            "Left",
            "Lemma",
            "Let",
            "Libraries",
            "Library",
            "Load",
            "LoadPath",
            "Local",
            "Locate",
            "Ltac",
            "ML",
            "Mode",
            "Module",
            "Modules",
            "Monomorphic",
            "Morphism",
            "Next",
            "NoInline",
            "Notation",
            "Obligation",
            "Obligations",
            "Opaque",
            "Open",
            "Optimize",
            "Options",
            "Parameter",
            "Parameters",
            "Parametric",
            "Path",
            "Paths",
            "pattern",
            "Polymorphic",
            "Preterm",
            "Print",
            "Printing",
            "Program",
            "Projections",
            "Proof",
            "Proposition",
            "Pwd",
            "Qed",
            "Quit",
            "Rec",
            "Record",
            "Recursive",
            "Redirect",
            "Relation",
            "Remark",
            "Remove",
            "Require",
            "Reserved",
            "Reset",
            "Resolve",
            "Restart",
            "Rewrite",
            "Right",
            "Ring",
            "Rings",
            "Save",
            "Scheme",
            "Scope",
            "Scopes",
            "Script",
            "Search",
            "SearchAbout",
            "SearchHead",
            "SearchPattern",
            "SearchRewrite",
            "Section",
            "Separate",
            "Set",
            "Setoid",
            "Show",
            "Solve",
            "Sorted",
            "Step",
            "Strategies",
            "Strategy",
            "Structure",
            "SubClass",
            "Table",
            "Tables",
            "Tactic",
            "Term",
            "Test",
            "Theorem",
            "Time",
            "Timeout",
            "Transparent",
            "Type",
            "Typeclasses",
            "Types",
            "Undelimit",
            "Undo",
            "Unfocus",
            "Unfocused",
            "Unfold",
            "Universe",
            "Universes",
            "Unset",
            "Unshelve",
            "using",
            "Variable",
            "Variables",
            "Variant",
            "Verbose",
            "Visibility",
            "where",
            "with"
          ],
          built_in: [
            "abstract",
            "absurd",
            "admit",
            "after",
            "apply",
            "as",
            "assert",
            "assumption",
            "at",
            "auto",
            "autorewrite",
            "autounfold",
            "before",
            "bottom",
            "btauto",
            "by",
            "case",
            "case_eq",
            "cbn",
            "cbv",
            "change",
            "classical_left",
            "classical_right",
            "clear",
            "clearbody",
            "cofix",
            "compare",
            "compute",
            "congruence",
            "constr_eq",
            "constructor",
            "contradict",
            "contradiction",
            "cut",
            "cutrewrite",
            "cycle",
            "decide",
            "decompose",
            "dependent",
            "destruct",
            "destruction",
            "dintuition",
            "discriminate",
            "discrR",
            "do",
            "double",
            "dtauto",
            "eapply",
            "eassumption",
            "eauto",
            "ecase",
            "econstructor",
            "edestruct",
            "ediscriminate",
            "eelim",
            "eexact",
            "eexists",
            "einduction",
            "einjection",
            "eleft",
            "elim",
            "elimtype",
            "enough",
            "equality",
            "erewrite",
            "eright",
            "esimplify_eq",
            "esplit",
            "evar",
            "exact",
            "exactly_once",
            "exfalso",
            "exists",
            "f_equal",
            "fail",
            "field",
            "field_simplify",
            "field_simplify_eq",
            "first",
            "firstorder",
            "fix",
            "fold",
            "fourier",
            "functional",
            "generalize",
            "generalizing",
            "gfail",
            "give_up",
            "has_evar",
            "hnf",
            "idtac",
            "in",
            "induction",
            "injection",
            "instantiate",
            "intro",
            "intro_pattern",
            "intros",
            "intuition",
            "inversion",
            "inversion_clear",
            "is_evar",
            "is_var",
            "lapply",
            "lazy",
            "left",
            "lia",
            "lra",
            "move",
            "native_compute",
            "nia",
            "nsatz",
            "omega",
            "once",
            "pattern",
            "pose",
            "progress",
            "proof",
            "psatz",
            "quote",
            "record",
            "red",
            "refine",
            "reflexivity",
            "remember",
            "rename",
            "repeat",
            "replace",
            "revert",
            "revgoals",
            "rewrite",
            "rewrite_strat",
            "right",
            "ring",
            "ring_simplify",
            "rtauto",
            "set",
            "setoid_reflexivity",
            "setoid_replace",
            "setoid_rewrite",
            "setoid_symmetry",
            "setoid_transitivity",
            "shelve",
            "shelve_unifiable",
            "simpl",
            "simple",
            "simplify_eq",
            "solve",
            "specialize",
            "split",
            "split_Rabs",
            "split_Rmult",
            "stepl",
            "stepr",
            "subst",
            "sum",
            "swap",
            "symmetry",
            "tactic",
            "tauto",
            "time",
            "timeout",
            "top",
            "transitivity",
            "trivial",
            "try",
            "tryif",
            "unfold",
            "unify",
            "until",
            "using",
            "vm_compute",
            "with"
          ]
        },
        contains: [
          _.QUOTE_STRING_MODE,
          _.COMMENT("\\(\\*", "\\*\\)"),
          _.C_NUMBER_MODE,
          {
            className: "type",
            excludeBegin: !0,
            begin: "\\|\\s*",
            end: "\\w+"
          },
          {
            // relevance booster
            begin: /[-=]>/
          }
        ]
      };
    }
    return coq_1 = O, coq_1;
  }
  var cos_1, hasRequiredCos;
  function requireCos() {
    if (hasRequiredCos) return cos_1;
    hasRequiredCos = 1;
    function O(_) {
      return {
        name: "Caché Object Script",
        case_insensitive: !0,
        aliases: ["cls"],
        keywords: "property parameter class classmethod clientmethod extends as break catch close continue do d|0 else elseif for goto halt hang h|0 if job j|0 kill k|0 lock l|0 merge new open quit q|0 read r|0 return set s|0 tcommit throw trollback try tstart use view while write w|0 xecute x|0 zkill znspace zn ztrap zwrite zw zzdump zzwrite print zbreak zinsert zload zprint zremove zsave zzprint mv mvcall mvcrt mvdim mvprint zquit zsync ascii",
        contains: [
          {
            className: "number",
            begin: "\\b(\\d+(\\.\\d*)?|\\.\\d+)",
            relevance: 0
          },
          {
            className: "string",
            variants: [
              {
                begin: '"',
                end: '"',
                contains: [
                  {
                    // escaped
                    begin: '""',
                    relevance: 0
                  }
                ]
              }
            ]
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "comment",
            begin: /;/,
            end: "$",
            relevance: 0
          },
          {
            // Functions and user-defined functions: write $ztime(60*60*3), $$myFunc(10), $$^Val(1)
            className: "built_in",
            begin: /(?:\$\$?|\.\.)\^?[a-zA-Z]+/
          },
          {
            // Macro command: quit $$$OK
            className: "built_in",
            begin: /\$\$\$[a-zA-Z]+/
          },
          {
            // Special (global) variables: write %request.Content; Built-in classes: %Library.Integer
            className: "built_in",
            begin: /%[a-z]+(?:\.[a-z]+)*/
          },
          {
            // Global variable: set ^globalName = 12 write ^globalName
            className: "symbol",
            begin: /\^%?[a-zA-Z][\w]*/
          },
          {
            // Some control constructions: do ##class(Package.ClassName).Method(), ##super()
            className: "keyword",
            begin: /##class|##super|#define|#dim/
          },
          // sub-languages: are not fully supported by hljs by 11/15/2015
          // left for the future implementation.
          {
            begin: /&sql\(/,
            end: /\)/,
            excludeBegin: !0,
            excludeEnd: !0,
            subLanguage: "sql"
          },
          {
            begin: /&(js|jscript|javascript)</,
            end: />/,
            excludeBegin: !0,
            excludeEnd: !0,
            subLanguage: "javascript"
          },
          {
            // this brakes first and last tag, but this is the only way to embed a valid html
            begin: /&html<\s*</,
            end: />\s*>/,
            subLanguage: "xml"
          }
        ]
      };
    }
    return cos_1 = O, cos_1;
  }
  var crmsh_1, hasRequiredCrmsh;
  function requireCrmsh() {
    if (hasRequiredCrmsh) return crmsh_1;
    hasRequiredCrmsh = 1;
    function O(_) {
      const H = "primitive rsc_template", V = "group clone ms master location colocation order fencing_topology rsc_ticket acl_target acl_group user role tag xml";
      return {
        name: "crmsh",
        aliases: [
          "crm",
          "pcmk"
        ],
        case_insensitive: !0,
        keywords: {
          keyword: "params meta operations op rule attributes utilization" + " " + "read write deny defined not_defined in_range date spec in ref reference attribute type xpath version and or lt gt tag lte gte eq ne \\" + " " + "number string",
          literal: "Master Started Slave Stopped start promote demote stop monitor true false"
        },
        contains: [
          _.HASH_COMMENT_MODE,
          {
            beginKeywords: "node",
            starts: {
              end: "\\s*([\\w_-]+:)?",
              starts: {
                className: "title",
                end: "\\s*[\\$\\w_][\\w_-]*"
              }
            }
          },
          {
            beginKeywords: H,
            starts: {
              className: "title",
              end: "\\s*[\\$\\w_][\\w_-]*",
              starts: { end: "\\s*@?[\\w_][\\w_\\.:-]*" }
            }
          },
          {
            begin: "\\b(" + V.split(" ").join("|") + ")\\s+",
            keywords: V,
            starts: {
              className: "title",
              end: "[\\$\\w_][\\w_-]*"
            }
          },
          {
            beginKeywords: "property rsc_defaults op_defaults",
            starts: {
              className: "title",
              end: "\\s*([\\w_-]+:)?"
            }
          },
          _.QUOTE_STRING_MODE,
          {
            className: "meta",
            begin: "(ocf|systemd|service|lsb):[\\w_:-]+",
            relevance: 0
          },
          {
            className: "number",
            begin: "\\b\\d+(\\.\\d+)?(ms|s|h|m)?",
            relevance: 0
          },
          {
            className: "literal",
            begin: "[-]?(infinity|inf)",
            relevance: 0
          },
          {
            className: "attr",
            begin: /([A-Za-z$_#][\w_-]+)=/,
            relevance: 0
          },
          {
            className: "tag",
            begin: "</?",
            end: "/?>",
            relevance: 0
          }
        ]
      };
    }
    return crmsh_1 = O, crmsh_1;
  }
  var crystal_1, hasRequiredCrystal;
  function requireCrystal() {
    if (hasRequiredCrystal) return crystal_1;
    hasRequiredCrystal = 1;
    function O(_) {
      const H = "(_?[ui](8|16|32|64|128))?", V = "(_?f(32|64))?", q = "[a-zA-Z_]\\w*[!?=]?", G = "[a-zA-Z_]\\w*[!?=]?|[-+~]@|<<|>>|[=!]~|===?|<=>|[<>]=?|\\*\\*|[-/+%^&*~|]|//|//=|&[-+*]=?|&\\*\\*|\\[\\][=?]?", J = "[A-Za-z_]\\w*(::\\w+)*(\\?|!)?", Z = {
        $pattern: q,
        keyword: "abstract alias annotation as as? asm begin break case class def do else elsif end ensure enum extend for fun if include instance_sizeof is_a? lib macro module next nil? of out pointerof private protected rescue responds_to? return require select self sizeof struct super then type typeof union uninitialized unless until verbatim when while with yield __DIR__ __END_LINE__ __FILE__ __LINE__",
        literal: "false nil true"
      }, te = {
        className: "subst",
        begin: /#\{/,
        end: /\}/,
        keywords: Z
      }, ne = {
        // negative-look forward attemps to prevent false matches like:
        // @ident@ or $ident$ that might indicate this is not ruby at all
        className: "variable",
        begin: "(\\$\\W)|((\\$|@@?)(\\w+))(?=[^@$?])(?![A-Za-z])(?![@$?'])"
      }, ee = {
        className: "template-variable",
        variants: [
          {
            begin: "\\{\\{",
            end: "\\}\\}"
          },
          {
            begin: "\\{%",
            end: "%\\}"
          }
        ],
        keywords: Z
      };
      function ae(fe, he) {
        const me = [
          {
            begin: fe,
            end: he
          }
        ];
        return me[0].contains = me, me;
      }
      const oe = {
        className: "string",
        contains: [
          _.BACKSLASH_ESCAPE,
          te
        ],
        variants: [
          {
            begin: /'/,
            end: /'/
          },
          {
            begin: /"/,
            end: /"/
          },
          {
            begin: /`/,
            end: /`/
          },
          {
            begin: "%[Qwi]?\\(",
            end: "\\)",
            contains: ae("\\(", "\\)")
          },
          {
            begin: "%[Qwi]?\\[",
            end: "\\]",
            contains: ae("\\[", "\\]")
          },
          {
            begin: "%[Qwi]?\\{",
            end: /\}/,
            contains: ae(/\{/, /\}/)
          },
          {
            begin: "%[Qwi]?<",
            end: ">",
            contains: ae("<", ">")
          },
          {
            begin: "%[Qwi]?\\|",
            end: "\\|"
          },
          {
            begin: /<<-\w+$/,
            end: /^\s*\w+$/
          }
        ],
        relevance: 0
      }, se = {
        className: "string",
        variants: [
          {
            begin: "%q\\(",
            end: "\\)",
            contains: ae("\\(", "\\)")
          },
          {
            begin: "%q\\[",
            end: "\\]",
            contains: ae("\\[", "\\]")
          },
          {
            begin: "%q\\{",
            end: /\}/,
            contains: ae(/\{/, /\}/)
          },
          {
            begin: "%q<",
            end: ">",
            contains: ae("<", ">")
          },
          {
            begin: "%q\\|",
            end: "\\|"
          },
          {
            begin: /<<-'\w+'$/,
            end: /^\s*\w+$/
          }
        ],
        relevance: 0
      }, ce = {
        begin: "(?!%\\})(" + _.RE_STARTERS_RE + "|\\n|\\b(case|if|select|unless|until|when|while)\\b)\\s*",
        keywords: "case if select unless until when while",
        contains: [
          {
            className: "regexp",
            contains: [
              _.BACKSLASH_ESCAPE,
              te
            ],
            variants: [
              {
                begin: "//[a-z]*",
                relevance: 0
              },
              {
                begin: "/(?!\\/)",
                end: "/[a-z]*"
              }
            ]
          }
        ],
        relevance: 0
      }, le = {
        className: "regexp",
        contains: [
          _.BACKSLASH_ESCAPE,
          te
        ],
        variants: [
          {
            begin: "%r\\(",
            end: "\\)",
            contains: ae("\\(", "\\)")
          },
          {
            begin: "%r\\[",
            end: "\\]",
            contains: ae("\\[", "\\]")
          },
          {
            begin: "%r\\{",
            end: /\}/,
            contains: ae(/\{/, /\}/)
          },
          {
            begin: "%r<",
            end: ">",
            contains: ae("<", ">")
          },
          {
            begin: "%r\\|",
            end: "\\|"
          }
        ],
        relevance: 0
      }, ue = {
        className: "meta",
        begin: "@\\[",
        end: "\\]",
        contains: [_.inherit(_.QUOTE_STRING_MODE, { className: "string" })]
      }, de = [
        ee,
        oe,
        se,
        le,
        ce,
        ue,
        ne,
        _.HASH_COMMENT_MODE,
        {
          className: "class",
          beginKeywords: "class module struct",
          end: "$|;",
          illegal: /=/,
          contains: [
            _.HASH_COMMENT_MODE,
            _.inherit(_.TITLE_MODE, { begin: J }),
            {
              // relevance booster for inheritance
              begin: "<"
            }
          ]
        },
        {
          className: "class",
          beginKeywords: "lib enum union",
          end: "$|;",
          illegal: /=/,
          contains: [
            _.HASH_COMMENT_MODE,
            _.inherit(_.TITLE_MODE, { begin: J })
          ]
        },
        {
          beginKeywords: "annotation",
          end: "$|;",
          illegal: /=/,
          contains: [
            _.HASH_COMMENT_MODE,
            _.inherit(_.TITLE_MODE, { begin: J })
          ],
          relevance: 2
        },
        {
          className: "function",
          beginKeywords: "def",
          end: /\B\b/,
          contains: [
            _.inherit(_.TITLE_MODE, {
              begin: G,
              endsParent: !0
            })
          ]
        },
        {
          className: "function",
          beginKeywords: "fun macro",
          end: /\B\b/,
          contains: [
            _.inherit(_.TITLE_MODE, {
              begin: G,
              endsParent: !0
            })
          ],
          relevance: 2
        },
        {
          className: "symbol",
          begin: _.UNDERSCORE_IDENT_RE + "(!|\\?)?:",
          relevance: 0
        },
        {
          className: "symbol",
          begin: ":",
          contains: [
            oe,
            { begin: G }
          ],
          relevance: 0
        },
        {
          className: "number",
          variants: [
            { begin: "\\b0b([01_]+)" + H },
            { begin: "\\b0o([0-7_]+)" + H },
            { begin: "\\b0x([A-Fa-f0-9_]+)" + H },
            { begin: "\\b([1-9][0-9_]*[0-9]|[0-9])(\\.[0-9][0-9_]*)?([eE]_?[-+]?[0-9_]*)?" + V + "(?!_)" },
            { begin: "\\b([1-9][0-9_]*|0)" + H }
          ],
          relevance: 0
        }
      ];
      return te.contains = de, ee.contains = de.slice(1), {
        name: "Crystal",
        aliases: ["cr"],
        keywords: Z,
        contains: de
      };
    }
    return crystal_1 = O, crystal_1;
  }
  var csp_1, hasRequiredCsp;
  function requireCsp() {
    if (hasRequiredCsp) return csp_1;
    hasRequiredCsp = 1;
    function O(_) {
      return {
        name: "CSP",
        case_insensitive: !1,
        keywords: {
          $pattern: "[a-zA-Z][a-zA-Z0-9_-]*",
          keyword: [
            "base-uri",
            "child-src",
            "connect-src",
            "default-src",
            "font-src",
            "form-action",
            "frame-ancestors",
            "frame-src",
            "img-src",
            "manifest-src",
            "media-src",
            "object-src",
            "plugin-types",
            "report-uri",
            "sandbox",
            "script-src",
            "style-src",
            "trusted-types",
            "unsafe-hashes",
            "worker-src"
          ]
        },
        contains: [
          {
            className: "string",
            begin: "'",
            end: "'"
          },
          {
            className: "attribute",
            begin: "^Content",
            end: ":",
            excludeEnd: !0
          }
        ]
      };
    }
    return csp_1 = O, csp_1;
  }
  var d_1, hasRequiredD;
  function requireD() {
    if (hasRequiredD) return d_1;
    hasRequiredD = 1;
    function O(_) {
      const H = {
        $pattern: _.UNDERSCORE_IDENT_RE,
        keyword: "abstract alias align asm assert auto body break byte case cast catch class const continue debug default delete deprecated do else enum export extern final finally for foreach foreach_reverse|10 goto if immutable import in inout int interface invariant is lazy macro mixin module new nothrow out override package pragma private protected public pure ref return scope shared static struct super switch synchronized template this throw try typedef typeid typeof union unittest version void volatile while with __FILE__ __LINE__ __gshared|10 __thread __traits __DATE__ __EOF__ __TIME__ __TIMESTAMP__ __VENDOR__ __VERSION__",
        built_in: "bool cdouble cent cfloat char creal dchar delegate double dstring float function idouble ifloat ireal long real short string ubyte ucent uint ulong ushort wchar wstring",
        literal: "false null true"
      }, V = "(0|[1-9][\\d_]*)", q = "(0|[1-9][\\d_]*|\\d[\\d_]*|[\\d_]+?\\d)", G = "0[bB][01_]+", J = "([\\da-fA-F][\\da-fA-F_]*|_[\\da-fA-F][\\da-fA-F_]*)", Z = "0[xX]" + J, te = "([eE][+-]?" + q + ")", ne = "(" + q + "(\\.\\d*|" + te + ")|\\d+\\." + q + "|\\." + V + te + "?)", ee = "(0[xX](" + J + "\\." + J + "|\\.?" + J + ")[pP][+-]?" + q + ")", ae = "(" + V + "|" + G + "|" + Z + ")", oe = "(" + ee + "|" + ne + ")", se = `\\\\(['"\\?\\\\abfnrtv]|u[\\dA-Fa-f]{4}|[0-7]{1,3}|x[\\dA-Fa-f]{2}|U[\\dA-Fa-f]{8})|&[a-zA-Z\\d]{2,};`, ce = {
        className: "number",
        begin: "\\b" + ae + "(L|u|U|Lu|LU|uL|UL)?",
        relevance: 0
      }, le = {
        className: "number",
        begin: "\\b(" + oe + "([fF]|L|i|[fF]i|Li)?|" + ae + "(i|[fF]i|Li))",
        relevance: 0
      }, ue = {
        className: "string",
        begin: "'(" + se + "|.)",
        end: "'",
        illegal: "."
      }, fe = {
        className: "string",
        begin: '"',
        contains: [{
          begin: se,
          relevance: 0
        }],
        end: '"[cwd]?'
      }, he = {
        className: "string",
        begin: '[rq]"',
        end: '"[cwd]?',
        relevance: 5
      }, me = {
        className: "string",
        begin: "`",
        end: "`[cwd]?"
      }, pe = {
        className: "string",
        begin: 'x"[\\da-fA-F\\s\\n\\r]*"[cwd]?',
        relevance: 10
      }, ye = {
        className: "string",
        begin: 'q"\\{',
        end: '\\}"'
      }, ge = {
        className: "meta",
        begin: "^#!",
        end: "$",
        relevance: 5
      }, ve = {
        className: "meta",
        begin: "#(line)",
        end: "$",
        relevance: 5
      }, be = {
        className: "keyword",
        begin: "@[a-zA-Z_][a-zA-Z_\\d]*"
      }, Ce = _.COMMENT(
        "\\/\\+",
        "\\+\\/",
        {
          contains: ["self"],
          relevance: 10
        }
      );
      return {
        name: "D",
        keywords: H,
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          Ce,
          pe,
          fe,
          he,
          me,
          ye,
          le,
          ce,
          ue,
          ge,
          ve,
          be
        ]
      };
    }
    return d_1 = O, d_1;
  }
  var dart_1, hasRequiredDart;
  function requireDart() {
    if (hasRequiredDart) return dart_1;
    hasRequiredDart = 1;
    function O(_) {
      const H = {
        className: "subst",
        variants: [{ begin: "\\$[A-Za-z0-9_]+" }]
      }, V = {
        className: "subst",
        variants: [
          {
            begin: /\$\{/,
            end: /\}/
          }
        ],
        keywords: "true false null this is new super"
      }, q = {
        className: "number",
        relevance: 0,
        variants: [
          { match: /\b[0-9][0-9_]*(\.[0-9][0-9_]*)?([eE][+-]?[0-9][0-9_]*)?\b/ },
          { match: /\b0[xX][0-9A-Fa-f][0-9A-Fa-f_]*\b/ }
        ]
      }, G = {
        className: "string",
        variants: [
          {
            begin: "r'''",
            end: "'''"
          },
          {
            begin: 'r"""',
            end: '"""'
          },
          {
            begin: "r'",
            end: "'",
            illegal: "\\n"
          },
          {
            begin: 'r"',
            end: '"',
            illegal: "\\n"
          },
          {
            begin: "'''",
            end: "'''",
            contains: [
              _.BACKSLASH_ESCAPE,
              H,
              V
            ]
          },
          {
            begin: '"""',
            end: '"""',
            contains: [
              _.BACKSLASH_ESCAPE,
              H,
              V
            ]
          },
          {
            begin: "'",
            end: "'",
            illegal: "\\n",
            contains: [
              _.BACKSLASH_ESCAPE,
              H,
              V
            ]
          },
          {
            begin: '"',
            end: '"',
            illegal: "\\n",
            contains: [
              _.BACKSLASH_ESCAPE,
              H,
              V
            ]
          }
        ]
      };
      V.contains = [
        q,
        G
      ];
      const J = [
        // dart:core
        "Comparable",
        "DateTime",
        "Duration",
        "Function",
        "Iterable",
        "Iterator",
        "List",
        "Map",
        "Match",
        "Object",
        "Pattern",
        "RegExp",
        "Set",
        "Stopwatch",
        "String",
        "StringBuffer",
        "StringSink",
        "Symbol",
        "Type",
        "Uri",
        "bool",
        "double",
        "int",
        "num",
        // dart:html
        "Element",
        "ElementList"
      ], Z = J.map((ee) => `${ee}?`);
      return {
        name: "Dart",
        keywords: {
          keyword: [
            "abstract",
            "as",
            "assert",
            "async",
            "await",
            "base",
            "break",
            "case",
            "catch",
            "class",
            "const",
            "continue",
            "covariant",
            "default",
            "deferred",
            "do",
            "dynamic",
            "else",
            "enum",
            "export",
            "extends",
            "extension",
            "external",
            "factory",
            "false",
            "final",
            "finally",
            "for",
            "Function",
            "get",
            "hide",
            "if",
            "implements",
            "import",
            "in",
            "interface",
            "is",
            "late",
            "library",
            "mixin",
            "new",
            "null",
            "on",
            "operator",
            "part",
            "required",
            "rethrow",
            "return",
            "sealed",
            "set",
            "show",
            "static",
            "super",
            "switch",
            "sync",
            "this",
            "throw",
            "true",
            "try",
            "typedef",
            "var",
            "void",
            "when",
            "while",
            "with",
            "yield"
          ],
          built_in: J.concat(Z).concat([
            // dart:core
            "Never",
            "Null",
            "dynamic",
            "print",
            // dart:html
            "document",
            "querySelector",
            "querySelectorAll",
            "window"
          ]),
          $pattern: /[A-Za-z][A-Za-z0-9_]*\??/
        },
        contains: [
          G,
          _.COMMENT(
            /\/\*\*(?!\/)/,
            /\*\//,
            {
              subLanguage: "markdown",
              relevance: 0
            }
          ),
          _.COMMENT(
            /\/{3,} ?/,
            /$/,
            { contains: [
              {
                subLanguage: "markdown",
                begin: ".",
                end: "$",
                relevance: 0
              }
            ] }
          ),
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "class",
            beginKeywords: "class interface",
            end: /\{/,
            excludeEnd: !0,
            contains: [
              { beginKeywords: "extends implements" },
              _.UNDERSCORE_TITLE_MODE
            ]
          },
          q,
          {
            className: "meta",
            begin: "@[A-Za-z]+"
          },
          {
            begin: "=>"
            // No markup, just a relevance booster
          }
        ]
      };
    }
    return dart_1 = O, dart_1;
  }
  var delphi_1, hasRequiredDelphi;
  function requireDelphi() {
    if (hasRequiredDelphi) return delphi_1;
    hasRequiredDelphi = 1;
    function O(_) {
      const H = [
        "exports",
        "register",
        "file",
        "shl",
        "array",
        "record",
        "property",
        "for",
        "mod",
        "while",
        "set",
        "ally",
        "label",
        "uses",
        "raise",
        "not",
        "stored",
        "class",
        "safecall",
        "var",
        "interface",
        "or",
        "private",
        "static",
        "exit",
        "index",
        "inherited",
        "to",
        "else",
        "stdcall",
        "override",
        "shr",
        "asm",
        "far",
        "resourcestring",
        "finalization",
        "packed",
        "virtual",
        "out",
        "and",
        "protected",
        "library",
        "do",
        "xorwrite",
        "goto",
        "near",
        "function",
        "end",
        "div",
        "overload",
        "object",
        "unit",
        "begin",
        "string",
        "on",
        "inline",
        "repeat",
        "until",
        "destructor",
        "write",
        "message",
        "program",
        "with",
        "read",
        "initialization",
        "except",
        "default",
        "nil",
        "if",
        "case",
        "cdecl",
        "in",
        "downto",
        "threadvar",
        "of",
        "try",
        "pascal",
        "const",
        "external",
        "constructor",
        "type",
        "public",
        "then",
        "implementation",
        "finally",
        "published",
        "procedure",
        "absolute",
        "reintroduce",
        "operator",
        "as",
        "is",
        "abstract",
        "alias",
        "assembler",
        "bitpacked",
        "break",
        "continue",
        "cppdecl",
        "cvar",
        "enumerator",
        "experimental",
        "platform",
        "deprecated",
        "unimplemented",
        "dynamic",
        "export",
        "far16",
        "forward",
        "generic",
        "helper",
        "implements",
        "interrupt",
        "iochecks",
        "local",
        "name",
        "nodefault",
        "noreturn",
        "nostackframe",
        "oldfpccall",
        "otherwise",
        "saveregisters",
        "softfloat",
        "specialize",
        "strict",
        "unaligned",
        "varargs"
      ], V = [
        _.C_LINE_COMMENT_MODE,
        _.COMMENT(/\{/, /\}/, { relevance: 0 }),
        _.COMMENT(/\(\*/, /\*\)/, { relevance: 10 })
      ], q = {
        className: "meta",
        variants: [
          {
            begin: /\{\$/,
            end: /\}/
          },
          {
            begin: /\(\*\$/,
            end: /\*\)/
          }
        ]
      }, G = {
        className: "string",
        begin: /'/,
        end: /'/,
        contains: [{ begin: /''/ }]
      }, J = {
        className: "number",
        relevance: 0,
        // Source: https://www.freepascal.org/docs-html/ref/refse6.html
        variants: [
          {
            // Regular numbers, e.g., 123, 123.456.
            match: /\b\d[\d_]*(\.\d[\d_]*)?/
          },
          {
            // Hexadecimal notation, e.g., $7F.
            match: /\$[\dA-Fa-f_]+/
          },
          {
            // Hexadecimal literal with no digits
            match: /\$/,
            relevance: 0
          },
          {
            // Octal notation, e.g., &42.
            match: /&[0-7][0-7_]*/
          },
          {
            // Binary notation, e.g., %1010.
            match: /%[01_]+/
          },
          {
            // Binary literal with no digits
            match: /%/,
            relevance: 0
          }
        ]
      }, Z = {
        className: "string",
        variants: [
          { match: /#\d[\d_]*/ },
          { match: /#\$[\dA-Fa-f][\dA-Fa-f_]*/ },
          { match: /#&[0-7][0-7_]*/ },
          { match: /#%[01][01_]*/ }
        ]
      }, te = {
        begin: _.IDENT_RE + "\\s*=\\s*class\\s*\\(",
        returnBegin: !0,
        contains: [_.TITLE_MODE]
      }, ne = {
        className: "function",
        beginKeywords: "function constructor destructor procedure",
        end: /[:;]/,
        keywords: "function constructor|10 destructor|10 procedure|10",
        contains: [
          _.TITLE_MODE,
          {
            className: "params",
            begin: /\(/,
            end: /\)/,
            keywords: H,
            contains: [
              G,
              Z,
              q
            ].concat(V)
          },
          q
        ].concat(V)
      };
      return {
        name: "Delphi",
        aliases: [
          "dpr",
          "dfm",
          "pas",
          "pascal"
        ],
        case_insensitive: !0,
        keywords: H,
        illegal: /"|\$[G-Zg-z]|\/\*|<\/|\|/,
        contains: [
          G,
          Z,
          J,
          te,
          ne,
          q
        ].concat(V)
      };
    }
    return delphi_1 = O, delphi_1;
  }
  var django_1, hasRequiredDjango;
  function requireDjango() {
    if (hasRequiredDjango) return django_1;
    hasRequiredDjango = 1;
    function O(_) {
      const H = {
        begin: /\|[A-Za-z]+:?/,
        keywords: { name: "truncatewords removetags linebreaksbr yesno get_digit timesince random striptags filesizeformat escape linebreaks length_is ljust rjust cut urlize fix_ampersands title floatformat capfirst pprint divisibleby add make_list unordered_list urlencode timeuntil urlizetrunc wordcount stringformat linenumbers slice date dictsort dictsortreversed default_if_none pluralize lower join center default truncatewords_html upper length phone2numeric wordwrap time addslashes slugify first escapejs force_escape iriencode last safe safeseq truncatechars localize unlocalize localtime utc timezone" },
        contains: [
          _.QUOTE_STRING_MODE,
          _.APOS_STRING_MODE
        ]
      };
      return {
        name: "Django",
        aliases: ["jinja"],
        case_insensitive: !0,
        subLanguage: "xml",
        contains: [
          _.COMMENT(/\{%\s*comment\s*%\}/, /\{%\s*endcomment\s*%\}/),
          _.COMMENT(/\{#/, /#\}/),
          {
            className: "template-tag",
            begin: /\{%/,
            end: /%\}/,
            contains: [
              {
                className: "name",
                begin: /\w+/,
                keywords: { name: "comment endcomment load templatetag ifchanged endifchanged if endif firstof for endfor ifnotequal endifnotequal widthratio extends include spaceless endspaceless regroup ifequal endifequal ssi now with cycle url filter endfilter debug block endblock else autoescape endautoescape csrf_token empty elif endwith static trans blocktrans endblocktrans get_static_prefix get_media_prefix plural get_current_language language get_available_languages get_current_language_bidi get_language_info get_language_info_list localize endlocalize localtime endlocaltime timezone endtimezone get_current_timezone verbatim" },
                starts: {
                  endsWithParent: !0,
                  keywords: "in by as",
                  contains: [H],
                  relevance: 0
                }
              }
            ]
          },
          {
            className: "template-variable",
            begin: /\{\{/,
            end: /\}\}/,
            contains: [H]
          }
        ]
      };
    }
    return django_1 = O, django_1;
  }
  var dns_1, hasRequiredDns;
  function requireDns() {
    if (hasRequiredDns) return dns_1;
    hasRequiredDns = 1;
    function O(_) {
      return {
        name: "DNS Zone",
        aliases: [
          "bind",
          "zone"
        ],
        keywords: [
          "IN",
          "A",
          "AAAA",
          "AFSDB",
          "APL",
          "CAA",
          "CDNSKEY",
          "CDS",
          "CERT",
          "CNAME",
          "DHCID",
          "DLV",
          "DNAME",
          "DNSKEY",
          "DS",
          "HIP",
          "IPSECKEY",
          "KEY",
          "KX",
          "LOC",
          "MX",
          "NAPTR",
          "NS",
          "NSEC",
          "NSEC3",
          "NSEC3PARAM",
          "PTR",
          "RRSIG",
          "RP",
          "SIG",
          "SOA",
          "SRV",
          "SSHFP",
          "TA",
          "TKEY",
          "TLSA",
          "TSIG",
          "TXT"
        ],
        contains: [
          _.COMMENT(";", "$", { relevance: 0 }),
          {
            className: "meta",
            begin: /^\$(TTL|GENERATE|INCLUDE|ORIGIN)\b/
          },
          // IPv6
          {
            className: "number",
            begin: "((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:)))\\b"
          },
          // IPv4
          {
            className: "number",
            begin: "((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]).){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\\b"
          },
          _.inherit(_.NUMBER_MODE, { begin: /\b\d+[dhwm]?/ })
        ]
      };
    }
    return dns_1 = O, dns_1;
  }
  var dockerfile_1, hasRequiredDockerfile;
  function requireDockerfile() {
    if (hasRequiredDockerfile) return dockerfile_1;
    hasRequiredDockerfile = 1;
    function O(_) {
      return {
        name: "Dockerfile",
        aliases: ["docker"],
        case_insensitive: !0,
        keywords: [
          "from",
          "maintainer",
          "expose",
          "env",
          "arg",
          "user",
          "onbuild",
          "stopsignal"
        ],
        contains: [
          _.HASH_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE,
          {
            beginKeywords: "run cmd entrypoint volume add copy workdir label healthcheck shell",
            starts: {
              end: /[^\\]$/,
              subLanguage: "bash"
            }
          }
        ],
        illegal: "</"
      };
    }
    return dockerfile_1 = O, dockerfile_1;
  }
  var dos_1, hasRequiredDos;
  function requireDos() {
    if (hasRequiredDos) return dos_1;
    hasRequiredDos = 1;
    function O(_) {
      const H = _.COMMENT(
        /^\s*@?rem\b/,
        /$/,
        { relevance: 10 }
      );
      return {
        name: "Batch file (DOS)",
        aliases: [
          "bat",
          "cmd"
        ],
        case_insensitive: !0,
        illegal: /\/\*/,
        keywords: {
          keyword: [
            "if",
            "else",
            "goto",
            "for",
            "in",
            "do",
            "call",
            "exit",
            "not",
            "exist",
            "errorlevel",
            "defined",
            "equ",
            "neq",
            "lss",
            "leq",
            "gtr",
            "geq"
          ],
          built_in: [
            "prn",
            "nul",
            "lpt3",
            "lpt2",
            "lpt1",
            "con",
            "com4",
            "com3",
            "com2",
            "com1",
            "aux",
            "shift",
            "cd",
            "dir",
            "echo",
            "setlocal",
            "endlocal",
            "set",
            "pause",
            "copy",
            "append",
            "assoc",
            "at",
            "attrib",
            "break",
            "cacls",
            "cd",
            "chcp",
            "chdir",
            "chkdsk",
            "chkntfs",
            "cls",
            "cmd",
            "color",
            "comp",
            "compact",
            "convert",
            "date",
            "dir",
            "diskcomp",
            "diskcopy",
            "doskey",
            "erase",
            "fs",
            "find",
            "findstr",
            "format",
            "ftype",
            "graftabl",
            "help",
            "keyb",
            "label",
            "md",
            "mkdir",
            "mode",
            "more",
            "move",
            "path",
            "pause",
            "print",
            "popd",
            "pushd",
            "promt",
            "rd",
            "recover",
            "rem",
            "rename",
            "replace",
            "restore",
            "rmdir",
            "shift",
            "sort",
            "start",
            "subst",
            "time",
            "title",
            "tree",
            "type",
            "ver",
            "verify",
            "vol",
            // winutils
            "ping",
            "net",
            "ipconfig",
            "taskkill",
            "xcopy",
            "ren",
            "del"
          ]
        },
        contains: [
          {
            className: "variable",
            begin: /%%[^ ]|%[^ ]+?%|![^ ]+?!/
          },
          {
            className: "function",
            begin: {
              className: "symbol",
              begin: "^\\s*[A-Za-z._?][A-Za-z0-9_$#@~.?]*(:|\\s+label)",
              relevance: 0
            }.begin,
            end: "goto:eof",
            contains: [
              _.inherit(_.TITLE_MODE, { begin: "([_a-zA-Z]\\w*\\.)*([_a-zA-Z]\\w*:)?[_a-zA-Z]\\w*" }),
              H
            ]
          },
          {
            className: "number",
            begin: "\\b\\d+",
            relevance: 0
          },
          H
        ]
      };
    }
    return dos_1 = O, dos_1;
  }
  var dsconfig_1, hasRequiredDsconfig;
  function requireDsconfig() {
    if (hasRequiredDsconfig) return dsconfig_1;
    hasRequiredDsconfig = 1;
    function O(_) {
      return {
        keywords: "dsconfig",
        contains: [
          {
            className: "keyword",
            begin: "^dsconfig",
            end: /\s/,
            excludeEnd: !0,
            relevance: 10
          },
          {
            className: "built_in",
            begin: /(list|create|get|set|delete)-(\w+)/,
            end: /\s/,
            excludeEnd: !0,
            illegal: "!@#$%^&*()",
            relevance: 10
          },
          {
            className: "built_in",
            begin: /--(\w+)/,
            end: /\s/,
            excludeEnd: !0
          },
          {
            className: "string",
            begin: /"/,
            end: /"/
          },
          {
            className: "string",
            begin: /'/,
            end: /'/
          },
          {
            className: "string",
            begin: /[\w\-?]+:\w+/,
            end: /\W/,
            relevance: 0
          },
          {
            className: "string",
            begin: /\w+(\-\w+)*/,
            end: /(?=\W)/,
            relevance: 0
          },
          _.HASH_COMMENT_MODE
        ]
      };
    }
    return dsconfig_1 = O, dsconfig_1;
  }
  var dts_1, hasRequiredDts;
  function requireDts() {
    if (hasRequiredDts) return dts_1;
    hasRequiredDts = 1;
    function O(_) {
      const H = {
        className: "string",
        variants: [
          _.inherit(_.QUOTE_STRING_MODE, { begin: '((u8?|U)|L)?"' }),
          {
            begin: '(u8?|U)?R"',
            end: '"',
            contains: [_.BACKSLASH_ESCAPE]
          },
          {
            begin: "'\\\\?.",
            end: "'",
            illegal: "."
          }
        ]
      }, V = {
        className: "number",
        variants: [
          { begin: "\\b(\\d+(\\.\\d*)?|\\.\\d+)(u|U|l|L|ul|UL|f|F)" },
          { begin: _.C_NUMBER_RE }
        ],
        relevance: 0
      }, q = {
        className: "meta",
        begin: "#",
        end: "$",
        keywords: { keyword: "if else elif endif define undef ifdef ifndef" },
        contains: [
          {
            begin: /\\\n/,
            relevance: 0
          },
          {
            beginKeywords: "include",
            end: "$",
            keywords: { keyword: "include" },
            contains: [
              _.inherit(H, { className: "string" }),
              {
                className: "string",
                begin: "<",
                end: ">",
                illegal: "\\n"
              }
            ]
          },
          H,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE
        ]
      }, G = {
        className: "variable",
        begin: /&[a-z\d_]*\b/
      }, J = {
        className: "keyword",
        begin: "/[a-z][a-z\\d-]*/"
      }, Z = {
        className: "symbol",
        begin: "^\\s*[a-zA-Z_][a-zA-Z\\d_]*:"
      }, te = {
        className: "params",
        relevance: 0,
        begin: "<",
        end: ">",
        contains: [
          V,
          G
        ]
      }, ne = {
        className: "title.class",
        begin: /[a-zA-Z_][a-zA-Z\d_@-]*(?=\s\{)/,
        relevance: 0.2
      }, ee = {
        className: "title.class",
        begin: /^\/(?=\s*\{)/,
        relevance: 10
      }, ae = {
        match: /[a-z][a-z-,]+(?=;)/,
        relevance: 0,
        scope: "attr"
      }, oe = {
        relevance: 0,
        match: [
          /[a-z][a-z-,]+/,
          /\s*/,
          /=/
        ],
        scope: {
          1: "attr",
          3: "operator"
        }
      }, se = {
        scope: "punctuation",
        relevance: 0,
        // `};` combined is just to avoid tons of useless punctuation nodes
        match: /\};|[;{}]/
      };
      return {
        name: "Device Tree",
        contains: [
          ee,
          G,
          J,
          Z,
          ne,
          oe,
          ae,
          te,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          V,
          H,
          q,
          se,
          {
            begin: _.IDENT_RE + "::",
            keywords: ""
          }
        ]
      };
    }
    return dts_1 = O, dts_1;
  }
  var dust_1, hasRequiredDust;
  function requireDust() {
    if (hasRequiredDust) return dust_1;
    hasRequiredDust = 1;
    function O(_) {
      return {
        name: "Dust",
        aliases: ["dst"],
        case_insensitive: !0,
        subLanguage: "xml",
        contains: [
          {
            className: "template-tag",
            begin: /\{[#\/]/,
            end: /\}/,
            illegal: /;/,
            contains: [
              {
                className: "name",
                begin: /[a-zA-Z\.-]+/,
                starts: {
                  endsWithParent: !0,
                  relevance: 0,
                  contains: [_.QUOTE_STRING_MODE]
                }
              }
            ]
          },
          {
            className: "template-variable",
            begin: /\{/,
            end: /\}/,
            illegal: /;/,
            keywords: "if eq ne lt lte gt gte select default math sep"
          }
        ]
      };
    }
    return dust_1 = O, dust_1;
  }
  var ebnf_1, hasRequiredEbnf;
  function requireEbnf() {
    if (hasRequiredEbnf) return ebnf_1;
    hasRequiredEbnf = 1;
    function O(_) {
      const H = _.COMMENT(/\(\*/, /\*\)/), V = {
        className: "attribute",
        begin: /^[ ]*[a-zA-Z]+([\s_-]+[a-zA-Z]+)*/
      }, G = {
        begin: /=/,
        end: /[.;]/,
        contains: [
          H,
          {
            className: "meta",
            begin: /\?.*\?/
          },
          {
            // terminals
            className: "string",
            variants: [
              _.APOS_STRING_MODE,
              _.QUOTE_STRING_MODE,
              {
                begin: "`",
                end: "`"
              }
            ]
          }
        ]
      };
      return {
        name: "Extended Backus-Naur Form",
        illegal: /\S/,
        contains: [
          H,
          V,
          G
        ]
      };
    }
    return ebnf_1 = O, ebnf_1;
  }
  var elixir_1, hasRequiredElixir;
  function requireElixir() {
    if (hasRequiredElixir) return elixir_1;
    hasRequiredElixir = 1;
    function O(_) {
      const H = _.regex, V = "[a-zA-Z_][a-zA-Z0-9_.]*(!|\\?)?", q = "[a-zA-Z_]\\w*[!?=]?|[-+~]@|<<|>>|=~|===?|<=>|[<>]=?|\\*\\*|[-/+%^&*~`|]|\\[\\]=?", Z = {
        $pattern: V,
        keyword: [
          "after",
          "alias",
          "and",
          "case",
          "catch",
          "cond",
          "defstruct",
          "defguard",
          "do",
          "else",
          "end",
          "fn",
          "for",
          "if",
          "import",
          "in",
          "not",
          "or",
          "quote",
          "raise",
          "receive",
          "require",
          "reraise",
          "rescue",
          "try",
          "unless",
          "unquote",
          "unquote_splicing",
          "use",
          "when",
          "with|0"
        ],
        literal: [
          "false",
          "nil",
          "true"
        ]
      }, te = {
        className: "subst",
        begin: /#\{/,
        end: /\}/,
        keywords: Z
      }, ne = {
        className: "number",
        begin: "(\\b0o[0-7_]+)|(\\b0b[01_]+)|(\\b0x[0-9a-fA-F_]+)|(-?\\b[0-9][0-9_]*(\\.[0-9_]+([eE][-+]?[0-9]+)?)?)",
        relevance: 0
      }, ae = {
        match: /\\[\s\S]/,
        scope: "char.escape",
        relevance: 0
      }, oe = `[/|([{<"']`, se = [
        {
          begin: /"/,
          end: /"/
        },
        {
          begin: /'/,
          end: /'/
        },
        {
          begin: /\//,
          end: /\//
        },
        {
          begin: /\|/,
          end: /\|/
        },
        {
          begin: /\(/,
          end: /\)/
        },
        {
          begin: /\[/,
          end: /\]/
        },
        {
          begin: /\{/,
          end: /\}/
        },
        {
          begin: /</,
          end: />/
        }
      ], ce = (ye) => ({
        scope: "char.escape",
        begin: H.concat(/\\/, ye),
        relevance: 0
      }), le = {
        className: "string",
        begin: "~[a-z](?=" + oe + ")",
        contains: se.map((ye) => _.inherit(
          ye,
          { contains: [
            ce(ye.end),
            ae,
            te
          ] }
        ))
      }, ue = {
        className: "string",
        begin: "~[A-Z](?=" + oe + ")",
        contains: se.map((ye) => _.inherit(
          ye,
          { contains: [ce(ye.end)] }
        ))
      }, de = {
        className: "regex",
        variants: [
          {
            begin: "~r(?=" + oe + ")",
            contains: se.map((ye) => _.inherit(
              ye,
              {
                end: H.concat(ye.end, /[uismxfU]{0,7}/),
                contains: [
                  ce(ye.end),
                  ae,
                  te
                ]
              }
            ))
          },
          {
            begin: "~R(?=" + oe + ")",
            contains: se.map(
              (ye) => _.inherit(
                ye,
                {
                  end: H.concat(ye.end, /[uismxfU]{0,7}/),
                  contains: [ce(ye.end)]
                }
              )
            )
          }
        ]
      }, fe = {
        className: "string",
        contains: [
          _.BACKSLASH_ESCAPE,
          te
        ],
        variants: [
          {
            begin: /"""/,
            end: /"""/
          },
          {
            begin: /'''/,
            end: /'''/
          },
          {
            begin: /~S"""/,
            end: /"""/,
            contains: []
            // override default
          },
          {
            begin: /~S"/,
            end: /"/,
            contains: []
            // override default
          },
          {
            begin: /~S'''/,
            end: /'''/,
            contains: []
            // override default
          },
          {
            begin: /~S'/,
            end: /'/,
            contains: []
            // override default
          },
          {
            begin: /'/,
            end: /'/
          },
          {
            begin: /"/,
            end: /"/
          }
        ]
      }, he = {
        className: "function",
        beginKeywords: "def defp defmacro defmacrop",
        end: /\B\b/,
        // the mode is ended by the title
        contains: [
          _.inherit(_.TITLE_MODE, {
            begin: V,
            endsParent: !0
          })
        ]
      }, me = _.inherit(he, {
        className: "class",
        beginKeywords: "defimpl defmodule defprotocol defrecord",
        end: /\bdo\b|$|;/
      }), pe = [
        fe,
        de,
        ue,
        le,
        _.HASH_COMMENT_MODE,
        me,
        he,
        { begin: "::" },
        {
          className: "symbol",
          begin: ":(?![\\s:])",
          contains: [
            fe,
            { begin: q }
          ],
          relevance: 0
        },
        {
          className: "symbol",
          begin: V + ":(?!:)",
          relevance: 0
        },
        {
          // Usage of a module, struct, etc.
          className: "title.class",
          begin: /(\b[A-Z][a-zA-Z0-9_]+)/,
          relevance: 0
        },
        ne,
        {
          className: "variable",
          begin: "(\\$\\W)|((\\$|@@?)(\\w+))"
        }
        // -> has been removed, capnproto always uses this grammar construct
      ];
      return te.contains = pe, {
        name: "Elixir",
        aliases: [
          "ex",
          "exs"
        ],
        keywords: Z,
        contains: pe
      };
    }
    return elixir_1 = O, elixir_1;
  }
  var elm_1, hasRequiredElm;
  function requireElm() {
    if (hasRequiredElm) return elm_1;
    hasRequiredElm = 1;
    function O(_) {
      const H = { variants: [
        _.COMMENT("--", "$"),
        _.COMMENT(
          /\{-/,
          /-\}/,
          { contains: ["self"] }
        )
      ] }, V = {
        className: "type",
        begin: "\\b[A-Z][\\w']*",
        // TODO: other constructors (built-in, infix).
        relevance: 0
      }, q = {
        begin: "\\(",
        end: "\\)",
        illegal: '"',
        contains: [
          {
            className: "type",
            begin: "\\b[A-Z][\\w]*(\\((\\.\\.|,|\\w+)\\))?"
          },
          H
        ]
      }, G = {
        begin: /\{/,
        end: /\}/,
        contains: q.contains
      }, J = {
        className: "string",
        begin: "'\\\\?.",
        end: "'",
        illegal: "."
      };
      return {
        name: "Elm",
        keywords: [
          "let",
          "in",
          "if",
          "then",
          "else",
          "case",
          "of",
          "where",
          "module",
          "import",
          "exposing",
          "type",
          "alias",
          "as",
          "infix",
          "infixl",
          "infixr",
          "port",
          "effect",
          "command",
          "subscription"
        ],
        contains: [
          // Top-level constructions.
          {
            beginKeywords: "port effect module",
            end: "exposing",
            keywords: "port effect module where command subscription exposing",
            contains: [
              q,
              H
            ],
            illegal: "\\W\\.|;"
          },
          {
            begin: "import",
            end: "$",
            keywords: "import as exposing",
            contains: [
              q,
              H
            ],
            illegal: "\\W\\.|;"
          },
          {
            begin: "type",
            end: "$",
            keywords: "type alias",
            contains: [
              V,
              q,
              G,
              H
            ]
          },
          {
            beginKeywords: "infix infixl infixr",
            end: "$",
            contains: [
              _.C_NUMBER_MODE,
              H
            ]
          },
          {
            begin: "port",
            end: "$",
            keywords: "port",
            contains: [H]
          },
          // Literals and names.
          J,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE,
          V,
          _.inherit(_.TITLE_MODE, { begin: "^[_a-z][\\w']*" }),
          H,
          {
            // No markup, relevance booster
            begin: "->|<-"
          }
        ],
        illegal: /;/
      };
    }
    return elm_1 = O, elm_1;
  }
  var erb_1, hasRequiredErb;
  function requireErb() {
    if (hasRequiredErb) return erb_1;
    hasRequiredErb = 1;
    function O(_) {
      return {
        name: "ERB",
        subLanguage: "xml",
        contains: [
          _.COMMENT("<%#", "%>"),
          {
            begin: "<%[%=-]?",
            end: "[%-]?%>",
            subLanguage: "ruby",
            excludeBegin: !0,
            excludeEnd: !0
          }
        ]
      };
    }
    return erb_1 = O, erb_1;
  }
  var erlangRepl_1, hasRequiredErlangRepl;
  function requireErlangRepl() {
    if (hasRequiredErlangRepl) return erlangRepl_1;
    hasRequiredErlangRepl = 1;
    function O(_) {
      const H = _.regex;
      return {
        name: "Erlang REPL",
        keywords: {
          built_in: "spawn spawn_link self",
          keyword: "after and andalso|10 band begin bnot bor bsl bsr bxor case catch cond div end fun if let not of or orelse|10 query receive rem try when xor"
        },
        contains: [
          {
            className: "meta.prompt",
            begin: "^[0-9]+> ",
            relevance: 10
          },
          _.COMMENT("%", "$"),
          {
            className: "number",
            begin: "\\b(\\d+(_\\d+)*#[a-fA-F0-9]+(_[a-fA-F0-9]+)*|\\d+(_\\d+)*(\\.\\d+(_\\d+)*)?([eE][-+]?\\d+)?)",
            relevance: 0
          },
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          { begin: H.concat(
            /\?(::)?/,
            /([A-Z]\w*)/,
            // at least one identifier
            /((::)[A-Z]\w*)*/
            // perhaps more
          ) },
          { begin: "->" },
          { begin: "ok" },
          { begin: "!" },
          {
            begin: "(\\b[a-z'][a-zA-Z0-9_']*:[a-z'][a-zA-Z0-9_']*)|(\\b[a-z'][a-zA-Z0-9_']*)",
            relevance: 0
          },
          {
            begin: "[A-Z][a-zA-Z0-9_']*",
            relevance: 0
          }
        ]
      };
    }
    return erlangRepl_1 = O, erlangRepl_1;
  }
  var erlang_1, hasRequiredErlang;
  function requireErlang() {
    if (hasRequiredErlang) return erlang_1;
    hasRequiredErlang = 1;
    function O(_) {
      const H = "[a-z'][a-zA-Z0-9_']*", V = "(" + H + ":" + H + "|" + H + ")", q = {
        keyword: "after and andalso|10 band begin bnot bor bsl bzr bxor case catch cond div end fun if let not of orelse|10 query receive rem try when xor maybe else",
        literal: "false true"
      }, G = _.COMMENT("%", "$"), J = {
        className: "number",
        begin: "\\b(\\d+(_\\d+)*#[a-fA-F0-9]+(_[a-fA-F0-9]+)*|\\d+(_\\d+)*(\\.\\d+(_\\d+)*)?([eE][-+]?\\d+)?)",
        relevance: 0
      }, Z = { begin: "fun\\s+" + H + "/\\d+" }, te = {
        begin: V + "\\(",
        end: "\\)",
        returnBegin: !0,
        relevance: 0,
        contains: [
          {
            begin: V,
            relevance: 0
          },
          {
            begin: "\\(",
            end: "\\)",
            endsWithParent: !0,
            returnEnd: !0,
            relevance: 0
            // "contains" defined later
          }
        ]
      }, ne = {
        begin: /\{/,
        end: /\}/,
        relevance: 0
        // "contains" defined later
      }, ee = {
        begin: "\\b_([A-Z][A-Za-z0-9_]*)?",
        relevance: 0
      }, ae = {
        begin: "[A-Z][a-zA-Z0-9_]*",
        relevance: 0
      }, oe = {
        begin: "#" + _.UNDERSCORE_IDENT_RE,
        relevance: 0,
        returnBegin: !0,
        contains: [
          {
            begin: "#" + _.UNDERSCORE_IDENT_RE,
            relevance: 0
          },
          {
            begin: /\{/,
            end: /\}/,
            relevance: 0
            // "contains" defined later
          }
        ]
      }, se = {
        scope: "string",
        match: /\$(\\([^0-9]|[0-9]{1,3}|)|.)/
      }, ce = {
        scope: "string",
        match: /"""("*)(?!")[\s\S]*?"""\1/
      }, le = {
        scope: "string",
        contains: [_.BACKSLASH_ESCAPE],
        variants: [
          { match: /~\w?"""("*)(?!")[\s\S]*?"""\1/ },
          { begin: /~\w?\(/, end: /\)/ },
          { begin: /~\w?\[/, end: /\]/ },
          { begin: /~\w?{/, end: /}/ },
          { begin: /~\w?</, end: />/ },
          { begin: /~\w?\//, end: /\// },
          { begin: /~\w?\|/, end: /\|/ },
          { begin: /~\w?'/, end: /'/ },
          { begin: /~\w?"/, end: /"/ },
          { begin: /~\w?`/, end: /`/ },
          { begin: /~\w?#/, end: /#/ }
        ]
      }, ue = {
        beginKeywords: "fun receive if try case maybe",
        end: "end",
        keywords: q
      };
      ue.contains = [
        G,
        Z,
        _.inherit(_.APOS_STRING_MODE, { className: "" }),
        ue,
        te,
        le,
        ce,
        _.QUOTE_STRING_MODE,
        J,
        ne,
        ee,
        ae,
        oe,
        se
      ];
      const de = [
        G,
        Z,
        ue,
        te,
        le,
        ce,
        _.QUOTE_STRING_MODE,
        J,
        ne,
        ee,
        ae,
        oe,
        se
      ];
      te.contains[1].contains = de, ne.contains = de, oe.contains[1].contains = de;
      const fe = [
        "-module",
        "-record",
        "-undef",
        "-export",
        "-ifdef",
        "-ifndef",
        "-author",
        "-copyright",
        "-doc",
        "-moduledoc",
        "-vsn",
        "-import",
        "-include",
        "-include_lib",
        "-compile",
        "-define",
        "-else",
        "-endif",
        "-file",
        "-behaviour",
        "-behavior",
        "-spec",
        "-on_load",
        "-nifs"
      ], he = {
        className: "params",
        begin: "\\(",
        end: "\\)",
        contains: de
      };
      return {
        name: "Erlang",
        aliases: ["erl"],
        keywords: q,
        illegal: "(</|\\*=|\\+=|-=|/\\*|\\*/|\\(\\*|\\*\\))",
        contains: [
          {
            className: "function",
            begin: "^" + H + "\\s*\\(",
            end: "->",
            returnBegin: !0,
            illegal: "\\(|#|//|/\\*|\\\\|:|;",
            contains: [
              he,
              _.inherit(_.TITLE_MODE, { begin: H })
            ],
            starts: {
              end: ";|\\.",
              keywords: q,
              contains: de
            }
          },
          G,
          {
            begin: "^-",
            end: "\\.",
            relevance: 0,
            excludeEnd: !0,
            returnBegin: !0,
            keywords: {
              $pattern: "-" + _.IDENT_RE,
              keyword: fe.map((me) => `${me}|1.5`).join(" ")
            },
            contains: [
              he,
              le,
              ce,
              _.QUOTE_STRING_MODE
            ]
          },
          J,
          le,
          ce,
          _.QUOTE_STRING_MODE,
          oe,
          ee,
          ae,
          ne,
          se,
          { begin: /\.$/ }
          // relevance booster
        ]
      };
    }
    return erlang_1 = O, erlang_1;
  }
  var excel_1, hasRequiredExcel;
  function requireExcel() {
    if (hasRequiredExcel) return excel_1;
    hasRequiredExcel = 1;
    function O(_) {
      return {
        name: "Excel formulae",
        aliases: [
          "xlsx",
          "xls"
        ],
        case_insensitive: !0,
        keywords: {
          $pattern: /[a-zA-Z][\w\.]*/,
          built_in: [
            "ABS",
            "ACCRINT",
            "ACCRINTM",
            "ACOS",
            "ACOSH",
            "ACOT",
            "ACOTH",
            "AGGREGATE",
            "ADDRESS",
            "AMORDEGRC",
            "AMORLINC",
            "AND",
            "ARABIC",
            "AREAS",
            "ARRAYTOTEXT",
            "ASC",
            "ASIN",
            "ASINH",
            "ATAN",
            "ATAN2",
            "ATANH",
            "AVEDEV",
            "AVERAGE",
            "AVERAGEA",
            "AVERAGEIF",
            "AVERAGEIFS",
            "BAHTTEXT",
            "BASE",
            "BESSELI",
            "BESSELJ",
            "BESSELK",
            "BESSELY",
            "BETADIST",
            "BETA.DIST",
            "BETAINV",
            "BETA.INV",
            "BIN2DEC",
            "BIN2HEX",
            "BIN2OCT",
            "BINOMDIST",
            "BINOM.DIST",
            "BINOM.DIST.RANGE",
            "BINOM.INV",
            "BITAND",
            "BITLSHIFT",
            "BITOR",
            "BITRSHIFT",
            "BITXOR",
            "BYCOL",
            "BYROW",
            "CALL",
            "CEILING",
            "CEILING.MATH",
            "CEILING.PRECISE",
            "CELL",
            "CHAR",
            "CHIDIST",
            "CHIINV",
            "CHITEST",
            "CHISQ.DIST",
            "CHISQ.DIST.RT",
            "CHISQ.INV",
            "CHISQ.INV.RT",
            "CHISQ.TEST",
            "CHOOSE",
            "CHOOSECOLS",
            "CHOOSEROWS",
            "CLEAN",
            "CODE",
            "COLUMN",
            "COLUMNS",
            "COMBIN",
            "COMBINA",
            "COMPLEX",
            "CONCAT",
            "CONCATENATE",
            "CONFIDENCE",
            "CONFIDENCE.NORM",
            "CONFIDENCE.T",
            "CONVERT",
            "CORREL",
            "COS",
            "COSH",
            "COT",
            "COTH",
            "COUNT",
            "COUNTA",
            "COUNTBLANK",
            "COUNTIF",
            "COUNTIFS",
            "COUPDAYBS",
            "COUPDAYS",
            "COUPDAYSNC",
            "COUPNCD",
            "COUPNUM",
            "COUPPCD",
            "COVAR",
            "COVARIANCE.P",
            "COVARIANCE.S",
            "CRITBINOM",
            "CSC",
            "CSCH",
            "CUBEKPIMEMBER",
            "CUBEMEMBER",
            "CUBEMEMBERPROPERTY",
            "CUBERANKEDMEMBER",
            "CUBESET",
            "CUBESETCOUNT",
            "CUBEVALUE",
            "CUMIPMT",
            "CUMPRINC",
            "DATE",
            "DATEDIF",
            "DATEVALUE",
            "DAVERAGE",
            "DAY",
            "DAYS",
            "DAYS360",
            "DB",
            "DBCS",
            "DCOUNT",
            "DCOUNTA",
            "DDB",
            "DEC2BIN",
            "DEC2HEX",
            "DEC2OCT",
            "DECIMAL",
            "DEGREES",
            "DELTA",
            "DEVSQ",
            "DGET",
            "DISC",
            "DMAX",
            "DMIN",
            "DOLLAR",
            "DOLLARDE",
            "DOLLARFR",
            "DPRODUCT",
            "DROP",
            "DSTDEV",
            "DSTDEVP",
            "DSUM",
            "DURATION",
            "DVAR",
            "DVARP",
            "EDATE",
            "EFFECT",
            "ENCODEURL",
            "EOMONTH",
            "ERF",
            "ERF.PRECISE",
            "ERFC",
            "ERFC.PRECISE",
            "ERROR.TYPE",
            "EUROCONVERT",
            "EVEN",
            "EXACT",
            "EXP",
            "EXPAND",
            "EXPON.DIST",
            "EXPONDIST",
            "FACT",
            "FACTDOUBLE",
            "FALSE",
            "F.DIST",
            "FDIST",
            "F.DIST.RT",
            "FILTER",
            "FILTERXML",
            "FIND",
            "FINDB",
            "F.INV",
            "F.INV.RT",
            "FINV",
            "FISHER",
            "FISHERINV",
            "FIXED",
            "FLOOR",
            "FLOOR.MATH",
            "FLOOR.PRECISE",
            "FORECAST",
            "FORECAST.ETS",
            "FORECAST.ETS.CONFINT",
            "FORECAST.ETS.SEASONALITY",
            "FORECAST.ETS.STAT",
            "FORECAST.LINEAR",
            "FORMULATEXT",
            "FREQUENCY",
            "F.TEST",
            "FTEST",
            "FV",
            "FVSCHEDULE",
            "GAMMA",
            "GAMMA.DIST",
            "GAMMADIST",
            "GAMMA.INV",
            "GAMMAINV",
            "GAMMALN",
            "GAMMALN.PRECISE",
            "GAUSS",
            "GCD",
            "GEOMEAN",
            "GESTEP",
            "GETPIVOTDATA",
            "GROWTH",
            "HARMEAN",
            "HEX2BIN",
            "HEX2DEC",
            "HEX2OCT",
            "HLOOKUP",
            "HOUR",
            "HSTACK",
            "HYPERLINK",
            "HYPGEOM.DIST",
            "HYPGEOMDIST",
            "IF",
            "IFERROR",
            "IFNA",
            "IFS",
            "IMABS",
            "IMAGE",
            "IMAGINARY",
            "IMARGUMENT",
            "IMCONJUGATE",
            "IMCOS",
            "IMCOSH",
            "IMCOT",
            "IMCSC",
            "IMCSCH",
            "IMDIV",
            "IMEXP",
            "IMLN",
            "IMLOG10",
            "IMLOG2",
            "IMPOWER",
            "IMPRODUCT",
            "IMREAL",
            "IMSEC",
            "IMSECH",
            "IMSIN",
            "IMSINH",
            "IMSQRT",
            "IMSUB",
            "IMSUM",
            "IMTAN",
            "INDEX",
            "INDIRECT",
            "INFO",
            "INT",
            "INTERCEPT",
            "INTRATE",
            "IPMT",
            "IRR",
            "ISBLANK",
            "ISERR",
            "ISERROR",
            "ISEVEN",
            "ISFORMULA",
            "ISLOGICAL",
            "ISNA",
            "ISNONTEXT",
            "ISNUMBER",
            "ISODD",
            "ISOMITTED",
            "ISREF",
            "ISTEXT",
            "ISO.CEILING",
            "ISOWEEKNUM",
            "ISPMT",
            "JIS",
            "KURT",
            "LAMBDA",
            "LARGE",
            "LCM",
            "LEFT",
            "LEFTB",
            "LEN",
            "LENB",
            "LET",
            "LINEST",
            "LN",
            "LOG",
            "LOG10",
            "LOGEST",
            "LOGINV",
            "LOGNORM.DIST",
            "LOGNORMDIST",
            "LOGNORM.INV",
            "LOOKUP",
            "LOWER",
            "MAKEARRAY",
            "MAP",
            "MATCH",
            "MAX",
            "MAXA",
            "MAXIFS",
            "MDETERM",
            "MDURATION",
            "MEDIAN",
            "MID",
            "MIDB",
            "MIN",
            "MINIFS",
            "MINA",
            "MINUTE",
            "MINVERSE",
            "MIRR",
            "MMULT",
            "MOD",
            "MODE",
            "MODE.MULT",
            "MODE.SNGL",
            "MONTH",
            "MROUND",
            "MULTINOMIAL",
            "MUNIT",
            "N",
            "NA",
            "NEGBINOM.DIST",
            "NEGBINOMDIST",
            "NETWORKDAYS",
            "NETWORKDAYS.INTL",
            "NOMINAL",
            "NORM.DIST",
            "NORMDIST",
            "NORMINV",
            "NORM.INV",
            "NORM.S.DIST",
            "NORMSDIST",
            "NORM.S.INV",
            "NORMSINV",
            "NOT",
            "NOW",
            "NPER",
            "NPV",
            "NUMBERVALUE",
            "OCT2BIN",
            "OCT2DEC",
            "OCT2HEX",
            "ODD",
            "ODDFPRICE",
            "ODDFYIELD",
            "ODDLPRICE",
            "ODDLYIELD",
            "OFFSET",
            "OR",
            "PDURATION",
            "PEARSON",
            "PERCENTILE.EXC",
            "PERCENTILE.INC",
            "PERCENTILE",
            "PERCENTRANK.EXC",
            "PERCENTRANK.INC",
            "PERCENTRANK",
            "PERMUT",
            "PERMUTATIONA",
            "PHI",
            "PHONETIC",
            "PI",
            "PMT",
            "POISSON.DIST",
            "POISSON",
            "POWER",
            "PPMT",
            "PRICE",
            "PRICEDISC",
            "PRICEMAT",
            "PROB",
            "PRODUCT",
            "PROPER",
            "PV",
            "QUARTILE",
            "QUARTILE.EXC",
            "QUARTILE.INC",
            "QUOTIENT",
            "RADIANS",
            "RAND",
            "RANDARRAY",
            "RANDBETWEEN",
            "RANK.AVG",
            "RANK.EQ",
            "RANK",
            "RATE",
            "RECEIVED",
            "REDUCE",
            "REGISTER.ID",
            "REPLACE",
            "REPLACEB",
            "REPT",
            "RIGHT",
            "RIGHTB",
            "ROMAN",
            "ROUND",
            "ROUNDDOWN",
            "ROUNDUP",
            "ROW",
            "ROWS",
            "RRI",
            "RSQ",
            "RTD",
            "SCAN",
            "SEARCH",
            "SEARCHB",
            "SEC",
            "SECH",
            "SECOND",
            "SEQUENCE",
            "SERIESSUM",
            "SHEET",
            "SHEETS",
            "SIGN",
            "SIN",
            "SINH",
            "SKEW",
            "SKEW.P",
            "SLN",
            "SLOPE",
            "SMALL",
            "SORT",
            "SORTBY",
            "SQRT",
            "SQRTPI",
            "SQL.REQUEST",
            "STANDARDIZE",
            "STOCKHISTORY",
            "STDEV",
            "STDEV.P",
            "STDEV.S",
            "STDEVA",
            "STDEVP",
            "STDEVPA",
            "STEYX",
            "SUBSTITUTE",
            "SUBTOTAL",
            "SUM",
            "SUMIF",
            "SUMIFS",
            "SUMPRODUCT",
            "SUMSQ",
            "SUMX2MY2",
            "SUMX2PY2",
            "SUMXMY2",
            "SWITCH",
            "SYD",
            "T",
            "TAN",
            "TANH",
            "TAKE",
            "TBILLEQ",
            "TBILLPRICE",
            "TBILLYIELD",
            "T.DIST",
            "T.DIST.2T",
            "T.DIST.RT",
            "TDIST",
            "TEXT",
            "TEXTAFTER",
            "TEXTBEFORE",
            "TEXTJOIN",
            "TEXTSPLIT",
            "TIME",
            "TIMEVALUE",
            "T.INV",
            "T.INV.2T",
            "TINV",
            "TOCOL",
            "TOROW",
            "TODAY",
            "TRANSPOSE",
            "TREND",
            "TRIM",
            "TRIMMEAN",
            "TRUE",
            "TRUNC",
            "T.TEST",
            "TTEST",
            "TYPE",
            "UNICHAR",
            "UNICODE",
            "UNIQUE",
            "UPPER",
            "VALUE",
            "VALUETOTEXT",
            "VAR",
            "VAR.P",
            "VAR.S",
            "VARA",
            "VARP",
            "VARPA",
            "VDB",
            "VLOOKUP",
            "VSTACK",
            "WEBSERVICE",
            "WEEKDAY",
            "WEEKNUM",
            "WEIBULL",
            "WEIBULL.DIST",
            "WORKDAY",
            "WORKDAY.INTL",
            "WRAPCOLS",
            "WRAPROWS",
            "XIRR",
            "XLOOKUP",
            "XMATCH",
            "XNPV",
            "XOR",
            "YEAR",
            "YEARFRAC",
            "YIELD",
            "YIELDDISC",
            "YIELDMAT",
            "Z.TEST",
            "ZTEST"
          ]
        },
        contains: [
          {
            /* matches a beginning equal sign found in Excel formula examples */
            begin: /^=/,
            end: /[^=]/,
            returnEnd: !0,
            illegal: /=/,
            /* only allow single equal sign at front of line */
            relevance: 10
          },
          /* technically, there can be more than 2 letters in column names, but this prevents conflict with some keywords */
          {
            /* matches a reference to a single cell */
            className: "symbol",
            begin: /\b[A-Z]{1,2}\d+\b/,
            end: /[^\d]/,
            excludeEnd: !0,
            relevance: 0
          },
          {
            /* matches a reference to a range of cells */
            className: "symbol",
            begin: /[A-Z]{0,2}\d*:[A-Z]{0,2}\d*/,
            relevance: 0
          },
          _.BACKSLASH_ESCAPE,
          _.QUOTE_STRING_MODE,
          {
            className: "number",
            begin: _.NUMBER_RE + "(%)?",
            relevance: 0
          },
          /* Excel formula comments are done by putting the comment in a function call to N() */
          _.COMMENT(
            /\bN\(/,
            /\)/,
            {
              excludeBegin: !0,
              excludeEnd: !0,
              illegal: /\n/
            }
          )
        ]
      };
    }
    return excel_1 = O, excel_1;
  }
  var fix_1, hasRequiredFix;
  function requireFix() {
    if (hasRequiredFix) return fix_1;
    hasRequiredFix = 1;
    function O(_) {
      return {
        name: "FIX",
        contains: [
          {
            begin: /[^\u2401\u0001]+/,
            end: /[\u2401\u0001]/,
            excludeEnd: !0,
            returnBegin: !0,
            returnEnd: !1,
            contains: [
              {
                begin: /([^\u2401\u0001=]+)/,
                end: /=([^\u2401\u0001=]+)/,
                returnEnd: !0,
                returnBegin: !1,
                className: "attr"
              },
              {
                begin: /=/,
                end: /([\u2401\u0001])/,
                excludeEnd: !0,
                excludeBegin: !0,
                className: "string"
              }
            ]
          }
        ],
        case_insensitive: !0
      };
    }
    return fix_1 = O, fix_1;
  }
  var flix_1, hasRequiredFlix;
  function requireFlix() {
    if (hasRequiredFlix) return flix_1;
    hasRequiredFlix = 1;
    function O(_) {
      const H = {
        className: "string",
        begin: /'(.|\\[xXuU][a-zA-Z0-9]+)'/
      }, V = {
        className: "string",
        variants: [
          {
            begin: '"',
            end: '"'
          }
        ]
      }, G = {
        className: "function",
        beginKeywords: "def",
        end: /[:={\[(\n;]/,
        excludeEnd: !0,
        contains: [{
          className: "title",
          relevance: 0,
          begin: /[^0-9\n\t "'(),.`{}\[\]:;][^\n\t "'(),.`{}\[\]:;]+|[^0-9\n\t "'(),.`{}\[\]:;=]/
        }]
      };
      return {
        name: "Flix",
        keywords: {
          keyword: [
            "case",
            "class",
            "def",
            "else",
            "enum",
            "if",
            "impl",
            "import",
            "in",
            "lat",
            "rel",
            "index",
            "let",
            "match",
            "namespace",
            "switch",
            "type",
            "yield",
            "with"
          ],
          literal: [
            "true",
            "false"
          ]
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          H,
          V,
          G,
          _.C_NUMBER_MODE
        ]
      };
    }
    return flix_1 = O, flix_1;
  }
  var fortran_1, hasRequiredFortran;
  function requireFortran() {
    if (hasRequiredFortran) return fortran_1;
    hasRequiredFortran = 1;
    function O(_) {
      const H = _.regex, V = {
        className: "params",
        begin: "\\(",
        end: "\\)"
      }, q = { variants: [
        _.COMMENT("!", "$", { relevance: 0 }),
        // allow FORTRAN 77 style comments
        _.COMMENT("^C[ ]", "$", { relevance: 0 }),
        _.COMMENT("^C$", "$", { relevance: 0 })
      ] }, G = /(_[a-z_\d]+)?/, J = /([de][+-]?\d+)?/, Z = {
        className: "number",
        variants: [
          { begin: H.concat(/\b\d+/, /\.(\d*)/, J, G) },
          { begin: H.concat(/\b\d+/, J, G) },
          { begin: H.concat(/\.\d+/, J, G) }
        ],
        relevance: 0
      }, te = {
        className: "function",
        beginKeywords: "subroutine function program",
        illegal: "[${=\\n]",
        contains: [
          _.UNDERSCORE_TITLE_MODE,
          V
        ]
      }, ne = {
        className: "string",
        relevance: 0,
        variants: [
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE
        ]
      };
      return {
        name: "Fortran",
        case_insensitive: !0,
        aliases: [
          "f90",
          "f95"
        ],
        keywords: {
          $pattern: /\b[a-z][a-z0-9_]+\b|\.[a-z][a-z0-9_]+\./,
          keyword: [
            "kind",
            "do",
            "concurrent",
            "local",
            "shared",
            "while",
            "private",
            "call",
            "intrinsic",
            "where",
            "elsewhere",
            "type",
            "endtype",
            "endmodule",
            "endselect",
            "endinterface",
            "end",
            "enddo",
            "endif",
            "if",
            "forall",
            "endforall",
            "only",
            "contains",
            "default",
            "return",
            "stop",
            "then",
            "block",
            "endblock",
            "endassociate",
            "public",
            "subroutine|10",
            "function",
            "program",
            ".and.",
            ".or.",
            ".not.",
            ".le.",
            ".eq.",
            ".ge.",
            ".gt.",
            ".lt.",
            "goto",
            "save",
            "else",
            "use",
            "module",
            "select",
            "case",
            "access",
            "blank",
            "direct",
            "exist",
            "file",
            "fmt",
            "form",
            "formatted",
            "iostat",
            "name",
            "named",
            "nextrec",
            "number",
            "opened",
            "rec",
            "recl",
            "sequential",
            "status",
            "unformatted",
            "unit",
            "continue",
            "format",
            "pause",
            "cycle",
            "exit",
            "c_null_char",
            "c_alert",
            "c_backspace",
            "c_form_feed",
            "flush",
            "wait",
            "decimal",
            "round",
            "iomsg",
            "synchronous",
            "nopass",
            "non_overridable",
            "pass",
            "protected",
            "volatile",
            "abstract",
            "extends",
            "import",
            "non_intrinsic",
            "value",
            "deferred",
            "generic",
            "final",
            "enumerator",
            "class",
            "associate",
            "bind",
            "enum",
            "c_int",
            "c_short",
            "c_long",
            "c_long_long",
            "c_signed_char",
            "c_size_t",
            "c_int8_t",
            "c_int16_t",
            "c_int32_t",
            "c_int64_t",
            "c_int_least8_t",
            "c_int_least16_t",
            "c_int_least32_t",
            "c_int_least64_t",
            "c_int_fast8_t",
            "c_int_fast16_t",
            "c_int_fast32_t",
            "c_int_fast64_t",
            "c_intmax_t",
            "C_intptr_t",
            "c_float",
            "c_double",
            "c_long_double",
            "c_float_complex",
            "c_double_complex",
            "c_long_double_complex",
            "c_bool",
            "c_char",
            "c_null_ptr",
            "c_null_funptr",
            "c_new_line",
            "c_carriage_return",
            "c_horizontal_tab",
            "c_vertical_tab",
            "iso_c_binding",
            "c_loc",
            "c_funloc",
            "c_associated",
            "c_f_pointer",
            "c_ptr",
            "c_funptr",
            "iso_fortran_env",
            "character_storage_size",
            "error_unit",
            "file_storage_size",
            "input_unit",
            "iostat_end",
            "iostat_eor",
            "numeric_storage_size",
            "output_unit",
            "c_f_procpointer",
            "ieee_arithmetic",
            "ieee_support_underflow_control",
            "ieee_get_underflow_mode",
            "ieee_set_underflow_mode",
            "newunit",
            "contiguous",
            "recursive",
            "pad",
            "position",
            "action",
            "delim",
            "readwrite",
            "eor",
            "advance",
            "nml",
            "interface",
            "procedure",
            "namelist",
            "include",
            "sequence",
            "elemental",
            "pure",
            "impure",
            "integer",
            "real",
            "character",
            "complex",
            "logical",
            "codimension",
            "dimension",
            "allocatable|10",
            "parameter",
            "external",
            "implicit|10",
            "none",
            "double",
            "precision",
            "assign",
            "intent",
            "optional",
            "pointer",
            "target",
            "in",
            "out",
            "common",
            "equivalence",
            "data"
          ],
          literal: [
            ".False.",
            ".True."
          ],
          built_in: [
            "alog",
            "alog10",
            "amax0",
            "amax1",
            "amin0",
            "amin1",
            "amod",
            "cabs",
            "ccos",
            "cexp",
            "clog",
            "csin",
            "csqrt",
            "dabs",
            "dacos",
            "dasin",
            "datan",
            "datan2",
            "dcos",
            "dcosh",
            "ddim",
            "dexp",
            "dint",
            "dlog",
            "dlog10",
            "dmax1",
            "dmin1",
            "dmod",
            "dnint",
            "dsign",
            "dsin",
            "dsinh",
            "dsqrt",
            "dtan",
            "dtanh",
            "float",
            "iabs",
            "idim",
            "idint",
            "idnint",
            "ifix",
            "isign",
            "max0",
            "max1",
            "min0",
            "min1",
            "sngl",
            "algama",
            "cdabs",
            "cdcos",
            "cdexp",
            "cdlog",
            "cdsin",
            "cdsqrt",
            "cqabs",
            "cqcos",
            "cqexp",
            "cqlog",
            "cqsin",
            "cqsqrt",
            "dcmplx",
            "dconjg",
            "derf",
            "derfc",
            "dfloat",
            "dgamma",
            "dimag",
            "dlgama",
            "iqint",
            "qabs",
            "qacos",
            "qasin",
            "qatan",
            "qatan2",
            "qcmplx",
            "qconjg",
            "qcos",
            "qcosh",
            "qdim",
            "qerf",
            "qerfc",
            "qexp",
            "qgamma",
            "qimag",
            "qlgama",
            "qlog",
            "qlog10",
            "qmax1",
            "qmin1",
            "qmod",
            "qnint",
            "qsign",
            "qsin",
            "qsinh",
            "qsqrt",
            "qtan",
            "qtanh",
            "abs",
            "acos",
            "aimag",
            "aint",
            "anint",
            "asin",
            "atan",
            "atan2",
            "char",
            "cmplx",
            "conjg",
            "cos",
            "cosh",
            "exp",
            "ichar",
            "index",
            "int",
            "log",
            "log10",
            "max",
            "min",
            "nint",
            "sign",
            "sin",
            "sinh",
            "sqrt",
            "tan",
            "tanh",
            "print",
            "write",
            "dim",
            "lge",
            "lgt",
            "lle",
            "llt",
            "mod",
            "nullify",
            "allocate",
            "deallocate",
            "adjustl",
            "adjustr",
            "all",
            "allocated",
            "any",
            "associated",
            "bit_size",
            "btest",
            "ceiling",
            "count",
            "cshift",
            "date_and_time",
            "digits",
            "dot_product",
            "eoshift",
            "epsilon",
            "exponent",
            "floor",
            "fraction",
            "huge",
            "iand",
            "ibclr",
            "ibits",
            "ibset",
            "ieor",
            "ior",
            "ishft",
            "ishftc",
            "lbound",
            "len_trim",
            "matmul",
            "maxexponent",
            "maxloc",
            "maxval",
            "merge",
            "minexponent",
            "minloc",
            "minval",
            "modulo",
            "mvbits",
            "nearest",
            "pack",
            "present",
            "product",
            "radix",
            "random_number",
            "random_seed",
            "range",
            "repeat",
            "reshape",
            "rrspacing",
            "scale",
            "scan",
            "selected_int_kind",
            "selected_real_kind",
            "set_exponent",
            "shape",
            "size",
            "spacing",
            "spread",
            "sum",
            "system_clock",
            "tiny",
            "transpose",
            "trim",
            "ubound",
            "unpack",
            "verify",
            "achar",
            "iachar",
            "transfer",
            "dble",
            "entry",
            "dprod",
            "cpu_time",
            "command_argument_count",
            "get_command",
            "get_command_argument",
            "get_environment_variable",
            "is_iostat_end",
            "ieee_arithmetic",
            "ieee_support_underflow_control",
            "ieee_get_underflow_mode",
            "ieee_set_underflow_mode",
            "is_iostat_eor",
            "move_alloc",
            "new_line",
            "selected_char_kind",
            "same_type_as",
            "extends_type_of",
            "acosh",
            "asinh",
            "atanh",
            "bessel_j0",
            "bessel_j1",
            "bessel_jn",
            "bessel_y0",
            "bessel_y1",
            "bessel_yn",
            "erf",
            "erfc",
            "erfc_scaled",
            "gamma",
            "log_gamma",
            "hypot",
            "norm2",
            "atomic_define",
            "atomic_ref",
            "execute_command_line",
            "leadz",
            "trailz",
            "storage_size",
            "merge_bits",
            "bge",
            "bgt",
            "ble",
            "blt",
            "dshiftl",
            "dshiftr",
            "findloc",
            "iall",
            "iany",
            "iparity",
            "image_index",
            "lcobound",
            "ucobound",
            "maskl",
            "maskr",
            "num_images",
            "parity",
            "popcnt",
            "poppar",
            "shifta",
            "shiftl",
            "shiftr",
            "this_image",
            "sync",
            "change",
            "team",
            "co_broadcast",
            "co_max",
            "co_min",
            "co_sum",
            "co_reduce"
          ]
        },
        illegal: /\/\*/,
        contains: [
          ne,
          te,
          // allow `C = value` for assignments so they aren't misdetected
          // as Fortran 77 style comments
          {
            begin: /^C\s*=(?!=)/,
            relevance: 0
          },
          q,
          Z
        ]
      };
    }
    return fortran_1 = O, fortran_1;
  }
  var fsharp_1, hasRequiredFsharp;
  function requireFsharp() {
    if (hasRequiredFsharp) return fsharp_1;
    hasRequiredFsharp = 1;
    function O(Z) {
      return new RegExp(Z.replace(/[-/\\^$*+?.()|[\]{}]/g, "\\$&"), "m");
    }
    function _(Z) {
      return Z ? typeof Z == "string" ? Z : Z.source : null;
    }
    function H(Z) {
      return V("(?=", Z, ")");
    }
    function V(...Z) {
      return Z.map((ne) => _(ne)).join("");
    }
    function q(Z) {
      const te = Z[Z.length - 1];
      return typeof te == "object" && te.constructor === Object ? (Z.splice(Z.length - 1, 1), te) : {};
    }
    function G(...Z) {
      return "(" + (q(Z).capture ? "" : "?:") + Z.map((ee) => _(ee)).join("|") + ")";
    }
    function J(Z) {
      const te = [
        "abstract",
        "and",
        "as",
        "assert",
        "base",
        "begin",
        "class",
        "default",
        "delegate",
        "do",
        "done",
        "downcast",
        "downto",
        "elif",
        "else",
        "end",
        "exception",
        "extern",
        // "false", // literal
        "finally",
        "fixed",
        "for",
        "fun",
        "function",
        "global",
        "if",
        "in",
        "inherit",
        "inline",
        "interface",
        "internal",
        "lazy",
        "let",
        "match",
        "member",
        "module",
        "mutable",
        "namespace",
        "new",
        // "not", // built_in
        // "null", // literal
        "of",
        "open",
        "or",
        "override",
        "private",
        "public",
        "rec",
        "return",
        "static",
        "struct",
        "then",
        "to",
        // "true", // literal
        "try",
        "type",
        "upcast",
        "use",
        "val",
        "void",
        "when",
        "while",
        "with",
        "yield"
      ], ne = {
        // monad builder keywords (matches before non-bang keywords)
        scope: "keyword",
        match: /\b(yield|return|let|do|match|use)!/
      }, ee = [
        "if",
        "else",
        "endif",
        "line",
        "nowarn",
        "light",
        "r",
        "i",
        "I",
        "load",
        "time",
        "help",
        "quit"
      ], ae = [
        "true",
        "false",
        "null",
        "Some",
        "None",
        "Ok",
        "Error",
        "infinity",
        "infinityf",
        "nan",
        "nanf"
      ], oe = [
        "__LINE__",
        "__SOURCE_DIRECTORY__",
        "__SOURCE_FILE__"
      ], se = [
        // basic types
        "bool",
        "byte",
        "sbyte",
        "int8",
        "int16",
        "int32",
        "uint8",
        "uint16",
        "uint32",
        "int",
        "uint",
        "int64",
        "uint64",
        "nativeint",
        "unativeint",
        "decimal",
        "float",
        "double",
        "float32",
        "single",
        "char",
        "string",
        "unit",
        "bigint",
        // other native types or lowercase aliases
        "option",
        "voption",
        "list",
        "array",
        "seq",
        "byref",
        "exn",
        "inref",
        "nativeptr",
        "obj",
        "outref",
        "voidptr",
        // other important FSharp types
        "Result"
      ], le = {
        keyword: te,
        literal: ae,
        built_in: [
          // Somewhat arbitrary list of builtin functions and values.
          // Most of them are declared in Microsoft.FSharp.Core
          // I tried to stay relevant by adding only the most idiomatic
          // and most used symbols that are not already declared as types.
          "not",
          "ref",
          "raise",
          "reraise",
          "dict",
          "readOnlyDict",
          "set",
          "get",
          "enum",
          "sizeof",
          "typeof",
          "typedefof",
          "nameof",
          "nullArg",
          "invalidArg",
          "invalidOp",
          "id",
          "fst",
          "snd",
          "ignore",
          "lock",
          "using",
          "box",
          "unbox",
          "tryUnbox",
          "printf",
          "printfn",
          "sprintf",
          "eprintf",
          "eprintfn",
          "fprintf",
          "fprintfn",
          "failwith",
          "failwithf"
        ],
        "variable.constant": oe
      }, de = {
        variants: [
          Z.COMMENT(/\(\*(?!\))/, /\*\)/, {
            contains: ["self"]
          }),
          Z.C_LINE_COMMENT_MODE
        ]
      }, fe = /[a-zA-Z_](\w|')*/, he = {
        scope: "variable",
        begin: /``/,
        end: /``/
      }, me = /\B('|\^)/, pe = {
        scope: "symbol",
        variants: [
          // the type name is a quoted identifier:
          { match: V(me, /``.*?``/) },
          // the type name is a normal identifier (we don't use IDENTIFIER_RE because there cannot be another apostrophe here):
          { match: V(me, Z.UNDERSCORE_IDENT_RE) }
        ],
        relevance: 0
      }, ye = function({ includeEqual: Ve }) {
        let Ze;
        Ve ? Ze = "!%&*+-/<=>@^|~?" : Ze = "!%&*+-/<>@^|~?";
        const Ge = Array.from(Ze), ze = V("[", ...Ge.map(O), "]"), We = G(ze, /\./), at = V(We, H(We)), ct = G(
          V(at, We, "*"),
          // Matches at least 2 chars operators
          V(ze, "+")
          // Matches at least one char operators
        );
        return {
          scope: "operator",
          match: G(
            // symbolic operators:
            ct,
            // other symbolic keywords:
            // Type casting and conversion operators:
            /:\?>/,
            /:\?/,
            /:>/,
            /:=/,
            // Reference cell assignment
            /::?/,
            // : or ::
            /\$/
          ),
          // A single $ can be used as an operator
          relevance: 0
        };
      }, ge = ye({ includeEqual: !0 }), ve = ye({ includeEqual: !1 }), be = function(Ve, Ze) {
        return {
          begin: V(
            // a type annotation is a
            Ve,
            // should be a colon or the 'of' keyword
            H(
              // that has to be followed by
              V(
                /\s*/,
                // optional space
                G(
                  // then either of:
                  /\w/,
                  // word
                  /'/,
                  // generic type name
                  /\^/,
                  // generic type name
                  /#/,
                  // flexible type name
                  /``/,
                  // quoted type name
                  /\(/,
                  // parens type expression
                  /{\|/
                  // anonymous type annotation
                )
              )
            )
          ),
          beginScope: Ze,
          // BUG: because ending with \n is necessary for some cases, multi-line type annotations are not properly supported.
          // Examples where \n is required at the end:
          // - abstract member definitions in classes: abstract Property : int * string
          // - return type annotations: let f f' = f' () : returnTypeAnnotation
          // - record fields definitions: { A : int \n B : string }
          end: H(
            G(
              /\n/,
              /=/
            )
          ),
          relevance: 0,
          // we need the known types, and we need the type constraint keywords and literals. e.g.: when 'a : null
          keywords: Z.inherit(le, { type: se }),
          contains: [
            de,
            pe,
            Z.inherit(he, { scope: null }),
            // match to avoid strange patterns inside that may break the parsing
            ve
          ]
        };
      }, Ce = be(/:/, "operator"), Se = be(/\bof\b/, "keyword"), we = {
        begin: [
          /(^|\s+)/,
          // prevents matching the following: `match s.stype with`
          /type/,
          /\s+/,
          fe
        ],
        beginScope: {
          2: "keyword",
          4: "title.class"
        },
        end: H(/\(|=|$/),
        keywords: le,
        // match keywords in type constraints. e.g.: when 'a : null
        contains: [
          de,
          Z.inherit(he, { scope: null }),
          // match to avoid strange patterns inside that may break the parsing
          pe,
          {
            // For visual consistency, highlight type brackets as operators.
            scope: "operator",
            match: /<|>/
          },
          Ce
          // generic types can have constraints, which are type annotations. e.g. type MyType<'T when 'T : delegate<obj * string>> =
        ]
      }, Ee = {
        // computation expressions:
        scope: "computation-expression",
        // BUG: might conflict with record deconstruction. e.g. let f { Name = name } = name // will highlight f
        match: /\b[_a-z]\w*(?=\s*\{)/
      }, xe = {
        // preprocessor directives and fsi commands:
        begin: [
          /^\s*/,
          V(/#/, G(...ee)),
          /\b/
        ],
        beginScope: { 2: "meta" },
        end: H(/\s|$/)
      }, Le = {
        variants: [
          Z.BINARY_NUMBER_MODE,
          Z.C_NUMBER_MODE
        ]
      }, Qe = {
        scope: "string",
        begin: /"/,
        end: /"/,
        contains: [
          Z.BACKSLASH_ESCAPE
        ]
      }, ke = {
        scope: "string",
        begin: /@"/,
        end: /"/,
        contains: [
          {
            match: /""/
            // escaped "
          },
          Z.BACKSLASH_ESCAPE
        ]
      }, Me = {
        scope: "string",
        begin: /"""/,
        end: /"""/,
        relevance: 2
      }, Re = {
        scope: "subst",
        begin: /\{/,
        end: /\}/,
        keywords: le
      }, De = {
        scope: "string",
        begin: /\$"/,
        end: /"/,
        contains: [
          {
            match: /\{\{/
            // escaped {
          },
          {
            match: /\}\}/
            // escaped }
          },
          Z.BACKSLASH_ESCAPE,
          Re
        ]
      }, Pe = {
        scope: "string",
        begin: /(\$@|@\$)"/,
        end: /"/,
        contains: [
          {
            match: /\{\{/
            // escaped {
          },
          {
            match: /\}\}/
            // escaped }
          },
          {
            match: /""/
          },
          Z.BACKSLASH_ESCAPE,
          Re
        ]
      }, $e = {
        scope: "string",
        begin: /\$"""/,
        end: /"""/,
        contains: [
          {
            match: /\{\{/
            // escaped {
          },
          {
            match: /\}\}/
            // escaped }
          },
          Re
        ],
        relevance: 2
      }, qe = {
        scope: "string",
        match: V(
          /'/,
          G(
            /[^\\']/,
            // either a single non escaped char...
            /\\(?:.|\d{3}|x[a-fA-F\d]{2}|u[a-fA-F\d]{4}|U[a-fA-F\d]{8})/
            // ...or an escape sequence
          ),
          /'/
        )
      };
      return Re.contains = [
        Pe,
        De,
        ke,
        Qe,
        qe,
        ne,
        de,
        he,
        Ce,
        Ee,
        xe,
        Le,
        pe,
        ge
      ], {
        name: "F#",
        aliases: [
          "fs",
          "f#"
        ],
        keywords: le,
        illegal: /\/\*/,
        classNameAliases: {
          "computation-expression": "keyword"
        },
        contains: [
          ne,
          {
            variants: [
              $e,
              Pe,
              De,
              Me,
              ke,
              Qe,
              qe
            ]
          },
          de,
          he,
          we,
          {
            // e.g. [<Attributes("")>] or [<``module``: MyCustomAttributeThatWorksOnModules>]
            // or [<Sealed; NoEquality; NoComparison; CompiledName("FSharpAsync`1")>]
            scope: "meta",
            begin: /\[</,
            end: />\]/,
            relevance: 2,
            contains: [
              he,
              // can contain any constant value
              Me,
              ke,
              Qe,
              qe,
              Le
            ]
          },
          Se,
          Ce,
          Ee,
          xe,
          Le,
          pe,
          ge
        ]
      };
    }
    return fsharp_1 = J, fsharp_1;
  }
  var gams_1, hasRequiredGams;
  function requireGams() {
    if (hasRequiredGams) return gams_1;
    hasRequiredGams = 1;
    function O(_) {
      const H = _.regex, V = {
        keyword: "abort acronym acronyms alias all and assign binary card diag display else eq file files for free ge gt if integer le loop lt maximizing minimizing model models ne negative no not option options or ord positive prod put putpage puttl repeat sameas semicont semiint smax smin solve sos1 sos2 sum system table then until using while xor yes",
        literal: "eps inf na",
        built_in: "abs arccos arcsin arctan arctan2 Beta betaReg binomial ceil centropy cos cosh cvPower div div0 eDist entropy errorf execSeed exp fact floor frac gamma gammaReg log logBeta logGamma log10 log2 mapVal max min mod ncpCM ncpF ncpVUpow ncpVUsin normal pi poly power randBinomial randLinear randTriangle round rPower sigmoid sign signPower sin sinh slexp sllog10 slrec sqexp sqlog10 sqr sqrec sqrt tan tanh trunc uniform uniformInt vcPower bool_and bool_eqv bool_imp bool_not bool_or bool_xor ifThen rel_eq rel_ge rel_gt rel_le rel_lt rel_ne gday gdow ghour gleap gmillisec gminute gmonth gsecond gyear jdate jnow jstart jtime errorLevel execError gamsRelease gamsVersion handleCollect handleDelete handleStatus handleSubmit heapFree heapLimit heapSize jobHandle jobKill jobStatus jobTerminate licenseLevel licenseStatus maxExecError sleep timeClose timeComp timeElapsed timeExec timeStart"
      }, q = {
        className: "params",
        begin: /\(/,
        end: /\)/,
        excludeBegin: !0,
        excludeEnd: !0
      }, G = {
        className: "symbol",
        variants: [
          { begin: /=[lgenxc]=/ },
          { begin: /\$/ }
        ]
      }, J = {
        // One-line quoted comment string
        className: "comment",
        variants: [
          {
            begin: "'",
            end: "'"
          },
          {
            begin: '"',
            end: '"'
          }
        ],
        illegal: "\\n",
        contains: [_.BACKSLASH_ESCAPE]
      }, Z = {
        begin: "/",
        end: "/",
        keywords: V,
        contains: [
          J,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.APOS_STRING_MODE,
          _.C_NUMBER_MODE
        ]
      }, te = /[a-z0-9&#*=?@\\><:,()$[\]_.{}!+%^-]+/, ne = {
        // Parameter/set/variable description text
        begin: /[a-z][a-z0-9_]*(\([a-z0-9_, ]*\))?[ \t]+/,
        excludeBegin: !0,
        end: "$",
        endsWithParent: !0,
        contains: [
          J,
          Z,
          {
            className: "comment",
            // one comment word, then possibly more
            begin: H.concat(
              te,
              // [ ] because \s would be too broad (matching newlines)
              H.anyNumberOfTimes(H.concat(/[ ]+/, te))
            ),
            relevance: 0
          }
        ]
      };
      return {
        name: "GAMS",
        aliases: ["gms"],
        case_insensitive: !0,
        keywords: V,
        contains: [
          _.COMMENT(/^\$ontext/, /^\$offtext/),
          {
            className: "meta",
            begin: "^\\$[a-z0-9]+",
            end: "$",
            returnBegin: !0,
            contains: [
              {
                className: "keyword",
                begin: "^\\$[a-z0-9]+"
              }
            ]
          },
          _.COMMENT("^\\*", "$"),
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.APOS_STRING_MODE,
          // Declarations
          {
            beginKeywords: "set sets parameter parameters variable variables scalar scalars equation equations",
            end: ";",
            contains: [
              _.COMMENT("^\\*", "$"),
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE,
              _.QUOTE_STRING_MODE,
              _.APOS_STRING_MODE,
              Z,
              ne
            ]
          },
          {
            // table environment
            beginKeywords: "table",
            end: ";",
            returnBegin: !0,
            contains: [
              {
                // table header row
                beginKeywords: "table",
                end: "$",
                contains: [ne]
              },
              _.COMMENT("^\\*", "$"),
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE,
              _.QUOTE_STRING_MODE,
              _.APOS_STRING_MODE,
              _.C_NUMBER_MODE
              // Table does not contain DESCTEXT or ASSIGNMENT
            ]
          },
          // Function definitions
          {
            className: "function",
            begin: /^[a-z][a-z0-9_,\-+' ()$]+\.{2}/,
            returnBegin: !0,
            contains: [
              {
                // Function title
                className: "title",
                begin: /^[a-z0-9_]+/
              },
              q,
              G
            ]
          },
          _.C_NUMBER_MODE,
          G
        ]
      };
    }
    return gams_1 = O, gams_1;
  }
  var gauss_1, hasRequiredGauss;
  function requireGauss() {
    if (hasRequiredGauss) return gauss_1;
    hasRequiredGauss = 1;
    function O(_) {
      const H = {
        keyword: "bool break call callexe checkinterrupt clear clearg closeall cls comlog compile continue create debug declare delete disable dlibrary dllcall do dos ed edit else elseif enable end endfor endif endp endo errorlog errorlogat expr external fn for format goto gosub graph if keyword let lib library line load loadarray loadexe loadf loadk loadm loadp loads loadx local locate loopnextindex lprint lpwidth lshow matrix msym ndpclex new open output outwidth plot plotsym pop prcsn print printdos proc push retp return rndcon rndmod rndmult rndseed run save saveall screen scroll setarray show sparse stop string struct system trace trap threadfor threadendfor threadbegin threadjoin threadstat threadend until use while winprint ne ge le gt lt and xor or not eq eqv",
        built_in: "abs acf aconcat aeye amax amean AmericanBinomCall AmericanBinomCall_Greeks AmericanBinomCall_ImpVol AmericanBinomPut AmericanBinomPut_Greeks AmericanBinomPut_ImpVol AmericanBSCall AmericanBSCall_Greeks AmericanBSCall_ImpVol AmericanBSPut AmericanBSPut_Greeks AmericanBSPut_ImpVol amin amult annotationGetDefaults annotationSetBkd annotationSetFont annotationSetLineColor annotationSetLineStyle annotationSetLineThickness annualTradingDays arccos arcsin areshape arrayalloc arrayindex arrayinit arraytomat asciiload asclabel astd astds asum atan atan2 atranspose axmargin balance band bandchol bandcholsol bandltsol bandrv bandsolpd bar base10 begwind besselj bessely beta box boxcox cdfBeta cdfBetaInv cdfBinomial cdfBinomialInv cdfBvn cdfBvn2 cdfBvn2e cdfCauchy cdfCauchyInv cdfChic cdfChii cdfChinc cdfChincInv cdfExp cdfExpInv cdfFc cdfFnc cdfFncInv cdfGam cdfGenPareto cdfHyperGeo cdfLaplace cdfLaplaceInv cdfLogistic cdfLogisticInv cdfmControlCreate cdfMvn cdfMvn2e cdfMvnce cdfMvne cdfMvt2e cdfMvtce cdfMvte cdfN cdfN2 cdfNc cdfNegBinomial cdfNegBinomialInv cdfNi cdfPoisson cdfPoissonInv cdfRayleigh cdfRayleighInv cdfTc cdfTci cdfTnc cdfTvn cdfWeibull cdfWeibullInv cdir ceil ChangeDir chdir chiBarSquare chol choldn cholsol cholup chrs close code cols colsf combinate combinated complex con cond conj cons ConScore contour conv convertsatostr convertstrtosa corrm corrms corrvc corrx corrxs cos cosh counts countwts crossprd crout croutp csrcol csrlin csvReadM csvReadSA cumprodc cumsumc curve cvtos datacreate datacreatecomplex datalist dataload dataloop dataopen datasave date datestr datestring datestrymd dayinyr dayofweek dbAddDatabase dbClose dbCommit dbCreateQuery dbExecQuery dbGetConnectOptions dbGetDatabaseName dbGetDriverName dbGetDrivers dbGetHostName dbGetLastErrorNum dbGetLastErrorText dbGetNumericalPrecPolicy dbGetPassword dbGetPort dbGetTableHeaders dbGetTables dbGetUserName dbHasFeature dbIsDriverAvailable dbIsOpen dbIsOpenError dbOpen dbQueryBindValue dbQueryClear dbQueryCols dbQueryExecPrepared dbQueryFetchAllM dbQueryFetchAllSA dbQueryFetchOneM dbQueryFetchOneSA dbQueryFinish dbQueryGetBoundValue dbQueryGetBoundValues dbQueryGetField dbQueryGetLastErrorNum dbQueryGetLastErrorText dbQueryGetLastInsertID dbQueryGetLastQuery dbQueryGetPosition dbQueryIsActive dbQueryIsForwardOnly dbQueryIsNull dbQueryIsSelect dbQueryIsValid dbQueryPrepare dbQueryRows dbQuerySeek dbQuerySeekFirst dbQuerySeekLast dbQuerySeekNext dbQuerySeekPrevious dbQuerySetForwardOnly dbRemoveDatabase dbRollback dbSetConnectOptions dbSetDatabaseName dbSetHostName dbSetNumericalPrecPolicy dbSetPort dbSetUserName dbTransaction DeleteFile delif delrows denseToSp denseToSpRE denToZero design det detl dfft dffti diag diagrv digamma doswin DOSWinCloseall DOSWinOpen dotfeq dotfeqmt dotfge dotfgemt dotfgt dotfgtmt dotfle dotflemt dotflt dotfltmt dotfne dotfnemt draw drop dsCreate dstat dstatmt dstatmtControlCreate dtdate dtday dttime dttodtv dttostr dttoutc dtvnormal dtvtodt dtvtoutc dummy dummybr dummydn eig eigh eighv eigv elapsedTradingDays endwind envget eof eqSolve eqSolvemt eqSolvemtControlCreate eqSolvemtOutCreate eqSolveset erf erfc erfccplx erfcplx error etdays ethsec etstr EuropeanBinomCall EuropeanBinomCall_Greeks EuropeanBinomCall_ImpVol EuropeanBinomPut EuropeanBinomPut_Greeks EuropeanBinomPut_ImpVol EuropeanBSCall EuropeanBSCall_Greeks EuropeanBSCall_ImpVol EuropeanBSPut EuropeanBSPut_Greeks EuropeanBSPut_ImpVol exctsmpl exec execbg exp extern eye fcheckerr fclearerr feq feqmt fflush fft ffti fftm fftmi fftn fge fgemt fgets fgetsa fgetsat fgetst fgt fgtmt fileinfo filesa fle flemt floor flt fltmt fmod fne fnemt fonts fopen formatcv formatnv fputs fputst fseek fstrerror ftell ftocv ftos ftostrC gamma gammacplx gammaii gausset gdaAppend gdaCreate gdaDStat gdaDStatMat gdaGetIndex gdaGetName gdaGetNames gdaGetOrders gdaGetType gdaGetTypes gdaGetVarInfo gdaIsCplx gdaLoad gdaPack gdaRead gdaReadByIndex gdaReadSome gdaReadSparse gdaReadStruct gdaReportVarInfo gdaSave gdaUpdate gdaUpdateAndPack gdaVars gdaWrite gdaWrite32 gdaWriteSome getarray getdims getf getGAUSShome getmatrix getmatrix4D getname getnamef getNextTradingDay getNextWeekDay getnr getorders getpath getPreviousTradingDay getPreviousWeekDay getRow getscalar3D getscalar4D getTrRow getwind glm gradcplx gradMT gradMTm gradMTT gradMTTm gradp graphprt graphset hasimag header headermt hess hessMT hessMTg hessMTgw hessMTm hessMTmw hessMTT hessMTTg hessMTTgw hessMTTm hessMTw hessp hist histf histp hsec imag indcv indexcat indices indices2 indicesf indicesfn indnv indsav integrate1d integrateControlCreate intgrat2 intgrat3 inthp1 inthp2 inthp3 inthp4 inthpControlCreate intquad1 intquad2 intquad3 intrleav intrleavsa intrsect intsimp inv invpd invswp iscplx iscplxf isden isinfnanmiss ismiss key keyav keyw lag lag1 lagn lapEighb lapEighi lapEighvb lapEighvi lapgEig lapgEigh lapgEighv lapgEigv lapgSchur lapgSvdcst lapgSvds lapgSvdst lapSvdcusv lapSvds lapSvdusv ldlp ldlsol linSolve listwise ln lncdfbvn lncdfbvn2 lncdfmvn lncdfn lncdfn2 lncdfnc lnfact lngammacplx lnpdfmvn lnpdfmvt lnpdfn lnpdft loadd loadstruct loadwind loess loessmt loessmtControlCreate log loglog logx logy lower lowmat lowmat1 ltrisol lu lusol machEpsilon make makevars makewind margin matalloc matinit mattoarray maxbytes maxc maxindc maxv maxvec mbesselei mbesselei0 mbesselei1 mbesseli mbesseli0 mbesseli1 meanc median mergeby mergevar minc minindc minv miss missex missrv moment momentd movingave movingaveExpwgt movingaveWgt nextindex nextn nextnevn nextwind ntos null null1 numCombinations ols olsmt olsmtControlCreate olsqr olsqr2 olsqrmt ones optn optnevn orth outtyp pacf packedToSp packr parse pause pdfCauchy pdfChi pdfExp pdfGenPareto pdfHyperGeo pdfLaplace pdfLogistic pdfn pdfPoisson pdfRayleigh pdfWeibull pi pinv pinvmt plotAddArrow plotAddBar plotAddBox plotAddHist plotAddHistF plotAddHistP plotAddPolar plotAddScatter plotAddShape plotAddTextbox plotAddTS plotAddXY plotArea plotBar plotBox plotClearLayout plotContour plotCustomLayout plotGetDefaults plotHist plotHistF plotHistP plotLayout plotLogLog plotLogX plotLogY plotOpenWindow plotPolar plotSave plotScatter plotSetAxesPen plotSetBar plotSetBarFill plotSetBarStacked plotSetBkdColor plotSetFill plotSetGrid plotSetLegend plotSetLineColor plotSetLineStyle plotSetLineSymbol plotSetLineThickness plotSetNewWindow plotSetTitle plotSetWhichYAxis plotSetXAxisShow plotSetXLabel plotSetXRange plotSetXTicInterval plotSetXTicLabel plotSetYAxisShow plotSetYLabel plotSetYRange plotSetZAxisShow plotSetZLabel plotSurface plotTS plotXY polar polychar polyeval polygamma polyint polymake polymat polymroot polymult polyroot pqgwin previousindex princomp printfm printfmt prodc psi putarray putf putvals pvCreate pvGetIndex pvGetParNames pvGetParVector pvLength pvList pvPack pvPacki pvPackm pvPackmi pvPacks pvPacksi pvPacksm pvPacksmi pvPutParVector pvTest pvUnpack QNewton QNewtonmt QNewtonmtControlCreate QNewtonmtOutCreate QNewtonSet QProg QProgmt QProgmtInCreate qqr qqre qqrep qr qre qrep qrsol qrtsol qtyr qtyre qtyrep quantile quantiled qyr qyre qyrep qz rank rankindx readr real reclassify reclassifyCuts recode recserar recsercp recserrc rerun rescale reshape rets rev rfft rffti rfftip rfftn rfftnp rfftp rndBernoulli rndBeta rndBinomial rndCauchy rndChiSquare rndCon rndCreateState rndExp rndGamma rndGeo rndGumbel rndHyperGeo rndi rndKMbeta rndKMgam rndKMi rndKMn rndKMnb rndKMp rndKMu rndKMvm rndLaplace rndLCbeta rndLCgam rndLCi rndLCn rndLCnb rndLCp rndLCu rndLCvm rndLogNorm rndMTu rndMVn rndMVt rndn rndnb rndNegBinomial rndp rndPoisson rndRayleigh rndStateSkip rndu rndvm rndWeibull rndWishart rotater round rows rowsf rref sampleData satostrC saved saveStruct savewind scale scale3d scalerr scalinfnanmiss scalmiss schtoc schur searchsourcepath seekr select selif seqa seqm setdif setdifsa setvars setvwrmode setwind shell shiftr sin singleindex sinh sleep solpd sortc sortcc sortd sorthc sorthcc sortind sortindc sortmc sortr sortrc spBiconjGradSol spChol spConjGradSol spCreate spDenseSubmat spDiagRvMat spEigv spEye spLDL spline spLU spNumNZE spOnes spreadSheetReadM spreadSheetReadSA spreadSheetWrite spScale spSubmat spToDense spTrTDense spTScalar spZeros sqpSolve sqpSolveMT sqpSolveMTControlCreate sqpSolveMTlagrangeCreate sqpSolveMToutCreate sqpSolveSet sqrt statements stdc stdsc stocv stof strcombine strindx strlen strput strrindx strsect strsplit strsplitPad strtodt strtof strtofcplx strtriml strtrimr strtrunc strtruncl strtruncpad strtruncr submat subscat substute subvec sumc sumr surface svd svd1 svd2 svdcusv svds svdusv sysstate tab tan tanh tempname time timedt timestr timeutc title tkf2eps tkf2ps tocart todaydt toeplitz token topolar trapchk trigamma trimr trunc type typecv typef union unionsa uniqindx uniqindxsa unique uniquesa upmat upmat1 upper utctodt utctodtv utrisol vals varCovMS varCovXS varget vargetl varmall varmares varput varputl vartypef vcm vcms vcx vcxs vec vech vecr vector vget view viewxyz vlist vnamecv volume vput vread vtypecv wait waitc walkindex where window writer xlabel xlsGetSheetCount xlsGetSheetSize xlsGetSheetTypes xlsMakeRange xlsReadM xlsReadSA xlsWrite xlsWriteM xlsWriteSA xpnd xtics xy xyz ylabel ytics zeros zeta zlabel ztics cdfEmpirical dot h5create h5open h5read h5readAttribute h5write h5writeAttribute ldl plotAddErrorBar plotAddSurface plotCDFEmpirical plotSetColormap plotSetContourLabels plotSetLegendFont plotSetTextInterpreter plotSetXTicCount plotSetYTicCount plotSetZLevels powerm strjoin sylvester strtrim",
        literal: "DB_AFTER_LAST_ROW DB_ALL_TABLES DB_BATCH_OPERATIONS DB_BEFORE_FIRST_ROW DB_BLOB DB_EVENT_NOTIFICATIONS DB_FINISH_QUERY DB_HIGH_PRECISION DB_LAST_INSERT_ID DB_LOW_PRECISION_DOUBLE DB_LOW_PRECISION_INT32 DB_LOW_PRECISION_INT64 DB_LOW_PRECISION_NUMBERS DB_MULTIPLE_RESULT_SETS DB_NAMED_PLACEHOLDERS DB_POSITIONAL_PLACEHOLDERS DB_PREPARED_QUERIES DB_QUERY_SIZE DB_SIMPLE_LOCKING DB_SYSTEM_TABLES DB_TABLES DB_TRANSACTIONS DB_UNICODE DB_VIEWS __STDIN __STDOUT __STDERR __FILE_DIR"
      }, V = _.COMMENT("@", "@"), q = {
        className: "meta",
        begin: "#",
        end: "$",
        keywords: { keyword: "define definecs|10 undef ifdef ifndef iflight ifdllcall ifmac ifos2win ifunix else endif lineson linesoff srcfile srcline" },
        contains: [
          {
            begin: /\\\n/,
            relevance: 0
          },
          {
            beginKeywords: "include",
            end: "$",
            keywords: { keyword: "include" },
            contains: [
              {
                className: "string",
                begin: '"',
                end: '"',
                illegal: "\\n"
              }
            ]
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          V
        ]
      }, G = {
        begin: /\bstruct\s+/,
        end: /\s/,
        keywords: "struct",
        contains: [
          {
            className: "type",
            begin: _.UNDERSCORE_IDENT_RE,
            relevance: 0
          }
        ]
      }, J = [
        {
          className: "params",
          begin: /\(/,
          end: /\)/,
          excludeBegin: !0,
          excludeEnd: !0,
          endsWithParent: !0,
          relevance: 0,
          contains: [
            {
              // dots
              className: "literal",
              begin: /\.\.\./
            },
            _.C_NUMBER_MODE,
            _.C_BLOCK_COMMENT_MODE,
            V,
            G
          ]
        }
      ], Z = {
        className: "title",
        begin: _.UNDERSCORE_IDENT_RE,
        relevance: 0
      }, te = function(se, ce, le) {
        const ue = _.inherit(
          {
            className: "function",
            beginKeywords: se,
            end: ce,
            excludeEnd: !0,
            contains: [].concat(J)
          },
          {}
        );
        return ue.contains.push(Z), ue.contains.push(_.C_NUMBER_MODE), ue.contains.push(_.C_BLOCK_COMMENT_MODE), ue.contains.push(V), ue;
      }, ne = {
        // these are explicitly named internal function calls
        className: "built_in",
        begin: "\\b(" + H.built_in.split(" ").join("|") + ")\\b"
      }, ee = {
        className: "string",
        begin: '"',
        end: '"',
        contains: [_.BACKSLASH_ESCAPE],
        relevance: 0
      }, ae = {
        // className: "fn_ref",
        begin: _.UNDERSCORE_IDENT_RE + "\\s*\\(",
        returnBegin: !0,
        keywords: H,
        relevance: 0,
        contains: [
          { beginKeywords: H.keyword },
          ne,
          {
            // ambiguously named function calls get a relevance of 0
            className: "built_in",
            begin: _.UNDERSCORE_IDENT_RE,
            relevance: 0
          }
        ]
      }, oe = {
        // className: "fn_ref_params",
        begin: /\(/,
        end: /\)/,
        relevance: 0,
        keywords: {
          built_in: H.built_in,
          literal: H.literal
        },
        contains: [
          _.C_NUMBER_MODE,
          _.C_BLOCK_COMMENT_MODE,
          V,
          ne,
          ae,
          ee,
          "self"
        ]
      };
      return ae.contains.push(oe), {
        name: "GAUSS",
        aliases: ["gss"],
        case_insensitive: !0,
        // language is case-insensitive
        keywords: H,
        illegal: /(\{[%#]|[%#]\}| <- )/,
        contains: [
          _.C_NUMBER_MODE,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          V,
          ee,
          q,
          {
            className: "keyword",
            begin: /\bexternal (matrix|string|array|sparse matrix|struct|proc|keyword|fn)/
          },
          te("proc keyword", ";"),
          te("fn", "="),
          {
            beginKeywords: "for threadfor",
            end: /;/,
            // end: /\(/,
            relevance: 0,
            contains: [
              _.C_BLOCK_COMMENT_MODE,
              V,
              oe
            ]
          },
          {
            // custom method guard
            // excludes method names from keyword processing
            variants: [
              { begin: _.UNDERSCORE_IDENT_RE + "\\." + _.UNDERSCORE_IDENT_RE },
              { begin: _.UNDERSCORE_IDENT_RE + "\\s*=" }
            ],
            relevance: 0
          },
          ae,
          G
        ]
      };
    }
    return gauss_1 = O, gauss_1;
  }
  var gcode_1, hasRequiredGcode;
  function requireGcode() {
    if (hasRequiredGcode) return gcode_1;
    hasRequiredGcode = 1;
    function O(_) {
      const H = _.regex, V = {
        $pattern: /[A-Z]+|%/,
        keyword: [
          // conditions
          "THEN",
          "ELSE",
          "ENDIF",
          "IF",
          // controls
          "GOTO",
          "DO",
          "WHILE",
          "WH",
          "END",
          "CALL",
          // scoping
          "SUB",
          "ENDSUB",
          // comparisons
          "EQ",
          "NE",
          "LT",
          "GT",
          "LE",
          "GE",
          "AND",
          "OR",
          "XOR",
          // start/end of program
          "%"
        ],
        built_in: [
          "ATAN",
          "ABS",
          "ACOS",
          "ASIN",
          "COS",
          "EXP",
          "FIX",
          "FUP",
          "ROUND",
          "LN",
          "SIN",
          "SQRT",
          "TAN",
          "EXISTS"
        ]
      }, q = /\b/;
      function G(ce, le) {
        if (ce.index === 0)
          return;
        const ue = ce.input[ce.index - 1];
        ue >= "0" && ue <= "9" || ue !== "_" && le.ignoreMatch();
      }
      const J = /[+-]?((\.\d+)|(\d+)(\.\d*)?)/, Z = /[GM]\s*\d+(\.\d+)?/, te = /T\s*\d+/, ne = /O\s*\d+/, ee = /O<.+>/, ae = /[ABCUVWXYZ]\s*/, oe = /[FHIJKPQRS]\s*/, se = [
        // comments
        _.COMMENT(/\(/, /\)/),
        _.COMMENT(/;/, /$/),
        _.APOS_STRING_MODE,
        _.QUOTE_STRING_MODE,
        _.C_NUMBER_MODE,
        // gcodes
        {
          scope: "title.function",
          variants: [
            // G General functions: G0, G5.1, G5.2, …
            // M Misc functions: M0, M55.6, M199, …
            { match: H.concat(q, Z) },
            {
              begin: Z,
              "on:begin": G
            },
            // T Tools
            { match: H.concat(q, te) },
            {
              begin: te,
              "on:begin": G
            }
          ]
        },
        {
          scope: "symbol",
          variants: [
            // O Subroutine ID: O100, O110, …
            { match: H.concat(q, ne) },
            {
              begin: ne,
              "on:begin": G
            },
            // O Subroutine name: O<some>, …
            { match: H.concat(q, ee) },
            {
              begin: ee,
              "on:begin": G
            },
            // Checksum at end of line: *71, *199, …
            { match: /\*\s*\d+\s*$/ }
          ]
        },
        {
          scope: "operator",
          // N Line number: N1, N2, N1020, …
          match: /^N\s*\d+/
        },
        {
          scope: "variable",
          match: /-?#\s*\d+/
        },
        {
          scope: "property",
          // Physical axes,
          variants: [
            { match: H.concat(q, ae, J) },
            {
              begin: H.concat(ae, J),
              "on:begin": G
            }
          ]
        },
        {
          scope: "params",
          // Different types of parameters
          variants: [
            { match: H.concat(q, oe, J) },
            {
              begin: H.concat(oe, J),
              "on:begin": G
            }
          ]
        }
      ];
      return {
        name: "G-code (ISO 6983)",
        aliases: ["nc"],
        // Some implementations (CNC controls) of G-code are interoperable with uppercase and lowercase letters seamlessly.
        // However, most prefer all uppercase and uppercase is customary.
        case_insensitive: !0,
        // TODO: post v12 with the use of look-behind this can be enabled
        disableAutodetect: !0,
        keywords: V,
        contains: se
      };
    }
    return gcode_1 = O, gcode_1;
  }
  var gherkin_1, hasRequiredGherkin;
  function requireGherkin() {
    if (hasRequiredGherkin) return gherkin_1;
    hasRequiredGherkin = 1;
    function O(_) {
      return {
        name: "Gherkin",
        aliases: ["feature"],
        keywords: "Feature Background Ability Business Need Scenario Scenarios Scenario Outline Scenario Template Examples Given And Then But When",
        contains: [
          {
            className: "symbol",
            begin: "\\*",
            relevance: 0
          },
          {
            className: "meta",
            begin: "@[^@\\s]+"
          },
          {
            begin: "\\|",
            end: "\\|\\w*$",
            contains: [
              {
                className: "string",
                begin: "[^|]+"
              }
            ]
          },
          {
            className: "variable",
            begin: "<",
            end: ">"
          },
          _.HASH_COMMENT_MODE,
          {
            className: "string",
            begin: '"""',
            end: '"""'
          },
          _.QUOTE_STRING_MODE
        ]
      };
    }
    return gherkin_1 = O, gherkin_1;
  }
  var glsl_1, hasRequiredGlsl;
  function requireGlsl() {
    if (hasRequiredGlsl) return glsl_1;
    hasRequiredGlsl = 1;
    function O(_) {
      return {
        name: "GLSL",
        keywords: {
          keyword: (
            // Statements
            "break continue discard do else for if return while switch case default attribute binding buffer ccw centroid centroid varying coherent column_major const cw depth_any depth_greater depth_less depth_unchanged early_fragment_tests equal_spacing flat fractional_even_spacing fractional_odd_spacing highp in index inout invariant invocations isolines layout line_strip lines lines_adjacency local_size_x local_size_y local_size_z location lowp max_vertices mediump noperspective offset origin_upper_left out packed patch pixel_center_integer point_mode points precise precision quads r11f_g11f_b10f r16 r16_snorm r16f r16i r16ui r32f r32i r32ui r8 r8_snorm r8i r8ui readonly restrict rg16 rg16_snorm rg16f rg16i rg16ui rg32f rg32i rg32ui rg8 rg8_snorm rg8i rg8ui rgb10_a2 rgb10_a2ui rgba16 rgba16_snorm rgba16f rgba16i rgba16ui rgba32f rgba32i rgba32ui rgba8 rgba8_snorm rgba8i rgba8ui row_major sample shared smooth std140 std430 stream triangle_strip triangles triangles_adjacency uniform varying vertices volatile writeonly"
          ),
          type: "atomic_uint bool bvec2 bvec3 bvec4 dmat2 dmat2x2 dmat2x3 dmat2x4 dmat3 dmat3x2 dmat3x3 dmat3x4 dmat4 dmat4x2 dmat4x3 dmat4x4 double dvec2 dvec3 dvec4 float iimage1D iimage1DArray iimage2D iimage2DArray iimage2DMS iimage2DMSArray iimage2DRect iimage3D iimageBuffer iimageCube iimageCubeArray image1D image1DArray image2D image2DArray image2DMS image2DMSArray image2DRect image3D imageBuffer imageCube imageCubeArray int isampler1D isampler1DArray isampler2D isampler2DArray isampler2DMS isampler2DMSArray isampler2DRect isampler3D isamplerBuffer isamplerCube isamplerCubeArray ivec2 ivec3 ivec4 mat2 mat2x2 mat2x3 mat2x4 mat3 mat3x2 mat3x3 mat3x4 mat4 mat4x2 mat4x3 mat4x4 sampler1D sampler1DArray sampler1DArrayShadow sampler1DShadow sampler2D sampler2DArray sampler2DArrayShadow sampler2DMS sampler2DMSArray sampler2DRect sampler2DRectShadow sampler2DShadow sampler3D samplerBuffer samplerCube samplerCubeArray samplerCubeArrayShadow samplerCubeShadow image1D uimage1DArray uimage2D uimage2DArray uimage2DMS uimage2DMSArray uimage2DRect uimage3D uimageBuffer uimageCube uimageCubeArray uint usampler1D usampler1DArray usampler2D usampler2DArray usampler2DMS usampler2DMSArray usampler2DRect usampler3D samplerBuffer usamplerCube usamplerCubeArray uvec2 uvec3 uvec4 vec2 vec3 vec4 void",
          built_in: (
            // Constants
            "gl_MaxAtomicCounterBindings gl_MaxAtomicCounterBufferSize gl_MaxClipDistances gl_MaxClipPlanes gl_MaxCombinedAtomicCounterBuffers gl_MaxCombinedAtomicCounters gl_MaxCombinedImageUniforms gl_MaxCombinedImageUnitsAndFragmentOutputs gl_MaxCombinedTextureImageUnits gl_MaxComputeAtomicCounterBuffers gl_MaxComputeAtomicCounters gl_MaxComputeImageUniforms gl_MaxComputeTextureImageUnits gl_MaxComputeUniformComponents gl_MaxComputeWorkGroupCount gl_MaxComputeWorkGroupSize gl_MaxDrawBuffers gl_MaxFragmentAtomicCounterBuffers gl_MaxFragmentAtomicCounters gl_MaxFragmentImageUniforms gl_MaxFragmentInputComponents gl_MaxFragmentInputVectors gl_MaxFragmentUniformComponents gl_MaxFragmentUniformVectors gl_MaxGeometryAtomicCounterBuffers gl_MaxGeometryAtomicCounters gl_MaxGeometryImageUniforms gl_MaxGeometryInputComponents gl_MaxGeometryOutputComponents gl_MaxGeometryOutputVertices gl_MaxGeometryTextureImageUnits gl_MaxGeometryTotalOutputComponents gl_MaxGeometryUniformComponents gl_MaxGeometryVaryingComponents gl_MaxImageSamples gl_MaxImageUnits gl_MaxLights gl_MaxPatchVertices gl_MaxProgramTexelOffset gl_MaxTessControlAtomicCounterBuffers gl_MaxTessControlAtomicCounters gl_MaxTessControlImageUniforms gl_MaxTessControlInputComponents gl_MaxTessControlOutputComponents gl_MaxTessControlTextureImageUnits gl_MaxTessControlTotalOutputComponents gl_MaxTessControlUniformComponents gl_MaxTessEvaluationAtomicCounterBuffers gl_MaxTessEvaluationAtomicCounters gl_MaxTessEvaluationImageUniforms gl_MaxTessEvaluationInputComponents gl_MaxTessEvaluationOutputComponents gl_MaxTessEvaluationTextureImageUnits gl_MaxTessEvaluationUniformComponents gl_MaxTessGenLevel gl_MaxTessPatchComponents gl_MaxTextureCoords gl_MaxTextureImageUnits gl_MaxTextureUnits gl_MaxVaryingComponents gl_MaxVaryingFloats gl_MaxVaryingVectors gl_MaxVertexAtomicCounterBuffers gl_MaxVertexAtomicCounters gl_MaxVertexAttribs gl_MaxVertexImageUniforms gl_MaxVertexOutputComponents gl_MaxVertexOutputVectors gl_MaxVertexTextureImageUnits gl_MaxVertexUniformComponents gl_MaxVertexUniformVectors gl_MaxViewports gl_MinProgramTexelOffset gl_BackColor gl_BackLightModelProduct gl_BackLightProduct gl_BackMaterial gl_BackSecondaryColor gl_ClipDistance gl_ClipPlane gl_ClipVertex gl_Color gl_DepthRange gl_EyePlaneQ gl_EyePlaneR gl_EyePlaneS gl_EyePlaneT gl_Fog gl_FogCoord gl_FogFragCoord gl_FragColor gl_FragCoord gl_FragData gl_FragDepth gl_FrontColor gl_FrontFacing gl_FrontLightModelProduct gl_FrontLightProduct gl_FrontMaterial gl_FrontSecondaryColor gl_GlobalInvocationID gl_InstanceID gl_InvocationID gl_Layer gl_LightModel gl_LightSource gl_LocalInvocationID gl_LocalInvocationIndex gl_ModelViewMatrix gl_ModelViewMatrixInverse gl_ModelViewMatrixInverseTranspose gl_ModelViewMatrixTranspose gl_ModelViewProjectionMatrix gl_ModelViewProjectionMatrixInverse gl_ModelViewProjectionMatrixInverseTranspose gl_ModelViewProjectionMatrixTranspose gl_MultiTexCoord0 gl_MultiTexCoord1 gl_MultiTexCoord2 gl_MultiTexCoord3 gl_MultiTexCoord4 gl_MultiTexCoord5 gl_MultiTexCoord6 gl_MultiTexCoord7 gl_Normal gl_NormalMatrix gl_NormalScale gl_NumSamples gl_NumWorkGroups gl_ObjectPlaneQ gl_ObjectPlaneR gl_ObjectPlaneS gl_ObjectPlaneT gl_PatchVerticesIn gl_Point gl_PointCoord gl_PointSize gl_Position gl_PrimitiveID gl_PrimitiveIDIn gl_ProjectionMatrix gl_ProjectionMatrixInverse gl_ProjectionMatrixInverseTranspose gl_ProjectionMatrixTranspose gl_SampleID gl_SampleMask gl_SampleMaskIn gl_SamplePosition gl_SecondaryColor gl_TessCoord gl_TessLevelInner gl_TessLevelOuter gl_TexCoord gl_TextureEnvColor gl_TextureMatrix gl_TextureMatrixInverse gl_TextureMatrixInverseTranspose gl_TextureMatrixTranspose gl_Vertex gl_VertexID gl_ViewportIndex gl_WorkGroupID gl_WorkGroupSize gl_in gl_out EmitStreamVertex EmitVertex EndPrimitive EndStreamPrimitive abs acos acosh all any asin asinh atan atanh atomicAdd atomicAnd atomicCompSwap atomicCounter atomicCounterDecrement atomicCounterIncrement atomicExchange atomicMax atomicMin atomicOr atomicXor barrier bitCount bitfieldExtract bitfieldInsert bitfieldReverse ceil clamp cos cosh cross dFdx dFdy degrees determinant distance dot equal exp exp2 faceforward findLSB findMSB floatBitsToInt floatBitsToUint floor fma fract frexp ftransform fwidth greaterThan greaterThanEqual groupMemoryBarrier imageAtomicAdd imageAtomicAnd imageAtomicCompSwap imageAtomicExchange imageAtomicMax imageAtomicMin imageAtomicOr imageAtomicXor imageLoad imageSize imageStore imulExtended intBitsToFloat interpolateAtCentroid interpolateAtOffset interpolateAtSample inverse inversesqrt isinf isnan ldexp length lessThan lessThanEqual log log2 matrixCompMult max memoryBarrier memoryBarrierAtomicCounter memoryBarrierBuffer memoryBarrierImage memoryBarrierShared min mix mod modf noise1 noise2 noise3 noise4 normalize not notEqual outerProduct packDouble2x32 packHalf2x16 packSnorm2x16 packSnorm4x8 packUnorm2x16 packUnorm4x8 pow radians reflect refract round roundEven shadow1D shadow1DLod shadow1DProj shadow1DProjLod shadow2D shadow2DLod shadow2DProj shadow2DProjLod sign sin sinh smoothstep sqrt step tan tanh texelFetch texelFetchOffset texture texture1D texture1DLod texture1DProj texture1DProjLod texture2D texture2DLod texture2DProj texture2DProjLod texture3D texture3DLod texture3DProj texture3DProjLod textureCube textureCubeLod textureGather textureGatherOffset textureGatherOffsets textureGrad textureGradOffset textureLod textureLodOffset textureOffset textureProj textureProjGrad textureProjGradOffset textureProjLod textureProjLodOffset textureProjOffset textureQueryLevels textureQueryLod textureSize transpose trunc uaddCarry uintBitsToFloat umulExtended unpackDouble2x32 unpackHalf2x16 unpackSnorm2x16 unpackSnorm4x8 unpackUnorm2x16 unpackUnorm4x8 usubBorrow"
          ),
          literal: "true false"
        },
        illegal: '"',
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.C_NUMBER_MODE,
          {
            className: "meta",
            begin: "#",
            end: "$"
          }
        ]
      };
    }
    return glsl_1 = O, glsl_1;
  }
  var gml_1, hasRequiredGml;
  function requireGml() {
    if (hasRequiredGml) return gml_1;
    hasRequiredGml = 1;
    function O(_) {
      return {
        name: "GML",
        case_insensitive: !1,
        // language is case-insensitive
        keywords: {
          keyword: [
            "#endregion",
            "#macro",
            "#region",
            "and",
            "begin",
            "break",
            "case",
            "constructor",
            "continue",
            "default",
            "delete",
            "div",
            "do",
            "else",
            "end",
            "enum",
            "exit",
            "for",
            "function",
            "globalvar",
            "if",
            "mod",
            "new",
            "not",
            "or",
            "repeat",
            "return",
            "static",
            "switch",
            "then",
            "until",
            "var",
            "while",
            "with",
            "xor"
          ],
          built_in: [
            "abs",
            "alarm_get",
            "alarm_set",
            "angle_difference",
            "animcurve_channel_evaluate",
            "animcurve_channel_new",
            "animcurve_create",
            "animcurve_destroy",
            "animcurve_exists",
            "animcurve_get",
            "animcurve_get_channel",
            "animcurve_get_channel_index",
            "animcurve_point_new",
            "ansi_char",
            "application_get_position",
            "application_surface_draw_enable",
            "application_surface_enable",
            "application_surface_is_enabled",
            "arccos",
            "arcsin",
            "arctan",
            "arctan2",
            "array_all",
            "array_any",
            "array_concat",
            "array_contains",
            "array_contains_ext",
            "array_copy",
            "array_copy_while",
            "array_create",
            "array_create_ext",
            "array_delete",
            "array_equals",
            "array_filter",
            "array_filter_ext",
            "array_find_index",
            "array_first",
            "array_foreach",
            "array_get",
            "array_get_index",
            "array_insert",
            "array_intersection",
            "array_last",
            "array_length",
            "array_map",
            "array_map_ext",
            "array_pop",
            "array_push",
            "array_reduce",
            "array_resize",
            "array_reverse",
            "array_reverse_ext",
            "array_set",
            "array_shuffle",
            "array_shuffle_ext",
            "array_sort",
            "array_union",
            "array_unique",
            "array_unique_ext",
            "asset_add_tags",
            "asset_clear_tags",
            "asset_get_ids",
            "asset_get_index",
            "asset_get_tags",
            "asset_get_type",
            "asset_has_any_tag",
            "asset_has_tags",
            "asset_remove_tags",
            "audio_bus_clear_emitters",
            "audio_bus_create",
            "audio_bus_get_emitters",
            "audio_channel_num",
            "audio_create_buffer_sound",
            "audio_create_play_queue",
            "audio_create_stream",
            "audio_create_sync_group",
            "audio_debug",
            "audio_destroy_stream",
            "audio_destroy_sync_group",
            "audio_effect_create",
            "audio_emitter_bus",
            "audio_emitter_create",
            "audio_emitter_exists",
            "audio_emitter_falloff",
            "audio_emitter_free",
            "audio_emitter_gain",
            "audio_emitter_get_bus",
            "audio_emitter_get_gain",
            "audio_emitter_get_listener_mask",
            "audio_emitter_get_pitch",
            "audio_emitter_get_vx",
            "audio_emitter_get_vy",
            "audio_emitter_get_vz",
            "audio_emitter_get_x",
            "audio_emitter_get_y",
            "audio_emitter_get_z",
            "audio_emitter_pitch",
            "audio_emitter_position",
            "audio_emitter_set_listener_mask",
            "audio_emitter_velocity",
            "audio_exists",
            "audio_falloff_set_model",
            "audio_free_buffer_sound",
            "audio_free_play_queue",
            "audio_get_listener_count",
            "audio_get_listener_info",
            "audio_get_listener_mask",
            "audio_get_master_gain",
            "audio_get_name",
            "audio_get_recorder_count",
            "audio_get_recorder_info",
            "audio_get_type",
            "audio_group_get_assets",
            "audio_group_get_gain",
            "audio_group_is_loaded",
            "audio_group_load",
            "audio_group_load_progress",
            "audio_group_name",
            "audio_group_set_gain",
            "audio_group_stop_all",
            "audio_group_unload",
            "audio_is_paused",
            "audio_is_playing",
            "audio_listener_get_data",
            "audio_listener_orientation",
            "audio_listener_position",
            "audio_listener_set_orientation",
            "audio_listener_set_position",
            "audio_listener_set_velocity",
            "audio_listener_velocity",
            "audio_master_gain",
            "audio_pause_all",
            "audio_pause_sound",
            "audio_pause_sync_group",
            "audio_play_in_sync_group",
            "audio_play_sound",
            "audio_play_sound_at",
            "audio_play_sound_ext",
            "audio_play_sound_on",
            "audio_queue_sound",
            "audio_resume_all",
            "audio_resume_sound",
            "audio_resume_sync_group",
            "audio_set_listener_mask",
            "audio_set_master_gain",
            "audio_sound_gain",
            "audio_sound_get_audio_group",
            "audio_sound_get_gain",
            "audio_sound_get_listener_mask",
            "audio_sound_get_loop",
            "audio_sound_get_loop_end",
            "audio_sound_get_loop_start",
            "audio_sound_get_pitch",
            "audio_sound_get_track_position",
            "audio_sound_is_playable",
            "audio_sound_length",
            "audio_sound_loop",
            "audio_sound_loop_end",
            "audio_sound_loop_start",
            "audio_sound_pitch",
            "audio_sound_set_listener_mask",
            "audio_sound_set_track_position",
            "audio_start_recording",
            "audio_start_sync_group",
            "audio_stop_all",
            "audio_stop_recording",
            "audio_stop_sound",
            "audio_stop_sync_group",
            "audio_sync_group_debug",
            "audio_sync_group_get_track_pos",
            "audio_sync_group_is_paused",
            "audio_sync_group_is_playing",
            "audio_system_is_available",
            "audio_system_is_initialised",
            "base64_decode",
            "base64_encode",
            "bool",
            "browser_input_capture",
            "buffer_async_group_begin",
            "buffer_async_group_end",
            "buffer_async_group_option",
            "buffer_base64_decode",
            "buffer_base64_decode_ext",
            "buffer_base64_encode",
            "buffer_compress",
            "buffer_copy",
            "buffer_copy_from_vertex_buffer",
            "buffer_copy_stride",
            "buffer_crc32",
            "buffer_create",
            "buffer_create_from_vertex_buffer",
            "buffer_create_from_vertex_buffer_ext",
            "buffer_decompress",
            "buffer_delete",
            "buffer_exists",
            "buffer_fill",
            "buffer_get_address",
            "buffer_get_alignment",
            "buffer_get_size",
            "buffer_get_surface",
            "buffer_get_type",
            "buffer_load",
            "buffer_load_async",
            "buffer_load_ext",
            "buffer_load_partial",
            "buffer_md5",
            "buffer_peek",
            "buffer_poke",
            "buffer_read",
            "buffer_resize",
            "buffer_save",
            "buffer_save_async",
            "buffer_save_ext",
            "buffer_seek",
            "buffer_set_surface",
            "buffer_set_used_size",
            "buffer_sha1",
            "buffer_sizeof",
            "buffer_tell",
            "buffer_write",
            "call_cancel",
            "call_later",
            "camera_apply",
            "camera_copy_transforms",
            "camera_create",
            "camera_create_view",
            "camera_destroy",
            "camera_get_active",
            "camera_get_begin_script",
            "camera_get_default",
            "camera_get_end_script",
            "camera_get_proj_mat",
            "camera_get_update_script",
            "camera_get_view_angle",
            "camera_get_view_border_x",
            "camera_get_view_border_y",
            "camera_get_view_height",
            "camera_get_view_mat",
            "camera_get_view_speed_x",
            "camera_get_view_speed_y",
            "camera_get_view_target",
            "camera_get_view_width",
            "camera_get_view_x",
            "camera_get_view_y",
            "camera_set_begin_script",
            "camera_set_default",
            "camera_set_end_script",
            "camera_set_proj_mat",
            "camera_set_update_script",
            "camera_set_view_angle",
            "camera_set_view_border",
            "camera_set_view_mat",
            "camera_set_view_pos",
            "camera_set_view_size",
            "camera_set_view_speed",
            "camera_set_view_target",
            "ceil",
            "choose",
            "chr",
            "clamp",
            "clickable_add",
            "clickable_add_ext",
            "clickable_change",
            "clickable_change_ext",
            "clickable_delete",
            "clickable_exists",
            "clickable_set_style",
            "clipboard_get_text",
            "clipboard_has_text",
            "clipboard_set_text",
            "cloud_file_save",
            "cloud_string_save",
            "cloud_synchronise",
            "code_is_compiled",
            "collision_circle",
            "collision_circle_list",
            "collision_ellipse",
            "collision_ellipse_list",
            "collision_line",
            "collision_line_list",
            "collision_point",
            "collision_point_list",
            "collision_rectangle",
            "collision_rectangle_list",
            "color_get_blue",
            "color_get_green",
            "color_get_hue",
            "color_get_red",
            "color_get_saturation",
            "color_get_value",
            "colour_get_blue",
            "colour_get_green",
            "colour_get_hue",
            "colour_get_red",
            "colour_get_saturation",
            "colour_get_value",
            "cos",
            "darccos",
            "darcsin",
            "darctan",
            "darctan2",
            "date_compare_date",
            "date_compare_datetime",
            "date_compare_time",
            "date_create_datetime",
            "date_current_datetime",
            "date_date_of",
            "date_date_string",
            "date_datetime_string",
            "date_day_span",
            "date_days_in_month",
            "date_days_in_year",
            "date_get_day",
            "date_get_day_of_year",
            "date_get_hour",
            "date_get_hour_of_year",
            "date_get_minute",
            "date_get_minute_of_year",
            "date_get_month",
            "date_get_second",
            "date_get_second_of_year",
            "date_get_timezone",
            "date_get_week",
            "date_get_weekday",
            "date_get_year",
            "date_hour_span",
            "date_inc_day",
            "date_inc_hour",
            "date_inc_minute",
            "date_inc_month",
            "date_inc_second",
            "date_inc_week",
            "date_inc_year",
            "date_is_today",
            "date_leap_year",
            "date_minute_span",
            "date_month_span",
            "date_second_span",
            "date_set_timezone",
            "date_time_of",
            "date_time_string",
            "date_valid_datetime",
            "date_week_span",
            "date_year_span",
            "db_to_lin",
            "dbg_add_font_glyphs",
            "dbg_button",
            "dbg_checkbox",
            "dbg_color",
            "dbg_colour",
            "dbg_drop_down",
            "dbg_same_line",
            "dbg_section",
            "dbg_section_delete",
            "dbg_section_exists",
            "dbg_slider",
            "dbg_slider_int",
            "dbg_sprite",
            "dbg_text",
            "dbg_text_input",
            "dbg_view",
            "dbg_view_delete",
            "dbg_view_exists",
            "dbg_watch",
            "dcos",
            "debug_event",
            "debug_get_callstack",
            "degtorad",
            "device_get_tilt_x",
            "device_get_tilt_y",
            "device_get_tilt_z",
            "device_is_keypad_open",
            "device_mouse_check_button",
            "device_mouse_check_button_pressed",
            "device_mouse_check_button_released",
            "device_mouse_dbclick_enable",
            "device_mouse_raw_x",
            "device_mouse_raw_y",
            "device_mouse_x",
            "device_mouse_x_to_gui",
            "device_mouse_y",
            "device_mouse_y_to_gui",
            "directory_create",
            "directory_destroy",
            "directory_exists",
            "display_get_dpi_x",
            "display_get_dpi_y",
            "display_get_frequency",
            "display_get_gui_height",
            "display_get_gui_width",
            "display_get_height",
            "display_get_orientation",
            "display_get_sleep_margin",
            "display_get_timing_method",
            "display_get_width",
            "display_mouse_get_x",
            "display_mouse_get_y",
            "display_mouse_set",
            "display_reset",
            "display_set_gui_maximise",
            "display_set_gui_maximize",
            "display_set_gui_size",
            "display_set_sleep_margin",
            "display_set_timing_method",
            "display_set_ui_visibility",
            "distance_to_object",
            "distance_to_point",
            "dot_product",
            "dot_product_3d",
            "dot_product_3d_normalised",
            "dot_product_3d_normalized",
            "dot_product_normalised",
            "dot_product_normalized",
            "draw_arrow",
            "draw_button",
            "draw_circle",
            "draw_circle_color",
            "draw_circle_colour",
            "draw_clear",
            "draw_clear_alpha",
            "draw_ellipse",
            "draw_ellipse_color",
            "draw_ellipse_colour",
            "draw_enable_drawevent",
            "draw_enable_skeleton_blendmodes",
            "draw_enable_swf_aa",
            "draw_flush",
            "draw_get_alpha",
            "draw_get_color",
            "draw_get_colour",
            "draw_get_enable_skeleton_blendmodes",
            "draw_get_font",
            "draw_get_halign",
            "draw_get_lighting",
            "draw_get_swf_aa_level",
            "draw_get_valign",
            "draw_getpixel",
            "draw_getpixel_ext",
            "draw_healthbar",
            "draw_highscore",
            "draw_light_define_ambient",
            "draw_light_define_direction",
            "draw_light_define_point",
            "draw_light_enable",
            "draw_light_get",
            "draw_light_get_ambient",
            "draw_line",
            "draw_line_color",
            "draw_line_colour",
            "draw_line_width",
            "draw_line_width_color",
            "draw_line_width_colour",
            "draw_path",
            "draw_point",
            "draw_point_color",
            "draw_point_colour",
            "draw_primitive_begin",
            "draw_primitive_begin_texture",
            "draw_primitive_end",
            "draw_rectangle",
            "draw_rectangle_color",
            "draw_rectangle_colour",
            "draw_roundrect",
            "draw_roundrect_color",
            "draw_roundrect_color_ext",
            "draw_roundrect_colour",
            "draw_roundrect_colour_ext",
            "draw_roundrect_ext",
            "draw_self",
            "draw_set_alpha",
            "draw_set_circle_precision",
            "draw_set_color",
            "draw_set_colour",
            "draw_set_font",
            "draw_set_halign",
            "draw_set_lighting",
            "draw_set_swf_aa_level",
            "draw_set_valign",
            "draw_skeleton",
            "draw_skeleton_collision",
            "draw_skeleton_instance",
            "draw_skeleton_time",
            "draw_sprite",
            "draw_sprite_ext",
            "draw_sprite_general",
            "draw_sprite_part",
            "draw_sprite_part_ext",
            "draw_sprite_pos",
            "draw_sprite_stretched",
            "draw_sprite_stretched_ext",
            "draw_sprite_tiled",
            "draw_sprite_tiled_ext",
            "draw_surface",
            "draw_surface_ext",
            "draw_surface_general",
            "draw_surface_part",
            "draw_surface_part_ext",
            "draw_surface_stretched",
            "draw_surface_stretched_ext",
            "draw_surface_tiled",
            "draw_surface_tiled_ext",
            "draw_text",
            "draw_text_color",
            "draw_text_colour",
            "draw_text_ext",
            "draw_text_ext_color",
            "draw_text_ext_colour",
            "draw_text_ext_transformed",
            "draw_text_ext_transformed_color",
            "draw_text_ext_transformed_colour",
            "draw_text_transformed",
            "draw_text_transformed_color",
            "draw_text_transformed_colour",
            "draw_texture_flush",
            "draw_tile",
            "draw_tilemap",
            "draw_triangle",
            "draw_triangle_color",
            "draw_triangle_colour",
            "draw_vertex",
            "draw_vertex_color",
            "draw_vertex_colour",
            "draw_vertex_texture",
            "draw_vertex_texture_color",
            "draw_vertex_texture_colour",
            "ds_exists",
            "ds_grid_add",
            "ds_grid_add_disk",
            "ds_grid_add_grid_region",
            "ds_grid_add_region",
            "ds_grid_clear",
            "ds_grid_copy",
            "ds_grid_create",
            "ds_grid_destroy",
            "ds_grid_get",
            "ds_grid_get_disk_max",
            "ds_grid_get_disk_mean",
            "ds_grid_get_disk_min",
            "ds_grid_get_disk_sum",
            "ds_grid_get_max",
            "ds_grid_get_mean",
            "ds_grid_get_min",
            "ds_grid_get_sum",
            "ds_grid_height",
            "ds_grid_multiply",
            "ds_grid_multiply_disk",
            "ds_grid_multiply_grid_region",
            "ds_grid_multiply_region",
            "ds_grid_read",
            "ds_grid_resize",
            "ds_grid_set",
            "ds_grid_set_disk",
            "ds_grid_set_grid_region",
            "ds_grid_set_region",
            "ds_grid_shuffle",
            "ds_grid_sort",
            "ds_grid_to_mp_grid",
            "ds_grid_value_disk_exists",
            "ds_grid_value_disk_x",
            "ds_grid_value_disk_y",
            "ds_grid_value_exists",
            "ds_grid_value_x",
            "ds_grid_value_y",
            "ds_grid_width",
            "ds_grid_write",
            "ds_list_add",
            "ds_list_clear",
            "ds_list_copy",
            "ds_list_create",
            "ds_list_delete",
            "ds_list_destroy",
            "ds_list_empty",
            "ds_list_find_index",
            "ds_list_find_value",
            "ds_list_insert",
            "ds_list_is_list",
            "ds_list_is_map",
            "ds_list_mark_as_list",
            "ds_list_mark_as_map",
            "ds_list_read",
            "ds_list_replace",
            "ds_list_set",
            "ds_list_shuffle",
            "ds_list_size",
            "ds_list_sort",
            "ds_list_write",
            "ds_map_add",
            "ds_map_add_list",
            "ds_map_add_map",
            "ds_map_clear",
            "ds_map_copy",
            "ds_map_create",
            "ds_map_delete",
            "ds_map_destroy",
            "ds_map_empty",
            "ds_map_exists",
            "ds_map_find_first",
            "ds_map_find_last",
            "ds_map_find_next",
            "ds_map_find_previous",
            "ds_map_find_value",
            "ds_map_is_list",
            "ds_map_is_map",
            "ds_map_keys_to_array",
            "ds_map_read",
            "ds_map_replace",
            "ds_map_replace_list",
            "ds_map_replace_map",
            "ds_map_secure_load",
            "ds_map_secure_load_buffer",
            "ds_map_secure_save",
            "ds_map_secure_save_buffer",
            "ds_map_set",
            "ds_map_size",
            "ds_map_values_to_array",
            "ds_map_write",
            "ds_priority_add",
            "ds_priority_change_priority",
            "ds_priority_clear",
            "ds_priority_copy",
            "ds_priority_create",
            "ds_priority_delete_max",
            "ds_priority_delete_min",
            "ds_priority_delete_value",
            "ds_priority_destroy",
            "ds_priority_empty",
            "ds_priority_find_max",
            "ds_priority_find_min",
            "ds_priority_find_priority",
            "ds_priority_read",
            "ds_priority_size",
            "ds_priority_write",
            "ds_queue_clear",
            "ds_queue_copy",
            "ds_queue_create",
            "ds_queue_dequeue",
            "ds_queue_destroy",
            "ds_queue_empty",
            "ds_queue_enqueue",
            "ds_queue_head",
            "ds_queue_read",
            "ds_queue_size",
            "ds_queue_tail",
            "ds_queue_write",
            "ds_set_precision",
            "ds_stack_clear",
            "ds_stack_copy",
            "ds_stack_create",
            "ds_stack_destroy",
            "ds_stack_empty",
            "ds_stack_pop",
            "ds_stack_push",
            "ds_stack_read",
            "ds_stack_size",
            "ds_stack_top",
            "ds_stack_write",
            "dsin",
            "dtan",
            "effect_clear",
            "effect_create_above",
            "effect_create_below",
            "effect_create_depth",
            "effect_create_layer",
            "environment_get_variable",
            "event_inherited",
            "event_perform",
            "event_perform_async",
            "event_perform_object",
            "event_user",
            "exception_unhandled_handler",
            "exp",
            "extension_exists",
            "extension_get_option_count",
            "extension_get_option_names",
            "extension_get_option_value",
            "extension_get_options",
            "extension_get_version",
            "external_call",
            "external_define",
            "external_free",
            "file_attributes",
            "file_bin_close",
            "file_bin_open",
            "file_bin_position",
            "file_bin_read_byte",
            "file_bin_rewrite",
            "file_bin_seek",
            "file_bin_size",
            "file_bin_write_byte",
            "file_copy",
            "file_delete",
            "file_exists",
            "file_find_close",
            "file_find_first",
            "file_find_next",
            "file_rename",
            "file_text_close",
            "file_text_eof",
            "file_text_eoln",
            "file_text_open_append",
            "file_text_open_from_string",
            "file_text_open_read",
            "file_text_open_write",
            "file_text_read_real",
            "file_text_read_string",
            "file_text_readln",
            "file_text_write_real",
            "file_text_write_string",
            "file_text_writeln",
            "filename_change_ext",
            "filename_dir",
            "filename_drive",
            "filename_ext",
            "filename_name",
            "filename_path",
            "floor",
            "font_add",
            "font_add_enable_aa",
            "font_add_get_enable_aa",
            "font_add_sprite",
            "font_add_sprite_ext",
            "font_cache_glyph",
            "font_delete",
            "font_enable_effects",
            "font_enable_sdf",
            "font_exists",
            "font_get_bold",
            "font_get_first",
            "font_get_fontname",
            "font_get_info",
            "font_get_italic",
            "font_get_last",
            "font_get_name",
            "font_get_sdf_enabled",
            "font_get_sdf_spread",
            "font_get_size",
            "font_get_texture",
            "font_get_uvs",
            "font_replace_sprite",
            "font_replace_sprite_ext",
            "font_sdf_spread",
            "font_set_cache_size",
            "frac",
            "fx_create",
            "fx_get_name",
            "fx_get_parameter",
            "fx_get_parameter_names",
            "fx_get_parameters",
            "fx_get_single_layer",
            "fx_set_parameter",
            "fx_set_parameters",
            "fx_set_single_layer",
            "game_change",
            "game_end",
            "game_get_speed",
            "game_load",
            "game_load_buffer",
            "game_restart",
            "game_save",
            "game_save_buffer",
            "game_set_speed",
            "gamepad_axis_count",
            "gamepad_axis_value",
            "gamepad_button_check",
            "gamepad_button_check_pressed",
            "gamepad_button_check_released",
            "gamepad_button_count",
            "gamepad_button_value",
            "gamepad_get_axis_deadzone",
            "gamepad_get_button_threshold",
            "gamepad_get_description",
            "gamepad_get_device_count",
            "gamepad_get_guid",
            "gamepad_get_mapping",
            "gamepad_get_option",
            "gamepad_hat_count",
            "gamepad_hat_value",
            "gamepad_is_connected",
            "gamepad_is_supported",
            "gamepad_remove_mapping",
            "gamepad_set_axis_deadzone",
            "gamepad_set_button_threshold",
            "gamepad_set_color",
            "gamepad_set_colour",
            "gamepad_set_option",
            "gamepad_set_vibration",
            "gamepad_test_mapping",
            "gc_collect",
            "gc_enable",
            "gc_get_stats",
            "gc_get_target_frame_time",
            "gc_is_enabled",
            "gc_target_frame_time",
            "gesture_double_tap_distance",
            "gesture_double_tap_time",
            "gesture_drag_distance",
            "gesture_drag_time",
            "gesture_flick_speed",
            "gesture_get_double_tap_distance",
            "gesture_get_double_tap_time",
            "gesture_get_drag_distance",
            "gesture_get_drag_time",
            "gesture_get_flick_speed",
            "gesture_get_pinch_angle_away",
            "gesture_get_pinch_angle_towards",
            "gesture_get_pinch_distance",
            "gesture_get_rotate_angle",
            "gesture_get_rotate_time",
            "gesture_get_tap_count",
            "gesture_pinch_angle_away",
            "gesture_pinch_angle_towards",
            "gesture_pinch_distance",
            "gesture_rotate_angle",
            "gesture_rotate_time",
            "gesture_tap_count",
            "get_integer",
            "get_integer_async",
            "get_login_async",
            "get_open_filename",
            "get_open_filename_ext",
            "get_save_filename",
            "get_save_filename_ext",
            "get_string",
            "get_string_async",
            "get_timer",
            "gif_add_surface",
            "gif_open",
            "gif_save",
            "gif_save_buffer",
            "gml_pragma",
            "gml_release_mode",
            "gpu_get_alphatestenable",
            "gpu_get_alphatestref",
            "gpu_get_blendenable",
            "gpu_get_blendmode",
            "gpu_get_blendmode_dest",
            "gpu_get_blendmode_destalpha",
            "gpu_get_blendmode_ext",
            "gpu_get_blendmode_ext_sepalpha",
            "gpu_get_blendmode_src",
            "gpu_get_blendmode_srcalpha",
            "gpu_get_colorwriteenable",
            "gpu_get_colourwriteenable",
            "gpu_get_cullmode",
            "gpu_get_depth",
            "gpu_get_fog",
            "gpu_get_state",
            "gpu_get_tex_filter",
            "gpu_get_tex_filter_ext",
            "gpu_get_tex_max_aniso",
            "gpu_get_tex_max_aniso_ext",
            "gpu_get_tex_max_mip",
            "gpu_get_tex_max_mip_ext",
            "gpu_get_tex_min_mip",
            "gpu_get_tex_min_mip_ext",
            "gpu_get_tex_mip_bias",
            "gpu_get_tex_mip_bias_ext",
            "gpu_get_tex_mip_enable",
            "gpu_get_tex_mip_enable_ext",
            "gpu_get_tex_mip_filter",
            "gpu_get_tex_mip_filter_ext",
            "gpu_get_tex_repeat",
            "gpu_get_tex_repeat_ext",
            "gpu_get_texfilter",
            "gpu_get_texfilter_ext",
            "gpu_get_texrepeat",
            "gpu_get_texrepeat_ext",
            "gpu_get_zfunc",
            "gpu_get_ztestenable",
            "gpu_get_zwriteenable",
            "gpu_pop_state",
            "gpu_push_state",
            "gpu_set_alphatestenable",
            "gpu_set_alphatestref",
            "gpu_set_blendenable",
            "gpu_set_blendmode",
            "gpu_set_blendmode_ext",
            "gpu_set_blendmode_ext_sepalpha",
            "gpu_set_colorwriteenable",
            "gpu_set_colourwriteenable",
            "gpu_set_cullmode",
            "gpu_set_depth",
            "gpu_set_fog",
            "gpu_set_state",
            "gpu_set_tex_filter",
            "gpu_set_tex_filter_ext",
            "gpu_set_tex_max_aniso",
            "gpu_set_tex_max_aniso_ext",
            "gpu_set_tex_max_mip",
            "gpu_set_tex_max_mip_ext",
            "gpu_set_tex_min_mip",
            "gpu_set_tex_min_mip_ext",
            "gpu_set_tex_mip_bias",
            "gpu_set_tex_mip_bias_ext",
            "gpu_set_tex_mip_enable",
            "gpu_set_tex_mip_enable_ext",
            "gpu_set_tex_mip_filter",
            "gpu_set_tex_mip_filter_ext",
            "gpu_set_tex_repeat",
            "gpu_set_tex_repeat_ext",
            "gpu_set_texfilter",
            "gpu_set_texfilter_ext",
            "gpu_set_texrepeat",
            "gpu_set_texrepeat_ext",
            "gpu_set_zfunc",
            "gpu_set_ztestenable",
            "gpu_set_zwriteenable",
            "handle_parse",
            "highscore_add",
            "highscore_clear",
            "highscore_name",
            "highscore_value",
            "http_get",
            "http_get_file",
            "http_get_request_crossorigin",
            "http_post_string",
            "http_request",
            "http_set_request_crossorigin",
            "iap_acquire",
            "iap_activate",
            "iap_consume",
            "iap_enumerate_products",
            "iap_product_details",
            "iap_purchase_details",
            "iap_restore_all",
            "iap_status",
            "ini_close",
            "ini_key_delete",
            "ini_key_exists",
            "ini_open",
            "ini_open_from_string",
            "ini_read_real",
            "ini_read_string",
            "ini_section_delete",
            "ini_section_exists",
            "ini_write_real",
            "ini_write_string",
            "instance_activate_all",
            "instance_activate_layer",
            "instance_activate_object",
            "instance_activate_region",
            "instance_change",
            "instance_copy",
            "instance_create_depth",
            "instance_create_layer",
            "instance_deactivate_all",
            "instance_deactivate_layer",
            "instance_deactivate_object",
            "instance_deactivate_region",
            "instance_destroy",
            "instance_exists",
            "instance_find",
            "instance_furthest",
            "instance_id_get",
            "instance_nearest",
            "instance_number",
            "instance_place",
            "instance_place_list",
            "instance_position",
            "instance_position_list",
            "instanceof",
            "int64",
            "io_clear",
            "irandom",
            "irandom_range",
            "is_array",
            "is_bool",
            "is_callable",
            "is_debug_overlay_open",
            "is_handle",
            "is_infinity",
            "is_instanceof",
            "is_int32",
            "is_int64",
            "is_keyboard_used_debug_overlay",
            "is_method",
            "is_mouse_over_debug_overlay",
            "is_nan",
            "is_numeric",
            "is_ptr",
            "is_real",
            "is_string",
            "is_struct",
            "is_undefined",
            "json_decode",
            "json_encode",
            "json_parse",
            "json_stringify",
            "keyboard_check",
            "keyboard_check_direct",
            "keyboard_check_pressed",
            "keyboard_check_released",
            "keyboard_clear",
            "keyboard_get_map",
            "keyboard_get_numlock",
            "keyboard_key_press",
            "keyboard_key_release",
            "keyboard_set_map",
            "keyboard_set_numlock",
            "keyboard_unset_map",
            "keyboard_virtual_height",
            "keyboard_virtual_hide",
            "keyboard_virtual_show",
            "keyboard_virtual_status",
            "layer_add_instance",
            "layer_background_alpha",
            "layer_background_blend",
            "layer_background_change",
            "layer_background_create",
            "layer_background_destroy",
            "layer_background_exists",
            "layer_background_get_alpha",
            "layer_background_get_blend",
            "layer_background_get_htiled",
            "layer_background_get_id",
            "layer_background_get_index",
            "layer_background_get_speed",
            "layer_background_get_sprite",
            "layer_background_get_stretch",
            "layer_background_get_visible",
            "layer_background_get_vtiled",
            "layer_background_get_xscale",
            "layer_background_get_yscale",
            "layer_background_htiled",
            "layer_background_index",
            "layer_background_speed",
            "layer_background_sprite",
            "layer_background_stretch",
            "layer_background_visible",
            "layer_background_vtiled",
            "layer_background_xscale",
            "layer_background_yscale",
            "layer_clear_fx",
            "layer_create",
            "layer_depth",
            "layer_destroy",
            "layer_destroy_instances",
            "layer_element_move",
            "layer_enable_fx",
            "layer_exists",
            "layer_force_draw_depth",
            "layer_fx_is_enabled",
            "layer_get_all",
            "layer_get_all_elements",
            "layer_get_depth",
            "layer_get_element_layer",
            "layer_get_element_type",
            "layer_get_forced_depth",
            "layer_get_fx",
            "layer_get_hspeed",
            "layer_get_id",
            "layer_get_id_at_depth",
            "layer_get_name",
            "layer_get_script_begin",
            "layer_get_script_end",
            "layer_get_shader",
            "layer_get_target_room",
            "layer_get_visible",
            "layer_get_vspeed",
            "layer_get_x",
            "layer_get_y",
            "layer_has_instance",
            "layer_hspeed",
            "layer_instance_get_instance",
            "layer_is_draw_depth_forced",
            "layer_reset_target_room",
            "layer_script_begin",
            "layer_script_end",
            "layer_sequence_angle",
            "layer_sequence_create",
            "layer_sequence_destroy",
            "layer_sequence_exists",
            "layer_sequence_get_angle",
            "layer_sequence_get_headdir",
            "layer_sequence_get_headpos",
            "layer_sequence_get_instance",
            "layer_sequence_get_length",
            "layer_sequence_get_sequence",
            "layer_sequence_get_speedscale",
            "layer_sequence_get_x",
            "layer_sequence_get_xscale",
            "layer_sequence_get_y",
            "layer_sequence_get_yscale",
            "layer_sequence_headdir",
            "layer_sequence_headpos",
            "layer_sequence_is_finished",
            "layer_sequence_is_paused",
            "layer_sequence_pause",
            "layer_sequence_play",
            "layer_sequence_speedscale",
            "layer_sequence_x",
            "layer_sequence_xscale",
            "layer_sequence_y",
            "layer_sequence_yscale",
            "layer_set_fx",
            "layer_set_target_room",
            "layer_set_visible",
            "layer_shader",
            "layer_sprite_alpha",
            "layer_sprite_angle",
            "layer_sprite_blend",
            "layer_sprite_change",
            "layer_sprite_create",
            "layer_sprite_destroy",
            "layer_sprite_exists",
            "layer_sprite_get_alpha",
            "layer_sprite_get_angle",
            "layer_sprite_get_blend",
            "layer_sprite_get_id",
            "layer_sprite_get_index",
            "layer_sprite_get_speed",
            "layer_sprite_get_sprite",
            "layer_sprite_get_x",
            "layer_sprite_get_xscale",
            "layer_sprite_get_y",
            "layer_sprite_get_yscale",
            "layer_sprite_index",
            "layer_sprite_speed",
            "layer_sprite_x",
            "layer_sprite_xscale",
            "layer_sprite_y",
            "layer_sprite_yscale",
            "layer_tile_alpha",
            "layer_tile_blend",
            "layer_tile_change",
            "layer_tile_create",
            "layer_tile_destroy",
            "layer_tile_exists",
            "layer_tile_get_alpha",
            "layer_tile_get_blend",
            "layer_tile_get_region",
            "layer_tile_get_sprite",
            "layer_tile_get_visible",
            "layer_tile_get_x",
            "layer_tile_get_xscale",
            "layer_tile_get_y",
            "layer_tile_get_yscale",
            "layer_tile_region",
            "layer_tile_visible",
            "layer_tile_x",
            "layer_tile_xscale",
            "layer_tile_y",
            "layer_tile_yscale",
            "layer_tilemap_create",
            "layer_tilemap_destroy",
            "layer_tilemap_exists",
            "layer_tilemap_get_id",
            "layer_vspeed",
            "layer_x",
            "layer_y",
            "lengthdir_x",
            "lengthdir_y",
            "lerp",
            "lin_to_db",
            "ln",
            "load_csv",
            "log10",
            "log2",
            "logn",
            "make_color_hsv",
            "make_color_rgb",
            "make_colour_hsv",
            "make_colour_rgb",
            "math_get_epsilon",
            "math_set_epsilon",
            "matrix_build",
            "matrix_build_identity",
            "matrix_build_lookat",
            "matrix_build_projection_ortho",
            "matrix_build_projection_perspective",
            "matrix_build_projection_perspective_fov",
            "matrix_get",
            "matrix_multiply",
            "matrix_set",
            "matrix_stack_clear",
            "matrix_stack_is_empty",
            "matrix_stack_pop",
            "matrix_stack_push",
            "matrix_stack_set",
            "matrix_stack_top",
            "matrix_transform_vertex",
            "max",
            "md5_file",
            "md5_string_unicode",
            "md5_string_utf8",
            "mean",
            "median",
            "merge_color",
            "merge_colour",
            "method",
            "method_call",
            "method_get_index",
            "method_get_self",
            "min",
            "motion_add",
            "motion_set",
            "mouse_check_button",
            "mouse_check_button_pressed",
            "mouse_check_button_released",
            "mouse_clear",
            "mouse_wheel_down",
            "mouse_wheel_up",
            "move_and_collide",
            "move_bounce_all",
            "move_bounce_solid",
            "move_contact_all",
            "move_contact_solid",
            "move_outside_all",
            "move_outside_solid",
            "move_random",
            "move_snap",
            "move_towards_point",
            "move_wrap",
            "mp_grid_add_cell",
            "mp_grid_add_instances",
            "mp_grid_add_rectangle",
            "mp_grid_clear_all",
            "mp_grid_clear_cell",
            "mp_grid_clear_rectangle",
            "mp_grid_create",
            "mp_grid_destroy",
            "mp_grid_draw",
            "mp_grid_get_cell",
            "mp_grid_path",
            "mp_grid_to_ds_grid",
            "mp_linear_path",
            "mp_linear_path_object",
            "mp_linear_step",
            "mp_linear_step_object",
            "mp_potential_path",
            "mp_potential_path_object",
            "mp_potential_settings",
            "mp_potential_step",
            "mp_potential_step_object",
            "nameof",
            "network_connect",
            "network_connect_async",
            "network_connect_raw",
            "network_connect_raw_async",
            "network_create_server",
            "network_create_server_raw",
            "network_create_socket",
            "network_create_socket_ext",
            "network_destroy",
            "network_resolve",
            "network_send_broadcast",
            "network_send_packet",
            "network_send_raw",
            "network_send_udp",
            "network_send_udp_raw",
            "network_set_config",
            "network_set_timeout",
            "object_exists",
            "object_get_mask",
            "object_get_name",
            "object_get_parent",
            "object_get_persistent",
            "object_get_physics",
            "object_get_solid",
            "object_get_sprite",
            "object_get_visible",
            "object_is_ancestor",
            "object_set_mask",
            "object_set_persistent",
            "object_set_solid",
            "object_set_sprite",
            "object_set_visible",
            "ord",
            "os_check_permission",
            "os_get_config",
            "os_get_info",
            "os_get_language",
            "os_get_region",
            "os_is_network_connected",
            "os_is_paused",
            "os_lock_orientation",
            "os_powersave_enable",
            "os_request_permission",
            "os_set_orientation_lock",
            "parameter_count",
            "parameter_string",
            "part_emitter_burst",
            "part_emitter_clear",
            "part_emitter_create",
            "part_emitter_delay",
            "part_emitter_destroy",
            "part_emitter_destroy_all",
            "part_emitter_enable",
            "part_emitter_exists",
            "part_emitter_interval",
            "part_emitter_region",
            "part_emitter_relative",
            "part_emitter_stream",
            "part_particles_burst",
            "part_particles_clear",
            "part_particles_count",
            "part_particles_create",
            "part_particles_create_color",
            "part_particles_create_colour",
            "part_system_angle",
            "part_system_automatic_draw",
            "part_system_automatic_update",
            "part_system_clear",
            "part_system_color",
            "part_system_colour",
            "part_system_create",
            "part_system_create_layer",
            "part_system_depth",
            "part_system_destroy",
            "part_system_draw_order",
            "part_system_drawit",
            "part_system_exists",
            "part_system_get_info",
            "part_system_get_layer",
            "part_system_global_space",
            "part_system_layer",
            "part_system_position",
            "part_system_update",
            "part_type_alpha1",
            "part_type_alpha2",
            "part_type_alpha3",
            "part_type_blend",
            "part_type_clear",
            "part_type_color1",
            "part_type_color2",
            "part_type_color3",
            "part_type_color_hsv",
            "part_type_color_mix",
            "part_type_color_rgb",
            "part_type_colour1",
            "part_type_colour2",
            "part_type_colour3",
            "part_type_colour_hsv",
            "part_type_colour_mix",
            "part_type_colour_rgb",
            "part_type_create",
            "part_type_death",
            "part_type_destroy",
            "part_type_direction",
            "part_type_exists",
            "part_type_gravity",
            "part_type_life",
            "part_type_orientation",
            "part_type_scale",
            "part_type_shape",
            "part_type_size",
            "part_type_size_x",
            "part_type_size_y",
            "part_type_speed",
            "part_type_sprite",
            "part_type_step",
            "part_type_subimage",
            "particle_exists",
            "particle_get_info",
            "path_add",
            "path_add_point",
            "path_append",
            "path_assign",
            "path_change_point",
            "path_clear_points",
            "path_delete",
            "path_delete_point",
            "path_duplicate",
            "path_end",
            "path_exists",
            "path_flip",
            "path_get_closed",
            "path_get_kind",
            "path_get_length",
            "path_get_name",
            "path_get_number",
            "path_get_point_speed",
            "path_get_point_x",
            "path_get_point_y",
            "path_get_precision",
            "path_get_speed",
            "path_get_x",
            "path_get_y",
            "path_insert_point",
            "path_mirror",
            "path_rescale",
            "path_reverse",
            "path_rotate",
            "path_set_closed",
            "path_set_kind",
            "path_set_precision",
            "path_shift",
            "path_start",
            "physics_apply_angular_impulse",
            "physics_apply_force",
            "physics_apply_impulse",
            "physics_apply_local_force",
            "physics_apply_local_impulse",
            "physics_apply_torque",
            "physics_draw_debug",
            "physics_fixture_add_point",
            "physics_fixture_bind",
            "physics_fixture_bind_ext",
            "physics_fixture_create",
            "physics_fixture_delete",
            "physics_fixture_set_angular_damping",
            "physics_fixture_set_awake",
            "physics_fixture_set_box_shape",
            "physics_fixture_set_chain_shape",
            "physics_fixture_set_circle_shape",
            "physics_fixture_set_collision_group",
            "physics_fixture_set_density",
            "physics_fixture_set_edge_shape",
            "physics_fixture_set_friction",
            "physics_fixture_set_kinematic",
            "physics_fixture_set_linear_damping",
            "physics_fixture_set_polygon_shape",
            "physics_fixture_set_restitution",
            "physics_fixture_set_sensor",
            "physics_get_density",
            "physics_get_friction",
            "physics_get_restitution",
            "physics_joint_delete",
            "physics_joint_distance_create",
            "physics_joint_enable_motor",
            "physics_joint_friction_create",
            "physics_joint_gear_create",
            "physics_joint_get_value",
            "physics_joint_prismatic_create",
            "physics_joint_pulley_create",
            "physics_joint_revolute_create",
            "physics_joint_rope_create",
            "physics_joint_set_value",
            "physics_joint_weld_create",
            "physics_joint_wheel_create",
            "physics_mass_properties",
            "physics_particle_count",
            "physics_particle_create",
            "physics_particle_delete",
            "physics_particle_delete_region_box",
            "physics_particle_delete_region_circle",
            "physics_particle_delete_region_poly",
            "physics_particle_draw",
            "physics_particle_draw_ext",
            "physics_particle_get_damping",
            "physics_particle_get_data",
            "physics_particle_get_data_particle",
            "physics_particle_get_density",
            "physics_particle_get_gravity_scale",
            "physics_particle_get_group_flags",
            "physics_particle_get_max_count",
            "physics_particle_get_radius",
            "physics_particle_group_add_point",
            "physics_particle_group_begin",
            "physics_particle_group_box",
            "physics_particle_group_circle",
            "physics_particle_group_count",
            "physics_particle_group_delete",
            "physics_particle_group_end",
            "physics_particle_group_get_ang_vel",
            "physics_particle_group_get_angle",
            "physics_particle_group_get_centre_x",
            "physics_particle_group_get_centre_y",
            "physics_particle_group_get_data",
            "physics_particle_group_get_inertia",
            "physics_particle_group_get_mass",
            "physics_particle_group_get_vel_x",
            "physics_particle_group_get_vel_y",
            "physics_particle_group_get_x",
            "physics_particle_group_get_y",
            "physics_particle_group_join",
            "physics_particle_group_polygon",
            "physics_particle_set_category_flags",
            "physics_particle_set_damping",
            "physics_particle_set_density",
            "physics_particle_set_flags",
            "physics_particle_set_gravity_scale",
            "physics_particle_set_group_flags",
            "physics_particle_set_max_count",
            "physics_particle_set_radius",
            "physics_pause_enable",
            "physics_remove_fixture",
            "physics_set_density",
            "physics_set_friction",
            "physics_set_restitution",
            "physics_test_overlap",
            "physics_world_create",
            "physics_world_draw_debug",
            "physics_world_gravity",
            "physics_world_update_iterations",
            "physics_world_update_speed",
            "place_empty",
            "place_free",
            "place_meeting",
            "place_snapped",
            "point_direction",
            "point_distance",
            "point_distance_3d",
            "point_in_circle",
            "point_in_rectangle",
            "point_in_triangle",
            "position_change",
            "position_destroy",
            "position_empty",
            "position_meeting",
            "power",
            "ptr",
            "radtodeg",
            "random",
            "random_get_seed",
            "random_range",
            "random_set_seed",
            "randomise",
            "randomize",
            "real",
            "rectangle_in_circle",
            "rectangle_in_rectangle",
            "rectangle_in_triangle",
            "ref_create",
            "rollback_chat",
            "rollback_create_game",
            "rollback_define_extra_network_latency",
            "rollback_define_input",
            "rollback_define_input_frame_delay",
            "rollback_define_mock_input",
            "rollback_define_player",
            "rollback_display_events",
            "rollback_get_info",
            "rollback_get_input",
            "rollback_get_player_prefs",
            "rollback_join_game",
            "rollback_leave_game",
            "rollback_set_player_prefs",
            "rollback_start_game",
            "rollback_sync_on_frame",
            "rollback_use_late_join",
            "rollback_use_manual_start",
            "rollback_use_player_prefs",
            "rollback_use_random_input",
            "room_add",
            "room_assign",
            "room_duplicate",
            "room_exists",
            "room_get_camera",
            "room_get_info",
            "room_get_name",
            "room_get_viewport",
            "room_goto",
            "room_goto_next",
            "room_goto_previous",
            "room_instance_add",
            "room_instance_clear",
            "room_next",
            "room_previous",
            "room_restart",
            "room_set_camera",
            "room_set_height",
            "room_set_persistent",
            "room_set_view_enabled",
            "room_set_viewport",
            "room_set_width",
            "round",
            "scheduler_resolution_get",
            "scheduler_resolution_set",
            "screen_save",
            "screen_save_part",
            "script_execute",
            "script_execute_ext",
            "script_exists",
            "script_get_name",
            "sequence_create",
            "sequence_destroy",
            "sequence_exists",
            "sequence_get",
            "sequence_get_objects",
            "sequence_instance_override_object",
            "sequence_keyframe_new",
            "sequence_keyframedata_new",
            "sequence_track_new",
            "sha1_file",
            "sha1_string_unicode",
            "sha1_string_utf8",
            "shader_current",
            "shader_enable_corner_id",
            "shader_get_name",
            "shader_get_sampler_index",
            "shader_get_uniform",
            "shader_is_compiled",
            "shader_reset",
            "shader_set",
            "shader_set_uniform_f",
            "shader_set_uniform_f_array",
            "shader_set_uniform_f_buffer",
            "shader_set_uniform_i",
            "shader_set_uniform_i_array",
            "shader_set_uniform_matrix",
            "shader_set_uniform_matrix_array",
            "shaders_are_supported",
            "shop_leave_rating",
            "show_debug_message",
            "show_debug_message_ext",
            "show_debug_overlay",
            "show_error",
            "show_message",
            "show_message_async",
            "show_question",
            "show_question_async",
            "sign",
            "sin",
            "skeleton_animation_clear",
            "skeleton_animation_get",
            "skeleton_animation_get_duration",
            "skeleton_animation_get_event_frames",
            "skeleton_animation_get_ext",
            "skeleton_animation_get_frame",
            "skeleton_animation_get_frames",
            "skeleton_animation_get_position",
            "skeleton_animation_is_finished",
            "skeleton_animation_is_looping",
            "skeleton_animation_list",
            "skeleton_animation_mix",
            "skeleton_animation_set",
            "skeleton_animation_set_ext",
            "skeleton_animation_set_frame",
            "skeleton_animation_set_position",
            "skeleton_attachment_create",
            "skeleton_attachment_create_color",
            "skeleton_attachment_create_colour",
            "skeleton_attachment_destroy",
            "skeleton_attachment_exists",
            "skeleton_attachment_get",
            "skeleton_attachment_replace",
            "skeleton_attachment_replace_color",
            "skeleton_attachment_replace_colour",
            "skeleton_attachment_set",
            "skeleton_bone_data_get",
            "skeleton_bone_data_set",
            "skeleton_bone_list",
            "skeleton_bone_state_get",
            "skeleton_bone_state_set",
            "skeleton_collision_draw_set",
            "skeleton_find_slot",
            "skeleton_get_bounds",
            "skeleton_get_minmax",
            "skeleton_get_num_bounds",
            "skeleton_skin_create",
            "skeleton_skin_get",
            "skeleton_skin_list",
            "skeleton_skin_set",
            "skeleton_slot_alpha_get",
            "skeleton_slot_color_get",
            "skeleton_slot_color_set",
            "skeleton_slot_colour_get",
            "skeleton_slot_colour_set",
            "skeleton_slot_data",
            "skeleton_slot_data_instance",
            "skeleton_slot_list",
            "sprite_add",
            "sprite_add_ext",
            "sprite_add_from_surface",
            "sprite_assign",
            "sprite_collision_mask",
            "sprite_create_from_surface",
            "sprite_delete",
            "sprite_duplicate",
            "sprite_exists",
            "sprite_flush",
            "sprite_flush_multi",
            "sprite_get_bbox_bottom",
            "sprite_get_bbox_left",
            "sprite_get_bbox_mode",
            "sprite_get_bbox_right",
            "sprite_get_bbox_top",
            "sprite_get_height",
            "sprite_get_info",
            "sprite_get_name",
            "sprite_get_nineslice",
            "sprite_get_number",
            "sprite_get_speed",
            "sprite_get_speed_type",
            "sprite_get_texture",
            "sprite_get_tpe",
            "sprite_get_uvs",
            "sprite_get_width",
            "sprite_get_xoffset",
            "sprite_get_yoffset",
            "sprite_merge",
            "sprite_nineslice_create",
            "sprite_prefetch",
            "sprite_prefetch_multi",
            "sprite_replace",
            "sprite_save",
            "sprite_save_strip",
            "sprite_set_alpha_from_sprite",
            "sprite_set_bbox",
            "sprite_set_bbox_mode",
            "sprite_set_cache_size",
            "sprite_set_cache_size_ext",
            "sprite_set_nineslice",
            "sprite_set_offset",
            "sprite_set_speed",
            "sqr",
            "sqrt",
            "static_get",
            "static_set",
            "string",
            "string_byte_at",
            "string_byte_length",
            "string_char_at",
            "string_concat",
            "string_concat_ext",
            "string_copy",
            "string_count",
            "string_delete",
            "string_digits",
            "string_ends_with",
            "string_ext",
            "string_foreach",
            "string_format",
            "string_hash_to_newline",
            "string_height",
            "string_height_ext",
            "string_insert",
            "string_join",
            "string_join_ext",
            "string_last_pos",
            "string_last_pos_ext",
            "string_length",
            "string_letters",
            "string_lettersdigits",
            "string_lower",
            "string_ord_at",
            "string_pos",
            "string_pos_ext",
            "string_repeat",
            "string_replace",
            "string_replace_all",
            "string_set_byte_at",
            "string_split",
            "string_split_ext",
            "string_starts_with",
            "string_trim",
            "string_trim_end",
            "string_trim_start",
            "string_upper",
            "string_width",
            "string_width_ext",
            "struct_exists",
            "struct_foreach",
            "struct_get",
            "struct_get_from_hash",
            "struct_get_names",
            "struct_names_count",
            "struct_remove",
            "struct_set",
            "struct_set_from_hash",
            "surface_copy",
            "surface_copy_part",
            "surface_create",
            "surface_create_ext",
            "surface_depth_disable",
            "surface_exists",
            "surface_format_is_supported",
            "surface_free",
            "surface_get_depth_disable",
            "surface_get_format",
            "surface_get_height",
            "surface_get_target",
            "surface_get_target_ext",
            "surface_get_texture",
            "surface_get_width",
            "surface_getpixel",
            "surface_getpixel_ext",
            "surface_reset_target",
            "surface_resize",
            "surface_save",
            "surface_save_part",
            "surface_set_target",
            "surface_set_target_ext",
            "tag_get_asset_ids",
            "tag_get_assets",
            "tan",
            "texture_debug_messages",
            "texture_flush",
            "texture_get_height",
            "texture_get_texel_height",
            "texture_get_texel_width",
            "texture_get_uvs",
            "texture_get_width",
            "texture_global_scale",
            "texture_is_ready",
            "texture_prefetch",
            "texture_set_stage",
            "texturegroup_get_fonts",
            "texturegroup_get_names",
            "texturegroup_get_sprites",
            "texturegroup_get_status",
            "texturegroup_get_textures",
            "texturegroup_get_tilesets",
            "texturegroup_load",
            "texturegroup_set_mode",
            "texturegroup_unload",
            "tile_get_empty",
            "tile_get_flip",
            "tile_get_index",
            "tile_get_mirror",
            "tile_get_rotate",
            "tile_set_empty",
            "tile_set_flip",
            "tile_set_index",
            "tile_set_mirror",
            "tile_set_rotate",
            "tilemap_clear",
            "tilemap_get",
            "tilemap_get_at_pixel",
            "tilemap_get_cell_x_at_pixel",
            "tilemap_get_cell_y_at_pixel",
            "tilemap_get_frame",
            "tilemap_get_global_mask",
            "tilemap_get_height",
            "tilemap_get_mask",
            "tilemap_get_tile_height",
            "tilemap_get_tile_width",
            "tilemap_get_tileset",
            "tilemap_get_width",
            "tilemap_get_x",
            "tilemap_get_y",
            "tilemap_set",
            "tilemap_set_at_pixel",
            "tilemap_set_global_mask",
            "tilemap_set_height",
            "tilemap_set_mask",
            "tilemap_set_width",
            "tilemap_tileset",
            "tilemap_x",
            "tilemap_y",
            "tileset_get_info",
            "tileset_get_name",
            "tileset_get_texture",
            "tileset_get_uvs",
            "time_bpm_to_seconds",
            "time_seconds_to_bpm",
            "time_source_create",
            "time_source_destroy",
            "time_source_exists",
            "time_source_get_children",
            "time_source_get_parent",
            "time_source_get_period",
            "time_source_get_reps_completed",
            "time_source_get_reps_remaining",
            "time_source_get_state",
            "time_source_get_time_remaining",
            "time_source_get_units",
            "time_source_pause",
            "time_source_reconfigure",
            "time_source_reset",
            "time_source_resume",
            "time_source_start",
            "time_source_stop",
            "timeline_add",
            "timeline_clear",
            "timeline_delete",
            "timeline_exists",
            "timeline_get_name",
            "timeline_max_moment",
            "timeline_moment_add_script",
            "timeline_moment_clear",
            "timeline_size",
            "typeof",
            "url_get_domain",
            "url_open",
            "url_open_ext",
            "url_open_full",
            "uwp_device_touchscreen_available",
            "uwp_livetile_badge_clear",
            "uwp_livetile_badge_notification",
            "uwp_livetile_notification_begin",
            "uwp_livetile_notification_end",
            "uwp_livetile_notification_expiry",
            "uwp_livetile_notification_image_add",
            "uwp_livetile_notification_secondary_begin",
            "uwp_livetile_notification_tag",
            "uwp_livetile_notification_template_add",
            "uwp_livetile_notification_text_add",
            "uwp_livetile_queue_enable",
            "uwp_livetile_tile_clear",
            "uwp_secondarytile_badge_clear",
            "uwp_secondarytile_badge_notification",
            "uwp_secondarytile_delete",
            "uwp_secondarytile_pin",
            "uwp_secondarytile_tile_clear",
            "variable_clone",
            "variable_get_hash",
            "variable_global_exists",
            "variable_global_get",
            "variable_global_set",
            "variable_instance_exists",
            "variable_instance_get",
            "variable_instance_get_names",
            "variable_instance_names_count",
            "variable_instance_set",
            "variable_struct_exists",
            "variable_struct_get",
            "variable_struct_get_names",
            "variable_struct_names_count",
            "variable_struct_remove",
            "variable_struct_set",
            "vertex_argb",
            "vertex_begin",
            "vertex_color",
            "vertex_colour",
            "vertex_create_buffer",
            "vertex_create_buffer_ext",
            "vertex_create_buffer_from_buffer",
            "vertex_create_buffer_from_buffer_ext",
            "vertex_delete_buffer",
            "vertex_end",
            "vertex_float1",
            "vertex_float2",
            "vertex_float3",
            "vertex_float4",
            "vertex_format_add_color",
            "vertex_format_add_colour",
            "vertex_format_add_custom",
            "vertex_format_add_normal",
            "vertex_format_add_position",
            "vertex_format_add_position_3d",
            "vertex_format_add_texcoord",
            "vertex_format_begin",
            "vertex_format_delete",
            "vertex_format_end",
            "vertex_format_get_info",
            "vertex_freeze",
            "vertex_get_buffer_size",
            "vertex_get_number",
            "vertex_normal",
            "vertex_position",
            "vertex_position_3d",
            "vertex_submit",
            "vertex_submit_ext",
            "vertex_texcoord",
            "vertex_ubyte4",
            "vertex_update_buffer_from_buffer",
            "vertex_update_buffer_from_vertex",
            "video_close",
            "video_draw",
            "video_enable_loop",
            "video_get_duration",
            "video_get_format",
            "video_get_position",
            "video_get_status",
            "video_get_volume",
            "video_is_looping",
            "video_open",
            "video_pause",
            "video_resume",
            "video_seek_to",
            "video_set_volume",
            "view_get_camera",
            "view_get_hport",
            "view_get_surface_id",
            "view_get_visible",
            "view_get_wport",
            "view_get_xport",
            "view_get_yport",
            "view_set_camera",
            "view_set_hport",
            "view_set_surface_id",
            "view_set_visible",
            "view_set_wport",
            "view_set_xport",
            "view_set_yport",
            "virtual_key_add",
            "virtual_key_delete",
            "virtual_key_hide",
            "virtual_key_show",
            "wallpaper_set_config",
            "wallpaper_set_subscriptions",
            "weak_ref_alive",
            "weak_ref_any_alive",
            "weak_ref_create",
            "window_center",
            "window_device",
            "window_enable_borderless_fullscreen",
            "window_get_borderless_fullscreen",
            "window_get_caption",
            "window_get_color",
            "window_get_colour",
            "window_get_cursor",
            "window_get_fullscreen",
            "window_get_height",
            "window_get_showborder",
            "window_get_visible_rects",
            "window_get_width",
            "window_get_x",
            "window_get_y",
            "window_handle",
            "window_has_focus",
            "window_mouse_get_delta_x",
            "window_mouse_get_delta_y",
            "window_mouse_get_locked",
            "window_mouse_get_x",
            "window_mouse_get_y",
            "window_mouse_set",
            "window_mouse_set_locked",
            "window_set_caption",
            "window_set_color",
            "window_set_colour",
            "window_set_cursor",
            "window_set_fullscreen",
            "window_set_max_height",
            "window_set_max_width",
            "window_set_min_height",
            "window_set_min_width",
            "window_set_position",
            "window_set_rectangle",
            "window_set_showborder",
            "window_set_size",
            "window_view_mouse_get_x",
            "window_view_mouse_get_y",
            "window_views_mouse_get_x",
            "window_views_mouse_get_y",
            "winphone_tile_background_color",
            "winphone_tile_background_colour",
            "zip_add_file",
            "zip_create",
            "zip_save",
            "zip_unzip",
            "zip_unzip_async"
          ],
          symbol: [
            "AudioEffect",
            "AudioEffectType",
            "AudioLFOType",
            "GM_build_date",
            "GM_build_type",
            "GM_is_sandboxed",
            "GM_project_filename",
            "GM_runtime_version",
            "GM_version",
            "NaN",
            "_GMFILE_",
            "_GMFUNCTION_",
            "_GMLINE_",
            "alignmentH",
            "alignmentV",
            "all",
            "animcurvetype_bezier",
            "animcurvetype_catmullrom",
            "animcurvetype_linear",
            "asset_animationcurve",
            "asset_font",
            "asset_object",
            "asset_path",
            "asset_room",
            "asset_script",
            "asset_sequence",
            "asset_shader",
            "asset_sound",
            "asset_sprite",
            "asset_tiles",
            "asset_timeline",
            "asset_unknown",
            "audio_3D",
            "audio_bus_main",
            "audio_falloff_exponent_distance",
            "audio_falloff_exponent_distance_clamped",
            "audio_falloff_exponent_distance_scaled",
            "audio_falloff_inverse_distance",
            "audio_falloff_inverse_distance_clamped",
            "audio_falloff_inverse_distance_scaled",
            "audio_falloff_linear_distance",
            "audio_falloff_linear_distance_clamped",
            "audio_falloff_none",
            "audio_mono",
            "audio_stereo",
            "bboxkind_diamond",
            "bboxkind_ellipse",
            "bboxkind_precise",
            "bboxkind_rectangular",
            "bboxmode_automatic",
            "bboxmode_fullimage",
            "bboxmode_manual",
            "bm_add",
            "bm_dest_alpha",
            "bm_dest_color",
            "bm_dest_colour",
            "bm_inv_dest_alpha",
            "bm_inv_dest_color",
            "bm_inv_dest_colour",
            "bm_inv_src_alpha",
            "bm_inv_src_color",
            "bm_inv_src_colour",
            "bm_max",
            "bm_normal",
            "bm_one",
            "bm_src_alpha",
            "bm_src_alpha_sat",
            "bm_src_color",
            "bm_src_colour",
            "bm_subtract",
            "bm_zero",
            "browser_chrome",
            "browser_edge",
            "browser_firefox",
            "browser_ie",
            "browser_ie_mobile",
            "browser_not_a_browser",
            "browser_opera",
            "browser_safari",
            "browser_safari_mobile",
            "browser_tizen",
            "browser_unknown",
            "browser_windows_store",
            "buffer_bool",
            "buffer_f16",
            "buffer_f32",
            "buffer_f64",
            "buffer_fast",
            "buffer_fixed",
            "buffer_grow",
            "buffer_s16",
            "buffer_s32",
            "buffer_s8",
            "buffer_seek_end",
            "buffer_seek_relative",
            "buffer_seek_start",
            "buffer_string",
            "buffer_text",
            "buffer_u16",
            "buffer_u32",
            "buffer_u64",
            "buffer_u8",
            "buffer_vbuffer",
            "buffer_wrap",
            "c_aqua",
            "c_black",
            "c_blue",
            "c_dkgray",
            "c_dkgrey",
            "c_fuchsia",
            "c_gray",
            "c_green",
            "c_grey",
            "c_lime",
            "c_ltgray",
            "c_ltgrey",
            "c_maroon",
            "c_navy",
            "c_olive",
            "c_orange",
            "c_purple",
            "c_red",
            "c_silver",
            "c_teal",
            "c_white",
            "c_yellow",
            "cache_directory",
            "characterSpacing",
            "cmpfunc_always",
            "cmpfunc_equal",
            "cmpfunc_greater",
            "cmpfunc_greaterequal",
            "cmpfunc_less",
            "cmpfunc_lessequal",
            "cmpfunc_never",
            "cmpfunc_notequal",
            "coreColor",
            "coreColour",
            "cr_appstart",
            "cr_arrow",
            "cr_beam",
            "cr_cross",
            "cr_default",
            "cr_drag",
            "cr_handpoint",
            "cr_hourglass",
            "cr_none",
            "cr_size_all",
            "cr_size_nesw",
            "cr_size_ns",
            "cr_size_nwse",
            "cr_size_we",
            "cr_uparrow",
            "cull_clockwise",
            "cull_counterclockwise",
            "cull_noculling",
            "device_emulator",
            "device_ios_ipad",
            "device_ios_ipad_retina",
            "device_ios_iphone",
            "device_ios_iphone5",
            "device_ios_iphone6",
            "device_ios_iphone6plus",
            "device_ios_iphone_retina",
            "device_ios_unknown",
            "device_tablet",
            "display_landscape",
            "display_landscape_flipped",
            "display_portrait",
            "display_portrait_flipped",
            "dll_cdecl",
            "dll_stdcall",
            "dropShadowEnabled",
            "dropShadowEnabled",
            "ds_type_grid",
            "ds_type_list",
            "ds_type_map",
            "ds_type_priority",
            "ds_type_queue",
            "ds_type_stack",
            "ef_cloud",
            "ef_ellipse",
            "ef_explosion",
            "ef_firework",
            "ef_flare",
            "ef_rain",
            "ef_ring",
            "ef_smoke",
            "ef_smokeup",
            "ef_snow",
            "ef_spark",
            "ef_star",
            "effectsEnabled",
            "effectsEnabled",
            "ev_alarm",
            "ev_animation_end",
            "ev_animation_event",
            "ev_animation_update",
            "ev_async_audio_playback",
            "ev_async_audio_playback_ended",
            "ev_async_audio_recording",
            "ev_async_dialog",
            "ev_async_push_notification",
            "ev_async_save_load",
            "ev_async_save_load",
            "ev_async_social",
            "ev_async_system_event",
            "ev_async_web",
            "ev_async_web_cloud",
            "ev_async_web_iap",
            "ev_async_web_image_load",
            "ev_async_web_networking",
            "ev_async_web_steam",
            "ev_audio_playback",
            "ev_audio_playback_ended",
            "ev_audio_recording",
            "ev_boundary",
            "ev_boundary_view0",
            "ev_boundary_view1",
            "ev_boundary_view2",
            "ev_boundary_view3",
            "ev_boundary_view4",
            "ev_boundary_view5",
            "ev_boundary_view6",
            "ev_boundary_view7",
            "ev_broadcast_message",
            "ev_cleanup",
            "ev_collision",
            "ev_create",
            "ev_destroy",
            "ev_dialog_async",
            "ev_draw",
            "ev_draw_begin",
            "ev_draw_end",
            "ev_draw_normal",
            "ev_draw_post",
            "ev_draw_pre",
            "ev_end_of_path",
            "ev_game_end",
            "ev_game_start",
            "ev_gesture",
            "ev_gesture_double_tap",
            "ev_gesture_drag_end",
            "ev_gesture_drag_start",
            "ev_gesture_dragging",
            "ev_gesture_flick",
            "ev_gesture_pinch_end",
            "ev_gesture_pinch_in",
            "ev_gesture_pinch_out",
            "ev_gesture_pinch_start",
            "ev_gesture_rotate_end",
            "ev_gesture_rotate_start",
            "ev_gesture_rotating",
            "ev_gesture_tap",
            "ev_global_gesture_double_tap",
            "ev_global_gesture_drag_end",
            "ev_global_gesture_drag_start",
            "ev_global_gesture_dragging",
            "ev_global_gesture_flick",
            "ev_global_gesture_pinch_end",
            "ev_global_gesture_pinch_in",
            "ev_global_gesture_pinch_out",
            "ev_global_gesture_pinch_start",
            "ev_global_gesture_rotate_end",
            "ev_global_gesture_rotate_start",
            "ev_global_gesture_rotating",
            "ev_global_gesture_tap",
            "ev_global_left_button",
            "ev_global_left_press",
            "ev_global_left_release",
            "ev_global_middle_button",
            "ev_global_middle_press",
            "ev_global_middle_release",
            "ev_global_right_button",
            "ev_global_right_press",
            "ev_global_right_release",
            "ev_gui",
            "ev_gui_begin",
            "ev_gui_end",
            "ev_joystick1_button1",
            "ev_joystick1_button2",
            "ev_joystick1_button3",
            "ev_joystick1_button4",
            "ev_joystick1_button5",
            "ev_joystick1_button6",
            "ev_joystick1_button7",
            "ev_joystick1_button8",
            "ev_joystick1_down",
            "ev_joystick1_left",
            "ev_joystick1_right",
            "ev_joystick1_up",
            "ev_joystick2_button1",
            "ev_joystick2_button2",
            "ev_joystick2_button3",
            "ev_joystick2_button4",
            "ev_joystick2_button5",
            "ev_joystick2_button6",
            "ev_joystick2_button7",
            "ev_joystick2_button8",
            "ev_joystick2_down",
            "ev_joystick2_left",
            "ev_joystick2_right",
            "ev_joystick2_up",
            "ev_keyboard",
            "ev_keypress",
            "ev_keyrelease",
            "ev_left_button",
            "ev_left_press",
            "ev_left_release",
            "ev_middle_button",
            "ev_middle_press",
            "ev_middle_release",
            "ev_mouse",
            "ev_mouse_enter",
            "ev_mouse_leave",
            "ev_mouse_wheel_down",
            "ev_mouse_wheel_up",
            "ev_no_button",
            "ev_no_more_health",
            "ev_no_more_lives",
            "ev_other",
            "ev_outside",
            "ev_outside_view0",
            "ev_outside_view1",
            "ev_outside_view2",
            "ev_outside_view3",
            "ev_outside_view4",
            "ev_outside_view5",
            "ev_outside_view6",
            "ev_outside_view7",
            "ev_pre_create",
            "ev_push_notification",
            "ev_right_button",
            "ev_right_press",
            "ev_right_release",
            "ev_room_end",
            "ev_room_start",
            "ev_social",
            "ev_step",
            "ev_step_begin",
            "ev_step_end",
            "ev_step_normal",
            "ev_system_event",
            "ev_trigger",
            "ev_user0",
            "ev_user1",
            "ev_user10",
            "ev_user11",
            "ev_user12",
            "ev_user13",
            "ev_user14",
            "ev_user15",
            "ev_user2",
            "ev_user3",
            "ev_user4",
            "ev_user5",
            "ev_user6",
            "ev_user7",
            "ev_user8",
            "ev_user9",
            "ev_web_async",
            "ev_web_cloud",
            "ev_web_iap",
            "ev_web_image_load",
            "ev_web_networking",
            "ev_web_sound_load",
            "ev_web_steam",
            "fa_archive",
            "fa_bottom",
            "fa_center",
            "fa_directory",
            "fa_hidden",
            "fa_left",
            "fa_middle",
            "fa_none",
            "fa_readonly",
            "fa_right",
            "fa_sysfile",
            "fa_top",
            "fa_volumeid",
            "false",
            "frameSizeX",
            "frameSizeY",
            "gamespeed_fps",
            "gamespeed_microseconds",
            "global",
            "glowColor",
            "glowColour",
            "glowEnabled",
            "glowEnabled",
            "glowEnd",
            "glowStart",
            "gp_axis_acceleration_x",
            "gp_axis_acceleration_y",
            "gp_axis_acceleration_z",
            "gp_axis_angular_velocity_x",
            "gp_axis_angular_velocity_y",
            "gp_axis_angular_velocity_z",
            "gp_axis_orientation_w",
            "gp_axis_orientation_x",
            "gp_axis_orientation_y",
            "gp_axis_orientation_z",
            "gp_axislh",
            "gp_axislv",
            "gp_axisrh",
            "gp_axisrv",
            "gp_face1",
            "gp_face2",
            "gp_face3",
            "gp_face4",
            "gp_padd",
            "gp_padl",
            "gp_padr",
            "gp_padu",
            "gp_select",
            "gp_shoulderl",
            "gp_shoulderlb",
            "gp_shoulderr",
            "gp_shoulderrb",
            "gp_start",
            "gp_stickl",
            "gp_stickr",
            "iap_available",
            "iap_canceled",
            "iap_ev_consume",
            "iap_ev_product",
            "iap_ev_purchase",
            "iap_ev_restore",
            "iap_ev_storeload",
            "iap_failed",
            "iap_purchased",
            "iap_refunded",
            "iap_status_available",
            "iap_status_loading",
            "iap_status_processing",
            "iap_status_restoring",
            "iap_status_unavailable",
            "iap_status_uninitialised",
            "iap_storeload_failed",
            "iap_storeload_ok",
            "iap_unavailable",
            "infinity",
            "kbv_autocapitalize_characters",
            "kbv_autocapitalize_none",
            "kbv_autocapitalize_sentences",
            "kbv_autocapitalize_words",
            "kbv_returnkey_continue",
            "kbv_returnkey_default",
            "kbv_returnkey_done",
            "kbv_returnkey_emergency",
            "kbv_returnkey_go",
            "kbv_returnkey_google",
            "kbv_returnkey_join",
            "kbv_returnkey_next",
            "kbv_returnkey_route",
            "kbv_returnkey_search",
            "kbv_returnkey_send",
            "kbv_returnkey_yahoo",
            "kbv_type_ascii",
            "kbv_type_default",
            "kbv_type_email",
            "kbv_type_numbers",
            "kbv_type_phone",
            "kbv_type_phone_name",
            "kbv_type_url",
            "layerelementtype_background",
            "layerelementtype_instance",
            "layerelementtype_oldtilemap",
            "layerelementtype_particlesystem",
            "layerelementtype_sequence",
            "layerelementtype_sprite",
            "layerelementtype_tile",
            "layerelementtype_tilemap",
            "layerelementtype_undefined",
            "leaderboard_type_number",
            "leaderboard_type_time_mins_secs",
            "lighttype_dir",
            "lighttype_point",
            "lineSpacing",
            "m_axisx",
            "m_axisx_gui",
            "m_axisy",
            "m_axisy_gui",
            "m_scroll_down",
            "m_scroll_up",
            "matrix_projection",
            "matrix_view",
            "matrix_world",
            "mb_any",
            "mb_left",
            "mb_middle",
            "mb_none",
            "mb_right",
            "mb_side1",
            "mb_side2",
            "mip_markedonly",
            "mip_off",
            "mip_on",
            "network_config_avoid_time_wait",
            "network_config_connect_timeout",
            "network_config_disable_multicast",
            "network_config_disable_reliable_udp",
            "network_config_enable_multicast",
            "network_config_enable_reliable_udp",
            "network_config_use_non_blocking_socket",
            "network_config_websocket_protocol",
            "network_connect_active",
            "network_connect_blocking",
            "network_connect_nonblocking",
            "network_connect_none",
            "network_connect_passive",
            "network_send_binary",
            "network_send_text",
            "network_socket_bluetooth",
            "network_socket_tcp",
            "network_socket_udp",
            "network_socket_ws",
            "network_socket_wss",
            "network_type_connect",
            "network_type_data",
            "network_type_disconnect",
            "network_type_down",
            "network_type_non_blocking_connect",
            "network_type_up",
            "network_type_up_failed",
            "nineslice_blank",
            "nineslice_bottom",
            "nineslice_center",
            "nineslice_centre",
            "nineslice_hide",
            "nineslice_left",
            "nineslice_mirror",
            "nineslice_repeat",
            "nineslice_right",
            "nineslice_stretch",
            "nineslice_top",
            "noone",
            "of_challenge_lose",
            "of_challenge_tie",
            "of_challenge_win",
            "os_android",
            "os_gdk",
            "os_gxgames",
            "os_ios",
            "os_linux",
            "os_macosx",
            "os_operagx",
            "os_permission_denied",
            "os_permission_denied_dont_request",
            "os_permission_granted",
            "os_ps3",
            "os_ps4",
            "os_ps5",
            "os_psvita",
            "os_switch",
            "os_tvos",
            "os_unknown",
            "os_uwp",
            "os_win8native",
            "os_windows",
            "os_winphone",
            "os_xboxone",
            "os_xboxseriesxs",
            "other",
            "outlineColor",
            "outlineColour",
            "outlineDist",
            "outlineEnabled",
            "outlineEnabled",
            "paragraphSpacing",
            "path_action_continue",
            "path_action_restart",
            "path_action_reverse",
            "path_action_stop",
            "phy_debug_render_aabb",
            "phy_debug_render_collision_pairs",
            "phy_debug_render_coms",
            "phy_debug_render_core_shapes",
            "phy_debug_render_joints",
            "phy_debug_render_obb",
            "phy_debug_render_shapes",
            "phy_joint_anchor_1_x",
            "phy_joint_anchor_1_y",
            "phy_joint_anchor_2_x",
            "phy_joint_anchor_2_y",
            "phy_joint_angle",
            "phy_joint_angle_limits",
            "phy_joint_damping_ratio",
            "phy_joint_frequency",
            "phy_joint_length_1",
            "phy_joint_length_2",
            "phy_joint_lower_angle_limit",
            "phy_joint_max_force",
            "phy_joint_max_length",
            "phy_joint_max_motor_force",
            "phy_joint_max_motor_torque",
            "phy_joint_max_torque",
            "phy_joint_motor_force",
            "phy_joint_motor_speed",
            "phy_joint_motor_torque",
            "phy_joint_reaction_force_x",
            "phy_joint_reaction_force_y",
            "phy_joint_reaction_torque",
            "phy_joint_speed",
            "phy_joint_translation",
            "phy_joint_upper_angle_limit",
            "phy_particle_data_flag_category",
            "phy_particle_data_flag_color",
            "phy_particle_data_flag_colour",
            "phy_particle_data_flag_position",
            "phy_particle_data_flag_typeflags",
            "phy_particle_data_flag_velocity",
            "phy_particle_flag_colormixing",
            "phy_particle_flag_colourmixing",
            "phy_particle_flag_elastic",
            "phy_particle_flag_powder",
            "phy_particle_flag_spring",
            "phy_particle_flag_tensile",
            "phy_particle_flag_viscous",
            "phy_particle_flag_wall",
            "phy_particle_flag_water",
            "phy_particle_flag_zombie",
            "phy_particle_group_flag_rigid",
            "phy_particle_group_flag_solid",
            "pi",
            "pointer_invalid",
            "pointer_null",
            "pr_linelist",
            "pr_linestrip",
            "pr_pointlist",
            "pr_trianglefan",
            "pr_trianglelist",
            "pr_trianglestrip",
            "ps_distr_gaussian",
            "ps_distr_invgaussian",
            "ps_distr_linear",
            "ps_mode_burst",
            "ps_mode_stream",
            "ps_shape_diamond",
            "ps_shape_ellipse",
            "ps_shape_line",
            "ps_shape_rectangle",
            "pt_shape_circle",
            "pt_shape_cloud",
            "pt_shape_disk",
            "pt_shape_explosion",
            "pt_shape_flare",
            "pt_shape_line",
            "pt_shape_pixel",
            "pt_shape_ring",
            "pt_shape_smoke",
            "pt_shape_snow",
            "pt_shape_spark",
            "pt_shape_sphere",
            "pt_shape_square",
            "pt_shape_star",
            "rollback_chat_message",
            "rollback_connect_error",
            "rollback_connect_info",
            "rollback_connected_to_peer",
            "rollback_connection_rejected",
            "rollback_disconnected_from_peer",
            "rollback_end_game",
            "rollback_game_full",
            "rollback_game_info",
            "rollback_game_interrupted",
            "rollback_game_resumed",
            "rollback_high_latency",
            "rollback_player_prefs",
            "rollback_protocol_rejected",
            "rollback_synchronized_with_peer",
            "rollback_synchronizing_with_peer",
            "self",
            "seqaudiokey_loop",
            "seqaudiokey_oneshot",
            "seqdir_left",
            "seqdir_right",
            "seqinterpolation_assign",
            "seqinterpolation_lerp",
            "seqplay_loop",
            "seqplay_oneshot",
            "seqplay_pingpong",
            "seqtextkey_bottom",
            "seqtextkey_center",
            "seqtextkey_justify",
            "seqtextkey_left",
            "seqtextkey_middle",
            "seqtextkey_right",
            "seqtextkey_top",
            "seqtracktype_audio",
            "seqtracktype_bool",
            "seqtracktype_clipmask",
            "seqtracktype_clipmask_mask",
            "seqtracktype_clipmask_subject",
            "seqtracktype_color",
            "seqtracktype_colour",
            "seqtracktype_empty",
            "seqtracktype_graphic",
            "seqtracktype_group",
            "seqtracktype_instance",
            "seqtracktype_message",
            "seqtracktype_moment",
            "seqtracktype_particlesystem",
            "seqtracktype_real",
            "seqtracktype_sequence",
            "seqtracktype_spriteframes",
            "seqtracktype_string",
            "seqtracktype_text",
            "shadowColor",
            "shadowColour",
            "shadowOffsetX",
            "shadowOffsetY",
            "shadowSoftness",
            "sprite_add_ext_error_cancelled",
            "sprite_add_ext_error_decompressfailed",
            "sprite_add_ext_error_loadfailed",
            "sprite_add_ext_error_setupfailed",
            "sprite_add_ext_error_spritenotfound",
            "sprite_add_ext_error_unknown",
            "spritespeed_framespergameframe",
            "spritespeed_framespersecond",
            "surface_r16float",
            "surface_r32float",
            "surface_r8unorm",
            "surface_rg8unorm",
            "surface_rgba16float",
            "surface_rgba32float",
            "surface_rgba4unorm",
            "surface_rgba8unorm",
            "texturegroup_status_fetched",
            "texturegroup_status_loaded",
            "texturegroup_status_loading",
            "texturegroup_status_unloaded",
            "tf_anisotropic",
            "tf_linear",
            "tf_point",
            "thickness",
            "tile_flip",
            "tile_index_mask",
            "tile_mirror",
            "tile_rotate",
            "time_source_expire_after",
            "time_source_expire_nearest",
            "time_source_game",
            "time_source_global",
            "time_source_state_active",
            "time_source_state_initial",
            "time_source_state_paused",
            "time_source_state_stopped",
            "time_source_units_frames",
            "time_source_units_seconds",
            "timezone_local",
            "timezone_utc",
            "tm_countvsyncs",
            "tm_sleep",
            "tm_systemtiming",
            "true",
            "ty_real",
            "ty_string",
            "undefined",
            "vertex_type_color",
            "vertex_type_colour",
            "vertex_type_float1",
            "vertex_type_float2",
            "vertex_type_float3",
            "vertex_type_float4",
            "vertex_type_ubyte4",
            "vertex_usage_binormal",
            "vertex_usage_blendindices",
            "vertex_usage_blendweight",
            "vertex_usage_color",
            "vertex_usage_colour",
            "vertex_usage_depth",
            "vertex_usage_fog",
            "vertex_usage_normal",
            "vertex_usage_position",
            "vertex_usage_psize",
            "vertex_usage_sample",
            "vertex_usage_tangent",
            "vertex_usage_texcoord",
            "video_format_rgba",
            "video_format_yuv",
            "video_status_closed",
            "video_status_paused",
            "video_status_playing",
            "video_status_preparing",
            "vk_add",
            "vk_alt",
            "vk_anykey",
            "vk_backspace",
            "vk_control",
            "vk_decimal",
            "vk_delete",
            "vk_divide",
            "vk_down",
            "vk_end",
            "vk_enter",
            "vk_escape",
            "vk_f1",
            "vk_f10",
            "vk_f11",
            "vk_f12",
            "vk_f2",
            "vk_f3",
            "vk_f4",
            "vk_f5",
            "vk_f6",
            "vk_f7",
            "vk_f8",
            "vk_f9",
            "vk_home",
            "vk_insert",
            "vk_lalt",
            "vk_lcontrol",
            "vk_left",
            "vk_lshift",
            "vk_multiply",
            "vk_nokey",
            "vk_numpad0",
            "vk_numpad1",
            "vk_numpad2",
            "vk_numpad3",
            "vk_numpad4",
            "vk_numpad5",
            "vk_numpad6",
            "vk_numpad7",
            "vk_numpad8",
            "vk_numpad9",
            "vk_pagedown",
            "vk_pageup",
            "vk_pause",
            "vk_printscreen",
            "vk_ralt",
            "vk_rcontrol",
            "vk_return",
            "vk_right",
            "vk_rshift",
            "vk_shift",
            "vk_space",
            "vk_subtract",
            "vk_tab",
            "vk_up",
            "wallpaper_config",
            "wallpaper_subscription_data",
            "wrap"
          ],
          "variable.language": [
            "alarm",
            "application_surface",
            "argument",
            "argument0",
            "argument1",
            "argument2",
            "argument3",
            "argument4",
            "argument5",
            "argument6",
            "argument7",
            "argument8",
            "argument9",
            "argument10",
            "argument11",
            "argument12",
            "argument13",
            "argument14",
            "argument15",
            "argument_count",
            "async_load",
            "background_color",
            "background_colour",
            "background_showcolor",
            "background_showcolour",
            "bbox_bottom",
            "bbox_left",
            "bbox_right",
            "bbox_top",
            "browser_height",
            "browser_width",
            "colour?ColourTrack",
            "current_day",
            "current_hour",
            "current_minute",
            "current_month",
            "current_second",
            "current_time",
            "current_weekday",
            "current_year",
            "cursor_sprite",
            "debug_mode",
            "delta_time",
            "depth",
            "direction",
            "display_aa",
            "drawn_by_sequence",
            "event_action",
            "event_data",
            "event_number",
            "event_object",
            "event_type",
            "font_texture_page_size",
            "fps",
            "fps_real",
            "friction",
            "game_display_name",
            "game_id",
            "game_project_name",
            "game_save_id",
            "gravity",
            "gravity_direction",
            "health",
            "hspeed",
            "iap_data",
            "id",
            "image_alpha",
            "image_angle",
            "image_blend",
            "image_index",
            "image_number",
            "image_speed",
            "image_xscale",
            "image_yscale",
            "in_collision_tree",
            "in_sequence",
            "instance_count",
            "instance_id",
            "keyboard_key",
            "keyboard_lastchar",
            "keyboard_lastkey",
            "keyboard_string",
            "layer",
            "lives",
            "longMessage",
            "managed",
            "mask_index",
            "message",
            "mouse_button",
            "mouse_lastbutton",
            "mouse_x",
            "mouse_y",
            "object_index",
            "os_browser",
            "os_device",
            "os_type",
            "os_version",
            "path_endaction",
            "path_index",
            "path_orientation",
            "path_position",
            "path_positionprevious",
            "path_scale",
            "path_speed",
            "persistent",
            "phy_active",
            "phy_angular_damping",
            "phy_angular_velocity",
            "phy_bullet",
            "phy_col_normal_x",
            "phy_col_normal_y",
            "phy_collision_points",
            "phy_collision_x",
            "phy_collision_y",
            "phy_com_x",
            "phy_com_y",
            "phy_dynamic",
            "phy_fixed_rotation",
            "phy_inertia",
            "phy_kinematic",
            "phy_linear_damping",
            "phy_linear_velocity_x",
            "phy_linear_velocity_y",
            "phy_mass",
            "phy_position_x",
            "phy_position_xprevious",
            "phy_position_y",
            "phy_position_yprevious",
            "phy_rotation",
            "phy_sleeping",
            "phy_speed",
            "phy_speed_x",
            "phy_speed_y",
            "player_avatar_sprite",
            "player_avatar_url",
            "player_id",
            "player_local",
            "player_type",
            "player_user_id",
            "program_directory",
            "rollback_api_server",
            "rollback_confirmed_frame",
            "rollback_current_frame",
            "rollback_event_id",
            "rollback_event_param",
            "rollback_game_running",
            "room",
            "room_first",
            "room_height",
            "room_last",
            "room_persistent",
            "room_speed",
            "room_width",
            "score",
            "script",
            "sequence_instance",
            "solid",
            "speed",
            "sprite_height",
            "sprite_index",
            "sprite_width",
            "sprite_xoffset",
            "sprite_yoffset",
            "stacktrace",
            "temp_directory",
            "timeline_index",
            "timeline_loop",
            "timeline_position",
            "timeline_running",
            "timeline_speed",
            "view_camera",
            "view_current",
            "view_enabled",
            "view_hport",
            "view_surface_id",
            "view_visible",
            "view_wport",
            "view_xport",
            "view_yport",
            "visible",
            "vspeed",
            "webgl_enabled",
            "working_directory",
            "x",
            "xprevious",
            "xstart",
            "y",
            "yprevious",
            "ystart"
          ]
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE
        ]
      };
    }
    return gml_1 = O, gml_1;
  }
  var golo_1, hasRequiredGolo;
  function requireGolo() {
    if (hasRequiredGolo) return golo_1;
    hasRequiredGolo = 1;
    function O(_) {
      return {
        name: "Golo",
        keywords: {
          keyword: [
            "println",
            "readln",
            "print",
            "import",
            "module",
            "function",
            "local",
            "return",
            "let",
            "var",
            "while",
            "for",
            "foreach",
            "times",
            "in",
            "case",
            "when",
            "match",
            "with",
            "break",
            "continue",
            "augment",
            "augmentation",
            "each",
            "find",
            "filter",
            "reduce",
            "if",
            "then",
            "else",
            "otherwise",
            "try",
            "catch",
            "finally",
            "raise",
            "throw",
            "orIfNull",
            "DynamicObject|10",
            "DynamicVariable",
            "struct",
            "Observable",
            "map",
            "set",
            "vector",
            "list",
            "array"
          ],
          literal: [
            "true",
            "false",
            "null"
          ]
        },
        contains: [
          _.HASH_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE,
          {
            className: "meta",
            begin: "@[A-Za-z]+"
          }
        ]
      };
    }
    return golo_1 = O, golo_1;
  }
  var gradle_1, hasRequiredGradle;
  function requireGradle() {
    if (hasRequiredGradle) return gradle_1;
    hasRequiredGradle = 1;
    function O(_) {
      return {
        name: "Gradle",
        case_insensitive: !0,
        keywords: [
          "task",
          "project",
          "allprojects",
          "subprojects",
          "artifacts",
          "buildscript",
          "configurations",
          "dependencies",
          "repositories",
          "sourceSets",
          "description",
          "delete",
          "from",
          "into",
          "include",
          "exclude",
          "source",
          "classpath",
          "destinationDir",
          "includes",
          "options",
          "sourceCompatibility",
          "targetCompatibility",
          "group",
          "flatDir",
          "doLast",
          "doFirst",
          "flatten",
          "todir",
          "fromdir",
          "ant",
          "def",
          "abstract",
          "break",
          "case",
          "catch",
          "continue",
          "default",
          "do",
          "else",
          "extends",
          "final",
          "finally",
          "for",
          "if",
          "implements",
          "instanceof",
          "native",
          "new",
          "private",
          "protected",
          "public",
          "return",
          "static",
          "switch",
          "synchronized",
          "throw",
          "throws",
          "transient",
          "try",
          "volatile",
          "while",
          "strictfp",
          "package",
          "import",
          "false",
          "null",
          "super",
          "this",
          "true",
          "antlrtask",
          "checkstyle",
          "codenarc",
          "copy",
          "boolean",
          "byte",
          "char",
          "class",
          "double",
          "float",
          "int",
          "interface",
          "long",
          "short",
          "void",
          "compile",
          "runTime",
          "file",
          "fileTree",
          "abs",
          "any",
          "append",
          "asList",
          "asWritable",
          "call",
          "collect",
          "compareTo",
          "count",
          "div",
          "dump",
          "each",
          "eachByte",
          "eachFile",
          "eachLine",
          "every",
          "find",
          "findAll",
          "flatten",
          "getAt",
          "getErr",
          "getIn",
          "getOut",
          "getText",
          "grep",
          "immutable",
          "inject",
          "inspect",
          "intersect",
          "invokeMethods",
          "isCase",
          "join",
          "leftShift",
          "minus",
          "multiply",
          "newInputStream",
          "newOutputStream",
          "newPrintWriter",
          "newReader",
          "newWriter",
          "next",
          "plus",
          "pop",
          "power",
          "previous",
          "print",
          "println",
          "push",
          "putAt",
          "read",
          "readBytes",
          "readLines",
          "reverse",
          "reverseEach",
          "round",
          "size",
          "sort",
          "splitEachLine",
          "step",
          "subMap",
          "times",
          "toInteger",
          "toList",
          "tokenize",
          "upto",
          "waitForOrKill",
          "withPrintWriter",
          "withReader",
          "withStream",
          "withWriter",
          "withWriterAppend",
          "write",
          "writeLine"
        ],
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE,
          _.REGEXP_MODE
        ]
      };
    }
    return gradle_1 = O, gradle_1;
  }
  var groovy_1, hasRequiredGroovy;
  function requireGroovy() {
    if (hasRequiredGroovy) return groovy_1;
    hasRequiredGroovy = 1;
    function O(H, V = {}) {
      return V.variants = H, V;
    }
    function _(H) {
      const V = H.regex, q = "[A-Za-z0-9_$]+", G = O([
        H.C_LINE_COMMENT_MODE,
        H.C_BLOCK_COMMENT_MODE,
        H.COMMENT(
          "/\\*\\*",
          "\\*/",
          {
            relevance: 0,
            contains: [
              {
                // eat up @'s in emails to prevent them to be recognized as doctags
                begin: /\w+@/,
                relevance: 0
              },
              {
                className: "doctag",
                begin: "@[A-Za-z]+"
              }
            ]
          }
        )
      ]), J = {
        className: "regexp",
        begin: /~?\/[^\/\n]+\//,
        contains: [H.BACKSLASH_ESCAPE]
      }, Z = O([
        H.BINARY_NUMBER_MODE,
        H.C_NUMBER_MODE
      ]), te = O(
        [
          {
            begin: /"""/,
            end: /"""/
          },
          {
            begin: /'''/,
            end: /'''/
          },
          {
            begin: "\\$/",
            end: "/\\$",
            relevance: 10
          },
          H.APOS_STRING_MODE,
          H.QUOTE_STRING_MODE
        ],
        { className: "string" }
      ), ne = {
        match: [
          /(class|interface|trait|enum|record|extends|implements)/,
          /\s+/,
          H.UNDERSCORE_IDENT_RE
        ],
        scope: {
          1: "keyword",
          3: "title.class"
        }
      };
      return {
        name: "Groovy",
        keywords: {
          "variable.language": "this super",
          literal: "true false null",
          type: [
            "byte",
            "short",
            "char",
            "int",
            "long",
            "boolean",
            "float",
            "double",
            "void"
          ],
          keyword: [
            // groovy specific keywords
            "def",
            "as",
            "in",
            "assert",
            "trait",
            // common keywords with Java
            "abstract",
            "static",
            "volatile",
            "transient",
            "public",
            "private",
            "protected",
            "synchronized",
            "final",
            "class",
            "interface",
            "enum",
            "if",
            "else",
            "for",
            "while",
            "switch",
            "case",
            "break",
            "default",
            "continue",
            "throw",
            "throws",
            "try",
            "catch",
            "finally",
            "implements",
            "extends",
            "new",
            "import",
            "package",
            "return",
            "instanceof",
            "var"
          ]
        },
        contains: [
          H.SHEBANG({
            binary: "groovy",
            relevance: 10
          }),
          G,
          te,
          J,
          Z,
          ne,
          {
            className: "meta",
            begin: "@[A-Za-z]+",
            relevance: 0
          },
          {
            // highlight map keys and named parameters as attrs
            className: "attr",
            begin: q + "[ 	]*:",
            relevance: 0
          },
          {
            // catch middle element of the ternary operator
            // to avoid highlight it as a label, named parameter, or map key
            begin: /\?/,
            end: /:/,
            relevance: 0,
            contains: [
              G,
              te,
              J,
              Z,
              "self"
            ]
          },
          {
            // highlight labeled statements
            className: "symbol",
            begin: "^[ 	]*" + V.lookahead(q + ":"),
            excludeBegin: !0,
            end: q + ":",
            relevance: 0
          }
        ],
        illegal: /#|<\//
      };
    }
    return groovy_1 = _, groovy_1;
  }
  var haml_1, hasRequiredHaml;
  function requireHaml() {
    if (hasRequiredHaml) return haml_1;
    hasRequiredHaml = 1;
    function O(_) {
      return {
        name: "HAML",
        case_insensitive: !0,
        contains: [
          {
            className: "meta",
            begin: "^!!!( (5|1\\.1|Strict|Frameset|Basic|Mobile|RDFa|XML\\b.*))?$",
            relevance: 10
          },
          // FIXME these comments should be allowed to span indented lines
          _.COMMENT(
            "^\\s*(!=#|=#|-#|/).*$",
            null,
            { relevance: 0 }
          ),
          {
            begin: "^\\s*(-|=|!=)(?!#)",
            end: /$/,
            subLanguage: "ruby",
            excludeBegin: !0,
            excludeEnd: !0
          },
          {
            className: "tag",
            begin: "^\\s*%",
            contains: [
              {
                className: "selector-tag",
                begin: "\\w+"
              },
              {
                className: "selector-id",
                begin: "#[\\w-]+"
              },
              {
                className: "selector-class",
                begin: "\\.[\\w-]+"
              },
              {
                begin: /\{\s*/,
                end: /\s*\}/,
                contains: [
                  {
                    begin: ":\\w+\\s*=>",
                    end: ",\\s+",
                    returnBegin: !0,
                    endsWithParent: !0,
                    contains: [
                      {
                        className: "attr",
                        begin: ":\\w+"
                      },
                      _.APOS_STRING_MODE,
                      _.QUOTE_STRING_MODE,
                      {
                        begin: "\\w+",
                        relevance: 0
                      }
                    ]
                  }
                ]
              },
              {
                begin: "\\(\\s*",
                end: "\\s*\\)",
                excludeEnd: !0,
                contains: [
                  {
                    begin: "\\w+\\s*=",
                    end: "\\s+",
                    returnBegin: !0,
                    endsWithParent: !0,
                    contains: [
                      {
                        className: "attr",
                        begin: "\\w+",
                        relevance: 0
                      },
                      _.APOS_STRING_MODE,
                      _.QUOTE_STRING_MODE,
                      {
                        begin: "\\w+",
                        relevance: 0
                      }
                    ]
                  }
                ]
              }
            ]
          },
          { begin: "^\\s*[=~]\\s*" },
          {
            begin: /#\{/,
            end: /\}/,
            subLanguage: "ruby",
            excludeBegin: !0,
            excludeEnd: !0
          }
        ]
      };
    }
    return haml_1 = O, haml_1;
  }
  var handlebars_1, hasRequiredHandlebars;
  function requireHandlebars() {
    if (hasRequiredHandlebars) return handlebars_1;
    hasRequiredHandlebars = 1;
    function O(_) {
      const H = _.regex, V = {
        $pattern: /[\w.\/]+/,
        built_in: [
          "action",
          "bindattr",
          "collection",
          "component",
          "concat",
          "debugger",
          "each",
          "each-in",
          "get",
          "hash",
          "if",
          "in",
          "input",
          "link-to",
          "loc",
          "log",
          "lookup",
          "mut",
          "outlet",
          "partial",
          "query-params",
          "render",
          "template",
          "textarea",
          "unbound",
          "unless",
          "view",
          "with",
          "yield"
        ]
      }, q = {
        $pattern: /[\w.\/]+/,
        literal: [
          "true",
          "false",
          "undefined",
          "null"
        ]
      }, G = /""|"[^"]+"/, J = /''|'[^']+'/, Z = /\[\]|\[[^\]]+\]/, te = /[^\s!"#%&'()*+,.\/;<=>@\[\\\]^`{|}~]+/, ne = /(\.|\/)/, ee = H.either(
        G,
        J,
        Z,
        te
      ), ae = H.concat(
        H.optional(/\.|\.\/|\//),
        // relative or absolute path
        ee,
        H.anyNumberOfTimes(H.concat(
          ne,
          ee
        ))
      ), oe = H.concat(
        "(",
        Z,
        "|",
        te,
        ")(?==)"
      ), se = { begin: ae }, ce = _.inherit(se, { keywords: q }), le = {
        begin: /\(/,
        end: /\)/
        // the "contains" is added below when all necessary sub-modes are defined
      }, ue = {
        // fka "attribute-assignment", parameters of the form 'key=value'
        className: "attr",
        begin: oe,
        relevance: 0,
        starts: {
          begin: /=/,
          end: /=/,
          starts: { contains: [
            _.NUMBER_MODE,
            _.QUOTE_STRING_MODE,
            _.APOS_STRING_MODE,
            ce,
            le
          ] }
        }
      }, de = {
        // parameters of the form '{{#with x as | y |}}...{{/with}}'
        begin: /as\s+\|/,
        keywords: { keyword: "as" },
        end: /\|/,
        contains: [
          {
            // define sub-mode in order to prevent highlighting of block-parameter named "as"
            begin: /\w+/
          }
        ]
      }, fe = {
        contains: [
          _.NUMBER_MODE,
          _.QUOTE_STRING_MODE,
          _.APOS_STRING_MODE,
          de,
          ue,
          ce,
          le
        ],
        returnEnd: !0
        // the property "end" is defined through inheritance when the mode is used. If depends
        // on the surrounding mode, but "endsWithParent" does not work here (i.e. it includes the
        // end-token of the surrounding mode)
      }, he = _.inherit(se, {
        className: "name",
        keywords: V,
        starts: _.inherit(fe, { end: /\)/ })
      });
      le.contains = [he];
      const me = _.inherit(se, {
        keywords: V,
        className: "name",
        starts: _.inherit(fe, { end: /\}\}/ })
      }), pe = _.inherit(se, {
        keywords: V,
        className: "name"
      }), ye = _.inherit(se, {
        className: "name",
        keywords: V,
        starts: _.inherit(fe, { end: /\}\}/ })
      });
      return {
        name: "Handlebars",
        aliases: [
          "hbs",
          "html.hbs",
          "html.handlebars",
          "htmlbars"
        ],
        case_insensitive: !0,
        subLanguage: "xml",
        contains: [
          {
            begin: /\\\{\{/,
            skip: !0
          },
          {
            begin: /\\\\(?=\{\{)/,
            skip: !0
          },
          _.COMMENT(/\{\{!--/, /--\}\}/),
          _.COMMENT(/\{\{!/, /\}\}/),
          {
            // open raw block "{{{{raw}}}} content not evaluated {{{{/raw}}}}"
            className: "template-tag",
            begin: /\{\{\{\{(?!\/)/,
            end: /\}\}\}\}/,
            contains: [me],
            starts: {
              end: /\{\{\{\{\//,
              returnEnd: !0,
              subLanguage: "xml"
            }
          },
          {
            // close raw block
            className: "template-tag",
            begin: /\{\{\{\{\//,
            end: /\}\}\}\}/,
            contains: [pe]
          },
          {
            // open block statement
            className: "template-tag",
            begin: /\{\{#/,
            end: /\}\}/,
            contains: [me]
          },
          {
            className: "template-tag",
            begin: /\{\{(?=else\}\})/,
            end: /\}\}/,
            keywords: "else"
          },
          {
            className: "template-tag",
            begin: /\{\{(?=else if)/,
            end: /\}\}/,
            keywords: "else if"
          },
          {
            // closing block statement
            className: "template-tag",
            begin: /\{\{\//,
            end: /\}\}/,
            contains: [pe]
          },
          {
            // template variable or helper-call that is NOT html-escaped
            className: "template-variable",
            begin: /\{\{\{/,
            end: /\}\}\}/,
            contains: [ye]
          },
          {
            // template variable or helper-call that is html-escaped
            className: "template-variable",
            begin: /\{\{/,
            end: /\}\}/,
            contains: [ye]
          }
        ]
      };
    }
    return handlebars_1 = O, handlebars_1;
  }
  var haskell_1, hasRequiredHaskell;
  function requireHaskell() {
    if (hasRequiredHaskell) return haskell_1;
    hasRequiredHaskell = 1;
    function O(_) {
      const H = "([0-9]_*)+", V = "([0-9a-fA-F]_*)+", q = "([01]_*)+", G = "([0-7]_*)+", ne = "([!#$%&*+.\\/<=>?@\\\\^~-]|(?!([(),;\\[\\]`|{}]|[_:\"']))(\\p{S}|\\p{P}))", ee = { variants: [
        // Double dash forms a valid comment only if it's not part of legal lexeme.
        // See: Haskell 98 report: https://www.haskell.org/onlinereport/lexemes.html
        //
        // The commented code does the job, but we can't use negative lookbehind,
        // due to poor support by Safari browser.
        // > hljs.COMMENT(`(?<!${symbol})--+(?!${symbol})`, '$'),
        // So instead, we'll add a no-markup rule before the COMMENT rule in the rules list
        // to match the problematic infix operators that contain double dash.
        _.COMMENT("--+", "$"),
        _.COMMENT(
          /\{-/,
          /-\}/,
          { contains: ["self"] }
        )
      ] }, ae = {
        className: "meta",
        begin: /\{-#/,
        end: /#-\}/
      }, oe = {
        className: "meta",
        begin: "^#",
        end: "$"
      }, se = {
        className: "type",
        begin: "\\b[A-Z][\\w']*",
        // TODO: other constructors (build-in, infix).
        relevance: 0
      }, ce = {
        begin: "\\(",
        end: "\\)",
        illegal: '"',
        contains: [
          ae,
          oe,
          {
            className: "type",
            begin: "\\b[A-Z][\\w]*(\\((\\.\\.|,|\\w+)\\))?"
          },
          _.inherit(_.TITLE_MODE, { begin: "[_a-z][\\w']*" }),
          ee
        ]
      }, le = {
        begin: /\{/,
        end: /\}/,
        contains: ce.contains
      }, ue = {
        className: "number",
        relevance: 0,
        variants: [
          // decimal floating-point-literal (subsumes decimal-literal)
          { match: `\\b(${H})(\\.(${H}))?([eE][+-]?(${H}))?\\b` },
          // hexadecimal floating-point-literal (subsumes hexadecimal-literal)
          { match: `\\b0[xX]_*(${V})(\\.(${V}))?([pP][+-]?(${H}))?\\b` },
          // octal-literal
          { match: `\\b0[oO](${G})\\b` },
          // binary-literal
          { match: `\\b0[bB](${q})\\b` }
        ]
      };
      return {
        name: "Haskell",
        aliases: ["hs"],
        keywords: "let in if then else case of where do module import hiding qualified type data newtype deriving class instance as default infix infixl infixr foreign export ccall stdcall cplusplus jvm dotnet safe unsafe family forall mdo proc rec",
        unicodeRegex: !0,
        contains: [
          // Top-level constructions.
          {
            beginKeywords: "module",
            end: "where",
            keywords: "module where",
            contains: [
              ce,
              ee
            ],
            illegal: "\\W\\.|;"
          },
          {
            begin: "\\bimport\\b",
            end: "$",
            keywords: "import qualified as hiding",
            contains: [
              ce,
              ee
            ],
            illegal: "\\W\\.|;"
          },
          {
            className: "class",
            begin: "^(\\s*)?(class|instance)\\b",
            end: "where",
            keywords: "class family instance where",
            contains: [
              se,
              ce,
              ee
            ]
          },
          {
            className: "class",
            begin: "\\b(data|(new)?type)\\b",
            end: "$",
            keywords: "data family type newtype deriving",
            contains: [
              ae,
              se,
              ce,
              le,
              ee
            ]
          },
          {
            beginKeywords: "default",
            end: "$",
            contains: [
              se,
              ce,
              ee
            ]
          },
          {
            beginKeywords: "infix infixl infixr",
            end: "$",
            contains: [
              _.C_NUMBER_MODE,
              ee
            ]
          },
          {
            begin: "\\bforeign\\b",
            end: "$",
            keywords: "foreign import export ccall stdcall cplusplus jvm dotnet safe unsafe",
            contains: [
              se,
              _.QUOTE_STRING_MODE,
              ee
            ]
          },
          {
            className: "meta",
            begin: "#!\\/usr\\/bin\\/env runhaskell",
            end: "$"
          },
          // "Whitespaces".
          ae,
          oe,
          // Literals and names.
          // Single characters.
          {
            scope: "string",
            begin: /'(?=\\?.')/,
            end: /'/,
            contains: [
              {
                scope: "char.escape",
                match: /\\./
              }
            ]
          },
          _.QUOTE_STRING_MODE,
          ue,
          se,
          _.inherit(_.TITLE_MODE, { begin: "^[_a-z][\\w']*" }),
          // No markup, prevents infix operators from being recognized as comments.
          { begin: `(?!-)${ne}--+|--+(?!-)${ne}` },
          ee,
          {
            // No markup, relevance booster
            begin: "->|<-"
          }
        ]
      };
    }
    return haskell_1 = O, haskell_1;
  }
  var haxe_1, hasRequiredHaxe;
  function requireHaxe() {
    if (hasRequiredHaxe) return haxe_1;
    hasRequiredHaxe = 1;
    function O(_) {
      const H = "[a-zA-Z_$][a-zA-Z0-9_$]*", V = /(-?)(\b0[xX][a-fA-F0-9_]+|(\b\d+(\.[\d_]*)?|\.[\d_]+)(([eE][-+]?\d+)|i32|u32|i64|f64)?)/;
      return {
        name: "Haxe",
        aliases: ["hx"],
        keywords: {
          keyword: "abstract break case cast catch continue default do dynamic else enum extern final for function here if import in inline is macro never new override package private get set public return static super switch this throw trace try typedef untyped using var while " + "Int Float String Bool Dynamic Void Array ",
          built_in: "trace this",
          literal: "true false null _"
        },
        contains: [
          {
            className: "string",
            // interpolate-able strings
            begin: "'",
            end: "'",
            contains: [
              _.BACKSLASH_ESCAPE,
              {
                className: "subst",
                // interpolation
                begin: /\$\{/,
                end: /\}/
              },
              {
                className: "subst",
                // interpolation
                begin: /\$/,
                end: /\W\}/
              }
            ]
          },
          _.QUOTE_STRING_MODE,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "number",
            begin: V,
            relevance: 0
          },
          {
            className: "variable",
            begin: "\\$" + H
          },
          {
            className: "meta",
            // compiler meta
            begin: /@:?/,
            end: /\(|$/,
            excludeEnd: !0
          },
          {
            className: "meta",
            // compiler conditionals
            begin: "#",
            end: "$",
            keywords: { keyword: "if else elseif end error" }
          },
          {
            className: "type",
            // function types
            begin: /:[ \t]*/,
            end: /[^A-Za-z0-9_ \t\->]/,
            excludeBegin: !0,
            excludeEnd: !0,
            relevance: 0
          },
          {
            className: "type",
            // types
            begin: /:[ \t]*/,
            end: /\W/,
            excludeBegin: !0,
            excludeEnd: !0
          },
          {
            className: "type",
            // instantiation
            beginKeywords: "new",
            end: /\W/,
            excludeBegin: !0,
            excludeEnd: !0
          },
          {
            className: "title.class",
            // enums
            beginKeywords: "enum",
            end: /\{/,
            contains: [_.TITLE_MODE]
          },
          {
            className: "title.class",
            // abstracts
            begin: "\\babstract\\b(?=\\s*" + _.IDENT_RE + "\\s*\\()",
            end: /[\{$]/,
            contains: [
              {
                className: "type",
                begin: /\(/,
                end: /\)/,
                excludeBegin: !0,
                excludeEnd: !0
              },
              {
                className: "type",
                begin: /from +/,
                end: /\W/,
                excludeBegin: !0,
                excludeEnd: !0
              },
              {
                className: "type",
                begin: /to +/,
                end: /\W/,
                excludeBegin: !0,
                excludeEnd: !0
              },
              _.TITLE_MODE
            ],
            keywords: { keyword: "abstract from to" }
          },
          {
            className: "title.class",
            // classes
            begin: /\b(class|interface) +/,
            end: /[\{$]/,
            excludeEnd: !0,
            keywords: "class interface",
            contains: [
              {
                className: "keyword",
                begin: /\b(extends|implements) +/,
                keywords: "extends implements",
                contains: [
                  {
                    className: "type",
                    begin: _.IDENT_RE,
                    relevance: 0
                  }
                ]
              },
              _.TITLE_MODE
            ]
          },
          {
            className: "title.function",
            beginKeywords: "function",
            end: /\(/,
            excludeEnd: !0,
            illegal: /\S/,
            contains: [_.TITLE_MODE]
          }
        ],
        illegal: /<\//
      };
    }
    return haxe_1 = O, haxe_1;
  }
  var hsp_1, hasRequiredHsp;
  function requireHsp() {
    if (hasRequiredHsp) return hsp_1;
    hasRequiredHsp = 1;
    function O(_) {
      return {
        name: "HSP",
        case_insensitive: !0,
        keywords: {
          $pattern: /[\w._]+/,
          keyword: "goto gosub return break repeat loop continue wait await dim sdim foreach dimtype dup dupptr end stop newmod delmod mref run exgoto on mcall assert logmes newlab resume yield onexit onerror onkey onclick oncmd exist delete mkdir chdir dirlist bload bsave bcopy memfile if else poke wpoke lpoke getstr chdpm memexpand memcpy memset notesel noteadd notedel noteload notesave randomize noteunsel noteget split strrep setease button chgdisp exec dialog mmload mmplay mmstop mci pset pget syscolor mes print title pos circle cls font sysfont objsize picload color palcolor palette redraw width gsel gcopy gzoom gmode bmpsave hsvcolor getkey listbox chkbox combox input mesbox buffer screen bgscr mouse objsel groll line clrobj boxf objprm objmode stick grect grotate gsquare gradf objimage objskip objenable celload celdiv celput newcom querycom delcom cnvstow comres axobj winobj sendmsg comevent comevarg sarrayconv callfunc cnvwtos comevdisp libptr system hspstat hspver stat cnt err strsize looplev sublev iparam wparam lparam refstr refdval int rnd strlen length length2 length3 length4 vartype gettime peek wpeek lpeek varptr varuse noteinfo instr abs limit getease str strmid strf getpath strtrim sin cos tan atan sqrt double absf expf logf limitf powf geteasef mousex mousey mousew hwnd hinstance hdc ginfo objinfo dirinfo sysinfo thismod __hspver__ __hsp30__ __date__ __time__ __line__ __file__ _debug __hspdef__ and or xor not screen_normal screen_palette screen_hide screen_fixedsize screen_tool screen_frame gmode_gdi gmode_mem gmode_rgb0 gmode_alpha gmode_rgb0alpha gmode_add gmode_sub gmode_pixela ginfo_mx ginfo_my ginfo_act ginfo_sel ginfo_wx1 ginfo_wy1 ginfo_wx2 ginfo_wy2 ginfo_vx ginfo_vy ginfo_sizex ginfo_sizey ginfo_winx ginfo_winy ginfo_mesx ginfo_mesy ginfo_r ginfo_g ginfo_b ginfo_paluse ginfo_dispx ginfo_dispy ginfo_cx ginfo_cy ginfo_intid ginfo_newid ginfo_sx ginfo_sy objinfo_mode objinfo_bmscr objinfo_hwnd notemax notesize dir_cur dir_exe dir_win dir_sys dir_cmdline dir_desktop dir_mydoc dir_tv font_normal font_bold font_italic font_underline font_strikeout font_antialias objmode_normal objmode_guifont objmode_usefont gsquare_grad msgothic msmincho do until while wend for next _break _continue switch case default swbreak swend ddim ldim alloc m_pi rad2deg deg2rad ease_linear ease_quad_in ease_quad_out ease_quad_inout ease_cubic_in ease_cubic_out ease_cubic_inout ease_quartic_in ease_quartic_out ease_quartic_inout ease_bounce_in ease_bounce_out ease_bounce_inout ease_shake_in ease_shake_out ease_shake_inout ease_loop"
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.APOS_STRING_MODE,
          {
            // multi-line string
            className: "string",
            begin: /\{"/,
            end: /"\}/,
            contains: [_.BACKSLASH_ESCAPE]
          },
          _.COMMENT(";", "$", { relevance: 0 }),
          {
            // pre-processor
            className: "meta",
            begin: "#",
            end: "$",
            keywords: { keyword: "addion cfunc cmd cmpopt comfunc const defcfunc deffunc define else endif enum epack func global if ifdef ifndef include modcfunc modfunc modinit modterm module pack packopt regcmd runtime undef usecom uselib" },
            contains: [
              _.inherit(_.QUOTE_STRING_MODE, { className: "string" }),
              _.NUMBER_MODE,
              _.C_NUMBER_MODE,
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          {
            // label
            className: "symbol",
            begin: "^\\*(\\w+|@)"
          },
          _.NUMBER_MODE,
          _.C_NUMBER_MODE
        ]
      };
    }
    return hsp_1 = O, hsp_1;
  }
  var http_1, hasRequiredHttp;
  function requireHttp() {
    if (hasRequiredHttp) return http_1;
    hasRequiredHttp = 1;
    function O(_) {
      const H = _.regex, V = "HTTP/([32]|1\\.[01])", q = /[A-Za-z][A-Za-z0-9-]*/, G = {
        className: "attribute",
        begin: H.concat("^", q, "(?=\\:\\s)"),
        starts: { contains: [
          {
            className: "punctuation",
            begin: /: /,
            relevance: 0,
            starts: {
              end: "$",
              relevance: 0
            }
          }
        ] }
      }, J = [
        G,
        {
          begin: "\\n\\n",
          starts: {
            subLanguage: [],
            endsWithParent: !0
          }
        }
      ];
      return {
        name: "HTTP",
        aliases: ["https"],
        illegal: /\S/,
        contains: [
          // response
          {
            begin: "^(?=" + V + " \\d{3})",
            end: /$/,
            contains: [
              {
                className: "meta",
                begin: V
              },
              {
                className: "number",
                begin: "\\b\\d{3}\\b"
              }
            ],
            starts: {
              end: /\b\B/,
              illegal: /\S/,
              contains: J
            }
          },
          // request
          {
            begin: "(?=^[A-Z]+ (.*?) " + V + "$)",
            end: /$/,
            contains: [
              {
                className: "string",
                begin: " ",
                end: " ",
                excludeBegin: !0,
                excludeEnd: !0
              },
              {
                className: "meta",
                begin: V
              },
              {
                className: "keyword",
                begin: "[A-Z]+"
              }
            ],
            starts: {
              end: /\b\B/,
              illegal: /\S/,
              contains: J
            }
          },
          // to allow headers to work even without a preamble
          _.inherit(G, { relevance: 0 })
        ]
      };
    }
    return http_1 = O, http_1;
  }
  var hy_1, hasRequiredHy;
  function requireHy() {
    if (hasRequiredHy) return hy_1;
    hasRequiredHy = 1;
    function O(_) {
      const H = "a-zA-Z_\\-!.?+*=<>&#'", V = "[" + H + "][" + H + "0-9/;:]*", q = {
        $pattern: V,
        built_in: (
          // keywords
          "!= % %= & &= * ** **= *= *map + += , --build-class-- --import-- -= . / // //= /= < << <<= <= = > >= >> >>= @ @= ^ ^= abs accumulate all and any ap-compose ap-dotimes ap-each ap-each-while ap-filter ap-first ap-if ap-last ap-map ap-map-when ap-pipe ap-reduce ap-reject apply as-> ascii assert assoc bin break butlast callable calling-module-name car case cdr chain chr coll? combinations compile compress cond cons cons? continue count curry cut cycle dec def default-method defclass defmacro defmacro-alias defmacro/g! defmain defmethod defmulti defn defn-alias defnc defnr defreader defseq del delattr delete-route dict-comp dir disassemble dispatch-reader-macro distinct divmod do doto drop drop-last drop-while empty? end-sequence eval eval-and-compile eval-when-compile even? every? except exec filter first flatten float? fn fnc fnr for for* format fraction genexpr gensym get getattr global globals group-by hasattr hash hex id identity if if* if-not if-python2 import in inc input instance? integer integer-char? integer? interleave interpose is is-coll is-cons is-empty is-even is-every is-float is-instance is-integer is-integer-char is-iterable is-iterator is-keyword is-neg is-none is-not is-numeric is-odd is-pos is-string is-symbol is-zero isinstance islice issubclass iter iterable? iterate iterator? keyword keyword? lambda last len let lif lif-not list* list-comp locals loop macro-error macroexpand macroexpand-1 macroexpand-all map max merge-with method-decorator min multi-decorator multicombinations name neg? next none? nonlocal not not-in not? nth numeric? oct odd? open or ord partition permutations pos? post-route postwalk pow prewalk print product profile/calls profile/cpu put-route quasiquote quote raise range read read-str recursive-replace reduce remove repeat repeatedly repr require rest round route route-with-methods rwm second seq set-comp setattr setv some sorted string string? sum switch symbol? take take-nth take-while tee try unless unquote unquote-splicing vars walk when while with with* with-decorator with-gensyms xi xor yield yield-from zero? zip zip-longest | |= ~"
        )
      }, G = "[-+]?\\d+(\\.\\d+)?", J = {
        begin: V,
        relevance: 0
      }, Z = {
        className: "number",
        begin: G,
        relevance: 0
      }, te = _.inherit(_.QUOTE_STRING_MODE, { illegal: null }), ne = _.COMMENT(
        ";",
        "$",
        { relevance: 0 }
      ), ee = {
        className: "literal",
        begin: /\b([Tt]rue|[Ff]alse|nil|None)\b/
      }, ae = {
        begin: "[\\[\\{]",
        end: "[\\]\\}]",
        relevance: 0
      }, oe = {
        className: "comment",
        begin: "\\^" + V
      }, se = _.COMMENT("\\^\\{", "\\}"), ce = {
        className: "symbol",
        begin: "[:]{1,2}" + V
      }, le = {
        begin: "\\(",
        end: "\\)"
      }, ue = {
        endsWithParent: !0,
        relevance: 0
      }, de = {
        className: "name",
        relevance: 0,
        keywords: q,
        begin: V,
        starts: ue
      }, fe = [
        le,
        te,
        oe,
        se,
        ne,
        ce,
        ae,
        Z,
        ee,
        J
      ];
      return le.contains = [
        _.COMMENT("comment", ""),
        de,
        ue
      ], ue.contains = fe, ae.contains = fe, {
        name: "Hy",
        aliases: ["hylang"],
        illegal: /\S/,
        contains: [
          _.SHEBANG(),
          le,
          te,
          oe,
          se,
          ne,
          ce,
          ae,
          Z,
          ee
        ]
      };
    }
    return hy_1 = O, hy_1;
  }
  var inform7_1, hasRequiredInform7;
  function requireInform7() {
    if (hasRequiredInform7) return inform7_1;
    hasRequiredInform7 = 1;
    function O(_) {
      const H = "\\[", V = "\\]";
      return {
        name: "Inform 7",
        aliases: ["i7"],
        case_insensitive: !0,
        keywords: {
          // Some keywords more or less unique to I7, for relevance.
          keyword: (
            // kind:
            "thing room person man woman animal container supporter backdrop door scenery open closed locked inside gender is are say understand kind of rule"
          )
        },
        contains: [
          {
            className: "string",
            begin: '"',
            end: '"',
            relevance: 0,
            contains: [
              {
                className: "subst",
                begin: H,
                end: V
              }
            ]
          },
          {
            className: "section",
            begin: /^(Volume|Book|Part|Chapter|Section|Table)\b/,
            end: "$"
          },
          {
            // Rule definition
            // This is here for relevance.
            begin: /^(Check|Carry out|Report|Instead of|To|Rule|When|Before|After)\b/,
            end: ":",
            contains: [
              {
                // Rule name
                begin: "\\(This",
                end: "\\)"
              }
            ]
          },
          {
            className: "comment",
            begin: H,
            end: V,
            contains: ["self"]
          }
        ]
      };
    }
    return inform7_1 = O, inform7_1;
  }
  var irpf90_1, hasRequiredIrpf90;
  function requireIrpf90() {
    if (hasRequiredIrpf90) return irpf90_1;
    hasRequiredIrpf90 = 1;
    function O(_) {
      const H = _.regex, V = {
        className: "params",
        begin: "\\(",
        end: "\\)"
      }, q = /(_[a-z_\d]+)?/, G = /([de][+-]?\d+)?/, J = {
        className: "number",
        variants: [
          { begin: H.concat(/\b\d+/, /\.(\d*)/, G, q) },
          { begin: H.concat(/\b\d+/, G, q) },
          { begin: H.concat(/\.\d+/, G, q) }
        ],
        relevance: 0
      };
      return {
        name: "IRPF90",
        case_insensitive: !0,
        keywords: {
          literal: ".False. .True.",
          keyword: "kind do while private call intrinsic where elsewhere type endtype endmodule endselect endinterface end enddo endif if forall endforall only contains default return stop then public subroutine|10 function program .and. .or. .not. .le. .eq. .ge. .gt. .lt. goto save else use module select case access blank direct exist file fmt form formatted iostat name named nextrec number opened rec recl sequential status unformatted unit continue format pause cycle exit c_null_char c_alert c_backspace c_form_feed flush wait decimal round iomsg synchronous nopass non_overridable pass protected volatile abstract extends import non_intrinsic value deferred generic final enumerator class associate bind enum c_int c_short c_long c_long_long c_signed_char c_size_t c_int8_t c_int16_t c_int32_t c_int64_t c_int_least8_t c_int_least16_t c_int_least32_t c_int_least64_t c_int_fast8_t c_int_fast16_t c_int_fast32_t c_int_fast64_t c_intmax_t C_intptr_t c_float c_double c_long_double c_float_complex c_double_complex c_long_double_complex c_bool c_char c_null_ptr c_null_funptr c_new_line c_carriage_return c_horizontal_tab c_vertical_tab iso_c_binding c_loc c_funloc c_associated  c_f_pointer c_ptr c_funptr iso_fortran_env character_storage_size error_unit file_storage_size input_unit iostat_end iostat_eor numeric_storage_size output_unit c_f_procpointer ieee_arithmetic ieee_support_underflow_control ieee_get_underflow_mode ieee_set_underflow_mode newunit contiguous recursive pad position action delim readwrite eor advance nml interface procedure namelist include sequence elemental pure integer real character complex logical dimension allocatable|10 parameter external implicit|10 none double precision assign intent optional pointer target in out common equivalence data begin_provider &begin_provider end_provider begin_shell end_shell begin_template end_template subst assert touch soft_touch provide no_dep free irp_if irp_else irp_endif irp_write irp_read",
          built_in: "alog alog10 amax0 amax1 amin0 amin1 amod cabs ccos cexp clog csin csqrt dabs dacos dasin datan datan2 dcos dcosh ddim dexp dint dlog dlog10 dmax1 dmin1 dmod dnint dsign dsin dsinh dsqrt dtan dtanh float iabs idim idint idnint ifix isign max0 max1 min0 min1 sngl algama cdabs cdcos cdexp cdlog cdsin cdsqrt cqabs cqcos cqexp cqlog cqsin cqsqrt dcmplx dconjg derf derfc dfloat dgamma dimag dlgama iqint qabs qacos qasin qatan qatan2 qcmplx qconjg qcos qcosh qdim qerf qerfc qexp qgamma qimag qlgama qlog qlog10 qmax1 qmin1 qmod qnint qsign qsin qsinh qsqrt qtan qtanh abs acos aimag aint anint asin atan atan2 char cmplx conjg cos cosh exp ichar index int log log10 max min nint sign sin sinh sqrt tan tanh print write dim lge lgt lle llt mod nullify allocate deallocate adjustl adjustr all allocated any associated bit_size btest ceiling count cshift date_and_time digits dot_product eoshift epsilon exponent floor fraction huge iand ibclr ibits ibset ieor ior ishft ishftc lbound len_trim matmul maxexponent maxloc maxval merge minexponent minloc minval modulo mvbits nearest pack present product radix random_number random_seed range repeat reshape rrspacing scale scan selected_int_kind selected_real_kind set_exponent shape size spacing spread sum system_clock tiny transpose trim ubound unpack verify achar iachar transfer dble entry dprod cpu_time command_argument_count get_command get_command_argument get_environment_variable is_iostat_end ieee_arithmetic ieee_support_underflow_control ieee_get_underflow_mode ieee_set_underflow_mode is_iostat_eor move_alloc new_line selected_char_kind same_type_as extends_type_of acosh asinh atanh bessel_j0 bessel_j1 bessel_jn bessel_y0 bessel_y1 bessel_yn erf erfc erfc_scaled gamma log_gamma hypot norm2 atomic_define atomic_ref execute_command_line leadz trailz storage_size merge_bits bge bgt ble blt dshiftl dshiftr findloc iall iany iparity image_index lcobound ucobound maskl maskr num_images parity popcnt poppar shifta shiftl shiftr this_image IRP_ALIGN irp_here"
        },
        illegal: /\/\*/,
        contains: [
          _.inherit(_.APOS_STRING_MODE, {
            className: "string",
            relevance: 0
          }),
          _.inherit(_.QUOTE_STRING_MODE, {
            className: "string",
            relevance: 0
          }),
          {
            className: "function",
            beginKeywords: "subroutine function program",
            illegal: "[${=\\n]",
            contains: [
              _.UNDERSCORE_TITLE_MODE,
              V
            ]
          },
          _.COMMENT("!", "$", { relevance: 0 }),
          _.COMMENT("begin_doc", "end_doc", { relevance: 10 }),
          J
        ]
      };
    }
    return irpf90_1 = O, irpf90_1;
  }
  var isbl_1, hasRequiredIsbl;
  function requireIsbl() {
    if (hasRequiredIsbl) return isbl_1;
    hasRequiredIsbl = 1;
    function O(_) {
      const H = "[A-Za-zА-Яа-яёЁ_!][A-Za-zА-Яа-яёЁ_0-9]*", V = "[A-Za-zА-Яа-яёЁ_][A-Za-zА-Яа-яёЁ_0-9]*", q = "and и else иначе endexcept endfinally endforeach конецвсе endif конецесли endwhile конецпока except exitfor finally foreach все if если in в not не or или try while пока ", Ve = "SYSRES_CONST_ACCES_RIGHT_TYPE_EDIT SYSRES_CONST_ACCES_RIGHT_TYPE_FULL SYSRES_CONST_ACCES_RIGHT_TYPE_VIEW SYSRES_CONST_ACCESS_MODE_REQUISITE_CODE SYSRES_CONST_ACCESS_NO_ACCESS_VIEW SYSRES_CONST_ACCESS_NO_ACCESS_VIEW_CODE SYSRES_CONST_ACCESS_RIGHTS_ADD_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_ADD_REQUISITE_YES_CODE SYSRES_CONST_ACCESS_RIGHTS_CHANGE_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_CHANGE_REQUISITE_YES_CODE SYSRES_CONST_ACCESS_RIGHTS_DELETE_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_DELETE_REQUISITE_YES_CODE SYSRES_CONST_ACCESS_RIGHTS_EXECUTE_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_EXECUTE_REQUISITE_YES_CODE SYSRES_CONST_ACCESS_RIGHTS_NO_ACCESS_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_NO_ACCESS_REQUISITE_YES_CODE SYSRES_CONST_ACCESS_RIGHTS_RATIFY_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_RATIFY_REQUISITE_YES_CODE SYSRES_CONST_ACCESS_RIGHTS_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_VIEW SYSRES_CONST_ACCESS_RIGHTS_VIEW_CODE SYSRES_CONST_ACCESS_RIGHTS_VIEW_REQUISITE_CODE SYSRES_CONST_ACCESS_RIGHTS_VIEW_REQUISITE_YES_CODE SYSRES_CONST_ACCESS_TYPE_CHANGE SYSRES_CONST_ACCESS_TYPE_CHANGE_CODE SYSRES_CONST_ACCESS_TYPE_EXISTS SYSRES_CONST_ACCESS_TYPE_EXISTS_CODE SYSRES_CONST_ACCESS_TYPE_FULL SYSRES_CONST_ACCESS_TYPE_FULL_CODE SYSRES_CONST_ACCESS_TYPE_VIEW SYSRES_CONST_ACCESS_TYPE_VIEW_CODE SYSRES_CONST_ACTION_TYPE_ABORT SYSRES_CONST_ACTION_TYPE_ACCEPT SYSRES_CONST_ACTION_TYPE_ACCESS_RIGHTS SYSRES_CONST_ACTION_TYPE_ADD_ATTACHMENT SYSRES_CONST_ACTION_TYPE_CHANGE_CARD SYSRES_CONST_ACTION_TYPE_CHANGE_KIND SYSRES_CONST_ACTION_TYPE_CHANGE_STORAGE SYSRES_CONST_ACTION_TYPE_CONTINUE SYSRES_CONST_ACTION_TYPE_COPY SYSRES_CONST_ACTION_TYPE_CREATE SYSRES_CONST_ACTION_TYPE_CREATE_VERSION SYSRES_CONST_ACTION_TYPE_DELETE SYSRES_CONST_ACTION_TYPE_DELETE_ATTACHMENT SYSRES_CONST_ACTION_TYPE_DELETE_VERSION SYSRES_CONST_ACTION_TYPE_DISABLE_DELEGATE_ACCESS_RIGHTS SYSRES_CONST_ACTION_TYPE_ENABLE_DELEGATE_ACCESS_RIGHTS SYSRES_CONST_ACTION_TYPE_ENCRYPTION_BY_CERTIFICATE SYSRES_CONST_ACTION_TYPE_ENCRYPTION_BY_CERTIFICATE_AND_PASSWORD SYSRES_CONST_ACTION_TYPE_ENCRYPTION_BY_PASSWORD SYSRES_CONST_ACTION_TYPE_EXPORT_WITH_LOCK SYSRES_CONST_ACTION_TYPE_EXPORT_WITHOUT_LOCK SYSRES_CONST_ACTION_TYPE_IMPORT_WITH_UNLOCK SYSRES_CONST_ACTION_TYPE_IMPORT_WITHOUT_UNLOCK SYSRES_CONST_ACTION_TYPE_LIFE_CYCLE_STAGE SYSRES_CONST_ACTION_TYPE_LOCK SYSRES_CONST_ACTION_TYPE_LOCK_FOR_SERVER SYSRES_CONST_ACTION_TYPE_LOCK_MODIFY SYSRES_CONST_ACTION_TYPE_MARK_AS_READED SYSRES_CONST_ACTION_TYPE_MARK_AS_UNREADED SYSRES_CONST_ACTION_TYPE_MODIFY SYSRES_CONST_ACTION_TYPE_MODIFY_CARD SYSRES_CONST_ACTION_TYPE_MOVE_TO_ARCHIVE SYSRES_CONST_ACTION_TYPE_OFF_ENCRYPTION SYSRES_CONST_ACTION_TYPE_PASSWORD_CHANGE SYSRES_CONST_ACTION_TYPE_PERFORM SYSRES_CONST_ACTION_TYPE_RECOVER_FROM_LOCAL_COPY SYSRES_CONST_ACTION_TYPE_RESTART SYSRES_CONST_ACTION_TYPE_RESTORE_FROM_ARCHIVE SYSRES_CONST_ACTION_TYPE_REVISION SYSRES_CONST_ACTION_TYPE_SEND_BY_MAIL SYSRES_CONST_ACTION_TYPE_SIGN SYSRES_CONST_ACTION_TYPE_START SYSRES_CONST_ACTION_TYPE_UNLOCK SYSRES_CONST_ACTION_TYPE_UNLOCK_FROM_SERVER SYSRES_CONST_ACTION_TYPE_VERSION_STATE SYSRES_CONST_ACTION_TYPE_VERSION_VISIBILITY SYSRES_CONST_ACTION_TYPE_VIEW SYSRES_CONST_ACTION_TYPE_VIEW_SHADOW_COPY SYSRES_CONST_ACTION_TYPE_WORKFLOW_DESCRIPTION_MODIFY SYSRES_CONST_ACTION_TYPE_WRITE_HISTORY SYSRES_CONST_ACTIVE_VERSION_STATE_PICK_VALUE SYSRES_CONST_ADD_REFERENCE_MODE_NAME SYSRES_CONST_ADDITION_REQUISITE_CODE SYSRES_CONST_ADDITIONAL_PARAMS_REQUISITE_CODE SYSRES_CONST_ADITIONAL_JOB_END_DATE_REQUISITE_NAME SYSRES_CONST_ADITIONAL_JOB_READ_REQUISITE_NAME SYSRES_CONST_ADITIONAL_JOB_START_DATE_REQUISITE_NAME SYSRES_CONST_ADITIONAL_JOB_STATE_REQUISITE_NAME SYSRES_CONST_ADMINISTRATION_HISTORY_ADDING_USER_TO_GROUP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_ADDING_USER_TO_GROUP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_CREATION_COMP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_CREATION_COMP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_CREATION_GROUP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_CREATION_GROUP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_CREATION_USER_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_CREATION_USER_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_DATABASE_USER_CREATION SYSRES_CONST_ADMINISTRATION_HISTORY_DATABASE_USER_CREATION_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_DATABASE_USER_DELETION SYSRES_CONST_ADMINISTRATION_HISTORY_DATABASE_USER_DELETION_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_COMP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_COMP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_GROUP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_GROUP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_USER_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_USER_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_USER_FROM_GROUP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_DELETION_USER_FROM_GROUP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_FILTERER_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_FILTERER_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_FILTERER_RESTRICTION_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_FILTERER_RESTRICTION_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_PRIVILEGE_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_PRIVILEGE_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_RIGHTS_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_GRANTING_RIGHTS_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_IS_MAIN_SERVER_CHANGED_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_IS_MAIN_SERVER_CHANGED_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_IS_PUBLIC_CHANGED_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_IS_PUBLIC_CHANGED_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_FILTERER_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_FILTERER_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_FILTERER_RESTRICTION_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_FILTERER_RESTRICTION_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_PRIVILEGE_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_PRIVILEGE_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_RIGHTS_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_REMOVING_RIGHTS_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_SERVER_LOGIN_CREATION SYSRES_CONST_ADMINISTRATION_HISTORY_SERVER_LOGIN_CREATION_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_SERVER_LOGIN_DELETION SYSRES_CONST_ADMINISTRATION_HISTORY_SERVER_LOGIN_DELETION_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_CATEGORY_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_CATEGORY_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_COMP_TITLE_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_COMP_TITLE_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_FULL_NAME_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_FULL_NAME_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_GROUP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_GROUP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_PARENT_GROUP_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_PARENT_GROUP_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_USER_AUTH_TYPE_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_USER_AUTH_TYPE_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_USER_LOGIN_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_USER_LOGIN_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_USER_STATUS_ACTION SYSRES_CONST_ADMINISTRATION_HISTORY_UPDATING_USER_STATUS_ACTION_CODE SYSRES_CONST_ADMINISTRATION_HISTORY_USER_PASSWORD_CHANGE SYSRES_CONST_ADMINISTRATION_HISTORY_USER_PASSWORD_CHANGE_ACTION SYSRES_CONST_ALL_ACCEPT_CONDITION_RUS SYSRES_CONST_ALL_USERS_GROUP SYSRES_CONST_ALL_USERS_GROUP_NAME SYSRES_CONST_ALL_USERS_SERVER_GROUP_NAME SYSRES_CONST_ALLOWED_ACCESS_TYPE_CODE SYSRES_CONST_ALLOWED_ACCESS_TYPE_NAME SYSRES_CONST_APP_VIEWER_TYPE_REQUISITE_CODE SYSRES_CONST_APPROVING_SIGNATURE_NAME SYSRES_CONST_APPROVING_SIGNATURE_REQUISITE_CODE SYSRES_CONST_ASSISTANT_SUBSTITUE_TYPE SYSRES_CONST_ASSISTANT_SUBSTITUE_TYPE_CODE SYSRES_CONST_ATTACH_TYPE_COMPONENT_TOKEN SYSRES_CONST_ATTACH_TYPE_DOC SYSRES_CONST_ATTACH_TYPE_EDOC SYSRES_CONST_ATTACH_TYPE_FOLDER SYSRES_CONST_ATTACH_TYPE_JOB SYSRES_CONST_ATTACH_TYPE_REFERENCE SYSRES_CONST_ATTACH_TYPE_TASK SYSRES_CONST_AUTH_ENCODED_PASSWORD SYSRES_CONST_AUTH_ENCODED_PASSWORD_CODE SYSRES_CONST_AUTH_NOVELL SYSRES_CONST_AUTH_PASSWORD SYSRES_CONST_AUTH_PASSWORD_CODE SYSRES_CONST_AUTH_WINDOWS SYSRES_CONST_AUTHENTICATING_SIGNATURE_NAME SYSRES_CONST_AUTHENTICATING_SIGNATURE_REQUISITE_CODE SYSRES_CONST_AUTO_ENUM_METHOD_FLAG SYSRES_CONST_AUTO_NUMERATION_CODE SYSRES_CONST_AUTO_STRONG_ENUM_METHOD_FLAG SYSRES_CONST_AUTOTEXT_NAME_REQUISITE_CODE SYSRES_CONST_AUTOTEXT_TEXT_REQUISITE_CODE SYSRES_CONST_AUTOTEXT_USAGE_ALL SYSRES_CONST_AUTOTEXT_USAGE_ALL_CODE SYSRES_CONST_AUTOTEXT_USAGE_SIGN SYSRES_CONST_AUTOTEXT_USAGE_SIGN_CODE SYSRES_CONST_AUTOTEXT_USAGE_WORK SYSRES_CONST_AUTOTEXT_USAGE_WORK_CODE SYSRES_CONST_AUTOTEXT_USE_ANYWHERE_CODE SYSRES_CONST_AUTOTEXT_USE_ON_SIGNING_CODE SYSRES_CONST_AUTOTEXT_USE_ON_WORK_CODE SYSRES_CONST_BEGIN_DATE_REQUISITE_CODE SYSRES_CONST_BLACK_LIFE_CYCLE_STAGE_FONT_COLOR SYSRES_CONST_BLUE_LIFE_CYCLE_STAGE_FONT_COLOR SYSRES_CONST_BTN_PART SYSRES_CONST_CALCULATED_ROLE_TYPE_CODE SYSRES_CONST_CALL_TYPE_VARIABLE_BUTTON_VALUE SYSRES_CONST_CALL_TYPE_VARIABLE_PROGRAM_VALUE SYSRES_CONST_CANCEL_MESSAGE_FUNCTION_RESULT SYSRES_CONST_CARD_PART SYSRES_CONST_CARD_REFERENCE_MODE_NAME SYSRES_CONST_CERTIFICATE_TYPE_REQUISITE_ENCRYPT_VALUE SYSRES_CONST_CERTIFICATE_TYPE_REQUISITE_SIGN_AND_ENCRYPT_VALUE SYSRES_CONST_CERTIFICATE_TYPE_REQUISITE_SIGN_VALUE SYSRES_CONST_CHECK_PARAM_VALUE_DATE_PARAM_TYPE SYSRES_CONST_CHECK_PARAM_VALUE_FLOAT_PARAM_TYPE SYSRES_CONST_CHECK_PARAM_VALUE_INTEGER_PARAM_TYPE SYSRES_CONST_CHECK_PARAM_VALUE_PICK_PARAM_TYPE SYSRES_CONST_CHECK_PARAM_VALUE_REEFRENCE_PARAM_TYPE SYSRES_CONST_CLOSED_RECORD_FLAG_VALUE_FEMININE SYSRES_CONST_CLOSED_RECORD_FLAG_VALUE_MASCULINE SYSRES_CONST_CODE_COMPONENT_TYPE_ADMIN SYSRES_CONST_CODE_COMPONENT_TYPE_DEVELOPER SYSRES_CONST_CODE_COMPONENT_TYPE_DOCS SYSRES_CONST_CODE_COMPONENT_TYPE_EDOC_CARDS SYSRES_CONST_CODE_COMPONENT_TYPE_EXTERNAL_EXECUTABLE SYSRES_CONST_CODE_COMPONENT_TYPE_OTHER SYSRES_CONST_CODE_COMPONENT_TYPE_REFERENCE SYSRES_CONST_CODE_COMPONENT_TYPE_REPORT SYSRES_CONST_CODE_COMPONENT_TYPE_SCRIPT SYSRES_CONST_CODE_COMPONENT_TYPE_URL SYSRES_CONST_CODE_REQUISITE_ACCESS SYSRES_CONST_CODE_REQUISITE_CODE SYSRES_CONST_CODE_REQUISITE_COMPONENT SYSRES_CONST_CODE_REQUISITE_DESCRIPTION SYSRES_CONST_CODE_REQUISITE_EXCLUDE_COMPONENT SYSRES_CONST_CODE_REQUISITE_RECORD SYSRES_CONST_COMMENT_REQ_CODE SYSRES_CONST_COMMON_SETTINGS_REQUISITE_CODE SYSRES_CONST_COMP_CODE_GRD SYSRES_CONST_COMPONENT_GROUP_TYPE_REQUISITE_CODE SYSRES_CONST_COMPONENT_TYPE_ADMIN_COMPONENTS SYSRES_CONST_COMPONENT_TYPE_DEVELOPER_COMPONENTS SYSRES_CONST_COMPONENT_TYPE_DOCS SYSRES_CONST_COMPONENT_TYPE_EDOC_CARDS SYSRES_CONST_COMPONENT_TYPE_EDOCS SYSRES_CONST_COMPONENT_TYPE_EXTERNAL_EXECUTABLE SYSRES_CONST_COMPONENT_TYPE_OTHER SYSRES_CONST_COMPONENT_TYPE_REFERENCE_TYPES SYSRES_CONST_COMPONENT_TYPE_REFERENCES SYSRES_CONST_COMPONENT_TYPE_REPORTS SYSRES_CONST_COMPONENT_TYPE_SCRIPTS SYSRES_CONST_COMPONENT_TYPE_URL SYSRES_CONST_COMPONENTS_REMOTE_SERVERS_VIEW_CODE SYSRES_CONST_CONDITION_BLOCK_DESCRIPTION SYSRES_CONST_CONST_FIRM_STATUS_COMMON SYSRES_CONST_CONST_FIRM_STATUS_INDIVIDUAL SYSRES_CONST_CONST_NEGATIVE_VALUE SYSRES_CONST_CONST_POSITIVE_VALUE SYSRES_CONST_CONST_SERVER_STATUS_DONT_REPLICATE SYSRES_CONST_CONST_SERVER_STATUS_REPLICATE SYSRES_CONST_CONTENTS_REQUISITE_CODE SYSRES_CONST_DATA_TYPE_BOOLEAN SYSRES_CONST_DATA_TYPE_DATE SYSRES_CONST_DATA_TYPE_FLOAT SYSRES_CONST_DATA_TYPE_INTEGER SYSRES_CONST_DATA_TYPE_PICK SYSRES_CONST_DATA_TYPE_REFERENCE SYSRES_CONST_DATA_TYPE_STRING SYSRES_CONST_DATA_TYPE_TEXT SYSRES_CONST_DATA_TYPE_VARIANT SYSRES_CONST_DATE_CLOSE_REQ_CODE SYSRES_CONST_DATE_FORMAT_DATE_ONLY_CHAR SYSRES_CONST_DATE_OPEN_REQ_CODE SYSRES_CONST_DATE_REQUISITE SYSRES_CONST_DATE_REQUISITE_CODE SYSRES_CONST_DATE_REQUISITE_NAME SYSRES_CONST_DATE_REQUISITE_TYPE SYSRES_CONST_DATE_TYPE_CHAR SYSRES_CONST_DATETIME_FORMAT_VALUE SYSRES_CONST_DEA_ACCESS_RIGHTS_ACTION_CODE SYSRES_CONST_DESCRIPTION_LOCALIZE_ID_REQUISITE_CODE SYSRES_CONST_DESCRIPTION_REQUISITE_CODE SYSRES_CONST_DET1_PART SYSRES_CONST_DET2_PART SYSRES_CONST_DET3_PART SYSRES_CONST_DET4_PART SYSRES_CONST_DET5_PART SYSRES_CONST_DET6_PART SYSRES_CONST_DETAIL_DATASET_KEY_REQUISITE_CODE SYSRES_CONST_DETAIL_PICK_REQUISITE_CODE SYSRES_CONST_DETAIL_REQ_CODE SYSRES_CONST_DO_NOT_USE_ACCESS_TYPE_CODE SYSRES_CONST_DO_NOT_USE_ACCESS_TYPE_NAME SYSRES_CONST_DO_NOT_USE_ON_VIEW_ACCESS_TYPE_CODE SYSRES_CONST_DO_NOT_USE_ON_VIEW_ACCESS_TYPE_NAME SYSRES_CONST_DOCUMENT_STORAGES_CODE SYSRES_CONST_DOCUMENT_TEMPLATES_TYPE_NAME SYSRES_CONST_DOUBLE_REQUISITE_CODE SYSRES_CONST_EDITOR_CLOSE_FILE_OBSERV_TYPE_CODE SYSRES_CONST_EDITOR_CLOSE_PROCESS_OBSERV_TYPE_CODE SYSRES_CONST_EDITOR_TYPE_REQUISITE_CODE SYSRES_CONST_EDITORS_APPLICATION_NAME_REQUISITE_CODE SYSRES_CONST_EDITORS_CREATE_SEVERAL_PROCESSES_REQUISITE_CODE SYSRES_CONST_EDITORS_EXTENSION_REQUISITE_CODE SYSRES_CONST_EDITORS_OBSERVER_BY_PROCESS_TYPE SYSRES_CONST_EDITORS_REFERENCE_CODE SYSRES_CONST_EDITORS_REPLACE_SPEC_CHARS_REQUISITE_CODE SYSRES_CONST_EDITORS_USE_PLUGINS_REQUISITE_CODE SYSRES_CONST_EDITORS_VIEW_DOCUMENT_OPENED_TO_EDIT_CODE SYSRES_CONST_EDOC_CARD_TYPE_REQUISITE_CODE SYSRES_CONST_EDOC_CARD_TYPES_LINK_REQUISITE_CODE SYSRES_CONST_EDOC_CERTIFICATE_AND_PASSWORD_ENCODE_CODE SYSRES_CONST_EDOC_CERTIFICATE_ENCODE_CODE SYSRES_CONST_EDOC_DATE_REQUISITE_CODE SYSRES_CONST_EDOC_KIND_REFERENCE_CODE SYSRES_CONST_EDOC_KINDS_BY_TEMPLATE_ACTION_CODE SYSRES_CONST_EDOC_MANAGE_ACCESS_CODE SYSRES_CONST_EDOC_NONE_ENCODE_CODE SYSRES_CONST_EDOC_NUMBER_REQUISITE_CODE SYSRES_CONST_EDOC_PASSWORD_ENCODE_CODE SYSRES_CONST_EDOC_READONLY_ACCESS_CODE SYSRES_CONST_EDOC_SHELL_LIFE_TYPE_VIEW_VALUE SYSRES_CONST_EDOC_SIZE_RESTRICTION_PRIORITY_REQUISITE_CODE SYSRES_CONST_EDOC_STORAGE_CHECK_ACCESS_RIGHTS_REQUISITE_CODE SYSRES_CONST_EDOC_STORAGE_COMPUTER_NAME_REQUISITE_CODE SYSRES_CONST_EDOC_STORAGE_DATABASE_NAME_REQUISITE_CODE SYSRES_CONST_EDOC_STORAGE_EDIT_IN_STORAGE_REQUISITE_CODE SYSRES_CONST_EDOC_STORAGE_LOCAL_PATH_REQUISITE_CODE SYSRES_CONST_EDOC_STORAGE_SHARED_SOURCE_NAME_REQUISITE_CODE SYSRES_CONST_EDOC_TEMPLATE_REQUISITE_CODE SYSRES_CONST_EDOC_TYPES_REFERENCE_CODE SYSRES_CONST_EDOC_VERSION_ACTIVE_STAGE_CODE SYSRES_CONST_EDOC_VERSION_DESIGN_STAGE_CODE SYSRES_CONST_EDOC_VERSION_OBSOLETE_STAGE_CODE SYSRES_CONST_EDOC_WRITE_ACCES_CODE SYSRES_CONST_EDOCUMENT_CARD_REQUISITES_REFERENCE_CODE_SELECTED_REQUISITE SYSRES_CONST_ENCODE_CERTIFICATE_TYPE_CODE SYSRES_CONST_END_DATE_REQUISITE_CODE SYSRES_CONST_ENUMERATION_TYPE_REQUISITE_CODE SYSRES_CONST_EXECUTE_ACCESS_RIGHTS_TYPE_CODE SYSRES_CONST_EXECUTIVE_FILE_STORAGE_TYPE SYSRES_CONST_EXIST_CONST SYSRES_CONST_EXIST_VALUE SYSRES_CONST_EXPORT_LOCK_TYPE_ASK SYSRES_CONST_EXPORT_LOCK_TYPE_WITH_LOCK SYSRES_CONST_EXPORT_LOCK_TYPE_WITHOUT_LOCK SYSRES_CONST_EXPORT_VERSION_TYPE_ASK SYSRES_CONST_EXPORT_VERSION_TYPE_LAST SYSRES_CONST_EXPORT_VERSION_TYPE_LAST_ACTIVE SYSRES_CONST_EXTENSION_REQUISITE_CODE SYSRES_CONST_FILTER_NAME_REQUISITE_CODE SYSRES_CONST_FILTER_REQUISITE_CODE SYSRES_CONST_FILTER_TYPE_COMMON_CODE SYSRES_CONST_FILTER_TYPE_COMMON_NAME SYSRES_CONST_FILTER_TYPE_USER_CODE SYSRES_CONST_FILTER_TYPE_USER_NAME SYSRES_CONST_FILTER_VALUE_REQUISITE_NAME SYSRES_CONST_FLOAT_NUMBER_FORMAT_CHAR SYSRES_CONST_FLOAT_REQUISITE_TYPE SYSRES_CONST_FOLDER_AUTHOR_VALUE SYSRES_CONST_FOLDER_KIND_ANY_OBJECTS SYSRES_CONST_FOLDER_KIND_COMPONENTS SYSRES_CONST_FOLDER_KIND_EDOCS SYSRES_CONST_FOLDER_KIND_JOBS SYSRES_CONST_FOLDER_KIND_TASKS SYSRES_CONST_FOLDER_TYPE_COMMON SYSRES_CONST_FOLDER_TYPE_COMPONENT SYSRES_CONST_FOLDER_TYPE_FAVORITES SYSRES_CONST_FOLDER_TYPE_INBOX SYSRES_CONST_FOLDER_TYPE_OUTBOX SYSRES_CONST_FOLDER_TYPE_QUICK_LAUNCH SYSRES_CONST_FOLDER_TYPE_SEARCH SYSRES_CONST_FOLDER_TYPE_SHORTCUTS SYSRES_CONST_FOLDER_TYPE_USER SYSRES_CONST_FROM_DICTIONARY_ENUM_METHOD_FLAG SYSRES_CONST_FULL_SUBSTITUTE_TYPE SYSRES_CONST_FULL_SUBSTITUTE_TYPE_CODE SYSRES_CONST_FUNCTION_CANCEL_RESULT SYSRES_CONST_FUNCTION_CATEGORY_SYSTEM SYSRES_CONST_FUNCTION_CATEGORY_USER SYSRES_CONST_FUNCTION_FAILURE_RESULT SYSRES_CONST_FUNCTION_SAVE_RESULT SYSRES_CONST_GENERATED_REQUISITE SYSRES_CONST_GREEN_LIFE_CYCLE_STAGE_FONT_COLOR SYSRES_CONST_GROUP_ACCOUNT_TYPE_VALUE_CODE SYSRES_CONST_GROUP_CATEGORY_NORMAL_CODE SYSRES_CONST_GROUP_CATEGORY_NORMAL_NAME SYSRES_CONST_GROUP_CATEGORY_SERVICE_CODE SYSRES_CONST_GROUP_CATEGORY_SERVICE_NAME SYSRES_CONST_GROUP_COMMON_CATEGORY_FIELD_VALUE SYSRES_CONST_GROUP_FULL_NAME_REQUISITE_CODE SYSRES_CONST_GROUP_NAME_REQUISITE_CODE SYSRES_CONST_GROUP_RIGHTS_T_REQUISITE_CODE SYSRES_CONST_GROUP_SERVER_CODES_REQUISITE_CODE SYSRES_CONST_GROUP_SERVER_NAME_REQUISITE_CODE SYSRES_CONST_GROUP_SERVICE_CATEGORY_FIELD_VALUE SYSRES_CONST_GROUP_USER_REQUISITE_CODE SYSRES_CONST_GROUPS_REFERENCE_CODE SYSRES_CONST_GROUPS_REQUISITE_CODE SYSRES_CONST_HIDDEN_MODE_NAME SYSRES_CONST_HIGH_LVL_REQUISITE_CODE SYSRES_CONST_HISTORY_ACTION_CREATE_CODE SYSRES_CONST_HISTORY_ACTION_DELETE_CODE SYSRES_CONST_HISTORY_ACTION_EDIT_CODE SYSRES_CONST_HOUR_CHAR SYSRES_CONST_ID_REQUISITE_CODE SYSRES_CONST_IDSPS_REQUISITE_CODE SYSRES_CONST_IMAGE_MODE_COLOR SYSRES_CONST_IMAGE_MODE_GREYSCALE SYSRES_CONST_IMAGE_MODE_MONOCHROME SYSRES_CONST_IMPORTANCE_HIGH SYSRES_CONST_IMPORTANCE_LOW SYSRES_CONST_IMPORTANCE_NORMAL SYSRES_CONST_IN_DESIGN_VERSION_STATE_PICK_VALUE SYSRES_CONST_INCOMING_WORK_RULE_TYPE_CODE SYSRES_CONST_INT_REQUISITE SYSRES_CONST_INT_REQUISITE_TYPE SYSRES_CONST_INTEGER_NUMBER_FORMAT_CHAR SYSRES_CONST_INTEGER_TYPE_CHAR SYSRES_CONST_IS_GENERATED_REQUISITE_NEGATIVE_VALUE SYSRES_CONST_IS_PUBLIC_ROLE_REQUISITE_CODE SYSRES_CONST_IS_REMOTE_USER_NEGATIVE_VALUE SYSRES_CONST_IS_REMOTE_USER_POSITIVE_VALUE SYSRES_CONST_IS_STORED_REQUISITE_NEGATIVE_VALUE SYSRES_CONST_IS_STORED_REQUISITE_STORED_VALUE SYSRES_CONST_ITALIC_LIFE_CYCLE_STAGE_DRAW_STYLE SYSRES_CONST_JOB_BLOCK_DESCRIPTION SYSRES_CONST_JOB_KIND_CONTROL_JOB SYSRES_CONST_JOB_KIND_JOB SYSRES_CONST_JOB_KIND_NOTICE SYSRES_CONST_JOB_STATE_ABORTED SYSRES_CONST_JOB_STATE_COMPLETE SYSRES_CONST_JOB_STATE_WORKING SYSRES_CONST_KIND_REQUISITE_CODE SYSRES_CONST_KIND_REQUISITE_NAME SYSRES_CONST_KINDS_CREATE_SHADOW_COPIES_REQUISITE_CODE SYSRES_CONST_KINDS_DEFAULT_EDOC_LIFE_STAGE_REQUISITE_CODE SYSRES_CONST_KINDS_EDOC_ALL_TEPLATES_ALLOWED_REQUISITE_CODE SYSRES_CONST_KINDS_EDOC_ALLOW_LIFE_CYCLE_STAGE_CHANGING_REQUISITE_CODE SYSRES_CONST_KINDS_EDOC_ALLOW_MULTIPLE_ACTIVE_VERSIONS_REQUISITE_CODE SYSRES_CONST_KINDS_EDOC_SHARE_ACCES_RIGHTS_BY_DEFAULT_CODE SYSRES_CONST_KINDS_EDOC_TEMPLATE_REQUISITE_CODE SYSRES_CONST_KINDS_EDOC_TYPE_REQUISITE_CODE SYSRES_CONST_KINDS_SIGNERS_REQUISITES_CODE SYSRES_CONST_KOD_INPUT_TYPE SYSRES_CONST_LAST_UPDATE_DATE_REQUISITE_CODE SYSRES_CONST_LIFE_CYCLE_START_STAGE_REQUISITE_CODE SYSRES_CONST_LILAC_LIFE_CYCLE_STAGE_FONT_COLOR SYSRES_CONST_LINK_OBJECT_KIND_COMPONENT SYSRES_CONST_LINK_OBJECT_KIND_DOCUMENT SYSRES_CONST_LINK_OBJECT_KIND_EDOC SYSRES_CONST_LINK_OBJECT_KIND_FOLDER SYSRES_CONST_LINK_OBJECT_KIND_JOB SYSRES_CONST_LINK_OBJECT_KIND_REFERENCE SYSRES_CONST_LINK_OBJECT_KIND_TASK SYSRES_CONST_LINK_REF_TYPE_REQUISITE_CODE SYSRES_CONST_LIST_REFERENCE_MODE_NAME SYSRES_CONST_LOCALIZATION_DICTIONARY_MAIN_VIEW_CODE SYSRES_CONST_MAIN_VIEW_CODE SYSRES_CONST_MANUAL_ENUM_METHOD_FLAG SYSRES_CONST_MASTER_COMP_TYPE_REQUISITE_CODE SYSRES_CONST_MASTER_TABLE_REC_ID_REQUISITE_CODE SYSRES_CONST_MAXIMIZED_MODE_NAME SYSRES_CONST_ME_VALUE SYSRES_CONST_MESSAGE_ATTENTION_CAPTION SYSRES_CONST_MESSAGE_CONFIRMATION_CAPTION SYSRES_CONST_MESSAGE_ERROR_CAPTION SYSRES_CONST_MESSAGE_INFORMATION_CAPTION SYSRES_CONST_MINIMIZED_MODE_NAME SYSRES_CONST_MINUTE_CHAR SYSRES_CONST_MODULE_REQUISITE_CODE SYSRES_CONST_MONITORING_BLOCK_DESCRIPTION SYSRES_CONST_MONTH_FORMAT_VALUE SYSRES_CONST_NAME_LOCALIZE_ID_REQUISITE_CODE SYSRES_CONST_NAME_REQUISITE_CODE SYSRES_CONST_NAME_SINGULAR_REQUISITE_CODE SYSRES_CONST_NAMEAN_INPUT_TYPE SYSRES_CONST_NEGATIVE_PICK_VALUE SYSRES_CONST_NEGATIVE_VALUE SYSRES_CONST_NO SYSRES_CONST_NO_PICK_VALUE SYSRES_CONST_NO_SIGNATURE_REQUISITE_CODE SYSRES_CONST_NO_VALUE SYSRES_CONST_NONE_ACCESS_RIGHTS_TYPE_CODE SYSRES_CONST_NONOPERATING_RECORD_FLAG_VALUE SYSRES_CONST_NONOPERATING_RECORD_FLAG_VALUE_MASCULINE SYSRES_CONST_NORMAL_ACCESS_RIGHTS_TYPE_CODE SYSRES_CONST_NORMAL_LIFE_CYCLE_STAGE_DRAW_STYLE SYSRES_CONST_NORMAL_MODE_NAME SYSRES_CONST_NOT_ALLOWED_ACCESS_TYPE_CODE SYSRES_CONST_NOT_ALLOWED_ACCESS_TYPE_NAME SYSRES_CONST_NOTE_REQUISITE_CODE SYSRES_CONST_NOTICE_BLOCK_DESCRIPTION SYSRES_CONST_NUM_REQUISITE SYSRES_CONST_NUM_STR_REQUISITE_CODE SYSRES_CONST_NUMERATION_AUTO_NOT_STRONG SYSRES_CONST_NUMERATION_AUTO_STRONG SYSRES_CONST_NUMERATION_FROM_DICTONARY SYSRES_CONST_NUMERATION_MANUAL SYSRES_CONST_NUMERIC_TYPE_CHAR SYSRES_CONST_NUMREQ_REQUISITE_CODE SYSRES_CONST_OBSOLETE_VERSION_STATE_PICK_VALUE SYSRES_CONST_OPERATING_RECORD_FLAG_VALUE SYSRES_CONST_OPERATING_RECORD_FLAG_VALUE_CODE SYSRES_CONST_OPERATING_RECORD_FLAG_VALUE_FEMININE SYSRES_CONST_OPERATING_RECORD_FLAG_VALUE_MASCULINE SYSRES_CONST_OPTIONAL_FORM_COMP_REQCODE_PREFIX SYSRES_CONST_ORANGE_LIFE_CYCLE_STAGE_FONT_COLOR SYSRES_CONST_ORIGINALREF_REQUISITE_CODE SYSRES_CONST_OURFIRM_REF_CODE SYSRES_CONST_OURFIRM_REQUISITE_CODE SYSRES_CONST_OURFIRM_VAR SYSRES_CONST_OUTGOING_WORK_RULE_TYPE_CODE SYSRES_CONST_PICK_NEGATIVE_RESULT SYSRES_CONST_PICK_POSITIVE_RESULT SYSRES_CONST_PICK_REQUISITE SYSRES_CONST_PICK_REQUISITE_TYPE SYSRES_CONST_PICK_TYPE_CHAR SYSRES_CONST_PLAN_STATUS_REQUISITE_CODE SYSRES_CONST_PLATFORM_VERSION_COMMENT SYSRES_CONST_PLUGINS_SETTINGS_DESCRIPTION_REQUISITE_CODE SYSRES_CONST_POSITIVE_PICK_VALUE SYSRES_CONST_POWER_TO_CREATE_ACTION_CODE SYSRES_CONST_POWER_TO_SIGN_ACTION_CODE SYSRES_CONST_PRIORITY_REQUISITE_CODE SYSRES_CONST_QUALIFIED_TASK_TYPE SYSRES_CONST_QUALIFIED_TASK_TYPE_CODE SYSRES_CONST_RECSTAT_REQUISITE_CODE SYSRES_CONST_RED_LIFE_CYCLE_STAGE_FONT_COLOR SYSRES_CONST_REF_ID_T_REF_TYPE_REQUISITE_CODE SYSRES_CONST_REF_REQUISITE SYSRES_CONST_REF_REQUISITE_TYPE SYSRES_CONST_REF_REQUISITES_REFERENCE_CODE_SELECTED_REQUISITE SYSRES_CONST_REFERENCE_RECORD_HISTORY_CREATE_ACTION_CODE SYSRES_CONST_REFERENCE_RECORD_HISTORY_DELETE_ACTION_CODE SYSRES_CONST_REFERENCE_RECORD_HISTORY_MODIFY_ACTION_CODE SYSRES_CONST_REFERENCE_TYPE_CHAR SYSRES_CONST_REFERENCE_TYPE_REQUISITE_NAME SYSRES_CONST_REFERENCES_ADD_PARAMS_REQUISITE_CODE SYSRES_CONST_REFERENCES_DISPLAY_REQUISITE_REQUISITE_CODE SYSRES_CONST_REMOTE_SERVER_STATUS_WORKING SYSRES_CONST_REMOTE_SERVER_TYPE_MAIN SYSRES_CONST_REMOTE_SERVER_TYPE_SECONDARY SYSRES_CONST_REMOTE_USER_FLAG_VALUE_CODE SYSRES_CONST_REPORT_APP_EDITOR_INTERNAL SYSRES_CONST_REPORT_BASE_REPORT_ID_REQUISITE_CODE SYSRES_CONST_REPORT_BASE_REPORT_REQUISITE_CODE SYSRES_CONST_REPORT_SCRIPT_REQUISITE_CODE SYSRES_CONST_REPORT_TEMPLATE_REQUISITE_CODE SYSRES_CONST_REPORT_VIEWER_CODE_REQUISITE_CODE SYSRES_CONST_REQ_ALLOW_COMPONENT_DEFAULT_VALUE SYSRES_CONST_REQ_ALLOW_RECORD_DEFAULT_VALUE SYSRES_CONST_REQ_ALLOW_SERVER_COMPONENT_DEFAULT_VALUE SYSRES_CONST_REQ_MODE_AVAILABLE_CODE SYSRES_CONST_REQ_MODE_EDIT_CODE SYSRES_CONST_REQ_MODE_HIDDEN_CODE SYSRES_CONST_REQ_MODE_NOT_AVAILABLE_CODE SYSRES_CONST_REQ_MODE_VIEW_CODE SYSRES_CONST_REQ_NUMBER_REQUISITE_CODE SYSRES_CONST_REQ_SECTION_VALUE SYSRES_CONST_REQ_TYPE_VALUE SYSRES_CONST_REQUISITE_FORMAT_BY_UNIT SYSRES_CONST_REQUISITE_FORMAT_DATE_FULL SYSRES_CONST_REQUISITE_FORMAT_DATE_TIME SYSRES_CONST_REQUISITE_FORMAT_LEFT SYSRES_CONST_REQUISITE_FORMAT_RIGHT SYSRES_CONST_REQUISITE_FORMAT_WITHOUT_UNIT SYSRES_CONST_REQUISITE_NUMBER_REQUISITE_CODE SYSRES_CONST_REQUISITE_SECTION_ACTIONS SYSRES_CONST_REQUISITE_SECTION_BUTTON SYSRES_CONST_REQUISITE_SECTION_BUTTONS SYSRES_CONST_REQUISITE_SECTION_CARD SYSRES_CONST_REQUISITE_SECTION_TABLE SYSRES_CONST_REQUISITE_SECTION_TABLE10 SYSRES_CONST_REQUISITE_SECTION_TABLE11 SYSRES_CONST_REQUISITE_SECTION_TABLE12 SYSRES_CONST_REQUISITE_SECTION_TABLE13 SYSRES_CONST_REQUISITE_SECTION_TABLE14 SYSRES_CONST_REQUISITE_SECTION_TABLE15 SYSRES_CONST_REQUISITE_SECTION_TABLE16 SYSRES_CONST_REQUISITE_SECTION_TABLE17 SYSRES_CONST_REQUISITE_SECTION_TABLE18 SYSRES_CONST_REQUISITE_SECTION_TABLE19 SYSRES_CONST_REQUISITE_SECTION_TABLE2 SYSRES_CONST_REQUISITE_SECTION_TABLE20 SYSRES_CONST_REQUISITE_SECTION_TABLE21 SYSRES_CONST_REQUISITE_SECTION_TABLE22 SYSRES_CONST_REQUISITE_SECTION_TABLE23 SYSRES_CONST_REQUISITE_SECTION_TABLE24 SYSRES_CONST_REQUISITE_SECTION_TABLE3 SYSRES_CONST_REQUISITE_SECTION_TABLE4 SYSRES_CONST_REQUISITE_SECTION_TABLE5 SYSRES_CONST_REQUISITE_SECTION_TABLE6 SYSRES_CONST_REQUISITE_SECTION_TABLE7 SYSRES_CONST_REQUISITE_SECTION_TABLE8 SYSRES_CONST_REQUISITE_SECTION_TABLE9 SYSRES_CONST_REQUISITES_PSEUDOREFERENCE_REQUISITE_NUMBER_REQUISITE_CODE SYSRES_CONST_RIGHT_ALIGNMENT_CODE SYSRES_CONST_ROLES_REFERENCE_CODE SYSRES_CONST_ROUTE_STEP_AFTER_RUS SYSRES_CONST_ROUTE_STEP_AND_CONDITION_RUS SYSRES_CONST_ROUTE_STEP_OR_CONDITION_RUS SYSRES_CONST_ROUTE_TYPE_COMPLEX SYSRES_CONST_ROUTE_TYPE_PARALLEL SYSRES_CONST_ROUTE_TYPE_SERIAL SYSRES_CONST_SBDATASETDESC_NEGATIVE_VALUE SYSRES_CONST_SBDATASETDESC_POSITIVE_VALUE SYSRES_CONST_SBVIEWSDESC_POSITIVE_VALUE SYSRES_CONST_SCRIPT_BLOCK_DESCRIPTION SYSRES_CONST_SEARCH_BY_TEXT_REQUISITE_CODE SYSRES_CONST_SEARCHES_COMPONENT_CONTENT SYSRES_CONST_SEARCHES_CRITERIA_ACTION_NAME SYSRES_CONST_SEARCHES_EDOC_CONTENT SYSRES_CONST_SEARCHES_FOLDER_CONTENT SYSRES_CONST_SEARCHES_JOB_CONTENT SYSRES_CONST_SEARCHES_REFERENCE_CODE SYSRES_CONST_SEARCHES_TASK_CONTENT SYSRES_CONST_SECOND_CHAR SYSRES_CONST_SECTION_REQUISITE_ACTIONS_VALUE SYSRES_CONST_SECTION_REQUISITE_CARD_VALUE SYSRES_CONST_SECTION_REQUISITE_CODE SYSRES_CONST_SECTION_REQUISITE_DETAIL_1_VALUE SYSRES_CONST_SECTION_REQUISITE_DETAIL_2_VALUE SYSRES_CONST_SECTION_REQUISITE_DETAIL_3_VALUE SYSRES_CONST_SECTION_REQUISITE_DETAIL_4_VALUE SYSRES_CONST_SECTION_REQUISITE_DETAIL_5_VALUE SYSRES_CONST_SECTION_REQUISITE_DETAIL_6_VALUE SYSRES_CONST_SELECT_REFERENCE_MODE_NAME SYSRES_CONST_SELECT_TYPE_SELECTABLE SYSRES_CONST_SELECT_TYPE_SELECTABLE_ONLY_CHILD SYSRES_CONST_SELECT_TYPE_SELECTABLE_WITH_CHILD SYSRES_CONST_SELECT_TYPE_UNSLECTABLE SYSRES_CONST_SERVER_TYPE_MAIN SYSRES_CONST_SERVICE_USER_CATEGORY_FIELD_VALUE SYSRES_CONST_SETTINGS_USER_REQUISITE_CODE SYSRES_CONST_SIGNATURE_AND_ENCODE_CERTIFICATE_TYPE_CODE SYSRES_CONST_SIGNATURE_CERTIFICATE_TYPE_CODE SYSRES_CONST_SINGULAR_TITLE_REQUISITE_CODE SYSRES_CONST_SQL_SERVER_AUTHENTIFICATION_FLAG_VALUE_CODE SYSRES_CONST_SQL_SERVER_ENCODE_AUTHENTIFICATION_FLAG_VALUE_CODE SYSRES_CONST_STANDART_ROUTE_REFERENCE_CODE SYSRES_CONST_STANDART_ROUTE_REFERENCE_COMMENT_REQUISITE_CODE SYSRES_CONST_STANDART_ROUTES_GROUPS_REFERENCE_CODE SYSRES_CONST_STATE_REQ_NAME SYSRES_CONST_STATE_REQUISITE_ACTIVE_VALUE SYSRES_CONST_STATE_REQUISITE_CLOSED_VALUE SYSRES_CONST_STATE_REQUISITE_CODE SYSRES_CONST_STATIC_ROLE_TYPE_CODE SYSRES_CONST_STATUS_PLAN_DEFAULT_VALUE SYSRES_CONST_STATUS_VALUE_AUTOCLEANING SYSRES_CONST_STATUS_VALUE_BLUE_SQUARE SYSRES_CONST_STATUS_VALUE_COMPLETE SYSRES_CONST_STATUS_VALUE_GREEN_SQUARE SYSRES_CONST_STATUS_VALUE_ORANGE_SQUARE SYSRES_CONST_STATUS_VALUE_PURPLE_SQUARE SYSRES_CONST_STATUS_VALUE_RED_SQUARE SYSRES_CONST_STATUS_VALUE_SUSPEND SYSRES_CONST_STATUS_VALUE_YELLOW_SQUARE SYSRES_CONST_STDROUTE_SHOW_TO_USERS_REQUISITE_CODE SYSRES_CONST_STORAGE_TYPE_FILE SYSRES_CONST_STORAGE_TYPE_SQL_SERVER SYSRES_CONST_STR_REQUISITE SYSRES_CONST_STRIKEOUT_LIFE_CYCLE_STAGE_DRAW_STYLE SYSRES_CONST_STRING_FORMAT_LEFT_ALIGN_CHAR SYSRES_CONST_STRING_FORMAT_RIGHT_ALIGN_CHAR SYSRES_CONST_STRING_REQUISITE_CODE SYSRES_CONST_STRING_REQUISITE_TYPE SYSRES_CONST_STRING_TYPE_CHAR SYSRES_CONST_SUBSTITUTES_PSEUDOREFERENCE_CODE SYSRES_CONST_SUBTASK_BLOCK_DESCRIPTION SYSRES_CONST_SYSTEM_SETTING_CURRENT_USER_PARAM_VALUE SYSRES_CONST_SYSTEM_SETTING_EMPTY_VALUE_PARAM_VALUE SYSRES_CONST_SYSTEM_VERSION_COMMENT SYSRES_CONST_TASK_ACCESS_TYPE_ALL SYSRES_CONST_TASK_ACCESS_TYPE_ALL_MEMBERS SYSRES_CONST_TASK_ACCESS_TYPE_MANUAL SYSRES_CONST_TASK_ENCODE_TYPE_CERTIFICATION SYSRES_CONST_TASK_ENCODE_TYPE_CERTIFICATION_AND_PASSWORD SYSRES_CONST_TASK_ENCODE_TYPE_NONE SYSRES_CONST_TASK_ENCODE_TYPE_PASSWORD SYSRES_CONST_TASK_ROUTE_ALL_CONDITION SYSRES_CONST_TASK_ROUTE_AND_CONDITION SYSRES_CONST_TASK_ROUTE_OR_CONDITION SYSRES_CONST_TASK_STATE_ABORTED SYSRES_CONST_TASK_STATE_COMPLETE SYSRES_CONST_TASK_STATE_CONTINUED SYSRES_CONST_TASK_STATE_CONTROL SYSRES_CONST_TASK_STATE_INIT SYSRES_CONST_TASK_STATE_WORKING SYSRES_CONST_TASK_TITLE SYSRES_CONST_TASK_TYPES_GROUPS_REFERENCE_CODE SYSRES_CONST_TASK_TYPES_REFERENCE_CODE SYSRES_CONST_TEMPLATES_REFERENCE_CODE SYSRES_CONST_TEST_DATE_REQUISITE_NAME SYSRES_CONST_TEST_DEV_DATABASE_NAME SYSRES_CONST_TEST_DEV_SYSTEM_CODE SYSRES_CONST_TEST_EDMS_DATABASE_NAME SYSRES_CONST_TEST_EDMS_MAIN_CODE SYSRES_CONST_TEST_EDMS_MAIN_DB_NAME SYSRES_CONST_TEST_EDMS_SECOND_CODE SYSRES_CONST_TEST_EDMS_SECOND_DB_NAME SYSRES_CONST_TEST_EDMS_SYSTEM_CODE SYSRES_CONST_TEST_NUMERIC_REQUISITE_NAME SYSRES_CONST_TEXT_REQUISITE SYSRES_CONST_TEXT_REQUISITE_CODE SYSRES_CONST_TEXT_REQUISITE_TYPE SYSRES_CONST_TEXT_TYPE_CHAR SYSRES_CONST_TYPE_CODE_REQUISITE_CODE SYSRES_CONST_TYPE_REQUISITE_CODE SYSRES_CONST_UNDEFINED_LIFE_CYCLE_STAGE_FONT_COLOR SYSRES_CONST_UNITS_SECTION_ID_REQUISITE_CODE SYSRES_CONST_UNITS_SECTION_REQUISITE_CODE SYSRES_CONST_UNOPERATING_RECORD_FLAG_VALUE_CODE SYSRES_CONST_UNSTORED_DATA_REQUISITE_CODE SYSRES_CONST_UNSTORED_DATA_REQUISITE_NAME SYSRES_CONST_USE_ACCESS_TYPE_CODE SYSRES_CONST_USE_ACCESS_TYPE_NAME SYSRES_CONST_USER_ACCOUNT_TYPE_VALUE_CODE SYSRES_CONST_USER_ADDITIONAL_INFORMATION_REQUISITE_CODE SYSRES_CONST_USER_AND_GROUP_ID_FROM_PSEUDOREFERENCE_REQUISITE_CODE SYSRES_CONST_USER_CATEGORY_NORMAL SYSRES_CONST_USER_CERTIFICATE_REQUISITE_CODE SYSRES_CONST_USER_CERTIFICATE_STATE_REQUISITE_CODE SYSRES_CONST_USER_CERTIFICATE_SUBJECT_NAME_REQUISITE_CODE SYSRES_CONST_USER_CERTIFICATE_THUMBPRINT_REQUISITE_CODE SYSRES_CONST_USER_COMMON_CATEGORY SYSRES_CONST_USER_COMMON_CATEGORY_CODE SYSRES_CONST_USER_FULL_NAME_REQUISITE_CODE SYSRES_CONST_USER_GROUP_TYPE_REQUISITE_CODE SYSRES_CONST_USER_LOGIN_REQUISITE_CODE SYSRES_CONST_USER_REMOTE_CONTROLLER_REQUISITE_CODE SYSRES_CONST_USER_REMOTE_SYSTEM_REQUISITE_CODE SYSRES_CONST_USER_RIGHTS_T_REQUISITE_CODE SYSRES_CONST_USER_SERVER_NAME_REQUISITE_CODE SYSRES_CONST_USER_SERVICE_CATEGORY SYSRES_CONST_USER_SERVICE_CATEGORY_CODE SYSRES_CONST_USER_STATUS_ADMINISTRATOR_CODE SYSRES_CONST_USER_STATUS_ADMINISTRATOR_NAME SYSRES_CONST_USER_STATUS_DEVELOPER_CODE SYSRES_CONST_USER_STATUS_DEVELOPER_NAME SYSRES_CONST_USER_STATUS_DISABLED_CODE SYSRES_CONST_USER_STATUS_DISABLED_NAME SYSRES_CONST_USER_STATUS_SYSTEM_DEVELOPER_CODE SYSRES_CONST_USER_STATUS_USER_CODE SYSRES_CONST_USER_STATUS_USER_NAME SYSRES_CONST_USER_STATUS_USER_NAME_DEPRECATED SYSRES_CONST_USER_TYPE_FIELD_VALUE_USER SYSRES_CONST_USER_TYPE_REQUISITE_CODE SYSRES_CONST_USERS_CONTROLLER_REQUISITE_CODE SYSRES_CONST_USERS_IS_MAIN_SERVER_REQUISITE_CODE SYSRES_CONST_USERS_REFERENCE_CODE SYSRES_CONST_USERS_REGISTRATION_CERTIFICATES_ACTION_NAME SYSRES_CONST_USERS_REQUISITE_CODE SYSRES_CONST_USERS_SYSTEM_REQUISITE_CODE SYSRES_CONST_USERS_USER_ACCESS_RIGHTS_TYPR_REQUISITE_CODE SYSRES_CONST_USERS_USER_AUTHENTICATION_REQUISITE_CODE SYSRES_CONST_USERS_USER_COMPONENT_REQUISITE_CODE SYSRES_CONST_USERS_USER_GROUP_REQUISITE_CODE SYSRES_CONST_USERS_VIEW_CERTIFICATES_ACTION_NAME SYSRES_CONST_VIEW_DEFAULT_CODE SYSRES_CONST_VIEW_DEFAULT_NAME SYSRES_CONST_VIEWER_REQUISITE_CODE SYSRES_CONST_WAITING_BLOCK_DESCRIPTION SYSRES_CONST_WIZARD_FORM_LABEL_TEST_STRING  SYSRES_CONST_WIZARD_QUERY_PARAM_HEIGHT_ETALON_STRING SYSRES_CONST_WIZARD_REFERENCE_COMMENT_REQUISITE_CODE SYSRES_CONST_WORK_RULES_DESCRIPTION_REQUISITE_CODE SYSRES_CONST_WORK_TIME_CALENDAR_REFERENCE_CODE SYSRES_CONST_WORK_WORKFLOW_HARD_ROUTE_TYPE_VALUE SYSRES_CONST_WORK_WORKFLOW_HARD_ROUTE_TYPE_VALUE_CODE SYSRES_CONST_WORK_WORKFLOW_HARD_ROUTE_TYPE_VALUE_CODE_RUS SYSRES_CONST_WORK_WORKFLOW_SOFT_ROUTE_TYPE_VALUE_CODE_RUS SYSRES_CONST_WORKFLOW_ROUTE_TYPR_HARD SYSRES_CONST_WORKFLOW_ROUTE_TYPR_SOFT SYSRES_CONST_XML_ENCODING SYSRES_CONST_XREC_STAT_REQUISITE_CODE SYSRES_CONST_XRECID_FIELD_NAME SYSRES_CONST_YES SYSRES_CONST_YES_NO_2_REQUISITE_CODE SYSRES_CONST_YES_NO_REQUISITE_CODE SYSRES_CONST_YES_NO_T_REF_TYPE_REQUISITE_CODE SYSRES_CONST_YES_PICK_VALUE SYSRES_CONST_YES_VALUE " + "CR FALSE nil NO_VALUE NULL TAB TRUE YES_VALUE " + "ADMINISTRATORS_GROUP_NAME CUSTOMIZERS_GROUP_NAME DEVELOPERS_GROUP_NAME SERVICE_USERS_GROUP_NAME " + "DECISION_BLOCK_FIRST_OPERAND_PROPERTY DECISION_BLOCK_NAME_PROPERTY DECISION_BLOCK_OPERATION_PROPERTY DECISION_BLOCK_RESULT_TYPE_PROPERTY DECISION_BLOCK_SECOND_OPERAND_PROPERTY " + "ANY_FILE_EXTENTION COMPRESSED_DOCUMENT_EXTENSION EXTENDED_DOCUMENT_EXTENSION SHORT_COMPRESSED_DOCUMENT_EXTENSION SHORT_EXTENDED_DOCUMENT_EXTENSION " + "JOB_BLOCK_ABORT_DEADLINE_PROPERTY JOB_BLOCK_AFTER_FINISH_EVENT JOB_BLOCK_AFTER_QUERY_PARAMETERS_EVENT JOB_BLOCK_ATTACHMENT_PROPERTY JOB_BLOCK_ATTACHMENTS_RIGHTS_GROUP_PROPERTY JOB_BLOCK_ATTACHMENTS_RIGHTS_TYPE_PROPERTY JOB_BLOCK_BEFORE_QUERY_PARAMETERS_EVENT JOB_BLOCK_BEFORE_START_EVENT JOB_BLOCK_CREATED_JOBS_PROPERTY JOB_BLOCK_DEADLINE_PROPERTY JOB_BLOCK_EXECUTION_RESULTS_PROPERTY JOB_BLOCK_IS_PARALLEL_PROPERTY JOB_BLOCK_IS_RELATIVE_ABORT_DEADLINE_PROPERTY JOB_BLOCK_IS_RELATIVE_DEADLINE_PROPERTY JOB_BLOCK_JOB_TEXT_PROPERTY JOB_BLOCK_NAME_PROPERTY JOB_BLOCK_NEED_SIGN_ON_PERFORM_PROPERTY JOB_BLOCK_PERFORMER_PROPERTY JOB_BLOCK_RELATIVE_ABORT_DEADLINE_TYPE_PROPERTY JOB_BLOCK_RELATIVE_DEADLINE_TYPE_PROPERTY JOB_BLOCK_SUBJECT_PROPERTY " + "ENGLISH_LANGUAGE_CODE RUSSIAN_LANGUAGE_CODE " + "smHidden smMaximized smMinimized smNormal wmNo wmYes " + "COMPONENT_TOKEN_LINK_KIND DOCUMENT_LINK_KIND EDOCUMENT_LINK_KIND FOLDER_LINK_KIND JOB_LINK_KIND REFERENCE_LINK_KIND TASK_LINK_KIND " + "COMPONENT_TOKEN_LOCK_TYPE EDOCUMENT_VERSION_LOCK_TYPE " + "MONITOR_BLOCK_AFTER_FINISH_EVENT MONITOR_BLOCK_BEFORE_START_EVENT MONITOR_BLOCK_DEADLINE_PROPERTY MONITOR_BLOCK_INTERVAL_PROPERTY MONITOR_BLOCK_INTERVAL_TYPE_PROPERTY MONITOR_BLOCK_IS_RELATIVE_DEADLINE_PROPERTY MONITOR_BLOCK_NAME_PROPERTY MONITOR_BLOCK_RELATIVE_DEADLINE_TYPE_PROPERTY MONITOR_BLOCK_SEARCH_SCRIPT_PROPERTY " + "NOTICE_BLOCK_AFTER_FINISH_EVENT NOTICE_BLOCK_ATTACHMENT_PROPERTY NOTICE_BLOCK_ATTACHMENTS_RIGHTS_GROUP_PROPERTY NOTICE_BLOCK_ATTACHMENTS_RIGHTS_TYPE_PROPERTY NOTICE_BLOCK_BEFORE_START_EVENT NOTICE_BLOCK_CREATED_NOTICES_PROPERTY NOTICE_BLOCK_DEADLINE_PROPERTY NOTICE_BLOCK_IS_RELATIVE_DEADLINE_PROPERTY NOTICE_BLOCK_NAME_PROPERTY NOTICE_BLOCK_NOTICE_TEXT_PROPERTY NOTICE_BLOCK_PERFORMER_PROPERTY NOTICE_BLOCK_RELATIVE_DEADLINE_TYPE_PROPERTY NOTICE_BLOCK_SUBJECT_PROPERTY " + "dseAfterCancel dseAfterClose dseAfterDelete dseAfterDeleteOutOfTransaction dseAfterInsert dseAfterOpen dseAfterScroll dseAfterUpdate dseAfterUpdateOutOfTransaction dseBeforeCancel dseBeforeClose dseBeforeDelete dseBeforeDetailUpdate dseBeforeInsert dseBeforeOpen dseBeforeUpdate dseOnAnyRequisiteChange dseOnCloseRecord dseOnDeleteError dseOnOpenRecord dseOnPrepareUpdate dseOnUpdateError dseOnUpdateRatifiedRecord dseOnValidDelete dseOnValidUpdate reOnChange reOnChangeValues SELECTION_BEGIN_ROUTE_EVENT SELECTION_END_ROUTE_EVENT " + "CURRENT_PERIOD_IS_REQUIRED PREVIOUS_CARD_TYPE_NAME SHOW_RECORD_PROPERTIES_FORM " + "ACCESS_RIGHTS_SETTING_DIALOG_CODE ADMINISTRATOR_USER_CODE ANALYTIC_REPORT_TYPE asrtHideLocal asrtHideRemote CALCULATED_ROLE_TYPE_CODE COMPONENTS_REFERENCE_DEVELOPER_VIEW_CODE DCTS_TEST_PROTOCOLS_FOLDER_PATH E_EDOC_VERSION_ALREADY_APPROVINGLY_SIGNED E_EDOC_VERSION_ALREADY_APPROVINGLY_SIGNED_BY_USER E_EDOC_VERSION_ALREDY_SIGNED E_EDOC_VERSION_ALREDY_SIGNED_BY_USER EDOC_TYPES_CODE_REQUISITE_FIELD_NAME EDOCUMENTS_ALIAS_NAME FILES_FOLDER_PATH FILTER_OPERANDS_DELIMITER FILTER_OPERATIONS_DELIMITER FORMCARD_NAME FORMLIST_NAME GET_EXTENDED_DOCUMENT_EXTENSION_CREATION_MODE GET_EXTENDED_DOCUMENT_EXTENSION_IMPORT_MODE INTEGRATED_REPORT_TYPE IS_BUILDER_APPLICATION_ROLE IS_BUILDER_APPLICATION_ROLE2 IS_BUILDER_USERS ISBSYSDEV LOG_FOLDER_PATH mbCancel mbNo mbNoToAll mbOK mbYes mbYesToAll MEMORY_DATASET_DESRIPTIONS_FILENAME mrNo mrNoToAll mrYes mrYesToAll MULTIPLE_SELECT_DIALOG_CODE NONOPERATING_RECORD_FLAG_FEMININE NONOPERATING_RECORD_FLAG_MASCULINE OPERATING_RECORD_FLAG_FEMININE OPERATING_RECORD_FLAG_MASCULINE PROFILING_SETTINGS_COMMON_SETTINGS_CODE_VALUE PROGRAM_INITIATED_LOOKUP_ACTION ratDelete ratEdit ratInsert REPORT_TYPE REQUIRED_PICK_VALUES_VARIABLE rmCard rmList SBRTE_PROGID_DEV SBRTE_PROGID_RELEASE STATIC_ROLE_TYPE_CODE SUPPRESS_EMPTY_TEMPLATE_CREATION SYSTEM_USER_CODE UPDATE_DIALOG_DATASET USED_IN_OBJECT_HINT_PARAM USER_INITIATED_LOOKUP_ACTION USER_NAME_FORMAT USER_SELECTION_RESTRICTIONS WORKFLOW_TEST_PROTOCOLS_FOLDER_PATH ELS_SUBTYPE_CONTROL_NAME ELS_FOLDER_KIND_CONTROL_NAME REPEAT_PROCESS_CURRENT_OBJECT_EXCEPTION_NAME " + "PRIVILEGE_COMPONENT_FULL_ACCESS PRIVILEGE_DEVELOPMENT_EXPORT PRIVILEGE_DEVELOPMENT_IMPORT PRIVILEGE_DOCUMENT_DELETE PRIVILEGE_ESD PRIVILEGE_FOLDER_DELETE PRIVILEGE_MANAGE_ACCESS_RIGHTS PRIVILEGE_MANAGE_REPLICATION PRIVILEGE_MANAGE_SESSION_SERVER PRIVILEGE_OBJECT_FULL_ACCESS PRIVILEGE_OBJECT_VIEW PRIVILEGE_RESERVE_LICENSE PRIVILEGE_SYSTEM_CUSTOMIZE PRIVILEGE_SYSTEM_DEVELOP PRIVILEGE_SYSTEM_INSTALL PRIVILEGE_TASK_DELETE PRIVILEGE_USER_PLUGIN_SETTINGS_CUSTOMIZE PRIVILEGES_PSEUDOREFERENCE_CODE " + "ACCESS_TYPES_PSEUDOREFERENCE_CODE ALL_AVAILABLE_COMPONENTS_PSEUDOREFERENCE_CODE ALL_AVAILABLE_PRIVILEGES_PSEUDOREFERENCE_CODE ALL_REPLICATE_COMPONENTS_PSEUDOREFERENCE_CODE AVAILABLE_DEVELOPERS_COMPONENTS_PSEUDOREFERENCE_CODE COMPONENTS_PSEUDOREFERENCE_CODE FILTRATER_SETTINGS_CONFLICTS_PSEUDOREFERENCE_CODE GROUPS_PSEUDOREFERENCE_CODE RECEIVE_PROTOCOL_PSEUDOREFERENCE_CODE REFERENCE_REQUISITE_PSEUDOREFERENCE_CODE REFERENCE_REQUISITES_PSEUDOREFERENCE_CODE REFTYPES_PSEUDOREFERENCE_CODE REPLICATION_SEANCES_DIARY_PSEUDOREFERENCE_CODE SEND_PROTOCOL_PSEUDOREFERENCE_CODE SUBSTITUTES_PSEUDOREFERENCE_CODE SYSTEM_SETTINGS_PSEUDOREFERENCE_CODE UNITS_PSEUDOREFERENCE_CODE USERS_PSEUDOREFERENCE_CODE VIEWERS_PSEUDOREFERENCE_CODE " + "CERTIFICATE_TYPE_ENCRYPT CERTIFICATE_TYPE_SIGN CERTIFICATE_TYPE_SIGN_AND_ENCRYPT " + "STORAGE_TYPE_FILE STORAGE_TYPE_NAS_CIFS STORAGE_TYPE_SAPERION STORAGE_TYPE_SQL_SERVER " + "COMPTYPE2_REQUISITE_DOCUMENTS_VALUE COMPTYPE2_REQUISITE_TASKS_VALUE COMPTYPE2_REQUISITE_FOLDERS_VALUE COMPTYPE2_REQUISITE_REFERENCES_VALUE " + "SYSREQ_CODE SYSREQ_COMPTYPE2 SYSREQ_CONST_AVAILABLE_FOR_WEB SYSREQ_CONST_COMMON_CODE SYSREQ_CONST_COMMON_VALUE SYSREQ_CONST_FIRM_CODE SYSREQ_CONST_FIRM_STATUS SYSREQ_CONST_FIRM_VALUE SYSREQ_CONST_SERVER_STATUS SYSREQ_CONTENTS SYSREQ_DATE_OPEN SYSREQ_DATE_CLOSE SYSREQ_DESCRIPTION SYSREQ_DESCRIPTION_LOCALIZE_ID SYSREQ_DOUBLE SYSREQ_EDOC_ACCESS_TYPE SYSREQ_EDOC_AUTHOR SYSREQ_EDOC_CREATED SYSREQ_EDOC_DELEGATE_RIGHTS_REQUISITE_CODE SYSREQ_EDOC_EDITOR SYSREQ_EDOC_ENCODE_TYPE SYSREQ_EDOC_ENCRYPTION_PLUGIN_NAME SYSREQ_EDOC_ENCRYPTION_PLUGIN_VERSION SYSREQ_EDOC_EXPORT_DATE SYSREQ_EDOC_EXPORTER SYSREQ_EDOC_KIND SYSREQ_EDOC_LIFE_STAGE_NAME SYSREQ_EDOC_LOCKED_FOR_SERVER_CODE SYSREQ_EDOC_MODIFIED SYSREQ_EDOC_NAME SYSREQ_EDOC_NOTE SYSREQ_EDOC_QUALIFIED_ID SYSREQ_EDOC_SESSION_KEY SYSREQ_EDOC_SESSION_KEY_ENCRYPTION_PLUGIN_NAME SYSREQ_EDOC_SESSION_KEY_ENCRYPTION_PLUGIN_VERSION SYSREQ_EDOC_SIGNATURE_TYPE SYSREQ_EDOC_SIGNED SYSREQ_EDOC_STORAGE SYSREQ_EDOC_STORAGES_ARCHIVE_STORAGE SYSREQ_EDOC_STORAGES_CHECK_RIGHTS SYSREQ_EDOC_STORAGES_COMPUTER_NAME SYSREQ_EDOC_STORAGES_EDIT_IN_STORAGE SYSREQ_EDOC_STORAGES_EXECUTIVE_STORAGE SYSREQ_EDOC_STORAGES_FUNCTION SYSREQ_EDOC_STORAGES_INITIALIZED SYSREQ_EDOC_STORAGES_LOCAL_PATH SYSREQ_EDOC_STORAGES_SAPERION_DATABASE_NAME SYSREQ_EDOC_STORAGES_SEARCH_BY_TEXT SYSREQ_EDOC_STORAGES_SERVER_NAME SYSREQ_EDOC_STORAGES_SHARED_SOURCE_NAME SYSREQ_EDOC_STORAGES_TYPE SYSREQ_EDOC_TEXT_MODIFIED SYSREQ_EDOC_TYPE_ACT_CODE SYSREQ_EDOC_TYPE_ACT_DESCRIPTION SYSREQ_EDOC_TYPE_ACT_DESCRIPTION_LOCALIZE_ID SYSREQ_EDOC_TYPE_ACT_ON_EXECUTE SYSREQ_EDOC_TYPE_ACT_ON_EXECUTE_EXISTS SYSREQ_EDOC_TYPE_ACT_SECTION SYSREQ_EDOC_TYPE_ADD_PARAMS SYSREQ_EDOC_TYPE_COMMENT SYSREQ_EDOC_TYPE_EVENT_TEXT SYSREQ_EDOC_TYPE_NAME_IN_SINGULAR SYSREQ_EDOC_TYPE_NAME_IN_SINGULAR_LOCALIZE_ID SYSREQ_EDOC_TYPE_NAME_LOCALIZE_ID SYSREQ_EDOC_TYPE_NUMERATION_METHOD SYSREQ_EDOC_TYPE_PSEUDO_REQUISITE_CODE SYSREQ_EDOC_TYPE_REQ_CODE SYSREQ_EDOC_TYPE_REQ_DESCRIPTION SYSREQ_EDOC_TYPE_REQ_DESCRIPTION_LOCALIZE_ID SYSREQ_EDOC_TYPE_REQ_IS_LEADING SYSREQ_EDOC_TYPE_REQ_IS_REQUIRED SYSREQ_EDOC_TYPE_REQ_NUMBER SYSREQ_EDOC_TYPE_REQ_ON_CHANGE SYSREQ_EDOC_TYPE_REQ_ON_CHANGE_EXISTS SYSREQ_EDOC_TYPE_REQ_ON_SELECT SYSREQ_EDOC_TYPE_REQ_ON_SELECT_KIND SYSREQ_EDOC_TYPE_REQ_SECTION SYSREQ_EDOC_TYPE_VIEW_CARD SYSREQ_EDOC_TYPE_VIEW_CODE SYSREQ_EDOC_TYPE_VIEW_COMMENT SYSREQ_EDOC_TYPE_VIEW_IS_MAIN SYSREQ_EDOC_TYPE_VIEW_NAME SYSREQ_EDOC_TYPE_VIEW_NAME_LOCALIZE_ID SYSREQ_EDOC_VERSION_AUTHOR SYSREQ_EDOC_VERSION_CRC SYSREQ_EDOC_VERSION_DATA SYSREQ_EDOC_VERSION_EDITOR SYSREQ_EDOC_VERSION_EXPORT_DATE SYSREQ_EDOC_VERSION_EXPORTER SYSREQ_EDOC_VERSION_HIDDEN SYSREQ_EDOC_VERSION_LIFE_STAGE SYSREQ_EDOC_VERSION_MODIFIED SYSREQ_EDOC_VERSION_NOTE SYSREQ_EDOC_VERSION_SIGNATURE_TYPE SYSREQ_EDOC_VERSION_SIGNED SYSREQ_EDOC_VERSION_SIZE SYSREQ_EDOC_VERSION_SOURCE SYSREQ_EDOC_VERSION_TEXT_MODIFIED SYSREQ_EDOCKIND_DEFAULT_VERSION_STATE_CODE SYSREQ_FOLDER_KIND SYSREQ_FUNC_CATEGORY SYSREQ_FUNC_COMMENT SYSREQ_FUNC_GROUP SYSREQ_FUNC_GROUP_COMMENT SYSREQ_FUNC_GROUP_NUMBER SYSREQ_FUNC_HELP SYSREQ_FUNC_PARAM_DEF_VALUE SYSREQ_FUNC_PARAM_IDENT SYSREQ_FUNC_PARAM_NUMBER SYSREQ_FUNC_PARAM_TYPE SYSREQ_FUNC_TEXT SYSREQ_GROUP_CATEGORY SYSREQ_ID SYSREQ_LAST_UPDATE SYSREQ_LEADER_REFERENCE SYSREQ_LINE_NUMBER SYSREQ_MAIN_RECORD_ID SYSREQ_NAME SYSREQ_NAME_LOCALIZE_ID SYSREQ_NOTE SYSREQ_ORIGINAL_RECORD SYSREQ_OUR_FIRM SYSREQ_PROFILING_SETTINGS_BATCH_LOGING SYSREQ_PROFILING_SETTINGS_BATCH_SIZE SYSREQ_PROFILING_SETTINGS_PROFILING_ENABLED SYSREQ_PROFILING_SETTINGS_SQL_PROFILING_ENABLED SYSREQ_PROFILING_SETTINGS_START_LOGGED SYSREQ_RECORD_STATUS SYSREQ_REF_REQ_FIELD_NAME SYSREQ_REF_REQ_FORMAT SYSREQ_REF_REQ_GENERATED SYSREQ_REF_REQ_LENGTH SYSREQ_REF_REQ_PRECISION SYSREQ_REF_REQ_REFERENCE SYSREQ_REF_REQ_SECTION SYSREQ_REF_REQ_STORED SYSREQ_REF_REQ_TOKENS SYSREQ_REF_REQ_TYPE SYSREQ_REF_REQ_VIEW SYSREQ_REF_TYPE_ACT_CODE SYSREQ_REF_TYPE_ACT_DESCRIPTION SYSREQ_REF_TYPE_ACT_DESCRIPTION_LOCALIZE_ID SYSREQ_REF_TYPE_ACT_ON_EXECUTE SYSREQ_REF_TYPE_ACT_ON_EXECUTE_EXISTS SYSREQ_REF_TYPE_ACT_SECTION SYSREQ_REF_TYPE_ADD_PARAMS SYSREQ_REF_TYPE_COMMENT SYSREQ_REF_TYPE_COMMON_SETTINGS SYSREQ_REF_TYPE_DISPLAY_REQUISITE_NAME SYSREQ_REF_TYPE_EVENT_TEXT SYSREQ_REF_TYPE_MAIN_LEADING_REF SYSREQ_REF_TYPE_NAME_IN_SINGULAR SYSREQ_REF_TYPE_NAME_IN_SINGULAR_LOCALIZE_ID SYSREQ_REF_TYPE_NAME_LOCALIZE_ID SYSREQ_REF_TYPE_NUMERATION_METHOD SYSREQ_REF_TYPE_REQ_CODE SYSREQ_REF_TYPE_REQ_DESCRIPTION SYSREQ_REF_TYPE_REQ_DESCRIPTION_LOCALIZE_ID SYSREQ_REF_TYPE_REQ_IS_CONTROL SYSREQ_REF_TYPE_REQ_IS_FILTER SYSREQ_REF_TYPE_REQ_IS_LEADING SYSREQ_REF_TYPE_REQ_IS_REQUIRED SYSREQ_REF_TYPE_REQ_NUMBER SYSREQ_REF_TYPE_REQ_ON_CHANGE SYSREQ_REF_TYPE_REQ_ON_CHANGE_EXISTS SYSREQ_REF_TYPE_REQ_ON_SELECT SYSREQ_REF_TYPE_REQ_ON_SELECT_KIND SYSREQ_REF_TYPE_REQ_SECTION SYSREQ_REF_TYPE_VIEW_CARD SYSREQ_REF_TYPE_VIEW_CODE SYSREQ_REF_TYPE_VIEW_COMMENT SYSREQ_REF_TYPE_VIEW_IS_MAIN SYSREQ_REF_TYPE_VIEW_NAME SYSREQ_REF_TYPE_VIEW_NAME_LOCALIZE_ID SYSREQ_REFERENCE_TYPE_ID SYSREQ_STATE SYSREQ_STATЕ SYSREQ_SYSTEM_SETTINGS_VALUE SYSREQ_TYPE SYSREQ_UNIT SYSREQ_UNIT_ID SYSREQ_USER_GROUPS_GROUP_FULL_NAME SYSREQ_USER_GROUPS_GROUP_NAME SYSREQ_USER_GROUPS_GROUP_SERVER_NAME SYSREQ_USERS_ACCESS_RIGHTS SYSREQ_USERS_AUTHENTICATION SYSREQ_USERS_CATEGORY SYSREQ_USERS_COMPONENT SYSREQ_USERS_COMPONENT_USER_IS_PUBLIC SYSREQ_USERS_DOMAIN SYSREQ_USERS_FULL_USER_NAME SYSREQ_USERS_GROUP SYSREQ_USERS_IS_MAIN_SERVER SYSREQ_USERS_LOGIN SYSREQ_USERS_REFERENCE_USER_IS_PUBLIC SYSREQ_USERS_STATUS SYSREQ_USERS_USER_CERTIFICATE SYSREQ_USERS_USER_CERTIFICATE_INFO SYSREQ_USERS_USER_CERTIFICATE_PLUGIN_NAME SYSREQ_USERS_USER_CERTIFICATE_PLUGIN_VERSION SYSREQ_USERS_USER_CERTIFICATE_STATE SYSREQ_USERS_USER_CERTIFICATE_SUBJECT_NAME SYSREQ_USERS_USER_CERTIFICATE_THUMBPRINT SYSREQ_USERS_USER_DEFAULT_CERTIFICATE SYSREQ_USERS_USER_DESCRIPTION SYSREQ_USERS_USER_GLOBAL_NAME SYSREQ_USERS_USER_LOGIN SYSREQ_USERS_USER_MAIN_SERVER SYSREQ_USERS_USER_TYPE SYSREQ_WORK_RULES_FOLDER_ID " + "RESULT_VAR_NAME RESULT_VAR_NAME_ENG " + "AUTO_NUMERATION_RULE_ID CANT_CHANGE_ID_REQUISITE_RULE_ID CANT_CHANGE_OURFIRM_REQUISITE_RULE_ID CHECK_CHANGING_REFERENCE_RECORD_USE_RULE_ID CHECK_CODE_REQUISITE_RULE_ID CHECK_DELETING_REFERENCE_RECORD_USE_RULE_ID CHECK_FILTRATER_CHANGES_RULE_ID CHECK_RECORD_INTERVAL_RULE_ID CHECK_REFERENCE_INTERVAL_RULE_ID CHECK_REQUIRED_DATA_FULLNESS_RULE_ID CHECK_REQUIRED_REQUISITES_FULLNESS_RULE_ID MAKE_RECORD_UNRATIFIED_RULE_ID RESTORE_AUTO_NUMERATION_RULE_ID SET_FIRM_CONTEXT_FROM_RECORD_RULE_ID SET_FIRST_RECORD_IN_LIST_FORM_RULE_ID SET_IDSPS_VALUE_RULE_ID SET_NEXT_CODE_VALUE_RULE_ID SET_OURFIRM_BOUNDS_RULE_ID SET_OURFIRM_REQUISITE_RULE_ID " + "SCRIPT_BLOCK_AFTER_FINISH_EVENT SCRIPT_BLOCK_BEFORE_START_EVENT SCRIPT_BLOCK_EXECUTION_RESULTS_PROPERTY SCRIPT_BLOCK_NAME_PROPERTY SCRIPT_BLOCK_SCRIPT_PROPERTY " + "SUBTASK_BLOCK_ABORT_DEADLINE_PROPERTY SUBTASK_BLOCK_AFTER_FINISH_EVENT SUBTASK_BLOCK_ASSIGN_PARAMS_EVENT SUBTASK_BLOCK_ATTACHMENTS_PROPERTY SUBTASK_BLOCK_ATTACHMENTS_RIGHTS_GROUP_PROPERTY SUBTASK_BLOCK_ATTACHMENTS_RIGHTS_TYPE_PROPERTY SUBTASK_BLOCK_BEFORE_START_EVENT SUBTASK_BLOCK_CREATED_TASK_PROPERTY SUBTASK_BLOCK_CREATION_EVENT SUBTASK_BLOCK_DEADLINE_PROPERTY SUBTASK_BLOCK_IMPORTANCE_PROPERTY SUBTASK_BLOCK_INITIATOR_PROPERTY SUBTASK_BLOCK_IS_RELATIVE_ABORT_DEADLINE_PROPERTY SUBTASK_BLOCK_IS_RELATIVE_DEADLINE_PROPERTY SUBTASK_BLOCK_JOBS_TYPE_PROPERTY SUBTASK_BLOCK_NAME_PROPERTY SUBTASK_BLOCK_PARALLEL_ROUTE_PROPERTY SUBTASK_BLOCK_PERFORMERS_PROPERTY SUBTASK_BLOCK_RELATIVE_ABORT_DEADLINE_TYPE_PROPERTY SUBTASK_BLOCK_RELATIVE_DEADLINE_TYPE_PROPERTY SUBTASK_BLOCK_REQUIRE_SIGN_PROPERTY SUBTASK_BLOCK_STANDARD_ROUTE_PROPERTY SUBTASK_BLOCK_START_EVENT SUBTASK_BLOCK_STEP_CONTROL_PROPERTY SUBTASK_BLOCK_SUBJECT_PROPERTY SUBTASK_BLOCK_TASK_CONTROL_PROPERTY SUBTASK_BLOCK_TEXT_PROPERTY SUBTASK_BLOCK_UNLOCK_ATTACHMENTS_ON_STOP_PROPERTY SUBTASK_BLOCK_USE_STANDARD_ROUTE_PROPERTY SUBTASK_BLOCK_WAIT_FOR_TASK_COMPLETE_PROPERTY " + "SYSCOMP_CONTROL_JOBS SYSCOMP_FOLDERS SYSCOMP_JOBS SYSCOMP_NOTICES SYSCOMP_TASKS " + "SYSDLG_CREATE_EDOCUMENT SYSDLG_CREATE_EDOCUMENT_VERSION SYSDLG_CURRENT_PERIOD SYSDLG_EDIT_FUNCTION_HELP SYSDLG_EDOCUMENT_KINDS_FOR_TEMPLATE SYSDLG_EXPORT_MULTIPLE_EDOCUMENTS SYSDLG_EXPORT_SINGLE_EDOCUMENT SYSDLG_IMPORT_EDOCUMENT SYSDLG_MULTIPLE_SELECT SYSDLG_SETUP_ACCESS_RIGHTS SYSDLG_SETUP_DEFAULT_RIGHTS SYSDLG_SETUP_FILTER_CONDITION SYSDLG_SETUP_SIGN_RIGHTS SYSDLG_SETUP_TASK_OBSERVERS SYSDLG_SETUP_TASK_ROUTE SYSDLG_SETUP_USERS_LIST SYSDLG_SIGN_EDOCUMENT SYSDLG_SIGN_MULTIPLE_EDOCUMENTS " + "SYSREF_ACCESS_RIGHTS_TYPES SYSREF_ADMINISTRATION_HISTORY SYSREF_ALL_AVAILABLE_COMPONENTS SYSREF_ALL_AVAILABLE_PRIVILEGES SYSREF_ALL_REPLICATING_COMPONENTS SYSREF_AVAILABLE_DEVELOPERS_COMPONENTS SYSREF_CALENDAR_EVENTS SYSREF_COMPONENT_TOKEN_HISTORY SYSREF_COMPONENT_TOKENS SYSREF_COMPONENTS SYSREF_CONSTANTS SYSREF_DATA_RECEIVE_PROTOCOL SYSREF_DATA_SEND_PROTOCOL SYSREF_DIALOGS SYSREF_DIALOGS_REQUISITES SYSREF_EDITORS SYSREF_EDOC_CARDS SYSREF_EDOC_TYPES SYSREF_EDOCUMENT_CARD_REQUISITES SYSREF_EDOCUMENT_CARD_TYPES SYSREF_EDOCUMENT_CARD_TYPES_REFERENCE SYSREF_EDOCUMENT_CARDS SYSREF_EDOCUMENT_HISTORY SYSREF_EDOCUMENT_KINDS SYSREF_EDOCUMENT_REQUISITES SYSREF_EDOCUMENT_SIGNATURES SYSREF_EDOCUMENT_TEMPLATES SYSREF_EDOCUMENT_TEXT_STORAGES SYSREF_EDOCUMENT_VIEWS SYSREF_FILTERER_SETUP_CONFLICTS SYSREF_FILTRATER_SETTING_CONFLICTS SYSREF_FOLDER_HISTORY SYSREF_FOLDERS SYSREF_FUNCTION_GROUPS SYSREF_FUNCTION_PARAMS SYSREF_FUNCTIONS SYSREF_JOB_HISTORY SYSREF_LINKS SYSREF_LOCALIZATION_DICTIONARY SYSREF_LOCALIZATION_LANGUAGES SYSREF_MODULES SYSREF_PRIVILEGES SYSREF_RECORD_HISTORY SYSREF_REFERENCE_REQUISITES SYSREF_REFERENCE_TYPE_VIEWS SYSREF_REFERENCE_TYPES SYSREF_REFERENCES SYSREF_REFERENCES_REQUISITES SYSREF_REMOTE_SERVERS SYSREF_REPLICATION_SESSIONS_LOG SYSREF_REPLICATION_SESSIONS_PROTOCOL SYSREF_REPORTS SYSREF_ROLES SYSREF_ROUTE_BLOCK_GROUPS SYSREF_ROUTE_BLOCKS SYSREF_SCRIPTS SYSREF_SEARCHES SYSREF_SERVER_EVENTS SYSREF_SERVER_EVENTS_HISTORY SYSREF_STANDARD_ROUTE_GROUPS SYSREF_STANDARD_ROUTES SYSREF_STATUSES SYSREF_SYSTEM_SETTINGS SYSREF_TASK_HISTORY SYSREF_TASK_KIND_GROUPS SYSREF_TASK_KINDS SYSREF_TASK_RIGHTS SYSREF_TASK_SIGNATURES SYSREF_TASKS SYSREF_UNITS SYSREF_USER_GROUPS SYSREF_USER_GROUPS_REFERENCE SYSREF_USER_SUBSTITUTION SYSREF_USERS SYSREF_USERS_REFERENCE SYSREF_VIEWERS SYSREF_WORKING_TIME_CALENDARS " + "ACCESS_RIGHTS_TABLE_NAME EDMS_ACCESS_TABLE_NAME EDOC_TYPES_TABLE_NAME " + "TEST_DEV_DB_NAME TEST_DEV_SYSTEM_CODE TEST_EDMS_DB_NAME TEST_EDMS_MAIN_CODE TEST_EDMS_MAIN_DB_NAME TEST_EDMS_SECOND_CODE TEST_EDMS_SECOND_DB_NAME TEST_EDMS_SYSTEM_CODE TEST_ISB5_MAIN_CODE TEST_ISB5_SECOND_CODE TEST_SQL_SERVER_2005_NAME TEST_SQL_SERVER_NAME " + "ATTENTION_CAPTION cbsCommandLinks cbsDefault CONFIRMATION_CAPTION ERROR_CAPTION INFORMATION_CAPTION mrCancel mrOk " + "EDOC_VERSION_ACTIVE_STAGE_CODE EDOC_VERSION_DESIGN_STAGE_CODE EDOC_VERSION_OBSOLETE_STAGE_CODE " + "cpDataEnciphermentEnabled cpDigitalSignatureEnabled cpID cpIssuer cpPluginVersion cpSerial cpSubjectName cpSubjSimpleName cpValidFromDate cpValidToDate " + "ISBL_SYNTAX NO_SYNTAX XML_SYNTAX " + "WAIT_BLOCK_AFTER_FINISH_EVENT WAIT_BLOCK_BEFORE_START_EVENT WAIT_BLOCK_DEADLINE_PROPERTY WAIT_BLOCK_IS_RELATIVE_DEADLINE_PROPERTY WAIT_BLOCK_NAME_PROPERTY WAIT_BLOCK_RELATIVE_DEADLINE_TYPE_PROPERTY " + "SYSRES_COMMON SYSRES_CONST SYSRES_MBFUNC SYSRES_SBDATA SYSRES_SBGUI SYSRES_SBINTF SYSRES_SBREFDSC SYSRES_SQLERRORS SYSRES_SYSCOMP ", oa = "atUser atGroup atRole " + "aemEnabledAlways aemDisabledAlways aemEnabledOnBrowse aemEnabledOnEdit aemDisabledOnBrowseEmpty " + "apBegin apEnd " + "alLeft alRight " + "asmNever asmNoButCustomize asmAsLastTime asmYesButCustomize asmAlways " + "cirCommon cirRevoked " + "ctSignature ctEncode ctSignatureEncode " + "clbUnchecked clbChecked clbGrayed " + "ceISB ceAlways ceNever " + "ctDocument ctReference ctScript ctUnknown ctReport ctDialog ctFunction ctFolder ctEDocument ctTask ctJob ctNotice ctControlJob " + "cfInternal cfDisplay " + "ciUnspecified ciWrite ciRead " + "ckFolder ckEDocument ckTask ckJob ckComponentToken ckAny ckReference ckScript ckReport ckDialog " + "ctISBLEditor ctBevel ctButton ctCheckListBox ctComboBox ctComboEdit ctGrid ctDBCheckBox ctDBComboBox ctDBEdit ctDBEllipsis ctDBMemo ctDBNavigator ctDBRadioGroup ctDBStatusLabel ctEdit ctGroupBox ctInplaceHint ctMemo ctPanel ctListBox ctRadioButton ctRichEdit ctTabSheet ctWebBrowser ctImage ctHyperLink ctLabel ctDBMultiEllipsis ctRibbon ctRichView ctInnerPanel ctPanelGroup ctBitButton " + "cctDate cctInteger cctNumeric cctPick cctReference cctString cctText " + "cltInternal cltPrimary cltGUI " + "dseBeforeOpen dseAfterOpen dseBeforeClose dseAfterClose dseOnValidDelete dseBeforeDelete dseAfterDelete dseAfterDeleteOutOfTransaction dseOnDeleteError dseBeforeInsert dseAfterInsert dseOnValidUpdate dseBeforeUpdate dseOnUpdateRatifiedRecord dseAfterUpdate dseAfterUpdateOutOfTransaction dseOnUpdateError dseAfterScroll dseOnOpenRecord dseOnCloseRecord dseBeforeCancel dseAfterCancel dseOnUpdateDeadlockError dseBeforeDetailUpdate dseOnPrepareUpdate dseOnAnyRequisiteChange " + "dssEdit dssInsert dssBrowse dssInActive " + "dftDate dftShortDate dftDateTime dftTimeStamp " + "dotDays dotHours dotMinutes dotSeconds " + "dtkndLocal dtkndUTC " + "arNone arView arEdit arFull " + "ddaView ddaEdit " + "emLock emEdit emSign emExportWithLock emImportWithUnlock emChangeVersionNote emOpenForModify emChangeLifeStage emDelete emCreateVersion emImport emUnlockExportedWithLock emStart emAbort emReInit emMarkAsReaded emMarkAsUnreaded emPerform emAccept emResume emChangeRights emEditRoute emEditObserver emRecoveryFromLocalCopy emChangeWorkAccessType emChangeEncodeTypeToCertificate emChangeEncodeTypeToPassword emChangeEncodeTypeToNone emChangeEncodeTypeToCertificatePassword emChangeStandardRoute emGetText emOpenForView emMoveToStorage emCreateObject emChangeVersionHidden emDeleteVersion emChangeLifeCycleStage emApprovingSign emExport emContinue emLockFromEdit emUnLockForEdit emLockForServer emUnlockFromServer emDelegateAccessRights emReEncode " + "ecotFile ecotProcess " + "eaGet eaCopy eaCreate eaCreateStandardRoute " + "edltAll edltNothing edltQuery " + "essmText essmCard " + "esvtLast esvtLastActive esvtSpecified " + "edsfExecutive edsfArchive " + "edstSQLServer edstFile " + "edvstNone edvstEDocumentVersionCopy edvstFile edvstTemplate edvstScannedFile " + "vsDefault vsDesign vsActive vsObsolete " + "etNone etCertificate etPassword etCertificatePassword " + "ecException ecWarning ecInformation " + "estAll estApprovingOnly " + "evtLast evtLastActive evtQuery " + "fdtString fdtNumeric fdtInteger fdtDate fdtText fdtUnknown fdtWideString fdtLargeInteger " + "ftInbox ftOutbox ftFavorites ftCommonFolder ftUserFolder ftComponents ftQuickLaunch ftShortcuts ftSearch " + "grhAuto grhX1 grhX2 grhX3 " + "hltText hltRTF hltHTML " + "iffBMP iffJPEG iffMultiPageTIFF iffSinglePageTIFF iffTIFF iffPNG " + "im8bGrayscale im24bRGB im1bMonochrome " + "itBMP itJPEG itWMF itPNG " + "ikhInformation ikhWarning ikhError ikhNoIcon " + "icUnknown icScript icFunction icIntegratedReport icAnalyticReport icDataSetEventHandler icActionHandler icFormEventHandler icLookUpEventHandler icRequisiteChangeEventHandler icBeforeSearchEventHandler icRoleCalculation icSelectRouteEventHandler icBlockPropertyCalculation icBlockQueryParamsEventHandler icChangeSearchResultEventHandler icBlockEventHandler icSubTaskInitEventHandler icEDocDataSetEventHandler icEDocLookUpEventHandler icEDocActionHandler icEDocFormEventHandler icEDocRequisiteChangeEventHandler icStructuredConversionRule icStructuredConversionEventBefore icStructuredConversionEventAfter icWizardEventHandler icWizardFinishEventHandler icWizardStepEventHandler icWizardStepFinishEventHandler icWizardActionEnableEventHandler icWizardActionExecuteEventHandler icCreateJobsHandler icCreateNoticesHandler icBeforeLookUpEventHandler icAfterLookUpEventHandler icTaskAbortEventHandler icWorkflowBlockActionHandler icDialogDataSetEventHandler icDialogActionHandler icDialogLookUpEventHandler icDialogRequisiteChangeEventHandler icDialogFormEventHandler icDialogValidCloseEventHandler icBlockFormEventHandler icTaskFormEventHandler icReferenceMethod icEDocMethod icDialogMethod icProcessMessageHandler " + "isShow isHide isByUserSettings " + "jkJob jkNotice jkControlJob " + "jtInner jtLeft jtRight jtFull jtCross " + "lbpAbove lbpBelow lbpLeft lbpRight " + "eltPerConnection eltPerUser " + "sfcUndefined sfcBlack sfcGreen sfcRed sfcBlue sfcOrange sfcLilac " + "sfsItalic sfsStrikeout sfsNormal " + "ldctStandardRoute ldctWizard ldctScript ldctFunction ldctRouteBlock ldctIntegratedReport ldctAnalyticReport ldctReferenceType ldctEDocumentType ldctDialog ldctServerEvents " + "mrcrtNone mrcrtUser mrcrtMaximal mrcrtCustom " + "vtEqual vtGreaterOrEqual vtLessOrEqual vtRange " + "rdYesterday rdToday rdTomorrow rdThisWeek rdThisMonth rdThisYear rdNextMonth rdNextWeek rdLastWeek rdLastMonth " + "rdWindow rdFile rdPrinter " + "rdtString rdtNumeric rdtInteger rdtDate rdtReference rdtAccount rdtText rdtPick rdtUnknown rdtLargeInteger rdtDocument " + "reOnChange reOnChangeValues " + "ttGlobal ttLocal ttUser ttSystem " + "ssmBrowse ssmSelect ssmMultiSelect ssmBrowseModal " + "smSelect smLike smCard " + "stNone stAuthenticating stApproving " + "sctString sctStream " + "sstAnsiSort sstNaturalSort " + "svtEqual svtContain " + "soatString soatNumeric soatInteger soatDatetime soatReferenceRecord soatText soatPick soatBoolean soatEDocument soatAccount soatIntegerCollection soatNumericCollection soatStringCollection soatPickCollection soatDatetimeCollection soatBooleanCollection soatReferenceRecordCollection soatEDocumentCollection soatAccountCollection soatContents soatUnknown " + "tarAbortByUser tarAbortByWorkflowException " + "tvtAllWords tvtExactPhrase tvtAnyWord " + "usNone usCompleted usRedSquare usBlueSquare usYellowSquare usGreenSquare usOrangeSquare usPurpleSquare usFollowUp " + "utUnknown utUser utDeveloper utAdministrator utSystemDeveloper utDisconnected " + "btAnd btDetailAnd btOr btNotOr btOnly " + "vmView vmSelect vmNavigation " + "vsmSingle vsmMultiple vsmMultipleCheck vsmNoSelection " + "wfatPrevious wfatNext wfatCancel wfatFinish " + "wfepUndefined wfepText3 wfepText6 wfepText9 wfepSpinEdit wfepDropDown wfepRadioGroup wfepFlag wfepText12 wfepText15 wfepText18 wfepText21 wfepText24 wfepText27 wfepText30 wfepRadioGroupColumn1 wfepRadioGroupColumn2 wfepRadioGroupColumn3 " + "wfetQueryParameter wfetText wfetDelimiter wfetLabel " + "wptString wptInteger wptNumeric wptBoolean wptDateTime wptPick wptText wptUser wptUserList wptEDocumentInfo wptEDocumentInfoList wptReferenceRecordInfo wptReferenceRecordInfoList wptFolderInfo wptTaskInfo wptContents wptFileName wptDate " + "wsrComplete wsrGoNext wsrGoPrevious wsrCustom wsrCancel wsrGoFinal " + "wstForm wstEDocument wstTaskCard wstReferenceRecordCard wstFinal " + "waAll waPerformers waManual " + "wsbStart wsbFinish wsbNotice wsbStep wsbDecision wsbWait wsbMonitor wsbScript wsbConnector wsbSubTask wsbLifeCycleStage wsbPause " + "wdtInteger wdtFloat wdtString wdtPick wdtDateTime wdtBoolean wdtTask wdtJob wdtFolder wdtEDocument wdtReferenceRecord wdtUser wdtGroup wdtRole wdtIntegerCollection wdtFloatCollection wdtStringCollection wdtPickCollection wdtDateTimeCollection wdtBooleanCollection wdtTaskCollection wdtJobCollection wdtFolderCollection wdtEDocumentCollection wdtReferenceRecordCollection wdtUserCollection wdtGroupCollection wdtRoleCollection wdtContents wdtUserList wdtSearchDescription wdtDeadLine wdtPickSet wdtAccountCollection " + "wiLow wiNormal wiHigh " + "wrtSoft wrtHard " + "wsInit wsRunning wsDone wsControlled wsAborted wsContinued " + "wtmFull wtmFromCurrent wtmOnlyCurrent ", ka = "AddSubString AdjustLineBreaks AmountInWords Analysis ArrayDimCount ArrayHighBound ArrayLowBound ArrayOf ArrayReDim Assert Assigned BeginOfMonth BeginOfPeriod BuildProfilingOperationAnalysis CallProcedure CanReadFile CArrayElement CDataSetRequisite ChangeDate ChangeReferenceDataset Char CharPos CheckParam CheckParamValue CompareStrings ConstantExists ControlState ConvertDateStr Copy CopyFile CreateArray CreateCachedReference CreateConnection CreateDialog CreateDualListDialog CreateEditor CreateException CreateFile CreateFolderDialog CreateInputDialog CreateLinkFile CreateList CreateLock CreateMemoryDataSet CreateObject CreateOpenDialog CreateProgress CreateQuery CreateReference CreateReport CreateSaveDialog CreateScript CreateSQLPivotFunction CreateStringList CreateTreeListSelectDialog CSelectSQL CSQL CSubString CurrentUserID CurrentUserName CurrentVersion DataSetLocateEx DateDiff DateTimeDiff DateToStr DayOfWeek DeleteFile DirectoryExists DisableCheckAccessRights DisableCheckFullShowingRestriction DisableMassTaskSendingRestrictions DropTable DupeString EditText EnableCheckAccessRights EnableCheckFullShowingRestriction EnableMassTaskSendingRestrictions EndOfMonth EndOfPeriod ExceptionExists ExceptionsOff ExceptionsOn Execute ExecuteProcess Exit ExpandEnvironmentVariables ExtractFileDrive ExtractFileExt ExtractFileName ExtractFilePath ExtractParams FileExists FileSize FindFile FindSubString FirmContext ForceDirectories Format FormatDate FormatNumeric FormatSQLDate FormatString FreeException GetComponent GetComponentLaunchParam GetConstant GetLastException GetReferenceRecord GetRefTypeByRefID GetTableID GetTempFolder IfThen In IndexOf InputDialog InputDialogEx InteractiveMode IsFileLocked IsGraphicFile IsNumeric Length LoadString LoadStringFmt LocalTimeToUTC LowerCase Max MessageBox MessageBoxEx MimeDecodeBinary MimeDecodeString MimeEncodeBinary MimeEncodeString Min MoneyInWords MoveFile NewID Now OpenFile Ord Precision Raise ReadCertificateFromFile ReadFile ReferenceCodeByID ReferenceNumber ReferenceRequisiteMode ReferenceRequisiteValue RegionDateSettings RegionNumberSettings RegionTimeSettings RegRead RegWrite RenameFile Replace Round SelectServerCode SelectSQL ServerDateTime SetConstant SetManagedFolderFieldsState ShowConstantsInputDialog ShowMessage Sleep Split SQL SQL2XLSTAB SQLProfilingSendReport StrToDate SubString SubStringCount SystemSetting Time TimeDiff Today Transliterate Trim UpperCase UserStatus UTCToLocalTime ValidateXML VarIsClear VarIsEmpty VarIsNull WorkTimeDiff WriteFile WriteFileEx WriteObjectHistory Анализ БазаДанных БлокЕсть БлокЕстьРасш БлокИнфо БлокСнять БлокСнятьРасш БлокУстановить Ввод ВводМеню ВедС ВедСпр ВерхняяГраницаМассива ВнешПрогр Восст ВременнаяПапка Время ВыборSQL ВыбратьЗапись ВыделитьСтр Вызвать Выполнить ВыпПрогр ГрафическийФайл ГруппаДополнительно ДатаВремяСерв ДеньНедели ДиалогДаНет ДлинаСтр ДобПодстр ЕПусто ЕслиТо ЕЧисло ЗамПодстр ЗаписьСправочника ЗначПоляСпр ИДТипСпр ИзвлечьДиск ИзвлечьИмяФайла ИзвлечьПуть ИзвлечьРасширение ИзмДат ИзменитьРазмерМассива ИзмеренийМассива ИмяОрг ИмяПоляСпр Индекс ИндикаторЗакрыть ИндикаторОткрыть ИндикаторШаг ИнтерактивныйРежим ИтогТблСпр КодВидВедСпр КодВидСпрПоИД КодПоAnalit КодСимвола КодСпр КолПодстр КолПроп КонМес Конст КонстЕсть КонстЗнач КонТран КопироватьФайл КопияСтр КПериод КСтрТблСпр Макс МаксСтрТблСпр Массив Меню МенюРасш Мин НаборДанныхНайтиРасш НаимВидСпр НаимПоAnalit НаимСпр НастроитьПереводыСтрок НачМес НачТран НижняяГраницаМассива НомерСпр НПериод Окно Окр Окружение ОтлИнфДобавить ОтлИнфУдалить Отчет ОтчетАнал ОтчетИнт ПапкаСуществует Пауза ПВыборSQL ПереименоватьФайл Переменные ПереместитьФайл Подстр ПоискПодстр ПоискСтр ПолучитьИДТаблицы ПользовательДополнительно ПользовательИД ПользовательИмя ПользовательСтатус Прервать ПроверитьПараметр ПроверитьПараметрЗнач ПроверитьУсловие РазбСтр РазнВремя РазнДат РазнДатаВремя РазнРабВремя РегУстВрем РегУстДат РегУстЧсл РедТекст РеестрЗапись РеестрСписокИменПарам РеестрЧтение РеквСпр РеквСпрПр Сегодня Сейчас Сервер СерверПроцессИД СертификатФайлСчитать СжПроб Символ СистемаДиректумКод СистемаИнформация СистемаКод Содержит СоединениеЗакрыть СоединениеОткрыть СоздатьДиалог СоздатьДиалогВыбораИзДвухСписков СоздатьДиалогВыбораПапки СоздатьДиалогОткрытияФайла СоздатьДиалогСохраненияФайла СоздатьЗапрос СоздатьИндикатор СоздатьИсключение СоздатьКэшированныйСправочник СоздатьМассив СоздатьНаборДанных СоздатьОбъект СоздатьОтчет СоздатьПапку СоздатьРедактор СоздатьСоединение СоздатьСписок СоздатьСписокСтрок СоздатьСправочник СоздатьСценарий СоздСпр СостСпр Сохр СохрСпр СписокСистем Спр Справочник СпрБлокЕсть СпрБлокСнять СпрБлокСнятьРасш СпрБлокУстановить СпрИзмНабДан СпрКод СпрНомер СпрОбновить СпрОткрыть СпрОтменить СпрПарам СпрПолеЗнач СпрПолеИмя СпрРекв СпрРеквВведЗн СпрРеквНовые СпрРеквПр СпрРеквПредЗн СпрРеквРежим СпрРеквТипТекст СпрСоздать СпрСост СпрСохранить СпрТблИтог СпрТблСтр СпрТблСтрКол СпрТблСтрМакс СпрТблСтрМин СпрТблСтрПред СпрТблСтрСлед СпрТблСтрСозд СпрТблСтрУд СпрТекПредст СпрУдалить СравнитьСтр СтрВерхРегистр СтрНижнРегистр СтрТблСпр СумПроп Сценарий СценарийПарам ТекВерсия ТекОрг Точн Тран Транслитерация УдалитьТаблицу УдалитьФайл УдСпр УдСтрТблСпр Уст УстановкиКонстант ФайлАтрибутСчитать ФайлАтрибутУстановить ФайлВремя ФайлВремяУстановить ФайлВыбрать ФайлЗанят ФайлЗаписать ФайлИскать ФайлКопировать ФайлМожноЧитать ФайлОткрыть ФайлПереименовать ФайлПерекодировать ФайлПереместить ФайлПросмотреть ФайлРазмер ФайлСоздать ФайлСсылкаСоздать ФайлСуществует ФайлСчитать ФайлУдалить ФмтSQLДат ФмтДат ФмтСтр ФмтЧсл Формат ЦМассивЭлемент ЦНаборДанныхРеквизит ЦПодстр ", aa = "AltState Application CallType ComponentTokens CreatedJobs CreatedNotices ControlState DialogResult Dialogs EDocuments EDocumentVersionSource Folders GlobalIDs Job Jobs InputValue LookUpReference LookUpRequisiteNames LookUpSearch Object ParentComponent Processes References Requisite ReportName Reports Result Scripts Searches SelectedAttachments SelectedItems SelectMode Sender ServerEvents ServiceFactory ShiftState SubTask SystemDialogs Tasks Wizard Wizards Work ВызовСпособ ИмяОтчета РеквЗнач ", hi = "IApplication IAccessRights IAccountRepository IAccountSelectionRestrictions IAction IActionList IAdministrationHistoryDescription IAnchors IApplication IArchiveInfo IAttachment IAttachmentList ICheckListBox ICheckPointedList IColumn IComponent IComponentDescription IComponentToken IComponentTokenFactory IComponentTokenInfo ICompRecordInfo IConnection IContents IControl IControlJob IControlJobInfo IControlList ICrypto ICrypto2 ICustomJob ICustomJobInfo ICustomListBox ICustomObjectWizardStep ICustomWork ICustomWorkInfo IDataSet IDataSetAccessInfo IDataSigner IDateCriterion IDateRequisite IDateRequisiteDescription IDateValue IDeaAccessRights IDeaObjectInfo IDevelopmentComponentLock IDialog IDialogFactory IDialogPickRequisiteItems IDialogsFactory IDICSFactory IDocRequisite IDocumentInfo IDualListDialog IECertificate IECertificateInfo IECertificates IEditControl IEditorForm IEdmsExplorer IEdmsObject IEdmsObjectDescription IEdmsObjectFactory IEdmsObjectInfo IEDocument IEDocumentAccessRights IEDocumentDescription IEDocumentEditor IEDocumentFactory IEDocumentInfo IEDocumentStorage IEDocumentVersion IEDocumentVersionListDialog IEDocumentVersionSource IEDocumentWizardStep IEDocVerSignature IEDocVersionState IEnabledMode IEncodeProvider IEncrypter IEvent IEventList IException IExternalEvents IExternalHandler IFactory IField IFileDialog IFolder IFolderDescription IFolderDialog IFolderFactory IFolderInfo IForEach IForm IFormTitle IFormWizardStep IGlobalIDFactory IGlobalIDInfo IGrid IHasher IHistoryDescription IHyperLinkControl IImageButton IImageControl IInnerPanel IInplaceHint IIntegerCriterion IIntegerList IIntegerRequisite IIntegerValue IISBLEditorForm IJob IJobDescription IJobFactory IJobForm IJobInfo ILabelControl ILargeIntegerCriterion ILargeIntegerRequisite ILargeIntegerValue ILicenseInfo ILifeCycleStage IList IListBox ILocalIDInfo ILocalization ILock IMemoryDataSet IMessagingFactory IMetadataRepository INotice INoticeInfo INumericCriterion INumericRequisite INumericValue IObject IObjectDescription IObjectImporter IObjectInfo IObserver IPanelGroup IPickCriterion IPickProperty IPickRequisite IPickRequisiteDescription IPickRequisiteItem IPickRequisiteItems IPickValue IPrivilege IPrivilegeList IProcess IProcessFactory IProcessMessage IProgress IProperty IPropertyChangeEvent IQuery IReference IReferenceCriterion IReferenceEnabledMode IReferenceFactory IReferenceHistoryDescription IReferenceInfo IReferenceRecordCardWizardStep IReferenceRequisiteDescription IReferencesFactory IReferenceValue IRefRequisite IReport IReportFactory IRequisite IRequisiteDescription IRequisiteDescriptionList IRequisiteFactory IRichEdit IRouteStep IRule IRuleList ISchemeBlock IScript IScriptFactory ISearchCriteria ISearchCriterion ISearchDescription ISearchFactory ISearchFolderInfo ISearchForObjectDescription ISearchResultRestrictions ISecuredContext ISelectDialog IServerEvent IServerEventFactory IServiceDialog IServiceFactory ISignature ISignProvider ISignProvider2 ISignProvider3 ISimpleCriterion IStringCriterion IStringList IStringRequisite IStringRequisiteDescription IStringValue ISystemDialogsFactory ISystemInfo ITabSheet ITask ITaskAbortReasonInfo ITaskCardWizardStep ITaskDescription ITaskFactory ITaskInfo ITaskRoute ITextCriterion ITextRequisite ITextValue ITreeListSelectDialog IUser IUserList IValue IView IWebBrowserControl IWizard IWizardAction IWizardFactory IWizardFormElement IWizardParam IWizardPickParam IWizardReferenceParam IWizardStep IWorkAccessRights IWorkDescription IWorkflowAskableParam IWorkflowAskableParams IWorkflowBlock IWorkflowBlockResult IWorkflowEnabledMode IWorkflowParam IWorkflowPickParam IWorkflowReferenceParam IWorkState IWorkTreeCustomNode IWorkTreeJobNode IWorkTreeTaskNode IXMLEditorForm SBCrypto ", Gr = Ve + oa, Ca = aa, Sr = "null true false nil ", Dr = {
        className: "number",
        begin: _.NUMBER_RE,
        relevance: 0
      }, si = {
        className: "string",
        variants: [
          {
            begin: '"',
            end: '"'
          },
          {
            begin: "'",
            end: "'"
          }
        ]
      }, zr = {
        className: "doctag",
        begin: "\\b(?:TODO|DONE|BEGIN|END|STUB|CHG|FIXME|NOTE|BUG|XXX)\\b",
        relevance: 0
      }, di = {
        className: "comment",
        begin: "//",
        end: "$",
        relevance: 0,
        contains: [
          _.PHRASAL_WORDS_MODE,
          zr
        ]
      }, Da = {
        className: "comment",
        begin: "/\\*",
        end: "\\*/",
        relevance: 0,
        contains: [
          _.PHRASAL_WORDS_MODE,
          zr
        ]
      }, la = { variants: [
        di,
        Da
      ] }, wa = {
        $pattern: H,
        keyword: q,
        built_in: Gr,
        class: Ca,
        literal: Sr
      }, Wa = {
        begin: "\\.\\s*" + _.UNDERSCORE_IDENT_RE,
        keywords: wa,
        relevance: 0
      }, Qr = {
        className: "type",
        begin: ":[ \\t]*(" + hi.trim().replace(/\s/g, "|") + ")",
        end: "[ \\t]*=",
        excludeEnd: !0
      }, Ur = {
        className: "variable",
        keywords: wa,
        begin: H,
        relevance: 0,
        contains: [
          Qr,
          Wa
        ]
      }, rr = V + "\\(";
      return {
        name: "ISBL",
        case_insensitive: !0,
        keywords: wa,
        illegal: "\\$|\\?|%|,|;$|~|#|@|</",
        contains: [
          {
            className: "function",
            begin: rr,
            end: "\\)$",
            returnBegin: !0,
            keywords: wa,
            illegal: "[\\[\\]\\|\\$\\?%,~#@]",
            contains: [
              {
                className: "title",
                keywords: {
                  $pattern: H,
                  built_in: ka
                },
                begin: rr,
                end: "\\(",
                returnBegin: !0,
                excludeEnd: !0
              },
              Wa,
              Ur,
              si,
              Dr,
              la
            ]
          },
          Qr,
          Wa,
          Ur,
          si,
          Dr,
          la
        ]
      };
    }
    return isbl_1 = O, isbl_1;
  }
  var jbossCli_1, hasRequiredJbossCli;
  function requireJbossCli() {
    if (hasRequiredJbossCli) return jbossCli_1;
    hasRequiredJbossCli = 1;
    function O(_) {
      const V = {
        className: "params",
        begin: /\(/,
        end: /\)/,
        contains: [{
          begin: /[\w-]+ *=/,
          returnBegin: !0,
          relevance: 0,
          contains: [
            {
              className: "attr",
              begin: /[\w-]+/
            }
          ]
        }],
        relevance: 0
      }, q = {
        className: "function",
        begin: /:[\w\-.]+/,
        relevance: 0
      }, G = {
        className: "string",
        begin: /\B([\/.])[\w\-.\/=]+/
      }, J = {
        className: "params",
        begin: /--[\w\-=\/]+/
      };
      return {
        name: "JBoss CLI",
        aliases: ["wildfly-cli"],
        keywords: {
          $pattern: "[a-z-]+",
          keyword: "alias batch cd clear command connect connection-factory connection-info data-source deploy deployment-info deployment-overlay echo echo-dmr help history if jdbc-driver-info jms-queue|20 jms-topic|20 ls patch pwd quit read-attribute read-operation reload rollout-plan run-batch set shutdown try unalias undeploy unset version xa-data-source",
          // module
          literal: "true false"
        },
        contains: [
          _.HASH_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          J,
          q,
          G,
          V
        ]
      };
    }
    return jbossCli_1 = O, jbossCli_1;
  }
  var julia_1, hasRequiredJulia;
  function requireJulia() {
    if (hasRequiredJulia) return julia_1;
    hasRequiredJulia = 1;
    function O(_) {
      const H = "[A-Za-z_\\u00A1-\\uFFFF][A-Za-z_0-9\\u00A1-\\uFFFF]*", J = {
        $pattern: H,
        keyword: [
          "baremodule",
          "begin",
          "break",
          "catch",
          "ccall",
          "const",
          "continue",
          "do",
          "else",
          "elseif",
          "end",
          "export",
          "false",
          "finally",
          "for",
          "function",
          "global",
          "if",
          "import",
          "in",
          "isa",
          "let",
          "local",
          "macro",
          "module",
          "quote",
          "return",
          "true",
          "try",
          "using",
          "where",
          "while"
        ],
        literal: [
          "ARGS",
          "C_NULL",
          "DEPOT_PATH",
          "ENDIAN_BOM",
          "ENV",
          "Inf",
          "Inf16",
          "Inf32",
          "Inf64",
          "InsertionSort",
          "LOAD_PATH",
          "MergeSort",
          "NaN",
          "NaN16",
          "NaN32",
          "NaN64",
          "PROGRAM_FILE",
          "QuickSort",
          "RoundDown",
          "RoundFromZero",
          "RoundNearest",
          "RoundNearestTiesAway",
          "RoundNearestTiesUp",
          "RoundToZero",
          "RoundUp",
          "VERSION|0",
          "devnull",
          "false",
          "im",
          "missing",
          "nothing",
          "pi",
          "stderr",
          "stdin",
          "stdout",
          "true",
          "undef",
          "π",
          "ℯ"
        ],
        built_in: [
          "AbstractArray",
          "AbstractChannel",
          "AbstractChar",
          "AbstractDict",
          "AbstractDisplay",
          "AbstractFloat",
          "AbstractIrrational",
          "AbstractMatrix",
          "AbstractRange",
          "AbstractSet",
          "AbstractString",
          "AbstractUnitRange",
          "AbstractVecOrMat",
          "AbstractVector",
          "Any",
          "ArgumentError",
          "Array",
          "AssertionError",
          "BigFloat",
          "BigInt",
          "BitArray",
          "BitMatrix",
          "BitSet",
          "BitVector",
          "Bool",
          "BoundsError",
          "CapturedException",
          "CartesianIndex",
          "CartesianIndices",
          "Cchar",
          "Cdouble",
          "Cfloat",
          "Channel",
          "Char",
          "Cint",
          "Cintmax_t",
          "Clong",
          "Clonglong",
          "Cmd",
          "Colon",
          "Complex",
          "ComplexF16",
          "ComplexF32",
          "ComplexF64",
          "CompositeException",
          "Condition",
          "Cptrdiff_t",
          "Cshort",
          "Csize_t",
          "Cssize_t",
          "Cstring",
          "Cuchar",
          "Cuint",
          "Cuintmax_t",
          "Culong",
          "Culonglong",
          "Cushort",
          "Cvoid",
          "Cwchar_t",
          "Cwstring",
          "DataType",
          "DenseArray",
          "DenseMatrix",
          "DenseVecOrMat",
          "DenseVector",
          "Dict",
          "DimensionMismatch",
          "Dims",
          "DivideError",
          "DomainError",
          "EOFError",
          "Enum",
          "ErrorException",
          "Exception",
          "ExponentialBackOff",
          "Expr",
          "Float16",
          "Float32",
          "Float64",
          "Function",
          "GlobalRef",
          "HTML",
          "IO",
          "IOBuffer",
          "IOContext",
          "IOStream",
          "IdDict",
          "IndexCartesian",
          "IndexLinear",
          "IndexStyle",
          "InexactError",
          "InitError",
          "Int",
          "Int128",
          "Int16",
          "Int32",
          "Int64",
          "Int8",
          "Integer",
          "InterruptException",
          "InvalidStateException",
          "Irrational",
          "KeyError",
          "LinRange",
          "LineNumberNode",
          "LinearIndices",
          "LoadError",
          "MIME",
          "Matrix",
          "Method",
          "MethodError",
          "Missing",
          "MissingException",
          "Module",
          "NTuple",
          "NamedTuple",
          "Nothing",
          "Number",
          "OrdinalRange",
          "OutOfMemoryError",
          "OverflowError",
          "Pair",
          "PartialQuickSort",
          "PermutedDimsArray",
          "Pipe",
          "ProcessFailedException",
          "Ptr",
          "QuoteNode",
          "Rational",
          "RawFD",
          "ReadOnlyMemoryError",
          "Real",
          "ReentrantLock",
          "Ref",
          "Regex",
          "RegexMatch",
          "RoundingMode",
          "SegmentationFault",
          "Set",
          "Signed",
          "Some",
          "StackOverflowError",
          "StepRange",
          "StepRangeLen",
          "StridedArray",
          "StridedMatrix",
          "StridedVecOrMat",
          "StridedVector",
          "String",
          "StringIndexError",
          "SubArray",
          "SubString",
          "SubstitutionString",
          "Symbol",
          "SystemError",
          "Task",
          "TaskFailedException",
          "Text",
          "TextDisplay",
          "Timer",
          "Tuple",
          "Type",
          "TypeError",
          "TypeVar",
          "UInt",
          "UInt128",
          "UInt16",
          "UInt32",
          "UInt64",
          "UInt8",
          "UndefInitializer",
          "UndefKeywordError",
          "UndefRefError",
          "UndefVarError",
          "Union",
          "UnionAll",
          "UnitRange",
          "Unsigned",
          "Val",
          "Vararg",
          "VecElement",
          "VecOrMat",
          "Vector",
          "VersionNumber",
          "WeakKeyDict",
          "WeakRef"
        ]
      }, Z = {
        keywords: J,
        illegal: /<\//
      }, te = {
        className: "number",
        // supported numeric literals:
        //  * binary literal (e.g. 0x10)
        //  * octal literal (e.g. 0o76543210)
        //  * hexadecimal literal (e.g. 0xfedcba876543210)
        //  * hexadecimal floating point literal (e.g. 0x1p0, 0x1.2p2)
        //  * decimal literal (e.g. 9876543210, 100_000_000)
        //  * floating pointe literal (e.g. 1.2, 1.2f, .2, 1., 1.2e10, 1.2e-10)
        begin: /(\b0x[\d_]*(\.[\d_]*)?|0x\.\d[\d_]*)p[-+]?\d+|\b0[box][a-fA-F0-9][a-fA-F0-9_]*|(\b\d[\d_]*(\.[\d_]*)?|\.\d[\d_]*)([eEfF][-+]?\d+)?/,
        relevance: 0
      }, ne = {
        className: "string",
        begin: /'(.|\\[xXuU][a-zA-Z0-9]+)'/
      }, ee = {
        className: "subst",
        begin: /\$\(/,
        end: /\)/,
        keywords: J
      }, ae = {
        className: "variable",
        begin: "\\$" + H
      }, oe = {
        className: "string",
        contains: [
          _.BACKSLASH_ESCAPE,
          ee,
          ae
        ],
        variants: [
          {
            begin: /\w*"""/,
            end: /"""\w*/,
            relevance: 10
          },
          {
            begin: /\w*"/,
            end: /"\w*/
          }
        ]
      }, se = {
        className: "string",
        contains: [
          _.BACKSLASH_ESCAPE,
          ee,
          ae
        ],
        begin: "`",
        end: "`"
      }, ce = {
        className: "meta",
        begin: "@" + H
      }, le = {
        className: "comment",
        variants: [
          {
            begin: "#=",
            end: "=#",
            relevance: 10
          },
          {
            begin: "#",
            end: "$"
          }
        ]
      };
      return Z.name = "Julia", Z.contains = [
        te,
        ne,
        oe,
        se,
        ce,
        le,
        _.HASH_COMMENT_MODE,
        {
          className: "keyword",
          begin: "\\b(((abstract|primitive)\\s+)type|(mutable\\s+)?struct)\\b"
        },
        { begin: /<:/ }
        // relevance booster
      ], ee.contains = Z.contains, Z;
    }
    return julia_1 = O, julia_1;
  }
  var juliaRepl_1, hasRequiredJuliaRepl;
  function requireJuliaRepl() {
    if (hasRequiredJuliaRepl) return juliaRepl_1;
    hasRequiredJuliaRepl = 1;
    function O(_) {
      return {
        name: "Julia REPL",
        contains: [
          {
            className: "meta.prompt",
            begin: /^julia>/,
            relevance: 10,
            starts: {
              // end the highlighting if we are on a new line and the line does not have at
              // least six spaces in the beginning
              end: /^(?![ ]{6})/,
              subLanguage: "julia"
            }
          }
        ],
        // jldoctest Markdown blocks are used in the Julia manual and package docs indicate
        // code snippets that should be verified when the documentation is built. They can be
        // either REPL-like or script-like, but are usually REPL-like and therefore we apply
        // julia-repl highlighting to them. More information can be found in Documenter's
        // manual: https://juliadocs.github.io/Documenter.jl/latest/man/doctests.html
        aliases: ["jldoctest"]
      };
    }
    return juliaRepl_1 = O, juliaRepl_1;
  }
  var lasso_1, hasRequiredLasso;
  function requireLasso() {
    if (hasRequiredLasso) return lasso_1;
    hasRequiredLasso = 1;
    function O(_) {
      const H = "[a-zA-Z_][\\w.]*", V = "<\\?(lasso(script)?|=)", q = "\\]|\\?>", G = {
        $pattern: H + "|&[lg]t;",
        literal: "true false none minimal full all void and or not bw nbw ew new cn ncn lt lte gt gte eq neq rx nrx ft",
        built_in: "array date decimal duration integer map pair string tag xml null boolean bytes keyword list locale queue set stack staticarray local var variable global data self inherited currentcapture givenblock",
        keyword: "cache database_names database_schemanames database_tablenames define_tag define_type email_batch encode_set html_comment handle handle_error header if inline iterate ljax_target link link_currentaction link_currentgroup link_currentrecord link_detail link_firstgroup link_firstrecord link_lastgroup link_lastrecord link_nextgroup link_nextrecord link_prevgroup link_prevrecord log loop namespace_using output_none portal private protect records referer referrer repeating resultset rows search_args search_arguments select sort_args sort_arguments thread_atomic value_list while abort case else fail_if fail_ifnot fail if_empty if_false if_null if_true loop_abort loop_continue loop_count params params_up return return_value run_children soap_definetag soap_lastrequest soap_lastresponse tag_name ascending average by define descending do equals frozen group handle_failure import in into join let match max min on order parent protected provide public require returnhome skip split_thread sum take thread to trait type where with yield yieldhome"
      }, J = _.COMMENT(
        "<!--",
        "-->",
        { relevance: 0 }
      ), Z = {
        className: "meta",
        begin: "\\[noprocess\\]",
        starts: {
          end: "\\[/noprocess\\]",
          returnEnd: !0,
          contains: [J]
        }
      }, te = {
        className: "meta",
        begin: "\\[/noprocess|" + V
      }, ne = {
        className: "symbol",
        begin: "'" + H + "'"
      }, ee = [
        _.C_LINE_COMMENT_MODE,
        _.C_BLOCK_COMMENT_MODE,
        _.inherit(_.C_NUMBER_MODE, { begin: _.C_NUMBER_RE + "|(-?infinity|NaN)\\b" }),
        _.inherit(_.APOS_STRING_MODE, { illegal: null }),
        _.inherit(_.QUOTE_STRING_MODE, { illegal: null }),
        {
          className: "string",
          begin: "`",
          end: "`"
        },
        {
          // variables
          variants: [
            { begin: "[#$]" + H },
            {
              begin: "#",
              end: "\\d+",
              illegal: "\\W"
            }
          ]
        },
        {
          className: "type",
          begin: "::\\s*",
          end: H,
          illegal: "\\W"
        },
        {
          className: "params",
          variants: [
            {
              begin: "-(?!infinity)" + H,
              relevance: 0
            },
            { begin: "(\\.\\.\\.)" }
          ]
        },
        {
          begin: /(->|\.)\s*/,
          relevance: 0,
          contains: [ne]
        },
        {
          className: "class",
          beginKeywords: "define",
          returnEnd: !0,
          end: "\\(|=>",
          contains: [_.inherit(_.TITLE_MODE, { begin: H + "(=(?!>))?|[-+*/%](?!>)" })]
        }
      ];
      return {
        name: "Lasso",
        aliases: [
          "ls",
          "lassoscript"
        ],
        case_insensitive: !0,
        keywords: G,
        contains: [
          {
            className: "meta",
            begin: q,
            relevance: 0,
            starts: {
              // markup
              end: "\\[|" + V,
              returnEnd: !0,
              relevance: 0,
              contains: [J]
            }
          },
          Z,
          te,
          {
            className: "meta",
            begin: "\\[no_square_brackets",
            starts: {
              end: "\\[/no_square_brackets\\]",
              // not implemented in the language
              keywords: G,
              contains: [
                {
                  className: "meta",
                  begin: q,
                  relevance: 0,
                  starts: {
                    end: "\\[noprocess\\]|" + V,
                    returnEnd: !0,
                    contains: [J]
                  }
                },
                Z,
                te
              ].concat(ee)
            }
          },
          {
            className: "meta",
            begin: "\\[",
            relevance: 0
          },
          {
            className: "meta",
            begin: "^#!",
            end: "lasso9$",
            relevance: 10
          }
        ].concat(ee)
      };
    }
    return lasso_1 = O, lasso_1;
  }
  var latex_1, hasRequiredLatex;
  function requireLatex() {
    if (hasRequiredLatex) return latex_1;
    hasRequiredLatex = 1;
    function O(_) {
      const V = _.regex.either(...[
        "(?:NeedsTeXFormat|RequirePackage|GetIdInfo)",
        "Provides(?:Expl)?(?:Package|Class|File)",
        "(?:DeclareOption|ProcessOptions)",
        "(?:documentclass|usepackage|input|include)",
        "makeat(?:letter|other)",
        "ExplSyntax(?:On|Off)",
        "(?:new|renew|provide)?command",
        "(?:re)newenvironment",
        "(?:New|Renew|Provide|Declare)(?:Expandable)?DocumentCommand",
        "(?:New|Renew|Provide|Declare)DocumentEnvironment",
        "(?:(?:e|g|x)?def|let)",
        "(?:begin|end)",
        "(?:part|chapter|(?:sub){0,2}section|(?:sub)?paragraph)",
        "caption",
        "(?:label|(?:eq|page|name)?ref|(?:paren|foot|super)?cite)",
        "(?:alpha|beta|[Gg]amma|[Dd]elta|(?:var)?epsilon|zeta|eta|[Tt]heta|vartheta)",
        "(?:iota|(?:var)?kappa|[Ll]ambda|mu|nu|[Xx]i|[Pp]i|varpi|(?:var)rho)",
        "(?:[Ss]igma|varsigma|tau|[Uu]psilon|[Pp]hi|varphi|chi|[Pp]si|[Oo]mega)",
        "(?:frac|sum|prod|lim|infty|times|sqrt|leq|geq|left|right|middle|[bB]igg?)",
        "(?:[lr]angle|q?quad|[lcvdi]?dots|d?dot|hat|tilde|bar)"
      ].map((Se) => Se + "(?![a-zA-Z@:_])")), q = new RegExp([
        // A function \module_function_name:signature or \__module_function_name:signature,
        // where both module and function_name need at least two characters and
        // function_name may contain single underscores.
        "(?:__)?[a-zA-Z]{2,}_[a-zA-Z](?:_?[a-zA-Z])+:[a-zA-Z]*",
        // A variable \scope_module_and_name_type or \scope__module_ane_name_type,
        // where scope is one of l, g or c, type needs at least two characters
        // and module_and_name may contain single underscores.
        "[lgc]__?[a-zA-Z](?:_?[a-zA-Z])*_[a-zA-Z]{2,}",
        // A quark \q_the_name or \q__the_name or
        // scan mark \s_the_name or \s__vthe_name,
        // where variable_name needs at least two characters and
        // may contain single underscores.
        "[qs]__?[a-zA-Z](?:_?[a-zA-Z])+",
        // Other LaTeX3 macro names that are not covered by the three rules above.
        "use(?:_i)?:[a-zA-Z]*",
        "(?:else|fi|or):",
        "(?:if|cs|exp):w",
        "(?:hbox|vbox):n",
        "::[a-zA-Z]_unbraced",
        "::[a-zA-Z:]"
      ].map((Se) => Se + "(?![a-zA-Z:_])").join("|")), G = [
        { begin: /[a-zA-Z@]+/ },
        // control word
        { begin: /[^a-zA-Z@]?/ }
        // control symbol
      ], J = [
        { begin: /\^{6}[0-9a-f]{6}/ },
        { begin: /\^{5}[0-9a-f]{5}/ },
        { begin: /\^{4}[0-9a-f]{4}/ },
        { begin: /\^{3}[0-9a-f]{3}/ },
        { begin: /\^{2}[0-9a-f]{2}/ },
        { begin: /\^{2}[\u0000-\u007f]/ }
      ], Z = {
        className: "keyword",
        begin: /\\/,
        relevance: 0,
        contains: [
          {
            endsParent: !0,
            begin: V
          },
          {
            endsParent: !0,
            begin: q
          },
          {
            endsParent: !0,
            variants: J
          },
          {
            endsParent: !0,
            relevance: 0,
            variants: G
          }
        ]
      }, te = {
        className: "params",
        relevance: 0,
        begin: /#+\d?/
      }, ne = {
        // relevance: 1
        variants: J
      }, ee = {
        className: "built_in",
        relevance: 0,
        begin: /[$&^_]/
      }, ae = {
        className: "meta",
        begin: /% ?!(T[eE]X|tex|BIB|bib)/,
        end: "$",
        relevance: 10
      }, oe = _.COMMENT(
        "%",
        "$",
        { relevance: 0 }
      ), se = [
        Z,
        te,
        ne,
        ee,
        ae,
        oe
      ], ce = {
        begin: /\{/,
        end: /\}/,
        relevance: 0,
        contains: [
          "self",
          ...se
        ]
      }, le = _.inherit(
        ce,
        {
          relevance: 0,
          endsParent: !0,
          contains: [
            ce,
            ...se
          ]
        }
      ), ue = {
        begin: /\[/,
        end: /\]/,
        endsParent: !0,
        relevance: 0,
        contains: [
          ce,
          ...se
        ]
      }, de = {
        begin: /\s+/,
        relevance: 0
      }, fe = [le], he = [ue], me = function(Se, we) {
        return {
          contains: [de],
          starts: {
            relevance: 0,
            contains: Se,
            starts: we
          }
        };
      }, pe = function(Se, we) {
        return {
          begin: "\\\\" + Se + "(?![a-zA-Z@:_])",
          keywords: {
            $pattern: /\\[a-zA-Z]+/,
            keyword: "\\" + Se
          },
          relevance: 0,
          contains: [de],
          starts: we
        };
      }, ye = function(Se, we) {
        return _.inherit(
          {
            begin: "\\\\begin(?=[ 	]*(\\r?\\n[ 	]*)?\\{" + Se + "\\})",
            keywords: {
              $pattern: /\\[a-zA-Z]+/,
              keyword: "\\begin"
            },
            relevance: 0
          },
          me(fe, we)
        );
      }, ge = (Se = "string") => _.END_SAME_AS_BEGIN({
        className: Se,
        begin: /(.|\r?\n)/,
        end: /(.|\r?\n)/,
        excludeBegin: !0,
        excludeEnd: !0,
        endsParent: !0
      }), ve = function(Se) {
        return {
          className: "string",
          end: "(?=\\\\end\\{" + Se + "\\})"
        };
      }, be = (Se = "string") => ({
        relevance: 0,
        begin: /\{/,
        starts: {
          endsParent: !0,
          contains: [
            {
              className: Se,
              end: /(?=\})/,
              endsParent: !0,
              contains: [
                {
                  begin: /\{/,
                  end: /\}/,
                  relevance: 0,
                  contains: ["self"]
                }
              ]
            }
          ]
        }
      }), Ce = [
        ...[
          "verb",
          "lstinline"
        ].map((Se) => pe(Se, { contains: [ge()] })),
        pe("mint", me(fe, { contains: [ge()] })),
        pe("mintinline", me(fe, { contains: [
          be(),
          ge()
        ] })),
        pe("url", { contains: [
          be("link"),
          be("link")
        ] }),
        pe("hyperref", { contains: [be("link")] }),
        pe("href", me(he, { contains: [be("link")] })),
        ...[].concat(...[
          "",
          "\\*"
        ].map((Se) => [
          ye("verbatim" + Se, ve("verbatim" + Se)),
          ye("filecontents" + Se, me(fe, ve("filecontents" + Se))),
          ...[
            "",
            "B",
            "L"
          ].map(
            (we) => ye(we + "Verbatim" + Se, me(he, ve(we + "Verbatim" + Se)))
          )
        ])),
        ye("minted", me(he, me(fe, ve("minted"))))
      ];
      return {
        name: "LaTeX",
        aliases: ["tex"],
        contains: [
          ...Ce,
          ...se
        ]
      };
    }
    return latex_1 = O, latex_1;
  }
  var ldif_1, hasRequiredLdif;
  function requireLdif() {
    if (hasRequiredLdif) return ldif_1;
    hasRequiredLdif = 1;
    function O(_) {
      return {
        name: "LDIF",
        contains: [
          {
            className: "attribute",
            match: "^dn(?=:)",
            relevance: 10
          },
          {
            className: "attribute",
            match: "^\\w+(?=:)"
          },
          {
            className: "literal",
            match: "^-"
          },
          _.HASH_COMMENT_MODE
        ]
      };
    }
    return ldif_1 = O, ldif_1;
  }
  var leaf_1, hasRequiredLeaf;
  function requireLeaf() {
    if (hasRequiredLeaf) return leaf_1;
    hasRequiredLeaf = 1;
    function O(_) {
      const H = /([A-Za-z_][A-Za-z_0-9]*)?/, q = {
        scope: "params",
        begin: /\(/,
        end: /\)(?=\:?)/,
        endsParent: !0,
        relevance: 7,
        contains: [
          {
            scope: "string",
            begin: '"',
            end: '"'
          },
          {
            scope: "keyword",
            match: [
              "true",
              "false",
              "in"
            ].join("|")
          },
          {
            scope: "variable",
            match: /[A-Za-z_][A-Za-z_0-9]*/
          },
          {
            scope: "operator",
            match: /\+|\-|\*|\/|\%|\=\=|\=|\!|\>|\<|\&\&|\|\|/
          }
        ]
      }, G = {
        match: [
          H,
          /(?=\()/
        ],
        scope: {
          1: "keyword"
        },
        contains: [q]
      };
      return q.contains.unshift(G), {
        name: "Leaf",
        contains: [
          // #ident():
          {
            match: [
              /#+/,
              H,
              /(?=\()/
            ],
            scope: {
              1: "punctuation",
              2: "keyword"
            },
            // will start up after the ending `)` match from line ~44
            // just to grab the trailing `:` if we can match it
            starts: {
              contains: [
                {
                  match: /\:/,
                  scope: "punctuation"
                }
              ]
            },
            contains: [
              q
            ]
          },
          // #ident or #ident:
          {
            match: [
              /#+/,
              H,
              /:?/
            ],
            scope: {
              1: "punctuation",
              2: "keyword",
              3: "punctuation"
            }
          }
        ]
      };
    }
    return leaf_1 = O, leaf_1;
  }
  var lisp_1, hasRequiredLisp;
  function requireLisp() {
    if (hasRequiredLisp) return lisp_1;
    hasRequiredLisp = 1;
    function O(_) {
      const H = "[a-zA-Z_\\-+\\*\\/<=>&#][a-zA-Z0-9_\\-+*\\/<=>&#!]*", V = "\\|[^]*?\\|", q = "(-|\\+)?\\d+(\\.\\d+|\\/\\d+)?((d|e|f|l|s|D|E|F|L|S)(\\+|-)?\\d+)?", G = {
        className: "literal",
        begin: "\\b(t{1}|nil)\\b"
      }, J = {
        className: "number",
        variants: [
          {
            begin: q,
            relevance: 0
          },
          { begin: "#(b|B)[0-1]+(/[0-1]+)?" },
          { begin: "#(o|O)[0-7]+(/[0-7]+)?" },
          { begin: "#(x|X)[0-9a-fA-F]+(/[0-9a-fA-F]+)?" },
          {
            begin: "#(c|C)\\(" + q + " +" + q,
            end: "\\)"
          }
        ]
      }, Z = _.inherit(_.QUOTE_STRING_MODE, { illegal: null }), te = _.COMMENT(
        ";",
        "$",
        { relevance: 0 }
      ), ne = {
        begin: "\\*",
        end: "\\*"
      }, ee = {
        className: "symbol",
        begin: "[:&]" + H
      }, ae = {
        begin: H,
        relevance: 0
      }, oe = { begin: V }, ce = {
        contains: [
          J,
          Z,
          ne,
          ee,
          {
            begin: "\\(",
            end: "\\)",
            contains: [
              "self",
              G,
              Z,
              J,
              ae
            ]
          },
          ae
        ],
        variants: [
          {
            begin: "['`]\\(",
            end: "\\)"
          },
          {
            begin: "\\(quote ",
            end: "\\)",
            keywords: { name: "quote" }
          },
          { begin: "'" + V }
        ]
      }, le = { variants: [
        { begin: "'" + H },
        { begin: "#'" + H + "(::" + H + ")*" }
      ] }, ue = {
        begin: "\\(\\s*",
        end: "\\)"
      }, de = {
        endsWithParent: !0,
        relevance: 0
      };
      return ue.contains = [
        {
          className: "name",
          variants: [
            {
              begin: H,
              relevance: 0
            },
            { begin: V }
          ]
        },
        de
      ], de.contains = [
        ce,
        le,
        ue,
        G,
        J,
        Z,
        te,
        ne,
        ee,
        oe,
        ae
      ], {
        name: "Lisp",
        illegal: /\S/,
        contains: [
          J,
          _.SHEBANG(),
          G,
          Z,
          te,
          ce,
          le,
          ue,
          ae
        ]
      };
    }
    return lisp_1 = O, lisp_1;
  }
  var livecodeserver_1, hasRequiredLivecodeserver;
  function requireLivecodeserver() {
    if (hasRequiredLivecodeserver) return livecodeserver_1;
    hasRequiredLivecodeserver = 1;
    function O(_) {
      const H = {
        className: "variable",
        variants: [
          { begin: "\\b([gtps][A-Z]{1}[a-zA-Z0-9]*)(\\[.+\\])?(?:\\s*?)" },
          { begin: "\\$_[A-Z]+" }
        ],
        relevance: 0
      }, V = [
        _.C_BLOCK_COMMENT_MODE,
        _.HASH_COMMENT_MODE,
        _.COMMENT("--", "$"),
        _.COMMENT("[^:]//", "$")
      ], q = _.inherit(_.TITLE_MODE, { variants: [
        { begin: "\\b_*rig[A-Z][A-Za-z0-9_\\-]*" },
        { begin: "\\b_[a-z0-9\\-]+" }
      ] }), G = _.inherit(_.TITLE_MODE, { begin: "\\b([A-Za-z0-9_\\-]+)\\b" });
      return {
        name: "LiveCode",
        case_insensitive: !1,
        keywords: {
          keyword: "$_COOKIE $_FILES $_GET $_GET_BINARY $_GET_RAW $_POST $_POST_BINARY $_POST_RAW $_SESSION $_SERVER codepoint codepoints segment segments codeunit codeunits sentence sentences trueWord trueWords paragraph after byte bytes english the until http forever descending using line real8 with seventh for stdout finally element word words fourth before black ninth sixth characters chars stderr uInt1 uInt1s uInt2 uInt2s stdin string lines relative rel any fifth items from middle mid at else of catch then third it file milliseconds seconds second secs sec int1 int1s int4 int4s internet int2 int2s normal text item last long detailed effective uInt4 uInt4s repeat end repeat URL in try into switch to words https token binfile each tenth as ticks tick system real4 by dateItems without char character ascending eighth whole dateTime numeric short first ftp integer abbreviated abbr abbrev private case while if div mod wrap and or bitAnd bitNot bitOr bitXor among not in a an within contains ends with begins the keys of keys",
          literal: "SIX TEN FORMFEED NINE ZERO NONE SPACE FOUR FALSE COLON CRLF PI COMMA ENDOFFILE EOF EIGHT FIVE QUOTE EMPTY ONE TRUE RETURN CR LINEFEED RIGHT BACKSLASH NULL SEVEN TAB THREE TWO six ten formfeed nine zero none space four false colon crlf pi comma endoffile eof eight five quote empty one true return cr linefeed right backslash null seven tab three two RIVERSION RISTATE FILE_READ_MODE FILE_WRITE_MODE FILE_WRITE_MODE DIR_WRITE_MODE FILE_READ_UMASK FILE_WRITE_UMASK DIR_READ_UMASK DIR_WRITE_UMASK",
          built_in: "put abs acos aliasReference annuity arrayDecode arrayEncode asin atan atan2 average avg avgDev base64Decode base64Encode baseConvert binaryDecode binaryEncode byteOffset byteToNum cachedURL cachedURLs charToNum cipherNames codepointOffset codepointProperty codepointToNum codeunitOffset commandNames compound compress constantNames cos date dateFormat decompress difference directories diskSpace DNSServers exp exp1 exp2 exp10 extents files flushEvents folders format functionNames geometricMean global globals hasMemory harmonicMean hostAddress hostAddressToName hostName hostNameToAddress isNumber ISOToMac itemOffset keys len length libURLErrorData libUrlFormData libURLftpCommand libURLLastHTTPHeaders libURLLastRHHeaders libUrlMultipartFormAddPart libUrlMultipartFormData libURLVersion lineOffset ln ln1 localNames log log2 log10 longFilePath lower macToISO matchChunk matchText matrixMultiply max md5Digest median merge messageAuthenticationCode messageDigest millisec millisecs millisecond milliseconds min monthNames nativeCharToNum normalizeText num number numToByte numToChar numToCodepoint numToNativeChar offset open openfiles openProcesses openProcessIDs openSockets paragraphOffset paramCount param params peerAddress pendingMessages platform popStdDev populationStandardDeviation populationVariance popVariance processID random randomBytes replaceText result revCreateXMLTree revCreateXMLTreeFromFile revCurrentRecord revCurrentRecordIsFirst revCurrentRecordIsLast revDatabaseColumnCount revDatabaseColumnIsNull revDatabaseColumnLengths revDatabaseColumnNames revDatabaseColumnNamed revDatabaseColumnNumbered revDatabaseColumnTypes revDatabaseConnectResult revDatabaseCursors revDatabaseID revDatabaseTableNames revDatabaseType revDataFromQuery revdb_closeCursor revdb_columnbynumber revdb_columncount revdb_columnisnull revdb_columnlengths revdb_columnnames revdb_columntypes revdb_commit revdb_connect revdb_connections revdb_connectionerr revdb_currentrecord revdb_cursorconnection revdb_cursorerr revdb_cursors revdb_dbtype revdb_disconnect revdb_execute revdb_iseof revdb_isbof revdb_movefirst revdb_movelast revdb_movenext revdb_moveprev revdb_query revdb_querylist revdb_recordcount revdb_rollback revdb_tablenames revGetDatabaseDriverPath revNumberOfRecords revOpenDatabase revOpenDatabases revQueryDatabase revQueryDatabaseBlob revQueryResult revQueryIsAtStart revQueryIsAtEnd revUnixFromMacPath revXMLAttribute revXMLAttributes revXMLAttributeValues revXMLChildContents revXMLChildNames revXMLCreateTreeFromFileWithNamespaces revXMLCreateTreeWithNamespaces revXMLDataFromXPathQuery revXMLEvaluateXPath revXMLFirstChild revXMLMatchingNode revXMLNextSibling revXMLNodeContents revXMLNumberOfChildren revXMLParent revXMLPreviousSibling revXMLRootNode revXMLRPC_CreateRequest revXMLRPC_Documents revXMLRPC_Error revXMLRPC_GetHost revXMLRPC_GetMethod revXMLRPC_GetParam revXMLText revXMLRPC_Execute revXMLRPC_GetParamCount revXMLRPC_GetParamNode revXMLRPC_GetParamType revXMLRPC_GetPath revXMLRPC_GetPort revXMLRPC_GetProtocol revXMLRPC_GetRequest revXMLRPC_GetResponse revXMLRPC_GetSocket revXMLTree revXMLTrees revXMLValidateDTD revZipDescribeItem revZipEnumerateItems revZipOpenArchives round sampVariance sec secs seconds sentenceOffset sha1Digest shell shortFilePath sin specialFolderPath sqrt standardDeviation statRound stdDev sum sysError systemVersion tan tempName textDecode textEncode tick ticks time to tokenOffset toLower toUpper transpose truewordOffset trunc uniDecode uniEncode upper URLDecode URLEncode URLStatus uuid value variableNames variance version waitDepth weekdayNames wordOffset xsltApplyStylesheet xsltApplyStylesheetFromFile xsltLoadStylesheet xsltLoadStylesheetFromFile add breakpoint cancel clear local variable file word line folder directory URL close socket process combine constant convert create new alias folder directory decrypt delete variable word line folder directory URL dispatch divide do encrypt filter get include intersect kill libURLDownloadToFile libURLFollowHttpRedirects libURLftpUpload libURLftpUploadFile libURLresetAll libUrlSetAuthCallback libURLSetDriver libURLSetCustomHTTPHeaders libUrlSetExpect100 libURLSetFTPListCommand libURLSetFTPMode libURLSetFTPStopTime libURLSetStatusCallback load extension loadedExtensions multiply socket prepare process post seek rel relative read from process rename replace require resetAll resolve revAddXMLNode revAppendXML revCloseCursor revCloseDatabase revCommitDatabase revCopyFile revCopyFolder revCopyXMLNode revDeleteFolder revDeleteXMLNode revDeleteAllXMLTrees revDeleteXMLTree revExecuteSQL revGoURL revInsertXMLNode revMoveFolder revMoveToFirstRecord revMoveToLastRecord revMoveToNextRecord revMoveToPreviousRecord revMoveToRecord revMoveXMLNode revPutIntoXMLNode revRollBackDatabase revSetDatabaseDriverPath revSetXMLAttribute revXMLRPC_AddParam revXMLRPC_DeleteAllDocuments revXMLAddDTD revXMLRPC_Free revXMLRPC_FreeAll revXMLRPC_DeleteDocument revXMLRPC_DeleteParam revXMLRPC_SetHost revXMLRPC_SetMethod revXMLRPC_SetPort revXMLRPC_SetProtocol revXMLRPC_SetSocket revZipAddItemWithData revZipAddItemWithFile revZipAddUncompressedItemWithData revZipAddUncompressedItemWithFile revZipCancel revZipCloseArchive revZipDeleteItem revZipExtractItemToFile revZipExtractItemToVariable revZipSetProgressCallback revZipRenameItem revZipReplaceItemWithData revZipReplaceItemWithFile revZipOpenArchive send set sort split start stop subtract symmetric union unload vectorDotProduct wait write"
        },
        contains: [
          H,
          {
            className: "keyword",
            begin: "\\bend\\sif\\b"
          },
          {
            className: "function",
            beginKeywords: "function",
            end: "$",
            contains: [
              H,
              G,
              _.APOS_STRING_MODE,
              _.QUOTE_STRING_MODE,
              _.BINARY_NUMBER_MODE,
              _.C_NUMBER_MODE,
              q
            ]
          },
          {
            className: "function",
            begin: "\\bend\\s+",
            end: "$",
            keywords: "end",
            contains: [
              G,
              q
            ],
            relevance: 0
          },
          {
            beginKeywords: "command on",
            end: "$",
            contains: [
              H,
              G,
              _.APOS_STRING_MODE,
              _.QUOTE_STRING_MODE,
              _.BINARY_NUMBER_MODE,
              _.C_NUMBER_MODE,
              q
            ]
          },
          {
            className: "meta",
            variants: [
              {
                begin: "<\\?(rev|lc|livecode)",
                relevance: 10
              },
              { begin: "<\\?" },
              { begin: "\\?>" }
            ]
          },
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.BINARY_NUMBER_MODE,
          _.C_NUMBER_MODE,
          q
        ].concat(V),
        illegal: ";$|^\\[|^=|&|\\{"
      };
    }
    return livecodeserver_1 = O, livecodeserver_1;
  }
  var livescript_1, hasRequiredLivescript;
  function requireLivescript() {
    if (hasRequiredLivescript) return livescript_1;
    hasRequiredLivescript = 1;
    const O = [
      "as",
      // for exports
      "in",
      "of",
      "if",
      "for",
      "while",
      "finally",
      "var",
      "new",
      "function",
      "do",
      "return",
      "void",
      "else",
      "break",
      "catch",
      "instanceof",
      "with",
      "throw",
      "case",
      "default",
      "try",
      "switch",
      "continue",
      "typeof",
      "delete",
      "let",
      "yield",
      "const",
      "class",
      // JS handles these with a special rule
      // "get",
      // "set",
      "debugger",
      "async",
      "await",
      "static",
      "import",
      "from",
      "export",
      "extends",
      // It's reached stage 3, which is "recommended for implementation":
      "using"
    ], _ = [
      "true",
      "false",
      "null",
      "undefined",
      "NaN",
      "Infinity"
    ], H = [
      // Fundamental objects
      "Object",
      "Function",
      "Boolean",
      "Symbol",
      // numbers and dates
      "Math",
      "Date",
      "Number",
      "BigInt",
      // text
      "String",
      "RegExp",
      // Indexed collections
      "Array",
      "Float32Array",
      "Float64Array",
      "Int8Array",
      "Uint8Array",
      "Uint8ClampedArray",
      "Int16Array",
      "Int32Array",
      "Uint16Array",
      "Uint32Array",
      "BigInt64Array",
      "BigUint64Array",
      // Keyed collections
      "Set",
      "Map",
      "WeakSet",
      "WeakMap",
      // Structured data
      "ArrayBuffer",
      "SharedArrayBuffer",
      "Atomics",
      "DataView",
      "JSON",
      // Control abstraction objects
      "Promise",
      "Generator",
      "GeneratorFunction",
      "AsyncFunction",
      // Reflection
      "Reflect",
      "Proxy",
      // Internationalization
      "Intl",
      // WebAssembly
      "WebAssembly"
    ], V = [
      "Error",
      "EvalError",
      "InternalError",
      "RangeError",
      "ReferenceError",
      "SyntaxError",
      "TypeError",
      "URIError"
    ], q = [
      "setInterval",
      "setTimeout",
      "clearInterval",
      "clearTimeout",
      "require",
      "exports",
      "eval",
      "isFinite",
      "isNaN",
      "parseFloat",
      "parseInt",
      "decodeURI",
      "decodeURIComponent",
      "encodeURI",
      "encodeURIComponent",
      "escape",
      "unescape"
    ], G = [].concat(
      q,
      H,
      V
    );
    function J(Z) {
      const te = [
        "npm",
        "print"
      ], ne = [
        "yes",
        "no",
        "on",
        "off",
        "it",
        "that",
        "void"
      ], ee = [
        "then",
        "unless",
        "until",
        "loop",
        "of",
        "by",
        "when",
        "and",
        "or",
        "is",
        "isnt",
        "not",
        "it",
        "that",
        "otherwise",
        "from",
        "to",
        "til",
        "fallthrough",
        "case",
        "enum",
        "native",
        "list",
        "map",
        "__hasProp",
        "__extends",
        "__slice",
        "__bind",
        "__indexOf"
      ], ae = {
        keyword: O.concat(ee),
        literal: _.concat(ne),
        built_in: G.concat(te)
      }, oe = "[A-Za-z$_](?:-[0-9A-Za-z$_]|[0-9A-Za-z$_])*", se = Z.inherit(Z.TITLE_MODE, { begin: oe }), ce = {
        className: "subst",
        begin: /#\{/,
        end: /\}/,
        keywords: ae
      }, le = {
        className: "subst",
        begin: /#[A-Za-z$_]/,
        end: /(?:-[0-9A-Za-z$_]|[0-9A-Za-z$_])*/,
        keywords: ae
      }, ue = [
        Z.BINARY_NUMBER_MODE,
        {
          className: "number",
          begin: "(\\b0[xX][a-fA-F0-9_]+)|(\\b\\d(\\d|_\\d)*(\\.(\\d(\\d|_\\d)*)?)?(_*[eE]([-+]\\d(_\\d|\\d)*)?)?[_a-z]*)",
          relevance: 0,
          starts: {
            end: "(\\s*/)?",
            relevance: 0
          }
          // a number tries to eat the following slash to prevent treating it as a regexp
        },
        {
          className: "string",
          variants: [
            {
              begin: /'''/,
              end: /'''/,
              contains: [Z.BACKSLASH_ESCAPE]
            },
            {
              begin: /'/,
              end: /'/,
              contains: [Z.BACKSLASH_ESCAPE]
            },
            {
              begin: /"""/,
              end: /"""/,
              contains: [
                Z.BACKSLASH_ESCAPE,
                ce,
                le
              ]
            },
            {
              begin: /"/,
              end: /"/,
              contains: [
                Z.BACKSLASH_ESCAPE,
                ce,
                le
              ]
            },
            {
              begin: /\\/,
              end: /(\s|$)/,
              excludeEnd: !0
            }
          ]
        },
        {
          className: "regexp",
          variants: [
            {
              begin: "//",
              end: "//[gim]*",
              contains: [
                ce,
                Z.HASH_COMMENT_MODE
              ]
            },
            {
              // regex can't start with space to parse x / 2 / 3 as two divisions
              // regex can't start with *, and it supports an "illegal" in the main mode
              begin: /\/(?![ *])(\\.|[^\\\n])*?\/[gim]*(?=\W)/
            }
          ]
        },
        { begin: "@" + oe },
        {
          begin: "``",
          end: "``",
          excludeBegin: !0,
          excludeEnd: !0,
          subLanguage: "javascript"
        }
      ];
      ce.contains = ue;
      const de = {
        className: "params",
        begin: "\\(",
        returnBegin: !0,
        /* We need another contained nameless mode to not have every nested
        pair of parens to be called "params" */
        contains: [
          {
            begin: /\(/,
            end: /\)/,
            keywords: ae,
            contains: ["self"].concat(ue)
          }
        ]
      }, fe = { begin: "(#=>|=>|\\|>>|-?->|!->)" }, he = {
        variants: [
          { match: [
            /class\s+/,
            oe,
            /\s+extends\s+/,
            oe
          ] },
          { match: [
            /class\s+/,
            oe
          ] }
        ],
        scope: {
          2: "title.class",
          4: "title.class.inherited"
        },
        keywords: ae
      };
      return {
        name: "LiveScript",
        aliases: ["ls"],
        keywords: ae,
        illegal: /\/\*/,
        contains: ue.concat([
          Z.COMMENT("\\/\\*", "\\*\\/"),
          Z.HASH_COMMENT_MODE,
          fe,
          // relevance booster
          {
            className: "function",
            contains: [
              se,
              de
            ],
            returnBegin: !0,
            variants: [
              {
                begin: "(" + oe + "\\s*(?:=|:=)\\s*)?(\\(.*\\)\\s*)?\\B->\\*?",
                end: "->\\*?"
              },
              {
                begin: "(" + oe + "\\s*(?:=|:=)\\s*)?!?(\\(.*\\)\\s*)?\\B[-~]{1,2}>\\*?",
                end: "[-~]{1,2}>\\*?"
              },
              {
                begin: "(" + oe + "\\s*(?:=|:=)\\s*)?(\\(.*\\)\\s*)?\\B!?[-~]{1,2}>\\*?",
                end: "!?[-~]{1,2}>\\*?"
              }
            ]
          },
          he,
          {
            begin: oe + ":",
            end: ":",
            returnBegin: !0,
            returnEnd: !0,
            relevance: 0
          }
        ])
      };
    }
    return livescript_1 = J, livescript_1;
  }
  var llvm_1, hasRequiredLlvm;
  function requireLlvm() {
    if (hasRequiredLlvm) return llvm_1;
    hasRequiredLlvm = 1;
    function O(_) {
      const H = _.regex, V = /([-a-zA-Z$._][\w$.-]*)/, q = {
        className: "type",
        begin: /\bi\d+(?=\s|\b)/
      }, G = {
        className: "operator",
        relevance: 0,
        begin: /=/
      }, J = {
        className: "punctuation",
        relevance: 0,
        begin: /,/
      }, Z = {
        className: "number",
        variants: [
          { begin: /[su]?0[xX][KMLHR]?[a-fA-F0-9]+/ },
          { begin: /[-+]?\d+(?:[.]\d+)?(?:[eE][-+]?\d+(?:[.]\d+)?)?/ }
        ],
        relevance: 0
      }, te = {
        className: "symbol",
        variants: [
          { begin: /^\s*[a-z]+:/ }
          // labels
        ],
        relevance: 0
      }, ne = {
        className: "variable",
        variants: [
          { begin: H.concat(/%/, V) },
          { begin: /%\d+/ },
          { begin: /#\d+/ }
        ]
      }, ee = {
        className: "title",
        variants: [
          { begin: H.concat(/@/, V) },
          { begin: /@\d+/ },
          { begin: H.concat(/!/, V) },
          { begin: H.concat(/!\d+/, V) },
          // https://llvm.org/docs/LangRef.html#namedmetadatastructure
          // obviously a single digit can also be used in this fashion
          { begin: /!\d+/ }
        ]
      };
      return {
        name: "LLVM IR",
        // TODO: split into different categories of keywords
        keywords: {
          keyword: "begin end true false declare define global constant private linker_private internal available_externally linkonce linkonce_odr weak weak_odr appending dllimport dllexport common default hidden protected extern_weak external thread_local zeroinitializer undef null to tail target triple datalayout volatile nuw nsw nnan ninf nsz arcp fast exact inbounds align addrspace section alias module asm sideeffect gc dbg linker_private_weak attributes blockaddress initialexec localdynamic localexec prefix unnamed_addr ccc fastcc coldcc x86_stdcallcc x86_fastcallcc arm_apcscc arm_aapcscc arm_aapcs_vfpcc ptx_device ptx_kernel intel_ocl_bicc msp430_intrcc spir_func spir_kernel x86_64_sysvcc x86_64_win64cc x86_thiscallcc cc c signext zeroext inreg sret nounwind noreturn noalias nocapture byval nest readnone readonly inlinehint noinline alwaysinline optsize ssp sspreq noredzone noimplicitfloat naked builtin cold nobuiltin noduplicate nonlazybind optnone returns_twice sanitize_address sanitize_memory sanitize_thread sspstrong uwtable returned type opaque eq ne slt sgt sle sge ult ugt ule uge oeq one olt ogt ole oge ord uno ueq une x acq_rel acquire alignstack atomic catch cleanup filter inteldialect max min monotonic nand personality release seq_cst singlethread umax umin unordered xchg add fadd sub fsub mul fmul udiv sdiv fdiv urem srem frem shl lshr ashr and or xor icmp fcmp phi call trunc zext sext fptrunc fpext uitofp sitofp fptoui fptosi inttoptr ptrtoint bitcast addrspacecast select va_arg ret br switch invoke unwind unreachable indirectbr landingpad resume malloc alloca free load store getelementptr extractelement insertelement shufflevector getresult extractvalue insertvalue atomicrmw cmpxchg fence argmemonly",
          type: "void half bfloat float double fp128 x86_fp80 ppc_fp128 x86_amx x86_mmx ptr label token metadata opaque"
        },
        contains: [
          q,
          // this matches "empty comments"...
          // ...because it's far more likely this is a statement terminator in
          // another language than an actual comment
          _.COMMENT(/;\s*$/, null, { relevance: 0 }),
          _.COMMENT(/;/, /$/),
          {
            className: "string",
            begin: /"/,
            end: /"/,
            contains: [
              {
                className: "char.escape",
                match: /\\\d\d/
              }
            ]
          },
          ee,
          J,
          G,
          ne,
          te,
          Z
        ]
      };
    }
    return llvm_1 = O, llvm_1;
  }
  var lsl_1, hasRequiredLsl;
  function requireLsl() {
    if (hasRequiredLsl) return lsl_1;
    hasRequiredLsl = 1;
    function O(_) {
      const V = {
        className: "string",
        begin: '"',
        end: '"',
        contains: [{
          className: "subst",
          begin: /\\[tn"\\]/
        }]
      }, q = {
        className: "number",
        relevance: 0,
        begin: _.C_NUMBER_RE
      }, G = {
        className: "literal",
        variants: [
          { begin: "\\b(PI|TWO_PI|PI_BY_TWO|DEG_TO_RAD|RAD_TO_DEG|SQRT2)\\b" },
          { begin: "\\b(XP_ERROR_(EXPERIENCES_DISABLED|EXPERIENCE_(DISABLED|SUSPENDED)|INVALID_(EXPERIENCE|PARAMETERS)|KEY_NOT_FOUND|MATURITY_EXCEEDED|NONE|NOT_(FOUND|PERMITTED(_LAND)?)|NO_EXPERIENCE|QUOTA_EXCEEDED|RETRY_UPDATE|STORAGE_EXCEPTION|STORE_DISABLED|THROTTLED|UNKNOWN_ERROR)|JSON_APPEND|STATUS_(PHYSICS|ROTATE_[XYZ]|PHANTOM|SANDBOX|BLOCK_GRAB(_OBJECT)?|(DIE|RETURN)_AT_EDGE|CAST_SHADOWS|OK|MALFORMED_PARAMS|TYPE_MISMATCH|BOUNDS_ERROR|NOT_(FOUND|SUPPORTED)|INTERNAL_ERROR|WHITELIST_FAILED)|AGENT(_(BY_(LEGACY_|USER)NAME|FLYING|ATTACHMENTS|SCRIPTED|MOUSELOOK|SITTING|ON_OBJECT|AWAY|WALKING|IN_AIR|TYPING|CROUCHING|BUSY|ALWAYS_RUN|AUTOPILOT|LIST_(PARCEL(_OWNER)?|REGION)))?|CAMERA_(PITCH|DISTANCE|BEHINDNESS_(ANGLE|LAG)|(FOCUS|POSITION)(_(THRESHOLD|LOCKED|LAG))?|FOCUS_OFFSET|ACTIVE)|ANIM_ON|LOOP|REVERSE|PING_PONG|SMOOTH|ROTATE|SCALE|ALL_SIDES|LINK_(ROOT|SET|ALL_(OTHERS|CHILDREN)|THIS)|ACTIVE|PASS(IVE|_(ALWAYS|IF_NOT_HANDLED|NEVER))|SCRIPTED|CONTROL_(FWD|BACK|(ROT_)?(LEFT|RIGHT)|UP|DOWN|(ML_)?LBUTTON)|PERMISSION_(RETURN_OBJECTS|DEBIT|OVERRIDE_ANIMATIONS|SILENT_ESTATE_MANAGEMENT|TAKE_CONTROLS|TRIGGER_ANIMATION|ATTACH|CHANGE_LINKS|(CONTROL|TRACK)_CAMERA|TELEPORT)|INVENTORY_(TEXTURE|SOUND|OBJECT|SCRIPT|LANDMARK|CLOTHING|NOTECARD|BODYPART|ANIMATION|GESTURE|ALL|NONE)|CHANGED_(INVENTORY|COLOR|SHAPE|SCALE|TEXTURE|LINK|ALLOWED_DROP|OWNER|REGION(_START)?|TELEPORT|MEDIA)|OBJECT_(CLICK_ACTION|HOVER_HEIGHT|LAST_OWNER_ID|(PHYSICS|SERVER|STREAMING)_COST|UNKNOWN_DETAIL|CHARACTER_TIME|PHANTOM|PHYSICS|TEMP_(ATTACHED|ON_REZ)|NAME|DESC|POS|PRIM_(COUNT|EQUIVALENCE)|RETURN_(PARCEL(_OWNER)?|REGION)|REZZER_KEY|ROO?T|VELOCITY|OMEGA|OWNER|GROUP(_TAG)?|CREATOR|ATTACHED_(POINT|SLOTS_AVAILABLE)|RENDER_WEIGHT|(BODY_SHAPE|PATHFINDING)_TYPE|(RUNNING|TOTAL)_SCRIPT_COUNT|TOTAL_INVENTORY_COUNT|SCRIPT_(MEMORY|TIME))|TYPE_(INTEGER|FLOAT|STRING|KEY|VECTOR|ROTATION|INVALID)|(DEBUG|PUBLIC)_CHANNEL|ATTACH_(AVATAR_CENTER|CHEST|HEAD|BACK|PELVIS|MOUTH|CHIN|NECK|NOSE|BELLY|[LR](SHOULDER|HAND|FOOT|EAR|EYE|[UL](ARM|LEG)|HIP)|(LEFT|RIGHT)_PEC|HUD_(CENTER_[12]|TOP_(RIGHT|CENTER|LEFT)|BOTTOM(_(RIGHT|LEFT))?)|[LR]HAND_RING1|TAIL_(BASE|TIP)|[LR]WING|FACE_(JAW|[LR]EAR|[LR]EYE|TOUNGE)|GROIN|HIND_[LR]FOOT)|LAND_(LEVEL|RAISE|LOWER|SMOOTH|NOISE|REVERT)|DATA_(ONLINE|NAME|BORN|SIM_(POS|STATUS|RATING)|PAYINFO)|PAYMENT_INFO_(ON_FILE|USED)|REMOTE_DATA_(CHANNEL|REQUEST|REPLY)|PSYS_(PART_(BF_(ZERO|ONE(_MINUS_(DEST_COLOR|SOURCE_(ALPHA|COLOR)))?|DEST_COLOR|SOURCE_(ALPHA|COLOR))|BLEND_FUNC_(DEST|SOURCE)|FLAGS|(START|END)_(COLOR|ALPHA|SCALE|GLOW)|MAX_AGE|(RIBBON|WIND|INTERP_(COLOR|SCALE)|BOUNCE|FOLLOW_(SRC|VELOCITY)|TARGET_(POS|LINEAR)|EMISSIVE)_MASK)|SRC_(MAX_AGE|PATTERN|ANGLE_(BEGIN|END)|BURST_(RATE|PART_COUNT|RADIUS|SPEED_(MIN|MAX))|ACCEL|TEXTURE|TARGET_KEY|OMEGA|PATTERN_(DROP|EXPLODE|ANGLE(_CONE(_EMPTY)?)?)))|VEHICLE_(REFERENCE_FRAME|TYPE_(NONE|SLED|CAR|BOAT|AIRPLANE|BALLOON)|(LINEAR|ANGULAR)_(FRICTION_TIMESCALE|MOTOR_DIRECTION)|LINEAR_MOTOR_OFFSET|HOVER_(HEIGHT|EFFICIENCY|TIMESCALE)|BUOYANCY|(LINEAR|ANGULAR)_(DEFLECTION_(EFFICIENCY|TIMESCALE)|MOTOR_(DECAY_)?TIMESCALE)|VERTICAL_ATTRACTION_(EFFICIENCY|TIMESCALE)|BANKING_(EFFICIENCY|MIX|TIMESCALE)|FLAG_(NO_DEFLECTION_UP|LIMIT_(ROLL_ONLY|MOTOR_UP)|HOVER_((WATER|TERRAIN|UP)_ONLY|GLOBAL_HEIGHT)|MOUSELOOK_(STEER|BANK)|CAMERA_DECOUPLED))|PRIM_(ALLOW_UNSIT|ALPHA_MODE(_(BLEND|EMISSIVE|MASK|NONE))?|NORMAL|SPECULAR|TYPE(_(BOX|CYLINDER|PRISM|SPHERE|TORUS|TUBE|RING|SCULPT))?|HOLE_(DEFAULT|CIRCLE|SQUARE|TRIANGLE)|MATERIAL(_(STONE|METAL|GLASS|WOOD|FLESH|PLASTIC|RUBBER))?|SHINY_(NONE|LOW|MEDIUM|HIGH)|BUMP_(NONE|BRIGHT|DARK|WOOD|BARK|BRICKS|CHECKER|CONCRETE|TILE|STONE|DISKS|GRAVEL|BLOBS|SIDING|LARGETILE|STUCCO|SUCTION|WEAVE)|TEXGEN_(DEFAULT|PLANAR)|SCRIPTED_SIT_ONLY|SCULPT_(TYPE_(SPHERE|TORUS|PLANE|CYLINDER|MASK)|FLAG_(MIRROR|INVERT))|PHYSICS(_(SHAPE_(CONVEX|NONE|PRIM|TYPE)))?|(POS|ROT)_LOCAL|SLICE|TEXT|FLEXIBLE|POINT_LIGHT|TEMP_ON_REZ|PHANTOM|POSITION|SIT_TARGET|SIZE|ROTATION|TEXTURE|NAME|OMEGA|DESC|LINK_TARGET|COLOR|BUMP_SHINY|FULLBRIGHT|TEXGEN|GLOW|MEDIA_(ALT_IMAGE_ENABLE|CONTROLS|(CURRENT|HOME)_URL|AUTO_(LOOP|PLAY|SCALE|ZOOM)|FIRST_CLICK_INTERACT|(WIDTH|HEIGHT)_PIXELS|WHITELIST(_ENABLE)?|PERMS_(INTERACT|CONTROL)|PARAM_MAX|CONTROLS_(STANDARD|MINI)|PERM_(NONE|OWNER|GROUP|ANYONE)|MAX_(URL_LENGTH|WHITELIST_(SIZE|COUNT)|(WIDTH|HEIGHT)_PIXELS)))|MASK_(BASE|OWNER|GROUP|EVERYONE|NEXT)|PERM_(TRANSFER|MODIFY|COPY|MOVE|ALL)|PARCEL_(MEDIA_COMMAND_(STOP|PAUSE|PLAY|LOOP|TEXTURE|URL|TIME|AGENT|UNLOAD|AUTO_ALIGN|TYPE|SIZE|DESC|LOOP_SET)|FLAG_(ALLOW_(FLY|(GROUP_)?SCRIPTS|LANDMARK|TERRAFORM|DAMAGE|CREATE_(GROUP_)?OBJECTS)|USE_(ACCESS_(GROUP|LIST)|BAN_LIST|LAND_PASS_LIST)|LOCAL_SOUND_ONLY|RESTRICT_PUSHOBJECT|ALLOW_(GROUP|ALL)_OBJECT_ENTRY)|COUNT_(TOTAL|OWNER|GROUP|OTHER|SELECTED|TEMP)|DETAILS_(NAME|DESC|OWNER|GROUP|AREA|ID|SEE_AVATARS))|LIST_STAT_(MAX|MIN|MEAN|MEDIAN|STD_DEV|SUM(_SQUARES)?|NUM_COUNT|GEOMETRIC_MEAN|RANGE)|PAY_(HIDE|DEFAULT)|REGION_FLAG_(ALLOW_DAMAGE|FIXED_SUN|BLOCK_TERRAFORM|SANDBOX|DISABLE_(COLLISIONS|PHYSICS)|BLOCK_FLY|ALLOW_DIRECT_TELEPORT|RESTRICT_PUSHOBJECT)|HTTP_(METHOD|MIMETYPE|BODY_(MAXLENGTH|TRUNCATED)|CUSTOM_HEADER|PRAGMA_NO_CACHE|VERBOSE_THROTTLE|VERIFY_CERT)|SIT_(INVALID_(AGENT|LINK_OBJECT)|NO(T_EXPERIENCE|_(ACCESS|EXPERIENCE_PERMISSION|SIT_TARGET)))|STRING_(TRIM(_(HEAD|TAIL))?)|CLICK_ACTION_(NONE|TOUCH|SIT|BUY|PAY|OPEN(_MEDIA)?|PLAY|ZOOM)|TOUCH_INVALID_FACE|PROFILE_(NONE|SCRIPT_MEMORY)|RC_(DATA_FLAGS|DETECT_PHANTOM|GET_(LINK_NUM|NORMAL|ROOT_KEY)|MAX_HITS|REJECT_(TYPES|AGENTS|(NON)?PHYSICAL|LAND))|RCERR_(CAST_TIME_EXCEEDED|SIM_PERF_LOW|UNKNOWN)|ESTATE_ACCESS_(ALLOWED_(AGENT|GROUP)_(ADD|REMOVE)|BANNED_AGENT_(ADD|REMOVE))|DENSITY|FRICTION|RESTITUTION|GRAVITY_MULTIPLIER|KFM_(COMMAND|CMD_(PLAY|STOP|PAUSE)|MODE|FORWARD|LOOP|PING_PONG|REVERSE|DATA|ROTATION|TRANSLATION)|ERR_(GENERIC|PARCEL_PERMISSIONS|MALFORMED_PARAMS|RUNTIME_PERMISSIONS|THROTTLED)|CHARACTER_(CMD_((SMOOTH_)?STOP|JUMP)|DESIRED_(TURN_)?SPEED|RADIUS|STAY_WITHIN_PARCEL|LENGTH|ORIENTATION|ACCOUNT_FOR_SKIPPED_FRAMES|AVOIDANCE_MODE|TYPE(_([ABCD]|NONE))?|MAX_(DECEL|TURN_RADIUS|(ACCEL|SPEED)))|PURSUIT_(OFFSET|FUZZ_FACTOR|GOAL_TOLERANCE|INTERCEPT)|REQUIRE_LINE_OF_SIGHT|FORCE_DIRECT_PATH|VERTICAL|HORIZONTAL|AVOID_(CHARACTERS|DYNAMIC_OBSTACLES|NONE)|PU_(EVADE_(HIDDEN|SPOTTED)|FAILURE_(DYNAMIC_PATHFINDING_DISABLED|INVALID_(GOAL|START)|NO_(NAVMESH|VALID_DESTINATION)|OTHER|TARGET_GONE|(PARCEL_)?UNREACHABLE)|(GOAL|SLOWDOWN_DISTANCE)_REACHED)|TRAVERSAL_TYPE(_(FAST|NONE|SLOW))?|CONTENT_TYPE_(ATOM|FORM|HTML|JSON|LLSD|RSS|TEXT|XHTML|XML)|GCNP_(RADIUS|STATIC)|(PATROL|WANDER)_PAUSE_AT_WAYPOINTS|OPT_(AVATAR|CHARACTER|EXCLUSION_VOLUME|LEGACY_LINKSET|MATERIAL_VOLUME|OTHER|STATIC_OBSTACLE|WALKABLE)|SIM_STAT_PCT_CHARS_STEPPED)\\b" },
          { begin: "\\b(FALSE|TRUE)\\b" },
          { begin: "\\b(ZERO_ROTATION)\\b" },
          { begin: "\\b(EOF|JSON_(ARRAY|DELETE|FALSE|INVALID|NULL|NUMBER|OBJECT|STRING|TRUE)|NULL_KEY|TEXTURE_(BLANK|DEFAULT|MEDIA|PLYWOOD|TRANSPARENT)|URL_REQUEST_(GRANTED|DENIED))\\b" },
          { begin: "\\b(ZERO_VECTOR|TOUCH_INVALID_(TEXCOORD|VECTOR))\\b" }
        ]
      }, J = {
        className: "built_in",
        begin: "\\b(ll(AgentInExperience|(Create|DataSize|Delete|KeyCount|Keys|Read|Update)KeyValue|GetExperience(Details|ErrorMessage)|ReturnObjectsBy(ID|Owner)|Json(2List|[GS]etValue|ValueType)|Sin|Cos|Tan|Atan2|Sqrt|Pow|Abs|Fabs|Frand|Floor|Ceil|Round|Vec(Mag|Norm|Dist)|Rot(Between|2(Euler|Fwd|Left|Up))|(Euler|Axes)2Rot|Whisper|(Region|Owner)?Say|Shout|Listen(Control|Remove)?|Sensor(Repeat|Remove)?|Detected(Name|Key|Owner|Type|Pos|Vel|Grab|Rot|Group|LinkNumber)|Die|Ground|Wind|([GS]et)(AnimationOverride|MemoryLimit|PrimMediaParams|ParcelMusicURL|Object(Desc|Name)|PhysicsMaterial|Status|Scale|Color|Alpha|Texture|Pos|Rot|Force|Torque)|ResetAnimationOverride|(Scale|Offset|Rotate)Texture|(Rot)?Target(Remove)?|(Stop)?MoveToTarget|Apply(Rotational)?Impulse|Set(KeyframedMotion|ContentType|RegionPos|(Angular)?Velocity|Buoyancy|HoverHeight|ForceAndTorque|TimerEvent|ScriptState|Damage|TextureAnim|Sound(Queueing|Radius)|Vehicle(Type|(Float|Vector|Rotation)Param)|(Touch|Sit)?Text|Camera(Eye|At)Offset|PrimitiveParams|ClickAction|Link(Alpha|Color|PrimitiveParams(Fast)?|Texture(Anim)?|Camera|Media)|RemoteScriptAccessPin|PayPrice|LocalRot)|ScaleByFactor|Get((Max|Min)ScaleFactor|ClosestNavPoint|StaticPath|SimStats|Env|PrimitiveParams|Link(PrimitiveParams|Number(OfSides)?|Key|Name|Media)|HTTPHeader|FreeURLs|Object(Details|PermMask|PrimCount)|Parcel(MaxPrims|Details|Prim(Count|Owners))|Attached(List)?|(SPMax|Free|Used)Memory|Region(Name|TimeDilation|FPS|Corner|AgentCount)|Root(Position|Rotation)|UnixTime|(Parcel|Region)Flags|(Wall|GMT)clock|SimulatorHostname|BoundingBox|GeometricCenter|Creator|NumberOf(Prims|NotecardLines|Sides)|Animation(List)?|(Camera|Local)(Pos|Rot)|Vel|Accel|Omega|Time(stamp|OfDay)|(Object|CenterOf)?Mass|MassMKS|Energy|Owner|(Owner)?Key|SunDirection|Texture(Offset|Scale|Rot)|Inventory(Number|Name|Key|Type|Creator|PermMask)|Permissions(Key)?|StartParameter|List(Length|EntryType)|Date|Agent(Size|Info|Language|List)|LandOwnerAt|NotecardLine|Script(Name|State))|(Get|Reset|GetAndReset)Time|PlaySound(Slave)?|LoopSound(Master|Slave)?|(Trigger|Stop|Preload)Sound|((Get|Delete)Sub|Insert)String|To(Upper|Lower)|Give(InventoryList|Money)|RezObject|(Stop)?LookAt|Sleep|CollisionFilter|(Take|Release)Controls|DetachFromAvatar|AttachToAvatar(Temp)?|InstantMessage|(GetNext)?Email|StopHover|MinEventDelay|RotLookAt|String(Length|Trim)|(Start|Stop)Animation|TargetOmega|Request(Experience)?Permissions|(Create|Break)Link|BreakAllLinks|(Give|Remove)Inventory|Water|PassTouches|Request(Agent|Inventory)Data|TeleportAgent(Home|GlobalCoords)?|ModifyLand|CollisionSound|ResetScript|MessageLinked|PushObject|PassCollisions|AxisAngle2Rot|Rot2(Axis|Angle)|A(cos|sin)|AngleBetween|AllowInventoryDrop|SubStringIndex|List2(CSV|Integer|Json|Float|String|Key|Vector|Rot|List(Strided)?)|DeleteSubList|List(Statistics|Sort|Randomize|(Insert|Find|Replace)List)|EdgeOfWorld|AdjustSoundVolume|Key2Name|TriggerSoundLimited|EjectFromLand|(CSV|ParseString)2List|OverMyLand|SameGroup|UnSit|Ground(Slope|Normal|Contour)|GroundRepel|(Set|Remove)VehicleFlags|SitOnLink|(AvatarOn)?(Link)?SitTarget|Script(Danger|Profiler)|Dialog|VolumeDetect|ResetOtherScript|RemoteLoadScriptPin|(Open|Close)RemoteDataChannel|SendRemoteData|RemoteDataReply|(Integer|String)ToBase64|XorBase64|Log(10)?|Base64To(String|Integer)|ParseStringKeepNulls|RezAtRoot|RequestSimulatorData|ForceMouselook|(Load|Release|(E|Une)scape)URL|ParcelMedia(CommandList|Query)|ModPow|MapDestination|(RemoveFrom|AddTo|Reset)Land(Pass|Ban)List|(Set|Clear)CameraParams|HTTP(Request|Response)|TextBox|DetectedTouch(UV|Face|Pos|(N|Bin)ormal|ST)|(MD5|SHA1|DumpList2)String|Request(Secure)?URL|Clear(Prim|Link)Media|(Link)?ParticleSystem|(Get|Request)(Username|DisplayName)|RegionSayTo|CastRay|GenerateKey|TransferLindenDollars|ManageEstateAccess|(Create|Delete)Character|ExecCharacterCmd|Evade|FleeFrom|NavigateTo|PatrolPoints|Pursue|UpdateCharacter|WanderWithin))\\b"
      };
      return {
        name: "LSL (Linden Scripting Language)",
        illegal: ":",
        contains: [
          V,
          {
            className: "comment",
            variants: [
              _.COMMENT("//", "$"),
              _.COMMENT("/\\*", "\\*/")
            ],
            relevance: 0
          },
          q,
          {
            className: "section",
            variants: [
              { begin: "\\b(state|default)\\b" },
              { begin: "\\b(state_(entry|exit)|touch(_(start|end))?|(land_)?collision(_(start|end))?|timer|listen|(no_)?sensor|control|(not_)?at_(rot_)?target|money|email|experience_permissions(_denied)?|run_time_permissions|changed|attach|dataserver|moving_(start|end)|link_message|(on|object)_rez|remote_data|http_re(sponse|quest)|path_update|transaction_result)\\b" }
            ]
          },
          J,
          G,
          {
            className: "type",
            begin: "\\b(integer|float|string|key|vector|quaternion|rotation|list)\\b"
          }
        ]
      };
    }
    return lsl_1 = O, lsl_1;
  }
  var mathematica_1, hasRequiredMathematica;
  function requireMathematica() {
    if (hasRequiredMathematica) return mathematica_1;
    hasRequiredMathematica = 1;
    const O = [
      "AASTriangle",
      "AbelianGroup",
      "Abort",
      "AbortKernels",
      "AbortProtect",
      "AbortScheduledTask",
      "Above",
      "Abs",
      "AbsArg",
      "AbsArgPlot",
      "Absolute",
      "AbsoluteCorrelation",
      "AbsoluteCorrelationFunction",
      "AbsoluteCurrentValue",
      "AbsoluteDashing",
      "AbsoluteFileName",
      "AbsoluteOptions",
      "AbsolutePointSize",
      "AbsoluteThickness",
      "AbsoluteTime",
      "AbsoluteTiming",
      "AcceptanceThreshold",
      "AccountingForm",
      "Accumulate",
      "Accuracy",
      "AccuracyGoal",
      "AcousticAbsorbingValue",
      "AcousticImpedanceValue",
      "AcousticNormalVelocityValue",
      "AcousticPDEComponent",
      "AcousticPressureCondition",
      "AcousticRadiationValue",
      "AcousticSoundHardValue",
      "AcousticSoundSoftCondition",
      "ActionDelay",
      "ActionMenu",
      "ActionMenuBox",
      "ActionMenuBoxOptions",
      "Activate",
      "Active",
      "ActiveClassification",
      "ActiveClassificationObject",
      "ActiveItem",
      "ActivePrediction",
      "ActivePredictionObject",
      "ActiveStyle",
      "AcyclicGraphQ",
      "AddOnHelpPath",
      "AddSides",
      "AddTo",
      "AddToSearchIndex",
      "AddUsers",
      "AdjacencyGraph",
      "AdjacencyList",
      "AdjacencyMatrix",
      "AdjacentMeshCells",
      "Adjugate",
      "AdjustmentBox",
      "AdjustmentBoxOptions",
      "AdjustTimeSeriesForecast",
      "AdministrativeDivisionData",
      "AffineHalfSpace",
      "AffineSpace",
      "AffineStateSpaceModel",
      "AffineTransform",
      "After",
      "AggregatedEntityClass",
      "AggregationLayer",
      "AircraftData",
      "AirportData",
      "AirPressureData",
      "AirSoundAttenuation",
      "AirTemperatureData",
      "AiryAi",
      "AiryAiPrime",
      "AiryAiZero",
      "AiryBi",
      "AiryBiPrime",
      "AiryBiZero",
      "AlgebraicIntegerQ",
      "AlgebraicNumber",
      "AlgebraicNumberDenominator",
      "AlgebraicNumberNorm",
      "AlgebraicNumberPolynomial",
      "AlgebraicNumberTrace",
      "AlgebraicRules",
      "AlgebraicRulesData",
      "Algebraics",
      "AlgebraicUnitQ",
      "Alignment",
      "AlignmentMarker",
      "AlignmentPoint",
      "All",
      "AllowAdultContent",
      "AllowChatServices",
      "AllowedCloudExtraParameters",
      "AllowedCloudParameterExtensions",
      "AllowedDimensions",
      "AllowedFrequencyRange",
      "AllowedHeads",
      "AllowGroupClose",
      "AllowIncomplete",
      "AllowInlineCells",
      "AllowKernelInitialization",
      "AllowLooseGrammar",
      "AllowReverseGroupClose",
      "AllowScriptLevelChange",
      "AllowVersionUpdate",
      "AllTrue",
      "Alphabet",
      "AlphabeticOrder",
      "AlphabeticSort",
      "AlphaChannel",
      "AlternateImage",
      "AlternatingFactorial",
      "AlternatingGroup",
      "AlternativeHypothesis",
      "Alternatives",
      "AltitudeMethod",
      "AmbientLight",
      "AmbiguityFunction",
      "AmbiguityList",
      "Analytic",
      "AnatomyData",
      "AnatomyForm",
      "AnatomyPlot3D",
      "AnatomySkinStyle",
      "AnatomyStyling",
      "AnchoredSearch",
      "And",
      "AndersonDarlingTest",
      "AngerJ",
      "AngleBisector",
      "AngleBracket",
      "AnglePath",
      "AnglePath3D",
      "AngleVector",
      "AngularGauge",
      "Animate",
      "AnimatedImage",
      "AnimationCycleOffset",
      "AnimationCycleRepetitions",
      "AnimationDirection",
      "AnimationDisplayTime",
      "AnimationRate",
      "AnimationRepetitions",
      "AnimationRunning",
      "AnimationRunTime",
      "AnimationTimeIndex",
      "AnimationVideo",
      "Animator",
      "AnimatorBox",
      "AnimatorBoxOptions",
      "AnimatorElements",
      "Annotate",
      "Annotation",
      "AnnotationDelete",
      "AnnotationKeys",
      "AnnotationRules",
      "AnnotationValue",
      "Annuity",
      "AnnuityDue",
      "Annulus",
      "AnomalyDetection",
      "AnomalyDetector",
      "AnomalyDetectorFunction",
      "Anonymous",
      "Antialiasing",
      "Antihermitian",
      "AntihermitianMatrixQ",
      "Antisymmetric",
      "AntisymmetricMatrixQ",
      "Antonyms",
      "AnyOrder",
      "AnySubset",
      "AnyTrue",
      "Apart",
      "ApartSquareFree",
      "APIFunction",
      "Appearance",
      "AppearanceElements",
      "AppearanceRules",
      "AppellF1",
      "Append",
      "AppendCheck",
      "AppendLayer",
      "AppendTo",
      "Application",
      "Apply",
      "ApplyReaction",
      "ApplySides",
      "ApplyTo",
      "ArcCos",
      "ArcCosh",
      "ArcCot",
      "ArcCoth",
      "ArcCsc",
      "ArcCsch",
      "ArcCurvature",
      "ARCHProcess",
      "ArcLength",
      "ArcSec",
      "ArcSech",
      "ArcSin",
      "ArcSinDistribution",
      "ArcSinh",
      "ArcTan",
      "ArcTanh",
      "Area",
      "Arg",
      "ArgMax",
      "ArgMin",
      "ArgumentCountQ",
      "ArgumentsOptions",
      "ARIMAProcess",
      "ArithmeticGeometricMean",
      "ARMAProcess",
      "Around",
      "AroundReplace",
      "ARProcess",
      "Array",
      "ArrayComponents",
      "ArrayDepth",
      "ArrayFilter",
      "ArrayFlatten",
      "ArrayMesh",
      "ArrayPad",
      "ArrayPlot",
      "ArrayPlot3D",
      "ArrayQ",
      "ArrayReduce",
      "ArrayResample",
      "ArrayReshape",
      "ArrayRules",
      "Arrays",
      "Arrow",
      "Arrow3DBox",
      "ArrowBox",
      "Arrowheads",
      "ASATriangle",
      "Ask",
      "AskAppend",
      "AskConfirm",
      "AskDisplay",
      "AskedQ",
      "AskedValue",
      "AskFunction",
      "AskState",
      "AskTemplateDisplay",
      "AspectRatio",
      "AspectRatioFixed",
      "Assert",
      "AssessmentFunction",
      "AssessmentResultObject",
      "AssociateTo",
      "Association",
      "AssociationFormat",
      "AssociationMap",
      "AssociationQ",
      "AssociationThread",
      "AssumeDeterministic",
      "Assuming",
      "Assumptions",
      "AstroAngularSeparation",
      "AstroBackground",
      "AstroCenter",
      "AstroDistance",
      "AstroGraphics",
      "AstroGridLines",
      "AstroGridLinesStyle",
      "AstronomicalData",
      "AstroPosition",
      "AstroProjection",
      "AstroRange",
      "AstroRangePadding",
      "AstroReferenceFrame",
      "AstroStyling",
      "AstroZoomLevel",
      "Asymptotic",
      "AsymptoticDSolveValue",
      "AsymptoticEqual",
      "AsymptoticEquivalent",
      "AsymptoticExpectation",
      "AsymptoticGreater",
      "AsymptoticGreaterEqual",
      "AsymptoticIntegrate",
      "AsymptoticLess",
      "AsymptoticLessEqual",
      "AsymptoticOutputTracker",
      "AsymptoticProbability",
      "AsymptoticProduct",
      "AsymptoticRSolveValue",
      "AsymptoticSolve",
      "AsymptoticSum",
      "Asynchronous",
      "AsynchronousTaskObject",
      "AsynchronousTasks",
      "Atom",
      "AtomCoordinates",
      "AtomCount",
      "AtomDiagramCoordinates",
      "AtomLabels",
      "AtomLabelStyle",
      "AtomList",
      "AtomQ",
      "AttachCell",
      "AttachedCell",
      "AttentionLayer",
      "Attributes",
      "Audio",
      "AudioAmplify",
      "AudioAnnotate",
      "AudioAnnotationLookup",
      "AudioBlockMap",
      "AudioCapture",
      "AudioChannelAssignment",
      "AudioChannelCombine",
      "AudioChannelMix",
      "AudioChannels",
      "AudioChannelSeparate",
      "AudioData",
      "AudioDelay",
      "AudioDelete",
      "AudioDevice",
      "AudioDistance",
      "AudioEncoding",
      "AudioFade",
      "AudioFrequencyShift",
      "AudioGenerator",
      "AudioIdentify",
      "AudioInputDevice",
      "AudioInsert",
      "AudioInstanceQ",
      "AudioIntervals",
      "AudioJoin",
      "AudioLabel",
      "AudioLength",
      "AudioLocalMeasurements",
      "AudioLooping",
      "AudioLoudness",
      "AudioMeasurements",
      "AudioNormalize",
      "AudioOutputDevice",
      "AudioOverlay",
      "AudioPad",
      "AudioPan",
      "AudioPartition",
      "AudioPause",
      "AudioPitchShift",
      "AudioPlay",
      "AudioPlot",
      "AudioQ",
      "AudioRecord",
      "AudioReplace",
      "AudioResample",
      "AudioReverb",
      "AudioReverse",
      "AudioSampleRate",
      "AudioSpectralMap",
      "AudioSpectralTransformation",
      "AudioSplit",
      "AudioStop",
      "AudioStream",
      "AudioStreams",
      "AudioTimeStretch",
      "AudioTrackApply",
      "AudioTrackSelection",
      "AudioTrim",
      "AudioType",
      "AugmentedPolyhedron",
      "AugmentedSymmetricPolynomial",
      "Authenticate",
      "Authentication",
      "AuthenticationDialog",
      "AutoAction",
      "Autocomplete",
      "AutocompletionFunction",
      "AutoCopy",
      "AutocorrelationTest",
      "AutoDelete",
      "AutoEvaluateEvents",
      "AutoGeneratedPackage",
      "AutoIndent",
      "AutoIndentSpacings",
      "AutoItalicWords",
      "AutoloadPath",
      "AutoMatch",
      "Automatic",
      "AutomaticImageSize",
      "AutoMultiplicationSymbol",
      "AutoNumberFormatting",
      "AutoOpenNotebooks",
      "AutoOpenPalettes",
      "AutoOperatorRenderings",
      "AutoQuoteCharacters",
      "AutoRefreshed",
      "AutoRemove",
      "AutorunSequencing",
      "AutoScaling",
      "AutoScroll",
      "AutoSpacing",
      "AutoStyleOptions",
      "AutoStyleWords",
      "AutoSubmitting",
      "Axes",
      "AxesEdge",
      "AxesLabel",
      "AxesOrigin",
      "AxesStyle",
      "AxiomaticTheory",
      "Axis",
      "Axis3DBox",
      "Axis3DBoxOptions",
      "AxisBox",
      "AxisBoxOptions",
      "AxisLabel",
      "AxisObject",
      "AxisStyle",
      "BabyMonsterGroupB",
      "Back",
      "BackFaceColor",
      "BackFaceGlowColor",
      "BackFaceOpacity",
      "BackFaceSpecularColor",
      "BackFaceSpecularExponent",
      "BackFaceSurfaceAppearance",
      "BackFaceTexture",
      "Background",
      "BackgroundAppearance",
      "BackgroundTasksSettings",
      "Backslash",
      "Backsubstitution",
      "Backward",
      "Ball",
      "Band",
      "BandpassFilter",
      "BandstopFilter",
      "BarabasiAlbertGraphDistribution",
      "BarChart",
      "BarChart3D",
      "BarcodeImage",
      "BarcodeRecognize",
      "BaringhausHenzeTest",
      "BarLegend",
      "BarlowProschanImportance",
      "BarnesG",
      "BarOrigin",
      "BarSpacing",
      "BartlettHannWindow",
      "BartlettWindow",
      "BaseDecode",
      "BaseEncode",
      "BaseForm",
      "Baseline",
      "BaselinePosition",
      "BaseStyle",
      "BasicRecurrentLayer",
      "BatchNormalizationLayer",
      "BatchSize",
      "BatesDistribution",
      "BattleLemarieWavelet",
      "BayesianMaximization",
      "BayesianMaximizationObject",
      "BayesianMinimization",
      "BayesianMinimizationObject",
      "Because",
      "BeckmannDistribution",
      "Beep",
      "Before",
      "Begin",
      "BeginDialogPacket",
      "BeginPackage",
      "BellB",
      "BellY",
      "Below",
      "BenfordDistribution",
      "BeniniDistribution",
      "BenktanderGibratDistribution",
      "BenktanderWeibullDistribution",
      "BernoulliB",
      "BernoulliDistribution",
      "BernoulliGraphDistribution",
      "BernoulliProcess",
      "BernsteinBasis",
      "BesagL",
      "BesselFilterModel",
      "BesselI",
      "BesselJ",
      "BesselJZero",
      "BesselK",
      "BesselY",
      "BesselYZero",
      "Beta",
      "BetaBinomialDistribution",
      "BetaDistribution",
      "BetaNegativeBinomialDistribution",
      "BetaPrimeDistribution",
      "BetaRegularized",
      "Between",
      "BetweennessCentrality",
      "Beveled",
      "BeveledPolyhedron",
      "BezierCurve",
      "BezierCurve3DBox",
      "BezierCurve3DBoxOptions",
      "BezierCurveBox",
      "BezierCurveBoxOptions",
      "BezierFunction",
      "BilateralFilter",
      "BilateralLaplaceTransform",
      "BilateralZTransform",
      "Binarize",
      "BinaryDeserialize",
      "BinaryDistance",
      "BinaryFormat",
      "BinaryImageQ",
      "BinaryRead",
      "BinaryReadList",
      "BinarySerialize",
      "BinaryWrite",
      "BinCounts",
      "BinLists",
      "BinnedVariogramList",
      "Binomial",
      "BinomialDistribution",
      "BinomialPointProcess",
      "BinomialProcess",
      "BinormalDistribution",
      "BiorthogonalSplineWavelet",
      "BioSequence",
      "BioSequenceBackTranslateList",
      "BioSequenceComplement",
      "BioSequenceInstances",
      "BioSequenceModify",
      "BioSequencePlot",
      "BioSequenceQ",
      "BioSequenceReverseComplement",
      "BioSequenceTranscribe",
      "BioSequenceTranslate",
      "BipartiteGraphQ",
      "BiquadraticFilterModel",
      "BirnbaumImportance",
      "BirnbaumSaundersDistribution",
      "BitAnd",
      "BitClear",
      "BitGet",
      "BitLength",
      "BitNot",
      "BitOr",
      "BitRate",
      "BitSet",
      "BitShiftLeft",
      "BitShiftRight",
      "BitXor",
      "BiweightLocation",
      "BiweightMidvariance",
      "Black",
      "BlackmanHarrisWindow",
      "BlackmanNuttallWindow",
      "BlackmanWindow",
      "Blank",
      "BlankForm",
      "BlankNullSequence",
      "BlankSequence",
      "Blend",
      "Block",
      "BlockchainAddressData",
      "BlockchainBase",
      "BlockchainBlockData",
      "BlockchainContractValue",
      "BlockchainData",
      "BlockchainGet",
      "BlockchainKeyEncode",
      "BlockchainPut",
      "BlockchainTokenData",
      "BlockchainTransaction",
      "BlockchainTransactionData",
      "BlockchainTransactionSign",
      "BlockchainTransactionSubmit",
      "BlockDiagonalMatrix",
      "BlockLowerTriangularMatrix",
      "BlockMap",
      "BlockRandom",
      "BlockUpperTriangularMatrix",
      "BlomqvistBeta",
      "BlomqvistBetaTest",
      "Blue",
      "Blur",
      "Blurring",
      "BodePlot",
      "BohmanWindow",
      "Bold",
      "Bond",
      "BondCount",
      "BondLabels",
      "BondLabelStyle",
      "BondList",
      "BondQ",
      "Bookmarks",
      "Boole",
      "BooleanConsecutiveFunction",
      "BooleanConvert",
      "BooleanCountingFunction",
      "BooleanFunction",
      "BooleanGraph",
      "BooleanMaxterms",
      "BooleanMinimize",
      "BooleanMinterms",
      "BooleanQ",
      "BooleanRegion",
      "Booleans",
      "BooleanStrings",
      "BooleanTable",
      "BooleanVariables",
      "BorderDimensions",
      "BorelTannerDistribution",
      "Bottom",
      "BottomHatTransform",
      "BoundaryDiscretizeGraphics",
      "BoundaryDiscretizeRegion",
      "BoundaryMesh",
      "BoundaryMeshRegion",
      "BoundaryMeshRegionQ",
      "BoundaryStyle",
      "BoundedRegionQ",
      "BoundingRegion",
      "Bounds",
      "Box",
      "BoxBaselineShift",
      "BoxData",
      "BoxDimensions",
      "Boxed",
      "Boxes",
      "BoxForm",
      "BoxFormFormatTypes",
      "BoxFrame",
      "BoxID",
      "BoxMargins",
      "BoxMatrix",
      "BoxObject",
      "BoxRatios",
      "BoxRotation",
      "BoxRotationPoint",
      "BoxStyle",
      "BoxWhiskerChart",
      "Bra",
      "BracketingBar",
      "BraKet",
      "BrayCurtisDistance",
      "BreadthFirstScan",
      "Break",
      "BridgeData",
      "BrightnessEqualize",
      "BroadcastStationData",
      "Brown",
      "BrownForsytheTest",
      "BrownianBridgeProcess",
      "BrowserCategory",
      "BSplineBasis",
      "BSplineCurve",
      "BSplineCurve3DBox",
      "BSplineCurve3DBoxOptions",
      "BSplineCurveBox",
      "BSplineCurveBoxOptions",
      "BSplineFunction",
      "BSplineSurface",
      "BSplineSurface3DBox",
      "BSplineSurface3DBoxOptions",
      "BubbleChart",
      "BubbleChart3D",
      "BubbleScale",
      "BubbleSizes",
      "BuckyballGraph",
      "BuildCompiledComponent",
      "BuildingData",
      "BulletGauge",
      "BusinessDayQ",
      "ButterflyGraph",
      "ButterworthFilterModel",
      "Button",
      "ButtonBar",
      "ButtonBox",
      "ButtonBoxOptions",
      "ButtonCell",
      "ButtonContents",
      "ButtonData",
      "ButtonEvaluator",
      "ButtonExpandable",
      "ButtonFrame",
      "ButtonFunction",
      "ButtonMargins",
      "ButtonMinHeight",
      "ButtonNote",
      "ButtonNotebook",
      "ButtonSource",
      "ButtonStyle",
      "ButtonStyleMenuListing",
      "Byte",
      "ByteArray",
      "ByteArrayFormat",
      "ByteArrayFormatQ",
      "ByteArrayQ",
      "ByteArrayToString",
      "ByteCount",
      "ByteOrdering",
      "C",
      "CachedValue",
      "CacheGraphics",
      "CachePersistence",
      "CalendarConvert",
      "CalendarData",
      "CalendarType",
      "Callout",
      "CalloutMarker",
      "CalloutStyle",
      "CallPacket",
      "CanberraDistance",
      "Cancel",
      "CancelButton",
      "CandlestickChart",
      "CanonicalGraph",
      "CanonicalizePolygon",
      "CanonicalizePolyhedron",
      "CanonicalizeRegion",
      "CanonicalName",
      "CanonicalWarpingCorrespondence",
      "CanonicalWarpingDistance",
      "CantorMesh",
      "CantorStaircase",
      "Canvas",
      "Cap",
      "CapForm",
      "CapitalDifferentialD",
      "Capitalize",
      "CapsuleShape",
      "CaptureRunning",
      "CaputoD",
      "CardinalBSplineBasis",
      "CarlemanLinearize",
      "CarlsonRC",
      "CarlsonRD",
      "CarlsonRE",
      "CarlsonRF",
      "CarlsonRG",
      "CarlsonRJ",
      "CarlsonRK",
      "CarlsonRM",
      "CarmichaelLambda",
      "CaseOrdering",
      "Cases",
      "CaseSensitive",
      "Cashflow",
      "Casoratian",
      "Cast",
      "Catalan",
      "CatalanNumber",
      "Catch",
      "CategoricalDistribution",
      "Catenate",
      "CatenateLayer",
      "CauchyDistribution",
      "CauchyMatrix",
      "CauchyPointProcess",
      "CauchyWindow",
      "CayleyGraph",
      "CDF",
      "CDFDeploy",
      "CDFInformation",
      "CDFWavelet",
      "Ceiling",
      "CelestialSystem",
      "Cell",
      "CellAutoOverwrite",
      "CellBaseline",
      "CellBoundingBox",
      "CellBracketOptions",
      "CellChangeTimes",
      "CellContents",
      "CellContext",
      "CellDingbat",
      "CellDingbatMargin",
      "CellDynamicExpression",
      "CellEditDuplicate",
      "CellElementsBoundingBox",
      "CellElementSpacings",
      "CellEpilog",
      "CellEvaluationDuplicate",
      "CellEvaluationFunction",
      "CellEvaluationLanguage",
      "CellEventActions",
      "CellFrame",
      "CellFrameColor",
      "CellFrameLabelMargins",
      "CellFrameLabels",
      "CellFrameMargins",
      "CellFrameStyle",
      "CellGroup",
      "CellGroupData",
      "CellGrouping",
      "CellGroupingRules",
      "CellHorizontalScrolling",
      "CellID",
      "CellInsertionPointCell",
      "CellLabel",
      "CellLabelAutoDelete",
      "CellLabelMargins",
      "CellLabelPositioning",
      "CellLabelStyle",
      "CellLabelTemplate",
      "CellMargins",
      "CellObject",
      "CellOpen",
      "CellPrint",
      "CellProlog",
      "Cells",
      "CellSize",
      "CellStyle",
      "CellTags",
      "CellTrayPosition",
      "CellTrayWidgets",
      "CellularAutomaton",
      "CensoredDistribution",
      "Censoring",
      "Center",
      "CenterArray",
      "CenterDot",
      "CenteredInterval",
      "CentralFeature",
      "CentralMoment",
      "CentralMomentGeneratingFunction",
      "Cepstrogram",
      "CepstrogramArray",
      "CepstrumArray",
      "CForm",
      "ChampernowneNumber",
      "ChangeOptions",
      "ChannelBase",
      "ChannelBrokerAction",
      "ChannelDatabin",
      "ChannelHistoryLength",
      "ChannelListen",
      "ChannelListener",
      "ChannelListeners",
      "ChannelListenerWait",
      "ChannelObject",
      "ChannelPreSendFunction",
      "ChannelReceiverFunction",
      "ChannelSend",
      "ChannelSubscribers",
      "ChanVeseBinarize",
      "Character",
      "CharacterCounts",
      "CharacterEncoding",
      "CharacterEncodingsPath",
      "CharacteristicFunction",
      "CharacteristicPolynomial",
      "CharacterName",
      "CharacterNormalize",
      "CharacterRange",
      "Characters",
      "ChartBaseStyle",
      "ChartElementData",
      "ChartElementDataFunction",
      "ChartElementFunction",
      "ChartElements",
      "ChartLabels",
      "ChartLayout",
      "ChartLegends",
      "ChartStyle",
      "Chebyshev1FilterModel",
      "Chebyshev2FilterModel",
      "ChebyshevDistance",
      "ChebyshevT",
      "ChebyshevU",
      "Check",
      "CheckAbort",
      "CheckAll",
      "CheckArguments",
      "Checkbox",
      "CheckboxBar",
      "CheckboxBox",
      "CheckboxBoxOptions",
      "ChemicalConvert",
      "ChemicalData",
      "ChemicalFormula",
      "ChemicalInstance",
      "ChemicalReaction",
      "ChessboardDistance",
      "ChiDistribution",
      "ChineseRemainder",
      "ChiSquareDistribution",
      "ChoiceButtons",
      "ChoiceDialog",
      "CholeskyDecomposition",
      "Chop",
      "ChromaticityPlot",
      "ChromaticityPlot3D",
      "ChromaticPolynomial",
      "Circle",
      "CircleBox",
      "CircleDot",
      "CircleMinus",
      "CirclePlus",
      "CirclePoints",
      "CircleThrough",
      "CircleTimes",
      "CirculantGraph",
      "CircularArcThrough",
      "CircularOrthogonalMatrixDistribution",
      "CircularQuaternionMatrixDistribution",
      "CircularRealMatrixDistribution",
      "CircularSymplecticMatrixDistribution",
      "CircularUnitaryMatrixDistribution",
      "Circumsphere",
      "CityData",
      "ClassifierFunction",
      "ClassifierInformation",
      "ClassifierMeasurements",
      "ClassifierMeasurementsObject",
      "Classify",
      "ClassPriors",
      "Clear",
      "ClearAll",
      "ClearAttributes",
      "ClearCookies",
      "ClearPermissions",
      "ClearSystemCache",
      "ClebschGordan",
      "ClickPane",
      "ClickToCopy",
      "ClickToCopyEnabled",
      "Clip",
      "ClipboardNotebook",
      "ClipFill",
      "ClippingStyle",
      "ClipPlanes",
      "ClipPlanesStyle",
      "ClipRange",
      "Clock",
      "ClockGauge",
      "ClockwiseContourIntegral",
      "Close",
      "Closed",
      "CloseKernels",
      "ClosenessCentrality",
      "Closing",
      "ClosingAutoSave",
      "ClosingEvent",
      "CloudAccountData",
      "CloudBase",
      "CloudConnect",
      "CloudConnections",
      "CloudDeploy",
      "CloudDirectory",
      "CloudDisconnect",
      "CloudEvaluate",
      "CloudExport",
      "CloudExpression",
      "CloudExpressions",
      "CloudFunction",
      "CloudGet",
      "CloudImport",
      "CloudLoggingData",
      "CloudObject",
      "CloudObjectInformation",
      "CloudObjectInformationData",
      "CloudObjectNameFormat",
      "CloudObjects",
      "CloudObjectURLType",
      "CloudPublish",
      "CloudPut",
      "CloudRenderingMethod",
      "CloudSave",
      "CloudShare",
      "CloudSubmit",
      "CloudSymbol",
      "CloudUnshare",
      "CloudUserID",
      "ClusterClassify",
      "ClusterDissimilarityFunction",
      "ClusteringComponents",
      "ClusteringMeasurements",
      "ClusteringTree",
      "CMYKColor",
      "Coarse",
      "CodeAssistOptions",
      "Coefficient",
      "CoefficientArrays",
      "CoefficientDomain",
      "CoefficientList",
      "CoefficientRules",
      "CoifletWavelet",
      "Collect",
      "CollinearPoints",
      "Colon",
      "ColonForm",
      "ColorBalance",
      "ColorCombine",
      "ColorConvert",
      "ColorCoverage",
      "ColorData",
      "ColorDataFunction",
      "ColorDetect",
      "ColorDistance",
      "ColorFunction",
      "ColorFunctionBinning",
      "ColorFunctionScaling",
      "Colorize",
      "ColorNegate",
      "ColorOutput",
      "ColorProfileData",
      "ColorQ",
      "ColorQuantize",
      "ColorReplace",
      "ColorRules",
      "ColorSelectorSettings",
      "ColorSeparate",
      "ColorSetter",
      "ColorSetterBox",
      "ColorSetterBoxOptions",
      "ColorSlider",
      "ColorsNear",
      "ColorSpace",
      "ColorToneMapping",
      "Column",
      "ColumnAlignments",
      "ColumnBackgrounds",
      "ColumnForm",
      "ColumnLines",
      "ColumnsEqual",
      "ColumnSpacings",
      "ColumnWidths",
      "CombinatorB",
      "CombinatorC",
      "CombinatorI",
      "CombinatorK",
      "CombinatorS",
      "CombinatorW",
      "CombinatorY",
      "CombinedEntityClass",
      "CombinerFunction",
      "CometData",
      "CommonDefaultFormatTypes",
      "Commonest",
      "CommonestFilter",
      "CommonName",
      "CommonUnits",
      "CommunityBoundaryStyle",
      "CommunityGraphPlot",
      "CommunityLabels",
      "CommunityRegionStyle",
      "CompanyData",
      "CompatibleUnitQ",
      "CompilationOptions",
      "CompilationTarget",
      "Compile",
      "Compiled",
      "CompiledCodeFunction",
      "CompiledComponent",
      "CompiledExpressionDeclaration",
      "CompiledFunction",
      "CompiledLayer",
      "CompilerCallback",
      "CompilerEnvironment",
      "CompilerEnvironmentAppend",
      "CompilerEnvironmentAppendTo",
      "CompilerEnvironmentObject",
      "CompilerOptions",
      "Complement",
      "ComplementedEntityClass",
      "CompleteGraph",
      "CompleteGraphQ",
      "CompleteIntegral",
      "CompleteKaryTree",
      "CompletionsListPacket",
      "Complex",
      "ComplexArrayPlot",
      "ComplexContourPlot",
      "Complexes",
      "ComplexExpand",
      "ComplexInfinity",
      "ComplexityFunction",
      "ComplexListPlot",
      "ComplexPlot",
      "ComplexPlot3D",
      "ComplexRegionPlot",
      "ComplexStreamPlot",
      "ComplexVectorPlot",
      "ComponentMeasurements",
      "ComponentwiseContextMenu",
      "Compose",
      "ComposeList",
      "ComposeSeries",
      "CompositeQ",
      "Composition",
      "CompoundElement",
      "CompoundExpression",
      "CompoundPoissonDistribution",
      "CompoundPoissonProcess",
      "CompoundRenewalProcess",
      "Compress",
      "CompressedData",
      "CompressionLevel",
      "ComputeUncertainty",
      "ConcaveHullMesh",
      "Condition",
      "ConditionalExpression",
      "Conditioned",
      "Cone",
      "ConeBox",
      "ConfidenceLevel",
      "ConfidenceRange",
      "ConfidenceTransform",
      "ConfigurationPath",
      "Confirm",
      "ConfirmAssert",
      "ConfirmBy",
      "ConfirmMatch",
      "ConfirmQuiet",
      "ConformationMethod",
      "ConformAudio",
      "ConformImages",
      "Congruent",
      "ConicGradientFilling",
      "ConicHullRegion",
      "ConicHullRegion3DBox",
      "ConicHullRegion3DBoxOptions",
      "ConicHullRegionBox",
      "ConicHullRegionBoxOptions",
      "ConicOptimization",
      "Conjugate",
      "ConjugateTranspose",
      "Conjunction",
      "Connect",
      "ConnectedComponents",
      "ConnectedGraphComponents",
      "ConnectedGraphQ",
      "ConnectedMeshComponents",
      "ConnectedMoleculeComponents",
      "ConnectedMoleculeQ",
      "ConnectionSettings",
      "ConnectLibraryCallbackFunction",
      "ConnectSystemModelComponents",
      "ConnectSystemModelController",
      "ConnesWindow",
      "ConoverTest",
      "ConservativeConvectionPDETerm",
      "ConsoleMessage",
      "Constant",
      "ConstantArray",
      "ConstantArrayLayer",
      "ConstantImage",
      "ConstantPlusLayer",
      "ConstantRegionQ",
      "Constants",
      "ConstantTimesLayer",
      "ConstellationData",
      "ConstrainedMax",
      "ConstrainedMin",
      "Construct",
      "Containing",
      "ContainsAll",
      "ContainsAny",
      "ContainsExactly",
      "ContainsNone",
      "ContainsOnly",
      "ContentDetectorFunction",
      "ContentFieldOptions",
      "ContentLocationFunction",
      "ContentObject",
      "ContentPadding",
      "ContentsBoundingBox",
      "ContentSelectable",
      "ContentSize",
      "Context",
      "ContextMenu",
      "Contexts",
      "ContextToFileName",
      "Continuation",
      "Continue",
      "ContinuedFraction",
      "ContinuedFractionK",
      "ContinuousAction",
      "ContinuousMarkovProcess",
      "ContinuousTask",
      "ContinuousTimeModelQ",
      "ContinuousWaveletData",
      "ContinuousWaveletTransform",
      "ContourDetect",
      "ContourGraphics",
      "ContourIntegral",
      "ContourLabels",
      "ContourLines",
      "ContourPlot",
      "ContourPlot3D",
      "Contours",
      "ContourShading",
      "ContourSmoothing",
      "ContourStyle",
      "ContraharmonicMean",
      "ContrastiveLossLayer",
      "Control",
      "ControlActive",
      "ControlAlignment",
      "ControlGroupContentsBox",
      "ControllabilityGramian",
      "ControllabilityMatrix",
      "ControllableDecomposition",
      "ControllableModelQ",
      "ControllerDuration",
      "ControllerInformation",
      "ControllerInformationData",
      "ControllerLinking",
      "ControllerManipulate",
      "ControllerMethod",
      "ControllerPath",
      "ControllerState",
      "ControlPlacement",
      "ControlsRendering",
      "ControlType",
      "ConvectionPDETerm",
      "Convergents",
      "ConversionOptions",
      "ConversionRules",
      "ConvertToPostScript",
      "ConvertToPostScriptPacket",
      "ConvexHullMesh",
      "ConvexHullRegion",
      "ConvexOptimization",
      "ConvexPolygonQ",
      "ConvexPolyhedronQ",
      "ConvexRegionQ",
      "ConvolutionLayer",
      "Convolve",
      "ConwayGroupCo1",
      "ConwayGroupCo2",
      "ConwayGroupCo3",
      "CookieFunction",
      "Cookies",
      "CoordinateBoundingBox",
      "CoordinateBoundingBoxArray",
      "CoordinateBounds",
      "CoordinateBoundsArray",
      "CoordinateChartData",
      "CoordinatesToolOptions",
      "CoordinateTransform",
      "CoordinateTransformData",
      "CoplanarPoints",
      "CoprimeQ",
      "Coproduct",
      "CopulaDistribution",
      "Copyable",
      "CopyDatabin",
      "CopyDirectory",
      "CopyFile",
      "CopyFunction",
      "CopyTag",
      "CopyToClipboard",
      "CoreNilpotentDecomposition",
      "CornerFilter",
      "CornerNeighbors",
      "Correlation",
      "CorrelationDistance",
      "CorrelationFunction",
      "CorrelationTest",
      "Cos",
      "Cosh",
      "CoshIntegral",
      "CosineDistance",
      "CosineWindow",
      "CosIntegral",
      "Cot",
      "Coth",
      "CoulombF",
      "CoulombG",
      "CoulombH1",
      "CoulombH2",
      "Count",
      "CountDistinct",
      "CountDistinctBy",
      "CounterAssignments",
      "CounterBox",
      "CounterBoxOptions",
      "CounterClockwiseContourIntegral",
      "CounterEvaluator",
      "CounterFunction",
      "CounterIncrements",
      "CounterStyle",
      "CounterStyleMenuListing",
      "CountRoots",
      "CountryData",
      "Counts",
      "CountsBy",
      "Covariance",
      "CovarianceEstimatorFunction",
      "CovarianceFunction",
      "CoxianDistribution",
      "CoxIngersollRossProcess",
      "CoxModel",
      "CoxModelFit",
      "CramerVonMisesTest",
      "CreateArchive",
      "CreateCellID",
      "CreateChannel",
      "CreateCloudExpression",
      "CreateCompilerEnvironment",
      "CreateDatabin",
      "CreateDataStructure",
      "CreateDataSystemModel",
      "CreateDialog",
      "CreateDirectory",
      "CreateDocument",
      "CreateFile",
      "CreateIntermediateDirectories",
      "CreateLicenseEntitlement",
      "CreateManagedLibraryExpression",
      "CreateNotebook",
      "CreatePacletArchive",
      "CreatePalette",
      "CreatePermissionsGroup",
      "CreateScheduledTask",
      "CreateSearchIndex",
      "CreateSystemModel",
      "CreateTemporary",
      "CreateTypeInstance",
      "CreateUUID",
      "CreateWindow",
      "CriterionFunction",
      "CriticalityFailureImportance",
      "CriticalitySuccessImportance",
      "CriticalSection",
      "Cross",
      "CrossEntropyLossLayer",
      "CrossingCount",
      "CrossingDetect",
      "CrossingPolygon",
      "CrossMatrix",
      "Csc",
      "Csch",
      "CSGRegion",
      "CSGRegionQ",
      "CSGRegionTree",
      "CTCLossLayer",
      "Cube",
      "CubeRoot",
      "Cubics",
      "Cuboid",
      "CuboidBox",
      "CuboidBoxOptions",
      "Cumulant",
      "CumulantGeneratingFunction",
      "CumulativeFeatureImpactPlot",
      "Cup",
      "CupCap",
      "Curl",
      "CurlyDoubleQuote",
      "CurlyQuote",
      "CurrencyConvert",
      "CurrentDate",
      "CurrentImage",
      "CurrentNotebookImage",
      "CurrentScreenImage",
      "CurrentValue",
      "Curry",
      "CurryApplied",
      "CurvatureFlowFilter",
      "CurveClosed",
      "Cyan",
      "CycleGraph",
      "CycleIndexPolynomial",
      "Cycles",
      "CyclicGroup",
      "Cyclotomic",
      "Cylinder",
      "CylinderBox",
      "CylinderBoxOptions",
      "CylindricalDecomposition",
      "CylindricalDecompositionFunction",
      "D",
      "DagumDistribution",
      "DamData",
      "DamerauLevenshteinDistance",
      "DampingFactor",
      "Darker",
      "Dashed",
      "Dashing",
      "DatabaseConnect",
      "DatabaseDisconnect",
      "DatabaseReference",
      "Databin",
      "DatabinAdd",
      "DatabinRemove",
      "Databins",
      "DatabinSubmit",
      "DatabinUpload",
      "DataCompression",
      "DataDistribution",
      "DataRange",
      "DataReversed",
      "Dataset",
      "DatasetDisplayPanel",
      "DatasetTheme",
      "DataStructure",
      "DataStructureQ",
      "Date",
      "DateBounds",
      "Dated",
      "DateDelimiters",
      "DateDifference",
      "DatedUnit",
      "DateFormat",
      "DateFunction",
      "DateGranularity",
      "DateHistogram",
      "DateInterval",
      "DateList",
      "DateListLogPlot",
      "DateListPlot",
      "DateListStepPlot",
      "DateObject",
      "DateObjectQ",
      "DateOverlapsQ",
      "DatePattern",
      "DatePlus",
      "DateRange",
      "DateReduction",
      "DateScale",
      "DateSelect",
      "DateString",
      "DateTicksFormat",
      "DateValue",
      "DateWithinQ",
      "DaubechiesWavelet",
      "DavisDistribution",
      "DawsonF",
      "DayCount",
      "DayCountConvention",
      "DayHemisphere",
      "DaylightQ",
      "DayMatchQ",
      "DayName",
      "DayNightTerminator",
      "DayPlus",
      "DayRange",
      "DayRound",
      "DeBruijnGraph",
      "DeBruijnSequence",
      "Debug",
      "DebugTag",
      "Decapitalize",
      "Decimal",
      "DecimalForm",
      "DeclareCompiledComponent",
      "DeclareKnownSymbols",
      "DeclarePackage",
      "Decompose",
      "DeconvolutionLayer",
      "Decrement",
      "Decrypt",
      "DecryptFile",
      "DedekindEta",
      "DeepSpaceProbeData",
      "Default",
      "Default2DTool",
      "Default3DTool",
      "DefaultAttachedCellStyle",
      "DefaultAxesStyle",
      "DefaultBaseStyle",
      "DefaultBoxStyle",
      "DefaultButton",
      "DefaultColor",
      "DefaultControlPlacement",
      "DefaultDockedCellStyle",
      "DefaultDuplicateCellStyle",
      "DefaultDuration",
      "DefaultElement",
      "DefaultFaceGridsStyle",
      "DefaultFieldHintStyle",
      "DefaultFont",
      "DefaultFontProperties",
      "DefaultFormatType",
      "DefaultFrameStyle",
      "DefaultFrameTicksStyle",
      "DefaultGridLinesStyle",
      "DefaultInlineFormatType",
      "DefaultInputFormatType",
      "DefaultLabelStyle",
      "DefaultMenuStyle",
      "DefaultNaturalLanguage",
      "DefaultNewCellStyle",
      "DefaultNewInlineCellStyle",
      "DefaultNotebook",
      "DefaultOptions",
      "DefaultOutputFormatType",
      "DefaultPrintPrecision",
      "DefaultStyle",
      "DefaultStyleDefinitions",
      "DefaultTextFormatType",
      "DefaultTextInlineFormatType",
      "DefaultTicksStyle",
      "DefaultTooltipStyle",
      "DefaultValue",
      "DefaultValues",
      "Defer",
      "DefineExternal",
      "DefineInputStreamMethod",
      "DefineOutputStreamMethod",
      "DefineResourceFunction",
      "Definition",
      "Degree",
      "DegreeCentrality",
      "DegreeGraphDistribution",
      "DegreeLexicographic",
      "DegreeReverseLexicographic",
      "DEigensystem",
      "DEigenvalues",
      "Deinitialization",
      "Del",
      "DelaunayMesh",
      "Delayed",
      "Deletable",
      "Delete",
      "DeleteAdjacentDuplicates",
      "DeleteAnomalies",
      "DeleteBorderComponents",
      "DeleteCases",
      "DeleteChannel",
      "DeleteCloudExpression",
      "DeleteContents",
      "DeleteDirectory",
      "DeleteDuplicates",
      "DeleteDuplicatesBy",
      "DeleteElements",
      "DeleteFile",
      "DeleteMissing",
      "DeleteObject",
      "DeletePermissionsKey",
      "DeleteSearchIndex",
      "DeleteSmallComponents",
      "DeleteStopwords",
      "DeleteWithContents",
      "DeletionWarning",
      "DelimitedArray",
      "DelimitedSequence",
      "Delimiter",
      "DelimiterAutoMatching",
      "DelimiterFlashTime",
      "DelimiterMatching",
      "Delimiters",
      "DeliveryFunction",
      "Dendrogram",
      "Denominator",
      "DensityGraphics",
      "DensityHistogram",
      "DensityPlot",
      "DensityPlot3D",
      "DependentVariables",
      "Deploy",
      "Deployed",
      "Depth",
      "DepthFirstScan",
      "Derivative",
      "DerivativeFilter",
      "DerivativePDETerm",
      "DerivedKey",
      "DescriptorStateSpace",
      "DesignMatrix",
      "DestroyAfterEvaluation",
      "Det",
      "DeviceClose",
      "DeviceConfigure",
      "DeviceExecute",
      "DeviceExecuteAsynchronous",
      "DeviceObject",
      "DeviceOpen",
      "DeviceOpenQ",
      "DeviceRead",
      "DeviceReadBuffer",
      "DeviceReadLatest",
      "DeviceReadList",
      "DeviceReadTimeSeries",
      "Devices",
      "DeviceStreams",
      "DeviceWrite",
      "DeviceWriteBuffer",
      "DGaussianWavelet",
      "DiacriticalPositioning",
      "Diagonal",
      "DiagonalizableMatrixQ",
      "DiagonalMatrix",
      "DiagonalMatrixQ",
      "Dialog",
      "DialogIndent",
      "DialogInput",
      "DialogLevel",
      "DialogNotebook",
      "DialogProlog",
      "DialogReturn",
      "DialogSymbols",
      "Diamond",
      "DiamondMatrix",
      "DiceDissimilarity",
      "DictionaryLookup",
      "DictionaryWordQ",
      "DifferenceDelta",
      "DifferenceOrder",
      "DifferenceQuotient",
      "DifferenceRoot",
      "DifferenceRootReduce",
      "Differences",
      "DifferentialD",
      "DifferentialRoot",
      "DifferentialRootReduce",
      "DifferentiatorFilter",
      "DiffusionPDETerm",
      "DiggleGatesPointProcess",
      "DiggleGrattonPointProcess",
      "DigitalSignature",
      "DigitBlock",
      "DigitBlockMinimum",
      "DigitCharacter",
      "DigitCount",
      "DigitQ",
      "DihedralAngle",
      "DihedralGroup",
      "Dilation",
      "DimensionalCombinations",
      "DimensionalMeshComponents",
      "DimensionReduce",
      "DimensionReducerFunction",
      "DimensionReduction",
      "Dimensions",
      "DiracComb",
      "DiracDelta",
      "DirectedEdge",
      "DirectedEdges",
      "DirectedGraph",
      "DirectedGraphQ",
      "DirectedInfinity",
      "Direction",
      "DirectionalLight",
      "Directive",
      "Directory",
      "DirectoryName",
      "DirectoryQ",
      "DirectoryStack",
      "DirichletBeta",
      "DirichletCharacter",
      "DirichletCondition",
      "DirichletConvolve",
      "DirichletDistribution",
      "DirichletEta",
      "DirichletL",
      "DirichletLambda",
      "DirichletTransform",
      "DirichletWindow",
      "DisableConsolePrintPacket",
      "DisableFormatting",
      "DiscreteAsymptotic",
      "DiscreteChirpZTransform",
      "DiscreteConvolve",
      "DiscreteDelta",
      "DiscreteHadamardTransform",
      "DiscreteIndicator",
      "DiscreteInputOutputModel",
      "DiscreteLimit",
      "DiscreteLQEstimatorGains",
      "DiscreteLQRegulatorGains",
      "DiscreteLyapunovSolve",
      "DiscreteMarkovProcess",
      "DiscreteMaxLimit",
      "DiscreteMinLimit",
      "DiscretePlot",
      "DiscretePlot3D",
      "DiscreteRatio",
      "DiscreteRiccatiSolve",
      "DiscreteShift",
      "DiscreteTimeModelQ",
      "DiscreteUniformDistribution",
      "DiscreteVariables",
      "DiscreteWaveletData",
      "DiscreteWaveletPacketTransform",
      "DiscreteWaveletTransform",
      "DiscretizeGraphics",
      "DiscretizeRegion",
      "Discriminant",
      "DisjointQ",
      "Disjunction",
      "Disk",
      "DiskBox",
      "DiskBoxOptions",
      "DiskMatrix",
      "DiskSegment",
      "Dispatch",
      "DispatchQ",
      "DispersionEstimatorFunction",
      "Display",
      "DisplayAllSteps",
      "DisplayEndPacket",
      "DisplayForm",
      "DisplayFunction",
      "DisplayPacket",
      "DisplayRules",
      "DisplayString",
      "DisplayTemporary",
      "DisplayWith",
      "DisplayWithRef",
      "DisplayWithVariable",
      "DistanceFunction",
      "DistanceMatrix",
      "DistanceTransform",
      "Distribute",
      "Distributed",
      "DistributedContexts",
      "DistributeDefinitions",
      "DistributionChart",
      "DistributionDomain",
      "DistributionFitTest",
      "DistributionParameterAssumptions",
      "DistributionParameterQ",
      "Dithering",
      "Div",
      "Divergence",
      "Divide",
      "DivideBy",
      "Dividers",
      "DivideSides",
      "Divisible",
      "Divisors",
      "DivisorSigma",
      "DivisorSum",
      "DMSList",
      "DMSString",
      "Do",
      "DockedCell",
      "DockedCells",
      "DocumentGenerator",
      "DocumentGeneratorInformation",
      "DocumentGeneratorInformationData",
      "DocumentGenerators",
      "DocumentNotebook",
      "DocumentWeightingRules",
      "Dodecahedron",
      "DomainRegistrationInformation",
      "DominantColors",
      "DominatorTreeGraph",
      "DominatorVertexList",
      "DOSTextFormat",
      "Dot",
      "DotDashed",
      "DotEqual",
      "DotLayer",
      "DotPlusLayer",
      "Dotted",
      "DoubleBracketingBar",
      "DoubleContourIntegral",
      "DoubleDownArrow",
      "DoubleLeftArrow",
      "DoubleLeftRightArrow",
      "DoubleLeftTee",
      "DoubleLongLeftArrow",
      "DoubleLongLeftRightArrow",
      "DoubleLongRightArrow",
      "DoubleRightArrow",
      "DoubleRightTee",
      "DoubleUpArrow",
      "DoubleUpDownArrow",
      "DoubleVerticalBar",
      "DoublyInfinite",
      "Down",
      "DownArrow",
      "DownArrowBar",
      "DownArrowUpArrow",
      "DownLeftRightVector",
      "DownLeftTeeVector",
      "DownLeftVector",
      "DownLeftVectorBar",
      "DownRightTeeVector",
      "DownRightVector",
      "DownRightVectorBar",
      "Downsample",
      "DownTee",
      "DownTeeArrow",
      "DownValues",
      "DownValuesFunction",
      "DragAndDrop",
      "DrawBackFaces",
      "DrawEdges",
      "DrawFrontFaces",
      "DrawHighlighted",
      "DrazinInverse",
      "Drop",
      "DropoutLayer",
      "DropShadowing",
      "DSolve",
      "DSolveChangeVariables",
      "DSolveValue",
      "Dt",
      "DualLinearProgramming",
      "DualPlanarGraph",
      "DualPolyhedron",
      "DualSystemsModel",
      "DumpGet",
      "DumpSave",
      "DuplicateFreeQ",
      "Duration",
      "Dynamic",
      "DynamicBox",
      "DynamicBoxOptions",
      "DynamicEvaluationTimeout",
      "DynamicGeoGraphics",
      "DynamicImage",
      "DynamicLocation",
      "DynamicModule",
      "DynamicModuleBox",
      "DynamicModuleBoxOptions",
      "DynamicModuleParent",
      "DynamicModuleValues",
      "DynamicName",
      "DynamicNamespace",
      "DynamicReference",
      "DynamicSetting",
      "DynamicUpdating",
      "DynamicWrapper",
      "DynamicWrapperBox",
      "DynamicWrapperBoxOptions",
      "E",
      "EarthImpactData",
      "EarthquakeData",
      "EccentricityCentrality",
      "Echo",
      "EchoEvaluation",
      "EchoFunction",
      "EchoLabel",
      "EchoTiming",
      "EclipseType",
      "EdgeAdd",
      "EdgeBetweennessCentrality",
      "EdgeCapacity",
      "EdgeCapForm",
      "EdgeChromaticNumber",
      "EdgeColor",
      "EdgeConnectivity",
      "EdgeContract",
      "EdgeCost",
      "EdgeCount",
      "EdgeCoverQ",
      "EdgeCycleMatrix",
      "EdgeDashing",
      "EdgeDelete",
      "EdgeDetect",
      "EdgeForm",
      "EdgeIndex",
      "EdgeJoinForm",
      "EdgeLabeling",
      "EdgeLabels",
      "EdgeLabelStyle",
      "EdgeList",
      "EdgeOpacity",
      "EdgeQ",
      "EdgeRenderingFunction",
      "EdgeRules",
      "EdgeShapeFunction",
      "EdgeStyle",
      "EdgeTaggedGraph",
      "EdgeTaggedGraphQ",
      "EdgeTags",
      "EdgeThickness",
      "EdgeTransitiveGraphQ",
      "EdgeValueRange",
      "EdgeValueSizes",
      "EdgeWeight",
      "EdgeWeightedGraphQ",
      "Editable",
      "EditButtonSettings",
      "EditCellTagsSettings",
      "EditDistance",
      "EffectiveInterest",
      "Eigensystem",
      "Eigenvalues",
      "EigenvectorCentrality",
      "Eigenvectors",
      "Element",
      "ElementData",
      "ElementwiseLayer",
      "ElidedForms",
      "Eliminate",
      "EliminationOrder",
      "Ellipsoid",
      "EllipticE",
      "EllipticExp",
      "EllipticExpPrime",
      "EllipticF",
      "EllipticFilterModel",
      "EllipticK",
      "EllipticLog",
      "EllipticNomeQ",
      "EllipticPi",
      "EllipticReducedHalfPeriods",
      "EllipticTheta",
      "EllipticThetaPrime",
      "EmbedCode",
      "EmbeddedHTML",
      "EmbeddedService",
      "EmbeddedSQLEntityClass",
      "EmbeddedSQLExpression",
      "EmbeddingLayer",
      "EmbeddingObject",
      "EmitSound",
      "EmphasizeSyntaxErrors",
      "EmpiricalDistribution",
      "Empty",
      "EmptyGraphQ",
      "EmptyRegion",
      "EmptySpaceF",
      "EnableConsolePrintPacket",
      "Enabled",
      "Enclose",
      "Encode",
      "Encrypt",
      "EncryptedObject",
      "EncryptFile",
      "End",
      "EndAdd",
      "EndDialogPacket",
      "EndOfBuffer",
      "EndOfFile",
      "EndOfLine",
      "EndOfString",
      "EndPackage",
      "EngineEnvironment",
      "EngineeringForm",
      "Enter",
      "EnterExpressionPacket",
      "EnterTextPacket",
      "Entity",
      "EntityClass",
      "EntityClassList",
      "EntityCopies",
      "EntityFunction",
      "EntityGroup",
      "EntityInstance",
      "EntityList",
      "EntityPrefetch",
      "EntityProperties",
      "EntityProperty",
      "EntityPropertyClass",
      "EntityRegister",
      "EntityStore",
      "EntityStores",
      "EntityTypeName",
      "EntityUnregister",
      "EntityValue",
      "Entropy",
      "EntropyFilter",
      "Environment",
      "Epilog",
      "EpilogFunction",
      "Equal",
      "EqualColumns",
      "EqualRows",
      "EqualTilde",
      "EqualTo",
      "EquatedTo",
      "Equilibrium",
      "EquirippleFilterKernel",
      "Equivalent",
      "Erf",
      "Erfc",
      "Erfi",
      "ErlangB",
      "ErlangC",
      "ErlangDistribution",
      "Erosion",
      "ErrorBox",
      "ErrorBoxOptions",
      "ErrorNorm",
      "ErrorPacket",
      "ErrorsDialogSettings",
      "EscapeRadius",
      "EstimatedBackground",
      "EstimatedDistribution",
      "EstimatedPointNormals",
      "EstimatedPointProcess",
      "EstimatedProcess",
      "EstimatedVariogramModel",
      "EstimatorGains",
      "EstimatorRegulator",
      "EuclideanDistance",
      "EulerAngles",
      "EulerCharacteristic",
      "EulerE",
      "EulerGamma",
      "EulerianGraphQ",
      "EulerMatrix",
      "EulerPhi",
      "Evaluatable",
      "Evaluate",
      "Evaluated",
      "EvaluatePacket",
      "EvaluateScheduledTask",
      "EvaluationBox",
      "EvaluationCell",
      "EvaluationCompletionAction",
      "EvaluationData",
      "EvaluationElements",
      "EvaluationEnvironment",
      "EvaluationMode",
      "EvaluationMonitor",
      "EvaluationNotebook",
      "EvaluationObject",
      "EvaluationOrder",
      "EvaluationPrivileges",
      "EvaluationRateLimit",
      "Evaluator",
      "EvaluatorNames",
      "EvenQ",
      "EventData",
      "EventEvaluator",
      "EventHandler",
      "EventHandlerTag",
      "EventLabels",
      "EventSeries",
      "ExactBlackmanWindow",
      "ExactNumberQ",
      "ExactRootIsolation",
      "ExampleData",
      "Except",
      "ExcludedContexts",
      "ExcludedForms",
      "ExcludedLines",
      "ExcludedPhysicalQuantities",
      "ExcludePods",
      "Exclusions",
      "ExclusionsStyle",
      "Exists",
      "Exit",
      "ExitDialog",
      "ExoplanetData",
      "Exp",
      "Expand",
      "ExpandAll",
      "ExpandDenominator",
      "ExpandFileName",
      "ExpandNumerator",
      "Expectation",
      "ExpectationE",
      "ExpectedValue",
      "ExpGammaDistribution",
      "ExpIntegralE",
      "ExpIntegralEi",
      "ExpirationDate",
      "Exponent",
      "ExponentFunction",
      "ExponentialDistribution",
      "ExponentialFamily",
      "ExponentialGeneratingFunction",
      "ExponentialMovingAverage",
      "ExponentialPowerDistribution",
      "ExponentPosition",
      "ExponentStep",
      "Export",
      "ExportAutoReplacements",
      "ExportByteArray",
      "ExportForm",
      "ExportPacket",
      "ExportString",
      "Expression",
      "ExpressionCell",
      "ExpressionGraph",
      "ExpressionPacket",
      "ExpressionTree",
      "ExpressionUUID",
      "ExpToTrig",
      "ExtendedEntityClass",
      "ExtendedGCD",
      "Extension",
      "ExtentElementFunction",
      "ExtentMarkers",
      "ExtentSize",
      "ExternalBundle",
      "ExternalCall",
      "ExternalDataCharacterEncoding",
      "ExternalEvaluate",
      "ExternalFunction",
      "ExternalFunctionName",
      "ExternalIdentifier",
      "ExternalObject",
      "ExternalOptions",
      "ExternalSessionObject",
      "ExternalSessions",
      "ExternalStorageBase",
      "ExternalStorageDownload",
      "ExternalStorageGet",
      "ExternalStorageObject",
      "ExternalStoragePut",
      "ExternalStorageUpload",
      "ExternalTypeSignature",
      "ExternalValue",
      "Extract",
      "ExtractArchive",
      "ExtractLayer",
      "ExtractPacletArchive",
      "ExtremeValueDistribution",
      "FaceAlign",
      "FaceForm",
      "FaceGrids",
      "FaceGridsStyle",
      "FaceRecognize",
      "FacialFeatures",
      "Factor",
      "FactorComplete",
      "Factorial",
      "Factorial2",
      "FactorialMoment",
      "FactorialMomentGeneratingFunction",
      "FactorialPower",
      "FactorInteger",
      "FactorList",
      "FactorSquareFree",
      "FactorSquareFreeList",
      "FactorTerms",
      "FactorTermsList",
      "Fail",
      "Failure",
      "FailureAction",
      "FailureDistribution",
      "FailureQ",
      "False",
      "FareySequence",
      "FARIMAProcess",
      "FeatureDistance",
      "FeatureExtract",
      "FeatureExtraction",
      "FeatureExtractor",
      "FeatureExtractorFunction",
      "FeatureImpactPlot",
      "FeatureNames",
      "FeatureNearest",
      "FeatureSpacePlot",
      "FeatureSpacePlot3D",
      "FeatureTypes",
      "FeatureValueDependencyPlot",
      "FeatureValueImpactPlot",
      "FEDisableConsolePrintPacket",
      "FeedbackLinearize",
      "FeedbackSector",
      "FeedbackSectorStyle",
      "FeedbackType",
      "FEEnableConsolePrintPacket",
      "FetalGrowthData",
      "Fibonacci",
      "Fibonorial",
      "FieldCompletionFunction",
      "FieldHint",
      "FieldHintStyle",
      "FieldMasked",
      "FieldSize",
      "File",
      "FileBaseName",
      "FileByteCount",
      "FileConvert",
      "FileDate",
      "FileExistsQ",
      "FileExtension",
      "FileFormat",
      "FileFormatProperties",
      "FileFormatQ",
      "FileHandler",
      "FileHash",
      "FileInformation",
      "FileName",
      "FileNameDepth",
      "FileNameDialogSettings",
      "FileNameDrop",
      "FileNameForms",
      "FileNameJoin",
      "FileNames",
      "FileNameSetter",
      "FileNameSplit",
      "FileNameTake",
      "FileNameToFormatList",
      "FilePrint",
      "FileSize",
      "FileSystemMap",
      "FileSystemScan",
      "FileSystemTree",
      "FileTemplate",
      "FileTemplateApply",
      "FileType",
      "FilledCurve",
      "FilledCurveBox",
      "FilledCurveBoxOptions",
      "FilledTorus",
      "FillForm",
      "Filling",
      "FillingStyle",
      "FillingTransform",
      "FilteredEntityClass",
      "FilterRules",
      "FinancialBond",
      "FinancialData",
      "FinancialDerivative",
      "FinancialIndicator",
      "Find",
      "FindAnomalies",
      "FindArgMax",
      "FindArgMin",
      "FindChannels",
      "FindClique",
      "FindClusters",
      "FindCookies",
      "FindCurvePath",
      "FindCycle",
      "FindDevices",
      "FindDistribution",
      "FindDistributionParameters",
      "FindDivisions",
      "FindEdgeColoring",
      "FindEdgeCover",
      "FindEdgeCut",
      "FindEdgeIndependentPaths",
      "FindEquationalProof",
      "FindEulerianCycle",
      "FindExternalEvaluators",
      "FindFaces",
      "FindFile",
      "FindFit",
      "FindFormula",
      "FindFundamentalCycles",
      "FindGeneratingFunction",
      "FindGeoLocation",
      "FindGeometricConjectures",
      "FindGeometricTransform",
      "FindGraphCommunities",
      "FindGraphIsomorphism",
      "FindGraphPartition",
      "FindHamiltonianCycle",
      "FindHamiltonianPath",
      "FindHiddenMarkovStates",
      "FindImageText",
      "FindIndependentEdgeSet",
      "FindIndependentVertexSet",
      "FindInstance",
      "FindIntegerNullVector",
      "FindIsomers",
      "FindIsomorphicSubgraph",
      "FindKClan",
      "FindKClique",
      "FindKClub",
      "FindKPlex",
      "FindLibrary",
      "FindLinearRecurrence",
      "FindList",
      "FindMatchingColor",
      "FindMaximum",
      "FindMaximumCut",
      "FindMaximumFlow",
      "FindMaxValue",
      "FindMeshDefects",
      "FindMinimum",
      "FindMinimumCostFlow",
      "FindMinimumCut",
      "FindMinValue",
      "FindMoleculeSubstructure",
      "FindPath",
      "FindPeaks",
      "FindPermutation",
      "FindPlanarColoring",
      "FindPointProcessParameters",
      "FindPostmanTour",
      "FindProcessParameters",
      "FindRegionTransform",
      "FindRepeat",
      "FindRoot",
      "FindSequenceFunction",
      "FindSettings",
      "FindShortestPath",
      "FindShortestTour",
      "FindSpanningTree",
      "FindSubgraphIsomorphism",
      "FindSystemModelEquilibrium",
      "FindTextualAnswer",
      "FindThreshold",
      "FindTransientRepeat",
      "FindVertexColoring",
      "FindVertexCover",
      "FindVertexCut",
      "FindVertexIndependentPaths",
      "Fine",
      "FinishDynamic",
      "FiniteAbelianGroupCount",
      "FiniteGroupCount",
      "FiniteGroupData",
      "First",
      "FirstCase",
      "FirstPassageTimeDistribution",
      "FirstPosition",
      "FischerGroupFi22",
      "FischerGroupFi23",
      "FischerGroupFi24Prime",
      "FisherHypergeometricDistribution",
      "FisherRatioTest",
      "FisherZDistribution",
      "Fit",
      "FitAll",
      "FitRegularization",
      "FittedModel",
      "FixedOrder",
      "FixedPoint",
      "FixedPointList",
      "FlashSelection",
      "Flat",
      "FlatShading",
      "Flatten",
      "FlattenAt",
      "FlattenLayer",
      "FlatTopWindow",
      "FlightData",
      "FlipView",
      "Floor",
      "FlowPolynomial",
      "Fold",
      "FoldList",
      "FoldPair",
      "FoldPairList",
      "FoldWhile",
      "FoldWhileList",
      "FollowRedirects",
      "Font",
      "FontColor",
      "FontFamily",
      "FontForm",
      "FontName",
      "FontOpacity",
      "FontPostScriptName",
      "FontProperties",
      "FontReencoding",
      "FontSize",
      "FontSlant",
      "FontSubstitutions",
      "FontTracking",
      "FontVariations",
      "FontWeight",
      "For",
      "ForAll",
      "ForAllType",
      "ForceVersionInstall",
      "Format",
      "FormatRules",
      "FormatType",
      "FormatTypeAutoConvert",
      "FormatValues",
      "FormBox",
      "FormBoxOptions",
      "FormControl",
      "FormFunction",
      "FormLayoutFunction",
      "FormObject",
      "FormPage",
      "FormProtectionMethod",
      "FormTheme",
      "FormulaData",
      "FormulaLookup",
      "FortranForm",
      "Forward",
      "ForwardBackward",
      "ForwardCloudCredentials",
      "Fourier",
      "FourierCoefficient",
      "FourierCosCoefficient",
      "FourierCosSeries",
      "FourierCosTransform",
      "FourierDCT",
      "FourierDCTFilter",
      "FourierDCTMatrix",
      "FourierDST",
      "FourierDSTMatrix",
      "FourierMatrix",
      "FourierParameters",
      "FourierSequenceTransform",
      "FourierSeries",
      "FourierSinCoefficient",
      "FourierSinSeries",
      "FourierSinTransform",
      "FourierTransform",
      "FourierTrigSeries",
      "FoxH",
      "FoxHReduce",
      "FractionalBrownianMotionProcess",
      "FractionalD",
      "FractionalGaussianNoiseProcess",
      "FractionalPart",
      "FractionBox",
      "FractionBoxOptions",
      "FractionLine",
      "Frame",
      "FrameBox",
      "FrameBoxOptions",
      "Framed",
      "FrameInset",
      "FrameLabel",
      "Frameless",
      "FrameListVideo",
      "FrameMargins",
      "FrameRate",
      "FrameStyle",
      "FrameTicks",
      "FrameTicksStyle",
      "FRatioDistribution",
      "FrechetDistribution",
      "FreeQ",
      "FrenetSerretSystem",
      "FrequencySamplingFilterKernel",
      "FresnelC",
      "FresnelF",
      "FresnelG",
      "FresnelS",
      "Friday",
      "FrobeniusNumber",
      "FrobeniusSolve",
      "FromAbsoluteTime",
      "FromCharacterCode",
      "FromCoefficientRules",
      "FromContinuedFraction",
      "FromDate",
      "FromDateString",
      "FromDigits",
      "FromDMS",
      "FromEntity",
      "FromJulianDate",
      "FromLetterNumber",
      "FromPolarCoordinates",
      "FromRawPointer",
      "FromRomanNumeral",
      "FromSphericalCoordinates",
      "FromUnixTime",
      "Front",
      "FrontEndDynamicExpression",
      "FrontEndEventActions",
      "FrontEndExecute",
      "FrontEndObject",
      "FrontEndResource",
      "FrontEndResourceString",
      "FrontEndStackSize",
      "FrontEndToken",
      "FrontEndTokenExecute",
      "FrontEndValueCache",
      "FrontEndVersion",
      "FrontFaceColor",
      "FrontFaceGlowColor",
      "FrontFaceOpacity",
      "FrontFaceSpecularColor",
      "FrontFaceSpecularExponent",
      "FrontFaceSurfaceAppearance",
      "FrontFaceTexture",
      "Full",
      "FullAxes",
      "FullDefinition",
      "FullForm",
      "FullGraphics",
      "FullInformationOutputRegulator",
      "FullOptions",
      "FullRegion",
      "FullSimplify",
      "Function",
      "FunctionAnalytic",
      "FunctionBijective",
      "FunctionCompile",
      "FunctionCompileExport",
      "FunctionCompileExportByteArray",
      "FunctionCompileExportLibrary",
      "FunctionCompileExportString",
      "FunctionContinuous",
      "FunctionConvexity",
      "FunctionDeclaration",
      "FunctionDiscontinuities",
      "FunctionDomain",
      "FunctionExpand",
      "FunctionInjective",
      "FunctionInterpolation",
      "FunctionLayer",
      "FunctionMeromorphic",
      "FunctionMonotonicity",
      "FunctionPeriod",
      "FunctionPoles",
      "FunctionRange",
      "FunctionSign",
      "FunctionSingularities",
      "FunctionSpace",
      "FunctionSurjective",
      "FussellVeselyImportance",
      "GaborFilter",
      "GaborMatrix",
      "GaborWavelet",
      "GainMargins",
      "GainPhaseMargins",
      "GalaxyData",
      "GalleryView",
      "Gamma",
      "GammaDistribution",
      "GammaRegularized",
      "GapPenalty",
      "GARCHProcess",
      "GatedRecurrentLayer",
      "Gather",
      "GatherBy",
      "GaugeFaceElementFunction",
      "GaugeFaceStyle",
      "GaugeFrameElementFunction",
      "GaugeFrameSize",
      "GaugeFrameStyle",
      "GaugeLabels",
      "GaugeMarkers",
      "GaugeStyle",
      "GaussianFilter",
      "GaussianIntegers",
      "GaussianMatrix",
      "GaussianOrthogonalMatrixDistribution",
      "GaussianSymplecticMatrixDistribution",
      "GaussianUnitaryMatrixDistribution",
      "GaussianWindow",
      "GCD",
      "GegenbauerC",
      "General",
      "GeneralizedLinearModelFit",
      "GenerateAsymmetricKeyPair",
      "GenerateConditions",
      "GeneratedAssetFormat",
      "GeneratedAssetLocation",
      "GeneratedCell",
      "GeneratedCellStyles",
      "GeneratedDocumentBinding",
      "GenerateDerivedKey",
      "GenerateDigitalSignature",
      "GenerateDocument",
      "GeneratedParameters",
      "GeneratedQuantityMagnitudes",
      "GenerateFileSignature",
      "GenerateHTTPResponse",
      "GenerateSecuredAuthenticationKey",
      "GenerateSymmetricKey",
      "GeneratingFunction",
      "GeneratorDescription",
      "GeneratorHistoryLength",
      "GeneratorOutputType",
      "Generic",
      "GenericCylindricalDecomposition",
      "GenomeData",
      "GenomeLookup",
      "GeoAntipode",
      "GeoArea",
      "GeoArraySize",
      "GeoBackground",
      "GeoBoundary",
      "GeoBoundingBox",
      "GeoBounds",
      "GeoBoundsRegion",
      "GeoBoundsRegionBoundary",
      "GeoBubbleChart",
      "GeoCenter",
      "GeoCircle",
      "GeoContourPlot",
      "GeoDensityPlot",
      "GeodesicClosing",
      "GeodesicDilation",
      "GeodesicErosion",
      "GeodesicOpening",
      "GeodesicPolyhedron",
      "GeoDestination",
      "GeodesyData",
      "GeoDirection",
      "GeoDisk",
      "GeoDisplacement",
      "GeoDistance",
      "GeoDistanceList",
      "GeoElevationData",
      "GeoEntities",
      "GeoGraphics",
      "GeoGraphPlot",
      "GeoGraphValuePlot",
      "GeogravityModelData",
      "GeoGridDirectionDifference",
      "GeoGridLines",
      "GeoGridLinesStyle",
      "GeoGridPosition",
      "GeoGridRange",
      "GeoGridRangePadding",
      "GeoGridUnitArea",
      "GeoGridUnitDistance",
      "GeoGridVector",
      "GeoGroup",
      "GeoHemisphere",
      "GeoHemisphereBoundary",
      "GeoHistogram",
      "GeoIdentify",
      "GeoImage",
      "GeoLabels",
      "GeoLength",
      "GeoListPlot",
      "GeoLocation",
      "GeologicalPeriodData",
      "GeomagneticModelData",
      "GeoMarker",
      "GeometricAssertion",
      "GeometricBrownianMotionProcess",
      "GeometricDistribution",
      "GeometricMean",
      "GeometricMeanFilter",
      "GeometricOptimization",
      "GeometricScene",
      "GeometricStep",
      "GeometricStylingRules",
      "GeometricTest",
      "GeometricTransformation",
      "GeometricTransformation3DBox",
      "GeometricTransformation3DBoxOptions",
      "GeometricTransformationBox",
      "GeometricTransformationBoxOptions",
      "GeoModel",
      "GeoNearest",
      "GeoOrientationData",
      "GeoPath",
      "GeoPolygon",
      "GeoPosition",
      "GeoPositionENU",
      "GeoPositionXYZ",
      "GeoProjection",
      "GeoProjectionData",
      "GeoRange",
      "GeoRangePadding",
      "GeoRegionValuePlot",
      "GeoResolution",
      "GeoScaleBar",
      "GeoServer",
      "GeoSmoothHistogram",
      "GeoStreamPlot",
      "GeoStyling",
      "GeoStylingImageFunction",
      "GeoVariant",
      "GeoVector",
      "GeoVectorENU",
      "GeoVectorPlot",
      "GeoVectorXYZ",
      "GeoVisibleRegion",
      "GeoVisibleRegionBoundary",
      "GeoWithinQ",
      "GeoZoomLevel",
      "GestureHandler",
      "GestureHandlerTag",
      "Get",
      "GetContext",
      "GetEnvironment",
      "GetFileName",
      "GetLinebreakInformationPacket",
      "GibbsPointProcess",
      "Glaisher",
      "GlobalClusteringCoefficient",
      "GlobalPreferences",
      "GlobalSession",
      "Glow",
      "GoldenAngle",
      "GoldenRatio",
      "GompertzMakehamDistribution",
      "GoochShading",
      "GoodmanKruskalGamma",
      "GoodmanKruskalGammaTest",
      "Goto",
      "GouraudShading",
      "Grad",
      "Gradient",
      "GradientFilter",
      "GradientFittedMesh",
      "GradientOrientationFilter",
      "GrammarApply",
      "GrammarRules",
      "GrammarToken",
      "Graph",
      "Graph3D",
      "GraphAssortativity",
      "GraphAutomorphismGroup",
      "GraphCenter",
      "GraphComplement",
      "GraphData",
      "GraphDensity",
      "GraphDiameter",
      "GraphDifference",
      "GraphDisjointUnion",
      "GraphDistance",
      "GraphDistanceMatrix",
      "GraphEmbedding",
      "GraphHighlight",
      "GraphHighlightStyle",
      "GraphHub",
      "Graphics",
      "Graphics3D",
      "Graphics3DBox",
      "Graphics3DBoxOptions",
      "GraphicsArray",
      "GraphicsBaseline",
      "GraphicsBox",
      "GraphicsBoxOptions",
      "GraphicsColor",
      "GraphicsColumn",
      "GraphicsComplex",
      "GraphicsComplex3DBox",
      "GraphicsComplex3DBoxOptions",
      "GraphicsComplexBox",
      "GraphicsComplexBoxOptions",
      "GraphicsContents",
      "GraphicsData",
      "GraphicsGrid",
      "GraphicsGridBox",
      "GraphicsGroup",
      "GraphicsGroup3DBox",
      "GraphicsGroup3DBoxOptions",
      "GraphicsGroupBox",
      "GraphicsGroupBoxOptions",
      "GraphicsGrouping",
      "GraphicsHighlightColor",
      "GraphicsRow",
      "GraphicsSpacing",
      "GraphicsStyle",
      "GraphIntersection",
      "GraphJoin",
      "GraphLayerLabels",
      "GraphLayers",
      "GraphLayerStyle",
      "GraphLayout",
      "GraphLinkEfficiency",
      "GraphPeriphery",
      "GraphPlot",
      "GraphPlot3D",
      "GraphPower",
      "GraphProduct",
      "GraphPropertyDistribution",
      "GraphQ",
      "GraphRadius",
      "GraphReciprocity",
      "GraphRoot",
      "GraphStyle",
      "GraphSum",
      "GraphTree",
      "GraphUnion",
      "Gray",
      "GrayLevel",
      "Greater",
      "GreaterEqual",
      "GreaterEqualLess",
      "GreaterEqualThan",
      "GreaterFullEqual",
      "GreaterGreater",
      "GreaterLess",
      "GreaterSlantEqual",
      "GreaterThan",
      "GreaterTilde",
      "GreekStyle",
      "Green",
      "GreenFunction",
      "Grid",
      "GridBaseline",
      "GridBox",
      "GridBoxAlignment",
      "GridBoxBackground",
      "GridBoxDividers",
      "GridBoxFrame",
      "GridBoxItemSize",
      "GridBoxItemStyle",
      "GridBoxOptions",
      "GridBoxSpacings",
      "GridCreationSettings",
      "GridDefaultElement",
      "GridElementStyleOptions",
      "GridFrame",
      "GridFrameMargins",
      "GridGraph",
      "GridLines",
      "GridLinesStyle",
      "GridVideo",
      "GroebnerBasis",
      "GroupActionBase",
      "GroupBy",
      "GroupCentralizer",
      "GroupElementFromWord",
      "GroupElementPosition",
      "GroupElementQ",
      "GroupElements",
      "GroupElementToWord",
      "GroupGenerators",
      "Groupings",
      "GroupMultiplicationTable",
      "GroupOpenerColor",
      "GroupOpenerInsideFrame",
      "GroupOrbits",
      "GroupOrder",
      "GroupPageBreakWithin",
      "GroupSetwiseStabilizer",
      "GroupStabilizer",
      "GroupStabilizerChain",
      "GroupTogetherGrouping",
      "GroupTogetherNestedGrouping",
      "GrowCutComponents",
      "Gudermannian",
      "GuidedFilter",
      "GumbelDistribution",
      "HaarWavelet",
      "HadamardMatrix",
      "HalfLine",
      "HalfNormalDistribution",
      "HalfPlane",
      "HalfSpace",
      "HalftoneShading",
      "HamiltonianGraphQ",
      "HammingDistance",
      "HammingWindow",
      "HandlerFunctions",
      "HandlerFunctionsKeys",
      "HankelH1",
      "HankelH2",
      "HankelMatrix",
      "HankelTransform",
      "HannPoissonWindow",
      "HannWindow",
      "HaradaNortonGroupHN",
      "HararyGraph",
      "HardcorePointProcess",
      "HarmonicMean",
      "HarmonicMeanFilter",
      "HarmonicNumber",
      "Hash",
      "HatchFilling",
      "HatchShading",
      "Haversine",
      "HazardFunction",
      "Head",
      "HeadCompose",
      "HeaderAlignment",
      "HeaderBackground",
      "HeaderDisplayFunction",
      "HeaderLines",
      "Headers",
      "HeaderSize",
      "HeaderStyle",
      "Heads",
      "HeatFluxValue",
      "HeatInsulationValue",
      "HeatOutflowValue",
      "HeatRadiationValue",
      "HeatSymmetryValue",
      "HeatTemperatureCondition",
      "HeatTransferPDEComponent",
      "HeatTransferValue",
      "HeavisideLambda",
      "HeavisidePi",
      "HeavisideTheta",
      "HeldGroupHe",
      "HeldPart",
      "HelmholtzPDEComponent",
      "HelpBrowserLookup",
      "HelpBrowserNotebook",
      "HelpBrowserSettings",
      "HelpViewerSettings",
      "Here",
      "HermiteDecomposition",
      "HermiteH",
      "Hermitian",
      "HermitianMatrixQ",
      "HessenbergDecomposition",
      "Hessian",
      "HeunB",
      "HeunBPrime",
      "HeunC",
      "HeunCPrime",
      "HeunD",
      "HeunDPrime",
      "HeunG",
      "HeunGPrime",
      "HeunT",
      "HeunTPrime",
      "HexadecimalCharacter",
      "Hexahedron",
      "HexahedronBox",
      "HexahedronBoxOptions",
      "HiddenItems",
      "HiddenMarkovProcess",
      "HiddenSurface",
      "Highlighted",
      "HighlightGraph",
      "HighlightImage",
      "HighlightMesh",
      "HighlightString",
      "HighpassFilter",
      "HigmanSimsGroupHS",
      "HilbertCurve",
      "HilbertFilter",
      "HilbertMatrix",
      "Histogram",
      "Histogram3D",
      "HistogramDistribution",
      "HistogramList",
      "HistogramPointDensity",
      "HistogramTransform",
      "HistogramTransformInterpolation",
      "HistoricalPeriodData",
      "HitMissTransform",
      "HITSCentrality",
      "HjorthDistribution",
      "HodgeDual",
      "HoeffdingD",
      "HoeffdingDTest",
      "Hold",
      "HoldAll",
      "HoldAllComplete",
      "HoldComplete",
      "HoldFirst",
      "HoldForm",
      "HoldPattern",
      "HoldRest",
      "HolidayCalendar",
      "HomeDirectory",
      "HomePage",
      "Horizontal",
      "HorizontalForm",
      "HorizontalGauge",
      "HorizontalScrollPosition",
      "HornerForm",
      "HostLookup",
      "HotellingTSquareDistribution",
      "HoytDistribution",
      "HTMLSave",
      "HTTPErrorResponse",
      "HTTPRedirect",
      "HTTPRequest",
      "HTTPRequestData",
      "HTTPResponse",
      "Hue",
      "HumanGrowthData",
      "HumpDownHump",
      "HumpEqual",
      "HurwitzLerchPhi",
      "HurwitzZeta",
      "HyperbolicDistribution",
      "HypercubeGraph",
      "HyperexponentialDistribution",
      "Hyperfactorial",
      "Hypergeometric0F1",
      "Hypergeometric0F1Regularized",
      "Hypergeometric1F1",
      "Hypergeometric1F1Regularized",
      "Hypergeometric2F1",
      "Hypergeometric2F1Regularized",
      "HypergeometricDistribution",
      "HypergeometricPFQ",
      "HypergeometricPFQRegularized",
      "HypergeometricU",
      "Hyperlink",
      "HyperlinkAction",
      "HyperlinkCreationSettings",
      "Hyperplane",
      "Hyphenation",
      "HyphenationOptions",
      "HypoexponentialDistribution",
      "HypothesisTestData",
      "I",
      "IconData",
      "Iconize",
      "IconizedObject",
      "IconRules",
      "Icosahedron",
      "Identity",
      "IdentityMatrix",
      "If",
      "IfCompiled",
      "IgnoreCase",
      "IgnoreDiacritics",
      "IgnoreIsotopes",
      "IgnorePunctuation",
      "IgnoreSpellCheck",
      "IgnoreStereochemistry",
      "IgnoringInactive",
      "Im",
      "Image",
      "Image3D",
      "Image3DProjection",
      "Image3DSlices",
      "ImageAccumulate",
      "ImageAdd",
      "ImageAdjust",
      "ImageAlign",
      "ImageApply",
      "ImageApplyIndexed",
      "ImageAspectRatio",
      "ImageAssemble",
      "ImageAugmentationLayer",
      "ImageBoundingBoxes",
      "ImageCache",
      "ImageCacheValid",
      "ImageCapture",
      "ImageCaptureFunction",
      "ImageCases",
      "ImageChannels",
      "ImageClip",
      "ImageCollage",
      "ImageColorSpace",
      "ImageCompose",
      "ImageContainsQ",
      "ImageContents",
      "ImageConvolve",
      "ImageCooccurrence",
      "ImageCorners",
      "ImageCorrelate",
      "ImageCorrespondingPoints",
      "ImageCrop",
      "ImageData",
      "ImageDeconvolve",
      "ImageDemosaic",
      "ImageDifference",
      "ImageDimensions",
      "ImageDisplacements",
      "ImageDistance",
      "ImageEditMode",
      "ImageEffect",
      "ImageExposureCombine",
      "ImageFeatureTrack",
      "ImageFileApply",
      "ImageFileFilter",
      "ImageFileScan",
      "ImageFilter",
      "ImageFocusCombine",
      "ImageForestingComponents",
      "ImageFormattingWidth",
      "ImageForwardTransformation",
      "ImageGraphics",
      "ImageHistogram",
      "ImageIdentify",
      "ImageInstanceQ",
      "ImageKeypoints",
      "ImageLabels",
      "ImageLegends",
      "ImageLevels",
      "ImageLines",
      "ImageMargins",
      "ImageMarker",
      "ImageMarkers",
      "ImageMeasurements",
      "ImageMesh",
      "ImageMultiply",
      "ImageOffset",
      "ImagePad",
      "ImagePadding",
      "ImagePartition",
      "ImagePeriodogram",
      "ImagePerspectiveTransformation",
      "ImagePosition",
      "ImagePreviewFunction",
      "ImagePyramid",
      "ImagePyramidApply",
      "ImageQ",
      "ImageRangeCache",
      "ImageRecolor",
      "ImageReflect",
      "ImageRegion",
      "ImageResize",
      "ImageResolution",
      "ImageRestyle",
      "ImageRotate",
      "ImageRotated",
      "ImageSaliencyFilter",
      "ImageScaled",
      "ImageScan",
      "ImageSize",
      "ImageSizeAction",
      "ImageSizeCache",
      "ImageSizeMultipliers",
      "ImageSizeRaw",
      "ImageStitch",
      "ImageSubtract",
      "ImageTake",
      "ImageTransformation",
      "ImageTrim",
      "ImageType",
      "ImageValue",
      "ImageValuePositions",
      "ImageVectorscopePlot",
      "ImageWaveformPlot",
      "ImagingDevice",
      "ImplicitD",
      "ImplicitRegion",
      "Implies",
      "Import",
      "ImportAutoReplacements",
      "ImportByteArray",
      "ImportedObject",
      "ImportOptions",
      "ImportString",
      "ImprovementImportance",
      "In",
      "Inactivate",
      "Inactive",
      "InactiveStyle",
      "IncidenceGraph",
      "IncidenceList",
      "IncidenceMatrix",
      "IncludeAromaticBonds",
      "IncludeConstantBasis",
      "IncludedContexts",
      "IncludeDefinitions",
      "IncludeDirectories",
      "IncludeFileExtension",
      "IncludeGeneratorTasks",
      "IncludeHydrogens",
      "IncludeInflections",
      "IncludeMetaInformation",
      "IncludePods",
      "IncludeQuantities",
      "IncludeRelatedTables",
      "IncludeSingularSolutions",
      "IncludeSingularTerm",
      "IncludeWindowTimes",
      "Increment",
      "IndefiniteMatrixQ",
      "Indent",
      "IndentingNewlineSpacings",
      "IndentMaxFraction",
      "IndependenceTest",
      "IndependentEdgeSetQ",
      "IndependentPhysicalQuantity",
      "IndependentUnit",
      "IndependentUnitDimension",
      "IndependentVertexSetQ",
      "Indeterminate",
      "IndeterminateThreshold",
      "IndexCreationOptions",
      "Indexed",
      "IndexEdgeTaggedGraph",
      "IndexGraph",
      "IndexTag",
      "Inequality",
      "InertEvaluate",
      "InertExpression",
      "InexactNumberQ",
      "InexactNumbers",
      "InfiniteFuture",
      "InfiniteLine",
      "InfiniteLineThrough",
      "InfinitePast",
      "InfinitePlane",
      "Infinity",
      "Infix",
      "InflationAdjust",
      "InflationMethod",
      "Information",
      "InformationData",
      "InformationDataGrid",
      "Inherited",
      "InheritScope",
      "InhomogeneousPoissonPointProcess",
      "InhomogeneousPoissonProcess",
      "InitialEvaluationHistory",
      "Initialization",
      "InitializationCell",
      "InitializationCellEvaluation",
      "InitializationCellWarning",
      "InitializationObject",
      "InitializationObjects",
      "InitializationValue",
      "Initialize",
      "InitialSeeding",
      "InlineCounterAssignments",
      "InlineCounterIncrements",
      "InlineRules",
      "Inner",
      "InnerPolygon",
      "InnerPolyhedron",
      "Inpaint",
      "Input",
      "InputAliases",
      "InputAssumptions",
      "InputAutoReplacements",
      "InputField",
      "InputFieldBox",
      "InputFieldBoxOptions",
      "InputForm",
      "InputGrouping",
      "InputNamePacket",
      "InputNotebook",
      "InputPacket",
      "InputPorts",
      "InputSettings",
      "InputStream",
      "InputString",
      "InputStringPacket",
      "InputToBoxFormPacket",
      "Insert",
      "InsertionFunction",
      "InsertionPointObject",
      "InsertLinebreaks",
      "InsertResults",
      "Inset",
      "Inset3DBox",
      "Inset3DBoxOptions",
      "InsetBox",
      "InsetBoxOptions",
      "Insphere",
      "Install",
      "InstallService",
      "InstanceNormalizationLayer",
      "InString",
      "Integer",
      "IntegerDigits",
      "IntegerExponent",
      "IntegerLength",
      "IntegerName",
      "IntegerPart",
      "IntegerPartitions",
      "IntegerQ",
      "IntegerReverse",
      "Integers",
      "IntegerString",
      "Integral",
      "Integrate",
      "IntegrateChangeVariables",
      "Interactive",
      "InteractiveTradingChart",
      "InterfaceSwitched",
      "Interlaced",
      "Interleaving",
      "InternallyBalancedDecomposition",
      "InterpolatingFunction",
      "InterpolatingPolynomial",
      "Interpolation",
      "InterpolationOrder",
      "InterpolationPoints",
      "InterpolationPrecision",
      "Interpretation",
      "InterpretationBox",
      "InterpretationBoxOptions",
      "InterpretationFunction",
      "Interpreter",
      "InterpretTemplate",
      "InterquartileRange",
      "Interrupt",
      "InterruptSettings",
      "IntersectedEntityClass",
      "IntersectingQ",
      "Intersection",
      "Interval",
      "IntervalIntersection",
      "IntervalMarkers",
      "IntervalMarkersStyle",
      "IntervalMemberQ",
      "IntervalSlider",
      "IntervalUnion",
      "Into",
      "Inverse",
      "InverseBetaRegularized",
      "InverseBilateralLaplaceTransform",
      "InverseBilateralZTransform",
      "InverseCDF",
      "InverseChiSquareDistribution",
      "InverseContinuousWaveletTransform",
      "InverseDistanceTransform",
      "InverseEllipticNomeQ",
      "InverseErf",
      "InverseErfc",
      "InverseFourier",
      "InverseFourierCosTransform",
      "InverseFourierSequenceTransform",
      "InverseFourierSinTransform",
      "InverseFourierTransform",
      "InverseFunction",
      "InverseFunctions",
      "InverseGammaDistribution",
      "InverseGammaRegularized",
      "InverseGaussianDistribution",
      "InverseGudermannian",
      "InverseHankelTransform",
      "InverseHaversine",
      "InverseImagePyramid",
      "InverseJacobiCD",
      "InverseJacobiCN",
      "InverseJacobiCS",
      "InverseJacobiDC",
      "InverseJacobiDN",
      "InverseJacobiDS",
      "InverseJacobiNC",
      "InverseJacobiND",
      "InverseJacobiNS",
      "InverseJacobiSC",
      "InverseJacobiSD",
      "InverseJacobiSN",
      "InverseLaplaceTransform",
      "InverseMellinTransform",
      "InversePermutation",
      "InverseRadon",
      "InverseRadonTransform",
      "InverseSeries",
      "InverseShortTimeFourier",
      "InverseSpectrogram",
      "InverseSurvivalFunction",
      "InverseTransformedRegion",
      "InverseWaveletTransform",
      "InverseWeierstrassP",
      "InverseWishartMatrixDistribution",
      "InverseZTransform",
      "Invisible",
      "InvisibleApplication",
      "InvisibleTimes",
      "IPAddress",
      "IrreduciblePolynomialQ",
      "IslandData",
      "IsolatingInterval",
      "IsomorphicGraphQ",
      "IsomorphicSubgraphQ",
      "IsotopeData",
      "Italic",
      "Item",
      "ItemAspectRatio",
      "ItemBox",
      "ItemBoxOptions",
      "ItemDisplayFunction",
      "ItemSize",
      "ItemStyle",
      "ItoProcess",
      "JaccardDissimilarity",
      "JacobiAmplitude",
      "Jacobian",
      "JacobiCD",
      "JacobiCN",
      "JacobiCS",
      "JacobiDC",
      "JacobiDN",
      "JacobiDS",
      "JacobiEpsilon",
      "JacobiNC",
      "JacobiND",
      "JacobiNS",
      "JacobiP",
      "JacobiSC",
      "JacobiSD",
      "JacobiSN",
      "JacobiSymbol",
      "JacobiZeta",
      "JacobiZN",
      "JankoGroupJ1",
      "JankoGroupJ2",
      "JankoGroupJ3",
      "JankoGroupJ4",
      "JarqueBeraALMTest",
      "JohnsonDistribution",
      "Join",
      "JoinAcross",
      "Joined",
      "JoinedCurve",
      "JoinedCurveBox",
      "JoinedCurveBoxOptions",
      "JoinForm",
      "JordanDecomposition",
      "JordanModelDecomposition",
      "JulianDate",
      "JuliaSetBoettcher",
      "JuliaSetIterationCount",
      "JuliaSetPlot",
      "JuliaSetPoints",
      "K",
      "KagiChart",
      "KaiserBesselWindow",
      "KaiserWindow",
      "KalmanEstimator",
      "KalmanFilter",
      "KarhunenLoeveDecomposition",
      "KaryTree",
      "KatzCentrality",
      "KCoreComponents",
      "KDistribution",
      "KEdgeConnectedComponents",
      "KEdgeConnectedGraphQ",
      "KeepExistingVersion",
      "KelvinBei",
      "KelvinBer",
      "KelvinKei",
      "KelvinKer",
      "KendallTau",
      "KendallTauTest",
      "KernelConfiguration",
      "KernelExecute",
      "KernelFunction",
      "KernelMixtureDistribution",
      "KernelObject",
      "Kernels",
      "Ket",
      "Key",
      "KeyCollisionFunction",
      "KeyComplement",
      "KeyDrop",
      "KeyDropFrom",
      "KeyExistsQ",
      "KeyFreeQ",
      "KeyIntersection",
      "KeyMap",
      "KeyMemberQ",
      "KeypointStrength",
      "Keys",
      "KeySelect",
      "KeySort",
      "KeySortBy",
      "KeyTake",
      "KeyUnion",
      "KeyValueMap",
      "KeyValuePattern",
      "Khinchin",
      "KillProcess",
      "KirchhoffGraph",
      "KirchhoffMatrix",
      "KleinInvariantJ",
      "KnapsackSolve",
      "KnightTourGraph",
      "KnotData",
      "KnownUnitQ",
      "KochCurve",
      "KolmogorovSmirnovTest",
      "KroneckerDelta",
      "KroneckerModelDecomposition",
      "KroneckerProduct",
      "KroneckerSymbol",
      "KuiperTest",
      "KumaraswamyDistribution",
      "Kurtosis",
      "KuwaharaFilter",
      "KVertexConnectedComponents",
      "KVertexConnectedGraphQ",
      "LABColor",
      "Label",
      "Labeled",
      "LabeledSlider",
      "LabelingFunction",
      "LabelingSize",
      "LabelStyle",
      "LabelVisibility",
      "LaguerreL",
      "LakeData",
      "LambdaComponents",
      "LambertW",
      "LameC",
      "LameCPrime",
      "LameEigenvalueA",
      "LameEigenvalueB",
      "LameS",
      "LameSPrime",
      "LaminaData",
      "LanczosWindow",
      "LandauDistribution",
      "Language",
      "LanguageCategory",
      "LanguageData",
      "LanguageIdentify",
      "LanguageOptions",
      "LaplaceDistribution",
      "LaplaceTransform",
      "Laplacian",
      "LaplacianFilter",
      "LaplacianGaussianFilter",
      "LaplacianPDETerm",
      "Large",
      "Larger",
      "Last",
      "Latitude",
      "LatitudeLongitude",
      "LatticeData",
      "LatticeReduce",
      "Launch",
      "LaunchKernels",
      "LayeredGraphPlot",
      "LayeredGraphPlot3D",
      "LayerSizeFunction",
      "LayoutInformation",
      "LCHColor",
      "LCM",
      "LeaderSize",
      "LeafCount",
      "LeapVariant",
      "LeapYearQ",
      "LearnDistribution",
      "LearnedDistribution",
      "LearningRate",
      "LearningRateMultipliers",
      "LeastSquares",
      "LeastSquaresFilterKernel",
      "Left",
      "LeftArrow",
      "LeftArrowBar",
      "LeftArrowRightArrow",
      "LeftDownTeeVector",
      "LeftDownVector",
      "LeftDownVectorBar",
      "LeftRightArrow",
      "LeftRightVector",
      "LeftTee",
      "LeftTeeArrow",
      "LeftTeeVector",
      "LeftTriangle",
      "LeftTriangleBar",
      "LeftTriangleEqual",
      "LeftUpDownVector",
      "LeftUpTeeVector",
      "LeftUpVector",
      "LeftUpVectorBar",
      "LeftVector",
      "LeftVectorBar",
      "LegendAppearance",
      "Legended",
      "LegendFunction",
      "LegendLabel",
      "LegendLayout",
      "LegendMargins",
      "LegendMarkers",
      "LegendMarkerSize",
      "LegendreP",
      "LegendreQ",
      "LegendreType",
      "Length",
      "LengthWhile",
      "LerchPhi",
      "Less",
      "LessEqual",
      "LessEqualGreater",
      "LessEqualThan",
      "LessFullEqual",
      "LessGreater",
      "LessLess",
      "LessSlantEqual",
      "LessThan",
      "LessTilde",
      "LetterCharacter",
      "LetterCounts",
      "LetterNumber",
      "LetterQ",
      "Level",
      "LeveneTest",
      "LeviCivitaTensor",
      "LevyDistribution",
      "Lexicographic",
      "LexicographicOrder",
      "LexicographicSort",
      "LibraryDataType",
      "LibraryFunction",
      "LibraryFunctionDeclaration",
      "LibraryFunctionError",
      "LibraryFunctionInformation",
      "LibraryFunctionLoad",
      "LibraryFunctionUnload",
      "LibraryLoad",
      "LibraryUnload",
      "LicenseEntitlementObject",
      "LicenseEntitlements",
      "LicenseID",
      "LicensingSettings",
      "LiftingFilterData",
      "LiftingWaveletTransform",
      "LightBlue",
      "LightBrown",
      "LightCyan",
      "Lighter",
      "LightGray",
      "LightGreen",
      "Lighting",
      "LightingAngle",
      "LightMagenta",
      "LightOrange",
      "LightPink",
      "LightPurple",
      "LightRed",
      "LightSources",
      "LightYellow",
      "Likelihood",
      "Limit",
      "LimitsPositioning",
      "LimitsPositioningTokens",
      "LindleyDistribution",
      "Line",
      "Line3DBox",
      "Line3DBoxOptions",
      "LinearFilter",
      "LinearFractionalOptimization",
      "LinearFractionalTransform",
      "LinearGradientFilling",
      "LinearGradientImage",
      "LinearizingTransformationData",
      "LinearLayer",
      "LinearModelFit",
      "LinearOffsetFunction",
      "LinearOptimization",
      "LinearProgramming",
      "LinearRecurrence",
      "LinearSolve",
      "LinearSolveFunction",
      "LineBox",
      "LineBoxOptions",
      "LineBreak",
      "LinebreakAdjustments",
      "LineBreakChart",
      "LinebreakSemicolonWeighting",
      "LineBreakWithin",
      "LineColor",
      "LineGraph",
      "LineIndent",
      "LineIndentMaxFraction",
      "LineIntegralConvolutionPlot",
      "LineIntegralConvolutionScale",
      "LineLegend",
      "LineOpacity",
      "LineSpacing",
      "LineWrapParts",
      "LinkActivate",
      "LinkClose",
      "LinkConnect",
      "LinkConnectedQ",
      "LinkCreate",
      "LinkError",
      "LinkFlush",
      "LinkFunction",
      "LinkHost",
      "LinkInterrupt",
      "LinkLaunch",
      "LinkMode",
      "LinkObject",
      "LinkOpen",
      "LinkOptions",
      "LinkPatterns",
      "LinkProtocol",
      "LinkRankCentrality",
      "LinkRead",
      "LinkReadHeld",
      "LinkReadyQ",
      "Links",
      "LinkService",
      "LinkWrite",
      "LinkWriteHeld",
      "LiouvilleLambda",
      "List",
      "Listable",
      "ListAnimate",
      "ListContourPlot",
      "ListContourPlot3D",
      "ListConvolve",
      "ListCorrelate",
      "ListCurvePathPlot",
      "ListDeconvolve",
      "ListDensityPlot",
      "ListDensityPlot3D",
      "Listen",
      "ListFormat",
      "ListFourierSequenceTransform",
      "ListInterpolation",
      "ListLineIntegralConvolutionPlot",
      "ListLinePlot",
      "ListLinePlot3D",
      "ListLogLinearPlot",
      "ListLogLogPlot",
      "ListLogPlot",
      "ListPicker",
      "ListPickerBox",
      "ListPickerBoxBackground",
      "ListPickerBoxOptions",
      "ListPlay",
      "ListPlot",
      "ListPlot3D",
      "ListPointPlot3D",
      "ListPolarPlot",
      "ListQ",
      "ListSliceContourPlot3D",
      "ListSliceDensityPlot3D",
      "ListSliceVectorPlot3D",
      "ListStepPlot",
      "ListStreamDensityPlot",
      "ListStreamPlot",
      "ListStreamPlot3D",
      "ListSurfacePlot3D",
      "ListVectorDensityPlot",
      "ListVectorDisplacementPlot",
      "ListVectorDisplacementPlot3D",
      "ListVectorPlot",
      "ListVectorPlot3D",
      "ListZTransform",
      "Literal",
      "LiteralSearch",
      "LiteralType",
      "LoadCompiledComponent",
      "LocalAdaptiveBinarize",
      "LocalCache",
      "LocalClusteringCoefficient",
      "LocalEvaluate",
      "LocalizeDefinitions",
      "LocalizeVariables",
      "LocalObject",
      "LocalObjects",
      "LocalResponseNormalizationLayer",
      "LocalSubmit",
      "LocalSymbol",
      "LocalTime",
      "LocalTimeZone",
      "LocationEquivalenceTest",
      "LocationTest",
      "Locator",
      "LocatorAutoCreate",
      "LocatorBox",
      "LocatorBoxOptions",
      "LocatorCentering",
      "LocatorPane",
      "LocatorPaneBox",
      "LocatorPaneBoxOptions",
      "LocatorRegion",
      "Locked",
      "Log",
      "Log10",
      "Log2",
      "LogBarnesG",
      "LogGamma",
      "LogGammaDistribution",
      "LogicalExpand",
      "LogIntegral",
      "LogisticDistribution",
      "LogisticSigmoid",
      "LogitModelFit",
      "LogLikelihood",
      "LogLinearPlot",
      "LogLogisticDistribution",
      "LogLogPlot",
      "LogMultinormalDistribution",
      "LogNormalDistribution",
      "LogPlot",
      "LogRankTest",
      "LogSeriesDistribution",
      "LongEqual",
      "Longest",
      "LongestCommonSequence",
      "LongestCommonSequencePositions",
      "LongestCommonSubsequence",
      "LongestCommonSubsequencePositions",
      "LongestMatch",
      "LongestOrderedSequence",
      "LongForm",
      "Longitude",
      "LongLeftArrow",
      "LongLeftRightArrow",
      "LongRightArrow",
      "LongShortTermMemoryLayer",
      "Lookup",
      "Loopback",
      "LoopFreeGraphQ",
      "Looping",
      "LossFunction",
      "LowerCaseQ",
      "LowerLeftArrow",
      "LowerRightArrow",
      "LowerTriangularize",
      "LowerTriangularMatrix",
      "LowerTriangularMatrixQ",
      "LowpassFilter",
      "LQEstimatorGains",
      "LQGRegulator",
      "LQOutputRegulatorGains",
      "LQRegulatorGains",
      "LUBackSubstitution",
      "LucasL",
      "LuccioSamiComponents",
      "LUDecomposition",
      "LunarEclipse",
      "LUVColor",
      "LyapunovSolve",
      "LyonsGroupLy",
      "MachineID",
      "MachineName",
      "MachineNumberQ",
      "MachinePrecision",
      "MacintoshSystemPageSetup",
      "Magenta",
      "Magnification",
      "Magnify",
      "MailAddressValidation",
      "MailExecute",
      "MailFolder",
      "MailItem",
      "MailReceiverFunction",
      "MailResponseFunction",
      "MailSearch",
      "MailServerConnect",
      "MailServerConnection",
      "MailSettings",
      "MainSolve",
      "MaintainDynamicCaches",
      "Majority",
      "MakeBoxes",
      "MakeExpression",
      "MakeRules",
      "ManagedLibraryExpressionID",
      "ManagedLibraryExpressionQ",
      "MandelbrotSetBoettcher",
      "MandelbrotSetDistance",
      "MandelbrotSetIterationCount",
      "MandelbrotSetMemberQ",
      "MandelbrotSetPlot",
      "MangoldtLambda",
      "ManhattanDistance",
      "Manipulate",
      "Manipulator",
      "MannedSpaceMissionData",
      "MannWhitneyTest",
      "MantissaExponent",
      "Manual",
      "Map",
      "MapAll",
      "MapApply",
      "MapAt",
      "MapIndexed",
      "MAProcess",
      "MapThread",
      "MarchenkoPasturDistribution",
      "MarcumQ",
      "MardiaCombinedTest",
      "MardiaKurtosisTest",
      "MardiaSkewnessTest",
      "MarginalDistribution",
      "MarkovProcessProperties",
      "Masking",
      "MassConcentrationCondition",
      "MassFluxValue",
      "MassImpermeableBoundaryValue",
      "MassOutflowValue",
      "MassSymmetryValue",
      "MassTransferValue",
      "MassTransportPDEComponent",
      "MatchingDissimilarity",
      "MatchLocalNameQ",
      "MatchLocalNames",
      "MatchQ",
      "Material",
      "MaterialShading",
      "MaternPointProcess",
      "MathematicalFunctionData",
      "MathematicaNotation",
      "MathieuC",
      "MathieuCharacteristicA",
      "MathieuCharacteristicB",
      "MathieuCharacteristicExponent",
      "MathieuCPrime",
      "MathieuGroupM11",
      "MathieuGroupM12",
      "MathieuGroupM22",
      "MathieuGroupM23",
      "MathieuGroupM24",
      "MathieuS",
      "MathieuSPrime",
      "MathMLForm",
      "MathMLText",
      "Matrices",
      "MatrixExp",
      "MatrixForm",
      "MatrixFunction",
      "MatrixLog",
      "MatrixNormalDistribution",
      "MatrixPlot",
      "MatrixPower",
      "MatrixPropertyDistribution",
      "MatrixQ",
      "MatrixRank",
      "MatrixTDistribution",
      "Max",
      "MaxBend",
      "MaxCellMeasure",
      "MaxColorDistance",
      "MaxDate",
      "MaxDetect",
      "MaxDisplayedChildren",
      "MaxDuration",
      "MaxExtraBandwidths",
      "MaxExtraConditions",
      "MaxFeatureDisplacement",
      "MaxFeatures",
      "MaxFilter",
      "MaximalBy",
      "Maximize",
      "MaxItems",
      "MaxIterations",
      "MaxLimit",
      "MaxMemoryUsed",
      "MaxMixtureKernels",
      "MaxOverlapFraction",
      "MaxPlotPoints",
      "MaxPoints",
      "MaxRecursion",
      "MaxStableDistribution",
      "MaxStepFraction",
      "MaxSteps",
      "MaxStepSize",
      "MaxTrainingRounds",
      "MaxValue",
      "MaxwellDistribution",
      "MaxWordGap",
      "McLaughlinGroupMcL",
      "Mean",
      "MeanAbsoluteLossLayer",
      "MeanAround",
      "MeanClusteringCoefficient",
      "MeanDegreeConnectivity",
      "MeanDeviation",
      "MeanFilter",
      "MeanGraphDistance",
      "MeanNeighborDegree",
      "MeanPointDensity",
      "MeanShift",
      "MeanShiftFilter",
      "MeanSquaredLossLayer",
      "Median",
      "MedianDeviation",
      "MedianFilter",
      "MedicalTestData",
      "Medium",
      "MeijerG",
      "MeijerGReduce",
      "MeixnerDistribution",
      "MellinConvolve",
      "MellinTransform",
      "MemberQ",
      "MemoryAvailable",
      "MemoryConstrained",
      "MemoryConstraint",
      "MemoryInUse",
      "MengerMesh",
      "Menu",
      "MenuAppearance",
      "MenuCommandKey",
      "MenuEvaluator",
      "MenuItem",
      "MenuList",
      "MenuPacket",
      "MenuSortingValue",
      "MenuStyle",
      "MenuView",
      "Merge",
      "MergeDifferences",
      "MergingFunction",
      "MersennePrimeExponent",
      "MersennePrimeExponentQ",
      "Mesh",
      "MeshCellCentroid",
      "MeshCellCount",
      "MeshCellHighlight",
      "MeshCellIndex",
      "MeshCellLabel",
      "MeshCellMarker",
      "MeshCellMeasure",
      "MeshCellQuality",
      "MeshCells",
      "MeshCellShapeFunction",
      "MeshCellStyle",
      "MeshConnectivityGraph",
      "MeshCoordinates",
      "MeshFunctions",
      "MeshPrimitives",
      "MeshQualityGoal",
      "MeshRange",
      "MeshRefinementFunction",
      "MeshRegion",
      "MeshRegionQ",
      "MeshShading",
      "MeshStyle",
      "Message",
      "MessageDialog",
      "MessageList",
      "MessageName",
      "MessageObject",
      "MessageOptions",
      "MessagePacket",
      "Messages",
      "MessagesNotebook",
      "MetaCharacters",
      "MetaInformation",
      "MeteorShowerData",
      "Method",
      "MethodOptions",
      "MexicanHatWavelet",
      "MeyerWavelet",
      "Midpoint",
      "MIMETypeToFormatList",
      "Min",
      "MinColorDistance",
      "MinDate",
      "MinDetect",
      "MineralData",
      "MinFilter",
      "MinimalBy",
      "MinimalPolynomial",
      "MinimalStateSpaceModel",
      "Minimize",
      "MinimumTimeIncrement",
      "MinIntervalSize",
      "MinkowskiQuestionMark",
      "MinLimit",
      "MinMax",
      "MinorPlanetData",
      "Minors",
      "MinPointSeparation",
      "MinRecursion",
      "MinSize",
      "MinStableDistribution",
      "Minus",
      "MinusPlus",
      "MinValue",
      "Missing",
      "MissingBehavior",
      "MissingDataMethod",
      "MissingDataRules",
      "MissingQ",
      "MissingString",
      "MissingStyle",
      "MissingValuePattern",
      "MissingValueSynthesis",
      "MittagLefflerE",
      "MixedFractionParts",
      "MixedGraphQ",
      "MixedMagnitude",
      "MixedRadix",
      "MixedRadixQuantity",
      "MixedUnit",
      "MixtureDistribution",
      "Mod",
      "Modal",
      "Mode",
      "ModelPredictiveController",
      "Modular",
      "ModularInverse",
      "ModularLambda",
      "Module",
      "Modulus",
      "MoebiusMu",
      "Molecule",
      "MoleculeAlign",
      "MoleculeContainsQ",
      "MoleculeDraw",
      "MoleculeEquivalentQ",
      "MoleculeFreeQ",
      "MoleculeGraph",
      "MoleculeMatchQ",
      "MoleculeMaximumCommonSubstructure",
      "MoleculeModify",
      "MoleculeName",
      "MoleculePattern",
      "MoleculePlot",
      "MoleculePlot3D",
      "MoleculeProperty",
      "MoleculeQ",
      "MoleculeRecognize",
      "MoleculeSubstructureCount",
      "MoleculeValue",
      "Moment",
      "MomentConvert",
      "MomentEvaluate",
      "MomentGeneratingFunction",
      "MomentOfInertia",
      "Monday",
      "Monitor",
      "MonomialList",
      "MonomialOrder",
      "MonsterGroupM",
      "MoonPhase",
      "MoonPosition",
      "MorletWavelet",
      "MorphologicalBinarize",
      "MorphologicalBranchPoints",
      "MorphologicalComponents",
      "MorphologicalEulerNumber",
      "MorphologicalGraph",
      "MorphologicalPerimeter",
      "MorphologicalTransform",
      "MortalityData",
      "Most",
      "MountainData",
      "MouseAnnotation",
      "MouseAppearance",
      "MouseAppearanceTag",
      "MouseButtons",
      "Mouseover",
      "MousePointerNote",
      "MousePosition",
      "MovieData",
      "MovingAverage",
      "MovingMap",
      "MovingMedian",
      "MoyalDistribution",
      "MultiaxisArrangement",
      "Multicolumn",
      "MultiedgeStyle",
      "MultigraphQ",
      "MultilaunchWarning",
      "MultiLetterItalics",
      "MultiLetterStyle",
      "MultilineFunction",
      "Multinomial",
      "MultinomialDistribution",
      "MultinormalDistribution",
      "MultiplicativeOrder",
      "Multiplicity",
      "MultiplySides",
      "MultiscriptBoxOptions",
      "Multiselection",
      "MultivariateHypergeometricDistribution",
      "MultivariatePoissonDistribution",
      "MultivariateTDistribution",
      "N",
      "NakagamiDistribution",
      "NameQ",
      "Names",
      "NamespaceBox",
      "NamespaceBoxOptions",
      "Nand",
      "NArgMax",
      "NArgMin",
      "NBernoulliB",
      "NBodySimulation",
      "NBodySimulationData",
      "NCache",
      "NCaputoD",
      "NDEigensystem",
      "NDEigenvalues",
      "NDSolve",
      "NDSolveValue",
      "Nearest",
      "NearestFunction",
      "NearestMeshCells",
      "NearestNeighborG",
      "NearestNeighborGraph",
      "NearestTo",
      "NebulaData",
      "NeedlemanWunschSimilarity",
      "Needs",
      "Negative",
      "NegativeBinomialDistribution",
      "NegativeDefiniteMatrixQ",
      "NegativeIntegers",
      "NegativelyOrientedPoints",
      "NegativeMultinomialDistribution",
      "NegativeRationals",
      "NegativeReals",
      "NegativeSemidefiniteMatrixQ",
      "NeighborhoodData",
      "NeighborhoodGraph",
      "Nest",
      "NestedGreaterGreater",
      "NestedLessLess",
      "NestedScriptRules",
      "NestGraph",
      "NestList",
      "NestTree",
      "NestWhile",
      "NestWhileList",
      "NetAppend",
      "NetArray",
      "NetArrayLayer",
      "NetBidirectionalOperator",
      "NetChain",
      "NetDecoder",
      "NetDelete",
      "NetDrop",
      "NetEncoder",
      "NetEvaluationMode",
      "NetExternalObject",
      "NetExtract",
      "NetFlatten",
      "NetFoldOperator",
      "NetGANOperator",
      "NetGraph",
      "NetInformation",
      "NetInitialize",
      "NetInsert",
      "NetInsertSharedArrays",
      "NetJoin",
      "NetMapOperator",
      "NetMapThreadOperator",
      "NetMeasurements",
      "NetModel",
      "NetNestOperator",
      "NetPairEmbeddingOperator",
      "NetPort",
      "NetPortGradient",
      "NetPrepend",
      "NetRename",
      "NetReplace",
      "NetReplacePart",
      "NetSharedArray",
      "NetStateObject",
      "NetTake",
      "NetTrain",
      "NetTrainResultsObject",
      "NetUnfold",
      "NetworkPacketCapture",
      "NetworkPacketRecording",
      "NetworkPacketRecordingDuring",
      "NetworkPacketTrace",
      "NeumannValue",
      "NevilleThetaC",
      "NevilleThetaD",
      "NevilleThetaN",
      "NevilleThetaS",
      "NewPrimitiveStyle",
      "NExpectation",
      "Next",
      "NextCell",
      "NextDate",
      "NextPrime",
      "NextScheduledTaskTime",
      "NeymanScottPointProcess",
      "NFractionalD",
      "NHoldAll",
      "NHoldFirst",
      "NHoldRest",
      "NicholsGridLines",
      "NicholsPlot",
      "NightHemisphere",
      "NIntegrate",
      "NMaximize",
      "NMaxValue",
      "NMinimize",
      "NMinValue",
      "NominalScale",
      "NominalVariables",
      "NonAssociative",
      "NoncentralBetaDistribution",
      "NoncentralChiSquareDistribution",
      "NoncentralFRatioDistribution",
      "NoncentralStudentTDistribution",
      "NonCommutativeMultiply",
      "NonConstants",
      "NondimensionalizationTransform",
      "None",
      "NoneTrue",
      "NonlinearModelFit",
      "NonlinearStateSpaceModel",
      "NonlocalMeansFilter",
      "NonNegative",
      "NonNegativeIntegers",
      "NonNegativeRationals",
      "NonNegativeReals",
      "NonPositive",
      "NonPositiveIntegers",
      "NonPositiveRationals",
      "NonPositiveReals",
      "Nor",
      "NorlundB",
      "Norm",
      "Normal",
      "NormalDistribution",
      "NormalGrouping",
      "NormalizationLayer",
      "Normalize",
      "Normalized",
      "NormalizedSquaredEuclideanDistance",
      "NormalMatrixQ",
      "NormalsFunction",
      "NormFunction",
      "Not",
      "NotCongruent",
      "NotCupCap",
      "NotDoubleVerticalBar",
      "Notebook",
      "NotebookApply",
      "NotebookAutoSave",
      "NotebookBrowseDirectory",
      "NotebookClose",
      "NotebookConvertSettings",
      "NotebookCreate",
      "NotebookDefault",
      "NotebookDelete",
      "NotebookDirectory",
      "NotebookDynamicExpression",
      "NotebookEvaluate",
      "NotebookEventActions",
      "NotebookFileName",
      "NotebookFind",
      "NotebookGet",
      "NotebookImport",
      "NotebookInformation",
      "NotebookInterfaceObject",
      "NotebookLocate",
      "NotebookObject",
      "NotebookOpen",
      "NotebookPath",
      "NotebookPrint",
      "NotebookPut",
      "NotebookRead",
      "Notebooks",
      "NotebookSave",
      "NotebookSelection",
      "NotebooksMenu",
      "NotebookTemplate",
      "NotebookWrite",
      "NotElement",
      "NotEqualTilde",
      "NotExists",
      "NotGreater",
      "NotGreaterEqual",
      "NotGreaterFullEqual",
      "NotGreaterGreater",
      "NotGreaterLess",
      "NotGreaterSlantEqual",
      "NotGreaterTilde",
      "Nothing",
      "NotHumpDownHump",
      "NotHumpEqual",
      "NotificationFunction",
      "NotLeftTriangle",
      "NotLeftTriangleBar",
      "NotLeftTriangleEqual",
      "NotLess",
      "NotLessEqual",
      "NotLessFullEqual",
      "NotLessGreater",
      "NotLessLess",
      "NotLessSlantEqual",
      "NotLessTilde",
      "NotNestedGreaterGreater",
      "NotNestedLessLess",
      "NotPrecedes",
      "NotPrecedesEqual",
      "NotPrecedesSlantEqual",
      "NotPrecedesTilde",
      "NotReverseElement",
      "NotRightTriangle",
      "NotRightTriangleBar",
      "NotRightTriangleEqual",
      "NotSquareSubset",
      "NotSquareSubsetEqual",
      "NotSquareSuperset",
      "NotSquareSupersetEqual",
      "NotSubset",
      "NotSubsetEqual",
      "NotSucceeds",
      "NotSucceedsEqual",
      "NotSucceedsSlantEqual",
      "NotSucceedsTilde",
      "NotSuperset",
      "NotSupersetEqual",
      "NotTilde",
      "NotTildeEqual",
      "NotTildeFullEqual",
      "NotTildeTilde",
      "NotVerticalBar",
      "Now",
      "NoWhitespace",
      "NProbability",
      "NProduct",
      "NProductFactors",
      "NRoots",
      "NSolve",
      "NSolveValues",
      "NSum",
      "NSumTerms",
      "NuclearExplosionData",
      "NuclearReactorData",
      "Null",
      "NullRecords",
      "NullSpace",
      "NullWords",
      "Number",
      "NumberCompose",
      "NumberDecompose",
      "NumberDigit",
      "NumberExpand",
      "NumberFieldClassNumber",
      "NumberFieldDiscriminant",
      "NumberFieldFundamentalUnits",
      "NumberFieldIntegralBasis",
      "NumberFieldNormRepresentatives",
      "NumberFieldRegulator",
      "NumberFieldRootsOfUnity",
      "NumberFieldSignature",
      "NumberForm",
      "NumberFormat",
      "NumberLinePlot",
      "NumberMarks",
      "NumberMultiplier",
      "NumberPadding",
      "NumberPoint",
      "NumberQ",
      "NumberSeparator",
      "NumberSigns",
      "NumberString",
      "Numerator",
      "NumeratorDenominator",
      "NumericalOrder",
      "NumericalSort",
      "NumericArray",
      "NumericArrayQ",
      "NumericArrayType",
      "NumericFunction",
      "NumericQ",
      "NuttallWindow",
      "NValues",
      "NyquistGridLines",
      "NyquistPlot",
      "O",
      "ObjectExistsQ",
      "ObservabilityGramian",
      "ObservabilityMatrix",
      "ObservableDecomposition",
      "ObservableModelQ",
      "OceanData",
      "Octahedron",
      "OddQ",
      "Off",
      "Offset",
      "OLEData",
      "On",
      "ONanGroupON",
      "Once",
      "OneIdentity",
      "Opacity",
      "OpacityFunction",
      "OpacityFunctionScaling",
      "Open",
      "OpenAppend",
      "Opener",
      "OpenerBox",
      "OpenerBoxOptions",
      "OpenerView",
      "OpenFunctionInspectorPacket",
      "Opening",
      "OpenRead",
      "OpenSpecialOptions",
      "OpenTemporary",
      "OpenWrite",
      "Operate",
      "OperatingSystem",
      "OperatorApplied",
      "OptimumFlowData",
      "Optional",
      "OptionalElement",
      "OptionInspectorSettings",
      "OptionQ",
      "Options",
      "OptionsPacket",
      "OptionsPattern",
      "OptionValue",
      "OptionValueBox",
      "OptionValueBoxOptions",
      "Or",
      "Orange",
      "Order",
      "OrderDistribution",
      "OrderedQ",
      "Ordering",
      "OrderingBy",
      "OrderingLayer",
      "Orderless",
      "OrderlessPatternSequence",
      "OrdinalScale",
      "OrnsteinUhlenbeckProcess",
      "Orthogonalize",
      "OrthogonalMatrixQ",
      "Out",
      "Outer",
      "OuterPolygon",
      "OuterPolyhedron",
      "OutputAutoOverwrite",
      "OutputControllabilityMatrix",
      "OutputControllableModelQ",
      "OutputForm",
      "OutputFormData",
      "OutputGrouping",
      "OutputMathEditExpression",
      "OutputNamePacket",
      "OutputPorts",
      "OutputResponse",
      "OutputSizeLimit",
      "OutputStream",
      "Over",
      "OverBar",
      "OverDot",
      "Overflow",
      "OverHat",
      "Overlaps",
      "Overlay",
      "OverlayBox",
      "OverlayBoxOptions",
      "OverlayVideo",
      "Overscript",
      "OverscriptBox",
      "OverscriptBoxOptions",
      "OverTilde",
      "OverVector",
      "OverwriteTarget",
      "OwenT",
      "OwnValues",
      "Package",
      "PackingMethod",
      "PackPaclet",
      "PacletDataRebuild",
      "PacletDirectoryAdd",
      "PacletDirectoryLoad",
      "PacletDirectoryRemove",
      "PacletDirectoryUnload",
      "PacletDisable",
      "PacletEnable",
      "PacletFind",
      "PacletFindRemote",
      "PacletInformation",
      "PacletInstall",
      "PacletInstallSubmit",
      "PacletNewerQ",
      "PacletObject",
      "PacletObjectQ",
      "PacletSite",
      "PacletSiteObject",
      "PacletSiteRegister",
      "PacletSites",
      "PacletSiteUnregister",
      "PacletSiteUpdate",
      "PacletSymbol",
      "PacletUninstall",
      "PacletUpdate",
      "PaddedForm",
      "Padding",
      "PaddingLayer",
      "PaddingSize",
      "PadeApproximant",
      "PadLeft",
      "PadRight",
      "PageBreakAbove",
      "PageBreakBelow",
      "PageBreakWithin",
      "PageFooterLines",
      "PageFooters",
      "PageHeaderLines",
      "PageHeaders",
      "PageHeight",
      "PageRankCentrality",
      "PageTheme",
      "PageWidth",
      "Pagination",
      "PairCorrelationG",
      "PairedBarChart",
      "PairedHistogram",
      "PairedSmoothHistogram",
      "PairedTTest",
      "PairedZTest",
      "PaletteNotebook",
      "PalettePath",
      "PalettesMenuSettings",
      "PalindromeQ",
      "Pane",
      "PaneBox",
      "PaneBoxOptions",
      "Panel",
      "PanelBox",
      "PanelBoxOptions",
      "Paneled",
      "PaneSelector",
      "PaneSelectorBox",
      "PaneSelectorBoxOptions",
      "PaperWidth",
      "ParabolicCylinderD",
      "ParagraphIndent",
      "ParagraphSpacing",
      "ParallelArray",
      "ParallelAxisPlot",
      "ParallelCombine",
      "ParallelDo",
      "Parallelepiped",
      "ParallelEvaluate",
      "Parallelization",
      "Parallelize",
      "ParallelKernels",
      "ParallelMap",
      "ParallelNeeds",
      "Parallelogram",
      "ParallelProduct",
      "ParallelSubmit",
      "ParallelSum",
      "ParallelTable",
      "ParallelTry",
      "Parameter",
      "ParameterEstimator",
      "ParameterMixtureDistribution",
      "ParameterVariables",
      "ParametricConvexOptimization",
      "ParametricFunction",
      "ParametricNDSolve",
      "ParametricNDSolveValue",
      "ParametricPlot",
      "ParametricPlot3D",
      "ParametricRampLayer",
      "ParametricRegion",
      "ParentBox",
      "ParentCell",
      "ParentConnect",
      "ParentDirectory",
      "ParentEdgeLabel",
      "ParentEdgeLabelFunction",
      "ParentEdgeLabelStyle",
      "ParentEdgeShapeFunction",
      "ParentEdgeStyle",
      "ParentEdgeStyleFunction",
      "ParentForm",
      "Parenthesize",
      "ParentList",
      "ParentNotebook",
      "ParetoDistribution",
      "ParetoPickandsDistribution",
      "ParkData",
      "Part",
      "PartBehavior",
      "PartialCorrelationFunction",
      "PartialD",
      "ParticleAcceleratorData",
      "ParticleData",
      "Partition",
      "PartitionGranularity",
      "PartitionsP",
      "PartitionsQ",
      "PartLayer",
      "PartOfSpeech",
      "PartProtection",
      "ParzenWindow",
      "PascalDistribution",
      "PassEventsDown",
      "PassEventsUp",
      "Paste",
      "PasteAutoQuoteCharacters",
      "PasteBoxFormInlineCells",
      "PasteButton",
      "Path",
      "PathGraph",
      "PathGraphQ",
      "Pattern",
      "PatternFilling",
      "PatternReaction",
      "PatternSequence",
      "PatternTest",
      "PauliMatrix",
      "PaulWavelet",
      "Pause",
      "PausedTime",
      "PDF",
      "PeakDetect",
      "PeanoCurve",
      "PearsonChiSquareTest",
      "PearsonCorrelationTest",
      "PearsonDistribution",
      "PenttinenPointProcess",
      "PercentForm",
      "PerfectNumber",
      "PerfectNumberQ",
      "PerformanceGoal",
      "Perimeter",
      "PeriodicBoundaryCondition",
      "PeriodicInterpolation",
      "Periodogram",
      "PeriodogramArray",
      "Permanent",
      "Permissions",
      "PermissionsGroup",
      "PermissionsGroupMemberQ",
      "PermissionsGroups",
      "PermissionsKey",
      "PermissionsKeys",
      "PermutationCycles",
      "PermutationCyclesQ",
      "PermutationGroup",
      "PermutationLength",
      "PermutationList",
      "PermutationListQ",
      "PermutationMatrix",
      "PermutationMax",
      "PermutationMin",
      "PermutationOrder",
      "PermutationPower",
      "PermutationProduct",
      "PermutationReplace",
      "Permutations",
      "PermutationSupport",
      "Permute",
      "PeronaMalikFilter",
      "Perpendicular",
      "PerpendicularBisector",
      "PersistenceLocation",
      "PersistenceTime",
      "PersistentObject",
      "PersistentObjects",
      "PersistentSymbol",
      "PersistentValue",
      "PersonData",
      "PERTDistribution",
      "PetersenGraph",
      "PhaseMargins",
      "PhaseRange",
      "PhongShading",
      "PhysicalSystemData",
      "Pi",
      "Pick",
      "PickedElements",
      "PickMode",
      "PIDData",
      "PIDDerivativeFilter",
      "PIDFeedforward",
      "PIDTune",
      "Piecewise",
      "PiecewiseExpand",
      "PieChart",
      "PieChart3D",
      "PillaiTrace",
      "PillaiTraceTest",
      "PingTime",
      "Pink",
      "PitchRecognize",
      "Pivoting",
      "PixelConstrained",
      "PixelValue",
      "PixelValuePositions",
      "Placed",
      "Placeholder",
      "PlaceholderLayer",
      "PlaceholderReplace",
      "Plain",
      "PlanarAngle",
      "PlanarFaceList",
      "PlanarGraph",
      "PlanarGraphQ",
      "PlanckRadiationLaw",
      "PlaneCurveData",
      "PlanetaryMoonData",
      "PlanetData",
      "PlantData",
      "Play",
      "PlaybackSettings",
      "PlayRange",
      "Plot",
      "Plot3D",
      "Plot3Matrix",
      "PlotDivision",
      "PlotJoined",
      "PlotLabel",
      "PlotLabels",
      "PlotLayout",
      "PlotLegends",
      "PlotMarkers",
      "PlotPoints",
      "PlotRange",
      "PlotRangeClipping",
      "PlotRangeClipPlanesStyle",
      "PlotRangePadding",
      "PlotRegion",
      "PlotStyle",
      "PlotTheme",
      "Pluralize",
      "Plus",
      "PlusMinus",
      "Pochhammer",
      "PodStates",
      "PodWidth",
      "Point",
      "Point3DBox",
      "Point3DBoxOptions",
      "PointBox",
      "PointBoxOptions",
      "PointCountDistribution",
      "PointDensity",
      "PointDensityFunction",
      "PointFigureChart",
      "PointLegend",
      "PointLight",
      "PointProcessEstimator",
      "PointProcessFitTest",
      "PointProcessParameterAssumptions",
      "PointProcessParameterQ",
      "PointSize",
      "PointStatisticFunction",
      "PointValuePlot",
      "PoissonConsulDistribution",
      "PoissonDistribution",
      "PoissonPDEComponent",
      "PoissonPointProcess",
      "PoissonProcess",
      "PoissonWindow",
      "PolarAxes",
      "PolarAxesOrigin",
      "PolarGridLines",
      "PolarPlot",
      "PolarTicks",
      "PoleZeroMarkers",
      "PolyaAeppliDistribution",
      "PolyGamma",
      "Polygon",
      "Polygon3DBox",
      "Polygon3DBoxOptions",
      "PolygonalNumber",
      "PolygonAngle",
      "PolygonBox",
      "PolygonBoxOptions",
      "PolygonCoordinates",
      "PolygonDecomposition",
      "PolygonHoleScale",
      "PolygonIntersections",
      "PolygonScale",
      "Polyhedron",
      "PolyhedronAngle",
      "PolyhedronBox",
      "PolyhedronBoxOptions",
      "PolyhedronCoordinates",
      "PolyhedronData",
      "PolyhedronDecomposition",
      "PolyhedronGenus",
      "PolyLog",
      "PolynomialExpressionQ",
      "PolynomialExtendedGCD",
      "PolynomialForm",
      "PolynomialGCD",
      "PolynomialLCM",
      "PolynomialMod",
      "PolynomialQ",
      "PolynomialQuotient",
      "PolynomialQuotientRemainder",
      "PolynomialReduce",
      "PolynomialRemainder",
      "Polynomials",
      "PolynomialSumOfSquaresList",
      "PoolingLayer",
      "PopupMenu",
      "PopupMenuBox",
      "PopupMenuBoxOptions",
      "PopupView",
      "PopupWindow",
      "Position",
      "PositionIndex",
      "PositionLargest",
      "PositionSmallest",
      "Positive",
      "PositiveDefiniteMatrixQ",
      "PositiveIntegers",
      "PositivelyOrientedPoints",
      "PositiveRationals",
      "PositiveReals",
      "PositiveSemidefiniteMatrixQ",
      "PossibleZeroQ",
      "Postfix",
      "PostScript",
      "Power",
      "PowerDistribution",
      "PowerExpand",
      "PowerMod",
      "PowerModList",
      "PowerRange",
      "PowerSpectralDensity",
      "PowersRepresentations",
      "PowerSymmetricPolynomial",
      "Precedence",
      "PrecedenceForm",
      "Precedes",
      "PrecedesEqual",
      "PrecedesSlantEqual",
      "PrecedesTilde",
      "Precision",
      "PrecisionGoal",
      "PreDecrement",
      "Predict",
      "PredictionRoot",
      "PredictorFunction",
      "PredictorInformation",
      "PredictorMeasurements",
      "PredictorMeasurementsObject",
      "PreemptProtect",
      "PreferencesPath",
      "PreferencesSettings",
      "Prefix",
      "PreIncrement",
      "Prepend",
      "PrependLayer",
      "PrependTo",
      "PreprocessingRules",
      "PreserveColor",
      "PreserveImageOptions",
      "Previous",
      "PreviousCell",
      "PreviousDate",
      "PriceGraphDistribution",
      "PrimaryPlaceholder",
      "Prime",
      "PrimeNu",
      "PrimeOmega",
      "PrimePi",
      "PrimePowerQ",
      "PrimeQ",
      "Primes",
      "PrimeZetaP",
      "PrimitivePolynomialQ",
      "PrimitiveRoot",
      "PrimitiveRootList",
      "PrincipalComponents",
      "PrincipalValue",
      "Print",
      "PrintableASCIIQ",
      "PrintAction",
      "PrintForm",
      "PrintingCopies",
      "PrintingOptions",
      "PrintingPageRange",
      "PrintingStartingPageNumber",
      "PrintingStyleEnvironment",
      "Printout3D",
      "Printout3DPreviewer",
      "PrintPrecision",
      "PrintTemporary",
      "Prism",
      "PrismBox",
      "PrismBoxOptions",
      "PrivateCellOptions",
      "PrivateEvaluationOptions",
      "PrivateFontOptions",
      "PrivateFrontEndOptions",
      "PrivateKey",
      "PrivateNotebookOptions",
      "PrivatePaths",
      "Probability",
      "ProbabilityDistribution",
      "ProbabilityPlot",
      "ProbabilityPr",
      "ProbabilityScalePlot",
      "ProbitModelFit",
      "ProcessConnection",
      "ProcessDirectory",
      "ProcessEnvironment",
      "Processes",
      "ProcessEstimator",
      "ProcessInformation",
      "ProcessObject",
      "ProcessParameterAssumptions",
      "ProcessParameterQ",
      "ProcessStateDomain",
      "ProcessStatus",
      "ProcessTimeDomain",
      "Product",
      "ProductDistribution",
      "ProductLog",
      "ProgressIndicator",
      "ProgressIndicatorBox",
      "ProgressIndicatorBoxOptions",
      "ProgressReporting",
      "Projection",
      "Prolog",
      "PromptForm",
      "ProofObject",
      "PropagateAborts",
      "Properties",
      "Property",
      "PropertyList",
      "PropertyValue",
      "Proportion",
      "Proportional",
      "Protect",
      "Protected",
      "ProteinData",
      "Pruning",
      "PseudoInverse",
      "PsychrometricPropertyData",
      "PublicKey",
      "PublisherID",
      "PulsarData",
      "PunctuationCharacter",
      "Purple",
      "Put",
      "PutAppend",
      "Pyramid",
      "PyramidBox",
      "PyramidBoxOptions",
      "QBinomial",
      "QFactorial",
      "QGamma",
      "QHypergeometricPFQ",
      "QnDispersion",
      "QPochhammer",
      "QPolyGamma",
      "QRDecomposition",
      "QuadraticIrrationalQ",
      "QuadraticOptimization",
      "Quantile",
      "QuantilePlot",
      "Quantity",
      "QuantityArray",
      "QuantityDistribution",
      "QuantityForm",
      "QuantityMagnitude",
      "QuantityQ",
      "QuantityUnit",
      "QuantityVariable",
      "QuantityVariableCanonicalUnit",
      "QuantityVariableDimensions",
      "QuantityVariableIdentifier",
      "QuantityVariablePhysicalQuantity",
      "Quartics",
      "QuartileDeviation",
      "Quartiles",
      "QuartileSkewness",
      "Query",
      "QuestionGenerator",
      "QuestionInterface",
      "QuestionObject",
      "QuestionSelector",
      "QueueingNetworkProcess",
      "QueueingProcess",
      "QueueProperties",
      "Quiet",
      "QuietEcho",
      "Quit",
      "Quotient",
      "QuotientRemainder",
      "RadialAxisPlot",
      "RadialGradientFilling",
      "RadialGradientImage",
      "RadialityCentrality",
      "RadicalBox",
      "RadicalBoxOptions",
      "RadioButton",
      "RadioButtonBar",
      "RadioButtonBox",
      "RadioButtonBoxOptions",
      "Radon",
      "RadonTransform",
      "RamanujanTau",
      "RamanujanTauL",
      "RamanujanTauTheta",
      "RamanujanTauZ",
      "Ramp",
      "Random",
      "RandomArrayLayer",
      "RandomChoice",
      "RandomColor",
      "RandomComplex",
      "RandomDate",
      "RandomEntity",
      "RandomFunction",
      "RandomGeneratorState",
      "RandomGeoPosition",
      "RandomGraph",
      "RandomImage",
      "RandomInstance",
      "RandomInteger",
      "RandomPermutation",
      "RandomPoint",
      "RandomPointConfiguration",
      "RandomPolygon",
      "RandomPolyhedron",
      "RandomPrime",
      "RandomReal",
      "RandomSample",
      "RandomSeed",
      "RandomSeeding",
      "RandomTime",
      "RandomTree",
      "RandomVariate",
      "RandomWalkProcess",
      "RandomWord",
      "Range",
      "RangeFilter",
      "RangeSpecification",
      "RankedMax",
      "RankedMin",
      "RarerProbability",
      "Raster",
      "Raster3D",
      "Raster3DBox",
      "Raster3DBoxOptions",
      "RasterArray",
      "RasterBox",
      "RasterBoxOptions",
      "Rasterize",
      "RasterSize",
      "Rational",
      "RationalExpressionQ",
      "RationalFunctions",
      "Rationalize",
      "Rationals",
      "Ratios",
      "RawArray",
      "RawBoxes",
      "RawData",
      "RawMedium",
      "RayleighDistribution",
      "Re",
      "ReactionBalance",
      "ReactionBalancedQ",
      "ReactionPDETerm",
      "Read",
      "ReadByteArray",
      "ReadLine",
      "ReadList",
      "ReadProtected",
      "ReadString",
      "Real",
      "RealAbs",
      "RealBlockDiagonalForm",
      "RealDigits",
      "RealExponent",
      "Reals",
      "RealSign",
      "Reap",
      "RebuildPacletData",
      "RecalibrationFunction",
      "RecognitionPrior",
      "RecognitionThreshold",
      "ReconstructionMesh",
      "Record",
      "RecordLists",
      "RecordSeparators",
      "Rectangle",
      "RectangleBox",
      "RectangleBoxOptions",
      "RectangleChart",
      "RectangleChart3D",
      "RectangularRepeatingElement",
      "RecurrenceFilter",
      "RecurrenceTable",
      "RecurringDigitsForm",
      "Red",
      "Reduce",
      "RefBox",
      "ReferenceLineStyle",
      "ReferenceMarkers",
      "ReferenceMarkerStyle",
      "Refine",
      "ReflectionMatrix",
      "ReflectionTransform",
      "Refresh",
      "RefreshRate",
      "Region",
      "RegionBinarize",
      "RegionBoundary",
      "RegionBoundaryStyle",
      "RegionBounds",
      "RegionCentroid",
      "RegionCongruent",
      "RegionConvert",
      "RegionDifference",
      "RegionDilation",
      "RegionDimension",
      "RegionDisjoint",
      "RegionDistance",
      "RegionDistanceFunction",
      "RegionEmbeddingDimension",
      "RegionEqual",
      "RegionErosion",
      "RegionFillingStyle",
      "RegionFit",
      "RegionFunction",
      "RegionImage",
      "RegionIntersection",
      "RegionMeasure",
      "RegionMember",
      "RegionMemberFunction",
      "RegionMoment",
      "RegionNearest",
      "RegionNearestFunction",
      "RegionPlot",
      "RegionPlot3D",
      "RegionProduct",
      "RegionQ",
      "RegionResize",
      "RegionSimilar",
      "RegionSize",
      "RegionSymmetricDifference",
      "RegionUnion",
      "RegionWithin",
      "RegisterExternalEvaluator",
      "RegularExpression",
      "Regularization",
      "RegularlySampledQ",
      "RegularPolygon",
      "ReIm",
      "ReImLabels",
      "ReImPlot",
      "ReImStyle",
      "Reinstall",
      "RelationalDatabase",
      "RelationGraph",
      "Release",
      "ReleaseHold",
      "ReliabilityDistribution",
      "ReliefImage",
      "ReliefPlot",
      "RemoteAuthorizationCaching",
      "RemoteBatchJobAbort",
      "RemoteBatchJobObject",
      "RemoteBatchJobs",
      "RemoteBatchMapSubmit",
      "RemoteBatchSubmissionEnvironment",
      "RemoteBatchSubmit",
      "RemoteConnect",
      "RemoteConnectionObject",
      "RemoteEvaluate",
      "RemoteFile",
      "RemoteInputFiles",
      "RemoteKernelObject",
      "RemoteProviderSettings",
      "RemoteRun",
      "RemoteRunProcess",
      "RemovalConditions",
      "Remove",
      "RemoveAlphaChannel",
      "RemoveAsynchronousTask",
      "RemoveAudioStream",
      "RemoveBackground",
      "RemoveChannelListener",
      "RemoveChannelSubscribers",
      "Removed",
      "RemoveDiacritics",
      "RemoveInputStreamMethod",
      "RemoveOutputStreamMethod",
      "RemoveProperty",
      "RemoveScheduledTask",
      "RemoveUsers",
      "RemoveVideoStream",
      "RenameDirectory",
      "RenameFile",
      "RenderAll",
      "RenderingOptions",
      "RenewalProcess",
      "RenkoChart",
      "RepairMesh",
      "Repeated",
      "RepeatedNull",
      "RepeatedString",
      "RepeatedTiming",
      "RepeatingElement",
      "Replace",
      "ReplaceAll",
      "ReplaceAt",
      "ReplaceHeldPart",
      "ReplaceImageValue",
      "ReplaceList",
      "ReplacePart",
      "ReplacePixelValue",
      "ReplaceRepeated",
      "ReplicateLayer",
      "RequiredPhysicalQuantities",
      "Resampling",
      "ResamplingAlgorithmData",
      "ResamplingMethod",
      "Rescale",
      "RescalingTransform",
      "ResetDirectory",
      "ResetScheduledTask",
      "ReshapeLayer",
      "Residue",
      "ResidueSum",
      "ResizeLayer",
      "Resolve",
      "ResolveContextAliases",
      "ResourceAcquire",
      "ResourceData",
      "ResourceFunction",
      "ResourceObject",
      "ResourceRegister",
      "ResourceRemove",
      "ResourceSearch",
      "ResourceSubmissionObject",
      "ResourceSubmit",
      "ResourceSystemBase",
      "ResourceSystemPath",
      "ResourceUpdate",
      "ResourceVersion",
      "ResponseForm",
      "Rest",
      "RestartInterval",
      "Restricted",
      "Resultant",
      "ResumePacket",
      "Return",
      "ReturnCreatesNewCell",
      "ReturnEntersInput",
      "ReturnExpressionPacket",
      "ReturnInputFormPacket",
      "ReturnPacket",
      "ReturnReceiptFunction",
      "ReturnTextPacket",
      "Reverse",
      "ReverseApplied",
      "ReverseBiorthogonalSplineWavelet",
      "ReverseElement",
      "ReverseEquilibrium",
      "ReverseGraph",
      "ReverseSort",
      "ReverseSortBy",
      "ReverseUpEquilibrium",
      "RevolutionAxis",
      "RevolutionPlot3D",
      "RGBColor",
      "RiccatiSolve",
      "RiceDistribution",
      "RidgeFilter",
      "RiemannR",
      "RiemannSiegelTheta",
      "RiemannSiegelZ",
      "RiemannXi",
      "Riffle",
      "Right",
      "RightArrow",
      "RightArrowBar",
      "RightArrowLeftArrow",
      "RightComposition",
      "RightCosetRepresentative",
      "RightDownTeeVector",
      "RightDownVector",
      "RightDownVectorBar",
      "RightTee",
      "RightTeeArrow",
      "RightTeeVector",
      "RightTriangle",
      "RightTriangleBar",
      "RightTriangleEqual",
      "RightUpDownVector",
      "RightUpTeeVector",
      "RightUpVector",
      "RightUpVectorBar",
      "RightVector",
      "RightVectorBar",
      "RipleyK",
      "RipleyRassonRegion",
      "RiskAchievementImportance",
      "RiskReductionImportance",
      "RobustConvexOptimization",
      "RogersTanimotoDissimilarity",
      "RollPitchYawAngles",
      "RollPitchYawMatrix",
      "RomanNumeral",
      "Root",
      "RootApproximant",
      "RootIntervals",
      "RootLocusPlot",
      "RootMeanSquare",
      "RootOfUnityQ",
      "RootReduce",
      "Roots",
      "RootSum",
      "RootTree",
      "Rotate",
      "RotateLabel",
      "RotateLeft",
      "RotateRight",
      "RotationAction",
      "RotationBox",
      "RotationBoxOptions",
      "RotationMatrix",
      "RotationTransform",
      "Round",
      "RoundImplies",
      "RoundingRadius",
      "Row",
      "RowAlignments",
      "RowBackgrounds",
      "RowBox",
      "RowHeights",
      "RowLines",
      "RowMinHeight",
      "RowReduce",
      "RowsEqual",
      "RowSpacings",
      "RSolve",
      "RSolveValue",
      "RudinShapiro",
      "RudvalisGroupRu",
      "Rule",
      "RuleCondition",
      "RuleDelayed",
      "RuleForm",
      "RulePlot",
      "RulerUnits",
      "RulesTree",
      "Run",
      "RunProcess",
      "RunScheduledTask",
      "RunThrough",
      "RuntimeAttributes",
      "RuntimeOptions",
      "RussellRaoDissimilarity",
      "SameAs",
      "SameQ",
      "SameTest",
      "SameTestProperties",
      "SampledEntityClass",
      "SampleDepth",
      "SampledSoundFunction",
      "SampledSoundList",
      "SampleRate",
      "SamplingPeriod",
      "SARIMAProcess",
      "SARMAProcess",
      "SASTriangle",
      "SatelliteData",
      "SatisfiabilityCount",
      "SatisfiabilityInstances",
      "SatisfiableQ",
      "Saturday",
      "Save",
      "Saveable",
      "SaveAutoDelete",
      "SaveConnection",
      "SaveDefinitions",
      "SavitzkyGolayMatrix",
      "SawtoothWave",
      "Scale",
      "Scaled",
      "ScaleDivisions",
      "ScaledMousePosition",
      "ScaleOrigin",
      "ScalePadding",
      "ScaleRanges",
      "ScaleRangeStyle",
      "ScalingFunctions",
      "ScalingMatrix",
      "ScalingTransform",
      "Scan",
      "ScheduledTask",
      "ScheduledTaskActiveQ",
      "ScheduledTaskInformation",
      "ScheduledTaskInformationData",
      "ScheduledTaskObject",
      "ScheduledTasks",
      "SchurDecomposition",
      "ScientificForm",
      "ScientificNotationThreshold",
      "ScorerGi",
      "ScorerGiPrime",
      "ScorerHi",
      "ScorerHiPrime",
      "ScreenRectangle",
      "ScreenStyleEnvironment",
      "ScriptBaselineShifts",
      "ScriptForm",
      "ScriptLevel",
      "ScriptMinSize",
      "ScriptRules",
      "ScriptSizeMultipliers",
      "Scrollbars",
      "ScrollingOptions",
      "ScrollPosition",
      "SearchAdjustment",
      "SearchIndexObject",
      "SearchIndices",
      "SearchQueryString",
      "SearchResultObject",
      "Sec",
      "Sech",
      "SechDistribution",
      "SecondOrderConeOptimization",
      "SectionGrouping",
      "SectorChart",
      "SectorChart3D",
      "SectorOrigin",
      "SectorSpacing",
      "SecuredAuthenticationKey",
      "SecuredAuthenticationKeys",
      "SecurityCertificate",
      "SeedRandom",
      "Select",
      "Selectable",
      "SelectComponents",
      "SelectedCells",
      "SelectedNotebook",
      "SelectFirst",
      "Selection",
      "SelectionAnimate",
      "SelectionCell",
      "SelectionCellCreateCell",
      "SelectionCellDefaultStyle",
      "SelectionCellParentStyle",
      "SelectionCreateCell",
      "SelectionDebuggerTag",
      "SelectionEvaluate",
      "SelectionEvaluateCreateCell",
      "SelectionMove",
      "SelectionPlaceholder",
      "SelectWithContents",
      "SelfLoops",
      "SelfLoopStyle",
      "SemanticImport",
      "SemanticImportString",
      "SemanticInterpretation",
      "SemialgebraicComponentInstances",
      "SemidefiniteOptimization",
      "SendMail",
      "SendMessage",
      "Sequence",
      "SequenceAlignment",
      "SequenceAttentionLayer",
      "SequenceCases",
      "SequenceCount",
      "SequenceFold",
      "SequenceFoldList",
      "SequenceForm",
      "SequenceHold",
      "SequenceIndicesLayer",
      "SequenceLastLayer",
      "SequenceMostLayer",
      "SequencePosition",
      "SequencePredict",
      "SequencePredictorFunction",
      "SequenceReplace",
      "SequenceRestLayer",
      "SequenceReverseLayer",
      "SequenceSplit",
      "Series",
      "SeriesCoefficient",
      "SeriesData",
      "SeriesTermGoal",
      "ServiceConnect",
      "ServiceDisconnect",
      "ServiceExecute",
      "ServiceObject",
      "ServiceRequest",
      "ServiceResponse",
      "ServiceSubmit",
      "SessionSubmit",
      "SessionTime",
      "Set",
      "SetAccuracy",
      "SetAlphaChannel",
      "SetAttributes",
      "Setbacks",
      "SetCloudDirectory",
      "SetCookies",
      "SetDelayed",
      "SetDirectory",
      "SetEnvironment",
      "SetFileDate",
      "SetFileFormatProperties",
      "SetOptions",
      "SetOptionsPacket",
      "SetPermissions",
      "SetPrecision",
      "SetProperty",
      "SetSecuredAuthenticationKey",
      "SetSelectedNotebook",
      "SetSharedFunction",
      "SetSharedVariable",
      "SetStreamPosition",
      "SetSystemModel",
      "SetSystemOptions",
      "Setter",
      "SetterBar",
      "SetterBox",
      "SetterBoxOptions",
      "Setting",
      "SetUsers",
      "Shading",
      "Shallow",
      "ShannonWavelet",
      "ShapiroWilkTest",
      "Share",
      "SharingList",
      "Sharpen",
      "ShearingMatrix",
      "ShearingTransform",
      "ShellRegion",
      "ShenCastanMatrix",
      "ShiftedGompertzDistribution",
      "ShiftRegisterSequence",
      "Short",
      "ShortDownArrow",
      "Shortest",
      "ShortestMatch",
      "ShortestPathFunction",
      "ShortLeftArrow",
      "ShortRightArrow",
      "ShortTimeFourier",
      "ShortTimeFourierData",
      "ShortUpArrow",
      "Show",
      "ShowAutoConvert",
      "ShowAutoSpellCheck",
      "ShowAutoStyles",
      "ShowCellBracket",
      "ShowCellLabel",
      "ShowCellTags",
      "ShowClosedCellArea",
      "ShowCodeAssist",
      "ShowContents",
      "ShowControls",
      "ShowCursorTracker",
      "ShowGroupOpenCloseIcon",
      "ShowGroupOpener",
      "ShowInvisibleCharacters",
      "ShowPageBreaks",
      "ShowPredictiveInterface",
      "ShowSelection",
      "ShowShortBoxForm",
      "ShowSpecialCharacters",
      "ShowStringCharacters",
      "ShowSyntaxStyles",
      "ShrinkingDelay",
      "ShrinkWrapBoundingBox",
      "SiderealTime",
      "SiegelTheta",
      "SiegelTukeyTest",
      "SierpinskiCurve",
      "SierpinskiMesh",
      "Sign",
      "Signature",
      "SignedRankTest",
      "SignedRegionDistance",
      "SignificanceLevel",
      "SignPadding",
      "SignTest",
      "SimilarityRules",
      "SimpleGraph",
      "SimpleGraphQ",
      "SimplePolygonQ",
      "SimplePolyhedronQ",
      "Simplex",
      "Simplify",
      "Sin",
      "Sinc",
      "SinghMaddalaDistribution",
      "SingleEvaluation",
      "SingleLetterItalics",
      "SingleLetterStyle",
      "SingularValueDecomposition",
      "SingularValueList",
      "SingularValuePlot",
      "SingularValues",
      "Sinh",
      "SinhIntegral",
      "SinIntegral",
      "SixJSymbol",
      "Skeleton",
      "SkeletonTransform",
      "SkellamDistribution",
      "Skewness",
      "SkewNormalDistribution",
      "SkinStyle",
      "Skip",
      "SliceContourPlot3D",
      "SliceDensityPlot3D",
      "SliceDistribution",
      "SliceVectorPlot3D",
      "Slider",
      "Slider2D",
      "Slider2DBox",
      "Slider2DBoxOptions",
      "SliderBox",
      "SliderBoxOptions",
      "SlideShowVideo",
      "SlideView",
      "Slot",
      "SlotSequence",
      "Small",
      "SmallCircle",
      "Smaller",
      "SmithDecomposition",
      "SmithDelayCompensator",
      "SmithWatermanSimilarity",
      "SmoothDensityHistogram",
      "SmoothHistogram",
      "SmoothHistogram3D",
      "SmoothKernelDistribution",
      "SmoothPointDensity",
      "SnDispersion",
      "Snippet",
      "SnippetsVideo",
      "SnubPolyhedron",
      "SocialMediaData",
      "Socket",
      "SocketConnect",
      "SocketListen",
      "SocketListener",
      "SocketObject",
      "SocketOpen",
      "SocketReadMessage",
      "SocketReadyQ",
      "Sockets",
      "SocketWaitAll",
      "SocketWaitNext",
      "SoftmaxLayer",
      "SokalSneathDissimilarity",
      "SolarEclipse",
      "SolarSystemFeatureData",
      "SolarTime",
      "SolidAngle",
      "SolidBoundaryLoadValue",
      "SolidData",
      "SolidDisplacementCondition",
      "SolidFixedCondition",
      "SolidMechanicsPDEComponent",
      "SolidMechanicsStrain",
      "SolidMechanicsStress",
      "SolidRegionQ",
      "Solve",
      "SolveAlways",
      "SolveDelayed",
      "SolveValues",
      "Sort",
      "SortBy",
      "SortedBy",
      "SortedEntityClass",
      "Sound",
      "SoundAndGraphics",
      "SoundNote",
      "SoundVolume",
      "SourceLink",
      "SourcePDETerm",
      "Sow",
      "Space",
      "SpaceCurveData",
      "SpaceForm",
      "Spacer",
      "Spacings",
      "Span",
      "SpanAdjustments",
      "SpanCharacterRounding",
      "SpanFromAbove",
      "SpanFromBoth",
      "SpanFromLeft",
      "SpanLineThickness",
      "SpanMaxSize",
      "SpanMinSize",
      "SpanningCharacters",
      "SpanSymmetric",
      "SparseArray",
      "SparseArrayQ",
      "SpatialBinnedPointData",
      "SpatialBoundaryCorrection",
      "SpatialEstimate",
      "SpatialEstimatorFunction",
      "SpatialGraphDistribution",
      "SpatialJ",
      "SpatialMedian",
      "SpatialNoiseLevel",
      "SpatialObservationRegionQ",
      "SpatialPointData",
      "SpatialPointSelect",
      "SpatialRandomnessTest",
      "SpatialTransformationLayer",
      "SpatialTrendFunction",
      "Speak",
      "SpeakerMatchQ",
      "SpearmanRankTest",
      "SpearmanRho",
      "SpeciesData",
      "SpecificityGoal",
      "SpectralLineData",
      "Spectrogram",
      "SpectrogramArray",
      "Specularity",
      "SpeechCases",
      "SpeechInterpreter",
      "SpeechRecognize",
      "SpeechSynthesize",
      "SpellingCorrection",
      "SpellingCorrectionList",
      "SpellingDictionaries",
      "SpellingDictionariesPath",
      "SpellingOptions",
      "Sphere",
      "SphereBox",
      "SphereBoxOptions",
      "SpherePoints",
      "SphericalBesselJ",
      "SphericalBesselY",
      "SphericalHankelH1",
      "SphericalHankelH2",
      "SphericalHarmonicY",
      "SphericalPlot3D",
      "SphericalRegion",
      "SphericalShell",
      "SpheroidalEigenvalue",
      "SpheroidalJoiningFactor",
      "SpheroidalPS",
      "SpheroidalPSPrime",
      "SpheroidalQS",
      "SpheroidalQSPrime",
      "SpheroidalRadialFactor",
      "SpheroidalS1",
      "SpheroidalS1Prime",
      "SpheroidalS2",
      "SpheroidalS2Prime",
      "Splice",
      "SplicedDistribution",
      "SplineClosed",
      "SplineDegree",
      "SplineKnots",
      "SplineWeights",
      "Split",
      "SplitBy",
      "SpokenString",
      "SpotLight",
      "Sqrt",
      "SqrtBox",
      "SqrtBoxOptions",
      "Square",
      "SquaredEuclideanDistance",
      "SquareFreeQ",
      "SquareIntersection",
      "SquareMatrixQ",
      "SquareRepeatingElement",
      "SquaresR",
      "SquareSubset",
      "SquareSubsetEqual",
      "SquareSuperset",
      "SquareSupersetEqual",
      "SquareUnion",
      "SquareWave",
      "SSSTriangle",
      "StabilityMargins",
      "StabilityMarginsStyle",
      "StableDistribution",
      "Stack",
      "StackBegin",
      "StackComplete",
      "StackedDateListPlot",
      "StackedListPlot",
      "StackInhibit",
      "StadiumShape",
      "StandardAtmosphereData",
      "StandardDeviation",
      "StandardDeviationFilter",
      "StandardForm",
      "Standardize",
      "Standardized",
      "StandardOceanData",
      "StandbyDistribution",
      "Star",
      "StarClusterData",
      "StarData",
      "StarGraph",
      "StartAsynchronousTask",
      "StartExternalSession",
      "StartingStepSize",
      "StartOfLine",
      "StartOfString",
      "StartProcess",
      "StartScheduledTask",
      "StartupSound",
      "StartWebSession",
      "StateDimensions",
      "StateFeedbackGains",
      "StateOutputEstimator",
      "StateResponse",
      "StateSpaceModel",
      "StateSpaceRealization",
      "StateSpaceTransform",
      "StateTransformationLinearize",
      "StationaryDistribution",
      "StationaryWaveletPacketTransform",
      "StationaryWaveletTransform",
      "StatusArea",
      "StatusCentrality",
      "StepMonitor",
      "StereochemistryElements",
      "StieltjesGamma",
      "StippleShading",
      "StirlingS1",
      "StirlingS2",
      "StopAsynchronousTask",
      "StoppingPowerData",
      "StopScheduledTask",
      "StrataVariables",
      "StratonovichProcess",
      "StraussHardcorePointProcess",
      "StraussPointProcess",
      "StreamColorFunction",
      "StreamColorFunctionScaling",
      "StreamDensityPlot",
      "StreamMarkers",
      "StreamPlot",
      "StreamPlot3D",
      "StreamPoints",
      "StreamPosition",
      "Streams",
      "StreamScale",
      "StreamStyle",
      "StrictInequalities",
      "String",
      "StringBreak",
      "StringByteCount",
      "StringCases",
      "StringContainsQ",
      "StringCount",
      "StringDelete",
      "StringDrop",
      "StringEndsQ",
      "StringExpression",
      "StringExtract",
      "StringForm",
      "StringFormat",
      "StringFormatQ",
      "StringFreeQ",
      "StringInsert",
      "StringJoin",
      "StringLength",
      "StringMatchQ",
      "StringPadLeft",
      "StringPadRight",
      "StringPart",
      "StringPartition",
      "StringPosition",
      "StringQ",
      "StringRepeat",
      "StringReplace",
      "StringReplaceList",
      "StringReplacePart",
      "StringReverse",
      "StringRiffle",
      "StringRotateLeft",
      "StringRotateRight",
      "StringSkeleton",
      "StringSplit",
      "StringStartsQ",
      "StringTake",
      "StringTakeDrop",
      "StringTemplate",
      "StringToByteArray",
      "StringToStream",
      "StringTrim",
      "StripBoxes",
      "StripOnInput",
      "StripStyleOnPaste",
      "StripWrapperBoxes",
      "StrokeForm",
      "Struckthrough",
      "StructuralImportance",
      "StructuredArray",
      "StructuredArrayHeadQ",
      "StructuredSelection",
      "StruveH",
      "StruveL",
      "Stub",
      "StudentTDistribution",
      "Style",
      "StyleBox",
      "StyleBoxAutoDelete",
      "StyleData",
      "StyleDefinitions",
      "StyleForm",
      "StyleHints",
      "StyleKeyMapping",
      "StyleMenuListing",
      "StyleNameDialogSettings",
      "StyleNames",
      "StylePrint",
      "StyleSheetPath",
      "Subdivide",
      "Subfactorial",
      "Subgraph",
      "SubMinus",
      "SubPlus",
      "SubresultantPolynomialRemainders",
      "SubresultantPolynomials",
      "Subresultants",
      "Subscript",
      "SubscriptBox",
      "SubscriptBoxOptions",
      "Subscripted",
      "Subsequences",
      "Subset",
      "SubsetCases",
      "SubsetCount",
      "SubsetEqual",
      "SubsetMap",
      "SubsetPosition",
      "SubsetQ",
      "SubsetReplace",
      "Subsets",
      "SubStar",
      "SubstitutionSystem",
      "Subsuperscript",
      "SubsuperscriptBox",
      "SubsuperscriptBoxOptions",
      "SubtitleEncoding",
      "SubtitleTrackSelection",
      "Subtract",
      "SubtractFrom",
      "SubtractSides",
      "SubValues",
      "Succeeds",
      "SucceedsEqual",
      "SucceedsSlantEqual",
      "SucceedsTilde",
      "Success",
      "SuchThat",
      "Sum",
      "SumConvergence",
      "SummationLayer",
      "Sunday",
      "SunPosition",
      "Sunrise",
      "Sunset",
      "SuperDagger",
      "SuperMinus",
      "SupernovaData",
      "SuperPlus",
      "Superscript",
      "SuperscriptBox",
      "SuperscriptBoxOptions",
      "Superset",
      "SupersetEqual",
      "SuperStar",
      "Surd",
      "SurdForm",
      "SurfaceAppearance",
      "SurfaceArea",
      "SurfaceColor",
      "SurfaceData",
      "SurfaceGraphics",
      "SurvivalDistribution",
      "SurvivalFunction",
      "SurvivalModel",
      "SurvivalModelFit",
      "SuspendPacket",
      "SuzukiDistribution",
      "SuzukiGroupSuz",
      "SwatchLegend",
      "Switch",
      "Symbol",
      "SymbolName",
      "SymletWavelet",
      "Symmetric",
      "SymmetricDifference",
      "SymmetricGroup",
      "SymmetricKey",
      "SymmetricMatrixQ",
      "SymmetricPolynomial",
      "SymmetricReduction",
      "Symmetrize",
      "SymmetrizedArray",
      "SymmetrizedArrayRules",
      "SymmetrizedDependentComponents",
      "SymmetrizedIndependentComponents",
      "SymmetrizedReplacePart",
      "SynchronousInitialization",
      "SynchronousUpdating",
      "Synonyms",
      "Syntax",
      "SyntaxForm",
      "SyntaxInformation",
      "SyntaxLength",
      "SyntaxPacket",
      "SyntaxQ",
      "SynthesizeMissingValues",
      "SystemCredential",
      "SystemCredentialData",
      "SystemCredentialKey",
      "SystemCredentialKeys",
      "SystemCredentialStoreObject",
      "SystemDialogInput",
      "SystemException",
      "SystemGet",
      "SystemHelpPath",
      "SystemInformation",
      "SystemInformationData",
      "SystemInstall",
      "SystemModel",
      "SystemModeler",
      "SystemModelExamples",
      "SystemModelLinearize",
      "SystemModelMeasurements",
      "SystemModelParametricSimulate",
      "SystemModelPlot",
      "SystemModelProgressReporting",
      "SystemModelReliability",
      "SystemModels",
      "SystemModelSimulate",
      "SystemModelSimulateSensitivity",
      "SystemModelSimulationData",
      "SystemOpen",
      "SystemOptions",
      "SystemProcessData",
      "SystemProcesses",
      "SystemsConnectionsModel",
      "SystemsModelControllerData",
      "SystemsModelDelay",
      "SystemsModelDelayApproximate",
      "SystemsModelDelete",
      "SystemsModelDimensions",
      "SystemsModelExtract",
      "SystemsModelFeedbackConnect",
      "SystemsModelLabels",
      "SystemsModelLinearity",
      "SystemsModelMerge",
      "SystemsModelOrder",
      "SystemsModelParallelConnect",
      "SystemsModelSeriesConnect",
      "SystemsModelStateFeedbackConnect",
      "SystemsModelVectorRelativeOrders",
      "SystemStub",
      "SystemTest",
      "Tab",
      "TabFilling",
      "Table",
      "TableAlignments",
      "TableDepth",
      "TableDirections",
      "TableForm",
      "TableHeadings",
      "TableSpacing",
      "TableView",
      "TableViewBox",
      "TableViewBoxAlignment",
      "TableViewBoxBackground",
      "TableViewBoxHeaders",
      "TableViewBoxItemSize",
      "TableViewBoxItemStyle",
      "TableViewBoxOptions",
      "TabSpacings",
      "TabView",
      "TabViewBox",
      "TabViewBoxOptions",
      "TagBox",
      "TagBoxNote",
      "TagBoxOptions",
      "TaggingRules",
      "TagSet",
      "TagSetDelayed",
      "TagStyle",
      "TagUnset",
      "Take",
      "TakeDrop",
      "TakeLargest",
      "TakeLargestBy",
      "TakeList",
      "TakeSmallest",
      "TakeSmallestBy",
      "TakeWhile",
      "Tally",
      "Tan",
      "Tanh",
      "TargetDevice",
      "TargetFunctions",
      "TargetSystem",
      "TargetUnits",
      "TaskAbort",
      "TaskExecute",
      "TaskObject",
      "TaskRemove",
      "TaskResume",
      "Tasks",
      "TaskSuspend",
      "TaskWait",
      "TautologyQ",
      "TelegraphProcess",
      "TemplateApply",
      "TemplateArgBox",
      "TemplateBox",
      "TemplateBoxOptions",
      "TemplateEvaluate",
      "TemplateExpression",
      "TemplateIf",
      "TemplateObject",
      "TemplateSequence",
      "TemplateSlot",
      "TemplateSlotSequence",
      "TemplateUnevaluated",
      "TemplateVerbatim",
      "TemplateWith",
      "TemporalData",
      "TemporalRegularity",
      "Temporary",
      "TemporaryVariable",
      "TensorContract",
      "TensorDimensions",
      "TensorExpand",
      "TensorProduct",
      "TensorQ",
      "TensorRank",
      "TensorReduce",
      "TensorSymmetry",
      "TensorTranspose",
      "TensorWedge",
      "TerminatedEvaluation",
      "TernaryListPlot",
      "TernaryPlotCorners",
      "TestID",
      "TestReport",
      "TestReportObject",
      "TestResultObject",
      "Tetrahedron",
      "TetrahedronBox",
      "TetrahedronBoxOptions",
      "TeXForm",
      "TeXSave",
      "Text",
      "Text3DBox",
      "Text3DBoxOptions",
      "TextAlignment",
      "TextBand",
      "TextBoundingBox",
      "TextBox",
      "TextCases",
      "TextCell",
      "TextClipboardType",
      "TextContents",
      "TextData",
      "TextElement",
      "TextForm",
      "TextGrid",
      "TextJustification",
      "TextLine",
      "TextPacket",
      "TextParagraph",
      "TextPosition",
      "TextRecognize",
      "TextSearch",
      "TextSearchReport",
      "TextSentences",
      "TextString",
      "TextStructure",
      "TextStyle",
      "TextTranslation",
      "Texture",
      "TextureCoordinateFunction",
      "TextureCoordinateScaling",
      "TextWords",
      "Therefore",
      "ThermodynamicData",
      "ThermometerGauge",
      "Thick",
      "Thickness",
      "Thin",
      "Thinning",
      "ThisLink",
      "ThomasPointProcess",
      "ThompsonGroupTh",
      "Thread",
      "Threaded",
      "ThreadingLayer",
      "ThreeJSymbol",
      "Threshold",
      "Through",
      "Throw",
      "ThueMorse",
      "Thumbnail",
      "Thursday",
      "TickDirection",
      "TickLabelOrientation",
      "TickLabelPositioning",
      "TickLabels",
      "TickLengths",
      "TickPositions",
      "Ticks",
      "TicksStyle",
      "TideData",
      "Tilde",
      "TildeEqual",
      "TildeFullEqual",
      "TildeTilde",
      "TimeConstrained",
      "TimeConstraint",
      "TimeDirection",
      "TimeFormat",
      "TimeGoal",
      "TimelinePlot",
      "TimeObject",
      "TimeObjectQ",
      "TimeRemaining",
      "Times",
      "TimesBy",
      "TimeSeries",
      "TimeSeriesAggregate",
      "TimeSeriesForecast",
      "TimeSeriesInsert",
      "TimeSeriesInvertibility",
      "TimeSeriesMap",
      "TimeSeriesMapThread",
      "TimeSeriesModel",
      "TimeSeriesModelFit",
      "TimeSeriesResample",
      "TimeSeriesRescale",
      "TimeSeriesShift",
      "TimeSeriesThread",
      "TimeSeriesWindow",
      "TimeSystem",
      "TimeSystemConvert",
      "TimeUsed",
      "TimeValue",
      "TimeWarpingCorrespondence",
      "TimeWarpingDistance",
      "TimeZone",
      "TimeZoneConvert",
      "TimeZoneOffset",
      "Timing",
      "Tiny",
      "TitleGrouping",
      "TitsGroupT",
      "ToBoxes",
      "ToCharacterCode",
      "ToColor",
      "ToContinuousTimeModel",
      "ToDate",
      "Today",
      "ToDiscreteTimeModel",
      "ToEntity",
      "ToeplitzMatrix",
      "ToExpression",
      "ToFileName",
      "Together",
      "Toggle",
      "ToggleFalse",
      "Toggler",
      "TogglerBar",
      "TogglerBox",
      "TogglerBoxOptions",
      "ToHeldExpression",
      "ToInvertibleTimeSeries",
      "TokenWords",
      "Tolerance",
      "ToLowerCase",
      "Tomorrow",
      "ToNumberField",
      "TooBig",
      "Tooltip",
      "TooltipBox",
      "TooltipBoxOptions",
      "TooltipDelay",
      "TooltipStyle",
      "ToonShading",
      "Top",
      "TopHatTransform",
      "ToPolarCoordinates",
      "TopologicalSort",
      "ToRadicals",
      "ToRawPointer",
      "ToRules",
      "Torus",
      "TorusGraph",
      "ToSphericalCoordinates",
      "ToString",
      "Total",
      "TotalHeight",
      "TotalLayer",
      "TotalVariationFilter",
      "TotalWidth",
      "TouchPosition",
      "TouchscreenAutoZoom",
      "TouchscreenControlPlacement",
      "ToUpperCase",
      "TourVideo",
      "Tr",
      "Trace",
      "TraceAbove",
      "TraceAction",
      "TraceBackward",
      "TraceDepth",
      "TraceDialog",
      "TraceForward",
      "TraceInternal",
      "TraceLevel",
      "TraceOff",
      "TraceOn",
      "TraceOriginal",
      "TracePrint",
      "TraceScan",
      "TrackCellChangeTimes",
      "TrackedSymbols",
      "TrackingFunction",
      "TracyWidomDistribution",
      "TradingChart",
      "TraditionalForm",
      "TraditionalFunctionNotation",
      "TraditionalNotation",
      "TraditionalOrder",
      "TrainImageContentDetector",
      "TrainingProgressCheckpointing",
      "TrainingProgressFunction",
      "TrainingProgressMeasurements",
      "TrainingProgressReporting",
      "TrainingStoppingCriterion",
      "TrainingUpdateSchedule",
      "TrainTextContentDetector",
      "TransferFunctionCancel",
      "TransferFunctionExpand",
      "TransferFunctionFactor",
      "TransferFunctionModel",
      "TransferFunctionPoles",
      "TransferFunctionTransform",
      "TransferFunctionZeros",
      "TransformationClass",
      "TransformationFunction",
      "TransformationFunctions",
      "TransformationMatrix",
      "TransformedDistribution",
      "TransformedField",
      "TransformedProcess",
      "TransformedRegion",
      "TransitionDirection",
      "TransitionDuration",
      "TransitionEffect",
      "TransitiveClosureGraph",
      "TransitiveReductionGraph",
      "Translate",
      "TranslationOptions",
      "TranslationTransform",
      "Transliterate",
      "Transparent",
      "TransparentColor",
      "Transpose",
      "TransposeLayer",
      "TrapEnterKey",
      "TrapSelection",
      "TravelDirections",
      "TravelDirectionsData",
      "TravelDistance",
      "TravelDistanceList",
      "TravelMethod",
      "TravelTime",
      "Tree",
      "TreeCases",
      "TreeChildren",
      "TreeCount",
      "TreeData",
      "TreeDelete",
      "TreeDepth",
      "TreeElementCoordinates",
      "TreeElementLabel",
      "TreeElementLabelFunction",
      "TreeElementLabelStyle",
      "TreeElementShape",
      "TreeElementShapeFunction",
      "TreeElementSize",
      "TreeElementSizeFunction",
      "TreeElementStyle",
      "TreeElementStyleFunction",
      "TreeExpression",
      "TreeExtract",
      "TreeFold",
      "TreeForm",
      "TreeGraph",
      "TreeGraphQ",
      "TreeInsert",
      "TreeLayout",
      "TreeLeafCount",
      "TreeLeafQ",
      "TreeLeaves",
      "TreeLevel",
      "TreeMap",
      "TreeMapAt",
      "TreeOutline",
      "TreePlot",
      "TreePosition",
      "TreeQ",
      "TreeReplacePart",
      "TreeRules",
      "TreeScan",
      "TreeSelect",
      "TreeSize",
      "TreeTraversalOrder",
      "TrendStyle",
      "Triangle",
      "TriangleCenter",
      "TriangleConstruct",
      "TriangleMeasurement",
      "TriangleWave",
      "TriangularDistribution",
      "TriangulateMesh",
      "Trig",
      "TrigExpand",
      "TrigFactor",
      "TrigFactorList",
      "Trigger",
      "TrigReduce",
      "TrigToExp",
      "TrimmedMean",
      "TrimmedVariance",
      "TropicalStormData",
      "True",
      "TrueQ",
      "TruncatedDistribution",
      "TruncatedPolyhedron",
      "TsallisQExponentialDistribution",
      "TsallisQGaussianDistribution",
      "TTest",
      "Tube",
      "TubeBezierCurveBox",
      "TubeBezierCurveBoxOptions",
      "TubeBox",
      "TubeBoxOptions",
      "TubeBSplineCurveBox",
      "TubeBSplineCurveBoxOptions",
      "Tuesday",
      "TukeyLambdaDistribution",
      "TukeyWindow",
      "TunnelData",
      "Tuples",
      "TuranGraph",
      "TuringMachine",
      "TuttePolynomial",
      "TwoWayRule",
      "Typed",
      "TypeDeclaration",
      "TypeEvaluate",
      "TypeHint",
      "TypeOf",
      "TypeSpecifier",
      "UnateQ",
      "Uncompress",
      "UnconstrainedParameters",
      "Undefined",
      "UnderBar",
      "Underflow",
      "Underlined",
      "Underoverscript",
      "UnderoverscriptBox",
      "UnderoverscriptBoxOptions",
      "Underscript",
      "UnderscriptBox",
      "UnderscriptBoxOptions",
      "UnderseaFeatureData",
      "UndirectedEdge",
      "UndirectedGraph",
      "UndirectedGraphQ",
      "UndoOptions",
      "UndoTrackedVariables",
      "Unequal",
      "UnequalTo",
      "Unevaluated",
      "UniformDistribution",
      "UniformGraphDistribution",
      "UniformPolyhedron",
      "UniformSumDistribution",
      "Uninstall",
      "Union",
      "UnionedEntityClass",
      "UnionPlus",
      "Unique",
      "UniqueElements",
      "UnitaryMatrixQ",
      "UnitBox",
      "UnitConvert",
      "UnitDimensions",
      "Unitize",
      "UnitRootTest",
      "UnitSimplify",
      "UnitStep",
      "UnitSystem",
      "UnitTriangle",
      "UnitVector",
      "UnitVectorLayer",
      "UnityDimensions",
      "UniverseModelData",
      "UniversityData",
      "UnixTime",
      "UnlabeledTree",
      "UnmanageObject",
      "Unprotect",
      "UnregisterExternalEvaluator",
      "UnsameQ",
      "UnsavedVariables",
      "Unset",
      "UnsetShared",
      "Until",
      "UntrackedVariables",
      "Up",
      "UpArrow",
      "UpArrowBar",
      "UpArrowDownArrow",
      "Update",
      "UpdateDynamicObjects",
      "UpdateDynamicObjectsSynchronous",
      "UpdateInterval",
      "UpdatePacletSites",
      "UpdateSearchIndex",
      "UpDownArrow",
      "UpEquilibrium",
      "UpperCaseQ",
      "UpperLeftArrow",
      "UpperRightArrow",
      "UpperTriangularize",
      "UpperTriangularMatrix",
      "UpperTriangularMatrixQ",
      "Upsample",
      "UpSet",
      "UpSetDelayed",
      "UpTee",
      "UpTeeArrow",
      "UpTo",
      "UpValues",
      "URL",
      "URLBuild",
      "URLDecode",
      "URLDispatcher",
      "URLDownload",
      "URLDownloadSubmit",
      "URLEncode",
      "URLExecute",
      "URLExpand",
      "URLFetch",
      "URLFetchAsynchronous",
      "URLParse",
      "URLQueryDecode",
      "URLQueryEncode",
      "URLRead",
      "URLResponseTime",
      "URLSave",
      "URLSaveAsynchronous",
      "URLShorten",
      "URLSubmit",
      "UseEmbeddedLibrary",
      "UseGraphicsRange",
      "UserDefinedWavelet",
      "Using",
      "UsingFrontEnd",
      "UtilityFunction",
      "V2Get",
      "ValenceErrorHandling",
      "ValenceFilling",
      "ValidationLength",
      "ValidationSet",
      "ValueBox",
      "ValueBoxOptions",
      "ValueDimensions",
      "ValueForm",
      "ValuePreprocessingFunction",
      "ValueQ",
      "Values",
      "ValuesData",
      "VandermondeMatrix",
      "Variables",
      "Variance",
      "VarianceEquivalenceTest",
      "VarianceEstimatorFunction",
      "VarianceGammaDistribution",
      "VarianceGammaPointProcess",
      "VarianceTest",
      "VariogramFunction",
      "VariogramModel",
      "VectorAngle",
      "VectorAround",
      "VectorAspectRatio",
      "VectorColorFunction",
      "VectorColorFunctionScaling",
      "VectorDensityPlot",
      "VectorDisplacementPlot",
      "VectorDisplacementPlot3D",
      "VectorGlyphData",
      "VectorGreater",
      "VectorGreaterEqual",
      "VectorLess",
      "VectorLessEqual",
      "VectorMarkers",
      "VectorPlot",
      "VectorPlot3D",
      "VectorPoints",
      "VectorQ",
      "VectorRange",
      "Vectors",
      "VectorScale",
      "VectorScaling",
      "VectorSizes",
      "VectorStyle",
      "Vee",
      "Verbatim",
      "Verbose",
      "VerificationTest",
      "VerifyConvergence",
      "VerifyDerivedKey",
      "VerifyDigitalSignature",
      "VerifyFileSignature",
      "VerifyInterpretation",
      "VerifySecurityCertificates",
      "VerifySolutions",
      "VerifyTestAssumptions",
      "VersionedPreferences",
      "VertexAdd",
      "VertexCapacity",
      "VertexChromaticNumber",
      "VertexColors",
      "VertexComponent",
      "VertexConnectivity",
      "VertexContract",
      "VertexCoordinateRules",
      "VertexCoordinates",
      "VertexCorrelationSimilarity",
      "VertexCosineSimilarity",
      "VertexCount",
      "VertexCoverQ",
      "VertexDataCoordinates",
      "VertexDegree",
      "VertexDelete",
      "VertexDiceSimilarity",
      "VertexEccentricity",
      "VertexInComponent",
      "VertexInComponentGraph",
      "VertexInDegree",
      "VertexIndex",
      "VertexJaccardSimilarity",
      "VertexLabeling",
      "VertexLabels",
      "VertexLabelStyle",
      "VertexList",
      "VertexNormals",
      "VertexOutComponent",
      "VertexOutComponentGraph",
      "VertexOutDegree",
      "VertexQ",
      "VertexRenderingFunction",
      "VertexReplace",
      "VertexShape",
      "VertexShapeFunction",
      "VertexSize",
      "VertexStyle",
      "VertexTextureCoordinates",
      "VertexTransitiveGraphQ",
      "VertexWeight",
      "VertexWeightedGraphQ",
      "Vertical",
      "VerticalBar",
      "VerticalForm",
      "VerticalGauge",
      "VerticalSeparator",
      "VerticalSlider",
      "VerticalTilde",
      "Video",
      "VideoCapture",
      "VideoCombine",
      "VideoDelete",
      "VideoEncoding",
      "VideoExtractFrames",
      "VideoFrameList",
      "VideoFrameMap",
      "VideoGenerator",
      "VideoInsert",
      "VideoIntervals",
      "VideoJoin",
      "VideoMap",
      "VideoMapList",
      "VideoMapTimeSeries",
      "VideoPadding",
      "VideoPause",
      "VideoPlay",
      "VideoQ",
      "VideoRecord",
      "VideoReplace",
      "VideoScreenCapture",
      "VideoSplit",
      "VideoStop",
      "VideoStream",
      "VideoStreams",
      "VideoTimeStretch",
      "VideoTrackSelection",
      "VideoTranscode",
      "VideoTransparency",
      "VideoTrim",
      "ViewAngle",
      "ViewCenter",
      "ViewMatrix",
      "ViewPoint",
      "ViewPointSelectorSettings",
      "ViewPort",
      "ViewProjection",
      "ViewRange",
      "ViewVector",
      "ViewVertical",
      "VirtualGroupData",
      "Visible",
      "VisibleCell",
      "VoiceStyleData",
      "VoigtDistribution",
      "VolcanoData",
      "Volume",
      "VonMisesDistribution",
      "VoronoiMesh",
      "WaitAll",
      "WaitAsynchronousTask",
      "WaitNext",
      "WaitUntil",
      "WakebyDistribution",
      "WalleniusHypergeometricDistribution",
      "WaringYuleDistribution",
      "WarpingCorrespondence",
      "WarpingDistance",
      "WatershedComponents",
      "WatsonUSquareTest",
      "WattsStrogatzGraphDistribution",
      "WaveletBestBasis",
      "WaveletFilterCoefficients",
      "WaveletImagePlot",
      "WaveletListPlot",
      "WaveletMapIndexed",
      "WaveletMatrixPlot",
      "WaveletPhi",
      "WaveletPsi",
      "WaveletScale",
      "WaveletScalogram",
      "WaveletThreshold",
      "WavePDEComponent",
      "WeaklyConnectedComponents",
      "WeaklyConnectedGraphComponents",
      "WeaklyConnectedGraphQ",
      "WeakStationarity",
      "WeatherData",
      "WeatherForecastData",
      "WebAudioSearch",
      "WebColumn",
      "WebElementObject",
      "WeberE",
      "WebExecute",
      "WebImage",
      "WebImageSearch",
      "WebItem",
      "WebPageMetaInformation",
      "WebRow",
      "WebSearch",
      "WebSessionObject",
      "WebSessions",
      "WebWindowObject",
      "Wedge",
      "Wednesday",
      "WeibullDistribution",
      "WeierstrassE1",
      "WeierstrassE2",
      "WeierstrassE3",
      "WeierstrassEta1",
      "WeierstrassEta2",
      "WeierstrassEta3",
      "WeierstrassHalfPeriods",
      "WeierstrassHalfPeriodW1",
      "WeierstrassHalfPeriodW2",
      "WeierstrassHalfPeriodW3",
      "WeierstrassInvariantG2",
      "WeierstrassInvariantG3",
      "WeierstrassInvariants",
      "WeierstrassP",
      "WeierstrassPPrime",
      "WeierstrassSigma",
      "WeierstrassZeta",
      "WeightedAdjacencyGraph",
      "WeightedAdjacencyMatrix",
      "WeightedData",
      "WeightedGraphQ",
      "Weights",
      "WelchWindow",
      "WheelGraph",
      "WhenEvent",
      "Which",
      "While",
      "White",
      "WhiteNoiseProcess",
      "WhitePoint",
      "Whitespace",
      "WhitespaceCharacter",
      "WhittakerM",
      "WhittakerW",
      "WholeCellGroupOpener",
      "WienerFilter",
      "WienerProcess",
      "WignerD",
      "WignerSemicircleDistribution",
      "WikidataData",
      "WikidataSearch",
      "WikipediaData",
      "WikipediaSearch",
      "WilksW",
      "WilksWTest",
      "WindDirectionData",
      "WindingCount",
      "WindingPolygon",
      "WindowClickSelect",
      "WindowElements",
      "WindowFloating",
      "WindowFrame",
      "WindowFrameElements",
      "WindowMargins",
      "WindowMovable",
      "WindowOpacity",
      "WindowPersistentStyles",
      "WindowSelected",
      "WindowSize",
      "WindowStatusArea",
      "WindowTitle",
      "WindowToolbars",
      "WindowWidth",
      "WindSpeedData",
      "WindVectorData",
      "WinsorizedMean",
      "WinsorizedVariance",
      "WishartMatrixDistribution",
      "With",
      "WithCleanup",
      "WithLock",
      "WolframAlpha",
      "WolframAlphaDate",
      "WolframAlphaQuantity",
      "WolframAlphaResult",
      "WolframCloudSettings",
      "WolframLanguageData",
      "Word",
      "WordBoundary",
      "WordCharacter",
      "WordCloud",
      "WordCount",
      "WordCounts",
      "WordData",
      "WordDefinition",
      "WordFrequency",
      "WordFrequencyData",
      "WordList",
      "WordOrientation",
      "WordSearch",
      "WordSelectionFunction",
      "WordSeparators",
      "WordSpacings",
      "WordStem",
      "WordTranslation",
      "WorkingPrecision",
      "WrapAround",
      "Write",
      "WriteLine",
      "WriteString",
      "Wronskian",
      "XMLElement",
      "XMLObject",
      "XMLTemplate",
      "Xnor",
      "Xor",
      "XYZColor",
      "Yellow",
      "Yesterday",
      "YuleDissimilarity",
      "ZernikeR",
      "ZeroSymmetric",
      "ZeroTest",
      "ZeroWidthTimes",
      "Zeta",
      "ZetaZero",
      "ZIPCodeData",
      "ZipfDistribution",
      "ZoomCenter",
      "ZoomFactor",
      "ZTest",
      "ZTransform",
      "$Aborted",
      "$ActivationGroupID",
      "$ActivationKey",
      "$ActivationUserRegistered",
      "$AddOnsDirectory",
      "$AllowDataUpdates",
      "$AllowExternalChannelFunctions",
      "$AllowInternet",
      "$AssertFunction",
      "$Assumptions",
      "$AsynchronousTask",
      "$AudioDecoders",
      "$AudioEncoders",
      "$AudioInputDevices",
      "$AudioOutputDevices",
      "$BaseDirectory",
      "$BasePacletsDirectory",
      "$BatchInput",
      "$BatchOutput",
      "$BlockchainBase",
      "$BoxForms",
      "$ByteOrdering",
      "$CacheBaseDirectory",
      "$Canceled",
      "$ChannelBase",
      "$CharacterEncoding",
      "$CharacterEncodings",
      "$CloudAccountName",
      "$CloudBase",
      "$CloudConnected",
      "$CloudConnection",
      "$CloudCreditsAvailable",
      "$CloudEvaluation",
      "$CloudExpressionBase",
      "$CloudObjectNameFormat",
      "$CloudObjectURLType",
      "$CloudRootDirectory",
      "$CloudSymbolBase",
      "$CloudUserID",
      "$CloudUserUUID",
      "$CloudVersion",
      "$CloudVersionNumber",
      "$CloudWolframEngineVersionNumber",
      "$CommandLine",
      "$CompilationTarget",
      "$CompilerEnvironment",
      "$ConditionHold",
      "$ConfiguredKernels",
      "$Context",
      "$ContextAliases",
      "$ContextPath",
      "$ControlActiveSetting",
      "$Cookies",
      "$CookieStore",
      "$CreationDate",
      "$CryptographicEllipticCurveNames",
      "$CurrentLink",
      "$CurrentTask",
      "$CurrentWebSession",
      "$DataStructures",
      "$DateStringFormat",
      "$DefaultAudioInputDevice",
      "$DefaultAudioOutputDevice",
      "$DefaultFont",
      "$DefaultFrontEnd",
      "$DefaultImagingDevice",
      "$DefaultKernels",
      "$DefaultLocalBase",
      "$DefaultLocalKernel",
      "$DefaultMailbox",
      "$DefaultNetworkInterface",
      "$DefaultPath",
      "$DefaultProxyRules",
      "$DefaultRemoteBatchSubmissionEnvironment",
      "$DefaultRemoteKernel",
      "$DefaultSystemCredentialStore",
      "$Display",
      "$DisplayFunction",
      "$DistributedContexts",
      "$DynamicEvaluation",
      "$Echo",
      "$EmbedCodeEnvironments",
      "$EmbeddableServices",
      "$EntityStores",
      "$Epilog",
      "$EvaluationCloudBase",
      "$EvaluationCloudObject",
      "$EvaluationEnvironment",
      "$ExportFormats",
      "$ExternalIdentifierTypes",
      "$ExternalStorageBase",
      "$Failed",
      "$FinancialDataSource",
      "$FontFamilies",
      "$FormatType",
      "$FrontEnd",
      "$FrontEndSession",
      "$GeneratedAssetLocation",
      "$GeoEntityTypes",
      "$GeoLocation",
      "$GeoLocationCity",
      "$GeoLocationCountry",
      "$GeoLocationPrecision",
      "$GeoLocationSource",
      "$HistoryLength",
      "$HomeDirectory",
      "$HTMLExportRules",
      "$HTTPCookies",
      "$HTTPRequest",
      "$IgnoreEOF",
      "$ImageFormattingWidth",
      "$ImageResolution",
      "$ImagingDevice",
      "$ImagingDevices",
      "$ImportFormats",
      "$IncomingMailSettings",
      "$InitialDirectory",
      "$Initialization",
      "$InitializationContexts",
      "$Input",
      "$InputFileName",
      "$InputStreamMethods",
      "$Inspector",
      "$InstallationDate",
      "$InstallationDirectory",
      "$InterfaceEnvironment",
      "$InterpreterTypes",
      "$IterationLimit",
      "$KernelCount",
      "$KernelID",
      "$Language",
      "$LaunchDirectory",
      "$LibraryPath",
      "$LicenseExpirationDate",
      "$LicenseID",
      "$LicenseProcesses",
      "$LicenseServer",
      "$LicenseSubprocesses",
      "$LicenseType",
      "$Line",
      "$Linked",
      "$LinkSupported",
      "$LoadedFiles",
      "$LocalBase",
      "$LocalSymbolBase",
      "$MachineAddresses",
      "$MachineDomain",
      "$MachineDomains",
      "$MachineEpsilon",
      "$MachineID",
      "$MachineName",
      "$MachinePrecision",
      "$MachineType",
      "$MaxDisplayedChildren",
      "$MaxExtraPrecision",
      "$MaxLicenseProcesses",
      "$MaxLicenseSubprocesses",
      "$MaxMachineNumber",
      "$MaxNumber",
      "$MaxPiecewiseCases",
      "$MaxPrecision",
      "$MaxRootDegree",
      "$MessageGroups",
      "$MessageList",
      "$MessagePrePrint",
      "$Messages",
      "$MinMachineNumber",
      "$MinNumber",
      "$MinorReleaseNumber",
      "$MinPrecision",
      "$MobilePhone",
      "$ModuleNumber",
      "$NetworkConnected",
      "$NetworkInterfaces",
      "$NetworkLicense",
      "$NewMessage",
      "$NewSymbol",
      "$NotebookInlineStorageLimit",
      "$Notebooks",
      "$NoValue",
      "$NumberMarks",
      "$Off",
      "$OperatingSystem",
      "$Output",
      "$OutputForms",
      "$OutputSizeLimit",
      "$OutputStreamMethods",
      "$Packages",
      "$ParentLink",
      "$ParentProcessID",
      "$PasswordFile",
      "$PatchLevelID",
      "$Path",
      "$PathnameSeparator",
      "$PerformanceGoal",
      "$Permissions",
      "$PermissionsGroupBase",
      "$PersistenceBase",
      "$PersistencePath",
      "$PipeSupported",
      "$PlotTheme",
      "$Post",
      "$Pre",
      "$PreferencesDirectory",
      "$PreInitialization",
      "$PrePrint",
      "$PreRead",
      "$PrintForms",
      "$PrintLiteral",
      "$Printout3DPreviewer",
      "$ProcessID",
      "$ProcessorCount",
      "$ProcessorType",
      "$ProductInformation",
      "$ProgramName",
      "$ProgressReporting",
      "$PublisherID",
      "$RandomGeneratorState",
      "$RandomState",
      "$RecursionLimit",
      "$RegisteredDeviceClasses",
      "$RegisteredUserName",
      "$ReleaseNumber",
      "$RequesterAddress",
      "$RequesterCloudUserID",
      "$RequesterCloudUserUUID",
      "$RequesterWolframID",
      "$RequesterWolframUUID",
      "$ResourceSystemBase",
      "$ResourceSystemPath",
      "$RootDirectory",
      "$ScheduledTask",
      "$ScriptCommandLine",
      "$ScriptInputString",
      "$SecuredAuthenticationKeyTokens",
      "$ServiceCreditsAvailable",
      "$Services",
      "$SessionID",
      "$SetParentLink",
      "$SharedFunctions",
      "$SharedVariables",
      "$SoundDisplay",
      "$SoundDisplayFunction",
      "$SourceLink",
      "$SSHAuthentication",
      "$SubtitleDecoders",
      "$SubtitleEncoders",
      "$SummaryBoxDataSizeLimit",
      "$SuppressInputFormHeads",
      "$SynchronousEvaluation",
      "$SyntaxHandler",
      "$System",
      "$SystemCharacterEncoding",
      "$SystemCredentialStore",
      "$SystemID",
      "$SystemMemory",
      "$SystemShell",
      "$SystemTimeZone",
      "$SystemWordLength",
      "$TargetSystems",
      "$TemplatePath",
      "$TemporaryDirectory",
      "$TemporaryPrefix",
      "$TestFileName",
      "$TextStyle",
      "$TimedOut",
      "$TimeUnit",
      "$TimeZone",
      "$TimeZoneEntity",
      "$TopDirectory",
      "$TraceOff",
      "$TraceOn",
      "$TracePattern",
      "$TracePostAction",
      "$TracePreAction",
      "$UnitSystem",
      "$Urgent",
      "$UserAddOnsDirectory",
      "$UserAgentLanguages",
      "$UserAgentMachine",
      "$UserAgentName",
      "$UserAgentOperatingSystem",
      "$UserAgentString",
      "$UserAgentVersion",
      "$UserBaseDirectory",
      "$UserBasePacletsDirectory",
      "$UserDocumentsDirectory",
      "$Username",
      "$UserName",
      "$UserURLBase",
      "$Version",
      "$VersionNumber",
      "$VideoDecoders",
      "$VideoEncoders",
      "$VoiceStyles",
      "$WolframDocumentsDirectory",
      "$WolframID",
      "$WolframUUID"
    ];
    function _(H) {
      const V = H.regex, q = /([2-9]|[1-2]\d|[3][0-5])\^\^/, G = /(\w*\.\w+|\w+\.\w*|\w+)/, J = /(\d*\.\d+|\d+\.\d*|\d+)/, Z = V.either(V.concat(q, G), J), te = /``[+-]?(\d*\.\d+|\d+\.\d*|\d+)/, ne = /`([+-]?(\d*\.\d+|\d+\.\d*|\d+))?/, ee = V.either(te, ne), ae = /\*\^[+-]?\d+/, se = {
        className: "number",
        relevance: 0,
        begin: V.concat(
          Z,
          V.optional(ee),
          V.optional(ae)
        )
      }, ce = /[a-zA-Z$][a-zA-Z0-9$]*/, le = new Set(O), ue = { variants: [
        {
          className: "builtin-symbol",
          begin: ce,
          // for performance out of fear of regex.either(...Mathematica.SYSTEM_SYMBOLS)
          "on:begin": (ge, ve) => {
            le.has(ge[0]) || ve.ignoreMatch();
          }
        },
        {
          className: "symbol",
          relevance: 0,
          begin: ce
        }
      ] }, de = {
        className: "named-character",
        begin: /\\\[[$a-zA-Z][$a-zA-Z0-9]+\]/
      }, fe = {
        className: "operator",
        relevance: 0,
        begin: /[+\-*/,;.:@~=><&|_`'^?!%]+/
      }, he = {
        className: "pattern",
        relevance: 0,
        begin: /([a-zA-Z$][a-zA-Z0-9$]*)?_+([a-zA-Z$][a-zA-Z0-9$]*)?/
      }, me = {
        className: "slot",
        relevance: 0,
        begin: /#[a-zA-Z$][a-zA-Z0-9$]*|#+[0-9]?/
      }, pe = {
        className: "brace",
        relevance: 0,
        begin: /[[\](){}]/
      }, ye = {
        className: "message-name",
        relevance: 0,
        begin: V.concat("::", ce)
      };
      return {
        name: "Mathematica",
        aliases: [
          "mma",
          "wl"
        ],
        classNameAliases: {
          brace: "punctuation",
          pattern: "type",
          slot: "type",
          symbol: "variable",
          "named-character": "variable",
          "builtin-symbol": "built_in",
          "message-name": "string"
        },
        contains: [
          H.COMMENT(/\(\*/, /\*\)/, { contains: ["self"] }),
          he,
          me,
          ye,
          ue,
          de,
          H.QUOTE_STRING_MODE,
          se,
          fe,
          pe
        ]
      };
    }
    return mathematica_1 = _, mathematica_1;
  }
  var matlab_1, hasRequiredMatlab;
  function requireMatlab() {
    if (hasRequiredMatlab) return matlab_1;
    hasRequiredMatlab = 1;
    function O(_) {
      const H = "('|\\.')+", V = {
        relevance: 0,
        contains: [{ begin: H }]
      };
      return {
        name: "Matlab",
        keywords: {
          keyword: "arguments break case catch classdef continue else elseif end enumeration events for function global if methods otherwise parfor persistent properties return spmd switch try while",
          built_in: "sin sind sinh asin asind asinh cos cosd cosh acos acosd acosh tan tand tanh atan atand atan2 atanh sec secd sech asec asecd asech csc cscd csch acsc acscd acsch cot cotd coth acot acotd acoth hypot exp expm1 log log1p log10 log2 pow2 realpow reallog realsqrt sqrt nthroot nextpow2 abs angle complex conj imag real unwrap isreal cplxpair fix floor ceil round mod rem sign airy besselj bessely besselh besseli besselk beta betainc betaln ellipj ellipke erf erfc erfcx erfinv expint gamma gammainc gammaln psi legendre cross dot factor isprime primes gcd lcm rat rats perms nchoosek factorial cart2sph cart2pol pol2cart sph2cart hsv2rgb rgb2hsv zeros ones eye repmat rand randn linspace logspace freqspace meshgrid accumarray size length ndims numel disp isempty isequal isequalwithequalnans cat reshape diag blkdiag tril triu fliplr flipud flipdim rot90 find sub2ind ind2sub bsxfun ndgrid permute ipermute shiftdim circshift squeeze isscalar isvector ans eps realmax realmin pi i|0 inf nan isnan isinf isfinite j|0 why compan gallery hadamard hankel hilb invhilb magic pascal rosser toeplitz vander wilkinson max min nanmax nanmin mean nanmean type table readtable writetable sortrows sort figure plot plot3 scatter scatter3 cellfun legend intersect ismember procrustes hold num2cell "
        },
        illegal: '(//|"|#|/\\*|\\s+/\\w+)',
        contains: [
          {
            className: "function",
            beginKeywords: "function",
            end: "$",
            contains: [
              _.UNDERSCORE_TITLE_MODE,
              {
                className: "params",
                variants: [
                  {
                    begin: "\\(",
                    end: "\\)"
                  },
                  {
                    begin: "\\[",
                    end: "\\]"
                  }
                ]
              }
            ]
          },
          {
            className: "built_in",
            begin: /true|false/,
            relevance: 0,
            starts: V
          },
          {
            begin: "[a-zA-Z][a-zA-Z_0-9]*" + H,
            relevance: 0
          },
          {
            className: "number",
            begin: _.C_NUMBER_RE,
            relevance: 0,
            starts: V
          },
          {
            className: "string",
            begin: "'",
            end: "'",
            contains: [{ begin: "''" }]
          },
          {
            begin: /\]|\}|\)/,
            relevance: 0,
            starts: V
          },
          {
            className: "string",
            begin: '"',
            end: '"',
            contains: [{ begin: '""' }],
            starts: V
          },
          _.COMMENT("^\\s*%\\{\\s*$", "^\\s*%\\}\\s*$"),
          _.COMMENT("%", "$")
        ]
      };
    }
    return matlab_1 = O, matlab_1;
  }
  var maxima_1, hasRequiredMaxima;
  function requireMaxima() {
    if (hasRequiredMaxima) return maxima_1;
    hasRequiredMaxima = 1;
    function O(_) {
      return {
        name: "Maxima",
        keywords: {
          $pattern: "[A-Za-z_%][0-9A-Za-z_%]*",
          keyword: "if then else elseif for thru do while unless step in and or not",
          literal: "true false unknown inf minf ind und %e %i %pi %phi %gamma",
          built_in: " abasep abs absint absolute_real_time acos acosh acot acoth acsc acsch activate addcol add_edge add_edges addmatrices addrow add_vertex add_vertices adjacency_matrix adjoin adjoint af agd airy airy_ai airy_bi airy_dai airy_dbi algsys alg_type alias allroots alphacharp alphanumericp amortization %and annuity_fv annuity_pv antid antidiff AntiDifference append appendfile apply apply1 apply2 applyb1 apropos args arit_amortization arithmetic arithsum array arrayapply arrayinfo arraymake arraysetapply ascii asec asech asin asinh askinteger asksign assoc assoc_legendre_p assoc_legendre_q assume assume_external_byte_order asympa at atan atan2 atanh atensimp atom atvalue augcoefmatrix augmented_lagrangian_method av average_degree backtrace bars barsplot barsplot_description base64 base64_decode bashindices batch batchload bc2 bdvac belln benefit_cost bern bernpoly bernstein_approx bernstein_expand bernstein_poly bessel bessel_i bessel_j bessel_k bessel_simplify bessel_y beta beta_incomplete beta_incomplete_generalized beta_incomplete_regularized bezout bfallroots bffac bf_find_root bf_fmin_cobyla bfhzeta bfloat bfloatp bfpsi bfpsi0 bfzeta biconnected_components bimetric binomial bipartition block blockmatrixp bode_gain bode_phase bothcoef box boxplot boxplot_description break bug_report build_info|10 buildq build_sample burn cabs canform canten cardinality carg cartan cartesian_product catch cauchy_matrix cbffac cdf_bernoulli cdf_beta cdf_binomial cdf_cauchy cdf_chi2 cdf_continuous_uniform cdf_discrete_uniform cdf_exp cdf_f cdf_gamma cdf_general_finite_discrete cdf_geometric cdf_gumbel cdf_hypergeometric cdf_laplace cdf_logistic cdf_lognormal cdf_negative_binomial cdf_noncentral_chi2 cdf_noncentral_student_t cdf_normal cdf_pareto cdf_poisson cdf_rank_sum cdf_rayleigh cdf_signed_rank cdf_student_t cdf_weibull cdisplay ceiling central_moment cequal cequalignore cf cfdisrep cfexpand cgeodesic cgreaterp cgreaterpignore changename changevar chaosgame charat charfun charfun2 charlist charp charpoly chdir chebyshev_t chebyshev_u checkdiv check_overlaps chinese cholesky christof chromatic_index chromatic_number cint circulant_graph clear_edge_weight clear_rules clear_vertex_label clebsch_gordan clebsch_graph clessp clesspignore close closefile cmetric coeff coefmatrix cograd col collapse collectterms columnop columnspace columnswap columnvector combination combine comp2pui compare compfile compile compile_file complement_graph complete_bipartite_graph complete_graph complex_number_p components compose_functions concan concat conjugate conmetderiv connected_components connect_vertices cons constant constantp constituent constvalue cont2part content continuous_freq contortion contour_plot contract contract_edge contragrad contrib_ode convert coord copy copy_file copy_graph copylist copymatrix cor cos cosh cot coth cov cov1 covdiff covect covers crc24sum create_graph create_list csc csch csetup cspline ctaylor ct_coordsys ctransform ctranspose cube_graph cuboctahedron_graph cunlisp cv cycle_digraph cycle_graph cylindrical days360 dblint deactivate declare declare_constvalue declare_dimensions declare_fundamental_dimensions declare_fundamental_units declare_qty declare_translated declare_unit_conversion declare_units declare_weights decsym defcon define define_alt_display define_variable defint defmatch defrule defstruct deftaylor degree_sequence del delete deleten delta demo demoivre denom depends derivdegree derivlist describe desolve determinant dfloat dgauss_a dgauss_b dgeev dgemm dgeqrf dgesv dgesvd diag diagmatrix diag_matrix diagmatrixp diameter diff digitcharp dimacs_export dimacs_import dimension dimensionless dimensions dimensions_as_list direct directory discrete_freq disjoin disjointp disolate disp dispcon dispform dispfun dispJordan display disprule dispterms distrib divide divisors divsum dkummer_m dkummer_u dlange dodecahedron_graph dotproduct dotsimp dpart draw draw2d draw3d drawdf draw_file draw_graph dscalar echelon edge_coloring edge_connectivity edges eigens_by_jacobi eigenvalues eigenvectors eighth einstein eivals eivects elapsed_real_time elapsed_run_time ele2comp ele2polynome ele2pui elem elementp elevation_grid elim elim_allbut eliminate eliminate_using ellipse elliptic_e elliptic_ec elliptic_eu elliptic_f elliptic_kc elliptic_pi ematrix empty_graph emptyp endcons entermatrix entertensor entier equal equalp equiv_classes erf erfc erf_generalized erfi errcatch error errormsg errors euler ev eval_string evenp every evolution evolution2d evundiff example exp expand expandwrt expandwrt_factored expint expintegral_chi expintegral_ci expintegral_e expintegral_e1 expintegral_ei expintegral_e_simplify expintegral_li expintegral_shi expintegral_si explicit explose exponentialize express expt exsec extdiff extract_linear_equations extremal_subset ezgcd %f f90 facsum factcomb factor factorfacsum factorial factorout factorsum facts fast_central_elements fast_linsolve fasttimes featurep fernfale fft fib fibtophi fifth filename_merge file_search file_type fillarray findde find_root find_root_abs find_root_error find_root_rel first fix flatten flength float floatnump floor flower_snark flush flush1deriv flushd flushnd flush_output fmin_cobyla forget fortran fourcos fourexpand fourier fourier_elim fourint fourintcos fourintsin foursimp foursin fourth fposition frame_bracket freeof freshline fresnel_c fresnel_s from_adjacency_matrix frucht_graph full_listify fullmap fullmapl fullratsimp fullratsubst fullsetify funcsolve fundamental_dimensions fundamental_units fundef funmake funp fv g0 g1 gamma gamma_greek gamma_incomplete gamma_incomplete_generalized gamma_incomplete_regularized gauss gauss_a gauss_b gaussprob gcd gcdex gcdivide gcfac gcfactor gd generalized_lambert_w genfact gen_laguerre genmatrix gensym geo_amortization geo_annuity_fv geo_annuity_pv geomap geometric geometric_mean geosum get getcurrentdirectory get_edge_weight getenv get_lu_factors get_output_stream_string get_pixel get_plot_option get_tex_environment get_tex_environment_default get_vertex_label gfactor gfactorsum ggf girth global_variances gn gnuplot_close gnuplot_replot gnuplot_reset gnuplot_restart gnuplot_start go Gosper GosperSum gr2d gr3d gradef gramschmidt graph6_decode graph6_encode graph6_export graph6_import graph_center graph_charpoly graph_eigenvalues graph_flow graph_order graph_periphery graph_product graph_size graph_union great_rhombicosidodecahedron_graph great_rhombicuboctahedron_graph grid_graph grind grobner_basis grotzch_graph hamilton_cycle hamilton_path hankel hankel_1 hankel_2 harmonic harmonic_mean hav heawood_graph hermite hessian hgfred hilbertmap hilbert_matrix hipow histogram histogram_description hodge horner hypergeometric i0 i1 %ibes ic1 ic2 ic_convert ichr1 ichr2 icosahedron_graph icosidodecahedron_graph icurvature ident identfor identity idiff idim idummy ieqn %if ifactors iframes ifs igcdex igeodesic_coords ilt image imagpart imetric implicit implicit_derivative implicit_plot indexed_tensor indices induced_subgraph inferencep inference_result infix info_display init_atensor init_ctensor in_neighbors innerproduct inpart inprod inrt integerp integer_partitions integrate intersect intersection intervalp intopois intosum invariant1 invariant2 inverse_fft inverse_jacobi_cd inverse_jacobi_cn inverse_jacobi_cs inverse_jacobi_dc inverse_jacobi_dn inverse_jacobi_ds inverse_jacobi_nc inverse_jacobi_nd inverse_jacobi_ns inverse_jacobi_sc inverse_jacobi_sd inverse_jacobi_sn invert invert_by_adjoint invert_by_lu inv_mod irr is is_biconnected is_bipartite is_connected is_digraph is_edge_in_graph is_graph is_graph_or_digraph ishow is_isomorphic isolate isomorphism is_planar isqrt isreal_p is_sconnected is_tree is_vertex_in_graph items_inference %j j0 j1 jacobi jacobian jacobi_cd jacobi_cn jacobi_cs jacobi_dc jacobi_dn jacobi_ds jacobi_nc jacobi_nd jacobi_ns jacobi_p jacobi_sc jacobi_sd jacobi_sn JF jn join jordan julia julia_set julia_sin %k kdels kdelta kill killcontext kostka kron_delta kronecker_product kummer_m kummer_u kurtosis kurtosis_bernoulli kurtosis_beta kurtosis_binomial kurtosis_chi2 kurtosis_continuous_uniform kurtosis_discrete_uniform kurtosis_exp kurtosis_f kurtosis_gamma kurtosis_general_finite_discrete kurtosis_geometric kurtosis_gumbel kurtosis_hypergeometric kurtosis_laplace kurtosis_logistic kurtosis_lognormal kurtosis_negative_binomial kurtosis_noncentral_chi2 kurtosis_noncentral_student_t kurtosis_normal kurtosis_pareto kurtosis_poisson kurtosis_rayleigh kurtosis_student_t kurtosis_weibull label labels lagrange laguerre lambda lambert_w laplace laplacian_matrix last lbfgs lc2kdt lcharp lc_l lcm lc_u ldefint ldisp ldisplay legendre_p legendre_q leinstein length let letrules letsimp levi_civita lfreeof lgtreillis lhs li liediff limit Lindstedt linear linearinterpol linear_program linear_regression line_graph linsolve listarray list_correlations listify list_matrix_entries list_nc_monomials listoftens listofvars listp lmax lmin load loadfile local locate_matrix_entry log logcontract log_gamma lopow lorentz_gauge lowercasep lpart lratsubst lreduce lriemann lsquares_estimates lsquares_estimates_approximate lsquares_estimates_exact lsquares_mse lsquares_residual_mse lsquares_residuals lsum ltreillis lu_backsub lucas lu_factor %m macroexpand macroexpand1 make_array makebox makefact makegamma make_graph make_level_picture makelist makeOrders make_poly_continent make_poly_country make_polygon make_random_state make_rgb_picture makeset make_string_input_stream make_string_output_stream make_transform mandelbrot mandelbrot_set map mapatom maplist matchdeclare matchfix mat_cond mat_fullunblocker mat_function mathml_display mat_norm matrix matrixmap matrixp matrix_size mattrace mat_trace mat_unblocker max max_clique max_degree max_flow maximize_lp max_independent_set max_matching maybe md5sum mean mean_bernoulli mean_beta mean_binomial mean_chi2 mean_continuous_uniform mean_deviation mean_discrete_uniform mean_exp mean_f mean_gamma mean_general_finite_discrete mean_geometric mean_gumbel mean_hypergeometric mean_laplace mean_logistic mean_lognormal mean_negative_binomial mean_noncentral_chi2 mean_noncentral_student_t mean_normal mean_pareto mean_poisson mean_rayleigh mean_student_t mean_weibull median median_deviation member mesh metricexpandall mgf1_sha1 min min_degree min_edge_cut minfactorial minimalPoly minimize_lp minimum_spanning_tree minor minpack_lsquares minpack_solve min_vertex_cover min_vertex_cut mkdir mnewton mod mode_declare mode_identity ModeMatrix moebius mon2schur mono monomial_dimensions multibernstein_poly multi_display_for_texinfo multi_elem multinomial multinomial_coeff multi_orbit multiplot_mode multi_pui multsym multthru mycielski_graph nary natural_unit nc_degree ncexpt ncharpoly negative_picture neighbors new newcontext newdet new_graph newline newton new_variable next_prime nicedummies niceindices ninth nofix nonarray noncentral_moment nonmetricity nonnegintegerp nonscalarp nonzeroandfreeof notequal nounify nptetrad npv nroots nterms ntermst nthroot nullity nullspace num numbered_boundaries numberp number_to_octets num_distinct_partitions numerval numfactor num_partitions nusum nzeta nzetai nzetar octets_to_number octets_to_oid odd_girth oddp ode2 ode_check odelin oid_to_octets op opena opena_binary openr openr_binary openw openw_binary operatorp opsubst optimize %or orbit orbits ordergreat ordergreatp orderless orderlessp orthogonal_complement orthopoly_recur orthopoly_weight outermap out_neighbors outofpois pade parabolic_cylinder_d parametric parametric_surface parg parGosper parse_string parse_timedate part part2cont partfrac partition partition_set partpol path_digraph path_graph pathname_directory pathname_name pathname_type pdf_bernoulli pdf_beta pdf_binomial pdf_cauchy pdf_chi2 pdf_continuous_uniform pdf_discrete_uniform pdf_exp pdf_f pdf_gamma pdf_general_finite_discrete pdf_geometric pdf_gumbel pdf_hypergeometric pdf_laplace pdf_logistic pdf_lognormal pdf_negative_binomial pdf_noncentral_chi2 pdf_noncentral_student_t pdf_normal pdf_pareto pdf_poisson pdf_rank_sum pdf_rayleigh pdf_signed_rank pdf_student_t pdf_weibull pearson_skewness permanent permut permutation permutations petersen_graph petrov pickapart picture_equalp picturep piechart piechart_description planar_embedding playback plog plot2d plot3d plotdf ploteq plsquares pochhammer points poisdiff poisexpt poisint poismap poisplus poissimp poissubst poistimes poistrim polar polarform polartorect polar_to_xy poly_add poly_buchberger poly_buchberger_criterion poly_colon_ideal poly_content polydecomp poly_depends_p poly_elimination_ideal poly_exact_divide poly_expand poly_expt poly_gcd polygon poly_grobner poly_grobner_equal poly_grobner_member poly_grobner_subsetp poly_ideal_intersection poly_ideal_polysaturation poly_ideal_polysaturation1 poly_ideal_saturation poly_ideal_saturation1 poly_lcm poly_minimization polymod poly_multiply polynome2ele polynomialp poly_normal_form poly_normalize poly_normalize_list poly_polysaturation_extension poly_primitive_part poly_pseudo_divide poly_reduced_grobner poly_reduction poly_saturation_extension poly_s_polynomial poly_subtract polytocompanion pop postfix potential power_mod powerseries powerset prefix prev_prime primep primes principal_components print printf printfile print_graph printpois printprops prodrac product properties propvars psi psubst ptriangularize pui pui2comp pui2ele pui2polynome pui_direct puireduc push put pv qput qrange qty quad_control quad_qag quad_qagi quad_qagp quad_qags quad_qawc quad_qawf quad_qawo quad_qaws quadrilateral quantile quantile_bernoulli quantile_beta quantile_binomial quantile_cauchy quantile_chi2 quantile_continuous_uniform quantile_discrete_uniform quantile_exp quantile_f quantile_gamma quantile_general_finite_discrete quantile_geometric quantile_gumbel quantile_hypergeometric quantile_laplace quantile_logistic quantile_lognormal quantile_negative_binomial quantile_noncentral_chi2 quantile_noncentral_student_t quantile_normal quantile_pareto quantile_poisson quantile_rayleigh quantile_student_t quantile_weibull quartile_skewness quit qunit quotient racah_v racah_w radcan radius random random_bernoulli random_beta random_binomial random_bipartite_graph random_cauchy random_chi2 random_continuous_uniform random_digraph random_discrete_uniform random_exp random_f random_gamma random_general_finite_discrete random_geometric random_graph random_graph1 random_gumbel random_hypergeometric random_laplace random_logistic random_lognormal random_negative_binomial random_network random_noncentral_chi2 random_noncentral_student_t random_normal random_pareto random_permutation random_poisson random_rayleigh random_regular_graph random_student_t random_tournament random_tree random_weibull range rank rat ratcoef ratdenom ratdiff ratdisrep ratexpand ratinterpol rational rationalize ratnumer ratnump ratp ratsimp ratsubst ratvars ratweight read read_array read_binary_array read_binary_list read_binary_matrix readbyte readchar read_hashed_array readline read_list read_matrix read_nested_list readonly read_xpm real_imagpart_to_conjugate realpart realroots rearray rectangle rectform rectform_log_if_constant recttopolar rediff reduce_consts reduce_order region region_boundaries region_boundaries_plus rem remainder remarray rembox remcomps remcon remcoord remfun remfunction remlet remove remove_constvalue remove_dimensions remove_edge remove_fundamental_dimensions remove_fundamental_units remove_plot_option remove_vertex rempart remrule remsym remvalue rename rename_file reset reset_displays residue resolvante resolvante_alternee1 resolvante_bipartite resolvante_diedrale resolvante_klein resolvante_klein3 resolvante_produit_sym resolvante_unitaire resolvante_vierer rest resultant return reveal reverse revert revert2 rgb2level rhs ricci riemann rinvariant risch rk rmdir rncombine romberg room rootscontract round row rowop rowswap rreduce run_testsuite %s save saving scalarp scaled_bessel_i scaled_bessel_i0 scaled_bessel_i1 scalefactors scanmap scatterplot scatterplot_description scene schur2comp sconcat scopy scsimp scurvature sdowncase sec sech second sequal sequalignore set_alt_display setdifference set_draw_defaults set_edge_weight setelmx setequalp setify setp set_partitions set_plot_option set_prompt set_random_state set_tex_environment set_tex_environment_default setunits setup_autoload set_up_dot_simplifications set_vertex_label seventh sexplode sf sha1sum sha256sum shortest_path shortest_weighted_path show showcomps showratvars sierpinskiale sierpinskimap sign signum similaritytransform simp_inequality simplify_sum simplode simpmetderiv simtran sin sinh sinsert sinvertcase sixth skewness skewness_bernoulli skewness_beta skewness_binomial skewness_chi2 skewness_continuous_uniform skewness_discrete_uniform skewness_exp skewness_f skewness_gamma skewness_general_finite_discrete skewness_geometric skewness_gumbel skewness_hypergeometric skewness_laplace skewness_logistic skewness_lognormal skewness_negative_binomial skewness_noncentral_chi2 skewness_noncentral_student_t skewness_normal skewness_pareto skewness_poisson skewness_rayleigh skewness_student_t skewness_weibull slength smake small_rhombicosidodecahedron_graph small_rhombicuboctahedron_graph smax smin smismatch snowmap snub_cube_graph snub_dodecahedron_graph solve solve_rec solve_rec_rat some somrac sort sparse6_decode sparse6_encode sparse6_export sparse6_import specint spherical spherical_bessel_j spherical_bessel_y spherical_hankel1 spherical_hankel2 spherical_harmonic spherical_to_xyz splice split sposition sprint sqfr sqrt sqrtdenest sremove sremovefirst sreverse ssearch ssort sstatus ssubst ssubstfirst staircase standardize standardize_inverse_trig starplot starplot_description status std std1 std_bernoulli std_beta std_binomial std_chi2 std_continuous_uniform std_discrete_uniform std_exp std_f std_gamma std_general_finite_discrete std_geometric std_gumbel std_hypergeometric std_laplace std_logistic std_lognormal std_negative_binomial std_noncentral_chi2 std_noncentral_student_t std_normal std_pareto std_poisson std_rayleigh std_student_t std_weibull stemplot stirling stirling1 stirling2 strim striml strimr string stringout stringp strong_components struve_h struve_l sublis sublist sublist_indices submatrix subsample subset subsetp subst substinpart subst_parallel substpart substring subvar subvarp sum sumcontract summand_to_rec supcase supcontext symbolp symmdifference symmetricp system take_channel take_inference tan tanh taylor taylorinfo taylorp taylor_simplifier taytorat tcl_output tcontract tellrat tellsimp tellsimpafter tentex tenth test_mean test_means_difference test_normality test_proportion test_proportions_difference test_rank_sum test_sign test_signed_rank test_variance test_variance_ratio tex tex1 tex_display texput %th third throw time timedate timer timer_info tldefint tlimit todd_coxeter toeplitz tokens to_lisp topological_sort to_poly to_poly_solve totaldisrep totalfourier totient tpartpol trace tracematrix trace_options transform_sample translate translate_file transpose treefale tree_reduce treillis treinat triangle triangularize trigexpand trigrat trigreduce trigsimp trunc truncate truncated_cube_graph truncated_dodecahedron_graph truncated_icosahedron_graph truncated_tetrahedron_graph tr_warnings_get tube tutte_graph ueivects uforget ultraspherical underlying_graph undiff union unique uniteigenvectors unitp units unit_step unitvector unorder unsum untellrat untimer untrace uppercasep uricci uriemann uvect vandermonde_matrix var var1 var_bernoulli var_beta var_binomial var_chi2 var_continuous_uniform var_discrete_uniform var_exp var_f var_gamma var_general_finite_discrete var_geometric var_gumbel var_hypergeometric var_laplace var_logistic var_lognormal var_negative_binomial var_noncentral_chi2 var_noncentral_student_t var_normal var_pareto var_poisson var_rayleigh var_student_t var_weibull vector vectorpotential vectorsimp verbify vers vertex_coloring vertex_connectivity vertex_degree vertex_distance vertex_eccentricity vertex_in_degree vertex_out_degree vertices vertices_to_cycle vertices_to_path %w weyl wheel_graph wiener_index wigner_3j wigner_6j wigner_9j with_stdout write_binary_data writebyte write_data writefile wronskian xreduce xthru %y Zeilberger zeroequiv zerofor zeromatrix zeromatrixp zeta zgeev zheev zlange zn_add_table zn_carmichael_lambda zn_characteristic_factors zn_determinant zn_factor_generators zn_invert_by_lu zn_log zn_mult_table absboxchar activecontexts adapt_depth additive adim aform algebraic algepsilon algexact aliases allbut all_dotsimp_denoms allocation allsym alphabetic animation antisymmetric arrays askexp assume_pos assume_pos_pred assumescalar asymbol atomgrad atrig1 axes axis_3d axis_bottom axis_left axis_right axis_top azimuth background background_color backsubst berlefact bernstein_explicit besselexpand beta_args_sum_to_integer beta_expand bftorat bftrunc bindtest border boundaries_array box boxchar breakup %c capping cauchysum cbrange cbtics center cflength cframe_flag cnonmet_flag color color_bar color_bar_tics colorbox columns commutative complex cone context contexts contour contour_levels cosnpiflag ctaypov ctaypt ctayswitch ctayvar ct_coords ctorsion_flag ctrgsimp cube current_let_rule_package cylinder data_file_name debugmode decreasing default_let_rule_package delay dependencies derivabbrev derivsubst detout diagmetric diff dim dimensions dispflag display2d|10 display_format_internal distribute_over doallmxops domain domxexpt domxmxops domxnctimes dontfactor doscmxops doscmxplus dot0nscsimp dot0simp dot1simp dotassoc dotconstrules dotdistrib dotexptsimp dotident dotscrules draw_graph_program draw_realpart edge_color edge_coloring edge_partition edge_type edge_width %edispflag elevation %emode endphi endtheta engineering_format_floats enhanced3d %enumer epsilon_lp erfflag erf_representation errormsg error_size error_syms error_type %e_to_numlog eval even evenfun evflag evfun ev_point expandwrt_denom expintexpand expintrep expon expop exptdispflag exptisolate exptsubst facexpand facsum_combine factlim factorflag factorial_expand factors_only fb feature features file_name file_output_append file_search_demo file_search_lisp file_search_maxima|10 file_search_tests file_search_usage file_type_lisp file_type_maxima|10 fill_color fill_density filled_func fixed_vertices flipflag float2bf font font_size fortindent fortspaces fpprec fpprintprec functions gamma_expand gammalim gdet genindex gensumnum GGFCFMAX GGFINFINITY globalsolve gnuplot_command gnuplot_curve_styles gnuplot_curve_titles gnuplot_default_term_command gnuplot_dumb_term_command gnuplot_file_args gnuplot_file_name gnuplot_out_file gnuplot_pdf_term_command gnuplot_pm3d gnuplot_png_term_command gnuplot_postamble gnuplot_preamble gnuplot_ps_term_command gnuplot_svg_term_command gnuplot_term gnuplot_view_args Gosper_in_Zeilberger gradefs grid grid2d grind halfangles head_angle head_both head_length head_type height hypergeometric_representation %iargs ibase icc1 icc2 icounter idummyx ieqnprint ifb ifc1 ifc2 ifg ifgi ifr iframe_bracket_form ifri igeowedge_flag ikt1 ikt2 imaginary inchar increasing infeval infinity inflag infolists inm inmc1 inmc2 intanalysis integer integervalued integrate_use_rootsof integration_constant integration_constant_counter interpolate_color intfaclim ip_grid ip_grid_in irrational isolate_wrt_times iterations itr julia_parameter %k1 %k2 keepfloat key key_pos kinvariant kt label label_alignment label_orientation labels lassociative lbfgs_ncorrections lbfgs_nfeval_max leftjust legend letrat let_rule_packages lfg lg lhospitallim limsubst linear linear_solver linechar linel|10 linenum line_type linewidth line_width linsolve_params linsolvewarn lispdisp listarith listconstvars listdummyvars lmxchar load_pathname loadprint logabs logarc logcb logconcoeffp logexpand lognegint logsimp logx logx_secondary logy logy_secondary logz lriem m1pbranch macroexpansion macros mainvar manual_demo maperror mapprint matrix_element_add matrix_element_mult matrix_element_transpose maxapplydepth maxapplyheight maxima_tempdir|10 maxima_userdir|10 maxnegex MAX_ORD maxposex maxpsifracdenom maxpsifracnum maxpsinegint maxpsiposint maxtayorder mesh_lines_color method mod_big_prime mode_check_errorp mode_checkp mode_check_warnp mod_test mod_threshold modular_linear_solver modulus multiplicative multiplicities myoptions nary negdistrib negsumdispflag newline newtonepsilon newtonmaxiter nextlayerfactor niceindicespref nm nmc noeval nolabels nonegative_lp noninteger nonscalar noun noundisp nouns np npi nticks ntrig numer numer_pbranch obase odd oddfun opacity opproperties opsubst optimprefix optionset orientation origin orthopoly_returns_intervals outative outchar packagefile palette partswitch pdf_file pfeformat phiresolution %piargs piece pivot_count_sx pivot_max_sx plot_format plot_options plot_realpart png_file pochhammer_max_index points pointsize point_size points_joined point_type poislim poisson poly_coefficient_ring poly_elimination_order polyfactor poly_grobner_algorithm poly_grobner_debug poly_monomial_order poly_primary_elimination_order poly_return_term_list poly_secondary_elimination_order poly_top_reduction_only posfun position powerdisp pred prederror primep_number_of_tests product_use_gamma program programmode promote_float_to_bigfloat prompt proportional_axes props psexpand ps_file radexpand radius radsubstflag rassociative ratalgdenom ratchristof ratdenomdivide rateinstein ratepsilon ratfac rational ratmx ratprint ratriemann ratsimpexpons ratvarswitch ratweights ratweyl ratwtlvl real realonly redraw refcheck resolution restart resultant ric riem rmxchar %rnum_list rombergabs rombergit rombergmin rombergtol rootsconmode rootsepsilon run_viewer same_xy same_xyz savedef savefactors scalar scalarmatrixp scale scale_lp setcheck setcheckbreak setval show_edge_color show_edges show_edge_type show_edge_width show_id show_label showtime show_vertex_color show_vertex_size show_vertex_type show_vertices show_weight simp simplified_output simplify_products simpproduct simpsum sinnpiflag solvedecomposes solveexplicit solvefactors solvenullwarn solveradcan solvetrigwarn space sparse sphere spring_embedding_depth sqrtdispflag stardisp startphi starttheta stats_numer stringdisp structures style sublis_apply_lambda subnumsimp sumexpand sumsplitfact surface surface_hide svg_file symmetric tab taylordepth taylor_logexpand taylor_order_coefficients taylor_truncate_polynomials tensorkill terminal testsuite_files thetaresolution timer_devalue title tlimswitch tr track transcompile transform transform_xy translate_fast_arrays transparent transrun tr_array_as_ref tr_bound_function_applyp tr_file_tty_messagesp tr_float_can_branch_complex tr_function_call_default trigexpandplus trigexpandtimes triginverses trigsign trivial_solutions tr_numer tr_optimize_max_loop tr_semicompile tr_state_vars tr_warn_bad_function_calls tr_warn_fexpr tr_warn_meval tr_warn_mode tr_warn_undeclared tr_warn_undefined_variable tstep ttyoff tube_extremes ufg ug %unitexpand unit_vectors uric uriem use_fast_arrays user_preamble usersetunits values vect_cross verbose vertex_color vertex_coloring vertex_partition vertex_size vertex_type view warnings weyl width windowname windowtitle wired_surface wireframe xaxis xaxis_color xaxis_secondary xaxis_type xaxis_width xlabel xlabel_secondary xlength xrange xrange_secondary xtics xtics_axis xtics_rotate xtics_rotate_secondary xtics_secondary xtics_secondary_axis xu_grid x_voxel xy_file xyplane xy_scale yaxis yaxis_color yaxis_secondary yaxis_type yaxis_width ylabel ylabel_secondary ylength yrange yrange_secondary ytics ytics_axis ytics_rotate ytics_rotate_secondary ytics_secondary ytics_secondary_axis yv_grid y_voxel yx_ratio zaxis zaxis_color zaxis_type zaxis_width zeroa zerob zerobern zeta%pi zlabel zlabel_rotate zlength zmin zn_primroot_limit zn_primroot_pretest",
          symbol: "_ __ %|0 %%|0"
        },
        contains: [
          {
            className: "comment",
            begin: "/\\*",
            end: "\\*/",
            contains: ["self"]
          },
          _.QUOTE_STRING_MODE,
          {
            className: "number",
            relevance: 0,
            variants: [
              {
                // float number w/ exponent
                // hmm, I wonder if we ought to include other exponent markers?
                begin: "\\b(\\d+|\\d+\\.|\\.\\d+|\\d+\\.\\d+)[Ee][-+]?\\d+\\b"
              },
              {
                // bigfloat number
                begin: "\\b(\\d+|\\d+\\.|\\.\\d+|\\d+\\.\\d+)[Bb][-+]?\\d+\\b",
                relevance: 10
              },
              {
                // float number w/out exponent
                // Doesn't seem to recognize floats which start with '.'
                begin: "\\b(\\.\\d+|\\d+\\.\\d+)\\b"
              },
              {
                // integer in base up to 36
                // Doesn't seem to recognize integers which end with '.'
                begin: "\\b(\\d+|0[0-9A-Za-z]+)\\.?\\b"
              }
            ]
          }
        ],
        illegal: /@/
      };
    }
    return maxima_1 = O, maxima_1;
  }
  var mel_1, hasRequiredMel;
  function requireMel() {
    if (hasRequiredMel) return mel_1;
    hasRequiredMel = 1;
    function O(_) {
      return {
        name: "MEL",
        keywords: "int float string vector matrix if else switch case default while do for in break continue global proc return about abs addAttr addAttributeEditorNodeHelp addDynamic addNewShelfTab addPP addPanelCategory addPrefixToName advanceToNextDrivenKey affectedNet affects aimConstraint air alias aliasAttr align alignCtx alignCurve alignSurface allViewFit ambientLight angle angleBetween animCone animCurveEditor animDisplay animView annotate appendStringArray applicationName applyAttrPreset applyTake arcLenDimContext arcLengthDimension arclen arrayMapper art3dPaintCtx artAttrCtx artAttrPaintVertexCtx artAttrSkinPaintCtx artAttrTool artBuildPaintMenu artFluidAttrCtx artPuttyCtx artSelectCtx artSetPaintCtx artUserPaintCtx assignCommand assignInputDevice assignViewportFactories attachCurve attachDeviceAttr attachSurface attrColorSliderGrp attrCompatibility attrControlGrp attrEnumOptionMenu attrEnumOptionMenuGrp attrFieldGrp attrFieldSliderGrp attrNavigationControlGrp attrPresetEditWin attributeExists attributeInfo attributeMenu attributeQuery autoKeyframe autoPlace bakeClip bakeFluidShading bakePartialHistory bakeResults bakeSimulation basename basenameEx batchRender bessel bevel bevelPlus binMembership bindSkin blend2 blendShape blendShapeEditor blendShapePanel blendTwoAttr blindDataType boneLattice boundary boxDollyCtx boxZoomCtx bufferCurve buildBookmarkMenu buildKeyframeMenu button buttonManip CBG cacheFile cacheFileCombine cacheFileMerge cacheFileTrack camera cameraView canCreateManip canvas capitalizeString catch catchQuiet ceil changeSubdivComponentDisplayLevel changeSubdivRegion channelBox character characterMap characterOutlineEditor characterize chdir checkBox checkBoxGrp checkDefaultRenderGlobals choice circle circularFillet clamp clear clearCache clip clipEditor clipEditorCurrentTimeCtx clipSchedule clipSchedulerOutliner clipTrimBefore closeCurve closeSurface cluster cmdFileOutput cmdScrollFieldExecuter cmdScrollFieldReporter cmdShell coarsenSubdivSelectionList collision color colorAtPoint colorEditor colorIndex colorIndexSliderGrp colorSliderButtonGrp colorSliderGrp columnLayout commandEcho commandLine commandPort compactHairSystem componentEditor compositingInterop computePolysetVolume condition cone confirmDialog connectAttr connectControl connectDynamic connectJoint connectionInfo constrain constrainValue constructionHistory container containsMultibyte contextInfo control convertFromOldLayers convertIffToPsd convertLightmap convertSolidTx convertTessellation convertUnit copyArray copyFlexor copyKey copySkinWeights cos cpButton cpCache cpClothSet cpCollision cpConstraint cpConvClothToMesh cpForces cpGetSolverAttr cpPanel cpProperty cpRigidCollisionFilter cpSeam cpSetEdit cpSetSolverAttr cpSolver cpSolverTypes cpTool cpUpdateClothUVs createDisplayLayer createDrawCtx createEditor createLayeredPsdFile createMotionField createNewShelf createNode createRenderLayer createSubdivRegion cross crossProduct ctxAbort ctxCompletion ctxEditMode ctxTraverse currentCtx currentTime currentTimeCtx currentUnit curve curveAddPtCtx curveCVCtx curveEPCtx curveEditorCtx curveIntersect curveMoveEPCtx curveOnSurface curveSketchCtx cutKey cycleCheck cylinder dagPose date defaultLightListCheckBox defaultNavigation defineDataServer defineVirtualDevice deformer deg_to_rad delete deleteAttr deleteShadingGroupsAndMaterials deleteShelfTab deleteUI deleteUnusedBrushes delrandstr detachCurve detachDeviceAttr detachSurface deviceEditor devicePanel dgInfo dgdirty dgeval dgtimer dimWhen directKeyCtx directionalLight dirmap dirname disable disconnectAttr disconnectJoint diskCache displacementToPoly displayAffected displayColor displayCull displayLevelOfDetail displayPref displayRGBColor displaySmoothness displayStats displayString displaySurface distanceDimContext distanceDimension doBlur dolly dollyCtx dopeSheetEditor dot dotProduct doubleProfileBirailSurface drag dragAttrContext draggerContext dropoffLocator duplicate duplicateCurve duplicateSurface dynCache dynControl dynExport dynExpression dynGlobals dynPaintEditor dynParticleCtx dynPref dynRelEdPanel dynRelEditor dynamicLoad editAttrLimits editDisplayLayerGlobals editDisplayLayerMembers editRenderLayerAdjustment editRenderLayerGlobals editRenderLayerMembers editor editorTemplate effector emit emitter enableDevice encodeString endString endsWith env equivalent equivalentTol erf error eval evalDeferred evalEcho event exactWorldBoundingBox exclusiveLightCheckBox exec executeForEachObject exists exp expression expressionEditorListen extendCurve extendSurface extrude fcheck fclose feof fflush fgetline fgetword file fileBrowserDialog fileDialog fileExtension fileInfo filetest filletCurve filter filterCurve filterExpand filterStudioImport findAllIntersections findAnimCurves findKeyframe findMenuItem findRelatedSkinCluster finder firstParentOf fitBspline flexor floatEq floatField floatFieldGrp floatScrollBar floatSlider floatSlider2 floatSliderButtonGrp floatSliderGrp floor flow fluidCacheInfo fluidEmitter fluidVoxelInfo flushUndo fmod fontDialog fopen formLayout format fprint frameLayout fread freeFormFillet frewind fromNativePath fwrite gamma gauss geometryConstraint getApplicationVersionAsFloat getAttr getClassification getDefaultBrush getFileList getFluidAttr getInputDeviceRange getMayaPanelTypes getModifiers getPanel getParticleAttr getPluginResource getenv getpid glRender glRenderEditor globalStitch gmatch goal gotoBindPose grabColor gradientControl gradientControlNoAttr graphDollyCtx graphSelectContext graphTrackCtx gravity grid gridLayout group groupObjectsByName HfAddAttractorToAS HfAssignAS HfBuildEqualMap HfBuildFurFiles HfBuildFurImages HfCancelAFR HfConnectASToHF HfCreateAttractor HfDeleteAS HfEditAS HfPerformCreateAS HfRemoveAttractorFromAS HfSelectAttached HfSelectAttractors HfUnAssignAS hardenPointCurve hardware hardwareRenderPanel headsUpDisplay headsUpMessage help helpLine hermite hide hilite hitTest hotBox hotkey hotkeyCheck hsv_to_rgb hudButton hudSlider hudSliderButton hwReflectionMap hwRender hwRenderLoad hyperGraph hyperPanel hyperShade hypot iconTextButton iconTextCheckBox iconTextRadioButton iconTextRadioCollection iconTextScrollList iconTextStaticLabel ikHandle ikHandleCtx ikHandleDisplayScale ikSolver ikSplineHandleCtx ikSystem ikSystemInfo ikfkDisplayMethod illustratorCurves image imfPlugins inheritTransform insertJoint insertJointCtx insertKeyCtx insertKnotCurve insertKnotSurface instance instanceable instancer intField intFieldGrp intScrollBar intSlider intSliderGrp interToUI internalVar intersect iprEngine isAnimCurve isConnected isDirty isParentOf isSameObject isTrue isValidObjectName isValidString isValidUiName isolateSelect itemFilter itemFilterAttr itemFilterRender itemFilterType joint jointCluster jointCtx jointDisplayScale jointLattice keyTangent keyframe keyframeOutliner keyframeRegionCurrentTimeCtx keyframeRegionDirectKeyCtx keyframeRegionDollyCtx keyframeRegionInsertKeyCtx keyframeRegionMoveKeyCtx keyframeRegionScaleKeyCtx keyframeRegionSelectKeyCtx keyframeRegionSetKeyCtx keyframeRegionTrackCtx keyframeStats lassoContext lattice latticeDeformKeyCtx launch launchImageEditor layerButton layeredShaderPort layeredTexturePort layout layoutDialog lightList lightListEditor lightListPanel lightlink lineIntersection linearPrecision linstep listAnimatable listAttr listCameras listConnections listDeviceAttachments listHistory listInputDeviceAxes listInputDeviceButtons listInputDevices listMenuAnnotation listNodeTypes listPanelCategories listRelatives listSets listTransforms listUnselected listerEditor loadFluid loadNewShelf loadPlugin loadPluginLanguageResources loadPrefObjects localizedPanelLabel lockNode loft log longNameOf lookThru ls lsThroughFilter lsType lsUI Mayatomr mag makeIdentity makeLive makePaintable makeRoll makeSingleSurface makeTubeOn makebot manipMoveContext manipMoveLimitsCtx manipOptions manipRotateContext manipRotateLimitsCtx manipScaleContext manipScaleLimitsCtx marker match max memory menu menuBarLayout menuEditor menuItem menuItemToShelf menuSet menuSetPref messageLine min minimizeApp mirrorJoint modelCurrentTimeCtx modelEditor modelPanel mouse movIn movOut move moveIKtoFK moveKeyCtx moveVertexAlongDirection multiProfileBirailSurface mute nParticle nameCommand nameField namespace namespaceInfo newPanelItems newton nodeCast nodeIconButton nodeOutliner nodePreset nodeType noise nonLinear normalConstraint normalize nurbsBoolean nurbsCopyUVSet nurbsCube nurbsEditUV nurbsPlane nurbsSelect nurbsSquare nurbsToPoly nurbsToPolygonsPref nurbsToSubdiv nurbsToSubdivPref nurbsUVSet nurbsViewDirectionVector objExists objectCenter objectLayer objectType objectTypeUI obsoleteProc oceanNurbsPreviewPlane offsetCurve offsetCurveOnSurface offsetSurface openGLExtension openMayaPref optionMenu optionMenuGrp optionVar orbit orbitCtx orientConstraint outlinerEditor outlinerPanel overrideModifier paintEffectsDisplay pairBlend palettePort paneLayout panel panelConfiguration panelHistory paramDimContext paramDimension paramLocator parent parentConstraint particle particleExists particleInstancer particleRenderInfo partition pasteKey pathAnimation pause pclose percent performanceOptions pfxstrokes pickWalk picture pixelMove planarSrf plane play playbackOptions playblast plugAttr plugNode pluginInfo pluginResourceUtil pointConstraint pointCurveConstraint pointLight pointMatrixMult pointOnCurve pointOnSurface pointPosition poleVectorConstraint polyAppend polyAppendFacetCtx polyAppendVertex polyAutoProjection polyAverageNormal polyAverageVertex polyBevel polyBlendColor polyBlindData polyBoolOp polyBridgeEdge polyCacheMonitor polyCheck polyChipOff polyClipboard polyCloseBorder polyCollapseEdge polyCollapseFacet polyColorBlindData polyColorDel polyColorPerVertex polyColorSet polyCompare polyCone polyCopyUV polyCrease polyCreaseCtx polyCreateFacet polyCreateFacetCtx polyCube polyCut polyCutCtx polyCylinder polyCylindricalProjection polyDelEdge polyDelFacet polyDelVertex polyDuplicateAndConnect polyDuplicateEdge polyEditUV polyEditUVShell polyEvaluate polyExtrudeEdge polyExtrudeFacet polyExtrudeVertex polyFlipEdge polyFlipUV polyForceUV polyGeoSampler polyHelix polyInfo polyInstallAction polyLayoutUV polyListComponentConversion polyMapCut polyMapDel polyMapSew polyMapSewMove polyMergeEdge polyMergeEdgeCtx polyMergeFacet polyMergeFacetCtx polyMergeUV polyMergeVertex polyMirrorFace polyMoveEdge polyMoveFacet polyMoveFacetUV polyMoveUV polyMoveVertex polyNormal polyNormalPerVertex polyNormalizeUV polyOptUvs polyOptions polyOutput polyPipe polyPlanarProjection polyPlane polyPlatonicSolid polyPoke polyPrimitive polyPrism polyProjection polyPyramid polyQuad polyQueryBlindData polyReduce polySelect polySelectConstraint polySelectConstraintMonitor polySelectCtx polySelectEditCtx polySeparate polySetToFaceNormal polySewEdge polyShortestPathCtx polySmooth polySoftEdge polySphere polySphericalProjection polySplit polySplitCtx polySplitEdge polySplitRing polySplitVertex polyStraightenUVBorder polySubdivideEdge polySubdivideFacet polyToSubdiv polyTorus polyTransfer polyTriangulate polyUVSet polyUnite polyWedgeFace popen popupMenu pose pow preloadRefEd print progressBar progressWindow projFileViewer projectCurve projectTangent projectionContext projectionManip promptDialog propModCtx propMove psdChannelOutliner psdEditTextureFile psdExport psdTextureFile putenv pwd python querySubdiv quit rad_to_deg radial radioButton radioButtonGrp radioCollection radioMenuItemCollection rampColorPort rand randomizeFollicles randstate rangeControl readTake rebuildCurve rebuildSurface recordAttr recordDevice redo reference referenceEdit referenceQuery refineSubdivSelectionList refresh refreshAE registerPluginResource rehash reloadImage removeJoint removeMultiInstance removePanelCategory rename renameAttr renameSelectionList renameUI render renderGlobalsNode renderInfo renderLayerButton renderLayerParent renderLayerPostProcess renderLayerUnparent renderManip renderPartition renderQualityNode renderSettings renderThumbnailUpdate renderWindowEditor renderWindowSelectContext renderer reorder reorderDeformers requires reroot resampleFluid resetAE resetPfxToPolyCamera resetTool resolutionNode retarget reverseCurve reverseSurface revolve rgb_to_hsv rigidBody rigidSolver roll rollCtx rootOf rot rotate rotationInterpolation roundConstantRadius rowColumnLayout rowLayout runTimeCommand runup sampleImage saveAllShelves saveAttrPreset saveFluid saveImage saveInitialState saveMenu savePrefObjects savePrefs saveShelf saveToolSettings scale scaleBrushBrightness scaleComponents scaleConstraint scaleKey scaleKeyCtx sceneEditor sceneUIReplacement scmh scriptCtx scriptEditorInfo scriptJob scriptNode scriptTable scriptToShelf scriptedPanel scriptedPanelType scrollField scrollLayout sculpt searchPathArray seed selLoadSettings select selectContext selectCurveCV selectKey selectKeyCtx selectKeyframeRegionCtx selectMode selectPref selectPriority selectType selectedNodes selectionConnection separator setAttr setAttrEnumResource setAttrMapping setAttrNiceNameResource setConstraintRestPosition setDefaultShadingGroup setDrivenKeyframe setDynamic setEditCtx setEditor setFluidAttr setFocus setInfinity setInputDeviceMapping setKeyCtx setKeyPath setKeyframe setKeyframeBlendshapeTargetWts setMenuMode setNodeNiceNameResource setNodeTypeFlag setParent setParticleAttr setPfxToPolyCamera setPluginResource setProject setStampDensity setStartupMessage setState setToolTo setUITemplate setXformManip sets shadingConnection shadingGeometryRelCtx shadingLightRelCtx shadingNetworkCompare shadingNode shapeCompare shelfButton shelfLayout shelfTabLayout shellField shortNameOf showHelp showHidden showManipCtx showSelectionInTitle showShadingGroupAttrEditor showWindow sign simplify sin singleProfileBirailSurface size sizeBytes skinCluster skinPercent smoothCurve smoothTangentSurface smoothstep snap2to2 snapKey snapMode snapTogetherCtx snapshot soft softMod softModCtx sort sound soundControl source spaceLocator sphere sphrand spotLight spotLightPreviewPort spreadSheetEditor spring sqrt squareSurface srtContext stackTrace startString startsWith stitchAndExplodeShell stitchSurface stitchSurfacePoints strcmp stringArrayCatenate stringArrayContains stringArrayCount stringArrayInsertAtIndex stringArrayIntersector stringArrayRemove stringArrayRemoveAtIndex stringArrayRemoveDuplicates stringArrayRemoveExact stringArrayToString stringToStringArray strip stripPrefixFromName stroke subdAutoProjection subdCleanTopology subdCollapse subdDuplicateAndConnect subdEditUV subdListComponentConversion subdMapCut subdMapSewMove subdMatchTopology subdMirror subdToBlind subdToPoly subdTransferUVsToCache subdiv subdivCrease subdivDisplaySmoothness substitute substituteAllString substituteGeometry substring surface surfaceSampler surfaceShaderList swatchDisplayPort switchTable symbolButton symbolCheckBox sysFile system tabLayout tan tangentConstraint texLatticeDeformContext texManipContext texMoveContext texMoveUVShellContext texRotateContext texScaleContext texSelectContext texSelectShortestPathCtx texSmudgeUVContext texWinToolCtx text textCurves textField textFieldButtonGrp textFieldGrp textManip textScrollList textToShelf textureDisplacePlane textureHairColor texturePlacementContext textureWindow threadCount threePointArcCtx timeControl timePort timerX toNativePath toggle toggleAxis toggleWindowVisibility tokenize tokenizeList tolerance tolower toolButton toolCollection toolDropped toolHasOptions toolPropertyWindow torus toupper trace track trackCtx transferAttributes transformCompare transformLimits translator trim trunc truncateFluidCache truncateHairCache tumble tumbleCtx turbulence twoPointArcCtx uiRes uiTemplate unassignInputDevice undo undoInfo ungroup uniform unit unloadPlugin untangleUV untitledFileName untrim upAxis updateAE userCtx uvLink uvSnapshot validateShelfName vectorize view2dToolCtx viewCamera viewClipPlane viewFit viewHeadOn viewLookAt viewManip viewPlace viewSet visor volumeAxis vortex waitCursor warning webBrowser webBrowserPrefs whatIs window windowPref wire wireContext workspace wrinkle wrinkleContext writeTake xbmLangPathList xform",
        illegal: "</",
        contains: [
          _.C_NUMBER_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            begin: "`",
            end: "`",
            contains: [_.BACKSLASH_ESCAPE]
          },
          {
            // eats variables
            begin: /[$%@](\^\w\b|#\w+|[^\s\w{]|\{\w+\}|\w+)/
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE
        ]
      };
    }
    return mel_1 = O, mel_1;
  }
  var mercury_1, hasRequiredMercury;
  function requireMercury() {
    if (hasRequiredMercury) return mercury_1;
    hasRequiredMercury = 1;
    function O(_) {
      const H = {
        keyword: "module use_module import_module include_module end_module initialise mutable initialize finalize finalise interface implementation pred mode func type inst solver any_pred any_func is semidet det nondet multi erroneous failure cc_nondet cc_multi typeclass instance where pragma promise external trace atomic or_else require_complete_switch require_det require_semidet require_multi require_nondet require_cc_multi require_cc_nondet require_erroneous require_failure",
        meta: (
          // pragma
          "inline no_inline type_spec source_file fact_table obsolete memo loop_check minimal_model terminates does_not_terminate check_termination promise_equivalent_clauses foreign_proc foreign_decl foreign_code foreign_type foreign_import_module foreign_export_enum foreign_export foreign_enum may_call_mercury will_not_call_mercury thread_safe not_thread_safe maybe_thread_safe promise_pure promise_semipure tabled_for_io local untrailed trailed attach_to_io_state can_pass_as_mercury_type stable will_not_throw_exception may_modify_trail will_not_modify_trail may_duplicate may_not_duplicate affects_liveness does_not_affect_liveness doesnt_affect_liveness no_sharing unknown_sharing sharing"
        ),
        built_in: "some all not if then else true fail false try catch catch_any semidet_true semidet_false semidet_fail impure_true impure semipure"
      }, V = _.COMMENT("%", "$"), q = {
        className: "number",
        begin: "0'.\\|0[box][0-9a-fA-F]*"
      }, G = _.inherit(_.APOS_STRING_MODE, { relevance: 0 }), J = _.inherit(_.QUOTE_STRING_MODE, { relevance: 0 }), Z = {
        className: "subst",
        begin: "\\\\[abfnrtv]\\|\\\\x[0-9a-fA-F]*\\\\\\|%[-+# *.0-9]*[dioxXucsfeEgGp]",
        relevance: 0
      };
      return J.contains = J.contains.slice(), J.contains.push(Z), {
        name: "Mercury",
        aliases: [
          "m",
          "moo"
        ],
        keywords: H,
        contains: [
          {
            className: "built_in",
            variants: [
              { begin: "<=>" },
              {
                begin: "<=",
                relevance: 0
              },
              {
                begin: "=>",
                relevance: 0
              },
              { begin: "/\\\\" },
              { begin: "\\\\/" }
            ]
          },
          {
            className: "built_in",
            variants: [
              { begin: ":-\\|-->" },
              {
                begin: "=",
                relevance: 0
              }
            ]
          },
          V,
          _.C_BLOCK_COMMENT_MODE,
          q,
          _.NUMBER_MODE,
          G,
          J,
          {
            // relevance booster
            begin: /:-/
          },
          {
            // relevance booster
            begin: /\.$/
          }
        ]
      };
    }
    return mercury_1 = O, mercury_1;
  }
  var mipsasm_1, hasRequiredMipsasm;
  function requireMipsasm() {
    if (hasRequiredMipsasm) return mipsasm_1;
    hasRequiredMipsasm = 1;
    function O(_) {
      return {
        name: "MIPS Assembly",
        case_insensitive: !0,
        aliases: ["mips"],
        keywords: {
          $pattern: "\\.?" + _.IDENT_RE,
          meta: (
            // GNU preprocs
            ".2byte .4byte .align .ascii .asciz .balign .byte .code .data .else .end .endif .endm .endr .equ .err .exitm .extern .global .hword .if .ifdef .ifndef .include .irp .long .macro .rept .req .section .set .skip .space .text .word .ltorg "
          ),
          built_in: "$0 $1 $2 $3 $4 $5 $6 $7 $8 $9 $10 $11 $12 $13 $14 $15 $16 $17 $18 $19 $20 $21 $22 $23 $24 $25 $26 $27 $28 $29 $30 $31 zero at v0 v1 a0 a1 a2 a3 a4 a5 a6 a7 t0 t1 t2 t3 t4 t5 t6 t7 t8 t9 s0 s1 s2 s3 s4 s5 s6 s7 s8 k0 k1 gp sp fp ra $f0 $f1 $f2 $f2 $f4 $f5 $f6 $f7 $f8 $f9 $f10 $f11 $f12 $f13 $f14 $f15 $f16 $f17 $f18 $f19 $f20 $f21 $f22 $f23 $f24 $f25 $f26 $f27 $f28 $f29 $f30 $f31 Context Random EntryLo0 EntryLo1 Context PageMask Wired EntryHi HWREna BadVAddr Count Compare SR IntCtl SRSCtl SRSMap Cause EPC PRId EBase Config Config1 Config2 Config3 LLAddr Debug DEPC DESAVE CacheErr ECC ErrorEPC TagLo DataLo TagHi DataHi WatchLo WatchHi PerfCtl PerfCnt "
          // Coprocessor 0 registers
        },
        contains: [
          {
            className: "keyword",
            begin: "\\b(addi?u?|andi?|b(al)?|beql?|bgez(al)?l?|bgtzl?|blezl?|bltz(al)?l?|bnel?|cl[oz]|divu?|ext|ins|j(al)?|jalr(\\.hb)?|jr(\\.hb)?|lbu?|lhu?|ll|lui|lw[lr]?|maddu?|mfhi|mflo|movn|movz|move|msubu?|mthi|mtlo|mul|multu?|nop|nor|ori?|rotrv?|sb|sc|se[bh]|sh|sllv?|slti?u?|srav?|srlv?|subu?|sw[lr]?|xori?|wsbh|abs\\.[sd]|add\\.[sd]|alnv.ps|bc1[ft]l?|c\\.(s?f|un|u?eq|[ou]lt|[ou]le|ngle?|seq|l[et]|ng[et])\\.[sd]|(ceil|floor|round|trunc)\\.[lw]\\.[sd]|cfc1|cvt\\.d\\.[lsw]|cvt\\.l\\.[dsw]|cvt\\.ps\\.s|cvt\\.s\\.[dlw]|cvt\\.s\\.p[lu]|cvt\\.w\\.[dls]|div\\.[ds]|ldx?c1|luxc1|lwx?c1|madd\\.[sd]|mfc1|mov[fntz]?\\.[ds]|msub\\.[sd]|mth?c1|mul\\.[ds]|neg\\.[ds]|nmadd\\.[ds]|nmsub\\.[ds]|p[lu][lu]\\.ps|recip\\.fmt|r?sqrt\\.[ds]|sdx?c1|sub\\.[ds]|suxc1|swx?c1|break|cache|d?eret|[de]i|ehb|mfc0|mtc0|pause|prefx?|rdhwr|rdpgpr|sdbbp|ssnop|synci?|syscall|teqi?|tgei?u?|tlb(p|r|w[ir])|tlti?u?|tnei?|wait|wrpgpr)",
            end: "\\s"
          },
          // lines ending with ; or # aren't really comments, probably auto-detect fail
          _.COMMENT("[;#](?!\\s*$)", "$"),
          _.C_BLOCK_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            begin: "'",
            end: "[^\\\\]'",
            relevance: 0
          },
          {
            className: "title",
            begin: "\\|",
            end: "\\|",
            illegal: "\\n",
            relevance: 0
          },
          {
            className: "number",
            variants: [
              {
                // hex
                begin: "0x[0-9a-f]+"
              },
              {
                // bare number
                begin: "\\b-?\\d+"
              }
            ],
            relevance: 0
          },
          {
            className: "symbol",
            variants: [
              {
                // GNU MIPS syntax
                begin: "^\\s*[a-z_\\.\\$][a-z0-9_\\.\\$]+:"
              },
              {
                // numbered local labels
                begin: "^\\s*[0-9]+:"
              },
              {
                // number local label reference (backwards, forwards)
                begin: "[0-9]+[bf]"
              }
            ],
            relevance: 0
          }
        ],
        // forward slashes are not allowed
        illegal: /\//
      };
    }
    return mipsasm_1 = O, mipsasm_1;
  }
  var mizar_1, hasRequiredMizar;
  function requireMizar() {
    if (hasRequiredMizar) return mizar_1;
    hasRequiredMizar = 1;
    function O(_) {
      return {
        name: "Mizar",
        keywords: "environ vocabularies notations constructors definitions registrations theorems schemes requirements begin end definition registration cluster existence pred func defpred deffunc theorem proof let take assume then thus hence ex for st holds consider reconsider such that and in provided of as from be being by means equals implies iff redefine define now not or attr is mode suppose per cases set thesis contradiction scheme reserve struct correctness compatibility coherence symmetry assymetry reflexivity irreflexivity connectedness uniqueness commutativity idempotence involutiveness projectivity",
        contains: [_.COMMENT("::", "$")]
      };
    }
    return mizar_1 = O, mizar_1;
  }
  var mojolicious_1, hasRequiredMojolicious;
  function requireMojolicious() {
    if (hasRequiredMojolicious) return mojolicious_1;
    hasRequiredMojolicious = 1;
    function O(_) {
      return {
        name: "Mojolicious",
        subLanguage: "xml",
        contains: [
          {
            className: "meta",
            begin: "^__(END|DATA)__$"
          },
          // mojolicious line
          {
            begin: "^\\s*%{1,2}={0,2}",
            end: "$",
            subLanguage: "perl"
          },
          // mojolicious block
          {
            begin: "<%{1,2}={0,2}",
            end: "={0,1}%>",
            subLanguage: "perl",
            excludeBegin: !0,
            excludeEnd: !0
          }
        ]
      };
    }
    return mojolicious_1 = O, mojolicious_1;
  }
  var monkey_1, hasRequiredMonkey;
  function requireMonkey() {
    if (hasRequiredMonkey) return monkey_1;
    hasRequiredMonkey = 1;
    function O(_) {
      const H = {
        className: "number",
        relevance: 0,
        variants: [
          { begin: "[$][a-fA-F0-9]+" },
          _.NUMBER_MODE
        ]
      }, V = {
        variants: [
          { match: [
            /(function|method)/,
            /\s+/,
            _.UNDERSCORE_IDENT_RE
          ] }
        ],
        scope: {
          1: "keyword",
          3: "title.function"
        }
      }, q = {
        variants: [
          { match: [
            /(class|interface|extends|implements)/,
            /\s+/,
            _.UNDERSCORE_IDENT_RE
          ] }
        ],
        scope: {
          1: "keyword",
          3: "title.class"
        }
      };
      return {
        name: "Monkey",
        case_insensitive: !0,
        keywords: {
          keyword: [
            "public",
            "private",
            "property",
            "continue",
            "exit",
            "extern",
            "new",
            "try",
            "catch",
            "eachin",
            "not",
            "abstract",
            "final",
            "select",
            "case",
            "default",
            "const",
            "local",
            "global",
            "field",
            "end",
            "if",
            "then",
            "else",
            "elseif",
            "endif",
            "while",
            "wend",
            "repeat",
            "until",
            "forever",
            "for",
            "to",
            "step",
            "next",
            "return",
            "module",
            "inline",
            "throw",
            "import",
            // not positive, but these are not literals
            "and",
            "or",
            "shl",
            "shr",
            "mod"
          ],
          built_in: [
            "DebugLog",
            "DebugStop",
            "Error",
            "Print",
            "ACos",
            "ACosr",
            "ASin",
            "ASinr",
            "ATan",
            "ATan2",
            "ATan2r",
            "ATanr",
            "Abs",
            "Abs",
            "Ceil",
            "Clamp",
            "Clamp",
            "Cos",
            "Cosr",
            "Exp",
            "Floor",
            "Log",
            "Max",
            "Max",
            "Min",
            "Min",
            "Pow",
            "Sgn",
            "Sgn",
            "Sin",
            "Sinr",
            "Sqrt",
            "Tan",
            "Tanr",
            "Seed",
            "PI",
            "HALFPI",
            "TWOPI"
          ],
          literal: [
            "true",
            "false",
            "null"
          ]
        },
        illegal: /\/\*/,
        contains: [
          _.COMMENT("#rem", "#end"),
          _.COMMENT(
            "'",
            "$",
            { relevance: 0 }
          ),
          V,
          q,
          {
            className: "variable.language",
            begin: /\b(self|super)\b/
          },
          {
            className: "meta",
            begin: /\s*#/,
            end: "$",
            keywords: { keyword: "if else elseif endif end then" }
          },
          {
            match: [
              /^\s*/,
              /strict\b/
            ],
            scope: { 2: "meta" }
          },
          {
            beginKeywords: "alias",
            end: "=",
            contains: [_.UNDERSCORE_TITLE_MODE]
          },
          _.QUOTE_STRING_MODE,
          H
        ]
      };
    }
    return monkey_1 = O, monkey_1;
  }
  var moonscript_1, hasRequiredMoonscript;
  function requireMoonscript() {
    if (hasRequiredMoonscript) return moonscript_1;
    hasRequiredMoonscript = 1;
    function O(_) {
      const H = {
        keyword: (
          // Moonscript keywords
          "if then not for in while do return else elseif break continue switch and or unless when class extends super local import export from using"
        ),
        literal: "true false nil",
        built_in: "_G _VERSION assert collectgarbage dofile error getfenv getmetatable ipairs load loadfile loadstring module next pairs pcall print rawequal rawget rawset require select setfenv setmetatable tonumber tostring type unpack xpcall coroutine debug io math os package string table"
      }, V = "[A-Za-z$_][0-9A-Za-z$_]*", q = {
        className: "subst",
        begin: /#\{/,
        end: /\}/,
        keywords: H
      }, G = [
        _.inherit(
          _.C_NUMBER_MODE,
          { starts: {
            end: "(\\s*/)?",
            relevance: 0
          } }
        ),
        // a number tries to eat the following slash to prevent treating it as a regexp
        {
          className: "string",
          variants: [
            {
              begin: /'/,
              end: /'/,
              contains: [_.BACKSLASH_ESCAPE]
            },
            {
              begin: /"/,
              end: /"/,
              contains: [
                _.BACKSLASH_ESCAPE,
                q
              ]
            }
          ]
        },
        {
          className: "built_in",
          begin: "@__" + _.IDENT_RE
        },
        {
          begin: "@" + _.IDENT_RE
          // relevance booster on par with CoffeeScript
        },
        {
          begin: _.IDENT_RE + "\\\\" + _.IDENT_RE
          // inst\method
        }
      ];
      q.contains = G;
      const J = _.inherit(_.TITLE_MODE, { begin: V }), Z = "(\\(.*\\)\\s*)?\\B[-=]>", te = {
        className: "params",
        begin: "\\([^\\(]",
        returnBegin: !0,
        /* We need another contained nameless mode to not have every nested
        pair of parens to be called "params" */
        contains: [
          {
            begin: /\(/,
            end: /\)/,
            keywords: H,
            contains: ["self"].concat(G)
          }
        ]
      };
      return {
        name: "MoonScript",
        aliases: ["moon"],
        keywords: H,
        illegal: /\/\*/,
        contains: G.concat([
          _.COMMENT("--", "$"),
          {
            className: "function",
            // function: -> =>
            begin: "^\\s*" + V + "\\s*=\\s*" + Z,
            end: "[-=]>",
            returnBegin: !0,
            contains: [
              J,
              te
            ]
          },
          {
            begin: /[\(,:=]\s*/,
            // anonymous function start
            relevance: 0,
            contains: [
              {
                className: "function",
                begin: Z,
                end: "[-=]>",
                returnBegin: !0,
                contains: [te]
              }
            ]
          },
          {
            className: "class",
            beginKeywords: "class",
            end: "$",
            illegal: /[:="\[\]]/,
            contains: [
              {
                beginKeywords: "extends",
                endsWithParent: !0,
                illegal: /[:="\[\]]/,
                contains: [J]
              },
              J
            ]
          },
          {
            className: "name",
            // table
            begin: V + ":",
            end: ":",
            returnBegin: !0,
            returnEnd: !0,
            relevance: 0
          }
        ])
      };
    }
    return moonscript_1 = O, moonscript_1;
  }
  var n1ql_1, hasRequiredN1ql;
  function requireN1ql() {
    if (hasRequiredN1ql) return n1ql_1;
    hasRequiredN1ql = 1;
    function O(_) {
      return {
        name: "N1QL",
        case_insensitive: !0,
        contains: [
          {
            beginKeywords: "build create index delete drop explain infer|10 insert merge prepare select update upsert|10",
            end: /;/,
            keywords: {
              keyword: [
                "all",
                "alter",
                "analyze",
                "and",
                "any",
                "array",
                "as",
                "asc",
                "begin",
                "between",
                "binary",
                "boolean",
                "break",
                "bucket",
                "build",
                "by",
                "call",
                "case",
                "cast",
                "cluster",
                "collate",
                "collection",
                "commit",
                "connect",
                "continue",
                "correlate",
                "cover",
                "create",
                "database",
                "dataset",
                "datastore",
                "declare",
                "decrement",
                "delete",
                "derived",
                "desc",
                "describe",
                "distinct",
                "do",
                "drop",
                "each",
                "element",
                "else",
                "end",
                "every",
                "except",
                "exclude",
                "execute",
                "exists",
                "explain",
                "fetch",
                "first",
                "flatten",
                "for",
                "force",
                "from",
                "function",
                "grant",
                "group",
                "gsi",
                "having",
                "if",
                "ignore",
                "ilike",
                "in",
                "include",
                "increment",
                "index",
                "infer",
                "inline",
                "inner",
                "insert",
                "intersect",
                "into",
                "is",
                "join",
                "key",
                "keys",
                "keyspace",
                "known",
                "last",
                "left",
                "let",
                "letting",
                "like",
                "limit",
                "lsm",
                "map",
                "mapping",
                "matched",
                "materialized",
                "merge",
                "minus",
                "namespace",
                "nest",
                "not",
                "number",
                "object",
                "offset",
                "on",
                "option",
                "or",
                "order",
                "outer",
                "over",
                "parse",
                "partition",
                "password",
                "path",
                "pool",
                "prepare",
                "primary",
                "private",
                "privilege",
                "procedure",
                "public",
                "raw",
                "realm",
                "reduce",
                "rename",
                "return",
                "returning",
                "revoke",
                "right",
                "role",
                "rollback",
                "satisfies",
                "schema",
                "select",
                "self",
                "semi",
                "set",
                "show",
                "some",
                "start",
                "statistics",
                "string",
                "system",
                "then",
                "to",
                "transaction",
                "trigger",
                "truncate",
                "under",
                "union",
                "unique",
                "unknown",
                "unnest",
                "unset",
                "update",
                "upsert",
                "use",
                "user",
                "using",
                "validate",
                "value",
                "valued",
                "values",
                "via",
                "view",
                "when",
                "where",
                "while",
                "with",
                "within",
                "work",
                "xor"
              ],
              literal: [
                "true",
                "false",
                "null",
                "missing|5"
              ],
              built_in: [
                "array_agg",
                "array_append",
                "array_concat",
                "array_contains",
                "array_count",
                "array_distinct",
                "array_ifnull",
                "array_length",
                "array_max",
                "array_min",
                "array_position",
                "array_prepend",
                "array_put",
                "array_range",
                "array_remove",
                "array_repeat",
                "array_replace",
                "array_reverse",
                "array_sort",
                "array_sum",
                "avg",
                "count",
                "max",
                "min",
                "sum",
                "greatest",
                "least",
                "ifmissing",
                "ifmissingornull",
                "ifnull",
                "missingif",
                "nullif",
                "ifinf",
                "ifnan",
                "ifnanorinf",
                "naninf",
                "neginfif",
                "posinfif",
                "clock_millis",
                "clock_str",
                "date_add_millis",
                "date_add_str",
                "date_diff_millis",
                "date_diff_str",
                "date_part_millis",
                "date_part_str",
                "date_trunc_millis",
                "date_trunc_str",
                "duration_to_str",
                "millis",
                "str_to_millis",
                "millis_to_str",
                "millis_to_utc",
                "millis_to_zone_name",
                "now_millis",
                "now_str",
                "str_to_duration",
                "str_to_utc",
                "str_to_zone_name",
                "decode_json",
                "encode_json",
                "encoded_size",
                "poly_length",
                "base64",
                "base64_encode",
                "base64_decode",
                "meta",
                "uuid",
                "abs",
                "acos",
                "asin",
                "atan",
                "atan2",
                "ceil",
                "cos",
                "degrees",
                "e",
                "exp",
                "ln",
                "log",
                "floor",
                "pi",
                "power",
                "radians",
                "random",
                "round",
                "sign",
                "sin",
                "sqrt",
                "tan",
                "trunc",
                "object_length",
                "object_names",
                "object_pairs",
                "object_inner_pairs",
                "object_values",
                "object_inner_values",
                "object_add",
                "object_put",
                "object_remove",
                "object_unwrap",
                "regexp_contains",
                "regexp_like",
                "regexp_position",
                "regexp_replace",
                "contains",
                "initcap",
                "length",
                "lower",
                "ltrim",
                "position",
                "repeat",
                "replace",
                "rtrim",
                "split",
                "substr",
                "title",
                "trim",
                "upper",
                "isarray",
                "isatom",
                "isboolean",
                "isnumber",
                "isobject",
                "isstring",
                "type",
                "toarray",
                "toatom",
                "toboolean",
                "tonumber",
                "toobject",
                "tostring"
              ]
            },
            contains: [
              {
                className: "string",
                begin: "'",
                end: "'",
                contains: [_.BACKSLASH_ESCAPE]
              },
              {
                className: "string",
                begin: '"',
                end: '"',
                contains: [_.BACKSLASH_ESCAPE]
              },
              {
                className: "symbol",
                begin: "`",
                end: "`",
                contains: [_.BACKSLASH_ESCAPE]
              },
              _.C_NUMBER_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          _.C_BLOCK_COMMENT_MODE
        ]
      };
    }
    return n1ql_1 = O, n1ql_1;
  }
  var nestedtext_1, hasRequiredNestedtext;
  function requireNestedtext() {
    if (hasRequiredNestedtext) return nestedtext_1;
    hasRequiredNestedtext = 1;
    function O(_) {
      const H = {
        match: [
          /^\s*(?=\S)/,
          // have to look forward here to avoid polynomial backtracking
          /[^:]+/,
          /:\s*/,
          /$/
        ],
        className: {
          2: "attribute",
          3: "punctuation"
        }
      }, V = {
        match: [
          /^\s*(?=\S)/,
          // have to look forward here to avoid polynomial backtracking
          /[^:]*[^: ]/,
          /[ ]*:/,
          /[ ]/,
          /.*$/
        ],
        className: {
          2: "attribute",
          3: "punctuation",
          5: "string"
        }
      }, q = {
        match: [
          /^\s*/,
          />/,
          /[ ]/,
          /.*$/
        ],
        className: {
          2: "punctuation",
          4: "string"
        }
      }, G = {
        variants: [
          { match: [
            /^\s*/,
            /-/,
            /[ ]/,
            /.*$/
          ] },
          { match: [
            /^\s*/,
            /-$/
          ] }
        ],
        className: {
          2: "bullet",
          4: "string"
        }
      };
      return {
        name: "Nested Text",
        aliases: ["nt"],
        contains: [
          _.inherit(_.HASH_COMMENT_MODE, {
            begin: /^\s*(?=#)/,
            excludeBegin: !0
          }),
          G,
          q,
          H,
          V
        ]
      };
    }
    return nestedtext_1 = O, nestedtext_1;
  }
  var nginx_1, hasRequiredNginx;
  function requireNginx() {
    if (hasRequiredNginx) return nginx_1;
    hasRequiredNginx = 1;
    function O(_) {
      const H = _.regex, V = {
        className: "variable",
        variants: [
          { begin: /\$\d+/ },
          { begin: /\$\{\w+\}/ },
          { begin: H.concat(/[$@]/, _.UNDERSCORE_IDENT_RE) }
        ]
      }, G = {
        endsWithParent: !0,
        keywords: {
          $pattern: /[a-z_]{2,}|\/dev\/poll/,
          literal: [
            "on",
            "off",
            "yes",
            "no",
            "true",
            "false",
            "none",
            "blocked",
            "debug",
            "info",
            "notice",
            "warn",
            "error",
            "crit",
            "select",
            "break",
            "last",
            "permanent",
            "redirect",
            "kqueue",
            "rtsig",
            "epoll",
            "poll",
            "/dev/poll"
          ]
        },
        relevance: 0,
        illegal: "=>",
        contains: [
          _.HASH_COMMENT_MODE,
          {
            className: "string",
            contains: [
              _.BACKSLASH_ESCAPE,
              V
            ],
            variants: [
              {
                begin: /"/,
                end: /"/
              },
              {
                begin: /'/,
                end: /'/
              }
            ]
          },
          // this swallows entire URLs to avoid detecting numbers within
          {
            begin: "([a-z]+):/",
            end: "\\s",
            endsWithParent: !0,
            excludeEnd: !0,
            contains: [V]
          },
          {
            className: "regexp",
            contains: [
              _.BACKSLASH_ESCAPE,
              V
            ],
            variants: [
              {
                begin: "\\s\\^",
                end: "\\s|\\{|;",
                returnEnd: !0
              },
              // regexp locations (~, ~*)
              {
                begin: "~\\*?\\s+",
                end: "\\s|\\{|;",
                returnEnd: !0
              },
              // *.example.com
              { begin: "\\*(\\.[a-z\\-]+)+" },
              // sub.example.*
              { begin: "([a-z\\-]+\\.)+\\*" }
            ]
          },
          // IP
          {
            className: "number",
            begin: "\\b\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}(:\\d{1,5})?\\b"
          },
          // units
          {
            className: "number",
            begin: "\\b\\d+[kKmMgGdshdwy]?\\b",
            relevance: 0
          },
          V
        ]
      };
      return {
        name: "Nginx config",
        aliases: ["nginxconf"],
        contains: [
          _.HASH_COMMENT_MODE,
          {
            beginKeywords: "upstream location",
            end: /;|\{/,
            contains: G.contains,
            keywords: { section: "upstream location" }
          },
          {
            className: "section",
            begin: H.concat(_.UNDERSCORE_IDENT_RE + H.lookahead(/\s+\{/)),
            relevance: 0
          },
          {
            begin: H.lookahead(_.UNDERSCORE_IDENT_RE + "\\s"),
            end: ";|\\{",
            contains: [
              {
                className: "attribute",
                begin: _.UNDERSCORE_IDENT_RE,
                starts: G
              }
            ],
            relevance: 0
          }
        ],
        illegal: "[^\\s\\}\\{]"
      };
    }
    return nginx_1 = O, nginx_1;
  }
  var nim_1, hasRequiredNim;
  function requireNim() {
    if (hasRequiredNim) return nim_1;
    hasRequiredNim = 1;
    function O(_) {
      return {
        name: "Nim",
        keywords: {
          keyword: [
            "addr",
            "and",
            "as",
            "asm",
            "bind",
            "block",
            "break",
            "case",
            "cast",
            "concept",
            "const",
            "continue",
            "converter",
            "defer",
            "discard",
            "distinct",
            "div",
            "do",
            "elif",
            "else",
            "end",
            "enum",
            "except",
            "export",
            "finally",
            "for",
            "from",
            "func",
            "generic",
            "guarded",
            "if",
            "import",
            "in",
            "include",
            "interface",
            "is",
            "isnot",
            "iterator",
            "let",
            "macro",
            "method",
            "mixin",
            "mod",
            "nil",
            "not",
            "notin",
            "object",
            "of",
            "or",
            "out",
            "proc",
            "ptr",
            "raise",
            "ref",
            "return",
            "shared",
            "shl",
            "shr",
            "static",
            "template",
            "try",
            "tuple",
            "type",
            "using",
            "var",
            "when",
            "while",
            "with",
            "without",
            "xor",
            "yield"
          ],
          literal: [
            "true",
            "false"
          ],
          type: [
            "int",
            "int8",
            "int16",
            "int32",
            "int64",
            "uint",
            "uint8",
            "uint16",
            "uint32",
            "uint64",
            "float",
            "float32",
            "float64",
            "bool",
            "char",
            "string",
            "cstring",
            "pointer",
            "expr",
            "stmt",
            "void",
            "auto",
            "any",
            "range",
            "array",
            "openarray",
            "varargs",
            "seq",
            "set",
            "clong",
            "culong",
            "cchar",
            "cschar",
            "cshort",
            "cint",
            "csize",
            "clonglong",
            "cfloat",
            "cdouble",
            "clongdouble",
            "cuchar",
            "cushort",
            "cuint",
            "culonglong",
            "cstringarray",
            "semistatic"
          ],
          built_in: [
            "stdin",
            "stdout",
            "stderr",
            "result"
          ]
        },
        contains: [
          {
            className: "meta",
            // Actually pragma
            begin: /\{\./,
            end: /\.\}/,
            relevance: 10
          },
          {
            className: "string",
            begin: /[a-zA-Z]\w*"/,
            end: /"/,
            contains: [{ begin: /""/ }]
          },
          {
            className: "string",
            begin: /([a-zA-Z]\w*)?"""/,
            end: /"""/
          },
          _.QUOTE_STRING_MODE,
          {
            className: "type",
            begin: /\b[A-Z]\w+\b/,
            relevance: 0
          },
          {
            className: "number",
            relevance: 0,
            variants: [
              { begin: /\b(0[xX][0-9a-fA-F][_0-9a-fA-F]*)('?[iIuU](8|16|32|64))?/ },
              { begin: /\b(0o[0-7][_0-7]*)('?[iIuUfF](8|16|32|64))?/ },
              { begin: /\b(0(b|B)[01][_01]*)('?[iIuUfF](8|16|32|64))?/ },
              { begin: /\b(\d[_\d]*)('?[iIuUfF](8|16|32|64))?/ }
            ]
          },
          _.HASH_COMMENT_MODE
        ]
      };
    }
    return nim_1 = O, nim_1;
  }
  var nix_1, hasRequiredNix;
  function requireNix() {
    if (hasRequiredNix) return nix_1;
    hasRequiredNix = 1;
    function O(_) {
      const H = _.regex, V = {
        keyword: [
          "assert",
          "else",
          "if",
          "in",
          "inherit",
          "let",
          "or",
          "rec",
          "then",
          "with"
        ],
        literal: [
          "true",
          "false",
          "null"
        ],
        built_in: [
          // toplevel builtins
          "abort",
          "baseNameOf",
          "builtins",
          "derivation",
          "derivationStrict",
          "dirOf",
          "fetchGit",
          "fetchMercurial",
          "fetchTarball",
          "fetchTree",
          "fromTOML",
          "import",
          "isNull",
          "map",
          "placeholder",
          "removeAttrs",
          "scopedImport",
          "throw",
          "toString"
        ]
      }, q = {
        scope: "built_in",
        match: H.either(...[
          "abort",
          "add",
          "addDrvOutputDependencies",
          "addErrorContext",
          "all",
          "any",
          "appendContext",
          "attrNames",
          "attrValues",
          "baseNameOf",
          "bitAnd",
          "bitOr",
          "bitXor",
          "break",
          "builtins",
          "catAttrs",
          "ceil",
          "compareVersions",
          "concatLists",
          "concatMap",
          "concatStringsSep",
          "convertHash",
          "currentSystem",
          "currentTime",
          "deepSeq",
          "derivation",
          "derivationStrict",
          "dirOf",
          "div",
          "elem",
          "elemAt",
          "false",
          "fetchGit",
          "fetchMercurial",
          "fetchTarball",
          "fetchTree",
          "fetchurl",
          "filter",
          "filterSource",
          "findFile",
          "flakeRefToString",
          "floor",
          "foldl'",
          "fromJSON",
          "fromTOML",
          "functionArgs",
          "genList",
          "genericClosure",
          "getAttr",
          "getContext",
          "getEnv",
          "getFlake",
          "groupBy",
          "hasAttr",
          "hasContext",
          "hashFile",
          "hashString",
          "head",
          "import",
          "intersectAttrs",
          "isAttrs",
          "isBool",
          "isFloat",
          "isFunction",
          "isInt",
          "isList",
          "isNull",
          "isPath",
          "isString",
          "langVersion",
          "length",
          "lessThan",
          "listToAttrs",
          "map",
          "mapAttrs",
          "match",
          "mul",
          "nixPath",
          "nixVersion",
          "null",
          "parseDrvName",
          "parseFlakeRef",
          "partition",
          "path",
          "pathExists",
          "placeholder",
          "readDir",
          "readFile",
          "readFileType",
          "removeAttrs",
          "replaceStrings",
          "scopedImport",
          "seq",
          "sort",
          "split",
          "splitVersion",
          "storeDir",
          "storePath",
          "stringLength",
          "sub",
          "substring",
          "tail",
          "throw",
          "toFile",
          "toJSON",
          "toPath",
          "toString",
          "toXML",
          "trace",
          "traceVerbose",
          "true",
          "tryEval",
          "typeOf",
          "unsafeDiscardOutputDependency",
          "unsafeDiscardStringContext",
          "unsafeGetAttrPos",
          "warn",
          "zipAttrsWith"
        ].map((ge) => `builtins\\.${ge}`)),
        relevance: 10
      }, G = "[A-Za-z_][A-Za-z0-9_'-]*", J = {
        scope: "symbol",
        match: new RegExp(`<${G}(/${G})*>`)
      }, Z = "[A-Za-z0-9_\\+\\.-]+", te = {
        scope: "symbol",
        match: new RegExp(`(\\.\\.|\\.|~)?/(${Z})?(/${Z})*(?=[\\s;])`)
      }, ne = H.either("==", "=", "\\+\\+", "\\+", "<=", "<\\|", "<", ">=", ">", "->", "//", "/", "!=", "!", "\\|\\|", "\\|>", "\\?", "\\*", "&&"), ee = {
        scope: "operator",
        match: H.concat(ne, /(?!-)/),
        relevance: 0
      }, ae = {
        scope: "number",
        match: new RegExp(`${_.NUMBER_RE}(?!-)`),
        relevance: 0
      }, oe = {
        variants: [
          {
            scope: "operator",
            beforeMatch: /\s/,
            // The (?!>) is used to ensure this doesn't collide with the '->' operator
            begin: /-(?!>)/
          },
          {
            begin: [
              new RegExp(`${_.NUMBER_RE}`),
              /-/,
              /(?!>)/
            ],
            beginScope: {
              1: "number",
              2: "operator"
            }
          },
          {
            begin: [
              ne,
              /-/,
              /(?!>)/
            ],
            beginScope: {
              1: "operator",
              2: "operator"
            }
          }
        ],
        relevance: 0
      }, se = {
        beforeMatch: /(^|\{|;)\s*/,
        begin: new RegExp(`${G}(\\.${G})*\\s*=(?!=)`),
        returnBegin: !0,
        relevance: 0,
        contains: [
          {
            scope: "attr",
            match: new RegExp(`${G}(\\.${G})*(?=\\s*=)`),
            relevance: 0.2
          }
        ]
      }, ce = {
        scope: "char.escape",
        match: /\\\$/
      }, le = {
        scope: "char.escape",
        match: /''\$/
      }, ue = {
        scope: "subst",
        begin: /\$\{/,
        end: /\}/,
        keywords: V
      }, de = {
        scope: "char.escape",
        match: /'''/
      }, fe = {
        scope: "char.escape",
        match: /\\(?!\$)./
      }, he = {
        scope: "string",
        variants: [
          {
            begin: "''",
            end: "''",
            contains: [
              le,
              ue,
              de,
              fe
            ]
          },
          {
            begin: '"',
            end: '"',
            contains: [
              ce,
              ue,
              fe
            ]
          }
        ]
      }, me = {
        scope: "params",
        match: new RegExp(`${G}\\s*:(?=\\s)`)
      }, pe = [
        ae,
        _.HASH_COMMENT_MODE,
        _.C_BLOCK_COMMENT_MODE,
        _.COMMENT(
          /\/\*\*(?!\/)/,
          /\*\//,
          {
            subLanguage: "markdown",
            relevance: 0
          }
        ),
        q,
        he,
        J,
        te,
        me,
        se,
        oe,
        ee
      ];
      ue.contains = pe;
      const ye = [
        {
          scope: "meta.prompt",
          match: /^nix-repl>(?=\s)/,
          relevance: 10
        },
        {
          scope: "meta",
          beforeMatch: /\s+/,
          begin: /:([a-z]+|\?)/
        }
      ];
      return {
        name: "Nix",
        aliases: ["nixos"],
        keywords: V,
        contains: pe.concat(ye)
      };
    }
    return nix_1 = O, nix_1;
  }
  var nodeRepl_1, hasRequiredNodeRepl;
  function requireNodeRepl() {
    if (hasRequiredNodeRepl) return nodeRepl_1;
    hasRequiredNodeRepl = 1;
    function O(_) {
      return {
        name: "Node REPL",
        contains: [
          {
            className: "meta.prompt",
            starts: {
              // a space separates the REPL prefix from the actual code
              // this is purely for cleaner HTML output
              end: / |$/,
              starts: {
                end: "$",
                subLanguage: "javascript"
              }
            },
            variants: [
              { begin: /^>(?=[ ]|$)/ },
              { begin: /^\.\.\.(?=[ ]|$)/ }
            ]
          }
        ]
      };
    }
    return nodeRepl_1 = O, nodeRepl_1;
  }
  var nsis_1, hasRequiredNsis;
  function requireNsis() {
    if (hasRequiredNsis) return nsis_1;
    hasRequiredNsis = 1;
    function O(_) {
      const H = _.regex, V = [
        "ADMINTOOLS",
        "APPDATA",
        "CDBURN_AREA",
        "CMDLINE",
        "COMMONFILES32",
        "COMMONFILES64",
        "COMMONFILES",
        "COOKIES",
        "DESKTOP",
        "DOCUMENTS",
        "EXEDIR",
        "EXEFILE",
        "EXEPATH",
        "FAVORITES",
        "FONTS",
        "HISTORY",
        "HWNDPARENT",
        "INSTDIR",
        "INTERNET_CACHE",
        "LANGUAGE",
        "LOCALAPPDATA",
        "MUSIC",
        "NETHOOD",
        "OUTDIR",
        "PICTURES",
        "PLUGINSDIR",
        "PRINTHOOD",
        "PROFILE",
        "PROGRAMFILES32",
        "PROGRAMFILES64",
        "PROGRAMFILES",
        "QUICKLAUNCH",
        "RECENT",
        "RESOURCES_LOCALIZED",
        "RESOURCES",
        "SENDTO",
        "SMPROGRAMS",
        "SMSTARTUP",
        "STARTMENU",
        "SYSDIR",
        "TEMP",
        "TEMPLATES",
        "VIDEOS",
        "WINDIR"
      ], q = [
        "ARCHIVE",
        "FILE_ATTRIBUTE_ARCHIVE",
        "FILE_ATTRIBUTE_NORMAL",
        "FILE_ATTRIBUTE_OFFLINE",
        "FILE_ATTRIBUTE_READONLY",
        "FILE_ATTRIBUTE_SYSTEM",
        "FILE_ATTRIBUTE_TEMPORARY",
        "HKCR",
        "HKCU",
        "HKDD",
        "HKEY_CLASSES_ROOT",
        "HKEY_CURRENT_CONFIG",
        "HKEY_CURRENT_USER",
        "HKEY_DYN_DATA",
        "HKEY_LOCAL_MACHINE",
        "HKEY_PERFORMANCE_DATA",
        "HKEY_USERS",
        "HKLM",
        "HKPD",
        "HKU",
        "IDABORT",
        "IDCANCEL",
        "IDIGNORE",
        "IDNO",
        "IDOK",
        "IDRETRY",
        "IDYES",
        "MB_ABORTRETRYIGNORE",
        "MB_DEFBUTTON1",
        "MB_DEFBUTTON2",
        "MB_DEFBUTTON3",
        "MB_DEFBUTTON4",
        "MB_ICONEXCLAMATION",
        "MB_ICONINFORMATION",
        "MB_ICONQUESTION",
        "MB_ICONSTOP",
        "MB_OK",
        "MB_OKCANCEL",
        "MB_RETRYCANCEL",
        "MB_RIGHT",
        "MB_RTLREADING",
        "MB_SETFOREGROUND",
        "MB_TOPMOST",
        "MB_USERICON",
        "MB_YESNO",
        "NORMAL",
        "OFFLINE",
        "READONLY",
        "SHCTX",
        "SHELL_CONTEXT",
        "SYSTEM|TEMPORARY"
      ], G = [
        "addincludedir",
        "addplugindir",
        "appendfile",
        "assert",
        "cd",
        "define",
        "delfile",
        "echo",
        "else",
        "endif",
        "error",
        "execute",
        "finalize",
        "getdllversion",
        "gettlbversion",
        "if",
        "ifdef",
        "ifmacrodef",
        "ifmacrondef",
        "ifndef",
        "include",
        "insertmacro",
        "macro",
        "macroend",
        "makensis",
        "packhdr",
        "searchparse",
        "searchreplace",
        "system",
        "tempfile",
        "undef",
        "uninstfinalize",
        "verbose",
        "warning"
      ], J = {
        className: "variable.constant",
        begin: H.concat(/\$/, H.either(...V))
      }, Z = {
        // ${defines}
        className: "variable",
        begin: /\$+\{[\!\w.:-]+\}/
      }, te = {
        // $variables
        className: "variable",
        begin: /\$+\w[\w\.]*/,
        illegal: /\(\)\{\}/
      }, ne = {
        // $(language_strings)
        className: "variable",
        begin: /\$+\([\w^.:!-]+\)/
      }, ee = {
        // command parameters
        className: "params",
        begin: H.either(...q)
      }, ae = {
        // !compiler_flags
        className: "keyword",
        begin: H.concat(
          /!/,
          H.either(...G)
        )
      }, oe = {
        // $\n, $\r, $\t, $$
        className: "char.escape",
        begin: /\$(\\[nrt]|\$)/
      }, se = {
        // plug::ins
        className: "title.function",
        begin: /\w+::\w+/
      }, ce = {
        className: "string",
        variants: [
          {
            begin: '"',
            end: '"'
          },
          {
            begin: "'",
            end: "'"
          },
          {
            begin: "`",
            end: "`"
          }
        ],
        illegal: /\n/,
        contains: [
          oe,
          J,
          Z,
          te,
          ne
        ]
      }, le = [
        "Abort",
        "AddBrandingImage",
        "AddSize",
        "AllowRootDirInstall",
        "AllowSkipFiles",
        "AutoCloseWindow",
        "BGFont",
        "BGGradient",
        "BrandingText",
        "BringToFront",
        "Call",
        "CallInstDLL",
        "Caption",
        "ChangeUI",
        "CheckBitmap",
        "ClearErrors",
        "CompletedText",
        "ComponentText",
        "CopyFiles",
        "CRCCheck",
        "CreateDirectory",
        "CreateFont",
        "CreateShortCut",
        "Delete",
        "DeleteINISec",
        "DeleteINIStr",
        "DeleteRegKey",
        "DeleteRegValue",
        "DetailPrint",
        "DetailsButtonText",
        "DirText",
        "DirVar",
        "DirVerify",
        "EnableWindow",
        "EnumRegKey",
        "EnumRegValue",
        "Exch",
        "Exec",
        "ExecShell",
        "ExecShellWait",
        "ExecWait",
        "ExpandEnvStrings",
        "File",
        "FileBufSize",
        "FileClose",
        "FileErrorText",
        "FileOpen",
        "FileRead",
        "FileReadByte",
        "FileReadUTF16LE",
        "FileReadWord",
        "FileWriteUTF16LE",
        "FileSeek",
        "FileWrite",
        "FileWriteByte",
        "FileWriteWord",
        "FindClose",
        "FindFirst",
        "FindNext",
        "FindWindow",
        "FlushINI",
        "GetCurInstType",
        "GetCurrentAddress",
        "GetDlgItem",
        "GetDLLVersion",
        "GetDLLVersionLocal",
        "GetErrorLevel",
        "GetFileTime",
        "GetFileTimeLocal",
        "GetFullPathName",
        "GetFunctionAddress",
        "GetInstDirError",
        "GetKnownFolderPath",
        "GetLabelAddress",
        "GetTempFileName",
        "GetWinVer",
        "Goto",
        "HideWindow",
        "Icon",
        "IfAbort",
        "IfErrors",
        "IfFileExists",
        "IfRebootFlag",
        "IfRtlLanguage",
        "IfShellVarContextAll",
        "IfSilent",
        "InitPluginsDir",
        "InstallButtonText",
        "InstallColors",
        "InstallDir",
        "InstallDirRegKey",
        "InstProgressFlags",
        "InstType",
        "InstTypeGetText",
        "InstTypeSetText",
        "Int64Cmp",
        "Int64CmpU",
        "Int64Fmt",
        "IntCmp",
        "IntCmpU",
        "IntFmt",
        "IntOp",
        "IntPtrCmp",
        "IntPtrCmpU",
        "IntPtrOp",
        "IsWindow",
        "LangString",
        "LicenseBkColor",
        "LicenseData",
        "LicenseForceSelection",
        "LicenseLangString",
        "LicenseText",
        "LoadAndSetImage",
        "LoadLanguageFile",
        "LockWindow",
        "LogSet",
        "LogText",
        "ManifestDPIAware",
        "ManifestLongPathAware",
        "ManifestMaxVersionTested",
        "ManifestSupportedOS",
        "MessageBox",
        "MiscButtonText",
        "Name|0",
        "Nop",
        "OutFile",
        "Page",
        "PageCallbacks",
        "PEAddResource",
        "PEDllCharacteristics",
        "PERemoveResource",
        "PESubsysVer",
        "Pop",
        "Push",
        "Quit",
        "ReadEnvStr",
        "ReadINIStr",
        "ReadRegDWORD",
        "ReadRegStr",
        "Reboot",
        "RegDLL",
        "Rename",
        "RequestExecutionLevel",
        "ReserveFile",
        "Return",
        "RMDir",
        "SearchPath",
        "SectionGetFlags",
        "SectionGetInstTypes",
        "SectionGetSize",
        "SectionGetText",
        "SectionIn",
        "SectionSetFlags",
        "SectionSetInstTypes",
        "SectionSetSize",
        "SectionSetText",
        "SendMessage",
        "SetAutoClose",
        "SetBrandingImage",
        "SetCompress",
        "SetCompressor",
        "SetCompressorDictSize",
        "SetCtlColors",
        "SetCurInstType",
        "SetDatablockOptimize",
        "SetDateSave",
        "SetDetailsPrint",
        "SetDetailsView",
        "SetErrorLevel",
        "SetErrors",
        "SetFileAttributes",
        "SetFont",
        "SetOutPath",
        "SetOverwrite",
        "SetRebootFlag",
        "SetRegView",
        "SetShellVarContext",
        "SetSilent",
        "ShowInstDetails",
        "ShowUninstDetails",
        "ShowWindow",
        "SilentInstall",
        "SilentUnInstall",
        "Sleep",
        "SpaceTexts",
        "StrCmp",
        "StrCmpS",
        "StrCpy",
        "StrLen",
        "SubCaption",
        "Unicode",
        "UninstallButtonText",
        "UninstallCaption",
        "UninstallIcon",
        "UninstallSubCaption",
        "UninstallText",
        "UninstPage",
        "UnRegDLL",
        "Var",
        "VIAddVersionKey",
        "VIFileVersion",
        "VIProductVersion",
        "WindowIcon",
        "WriteINIStr",
        "WriteRegBin",
        "WriteRegDWORD",
        "WriteRegExpandStr",
        "WriteRegMultiStr",
        "WriteRegNone",
        "WriteRegStr",
        "WriteUninstaller",
        "XPStyle"
      ], ue = [
        "admin",
        "all",
        "auto",
        "both",
        "bottom",
        "bzip2",
        "colored",
        "components",
        "current",
        "custom",
        "directory",
        "false",
        "force",
        "hide",
        "highest",
        "ifdiff",
        "ifnewer",
        "instfiles",
        "lastused",
        "leave",
        "left",
        "license",
        "listonly",
        "lzma",
        "nevershow",
        "none",
        "normal",
        "notset",
        "off",
        "on",
        "open",
        "print",
        "right",
        "show",
        "silent",
        "silentlog",
        "smooth",
        "textonly",
        "top",
        "true",
        "try",
        "un.components",
        "un.custom",
        "un.directory",
        "un.instfiles",
        "un.license",
        "uninstConfirm",
        "user",
        "Win10",
        "Win7",
        "Win8",
        "WinVista",
        "zlib"
      ], de = {
        match: [
          /Function/,
          /\s+/,
          H.concat(/(\.)?/, _.IDENT_RE)
        ],
        scope: {
          1: "keyword",
          3: "title.function"
        }
      }, he = {
        match: [
          /Var/,
          /\s+/,
          /(?:\/GLOBAL\s+)?/,
          /[A-Za-z][\w.]*/
        ],
        scope: {
          1: "keyword",
          3: "params",
          4: "variable"
        }
      };
      return {
        name: "NSIS",
        case_insensitive: !0,
        keywords: {
          keyword: le,
          literal: ue
        },
        contains: [
          _.HASH_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.COMMENT(
            ";",
            "$",
            { relevance: 0 }
          ),
          he,
          de,
          { beginKeywords: "Function PageEx Section SectionGroup FunctionEnd SectionEnd" },
          ce,
          ae,
          Z,
          te,
          ne,
          ee,
          se,
          _.NUMBER_MODE
        ]
      };
    }
    return nsis_1 = O, nsis_1;
  }
  var ocaml_1, hasRequiredOcaml;
  function requireOcaml() {
    if (hasRequiredOcaml) return ocaml_1;
    hasRequiredOcaml = 1;
    function O(_) {
      return {
        name: "OCaml",
        aliases: ["ml"],
        keywords: {
          $pattern: "[a-z_]\\w*!?",
          keyword: "and as assert asr begin class constraint do done downto else end exception external for fun function functor if in include inherit! inherit initializer land lazy let lor lsl lsr lxor match method!|10 method mod module mutable new object of open! open or private rec sig struct then to try type val! val virtual when while with parser value",
          built_in: (
            /* built-in types */
            "array bool bytes char exn|5 float int int32 int64 list lazy_t|5 nativeint|5 string unit in_channel out_channel ref"
          ),
          literal: "true false"
        },
        illegal: /\/\/|>>/,
        contains: [
          {
            className: "literal",
            begin: "\\[(\\|\\|)?\\]|\\(\\)",
            relevance: 0
          },
          _.COMMENT(
            "\\(\\*",
            "\\*\\)",
            { contains: ["self"] }
          ),
          {
            /* type variable */
            className: "symbol",
            begin: "'[A-Za-z_](?!')[\\w']*"
            /* the grammar is ambiguous on how 'a'b should be interpreted but not the compiler */
          },
          {
            /* polymorphic variant */
            className: "type",
            begin: "`[A-Z][\\w']*"
          },
          {
            /* module or constructor */
            className: "type",
            begin: "\\b[A-Z][\\w']*",
            relevance: 0
          },
          {
            /* don't color identifiers, but safely catch all identifiers with ' */
            begin: "[a-z_]\\w*'[\\w']*",
            relevance: 0
          },
          _.inherit(_.APOS_STRING_MODE, {
            className: "string",
            relevance: 0
          }),
          _.inherit(_.QUOTE_STRING_MODE, { illegal: null }),
          {
            className: "number",
            begin: "\\b(0[xX][a-fA-F0-9_]+[Lln]?|0[oO][0-7_]+[Lln]?|0[bB][01_]+[Lln]?|[0-9][0-9_]*([Lln]|(\\.[0-9_]*)?([eE][-+]?[0-9_]+)?)?)",
            relevance: 0
          },
          {
            begin: /->/
            // relevance booster
          }
        ]
      };
    }
    return ocaml_1 = O, ocaml_1;
  }
  var openscad_1, hasRequiredOpenscad;
  function requireOpenscad() {
    if (hasRequiredOpenscad) return openscad_1;
    hasRequiredOpenscad = 1;
    function O(_) {
      const H = {
        className: "keyword",
        begin: "\\$(f[asn]|t|vp[rtd]|children)"
      }, V = {
        className: "literal",
        begin: "false|true|PI|undef"
      }, q = {
        className: "number",
        begin: "\\b\\d+(\\.\\d+)?(e-?\\d+)?",
        // adds 1e5, 1e-10
        relevance: 0
      }, G = _.inherit(_.QUOTE_STRING_MODE, { illegal: null }), J = {
        className: "meta",
        keywords: { keyword: "include use" },
        begin: "include|use <",
        end: ">"
      }, Z = {
        className: "params",
        begin: "\\(",
        end: "\\)",
        contains: [
          "self",
          q,
          G,
          H,
          V
        ]
      }, te = {
        begin: "[*!#%]",
        relevance: 0
      }, ne = {
        className: "function",
        beginKeywords: "module function",
        end: /=|\{/,
        contains: [
          Z,
          _.UNDERSCORE_TITLE_MODE
        ]
      };
      return {
        name: "OpenSCAD",
        aliases: ["scad"],
        keywords: {
          keyword: "function module include use for intersection_for if else \\%",
          literal: "false true PI undef",
          built_in: "circle square polygon text sphere cube cylinder polyhedron translate rotate scale resize mirror multmatrix color offset hull minkowski union difference intersection abs sign sin cos tan acos asin atan atan2 floor round ceil ln log pow sqrt exp rands min max concat lookup str chr search version version_num norm cross parent_module echo import import_dxf dxf_linear_extrude linear_extrude rotate_extrude surface projection render children dxf_cross dxf_dim let assign"
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          q,
          J,
          G,
          H,
          te,
          ne
        ]
      };
    }
    return openscad_1 = O, openscad_1;
  }
  var oxygene_1, hasRequiredOxygene;
  function requireOxygene() {
    if (hasRequiredOxygene) return oxygene_1;
    hasRequiredOxygene = 1;
    function O(_) {
      const H = {
        $pattern: /\.?\w+/,
        keyword: "abstract add and array as asc aspect assembly async begin break block by case class concat const copy constructor continue create default delegate desc distinct div do downto dynamic each else empty end ensure enum equals event except exit extension external false final finalize finalizer finally flags for forward from function future global group has if implementation implements implies in index inherited inline interface into invariants is iterator join locked locking loop matching method mod module namespace nested new nil not notify nullable of old on operator or order out override parallel params partial pinned private procedure property protected public queryable raise read readonly record reintroduce remove repeat require result reverse sealed select self sequence set shl shr skip static step soft take then to true try tuple type union unit unsafe until uses using var virtual raises volatile where while with write xor yield await mapped deprecated stdcall cdecl pascal register safecall overload library platform reference packed strict published autoreleasepool selector strong weak unretained"
      }, V = _.COMMENT(
        /\{/,
        /\}/,
        { relevance: 0 }
      ), q = _.COMMENT(
        "\\(\\*",
        "\\*\\)",
        { relevance: 10 }
      ), G = {
        className: "string",
        begin: "'",
        end: "'",
        contains: [{ begin: "''" }]
      }, J = {
        className: "string",
        begin: "(#\\d+)+"
      }, Z = {
        beginKeywords: "function constructor destructor procedure method",
        end: "[:;]",
        keywords: "function constructor|10 destructor|10 procedure|10 method|10",
        contains: [
          _.inherit(_.TITLE_MODE, { scope: "title.function" }),
          {
            className: "params",
            begin: "\\(",
            end: "\\)",
            keywords: H,
            contains: [
              G,
              J
            ]
          },
          V,
          q
        ]
      }, te = {
        scope: "punctuation",
        match: /;/,
        relevance: 0
      };
      return {
        name: "Oxygene",
        case_insensitive: !0,
        keywords: H,
        illegal: '("|\\$[G-Zg-z]|\\/\\*|</|=>|->)',
        contains: [
          V,
          q,
          _.C_LINE_COMMENT_MODE,
          G,
          J,
          _.NUMBER_MODE,
          Z,
          te
        ]
      };
    }
    return oxygene_1 = O, oxygene_1;
  }
  var parser3_1, hasRequiredParser3;
  function requireParser3() {
    if (hasRequiredParser3) return parser3_1;
    hasRequiredParser3 = 1;
    function O(_) {
      const H = _.COMMENT(
        /\{/,
        /\}/,
        { contains: ["self"] }
      );
      return {
        name: "Parser3",
        subLanguage: "xml",
        relevance: 0,
        contains: [
          _.COMMENT("^#", "$"),
          _.COMMENT(
            /\^rem\{/,
            /\}/,
            {
              relevance: 10,
              contains: [H]
            }
          ),
          {
            className: "meta",
            begin: "^@(?:BASE|USE|CLASS|OPTIONS)$",
            relevance: 10
          },
          {
            className: "title",
            begin: "@[\\w\\-]+\\[[\\w^;\\-]*\\](?:\\[[\\w^;\\-]*\\])?(?:.*)$"
          },
          {
            className: "variable",
            begin: /\$\{?[\w\-.:]+\}?/
          },
          {
            className: "keyword",
            begin: /\^[\w\-.:]+/
          },
          {
            className: "number",
            begin: "\\^#[0-9a-fA-F]+"
          },
          _.C_NUMBER_MODE
        ]
      };
    }
    return parser3_1 = O, parser3_1;
  }
  var pf_1, hasRequiredPf;
  function requirePf() {
    if (hasRequiredPf) return pf_1;
    hasRequiredPf = 1;
    function O(_) {
      const H = {
        className: "variable",
        begin: /\$[\w\d#@][\w\d_]*/,
        relevance: 0
      }, V = {
        className: "variable",
        begin: /<(?!\/)/,
        end: />/
      };
      return {
        name: "Packet Filter config",
        aliases: ["pf.conf"],
        keywords: {
          $pattern: /[a-z0-9_<>-]+/,
          built_in: (
            /* block match pass are "actions" in pf.conf(5), the rest are
             * lexically similar top-level commands.
             */
            "block match pass load anchor|5 antispoof|10 set table"
          ),
          keyword: "in out log quick on rdomain inet inet6 proto from port os to route allow-opts divert-packet divert-reply divert-to flags group icmp-type icmp6-type label once probability recieved-on rtable prio queue tos tag tagged user keep fragment for os drop af-to|10 binat-to|10 nat-to|10 rdr-to|10 bitmask least-stats random round-robin source-hash static-port dup-to reply-to route-to parent bandwidth default min max qlimit block-policy debug fingerprints hostid limit loginterface optimization reassemble ruleset-optimization basic none profile skip state-defaults state-policy timeout const counters persist no modulate synproxy state|5 floating if-bound no-sync pflow|10 sloppy source-track global rule max-src-nodes max-src-states max-src-conn max-src-conn-rate overload flush scrub|5 max-mss min-ttl no-df|10 random-id",
          literal: "all any no-route self urpf-failed egress|5 unknown"
        },
        contains: [
          _.HASH_COMMENT_MODE,
          _.NUMBER_MODE,
          _.QUOTE_STRING_MODE,
          H,
          V
        ]
      };
    }
    return pf_1 = O, pf_1;
  }
  var pgsql_1, hasRequiredPgsql;
  function requirePgsql() {
    if (hasRequiredPgsql) return pgsql_1;
    hasRequiredPgsql = 1;
    function O(_) {
      const H = _.COMMENT("--", "$"), V = "[a-zA-Z_][a-zA-Z_0-9$]*", q = "\\$([a-zA-Z_]?|[a-zA-Z_][a-zA-Z_0-9]*)\\$", G = "<<\\s*" + V + "\\s*>>", J = (
        // https://www.postgresql.org/docs/11/static/sql-keywords-appendix.html
        // https://www.postgresql.org/docs/11/static/sql-commands.html
        // SQL commands (starting words)
        "ABORT ALTER ANALYZE BEGIN CALL CHECKPOINT|10 CLOSE CLUSTER COMMENT COMMIT COPY CREATE DEALLOCATE DECLARE DELETE DISCARD DO DROP END EXECUTE EXPLAIN FETCH GRANT IMPORT INSERT LISTEN LOAD LOCK MOVE NOTIFY PREPARE REASSIGN|10 REFRESH REINDEX RELEASE RESET REVOKE ROLLBACK SAVEPOINT SECURITY SELECT SET SHOW START TRUNCATE UNLISTEN|10 UPDATE VACUUM|10 VALUES AGGREGATE COLLATION CONVERSION|10 DATABASE DEFAULT PRIVILEGES DOMAIN TRIGGER EXTENSION FOREIGN WRAPPER|10 TABLE FUNCTION GROUP LANGUAGE LARGE OBJECT MATERIALIZED VIEW OPERATOR CLASS FAMILY POLICY PUBLICATION|10 ROLE RULE SCHEMA SEQUENCE SERVER STATISTICS SUBSCRIPTION SYSTEM TABLESPACE CONFIGURATION DICTIONARY PARSER TEMPLATE TYPE USER MAPPING PREPARED ACCESS METHOD CAST AS TRANSFORM TRANSACTION OWNED TO INTO SESSION AUTHORIZATION INDEX PROCEDURE ASSERTION ALL ANALYSE AND ANY ARRAY ASC ASYMMETRIC|10 BOTH CASE CHECK COLLATE COLUMN CONCURRENTLY|10 CONSTRAINT CROSS DEFERRABLE RANGE DESC DISTINCT ELSE EXCEPT FOR FREEZE|10 FROM FULL HAVING ILIKE IN INITIALLY INNER INTERSECT IS ISNULL JOIN LATERAL LEADING LIKE LIMIT NATURAL NOT NOTNULL NULL OFFSET ON ONLY OR ORDER OUTER OVERLAPS PLACING PRIMARY REFERENCES RETURNING SIMILAR SOME SYMMETRIC TABLESAMPLE THEN TRAILING UNION UNIQUE USING VARIADIC|10 VERBOSE WHEN WHERE WINDOW WITH BY RETURNS INOUT OUT SETOF|10 IF STRICT CURRENT CONTINUE OWNER LOCATION OVER PARTITION WITHIN BETWEEN ESCAPE EXTERNAL INVOKER DEFINER WORK RENAME VERSION CONNECTION CONNECT TABLES TEMP TEMPORARY FUNCTIONS SEQUENCES TYPES SCHEMAS OPTION CASCADE RESTRICT ADD ADMIN EXISTS VALID VALIDATE ENABLE DISABLE REPLICA|10 ALWAYS PASSING COLUMNS PATH REF VALUE OVERRIDING IMMUTABLE STABLE VOLATILE BEFORE AFTER EACH ROW PROCEDURAL ROUTINE NO HANDLER VALIDATOR OPTIONS STORAGE OIDS|10 WITHOUT INHERIT DEPENDS CALLED INPUT LEAKPROOF|10 COST ROWS NOWAIT SEARCH UNTIL ENCRYPTED|10 PASSWORD CONFLICT|10 INSTEAD INHERITS CHARACTERISTICS WRITE CURSOR ALSO STATEMENT SHARE EXCLUSIVE INLINE ISOLATION REPEATABLE READ COMMITTED SERIALIZABLE UNCOMMITTED LOCAL GLOBAL SQL PROCEDURES RECURSIVE SNAPSHOT ROLLUP CUBE TRUSTED|10 INCLUDE FOLLOWING PRECEDING UNBOUNDED RANGE GROUPS UNENCRYPTED|10 SYSID FORMAT DELIMITER HEADER QUOTE ENCODING FILTER OFF FORCE_QUOTE FORCE_NOT_NULL FORCE_NULL COSTS BUFFERS TIMING SUMMARY DISABLE_PAGE_SKIPPING RESTART CYCLE GENERATED IDENTITY DEFERRED IMMEDIATE LEVEL LOGGED UNLOGGED OF NOTHING NONE EXCLUDE ATTRIBUTE USAGE ROUTINES TRUE FALSE NAN INFINITY "
      ), Z = (
        // only those not in keywrods already
        "SUPERUSER NOSUPERUSER CREATEDB NOCREATEDB CREATEROLE NOCREATEROLE INHERIT NOINHERIT LOGIN NOLOGIN REPLICATION NOREPLICATION BYPASSRLS NOBYPASSRLS "
      ), te = "ALIAS BEGIN CONSTANT DECLARE END EXCEPTION RETURN PERFORM|10 RAISE GET DIAGNOSTICS STACKED|10 FOREACH LOOP ELSIF EXIT WHILE REVERSE SLICE DEBUG LOG INFO NOTICE WARNING ASSERT OPEN ", ne = (
        // https://www.postgresql.org/docs/11/static/datatype.html
        "BIGINT INT8 BIGSERIAL SERIAL8 BIT VARYING VARBIT BOOLEAN BOOL BOX BYTEA CHARACTER CHAR VARCHAR CIDR CIRCLE DATE DOUBLE PRECISION FLOAT8 FLOAT INET INTEGER INT INT4 INTERVAL JSON JSONB LINE LSEG|10 MACADDR MACADDR8 MONEY NUMERIC DEC DECIMAL PATH POINT POLYGON REAL FLOAT4 SMALLINT INT2 SMALLSERIAL|10 SERIAL2|10 SERIAL|10 SERIAL4|10 TEXT TIME ZONE TIMETZ|10 TIMESTAMP TIMESTAMPTZ|10 TSQUERY|10 TSVECTOR|10 TXID_SNAPSHOT|10 UUID XML NATIONAL NCHAR INT4RANGE|10 INT8RANGE|10 NUMRANGE|10 TSRANGE|10 TSTZRANGE|10 DATERANGE|10 ANYELEMENT ANYARRAY ANYNONARRAY ANYENUM ANYRANGE CSTRING INTERNAL RECORD PG_DDL_COMMAND VOID UNKNOWN OPAQUE REFCURSOR NAME OID REGPROC|10 REGPROCEDURE|10 REGOPER|10 REGOPERATOR|10 REGCLASS|10 REGTYPE|10 REGROLE|10 REGNAMESPACE|10 REGCONFIG|10 REGDICTIONARY|10 "
      ), ee = ne.trim().split(" ").map(function(ue) {
        return ue.split("|")[0];
      }).join("|"), ae = "CURRENT_TIME CURRENT_TIMESTAMP CURRENT_USER CURRENT_CATALOG|10 CURRENT_DATE LOCALTIME LOCALTIMESTAMP CURRENT_ROLE|10 CURRENT_SCHEMA|10 SESSION_USER PUBLIC ", oe = "FOUND NEW OLD TG_NAME|10 TG_WHEN|10 TG_LEVEL|10 TG_OP|10 TG_RELID|10 TG_RELNAME|10 TG_TABLE_NAME|10 TG_TABLE_SCHEMA|10 TG_NARGS|10 TG_ARGV|10 TG_EVENT|10 TG_TAG|10 ROW_COUNT RESULT_OID|10 PG_CONTEXT|10 RETURNED_SQLSTATE COLUMN_NAME CONSTRAINT_NAME PG_DATATYPE_NAME|10 MESSAGE_TEXT TABLE_NAME SCHEMA_NAME PG_EXCEPTION_DETAIL|10 PG_EXCEPTION_HINT|10 PG_EXCEPTION_CONTEXT|10 ", se = (
        // exceptions https://www.postgresql.org/docs/current/static/errcodes-appendix.html
        "SQLSTATE SQLERRM|10 SUCCESSFUL_COMPLETION WARNING DYNAMIC_RESULT_SETS_RETURNED IMPLICIT_ZERO_BIT_PADDING NULL_VALUE_ELIMINATED_IN_SET_FUNCTION PRIVILEGE_NOT_GRANTED PRIVILEGE_NOT_REVOKED STRING_DATA_RIGHT_TRUNCATION DEPRECATED_FEATURE NO_DATA NO_ADDITIONAL_DYNAMIC_RESULT_SETS_RETURNED SQL_STATEMENT_NOT_YET_COMPLETE CONNECTION_EXCEPTION CONNECTION_DOES_NOT_EXIST CONNECTION_FAILURE SQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION SQLSERVER_REJECTED_ESTABLISHMENT_OF_SQLCONNECTION TRANSACTION_RESOLUTION_UNKNOWN PROTOCOL_VIOLATION TRIGGERED_ACTION_EXCEPTION FEATURE_NOT_SUPPORTED INVALID_TRANSACTION_INITIATION LOCATOR_EXCEPTION INVALID_LOCATOR_SPECIFICATION INVALID_GRANTOR INVALID_GRANT_OPERATION INVALID_ROLE_SPECIFICATION DIAGNOSTICS_EXCEPTION STACKED_DIAGNOSTICS_ACCESSED_WITHOUT_ACTIVE_HANDLER CASE_NOT_FOUND CARDINALITY_VIOLATION DATA_EXCEPTION ARRAY_SUBSCRIPT_ERROR CHARACTER_NOT_IN_REPERTOIRE DATETIME_FIELD_OVERFLOW DIVISION_BY_ZERO ERROR_IN_ASSIGNMENT ESCAPE_CHARACTER_CONFLICT INDICATOR_OVERFLOW INTERVAL_FIELD_OVERFLOW INVALID_ARGUMENT_FOR_LOGARITHM INVALID_ARGUMENT_FOR_NTILE_FUNCTION INVALID_ARGUMENT_FOR_NTH_VALUE_FUNCTION INVALID_ARGUMENT_FOR_POWER_FUNCTION INVALID_ARGUMENT_FOR_WIDTH_BUCKET_FUNCTION INVALID_CHARACTER_VALUE_FOR_CAST INVALID_DATETIME_FORMAT INVALID_ESCAPE_CHARACTER INVALID_ESCAPE_OCTET INVALID_ESCAPE_SEQUENCE NONSTANDARD_USE_OF_ESCAPE_CHARACTER INVALID_INDICATOR_PARAMETER_VALUE INVALID_PARAMETER_VALUE INVALID_REGULAR_EXPRESSION INVALID_ROW_COUNT_IN_LIMIT_CLAUSE INVALID_ROW_COUNT_IN_RESULT_OFFSET_CLAUSE INVALID_TABLESAMPLE_ARGUMENT INVALID_TABLESAMPLE_REPEAT INVALID_TIME_ZONE_DISPLACEMENT_VALUE INVALID_USE_OF_ESCAPE_CHARACTER MOST_SPECIFIC_TYPE_MISMATCH NULL_VALUE_NOT_ALLOWED NULL_VALUE_NO_INDICATOR_PARAMETER NUMERIC_VALUE_OUT_OF_RANGE SEQUENCE_GENERATOR_LIMIT_EXCEEDED STRING_DATA_LENGTH_MISMATCH STRING_DATA_RIGHT_TRUNCATION SUBSTRING_ERROR TRIM_ERROR UNTERMINATED_C_STRING ZERO_LENGTH_CHARACTER_STRING FLOATING_POINT_EXCEPTION INVALID_TEXT_REPRESENTATION INVALID_BINARY_REPRESENTATION BAD_COPY_FILE_FORMAT UNTRANSLATABLE_CHARACTER NOT_AN_XML_DOCUMENT INVALID_XML_DOCUMENT INVALID_XML_CONTENT INVALID_XML_COMMENT INVALID_XML_PROCESSING_INSTRUCTION INTEGRITY_CONSTRAINT_VIOLATION RESTRICT_VIOLATION NOT_NULL_VIOLATION FOREIGN_KEY_VIOLATION UNIQUE_VIOLATION CHECK_VIOLATION EXCLUSION_VIOLATION INVALID_CURSOR_STATE INVALID_TRANSACTION_STATE ACTIVE_SQL_TRANSACTION BRANCH_TRANSACTION_ALREADY_ACTIVE HELD_CURSOR_REQUIRES_SAME_ISOLATION_LEVEL INAPPROPRIATE_ACCESS_MODE_FOR_BRANCH_TRANSACTION INAPPROPRIATE_ISOLATION_LEVEL_FOR_BRANCH_TRANSACTION NO_ACTIVE_SQL_TRANSACTION_FOR_BRANCH_TRANSACTION READ_ONLY_SQL_TRANSACTION SCHEMA_AND_DATA_STATEMENT_MIXING_NOT_SUPPORTED NO_ACTIVE_SQL_TRANSACTION IN_FAILED_SQL_TRANSACTION IDLE_IN_TRANSACTION_SESSION_TIMEOUT INVALID_SQL_STATEMENT_NAME TRIGGERED_DATA_CHANGE_VIOLATION INVALID_AUTHORIZATION_SPECIFICATION INVALID_PASSWORD DEPENDENT_PRIVILEGE_DESCRIPTORS_STILL_EXIST DEPENDENT_OBJECTS_STILL_EXIST INVALID_TRANSACTION_TERMINATION SQL_ROUTINE_EXCEPTION FUNCTION_EXECUTED_NO_RETURN_STATEMENT MODIFYING_SQL_DATA_NOT_PERMITTED PROHIBITED_SQL_STATEMENT_ATTEMPTED READING_SQL_DATA_NOT_PERMITTED INVALID_CURSOR_NAME EXTERNAL_ROUTINE_EXCEPTION CONTAINING_SQL_NOT_PERMITTED MODIFYING_SQL_DATA_NOT_PERMITTED PROHIBITED_SQL_STATEMENT_ATTEMPTED READING_SQL_DATA_NOT_PERMITTED EXTERNAL_ROUTINE_INVOCATION_EXCEPTION INVALID_SQLSTATE_RETURNED NULL_VALUE_NOT_ALLOWED TRIGGER_PROTOCOL_VIOLATED SRF_PROTOCOL_VIOLATED EVENT_TRIGGER_PROTOCOL_VIOLATED SAVEPOINT_EXCEPTION INVALID_SAVEPOINT_SPECIFICATION INVALID_CATALOG_NAME INVALID_SCHEMA_NAME TRANSACTION_ROLLBACK TRANSACTION_INTEGRITY_CONSTRAINT_VIOLATION SERIALIZATION_FAILURE STATEMENT_COMPLETION_UNKNOWN DEADLOCK_DETECTED SYNTAX_ERROR_OR_ACCESS_RULE_VIOLATION SYNTAX_ERROR INSUFFICIENT_PRIVILEGE CANNOT_COERCE GROUPING_ERROR WINDOWING_ERROR INVALID_RECURSION INVALID_FOREIGN_KEY INVALID_NAME NAME_TOO_LONG RESERVED_NAME DATATYPE_MISMATCH INDETERMINATE_DATATYPE COLLATION_MISMATCH INDETERMINATE_COLLATION WRONG_OBJECT_TYPE GENERATED_ALWAYS UNDEFINED_COLUMN UNDEFINED_FUNCTION UNDEFINED_TABLE UNDEFINED_PARAMETER UNDEFINED_OBJECT DUPLICATE_COLUMN DUPLICATE_CURSOR DUPLICATE_DATABASE DUPLICATE_FUNCTION DUPLICATE_PREPARED_STATEMENT DUPLICATE_SCHEMA DUPLICATE_TABLE DUPLICATE_ALIAS DUPLICATE_OBJECT AMBIGUOUS_COLUMN AMBIGUOUS_FUNCTION AMBIGUOUS_PARAMETER AMBIGUOUS_ALIAS INVALID_COLUMN_REFERENCE INVALID_COLUMN_DEFINITION INVALID_CURSOR_DEFINITION INVALID_DATABASE_DEFINITION INVALID_FUNCTION_DEFINITION INVALID_PREPARED_STATEMENT_DEFINITION INVALID_SCHEMA_DEFINITION INVALID_TABLE_DEFINITION INVALID_OBJECT_DEFINITION WITH_CHECK_OPTION_VIOLATION INSUFFICIENT_RESOURCES DISK_FULL OUT_OF_MEMORY TOO_MANY_CONNECTIONS CONFIGURATION_LIMIT_EXCEEDED PROGRAM_LIMIT_EXCEEDED STATEMENT_TOO_COMPLEX TOO_MANY_COLUMNS TOO_MANY_ARGUMENTS OBJECT_NOT_IN_PREREQUISITE_STATE OBJECT_IN_USE CANT_CHANGE_RUNTIME_PARAM LOCK_NOT_AVAILABLE OPERATOR_INTERVENTION QUERY_CANCELED ADMIN_SHUTDOWN CRASH_SHUTDOWN CANNOT_CONNECT_NOW DATABASE_DROPPED SYSTEM_ERROR IO_ERROR UNDEFINED_FILE DUPLICATE_FILE SNAPSHOT_TOO_OLD CONFIG_FILE_ERROR LOCK_FILE_EXISTS FDW_ERROR FDW_COLUMN_NAME_NOT_FOUND FDW_DYNAMIC_PARAMETER_VALUE_NEEDED FDW_FUNCTION_SEQUENCE_ERROR FDW_INCONSISTENT_DESCRIPTOR_INFORMATION FDW_INVALID_ATTRIBUTE_VALUE FDW_INVALID_COLUMN_NAME FDW_INVALID_COLUMN_NUMBER FDW_INVALID_DATA_TYPE FDW_INVALID_DATA_TYPE_DESCRIPTORS FDW_INVALID_DESCRIPTOR_FIELD_IDENTIFIER FDW_INVALID_HANDLE FDW_INVALID_OPTION_INDEX FDW_INVALID_OPTION_NAME FDW_INVALID_STRING_LENGTH_OR_BUFFER_LENGTH FDW_INVALID_STRING_FORMAT FDW_INVALID_USE_OF_NULL_POINTER FDW_TOO_MANY_HANDLES FDW_OUT_OF_MEMORY FDW_NO_SCHEMAS FDW_OPTION_NAME_NOT_FOUND FDW_REPLY_HANDLE FDW_SCHEMA_NOT_FOUND FDW_TABLE_NOT_FOUND FDW_UNABLE_TO_CREATE_EXECUTION FDW_UNABLE_TO_CREATE_REPLY FDW_UNABLE_TO_ESTABLISH_CONNECTION PLPGSQL_ERROR RAISE_EXCEPTION NO_DATA_FOUND TOO_MANY_ROWS ASSERT_FAILURE INTERNAL_ERROR DATA_CORRUPTED INDEX_CORRUPTED "
      ), le = /* https://www.postgresql.org/docs/11/static/functions-aggregate.html */ "ARRAY_AGG AVG BIT_AND BIT_OR BOOL_AND BOOL_OR COUNT EVERY JSON_AGG JSONB_AGG JSON_OBJECT_AGG JSONB_OBJECT_AGG MAX MIN MODE STRING_AGG SUM XMLAGG CORR COVAR_POP COVAR_SAMP REGR_AVGX REGR_AVGY REGR_COUNT REGR_INTERCEPT REGR_R2 REGR_SLOPE REGR_SXX REGR_SXY REGR_SYY STDDEV STDDEV_POP STDDEV_SAMP VARIANCE VAR_POP VAR_SAMP PERCENTILE_CONT PERCENTILE_DISC ROW_NUMBER RANK DENSE_RANK PERCENT_RANK CUME_DIST NTILE LAG LEAD FIRST_VALUE LAST_VALUE NTH_VALUE NUM_NONNULLS NUM_NULLS ABS CBRT CEIL CEILING DEGREES DIV EXP FLOOR LN LOG MOD PI POWER RADIANS ROUND SCALE SIGN SQRT TRUNC WIDTH_BUCKET RANDOM SETSEED ACOS ACOSD ASIN ASIND ATAN ATAND ATAN2 ATAN2D COS COSD COT COTD SIN SIND TAN TAND BIT_LENGTH CHAR_LENGTH CHARACTER_LENGTH LOWER OCTET_LENGTH OVERLAY POSITION SUBSTRING TREAT TRIM UPPER ASCII BTRIM CHR CONCAT CONCAT_WS CONVERT CONVERT_FROM CONVERT_TO DECODE ENCODE INITCAP LEFT LENGTH LPAD LTRIM MD5 PARSE_IDENT PG_CLIENT_ENCODING QUOTE_IDENT|10 QUOTE_LITERAL|10 QUOTE_NULLABLE|10 REGEXP_MATCH REGEXP_MATCHES REGEXP_REPLACE REGEXP_SPLIT_TO_ARRAY REGEXP_SPLIT_TO_TABLE REPEAT REPLACE REVERSE RIGHT RPAD RTRIM SPLIT_PART STRPOS SUBSTR TO_ASCII TO_HEX TRANSLATE OCTET_LENGTH GET_BIT GET_BYTE SET_BIT SET_BYTE TO_CHAR TO_DATE TO_NUMBER TO_TIMESTAMP AGE CLOCK_TIMESTAMP|10 DATE_PART DATE_TRUNC ISFINITE JUSTIFY_DAYS JUSTIFY_HOURS JUSTIFY_INTERVAL MAKE_DATE MAKE_INTERVAL|10 MAKE_TIME MAKE_TIMESTAMP|10 MAKE_TIMESTAMPTZ|10 NOW STATEMENT_TIMESTAMP|10 TIMEOFDAY TRANSACTION_TIMESTAMP|10 ENUM_FIRST ENUM_LAST ENUM_RANGE AREA CENTER DIAMETER HEIGHT ISCLOSED ISOPEN NPOINTS PCLOSE POPEN RADIUS WIDTH BOX BOUND_BOX CIRCLE LINE LSEG PATH POLYGON ABBREV BROADCAST HOST HOSTMASK MASKLEN NETMASK NETWORK SET_MASKLEN TEXT INET_SAME_FAMILY INET_MERGE MACADDR8_SET7BIT ARRAY_TO_TSVECTOR GET_CURRENT_TS_CONFIG NUMNODE PLAINTO_TSQUERY PHRASETO_TSQUERY WEBSEARCH_TO_TSQUERY QUERYTREE SETWEIGHT STRIP TO_TSQUERY TO_TSVECTOR JSON_TO_TSVECTOR JSONB_TO_TSVECTOR TS_DELETE TS_FILTER TS_HEADLINE TS_RANK TS_RANK_CD TS_REWRITE TSQUERY_PHRASE TSVECTOR_TO_ARRAY TSVECTOR_UPDATE_TRIGGER TSVECTOR_UPDATE_TRIGGER_COLUMN XMLCOMMENT XMLCONCAT XMLELEMENT XMLFOREST XMLPI XMLROOT XMLEXISTS XML_IS_WELL_FORMED XML_IS_WELL_FORMED_DOCUMENT XML_IS_WELL_FORMED_CONTENT XPATH XPATH_EXISTS XMLTABLE XMLNAMESPACES TABLE_TO_XML TABLE_TO_XMLSCHEMA TABLE_TO_XML_AND_XMLSCHEMA QUERY_TO_XML QUERY_TO_XMLSCHEMA QUERY_TO_XML_AND_XMLSCHEMA CURSOR_TO_XML CURSOR_TO_XMLSCHEMA SCHEMA_TO_XML SCHEMA_TO_XMLSCHEMA SCHEMA_TO_XML_AND_XMLSCHEMA DATABASE_TO_XML DATABASE_TO_XMLSCHEMA DATABASE_TO_XML_AND_XMLSCHEMA XMLATTRIBUTES TO_JSON TO_JSONB ARRAY_TO_JSON ROW_TO_JSON JSON_BUILD_ARRAY JSONB_BUILD_ARRAY JSON_BUILD_OBJECT JSONB_BUILD_OBJECT JSON_OBJECT JSONB_OBJECT JSON_ARRAY_LENGTH JSONB_ARRAY_LENGTH JSON_EACH JSONB_EACH JSON_EACH_TEXT JSONB_EACH_TEXT JSON_EXTRACT_PATH JSONB_EXTRACT_PATH JSON_OBJECT_KEYS JSONB_OBJECT_KEYS JSON_POPULATE_RECORD JSONB_POPULATE_RECORD JSON_POPULATE_RECORDSET JSONB_POPULATE_RECORDSET JSON_ARRAY_ELEMENTS JSONB_ARRAY_ELEMENTS JSON_ARRAY_ELEMENTS_TEXT JSONB_ARRAY_ELEMENTS_TEXT JSON_TYPEOF JSONB_TYPEOF JSON_TO_RECORD JSONB_TO_RECORD JSON_TO_RECORDSET JSONB_TO_RECORDSET JSON_STRIP_NULLS JSONB_STRIP_NULLS JSONB_SET JSONB_INSERT JSONB_PRETTY CURRVAL LASTVAL NEXTVAL SETVAL COALESCE NULLIF GREATEST LEAST ARRAY_APPEND ARRAY_CAT ARRAY_NDIMS ARRAY_DIMS ARRAY_FILL ARRAY_LENGTH ARRAY_LOWER ARRAY_POSITION ARRAY_POSITIONS ARRAY_PREPEND ARRAY_REMOVE ARRAY_REPLACE ARRAY_TO_STRING ARRAY_UPPER CARDINALITY STRING_TO_ARRAY UNNEST ISEMPTY LOWER_INC UPPER_INC LOWER_INF UPPER_INF RANGE_MERGE GENERATE_SERIES GENERATE_SUBSCRIPTS CURRENT_DATABASE CURRENT_QUERY CURRENT_SCHEMA|10 CURRENT_SCHEMAS|10 INET_CLIENT_ADDR INET_CLIENT_PORT INET_SERVER_ADDR INET_SERVER_PORT ROW_SECURITY_ACTIVE FORMAT_TYPE TO_REGCLASS TO_REGPROC TO_REGPROCEDURE TO_REGOPER TO_REGOPERATOR TO_REGTYPE TO_REGNAMESPACE TO_REGROLE COL_DESCRIPTION OBJ_DESCRIPTION SHOBJ_DESCRIPTION TXID_CURRENT TXID_CURRENT_IF_ASSIGNED TXID_CURRENT_SNAPSHOT TXID_SNAPSHOT_XIP TXID_SNAPSHOT_XMAX TXID_SNAPSHOT_XMIN TXID_VISIBLE_IN_SNAPSHOT TXID_STATUS CURRENT_SETTING SET_CONFIG BRIN_SUMMARIZE_NEW_VALUES BRIN_SUMMARIZE_RANGE BRIN_DESUMMARIZE_RANGE GIN_CLEAN_PENDING_LIST SUPPRESS_REDUNDANT_UPDATES_TRIGGER LO_FROM_BYTEA LO_PUT LO_GET LO_CREAT LO_CREATE LO_UNLINK LO_IMPORT LO_EXPORT LOREAD LOWRITE GROUPING CAST ".trim().split(" ").map(function(ue) {
        return ue.split("|")[0];
      }).join("|");
      return {
        name: "PostgreSQL",
        aliases: [
          "postgres",
          "postgresql"
        ],
        supersetOf: "sql",
        case_insensitive: !0,
        keywords: {
          keyword: J + te + Z,
          built_in: ae + oe + se
        },
        // Forbid some cunstructs from other languages to improve autodetect. In fact
        // "[a-z]:" is legal (as part of array slice), but improbabal.
        illegal: /:==|\W\s*\(\*|(^|\s)\$[a-z]|\{\{|[a-z]:\s*$|\.\.\.|TO:|DO:/,
        contains: [
          // special handling of some words, which are reserved only in some contexts
          {
            className: "keyword",
            variants: [
              { begin: /\bTEXT\s*SEARCH\b/ },
              { begin: /\b(PRIMARY|FOREIGN|FOR(\s+NO)?)\s+KEY\b/ },
              { begin: /\bPARALLEL\s+(UNSAFE|RESTRICTED|SAFE)\b/ },
              { begin: /\bSTORAGE\s+(PLAIN|EXTERNAL|EXTENDED|MAIN)\b/ },
              { begin: /\bMATCH\s+(FULL|PARTIAL|SIMPLE)\b/ },
              { begin: /\bNULLS\s+(FIRST|LAST)\b/ },
              { begin: /\bEVENT\s+TRIGGER\b/ },
              { begin: /\b(MAPPING|OR)\s+REPLACE\b/ },
              { begin: /\b(FROM|TO)\s+(PROGRAM|STDIN|STDOUT)\b/ },
              { begin: /\b(SHARE|EXCLUSIVE)\s+MODE\b/ },
              { begin: /\b(LEFT|RIGHT)\s+(OUTER\s+)?JOIN\b/ },
              { begin: /\b(FETCH|MOVE)\s+(NEXT|PRIOR|FIRST|LAST|ABSOLUTE|RELATIVE|FORWARD|BACKWARD)\b/ },
              { begin: /\bPRESERVE\s+ROWS\b/ },
              { begin: /\bDISCARD\s+PLANS\b/ },
              { begin: /\bREFERENCING\s+(OLD|NEW)\b/ },
              { begin: /\bSKIP\s+LOCKED\b/ },
              { begin: /\bGROUPING\s+SETS\b/ },
              { begin: /\b(BINARY|INSENSITIVE|SCROLL|NO\s+SCROLL)\s+(CURSOR|FOR)\b/ },
              { begin: /\b(WITH|WITHOUT)\s+HOLD\b/ },
              { begin: /\bWITH\s+(CASCADED|LOCAL)\s+CHECK\s+OPTION\b/ },
              { begin: /\bEXCLUDE\s+(TIES|NO\s+OTHERS)\b/ },
              { begin: /\bFORMAT\s+(TEXT|XML|JSON|YAML)\b/ },
              { begin: /\bSET\s+((SESSION|LOCAL)\s+)?NAMES\b/ },
              { begin: /\bIS\s+(NOT\s+)?UNKNOWN\b/ },
              { begin: /\bSECURITY\s+LABEL\b/ },
              { begin: /\bSTANDALONE\s+(YES|NO|NO\s+VALUE)\b/ },
              { begin: /\bWITH\s+(NO\s+)?DATA\b/ },
              { begin: /\b(FOREIGN|SET)\s+DATA\b/ },
              { begin: /\bSET\s+(CATALOG|CONSTRAINTS)\b/ },
              { begin: /\b(WITH|FOR)\s+ORDINALITY\b/ },
              { begin: /\bIS\s+(NOT\s+)?DOCUMENT\b/ },
              { begin: /\bXML\s+OPTION\s+(DOCUMENT|CONTENT)\b/ },
              { begin: /\b(STRIP|PRESERVE)\s+WHITESPACE\b/ },
              { begin: /\bNO\s+(ACTION|MAXVALUE|MINVALUE)\b/ },
              { begin: /\bPARTITION\s+BY\s+(RANGE|LIST|HASH)\b/ },
              { begin: /\bAT\s+TIME\s+ZONE\b/ },
              { begin: /\bGRANTED\s+BY\b/ },
              { begin: /\bRETURN\s+(QUERY|NEXT)\b/ },
              { begin: /\b(ATTACH|DETACH)\s+PARTITION\b/ },
              { begin: /\bFORCE\s+ROW\s+LEVEL\s+SECURITY\b/ },
              { begin: /\b(INCLUDING|EXCLUDING)\s+(COMMENTS|CONSTRAINTS|DEFAULTS|IDENTITY|INDEXES|STATISTICS|STORAGE|ALL)\b/ },
              { begin: /\bAS\s+(ASSIGNMENT|IMPLICIT|PERMISSIVE|RESTRICTIVE|ENUM|RANGE)\b/ }
            ]
          },
          // functions named as keywords, followed by '('
          {
            begin: /\b(FORMAT|FAMILY|VERSION)\s*\(/
            // keywords: { built_in: 'FORMAT FAMILY VERSION' }
          },
          // INCLUDE ( ... ) in index_parameters in CREATE TABLE
          {
            begin: /\bINCLUDE\s*\(/,
            keywords: "INCLUDE"
          },
          // not highlight RANGE if not in frame_clause (not 100% correct, but seems satisfactory)
          { begin: /\bRANGE(?!\s*(BETWEEN|UNBOUNDED|CURRENT|[-0-9]+))/ },
          // disable highlighting in commands CREATE AGGREGATE/COLLATION/DATABASE/OPERTOR/TEXT SEARCH .../TYPE
          // and in PL/pgSQL RAISE ... USING
          { begin: /\b(VERSION|OWNER|TEMPLATE|TABLESPACE|CONNECTION\s+LIMIT|PROCEDURE|RESTRICT|JOIN|PARSER|COPY|START|END|COLLATION|INPUT|ANALYZE|STORAGE|LIKE|DEFAULT|DELIMITER|ENCODING|COLUMN|CONSTRAINT|TABLE|SCHEMA)\s*=/ },
          // PG_smth; HAS_some_PRIVILEGE
          {
            // className: 'built_in',
            begin: /\b(PG_\w+?|HAS_[A-Z_]+_PRIVILEGE)\b/,
            relevance: 10
          },
          // extract
          {
            begin: /\bEXTRACT\s*\(/,
            end: /\bFROM\b/,
            returnEnd: !0,
            keywords: {
              // built_in: 'EXTRACT',
              type: "CENTURY DAY DECADE DOW DOY EPOCH HOUR ISODOW ISOYEAR MICROSECONDS MILLENNIUM MILLISECONDS MINUTE MONTH QUARTER SECOND TIMEZONE TIMEZONE_HOUR TIMEZONE_MINUTE WEEK YEAR"
            }
          },
          // xmlelement, xmlpi - special NAME
          {
            begin: /\b(XMLELEMENT|XMLPI)\s*\(\s*NAME/,
            keywords: {
              // built_in: 'XMLELEMENT XMLPI',
              keyword: "NAME"
            }
          },
          // xmlparse, xmlserialize
          {
            begin: /\b(XMLPARSE|XMLSERIALIZE)\s*\(\s*(DOCUMENT|CONTENT)/,
            keywords: {
              // built_in: 'XMLPARSE XMLSERIALIZE',
              keyword: "DOCUMENT CONTENT"
            }
          },
          // Sequences. We actually skip everything between CACHE|INCREMENT|MAXVALUE|MINVALUE and
          // nearest following numeric constant. Without with trick we find a lot of "keywords"
          // in 'avrasm' autodetection test...
          {
            beginKeywords: "CACHE INCREMENT MAXVALUE MINVALUE",
            end: _.C_NUMBER_RE,
            returnEnd: !0,
            keywords: "BY CACHE INCREMENT MAXVALUE MINVALUE"
          },
          // WITH|WITHOUT TIME ZONE as part of datatype
          {
            className: "type",
            begin: /\b(WITH|WITHOUT)\s+TIME\s+ZONE\b/
          },
          // INTERVAL optional fields
          {
            className: "type",
            begin: /\bINTERVAL\s+(YEAR|MONTH|DAY|HOUR|MINUTE|SECOND)(\s+TO\s+(MONTH|HOUR|MINUTE|SECOND))?\b/
          },
          // Pseudo-types which allowed only as return type
          {
            begin: /\bRETURNS\s+(LANGUAGE_HANDLER|TRIGGER|EVENT_TRIGGER|FDW_HANDLER|INDEX_AM_HANDLER|TSM_HANDLER)\b/,
            keywords: {
              keyword: "RETURNS",
              type: "LANGUAGE_HANDLER TRIGGER EVENT_TRIGGER FDW_HANDLER INDEX_AM_HANDLER TSM_HANDLER"
            }
          },
          // Known functions - only when followed by '('
          {
            begin: "\\b(" + le + ")\\s*\\("
            // keywords: { built_in: FUNCTIONS }
          },
          // Types
          {
            begin: "\\.(" + ee + ")\\b"
            // prevent highlight as type, say, 'oid' in 'pgclass.oid'
          },
          {
            begin: "\\b(" + ee + ")\\s+PATH\\b",
            // in XMLTABLE
            keywords: {
              keyword: "PATH",
              // hopefully no one would use PATH type in XMLTABLE...
              type: ne.replace("PATH ", "")
            }
          },
          {
            className: "type",
            begin: "\\b(" + ee + ")\\b"
          },
          // Strings, see https://www.postgresql.org/docs/11/static/sql-syntax-lexical.html#SQL-SYNTAX-CONSTANTS
          {
            className: "string",
            begin: "'",
            end: "'",
            contains: [{ begin: "''" }]
          },
          {
            className: "string",
            begin: "(e|E|u&|U&)'",
            end: "'",
            contains: [{ begin: "\\\\." }],
            relevance: 10
          },
          _.END_SAME_AS_BEGIN({
            begin: q,
            end: q,
            contains: [
              {
                // actually we want them all except SQL; listed are those with known implementations
                // and XML + JSON just in case
                subLanguage: [
                  "pgsql",
                  "perl",
                  "python",
                  "tcl",
                  "r",
                  "lua",
                  "java",
                  "php",
                  "ruby",
                  "bash",
                  "scheme",
                  "xml",
                  "json"
                ],
                endsWithParent: !0
              }
            ]
          }),
          // identifiers in quotes
          {
            begin: '"',
            end: '"',
            contains: [{ begin: '""' }]
          },
          // numbers
          _.C_NUMBER_MODE,
          // comments
          _.C_BLOCK_COMMENT_MODE,
          H,
          // PL/pgSQL staff
          // %ROWTYPE, %TYPE, $n
          {
            className: "meta",
            variants: [
              {
                // %TYPE, %ROWTYPE
                begin: "%(ROW)?TYPE",
                relevance: 10
              },
              {
                // $n
                begin: "\\$\\d+"
              },
              {
                // #compiler option
                begin: "^#\\w",
                end: "$"
              }
            ]
          },
          // <<labeles>>
          {
            className: "symbol",
            begin: G,
            relevance: 10
          }
        ]
      };
    }
    return pgsql_1 = O, pgsql_1;
  }
  var pony_1, hasRequiredPony;
  function requirePony() {
    if (hasRequiredPony) return pony_1;
    hasRequiredPony = 1;
    function O(_) {
      const H = {
        keyword: "actor addressof and as be break class compile_error compile_intrinsic consume continue delegate digestof do else elseif embed end error for fun if ifdef in interface is isnt lambda let match new not object or primitive recover repeat return struct then trait try type until use var where while with xor",
        meta: "iso val tag trn box ref",
        literal: "this false true"
      }, V = {
        className: "string",
        begin: '"""',
        end: '"""',
        relevance: 10
      }, q = {
        className: "string",
        begin: '"',
        end: '"',
        contains: [_.BACKSLASH_ESCAPE]
      }, G = {
        className: "string",
        begin: "'",
        end: "'",
        contains: [_.BACKSLASH_ESCAPE],
        relevance: 0
      }, J = {
        className: "type",
        begin: "\\b_?[A-Z][\\w]*",
        relevance: 0
      }, Z = {
        begin: _.IDENT_RE + "'",
        relevance: 0
      };
      return {
        name: "Pony",
        keywords: H,
        contains: [
          J,
          V,
          q,
          G,
          Z,
          {
            className: "number",
            begin: "(-?)(\\b0[xX][a-fA-F0-9]+|\\b0[bB][01]+|(\\b\\d+(_\\d+)?(\\.\\d*)?|\\.\\d+)([eE][-+]?\\d+)?)",
            relevance: 0
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE
        ]
      };
    }
    return pony_1 = O, pony_1;
  }
  var powershell_1, hasRequiredPowershell;
  function requirePowershell() {
    if (hasRequiredPowershell) return powershell_1;
    hasRequiredPowershell = 1;
    function O(_) {
      const H = [
        "string",
        "char",
        "byte",
        "int",
        "long",
        "bool",
        "decimal",
        "single",
        "double",
        "DateTime",
        "xml",
        "array",
        "hashtable",
        "void"
      ], V = "Add|Clear|Close|Copy|Enter|Exit|Find|Format|Get|Hide|Join|Lock|Move|New|Open|Optimize|Pop|Push|Redo|Remove|Rename|Reset|Resize|Search|Select|Set|Show|Skip|Split|Step|Switch|Undo|Unlock|Watch|Backup|Checkpoint|Compare|Compress|Convert|ConvertFrom|ConvertTo|Dismount|Edit|Expand|Export|Group|Import|Initialize|Limit|Merge|Mount|Out|Publish|Restore|Save|Sync|Unpublish|Update|Approve|Assert|Build|Complete|Confirm|Deny|Deploy|Disable|Enable|Install|Invoke|Register|Request|Restart|Resume|Start|Stop|Submit|Suspend|Uninstall|Unregister|Wait|Debug|Measure|Ping|Repair|Resolve|Test|Trace|Connect|Disconnect|Read|Receive|Send|Write|Block|Grant|Protect|Revoke|Unblock|Unprotect|Use|ForEach|Sort|Tee|Where", q = "-and|-as|-band|-bnot|-bor|-bxor|-casesensitive|-ccontains|-ceq|-cge|-cgt|-cle|-clike|-clt|-cmatch|-cne|-cnotcontains|-cnotlike|-cnotmatch|-contains|-creplace|-csplit|-eq|-exact|-f|-file|-ge|-gt|-icontains|-ieq|-ige|-igt|-ile|-ilike|-ilt|-imatch|-in|-ine|-inotcontains|-inotlike|-inotmatch|-ireplace|-is|-isnot|-isplit|-join|-le|-like|-lt|-match|-ne|-not|-notcontains|-notin|-notlike|-notmatch|-or|-regex|-replace|-shl|-shr|-split|-wildcard|-xor", G = {
        $pattern: /-?[A-z\.\-]+\b/,
        keyword: "if else foreach return do while until elseif begin for trap data dynamicparam end break throw param continue finally in switch exit filter try process catch hidden static parameter",
        // "echo" relevance has been set to 0 to avoid auto-detect conflicts with shell transcripts
        built_in: "ac asnp cat cd CFS chdir clc clear clhy cli clp cls clv cnsn compare copy cp cpi cpp curl cvpa dbp del diff dir dnsn ebp echo|0 epal epcsv epsn erase etsn exsn fc fhx fl ft fw gal gbp gc gcb gci gcm gcs gdr gerr ghy gi gin gjb gl gm gmo gp gps gpv group gsn gsnp gsv gtz gu gv gwmi h history icm iex ihy ii ipal ipcsv ipmo ipsn irm ise iwmi iwr kill lp ls man md measure mi mount move mp mv nal ndr ni nmo npssc nsn nv ogv oh popd ps pushd pwd r rbp rcjb rcsn rd rdr ren ri rjb rm rmdir rmo rni rnp rp rsn rsnp rujb rv rvpa rwmi sajb sal saps sasv sbp sc scb select set shcm si sl sleep sls sort sp spjb spps spsv start stz sujb sv swmi tee trcm type wget where wjb write"
        // TODO: 'validate[A-Z]+' can't work in keywords
      }, J = /\w[\w\d]*((-)[\w\d]+)*/, Z = {
        begin: "`[\\s\\S]",
        relevance: 0
      }, te = {
        className: "variable",
        variants: [
          { begin: /\$\B/ },
          {
            className: "keyword",
            begin: /\$this/
          },
          { begin: /\$[\w\d][\w\d_:]*/ }
        ]
      }, ne = {
        className: "literal",
        begin: /\$(null|true|false)\b/
      }, ee = {
        className: "string",
        variants: [
          {
            begin: /"/,
            end: /"/
          },
          {
            begin: /@"/,
            end: /^"@/
          }
        ],
        contains: [
          Z,
          te,
          {
            className: "variable",
            begin: /\$[A-z]/,
            end: /[^A-z]/
          }
        ]
      }, ae = {
        className: "string",
        variants: [
          {
            begin: /'/,
            end: /'/
          },
          {
            begin: /@'/,
            end: /^'@/
          }
        ]
      }, oe = {
        className: "doctag",
        variants: [
          /* no paramater help tags */
          { begin: /\.(synopsis|description|example|inputs|outputs|notes|link|component|role|functionality)/ },
          /* one parameter help tags */
          { begin: /\.(parameter|forwardhelptargetname|forwardhelpcategory|remotehelprunspace|externalhelp)\s+\S+/ }
        ]
      }, se = _.inherit(
        _.COMMENT(null, null),
        {
          variants: [
            /* single-line comment */
            {
              begin: /#/,
              end: /$/
            },
            /* multi-line comment */
            {
              begin: /<#/,
              end: /#>/
            }
          ],
          contains: [oe]
        }
      ), ce = {
        className: "built_in",
        variants: [{ begin: "(".concat(V, ")+(-)[\\w\\d]+") }]
      }, le = {
        className: "class",
        beginKeywords: "class enum",
        end: /\s*[{]/,
        excludeEnd: !0,
        relevance: 0,
        contains: [_.TITLE_MODE]
      }, ue = {
        className: "function",
        begin: /function\s+/,
        end: /\s*\{|$/,
        excludeEnd: !0,
        returnBegin: !0,
        relevance: 0,
        contains: [
          {
            begin: "function",
            relevance: 0,
            className: "keyword"
          },
          {
            className: "title",
            begin: J,
            relevance: 0
          },
          {
            begin: /\(/,
            end: /\)/,
            className: "params",
            relevance: 0,
            contains: [te]
          }
          // CMDLETS
        ]
      }, de = {
        begin: /using\s/,
        end: /$/,
        returnBegin: !0,
        contains: [
          ee,
          ae,
          {
            className: "keyword",
            begin: /(using|assembly|command|module|namespace|type)/
          }
        ]
      }, fe = { variants: [
        // PS literals are pretty verbose so it's a good idea to accent them a bit.
        {
          className: "operator",
          begin: "(".concat(q, ")\\b")
        },
        {
          className: "literal",
          begin: /(-){1,2}[\w\d-]+/,
          relevance: 0
        }
      ] }, he = {
        className: "selector-tag",
        begin: /@\B/,
        relevance: 0
      }, me = {
        className: "function",
        begin: /\[.*\]\s*[\w]+[ ]??\(/,
        end: /$/,
        returnBegin: !0,
        relevance: 0,
        contains: [
          {
            className: "keyword",
            begin: "(".concat(
              G.keyword.toString().replace(
                /\s/g,
                "|"
              ),
              ")\\b"
            ),
            endsParent: !0,
            relevance: 0
          },
          _.inherit(_.TITLE_MODE, { endsParent: !0 })
        ]
      }, pe = [
        // STATIC_MEMBER,
        me,
        se,
        Z,
        _.NUMBER_MODE,
        ee,
        ae,
        // PS_NEW_OBJECT_TYPE,
        ce,
        te,
        ne,
        he
      ], ye = {
        begin: /\[/,
        end: /\]/,
        excludeBegin: !0,
        excludeEnd: !0,
        relevance: 0,
        contains: [].concat(
          "self",
          pe,
          {
            begin: "(" + H.join("|") + ")",
            className: "built_in",
            relevance: 0
          },
          {
            className: "type",
            begin: /[\.\w\d]+/,
            relevance: 0
          }
        )
      };
      return me.contains.unshift(ye), {
        name: "PowerShell",
        aliases: [
          "pwsh",
          "ps",
          "ps1"
        ],
        case_insensitive: !0,
        keywords: G,
        contains: pe.concat(
          le,
          ue,
          de,
          fe,
          ye
        )
      };
    }
    return powershell_1 = O, powershell_1;
  }
  var processing_1, hasRequiredProcessing;
  function requireProcessing() {
    if (hasRequiredProcessing) return processing_1;
    hasRequiredProcessing = 1;
    function O(_) {
      const H = _.regex, V = [
        "displayHeight",
        "displayWidth",
        "mouseY",
        "mouseX",
        "mousePressed",
        "pmouseX",
        "pmouseY",
        "key",
        "keyCode",
        "pixels",
        "focused",
        "frameCount",
        "frameRate",
        "height",
        "width",
        "size",
        "createGraphics",
        "beginDraw",
        "createShape",
        "loadShape",
        "PShape",
        "arc",
        "ellipse",
        "line",
        "point",
        "quad",
        "rect",
        "triangle",
        "bezier",
        "bezierDetail",
        "bezierPoint",
        "bezierTangent",
        "curve",
        "curveDetail",
        "curvePoint",
        "curveTangent",
        "curveTightness",
        "shape",
        "shapeMode",
        "beginContour",
        "beginShape",
        "bezierVertex",
        "curveVertex",
        "endContour",
        "endShape",
        "quadraticVertex",
        "vertex",
        "ellipseMode",
        "noSmooth",
        "rectMode",
        "smooth",
        "strokeCap",
        "strokeJoin",
        "strokeWeight",
        "mouseClicked",
        "mouseDragged",
        "mouseMoved",
        "mousePressed",
        "mouseReleased",
        "mouseWheel",
        "keyPressed",
        "keyPressedkeyReleased",
        "keyTyped",
        "print",
        "println",
        "save",
        "saveFrame",
        "day",
        "hour",
        "millis",
        "minute",
        "month",
        "second",
        "year",
        "background",
        "clear",
        "colorMode",
        "fill",
        "noFill",
        "noStroke",
        "stroke",
        "alpha",
        "blue",
        "brightness",
        "color",
        "green",
        "hue",
        "lerpColor",
        "red",
        "saturation",
        "modelX",
        "modelY",
        "modelZ",
        "screenX",
        "screenY",
        "screenZ",
        "ambient",
        "emissive",
        "shininess",
        "specular",
        "add",
        "createImage",
        "beginCamera",
        "camera",
        "endCamera",
        "frustum",
        "ortho",
        "perspective",
        "printCamera",
        "printProjection",
        "cursor",
        "frameRate",
        "noCursor",
        "exit",
        "loop",
        "noLoop",
        "popStyle",
        "pushStyle",
        "redraw",
        "binary",
        "boolean",
        "byte",
        "char",
        "float",
        "hex",
        "int",
        "str",
        "unbinary",
        "unhex",
        "join",
        "match",
        "matchAll",
        "nf",
        "nfc",
        "nfp",
        "nfs",
        "split",
        "splitTokens",
        "trim",
        "append",
        "arrayCopy",
        "concat",
        "expand",
        "reverse",
        "shorten",
        "sort",
        "splice",
        "subset",
        "box",
        "sphere",
        "sphereDetail",
        "createInput",
        "createReader",
        "loadBytes",
        "loadJSONArray",
        "loadJSONObject",
        "loadStrings",
        "loadTable",
        "loadXML",
        "open",
        "parseXML",
        "saveTable",
        "selectFolder",
        "selectInput",
        "beginRaw",
        "beginRecord",
        "createOutput",
        "createWriter",
        "endRaw",
        "endRecord",
        "PrintWritersaveBytes",
        "saveJSONArray",
        "saveJSONObject",
        "saveStream",
        "saveStrings",
        "saveXML",
        "selectOutput",
        "popMatrix",
        "printMatrix",
        "pushMatrix",
        "resetMatrix",
        "rotate",
        "rotateX",
        "rotateY",
        "rotateZ",
        "scale",
        "shearX",
        "shearY",
        "translate",
        "ambientLight",
        "directionalLight",
        "lightFalloff",
        "lights",
        "lightSpecular",
        "noLights",
        "normal",
        "pointLight",
        "spotLight",
        "image",
        "imageMode",
        "loadImage",
        "noTint",
        "requestImage",
        "tint",
        "texture",
        "textureMode",
        "textureWrap",
        "blend",
        "copy",
        "filter",
        "get",
        "loadPixels",
        "set",
        "updatePixels",
        "blendMode",
        "loadShader",
        "PShaderresetShader",
        "shader",
        "createFont",
        "loadFont",
        "text",
        "textFont",
        "textAlign",
        "textLeading",
        "textMode",
        "textSize",
        "textWidth",
        "textAscent",
        "textDescent",
        "abs",
        "ceil",
        "constrain",
        "dist",
        "exp",
        "floor",
        "lerp",
        "log",
        "mag",
        "map",
        "max",
        "min",
        "norm",
        "pow",
        "round",
        "sq",
        "sqrt",
        "acos",
        "asin",
        "atan",
        "atan2",
        "cos",
        "degrees",
        "radians",
        "sin",
        "tan",
        "noise",
        "noiseDetail",
        "noiseSeed",
        "random",
        "randomGaussian",
        "randomSeed"
      ], q = _.IDENT_RE, G = { variants: [
        {
          match: H.concat(H.either(...V), H.lookahead(/\s*\(/)),
          className: "built_in"
        },
        {
          relevance: 0,
          match: H.concat(
            /\b(?!for|if|while)/,
            q,
            H.lookahead(/\s*\(/)
          ),
          className: "title.function"
        }
      ] }, J = {
        match: [
          /new\s+/,
          q
        ],
        className: {
          1: "keyword",
          2: "class.title"
        }
      }, Z = {
        relevance: 0,
        match: [
          /\./,
          q
        ],
        className: { 2: "property" }
      }, te = {
        variants: [
          { match: [
            /class/,
            /\s+/,
            q,
            /\s+/,
            /extends/,
            /\s+/,
            q
          ] },
          { match: [
            /class/,
            /\s+/,
            q
          ] }
        ],
        className: {
          1: "keyword",
          3: "title.class",
          5: "keyword",
          7: "title.class.inherited"
        }
      }, ne = [
        "boolean",
        "byte",
        "char",
        "color",
        "double",
        "float",
        "int",
        "long",
        "short"
      ], ee = [
        "BufferedReader",
        "PVector",
        "PFont",
        "PImage",
        "PGraphics",
        "HashMap",
        "String",
        "Array",
        "FloatDict",
        "ArrayList",
        "FloatList",
        "IntDict",
        "IntList",
        "JSONArray",
        "JSONObject",
        "Object",
        "StringDict",
        "StringList",
        "Table",
        "TableRow",
        "XML"
      ];
      return {
        name: "Processing",
        aliases: ["pde"],
        keywords: {
          keyword: [...[
            "abstract",
            "assert",
            "break",
            "case",
            "catch",
            "const",
            "continue",
            "default",
            "else",
            "enum",
            "final",
            "finally",
            "for",
            "if",
            "import",
            "instanceof",
            "long",
            "native",
            "new",
            "package",
            "private",
            "private",
            "protected",
            "protected",
            "public",
            "public",
            "return",
            "static",
            "strictfp",
            "switch",
            "synchronized",
            "throw",
            "throws",
            "transient",
            "try",
            "void",
            "volatile",
            "while"
          ]],
          literal: "P2D P3D HALF_PI PI QUARTER_PI TAU TWO_PI null true false",
          title: "setup draw",
          variable: "super this",
          built_in: [
            ...V,
            ...ee
          ],
          type: ne
        },
        contains: [
          te,
          J,
          G,
          Z,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE
        ]
      };
    }
    return processing_1 = O, processing_1;
  }
  var profile_1, hasRequiredProfile;
  function requireProfile() {
    if (hasRequiredProfile) return profile_1;
    hasRequiredProfile = 1;
    function O(_) {
      return {
        name: "Python profiler",
        contains: [
          _.C_NUMBER_MODE,
          {
            begin: "[a-zA-Z_][\\da-zA-Z_]+\\.[\\da-zA-Z_]{1,3}",
            end: ":",
            excludeEnd: !0
          },
          {
            begin: "(ncalls|tottime|cumtime)",
            end: "$",
            keywords: "ncalls tottime|10 cumtime|10 filename",
            relevance: 10
          },
          {
            begin: "function calls",
            end: "$",
            contains: [_.C_NUMBER_MODE],
            relevance: 10
          },
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            begin: "\\(",
            end: "\\)$",
            excludeBegin: !0,
            excludeEnd: !0,
            relevance: 0
          }
        ]
      };
    }
    return profile_1 = O, profile_1;
  }
  var prolog_1, hasRequiredProlog;
  function requireProlog() {
    if (hasRequiredProlog) return prolog_1;
    hasRequiredProlog = 1;
    function O(_) {
      const H = {
        begin: /[a-z][A-Za-z0-9_]*/,
        relevance: 0
      }, V = {
        className: "symbol",
        variants: [
          { begin: /[A-Z][a-zA-Z0-9_]*/ },
          { begin: /_[A-Za-z0-9_]*/ }
        ],
        relevance: 0
      }, q = {
        begin: /\(/,
        end: /\)/,
        relevance: 0
      }, G = {
        begin: /\[/,
        end: /\]/
      }, J = {
        className: "comment",
        begin: /%/,
        end: /$/,
        contains: [_.PHRASAL_WORDS_MODE]
      }, Z = {
        className: "string",
        begin: /`/,
        end: /`/,
        contains: [_.BACKSLASH_ESCAPE]
      }, te = {
        className: "string",
        // 0'a etc.
        begin: /0'(\\'|.)/
      }, ne = {
        className: "string",
        begin: /0'\\s/
        // 0'\s
      }, ae = [
        H,
        V,
        q,
        {
          // relevance booster
          begin: /:-/
        },
        G,
        J,
        _.C_BLOCK_COMMENT_MODE,
        _.QUOTE_STRING_MODE,
        _.APOS_STRING_MODE,
        Z,
        te,
        ne,
        _.C_NUMBER_MODE
      ];
      return q.contains = ae, G.contains = ae, {
        name: "Prolog",
        contains: ae.concat([
          {
            // relevance booster
            begin: /\.$/
          }
        ])
      };
    }
    return prolog_1 = O, prolog_1;
  }
  var properties_1, hasRequiredProperties;
  function requireProperties() {
    if (hasRequiredProperties) return properties_1;
    hasRequiredProperties = 1;
    function O(_) {
      const H = "[ \\t\\f]*", V = "[ \\t\\f]+", q = H + "[:=]" + H, G = V, J = "(" + q + "|" + G + ")", Z = "([^\\\\:= \\t\\f\\n]|\\\\.)+", te = {
        // skip DELIM
        end: J,
        relevance: 0,
        starts: {
          // value: everything until end of line (again, taking into account backslashes)
          className: "string",
          end: /$/,
          relevance: 0,
          contains: [
            { begin: "\\\\\\\\" },
            { begin: "\\\\\\n" }
          ]
        }
      };
      return {
        name: ".properties",
        disableAutodetect: !0,
        case_insensitive: !0,
        illegal: /\S/,
        contains: [
          _.COMMENT("^\\s*[!#]", "$"),
          // key: everything until whitespace or = or : (taking into account backslashes)
          // case of a key-value pair
          {
            returnBegin: !0,
            variants: [
              { begin: Z + q },
              { begin: Z + G }
            ],
            contains: [
              {
                className: "attr",
                begin: Z,
                endsParent: !0
              }
            ],
            starts: te
          },
          // case of an empty key
          {
            className: "attr",
            begin: Z + H + "$"
          }
        ]
      };
    }
    return properties_1 = O, properties_1;
  }
  var protobuf_1, hasRequiredProtobuf;
  function requireProtobuf() {
    if (hasRequiredProtobuf) return protobuf_1;
    hasRequiredProtobuf = 1;
    function O(_) {
      const H = [
        "package",
        "import",
        "option",
        "optional",
        "required",
        "repeated",
        "group",
        "oneof"
      ], V = [
        "double",
        "float",
        "int32",
        "int64",
        "uint32",
        "uint64",
        "sint32",
        "sint64",
        "fixed32",
        "fixed64",
        "sfixed32",
        "sfixed64",
        "bool",
        "string",
        "bytes"
      ], q = {
        match: [
          /(message|enum|service)\s+/,
          _.IDENT_RE
        ],
        scope: {
          1: "keyword",
          2: "title.class"
        }
      };
      return {
        name: "Protocol Buffers",
        aliases: ["proto"],
        keywords: {
          keyword: H,
          type: V,
          literal: [
            "true",
            "false"
          ]
        },
        contains: [
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          q,
          {
            className: "function",
            beginKeywords: "rpc",
            end: /[{;]/,
            excludeEnd: !0,
            keywords: "rpc returns"
          },
          {
            // match enum items (relevance)
            // BLAH = ...;
            begin: /^\s*[A-Z_]+(?=\s*=[^\n]+;$)/
          }
        ]
      };
    }
    return protobuf_1 = O, protobuf_1;
  }
  var puppet_1, hasRequiredPuppet;
  function requirePuppet() {
    if (hasRequiredPuppet) return puppet_1;
    hasRequiredPuppet = 1;
    function O(_) {
      const H = {
        keyword: (
          /* language keywords */
          "and case default else elsif false if in import enherits node or true undef unless main settings $string "
        ),
        literal: (
          /* metaparameters */
          "alias audit before loglevel noop require subscribe tag owner ensure group mode name|0 changes context force incl lens load_path onlyif provider returns root show_diff type_check en_address ip_address realname command environment hour monute month monthday special target weekday creates cwd ogoutput refresh refreshonly tries try_sleep umask backup checksum content ctime force ignore links mtime purge recurse recurselimit replace selinux_ignore_defaults selrange selrole seltype seluser source souirce_permissions sourceselect validate_cmd validate_replacement allowdupe attribute_membership auth_membership forcelocal gid ia_load_module members system host_aliases ip allowed_trunk_vlans description device_url duplex encapsulation etherchannel native_vlan speed principals allow_root auth_class auth_type authenticate_user k_of_n mechanisms rule session_owner shared options device fstype enable hasrestart directory present absent link atboot blockdevice device dump pass remounts poller_tag use message withpath adminfile allow_virtual allowcdrom category configfiles flavor install_options instance package_settings platform responsefile status uninstall_options vendor unless_system_user unless_uid binary control flags hasstatus manifest pattern restart running start stop allowdupe auths expiry gid groups home iterations key_membership keys managehome membership password password_max_age password_min_age profile_membership profiles project purge_ssh_keys role_membership roles salt shell uid baseurl cost descr enabled enablegroups exclude failovermethod gpgcheck gpgkey http_caching include includepkgs keepalive metadata_expire metalink mirrorlist priority protect proxy proxy_password proxy_username repo_gpgcheck s3_enabled skip_if_unavailable sslcacert sslclientcert sslclientkey sslverify mounted"
        ),
        built_in: (
          /* core facts */
          "architecture augeasversion blockdevices boardmanufacturer boardproductname boardserialnumber cfkey dhcp_servers domain ec2_ ec2_userdata facterversion filesystems ldom fqdn gid hardwareisa hardwaremodel hostname id|0 interfaces ipaddress ipaddress_ ipaddress6 ipaddress6_ iphostnumber is_virtual kernel kernelmajversion kernelrelease kernelversion kernelrelease kernelversion lsbdistcodename lsbdistdescription lsbdistid lsbdistrelease lsbmajdistrelease lsbminordistrelease lsbrelease macaddress macaddress_ macosx_buildversion macosx_productname macosx_productversion macosx_productverson_major macosx_productversion_minor manufacturer memoryfree memorysize netmask metmask_ network_ operatingsystem operatingsystemmajrelease operatingsystemrelease osfamily partitions path physicalprocessorcount processor processorcount productname ps puppetversion rubysitedir rubyversion selinux selinux_config_mode selinux_config_policy selinux_current_mode selinux_current_mode selinux_enforced selinux_policyversion serialnumber sp_ sshdsakey sshecdsakey sshrsakey swapencrypted swapfree swapsize timezone type uniqueid uptime uptime_days uptime_hours uptime_seconds uuid virtual vlans xendomains zfs_version zonenae zones zpool_version"
        )
      }, V = _.COMMENT("#", "$"), q = "([A-Za-z_]|::)(\\w|::)*", G = _.inherit(_.TITLE_MODE, { begin: q }), J = {
        className: "variable",
        begin: "\\$" + q
      }, Z = {
        className: "string",
        contains: [
          _.BACKSLASH_ESCAPE,
          J
        ],
        variants: [
          {
            begin: /'/,
            end: /'/
          },
          {
            begin: /"/,
            end: /"/
          }
        ]
      };
      return {
        name: "Puppet",
        aliases: ["pp"],
        contains: [
          V,
          J,
          Z,
          {
            beginKeywords: "class",
            end: "\\{|;",
            illegal: /=/,
            contains: [
              G,
              V
            ]
          },
          {
            beginKeywords: "define",
            end: /\{/,
            contains: [
              {
                className: "section",
                begin: _.IDENT_RE,
                endsParent: !0
              }
            ]
          },
          {
            begin: _.IDENT_RE + "\\s+\\{",
            returnBegin: !0,
            end: /\S/,
            contains: [
              {
                className: "keyword",
                begin: _.IDENT_RE,
                relevance: 0.2
              },
              {
                begin: /\{/,
                end: /\}/,
                keywords: H,
                relevance: 0,
                contains: [
                  Z,
                  V,
                  {
                    begin: "[a-zA-Z_]+\\s*=>",
                    returnBegin: !0,
                    end: "=>",
                    contains: [
                      {
                        className: "attr",
                        begin: _.IDENT_RE
                      }
                    ]
                  },
                  {
                    className: "number",
                    begin: "(\\b0[0-7_]+)|(\\b0x[0-9a-fA-F_]+)|(\\b[1-9][0-9_]*(\\.[0-9_]+)?)|[0_]\\b",
                    relevance: 0
                  },
                  J
                ]
              }
            ],
            relevance: 0
          }
        ]
      };
    }
    return puppet_1 = O, puppet_1;
  }
  var purebasic_1, hasRequiredPurebasic;
  function requirePurebasic() {
    if (hasRequiredPurebasic) return purebasic_1;
    hasRequiredPurebasic = 1;
    function O(_) {
      const H = {
        // PB IDE color: #0080FF (Azure Radiance)
        className: "string",
        begin: '(~)?"',
        end: '"',
        illegal: "\\n"
      }, V = {
        // PB IDE color: #924B72 (Cannon Pink)
        //  "#" + a letter or underscore + letters, digits or underscores + (optional) "$"
        className: "symbol",
        begin: "#[a-zA-Z_]\\w*\\$?"
      };
      return {
        name: "PureBASIC",
        aliases: [
          "pb",
          "pbi"
        ],
        keywords: (
          // PB IDE color: #006666 (Blue Stone) + Bold
          // Keywords from all version of PureBASIC 5.00 upward ...
          "Align And Array As Break CallDebugger Case CompilerCase CompilerDefault CompilerElse CompilerElseIf CompilerEndIf CompilerEndSelect CompilerError CompilerIf CompilerSelect CompilerWarning Continue Data DataSection Debug DebugLevel Declare DeclareC DeclareCDLL DeclareDLL DeclareModule Default Define Dim DisableASM DisableDebugger DisableExplicit Else ElseIf EnableASM EnableDebugger EnableExplicit End EndDataSection EndDeclareModule EndEnumeration EndIf EndImport EndInterface EndMacro EndModule EndProcedure EndSelect EndStructure EndStructureUnion EndWith Enumeration EnumerationBinary Extends FakeReturn For ForEach ForEver Global Gosub Goto If Import ImportC IncludeBinary IncludeFile IncludePath Interface List Macro MacroExpandedCount Map Module NewList NewMap Next Not Or Procedure ProcedureC ProcedureCDLL ProcedureDLL ProcedureReturn Protected Prototype PrototypeC ReDim Read Repeat Restore Return Runtime Select Shared Static Step Structure StructureUnion Swap Threaded To UndefineMacro Until Until  UnuseModule UseModule Wend While With XIncludeFile XOr"
        ),
        contains: [
          // COMMENTS | PB IDE color: #00AAAA (Persian Green)
          _.COMMENT(";", "$", { relevance: 0 }),
          {
            // PROCEDURES DEFINITIONS
            className: "function",
            begin: "\\b(Procedure|Declare)(C|CDLL|DLL)?\\b",
            end: "\\(",
            excludeEnd: !0,
            returnBegin: !0,
            contains: [
              {
                // PROCEDURE KEYWORDS | PB IDE color: #006666 (Blue Stone) + Bold
                className: "keyword",
                begin: "(Procedure|Declare)(C|CDLL|DLL)?",
                excludeEnd: !0
              },
              {
                // PROCEDURE RETURN TYPE SETTING | PB IDE color: #000000 (Black)
                className: "type",
                begin: "\\.\\w*"
                // end: ' ',
              },
              _.UNDERSCORE_TITLE_MODE
              // PROCEDURE NAME | PB IDE color: #006666 (Blue Stone)
            ]
          },
          H,
          V
        ]
      };
    }
    return purebasic_1 = O, purebasic_1;
  }
  var q_1, hasRequiredQ;
  function requireQ() {
    if (hasRequiredQ) return q_1;
    hasRequiredQ = 1;
    function O(_) {
      return {
        name: "Q",
        aliases: [
          "k",
          "kdb"
        ],
        keywords: {
          $pattern: /(`?)[A-Za-z0-9_]+\b/,
          keyword: "do while select delete by update from",
          literal: "0b 1b",
          built_in: "neg not null string reciprocal floor ceiling signum mod xbar xlog and or each scan over prior mmu lsq inv md5 ltime gtime count first var dev med cov cor all any rand sums prds mins maxs fills deltas ratios avgs differ prev next rank reverse iasc idesc asc desc msum mcount mavg mdev xrank mmin mmax xprev rotate distinct group where flip type key til get value attr cut set upsert raze union inter except cross sv vs sublist enlist read0 read1 hopen hclose hdel hsym hcount peach system ltrim rtrim trim lower upper ssr view tables views cols xcols keys xkey xcol xasc xdesc fkeys meta lj aj aj0 ij pj asof uj ww wj wj1 fby xgroup ungroup ej save load rsave rload show csv parse eval min max avg wavg wsum sin cos tan sum",
          type: "`float `double int `timestamp `timespan `datetime `time `boolean `symbol `char `byte `short `long `real `month `date `minute `second `guid"
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE
        ]
      };
    }
    return q_1 = O, q_1;
  }
  var qml_1, hasRequiredQml;
  function requireQml() {
    if (hasRequiredQml) return qml_1;
    hasRequiredQml = 1;
    function O(_) {
      const H = _.regex, V = {
        keyword: "in of on if for while finally var new function do return void else break catch instanceof with throw case default try this switch continue typeof delete let yield const export super debugger as async await import",
        literal: "true false null undefined NaN Infinity",
        built_in: "eval isFinite isNaN parseFloat parseInt decodeURI decodeURIComponent encodeURI encodeURIComponent escape unescape Object Function Boolean Error EvalError InternalError RangeError ReferenceError StopIteration SyntaxError TypeError URIError Number Math Date String RegExp Array Float32Array Float64Array Int16Array Int32Array Int8Array Uint16Array Uint32Array Uint8Array Uint8ClampedArray ArrayBuffer DataView JSON Intl arguments require module console window document Symbol Set Map WeakSet WeakMap Proxy Reflect Behavior bool color coordinate date double enumeration font geocircle georectangle geoshape int list matrix4x4 parent point quaternion real rect size string url variant vector2d vector3d vector4d Promise"
      }, q = "[a-zA-Z_][a-zA-Z0-9\\._]*", G = {
        className: "keyword",
        begin: "\\bproperty\\b",
        starts: {
          className: "string",
          end: "(:|=|;|,|//|/\\*|$)",
          returnEnd: !0
        }
      }, J = {
        className: "keyword",
        begin: "\\bsignal\\b",
        starts: {
          className: "string",
          end: "(\\(|:|=|;|,|//|/\\*|$)",
          returnEnd: !0
        }
      }, Z = {
        className: "attribute",
        begin: "\\bid\\s*:",
        starts: {
          className: "string",
          end: q,
          returnEnd: !1
        }
      }, te = {
        begin: q + "\\s*:",
        returnBegin: !0,
        contains: [
          {
            className: "attribute",
            begin: q,
            end: "\\s*:",
            excludeEnd: !0,
            relevance: 0
          }
        ],
        relevance: 0
      }, ne = {
        begin: H.concat(q, /\s*\{/),
        end: /\{/,
        returnBegin: !0,
        relevance: 0,
        contains: [_.inherit(_.TITLE_MODE, { begin: q })]
      };
      return {
        name: "QML",
        aliases: ["qt"],
        case_insensitive: !1,
        keywords: V,
        contains: [
          {
            className: "meta",
            begin: /^\s*['"]use (strict|asm)['"]/
          },
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          {
            // template string
            className: "string",
            begin: "`",
            end: "`",
            contains: [
              _.BACKSLASH_ESCAPE,
              {
                className: "subst",
                begin: "\\$\\{",
                end: "\\}"
              }
            ]
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "number",
            variants: [
              { begin: "\\b(0[bB][01]+)" },
              { begin: "\\b(0[oO][0-7]+)" },
              { begin: _.C_NUMBER_RE }
            ],
            relevance: 0
          },
          {
            // "value" container
            begin: "(" + _.RE_STARTERS_RE + "|\\b(case|return|throw)\\b)\\s*",
            keywords: "return throw case",
            contains: [
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE,
              _.REGEXP_MODE,
              {
                // E4X / JSX
                begin: /</,
                end: />\s*[);\]]/,
                relevance: 0,
                subLanguage: "xml"
              }
            ],
            relevance: 0
          },
          J,
          G,
          {
            className: "function",
            beginKeywords: "function",
            end: /\{/,
            excludeEnd: !0,
            contains: [
              _.inherit(_.TITLE_MODE, { begin: /[A-Za-z$_][0-9A-Za-z$_]*/ }),
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                excludeBegin: !0,
                excludeEnd: !0,
                contains: [
                  _.C_LINE_COMMENT_MODE,
                  _.C_BLOCK_COMMENT_MODE
                ]
              }
            ],
            illegal: /\[|%/
          },
          {
            // hack: prevents detection of keywords after dots
            begin: "\\." + _.IDENT_RE,
            relevance: 0
          },
          Z,
          te,
          ne
        ],
        illegal: /#/
      };
    }
    return qml_1 = O, qml_1;
  }
  var reasonml_1, hasRequiredReasonml;
  function requireReasonml() {
    if (hasRequiredReasonml) return reasonml_1;
    hasRequiredReasonml = 1;
    function O(_) {
      return {
        name: "ReasonML",
        aliases: ["re"],
        keywords: {
          $pattern: /[a-z_]\w*!?/,
          keyword: [
            "and",
            "as",
            "asr",
            "assert",
            "begin",
            "class",
            "constraint",
            "do",
            "done",
            "downto",
            "else",
            "end",
            "esfun",
            "exception",
            "external",
            "for",
            "fun",
            "function",
            "functor",
            "if",
            "in",
            "include",
            "inherit",
            "initializer",
            "land",
            "lazy",
            "let",
            "lor",
            "lsl",
            "lsr",
            "lxor",
            "mod",
            "module",
            "mutable",
            "new",
            "nonrec",
            "object",
            "of",
            "open",
            "or",
            "pri",
            "pub",
            "rec",
            "sig",
            "struct",
            "switch",
            "then",
            "to",
            "try",
            "type",
            "val",
            "virtual",
            "when",
            "while",
            "with"
          ],
          built_in: [
            "array",
            "bool",
            "bytes",
            "char",
            "exn|5",
            "float",
            "int",
            "int32",
            "int64",
            "list",
            "lazy_t|5",
            "nativeint|5",
            "ref",
            "string",
            "unit"
          ],
          literal: ["true", "false"]
        },
        illegal: /(:-|:=|\$\{|\+=)/,
        contains: [
          {
            scope: "literal",
            match: /\[(\|\|)?\]|\(\)/,
            relevance: 0
          },
          _.C_LINE_COMMENT_MODE,
          _.COMMENT(/\/\*/, /\*\//, { illegal: /^(#,\/\/)/ }),
          {
            /* type variable */
            scope: "symbol",
            match: /\'[A-Za-z_](?!\')[\w\']*/
            /* the grammar is ambiguous on how 'a'b should be interpreted but not the compiler */
          },
          {
            /* polymorphic variant */
            scope: "type",
            match: /`[A-Z][\w\']*/
          },
          {
            /* module or constructor */
            scope: "type",
            match: /\b[A-Z][\w\']*/,
            relevance: 0
          },
          {
            /* don't color identifiers, but safely catch all identifiers with ' */
            match: /[a-z_]\w*\'[\w\']*/,
            relevance: 0
          },
          {
            scope: "operator",
            match: /\s+(\|\||\+[\+\.]?|\*[\*\/\.]?|\/[\.]?|\.\.\.|\|>|&&|===?)\s+/,
            relevance: 0
          },
          _.inherit(_.APOS_STRING_MODE, {
            scope: "string",
            relevance: 0
          }),
          _.inherit(_.QUOTE_STRING_MODE, { illegal: null }),
          {
            scope: "number",
            variants: [
              { match: /\b0[xX][a-fA-F0-9_]+[Lln]?/ },
              { match: /\b0[oO][0-7_]+[Lln]?/ },
              { match: /\b0[bB][01_]+[Lln]?/ },
              { match: /\b[0-9][0-9_]*([Lln]|(\.[0-9_]*)?([eE][-+]?[0-9_]+)?)/ }
            ],
            relevance: 0
          }
        ]
      };
    }
    return reasonml_1 = O, reasonml_1;
  }
  var rib_1, hasRequiredRib;
  function requireRib() {
    if (hasRequiredRib) return rib_1;
    hasRequiredRib = 1;
    function O(_) {
      return {
        name: "RenderMan RIB",
        keywords: "ArchiveRecord AreaLightSource Atmosphere Attribute AttributeBegin AttributeEnd Basis Begin Blobby Bound Clipping ClippingPlane Color ColorSamples ConcatTransform Cone CoordinateSystem CoordSysTransform CropWindow Curves Cylinder DepthOfField Detail DetailRange Disk Displacement Display End ErrorHandler Exposure Exterior Format FrameAspectRatio FrameBegin FrameEnd GeneralPolygon GeometricApproximation Geometry Hider Hyperboloid Identity Illuminate Imager Interior LightSource MakeCubeFaceEnvironment MakeLatLongEnvironment MakeShadow MakeTexture Matte MotionBegin MotionEnd NuPatch ObjectBegin ObjectEnd ObjectInstance Opacity Option Orientation Paraboloid Patch PatchMesh Perspective PixelFilter PixelSamples PixelVariance Points PointsGeneralPolygons PointsPolygons Polygon Procedural Projection Quantize ReadArchive RelativeDetail ReverseOrientation Rotate Scale ScreenWindow ShadingInterpolation ShadingRate Shutter Sides Skew SolidBegin SolidEnd Sphere SubdivisionMesh Surface TextureCoordinates Torus Transform TransformBegin TransformEnd TransformPoints Translate TrimCurve WorldBegin WorldEnd",
        illegal: "</",
        contains: [
          _.HASH_COMMENT_MODE,
          _.C_NUMBER_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE
        ]
      };
    }
    return rib_1 = O, rib_1;
  }
  var roboconf_1, hasRequiredRoboconf;
  function requireRoboconf() {
    if (hasRequiredRoboconf) return roboconf_1;
    hasRequiredRoboconf = 1;
    function O(_) {
      const H = "[a-zA-Z-_][^\\n{]+\\{", V = {
        className: "attribute",
        begin: /[a-zA-Z-_]+/,
        end: /\s*:/,
        excludeEnd: !0,
        starts: {
          end: ";",
          relevance: 0,
          contains: [
            {
              className: "variable",
              begin: /\.[a-zA-Z-_]+/
            },
            {
              className: "keyword",
              begin: /\(optional\)/
            }
          ]
        }
      };
      return {
        name: "Roboconf",
        aliases: [
          "graph",
          "instances"
        ],
        case_insensitive: !0,
        keywords: "import",
        contains: [
          // Facet sections
          {
            begin: "^facet " + H,
            end: /\}/,
            keywords: "facet",
            contains: [
              V,
              _.HASH_COMMENT_MODE
            ]
          },
          // Instance sections
          {
            begin: "^\\s*instance of " + H,
            end: /\}/,
            keywords: "name count channels instance-data instance-state instance of",
            illegal: /\S/,
            contains: [
              "self",
              V,
              _.HASH_COMMENT_MODE
            ]
          },
          // Component sections
          {
            begin: "^" + H,
            end: /\}/,
            contains: [
              V,
              _.HASH_COMMENT_MODE
            ]
          },
          // Comments
          _.HASH_COMMENT_MODE
        ]
      };
    }
    return roboconf_1 = O, roboconf_1;
  }
  var routeros_1, hasRequiredRouteros;
  function requireRouteros() {
    if (hasRequiredRouteros) return routeros_1;
    hasRequiredRouteros = 1;
    function O(_) {
      const H = "foreach do while for if from to step else on-error and or not in", V = "global local beep delay put len typeof pick log time set find environment terminal error execute parse resolve toarray tobool toid toip toip6 tonum tostr totime", q = "add remove enable disable set get print export edit find run debug error info warning", G = "true false yes no nothing nil null", J = "traffic-flow traffic-generator firewall scheduler aaa accounting address-list address align area bandwidth-server bfd bgp bridge client clock community config connection console customer default dhcp-client dhcp-server discovery dns e-mail ethernet filter firmware gps graphing group hardware health hotspot identity igmp-proxy incoming instance interface ip ipsec ipv6 irq l2tp-server lcd ldp logging mac-server mac-winbox mangle manual mirror mme mpls nat nd neighbor network note ntp ospf ospf-v3 ovpn-server page peer pim ping policy pool port ppp pppoe-client pptp-server prefix profile proposal proxy queue radius resource rip ripng route routing screen script security-profiles server service service-port settings shares smb sms sniffer snmp snooper socks sstp-server system tool tracking type upgrade upnp user-manager users user vlan secret vrrp watchdog web-access wireless pptp pppoe lan wan layer7-protocol lease simple raw", Z = {
        className: "variable",
        variants: [
          { begin: /\$[\w\d#@][\w\d_]*/ },
          { begin: /\$\{(.*?)\}/ }
        ]
      }, te = {
        className: "string",
        begin: /"/,
        end: /"/,
        contains: [
          _.BACKSLASH_ESCAPE,
          Z,
          {
            className: "variable",
            begin: /\$\(/,
            end: /\)/,
            contains: [_.BACKSLASH_ESCAPE]
          }
        ]
      }, ne = {
        className: "string",
        begin: /'/,
        end: /'/
      };
      return {
        name: "MikroTik RouterOS script",
        aliases: ["mikrotik"],
        case_insensitive: !0,
        keywords: {
          $pattern: /:?[\w-]+/,
          literal: G,
          keyword: H + " :" + H.split(" ").join(" :") + " :" + V.split(" ").join(" :")
        },
        contains: [
          {
            // illegal syntax
            variants: [
              {
                // -- comment
                begin: /\/\*/,
                end: /\*\//
              },
              {
                // Stan comment
                begin: /\/\//,
                end: /$/
              },
              {
                // HTML tags
                begin: /<\//,
                end: />/
              }
            ],
            illegal: /./
          },
          _.COMMENT("^#", "$"),
          te,
          ne,
          Z,
          // attribute=value
          {
            // > is to avoid matches with => in other grammars
            begin: /[\w-]+=([^\s{}[\]()>]+)/,
            relevance: 0,
            returnBegin: !0,
            contains: [
              {
                className: "attribute",
                begin: /[^=]+/
              },
              {
                begin: /=/,
                endsWithParent: !0,
                relevance: 0,
                contains: [
                  te,
                  ne,
                  Z,
                  {
                    className: "literal",
                    begin: "\\b(" + G.split(" ").join("|") + ")\\b"
                  },
                  {
                    // Do not format unclassified values. Needed to exclude highlighting of values as built_in.
                    begin: /("[^"]*"|[^\s{}[\]]+)/
                  }
                  /*
                  {
                    // IPv4 addresses and subnets
                    className: 'number',
                    variants: [
                      {begin: IPADDR_wBITMASK+'(,'+IPADDR_wBITMASK+')*'}, //192.168.0.0/24,1.2.3.0/24
                      {begin: IPADDR+'-'+IPADDR},       // 192.168.0.1-192.168.0.3
                      {begin: IPADDR+'(,'+IPADDR+')*'}, // 192.168.0.1,192.168.0.34,192.168.24.1,192.168.0.1
                    ]
                  },
                  {
                    // MAC addresses and DHCP Client IDs
                    className: 'number',
                    begin: /\b(1:)?([0-9A-Fa-f]{1,2}[:-]){5}([0-9A-Fa-f]){1,2}\b/,
                  },
                  */
                ]
              }
            ]
          },
          {
            // HEX values
            className: "number",
            begin: /\*[0-9a-fA-F]+/
          },
          {
            begin: "\\b(" + q.split(" ").join("|") + ")([\\s[(\\]|])",
            returnBegin: !0,
            contains: [
              {
                className: "built_in",
                // 'function',
                begin: /\w+/
              }
            ]
          },
          {
            className: "built_in",
            variants: [
              { begin: "(\\.\\./|/|\\s)((" + J.split(" ").join("|") + ");?\\s)+" },
              {
                begin: /\.\./,
                relevance: 0
              }
            ]
          }
        ]
      };
    }
    return routeros_1 = O, routeros_1;
  }
  var rsl_1, hasRequiredRsl;
  function requireRsl() {
    if (hasRequiredRsl) return rsl_1;
    hasRequiredRsl = 1;
    function O(_) {
      const H = [
        "abs",
        "acos",
        "ambient",
        "area",
        "asin",
        "atan",
        "atmosphere",
        "attribute",
        "calculatenormal",
        "ceil",
        "cellnoise",
        "clamp",
        "comp",
        "concat",
        "cos",
        "degrees",
        "depth",
        "Deriv",
        "diffuse",
        "distance",
        "Du",
        "Dv",
        "environment",
        "exp",
        "faceforward",
        "filterstep",
        "floor",
        "format",
        "fresnel",
        "incident",
        "length",
        "lightsource",
        "log",
        "match",
        "max",
        "min",
        "mod",
        "noise",
        "normalize",
        "ntransform",
        "opposite",
        "option",
        "phong",
        "pnoise",
        "pow",
        "printf",
        "ptlined",
        "radians",
        "random",
        "reflect",
        "refract",
        "renderinfo",
        "round",
        "setcomp",
        "setxcomp",
        "setycomp",
        "setzcomp",
        "shadow",
        "sign",
        "sin",
        "smoothstep",
        "specular",
        "specularbrdf",
        "spline",
        "sqrt",
        "step",
        "tan",
        "texture",
        "textureinfo",
        "trace",
        "transform",
        "vtransform",
        "xcomp",
        "ycomp",
        "zcomp"
      ], V = [
        "matrix",
        "float",
        "color",
        "point",
        "normal",
        "vector"
      ], q = [
        "while",
        "for",
        "if",
        "do",
        "return",
        "else",
        "break",
        "extern",
        "continue"
      ], G = {
        match: [
          /(surface|displacement|light|volume|imager)/,
          /\s+/,
          _.IDENT_RE
        ],
        scope: {
          1: "keyword",
          3: "title.class"
        }
      };
      return {
        name: "RenderMan RSL",
        keywords: {
          keyword: q,
          built_in: H,
          type: V
        },
        illegal: "</",
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          _.APOS_STRING_MODE,
          _.C_NUMBER_MODE,
          {
            className: "meta",
            begin: "#",
            end: "$"
          },
          G,
          {
            beginKeywords: "illuminate illuminance gather",
            end: "\\("
          }
        ]
      };
    }
    return rsl_1 = O, rsl_1;
  }
  var ruleslanguage_1, hasRequiredRuleslanguage;
  function requireRuleslanguage() {
    if (hasRequiredRuleslanguage) return ruleslanguage_1;
    hasRequiredRuleslanguage = 1;
    function O(_) {
      return {
        name: "Oracle Rules Language",
        keywords: {
          keyword: "BILL_PERIOD BILL_START BILL_STOP RS_EFFECTIVE_START RS_EFFECTIVE_STOP RS_JURIS_CODE RS_OPCO_CODE INTDADDATTRIBUTE|5 INTDADDVMSG|5 INTDBLOCKOP|5 INTDBLOCKOPNA|5 INTDCLOSE|5 INTDCOUNT|5 INTDCOUNTSTATUSCODE|5 INTDCREATEMASK|5 INTDCREATEDAYMASK|5 INTDCREATEFACTORMASK|5 INTDCREATEHANDLE|5 INTDCREATEOVERRIDEDAYMASK|5 INTDCREATEOVERRIDEMASK|5 INTDCREATESTATUSCODEMASK|5 INTDCREATETOUPERIOD|5 INTDDELETE|5 INTDDIPTEST|5 INTDEXPORT|5 INTDGETERRORCODE|5 INTDGETERRORMESSAGE|5 INTDISEQUAL|5 INTDJOIN|5 INTDLOAD|5 INTDLOADACTUALCUT|5 INTDLOADDATES|5 INTDLOADHIST|5 INTDLOADLIST|5 INTDLOADLISTDATES|5 INTDLOADLISTENERGY|5 INTDLOADLISTHIST|5 INTDLOADRELATEDCHANNEL|5 INTDLOADSP|5 INTDLOADSTAGING|5 INTDLOADUOM|5 INTDLOADUOMDATES|5 INTDLOADUOMHIST|5 INTDLOADVERSION|5 INTDOPEN|5 INTDREADFIRST|5 INTDREADNEXT|5 INTDRECCOUNT|5 INTDRELEASE|5 INTDREPLACE|5 INTDROLLAVG|5 INTDROLLPEAK|5 INTDSCALAROP|5 INTDSCALE|5 INTDSETATTRIBUTE|5 INTDSETDSTPARTICIPANT|5 INTDSETSTRING|5 INTDSETVALUE|5 INTDSETVALUESTATUS|5 INTDSHIFTSTARTTIME|5 INTDSMOOTH|5 INTDSORT|5 INTDSPIKETEST|5 INTDSUBSET|5 INTDTOU|5 INTDTOURELEASE|5 INTDTOUVALUE|5 INTDUPDATESTATS|5 INTDVALUE|5 STDEV INTDDELETEEX|5 INTDLOADEXACTUAL|5 INTDLOADEXCUT|5 INTDLOADEXDATES|5 INTDLOADEX|5 INTDLOADEXRELATEDCHANNEL|5 INTDSAVEEX|5 MVLOAD|5 MVLOADACCT|5 MVLOADACCTDATES|5 MVLOADACCTHIST|5 MVLOADDATES|5 MVLOADHIST|5 MVLOADLIST|5 MVLOADLISTDATES|5 MVLOADLISTHIST|5 IF FOR NEXT DONE SELECT END CALL ABORT CLEAR CHANNEL FACTOR LIST NUMBER OVERRIDE SET WEEK DISTRIBUTIONNODE ELSE WHEN THEN OTHERWISE IENUM CSV INCLUDE LEAVE RIDER SAVE DELETE NOVALUE SECTION WARN SAVE_UPDATE DETERMINANT LABEL REPORT REVENUE EACH IN FROM TOTAL CHARGE BLOCK AND OR CSV_FILE RATE_CODE AUXILIARY_DEMAND UIDACCOUNT RS BILL_PERIOD_SELECT HOURS_PER_MONTH INTD_ERROR_STOP SEASON_SCHEDULE_NAME ACCOUNTFACTOR ARRAYUPPERBOUND CALLSTOREDPROC GETADOCONNECTION GETCONNECT GETDATASOURCE GETQUALIFIER GETUSERID HASVALUE LISTCOUNT LISTOP LISTUPDATE LISTVALUE PRORATEFACTOR RSPRORATE SETBINPATH SETDBMONITOR WQ_OPEN BILLINGHOURS DATE DATEFROMFLOAT DATETIMEFROMSTRING DATETIMETOSTRING DATETOFLOAT DAY DAYDIFF DAYNAME DBDATETIME HOUR MINUTE MONTH MONTHDIFF MONTHHOURS MONTHNAME ROUNDDATE SAMEWEEKDAYLASTYEAR SECOND WEEKDAY WEEKDIFF YEAR YEARDAY YEARSTR COMPSUM HISTCOUNT HISTMAX HISTMIN HISTMINNZ HISTVALUE MAXNRANGE MAXRANGE MINRANGE COMPIKVA COMPKVA COMPKVARFROMKQKW COMPLF IDATTR FLAG LF2KW LF2KWH MAXKW POWERFACTOR READING2USAGE AVGSEASON MAXSEASON MONTHLYMERGE SEASONVALUE SUMSEASON ACCTREADDATES ACCTTABLELOAD CONFIGADD CONFIGGET CREATEOBJECT CREATEREPORT EMAILCLIENT EXPBLKMDMUSAGE EXPMDMUSAGE EXPORT_USAGE FACTORINEFFECT GETUSERSPECIFIEDSTOP INEFFECT ISHOLIDAY RUNRATE SAVE_PROFILE SETREPORTTITLE USEREXIT WATFORRUNRATE TO TABLE ACOS ASIN ATAN ATAN2 BITAND CEIL COS COSECANT COSH COTANGENT DIVQUOT DIVREM EXP FABS FLOOR FMOD FREPM FREXPN LOG LOG10 MAX MAXN MIN MINNZ MODF POW ROUND ROUND2VALUE ROUNDINT SECANT SIN SINH SQROOT TAN TANH FLOAT2STRING FLOAT2STRINGNC INSTR LEFT LEN LTRIM MID RIGHT RTRIM STRING STRINGNC TOLOWER TOUPPER TRIM NUMDAYS READ_DATE STAGING",
          built_in: "IDENTIFIER OPTIONS XML_ELEMENT XML_OP XML_ELEMENT_OF DOMDOCCREATE DOMDOCLOADFILE DOMDOCLOADXML DOMDOCSAVEFILE DOMDOCGETROOT DOMDOCADDPI DOMNODEGETNAME DOMNODEGETTYPE DOMNODEGETVALUE DOMNODEGETCHILDCT DOMNODEGETFIRSTCHILD DOMNODEGETSIBLING DOMNODECREATECHILDELEMENT DOMNODESETATTRIBUTE DOMNODEGETCHILDELEMENTCT DOMNODEGETFIRSTCHILDELEMENT DOMNODEGETSIBLINGELEMENT DOMNODEGETATTRIBUTECT DOMNODEGETATTRIBUTEI DOMNODEGETATTRIBUTEBYNAME DOMNODEGETBYNAME"
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE,
          {
            className: "literal",
            variants: [
              {
                // looks like #-comment
                begin: "#\\s+",
                relevance: 0
              },
              { begin: "#[a-zA-Z .]+" }
            ]
          }
        ]
      };
    }
    return ruleslanguage_1 = O, ruleslanguage_1;
  }
  var sas_1, hasRequiredSas;
  function requireSas() {
    if (hasRequiredSas) return sas_1;
    hasRequiredSas = 1;
    function O(_) {
      const H = _.regex, V = [
        "do",
        "if",
        "then",
        "else",
        "end",
        "until",
        "while",
        "abort",
        "array",
        "attrib",
        "by",
        "call",
        "cards",
        "cards4",
        "catname",
        "continue",
        "datalines",
        "datalines4",
        "delete",
        "delim",
        "delimiter",
        "display",
        "dm",
        "drop",
        "endsas",
        "error",
        "file",
        "filename",
        "footnote",
        "format",
        "goto",
        "in",
        "infile",
        "informat",
        "input",
        "keep",
        "label",
        "leave",
        "length",
        "libname",
        "link",
        "list",
        "lostcard",
        "merge",
        "missing",
        "modify",
        "options",
        "output",
        "out",
        "page",
        "put",
        "redirect",
        "remove",
        "rename",
        "replace",
        "retain",
        "return",
        "select",
        "set",
        "skip",
        "startsas",
        "stop",
        "title",
        "update",
        "waitsas",
        "where",
        "window",
        "x|0",
        "systask",
        "add",
        "and",
        "alter",
        "as",
        "cascade",
        "check",
        "create",
        "delete",
        "describe",
        "distinct",
        "drop",
        "foreign",
        "from",
        "group",
        "having",
        "index",
        "insert",
        "into",
        "in",
        "key",
        "like",
        "message",
        "modify",
        "msgtype",
        "not",
        "null",
        "on",
        "or",
        "order",
        "primary",
        "references",
        "reset",
        "restrict",
        "select",
        "set",
        "table",
        "unique",
        "update",
        "validate",
        "view",
        "where"
      ], q = [
        "abs",
        "addr",
        "airy",
        "arcos",
        "arsin",
        "atan",
        "attrc",
        "attrn",
        "band",
        "betainv",
        "blshift",
        "bnot",
        "bor",
        "brshift",
        "bxor",
        "byte",
        "cdf",
        "ceil",
        "cexist",
        "cinv",
        "close",
        "cnonct",
        "collate",
        "compbl",
        "compound",
        "compress",
        "cos",
        "cosh",
        "css",
        "curobs",
        "cv",
        "daccdb",
        "daccdbsl",
        "daccsl",
        "daccsyd",
        "dacctab",
        "dairy",
        "date",
        "datejul",
        "datepart",
        "datetime",
        "day",
        "dclose",
        "depdb",
        "depdbsl",
        "depdbsl",
        "depsl",
        "depsl",
        "depsyd",
        "depsyd",
        "deptab",
        "deptab",
        "dequote",
        "dhms",
        "dif",
        "digamma",
        "dim",
        "dinfo",
        "dnum",
        "dopen",
        "doptname",
        "doptnum",
        "dread",
        "dropnote",
        "dsname",
        "erf",
        "erfc",
        "exist",
        "exp",
        "fappend",
        "fclose",
        "fcol",
        "fdelete",
        "fetch",
        "fetchobs",
        "fexist",
        "fget",
        "fileexist",
        "filename",
        "fileref",
        "finfo",
        "finv",
        "fipname",
        "fipnamel",
        "fipstate",
        "floor",
        "fnonct",
        "fnote",
        "fopen",
        "foptname",
        "foptnum",
        "fpoint",
        "fpos",
        "fput",
        "fread",
        "frewind",
        "frlen",
        "fsep",
        "fuzz",
        "fwrite",
        "gaminv",
        "gamma",
        "getoption",
        "getvarc",
        "getvarn",
        "hbound",
        "hms",
        "hosthelp",
        "hour",
        "ibessel",
        "index",
        "indexc",
        "indexw",
        "input",
        "inputc",
        "inputn",
        "int",
        "intck",
        "intnx",
        "intrr",
        "irr",
        "jbessel",
        "juldate",
        "kurtosis",
        "lag",
        "lbound",
        "left",
        "length",
        "lgamma",
        "libname",
        "libref",
        "log",
        "log10",
        "log2",
        "logpdf",
        "logpmf",
        "logsdf",
        "lowcase",
        "max",
        "mdy",
        "mean",
        "min",
        "minute",
        "mod",
        "month",
        "mopen",
        "mort",
        "n",
        "netpv",
        "nmiss",
        "normal",
        "note",
        "npv",
        "open",
        "ordinal",
        "pathname",
        "pdf",
        "peek",
        "peekc",
        "pmf",
        "point",
        "poisson",
        "poke",
        "probbeta",
        "probbnml",
        "probchi",
        "probf",
        "probgam",
        "probhypr",
        "probit",
        "probnegb",
        "probnorm",
        "probt",
        "put",
        "putc",
        "putn",
        "qtr",
        "quote",
        "ranbin",
        "rancau",
        "ranexp",
        "rangam",
        "range",
        "rank",
        "rannor",
        "ranpoi",
        "rantbl",
        "rantri",
        "ranuni",
        "repeat",
        "resolve",
        "reverse",
        "rewind",
        "right",
        "round",
        "saving",
        "scan",
        "sdf",
        "second",
        "sign",
        "sin",
        "sinh",
        "skewness",
        "soundex",
        "spedis",
        "sqrt",
        "std",
        "stderr",
        "stfips",
        "stname",
        "stnamel",
        "substr",
        "sum",
        "symget",
        "sysget",
        "sysmsg",
        "sysprod",
        "sysrc",
        "system",
        "tan",
        "tanh",
        "time",
        "timepart",
        "tinv",
        "tnonct",
        "today",
        "translate",
        "tranwrd",
        "trigamma",
        "trim",
        "trimn",
        "trunc",
        "uniform",
        "upcase",
        "uss",
        "var",
        "varfmt",
        "varinfmt",
        "varlabel",
        "varlen",
        "varname",
        "varnum",
        "varray",
        "varrayx",
        "vartype",
        "verify",
        "vformat",
        "vformatd",
        "vformatdx",
        "vformatn",
        "vformatnx",
        "vformatw",
        "vformatwx",
        "vformatx",
        "vinarray",
        "vinarrayx",
        "vinformat",
        "vinformatd",
        "vinformatdx",
        "vinformatn",
        "vinformatnx",
        "vinformatw",
        "vinformatwx",
        "vinformatx",
        "vlabel",
        "vlabelx",
        "vlength",
        "vlengthx",
        "vname",
        "vnamex",
        "vtype",
        "vtypex",
        "weekday",
        "year",
        "yyq",
        "zipfips",
        "zipname",
        "zipnamel",
        "zipstate"
      ], G = [
        "bquote",
        "nrbquote",
        "cmpres",
        "qcmpres",
        "compstor",
        "datatyp",
        "display",
        "do",
        "else",
        "end",
        "eval",
        "global",
        "goto",
        "if",
        "index",
        "input",
        "keydef",
        "label",
        "left",
        "length",
        "let",
        "local",
        "lowcase",
        "macro",
        "mend",
        "nrbquote",
        "nrquote",
        "nrstr",
        "put",
        "qcmpres",
        "qleft",
        "qlowcase",
        "qscan",
        "qsubstr",
        "qsysfunc",
        "qtrim",
        "quote",
        "qupcase",
        "scan",
        "str",
        "substr",
        "superq",
        "syscall",
        "sysevalf",
        "sysexec",
        "sysfunc",
        "sysget",
        "syslput",
        "sysprod",
        "sysrc",
        "sysrput",
        "then",
        "to",
        "trim",
        "unquote",
        "until",
        "upcase",
        "verify",
        "while",
        "window"
      ];
      return {
        name: "SAS",
        case_insensitive: !0,
        keywords: {
          literal: [
            "null",
            "missing",
            "_all_",
            "_automatic_",
            "_character_",
            "_infile_",
            "_n_",
            "_name_",
            "_null_",
            "_numeric_",
            "_user_",
            "_webout_"
          ],
          keyword: V
        },
        contains: [
          {
            // Distinct highlight for proc <proc>, data, run, quit
            className: "keyword",
            begin: /^\s*(proc [\w\d_]+|data|run|quit)[\s;]/
          },
          {
            // Macro variables
            className: "variable",
            begin: /&[a-zA-Z_&][a-zA-Z0-9_]*\.?/
          },
          {
            begin: [
              /^\s*/,
              /datalines;|cards;/,
              /(?:.*\n)+/,
              /^\s*;\s*$/
            ],
            className: {
              2: "keyword",
              3: "string"
            }
          },
          {
            begin: [
              /%mend|%macro/,
              /\s+/,
              /[a-zA-Z_&][a-zA-Z0-9_]*/
            ],
            className: {
              1: "built_in",
              3: "title.function"
            }
          },
          {
            // Built-in macro variables
            className: "built_in",
            begin: "%" + H.either(...G)
          },
          {
            // User-defined macro functions
            className: "title.function",
            begin: /%[a-zA-Z_][a-zA-Z_0-9]*/
          },
          {
            // TODO: this is most likely an incorrect classification
            // built_in may need more nuance
            // https://github.com/highlightjs/highlight.js/issues/2521
            className: "meta",
            begin: H.either(...q) + "(?=\\()"
          },
          {
            className: "string",
            variants: [
              _.APOS_STRING_MODE,
              _.QUOTE_STRING_MODE
            ]
          },
          _.COMMENT("\\*", ";"),
          _.C_BLOCK_COMMENT_MODE
        ]
      };
    }
    return sas_1 = O, sas_1;
  }
  var scala_1, hasRequiredScala;
  function requireScala() {
    if (hasRequiredScala) return scala_1;
    hasRequiredScala = 1;
    function O(_) {
      const H = _.regex, V = {
        className: "meta",
        begin: "@[A-Za-z]+"
      }, q = {
        className: "subst",
        variants: [
          { begin: "\\$[A-Za-z0-9_]+" },
          {
            begin: /\$\{/,
            end: /\}/
          }
        ]
      }, G = {
        className: "string",
        variants: [
          {
            begin: '"""',
            end: '"""'
          },
          {
            begin: '"',
            end: '"',
            illegal: "\\n",
            contains: [_.BACKSLASH_ESCAPE]
          },
          {
            begin: '[a-z]+"',
            end: '"',
            illegal: "\\n",
            contains: [
              _.BACKSLASH_ESCAPE,
              q
            ]
          },
          {
            className: "string",
            begin: '[a-z]+"""',
            end: '"""',
            contains: [q],
            relevance: 10
          }
        ]
      }, J = {
        className: "type",
        begin: "\\b[A-Z][A-Za-z0-9_]*",
        relevance: 0
      }, Z = {
        className: "title",
        begin: /[^0-9\n\t "'(),.`{}\[\]:;][^\n\t "'(),.`{}\[\]:;]+|[^0-9\n\t "'(),.`{}\[\]:;=]/,
        relevance: 0
      }, te = {
        className: "class",
        beginKeywords: "class object trait type",
        end: /[:={\[\n;]/,
        excludeEnd: !0,
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            beginKeywords: "extends with",
            relevance: 10
          },
          {
            begin: /\[/,
            end: /\]/,
            excludeBegin: !0,
            excludeEnd: !0,
            relevance: 0,
            contains: [
              J,
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          {
            className: "params",
            begin: /\(/,
            end: /\)/,
            excludeBegin: !0,
            excludeEnd: !0,
            relevance: 0,
            contains: [
              J,
              _.C_LINE_COMMENT_MODE,
              _.C_BLOCK_COMMENT_MODE
            ]
          },
          Z
        ]
      }, ne = {
        className: "function",
        beginKeywords: "def",
        end: H.lookahead(/[:={\[(\n;]/),
        contains: [Z]
      }, ee = {
        begin: [
          /^\s*/,
          // Is first token on the line
          "extension",
          /\s+(?=[[(])/
          // followed by at least one space and `[` or `(`
        ],
        beginScope: { 2: "keyword" }
      }, ae = {
        begin: [
          /^\s*/,
          // Is first token on the line
          /end/,
          /\s+/,
          /(extension\b)?/
          // `extension` is the only marker that follows an `end` that cannot be captured by another rule.
        ],
        beginScope: {
          2: "keyword",
          4: "keyword"
        }
      }, oe = [
        { match: /\.inline\b/ },
        {
          begin: /\binline(?=\s)/,
          keywords: "inline"
        }
      ], se = {
        begin: [
          /\(\s*/,
          // Opening `(` of a parameter or argument list
          /using/,
          /\s+(?!\))/
          // Spaces not followed by `)`
        ],
        beginScope: { 2: "keyword" }
      };
      return {
        name: "Scala",
        keywords: {
          literal: "true false null",
          keyword: "type yield lazy override def with val var sealed abstract private trait object if then forSome for while do throw finally protected extends import final return else break new catch super class case package default try this match continue throws implicit export enum given transparent"
        },
        contains: [
          {
            begin: [
              "//>",
              /\s+/,
              /using/,
              /\s+/,
              /\S+/
            ],
            beginScope: {
              1: "comment",
              3: "keyword",
              5: "type"
            },
            end: /$/,
            contains: [
              {
                className: "string",
                begin: /\S+/
              }
            ]
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          G,
          J,
          ne,
          te,
          _.C_NUMBER_MODE,
          ee,
          ae,
          ...oe,
          se,
          V
        ]
      };
    }
    return scala_1 = O, scala_1;
  }
  var scheme_1, hasRequiredScheme;
  function requireScheme() {
    if (hasRequiredScheme) return scheme_1;
    hasRequiredScheme = 1;
    function O(_) {
      const H = "[^\\(\\)\\[\\]\\{\\}\",'`;#|\\\\\\s]+", V = "(-|\\+)?\\d+([./]\\d+)?", q = V + "[+\\-]" + V + "i", G = {
        $pattern: H,
        built_in: "case-lambda call/cc class define-class exit-handler field import inherit init-field interface let*-values let-values let/ec mixin opt-lambda override protect provide public rename require require-for-syntax syntax syntax-case syntax-error unit/sig unless when with-syntax and begin call-with-current-continuation call-with-input-file call-with-output-file case cond define define-syntax delay do dynamic-wind else for-each if lambda let let* let-syntax letrec letrec-syntax map or syntax-rules ' * + , ,@ - ... / ; < <= = => > >= ` abs acos angle append apply asin assoc assq assv atan boolean? caar cadr call-with-input-file call-with-output-file call-with-values car cdddar cddddr cdr ceiling char->integer char-alphabetic? char-ci<=? char-ci<? char-ci=? char-ci>=? char-ci>? char-downcase char-lower-case? char-numeric? char-ready? char-upcase char-upper-case? char-whitespace? char<=? char<? char=? char>=? char>? char? close-input-port close-output-port complex? cons cos current-input-port current-output-port denominator display eof-object? eq? equal? eqv? eval even? exact->inexact exact? exp expt floor force gcd imag-part inexact->exact inexact? input-port? integer->char integer? interaction-environment lcm length list list->string list->vector list-ref list-tail list? load log magnitude make-polar make-rectangular make-string make-vector max member memq memv min modulo negative? newline not null-environment null? number->string number? numerator odd? open-input-file open-output-file output-port? pair? peek-char port? positive? procedure? quasiquote quote quotient rational? rationalize read read-char real-part real? remainder reverse round scheme-report-environment set! set-car! set-cdr! sin sqrt string string->list string->number string->symbol string-append string-ci<=? string-ci<? string-ci=? string-ci>=? string-ci>? string-copy string-fill! string-length string-ref string-set! string<=? string<? string=? string>=? string>? string? substring symbol->string symbol? tan transcript-off transcript-on truncate values vector vector->list vector-fill! vector-length vector-ref vector-set! with-input-from-file with-output-to-file write write-char zero?"
      }, J = {
        className: "literal",
        begin: "(#t|#f|#\\\\" + H + "|#\\\\.)"
      }, Z = {
        className: "number",
        variants: [
          {
            begin: V,
            relevance: 0
          },
          {
            begin: q,
            relevance: 0
          },
          { begin: "#b[0-1]+(/[0-1]+)?" },
          { begin: "#o[0-7]+(/[0-7]+)?" },
          { begin: "#x[0-9a-f]+(/[0-9a-f]+)?" }
        ]
      }, te = _.QUOTE_STRING_MODE, ne = [
        _.COMMENT(
          ";",
          "$",
          { relevance: 0 }
        ),
        _.COMMENT("#\\|", "\\|#")
      ], ee = {
        begin: H,
        relevance: 0
      }, ae = {
        className: "symbol",
        begin: "'" + H
      }, oe = {
        endsWithParent: !0,
        relevance: 0
      }, se = {
        variants: [
          { begin: /'/ },
          { begin: "`" }
        ],
        contains: [
          {
            begin: "\\(",
            end: "\\)",
            contains: [
              "self",
              J,
              te,
              Z,
              ee,
              ae
            ]
          }
        ]
      }, ce = {
        className: "name",
        relevance: 0,
        begin: H,
        keywords: G
      }, ue = {
        variants: [
          {
            begin: "\\(",
            end: "\\)"
          },
          {
            begin: "\\[",
            end: "\\]"
          }
        ],
        contains: [
          {
            begin: /lambda/,
            endsWithParent: !0,
            returnBegin: !0,
            contains: [
              ce,
              {
                endsParent: !0,
                variants: [
                  {
                    begin: /\(/,
                    end: /\)/
                  },
                  {
                    begin: /\[/,
                    end: /\]/
                  }
                ],
                contains: [ee]
              }
            ]
          },
          ce,
          oe
        ]
      };
      return oe.contains = [
        J,
        Z,
        te,
        ee,
        ae,
        se,
        ue
      ].concat(ne), {
        name: "Scheme",
        aliases: ["scm"],
        illegal: /\S/,
        contains: [
          _.SHEBANG(),
          Z,
          te,
          ae,
          se,
          ue
        ].concat(ne)
      };
    }
    return scheme_1 = O, scheme_1;
  }
  var scilab_1, hasRequiredScilab;
  function requireScilab() {
    if (hasRequiredScilab) return scilab_1;
    hasRequiredScilab = 1;
    function O(_) {
      const H = [
        _.C_NUMBER_MODE,
        {
          className: "string",
          begin: `'|"`,
          end: `'|"`,
          contains: [
            _.BACKSLASH_ESCAPE,
            { begin: "''" }
          ]
        }
      ];
      return {
        name: "Scilab",
        aliases: ["sci"],
        keywords: {
          $pattern: /%?\w+/,
          keyword: "abort break case clear catch continue do elseif else endfunction end for function global if pause return resume select try then while",
          literal: "%f %F %t %T %pi %eps %inf %nan %e %i %z %s",
          built_in: (
            // Scilab has more than 2000 functions. Just list the most commons
            "abs and acos asin atan ceil cd chdir clearglobal cosh cos cumprod deff disp error exec execstr exists exp eye gettext floor fprintf fread fsolve imag isdef isempty isinfisnan isvector lasterror length load linspace list listfiles log10 log2 log max min msprintf mclose mopen ones or pathconvert poly printf prod pwd rand real round sinh sin size gsort sprintf sqrt strcat strcmps tring sum system tanh tan type typename warning zeros matrix"
          )
        },
        illegal: '("|#|/\\*|\\s+/\\w+)',
        contains: [
          {
            className: "function",
            beginKeywords: "function",
            end: "$",
            contains: [
              _.UNDERSCORE_TITLE_MODE,
              {
                className: "params",
                begin: "\\(",
                end: "\\)"
              }
            ]
          },
          // seems to be a guard against [ident]' or [ident].
          // perhaps to prevent attributes from flagging as keywords?
          {
            begin: "[a-zA-Z_][a-zA-Z_0-9]*[\\.']+",
            relevance: 0
          },
          {
            begin: "\\[",
            end: "\\][\\.']*",
            relevance: 0,
            contains: H
          },
          _.COMMENT("//", "$")
        ].concat(H)
      };
    }
    return scilab_1 = O, scilab_1;
  }
  var smali_1, hasRequiredSmali;
  function requireSmali() {
    if (hasRequiredSmali) return smali_1;
    hasRequiredSmali = 1;
    function O(_) {
      const H = [
        "add",
        "and",
        "cmp",
        "cmpg",
        "cmpl",
        "const",
        "div",
        "double",
        "float",
        "goto",
        "if",
        "int",
        "long",
        "move",
        "mul",
        "neg",
        "new",
        "nop",
        "not",
        "or",
        "rem",
        "return",
        "shl",
        "shr",
        "sput",
        "sub",
        "throw",
        "ushr",
        "xor"
      ], V = [
        "aget",
        "aput",
        "array",
        "check",
        "execute",
        "fill",
        "filled",
        "goto/16",
        "goto/32",
        "iget",
        "instance",
        "invoke",
        "iput",
        "monitor",
        "packed",
        "sget",
        "sparse"
      ], q = [
        "transient",
        "constructor",
        "abstract",
        "final",
        "synthetic",
        "public",
        "private",
        "protected",
        "static",
        "bridge",
        "system"
      ];
      return {
        name: "Smali",
        contains: [
          {
            className: "string",
            begin: '"',
            end: '"',
            relevance: 0
          },
          _.COMMENT(
            "#",
            "$",
            { relevance: 0 }
          ),
          {
            className: "keyword",
            variants: [
              { begin: "\\s*\\.end\\s[a-zA-Z0-9]*" },
              {
                begin: "^[ ]*\\.[a-zA-Z]*",
                relevance: 0
              },
              {
                begin: "\\s:[a-zA-Z_0-9]*",
                relevance: 0
              },
              { begin: "\\s(" + q.join("|") + ")" }
            ]
          },
          {
            className: "built_in",
            variants: [
              { begin: "\\s(" + H.join("|") + ")\\s" },
              {
                begin: "\\s(" + H.join("|") + ")((-|/)[a-zA-Z0-9]+)+\\s",
                relevance: 10
              },
              {
                begin: "\\s(" + V.join("|") + ")((-|/)[a-zA-Z0-9]+)*\\s",
                relevance: 10
              }
            ]
          },
          {
            className: "class",
            begin: `L[^(;:
]*;`,
            relevance: 0
          },
          { begin: "[vp][0-9]+" }
        ]
      };
    }
    return smali_1 = O, smali_1;
  }
  var smalltalk_1, hasRequiredSmalltalk;
  function requireSmalltalk() {
    if (hasRequiredSmalltalk) return smalltalk_1;
    hasRequiredSmalltalk = 1;
    function O(_) {
      const H = "[a-z][a-zA-Z0-9_]*", V = {
        className: "string",
        begin: "\\$.{1}"
      }, q = {
        className: "symbol",
        begin: "#" + _.UNDERSCORE_IDENT_RE
      };
      return {
        name: "Smalltalk",
        aliases: ["st"],
        keywords: [
          "self",
          "super",
          "nil",
          "true",
          "false",
          "thisContext"
        ],
        contains: [
          _.COMMENT('"', '"'),
          _.APOS_STRING_MODE,
          {
            className: "type",
            begin: "\\b[A-Z][A-Za-z0-9_]*",
            relevance: 0
          },
          {
            begin: H + ":",
            relevance: 0
          },
          _.C_NUMBER_MODE,
          q,
          V,
          {
            // This looks more complicated than needed to avoid combinatorial
            // explosion under V8. It effectively means `| var1 var2 ... |` with
            // whitespace adjacent to `|` being optional.
            begin: "\\|[ ]*" + H + "([ ]+" + H + ")*[ ]*\\|",
            returnBegin: !0,
            end: /\|/,
            illegal: /\S/,
            contains: [{ begin: "(\\|[ ]*)?" + H }]
          },
          {
            begin: "#\\(",
            end: "\\)",
            contains: [
              _.APOS_STRING_MODE,
              V,
              _.C_NUMBER_MODE,
              q
            ]
          }
        ]
      };
    }
    return smalltalk_1 = O, smalltalk_1;
  }
  var sml_1, hasRequiredSml;
  function requireSml() {
    if (hasRequiredSml) return sml_1;
    hasRequiredSml = 1;
    function O(_) {
      return {
        name: "SML (Standard ML)",
        aliases: ["ml"],
        keywords: {
          $pattern: "[a-z_]\\w*!?",
          keyword: (
            /* according to Definition of Standard ML 97  */
            "abstype and andalso as case datatype do else end eqtype exception fn fun functor handle if in include infix infixr let local nonfix of op open orelse raise rec sharing sig signature struct structure then type val with withtype where while"
          ),
          built_in: (
            /* built-in types according to basis library */
            "array bool char exn int list option order real ref string substring vector unit word"
          ),
          literal: "true false NONE SOME LESS EQUAL GREATER nil"
        },
        illegal: /\/\/|>>/,
        contains: [
          {
            className: "literal",
            begin: /\[(\|\|)?\]|\(\)/,
            relevance: 0
          },
          _.COMMENT(
            "\\(\\*",
            "\\*\\)",
            { contains: ["self"] }
          ),
          {
            /* type variable */
            className: "symbol",
            begin: "'[A-Za-z_](?!')[\\w']*"
            /* the grammar is ambiguous on how 'a'b should be interpreted but not the compiler */
          },
          {
            /* polymorphic variant */
            className: "type",
            begin: "`[A-Z][\\w']*"
          },
          {
            /* module or constructor */
            className: "type",
            begin: "\\b[A-Z][\\w']*",
            relevance: 0
          },
          {
            /* don't color identifiers, but safely catch all identifiers with ' */
            begin: "[a-z_]\\w*'[\\w']*"
          },
          _.inherit(_.APOS_STRING_MODE, {
            className: "string",
            relevance: 0
          }),
          _.inherit(_.QUOTE_STRING_MODE, { illegal: null }),
          {
            className: "number",
            begin: "\\b(0[xX][a-fA-F0-9_]+[Lln]?|0[oO][0-7_]+[Lln]?|0[bB][01_]+[Lln]?|[0-9][0-9_]*([Lln]|(\\.[0-9_]*)?([eE][-+]?[0-9_]+)?)?)",
            relevance: 0
          },
          {
            begin: /[-=]>/
            // relevance booster
          }
        ]
      };
    }
    return sml_1 = O, sml_1;
  }
  var sqf_1, hasRequiredSqf;
  function requireSqf() {
    if (hasRequiredSqf) return sqf_1;
    hasRequiredSqf = 1;
    function O(_) {
      const H = {
        className: "variable",
        begin: /\b_+[a-zA-Z]\w*/
      }, V = {
        className: "title",
        begin: /[a-zA-Z][a-zA-Z_0-9]*_fnc_[a-zA-Z_0-9]+/
      }, q = {
        className: "string",
        variants: [
          {
            begin: '"',
            end: '"',
            contains: [
              {
                begin: '""',
                relevance: 0
              }
            ]
          },
          {
            begin: "'",
            end: "'",
            contains: [
              {
                begin: "''",
                relevance: 0
              }
            ]
          }
        ]
      }, G = [
        "break",
        "breakWith",
        "breakOut",
        "breakTo",
        "case",
        "catch",
        "continue",
        "continueWith",
        "default",
        "do",
        "else",
        "exit",
        "exitWith",
        "for",
        "forEach",
        "from",
        "if",
        "local",
        "private",
        "switch",
        "step",
        "then",
        "throw",
        "to",
        "try",
        "waitUntil",
        "while",
        "with"
      ], J = [
        "blufor",
        "civilian",
        "configNull",
        "controlNull",
        "displayNull",
        "diaryRecordNull",
        "east",
        "endl",
        "false",
        "grpNull",
        "independent",
        "lineBreak",
        "locationNull",
        "nil",
        "objNull",
        "opfor",
        "pi",
        "resistance",
        "scriptNull",
        "sideAmbientLife",
        "sideEmpty",
        "sideEnemy",
        "sideFriendly",
        "sideLogic",
        "sideUnknown",
        "taskNull",
        "teamMemberNull",
        "true",
        "west"
      ], Z = [
        "abs",
        "accTime",
        "acos",
        "action",
        "actionIDs",
        "actionKeys",
        "actionKeysEx",
        "actionKeysImages",
        "actionKeysNames",
        "actionKeysNamesArray",
        "actionName",
        "actionParams",
        "activateAddons",
        "activatedAddons",
        "activateKey",
        "activeTitleEffectParams",
        "add3DENConnection",
        "add3DENEventHandler",
        "add3DENLayer",
        "addAction",
        "addBackpack",
        "addBackpackCargo",
        "addBackpackCargoGlobal",
        "addBackpackGlobal",
        "addBinocularItem",
        "addCamShake",
        "addCuratorAddons",
        "addCuratorCameraArea",
        "addCuratorEditableObjects",
        "addCuratorEditingArea",
        "addCuratorPoints",
        "addEditorObject",
        "addEventHandler",
        "addForce",
        "addForceGeneratorRTD",
        "addGoggles",
        "addGroupIcon",
        "addHandgunItem",
        "addHeadgear",
        "addItem",
        "addItemCargo",
        "addItemCargoGlobal",
        "addItemPool",
        "addItemToBackpack",
        "addItemToUniform",
        "addItemToVest",
        "addLiveStats",
        "addMagazine",
        "addMagazineAmmoCargo",
        "addMagazineCargo",
        "addMagazineCargoGlobal",
        "addMagazineGlobal",
        "addMagazinePool",
        "addMagazines",
        "addMagazineTurret",
        "addMenu",
        "addMenuItem",
        "addMissionEventHandler",
        "addMPEventHandler",
        "addMusicEventHandler",
        "addonFiles",
        "addOwnedMine",
        "addPlayerScores",
        "addPrimaryWeaponItem",
        "addPublicVariableEventHandler",
        "addRating",
        "addResources",
        "addScore",
        "addScoreSide",
        "addSecondaryWeaponItem",
        "addSwitchableUnit",
        "addTeamMember",
        "addToRemainsCollector",
        "addTorque",
        "addUniform",
        "addUserActionEventHandler",
        "addVehicle",
        "addVest",
        "addWaypoint",
        "addWeapon",
        "addWeaponCargo",
        "addWeaponCargoGlobal",
        "addWeaponGlobal",
        "addWeaponItem",
        "addWeaponPool",
        "addWeaponTurret",
        "addWeaponWithAttachmentsCargo",
        "addWeaponWithAttachmentsCargoGlobal",
        "admin",
        "agent",
        "agents",
        "AGLToASL",
        "aimedAtTarget",
        "aimPos",
        "airDensityCurveRTD",
        "airDensityRTD",
        "airplaneThrottle",
        "airportSide",
        "AISFinishHeal",
        "alive",
        "all3DENEntities",
        "allActiveTitleEffects",
        "allAddonsInfo",
        "allAirports",
        "allControls",
        "allCurators",
        "allCutLayers",
        "allDead",
        "allDeadMen",
        "allDiaryRecords",
        "allDiarySubjects",
        "allDisplays",
        "allEnv3DSoundSources",
        "allGroups",
        "allLODs",
        "allMapMarkers",
        "allMines",
        "allMissionObjects",
        "allObjects",
        "allow3DMode",
        "allowCrewInImmobile",
        "allowCuratorLogicIgnoreAreas",
        "allowDamage",
        "allowDammage",
        "allowedService",
        "allowFileOperations",
        "allowFleeing",
        "allowGetIn",
        "allowService",
        "allowSprint",
        "allPlayers",
        "allSimpleObjects",
        "allSites",
        "allTurrets",
        "allUnits",
        "allUnitsUAV",
        "allUsers",
        "allVariables",
        "ambientTemperature",
        "ammo",
        "ammoOnPylon",
        "and",
        "animate",
        "animateBay",
        "animateDoor",
        "animatePylon",
        "animateSource",
        "animationNames",
        "animationPhase",
        "animationSourcePhase",
        "animationState",
        "apertureParams",
        "append",
        "apply",
        "armoryPoints",
        "arrayIntersect",
        "asin",
        "ASLToAGL",
        "ASLToATL",
        "assert",
        "assignAsCargo",
        "assignAsCargoIndex",
        "assignAsCommander",
        "assignAsDriver",
        "assignAsGunner",
        "assignAsTurret",
        "assignCurator",
        "assignedCargo",
        "assignedCommander",
        "assignedDriver",
        "assignedGroup",
        "assignedGunner",
        "assignedItems",
        "assignedTarget",
        "assignedTeam",
        "assignedVehicle",
        "assignedVehicleRole",
        "assignedVehicles",
        "assignItem",
        "assignTeam",
        "assignToAirport",
        "atan",
        "atan2",
        "atg",
        "ATLToASL",
        "attachedObject",
        "attachedObjects",
        "attachedTo",
        "attachObject",
        "attachTo",
        "attackEnabled",
        "awake",
        "backpack",
        "backpackCargo",
        "backpackContainer",
        "backpackItems",
        "backpackMagazines",
        "backpackSpaceFor",
        "behaviour",
        "benchmark",
        "bezierInterpolation",
        "binocular",
        "binocularItems",
        "binocularMagazine",
        "boundingBox",
        "boundingBoxReal",
        "boundingCenter",
        "brakesDisabled",
        "briefingName",
        "buildingExit",
        "buildingPos",
        "buldozer_EnableRoadDiag",
        "buldozer_IsEnabledRoadDiag",
        "buldozer_LoadNewRoads",
        "buldozer_reloadOperMap",
        "buttonAction",
        "buttonSetAction",
        "cadetMode",
        "calculatePath",
        "calculatePlayerVisibilityByFriendly",
        "call",
        "callExtension",
        "camCommand",
        "camCommit",
        "camCommitPrepared",
        "camCommitted",
        "camConstuctionSetParams",
        "camCreate",
        "camDestroy",
        "cameraEffect",
        "cameraEffectEnableHUD",
        "cameraInterest",
        "cameraOn",
        "cameraView",
        "campaignConfigFile",
        "camPreload",
        "camPreloaded",
        "camPrepareBank",
        "camPrepareDir",
        "camPrepareDive",
        "camPrepareFocus",
        "camPrepareFov",
        "camPrepareFovRange",
        "camPreparePos",
        "camPrepareRelPos",
        "camPrepareTarget",
        "camSetBank",
        "camSetDir",
        "camSetDive",
        "camSetFocus",
        "camSetFov",
        "camSetFovRange",
        "camSetPos",
        "camSetRelPos",
        "camSetTarget",
        "camTarget",
        "camUseNVG",
        "canAdd",
        "canAddItemToBackpack",
        "canAddItemToUniform",
        "canAddItemToVest",
        "cancelSimpleTaskDestination",
        "canDeployWeapon",
        "canFire",
        "canMove",
        "canSlingLoad",
        "canStand",
        "canSuspend",
        "canTriggerDynamicSimulation",
        "canUnloadInCombat",
        "canVehicleCargo",
        "captive",
        "captiveNum",
        "cbChecked",
        "cbSetChecked",
        "ceil",
        "channelEnabled",
        "cheatsEnabled",
        "checkAIFeature",
        "checkVisibility",
        "className",
        "clear3DENAttribute",
        "clear3DENInventory",
        "clearAllItemsFromBackpack",
        "clearBackpackCargo",
        "clearBackpackCargoGlobal",
        "clearForcesRTD",
        "clearGroupIcons",
        "clearItemCargo",
        "clearItemCargoGlobal",
        "clearItemPool",
        "clearMagazineCargo",
        "clearMagazineCargoGlobal",
        "clearMagazinePool",
        "clearOverlay",
        "clearRadio",
        "clearWeaponCargo",
        "clearWeaponCargoGlobal",
        "clearWeaponPool",
        "clientOwner",
        "closeDialog",
        "closeDisplay",
        "closeOverlay",
        "collapseObjectTree",
        "collect3DENHistory",
        "collectiveRTD",
        "collisionDisabledWith",
        "combatBehaviour",
        "combatMode",
        "commandArtilleryFire",
        "commandChat",
        "commander",
        "commandFire",
        "commandFollow",
        "commandFSM",
        "commandGetOut",
        "commandingMenu",
        "commandMove",
        "commandRadio",
        "commandStop",
        "commandSuppressiveFire",
        "commandTarget",
        "commandWatch",
        "comment",
        "commitOverlay",
        "compatibleItems",
        "compatibleMagazines",
        "compile",
        "compileFinal",
        "compileScript",
        "completedFSM",
        "composeText",
        "configClasses",
        "configFile",
        "configHierarchy",
        "configName",
        "configOf",
        "configProperties",
        "configSourceAddonList",
        "configSourceMod",
        "configSourceModList",
        "confirmSensorTarget",
        "connectTerminalToUAV",
        "connectToServer",
        "controlsGroupCtrl",
        "conversationDisabled",
        "copyFromClipboard",
        "copyToClipboard",
        "copyWaypoints",
        "cos",
        "count",
        "countEnemy",
        "countFriendly",
        "countSide",
        "countType",
        "countUnknown",
        "create3DENComposition",
        "create3DENEntity",
        "createAgent",
        "createCenter",
        "createDialog",
        "createDiaryLink",
        "createDiaryRecord",
        "createDiarySubject",
        "createDisplay",
        "createGearDialog",
        "createGroup",
        "createGuardedPoint",
        "createHashMap",
        "createHashMapFromArray",
        "createLocation",
        "createMarker",
        "createMarkerLocal",
        "createMenu",
        "createMine",
        "createMissionDisplay",
        "createMPCampaignDisplay",
        "createSimpleObject",
        "createSimpleTask",
        "createSite",
        "createSoundSource",
        "createTask",
        "createTeam",
        "createTrigger",
        "createUnit",
        "createVehicle",
        "createVehicleCrew",
        "createVehicleLocal",
        "crew",
        "ctAddHeader",
        "ctAddRow",
        "ctClear",
        "ctCurSel",
        "ctData",
        "ctFindHeaderRows",
        "ctFindRowHeader",
        "ctHeaderControls",
        "ctHeaderCount",
        "ctRemoveHeaders",
        "ctRemoveRows",
        "ctrlActivate",
        "ctrlAddEventHandler",
        "ctrlAngle",
        "ctrlAnimateModel",
        "ctrlAnimationPhaseModel",
        "ctrlAt",
        "ctrlAutoScrollDelay",
        "ctrlAutoScrollRewind",
        "ctrlAutoScrollSpeed",
        "ctrlBackgroundColor",
        "ctrlChecked",
        "ctrlClassName",
        "ctrlCommit",
        "ctrlCommitted",
        "ctrlCreate",
        "ctrlDelete",
        "ctrlEnable",
        "ctrlEnabled",
        "ctrlFade",
        "ctrlFontHeight",
        "ctrlForegroundColor",
        "ctrlHTMLLoaded",
        "ctrlIDC",
        "ctrlIDD",
        "ctrlMapAnimAdd",
        "ctrlMapAnimClear",
        "ctrlMapAnimCommit",
        "ctrlMapAnimDone",
        "ctrlMapCursor",
        "ctrlMapMouseOver",
        "ctrlMapPosition",
        "ctrlMapScale",
        "ctrlMapScreenToWorld",
        "ctrlMapSetPosition",
        "ctrlMapWorldToScreen",
        "ctrlModel",
        "ctrlModelDirAndUp",
        "ctrlModelScale",
        "ctrlMousePosition",
        "ctrlParent",
        "ctrlParentControlsGroup",
        "ctrlPosition",
        "ctrlRemoveAllEventHandlers",
        "ctrlRemoveEventHandler",
        "ctrlScale",
        "ctrlScrollValues",
        "ctrlSetActiveColor",
        "ctrlSetAngle",
        "ctrlSetAutoScrollDelay",
        "ctrlSetAutoScrollRewind",
        "ctrlSetAutoScrollSpeed",
        "ctrlSetBackgroundColor",
        "ctrlSetChecked",
        "ctrlSetDisabledColor",
        "ctrlSetEventHandler",
        "ctrlSetFade",
        "ctrlSetFocus",
        "ctrlSetFont",
        "ctrlSetFontH1",
        "ctrlSetFontH1B",
        "ctrlSetFontH2",
        "ctrlSetFontH2B",
        "ctrlSetFontH3",
        "ctrlSetFontH3B",
        "ctrlSetFontH4",
        "ctrlSetFontH4B",
        "ctrlSetFontH5",
        "ctrlSetFontH5B",
        "ctrlSetFontH6",
        "ctrlSetFontH6B",
        "ctrlSetFontHeight",
        "ctrlSetFontHeightH1",
        "ctrlSetFontHeightH2",
        "ctrlSetFontHeightH3",
        "ctrlSetFontHeightH4",
        "ctrlSetFontHeightH5",
        "ctrlSetFontHeightH6",
        "ctrlSetFontHeightSecondary",
        "ctrlSetFontP",
        "ctrlSetFontPB",
        "ctrlSetFontSecondary",
        "ctrlSetForegroundColor",
        "ctrlSetModel",
        "ctrlSetModelDirAndUp",
        "ctrlSetModelScale",
        "ctrlSetMousePosition",
        "ctrlSetPixelPrecision",
        "ctrlSetPosition",
        "ctrlSetPositionH",
        "ctrlSetPositionW",
        "ctrlSetPositionX",
        "ctrlSetPositionY",
        "ctrlSetScale",
        "ctrlSetScrollValues",
        "ctrlSetShadow",
        "ctrlSetStructuredText",
        "ctrlSetText",
        "ctrlSetTextColor",
        "ctrlSetTextColorSecondary",
        "ctrlSetTextSecondary",
        "ctrlSetTextSelection",
        "ctrlSetTooltip",
        "ctrlSetTooltipColorBox",
        "ctrlSetTooltipColorShade",
        "ctrlSetTooltipColorText",
        "ctrlSetTooltipMaxWidth",
        "ctrlSetURL",
        "ctrlSetURLOverlayMode",
        "ctrlShadow",
        "ctrlShow",
        "ctrlShown",
        "ctrlStyle",
        "ctrlText",
        "ctrlTextColor",
        "ctrlTextHeight",
        "ctrlTextSecondary",
        "ctrlTextSelection",
        "ctrlTextWidth",
        "ctrlTooltip",
        "ctrlType",
        "ctrlURL",
        "ctrlURLOverlayMode",
        "ctrlVisible",
        "ctRowControls",
        "ctRowCount",
        "ctSetCurSel",
        "ctSetData",
        "ctSetHeaderTemplate",
        "ctSetRowTemplate",
        "ctSetValue",
        "ctValue",
        "curatorAddons",
        "curatorCamera",
        "curatorCameraArea",
        "curatorCameraAreaCeiling",
        "curatorCoef",
        "curatorEditableObjects",
        "curatorEditingArea",
        "curatorEditingAreaType",
        "curatorMouseOver",
        "curatorPoints",
        "curatorRegisteredObjects",
        "curatorSelected",
        "curatorWaypointCost",
        "current3DENOperation",
        "currentChannel",
        "currentCommand",
        "currentMagazine",
        "currentMagazineDetail",
        "currentMagazineDetailTurret",
        "currentMagazineTurret",
        "currentMuzzle",
        "currentNamespace",
        "currentPilot",
        "currentTask",
        "currentTasks",
        "currentThrowable",
        "currentVisionMode",
        "currentWaypoint",
        "currentWeapon",
        "currentWeaponMode",
        "currentWeaponTurret",
        "currentZeroing",
        "cursorObject",
        "cursorTarget",
        "customChat",
        "customRadio",
        "customWaypointPosition",
        "cutFadeOut",
        "cutObj",
        "cutRsc",
        "cutText",
        "damage",
        "date",
        "dateToNumber",
        "dayTime",
        "deActivateKey",
        "debriefingText",
        "debugFSM",
        "debugLog",
        "decayGraphValues",
        "deg",
        "delete3DENEntities",
        "deleteAt",
        "deleteCenter",
        "deleteCollection",
        "deleteEditorObject",
        "deleteGroup",
        "deleteGroupWhenEmpty",
        "deleteIdentity",
        "deleteLocation",
        "deleteMarker",
        "deleteMarkerLocal",
        "deleteRange",
        "deleteResources",
        "deleteSite",
        "deleteStatus",
        "deleteTeam",
        "deleteVehicle",
        "deleteVehicleCrew",
        "deleteWaypoint",
        "detach",
        "detectedMines",
        "diag_activeMissionFSMs",
        "diag_activeScripts",
        "diag_activeSQFScripts",
        "diag_activeSQSScripts",
        "diag_allMissionEventHandlers",
        "diag_captureFrame",
        "diag_captureFrameToFile",
        "diag_captureSlowFrame",
        "diag_codePerformance",
        "diag_deltaTime",
        "diag_drawmode",
        "diag_dumpCalltraceToLog",
        "diag_dumpScriptAssembly",
        "diag_dumpTerrainSynth",
        "diag_dynamicSimulationEnd",
        "diag_enable",
        "diag_enabled",
        "diag_exportConfig",
        "diag_exportTerrainSVG",
        "diag_fps",
        "diag_fpsmin",
        "diag_frameno",
        "diag_getTerrainSegmentOffset",
        "diag_lightNewLoad",
        "diag_list",
        "diag_localized",
        "diag_log",
        "diag_logSlowFrame",
        "diag_mergeConfigFile",
        "diag_recordTurretLimits",
        "diag_resetFSM",
        "diag_resetshapes",
        "diag_scope",
        "diag_setLightNew",
        "diag_stacktrace",
        "diag_tickTime",
        "diag_toggle",
        "dialog",
        "diarySubjectExists",
        "didJIP",
        "didJIPOwner",
        "difficulty",
        "difficultyEnabled",
        "difficultyEnabledRTD",
        "difficultyOption",
        "direction",
        "directionStabilizationEnabled",
        "directSay",
        "disableAI",
        "disableBrakes",
        "disableCollisionWith",
        "disableConversation",
        "disableDebriefingStats",
        "disableMapIndicators",
        "disableNVGEquipment",
        "disableRemoteSensors",
        "disableSerialization",
        "disableTIEquipment",
        "disableUAVConnectability",
        "disableUserInput",
        "displayAddEventHandler",
        "displayChild",
        "displayCtrl",
        "displayParent",
        "displayRemoveAllEventHandlers",
        "displayRemoveEventHandler",
        "displaySetEventHandler",
        "displayUniqueName",
        "displayUpdate",
        "dissolveTeam",
        "distance",
        "distance2D",
        "distanceSqr",
        "distributionRegion",
        "do3DENAction",
        "doArtilleryFire",
        "doFire",
        "doFollow",
        "doFSM",
        "doGetOut",
        "doMove",
        "doorPhase",
        "doStop",
        "doSuppressiveFire",
        "doTarget",
        "doWatch",
        "drawArrow",
        "drawEllipse",
        "drawIcon",
        "drawIcon3D",
        "drawLaser",
        "drawLine",
        "drawLine3D",
        "drawLink",
        "drawLocation",
        "drawPolygon",
        "drawRectangle",
        "drawTriangle",
        "driver",
        "drop",
        "dynamicSimulationDistance",
        "dynamicSimulationDistanceCoef",
        "dynamicSimulationEnabled",
        "dynamicSimulationSystemEnabled",
        "echo",
        "edit3DENMissionAttributes",
        "editObject",
        "editorSetEventHandler",
        "effectiveCommander",
        "elevatePeriscope",
        "emptyPositions",
        "enableAI",
        "enableAIFeature",
        "enableAimPrecision",
        "enableAttack",
        "enableAudioFeature",
        "enableAutoStartUpRTD",
        "enableAutoTrimRTD",
        "enableCamShake",
        "enableCaustics",
        "enableChannel",
        "enableCollisionWith",
        "enableCopilot",
        "enableDebriefingStats",
        "enableDiagLegend",
        "enableDirectionStabilization",
        "enableDynamicSimulation",
        "enableDynamicSimulationSystem",
        "enableEndDialog",
        "enableEngineArtillery",
        "enableEnvironment",
        "enableFatigue",
        "enableGunLights",
        "enableInfoPanelComponent",
        "enableIRLasers",
        "enableMimics",
        "enablePersonTurret",
        "enableRadio",
        "enableReload",
        "enableRopeAttach",
        "enableSatNormalOnDetail",
        "enableSaving",
        "enableSentences",
        "enableSimulation",
        "enableSimulationGlobal",
        "enableStamina",
        "enableStressDamage",
        "enableTeamSwitch",
        "enableTraffic",
        "enableUAVConnectability",
        "enableUAVWaypoints",
        "enableVehicleCargo",
        "enableVehicleSensor",
        "enableWeaponDisassembly",
        "endLoadingScreen",
        "endMission",
        "engineOn",
        "enginesIsOnRTD",
        "enginesPowerRTD",
        "enginesRpmRTD",
        "enginesTorqueRTD",
        "entities",
        "environmentEnabled",
        "environmentVolume",
        "equipmentDisabled",
        "estimatedEndServerTime",
        "estimatedTimeLeft",
        "evalObjectArgument",
        "everyBackpack",
        "everyContainer",
        "exec",
        "execEditorScript",
        "execFSM",
        "execVM",
        "exp",
        "expectedDestination",
        "exportJIPMessages",
        "eyeDirection",
        "eyePos",
        "face",
        "faction",
        "fadeEnvironment",
        "fadeMusic",
        "fadeRadio",
        "fadeSound",
        "fadeSpeech",
        "failMission",
        "fileExists",
        "fillWeaponsFromPool",
        "find",
        "findAny",
        "findCover",
        "findDisplay",
        "findEditorObject",
        "findEmptyPosition",
        "findEmptyPositionReady",
        "findIf",
        "findNearestEnemy",
        "finishMissionInit",
        "finite",
        "fire",
        "fireAtTarget",
        "firstBackpack",
        "flag",
        "flagAnimationPhase",
        "flagOwner",
        "flagSide",
        "flagTexture",
        "flatten",
        "fleeing",
        "floor",
        "flyInHeight",
        "flyInHeightASL",
        "focusedCtrl",
        "fog",
        "fogForecast",
        "fogParams",
        "forceAddUniform",
        "forceAtPositionRTD",
        "forceCadetDifficulty",
        "forcedMap",
        "forceEnd",
        "forceFlagTexture",
        "forceFollowRoad",
        "forceGeneratorRTD",
        "forceMap",
        "forceRespawn",
        "forceSpeed",
        "forceUnicode",
        "forceWalk",
        "forceWeaponFire",
        "forceWeatherChange",
        "forEachMember",
        "forEachMemberAgent",
        "forEachMemberTeam",
        "forgetTarget",
        "format",
        "formation",
        "formationDirection",
        "formationLeader",
        "formationMembers",
        "formationPosition",
        "formationTask",
        "formatText",
        "formLeader",
        "freeExtension",
        "freeLook",
        "fromEditor",
        "fuel",
        "fullCrew",
        "gearIDCAmmoCount",
        "gearSlotAmmoCount",
        "gearSlotData",
        "gestureState",
        "get",
        "get3DENActionState",
        "get3DENAttribute",
        "get3DENCamera",
        "get3DENConnections",
        "get3DENEntity",
        "get3DENEntityID",
        "get3DENGrid",
        "get3DENIconsVisible",
        "get3DENLayerEntities",
        "get3DENLinesVisible",
        "get3DENMissionAttribute",
        "get3DENMouseOver",
        "get3DENSelected",
        "getAimingCoef",
        "getAllEnv3DSoundControllers",
        "getAllEnvSoundControllers",
        "getAllHitPointsDamage",
        "getAllOwnedMines",
        "getAllPylonsInfo",
        "getAllSoundControllers",
        "getAllUnitTraits",
        "getAmmoCargo",
        "getAnimAimPrecision",
        "getAnimSpeedCoef",
        "getArray",
        "getArtilleryAmmo",
        "getArtilleryComputerSettings",
        "getArtilleryETA",
        "getAssetDLCInfo",
        "getAssignedCuratorLogic",
        "getAssignedCuratorUnit",
        "getAttackTarget",
        "getAudioOptionVolumes",
        "getBackpackCargo",
        "getBleedingRemaining",
        "getBurningValue",
        "getCalculatePlayerVisibilityByFriendly",
        "getCameraViewDirection",
        "getCargoIndex",
        "getCenterOfMass",
        "getClientState",
        "getClientStateNumber",
        "getCompatiblePylonMagazines",
        "getConnectedUAV",
        "getConnectedUAVUnit",
        "getContainerMaxLoad",
        "getCorpse",
        "getCruiseControl",
        "getCursorObjectParams",
        "getCustomAimCoef",
        "getCustomSoundController",
        "getCustomSoundControllerCount",
        "getDammage",
        "getDebriefingText",
        "getDescription",
        "getDir",
        "getDirVisual",
        "getDiverState",
        "getDLCAssetsUsage",
        "getDLCAssetsUsageByName",
        "getDLCs",
        "getDLCUsageTime",
        "getEditorCamera",
        "getEditorMode",
        "getEditorObjectScope",
        "getElevationOffset",
        "getEngineTargetRPMRTD",
        "getEnv3DSoundController",
        "getEnvSoundController",
        "getEventHandlerInfo",
        "getFatigue",
        "getFieldManualStartPage",
        "getForcedFlagTexture",
        "getForcedSpeed",
        "getFriend",
        "getFSMVariable",
        "getFuelCargo",
        "getGraphValues",
        "getGroupIcon",
        "getGroupIconParams",
        "getGroupIcons",
        "getHideFrom",
        "getHit",
        "getHitIndex",
        "getHitPointDamage",
        "getItemCargo",
        "getLighting",
        "getLightingAt",
        "getLoadedModsInfo",
        "getMagazineCargo",
        "getMarkerColor",
        "getMarkerPos",
        "getMarkerSize",
        "getMarkerType",
        "getMass",
        "getMissionConfig",
        "getMissionConfigValue",
        "getMissionDLCs",
        "getMissionLayerEntities",
        "getMissionLayers",
        "getMissionPath",
        "getModelInfo",
        "getMousePosition",
        "getMusicPlayedTime",
        "getNumber",
        "getObjectArgument",
        "getObjectChildren",
        "getObjectDLC",
        "getObjectFOV",
        "getObjectID",
        "getObjectMaterials",
        "getObjectProxy",
        "getObjectScale",
        "getObjectTextures",
        "getObjectType",
        "getObjectViewDistance",
        "getOpticsMode",
        "getOrDefault",
        "getOrDefaultCall",
        "getOxygenRemaining",
        "getPersonUsedDLCs",
        "getPilotCameraDirection",
        "getPilotCameraPosition",
        "getPilotCameraRotation",
        "getPilotCameraTarget",
        "getPiPViewDistance",
        "getPlateNumber",
        "getPlayerChannel",
        "getPlayerID",
        "getPlayerScores",
        "getPlayerUID",
        "getPlayerVoNVolume",
        "getPos",
        "getPosASL",
        "getPosASLVisual",
        "getPosASLW",
        "getPosATL",
        "getPosATLVisual",
        "getPosVisual",
        "getPosWorld",
        "getPosWorldVisual",
        "getPylonMagazines",
        "getRelDir",
        "getRelPos",
        "getRemoteSensorsDisabled",
        "getRepairCargo",
        "getResolution",
        "getRoadInfo",
        "getRotorBrakeRTD",
        "getSensorTargets",
        "getSensorThreats",
        "getShadowDistance",
        "getShotParents",
        "getSlingLoad",
        "getSoundController",
        "getSoundControllerResult",
        "getSpeed",
        "getStamina",
        "getStatValue",
        "getSteamFriendsServers",
        "getSubtitleOptions",
        "getSuppression",
        "getTerrainGrid",
        "getTerrainHeight",
        "getTerrainHeightASL",
        "getTerrainInfo",
        "getText",
        "getTextRaw",
        "getTextureInfo",
        "getTextWidth",
        "getTiParameters",
        "getTotalDLCUsageTime",
        "getTrimOffsetRTD",
        "getTurretLimits",
        "getTurretOpticsMode",
        "getUnitFreefallInfo",
        "getUnitLoadout",
        "getUnitTrait",
        "getUnloadInCombat",
        "getUserInfo",
        "getUserMFDText",
        "getUserMFDValue",
        "getVariable",
        "getVehicleCargo",
        "getVehicleTiPars",
        "getWeaponCargo",
        "getWeaponSway",
        "getWingsOrientationRTD",
        "getWingsPositionRTD",
        "getWPPos",
        "glanceAt",
        "globalChat",
        "globalRadio",
        "goggles",
        "goto",
        "group",
        "groupChat",
        "groupFromNetId",
        "groupIconSelectable",
        "groupIconsVisible",
        "groupID",
        "groupOwner",
        "groupRadio",
        "groups",
        "groupSelectedUnits",
        "groupSelectUnit",
        "gunner",
        "gusts",
        "halt",
        "handgunItems",
        "handgunMagazine",
        "handgunWeapon",
        "handsHit",
        "hashValue",
        "hasInterface",
        "hasPilotCamera",
        "hasWeapon",
        "hcAllGroups",
        "hcGroupParams",
        "hcLeader",
        "hcRemoveAllGroups",
        "hcRemoveGroup",
        "hcSelected",
        "hcSelectGroup",
        "hcSetGroup",
        "hcShowBar",
        "hcShownBar",
        "headgear",
        "hideBody",
        "hideObject",
        "hideObjectGlobal",
        "hideSelection",
        "hint",
        "hintC",
        "hintCadet",
        "hintSilent",
        "hmd",
        "hostMission",
        "htmlLoad",
        "HUDMovementLevels",
        "humidity",
        "image",
        "importAllGroups",
        "importance",
        "in",
        "inArea",
        "inAreaArray",
        "incapacitatedState",
        "inflame",
        "inflamed",
        "infoPanel",
        "infoPanelComponentEnabled",
        "infoPanelComponents",
        "infoPanels",
        "inGameUISetEventHandler",
        "inheritsFrom",
        "initAmbientLife",
        "inPolygon",
        "inputAction",
        "inputController",
        "inputMouse",
        "inRangeOfArtillery",
        "insert",
        "insertEditorObject",
        "intersect",
        "is3DEN",
        "is3DENMultiplayer",
        "is3DENPreview",
        "isAbleToBreathe",
        "isActionMenuVisible",
        "isAgent",
        "isAimPrecisionEnabled",
        "isAllowedCrewInImmobile",
        "isArray",
        "isAutoHoverOn",
        "isAutonomous",
        "isAutoStartUpEnabledRTD",
        "isAutotest",
        "isAutoTrimOnRTD",
        "isAwake",
        "isBleeding",
        "isBurning",
        "isClass",
        "isCollisionLightOn",
        "isCopilotEnabled",
        "isDamageAllowed",
        "isDedicated",
        "isDLCAvailable",
        "isEngineOn",
        "isEqualRef",
        "isEqualTo",
        "isEqualType",
        "isEqualTypeAll",
        "isEqualTypeAny",
        "isEqualTypeArray",
        "isEqualTypeParams",
        "isFilePatchingEnabled",
        "isFinal",
        "isFlashlightOn",
        "isFlatEmpty",
        "isForcedWalk",
        "isFormationLeader",
        "isGameFocused",
        "isGamePaused",
        "isGroupDeletedWhenEmpty",
        "isHidden",
        "isInRemainsCollector",
        "isInstructorFigureEnabled",
        "isIRLaserOn",
        "isKeyActive",
        "isKindOf",
        "isLaserOn",
        "isLightOn",
        "isLocalized",
        "isManualFire",
        "isMarkedForCollection",
        "isMissionProfileNamespaceLoaded",
        "isMultiplayer",
        "isMultiplayerSolo",
        "isNil",
        "isNotEqualRef",
        "isNotEqualTo",
        "isNull",
        "isNumber",
        "isObjectHidden",
        "isObjectRTD",
        "isOnRoad",
        "isPiPEnabled",
        "isPlayer",
        "isRealTime",
        "isRemoteExecuted",
        "isRemoteExecutedJIP",
        "isSaving",
        "isSensorTargetConfirmed",
        "isServer",
        "isShowing3DIcons",
        "isSimpleObject",
        "isSprintAllowed",
        "isStaminaEnabled",
        "isSteamMission",
        "isSteamOverlayEnabled",
        "isStreamFriendlyUIEnabled",
        "isStressDamageEnabled",
        "isText",
        "isTouchingGround",
        "isTurnedOut",
        "isTutHintsEnabled",
        "isUAVConnectable",
        "isUAVConnected",
        "isUIContext",
        "isUniformAllowed",
        "isVehicleCargo",
        "isVehicleRadarOn",
        "isVehicleSensorEnabled",
        "isWalking",
        "isWeaponDeployed",
        "isWeaponRested",
        "itemCargo",
        "items",
        "itemsWithMagazines",
        "join",
        "joinAs",
        "joinAsSilent",
        "joinSilent",
        "joinString",
        "kbAddDatabase",
        "kbAddDatabaseTargets",
        "kbAddTopic",
        "kbHasTopic",
        "kbReact",
        "kbRemoveTopic",
        "kbTell",
        "kbWasSaid",
        "keyImage",
        "keyName",
        "keys",
        "knowsAbout",
        "land",
        "landAt",
        "landResult",
        "language",
        "laserTarget",
        "lbAdd",
        "lbClear",
        "lbColor",
        "lbColorRight",
        "lbCurSel",
        "lbData",
        "lbDelete",
        "lbIsSelected",
        "lbPicture",
        "lbPictureRight",
        "lbSelection",
        "lbSetColor",
        "lbSetColorRight",
        "lbSetCurSel",
        "lbSetData",
        "lbSetPicture",
        "lbSetPictureColor",
        "lbSetPictureColorDisabled",
        "lbSetPictureColorSelected",
        "lbSetPictureRight",
        "lbSetPictureRightColor",
        "lbSetPictureRightColorDisabled",
        "lbSetPictureRightColorSelected",
        "lbSetSelectColor",
        "lbSetSelectColorRight",
        "lbSetSelected",
        "lbSetText",
        "lbSetTextRight",
        "lbSetTooltip",
        "lbSetValue",
        "lbSize",
        "lbSort",
        "lbSortBy",
        "lbSortByValue",
        "lbText",
        "lbTextRight",
        "lbTooltip",
        "lbValue",
        "leader",
        "leaderboardDeInit",
        "leaderboardGetRows",
        "leaderboardInit",
        "leaderboardRequestRowsFriends",
        "leaderboardRequestRowsGlobal",
        "leaderboardRequestRowsGlobalAroundUser",
        "leaderboardsRequestUploadScore",
        "leaderboardsRequestUploadScoreKeepBest",
        "leaderboardState",
        "leaveVehicle",
        "libraryCredits",
        "libraryDisclaimers",
        "lifeState",
        "lightAttachObject",
        "lightDetachObject",
        "lightIsOn",
        "lightnings",
        "limitSpeed",
        "linearConversion",
        "lineIntersects",
        "lineIntersectsObjs",
        "lineIntersectsSurfaces",
        "lineIntersectsWith",
        "linkItem",
        "list",
        "listObjects",
        "listRemoteTargets",
        "listVehicleSensors",
        "ln",
        "lnbAddArray",
        "lnbAddColumn",
        "lnbAddRow",
        "lnbClear",
        "lnbColor",
        "lnbColorRight",
        "lnbCurSelRow",
        "lnbData",
        "lnbDeleteColumn",
        "lnbDeleteRow",
        "lnbGetColumnsPosition",
        "lnbPicture",
        "lnbPictureRight",
        "lnbSetColor",
        "lnbSetColorRight",
        "lnbSetColumnsPos",
        "lnbSetCurSelRow",
        "lnbSetData",
        "lnbSetPicture",
        "lnbSetPictureColor",
        "lnbSetPictureColorRight",
        "lnbSetPictureColorSelected",
        "lnbSetPictureColorSelectedRight",
        "lnbSetPictureRight",
        "lnbSetText",
        "lnbSetTextRight",
        "lnbSetTooltip",
        "lnbSetValue",
        "lnbSize",
        "lnbSort",
        "lnbSortBy",
        "lnbSortByValue",
        "lnbText",
        "lnbTextRight",
        "lnbValue",
        "load",
        "loadAbs",
        "loadBackpack",
        "loadConfig",
        "loadFile",
        "loadGame",
        "loadIdentity",
        "loadMagazine",
        "loadOverlay",
        "loadStatus",
        "loadUniform",
        "loadVest",
        "localize",
        "localNamespace",
        "locationPosition",
        "lock",
        "lockCameraTo",
        "lockCargo",
        "lockDriver",
        "locked",
        "lockedCameraTo",
        "lockedCargo",
        "lockedDriver",
        "lockedInventory",
        "lockedTurret",
        "lockIdentity",
        "lockInventory",
        "lockTurret",
        "lockWp",
        "log",
        "logEntities",
        "logNetwork",
        "logNetworkTerminate",
        "lookAt",
        "lookAtPos",
        "magazineCargo",
        "magazines",
        "magazinesAllTurrets",
        "magazinesAmmo",
        "magazinesAmmoCargo",
        "magazinesAmmoFull",
        "magazinesDetail",
        "magazinesDetailBackpack",
        "magazinesDetailUniform",
        "magazinesDetailVest",
        "magazinesTurret",
        "magazineTurretAmmo",
        "mapAnimAdd",
        "mapAnimClear",
        "mapAnimCommit",
        "mapAnimDone",
        "mapCenterOnCamera",
        "mapGridPosition",
        "markAsFinishedOnSteam",
        "markerAlpha",
        "markerBrush",
        "markerChannel",
        "markerColor",
        "markerDir",
        "markerPolyline",
        "markerPos",
        "markerShadow",
        "markerShape",
        "markerSize",
        "markerText",
        "markerType",
        "matrixMultiply",
        "matrixTranspose",
        "max",
        "maxLoad",
        "members",
        "menuAction",
        "menuAdd",
        "menuChecked",
        "menuClear",
        "menuCollapse",
        "menuData",
        "menuDelete",
        "menuEnable",
        "menuEnabled",
        "menuExpand",
        "menuHover",
        "menuPicture",
        "menuSetAction",
        "menuSetCheck",
        "menuSetData",
        "menuSetPicture",
        "menuSetShortcut",
        "menuSetText",
        "menuSetURL",
        "menuSetValue",
        "menuShortcut",
        "menuShortcutText",
        "menuSize",
        "menuSort",
        "menuText",
        "menuURL",
        "menuValue",
        "merge",
        "min",
        "mineActive",
        "mineDetectedBy",
        "missileTarget",
        "missileTargetPos",
        "missionConfigFile",
        "missionDifficulty",
        "missionEnd",
        "missionName",
        "missionNameSource",
        "missionNamespace",
        "missionProfileNamespace",
        "missionStart",
        "missionVersion",
        "mod",
        "modelToWorld",
        "modelToWorldVisual",
        "modelToWorldVisualWorld",
        "modelToWorldWorld",
        "modParams",
        "moonIntensity",
        "moonPhase",
        "morale",
        "move",
        "move3DENCamera",
        "moveInAny",
        "moveInCargo",
        "moveInCommander",
        "moveInDriver",
        "moveInGunner",
        "moveInTurret",
        "moveObjectToEnd",
        "moveOut",
        "moveTime",
        "moveTo",
        "moveToCompleted",
        "moveToFailed",
        "musicVolume",
        "name",
        "namedProperties",
        "nameSound",
        "nearEntities",
        "nearestBuilding",
        "nearestLocation",
        "nearestLocations",
        "nearestLocationWithDubbing",
        "nearestMines",
        "nearestObject",
        "nearestObjects",
        "nearestTerrainObjects",
        "nearObjects",
        "nearObjectsReady",
        "nearRoads",
        "nearSupplies",
        "nearTargets",
        "needReload",
        "needService",
        "netId",
        "netObjNull",
        "newOverlay",
        "nextMenuItemIndex",
        "nextWeatherChange",
        "nMenuItems",
        "not",
        "numberOfEnginesRTD",
        "numberToDate",
        "objectCurators",
        "objectFromNetId",
        "objectParent",
        "objStatus",
        "onBriefingGroup",
        "onBriefingNotes",
        "onBriefingPlan",
        "onBriefingTeamSwitch",
        "onCommandModeChanged",
        "onDoubleClick",
        "onEachFrame",
        "onGroupIconClick",
        "onGroupIconOverEnter",
        "onGroupIconOverLeave",
        "onHCGroupSelectionChanged",
        "onMapSingleClick",
        "onPlayerConnected",
        "onPlayerDisconnected",
        "onPreloadFinished",
        "onPreloadStarted",
        "onShowNewObject",
        "onTeamSwitch",
        "openCuratorInterface",
        "openDLCPage",
        "openGPS",
        "openMap",
        "openSteamApp",
        "openYoutubeVideo",
        "or",
        "orderGetIn",
        "overcast",
        "overcastForecast",
        "owner",
        "param",
        "params",
        "parseNumber",
        "parseSimpleArray",
        "parseText",
        "parsingNamespace",
        "particlesQuality",
        "periscopeElevation",
        "pickWeaponPool",
        "pitch",
        "pixelGrid",
        "pixelGridBase",
        "pixelGridNoUIScale",
        "pixelH",
        "pixelW",
        "playableSlotsNumber",
        "playableUnits",
        "playAction",
        "playActionNow",
        "player",
        "playerRespawnTime",
        "playerSide",
        "playersNumber",
        "playGesture",
        "playMission",
        "playMove",
        "playMoveNow",
        "playMusic",
        "playScriptedMission",
        "playSound",
        "playSound3D",
        "playSoundUI",
        "pose",
        "position",
        "positionCameraToWorld",
        "posScreenToWorld",
        "posWorldToScreen",
        "ppEffectAdjust",
        "ppEffectCommit",
        "ppEffectCommitted",
        "ppEffectCreate",
        "ppEffectDestroy",
        "ppEffectEnable",
        "ppEffectEnabled",
        "ppEffectForceInNVG",
        "precision",
        "preloadCamera",
        "preloadObject",
        "preloadSound",
        "preloadTitleObj",
        "preloadTitleRsc",
        "preprocessFile",
        "preprocessFileLineNumbers",
        "primaryWeapon",
        "primaryWeaponItems",
        "primaryWeaponMagazine",
        "priority",
        "processDiaryLink",
        "productVersion",
        "profileName",
        "profileNamespace",
        "profileNameSteam",
        "progressLoadingScreen",
        "progressPosition",
        "progressSetPosition",
        "publicVariable",
        "publicVariableClient",
        "publicVariableServer",
        "pushBack",
        "pushBackUnique",
        "putWeaponPool",
        "queryItemsPool",
        "queryMagazinePool",
        "queryWeaponPool",
        "rad",
        "radioChannelAdd",
        "radioChannelCreate",
        "radioChannelInfo",
        "radioChannelRemove",
        "radioChannelSetCallSign",
        "radioChannelSetLabel",
        "radioEnabled",
        "radioVolume",
        "rain",
        "rainbow",
        "rainParams",
        "random",
        "rank",
        "rankId",
        "rating",
        "rectangular",
        "regexFind",
        "regexMatch",
        "regexReplace",
        "registeredTasks",
        "registerTask",
        "reload",
        "reloadEnabled",
        "remoteControl",
        "remoteExec",
        "remoteExecCall",
        "remoteExecutedOwner",
        "remove3DENConnection",
        "remove3DENEventHandler",
        "remove3DENLayer",
        "removeAction",
        "removeAll3DENEventHandlers",
        "removeAllActions",
        "removeAllAssignedItems",
        "removeAllBinocularItems",
        "removeAllContainers",
        "removeAllCuratorAddons",
        "removeAllCuratorCameraAreas",
        "removeAllCuratorEditingAreas",
        "removeAllEventHandlers",
        "removeAllHandgunItems",
        "removeAllItems",
        "removeAllItemsWithMagazines",
        "removeAllMissionEventHandlers",
        "removeAllMPEventHandlers",
        "removeAllMusicEventHandlers",
        "removeAllOwnedMines",
        "removeAllPrimaryWeaponItems",
        "removeAllSecondaryWeaponItems",
        "removeAllUserActionEventHandlers",
        "removeAllWeapons",
        "removeBackpack",
        "removeBackpackGlobal",
        "removeBinocularItem",
        "removeCuratorAddons",
        "removeCuratorCameraArea",
        "removeCuratorEditableObjects",
        "removeCuratorEditingArea",
        "removeDiaryRecord",
        "removeDiarySubject",
        "removeDrawIcon",
        "removeDrawLinks",
        "removeEventHandler",
        "removeFromRemainsCollector",
        "removeGoggles",
        "removeGroupIcon",
        "removeHandgunItem",
        "removeHeadgear",
        "removeItem",
        "removeItemFromBackpack",
        "removeItemFromUniform",
        "removeItemFromVest",
        "removeItems",
        "removeMagazine",
        "removeMagazineGlobal",
        "removeMagazines",
        "removeMagazinesTurret",
        "removeMagazineTurret",
        "removeMenuItem",
        "removeMissionEventHandler",
        "removeMPEventHandler",
        "removeMusicEventHandler",
        "removeOwnedMine",
        "removePrimaryWeaponItem",
        "removeSecondaryWeaponItem",
        "removeSimpleTask",
        "removeSwitchableUnit",
        "removeTeamMember",
        "removeUniform",
        "removeUserActionEventHandler",
        "removeVest",
        "removeWeapon",
        "removeWeaponAttachmentCargo",
        "removeWeaponCargo",
        "removeWeaponGlobal",
        "removeWeaponTurret",
        "reportRemoteTarget",
        "requiredVersion",
        "resetCamShake",
        "resetSubgroupDirection",
        "resize",
        "resources",
        "respawnVehicle",
        "restartEditorCamera",
        "reveal",
        "revealMine",
        "reverse",
        "reversedMouseY",
        "roadAt",
        "roadsConnectedTo",
        "roleDescription",
        "ropeAttachedObjects",
        "ropeAttachedTo",
        "ropeAttachEnabled",
        "ropeAttachTo",
        "ropeCreate",
        "ropeCut",
        "ropeDestroy",
        "ropeDetach",
        "ropeEndPosition",
        "ropeLength",
        "ropes",
        "ropesAttachedTo",
        "ropeSegments",
        "ropeUnwind",
        "ropeUnwound",
        "rotorsForcesRTD",
        "rotorsRpmRTD",
        "round",
        "runInitScript",
        "safeZoneH",
        "safeZoneW",
        "safeZoneWAbs",
        "safeZoneX",
        "safeZoneXAbs",
        "safeZoneY",
        "save3DENInventory",
        "saveGame",
        "saveIdentity",
        "saveJoysticks",
        "saveMissionProfileNamespace",
        "saveOverlay",
        "saveProfileNamespace",
        "saveStatus",
        "saveVar",
        "savingEnabled",
        "say",
        "say2D",
        "say3D",
        "scopeName",
        "score",
        "scoreSide",
        "screenshot",
        "screenToWorld",
        "scriptDone",
        "scriptName",
        "scudState",
        "secondaryWeapon",
        "secondaryWeaponItems",
        "secondaryWeaponMagazine",
        "select",
        "selectBestPlaces",
        "selectDiarySubject",
        "selectedEditorObjects",
        "selectEditorObject",
        "selectionNames",
        "selectionPosition",
        "selectionVectorDirAndUp",
        "selectLeader",
        "selectMax",
        "selectMin",
        "selectNoPlayer",
        "selectPlayer",
        "selectRandom",
        "selectRandomWeighted",
        "selectWeapon",
        "selectWeaponTurret",
        "sendAUMessage",
        "sendSimpleCommand",
        "sendTask",
        "sendTaskResult",
        "sendUDPMessage",
        "sentencesEnabled",
        "serverCommand",
        "serverCommandAvailable",
        "serverCommandExecutable",
        "serverName",
        "serverNamespace",
        "serverTime",
        "set",
        "set3DENAttribute",
        "set3DENAttributes",
        "set3DENGrid",
        "set3DENIconsVisible",
        "set3DENLayer",
        "set3DENLinesVisible",
        "set3DENLogicType",
        "set3DENMissionAttribute",
        "set3DENMissionAttributes",
        "set3DENModelsVisible",
        "set3DENObjectType",
        "set3DENSelected",
        "setAccTime",
        "setActualCollectiveRTD",
        "setAirplaneThrottle",
        "setAirportSide",
        "setAmmo",
        "setAmmoCargo",
        "setAmmoOnPylon",
        "setAnimSpeedCoef",
        "setAperture",
        "setApertureNew",
        "setArmoryPoints",
        "setAttributes",
        "setAutonomous",
        "setBehaviour",
        "setBehaviourStrong",
        "setBleedingRemaining",
        "setBrakesRTD",
        "setCameraInterest",
        "setCamShakeDefParams",
        "setCamShakeParams",
        "setCamUseTi",
        "setCaptive",
        "setCenterOfMass",
        "setCollisionLight",
        "setCombatBehaviour",
        "setCombatMode",
        "setCompassOscillation",
        "setConvoySeparation",
        "setCruiseControl",
        "setCuratorCameraAreaCeiling",
        "setCuratorCoef",
        "setCuratorEditingAreaType",
        "setCuratorWaypointCost",
        "setCurrentChannel",
        "setCurrentTask",
        "setCurrentWaypoint",
        "setCustomAimCoef",
        "SetCustomMissionData",
        "setCustomSoundController",
        "setCustomWeightRTD",
        "setDamage",
        "setDammage",
        "setDate",
        "setDebriefingText",
        "setDefaultCamera",
        "setDestination",
        "setDetailMapBlendPars",
        "setDiaryRecordText",
        "setDiarySubjectPicture",
        "setDir",
        "setDirection",
        "setDrawIcon",
        "setDriveOnPath",
        "setDropInterval",
        "setDynamicSimulationDistance",
        "setDynamicSimulationDistanceCoef",
        "setEditorMode",
        "setEditorObjectScope",
        "setEffectCondition",
        "setEffectiveCommander",
        "setEngineRpmRTD",
        "setFace",
        "setFaceanimation",
        "setFatigue",
        "setFeatureType",
        "setFlagAnimationPhase",
        "setFlagOwner",
        "setFlagSide",
        "setFlagTexture",
        "setFog",
        "setForceGeneratorRTD",
        "setFormation",
        "setFormationTask",
        "setFormDir",
        "setFriend",
        "setFromEditor",
        "setFSMVariable",
        "setFuel",
        "setFuelCargo",
        "setGroupIcon",
        "setGroupIconParams",
        "setGroupIconsSelectable",
        "setGroupIconsVisible",
        "setGroupid",
        "setGroupIdGlobal",
        "setGroupOwner",
        "setGusts",
        "setHideBehind",
        "setHit",
        "setHitIndex",
        "setHitPointDamage",
        "setHorizonParallaxCoef",
        "setHUDMovementLevels",
        "setHumidity",
        "setIdentity",
        "setImportance",
        "setInfoPanel",
        "setLeader",
        "setLightAmbient",
        "setLightAttenuation",
        "setLightBrightness",
        "setLightColor",
        "setLightConePars",
        "setLightDayLight",
        "setLightFlareMaxDistance",
        "setLightFlareSize",
        "setLightIntensity",
        "setLightIR",
        "setLightnings",
        "setLightUseFlare",
        "setLightVolumeShape",
        "setLocalWindParams",
        "setMagazineTurretAmmo",
        "setMarkerAlpha",
        "setMarkerAlphaLocal",
        "setMarkerBrush",
        "setMarkerBrushLocal",
        "setMarkerColor",
        "setMarkerColorLocal",
        "setMarkerDir",
        "setMarkerDirLocal",
        "setMarkerPolyline",
        "setMarkerPolylineLocal",
        "setMarkerPos",
        "setMarkerPosLocal",
        "setMarkerShadow",
        "setMarkerShadowLocal",
        "setMarkerShape",
        "setMarkerShapeLocal",
        "setMarkerSize",
        "setMarkerSizeLocal",
        "setMarkerText",
        "setMarkerTextLocal",
        "setMarkerType",
        "setMarkerTypeLocal",
        "setMass",
        "setMaxLoad",
        "setMimic",
        "setMissileTarget",
        "setMissileTargetPos",
        "setMousePosition",
        "setMusicEffect",
        "setMusicEventHandler",
        "setName",
        "setNameSound",
        "setObjectArguments",
        "setObjectMaterial",
        "setObjectMaterialGlobal",
        "setObjectProxy",
        "setObjectScale",
        "setObjectTexture",
        "setObjectTextureGlobal",
        "setObjectViewDistance",
        "setOpticsMode",
        "setOvercast",
        "setOwner",
        "setOxygenRemaining",
        "setParticleCircle",
        "setParticleClass",
        "setParticleFire",
        "setParticleParams",
        "setParticleRandom",
        "setPilotCameraDirection",
        "setPilotCameraRotation",
        "setPilotCameraTarget",
        "setPilotLight",
        "setPiPEffect",
        "setPiPViewDistance",
        "setPitch",
        "setPlateNumber",
        "setPlayable",
        "setPlayerRespawnTime",
        "setPlayerVoNVolume",
        "setPos",
        "setPosASL",
        "setPosASL2",
        "setPosASLW",
        "setPosATL",
        "setPosition",
        "setPosWorld",
        "setPylonLoadout",
        "setPylonsPriority",
        "setRadioMsg",
        "setRain",
        "setRainbow",
        "setRandomLip",
        "setRank",
        "setRectangular",
        "setRepairCargo",
        "setRotorBrakeRTD",
        "setShadowDistance",
        "setShotParents",
        "setSide",
        "setSimpleTaskAlwaysVisible",
        "setSimpleTaskCustomData",
        "setSimpleTaskDescription",
        "setSimpleTaskDestination",
        "setSimpleTaskTarget",
        "setSimpleTaskType",
        "setSimulWeatherLayers",
        "setSize",
        "setSkill",
        "setSlingLoad",
        "setSoundEffect",
        "setSpeaker",
        "setSpeech",
        "setSpeedMode",
        "setStamina",
        "setStaminaScheme",
        "setStatValue",
        "setSuppression",
        "setSystemOfUnits",
        "setTargetAge",
        "setTaskMarkerOffset",
        "setTaskResult",
        "setTaskState",
        "setTerrainGrid",
        "setTerrainHeight",
        "setText",
        "setTimeMultiplier",
        "setTiParameter",
        "setTitleEffect",
        "setTowParent",
        "setTrafficDensity",
        "setTrafficDistance",
        "setTrafficGap",
        "setTrafficSpeed",
        "setTriggerActivation",
        "setTriggerArea",
        "setTriggerInterval",
        "setTriggerStatements",
        "setTriggerText",
        "setTriggerTimeout",
        "setTriggerType",
        "setTurretLimits",
        "setTurretOpticsMode",
        "setType",
        "setUnconscious",
        "setUnitAbility",
        "setUnitCombatMode",
        "setUnitFreefallHeight",
        "setUnitLoadout",
        "setUnitPos",
        "setUnitPosWeak",
        "setUnitRank",
        "setUnitRecoilCoefficient",
        "setUnitTrait",
        "setUnloadInCombat",
        "setUserActionText",
        "setUserMFDText",
        "setUserMFDValue",
        "setVariable",
        "setVectorDir",
        "setVectorDirAndUp",
        "setVectorUp",
        "setVehicleAmmo",
        "setVehicleAmmoDef",
        "setVehicleArmor",
        "setVehicleCargo",
        "setVehicleId",
        "setVehicleLock",
        "setVehiclePosition",
        "setVehicleRadar",
        "setVehicleReceiveRemoteTargets",
        "setVehicleReportOwnPosition",
        "setVehicleReportRemoteTargets",
        "setVehicleTiPars",
        "setVehicleVarName",
        "setVelocity",
        "setVelocityModelSpace",
        "setVelocityTransformation",
        "setViewDistance",
        "setVisibleIfTreeCollapsed",
        "setWantedRPMRTD",
        "setWaves",
        "setWaypointBehaviour",
        "setWaypointCombatMode",
        "setWaypointCompletionRadius",
        "setWaypointDescription",
        "setWaypointForceBehaviour",
        "setWaypointFormation",
        "setWaypointHousePosition",
        "setWaypointLoiterAltitude",
        "setWaypointLoiterRadius",
        "setWaypointLoiterType",
        "setWaypointName",
        "setWaypointPosition",
        "setWaypointScript",
        "setWaypointSpeed",
        "setWaypointStatements",
        "setWaypointTimeout",
        "setWaypointType",
        "setWaypointVisible",
        "setWeaponReloadingTime",
        "setWeaponZeroing",
        "setWind",
        "setWindDir",
        "setWindForce",
        "setWindStr",
        "setWingForceScaleRTD",
        "setWPPos",
        "show3DIcons",
        "showChat",
        "showCinemaBorder",
        "showCommandingMenu",
        "showCompass",
        "showCuratorCompass",
        "showGps",
        "showHUD",
        "showLegend",
        "showMap",
        "shownArtilleryComputer",
        "shownChat",
        "shownCompass",
        "shownCuratorCompass",
        "showNewEditorObject",
        "shownGps",
        "shownHUD",
        "shownMap",
        "shownPad",
        "shownRadio",
        "shownScoretable",
        "shownSubtitles",
        "shownUAVFeed",
        "shownWarrant",
        "shownWatch",
        "showPad",
        "showRadio",
        "showScoretable",
        "showSubtitles",
        "showUAVFeed",
        "showWarrant",
        "showWatch",
        "showWaypoint",
        "showWaypoints",
        "side",
        "sideChat",
        "sideRadio",
        "simpleTasks",
        "simulationEnabled",
        "simulCloudDensity",
        "simulCloudOcclusion",
        "simulInClouds",
        "simulWeatherSync",
        "sin",
        "size",
        "sizeOf",
        "skill",
        "skillFinal",
        "skipTime",
        "sleep",
        "sliderPosition",
        "sliderRange",
        "sliderSetPosition",
        "sliderSetRange",
        "sliderSetSpeed",
        "sliderSpeed",
        "slingLoadAssistantShown",
        "soldierMagazines",
        "someAmmo",
        "sort",
        "soundVolume",
        "spawn",
        "speaker",
        "speechVolume",
        "speed",
        "speedMode",
        "splitString",
        "sqrt",
        "squadParams",
        "stance",
        "startLoadingScreen",
        "stop",
        "stopEngineRTD",
        "stopped",
        "str",
        "sunOrMoon",
        "supportInfo",
        "suppressFor",
        "surfaceIsWater",
        "surfaceNormal",
        "surfaceTexture",
        "surfaceType",
        "swimInDepth",
        "switchableUnits",
        "switchAction",
        "switchCamera",
        "switchGesture",
        "switchLight",
        "switchMove",
        "synchronizedObjects",
        "synchronizedTriggers",
        "synchronizedWaypoints",
        "synchronizeObjectsAdd",
        "synchronizeObjectsRemove",
        "synchronizeTrigger",
        "synchronizeWaypoint",
        "systemChat",
        "systemOfUnits",
        "systemTime",
        "systemTimeUTC",
        "tan",
        "targetKnowledge",
        "targets",
        "targetsAggregate",
        "targetsQuery",
        "taskAlwaysVisible",
        "taskChildren",
        "taskCompleted",
        "taskCustomData",
        "taskDescription",
        "taskDestination",
        "taskHint",
        "taskMarkerOffset",
        "taskName",
        "taskParent",
        "taskResult",
        "taskState",
        "taskType",
        "teamMember",
        "teamName",
        "teams",
        "teamSwitch",
        "teamSwitchEnabled",
        "teamType",
        "terminate",
        "terrainIntersect",
        "terrainIntersectASL",
        "terrainIntersectAtASL",
        "text",
        "textLog",
        "textLogFormat",
        "tg",
        "time",
        "timeMultiplier",
        "titleCut",
        "titleFadeOut",
        "titleObj",
        "titleRsc",
        "titleText",
        "toArray",
        "toFixed",
        "toLower",
        "toLowerANSI",
        "toString",
        "toUpper",
        "toUpperANSI",
        "triggerActivated",
        "triggerActivation",
        "triggerAmmo",
        "triggerArea",
        "triggerAttachedVehicle",
        "triggerAttachObject",
        "triggerAttachVehicle",
        "triggerDynamicSimulation",
        "triggerInterval",
        "triggerStatements",
        "triggerText",
        "triggerTimeout",
        "triggerTimeoutCurrent",
        "triggerType",
        "trim",
        "turretLocal",
        "turretOwner",
        "turretUnit",
        "tvAdd",
        "tvClear",
        "tvCollapse",
        "tvCollapseAll",
        "tvCount",
        "tvCurSel",
        "tvData",
        "tvDelete",
        "tvExpand",
        "tvExpandAll",
        "tvIsSelected",
        "tvPicture",
        "tvPictureRight",
        "tvSelection",
        "tvSetColor",
        "tvSetCurSel",
        "tvSetData",
        "tvSetPicture",
        "tvSetPictureColor",
        "tvSetPictureColorDisabled",
        "tvSetPictureColorSelected",
        "tvSetPictureRight",
        "tvSetPictureRightColor",
        "tvSetPictureRightColorDisabled",
        "tvSetPictureRightColorSelected",
        "tvSetSelectColor",
        "tvSetSelected",
        "tvSetText",
        "tvSetTooltip",
        "tvSetValue",
        "tvSort",
        "tvSortAll",
        "tvSortByValue",
        "tvSortByValueAll",
        "tvText",
        "tvTooltip",
        "tvValue",
        "type",
        "typeName",
        "typeOf",
        "UAVControl",
        "uiNamespace",
        "uiSleep",
        "unassignCurator",
        "unassignItem",
        "unassignTeam",
        "unassignVehicle",
        "underwater",
        "uniform",
        "uniformContainer",
        "uniformItems",
        "uniformMagazines",
        "uniqueUnitItems",
        "unitAddons",
        "unitAimPosition",
        "unitAimPositionVisual",
        "unitBackpack",
        "unitCombatMode",
        "unitIsUAV",
        "unitPos",
        "unitReady",
        "unitRecoilCoefficient",
        "units",
        "unitsBelowHeight",
        "unitTurret",
        "unlinkItem",
        "unlockAchievement",
        "unregisterTask",
        "updateDrawIcon",
        "updateMenuItem",
        "updateObjectTree",
        "useAIOperMapObstructionTest",
        "useAISteeringComponent",
        "useAudioTimeForMoves",
        "userInputDisabled",
        "values",
        "vectorAdd",
        "vectorCos",
        "vectorCrossProduct",
        "vectorDiff",
        "vectorDir",
        "vectorDirVisual",
        "vectorDistance",
        "vectorDistanceSqr",
        "vectorDotProduct",
        "vectorFromTo",
        "vectorLinearConversion",
        "vectorMagnitude",
        "vectorMagnitudeSqr",
        "vectorModelToWorld",
        "vectorModelToWorldVisual",
        "vectorMultiply",
        "vectorNormalized",
        "vectorUp",
        "vectorUpVisual",
        "vectorWorldToModel",
        "vectorWorldToModelVisual",
        "vehicle",
        "vehicleCargoEnabled",
        "vehicleChat",
        "vehicleMoveInfo",
        "vehicleRadio",
        "vehicleReceiveRemoteTargets",
        "vehicleReportOwnPosition",
        "vehicleReportRemoteTargets",
        "vehicles",
        "vehicleVarName",
        "velocity",
        "velocityModelSpace",
        "verifySignature",
        "vest",
        "vestContainer",
        "vestItems",
        "vestMagazines",
        "viewDistance",
        "visibleCompass",
        "visibleGps",
        "visibleMap",
        "visiblePosition",
        "visiblePositionASL",
        "visibleScoretable",
        "visibleWatch",
        "waves",
        "waypointAttachedObject",
        "waypointAttachedVehicle",
        "waypointAttachObject",
        "waypointAttachVehicle",
        "waypointBehaviour",
        "waypointCombatMode",
        "waypointCompletionRadius",
        "waypointDescription",
        "waypointForceBehaviour",
        "waypointFormation",
        "waypointHousePosition",
        "waypointLoiterAltitude",
        "waypointLoiterRadius",
        "waypointLoiterType",
        "waypointName",
        "waypointPosition",
        "waypoints",
        "waypointScript",
        "waypointsEnabledUAV",
        "waypointShow",
        "waypointSpeed",
        "waypointStatements",
        "waypointTimeout",
        "waypointTimeoutCurrent",
        "waypointType",
        "waypointVisible",
        "weaponAccessories",
        "weaponAccessoriesCargo",
        "weaponCargo",
        "weaponDirection",
        "weaponInertia",
        "weaponLowered",
        "weaponReloadingTime",
        "weapons",
        "weaponsInfo",
        "weaponsItems",
        "weaponsItemsCargo",
        "weaponState",
        "weaponsTurret",
        "weightRTD",
        "WFSideText",
        "wind",
        "windDir",
        "windRTD",
        "windStr",
        "wingsForcesRTD",
        "worldName",
        "worldSize",
        "worldToModel",
        "worldToModelVisual",
        "worldToScreen"
      ], te = {
        className: "meta",
        begin: /#\s*[a-z]+\b/,
        end: /$/,
        keywords: "define undef ifdef ifndef else endif include if",
        contains: [
          {
            begin: /\\\n/,
            relevance: 0
          },
          _.inherit(q, { className: "string" }),
          {
            begin: /<[^\n>]*>/,
            end: /$/,
            illegal: "\\n"
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE
        ]
      };
      return {
        name: "SQF",
        case_insensitive: !0,
        keywords: {
          keyword: G,
          built_in: Z,
          literal: J
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.NUMBER_MODE,
          H,
          V,
          q,
          te
        ],
        illegal: [
          //$ is only valid when used with Hex numbers (e.g. $FF)
          /\$[^a-fA-F0-9]/,
          /\w\$/,
          /\?/,
          //There's no ? in SQF
          /@/,
          //There's no @ in SQF
          // Brute-force-fixing the build error. See https://github.com/highlightjs/highlight.js/pull/3193#issuecomment-843088729
          / \| /,
          // . is only used in numbers
          /[a-zA-Z_]\./,
          /\:\=/,
          /\[\:/
        ]
      };
    }
    return sqf_1 = O, sqf_1;
  }
  var stan_1, hasRequiredStan;
  function requireStan() {
    if (hasRequiredStan) return stan_1;
    hasRequiredStan = 1;
    function O(_) {
      const H = _.regex, V = [
        "functions",
        "model",
        "data",
        "parameters",
        "quantities",
        "transformed",
        "generated"
      ], q = [
        "for",
        "in",
        "if",
        "else",
        "while",
        "break",
        "continue",
        "return"
      ], G = [
        "array",
        "tuple",
        "complex",
        "int",
        "real",
        "vector",
        "complex_vector",
        "ordered",
        "positive_ordered",
        "simplex",
        "unit_vector",
        "row_vector",
        "complex_row_vector",
        "matrix",
        "complex_matrix",
        "cholesky_factor_corr|10",
        "cholesky_factor_cov|10",
        "corr_matrix|10",
        "cov_matrix|10",
        "void"
      ], J = [
        "abs",
        "acos",
        "acosh",
        "add_diag",
        "algebra_solver",
        "algebra_solver_newton",
        "append_array",
        "append_col",
        "append_row",
        "asin",
        "asinh",
        "atan",
        "atan2",
        "atanh",
        "bessel_first_kind",
        "bessel_second_kind",
        "binary_log_loss",
        "block",
        "cbrt",
        "ceil",
        "chol2inv",
        "cholesky_decompose",
        "choose",
        "col",
        "cols",
        "columns_dot_product",
        "columns_dot_self",
        "complex_schur_decompose",
        "complex_schur_decompose_t",
        "complex_schur_decompose_u",
        "conj",
        "cos",
        "cosh",
        "cov_exp_quad",
        "crossprod",
        "csr_extract",
        "csr_extract_u",
        "csr_extract_v",
        "csr_extract_w",
        "csr_matrix_times_vector",
        "csr_to_dense_matrix",
        "cumulative_sum",
        "dae",
        "dae_tol",
        "determinant",
        "diag_matrix",
        "diagonal",
        "diag_post_multiply",
        "diag_pre_multiply",
        "digamma",
        "dims",
        "distance",
        "dot_product",
        "dot_self",
        "eigendecompose",
        "eigendecompose_sym",
        "eigenvalues",
        "eigenvalues_sym",
        "eigenvectors",
        "eigenvectors_sym",
        "erf",
        "erfc",
        "exp",
        "exp2",
        "expm1",
        "falling_factorial",
        "fdim",
        "fft",
        "fft2",
        "floor",
        "fma",
        "fmax",
        "fmin",
        "fmod",
        "gamma_p",
        "gamma_q",
        "generalized_inverse",
        "get_imag",
        "get_real",
        "head",
        "hmm_hidden_state_prob",
        "hmm_marginal",
        "hypot",
        "identity_matrix",
        "inc_beta",
        "integrate_1d",
        "integrate_ode",
        "integrate_ode_adams",
        "integrate_ode_bdf",
        "integrate_ode_rk45",
        "int_step",
        "inv",
        "inv_cloglog",
        "inv_erfc",
        "inverse",
        "inverse_spd",
        "inv_fft",
        "inv_fft2",
        "inv_inc_beta",
        "inv_logit",
        "inv_Phi",
        "inv_sqrt",
        "inv_square",
        "is_inf",
        "is_nan",
        "lambert_w0",
        "lambert_wm1",
        "lbeta",
        "lchoose",
        "ldexp",
        "lgamma",
        "linspaced_array",
        "linspaced_int_array",
        "linspaced_row_vector",
        "linspaced_vector",
        "lmgamma",
        "lmultiply",
        "log",
        "log1m",
        "log1m_exp",
        "log1m_inv_logit",
        "log1p",
        "log1p_exp",
        "log_determinant",
        "log_diff_exp",
        "log_falling_factorial",
        "log_inv_logit",
        "log_inv_logit_diff",
        "logit",
        "log_mix",
        "log_modified_bessel_first_kind",
        "log_rising_factorial",
        "log_softmax",
        "log_sum_exp",
        "machine_precision",
        "map_rect",
        "matrix_exp",
        "matrix_exp_multiply",
        "matrix_power",
        "max",
        "mdivide_left_spd",
        "mdivide_left_tri_low",
        "mdivide_right_spd",
        "mdivide_right_tri_low",
        "mean",
        "min",
        "modified_bessel_first_kind",
        "modified_bessel_second_kind",
        "multiply_lower_tri_self_transpose",
        "negative_infinity",
        "norm",
        "norm1",
        "norm2",
        "not_a_number",
        "num_elements",
        "ode_adams",
        "ode_adams_tol",
        "ode_adjoint_tol_ctl",
        "ode_bdf",
        "ode_bdf_tol",
        "ode_ckrk",
        "ode_ckrk_tol",
        "ode_rk45",
        "ode_rk45_tol",
        "one_hot_array",
        "one_hot_int_array",
        "one_hot_row_vector",
        "one_hot_vector",
        "ones_array",
        "ones_int_array",
        "ones_row_vector",
        "ones_vector",
        "owens_t",
        "Phi",
        "Phi_approx",
        "polar",
        "positive_infinity",
        "pow",
        "print",
        "prod",
        "proj",
        "qr",
        "qr_Q",
        "qr_R",
        "qr_thin",
        "qr_thin_Q",
        "qr_thin_R",
        "quad_form",
        "quad_form_diag",
        "quad_form_sym",
        "quantile",
        "rank",
        "reduce_sum",
        "reject",
        "rep_array",
        "rep_matrix",
        "rep_row_vector",
        "rep_vector",
        "reverse",
        "rising_factorial",
        "round",
        "row",
        "rows",
        "rows_dot_product",
        "rows_dot_self",
        "scale_matrix_exp_multiply",
        "sd",
        "segment",
        "sin",
        "singular_values",
        "sinh",
        "size",
        "softmax",
        "sort_asc",
        "sort_desc",
        "sort_indices_asc",
        "sort_indices_desc",
        "sqrt",
        "square",
        "squared_distance",
        "step",
        "sub_col",
        "sub_row",
        "sum",
        "svd",
        "svd_U",
        "svd_V",
        "symmetrize_from_lower_tri",
        "tail",
        "tan",
        "tanh",
        "target",
        "tcrossprod",
        "tgamma",
        "to_array_1d",
        "to_array_2d",
        "to_complex",
        "to_int",
        "to_matrix",
        "to_row_vector",
        "to_vector",
        "trace",
        "trace_gen_quad_form",
        "trace_quad_form",
        "trigamma",
        "trunc",
        "uniform_simplex",
        "variance",
        "zeros_array",
        "zeros_int_array",
        "zeros_row_vector"
      ], Z = [
        "bernoulli",
        "bernoulli_logit",
        "bernoulli_logit_glm",
        "beta",
        "beta_binomial",
        "beta_proportion",
        "binomial",
        "binomial_logit",
        "categorical",
        "categorical_logit",
        "categorical_logit_glm",
        "cauchy",
        "chi_square",
        "dirichlet",
        "discrete_range",
        "double_exponential",
        "exp_mod_normal",
        "exponential",
        "frechet",
        "gamma",
        "gaussian_dlm_obs",
        "gumbel",
        "hmm_latent",
        "hypergeometric",
        "inv_chi_square",
        "inv_gamma",
        "inv_wishart",
        "inv_wishart_cholesky",
        "lkj_corr",
        "lkj_corr_cholesky",
        "logistic",
        "loglogistic",
        "lognormal",
        "multi_gp",
        "multi_gp_cholesky",
        "multinomial",
        "multinomial_logit",
        "multi_normal",
        "multi_normal_cholesky",
        "multi_normal_prec",
        "multi_student_cholesky_t",
        "multi_student_t",
        "multi_student_t_cholesky",
        "neg_binomial",
        "neg_binomial_2",
        "neg_binomial_2_log",
        "neg_binomial_2_log_glm",
        "normal",
        "normal_id_glm",
        "ordered_logistic",
        "ordered_logistic_glm",
        "ordered_probit",
        "pareto",
        "pareto_type_2",
        "poisson",
        "poisson_log",
        "poisson_log_glm",
        "rayleigh",
        "scaled_inv_chi_square",
        "skew_double_exponential",
        "skew_normal",
        "std_normal",
        "std_normal_log",
        "student_t",
        "uniform",
        "von_mises",
        "weibull",
        "wiener",
        "wishart",
        "wishart_cholesky"
      ], te = _.COMMENT(
        /\/\*/,
        /\*\//,
        {
          relevance: 0,
          contains: [
            {
              scope: "doctag",
              match: /@(return|param)/
            }
          ]
        }
      ), ne = {
        scope: "meta",
        begin: /#include\b/,
        end: /$/,
        contains: [
          {
            match: /[a-z][a-z-._]+/,
            scope: "string"
          },
          _.C_LINE_COMMENT_MODE
        ]
      }, ee = [
        "lower",
        "upper",
        "offset",
        "multiplier"
      ];
      return {
        name: "Stan",
        aliases: ["stanfuncs"],
        keywords: {
          $pattern: _.IDENT_RE,
          title: V,
          type: G,
          keyword: q,
          built_in: J
        },
        contains: [
          _.C_LINE_COMMENT_MODE,
          ne,
          _.HASH_COMMENT_MODE,
          te,
          {
            scope: "built_in",
            match: /\s(pi|e|sqrt2|log2|log10)(?=\()/,
            relevance: 0
          },
          {
            match: H.concat(/[<,]\s*/, H.either(...ee), /\s*=/),
            keywords: ee
          },
          {
            scope: "keyword",
            match: /\btarget(?=\s*\+=)/
          },
          {
            // highlights the 'T' in T[,] for only Stan language distributrions
            match: [
              /~\s*/,
              H.either(...Z),
              /(?:\(\))/,
              /\s*T(?=\s*\[)/
            ],
            scope: {
              2: "built_in",
              4: "keyword"
            }
          },
          {
            // highlights distributions that end with special endings
            scope: "built_in",
            keywords: Z,
            begin: H.concat(/\w*/, H.either(...Z), /(_lpdf|_lupdf|_lpmf|_cdf|_lcdf|_lccdf|_qf)(?=\s*[\(.*\)])/)
          },
          {
            // highlights distributions after ~
            begin: [
              /~/,
              /\s*/,
              H.concat(H.either(...Z), /(?=\s*[\(.*\)])/)
            ],
            scope: { 3: "built_in" }
          },
          {
            // highlights user defined distributions after ~
            begin: [
              /~/,
              /\s*\w+(?=\s*[\(.*\)])/,
              "(?!.*/\b(" + H.either(...Z) + ")\b)"
            ],
            scope: { 2: "title.function" }
          },
          {
            // highlights user defined distributions with special endings
            scope: "title.function",
            begin: /\w*(_lpdf|_lupdf|_lpmf|_cdf|_lcdf|_lccdf|_qf)(?=\s*[\(.*\)])/
          },
          {
            scope: "number",
            match: H.concat(
              // Comes from @RunDevelopment accessed 11/29/2021 at
              // https://github.com/PrismJS/prism/blob/c53ad2e65b7193ab4f03a1797506a54bbb33d5a2/components/prism-stan.js#L56
              // start of big noncapture group which
              // 1. gets numbers that are by themselves
              // 2. numbers that are separated by _
              // 3. numbers that are separted by .
              /(?:\b\d+(?:_\d+)*(?:\.(?:\d+(?:_\d+)*)?)?|\B\.\d+(?:_\d+)*)/,
              // grabs scientific notation
              // grabs complex numbers with i
              /(?:[eE][+-]?\d+(?:_\d+)*)?i?(?!\w)/
            ),
            relevance: 0
          },
          {
            scope: "string",
            begin: /"/,
            end: /"/
          }
        ]
      };
    }
    return stan_1 = O, stan_1;
  }
  var stata_1, hasRequiredStata;
  function requireStata() {
    if (hasRequiredStata) return stata_1;
    hasRequiredStata = 1;
    function O(_) {
      return {
        name: "Stata",
        aliases: [
          "do",
          "ado"
        ],
        case_insensitive: !0,
        keywords: "if else in foreach for forv forva forval forvalu forvalue forvalues by bys bysort xi quietly qui capture about ac ac_7 acprplot acprplot_7 adjust ado adopath adoupdate alpha ameans an ano anov anova anova_estat anova_terms anovadef aorder ap app appe appen append arch arch_dr arch_estat arch_p archlm areg areg_p args arima arima_dr arima_estat arima_p as asmprobit asmprobit_estat asmprobit_lf asmprobit_mfx__dlg asmprobit_p ass asse asser assert avplot avplot_7 avplots avplots_7 bcskew0 bgodfrey bias binreg bip0_lf biplot bipp_lf bipr_lf bipr_p biprobit bitest bitesti bitowt blogit bmemsize boot bootsamp bootstrap bootstrap_8 boxco_l boxco_p boxcox boxcox_6 boxcox_p bprobit br break brier bro brow brows browse brr brrstat bs bs_7 bsampl_w bsample bsample_7 bsqreg bstat bstat_7 bstat_8 bstrap bstrap_7 bubble bubbleplot ca ca_estat ca_p cabiplot camat canon canon_8 canon_8_p canon_estat canon_p cap caprojection capt captu captur capture cat cc cchart cchart_7 cci cd censobs_table centile cf char chdir checkdlgfiles checkestimationsample checkhlpfiles checksum chelp ci cii cl class classutil clear cli clis clist clo clog clog_lf clog_p clogi clogi_sw clogit clogit_lf clogit_p clogitp clogl_sw cloglog clonevar clslistarray cluster cluster_measures cluster_stop cluster_tree cluster_tree_8 clustermat cmdlog cnr cnre cnreg cnreg_p cnreg_sw cnsreg codebook collaps4 collapse colormult_nb colormult_nw compare compress conf confi confir confirm conren cons const constr constra constrai constrain constraint continue contract copy copyright copysource cor corc corr corr2data corr_anti corr_kmo corr_smc corre correl correla correlat correlate corrgram cou coun count cox cox_p cox_sw coxbase coxhaz coxvar cprplot cprplot_7 crc cret cretu cretur creturn cross cs cscript cscript_log csi ct ct_is ctset ctst_5 ctst_st cttost cumsp cumsp_7 cumul cusum cusum_7 cutil d|0 datasig datasign datasigna datasignat datasignatu datasignatur datasignature datetof db dbeta de dec deco decod decode deff des desc descr descri describ describe destring dfbeta dfgls dfuller di di_g dir dirstats dis discard disp disp_res disp_s displ displa display distinct do doe doed doedi doedit dotplot dotplot_7 dprobit drawnorm drop ds ds_util dstdize duplicates durbina dwstat dydx e|0 ed edi edit egen eivreg emdef en enc enco encod encode eq erase ereg ereg_lf ereg_p ereg_sw ereghet ereghet_glf ereghet_glf_sh ereghet_gp ereghet_ilf ereghet_ilf_sh ereghet_ip eret eretu eretur ereturn err erro error esize est est_cfexist est_cfname est_clickable est_expand est_hold est_table est_unhold est_unholdok estat estat_default estat_summ estat_vce_only esti estimates etodow etof etomdy ex exi exit expand expandcl fac fact facto factor factor_estat factor_p factor_pca_rotated factor_rotate factormat fcast fcast_compute fcast_graph fdades fdadesc fdadescr fdadescri fdadescrib fdadescribe fdasav fdasave fdause fh_st file open file read file close file filefilter fillin find_hlp_file findfile findit findit_7 fit fl fli flis flist for5_0 forest forestplot form forma format fpredict frac_154 frac_adj frac_chk frac_cox frac_ddp frac_dis frac_dv frac_in frac_mun frac_pp frac_pq frac_pv frac_wgt frac_xo fracgen fracplot fracplot_7 fracpoly fracpred fron_ex fron_hn fron_p fron_tn fron_tn2 frontier ftodate ftoe ftomdy ftowdate funnel funnelplot g|0 gamhet_glf gamhet_gp gamhet_ilf gamhet_ip gamma gamma_d2 gamma_p gamma_sw gammahet gdi_hexagon gdi_spokes ge gen gene gener genera generat generate genrank genstd genvmean gettoken gl gladder gladder_7 glim_l01 glim_l02 glim_l03 glim_l04 glim_l05 glim_l06 glim_l07 glim_l08 glim_l09 glim_l10 glim_l11 glim_l12 glim_lf glim_mu glim_nw1 glim_nw2 glim_nw3 glim_p glim_v1 glim_v2 glim_v3 glim_v4 glim_v5 glim_v6 glim_v7 glm glm_6 glm_p glm_sw glmpred glo glob globa global glogit glogit_8 glogit_p gmeans gnbre_lf gnbreg gnbreg_5 gnbreg_p gomp_lf gompe_sw gomper_p gompertz gompertzhet gomphet_glf gomphet_glf_sh gomphet_gp gomphet_ilf gomphet_ilf_sh gomphet_ip gphdot gphpen gphprint gprefs gprobi_p gprobit gprobit_8 gr gr7 gr_copy gr_current gr_db gr_describe gr_dir gr_draw gr_draw_replay gr_drop gr_edit gr_editviewopts gr_example gr_example2 gr_export gr_print gr_qscheme gr_query gr_read gr_rename gr_replay gr_save gr_set gr_setscheme gr_table gr_undo gr_use graph graph7 grebar greigen greigen_7 greigen_8 grmeanby grmeanby_7 gs_fileinfo gs_filetype gs_graphinfo gs_stat gsort gwood h|0 hadimvo hareg hausman haver he heck_d2 heckma_p heckman heckp_lf heckpr_p heckprob hel help hereg hetpr_lf hetpr_p hetprob hettest hexdump hilite hist hist_7 histogram hlogit hlu hmeans hotel hotelling hprobit hreg hsearch icd9 icd9_ff icd9p iis impute imtest inbase include inf infi infil infile infix inp inpu input ins insheet insp inspe inspec inspect integ inten intreg intreg_7 intreg_p intrg2_ll intrg_ll intrg_ll2 ipolate iqreg ir irf irf_create irfm iri is_svy is_svysum isid istdize ivprob_1_lf ivprob_lf ivprobit ivprobit_p ivreg ivreg_footnote ivtob_1_lf ivtob_lf ivtobit ivtobit_p jackknife jacknife jknife jknife_6 jknife_8 jkstat joinby kalarma1 kap kap_3 kapmeier kappa kapwgt kdensity kdensity_7 keep ksm ksmirnov ktau kwallis l|0 la lab labbe labbeplot labe label labelbook ladder levels levelsof leverage lfit lfit_p li lincom line linktest lis list lloghet_glf lloghet_glf_sh lloghet_gp lloghet_ilf lloghet_ilf_sh lloghet_ip llogi_sw llogis_p llogist llogistic llogistichet lnorm_lf lnorm_sw lnorma_p lnormal lnormalhet lnormhet_glf lnormhet_glf_sh lnormhet_gp lnormhet_ilf lnormhet_ilf_sh lnormhet_ip lnskew0 loadingplot loc loca local log logi logis_lf logistic logistic_p logit logit_estat logit_p loglogs logrank loneway lookfor lookup lowess lowess_7 lpredict lrecomp lroc lroc_7 lrtest ls lsens lsens_7 lsens_x lstat ltable ltable_7 ltriang lv lvr2plot lvr2plot_7 m|0 ma mac macr macro makecns man manova manova_estat manova_p manovatest mantel mark markin markout marksample mat mat_capp mat_order mat_put_rr mat_rapp mata mata_clear mata_describe mata_drop mata_matdescribe mata_matsave mata_matuse mata_memory mata_mlib mata_mosave mata_rename mata_which matalabel matcproc matlist matname matr matri matrix matrix_input__dlg matstrik mcc mcci md0_ md1_ md1debug_ md2_ md2debug_ mds mds_estat mds_p mdsconfig mdslong mdsmat mdsshepard mdytoe mdytof me_derd mean means median memory memsize menl meqparse mer merg merge meta mfp mfx mhelp mhodds minbound mixed_ll mixed_ll_reparm mkassert mkdir mkmat mkspline ml ml_5 ml_adjs ml_bhhhs ml_c_d ml_check ml_clear ml_cnt ml_debug ml_defd ml_e0 ml_e0_bfgs ml_e0_cycle ml_e0_dfp ml_e0i ml_e1 ml_e1_bfgs ml_e1_bhhh ml_e1_cycle ml_e1_dfp ml_e2 ml_e2_cycle ml_ebfg0 ml_ebfr0 ml_ebfr1 ml_ebh0q ml_ebhh0 ml_ebhr0 ml_ebr0i ml_ecr0i ml_edfp0 ml_edfr0 ml_edfr1 ml_edr0i ml_eds ml_eer0i ml_egr0i ml_elf ml_elf_bfgs ml_elf_bhhh ml_elf_cycle ml_elf_dfp ml_elfi ml_elfs ml_enr0i ml_enrr0 ml_erdu0 ml_erdu0_bfgs ml_erdu0_bhhh ml_erdu0_bhhhq ml_erdu0_cycle ml_erdu0_dfp ml_erdu0_nrbfgs ml_exde ml_footnote ml_geqnr ml_grad0 ml_graph ml_hbhhh ml_hd0 ml_hold ml_init ml_inv ml_log ml_max ml_mlout ml_mlout_8 ml_model ml_nb0 ml_opt ml_p ml_plot ml_query ml_rdgrd ml_repor ml_s_e ml_score ml_searc ml_technique ml_unhold mleval mlf_ mlmatbysum mlmatsum mlog mlogi mlogit mlogit_footnote mlogit_p mlopts mlsum mlvecsum mnl0_ mor more mov move mprobit mprobit_lf mprobit_p mrdu0_ mrdu1_ mvdecode mvencode mvreg mvreg_estat n|0 nbreg nbreg_al nbreg_lf nbreg_p nbreg_sw nestreg net newey newey_7 newey_p news nl nl_7 nl_9 nl_9_p nl_p nl_p_7 nlcom nlcom_p nlexp2 nlexp2_7 nlexp2a nlexp2a_7 nlexp3 nlexp3_7 nlgom3 nlgom3_7 nlgom4 nlgom4_7 nlinit nllog3 nllog3_7 nllog4 nllog4_7 nlog_rd nlogit nlogit_p nlogitgen nlogittree nlpred no nobreak noi nois noisi noisil noisily note notes notes_dlg nptrend numlabel numlist odbc old_ver olo olog ologi ologi_sw ologit ologit_p ologitp on one onew onewa oneway op_colnm op_comp op_diff op_inv op_str opr opro oprob oprob_sw oprobi oprobi_p oprobit oprobitp opts_exclusive order orthog orthpoly ou out outf outfi outfil outfile outs outsh outshe outshee outsheet ovtest pac pac_7 palette parse parse_dissim pause pca pca_8 pca_display pca_estat pca_p pca_rotate pcamat pchart pchart_7 pchi pchi_7 pcorr pctile pentium pergram pergram_7 permute permute_8 personal peto_st pkcollapse pkcross pkequiv pkexamine pkexamine_7 pkshape pksumm pksumm_7 pl plo plot plugin pnorm pnorm_7 poisgof poiss_lf poiss_sw poisso_p poisson poisson_estat post postclose postfile postutil pperron pr prais prais_e prais_e2 prais_p predict predictnl preserve print pro prob probi probit probit_estat probit_p proc_time procoverlay procrustes procrustes_estat procrustes_p profiler prog progr progra program prop proportion prtest prtesti pwcorr pwd q\\s qby qbys qchi qchi_7 qladder qladder_7 qnorm qnorm_7 qqplot qqplot_7 qreg qreg_c qreg_p qreg_sw qu quadchk quantile quantile_7 que quer query range ranksum ratio rchart rchart_7 rcof recast reclink recode reg reg3 reg3_p regdw regr regre regre_p2 regres regres_p regress regress_estat regriv_p remap ren rena renam rename renpfix repeat replace report reshape restore ret retu retur return rm rmdir robvar roccomp roccomp_7 roccomp_8 rocf_lf rocfit rocfit_8 rocgold rocplot rocplot_7 roctab roctab_7 rolling rologit rologit_p rot rota rotat rotate rotatemat rreg rreg_p ru run runtest rvfplot rvfplot_7 rvpplot rvpplot_7 sa safesum sample sampsi sav save savedresults saveold sc sca scal scala scalar scatter scm_mine sco scob_lf scob_p scobi_sw scobit scor score scoreplot scoreplot_help scree screeplot screeplot_help sdtest sdtesti se search separate seperate serrbar serrbar_7 serset set set_defaults sfrancia sh she shel shell shewhart shewhart_7 signestimationsample signrank signtest simul simul_7 simulate simulate_8 sktest sleep slogit slogit_d2 slogit_p smooth snapspan so sor sort spearman spikeplot spikeplot_7 spikeplt spline_x split sqreg sqreg_p sret sretu sretur sreturn ssc st st_ct st_hc st_hcd st_hcd_sh st_is st_issys st_note st_promo st_set st_show st_smpl st_subid stack statsby statsby_8 stbase stci stci_7 stcox stcox_estat stcox_fr stcox_fr_ll stcox_p stcox_sw stcoxkm stcoxkm_7 stcstat stcurv stcurve stcurve_7 stdes stem stepwise stereg stfill stgen stir stjoin stmc stmh stphplot stphplot_7 stphtest stphtest_7 stptime strate strate_7 streg streg_sw streset sts sts_7 stset stsplit stsum sttocc sttoct stvary stweib su suest suest_8 sum summ summa summar summari summariz summarize sunflower sureg survcurv survsum svar svar_p svmat svy svy_disp svy_dreg svy_est svy_est_7 svy_estat svy_get svy_gnbreg_p svy_head svy_header svy_heckman_p svy_heckprob_p svy_intreg_p svy_ivreg_p svy_logistic_p svy_logit_p svy_mlogit_p svy_nbreg_p svy_ologit_p svy_oprobit_p svy_poisson_p svy_probit_p svy_regress_p svy_sub svy_sub_7 svy_x svy_x_7 svy_x_p svydes svydes_8 svygen svygnbreg svyheckman svyheckprob svyintreg svyintreg_7 svyintrg svyivreg svylc svylog_p svylogit svymarkout svymarkout_8 svymean svymlog svymlogit svynbreg svyolog svyologit svyoprob svyoprobit svyopts svypois svypois_7 svypoisson svyprobit svyprobt svyprop svyprop_7 svyratio svyreg svyreg_p svyregress svyset svyset_7 svyset_8 svytab svytab_7 svytest svytotal sw sw_8 swcnreg swcox swereg swilk swlogis swlogit swologit swoprbt swpois swprobit swqreg swtobit swweib symmetry symmi symplot symplot_7 syntax sysdescribe sysdir sysuse szroeter ta tab tab1 tab2 tab_or tabd tabdi tabdis tabdisp tabi table tabodds tabodds_7 tabstat tabu tabul tabula tabulat tabulate te tempfile tempname tempvar tes test testnl testparm teststd tetrachoric time_it timer tis tob tobi tobit tobit_p tobit_sw token tokeni tokeniz tokenize tostring total translate translator transmap treat_ll treatr_p treatreg trim trimfill trnb_cons trnb_mean trpoiss_d2 trunc_ll truncr_p truncreg tsappend tset tsfill tsline tsline_ex tsreport tsrevar tsrline tsset tssmooth tsunab ttest ttesti tut_chk tut_wait tutorial tw tware_st two twoway twoway__fpfit_serset twoway__function_gen twoway__histogram_gen twoway__ipoint_serset twoway__ipoints_serset twoway__kdensity_gen twoway__lfit_serset twoway__normgen_gen twoway__pci_serset twoway__qfit_serset twoway__scatteri_serset twoway__sunflower_gen twoway_ksm_serset ty typ type typeof u|0 unab unabbrev unabcmd update us use uselabel var var_mkcompanion var_p varbasic varfcast vargranger varirf varirf_add varirf_cgraph varirf_create varirf_ctable varirf_describe varirf_dir varirf_drop varirf_erase varirf_graph varirf_ograph varirf_rename varirf_set varirf_table varlist varlmar varnorm varsoc varstable varstable_w varstable_w2 varwle vce vec vec_fevd vec_mkphi vec_p vec_p_w vecirf_create veclmar veclmar_w vecnorm vecnorm_w vecrank vecstable verinst vers versi versio version view viewsource vif vwls wdatetof webdescribe webseek webuse weib1_lf weib2_lf weib_lf weib_lf0 weibhet_glf weibhet_glf_sh weibhet_glfa weibhet_glfa_sh weibhet_gp weibhet_ilf weibhet_ilf_sh weibhet_ilfa weibhet_ilfa_sh weibhet_ip weibu_sw weibul_p weibull weibull_c weibull_s weibullhet wh whelp whi which whil while wilc_st wilcoxon win wind windo window winexec wntestb wntestb_7 wntestq xchart xchart_7 xcorr xcorr_7 xi xi_6 xmlsav xmlsave xmluse xpose xsh xshe xshel xshell xt_iis xt_tis xtab_p xtabond xtbin_p xtclog xtcloglog xtcloglog_8 xtcloglog_d2 xtcloglog_pa_p xtcloglog_re_p xtcnt_p xtcorr xtdata xtdes xtfront_p xtfrontier xtgee xtgee_elink xtgee_estat xtgee_makeivar xtgee_p xtgee_plink xtgls xtgls_p xthaus xthausman xtht_p xthtaylor xtile xtint_p xtintreg xtintreg_8 xtintreg_d2 xtintreg_p xtivp_1 xtivp_2 xtivreg xtline xtline_ex xtlogit xtlogit_8 xtlogit_d2 xtlogit_fe_p xtlogit_pa_p xtlogit_re_p xtmixed xtmixed_estat xtmixed_p xtnb_fe xtnb_lf xtnbreg xtnbreg_pa_p xtnbreg_refe_p xtpcse xtpcse_p xtpois xtpoisson xtpoisson_d2 xtpoisson_pa_p xtpoisson_refe_p xtpred xtprobit xtprobit_8 xtprobit_d2 xtprobit_re_p xtps_fe xtps_lf xtps_ren xtps_ren_8 xtrar_p xtrc xtrc_p xtrchh xtrefe_p xtreg xtreg_be xtreg_fe xtreg_ml xtreg_pa_p xtreg_re xtregar xtrere_p xtset xtsf_ll xtsf_llti xtsum xttab xttest0 xttobit xttobit_8 xttobit_p xttrans yx yxview__barlike_draw yxview_area_draw yxview_bar_draw yxview_dot_draw yxview_dropline_draw yxview_function_draw yxview_iarrow_draw yxview_ilabels_draw yxview_normal_draw yxview_pcarrow_draw yxview_pcbarrow_draw yxview_pccapsym_draw yxview_pcscatter_draw yxview_pcspike_draw yxview_rarea_draw yxview_rbar_draw yxview_rbarm_draw yxview_rcap_draw yxview_rcapsym_draw yxview_rconnected_draw yxview_rline_draw yxview_rscatter_draw yxview_rspike_draw yxview_spike_draw yxview_sunflower_draw zap_s zinb zinb_llf zinb_plf zip zip_llf zip_p zip_plf zt_ct_5 zt_hc_5 zt_hcd_5 zt_is_5 zt_iss_5 zt_sho_5 zt_smp_5 ztbase_5 ztcox_5 ztdes_5 ztereg_5 ztfill_5 ztgen_5 ztir_5 ztjoin_5 ztnb ztnb_p ztp ztp_p zts_5 ztset_5 ztspli_5 ztsum_5 zttoct_5 ztvary_5 ztweib_5",
        contains: [
          {
            className: "symbol",
            begin: /`[a-zA-Z0-9_]+'/
          },
          {
            className: "variable",
            begin: /\$\{?[a-zA-Z0-9_]+\}?/,
            relevance: 0
          },
          {
            className: "string",
            variants: [
              { begin: `\`"[^\r
]*?"'` },
              { begin: `"[^\r
"]*"` }
            ]
          },
          {
            className: "built_in",
            variants: [{ begin: "\\b(abs|acos|asin|atan|atan2|atanh|ceil|cloglog|comb|cos|digamma|exp|floor|invcloglog|invlogit|ln|lnfact|lnfactorial|lngamma|log|log10|max|min|mod|reldif|round|sign|sin|sqrt|sum|tan|tanh|trigamma|trunc|betaden|Binomial|binorm|binormal|chi2|chi2tail|dgammapda|dgammapdada|dgammapdadx|dgammapdx|dgammapdxdx|F|Fden|Ftail|gammaden|gammap|ibeta|invbinomial|invchi2|invchi2tail|invF|invFtail|invgammap|invibeta|invnchi2|invnFtail|invnibeta|invnorm|invnormal|invttail|nbetaden|nchi2|nFden|nFtail|nibeta|norm|normal|normalden|normd|npnchi2|tden|ttail|uniform|abbrev|char|index|indexnot|length|lower|ltrim|match|plural|proper|real|regexm|regexr|regexs|reverse|rtrim|string|strlen|strlower|strltrim|strmatch|strofreal|strpos|strproper|strreverse|strrtrim|strtrim|strupper|subinstr|subinword|substr|trim|upper|word|wordcount|_caller|autocode|byteorder|chop|clip|cond|e|epsdouble|epsfloat|group|inlist|inrange|irecode|matrix|maxbyte|maxdouble|maxfloat|maxint|maxlong|mi|minbyte|mindouble|minfloat|minint|minlong|missing|r|recode|replay|return|s|scalar|d|date|day|dow|doy|halfyear|mdy|month|quarter|week|year|d|daily|dofd|dofh|dofm|dofq|dofw|dofy|h|halfyearly|hofd|m|mofd|monthly|q|qofd|quarterly|tin|twithin|w|weekly|wofd|y|yearly|yh|ym|yofd|yq|yw|cholesky|colnumb|colsof|corr|det|diag|diag0cnt|el|get|hadamard|I|inv|invsym|issym|issymmetric|J|matmissing|matuniform|mreldif|nullmat|rownumb|rowsof|sweep|syminv|trace|vec|vecdiag)(?=\\()" }]
          },
          _.COMMENT("^[ 	]*\\*.*$", !1),
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE
        ]
      };
    }
    return stata_1 = O, stata_1;
  }
  var step21_1, hasRequiredStep21;
  function requireStep21() {
    if (hasRequiredStep21) return step21_1;
    hasRequiredStep21 = 1;
    function O(_) {
      return {
        name: "STEP Part 21",
        aliases: [
          "p21",
          "step",
          "stp"
        ],
        case_insensitive: !0,
        // STEP 21 is case insensitive in theory, in practice all non-comments are capitalized.
        keywords: {
          $pattern: "[A-Z_][A-Z0-9_.]*",
          keyword: [
            "HEADER",
            "ENDSEC",
            "DATA"
          ]
        },
        contains: [
          {
            className: "meta",
            begin: "ISO-10303-21;",
            relevance: 10
          },
          {
            className: "meta",
            begin: "END-ISO-10303-21;",
            relevance: 10
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          _.COMMENT("/\\*\\*!", "\\*/"),
          _.C_NUMBER_MODE,
          _.inherit(_.APOS_STRING_MODE, { illegal: null }),
          _.inherit(_.QUOTE_STRING_MODE, { illegal: null }),
          {
            className: "string",
            begin: "'",
            end: "'"
          },
          {
            className: "symbol",
            variants: [
              {
                begin: "#",
                end: "\\d+",
                illegal: "\\W"
              }
            ]
          }
        ]
      };
    }
    return step21_1 = O, step21_1;
  }
  var stylus_1, hasRequiredStylus;
  function requireStylus() {
    if (hasRequiredStylus) return stylus_1;
    hasRequiredStylus = 1;
    const O = (ne) => ({
      IMPORTANT: {
        scope: "meta",
        begin: "!important"
      },
      BLOCK_COMMENT: ne.C_BLOCK_COMMENT_MODE,
      HEXCOLOR: {
        scope: "number",
        begin: /#(([0-9a-fA-F]{3,4})|(([0-9a-fA-F]{2}){3,4}))\b/
      },
      FUNCTION_DISPATCH: {
        className: "built_in",
        begin: /[\w-]+(?=\()/
      },
      ATTRIBUTE_SELECTOR_MODE: {
        scope: "selector-attr",
        begin: /\[/,
        end: /\]/,
        illegal: "$",
        contains: [
          ne.APOS_STRING_MODE,
          ne.QUOTE_STRING_MODE
        ]
      },
      CSS_NUMBER_MODE: {
        scope: "number",
        begin: ne.NUMBER_RE + "(%|em|ex|ch|rem|vw|vh|vmin|vmax|cm|mm|in|pt|pc|px|deg|grad|rad|turn|s|ms|Hz|kHz|dpi|dpcm|dppx)?",
        relevance: 0
      },
      CSS_VARIABLE: {
        className: "attr",
        begin: /--[A-Za-z_][A-Za-z0-9_-]*/
      }
    }), _ = [
      "a",
      "abbr",
      "address",
      "article",
      "aside",
      "audio",
      "b",
      "blockquote",
      "body",
      "button",
      "canvas",
      "caption",
      "cite",
      "code",
      "dd",
      "del",
      "details",
      "dfn",
      "div",
      "dl",
      "dt",
      "em",
      "fieldset",
      "figcaption",
      "figure",
      "footer",
      "form",
      "h1",
      "h2",
      "h3",
      "h4",
      "h5",
      "h6",
      "header",
      "hgroup",
      "html",
      "i",
      "iframe",
      "img",
      "input",
      "ins",
      "kbd",
      "label",
      "legend",
      "li",
      "main",
      "mark",
      "menu",
      "nav",
      "object",
      "ol",
      "optgroup",
      "option",
      "p",
      "picture",
      "q",
      "quote",
      "samp",
      "section",
      "select",
      "source",
      "span",
      "strong",
      "summary",
      "sup",
      "table",
      "tbody",
      "td",
      "textarea",
      "tfoot",
      "th",
      "thead",
      "time",
      "tr",
      "ul",
      "var",
      "video"
    ], H = [
      "defs",
      "g",
      "marker",
      "mask",
      "pattern",
      "svg",
      "switch",
      "symbol",
      "feBlend",
      "feColorMatrix",
      "feComponentTransfer",
      "feComposite",
      "feConvolveMatrix",
      "feDiffuseLighting",
      "feDisplacementMap",
      "feFlood",
      "feGaussianBlur",
      "feImage",
      "feMerge",
      "feMorphology",
      "feOffset",
      "feSpecularLighting",
      "feTile",
      "feTurbulence",
      "linearGradient",
      "radialGradient",
      "stop",
      "circle",
      "ellipse",
      "image",
      "line",
      "path",
      "polygon",
      "polyline",
      "rect",
      "text",
      "use",
      "textPath",
      "tspan",
      "foreignObject",
      "clipPath"
    ], V = [
      ..._,
      ...H
    ], q = [
      "any-hover",
      "any-pointer",
      "aspect-ratio",
      "color",
      "color-gamut",
      "color-index",
      "device-aspect-ratio",
      "device-height",
      "device-width",
      "display-mode",
      "forced-colors",
      "grid",
      "height",
      "hover",
      "inverted-colors",
      "monochrome",
      "orientation",
      "overflow-block",
      "overflow-inline",
      "pointer",
      "prefers-color-scheme",
      "prefers-contrast",
      "prefers-reduced-motion",
      "prefers-reduced-transparency",
      "resolution",
      "scan",
      "scripting",
      "update",
      "width",
      // TODO: find a better solution?
      "min-width",
      "max-width",
      "min-height",
      "max-height"
    ].sort().reverse(), G = [
      "active",
      "any-link",
      "blank",
      "checked",
      "current",
      "default",
      "defined",
      "dir",
      // dir()
      "disabled",
      "drop",
      "empty",
      "enabled",
      "first",
      "first-child",
      "first-of-type",
      "fullscreen",
      "future",
      "focus",
      "focus-visible",
      "focus-within",
      "has",
      // has()
      "host",
      // host or host()
      "host-context",
      // host-context()
      "hover",
      "indeterminate",
      "in-range",
      "invalid",
      "is",
      // is()
      "lang",
      // lang()
      "last-child",
      "last-of-type",
      "left",
      "link",
      "local-link",
      "not",
      // not()
      "nth-child",
      // nth-child()
      "nth-col",
      // nth-col()
      "nth-last-child",
      // nth-last-child()
      "nth-last-col",
      // nth-last-col()
      "nth-last-of-type",
      //nth-last-of-type()
      "nth-of-type",
      //nth-of-type()
      "only-child",
      "only-of-type",
      "optional",
      "out-of-range",
      "past",
      "placeholder-shown",
      "read-only",
      "read-write",
      "required",
      "right",
      "root",
      "scope",
      "target",
      "target-within",
      "user-invalid",
      "valid",
      "visited",
      "where"
      // where()
    ].sort().reverse(), J = [
      "after",
      "backdrop",
      "before",
      "cue",
      "cue-region",
      "first-letter",
      "first-line",
      "grammar-error",
      "marker",
      "part",
      "placeholder",
      "selection",
      "slotted",
      "spelling-error"
    ].sort().reverse(), Z = [
      "accent-color",
      "align-content",
      "align-items",
      "align-self",
      "alignment-baseline",
      "all",
      "anchor-name",
      "animation",
      "animation-composition",
      "animation-delay",
      "animation-direction",
      "animation-duration",
      "animation-fill-mode",
      "animation-iteration-count",
      "animation-name",
      "animation-play-state",
      "animation-range",
      "animation-range-end",
      "animation-range-start",
      "animation-timeline",
      "animation-timing-function",
      "appearance",
      "aspect-ratio",
      "backdrop-filter",
      "backface-visibility",
      "background",
      "background-attachment",
      "background-blend-mode",
      "background-clip",
      "background-color",
      "background-image",
      "background-origin",
      "background-position",
      "background-position-x",
      "background-position-y",
      "background-repeat",
      "background-size",
      "baseline-shift",
      "block-size",
      "border",
      "border-block",
      "border-block-color",
      "border-block-end",
      "border-block-end-color",
      "border-block-end-style",
      "border-block-end-width",
      "border-block-start",
      "border-block-start-color",
      "border-block-start-style",
      "border-block-start-width",
      "border-block-style",
      "border-block-width",
      "border-bottom",
      "border-bottom-color",
      "border-bottom-left-radius",
      "border-bottom-right-radius",
      "border-bottom-style",
      "border-bottom-width",
      "border-collapse",
      "border-color",
      "border-end-end-radius",
      "border-end-start-radius",
      "border-image",
      "border-image-outset",
      "border-image-repeat",
      "border-image-slice",
      "border-image-source",
      "border-image-width",
      "border-inline",
      "border-inline-color",
      "border-inline-end",
      "border-inline-end-color",
      "border-inline-end-style",
      "border-inline-end-width",
      "border-inline-start",
      "border-inline-start-color",
      "border-inline-start-style",
      "border-inline-start-width",
      "border-inline-style",
      "border-inline-width",
      "border-left",
      "border-left-color",
      "border-left-style",
      "border-left-width",
      "border-radius",
      "border-right",
      "border-right-color",
      "border-right-style",
      "border-right-width",
      "border-spacing",
      "border-start-end-radius",
      "border-start-start-radius",
      "border-style",
      "border-top",
      "border-top-color",
      "border-top-left-radius",
      "border-top-right-radius",
      "border-top-style",
      "border-top-width",
      "border-width",
      "bottom",
      "box-align",
      "box-decoration-break",
      "box-direction",
      "box-flex",
      "box-flex-group",
      "box-lines",
      "box-ordinal-group",
      "box-orient",
      "box-pack",
      "box-shadow",
      "box-sizing",
      "break-after",
      "break-before",
      "break-inside",
      "caption-side",
      "caret-color",
      "clear",
      "clip",
      "clip-path",
      "clip-rule",
      "color",
      "color-interpolation",
      "color-interpolation-filters",
      "color-profile",
      "color-rendering",
      "color-scheme",
      "column-count",
      "column-fill",
      "column-gap",
      "column-rule",
      "column-rule-color",
      "column-rule-style",
      "column-rule-width",
      "column-span",
      "column-width",
      "columns",
      "contain",
      "contain-intrinsic-block-size",
      "contain-intrinsic-height",
      "contain-intrinsic-inline-size",
      "contain-intrinsic-size",
      "contain-intrinsic-width",
      "container",
      "container-name",
      "container-type",
      "content",
      "content-visibility",
      "counter-increment",
      "counter-reset",
      "counter-set",
      "cue",
      "cue-after",
      "cue-before",
      "cursor",
      "cx",
      "cy",
      "direction",
      "display",
      "dominant-baseline",
      "empty-cells",
      "enable-background",
      "field-sizing",
      "fill",
      "fill-opacity",
      "fill-rule",
      "filter",
      "flex",
      "flex-basis",
      "flex-direction",
      "flex-flow",
      "flex-grow",
      "flex-shrink",
      "flex-wrap",
      "float",
      "flood-color",
      "flood-opacity",
      "flow",
      "font",
      "font-display",
      "font-family",
      "font-feature-settings",
      "font-kerning",
      "font-language-override",
      "font-optical-sizing",
      "font-palette",
      "font-size",
      "font-size-adjust",
      "font-smooth",
      "font-smoothing",
      "font-stretch",
      "font-style",
      "font-synthesis",
      "font-synthesis-position",
      "font-synthesis-small-caps",
      "font-synthesis-style",
      "font-synthesis-weight",
      "font-variant",
      "font-variant-alternates",
      "font-variant-caps",
      "font-variant-east-asian",
      "font-variant-emoji",
      "font-variant-ligatures",
      "font-variant-numeric",
      "font-variant-position",
      "font-variation-settings",
      "font-weight",
      "forced-color-adjust",
      "gap",
      "glyph-orientation-horizontal",
      "glyph-orientation-vertical",
      "grid",
      "grid-area",
      "grid-auto-columns",
      "grid-auto-flow",
      "grid-auto-rows",
      "grid-column",
      "grid-column-end",
      "grid-column-start",
      "grid-gap",
      "grid-row",
      "grid-row-end",
      "grid-row-start",
      "grid-template",
      "grid-template-areas",
      "grid-template-columns",
      "grid-template-rows",
      "hanging-punctuation",
      "height",
      "hyphenate-character",
      "hyphenate-limit-chars",
      "hyphens",
      "icon",
      "image-orientation",
      "image-rendering",
      "image-resolution",
      "ime-mode",
      "initial-letter",
      "initial-letter-align",
      "inline-size",
      "inset",
      "inset-area",
      "inset-block",
      "inset-block-end",
      "inset-block-start",
      "inset-inline",
      "inset-inline-end",
      "inset-inline-start",
      "isolation",
      "justify-content",
      "justify-items",
      "justify-self",
      "kerning",
      "left",
      "letter-spacing",
      "lighting-color",
      "line-break",
      "line-height",
      "line-height-step",
      "list-style",
      "list-style-image",
      "list-style-position",
      "list-style-type",
      "margin",
      "margin-block",
      "margin-block-end",
      "margin-block-start",
      "margin-bottom",
      "margin-inline",
      "margin-inline-end",
      "margin-inline-start",
      "margin-left",
      "margin-right",
      "margin-top",
      "margin-trim",
      "marker",
      "marker-end",
      "marker-mid",
      "marker-start",
      "marks",
      "mask",
      "mask-border",
      "mask-border-mode",
      "mask-border-outset",
      "mask-border-repeat",
      "mask-border-slice",
      "mask-border-source",
      "mask-border-width",
      "mask-clip",
      "mask-composite",
      "mask-image",
      "mask-mode",
      "mask-origin",
      "mask-position",
      "mask-repeat",
      "mask-size",
      "mask-type",
      "masonry-auto-flow",
      "math-depth",
      "math-shift",
      "math-style",
      "max-block-size",
      "max-height",
      "max-inline-size",
      "max-width",
      "min-block-size",
      "min-height",
      "min-inline-size",
      "min-width",
      "mix-blend-mode",
      "nav-down",
      "nav-index",
      "nav-left",
      "nav-right",
      "nav-up",
      "none",
      "normal",
      "object-fit",
      "object-position",
      "offset",
      "offset-anchor",
      "offset-distance",
      "offset-path",
      "offset-position",
      "offset-rotate",
      "opacity",
      "order",
      "orphans",
      "outline",
      "outline-color",
      "outline-offset",
      "outline-style",
      "outline-width",
      "overflow",
      "overflow-anchor",
      "overflow-block",
      "overflow-clip-margin",
      "overflow-inline",
      "overflow-wrap",
      "overflow-x",
      "overflow-y",
      "overlay",
      "overscroll-behavior",
      "overscroll-behavior-block",
      "overscroll-behavior-inline",
      "overscroll-behavior-x",
      "overscroll-behavior-y",
      "padding",
      "padding-block",
      "padding-block-end",
      "padding-block-start",
      "padding-bottom",
      "padding-inline",
      "padding-inline-end",
      "padding-inline-start",
      "padding-left",
      "padding-right",
      "padding-top",
      "page",
      "page-break-after",
      "page-break-before",
      "page-break-inside",
      "paint-order",
      "pause",
      "pause-after",
      "pause-before",
      "perspective",
      "perspective-origin",
      "place-content",
      "place-items",
      "place-self",
      "pointer-events",
      "position",
      "position-anchor",
      "position-visibility",
      "print-color-adjust",
      "quotes",
      "r",
      "resize",
      "rest",
      "rest-after",
      "rest-before",
      "right",
      "rotate",
      "row-gap",
      "ruby-align",
      "ruby-position",
      "scale",
      "scroll-behavior",
      "scroll-margin",
      "scroll-margin-block",
      "scroll-margin-block-end",
      "scroll-margin-block-start",
      "scroll-margin-bottom",
      "scroll-margin-inline",
      "scroll-margin-inline-end",
      "scroll-margin-inline-start",
      "scroll-margin-left",
      "scroll-margin-right",
      "scroll-margin-top",
      "scroll-padding",
      "scroll-padding-block",
      "scroll-padding-block-end",
      "scroll-padding-block-start",
      "scroll-padding-bottom",
      "scroll-padding-inline",
      "scroll-padding-inline-end",
      "scroll-padding-inline-start",
      "scroll-padding-left",
      "scroll-padding-right",
      "scroll-padding-top",
      "scroll-snap-align",
      "scroll-snap-stop",
      "scroll-snap-type",
      "scroll-timeline",
      "scroll-timeline-axis",
      "scroll-timeline-name",
      "scrollbar-color",
      "scrollbar-gutter",
      "scrollbar-width",
      "shape-image-threshold",
      "shape-margin",
      "shape-outside",
      "shape-rendering",
      "speak",
      "speak-as",
      "src",
      // @font-face
      "stop-color",
      "stop-opacity",
      "stroke",
      "stroke-dasharray",
      "stroke-dashoffset",
      "stroke-linecap",
      "stroke-linejoin",
      "stroke-miterlimit",
      "stroke-opacity",
      "stroke-width",
      "tab-size",
      "table-layout",
      "text-align",
      "text-align-all",
      "text-align-last",
      "text-anchor",
      "text-combine-upright",
      "text-decoration",
      "text-decoration-color",
      "text-decoration-line",
      "text-decoration-skip",
      "text-decoration-skip-ink",
      "text-decoration-style",
      "text-decoration-thickness",
      "text-emphasis",
      "text-emphasis-color",
      "text-emphasis-position",
      "text-emphasis-style",
      "text-indent",
      "text-justify",
      "text-orientation",
      "text-overflow",
      "text-rendering",
      "text-shadow",
      "text-size-adjust",
      "text-transform",
      "text-underline-offset",
      "text-underline-position",
      "text-wrap",
      "text-wrap-mode",
      "text-wrap-style",
      "timeline-scope",
      "top",
      "touch-action",
      "transform",
      "transform-box",
      "transform-origin",
      "transform-style",
      "transition",
      "transition-behavior",
      "transition-delay",
      "transition-duration",
      "transition-property",
      "transition-timing-function",
      "translate",
      "unicode-bidi",
      "user-modify",
      "user-select",
      "vector-effect",
      "vertical-align",
      "view-timeline",
      "view-timeline-axis",
      "view-timeline-inset",
      "view-timeline-name",
      "view-transition-name",
      "visibility",
      "voice-balance",
      "voice-duration",
      "voice-family",
      "voice-pitch",
      "voice-range",
      "voice-rate",
      "voice-stress",
      "voice-volume",
      "white-space",
      "white-space-collapse",
      "widows",
      "width",
      "will-change",
      "word-break",
      "word-spacing",
      "word-wrap",
      "writing-mode",
      "x",
      "y",
      "z-index",
      "zoom"
    ].sort().reverse();
    function te(ne) {
      const ee = O(ne), ae = "and or not only", oe = {
        className: "variable",
        begin: "\\$" + ne.IDENT_RE
      }, se = [
        "charset",
        "css",
        "debug",
        "extend",
        "font-face",
        "for",
        "import",
        "include",
        "keyframes",
        "media",
        "mixin",
        "page",
        "warn",
        "while"
      ], ce = "(?=[.\\s\\n[:,(])";
      return {
        name: "Stylus",
        aliases: ["styl"],
        case_insensitive: !1,
        keywords: "if else for in",
        illegal: "(" + [
          "\\?",
          "(\\bReturn\\b)",
          // monkey
          "(\\bEnd\\b)",
          // monkey
          "(\\bend\\b)",
          // vbscript
          "(\\bdef\\b)",
          // gradle
          ";",
          // a whole lot of languages
          "#\\s",
          // markdown
          "\\*\\s",
          // markdown
          "===\\s",
          // markdown
          "\\|",
          "%"
          // prolog
        ].join("|") + ")",
        contains: [
          // strings
          ne.QUOTE_STRING_MODE,
          ne.APOS_STRING_MODE,
          // comments
          ne.C_LINE_COMMENT_MODE,
          ne.C_BLOCK_COMMENT_MODE,
          // hex colors
          ee.HEXCOLOR,
          // class tag
          {
            begin: "\\.[a-zA-Z][a-zA-Z0-9_-]*" + ce,
            className: "selector-class"
          },
          // id tag
          {
            begin: "#[a-zA-Z][a-zA-Z0-9_-]*" + ce,
            className: "selector-id"
          },
          // tags
          {
            begin: "\\b(" + V.join("|") + ")" + ce,
            className: "selector-tag"
          },
          // psuedo selectors
          {
            className: "selector-pseudo",
            begin: "&?:(" + G.join("|") + ")" + ce
          },
          {
            className: "selector-pseudo",
            begin: "&?:(:)?(" + J.join("|") + ")" + ce
          },
          ee.ATTRIBUTE_SELECTOR_MODE,
          {
            className: "keyword",
            begin: /@media/,
            starts: {
              end: /[{;}]/,
              keywords: {
                $pattern: /[a-z-]+/,
                keyword: ae,
                attribute: q.join(" ")
              },
              contains: [ee.CSS_NUMBER_MODE]
            }
          },
          // @ keywords
          {
            className: "keyword",
            begin: "@((-(o|moz|ms|webkit)-)?(" + se.join("|") + "))\\b"
          },
          // variables
          oe,
          // dimension
          ee.CSS_NUMBER_MODE,
          // functions
          //  - only from beginning of line + whitespace
          {
            className: "function",
            begin: "^[a-zA-Z][a-zA-Z0-9_-]*\\(.*\\)",
            illegal: "[\\n]",
            returnBegin: !0,
            contains: [
              {
                className: "title",
                begin: "\\b[a-zA-Z][a-zA-Z0-9_-]*"
              },
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                contains: [
                  ee.HEXCOLOR,
                  oe,
                  ne.APOS_STRING_MODE,
                  ee.CSS_NUMBER_MODE,
                  ne.QUOTE_STRING_MODE
                ]
              }
            ]
          },
          // css variables
          ee.CSS_VARIABLE,
          // attributes
          //  - only from beginning of line + whitespace
          //  - must have whitespace after it
          {
            className: "attribute",
            begin: "\\b(" + Z.join("|") + ")\\b",
            starts: {
              // value container
              end: /;|$/,
              contains: [
                ee.HEXCOLOR,
                oe,
                ne.APOS_STRING_MODE,
                ne.QUOTE_STRING_MODE,
                ee.CSS_NUMBER_MODE,
                ne.C_BLOCK_COMMENT_MODE,
                ee.IMPORTANT,
                ee.FUNCTION_DISPATCH
              ],
              illegal: /\./,
              relevance: 0
            }
          },
          ee.FUNCTION_DISPATCH
        ]
      };
    }
    return stylus_1 = te, stylus_1;
  }
  var subunit_1, hasRequiredSubunit;
  function requireSubunit() {
    if (hasRequiredSubunit) return subunit_1;
    hasRequiredSubunit = 1;
    function O(_) {
      return {
        name: "SubUnit",
        case_insensitive: !0,
        contains: [
          {
            className: "string",
            begin: `\\[
(multipart)?`,
            end: `\\]
`
          },
          {
            className: "string",
            begin: "\\d{4}-\\d{2}-\\d{2}(\\s+)\\d{2}:\\d{2}:\\d{2}.\\d+Z"
          },
          {
            className: "string",
            begin: "(\\+|-)\\d+"
          },
          {
            className: "keyword",
            relevance: 10,
            variants: [
              { begin: "^(test|testing|success|successful|failure|error|skip|xfail|uxsuccess)(:?)\\s+(test)?" },
              { begin: "^progress(:?)(\\s+)?(pop|push)?" },
              { begin: "^tags:" },
              { begin: "^time:" }
            ]
          }
        ]
      };
    }
    return subunit_1 = O, subunit_1;
  }
  var taggerscript_1, hasRequiredTaggerscript;
  function requireTaggerscript() {
    if (hasRequiredTaggerscript) return taggerscript_1;
    hasRequiredTaggerscript = 1;
    function O(_) {
      return {
        name: "Tagger Script",
        contains: [
          {
            className: "comment",
            begin: /\$noop\(/,
            end: /\)/,
            contains: [
              { begin: /\\[()]/ },
              {
                begin: /\(/,
                end: /\)/,
                contains: [
                  { begin: /\\[()]/ },
                  "self"
                ]
              }
            ],
            relevance: 10
          },
          {
            className: "keyword",
            begin: /\$[_a-zA-Z0-9]+(?=\()/
          },
          {
            className: "variable",
            begin: /%[_a-zA-Z0-9:]+%/
          },
          {
            className: "symbol",
            begin: /\\[\\nt$%,()]/
          },
          {
            className: "symbol",
            begin: /\\u[a-fA-F0-9]{4}/
          }
        ]
      };
    }
    return taggerscript_1 = O, taggerscript_1;
  }
  var tap_1, hasRequiredTap;
  function requireTap() {
    if (hasRequiredTap) return tap_1;
    hasRequiredTap = 1;
    function O(_) {
      return {
        name: "Test Anything Protocol",
        case_insensitive: !0,
        contains: [
          _.HASH_COMMENT_MODE,
          // version of format and total amount of testcases
          {
            className: "meta",
            variants: [
              { begin: "^TAP version (\\d+)$" },
              { begin: "^1\\.\\.(\\d+)$" }
            ]
          },
          // YAML block
          {
            begin: /---$/,
            end: "\\.\\.\\.$",
            subLanguage: "yaml",
            relevance: 0
          },
          // testcase number
          {
            className: "number",
            begin: " (\\d+) "
          },
          // testcase status and description
          {
            className: "symbol",
            variants: [
              { begin: "^ok" },
              { begin: "^not ok" }
            ]
          }
        ]
      };
    }
    return tap_1 = O, tap_1;
  }
  var tcl_1, hasRequiredTcl;
  function requireTcl() {
    if (hasRequiredTcl) return tcl_1;
    hasRequiredTcl = 1;
    function O(_) {
      const H = _.regex, V = /[a-zA-Z_][a-zA-Z0-9_]*/, q = {
        className: "number",
        variants: [
          _.BINARY_NUMBER_MODE,
          _.C_NUMBER_MODE
        ]
      };
      return {
        name: "Tcl",
        aliases: ["tk"],
        keywords: [
          "after",
          "append",
          "apply",
          "array",
          "auto_execok",
          "auto_import",
          "auto_load",
          "auto_mkindex",
          "auto_mkindex_old",
          "auto_qualify",
          "auto_reset",
          "bgerror",
          "binary",
          "break",
          "catch",
          "cd",
          "chan",
          "clock",
          "close",
          "concat",
          "continue",
          "dde",
          "dict",
          "encoding",
          "eof",
          "error",
          "eval",
          "exec",
          "exit",
          "expr",
          "fblocked",
          "fconfigure",
          "fcopy",
          "file",
          "fileevent",
          "filename",
          "flush",
          "for",
          "foreach",
          "format",
          "gets",
          "glob",
          "global",
          "history",
          "http",
          "if",
          "incr",
          "info",
          "interp",
          "join",
          "lappend|10",
          "lassign|10",
          "lindex|10",
          "linsert|10",
          "list",
          "llength|10",
          "load",
          "lrange|10",
          "lrepeat|10",
          "lreplace|10",
          "lreverse|10",
          "lsearch|10",
          "lset|10",
          "lsort|10",
          "mathfunc",
          "mathop",
          "memory",
          "msgcat",
          "namespace",
          "open",
          "package",
          "parray",
          "pid",
          "pkg::create",
          "pkg_mkIndex",
          "platform",
          "platform::shell",
          "proc",
          "puts",
          "pwd",
          "read",
          "refchan",
          "regexp",
          "registry",
          "regsub|10",
          "rename",
          "return",
          "safe",
          "scan",
          "seek",
          "set",
          "socket",
          "source",
          "split",
          "string",
          "subst",
          "switch",
          "tcl_endOfWord",
          "tcl_findLibrary",
          "tcl_startOfNextWord",
          "tcl_startOfPreviousWord",
          "tcl_wordBreakAfter",
          "tcl_wordBreakBefore",
          "tcltest",
          "tclvars",
          "tell",
          "time",
          "tm",
          "trace",
          "unknown",
          "unload",
          "unset",
          "update",
          "uplevel",
          "upvar",
          "variable",
          "vwait",
          "while"
        ],
        contains: [
          _.COMMENT(";[ \\t]*#", "$"),
          _.COMMENT("^[ \\t]*#", "$"),
          {
            beginKeywords: "proc",
            end: "[\\{]",
            excludeEnd: !0,
            contains: [
              {
                className: "title",
                begin: "[ \\t\\n\\r]+(::)?[a-zA-Z_]((::)?[a-zA-Z0-9_])*",
                end: "[ \\t\\n\\r]",
                endsWithParent: !0,
                excludeEnd: !0
              }
            ]
          },
          {
            className: "variable",
            variants: [
              { begin: H.concat(
                /\$/,
                H.optional(/::/),
                V,
                "(::",
                V,
                ")*"
              ) },
              {
                begin: "\\$\\{(::)?[a-zA-Z_]((::)?[a-zA-Z0-9_])*",
                end: "\\}",
                contains: [q]
              }
            ]
          },
          {
            className: "string",
            contains: [_.BACKSLASH_ESCAPE],
            variants: [_.inherit(_.QUOTE_STRING_MODE, { illegal: null })]
          },
          q
        ]
      };
    }
    return tcl_1 = O, tcl_1;
  }
  var thrift_1, hasRequiredThrift;
  function requireThrift() {
    if (hasRequiredThrift) return thrift_1;
    hasRequiredThrift = 1;
    function O(_) {
      const H = [
        "bool",
        "byte",
        "i16",
        "i32",
        "i64",
        "double",
        "string",
        "binary"
      ];
      return {
        name: "Thrift",
        keywords: {
          keyword: [
            "namespace",
            "const",
            "typedef",
            "struct",
            "enum",
            "service",
            "exception",
            "void",
            "oneway",
            "set",
            "list",
            "map",
            "required",
            "optional"
          ],
          type: H,
          literal: "true false"
        },
        contains: [
          _.QUOTE_STRING_MODE,
          _.NUMBER_MODE,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "class",
            beginKeywords: "struct enum service exception",
            end: /\{/,
            illegal: /\n/,
            contains: [
              _.inherit(_.TITLE_MODE, {
                // hack: eating everything after the first title
                starts: {
                  endsWithParent: !0,
                  excludeEnd: !0
                }
              })
            ]
          },
          {
            begin: "\\b(set|list|map)\\s*<",
            keywords: { type: [
              ...H,
              "set",
              "list",
              "map"
            ] },
            end: ">",
            contains: ["self"]
          }
        ]
      };
    }
    return thrift_1 = O, thrift_1;
  }
  var tp_1, hasRequiredTp;
  function requireTp() {
    if (hasRequiredTp) return tp_1;
    hasRequiredTp = 1;
    function O(_) {
      const H = {
        className: "number",
        begin: "[1-9][0-9]*",
        /* no leading zeros */
        relevance: 0
      }, V = {
        className: "symbol",
        begin: ":[^\\]]+"
      }, q = {
        className: "built_in",
        begin: "(AR|P|PAYLOAD|PR|R|SR|RSR|LBL|VR|UALM|MESSAGE|UTOOL|UFRAME|TIMER|TIMER_OVERFLOW|JOINT_MAX_SPEED|RESUME_PROG|DIAG_REC)\\[",
        end: "\\]",
        contains: [
          "self",
          H,
          V
        ]
      }, G = {
        className: "built_in",
        begin: "(AI|AO|DI|DO|F|RI|RO|UI|UO|GI|GO|SI|SO)\\[",
        end: "\\]",
        contains: [
          "self",
          H,
          _.QUOTE_STRING_MODE,
          /* for pos section at bottom */
          V
        ]
      };
      return {
        name: "TP",
        keywords: {
          keyword: [
            "ABORT",
            "ACC",
            "ADJUST",
            "AND",
            "AP_LD",
            "BREAK",
            "CALL",
            "CNT",
            "COL",
            "CONDITION",
            "CONFIG",
            "DA",
            "DB",
            "DIV",
            "DETECT",
            "ELSE",
            "END",
            "ENDFOR",
            "ERR_NUM",
            "ERROR_PROG",
            "FINE",
            "FOR",
            "GP",
            "GUARD",
            "INC",
            "IF",
            "JMP",
            "LINEAR_MAX_SPEED",
            "LOCK",
            "MOD",
            "MONITOR",
            "OFFSET",
            "Offset",
            "OR",
            "OVERRIDE",
            "PAUSE",
            "PREG",
            "PTH",
            "RT_LD",
            "RUN",
            "SELECT",
            "SKIP",
            "Skip",
            "TA",
            "TB",
            "TO",
            "TOOL_OFFSET",
            "Tool_Offset",
            "UF",
            "UT",
            "UFRAME_NUM",
            "UTOOL_NUM",
            "UNLOCK",
            "WAIT",
            "X",
            "Y",
            "Z",
            "W",
            "P",
            "R",
            "STRLEN",
            "SUBSTR",
            "FINDSTR",
            "VOFFSET",
            "PROG",
            "ATTR",
            "MN",
            "POS"
          ],
          literal: [
            "ON",
            "OFF",
            "max_speed",
            "LPOS",
            "JPOS",
            "ENABLE",
            "DISABLE",
            "START",
            "STOP",
            "RESET"
          ]
        },
        contains: [
          q,
          G,
          {
            className: "keyword",
            begin: "/(PROG|ATTR|MN|POS|END)\\b"
          },
          {
            /* this is for cases like ,CALL */
            className: "keyword",
            begin: "(CALL|RUN|POINT_LOGIC|LBL)\\b"
          },
          {
            /* this is for cases like CNT100 where the default lexemes do not
             * separate the keyword and the number */
            className: "keyword",
            begin: "\\b(ACC|CNT|Skip|Offset|PSPD|RT_LD|AP_LD|Tool_Offset)"
          },
          {
            /* to catch numbers that do not have a word boundary on the left */
            className: "number",
            begin: "\\d+(sec|msec|mm/sec|cm/min|inch/min|deg/sec|mm|in|cm)?\\b",
            relevance: 0
          },
          _.COMMENT("//", "[;$]"),
          _.COMMENT("!", "[;$]"),
          _.COMMENT("--eg:", "$"),
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            begin: "'",
            end: "'"
          },
          _.C_NUMBER_MODE,
          {
            className: "variable",
            begin: "\\$[A-Za-z0-9_]+"
          }
        ]
      };
    }
    return tp_1 = O, tp_1;
  }
  var twig_1, hasRequiredTwig;
  function requireTwig() {
    if (hasRequiredTwig) return twig_1;
    hasRequiredTwig = 1;
    function O(_) {
      const H = _.regex, V = [
        "absolute_url",
        "asset|0",
        "asset_version",
        "attribute",
        "block",
        "constant",
        "controller|0",
        "country_timezones",
        "csrf_token",
        "cycle",
        "date",
        "dump",
        "expression",
        "form|0",
        "form_end",
        "form_errors",
        "form_help",
        "form_label",
        "form_rest",
        "form_row",
        "form_start",
        "form_widget",
        "html_classes",
        "include",
        "is_granted",
        "logout_path",
        "logout_url",
        "max",
        "min",
        "parent",
        "path|0",
        "random",
        "range",
        "relative_path",
        "render",
        "render_esi",
        "source",
        "template_from_string",
        "url|0"
      ], q = [
        "abs",
        "abbr_class",
        "abbr_method",
        "batch",
        "capitalize",
        "column",
        "convert_encoding",
        "country_name",
        "currency_name",
        "currency_symbol",
        "data_uri",
        "date",
        "date_modify",
        "default",
        "escape",
        "file_excerpt",
        "file_link",
        "file_relative",
        "filter",
        "first",
        "format",
        "format_args",
        "format_args_as_text",
        "format_currency",
        "format_date",
        "format_datetime",
        "format_file",
        "format_file_from_text",
        "format_number",
        "format_time",
        "html_to_markdown",
        "humanize",
        "inky_to_html",
        "inline_css",
        "join",
        "json_encode",
        "keys",
        "language_name",
        "last",
        "length",
        "locale_name",
        "lower",
        "map",
        "markdown",
        "markdown_to_html",
        "merge",
        "nl2br",
        "number_format",
        "raw",
        "reduce",
        "replace",
        "reverse",
        "round",
        "slice",
        "slug",
        "sort",
        "spaceless",
        "split",
        "striptags",
        "timezone_name",
        "title",
        "trans",
        "transchoice",
        "trim",
        "u|0",
        "upper",
        "url_encode",
        "yaml_dump",
        "yaml_encode"
      ];
      let G = [
        "apply",
        "autoescape",
        "block",
        "cache",
        "deprecated",
        "do",
        "embed",
        "extends",
        "filter",
        "flush",
        "for",
        "form_theme",
        "from",
        "if",
        "import",
        "include",
        "macro",
        "sandbox",
        "set",
        "stopwatch",
        "trans",
        "trans_default_domain",
        "transchoice",
        "use",
        "verbatim",
        "with"
      ];
      G = G.concat(G.map((le) => `end${le}`));
      const J = {
        scope: "string",
        variants: [
          {
            begin: /'/,
            end: /'/
          },
          {
            begin: /"/,
            end: /"/
          }
        ]
      }, Z = {
        scope: "number",
        match: /\d+/
      }, te = {
        begin: /\(/,
        end: /\)/,
        excludeBegin: !0,
        excludeEnd: !0,
        contains: [
          J,
          Z
        ]
      }, ne = {
        beginKeywords: V.join(" "),
        keywords: { name: V },
        relevance: 0,
        contains: [te]
      }, ee = {
        match: /\|(?=[A-Za-z_]+:?)/,
        beginScope: "punctuation",
        relevance: 0,
        contains: [
          {
            match: /[A-Za-z_]+:?/,
            keywords: q
          }
        ]
      }, ae = (le, { relevance: ue }) => ({
        beginScope: {
          1: "template-tag",
          3: "name"
        },
        relevance: ue || 2,
        endScope: "template-tag",
        begin: [
          /\{%/,
          /\s*/,
          H.either(...le)
        ],
        end: /%\}/,
        keywords: "in",
        contains: [
          ee,
          ne,
          J,
          Z
        ]
      }), oe = /[a-z_]+/, se = ae(G, { relevance: 2 }), ce = ae([oe], { relevance: 1 });
      return {
        name: "Twig",
        aliases: ["craftcms"],
        case_insensitive: !0,
        subLanguage: "xml",
        contains: [
          _.COMMENT(/\{#/, /#\}/),
          se,
          ce,
          {
            className: "template-variable",
            begin: /\{\{/,
            end: /\}\}/,
            contains: [
              "self",
              ee,
              ne,
              J,
              Z
            ]
          }
        ]
      };
    }
    return twig_1 = O, twig_1;
  }
  var vala_1, hasRequiredVala;
  function requireVala() {
    if (hasRequiredVala) return vala_1;
    hasRequiredVala = 1;
    function O(_) {
      return {
        name: "Vala",
        keywords: {
          keyword: (
            // Value types
            "char uchar unichar int uint long ulong short ushort int8 int16 int32 int64 uint8 uint16 uint32 uint64 float double bool struct enum string void weak unowned owned async signal static abstract interface override virtual delegate if while do for foreach else switch case break default return try catch public private protected internal using new this get set const stdout stdin stderr var"
          ),
          built_in: "DBus GLib CCode Gee Object Gtk Posix",
          literal: "false true null"
        },
        contains: [
          {
            className: "class",
            beginKeywords: "class interface namespace",
            end: /\{/,
            excludeEnd: !0,
            illegal: "[^,:\\n\\s\\.]",
            contains: [_.UNDERSCORE_TITLE_MODE]
          },
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          {
            className: "string",
            begin: '"""',
            end: '"""',
            relevance: 5
          },
          _.APOS_STRING_MODE,
          _.QUOTE_STRING_MODE,
          _.C_NUMBER_MODE,
          {
            className: "meta",
            begin: "^#",
            end: "$"
          }
        ]
      };
    }
    return vala_1 = O, vala_1;
  }
  var vbscript_1, hasRequiredVbscript;
  function requireVbscript() {
    if (hasRequiredVbscript) return vbscript_1;
    hasRequiredVbscript = 1;
    function O(_) {
      const H = _.regex, V = [
        "lcase",
        "month",
        "vartype",
        "instrrev",
        "ubound",
        "setlocale",
        "getobject",
        "rgb",
        "getref",
        "string",
        "weekdayname",
        "rnd",
        "dateadd",
        "monthname",
        "now",
        "day",
        "minute",
        "isarray",
        "cbool",
        "round",
        "formatcurrency",
        "conversions",
        "csng",
        "timevalue",
        "second",
        "year",
        "space",
        "abs",
        "clng",
        "timeserial",
        "fixs",
        "len",
        "asc",
        "isempty",
        "maths",
        "dateserial",
        "atn",
        "timer",
        "isobject",
        "filter",
        "weekday",
        "datevalue",
        "ccur",
        "isdate",
        "instr",
        "datediff",
        "formatdatetime",
        "replace",
        "isnull",
        "right",
        "sgn",
        "array",
        "snumeric",
        "log",
        "cdbl",
        "hex",
        "chr",
        "lbound",
        "msgbox",
        "ucase",
        "getlocale",
        "cos",
        "cdate",
        "cbyte",
        "rtrim",
        "join",
        "hour",
        "oct",
        "typename",
        "trim",
        "strcomp",
        "int",
        "createobject",
        "loadpicture",
        "tan",
        "formatnumber",
        "mid",
        "split",
        "cint",
        "sin",
        "datepart",
        "ltrim",
        "sqr",
        "time",
        "derived",
        "eval",
        "date",
        "formatpercent",
        "exp",
        "inputbox",
        "left",
        "ascw",
        "chrw",
        "regexp",
        "cstr",
        "err"
      ], q = [
        "server",
        "response",
        "request",
        // take no arguments so can be called without ()
        "scriptengine",
        "scriptenginebuildversion",
        "scriptengineminorversion",
        "scriptenginemajorversion"
      ], G = {
        begin: H.concat(H.either(...V), "\\s*\\("),
        // relevance 0 because this is acting as a beginKeywords really
        relevance: 0,
        keywords: { built_in: V }
      };
      return {
        name: "VBScript",
        aliases: ["vbs"],
        case_insensitive: !0,
        keywords: {
          keyword: [
            "call",
            "class",
            "const",
            "dim",
            "do",
            "loop",
            "erase",
            "execute",
            "executeglobal",
            "exit",
            "for",
            "each",
            "next",
            "function",
            "if",
            "then",
            "else",
            "on",
            "error",
            "option",
            "explicit",
            "new",
            "private",
            "property",
            "let",
            "get",
            "public",
            "randomize",
            "redim",
            "rem",
            "select",
            "case",
            "set",
            "stop",
            "sub",
            "while",
            "wend",
            "with",
            "end",
            "to",
            "elseif",
            "is",
            "or",
            "xor",
            "and",
            "not",
            "class_initialize",
            "class_terminate",
            "default",
            "preserve",
            "in",
            "me",
            "byval",
            "byref",
            "step",
            "resume",
            "goto"
          ],
          built_in: q,
          literal: [
            "true",
            "false",
            "null",
            "nothing",
            "empty"
          ]
        },
        illegal: "//",
        contains: [
          G,
          _.inherit(_.QUOTE_STRING_MODE, { contains: [{ begin: '""' }] }),
          _.COMMENT(
            /'/,
            /$/,
            { relevance: 0 }
          ),
          _.C_NUMBER_MODE
        ]
      };
    }
    return vbscript_1 = O, vbscript_1;
  }
  var vbscriptHtml_1, hasRequiredVbscriptHtml;
  function requireVbscriptHtml() {
    if (hasRequiredVbscriptHtml) return vbscriptHtml_1;
    hasRequiredVbscriptHtml = 1;
    function O(_) {
      return {
        name: "VBScript in HTML",
        subLanguage: "xml",
        contains: [
          {
            begin: "<%",
            end: "%>",
            subLanguage: "vbscript"
          }
        ]
      };
    }
    return vbscriptHtml_1 = O, vbscriptHtml_1;
  }
  var verilog_1, hasRequiredVerilog;
  function requireVerilog() {
    if (hasRequiredVerilog) return verilog_1;
    hasRequiredVerilog = 1;
    function O(_) {
      const H = _.regex, V = {
        $pattern: /\$?[\w]+(\$[\w]+)*/,
        keyword: [
          "accept_on",
          "alias",
          "always",
          "always_comb",
          "always_ff",
          "always_latch",
          "and",
          "assert",
          "assign",
          "assume",
          "automatic",
          "before",
          "begin",
          "bind",
          "bins",
          "binsof",
          "bit",
          "break",
          "buf|0",
          "bufif0",
          "bufif1",
          "byte",
          "case",
          "casex",
          "casez",
          "cell",
          "chandle",
          "checker",
          "class",
          "clocking",
          "cmos",
          "config",
          "const",
          "constraint",
          "context",
          "continue",
          "cover",
          "covergroup",
          "coverpoint",
          "cross",
          "deassign",
          "default",
          "defparam",
          "design",
          "disable",
          "dist",
          "do",
          "edge",
          "else",
          "end",
          "endcase",
          "endchecker",
          "endclass",
          "endclocking",
          "endconfig",
          "endfunction",
          "endgenerate",
          "endgroup",
          "endinterface",
          "endmodule",
          "endpackage",
          "endprimitive",
          "endprogram",
          "endproperty",
          "endspecify",
          "endsequence",
          "endtable",
          "endtask",
          "enum",
          "event",
          "eventually",
          "expect",
          "export",
          "extends",
          "extern",
          "final",
          "first_match",
          "for",
          "force",
          "foreach",
          "forever",
          "fork",
          "forkjoin",
          "function",
          "generate|5",
          "genvar",
          "global",
          "highz0",
          "highz1",
          "if",
          "iff",
          "ifnone",
          "ignore_bins",
          "illegal_bins",
          "implements",
          "implies",
          "import",
          "incdir",
          "include",
          "initial",
          "inout",
          "input",
          "inside",
          "instance",
          "int",
          "integer",
          "interconnect",
          "interface",
          "intersect",
          "join",
          "join_any",
          "join_none",
          "large",
          "let",
          "liblist",
          "library",
          "local",
          "localparam",
          "logic",
          "longint",
          "macromodule",
          "matches",
          "medium",
          "modport",
          "module",
          "nand",
          "negedge",
          "nettype",
          "new",
          "nexttime",
          "nmos",
          "nor",
          "noshowcancelled",
          "not",
          "notif0",
          "notif1",
          "or",
          "output",
          "package",
          "packed",
          "parameter",
          "pmos",
          "posedge",
          "primitive",
          "priority",
          "program",
          "property",
          "protected",
          "pull0",
          "pull1",
          "pulldown",
          "pullup",
          "pulsestyle_ondetect",
          "pulsestyle_onevent",
          "pure",
          "rand",
          "randc",
          "randcase",
          "randsequence",
          "rcmos",
          "real",
          "realtime",
          "ref",
          "reg",
          "reject_on",
          "release",
          "repeat",
          "restrict",
          "return",
          "rnmos",
          "rpmos",
          "rtran",
          "rtranif0",
          "rtranif1",
          "s_always",
          "s_eventually",
          "s_nexttime",
          "s_until",
          "s_until_with",
          "scalared",
          "sequence",
          "shortint",
          "shortreal",
          "showcancelled",
          "signed",
          "small",
          "soft",
          "solve",
          "specify",
          "specparam",
          "static",
          "string",
          "strong",
          "strong0",
          "strong1",
          "struct",
          "super",
          "supply0",
          "supply1",
          "sync_accept_on",
          "sync_reject_on",
          "table",
          "tagged",
          "task",
          "this",
          "throughout",
          "time",
          "timeprecision",
          "timeunit",
          "tran",
          "tranif0",
          "tranif1",
          "tri",
          "tri0",
          "tri1",
          "triand",
          "trior",
          "trireg",
          "type",
          "typedef",
          "union",
          "unique",
          "unique0",
          "unsigned",
          "until",
          "until_with",
          "untyped",
          "use",
          "uwire",
          "var",
          "vectored",
          "virtual",
          "void",
          "wait",
          "wait_order",
          "wand",
          "weak",
          "weak0",
          "weak1",
          "while",
          "wildcard",
          "wire",
          "with",
          "within",
          "wor",
          "xnor",
          "xor"
        ],
        literal: ["null"],
        built_in: [
          "$finish",
          "$stop",
          "$exit",
          "$fatal",
          "$error",
          "$warning",
          "$info",
          "$realtime",
          "$time",
          "$printtimescale",
          "$bitstoreal",
          "$bitstoshortreal",
          "$itor",
          "$signed",
          "$cast",
          "$bits",
          "$stime",
          "$timeformat",
          "$realtobits",
          "$shortrealtobits",
          "$rtoi",
          "$unsigned",
          "$asserton",
          "$assertkill",
          "$assertpasson",
          "$assertfailon",
          "$assertnonvacuouson",
          "$assertoff",
          "$assertcontrol",
          "$assertpassoff",
          "$assertfailoff",
          "$assertvacuousoff",
          "$isunbounded",
          "$sampled",
          "$fell",
          "$changed",
          "$past_gclk",
          "$fell_gclk",
          "$changed_gclk",
          "$rising_gclk",
          "$steady_gclk",
          "$coverage_control",
          "$coverage_get",
          "$coverage_save",
          "$set_coverage_db_name",
          "$rose",
          "$stable",
          "$past",
          "$rose_gclk",
          "$stable_gclk",
          "$future_gclk",
          "$falling_gclk",
          "$changing_gclk",
          "$display",
          "$coverage_get_max",
          "$coverage_merge",
          "$get_coverage",
          "$load_coverage_db",
          "$typename",
          "$unpacked_dimensions",
          "$left",
          "$low",
          "$increment",
          "$clog2",
          "$ln",
          "$log10",
          "$exp",
          "$sqrt",
          "$pow",
          "$floor",
          "$ceil",
          "$sin",
          "$cos",
          "$tan",
          "$countbits",
          "$onehot",
          "$isunknown",
          "$fatal",
          "$warning",
          "$dimensions",
          "$right",
          "$high",
          "$size",
          "$asin",
          "$acos",
          "$atan",
          "$atan2",
          "$hypot",
          "$sinh",
          "$cosh",
          "$tanh",
          "$asinh",
          "$acosh",
          "$atanh",
          "$countones",
          "$onehot0",
          "$error",
          "$info",
          "$random",
          "$dist_chi_square",
          "$dist_erlang",
          "$dist_exponential",
          "$dist_normal",
          "$dist_poisson",
          "$dist_t",
          "$dist_uniform",
          "$q_initialize",
          "$q_remove",
          "$q_exam",
          "$async$and$array",
          "$async$nand$array",
          "$async$or$array",
          "$async$nor$array",
          "$sync$and$array",
          "$sync$nand$array",
          "$sync$or$array",
          "$sync$nor$array",
          "$q_add",
          "$q_full",
          "$psprintf",
          "$async$and$plane",
          "$async$nand$plane",
          "$async$or$plane",
          "$async$nor$plane",
          "$sync$and$plane",
          "$sync$nand$plane",
          "$sync$or$plane",
          "$sync$nor$plane",
          "$system",
          "$display",
          "$displayb",
          "$displayh",
          "$displayo",
          "$strobe",
          "$strobeb",
          "$strobeh",
          "$strobeo",
          "$write",
          "$readmemb",
          "$readmemh",
          "$writememh",
          "$value$plusargs",
          "$dumpvars",
          "$dumpon",
          "$dumplimit",
          "$dumpports",
          "$dumpportson",
          "$dumpportslimit",
          "$writeb",
          "$writeh",
          "$writeo",
          "$monitor",
          "$monitorb",
          "$monitorh",
          "$monitoro",
          "$writememb",
          "$dumpfile",
          "$dumpoff",
          "$dumpall",
          "$dumpflush",
          "$dumpportsoff",
          "$dumpportsall",
          "$dumpportsflush",
          "$fclose",
          "$fdisplay",
          "$fdisplayb",
          "$fdisplayh",
          "$fdisplayo",
          "$fstrobe",
          "$fstrobeb",
          "$fstrobeh",
          "$fstrobeo",
          "$swrite",
          "$swriteb",
          "$swriteh",
          "$swriteo",
          "$fscanf",
          "$fread",
          "$fseek",
          "$fflush",
          "$feof",
          "$fopen",
          "$fwrite",
          "$fwriteb",
          "$fwriteh",
          "$fwriteo",
          "$fmonitor",
          "$fmonitorb",
          "$fmonitorh",
          "$fmonitoro",
          "$sformat",
          "$sformatf",
          "$fgetc",
          "$ungetc",
          "$fgets",
          "$sscanf",
          "$rewind",
          "$ftell",
          "$ferror"
        ]
      }, q = [
        "__FILE__",
        "__LINE__"
      ], G = [
        "begin_keywords",
        "celldefine",
        "default_nettype",
        "default_decay_time",
        "default_trireg_strength",
        "define",
        "delay_mode_distributed",
        "delay_mode_path",
        "delay_mode_unit",
        "delay_mode_zero",
        "else",
        "elsif",
        "end_keywords",
        "endcelldefine",
        "endif",
        "ifdef",
        "ifndef",
        "include",
        "line",
        "nounconnected_drive",
        "pragma",
        "resetall",
        "timescale",
        "unconnected_drive",
        "undef",
        "undefineall"
      ];
      return {
        name: "Verilog",
        aliases: [
          "v",
          "sv",
          "svh"
        ],
        case_insensitive: !1,
        keywords: V,
        contains: [
          _.C_BLOCK_COMMENT_MODE,
          _.C_LINE_COMMENT_MODE,
          _.QUOTE_STRING_MODE,
          {
            scope: "number",
            contains: [_.BACKSLASH_ESCAPE],
            variants: [
              { begin: /\b((\d+'([bhodBHOD]))[0-9xzXZa-fA-F_]+)/ },
              { begin: /\B(('([bhodBHOD]))[0-9xzXZa-fA-F_]+)/ },
              {
                // decimal
                begin: /\b[0-9][0-9_]*/,
                relevance: 0
              }
            ]
          },
          /* parameters to instances */
          {
            scope: "variable",
            variants: [
              { begin: "#\\((?!parameter).+\\)" },
              {
                begin: "\\.\\w+",
                relevance: 0
              }
            ]
          },
          {
            scope: "variable.constant",
            match: H.concat(/`/, H.either(...q))
          },
          {
            scope: "meta",
            begin: H.concat(/`/, H.either(...G)),
            end: /$|\/\/|\/\*/,
            returnEnd: !0,
            keywords: G
          }
        ]
      };
    }
    return verilog_1 = O, verilog_1;
  }
  var vhdl_1, hasRequiredVhdl;
  function requireVhdl() {
    if (hasRequiredVhdl) return vhdl_1;
    hasRequiredVhdl = 1;
    function O(_) {
      const H = "\\d(_|\\d)*", V = "[eE][-+]?" + H, q = H + "(\\." + H + ")?(" + V + ")?", G = "\\w+", Z = "\\b(" + (H + "#" + G + "(\\." + G + ")?#(" + V + ")?") + "|" + q + ")";
      return {
        name: "VHDL",
        case_insensitive: !0,
        keywords: {
          keyword: [
            "abs",
            "access",
            "after",
            "alias",
            "all",
            "and",
            "architecture",
            "array",
            "assert",
            "assume",
            "assume_guarantee",
            "attribute",
            "begin",
            "block",
            "body",
            "buffer",
            "bus",
            "case",
            "component",
            "configuration",
            "constant",
            "context",
            "cover",
            "disconnect",
            "downto",
            "default",
            "else",
            "elsif",
            "end",
            "entity",
            "exit",
            "fairness",
            "file",
            "for",
            "force",
            "function",
            "generate",
            "generic",
            "group",
            "guarded",
            "if",
            "impure",
            "in",
            "inertial",
            "inout",
            "is",
            "label",
            "library",
            "linkage",
            "literal",
            "loop",
            "map",
            "mod",
            "nand",
            "new",
            "next",
            "nor",
            "not",
            "null",
            "of",
            "on",
            "open",
            "or",
            "others",
            "out",
            "package",
            "parameter",
            "port",
            "postponed",
            "procedure",
            "process",
            "property",
            "protected",
            "pure",
            "range",
            "record",
            "register",
            "reject",
            "release",
            "rem",
            "report",
            "restrict",
            "restrict_guarantee",
            "return",
            "rol",
            "ror",
            "select",
            "sequence",
            "severity",
            "shared",
            "signal",
            "sla",
            "sll",
            "sra",
            "srl",
            "strong",
            "subtype",
            "then",
            "to",
            "transport",
            "type",
            "unaffected",
            "units",
            "until",
            "use",
            "variable",
            "view",
            "vmode",
            "vprop",
            "vunit",
            "wait",
            "when",
            "while",
            "with",
            "xnor",
            "xor"
          ],
          built_in: [
            "boolean",
            "bit",
            "character",
            "integer",
            "time",
            "delay_length",
            "natural",
            "positive",
            "string",
            "bit_vector",
            "file_open_kind",
            "file_open_status",
            "std_logic",
            "std_logic_vector",
            "unsigned",
            "signed",
            "boolean_vector",
            "integer_vector",
            "std_ulogic",
            "std_ulogic_vector",
            "unresolved_unsigned",
            "u_unsigned",
            "unresolved_signed",
            "u_signed",
            "real_vector",
            "time_vector"
          ],
          literal: [
            // severity_level
            "false",
            "true",
            "note",
            "warning",
            "error",
            "failure",
            // textio
            "line",
            "text",
            "side",
            "width"
          ]
        },
        illegal: /\{/,
        contains: [
          _.C_BLOCK_COMMENT_MODE,
          // VHDL-2008 block commenting.
          _.COMMENT("--", "$"),
          _.QUOTE_STRING_MODE,
          {
            className: "number",
            begin: Z,
            relevance: 0
          },
          {
            className: "string",
            begin: "'(U|X|0|1|Z|W|L|H|-)'",
            contains: [_.BACKSLASH_ESCAPE]
          },
          {
            className: "symbol",
            begin: "'[A-Za-z](_?[A-Za-z0-9])*",
            contains: [_.BACKSLASH_ESCAPE]
          }
        ]
      };
    }
    return vhdl_1 = O, vhdl_1;
  }
  var vim_1, hasRequiredVim;
  function requireVim() {
    if (hasRequiredVim) return vim_1;
    hasRequiredVim = 1;
    function O(_) {
      return {
        name: "Vim Script",
        keywords: {
          $pattern: /[!#@\w]+/,
          keyword: (
            // express version except: ! & * < = > !! # @ @@
            "N|0 P|0 X|0 a|0 ab abc abo al am an|0 ar arga argd arge argdo argg argl argu as au aug aun b|0 bN ba bad bd be bel bf bl bm bn bo bp br brea breaka breakd breakl bro bufdo buffers bun bw c|0 cN cNf ca cabc caddb cad caddf cal cat cb cc ccl cd ce cex cf cfir cgetb cgete cg changes chd che checkt cl cla clo cm cmapc cme cn cnew cnf cno cnorea cnoreme co col colo com comc comp con conf cope cp cpf cq cr cs cst cu cuna cunme cw delm deb debugg delc delf dif diffg diffo diffp diffpu diffs diffthis dig di dl dell dj dli do doautoa dp dr ds dsp e|0 ea ec echoe echoh echom echon el elsei em en endfo endf endt endw ene ex exe exi exu f|0 files filet fin fina fini fir fix fo foldc foldd folddoc foldo for fu go gr grepa gu gv ha helpf helpg helpt hi hid his ia iabc if ij il im imapc ime ino inorea inoreme int is isp iu iuna iunme j|0 ju k|0 keepa kee keepj lN lNf l|0 lad laddb laddf la lan lat lb lc lch lcl lcs le lefta let lex lf lfir lgetb lgete lg lgr lgrepa lh ll lla lli lmak lm lmapc lne lnew lnf ln loadk lo loc lockv lol lope lp lpf lr ls lt lu lua luad luaf lv lvimgrepa lw m|0 ma mak map mapc marks mat me menut mes mk mks mksp mkv mkvie mod mz mzf nbc nb nbs new nm nmapc nme nn nnoreme noa no noh norea noreme norm nu nun nunme ol o|0 om omapc ome on ono onoreme opt ou ounme ow p|0 profd prof pro promptr pc ped pe perld po popu pp pre prev ps pt ptN ptf ptj ptl ptn ptp ptr pts pu pw py3 python3 py3d py3f py pyd pyf quita qa rec red redi redr redraws reg res ret retu rew ri rightb rub rubyd rubyf rund ru rv sN san sa sal sav sb sbN sba sbf sbl sbm sbn sbp sbr scrip scripte scs se setf setg setl sf sfir sh sim sig sil sl sla sm smap smapc sme sn sni sno snor snoreme sor so spelld spe spelli spellr spellu spellw sp spr sre st sta startg startr star stopi stj sts sun sunm sunme sus sv sw sy synti sync tN tabN tabc tabdo tabe tabf tabfir tabl tabm tabnew tabn tabo tabp tabr tabs tab ta tags tc tcld tclf te tf th tj tl tm tn to tp tr try ts tu u|0 undoj undol una unh unl unlo unm unme uns up ve verb vert vim vimgrepa vi viu vie vm vmapc vme vne vn vnoreme vs vu vunme windo w|0 wN wa wh wi winc winp wn wp wq wqa ws wu wv x|0 xa xmapc xm xme xn xnoreme xu xunme y|0 z|0 ~ Next Print append abbreviate abclear aboveleft all amenu anoremenu args argadd argdelete argedit argglobal arglocal argument ascii autocmd augroup aunmenu buffer bNext ball badd bdelete behave belowright bfirst blast bmodified bnext botright bprevious brewind break breakadd breakdel breaklist browse bunload bwipeout change cNext cNfile cabbrev cabclear caddbuffer caddexpr caddfile call catch cbuffer cclose center cexpr cfile cfirst cgetbuffer cgetexpr cgetfile chdir checkpath checktime clist clast close cmap cmapclear cmenu cnext cnewer cnfile cnoremap cnoreabbrev cnoremenu copy colder colorscheme command comclear compiler continue confirm copen cprevious cpfile cquit crewind cscope cstag cunmap cunabbrev cunmenu cwindow delete delmarks debug debuggreedy delcommand delfunction diffupdate diffget diffoff diffpatch diffput diffsplit digraphs display deletel djump dlist doautocmd doautoall deletep drop dsearch dsplit edit earlier echo echoerr echohl echomsg else elseif emenu endif endfor endfunction endtry endwhile enew execute exit exusage file filetype find finally finish first fixdel fold foldclose folddoopen folddoclosed foldopen function global goto grep grepadd gui gvim hardcopy help helpfind helpgrep helptags highlight hide history insert iabbrev iabclear ijump ilist imap imapclear imenu inoremap inoreabbrev inoremenu intro isearch isplit iunmap iunabbrev iunmenu join jumps keepalt keepmarks keepjumps lNext lNfile list laddexpr laddbuffer laddfile last language later lbuffer lcd lchdir lclose lcscope left leftabove lexpr lfile lfirst lgetbuffer lgetexpr lgetfile lgrep lgrepadd lhelpgrep llast llist lmake lmap lmapclear lnext lnewer lnfile lnoremap loadkeymap loadview lockmarks lockvar lolder lopen lprevious lpfile lrewind ltag lunmap luado luafile lvimgrep lvimgrepadd lwindow move mark make mapclear match menu menutranslate messages mkexrc mksession mkspell mkvimrc mkview mode mzscheme mzfile nbclose nbkey nbsart next nmap nmapclear nmenu nnoremap nnoremenu noautocmd noremap nohlsearch noreabbrev noremenu normal number nunmap nunmenu oldfiles open omap omapclear omenu only onoremap onoremenu options ounmap ounmenu ownsyntax print profdel profile promptfind promptrepl pclose pedit perl perldo pop popup ppop preserve previous psearch ptag ptNext ptfirst ptjump ptlast ptnext ptprevious ptrewind ptselect put pwd py3do py3file python pydo pyfile quit quitall qall read recover redo redir redraw redrawstatus registers resize retab return rewind right rightbelow ruby rubydo rubyfile rundo runtime rviminfo substitute sNext sandbox sargument sall saveas sbuffer sbNext sball sbfirst sblast sbmodified sbnext sbprevious sbrewind scriptnames scriptencoding scscope set setfiletype setglobal setlocal sfind sfirst shell simalt sign silent sleep slast smagic smapclear smenu snext sniff snomagic snoremap snoremenu sort source spelldump spellgood spellinfo spellrepall spellundo spellwrong split sprevious srewind stop stag startgreplace startreplace startinsert stopinsert stjump stselect sunhide sunmap sunmenu suspend sview swapname syntax syntime syncbind tNext tabNext tabclose tabedit tabfind tabfirst tablast tabmove tabnext tabonly tabprevious tabrewind tag tcl tcldo tclfile tearoff tfirst throw tjump tlast tmenu tnext topleft tprevious trewind tselect tunmenu undo undojoin undolist unabbreviate unhide unlet unlockvar unmap unmenu unsilent update vglobal version verbose vertical vimgrep vimgrepadd visual viusage view vmap vmapclear vmenu vnew vnoremap vnoremenu vsplit vunmap vunmenu write wNext wall while winsize wincmd winpos wnext wprevious wqall wsverb wundo wviminfo xit xall xmapclear xmap xmenu xnoremap xnoremenu xunmap xunmenu yank"
          ),
          built_in: (
            // built in func
            "synIDtrans atan2 range matcharg did_filetype asin feedkeys xor argv complete_check add getwinposx getqflist getwinposy screencol clearmatches empty extend getcmdpos mzeval garbagecollect setreg ceil sqrt diff_hlID inputsecret get getfperm getpid filewritable shiftwidth max sinh isdirectory synID system inputrestore winline atan visualmode inputlist tabpagewinnr round getregtype mapcheck hasmapto histdel argidx findfile sha256 exists toupper getcmdline taglist string getmatches bufnr strftime winwidth bufexists strtrans tabpagebuflist setcmdpos remote_read printf setloclist getpos getline bufwinnr float2nr len getcmdtype diff_filler luaeval resolve libcallnr foldclosedend reverse filter has_key bufname str2float strlen setline getcharmod setbufvar index searchpos shellescape undofile foldclosed setqflist buflisted strchars str2nr virtcol floor remove undotree remote_expr winheight gettabwinvar reltime cursor tabpagenr finddir localtime acos getloclist search tanh matchend rename gettabvar strdisplaywidth type abs py3eval setwinvar tolower wildmenumode log10 spellsuggest bufloaded synconcealed nextnonblank server2client complete settabwinvar executable input wincol setmatches getftype hlID inputsave searchpair or screenrow line settabvar histadd deepcopy strpart remote_peek and eval getftime submatch screenchar winsaveview matchadd mkdir screenattr getfontname libcall reltimestr getfsize winnr invert pow getbufline byte2line soundfold repeat fnameescape tagfiles sin strwidth spellbadword trunc maparg log lispindent hostname setpos globpath remote_foreground getchar synIDattr fnamemodify cscope_connection stridx winbufnr indent min complete_add nr2char searchpairpos inputdialog values matchlist items hlexists strridx browsedir expand fmod pathshorten line2byte argc count getwinvar glob foldtextresult getreg foreground cosh matchdelete has char2nr simplify histget searchdecl iconv winrestcmd pumvisible writefile foldlevel haslocaldir keys cos matchstr foldtext histnr tan tempname getcwd byteidx getbufvar islocked escape eventhandler remote_send serverlist winrestview synstack pyeval prevnonblank readfile cindent filereadable changenr exp"
          )
        },
        illegal: /;/,
        contains: [
          _.NUMBER_MODE,
          {
            className: "string",
            begin: "'",
            end: "'",
            illegal: "\\n"
          },
          /*
          	      A double quote can start either a string or a line comment. Strings are
          	      ended before the end of a line by another double quote and can contain
          	      escaped double-quotes and post-escaped line breaks.
          
          	      Also, any double quote at the beginning of a line is a comment but we
          	      don't handle that properly at the moment: any double quote inside will
          	      turn them into a string. Handling it properly will require a smarter
          	      parser.
          	      */
          {
            className: "string",
            begin: /"(\\"|\n\\|[^"\n])*"/
          },
          _.COMMENT('"', "$"),
          {
            className: "variable",
            begin: /[bwtglsav]:[\w\d_]+/
          },
          {
            begin: [
              /\b(?:function|function!)/,
              /\s+/,
              _.IDENT_RE
            ],
            className: {
              1: "keyword",
              3: "title"
            },
            end: "$",
            relevance: 0,
            contains: [
              {
                className: "params",
                begin: "\\(",
                end: "\\)"
              }
            ]
          },
          {
            className: "symbol",
            begin: /<[\w-]+>/
          }
        ]
      };
    }
    return vim_1 = O, vim_1;
  }
  var wren_1, hasRequiredWren;
  function requireWren() {
    if (hasRequiredWren) return wren_1;
    hasRequiredWren = 1;
    function O(_) {
      const H = _.regex, V = /[a-zA-Z]\w*/, q = [
        "as",
        "break",
        "class",
        "construct",
        "continue",
        "else",
        "for",
        "foreign",
        "if",
        "import",
        "in",
        "is",
        "return",
        "static",
        "var",
        "while"
      ], G = [
        "true",
        "false",
        "null"
      ], J = [
        "this",
        "super"
      ], Z = [
        "Bool",
        "Class",
        "Fiber",
        "Fn",
        "List",
        "Map",
        "Null",
        "Num",
        "Object",
        "Range",
        "Sequence",
        "String",
        "System"
      ], te = [
        "-",
        "~",
        /\*/,
        "%",
        /\.\.\./,
        /\.\./,
        /\+/,
        "<<",
        ">>",
        ">=",
        "<=",
        "<",
        ">",
        /\^/,
        /!=/,
        /!/,
        /\bis\b/,
        "==",
        "&&",
        "&",
        /\|\|/,
        /\|/,
        /\?:/,
        "="
      ], ne = {
        relevance: 0,
        match: H.concat(/\b(?!(if|while|for|else|super)\b)/, V, /(?=\s*[({])/),
        className: "title.function"
      }, ee = {
        match: H.concat(
          H.either(
            H.concat(/\b(?!(if|while|for|else|super)\b)/, V),
            H.either(...te)
          ),
          /(?=\s*\([^)]+\)\s*\{)/
        ),
        className: "title.function",
        starts: { contains: [
          {
            begin: /\(/,
            end: /\)/,
            contains: [
              {
                relevance: 0,
                scope: "params",
                match: V
              }
            ]
          }
        ] }
      }, ae = {
        variants: [
          { match: [
            /class\s+/,
            V,
            /\s+is\s+/,
            V
          ] },
          { match: [
            /class\s+/,
            V
          ] }
        ],
        scope: {
          2: "title.class",
          4: "title.class.inherited"
        },
        keywords: q
      }, oe = {
        relevance: 0,
        match: H.either(...te),
        className: "operator"
      }, se = {
        className: "string",
        begin: /"""/,
        end: /"""/
      }, ce = {
        className: "property",
        begin: H.concat(/\./, H.lookahead(V)),
        end: V,
        excludeBegin: !0,
        relevance: 0
      }, le = {
        relevance: 0,
        match: H.concat(/\b_/, V),
        scope: "variable"
      }, ue = {
        relevance: 0,
        match: /\b[A-Z]+[a-z]+([A-Z]+[a-z]+)*/,
        scope: "title.class",
        keywords: { _: Z }
      }, de = _.C_NUMBER_MODE, fe = {
        match: [
          V,
          /\s*/,
          /=/,
          /\s*/,
          /\(/,
          V,
          /\)\s*\{/
        ],
        scope: {
          1: "title.function",
          3: "operator",
          6: "params"
        }
      }, he = _.COMMENT(
        /\/\*\*/,
        /\*\//,
        { contains: [
          {
            match: /@[a-z]+/,
            scope: "doctag"
          },
          "self"
        ] }
      ), me = {
        scope: "subst",
        begin: /%\(/,
        end: /\)/,
        contains: [
          de,
          ue,
          ne,
          le,
          oe
        ]
      }, pe = {
        scope: "string",
        begin: /"/,
        end: /"/,
        contains: [
          me,
          {
            scope: "char.escape",
            variants: [
              { match: /\\\\|\\["0%abefnrtv]/ },
              { match: /\\x[0-9A-F]{2}/ },
              { match: /\\u[0-9A-F]{4}/ },
              { match: /\\U[0-9A-F]{8}/ }
            ]
          }
        ]
      };
      me.contains.push(pe);
      const ye = [
        ...q,
        ...J,
        ...G
      ], ge = {
        relevance: 0,
        match: H.concat(
          "\\b(?!",
          ye.join("|"),
          "\\b)",
          /[a-zA-Z_]\w*(?:[?!]|\b)/
        ),
        className: "variable"
      };
      return {
        name: "Wren",
        keywords: {
          keyword: q,
          "variable.language": J,
          literal: G
        },
        contains: [
          {
            // scope: "meta",
            scope: "comment",
            variants: [
              {
                begin: [
                  /#!?/,
                  /[A-Za-z_]+(?=\()/
                ],
                beginScope: {
                  // 2: "attr"
                },
                keywords: { literal: G },
                contains: [
                  // NUMBER,
                  // VARIABLE
                ],
                end: /\)/
              },
              {
                begin: [
                  /#!?/,
                  /[A-Za-z_]+/
                ],
                beginScope: {
                  // 2: "attr"
                },
                end: /$/
              }
            ]
          },
          de,
          pe,
          se,
          he,
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          ue,
          ae,
          fe,
          ee,
          ne,
          oe,
          le,
          ce,
          ge
        ]
      };
    }
    return wren_1 = O, wren_1;
  }
  var x86asm_1, hasRequiredX86asm;
  function requireX86asm() {
    if (hasRequiredX86asm) return x86asm_1;
    hasRequiredX86asm = 1;
    function O(_) {
      return {
        name: "Intel x86 Assembly",
        case_insensitive: !0,
        keywords: {
          $pattern: "[.%]?" + _.IDENT_RE,
          keyword: "lock rep repe repz repne repnz xaquire xrelease bnd nobnd aaa aad aam aas adc add and arpl bb0_reset bb1_reset bound bsf bsr bswap bt btc btr bts call cbw cdq cdqe clc cld cli clts cmc cmp cmpsb cmpsd cmpsq cmpsw cmpxchg cmpxchg486 cmpxchg8b cmpxchg16b cpuid cpu_read cpu_write cqo cwd cwde daa das dec div dmint emms enter equ f2xm1 fabs fadd faddp fbld fbstp fchs fclex fcmovb fcmovbe fcmove fcmovnb fcmovnbe fcmovne fcmovnu fcmovu fcom fcomi fcomip fcomp fcompp fcos fdecstp fdisi fdiv fdivp fdivr fdivrp femms feni ffree ffreep fiadd ficom ficomp fidiv fidivr fild fimul fincstp finit fist fistp fisttp fisub fisubr fld fld1 fldcw fldenv fldl2e fldl2t fldlg2 fldln2 fldpi fldz fmul fmulp fnclex fndisi fneni fninit fnop fnsave fnstcw fnstenv fnstsw fpatan fprem fprem1 fptan frndint frstor fsave fscale fsetpm fsin fsincos fsqrt fst fstcw fstenv fstp fstsw fsub fsubp fsubr fsubrp ftst fucom fucomi fucomip fucomp fucompp fxam fxch fxtract fyl2x fyl2xp1 hlt ibts icebp idiv imul in inc incbin insb insd insw int int01 int1 int03 int3 into invd invpcid invlpg invlpga iret iretd iretq iretw jcxz jecxz jrcxz jmp jmpe lahf lar lds lea leave les lfence lfs lgdt lgs lidt lldt lmsw loadall loadall286 lodsb lodsd lodsq lodsw loop loope loopne loopnz loopz lsl lss ltr mfence monitor mov movd movq movsb movsd movsq movsw movsx movsxd movzx mul mwait neg nop not or out outsb outsd outsw packssdw packsswb packuswb paddb paddd paddsb paddsiw paddsw paddusb paddusw paddw pand pandn pause paveb pavgusb pcmpeqb pcmpeqd pcmpeqw pcmpgtb pcmpgtd pcmpgtw pdistib pf2id pfacc pfadd pfcmpeq pfcmpge pfcmpgt pfmax pfmin pfmul pfrcp pfrcpit1 pfrcpit2 pfrsqit1 pfrsqrt pfsub pfsubr pi2fd pmachriw pmaddwd pmagw pmulhriw pmulhrwa pmulhrwc pmulhw pmullw pmvgezb pmvlzb pmvnzb pmvzb pop popa popad popaw popf popfd popfq popfw por prefetch prefetchw pslld psllq psllw psrad psraw psrld psrlq psrlw psubb psubd psubsb psubsiw psubsw psubusb psubusw psubw punpckhbw punpckhdq punpckhwd punpcklbw punpckldq punpcklwd push pusha pushad pushaw pushf pushfd pushfq pushfw pxor rcl rcr rdshr rdmsr rdpmc rdtsc rdtscp ret retf retn rol ror rdm rsdc rsldt rsm rsts sahf sal salc sar sbb scasb scasd scasq scasw sfence sgdt shl shld shr shrd sidt sldt skinit smi smint smintold smsw stc std sti stosb stosd stosq stosw str sub svdc svldt svts swapgs syscall sysenter sysexit sysret test ud0 ud1 ud2b ud2 ud2a umov verr verw fwait wbinvd wrshr wrmsr xadd xbts xchg xlatb xlat xor cmove cmovz cmovne cmovnz cmova cmovnbe cmovae cmovnb cmovb cmovnae cmovbe cmovna cmovg cmovnle cmovge cmovnl cmovl cmovnge cmovle cmovng cmovc cmovnc cmovo cmovno cmovs cmovns cmovp cmovpe cmovnp cmovpo je jz jne jnz ja jnbe jae jnb jb jnae jbe jna jg jnle jge jnl jl jnge jle jng jc jnc jo jno js jns jpo jnp jpe jp sete setz setne setnz seta setnbe setae setnb setnc setb setnae setcset setbe setna setg setnle setge setnl setl setnge setle setng sets setns seto setno setpe setp setpo setnp addps addss andnps andps cmpeqps cmpeqss cmpleps cmpless cmpltps cmpltss cmpneqps cmpneqss cmpnleps cmpnless cmpnltps cmpnltss cmpordps cmpordss cmpunordps cmpunordss cmpps cmpss comiss cvtpi2ps cvtps2pi cvtsi2ss cvtss2si cvttps2pi cvttss2si divps divss ldmxcsr maxps maxss minps minss movaps movhps movlhps movlps movhlps movmskps movntps movss movups mulps mulss orps rcpps rcpss rsqrtps rsqrtss shufps sqrtps sqrtss stmxcsr subps subss ucomiss unpckhps unpcklps xorps fxrstor fxrstor64 fxsave fxsave64 xgetbv xsetbv xsave xsave64 xsaveopt xsaveopt64 xrstor xrstor64 prefetchnta prefetcht0 prefetcht1 prefetcht2 maskmovq movntq pavgb pavgw pextrw pinsrw pmaxsw pmaxub pminsw pminub pmovmskb pmulhuw psadbw pshufw pf2iw pfnacc pfpnacc pi2fw pswapd maskmovdqu clflush movntdq movnti movntpd movdqa movdqu movdq2q movq2dq paddq pmuludq pshufd pshufhw pshuflw pslldq psrldq psubq punpckhqdq punpcklqdq addpd addsd andnpd andpd cmpeqpd cmpeqsd cmplepd cmplesd cmpltpd cmpltsd cmpneqpd cmpneqsd cmpnlepd cmpnlesd cmpnltpd cmpnltsd cmpordpd cmpordsd cmpunordpd cmpunordsd cmppd comisd cvtdq2pd cvtdq2ps cvtpd2dq cvtpd2pi cvtpd2ps cvtpi2pd cvtps2dq cvtps2pd cvtsd2si cvtsd2ss cvtsi2sd cvtss2sd cvttpd2pi cvttpd2dq cvttps2dq cvttsd2si divpd divsd maxpd maxsd minpd minsd movapd movhpd movlpd movmskpd movupd mulpd mulsd orpd shufpd sqrtpd sqrtsd subpd subsd ucomisd unpckhpd unpcklpd xorpd addsubpd addsubps haddpd haddps hsubpd hsubps lddqu movddup movshdup movsldup clgi stgi vmcall vmclear vmfunc vmlaunch vmload vmmcall vmptrld vmptrst vmread vmresume vmrun vmsave vmwrite vmxoff vmxon invept invvpid pabsb pabsw pabsd palignr phaddw phaddd phaddsw phsubw phsubd phsubsw pmaddubsw pmulhrsw pshufb psignb psignw psignd extrq insertq movntsd movntss lzcnt blendpd blendps blendvpd blendvps dppd dpps extractps insertps movntdqa mpsadbw packusdw pblendvb pblendw pcmpeqq pextrb pextrd pextrq phminposuw pinsrb pinsrd pinsrq pmaxsb pmaxsd pmaxud pmaxuw pminsb pminsd pminud pminuw pmovsxbw pmovsxbd pmovsxbq pmovsxwd pmovsxwq pmovsxdq pmovzxbw pmovzxbd pmovzxbq pmovzxwd pmovzxwq pmovzxdq pmuldq pmulld ptest roundpd roundps roundsd roundss crc32 pcmpestri pcmpestrm pcmpistri pcmpistrm pcmpgtq popcnt getsec pfrcpv pfrsqrtv movbe aesenc aesenclast aesdec aesdeclast aesimc aeskeygenassist vaesenc vaesenclast vaesdec vaesdeclast vaesimc vaeskeygenassist vaddpd vaddps vaddsd vaddss vaddsubpd vaddsubps vandpd vandps vandnpd vandnps vblendpd vblendps vblendvpd vblendvps vbroadcastss vbroadcastsd vbroadcastf128 vcmpeq_ospd vcmpeqpd vcmplt_ospd vcmpltpd vcmple_ospd vcmplepd vcmpunord_qpd vcmpunordpd vcmpneq_uqpd vcmpneqpd vcmpnlt_uspd vcmpnltpd vcmpnle_uspd vcmpnlepd vcmpord_qpd vcmpordpd vcmpeq_uqpd vcmpnge_uspd vcmpngepd vcmpngt_uspd vcmpngtpd vcmpfalse_oqpd vcmpfalsepd vcmpneq_oqpd vcmpge_ospd vcmpgepd vcmpgt_ospd vcmpgtpd vcmptrue_uqpd vcmptruepd vcmplt_oqpd vcmple_oqpd vcmpunord_spd vcmpneq_uspd vcmpnlt_uqpd vcmpnle_uqpd vcmpord_spd vcmpeq_uspd vcmpnge_uqpd vcmpngt_uqpd vcmpfalse_ospd vcmpneq_ospd vcmpge_oqpd vcmpgt_oqpd vcmptrue_uspd vcmppd vcmpeq_osps vcmpeqps vcmplt_osps vcmpltps vcmple_osps vcmpleps vcmpunord_qps vcmpunordps vcmpneq_uqps vcmpneqps vcmpnlt_usps vcmpnltps vcmpnle_usps vcmpnleps vcmpord_qps vcmpordps vcmpeq_uqps vcmpnge_usps vcmpngeps vcmpngt_usps vcmpngtps vcmpfalse_oqps vcmpfalseps vcmpneq_oqps vcmpge_osps vcmpgeps vcmpgt_osps vcmpgtps vcmptrue_uqps vcmptrueps vcmplt_oqps vcmple_oqps vcmpunord_sps vcmpneq_usps vcmpnlt_uqps vcmpnle_uqps vcmpord_sps vcmpeq_usps vcmpnge_uqps vcmpngt_uqps vcmpfalse_osps vcmpneq_osps vcmpge_oqps vcmpgt_oqps vcmptrue_usps vcmpps vcmpeq_ossd vcmpeqsd vcmplt_ossd vcmpltsd vcmple_ossd vcmplesd vcmpunord_qsd vcmpunordsd vcmpneq_uqsd vcmpneqsd vcmpnlt_ussd vcmpnltsd vcmpnle_ussd vcmpnlesd vcmpord_qsd vcmpordsd vcmpeq_uqsd vcmpnge_ussd vcmpngesd vcmpngt_ussd vcmpngtsd vcmpfalse_oqsd vcmpfalsesd vcmpneq_oqsd vcmpge_ossd vcmpgesd vcmpgt_ossd vcmpgtsd vcmptrue_uqsd vcmptruesd vcmplt_oqsd vcmple_oqsd vcmpunord_ssd vcmpneq_ussd vcmpnlt_uqsd vcmpnle_uqsd vcmpord_ssd vcmpeq_ussd vcmpnge_uqsd vcmpngt_uqsd vcmpfalse_ossd vcmpneq_ossd vcmpge_oqsd vcmpgt_oqsd vcmptrue_ussd vcmpsd vcmpeq_osss vcmpeqss vcmplt_osss vcmpltss vcmple_osss vcmpless vcmpunord_qss vcmpunordss vcmpneq_uqss vcmpneqss vcmpnlt_usss vcmpnltss vcmpnle_usss vcmpnless vcmpord_qss vcmpordss vcmpeq_uqss vcmpnge_usss vcmpngess vcmpngt_usss vcmpngtss vcmpfalse_oqss vcmpfalsess vcmpneq_oqss vcmpge_osss vcmpgess vcmpgt_osss vcmpgtss vcmptrue_uqss vcmptruess vcmplt_oqss vcmple_oqss vcmpunord_sss vcmpneq_usss vcmpnlt_uqss vcmpnle_uqss vcmpord_sss vcmpeq_usss vcmpnge_uqss vcmpngt_uqss vcmpfalse_osss vcmpneq_osss vcmpge_oqss vcmpgt_oqss vcmptrue_usss vcmpss vcomisd vcomiss vcvtdq2pd vcvtdq2ps vcvtpd2dq vcvtpd2ps vcvtps2dq vcvtps2pd vcvtsd2si vcvtsd2ss vcvtsi2sd vcvtsi2ss vcvtss2sd vcvtss2si vcvttpd2dq vcvttps2dq vcvttsd2si vcvttss2si vdivpd vdivps vdivsd vdivss vdppd vdpps vextractf128 vextractps vhaddpd vhaddps vhsubpd vhsubps vinsertf128 vinsertps vlddqu vldqqu vldmxcsr vmaskmovdqu vmaskmovps vmaskmovpd vmaxpd vmaxps vmaxsd vmaxss vminpd vminps vminsd vminss vmovapd vmovaps vmovd vmovq vmovddup vmovdqa vmovqqa vmovdqu vmovqqu vmovhlps vmovhpd vmovhps vmovlhps vmovlpd vmovlps vmovmskpd vmovmskps vmovntdq vmovntqq vmovntdqa vmovntpd vmovntps vmovsd vmovshdup vmovsldup vmovss vmovupd vmovups vmpsadbw vmulpd vmulps vmulsd vmulss vorpd vorps vpabsb vpabsw vpabsd vpacksswb vpackssdw vpackuswb vpackusdw vpaddb vpaddw vpaddd vpaddq vpaddsb vpaddsw vpaddusb vpaddusw vpalignr vpand vpandn vpavgb vpavgw vpblendvb vpblendw vpcmpestri vpcmpestrm vpcmpistri vpcmpistrm vpcmpeqb vpcmpeqw vpcmpeqd vpcmpeqq vpcmpgtb vpcmpgtw vpcmpgtd vpcmpgtq vpermilpd vpermilps vperm2f128 vpextrb vpextrw vpextrd vpextrq vphaddw vphaddd vphaddsw vphminposuw vphsubw vphsubd vphsubsw vpinsrb vpinsrw vpinsrd vpinsrq vpmaddwd vpmaddubsw vpmaxsb vpmaxsw vpmaxsd vpmaxub vpmaxuw vpmaxud vpminsb vpminsw vpminsd vpminub vpminuw vpminud vpmovmskb vpmovsxbw vpmovsxbd vpmovsxbq vpmovsxwd vpmovsxwq vpmovsxdq vpmovzxbw vpmovzxbd vpmovzxbq vpmovzxwd vpmovzxwq vpmovzxdq vpmulhuw vpmulhrsw vpmulhw vpmullw vpmulld vpmuludq vpmuldq vpor vpsadbw vpshufb vpshufd vpshufhw vpshuflw vpsignb vpsignw vpsignd vpslldq vpsrldq vpsllw vpslld vpsllq vpsraw vpsrad vpsrlw vpsrld vpsrlq vptest vpsubb vpsubw vpsubd vpsubq vpsubsb vpsubsw vpsubusb vpsubusw vpunpckhbw vpunpckhwd vpunpckhdq vpunpckhqdq vpunpcklbw vpunpcklwd vpunpckldq vpunpcklqdq vpxor vrcpps vrcpss vrsqrtps vrsqrtss vroundpd vroundps vroundsd vroundss vshufpd vshufps vsqrtpd vsqrtps vsqrtsd vsqrtss vstmxcsr vsubpd vsubps vsubsd vsubss vtestps vtestpd vucomisd vucomiss vunpckhpd vunpckhps vunpcklpd vunpcklps vxorpd vxorps vzeroall vzeroupper pclmullqlqdq pclmulhqlqdq pclmullqhqdq pclmulhqhqdq pclmulqdq vpclmullqlqdq vpclmulhqlqdq vpclmullqhqdq vpclmulhqhqdq vpclmulqdq vfmadd132ps vfmadd132pd vfmadd312ps vfmadd312pd vfmadd213ps vfmadd213pd vfmadd123ps vfmadd123pd vfmadd231ps vfmadd231pd vfmadd321ps vfmadd321pd vfmaddsub132ps vfmaddsub132pd vfmaddsub312ps vfmaddsub312pd vfmaddsub213ps vfmaddsub213pd vfmaddsub123ps vfmaddsub123pd vfmaddsub231ps vfmaddsub231pd vfmaddsub321ps vfmaddsub321pd vfmsub132ps vfmsub132pd vfmsub312ps vfmsub312pd vfmsub213ps vfmsub213pd vfmsub123ps vfmsub123pd vfmsub231ps vfmsub231pd vfmsub321ps vfmsub321pd vfmsubadd132ps vfmsubadd132pd vfmsubadd312ps vfmsubadd312pd vfmsubadd213ps vfmsubadd213pd vfmsubadd123ps vfmsubadd123pd vfmsubadd231ps vfmsubadd231pd vfmsubadd321ps vfmsubadd321pd vfnmadd132ps vfnmadd132pd vfnmadd312ps vfnmadd312pd vfnmadd213ps vfnmadd213pd vfnmadd123ps vfnmadd123pd vfnmadd231ps vfnmadd231pd vfnmadd321ps vfnmadd321pd vfnmsub132ps vfnmsub132pd vfnmsub312ps vfnmsub312pd vfnmsub213ps vfnmsub213pd vfnmsub123ps vfnmsub123pd vfnmsub231ps vfnmsub231pd vfnmsub321ps vfnmsub321pd vfmadd132ss vfmadd132sd vfmadd312ss vfmadd312sd vfmadd213ss vfmadd213sd vfmadd123ss vfmadd123sd vfmadd231ss vfmadd231sd vfmadd321ss vfmadd321sd vfmsub132ss vfmsub132sd vfmsub312ss vfmsub312sd vfmsub213ss vfmsub213sd vfmsub123ss vfmsub123sd vfmsub231ss vfmsub231sd vfmsub321ss vfmsub321sd vfnmadd132ss vfnmadd132sd vfnmadd312ss vfnmadd312sd vfnmadd213ss vfnmadd213sd vfnmadd123ss vfnmadd123sd vfnmadd231ss vfnmadd231sd vfnmadd321ss vfnmadd321sd vfnmsub132ss vfnmsub132sd vfnmsub312ss vfnmsub312sd vfnmsub213ss vfnmsub213sd vfnmsub123ss vfnmsub123sd vfnmsub231ss vfnmsub231sd vfnmsub321ss vfnmsub321sd rdfsbase rdgsbase rdrand wrfsbase wrgsbase vcvtph2ps vcvtps2ph adcx adox rdseed clac stac xstore xcryptecb xcryptcbc xcryptctr xcryptcfb xcryptofb montmul xsha1 xsha256 llwpcb slwpcb lwpval lwpins vfmaddpd vfmaddps vfmaddsd vfmaddss vfmaddsubpd vfmaddsubps vfmsubaddpd vfmsubaddps vfmsubpd vfmsubps vfmsubsd vfmsubss vfnmaddpd vfnmaddps vfnmaddsd vfnmaddss vfnmsubpd vfnmsubps vfnmsubsd vfnmsubss vfrczpd vfrczps vfrczsd vfrczss vpcmov vpcomb vpcomd vpcomq vpcomub vpcomud vpcomuq vpcomuw vpcomw vphaddbd vphaddbq vphaddbw vphadddq vphaddubd vphaddubq vphaddubw vphaddudq vphadduwd vphadduwq vphaddwd vphaddwq vphsubbw vphsubdq vphsubwd vpmacsdd vpmacsdqh vpmacsdql vpmacssdd vpmacssdqh vpmacssdql vpmacsswd vpmacssww vpmacswd vpmacsww vpmadcsswd vpmadcswd vpperm vprotb vprotd vprotq vprotw vpshab vpshad vpshaq vpshaw vpshlb vpshld vpshlq vpshlw vbroadcasti128 vpblendd vpbroadcastb vpbroadcastw vpbroadcastd vpbroadcastq vpermd vpermpd vpermps vpermq vperm2i128 vextracti128 vinserti128 vpmaskmovd vpmaskmovq vpsllvd vpsllvq vpsravd vpsrlvd vpsrlvq vgatherdpd vgatherqpd vgatherdps vgatherqps vpgatherdd vpgatherqd vpgatherdq vpgatherqq xabort xbegin xend xtest andn bextr blci blcic blsi blsic blcfill blsfill blcmsk blsmsk blsr blcs bzhi mulx pdep pext rorx sarx shlx shrx tzcnt tzmsk t1mskc valignd valignq vblendmpd vblendmps vbroadcastf32x4 vbroadcastf64x4 vbroadcasti32x4 vbroadcasti64x4 vcompresspd vcompressps vcvtpd2udq vcvtps2udq vcvtsd2usi vcvtss2usi vcvttpd2udq vcvttps2udq vcvttsd2usi vcvttss2usi vcvtudq2pd vcvtudq2ps vcvtusi2sd vcvtusi2ss vexpandpd vexpandps vextractf32x4 vextractf64x4 vextracti32x4 vextracti64x4 vfixupimmpd vfixupimmps vfixupimmsd vfixupimmss vgetexppd vgetexpps vgetexpsd vgetexpss vgetmantpd vgetmantps vgetmantsd vgetmantss vinsertf32x4 vinsertf64x4 vinserti32x4 vinserti64x4 vmovdqa32 vmovdqa64 vmovdqu32 vmovdqu64 vpabsq vpandd vpandnd vpandnq vpandq vpblendmd vpblendmq vpcmpltd vpcmpled vpcmpneqd vpcmpnltd vpcmpnled vpcmpd vpcmpltq vpcmpleq vpcmpneqq vpcmpnltq vpcmpnleq vpcmpq vpcmpequd vpcmpltud vpcmpleud vpcmpnequd vpcmpnltud vpcmpnleud vpcmpud vpcmpequq vpcmpltuq vpcmpleuq vpcmpnequq vpcmpnltuq vpcmpnleuq vpcmpuq vpcompressd vpcompressq vpermi2d vpermi2pd vpermi2ps vpermi2q vpermt2d vpermt2pd vpermt2ps vpermt2q vpexpandd vpexpandq vpmaxsq vpmaxuq vpminsq vpminuq vpmovdb vpmovdw vpmovqb vpmovqd vpmovqw vpmovsdb vpmovsdw vpmovsqb vpmovsqd vpmovsqw vpmovusdb vpmovusdw vpmovusqb vpmovusqd vpmovusqw vpord vporq vprold vprolq vprolvd vprolvq vprord vprorq vprorvd vprorvq vpscatterdd vpscatterdq vpscatterqd vpscatterqq vpsraq vpsravq vpternlogd vpternlogq vptestmd vptestmq vptestnmd vptestnmq vpxord vpxorq vrcp14pd vrcp14ps vrcp14sd vrcp14ss vrndscalepd vrndscaleps vrndscalesd vrndscaless vrsqrt14pd vrsqrt14ps vrsqrt14sd vrsqrt14ss vscalefpd vscalefps vscalefsd vscalefss vscatterdpd vscatterdps vscatterqpd vscatterqps vshuff32x4 vshuff64x2 vshufi32x4 vshufi64x2 kandnw kandw kmovw knotw kortestw korw kshiftlw kshiftrw kunpckbw kxnorw kxorw vpbroadcastmb2q vpbroadcastmw2d vpconflictd vpconflictq vplzcntd vplzcntq vexp2pd vexp2ps vrcp28pd vrcp28ps vrcp28sd vrcp28ss vrsqrt28pd vrsqrt28ps vrsqrt28sd vrsqrt28ss vgatherpf0dpd vgatherpf0dps vgatherpf0qpd vgatherpf0qps vgatherpf1dpd vgatherpf1dps vgatherpf1qpd vgatherpf1qps vscatterpf0dpd vscatterpf0dps vscatterpf0qpd vscatterpf0qps vscatterpf1dpd vscatterpf1dps vscatterpf1qpd vscatterpf1qps prefetchwt1 bndmk bndcl bndcu bndcn bndmov bndldx bndstx sha1rnds4 sha1nexte sha1msg1 sha1msg2 sha256rnds2 sha256msg1 sha256msg2 hint_nop0 hint_nop1 hint_nop2 hint_nop3 hint_nop4 hint_nop5 hint_nop6 hint_nop7 hint_nop8 hint_nop9 hint_nop10 hint_nop11 hint_nop12 hint_nop13 hint_nop14 hint_nop15 hint_nop16 hint_nop17 hint_nop18 hint_nop19 hint_nop20 hint_nop21 hint_nop22 hint_nop23 hint_nop24 hint_nop25 hint_nop26 hint_nop27 hint_nop28 hint_nop29 hint_nop30 hint_nop31 hint_nop32 hint_nop33 hint_nop34 hint_nop35 hint_nop36 hint_nop37 hint_nop38 hint_nop39 hint_nop40 hint_nop41 hint_nop42 hint_nop43 hint_nop44 hint_nop45 hint_nop46 hint_nop47 hint_nop48 hint_nop49 hint_nop50 hint_nop51 hint_nop52 hint_nop53 hint_nop54 hint_nop55 hint_nop56 hint_nop57 hint_nop58 hint_nop59 hint_nop60 hint_nop61 hint_nop62 hint_nop63",
          built_in: (
            // Instruction pointer
            "ip eip rip al ah bl bh cl ch dl dh sil dil bpl spl r8b r9b r10b r11b r12b r13b r14b r15b ax bx cx dx si di bp sp r8w r9w r10w r11w r12w r13w r14w r15w eax ebx ecx edx esi edi ebp esp eip r8d r9d r10d r11d r12d r13d r14d r15d rax rbx rcx rdx rsi rdi rbp rsp r8 r9 r10 r11 r12 r13 r14 r15 cs ds es fs gs ss st st0 st1 st2 st3 st4 st5 st6 st7 mm0 mm1 mm2 mm3 mm4 mm5 mm6 mm7 xmm0  xmm1  xmm2  xmm3  xmm4  xmm5  xmm6  xmm7  xmm8  xmm9 xmm10  xmm11 xmm12 xmm13 xmm14 xmm15 xmm16 xmm17 xmm18 xmm19 xmm20 xmm21 xmm22 xmm23 xmm24 xmm25 xmm26 xmm27 xmm28 xmm29 xmm30 xmm31 ymm0  ymm1  ymm2  ymm3  ymm4  ymm5  ymm6  ymm7  ymm8  ymm9 ymm10  ymm11 ymm12 ymm13 ymm14 ymm15 ymm16 ymm17 ymm18 ymm19 ymm20 ymm21 ymm22 ymm23 ymm24 ymm25 ymm26 ymm27 ymm28 ymm29 ymm30 ymm31 zmm0  zmm1  zmm2  zmm3  zmm4  zmm5  zmm6  zmm7  zmm8  zmm9 zmm10  zmm11 zmm12 zmm13 zmm14 zmm15 zmm16 zmm17 zmm18 zmm19 zmm20 zmm21 zmm22 zmm23 zmm24 zmm25 zmm26 zmm27 zmm28 zmm29 zmm30 zmm31 k0 k1 k2 k3 k4 k5 k6 k7 bnd0 bnd1 bnd2 bnd3 cr0 cr1 cr2 cr3 cr4 cr8 dr0 dr1 dr2 dr3 dr8 tr3 tr4 tr5 tr6 tr7 r0 r1 r2 r3 r4 r5 r6 r7 r0b r1b r2b r3b r4b r5b r6b r7b r0w r1w r2w r3w r4w r5w r6w r7w r0d r1d r2d r3d r4d r5d r6d r7d r0h r1h r2h r3h r0l r1l r2l r3l r4l r5l r6l r7l r8l r9l r10l r11l r12l r13l r14l r15l db dw dd dq dt ddq do dy dz resb resw resd resq rest resdq reso resy resz incbin equ times byte word dword qword nosplit rel abs seg wrt strict near far a32 ptr"
          ),
          meta: "%define %xdefine %+ %undef %defstr %deftok %assign %strcat %strlen %substr %rotate %elif %else %endif %if %ifmacro %ifctx %ifidn %ifidni %ifid %ifnum %ifstr %iftoken %ifempty %ifenv %error %warning %fatal %rep %endrep %include %push %pop %repl %pathsearch %depend %use %arg %stacksize %local %line %comment %endcomment .nolist __FILE__ __LINE__ __SECT__  __BITS__ __OUTPUT_FORMAT__ __DATE__ __TIME__ __DATE_NUM__ __TIME_NUM__ __UTC_DATE__ __UTC_TIME__ __UTC_DATE_NUM__ __UTC_TIME_NUM__  __PASS__ struc endstruc istruc at iend align alignb sectalign daz nodaz up down zero default option assume public bits use16 use32 use64 default section segment absolute extern global common cpu float __utf16__ __utf16le__ __utf16be__ __utf32__ __utf32le__ __utf32be__ __float8__ __float16__ __float32__ __float64__ __float80m__ __float80e__ __float128l__ __float128h__ __Infinity__ __QNaN__ __SNaN__ Inf NaN QNaN SNaN float8 float16 float32 float64 float80m float80e float128l float128h __FLOAT_DAZ__ __FLOAT_ROUND__ __FLOAT__"
        },
        contains: [
          _.COMMENT(
            ";",
            "$",
            { relevance: 0 }
          ),
          {
            className: "number",
            variants: [
              // Float number and x87 BCD
              {
                begin: "\\b(?:([0-9][0-9_]*)?\\.[0-9_]*(?:[eE][+-]?[0-9_]+)?|(0[Xx])?[0-9][0-9_]*(\\.[0-9_]*)?(?:[pP](?:[+-]?[0-9_]+)?)?)\\b",
                relevance: 0
              },
              // Hex number in $
              {
                begin: "\\$[0-9][0-9A-Fa-f]*",
                relevance: 0
              },
              // Number in H,D,T,Q,O,B,Y suffix
              { begin: "\\b(?:[0-9A-Fa-f][0-9A-Fa-f_]*[Hh]|[0-9][0-9_]*[DdTt]?|[0-7][0-7_]*[QqOo]|[0-1][0-1_]*[BbYy])\\b" },
              // Number in X,D,T,Q,O,B,Y prefix
              { begin: "\\b(?:0[Xx][0-9A-Fa-f_]+|0[DdTt][0-9_]+|0[QqOo][0-7_]+|0[BbYy][0-1_]+)\\b" }
            ]
          },
          // Double quote string
          _.QUOTE_STRING_MODE,
          {
            className: "string",
            variants: [
              // Single-quoted string
              {
                begin: "'",
                end: "[^\\\\]'"
              },
              // Backquoted string
              {
                begin: "`",
                end: "[^\\\\]`"
              }
            ],
            relevance: 0
          },
          {
            className: "symbol",
            variants: [
              // Global label and local label
              { begin: "^\\s*[A-Za-z._?][A-Za-z0-9_$#@~.?]*(:|\\s+label)" },
              // Macro-local label
              { begin: "^\\s*%%[A-Za-z0-9_$#@~.?]*:" }
            ],
            relevance: 0
          },
          // Macro parameter
          {
            className: "subst",
            begin: "%[0-9]+",
            relevance: 0
          },
          // Macro parameter
          {
            className: "subst",
            begin: "%!S+",
            relevance: 0
          },
          {
            className: "meta",
            begin: /^\s*\.[\w_-]+/
          }
        ]
      };
    }
    return x86asm_1 = O, x86asm_1;
  }
  var xl_1, hasRequiredXl;
  function requireXl() {
    if (hasRequiredXl) return xl_1;
    hasRequiredXl = 1;
    function O(_) {
      const H = [
        "if",
        "then",
        "else",
        "do",
        "while",
        "until",
        "for",
        "loop",
        "import",
        "with",
        "is",
        "as",
        "where",
        "when",
        "by",
        "data",
        "constant",
        "integer",
        "real",
        "text",
        "name",
        "boolean",
        "symbol",
        "infix",
        "prefix",
        "postfix",
        "block",
        "tree"
      ], V = [
        "in",
        "mod",
        "rem",
        "and",
        "or",
        "xor",
        "not",
        "abs",
        "sign",
        "floor",
        "ceil",
        "sqrt",
        "sin",
        "cos",
        "tan",
        "asin",
        "acos",
        "atan",
        "exp",
        "expm1",
        "log",
        "log2",
        "log10",
        "log1p",
        "pi",
        "at",
        "text_length",
        "text_range",
        "text_find",
        "text_replace",
        "contains",
        "page",
        "slide",
        "basic_slide",
        "title_slide",
        "title",
        "subtitle",
        "fade_in",
        "fade_out",
        "fade_at",
        "clear_color",
        "color",
        "line_color",
        "line_width",
        "texture_wrap",
        "texture_transform",
        "texture",
        "scale_?x",
        "scale_?y",
        "scale_?z?",
        "translate_?x",
        "translate_?y",
        "translate_?z?",
        "rotate_?x",
        "rotate_?y",
        "rotate_?z?",
        "rectangle",
        "circle",
        "ellipse",
        "sphere",
        "path",
        "line_to",
        "move_to",
        "quad_to",
        "curve_to",
        "theme",
        "background",
        "contents",
        "locally",
        "time",
        "mouse_?x",
        "mouse_?y",
        "mouse_buttons"
      ], q = [
        "ObjectLoader",
        "Animate",
        "MovieCredits",
        "Slides",
        "Filters",
        "Shading",
        "Materials",
        "LensFlare",
        "Mapping",
        "VLCAudioVideo",
        "StereoDecoder",
        "PointCloud",
        "NetworkAccess",
        "RemoteControl",
        "RegExp",
        "ChromaKey",
        "Snowfall",
        "NodeJS",
        "Speech",
        "Charts"
      ], J = {
        $pattern: /[a-zA-Z][a-zA-Z0-9_?]*/,
        keyword: H,
        literal: [
          "true",
          "false",
          "nil"
        ],
        built_in: V.concat(q)
      }, Z = {
        className: "string",
        begin: '"',
        end: '"',
        illegal: "\\n"
      }, te = {
        className: "string",
        begin: "'",
        end: "'",
        illegal: "\\n"
      }, ne = {
        className: "string",
        begin: "<<",
        end: ">>"
      }, ee = {
        className: "number",
        begin: "[0-9]+#[0-9A-Z_]+(\\.[0-9-A-Z_]+)?#?([Ee][+-]?[0-9]+)?"
      }, ae = {
        beginKeywords: "import",
        end: "$",
        keywords: J,
        contains: [Z]
      }, oe = {
        className: "function",
        begin: /[a-z][^\n]*->/,
        returnBegin: !0,
        end: /->/,
        contains: [
          _.inherit(_.TITLE_MODE, { starts: {
            endsWithParent: !0,
            keywords: J
          } })
        ]
      };
      return {
        name: "XL",
        aliases: ["tao"],
        keywords: J,
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.C_BLOCK_COMMENT_MODE,
          Z,
          te,
          ne,
          oe,
          ae,
          ee,
          _.NUMBER_MODE
        ]
      };
    }
    return xl_1 = O, xl_1;
  }
  var xquery_1, hasRequiredXquery;
  function requireXquery() {
    if (hasRequiredXquery) return xquery_1;
    hasRequiredXquery = 1;
    function O(_) {
      return {
        name: "XQuery",
        aliases: [
          "xpath",
          "xq",
          "xqm"
        ],
        case_insensitive: !1,
        illegal: /(proc)|(abstract)|(extends)|(until)|(#)/,
        keywords: {
          $pattern: /[a-zA-Z$][a-zA-Z0-9_:-]*/,
          keyword: [
            "module",
            "schema",
            "namespace",
            "boundary-space",
            "preserve",
            "no-preserve",
            "strip",
            "default",
            "collation",
            "base-uri",
            "ordering",
            "context",
            "decimal-format",
            "decimal-separator",
            "copy-namespaces",
            "empty-sequence",
            "except",
            "exponent-separator",
            "external",
            "grouping-separator",
            "inherit",
            "no-inherit",
            "lax",
            "minus-sign",
            "per-mille",
            "percent",
            "schema-attribute",
            "schema-element",
            "strict",
            "unordered",
            "zero-digit",
            "declare",
            "import",
            "option",
            "function",
            "validate",
            "variable",
            "for",
            "at",
            "in",
            "let",
            "where",
            "order",
            "group",
            "by",
            "return",
            "if",
            "then",
            "else",
            "tumbling",
            "sliding",
            "window",
            "start",
            "when",
            "only",
            "end",
            "previous",
            "next",
            "stable",
            "ascending",
            "descending",
            "allowing",
            "empty",
            "greatest",
            "least",
            "some",
            "every",
            "satisfies",
            "switch",
            "case",
            "typeswitch",
            "try",
            "catch",
            "and",
            "or",
            "to",
            "union",
            "intersect",
            "instance",
            "of",
            "treat",
            "as",
            "castable",
            "cast",
            "map",
            "array",
            "delete",
            "insert",
            "into",
            "replace",
            "value",
            "rename",
            "copy",
            "modify",
            "update"
          ],
          type: [
            "item",
            "document-node",
            "node",
            "attribute",
            "document",
            "element",
            "comment",
            "namespace",
            "namespace-node",
            "processing-instruction",
            "text",
            "construction",
            "xs:anyAtomicType",
            "xs:untypedAtomic",
            "xs:duration",
            "xs:time",
            "xs:decimal",
            "xs:float",
            "xs:double",
            "xs:gYearMonth",
            "xs:gYear",
            "xs:gMonthDay",
            "xs:gMonth",
            "xs:gDay",
            "xs:boolean",
            "xs:base64Binary",
            "xs:hexBinary",
            "xs:anyURI",
            "xs:QName",
            "xs:NOTATION",
            "xs:dateTime",
            "xs:dateTimeStamp",
            "xs:date",
            "xs:string",
            "xs:normalizedString",
            "xs:token",
            "xs:language",
            "xs:NMTOKEN",
            "xs:Name",
            "xs:NCName",
            "xs:ID",
            "xs:IDREF",
            "xs:ENTITY",
            "xs:integer",
            "xs:nonPositiveInteger",
            "xs:negativeInteger",
            "xs:long",
            "xs:int",
            "xs:short",
            "xs:byte",
            "xs:nonNegativeInteger",
            "xs:unisignedLong",
            "xs:unsignedInt",
            "xs:unsignedShort",
            "xs:unsignedByte",
            "xs:positiveInteger",
            "xs:yearMonthDuration",
            "xs:dayTimeDuration"
          ],
          literal: [
            "eq",
            "ne",
            "lt",
            "le",
            "gt",
            "ge",
            "is",
            "self::",
            "child::",
            "descendant::",
            "descendant-or-self::",
            "attribute::",
            "following::",
            "following-sibling::",
            "parent::",
            "ancestor::",
            "ancestor-or-self::",
            "preceding::",
            "preceding-sibling::",
            "NaN"
          ]
        },
        contains: [
          {
            className: "variable",
            begin: /[$][\w\-:]+/
          },
          {
            className: "built_in",
            variants: [
              {
                begin: /\barray:/,
                end: /(?:append|filter|flatten|fold-(?:left|right)|for-each(?:-pair)?|get|head|insert-before|join|put|remove|reverse|size|sort|subarray|tail)\b/
              },
              {
                begin: /\bmap:/,
                end: /(?:contains|entry|find|for-each|get|keys|merge|put|remove|size)\b/
              },
              {
                begin: /\bmath:/,
                end: /(?:a(?:cos|sin|tan[2]?)|cos|exp(?:10)?|log(?:10)?|pi|pow|sin|sqrt|tan)\b/
              },
              {
                begin: /\bop:/,
                end: /\(/,
                excludeEnd: !0
              },
              {
                begin: /\bfn:/,
                end: /\(/,
                excludeEnd: !0
              },
              // do not highlight inbuilt strings as variable or xml element names
              { begin: /[^</$:'"-]\b(?:abs|accumulator-(?:after|before)|adjust-(?:date(?:Time)?|time)-to-timezone|analyze-string|apply|available-(?:environment-variables|system-properties)|avg|base-uri|boolean|ceiling|codepoints?-(?:equal|to-string)|collation-key|collection|compare|concat|contains(?:-token)?|copy-of|count|current(?:-)?(?:date(?:Time)?|time|group(?:ing-key)?|output-uri|merge-(?:group|key))?data|dateTime|days?-from-(?:date(?:Time)?|duration)|deep-equal|default-(?:collation|language)|distinct-values|document(?:-uri)?|doc(?:-available)?|element-(?:available|with-id)|empty|encode-for-uri|ends-with|environment-variable|error|escape-html-uri|exactly-one|exists|false|filter|floor|fold-(?:left|right)|for-each(?:-pair)?|format-(?:date(?:Time)?|time|integer|number)|function-(?:arity|available|lookup|name)|generate-id|has-children|head|hours-from-(?:dateTime|duration|time)|id(?:ref)?|implicit-timezone|in-scope-prefixes|index-of|innermost|insert-before|iri-to-uri|json-(?:doc|to-xml)|key|lang|last|load-xquery-module|local-name(?:-from-QName)?|(?:lower|upper)-case|matches|max|minutes-from-(?:dateTime|duration|time)|min|months?-from-(?:date(?:Time)?|duration)|name(?:space-uri-?(?:for-prefix|from-QName)?)?|nilled|node-name|normalize-(?:space|unicode)|not|number|one-or-more|outermost|parse-(?:ietf-date|json)|path|position|(?:prefix-from-)?QName|random-number-generator|regex-group|remove|replace|resolve-(?:QName|uri)|reverse|root|round(?:-half-to-even)?|seconds-from-(?:dateTime|duration|time)|snapshot|sort|starts-with|static-base-uri|stream-available|string-?(?:join|length|to-codepoints)?|subsequence|substring-?(?:after|before)?|sum|system-property|tail|timezone-from-(?:date(?:Time)?|time)|tokenize|trace|trans(?:form|late)|true|type-available|unordered|unparsed-(?:entity|text)?-?(?:public-id|uri|available|lines)?|uri-collection|xml-to-json|years?-from-(?:date(?:Time)?|duration)|zero-or-one)\b/ },
              {
                begin: /\blocal:/,
                end: /\(/,
                excludeEnd: !0
              },
              {
                begin: /\bzip:/,
                end: /(?:zip-file|(?:xml|html|text|binary)-entry| (?:update-)?entries)\b/
              },
              {
                begin: /\b(?:util|db|functx|app|xdmp|xmldb):/,
                end: /\(/,
                excludeEnd: !0
              }
            ]
          },
          {
            className: "string",
            variants: [
              {
                begin: /"/,
                end: /"/,
                contains: [
                  {
                    begin: /""/,
                    relevance: 0
                  }
                ]
              },
              {
                begin: /'/,
                end: /'/,
                contains: [
                  {
                    begin: /''/,
                    relevance: 0
                  }
                ]
              }
            ]
          },
          {
            className: "number",
            begin: /(\b0[0-7_]+)|(\b0x[0-9a-fA-F_]+)|(\b[1-9][0-9_]*(\.[0-9_]+)?)|[0_]\b/,
            relevance: 0
          },
          {
            className: "comment",
            begin: /\(:/,
            end: /:\)/,
            relevance: 10,
            contains: [
              {
                className: "doctag",
                begin: /@\w+/
              }
            ]
          },
          {
            className: "meta",
            begin: /%[\w\-:]+/
          },
          {
            className: "title",
            begin: /\bxquery version "[13]\.[01]"\s?(?:encoding ".+")?/,
            end: /;/
          },
          {
            beginKeywords: "element attribute comment document processing-instruction",
            end: /\{/,
            excludeEnd: !0
          },
          {
            begin: /<([\w._:-]+)(\s+\S*=('|").*('|"))?>/,
            end: /(\/[\w._:-]+>)/,
            subLanguage: "xml",
            contains: [
              {
                begin: /\{/,
                end: /\}/,
                subLanguage: "xquery"
              },
              "self"
            ]
          }
        ]
      };
    }
    return xquery_1 = O, xquery_1;
  }
  var zephir_1, hasRequiredZephir;
  function requireZephir() {
    if (hasRequiredZephir) return zephir_1;
    hasRequiredZephir = 1;
    function O(_) {
      const H = {
        className: "string",
        contains: [_.BACKSLASH_ESCAPE],
        variants: [
          _.inherit(_.APOS_STRING_MODE, { illegal: null }),
          _.inherit(_.QUOTE_STRING_MODE, { illegal: null })
        ]
      }, V = _.UNDERSCORE_TITLE_MODE, q = { variants: [
        _.BINARY_NUMBER_MODE,
        _.C_NUMBER_MODE
      ] }, G = (
        // classes and objects
        "namespace class interface use extends function return abstract final public protected private static deprecated throw try catch Exception echo empty isset instanceof unset let var new const self require if else elseif switch case default do while loop for continue break likely unlikely __LINE__ __FILE__ __DIR__ __FUNCTION__ __CLASS__ __TRAIT__ __METHOD__ __NAMESPACE__ array boolean float double integer object resource string char long unsigned bool int uint ulong uchar true false null undefined"
      );
      return {
        name: "Zephir",
        aliases: ["zep"],
        keywords: G,
        contains: [
          _.C_LINE_COMMENT_MODE,
          _.COMMENT(
            /\/\*/,
            /\*\//,
            { contains: [
              {
                className: "doctag",
                begin: /@[A-Za-z]+/
              }
            ] }
          ),
          {
            className: "string",
            begin: /<<<['"]?\w+['"]?$/,
            end: /^\w+;/,
            contains: [_.BACKSLASH_ESCAPE]
          },
          {
            // swallow composed identifiers to avoid parsing them as keywords
            begin: /(::|->)+[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*/
          },
          {
            className: "function",
            beginKeywords: "function fn",
            end: /[;{]/,
            excludeEnd: !0,
            illegal: /\$|\[|%/,
            contains: [
              V,
              {
                className: "params",
                begin: /\(/,
                end: /\)/,
                keywords: G,
                contains: [
                  "self",
                  _.C_BLOCK_COMMENT_MODE,
                  H,
                  q
                ]
              }
            ]
          },
          {
            className: "class",
            beginKeywords: "class interface",
            end: /\{/,
            excludeEnd: !0,
            illegal: /[:($"]/,
            contains: [
              { beginKeywords: "extends implements" },
              V
            ]
          },
          {
            beginKeywords: "namespace",
            end: /;/,
            illegal: /[.']/,
            contains: [V]
          },
          {
            beginKeywords: "use",
            end: /;/,
            contains: [V]
          },
          {
            begin: /=>/
            // No markup, just a relevance booster
          },
          H,
          q
        ]
      };
    }
    return zephir_1 = O, zephir_1;
  }
  var lib$1, hasRequiredLib$1;
  function requireLib$1() {
    if (hasRequiredLib$1) return lib$1;
    hasRequiredLib$1 = 1;
    var O = /* @__PURE__ */ requireCore();
    return O.registerLanguage("1c", /* @__PURE__ */ require_1c()), O.registerLanguage("abnf", /* @__PURE__ */ requireAbnf()), O.registerLanguage("accesslog", /* @__PURE__ */ requireAccesslog()), O.registerLanguage("actionscript", /* @__PURE__ */ requireActionscript()), O.registerLanguage("ada", /* @__PURE__ */ requireAda()), O.registerLanguage("angelscript", /* @__PURE__ */ requireAngelscript()), O.registerLanguage("apache", /* @__PURE__ */ requireApache()), O.registerLanguage("applescript", /* @__PURE__ */ requireApplescript()), O.registerLanguage("arcade", /* @__PURE__ */ requireArcade()), O.registerLanguage("arduino", /* @__PURE__ */ requireArduino()), O.registerLanguage("armasm", /* @__PURE__ */ requireArmasm()), O.registerLanguage("xml", /* @__PURE__ */ requireXml()), O.registerLanguage("asciidoc", /* @__PURE__ */ requireAsciidoc()), O.registerLanguage("aspectj", /* @__PURE__ */ requireAspectj()), O.registerLanguage("autohotkey", /* @__PURE__ */ requireAutohotkey()), O.registerLanguage("autoit", /* @__PURE__ */ requireAutoit()), O.registerLanguage("avrasm", /* @__PURE__ */ requireAvrasm()), O.registerLanguage("awk", /* @__PURE__ */ requireAwk()), O.registerLanguage("axapta", /* @__PURE__ */ requireAxapta()), O.registerLanguage("bash", /* @__PURE__ */ requireBash()), O.registerLanguage("basic", /* @__PURE__ */ requireBasic()), O.registerLanguage("bnf", /* @__PURE__ */ requireBnf()), O.registerLanguage("brainfuck", /* @__PURE__ */ requireBrainfuck()), O.registerLanguage("c", /* @__PURE__ */ requireC()), O.registerLanguage("cal", /* @__PURE__ */ requireCal()), O.registerLanguage("capnproto", /* @__PURE__ */ requireCapnproto()), O.registerLanguage("ceylon", /* @__PURE__ */ requireCeylon()), O.registerLanguage("clean", /* @__PURE__ */ requireClean()), O.registerLanguage("clojure", /* @__PURE__ */ requireClojure()), O.registerLanguage("clojure-repl", /* @__PURE__ */ requireClojureRepl()), O.registerLanguage("cmake", /* @__PURE__ */ requireCmake()), O.registerLanguage("coffeescript", /* @__PURE__ */ requireCoffeescript()), O.registerLanguage("coq", /* @__PURE__ */ requireCoq()), O.registerLanguage("cos", /* @__PURE__ */ requireCos()), O.registerLanguage("cpp", /* @__PURE__ */ requireCpp()), O.registerLanguage("crmsh", /* @__PURE__ */ requireCrmsh()), O.registerLanguage("crystal", /* @__PURE__ */ requireCrystal()), O.registerLanguage("csharp", /* @__PURE__ */ requireCsharp()), O.registerLanguage("csp", /* @__PURE__ */ requireCsp()), O.registerLanguage("css", /* @__PURE__ */ requireCss()), O.registerLanguage("d", /* @__PURE__ */ requireD()), O.registerLanguage("markdown", /* @__PURE__ */ requireMarkdown()), O.registerLanguage("dart", /* @__PURE__ */ requireDart()), O.registerLanguage("delphi", /* @__PURE__ */ requireDelphi()), O.registerLanguage("diff", /* @__PURE__ */ requireDiff()), O.registerLanguage("django", /* @__PURE__ */ requireDjango()), O.registerLanguage("dns", /* @__PURE__ */ requireDns()), O.registerLanguage("dockerfile", /* @__PURE__ */ requireDockerfile()), O.registerLanguage("dos", /* @__PURE__ */ requireDos()), O.registerLanguage("dsconfig", /* @__PURE__ */ requireDsconfig()), O.registerLanguage("dts", /* @__PURE__ */ requireDts()), O.registerLanguage("dust", /* @__PURE__ */ requireDust()), O.registerLanguage("ebnf", /* @__PURE__ */ requireEbnf()), O.registerLanguage("elixir", /* @__PURE__ */ requireElixir()), O.registerLanguage("elm", /* @__PURE__ */ requireElm()), O.registerLanguage("ruby", /* @__PURE__ */ requireRuby()), O.registerLanguage("erb", /* @__PURE__ */ requireErb()), O.registerLanguage("erlang-repl", /* @__PURE__ */ requireErlangRepl()), O.registerLanguage("erlang", /* @__PURE__ */ requireErlang()), O.registerLanguage("excel", /* @__PURE__ */ requireExcel()), O.registerLanguage("fix", /* @__PURE__ */ requireFix()), O.registerLanguage("flix", /* @__PURE__ */ requireFlix()), O.registerLanguage("fortran", /* @__PURE__ */ requireFortran()), O.registerLanguage("fsharp", /* @__PURE__ */ requireFsharp()), O.registerLanguage("gams", /* @__PURE__ */ requireGams()), O.registerLanguage("gauss", /* @__PURE__ */ requireGauss()), O.registerLanguage("gcode", /* @__PURE__ */ requireGcode()), O.registerLanguage("gherkin", /* @__PURE__ */ requireGherkin()), O.registerLanguage("glsl", /* @__PURE__ */ requireGlsl()), O.registerLanguage("gml", /* @__PURE__ */ requireGml()), O.registerLanguage("go", /* @__PURE__ */ requireGo()), O.registerLanguage("golo", /* @__PURE__ */ requireGolo()), O.registerLanguage("gradle", /* @__PURE__ */ requireGradle()), O.registerLanguage("graphql", /* @__PURE__ */ requireGraphql()), O.registerLanguage("groovy", /* @__PURE__ */ requireGroovy()), O.registerLanguage("haml", /* @__PURE__ */ requireHaml()), O.registerLanguage("handlebars", /* @__PURE__ */ requireHandlebars()), O.registerLanguage("haskell", /* @__PURE__ */ requireHaskell()), O.registerLanguage("haxe", /* @__PURE__ */ requireHaxe()), O.registerLanguage("hsp", /* @__PURE__ */ requireHsp()), O.registerLanguage("http", /* @__PURE__ */ requireHttp()), O.registerLanguage("hy", /* @__PURE__ */ requireHy()), O.registerLanguage("inform7", /* @__PURE__ */ requireInform7()), O.registerLanguage("ini", /* @__PURE__ */ requireIni()), O.registerLanguage("irpf90", /* @__PURE__ */ requireIrpf90()), O.registerLanguage("isbl", /* @__PURE__ */ requireIsbl()), O.registerLanguage("java", /* @__PURE__ */ requireJava()), O.registerLanguage("javascript", /* @__PURE__ */ requireJavascript()), O.registerLanguage("jboss-cli", /* @__PURE__ */ requireJbossCli()), O.registerLanguage("json", /* @__PURE__ */ requireJson()), O.registerLanguage("julia", /* @__PURE__ */ requireJulia()), O.registerLanguage("julia-repl", /* @__PURE__ */ requireJuliaRepl()), O.registerLanguage("kotlin", /* @__PURE__ */ requireKotlin()), O.registerLanguage("lasso", /* @__PURE__ */ requireLasso()), O.registerLanguage("latex", /* @__PURE__ */ requireLatex()), O.registerLanguage("ldif", /* @__PURE__ */ requireLdif()), O.registerLanguage("leaf", /* @__PURE__ */ requireLeaf()), O.registerLanguage("less", /* @__PURE__ */ requireLess()), O.registerLanguage("lisp", /* @__PURE__ */ requireLisp()), O.registerLanguage("livecodeserver", /* @__PURE__ */ requireLivecodeserver()), O.registerLanguage("livescript", /* @__PURE__ */ requireLivescript()), O.registerLanguage("llvm", /* @__PURE__ */ requireLlvm()), O.registerLanguage("lsl", /* @__PURE__ */ requireLsl()), O.registerLanguage("lua", /* @__PURE__ */ requireLua()), O.registerLanguage("makefile", /* @__PURE__ */ requireMakefile()), O.registerLanguage("mathematica", /* @__PURE__ */ requireMathematica()), O.registerLanguage("matlab", /* @__PURE__ */ requireMatlab()), O.registerLanguage("maxima", /* @__PURE__ */ requireMaxima()), O.registerLanguage("mel", /* @__PURE__ */ requireMel()), O.registerLanguage("mercury", /* @__PURE__ */ requireMercury()), O.registerLanguage("mipsasm", /* @__PURE__ */ requireMipsasm()), O.registerLanguage("mizar", /* @__PURE__ */ requireMizar()), O.registerLanguage("perl", /* @__PURE__ */ requirePerl()), O.registerLanguage("mojolicious", /* @__PURE__ */ requireMojolicious()), O.registerLanguage("monkey", /* @__PURE__ */ requireMonkey()), O.registerLanguage("moonscript", /* @__PURE__ */ requireMoonscript()), O.registerLanguage("n1ql", /* @__PURE__ */ requireN1ql()), O.registerLanguage("nestedtext", /* @__PURE__ */ requireNestedtext()), O.registerLanguage("nginx", /* @__PURE__ */ requireNginx()), O.registerLanguage("nim", /* @__PURE__ */ requireNim()), O.registerLanguage("nix", /* @__PURE__ */ requireNix()), O.registerLanguage("node-repl", /* @__PURE__ */ requireNodeRepl()), O.registerLanguage("nsis", /* @__PURE__ */ requireNsis()), O.registerLanguage("objectivec", /* @__PURE__ */ requireObjectivec()), O.registerLanguage("ocaml", /* @__PURE__ */ requireOcaml()), O.registerLanguage("openscad", /* @__PURE__ */ requireOpenscad()), O.registerLanguage("oxygene", /* @__PURE__ */ requireOxygene()), O.registerLanguage("parser3", /* @__PURE__ */ requireParser3()), O.registerLanguage("pf", /* @__PURE__ */ requirePf()), O.registerLanguage("pgsql", /* @__PURE__ */ requirePgsql()), O.registerLanguage("php", /* @__PURE__ */ requirePhp()), O.registerLanguage("php-template", /* @__PURE__ */ requirePhpTemplate()), O.registerLanguage("plaintext", /* @__PURE__ */ requirePlaintext()), O.registerLanguage("pony", /* @__PURE__ */ requirePony()), O.registerLanguage("powershell", /* @__PURE__ */ requirePowershell()), O.registerLanguage("processing", /* @__PURE__ */ requireProcessing()), O.registerLanguage("profile", /* @__PURE__ */ requireProfile()), O.registerLanguage("prolog", /* @__PURE__ */ requireProlog()), O.registerLanguage("properties", /* @__PURE__ */ requireProperties()), O.registerLanguage("protobuf", /* @__PURE__ */ requireProtobuf()), O.registerLanguage("puppet", /* @__PURE__ */ requirePuppet()), O.registerLanguage("purebasic", /* @__PURE__ */ requirePurebasic()), O.registerLanguage("python", /* @__PURE__ */ requirePython()), O.registerLanguage("python-repl", /* @__PURE__ */ requirePythonRepl()), O.registerLanguage("q", /* @__PURE__ */ requireQ()), O.registerLanguage("qml", /* @__PURE__ */ requireQml()), O.registerLanguage("r", /* @__PURE__ */ requireR()), O.registerLanguage("reasonml", /* @__PURE__ */ requireReasonml()), O.registerLanguage("rib", /* @__PURE__ */ requireRib()), O.registerLanguage("roboconf", /* @__PURE__ */ requireRoboconf()), O.registerLanguage("routeros", /* @__PURE__ */ requireRouteros()), O.registerLanguage("rsl", /* @__PURE__ */ requireRsl()), O.registerLanguage("ruleslanguage", /* @__PURE__ */ requireRuleslanguage()), O.registerLanguage("rust", /* @__PURE__ */ requireRust()), O.registerLanguage("sas", /* @__PURE__ */ requireSas()), O.registerLanguage("scala", /* @__PURE__ */ requireScala()), O.registerLanguage("scheme", /* @__PURE__ */ requireScheme()), O.registerLanguage("scilab", /* @__PURE__ */ requireScilab()), O.registerLanguage("scss", /* @__PURE__ */ requireScss()), O.registerLanguage("shell", /* @__PURE__ */ requireShell()), O.registerLanguage("smali", /* @__PURE__ */ requireSmali()), O.registerLanguage("smalltalk", /* @__PURE__ */ requireSmalltalk()), O.registerLanguage("sml", /* @__PURE__ */ requireSml()), O.registerLanguage("sqf", /* @__PURE__ */ requireSqf()), O.registerLanguage("sql", /* @__PURE__ */ requireSql()), O.registerLanguage("stan", /* @__PURE__ */ requireStan()), O.registerLanguage("stata", /* @__PURE__ */ requireStata()), O.registerLanguage("step21", /* @__PURE__ */ requireStep21()), O.registerLanguage("stylus", /* @__PURE__ */ requireStylus()), O.registerLanguage("subunit", /* @__PURE__ */ requireSubunit()), O.registerLanguage("swift", /* @__PURE__ */ requireSwift()), O.registerLanguage("taggerscript", /* @__PURE__ */ requireTaggerscript()), O.registerLanguage("yaml", /* @__PURE__ */ requireYaml()), O.registerLanguage("tap", /* @__PURE__ */ requireTap()), O.registerLanguage("tcl", /* @__PURE__ */ requireTcl()), O.registerLanguage("thrift", /* @__PURE__ */ requireThrift()), O.registerLanguage("tp", /* @__PURE__ */ requireTp()), O.registerLanguage("twig", /* @__PURE__ */ requireTwig()), O.registerLanguage("typescript", /* @__PURE__ */ requireTypescript()), O.registerLanguage("vala", /* @__PURE__ */ requireVala()), O.registerLanguage("vbnet", /* @__PURE__ */ requireVbnet()), O.registerLanguage("vbscript", /* @__PURE__ */ requireVbscript()), O.registerLanguage("vbscript-html", /* @__PURE__ */ requireVbscriptHtml()), O.registerLanguage("verilog", /* @__PURE__ */ requireVerilog()), O.registerLanguage("vhdl", /* @__PURE__ */ requireVhdl()), O.registerLanguage("vim", /* @__PURE__ */ requireVim()), O.registerLanguage("wasm", /* @__PURE__ */ requireWasm()), O.registerLanguage("wren", /* @__PURE__ */ requireWren()), O.registerLanguage("x86asm", /* @__PURE__ */ requireX86asm()), O.registerLanguage("xl", /* @__PURE__ */ requireXl()), O.registerLanguage("xquery", /* @__PURE__ */ requireXquery()), O.registerLanguage("zephir", /* @__PURE__ */ requireZephir()), O.HighlightJS = O, O.default = O, lib$1 = O, lib$1;
  }
  var libExports$1 = /* @__PURE__ */ requireLib$1();
  const HighlightJS = /* @__PURE__ */ getDefaultExportFromCjs$1(libExports$1);
  function _getDefaults() {
    return {
      async: !1,
      breaks: !1,
      extensions: null,
      gfm: !0,
      hooks: null,
      pedantic: !1,
      renderer: null,
      silent: !1,
      tokenizer: null,
      walkTokens: null
    };
  }
  let _defaults = _getDefaults();
  function changeDefaults(O) {
    _defaults = O;
  }
  const noopTest = { exec: () => null };
  function edit(O, _ = "") {
    let H = typeof O == "string" ? O : O.source;
    const V = {
      replace: (q, G) => {
        let J = typeof G == "string" ? G : G.source;
        return J = J.replace(other.caret, "$1"), H = H.replace(q, J), V;
      },
      getRegex: () => new RegExp(H, _)
    };
    return V;
  }
  const other = {
    codeRemoveIndent: /^(?: {1,4}| {0,3}\t)/gm,
    outputLinkReplace: /\\([\[\]])/g,
    indentCodeCompensation: /^(\s+)(?:```)/,
    beginningSpace: /^\s+/,
    endingHash: /#$/,
    startingSpaceChar: /^ /,
    endingSpaceChar: / $/,
    nonSpaceChar: /[^ ]/,
    newLineCharGlobal: /\n/g,
    tabCharGlobal: /\t/g,
    multipleSpaceGlobal: /\s+/g,
    blankLine: /^[ \t]*$/,
    doubleBlankLine: /\n[ \t]*\n[ \t]*$/,
    blockquoteStart: /^ {0,3}>/,
    blockquoteSetextReplace: /\n {0,3}((?:=+|-+) *)(?=\n|$)/g,
    blockquoteSetextReplace2: /^ {0,3}>[ \t]?/gm,
    listReplaceTabs: /^\t+/,
    listReplaceNesting: /^ {1,4}(?=( {4})*[^ ])/g,
    listIsTask: /^\[[ xX]\] /,
    listReplaceTask: /^\[[ xX]\] +/,
    anyLine: /\n.*\n/,
    hrefBrackets: /^<(.*)>$/,
    tableDelimiter: /[:|]/,
    tableAlignChars: /^\||\| *$/g,
    tableRowBlankLine: /\n[ \t]*$/,
    tableAlignRight: /^ *-+: *$/,
    tableAlignCenter: /^ *:-+: *$/,
    tableAlignLeft: /^ *:-+ *$/,
    startATag: /^<a /i,
    endATag: /^<\/a>/i,
    startPreScriptTag: /^<(pre|code|kbd|script)(\s|>)/i,
    endPreScriptTag: /^<\/(pre|code|kbd|script)(\s|>)/i,
    startAngleBracket: /^</,
    endAngleBracket: />$/,
    pedanticHrefTitle: /^([^'"]*[^\s])\s+(['"])(.*)\2/,
    unicodeAlphaNumeric: /[\p{L}\p{N}]/u,
    escapeTest: /[&<>"']/,
    escapeReplace: /[&<>"']/g,
    escapeTestNoEncode: /[<>"']|&(?!(#\d{1,7}|#[Xx][a-fA-F0-9]{1,6}|\w+);)/,
    escapeReplaceNoEncode: /[<>"']|&(?!(#\d{1,7}|#[Xx][a-fA-F0-9]{1,6}|\w+);)/g,
    unescapeTest: /&(#(?:\d+)|(?:#x[0-9A-Fa-f]+)|(?:\w+));?/ig,
    caret: /(^|[^\[])\^/g,
    percentDecode: /%25/g,
    findPipe: /\|/g,
    splitPipe: / \|/,
    slashPipe: /\\\|/g,
    carriageReturn: /\r\n|\r/g,
    spaceLine: /^ +$/gm,
    notSpaceStart: /^\S*/,
    endingNewline: /\n$/,
    listItemRegex: (O) => new RegExp(`^( {0,3}${O})((?:[	 ][^\\n]*)?(?:\\n|$))`),
    nextBulletRegex: (O) => new RegExp(`^ {0,${Math.min(3, O - 1)}}(?:[*+-]|\\d{1,9}[.)])((?:[ 	][^\\n]*)?(?:\\n|$))`),
    hrRegex: (O) => new RegExp(`^ {0,${Math.min(3, O - 1)}}((?:- *){3,}|(?:_ *){3,}|(?:\\* *){3,})(?:\\n+|$)`),
    fencesBeginRegex: (O) => new RegExp(`^ {0,${Math.min(3, O - 1)}}(?:\`\`\`|~~~)`),
    headingBeginRegex: (O) => new RegExp(`^ {0,${Math.min(3, O - 1)}}#`),
    htmlBeginRegex: (O) => new RegExp(`^ {0,${Math.min(3, O - 1)}}<(?:[a-z].*>|!--)`, "i")
  }, newline = /^(?:[ \t]*(?:\n|$))+/, blockCode = /^((?: {4}| {0,3}\t)[^\n]+(?:\n(?:[ \t]*(?:\n|$))*)?)+/, fences = /^ {0,3}(`{3,}(?=[^`\n]*(?:\n|$))|~{3,})([^\n]*)(?:\n|$)(?:|([\s\S]*?)(?:\n|$))(?: {0,3}\1[~`]* *(?=\n|$)|$)/, hr = /^ {0,3}((?:-[\t ]*){3,}|(?:_[ \t]*){3,}|(?:\*[ \t]*){3,})(?:\n+|$)/, heading$1 = /^ {0,3}(#{1,6})(?=\s|$)(.*)(?:\n+|$)/, bullet = /(?:[*+-]|\d{1,9}[.)])/, lheading = edit(/^(?!bull |blockCode|fences|blockquote|heading|html)((?:.|\n(?!\s*?\n|bull |blockCode|fences|blockquote|heading|html))+?)\n {0,3}(=+|-+) *(?:\n+|$)/).replace(/bull/g, bullet).replace(/blockCode/g, /(?: {4}| {0,3}\t)/).replace(/fences/g, / {0,3}(?:`{3,}|~{3,})/).replace(/blockquote/g, / {0,3}>/).replace(/heading/g, / {0,3}#{1,6}/).replace(/html/g, / {0,3}<[^\n>]+>\n/).getRegex(), _paragraph = /^([^\n]+(?:\n(?!hr|heading|lheading|blockquote|fences|list|html|table| +\n)[^\n]+)*)/, blockText = /^[^\n]+/, _blockLabel = /(?!\s*\])(?:\\.|[^\[\]\\])+/, def$1 = edit(/^ {0,3}\[(label)\]: *(?:\n[ \t]*)?([^<\s][^\s]*|<.*?>)(?:(?: +(?:\n[ \t]*)?| *\n[ \t]*)(title))? *(?:\n+|$)/).replace("label", _blockLabel).replace("title", /(?:"(?:\\"?|[^"\\])*"|'[^'\n]*(?:\n[^'\n]+)*\n?'|\([^()]*\))/).getRegex(), list = edit(/^( {0,3}bull)([ \t][^\n]+?)?(?:\n|$)/).replace(/bull/g, bullet).getRegex(), _tag = "address|article|aside|base|basefont|blockquote|body|caption|center|col|colgroup|dd|details|dialog|dir|div|dl|dt|fieldset|figcaption|figure|footer|form|frame|frameset|h[1-6]|head|header|hr|html|iframe|legend|li|link|main|menu|menuitem|meta|nav|noframes|ol|optgroup|option|p|param|search|section|summary|table|tbody|td|tfoot|th|thead|title|tr|track|ul", _comment = /<!--(?:-?>|[\s\S]*?(?:-->|$))/, html$1 = edit("^ {0,3}(?:<(script|pre|style|textarea)[\\s>][\\s\\S]*?(?:</\\1>[^\\n]*\\n+|$)|comment[^\\n]*(\\n+|$)|<\\?[\\s\\S]*?(?:\\?>\\n*|$)|<![A-Z][\\s\\S]*?(?:>\\n*|$)|<!\\[CDATA\\[[\\s\\S]*?(?:\\]\\]>\\n*|$)|</?(tag)(?: +|\\n|/?>)[\\s\\S]*?(?:(?:\\n[ 	]*)+\\n|$)|<(?!script|pre|style|textarea)([a-z][\\w-]*)(?:attribute)*? */?>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:(?:\\n[ 	]*)+\\n|$)|</(?!script|pre|style|textarea)[a-z][\\w-]*\\s*>(?=[ \\t]*(?:\\n|$))[\\s\\S]*?(?:(?:\\n[ 	]*)+\\n|$))", "i").replace("comment", _comment).replace("tag", _tag).replace("attribute", / +[a-zA-Z:_][\w.:-]*(?: *= *"[^"\n]*"| *= *'[^'\n]*'| *= *[^\s"'=<>`]+)?/).getRegex(), paragraph = edit(_paragraph).replace("hr", hr).replace("heading", " {0,3}#{1,6}(?:\\s|$)").replace("|lheading", "").replace("|table", "").replace("blockquote", " {0,3}>").replace("fences", " {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list", " {0,3}(?:[*+-]|1[.)]) ").replace("html", "</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag", _tag).getRegex(), blockquote = edit(/^( {0,3}> ?(paragraph|[^\n]*)(?:\n|$))+/).replace("paragraph", paragraph).getRegex(), blockNormal = {
    blockquote,
    code: blockCode,
    def: def$1,
    fences,
    heading: heading$1,
    hr,
    html: html$1,
    lheading,
    list,
    newline,
    paragraph,
    table: noopTest,
    text: blockText
  }, gfmTable = edit("^ *([^\\n ].*)\\n {0,3}((?:\\| *)?:?-+:? *(?:\\| *:?-+:? *)*(?:\\| *)?)(?:\\n((?:(?! *\\n|hr|heading|blockquote|code|fences|list|html).*(?:\\n|$))*)\\n*|$)").replace("hr", hr).replace("heading", " {0,3}#{1,6}(?:\\s|$)").replace("blockquote", " {0,3}>").replace("code", "(?: {4}| {0,3}	)[^\\n]").replace("fences", " {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list", " {0,3}(?:[*+-]|1[.)]) ").replace("html", "</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag", _tag).getRegex(), blockGfm = {
    ...blockNormal,
    table: gfmTable,
    paragraph: edit(_paragraph).replace("hr", hr).replace("heading", " {0,3}#{1,6}(?:\\s|$)").replace("|lheading", "").replace("table", gfmTable).replace("blockquote", " {0,3}>").replace("fences", " {0,3}(?:`{3,}(?=[^`\\n]*\\n)|~{3,})[^\\n]*\\n").replace("list", " {0,3}(?:[*+-]|1[.)]) ").replace("html", "</?(?:tag)(?: +|\\n|/?>)|<(?:script|pre|style|textarea|!--)").replace("tag", _tag).getRegex()
  }, blockPedantic = {
    ...blockNormal,
    html: edit(`^ *(?:comment *(?:\\n|\\s*$)|<(tag)[\\s\\S]+?</\\1> *(?:\\n{2,}|\\s*$)|<tag(?:"[^"]*"|'[^']*'|\\s[^'"/>\\s]*)*?/?> *(?:\\n{2,}|\\s*$))`).replace("comment", _comment).replace(/tag/g, "(?!(?:a|em|strong|small|s|cite|q|dfn|abbr|data|time|code|var|samp|kbd|sub|sup|i|b|u|mark|ruby|rt|rp|bdi|bdo|span|br|wbr|ins|del|img)\\b)\\w+(?!:|[^\\w\\s@]*@)\\b").getRegex(),
    def: /^ *\[([^\]]+)\]: *<?([^\s>]+)>?(?: +(["(][^\n]+[")]))? *(?:\n+|$)/,
    heading: /^(#{1,6})(.*)(?:\n+|$)/,
    fences: noopTest,
    // fences not supported
    lheading: /^(.+?)\n {0,3}(=+|-+) *(?:\n+|$)/,
    paragraph: edit(_paragraph).replace("hr", hr).replace("heading", ` *#{1,6} *[^
]`).replace("lheading", lheading).replace("|table", "").replace("blockquote", " {0,3}>").replace("|fences", "").replace("|list", "").replace("|html", "").replace("|tag", "").getRegex()
  }, escape$1$1 = /^\\([!"#$%&'()*+,\-./:;<=>?@\[\]\\^_`{|}~])/, inlineCode = /^(`+)([^`]|[^`][\s\S]*?[^`])\1(?!`)/, br$1 = /^( {2,}|\\)\n(?!\s*$)/, inlineText = /^(`+|[^`])(?:(?= {2,}\n)|[\s\S]*?(?:(?=[\\<!\[`*_]|\b_|$)|[^ ](?= {2,}\n)))/, _punctuation = /[\p{P}\p{S}]/u, _punctuationOrSpace = /[\s\p{P}\p{S}]/u, _notPunctuationOrSpace = /[^\s\p{P}\p{S}]/u, punctuation$1 = edit(/^((?![*_])punctSpace)/, "u").replace(/punctSpace/g, _punctuationOrSpace).getRegex(), _punctuationGfmStrongEm = /(?!~)[\p{P}\p{S}]/u, _punctuationOrSpaceGfmStrongEm = /(?!~)[\s\p{P}\p{S}]/u, _notPunctuationOrSpaceGfmStrongEm = /(?:[^\s\p{P}\p{S}]|~)/u, blockSkip = /\[[^[\]]*?\]\((?:\\.|[^\\\(\)]|\((?:\\.|[^\\\(\)])*\))*\)|`[^`]*?`|<[^<>]*?>/g, emStrongLDelim = edit(/^(?:\*+(?:((?!\*)punct)|[^\s*]))|^_+(?:((?!_)punct)|([^\s_]))/, "u").replace(/punct/g, _punctuation).getRegex(), emStrongRDelimAstCore = "^[^_*]*?__[^_*]*?\\*[^_*]*?(?=__)|[^*]+(?=[^*])|(?!\\*)punct(\\*+)(?=[\\s]|$)|notPunctSpace(\\*+)(?!\\*)(?=punctSpace|$)|(?!\\*)punctSpace(\\*+)(?=notPunctSpace)|[\\s](\\*+)(?!\\*)(?=punct)|(?!\\*)punct(\\*+)(?!\\*)(?=punct)|notPunctSpace(\\*+)(?=notPunctSpace)", emStrongRDelimAst = edit(emStrongRDelimAstCore, "gu").replace(/notPunctSpace/g, _notPunctuationOrSpace).replace(/punctSpace/g, _punctuationOrSpace).replace(/punct/g, _punctuation).getRegex(), emStrongRDelimAstGfm = edit(emStrongRDelimAstCore, "gu").replace(/notPunctSpace/g, _notPunctuationOrSpaceGfmStrongEm).replace(/punctSpace/g, _punctuationOrSpaceGfmStrongEm).replace(/punct/g, _punctuationGfmStrongEm).getRegex(), emStrongRDelimUnd = edit("^[^_*]*?\\*\\*[^_*]*?_[^_*]*?(?=\\*\\*)|[^_]+(?=[^_])|(?!_)punct(_+)(?=[\\s]|$)|notPunctSpace(_+)(?!_)(?=punctSpace|$)|(?!_)punctSpace(_+)(?=notPunctSpace)|[\\s](_+)(?!_)(?=punct)|(?!_)punct(_+)(?!_)(?=punct)", "gu").replace(/notPunctSpace/g, _notPunctuationOrSpace).replace(/punctSpace/g, _punctuationOrSpace).replace(/punct/g, _punctuation).getRegex(), anyPunctuation = edit(/\\(punct)/, "gu").replace(/punct/g, _punctuation).getRegex(), autolink = edit(/^<(scheme:[^\s\x00-\x1f<>]*|email)>/).replace("scheme", /[a-zA-Z][a-zA-Z0-9+.-]{1,31}/).replace("email", /[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+(@)[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+(?![-_])/).getRegex(), _inlineComment = edit(_comment).replace("(?:-->|$)", "-->").getRegex(), tag = edit("^comment|^</[a-zA-Z][\\w:-]*\\s*>|^<[a-zA-Z][\\w-]*(?:attribute)*?\\s*/?>|^<\\?[\\s\\S]*?\\?>|^<![a-zA-Z]+\\s[\\s\\S]*?>|^<!\\[CDATA\\[[\\s\\S]*?\\]\\]>").replace("comment", _inlineComment).replace("attribute", /\s+[a-zA-Z:_][\w.:-]*(?:\s*=\s*"[^"]*"|\s*=\s*'[^']*'|\s*=\s*[^\s"'=<>`]+)?/).getRegex(), _inlineLabel = /(?:\[(?:\\.|[^\[\]\\])*\]|\\.|`[^`]*`|[^\[\]\\`])*?/, link$2 = edit(/^!?\[(label)\]\(\s*(href)(?:\s+(title))?\s*\)/).replace("label", _inlineLabel).replace("href", /<(?:\\.|[^\n<>\\])+>|[^\s\x00-\x1f]*/).replace("title", /"(?:\\"?|[^"\\])*"|'(?:\\'?|[^'\\])*'|\((?:\\\)?|[^)\\])*\)/).getRegex(), reflink = edit(/^!?\[(label)\]\[(ref)\]/).replace("label", _inlineLabel).replace("ref", _blockLabel).getRegex(), nolink = edit(/^!?\[(ref)\](?:\[\])?/).replace("ref", _blockLabel).getRegex(), reflinkSearch = edit("reflink|nolink(?!\\()", "g").replace("reflink", reflink).replace("nolink", nolink).getRegex(), inlineNormal = {
    _backpedal: noopTest,
    // only used for GFM url
    anyPunctuation,
    autolink,
    blockSkip,
    br: br$1,
    code: inlineCode,
    del: noopTest,
    emStrongLDelim,
    emStrongRDelimAst,
    emStrongRDelimUnd,
    escape: escape$1$1,
    link: link$2,
    nolink,
    punctuation: punctuation$1,
    reflink,
    reflinkSearch,
    tag,
    text: inlineText,
    url: noopTest
  }, inlinePedantic = {
    ...inlineNormal,
    link: edit(/^!?\[(label)\]\((.*?)\)/).replace("label", _inlineLabel).getRegex(),
    reflink: edit(/^!?\[(label)\]\s*\[([^\]]*)\]/).replace("label", _inlineLabel).getRegex()
  }, inlineGfm = {
    ...inlineNormal,
    emStrongRDelimAst: emStrongRDelimAstGfm,
    url: edit(/^((?:ftp|https?):\/\/|www\.)(?:[a-zA-Z0-9\-]+\.?)+[^\s<]*|^email/, "i").replace("email", /[A-Za-z0-9._+-]+(@)[a-zA-Z0-9-_]+(?:\.[a-zA-Z0-9-_]*[a-zA-Z0-9])+(?![-_])/).getRegex(),
    _backpedal: /(?:[^?!.,:;*_'"~()&]+|\([^)]*\)|&(?![a-zA-Z0-9]+;$)|[?!.,:;*_'"~)]+(?!$))+/,
    del: /^(~~?)(?=[^\s~])((?:\\.|[^\\])*?(?:\\.|[^\s~\\]))\1(?=[^~]|$)/,
    text: /^([`~]+|[^`~])(?:(?= {2,}\n)|(?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@)|[\s\S]*?(?:(?=[\\<!\[`*~_]|\b_|https?:\/\/|ftp:\/\/|www\.|$)|[^ ](?= {2,}\n)|[^a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-](?=[a-zA-Z0-9.!#$%&'*+\/=?_`{\|}~-]+@)))/
  }, inlineBreaks = {
    ...inlineGfm,
    br: edit(br$1).replace("{2,}", "*").getRegex(),
    text: edit(inlineGfm.text).replace("\\b_", "\\b_| {2,}\\n").replace(/\{2,\}/g, "*").getRegex()
  }, block = {
    normal: blockNormal,
    gfm: blockGfm,
    pedantic: blockPedantic
  }, inline$1 = {
    normal: inlineNormal,
    gfm: inlineGfm,
    breaks: inlineBreaks,
    pedantic: inlinePedantic
  }, escapeReplacements$1 = {
    "&": "&amp;",
    "<": "&lt;",
    ">": "&gt;",
    '"': "&quot;",
    "'": "&#39;"
  }, getEscapeReplacement$1 = (O) => escapeReplacements$1[O];
  function escape$3(O, _) {
    if (_) {
      if (other.escapeTest.test(O))
        return O.replace(other.escapeReplace, getEscapeReplacement$1);
    } else if (other.escapeTestNoEncode.test(O))
      return O.replace(other.escapeReplaceNoEncode, getEscapeReplacement$1);
    return O;
  }
  function cleanUrl(O) {
    try {
      O = encodeURI(O).replace(other.percentDecode, "%");
    } catch {
      return null;
    }
    return O;
  }
  function splitCells(O, _) {
    var G;
    const H = O.replace(other.findPipe, (J, Z, te) => {
      let ne = !1, ee = Z;
      for (; --ee >= 0 && te[ee] === "\\"; )
        ne = !ne;
      return ne ? "|" : " |";
    }), V = H.split(other.splitPipe);
    let q = 0;
    if (V[0].trim() || V.shift(), V.length > 0 && !((G = V.at(-1)) != null && G.trim()) && V.pop(), _)
      if (V.length > _)
        V.splice(_);
      else
        for (; V.length < _; )
          V.push("");
    for (; q < V.length; q++)
      V[q] = V[q].trim().replace(other.slashPipe, "|");
    return V;
  }
  function rtrim(O, _, H) {
    const V = O.length;
    if (V === 0)
      return "";
    let q = 0;
    for (; q < V && O.charAt(V - q - 1) === _; )
      q++;
    return O.slice(0, V - q);
  }
  function findClosingBracket(O, _) {
    if (O.indexOf(_[1]) === -1)
      return -1;
    let H = 0;
    for (let V = 0; V < O.length; V++)
      if (O[V] === "\\")
        V++;
      else if (O[V] === _[0])
        H++;
      else if (O[V] === _[1] && (H--, H < 0))
        return V;
    return -1;
  }
  function outputLink(O, _, H, V, q) {
    const G = _.href, J = _.title || null, Z = O[1].replace(q.other.outputLinkReplace, "$1");
    if (O[0].charAt(0) !== "!") {
      V.state.inLink = !0;
      const te = {
        type: "link",
        raw: H,
        href: G,
        title: J,
        text: Z,
        tokens: V.inlineTokens(Z)
      };
      return V.state.inLink = !1, te;
    }
    return {
      type: "image",
      raw: H,
      href: G,
      title: J,
      text: Z
    };
  }
  function indentCodeCompensation(O, _, H) {
    const V = O.match(H.other.indentCodeCompensation);
    if (V === null)
      return _;
    const q = V[1];
    return _.split(`
`).map((G) => {
      const J = G.match(H.other.beginningSpace);
      if (J === null)
        return G;
      const [Z] = J;
      return Z.length >= q.length ? G.slice(q.length) : G;
    }).join(`
`);
  }
  class _Tokenizer {
    // set by the lexer
    constructor(_) {
      mt(this, "options");
      mt(this, "rules");
      // set by the lexer
      mt(this, "lexer");
      this.options = _ || _defaults;
    }
    space(_) {
      const H = this.rules.block.newline.exec(_);
      if (H && H[0].length > 0)
        return {
          type: "space",
          raw: H[0]
        };
    }
    code(_) {
      const H = this.rules.block.code.exec(_);
      if (H) {
        const V = H[0].replace(this.rules.other.codeRemoveIndent, "");
        return {
          type: "code",
          raw: H[0],
          codeBlockStyle: "indented",
          text: this.options.pedantic ? V : rtrim(V, `
`)
        };
      }
    }
    fences(_) {
      const H = this.rules.block.fences.exec(_);
      if (H) {
        const V = H[0], q = indentCodeCompensation(V, H[3] || "", this.rules);
        return {
          type: "code",
          raw: V,
          lang: H[2] ? H[2].trim().replace(this.rules.inline.anyPunctuation, "$1") : H[2],
          text: q
        };
      }
    }
    heading(_) {
      const H = this.rules.block.heading.exec(_);
      if (H) {
        let V = H[2].trim();
        if (this.rules.other.endingHash.test(V)) {
          const q = rtrim(V, "#");
          (this.options.pedantic || !q || this.rules.other.endingSpaceChar.test(q)) && (V = q.trim());
        }
        return {
          type: "heading",
          raw: H[0],
          depth: H[1].length,
          text: V,
          tokens: this.lexer.inline(V)
        };
      }
    }
    hr(_) {
      const H = this.rules.block.hr.exec(_);
      if (H)
        return {
          type: "hr",
          raw: rtrim(H[0], `
`)
        };
    }
    blockquote(_) {
      const H = this.rules.block.blockquote.exec(_);
      if (H) {
        let V = rtrim(H[0], `
`).split(`
`), q = "", G = "";
        const J = [];
        for (; V.length > 0; ) {
          let Z = !1;
          const te = [];
          let ne;
          for (ne = 0; ne < V.length; ne++)
            if (this.rules.other.blockquoteStart.test(V[ne]))
              te.push(V[ne]), Z = !0;
            else if (!Z)
              te.push(V[ne]);
            else
              break;
          V = V.slice(ne);
          const ee = te.join(`
`), ae = ee.replace(this.rules.other.blockquoteSetextReplace, `
    $1`).replace(this.rules.other.blockquoteSetextReplace2, "");
          q = q ? `${q}
${ee}` : ee, G = G ? `${G}
${ae}` : ae;
          const oe = this.lexer.state.top;
          if (this.lexer.state.top = !0, this.lexer.blockTokens(ae, J, !0), this.lexer.state.top = oe, V.length === 0)
            break;
          const se = J.at(-1);
          if ((se == null ? void 0 : se.type) === "code")
            break;
          if ((se == null ? void 0 : se.type) === "blockquote") {
            const ce = se, le = ce.raw + `
` + V.join(`
`), ue = this.blockquote(le);
            J[J.length - 1] = ue, q = q.substring(0, q.length - ce.raw.length) + ue.raw, G = G.substring(0, G.length - ce.text.length) + ue.text;
            break;
          } else if ((se == null ? void 0 : se.type) === "list") {
            const ce = se, le = ce.raw + `
` + V.join(`
`), ue = this.list(le);
            J[J.length - 1] = ue, q = q.substring(0, q.length - se.raw.length) + ue.raw, G = G.substring(0, G.length - ce.raw.length) + ue.raw, V = le.substring(J.at(-1).raw.length).split(`
`);
            continue;
          }
        }
        return {
          type: "blockquote",
          raw: q,
          tokens: J,
          text: G
        };
      }
    }
    list(_) {
      let H = this.rules.block.list.exec(_);
      if (H) {
        let V = H[1].trim();
        const q = V.length > 1, G = {
          type: "list",
          raw: "",
          ordered: q,
          start: q ? +V.slice(0, -1) : "",
          loose: !1,
          items: []
        };
        V = q ? `\\d{1,9}\\${V.slice(-1)}` : `\\${V}`, this.options.pedantic && (V = q ? V : "[*+-]");
        const J = this.rules.other.listItemRegex(V);
        let Z = !1;
        for (; _; ) {
          let ne = !1, ee = "", ae = "";
          if (!(H = J.exec(_)) || this.rules.block.hr.test(_))
            break;
          ee = H[0], _ = _.substring(ee.length);
          let oe = H[2].split(`
`, 1)[0].replace(this.rules.other.listReplaceTabs, (fe) => " ".repeat(3 * fe.length)), se = _.split(`
`, 1)[0], ce = !oe.trim(), le = 0;
          if (this.options.pedantic ? (le = 2, ae = oe.trimStart()) : ce ? le = H[1].length + 1 : (le = H[2].search(this.rules.other.nonSpaceChar), le = le > 4 ? 1 : le, ae = oe.slice(le), le += H[1].length), ce && this.rules.other.blankLine.test(se) && (ee += se + `
`, _ = _.substring(se.length + 1), ne = !0), !ne) {
            const fe = this.rules.other.nextBulletRegex(le), he = this.rules.other.hrRegex(le), me = this.rules.other.fencesBeginRegex(le), pe = this.rules.other.headingBeginRegex(le), ye = this.rules.other.htmlBeginRegex(le);
            for (; _; ) {
              const ge = _.split(`
`, 1)[0];
              let ve;
              if (se = ge, this.options.pedantic ? (se = se.replace(this.rules.other.listReplaceNesting, "  "), ve = se) : ve = se.replace(this.rules.other.tabCharGlobal, "    "), me.test(se) || pe.test(se) || ye.test(se) || fe.test(se) || he.test(se))
                break;
              if (ve.search(this.rules.other.nonSpaceChar) >= le || !se.trim())
                ae += `
` + ve.slice(le);
              else {
                if (ce || oe.replace(this.rules.other.tabCharGlobal, "    ").search(this.rules.other.nonSpaceChar) >= 4 || me.test(oe) || pe.test(oe) || he.test(oe))
                  break;
                ae += `
` + se;
              }
              !ce && !se.trim() && (ce = !0), ee += ge + `
`, _ = _.substring(ge.length + 1), oe = ve.slice(le);
            }
          }
          G.loose || (Z ? G.loose = !0 : this.rules.other.doubleBlankLine.test(ee) && (Z = !0));
          let ue = null, de;
          this.options.gfm && (ue = this.rules.other.listIsTask.exec(ae), ue && (de = ue[0] !== "[ ] ", ae = ae.replace(this.rules.other.listReplaceTask, ""))), G.items.push({
            type: "list_item",
            raw: ee,
            task: !!ue,
            checked: de,
            loose: !1,
            text: ae,
            tokens: []
          }), G.raw += ee;
        }
        const te = G.items.at(-1);
        if (te)
          te.raw = te.raw.trimEnd(), te.text = te.text.trimEnd();
        else
          return;
        G.raw = G.raw.trimEnd();
        for (let ne = 0; ne < G.items.length; ne++)
          if (this.lexer.state.top = !1, G.items[ne].tokens = this.lexer.blockTokens(G.items[ne].text, []), !G.loose) {
            const ee = G.items[ne].tokens.filter((oe) => oe.type === "space"), ae = ee.length > 0 && ee.some((oe) => this.rules.other.anyLine.test(oe.raw));
            G.loose = ae;
          }
        if (G.loose)
          for (let ne = 0; ne < G.items.length; ne++)
            G.items[ne].loose = !0;
        return G;
      }
    }
    html(_) {
      const H = this.rules.block.html.exec(_);
      if (H)
        return {
          type: "html",
          block: !0,
          raw: H[0],
          pre: H[1] === "pre" || H[1] === "script" || H[1] === "style",
          text: H[0]
        };
    }
    def(_) {
      const H = this.rules.block.def.exec(_);
      if (H) {
        const V = H[1].toLowerCase().replace(this.rules.other.multipleSpaceGlobal, " "), q = H[2] ? H[2].replace(this.rules.other.hrefBrackets, "$1").replace(this.rules.inline.anyPunctuation, "$1") : "", G = H[3] ? H[3].substring(1, H[3].length - 1).replace(this.rules.inline.anyPunctuation, "$1") : H[3];
        return {
          type: "def",
          tag: V,
          raw: H[0],
          href: q,
          title: G
        };
      }
    }
    table(_) {
      var Z;
      const H = this.rules.block.table.exec(_);
      if (!H || !this.rules.other.tableDelimiter.test(H[2]))
        return;
      const V = splitCells(H[1]), q = H[2].replace(this.rules.other.tableAlignChars, "").split("|"), G = (Z = H[3]) != null && Z.trim() ? H[3].replace(this.rules.other.tableRowBlankLine, "").split(`
`) : [], J = {
        type: "table",
        raw: H[0],
        header: [],
        align: [],
        rows: []
      };
      if (V.length === q.length) {
        for (const te of q)
          this.rules.other.tableAlignRight.test(te) ? J.align.push("right") : this.rules.other.tableAlignCenter.test(te) ? J.align.push("center") : this.rules.other.tableAlignLeft.test(te) ? J.align.push("left") : J.align.push(null);
        for (let te = 0; te < V.length; te++)
          J.header.push({
            text: V[te],
            tokens: this.lexer.inline(V[te]),
            header: !0,
            align: J.align[te]
          });
        for (const te of G)
          J.rows.push(splitCells(te, J.header.length).map((ne, ee) => ({
            text: ne,
            tokens: this.lexer.inline(ne),
            header: !1,
            align: J.align[ee]
          })));
        return J;
      }
    }
    lheading(_) {
      const H = this.rules.block.lheading.exec(_);
      if (H)
        return {
          type: "heading",
          raw: H[0],
          depth: H[2].charAt(0) === "=" ? 1 : 2,
          text: H[1],
          tokens: this.lexer.inline(H[1])
        };
    }
    paragraph(_) {
      const H = this.rules.block.paragraph.exec(_);
      if (H) {
        const V = H[1].charAt(H[1].length - 1) === `
` ? H[1].slice(0, -1) : H[1];
        return {
          type: "paragraph",
          raw: H[0],
          text: V,
          tokens: this.lexer.inline(V)
        };
      }
    }
    text(_) {
      const H = this.rules.block.text.exec(_);
      if (H)
        return {
          type: "text",
          raw: H[0],
          text: H[0],
          tokens: this.lexer.inline(H[0])
        };
    }
    escape(_) {
      const H = this.rules.inline.escape.exec(_);
      if (H)
        return {
          type: "escape",
          raw: H[0],
          text: H[1]
        };
    }
    tag(_) {
      const H = this.rules.inline.tag.exec(_);
      if (H)
        return !this.lexer.state.inLink && this.rules.other.startATag.test(H[0]) ? this.lexer.state.inLink = !0 : this.lexer.state.inLink && this.rules.other.endATag.test(H[0]) && (this.lexer.state.inLink = !1), !this.lexer.state.inRawBlock && this.rules.other.startPreScriptTag.test(H[0]) ? this.lexer.state.inRawBlock = !0 : this.lexer.state.inRawBlock && this.rules.other.endPreScriptTag.test(H[0]) && (this.lexer.state.inRawBlock = !1), {
          type: "html",
          raw: H[0],
          inLink: this.lexer.state.inLink,
          inRawBlock: this.lexer.state.inRawBlock,
          block: !1,
          text: H[0]
        };
    }
    link(_) {
      const H = this.rules.inline.link.exec(_);
      if (H) {
        const V = H[2].trim();
        if (!this.options.pedantic && this.rules.other.startAngleBracket.test(V)) {
          if (!this.rules.other.endAngleBracket.test(V))
            return;
          const J = rtrim(V.slice(0, -1), "\\");
          if ((V.length - J.length) % 2 === 0)
            return;
        } else {
          const J = findClosingBracket(H[2], "()");
          if (J > -1) {
            const te = (H[0].indexOf("!") === 0 ? 5 : 4) + H[1].length + J;
            H[2] = H[2].substring(0, J), H[0] = H[0].substring(0, te).trim(), H[3] = "";
          }
        }
        let q = H[2], G = "";
        if (this.options.pedantic) {
          const J = this.rules.other.pedanticHrefTitle.exec(q);
          J && (q = J[1], G = J[3]);
        } else
          G = H[3] ? H[3].slice(1, -1) : "";
        return q = q.trim(), this.rules.other.startAngleBracket.test(q) && (this.options.pedantic && !this.rules.other.endAngleBracket.test(V) ? q = q.slice(1) : q = q.slice(1, -1)), outputLink(H, {
          href: q && q.replace(this.rules.inline.anyPunctuation, "$1"),
          title: G && G.replace(this.rules.inline.anyPunctuation, "$1")
        }, H[0], this.lexer, this.rules);
      }
    }
    reflink(_, H) {
      let V;
      if ((V = this.rules.inline.reflink.exec(_)) || (V = this.rules.inline.nolink.exec(_))) {
        const q = (V[2] || V[1]).replace(this.rules.other.multipleSpaceGlobal, " "), G = H[q.toLowerCase()];
        if (!G) {
          const J = V[0].charAt(0);
          return {
            type: "text",
            raw: J,
            text: J
          };
        }
        return outputLink(V, G, V[0], this.lexer, this.rules);
      }
    }
    emStrong(_, H, V = "") {
      let q = this.rules.inline.emStrongLDelim.exec(_);
      if (!q || q[3] && V.match(this.rules.other.unicodeAlphaNumeric))
        return;
      if (!(q[1] || q[2] || "") || !V || this.rules.inline.punctuation.exec(V)) {
        const J = [...q[0]].length - 1;
        let Z, te, ne = J, ee = 0;
        const ae = q[0][0] === "*" ? this.rules.inline.emStrongRDelimAst : this.rules.inline.emStrongRDelimUnd;
        for (ae.lastIndex = 0, H = H.slice(-1 * _.length + J); (q = ae.exec(H)) != null; ) {
          if (Z = q[1] || q[2] || q[3] || q[4] || q[5] || q[6], !Z)
            continue;
          if (te = [...Z].length, q[3] || q[4]) {
            ne += te;
            continue;
          } else if ((q[5] || q[6]) && J % 3 && !((J + te) % 3)) {
            ee += te;
            continue;
          }
          if (ne -= te, ne > 0)
            continue;
          te = Math.min(te, te + ne + ee);
          const oe = [...q[0]][0].length, se = _.slice(0, J + q.index + oe + te);
          if (Math.min(J, te) % 2) {
            const le = se.slice(1, -1);
            return {
              type: "em",
              raw: se,
              text: le,
              tokens: this.lexer.inlineTokens(le)
            };
          }
          const ce = se.slice(2, -2);
          return {
            type: "strong",
            raw: se,
            text: ce,
            tokens: this.lexer.inlineTokens(ce)
          };
        }
      }
    }
    codespan(_) {
      const H = this.rules.inline.code.exec(_);
      if (H) {
        let V = H[2].replace(this.rules.other.newLineCharGlobal, " ");
        const q = this.rules.other.nonSpaceChar.test(V), G = this.rules.other.startingSpaceChar.test(V) && this.rules.other.endingSpaceChar.test(V);
        return q && G && (V = V.substring(1, V.length - 1)), {
          type: "codespan",
          raw: H[0],
          text: V
        };
      }
    }
    br(_) {
      const H = this.rules.inline.br.exec(_);
      if (H)
        return {
          type: "br",
          raw: H[0]
        };
    }
    del(_) {
      const H = this.rules.inline.del.exec(_);
      if (H)
        return {
          type: "del",
          raw: H[0],
          text: H[2],
          tokens: this.lexer.inlineTokens(H[2])
        };
    }
    autolink(_) {
      const H = this.rules.inline.autolink.exec(_);
      if (H) {
        let V, q;
        return H[2] === "@" ? (V = H[1], q = "mailto:" + V) : (V = H[1], q = V), {
          type: "link",
          raw: H[0],
          text: V,
          href: q,
          tokens: [
            {
              type: "text",
              raw: V,
              text: V
            }
          ]
        };
      }
    }
    url(_) {
      var V;
      let H;
      if (H = this.rules.inline.url.exec(_)) {
        let q, G;
        if (H[2] === "@")
          q = H[0], G = "mailto:" + q;
        else {
          let J;
          do
            J = H[0], H[0] = ((V = this.rules.inline._backpedal.exec(H[0])) == null ? void 0 : V[0]) ?? "";
          while (J !== H[0]);
          q = H[0], H[1] === "www." ? G = "http://" + H[0] : G = H[0];
        }
        return {
          type: "link",
          raw: H[0],
          text: q,
          href: G,
          tokens: [
            {
              type: "text",
              raw: q,
              text: q
            }
          ]
        };
      }
    }
    inlineText(_) {
      const H = this.rules.inline.text.exec(_);
      if (H) {
        const V = this.lexer.state.inRawBlock;
        return {
          type: "text",
          raw: H[0],
          text: H[0],
          escaped: V
        };
      }
    }
  }
  class _Lexer {
    constructor(_) {
      mt(this, "tokens");
      mt(this, "options");
      mt(this, "state");
      mt(this, "tokenizer");
      mt(this, "inlineQueue");
      this.tokens = [], this.tokens.links = /* @__PURE__ */ Object.create(null), this.options = _ || _defaults, this.options.tokenizer = this.options.tokenizer || new _Tokenizer(), this.tokenizer = this.options.tokenizer, this.tokenizer.options = this.options, this.tokenizer.lexer = this, this.inlineQueue = [], this.state = {
        inLink: !1,
        inRawBlock: !1,
        top: !0
      };
      const H = {
        other,
        block: block.normal,
        inline: inline$1.normal
      };
      this.options.pedantic ? (H.block = block.pedantic, H.inline = inline$1.pedantic) : this.options.gfm && (H.block = block.gfm, this.options.breaks ? H.inline = inline$1.breaks : H.inline = inline$1.gfm), this.tokenizer.rules = H;
    }
    /**
     * Expose Rules
     */
    static get rules() {
      return {
        block,
        inline: inline$1
      };
    }
    /**
     * Static Lex Method
     */
    static lex(_, H) {
      return new _Lexer(H).lex(_);
    }
    /**
     * Static Lex Inline Method
     */
    static lexInline(_, H) {
      return new _Lexer(H).inlineTokens(_);
    }
    /**
     * Preprocessing
     */
    lex(_) {
      _ = _.replace(other.carriageReturn, `
`), this.blockTokens(_, this.tokens);
      for (let H = 0; H < this.inlineQueue.length; H++) {
        const V = this.inlineQueue[H];
        this.inlineTokens(V.src, V.tokens);
      }
      return this.inlineQueue = [], this.tokens;
    }
    blockTokens(_, H = [], V = !1) {
      var q, G, J;
      for (this.options.pedantic && (_ = _.replace(other.tabCharGlobal, "    ").replace(other.spaceLine, "")); _; ) {
        let Z;
        if ((G = (q = this.options.extensions) == null ? void 0 : q.block) != null && G.some((ne) => (Z = ne.call({ lexer: this }, _, H)) ? (_ = _.substring(Z.raw.length), H.push(Z), !0) : !1))
          continue;
        if (Z = this.tokenizer.space(_)) {
          _ = _.substring(Z.raw.length);
          const ne = H.at(-1);
          Z.raw.length === 1 && ne !== void 0 ? ne.raw += `
` : H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.code(_)) {
          _ = _.substring(Z.raw.length);
          const ne = H.at(-1);
          (ne == null ? void 0 : ne.type) === "paragraph" || (ne == null ? void 0 : ne.type) === "text" ? (ne.raw += `
` + Z.raw, ne.text += `
` + Z.text, this.inlineQueue.at(-1).src = ne.text) : H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.fences(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.heading(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.hr(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.blockquote(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.list(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.html(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.def(_)) {
          _ = _.substring(Z.raw.length);
          const ne = H.at(-1);
          (ne == null ? void 0 : ne.type) === "paragraph" || (ne == null ? void 0 : ne.type) === "text" ? (ne.raw += `
` + Z.raw, ne.text += `
` + Z.raw, this.inlineQueue.at(-1).src = ne.text) : this.tokens.links[Z.tag] || (this.tokens.links[Z.tag] = {
            href: Z.href,
            title: Z.title
          });
          continue;
        }
        if (Z = this.tokenizer.table(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        if (Z = this.tokenizer.lheading(_)) {
          _ = _.substring(Z.raw.length), H.push(Z);
          continue;
        }
        let te = _;
        if ((J = this.options.extensions) != null && J.startBlock) {
          let ne = 1 / 0;
          const ee = _.slice(1);
          let ae;
          this.options.extensions.startBlock.forEach((oe) => {
            ae = oe.call({ lexer: this }, ee), typeof ae == "number" && ae >= 0 && (ne = Math.min(ne, ae));
          }), ne < 1 / 0 && ne >= 0 && (te = _.substring(0, ne + 1));
        }
        if (this.state.top && (Z = this.tokenizer.paragraph(te))) {
          const ne = H.at(-1);
          V && (ne == null ? void 0 : ne.type) === "paragraph" ? (ne.raw += `
` + Z.raw, ne.text += `
` + Z.text, this.inlineQueue.pop(), this.inlineQueue.at(-1).src = ne.text) : H.push(Z), V = te.length !== _.length, _ = _.substring(Z.raw.length);
          continue;
        }
        if (Z = this.tokenizer.text(_)) {
          _ = _.substring(Z.raw.length);
          const ne = H.at(-1);
          (ne == null ? void 0 : ne.type) === "text" ? (ne.raw += `
` + Z.raw, ne.text += `
` + Z.text, this.inlineQueue.pop(), this.inlineQueue.at(-1).src = ne.text) : H.push(Z);
          continue;
        }
        if (_) {
          const ne = "Infinite loop on byte: " + _.charCodeAt(0);
          if (this.options.silent) {
            console.error(ne);
            break;
          } else
            throw new Error(ne);
        }
      }
      return this.state.top = !0, H;
    }
    inline(_, H = []) {
      return this.inlineQueue.push({ src: _, tokens: H }), H;
    }
    /**
     * Lexing/Compiling
     */
    inlineTokens(_, H = []) {
      var Z, te, ne;
      let V = _, q = null;
      if (this.tokens.links) {
        const ee = Object.keys(this.tokens.links);
        if (ee.length > 0)
          for (; (q = this.tokenizer.rules.inline.reflinkSearch.exec(V)) != null; )
            ee.includes(q[0].slice(q[0].lastIndexOf("[") + 1, -1)) && (V = V.slice(0, q.index) + "[" + "a".repeat(q[0].length - 2) + "]" + V.slice(this.tokenizer.rules.inline.reflinkSearch.lastIndex));
      }
      for (; (q = this.tokenizer.rules.inline.blockSkip.exec(V)) != null; )
        V = V.slice(0, q.index) + "[" + "a".repeat(q[0].length - 2) + "]" + V.slice(this.tokenizer.rules.inline.blockSkip.lastIndex);
      for (; (q = this.tokenizer.rules.inline.anyPunctuation.exec(V)) != null; )
        V = V.slice(0, q.index) + "++" + V.slice(this.tokenizer.rules.inline.anyPunctuation.lastIndex);
      let G = !1, J = "";
      for (; _; ) {
        G || (J = ""), G = !1;
        let ee;
        if ((te = (Z = this.options.extensions) == null ? void 0 : Z.inline) != null && te.some((oe) => (ee = oe.call({ lexer: this }, _, H)) ? (_ = _.substring(ee.raw.length), H.push(ee), !0) : !1))
          continue;
        if (ee = this.tokenizer.escape(_)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.tag(_)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.link(_)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.reflink(_, this.tokens.links)) {
          _ = _.substring(ee.raw.length);
          const oe = H.at(-1);
          ee.type === "text" && (oe == null ? void 0 : oe.type) === "text" ? (oe.raw += ee.raw, oe.text += ee.text) : H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.emStrong(_, V, J)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.codespan(_)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.br(_)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.del(_)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (ee = this.tokenizer.autolink(_)) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        if (!this.state.inLink && (ee = this.tokenizer.url(_))) {
          _ = _.substring(ee.raw.length), H.push(ee);
          continue;
        }
        let ae = _;
        if ((ne = this.options.extensions) != null && ne.startInline) {
          let oe = 1 / 0;
          const se = _.slice(1);
          let ce;
          this.options.extensions.startInline.forEach((le) => {
            ce = le.call({ lexer: this }, se), typeof ce == "number" && ce >= 0 && (oe = Math.min(oe, ce));
          }), oe < 1 / 0 && oe >= 0 && (ae = _.substring(0, oe + 1));
        }
        if (ee = this.tokenizer.inlineText(ae)) {
          _ = _.substring(ee.raw.length), ee.raw.slice(-1) !== "_" && (J = ee.raw.slice(-1)), G = !0;
          const oe = H.at(-1);
          (oe == null ? void 0 : oe.type) === "text" ? (oe.raw += ee.raw, oe.text += ee.text) : H.push(ee);
          continue;
        }
        if (_) {
          const oe = "Infinite loop on byte: " + _.charCodeAt(0);
          if (this.options.silent) {
            console.error(oe);
            break;
          } else
            throw new Error(oe);
        }
      }
      return H;
    }
  }
  class _Renderer {
    // set by the parser
    constructor(_) {
      mt(this, "options");
      mt(this, "parser");
      this.options = _ || _defaults;
    }
    space(_) {
      return "";
    }
    code({ text: _, lang: H, escaped: V }) {
      var J;
      const q = (J = (H || "").match(other.notSpaceStart)) == null ? void 0 : J[0], G = _.replace(other.endingNewline, "") + `
`;
      return q ? '<pre><code class="language-' + escape$3(q) + '">' + (V ? G : escape$3(G, !0)) + `</code></pre>
` : "<pre><code>" + (V ? G : escape$3(G, !0)) + `</code></pre>
`;
    }
    blockquote({ tokens: _ }) {
      return `<blockquote>
${this.parser.parse(_)}</blockquote>
`;
    }
    html({ text: _ }) {
      return _;
    }
    heading({ tokens: _, depth: H }) {
      return `<h${H}>${this.parser.parseInline(_)}</h${H}>
`;
    }
    hr(_) {
      return `<hr>
`;
    }
    list(_) {
      const H = _.ordered, V = _.start;
      let q = "";
      for (let Z = 0; Z < _.items.length; Z++) {
        const te = _.items[Z];
        q += this.listitem(te);
      }
      const G = H ? "ol" : "ul", J = H && V !== 1 ? ' start="' + V + '"' : "";
      return "<" + G + J + `>
` + q + "</" + G + `>
`;
    }
    listitem(_) {
      var V;
      let H = "";
      if (_.task) {
        const q = this.checkbox({ checked: !!_.checked });
        _.loose ? ((V = _.tokens[0]) == null ? void 0 : V.type) === "paragraph" ? (_.tokens[0].text = q + " " + _.tokens[0].text, _.tokens[0].tokens && _.tokens[0].tokens.length > 0 && _.tokens[0].tokens[0].type === "text" && (_.tokens[0].tokens[0].text = q + " " + escape$3(_.tokens[0].tokens[0].text), _.tokens[0].tokens[0].escaped = !0)) : _.tokens.unshift({
          type: "text",
          raw: q + " ",
          text: q + " ",
          escaped: !0
        }) : H += q + " ";
      }
      return H += this.parser.parse(_.tokens, !!_.loose), `<li>${H}</li>
`;
    }
    checkbox({ checked: _ }) {
      return "<input " + (_ ? 'checked="" ' : "") + 'disabled="" type="checkbox">';
    }
    paragraph({ tokens: _ }) {
      return `<p>${this.parser.parseInline(_)}</p>
`;
    }
    table(_) {
      let H = "", V = "";
      for (let G = 0; G < _.header.length; G++)
        V += this.tablecell(_.header[G]);
      H += this.tablerow({ text: V });
      let q = "";
      for (let G = 0; G < _.rows.length; G++) {
        const J = _.rows[G];
        V = "";
        for (let Z = 0; Z < J.length; Z++)
          V += this.tablecell(J[Z]);
        q += this.tablerow({ text: V });
      }
      return q && (q = `<tbody>${q}</tbody>`), `<table>
<thead>
` + H + `</thead>
` + q + `</table>
`;
    }
    tablerow({ text: _ }) {
      return `<tr>
${_}</tr>
`;
    }
    tablecell(_) {
      const H = this.parser.parseInline(_.tokens), V = _.header ? "th" : "td";
      return (_.align ? `<${V} align="${_.align}">` : `<${V}>`) + H + `</${V}>
`;
    }
    /**
     * span level renderer
     */
    strong({ tokens: _ }) {
      return `<strong>${this.parser.parseInline(_)}</strong>`;
    }
    em({ tokens: _ }) {
      return `<em>${this.parser.parseInline(_)}</em>`;
    }
    codespan({ text: _ }) {
      return `<code>${escape$3(_, !0)}</code>`;
    }
    br(_) {
      return "<br>";
    }
    del({ tokens: _ }) {
      return `<del>${this.parser.parseInline(_)}</del>`;
    }
    link({ href: _, title: H, tokens: V }) {
      const q = this.parser.parseInline(V), G = cleanUrl(_);
      if (G === null)
        return q;
      _ = G;
      let J = '<a href="' + _ + '"';
      return H && (J += ' title="' + escape$3(H) + '"'), J += ">" + q + "</a>", J;
    }
    image({ href: _, title: H, text: V }) {
      const q = cleanUrl(_);
      if (q === null)
        return escape$3(V);
      _ = q;
      let G = `<img src="${_}" alt="${V}"`;
      return H && (G += ` title="${escape$3(H)}"`), G += ">", G;
    }
    text(_) {
      return "tokens" in _ && _.tokens ? this.parser.parseInline(_.tokens) : "escaped" in _ && _.escaped ? _.text : escape$3(_.text);
    }
  }
  class _TextRenderer {
    // no need for block level renderers
    strong({ text: _ }) {
      return _;
    }
    em({ text: _ }) {
      return _;
    }
    codespan({ text: _ }) {
      return _;
    }
    del({ text: _ }) {
      return _;
    }
    html({ text: _ }) {
      return _;
    }
    text({ text: _ }) {
      return _;
    }
    link({ text: _ }) {
      return "" + _;
    }
    image({ text: _ }) {
      return "" + _;
    }
    br() {
      return "";
    }
  }
  class _Parser {
    constructor(_) {
      mt(this, "options");
      mt(this, "renderer");
      mt(this, "textRenderer");
      this.options = _ || _defaults, this.options.renderer = this.options.renderer || new _Renderer(), this.renderer = this.options.renderer, this.renderer.options = this.options, this.renderer.parser = this, this.textRenderer = new _TextRenderer();
    }
    /**
     * Static Parse Method
     */
    static parse(_, H) {
      return new _Parser(H).parse(_);
    }
    /**
     * Static Parse Inline Method
     */
    static parseInline(_, H) {
      return new _Parser(H).parseInline(_);
    }
    /**
     * Parse Loop
     */
    parse(_, H = !0) {
      var q, G;
      let V = "";
      for (let J = 0; J < _.length; J++) {
        const Z = _[J];
        if ((G = (q = this.options.extensions) == null ? void 0 : q.renderers) != null && G[Z.type]) {
          const ne = Z, ee = this.options.extensions.renderers[ne.type].call({ parser: this }, ne);
          if (ee !== !1 || !["space", "hr", "heading", "code", "table", "blockquote", "list", "html", "paragraph", "text"].includes(ne.type)) {
            V += ee || "";
            continue;
          }
        }
        const te = Z;
        switch (te.type) {
          case "space": {
            V += this.renderer.space(te);
            continue;
          }
          case "hr": {
            V += this.renderer.hr(te);
            continue;
          }
          case "heading": {
            V += this.renderer.heading(te);
            continue;
          }
          case "code": {
            V += this.renderer.code(te);
            continue;
          }
          case "table": {
            V += this.renderer.table(te);
            continue;
          }
          case "blockquote": {
            V += this.renderer.blockquote(te);
            continue;
          }
          case "list": {
            V += this.renderer.list(te);
            continue;
          }
          case "html": {
            V += this.renderer.html(te);
            continue;
          }
          case "paragraph": {
            V += this.renderer.paragraph(te);
            continue;
          }
          case "text": {
            let ne = te, ee = this.renderer.text(ne);
            for (; J + 1 < _.length && _[J + 1].type === "text"; )
              ne = _[++J], ee += `
` + this.renderer.text(ne);
            H ? V += this.renderer.paragraph({
              type: "paragraph",
              raw: ee,
              text: ee,
              tokens: [{ type: "text", raw: ee, text: ee, escaped: !0 }]
            }) : V += ee;
            continue;
          }
          default: {
            const ne = 'Token with "' + te.type + '" type was not found.';
            if (this.options.silent)
              return console.error(ne), "";
            throw new Error(ne);
          }
        }
      }
      return V;
    }
    /**
     * Parse Inline Tokens
     */
    parseInline(_, H = this.renderer) {
      var q, G;
      let V = "";
      for (let J = 0; J < _.length; J++) {
        const Z = _[J];
        if ((G = (q = this.options.extensions) == null ? void 0 : q.renderers) != null && G[Z.type]) {
          const ne = this.options.extensions.renderers[Z.type].call({ parser: this }, Z);
          if (ne !== !1 || !["escape", "html", "link", "image", "strong", "em", "codespan", "br", "del", "text"].includes(Z.type)) {
            V += ne || "";
            continue;
          }
        }
        const te = Z;
        switch (te.type) {
          case "escape": {
            V += H.text(te);
            break;
          }
          case "html": {
            V += H.html(te);
            break;
          }
          case "link": {
            V += H.link(te);
            break;
          }
          case "image": {
            V += H.image(te);
            break;
          }
          case "strong": {
            V += H.strong(te);
            break;
          }
          case "em": {
            V += H.em(te);
            break;
          }
          case "codespan": {
            V += H.codespan(te);
            break;
          }
          case "br": {
            V += H.br(te);
            break;
          }
          case "del": {
            V += H.del(te);
            break;
          }
          case "text": {
            V += H.text(te);
            break;
          }
          default: {
            const ne = 'Token with "' + te.type + '" type was not found.';
            if (this.options.silent)
              return console.error(ne), "";
            throw new Error(ne);
          }
        }
      }
      return V;
    }
  }
  class _Hooks {
    constructor(_) {
      mt(this, "options");
      mt(this, "block");
      this.options = _ || _defaults;
    }
    /**
     * Process markdown before marked
     */
    preprocess(_) {
      return _;
    }
    /**
     * Process HTML after marked is finished
     */
    postprocess(_) {
      return _;
    }
    /**
     * Process all tokens before walk tokens
     */
    processAllTokens(_) {
      return _;
    }
    /**
     * Provide function to tokenize markdown
     */
    provideLexer() {
      return this.block ? _Lexer.lex : _Lexer.lexInline;
    }
    /**
     * Provide function to parse tokens
     */
    provideParser() {
      return this.block ? _Parser.parse : _Parser.parseInline;
    }
  }
  mt(_Hooks, "passThroughHooks", /* @__PURE__ */ new Set([
    "preprocess",
    "postprocess",
    "processAllTokens"
  ]));
  class Marked {
    constructor(..._) {
      mt(this, "defaults", _getDefaults());
      mt(this, "options", this.setOptions);
      mt(this, "parse", this.parseMarkdown(!0));
      mt(this, "parseInline", this.parseMarkdown(!1));
      mt(this, "Parser", _Parser);
      mt(this, "Renderer", _Renderer);
      mt(this, "TextRenderer", _TextRenderer);
      mt(this, "Lexer", _Lexer);
      mt(this, "Tokenizer", _Tokenizer);
      mt(this, "Hooks", _Hooks);
      this.use(..._);
    }
    /**
     * Run callback for every token
     */
    walkTokens(_, H) {
      var q, G;
      let V = [];
      for (const J of _)
        switch (V = V.concat(H.call(this, J)), J.type) {
          case "table": {
            const Z = J;
            for (const te of Z.header)
              V = V.concat(this.walkTokens(te.tokens, H));
            for (const te of Z.rows)
              for (const ne of te)
                V = V.concat(this.walkTokens(ne.tokens, H));
            break;
          }
          case "list": {
            const Z = J;
            V = V.concat(this.walkTokens(Z.items, H));
            break;
          }
          default: {
            const Z = J;
            (G = (q = this.defaults.extensions) == null ? void 0 : q.childTokens) != null && G[Z.type] ? this.defaults.extensions.childTokens[Z.type].forEach((te) => {
              const ne = Z[te].flat(1 / 0);
              V = V.concat(this.walkTokens(ne, H));
            }) : Z.tokens && (V = V.concat(this.walkTokens(Z.tokens, H)));
          }
        }
      return V;
    }
    use(..._) {
      const H = this.defaults.extensions || { renderers: {}, childTokens: {} };
      return _.forEach((V) => {
        const q = { ...V };
        if (q.async = this.defaults.async || q.async || !1, V.extensions && (V.extensions.forEach((G) => {
          if (!G.name)
            throw new Error("extension name required");
          if ("renderer" in G) {
            const J = H.renderers[G.name];
            J ? H.renderers[G.name] = function(...Z) {
              let te = G.renderer.apply(this, Z);
              return te === !1 && (te = J.apply(this, Z)), te;
            } : H.renderers[G.name] = G.renderer;
          }
          if ("tokenizer" in G) {
            if (!G.level || G.level !== "block" && G.level !== "inline")
              throw new Error("extension level must be 'block' or 'inline'");
            const J = H[G.level];
            J ? J.unshift(G.tokenizer) : H[G.level] = [G.tokenizer], G.start && (G.level === "block" ? H.startBlock ? H.startBlock.push(G.start) : H.startBlock = [G.start] : G.level === "inline" && (H.startInline ? H.startInline.push(G.start) : H.startInline = [G.start]));
          }
          "childTokens" in G && G.childTokens && (H.childTokens[G.name] = G.childTokens);
        }), q.extensions = H), V.renderer) {
          const G = this.defaults.renderer || new _Renderer(this.defaults);
          for (const J in V.renderer) {
            if (!(J in G))
              throw new Error(`renderer '${J}' does not exist`);
            if (["options", "parser"].includes(J))
              continue;
            const Z = J, te = V.renderer[Z], ne = G[Z];
            G[Z] = (...ee) => {
              let ae = te.apply(G, ee);
              return ae === !1 && (ae = ne.apply(G, ee)), ae || "";
            };
          }
          q.renderer = G;
        }
        if (V.tokenizer) {
          const G = this.defaults.tokenizer || new _Tokenizer(this.defaults);
          for (const J in V.tokenizer) {
            if (!(J in G))
              throw new Error(`tokenizer '${J}' does not exist`);
            if (["options", "rules", "lexer"].includes(J))
              continue;
            const Z = J, te = V.tokenizer[Z], ne = G[Z];
            G[Z] = (...ee) => {
              let ae = te.apply(G, ee);
              return ae === !1 && (ae = ne.apply(G, ee)), ae;
            };
          }
          q.tokenizer = G;
        }
        if (V.hooks) {
          const G = this.defaults.hooks || new _Hooks();
          for (const J in V.hooks) {
            if (!(J in G))
              throw new Error(`hook '${J}' does not exist`);
            if (["options", "block"].includes(J))
              continue;
            const Z = J, te = V.hooks[Z], ne = G[Z];
            _Hooks.passThroughHooks.has(J) ? G[Z] = (ee) => {
              if (this.defaults.async)
                return Promise.resolve(te.call(G, ee)).then((oe) => ne.call(G, oe));
              const ae = te.call(G, ee);
              return ne.call(G, ae);
            } : G[Z] = (...ee) => {
              let ae = te.apply(G, ee);
              return ae === !1 && (ae = ne.apply(G, ee)), ae;
            };
          }
          q.hooks = G;
        }
        if (V.walkTokens) {
          const G = this.defaults.walkTokens, J = V.walkTokens;
          q.walkTokens = function(Z) {
            let te = [];
            return te.push(J.call(this, Z)), G && (te = te.concat(G.call(this, Z))), te;
          };
        }
        this.defaults = { ...this.defaults, ...q };
      }), this;
    }
    setOptions(_) {
      return this.defaults = { ...this.defaults, ..._ }, this;
    }
    lexer(_, H) {
      return _Lexer.lex(_, H ?? this.defaults);
    }
    parser(_, H) {
      return _Parser.parse(_, H ?? this.defaults);
    }
    parseMarkdown(_) {
      return (V, q) => {
        const G = { ...q }, J = { ...this.defaults, ...G }, Z = this.onError(!!J.silent, !!J.async);
        if (this.defaults.async === !0 && G.async === !1)
          return Z(new Error("marked(): The async option was set to true by an extension. Remove async: false from the parse options object to return a Promise."));
        if (typeof V > "u" || V === null)
          return Z(new Error("marked(): input parameter is undefined or null"));
        if (typeof V != "string")
          return Z(new Error("marked(): input parameter is of type " + Object.prototype.toString.call(V) + ", string expected"));
        J.hooks && (J.hooks.options = J, J.hooks.block = _);
        const te = J.hooks ? J.hooks.provideLexer() : _ ? _Lexer.lex : _Lexer.lexInline, ne = J.hooks ? J.hooks.provideParser() : _ ? _Parser.parse : _Parser.parseInline;
        if (J.async)
          return Promise.resolve(J.hooks ? J.hooks.preprocess(V) : V).then((ee) => te(ee, J)).then((ee) => J.hooks ? J.hooks.processAllTokens(ee) : ee).then((ee) => J.walkTokens ? Promise.all(this.walkTokens(ee, J.walkTokens)).then(() => ee) : ee).then((ee) => ne(ee, J)).then((ee) => J.hooks ? J.hooks.postprocess(ee) : ee).catch(Z);
        try {
          J.hooks && (V = J.hooks.preprocess(V));
          let ee = te(V, J);
          J.hooks && (ee = J.hooks.processAllTokens(ee)), J.walkTokens && this.walkTokens(ee, J.walkTokens);
          let ae = ne(ee, J);
          return J.hooks && (ae = J.hooks.postprocess(ae)), ae;
        } catch (ee) {
          return Z(ee);
        }
      };
    }
    onError(_, H) {
      return (V) => {
        if (V.message += `
Please report this to https://github.com/markedjs/marked.`, _) {
          const q = "<p>An error occurred:</p><pre>" + escape$3(V.message + "", !0) + "</pre>";
          return H ? Promise.resolve(q) : q;
        }
        if (H)
          return Promise.reject(V);
        throw V;
      };
    }
  }
  const markedInstance = new Marked();
  function marked(O, _) {
    return markedInstance.parse(O, _);
  }
  marked.options = marked.setOptions = function(O) {
    return markedInstance.setOptions(O), marked.defaults = markedInstance.defaults, changeDefaults(marked.defaults), marked;
  };
  marked.getDefaults = _getDefaults;
  marked.defaults = _defaults;
  marked.use = function(...O) {
    return markedInstance.use(...O), marked.defaults = markedInstance.defaults, changeDefaults(marked.defaults), marked;
  };
  marked.walkTokens = function(O, _) {
    return markedInstance.walkTokens(O, _);
  };
  marked.parseInline = markedInstance.parseInline;
  marked.Parser = _Parser;
  marked.parser = _Parser.parse;
  marked.Renderer = _Renderer;
  marked.TextRenderer = _TextRenderer;
  marked.Lexer = _Lexer;
  marked.lexer = _Lexer.lex;
  marked.Tokenizer = _Tokenizer;
  marked.Hooks = _Hooks;
  marked.parse = marked;
  marked.options;
  marked.setOptions;
  marked.use;
  marked.walkTokens;
  marked.parseInline;
  _Parser.parse;
  _Lexer.lex;
  function markedHighlight(O) {
    if (typeof O == "function" && (O = {
      highlight: O
    }), !O || typeof O.highlight != "function")
      throw new Error("Must provide highlight function");
    return typeof O.langPrefix != "string" && (O.langPrefix = "language-"), typeof O.emptyLangClass != "string" && (O.emptyLangClass = ""), {
      async: !!O.async,
      walkTokens(_) {
        if (_.type !== "code")
          return;
        const H = getLang(_.lang);
        if (O.async)
          return Promise.resolve(O.highlight(_.text, H, _.lang || "")).then(updateToken(_));
        const V = O.highlight(_.text, H, _.lang || "");
        if (V instanceof Promise)
          throw new Error("markedHighlight is not set to async but the highlight function is async. Set the async option to true on markedHighlight to await the async highlight function.");
        updateToken(_)(V);
      },
      useNewRenderer: !0,
      renderer: {
        code(_, H, V) {
          typeof _ == "object" && (V = _.escaped, H = _.lang, _ = _.text);
          const q = getLang(H), G = q ? O.langPrefix + escape$2(q) : O.emptyLangClass, J = G ? ` class="${G}"` : "";
          return _ = _.replace(/\n$/, ""), `<pre><code${J}>${V ? _ : escape$2(_, !0)}
</code></pre>`;
        }
      }
    };
  }
  function getLang(O) {
    return (O || "").match(/\S*/)[0];
  }
  function updateToken(O) {
    return (_) => {
      typeof _ == "string" && _ !== O.text && (O.escaped = !0, O.text = _);
    };
  }
  const escapeTest = /[&<>"']/, escapeReplace = new RegExp(escapeTest.source, "g"), escapeTestNoEncode = /[<>"']|&(?!(#\d{1,7}|#[Xx][a-fA-F0-9]{1,6}|\w+);)/, escapeReplaceNoEncode = new RegExp(escapeTestNoEncode.source, "g"), escapeReplacements = {
    "&": "&amp;",
    "<": "&lt;",
    ">": "&gt;",
    '"': "&quot;",
    "'": "&#39;"
  }, getEscapeReplacement = (O) => escapeReplacements[O];
  function escape$2(O, _) {
    if (_) {
      if (escapeTest.test(O))
        return O.replace(escapeReplace, getEscapeReplacement);
    } else if (escapeTestNoEncode.test(O))
      return O.replace(escapeReplaceNoEncode, getEscapeReplacement);
    return O;
  }
  const _hoisted_1$h = ["innerHTML"], _sfc_main$w = /* @__PURE__ */ defineComponent({
    __name: "Text",
    props: {
      content: {},
      isLightProse: { type: Boolean },
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = new Marked(
        markedHighlight({
          highlight(q, G) {
            const J = HighlightJS.getLanguage(G) ? G : "plaintext";
            return HighlightJS.highlight(q, { language: J }).value;
          }
        })
      ), H = O, V = computed(() => _.parse(H.content));
      return (q, G) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: q.figure,
        "single-block-embed": q.singleBlockEmbed
      }, {
        default: withCtx(() => [
          createBaseVNode("div", {
            class: normalizeClass(["w-full overflow-y-hidden", { dark: q.isLightProse }]),
            "data-cy": "block-markdown"
          }, [
            createBaseVNode("div", {
              class: "w-full prose font-ar-prose dark:prose-invert text-container",
              innerHTML: V.value
            }, null, 8, _hoisted_1$h)
          ], 2)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), VTextBlock = /* @__PURE__ */ _export_sfc(_sfc_main$w, [["__scopeId", "data-v-1768524f"]]), jsonType = (O) => O.type || "", jsonIsMarkdown = (O) => O._type === "Text", jsonIsFormula = (O) => O._type === "Formula", jsonIsBigNumber = (O) => O._type === "BigNumber", jsonIsMedia = (O) => O._type === "Media", jsonIsHTML = (O) => O._type === "HTML", jsonIsCode = (O) => O._type === "Code", jsonIsEmbed = (O) => O._type === "Embed", jsonIsEmpty = (O) => O._type === "Empty", jsonIsAttachment = (O) => O._type === "Attachment", jsonIsTextBox = (O) => O._type === "TextBox", jsonIsURLField = (O) => O._type === "URLField", jsonIsEmailField = (O) => O._type === "EmailField", jsonIsTelephoneField = (O) => O._type === "TelephoneField", jsonIsSearchField = (O) => O._type === "SearchField", jsonIsTextareaField = (O) => O._type === "TextareaField", jsonIsPasswordField = (O) => O._type === "PasswordField", jsonIsHiddenField = (O) => O._type === "HiddenField", jsonIsColorField = (O) => O._type === "ColorField", jsonIsNumberBox = (O) => O._type === "NumberBox", jsonIsRangeField = (O) => O._type === "RangeField", jsonIsSwitchField = (O) => O._type === "SwitchField", jsonIsTagsField = (O) => O._type === "TagsField", jsonIsSelectField = (O) => O._type === "ChoiceField", jsonIsMultiChoiceField = (O) => O._type === "MultiChoiceField", jsonIsFileField = (O) => O._type === "FileField", jsonIsDateTimeField = (O) => O._type === "DateTimeField", jsonIsDateField = (O) => O._type === "DateField", jsonIsTimeField = (O) => O._type === "TimeField", jsonIsGroup = (O) => O._type === "Group", jsonIsView = (O) => O._type === "View", jsonIsSelect = (O) => O._type === "Select", jsonIsToggle = (O) => O._type === "Toggle", jsonIsCompute = (O) => O._type === "Compute", jsonIsVega = (O) => jsonType(O).includes("application/vnd.vegalite"), jsonIsArrowTable = (O) => jsonType(O).includes("application/vnd.apache.arrow+binary"), jsonIsHTMLTable = (O) => jsonType(O) === "application/vnd.arakawa.table+html", jsonIsIFrameHTML = (O) => jsonType(O) === "application/vnd.folium+html", jsonIsBokeh = (O) => jsonType(O) === "application/vnd.bokeh.show+json", jsonIsPlotly = (O) => jsonType(O) === "application/vnd.plotly.v1+json", jsonIsSvg = (O) => jsonType(O).includes("image/svg"), mkBlockMap = (O) => [
    {
      class_: TextBlock,
      test: jsonIsMarkdown,
      opts: { isLightProse: O }
    },
    { class_: BokehBlock, test: jsonIsBokeh },
    {
      class_: DataTableBlock,
      test: jsonIsArrowTable
    },
    { class_: CodeBlock, test: jsonIsCode },
    { class_: VegaBlock, test: jsonIsVega },
    { class_: PlotlyBlock, test: jsonIsPlotly },
    { class_: TableBlock, test: jsonIsHTMLTable },
    {
      class_: HTMLBlock,
      test: jsonIsHTML
    },
    // NOTE - `MediaBlock` check should go before `SVGBlock` check,
    // as SVGs in a `Media` tag have precedence over plot SVGs
    { class_: MediaBlock, test: jsonIsMedia },
    { class_: SVGBlock, test: jsonIsSvg },
    { class_: FormulaBlock, test: jsonIsFormula },
    { class_: EmbedBlock, test: jsonIsEmbed },
    { class_: FoliumBlock, test: jsonIsIFrameHTML },
    { class_: BigNumberBlock, test: jsonIsBigNumber },
    { class_: AttachmentBlock, test: jsonIsAttachment },
    { class_: ComputeBlock, test: jsonIsCompute },
    { class_: Group$1, test: jsonIsGroup },
    { class_: View$3, test: jsonIsView },
    { class_: Select$1, test: jsonIsSelect },
    { class_: Toggle$1, test: jsonIsToggle },
    { class_: EmptyBlock, test: jsonIsEmpty },
    // control blocks
    { class_: ColorField, test: jsonIsColorField },
    { class_: FileField, test: jsonIsFileField },
    { class_: HiddenField, test: jsonIsHiddenField },
    { class_: MultiChoiceField, test: jsonIsMultiChoiceField },
    { class_: NumberBox, test: jsonIsNumberBox },
    { class_: PasswordField, test: jsonIsPasswordField },
    { class_: RangeField, test: jsonIsRangeField },
    { class_: SelectField, test: jsonIsSelectField },
    { class_: SwitchField, test: jsonIsSwitchField },
    { class_: TagsField, test: jsonIsTagsField },
    {
      class_: TemporalTextBox,
      test: jsonIsTextBox,
      opts: { type: "text" }
    },
    {
      class_: TemporalTextBox,
      test: jsonIsURLField,
      opts: { type: "url" }
    },
    {
      class_: TemporalTextBox,
      test: jsonIsTelephoneField,
      opts: { type: "tel" }
    },
    {
      class_: TemporalTextBox,
      test: jsonIsEmailField,
      opts: { type: "email" }
    },
    {
      class_: TemporalTextBox,
      test: jsonIsSearchField,
      opts: { type: "search" }
    },
    {
      class_: TemporalTextBox,
      test: jsonIsTextareaField,
      opts: { type: "textarea" }
    },
    {
      class_: TemporalDateTimeField,
      test: jsonIsDateTimeField,
      opts: { timeFormat: "YYYY-MM-DDTHH:mm:ss", type: "datetime-local" }
    },
    {
      class_: TemporalDateTimeField,
      test: jsonIsDateField,
      opts: { timeFormat: "YYYY-MM-DD", type: "date" }
    },
    {
      class_: TemporalDateTimeField,
      test: jsonIsTimeField,
      opts: {
        timeFormat: "HH:mm:ss",
        type: "time",
        parseFormat: "HH:mm:ss"
      }
    }
  ], isSingleBlockEmbed = (O, _) => {
    const H = (V) => isLayoutBlock(V) && V.children.length === 1 ? H(V.children[0]) : !isLayoutBlock(V);
    return _ === "EMBED" && H(O);
  }, useRootStore = /* @__PURE__ */ defineStore("root", () => {
    const O = reactive({
      Figure: 0,
      Table: 0,
      Plot: 0
    }), _ = reactive([]), H = ref$1({}), V = reactive({}), q = ref$1(), G = (ne) => {
      const ee = _.find((ae) => ae.test(ne));
      if (ee) {
        const { class_: ae, opts: oe } = ee, { caption: se } = ne, ce = se ? te(ae.captionType) : void 0, le = { caption: se, count: ce, captionType: ae.captionType };
        return new ae(ne, le, oe);
      } else
        throw new Error(`Couldn't deserialize from JSON ${ne}`);
    }, J = async (ne, ee) => {
      const { viewJson: ae, assets: oe } = ee.data.result;
      _.push(...mkBlockMap(ne.isLightProse)), Object.assign(V, oe), H.value = Z(ae), q.value = isSingleBlockEmbed(
        H.value,
        ne.mode
      );
    }, Z = (ne, ee = !1) => (isViewElem(ne) && ee && (ne._type = "Group", ne.columns = 1), isParentElem(ne) && (ne.blocks = ne.blocks ? ne.blocks.map((ae) => Z(ae)) : []), G(ne));
    function te(ne) {
      return ++O[ne];
    }
    return {
      report: H,
      counts: O,
      assetMap: V,
      singleBlockEmbed: q,
      setReport: J
    };
  }), _sfc_main$v = /* @__PURE__ */ defineComponent({
    __name: "VegaConnector",
    props: {
      fetchAssetData: {},
      responsive: { type: Boolean },
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = /* @__PURE__ */ defineAsyncComponent(() => Promise.resolve().then(() => Vega)), H = useRootStore(), { singleBlockEmbed: V } = storeToRefs(H), q = O, G = ref$1(null);
      return (async () => G.value = await q.fetchAssetData())(), (J, Z) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: J.figure,
        "single-block-embed": J.singleBlockEmbed || unref(V)
      }, {
        default: withCtx(() => [
          G.value ? (openBlock(), createBlock(unref(_), {
            key: 0,
            "plot-json": G.value,
            responsive: J.responsive,
            "single-block-embed": J.singleBlockEmbed
          }, null, 8, ["plot-json", "responsive", "single-block-embed"])) : createCommentVNode("", !0)
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), readGcsTextOrJsonFile = (O) => axios.get(O).then((_) => _.data);
  var c_;
  let Block$2 = (c_ = class {
    constructor(_, H, V) {
      mt(this, "refId", v4());
      mt(this, "componentProps");
      mt(this, "component");
      mt(this, "caption");
      mt(this, "label");
      mt(this, "count");
      mt(this, "id");
      mt(this, "name");
      // block ID
      mt(this, "_id");
      const { name: q, label: G, _id: J } = _;
      this.count = H.count, this.caption = H.caption;
      const Z = useRootStore();
      this.componentProps = {
        figure: H,
        singleBlockEmbed: Z.singleBlockEmbed
      }, this.id = q, this.label = G, this._type = J;
    }
  }, mt(c_, "captionType", "Figure"), c_);
  class TextBlock extends Block$2 {
    constructor(H, V, q) {
      super(H, V);
      mt(this, "component", markRaw(VTextBlock));
      const { content: G } = H;
      this.componentProps = {
        ...this.componentProps,
        content: G,
        isLightProse: q.isLightProse
      };
    }
  }
  class CodeBlock extends Block$2 {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$I));
      const { content: q, language: G } = H;
      this.componentProps = { ...this.componentProps, code: q, language: G };
    }
  }
  class HTMLBlock extends Block$2 {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(VHTMLBlock));
      const { content: q, sandbox: G } = H;
      this.componentProps = {
        ...this.componentProps,
        html: q,
        sandbox: G
      };
    }
  }
  class FormulaBlock extends Block$2 {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$D));
      const q = H.content;
      this.componentProps = {
        ...this.componentProps,
        content: q
      };
    }
  }
  class EmptyBlock extends Block$2 {
  }
  class BigNumberBlock extends Block$2 {
    constructor(_, H) {
      super(_, H);
      const {
        preValue: V,
        change: q,
        heading: G,
        value: J,
        isPositiveIntent: Z,
        isUpwardChange: te
      } = _, ne = !V && !q;
      this.component = markRaw(
        ne ? _sfc_main$L : _sfc_main$M
      ), this.componentProps = {
        ...this.componentProps,
        heading: G,
        value: J
      }, ne || (this.componentProps = {
        ...this.componentProps,
        isPositiveIntent: Z || !1,
        isUpwardChange: te || !1,
        preValue: V,
        change: q
      });
    }
  }
  class EmbedBlock extends Block$2 {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(VEmbedBlock));
      mt(this, "_isIFrame");
      mt(this, "html");
      const { content: q } = H;
      this.html = q, this.componentProps = {
        ...this.componentProps,
        html: this.html,
        isIframe: this.isIFrame
      };
    }
    get isIFrame() {
      if (typeof this._isIFrame > "u") {
        const V = new DOMParser().parseFromString(
          this.html,
          "text/html"
        ).documentElement.querySelector("body");
        this._isIFrame = !!V && V.childElementCount === 1 && V.children[0].tagName.toLowerCase() === "iframe";
      }
      return this._isIFrame;
    }
  }
  class AssetBlock extends Block$2 {
    constructor(H, V) {
      super(H, V);
      /**
       * Blocks whose data should be fetched on load rather than in-lined in the XML CDATA
       */
      mt(this, "src");
      mt(this, "type");
      const q = useRootStore(), [, G] = H.src.split("://");
      if (!G)
        throw new Error(`Couldn't get block asset ID from src ${H.src}`);
      const { src: J, mime: Z } = q.assetMap[G];
      this.src = J, this.type = Z, this.componentProps = {
        ...this.componentProps,
        fetchAssetData: this.fetchAssetData.bind(this)
      };
    }
    async fetchAssetData() {
      return await readGcsTextOrJsonFile(this.src);
    }
  }
  class TableBlock extends AssetBlock {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$x));
    }
  }
  mt(TableBlock, "captionType", "Table");
  class FoliumBlock extends AssetBlock {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$E));
    }
  }
  mt(FoliumBlock, "captionType", "Plot");
  class AttachmentBlock extends AssetBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$O));
      mt(this, "filename");
      const { filename: q } = H;
      this.filename = q, this.componentProps = {
        ...this.componentProps,
        downloadFile: this.downloadFile.bind(this),
        filename: q
      };
    }
    async downloadFile() {
      return FileSaver_minExports.saveAs(this.src, this.filename);
    }
  }
  class MediaBlock extends AssetBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$B));
      this.componentProps = {
        ...this.componentProps,
        type: this.type,
        src: this.src
      };
    }
  }
  class PlotAssetBlock extends AssetBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "responsive");
      const { responsive: q } = H;
      this.responsive = q, this.componentProps = {
        ...this.componentProps,
        responsive: q
      };
    }
  }
  mt(PlotAssetBlock, "captionType", "Plot");
  class BokehBlock extends PlotAssetBlock {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$K));
    }
  }
  class VegaBlock extends PlotAssetBlock {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$v));
    }
  }
  class PlotlyBlock extends PlotAssetBlock {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$A));
    }
    async fetchAssetData() {
      const H = await readGcsTextOrJsonFile(this.src);
      return JSON.parse(H);
    }
  }
  class SVGBlock extends PlotAssetBlock {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$y));
    }
    async fetchAssetData() {
      return this.src;
    }
  }
  //! moment.js
  //! version : 2.30.1
  //! authors : Tim Wood, Iskren Chernev, Moment.js contributors
  //! license : MIT
  //! momentjs.com
  var hookCallback;
  function hooks() {
    return hookCallback.apply(null, arguments);
  }
  function setHookCallback(O) {
    hookCallback = O;
  }
  function isArray$4(O) {
    return O instanceof Array || Object.prototype.toString.call(O) === "[object Array]";
  }
  function isObject$4(O) {
    return O != null && Object.prototype.toString.call(O) === "[object Object]";
  }
  function hasOwnProp(O, _) {
    return Object.prototype.hasOwnProperty.call(O, _);
  }
  function isObjectEmpty(O) {
    if (Object.getOwnPropertyNames)
      return Object.getOwnPropertyNames(O).length === 0;
    var _;
    for (_ in O)
      if (hasOwnProp(O, _))
        return !1;
    return !0;
  }
  function isUndefined(O) {
    return O === void 0;
  }
  function isNumber$4(O) {
    return typeof O == "number" || Object.prototype.toString.call(O) === "[object Number]";
  }
  function isDate$2(O) {
    return O instanceof Date || Object.prototype.toString.call(O) === "[object Date]";
  }
  function map$5(O, _) {
    var H = [], V, q = O.length;
    for (V = 0; V < q; ++V)
      H.push(_(O[V], V));
    return H;
  }
  function extend$6(O, _) {
    for (var H in _)
      hasOwnProp(_, H) && (O[H] = _[H]);
    return hasOwnProp(_, "toString") && (O.toString = _.toString), hasOwnProp(_, "valueOf") && (O.valueOf = _.valueOf), O;
  }
  function createUTC(O, _, H, V) {
    return createLocalOrUTC(O, _, H, V, !0).utc();
  }
  function defaultParsingFlags() {
    return {
      empty: !1,
      unusedTokens: [],
      unusedInput: [],
      overflow: -2,
      charsLeftOver: 0,
      nullInput: !1,
      invalidEra: null,
      invalidMonth: null,
      invalidFormat: !1,
      userInvalidated: !1,
      iso: !1,
      parsedDateParts: [],
      era: null,
      meridiem: null,
      rfc2822: !1,
      weekdayMismatch: !1
    };
  }
  function getParsingFlags(O) {
    return O._pf == null && (O._pf = defaultParsingFlags()), O._pf;
  }
  var some$3;
  Array.prototype.some ? some$3 = Array.prototype.some : some$3 = function(O) {
    var _ = Object(this), H = _.length >>> 0, V;
    for (V = 0; V < H; V++)
      if (V in _ && O.call(this, _[V], V, _))
        return !0;
    return !1;
  };
  function isValid$1(O) {
    var _ = null, H = !1, V = O._d && !isNaN(O._d.getTime());
    if (V && (_ = getParsingFlags(O), H = some$3.call(_.parsedDateParts, function(q) {
      return q != null;
    }), V = _.overflow < 0 && !_.empty && !_.invalidEra && !_.invalidMonth && !_.invalidWeekday && !_.weekdayMismatch && !_.nullInput && !_.invalidFormat && !_.userInvalidated && (!_.meridiem || _.meridiem && H), O._strict && (V = V && _.charsLeftOver === 0 && _.unusedTokens.length === 0 && _.bigHour === void 0)), Object.isFrozen == null || !Object.isFrozen(O))
      O._isValid = V;
    else
      return V;
    return O._isValid;
  }
  function createInvalid(O) {
    var _ = createUTC(NaN);
    return O != null ? extend$6(getParsingFlags(_), O) : getParsingFlags(_).userInvalidated = !0, _;
  }
  var momentProperties = hooks.momentProperties = [], updateInProgress = !1;
  function copyConfig(O, _) {
    var H, V, q, G = momentProperties.length;
    if (isUndefined(_._isAMomentObject) || (O._isAMomentObject = _._isAMomentObject), isUndefined(_._i) || (O._i = _._i), isUndefined(_._f) || (O._f = _._f), isUndefined(_._l) || (O._l = _._l), isUndefined(_._strict) || (O._strict = _._strict), isUndefined(_._tzm) || (O._tzm = _._tzm), isUndefined(_._isUTC) || (O._isUTC = _._isUTC), isUndefined(_._offset) || (O._offset = _._offset), isUndefined(_._pf) || (O._pf = getParsingFlags(_)), isUndefined(_._locale) || (O._locale = _._locale), G > 0)
      for (H = 0; H < G; H++)
        V = momentProperties[H], q = _[V], isUndefined(q) || (O[V] = q);
    return O;
  }
  function Moment(O) {
    copyConfig(this, O), this._d = new Date(O._d != null ? O._d.getTime() : NaN), this.isValid() || (this._d = /* @__PURE__ */ new Date(NaN)), updateInProgress === !1 && (updateInProgress = !0, hooks.updateOffset(this), updateInProgress = !1);
  }
  function isMoment(O) {
    return O instanceof Moment || O != null && O._isAMomentObject != null;
  }
  function warn$2(O) {
    hooks.suppressDeprecationWarnings === !1 && typeof console < "u" && console.warn && console.warn("Deprecation warning: " + O);
  }
  function deprecate(O, _) {
    var H = !0;
    return extend$6(function() {
      if (hooks.deprecationHandler != null && hooks.deprecationHandler(null, O), H) {
        var V = [], q, G, J, Z = arguments.length;
        for (G = 0; G < Z; G++) {
          if (q = "", typeof arguments[G] == "object") {
            q += `
[` + G + "] ";
            for (J in arguments[0])
              hasOwnProp(arguments[0], J) && (q += J + ": " + arguments[0][J] + ", ");
            q = q.slice(0, -2);
          } else
            q = arguments[G];
          V.push(q);
        }
        warn$2(
          O + `
Arguments: ` + Array.prototype.slice.call(V).join("") + `
` + new Error().stack
        ), H = !1;
      }
      return _.apply(this, arguments);
    }, _);
  }
  var deprecations = {};
  function deprecateSimple(O, _) {
    hooks.deprecationHandler != null && hooks.deprecationHandler(O, _), deprecations[O] || (warn$2(_), deprecations[O] = !0);
  }
  hooks.suppressDeprecationWarnings = !1;
  hooks.deprecationHandler = null;
  function isFunction$4(O) {
    return typeof Function < "u" && O instanceof Function || Object.prototype.toString.call(O) === "[object Function]";
  }
  function set$7(O) {
    var _, H;
    for (H in O)
      hasOwnProp(O, H) && (_ = O[H], isFunction$4(_) ? this[H] = _ : this["_" + H] = _);
    this._config = O, this._dayOfMonthOrdinalParseLenient = new RegExp(
      (this._dayOfMonthOrdinalParse.source || this._ordinalParse.source) + "|" + /\d{1,2}/.source
    );
  }
  function mergeConfigs(O, _) {
    var H = extend$6({}, O), V;
    for (V in _)
      hasOwnProp(_, V) && (isObject$4(O[V]) && isObject$4(_[V]) ? (H[V] = {}, extend$6(H[V], O[V]), extend$6(H[V], _[V])) : _[V] != null ? H[V] = _[V] : delete H[V]);
    for (V in O)
      hasOwnProp(O, V) && !hasOwnProp(_, V) && isObject$4(O[V]) && (H[V] = extend$6({}, H[V]));
    return H;
  }
  function Locale(O) {
    O != null && this.set(O);
  }
  var keys$6;
  Object.keys ? keys$6 = Object.keys : keys$6 = function(O) {
    var _, H = [];
    for (_ in O)
      hasOwnProp(O, _) && H.push(_);
    return H;
  };
  var defaultCalendar = {
    sameDay: "[Today at] LT",
    nextDay: "[Tomorrow at] LT",
    nextWeek: "dddd [at] LT",
    lastDay: "[Yesterday at] LT",
    lastWeek: "[Last] dddd [at] LT",
    sameElse: "L"
  };
  function calendar$1(O, _, H) {
    var V = this._calendar[O] || this._calendar.sameElse;
    return isFunction$4(V) ? V.call(_, H) : V;
  }
  function zeroFill(O, _, H) {
    var V = "" + Math.abs(O), q = _ - V.length, G = O >= 0;
    return (G ? H ? "+" : "" : "-") + Math.pow(10, Math.max(0, q)).toString().substr(1) + V;
  }
  var formattingTokens = /(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g, localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g, formatFunctions = {}, formatTokenFunctions = {};
  function addFormatToken(O, _, H, V) {
    var q = V;
    typeof V == "string" && (q = function() {
      return this[V]();
    }), O && (formatTokenFunctions[O] = q), _ && (formatTokenFunctions[_[0]] = function() {
      return zeroFill(q.apply(this, arguments), _[1], _[2]);
    }), H && (formatTokenFunctions[H] = function() {
      return this.localeData().ordinal(
        q.apply(this, arguments),
        O
      );
    });
  }
  function removeFormattingTokens(O) {
    return O.match(/\[[\s\S]/) ? O.replace(/^\[|\]$/g, "") : O.replace(/\\/g, "");
  }
  function makeFormatFunction(O) {
    var _ = O.match(formattingTokens), H, V;
    for (H = 0, V = _.length; H < V; H++)
      formatTokenFunctions[_[H]] ? _[H] = formatTokenFunctions[_[H]] : _[H] = removeFormattingTokens(_[H]);
    return function(q) {
      var G = "", J;
      for (J = 0; J < V; J++)
        G += isFunction$4(_[J]) ? _[J].call(q, O) : _[J];
      return G;
    };
  }
  function formatMoment(O, _) {
    return O.isValid() ? (_ = expandFormat(_, O.localeData()), formatFunctions[_] = formatFunctions[_] || makeFormatFunction(_), formatFunctions[_](O)) : O.localeData().invalidDate();
  }
  function expandFormat(O, _) {
    var H = 5;
    function V(q) {
      return _.longDateFormat(q) || q;
    }
    for (localFormattingTokens.lastIndex = 0; H >= 0 && localFormattingTokens.test(O); )
      O = O.replace(
        localFormattingTokens,
        V
      ), localFormattingTokens.lastIndex = 0, H -= 1;
    return O;
  }
  var defaultLongDateFormat = {
    LTS: "h:mm:ss A",
    LT: "h:mm A",
    L: "MM/DD/YYYY",
    LL: "MMMM D, YYYY",
    LLL: "MMMM D, YYYY h:mm A",
    LLLL: "dddd, MMMM D, YYYY h:mm A"
  };
  function longDateFormat(O) {
    var _ = this._longDateFormat[O], H = this._longDateFormat[O.toUpperCase()];
    return _ || !H ? _ : (this._longDateFormat[O] = H.match(formattingTokens).map(function(V) {
      return V === "MMMM" || V === "MM" || V === "DD" || V === "dddd" ? V.slice(1) : V;
    }).join(""), this._longDateFormat[O]);
  }
  var defaultInvalidDate = "Invalid date";
  function invalidDate() {
    return this._invalidDate;
  }
  var defaultOrdinal = "%d", defaultDayOfMonthOrdinalParse = /\d{1,2}/;
  function ordinal$1(O) {
    return this._ordinal.replace("%d", O);
  }
  var defaultRelativeTime = {
    future: "in %s",
    past: "%s ago",
    s: "a few seconds",
    ss: "%d seconds",
    m: "a minute",
    mm: "%d minutes",
    h: "an hour",
    hh: "%d hours",
    d: "a day",
    dd: "%d days",
    w: "a week",
    ww: "%d weeks",
    M: "a month",
    MM: "%d months",
    y: "a year",
    yy: "%d years"
  };
  function relativeTime(O, _, H, V) {
    var q = this._relativeTime[H];
    return isFunction$4(q) ? q(O, _, H, V) : q.replace(/%d/i, O);
  }
  function pastFuture(O, _) {
    var H = this._relativeTime[O > 0 ? "future" : "past"];
    return isFunction$4(H) ? H(_) : H.replace(/%s/i, _);
  }
  var aliases = {
    D: "date",
    dates: "date",
    date: "date",
    d: "day",
    days: "day",
    day: "day",
    e: "weekday",
    weekdays: "weekday",
    weekday: "weekday",
    E: "isoWeekday",
    isoweekdays: "isoWeekday",
    isoweekday: "isoWeekday",
    DDD: "dayOfYear",
    dayofyears: "dayOfYear",
    dayofyear: "dayOfYear",
    h: "hour",
    hours: "hour",
    hour: "hour",
    ms: "millisecond",
    milliseconds: "millisecond",
    millisecond: "millisecond",
    m: "minute",
    minutes: "minute",
    minute: "minute",
    M: "month",
    months: "month",
    month: "month",
    Q: "quarter",
    quarters: "quarter",
    quarter: "quarter",
    s: "second",
    seconds: "second",
    second: "second",
    gg: "weekYear",
    weekyears: "weekYear",
    weekyear: "weekYear",
    GG: "isoWeekYear",
    isoweekyears: "isoWeekYear",
    isoweekyear: "isoWeekYear",
    w: "week",
    weeks: "week",
    week: "week",
    W: "isoWeek",
    isoweeks: "isoWeek",
    isoweek: "isoWeek",
    y: "year",
    years: "year",
    year: "year"
  };
  function normalizeUnits(O) {
    return typeof O == "string" ? aliases[O] || aliases[O.toLowerCase()] : void 0;
  }
  function normalizeObjectUnits(O) {
    var _ = {}, H, V;
    for (V in O)
      hasOwnProp(O, V) && (H = normalizeUnits(V), H && (_[H] = O[V]));
    return _;
  }
  var priorities = {
    date: 9,
    day: 11,
    weekday: 11,
    isoWeekday: 11,
    dayOfYear: 4,
    hour: 13,
    millisecond: 16,
    minute: 14,
    month: 8,
    quarter: 7,
    second: 15,
    weekYear: 1,
    isoWeekYear: 1,
    week: 5,
    isoWeek: 5,
    year: 1
  };
  function getPrioritizedUnits(O) {
    var _ = [], H;
    for (H in O)
      hasOwnProp(O, H) && _.push({ unit: H, priority: priorities[H] });
    return _.sort(function(V, q) {
      return V.priority - q.priority;
    }), _;
  }
  var match1 = /\d/, match2 = /\d\d/, match3 = /\d{3}/, match4 = /\d{4}/, match6 = /[+-]?\d{6}/, match1to2 = /\d\d?/, match3to4 = /\d\d\d\d?/, match5to6 = /\d\d\d\d\d\d?/, match1to3 = /\d{1,3}/, match1to4 = /\d{1,4}/, match1to6 = /[+-]?\d{1,6}/, matchUnsigned = /\d+/, matchSigned = /[+-]?\d+/, matchOffset = /Z|[+-]\d\d:?\d\d/gi, matchShortOffset = /Z|[+-]\d\d(?::?\d\d)?/gi, matchTimestamp = /[+-]?\d+(\.\d{1,3})?/, matchWord = /[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i, match1to2NoLeadingZero = /^[1-9]\d?/, match1to2HasZero = /^([1-9]\d|\d)/, regexes;
  regexes = {};
  function addRegexToken(O, _, H) {
    regexes[O] = isFunction$4(_) ? _ : function(V, q) {
      return V && H ? H : _;
    };
  }
  function getParseRegexForToken(O, _) {
    return hasOwnProp(regexes, O) ? regexes[O](_._strict, _._locale) : new RegExp(unescapeFormat(O));
  }
  function unescapeFormat(O) {
    return regexEscape(
      O.replace("\\", "").replace(
        /\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,
        function(_, H, V, q, G) {
          return H || V || q || G;
        }
      )
    );
  }
  function regexEscape(O) {
    return O.replace(/[-\/\\^$*+?.()|[\]{}]/g, "\\$&");
  }
  function absFloor(O) {
    return O < 0 ? Math.ceil(O) || 0 : Math.floor(O);
  }
  function toInt(O) {
    var _ = +O, H = 0;
    return _ !== 0 && isFinite(_) && (H = absFloor(_)), H;
  }
  var tokens$1 = {};
  function addParseToken(O, _) {
    var H, V = _, q;
    for (typeof O == "string" && (O = [O]), isNumber$4(_) && (V = function(G, J) {
      J[_] = toInt(G);
    }), q = O.length, H = 0; H < q; H++)
      tokens$1[O[H]] = V;
  }
  function addWeekParseToken(O, _) {
    addParseToken(O, function(H, V, q, G) {
      q._w = q._w || {}, _(H, q._w, q, G);
    });
  }
  function addTimeToArrayFromToken(O, _, H) {
    _ != null && hasOwnProp(tokens$1, O) && tokens$1[O](_, H._a, H, O);
  }
  function isLeapYear(O) {
    return O % 4 === 0 && O % 100 !== 0 || O % 400 === 0;
  }
  var YEAR$1 = 0, MONTH$1 = 1, DATE$1 = 2, HOUR = 3, MINUTE = 4, SECOND = 5, MILLISECOND = 6, WEEK$1 = 7, WEEKDAY = 8;
  addFormatToken("Y", 0, 0, function() {
    var O = this.year();
    return O <= 9999 ? zeroFill(O, 4) : "+" + O;
  });
  addFormatToken(0, ["YY", 2], 0, function() {
    return this.year() % 100;
  });
  addFormatToken(0, ["YYYY", 4], 0, "year");
  addFormatToken(0, ["YYYYY", 5], 0, "year");
  addFormatToken(0, ["YYYYYY", 6, !0], 0, "year");
  addRegexToken("Y", matchSigned);
  addRegexToken("YY", match1to2, match2);
  addRegexToken("YYYY", match1to4, match4);
  addRegexToken("YYYYY", match1to6, match6);
  addRegexToken("YYYYYY", match1to6, match6);
  addParseToken(["YYYYY", "YYYYYY"], YEAR$1);
  addParseToken("YYYY", function(O, _) {
    _[YEAR$1] = O.length === 2 ? hooks.parseTwoDigitYear(O) : toInt(O);
  });
  addParseToken("YY", function(O, _) {
    _[YEAR$1] = hooks.parseTwoDigitYear(O);
  });
  addParseToken("Y", function(O, _) {
    _[YEAR$1] = parseInt(O, 10);
  });
  function daysInYear(O) {
    return isLeapYear(O) ? 366 : 365;
  }
  hooks.parseTwoDigitYear = function(O) {
    return toInt(O) + (toInt(O) > 68 ? 1900 : 2e3);
  };
  var getSetYear = makeGetSet("FullYear", !0);
  function getIsLeapYear() {
    return isLeapYear(this.year());
  }
  function makeGetSet(O, _) {
    return function(H) {
      return H != null ? (set$1$1(this, O, H), hooks.updateOffset(this, _), this) : get$7(this, O);
    };
  }
  function get$7(O, _) {
    if (!O.isValid())
      return NaN;
    var H = O._d, V = O._isUTC;
    switch (_) {
      case "Milliseconds":
        return V ? H.getUTCMilliseconds() : H.getMilliseconds();
      case "Seconds":
        return V ? H.getUTCSeconds() : H.getSeconds();
      case "Minutes":
        return V ? H.getUTCMinutes() : H.getMinutes();
      case "Hours":
        return V ? H.getUTCHours() : H.getHours();
      case "Date":
        return V ? H.getUTCDate() : H.getDate();
      case "Day":
        return V ? H.getUTCDay() : H.getDay();
      case "Month":
        return V ? H.getUTCMonth() : H.getMonth();
      case "FullYear":
        return V ? H.getUTCFullYear() : H.getFullYear();
      default:
        return NaN;
    }
  }
  function set$1$1(O, _, H) {
    var V, q, G, J, Z;
    if (!(!O.isValid() || isNaN(H))) {
      switch (V = O._d, q = O._isUTC, _) {
        case "Milliseconds":
          return void (q ? V.setUTCMilliseconds(H) : V.setMilliseconds(H));
        case "Seconds":
          return void (q ? V.setUTCSeconds(H) : V.setSeconds(H));
        case "Minutes":
          return void (q ? V.setUTCMinutes(H) : V.setMinutes(H));
        case "Hours":
          return void (q ? V.setUTCHours(H) : V.setHours(H));
        case "Date":
          return void (q ? V.setUTCDate(H) : V.setDate(H));
        // case 'Day': // Not real
        //    return void (isUTC ? d.setUTCDay(value) : d.setDay(value));
        // case 'Month': // Not used because we need to pass two variables
        //     return void (isUTC ? d.setUTCMonth(value) : d.setMonth(value));
        case "FullYear":
          break;
        // See below ...
        default:
          return;
      }
      G = H, J = O.month(), Z = O.date(), Z = Z === 29 && J === 1 && !isLeapYear(G) ? 28 : Z, q ? V.setUTCFullYear(G, J, Z) : V.setFullYear(G, J, Z);
    }
  }
  function stringGet(O) {
    return O = normalizeUnits(O), isFunction$4(this[O]) ? this[O]() : this;
  }
  function stringSet(O, _) {
    if (typeof O == "object") {
      O = normalizeObjectUnits(O);
      var H = getPrioritizedUnits(O), V, q = H.length;
      for (V = 0; V < q; V++)
        this[H[V].unit](O[H[V].unit]);
    } else if (O = normalizeUnits(O), isFunction$4(this[O]))
      return this[O](_);
    return this;
  }
  function mod(O, _) {
    return (O % _ + _) % _;
  }
  var indexOf;
  Array.prototype.indexOf ? indexOf = Array.prototype.indexOf : indexOf = function(O) {
    var _;
    for (_ = 0; _ < this.length; ++_)
      if (this[_] === O)
        return _;
    return -1;
  };
  function daysInMonth(O, _) {
    if (isNaN(O) || isNaN(_))
      return NaN;
    var H = mod(_, 12);
    return O += (_ - H) / 12, H === 1 ? isLeapYear(O) ? 29 : 28 : 31 - H % 7 % 2;
  }
  addFormatToken("M", ["MM", 2], "Mo", function() {
    return this.month() + 1;
  });
  addFormatToken("MMM", 0, 0, function(O) {
    return this.localeData().monthsShort(this, O);
  });
  addFormatToken("MMMM", 0, 0, function(O) {
    return this.localeData().months(this, O);
  });
  addRegexToken("M", match1to2, match1to2NoLeadingZero);
  addRegexToken("MM", match1to2, match2);
  addRegexToken("MMM", function(O, _) {
    return _.monthsShortRegex(O);
  });
  addRegexToken("MMMM", function(O, _) {
    return _.monthsRegex(O);
  });
  addParseToken(["M", "MM"], function(O, _) {
    _[MONTH$1] = toInt(O) - 1;
  });
  addParseToken(["MMM", "MMMM"], function(O, _, H, V) {
    var q = H._locale.monthsParse(O, V, H._strict);
    q != null ? _[MONTH$1] = q : getParsingFlags(H).invalidMonth = O;
  });
  var defaultLocaleMonths = "January_February_March_April_May_June_July_August_September_October_November_December".split(
    "_"
  ), defaultLocaleMonthsShort = "Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"), MONTHS_IN_FORMAT = /D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/, defaultMonthsShortRegex = matchWord, defaultMonthsRegex = matchWord;
  function localeMonths(O, _) {
    return O ? isArray$4(this._months) ? this._months[O.month()] : this._months[(this._months.isFormat || MONTHS_IN_FORMAT).test(_) ? "format" : "standalone"][O.month()] : isArray$4(this._months) ? this._months : this._months.standalone;
  }
  function localeMonthsShort(O, _) {
    return O ? isArray$4(this._monthsShort) ? this._monthsShort[O.month()] : this._monthsShort[MONTHS_IN_FORMAT.test(_) ? "format" : "standalone"][O.month()] : isArray$4(this._monthsShort) ? this._monthsShort : this._monthsShort.standalone;
  }
  function handleStrictParse(O, _, H) {
    var V, q, G, J = O.toLocaleLowerCase();
    if (!this._monthsParse)
      for (this._monthsParse = [], this._longMonthsParse = [], this._shortMonthsParse = [], V = 0; V < 12; ++V)
        G = createUTC([2e3, V]), this._shortMonthsParse[V] = this.monthsShort(
          G,
          ""
        ).toLocaleLowerCase(), this._longMonthsParse[V] = this.months(G, "").toLocaleLowerCase();
    return H ? _ === "MMM" ? (q = indexOf.call(this._shortMonthsParse, J), q !== -1 ? q : null) : (q = indexOf.call(this._longMonthsParse, J), q !== -1 ? q : null) : _ === "MMM" ? (q = indexOf.call(this._shortMonthsParse, J), q !== -1 ? q : (q = indexOf.call(this._longMonthsParse, J), q !== -1 ? q : null)) : (q = indexOf.call(this._longMonthsParse, J), q !== -1 ? q : (q = indexOf.call(this._shortMonthsParse, J), q !== -1 ? q : null));
  }
  function localeMonthsParse(O, _, H) {
    var V, q, G;
    if (this._monthsParseExact)
      return handleStrictParse.call(this, O, _, H);
    for (this._monthsParse || (this._monthsParse = [], this._longMonthsParse = [], this._shortMonthsParse = []), V = 0; V < 12; V++) {
      if (q = createUTC([2e3, V]), H && !this._longMonthsParse[V] && (this._longMonthsParse[V] = new RegExp(
        "^" + this.months(q, "").replace(".", "") + "$",
        "i"
      ), this._shortMonthsParse[V] = new RegExp(
        "^" + this.monthsShort(q, "").replace(".", "") + "$",
        "i"
      )), !H && !this._monthsParse[V] && (G = "^" + this.months(q, "") + "|^" + this.monthsShort(q, ""), this._monthsParse[V] = new RegExp(G.replace(".", ""), "i")), H && _ === "MMMM" && this._longMonthsParse[V].test(O))
        return V;
      if (H && _ === "MMM" && this._shortMonthsParse[V].test(O))
        return V;
      if (!H && this._monthsParse[V].test(O))
        return V;
    }
  }
  function setMonth(O, _) {
    if (!O.isValid())
      return O;
    if (typeof _ == "string") {
      if (/^\d+$/.test(_))
        _ = toInt(_);
      else if (_ = O.localeData().monthsParse(_), !isNumber$4(_))
        return O;
    }
    var H = _, V = O.date();
    return V = V < 29 ? V : Math.min(V, daysInMonth(O.year(), H)), O._isUTC ? O._d.setUTCMonth(H, V) : O._d.setMonth(H, V), O;
  }
  function getSetMonth(O) {
    return O != null ? (setMonth(this, O), hooks.updateOffset(this, !0), this) : get$7(this, "Month");
  }
  function getDaysInMonth() {
    return daysInMonth(this.year(), this.month());
  }
  function monthsShortRegex(O) {
    return this._monthsParseExact ? (hasOwnProp(this, "_monthsRegex") || computeMonthsParse.call(this), O ? this._monthsShortStrictRegex : this._monthsShortRegex) : (hasOwnProp(this, "_monthsShortRegex") || (this._monthsShortRegex = defaultMonthsShortRegex), this._monthsShortStrictRegex && O ? this._monthsShortStrictRegex : this._monthsShortRegex);
  }
  function monthsRegex(O) {
    return this._monthsParseExact ? (hasOwnProp(this, "_monthsRegex") || computeMonthsParse.call(this), O ? this._monthsStrictRegex : this._monthsRegex) : (hasOwnProp(this, "_monthsRegex") || (this._monthsRegex = defaultMonthsRegex), this._monthsStrictRegex && O ? this._monthsStrictRegex : this._monthsRegex);
  }
  function computeMonthsParse() {
    function O(te, ne) {
      return ne.length - te.length;
    }
    var _ = [], H = [], V = [], q, G, J, Z;
    for (q = 0; q < 12; q++)
      G = createUTC([2e3, q]), J = regexEscape(this.monthsShort(G, "")), Z = regexEscape(this.months(G, "")), _.push(J), H.push(Z), V.push(Z), V.push(J);
    _.sort(O), H.sort(O), V.sort(O), this._monthsRegex = new RegExp("^(" + V.join("|") + ")", "i"), this._monthsShortRegex = this._monthsRegex, this._monthsStrictRegex = new RegExp(
      "^(" + H.join("|") + ")",
      "i"
    ), this._monthsShortStrictRegex = new RegExp(
      "^(" + _.join("|") + ")",
      "i"
    );
  }
  function createDate(O, _, H, V, q, G, J) {
    var Z;
    return O < 100 && O >= 0 ? (Z = new Date(O + 400, _, H, V, q, G, J), isFinite(Z.getFullYear()) && Z.setFullYear(O)) : Z = new Date(O, _, H, V, q, G, J), Z;
  }
  function createUTCDate(O) {
    var _, H;
    return O < 100 && O >= 0 ? (H = Array.prototype.slice.call(arguments), H[0] = O + 400, _ = new Date(Date.UTC.apply(null, H)), isFinite(_.getUTCFullYear()) && _.setUTCFullYear(O)) : _ = new Date(Date.UTC.apply(null, arguments)), _;
  }
  function firstWeekOffset(O, _, H) {
    var V = 7 + _ - H, q = (7 + createUTCDate(O, 0, V).getUTCDay() - _) % 7;
    return -q + V - 1;
  }
  function dayOfYearFromWeeks(O, _, H, V, q) {
    var G = (7 + H - V) % 7, J = firstWeekOffset(O, V, q), Z = 1 + 7 * (_ - 1) + G + J, te, ne;
    return Z <= 0 ? (te = O - 1, ne = daysInYear(te) + Z) : Z > daysInYear(O) ? (te = O + 1, ne = Z - daysInYear(O)) : (te = O, ne = Z), {
      year: te,
      dayOfYear: ne
    };
  }
  function weekOfYear(O, _, H) {
    var V = firstWeekOffset(O.year(), _, H), q = Math.floor((O.dayOfYear() - V - 1) / 7) + 1, G, J;
    return q < 1 ? (J = O.year() - 1, G = q + weeksInYear(J, _, H)) : q > weeksInYear(O.year(), _, H) ? (G = q - weeksInYear(O.year(), _, H), J = O.year() + 1) : (J = O.year(), G = q), {
      week: G,
      year: J
    };
  }
  function weeksInYear(O, _, H) {
    var V = firstWeekOffset(O, _, H), q = firstWeekOffset(O + 1, _, H);
    return (daysInYear(O) - V + q) / 7;
  }
  addFormatToken("w", ["ww", 2], "wo", "week");
  addFormatToken("W", ["WW", 2], "Wo", "isoWeek");
  addRegexToken("w", match1to2, match1to2NoLeadingZero);
  addRegexToken("ww", match1to2, match2);
  addRegexToken("W", match1to2, match1to2NoLeadingZero);
  addRegexToken("WW", match1to2, match2);
  addWeekParseToken(
    ["w", "ww", "W", "WW"],
    function(O, _, H, V) {
      _[V.substr(0, 1)] = toInt(O);
    }
  );
  function localeWeek(O) {
    return weekOfYear(O, this._week.dow, this._week.doy).week;
  }
  var defaultLocaleWeek = {
    dow: 0,
    // Sunday is the first day of the week.
    doy: 6
    // The week that contains Jan 6th is the first week of the year.
  };
  function localeFirstDayOfWeek() {
    return this._week.dow;
  }
  function localeFirstDayOfYear() {
    return this._week.doy;
  }
  function getSetWeek(O) {
    var _ = this.localeData().week(this);
    return O == null ? _ : this.add((O - _) * 7, "d");
  }
  function getSetISOWeek(O) {
    var _ = weekOfYear(this, 1, 4).week;
    return O == null ? _ : this.add((O - _) * 7, "d");
  }
  addFormatToken("d", 0, "do", "day");
  addFormatToken("dd", 0, 0, function(O) {
    return this.localeData().weekdaysMin(this, O);
  });
  addFormatToken("ddd", 0, 0, function(O) {
    return this.localeData().weekdaysShort(this, O);
  });
  addFormatToken("dddd", 0, 0, function(O) {
    return this.localeData().weekdays(this, O);
  });
  addFormatToken("e", 0, 0, "weekday");
  addFormatToken("E", 0, 0, "isoWeekday");
  addRegexToken("d", match1to2);
  addRegexToken("e", match1to2);
  addRegexToken("E", match1to2);
  addRegexToken("dd", function(O, _) {
    return _.weekdaysMinRegex(O);
  });
  addRegexToken("ddd", function(O, _) {
    return _.weekdaysShortRegex(O);
  });
  addRegexToken("dddd", function(O, _) {
    return _.weekdaysRegex(O);
  });
  addWeekParseToken(["dd", "ddd", "dddd"], function(O, _, H, V) {
    var q = H._locale.weekdaysParse(O, V, H._strict);
    q != null ? _.d = q : getParsingFlags(H).invalidWeekday = O;
  });
  addWeekParseToken(["d", "e", "E"], function(O, _, H, V) {
    _[V] = toInt(O);
  });
  function parseWeekday(O, _) {
    return typeof O != "string" ? O : isNaN(O) ? (O = _.weekdaysParse(O), typeof O == "number" ? O : null) : parseInt(O, 10);
  }
  function parseIsoWeekday(O, _) {
    return typeof O == "string" ? _.weekdaysParse(O) % 7 || 7 : isNaN(O) ? null : O;
  }
  function shiftWeekdays(O, _) {
    return O.slice(_, 7).concat(O.slice(0, _));
  }
  var defaultLocaleWeekdays = "Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"), defaultLocaleWeekdaysShort = "Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"), defaultLocaleWeekdaysMin = "Su_Mo_Tu_We_Th_Fr_Sa".split("_"), defaultWeekdaysRegex = matchWord, defaultWeekdaysShortRegex = matchWord, defaultWeekdaysMinRegex = matchWord;
  function localeWeekdays(O, _) {
    var H = isArray$4(this._weekdays) ? this._weekdays : this._weekdays[O && O !== !0 && this._weekdays.isFormat.test(_) ? "format" : "standalone"];
    return O === !0 ? shiftWeekdays(H, this._week.dow) : O ? H[O.day()] : H;
  }
  function localeWeekdaysShort(O) {
    return O === !0 ? shiftWeekdays(this._weekdaysShort, this._week.dow) : O ? this._weekdaysShort[O.day()] : this._weekdaysShort;
  }
  function localeWeekdaysMin(O) {
    return O === !0 ? shiftWeekdays(this._weekdaysMin, this._week.dow) : O ? this._weekdaysMin[O.day()] : this._weekdaysMin;
  }
  function handleStrictParse$1(O, _, H) {
    var V, q, G, J = O.toLocaleLowerCase();
    if (!this._weekdaysParse)
      for (this._weekdaysParse = [], this._shortWeekdaysParse = [], this._minWeekdaysParse = [], V = 0; V < 7; ++V)
        G = createUTC([2e3, 1]).day(V), this._minWeekdaysParse[V] = this.weekdaysMin(
          G,
          ""
        ).toLocaleLowerCase(), this._shortWeekdaysParse[V] = this.weekdaysShort(
          G,
          ""
        ).toLocaleLowerCase(), this._weekdaysParse[V] = this.weekdays(G, "").toLocaleLowerCase();
    return H ? _ === "dddd" ? (q = indexOf.call(this._weekdaysParse, J), q !== -1 ? q : null) : _ === "ddd" ? (q = indexOf.call(this._shortWeekdaysParse, J), q !== -1 ? q : null) : (q = indexOf.call(this._minWeekdaysParse, J), q !== -1 ? q : null) : _ === "dddd" ? (q = indexOf.call(this._weekdaysParse, J), q !== -1 || (q = indexOf.call(this._shortWeekdaysParse, J), q !== -1) ? q : (q = indexOf.call(this._minWeekdaysParse, J), q !== -1 ? q : null)) : _ === "ddd" ? (q = indexOf.call(this._shortWeekdaysParse, J), q !== -1 || (q = indexOf.call(this._weekdaysParse, J), q !== -1) ? q : (q = indexOf.call(this._minWeekdaysParse, J), q !== -1 ? q : null)) : (q = indexOf.call(this._minWeekdaysParse, J), q !== -1 || (q = indexOf.call(this._weekdaysParse, J), q !== -1) ? q : (q = indexOf.call(this._shortWeekdaysParse, J), q !== -1 ? q : null));
  }
  function localeWeekdaysParse(O, _, H) {
    var V, q, G;
    if (this._weekdaysParseExact)
      return handleStrictParse$1.call(this, O, _, H);
    for (this._weekdaysParse || (this._weekdaysParse = [], this._minWeekdaysParse = [], this._shortWeekdaysParse = [], this._fullWeekdaysParse = []), V = 0; V < 7; V++) {
      if (q = createUTC([2e3, 1]).day(V), H && !this._fullWeekdaysParse[V] && (this._fullWeekdaysParse[V] = new RegExp(
        "^" + this.weekdays(q, "").replace(".", "\\.?") + "$",
        "i"
      ), this._shortWeekdaysParse[V] = new RegExp(
        "^" + this.weekdaysShort(q, "").replace(".", "\\.?") + "$",
        "i"
      ), this._minWeekdaysParse[V] = new RegExp(
        "^" + this.weekdaysMin(q, "").replace(".", "\\.?") + "$",
        "i"
      )), this._weekdaysParse[V] || (G = "^" + this.weekdays(q, "") + "|^" + this.weekdaysShort(q, "") + "|^" + this.weekdaysMin(q, ""), this._weekdaysParse[V] = new RegExp(G.replace(".", ""), "i")), H && _ === "dddd" && this._fullWeekdaysParse[V].test(O))
        return V;
      if (H && _ === "ddd" && this._shortWeekdaysParse[V].test(O))
        return V;
      if (H && _ === "dd" && this._minWeekdaysParse[V].test(O))
        return V;
      if (!H && this._weekdaysParse[V].test(O))
        return V;
    }
  }
  function getSetDayOfWeek(O) {
    if (!this.isValid())
      return O != null ? this : NaN;
    var _ = get$7(this, "Day");
    return O != null ? (O = parseWeekday(O, this.localeData()), this.add(O - _, "d")) : _;
  }
  function getSetLocaleDayOfWeek(O) {
    if (!this.isValid())
      return O != null ? this : NaN;
    var _ = (this.day() + 7 - this.localeData()._week.dow) % 7;
    return O == null ? _ : this.add(O - _, "d");
  }
  function getSetISODayOfWeek(O) {
    if (!this.isValid())
      return O != null ? this : NaN;
    if (O != null) {
      var _ = parseIsoWeekday(O, this.localeData());
      return this.day(this.day() % 7 ? _ : _ - 7);
    } else
      return this.day() || 7;
  }
  function weekdaysRegex(O) {
    return this._weekdaysParseExact ? (hasOwnProp(this, "_weekdaysRegex") || computeWeekdaysParse.call(this), O ? this._weekdaysStrictRegex : this._weekdaysRegex) : (hasOwnProp(this, "_weekdaysRegex") || (this._weekdaysRegex = defaultWeekdaysRegex), this._weekdaysStrictRegex && O ? this._weekdaysStrictRegex : this._weekdaysRegex);
  }
  function weekdaysShortRegex(O) {
    return this._weekdaysParseExact ? (hasOwnProp(this, "_weekdaysRegex") || computeWeekdaysParse.call(this), O ? this._weekdaysShortStrictRegex : this._weekdaysShortRegex) : (hasOwnProp(this, "_weekdaysShortRegex") || (this._weekdaysShortRegex = defaultWeekdaysShortRegex), this._weekdaysShortStrictRegex && O ? this._weekdaysShortStrictRegex : this._weekdaysShortRegex);
  }
  function weekdaysMinRegex(O) {
    return this._weekdaysParseExact ? (hasOwnProp(this, "_weekdaysRegex") || computeWeekdaysParse.call(this), O ? this._weekdaysMinStrictRegex : this._weekdaysMinRegex) : (hasOwnProp(this, "_weekdaysMinRegex") || (this._weekdaysMinRegex = defaultWeekdaysMinRegex), this._weekdaysMinStrictRegex && O ? this._weekdaysMinStrictRegex : this._weekdaysMinRegex);
  }
  function computeWeekdaysParse() {
    function O(ee, ae) {
      return ae.length - ee.length;
    }
    var _ = [], H = [], V = [], q = [], G, J, Z, te, ne;
    for (G = 0; G < 7; G++)
      J = createUTC([2e3, 1]).day(G), Z = regexEscape(this.weekdaysMin(J, "")), te = regexEscape(this.weekdaysShort(J, "")), ne = regexEscape(this.weekdays(J, "")), _.push(Z), H.push(te), V.push(ne), q.push(Z), q.push(te), q.push(ne);
    _.sort(O), H.sort(O), V.sort(O), q.sort(O), this._weekdaysRegex = new RegExp("^(" + q.join("|") + ")", "i"), this._weekdaysShortRegex = this._weekdaysRegex, this._weekdaysMinRegex = this._weekdaysRegex, this._weekdaysStrictRegex = new RegExp(
      "^(" + V.join("|") + ")",
      "i"
    ), this._weekdaysShortStrictRegex = new RegExp(
      "^(" + H.join("|") + ")",
      "i"
    ), this._weekdaysMinStrictRegex = new RegExp(
      "^(" + _.join("|") + ")",
      "i"
    );
  }
  function hFormat() {
    return this.hours() % 12 || 12;
  }
  function kFormat() {
    return this.hours() || 24;
  }
  addFormatToken("H", ["HH", 2], 0, "hour");
  addFormatToken("h", ["hh", 2], 0, hFormat);
  addFormatToken("k", ["kk", 2], 0, kFormat);
  addFormatToken("hmm", 0, 0, function() {
    return "" + hFormat.apply(this) + zeroFill(this.minutes(), 2);
  });
  addFormatToken("hmmss", 0, 0, function() {
    return "" + hFormat.apply(this) + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  });
  addFormatToken("Hmm", 0, 0, function() {
    return "" + this.hours() + zeroFill(this.minutes(), 2);
  });
  addFormatToken("Hmmss", 0, 0, function() {
    return "" + this.hours() + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  });
  function meridiem(O, _) {
    addFormatToken(O, 0, 0, function() {
      return this.localeData().meridiem(
        this.hours(),
        this.minutes(),
        _
      );
    });
  }
  meridiem("a", !0);
  meridiem("A", !1);
  function matchMeridiem(O, _) {
    return _._meridiemParse;
  }
  addRegexToken("a", matchMeridiem);
  addRegexToken("A", matchMeridiem);
  addRegexToken("H", match1to2, match1to2HasZero);
  addRegexToken("h", match1to2, match1to2NoLeadingZero);
  addRegexToken("k", match1to2, match1to2NoLeadingZero);
  addRegexToken("HH", match1to2, match2);
  addRegexToken("hh", match1to2, match2);
  addRegexToken("kk", match1to2, match2);
  addRegexToken("hmm", match3to4);
  addRegexToken("hmmss", match5to6);
  addRegexToken("Hmm", match3to4);
  addRegexToken("Hmmss", match5to6);
  addParseToken(["H", "HH"], HOUR);
  addParseToken(["k", "kk"], function(O, _, H) {
    var V = toInt(O);
    _[HOUR] = V === 24 ? 0 : V;
  });
  addParseToken(["a", "A"], function(O, _, H) {
    H._isPm = H._locale.isPM(O), H._meridiem = O;
  });
  addParseToken(["h", "hh"], function(O, _, H) {
    _[HOUR] = toInt(O), getParsingFlags(H).bigHour = !0;
  });
  addParseToken("hmm", function(O, _, H) {
    var V = O.length - 2;
    _[HOUR] = toInt(O.substr(0, V)), _[MINUTE] = toInt(O.substr(V)), getParsingFlags(H).bigHour = !0;
  });
  addParseToken("hmmss", function(O, _, H) {
    var V = O.length - 4, q = O.length - 2;
    _[HOUR] = toInt(O.substr(0, V)), _[MINUTE] = toInt(O.substr(V, 2)), _[SECOND] = toInt(O.substr(q)), getParsingFlags(H).bigHour = !0;
  });
  addParseToken("Hmm", function(O, _, H) {
    var V = O.length - 2;
    _[HOUR] = toInt(O.substr(0, V)), _[MINUTE] = toInt(O.substr(V));
  });
  addParseToken("Hmmss", function(O, _, H) {
    var V = O.length - 4, q = O.length - 2;
    _[HOUR] = toInt(O.substr(0, V)), _[MINUTE] = toInt(O.substr(V, 2)), _[SECOND] = toInt(O.substr(q));
  });
  function localeIsPM(O) {
    return (O + "").toLowerCase().charAt(0) === "p";
  }
  var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i, getSetHour = makeGetSet("Hours", !0);
  function localeMeridiem(O, _, H) {
    return O > 11 ? H ? "pm" : "PM" : H ? "am" : "AM";
  }
  var baseConfig = {
    calendar: defaultCalendar,
    longDateFormat: defaultLongDateFormat,
    invalidDate: defaultInvalidDate,
    ordinal: defaultOrdinal,
    dayOfMonthOrdinalParse: defaultDayOfMonthOrdinalParse,
    relativeTime: defaultRelativeTime,
    months: defaultLocaleMonths,
    monthsShort: defaultLocaleMonthsShort,
    week: defaultLocaleWeek,
    weekdays: defaultLocaleWeekdays,
    weekdaysMin: defaultLocaleWeekdaysMin,
    weekdaysShort: defaultLocaleWeekdaysShort,
    meridiemParse: defaultLocaleMeridiemParse
  }, locales = {}, localeFamilies = {}, globalLocale;
  function commonPrefix(O, _) {
    var H, V = Math.min(O.length, _.length);
    for (H = 0; H < V; H += 1)
      if (O[H] !== _[H])
        return H;
    return V;
  }
  function normalizeLocale(O) {
    return O && O.toLowerCase().replace("_", "-");
  }
  function chooseLocale(O) {
    for (var _ = 0, H, V, q, G; _ < O.length; ) {
      for (G = normalizeLocale(O[_]).split("-"), H = G.length, V = normalizeLocale(O[_ + 1]), V = V ? V.split("-") : null; H > 0; ) {
        if (q = loadLocale(G.slice(0, H).join("-")), q)
          return q;
        if (V && V.length >= H && commonPrefix(G, V) >= H - 1)
          break;
        H--;
      }
      _++;
    }
    return globalLocale;
  }
  function isLocaleNameSane(O) {
    return !!(O && O.match("^[^/\\\\]*$"));
  }
  function loadLocale(O) {
    var _ = null, H;
    if (locales[O] === void 0 && typeof module < "u" && module && module.exports && isLocaleNameSane(O))
      try {
        _ = globalLocale._abbr, H = require, H("./locale/" + O), getSetGlobalLocale(_);
      } catch {
        locales[O] = null;
      }
    return locales[O];
  }
  function getSetGlobalLocale(O, _) {
    var H;
    return O && (isUndefined(_) ? H = getLocale(O) : H = defineLocale(O, _), H ? globalLocale = H : typeof console < "u" && console.warn && console.warn(
      "Locale " + O + " not found. Did you forget to load it?"
    )), globalLocale._abbr;
  }
  function defineLocale(O, _) {
    if (_ !== null) {
      var H, V = baseConfig;
      if (_.abbr = O, locales[O] != null)
        deprecateSimple(
          "defineLocaleOverride",
          "use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."
        ), V = locales[O]._config;
      else if (_.parentLocale != null)
        if (locales[_.parentLocale] != null)
          V = locales[_.parentLocale]._config;
        else if (H = loadLocale(_.parentLocale), H != null)
          V = H._config;
        else
          return localeFamilies[_.parentLocale] || (localeFamilies[_.parentLocale] = []), localeFamilies[_.parentLocale].push({
            name: O,
            config: _
          }), null;
      return locales[O] = new Locale(mergeConfigs(V, _)), localeFamilies[O] && localeFamilies[O].forEach(function(q) {
        defineLocale(q.name, q.config);
      }), getSetGlobalLocale(O), locales[O];
    } else
      return delete locales[O], null;
  }
  function updateLocale(O, _) {
    if (_ != null) {
      var H, V, q = baseConfig;
      locales[O] != null && locales[O].parentLocale != null ? locales[O].set(mergeConfigs(locales[O]._config, _)) : (V = loadLocale(O), V != null && (q = V._config), _ = mergeConfigs(q, _), V == null && (_.abbr = O), H = new Locale(_), H.parentLocale = locales[O], locales[O] = H), getSetGlobalLocale(O);
    } else
      locales[O] != null && (locales[O].parentLocale != null ? (locales[O] = locales[O].parentLocale, O === getSetGlobalLocale() && getSetGlobalLocale(O)) : locales[O] != null && delete locales[O]);
    return locales[O];
  }
  function getLocale(O) {
    var _;
    if (O && O._locale && O._locale._abbr && (O = O._locale._abbr), !O)
      return globalLocale;
    if (!isArray$4(O)) {
      if (_ = loadLocale(O), _)
        return _;
      O = [O];
    }
    return chooseLocale(O);
  }
  function listLocales() {
    return keys$6(locales);
  }
  function checkOverflow(O) {
    var _, H = O._a;
    return H && getParsingFlags(O).overflow === -2 && (_ = H[MONTH$1] < 0 || H[MONTH$1] > 11 ? MONTH$1 : H[DATE$1] < 1 || H[DATE$1] > daysInMonth(H[YEAR$1], H[MONTH$1]) ? DATE$1 : H[HOUR] < 0 || H[HOUR] > 24 || H[HOUR] === 24 && (H[MINUTE] !== 0 || H[SECOND] !== 0 || H[MILLISECOND] !== 0) ? HOUR : H[MINUTE] < 0 || H[MINUTE] > 59 ? MINUTE : H[SECOND] < 0 || H[SECOND] > 59 ? SECOND : H[MILLISECOND] < 0 || H[MILLISECOND] > 999 ? MILLISECOND : -1, getParsingFlags(O)._overflowDayOfYear && (_ < YEAR$1 || _ > DATE$1) && (_ = DATE$1), getParsingFlags(O)._overflowWeeks && _ === -1 && (_ = WEEK$1), getParsingFlags(O)._overflowWeekday && _ === -1 && (_ = WEEKDAY), getParsingFlags(O).overflow = _), O;
  }
  var extendedIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/, basicIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/, tzRegex = /Z|[+-]\d\d(?::?\d\d)?/, isoDates = [
    ["YYYYYY-MM-DD", /[+-]\d{6}-\d\d-\d\d/],
    ["YYYY-MM-DD", /\d{4}-\d\d-\d\d/],
    ["GGGG-[W]WW-E", /\d{4}-W\d\d-\d/],
    ["GGGG-[W]WW", /\d{4}-W\d\d/, !1],
    ["YYYY-DDD", /\d{4}-\d{3}/],
    ["YYYY-MM", /\d{4}-\d\d/, !1],
    ["YYYYYYMMDD", /[+-]\d{10}/],
    ["YYYYMMDD", /\d{8}/],
    ["GGGG[W]WWE", /\d{4}W\d{3}/],
    ["GGGG[W]WW", /\d{4}W\d{2}/, !1],
    ["YYYYDDD", /\d{7}/],
    ["YYYYMM", /\d{6}/, !1],
    ["YYYY", /\d{4}/, !1]
  ], isoTimes = [
    ["HH:mm:ss.SSSS", /\d\d:\d\d:\d\d\.\d+/],
    ["HH:mm:ss,SSSS", /\d\d:\d\d:\d\d,\d+/],
    ["HH:mm:ss", /\d\d:\d\d:\d\d/],
    ["HH:mm", /\d\d:\d\d/],
    ["HHmmss.SSSS", /\d\d\d\d\d\d\.\d+/],
    ["HHmmss,SSSS", /\d\d\d\d\d\d,\d+/],
    ["HHmmss", /\d\d\d\d\d\d/],
    ["HHmm", /\d\d\d\d/],
    ["HH", /\d\d/]
  ], aspNetJsonRegex = /^\/?Date\((-?\d+)/i, rfc2822 = /^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/, obsOffsets = {
    UT: 0,
    GMT: 0,
    EDT: -4 * 60,
    EST: -5 * 60,
    CDT: -5 * 60,
    CST: -6 * 60,
    MDT: -6 * 60,
    MST: -7 * 60,
    PDT: -7 * 60,
    PST: -8 * 60
  };
  function configFromISO(O) {
    var _, H, V = O._i, q = extendedIsoRegex.exec(V) || basicIsoRegex.exec(V), G, J, Z, te, ne = isoDates.length, ee = isoTimes.length;
    if (q) {
      for (getParsingFlags(O).iso = !0, _ = 0, H = ne; _ < H; _++)
        if (isoDates[_][1].exec(q[1])) {
          J = isoDates[_][0], G = isoDates[_][2] !== !1;
          break;
        }
      if (J == null) {
        O._isValid = !1;
        return;
      }
      if (q[3]) {
        for (_ = 0, H = ee; _ < H; _++)
          if (isoTimes[_][1].exec(q[3])) {
            Z = (q[2] || " ") + isoTimes[_][0];
            break;
          }
        if (Z == null) {
          O._isValid = !1;
          return;
        }
      }
      if (!G && Z != null) {
        O._isValid = !1;
        return;
      }
      if (q[4])
        if (tzRegex.exec(q[4]))
          te = "Z";
        else {
          O._isValid = !1;
          return;
        }
      O._f = J + (Z || "") + (te || ""), configFromStringAndFormat(O);
    } else
      O._isValid = !1;
  }
  function extractFromRFC2822Strings(O, _, H, V, q, G) {
    var J = [
      untruncateYear(O),
      defaultLocaleMonthsShort.indexOf(_),
      parseInt(H, 10),
      parseInt(V, 10),
      parseInt(q, 10)
    ];
    return G && J.push(parseInt(G, 10)), J;
  }
  function untruncateYear(O) {
    var _ = parseInt(O, 10);
    return _ <= 49 ? 2e3 + _ : _ <= 999 ? 1900 + _ : _;
  }
  function preprocessRFC2822(O) {
    return O.replace(/\([^()]*\)|[\n\t]/g, " ").replace(/(\s\s+)/g, " ").replace(/^\s\s*/, "").replace(/\s\s*$/, "");
  }
  function checkWeekday(O, _, H) {
    if (O) {
      var V = defaultLocaleWeekdaysShort.indexOf(O), q = new Date(
        _[0],
        _[1],
        _[2]
      ).getDay();
      if (V !== q)
        return getParsingFlags(H).weekdayMismatch = !0, H._isValid = !1, !1;
    }
    return !0;
  }
  function calculateOffset(O, _, H) {
    if (O)
      return obsOffsets[O];
    if (_)
      return 0;
    var V = parseInt(H, 10), q = V % 100, G = (V - q) / 100;
    return G * 60 + q;
  }
  function configFromRFC2822(O) {
    var _ = rfc2822.exec(preprocessRFC2822(O._i)), H;
    if (_) {
      if (H = extractFromRFC2822Strings(
        _[4],
        _[3],
        _[2],
        _[5],
        _[6],
        _[7]
      ), !checkWeekday(_[1], H, O))
        return;
      O._a = H, O._tzm = calculateOffset(_[8], _[9], _[10]), O._d = createUTCDate.apply(null, O._a), O._d.setUTCMinutes(O._d.getUTCMinutes() - O._tzm), getParsingFlags(O).rfc2822 = !0;
    } else
      O._isValid = !1;
  }
  function configFromString(O) {
    var _ = aspNetJsonRegex.exec(O._i);
    if (_ !== null) {
      O._d = /* @__PURE__ */ new Date(+_[1]);
      return;
    }
    if (configFromISO(O), O._isValid === !1)
      delete O._isValid;
    else
      return;
    if (configFromRFC2822(O), O._isValid === !1)
      delete O._isValid;
    else
      return;
    O._strict ? O._isValid = !1 : hooks.createFromInputFallback(O);
  }
  hooks.createFromInputFallback = deprecate(
    "value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",
    function(O) {
      O._d = /* @__PURE__ */ new Date(O._i + (O._useUTC ? " UTC" : ""));
    }
  );
  function defaults$6(O, _, H) {
    return O ?? _ ?? H;
  }
  function currentDateArray(O) {
    var _ = new Date(hooks.now());
    return O._useUTC ? [
      _.getUTCFullYear(),
      _.getUTCMonth(),
      _.getUTCDate()
    ] : [_.getFullYear(), _.getMonth(), _.getDate()];
  }
  function configFromArray(O) {
    var _, H, V = [], q, G, J;
    if (!O._d) {
      for (q = currentDateArray(O), O._w && O._a[DATE$1] == null && O._a[MONTH$1] == null && dayOfYearFromWeekInfo(O), O._dayOfYear != null && (J = defaults$6(O._a[YEAR$1], q[YEAR$1]), (O._dayOfYear > daysInYear(J) || O._dayOfYear === 0) && (getParsingFlags(O)._overflowDayOfYear = !0), H = createUTCDate(J, 0, O._dayOfYear), O._a[MONTH$1] = H.getUTCMonth(), O._a[DATE$1] = H.getUTCDate()), _ = 0; _ < 3 && O._a[_] == null; ++_)
        O._a[_] = V[_] = q[_];
      for (; _ < 7; _++)
        O._a[_] = V[_] = O._a[_] == null ? _ === 2 ? 1 : 0 : O._a[_];
      O._a[HOUR] === 24 && O._a[MINUTE] === 0 && O._a[SECOND] === 0 && O._a[MILLISECOND] === 0 && (O._nextDay = !0, O._a[HOUR] = 0), O._d = (O._useUTC ? createUTCDate : createDate).apply(
        null,
        V
      ), G = O._useUTC ? O._d.getUTCDay() : O._d.getDay(), O._tzm != null && O._d.setUTCMinutes(O._d.getUTCMinutes() - O._tzm), O._nextDay && (O._a[HOUR] = 24), O._w && typeof O._w.d < "u" && O._w.d !== G && (getParsingFlags(O).weekdayMismatch = !0);
    }
  }
  function dayOfYearFromWeekInfo(O) {
    var _, H, V, q, G, J, Z, te, ne;
    _ = O._w, _.GG != null || _.W != null || _.E != null ? (G = 1, J = 4, H = defaults$6(
      _.GG,
      O._a[YEAR$1],
      weekOfYear(createLocal(), 1, 4).year
    ), V = defaults$6(_.W, 1), q = defaults$6(_.E, 1), (q < 1 || q > 7) && (te = !0)) : (G = O._locale._week.dow, J = O._locale._week.doy, ne = weekOfYear(createLocal(), G, J), H = defaults$6(_.gg, O._a[YEAR$1], ne.year), V = defaults$6(_.w, ne.week), _.d != null ? (q = _.d, (q < 0 || q > 6) && (te = !0)) : _.e != null ? (q = _.e + G, (_.e < 0 || _.e > 6) && (te = !0)) : q = G), V < 1 || V > weeksInYear(H, G, J) ? getParsingFlags(O)._overflowWeeks = !0 : te != null ? getParsingFlags(O)._overflowWeekday = !0 : (Z = dayOfYearFromWeeks(H, V, q, G, J), O._a[YEAR$1] = Z.year, O._dayOfYear = Z.dayOfYear);
  }
  hooks.ISO_8601 = function() {
  };
  hooks.RFC_2822 = function() {
  };
  function configFromStringAndFormat(O) {
    if (O._f === hooks.ISO_8601) {
      configFromISO(O);
      return;
    }
    if (O._f === hooks.RFC_2822) {
      configFromRFC2822(O);
      return;
    }
    O._a = [], getParsingFlags(O).empty = !0;
    var _ = "" + O._i, H, V, q, G, J, Z = _.length, te = 0, ne, ee;
    for (q = expandFormat(O._f, O._locale).match(formattingTokens) || [], ee = q.length, H = 0; H < ee; H++)
      G = q[H], V = (_.match(getParseRegexForToken(G, O)) || [])[0], V && (J = _.substr(0, _.indexOf(V)), J.length > 0 && getParsingFlags(O).unusedInput.push(J), _ = _.slice(
        _.indexOf(V) + V.length
      ), te += V.length), formatTokenFunctions[G] ? (V ? getParsingFlags(O).empty = !1 : getParsingFlags(O).unusedTokens.push(G), addTimeToArrayFromToken(G, V, O)) : O._strict && !V && getParsingFlags(O).unusedTokens.push(G);
    getParsingFlags(O).charsLeftOver = Z - te, _.length > 0 && getParsingFlags(O).unusedInput.push(_), O._a[HOUR] <= 12 && getParsingFlags(O).bigHour === !0 && O._a[HOUR] > 0 && (getParsingFlags(O).bigHour = void 0), getParsingFlags(O).parsedDateParts = O._a.slice(0), getParsingFlags(O).meridiem = O._meridiem, O._a[HOUR] = meridiemFixWrap(
      O._locale,
      O._a[HOUR],
      O._meridiem
    ), ne = getParsingFlags(O).era, ne !== null && (O._a[YEAR$1] = O._locale.erasConvertYear(ne, O._a[YEAR$1])), configFromArray(O), checkOverflow(O);
  }
  function meridiemFixWrap(O, _, H) {
    var V;
    return H == null ? _ : O.meridiemHour != null ? O.meridiemHour(_, H) : (O.isPM != null && (V = O.isPM(H), V && _ < 12 && (_ += 12), !V && _ === 12 && (_ = 0)), _);
  }
  function configFromStringAndArray(O) {
    var _, H, V, q, G, J, Z = !1, te = O._f.length;
    if (te === 0) {
      getParsingFlags(O).invalidFormat = !0, O._d = /* @__PURE__ */ new Date(NaN);
      return;
    }
    for (q = 0; q < te; q++)
      G = 0, J = !1, _ = copyConfig({}, O), O._useUTC != null && (_._useUTC = O._useUTC), _._f = O._f[q], configFromStringAndFormat(_), isValid$1(_) && (J = !0), G += getParsingFlags(_).charsLeftOver, G += getParsingFlags(_).unusedTokens.length * 10, getParsingFlags(_).score = G, Z ? G < V && (V = G, H = _) : (V == null || G < V || J) && (V = G, H = _, J && (Z = !0));
    extend$6(O, H || _);
  }
  function configFromObject(O) {
    if (!O._d) {
      var _ = normalizeObjectUnits(O._i), H = _.day === void 0 ? _.date : _.day;
      O._a = map$5(
        [_.year, _.month, H, _.hour, _.minute, _.second, _.millisecond],
        function(V) {
          return V && parseInt(V, 10);
        }
      ), configFromArray(O);
    }
  }
  function createFromConfig(O) {
    var _ = new Moment(checkOverflow(prepareConfig(O)));
    return _._nextDay && (_.add(1, "d"), _._nextDay = void 0), _;
  }
  function prepareConfig(O) {
    var _ = O._i, H = O._f;
    return O._locale = O._locale || getLocale(O._l), _ === null || H === void 0 && _ === "" ? createInvalid({ nullInput: !0 }) : (typeof _ == "string" && (O._i = _ = O._locale.preparse(_)), isMoment(_) ? new Moment(checkOverflow(_)) : (isDate$2(_) ? O._d = _ : isArray$4(H) ? configFromStringAndArray(O) : H ? configFromStringAndFormat(O) : configFromInput(O), isValid$1(O) || (O._d = null), O));
  }
  function configFromInput(O) {
    var _ = O._i;
    isUndefined(_) ? O._d = new Date(hooks.now()) : isDate$2(_) ? O._d = new Date(_.valueOf()) : typeof _ == "string" ? configFromString(O) : isArray$4(_) ? (O._a = map$5(_.slice(0), function(H) {
      return parseInt(H, 10);
    }), configFromArray(O)) : isObject$4(_) ? configFromObject(O) : isNumber$4(_) ? O._d = new Date(_) : hooks.createFromInputFallback(O);
  }
  function createLocalOrUTC(O, _, H, V, q) {
    var G = {};
    return (_ === !0 || _ === !1) && (V = _, _ = void 0), (H === !0 || H === !1) && (V = H, H = void 0), (isObject$4(O) && isObjectEmpty(O) || isArray$4(O) && O.length === 0) && (O = void 0), G._isAMomentObject = !0, G._useUTC = G._isUTC = q, G._l = H, G._i = O, G._f = _, G._strict = V, createFromConfig(G);
  }
  function createLocal(O, _, H, V) {
    return createLocalOrUTC(O, _, H, V, !1);
  }
  var prototypeMin = deprecate(
    "moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",
    function() {
      var O = createLocal.apply(null, arguments);
      return this.isValid() && O.isValid() ? O < this ? this : O : createInvalid();
    }
  ), prototypeMax = deprecate(
    "moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",
    function() {
      var O = createLocal.apply(null, arguments);
      return this.isValid() && O.isValid() ? O > this ? this : O : createInvalid();
    }
  );
  function pickBy(O, _) {
    var H, V;
    if (_.length === 1 && isArray$4(_[0]) && (_ = _[0]), !_.length)
      return createLocal();
    for (H = _[0], V = 1; V < _.length; ++V)
      (!_[V].isValid() || _[V][O](H)) && (H = _[V]);
    return H;
  }
  function min$h() {
    var O = [].slice.call(arguments, 0);
    return pickBy("isBefore", O);
  }
  function max$p() {
    var O = [].slice.call(arguments, 0);
    return pickBy("isAfter", O);
  }
  var now$3 = function() {
    return Date.now ? Date.now() : +/* @__PURE__ */ new Date();
  }, ordering = [
    "year",
    "quarter",
    "month",
    "week",
    "day",
    "hour",
    "minute",
    "second",
    "millisecond"
  ];
  function isDurationValid(O) {
    var _, H = !1, V, q = ordering.length;
    for (_ in O)
      if (hasOwnProp(O, _) && !(indexOf.call(ordering, _) !== -1 && (O[_] == null || !isNaN(O[_]))))
        return !1;
    for (V = 0; V < q; ++V)
      if (O[ordering[V]]) {
        if (H)
          return !1;
        parseFloat(O[ordering[V]]) !== toInt(O[ordering[V]]) && (H = !0);
      }
    return !0;
  }
  function isValid$1$1() {
    return this._isValid;
  }
  function createInvalid$1() {
    return createDuration(NaN);
  }
  function Duration$2(O) {
    var _ = normalizeObjectUnits(O), H = _.year || 0, V = _.quarter || 0, q = _.month || 0, G = _.week || _.isoWeek || 0, J = _.day || 0, Z = _.hour || 0, te = _.minute || 0, ne = _.second || 0, ee = _.millisecond || 0;
    this._isValid = isDurationValid(_), this._milliseconds = +ee + ne * 1e3 + // 1000
    te * 6e4 + // 1000 * 60
    Z * 1e3 * 60 * 60, this._days = +J + G * 7, this._months = +q + V * 3 + H * 12, this._data = {}, this._locale = getLocale(), this._bubble();
  }
  function isDuration(O) {
    return O instanceof Duration$2;
  }
  function absRound(O) {
    return O < 0 ? Math.round(-1 * O) * -1 : Math.round(O);
  }
  function compareArrays(O, _, H) {
    var V = Math.min(O.length, _.length), q = Math.abs(O.length - _.length), G = 0, J;
    for (J = 0; J < V; J++)
      toInt(O[J]) !== toInt(_[J]) && G++;
    return G + q;
  }
  function offset$5(O, _) {
    addFormatToken(O, 0, 0, function() {
      var H = this.utcOffset(), V = "+";
      return H < 0 && (H = -H, V = "-"), V + zeroFill(~~(H / 60), 2) + _ + zeroFill(~~H % 60, 2);
    });
  }
  offset$5("Z", ":");
  offset$5("ZZ", "");
  addRegexToken("Z", matchShortOffset);
  addRegexToken("ZZ", matchShortOffset);
  addParseToken(["Z", "ZZ"], function(O, _, H) {
    H._useUTC = !0, H._tzm = offsetFromString(matchShortOffset, O);
  });
  var chunkOffset = /([\+\-]|\d\d)/gi;
  function offsetFromString(O, _) {
    var H = (_ || "").match(O), V, q, G;
    return H === null ? null : (V = H[H.length - 1] || [], q = (V + "").match(chunkOffset) || ["-", 0, 0], G = +(q[1] * 60) + toInt(q[2]), G === 0 ? 0 : q[0] === "+" ? G : -G);
  }
  function cloneWithOffset(O, _) {
    var H, V;
    return _._isUTC ? (H = _.clone(), V = (isMoment(O) || isDate$2(O) ? O.valueOf() : createLocal(O).valueOf()) - H.valueOf(), H._d.setTime(H._d.valueOf() + V), hooks.updateOffset(H, !1), H) : createLocal(O).local();
  }
  function getDateOffset(O) {
    return -Math.round(O._d.getTimezoneOffset());
  }
  hooks.updateOffset = function() {
  };
  function getSetOffset(O, _, H) {
    var V = this._offset || 0, q;
    if (!this.isValid())
      return O != null ? this : NaN;
    if (O != null) {
      if (typeof O == "string") {
        if (O = offsetFromString(matchShortOffset, O), O === null)
          return this;
      } else Math.abs(O) < 16 && !H && (O = O * 60);
      return !this._isUTC && _ && (q = getDateOffset(this)), this._offset = O, this._isUTC = !0, q != null && this.add(q, "m"), V !== O && (!_ || this._changeInProgress ? addSubtract(
        this,
        createDuration(O - V, "m"),
        1,
        !1
      ) : this._changeInProgress || (this._changeInProgress = !0, hooks.updateOffset(this, !0), this._changeInProgress = null)), this;
    } else
      return this._isUTC ? V : getDateOffset(this);
  }
  function getSetZone(O, _) {
    return O != null ? (typeof O != "string" && (O = -O), this.utcOffset(O, _), this) : -this.utcOffset();
  }
  function setOffsetToUTC(O) {
    return this.utcOffset(0, O);
  }
  function setOffsetToLocal(O) {
    return this._isUTC && (this.utcOffset(0, O), this._isUTC = !1, O && this.subtract(getDateOffset(this), "m")), this;
  }
  function setOffsetToParsedOffset() {
    if (this._tzm != null)
      this.utcOffset(this._tzm, !1, !0);
    else if (typeof this._i == "string") {
      var O = offsetFromString(matchOffset, this._i);
      O != null ? this.utcOffset(O) : this.utcOffset(0, !0);
    }
    return this;
  }
  function hasAlignedHourOffset(O) {
    return this.isValid() ? (O = O ? createLocal(O).utcOffset() : 0, (this.utcOffset() - O) % 60 === 0) : !1;
  }
  function isDaylightSavingTime() {
    return this.utcOffset() > this.clone().month(0).utcOffset() || this.utcOffset() > this.clone().month(5).utcOffset();
  }
  function isDaylightSavingTimeShifted() {
    if (!isUndefined(this._isDSTShifted))
      return this._isDSTShifted;
    var O = {}, _;
    return copyConfig(O, this), O = prepareConfig(O), O._a ? (_ = O._isUTC ? createUTC(O._a) : createLocal(O._a), this._isDSTShifted = this.isValid() && compareArrays(O._a, _.toArray()) > 0) : this._isDSTShifted = !1, this._isDSTShifted;
  }
  function isLocal() {
    return this.isValid() ? !this._isUTC : !1;
  }
  function isUtcOffset() {
    return this.isValid() ? this._isUTC : !1;
  }
  function isUtc() {
    return this.isValid() ? this._isUTC && this._offset === 0 : !1;
  }
  var aspNetRegex = /^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/, isoRegex = /^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;
  function createDuration(O, _) {
    var H = O, V = null, q, G, J;
    return isDuration(O) ? H = {
      ms: O._milliseconds,
      d: O._days,
      M: O._months
    } : isNumber$4(O) || !isNaN(+O) ? (H = {}, _ ? H[_] = +O : H.milliseconds = +O) : (V = aspNetRegex.exec(O)) ? (q = V[1] === "-" ? -1 : 1, H = {
      y: 0,
      d: toInt(V[DATE$1]) * q,
      h: toInt(V[HOUR]) * q,
      m: toInt(V[MINUTE]) * q,
      s: toInt(V[SECOND]) * q,
      ms: toInt(absRound(V[MILLISECOND] * 1e3)) * q
      // the millisecond decimal point is included in the match
    }) : (V = isoRegex.exec(O)) ? (q = V[1] === "-" ? -1 : 1, H = {
      y: parseIso(V[2], q),
      M: parseIso(V[3], q),
      w: parseIso(V[4], q),
      d: parseIso(V[5], q),
      h: parseIso(V[6], q),
      m: parseIso(V[7], q),
      s: parseIso(V[8], q)
    }) : H == null ? H = {} : typeof H == "object" && ("from" in H || "to" in H) && (J = momentsDifference(
      createLocal(H.from),
      createLocal(H.to)
    ), H = {}, H.ms = J.milliseconds, H.M = J.months), G = new Duration$2(H), isDuration(O) && hasOwnProp(O, "_locale") && (G._locale = O._locale), isDuration(O) && hasOwnProp(O, "_isValid") && (G._isValid = O._isValid), G;
  }
  createDuration.fn = Duration$2.prototype;
  createDuration.invalid = createInvalid$1;
  function parseIso(O, _) {
    var H = O && parseFloat(O.replace(",", "."));
    return (isNaN(H) ? 0 : H) * _;
  }
  function positiveMomentsDifference(O, _) {
    var H = {};
    return H.months = _.month() - O.month() + (_.year() - O.year()) * 12, O.clone().add(H.months, "M").isAfter(_) && --H.months, H.milliseconds = +_ - +O.clone().add(H.months, "M"), H;
  }
  function momentsDifference(O, _) {
    var H;
    return O.isValid() && _.isValid() ? (_ = cloneWithOffset(_, O), O.isBefore(_) ? H = positiveMomentsDifference(O, _) : (H = positiveMomentsDifference(_, O), H.milliseconds = -H.milliseconds, H.months = -H.months), H) : { milliseconds: 0, months: 0 };
  }
  function createAdder(O, _) {
    return function(H, V) {
      var q, G;
      return V !== null && !isNaN(+V) && (deprecateSimple(
        _,
        "moment()." + _ + "(period, number) is deprecated. Please use moment()." + _ + "(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."
      ), G = H, H = V, V = G), q = createDuration(H, V), addSubtract(this, q, O), this;
    };
  }
  function addSubtract(O, _, H, V) {
    var q = _._milliseconds, G = absRound(_._days), J = absRound(_._months);
    O.isValid() && (V = V ?? !0, J && setMonth(O, get$7(O, "Month") + J * H), G && set$1$1(O, "Date", get$7(O, "Date") + G * H), q && O._d.setTime(O._d.valueOf() + q * H), V && hooks.updateOffset(O, G || J));
  }
  var add$7 = createAdder(1, "add"), subtract = createAdder(-1, "subtract");
  function isString$3(O) {
    return typeof O == "string" || O instanceof String;
  }
  function isMomentInput(O) {
    return isMoment(O) || isDate$2(O) || isString$3(O) || isNumber$4(O) || isNumberOrStringArray(O) || isMomentInputObject(O) || O === null || O === void 0;
  }
  function isMomentInputObject(O) {
    var _ = isObject$4(O) && !isObjectEmpty(O), H = !1, V = [
      "years",
      "year",
      "y",
      "months",
      "month",
      "M",
      "days",
      "day",
      "d",
      "dates",
      "date",
      "D",
      "hours",
      "hour",
      "h",
      "minutes",
      "minute",
      "m",
      "seconds",
      "second",
      "s",
      "milliseconds",
      "millisecond",
      "ms"
    ], q, G, J = V.length;
    for (q = 0; q < J; q += 1)
      G = V[q], H = H || hasOwnProp(O, G);
    return _ && H;
  }
  function isNumberOrStringArray(O) {
    var _ = isArray$4(O), H = !1;
    return _ && (H = O.filter(function(V) {
      return !isNumber$4(V) && isString$3(O);
    }).length === 0), _ && H;
  }
  function isCalendarSpec(O) {
    var _ = isObject$4(O) && !isObjectEmpty(O), H = !1, V = [
      "sameDay",
      "nextDay",
      "lastDay",
      "nextWeek",
      "lastWeek",
      "sameElse"
    ], q, G;
    for (q = 0; q < V.length; q += 1)
      G = V[q], H = H || hasOwnProp(O, G);
    return _ && H;
  }
  function getCalendarFormat(O, _) {
    var H = O.diff(_, "days", !0);
    return H < -6 ? "sameElse" : H < -1 ? "lastWeek" : H < 0 ? "lastDay" : H < 1 ? "sameDay" : H < 2 ? "nextDay" : H < 7 ? "nextWeek" : "sameElse";
  }
  function calendar$1$1(O, _) {
    arguments.length === 1 && (arguments[0] ? isMomentInput(arguments[0]) ? (O = arguments[0], _ = void 0) : isCalendarSpec(arguments[0]) && (_ = arguments[0], O = void 0) : (O = void 0, _ = void 0));
    var H = O || createLocal(), V = cloneWithOffset(H, this).startOf("day"), q = hooks.calendarFormat(this, V) || "sameElse", G = _ && (isFunction$4(_[q]) ? _[q].call(this, H) : _[q]);
    return this.format(
      G || this.localeData().calendar(q, this, createLocal(H))
    );
  }
  function clone$2() {
    return new Moment(this);
  }
  function isAfter(O, _) {
    var H = isMoment(O) ? O : createLocal(O);
    return this.isValid() && H.isValid() ? (_ = normalizeUnits(_) || "millisecond", _ === "millisecond" ? this.valueOf() > H.valueOf() : H.valueOf() < this.clone().startOf(_).valueOf()) : !1;
  }
  function isBefore(O, _) {
    var H = isMoment(O) ? O : createLocal(O);
    return this.isValid() && H.isValid() ? (_ = normalizeUnits(_) || "millisecond", _ === "millisecond" ? this.valueOf() < H.valueOf() : this.clone().endOf(_).valueOf() < H.valueOf()) : !1;
  }
  function isBetween$1(O, _, H, V) {
    var q = isMoment(O) ? O : createLocal(O), G = isMoment(_) ? _ : createLocal(_);
    return this.isValid() && q.isValid() && G.isValid() ? (V = V || "()", (V[0] === "(" ? this.isAfter(q, H) : !this.isBefore(q, H)) && (V[1] === ")" ? this.isBefore(G, H) : !this.isAfter(G, H))) : !1;
  }
  function isSame(O, _) {
    var H = isMoment(O) ? O : createLocal(O), V;
    return this.isValid() && H.isValid() ? (_ = normalizeUnits(_) || "millisecond", _ === "millisecond" ? this.valueOf() === H.valueOf() : (V = H.valueOf(), this.clone().startOf(_).valueOf() <= V && V <= this.clone().endOf(_).valueOf())) : !1;
  }
  function isSameOrAfter(O, _) {
    return this.isSame(O, _) || this.isAfter(O, _);
  }
  function isSameOrBefore(O, _) {
    return this.isSame(O, _) || this.isBefore(O, _);
  }
  function diff(O, _, H) {
    var V, q, G;
    if (!this.isValid())
      return NaN;
    if (V = cloneWithOffset(O, this), !V.isValid())
      return NaN;
    switch (q = (V.utcOffset() - this.utcOffset()) * 6e4, _ = normalizeUnits(_), _) {
      case "year":
        G = monthDiff(this, V) / 12;
        break;
      case "month":
        G = monthDiff(this, V);
        break;
      case "quarter":
        G = monthDiff(this, V) / 3;
        break;
      case "second":
        G = (this - V) / 1e3;
        break;
      // 1000
      case "minute":
        G = (this - V) / 6e4;
        break;
      // 1000 * 60
      case "hour":
        G = (this - V) / 36e5;
        break;
      // 1000 * 60 * 60
      case "day":
        G = (this - V - q) / 864e5;
        break;
      // 1000 * 60 * 60 * 24, negate dst
      case "week":
        G = (this - V - q) / 6048e5;
        break;
      // 1000 * 60 * 60 * 24 * 7, negate dst
      default:
        G = this - V;
    }
    return H ? G : absFloor(G);
  }
  function monthDiff(O, _) {
    if (O.date() < _.date())
      return -monthDiff(_, O);
    var H = (_.year() - O.year()) * 12 + (_.month() - O.month()), V = O.clone().add(H, "months"), q, G;
    return _ - V < 0 ? (q = O.clone().add(H - 1, "months"), G = (_ - V) / (V - q)) : (q = O.clone().add(H + 1, "months"), G = (_ - V) / (q - V)), -(H + G) || 0;
  }
  hooks.defaultFormat = "YYYY-MM-DDTHH:mm:ssZ";
  hooks.defaultFormatUtc = "YYYY-MM-DDTHH:mm:ss[Z]";
  function toString$4() {
    return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ");
  }
  function toISOString(O) {
    if (!this.isValid())
      return null;
    var _ = O !== !0, H = _ ? this.clone().utc() : this;
    return H.year() < 0 || H.year() > 9999 ? formatMoment(
      H,
      _ ? "YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]" : "YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"
    ) : isFunction$4(Date.prototype.toISOString) ? _ ? this.toDate().toISOString() : new Date(this.valueOf() + this.utcOffset() * 60 * 1e3).toISOString().replace("Z", formatMoment(H, "Z")) : formatMoment(
      H,
      _ ? "YYYY-MM-DD[T]HH:mm:ss.SSS[Z]" : "YYYY-MM-DD[T]HH:mm:ss.SSSZ"
    );
  }
  function inspect() {
    if (!this.isValid())
      return "moment.invalid(/* " + this._i + " */)";
    var O = "moment", _ = "", H, V, q, G;
    return this.isLocal() || (O = this.utcOffset() === 0 ? "moment.utc" : "moment.parseZone", _ = "Z"), H = "[" + O + '("]', V = 0 <= this.year() && this.year() <= 9999 ? "YYYY" : "YYYYYY", q = "-MM-DD[T]HH:mm:ss.SSS", G = _ + '[")]', this.format(H + V + q + G);
  }
  function format$4(O) {
    O || (O = this.isUtc() ? hooks.defaultFormatUtc : hooks.defaultFormat);
    var _ = formatMoment(this, O);
    return this.localeData().postformat(_);
  }
  function from(O, _) {
    return this.isValid() && (isMoment(O) && O.isValid() || createLocal(O).isValid()) ? createDuration({ to: this, from: O }).locale(this.locale()).humanize(!_) : this.localeData().invalidDate();
  }
  function fromNow(O) {
    return this.from(createLocal(), O);
  }
  function to(O, _) {
    return this.isValid() && (isMoment(O) && O.isValid() || createLocal(O).isValid()) ? createDuration({ from: this, to: O }).locale(this.locale()).humanize(!_) : this.localeData().invalidDate();
  }
  function toNow(O) {
    return this.to(createLocal(), O);
  }
  function locale$3(O) {
    var _;
    return O === void 0 ? this._locale._abbr : (_ = getLocale(O), _ != null && (this._locale = _), this);
  }
  var lang = deprecate(
    "moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",
    function(O) {
      return O === void 0 ? this.localeData() : this.locale(O);
    }
  );
  function localeData() {
    return this._locale;
  }
  var MS_PER_SECOND = 1e3, MS_PER_MINUTE = 60 * MS_PER_SECOND, MS_PER_HOUR = 60 * MS_PER_MINUTE, MS_PER_400_YEARS = (365 * 400 + 97) * 24 * MS_PER_HOUR;
  function mod$1(O, _) {
    return (O % _ + _) % _;
  }
  function localStartOfDate(O, _, H) {
    return O < 100 && O >= 0 ? new Date(O + 400, _, H) - MS_PER_400_YEARS : new Date(O, _, H).valueOf();
  }
  function utcStartOfDate(O, _, H) {
    return O < 100 && O >= 0 ? Date.UTC(O + 400, _, H) - MS_PER_400_YEARS : Date.UTC(O, _, H);
  }
  function startOf(O) {
    var _, H;
    if (O = normalizeUnits(O), O === void 0 || O === "millisecond" || !this.isValid())
      return this;
    switch (H = this._isUTC ? utcStartOfDate : localStartOfDate, O) {
      case "year":
        _ = H(this.year(), 0, 1);
        break;
      case "quarter":
        _ = H(
          this.year(),
          this.month() - this.month() % 3,
          1
        );
        break;
      case "month":
        _ = H(this.year(), this.month(), 1);
        break;
      case "week":
        _ = H(
          this.year(),
          this.month(),
          this.date() - this.weekday()
        );
        break;
      case "isoWeek":
        _ = H(
          this.year(),
          this.month(),
          this.date() - (this.isoWeekday() - 1)
        );
        break;
      case "day":
      case "date":
        _ = H(this.year(), this.month(), this.date());
        break;
      case "hour":
        _ = this._d.valueOf(), _ -= mod$1(
          _ + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE),
          MS_PER_HOUR
        );
        break;
      case "minute":
        _ = this._d.valueOf(), _ -= mod$1(_, MS_PER_MINUTE);
        break;
      case "second":
        _ = this._d.valueOf(), _ -= mod$1(_, MS_PER_SECOND);
        break;
    }
    return this._d.setTime(_), hooks.updateOffset(this, !0), this;
  }
  function endOf(O) {
    var _, H;
    if (O = normalizeUnits(O), O === void 0 || O === "millisecond" || !this.isValid())
      return this;
    switch (H = this._isUTC ? utcStartOfDate : localStartOfDate, O) {
      case "year":
        _ = H(this.year() + 1, 0, 1) - 1;
        break;
      case "quarter":
        _ = H(
          this.year(),
          this.month() - this.month() % 3 + 3,
          1
        ) - 1;
        break;
      case "month":
        _ = H(this.year(), this.month() + 1, 1) - 1;
        break;
      case "week":
        _ = H(
          this.year(),
          this.month(),
          this.date() - this.weekday() + 7
        ) - 1;
        break;
      case "isoWeek":
        _ = H(
          this.year(),
          this.month(),
          this.date() - (this.isoWeekday() - 1) + 7
        ) - 1;
        break;
      case "day":
      case "date":
        _ = H(this.year(), this.month(), this.date() + 1) - 1;
        break;
      case "hour":
        _ = this._d.valueOf(), _ += MS_PER_HOUR - mod$1(
          _ + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE),
          MS_PER_HOUR
        ) - 1;
        break;
      case "minute":
        _ = this._d.valueOf(), _ += MS_PER_MINUTE - mod$1(_, MS_PER_MINUTE) - 1;
        break;
      case "second":
        _ = this._d.valueOf(), _ += MS_PER_SECOND - mod$1(_, MS_PER_SECOND) - 1;
        break;
    }
    return this._d.setTime(_), hooks.updateOffset(this, !0), this;
  }
  function valueOf() {
    return this._d.valueOf() - (this._offset || 0) * 6e4;
  }
  function unix() {
    return Math.floor(this.valueOf() / 1e3);
  }
  function toDate$1() {
    return new Date(this.valueOf());
  }
  function toArray() {
    var O = this;
    return [
      O.year(),
      O.month(),
      O.date(),
      O.hour(),
      O.minute(),
      O.second(),
      O.millisecond()
    ];
  }
  function toObject() {
    var O = this;
    return {
      years: O.year(),
      months: O.month(),
      date: O.date(),
      hours: O.hours(),
      minutes: O.minutes(),
      seconds: O.seconds(),
      milliseconds: O.milliseconds()
    };
  }
  function toJSON() {
    return this.isValid() ? this.toISOString() : null;
  }
  function isValid$2() {
    return isValid$1(this);
  }
  function parsingFlags() {
    return extend$6({}, getParsingFlags(this));
  }
  function invalidAt() {
    return getParsingFlags(this).overflow;
  }
  function creationData() {
    return {
      input: this._i,
      format: this._f,
      locale: this._locale,
      isUTC: this._isUTC,
      strict: this._strict
    };
  }
  addFormatToken("N", 0, 0, "eraAbbr");
  addFormatToken("NN", 0, 0, "eraAbbr");
  addFormatToken("NNN", 0, 0, "eraAbbr");
  addFormatToken("NNNN", 0, 0, "eraName");
  addFormatToken("NNNNN", 0, 0, "eraNarrow");
  addFormatToken("y", ["y", 1], "yo", "eraYear");
  addFormatToken("y", ["yy", 2], 0, "eraYear");
  addFormatToken("y", ["yyy", 3], 0, "eraYear");
  addFormatToken("y", ["yyyy", 4], 0, "eraYear");
  addRegexToken("N", matchEraAbbr);
  addRegexToken("NN", matchEraAbbr);
  addRegexToken("NNN", matchEraAbbr);
  addRegexToken("NNNN", matchEraName);
  addRegexToken("NNNNN", matchEraNarrow);
  addParseToken(
    ["N", "NN", "NNN", "NNNN", "NNNNN"],
    function(O, _, H, V) {
      var q = H._locale.erasParse(O, V, H._strict);
      q ? getParsingFlags(H).era = q : getParsingFlags(H).invalidEra = O;
    }
  );
  addRegexToken("y", matchUnsigned);
  addRegexToken("yy", matchUnsigned);
  addRegexToken("yyy", matchUnsigned);
  addRegexToken("yyyy", matchUnsigned);
  addRegexToken("yo", matchEraYearOrdinal);
  addParseToken(["y", "yy", "yyy", "yyyy"], YEAR$1);
  addParseToken(["yo"], function(O, _, H, V) {
    var q;
    H._locale._eraYearOrdinalRegex && (q = O.match(H._locale._eraYearOrdinalRegex)), H._locale.eraYearOrdinalParse ? _[YEAR$1] = H._locale.eraYearOrdinalParse(O, q) : _[YEAR$1] = parseInt(O, 10);
  });
  function localeEras(O, _) {
    var H, V, q, G = this._eras || getLocale("en")._eras;
    for (H = 0, V = G.length; H < V; ++H) {
      switch (typeof G[H].since) {
        case "string":
          q = hooks(G[H].since).startOf("day"), G[H].since = q.valueOf();
          break;
      }
      switch (typeof G[H].until) {
        case "undefined":
          G[H].until = 1 / 0;
          break;
        case "string":
          q = hooks(G[H].until).startOf("day").valueOf(), G[H].until = q.valueOf();
          break;
      }
    }
    return G;
  }
  function localeErasParse(O, _, H) {
    var V, q, G = this.eras(), J, Z, te;
    for (O = O.toUpperCase(), V = 0, q = G.length; V < q; ++V)
      if (J = G[V].name.toUpperCase(), Z = G[V].abbr.toUpperCase(), te = G[V].narrow.toUpperCase(), H)
        switch (_) {
          case "N":
          case "NN":
          case "NNN":
            if (Z === O)
              return G[V];
            break;
          case "NNNN":
            if (J === O)
              return G[V];
            break;
          case "NNNNN":
            if (te === O)
              return G[V];
            break;
        }
      else if ([J, Z, te].indexOf(O) >= 0)
        return G[V];
  }
  function localeErasConvertYear(O, _) {
    var H = O.since <= O.until ? 1 : -1;
    return _ === void 0 ? hooks(O.since).year() : hooks(O.since).year() + (_ - O.offset) * H;
  }
  function getEraName() {
    var O, _, H, V = this.localeData().eras();
    for (O = 0, _ = V.length; O < _; ++O)
      if (H = this.clone().startOf("day").valueOf(), V[O].since <= H && H <= V[O].until || V[O].until <= H && H <= V[O].since)
        return V[O].name;
    return "";
  }
  function getEraNarrow() {
    var O, _, H, V = this.localeData().eras();
    for (O = 0, _ = V.length; O < _; ++O)
      if (H = this.clone().startOf("day").valueOf(), V[O].since <= H && H <= V[O].until || V[O].until <= H && H <= V[O].since)
        return V[O].narrow;
    return "";
  }
  function getEraAbbr() {
    var O, _, H, V = this.localeData().eras();
    for (O = 0, _ = V.length; O < _; ++O)
      if (H = this.clone().startOf("day").valueOf(), V[O].since <= H && H <= V[O].until || V[O].until <= H && H <= V[O].since)
        return V[O].abbr;
    return "";
  }
  function getEraYear() {
    var O, _, H, V, q = this.localeData().eras();
    for (O = 0, _ = q.length; O < _; ++O)
      if (H = q[O].since <= q[O].until ? 1 : -1, V = this.clone().startOf("day").valueOf(), q[O].since <= V && V <= q[O].until || q[O].until <= V && V <= q[O].since)
        return (this.year() - hooks(q[O].since).year()) * H + q[O].offset;
    return this.year();
  }
  function erasNameRegex(O) {
    return hasOwnProp(this, "_erasNameRegex") || computeErasParse.call(this), O ? this._erasNameRegex : this._erasRegex;
  }
  function erasAbbrRegex(O) {
    return hasOwnProp(this, "_erasAbbrRegex") || computeErasParse.call(this), O ? this._erasAbbrRegex : this._erasRegex;
  }
  function erasNarrowRegex(O) {
    return hasOwnProp(this, "_erasNarrowRegex") || computeErasParse.call(this), O ? this._erasNarrowRegex : this._erasRegex;
  }
  function matchEraAbbr(O, _) {
    return _.erasAbbrRegex(O);
  }
  function matchEraName(O, _) {
    return _.erasNameRegex(O);
  }
  function matchEraNarrow(O, _) {
    return _.erasNarrowRegex(O);
  }
  function matchEraYearOrdinal(O, _) {
    return _._eraYearOrdinalRegex || matchUnsigned;
  }
  function computeErasParse() {
    var O = [], _ = [], H = [], V = [], q, G, J, Z, te, ne = this.eras();
    for (q = 0, G = ne.length; q < G; ++q)
      J = regexEscape(ne[q].name), Z = regexEscape(ne[q].abbr), te = regexEscape(ne[q].narrow), _.push(J), O.push(Z), H.push(te), V.push(J), V.push(Z), V.push(te);
    this._erasRegex = new RegExp("^(" + V.join("|") + ")", "i"), this._erasNameRegex = new RegExp("^(" + _.join("|") + ")", "i"), this._erasAbbrRegex = new RegExp("^(" + O.join("|") + ")", "i"), this._erasNarrowRegex = new RegExp(
      "^(" + H.join("|") + ")",
      "i"
    );
  }
  addFormatToken(0, ["gg", 2], 0, function() {
    return this.weekYear() % 100;
  });
  addFormatToken(0, ["GG", 2], 0, function() {
    return this.isoWeekYear() % 100;
  });
  function addWeekYearFormatToken(O, _) {
    addFormatToken(0, [O, O.length], 0, _);
  }
  addWeekYearFormatToken("gggg", "weekYear");
  addWeekYearFormatToken("ggggg", "weekYear");
  addWeekYearFormatToken("GGGG", "isoWeekYear");
  addWeekYearFormatToken("GGGGG", "isoWeekYear");
  addRegexToken("G", matchSigned);
  addRegexToken("g", matchSigned);
  addRegexToken("GG", match1to2, match2);
  addRegexToken("gg", match1to2, match2);
  addRegexToken("GGGG", match1to4, match4);
  addRegexToken("gggg", match1to4, match4);
  addRegexToken("GGGGG", match1to6, match6);
  addRegexToken("ggggg", match1to6, match6);
  addWeekParseToken(
    ["gggg", "ggggg", "GGGG", "GGGGG"],
    function(O, _, H, V) {
      _[V.substr(0, 2)] = toInt(O);
    }
  );
  addWeekParseToken(["gg", "GG"], function(O, _, H, V) {
    _[V] = hooks.parseTwoDigitYear(O);
  });
  function getSetWeekYear(O) {
    return getSetWeekYearHelper.call(
      this,
      O,
      this.week(),
      this.weekday() + this.localeData()._week.dow,
      this.localeData()._week.dow,
      this.localeData()._week.doy
    );
  }
  function getSetISOWeekYear(O) {
    return getSetWeekYearHelper.call(
      this,
      O,
      this.isoWeek(),
      this.isoWeekday(),
      1,
      4
    );
  }
  function getISOWeeksInYear() {
    return weeksInYear(this.year(), 1, 4);
  }
  function getISOWeeksInISOWeekYear() {
    return weeksInYear(this.isoWeekYear(), 1, 4);
  }
  function getWeeksInYear() {
    var O = this.localeData()._week;
    return weeksInYear(this.year(), O.dow, O.doy);
  }
  function getWeeksInWeekYear() {
    var O = this.localeData()._week;
    return weeksInYear(this.weekYear(), O.dow, O.doy);
  }
  function getSetWeekYearHelper(O, _, H, V, q) {
    var G;
    return O == null ? weekOfYear(this, V, q).year : (G = weeksInYear(O, V, q), _ > G && (_ = G), setWeekAll.call(this, O, _, H, V, q));
  }
  function setWeekAll(O, _, H, V, q) {
    var G = dayOfYearFromWeeks(O, _, H, V, q), J = createUTCDate(G.year, 0, G.dayOfYear);
    return this.year(J.getUTCFullYear()), this.month(J.getUTCMonth()), this.date(J.getUTCDate()), this;
  }
  addFormatToken("Q", 0, "Qo", "quarter");
  addRegexToken("Q", match1);
  addParseToken("Q", function(O, _) {
    _[MONTH$1] = (toInt(O) - 1) * 3;
  });
  function getSetQuarter(O) {
    return O == null ? Math.ceil((this.month() + 1) / 3) : this.month((O - 1) * 3 + this.month() % 3);
  }
  addFormatToken("D", ["DD", 2], "Do", "date");
  addRegexToken("D", match1to2, match1to2NoLeadingZero);
  addRegexToken("DD", match1to2, match2);
  addRegexToken("Do", function(O, _) {
    return O ? _._dayOfMonthOrdinalParse || _._ordinalParse : _._dayOfMonthOrdinalParseLenient;
  });
  addParseToken(["D", "DD"], DATE$1);
  addParseToken("Do", function(O, _) {
    _[DATE$1] = toInt(O.match(match1to2)[0]);
  });
  var getSetDayOfMonth = makeGetSet("Date", !0);
  addFormatToken("DDD", ["DDDD", 3], "DDDo", "dayOfYear");
  addRegexToken("DDD", match1to3);
  addRegexToken("DDDD", match3);
  addParseToken(["DDD", "DDDD"], function(O, _, H) {
    H._dayOfYear = toInt(O);
  });
  function getSetDayOfYear(O) {
    var _ = Math.round(
      (this.clone().startOf("day") - this.clone().startOf("year")) / 864e5
    ) + 1;
    return O == null ? _ : this.add(O - _, "d");
  }
  addFormatToken("m", ["mm", 2], 0, "minute");
  addRegexToken("m", match1to2, match1to2HasZero);
  addRegexToken("mm", match1to2, match2);
  addParseToken(["m", "mm"], MINUTE);
  var getSetMinute = makeGetSet("Minutes", !1);
  addFormatToken("s", ["ss", 2], 0, "second");
  addRegexToken("s", match1to2, match1to2HasZero);
  addRegexToken("ss", match1to2, match2);
  addParseToken(["s", "ss"], SECOND);
  var getSetSecond = makeGetSet("Seconds", !1);
  addFormatToken("S", 0, 0, function() {
    return ~~(this.millisecond() / 100);
  });
  addFormatToken(0, ["SS", 2], 0, function() {
    return ~~(this.millisecond() / 10);
  });
  addFormatToken(0, ["SSS", 3], 0, "millisecond");
  addFormatToken(0, ["SSSS", 4], 0, function() {
    return this.millisecond() * 10;
  });
  addFormatToken(0, ["SSSSS", 5], 0, function() {
    return this.millisecond() * 100;
  });
  addFormatToken(0, ["SSSSSS", 6], 0, function() {
    return this.millisecond() * 1e3;
  });
  addFormatToken(0, ["SSSSSSS", 7], 0, function() {
    return this.millisecond() * 1e4;
  });
  addFormatToken(0, ["SSSSSSSS", 8], 0, function() {
    return this.millisecond() * 1e5;
  });
  addFormatToken(0, ["SSSSSSSSS", 9], 0, function() {
    return this.millisecond() * 1e6;
  });
  addRegexToken("S", match1to3, match1);
  addRegexToken("SS", match1to3, match2);
  addRegexToken("SSS", match1to3, match3);
  var token$1, getSetMillisecond;
  for (token$1 = "SSSS"; token$1.length <= 9; token$1 += "S")
    addRegexToken(token$1, matchUnsigned);
  function parseMs(O, _) {
    _[MILLISECOND] = toInt(("0." + O) * 1e3);
  }
  for (token$1 = "S"; token$1.length <= 9; token$1 += "S")
    addParseToken(token$1, parseMs);
  getSetMillisecond = makeGetSet("Milliseconds", !1);
  addFormatToken("z", 0, 0, "zoneAbbr");
  addFormatToken("zz", 0, 0, "zoneName");
  function getZoneAbbr() {
    return this._isUTC ? "UTC" : "";
  }
  function getZoneName() {
    return this._isUTC ? "Coordinated Universal Time" : "";
  }
  var proto = Moment.prototype;
  proto.add = add$7;
  proto.calendar = calendar$1$1;
  proto.clone = clone$2;
  proto.diff = diff;
  proto.endOf = endOf;
  proto.format = format$4;
  proto.from = from;
  proto.fromNow = fromNow;
  proto.to = to;
  proto.toNow = toNow;
  proto.get = stringGet;
  proto.invalidAt = invalidAt;
  proto.isAfter = isAfter;
  proto.isBefore = isBefore;
  proto.isBetween = isBetween$1;
  proto.isSame = isSame;
  proto.isSameOrAfter = isSameOrAfter;
  proto.isSameOrBefore = isSameOrBefore;
  proto.isValid = isValid$2;
  proto.lang = lang;
  proto.locale = locale$3;
  proto.localeData = localeData;
  proto.max = prototypeMax;
  proto.min = prototypeMin;
  proto.parsingFlags = parsingFlags;
  proto.set = stringSet;
  proto.startOf = startOf;
  proto.subtract = subtract;
  proto.toArray = toArray;
  proto.toObject = toObject;
  proto.toDate = toDate$1;
  proto.toISOString = toISOString;
  proto.inspect = inspect;
  typeof Symbol < "u" && Symbol.for != null && (proto[Symbol.for("nodejs.util.inspect.custom")] = function() {
    return "Moment<" + this.format() + ">";
  });
  proto.toJSON = toJSON;
  proto.toString = toString$4;
  proto.unix = unix;
  proto.valueOf = valueOf;
  proto.creationData = creationData;
  proto.eraName = getEraName;
  proto.eraNarrow = getEraNarrow;
  proto.eraAbbr = getEraAbbr;
  proto.eraYear = getEraYear;
  proto.year = getSetYear;
  proto.isLeapYear = getIsLeapYear;
  proto.weekYear = getSetWeekYear;
  proto.isoWeekYear = getSetISOWeekYear;
  proto.quarter = proto.quarters = getSetQuarter;
  proto.month = getSetMonth;
  proto.daysInMonth = getDaysInMonth;
  proto.week = proto.weeks = getSetWeek;
  proto.isoWeek = proto.isoWeeks = getSetISOWeek;
  proto.weeksInYear = getWeeksInYear;
  proto.weeksInWeekYear = getWeeksInWeekYear;
  proto.isoWeeksInYear = getISOWeeksInYear;
  proto.isoWeeksInISOWeekYear = getISOWeeksInISOWeekYear;
  proto.date = getSetDayOfMonth;
  proto.day = proto.days = getSetDayOfWeek;
  proto.weekday = getSetLocaleDayOfWeek;
  proto.isoWeekday = getSetISODayOfWeek;
  proto.dayOfYear = getSetDayOfYear;
  proto.hour = proto.hours = getSetHour;
  proto.minute = proto.minutes = getSetMinute;
  proto.second = proto.seconds = getSetSecond;
  proto.millisecond = proto.milliseconds = getSetMillisecond;
  proto.utcOffset = getSetOffset;
  proto.utc = setOffsetToUTC;
  proto.local = setOffsetToLocal;
  proto.parseZone = setOffsetToParsedOffset;
  proto.hasAlignedHourOffset = hasAlignedHourOffset;
  proto.isDST = isDaylightSavingTime;
  proto.isLocal = isLocal;
  proto.isUtcOffset = isUtcOffset;
  proto.isUtc = isUtc;
  proto.isUTC = isUtc;
  proto.zoneAbbr = getZoneAbbr;
  proto.zoneName = getZoneName;
  proto.dates = deprecate(
    "dates accessor is deprecated. Use date instead.",
    getSetDayOfMonth
  );
  proto.months = deprecate(
    "months accessor is deprecated. Use month instead",
    getSetMonth
  );
  proto.years = deprecate(
    "years accessor is deprecated. Use year instead",
    getSetYear
  );
  proto.zone = deprecate(
    "moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",
    getSetZone
  );
  proto.isDSTShifted = deprecate(
    "isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",
    isDaylightSavingTimeShifted
  );
  function createUnix(O) {
    return createLocal(O * 1e3);
  }
  function createInZone() {
    return createLocal.apply(null, arguments).parseZone();
  }
  function preParsePostFormat(O) {
    return O;
  }
  var proto$1 = Locale.prototype;
  proto$1.calendar = calendar$1;
  proto$1.longDateFormat = longDateFormat;
  proto$1.invalidDate = invalidDate;
  proto$1.ordinal = ordinal$1;
  proto$1.preparse = preParsePostFormat;
  proto$1.postformat = preParsePostFormat;
  proto$1.relativeTime = relativeTime;
  proto$1.pastFuture = pastFuture;
  proto$1.set = set$7;
  proto$1.eras = localeEras;
  proto$1.erasParse = localeErasParse;
  proto$1.erasConvertYear = localeErasConvertYear;
  proto$1.erasAbbrRegex = erasAbbrRegex;
  proto$1.erasNameRegex = erasNameRegex;
  proto$1.erasNarrowRegex = erasNarrowRegex;
  proto$1.months = localeMonths;
  proto$1.monthsShort = localeMonthsShort;
  proto$1.monthsParse = localeMonthsParse;
  proto$1.monthsRegex = monthsRegex;
  proto$1.monthsShortRegex = monthsShortRegex;
  proto$1.week = localeWeek;
  proto$1.firstDayOfYear = localeFirstDayOfYear;
  proto$1.firstDayOfWeek = localeFirstDayOfWeek;
  proto$1.weekdays = localeWeekdays;
  proto$1.weekdaysMin = localeWeekdaysMin;
  proto$1.weekdaysShort = localeWeekdaysShort;
  proto$1.weekdaysParse = localeWeekdaysParse;
  proto$1.weekdaysRegex = weekdaysRegex;
  proto$1.weekdaysShortRegex = weekdaysShortRegex;
  proto$1.weekdaysMinRegex = weekdaysMinRegex;
  proto$1.isPM = localeIsPM;
  proto$1.meridiem = localeMeridiem;
  function get$1$1(O, _, H, V) {
    var q = getLocale(), G = createUTC().set(V, _);
    return q[H](G, O);
  }
  function listMonthsImpl(O, _, H) {
    if (isNumber$4(O) && (_ = O, O = void 0), O = O || "", _ != null)
      return get$1$1(O, _, H, "month");
    var V, q = [];
    for (V = 0; V < 12; V++)
      q[V] = get$1$1(O, V, H, "month");
    return q;
  }
  function listWeekdaysImpl(O, _, H, V) {
    typeof O == "boolean" ? (isNumber$4(_) && (H = _, _ = void 0), _ = _ || "") : (_ = O, H = _, O = !1, isNumber$4(_) && (H = _, _ = void 0), _ = _ || "");
    var q = getLocale(), G = O ? q._week.dow : 0, J, Z = [];
    if (H != null)
      return get$1$1(_, (H + G) % 7, V, "day");
    for (J = 0; J < 7; J++)
      Z[J] = get$1$1(_, (J + G) % 7, V, "day");
    return Z;
  }
  function listMonths(O, _) {
    return listMonthsImpl(O, _, "months");
  }
  function listMonthsShort(O, _) {
    return listMonthsImpl(O, _, "monthsShort");
  }
  function listWeekdays(O, _, H) {
    return listWeekdaysImpl(O, _, H, "weekdays");
  }
  function listWeekdaysShort(O, _, H) {
    return listWeekdaysImpl(O, _, H, "weekdaysShort");
  }
  function listWeekdaysMin(O, _, H) {
    return listWeekdaysImpl(O, _, H, "weekdaysMin");
  }
  getSetGlobalLocale("en", {
    eras: [
      {
        since: "0001-01-01",
        until: 1 / 0,
        offset: 1,
        name: "Anno Domini",
        narrow: "AD",
        abbr: "AD"
      },
      {
        since: "0000-12-31",
        until: -1 / 0,
        offset: 1,
        name: "Before Christ",
        narrow: "BC",
        abbr: "BC"
      }
    ],
    dayOfMonthOrdinalParse: /\d{1,2}(th|st|nd|rd)/,
    ordinal: function(O) {
      var _ = O % 10, H = toInt(O % 100 / 10) === 1 ? "th" : _ === 1 ? "st" : _ === 2 ? "nd" : _ === 3 ? "rd" : "th";
      return O + H;
    }
  });
  hooks.lang = deprecate(
    "moment.lang is deprecated. Use moment.locale instead.",
    getSetGlobalLocale
  );
  hooks.langData = deprecate(
    "moment.langData is deprecated. Use moment.localeData instead.",
    getLocale
  );
  var mathAbs = Math.abs;
  function abs$g() {
    var O = this._data;
    return this._milliseconds = mathAbs(this._milliseconds), this._days = mathAbs(this._days), this._months = mathAbs(this._months), O.milliseconds = mathAbs(O.milliseconds), O.seconds = mathAbs(O.seconds), O.minutes = mathAbs(O.minutes), O.hours = mathAbs(O.hours), O.months = mathAbs(O.months), O.years = mathAbs(O.years), this;
  }
  function addSubtract$1(O, _, H, V) {
    var q = createDuration(_, H);
    return O._milliseconds += V * q._milliseconds, O._days += V * q._days, O._months += V * q._months, O._bubble();
  }
  function add$1$1(O, _) {
    return addSubtract$1(this, O, _, 1);
  }
  function subtract$1(O, _) {
    return addSubtract$1(this, O, _, -1);
  }
  function absCeil(O) {
    return O < 0 ? Math.floor(O) : Math.ceil(O);
  }
  function bubble() {
    var O = this._milliseconds, _ = this._days, H = this._months, V = this._data, q, G, J, Z, te;
    return O >= 0 && _ >= 0 && H >= 0 || O <= 0 && _ <= 0 && H <= 0 || (O += absCeil(monthsToDays(H) + _) * 864e5, _ = 0, H = 0), V.milliseconds = O % 1e3, q = absFloor(O / 1e3), V.seconds = q % 60, G = absFloor(q / 60), V.minutes = G % 60, J = absFloor(G / 60), V.hours = J % 24, _ += absFloor(J / 24), te = absFloor(daysToMonths(_)), H += te, _ -= absCeil(monthsToDays(te)), Z = absFloor(H / 12), H %= 12, V.days = _, V.months = H, V.years = Z, this;
  }
  function daysToMonths(O) {
    return O * 4800 / 146097;
  }
  function monthsToDays(O) {
    return O * 146097 / 4800;
  }
  function as(O) {
    if (!this.isValid())
      return NaN;
    var _, H, V = this._milliseconds;
    if (O = normalizeUnits(O), O === "month" || O === "quarter" || O === "year")
      switch (_ = this._days + V / 864e5, H = this._months + daysToMonths(_), O) {
        case "month":
          return H;
        case "quarter":
          return H / 3;
        case "year":
          return H / 12;
      }
    else
      switch (_ = this._days + Math.round(monthsToDays(this._months)), O) {
        case "week":
          return _ / 7 + V / 6048e5;
        case "day":
          return _ + V / 864e5;
        case "hour":
          return _ * 24 + V / 36e5;
        case "minute":
          return _ * 1440 + V / 6e4;
        case "second":
          return _ * 86400 + V / 1e3;
        // Math.floor prevents floating point math errors here
        case "millisecond":
          return Math.floor(_ * 864e5) + V;
        default:
          throw new Error("Unknown unit " + O);
      }
  }
  function makeAs(O) {
    return function() {
      return this.as(O);
    };
  }
  var asMilliseconds = makeAs("ms"), asSeconds = makeAs("s"), asMinutes = makeAs("m"), asHours = makeAs("h"), asDays = makeAs("d"), asWeeks = makeAs("w"), asMonths = makeAs("M"), asQuarters = makeAs("Q"), asYears = makeAs("y"), valueOf$1 = asMilliseconds;
  function clone$1$1() {
    return createDuration(this);
  }
  function get$2$1(O) {
    return O = normalizeUnits(O), this.isValid() ? this[O + "s"]() : NaN;
  }
  function makeGetter(O) {
    return function() {
      return this.isValid() ? this._data[O] : NaN;
    };
  }
  var milliseconds = makeGetter("milliseconds"), seconds = makeGetter("seconds"), minutes = makeGetter("minutes"), hours = makeGetter("hours"), days = makeGetter("days"), months = makeGetter("months"), years = makeGetter("years");
  function weeks() {
    return absFloor(this.days() / 7);
  }
  var round$e = Math.round, thresholds = {
    ss: 44,
    // a few seconds to seconds
    s: 45,
    // seconds to minute
    m: 45,
    // minutes to hour
    h: 22,
    // hours to day
    d: 26,
    // days to month/week
    w: null,
    // weeks to month
    M: 11
    // months to year
  };
  function substituteTimeAgo(O, _, H, V, q) {
    return q.relativeTime(_ || 1, !!H, O, V);
  }
  function relativeTime$1(O, _, H, V) {
    var q = createDuration(O).abs(), G = round$e(q.as("s")), J = round$e(q.as("m")), Z = round$e(q.as("h")), te = round$e(q.as("d")), ne = round$e(q.as("M")), ee = round$e(q.as("w")), ae = round$e(q.as("y")), oe = G <= H.ss && ["s", G] || G < H.s && ["ss", G] || J <= 1 && ["m"] || J < H.m && ["mm", J] || Z <= 1 && ["h"] || Z < H.h && ["hh", Z] || te <= 1 && ["d"] || te < H.d && ["dd", te];
    return H.w != null && (oe = oe || ee <= 1 && ["w"] || ee < H.w && ["ww", ee]), oe = oe || ne <= 1 && ["M"] || ne < H.M && ["MM", ne] || ae <= 1 && ["y"] || ["yy", ae], oe[2] = _, oe[3] = +O > 0, oe[4] = V, substituteTimeAgo.apply(null, oe);
  }
  function getSetRelativeTimeRounding(O) {
    return O === void 0 ? round$e : typeof O == "function" ? (round$e = O, !0) : !1;
  }
  function getSetRelativeTimeThreshold(O, _) {
    return thresholds[O] === void 0 ? !1 : _ === void 0 ? thresholds[O] : (thresholds[O] = _, O === "s" && (thresholds.ss = _ - 1), !0);
  }
  function humanize(O, _) {
    if (!this.isValid())
      return this.localeData().invalidDate();
    var H = !1, V = thresholds, q, G;
    return typeof O == "object" && (_ = O, O = !1), typeof O == "boolean" && (H = O), typeof _ == "object" && (V = Object.assign({}, thresholds, _), _.s != null && _.ss == null && (V.ss = _.s - 1)), q = this.localeData(), G = relativeTime$1(this, !H, V, q), H && (G = q.pastFuture(+this, G)), q.postformat(G);
  }
  var abs$1$1 = Math.abs;
  function sign$4(O) {
    return (O > 0) - (O < 0) || +O;
  }
  function toISOString$1() {
    if (!this.isValid())
      return this.localeData().invalidDate();
    var O = abs$1$1(this._milliseconds) / 1e3, _ = abs$1$1(this._days), H = abs$1$1(this._months), V, q, G, J, Z = this.asSeconds(), te, ne, ee, ae;
    return Z ? (V = absFloor(O / 60), q = absFloor(V / 60), O %= 60, V %= 60, G = absFloor(H / 12), H %= 12, J = O ? O.toFixed(3).replace(/\.?0+$/, "") : "", te = Z < 0 ? "-" : "", ne = sign$4(this._months) !== sign$4(Z) ? "-" : "", ee = sign$4(this._days) !== sign$4(Z) ? "-" : "", ae = sign$4(this._milliseconds) !== sign$4(Z) ? "-" : "", te + "P" + (G ? ne + G + "Y" : "") + (H ? ne + H + "M" : "") + (_ ? ee + _ + "D" : "") + (q || V || O ? "T" : "") + (q ? ae + q + "H" : "") + (V ? ae + V + "M" : "") + (O ? ae + J + "S" : "")) : "P0D";
  }
  var proto$2 = Duration$2.prototype;
  proto$2.isValid = isValid$1$1;
  proto$2.abs = abs$g;
  proto$2.add = add$1$1;
  proto$2.subtract = subtract$1;
  proto$2.as = as;
  proto$2.asMilliseconds = asMilliseconds;
  proto$2.asSeconds = asSeconds;
  proto$2.asMinutes = asMinutes;
  proto$2.asHours = asHours;
  proto$2.asDays = asDays;
  proto$2.asWeeks = asWeeks;
  proto$2.asMonths = asMonths;
  proto$2.asQuarters = asQuarters;
  proto$2.asYears = asYears;
  proto$2.valueOf = valueOf$1;
  proto$2._bubble = bubble;
  proto$2.clone = clone$1$1;
  proto$2.get = get$2$1;
  proto$2.milliseconds = milliseconds;
  proto$2.seconds = seconds;
  proto$2.minutes = minutes;
  proto$2.hours = hours;
  proto$2.days = days;
  proto$2.weeks = weeks;
  proto$2.months = months;
  proto$2.years = years;
  proto$2.humanize = humanize;
  proto$2.toISOString = toISOString$1;
  proto$2.toString = toISOString$1;
  proto$2.toJSON = toISOString$1;
  proto$2.locale = locale$3;
  proto$2.localeData = localeData;
  proto$2.toIsoString = deprecate(
    "toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",
    toISOString$1
  );
  proto$2.lang = lang;
  addFormatToken("X", 0, 0, "unix");
  addFormatToken("x", 0, 0, "valueOf");
  addRegexToken("x", matchSigned);
  addRegexToken("X", matchTimestamp);
  addParseToken("X", function(O, _, H) {
    H._d = new Date(parseFloat(O) * 1e3);
  });
  addParseToken("x", function(O, _, H) {
    H._d = new Date(toInt(O));
  });
  //! moment.js
  hooks.version = "2.30.1";
  setHookCallback(createLocal);
  hooks.fn = proto;
  hooks.min = min$h;
  hooks.max = max$p;
  hooks.now = now$3;
  hooks.utc = createUTC;
  hooks.unix = createUnix;
  hooks.months = listMonths;
  hooks.isDate = isDate$2;
  hooks.locale = getSetGlobalLocale;
  hooks.invalid = createInvalid;
  hooks.duration = createDuration;
  hooks.isMoment = isMoment;
  hooks.weekdays = listWeekdays;
  hooks.parseZone = createInZone;
  hooks.localeData = getLocale;
  hooks.isDuration = isDuration;
  hooks.monthsShort = listMonthsShort;
  hooks.weekdaysMin = listWeekdaysMin;
  hooks.defineLocale = defineLocale;
  hooks.updateLocale = updateLocale;
  hooks.locales = listLocales;
  hooks.weekdaysShort = listWeekdaysShort;
  hooks.normalizeUnits = normalizeUnits;
  hooks.relativeTimeRounding = getSetRelativeTimeRounding;
  hooks.relativeTimeThreshold = getSetRelativeTimeThreshold;
  hooks.calendarFormat = getCalendarFormat;
  hooks.prototype = proto;
  hooks.HTML5_FMT = {
    DATETIME_LOCAL: "YYYY-MM-DDTHH:mm",
    // <input type="datetime-local" />
    DATETIME_LOCAL_SECONDS: "YYYY-MM-DDTHH:mm:ss",
    // <input type="datetime-local" step="1" />
    DATETIME_LOCAL_MS: "YYYY-MM-DDTHH:mm:ss.SSS",
    // <input type="datetime-local" step="0.001" />
    DATE: "YYYY-MM-DD",
    // <input type="date" />
    TIME: "HH:mm",
    // <input type="time" />
    TIME_SECONDS: "HH:mm:ss",
    // <input type="time" step="1" />
    TIME_MS: "HH:mm:ss.SSS",
    // <input type="time" step="0.001" />
    WEEK: "GGGG-[W]WW",
    // <input type="week" />
    MONTH: "YYYY-MM"
    // <input type="month" />
  };
  const _sfc_main$u = /* @__PURE__ */ defineComponent({
    __name: "ColorField",
    props: {
      name: {},
      label: {},
      initial: {},
      required: { type: Boolean },
      help: {},
      validation: {}
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (q) => q !== void 0
      ).join("|"));
      return (V, q) => {
        const G = resolveComponent("form-kit");
        return openBlock(), createBlock(G, {
          type: "color",
          label: V.label || V.name,
          name: V.name,
          value: V.initial,
          validation: H.value,
          help: V.help,
          "validation-visibility": "live",
          "outer-class": "flex-1"
        }, null, 8, ["label", "name", "value", "validation", "help"]);
      };
    }
  }), _sfc_main$t = /* @__PURE__ */ defineComponent({
    __name: "DateTimeField",
    props: {
      name: {},
      type: {},
      initial: {},
      label: {},
      help: {},
      validation: {},
      required: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (q) => q !== void 0
      ).join("|"));
      return (V, q) => {
        const G = resolveComponent("form-kit");
        return openBlock(), createBlock(G, {
          type: V.type,
          "data-cy": `${V.type}-field`,
          value: V.initial,
          name: V.name,
          label: V.label || V.name,
          validation: H.value,
          help: V.help,
          "validation-visibility": "live",
          step: "1"
        }, null, 8, ["type", "data-cy", "value", "name", "label", "validation", "help"]);
      };
    }
  });
  function render$3(O, _) {
    return openBlock(), createElementBlock("svg", {
      xmlns: "http://www.w3.org/2000/svg",
      fill: "none",
      viewBox: "0 0 24 24",
      "stroke-width": "1.5",
      stroke: "currentColor",
      "aria-hidden": "true",
      "data-slot": "icon"
    }, [
      createBaseVNode("path", {
        "stroke-linecap": "round",
        "stroke-linejoin": "round",
        d: "M10.5 6h9.75M10.5 6a1.5 1.5 0 1 1-3 0m3 0a1.5 1.5 0 1 0-3 0M3.75 6H7.5m3 12h9.75m-9.75 0a1.5 1.5 0 0 1-3 0m3 0a1.5 1.5 0 0 0-3 0m-3.75 0H7.5m9-6h3.75m-3.75 0a1.5 1.5 0 0 1-3 0m3 0a1.5 1.5 0 0 0-3 0m-9.75 0h9.75"
      })
    ]);
  }
  function render$2(O, _) {
    return openBlock(), createElementBlock("svg", {
      xmlns: "http://www.w3.org/2000/svg",
      fill: "none",
      viewBox: "0 0 24 24",
      "stroke-width": "1.5",
      stroke: "currentColor",
      "aria-hidden": "true",
      "data-slot": "icon"
    }, [
      createBaseVNode("path", {
        "stroke-linecap": "round",
        "stroke-linejoin": "round",
        d: "M12 16.5V9.75m0 0 3 3m-3-3-3 3M6.75 19.5a4.5 4.5 0 0 1-1.41-8.775 5.25 5.25 0 0 1 10.233-2.33 3 3 0 0 1 3.758 3.848A3.752 3.752 0 0 1 18 19.5H6.75Z"
      })
    ]);
  }
  const _sfc_main$s = /* @__PURE__ */ defineComponent({
    __name: "FileField",
    props: {
      name: {},
      label: {},
      help: {},
      validation: {},
      required: { type: Boolean },
      accept: {}
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (J) => J !== void 0
      ).join("|")), V = ref$1(!1), q = `file_${_.name}`;
      return (G, J) => {
        const Z = resolveComponent("form-kit");
        return openBlock(), createBlock(Z, {
          type: "file",
          id: q,
          label: G.label || G.name,
          name: "parameter_files",
          validation: H.value,
          help: G.help,
          accept: G.accept,
          "file-item-icon": "fileDoc",
          "file-remove-icon": "trash",
          "no-files-icon": "fileDoc",
          "validation-visibility": "live",
          form: "params-form"
        }, {
          noFiles: withCtx(() => [
            V.value ? createCommentVNode("", !0) : (openBlock(), createElementBlock("label", {
              key: 0,
              class: "flex w-full h-full items-center space-x-4 text-gray-500 hover:text-gray-700 formkit-no-files px-4 pt-3 pb-4 cursor-pointer",
              for: q
            }, [
              createVNode(unref(render$2), { class: "w-6 h-6" }),
              J[0] || (J[0] = createBaseVNode("div", {
                class: "flex flex-col",
                "data-cy": "file-field"
              }, [
                createBaseVNode("span", { class: "font-semibold text-sm" }, "Upload a file"),
                createBaseVNode("span", { class: "text-xs" }, "Up to 25mb supported.")
              ], -1))
            ]))
          ]),
          _: 1
        }, 8, ["label", "validation", "help", "accept"]);
      };
    }
  }), _sfc_main$r = /* @__PURE__ */ defineComponent({
    __name: "HiddenField",
    props: {
      name: {},
      initial: {}
    },
    setup(O) {
      return (_, H) => {
        const V = resolveComponent("form-kit");
        return openBlock(), createBlock(V, {
          type: "hidden",
          name: _.name,
          value: _.initial
        }, null, 8, ["name", "value"]);
      };
    }
  }), _hoisted_1$g = {
    class: "mb-6",
    "data-cy": "multi-select-field"
  }, _sfc_main$q = /* @__PURE__ */ defineComponent({
    __name: "MultiChoiceField",
    props: {
      initial: {},
      options: {},
      label: {},
      required: { type: Boolean },
      validation: {},
      help: {},
      name: {}
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (G) => G !== void 0
      ).join("|")), V = {
        closeOnSelect: !1,
        clearOnSelect: !1,
        preserveSearch: !0,
        placeholder: "",
        preselectFirst: !1,
        searchable: !1
      };
      return (q, G) => {
        const J = resolveComponent("form-kit");
        return openBlock(), createElementBlock("div", _hoisted_1$g, [
          createVNode(J, {
            type: "multiSelectField",
            label: q.label || q.name,
            multiSelectProps: V,
            name: q.name,
            help: q.help,
            validation: H.value,
            "validation-visibility": "live",
            tags: q.initial,
            options: q.options
          }, null, 8, ["label", "name", "help", "validation", "tags", "options"])
        ]);
      };
    }
  }), _sfc_main$p = /* @__PURE__ */ defineComponent({
    __name: "NumberBox",
    props: {
      name: {},
      label: {},
      initial: {},
      validation: {},
      help: {},
      required: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (q) => q !== void 0
      ).join("|"));
      return (V, q) => {
        const G = resolveComponent("form-kit");
        return openBlock(), createBlock(G, {
          type: "number",
          label: V.label || V.name,
          name: V.name,
          value: V.initial,
          validation: H.value,
          help: V.help,
          step: "any",
          "validation-visibility": "live",
          "outer-class": "flex-1",
          "data-cy": "number-field"
        }, null, 8, ["label", "name", "value", "validation", "help"]);
      };
    }
  }), _sfc_main$o = /* @__PURE__ */ defineComponent({
    __name: "PasswordField",
    props: {
      name: {},
      label: {},
      initial: {},
      required: { type: Boolean },
      validation: {},
      help: {}
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (q) => q !== void 0
      ).join("|"));
      return (V, q) => {
        const G = resolveComponent("form-kit");
        return openBlock(), createBlock(G, {
          type: "password",
          label: V.label || V.name,
          name: V.name,
          value: V.initial,
          validation: H.value,
          help: V.help,
          "validation-visibility": "live",
          "outer-class": "flex-1"
        }, null, 8, ["label", "name", "value", "validation", "help"]);
      };
    }
  }), _hoisted_1$f = { class: "pr-2" }, _sfc_main$n = /* @__PURE__ */ defineComponent({
    __name: "RangeField",
    props: {
      name: {},
      min: {},
      max: {},
      step: {},
      initial: {},
      label: {},
      validation: {},
      help: {}
    },
    setup(O) {
      const H = ref$1(O.initial);
      return (V, q) => {
        const G = resolveComponent("form-kit");
        return openBlock(), createBlock(G, {
          type: "range",
          label: V.label || V.name,
          name: V.name,
          modelValue: H.value,
          "onUpdate:modelValue": q[0] || (q[0] = (J) => H.value = J),
          min: V.min,
          max: V.max,
          step: V.step,
          validation: V.validation,
          help: V.help,
          "data-cy": "range-field",
          "outer-class": "flex-1"
        }, {
          prefix: withCtx(() => [
            createBaseVNode("div", _hoisted_1$f, toDisplayString(H.value), 1)
          ]),
          _: 1
        }, 8, ["label", "name", "modelValue", "min", "max", "step", "validation", "help"]);
      };
    }
  }), _hoisted_1$e = { "data-cy": "select-field" }, _sfc_main$m = /* @__PURE__ */ defineComponent({
    __name: "SelectField",
    props: {
      name: {},
      options: {},
      initial: {},
      label: {},
      required: { type: Boolean },
      validation: {},
      help: {}
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (q) => q !== void 0
      ).join("|"));
      return (V, q) => {
        const G = resolveComponent("form-kit");
        return openBlock(), createElementBlock("span", _hoisted_1$e, [
          createVNode(G, {
            type: "select",
            label: V.label || V.name,
            name: V.name,
            options: V.options,
            value: V.initial,
            validation: H.value,
            help: V.help,
            "data-cy": "select-field"
          }, null, 8, ["label", "name", "options", "value", "validation", "help"])
        ]);
      };
    }
  }), _sfc_main$l = /* @__PURE__ */ defineComponent({
    __name: "SwitchField",
    props: {
      initial: { type: Boolean },
      name: {},
      label: {},
      help: {}
    },
    setup(O) {
      return (_, H) => {
        const V = resolveComponent("form-kit");
        return openBlock(), createElementBlock("span", null, [
          createVNode(V, {
            type: "checkbox",
            value: _.initial,
            label: _.label || _.name,
            name: _.name,
            help: _.help,
            "data-cy": "switch-field"
          }, null, 8, ["value", "label", "name", "help"])
        ]);
      };
    }
  }), _hoisted_1$d = {
    class: "mb-6",
    "data-cy": "tags-field"
  }, _sfc_main$k = /* @__PURE__ */ defineComponent({
    __name: "TagsField",
    props: {
      initial: {},
      label: {},
      required: { type: Boolean },
      name: {},
      validation: {},
      help: {}
    },
    setup(O) {
      const _ = O, H = computed(() => [_.required ? "required" : void 0, _.validation].filter(
        (G) => G !== void 0
      ).join("|")), V = {
        taggable: !0,
        placeholder: "Search or add a tag"
      };
      return (q, G) => {
        const J = resolveComponent("form-kit");
        return openBlock(), createElementBlock("div", _hoisted_1$d, [
          createVNode(J, {
            type: "multiSelectField",
            label: q.label || q.name,
            name: q.name,
            validation: H.value,
            help: q.help,
            multiSelectProps: V,
            "validation-visibility": "live",
            tags: q.initial,
            options: q.initial
          }, null, 8, ["label", "name", "validation", "help", "tags", "options"])
        ]);
      };
    }
  }), _sfc_main$j = /* @__PURE__ */ defineComponent({
    __name: "TextBox",
    props: {
      name: {},
      label: {},
      initial: {},
      required: { type: Boolean },
      type: {},
      validation: {},
      help: {}
    },
    setup(O) {
      const _ = O, H = /* @__PURE__ */ new Map([
        ["url", "url"],
        ["email", "email"],
        ["search", "search"],
        ["textarea", "textarea"]
      ]), V = computed(() => [
        _.required ? "required" : void 0,
        H.get(_.type),
        _.validation
      ].filter((G) => G !== void 0).join("|"));
      return (q, G) => {
        const J = resolveComponent("form-kit");
        return openBlock(), createBlock(J, {
          type: q.type,
          label: q.label || q.name,
          name: q.name,
          value: q.initial,
          validation: V.value,
          help: q.help,
          "validation-visibility": "live",
          "outer-class": "flex-1"
        }, null, 8, ["type", "label", "name", "value", "validation", "help"]);
      };
    }
  });
  class ControlsField extends Block$2 {
    constructor(_, H) {
      super(_, H);
      const { name: V, required: q, initial: G, label: J, validation: Z, help: te } = _;
      this.componentProps = {
        ...this.componentProps,
        name: V,
        label: J,
        initial: G,
        required: q,
        validation: Z,
        help: te
      };
    }
  }
  class RangeField extends ControlsField {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$n));
      const { min: q, max: G, step: J, initial: Z } = H;
      this.componentProps = {
        ...this.componentProps,
        min: +q,
        max: +G,
        step: +J,
        initial: +Z
      };
    }
  }
  class PasswordField extends ControlsField {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$o));
    }
  }
  class TemporalTextBox extends ControlsField {
    constructor(H, V, q) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$j));
      const { type: G } = q;
      this.componentProps = {
        ...this.componentProps,
        type: G
      };
    }
  }
  class ColorField extends ControlsField {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$u));
    }
  }
  class NumberBox extends ControlsField {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$p));
      const { initial: q } = H;
      this.componentProps = {
        ...this.componentProps,
        initial: q
      };
    }
  }
  class HiddenField extends ControlsField {
    constructor() {
      super(...arguments);
      mt(this, "component", markRaw(_sfc_main$r));
    }
  }
  class TagsField extends ControlsField {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$k));
      const { initial: q } = H;
      this.componentProps = {
        ...this.componentProps,
        initial: q
      };
    }
  }
  class MultiChoiceField extends ControlsField {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$q));
      const { initial: q, options: G } = H;
      this.componentProps = {
        ...this.componentProps,
        options: G,
        initial: q
      };
    }
  }
  class FileField extends ControlsField {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$s));
      const { accept: q } = H;
      this.componentProps = {
        ...this.componentProps,
        accept: q
      };
    }
  }
  class TemporalDateTimeField extends ControlsField {
    constructor(H, V, q) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$t));
      const { initial: G } = H, { timeFormat: J, type: Z, parseFormat: te } = q;
      this.componentProps = {
        ...this.componentProps,
        // initial may be undefined -> moment() gives us current datetime
        // parseFormat may be undefined -> moment does automatic datetime parsing
        initial: (G ? hooks(G, te) : hooks()).format(
          J
        ),
        type: Z
      };
    }
  }
  class SelectField extends ControlsField {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$m));
      mt(this, "options");
      const { initial: q, options: G } = H;
      this.options = G, this.componentProps = {
        ...this.componentProps,
        options: this.options,
        initial: q || this.options[0]
      };
    }
  }
  class SwitchField extends ControlsField {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$l));
      const { initial: q } = H;
      this.componentProps = {
        ...this.componentProps,
        initial: q
      };
    }
  }
  const _sfc_main$i = /* @__PURE__ */ defineComponent({
    __name: "DataTableConnector",
    props: {
      streamContents: { type: Function },
      deferLoad: { type: Boolean },
      cells: {},
      refId: {},
      getCsvText: { type: Function },
      downloadLocal: { type: Function },
      figure: {},
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = /* @__PURE__ */ defineAsyncComponent(() => Promise.resolve().then(() => DataTable)), H = O, V = useRootStore(), { singleBlockEmbed: q } = storeToRefs(V), G = ref$1(H.deferLoad), J = ref$1([]), Z = ref$1({}), te = async () => {
        try {
          const ee = await H.streamContents();
          if (ee) {
            const { schema: ae, data: oe } = ee;
            J.value = J.value.concat(oe), Z.value = ae;
          }
        } catch (ee) {
          console.error("An error occurred downloading your dataset data: " + ee);
        }
      };
      H.deferLoad || te();
      const ne = async () => {
        await te(), J.value.length && (G.value = !1);
      };
      return (ee, ae) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: ee.figure,
        "single-block-embed": ee.singleBlockEmbed || unref(q)
      }, {
        default: withCtx(() => [
          createVNode(unref(_), {
            singleBlockEmbed: !!ee.singleBlockEmbed,
            data: J.value,
            cells: ee.cells,
            schema: Z.value,
            previewMode: G.value,
            getCsvText: ee.getCsvText,
            downloadLocal: ee.downloadLocal,
            refId: ee.refId,
            onLoadFull: ne
          }, null, 8, ["singleBlockEmbed", "data", "cells", "schema", "previewMode", "getCsvText", "downloadLocal", "refId"])
        ]),
        _: 1
      }, 8, ["figure", "single-block-embed"]));
    }
  }), AUTO_LOAD_CELLS_LIMIT = 5e5;
  class DataTableBlock extends AssetBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$i));
      mt(this, "rows");
      mt(this, "columns");
      mt(this, "_revogridExportPlugin");
      mt(this, "streamContents", async () => {
        const { apiResponseToArrow: H } = await Promise.resolve().then(() => arrowUtils), V = await this.fetchDataset();
        return H(V);
      });
      mt(this, "downloadLocal", async () => {
        try {
          (await this.getExportPlugin()).exportFile({
            filename: `ar-export-${this.refId}`
          });
        } catch (H) {
          console.error("An error occurred while exporting dataset: ", H);
        }
      });
      mt(this, "getCsvText", async () => {
        let H = "";
        try {
          H = (await this.getExportPlugin()).exportString();
        } catch (V) {
          console.error("An error occurred while copying text to clipboard: ", V);
        }
        return H;
      });
      const { rows: q, columns: G } = H;
      this.rows = q, this.columns = G, this.componentProps = {
        ...this.componentProps,
        streamContents: this.streamContents,
        getCsvText: this.getCsvText,
        downloadLocal: this.downloadLocal,
        deferLoad: this.deferLoad,
        cells: this.cells,
        refId: this.refId
      };
    }
    get cells() {
      return this.rows * this.columns;
    }
    get deferLoad() {
      return this.cells > AUTO_LOAD_CELLS_LIMIT;
    }
    get exportUrl() {
      return this.buildExtensionUrl("export");
    }
    fetchDataset() {
      return fetch(this.src).then((H) => {
        if (!H.ok)
          throw new Error("Failed to fetch dataset");
        return H.arrayBuffer();
      });
    }
    async getExportPlugin() {
      if (this._revogridExportPlugin)
        return this._revogridExportPlugin;
      const H = document.getElementById(
        `grid-${this.refId}`
      );
      if (H) {
        const V = await H.getPlugins();
        for (const q of V)
          if (q.exportFile && q.exportString)
            return this._revogridExportPlugin = q, this._revogridExportPlugin;
      } else
        throw "Grid element not found";
    }
    buildExtensionUrl(H) {
      return new URL(`extensions/${H}/`).toString();
    }
  }
  mt(DataTableBlock, "captionType", "Table");
  const _hoisted_1$c = { class: "border shadow-sm rounded-md w-full" }, _hoisted_2$8 = { class: "px-4 py-5 sm:p-6" }, _hoisted_3$7 = { class: "mb-6" }, _hoisted_4$5 = { key: 0 }, _hoisted_5$4 = { class: "text-lg font-medium leading-6 text-gray-900" }, _hoisted_6$1 = { key: 1 }, _hoisted_7$1 = { class: "mt-1 max-w-2xl text-sm text-gray-500" }, _hoisted_8$1 = { class: "px-4 py-3 sm:px-6 items-center justify-start" }, _sfc_main$h = /* @__PURE__ */ defineComponent({
    __name: "Compute",
    props: {
      children: {},
      prompt: {},
      label: {},
      method: {},
      subtitle: {},
      action: {}
    },
    setup(O) {
      const _ = O, H = v4(), V = (q) => q instanceof ControlsField;
      return (q, G) => {
        const J = resolveComponent("form-kit");
        return openBlock(), createElementBlock("div", _hoisted_1$c, [
          createVNode(J, {
            type: "form",
            id: unref(H),
            action: q.action,
            method: q.method,
            actions: !1
          }, {
            default: withCtx(() => [
              createBaseVNode("div", _hoisted_2$8, [
                createBaseVNode("div", _hoisted_3$7, [
                  q.label ? (openBlock(), createElementBlock("div", _hoisted_4$5, [
                    createBaseVNode("h3", _hoisted_5$4, toDisplayString(q.label), 1)
                  ])) : createCommentVNode("", !0),
                  q.subtitle ? (openBlock(), createElementBlock("div", _hoisted_6$1, [
                    createBaseVNode("p", _hoisted_7$1, toDisplayString(q.subtitle), 1)
                  ])) : createCommentVNode("", !0)
                ]),
                (openBlock(!0), createElementBlock(Fragment, null, renderList(q.children, (Z) => (openBlock(), createBlock(resolveDynamicComponent(Z.component), mergeProps$1({ ref_for: !0 }, Z.componentProps, {
                  key: Z.refId,
                  class: { "py-4": !V(Z) }
                }), null, 16, ["class"]))), 128))
              ]),
              createBaseVNode("div", _hoisted_8$1, [
                createVNode(J, {
                  type: "submit",
                  class: normalizeClass([
                    "ml-auto inline-flex justify-center rounded-md border border-transparent bg-indigo-600 py-2 px-4 text-sm font-medium text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2 disabled:opacity-50"
                  ])
                }, {
                  default: withCtx(() => [
                    createTextVNode(toDisplayString(_.prompt), 1)
                  ]),
                  _: 1
                }),
                createBaseVNode("div", null, [
                  createVNode(unref(FormKitMessages))
                ])
              ])
            ]),
            _: 1
          }, 8, ["id", "action", "method"])
        ]);
      };
    }
  }), _sfc_main$g = /* @__PURE__ */ defineComponent({
    __name: "ComputeConnector",
    props: {
      children: {},
      prompt: {},
      label: {},
      figure: {},
      subtitle: {},
      action: {},
      method: {}
    },
    setup(O) {
      return (_, H) => (openBlock(), createBlock(_sfc_main$Q, {
        figure: _.figure,
        "show-overflow": !0
      }, {
        default: withCtx(() => [
          createVNode(_sfc_main$h, {
            children: _.children,
            subtitle: _.subtitle,
            label: _.label,
            prompt: _.prompt,
            action: _.action,
            method: _.method
          }, null, 8, ["children", "subtitle", "label", "prompt", "action", "method"])
        ]),
        _: 1
      }, 8, ["figure"]));
    }
  });
  var SwapType = /* @__PURE__ */ ((O) => (O.REPLACE = "replace", O.INNER = "inner", O.APPEND = "append", O.PREPEND = "prepend", O))(SwapType || {}), VAlign$1 = /* @__PURE__ */ ((O) => (O.TOP = "top", O.CENTER = "center", O.BOTTOM = "bottom", O))(VAlign$1 || {});
  const _sfc_main$f = /* @__PURE__ */ defineComponent({
    __name: "Group",
    props: {
      columns: {},
      widths: {},
      store: {},
      valign: {}
    },
    setup(O) {
      const _ = O, H = useRootStore(), { singleBlockEmbed: V } = storeToRefs(H), { children: q } = storeToRefs(_.store), G = computed(() => {
        switch (_.valign) {
          default:
          case VAlign$1.TOP:
            return "start";
          case VAlign$1.CENTER:
            return "center";
          case VAlign$1.BOTTOM:
            return "end";
        }
      }), J = computed(() => {
        if (_.columns !== 0)
          return _.widths ? _.widths.map((Z) => `${Z}fr`).join(" ") : `repeat(${_.columns}, minmax(0, 1fr))`;
      });
      return (Z, te) => (openBlock(), createElementBlock("div", {
        class: normalizeClass([
          "w-full sm:grid sm:gap-4",
          {
            "grid-flow-col grid-cols-fit": !Z.columns,
            "py-4 px-1": !unref(V)
          }
        ]),
        style: normalizeStyle({ gridTemplateColumns: J.value, alignItems: G.value })
      }, [
        (openBlock(!0), createElementBlock(Fragment, null, renderList(unref(q), (ne) => (openBlock(), createBlock(resolveDynamicComponent(ne.component), mergeProps$1({ ref_for: !0 }, ne.componentProps, {
          key: ne.refId
        }), null, 16))), 128))
      ], 6));
    }
  }), _hoisted_1$b = {
    class: "w-full",
    "data-cy": "section-tabs"
  }, _hoisted_2$7 = ["value"], _hoisted_3$6 = ["value"], _hoisted_4$4 = { class: "flex space-x-4 mb-2 border-b-2 border-gray-200" }, _hoisted_5$3 = ["data-cy", "onClick"], _sfc_main$e = /* @__PURE__ */ defineComponent({
    __name: "SelectBlock",
    props: {
      type: {},
      store: {}
    },
    setup(O) {
      const _ = O, {
        children: H,
        tabNumber: V
      } = storeToRefs(_.store), q = computed(() => _.type ? _.type : H.value.length < 5 ? "tabs" : "dropdown"), G = computed(
        () => H.value.map((ee, ae) => ee.label || `Section ${ae + 1}`)
      ), J = computed(
        () => G.value.map((ee, ae) => ae)
      ), Z = (ee) => G.value[ee], te = (ee) => _.store.setTab(ee), ne = (ee) => void te(+ee.target.value);
      return (ee, ae) => (openBlock(), createElementBlock("div", _hoisted_1$b, [
        createBaseVNode("div", {
          class: normalizeClass(["w-full mb-2", { "sm:hidden": q.value === "tabs" }])
        }, [
          unref(H).length < 10 ? (openBlock(), createElementBlock("select", {
            key: 0,
            id: "tabs",
            name: "tabs",
            class: "block mb-1 w-auto focus:ring-indigo-500 focus:border-indigo-500 border-gray-300 rounded-md",
            value: unref(V),
            onChange: ne
          }, [
            (openBlock(!0), createElementBlock(Fragment, null, renderList(G.value, (oe, se) => (openBlock(), createElementBlock("option", {
              key: se,
              value: se
            }, toDisplayString(oe), 9, _hoisted_3$6))), 128))
          ], 40, _hoisted_2$7)) : (openBlock(), createBlock(unref(script$2), {
            key: 1,
            modelValue: unref(V),
            "onUpdate:modelValue": ae[0] || (ae[0] = (oe) => isRef(V) ? V.value = oe : null),
            options: J.value,
            "preselect-first": !0,
            "clear-on-select": !1,
            "allow-empty": !1,
            "custom-label": Z
          }, null, 8, ["modelValue", "options"]))
        ], 2),
        createBaseVNode("div", {
          class: normalizeClass(["hidden", { "sm:block": q.value === "tabs" }])
        }, [
          createBaseVNode("nav", _hoisted_4$4, [
            (openBlock(!0), createElementBlock(Fragment, null, renderList(unref(H), (oe, se) => (openBlock(), createElementBlock("a", {
              role: "button",
              key: se,
              "data-cy": `tab-${se}`,
              class: normalizeClass([
                "px-3 py-2 font-medium text-sm rounded-md rounded-b-none",
                {
                  "text-ar-accent bg-ar-accent-light": unref(V) === se,
                  "text-ar-light-gray hover:text-ar-dark-gray": unref(V) !== se
                }
              ]),
              onClick: () => te(se)
            }, toDisplayString(G.value[se]), 11, _hoisted_5$3))), 128))
          ])
        ], 2),
        createBaseVNode("div", null, [
          (openBlock(), createBlock(resolveDynamicComponent(unref(H)[unref(V)].component), mergeProps$1(unref(H)[unref(V)].componentProps, {
            key: unref(H)[unref(V)].refId,
            class: "py-4"
          }), null, 16))
        ])
      ]));
    }
  }), _hoisted_1$a = {
    class: "w-full bg-gray-100 p-2 rounded-md",
    "data-cy": "section-toggle"
  }, _hoisted_2$6 = { class: "flex-1" }, _hoisted_3$5 = { class: "flex-initial ml-auto" }, _sfc_main$d = /* @__PURE__ */ defineComponent({
    __name: "Toggle",
    props: {
      label: {},
      store: {}
    },
    setup(O) {
      const _ = O, H = ref$1(!1), { children: V } = storeToRefs(_.store);
      return (q, G) => (openBlock(), createElementBlock("div", _hoisted_1$a, [
        createBaseVNode("div", {
          class: "cursor-pointer px-2 w-full font-semibold flex items-center justify-start",
          onClick: G[0] || (G[0] = (J) => H.value = !H.value)
        }, [
          createBaseVNode("div", _hoisted_2$6, toDisplayString(q.label || "Toggle Section"), 1),
          createBaseVNode("div", _hoisted_3$5, [
            createBaseVNode("i", {
              class: normalizeClass([
                "opacity-50 fa",
                { "fa-chevron-up": H.value, "fa-chevron-down": !H.value }
              ])
            }, null, 2)
          ])
        ]),
        createBaseVNode("div", {
          class: normalizeClass(["w-full pt-2 m-auto overflow-hidden", { hidden: !H.value }])
        }, [
          (openBlock(!0), createElementBlock(Fragment, null, renderList(unref(V), (J) => (openBlock(), createBlock(resolveDynamicComponent(J.component), mergeProps$1({ ref_for: !0 }, J.componentProps, {
            key: J.refId
          }), null, 16))), 128))
        ], 2)
      ]));
    }
  }), useActions = (O) => {
    const _ = reactive(O), H = ref$1(0);
    function V(ne) {
      this.$patch(() => {
        _.unshift(...ne.children), H.value += ne.children.length;
      });
    }
    function q(ne) {
      this.$patch(() => {
        _.push(...ne.children), H.value -= Math.min(ne.children.length, 0);
      });
    }
    function G(ne, ee) {
      this.$patch(() => {
        ee.id || (ee.id = _[ne].id), _.splice(ne, 1, ee);
      });
    }
    function J(ne, ee) {
      const ae = _[ne];
      if (!isLayoutBlock(ae))
        throw new Error("Can't perform inner replace on non layout block");
      this.$patch(() => {
        ae.children.splice(
          0,
          ae.children.length,
          ...ee.children
        ), H.value = 0;
      });
    }
    function Z(ne) {
      this.tabNumber = ne;
    }
    function te(ne) {
      this.children = ne;
    }
    return {
      children: _,
      tabNumber: H,
      prepend: V,
      append: q,
      replace: G,
      setTab: Z,
      load: te,
      inner: J
    };
  }, useLayoutStore = (O) => /* @__PURE__ */ defineStore(`layout-${v4()}`, () => useActions(O)), useViewStore = (O, _) => /* @__PURE__ */ defineStore(`view-${v4()}`, () => {
    const H = useActions(O), { children: V } = H, q = ref$1(_), G = computed(
      () => V.length === 1 && V[0] instanceof Select$1
    ), J = computed(
      () => q.value || (V.length > 5 ? "side" : "top")
    );
    return { ...H, hasPages: G, layout: J };
  });
  class ParentBlock extends Block$2 {
    constructor(H, V) {
      super(H, V);
      /**
       * A non-atomic block that has children
       */
      mt(this, "children");
      const { blocks: q } = H;
      this.children = q;
    }
  }
  class LayoutBlock extends ParentBlock {
    constructor(H, V) {
      super(H, V);
      /**
       * A non-atomic block which uses children to control layout, e.g. in columns, selects, pages
       */
      mt(this, "store");
      this.store = useLayoutStore(this.children)(), this.componentProps = { ...this.componentProps, store: this.store };
    }
    update(H, V, q) {
      return this.id === H && (q === SwapType.APPEND || q === SwapType.PREPEND) ? (this.insertAtEdge(V, q), !0) : q === SwapType.REPLACE || q === SwapType.INNER ? this.swap(V, H, q) : !1;
    }
    swap(H, V, q) {
      for (const [G, J] of this.children.entries())
        if (J.id === V) {
          if (q === SwapType.REPLACE)
            this.store.replace(G, H);
          else if (q === SwapType.INNER)
            this.store.inner(G, H);
          else
            throw new Error(`Method ${q} not recognized`);
          return !0;
        }
      return !1;
    }
    insertAtEdge(H, V) {
      V === SwapType.APPEND ? this.store.append(H) : V === SwapType.PREPEND && this.store.prepend(H);
    }
  }
  let Group$1 = class extends LayoutBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$f));
      mt(this, "name", "Group");
      const { columns: q, widths: G, valign: J } = H;
      this.componentProps = {
        ...this.componentProps,
        widths: G || void 0,
        columns: +q,
        valign: J
      };
    }
  }, Select$1 = class extends LayoutBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$e));
      mt(this, "type");
      mt(this, "name", "Select");
      const { label: q, type: G } = H;
      this.label = q || void 0, this.type = G, this.componentProps = { ...this.componentProps, type: G };
    }
  }, Toggle$1 = class extends LayoutBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$d));
      mt(this, "name", "Toggle");
      const { blocks: q, label: G } = H;
      this.children = q, this.label = G || void 0, this.componentProps = { ...this.componentProps, label: G };
    }
  };
  class ComputeBlock extends ParentBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "component", markRaw(_sfc_main$g));
      mt(this, "name", "Compute");
      const { prompt: q, label: G, subtitle: J, action: Z, method: te } = H;
      this.componentProps = {
        ...this.componentProps,
        children: this.children,
        prompt: q || "Submit",
        label: G,
        subtitle: J,
        action: Z,
        method: te
      };
    }
  }
  let View$3 = class extends LayoutBlock {
    constructor(H, V) {
      super(H, V);
      mt(this, "id", "root");
      mt(this, "name", "View");
      const { fragment: q } = H;
      this.store = q ? void 0 : useViewStore(this.children, void 0)(), this.componentProps = { ...this.componentProps, store: this.store };
    }
  };
  const isComputeElem = (O) => O._type === "Compute", isGroupElem = (O) => O._type === "Group", isSelectElem = (O) => O._type === "Select", isToggleElem = (O) => O._type === "Toggle", isViewElem = (O) => O._type === "View", isParentElem = (O) => isSelectElem(O) || isToggleElem(O) || isViewElem(O) || isGroupElem(O) || isComputeElem(O), isLayoutBlock = (O) => O instanceof LayoutBlock, isView = (O) => O instanceof View$3, parseError = (O) => O instanceof Error ? O.toString() : typeof O == "string" ? O : "Unable to parse error; check console for more information", { min: min$g, max: max$o } = Math, limit$1 = (O, _ = 0, H = 1) => min$g(max$o(_, O), H), clip_rgb = (O) => {
    O._clipped = !1, O._unclipped = O.slice(0);
    for (let _ = 0; _ <= 3; _++)
      _ < 3 ? ((O[_] < 0 || O[_] > 255) && (O._clipped = !0), O[_] = limit$1(O[_], 0, 255)) : _ === 3 && (O[_] = limit$1(O[_], 0, 1));
    return O;
  }, classToType = {};
  for (let O of [
    "Boolean",
    "Number",
    "String",
    "Function",
    "Array",
    "Date",
    "RegExp",
    "Undefined",
    "Null"
  ])
    classToType[`[object ${O}]`] = O.toLowerCase();
  function type$2(O) {
    return classToType[Object.prototype.toString.call(O)] || "object";
  }
  const unpack = (O, _ = null) => O.length >= 3 ? Array.prototype.slice.call(O) : type$2(O[0]) == "object" && _ ? _.split("").filter((H) => O[0][H] !== void 0).map((H) => O[0][H]) : O[0].slice(0), last = (O) => {
    if (O.length < 2) return null;
    const _ = O.length - 1;
    return type$2(O[_]) == "string" ? O[_].toLowerCase() : null;
  }, { PI: PI$4, min: min$f, max: max$n } = Math, rnd2 = (O) => Math.round(O * 100) / 100, rnd3 = (O) => Math.round(O * 100) / 100, TWOPI = PI$4 * 2, PITHIRD = PI$4 / 3, DEG2RAD = PI$4 / 180, RAD2DEG = 180 / PI$4;
  function reverse3(O) {
    return [...O.slice(0, 3).reverse(), ...O.slice(3)];
  }
  const input$2 = {
    format: {},
    autodetect: []
  };
  let Color$2 = class {
    constructor(..._) {
      const H = this;
      if (type$2(_[0]) === "object" && _[0].constructor && _[0].constructor === this.constructor)
        return _[0];
      let V = last(_), q = !1;
      if (!V) {
        q = !0, input$2.sorted || (input$2.autodetect = input$2.autodetect.sort((G, J) => J.p - G.p), input$2.sorted = !0);
        for (let G of input$2.autodetect)
          if (V = G.test(..._), V) break;
      }
      if (input$2.format[V]) {
        const G = input$2.format[V].apply(
          null,
          q ? _ : _.slice(0, -1)
        );
        H._rgb = clip_rgb(G);
      } else
        throw new Error("unknown format: " + _);
      H._rgb.length === 3 && H._rgb.push(1);
    }
    toString() {
      return type$2(this.hex) == "function" ? this.hex() : `[${this._rgb.join(",")}]`;
    }
  };
  const version$7 = "3.1.2", chroma = (...O) => new Color$2(...O);
  chroma.version = version$7;
  const w3cx11 = {
    aliceblue: "#f0f8ff",
    antiquewhite: "#faebd7",
    aqua: "#00ffff",
    aquamarine: "#7fffd4",
    azure: "#f0ffff",
    beige: "#f5f5dc",
    bisque: "#ffe4c4",
    black: "#000000",
    blanchedalmond: "#ffebcd",
    blue: "#0000ff",
    blueviolet: "#8a2be2",
    brown: "#a52a2a",
    burlywood: "#deb887",
    cadetblue: "#5f9ea0",
    chartreuse: "#7fff00",
    chocolate: "#d2691e",
    coral: "#ff7f50",
    cornflowerblue: "#6495ed",
    cornsilk: "#fff8dc",
    crimson: "#dc143c",
    cyan: "#00ffff",
    darkblue: "#00008b",
    darkcyan: "#008b8b",
    darkgoldenrod: "#b8860b",
    darkgray: "#a9a9a9",
    darkgreen: "#006400",
    darkgrey: "#a9a9a9",
    darkkhaki: "#bdb76b",
    darkmagenta: "#8b008b",
    darkolivegreen: "#556b2f",
    darkorange: "#ff8c00",
    darkorchid: "#9932cc",
    darkred: "#8b0000",
    darksalmon: "#e9967a",
    darkseagreen: "#8fbc8f",
    darkslateblue: "#483d8b",
    darkslategray: "#2f4f4f",
    darkslategrey: "#2f4f4f",
    darkturquoise: "#00ced1",
    darkviolet: "#9400d3",
    deeppink: "#ff1493",
    deepskyblue: "#00bfff",
    dimgray: "#696969",
    dimgrey: "#696969",
    dodgerblue: "#1e90ff",
    firebrick: "#b22222",
    floralwhite: "#fffaf0",
    forestgreen: "#228b22",
    fuchsia: "#ff00ff",
    gainsboro: "#dcdcdc",
    ghostwhite: "#f8f8ff",
    gold: "#ffd700",
    goldenrod: "#daa520",
    gray: "#808080",
    green: "#008000",
    greenyellow: "#adff2f",
    grey: "#808080",
    honeydew: "#f0fff0",
    hotpink: "#ff69b4",
    indianred: "#cd5c5c",
    indigo: "#4b0082",
    ivory: "#fffff0",
    khaki: "#f0e68c",
    laserlemon: "#ffff54",
    lavender: "#e6e6fa",
    lavenderblush: "#fff0f5",
    lawngreen: "#7cfc00",
    lemonchiffon: "#fffacd",
    lightblue: "#add8e6",
    lightcoral: "#f08080",
    lightcyan: "#e0ffff",
    lightgoldenrod: "#fafad2",
    lightgoldenrodyellow: "#fafad2",
    lightgray: "#d3d3d3",
    lightgreen: "#90ee90",
    lightgrey: "#d3d3d3",
    lightpink: "#ffb6c1",
    lightsalmon: "#ffa07a",
    lightseagreen: "#20b2aa",
    lightskyblue: "#87cefa",
    lightslategray: "#778899",
    lightslategrey: "#778899",
    lightsteelblue: "#b0c4de",
    lightyellow: "#ffffe0",
    lime: "#00ff00",
    limegreen: "#32cd32",
    linen: "#faf0e6",
    magenta: "#ff00ff",
    maroon: "#800000",
    maroon2: "#7f0000",
    maroon3: "#b03060",
    mediumaquamarine: "#66cdaa",
    mediumblue: "#0000cd",
    mediumorchid: "#ba55d3",
    mediumpurple: "#9370db",
    mediumseagreen: "#3cb371",
    mediumslateblue: "#7b68ee",
    mediumspringgreen: "#00fa9a",
    mediumturquoise: "#48d1cc",
    mediumvioletred: "#c71585",
    midnightblue: "#191970",
    mintcream: "#f5fffa",
    mistyrose: "#ffe4e1",
    moccasin: "#ffe4b5",
    navajowhite: "#ffdead",
    navy: "#000080",
    oldlace: "#fdf5e6",
    olive: "#808000",
    olivedrab: "#6b8e23",
    orange: "#ffa500",
    orangered: "#ff4500",
    orchid: "#da70d6",
    palegoldenrod: "#eee8aa",
    palegreen: "#98fb98",
    paleturquoise: "#afeeee",
    palevioletred: "#db7093",
    papayawhip: "#ffefd5",
    peachpuff: "#ffdab9",
    peru: "#cd853f",
    pink: "#ffc0cb",
    plum: "#dda0dd",
    powderblue: "#b0e0e6",
    purple: "#800080",
    purple2: "#7f007f",
    purple3: "#a020f0",
    rebeccapurple: "#663399",
    red: "#ff0000",
    rosybrown: "#bc8f8f",
    royalblue: "#4169e1",
    saddlebrown: "#8b4513",
    salmon: "#fa8072",
    sandybrown: "#f4a460",
    seagreen: "#2e8b57",
    seashell: "#fff5ee",
    sienna: "#a0522d",
    silver: "#c0c0c0",
    skyblue: "#87ceeb",
    slateblue: "#6a5acd",
    slategray: "#708090",
    slategrey: "#708090",
    snow: "#fffafa",
    springgreen: "#00ff7f",
    steelblue: "#4682b4",
    tan: "#d2b48c",
    teal: "#008080",
    thistle: "#d8bfd8",
    tomato: "#ff6347",
    turquoise: "#40e0d0",
    violet: "#ee82ee",
    wheat: "#f5deb3",
    white: "#ffffff",
    whitesmoke: "#f5f5f5",
    yellow: "#ffff00",
    yellowgreen: "#9acd32"
  }, RE_HEX = /^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/, RE_HEXA = /^#?([A-Fa-f0-9]{8}|[A-Fa-f0-9]{4})$/, hex2rgb = (O) => {
    if (O.match(RE_HEX)) {
      (O.length === 4 || O.length === 7) && (O = O.substr(1)), O.length === 3 && (O = O.split(""), O = O[0] + O[0] + O[1] + O[1] + O[2] + O[2]);
      const _ = parseInt(O, 16), H = _ >> 16, V = _ >> 8 & 255, q = _ & 255;
      return [H, V, q, 1];
    }
    if (O.match(RE_HEXA)) {
      (O.length === 5 || O.length === 9) && (O = O.substr(1)), O.length === 4 && (O = O.split(""), O = O[0] + O[0] + O[1] + O[1] + O[2] + O[2] + O[3] + O[3]);
      const _ = parseInt(O, 16), H = _ >> 24 & 255, V = _ >> 16 & 255, q = _ >> 8 & 255, G = Math.round((_ & 255) / 255 * 100) / 100;
      return [H, V, q, G];
    }
    throw new Error(`unknown hex color: ${O}`);
  }, { round: round$d } = Math, rgb2hex = (...O) => {
    let [_, H, V, q] = unpack(O, "rgba"), G = last(O) || "auto";
    q === void 0 && (q = 1), G === "auto" && (G = q < 1 ? "rgba" : "rgb"), _ = round$d(_), H = round$d(H), V = round$d(V);
    let Z = "000000" + (_ << 16 | H << 8 | V).toString(16);
    Z = Z.substr(Z.length - 6);
    let te = "0" + round$d(q * 255).toString(16);
    switch (te = te.substr(te.length - 2), G.toLowerCase()) {
      case "rgba":
        return `#${Z}${te}`;
      case "argb":
        return `#${te}${Z}`;
      default:
        return `#${Z}`;
    }
  };
  Color$2.prototype.name = function() {
    const O = rgb2hex(this._rgb, "rgb");
    for (let _ of Object.keys(w3cx11))
      if (w3cx11[_] === O) return _.toLowerCase();
    return O;
  };
  input$2.format.named = (O) => {
    if (O = O.toLowerCase(), w3cx11[O]) return hex2rgb(w3cx11[O]);
    throw new Error("unknown color name: " + O);
  };
  input$2.autodetect.push({
    p: 5,
    test: (O, ..._) => {
      if (!_.length && type$2(O) === "string" && w3cx11[O.toLowerCase()])
        return "named";
    }
  });
  Color$2.prototype.alpha = function(O, _ = !1) {
    return O !== void 0 && type$2(O) === "number" ? _ ? (this._rgb[3] = O, this) : new Color$2([this._rgb[0], this._rgb[1], this._rgb[2], O], "rgb") : this._rgb[3];
  };
  Color$2.prototype.clipped = function() {
    return this._rgb._clipped || !1;
  };
  const labConstants = {
    // Corresponds roughly to RGB brighter/darker
    Kn: 18,
    // D65 standard referent
    labWhitePoint: "d65",
    Xn: 0.95047,
    Yn: 1,
    Zn: 1.08883,
    t0: 0.137931034,
    // 4 / 29
    t1: 0.206896552,
    // 6 / 29
    t2: 0.12841855,
    // 3 * t1 * t1
    t3: 8856452e-9,
    // t1 * t1 * t1,
    kE: 216 / 24389,
    kKE: 8,
    kK: 24389 / 27,
    RefWhiteRGB: {
      // sRGB
      X: 0.95047,
      Y: 1,
      Z: 1.08883
    },
    MtxRGB2XYZ: {
      m00: 0.4124564390896922,
      m01: 0.21267285140562253,
      m02: 0.0193338955823293,
      m10: 0.357576077643909,
      m11: 0.715152155287818,
      m12: 0.11919202588130297,
      m20: 0.18043748326639894,
      m21: 0.07217499330655958,
      m22: 0.9503040785363679
    },
    MtxXYZ2RGB: {
      m00: 3.2404541621141045,
      m01: -0.9692660305051868,
      m02: 0.055643430959114726,
      m10: -1.5371385127977166,
      m11: 1.8760108454466942,
      m12: -0.2040259135167538,
      m20: -0.498531409556016,
      m21: 0.041556017530349834,
      m22: 1.0572251882231791
    },
    // used in rgb2xyz
    As: 0.9414285350000001,
    Bs: 1.040417467,
    Cs: 1.089532651,
    MtxAdaptMa: {
      m00: 0.8951,
      m01: -0.7502,
      m02: 0.0389,
      m10: 0.2664,
      m11: 1.7135,
      m12: -0.0685,
      m20: -0.1614,
      m21: 0.0367,
      m22: 1.0296
    },
    MtxAdaptMaI: {
      m00: 0.9869929054667123,
      m01: 0.43230526972339456,
      m02: -0.008528664575177328,
      m10: -0.14705425642099013,
      m11: 0.5183602715367776,
      m12: 0.04004282165408487,
      m20: 0.15996265166373125,
      m21: 0.0492912282128556,
      m22: 0.9684866957875502
    }
  }, ILLUMINANTS = /* @__PURE__ */ new Map([
    // ASTM E308-01
    ["a", [1.0985, 0.35585]],
    // Wyszecki & Stiles, p. 769
    ["b", [1.0985, 0.35585]],
    // C ASTM E308-01
    ["c", [0.98074, 1.18232]],
    // D50 (ASTM E308-01)
    ["d50", [0.96422, 0.82521]],
    // D55 (ASTM E308-01)
    ["d55", [0.95682, 0.92149]],
    // D65 (ASTM E308-01)
    ["d65", [0.95047, 1.08883]],
    // E (ASTM E308-01)
    ["e", [1, 1, 1]],
    // F2 (ASTM E308-01)
    ["f2", [0.99186, 0.67393]],
    // F7 (ASTM E308-01)
    ["f7", [0.95041, 1.08747]],
    // F11 (ASTM E308-01)
    ["f11", [1.00962, 0.6435]],
    ["icc", [0.96422, 0.82521]]
  ]);
  function setLabWhitePoint(O) {
    const _ = ILLUMINANTS.get(String(O).toLowerCase());
    if (!_)
      throw new Error("unknown Lab illuminant " + O);
    labConstants.labWhitePoint = O, labConstants.Xn = _[0], labConstants.Zn = _[1];
  }
  function getLabWhitePoint() {
    return labConstants.labWhitePoint;
  }
  const lab2rgb = (...O) => {
    O = unpack(O, "lab");
    const [_, H, V] = O, [q, G, J] = lab2xyz$1(_, H, V), [Z, te, ne] = xyz2rgb(q, G, J);
    return [Z, te, ne, O.length > 3 ? O[3] : 1];
  }, lab2xyz$1 = (O, _, H) => {
    const { kE: V, kK: q, kKE: G, Xn: J, Yn: Z, Zn: te } = labConstants, ne = (O + 16) / 116, ee = 2e-3 * _ + ne, ae = ne - 5e-3 * H, oe = ee * ee * ee, se = ae * ae * ae, ce = oe > V ? oe : (116 * ee - 16) / q, le = O > G ? Math.pow((O + 16) / 116, 3) : O / q, ue = se > V ? se : (116 * ae - 16) / q, de = ce * J, fe = le * Z, he = ue * te;
    return [de, fe, he];
  }, compand = (O) => {
    const _ = Math.sign(O);
    return O = Math.abs(O), (O <= 31308e-7 ? O * 12.92 : 1.055 * Math.pow(O, 1 / 2.4) - 0.055) * _;
  }, xyz2rgb = (O, _, H) => {
    const { MtxAdaptMa: V, MtxAdaptMaI: q, MtxXYZ2RGB: G, RefWhiteRGB: J, Xn: Z, Yn: te, Zn: ne } = labConstants, ee = Z * V.m00 + te * V.m10 + ne * V.m20, ae = Z * V.m01 + te * V.m11 + ne * V.m21, oe = Z * V.m02 + te * V.m12 + ne * V.m22, se = J.X * V.m00 + J.Y * V.m10 + J.Z * V.m20, ce = J.X * V.m01 + J.Y * V.m11 + J.Z * V.m21, le = J.X * V.m02 + J.Y * V.m12 + J.Z * V.m22, ue = (O * V.m00 + _ * V.m10 + H * V.m20) * (se / ee), de = (O * V.m01 + _ * V.m11 + H * V.m21) * (ce / ae), fe = (O * V.m02 + _ * V.m12 + H * V.m22) * (le / oe), he = ue * q.m00 + de * q.m10 + fe * q.m20, me = ue * q.m01 + de * q.m11 + fe * q.m21, pe = ue * q.m02 + de * q.m12 + fe * q.m22, ye = compand(
      he * G.m00 + me * G.m10 + pe * G.m20
    ), ge = compand(
      he * G.m01 + me * G.m11 + pe * G.m21
    ), ve = compand(
      he * G.m02 + me * G.m12 + pe * G.m22
    );
    return [ye * 255, ge * 255, ve * 255];
  }, rgb2lab = (...O) => {
    const [_, H, V, ...q] = unpack(O, "rgb"), [G, J, Z] = rgb2xyz(_, H, V), [te, ne, ee] = xyz2lab$1(G, J, Z);
    return [te, ne, ee, ...q.length > 0 && q[0] < 1 ? [q[0]] : []];
  };
  function xyz2lab$1(O, _, H) {
    const { Xn: V, Yn: q, Zn: G, kE: J, kK: Z } = labConstants, te = O / V, ne = _ / q, ee = H / G, ae = te > J ? Math.pow(te, 1 / 3) : (Z * te + 16) / 116, oe = ne > J ? Math.pow(ne, 1 / 3) : (Z * ne + 16) / 116, se = ee > J ? Math.pow(ee, 1 / 3) : (Z * ee + 16) / 116;
    return [116 * oe - 16, 500 * (ae - oe), 200 * (oe - se)];
  }
  function gammaAdjustSRGB(O) {
    const _ = Math.sign(O);
    return O = Math.abs(O), (O <= 0.04045 ? O / 12.92 : Math.pow((O + 0.055) / 1.055, 2.4)) * _;
  }
  const rgb2xyz = (O, _, H) => {
    O = gammaAdjustSRGB(O / 255), _ = gammaAdjustSRGB(_ / 255), H = gammaAdjustSRGB(H / 255);
    const { MtxRGB2XYZ: V, MtxAdaptMa: q, MtxAdaptMaI: G, Xn: J, Yn: Z, Zn: te, As: ne, Bs: ee, Cs: ae } = labConstants;
    let oe = O * V.m00 + _ * V.m10 + H * V.m20, se = O * V.m01 + _ * V.m11 + H * V.m21, ce = O * V.m02 + _ * V.m12 + H * V.m22;
    const le = J * q.m00 + Z * q.m10 + te * q.m20, ue = J * q.m01 + Z * q.m11 + te * q.m21, de = J * q.m02 + Z * q.m12 + te * q.m22;
    let fe = oe * q.m00 + se * q.m10 + ce * q.m20, he = oe * q.m01 + se * q.m11 + ce * q.m21, me = oe * q.m02 + se * q.m12 + ce * q.m22;
    return fe *= le / ne, he *= ue / ee, me *= de / ae, oe = fe * G.m00 + he * G.m10 + me * G.m20, se = fe * G.m01 + he * G.m11 + me * G.m21, ce = fe * G.m02 + he * G.m12 + me * G.m22, [oe, se, ce];
  };
  Color$2.prototype.lab = function() {
    return rgb2lab(this._rgb);
  };
  const lab$3 = (...O) => new Color$2(...O, "lab");
  Object.assign(chroma, { lab: lab$3, getLabWhitePoint, setLabWhitePoint });
  input$2.format.lab = lab2rgb;
  input$2.autodetect.push({
    p: 2,
    test: (...O) => {
      if (O = unpack(O, "lab"), type$2(O) === "array" && O.length === 3)
        return "lab";
    }
  });
  Color$2.prototype.darken = function(O = 1) {
    const _ = this, H = _.lab();
    return H[0] -= labConstants.Kn * O, new Color$2(H, "lab").alpha(_.alpha(), !0);
  };
  Color$2.prototype.brighten = function(O = 1) {
    return this.darken(-O);
  };
  Color$2.prototype.darker = Color$2.prototype.darken;
  Color$2.prototype.brighter = Color$2.prototype.brighten;
  Color$2.prototype.get = function(O) {
    const [_, H] = O.split("."), V = this[_]();
    if (H) {
      const q = _.indexOf(H) - (_.substr(0, 2) === "ok" ? 2 : 0);
      if (q > -1) return V[q];
      throw new Error(`unknown channel ${H} in mode ${_}`);
    } else
      return V;
  };
  const { pow: pow$b } = Math, EPS = 1e-7, MAX_ITER = 20;
  Color$2.prototype.luminance = function(O, _ = "rgb") {
    if (O !== void 0 && type$2(O) === "number") {
      if (O === 0)
        return new Color$2([0, 0, 0, this._rgb[3]], "rgb");
      if (O === 1)
        return new Color$2([255, 255, 255, this._rgb[3]], "rgb");
      let H = this.luminance(), V = MAX_ITER;
      const q = (J, Z) => {
        const te = J.interpolate(Z, 0.5, _), ne = te.luminance();
        return Math.abs(O - ne) < EPS || !V-- ? te : ne > O ? q(J, te) : q(te, Z);
      }, G = (H > O ? q(new Color$2([0, 0, 0]), this) : q(this, new Color$2([255, 255, 255]))).rgb();
      return new Color$2([...G, this._rgb[3]]);
    }
    return rgb2luminance(...this._rgb.slice(0, 3));
  };
  const rgb2luminance = (O, _, H) => (O = luminance_x(O), _ = luminance_x(_), H = luminance_x(H), 0.2126 * O + 0.7152 * _ + 0.0722 * H), luminance_x = (O) => (O /= 255, O <= 0.03928 ? O / 12.92 : pow$b((O + 0.055) / 1.055, 2.4)), index$5 = {}, mix = (O, _, H = 0.5, ...V) => {
    let q = V[0] || "lrgb";
    if (!index$5[q] && !V.length && (q = Object.keys(index$5)[0]), !index$5[q])
      throw new Error(`interpolation mode ${q} is not defined`);
    return type$2(O) !== "object" && (O = new Color$2(O)), type$2(_) !== "object" && (_ = new Color$2(_)), index$5[q](O, _, H).alpha(
      O.alpha() + H * (_.alpha() - O.alpha())
    );
  };
  Color$2.prototype.mix = Color$2.prototype.interpolate = function(O, _ = 0.5, ...H) {
    return mix(this, O, _, ...H);
  };
  Color$2.prototype.premultiply = function(O = !1) {
    const _ = this._rgb, H = _[3];
    return O ? (this._rgb = [_[0] * H, _[1] * H, _[2] * H, H], this) : new Color$2([_[0] * H, _[1] * H, _[2] * H, H], "rgb");
  };
  const { sin: sin$8, cos: cos$9 } = Math, lch2lab = (...O) => {
    let [_, H, V] = unpack(O, "lch");
    return isNaN(V) && (V = 0), V = V * DEG2RAD, [_, cos$9(V) * H, sin$8(V) * H];
  }, lch2rgb = (...O) => {
    O = unpack(O, "lch");
    const [_, H, V] = O, [q, G, J] = lch2lab(_, H, V), [Z, te, ne] = lab2rgb(q, G, J);
    return [Z, te, ne, O.length > 3 ? O[3] : 1];
  }, hcl2rgb = (...O) => {
    const _ = reverse3(unpack(O, "hcl"));
    return lch2rgb(..._);
  }, { sqrt: sqrt$c, atan2: atan2$5, round: round$c } = Math, lab2lch = (...O) => {
    const [_, H, V] = unpack(O, "lab"), q = sqrt$c(H * H + V * V);
    let G = (atan2$5(V, H) * RAD2DEG + 360) % 360;
    return round$c(q * 1e4) === 0 && (G = Number.NaN), [_, q, G];
  }, rgb2lch = (...O) => {
    const [_, H, V, ...q] = unpack(O, "rgb"), [G, J, Z] = rgb2lab(_, H, V), [te, ne, ee] = lab2lch(G, J, Z);
    return [te, ne, ee, ...q.length > 0 && q[0] < 1 ? [q[0]] : []];
  };
  Color$2.prototype.lch = function() {
    return rgb2lch(this._rgb);
  };
  Color$2.prototype.hcl = function() {
    return reverse3(rgb2lch(this._rgb));
  };
  const lch$1 = (...O) => new Color$2(...O, "lch"), hcl$3 = (...O) => new Color$2(...O, "hcl");
  Object.assign(chroma, { lch: lch$1, hcl: hcl$3 });
  input$2.format.lch = lch2rgb;
  input$2.format.hcl = hcl2rgb;
  ["lch", "hcl"].forEach(
    (O) => input$2.autodetect.push({
      p: 2,
      test: (..._) => {
        if (_ = unpack(_, O), type$2(_) === "array" && _.length === 3)
          return O;
      }
    })
  );
  Color$2.prototype.saturate = function(O = 1) {
    const _ = this, H = _.lch();
    return H[1] += labConstants.Kn * O, H[1] < 0 && (H[1] = 0), new Color$2(H, "lch").alpha(_.alpha(), !0);
  };
  Color$2.prototype.desaturate = function(O = 1) {
    return this.saturate(-O);
  };
  Color$2.prototype.set = function(O, _, H = !1) {
    const [V, q] = O.split("."), G = this[V]();
    if (q) {
      const J = V.indexOf(q) - (V.substr(0, 2) === "ok" ? 2 : 0);
      if (J > -1) {
        if (type$2(_) == "string")
          switch (_.charAt(0)) {
            case "+":
              G[J] += +_;
              break;
            case "-":
              G[J] += +_;
              break;
            case "*":
              G[J] *= +_.substr(1);
              break;
            case "/":
              G[J] /= +_.substr(1);
              break;
            default:
              G[J] = +_;
          }
        else if (type$2(_) === "number")
          G[J] = _;
        else
          throw new Error("unsupported value for Color.set");
        const Z = new Color$2(G, V);
        return H ? (this._rgb = Z._rgb, this) : Z;
      }
      throw new Error(`unknown channel ${q} in mode ${V}`);
    } else
      return G;
  };
  Color$2.prototype.tint = function(O = 0.5, ..._) {
    return mix(this, "white", O, ..._);
  };
  Color$2.prototype.shade = function(O = 0.5, ..._) {
    return mix(this, "black", O, ..._);
  };
  const rgb$3 = (O, _, H) => {
    const V = O._rgb, q = _._rgb;
    return new Color$2(
      V[0] + H * (q[0] - V[0]),
      V[1] + H * (q[1] - V[1]),
      V[2] + H * (q[2] - V[2]),
      "rgb"
    );
  };
  index$5.rgb = rgb$3;
  const { sqrt: sqrt$b, pow: pow$a } = Math, lrgb = (O, _, H) => {
    const [V, q, G] = O._rgb, [J, Z, te] = _._rgb;
    return new Color$2(
      sqrt$b(pow$a(V, 2) * (1 - H) + pow$a(J, 2) * H),
      sqrt$b(pow$a(q, 2) * (1 - H) + pow$a(Z, 2) * H),
      sqrt$b(pow$a(G, 2) * (1 - H) + pow$a(te, 2) * H),
      "rgb"
    );
  };
  index$5.lrgb = lrgb;
  const lab$2 = (O, _, H) => {
    const V = O.lab(), q = _.lab();
    return new Color$2(
      V[0] + H * (q[0] - V[0]),
      V[1] + H * (q[1] - V[1]),
      V[2] + H * (q[2] - V[2]),
      "lab"
    );
  };
  index$5.lab = lab$2;
  const interpolate_hsx = (O, _, H, V) => {
    let q, G;
    V === "hsl" ? (q = O.hsl(), G = _.hsl()) : V === "hsv" ? (q = O.hsv(), G = _.hsv()) : V === "hcg" ? (q = O.hcg(), G = _.hcg()) : V === "hsi" ? (q = O.hsi(), G = _.hsi()) : V === "lch" || V === "hcl" ? (V = "hcl", q = O.hcl(), G = _.hcl()) : V === "oklch" && (q = O.oklch().reverse(), G = _.oklch().reverse());
    let J, Z, te, ne, ee, ae;
    (V.substr(0, 1) === "h" || V === "oklch") && ([J, te, ee] = q, [Z, ne, ae] = G);
    let oe, se, ce, le;
    return !isNaN(J) && !isNaN(Z) ? (Z > J && Z - J > 180 ? le = Z - (J + 360) : Z < J && J - Z > 180 ? le = Z + 360 - J : le = Z - J, se = J + H * le) : isNaN(J) ? isNaN(Z) ? se = Number.NaN : (se = Z, (ee == 1 || ee == 0) && V != "hsv" && (oe = ne)) : (se = J, (ae == 1 || ae == 0) && V != "hsv" && (oe = te)), oe === void 0 && (oe = te + H * (ne - te)), ce = ee + H * (ae - ee), V === "oklch" ? new Color$2([ce, oe, se], V) : new Color$2([se, oe, ce], V);
  }, lch = (O, _, H) => interpolate_hsx(O, _, H, "lch");
  index$5.lch = lch;
  index$5.hcl = lch;
  const num2rgb = (O) => {
    if (type$2(O) == "number" && O >= 0 && O <= 16777215) {
      const _ = O >> 16, H = O >> 8 & 255, V = O & 255;
      return [_, H, V, 1];
    }
    throw new Error("unknown num color: " + O);
  }, rgb2num = (...O) => {
    const [_, H, V] = unpack(O, "rgb");
    return (_ << 16) + (H << 8) + V;
  };
  Color$2.prototype.num = function() {
    return rgb2num(this._rgb);
  };
  const num$1 = (...O) => new Color$2(...O, "num");
  Object.assign(chroma, { num: num$1 });
  input$2.format.num = num2rgb;
  input$2.autodetect.push({
    p: 5,
    test: (...O) => {
      if (O.length === 1 && type$2(O[0]) === "number" && O[0] >= 0 && O[0] <= 16777215)
        return "num";
    }
  });
  const num = (O, _, H) => {
    const V = O.num(), q = _.num();
    return new Color$2(V + H * (q - V), "num");
  };
  index$5.num = num;
  const { floor: floor$7 } = Math, hcg2rgb = (...O) => {
    O = unpack(O, "hcg");
    let [_, H, V] = O, q, G, J;
    V = V * 255;
    const Z = H * 255;
    if (H === 0)
      q = G = J = V;
    else {
      _ === 360 && (_ = 0), _ > 360 && (_ -= 360), _ < 0 && (_ += 360), _ /= 60;
      const te = floor$7(_), ne = _ - te, ee = V * (1 - H), ae = ee + Z * (1 - ne), oe = ee + Z * ne, se = ee + Z;
      switch (te) {
        case 0:
          [q, G, J] = [se, oe, ee];
          break;
        case 1:
          [q, G, J] = [ae, se, ee];
          break;
        case 2:
          [q, G, J] = [ee, se, oe];
          break;
        case 3:
          [q, G, J] = [ee, ae, se];
          break;
        case 4:
          [q, G, J] = [oe, ee, se];
          break;
        case 5:
          [q, G, J] = [se, ee, ae];
          break;
      }
    }
    return [q, G, J, O.length > 3 ? O[3] : 1];
  }, rgb2hcg = (...O) => {
    const [_, H, V] = unpack(O, "rgb"), q = min$f(_, H, V), G = max$n(_, H, V), J = G - q, Z = J * 100 / 255, te = q / (255 - J) * 100;
    let ne;
    return J === 0 ? ne = Number.NaN : (_ === G && (ne = (H - V) / J), H === G && (ne = 2 + (V - _) / J), V === G && (ne = 4 + (_ - H) / J), ne *= 60, ne < 0 && (ne += 360)), [ne, Z, te];
  };
  Color$2.prototype.hcg = function() {
    return rgb2hcg(this._rgb);
  };
  const hcg$1 = (...O) => new Color$2(...O, "hcg");
  chroma.hcg = hcg$1;
  input$2.format.hcg = hcg2rgb;
  input$2.autodetect.push({
    p: 1,
    test: (...O) => {
      if (O = unpack(O, "hcg"), type$2(O) === "array" && O.length === 3)
        return "hcg";
    }
  });
  const hcg = (O, _, H) => interpolate_hsx(O, _, H, "hcg");
  index$5.hcg = hcg;
  const { cos: cos$8 } = Math, hsi2rgb = (...O) => {
    O = unpack(O, "hsi");
    let [_, H, V] = O, q, G, J;
    return isNaN(_) && (_ = 0), isNaN(H) && (H = 0), _ > 360 && (_ -= 360), _ < 0 && (_ += 360), _ /= 360, _ < 1 / 3 ? (J = (1 - H) / 3, q = (1 + H * cos$8(TWOPI * _) / cos$8(PITHIRD - TWOPI * _)) / 3, G = 1 - (J + q)) : _ < 2 / 3 ? (_ -= 1 / 3, q = (1 - H) / 3, G = (1 + H * cos$8(TWOPI * _) / cos$8(PITHIRD - TWOPI * _)) / 3, J = 1 - (q + G)) : (_ -= 2 / 3, G = (1 - H) / 3, J = (1 + H * cos$8(TWOPI * _) / cos$8(PITHIRD - TWOPI * _)) / 3, q = 1 - (G + J)), q = limit$1(V * q * 3), G = limit$1(V * G * 3), J = limit$1(V * J * 3), [q * 255, G * 255, J * 255, O.length > 3 ? O[3] : 1];
  }, { min: min$e, sqrt: sqrt$a, acos: acos$2 } = Math, rgb2hsi = (...O) => {
    let [_, H, V] = unpack(O, "rgb");
    _ /= 255, H /= 255, V /= 255;
    let q;
    const G = min$e(_, H, V), J = (_ + H + V) / 3, Z = J > 0 ? 1 - G / J : 0;
    return Z === 0 ? q = NaN : (q = (_ - H + (_ - V)) / 2, q /= sqrt$a((_ - H) * (_ - H) + (_ - V) * (H - V)), q = acos$2(q), V > H && (q = TWOPI - q), q /= TWOPI), [q * 360, Z, J];
  };
  Color$2.prototype.hsi = function() {
    return rgb2hsi(this._rgb);
  };
  const hsi$1 = (...O) => new Color$2(...O, "hsi");
  chroma.hsi = hsi$1;
  input$2.format.hsi = hsi2rgb;
  input$2.autodetect.push({
    p: 2,
    test: (...O) => {
      if (O = unpack(O, "hsi"), type$2(O) === "array" && O.length === 3)
        return "hsi";
    }
  });
  const hsi = (O, _, H) => interpolate_hsx(O, _, H, "hsi");
  index$5.hsi = hsi;
  const hsl2rgb$1 = (...O) => {
    O = unpack(O, "hsl");
    const [_, H, V] = O;
    let q, G, J;
    if (H === 0)
      q = G = J = V * 255;
    else {
      const Z = [0, 0, 0], te = [0, 0, 0], ne = V < 0.5 ? V * (1 + H) : V + H - V * H, ee = 2 * V - ne, ae = _ / 360;
      Z[0] = ae + 1 / 3, Z[1] = ae, Z[2] = ae - 1 / 3;
      for (let oe = 0; oe < 3; oe++)
        Z[oe] < 0 && (Z[oe] += 1), Z[oe] > 1 && (Z[oe] -= 1), 6 * Z[oe] < 1 ? te[oe] = ee + (ne - ee) * 6 * Z[oe] : 2 * Z[oe] < 1 ? te[oe] = ne : 3 * Z[oe] < 2 ? te[oe] = ee + (ne - ee) * (2 / 3 - Z[oe]) * 6 : te[oe] = ee;
      [q, G, J] = [te[0] * 255, te[1] * 255, te[2] * 255];
    }
    return O.length > 3 ? [q, G, J, O[3]] : [q, G, J, 1];
  }, rgb2hsl$1 = (...O) => {
    O = unpack(O, "rgba");
    let [_, H, V] = O;
    _ /= 255, H /= 255, V /= 255;
    const q = min$f(_, H, V), G = max$n(_, H, V), J = (G + q) / 2;
    let Z, te;
    return G === q ? (Z = 0, te = Number.NaN) : Z = J < 0.5 ? (G - q) / (G + q) : (G - q) / (2 - G - q), _ == G ? te = (H - V) / (G - q) : H == G ? te = 2 + (V - _) / (G - q) : V == G && (te = 4 + (_ - H) / (G - q)), te *= 60, te < 0 && (te += 360), O.length > 3 && O[3] !== void 0 ? [te, Z, J, O[3]] : [te, Z, J];
  };
  Color$2.prototype.hsl = function() {
    return rgb2hsl$1(this._rgb);
  };
  const hsl$4 = (...O) => new Color$2(...O, "hsl");
  chroma.hsl = hsl$4;
  input$2.format.hsl = hsl2rgb$1;
  input$2.autodetect.push({
    p: 2,
    test: (...O) => {
      if (O = unpack(O, "hsl"), type$2(O) === "array" && O.length === 3)
        return "hsl";
    }
  });
  const hsl$3 = (O, _, H) => interpolate_hsx(O, _, H, "hsl");
  index$5.hsl = hsl$3;
  const { floor: floor$6 } = Math, hsv2rgb = (...O) => {
    O = unpack(O, "hsv");
    let [_, H, V] = O, q, G, J;
    if (V *= 255, H === 0)
      q = G = J = V;
    else {
      _ === 360 && (_ = 0), _ > 360 && (_ -= 360), _ < 0 && (_ += 360), _ /= 60;
      const Z = floor$6(_), te = _ - Z, ne = V * (1 - H), ee = V * (1 - H * te), ae = V * (1 - H * (1 - te));
      switch (Z) {
        case 0:
          [q, G, J] = [V, ae, ne];
          break;
        case 1:
          [q, G, J] = [ee, V, ne];
          break;
        case 2:
          [q, G, J] = [ne, V, ae];
          break;
        case 3:
          [q, G, J] = [ne, ee, V];
          break;
        case 4:
          [q, G, J] = [ae, ne, V];
          break;
        case 5:
          [q, G, J] = [V, ne, ee];
          break;
      }
    }
    return [q, G, J, O.length > 3 ? O[3] : 1];
  }, { min: min$d, max: max$m } = Math, rgb2hsl = (...O) => {
    O = unpack(O, "rgb");
    let [_, H, V] = O;
    const q = min$d(_, H, V), G = max$m(_, H, V), J = G - q;
    let Z, te, ne;
    return ne = G / 255, G === 0 ? (Z = Number.NaN, te = 0) : (te = J / G, _ === G && (Z = (H - V) / J), H === G && (Z = 2 + (V - _) / J), V === G && (Z = 4 + (_ - H) / J), Z *= 60, Z < 0 && (Z += 360)), [Z, te, ne];
  };
  Color$2.prototype.hsv = function() {
    return rgb2hsl(this._rgb);
  };
  const hsv$1 = (...O) => new Color$2(...O, "hsv");
  chroma.hsv = hsv$1;
  input$2.format.hsv = hsv2rgb;
  input$2.autodetect.push({
    p: 2,
    test: (...O) => {
      if (O = unpack(O, "hsv"), type$2(O) === "array" && O.length === 3)
        return "hsv";
    }
  });
  const hsv = (O, _, H) => interpolate_hsx(O, _, H, "hsv");
  index$5.hsv = hsv;
  function multiplyMatrices(O, _) {
    let H = O.length;
    Array.isArray(O[0]) || (O = [O]), Array.isArray(_[0]) || (_ = _.map((J) => [J]));
    let V = _[0].length, q = _[0].map((J, Z) => _.map((te) => te[Z])), G = O.map(
      (J) => q.map((Z) => Array.isArray(J) ? J.reduce((te, ne, ee) => te + ne * (Z[ee] || 0), 0) : Z.reduce((te, ne) => te + ne * J, 0))
    );
    return H === 1 && (G = G[0]), V === 1 ? G.map((J) => J[0]) : G;
  }
  const oklab2rgb = (...O) => {
    O = unpack(O, "lab");
    const [_, H, V, ...q] = O, [G, J, Z] = OKLab_to_XYZ([_, H, V]), [te, ne, ee] = xyz2rgb(G, J, Z);
    return [te, ne, ee, ...q.length > 0 && q[0] < 1 ? [q[0]] : []];
  };
  function OKLab_to_XYZ(O) {
    var _ = [
      [1.2268798758459243, -0.5578149944602171, 0.2813910456659647],
      [-0.0405757452148008, 1.112286803280317, -0.0717110580655164],
      [-0.0763729366746601, -0.4214933324022432, 1.5869240198367816]
    ], H = [
      [1, 0.3963377773761749, 0.2158037573099136],
      [1, -0.1055613458156586, -0.0638541728258133],
      [1, -0.0894841775298119, -1.2914855480194092]
    ], V = multiplyMatrices(H, O);
    return multiplyMatrices(
      _,
      V.map((q) => q ** 3)
    );
  }
  const rgb2oklab = (...O) => {
    const [_, H, V, ...q] = unpack(O, "rgb"), G = rgb2xyz(_, H, V);
    return [...XYZ_to_OKLab(G), ...q.length > 0 && q[0] < 1 ? [q[0]] : []];
  };
  function XYZ_to_OKLab(O) {
    const _ = [
      [0.819022437996703, 0.3619062600528904, -0.1288737815209879],
      [0.0329836539323885, 0.9292868615863434, 0.0361446663506424],
      [0.0481771893596242, 0.2642395317527308, 0.6335478284694309]
    ], H = [
      [0.210454268309314, 0.7936177747023054, -0.0040720430116193],
      [1.9779985324311684, -2.42859224204858, 0.450593709617411],
      [0.0259040424655478, 0.7827717124575296, -0.8086757549230774]
    ], V = multiplyMatrices(_, O);
    return multiplyMatrices(
      H,
      V.map((q) => Math.cbrt(q))
    );
  }
  Color$2.prototype.oklab = function() {
    return rgb2oklab(this._rgb);
  };
  const oklab$1 = (...O) => new Color$2(...O, "oklab");
  Object.assign(chroma, { oklab: oklab$1 });
  input$2.format.oklab = oklab2rgb;
  input$2.autodetect.push({
    p: 2,
    test: (...O) => {
      if (O = unpack(O, "oklab"), type$2(O) === "array" && O.length === 3)
        return "oklab";
    }
  });
  const oklab = (O, _, H) => {
    const V = O.oklab(), q = _.oklab();
    return new Color$2(
      V[0] + H * (q[0] - V[0]),
      V[1] + H * (q[1] - V[1]),
      V[2] + H * (q[2] - V[2]),
      "oklab"
    );
  };
  index$5.oklab = oklab;
  const oklch$1 = (O, _, H) => interpolate_hsx(O, _, H, "oklch");
  index$5.oklch = oklch$1;
  const { pow: pow$9, sqrt: sqrt$9, PI: PI$3, cos: cos$7, sin: sin$7, atan2: atan2$4 } = Math, average = (O, _ = "lrgb", H = null) => {
    const V = O.length;
    H || (H = Array.from(new Array(V)).map(() => 1));
    const q = V / H.reduce(function(ae, oe) {
      return ae + oe;
    });
    if (H.forEach((ae, oe) => {
      H[oe] *= q;
    }), O = O.map((ae) => new Color$2(ae)), _ === "lrgb")
      return _average_lrgb(O, H);
    const G = O.shift(), J = G.get(_), Z = [];
    let te = 0, ne = 0;
    for (let ae = 0; ae < J.length; ae++)
      if (J[ae] = (J[ae] || 0) * H[0], Z.push(isNaN(J[ae]) ? 0 : H[0]), _.charAt(ae) === "h" && !isNaN(J[ae])) {
        const oe = J[ae] / 180 * PI$3;
        te += cos$7(oe) * H[0], ne += sin$7(oe) * H[0];
      }
    let ee = G.alpha() * H[0];
    O.forEach((ae, oe) => {
      const se = ae.get(_);
      ee += ae.alpha() * H[oe + 1];
      for (let ce = 0; ce < J.length; ce++)
        if (!isNaN(se[ce]))
          if (Z[ce] += H[oe + 1], _.charAt(ce) === "h") {
            const le = se[ce] / 180 * PI$3;
            te += cos$7(le) * H[oe + 1], ne += sin$7(le) * H[oe + 1];
          } else
            J[ce] += se[ce] * H[oe + 1];
    });
    for (let ae = 0; ae < J.length; ae++)
      if (_.charAt(ae) === "h") {
        let oe = atan2$4(ne / Z[ae], te / Z[ae]) / PI$3 * 180;
        for (; oe < 0; ) oe += 360;
        for (; oe >= 360; ) oe -= 360;
        J[ae] = oe;
      } else
        J[ae] = J[ae] / Z[ae];
    return ee /= V, new Color$2(J, _).alpha(ee > 0.99999 ? 1 : ee, !0);
  }, _average_lrgb = (O, _) => {
    const H = O.length, V = [0, 0, 0, 0];
    for (let q = 0; q < O.length; q++) {
      const G = O[q], J = _[q] / H, Z = G._rgb;
      V[0] += pow$9(Z[0], 2) * J, V[1] += pow$9(Z[1], 2) * J, V[2] += pow$9(Z[2], 2) * J, V[3] += Z[3] * J;
    }
    return V[0] = sqrt$9(V[0]), V[1] = sqrt$9(V[1]), V[2] = sqrt$9(V[2]), V[3] > 0.9999999 && (V[3] = 1), new Color$2(clip_rgb(V));
  }, { pow: pow$8 } = Math;
  function scale$5(O) {
    let _ = "rgb", H = chroma("#ccc"), V = 0, q = [0, 1], G = [], J = [0, 0], Z = !1, te = [], ne = !1, ee = 0, ae = 1, oe = !1, se = {}, ce = !0, le = 1;
    const ue = function(ge) {
      if (ge = ge || ["#fff", "#000"], ge && type$2(ge) === "string" && chroma.brewer && chroma.brewer[ge.toLowerCase()] && (ge = chroma.brewer[ge.toLowerCase()]), type$2(ge) === "array") {
        ge.length === 1 && (ge = [ge[0], ge[0]]), ge = ge.slice(0);
        for (let ve = 0; ve < ge.length; ve++)
          ge[ve] = chroma(ge[ve]);
        G.length = 0;
        for (let ve = 0; ve < ge.length; ve++)
          G.push(ve / (ge.length - 1));
      }
      return pe(), te = ge;
    }, de = function(ge) {
      if (Z != null) {
        const ve = Z.length - 1;
        let be = 0;
        for (; be < ve && ge >= Z[be]; )
          be++;
        return be - 1;
      }
      return 0;
    };
    let fe = (ge) => ge, he = (ge) => ge;
    const me = function(ge, ve) {
      let be, Ce;
      if (ve == null && (ve = !1), isNaN(ge) || ge === null)
        return H;
      ve ? Ce = ge : Z && Z.length > 2 ? Ce = de(ge) / (Z.length - 2) : ae !== ee ? Ce = (ge - ee) / (ae - ee) : Ce = 1, Ce = he(Ce), ve || (Ce = fe(Ce)), le !== 1 && (Ce = pow$8(Ce, le)), Ce = J[0] + Ce * (1 - J[0] - J[1]), Ce = limit$1(Ce, 0, 1);
      const Se = Math.floor(Ce * 1e4);
      if (ce && se[Se])
        be = se[Se];
      else {
        if (type$2(te) === "array")
          for (let we = 0; we < G.length; we++) {
            const Ee = G[we];
            if (Ce <= Ee) {
              be = te[we];
              break;
            }
            if (Ce >= Ee && we === G.length - 1) {
              be = te[we];
              break;
            }
            if (Ce > Ee && Ce < G[we + 1]) {
              Ce = (Ce - Ee) / (G[we + 1] - Ee), be = chroma.interpolate(
                te[we],
                te[we + 1],
                Ce,
                _
              );
              break;
            }
          }
        else type$2(te) === "function" && (be = te(Ce));
        ce && (se[Se] = be);
      }
      return be;
    };
    var pe = () => se = {};
    ue(O);
    const ye = function(ge) {
      const ve = chroma(me(ge));
      return ne && ve[ne] ? ve[ne]() : ve;
    };
    return ye.classes = function(ge) {
      if (ge != null) {
        if (type$2(ge) === "array")
          Z = ge, q = [ge[0], ge[ge.length - 1]];
        else {
          const ve = chroma.analyze(q);
          ge === 0 ? Z = [ve.min, ve.max] : Z = chroma.limits(ve, "e", ge);
        }
        return ye;
      }
      return Z;
    }, ye.domain = function(ge) {
      if (!arguments.length)
        return q;
      ee = ge[0], ae = ge[ge.length - 1], G = [];
      const ve = te.length;
      if (ge.length === ve && ee !== ae)
        for (let be of Array.from(ge))
          G.push((be - ee) / (ae - ee));
      else {
        for (let be = 0; be < ve; be++)
          G.push(be / (ve - 1));
        if (ge.length > 2) {
          const be = ge.map((Se, we) => we / (ge.length - 1)), Ce = ge.map((Se) => (Se - ee) / (ae - ee));
          Ce.every((Se, we) => be[we] === Se) || (he = (Se) => {
            if (Se <= 0 || Se >= 1) return Se;
            let we = 0;
            for (; Se >= Ce[we + 1]; ) we++;
            const Ee = (Se - Ce[we]) / (Ce[we + 1] - Ce[we]);
            return be[we] + Ee * (be[we + 1] - be[we]);
          });
        }
      }
      return q = [ee, ae], ye;
    }, ye.mode = function(ge) {
      return arguments.length ? (_ = ge, pe(), ye) : _;
    }, ye.range = function(ge, ve) {
      return ue(ge), ye;
    }, ye.out = function(ge) {
      return ne = ge, ye;
    }, ye.spread = function(ge) {
      return arguments.length ? (V = ge, ye) : V;
    }, ye.correctLightness = function(ge) {
      return ge == null && (ge = !0), oe = ge, pe(), oe ? fe = function(ve) {
        const be = me(0, !0).lab()[0], Ce = me(1, !0).lab()[0], Se = be > Ce;
        let we = me(ve, !0).lab()[0];
        const Ee = be + (Ce - be) * ve;
        let xe = we - Ee, Le = 0, Qe = 1, ke = 20;
        for (; Math.abs(xe) > 0.01 && ke-- > 0; )
          (function() {
            return Se && (xe *= -1), xe < 0 ? (Le = ve, ve += (Qe - ve) * 0.5) : (Qe = ve, ve += (Le - ve) * 0.5), we = me(ve, !0).lab()[0], xe = we - Ee;
          })();
        return ve;
      } : fe = (ve) => ve, ye;
    }, ye.padding = function(ge) {
      return ge != null ? (type$2(ge) === "number" && (ge = [ge, ge]), J = ge, ye) : J;
    }, ye.colors = function(ge, ve) {
      arguments.length < 2 && (ve = "hex");
      let be = [];
      if (arguments.length === 0)
        be = te.slice(0);
      else if (ge === 1)
        be = [ye(0.5)];
      else if (ge > 1) {
        const Ce = q[0], Se = q[1] - Ce;
        be = __range__(0, ge).map(
          (we) => ye(Ce + we / (ge - 1) * Se)
        );
      } else {
        O = [];
        let Ce = [];
        if (Z && Z.length > 2)
          for (let Se = 1, we = Z.length, Ee = 1 <= we; Ee ? Se < we : Se > we; Ee ? Se++ : Se--)
            Ce.push((Z[Se - 1] + Z[Se]) * 0.5);
        else
          Ce = q;
        be = Ce.map((Se) => ye(Se));
      }
      return chroma[ve] && (be = be.map((Ce) => Ce[ve]())), be;
    }, ye.cache = function(ge) {
      return ge != null ? (ce = ge, ye) : ce;
    }, ye.gamma = function(ge) {
      return ge != null ? (le = ge, ye) : le;
    }, ye.nodata = function(ge) {
      return ge != null ? (H = chroma(ge), ye) : H;
    }, ye;
  }
  function __range__(O, _, H) {
    let V = [], q = O < _, G = _;
    for (let J = O; q ? J < G : J > G; q ? J++ : J--)
      V.push(J);
    return V;
  }
  const binom_row = function(O) {
    let _ = [1, 1];
    for (let H = 1; H < O; H++) {
      let V = [1];
      for (let q = 1; q <= _.length; q++)
        V[q] = (_[q] || 0) + _[q - 1];
      _ = V;
    }
    return _;
  }, bezier$1 = function(O) {
    let _, H, V, q;
    if (O = O.map((G) => new Color$2(G)), O.length === 2)
      [H, V] = O.map((G) => G.lab()), _ = function(G) {
        const J = [0, 1, 2].map((Z) => H[Z] + G * (V[Z] - H[Z]));
        return new Color$2(J, "lab");
      };
    else if (O.length === 3)
      [H, V, q] = O.map((G) => G.lab()), _ = function(G) {
        const J = [0, 1, 2].map(
          (Z) => (1 - G) * (1 - G) * H[Z] + 2 * (1 - G) * G * V[Z] + G * G * q[Z]
        );
        return new Color$2(J, "lab");
      };
    else if (O.length === 4) {
      let G;
      [H, V, q, G] = O.map((J) => J.lab()), _ = function(J) {
        const Z = [0, 1, 2].map(
          (te) => (1 - J) * (1 - J) * (1 - J) * H[te] + 3 * (1 - J) * (1 - J) * J * V[te] + 3 * (1 - J) * J * J * q[te] + J * J * J * G[te]
        );
        return new Color$2(Z, "lab");
      };
    } else if (O.length >= 5) {
      let G, J, Z;
      G = O.map((te) => te.lab()), Z = O.length - 1, J = binom_row(Z), _ = function(te) {
        const ne = 1 - te, ee = [0, 1, 2].map(
          (ae) => G.reduce(
            (oe, se, ce) => oe + J[ce] * ne ** (Z - ce) * te ** ce * se[ae],
            0
          )
        );
        return new Color$2(ee, "lab");
      };
    } else
      throw new RangeError("No point in running bezier with only one color.");
    return _;
  }, bezier$2 = (O) => {
    const _ = bezier$1(O);
    return _.scale = () => scale$5(_), _;
  }, { round: round$b } = Math;
  Color$2.prototype.rgb = function(O = !0) {
    return O === !1 ? this._rgb.slice(0, 3) : this._rgb.slice(0, 3).map(round$b);
  };
  Color$2.prototype.rgba = function(O = !0) {
    return this._rgb.slice(0, 4).map((_, H) => H < 3 ? O === !1 ? _ : round$b(_) : _);
  };
  const rgb$2 = (...O) => new Color$2(...O, "rgb");
  Object.assign(chroma, { rgb: rgb$2 });
  input$2.format.rgb = (...O) => {
    const _ = unpack(O, "rgba");
    return _[3] === void 0 && (_[3] = 1), _;
  };
  input$2.autodetect.push({
    p: 3,
    test: (...O) => {
      if (O = unpack(O, "rgba"), type$2(O) === "array" && (O.length === 3 || O.length === 4 && type$2(O[3]) == "number" && O[3] >= 0 && O[3] <= 1))
        return "rgb";
    }
  });
  const blend$1 = (O, _, H) => {
    if (!blend$1[H])
      throw new Error("unknown blend mode " + H);
    return blend$1[H](O, _);
  }, blend_f = (O) => (_, H) => {
    const V = chroma(H).rgb(), q = chroma(_).rgb();
    return chroma.rgb(O(V, q));
  }, each = (O) => (_, H) => {
    const V = [];
    return V[0] = O(_[0], H[0]), V[1] = O(_[1], H[1]), V[2] = O(_[2], H[2]), V;
  }, normal$2 = (O) => O, multiply = (O, _) => O * _ / 255, darken = (O, _) => O > _ ? _ : O, lighten = (O, _) => O > _ ? O : _, screen$1 = (O, _) => 255 * (1 - (1 - O / 255) * (1 - _ / 255)), overlay = (O, _) => _ < 128 ? 2 * O * _ / 255 : 255 * (1 - 2 * (1 - O / 255) * (1 - _ / 255)), burn = (O, _) => 255 * (1 - (1 - _ / 255) / (O / 255)), dodge = (O, _) => O === 255 ? 255 : (O = 255 * (_ / 255) / (1 - O / 255), O > 255 ? 255 : O);
  blend$1.normal = blend_f(each(normal$2));
  blend$1.multiply = blend_f(each(multiply));
  blend$1.screen = blend_f(each(screen$1));
  blend$1.overlay = blend_f(each(overlay));
  blend$1.darken = blend_f(each(darken));
  blend$1.lighten = blend_f(each(lighten));
  blend$1.dodge = blend_f(each(dodge));
  blend$1.burn = blend_f(each(burn));
  const { pow: pow$7, sin: sin$6, cos: cos$6 } = Math;
  function cubehelix$3(O = 300, _ = -1.5, H = 1, V = 1, q = [0, 1]) {
    let G = 0, J;
    type$2(q) === "array" ? J = q[1] - q[0] : (J = 0, q = [q, q]);
    const Z = function(te) {
      const ne = TWOPI * ((O + 120) / 360 + _ * te), ee = pow$7(q[0] + J * te, V), oe = (G !== 0 ? H[0] + te * G : H) * ee * (1 - ee) / 2, se = cos$6(ne), ce = sin$6(ne), le = ee + oe * (-0.14861 * se + 1.78277 * ce), ue = ee + oe * (-0.29227 * se - 0.90649 * ce), de = ee + oe * (1.97294 * se);
      return chroma(clip_rgb([le * 255, ue * 255, de * 255, 1]));
    };
    return Z.start = function(te) {
      return te == null ? O : (O = te, Z);
    }, Z.rotations = function(te) {
      return te == null ? _ : (_ = te, Z);
    }, Z.gamma = function(te) {
      return te == null ? V : (V = te, Z);
    }, Z.hue = function(te) {
      return te == null ? H : (H = te, type$2(H) === "array" ? (G = H[1] - H[0], G === 0 && (H = H[1])) : G = 0, Z);
    }, Z.lightness = function(te) {
      return te == null ? q : (type$2(te) === "array" ? (q = te, J = te[1] - te[0]) : (q = [te, te], J = 0), Z);
    }, Z.scale = () => chroma.scale(Z), Z.hue(H), Z;
  }
  const digits = "0123456789abcdef", { floor: floor$5, random: random$2 } = Math, random$3 = () => {
    let O = "#";
    for (let _ = 0; _ < 6; _++)
      O += digits.charAt(floor$5(random$2() * 16));
    return new Color$2(O, "hex");
  }, { log: log$9, pow: pow$6, floor: floor$4, abs: abs$f } = Math;
  function analyze$1(O, _ = null) {
    const H = {
      min: Number.MAX_VALUE,
      max: Number.MAX_VALUE * -1,
      sum: 0,
      values: [],
      count: 0
    };
    return type$2(O) === "object" && (O = Object.values(O)), O.forEach((V) => {
      _ && type$2(V) === "object" && (V = V[_]), V != null && !isNaN(V) && (H.values.push(V), H.sum += V, V < H.min && (H.min = V), V > H.max && (H.max = V), H.count += 1);
    }), H.domain = [H.min, H.max], H.limits = (V, q) => limits(H, V, q), H;
  }
  function limits(O, _ = "equal", H = 7) {
    type$2(O) == "array" && (O = analyze$1(O));
    const { min: V, max: q } = O, G = O.values.sort((Z, te) => Z - te);
    if (H === 1)
      return [V, q];
    const J = [];
    if (_.substr(0, 1) === "c" && (J.push(V), J.push(q)), _.substr(0, 1) === "e") {
      J.push(V);
      for (let Z = 1; Z < H; Z++)
        J.push(V + Z / H * (q - V));
      J.push(q);
    } else if (_.substr(0, 1) === "l") {
      if (V <= 0)
        throw new Error(
          "Logarithmic scales are only possible for values > 0"
        );
      const Z = Math.LOG10E * log$9(V), te = Math.LOG10E * log$9(q);
      J.push(V);
      for (let ne = 1; ne < H; ne++)
        J.push(pow$6(10, Z + ne / H * (te - Z)));
      J.push(q);
    } else if (_.substr(0, 1) === "q") {
      J.push(V);
      for (let Z = 1; Z < H; Z++) {
        const te = (G.length - 1) * Z / H, ne = floor$4(te);
        if (ne === te)
          J.push(G[ne]);
        else {
          const ee = te - ne;
          J.push(G[ne] * (1 - ee) + G[ne + 1] * ee);
        }
      }
      J.push(q);
    } else if (_.substr(0, 1) === "k") {
      let Z;
      const te = G.length, ne = new Array(te), ee = new Array(H);
      let ae = !0, oe = 0, se = null;
      se = [], se.push(V);
      for (let ue = 1; ue < H; ue++)
        se.push(V + ue / H * (q - V));
      for (se.push(q); ae; ) {
        for (let de = 0; de < H; de++)
          ee[de] = 0;
        for (let de = 0; de < te; de++) {
          const fe = G[de];
          let he = Number.MAX_VALUE, me;
          for (let pe = 0; pe < H; pe++) {
            const ye = abs$f(se[pe] - fe);
            ye < he && (he = ye, me = pe), ee[me]++, ne[de] = me;
          }
        }
        const ue = new Array(H);
        for (let de = 0; de < H; de++)
          ue[de] = null;
        for (let de = 0; de < te; de++)
          Z = ne[de], ue[Z] === null ? ue[Z] = G[de] : ue[Z] += G[de];
        for (let de = 0; de < H; de++)
          ue[de] *= 1 / ee[de];
        ae = !1;
        for (let de = 0; de < H; de++)
          if (ue[de] !== se[de]) {
            ae = !0;
            break;
          }
        se = ue, oe++, oe > 200 && (ae = !1);
      }
      const ce = {};
      for (let ue = 0; ue < H; ue++)
        ce[ue] = [];
      for (let ue = 0; ue < te; ue++)
        Z = ne[ue], ce[Z].push(G[ue]);
      let le = [];
      for (let ue = 0; ue < H; ue++)
        le.push(ce[ue][0]), le.push(ce[ue][ce[ue].length - 1]);
      le = le.sort((ue, de) => ue - de), J.push(le[0]);
      for (let ue = 1; ue < le.length; ue += 2) {
        const de = le[ue];
        !isNaN(de) && J.indexOf(de) === -1 && J.push(de);
      }
    }
    return J;
  }
  const contrast$1 = (O, _) => {
    O = new Color$2(O), _ = new Color$2(_);
    const H = O.luminance(), V = _.luminance();
    return H > V ? (H + 0.05) / (V + 0.05) : (V + 0.05) / (H + 0.05);
  };
  /**
   * @license
   *
   * The APCA contrast prediction algorithm is based of the formulas published
   * in the APCA-1.0.98G specification by Myndex. The specification is available at:
   * https://raw.githubusercontent.com/Myndex/apca-w3/master/images/APCAw3_0.1.17_APCA0.0.98G.svg
   *
   * Note that the APCA implementation is still beta, so please update to
   * future versions of chroma.js when they become available.
   *
   * You can read more about the APCA Readability Criterion at
   * https://readtech.org/ARC/
   */
  const W_offset = 0.027, P_in = 5e-4, P_out = 0.1, R_scale = 1.14, B_threshold = 0.022, B_exp = 1.414, contrastAPCA = (O, _) => {
    O = new Color$2(O), _ = new Color$2(_), O.alpha() < 1 && (O = mix(_, O, O.alpha(), "rgb"));
    const H = lum(...O.rgb()), V = lum(..._.rgb()), q = H >= B_threshold ? H : H + Math.pow(B_threshold - H, B_exp), G = V >= B_threshold ? V : V + Math.pow(B_threshold - V, B_exp), J = Math.pow(G, 0.56) - Math.pow(q, 0.57), Z = Math.pow(G, 0.65) - Math.pow(q, 0.62), te = Math.abs(G - q) < P_in ? 0 : q < G ? J * R_scale : Z * R_scale;
    return (Math.abs(te) < P_out ? 0 : te > 0 ? te - W_offset : te + W_offset) * 100;
  };
  function lum(O, _, H) {
    return 0.2126729 * Math.pow(O / 255, 2.4) + 0.7151522 * Math.pow(_ / 255, 2.4) + 0.072175 * Math.pow(H / 255, 2.4);
  }
  const { sqrt: sqrt$8, pow: pow$5, min: min$c, max: max$l, atan2: atan2$3, abs: abs$e, cos: cos$5, sin: sin$5, exp: exp$3, PI: PI$2 } = Math;
  function deltaE(O, _, H = 1, V = 1, q = 1) {
    var G = function(qe) {
      return 360 * qe / (2 * PI$2);
    }, J = function(qe) {
      return 2 * PI$2 * qe / 360;
    };
    O = new Color$2(O), _ = new Color$2(_);
    const [Z, te, ne] = Array.from(O.lab()), [ee, ae, oe] = Array.from(_.lab()), se = (Z + ee) / 2, ce = sqrt$8(pow$5(te, 2) + pow$5(ne, 2)), le = sqrt$8(pow$5(ae, 2) + pow$5(oe, 2)), ue = (ce + le) / 2, de = 0.5 * (1 - sqrt$8(pow$5(ue, 7) / (pow$5(ue, 7) + pow$5(25, 7)))), fe = te * (1 + de), he = ae * (1 + de), me = sqrt$8(pow$5(fe, 2) + pow$5(ne, 2)), pe = sqrt$8(pow$5(he, 2) + pow$5(oe, 2)), ye = (me + pe) / 2, ge = G(atan2$3(ne, fe)), ve = G(atan2$3(oe, he)), be = ge >= 0 ? ge : ge + 360, Ce = ve >= 0 ? ve : ve + 360, Se = abs$e(be - Ce) > 180 ? (be + Ce + 360) / 2 : (be + Ce) / 2, we = 1 - 0.17 * cos$5(J(Se - 30)) + 0.24 * cos$5(J(2 * Se)) + 0.32 * cos$5(J(3 * Se + 6)) - 0.2 * cos$5(J(4 * Se - 63));
    let Ee = Ce - be;
    Ee = abs$e(Ee) <= 180 ? Ee : Ce <= be ? Ee + 360 : Ee - 360, Ee = 2 * sqrt$8(me * pe) * sin$5(J(Ee) / 2);
    const xe = ee - Z, Le = pe - me, Qe = 1 + 0.015 * pow$5(se - 50, 2) / sqrt$8(20 + pow$5(se - 50, 2)), ke = 1 + 0.045 * ye, Me = 1 + 0.015 * ye * we, Re = 30 * exp$3(-pow$5((Se - 275) / 25, 2)), Pe = -(2 * sqrt$8(pow$5(ye, 7) / (pow$5(ye, 7) + pow$5(25, 7)))) * sin$5(2 * J(Re)), $e = sqrt$8(
      pow$5(xe / (H * Qe), 2) + pow$5(Le / (V * ke), 2) + pow$5(Ee / (q * Me), 2) + Pe * (Le / (V * ke)) * (Ee / (q * Me))
    );
    return max$l(0, min$c(100, $e));
  }
  function distance(O, _, H = "lab") {
    O = new Color$2(O), _ = new Color$2(_);
    const V = O.get(H), q = _.get(H);
    let G = 0;
    for (let J in V) {
      const Z = (V[J] || 0) - (q[J] || 0);
      G += Z * Z;
    }
    return Math.sqrt(G);
  }
  const valid = (...O) => {
    try {
      return new Color$2(...O), !0;
    } catch {
      return !1;
    }
  }, scales$1 = {
    cool() {
      return scale$5([chroma.hsl(180, 1, 0.9), chroma.hsl(250, 0.7, 0.4)]);
    },
    hot() {
      return scale$5(["#000", "#f00", "#ff0", "#fff"]).mode(
        "rgb"
      );
    }
  }, colorbrewer = {
    // sequential
    OrRd: ["#fff7ec", "#fee8c8", "#fdd49e", "#fdbb84", "#fc8d59", "#ef6548", "#d7301f", "#b30000", "#7f0000"],
    PuBu: ["#fff7fb", "#ece7f2", "#d0d1e6", "#a6bddb", "#74a9cf", "#3690c0", "#0570b0", "#045a8d", "#023858"],
    BuPu: ["#f7fcfd", "#e0ecf4", "#bfd3e6", "#9ebcda", "#8c96c6", "#8c6bb1", "#88419d", "#810f7c", "#4d004b"],
    Oranges: ["#fff5eb", "#fee6ce", "#fdd0a2", "#fdae6b", "#fd8d3c", "#f16913", "#d94801", "#a63603", "#7f2704"],
    BuGn: ["#f7fcfd", "#e5f5f9", "#ccece6", "#99d8c9", "#66c2a4", "#41ae76", "#238b45", "#006d2c", "#00441b"],
    YlOrBr: ["#ffffe5", "#fff7bc", "#fee391", "#fec44f", "#fe9929", "#ec7014", "#cc4c02", "#993404", "#662506"],
    YlGn: ["#ffffe5", "#f7fcb9", "#d9f0a3", "#addd8e", "#78c679", "#41ab5d", "#238443", "#006837", "#004529"],
    Reds: ["#fff5f0", "#fee0d2", "#fcbba1", "#fc9272", "#fb6a4a", "#ef3b2c", "#cb181d", "#a50f15", "#67000d"],
    RdPu: ["#fff7f3", "#fde0dd", "#fcc5c0", "#fa9fb5", "#f768a1", "#dd3497", "#ae017e", "#7a0177", "#49006a"],
    Greens: ["#f7fcf5", "#e5f5e0", "#c7e9c0", "#a1d99b", "#74c476", "#41ab5d", "#238b45", "#006d2c", "#00441b"],
    YlGnBu: ["#ffffd9", "#edf8b1", "#c7e9b4", "#7fcdbb", "#41b6c4", "#1d91c0", "#225ea8", "#253494", "#081d58"],
    Purples: ["#fcfbfd", "#efedf5", "#dadaeb", "#bcbddc", "#9e9ac8", "#807dba", "#6a51a3", "#54278f", "#3f007d"],
    GnBu: ["#f7fcf0", "#e0f3db", "#ccebc5", "#a8ddb5", "#7bccc4", "#4eb3d3", "#2b8cbe", "#0868ac", "#084081"],
    Greys: ["#ffffff", "#f0f0f0", "#d9d9d9", "#bdbdbd", "#969696", "#737373", "#525252", "#252525", "#000000"],
    YlOrRd: ["#ffffcc", "#ffeda0", "#fed976", "#feb24c", "#fd8d3c", "#fc4e2a", "#e31a1c", "#bd0026", "#800026"],
    PuRd: ["#f7f4f9", "#e7e1ef", "#d4b9da", "#c994c7", "#df65b0", "#e7298a", "#ce1256", "#980043", "#67001f"],
    Blues: ["#f7fbff", "#deebf7", "#c6dbef", "#9ecae1", "#6baed6", "#4292c6", "#2171b5", "#08519c", "#08306b"],
    PuBuGn: ["#fff7fb", "#ece2f0", "#d0d1e6", "#a6bddb", "#67a9cf", "#3690c0", "#02818a", "#016c59", "#014636"],
    Viridis: ["#440154", "#482777", "#3f4a8a", "#31678e", "#26838f", "#1f9d8a", "#6cce5a", "#b6de2b", "#fee825"],
    // diverging
    Spectral: ["#9e0142", "#d53e4f", "#f46d43", "#fdae61", "#fee08b", "#ffffbf", "#e6f598", "#abdda4", "#66c2a5", "#3288bd", "#5e4fa2"],
    RdYlGn: ["#a50026", "#d73027", "#f46d43", "#fdae61", "#fee08b", "#ffffbf", "#d9ef8b", "#a6d96a", "#66bd63", "#1a9850", "#006837"],
    RdBu: ["#67001f", "#b2182b", "#d6604d", "#f4a582", "#fddbc7", "#f7f7f7", "#d1e5f0", "#92c5de", "#4393c3", "#2166ac", "#053061"],
    PiYG: ["#8e0152", "#c51b7d", "#de77ae", "#f1b6da", "#fde0ef", "#f7f7f7", "#e6f5d0", "#b8e186", "#7fbc41", "#4d9221", "#276419"],
    PRGn: ["#40004b", "#762a83", "#9970ab", "#c2a5cf", "#e7d4e8", "#f7f7f7", "#d9f0d3", "#a6dba0", "#5aae61", "#1b7837", "#00441b"],
    RdYlBu: ["#a50026", "#d73027", "#f46d43", "#fdae61", "#fee090", "#ffffbf", "#e0f3f8", "#abd9e9", "#74add1", "#4575b4", "#313695"],
    BrBG: ["#543005", "#8c510a", "#bf812d", "#dfc27d", "#f6e8c3", "#f5f5f5", "#c7eae5", "#80cdc1", "#35978f", "#01665e", "#003c30"],
    RdGy: ["#67001f", "#b2182b", "#d6604d", "#f4a582", "#fddbc7", "#ffffff", "#e0e0e0", "#bababa", "#878787", "#4d4d4d", "#1a1a1a"],
    PuOr: ["#7f3b08", "#b35806", "#e08214", "#fdb863", "#fee0b6", "#f7f7f7", "#d8daeb", "#b2abd2", "#8073ac", "#542788", "#2d004b"],
    // qualitative
    Set2: ["#66c2a5", "#fc8d62", "#8da0cb", "#e78ac3", "#a6d854", "#ffd92f", "#e5c494", "#b3b3b3"],
    Accent: ["#7fc97f", "#beaed4", "#fdc086", "#ffff99", "#386cb0", "#f0027f", "#bf5b17", "#666666"],
    Set1: ["#e41a1c", "#377eb8", "#4daf4a", "#984ea3", "#ff7f00", "#ffff33", "#a65628", "#f781bf", "#999999"],
    Set3: ["#8dd3c7", "#ffffb3", "#bebada", "#fb8072", "#80b1d3", "#fdb462", "#b3de69", "#fccde5", "#d9d9d9", "#bc80bd", "#ccebc5", "#ffed6f"],
    Dark2: ["#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e", "#e6ab02", "#a6761d", "#666666"],
    Paired: ["#a6cee3", "#1f78b4", "#b2df8a", "#33a02c", "#fb9a99", "#e31a1c", "#fdbf6f", "#ff7f00", "#cab2d6", "#6a3d9a", "#ffff99", "#b15928"],
    Pastel2: ["#b3e2cd", "#fdcdac", "#cbd5e8", "#f4cae4", "#e6f5c9", "#fff2ae", "#f1e2cc", "#cccccc"],
    Pastel1: ["#fbb4ae", "#b3cde3", "#ccebc5", "#decbe4", "#fed9a6", "#ffffcc", "#e5d8bd", "#fddaec", "#f2f2f2"]
  }, colorbrewerTypes = Object.keys(colorbrewer), typeMap = new Map(colorbrewerTypes.map((O) => [O.toLowerCase(), O])), colorbrewerProxy = typeof Proxy == "function" ? new Proxy(colorbrewer, {
    get(O, _) {
      const H = _.toLowerCase();
      if (typeMap.has(H))
        return O[typeMap.get(H)];
    },
    getOwnPropertyNames() {
      return Object.getOwnPropertyNames(colorbrewerTypes);
    }
  }) : colorbrewer, cmyk2rgb = (...O) => {
    O = unpack(O, "cmyk");
    const [_, H, V, q] = O, G = O.length > 4 ? O[4] : 1;
    return q === 1 ? [0, 0, 0, G] : [
      _ >= 1 ? 0 : 255 * (1 - _) * (1 - q),
      // r
      H >= 1 ? 0 : 255 * (1 - H) * (1 - q),
      // g
      V >= 1 ? 0 : 255 * (1 - V) * (1 - q),
      // b
      G
    ];
  }, { max: max$k } = Math, rgb2cmyk = (...O) => {
    let [_, H, V] = unpack(O, "rgb");
    _ = _ / 255, H = H / 255, V = V / 255;
    const q = 1 - max$k(_, max$k(H, V)), G = q < 1 ? 1 / (1 - q) : 0, J = (1 - _ - q) * G, Z = (1 - H - q) * G, te = (1 - V - q) * G;
    return [J, Z, te, q];
  };
  Color$2.prototype.cmyk = function() {
    return rgb2cmyk(this._rgb);
  };
  const cmyk = (...O) => new Color$2(...O, "cmyk");
  Object.assign(chroma, { cmyk });
  input$2.format.cmyk = cmyk2rgb;
  input$2.autodetect.push({
    p: 2,
    test: (...O) => {
      if (O = unpack(O, "cmyk"), type$2(O) === "array" && O.length === 4)
        return "cmyk";
    }
  });
  const hsl2css = (...O) => {
    const _ = unpack(O, "hsla");
    let H = last(O) || "lsa";
    return _[0] = rnd2(_[0] || 0) + "deg", _[1] = rnd2(_[1] * 100) + "%", _[2] = rnd2(_[2] * 100) + "%", H === "hsla" || _.length > 3 && _[3] < 1 ? (_[3] = "/ " + (_.length > 3 ? _[3] : 1), H = "hsla") : _.length = 3, `${H.substr(0, 3)}(${_.join(" ")})`;
  }, lab2css = (...O) => {
    const _ = unpack(O, "lab");
    let H = last(O) || "lab";
    return _[0] = rnd2(_[0]) + "%", _[1] = rnd2(_[1]), _[2] = rnd2(_[2]), H === "laba" || _.length > 3 && _[3] < 1 ? _[3] = "/ " + (_.length > 3 ? _[3] : 1) : _.length = 3, `lab(${_.join(" ")})`;
  }, lch2css = (...O) => {
    const _ = unpack(O, "lch");
    let H = last(O) || "lab";
    return _[0] = rnd2(_[0]) + "%", _[1] = rnd2(_[1]), _[2] = isNaN(_[2]) ? "none" : rnd2(_[2]) + "deg", H === "lcha" || _.length > 3 && _[3] < 1 ? _[3] = "/ " + (_.length > 3 ? _[3] : 1) : _.length = 3, `lch(${_.join(" ")})`;
  }, oklab2css = (...O) => {
    const _ = unpack(O, "lab");
    return _[0] = rnd2(_[0] * 100) + "%", _[1] = rnd3(_[1]), _[2] = rnd3(_[2]), _.length > 3 && _[3] < 1 ? _[3] = "/ " + (_.length > 3 ? _[3] : 1) : _.length = 3, `oklab(${_.join(" ")})`;
  }, rgb2oklch = (...O) => {
    const [_, H, V, ...q] = unpack(O, "rgb"), [G, J, Z] = rgb2oklab(_, H, V), [te, ne, ee] = lab2lch(G, J, Z);
    return [te, ne, ee, ...q.length > 0 && q[0] < 1 ? [q[0]] : []];
  }, oklch2css = (...O) => {
    const _ = unpack(O, "lch");
    return _[0] = rnd2(_[0] * 100) + "%", _[1] = rnd3(_[1]), _[2] = isNaN(_[2]) ? "none" : rnd2(_[2]) + "deg", _.length > 3 && _[3] < 1 ? _[3] = "/ " + (_.length > 3 ? _[3] : 1) : _.length = 3, `oklch(${_.join(" ")})`;
  }, { round: round$a } = Math, rgb2css = (...O) => {
    const _ = unpack(O, "rgba");
    let H = last(O) || "rgb";
    if (H.substr(0, 3) === "hsl")
      return hsl2css(rgb2hsl$1(_), H);
    if (H.substr(0, 3) === "lab") {
      const V = getLabWhitePoint();
      setLabWhitePoint("d50");
      const q = lab2css(rgb2lab(_), H);
      return setLabWhitePoint(V), q;
    }
    if (H.substr(0, 3) === "lch") {
      const V = getLabWhitePoint();
      setLabWhitePoint("d50");
      const q = lch2css(rgb2lch(_), H);
      return setLabWhitePoint(V), q;
    }
    return H.substr(0, 5) === "oklab" ? oklab2css(rgb2oklab(_)) : H.substr(0, 5) === "oklch" ? oklch2css(rgb2oklch(_)) : (_[0] = round$a(_[0]), _[1] = round$a(_[1]), _[2] = round$a(_[2]), (H === "rgba" || _.length > 3 && _[3] < 1) && (_[3] = "/ " + (_.length > 3 ? _[3] : 1), H = "rgba"), `${H.substr(0, 3)}(${_.slice(0, H === "rgb" ? 3 : 4).join(" ")})`);
  }, oklch2rgb = (...O) => {
    O = unpack(O, "lch");
    const [_, H, V, ...q] = O, [G, J, Z] = lch2lab(_, H, V), [te, ne, ee] = oklab2rgb(G, J, Z);
    return [te, ne, ee, ...q.length > 0 && q[0] < 1 ? [q[0]] : []];
  }, INT_OR_PCT = /((?:-?\d+)|(?:-?\d+(?:\.\d+)?)%|none)/.source, FLOAT_OR_PCT = /((?:-?(?:\d+(?:\.\d*)?|\.\d+)%?)|none)/.source, PCT = /((?:-?(?:\d+(?:\.\d*)?|\.\d+)%)|none)/.source, RE_S = /\s*/.source, SEP = /\s+/.source, COMMA$1 = /\s*,\s*/.source, ANLGE = /((?:-?(?:\d+(?:\.\d*)?|\.\d+)(?:deg)?)|none)/.source, ALPHA = /\s*(?:\/\s*((?:[01]|[01]?\.\d+)|\d+(?:\.\d+)?%))?/.source, RE_RGB = new RegExp(
    "^rgba?\\(" + RE_S + [INT_OR_PCT, INT_OR_PCT, INT_OR_PCT].join(SEP) + ALPHA + "\\)$"
  ), RE_RGB_LEGACY = new RegExp(
    "^rgb\\(" + RE_S + [INT_OR_PCT, INT_OR_PCT, INT_OR_PCT].join(COMMA$1) + RE_S + "\\)$"
  ), RE_RGBA_LEGACY = new RegExp(
    "^rgba\\(" + RE_S + [INT_OR_PCT, INT_OR_PCT, INT_OR_PCT, FLOAT_OR_PCT].join(COMMA$1) + RE_S + "\\)$"
  ), RE_HSL = new RegExp(
    "^hsla?\\(" + RE_S + [ANLGE, PCT, PCT].join(SEP) + ALPHA + "\\)$"
  ), RE_HSL_LEGACY = new RegExp(
    "^hsl?\\(" + RE_S + [ANLGE, PCT, PCT].join(COMMA$1) + RE_S + "\\)$"
  ), RE_HSLA_LEGACY = /^hsla\(\s*(-?\d+(?:\.\d+)?),\s*(-?\d+(?:\.\d+)?)%\s*,\s*(-?\d+(?:\.\d+)?)%\s*,\s*([01]|[01]?\.\d+)\)$/, RE_LAB = new RegExp(
    "^lab\\(" + RE_S + [FLOAT_OR_PCT, FLOAT_OR_PCT, FLOAT_OR_PCT].join(SEP) + ALPHA + "\\)$"
  ), RE_LCH = new RegExp(
    "^lch\\(" + RE_S + [FLOAT_OR_PCT, FLOAT_OR_PCT, ANLGE].join(SEP) + ALPHA + "\\)$"
  ), RE_OKLAB = new RegExp(
    "^oklab\\(" + RE_S + [FLOAT_OR_PCT, FLOAT_OR_PCT, FLOAT_OR_PCT].join(SEP) + ALPHA + "\\)$"
  ), RE_OKLCH = new RegExp(
    "^oklch\\(" + RE_S + [FLOAT_OR_PCT, FLOAT_OR_PCT, ANLGE].join(SEP) + ALPHA + "\\)$"
  ), { round: round$9 } = Math, roundRGB = (O) => O.map((_, H) => H <= 2 ? limit$1(round$9(_), 0, 255) : _), percentToAbsolute = (O, _ = 0, H = 100, V = !1) => (typeof O == "string" && O.endsWith("%") && (O = parseFloat(O.substring(0, O.length - 1)) / 100, V ? O = _ + (O + 1) * 0.5 * (H - _) : O = _ + O * (H - _)), +O), noneToValue = (O, _) => O === "none" ? _ : O, css2rgb = (O) => {
    if (O = O.toLowerCase().trim(), O === "transparent")
      return [0, 0, 0, 0];
    let _;
    if (input$2.format.named)
      try {
        return input$2.format.named(O);
      } catch {
      }
    if ((_ = O.match(RE_RGB)) || (_ = O.match(RE_RGB_LEGACY))) {
      let H = _.slice(1, 4);
      for (let q = 0; q < 3; q++)
        H[q] = +percentToAbsolute(noneToValue(H[q], 0), 0, 255);
      H = roundRGB(H);
      const V = _[4] !== void 0 ? +percentToAbsolute(_[4], 0, 1) : 1;
      return H[3] = V, H;
    }
    if (_ = O.match(RE_RGBA_LEGACY)) {
      const H = _.slice(1, 5);
      for (let V = 0; V < 4; V++)
        H[V] = +percentToAbsolute(H[V], 0, 255);
      return H;
    }
    if ((_ = O.match(RE_HSL)) || (_ = O.match(RE_HSL_LEGACY))) {
      const H = _.slice(1, 4);
      H[0] = +noneToValue(H[0].replace("deg", ""), 0), H[1] = +percentToAbsolute(noneToValue(H[1], 0), 0, 100) * 0.01, H[2] = +percentToAbsolute(noneToValue(H[2], 0), 0, 100) * 0.01;
      const V = roundRGB(hsl2rgb$1(H)), q = _[4] !== void 0 ? +percentToAbsolute(_[4], 0, 1) : 1;
      return V[3] = q, V;
    }
    if (_ = O.match(RE_HSLA_LEGACY)) {
      const H = _.slice(1, 4);
      H[1] *= 0.01, H[2] *= 0.01;
      const V = hsl2rgb$1(H);
      for (let q = 0; q < 3; q++)
        V[q] = round$9(V[q]);
      return V[3] = +_[4], V;
    }
    if (_ = O.match(RE_LAB)) {
      const H = _.slice(1, 4);
      H[0] = percentToAbsolute(noneToValue(H[0], 0), 0, 100), H[1] = percentToAbsolute(noneToValue(H[1], 0), -125, 125, !0), H[2] = percentToAbsolute(noneToValue(H[2], 0), -125, 125, !0);
      const V = getLabWhitePoint();
      setLabWhitePoint("d50");
      const q = roundRGB(lab2rgb(H));
      setLabWhitePoint(V);
      const G = _[4] !== void 0 ? +percentToAbsolute(_[4], 0, 1) : 1;
      return q[3] = G, q;
    }
    if (_ = O.match(RE_LCH)) {
      const H = _.slice(1, 4);
      H[0] = percentToAbsolute(H[0], 0, 100), H[1] = percentToAbsolute(noneToValue(H[1], 0), 0, 150, !1), H[2] = +noneToValue(H[2].replace("deg", ""), 0);
      const V = getLabWhitePoint();
      setLabWhitePoint("d50");
      const q = roundRGB(lch2rgb(H));
      setLabWhitePoint(V);
      const G = _[4] !== void 0 ? +percentToAbsolute(_[4], 0, 1) : 1;
      return q[3] = G, q;
    }
    if (_ = O.match(RE_OKLAB)) {
      const H = _.slice(1, 4);
      H[0] = percentToAbsolute(noneToValue(H[0], 0), 0, 1), H[1] = percentToAbsolute(noneToValue(H[1], 0), -0.4, 0.4, !0), H[2] = percentToAbsolute(noneToValue(H[2], 0), -0.4, 0.4, !0);
      const V = roundRGB(oklab2rgb(H)), q = _[4] !== void 0 ? +percentToAbsolute(_[4], 0, 1) : 1;
      return V[3] = q, V;
    }
    if (_ = O.match(RE_OKLCH)) {
      const H = _.slice(1, 4);
      H[0] = percentToAbsolute(noneToValue(H[0], 0), 0, 1), H[1] = percentToAbsolute(noneToValue(H[1], 0), 0, 0.4, !1), H[2] = +noneToValue(H[2].replace("deg", ""), 0);
      const V = roundRGB(oklch2rgb(H)), q = _[4] !== void 0 ? +percentToAbsolute(_[4], 0, 1) : 1;
      return V[3] = q, V;
    }
  };
  css2rgb.test = (O) => (
    // modern
    RE_RGB.test(O) || RE_HSL.test(O) || RE_LAB.test(O) || RE_LCH.test(O) || RE_OKLAB.test(O) || RE_OKLCH.test(O) || // legacy
    RE_RGB_LEGACY.test(O) || RE_RGBA_LEGACY.test(O) || RE_HSL_LEGACY.test(O) || RE_HSLA_LEGACY.test(O) || O === "transparent"
  );
  Color$2.prototype.css = function(O) {
    return rgb2css(this._rgb, O);
  };
  const css = (...O) => new Color$2(...O, "css");
  chroma.css = css;
  input$2.format.css = css2rgb;
  input$2.autodetect.push({
    p: 5,
    test: (O, ..._) => {
      if (!_.length && type$2(O) === "string" && css2rgb.test(O))
        return "css";
    }
  });
  input$2.format.gl = (...O) => {
    const _ = unpack(O, "rgba");
    return _[0] *= 255, _[1] *= 255, _[2] *= 255, _;
  };
  const gl$1 = (...O) => new Color$2(...O, "gl");
  chroma.gl = gl$1;
  Color$2.prototype.gl = function() {
    const O = this._rgb;
    return [O[0] / 255, O[1] / 255, O[2] / 255, O[3]];
  };
  Color$2.prototype.hex = function(O) {
    return rgb2hex(this._rgb, O);
  };
  const hex$3 = (...O) => new Color$2(...O, "hex");
  chroma.hex = hex$3;
  input$2.format.hex = hex2rgb;
  input$2.autodetect.push({
    p: 4,
    test: (O, ..._) => {
      if (!_.length && type$2(O) === "string" && [3, 4, 5, 6, 7, 8, 9].indexOf(O.length) >= 0)
        return "hex";
    }
  });
  const { log: log$8 } = Math, temperature2rgb = (O) => {
    const _ = O / 100;
    let H, V, q;
    return _ < 66 ? (H = 255, V = _ < 6 ? 0 : -155.25485562709179 - 0.44596950469579133 * (V = _ - 2) + 104.49216199393888 * log$8(V), q = _ < 20 ? 0 : -254.76935184120902 + 0.8274096064007395 * (q = _ - 10) + 115.67994401066147 * log$8(q)) : (H = 351.97690566805693 + 0.114206453784165 * (H = _ - 55) - 40.25366309332127 * log$8(H), V = 325.4494125711974 + 0.07943456536662342 * (V = _ - 50) - 28.0852963507957 * log$8(V), q = 255), [H, V, q, 1];
  }, { round: round$8 } = Math, rgb2temperature = (...O) => {
    const _ = unpack(O, "rgb"), H = _[0], V = _[2];
    let q = 1e3, G = 4e4;
    const J = 0.4;
    let Z;
    for (; G - q > J; ) {
      Z = (G + q) * 0.5;
      const te = temperature2rgb(Z);
      te[2] / te[0] >= V / H ? G = Z : q = Z;
    }
    return round$8(Z);
  };
  Color$2.prototype.temp = Color$2.prototype.kelvin = Color$2.prototype.temperature = function() {
    return rgb2temperature(this._rgb);
  };
  const temp$1 = (...O) => new Color$2(...O, "temp");
  Object.assign(chroma, { temp: temp$1, kelvin: temp$1, temperature: temp$1 });
  input$2.format.temp = input$2.format.kelvin = input$2.format.temperature = temperature2rgb;
  Color$2.prototype.oklch = function() {
    return rgb2oklch(this._rgb);
  };
  const oklch = (...O) => new Color$2(...O, "oklch");
  Object.assign(chroma, { oklch });
  input$2.format.oklch = oklch2rgb;
  input$2.autodetect.push({
    p: 2,
    test: (...O) => {
      if (O = unpack(O, "oklch"), type$2(O) === "array" && O.length === 3)
        return "oklch";
    }
  });
  Object.assign(chroma, {
    analyze: analyze$1,
    average,
    bezier: bezier$2,
    blend: blend$1,
    brewer: colorbrewerProxy,
    Color: Color$2,
    colors: w3cx11,
    contrast: contrast$1,
    contrastAPCA,
    cubehelix: cubehelix$3,
    deltaE,
    distance,
    input: input$2,
    interpolate: mix,
    limits,
    mix,
    random: random$3,
    scale: scale$5,
    scales: scales$1,
    valid
  });
  const getCssVar = (O) => getComputedStyle(document.body).getPropertyValue(O).trim().replace(/["']/g, ""), setTheme = (O) => {
    const _ = O ? "rgb(209, 213, 219)" : "rgb(107, 114, 128)", H = O ? "rgb(107, 114, 128)" : "rgb(55, 65, 81)", V = getCssVar("--ar-accent-color"), q = document.documentElement;
    try {
      const G = chroma(V);
      q.style.setProperty(
        "--ar-accent-secondary-color",
        G.alpha(0.14).hex()
      ), q.style.setProperty(
        "--ar-accent-text",
        G.get("lab.l") < 70 ? "white" : "black"
      ), q.style.setProperty("--ar-light-gray", _), q.style.setProperty("--ar-dark-gray", H);
    } catch (G) {
      console.error("An error occurred while setting a theme property: ", G);
    }
  }, _hoisted_1$9 = { class: "bg-red-100 p-4 mt-4" }, _sfc_main$c = /* @__PURE__ */ defineComponent({
    __name: "ErrorCallout",
    props: {
      error: {}
    },
    setup(O) {
      return (_, H) => (openBlock(), createElementBlock("div", _hoisted_1$9, [
        H[0] || (H[0] = createBaseVNode("h3", { class: "flex justify-start items-center" }, [
          createBaseVNode("i", { class: "fa fa-exclamation-circle mr-1" }),
          createBaseVNode("div", null, "Something went wrong")
        ], -1)),
        createBaseVNode("p", null, toDisplayString(_.error), 1)
      ]));
    }
  });
  function render$1(O, _) {
    return openBlock(), createElementBlock("svg", {
      xmlns: "http://www.w3.org/2000/svg",
      viewBox: "0 0 20 20",
      fill: "currentColor",
      "aria-hidden": "true",
      "data-slot": "icon"
    }, [
      createBaseVNode("path", {
        "fill-rule": "evenodd",
        d: "M5.22 8.22a.75.75 0 0 1 1.06 0L10 11.94l3.72-3.72a.75.75 0 1 1 1.06 1.06l-4.25 4.25a.75.75 0 0 1-1.06 0L5.22 9.28a.75.75 0 0 1 0-1.06Z",
        "clip-rule": "evenodd"
      })
    ]);
  }
  const _hoisted_1$8 = { class: "flex h-16 justify-between" }, _hoisted_2$5 = { class: "flex" }, _hoisted_3$4 = {
    key: 0,
    class: "hidden sm:ml-6 sm:flex sm:space-x-8"
  }, _hoisted_4$3 = ["data-cy", "onClick"], _hoisted_5$2 = {
    key: 0,
    class: "ml-6 flex items-center"
  }, _hoisted_6 = { class: "relative" }, _hoisted_7 = {
    key: 0,
    class: "absolute right-0 z-10 mt-2 w-56 origin-top-right divide-y divide-gray-100 rounded-md bg-white shadow-lg ring-1 ring-black ring-opacity-5 focus:outline-none",
    role: "menu",
    "aria-orientation": "vertical",
    "aria-labelledby": "user-menu-button",
    tabindex: "-1"
  }, _hoisted_8 = {
    class: "py-1",
    role: "none"
  }, _hoisted_9 = ["href"], _hoisted_10 = { class: "space-y-1 pt-2 pb-3" }, _hoisted_11 = ["data-cy", "onClick"], _sfc_main$b = /* @__PURE__ */ defineComponent({
    __name: "NavBar",
    props: {
      labels: {},
      pageNumber: {},
      reportWidthClass: {}
    },
    emits: ["page-change"],
    setup(O, { emit: _ }) {
      const H = O, V = _, q = ref$1(!1), G = ref$1(!1), J = ref$1(), Z = (ae) => {
        ae.target instanceof Node && J.value && !J.value.contains(ae.target) && (q.value = !1);
      };
      onMounted(() => {
        document.addEventListener("click", Z);
      }), onUnmounted(() => {
        document.removeEventListener("click", Z);
      });
      const te = !window.arLocal, ne = computed(() => H.labels.length > 6), ee = [
        { label: "Documentation", href: "https://ninoseki.github.io/arakawa/" }
      ];
      return (ae, oe) => (openBlock(), createElementBlock("nav", {
        class: normalizeClass([
          // z-40 to be below modals and side bars on datapane.com
          "bg-white relative z-40",
          { "border-b border-gray-100 sticky top-0": ae.labels.length > 0 }
        ])
      }, [
        createBaseVNode("div", {
          class: normalizeClass(["mx-auto px-4", ae.reportWidthClass])
        }, [
          createBaseVNode("div", _hoisted_1$8, [
            createBaseVNode("div", _hoisted_2$5, [
              ne.value ? createCommentVNode("", !0) : (openBlock(), createElementBlock("div", _hoisted_3$4, [
                (openBlock(!0), createElementBlock(Fragment, null, renderList(ae.labels, (se, ce) => (openBlock(), createElementBlock("a", {
                  key: ce,
                  "data-cy": `page-${ce}`,
                  class: normalizeClass([
                    "inline-flex items-center border-b-2 px-1 pt-1 text-sm font-medium cursor-pointer",
                    {
                      "border-transparent text-gray-500 hover:border-gray-300 hover:text-gray-700": ce !== ae.pageNumber,
                      "border-ar-accent text-gray-900": ce === ae.pageNumber
                    }
                  ]),
                  onClick: (le) => V("page-change", ce)
                }, toDisplayString(se), 11, _hoisted_4$3))), 128))
              ])),
              ae.labels.length > 0 ? (openBlock(), createElementBlock("div", {
                key: 1,
                class: normalizeClass(["ml-6 flex", { "sm:hidden": !ne.value }])
              }, [
                createBaseVNode("a", {
                  class: "group cursor-pointer inline-flex items-center rounded-md bg-white text-base font-medium hover:text-gray-900 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2 text-gray-500",
                  onClick: oe[0] || (oe[0] = (se) => G.value = !G.value)
                }, [
                  createTextVNode(toDisplayString(ae.labels.length) + " pages ", 1),
                  createVNode(unref(render$1), {
                    class: normalizeClass([
                      "ml-2 h-5 w-5 group-hover:text-gray-500 text-gray-400",
                      {
                        "text-gray-600": G.value,
                        "text-gray-400": !G.value
                      }
                    ])
                  }, null, 8, ["class"])
                ])
              ], 2)) : createCommentVNode("", !0)
            ]),
            te ? (openBlock(), createElementBlock("div", _hoisted_5$2, [
              createBaseVNode("div", _hoisted_6, [
                createBaseVNode("button", {
                  type: "button",
                  class: "rounded-full bg-white p-1 text-gray-400 hover:text-gray-500 focus:outline-none",
                  ref_key: "dropdownBtnEl",
                  ref: J,
                  onClick: oe[1] || (oe[1] = (se) => q.value = !q.value)
                }, [
                  createVNode(unref(render$3), { class: "h-7 w-7 hover:transform hover:rotate-180" })
                ], 512),
                q.value ? (openBlock(), createElementBlock("div", _hoisted_7, [
                  createBaseVNode("div", _hoisted_8, [
                    (openBlock(), createElementBlock(Fragment, null, renderList(ee, (se, ce) => createBaseVNode("a", {
                      tabindex: "-1",
                      key: ce,
                      href: se.href,
                      target: "_blank",
                      class: "block px-4 py-2 text-sm text-gray-700 cursor-pointer hover:bg-gray-100 hover:text-gray-900"
                    }, toDisplayString(se.label), 9, _hoisted_9)), 64))
                  ])
                ])) : createCommentVNode("", !0)
              ])
            ])) : createCommentVNode("", !0)
          ]),
          G.value ? (openBlock(), createElementBlock("div", {
            key: 0,
            class: normalizeClass({ "sm:hidden": !ne.value })
          }, [
            createBaseVNode("div", _hoisted_10, [
              (openBlock(!0), createElementBlock(Fragment, null, renderList(ae.labels, (se, ce) => (openBlock(), createElementBlock("a", {
                key: ce,
                "data-cy": `page-${ce}`,
                class: normalizeClass([
                  "cursor-pointer block border-l-4 py-2 pl-3 pr-4 text-base font-medium",
                  {
                    "border-transparent text-gray-500 hover:border-gray-300 hover:bg-gray-50 hover:text-gray-700": ce !== ae.pageNumber,
                    "border-indigo-500 bg-indigo-50 text-indigo-700": ce === ae.pageNumber
                  }
                ]),
                onClick: (le) => V("page-change", ce)
              }, toDisplayString(se), 11, _hoisted_11))), 128))
            ])
          ], 2)) : createCommentVNode("", !0)
        ], 2)
      ], 2));
    }
  }), _hoisted_1$7 = { class: "flex flex-col justify-end bg-ar-background" }, _hoisted_2$4 = { class: "sm:flex block" }, _hoisted_3$3 = { class: "flex-1 flex flex-col min-w-0" }, _hoisted_4$2 = { class: "grow px-4" }, _sfc_main$a = /* @__PURE__ */ defineComponent({
    __name: "ReportComponent",
    props: {
      reportWidthClass: {},
      mode: {},
      htmlHeader: {},
      report: {}
    },
    setup(O) {
      const _ = O, H = useRootStore(), { singleBlockEmbed: V } = storeToRefs(H), { children: q, tabNumber: G, hasPages: J } = storeToRefs(_.report.store), Z = computed(
        () => J.value ? q.value[0].children : []
      ), te = computed(
        () => Z.value.map((oe, se) => oe.label || `Page ${se + 1}`)
      ), ne = computed(
        () => J.value ? [Z.value[G.value]] : q.value
      ), ee = (oe) => _.report.store.setTab(oe), { isIPythonEmbed: ae } = window;
      return (oe, se) => oe.report ? (openBlock(), createElementBlock(Fragment, { key: 0 }, [
        unref(J) || !(unref(ae) || unref(V)) ? (openBlock(), createBlock(_sfc_main$b, {
          key: 0,
          labels: te.value,
          "page-number": unref(G),
          "report-width-class": oe.reportWidthClass,
          onPageChange: ee
        }, null, 8, ["labels", "page-number", "report-width-class"])) : createCommentVNode("", !0),
        createBaseVNode("main", {
          class: normalizeClass(["w-full bg-ar-background mx-auto pb-4", oe.reportWidthClass]),
          "data-cy": "report-component"
        }, [
          createBaseVNode("div", _hoisted_1$7, [
            createBaseVNode("div", _hoisted_2$4, [
              createBaseVNode("div", _hoisted_3$3, [
                createBaseVNode("div", _hoisted_4$2, [
                  (openBlock(!0), createElementBlock(Fragment, null, renderList(ne.value, (ce) => (openBlock(), createBlock(resolveDynamicComponent(ce.component), mergeProps$1({
                    key: ce.refId,
                    ref_for: !0
                  }, ce.componentProps, { class: "py-4" }), null, 16))), 128))
                ])
              ])
            ])
          ])
        ], 2)
      ], 64)) : createCommentVNode("", !0);
    }
  }), _hoisted_1$6 = ["innerHTML"], _sfc_main$9 = /* @__PURE__ */ defineComponent({
    __name: "ReportContainer",
    props: {
      isLightProse: { type: Boolean },
      reportWidthClass: {},
      mode: {},
      id: {},
      htmlHeader: {},
      appData: {}
    },
    setup(O) {
      const _ = O, H = useRootStore(), V = ref$1();
      (async () => {
        try {
          await H.setReport(
            { isLightProse: _.isLightProse, mode: _.mode },
            _.appData
          );
        } catch (Z) {
          V.value = parseError(Z), console.error(Z);
        }
      })();
      const G = storeToRefs(H), { report: J } = G;
      return onMounted(() => {
        setTheme(_.isLightProse);
      }), (Z, te) => (openBlock(), createElementBlock(Fragment, null, [
        createBaseVNode("div", {
          id: "html-header",
          innerHTML: Z.htmlHeader
        }, null, 8, _hoisted_1$6),
        unref(isView)(unref(J)) && !V.value ? (openBlock(), createBlock(_sfc_main$a, {
          "report-width-class": Z.reportWidthClass,
          mode: Z.mode,
          report: unref(J),
          key: unref(J).refId
        }, null, 8, ["report-width-class", "mode", "report"])) : createCommentVNode("", !0),
        V.value ? (openBlock(), createBlock(_sfc_main$c, {
          key: 1,
          error: V.value
        }, null, 8, ["error"])) : createCommentVNode("", !0)
      ], 64));
    }
  });
  Promise.resolve().then(() => tailwind);
  customElements.define("x-table-block", /* @__PURE__ */ defineCustomElement(TableBlock$1));
  window.addEventListener("DOMContentLoaded", () => {
    const { reportProps: O } = window, _ = createApp(_sfc_main$9, O), H = createPinia();
    return _.use(H), _.use(plugin, defaultConfig$3(formkitConfig)), _.mount("#report"), _;
  });
  const version$6 = "3.4.3", { toString: toString$3 } = Object.prototype;
  function is_undefined(O) {
    return typeof O > "u";
  }
  function is_defined(O) {
    return typeof O < "u";
  }
  function is_nullish(O) {
    return O == null;
  }
  function isNotNull(O) {
    return O != null;
  }
  const non_null = isNotNull;
  function isBoolean$4(O) {
    return O === !0 || O === !1 || toString$3.call(O) === "[object Boolean]";
  }
  function isNumber$3(O) {
    return toString$3.call(O) === "[object Number]";
  }
  function isInteger$2(O) {
    return isNumber$3(O) && Number.isInteger(O);
  }
  function isString$2(O) {
    return toString$3.call(O) === "[object String]";
  }
  function isSymbol$1(O) {
    return typeof O == "symbol";
  }
  function isPrimitive(O) {
    return O === null || isBoolean$4(O) || isNumber$3(O) || isString$2(O) || isSymbol$1(O);
  }
  function isFunction$3(O) {
    switch (toString$3.call(O)) {
      case "[object Function]":
      case "[object AsyncFunction]":
      case "[object GeneratorFunction]":
      case "[object AsyncGeneratorFunction]":
        return !0;
      default:
        return !1;
    }
  }
  function isArray$3(O) {
    return Array.isArray(O);
  }
  function isArrayOf(O, _) {
    for (const H of O)
      if (!_(H))
        return !1;
    return !0;
  }
  function isArrayableOf(O, _) {
    for (const H of O)
      if (!_(H))
        return !1;
    return !0;
  }
  function isTypedArray$2(O) {
    return ArrayBuffer.isView(O) && !(O instanceof DataView);
  }
  function isObject$3(O) {
    const _ = typeof O;
    return _ === "function" || _ === "object" && !!O;
  }
  function isPlainObject(O) {
    return isObject$3(O) && (is_nullish(O.constructor) || O.constructor === Object);
  }
  function isIterable$2(O) {
    return isObject$3(O) && Symbol.iterator in O;
  }
  class AssertionError extends Error {
  }
  mt(AssertionError, "__name__", "AssertionError");
  class UnreachableError extends Error {
  }
  mt(UnreachableError, "__name__", "UnreachableError");
  function assert$2(O, _) {
    if (!(O === !0 || O !== !1 && O()))
      throw new AssertionError(_ ?? "Assertion failed");
  }
  function assert_debug(O, _) {
    typeof DEBUG < "u" && DEBUG && assert$2(O, _);
  }
  function unreachable(O) {
    const _ = O != null ? `: ${O}` : "";
    throw new UnreachableError(`unreachable code${_}`);
  }
  const { PI: PI$1, abs: abs$d, sign: sign$3 } = Math;
  function angle_norm(O) {
    if (O == 0)
      return 0;
    for (; O <= 0; )
      O += 2 * PI$1;
    for (; O > 2 * PI$1; )
      O -= 2 * PI$1;
    return O;
  }
  function angle_dist(O, _) {
    return angle_norm(O - _);
  }
  function angle_between(O, _, H, V = !1) {
    const q = angle_dist(_, H);
    if (q == 0)
      return !1;
    if (q == 2 * PI$1)
      return !0;
    const G = angle_norm(O), J = angle_dist(_, G) <= q && angle_dist(G, H) <= q;
    return V ? !J : J;
  }
  function atan2$2(O, _) {
    return Math.atan2(_[1] - O[1], _[0] - O[0]);
  }
  function compute_angle(O, _, H = "anticlock") {
    return -(H == "anticlock" ? 1 : -1) * O * to_radians_coeff(_);
  }
  function invert_angle(O, _, H = "anticlock") {
    return -(H == "anticlock" ? 1 : -1) * O / to_radians_coeff(_);
  }
  function to_radians_coeff(O) {
    switch (O) {
      case "deg":
        return PI$1 / 180;
      case "rad":
        return 1;
      case "grad":
        return PI$1 / 200;
      case "turn":
        return 2 * PI$1;
    }
  }
  function minmax(O, _) {
    return O <= _ ? [O, _] : [_, O];
  }
  function clamp$1(O, _, H) {
    return O < _ ? _ : O > H ? H : O;
  }
  function cycle(O, _, H) {
    return O > H ? _ : O < _ ? H : O;
  }
  function log$7(O, _ = Math.E) {
    return Math.log(O) / Math.log(_);
  }
  const float = Symbol("float");
  function is_Floating(O) {
    return isObject$3(O) && float in O;
  }
  const float32_epsilon = 11920928955078125e-23;
  function factorial(O) {
    let _ = 1;
    for (let H = 2; H <= O; H++)
      _ *= H;
    return _;
  }
  function hermite(O) {
    const _ = new Array(O + 1);
    _.fill(0);
    const H = factorial(O);
    for (let V = 0; V <= Math.floor(O / 2); V++) {
      const q = (-1) ** V * H / (factorial(V) * factorial(O - 2 * V)) * 2 ** (O - 2 * V);
      _[2 * V] = q;
    }
    return _;
  }
  function eval_poly(O, _) {
    const H = O.length - 1;
    let V = 0, q = 1;
    for (let G = H; G >= 0; G--)
      V += q * O[G], q *= _;
    return V;
  }
  const { floor: floor$3 } = Math;
  function is_empty$1(O) {
    return O.length == 0;
  }
  function is_sorted(O) {
    const _ = O.length;
    if (_ == 0)
      return !0;
    let H = O[0];
    for (let V = 1; V < _; V++) {
      const q = O[V];
      if (H <= q)
        H = q;
      else
        return !1;
    }
    return !0;
  }
  function copy$4(O) {
    return Array.isArray(O) ? O.slice() : new O.constructor(O);
  }
  function splice$1(O, _, H, ...V) {
    if (Array.isArray(O)) {
      const te = copy$4(O);
      return H === void 0 ? te.splice(_) : te.splice(_, H, ...V), te;
    }
    const q = O.length;
    _ < 0 && (_ += q), _ < 0 ? _ = 0 : _ > q && (_ = q), H == null || H > q - _ ? H = q - _ : H < 0 && (H = 0);
    const G = q - H + V.length, J = new O.constructor(G);
    let Z = 0;
    for (; Z < _; Z++)
      J[Z] = O[Z];
    for (const te of V)
      J[Z++] = te;
    for (let te = _ + H; te < q; te++)
      J[Z++] = O[te];
    return J;
  }
  function head(O, _) {
    return splice$1(O, _, O.length - _);
  }
  function prepend$1(O, _) {
    return splice$1(O, 0, 0, _);
  }
  function index_of(O, _) {
    return O.indexOf(_);
  }
  function includes(O, _) {
    return O.indexOf(_) !== -1;
  }
  const contains$6 = includes;
  function mul(O, _, H) {
    const V = O.length, q = H ?? new O.constructor(V);
    for (let G = 0; G < V; G++)
      q[G] = O[G] * _;
    return q;
  }
  function map$4(O, _) {
    const H = O.length, V = new O.constructor(H);
    for (let q = 0; q < H; q++)
      V[q] = _(O[q], q, O);
    return V;
  }
  function inplace_map(O, _, H) {
    const V = O.length, q = O;
    for (let G = 0; G < V; G++)
      q[G] = _(O[G], G);
  }
  function filter$2(O, _) {
    const H = O.length, V = new O.constructor(H);
    let q = 0;
    for (let G = 0; G < H; G++) {
      const J = O[G];
      _(J, G, O) && (V[q++] = J);
    }
    return head(V, q);
  }
  function reduce$3(O, _, H) {
    const V = O.length;
    let q, G;
    for (q = H, G = 0; G < V; G++)
      q = _(q, O[G], G, O);
    return q;
  }
  function sort_by(O, _) {
    const H = Array.from(O, (V, q) => ({ index: q, key: _(V) }));
    return H.sort((V, q) => {
      const G = V.key, J = q.key;
      if (G !== J) {
        if (G > J)
          return 1;
        if (G < J)
          return -1;
      }
      return V.index - q.index;
    }), map$4(O, (V, q) => O[H[q].index]);
  }
  function min$b(O) {
    let _ = 1 / 0;
    for (const H of O)
      !isNaN(H) && H < _ && (_ = H);
    return _;
  }
  function max$j(O) {
    let _ = -1 / 0;
    for (const H of O)
      !isNaN(H) && H > _ && (_ = H);
    return _;
  }
  function minmax2(O, _) {
    let H, V, q = 1 / 0, G = -1 / 0, J = 1 / 0, Z = -1 / 0;
    const te = Math.min(O.length, _.length);
    for (let ne = 0; ne < te; ne++)
      H = O[ne], V = _[ne], !isNaN(H) && !isNaN(V) && (H < q && (q = H), H > G && (G = H), V < J && (J = V), V > Z && (Z = V));
    return [q, G, J, Z];
  }
  function min_by(O, _) {
    if (O.length == 0)
      throw new Error("min_by() called with an empty array");
    let H = O[0], V = _(H, 0);
    for (let q = 1, G = O.length; q < G; q++) {
      const J = O[q], Z = _(J, q);
      Z < V && (H = J, V = Z);
    }
    return H;
  }
  function max_by(O, _) {
    if (O.length == 0)
      throw new Error("max_by() called with an empty array");
    let H = O[0], V = _(H, 0);
    for (let q = 1, G = O.length; q < G; q++) {
      const J = O[q], Z = _(J, q);
      Z > V && (H = J, V = Z);
    }
    return H;
  }
  function sum$3(O) {
    let _ = 0;
    for (let H = 0, V = O.length; H < V; H++)
      _ += O[H];
    return _;
  }
  function cumsum(O) {
    const _ = new O.constructor(O.length);
    return reduce$3(O, (H, V, q) => _[q] = H + V, 0), _;
  }
  function every$2(O, _) {
    for (const H of O)
      if (!_(H))
        return !1;
    return !0;
  }
  function some$2(O, _) {
    for (const H of O)
      if (_(H))
        return !0;
    return !1;
  }
  function _find_index(O) {
    return function(_, H) {
      const V = _.length;
      let q = O > 0 ? 0 : V - 1;
      for (; q >= 0 && q < V; q += O)
        if (H(_[q]))
          return q;
      return -1;
    };
  }
  const find_index = _find_index(1), find_last_index = _find_index(-1);
  function find$4(O, _) {
    const H = find_index(O, _);
    return H == -1 ? void 0 : O[H];
  }
  function bisect_left_by(O, _, H, V = 0, q = O.length) {
    for (assert_debug(() => is_sorted(O)), assert$2(0 <= V && q <= O.length); V < q; ) {
      const G = floor$3((V + q) / 2);
      H(O[G]) < _ ? V = G + 1 : q = G;
    }
    return V;
  }
  function bisect_right_by(O, _, H, V = 0, q = O.length) {
    for (assert_debug(() => is_sorted(O)), assert$2(0 <= V && q <= O.length); V < q; ) {
      const G = floor$3((V + q) / 2);
      H(O[G]) <= _ ? V = G + 1 : q = G;
    }
    return V;
  }
  function bisect_left(O, _, H = 0, V) {
    return bisect_left_by(O, _, (q) => q, H, V);
  }
  function bisect_right(O, _, H = 0, V) {
    return bisect_right_by(O, _, (q) => q, H, V);
  }
  const sorted_index = bisect_left;
  function bin_counts(O, _) {
    const H = _.length - 1, V = Array(H).fill(0);
    for (let q = 0; q < O.length; q++) {
      const G = O[q], J = sorted_index(_, G), Z = clamp$1(J - 1, 0, H - 1);
      V[Z] += 1;
    }
    return V;
  }
  function interpolate$2(O, _, H) {
    const V = O.length, q = new Array(V);
    for (let G = 0; G < V; G++) {
      const J = O[G];
      if (isNaN(J) || _.length == 0) {
        q[G] = NaN;
        continue;
      }
      const Z = left_edge_index(J, _);
      if (Z == -1)
        q[G] = H[0];
      else if (Z == _.length)
        q[G] = H[H.length - 1];
      else if (Z == _.length - 1 || _[Z] == J)
        q[G] = H[Z];
      else {
        const te = _[Z], ne = H[Z], ee = _[Z + 1], ae = H[Z + 1];
        q[G] = lerp$1(J, te, ne, ee, ae);
      }
    }
    return q;
  }
  function lerp$1(O, _, H, V, q) {
    const G = (q - H) / (V - _);
    let J = G * (O - _) + H;
    return isFinite(J) || (J = G * (O - V) + q, !isFinite(J) && H == q && (J = H)), J;
  }
  function left_edge_index(O, _) {
    if (O < _[0])
      return -1;
    if (O > _[_.length - 1])
      return _.length;
    if (_.length == 1)
      return 0;
    let H = 0, V = _.length - 1;
    for (; V - H != 1; ) {
      const q = H + Math.floor((V - H) / 2);
      O >= _[q] ? H = q : V = q;
    }
    return H;
  }
  const { slice: slice$6 } = Array.prototype;
  function copy$3(O) {
    return slice$6.call(O);
  }
  function concat$1(O) {
    return [].concat(...O);
  }
  function nth(O, _) {
    return O[_ >= 0 ? _ : O.length + _];
  }
  function unzip(O) {
    const _ = O.length;
    if (_ == 0)
      return [];
    const H = min$b(O.map((q) => q.length)), V = Array(H);
    for (let q = 0; q < H; q++)
      V[q] = new Array(_);
    for (let q = 0; q < _; q++)
      for (let G = 0; G < H; G++)
        V[G][q] = O[q][G];
    return V;
  }
  function range$9(O, _, H = 1) {
    assert$2(H > 0, "'step' must be a positive number"), _ == null && (_ = O, O = 0);
    const { max: V, ceil: q, abs: G } = Math, J = O <= _ ? H : -H, Z = V(q(G(_ - O) / H), 0), te = new Array(Z);
    for (let ne = 0; ne < Z; ne++, O += J)
      te[ne] = O;
    return te;
  }
  function linspace(O, _, H = 100) {
    const V = H == 1 ? 0 : (_ - O) / (H - 1), q = new Array(H);
    for (let G = 0; G < H; G++)
      q[G] = O + V * G;
    return q;
  }
  function argmin(O) {
    return min_by(range$9(O.length), (_) => O[_]);
  }
  function uniq(O) {
    const _ = /* @__PURE__ */ new Set();
    for (const H of O)
      _.add(H);
    return [..._];
  }
  function _union(O) {
    const _ = /* @__PURE__ */ new Set();
    for (const H of O)
      for (const V of H)
        _.add(V);
    return _;
  }
  function union$3(...O) {
    return [..._union(O)];
  }
  function intersection$1(O, ..._) {
    const H = [];
    e: for (const V of O)
      if (!includes(H, V)) {
        for (const q of _)
          if (!includes(q, V))
            continue e;
        H.push(V);
      }
    return H;
  }
  function difference$1(O, ..._) {
    const H = _union(_);
    return filter$2(O, (V) => !H.has(V));
  }
  function symmetric_difference(O, _) {
    const H = new Set(O), V = new Set(_), q = [];
    for (const G of H)
      V.has(G) || q.push(G);
    for (const G of V)
      H.has(G) || q.push(G);
    return q;
  }
  function remove_at(O, _) {
    assert$2(isInteger$2(_) && _ >= 0);
    const H = copy$3(O);
    return H.splice(_, 1), H;
  }
  function remove$3(O, _) {
    remove_by(O, (H) => H == _);
  }
  function remove_by(O, _) {
    for (let H = 0; H < O.length; )
      _(O[H]) ? O.splice(H, 1) : H++;
  }
  function clear$1(O) {
    O.splice(0, O.length);
  }
  function split(O, _) {
    const H = [], V = O.length;
    let q = 0, G = 0;
    for (; G < V; )
      O[G] === _ ? (H.push(O.slice(q, G)), q = ++G) : ++G;
    return H.push(O.slice(q)), H;
  }
  function elementwise(O, _, H) {
    const V = Math.min(O.length, _.length), q = Array(V);
    for (let G = 0; G < V; G++)
      q[G] = H(O[G], _[G]);
    return q;
  }
  function reversed(O) {
    const _ = O.length, H = new Array(_);
    for (let V = 0; V < _; V++)
      H[_ - V - 1] = O[V];
    return H;
  }
  function repeat$1(O, _) {
    return new Array(_).fill(O);
  }
  function resize$1(O, _, H) {
    if (O.length >= _)
      return O.slice(0, _);
    {
      const V = new Array(_ - O.length);
      return V.fill(H), O.concat(V);
    }
  }
  const { assign } = Object, extend$5 = assign;
  function to_object(O) {
    return isPlainObject(O) ? O : Object.fromEntries(O);
  }
  function keys$5(O) {
    return O instanceof Map ? [...O.keys()] : Object.keys(O);
  }
  function values$3(O) {
    return O instanceof Map ? [...O.values()] : Object.values(O);
  }
  function entries$2(O) {
    return O instanceof Map ? [...O.entries()] : Object.entries(O);
  }
  const typed_keys = Object.keys, typed_values = Object.values, typed_entries = Object.entries;
  function clone$1(O) {
    return O instanceof Map ? new Map(O) : { ...O };
  }
  function merge$4(O, _) {
    const H = /* @__PURE__ */ new Map(), V = [...O.keys(), ..._.keys()];
    for (const q of V) {
      const G = O.get(q), J = _.get(q), Z = G === void 0 ? [] : G, te = J === void 0 ? [] : J;
      H.set(q, union$3(Z, te));
    }
    return H;
  }
  function size$2(O) {
    return O instanceof Map ? O.size : Object.keys(O).length;
  }
  function is_empty(O) {
    return size$2(O) == 0;
  }
  const { hasOwnProperty: hasOwnProperty$c } = Object.prototype;
  var My;
  class PlainObjectProxy {
    constructor(_) {
      mt(this, "obj");
      mt(this, My, "PlainObjectProxy");
      this.obj = _;
    }
    clear() {
      for (const _ of this.keys())
        delete this.obj[_];
    }
    delete(_) {
      const H = this.has(_);
      return H && delete this.obj[_], H;
    }
    has(_) {
      return hasOwnProperty$c.call(this.obj, _);
    }
    get(_) {
      return this.has(_) ? this.obj[_] : void 0;
    }
    set(_, H) {
      return this.obj[_] = H, this;
    }
    get size() {
      return size$2(this.obj);
    }
    [(My = Symbol.toStringTag, Symbol.iterator)]() {
      return this.entries();
    }
    *keys() {
      yield* keys$5(this.obj);
    }
    *values() {
      yield* values$3(this.obj);
    }
    *entries() {
      yield* entries$2(this.obj);
    }
    forEach(_, H) {
      for (const [V, q] of this.entries())
        _.call(H, q, V, this);
    }
  }
  mt(PlainObjectProxy, "__name__", "PlainObjectProxy");
  function dict(O) {
    return isPlainObject(O) ? new PlainObjectProxy(O) : O;
  }
  const channel = new MessageChannel(), tasks = /* @__PURE__ */ new Map();
  channel.port1.onmessage = (O) => {
    const _ = O.data, H = tasks.get(_);
    if (H != null)
      try {
        H();
      } finally {
        tasks.delete(_);
      }
  };
  let counter$1 = 1;
  function defer() {
    return new Promise((O) => {
      const _ = counter$1++;
      tasks.set(_, O), channel.port2.postMessage(_);
    });
  }
  class Signal {
    constructor(_, H) {
      mt(this, "sender");
      mt(this, "name");
      this.sender = _, this.name = H;
    }
    connect(_, H = null) {
      receivers_for_sender.has(this.sender) || receivers_for_sender.set(this.sender, []);
      const V = receivers_for_sender.get(this.sender);
      if (find_connection(V, this, _, H) != null)
        return !1;
      const q = H ?? _;
      senders_for_receiver.has(q) || senders_for_receiver.set(q, []);
      const G = senders_for_receiver.get(q), J = { signal: this, slot: _, context: H };
      return V.push(J), G.push(J), !0;
    }
    disconnect(_, H = null) {
      const V = receivers_for_sender.get(this.sender);
      if (V == null || V.length === 0)
        return !1;
      const q = find_connection(V, this, _, H);
      if (q == null)
        return !1;
      const G = H ?? _, J = senders_for_receiver.get(G);
      return q.signal = null, schedule_cleanup(V), schedule_cleanup(J), !0;
    }
    emit(_) {
      const H = receivers_for_sender.get(this.sender) ?? [];
      for (const { signal: V, slot: q, context: G } of H)
        V === this && q.call(G, _, this.sender);
    }
  }
  mt(Signal, "__name__", "Signal");
  class Signal0 extends Signal {
    emit() {
      super.emit(void 0);
    }
  }
  mt(Signal0, "__name__", "Signal0");
  (function(O) {
    function _(G, J) {
      const Z = receivers_for_sender.get(G);
      if (Z == null || Z.length === 0)
        return;
      const te = senders_for_receiver.get(J);
      if (!(te == null || te.length === 0)) {
        for (const ne of te) {
          if (ne.signal == null)
            return;
          ne.signal.sender === G && (ne.signal = null);
        }
        schedule_cleanup(Z), schedule_cleanup(te);
      }
    }
    O.disconnect_between = _;
    function H(G) {
      const J = receivers_for_sender.get(G);
      if (!(J == null || J.length === 0)) {
        for (const Z of J) {
          if (Z.signal == null)
            return;
          const te = Z.context ?? Z.slot;
          Z.signal = null, schedule_cleanup(senders_for_receiver.get(te));
        }
        schedule_cleanup(J);
      }
    }
    O.disconnect_sender = H;
    function V(G, J, Z) {
      const te = senders_for_receiver.get(G);
      if (!(te == null || te.length === 0)) {
        for (const ne of te) {
          if (ne.signal == null)
            return;
          if (J != null && ne.slot != J)
            continue;
          const ee = ne.signal.sender;
          Z != null && Z.has(ee) || (ne.signal = null, schedule_cleanup(receivers_for_sender.get(ee)));
        }
        schedule_cleanup(te);
      }
    }
    O.disconnect_receiver = V;
    function q(G) {
      const J = receivers_for_sender.get(G);
      if (J != null && J.length !== 0) {
        for (const te of J)
          te.signal = null;
        schedule_cleanup(J);
      }
      const Z = senders_for_receiver.get(G);
      if (Z != null && Z.length !== 0) {
        for (const te of Z)
          te.signal = null;
        schedule_cleanup(Z);
      }
    }
    O.disconnect_all = q;
  })(Signal);
  function Signalable() {
    return class {
      connect(O, _) {
        return O.connect(_, this);
      }
      disconnect(O, _) {
        return O.disconnect(_, this);
      }
    };
  }
  const receivers_for_sender = /* @__PURE__ */ new WeakMap(), senders_for_receiver = /* @__PURE__ */ new WeakMap();
  function find_connection(O, _, H, V) {
    return find$4(O, (q) => q.signal === _ && q.slot === H && q.context === V);
  }
  const dirty_set = /* @__PURE__ */ new Set();
  function schedule_cleanup(O) {
    dirty_set.size == 0 && (async () => (await defer(), cleanup_dirty_set()))(), dirty_set.add(O);
  }
  function cleanup_dirty_set() {
    for (const O of dirty_set)
      remove_by(O, (_) => _.signal == null);
    dirty_set.clear();
  }
  function is_ref(O) {
    return isPlainObject(O) && "id" in O && !("type" in O);
  }
  const has_refs = Symbol("has_refs");
  function _is_HasRefs(O) {
    return has_refs in O;
  }
  function is_HasRefs(O) {
    return isObject$3(O) && _is_HasRefs(O);
  }
  function may_have_refs(O) {
    if (_is_HasRefs(O))
      return O[has_refs];
    const _ = O.constructor;
    return is_HasRefs(_) ? _[has_refs] : !0;
  }
  const _loggers = {};
  class LogLevel {
    constructor(_, H, V) {
      mt(this, "name");
      mt(this, "level");
      mt(this, "method");
      this.name = _, this.level = H, this.method = V;
    }
  }
  mt(LogLevel, "__name__", "LogLevel");
  const K1 = class K1 {
    constructor(_, H = K1.INFO) {
      mt(this, "_name");
      mt(this, "_log_level");
      this._name = _, this.set_level(H);
    }
    static get levels() {
      return Object.keys(K1.log_levels);
    }
    static get(_, H = K1.INFO) {
      if (_.length > 0)
        return _ in _loggers ? _loggers[_] : _loggers[_] = new K1(_, H);
      throw new TypeError("Logger.get() expects a non-empty string name and an optional log-level");
    }
    get level() {
      return this.get_level();
    }
    get_level() {
      return this._log_level;
    }
    set_level(_) {
      if (_ instanceof LogLevel)
        this._log_level = _;
      else if (K1.log_levels.hasOwnProperty(_))
        this._log_level = K1.log_levels[_];
      else
        throw new Error("Logger.set_level() expects a log-level object or a string name of a log-level");
      const H = `[${this._name}]`;
      for (const { level: V, method: q } of values$3(K1.log_levels))
        V < this._log_level.level || this._log_level.level === K1.OFF.level ? this[q] = function() {
        } : this[q] = _method_factory(q, H);
      this.log = _method_factory("log", H);
    }
    log(..._) {
    }
    trace(..._) {
    }
    debug(..._) {
    }
    info(..._) {
    }
    warn(..._) {
    }
    error(..._) {
    }
  };
  mt(K1, "__name__", "Logger"), mt(K1, "TRACE", new LogLevel("trace", 0, "trace")), mt(K1, "DEBUG", new LogLevel("debug", 1, "debug")), mt(K1, "INFO", new LogLevel("info", 2, "info")), mt(K1, "WARN", new LogLevel("warn", 6, "warn")), mt(K1, "ERROR", new LogLevel("error", 7, "error")), mt(K1, "FATAL", new LogLevel("fatal", 8, "error")), mt(K1, "OFF", new LogLevel("off", 9, "log")), mt(K1, "log_levels", {
    trace: K1.TRACE,
    debug: K1.DEBUG,
    info: K1.INFO,
    warn: K1.WARN,
    error: K1.ERROR,
    fatal: K1.FATAL,
    off: K1.OFF
  });
  let Logger = K1;
  function _method_factory(O, _) {
    const H = console[O];
    return (typeof H < "u" ? H : console.log).bind(console, _);
  }
  const logger$1 = Logger.get(`bokeh ${version$6}`), _named_colors = {
    aliceblue: 4042850303,
    antiquewhite: 4209760255,
    aqua: 16777215,
    aquamarine: 2147472639,
    azure: 4043309055,
    beige: 4126530815,
    bisque: 4293182719,
    black: 255,
    blanchedalmond: 4293643775,
    blue: 65535,
    blueviolet: 2318131967,
    brown: 2771004159,
    burlywood: 3736635391,
    cadetblue: 1604231423,
    chartreuse: 2147418367,
    chocolate: 3530104575,
    coral: 4286533887,
    cornflowerblue: 1687547391,
    cornsilk: 4294499583,
    crimson: 3692313855,
    cyan: 16777215,
    darkblue: 35839,
    darkcyan: 9145343,
    darkgoldenrod: 3095792639,
    darkgray: 2846468607,
    darkgreen: 6553855,
    darkgrey: 2846468607,
    darkkhaki: 3182914559,
    darkmagenta: 2332068863,
    darkolivegreen: 1433087999,
    darkorange: 4287365375,
    darkorchid: 2570243327,
    darkred: 2332033279,
    darksalmon: 3918953215,
    darkseagreen: 2411499519,
    darkslateblue: 1211993087,
    darkslategray: 793726975,
    darkslategrey: 793726975,
    darkturquoise: 13554175,
    darkviolet: 2483082239,
    deeppink: 4279538687,
    deepskyblue: 12582911,
    dimgray: 1768516095,
    dimgrey: 1768516095,
    dodgerblue: 512819199,
    firebrick: 2988581631,
    floralwhite: 4294635775,
    forestgreen: 579543807,
    fuchsia: 4278255615,
    gainsboro: 3705462015,
    ghostwhite: 4177068031,
    gold: 4292280575,
    goldenrod: 3668254975,
    gray: 2155905279,
    green: 8388863,
    greenyellow: 2919182335,
    grey: 2155905279,
    honeydew: 4043305215,
    hotpink: 4285117695,
    indianred: 3445382399,
    indigo: 1258324735,
    ivory: 4294963455,
    khaki: 4041641215,
    lavender: 3873897215,
    lavenderblush: 4293981695,
    lawngreen: 2096890111,
    lemonchiffon: 4294626815,
    lightblue: 2916673279,
    lightcoral: 4034953471,
    lightcyan: 3774873599,
    lightgoldenrodyellow: 4210742015,
    lightgray: 3553874943,
    lightgreen: 2431553791,
    lightgrey: 3553874943,
    lightpink: 4290167295,
    lightsalmon: 4288707327,
    lightseagreen: 548580095,
    lightskyblue: 2278488831,
    lightslategray: 2005441023,
    lightslategrey: 2005441023,
    lightsteelblue: 2965692159,
    lightyellow: 4294959359,
    lime: 16711935,
    limegreen: 852308735,
    linen: 4210091775,
    magenta: 4278255615,
    maroon: 2147483903,
    mediumaquamarine: 1724754687,
    mediumblue: 52735,
    mediumorchid: 3126187007,
    mediumpurple: 2473647103,
    mediumseagreen: 1018393087,
    mediumslateblue: 2070474495,
    mediumspringgreen: 16423679,
    mediumturquoise: 1221709055,
    mediumvioletred: 3340076543,
    midnightblue: 421097727,
    mintcream: 4127193855,
    mistyrose: 4293190143,
    moccasin: 4293178879,
    navajowhite: 4292783615,
    navy: 33023,
    oldlace: 4260751103,
    olive: 2155872511,
    olivedrab: 1804477439,
    orange: 4289003775,
    orangered: 4282712319,
    orchid: 3664828159,
    palegoldenrod: 4008225535,
    palegreen: 2566625535,
    paleturquoise: 2951671551,
    palevioletred: 3681588223,
    papayawhip: 4293907967,
    peachpuff: 4292524543,
    peru: 3448061951,
    pink: 4290825215,
    plum: 3718307327,
    powderblue: 2967529215,
    purple: 2147516671,
    rebeccapurple: 1714657791,
    red: 4278190335,
    rosybrown: 3163525119,
    royalblue: 1097458175,
    saddlebrown: 2336560127,
    salmon: 4202722047,
    sandybrown: 4104413439,
    seagreen: 780883967,
    seashell: 4294307583,
    sienna: 2689740287,
    silver: 3233857791,
    skyblue: 2278484991,
    slateblue: 1784335871,
    slategray: 1887473919,
    slategrey: 1887473919,
    snow: 4294638335,
    springgreen: 16744447,
    steelblue: 1182971135,
    tan: 3535047935,
    teal: 8421631,
    thistle: 3636451583,
    tomato: 4284696575,
    turquoise: 1088475391,
    violet: 4001558271,
    wheat: 4125012991,
    white: 4294967295,
    whitesmoke: 4126537215,
    yellow: 4294902015,
    yellowgreen: 2597139199
  }, named_colors = _named_colors;
  function is_named_color(O) {
    return O in named_colors;
  }
  const { round: round$7, sqrt: sqrt$7 } = Math;
  function byte(O) {
    return clamp$1(round$7(O), 0, 255);
  }
  function transparent() {
    return [0, 0, 0, 0];
  }
  function encode_rgba([O, _, H, V]) {
    return O << 24 | _ << 16 | H << 8 | V;
  }
  function decode_rgba(O) {
    const _ = O >> 24 & 255, H = O >> 16 & 255, V = O >> 8 & 255, q = O & 255;
    return [_, H, V, q];
  }
  function color2rgba(O, _ = 1) {
    const [H, V, q, G] = (() => {
      if (O == null)
        return transparent();
      if (isInteger$2(O))
        return decode_rgba(O);
      if (isString$2(O))
        return css4_parse(O) ?? transparent();
      if (O.length == 2) {
        const [J, Z] = O;
        return color2rgba(J, Z);
      } else {
        const [J, Z, te, ne = 1] = O;
        return [J, Z, te, byte(ne * 255)];
      }
    })();
    return [H, V, q, byte(_ * G)];
  }
  const _hex_table = {
    0: "0",
    1: "1",
    2: "2",
    3: "3",
    4: "4",
    5: "5",
    6: "6",
    7: "7",
    8: "8",
    9: "9",
    10: "a",
    11: "b",
    12: "c",
    13: "d",
    14: "e",
    15: "f"
  };
  function hex$2(O) {
    return _hex_table[O >> 4] + _hex_table[O & 15];
  }
  function rgba2css([O, _, H, V]) {
    return `rgba(${O}, ${_}, ${H}, ${V / 255})`;
  }
  function color2css(O, _) {
    const [H, V, q, G] = color2rgba(O, _);
    return rgba2css([H, V, q, G]);
  }
  function color2hex(O, _) {
    const [H, V, q, G] = color2rgba(O, _), J = `#${hex$2(H)}${hex$2(V)}${hex$2(q)}`;
    return G == 255 ? J : `${J}${hex$2(G)}`;
  }
  function color2hexrgb(O) {
    const [_, H, V] = color2rgba(O);
    return `#${hex$2(_)}${hex$2(H)}${hex$2(V)}`;
  }
  const rgb_modern = /^rgba?\(\s*(?<r>[^\s,]+?)\s+(?<g>[^\s,]+?)\s+(?<b>[^\s,]+?)(?:\s*\/\s*(?<a>[^\s,]+?))?\s*\)$/, rgb_legacy = /^rgba?\(\s*(?<r>[^\s,]+?)\s*,\s*(?<g>[^\s,]+?)\s*,\s*(?<b>[^\s,]+?)(?:\s*,\s*(?<a>[^\s,]+?))?\s*\)$/, css4_normalize = (() => {
    const O = document.createElement("canvas");
    O.width = 1, O.height = 1;
    const _ = O.getContext("2d"), H = _.createLinearGradient(0, 0, 1, 1);
    return (V) => {
      _.fillStyle = H, _.fillStyle = V;
      const q = _.fillStyle;
      return q != H ? q : null;
    };
  })();
  function css4_parse(O) {
    if (O = O.trim().toLowerCase(), O == "")
      return null;
    if (O == "transparent")
      return transparent();
    if (is_named_color(O))
      return decode_rgba(named_colors[O]);
    if (O[0] == "#") {
      const _ = +`0x${O.substring(1)}`;
      if (isNaN(_))
        return null;
      switch (O.length - 1) {
        case 3: {
          const H = _ >> 8 & 15, V = _ >> 4 & 15, q = _ >> 0 & 15, G = H << 4 | H, J = V << 4 | V, Z = q << 4 | q;
          return [G, J, Z, 255];
        }
        case 4: {
          const H = _ >> 12 & 15, V = _ >> 8 & 15, q = _ >> 4 & 15, G = _ >> 0 & 15, J = H << 4 | H, Z = V << 4 | V, te = q << 4 | q, ne = G << 4 | G;
          return [J, Z, te, ne];
        }
        case 6: {
          const H = _ >> 16 & 255, V = _ >> 8 & 255, q = _ >> 0 & 255;
          return [H, V, q, 255];
        }
        case 8: {
          const H = _ >> 24 & 255, V = _ >> 16 & 255, q = _ >> 8 & 255, G = _ >> 0 & 255;
          return [H, V, q, G];
        }
      }
    } else if (O.startsWith("rgb")) {
      const _ = O.match(rgb_modern) ?? O.match(rgb_legacy);
      if ((_ == null ? void 0 : _.groups) != null) {
        let { r: H, g: V, b: q, a: G = "1" } = _.groups;
        const J = H.endsWith("%"), Z = V.endsWith("%"), te = q.endsWith("%"), ne = G.endsWith("%");
        if (!(J && Z && te || !J && !Z && !te))
          return null;
        J && (H = H.slice(0, -1)), Z && (V = V.slice(0, -1)), te && (q = q.slice(0, -1)), ne && (G = G.slice(0, -1));
        let ee = Number(H), ae = Number(V), oe = Number(q), se = Number(G);
        return isNaN(ee + ae + oe + se) ? null : (J && (ee = 255 * (ee / 100)), Z && (ae = 255 * (ae / 100)), te && (oe = 255 * (oe / 100)), se = 255 * (ne ? se / 100 : se), ee = byte(ee), ae = byte(ae), oe = byte(oe), se = byte(se), [ee, ae, oe, se]);
      }
    } else {
      const _ = css4_normalize(O);
      if (_ != null)
        return css4_parse(_);
    }
    return null;
  }
  function is_Color(O) {
    return !!(isInteger$2(O) || isString$2(O) && css4_parse(O) != null || isArray$3(O) && (O.length == 3 || O.length == 4));
  }
  const ESMap = globalThis.Map, ESSet = globalThis.Set, DOMNode$1 = globalThis.Node;
  class Kind {
    constructor() {
      mt(this, "__type__");
    }
  }
  mt(Kind, "__name__", "Kind");
  var Kinds;
  (function(O) {
    class _ extends Kind {
      may_have_refs() {
        return !1;
      }
    }
    mt(_, "__name__", "Primitive"), O.Primitive = _;
    class H extends _ {
      valid(Me) {
        return Me !== void 0;
      }
      toString() {
        return "Any";
      }
      may_have_refs() {
        return !0;
      }
    }
    mt(H, "__name__", "Any"), O.Any = H;
    class V extends _ {
      valid(Me) {
        return Me !== void 0;
      }
      toString() {
        return "Unknown";
      }
      may_have_refs() {
        return !0;
      }
    }
    mt(V, "__name__", "Unknown"), O.Unknown = V;
    class q extends _ {
      valid(Me) {
        return isBoolean$4(Me);
      }
      toString() {
        return "Bool";
      }
    }
    mt(q, "__name__", "Bool"), O.Bool = q;
    class G extends Kind {
      constructor(Re) {
        super();
        mt(this, "obj_type");
        this.obj_type = Re;
      }
      valid(Re) {
        return Re instanceof this.obj_type;
      }
      toString() {
        const Re = this.obj_type;
        return `Ref(${Re.__name__ ?? Re.toString()})`;
      }
      may_have_refs() {
        const { obj_type: Re } = this;
        return has_refs in Re ? Re[has_refs] : !0;
      }
    }
    mt(G, "__name__", "Ref"), O.Ref = G;
    class J extends Kind {
      valid(Me) {
        return isObject$3(Me);
      }
      toString() {
        return "AnyRef";
      }
      may_have_refs() {
        return !0;
      }
    }
    mt(J, "__name__", "AnyRef"), O.AnyRef = J;
    class Z extends _ {
      valid(Me) {
        return isNumber$3(Me);
      }
      toString() {
        return "Float";
      }
    }
    mt(Z, "__name__", "Float"), O.Float = Z;
    class te extends Z {
      valid(Me) {
        return super.valid(Me) && isInteger$2(Me);
      }
      toString() {
        return "Int";
      }
    }
    mt(te, "__name__", "Int"), O.Int = te;
    class ne extends Z {
      valid(Me) {
        return super.valid(Me) && 0 <= Me && Me <= 1;
      }
      toString() {
        return "Percent";
      }
    }
    mt(ne, "__name__", "Percent"), O.Percent = ne;
    class ee extends Kind {
      constructor(Re) {
        super();
        mt(this, "types");
        this.types = Re, this.types = Re;
      }
      valid(Re) {
        return this.types.some((De) => De.valid(Re));
      }
      toString() {
        return `Or(${this.types.map((Re) => Re.toString()).join(", ")})`;
      }
      may_have_refs() {
        return this.types.some((Re) => Re.may_have_refs());
      }
    }
    mt(ee, "__name__", "Or"), O.Or = ee;
    class ae extends Kind {
      constructor(Re) {
        super();
        mt(this, "types");
        this.types = Re, this.types = Re;
      }
      valid(Re) {
        if (!isArray$3(Re))
          return !1;
        for (let De = 0; De < this.types.length; De++) {
          const Pe = this.types[De], $e = Re[De];
          if (!Pe.valid($e))
            return !1;
        }
        return !0;
      }
      toString() {
        return `Tuple(${this.types.map((Re) => Re.toString()).join(", ")})`;
      }
      may_have_refs() {
        return this.types.some((Re) => Re.may_have_refs());
      }
    }
    mt(ae, "__name__", "Tuple"), O.Tuple = ae;
    class oe extends Kind {
      constructor(Re) {
        super();
        mt(this, "struct_type");
        this.struct_type = Re;
      }
      valid(Re) {
        if (!isPlainObject(Re))
          return !1;
        const De = new PlainObjectProxy(this.struct_type);
        for (const Pe of keys$5(Re))
          if (!De.has(Pe))
            return !1;
        for (const [Pe, $e] of De) {
          const qe = Re[Pe];
          if (!$e.valid(qe))
            return !1;
        }
        return !0;
      }
      toString() {
        return `Struct({${typed_entries(this.struct_type).map(([De, Pe]) => `${De.toString()}: ${Pe}`).join(", ")}})`;
      }
      may_have_refs() {
        return typed_values(this.struct_type).some((Re) => Re.may_have_refs());
      }
    }
    mt(oe, "__name__", "Struct"), O.Struct = oe;
    class se extends Kind {
      constructor(Re) {
        super();
        mt(this, "struct_type");
        this.struct_type = Re;
      }
      valid(Re) {
        if (!isPlainObject(Re))
          return !1;
        const De = new PlainObjectProxy(Re), Pe = new PlainObjectProxy(this.struct_type);
        for (const $e of De.keys())
          if (!Pe.has($e))
            return !1;
        for (const [$e, qe] of Pe) {
          const Xe = De.get($e);
          if (Xe !== void 0 && !qe.valid(Xe))
            return !1;
        }
        return !0;
      }
      toString() {
        return `Struct({${typed_entries(this.struct_type).map(([De, Pe]) => `${De.toString()}?: ${Pe}`).join(", ")}})`;
      }
      may_have_refs() {
        return typed_values(this.struct_type).some((Re) => Re.may_have_refs());
      }
    }
    mt(se, "__name__", "PartialStruct"), O.PartialStruct = se;
    class ce extends Kind {
      constructor(Re) {
        super();
        mt(this, "item_type");
        this.item_type = Re;
      }
      valid(Re) {
        return isIterable$2(Re);
      }
      toString() {
        return `Iterable(${this.item_type.toString()})`;
      }
      may_have_refs() {
        return this.item_type.may_have_refs();
      }
    }
    mt(ce, "__name__", "Iterable"), O.Iterable = ce;
    class le extends Kind {
      constructor(Re) {
        super();
        mt(this, "item_type");
        this.item_type = Re;
      }
      valid(Re) {
        return isArray$3(Re) || isTypedArray$2(Re);
      }
      toString() {
        return `Arrayable(${this.item_type.toString()})`;
      }
      may_have_refs() {
        return this.item_type.may_have_refs();
      }
    }
    mt(le, "__name__", "Arrayable"), O.Arrayable = le;
    class ue extends Kind {
      constructor(Re) {
        super();
        mt(this, "item_type");
        this.item_type = Re;
      }
      valid(Re) {
        return isArray$3(Re) && Re.every((De) => this.item_type.valid(De));
      }
      toString() {
        return `List(${this.item_type.toString()})`;
      }
      may_have_refs() {
        return this.item_type.may_have_refs();
      }
    }
    mt(ue, "__name__", "List"), O.List = ue;
    class de extends _ {
      valid(Me) {
        return Me === null;
      }
      toString() {
        return "Null";
      }
    }
    mt(de, "__name__", "Null"), O.Null = de;
    class fe extends Kind {
      constructor(Re) {
        super();
        mt(this, "base_type");
        this.base_type = Re;
      }
      valid(Re) {
        return Re === null || this.base_type.valid(Re);
      }
      toString() {
        return `Nullable(${this.base_type.toString()})`;
      }
      may_have_refs() {
        return this.base_type.may_have_refs();
      }
    }
    mt(fe, "__name__", "Nullable"), O.Nullable = fe;
    class he extends Kind {
      constructor(Re) {
        super();
        mt(this, "base_type");
        this.base_type = Re;
      }
      valid(Re) {
        return Re === void 0 || this.base_type.valid(Re);
      }
      toString() {
        return `Opt(${this.base_type.toString()})`;
      }
      may_have_refs() {
        return this.base_type.may_have_refs();
      }
    }
    mt(he, "__name__", "Opt"), O.Opt = he;
    class me extends Kind {
      valid(Me) {
        return Me instanceof ArrayBuffer;
      }
      toString() {
        return "Bytes";
      }
      may_have_refs() {
        return !1;
      }
    }
    mt(me, "__name__", "Bytes"), O.Bytes = me;
    class pe extends _ {
      valid(Me) {
        return isString$2(Me);
      }
      toString() {
        return "Str";
      }
    }
    mt(pe, "__name__", "Str"), O.Str = pe;
    class ye extends pe {
      constructor(Re) {
        super();
        mt(this, "regex");
        this.regex = Re;
      }
      valid(Re) {
        return super.valid(Re) && this.regex.test(Re);
      }
      toString() {
        return `Regex(${this.regex.toString()})`;
      }
    }
    mt(ye, "__name__", "Regex"), O.Regex = ye;
    class ge extends _ {
      constructor(Re) {
        super();
        mt(this, "values");
        this.values = new ESSet(Re);
      }
      valid(Re) {
        return this.values.has(Re);
      }
      *[Symbol.iterator]() {
        yield* this.values;
      }
      toString() {
        return `Enum(${[...this.values].map((Re) => Re.toString()).join(", ")})`;
      }
    }
    mt(ge, "__name__", "Enum"), O.Enum = ge;
    class ve extends Kind {
      constructor(Re) {
        super();
        mt(this, "item_type");
        this.item_type = Re;
      }
      valid(Re) {
        if (!(Re instanceof ESMap || isPlainObject(Re)))
          return !1;
        for (const De of values$3(Re))
          if (!this.item_type.valid(De))
            return !1;
        return !0;
      }
      toString() {
        return `Dict(${this.item_type.toString()})`;
      }
      may_have_refs() {
        return this.item_type.may_have_refs();
      }
    }
    mt(ve, "__name__", "Dict"), O.Dict = ve;
    class be extends Kind {
      constructor(Re, De) {
        super();
        mt(this, "key_type");
        mt(this, "item_type");
        this.key_type = Re, this.item_type = De;
      }
      coerce(Re) {
        return isPlainObject(Re) && is_empty(Re) ? new ESMap() : Re;
      }
      valid(Re) {
        if (!(Re instanceof ESMap))
          return !1;
        for (const [De, Pe] of Re.entries())
          if (!(this.key_type.valid(De) && this.item_type.valid(Pe)))
            return !1;
        return !0;
      }
      toString() {
        return `Mapping(${this.key_type.toString()}, ${this.item_type.toString()})`;
      }
      may_have_refs() {
        return this.key_type.may_have_refs() || this.item_type.may_have_refs();
      }
    }
    mt(be, "__name__", "Mapping"), O.Mapping = be;
    class Ce extends Kind {
      constructor(Re) {
        super();
        mt(this, "item_type");
        this.item_type = Re;
      }
      valid(Re) {
        if (!(Re instanceof ESSet))
          return !1;
        for (const De of Re)
          if (!this.item_type.valid(De))
            return !1;
        return !0;
      }
      toString() {
        return `Set(${this.item_type.toString()})`;
      }
      may_have_refs() {
        return this.item_type.may_have_refs();
      }
    }
    mt(Ce, "__name__", "Set"), O.Set = Ce;
    class Se extends Kind {
      valid(Me) {
        return is_Color(Me);
      }
      toString() {
        return "Color";
      }
      may_have_refs() {
        return !1;
      }
    }
    mt(Se, "__name__", "Color"), O.Color = Se;
    class we extends pe {
      /*
      override valid(value: unknown): value is string {
        return super.valid(value) // TODO: && this._parse(value)
      }
      */
      toString() {
        return "CSSLength";
      }
    }
    mt(we, "__name__", "CSSLength"), O.CSSLength = we;
    class Ee extends Kind {
      valid(Me) {
        return isFunction$3(Me);
      }
      toString() {
        return "Func(...)";
      }
      may_have_refs() {
        return !1;
      }
    }
    mt(Ee, "__name__", "Func"), O.Func = Ee;
    class xe extends Kind {
      constructor(Re) {
        super();
        mt(this, "base_type");
        this.base_type = Re;
      }
      valid(Re) {
        return this.base_type.valid(Re) && Re >= 0;
      }
      toString() {
        return `NonNegative(${this.base_type.toString()})`;
      }
      may_have_refs() {
        return this.base_type.may_have_refs();
      }
    }
    mt(xe, "__name__", "NonNegative"), O.NonNegative = xe;
    class Le extends Kind {
      constructor(Re) {
        super();
        mt(this, "base_type");
        this.base_type = Re;
      }
      valid(Re) {
        return this.base_type.valid(Re) && Re > 0;
      }
      toString() {
        return `Positive(${this.base_type.toString()})`;
      }
      may_have_refs() {
        return this.base_type.may_have_refs();
      }
    }
    mt(Le, "__name__", "Positive"), O.Positive = Le;
    class Qe extends Kind {
      valid(Me) {
        return Me instanceof DOMNode$1;
      }
      toString() {
        return "Node";
      }
      may_have_refs() {
        return !1;
      }
    }
    mt(Qe, "__name__", "Node"), O.Node = Qe;
  })(Kinds || (Kinds = {}));
  const Any = new Kinds.Any(), Unknown = new Kinds.Unknown(), Bool$3 = new Kinds.Bool(), Float$1 = new Kinds.Float(), Int$1 = new Kinds.Int(), Bytes$1 = new Kinds.Bytes(), Str = new Kinds.Str(), Regex = (O) => new Kinds.Regex(O), Null$3 = new Kinds.Null(), Nullable = (O) => new Kinds.Nullable(O), Opt = (O) => new Kinds.Opt(O), Or$1 = (...O) => new Kinds.Or(O), Tuple = (...O) => new Kinds.Tuple(O), Struct$1 = (O) => new Kinds.Struct(O), PartialStruct = (O) => new Kinds.PartialStruct(O), Iterable = (O) => new Kinds.Iterable(O), Arrayable = (O) => new Kinds.Arrayable(O), List$2 = (O) => new Kinds.List(O), Dict = (O) => new Kinds.Dict(O), Mapping = (O, _) => new Kinds.Mapping(O, _), Set$3 = (O) => new Kinds.Set(O), Enum = (...O) => new Kinds.Enum(O), Ref = (O) => new Kinds.Ref(O), AnyRef = () => new Kinds.AnyRef(), Func = () => new Kinds.Func(), Node$5 = new Kinds.Node(), NonNegative = (O) => new Kinds.NonNegative(O), Positive = (O) => new Kinds.Positive(O), Percent = new Kinds.Percent(), Alpha = Percent, Color$1 = new Kinds.Color(), Auto = Enum("auto"), CSSLength = new Kinds.CSSLength(), FontSize = Str, Font$1 = Str, Angle = Float$1, Boolean$1 = Bool$3, String$2 = Str, Number$2 = Float$1, Array$1 = List$2, Map$3 = Mapping, Function$1 = Func, kinds = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    Alpha,
    Angle,
    Any,
    AnyRef,
    Array: Array$1,
    Arrayable,
    Auto,
    Bool: Bool$3,
    Boolean: Boolean$1,
    Bytes: Bytes$1,
    CSSLength,
    Color: Color$1,
    Dict,
    Enum,
    Float: Float$1,
    Font: Font$1,
    FontSize,
    Func,
    Function: Function$1,
    Int: Int$1,
    Iterable,
    Kind,
    get Kinds() {
      return Kinds;
    },
    List: List$2,
    Map: Map$3,
    Mapping,
    Node: Node$5,
    NonNegative,
    Null: Null$3,
    Nullable,
    Number: Number$2,
    Opt,
    Or: Or$1,
    PartialStruct,
    Percent,
    Positive,
    Ref,
    Regex,
    Set: Set$3,
    Str,
    String: String$2,
    Struct: Struct$1,
    Tuple,
    Unknown
  }, Symbol.toStringTag, { value: "Module" })), Align = Enum("start", "center", "end"), HAlign = Enum("left", "center", "right"), VAlign = Enum("top", "center", "bottom"), Anchor$2 = Enum("top_left", "top_center", "top_right", "center_left", "center_center", "center_right", "bottom_left", "bottom_center", "bottom_right", "top", "left", "center", "right", "bottom"), AngleUnits = Enum("deg", "rad", "grad", "turn"), AlternationPolicy = Enum("none", "even", "odd", "every"), BoxOrigin = Enum("corner", "center"), ButtonType = Enum("default", "primary", "success", "warning", "danger", "light"), CalendarPosition = Enum("auto", "above", "below"), Clock = Enum("12h", "24h"), CoordinateUnits = Enum("canvas", "screen", "data"), ContextWhich = Enum("start", "center", "end", "all"), Dimension$1 = Enum("width", "height"), Dimensions = Enum("width", "height", "both"), Direction$1 = Enum("clock", "anticlock"), Distribution = Enum("uniform", "normal"), Face = Enum("front", "back"), FlowMode = Enum("block", "inline"), FontStyle = Enum("normal", "italic", "bold", "bold italic"), HatchPatternType = Enum("blank", "dot", "ring", "horizontal_line", "vertical_line", "cross", "horizontal_dash", "vertical_dash", "spiral", "right_diagonal_line", "left_diagonal_line", "diagonal_cross", "right_diagonal_dash", "left_diagonal_dash", "horizontal_wave", "vertical_wave", "criss_cross", " ", ".", "o", "-", "|", "+", '"', ":", "@", "/", "\\", "x", ",", "`", "v", ">", "*"), HTTPMethod = Enum("POST", "GET"), HexTileOrientation = Enum("pointytop", "flattop"), HoverMode = Enum("mouse", "hline", "vline"), ImageOrigin = Enum("bottom_left", "top_left", "bottom_right", "top_right"), LatLon = Enum("lat", "lon"), LegendClickPolicy = Enum("none", "hide", "mute"), LegendLocation = Anchor$2, LineCap = Enum("butt", "round", "square"), LineDash = Enum("solid", "dashed", "dotted", "dotdash", "dashdot"), LineJoin = Enum("miter", "round", "bevel"), LinePolicy = Enum("prev", "next", "nearest", "interp", "none"), Location = Enum("above", "below", "left", "right"), Logo = Enum("normal", "grey"), MapType = Enum("satellite", "roadmap", "terrain", "hybrid"), MarkerType = Enum("asterisk", "circle", "circle_cross", "circle_dot", "circle_x", "circle_y", "cross", "dash", "diamond", "diamond_cross", "diamond_dot", "dot", "hex", "hex_dot", "inverted_triangle", "plus", "square", "square_cross", "square_dot", "square_pin", "square_x", "star", "star_dot", "triangle", "triangle_dot", "triangle_pin", "x", "y"), MutedPolicy = Enum("show", "ignore"), Orientation = Enum("vertical", "horizontal"), OutputBackend = Enum("canvas", "svg", "webgl"), PaddingUnits = Enum("percent", "absolute");
  Enum("above", "below", "left", "right", "center");
  const PointPolicy = Enum("snap_to_data", "follow_mouse", "none"), RadiusDimension = Enum("x", "y", "max", "min"), RenderLevel = Enum("image", "underlay", "glyph", "guide", "annotation", "overlay"), ResetPolicy = Enum("standard", "event_only"), ResolutionType = Enum("microseconds", "milliseconds", "seconds", "minsec", "minutes", "hourmin", "hours", "days", "months", "years"), RoundingFunction = Enum("round", "nearest", "floor", "rounddown", "ceil", "roundup"), ScrollbarPolicy = Enum("auto", "visible", "hidden"), RegionSelectionMode = Enum("replace", "append", "intersect", "subtract", "xor"), SelectionMode = Enum(...RegionSelectionMode, "toggle");
  Enum("above", "below", "left", "right");
  const SizingMode = Enum("stretch_width", "stretch_height", "stretch_both", "scale_width", "scale_height", "scale_both", "fixed", "inherit"), Sort = Enum("ascending", "descending"), SpatialUnits = Enum("screen", "data"), StartEnd = Enum("start", "end"), StepMode = Enum("after", "before", "center"), TapBehavior = Enum("select", "inspect"), TapGesture = Enum("tap", "doubletap"), TextAlign = Enum("left", "right", "center"), TextBaseline = Enum("top", "middle", "bottom", "alphabetic", "hanging", "ideographic"), TextureRepetition = Enum("repeat", "repeat_x", "repeat_y", "no_repeat"), LabelOrientation = Enum("vertical", "horizontal", "parallel", "normal"), TooltipAttachment = Enum("horizontal", "vertical", "left", "right", "above", "below"), UpdateMode = Enum("replace", "append"), VerticalAlign = Enum("top", "middle", "bottom"), ToolIcon = Enum("append_mode", "arrow_down_to_bar", "arrow_up_from_bar", "auto_box_zoom", "bold", "box_edit", "box_select", "box_zoom", "caret_down", "caret_left", "caret_right", "caret_up", "check", "chevron_down", "chevron_left", "chevron_right", "chevron_up", "clear_selection", "copy", "crosshair", "delete", "freehand_draw", "fullscreen", "help", "hover", "intersect_mode", "invert_selection", "italic", "lasso_select", "line_edit", "maximize", "minimize", "pan", "pin", "point_draw", "pointer", "poly_draw", "poly_edit", "polygon_select", "range", "redo", "replace_mode", "reset", "save", "see_off", "see_on", "settings", "square", "square_check", "subtract_mode", "tap_select", "text_align_center", "text_align_left", "text_align_right", "undo", "unknown", "unpin", "wheel_pan", "wheel_zoom", "x_box_select", "x_box_zoom", "x_grip", "x_pan", "xor_mode", "y_box_select", "y_box_zoom", "y_grip", "y_pan", "zoom_in", "zoom_out"), { hasOwnProperty: hasOwnProperty$b } = Object.prototype, equals = Symbol("equals");
  function is_Equatable(O) {
    return isObject$3(O) && equals in O;
  }
  const wildcard = Symbol("wildcard"), toString$2 = Object.prototype.toString;
  class EqNotImplemented extends Error {
  }
  mt(EqNotImplemented, "__name__", "EqNotImplemented");
  class Comparator {
    constructor(_) {
      mt(this, "a_stack", []);
      mt(this, "b_stack", []);
      mt(this, "structural");
      this.structural = (_ == null ? void 0 : _.structural) ?? !1;
    }
    eq(_, H) {
      if (Object.is(_, H) || _ === wildcard || H === wildcard)
        return !0;
      if (_ == null || H == null)
        return _ === H;
      const V = toString$2.call(_);
      if (V != toString$2.call(H))
        return !1;
      switch (V) {
        case "[object Number]":
          return this.numbers(_, H);
        case "[object Symbol]":
          return _ === H;
        case "[object RegExp]":
        case "[object String]":
          return `${_}` == `${H}`;
        case "[object Date]":
        case "[object Boolean]":
          return +_ == +H;
      }
      const { a_stack: q, b_stack: G } = this;
      let J = q.length;
      for (; J-- > 0; )
        if (q[J] === _)
          return G[J] === H;
      q.push(_), G.push(H);
      const Z = (() => {
        if (is_Equatable(_) && is_Equatable(H))
          return _[equals](H, this);
        switch (V) {
          case "[object Array]":
          case "[object Uint8Array]":
          case "[object Int8Array]":
          case "[object Uint16Array]":
          case "[object Int16Array]":
          case "[object Uint32Array]":
          case "[object Int32Array]":
          case "[object Float32Array]":
          case "[object Float64Array]":
            return this.arrays(_, H);
          case "[object Map]":
            return this.maps(_, H);
          case "[object Set]":
            return this.sets(_, H);
          case "[object Object]":
            if (_.constructor == H.constructor && (_.constructor == null || _.constructor === Object))
              return this.objects(_, H);
          case "[object Function]":
            if (_.constructor == H.constructor && _.constructor === Function)
              return this.eq(`${_}`, `${H}`);
        }
        if (typeof Node < "u" && _ instanceof Node)
          return this.nodes(_, H);
        throw new EqNotImplemented(`can't compare objects of type ${V}`);
      })();
      return q.pop(), G.pop(), Z;
    }
    numbers(_, H) {
      return Object.is(_, H);
    }
    arrays(_, H) {
      const { length: V } = _;
      if (V != H.length)
        return !1;
      for (let q = 0; q < V; q++)
        if (!this.eq(_[q], H[q]))
          return !1;
      return !0;
    }
    iterables(_, H) {
      const V = _[Symbol.iterator](), q = H[Symbol.iterator]();
      for (; ; ) {
        const G = V.next(), J = q.next(), Z = G.done ?? !1, te = J.done ?? !1;
        if (Z && te)
          return !0;
        if (Z || te || !this.eq(G.value, J.value))
          return !1;
      }
    }
    maps(_, H) {
      if (_.size != H.size)
        return !1;
      if (this.structural)
        return this.iterables(_.entries(), H.entries());
      for (const [V, q] of _)
        if (!H.has(V) || !this.eq(q, H.get(V)))
          return !1;
      return !0;
    }
    sets(_, H) {
      if (_.size != H.size)
        return !1;
      if (this.structural)
        return this.iterables(_.entries(), H.entries());
      for (const V of _)
        if (!H.has(V))
          return !1;
      return !0;
    }
    objects(_, H) {
      const V = Object.keys(_);
      if (V.length != Object.keys(H).length)
        return !1;
      for (const q of V)
        if (!hasOwnProperty$b.call(H, q) || !this.eq(_[q], H[q]))
          return !1;
      return !0;
    }
    nodes(_, H) {
      return !(_.nodeType != H.nodeType || _.textContent != H.textContent || !this.iterables(_.childNodes, H.childNodes));
    }
  }
  mt(Comparator, "__name__", "Comparator");
  function is_equal(O, _) {
    return new Comparator().eq(O, _);
  }
  var Ry, Oy;
  const Wd = class Wd {
    constructor(_, H = 0) {
      mt(this, "size");
      mt(this, Oy, "BitSet");
      mt(this, "_array");
      mt(this, "_nwords");
      mt(this, "_count", null);
      this.size = _, this._nwords = Math.ceil(_ / Wd._word_length), H == 0 || H == 1 ? (this._array = new Uint32Array(this._nwords), H == 1 && this._array.fill(4294967295)) : (assert$2(H.length == this._nwords, "Initializer size mismatch"), this._array = H);
    }
    clone() {
      return new Wd(this.size, new Uint32Array(this._array));
    }
    [(Oy = Symbol.toStringTag, Ry = has_refs, equals)](_, H) {
      if (!H.eq(this.size, _.size))
        return !1;
      const { _nwords: V } = this, q = this.size % Wd._word_length, G = q == 0 ? V : V - 1;
      for (let J = 0; J < G; J++)
        if (this._array[J] != _._array[J])
          return !1;
      if (q == 0)
        return !0;
      {
        const J = 1 << q - 1, Z = J - 1 ^ J;
        return (this._array[G] & Z) == (_._array[G] & Z);
      }
    }
    static all_set(_) {
      return new Wd(_, 1);
    }
    static all_unset(_) {
      return new Wd(_, 0);
    }
    static from_indices(_, H) {
      const V = new Wd(_);
      for (const q of H)
        V.set(q);
      return V;
    }
    static from_booleans(_, H) {
      const V = new Wd(_);
      let q = 0;
      for (const G of H) {
        if (q == _)
          break;
        G && V.set(q), q += 1;
      }
      return V;
    }
    _check_bounds(_) {
      assert$2(0 <= _ && _ < this.size, `Out of bounds: 0 <= ${_} < ${this.size}`);
    }
    get(_) {
      this._check_bounds(_);
      const H = _ >>> 5, V = _ & 31;
      return (this._array[H] >> V & 1) == 1;
    }
    set(_, H = !0) {
      this._check_bounds(_), this._count = null;
      const V = _ >>> 5, q = _ & 31;
      H ? this._array[V] |= 1 << q : this._array[V] &= ~(1 << q);
    }
    unset(_) {
      this.set(_, !1);
    }
    *[Symbol.iterator]() {
      yield* this.ones();
    }
    get count() {
      let _ = this._count;
      return _ == null && (this._count = _ = this._get_count()), _;
    }
    _get_count() {
      const { _array: _, _nwords: H, size: V } = this;
      let q = 0;
      for (let G = 0, J = 0; J < H; J++) {
        const Z = _[J];
        if (Z == 0)
          G += Wd._word_length;
        else
          for (let te = 0; te < Wd._word_length && G < V; te++, G++)
            (Z >>> te & 1) == 1 && (q += 1);
      }
      return q;
    }
    *ones() {
      const { _array: _, _nwords: H, size: V } = this;
      for (let q = 0, G = 0; G < H; G++) {
        const J = _[G];
        if (J == 0) {
          q += Wd._word_length;
          continue;
        }
        for (let Z = 0; Z < Wd._word_length && q < V; Z++, q++)
          (J >>> Z & 1) == 1 && (yield q);
      }
    }
    *zeros() {
      const { _array: _, _nwords: H, size: V } = this;
      for (let q = 0, G = 0; G < H; G++) {
        const J = _[G];
        if (J == 4294967295) {
          q += Wd._word_length;
          continue;
        }
        for (let Z = 0; Z < Wd._word_length && q < V; Z++, q++)
          J >>> Z & 1 || (yield q);
      }
    }
    _check_size(_) {
      assert$2(this.size == _.size, `Size mismatch (${this.size} != ${_.size})`);
    }
    invert() {
      for (let _ = 0; _ < this._nwords; _++)
        this._array[_] = ~this._array[_] >>> 0;
    }
    add(_) {
      this._check_size(_);
      for (let H = 0; H < this._nwords; H++)
        this._array[H] |= _._array[H];
    }
    intersect(_) {
      this._check_size(_);
      for (let H = 0; H < this._nwords; H++)
        this._array[H] &= _._array[H];
    }
    subtract(_) {
      this._check_size(_);
      for (let H = 0; H < this._nwords; H++) {
        const V = this._array[H], q = _._array[H];
        this._array[H] = (V ^ q) & V;
      }
    }
    symmetric_subtract(_) {
      this._check_size(_);
      for (let H = 0; H < this._nwords; H++)
        this._array[H] ^= _._array[H];
    }
    inversion() {
      const _ = this.clone();
      return _.invert(), _;
    }
    union(_) {
      const H = this.clone();
      return H.add(_), H;
    }
    intersection(_) {
      const H = this.clone();
      return H.intersect(_), H;
    }
    difference(_) {
      const H = this.clone();
      return H.subtract(_), H;
    }
    symmetric_difference(_) {
      const H = this.clone();
      return H.symmetric_subtract(_), H;
    }
    select(_) {
      assert$2(this.size <= _.length, "Size mismatch");
      const H = this.count, V = new _.constructor(H);
      let q = 0;
      for (const G of this)
        V[q++] = _[G];
      return V;
    }
  };
  mt(Wd, "__name__", "BitSet"), mt(Wd, Ry, !1), mt(Wd, "_word_length", 32);
  let BitSet = Wd;
  const GeneratorFunction = Object.getPrototypeOf(function* () {
  }).constructor;
  Object.getPrototypeOf(async function* () {
  }).constructor;
  const ColorArray = Uint32Array, RGBAArray = Uint8ClampedArray;
  function infer_type(O, _) {
    return O instanceof Float64Array || O instanceof Array || _ instanceof Float64Array || _ instanceof Array ? Float64Array : Float32Array;
  }
  const ScreenArray = Float32Array;
  function to_screen(O) {
    return O instanceof Float32Array ? O : Float32Array.from(O);
  }
  const is_little_endian = (() => {
    const O = new ArrayBuffer(4), _ = new Uint8Array(O), H = new Uint32Array(O);
    H[1] = 168496141;
    let V = !0;
    return _[4] == 10 && _[5] == 11 && _[6] == 12 && _[7] == 13 && (V = !1), V;
  })(), BYTE_ORDER = is_little_endian ? "little" : "big";
  function to_big_endian(O) {
    if (is_little_endian) {
      const _ = new Uint32Array(O.length), H = new DataView(_.buffer);
      let V = 0;
      for (const q of O)
        H.setUint32(V, q), V += 4;
      return _;
    } else
      return O;
  }
  function isValue(O) {
    return isPlainObject(O) && "value" in O;
  }
  function isField$1(O) {
    return isPlainObject(O) && "field" in O;
  }
  function isExpr$2(O) {
    return isPlainObject(O) && "expr" in O;
  }
  var u_;
  let Settings$1 = (u_ = class {
    constructor() {
      mt(this, "_dev", !1);
      mt(this, "_wireframe", !1);
      mt(this, "_force_webgl", !1);
      mt(this, "_force_fields", !1);
    }
    set dev(_) {
      this._dev = _;
    }
    get dev() {
      return this._dev;
    }
    set wireframe(_) {
      this._wireframe = _;
    }
    get wireframe() {
      return this._wireframe;
    }
    set force_webgl(_) {
      this._force_webgl = _;
    }
    get force_webgl() {
      return this._force_webgl;
    }
    set force_fields(_) {
      this._force_fields = _;
    }
    get force_fields() {
      return this._force_fields;
    }
  }, mt(u_, "__name__", "Settings"), u_);
  const settings = new Settings$1(), clone = Symbol("clone");
  function is_Cloneable(O) {
    return isObject$3(O) && clone in O;
  }
  class CloningError extends Error {
  }
  mt(CloningError, "__name__", "CloningError");
  class Cloner {
    constructor() {
    }
    clone(_) {
      if (is_Cloneable(_))
        return _[clone](this);
      if (isPrimitive(_))
        return _;
      if (isArray$3(_)) {
        const H = _.length, V = new Array(H);
        for (let q = 0; q < H; q++) {
          const G = _[q];
          V[q] = this.clone(G);
        }
        return V;
      } else if (isPlainObject(_)) {
        const H = {};
        for (const [V, q] of entries$2(_))
          H[V] = this.clone(q);
        return H;
      } else {
        if (_ instanceof Map)
          return new Map([..._].map(([H, V]) => [this.clone(H), this.clone(V)]));
        if (_ instanceof Set)
          return new Set([..._].map((H) => this.clone(H)));
        throw new CloningError(`${Object.prototype.toString.call(_)} is not cloneable`);
      }
    }
  }
  mt(Cloner, "__name__", "Cloner");
  function* range$8(O, _, H = 1) {
    assert$2(H > 0);
    const { abs: V, ceil: q, max: G } = Math;
    _ == null && (_ = O, O = 0);
    const J = O <= _ ? H : -H, Z = G(q(V(_ - O) / H), 0);
    for (let te = 0; te < Z; te++, O += J)
      yield O;
  }
  function* reverse$4(O) {
    const _ = O.length;
    for (let H = 0; H < _; H++)
      yield O[_ - H - 1];
  }
  function* enumerate(O) {
    let _ = 0;
    for (const H of O)
      yield [H, _++];
  }
  function* take(O, _) {
    assert$2(_ >= 0);
    let H = 0;
    for (const V of O)
      if (H++ < _)
        yield V;
      else
        break;
  }
  function* skip(O, _) {
    assert$2(_ >= 0);
    for (const H of O)
      _ == 0 ? yield H : _ -= 1;
  }
  function* tail$1(O) {
    yield* skip(O, 1);
  }
  function* join$2(O, _) {
    let H = !0;
    for (const V of O)
      H ? H = !1 : _ != null && (yield _()), yield* V;
  }
  function* zip(O, _) {
    const H = O[Symbol.iterator](), V = _[Symbol.iterator]();
    do {
      const q = H.next(), G = V.next();
      if (q.done === !0 || G.done === !0)
        break;
      yield [q.value, G.value];
    } while (!0);
  }
  function* interleave$1(O, _) {
    let H = !0;
    for (const V of O)
      H ? H = !1 : yield _(), yield V;
  }
  function* map$3(O, _) {
    let H = 0;
    for (const V of O)
      yield _(V, H++);
  }
  function* flat_map(O, _) {
    let H = 0;
    for (const V of O)
      yield* _(V, H++);
  }
  function every$1(O, _) {
    for (const H of O)
      if (!_(H))
        return !1;
    return !0;
  }
  function some$1(O, _) {
    for (const H of O)
      if (_(H))
        return !0;
    return !1;
  }
  function* combinations(O, _) {
    const H = O.length;
    if (_ > H)
      return;
    const V = range$9(_);
    for (yield V.map((q) => O[q]); ; ) {
      let q;
      for (const G of reverse$4(range$9(_)))
        if (V[G] != G + H - _) {
          q = G;
          break;
        }
      if (q == null)
        return;
      V[q] += 1;
      for (const G of range$9(q + 1, _))
        V[G] = V[G - 1] + 1;
      yield V.map((G) => O[G]);
    }
  }
  function* subsets(O) {
    for (const _ of range$9(O.length + 1))
      yield* combinations(O, _);
  }
  const iter = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    combinations,
    enumerate,
    every: every$1,
    flat_map,
    interleave: interleave$1,
    join: join$2,
    map: map$3,
    max: max$j,
    min: min$b,
    range: range$8,
    reverse: reverse$4,
    skip,
    some: some$1,
    subsets,
    tail: tail$1,
    take,
    zip
  }, Symbol.toStringTag, { value: "Module" }));
  function buffer_to_base64(O) {
    const _ = new Uint8Array(O), H = Array.from(_).map((V) => String.fromCharCode(V));
    return btoa(H.join(""));
  }
  function base64_to_buffer(O) {
    const _ = atob(O), H = _.length, V = new Uint8Array(H);
    for (let q = 0, G = H; q < G; q++)
      V[q] = _.charCodeAt(q);
    return V.buffer;
  }
  function swap16(O) {
    const _ = new Uint8Array(O);
    for (let H = 0, V = _.length; H < V; H += 2) {
      const q = _[H];
      _[H] = _[H + 1], _[H + 1] = q;
    }
  }
  function swap32(O) {
    const _ = new Uint8Array(O);
    for (let H = 0, V = _.length; H < V; H += 4) {
      let q = _[H];
      _[H] = _[H + 3], _[H + 3] = q, q = _[H + 1], _[H + 1] = _[H + 2], _[H + 2] = q;
    }
  }
  function swap64(O) {
    const _ = new Uint8Array(O);
    for (let H = 0, V = _.length; H < V; H += 8) {
      let q = _[H];
      _[H] = _[H + 7], _[H + 7] = q, q = _[H + 1], _[H + 1] = _[H + 6], _[H + 6] = q, q = _[H + 2], _[H + 2] = _[H + 5], _[H + 5] = q, q = _[H + 3], _[H + 3] = _[H + 4], _[H + 4] = q;
    }
  }
  function swap$3(O, _) {
    switch (_) {
      case "uint16":
      case "int16":
        swap16(O);
        break;
      case "uint32":
      case "int32":
      case "float32":
        swap32(O);
        break;
      case "float64":
        swap64(O);
        break;
    }
  }
  var d_;
  let Buffer$3 = (d_ = class {
    constructor(_) {
      mt(this, "buffer");
      this.buffer = _;
    }
    to_base64() {
      return buffer_to_base64(this.buffer);
    }
    [equals](_, H) {
      return H.eq(this.buffer, _.buffer);
    }
  }, mt(d_, "__name__", "Buffer"), d_);
  class Base64Buffer extends Buffer$3 {
    toJSON() {
      return this.to_base64();
    }
  }
  mt(Base64Buffer, "__name__", "Base64Buffer");
  const serialize = Symbol("serialize");
  function is_Serializable(O) {
    return isObject$3(O) && serialize in O;
  }
  class SerializationError extends Error {
  }
  mt(SerializationError, "__name__", "SerializationError");
  class Serialized {
    constructor(_) {
      mt(this, "value");
      this.value = _;
    }
    to_json() {
      return JSON.stringify(this.value);
    }
  }
  mt(Serialized, "__name__", "Serialized");
  class Serializer {
    constructor(_) {
      mt(this, "_references");
      mt(this, "binary");
      mt(this, "include_defaults");
      mt(this, "_circular", /* @__PURE__ */ new WeakSet());
      this.binary = (_ == null ? void 0 : _.binary) ?? !1, this.include_defaults = (_ == null ? void 0 : _.include_defaults) ?? !1;
      const H = _ == null ? void 0 : _.references;
      this._references = H != null ? new Map(H) : /* @__PURE__ */ new Map();
    }
    get_ref(_) {
      return this._references.get(_);
    }
    add_ref(_, H) {
      assert$2(!this._references.has(_)), this._references.set(_, H);
    }
    to_serializable(_) {
      return new Serialized(this.encode(_));
    }
    encode(_) {
      const H = this.get_ref(_);
      if (H != null)
        return H;
      if (isObject$3(_)) {
        this._circular.has(_) && this.error("circular reference"), this._circular.add(_);
        try {
          return this._encode(_);
        } finally {
          this._circular.delete(_);
        }
      } else
        return this._encode(_);
    }
    _encode(_) {
      if (is_Serializable(_))
        return _[serialize](this);
      if (isArray$3(_)) {
        const H = _.length, V = new Array(H);
        for (let q = 0; q < H; q++) {
          const G = _[q];
          V[q] = this.encode(G);
        }
        return V;
      } else {
        if (isTypedArray$2(_))
          return this._encode_typed_array(_);
        if (_ instanceof ArrayBuffer)
          return { type: "bytes", data: this.binary ? new Buffer$3(_) : new Base64Buffer(_) };
        if (isPlainObject(_)) {
          const H = entries$2(_);
          return H.length == 0 ? { type: "map" } : { type: "map", entries: [...map$3(H, ([V, q]) => [this.encode(V), this.encode(q)])] };
        } else {
          if (_ === null || isBoolean$4(_) || isString$2(_))
            return _;
          if (isNumber$3(_))
            return isNaN(_) ? { type: "number", value: "nan" } : isFinite(_) ? _ : { type: "number", value: `${_ < 0 ? "-" : "+"}inf` };
          if (_ instanceof Date)
            return { type: "date", iso: _.toISOString() };
          if (_ instanceof Set)
            return _.size == 0 ? { type: "set" } : { type: "set", entries: [...map$3(_.values(), (H) => this.encode(H))] };
          if (_ instanceof Map)
            return _.size == 0 ? { type: "map" } : { type: "map", entries: [...map$3(_.entries(), ([H, V]) => [this.encode(H), this.encode(V)])] };
          if (isSymbol$1(_) && _.description != null)
            return { type: "symbol", name: _.description };
          throw new SerializationError(`${Object.prototype.toString.call(_)} is not serializable`);
        }
      }
    }
    encode_struct(_) {
      const H = {};
      for (const [V, q] of entries$2(_))
        q !== void 0 && (H[V] = this.encode(q));
      return H;
    }
    error(_) {
      throw new SerializationError(_);
    }
    _encode_typed_array(_) {
      const H = this.encode(_.buffer), V = (() => {
        switch (_.constructor) {
          case Uint8Array:
            return "uint8";
          case Int8Array:
            return "int8";
          case Uint16Array:
            return "uint16";
          case Int16Array:
            return "int16";
          case Uint32Array:
            return "uint32";
          case Int32Array:
            return "int32";
          // case BigUint64Array: return "uint64"
          // case BigInt64Array: return "int64"
          case Float32Array:
            return "float32";
          case Float64Array:
            return "float64";
          default:
            this.error(`can't serialize typed array of type '${_[Symbol.toStringTag]}'`);
        }
      })();
      return {
        type: "typed_array",
        array: H,
        order: BYTE_ORDER,
        dtype: V
      };
    }
  }
  mt(Serializer, "__name__", "Serializer");
  const __ndarray__ = Symbol("__ndarray__");
  function encode_NDArray(O, _) {
    return {
      type: "ndarray",
      array: _.encode(O.dtype == "object" ? Array.from(O) : O.buffer),
      order: BYTE_ORDER,
      dtype: O.dtype,
      shape: O.shape
    };
  }
  var Iy;
  const Z4 = class Z4 extends Uint8Array {
    constructor(H, V) {
      super(H);
      mt(this, Iy, !0);
      mt(this, "dtype", "bool");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(Iy = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new Z4(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H] == 1;
    }
  };
  mt(Z4, "__name__", "BoolNDArray");
  let BoolNDArray = Z4;
  var Dy;
  const K4 = class K4 extends Uint8Array {
    constructor(H, V) {
      super(H);
      mt(this, Dy, !0);
      mt(this, "dtype", "uint8");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(Dy = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new K4(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(K4, "__name__", "Uint8NDArray");
  let Uint8NDArray = K4;
  var Ny;
  const J4 = class J4 extends Int8Array {
    constructor(H, V) {
      super(H);
      mt(this, Ny, !0);
      mt(this, "dtype", "int8");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(Ny = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new J4(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(J4, "__name__", "Int8NDArray");
  let Int8NDArray = J4;
  var Py;
  const eg = class eg extends Uint16Array {
    constructor(H, V) {
      super(H);
      mt(this, Py, !0);
      mt(this, "dtype", "uint16");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(Py = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new eg(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(eg, "__name__", "Uint16NDArray");
  let Uint16NDArray = eg;
  var $y;
  const tg = class tg extends Int16Array {
    constructor(H, V) {
      super(H);
      mt(this, $y, !0);
      mt(this, "dtype", "int16");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [($y = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new tg(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(tg, "__name__", "Int16NDArray");
  let Int16NDArray = tg;
  var Fy;
  const ng = class ng extends Uint32Array {
    constructor(H, V) {
      super(H);
      mt(this, Fy, !0);
      mt(this, "dtype", "uint32");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(Fy = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new ng(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(ng, "__name__", "Uint32NDArray");
  let Uint32NDArray = ng;
  var By;
  const rg = class rg extends Int32Array {
    constructor(H, V) {
      super(H);
      mt(this, By, !0);
      mt(this, "dtype", "int32");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(By = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new rg(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(rg, "__name__", "Int32NDArray");
  let Int32NDArray = rg;
  var Hy;
  const ig = class ig extends Float32Array {
    constructor(H, V) {
      super(H);
      mt(this, Hy, !0);
      mt(this, "dtype", "float32");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(Hy = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new ig(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(ig, "__name__", "Float32NDArray");
  let Float32NDArray = ig;
  var zy;
  const ag = class ag extends Float64Array {
    constructor(H, V) {
      super(H);
      mt(this, zy, !0);
      mt(this, "dtype", "float64");
      mt(this, "shape");
      mt(this, "dimension");
      this.shape = V ?? (is_NDArray(H) ? H.shape : [this.length]), this.dimension = this.shape.length;
    }
    [(zy = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new ag(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(ag, "__name__", "Float64NDArray");
  let Float64NDArray = ag;
  var Vy;
  const og = class og extends Array {
    constructor(H, V) {
      const q = H instanceof ArrayBuffer ? new Float64Array(H) : H, G = isNumber$3(q) ? q : q.length;
      super(G);
      mt(this, Vy, !0);
      mt(this, "dtype", "object");
      mt(this, "_shape");
      if (!isNumber$3(q))
        for (let J = 0; J < q.length; J++)
          this[J] = q[J];
      this._shape = V ?? (is_NDArray(q) ? q.shape : void 0);
    }
    get shape() {
      return this._shape ?? [this.length];
    }
    get dimension() {
      return this.shape.length;
    }
    [(Vy = __ndarray__, equals)](H, V) {
      return V.eq(this.shape, H.shape) && V.arrays(this, H);
    }
    [clone](H) {
      return new og(this, H.clone(this.shape));
    }
    [serialize](H) {
      return encode_NDArray(this, H);
    }
    get(H) {
      return this[H];
    }
  };
  mt(og, "__name__", "ObjectNDArray");
  let ObjectNDArray = og;
  function is_NDArray(O) {
    return isObject$3(O) && __ndarray__ in O;
  }
  function ndarray(O, { dtype: _, shape: H } = {}) {
    switch (_ == null && (_ = (() => {
      switch (!0) {
        case O instanceof Uint8Array:
          return "uint8";
        case O instanceof Int8Array:
          return "int8";
        case O instanceof Uint16Array:
          return "uint16";
        case O instanceof Int16Array:
          return "int16";
        case O instanceof Uint32Array:
          return "uint32";
        case O instanceof Int32Array:
          return "int32";
        case O instanceof Float32Array:
          return "float32";
        case O instanceof ArrayBuffer:
        case O instanceof Float64Array:
          return "float64";
        default:
          return "object";
      }
    })()), _) {
      case "bool":
        return new BoolNDArray(O, H);
      case "uint8":
        return new Uint8NDArray(O, H);
      case "int8":
        return new Int8NDArray(O, H);
      case "uint16":
        return new Uint16NDArray(O, H);
      case "int16":
        return new Int16NDArray(O, H);
      case "uint32":
        return new Uint32NDArray(O, H);
      case "int32":
        return new Int32NDArray(O, H);
      case "float32":
        return new Float32NDArray(O, H);
      case "float64":
        return new Float64NDArray(O, H);
      case "object":
        return new ObjectNDArray(O, H);
    }
  }
  class Diagnostics {
    constructor() {
      mt(this, "listeners", /* @__PURE__ */ new Set());
    }
    connect(_) {
      this.listeners.add(_);
    }
    disconnect(_) {
      this.listeners.delete(_);
    }
    report(_) {
      for (const H of this.listeners)
        H(_);
    }
  }
  mt(Diagnostics, "__name__", "Diagnostics");
  const diagnostics = new Diagnostics();
  class Uniform {
    is_Scalar() {
      return this.is_scalar;
    }
    is_Vector() {
      return !this.is_scalar;
    }
  }
  mt(Uniform, "__name__", "Uniform");
  const C3 = class C3 extends Uniform {
    constructor(H, V) {
      super();
      mt(this, "value");
      mt(this, "length");
      mt(this, "is_scalar", !0);
      this.value = H, this.length = V;
    }
    get(H) {
      return this.value;
    }
    *[Symbol.iterator]() {
      const { length: H, value: V } = this;
      for (let q = 0; q < H; q++)
        yield V;
    }
    select(H) {
      return new C3(this.value, H.count);
    }
    [equals](H, V) {
      return V.eq(this.length, H.length) && V.eq(this.value, H.value);
    }
    map(H) {
      return new C3(H(this.value), this.length);
    }
  };
  mt(C3, "__name__", "UniformScalar");
  let UniformScalar = C3;
  const sg = class sg extends Uniform {
    constructor(H) {
      super();
      mt(this, "array");
      mt(this, "is_scalar", !1);
      mt(this, "length");
      this.array = H, this.length = this.array.length;
    }
    get(H) {
      return this.array[H];
    }
    *[Symbol.iterator]() {
      yield* this.array;
    }
    select(H) {
      const V = H.select(this.array);
      return new this.constructor(V);
    }
    [equals](H, V) {
      return V.eq(this.length, H.length) && V.eq(this.array, H.array);
    }
    map(H) {
      return new sg(map$4(this.array, H));
    }
  };
  mt(sg, "__name__", "UniformVector");
  let UniformVector = sg;
  class ColorUniformVector extends UniformVector {
    constructor(H) {
      super(H);
      mt(this, "array");
      mt(this, "_view");
      this.array = H, this._view = new DataView(H.buffer);
    }
    get(H) {
      return this._view.getUint32(4 * H);
    }
    *[Symbol.iterator]() {
      const H = this.length;
      for (let V = 0; V < H; V++)
        yield this.get(V);
    }
  }
  mt(ColorUniformVector, "__name__", "ColorUniformVector");
  function max$i(O) {
    return O.is_Scalar() ? O.value : max$j(O.array);
  }
  function valueToString$1(O) {
    try {
      return JSON.stringify(O);
    } catch {
      return O.toString();
    }
  }
  function isSpec(O) {
    return isPlainObject(O) && (O.value === void 0 ? 0 : 1) + (O.field === void 0 ? 0 : 1) + (O.expr === void 0 ? 0 : 1) == 1;
  }
  const unset = Symbol("unset");
  class UnsetValueError extends Error {
  }
  mt(UnsetValueError, "__name__", "UnsetValueError");
  var f_;
  let Property$1 = (f_ = class {
    constructor(_, H, V, q, G = {}) {
      mt(this, "obj");
      mt(this, "attr");
      mt(this, "kind");
      mt(this, "default_value");
      mt(this, "__value__");
      mt(this, "_value", unset);
      mt(this, "_initialized", !1);
      mt(this, "_dirty", !1);
      mt(this, "may_have_refs");
      mt(this, "change");
      /*readonly*/
      mt(this, "internal");
      mt(this, "readonly");
      this.obj = _, this.attr = H, this.kind = V, this.default_value = q, this.change = new Signal0(this.obj, "change"), this.internal = G.internal ?? !1, this.readonly = G.readonly ?? !1, this.convert = G.convert, this.on_update = G.on_update, this.may_have_refs = V.may_have_refs();
    }
    get syncable() {
      return !this.internal;
    }
    get is_unset() {
      return this._value === unset;
    }
    get initialized() {
      return this._initialized;
    }
    initialize(_ = unset) {
      if (this._initialized)
        throw new Error("already initialized");
      let H = unset;
      if (_ !== unset)
        H = _, this._dirty = !0;
      else {
        const V = this._default_override();
        V !== unset ? H = V : H = this.default_value(this.obj);
      }
      H !== unset ? (this.kind.coerce != null && (H = this.kind.coerce(H)), this._update(H)) : this._value = unset, this._initialized = !0;
    }
    get_value() {
      if (this._value !== unset)
        return this._value;
      throw new UnsetValueError(`${this.obj}.${this.attr} is unset`);
    }
    set_value(_) {
      this._initialized ? (this._update(_), this._dirty = !0) : this.initialize(_), diagnostics.report(this);
    }
    // abstract _intrinsic_default(): T
    _default_override() {
      return unset;
    }
    get dirty() {
      return this._dirty;
    }
    //protected abstract _update(attr_value: T): void
    _update(_) {
      var H;
      if (this.validate(_), this.convert != null) {
        const V = this.convert(_, this.obj);
        V !== void 0 && (_ = V);
      }
      this._value = _, (H = this.on_update) == null || H.call(this, _, this.obj);
    }
    toString() {
      return `Prop(${this.obj}.${this.attr}, value: ${valueToString$1(this._value)})`;
    }
    // ----- customizable policies
    normalize(_) {
      return _;
    }
    validate(_) {
      if (!this.valid(_))
        throw new Error(`${this.obj}.${this.attr} given invalid value: ${valueToString$1(_)}`);
    }
    valid(_) {
      return this.kind.valid(_);
    }
  }, mt(f_, "__name__", "Property"), f_);
  class PropertyAlias {
    constructor(_) {
      mt(this, "attr");
      this.attr = _;
    }
  }
  mt(PropertyAlias, "__name__", "PropertyAlias");
  class PrimitiveProperty extends Property$1 {
  }
  mt(PrimitiveProperty, "__name__", "PrimitiveProperty");
  class Font extends PrimitiveProperty {
    _default_override() {
      return settings.dev ? "Bokeh" : unset;
    }
  }
  mt(Font, "__name__", "Font");
  class ScalarSpec extends Property$1 {
    constructor() {
      super(...arguments);
      mt(this, "__scalar__");
      mt(this, "_value", unset);
    }
    get_value() {
      if (this._value !== unset)
        return this._value;
      throw new Error(`${this.obj}.${this.attr} is unset`);
    }
    _update(H) {
      if (isSpec(H) ? this._value = H : this._value = { value: H }, isPlainObject(this._value)) {
        const { _value: V } = this;
        this._value[serialize] = (q) => {
          const { value: G, field: J, expr: Z, transform: te, units: ne } = V;
          return q.encode_struct(G !== void 0 ? { type: "value", value: G, transform: te, units: ne } : J !== void 0 ? { type: "field", field: J, transform: te, units: ne } : { type: "expr", expr: Z, transform: te, units: ne });
        };
      }
      isValue(this._value) && this.validate(this._value.value);
    }
    materialize(H) {
      return H;
    }
    scalar(H, V) {
      return new UniformScalar(H, V);
    }
    uniform(H) {
      const V = this.get_value(), q = H.get_length() ?? 1;
      if (isExpr$2(V)) {
        const { expr: G, transform: J } = V;
        let Z = G.compute(H);
        return J != null && (Z = J.compute(Z)), Z = this.materialize(Z), this.scalar(Z, q);
      } else {
        const { value: G, transform: J } = V;
        let Z = G;
        return J != null && (Z = J.compute(Z)), Z = this.materialize(Z), this.scalar(Z, q);
      }
    }
  }
  mt(ScalarSpec, "__name__", "ScalarSpec");
  class DictScalar extends ScalarSpec {
  }
  mt(DictScalar, "__name__", "DictScalar");
  class ColorScalar extends ScalarSpec {
  }
  mt(ColorScalar, "__name__", "ColorScalar");
  class NumberScalar extends ScalarSpec {
  }
  mt(NumberScalar, "__name__", "NumberScalar");
  class NullStringScalar extends ScalarSpec {
  }
  mt(NullStringScalar, "__name__", "NullStringScalar");
  class LineJoinScalar extends ScalarSpec {
  }
  mt(LineJoinScalar, "__name__", "LineJoinScalar");
  class LineCapScalar extends ScalarSpec {
  }
  mt(LineCapScalar, "__name__", "LineCapScalar");
  class LineDashScalar extends ScalarSpec {
  }
  mt(LineDashScalar, "__name__", "LineDashScalar");
  class FontScalar extends ScalarSpec {
    _default_override() {
      return settings.dev ? "Bokeh" : unset;
    }
  }
  mt(FontScalar, "__name__", "FontScalar");
  class FontSizeScalar extends ScalarSpec {
  }
  mt(FontSizeScalar, "__name__", "FontSizeScalar");
  class FontStyleScalar extends ScalarSpec {
  }
  mt(FontStyleScalar, "__name__", "FontStyleScalar");
  class TextAlignScalar extends ScalarSpec {
  }
  mt(TextAlignScalar, "__name__", "TextAlignScalar");
  class TextBaselineScalar extends ScalarSpec {
  }
  mt(TextBaselineScalar, "__name__", "TextBaselineScalar");
  class VectorSpec extends Property$1 {
    constructor() {
      super(...arguments);
      mt(this, "__vector__");
      mt(this, "_value", unset);
    }
    get_value() {
      if (this._value !== unset)
        return this._value;
      throw new Error(`${this.obj}.${this.attr} is unset`);
    }
    _update(H) {
      if (isSpec(H) ? this._value = H : this._value = { value: H }, isPlainObject(this._value)) {
        const { _value: V } = this;
        this._value[serialize] = (q) => {
          const { value: G, field: J, expr: Z, transform: te, units: ne } = V;
          return q.encode_struct(G !== void 0 ? { type: "value", value: G, transform: te, units: ne } : J !== void 0 ? { type: "field", field: J, transform: te, units: ne } : { type: "expr", expr: Z, transform: te, units: ne });
        };
      }
      isValue(this._value) && this.validate(this._value.value);
    }
    materialize(H) {
      return H;
    }
    v_materialize(H) {
      return H;
    }
    scalar(H, V) {
      return new UniformScalar(H, V);
    }
    vector(H) {
      return new UniformVector(H);
    }
    uniform(H) {
      const V = this.get_value(), q = H.get_length() ?? 1;
      if (isField$1(V)) {
        const { field: G, transform: J } = V;
        let Z = H.get_column(G);
        if (Z != null)
          return J != null && (Z = J.v_compute(Z)), Z = this.v_materialize(Z), this.vector(Z);
        {
          const te = `attempted to retrieve property array for nonexistent field '${G}'`;
          if (settings.force_fields)
            throw new Error(te);
          return logger$1.warn(te), this.scalar(null, q);
        }
      } else if (isExpr$2(V)) {
        const { expr: G, transform: J } = V;
        let Z = G.v_compute(H);
        return J != null && (Z = J.v_compute(Z)), Z = this.v_materialize(Z), this.vector(Z);
      } else if (isValue(V)) {
        const { value: G, transform: J } = V;
        let Z = G;
        return J != null && (Z = J.compute(Z)), Z = this.materialize(Z), this.scalar(Z, q);
      } else
        unreachable();
    }
    array(H) {
      let V;
      const q = H.get_length() ?? 1, G = this.get_value();
      if (isField$1(G)) {
        const { field: Z } = G, te = H.get_column(Z);
        if (te != null)
          V = this.normalize(te);
        else {
          const ne = `attempted to retrieve property array for nonexistent field '${Z}'`;
          if (settings.force_fields)
            throw new Error(ne);
          logger$1.warn(ne);
          const ee = new Float64Array(q);
          ee.fill(NaN), V = ee;
        }
      } else if (isExpr$2(G)) {
        const { expr: Z } = G;
        V = this.normalize(Z.v_compute(H));
      } else {
        const Z = this.normalize([G.value])[0];
        if (isNumber$3(Z)) {
          const te = new Float64Array(q);
          te.fill(Z), V = te;
        } else
          V = repeat$1(Z, q);
      }
      const { transform: J } = G;
      return J != null && (V = J.v_compute(V)), V;
    }
  }
  mt(VectorSpec, "__name__", "VectorSpec");
  class DataSpec extends VectorSpec {
  }
  mt(DataSpec, "__name__", "DataSpec");
  class UnitsSpec extends VectorSpec {
    constructor() {
      super(...arguments);
      mt(this, "_value", unset);
    }
    _update(H) {
      if (super._update(H), this._value !== unset) {
        const { units: V } = this._value;
        if (V != null && !includes(this.valid_units, V))
          throw new Error(`units must be one of ${this.valid_units.join(", ")}; got: ${V}`);
      }
    }
    get units() {
      return this._value !== unset ? this._value.units ?? this.default_units : this.default_units;
    }
    set units(H) {
      if (this._value !== unset)
        H != this.default_units ? this._value.units = H : delete this._value.units;
      else
        throw new Error(`${this.obj}.${this.attr} is unset`);
    }
  }
  mt(UnitsSpec, "__name__", "UnitsSpec");
  class NumberUnitsSpec extends UnitsSpec {
    array(_) {
      return new Float64Array(super.array(_));
    }
  }
  mt(NumberUnitsSpec, "__name__", "NumberUnitsSpec");
  class BaseCoordinateSpec extends DataSpec {
  }
  mt(BaseCoordinateSpec, "__name__", "BaseCoordinateSpec");
  class CoordinateSpec extends BaseCoordinateSpec {
  }
  mt(CoordinateSpec, "__name__", "CoordinateSpec");
  class CoordinateSeqSpec extends BaseCoordinateSpec {
  }
  mt(CoordinateSeqSpec, "__name__", "CoordinateSeqSpec");
  class CoordinateSeqSeqSeqSpec extends BaseCoordinateSpec {
  }
  mt(CoordinateSeqSeqSeqSpec, "__name__", "CoordinateSeqSeqSeqSpec");
  class XCoordinateSpec extends CoordinateSpec {
    constructor() {
      super(...arguments);
      mt(this, "dimension", "x");
    }
  }
  mt(XCoordinateSpec, "__name__", "XCoordinateSpec");
  class YCoordinateSpec extends CoordinateSpec {
    constructor() {
      super(...arguments);
      mt(this, "dimension", "y");
    }
  }
  mt(YCoordinateSpec, "__name__", "YCoordinateSpec");
  class XCoordinateSeqSpec extends CoordinateSeqSpec {
    constructor() {
      super(...arguments);
      mt(this, "dimension", "x");
    }
  }
  mt(XCoordinateSeqSpec, "__name__", "XCoordinateSeqSpec");
  class YCoordinateSeqSpec extends CoordinateSeqSpec {
    constructor() {
      super(...arguments);
      mt(this, "dimension", "y");
    }
  }
  mt(YCoordinateSeqSpec, "__name__", "YCoordinateSeqSpec");
  class XCoordinateSeqSeqSeqSpec extends CoordinateSeqSeqSeqSpec {
    constructor() {
      super(...arguments);
      mt(this, "dimension", "x");
    }
  }
  mt(XCoordinateSeqSeqSeqSpec, "__name__", "XCoordinateSeqSeqSeqSpec");
  class YCoordinateSeqSeqSeqSpec extends CoordinateSeqSeqSeqSpec {
    constructor() {
      super(...arguments);
      mt(this, "dimension", "y");
    }
  }
  mt(YCoordinateSeqSeqSeqSpec, "__name__", "YCoordinateSeqSeqSeqSpec");
  class AngleSpec extends NumberUnitsSpec {
    get default_units() {
      return "rad";
    }
    get valid_units() {
      return [...AngleUnits];
    }
    materialize(_) {
      const H = -to_radians_coeff(this.units);
      return _ * H;
    }
    v_materialize(_) {
      const H = -to_radians_coeff(this.units), V = new Float32Array(_.length);
      return mul(_, H, V), V;
    }
    array(_) {
      throw new Error("not supported");
    }
  }
  mt(AngleSpec, "__name__", "AngleSpec");
  class DistanceSpec extends NumberUnitsSpec {
    get default_units() {
      return "data";
    }
    get valid_units() {
      return [...SpatialUnits];
    }
  }
  mt(DistanceSpec, "__name__", "DistanceSpec");
  class NullDistanceSpec extends DistanceSpec {
    materialize(_) {
      return _ ?? NaN;
    }
  }
  mt(NullDistanceSpec, "__name__", "NullDistanceSpec");
  class NumberSpec extends DataSpec {
    v_materialize(_) {
      return isTypedArray$2(_) ? _ : new Float64Array(_);
    }
    array(_) {
      return new Float64Array(super.array(_));
    }
  }
  mt(NumberSpec, "__name__", "NumberSpec");
  class ScreenSizeSpec extends NumberSpec {
    valid(_) {
      return isNumber$3(_) && _ >= 0;
    }
  }
  mt(ScreenSizeSpec, "__name__", "ScreenSizeSpec");
  class ColorSpec extends DataSpec {
    materialize(_) {
      return encode_rgba(color2rgba(_));
    }
    v_materialize(_) {
      if (is_NDArray(_)) {
        if (_.dtype == "uint32" && _.dimension == 1)
          return to_big_endian(_);
        if (_.dtype == "uint8" && _.dimension == 1) {
          const [H] = _.shape, V = new RGBAArray(4 * H);
          let q = 0;
          for (const G of _)
            V[q++] = G, V[q++] = G, V[q++] = G, V[q++] = 255;
          return new ColorArray(V.buffer);
        } else if (_.dtype == "uint8" && _.dimension == 2) {
          const [H, V] = _.shape;
          if (V == 4)
            return new ColorArray(_.buffer);
          if (V == 3) {
            const q = new RGBAArray(4 * H);
            for (let G = 0, J = 0; G < V * H; )
              q[J++] = _[G++], q[J++] = _[G++], q[J++] = _[G++], q[J++] = 255;
            return new ColorArray(q.buffer);
          }
        } else if ((_.dtype == "float32" || _.dtype == "float64") && _.dimension == 2) {
          const [H, V] = _.shape;
          if (V == 3 || V == 4) {
            const q = new RGBAArray(4 * H);
            for (let G = 0, J = 0; G < V * H; )
              q[J++] = _[G++] * 255, q[J++] = _[G++] * 255, q[J++] = _[G++] * 255, q[J++] = (V == 3 ? 1 : _[G++]) * 255;
            return new ColorArray(q.buffer);
          }
        } else if (_.dtype == "object" && _.dimension == 1)
          return this._from_css_array(_);
      } else
        return this._from_css_array(_);
      throw new Error("invalid color array");
    }
    _from_css_array(_) {
      const H = _.length, V = new RGBAArray(4 * H);
      let q = 0;
      for (const G of _) {
        const [J, Z, te, ne] = color2rgba(G);
        V[q++] = J, V[q++] = Z, V[q++] = te, V[q++] = ne;
      }
      return new ColorArray(V.buffer);
    }
    vector(_) {
      return new ColorUniformVector(_);
    }
  }
  mt(ColorSpec, "__name__", "ColorSpec");
  class NDArraySpec extends DataSpec {
  }
  mt(NDArraySpec, "__name__", "NDArraySpec");
  class StringSpec extends DataSpec {
  }
  mt(StringSpec, "__name__", "StringSpec");
  class NullStringSpec extends DataSpec {
  }
  mt(NullStringSpec, "__name__", "NullStringSpec");
  class MarkerSpec extends DataSpec {
  }
  mt(MarkerSpec, "__name__", "MarkerSpec");
  class LineJoinSpec extends DataSpec {
  }
  mt(LineJoinSpec, "__name__", "LineJoinSpec");
  class LineCapSpec extends DataSpec {
  }
  mt(LineCapSpec, "__name__", "LineCapSpec");
  class LineDashSpec extends DataSpec {
  }
  mt(LineDashSpec, "__name__", "LineDashSpec");
  class FontSpec extends DataSpec {
    _default_override() {
      return settings.dev ? "Bokeh" : unset;
    }
  }
  mt(FontSpec, "__name__", "FontSpec");
  class FontSizeSpec extends DataSpec {
  }
  mt(FontSizeSpec, "__name__", "FontSizeSpec");
  class FontStyleSpec extends DataSpec {
  }
  mt(FontStyleSpec, "__name__", "FontStyleSpec");
  class TextAlignSpec extends DataSpec {
  }
  mt(TextAlignSpec, "__name__", "TextAlignSpec");
  class TextBaselineSpec extends DataSpec {
  }
  mt(TextBaselineSpec, "__name__", "TextBaselineSpec");
  function uuid4() {
    const O = new Array(32), _ = "0123456789ABCDEF";
    for (let H = 0; H < 32; H++)
      O[H] = _[Math.floor(Math.random() * 16)];
    return O[12] = "4", O[16] = _[O[16].charCodeAt(0) & 3 | 8], O.join("");
  }
  let counter = 1e3;
  function unique_id(O) {
    return settings.dev ? `j${counter++}` : uuid4();
  }
  function unescape$1(O) {
    return O.replace(/&(amp|lt|gt|quot|#x27|#x60);/g, (_, H) => {
      switch (H) {
        case "amp":
          return "&";
        case "lt":
          return "<";
        case "gt":
          return ">";
        case "quot":
          return '"';
        case "#x27":
          return "'";
        case "#x60":
          return "`";
        default:
          return H;
      }
    });
  }
  function use_strict(O) {
    return `'use strict';
${O}`;
  }
  function to_fixed(O, _) {
    return O.toFixed(_).replace(/(\.[0-9]*?)0+$/, "$1").replace(/\.$/, "");
  }
  function insert_text_on_position(O, _, H) {
    const V = [];
    return V.push(O.slice(0, _)), V.push(H), V.push(O.slice(_)), V.join("");
  }
  var h_;
  let DocumentEvent$1 = (h_ = class {
    constructor(_) {
      mt(this, "document");
      /**
       * Indicates whether this event should be emitted internally within bokehjs,
       * or whether it should also be synchronized with the server, if any session
       * is listening for such events.
       */
      mt(this, "sync", !0);
      this.document = _;
    }
    get [Symbol.toStringTag]() {
      return this.constructor.__name__;
    }
    [equals](_, H) {
      return H.eq(this.document, _.document);
    }
  }, mt(h_, "__name__", "DocumentEvent"), h_);
  class DocumentEventBatch extends DocumentEvent$1 {
    constructor(H, V) {
      super(H);
      mt(this, "events");
      this.events = V;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.events, H.events);
    }
  }
  mt(DocumentEventBatch, "__name__", "DocumentEventBatch");
  class DocumentChangedEvent extends DocumentEvent$1 {
  }
  mt(DocumentChangedEvent, "__name__", "DocumentChangedEvent");
  class MessageSentEvent extends DocumentChangedEvent {
    constructor(H, V, q) {
      super(H);
      mt(this, "msg_type");
      mt(this, "msg_data");
      mt(this, "kind", "MessageSent");
      this.msg_type = V, this.msg_data = q;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.msg_type, H.msg_type) && V.eq(this.msg_data, H.msg_data);
    }
    [serialize](H) {
      return {
        kind: this.kind,
        msg_type: this.msg_type,
        msg_data: H.encode(this.msg_data)
      };
    }
  }
  mt(MessageSentEvent, "__name__", "MessageSentEvent");
  class ModelChangedEvent extends DocumentChangedEvent {
    constructor(H, V, q, G) {
      super(H);
      mt(this, "model");
      mt(this, "attr");
      mt(this, "value");
      mt(this, "kind", "ModelChanged");
      this.model = V, this.attr = q, this.value = G;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.model, H.model) && V.eq(this.attr, H.attr) && V.eq(this.value, H.value);
    }
    [serialize](H) {
      return {
        kind: this.kind,
        model: this.model.ref(),
        attr: this.attr,
        new: H.encode(this.value)
      };
    }
  }
  mt(ModelChangedEvent, "__name__", "ModelChangedEvent");
  class ColumnsStreamedEvent extends DocumentChangedEvent {
    constructor(H, V, q, G, J) {
      super(H);
      mt(this, "model");
      mt(this, "attr");
      mt(this, "data");
      mt(this, "rollover");
      mt(this, "kind", "ColumnsStreamed");
      this.model = V, this.attr = q, this.data = G, this.rollover = J;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.model, H.model) && V.eq(this.attr, H.attr) && V.eq(this.data, H.data) && V.eq(this.rollover, H.rollover);
    }
    [serialize](H) {
      return {
        kind: this.kind,
        model: this.model.ref(),
        attr: this.attr,
        data: H.encode(this.data),
        rollover: this.rollover
      };
    }
  }
  mt(ColumnsStreamedEvent, "__name__", "ColumnsStreamedEvent");
  class ColumnsPatchedEvent extends DocumentChangedEvent {
    constructor(H, V, q, G) {
      super(H);
      mt(this, "model");
      mt(this, "attr");
      mt(this, "patches");
      mt(this, "kind", "ColumnsPatched");
      this.model = V, this.attr = q, this.patches = G;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.model, H.model) && V.eq(this.attr, H.attr) && V.eq(this.patches, H.patches);
    }
    [serialize](H) {
      return {
        kind: this.kind,
        attr: this.attr,
        model: this.model.ref(),
        patches: H.encode(this.patches)
      };
    }
  }
  mt(ColumnsPatchedEvent, "__name__", "ColumnsPatchedEvent");
  class TitleChangedEvent extends DocumentChangedEvent {
    constructor(H, V) {
      super(H);
      mt(this, "title");
      mt(this, "kind", "TitleChanged");
      this.title = V;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.title, H.title);
    }
    [serialize](H) {
      return {
        kind: this.kind,
        title: this.title
      };
    }
  }
  mt(TitleChangedEvent, "__name__", "TitleChangedEvent");
  class RootAddedEvent extends DocumentChangedEvent {
    constructor(H, V) {
      super(H);
      mt(this, "model");
      mt(this, "kind", "RootAdded");
      this.model = V;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.model, H.model);
    }
    [serialize](H) {
      return {
        kind: this.kind,
        model: H.encode(this.model)
      };
    }
  }
  mt(RootAddedEvent, "__name__", "RootAddedEvent");
  class RootRemovedEvent extends DocumentChangedEvent {
    constructor(H, V) {
      super(H);
      mt(this, "model");
      mt(this, "kind", "RootRemoved");
      this.model = V;
    }
    [equals](H, V) {
      return super[equals](H, V) && V.eq(this.model, H.model);
    }
    [serialize](H) {
      return {
        kind: this.kind,
        model: this.model.ref()
      };
    }
  }
  mt(RootRemovedEvent, "__name__", "RootRemovedEvent");
  const pretty = Symbol("pretty");
  function is_Printable(O) {
    return isObject$3(O) && pretty in O;
  }
  class Printer {
    constructor(_) {
      mt(this, "precision");
      mt(this, "visited", /* @__PURE__ */ new Set());
      this.precision = _ == null ? void 0 : _.precision;
    }
    to_string(_) {
      if (isObject$3(_)) {
        if (this.visited.has(_))
          return "<circular>";
        this.visited.add(_);
      }
      return is_Printable(_) ? _[pretty](this) : isBoolean$4(_) ? this.boolean(_) : isNumber$3(_) ? this.number(_) : isString$2(_) ? this.string(_) : isArray$3(_) ? this.array(_) : isIterable$2(_) ? this.iterable(_) : isPlainObject(_) ? this.object(_) : isSymbol$1(_) ? this.symbol(_) : _ instanceof ArrayBuffer ? this.array_buffer(_) : `${_}`;
    }
    token(_) {
      return _;
    }
    boolean(_) {
      return `${_}`;
    }
    number(_) {
      return this.precision != null ? _.toFixed(this.precision) : `${_}`;
    }
    string(_) {
      const H = _.includes("'"), V = _.includes('"');
      return H && V ? `\`${_.replace(/`/g, "\\`")}\`` : V ? `'${_}'` : `"${_}"`;
    }
    symbol(_) {
      return _.toString();
    }
    array(_) {
      const H = this.token, V = [];
      for (const q of _)
        V.push(this.to_string(q));
      return `${H("[")}${V.join(`${H(",")} `)}${H("]")}`;
    }
    iterable(_) {
      const H = this.token, V = Object(_)[Symbol.toStringTag] ?? "Object", q = this.array(_);
      return `${V}${H("(")}${q}${H(")")}`;
    }
    object(_) {
      const H = this.token, V = [];
      for (const [q, G] of entries$2(_))
        V.push(`${q}${H(":")} ${this.to_string(G)}`);
      return `${H("{")}${V.join(`${H(",")} `)}${H("}")}`;
    }
    array_buffer(_) {
      return `ArrayBuffer(#${_.byteLength})`;
    }
  }
  mt(Printer, "__name__", "Printer");
  function to_string(O, _) {
    return new Printer(_).to_string(O);
  }
  function union$2(...O) {
    const _ = /* @__PURE__ */ new Set();
    for (const H of O)
      for (const V of H)
        _.add(V);
    return _;
  }
  function difference(O, ..._) {
    const H = new Set(O);
    for (const V of union$2(..._))
      H.delete(V);
    return H;
  }
  function concat(O, ..._) {
    let H = O.length;
    for (const G of _)
      H += G.length;
    const V = new O.constructor(H);
    V.set(O, 0);
    let q = O.length;
    for (const G of _)
      V.set(G, q), q += G.length;
    return V;
  }
  function stream_to_column(O, _, H) {
    if (isArray$3(O) && isArray$3(_)) {
      const q = O.concat(_);
      return H != null && q.length > H ? q.slice(-H) : q;
    }
    const V = O.length + _.length;
    if (H != null && V > H) {
      const q = V - H, G = O.length, J = (() => {
        if (O.length < H) {
          const Z = (() => {
            if (isTypedArray$2(O))
              return O.constructor;
            if (isTypedArray$2(_))
              return _.constructor;
            throw new Error("unsupported array types");
          })(), te = new Z(H);
          return te.set(O, 0), te;
        } else
          return O;
      })();
      for (let Z = q, te = G; Z < te; Z++)
        J[Z - q] = J[Z];
      for (let Z = 0, te = _.length; Z < te; Z++)
        J[Z + (G - q)] = _[Z];
      return J;
    } else {
      const q = (() => {
        if (isTypedArray$2(O))
          return O;
        if (isTypedArray$2(_))
          return new _.constructor(O);
        throw new Error("unsupported array types");
      })();
      return concat(q, _);
    }
  }
  function slice$5(O, _) {
    let H, V, q;
    return isNumber$3(O) ? (H = O, q = O + 1, V = 1) : (H = O.start != null ? O.start : 0, q = O.stop != null ? O.stop : _, V = O.step != null ? O.step : 1), [H, q, V];
  }
  function patch_to_column(O, _) {
    const H = /* @__PURE__ */ new Set();
    let V = !1;
    for (const [q, G] of _) {
      let J, Z, te, ne;
      if (isArray$3(q)) {
        const [de] = q;
        H.add(de), J = O[de].shape, Z = O[de], ne = G, q.length === 2 ? (J = [1, J[0]], te = [q[0], 0, q[1]]) : te = q;
      } else
        isNumber$3(q) ? (ne = [G], H.add(q)) : (ne = G, V = !0), te = [0, 0, q], J = [1, O.length], Z = O;
      let ee = 0;
      const [ae, oe, se] = slice$5(te[1], J[0]), [ce, le, ue] = slice$5(te[2], J[1]);
      for (let de = ae; de < oe; de += se)
        for (let fe = ce; fe < le; fe += ue)
          V && H.add(fe), Z[de * J[1] + fe] = ne[ee], ee++;
    }
    return H;
  }
  function stream_to_columns(O, _, H) {
    const V = dict(O);
    for (const [q, G] of dict(_)) {
      const J = V.get(q) ?? [];
      V.set(q, stream_to_column(J, G, H));
    }
  }
  function patch_to_columns(O, _) {
    const H = dict(O);
    let V = /* @__PURE__ */ new Set();
    for (const [q, G] of dict(_)) {
      const J = H.get(q) ?? [];
      V = union$2(V, patch_to_column(J, G));
    }
    return V;
  }
  const _qualified_names = /* @__PURE__ */ new WeakMap(), Pf = class Pf extends Signalable() {
    constructor(H = {}) {
      super();
      mt(this, "id");
      mt(this, "document", null);
      mt(this, "destroyed", new Signal0(this, "destroyed"));
      mt(this, "change", new Signal0(this, "change"));
      mt(this, "transformchange", new Signal0(this, "transformchange"));
      mt(this, "exprchange", new Signal0(this, "exprchange"));
      mt(this, "streaming", new Signal0(this, "streaming"));
      mt(this, "patching", new Signal(this, "patching"));
      mt(this, "properties", {});
      mt(this, "_watchers", /* @__PURE__ */ new WeakMap());
      mt(this, "_pending", !1);
      mt(this, "_changing", !1);
      const V = isPlainObject(H) && "id" in H;
      this.id = V ? H.id : unique_id();
      for (const [q, { type: G, default_value: J, options: Z }] of entries$2(this._props)) {
        let te;
        if (G instanceof PropertyAlias) {
          const ne = this.properties[G.attr];
          if (typeof ne > "u")
            throw new Error(`can't resolve ${G.attr} before ${q} to create an alias`);
          Object.defineProperty(this.properties, q, {
            get: () => ne,
            configurable: !1,
            enumerable: !1
          });
        } else
          G instanceof Kind ? te = new PrimitiveProperty(this, q, G, J, Z) : te = new G(this, q, Any, J, Z), this.properties[q] = te;
      }
      V ? assert$2(keys$5(H).length == 1, "'id' cannot be used together with property initializers") : (this.initialize_props(H), this.finalize(), this.connect_signals());
    }
    get is_syncable() {
      return !0;
    }
    get type() {
      return this.constructor.__qualified__;
    }
    static get __qualified__() {
      let H = _qualified_names.get(this);
      if (H == null) {
        const { __module__: V, __name__: q } = this;
        H = V != null ? `${V}.${q}` : q, _qualified_names.set(this, H);
      }
      return H;
    }
    static set __qualified__(H) {
      _qualified_names.set(this, H);
    }
    get [Symbol.toStringTag]() {
      return this.constructor.__qualified__;
    }
    static _fix_default(H, V) {
      if (H === void 0 || H === unset)
        return () => unset;
      if (isFunction$3(H))
        return H;
      if (isPrimitive(H))
        return () => H;
      {
        const q = new Cloner();
        return () => q.clone(H);
      }
    }
    // TODO: don't use Partial<>, but exclude inherited properties
    static define(H) {
      for (const [V, q] of entries$2(isFunction$3(H) ? H(kinds) : H)) {
        if (V in this.prototype._props)
          throw new Error(`attempted to redefine property '${this.prototype.type}.${V}'`);
        if (V in this.prototype)
          throw new Error(`attempted to redefine attribute '${this.prototype.type}.${V}'`);
        Object.defineProperty(this.prototype, V, {
          // XXX: don't use tail calls in getters/setters due to https://bugs.webkit.org/show_bug.cgi?id=164306
          get() {
            return this.properties[V].get_value();
          },
          set(ne) {
            return this.setv({ [V]: ne }), this;
          },
          configurable: !1,
          enumerable: !0
        });
        const [G, J, Z = {}] = q, te = {
          type: G,
          default_value: this._fix_default(J, V),
          options: Z
        };
        this.prototype._props = {
          ...this.prototype._props,
          [V]: te
        };
      }
    }
    static internal(H) {
      const V = {};
      for (const [q, G] of entries$2(isFunction$3(H) ? H(kinds) : H)) {
        const [J, Z, te = {}] = G;
        V[q] = [J, Z, { ...te, internal: !0 }];
      }
      this.define(V);
    }
    static mixins(H) {
      function V(J, Z) {
        const te = {};
        for (const [ne, ee] of entries$2(Z))
          te[J + ne] = ee;
        return te;
      }
      const q = {}, G = [];
      for (const J of isArray$3(H) ? H : [H])
        if (isArray$3(J)) {
          const [Z, te] = J;
          extend$5(q, V(Z, te)), G.push([Z, te]);
        } else {
          const Z = J;
          extend$5(q, Z), G.push(["", Z]);
        }
      this.define(q), this.prototype._mixins = [...this.prototype._mixins, ...G];
    }
    static override(H) {
      for (const [V, q] of entries$2(H)) {
        const G = this._fix_default(q, V);
        if (!(V in this.prototype._props))
          throw new Error(`attempted to override nonexistent '${this.prototype.type}.${V}'`);
        const J = this.prototype._props[V], Z = { ...this.prototype._props };
        Z[V] = { ...J, default_value: G }, this.prototype._props = Z;
      }
    }
    static toString() {
      return this.__qualified__;
    }
    toString() {
      return `${this.type}(${this.id})`;
    }
    property(H) {
      if (H in this.properties)
        return this.properties[H];
      throw new Error(`unknown property ${this.type}.${H}`);
    }
    get attributes() {
      const H = {};
      for (const V of this)
        V.is_unset || (H[V.attr] = V.get_value());
      return H;
    }
    [clone](H) {
      const V = /* @__PURE__ */ new Map();
      for (const q of this)
        q.dirty && V.set(q.attr, H.clone(q.get_value()));
      return new this.constructor(V);
    }
    [equals](H, V) {
      for (const q of this) {
        const G = H.property(q.attr);
        if (!V.eq(q.get_value(), G.get_value()))
          return !1;
      }
      return !0;
    }
    [pretty](H) {
      const V = H.token, q = [];
      for (const J of this)
        if (J.dirty) {
          const Z = J.get_value();
          q.push(`${J.attr}${V(":")} ${H.to_string(Z)}`);
        }
      return `${this.constructor.__qualified__}${V("(")}${V("{")}${q.join(`${V(",")} `)}${V("}")}${V(")")}`;
    }
    [serialize](H) {
      const V = this.ref();
      H.add_ref(this, V);
      const q = {};
      for (const te of this)
        if (te.syncable && (H.include_defaults || te.dirty) && !(te.readonly && te.is_unset)) {
          const ne = te.get_value();
          q[te.attr] = H.encode(ne);
        }
      const { type: G, id: J } = this, Z = { type: "object", name: G, id: J };
      return is_empty(q) ? Z : { ...Z, attributes: q };
    }
    initialize_props(H) {
      const V = dict(H), q = /* @__PURE__ */ new Set();
      for (const G of this) {
        const J = V.get(G.attr);
        G.initialize(J), q.add(G.attr);
      }
      for (const [G, J] of V)
        q.has(G) || this.property(G).set_value(J);
    }
    finalize() {
      this.initialize();
    }
    initialize() {
    }
    assert_initialized() {
      for (const H of this)
        H.syncable && !H.readonly && H.get_value();
    }
    connect_signals() {
      for (const H of this) {
        if (!(H instanceof VectorSpec || H instanceof ScalarSpec) || H.is_unset)
          continue;
        const V = H.get_value();
        V.transform != null && this.connect(V.transform.change, () => this.transformchange.emit()), isExpr$2(V) && this.connect(V.expr.change, () => this.exprchange.emit());
      }
    }
    disconnect_signals() {
      Signal.disconnect_receiver(this);
    }
    destroy() {
      this.disconnect_signals(), this.destroyed.emit();
    }
    // Create a new model with exact attribute values to this one, but new identity.
    clone() {
      return new Cloner().clone(this);
    }
    _clear_watchers() {
      this._watchers = /* @__PURE__ */ new WeakMap();
    }
    changed_for(H) {
      const V = this._watchers.get(H);
      return this._watchers.set(H, !1), V ?? !0;
    }
    // Set a hash of model attributes on the object, firing `"change"`. This is
    // the core primitive operation of a model, updating the data and notifying
    // anyone who needs to know about the change in state. The heart of the beast.
    _setv(H, V) {
      const q = V.check_eq, G = /* @__PURE__ */ new Set(), J = this._changing;
      this._changing = !0;
      for (const [Z, te] of H)
        (q === !1 || Z.is_unset || !is_equal(Z.get_value(), te)) && (Z.set_value(te), G.add(Z));
      G.size > 0 && (this._clear_watchers(), this._pending = !0);
      for (const Z of G)
        Z.change.emit();
      if (!J) {
        if (!(V.no_change ?? !1))
          for (; this._pending; )
            this._pending = !1, this.change.emit();
        this._pending = !1, this._changing = !1;
      }
      return G;
    }
    setv(H, V = {}) {
      const q = entries$2(H);
      if (q.length == 0)
        return;
      if (V.silent ?? !1) {
        this._clear_watchers();
        for (const [ne, ee] of q)
          this.properties[ne].set_value(ee);
        return;
      }
      const G = /* @__PURE__ */ new Map(), J = /* @__PURE__ */ new Map();
      for (const [ne, ee] of q) {
        const ae = this.properties[ne];
        G.set(ae, ee), J.set(ae, ae.is_unset ? void 0 : ae.get_value());
      }
      const Z = this._setv(G, V), { document: te } = this;
      if (te != null) {
        const ne = [];
        for (const [ae, oe] of J)
          Z.has(ae) && ne.push([ae, oe, ae.get_value()]);
        for (const [ae, oe, se] of ne)
          if (ae.may_have_refs && this._needs_invalidate(oe, se)) {
            te._invalidate_all_models();
            break;
          }
        const ee = V.sync ?? !0;
        this._push_changes(ne, ee);
      }
    }
    ref() {
      return { id: this.id };
    }
    *[Symbol.iterator]() {
      yield* values$3(this.properties);
    }
    *syncable_properties() {
      for (const H of this)
        H.syncable && (yield H);
    }
    *own_properties() {
      const H = Object.getPrototypeOf(this), V = Object.getPrototypeOf(H), q = new Set(keys$5(V._props));
      for (const G of this)
        q.has(G.attr) || (yield G);
    }
    // add all references from 'v' to 'result', if recurse
    // is true then descend into refs, if false only
    // descend into non-refs
    static _value_record_references(H, V, q) {
      if (!isObject$3(H) || !may_have_refs(H))
        return;
      const { recursive: G } = q;
      if (H instanceof Pf) {
        if (!V.has(H) && (V.add(H), G)) {
          for (const J of H.syncable_properties())
            if (!J.is_unset && J.may_have_refs) {
              const Z = J.get_value();
              Pf._value_record_references(Z, V, { recursive: G });
            }
        }
      } else if (isIterable$2(H))
        for (const J of H)
          Pf._value_record_references(J, V, { recursive: G });
      else if (isPlainObject(H))
        for (const J of values$3(H))
          Pf._value_record_references(J, V, { recursive: G });
    }
    static references(H, V) {
      const q = /* @__PURE__ */ new Set();
      return Pf._value_record_references(H, q, V), q;
    }
    references() {
      return Pf.references(this, { recursive: !0 });
    }
    _doc_attached() {
    }
    _doc_detached() {
    }
    attach_document(H) {
      if (this.document != null) {
        if (this.document == H)
          return;
        throw new Error("models must be owned by only a single document");
      }
      this.document = H, this._doc_attached();
    }
    detach_document() {
      this._doc_detached(), this.document = null;
    }
    _needs_invalidate(H, V) {
      const q = /* @__PURE__ */ new Set();
      Pf._value_record_references(V, q, { recursive: !1 });
      const G = /* @__PURE__ */ new Set();
      Pf._value_record_references(H, G, { recursive: !1 });
      for (const J of q)
        if (!G.has(J))
          return !0;
      for (const J of G)
        if (!q.has(J))
          return !0;
      return !1;
    }
    _push_changes(H, V) {
      if (!this.is_syncable)
        return;
      const { document: q } = this;
      if (q == null)
        return;
      const G = [];
      for (const [J, , Z] of H)
        if (J.syncable) {
          const te = new ModelChangedEvent(q, this, J.attr, Z);
          te.sync = V, G.push(te);
        }
      if (G.length != 0) {
        let J;
        G.length == 1 ? [J] = G : J = new DocumentEventBatch(q, G), q._trigger_on_change(J);
      }
    }
    on_change(H, V) {
      for (const q of isArray$3(H) ? H : [H])
        this.connect(q.change, V);
    }
    stream_to(H, V, q, { sync: G } = {}) {
      const J = H.get_value();
      if (stream_to_columns(J, V, q), this._clear_watchers(), H.set_value(J), this.streaming.emit(), this.document != null) {
        const Z = new ColumnsStreamedEvent(this.document, this, H.attr, V, q);
        Z.sync = G ?? !0, this.document._trigger_on_change(Z);
      }
    }
    patch_to(H, V, { sync: q } = {}) {
      const G = H.get_value(), J = patch_to_columns(G, V);
      if (this._clear_watchers(), H.set_value(G), this.patching.emit([...J]), this.document != null) {
        const Z = new ColumnsPatchedEvent(this.document, this, H.attr, V);
        Z.sync = q ?? !0, this.document._trigger_on_change(Z);
      }
    }
  };
  mt(Pf, "__name__"), mt(Pf, "__module__"), Pf.prototype._props = {}, Pf.prototype._mixins = [];
  let HasProps = Pf;
  class ModelResolver {
    constructor(_, H = []) {
      mt(this, "parent");
      mt(this, "_known_models", /* @__PURE__ */ new Map());
      this.parent = _;
      for (const V of H)
        this.register(V);
    }
    get(_) {
      var H;
      return this._known_models.get(_) ?? ((H = this.parent) == null ? void 0 : H.get(_)) ?? null;
    }
    register(_, H = !1) {
      const V = _.__qualified__;
      H || this.get(V) == null ? this._known_models.set(V, _) : console.warn(`Model '${V}' was already registered with this resolver`);
    }
    get names() {
      return [...this._known_models.keys()];
    }
  }
  mt(ModelResolver, "__name__", "ModelResolver");
  const default_resolver = new ModelResolver(null);
  function is_HasProps(O) {
    return isObject$3(O) && O.prototype instanceof HasProps;
  }
  function register_models(O, _ = !1) {
    for (const H of isArray$3(O) ? O : values$3(O))
      is_HasProps(H) && default_resolver.register(H, _);
  }
  class Slice {
    constructor({ start: _, stop: H, step: V } = {}) {
      mt(this, "start");
      mt(this, "stop");
      mt(this, "step");
      this.start = _ ?? null, this.stop = H ?? null, this.step = V ?? null;
    }
    [serialize](_) {
      return {
        type: "slice",
        start: _.encode(this.start),
        stop: _.encode(this.stop),
        step: _.encode(this.step)
      };
    }
  }
  mt(Slice, "__name__", "Slice");
  const _decoders = /* @__PURE__ */ new Map();
  class DeserializationError extends Error {
  }
  mt(DeserializationError, "__name__", "DeserializationError");
  class Deserializer {
    constructor(_, H = /* @__PURE__ */ new Map(), V) {
      mt(this, "resolver");
      mt(this, "references");
      mt(this, "finalize");
      mt(this, "_decoding", !1);
      mt(this, "_buffers", /* @__PURE__ */ new Map());
      mt(this, "_finalizable", /* @__PURE__ */ new Set());
      this.resolver = _, this.references = H, this.finalize = V;
    }
    static register(_, H) {
      if (!_decoders.has(_))
        _decoders.set(_, H);
      else
        throw new Error(`'${_}' already registered for decoding`);
    }
    decode(_, H) {
      var G;
      if (H != null)
        for (const [J, Z] of H)
          this._buffers.set(J, Z);
      if (this._decoding)
        return this._decode(_);
      this._decoding = !0;
      let V;
      const q = (() => {
        try {
          return this._decode(_);
        } finally {
          V = new Set(this._finalizable), this._decoding = !1, this._buffers.clear(), this._finalizable.clear();
        }
      })();
      for (const J of V)
        (G = this.finalize) == null || G.call(this, J), J.finalize(), J.assert_initialized();
      for (const J of V)
        J.connect_signals();
      return q;
    }
    _decode(_) {
      if (isArray$3(_))
        return this._decode_plain_array(_);
      if (isPlainObject(_))
        if (isString$2(_.type)) {
          const H = _decoders.get(_.type);
          if (H != null)
            return H(_, this);
          switch (_.type) {
            case "ref":
              return this._decode_ref(_);
            case "symbol":
              return this._decode_symbol(_);
            case "number":
              return this._decode_number(_);
            case "array":
              return this._decode_array(_);
            case "set":
              return this._decode_set(_);
            case "map":
              return this._decode_map(_);
            case "bytes":
              return this._decode_bytes(_);
            case "slice":
              return this._decode_slice(_);
            case "date":
              return this._decode_date(_);
            case "value":
              return this._decode_value(_);
            case "field":
              return this._decode_field(_);
            case "expr":
              return this._decode_expr(_);
            case "typed_array":
              return this._decode_typed_array(_);
            case "ndarray":
              return this._decode_ndarray(_);
            case "object":
              return isString$2(_.id) ? this._decode_object_ref(_) : this._decode_object(_);
            default:
              this.error(`unable to decode an object of type '${_.type}'`);
          }
        } else return isString$2(_.id) ? this._decode_ref(_) : this._decode_plain_object(_);
      else
        return _;
    }
    _decode_symbol(_) {
      this.error(`can't resolve named symbol '${_.name}'`);
    }
    _decode_number(_) {
      if ("value" in _) {
        const { value: H } = _;
        if (isString$2(H))
          switch (H) {
            case "nan":
              return NaN;
            case "+inf":
              return 1 / 0;
            case "-inf":
              return -1 / 0;
          }
        else if (isNumber$3(H))
          return H;
      }
      this.error(`invalid number representation '${_}'`);
    }
    _decode_plain_array(_) {
      return map$4(_, (H) => this._decode(H));
    }
    _decode_plain_object(_) {
      const H = {};
      for (const [V, q] of entries$2(_))
        H[V] = this._decode(q);
      return H;
    }
    _decode_array(_) {
      const H = [];
      for (const V of _.entries ?? [])
        H.push(this._decode(V));
      return H;
    }
    _decode_set(_) {
      const H = /* @__PURE__ */ new Set();
      for (const V of _.entries ?? [])
        H.add(this._decode(V));
      return H;
    }
    _decode_map(_) {
      const H = map$4(_.entries ?? [], ([q, G]) => [this._decode(q), this._decode(G)]);
      return H.every(([q, G]) => isString$2(q)) ? Object.fromEntries(H) : new Map(H);
    }
    _decode_bytes(_) {
      const { data: H } = _;
      if (is_ref(H)) {
        const V = this._buffers.get(H.id);
        if (V != null)
          return V;
        this.error(`buffer for id=${H.id} not found`);
      } else return isString$2(H) ? base64_to_buffer(H) : H.buffer;
    }
    _decode_slice(_) {
      const H = this._decode(_.start), V = this._decode(_.stop), q = this._decode(_.step);
      return new Slice({ start: H, stop: V, step: q });
    }
    _decode_date(_) {
      const H = this._decode(_.iso);
      return new Date(H);
    }
    _decode_value(_) {
      const H = this._decode(_.value), V = _.transform != null ? this._decode(_.transform) : void 0, q = _.units != null ? this._decode(_.units) : void 0;
      return { value: H, transform: V, units: q };
    }
    _decode_field(_) {
      const H = this._decode(_.field), V = _.transform != null ? this._decode(_.transform) : void 0, q = _.units != null ? this._decode(_.units) : void 0;
      return { field: H, transform: V, units: q };
    }
    _decode_expr(_) {
      const H = this._decode(_.expr), V = _.transform != null ? this._decode(_.transform) : void 0, q = _.units != null ? this._decode(_.units) : void 0;
      return { expr: H, transform: V, units: q };
    }
    _decode_typed_array(_) {
      const { array: H, order: V, dtype: q } = _, G = this._decode(H);
      switch (V != BYTE_ORDER && swap$3(G, q), q) {
        case "uint8":
          return new Uint8Array(G);
        case "int8":
          return new Int8Array(G);
        case "uint16":
          return new Uint16Array(G);
        case "int16":
          return new Int16Array(G);
        case "uint32":
          return new Uint32Array(G);
        case "int32":
          return new Int32Array(G);
        // case "uint64": return new BigInt64Array(buffer)
        // case "int64":  return new BigInt64Array(buffer)
        case "float32":
          return new Float32Array(G);
        case "float64":
          return new Float64Array(G);
        default:
          this.error(`unsupported dtype '${q}'`);
      }
    }
    _decode_ndarray(_) {
      const { array: H, order: V, dtype: q, shape: G } = _, J = this._decode(H);
      return J instanceof ArrayBuffer && V != BYTE_ORDER && swap$3(J, q), ndarray(J, { dtype: q, shape: G });
    }
    _decode_object(_) {
      const { type: H, attributes: V } = _, q = this._resolve_type(H);
      return V != null ? new q(this._decode(V)) : new q();
    }
    _decode_ref(_) {
      const H = this.references.get(_.id);
      if (H != null)
        return H;
      this.error(`reference ${_.id} isn't known`);
    }
    _decode_object_ref(_) {
      const { id: H, name: V, attributes: q } = _, G = this.references.get(H);
      if (G != null)
        if (G.type == V) {
          const J = this._decode(q ?? {});
          return G.setv(J, { sync: !1 }), G;
        } else
          this.error(`type mismatch for an existing reference '${G}', expected '${V}'`);
      else {
        const J = this._resolve_type(V), Z = new J({ id: H });
        this.references.set(H, Z);
        const te = this._decode(q ?? {});
        return Z.initialize_props(te), this._finalizable.add(Z), Z;
      }
    }
    error(_) {
      throw new DeserializationError(_);
    }
    warning(_) {
      logger$1.warn(_);
    }
    _resolve_type(_) {
      const H = this.resolver.get(_);
      if (H != null)
        return H;
      this.error(`could not resolve type '${_}', which could be due to a widget or a custom model not being registered before first usage`);
    }
  }
  mt(Deserializer, "__name__", "Deserializer");
  const version_re = /^(?<major>\d+)\.(?<minor>\d+)\.(?<patch>\d+)(?:(?<type>-dev\.|-rc\.|.dev|rc)(?<revision>\d+))?(?:\+(?<build>\d+)\..+)?$/;
  var ReleaseType;
  (function(O) {
    O[O.Dev = 0] = "Dev", O[O.Candidate = 1] = "Candidate", O[O.Release = 2] = "Release";
  })(ReleaseType || (ReleaseType = {}));
  class Version {
    constructor(_, H, V, q = ReleaseType.Release, G = 0, J = 0) {
      mt(this, "major");
      mt(this, "minor");
      mt(this, "patch");
      mt(this, "type");
      mt(this, "revision");
      mt(this, "build");
      this.major = _, this.minor = H, this.patch = V, this.type = q, this.revision = G, this.build = J;
    }
    static from(_) {
      return parse_version(_);
    }
    toString() {
      const { major: _, minor: H, patch: V, type: q, revision: G, build: J } = this;
      let Z = `${_}.${H}.${V}`;
      switch (q) {
        case ReleaseType.Dev:
          Z += `-dev.${G}`;
        case ReleaseType.Candidate:
          Z += `-rc.${G}`;
        case ReleaseType.Release:
      }
      return J != 0 && (Z += `+${J}`), Z;
    }
    [equals](_) {
      const { major: H, minor: V, patch: q, type: G, revision: J } = this;
      return H == _.major && V == _.minor && q == _.patch && G == _.type && J == _.revision;
    }
  }
  mt(Version, "__name__", "Version");
  function parse_version(O) {
    const _ = version_re.exec(O);
    if (_ == null || _.groups == null)
      return null;
    const { groups: H } = _, V = Number(H.major), q = Number(H.minor), G = Number(H.patch), J = (() => {
      switch (H.type) {
        case "-dev.":
        case ".dev":
          return ReleaseType.Dev;
        case "-rc.":
        case "rc":
          return ReleaseType.Candidate;
        default:
          return ReleaseType.Release;
      }
    })(), Z = typeof H.revision > "u" ? 0 : Number(H.revision), te = typeof H.build > "u" ? 0 : Number(H.build);
    return new Version(V, q, G, J, Z, te);
  }
  function execute(O, _, ...H) {
    return isFunction$3(O) ? O(_, ...H) : O.execute(_, ...H);
  }
  var E2;
  let Model$1 = (E2 = class extends HasProps {
    constructor(H) {
      super(H);
      mt(this, "_js_callbacks");
    }
    get is_syncable() {
      return this.syncable;
    }
    [equals](H, V) {
      return (V.structural ? !0 : V.eq(this.id, H.id)) && super[equals](H, V);
    }
    initialize() {
      super.initialize(), this._js_callbacks = /* @__PURE__ */ new Map();
    }
    connect_signals() {
      super.connect_signals(), this._update_property_callbacks(), this.connect(this.properties.js_property_callbacks.change, () => this._update_property_callbacks()), this.connect(this.properties.js_event_callbacks.change, () => this._update_event_callbacks()), this.connect(this.properties.subscribed_events.change, () => this._update_event_callbacks());
    }
    /*protected*/
    _process_event(H) {
      for (const V of dict(this.js_event_callbacks).get(H.event_name) ?? [])
        execute(V, H);
      this.document != null && this.subscribed_events.has(H.event_name) && this.document.event_manager.send_event(H);
    }
    trigger_event(H) {
      this.document != null && (H.origin = this, this.document.event_manager.trigger(H));
    }
    _update_event_callbacks() {
      if (this.document == null) {
        logger$1.warn("WARNING: Document not defined for updating event callbacks");
        return;
      }
      this.document.event_manager.subscribed_models.add(this);
    }
    _update_property_callbacks() {
      const H = (V) => {
        const [q, G = null] = V.split(":");
        return G != null ? this.properties[G][q] : this[q];
      };
      for (const [V, q] of this._js_callbacks) {
        const G = H(V);
        for (const J of q)
          this.disconnect(G, J);
      }
      this._js_callbacks.clear();
      for (const [V, q] of dict(this.js_property_callbacks)) {
        const G = q.map((Z) => () => execute(Z, this));
        this._js_callbacks.set(V, G);
        const J = H(V);
        for (const Z of G)
          this.connect(J, Z);
      }
    }
    _doc_attached() {
      (this.js_event_callbacks.size != 0 || this.subscribed_events.size != 0) && this._update_event_callbacks();
    }
    _doc_detached() {
      this.document.event_manager.subscribed_models.delete(this);
    }
    select(H) {
      if (isString$2(H))
        return [...this.references()].filter((V) => V instanceof E2 && V.name === H);
      if (isPlainObject(H) && "type" in H)
        return [...this.references()].filter((V) => V.type == H.type);
      if (H.prototype instanceof HasProps)
        return [...this.references()].filter((V) => V instanceof H);
      throw new Error(`invalid selector ${H}`);
    }
    select_one(H) {
      const V = this.select(H);
      switch (V.length) {
        case 0:
          return null;
        case 1:
          return V[0];
        default:
          throw new Error(`found multiple objects matching the given selector ${H}`);
      }
    }
    get_one(H) {
      const V = this.select_one(H);
      if (V != null)
        return V;
      throw new Error(`could not find any objects matching the given selector ${H}`);
    }
    on_event(H, V) {
      const q = isString$2(H) ? H : H.prototype.event_name, G = dict(this.js_event_callbacks), J = G.get(q) ?? [];
      G.set(q, [...J, V]);
    }
  }, mt(E2, "__name__", "Model"), E2.define(({ Any: H, Unknown: V, Bool: q, Str: G, List: J, Set: Z, Dict: te, Nullable: ne }) => ({
    tags: [J(V), []],
    name: [ne(G), null],
    js_property_callbacks: [te(J(
      H
      /*TODO*/
    )), {}],
    js_event_callbacks: [te(J(
      H
      /*TODO*/
    )), {}],
    subscribed_events: [Z(G), new globalThis.Set()],
    syncable: [q, !0]
  })), E2);
  function decode_def(O, _) {
    var J;
    function H(Z) {
      if (isString$2(Z))
        switch (Z) {
          case "Any":
            return Any;
          case "Unknown":
            return Unknown;
          case "Bool":
            return Bool$3;
          case "Float":
            return Float$1;
          case "Int":
            return Int$1;
          case "Bytes":
            return Bytes$1;
          case "Str":
            return Str;
          case "Null":
            return Null$3;
        }
      else
        switch (Z[0]) {
          case "Regex": {
            const [, te, ne] = Z;
            return Regex(new RegExp(te, ne));
          }
          case "Nullable": {
            const [, te] = Z;
            return Nullable(H(te));
          }
          case "Or": {
            const [, te, ...ne] = Z;
            return Or$1(H(te), ...ne.map(H));
          }
          case "Tuple": {
            const [, te, ...ne] = Z;
            return Tuple(H(te), ...ne.map(H));
          }
          case "List": {
            const [, te] = Z;
            return List$2(H(te));
          }
          case "Struct": {
            const [, ...te] = Z, ne = te.map(([ee, ae]) => [ee, H(ae)]);
            return Struct$1(to_object(ne));
          }
          case "Dict": {
            const [, te] = Z;
            return Dict(H(te));
          }
          case "Mapping": {
            const [, te, ne] = Z;
            return Mapping(H(te), H(ne));
          }
          case "Enum": {
            const [, ...te] = Z;
            return Enum(...te);
          }
          case "Ref": {
            const [, te] = Z, ne = _.resolver.get(te.id);
            if (ne != null)
              return Ref(ne);
            throw new Error(`${te.id} wasn't defined before referencing it`);
          }
          case "AnyRef":
            return AnyRef();
        }
    }
    const V = (() => {
      var ne;
      const Z = ((ne = O.extends) == null ? void 0 : ne.id) ?? "Model";
      if (Z == "Model")
        return Model$1;
      const te = _.resolver.get(Z);
      if (te != null)
        return te;
      throw new Error(`base model ${Z} of ${O.name} is not defined`);
    })(), q = (J = class extends V {
    }, mt(J, "__qualified__", O.name), J);
    function G(Z) {
      return Z === void 0 ? Z : _.decode(Z);
    }
    for (const Z of O.properties ?? []) {
      const te = H(Z.kind);
      q.define({ [Z.name]: [te, G(Z.default)] });
    }
    for (const Z of O.overrides ?? [])
      q.override({ [Z.name]: G(Z.default) });
    return _.resolver.register(q), q;
  }
  var __decorate$1 = function(O, _, H, V) {
    var q = arguments.length, G = q < 3 ? _ : V === null ? V = Object.getOwnPropertyDescriptor(_, H) : V, J;
    if (typeof Reflect == "object" && typeof Reflect.decorate == "function") G = Reflect.decorate(O, _, H, V);
    else for (var Z = O.length - 1; Z >= 0; Z--) (J = O[Z]) && (G = (q < 3 ? J(G) : q > 3 ? J(_, H, G) : J(_, H)) || G);
    return q > 3 && G && Object.defineProperty(_, H, G), G;
  };
  function event$1(O) {
    return (_) => {
      _.prototype.event_name = O;
    };
  }
  const lg = class lg {
    [serialize](_) {
      const { event_name: H, event_values: V } = this, q = _.encode(V);
      return { type: "event", name: H, values: q };
    }
    [equals](_, H) {
      return this.event_name == _.event_name && H.eq(this.event_values, _.event_values);
    }
  };
  mt(lg, "__name__", "BokehEvent"), lg.prototype.publish = !0;
  let BokehEvent = lg;
  class ModelEvent extends BokehEvent {
    constructor() {
      super(...arguments);
      mt(this, "origin", null);
    }
    get event_values() {
      return { model: this.origin };
    }
  }
  mt(ModelEvent, "__name__", "ModelEvent");
  class DocumentEvent extends BokehEvent {
  }
  mt(DocumentEvent, "__name__", "DocumentEvent");
  var p_;
  let DocumentReady = (p_ = class extends DocumentEvent {
    get event_values() {
      return {};
    }
  }, mt(p_, "__name__", "DocumentReady"), p_);
  DocumentReady = __decorate$1([
    event$1("document_ready")
  ], DocumentReady);
  class ConnectionEvent extends DocumentEvent {
  }
  mt(ConnectionEvent, "__name__", "ConnectionEvent");
  const L3 = class L3 extends ConnectionEvent {
    constructor() {
      super(...arguments);
      mt(this, "timestamp", /* @__PURE__ */ new Date());
    }
    get event_values() {
      const { timestamp: H } = this;
      return { timestamp: H };
    }
  };
  mt(L3, "__name__", "ConnectionLost"), L3.prototype.event_name = "connection_lost", L3.prototype.publish = !1;
  let ConnectionLost = L3;
  var m_;
  let ButtonClick = (m_ = class extends ModelEvent {
  }, mt(m_, "__name__", "ButtonClick"), m_);
  ButtonClick = __decorate$1([
    event$1("button_click")
  ], ButtonClick);
  var g_;
  let MenuItemClick = (g_ = class extends ModelEvent {
    constructor(H) {
      super();
      mt(this, "item");
      this.item = H;
    }
    get event_values() {
      const { item: H } = this;
      return { ...super.event_values, item: H };
    }
  }, mt(g_, "__name__", "MenuItemClick"), g_);
  MenuItemClick = __decorate$1([
    event$1("menu_item_click")
  ], MenuItemClick);
  var __;
  let ValueSubmit = (__ = class extends ModelEvent {
    constructor(H) {
      super();
      mt(this, "value");
      this.value = H;
    }
    get event_values() {
      const { value: H } = this;
      return { ...super.event_values, value: H };
    }
  }, mt(__, "__name__", "ValueSubmit"), __);
  ValueSubmit = __decorate$1([
    event$1("value_submit")
  ], ValueSubmit);
  class UIEvent extends ModelEvent {
  }
  mt(UIEvent, "__name__", "UIEvent");
  var y_;
  let LODStart = (y_ = class extends UIEvent {
  }, mt(y_, "__name__", "LODStart"), y_);
  LODStart = __decorate$1([
    event$1("lodstart")
  ], LODStart);
  var v_;
  let LODEnd = (v_ = class extends UIEvent {
  }, mt(v_, "__name__", "LODEnd"), v_);
  LODEnd = __decorate$1([
    event$1("lodend")
  ], LODEnd);
  var b_;
  let RangesUpdate = (b_ = class extends UIEvent {
    constructor(H, V, q, G) {
      super();
      mt(this, "x0");
      mt(this, "x1");
      mt(this, "y0");
      mt(this, "y1");
      this.x0 = H, this.x1 = V, this.y0 = q, this.y1 = G;
    }
    get event_values() {
      const { x0: H, x1: V, y0: q, y1: G } = this;
      return { ...super.event_values, x0: H, x1: V, y0: q, y1: G };
    }
  }, mt(b_, "__name__", "RangesUpdate"), b_);
  RangesUpdate = __decorate$1([
    event$1("rangesupdate")
  ], RangesUpdate);
  var T_;
  let SelectionGeometry = (T_ = class extends UIEvent {
    constructor(H, V) {
      super();
      mt(this, "geometry");
      mt(this, "final");
      this.geometry = H, this.final = V;
    }
    get event_values() {
      const { geometry: H, final: V } = this;
      return { ...super.event_values, geometry: H, final: V };
    }
  }, mt(T_, "__name__", "SelectionGeometry"), T_);
  SelectionGeometry = __decorate$1([
    event$1("selectiongeometry")
  ], SelectionGeometry);
  var x_;
  let Reset = (x_ = class extends UIEvent {
  }, mt(x_, "__name__", "Reset"), x_);
  Reset = __decorate$1([
    event$1("reset")
  ], Reset);
  class PointEvent extends UIEvent {
    constructor(H, V, q, G, J) {
      super();
      mt(this, "sx");
      mt(this, "sy");
      mt(this, "x");
      mt(this, "y");
      mt(this, "modifiers");
      this.sx = H, this.sy = V, this.x = q, this.y = G, this.modifiers = J;
    }
    get event_values() {
      const { sx: H, sy: V, x: q, y: G, modifiers: J } = this;
      return { ...super.event_values, sx: H, sy: V, x: q, y: G, modifiers: J };
    }
  }
  mt(PointEvent, "__name__", "PointEvent");
  var w_;
  let Pan = (w_ = class extends PointEvent {
    /* TODO: direction: -1 | 1 */
    constructor(H, V, q, G, J, Z, te) {
      super(H, V, q, G, te);
      mt(this, "delta_x");
      mt(this, "delta_y");
      this.delta_x = J, this.delta_y = Z;
    }
    get event_values() {
      const {
        delta_x: H,
        delta_y: V
        /*, direction*/
      } = this;
      return {
        ...super.event_values,
        delta_x: H,
        delta_y: V
        /*, direction*/
      };
    }
  }, mt(w_, "__name__", "Pan"), w_);
  Pan = __decorate$1([
    event$1("pan")
  ], Pan);
  var S_;
  let Pinch = (S_ = class extends PointEvent {
    constructor(H, V, q, G, J, Z) {
      super(H, V, q, G, Z);
      mt(this, "scale");
      this.scale = J;
    }
    get event_values() {
      const { scale: H } = this;
      return { ...super.event_values, scale: H };
    }
  }, mt(S_, "__name__", "Pinch"), S_);
  Pinch = __decorate$1([
    event$1("pinch")
  ], Pinch);
  var E_;
  let Rotate = (E_ = class extends PointEvent {
    constructor(H, V, q, G, J, Z) {
      super(H, V, q, G, Z);
      mt(this, "rotation");
      this.rotation = J;
    }
    get event_values() {
      const { rotation: H } = this;
      return { ...super.event_values, rotation: H };
    }
  }, mt(E_, "__name__", "Rotate"), E_);
  Rotate = __decorate$1([
    event$1("rotate")
  ], Rotate);
  var A_;
  let MouseWheel = (A_ = class extends PointEvent {
    constructor(H, V, q, G, J, Z) {
      super(H, V, q, G, Z);
      mt(this, "delta");
      this.delta = J;
    }
    get event_values() {
      const { delta: H } = this;
      return { ...super.event_values, delta: H };
    }
  }, mt(A_, "__name__", "MouseWheel"), A_);
  MouseWheel = __decorate$1([
    event$1("wheel")
  ], MouseWheel);
  var Q_;
  let MouseMove = (Q_ = class extends PointEvent {
  }, mt(Q_, "__name__", "MouseMove"), Q_);
  MouseMove = __decorate$1([
    event$1("mousemove")
  ], MouseMove);
  var C_;
  let MouseEnter = (C_ = class extends PointEvent {
  }, mt(C_, "__name__", "MouseEnter"), C_);
  MouseEnter = __decorate$1([
    event$1("mouseenter")
  ], MouseEnter);
  var L_;
  let MouseLeave = (L_ = class extends PointEvent {
  }, mt(L_, "__name__", "MouseLeave"), L_);
  MouseLeave = __decorate$1([
    event$1("mouseleave")
  ], MouseLeave);
  var k_;
  let Tap$1 = (k_ = class extends PointEvent {
  }, mt(k_, "__name__", "Tap"), k_);
  Tap$1 = __decorate$1([
    event$1("tap")
  ], Tap$1);
  var M_;
  let DoubleTap = (M_ = class extends PointEvent {
  }, mt(M_, "__name__", "DoubleTap"), M_);
  DoubleTap = __decorate$1([
    event$1("doubletap")
  ], DoubleTap);
  var R_;
  let Press = (R_ = class extends PointEvent {
  }, mt(R_, "__name__", "Press"), R_);
  Press = __decorate$1([
    event$1("press")
  ], Press);
  var O_;
  let PressUp = (O_ = class extends PointEvent {
  }, mt(O_, "__name__", "PressUp"), O_);
  PressUp = __decorate$1([
    event$1("pressup")
  ], PressUp);
  var I_;
  let PanStart = (I_ = class extends PointEvent {
  }, mt(I_, "__name__", "PanStart"), I_);
  PanStart = __decorate$1([
    event$1("panstart")
  ], PanStart);
  var D_;
  let PanEnd = (D_ = class extends PointEvent {
  }, mt(D_, "__name__", "PanEnd"), D_);
  PanEnd = __decorate$1([
    event$1("panend")
  ], PanEnd);
  var N_;
  let PinchStart = (N_ = class extends PointEvent {
  }, mt(N_, "__name__", "PinchStart"), N_);
  PinchStart = __decorate$1([
    event$1("pinchstart")
  ], PinchStart);
  var P_;
  let PinchEnd = (P_ = class extends PointEvent {
  }, mt(P_, "__name__", "PinchEnd"), P_);
  PinchEnd = __decorate$1([
    event$1("pinchend")
  ], PinchEnd);
  var $_;
  let RotateStart = ($_ = class extends PointEvent {
  }, mt($_, "__name__", "RotateStart"), $_);
  RotateStart = __decorate$1([
    event$1("rotatestart")
  ], RotateStart);
  var F_;
  let RotateEnd = (F_ = class extends PointEvent {
  }, mt(F_, "__name__", "RotateEnd"), F_);
  RotateEnd = __decorate$1([
    event$1("rotateend")
  ], RotateEnd);
  Deserializer.register("model", decode_def);
  class EventManager {
    constructor(_) {
      mt(this, "document");
      mt(this, "subscribed_models", /* @__PURE__ */ new Set());
      this.document = _;
    }
    send_event(_) {
      if (_.publish) {
        const H = new MessageSentEvent(this.document, "bokeh_event", _);
        this.document._trigger_on_change(H);
      }
      this.document._trigger_on_event(_);
    }
    trigger(_) {
      for (const H of this.subscribed_models)
        _.origin != null && _.origin != H || H._process_event(_);
    }
  }
  mt(EventManager, "__name__", "EventManager");
  const documents = [], DEFAULT_TITLE = "Bokeh Application", K2 = class K2 {
    constructor(_ = {}) {
      /** @experimental */
      mt(this, "views_manager");
      mt(this, "event_manager");
      mt(this, "idle");
      mt(this, "_init_timestamp");
      mt(this, "_resolver");
      mt(this, "_title");
      mt(this, "_roots");
      /*protected*/
      mt(this, "_all_models");
      mt(this, "_new_models");
      mt(this, "_all_models_freeze_count");
      mt(this, "_callbacks");
      mt(this, "_document_callbacks");
      mt(this, "_message_callbacks");
      mt(this, "_idle_roots");
      mt(this, "_interactive_timestamp");
      mt(this, "_interactive_plot");
      mt(this, "_interactive_finalize");
      documents.push(this), this._init_timestamp = Date.now(), this._resolver = _.resolver ?? new ModelResolver(default_resolver), this._title = DEFAULT_TITLE, this._roots = [], this._all_models = /* @__PURE__ */ new Map(), this._new_models = /* @__PURE__ */ new Set(), this._all_models_freeze_count = 0, this._callbacks = /* @__PURE__ */ new Map(), this._document_callbacks = /* @__PURE__ */ new Map(), this._message_callbacks = /* @__PURE__ */ new Map(), this.event_manager = new EventManager(this), this.idle = new Signal0(this, "idle"), this._idle_roots = /* @__PURE__ */ new WeakSet(), this._interactive_timestamp = null, this._interactive_plot = null, _.roots != null && this._add_roots(..._.roots);
    }
    [equals](_, H) {
      return this == _;
    }
    get all_models() {
      return new Set(this._all_models.values());
    }
    get is_idle() {
      for (const _ of this._roots)
        if (!this._idle_roots.has(_))
          return !1;
      return !0;
    }
    notify_idle(_) {
      this._idle_roots.add(_), this.is_idle && (logger$1.info(`document idle at ${Date.now() - this._init_timestamp} ms`), this.event_manager.send_event(new DocumentReady()), this.idle.emit());
    }
    clear() {
      this._push_all_models_freeze();
      try {
        for (; this._roots.length > 0; )
          this.remove_root(this._roots[0]);
      } finally {
        this._pop_all_models_freeze();
      }
    }
    interactive_start(_, H = null) {
      this._interactive_plot == null && (this._interactive_plot = _, this._interactive_plot.trigger_event(new LODStart())), this._interactive_finalize = H, this._interactive_timestamp = Date.now();
    }
    interactive_stop() {
      this._interactive_plot != null && (this._interactive_plot.trigger_event(new LODEnd()), this._interactive_finalize != null && this._interactive_finalize()), this._interactive_plot = null, this._interactive_timestamp = null, this._interactive_finalize = null;
    }
    interactive_duration() {
      return this._interactive_timestamp == null ? -1 : Date.now() - this._interactive_timestamp;
    }
    destructively_move(_) {
      if (_ === this)
        throw new Error("Attempted to overwrite a document with itself");
      _.clear();
      const H = copy$3(this._roots);
      this.clear();
      for (const V of H)
        if (V.document != null)
          throw new Error(`Somehow we didn't detach ${V}`);
      if (this._all_models.size != 0)
        throw new Error(`this._all_models still had stuff in it: ${this._all_models}`);
      for (const V of H)
        _.add_root(V);
      _.set_title(this._title);
    }
    // TODO other fields of doc
    _push_all_models_freeze() {
      this._all_models_freeze_count += 1;
    }
    _pop_all_models_freeze() {
      this._all_models_freeze_count -= 1, this._all_models_freeze_count === 0 && this._recompute_all_models();
    }
    /*protected*/
    _invalidate_all_models() {
      logger$1.debug("invalidating document models"), this._all_models_freeze_count === 0 && this._recompute_all_models();
    }
    _recompute_all_models() {
      let _ = /* @__PURE__ */ new Set();
      for (const J of this._roots)
        _ = union$2(_, J.references());
      const H = new Set(this._all_models.values()), V = difference(H, _), q = difference(_, H), G = /* @__PURE__ */ new Map();
      for (const J of _)
        G.set(J.id, J);
      for (const J of V)
        J.detach_document();
      for (const J of q)
        J.attach_document(this), this._new_models.add(J);
      this._all_models = G;
    }
    roots() {
      return this._roots;
    }
    _add_roots(..._) {
      if (_ = _.filter((H) => !this._roots.includes(H)), _.length == 0)
        return !1;
      this._push_all_models_freeze();
      try {
        this._roots.push(..._);
      } finally {
        this._pop_all_models_freeze();
      }
      return !0;
    }
    _remove_root(_) {
      const H = this._roots.indexOf(_);
      if (H < 0)
        return !1;
      this._push_all_models_freeze();
      try {
        this._roots.splice(H, 1);
      } finally {
        this._pop_all_models_freeze();
      }
      return !0;
    }
    _set_title(_) {
      const H = _ != this._title;
      return H && (this._title = _), H;
    }
    add_root(_, { sync: H } = {}) {
      if (this._add_roots(_)) {
        const V = new RootAddedEvent(this, _);
        V.sync = H ?? !0, this._trigger_on_change(V);
      }
    }
    remove_root(_, { sync: H } = {}) {
      if (this._remove_root(_)) {
        const V = new RootRemovedEvent(this, _);
        V.sync = H ?? !0, this._trigger_on_change(V);
      }
    }
    set_title(_, { sync: H } = {}) {
      if (this._set_title(_)) {
        const V = new TitleChangedEvent(this, _);
        V.sync = H ?? !0, this._trigger_on_change(V);
      }
    }
    title() {
      return this._title;
    }
    get_model_by_id(_) {
      return this._all_models.get(_) ?? null;
    }
    get_model_by_name(_) {
      const H = [];
      for (const V of this._all_models.values())
        V instanceof Model$1 && V.name == _ && H.push(V);
      switch (H.length) {
        case 0:
          return null;
        case 1:
          return H[0];
        default:
          throw new Error(`Multiple models are named '${_}'`);
      }
    }
    on_message(_, H) {
      const V = this._message_callbacks.get(_);
      V == null ? this._message_callbacks.set(_, /* @__PURE__ */ new Set([H])) : V.add(H);
    }
    remove_on_message(_, H) {
      var V;
      (V = this._message_callbacks.get(_)) == null || V.delete(H);
    }
    _trigger_on_message(_, H) {
      const V = this._message_callbacks.get(_);
      if (V != null)
        for (const q of V)
          q(H);
    }
    on_change(_, H = !1) {
      this._callbacks.has(_) || this._callbacks.set(_, H);
    }
    remove_on_change(_) {
      this._callbacks.delete(_);
    }
    _trigger_on_change(_) {
      for (const [H, V] of this._callbacks)
        if (!V && _ instanceof DocumentEventBatch)
          for (const q of _.events)
            H(q);
        else
          H(_);
    }
    _trigger_on_event(_) {
      const H = this._document_callbacks.get(_.event_name);
      if (H != null)
        for (const V of H)
          execute(V, this, _);
    }
    on_event(_, ...H) {
      const V = isString$2(_) ? _ : _.prototype.event_name, q = this._document_callbacks.get(V) ?? [], G = H;
      this._document_callbacks.set(V, [...q, ...G]);
    }
    to_json_string(_ = !0) {
      return JSON.stringify(this.to_json(_));
    }
    to_json(_ = !0) {
      const V = new Serializer({ include_defaults: _ }).encode(this._roots);
      return {
        version: version$6,
        title: this._title,
        roots: V
      };
    }
    static from_json_string(_, H) {
      const V = JSON.parse(_);
      return K2.from_json(V, H);
    }
    static _handle_version(_) {
      _.version == null && logger$1.warn("'version' field is missing");
      const H = _.version ?? "0.0.0", V = Version.from(H), q = Version.from(version$6), G = `new document using Bokeh ${H} and BokehJS ${version$6}`;
      is_equal(V, q) ? logger$1.debug(G) : logger$1.warn(`Bokeh/BokehJS version mismatch: ${G}`);
    }
    static from_json(_, H) {
      logger$1.debug("Creating Document from JSON"), K2._handle_version(_);
      const V = new ModelResolver(default_resolver);
      _.defs != null && new Deserializer(V).decode(_.defs);
      const q = new K2({ resolver: V });
      q._push_all_models_freeze();
      const G = (ne) => H == null ? void 0 : H.push(ne);
      q.on_change(G, !0);
      const J = new Deserializer(V, q._all_models, (ne) => ne.attach_document(q)), Z = J.decode(_.roots), te = _.callbacks != null ? J.decode(_.callbacks) : {};
      q.remove_on_change(G);
      for (const [ne, ee] of entries$2(te))
        q.on_event(ne, ...ee);
      for (const ne of Z)
        q.add_root(ne);
      return _.title != null && q.set_title(_.title), q._pop_all_models_freeze(), q;
    }
    replace_with_json(_) {
      K2.from_json(_).destructively_move(this);
    }
    create_json_patch(_) {
      for (const G of _)
        if (G.document != this)
          throw new Error("Cannot create a patch using events from a different document");
      const H = /* @__PURE__ */ new Map();
      for (const G of this._all_models.values())
        this._new_models.has(G) || H.set(G, G.ref());
      const q = { events: new Serializer({ references: H, binary: !0 }).encode(_) };
      return this._new_models.clear(), q;
    }
    apply_json_patch(_, H = /* @__PURE__ */ new Map()) {
      this._push_all_models_freeze();
      const q = new Deserializer(this._resolver, this._all_models, (G) => G.attach_document(this)).decode(_.events, H);
      for (const G of q)
        switch (G.kind) {
          case "MessageSent": {
            const { msg_type: J, msg_data: Z } = G;
            this._trigger_on_message(J, Z);
            break;
          }
          case "ModelChanged": {
            const { model: J, attr: Z, new: te } = G;
            J.setv({ [Z]: te }, { sync: !1 });
            break;
          }
          case "ColumnDataChanged": {
            const { model: J, attr: Z, data: te, cols: ne } = G;
            if (ne != null) {
              const ee = dict(te), ae = dict(J.property(Z).get_value());
              for (const [oe, se] of ae)
                ee.has(oe) || ee.set(oe, se);
            }
            J.setv({ data: te }, { sync: !1, check_eq: !1 });
            break;
          }
          case "ColumnsStreamed": {
            const { model: J, attr: Z, data: te, rollover: ne } = G, ee = J.property(Z);
            J.stream_to(ee, te, ne, { sync: !1 });
            break;
          }
          case "ColumnsPatched": {
            const { model: J, attr: Z, patches: te } = G, ne = J.property(Z);
            J.patch_to(ne, te, { sync: !1 });
            break;
          }
          case "RootAdded": {
            this.add_root(G.model, { sync: !1 });
            break;
          }
          case "RootRemoved": {
            this.remove_root(G.model, { sync: !1 });
            break;
          }
          case "TitleChanged": {
            this.set_title(G.title, { sync: !1 });
            break;
          }
          default:
            throw new Error(`unknown patch event type '${G.kind}'`);
        }
      this._pop_all_models_freeze();
    }
  };
  mt(K2, "__name__", "Document");
  let Document = K2;
  async function _build_view(O, _, H) {
    assert$2(O != null, "model doesn't implement a view");
    const V = new O({ ...H, model: _ });
    return V.initialize(), await V.lazy_initialize(), V;
  }
  async function build_view(O, _ = { parent: null }, H = (V) => V.default_view) {
    const V = await _build_view(H(O), O, _);
    return V.connect_signals(), V;
  }
  async function build_views(O, _, H = { parent: null }, V = (q) => q.default_view) {
    const q = difference$1([...O.keys()], _), G = [];
    for (const te of q) {
      const ne = O.get(te);
      ne != null && (O.delete(te), G.push(ne), ne.remove());
    }
    const J = [], Z = _.filter((te) => !O.has(te));
    for (const te of Z) {
      const ne = await _build_view(V(te), te, H);
      O.set(te, ne), J.push(ne);
    }
    for (const te of J)
      te.connect_signals();
    return {
      created: J,
      removed: G
    };
  }
  function remove_views(O) {
    for (const [_, H] of O)
      H.remove(), O.delete(_);
  }
  class ViewManager {
    constructor(_ = [], H) {
      mt(this, "global");
      mt(this, "_roots");
      this.global = H, this._roots = new Set(_);
    }
    toString() {
      return `ViewManager(${[...this._roots].map((H) => `${H}`).join(", ")})`;
    }
    async build_view(_, H = null) {
      const V = await build_view(_, { owner: this, parent: H });
      return H == null && this.add(V), V;
    }
    get(_) {
      for (const H of this._roots)
        if (H.model == _)
          return H;
      return null;
    }
    get_by_id(_) {
      for (const H of this._roots)
        if (H.model.id == _)
          return H;
      return null;
    }
    add(_) {
      var H;
      this._roots.add(_), (H = this.global) == null || H.add(_);
    }
    delete(_) {
      var H;
      this._roots.delete(_), (H = this.global) == null || H.delete(_);
    }
    remove(_) {
      this.delete(_);
    }
    clear() {
      for (const _ of this)
        _.remove();
    }
    /* TODO (TS 5.2)
    [Symbol.dispose](): void {
      this.clear()
    }
    */
    get roots() {
      return [...this._roots];
    }
    *[Symbol.iterator]() {
      yield* this._roots;
    }
    *views() {
      yield* this.query(() => !0);
    }
    *query(_) {
      const H = /* @__PURE__ */ new Set();
      function* V(q) {
        if (!H.has(q)) {
          H.add(q), _(q) && (yield q);
          for (const G of q.children())
            yield* V(G);
        }
      }
      for (const q of this._roots)
        yield* V(q);
    }
    query_one(_) {
      for (const H of this.query(_))
        return H;
      return null;
    }
    *find(_) {
      yield* this.query((H) => H.model == _);
    }
    *find_by_id(_) {
      yield* this.query((H) => H.model.id == _);
    }
    find_one(_) {
      for (const H of this.find(_))
        return H;
      return null;
    }
    find_one_by_id(_) {
      for (const H of this.find_by_id(_))
        return H;
      return null;
    }
    get_one(_) {
      const H = this.find_one(_);
      if (H != null)
        return H;
      throw new Error(`cannot find a view for ${_}`);
    }
    get_one_by_id(_) {
      const H = this.find_one_by_id(_);
      if (H != null)
        return H;
      throw new Error(`cannot find a view for a model with '${_}' identity`);
    }
    find_all(_) {
      return [...this.find(_)];
    }
    find_all_by_id(_) {
      return [...this.find_by_id(_)];
    }
  }
  mt(ViewManager, "__name__", "ViewManager");
  class Coordinate extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(Coordinate, "__name__", "Coordinate");
  const ImplicitTarget = Enum("canvas", "plot", "frame", "parent"), NodeTarget = Or$1(Ref(Model$1), ImplicitTarget), cg = class cg {
    constructor(_, H = !1) {
      mt(this, "target");
      mt(this, "frozen");
      mt(this, "_left", null);
      mt(this, "_right", null);
      mt(this, "_top", null);
      mt(this, "_bottom", null);
      this.target = _, this.frozen = H;
    }
    _node(_) {
      const { target: H, frozen: V } = this, q = new Node$4({ target: H, symbol: _ });
      return V && (this[`_${_}`] = q), q;
    }
    get left() {
      return this._left ?? this._node("left");
    }
    get right() {
      return this._right ?? this._node("right");
    }
    get top() {
      return this._top ?? this._node("top");
    }
    get bottom() {
      return this._bottom ?? this._node("bottom");
    }
    freeze() {
      return new cg(this.target, !0);
    }
  };
  mt(cg, "__name__", "BoxNodes");
  let BoxNodes = cg;
  var s2;
  let Node$4 = (s2 = class extends Coordinate {
    constructor(_) {
      super(_);
    }
    static get frame() {
      return this._frame_nodes;
    }
    static get canvas() {
      return this._canvas_nodes;
    }
  }, mt(s2, "__name__", "Node"), s2.define(({ Str: _, Int: H }) => ({
    target: [NodeTarget],
    symbol: [_],
    offset: [H, 0]
  })), mt(s2, "_frame_nodes", new BoxNodes("frame")), mt(s2, "_canvas_nodes", new BoxNodes("canvas")), s2);
  var A0;
  let XY$1 = (A0 = class extends Coordinate {
    constructor(_) {
      super(_);
    }
  }, mt(A0, "__name__", "XY"), A0.define(({ Float: _ }) => ({
    x: [_],
    y: [_]
  })), A0);
  const ug = class ug extends Coordinate {
    constructor(_) {
      super(_);
    }
  };
  mt(ug, "__name__", "Indexed"), ug.define(({ Int: _, AnyRef: H }) => ({
    index: [_],
    renderer: [H()]
  }));
  let Indexed = ug;
  var B_;
  let View$2 = (B_ = class {
    constructor(_) {
      mt(this, "removed", new Signal0(this, "removed"));
      mt(this, "model");
      mt(this, "parent");
      mt(this, "root");
      mt(this, "owner");
      mt(this, "_ready", Promise.resolve(void 0));
      mt(this, "_has_finished");
      /** @internal */
      mt(this, "_slots", /* @__PURE__ */ new WeakMap());
      mt(this, "_destroyed", !1);
      mt(this, "_idle_notified", !1);
      const { model: H, parent: V, owner: q } = _;
      this.model = H, this.parent = V, V == null ? (this.root = this, this.owner = q ?? new ViewManager([this])) : (this.root = V.root, this.owner = this.root.owner);
    }
    get ready() {
      return this._ready;
    }
    *children() {
    }
    mark_finished() {
      this._has_finished = !0;
    }
    connect(_, H) {
      let V = this._slots.get(H);
      return V == null && (V = (q, G) => {
        const J = Promise.resolve(H.call(this, q, G));
        this._ready = this._ready.then(() => J), this.root != this && (this.root._ready = this.root._ready.then(() => this._ready));
      }, this._slots.set(H, V)), _.connect(V, this);
    }
    disconnect(_, H) {
      return _.disconnect(H, this);
    }
    initialize() {
      this._has_finished = !1;
    }
    async lazy_initialize() {
    }
    remove() {
      this.disconnect_signals(), this.owner.remove(this), this.removed.emit(), this._destroyed = !0;
    }
    get is_destroyed() {
      return this._destroyed;
    }
    toString() {
      return `${this.model.type}View(${this.model.id})`;
    }
    serializable_state() {
      return { type: this.model.type };
    }
    get is_root() {
      return this.parent == null;
    }
    has_finished() {
      return this._has_finished;
    }
    get is_idle() {
      return this.has_finished();
    }
    connect_signals() {
    }
    disconnect_signals() {
      Signal.disconnect_receiver(this);
    }
    on_change(_, H) {
      for (const V of isArray$3(_) ? _ : [_])
        this.connect(V.change, H);
    }
    on_transitive_change(_, H) {
      const V = () => {
        const Z = _.is_unset ? [] : _.get_value();
        return HasProps.references(Z, { recursive: !1 });
      }, q = (Z) => {
        for (const te of Z)
          this.connect(te.change, H);
      }, G = (Z) => {
        for (const te of Z)
          this.disconnect(te.change, H);
      };
      let J = V();
      q(J), this.on_change(_, () => {
        G(J), J = V(), q(J), H();
      });
    }
    cursor(_, H) {
      return null;
    }
    notify_finished() {
      this.is_root ? !this._idle_notified && this.has_finished() && this.model.document != null && (this._idle_notified = !0, this.model.document.notify_idle(this.model)) : this.root.notify_finished();
    }
    resolve_frame() {
      return null;
    }
    resolve_canvas() {
      return null;
    }
    resolve_plot() {
      return null;
    }
    resolve_target(_) {
      if (isString$2(_)) {
        const H = (V) => {
          let q = this;
          for (; q != null; ) {
            const G = V(q);
            if (G != null)
              return G;
            q = q.parent;
          }
          return null;
        };
        switch (_) {
          case "parent":
            return this.parent;
          case "frame":
            return H((V) => V.resolve_frame());
          case "canvas":
            return H((V) => V.resolve_canvas());
          case "plot":
            return H((V) => V.resolve_plot());
        }
      } else {
        const H = [this.root];
        for (; ; ) {
          const V = H.shift();
          if (V == null)
            break;
          if (V.model == _)
            return V;
          H.push(...V.children());
        }
        return null;
      }
    }
    resolve_symbol(_) {
      return { x: NaN, y: NaN };
    }
    resolve_node(_) {
      const H = this.resolve_target(_.target);
      return H != null ? H.resolve_symbol(_) : { x: NaN, y: NaN };
    }
    resolve_coordinate(_) {
      var H, V;
      if (_ instanceof XY$1) {
        let q = this;
        for (; q != null && q.resolve_xy == null; )
          q = q.parent;
        return ((H = q == null ? void 0 : q.resolve_xy) == null ? void 0 : H.call(q, _)) ?? { x: NaN, y: NaN };
      } else if (_ instanceof Indexed) {
        let q = this;
        for (; q != null && q.resolve_indexed == null; )
          q = q.parent;
        return ((V = q == null ? void 0 : q.resolve_indexed) == null ? void 0 : V.call(q, _)) ?? { x: NaN, y: NaN };
      } else return _ instanceof Node$4 ? this.resolve_node(_) : { x: NaN, y: NaN };
    }
    resolve_as_xy(_) {
      const H = this.resolve_coordinate(_);
      return isNumber$3(H) ? { x: NaN, y: NaN } : H;
    }
    resolve_as_scalar(_, H) {
      const V = this.resolve_coordinate(_);
      return isNumber$3(V) ? V : V[H];
    }
  }, mt(B_, "__name__", "View"), B_);
  const { min: min$a, max: max$h, round: round$6 } = Math;
  function empty$2() {
    return {
      x0: 1 / 0,
      y0: 1 / 0,
      x1: -1 / 0,
      y1: -1 / 0
    };
  }
  function positive_x() {
    return {
      x0: Number.MIN_VALUE,
      y0: -1 / 0,
      x1: 1 / 0,
      y1: 1 / 0
    };
  }
  function positive_y() {
    return {
      x0: -1 / 0,
      y0: Number.MIN_VALUE,
      x1: 1 / 0,
      y1: 1 / 0
    };
  }
  function _min(O, _) {
    return isNaN(O) ? _ : isNaN(_) ? O : min$a(O, _);
  }
  function _max(O, _) {
    return isNaN(O) ? _ : isNaN(_) ? O : max$h(O, _);
  }
  function union$1(O, _) {
    return {
      x0: _min(O.x0, _.x0),
      x1: _max(O.x1, _.x1),
      y0: _min(O.y0, _.y0),
      y1: _max(O.y1, _.y1)
    };
  }
  var mf;
  let BBox$1 = (mf = class {
    constructor(_, H = !1) {
      mt(this, "x0");
      mt(this, "y0");
      mt(this, "x1");
      mt(this, "y1");
      mt(this, "_x_screen");
      mt(this, "_y_screen");
      mt(this, "_x_view");
      mt(this, "_y_view");
      if (_ == null)
        this.x0 = 0, this.y0 = 0, this.x1 = 0, this.y1 = 0;
      else if ("x0" in _) {
        const { x0: V, y0: q, x1: G, y1: J } = _;
        if (!isFinite(V + q + G + J))
          this.x0 = NaN, this.y0 = NaN, this.x1 = NaN, this.y1 = NaN;
        else {
          if (!(V <= G && q <= J))
            throw new Error(`invalid bbox {x0: ${V}, y0: ${q}, x1: ${G}, y1: ${J}}`);
          this.x0 = V, this.y0 = q, this.x1 = G, this.y1 = J;
        }
      } else if ("x" in _) {
        const { x: V, y: q, width: G, height: J } = _;
        if (!(G >= 0 && J >= 0))
          throw new Error(`invalid bbox {x: ${V}, y: ${q}, width: ${G}, height: ${J}}`);
        this.x0 = V, this.y0 = q, this.x1 = V + G, this.y1 = q + J;
      } else {
        let V, q, G, J;
        if ("width" in _)
          if ("left" in _)
            V = _.left, q = V + _.width;
          else if ("right" in _)
            q = _.right, V = q - _.width;
          else {
            const Z = _.width / 2;
            V = _.hcenter - Z, q = _.hcenter + Z;
          }
        else
          V = _.left, q = _.right;
        if ("height" in _)
          if ("top" in _)
            G = _.top, J = G + _.height;
          else if ("bottom" in _)
            J = _.bottom, G = J - _.height;
          else {
            const Z = _.height / 2;
            G = _.vcenter - Z, J = _.vcenter + Z;
          }
        else
          G = _.top, J = _.bottom;
        if (V > q || G > J)
          if (H)
            V > q && (V = q), G > J && (G = J);
          else
            throw new Error(`invalid bbox {left: ${V}, top: ${G}, right: ${q}, bottom: ${J}}`);
        this.x0 = V, this.y0 = G, this.x1 = q, this.y1 = J;
      }
    }
    static from_lrtb({ left: _, right: H, top: V, bottom: q }) {
      return new mf({
        x0: Math.min(_, H),
        y0: Math.min(V, q),
        x1: Math.max(_, H),
        y1: Math.max(V, q)
      });
    }
    clone() {
      return new mf(this);
    }
    equals(_) {
      return this.x0 == _.x0 && this.y0 == _.y0 && this.x1 == _.x1 && this.y1 == _.y1;
    }
    [equals](_, H) {
      return H.eq(this.x0, _.x0) && H.eq(this.y0, _.y0) && H.eq(this.x1, _.x1) && H.eq(this.y1, _.y1);
    }
    toString() {
      return `BBox({left: ${this.left}, top: ${this.top}, width: ${this.width}, height: ${this.height}})`;
    }
    get is_valid() {
      const { x0: _, x1: H, y0: V, y1: q } = this;
      return isFinite(_ + H + V + q);
    }
    get is_empty() {
      const { x0: _, x1: H, y0: V, y1: q } = this;
      return _ == 0 && H == 0 && V == 0 && q == 0;
    }
    get left() {
      return this.x0;
    }
    get top() {
      return this.y0;
    }
    get right() {
      return this.x1;
    }
    get bottom() {
      return this.y1;
    }
    get p0() {
      return { x: this.x0, y: this.y0 };
    }
    get p1() {
      return { x: this.x1, y: this.y1 };
    }
    get x() {
      return this.x0;
    }
    get y() {
      return this.y0;
    }
    get width() {
      return this.x1 - this.x0;
    }
    get height() {
      return this.y1 - this.y0;
    }
    get size() {
      return { width: this.width, height: this.height };
    }
    get rect() {
      const { x0: _, y0: H, x1: V, y1: q } = this;
      return {
        p0: { x: _, y: H },
        p1: { x: V, y: H },
        p2: { x: V, y: q },
        p3: { x: _, y: q }
      };
    }
    get box() {
      const { x: _, y: H, width: V, height: q } = this;
      return { x: _, y: H, width: V, height: q };
    }
    get lrtb() {
      const { left: _, right: H, top: V, bottom: q } = this;
      return { left: _, right: H, top: V, bottom: q };
    }
    get x_range() {
      return { start: this.x0, end: this.x1 };
    }
    get y_range() {
      return { start: this.y0, end: this.y1 };
    }
    get h_range() {
      return this.x_range;
    }
    get v_range() {
      return this.y_range;
    }
    get ranges() {
      return [this.x_range, this.y_range];
    }
    get aspect() {
      return this.width / this.height;
    }
    get hcenter() {
      return (this.left + this.right) / 2;
    }
    get vcenter() {
      return (this.top + this.bottom) / 2;
    }
    get area() {
      return this.width * this.height;
    }
    resolve(_) {
      switch (_) {
        case "top_left":
          return { x: this.left, y: this.top };
        case "top_center":
          return { x: this.hcenter, y: this.top };
        case "top_right":
          return { x: this.right, y: this.top };
        case "center_left":
          return { x: this.left, y: this.vcenter };
        case "center_center":
          return { x: this.hcenter, y: this.vcenter };
        case "center_right":
          return { x: this.right, y: this.vcenter };
        case "bottom_left":
          return { x: this.left, y: this.bottom };
        case "bottom_center":
          return { x: this.hcenter, y: this.bottom };
        case "bottom_right":
          return { x: this.right, y: this.bottom };
        case "center":
          return { x: this.hcenter, y: this.vcenter };
        case "top":
          return this.top;
        case "left":
          return this.left;
        case "right":
          return this.right;
        case "bottom":
          return this.bottom;
        case "width":
          return this.width;
        case "height":
          return this.height;
        default:
          return { x: NaN, y: NaN };
      }
    }
    round() {
      return new mf({
        x0: round$6(this.x0),
        x1: round$6(this.x1),
        y0: round$6(this.y0),
        y1: round$6(this.y1)
      });
    }
    relative() {
      const { width: _, height: H } = this;
      return new mf({ x: 0, y: 0, width: _, height: H });
    }
    translate(_, H) {
      const { x: V, y: q, width: G, height: J } = this;
      return new mf({ x: _ + V, y: H + q, width: G, height: J });
    }
    scale(_) {
      return new mf({
        x0: this.x0 * _,
        x1: this.x1 * _,
        y0: this.y0 * _,
        y1: this.y1 * _
      });
    }
    relativize(_, H) {
      return [_ - this.x, H - this.y];
    }
    contains(_, H) {
      return this.x0 <= _ && _ <= this.x1 && this.y0 <= H && H <= this.y1;
    }
    clip(_, H) {
      return _ < this.x0 ? _ = this.x0 : _ > this.x1 && (_ = this.x1), H < this.y0 ? H = this.y0 : H > this.y1 && (H = this.y1), [_, H];
    }
    grow_by(_) {
      return new mf({
        left: this.left - _,
        right: this.right + _,
        top: this.top - _,
        bottom: this.bottom + _
      });
    }
    shrink_by(_) {
      return new mf({
        left: this.left + _,
        right: this.right - _,
        top: this.top + _,
        bottom: this.bottom - _
      }, !0);
    }
    union(_) {
      return new mf({
        x0: min$a(this.x0, _.x0),
        y0: min$a(this.y0, _.y0),
        x1: max$h(this.x1, _.x1),
        y1: max$h(this.y1, _.y1)
      });
    }
    intersection(_) {
      return this.intersects(_) ? new mf({
        x0: max$h(this.x0, _.x0),
        y0: max$h(this.y0, _.y0),
        x1: min$a(this.x1, _.x1),
        y1: min$a(this.y1, _.y1)
      }) : null;
    }
    intersects(_) {
      return !(_.x1 < this.x0 || _.x0 > this.x1 || _.y1 < this.y0 || _.y0 > this.y1);
    }
    get x_screen() {
      const _ = this;
      return this._x_screen ?? (this._x_screen = {
        compute(H) {
          return _.left + H;
        },
        invert(H) {
          return H - _.left;
        },
        v_compute(H) {
          const { left: V } = _;
          return new ScreenArray(map$4(H, (q) => V + q));
        },
        v_invert(H) {
          const { left: V } = _;
          return map$4(H, (q) => q - V);
        },
        get source_range() {
          return _.x_range;
        },
        get target_range() {
          return _.x_range;
        }
      });
    }
    get y_screen() {
      const _ = this;
      return this._y_screen ?? (this._y_screen = {
        compute(H) {
          return _.top + H;
        },
        invert(H) {
          return H - _.top;
        },
        v_compute(H) {
          const { top: V } = _;
          return new ScreenArray(map$4(H, (q) => V + q));
        },
        v_invert(H) {
          const { top: V } = _;
          return map$4(H, (q) => q - V);
        },
        get source_range() {
          return _.y_range;
        },
        get target_range() {
          return _.y_range;
        }
      });
    }
    get x_view() {
      const _ = this;
      return this._x_view ?? (this._x_view = {
        compute(H) {
          return _.left + H;
        },
        invert(H) {
          return H - _.left;
        },
        v_compute(H) {
          const { left: V } = _;
          return new ScreenArray(map$4(H, (q) => V + q));
        },
        v_invert(H) {
          const { left: V } = _;
          return map$4(H, (q) => q - V);
        },
        get source_range() {
          return _.x_range;
        },
        get target_range() {
          return _.x_range;
        }
      });
    }
    get y_view() {
      const _ = this;
      return this._y_view ?? (this._y_view = {
        compute(H) {
          return _.bottom - H;
        },
        invert(H) {
          return _.bottom - H;
        },
        v_compute(H) {
          const { bottom: V } = _;
          return new ScreenArray(map$4(H, (q) => V - q));
        },
        v_invert(H) {
          const { bottom: V } = _;
          return map$4(H, (q) => V - q);
        },
        get source_range() {
          return _.y_range;
        },
        get target_range() {
          return { start: _.bottom, end: _.top };
        }
      });
    }
    get xview() {
      return this.x_view;
    }
    get yview() {
      return this.y_view;
    }
  }, mt(mf, "__name__", "BBox"), mf);
  const _style_decl = document.createElement("div").style;
  function _css_name(O) {
    if (O.startsWith("--"))
      return O;
    const _ = O.replaceAll(/_/g, "-").replaceAll(/[A-Z]/g, (q) => `-${q.toLowerCase()}`);
    if (_ in _style_decl)
      return _;
    const H = `-webkit-${_}`;
    if (H in _style_decl)
      return H;
    const V = `-moz-${_}`;
    return V in _style_decl ? V : (logger$1.warn(`unknown CSS property '${O}'`), null);
  }
  function* _iter_styles(O) {
    if (isPlainObject(O) || O instanceof Map)
      for (const [_, H] of entries$2(O)) {
        const V = _css_name(_);
        V != null && (yield [V, H]);
      }
    else
      for (const _ of O.own_properties())
        if (_.dirty) {
          const H = _css_name(_.attr);
          H != null && (yield [H, _.get_value()]);
        }
  }
  function apply_styles(O, _) {
    for (const [H, V] of _iter_styles(_))
      isString$2(V) ? O.setProperty(H, V) : O.removeProperty(H);
  }
  function compose_stylesheet(O) {
    const _ = [];
    for (const [H, V] of entries$2(O)) {
      _.push(`${H} {`);
      for (const [q, G] of _iter_styles(V))
        isString$2(G) && G.length != 0 && _.push(`  ${q}: ${G};`);
      _.push("}");
    }
    return _.join(`
`);
  }
  const _createElement = (O) => (_ = {}, ...H) => {
    const V = document.createElement(O);
    isPlainObject(_) || (H = [_, ...H], _ = {});
    for (let [G, J] of entries$2(_))
      if (!(J == null || isBoolean$4(J) && !J)) {
        if (G === "class" && (isString$2(J) && (J = J.split(/\s+/)), isArray$3(J))) {
          for (const Z of J)
            Z != null && V.classList.add(Z);
          continue;
        }
        if (G === "style" && isPlainObject(J)) {
          for (const [Z, te] of entries$2(J))
            V.style[Z] = te;
          continue;
        }
        if (G === "data" && isPlainObject(J)) {
          for (const [Z, te] of entries$2(J))
            V.dataset[Z] = te;
          continue;
        }
        V.setAttribute(G, J);
      }
    function q(G) {
      if (isString$2(G))
        V.appendChild(document.createTextNode(G));
      else if (G instanceof Node)
        V.appendChild(G);
      else if (G instanceof NodeList || G instanceof HTMLCollection)
        for (const J of G)
          V.appendChild(J);
      else if (G != null && G !== !1)
        throw new Error(`expected a DOM element, string, false or null, got ${JSON.stringify(G)}`);
    }
    for (const G of H)
      if (isArray$3(G))
        for (const J of G)
          q(J);
      else
        q(G);
    return V;
  };
  function createElement$1(O, _, ...H) {
    return _createElement(O)(_, ...H);
  }
  const div = _createElement("div"), span$1 = _createElement("span"), canvas = _createElement("canvas"), link$1 = _createElement("link"), style$1 = _createElement("style"), a$2 = _createElement("a"), p$1 = _createElement("p"), i$2 = _createElement("i"), pre = _createElement("pre"), button = _createElement("button"), label$2 = _createElement("label"), legend = _createElement("legend"), fieldset = _createElement("fieldset"), input$1 = _createElement("input"), select$1 = _createElement("select"), option = _createElement("option"), optgroup = _createElement("optgroup"), textarea = _createElement("textarea");
  function text$4(O) {
    return document.createTextNode(O);
  }
  function nbsp() {
    return text$4(" ");
  }
  function append$2(O, ..._) {
    for (const H of _)
      O.appendChild(H);
  }
  function remove$2(O) {
    const _ = O.parentNode;
    _ != null && _.removeChild(O);
  }
  function replaceWith(O, _) {
    const H = O.parentNode;
    H != null && H.replaceChild(_, O);
  }
  function prepend(O, ..._) {
    const H = O.firstChild;
    for (const V of _)
      O.insertBefore(V, H);
  }
  function empty$1(O, _ = !1) {
    let H;
    for (; (H = O.firstChild) != null; )
      O.removeChild(H);
    if (_ && O instanceof Element)
      for (const V of O.attributes)
        O.removeAttributeNode(V);
  }
  function contains$5(O, _) {
    let H = _;
    for (; H.parentNode != null; ) {
      const V = H.parentNode;
      if (V == O)
        return !0;
      V instanceof ShadowRoot ? H = V.host : H = V;
    }
    return !1;
  }
  function display(O, _ = !0) {
    O.style.display = _ ? "" : "none";
  }
  function undisplay(O) {
    O.style.display = "none";
  }
  function show(O) {
    O.style.visibility = "";
  }
  function hide(O) {
    O.style.visibility = "hidden";
  }
  function offset_bbox(O) {
    const { top: _, left: H, width: V, height: q } = O.getBoundingClientRect();
    return new BBox$1({
      left: H + scrollX - document.documentElement.clientLeft,
      top: _ + scrollY - document.documentElement.clientTop,
      width: V,
      height: q
    });
  }
  function bounding_box(O) {
    const { x: _, y: H, width: V, height: q } = O.getBoundingClientRect();
    return new BBox$1({ x: _, y: H, width: V, height: q });
  }
  function box_size(O) {
    const { width: _, height: H } = O.getBoundingClientRect();
    return { width: _, height: H };
  }
  function position$1(O, _, H) {
    const { style: V } = O;
    V.left = `${_.x}px`, V.top = `${_.y}px`, V.width = `${_.width}px`, V.height = `${_.height}px`, V.margin = "";
  }
  class ClassList {
    constructor(_) {
      mt(this, "class_list");
      this.class_list = _;
    }
    get values() {
      const _ = [];
      for (let H = 0; H < this.class_list.length; H++) {
        const V = this.class_list.item(H);
        V != null && _.push(V);
      }
      return _;
    }
    has(_) {
      return this.class_list.contains(_);
    }
    add(..._) {
      for (const H of _)
        this.class_list.add(H);
      return this;
    }
    remove(..._) {
      for (const H of _)
        isArray$3(H) ? H.forEach((V) => this.class_list.remove(V)) : this.class_list.remove(H);
      return this;
    }
    clear() {
      for (const _ of this.values)
        this.class_list.remove(_);
      return this;
    }
    toggle(_, H) {
      return H ?? !this.has(_) ? this.add(_) : this.remove(_), this;
    }
  }
  mt(ClassList, "__name__", "ClassList");
  function toggle_attribute(O, _, H) {
    H == null && (H = !O.hasAttribute(_)), H ? O.setAttribute(_, "true") : O.removeAttribute(_);
  }
  var MouseButton;
  (function(O) {
    O[O.None = 0] = "None", O[O.Primary = 1] = "Primary", O[O.Secondary = 2] = "Secondary", O[O.Auxiliary = 4] = "Auxiliary", O[O.Left = 1] = "Left", O[O.Right = 2] = "Right", O[O.Middle = 4] = "Middle";
  })(MouseButton || (MouseButton = {}));
  var H_;
  let StyleSheet$1 = (H_ = class {
    constructor() {
      mt(this, "el");
    }
    install(_) {
      _.append(this.el);
    }
    uninstall() {
      this.el.remove();
    }
  }, mt(H_, "__name__", "StyleSheet"), H_);
  var z_;
  let InlineStyleSheet$1 = (z_ = class extends StyleSheet$1 {
    constructor(H) {
      super();
      mt(this, "el", style$1({ type: "text/css" }));
      isString$2(H) ? this._update(H) : H != null && this._update(compose_stylesheet(H));
    }
    get css() {
      return this.el.textContent ?? "";
    }
    _update(H) {
      this.el.textContent = H;
    }
    clear() {
      this.replace("");
    }
    _to_css(H, V) {
      return V == null ? H : compose_stylesheet({ [H]: V });
    }
    replace(H, V) {
      this._update(this._to_css(H, V));
    }
    prepend(H, V) {
      this._update(`${this._to_css(H, V)}
${this.css}`);
    }
    append(H, V) {
      this._update(`${this.css}
${this._to_css(H, V)}`);
    }
    remove() {
      remove$2(this.el);
    }
  }, mt(z_, "__name__", "InlineStyleSheet"), z_);
  var V_;
  let GlobalInlineStyleSheet$1 = (V_ = class extends InlineStyleSheet$1 {
    install() {
      this.el.isConnected || document.head.appendChild(this.el);
    }
  }, mt(V_, "__name__", "GlobalInlineStyleSheet"), V_);
  var U_;
  let ImportedStyleSheet$1 = (U_ = class extends StyleSheet$1 {
    constructor(H) {
      super();
      mt(this, "el");
      this.el = link$1({ rel: "stylesheet", href: H });
    }
    replace(H) {
      this.el.href = H;
    }
    remove() {
      remove$2(this.el);
    }
  }, mt(U_, "__name__", "ImportedStyleSheet"), U_);
  var q_;
  let GlobalImportedStyleSheet$1 = (q_ = class extends ImportedStyleSheet$1 {
    install() {
      this.el.isConnected || document.head.appendChild(this.el);
    }
  }, mt(q_, "__name__", "GlobalImportedStyleSheet"), q_);
  async function dom_ready() {
    if (document.readyState == "loading")
      return new Promise((O, _) => {
        document.addEventListener("DOMContentLoaded", () => O(), { once: !0 });
      });
  }
  function px$1(O) {
    return isNumber$3(O) ? `${O}px` : O;
  }
  const base_css = ":host{--base-font:var(--bokeh-base-font, Helvetica, Arial, sans-serif);--mono-font:var(--bokeh-mono-font, monospace);--font-size:var(--bokeh-font-size, 12px);--line-height:calc(20 / 14);--line-height-computed:calc(var(--font-size) * var(--line-height));--border-radius:4px;--padding-vertical:6px;--padding-horizontal:12px;--bokeh-top-level:1000;}:host{box-sizing:border-box;font-family:var(--base-font);font-size:var(--font-size);line-height:var(--line-height);}*,*:before,*:after{box-sizing:inherit;font-family:inherit;}pre,code{font-family:var(--mono-font);margin:0;}";
  class DOMView extends View$2 {
    constructor() {
      super(...arguments);
      mt(this, "el");
      mt(this, "shadow_el");
    }
    get children_el() {
      return this.shadow_el ?? this.el;
    }
    initialize() {
      super.initialize(), this.el = this._createElement();
    }
    remove() {
      remove$2(this.el), super.remove();
    }
    stylesheets() {
      return [];
    }
    css_classes() {
      return [];
    }
    render_to(H) {
      this.render(), H == null || H.appendChild(this.el), this.after_render();
    }
    after_render() {
      this.reposition();
    }
    finish() {
      this._has_finished = !0, this.notify_finished();
    }
    _createElement() {
      return createElement$1(this.constructor.tag_name, { class: this.css_classes() });
    }
    reposition(H) {
    }
  }
  mt(DOMView, "__name__", "DOMView"), mt(DOMView, "tag_name", "div");
  var j_;
  let DOMElementView$1 = (j_ = class extends DOMView {
    constructor() {
      super(...arguments);
      mt(this, "class_list");
    }
    initialize() {
      super.initialize(), this.class_list = new ClassList(this.el.classList);
    }
  }, mt(j_, "__name__", "DOMElementView"), j_);
  class DOMComponentView extends DOMElementView$1 {
    constructor() {
      super(...arguments);
      mt(this, "_applied_stylesheets", []);
      mt(this, "_applied_css_classes", []);
    }
    initialize() {
      super.initialize(), this.shadow_el = this.el.attachShadow({ mode: "open" });
    }
    stylesheets() {
      return [...super.stylesheets(), base_css];
    }
    empty() {
      empty$1(this.shadow_el), this.class_list.clear(), this._applied_css_classes = [], this._applied_stylesheets = [];
    }
    render() {
      this.empty(), this._update_stylesheets(), this._update_css_classes(), this._update_css_variables();
    }
    reposition(H) {
      this._update_css_variables();
    }
    *_stylesheets() {
      for (const H of this.stylesheets())
        yield isString$2(H) ? new InlineStyleSheet$1(H) : H;
    }
    *_css_classes() {
      yield `bk-${this.model.type.replace(/\./g, "-")}`, yield* this.css_classes();
    }
    *_css_variables() {
    }
    _apply_stylesheets(H) {
      this._applied_stylesheets.push(...H), H.forEach((V) => V.install(this.shadow_el));
    }
    _apply_css_classes(H) {
      this._applied_css_classes.push(...H), this.class_list.add(...H);
    }
    _update_stylesheets() {
      this._applied_stylesheets.forEach((H) => H.uninstall()), this._applied_stylesheets = [], this._apply_stylesheets([...this._stylesheets()]);
    }
    _update_css_classes() {
      this.class_list.remove(this._applied_css_classes), this._applied_css_classes = [], this._apply_css_classes([...this._css_classes()]);
    }
    _update_css_variables() {
      for (const [H, V] of this._css_variables()) {
        const q = H.startsWith("--") ? H : `--${H}`;
        this.el.style.setProperty(q, V);
      }
    }
  }
  mt(DOMComponentView, "__name__", "DOMComponentView");
  const index$4 = new Proxy(new ViewManager(), {
    get(O, _) {
      if (isString$2(_)) {
        const H = O.get_by_id(_);
        if (H != null)
          return H;
      }
      return Reflect.get(O, _);
    },
    has(O, _) {
      return isString$2(_) && O.get_by_id(_) != null ? !0 : Reflect.has(O, _);
    },
    ownKeys(O) {
      return O.roots.map((_) => _.model.id);
    },
    getOwnPropertyDescriptor(O, _) {
      if (isString$2(_)) {
        const H = O.get_by_id(_);
        if (H != null)
          return { configurable: !0, enumerable: !0, writable: !1, value: H };
      }
      return Reflect.getOwnPropertyDescriptor(O, _);
    }
  });
  async function add_document_standalone(O, _, H = [], V = !1) {
    assert$2(O.views_manager == null);
    const q = new ViewManager([], index$4);
    O.views_manager = q;
    async function G(te) {
      const ne = await q.build_view(te);
      if (ne instanceof DOMView) {
        const ee = O.roots().indexOf(te), ae = H[ee] ?? _;
        ne.render_to(ae);
      }
      index$4.add(ne);
    }
    async function J(te) {
      te.default_view != null ? await G(te) : O.notify_idle(te);
    }
    function Z(te) {
      const ne = q.get(te);
      ne == null || ne.remove();
    }
    for (const te of O.roots())
      await J(te);
    return V && (window.document.title = O.title()), O.on_change((te) => {
      te instanceof RootAddedEvent ? J(te.model) : te instanceof RootRemovedEvent ? Z(te.model) : V && te instanceof TitleChangedEvent && (window.document.title = te.title);
    }), q;
  }
  var l2;
  let Message$2 = (l2 = class {
    constructor(_, H, V) {
      mt(this, "header");
      mt(this, "metadata");
      mt(this, "content");
      mt(this, "_buffers", /* @__PURE__ */ new Map());
      this.header = _, this.metadata = H, this.content = V;
    }
    get buffers() {
      return this._buffers;
    }
    static assemble(_, H, V) {
      const q = JSON.parse(_), G = JSON.parse(H), J = JSON.parse(V);
      return new l2(q, G, J);
    }
    assemble_buffer(_, H) {
      const V = this.header.num_buffers ?? 0;
      if (V <= this._buffers.size)
        throw new Error(`too many buffers received, expecting ${V}`);
      const { id: q } = JSON.parse(_);
      this._buffers.set(q, H);
    }
    static create(_, H, V) {
      const q = l2.create_header(_);
      return new l2(q, H, V);
    }
    static create_header(_) {
      return {
        msgid: unique_id(),
        msgtype: _
      };
    }
    complete() {
      const { num_buffers: _ } = this.header;
      return _ == null || this._buffers.size == _;
    }
    send(_) {
      assert$2(this.header.num_buffers == null);
      const H = [], V = JSON.stringify(this.content, (Z, te) => {
        if (te instanceof Buffer$3) {
          const ne = { id: `${H.length}` };
          return H.push([ne, te.buffer]), ne;
        } else
          return te;
      }), q = H.length;
      q > 0 && (this.header.num_buffers = q);
      const G = JSON.stringify(this.header), J = JSON.stringify(this.metadata);
      _.send(G), _.send(J), _.send(V);
      for (const [Z, te] of H)
        _.send(JSON.stringify(Z)), _.send(te);
    }
    msgid() {
      return this.header.msgid;
    }
    msgtype() {
      return this.header.msgtype;
    }
    reqid() {
      return this.header.reqid;
    }
    // return the reason we should close on bad protocol, if there is one
    problem() {
      return "msgid" in this.header ? "msgtype" in this.header ? null : "No msgtype in header" : "No msgid in header";
    }
  }, mt(l2, "__name__", "Message"), l2);
  class Receiver {
    constructor() {
      mt(this, "message", null);
      mt(this, "_partial", null);
      mt(this, "_fragments", []);
      mt(this, "_buf_header", null);
      mt(this, "_current_consumer", this._HEADER);
    }
    consume(_) {
      this._current_consumer(_);
    }
    _HEADER(_) {
      this._assume_text(_), this.message = null, this._partial = null, this._fragments = [_], this._buf_header = null, this._current_consumer = this._METADATA;
    }
    _METADATA(_) {
      this._assume_text(_), this._fragments.push(_), this._current_consumer = this._CONTENT;
    }
    _CONTENT(_) {
      this._assume_text(_), this._fragments.push(_);
      const [H, V, q] = this._fragments;
      assert$2(H != null && V != null && q != null), this._partial = Message$2.assemble(H, V, q), this._check_complete();
    }
    _BUFFER_HEADER(_) {
      this._assume_text(_), this._buf_header = _, this._current_consumer = this._BUFFER_PAYLOAD;
    }
    _BUFFER_PAYLOAD(_) {
      this._assume_binary(_), assert$2(this._partial != null && this._buf_header != null), this._partial.assemble_buffer(this._buf_header, _), this._check_complete();
    }
    _assume_text(_) {
      if (!isString$2(_))
        throw new Error("Expected text fragment but received binary fragment");
    }
    _assume_binary(_) {
      if (!(_ instanceof ArrayBuffer))
        throw new Error("Expected binary fragment but received text fragment");
    }
    _check_complete() {
      this._partial.complete() ? (this.message = this._partial, this._current_consumer = this._HEADER) : this._current_consumer = this._BUFFER_HEADER;
    }
  }
  mt(Receiver, "__name__", "Receiver");
  class ClientSession {
    constructor(_, H) {
      mt(this, "_connection");
      mt(this, "document");
      mt(this, "_document_listener", (_) => {
        this._document_changed(_);
      });
      this._connection = _, this.document = H, this.document.on_change(this._document_listener, !0);
    }
    // XXX: this is only needed in tests
    get id() {
      return this._connection.id;
    }
    handle(_) {
      const H = _.msgtype();
      switch (H) {
        case "PATCH-DOC": {
          this._handle_patch(_);
          break;
        }
        case "OK": {
          this._handle_ok(_);
          break;
        }
        case "ERROR": {
          this._handle_error(_);
          break;
        }
        default:
          logger$1.debug(`Doing nothing with message '${H}'`);
      }
    }
    notify_connection_lost() {
      this.document.event_manager.send_event(new ConnectionLost());
    }
    close() {
      this._connection.close();
    }
    /*protected*/
    _connection_closed() {
      this.document.remove_on_change(this._document_listener);
    }
    // Sends a request to the server for info about the server, such as its Bokeh
    // version. Returns a promise, the value of the promise is a free-form dictionary
    // of server details.
    async request_server_info() {
      const _ = Message$2.create("SERVER-INFO-REQ", {}, {});
      return (await this._connection.send_with_reply(_)).content;
    }
    // Sends some request to the server (no guarantee about which one) and returns
    // a promise which is completed when the server replies. The purpose of this
    // is that if you wait for the promise to be completed, you know the server
    // has processed the request. This is useful when writing tests because once
    // the server has processed this request it should also have processed any
    // events or requests you sent previously, which means you can check for the
    // results of that processing without a race condition. (This assumes the
    // server processes events in sequence, which it mostly has to semantically,
    // since reordering events might change the final state.)
    async force_roundtrip() {
      await this.request_server_info();
    }
    _document_changed(_) {
      const H = (_ instanceof DocumentEventBatch ? _.sync ? _.events : [] : [_]).filter((J) => J.sync);
      if (H.length == 0)
        return;
      const V = this.document.create_json_patch(H), q = Message$2.create("PATCH-DOC", {}, V);
      this._connection.send(q);
    }
    _handle_patch(_) {
      this.document.apply_json_patch(_.content, _.buffers);
    }
    _handle_ok(_) {
      logger$1.trace(`Unhandled OK reply to ${_.reqid()}`);
    }
    _handle_error(_) {
      logger$1.error(`Unhandled ERROR reply to ${_.reqid()}: ${_.content.text}`);
    }
  }
  mt(ClientSession, "__name__", "ClientSession");
  const DEFAULT_SERVER_WEBSOCKET_URL = "ws://localhost:5006/ws", DEFAULT_TOKEN = "eyJzZXNzaW9uX2lkIjogImRlZmF1bHQifQ";
  let _connection_count = 0;
  function parse_token(O) {
    let _ = O.split(".")[0];
    const H = _.length % 4;
    return H != 0 && (_ = _ + "=".repeat(4 - H)), JSON.parse(atob(_.replace(/_/g, "/").replace(/-/g, "+")));
  }
  class ClientConnection {
    constructor(_ = DEFAULT_SERVER_WEBSOCKET_URL, H = DEFAULT_TOKEN, V = null) {
      mt(this, "url");
      mt(this, "token");
      mt(this, "args_string");
      mt(this, "_number", _connection_count++);
      mt(this, "socket", null);
      mt(this, "session", null);
      mt(this, "closed_permanently", !1);
      mt(this, "id");
      mt(this, "_current_handler", null);
      mt(this, "_pending_replies", /* @__PURE__ */ new Map());
      mt(this, "_pending_messages", []);
      mt(this, "_receiver", new Receiver());
      this.url = _, this.token = H, this.args_string = V, this.id = parse_token(H).session_id.split(".")[0], logger$1.debug(`Creating websocket ${this._number} to '${this.url}' session '${this.id}'`);
    }
    async connect() {
      if (this.closed_permanently)
        throw new Error("Cannot connect() a closed ClientConnection");
      if (this.socket != null)
        throw new Error("Already connected");
      this._current_handler = null, this._pending_replies.clear(), this._pending_messages = [];
      try {
        let _ = `${this.url}`;
        return this.args_string != null && this.args_string.length > 0 && (_ += `?${this.args_string}`), this.socket = new WebSocket(_, ["bokeh", this.token]), new Promise((H, V) => {
          this.socket.binaryType = "arraybuffer", this.socket.onopen = () => this._on_open(H, V), this.socket.onmessage = (q) => this._on_message(q), this.socket.onclose = (q) => this._on_close(q, V), this.socket.onerror = () => this._on_error(V);
        });
      } catch (_) {
        throw logger$1.error(`websocket creation failed to url: ${this.url}`), logger$1.error(` - ${_}`), _;
      }
    }
    close() {
      this.closed_permanently || (logger$1.debug(`Permanently closing websocket connection ${this._number}`), this.closed_permanently = !0, this.socket != null && this.socket.close(1e3, `close method called on ClientConnection ${this._number}`), this.session._connection_closed());
    }
    _schedule_reconnect(_) {
      setTimeout(() => {
        var V;
        this.closed_permanently || (logger$1.info(`Websocket connection ${this._number} disconnected, will not attempt to reconnect`), (V = this.session) == null || V.notify_connection_lost());
      }, _);
    }
    send(_) {
      this.socket != null ? _.send(this.socket) : logger$1.error("not connected so cannot send", _);
    }
    async send_with_reply(_) {
      const H = await new Promise((V, q) => {
        this._pending_replies.set(_.msgid(), { resolve: V, reject: q }), this.send(_);
      });
      if (H.msgtype() == "ERROR")
        throw new Error(`Error reply ${H.content.text}`);
      return H;
    }
    async _pull_doc_json() {
      const _ = Message$2.create("PULL-DOC-REQ", {}, {}), H = await this.send_with_reply(_);
      if (!("doc" in H.content))
        throw new Error("No 'doc' field in PULL-DOC-REPLY");
      return H.content.doc;
    }
    async _repull_session_doc(_, H) {
      logger$1.debug(this.session != null ? "Repulling session" : "Pulling session for first time");
      try {
        const V = await this._pull_doc_json();
        if (this.session == null)
          if (this.closed_permanently)
            logger$1.debug("Got new document after connection was already closed"), H(new Error("The connection has been closed"));
          else {
            const q = [], G = Document.from_json(V, q);
            this.session = new ClientSession(this, G);
            for (const J of q)
              G._trigger_on_change(J);
            for (const J of this._pending_messages)
              this.session.handle(J);
            this._pending_messages = [], logger$1.debug("Created a new session from new pulled doc"), _(this.session);
          }
        else
          this.session.document.replace_with_json(V), logger$1.debug("Updated existing session with new pulled doc");
      } catch (V) {
        console.trace(V), logger$1.error(`Failed to repull session ${V}`), H(V instanceof Error ? V : `${V}`);
      }
    }
    _on_open(_, H) {
      logger$1.info(`Websocket connection ${this._number} is now open`), this._current_handler = (V) => {
        this._awaiting_ack_handler(V, _, H);
      };
    }
    _on_message(_) {
      this._current_handler == null && logger$1.error("Got a message with no current handler set");
      try {
        this._receiver.consume(_.data);
      } catch (V) {
        this._close_bad_protocol(`${V}`);
      }
      const H = this._receiver.message;
      if (H != null) {
        const V = H.problem();
        V != null && this._close_bad_protocol(V), this._current_handler(H);
      }
    }
    _on_close(_, H) {
      logger$1.info(`Lost websocket ${this._number} connection, ${_.code} (${_.reason})`), this.socket = null, this._pending_replies.forEach((V) => V.reject("Disconnected")), this._pending_replies.clear(), this.closed_permanently || this._schedule_reconnect(2e3), H(new Error(`Lost websocket connection, ${_.code} (${_.reason})`));
    }
    _on_error(_) {
      logger$1.debug(`Websocket error on socket ${this._number}`);
      const H = "Could not open websocket";
      logger$1.error(`Failed to connect to Bokeh server: ${H}`), _(new Error(H));
    }
    _close_bad_protocol(_) {
      logger$1.error(`Closing connection: ${_}`), this.socket != null && this.socket.close(1002, _);
    }
    _awaiting_ack_handler(_, H, V) {
      _.msgtype() === "ACK" ? (this._current_handler = (q) => this._steady_state_handler(q), this._repull_session_doc(H, V)) : this._close_bad_protocol("First message was not an ACK");
    }
    _steady_state_handler(_) {
      const H = _.reqid(), V = this._pending_replies.get(H);
      V != null ? (this._pending_replies.delete(H), V.resolve(_)) : this.session != null ? this.session.handle(_) : _.msgtype() != "PATCH-DOC" && this._pending_messages.push(_);
    }
  }
  mt(ClientConnection, "__name__", "ClientConnection");
  function pull_session(O, _, H) {
    return new ClientConnection(O, _, H).connect();
  }
  function _get_ws_url(O, _) {
    let H = "ws:";
    window.location.protocol == "https:" && (H = "wss:");
    let V;
    return V = window.location, O != null ? O == "/" && (O = "") : O = V.pathname.replace(/\/+$/, ""), `${H}//${V.host}${O}/ws`;
  }
  const _sessions = /* @__PURE__ */ new Map();
  function _get_session(O, _, H) {
    const V = parse_token(_).session_id;
    _sessions.has(O) || _sessions.set(O, /* @__PURE__ */ new Map());
    const q = _sessions.get(O);
    return q.has(V) || q.set(V, pull_session(O, _, H)), q.get(V);
  }
  async function add_document_from_session(O, _, H, V = [], q = !1) {
    const G = window.location.search.substring(1);
    let J;
    try {
      J = await _get_session(O, _, G);
    } catch (Z) {
      const te = parse_token(_).session_id;
      throw logger$1.error(`Failed to load Bokeh session ${te}: ${Z}`), Z;
    }
    return add_document_standalone(J.document, H, V, q);
  }
  function _get_element(O) {
    let _ = isString$2(O) ? document.getElementById(O) : O;
    if (_ == null)
      throw new Error(`Error rendering Bokeh model: could not find ${isString$2(O) ? `#${O}` : O} HTML tag`);
    if (!contains$5(document.body, _))
      throw new Error(`Error rendering Bokeh model: element ${isString$2(O) ? `#${O}` : O} must be under <body>`);
    if (_ instanceof HTMLElement && _.tagName == "SCRIPT") {
      const H = div();
      replaceWith(_, H), _ = H;
    }
    return _;
  }
  function _resolve_element(O) {
    const { elementid: _ } = O;
    return _ != null ? _get_element(_) : document.body;
  }
  function _resolve_root_elements(O) {
    const _ = [];
    if (O.root_ids != null && O.roots != null)
      for (const H of O.root_ids)
        _.push(_get_element(O.roots[H]));
    return _;
  }
  async function embed_item(O, _) {
    const H = {}, V = uuid4();
    H[V] = O.doc, _ == null && (_ = O.target_id);
    const G = { roots: { [O.root_id]: _ }, root_ids: [O.root_id], docid: V };
    await defer();
    const [J] = await _embed_items(H, [G]);
    return J;
  }
  async function _embed_items(O, _, H, V) {
    isString$2(O) && (O = JSON.parse(unescape$1(O)));
    const q = {};
    for (const [J, Z] of entries$2(O))
      q[J] = Document.from_json(Z);
    const G = [];
    for (const J of _) {
      const Z = _resolve_element(J), te = _resolve_root_elements(J);
      if (J.docid != null)
        G.push(await add_document_standalone(q[J.docid], Z, te, J.use_for_title));
      else if (J.token != null) {
        const ne = _get_ws_url(H);
        logger$1.debug(`embed: computed ws url: ${ne}`);
        try {
          G.push(await add_document_from_session(ne, J.token, Z, te, J.use_for_title)), console.log("Bokeh items were rendered successfully");
        } catch (ee) {
          if (settings.dev)
            throw ee;
          console.error("Error rendering Bokeh items:", ee);
        }
      } else
        throw new Error("Error rendering Bokeh items: either 'docid' or 'token' was expected.");
    }
    return G;
  }
  class VisualProperties {
    constructor(_, H = "") {
      mt(this, "obj");
      mt(this, "prefix");
      mt(this, "_props");
      this.obj = _, this.prefix = H;
      const V = this;
      this._props = [];
      for (const q of this.attrs) {
        const G = _.model.properties[H + q];
        G.change.connect(() => this.update()), V[q] = G, this._props.push(G);
      }
    }
    *[Symbol.iterator]() {
      yield* this._props;
    }
    update() {
    }
  }
  mt(VisualProperties, "__name__", "VisualProperties");
  class VisualUniforms {
    constructor(_, H = "") {
      mt(this, "obj");
      mt(this, "prefix");
      this.obj = _, this.prefix = H;
      for (const V of this.attrs)
        Object.defineProperty(this, V, {
          get() {
            return _[H + V];
          }
        });
    }
    *[Symbol.iterator]() {
      for (const _ of this.attrs)
        yield this.obj.model.properties[this.prefix + _];
    }
    update() {
    }
  }
  mt(VisualUniforms, "__name__", "VisualUniforms");
  const Line$4 = {
    line_color: [Nullable(Color$1), "black"],
    line_alpha: [Alpha, 1],
    line_width: [Float$1, 1],
    line_join: [LineJoin, "bevel"],
    line_cap: [LineCap, "butt"],
    line_dash: [Or$1(LineDash, Array$1(Float$1)), []],
    line_dash_offset: [Float$1, 0]
  }, Fill$2 = {
    fill_color: [Nullable(Color$1), "gray"],
    fill_alpha: [Alpha, 1]
  }, Image$3 = {
    global_alpha: [Alpha, 1]
  }, Hatch$1 = {
    hatch_color: [Nullable(Color$1), "black"],
    hatch_alpha: [Alpha, 1],
    hatch_scale: [Float$1, 12],
    hatch_pattern: [Nullable(Or$1(HatchPatternType, Str)), null],
    hatch_weight: [Float$1, 1],
    hatch_extra: [Dict(AnyRef()), {}]
    // XXX: recursive imports
  }, Text$4 = {
    text_color: [Nullable(Color$1), "#444444"],
    text_outline_color: [Nullable(Color$1), null],
    text_alpha: [Alpha, 1],
    text_font: [Font, "helvetica"],
    text_font_size: [FontSize, "16px"],
    text_font_style: [FontStyle, "normal"],
    text_align: [TextAlign, "left"],
    text_baseline: [TextBaseline, "bottom"],
    text_line_height: [Float$1, 1.2]
  }, LineScalar$1 = {
    line_color: [ColorScalar, "black"],
    line_alpha: [NumberScalar, 1],
    line_width: [NumberScalar, 1],
    line_join: [LineJoinScalar, "bevel"],
    line_cap: [LineCapScalar, "butt"],
    line_dash: [LineDashScalar, []],
    line_dash_offset: [NumberScalar, 0]
  }, FillScalar$1 = {
    fill_color: [ColorScalar, "gray"],
    fill_alpha: [NumberScalar, 1]
  }, ImageScalar$1 = {
    global_alpha: [NumberScalar, 1]
  }, HatchScalar$1 = {
    hatch_color: [ColorScalar, "black"],
    hatch_alpha: [NumberScalar, 1],
    hatch_scale: [NumberScalar, 12],
    hatch_pattern: [NullStringScalar, null],
    hatch_weight: [NumberScalar, 1],
    hatch_extra: [DictScalar, {}]
  }, TextScalar$1 = {
    text_color: [ColorScalar, "#444444"],
    text_outline_color: [ColorScalar, null],
    text_alpha: [NumberScalar, 1],
    text_font: [FontScalar, "helvetica"],
    text_font_size: [FontSizeScalar, "16px"],
    text_font_style: [FontStyleScalar, "normal"],
    text_align: [TextAlignScalar, "left"],
    text_baseline: [TextBaselineScalar, "bottom"],
    text_line_height: [NumberScalar, 1.2]
  }, LineVector$1 = {
    line_color: [ColorSpec, "black"],
    line_alpha: [NumberSpec, 1],
    line_width: [NumberSpec, 1],
    line_join: [LineJoinSpec, "bevel"],
    line_cap: [LineCapSpec, "butt"],
    line_dash: [LineDashSpec, []],
    line_dash_offset: [NumberSpec, 0]
  }, FillVector$1 = {
    fill_color: [ColorSpec, "gray"],
    fill_alpha: [NumberSpec, 1]
  }, ImageVector$1 = {
    global_alpha: [NumberSpec, 1]
  }, HatchVector$1 = {
    hatch_color: [ColorSpec, "black"],
    hatch_alpha: [NumberSpec, 1],
    hatch_scale: [NumberSpec, 12],
    hatch_pattern: [NullStringSpec, null],
    hatch_weight: [NumberSpec, 1],
    hatch_extra: [DictScalar, {}]
  }, TextVector$1 = {
    text_color: [ColorSpec, "#444444"],
    text_outline_color: [ColorSpec, null],
    text_alpha: [NumberSpec, 1],
    text_font: [FontSpec, "helvetica"],
    text_font_size: [FontSizeSpec, "16px"],
    text_font_style: [FontStyleSpec, "normal"],
    text_align: [TextAlignSpec, "left"],
    text_baseline: [TextBaselineSpec, "bottom"],
    text_line_height: [NumberSpec, 1.2]
  };
  function attrs_of(O, _, H, V = !1) {
    const q = {};
    for (const G of keys$5(H)) {
      const J = `${_}${G}`, Z = O[J], te = isString$2(V) ? `${V}${G}` : V ? J : G;
      q[te] = Z;
    }
    return q;
  }
  function resolve_line_dash(O) {
    if (isArray$3(O))
      return O;
    switch (O) {
      case "solid":
        return [];
      case "dashed":
        return [6];
      case "dotted":
        return [2, 4];
      case "dotdash":
        return [2, 4, 6, 4];
      case "dashdot":
        return [6, 4, 2, 4];
      default:
        return O.split(" ").map(Number).filter(isInteger$2);
    }
  }
  var G_;
  let Line$3 = (G_ = class extends VisualProperties {
    get doit() {
      const _ = this.line_color.get_value(), H = this.line_alpha.get_value(), V = this.line_width.get_value();
      return !(_ == null || H == 0 || V == 0);
    }
    apply(_) {
      const { doit: H } = this;
      return H && (this.set_value(_), _.stroke()), H;
    }
    values() {
      return {
        color: this.line_color.get_value(),
        alpha: this.line_alpha.get_value(),
        width: this.line_width.get_value(),
        join: this.line_join.get_value(),
        cap: this.line_cap.get_value(),
        dash: this.line_dash.get_value(),
        offset: this.line_dash_offset.get_value()
      };
    }
    set_value(_) {
      const H = this.line_color.get_value(), V = this.line_alpha.get_value();
      _.strokeStyle = color2css(H, V), _.lineWidth = this.line_width.get_value(), _.lineJoin = this.line_join.get_value(), _.lineCap = this.line_cap.get_value(), _.setLineDash(resolve_line_dash(this.line_dash.get_value())), _.lineDashOffset = this.line_dash_offset.get_value();
    }
  }, mt(G_, "__name__", "Line"), G_);
  class LineScalar extends VisualUniforms {
    get doit() {
      const _ = this.line_color.value, H = this.line_alpha.value, V = this.line_width.value;
      return !(_ == 0 || H == 0 || V == 0);
    }
    apply(_) {
      const { doit: H } = this;
      return H && (this.set_value(_), _.stroke()), H;
    }
    values() {
      return {
        color: this.line_color.value,
        alpha: this.line_alpha.value,
        width: this.line_width.value,
        join: this.line_join.value,
        cap: this.line_cap.value,
        dash: this.line_dash.value,
        offset: this.line_dash_offset.value
      };
    }
    set_value(_) {
      const H = this.line_color.value, V = this.line_alpha.value;
      _.strokeStyle = color2css(H, V), _.lineWidth = this.line_width.value, _.lineJoin = this.line_join.value, _.lineCap = this.line_cap.value, _.setLineDash(resolve_line_dash(this.line_dash.value)), _.lineDashOffset = this.line_dash_offset.value;
    }
  }
  mt(LineScalar, "__name__", "LineScalar");
  class LineVector extends VisualUniforms {
    get doit() {
      const { line_color: _ } = this;
      if (_.is_Scalar() && _.value == 0)
        return !1;
      const { line_alpha: H } = this;
      if (H.is_Scalar() && H.value == 0)
        return !1;
      const { line_width: V } = this;
      return !(V.is_Scalar() && V.value == 0);
    }
    v_doit(_) {
      return !(this.line_color.get(_) == 0 || this.line_alpha.get(_) == 0 || this.line_width.get(_) == 0);
    }
    apply(_, H) {
      const V = this.v_doit(H);
      return V && (this.set_vectorize(_, H), _.stroke()), V;
    }
    values(_) {
      return {
        color: this.line_color.get(_),
        alpha: this.line_alpha.get(_),
        width: this.line_width.get(_),
        join: this.line_join.get(_),
        cap: this.line_cap.get(_),
        dash: this.line_dash.get(_),
        offset: this.line_dash_offset.get(_)
      };
    }
    set_vectorize(_, H) {
      const V = this.line_color.get(H), q = this.line_alpha.get(H), G = this.line_width.get(H), J = this.line_join.get(H), Z = this.line_cap.get(H), te = this.line_dash.get(H), ne = this.line_dash_offset.get(H);
      _.strokeStyle = color2css(V, q), _.lineWidth = G, _.lineJoin = J, _.lineCap = Z, _.setLineDash(resolve_line_dash(te)), _.lineDashOffset = ne;
    }
  }
  mt(LineVector, "__name__", "LineVector");
  Line$3.prototype.type = "line";
  Line$3.prototype.attrs = Object.keys(Line$4);
  LineScalar.prototype.type = "line";
  LineScalar.prototype.attrs = Object.keys(LineScalar$1);
  LineVector.prototype.type = "line";
  LineVector.prototype.attrs = Object.keys(LineVector$1);
  var W_;
  let Fill$1 = (W_ = class extends VisualProperties {
    constructor() {
      super(...arguments);
      mt(this, "Values");
    }
    get doit() {
      const H = this.fill_color.get_value(), V = this.fill_alpha.get_value();
      return !(H == null || V == 0);
    }
    apply(H, V) {
      const { doit: q } = this;
      return q && (this.set_value(H), H.fill(V)), q;
    }
    values() {
      return {
        color: this.fill_color.get_value(),
        alpha: this.fill_alpha.get_value()
      };
    }
    set_value(H) {
      const V = this.fill_color.get_value(), q = this.fill_alpha.get_value();
      H.fillStyle = color2css(V, q);
    }
  }, mt(W_, "__name__", "Fill"), W_);
  class FillScalar extends VisualUniforms {
    constructor() {
      super(...arguments);
      mt(this, "Values");
    }
    get doit() {
      const H = this.fill_color.value, V = this.fill_alpha.value;
      return !(H == 0 || V == 0);
    }
    apply(H, V) {
      const { doit: q } = this;
      return q && (this.set_value(H), H.fill(V)), q;
    }
    values() {
      return {
        color: this.fill_color.value,
        alpha: this.fill_alpha.value
      };
    }
    set_value(H) {
      const V = this.fill_color.value, q = this.fill_alpha.value;
      H.fillStyle = color2css(V, q);
    }
  }
  mt(FillScalar, "__name__", "FillScalar");
  class FillVector extends VisualUniforms {
    constructor() {
      super(...arguments);
      mt(this, "Values");
    }
    get doit() {
      const { fill_color: H } = this;
      if (H.is_Scalar() && H.value == 0)
        return !1;
      const { fill_alpha: V } = this;
      return !(V.is_Scalar() && V.value == 0);
    }
    v_doit(H) {
      return !(this.fill_color.get(H) == 0 || this.fill_alpha.get(H) == 0);
    }
    apply(H, V, q) {
      const G = this.v_doit(V);
      return G && (this.set_vectorize(H, V), H.fill(q)), G;
    }
    values(H) {
      return {
        color: this.fill_color.get(H),
        alpha: this.fill_alpha.get(H)
      };
    }
    set_vectorize(H, V) {
      const q = this.fill_color.get(V), G = this.fill_alpha.get(V);
      H.fillStyle = color2css(q, G);
    }
  }
  mt(FillVector, "__name__", "FillVector");
  Fill$1.prototype.type = "fill";
  Fill$1.prototype.attrs = Object.keys(Fill$2);
  FillScalar.prototype.type = "fill";
  FillScalar.prototype.attrs = Object.keys(FillScalar$1);
  FillVector.prototype.type = "fill";
  FillVector.prototype.attrs = Object.keys(FillVector$1);
  const _font_cache = /* @__PURE__ */ new Map();
  function load_font(O, _) {
    const H = _font_cache.get(O);
    if (H == null) {
      const q = new WeakSet([_]);
      _font_cache.set(O, q);
    } else if (!H.has(_))
      H.add(_);
    else
      return;
    const { fonts: V } = document;
    V.check(O) || V.load(O).then(() => _.request_render());
  }
  var Y_;
  let Text$3 = (Y_ = class extends VisualProperties {
    get doit() {
      const _ = this.text_color.get_value(), H = this.text_alpha.get_value();
      return !(_ == null || H == 0);
    }
    update() {
      if (!this.doit)
        return;
      const _ = this.font_value();
      load_font(_, this.obj);
    }
    values() {
      return {
        color: this.text_color.get_value(),
        outline_color: this.text_outline_color.get_value(),
        alpha: this.text_alpha.get_value(),
        font: this.text_font.get_value(),
        font_size: this.text_font_size.get_value(),
        font_style: this.text_font_style.get_value(),
        align: this.text_align.get_value(),
        baseline: this.text_baseline.get_value(),
        line_height: this.text_line_height.get_value()
      };
    }
    set_value(_) {
      const H = this.text_color.get_value(), V = this.text_outline_color.get_value(), q = this.text_alpha.get_value();
      _.fillStyle = color2css(H, q), _.strokeStyle = color2css(V, q), _.font = this.font_value(), _.textAlign = this.text_align.get_value(), _.textBaseline = this.text_baseline.get_value();
    }
    font_value() {
      const _ = this.text_font_style.get_value(), H = this.text_font_size.get_value(), V = this.text_font.get_value();
      return `${_} ${H} ${V}`;
    }
  }, mt(Y_, "__name__", "Text"), Y_);
  class TextScalar extends VisualUniforms {
    get doit() {
      const _ = this.text_color.value, H = this.text_alpha.value;
      return !(_ == 0 || H == 0);
    }
    update() {
      if (!this.doit)
        return;
      const _ = this.font_value();
      load_font(_, this.obj);
    }
    values() {
      return {
        color: this.text_color.value,
        outline_color: this.text_outline_color.value,
        alpha: this.text_alpha.value,
        font: this.text_font.value,
        font_size: this.text_font_size.value,
        font_style: this.text_font_style.value,
        align: this.text_align.value,
        baseline: this.text_baseline.value,
        line_height: this.text_line_height.value
      };
    }
    set_value(_) {
      const H = this.text_color.value, V = this.text_alpha.value, q = this.text_outline_color.value, G = this.font_value(), J = this.text_align.value, Z = this.text_baseline.value;
      _.fillStyle = color2css(H, V), _.strokeStyle = color2css(q, V), _.font = G, _.textAlign = J, _.textBaseline = Z;
    }
    font_value() {
      const _ = this.text_font_style.value, H = this.text_font_size.value, V = this.text_font.value;
      return `${_} ${H} ${V}`;
    }
  }
  mt(TextScalar, "__name__", "TextScalar");
  class TextVector extends VisualUniforms {
    _assert_font(_) {
      const H = this.font_value(_);
      load_font(H, this.obj);
    }
    values(_) {
      return this._assert_font(_), {
        color: this.text_color.get(_),
        outline_color: this.text_outline_color.get(_),
        alpha: this.text_alpha.get(_),
        font: this.text_font.get(_),
        font_size: this.text_font_size.get(_),
        font_style: this.text_font_style.get(_),
        align: this.text_align.get(_),
        baseline: this.text_baseline.get(_),
        line_height: this.text_line_height.get(_)
      };
    }
    get doit() {
      const { text_color: _ } = this;
      if (_.is_Scalar() && _.value == 0)
        return !1;
      const { text_alpha: H } = this;
      return !(H.is_Scalar() && H.value == 0);
    }
    v_doit(_) {
      return !(this.text_color.get(_) == 0 || this.text_alpha.get(_) == 0);
    }
    apply(_, H) {
      const V = this.v_doit(H);
      return V && this.set_vectorize(_, H), V;
    }
    set_vectorize(_, H) {
      this._assert_font(H);
      const V = this.text_color.get(H), q = this.text_outline_color.get(H), G = this.text_alpha.get(H), J = this.font_value(H), Z = this.text_align.get(H), te = this.text_baseline.get(H);
      _.fillStyle = color2css(V, G), _.strokeStyle = color2css(q, G), _.font = J, _.textAlign = Z, _.textBaseline = te;
    }
    font_value(_) {
      const H = this.text_font_style.get(_), V = this.text_font_size.get(_), q = this.text_font.get(_);
      return `${H} ${V} ${q}`;
    }
  }
  mt(TextVector, "__name__", "TextVector");
  Text$3.prototype.type = "text";
  Text$3.prototype.attrs = Object.keys(Text$4);
  TextScalar.prototype.type = "text";
  TextScalar.prototype.attrs = Object.keys(TextScalar$1);
  TextVector.prototype.type = "text";
  TextVector.prototype.attrs = Object.keys(TextVector$1);
  function _horz(O, _, H) {
    O.moveTo(0, H + 0.5), O.lineTo(_, H + 0.5), O.stroke();
  }
  function _vert(O, _, H) {
    O.moveTo(H + 0.5, 0), O.lineTo(H + 0.5, _), O.stroke();
  }
  function _x$2(O, _) {
    O.moveTo(0, _), O.lineTo(_, 0), O.stroke(), O.moveTo(0, 0), O.lineTo(_, _), O.stroke();
  }
  const hatch_aliases = {
    " ": "blank",
    ".": "dot",
    o: "ring",
    "-": "horizontal_line",
    "|": "vertical_line",
    "+": "cross",
    '"': "horizontal_dash",
    ":": "vertical_dash",
    "@": "spiral",
    "/": "right_diagonal_line",
    "\\": "left_diagonal_line",
    x: "diagonal_cross",
    ",": "right_diagonal_dash",
    "`": "left_diagonal_dash",
    v: "horizontal_wave",
    ">": "vertical_wave",
    "*": "criss_cross"
  };
  function get_pattern(O, _, H, V, q, G) {
    return O.resize(q, q), O.prepare(), create_hatch_canvas(O.ctx, _, H, V, q, G), O.canvas;
  }
  function create_hatch_canvas(O, _, H, V, q, G) {
    const J = q, Z = J / 2, te = Z / 2, ne = color2css(H, V);
    switch (O.strokeStyle = ne, O.fillStyle = ne, O.lineCap = "square", O.lineWidth = G, hatch_aliases[_] ?? _) {
      // we should not need these if code conditions on hatch.doit, but
      // include them here just for completeness
      case "blank":
        break;
      case "dot":
        O.arc(Z, Z, Z / 2, 0, 2 * Math.PI, !0), O.fill();
        break;
      case "ring":
        O.arc(Z, Z, Z / 2, 0, 2 * Math.PI, !0), O.stroke();
        break;
      case "horizontal_line":
        _horz(O, J, Z);
        break;
      case "vertical_line":
        _vert(O, J, Z);
        break;
      case "cross":
        _horz(O, J, Z), _vert(O, J, Z);
        break;
      case "horizontal_dash":
        _horz(O, Z, Z);
        break;
      case "vertical_dash":
        _vert(O, Z, Z);
        break;
      case "spiral": {
        const ee = J / 30;
        O.moveTo(Z, Z);
        for (let ae = 0; ae < 360; ae++) {
          const oe = 0.1 * ae, se = Z + ee * oe * Math.cos(oe), ce = Z + ee * oe * Math.sin(oe);
          O.lineTo(se, ce);
        }
        O.stroke();
        break;
      }
      case "right_diagonal_line":
        O.moveTo(-te + 0.5, J), O.lineTo(te + 0.5, 0), O.stroke(), O.moveTo(te + 0.5, J), O.lineTo(3 * te + 0.5, 0), O.stroke(), O.moveTo(3 * te + 0.5, J), O.lineTo(5 * te + 0.5, 0), O.stroke();
        break;
      case "left_diagonal_line":
        O.moveTo(te + 0.5, J), O.lineTo(-te + 0.5, 0), O.stroke(), O.moveTo(3 * te + 0.5, J), O.lineTo(te + 0.5, 0), O.stroke(), O.moveTo(5 * te + 0.5, J), O.lineTo(3 * te + 0.5, 0), O.stroke();
        break;
      case "diagonal_cross":
        _x$2(O, J);
        break;
      case "right_diagonal_dash":
        O.moveTo(te + 0.5, 3 * te + 0.5), O.lineTo(3 * te + 0.5, te + 0.5), O.stroke();
        break;
      case "left_diagonal_dash":
        O.moveTo(te + 0.5, te + 0.5), O.lineTo(3 * te + 0.5, 3 * te + 0.5), O.stroke();
        break;
      case "horizontal_wave":
        O.moveTo(0, te), O.lineTo(Z, 3 * te), O.lineTo(J, te), O.stroke();
        break;
      case "vertical_wave":
        O.moveTo(te, 0), O.lineTo(3 * te, Z), O.lineTo(te, J), O.stroke();
        break;
      case "criss_cross":
        _x$2(O, J), _horz(O, J, Z), _vert(O, J, Z);
        break;
      default:
        logger$1.warn(`unknown hatch pattern: ${_}`);
    }
  }
  class Hatch extends VisualProperties {
    constructor() {
      super(...arguments);
      mt(this, "_hatch_image");
      mt(this, "_update_iteration", 0);
    }
    update() {
      if (this._update_iteration++, this._hatch_image = null, !this.doit)
        return;
      const H = this.hatch_color.get_value(), V = this.hatch_alpha.get_value(), q = this.hatch_scale.get_value(), G = this.hatch_pattern.get_value(), J = this.hatch_weight.get_value(), Z = (ee) => {
        this._hatch_image = ee;
      }, ne = dict(this.hatch_extra.get_value()).get(G);
      if (ne != null) {
        const ee = ne.get_pattern(H, V, q, J);
        if (ee instanceof Promise) {
          const { _update_iteration: ae } = this;
          ee.then((oe) => {
            this._update_iteration == ae && (Z(oe), this.obj.request_render());
          });
        } else
          Z(ee);
      } else {
        const ee = this.obj.canvas.create_layer(), ae = get_pattern(ee, G, H, V, q, J);
        Z(ae);
      }
    }
    get doit() {
      const H = this.hatch_color.get_value(), V = this.hatch_alpha.get_value(), q = this.hatch_pattern.get_value();
      return !(H == null || V == 0 || q == " " || q == "blank" || q == null);
    }
    apply(H, V) {
      const { doit: q } = this;
      return q && (this.set_value(H), H.layer.undo_transform(() => H.fill(V))), q;
    }
    set_value(H) {
      const V = this.pattern(H);
      H.fillStyle = V ?? "transparent";
    }
    pattern(H) {
      const V = this._hatch_image;
      return V == null ? null : H.createPattern(V, this.repetition());
    }
    repetition() {
      const H = this.hatch_pattern.get_value(), q = dict(this.hatch_extra.get_value()).get(H);
      if (q == null)
        return "repeat";
      switch (q.repetition) {
        case "repeat":
          return "repeat";
        case "repeat_x":
          return "repeat-x";
        case "repeat_y":
          return "repeat-y";
        case "no_repeat":
          return "no-repeat";
      }
    }
  }
  mt(Hatch, "__name__", "Hatch");
  class HatchScalar extends VisualUniforms {
    constructor() {
      super(...arguments);
      mt(this, "_hatch_image");
      mt(this, "_static_doit", !1);
      mt(this, "_update_iteration", 0);
    }
    _compute_static_doit() {
      const H = this.hatch_color.value, V = this.hatch_alpha.value, q = this.hatch_pattern.value;
      return !(H == 0 || V == 0 || q == " " || q == "blank" || q == null);
    }
    update() {
      this._update_iteration++;
      const H = this.hatch_color.length;
      if (this._hatch_image = new UniformScalar(null, H), this._static_doit = this._compute_static_doit(), !this._static_doit)
        return;
      const V = this.hatch_color.value, q = this.hatch_alpha.value, G = this.hatch_scale.value, J = this.hatch_pattern.value, Z = this.hatch_weight.value, te = (ae) => {
        this._hatch_image = new UniformScalar(ae, H);
      }, ee = dict(this.hatch_extra.value).get(J);
      if (ee != null) {
        const ae = ee.get_pattern(V, q, G, Z);
        if (ae instanceof Promise) {
          const { _update_iteration: oe } = this;
          ae.then((se) => {
            this._update_iteration == oe && (te(se), this.obj.request_render());
          });
        } else
          te(ae);
      } else {
        const ae = this.obj.canvas.create_layer(), oe = get_pattern(ae, J, V, q, G, Z);
        te(oe);
      }
    }
    get doit() {
      return this._static_doit;
    }
    apply(H, V) {
      const { doit: q } = this;
      return q && (this.set_value(H), H.layer.undo_transform(() => H.fill(V))), q;
    }
    set_value(H) {
      H.fillStyle = this.pattern(H) ?? "transparent";
    }
    pattern(H) {
      const V = this._hatch_image.value;
      return V == null ? null : H.createPattern(V, this.repetition());
    }
    repetition() {
      const H = this.hatch_pattern.value;
      if (H != null) {
        const q = dict(this.hatch_extra.value).get(H);
        if (q != null)
          switch (q.repetition) {
            case "repeat":
              return "repeat";
            case "repeat_x":
              return "repeat-x";
            case "repeat_y":
              return "repeat-y";
            case "no_repeat":
              return "no-repeat";
          }
      }
      return "repeat";
    }
  }
  mt(HatchScalar, "__name__", "HatchScalar");
  class HatchVector extends VisualUniforms {
    constructor() {
      super(...arguments);
      mt(this, "_hatch_image");
      mt(this, "_static_doit", !1);
      mt(this, "_update_iteration", 0);
    }
    _compute_static_doit() {
      const { hatch_color: H } = this;
      if (H.is_Scalar() && H.value == 0)
        return !1;
      const { hatch_alpha: V } = this;
      if (V.is_Scalar() && V.value == 0)
        return !1;
      const { hatch_pattern: q } = this;
      if (q.is_Scalar()) {
        const G = q.value;
        if (G == " " || G == "blank" || G == null)
          return !1;
      }
      return !0;
    }
    update() {
      this._update_iteration++;
      const H = this.hatch_color.length;
      if (this._hatch_image = new UniformScalar(null, H), this._static_doit = this._compute_static_doit(), !this._static_doit)
        return;
      const V = (q, G, J, Z, te, ne) => {
        const ae = dict(this.hatch_extra.value).get(q);
        if (ae != null) {
          const oe = ae.get_pattern(G, J, Z, te);
          if (oe instanceof Promise) {
            const { _update_iteration: se } = this;
            oe.then((ce) => {
              this._update_iteration == se && (ne(ce), this.obj.request_render());
            });
          } else
            ne(oe);
        } else {
          const oe = this.obj.canvas.create_layer(), se = get_pattern(oe, q, G, J, Z, te);
          ne(se);
        }
      };
      if (this.hatch_color.is_Scalar() && this.hatch_alpha.is_Scalar() && this.hatch_scale.is_Scalar() && this.hatch_pattern.is_Scalar() && this.hatch_weight.is_Scalar()) {
        const q = this.hatch_color.value, G = this.hatch_alpha.value, J = this.hatch_scale.value, Z = this.hatch_pattern.value, te = this.hatch_weight.value;
        V(Z, q, G, J, te, (ne) => {
          this._hatch_image = new UniformScalar(ne, H);
        });
      } else {
        const q = new Array(H);
        q.fill(null), this._hatch_image = new UniformVector(q);
        for (let G = 0; G < H; G++) {
          const J = this.hatch_color.get(G), Z = this.hatch_alpha.get(G), te = this.hatch_scale.get(G), ne = this.hatch_pattern.get(G), ee = this.hatch_weight.get(G);
          V(ne, J, Z, te, ee, (ae) => {
            q[G] = ae;
          });
        }
      }
    }
    get doit() {
      return this._static_doit;
    }
    v_doit(H) {
      if (!this.doit || this.hatch_color.get(H) == 0 || this.hatch_alpha.get(H) == 0)
        return !1;
      const V = this.hatch_pattern.get(H);
      return !(V == " " || V == "blank" || V == null);
    }
    apply(H, V, q) {
      const G = this.v_doit(V);
      return G && (this.set_vectorize(H, V), H.layer.undo_transform(() => H.fill(q))), G;
    }
    set_vectorize(H, V) {
      H.fillStyle = this.pattern(H, V) ?? "transparent";
    }
    pattern(H, V) {
      const q = this._hatch_image.get(V);
      return q == null ? null : H.createPattern(q, this.repetition(V));
    }
    repetition(H) {
      const V = this.hatch_pattern.get(H);
      if (V != null) {
        const G = dict(this.hatch_extra.value).get(V);
        if (G != null)
          switch (G.repetition) {
            case "repeat":
              return "repeat";
            case "repeat_x":
              return "repeat-x";
            case "repeat_y":
              return "repeat-y";
            case "no_repeat":
              return "no-repeat";
          }
      }
      return "repeat";
    }
  }
  mt(HatchVector, "__name__", "HatchVector");
  Hatch.prototype.type = "hatch";
  Hatch.prototype.attrs = Object.keys(Hatch$1);
  HatchScalar.prototype.type = "hatch";
  HatchScalar.prototype.attrs = Object.keys(HatchScalar$1);
  HatchVector.prototype.type = "hatch";
  HatchVector.prototype.attrs = Object.keys(HatchVector$1);
  var X_;
  let Image$2 = (X_ = class extends VisualProperties {
    get doit() {
      return this.global_alpha.get_value() != 0;
    }
    apply(_) {
      const { doit: H } = this;
      return H && this.set_value(_), H;
    }
    values() {
      return {
        global_alpha: this.global_alpha.get_value()
      };
    }
    set_value(_) {
      const H = this.global_alpha.get_value();
      _.globalAlpha = H;
    }
  }, mt(X_, "__name__", "Image"), X_);
  class ImageScalar extends VisualUniforms {
    get doit() {
      return this.global_alpha.value != 0;
    }
    apply(_) {
      const { doit: H } = this;
      return H && this.set_value(_), H;
    }
    values() {
      return {
        global_alpha: this.global_alpha.value
      };
    }
    set_value(_) {
      const H = this.global_alpha.value;
      _.globalAlpha = H;
    }
  }
  mt(ImageScalar, "__name__", "ImageScalar");
  class ImageVector extends VisualUniforms {
    get doit() {
      const { global_alpha: _ } = this;
      return !(_.is_Scalar() && _.value == 0);
    }
    v_doit(_) {
      return this.global_alpha.get(_) != 0;
    }
    apply(_, H) {
      const V = this.v_doit(H);
      return V && this.set_vectorize(_, H), V;
    }
    values(_) {
      return {
        alpha: this.global_alpha.get(_)
      };
    }
    set_vectorize(_, H) {
      const V = this.global_alpha.get(H);
      _.globalAlpha = V;
    }
  }
  mt(ImageVector, "__name__", "ImageVector");
  Image$2.prototype.type = "image";
  Image$2.prototype.attrs = Object.keys(Image$3);
  ImageScalar.prototype.type = "image";
  ImageScalar.prototype.attrs = Object.keys(ImageScalar$1);
  ImageVector.prototype.type = "image";
  ImageVector.prototype.attrs = Object.keys(ImageVector$1);
  class Visuals {
    constructor(_) {
      mt(this, "_visuals", []);
      for (const [H, V] of _.model._mixins) {
        const q = (() => {
          switch (V) {
            case Line$4:
              return new Line$3(_, H);
            case LineScalar$1:
              return new LineScalar(_, H);
            case LineVector$1:
              return new LineVector(_, H);
            case Fill$2:
              return new Fill$1(_, H);
            case FillScalar$1:
              return new FillScalar(_, H);
            case FillVector$1:
              return new FillVector(_, H);
            case Text$4:
              return new Text$3(_, H);
            case TextScalar$1:
              return new TextScalar(_, H);
            case TextVector$1:
              return new TextVector(_, H);
            case Hatch$1:
              return new Hatch(_, H);
            case HatchScalar$1:
              return new HatchScalar(_, H);
            case HatchVector$1:
              return new HatchVector(_, H);
            case Image$3:
              return new Image$2(_, H);
            case ImageScalar$1:
              return new ImageScalar(_, H);
            case ImageVector$1:
              return new ImageVector(_, H);
            default:
              throw new Error("unknown visual");
          }
        })();
        q instanceof VisualProperties && q.update(), this._visuals.push(q), Object.defineProperty(this, H + q.type, {
          get() {
            return q;
          },
          configurable: !1,
          enumerable: !0
        });
      }
    }
    *[Symbol.iterator]() {
      yield* this._visuals;
    }
  }
  mt(Visuals, "__name__", "Visuals");
  var Z_;
  let Transform$1 = (Z_ = class extends Model$1 {
    constructor(_) {
      super(_);
    }
  }, mt(Z_, "__name__", "Transform"), Z_);
  const Bounds$1 = Nullable(Or$1(Tuple(Nullable(Float$1), Nullable(Float$1)), Auto));
  var Q0;
  let Range$2 = (Q0 = class extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "_computed_bounds");
      mt(this, "have_updated_interactively", !1);
      /** internal */
      mt(this, "plots", /* @__PURE__ */ new Set());
    }
    get computed_bounds() {
      return this._computed_bounds;
    }
    get is_reversed() {
      return this.start > this.end;
    }
    get is_valid() {
      return isFinite(this.min) && isFinite(this.max);
    }
    get span() {
      return Math.abs(this.end - this.start);
    }
  }, mt(Q0, "__name__", "Range"), Q0.define(({ Float: H, Nullable: V }) => ({
    bounds: [Bounds$1, null, {
      on_update(q, G) {
        const [J, Z] = q == "auto" || q == null ? [null, null] : q;
        G._computed_bounds = [J ?? -1 / 0, Z ?? 1 / 0];
      }
    }],
    min_interval: [V(H), null],
    max_interval: [V(H), null]
  })), Q0);
  const dg = class dg extends Range$2 {
    constructor(_) {
      super(_);
    }
  };
  mt(dg, "__name__", "NumericalRange"), dg.define(({ Float: _ }) => ({
    start: [_, unset, {
      convert(H, V) {
        const [q, G] = V.computed_bounds;
        return clamp$1(H, q, G);
      }
    }],
    end: [_, unset, {
      convert(H, V) {
        const [q, G] = V.computed_bounds;
        return clamp$1(H, q, G);
      }
    }]
  }));
  let NumericalRange = dg;
  const J2 = class J2 extends NumericalRange {
    constructor(H) {
      super(H);
      mt(this, "_reset_start");
      mt(this, "_reset_end");
    }
    _set_auto_bounds() {
      if (this.bounds == "auto") {
        const H = Math.min(this._reset_start, this._reset_end), V = Math.max(this._reset_start, this._reset_end);
        this._computed_bounds = [H, V];
      }
    }
    initialize() {
      super.initialize(), this._set_auto_bounds();
    }
    get min() {
      return Math.min(this.start, this.end);
    }
    get max() {
      return Math.max(this.start, this.end);
    }
    reset() {
      this._set_auto_bounds();
      const { _reset_start: H, _reset_end: V } = this;
      this.start != H || this.end != V ? this.setv({ start: H, end: V }) : this.change.emit();
    }
    map(H) {
      return new J2({ start: H(this.start), end: H(this.end) });
    }
    widen(H) {
      let { start: V, end: q } = this;
      return this.is_reversed ? (V += H, q -= H) : (V -= H, q += H), new J2({ start: V, end: q });
    }
  };
  mt(J2, "__name__", "Range1d"), J2.define(({ Float: H, Nullable: V }) => ({
    reset_start: [V(H), null, {
      on_update(q, G) {
        G._reset_start = q ?? G.start;
      }
    }],
    reset_end: [V(H), null, {
      on_update(q, G) {
        G._reset_end = q ?? G.end;
      }
    }]
  })), J2.override({
    start: 0,
    end: 1
  });
  let Range1d = J2;
  var C0;
  let Scale$2 = (C0 = class extends Transform$1 {
    constructor(_) {
      super(_);
    }
    compute(_) {
      return this.s_compute(_);
    }
    v_compute(_) {
      const H = new ScreenArray(_.length), { s_compute: V } = this;
      for (let q = 0; q < _.length; q++)
        H[q] = V(_[q]);
      return H;
    }
    invert(_) {
      return this.s_invert(_);
    }
    v_invert(_) {
      const H = new Float64Array(_.length), { s_invert: V } = this;
      for (let q = 0; q < _.length; q++)
        H[q] = V(_[q]);
      return H;
    }
    r_compute(_, H) {
      const { s_compute: V } = this;
      return this.target_range.is_reversed ? [V(H), V(_)] : [V(_), V(H)];
    }
    r_invert(_, H) {
      const { s_invert: V } = this;
      return this.target_range.is_reversed ? [V(H), V(_)] : [V(_), V(H)];
    }
  }, mt(C0, "__name__", "Scale"), C0.internal(({ Ref: _ }) => ({
    source_range: [_(Range$2)],
    target_range: [_(Range1d)]
  })), C0);
  class ContinuousScale extends Scale$2 {
    constructor(_) {
      super(_);
    }
  }
  mt(ContinuousScale, "__name__", "ContinuousScale");
  class LinearScale extends ContinuousScale {
    constructor(_) {
      super(_);
    }
    get s_compute() {
      const [_, H] = this._linear_compute_state();
      return (V) => _ * V + H;
    }
    get s_invert() {
      const [_, H] = this._linear_compute_state();
      return (V) => (V - H) / _;
    }
    /*protected*/
    _linear_compute_state() {
      const _ = this.source_range.start, H = this.source_range.end, V = this.target_range.start, G = (this.target_range.end - V) / (H - _), J = -(G * _) + V;
      return [G, J];
    }
  }
  mt(LinearScale, "__name__", "LinearScale");
  class LogScale extends ContinuousScale {
    constructor(_) {
      super(_);
    }
    get s_compute() {
      const [_, H, V, q] = this._compute_state();
      return (G) => {
        if (V == 0)
          return 0;
        {
          const J = (Math.log(G) - q) / V;
          return isFinite(J) ? J * _ + H : NaN;
        }
      };
    }
    get s_invert() {
      const [_, H, V, q] = this._compute_state();
      return (G) => {
        const J = (G - H) / _;
        return Math.exp(V * J + q);
      };
    }
    _get_safe_factor(_, H) {
      let V = _ < 0 ? 0 : _, q = H < 0 ? 0 : H;
      if (V == q)
        if (V == 0)
          [V, q] = [1, 10];
        else {
          const G = Math.log10(V);
          V = 10 ** Math.floor(G), Math.ceil(G) != Math.floor(G) ? q = 10 ** Math.ceil(G) : q = 10 ** (Math.ceil(G) + 1);
        }
      return [V, q];
    }
    /*protected*/
    _compute_state() {
      const _ = this.source_range.start, H = this.source_range.end, V = this.target_range.start, G = this.target_range.end - V, [J, Z] = this._get_safe_factor(_, H);
      let te, ne;
      return J == 0 ? (te = Math.log(Z), ne = 0) : (te = Math.log(Z / J), ne = Math.log(J)), [G, V, te, ne];
    }
  }
  mt(LogScale, "__name__", "LogScale");
  const { _linear_compute_state } = LinearScale.prototype;
  class CategoricalScale extends Scale$2 {
    constructor(H) {
      super(H);
      mt(this, "source_range");
    }
    get s_compute() {
      const [H, V] = _linear_compute_state.call(this), q = this.source_range;
      return (G) => H * q.synthetic(G) + V;
    }
    get s_invert() {
      const [H, V] = _linear_compute_state.call(this);
      return (q) => (q - V) / H;
    }
  }
  mt(CategoricalScale, "__name__", "CategoricalScale");
  const fg = class fg extends Scale$2 {
    constructor(_) {
      super(_);
    }
    get s_compute() {
      const _ = this.source_scale.s_compute, H = this.target_scale.s_compute;
      return (V) => H(_(V));
    }
    get s_invert() {
      const _ = this.source_scale.s_invert, H = this.target_scale.s_invert;
      return (V) => _(H(V));
    }
    compute(_) {
      return this.s_compute(_);
    }
    v_compute(_) {
      const { s_compute: H } = this;
      return map$4(_, H);
    }
    invert(_) {
      return this.s_invert(_);
    }
    v_invert(_) {
      const { s_invert: H } = this;
      return map$4(_, H);
    }
  };
  mt(fg, "__name__", "CompositeScale"), fg.define(({ Ref: _ }) => ({
    source_scale: [_(Scale$2)],
    target_scale: [_(Scale$2)]
  }));
  let CompositeScale = fg;
  const k3 = class k3 extends NumericalRange {
    constructor(_) {
      super(_);
    }
  };
  mt(k3, "__name__", "DataRange"), k3.define(({ List: _, AnyRef: H, Or: V, Auto: q }) => ({
    renderers: [V(_(H()), q), []]
  })), k3.override({
    start: NaN,
    end: NaN
  });
  let DataRange = k3;
  function compute_renderers(O, _) {
    return O == "auto" ? _ : O ?? [];
  }
  const auto_ranged = Symbol("auto_ranged");
  function is_auto_ranged(O) {
    return auto_ranged in O;
  }
  const M3 = class M3 extends DataRange {
    constructor(H) {
      super(H);
      mt(this, "_initial_start");
      mt(this, "_initial_end");
      mt(this, "_initial_range_padding");
      mt(this, "_initial_range_padding_units");
      mt(this, "_initial_follow");
      mt(this, "_initial_follow_interval");
      mt(this, "_initial_default_span");
      mt(this, "_plot_bounds");
      mt(this, "have_updated_interactively", !1);
    }
    initialize() {
      super.initialize(), this._initial_start = isNaN(this.start) ? null : this.start, this._initial_end = isNaN(this.end) ? null : this.end, this._initial_range_padding = this.range_padding, this._initial_range_padding_units = this.range_padding_units, this._initial_follow = this.follow, this._initial_follow_interval = this.follow_interval, this._initial_default_span = this.default_span, this._plot_bounds = /* @__PURE__ */ new Map();
    }
    get min() {
      return Math.min(this.start, this.end);
    }
    get max() {
      return Math.max(this.start, this.end);
    }
    computed_renderers() {
      const { renderers: H } = this, V = flat_map(this.plots, (q) => q.auto_ranged_renderers.map((G) => G.model));
      return compute_renderers(H.length == 0 ? "auto" : H, [...V]);
    }
    /*protected*/
    _compute_plot_bounds(H, V) {
      let q = empty$2();
      for (const G of H) {
        const J = V.get(G);
        J != null && (G.visible || !this.only_visible) && (q = union$1(q, J));
      }
      return q;
    }
    adjust_bounds_for_aspect(H, V) {
      const q = empty$2();
      let G = H.x1 - H.x0;
      G <= 0 && (G = 1);
      let J = H.y1 - H.y0;
      J <= 0 && (J = 1);
      const Z = 0.5 * (H.x1 + H.x0), te = 0.5 * (H.y1 + H.y0);
      return G < V * J ? G = V * J : J = G / V, q.x1 = Z + 0.5 * G, q.x0 = Z - 0.5 * G, q.y1 = te + 0.5 * J, q.y0 = te - 0.5 * J, q;
    }
    /*protected*/
    _compute_min_max(H, V) {
      let q = empty$2();
      for (const [Z, te] of H)
        Z.model.visible && (q = union$1(q, te));
      let G, J;
      return V == 0 ? [G, J] = [q.x0, q.x1] : [G, J] = [q.y0, q.y1], [G, J];
    }
    /*protected*/
    _compute_range(H, V) {
      const { range_padding: q } = this, G = this.min_interval ?? 0, J = this.max_interval ?? 1 / 0;
      let Z, te;
      if (this._initial_start != null && (H = this._initial_start), this._initial_end != null && (V = this._initial_end), this.scale_hint == "log") {
        (isNaN(H) || !isFinite(H) || H <= 0) && (isNaN(V) || !isFinite(V) || V <= 0 ? H = 0.1 : H = V / 100, logger$1.warn(`could not determine minimum data value for log axis, DataRange1d using value ${H}`)), (isNaN(V) || !isFinite(V) || V <= 0) && (isNaN(H) || !isFinite(H) || H <= 0 ? V = 10 : V = H * 100, logger$1.warn(`could not determine maximum data value for log axis, DataRange1d using value ${V}`));
        let ae, oe;
        if (V == H)
          oe = this.default_span + 1e-3, ae = Math.log10(H);
        else {
          let se, ce;
          this.range_padding_units == "percent" ? (se = Math.log10(H), ce = Math.log10(V), oe = (ce - se) * (1 + q)) : (se = Math.log10(H - q), ce = Math.log10(V + q), oe = ce - se), ae = (se + ce) / 2;
        }
        oe = clamp$1(oe, G, J), Z = 10 ** (ae - oe / 2), te = 10 ** (ae + oe / 2);
      } else {
        let ae;
        V == H ? ae = this.default_span : this.range_padding_units == "percent" ? ae = (V - H) * (1 + q) : ae = V - H + 2 * q, ae = clamp$1(ae, G, J);
        const oe = (V + H) / 2;
        Z = oe - ae / 2, te = oe + ae / 2;
      }
      let ne = 1;
      this.flipped && ([Z, te] = [te, Z], ne = -1);
      const ee = this.follow_interval;
      return ee != null && Math.abs(Z - te) > ee && (this.follow == "start" ? te = Z + ne * ee : this.follow == "end" && (Z = te - ne * ee)), [Z, te];
    }
    update(H, V, q, G) {
      if (this.have_updated_interactively)
        return;
      const J = this.computed_renderers();
      let Z = this._compute_plot_bounds(J, H);
      G != null && (Z = this.adjust_bounds_for_aspect(Z, G)), this._plot_bounds.set(q, Z);
      const [te, ne] = this._compute_min_max(this._plot_bounds.entries(), V);
      let [ee, ae] = this._compute_range(te, ne);
      this._initial_start != null && (this.scale_hint == "log" ? this._initial_start > 0 && (ee = this._initial_start) : ee = this._initial_start), this._initial_end != null && (this.scale_hint == "log" ? this._initial_end > 0 && (ae = this._initial_end) : ae = this._initial_end);
      let oe = !1;
      this.bounds == "auto" && (this._computed_bounds = [ee, ae], oe = !0);
      const [se, ce] = [this.start, this.end];
      if (ee != se || ae != ce) {
        const le = {};
        ee != se && (le.start = ee), ae != ce && (le.end = ae), this.setv(le), oe = !1;
      }
      oe && this.change.emit();
    }
    reset() {
      this.have_updated_interactively = !1, this.setv({
        range_padding: this._initial_range_padding,
        range_padding_units: this._initial_range_padding_units,
        follow: this._initial_follow,
        follow_interval: this._initial_follow_interval,
        default_span: this._initial_default_span
      }, { silent: !0 }), this.change.emit();
    }
  };
  mt(M3, "__name__", "DataRange1d"), M3.define(({ Bool: H, Float: V, Nullable: q }) => ({
    range_padding: [V, 0.1],
    range_padding_units: [PaddingUnits, "percent"],
    flipped: [H, !1],
    follow: [q(StartEnd), null],
    follow_interval: [q(V), null],
    default_span: [V, 2],
    only_visible: [H, !1]
  })), M3.internal(({ Enum: H }) => ({
    scale_hint: [H("log", "auto"), "auto"]
  }));
  let DataRange1d = M3;
  Or$1(Str, Tuple(Str, Str), Tuple(Str, Str, Str));
  const FactorSeq = Or$1(List$2(Str), List$2(Tuple(Str, Str)), List$2(Tuple(Str, Str, Str)));
  function map_one_level(O, _, H = 0) {
    const V = /* @__PURE__ */ new Map();
    for (let q = 0; q < O.length; q++) {
      const G = O[q];
      if (!V.has(G))
        V.set(G, { value: 0.5 + q * (1 + _) + H });
      else
        throw new Error(`duplicate factor or subfactor: ${G}`);
    }
    return [V, (O.length - 1) * _];
  }
  function map_two_levels(O, _, H, V = 0) {
    const q = /* @__PURE__ */ new Map(), G = /* @__PURE__ */ new Map();
    for (const [te, ne] of O) {
      const ee = G.get(te) ?? [];
      G.set(te, [...ee, ne]);
    }
    let J = V, Z = 0;
    for (const [te, ne] of G) {
      const ee = ne.length, [ae, oe] = map_one_level(ne, H, J);
      Z += oe;
      const se = sum$3(ne.map((ce) => ae.get(ce).value));
      q.set(te, { value: se / ee, mapping: ae }), J += ee + _ + oe;
    }
    return [q, (G.size - 1) * _ + Z];
  }
  function map_three_levels(O, _, H, V, q = 0) {
    const G = /* @__PURE__ */ new Map(), J = /* @__PURE__ */ new Map();
    for (const [ne, ee, ae] of O) {
      const oe = J.get(ne) ?? [];
      J.set(ne, [...oe, [ee, ae]]);
    }
    let Z = q, te = 0;
    for (const [ne, ee] of J) {
      const ae = ee.length, [oe, se] = map_two_levels(ee, H, V, Z);
      te += se;
      const ce = sum$3(ee.map(([le]) => oe.get(le).value));
      G.set(ne, { value: ce / ae, mapping: oe }), Z += ae + _ + se;
    }
    return [G, (J.size - 1) * _ + te];
  }
  const R3 = class R3 extends Range$2 {
    constructor(H) {
      super(H);
      mt(this, "_mapping");
      mt(this, "invalidate_synthetic", new Signal0(this, "invalidate_synthetic"));
    }
    get min() {
      return this.start;
    }
    get max() {
      return this.end;
    }
    initialize() {
      super.initialize(), this._init();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.properties.factors.change, () => this.reset()), this.connect(this.properties.factor_padding.change, () => this.reset()), this.connect(this.properties.group_padding.change, () => this.reset()), this.connect(this.properties.subgroup_padding.change, () => this.reset()), this.connect(this.properties.range_padding.change, () => this.reset()), this.connect(this.properties.range_padding_units.change, () => this.reset());
    }
    reset() {
      this._init(), this.invalidate_synthetic.emit();
    }
    _lookup(H) {
      switch (H.length) {
        case 1: {
          const [V] = H, G = this._mapping.get(V);
          return G != null ? G.value : NaN;
        }
        case 2: {
          const [V, q] = H, J = this._mapping.get(V);
          if (J != null) {
            const Z = J.mapping.get(q);
            if (Z != null)
              return Z.value;
          }
          return NaN;
        }
        case 3: {
          const [V, q, G] = H, Z = this._mapping.get(V);
          if (Z != null) {
            const te = Z.mapping.get(q);
            if (te != null) {
              const ne = te.mapping.get(G);
              if (ne != null)
                return ne.value;
            }
          }
          return NaN;
        }
      }
    }
    // convert a string factor into a synthetic coordinate
    synthetic(H) {
      if (isNumber$3(H))
        return H;
      if (isString$2(H))
        return this._lookup([H]);
      let V = 0;
      const q = H[H.length - 1];
      return isNumber$3(q) && (V = q, H = H.slice(0, -1)), this._lookup(H) + V;
    }
    // convert an array of string factors into synthetic coordinates
    v_synthetic(H) {
      const V = H.length, q = new ScreenArray(V);
      for (let G = 0; G < V; G++)
        q[G] = this.synthetic(H[G]);
      return q;
    }
    _init() {
      const { levels: H, mapping: V, tops: q, mids: G, inside_padding: J } = (() => {
        if (every$2(this.factors, isString$2)) {
          const ne = this.factors, [ee, ae] = map_one_level(ne, this.factor_padding);
          return { levels: 1, mapping: ee, tops: null, mids: null, inside_padding: ae };
        } else if (every$2(this.factors, (ne) => isArray$3(ne) && ne.length == 2 && isString$2(ne[0]) && isString$2(ne[1]))) {
          const ne = this.factors, [ee, ae] = map_two_levels(ne, this.group_padding, this.factor_padding), oe = [...ee.keys()];
          return { levels: 2, mapping: ee, tops: oe, mids: null, inside_padding: ae };
        } else if (every$2(this.factors, (ne) => isArray$3(ne) && ne.length == 3 && isString$2(ne[0]) && isString$2(ne[1]) && isString$2(ne[2]))) {
          const ne = this.factors, [ee, ae] = map_three_levels(ne, this.group_padding, this.subgroup_padding, this.factor_padding), oe = [...ee.keys()], se = [];
          for (const [ce, le] of ee)
            for (const ue of le.mapping.keys())
              se.push([ce, ue]);
          return { levels: 3, mapping: ee, tops: oe, mids: se, inside_padding: ae };
        } else
          unreachable();
      })();
      this._mapping = V, this.tops = q, this.mids = G;
      let Z = 0, te = this.factors.length + J;
      if (this.range_padding_units == "percent") {
        const ne = (te - Z) * this.range_padding / 2;
        Z -= ne, te += ne;
      } else
        Z -= this.range_padding, te += this.range_padding;
      this.setv({ start: Z, end: te, levels: H }, { silent: !0 }), this.bounds == "auto" && (this._computed_bounds = [Z, te]);
    }
  };
  mt(R3, "__name__", "FactorRange"), R3.define(({ Float: H }) => ({
    factors: [FactorSeq, []],
    factor_padding: [H, 0],
    subgroup_padding: [H, 0.8],
    group_padding: [H, 1.4],
    range_padding: [H, 0],
    range_padding_units: [PaddingUnits, "percent"],
    start: [H, unset, { readonly: !0 }],
    end: [H, unset, { readonly: !0 }]
  })), R3.internal(({ Float: H, Str: V, List: q, Tuple: G, Nullable: J }) => ({
    levels: [H],
    // how many levels of
    mids: [J(q(G(V, V))), null],
    // mid level factors (if 3 total levels)
    tops: [J(q(V)), null]
    // top level factors (whether 2 or 3 total levels)
  }));
  let FactorRange = R3;
  var K_;
  let CoordinateTransform$1 = (K_ = class {
    constructor(_, H) {
      mt(this, "x_scale");
      mt(this, "y_scale");
      mt(this, "x_source");
      mt(this, "y_source");
      mt(this, "ranges");
      mt(this, "scales");
      this.x_scale = _, this.y_scale = H, this.x_source = this.x_scale.source_range, this.y_source = this.y_scale.source_range, this.ranges = [this.x_source, this.y_source], this.scales = [this.x_scale, this.y_scale];
    }
    map_to_screen(_, H) {
      const V = this.x_scale.v_compute(_), q = this.y_scale.v_compute(H);
      return [V, q];
    }
    map_from_screen(_, H) {
      const V = this.x_scale.v_invert(_), q = this.y_scale.v_invert(H);
      return [V, q];
    }
  }, mt(K_, "__name__", "CoordinateTransform"), K_);
  const hg = class hg extends Model$1 {
    constructor(_) {
      super(_);
    }
    get x_ranges() {
      return /* @__PURE__ */ new Map([["default", this.x_source]]);
    }
    get y_ranges() {
      return /* @__PURE__ */ new Map([["default", this.y_source]]);
    }
    _get_scale(_, H, V) {
      const q = _ instanceof FactorRange, G = H instanceof CategoricalScale;
      if (q != G)
        throw new Error(`Range ${_.type} is incompatible is Scale ${H.type}`);
      H instanceof LogScale && _ instanceof DataRange1d && (_.scale_hint = "log");
      const J = H.clone();
      return J.setv({ source_range: _, target_range: V }), J;
    }
    get_transform(_) {
      const { x_source: H, x_scale: V, x_target: q } = this, G = this._get_scale(H, V, q), { y_source: J, y_scale: Z, y_target: te } = this, ne = this._get_scale(J, Z, te), ee = new CompositeScale({
        source_scale: G,
        source_range: G.source_range,
        target_scale: _.x_scale,
        target_range: _.x_target
      }), ae = new CompositeScale({
        source_scale: ne,
        source_range: ne.source_range,
        target_scale: _.y_scale,
        target_range: _.y_target
      });
      return new CoordinateTransform$1(ee, ae);
    }
  };
  mt(hg, "__name__", "CoordinateMapping"), hg.define(({ Ref: _ }) => ({
    x_source: [_(Range$2), () => new DataRange1d()],
    y_source: [_(Range$2), () => new DataRange1d()],
    x_scale: [_(Scale$2), () => new LinearScale()],
    y_scale: [_(Scale$2), () => new LinearScale()],
    x_target: [_(Range$2)],
    y_target: [_(Range$2)]
  }));
  let CoordinateMapping = hg;
  var L0;
  let Styles$1 = (L0 = class extends Model$1 {
    constructor(_) {
      super(_);
    }
  }, mt(L0, "__name__", "Styles"), L0.define(({ Str: _, Nullable: H }) => ({
    align_content: [H(_), null],
    align_items: [H(_), null],
    align_self: [H(_), null],
    alignment_baseline: [H(_), null],
    all: [H(_), null],
    animation: [H(_), null],
    animation_delay: [H(_), null],
    animation_direction: [H(_), null],
    animation_duration: [H(_), null],
    animation_fill_mode: [H(_), null],
    animation_iteration_count: [H(_), null],
    animation_name: [H(_), null],
    animation_play_state: [H(_), null],
    animation_timing_function: [H(_), null],
    aspect_ratio: [H(_), null],
    backface_visibility: [H(_), null],
    background: [H(_), null],
    background_attachment: [H(_), null],
    background_clip: [H(_), null],
    background_color: [H(_), null],
    background_image: [H(_), null],
    background_origin: [H(_), null],
    background_position: [H(_), null],
    background_position_x: [H(_), null],
    background_position_y: [H(_), null],
    background_repeat: [H(_), null],
    background_size: [H(_), null],
    baseline_shift: [H(_), null],
    block_size: [H(_), null],
    border: [H(_), null],
    border_block_end: [H(_), null],
    border_block_end_color: [H(_), null],
    border_block_end_style: [H(_), null],
    border_block_end_width: [H(_), null],
    border_block_start: [H(_), null],
    border_block_start_color: [H(_), null],
    border_block_start_style: [H(_), null],
    border_block_start_width: [H(_), null],
    border_bottom: [H(_), null],
    border_bottom_color: [H(_), null],
    border_bottom_left_radius: [H(_), null],
    border_bottom_right_radius: [H(_), null],
    border_bottom_style: [H(_), null],
    border_bottom_width: [H(_), null],
    border_collapse: [H(_), null],
    border_color: [H(_), null],
    border_image: [H(_), null],
    border_image_outset: [H(_), null],
    border_image_repeat: [H(_), null],
    border_image_slice: [H(_), null],
    border_image_source: [H(_), null],
    border_image_width: [H(_), null],
    border_inline_end: [H(_), null],
    border_inline_end_color: [H(_), null],
    border_inline_end_style: [H(_), null],
    border_inline_end_width: [H(_), null],
    border_inline_start: [H(_), null],
    border_inline_start_color: [H(_), null],
    border_inline_start_style: [H(_), null],
    border_inline_start_width: [H(_), null],
    border_left: [H(_), null],
    border_left_color: [H(_), null],
    border_left_style: [H(_), null],
    border_left_width: [H(_), null],
    border_radius: [H(_), null],
    border_right: [H(_), null],
    border_right_color: [H(_), null],
    border_right_style: [H(_), null],
    border_right_width: [H(_), null],
    border_spacing: [H(_), null],
    border_style: [H(_), null],
    border_top: [H(_), null],
    border_top_color: [H(_), null],
    border_top_left_radius: [H(_), null],
    border_top_right_radius: [H(_), null],
    border_top_style: [H(_), null],
    border_top_width: [H(_), null],
    border_width: [H(_), null],
    bottom: [H(_), null],
    box_shadow: [H(_), null],
    box_sizing: [H(_), null],
    break_after: [H(_), null],
    break_before: [H(_), null],
    break_inside: [H(_), null],
    caption_side: [H(_), null],
    caret_color: [H(_), null],
    clear: [H(_), null],
    clip: [H(_), null],
    clip_path: [H(_), null],
    clip_rule: [H(_), null],
    color: [H(_), null],
    color_interpolation: [H(_), null],
    color_interpolation_filters: [H(_), null],
    column_count: [H(_), null],
    column_fill: [H(_), null],
    column_gap: [H(_), null],
    column_rule: [H(_), null],
    column_rule_color: [H(_), null],
    column_rule_style: [H(_), null],
    column_rule_width: [H(_), null],
    column_span: [H(_), null],
    column_width: [H(_), null],
    columns: [H(_), null],
    content: [H(_), null],
    counter_increment: [H(_), null],
    counter_reset: [H(_), null],
    cursor: [H(_), null],
    direction: [H(_), null],
    display: [H(_), null],
    dominant_baseline: [H(_), null],
    empty_cells: [H(_), null],
    fill: [H(_), null],
    fill_opacity: [H(_), null],
    fill_rule: [H(_), null],
    filter: [H(_), null],
    flex: [H(_), null],
    flex_basis: [H(_), null],
    flex_direction: [H(_), null],
    flex_flow: [H(_), null],
    flex_grow: [H(_), null],
    flex_shrink: [H(_), null],
    flex_wrap: [H(_), null],
    float: [H(_), null],
    flood_color: [H(_), null],
    flood_opacity: [H(_), null],
    font: [H(_), null],
    font_family: [H(_), null],
    font_feature_settings: [H(_), null],
    font_kerning: [H(_), null],
    font_size: [H(_), null],
    font_size_adjust: [H(_), null],
    font_stretch: [H(_), null],
    font_style: [H(_), null],
    font_synthesis: [H(_), null],
    font_variant: [H(_), null],
    font_variant_caps: [H(_), null],
    font_variant_east_asian: [H(_), null],
    font_variant_ligatures: [H(_), null],
    font_variant_numeric: [H(_), null],
    font_variant_position: [H(_), null],
    font_weight: [H(_), null],
    gap: [H(_), null],
    glyph_orientation_vertical: [H(_), null],
    grid: [H(_), null],
    grid_area: [H(_), null],
    grid_auto_columns: [H(_), null],
    grid_auto_flow: [H(_), null],
    grid_auto_rows: [H(_), null],
    grid_column: [H(_), null],
    grid_column_end: [H(_), null],
    grid_column_gap: [H(_), null],
    grid_column_start: [H(_), null],
    grid_gap: [H(_), null],
    grid_row: [H(_), null],
    grid_row_end: [H(_), null],
    grid_row_gap: [H(_), null],
    grid_row_start: [H(_), null],
    grid_template: [H(_), null],
    grid_template_areas: [H(_), null],
    grid_template_columns: [H(_), null],
    grid_template_rows: [H(_), null],
    height: [H(_), null],
    hyphens: [H(_), null],
    image_orientation: [H(_), null],
    image_rendering: [H(_), null],
    inline_size: [H(_), null],
    justify_content: [H(_), null],
    justify_items: [H(_), null],
    justify_self: [H(_), null],
    left: [H(_), null],
    letter_spacing: [H(_), null],
    lighting_color: [H(_), null],
    line_break: [H(_), null],
    line_height: [H(_), null],
    list_style: [H(_), null],
    list_style_image: [H(_), null],
    list_style_position: [H(_), null],
    list_style_type: [H(_), null],
    margin: [H(_), null],
    margin_block_end: [H(_), null],
    margin_block_start: [H(_), null],
    margin_bottom: [H(_), null],
    margin_inline_end: [H(_), null],
    margin_inline_start: [H(_), null],
    margin_left: [H(_), null],
    margin_right: [H(_), null],
    margin_top: [H(_), null],
    marker: [H(_), null],
    marker_end: [H(_), null],
    marker_mid: [H(_), null],
    marker_start: [H(_), null],
    mask: [H(_), null],
    mask_composite: [H(_), null],
    mask_image: [H(_), null],
    mask_position: [H(_), null],
    mask_repeat: [H(_), null],
    mask_size: [H(_), null],
    mask_type: [H(_), null],
    max_block_size: [H(_), null],
    max_height: [H(_), null],
    max_inline_size: [H(_), null],
    max_width: [H(_), null],
    min_block_size: [H(_), null],
    min_height: [H(_), null],
    min_inline_size: [H(_), null],
    min_width: [H(_), null],
    object_fit: [H(_), null],
    object_position: [H(_), null],
    opacity: [H(_), null],
    order: [H(_), null],
    orphans: [H(_), null],
    outline: [H(_), null],
    outline_color: [H(_), null],
    outline_offset: [H(_), null],
    outline_style: [H(_), null],
    outline_width: [H(_), null],
    overflow: [H(_), null],
    overflow_anchor: [H(_), null],
    overflow_wrap: [H(_), null],
    overflow_x: [H(_), null],
    overflow_y: [H(_), null],
    overscroll_behavior: [H(_), null],
    overscroll_behavior_block: [H(_), null],
    overscroll_behavior_inline: [H(_), null],
    overscroll_behavior_x: [H(_), null],
    overscroll_behavior_y: [H(_), null],
    padding: [H(_), null],
    padding_block_end: [H(_), null],
    padding_block_start: [H(_), null],
    padding_bottom: [H(_), null],
    padding_inline_end: [H(_), null],
    padding_inline_start: [H(_), null],
    padding_left: [H(_), null],
    padding_right: [H(_), null],
    padding_top: [H(_), null],
    page_break_after: [H(_), null],
    page_break_before: [H(_), null],
    page_break_inside: [H(_), null],
    paint_order: [H(_), null],
    perspective: [H(_), null],
    perspective_origin: [H(_), null],
    place_content: [H(_), null],
    place_items: [H(_), null],
    place_self: [H(_), null],
    pointer_events: [H(_), null],
    position: [H(_), null],
    quotes: [H(_), null],
    resize: [H(_), null],
    right: [H(_), null],
    rotate: [H(_), null],
    row_gap: [H(_), null],
    ruby_align: [H(_), null],
    ruby_position: [H(_), null],
    scale: [H(_), null],
    scroll_behavior: [H(_), null],
    shape_rendering: [H(_), null],
    stop_color: [H(_), null],
    stop_opacity: [H(_), null],
    stroke: [H(_), null],
    stroke_dasharray: [H(_), null],
    stroke_dashoffset: [H(_), null],
    stroke_linecap: [H(_), null],
    stroke_linejoin: [H(_), null],
    stroke_miterlimit: [H(_), null],
    stroke_opacity: [H(_), null],
    stroke_width: [H(_), null],
    tab_size: [H(_), null],
    table_layout: [H(_), null],
    text_align: [H(_), null],
    text_align_last: [H(_), null],
    text_anchor: [H(_), null],
    text_combine_upright: [H(_), null],
    text_decoration: [H(_), null],
    text_decoration_color: [H(_), null],
    text_decoration_line: [H(_), null],
    text_decoration_style: [H(_), null],
    text_emphasis: [H(_), null],
    text_emphasis_color: [H(_), null],
    text_emphasis_position: [H(_), null],
    text_emphasis_style: [H(_), null],
    text_indent: [H(_), null],
    text_justify: [H(_), null],
    text_orientation: [H(_), null],
    text_overflow: [H(_), null],
    text_rendering: [H(_), null],
    text_shadow: [H(_), null],
    text_transform: [H(_), null],
    text_underline_position: [H(_), null],
    top: [H(_), null],
    touch_action: [H(_), null],
    transform: [H(_), null],
    transform_box: [H(_), null],
    transform_origin: [H(_), null],
    transform_style: [H(_), null],
    transition: [H(_), null],
    transition_delay: [H(_), null],
    transition_duration: [H(_), null],
    transition_property: [H(_), null],
    transition_timing_function: [H(_), null],
    translate: [H(_), null],
    unicode_bidi: [H(_), null],
    user_select: [H(_), null],
    vertical_align: [H(_), null],
    visibility: [H(_), null],
    white_space: [H(_), null],
    widows: [H(_), null],
    width: [H(_), null],
    will_change: [H(_), null],
    word_break: [H(_), null],
    word_spacing: [H(_), null],
    word_wrap: [H(_), null],
    writing_mode: [H(_), null],
    z_index: [H(_), null]
  })), L0);
  class StyleSheet extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(StyleSheet, "__name__", "StyleSheet");
  const pg = class pg extends StyleSheet {
    constructor(_) {
      super(_);
    }
    underlying() {
      return new InlineStyleSheet$1(this.css);
    }
  };
  mt(pg, "__name__", "InlineStyleSheet"), pg.define(({ Str: _ }) => ({
    css: [_]
  }));
  let InlineStyleSheet = pg;
  const mg = class mg extends StyleSheet {
    constructor(_) {
      super(_);
    }
    underlying() {
      return new ImportedStyleSheet$1(this.url);
    }
  };
  mt(mg, "__name__", "ImportedStyleSheet"), mg.define(({ Str: _ }) => ({
    url: [_]
  }));
  let ImportedStyleSheet = mg;
  class GlobalInlineStyleSheet extends InlineStyleSheet {
    constructor(H) {
      super(H);
      mt(this, "_underlying", null);
    }
    underlying() {
      return this._underlying == null && (this._underlying = new GlobalInlineStyleSheet$1(this.css)), this._underlying;
    }
  }
  mt(GlobalInlineStyleSheet, "__name__", "GlobalInlineStyleSheet");
  class GlobalImportedStyleSheet extends ImportedStyleSheet {
    constructor(H) {
      super(H);
      mt(this, "_underlying", null);
    }
    underlying() {
      return this._underlying == null && (this._underlying = new GlobalImportedStyleSheet$1(this.url)), this._underlying;
    }
  }
  mt(GlobalImportedStyleSheet, "__name__", "GlobalImportedStyleSheet");
  const { sin: sin$4, cos: cos$4 } = Math, hp = class hp {
    constructor(_ = 1, H = 0, V = 0, q = 1, G = 0, J = 0) {
      mt(this, "a");
      mt(this, "b");
      mt(this, "c");
      mt(this, "d");
      mt(this, "e");
      mt(this, "f");
      this.a = _, this.b = H, this.c = V, this.d = q, this.e = G, this.f = J;
    }
    toString() {
      const { a: _, b: H, c: V, d: q, e: G, f: J } = this;
      return `matrix(${_}, ${H}, ${V}, ${q}, ${G}, ${J})`;
    }
    static from_DOMMatrix(_) {
      const { a: H, b: V, c: q, d: G, e: J, f: Z } = _;
      return new hp(H, V, q, G, J, Z);
    }
    to_DOMMatrix() {
      const { a: _, b: H, c: V, d: q, e: G, f: J } = this;
      return new DOMMatrix([_, H, V, q, G, J]);
    }
    clone() {
      const { a: _, b: H, c: V, d: q, e: G, f: J } = this;
      return new hp(_, H, V, q, G, J);
    }
    [equals](_, H) {
      return H.eq(this.a, _.a) && H.eq(this.b, _.b) && H.eq(this.c, _.c) && H.eq(this.d, _.d) && H.eq(this.e, _.e) && H.eq(this.f, _.f);
    }
    reset() {
      this.a = 1, this.b = 0, this.c = 0, this.d = 1, this.e = 0, this.f = 0;
    }
    get is_identity() {
      const { a: _, b: H, c: V, d: q, e: G, f: J } = this;
      return _ == 1 && H == 0 && V == 0 && q == 1 && G == 0 && J == 0;
    }
    apply_point(_) {
      const [H, V] = this.apply(_.x, _.y);
      return { x: H, y: V };
    }
    apply_rect(_) {
      const H = this.apply_point(_.p0), V = this.apply_point(_.p1), q = this.apply_point(_.p2), G = this.apply_point(_.p3);
      return { p0: H, p1: V, p2: q, p3: G };
    }
    apply(_, H) {
      const { a: V, b: q, c: G, d: J, e: Z, f: te } = this;
      return [
        V * _ + G * H + Z,
        q * _ + J * H + te
      ];
    }
    iv_apply(_, H) {
      const { a: V, b: q, c: G, d: J, e: Z, f: te } = this, ne = _.length;
      for (let ee = 0; ee < ne; ee++) {
        const ae = _[ee], oe = H[ee];
        _[ee] = V * ae + G * oe + Z, H[ee] = q * ae + J * oe + te;
      }
    }
    transform(_, H, V, q, G, J) {
      const { a: Z, b: te, c: ne, d: ee, e: ae, f: oe } = this;
      return this.a = Z * _ + ne * H, this.c = Z * V + ne * q, this.e = Z * G + ne * J + ae, this.b = te * _ + ee * H, this.d = te * V + ee * q, this.f = te * G + ee * J + oe, this;
    }
    translate(_, H) {
      return this.transform(1, 0, 0, 1, _, H);
    }
    scale(_, H) {
      return this.transform(_, 0, 0, H, 0, 0);
    }
    skew(_, H) {
      return this.transform(1, H, _, 1, 0, 0);
    }
    rotate(_) {
      if (_ == 0)
        return this;
      const H = sin$4(_), V = cos$4(_);
      return this.transform(V, H, -H, V, 0, 0);
    }
    rotate_ccw(_) {
      return this.rotate(-_);
    }
    rotate_around(_, H, V) {
      return this.translate(_, H), this.rotate(V), this.translate(-_, -H), this;
    }
    translate_x(_) {
      return this.translate(_, 0);
    }
    translate_y(_) {
      return this.translate(0, _);
    }
    flip() {
      return this.scale(-1, -1);
    }
    flip_x() {
      return this.scale(1, -1);
    }
    flip_y() {
      return this.scale(-1, 1);
    }
    inverse() {
      return hp.from_DOMMatrix(this.to_DOMMatrix().inverse());
    }
  };
  mt(hp, "__name__", "AffineTransform");
  let AffineTransform = hp;
  function rotate_around(O, _, H) {
    if (H == 0)
      return O;
    {
      const V = new AffineTransform();
      V.rotate_around(_.x, _.y, H);
      const [q, G] = V.apply(O.x, O.y);
      return { x: q, y: G };
    }
  }
  const { PI, log: log$6, sin: sin$3, cos: cos$3, sqrt: sqrt$6, floor: floor$2 } = Math, MAX_INT32 = 2147483647;
  class AbstractRandom {
    float() {
      return (this.integer() - 1) / (MAX_INT32 - 1);
    }
    floats(_, H = 0, V = 1) {
      const q = new Array(_);
      for (let G = 0; G < _; G++)
        q[G] = H + this.float() * (V - H);
      return q;
    }
    choices(_, H) {
      const V = H.length, q = new Array(_);
      for (let G = 0; G < _; G++)
        q[G] = H[this.integer() % V];
      return q;
    }
    uniform(_, H) {
      return _ + (this.float() - 0.5) * H;
    }
    uniforms(_, H, V) {
      return Float64Array.from({ length: V }, () => this.uniform(_, H));
    }
    normal(_, H) {
      return this.normals(_, H, 1)[0];
    }
    normals(_, H, V) {
      const [q, G] = [_, H], J = new Float64Array(V);
      for (let Z = 0; Z < V; Z += 2) {
        const te = this.float(), ne = this.float(), ee = sqrt$6(-2 * log$6(te));
        J[Z] = q + G * (ee * cos$3(2 * PI * ne)), Z + 1 < V && (J[Z + 1] = q + G * (ee * sin$3(2 * PI * ne)));
      }
      return J;
    }
  }
  mt(AbstractRandom, "__name__", "AbstractRandom");
  class SystemRandom extends AbstractRandom {
    integer() {
      return floor$2(Math.random() * MAX_INT32);
    }
  }
  mt(SystemRandom, "__name__", "SystemRandom");
  class LCGRandom extends AbstractRandom {
    constructor(H) {
      super();
      mt(this, "_seed");
      this._seed = H % MAX_INT32, this._seed <= 0 && (this._seed += MAX_INT32 - 1);
    }
    integer() {
      return this._seed = 48271 * this._seed % MAX_INT32, this._seed;
    }
  }
  mt(LCGRandom, "__name__", "LCGRandom");
  class Random extends LCGRandom {
  }
  mt(Random, "__name__", "Random");
  const random$1 = new Random(Date.now());
  function createNamedToNumberedLookup(O, _) {
    const H = /* @__PURE__ */ new Map(), V = O.split(",");
    _ = _ ?? 10;
    for (let q = 0; q < V.length; q += 2) {
      const G = `&${V[q + 1]};`, J = parseInt(V[q], _);
      H.set(G, `&#${J};`);
    }
    return H.set("\\xa0", "&#160;"), H;
  }
  function getTextAnchor(O) {
    const _ = { left: "start", right: "end", center: "middle", start: "start", end: "end" };
    return O in _ ? _[O] : _.start;
  }
  function getDominantBaseline(O) {
    const _ = { alphabetic: "alphabetic", hanging: "hanging", top: "text-before-edge", bottom: "text-after-edge", middle: "central" };
    return O in _ ? _[O] : _.alphabetic;
  }
  const namedEntities = createNamedToNumberedLookup("50,nbsp,51,iexcl,52,cent,53,pound,54,curren,55,yen,56,brvbar,57,sect,58,uml,59,copy,5a,ordf,5b,laquo,5c,not,5d,shy,5e,reg,5f,macr,5g,deg,5h,plusmn,5i,sup2,5j,sup3,5k,acute,5l,micro,5m,para,5n,middot,5o,cedil,5p,sup1,5q,ordm,5r,raquo,5s,frac14,5t,frac12,5u,frac34,5v,iquest,60,Agrave,61,Aacute,62,Acirc,63,Atilde,64,Auml,65,Aring,66,AElig,67,Ccedil,68,Egrave,69,Eacute,6a,Ecirc,6b,Euml,6c,Igrave,6d,Iacute,6e,Icirc,6f,Iuml,6g,ETH,6h,Ntilde,6i,Ograve,6j,Oacute,6k,Ocirc,6l,Otilde,6m,Ouml,6n,times,6o,Oslash,6p,Ugrave,6q,Uacute,6r,Ucirc,6s,Uuml,6t,Yacute,6u,THORN,6v,szlig,70,agrave,71,aacute,72,acirc,73,atilde,74,auml,75,aring,76,aelig,77,ccedil,78,egrave,79,eacute,7a,ecirc,7b,euml,7c,igrave,7d,iacute,7e,icirc,7f,iuml,7g,eth,7h,ntilde,7i,ograve,7j,oacute,7k,ocirc,7l,otilde,7m,ouml,7n,divide,7o,oslash,7p,ugrave,7q,uacute,7r,ucirc,7s,uuml,7t,yacute,7u,thorn,7v,yuml,ci,fnof,sh,Alpha,si,Beta,sj,Gamma,sk,Delta,sl,Epsilon,sm,Zeta,sn,Eta,so,Theta,sp,Iota,sq,Kappa,sr,Lambda,ss,Mu,st,Nu,su,Xi,sv,Omicron,t0,Pi,t1,Rho,t3,Sigma,t4,Tau,t5,Upsilon,t6,Phi,t7,Chi,t8,Psi,t9,Omega,th,alpha,ti,beta,tj,gamma,tk,delta,tl,epsilon,tm,zeta,tn,eta,to,theta,tp,iota,tq,kappa,tr,lambda,ts,mu,tt,nu,tu,xi,tv,omicron,u0,pi,u1,rho,u2,sigmaf,u3,sigma,u4,tau,u5,upsilon,u6,phi,u7,chi,u8,psi,u9,omega,uh,thetasym,ui,upsih,um,piv,812,bull,816,hellip,81i,prime,81j,Prime,81u,oline,824,frasl,88o,weierp,88h,image,88s,real,892,trade,89l,alefsym,8cg,larr,8ch,uarr,8ci,rarr,8cj,darr,8ck,harr,8dl,crarr,8eg,lArr,8eh,uArr,8ei,rArr,8ej,dArr,8ek,hArr,8g0,forall,8g2,part,8g3,exist,8g5,empty,8g7,nabla,8g8,isin,8g9,notin,8gb,ni,8gf,prod,8gh,sum,8gi,minus,8gn,lowast,8gq,radic,8gt,prop,8gu,infin,8h0,ang,8h7,and,8h8,or,8h9,cap,8ha,cup,8hb,int,8hk,there4,8hs,sim,8i5,cong,8i8,asymp,8j0,ne,8j1,equiv,8j4,le,8j5,ge,8k2,sub,8k3,sup,8k4,nsub,8k6,sube,8k7,supe,8kl,oplus,8kn,otimes,8l5,perp,8m5,sdot,8o8,lceil,8o9,rceil,8oa,lfloor,8ob,rfloor,8p9,lang,8pa,rang,9ea,loz,9j0,spades,9j3,clubs,9j5,hearts,9j6,diams,ai,OElig,aj,oelig,b0,Scaron,b1,scaron,bo,Yuml,m6,circ,ms,tilde,802,ensp,803,emsp,809,thinsp,80c,zwnj,80d,zwj,80e,lrm,80f,rlm,80j,ndash,80k,mdash,80o,lsquo,80p,rsquo,80q,sbquo,80s,ldquo,80t,rdquo,80u,bdquo,810,dagger,811,Dagger,81g,permil,81p,lsaquo,81q,rsaquo,85c,euro", 32), STYLES = {
    strokeStyle: {
      svgAttr: "stroke",
      // corresponding svg attribute
      canvas: "#000000",
      // canvas default
      svg: "none",
      // svg default
      apply: "stroke"
      // apply on stroke() or fill()
    },
    fillStyle: {
      svgAttr: "fill",
      canvas: "#000000",
      svg: null,
      // svg default is black, but we need to special case this to handle canvas stroke without fill
      apply: "fill"
    },
    lineCap: {
      svgAttr: "stroke-linecap",
      canvas: "butt",
      svg: "butt",
      apply: "stroke"
    },
    lineJoin: {
      svgAttr: "stroke-linejoin",
      canvas: "miter",
      svg: "miter",
      apply: "stroke"
    },
    miterLimit: {
      svgAttr: "stroke-miterlimit",
      canvas: 10,
      svg: 4,
      apply: "stroke"
    },
    lineWidth: {
      svgAttr: "stroke-width",
      canvas: 1,
      svg: 1,
      apply: "stroke"
    },
    globalAlpha: {
      svgAttr: "opacity",
      canvas: 1,
      svg: 1,
      apply: "fill stroke"
    },
    font: {
      // font converts to multiple svg attributes, there is custom logic for this
      canvas: "10px sans-serif"
    },
    shadowColor: {
      canvas: "#000000"
    },
    shadowOffsetX: {
      canvas: 0
    },
    shadowOffsetY: {
      canvas: 0
    },
    shadowBlur: {
      canvas: 0
    },
    fontKerning: {
      canvas: "auto"
    },
    textAlign: {
      canvas: "start"
    },
    textBaseline: {
      canvas: "alphabetic"
    },
    lineDash: {
      svgAttr: "stroke-dasharray",
      canvas: [],
      svg: null,
      apply: "stroke"
    },
    lineDashOffset: {
      svgAttr: "stroke-dashoffset",
      canvas: 0,
      svg: 0,
      apply: "stroke"
    }
  };
  function is_VideoFrame(O) {
    return typeof VideoFrame < "u" && O instanceof VideoFrame;
  }
  function width_height(O) {
    if (is_VideoFrame(O))
      return [O.codedWidth, O.codedHeight];
    {
      let H = function(G) {
        return isNumber$3(G) ? G : G.baseVal.value;
      };
      var _ = H;
      const { width: V, height: q } = O;
      return [H(V), H(q)];
    }
  }
  class CanvasGradient {
    constructor(_, H) {
      mt(this, "__root");
      mt(this, "__ctx");
      this.__root = _, this.__ctx = H;
    }
    /**
     * Adds a color stop to the gradient root
     */
    addColorStop(_, H) {
      if (this.__root.nodeName === "linearGradient" && this.__root.getAttribute("x1") === this.__root.getAttribute("x2") && this.__root.getAttribute("y1") === this.__root.getAttribute("y2") || this.__root.nodeName === "radialGradient" && this.__root.getAttribute("cx") === this.__root.getAttribute("fx") && this.__root.getAttribute("cy") === this.__root.getAttribute("fy") && this.__root.getAttribute("r") === this.__root.getAttribute("r0"))
        return;
      const V = this.__ctx.__createElement("stop");
      if (V.setAttribute("offset", `${_}`), H.indexOf("rgba") !== -1) {
        const G = /rgba\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d?\.?\d*)\s*\)/gi.exec(H), [, J, Z, te, ne] = G;
        V.setAttribute("stop-color", `rgb(${J},${Z},${te})`), V.setAttribute("stop-opacity", ne);
      } else
        V.setAttribute("stop-color", H);
      this.__root.appendChild(V);
    }
  }
  mt(CanvasGradient, "__name__", "CanvasGradient");
  class CanvasPattern {
    constructor(_, H) {
      mt(this, "__root");
      mt(this, "__ctx");
      this.__root = _, this.__ctx = H;
    }
    setTransform(_) {
      throw new Error("not implemented");
    }
  }
  mt(CanvasPattern, "__name__", "CanvasPattern");
  const e0 = class e0 {
    constructor(_) {
      mt(this, "__canvas");
      mt(this, "__ctx");
      mt(this, "__root");
      mt(this, "__ids");
      mt(this, "__defs");
      mt(this, "__stack");
      mt(this, "__document");
      mt(this, "__currentElement");
      mt(this, "__currentDefaultPath", "");
      mt(this, "__currentPosition", null);
      mt(this, "strokeStyle");
      mt(this, "fillStyle");
      mt(this, "lineCap");
      mt(this, "lineJoin");
      mt(this, "miterLimit");
      mt(this, "lineWidth");
      mt(this, "globalAlpha", 1);
      mt(this, "globalCompositeOperation");
      // TODO: implement
      mt(this, "font");
      mt(this, "direction");
      // TODO: implement
      mt(this, "shadowColor");
      mt(this, "shadowOffsetX");
      mt(this, "shadowOffsetY");
      mt(this, "shadowBlur");
      mt(this, "fontKerning");
      mt(this, "textAlign");
      mt(this, "textBaseline");
      mt(this, "lineDash");
      mt(this, "lineDashOffset");
      mt(this, "filter");
      // TODO: implement
      mt(this, "imageSmoothingEnabled");
      // TODO: implement
      mt(this, "imageSmoothingQuality");
      // TODO: implement
      mt(this, "_width");
      mt(this, "_height");
      mt(this, "_transform", new AffineTransform());
      mt(this, "_clip_path", null);
      this.__document = (_ == null ? void 0 : _.document) ?? document, (_ == null ? void 0 : _.ctx) != null ? this.__ctx = _.ctx : (this.__canvas = this.__document.createElement("canvas"), this.__ctx = this.__canvas.getContext("2d")), this.reset(), this.__root = this.__document.createElementNS("http://www.w3.org/2000/svg", "svg"), this.__root.setAttribute("version", "1.1"), this.__root.setAttribute("xmlns", "http://www.w3.org/2000/svg"), this.__currentElement = this.__root, this.width = (_ == null ? void 0 : _.width) ?? 500, this.height = (_ == null ? void 0 : _.height) ?? 500, this.__ids = /* @__PURE__ */ new Set(), this.__defs = this.__document.createElementNS("http://www.w3.org/2000/svg", "defs"), this.__root.appendChild(this.__defs);
    }
    get canvas() {
      return this;
    }
    get width() {
      return this._width;
    }
    set width(_) {
      this._width = _, this.__root.setAttribute("width", `${_}`);
    }
    get height() {
      return this._height;
    }
    set height(_) {
      this._height = _, this.__root.setAttribute("height", `${_}`);
    }
    // helper function that generates a random string
    _random_string() {
      const _ = "ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";
      let H;
      do
        H = e0.__random.choices(12, _).join("");
      while (this.__ids.has(H));
      return H;
    }
    /**
     * Creates the specified svg element
     */
    __createElement(_, H = {}, V = !1) {
      const q = this.__document.createElementNS("http://www.w3.org/2000/svg", _);
      V && (q.setAttribute("fill", "none"), q.setAttribute("stroke", "none"));
      const G = Object.keys(H);
      for (const J of G)
        q.setAttribute(J, `${H[J]}`);
      return q;
    }
    /**
     * Applies default canvas styles to the context
     */
    __setDefaultStyles() {
      const _ = Object.keys(STYLES), H = this;
      for (let V = 0; V < _.length; V++) {
        const q = _[V];
        H[q] = STYLES[q].canvas;
      }
    }
    /**
     * Applies styles on restore
     */
    __applyStyleState(_) {
      const H = Object.keys(_), V = this;
      for (let q = 0; q < H.length; q++) {
        const G = H[q];
        V[G] = _[G];
      }
    }
    /**
     * Gets the current style state
     */
    __getStyleState() {
      const _ = Object.keys(STYLES), H = {};
      for (let V = 0; V < _.length; V++) {
        const q = _[V];
        H[q] = this[q];
      }
      return H;
    }
    /**
     * Apples the current styles to the current SVG element. On "ctx.fill" or "ctx.stroke"
     */
    __applyStyleToCurrentElement(_) {
      const H = this.__currentElement, V = Object.keys(STYLES);
      for (let q = 0; q < V.length; q++) {
        const G = STYLES[V[q]], J = this[V[q]];
        if (G.apply != null && G.apply.includes(_)) {
          if (J instanceof CanvasPattern) {
            for (const te of [...J.__ctx.__defs.childNodes])
              if (te instanceof Element) {
                const ne = te.getAttribute("id");
                this.__ids.add(ne), this.__defs.appendChild(te);
              }
            const Z = J.__root.getAttribute("id");
            H.setAttribute(G.apply, `url(#${Z})`);
          } else if (J instanceof CanvasGradient) {
            const Z = J.__root.getAttribute("id");
            H.setAttribute(G.apply, `url(#${Z})`);
          } else if (G.svg !== J)
            if ((G.svgAttr === "stroke" || G.svgAttr === "fill") && isString$2(J) && J.indexOf("rgba") !== -1) {
              const te = /rgba\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d?\.?\d*)\s*\)/gi.exec(J), [, ne, ee, ae, oe] = te;
              H.setAttribute(G.svgAttr, `rgb(${ne},${ee},${ae})`);
              const se = parseFloat(oe) * this.globalAlpha;
              H.setAttribute(`${G.svgAttr}-opacity`, `${se}`);
            } else {
              let Z = G.svgAttr;
              if (V[q] === "globalAlpha" && (Z = `${_}-${G.svgAttr}`, H.getAttribute(Z) != null))
                continue;
              H.setAttribute(Z, `${J}`);
            }
        }
      }
    }
    /**
      * Returns the serialized value of the svg so far
      * @param fixNamedEntities - Standalone SVG doesn't support named entities, which document.createTextNode encodes.
      *                           If true, we attempt to find all named entities and encode it as a numeric entity.
      * @return serialized svg
      */
    get_serialized_svg(_ = !1) {
      let H = new XMLSerializer().serializeToString(this.__root);
      if (_)
        for (const [V, q] of namedEntities) {
          const G = new RegExp(V, "gi");
          G.test(H) && (H = H.replace(G, q));
        }
      return H;
    }
    get_svg() {
      return this.__root;
    }
    /**
      * Will generate a group tag.
      */
    save() {
      this.__stack.push({
        transform: this._transform,
        clip_path: this._clip_path,
        attributes: this.__getStyleState()
      }), this._transform = this._transform.clone();
    }
    /**
      * Sets current element to parent, or just root if already root
      */
    restore() {
      if (this.__stack.length == 0)
        return;
      const { transform: _, clip_path: H, attributes: V } = this.__stack.pop();
      this._transform = _, this._clip_path = H, this.__applyStyleState(V);
    }
    reset() {
      this.__setDefaultStyles(), this.__stack = [];
    }
    _apply_transform(_, H = this._transform) {
      H.is_identity || _.setAttribute("transform", H.toString());
    }
    /**
      *  scales the current element
      */
    scale(_, H) {
      !isFinite(_) || H != null && !isFinite(H) || this._transform.scale(_, H ?? _);
    }
    /**
      * rotates the current element
      */
    rotate(_) {
      isFinite(_) && this._transform.rotate(_);
    }
    /**
      * translates the current element
      */
    translate(_, H) {
      isFinite(_ + H) && this._transform.translate(_, H);
    }
    /**
      * applies a transform to the current element
      */
    transform(_, H, V, q, G, J) {
      isFinite(_ + H + V + q + G + J) && this._transform.transform(_, H, V, q, G, J);
    }
    /**
      * Create a new Path Element
      */
    beginPath() {
      this.__currentDefaultPath = "", this.__currentPosition = null, this.__init_element();
    }
    __init_element() {
      const _ = this.__createElement("path", {}, !0);
      this.__root.appendChild(_), this.__currentElement = _;
    }
    /**
      * Helper function to apply currentDefaultPath to current path element
      */
    __applyCurrentDefaultPath() {
      const _ = this.__currentElement;
      _.nodeName === "path" ? _.setAttribute("d", this.__currentDefaultPath) : console.error("Attempted to apply path command to node", _.nodeName);
    }
    /**
      * Helper function to add path command
      */
    __addPathCommand(_, H, V) {
      const q = this.__currentDefaultPath == "" ? "" : " ";
      this.__currentDefaultPath += q + V, this.__currentPosition = { x: _, y: H };
    }
    get _hasCurrentDefaultPath() {
      return this.__currentDefaultPath != "";
    }
    /**
      * Adds the move command to the current path element,
      * if the currentPathElement is not empty create a new path element
      */
    moveTo(_, H) {
      if (!isFinite(_ + H))
        return;
      this.__currentElement.nodeName !== "path" && this.beginPath();
      const [q, G] = this._transform.apply(_, H);
      this.__addPathCommand(q, G, `M ${q} ${G}`);
    }
    /**
      * Closes the current path
      */
    closePath() {
      this._hasCurrentDefaultPath && this.__addPathCommand(NaN, NaN, "Z");
    }
    /**
      * Adds a line to command
      */
    lineTo(_, H) {
      if (isFinite(_ + H))
        if (!this._hasCurrentDefaultPath)
          this.moveTo(_, H);
        else {
          const [V, q] = this._transform.apply(_, H);
          this.__addPathCommand(V, q, `L ${V} ${q}`);
        }
    }
    /**
      * Add a bezier command
      */
    bezierCurveTo(_, H, V, q, G, J) {
      if (!isFinite(_ + H + V + q + G + J))
        return;
      const [Z, te] = this._transform.apply(G, J), [ne, ee] = this._transform.apply(_, H), [ae, oe] = this._transform.apply(V, q);
      this.__addPathCommand(Z, te, `C ${ne} ${ee} ${ae} ${oe} ${Z} ${te}`);
    }
    /**
      * Adds a quadratic curve to command
      */
    quadraticCurveTo(_, H, V, q) {
      if (!isFinite(_ + H + V + q))
        return;
      const [G, J] = this._transform.apply(V, q), [Z, te] = this._transform.apply(_, H);
      this.__addPathCommand(G, J, `Q ${Z} ${te} ${G} ${J}`);
    }
    /**
      * Adds the arcTo to the current path
      *
      * @see http://www.w3.org/TR/2015/WD-2dcontext-20150514/#dom-context-2d-arcto
      */
    arcTo(_, H, V, q, G) {
      if (!isFinite(_ + H + V + q + G) || this.__currentPosition == null)
        return;
      const J = this._transform.inverse(), [Z, te] = J.apply(this.__currentPosition.x, this.__currentPosition.y);
      if (G < 0)
        throw new Error(`IndexSizeError: The radius provided (${G}) is negative.`);
      if (Z === _ && te === H || _ === V && H === q || G === 0) {
        this.lineTo(_, H);
        return;
      }
      function ne([ge, ve]) {
        const be = Math.sqrt(ge ** 2 + ve ** 2);
        return [ge / be, ve / be];
      }
      const ee = ne([Z - _, te - H]), ae = ne([V - _, q - H]);
      if (ee[0] * ae[1] === ee[1] * ae[0]) {
        this.lineTo(_, H);
        return;
      }
      const oe = ee[0] * ae[0] + ee[1] * ae[1], se = Math.acos(Math.abs(oe)), ce = ne([
        ee[0] + ae[0],
        ee[1] + ae[1]
      ]), le = G / Math.sin(se / 2), ue = _ + le * ce[0], de = H + le * ce[1], fe = [
        -ee[1],
        ee[0]
      ], he = [
        ae[1],
        -ae[0]
      ];
      function me(ge) {
        const ve = ge[0];
        return ge[1] >= 0 ? Math.acos(ve) : -Math.acos(ve);
      }
      const pe = me(fe), ye = me(he);
      this.lineTo(ue + fe[0] * G, de + fe[1] * G), this.arc(ue, de, G, pe, ye);
    }
    /**
      * Sets the stroke property on the current element
      */
    stroke() {
      this.__currentElement.nodeName === "path" && this.__currentElement.setAttribute("paint-order", "fill"), this.__applyCurrentDefaultPath(), this.__applyStyleToCurrentElement("stroke"), this._clip_path != null && this.__currentElement.setAttribute("clip-path", this._clip_path);
    }
    fill(_, H) {
      let V = null;
      if (_ instanceof Path2D)
        V = _;
      else if (H == null)
        H = _;
      else
        throw new Error("invalid arguments");
      if (V != null)
        throw new Error("not implemented");
      this.__currentElement.getAttribute("fill") != "none" && this.__init_element(), this.__currentElement.nodeName === "path" && this.__currentElement.setAttribute("paint-order", "stroke"), this.__applyCurrentDefaultPath(), this.__applyStyleToCurrentElement("fill"), H != null && this.__currentElement.setAttribute("fill-rule", H), this._clip_path != null && this.__currentElement.setAttribute("clip-path", this._clip_path);
    }
    /**
      *  Adds a rectangle to the path.
      */
    rect(_, H, V, q) {
      isFinite(_ + H + V + q) && (this.moveTo(_, H), this.lineTo(_ + V, H), this.lineTo(_ + V, H + q), this.lineTo(_, H + q), this.lineTo(_, H), this.closePath());
    }
    /**
      * adds a rectangle element
      */
    fillRect(_, H, V, q) {
      isFinite(_ + H + V + q) && (this.beginPath(), this.rect(_, H, V, q), this.fill());
    }
    /**
      * Draws a rectangle with no fill
      * @param x
      * @param y
      * @param width
      * @param height
      */
    strokeRect(_, H, V, q) {
      isFinite(_ + H + V + q) && (this.beginPath(), this.rect(_, H, V, q), this.stroke());
    }
    /**
      * Clear entire canvas:
      * 1. save current transforms
      * 2. remove all the childNodes of the root g element
      */
    __clearCanvas() {
      empty$1(this.__defs), empty$1(this.__root), this.__root.appendChild(this.__defs), this.__currentElement = this.__root;
    }
    /**
      * "Clears" a canvas by just drawing a white rectangle in the current group.
      */
    clearRect(_, H, V, q) {
      if (!isFinite(_ + H + V + q))
        return;
      if (_ === 0 && H === 0 && V === this.width && q === this.height) {
        this.__clearCanvas();
        return;
      }
      const G = this.__createElement("rect", { x: _, y: H, width: V, height: q, fill: "#FFFFFF" }, !0);
      this._apply_transform(G), this.__root.appendChild(G);
    }
    roundRect(_, H, V, q, G) {
      throw new Error("not implemented");
    }
    /**
      * Adds a linear gradient to a defs tag.
      * Returns a canvas gradient object that has a reference to it's parent def
      */
    createLinearGradient(_, H, V, q) {
      if (!isFinite(_ + H + V + q))
        throw new Error("The provided double value is non-finite");
      const [G, J] = this._transform.apply(_, H), [Z, te] = this._transform.apply(V, q), ne = this.__createElement("linearGradient", {
        id: this._random_string(),
        x1: `${G}px`,
        x2: `${Z}px`,
        y1: `${J}px`,
        y2: `${te}px`,
        gradientUnits: "userSpaceOnUse"
      }, !1);
      return this.__defs.appendChild(ne), new CanvasGradient(ne, this);
    }
    /**
      * Adds a radial gradient to a defs tag.
      * Returns a canvas gradient object that has a reference to it's parent def
      */
    createRadialGradient(_, H, V, q, G, J) {
      if (!isFinite(_ + H + V + q + G + J))
        throw new Error("The provided double value is non-finite");
      const [Z, te] = this._transform.apply(_, H), [ne, ee] = this._transform.apply(q, G), ae = this.__createElement("radialGradient", {
        id: this._random_string(),
        cx: `${ne}px`,
        cy: `${ee}px`,
        r: `${J}px`,
        r0: `${V}px`,
        fx: `${Z}px`,
        fy: `${te}px`,
        gradientUnits: "userSpaceOnUse"
      }, !1);
      return this.__defs.appendChild(ae), new CanvasGradient(ae, this);
    }
    createConicGradient(_, H, V) {
      throw Error("not implemented");
    }
    /**
      * Parses the font string and returns svg mapping
      */
    __parseFont() {
      const _ = /^\s*(?=(?:(?:[-a-z]+\s*){0,2}(italic|oblique))?)(?=(?:(?:[-a-z]+\s*){0,2}(small-caps))?)(?=(?:(?:[-a-z]+\s*){0,2}(bold(?:er)?|lighter|[1-9]00))?)(?:(?:normal|\1|\2|\3)\s*){0,3}((?:xx?-)?(?:small|large)|medium|smaller|larger|[.\d]+(?:\%|in|[cem]m|ex|p[ctx]))(?:\s*\/\s*(normal|[.\d]+(?:\%|in|[cem]m|ex|p[ctx])))?\s*([-,\'\"\sa-z0-9]+?)\s*$/i, [, H, V, q, G, , J] = _.exec(this.font);
      return {
        style: H ?? "normal",
        size: G ?? "10px",
        family: J ?? "sans-serif",
        weight: q ?? "normal",
        decoration: V ?? "normal"
      };
    }
    /**
      * Fills or strokes text
      */
    __applyText(_, H, V, q) {
      const G = this.__parseFont(), J = this.__createElement("text", {
        "font-family": G.family,
        "font-size": G.size,
        "font-style": G.style,
        "font-weight": G.weight,
        "text-decoration": G.decoration,
        x: H,
        y: V,
        "text-anchor": getTextAnchor(this.textAlign),
        "dominant-baseline": getDominantBaseline(this.textBaseline)
      }, !0);
      J.appendChild(this.__document.createTextNode(_)), this._apply_transform(J), this.__currentElement = J, this.__applyStyleToCurrentElement(q);
      const Z = (() => {
        if (this._clip_path != null) {
          const te = this.__createElement("g");
          return te.setAttribute("clip-path", this._clip_path), te.appendChild(J), te;
        } else
          return J;
      })();
      this.__root.appendChild(Z);
    }
    /**
      * Creates a text element, in position x,y
      */
    fillText(_, H, V) {
      isFinite(H + V) && this.__applyText(_, H, V, "fill");
    }
    /**
      * Strokes text
      */
    strokeText(_, H, V) {
      isFinite(H + V) && this.__applyText(_, H, V, "stroke");
    }
    /**
      * No need to implement this for svg.
      */
    measureText(_) {
      return this.__ctx.font = this.font, this.__ctx.measureText(_);
    }
    arc(_, H, V, q, G, J = !1) {
      this.ellipse(_, H, V, V, 0, q, G, J);
    }
    ellipse(_, H, V, q, G, J, Z, te = !1) {
      if (!isFinite(_ + H + V + q + G + J + Z))
        return;
      if (V < 0 || q < 0)
        throw new DOMException("IndexSizeError, radius can't be negative");
      const ne = te ? Z - J : J - Z;
      J = J % (2 * Math.PI), Z = Z % (2 * Math.PI);
      const ee = new AffineTransform().translate(_, H).rotate(G), ae = V * Math.cos(J), oe = q * Math.sin(J), [se, ce] = ee.apply(ae, oe);
      this.lineTo(se, ce);
      const le = G * 180 / Math.PI, ue = te ? 0 : 1;
      if (Math.abs(J - Z) < 2 * float32_epsilon && !(Math.abs(ne) < 2 * float32_epsilon && ne < 0)) {
        const [de, fe] = this._transform.apply(se, ce), he = V * Math.cos(J + Math.PI), me = q * Math.sin(J + Math.PI), [pe, ye] = ee.apply(he, me), [ge, ve] = this._transform.apply(pe, ye);
        this.__addPathCommand(de, fe, `A ${V} ${q} ${le} 0 ${ue} ${ge} ${ve} A ${V} ${q} ${le} 0 ${ue} ${de} ${fe}`);
      } else {
        const de = V * Math.cos(Z), fe = q * Math.sin(Z), [he, me] = ee.apply(de, fe);
        let pe = Z - J;
        pe < 0 && (pe += 2 * Math.PI);
        const ye = te !== pe > Math.PI ? 1 : 0, [ge, ve] = this._transform.apply(he, me);
        this.__addPathCommand(ge, ve, `A ${V} ${q} ${le} ${ye} ${ue} ${ge} ${ve}`);
      }
    }
    clip(_, H) {
      let V = null;
      if (_ instanceof Path2D)
        V = _;
      else if (H == null)
        H = _;
      else
        throw new Error("invalid arguments");
      if (V != null)
        throw new Error("not implemented");
      const q = this.__createElement("clipPath"), G = this._random_string();
      this.__applyCurrentDefaultPath(), H != null && this.__currentElement.setAttribute("clip-rule", H), q.setAttribute("id", G), q.appendChild(this.__currentElement), this.__defs.appendChild(q), this._clip_path = `url(#${G})`;
    }
    /**
      * Draws a canvas, image or mock context to this canvas.
      * Note that all svg dom manipulation uses node.childNodes rather than node.children for IE support.
      * http://www.whatwg.org/specs/web-apps/current-work/multipage/the-canvas-element.html#dom-context-2d-drawimage
      */
    drawImage(_, ...H) {
      let V, q, G, J, Z, te, ne, ee;
      if (H.length == 2) {
        if ([V, q] = H, !isFinite(V + q))
          return;
        Z = 0, te = 0;
        const [se, ce] = width_height(_);
        G = ne = se, J = ee = ce;
      } else if (H.length == 4) {
        if ([V, q, G, J] = H, !isFinite(V + q + G + J))
          return;
        Z = 0, te = 0;
        const [se, ce] = width_height(_);
        ne = se, ee = ce;
      } else if (H.length === 8) {
        if ([Z, te, ne, ee, V, q, G, J] = H, !isFinite(Z + te + ne + ee + V + q + G + J))
          return;
      } else
        throw new Error(`Inavlid number of arguments passed to drawImage: ${arguments.length}`);
      const ae = this.__root, oe = this._transform.clone().translate(V, q);
      if (_ instanceof e0 || _ instanceof SVGSVGElement) {
        const ce = (_ instanceof SVGSVGElement ? _ : _.get_svg()).cloneNode(!0);
        let le;
        oe.is_identity && this.globalAlpha == 1 && this._clip_path == null ? le = ae : (le = this.__createElement("g"), oe.is_identity || this._apply_transform(le, oe), this.globalAlpha != 1 && le.setAttribute("opacity", `${this.globalAlpha}`), this._clip_path != null && le.setAttribute("clip-path", this._clip_path), ae.appendChild(le));
        for (const ue of [...ce.childNodes])
          if (ue instanceof SVGDefsElement) {
            for (const de of [...ue.childNodes])
              if (de instanceof Element) {
                const fe = de.getAttribute("id");
                this.__ids.add(fe), this.__defs.appendChild(de.cloneNode(!0));
              }
          } else
            le.appendChild(ue.cloneNode(!0));
      } else if (_ instanceof HTMLImageElement || _ instanceof SVGImageElement) {
        const se = this.__createElement("image");
        if (se.setAttribute("width", `${G}`), se.setAttribute("height", `${J}`), se.setAttribute("preserveAspectRatio", "none"), this.globalAlpha != 1 && se.setAttribute("opacity", `${this.globalAlpha}`), Z != 0 || te != 0 || ne !== _.width || ee !== _.height) {
          const le = this.__document.createElement("canvas");
          le.width = G, le.height = J, le.getContext("2d").drawImage(_, Z, te, ne, ee, 0, 0, G, J), _ = le;
        }
        this._apply_transform(se, oe);
        const ce = _ instanceof HTMLCanvasElement ? _.toDataURL() : _.getAttribute("src");
        if (se.setAttribute("href", ce), this._clip_path != null) {
          const le = this.__createElement("g");
          le.setAttribute("clip-path", this._clip_path), le.appendChild(se), ae.appendChild(le);
        } else
          ae.appendChild(se);
      } else if (_ instanceof HTMLCanvasElement) {
        const se = this.__createElement("image");
        se.setAttribute("width", `${G}`), se.setAttribute("height", `${J}`), se.setAttribute("preserveAspectRatio", "none"), this.globalAlpha != 1 && se.setAttribute("opacity", `${this.globalAlpha}`);
        const ce = this.__document.createElement("canvas");
        ce.width = G, ce.height = J;
        const le = ce.getContext("2d");
        if (le.imageSmoothingEnabled = !1, le.drawImage(_, Z, te, ne, ee, 0, 0, G, J), _ = ce, this._apply_transform(se, oe), se.setAttribute("href", _.toDataURL()), this._clip_path != null) {
          const ue = this.__createElement("g");
          ue.setAttribute("clip-path", this._clip_path), ue.appendChild(se), ae.appendChild(ue);
        } else
          ae.appendChild(se);
      }
    }
    /**
      * Generates a pattern tag
      */
    createPattern(_, H) {
      const V = this.__document.createElementNS("http://www.w3.org/2000/svg", "pattern"), q = this._random_string(), [G, J] = width_height(_);
      if (V.setAttribute("id", q), V.setAttribute("width", `${G}`), V.setAttribute("height", `${J}`), V.setAttribute("patternUnits", "userSpaceOnUse"), _ instanceof HTMLCanvasElement || _ instanceof HTMLImageElement || _ instanceof SVGImageElement) {
        const Z = this.__document.createElementNS("http://www.w3.org/2000/svg", "image"), te = _ instanceof HTMLCanvasElement ? _.toDataURL() : _.getAttribute("src");
        Z.setAttribute("href", te), V.appendChild(Z), this.__defs.appendChild(V);
      } else if (_ instanceof e0) {
        for (const Z of [..._.__root.childNodes])
          Z instanceof SVGDefsElement || V.appendChild(Z.cloneNode(!0));
        this.__defs.appendChild(V);
      } else if (_ instanceof SVGSVGElement) {
        for (const Z of [..._.childNodes])
          Z instanceof SVGDefsElement || V.appendChild(Z.cloneNode(!0));
        this.__defs.appendChild(V);
      } else
        throw new Error("unsupported");
      return new CanvasPattern(V, this);
    }
    getLineDash() {
      const { lineDash: _ } = this;
      return isString$2(_) ? _.split(",").map((H) => parseInt(H)) : _ ?? [];
    }
    setLineDash(_) {
      _.length > 0 ? this.lineDash = _.join(",") : this.lineDash = null;
    }
    getTransform() {
      return this._transform.to_DOMMatrix();
    }
    setTransform(..._) {
      let H;
      isNumber$3(_[0]) ? H = new DOMMatrix(_) : _[0] instanceof DOMMatrix ? H = _[0] : H = new DOMMatrix(Object.values(_[0] == null)), this._transform = AffineTransform.from_DOMMatrix(H);
    }
    resetTransform() {
      this._transform = new AffineTransform();
    }
    isPointInPath(..._) {
      throw new Error("not implemented");
    }
    isPointInStroke(..._) {
      throw new Error("not implemented");
    }
    createImageData(..._) {
      throw new Error("not implemented");
    }
    getImageData(_, H, V, q) {
      throw new Error("not implemented");
    }
    putImageData(..._) {
      throw new Error("not implemented");
    }
    drawFocusIfNeeded(..._) {
      throw new Error("not implemented");
    }
    scrollPathIntoView(..._) {
      throw new Error("not implemented");
    }
  };
  mt(e0, "__name__", "SVGRenderingContext2D"), //__currentElementsToStyle: {element: SVGElement, children: SVGElement[]} | null = null
  mt(e0, "__random", random$1);
  let SVGRenderingContext2D = e0;
  class CanvasLayer {
    constructor(_, H) {
      mt(this, "backend");
      mt(this, "hidpi");
      mt(this, "_canvas");
      mt(this, "_ctx");
      mt(this, "_el");
      mt(this, "pixel_ratio", 1);
      mt(this, "bbox", new BBox$1());
      switch (this.backend = _, this.hidpi = H, _) {
        case "webgl":
        case "canvas": {
          this._el = this._canvas = canvas({ class: "bk-layer" });
          const V = this.canvas.getContext("2d");
          if (V == null)
            throw new Error("unable to obtain 2D rendering context");
          this._ctx = V, H && (this.pixel_ratio = devicePixelRatio);
          break;
        }
        case "svg": {
          const V = new SVGRenderingContext2D();
          this._ctx = V, this._canvas = V.get_svg(), this._el = div({ class: "bk-layer" }), this._el.attachShadow({ mode: "open" }).appendChild(this._canvas);
          break;
        }
      }
      this._ctx.layer = this;
    }
    get canvas() {
      return this._canvas;
    }
    get ctx() {
      return this._ctx;
    }
    get el() {
      return this._el;
    }
    resize(_, H) {
      if (this.bbox.width == _ && this.bbox.height == H)
        return;
      this.bbox = new BBox$1({ left: 0, top: 0, width: _, height: H });
      const { target: V } = this;
      V.width = _ * this.pixel_ratio, V.height = H * this.pixel_ratio;
    }
    get target() {
      return this._ctx instanceof SVGRenderingContext2D ? this._ctx : this.canvas;
    }
    undo_transform(_) {
      const { ctx: H } = this, V = H.getTransform();
      H.resetTransform();
      try {
        _(H);
      } finally {
        H.setTransform(V);
      }
    }
    prepare() {
      const { ctx: _, hidpi: H, pixel_ratio: V } = this;
      _.save(), H && (_.scale(V, V), _.translate(0.5, 0.5)), this.clear();
    }
    clear() {
      const { x: _, y: H, width: V, height: q } = this.bbox;
      this.ctx.clearRect(_, H, V, q);
    }
    finish() {
      this.ctx.restore();
    }
    to_blob() {
      const { _canvas: _ } = this;
      if (_ instanceof HTMLCanvasElement)
        return new Promise((H, V) => {
          _.toBlob((q) => q != null ? H(q) : V(), "image/png");
        });
      {
        const V = this._ctx.get_serialized_svg(!0), q = new Blob([V], { type: "image/svg+xml" });
        return Promise.resolve(q);
      }
    }
  }
  mt(CanvasLayer, "__name__", "CanvasLayer");
  const ui_css = ":host{position:relative;}", StylesLike = Or$1(Dict(Nullable(Str)), Ref(Styles$1)), StyleSheets = List$2(Or$1(Ref(StyleSheet), Str, Dict(StylesLike))), CSSVariables = Dict(Ref(Node$4)), { round: round$5, floor: floor$1 } = Math;
  class UIElementView extends DOMComponentView {
    constructor() {
      super(...arguments);
      mt(this, "_display", new InlineStyleSheet$1());
      mt(this, "style", new InlineStyleSheet$1());
      mt(this, "_bbox", new BBox$1());
      mt(this, "_resize_observer");
      mt(this, "_context_menu", null);
      mt(this, "_is_displayed", !1);
    }
    *_css_classes() {
      yield* super._css_classes(), yield* this.model.css_classes;
    }
    *_css_variables() {
      yield* super._css_variables();
      for (const [H, V] of entries$2(this.model.css_variables)) {
        const q = this.resolve_coordinate(V);
        isNumber$3(q) && (yield [H, `${q}px`]);
      }
    }
    *_stylesheets() {
      yield* super._stylesheets(), yield this.style, yield this._display, yield* this._computed_stylesheets();
    }
    *_computed_stylesheets() {
      for (const H of this.model.stylesheets)
        H instanceof StyleSheet ? yield H.underlying() : yield new InlineStyleSheet$1(H);
    }
    stylesheets() {
      return [...super.stylesheets(), ui_css];
    }
    update_style() {
      this.style.clear();
    }
    box_sizing() {
      return {
        width_policy: "auto",
        height_policy: "auto",
        width: null,
        height: null,
        aspect_ratio: null
      };
    }
    get bbox() {
      return this._bbox;
    }
    update_bbox() {
      return this._update_bbox();
    }
    _update_bbox() {
      const H = (() => {
        if (this.el.isConnected) {
          if (this.el.offsetParent != null)
            return !0;
          {
            const { position: G, display: J } = getComputedStyle(this.el);
            return G == "fixed" && J != "none";
          }
        } else return !1;
      })(), V = H ? (() => {
        const G = this.el.getBoundingClientRect(), { left: J, top: Z } = (() => {
          if (this.parent != null) {
            const te = this.parent.el.getBoundingClientRect();
            return {
              left: G.left - te.left,
              top: G.top - te.top
            };
          } else
            return { left: 0, top: 0 };
        })();
        return new BBox$1({
          left: round$5(J),
          top: round$5(Z),
          width: floor$1(G.width),
          height: floor$1(G.height)
        });
      })() : new BBox$1(), q = !this._bbox.equals(V);
      return this._bbox = V, this._is_displayed = H, q;
    }
    initialize() {
      super.initialize(), this._resize_observer = new ResizeObserver((H) => this.after_resize()), this._resize_observer.observe(this.el, { box: "border-box" });
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { context_menu: H } = this.model;
      H != null && (this._context_menu = await build_view(H, { parent: this }));
    }
    connect_signals() {
      super.connect_signals();
      const { visible: H, styles: V, css_classes: q, css_variables: G, stylesheets: J } = this.model.properties;
      this.on_change(H, () => this._update_visible()), this.on_change(V, () => this._update_styles()), this.on_change(q, () => this._update_css_classes()), this.on_transitive_change(G, () => this._update_css_variables()), this.on_change(J, () => this._update_stylesheets()), this.el.addEventListener("contextmenu", (Z) => this.show_context_menu(Z));
    }
    get_context_menu(H) {
      return this._context_menu;
    }
    show_context_menu(H) {
      if (!H.shiftKey) {
        const V = this.el.getBoundingClientRect(), q = H.x - V.x, G = H.y - V.y, J = this.get_context_menu({ x: q, y: G });
        J != null && (H.stopPropagation(), H.preventDefault(), J.show({ x: q, y: G }));
      }
    }
    remove() {
      var H;
      this._resize_observer.disconnect(), (H = this._context_menu) == null || H.remove(), super.remove();
    }
    _after_resize() {
    }
    after_resize() {
      this.update_bbox() && this._after_resize(), this.finish();
    }
    render() {
      super.render(), this._apply_styles(), this._apply_visible();
    }
    _after_render() {
    }
    after_render() {
      super.after_render(), this.update_style(), this.update_bbox(), this._after_render(), this.is_displayed || this.finish();
    }
    get is_displayed() {
      return this._is_displayed;
    }
    _apply_visible() {
      this.model.visible ? this._display.clear() : this._display.replace(":host { display: none !important; }");
    }
    _apply_styles() {
      apply_styles(this.el.style, this.model.styles);
    }
    _update_visible() {
      this._apply_visible();
    }
    _update_styles() {
      this.el.removeAttribute("style"), this._apply_styles();
    }
    export(H = "auto", V = !0) {
      const q = H == "auto" || H == "png" ? "canvas" : "svg", G = new CanvasLayer(q, V), { width: J, height: Z } = this.bbox;
      return G.resize(J, Z), G;
    }
    serializable_state() {
      return { ...super.serializable_state(), bbox: this.bbox };
    }
    resolve_symbol(H) {
      const V = this.bbox.resolve(H.symbol), { offset: q } = H;
      if (isNumber$3(V))
        return V + q;
      {
        const { x: G, y: J } = V;
        return { x: G + q, y: J + q };
      }
    }
  }
  mt(UIElementView, "__name__", "UIElementView");
  const gg = class gg extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(gg, "__name__", "UIElement"), gg.define(({ Bool: _, List: H, Str: V, AnyRef: q }) => ({
    visible: [_, !0],
    css_classes: [H(V), []],
    css_variables: [CSSVariables, {}],
    styles: [StylesLike, {}],
    stylesheets: [StyleSheets, []],
    context_menu: [Nullable(q()), null]
  }));
  let UIElement = gg;
  class MenuItem extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(MenuItem, "__name__", "MenuItem");
  class Callback extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(Callback, "__name__", "Callback");
  const IconLike = Or$1(ToolIcon, Regex(/^--/), Regex(/^\./), Regex(/^data:image/)), yg = class yg extends MenuItem {
    constructor(_) {
      super(_);
    }
  };
  mt(yg, "__name__", "ActionItem"), yg.define(({ Bool: _, Str: H, Nullable: V, AnyRef: q, Ref: G }) => ({
    icon: [V(IconLike), null],
    label: [H],
    tooltip: [V(H), null],
    shortcut: [V(H), null],
    menu: [V(q()), null],
    disabled: [_, !1],
    action: [V(G(Callback)), null]
  }));
  let ActionItem = yg;
  const vg = class vg extends ActionItem {
    constructor(_) {
      super(_);
    }
  };
  mt(vg, "__name__", "CheckableItem"), vg.define(({ Bool: _ }) => ({
    checked: [_, !1]
  }));
  let CheckableItem = vg;
  class DividerItem extends MenuItem {
    constructor(_) {
      super(_);
    }
  }
  mt(DividerItem, "__name__", "DividerItem");
  const item$1 = "bk-item", divider$3 = "bk-divider", check = "bk-check", icon$1 = "bk-icon", label$1 = "bk-label", shortcut = "bk-shortcut", chevron$1 = "bk-chevron", disabled$3 = "bk-disabled", menu$1 = "bk-menu", checkable = "bk-checkable", checked = "bk-checked", menus_css$1 = ":host{position:absolute;display:inline-grid;grid-template-columns:repeat(5, max-content);grid-template-rows:auto;user-select:none;-webkit-user-select:none;cursor:pointer;width:auto;height:auto;z-index:var(--bokeh-top-level);font-size:var(--font-size);background-color:#fff;border:1px solid #ccc;border-radius:var(--border-radius);box-shadow:2px 4px 8px rgba(0, 0, 0, 0.175);}.bk-item,.bk-divider{position:relative;display:grid;grid-template-columns:subgrid;grid-column:span 5;align-items:center;}.bk-check{grid-column:1;}.bk-icon{grid-column:2;}.bk-label{grid-column:3;}.bk-shortcut{grid-column:4;}.bk-chevron{grid-column:5;}.bk-divider{cursor:default;overflow:hidden;background-color:#e5e5e5;height:1px;margin:1px 5px;}.bk-item{padding:0.5em;}.bk-item:hover{background-color:#f9f9f9;}.bk-item:focus,.bk-item:focus-visible{outline:1px dotted var(--active-tool-highlight);outline-offset:-1px;}.bk-item::-moz-focus-inner{border:0;}.bk-item.bk-disabled{color:gray;}.bk-item.bk-disabled .bk-icon{color:var(--bokeh-icon-color-disabled);}.bk-item:first-of-type{border-top-left-radius:var(--border-radius);border-top-right-radius:var(--border-radius);}.bk-item:last-of-type{border-bottom-left-radius:var(--border-radius);border-bottom-right-radius:var(--border-radius);}.bk-icon{position:relative;width:16px;height:16px;mask-size:100% 100%;-webkit-mask-size:100% 100%;mask-position:center center;-webkit-mask-position:center center;mask-repeat:no-repeat;-webkit-mask-repeat:no-repeat;background-size:100% 100%;background-origin:border-box;background-position:center center;background-repeat:no-repeat;}.bk-label{padding:0 0.5em;}.bk-shortcut{text-align:right;font-size:90%;color:gray;}.bk-chevron{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-caret-right);-webkit-mask-image:var(--bokeh-icon-caret-right);mask-size:100% 100%;-webkit-mask-size:100% 100%;mask-position:center center;-webkit-mask-position:center center;mask-repeat:no-repeat;-webkit-mask-repeat:no-repeat;width:12px;height:12px;}.bk-item:not(.bk-menu) .bk-chevron{display:none;}.bk-item.bk-checkable .bk-check{width:16px;height:16px;}.bk-item.bk-checkable.bk-checked .bk-check{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-check);-webkit-mask-image:var(--bokeh-icon-check);mask-size:100% 100%;-webkit-mask-size:100% 100%;mask-position:center center;-webkit-mask-position:center center;mask-repeat:no-repeat;-webkit-mask-repeat:no-repeat;}", tool_icon_copy = "bk-tool-icon-copy", tool_icon_replace_mode = "bk-tool-icon-replace-mode", tool_icon_toggle_mode = "bk-tool-icon-toggle-mode", tool_icon_append_mode = "bk-tool-icon-append-mode", tool_icon_intersect_mode = "bk-tool-icon-intersect-mode", tool_icon_subtract_mode = "bk-tool-icon-subtract-mode", tool_icon_xor_mode = "bk-tool-icon-xor-mode", tool_icon_invert_selection = "bk-tool-icon-invert-selection", tool_icon_clear_selection = "bk-tool-icon-clear-selection", tool_icon_box_select = "bk-tool-icon-box-select", tool_icon_x_box_select = "bk-tool-icon-x-box-select", tool_icon_y_box_select = "bk-tool-icon-y-box-select", tool_icon_box_zoom = "bk-tool-icon-box-zoom", tool_icon_x_box_zoom = "bk-tool-icon-x-box-zoom", tool_icon_y_box_zoom = "bk-tool-icon-y-box-zoom", tool_icon_auto_box_zoom = "bk-tool-icon-auto-box-zoom", tool_icon_zoom_in = "bk-tool-icon-zoom-in", tool_icon_zoom_out = "bk-tool-icon-zoom-out", tool_icon_help = "bk-tool-icon-help", tool_icon_hover = "bk-tool-icon-hover", tool_icon_crosshair = "bk-tool-icon-crosshair", tool_icon_lasso_select = "bk-tool-icon-lasso-select", tool_icon_pan = "bk-tool-icon-pan", tool_icon_x_pan = "bk-tool-icon-x-pan", tool_icon_y_pan = "bk-tool-icon-y-pan", tool_icon_range = "bk-tool-icon-range", tool_icon_polygon_select = "bk-tool-icon-polygon-select", tool_icon_redo = "bk-tool-icon-redo", tool_icon_reset = "bk-tool-icon-reset", tool_icon_save = "bk-tool-icon-save", tool_icon_tap_select = "bk-tool-icon-tap-select", tool_icon_undo = "bk-tool-icon-undo", tool_icon_wheel_pan = "bk-tool-icon-wheel-pan", tool_icon_wheel_zoom = "bk-tool-icon-wheel-zoom", tool_icon_box_edit = "bk-tool-icon-box-edit", tool_icon_freehand_draw = "bk-tool-icon-freehand-draw", tool_icon_poly_draw = "bk-tool-icon-poly-draw", tool_icon_point_draw = "bk-tool-icon-point-draw", tool_icon_poly_edit = "bk-tool-icon-poly-edit", tool_icon_line_edit = "bk-tool-icon-line-edit", tool_icon_settings = "bk-tool-icon-settings", tool_icon_unknown = "bk-tool-icon-unknown", tool_icon_fullscreen = "bk-tool-icon-fullscreen", tool_icon_chevron_down = "bk-tool-icon-chevron-down", icons_css = ':host{--bokeh-icon-color:#a1a6a9;--bokeh-icon-color-disabled:#d4d9db;}.bk-tool-icon-copy{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-copy);-webkit-mask-image:var(--bokeh-icon-copy);}.bk-tool-icon-replace-mode{background-image:var(--bokeh-icon-replace-mode, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxokK3gniQAAAHpJREFUWMNjXLhsJcNAAiaGAQajDhhwB7DgEP+PxmeksvjgDwFcLmYkUh2hkBj8IcBIZXsYh1w2/I8v3sgAOM0bLYhGc8GgrwuICgldfQO88pcvXvg/aOuCUQeM5oLRuoCFCJcTbOMh5XOiW0JDNhdQS3y0IBp1ABwAAF8KGrhC1Eg6AAAAAElFTkSuQmCC"));}.bk-tool-icon-toggle-mode{background-image:var(--bokeh-icon-toggle-mode, url("data:image/png;base64, "));}.bk-tool-icon-append-mode{background-image:var(--bokeh-icon-append-mode, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxkZWD04WwAAAB1pVFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAAAoUlEQVRYw+1WQQ6AIAwrhO8Y/bIXEz9jIMSDr8ETCUEPQzA4pMeFLKNbu4l5WR0CDOMEALBGIzMuQIBEZQjPgP9JLjwTfBjY9sO9lZsFA9IafZng3BlIyVefgd8XQFZBAWe8jfNxwsDhir6rzoCiPiy1K+J8/FRQemv2XfAdFcQ9znU4Viqg9ta1qYJ+D1BnAIBrkgGVOrXNqUA9rbyZm/AEzFh4jEeY/soAAAAASUVORK5CYII="));}.bk-tool-icon-intersect-mode{background-image:var(--bokeh-icon-intersect-mode, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxkrkOpp2wAAAPhJREFUWMPtV1EKwjAMTUavI3oawR/vtn5srJdREfzwMvHHQlcT2mpdMzFfWxiP5r2+JMN+mAiCOB72CABgR1cln4oOGocJnuMTSxWk8jMm7OggYkYXA9gPE3uyd8NXHONJ+eYMdE/NqCJmEZ5ZqlJJ4sUksKN7cYSaPoCZFWR1QI+Xm1fBACU63Cw22x0AAJxudwrffVwvZ+JmQdAHZkw0d4EpAMCw8k87pMdbnwtizQumJYv3nwV6XOA1qbUT/oQLUJgFRbsiNwFVucBIlyR3p0tdMp+XmFjfLKi1LatyAXtCRjPWBdL3Ke3VuACJKFfDr/xFN2fgAR/Go0qaLlmEAAAAAElFTkSuQmCC"));}.bk-tool-icon-subtract-mode{background-image:var(--bokeh-icon-subtract-mode, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxgsF5XNOQAAAB1pVFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAABFUlEQVRYw9VWUQqDMAxNpWfxQxD1MoP97G7zQ5mH2RTZYLtM9lWoMbXtxLXNX4OG9r28l4hrd0PQoqxqAACYpxH25C/nkwCHyCBwSPoS09k1T5Fo+4EiExcC4v584xGFmyIXHBLRISAVZyZufUPVa4rcrwmPDgr93ylo+2GliLRUYHK6th/o/6r7nfLpqaCsagEA8Hh9FmcNKeRmgeYDC+SCq0B6FFi8/BcV6BdR9cL3gCv3ijPKOacsn3rBEcjmaVxpfGcg4wHxzgJJnc6241Hn23DERFRAu1bNcWa3Q0uXi62XR6sCaWoSejbtdLYmU3kTEunNgj0bUbQqYG/IcMaqwPS9jftoVCAQ0ZVDJwf0zQdH4AsyW6fpQu4YegAAAABJRU5ErkJggg=="));}.bk-tool-icon-xor-mode{background-image:var(--bokeh-icon-xor-mode, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAQAAADZc7J/AAAAAmJLR0QA/4ePzL8AAACXSURBVEjH3VXBCcAwCDxLxgl0oM7nQEL3sa9ASLQIqS3UT4SIOfXOECsAADsAwb1/ECbbsGilvdKfli9OgmUEdGo8uKb0QIZ6PUvrwfcJzCn0zGtM9fx1BKzzFMTgvRf3jBZGs1TnxeVNwUIxxsqrWmBNYmJptewBBuZMwas3uiP+sA+8i8i2rpSlRg7+FaxZCHpFpu/EC1y3NtihGuwKAAAAAElFTkSuQmCC"));}.bk-tool-icon-invert-selection{background-image:var(--bokeh-icon-invert-selection, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAQAAADZc7J/AAAAAmJLR0QA/4ePzL8AAAC2SURBVEjH7VVJEoQwCGysPMeqPCjv40FU+R/moCbMmEWNenFygUug6aYTmhRdZwBeX8ClVJboCRDN5SN4zc8gkBaCtaONKZ+vB0JEcT0HtmvtBALS+rnatAEAF/KcCroThXKZRAP0J3IUT25bpG33m1TwBCCqQOY9IKtCDsVkiOxQQdR6we2dlR+ws7FMYQObKsgJFUyB8Wtev2W+hYB1dtoxFa4mMaJQqb6PDRmPeIH+X1t/gQ8PhzjuVbw+wwAAAABJRU5ErkJggg=="));}.bk-tool-icon-clear-selection{background-image:var(--bokeh-icon-clear-selection, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUGEhcuan3d3wAAAoRJREFUWMPtlzFP3EAQhd+b3TNSzg0N5TWXLkJQUUaKhIQ4fgP/g5ArrriE/I3opEgRrZtIVJR0FJQ010SioUmEZHtnUpwN9gWHGA5BJCy58MraffvmfZ41v3z9hqe8BE98vQh4cgG+Ydzmnrng8efvQJNi/uN7dznx/B3ggtfhf4ehNdUttRzBDIm/2VTiiWCG1HK0nc+3UWtq8BQIiEEakEQOADBIA4QCQmBqoHBhFNR27ikQSmGdYCdTqCpEHMDZmEKRWUBEv1gBDg5SzRJnpopILWICgWuRYflLamuzxB2BmtYqSRIka5VWU8QduXO+1hRc5YZu5GAwmP2ZJzND0IBu5HCV2+NQcAhAVRsnC2IbPzPdSjzd6to6VtfWkXi6YLaVWr7xoAwkfpb8MnC3SH7rKSMBe4M0jA/OTicFIbtCGRIyNbURhcf3ErCd6YwA1m0HgAxhw1NGQnlXBHG4kylVlSJuH0RfIP2CkL2I/qS1gIAAQiBl1QwFggIHtyxgrxK5PgyfC0JWKoT0HLh8LwoietB4TYKaIl7yeNURxB05UtMxDOcVQlZIrlRKdK6m47gjR/fuBRQihyLArtNeJD50Izcx2Eczu7iFkIug4VM3cpOr3MKDekFED0fWUHv9Zq0kpLnridjhY3XDg7NTN0jDrhO3X7O9Wg7wwyANu4mnayNg3gmbu0tCNoUyBNGv2l4rB9EXynA7082FOxAQLhU6rQVO9T2AvWowFToNCJcPORGxIRcnpjZSKATSU9NxvOQnAPArDSaQoUKnNI4iufkGtD4P3EHIcWZhz4HLceSOyrR3Izf5memPAL2cX3yhAkonysZVaWLBkd9dw1Ivv2a/AYPkK+ty1U1DAAAAAElFTkSuQmCC"));}.bk-tool-icon-box-select{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-box-select);-webkit-mask-image:var(--bokeh-icon-box-select);}.bk-tool-icon-x-box-select{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-x-box-select);-webkit-mask-image:var(--bokeh-icon-x-box-select);}.bk-tool-icon-y-box-select{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-y-box-select);-webkit-mask-image:var(--bokeh-icon-y-box-select);}.bk-tool-icon-box-zoom{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-box-zoom);-webkit-mask-image:var(--bokeh-icon-box-zoom);}.bk-tool-icon-x-box-zoom{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-x-box-zoom);-webkit-mask-image:var(--bokeh-icon-x-box-zoom);}.bk-tool-icon-y-box-zoom{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-y-box-zoom);-webkit-mask-image:var(--bokeh-icon-y-box-zoom);}.bk-tool-icon-auto-box-zoom{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-auto-box-zoom);-webkit-mask-image:var(--bokeh-icon-auto-box-zoom);}.bk-tool-icon-zoom-in{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-zoom-in);-webkit-mask-image:var(--bokeh-icon-zoom-in);}.bk-tool-icon-zoom-out{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-zoom-out);-webkit-mask-image:var(--bokeh-icon-zoom-out);}.bk-tool-icon-help{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-help);-webkit-mask-image:var(--bokeh-icon-help);}.bk-tool-icon-hover{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-hover);-webkit-mask-image:var(--bokeh-icon-hover);}.bk-tool-icon-crosshair{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-crosshair);-webkit-mask-image:var(--bokeh-icon-crosshair);}.bk-tool-icon-lasso-select{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-lasso-select);-webkit-mask-image:var(--bokeh-icon-lasso-select);}.bk-tool-icon-pan{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-pan);-webkit-mask-image:var(--bokeh-icon-pan);}.bk-tool-icon-x-pan{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-x-pan);-webkit-mask-image:var(--bokeh-icon-x-pan);}.bk-tool-icon-y-pan{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-y-pan);-webkit-mask-image:var(--bokeh-icon-y-pan);}.bk-tool-icon-range{background-image:var(--bokeh-icon-range, url("data:image/png;base64,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"));}.bk-tool-icon-polygon-select{background-image:var(--bokeh-icon-polygon-select, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEjc1OfiVKAAAAe1JREFUWMPt1r9rU1EUB/DPK0XbqphFHETo4OCiFhwF0V1KHbRSROLqon+AUMVRRFBwEbRFMBiV+mMW/wIxi5OD1kERRVKRJHUwLvfBTZrU5OWBGXLgQu7Jfe98z/ec7z0vKa88b2q1BDtRHdAPBaylm1NzsxsOjPnPNt6WSWprbft+/c3I3zOAjhT1Y4+fvcjEQJIXnVECSa+AhqIHqlHH5lWCZoe+Gk4GRgDG86j9SAUdlDBSQaZhlOkuHyoVdJmsw98D1S5fM4NYM1LCpqM+Lwa240oLgmZzpVZvzKT75VLZcqksSZKWlQeAy/iORVwIvh31xvotvK7VG3Px4aWHj3Jl4C2uYSvq+Bn8v6LLbaVWb9zsBiKLCvbiNG7gLm7jAYqbPHMJMziZ9lsKoh8GtqCEVVzHftwJn+TFHp4/hg8BSCYVfMOZoPEv2NZGdy9WCGUr9toDR3E2/H4V6nwRe/BmgN65H1ZhvMuB3XiKIyFoGefwO6ysVkUlrNUNsyAK/jli533Q+Y8cJFvAeXyMS1CI/jiMr/gUtD2LQwMGr4R3p7bY3oQHQ5b38CT4D2AXXg6YcQXHpyYnlqKsi5iOAVSwL9zd7zJ09r+Cpwq72omFMazjT9Dnibym0dTkRDUKrrgwH7MwXVyYB38BstaGDfLUTsgAAAAASUVORK5CYII="));}.bk-tool-icon-redo{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-redo);-webkit-mask-image:var(--bokeh-icon-redo);}.bk-tool-icon-reset{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-reset);-webkit-mask-image:var(--bokeh-icon-reset);}.bk-tool-icon-save{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-save);-webkit-mask-image:var(--bokeh-icon-save);}.bk-tool-icon-tap-select{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-tap-select);-webkit-mask-image:var(--bokeh-icon-tap-select);}.bk-tool-icon-undo{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-undo);-webkit-mask-image:var(--bokeh-icon-undo);}.bk-tool-icon-wheel-pan{background-image:var(--bokeh-icon-wheel-pan, url("data:image/png;base64,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"));}.bk-tool-icon-wheel-zoom{background-image:var(--bokeh-icon-wheel-zoom, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEgskILvMJQAABTtJREFUWMPdl1+MXVUVxn/fPvf2zrSFmUKnoBCUdjRoVaIxEpO2JhilMYBCtBQS2hejpg1Uo2NUrIFAoyGmtiE+GHwQGtvQJhqDmKYRBv+URFsFDNCSptH60DJTO3dKnX/33rM/H7rvsDu9M20fDMaVnGTvtb69z7fWXmvtc/TEzqd4OyXwNsv/FwFJQVI/sA14SZKRLOlPkr5TrVYXHz70quYkEEK4TtI2YAgYkrQthHDdhV5uuw+43/ZrwCbgRttgY/tjtrc0m83X3/f+D6ydnJhYcB4BSZcBA7aP2d4ELAGW2N5k+xgwkDB0IH19CGGH7R8B1aQeAf4KvAw0ku4K2zu7uru3ApdPEyiKohd4TNKjtjt5h6RHgccSNrddbvuHtm9Jqoak7xVF8WFgdavV+pSk5cCObNmXgK++85prCj3z28HKqZMnH7D9YAY4BvwujT8BvCuL1INX9vVt+dfwcCvNb7f9q2RuSfrGvWu/sL2Nf3LX7pzvj4ENSGBPVarVd4fRkZFltjdmoMGiKO4IIWwIIWwoiuIOYDDzeOPoyMiyFLkum7WJCMDztrcrTTrIRuAQZ6NcK1utL4dWq/VZoC8BhqvV6l1lWb4YYxyLMY6VZflitVq9CxhOmL60hhCKeYiV7WMKIXw9jT1HpXw3c+bOAKzOjJubzebJrKQCQLPZPClpc7bP6rMYKtjXth2OMf7tIkr11Wz8oQDc1Fb09vY+kQw1YAuwJY2nbUluAnCWpKkaFl6IQIzxivaR2SYA89sJVK/Xp2x32R6w/a30DNjuqtfrU0ArYecDCEqgLqm94T0dEm9mBG7PxkdDlkBnkhebgIezNQ8nHcCZPL9ijE1Jf/bZZoPtzbavmqNZLbf9tSxq+yoduuJ+SZ+zXSZyBXCqU+d8fvC5yRUrV+0G2j3g2hDCLyXd/+Su3QdnvP/zCuH72LWsgf2k0oHlH2c2odlkxcpVEdgr6aDtjyb8x20/J+mA7T9I6rL9SWA5dne2/GdXLl58qNJh398An85yTMA+4DOz8Dgu6Zu2dwJXJ91ltm8Gbp7Fgb+EEB4aHhpq5CEtACqVyr3AC0AlPS8k3TSmQ2YPhhBuS/1/LpmS9JTtNTHGfwBU2uUALARotVqniqJYH2Pck85pfavVaufAwnQvnHc0McaDKVptebN94QAnJB0EdtjekydyZXqjs/0ZgLIs/w6sy8bnYGYJ63pgERKC05JutT1kOwITwL9tvzlzUQUYB+Zjs2DBgu6xsbGJZHstByZbezregcBXeCsEz1bnzXt5anLyzLq71zDLxTRdVgemdx0fv2e2w5thO5DbiqL4oKT3ZKpnpyYnz+SY2ZpTAPZmJfdIrVZbNBNUq9UW2X4kU+2dcf53Aj1pj2PA7y/6m1DS00A9za9uNBq7iqJYBuoGdRdFsazRaOzKSqye1rTbaa/tlbYrqXQP2X4FIA9/J1l39xrC0v7+w5IeB8XkwS1lWe6TGJAYKMty31tfO4qSHl/a3384I3CDpI+kzC4lnRfrue6GytEjR8oQwlY73gC0L4qlth/q0M1/LYWtR48cKQF6enrC6dOnVwGLEpnxnp7en4+O1i/tszzGOCTpPmB7ahb57QUwBWyXdF+McWg6MScmuoA8OX8xOlpvXGz422XYTsB/SnpA0h7bX5R0WzI9HUL4qe2XbI+dk3xl+V7gxoztD5jRI+YK/zkEEokx2/uB/RdzIfUtueqVN04cXwF8G3iHY3z9Urw/j8ClyhsnjrcS2Vv/J/8NLxT+/zqBTkcxU/cfEkyEAu3kmjAAAAAASUVORK5CYII="));}.bk-tool-icon-box-edit{background-image:var(--bokeh-icon-box-edit, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4QfHjM1QAAAGRJREFUWMNjXLhsJcNAAiaGAQYsDAwM/+lsJ+OgCwGsLqMB+D8o08CoA0YdMOqAUQewDFQdMBoFIyoN/B/U7YFRB7DQIc7xyo9GwbBMA4xDqhxgISH1klXbDYk0QOseEeOgDgEAIS0JQleje6IAAAAASUVORK5CYII="));}.bk-tool-icon-freehand-draw{background-image:var(--bokeh-icon-freehand-draw, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAADTElEQVRYCeWWTWwMYRjH/88721X1lZJIGxJxcEE4OOiBgzjXWh8TJKR76kWacOBGxdEJIdk4VChZI/phidRBHMRRIr7DSUiaSCRFRM3u88gz+o7Z6bBTdjmYZPf9eJ55fv/5zzvvDPC/H9QsA66Olo9Ga+/MdR+Ljm2/KQIULsz9FqItGdOfJKLhApLgVkiSCGODjWit7QpKWy+TNrFeXvzKVUT8NiTVaIgDcbiCFJ7GiT8WkARXAdYBK0Lbhi/CenArRNskuM7/tgNp4ArQ42dwjf3WY5gWTqC7O/NbNn2Xkfw/YwdSw/We14HP2IEZwX+y9cZ9SH0LmgFP7UCz4KkENBNeV0Cz4b8U8DfgKiDxMWwUXETqLvJpCQpXZfawbzS7t9v5pL19cHBwfja7YA0y/lyCM0+E5hv5+piZXwKYcF23as+37bTXsQVqgkL0p/34fHR7DcBtbetFsBmGDwMOJCggYG55yw7dMlk6DuC1Bdu2RsCU9TYWQq2IoGbsreZ5NzvEqfSBsIsIy8OTbcdgiRHeh4o8AFAEwDakbY2AaCCpH7V9aGhoUUUy3UyVbkPYFuYLDlUZH8XBpwxkK0Dbgxg5HcVi0ent7a0RULMIozaHBSMfF9b2SzdutFcFB2FkwMIJOG6qfteXOa1nHZ48tyefuwyfT9s6wtzZ3t7eZse2DR2I228TtHXzuWCx9g8MtK5cuHCZTH4tiHEOa4xFngvTyS8f35d6enomiCi4/foEXBkZaQuukChL4FYA2Whd7YcC4gEdW3CpdL3LtGAVCVYJywEyTpAuJKeMOKXZs/Bw947C50KhUFOG4cwz35cjWNBlHGeD53n3xsfHP/T19U1qciggar8Fa4I3PHobIotBWBtc2hSiChyZxVzM53Pv7FVH6Tp3uVy+g0r1ImD2GjIrQGYIxjnfuXTZGICS5k/bBwJoubwEFX4TLah9EXomJGMA3za+f9913Yl4TnzsDQ+vE6YTZOjHh4ngibstt1pzQwd04F0bPStEBpXqRoBeQ/AKghfBnOEKgS+Q7z91Xfdz/HGKg8Ox7z8iYD9z6wqTkZFgnvhMGP9VZ2or1XVkPM9z0mytSfVsHa1RLBZbLoyNzUnK+ydz3wC6I9x+lwbngwAAAABJRU5ErkJggg=="));}.bk-tool-icon-poly-draw{background-image:var(--bokeh-icon-poly-draw, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEjglo9eZgwAAAc5JREFUWMPt1zFrU1EUB/DfS4OmVTGDIChCP4BgnQXRxVHqIJUupp9AB8VBQcRBQUXIB9DWQoMRiXZzcnQSA34A7aAuHSJKkgo2LvfBrU3aJnlYkBy4vHcP557zP/9z3r33JdXa647N0kHSZd5Nn0rSxc8G3cXp85sMcnZZ8vge3osZ+l3vB8CWFA0iL14t79h210swAjACMAIwAjACkB90D/8/GchI9ve4nPwTBh5E9ws7OepzGWb9EddSn51Op9ZstadSg4VK1UKlKkmSDSMLALewiuNh/hVJq71Wxttmqz0dG88vPc+MgWP4grvYG3SLOBrZFFFrttqPe4HIDxh4GSei+98iSlusuYopXEAjBtEPA3tQwUpwluAbDm4TPJUz+BTW9l2Ce6G7L0X/Bw8D3T/7SKKIDzHg7QCcxjvcQAEtXAnrrg/RP0/DKPbqgcN4iVOR7gcO4dcQgRuoh7HSqwlP4n20m63jJu5n8MkWMYfP3UowhzdR8FU8w9iQwevBdyq3/27CMRzAE5yLuvsRLg+ZcR1nJ8YL81HWJUzGAPaFZwe/Q5MdyYDyNHgjzO90YyGHtVDncuiJchaHw8R4oREFV5qdiVmYLM3OgD9k5209/atmIAAAAABJRU5ErkJggg=="));}.bk-tool-icon-point-draw{background-image:var(--bokeh-icon-point-draw, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEiERGWPELgAAA4RJREFUWMO1lr1uG1cQhb9ztdRSP7AF1QxgwKlcuZSqRC9gWUUUINWqTh5AnaFOnVPEteQmRuhCURqWsSqqc9IolREXdEvQBElxtdw7KURSFEVKu4w8wAKLxdw9Z+bMnRmZGXfZ29//II8th4WwGVNyIoQLYB5vxA9Caq04iUd9A+7ZlsNC2I7TdSd2hZXMJKlnTqp9jtl/GBaqoyQ0noFKpUIzBicYYc+DEFpxkglc4oVJa5gvDn8v1xV2irG3FM4NSVwjUKlUaMcpJhCGmSEJQ6QGD8M5WnHCd8+f3QCXpPLx8WNwv0j6Bm9FMK7FJ3WBE+R/2t7c/GBmFvSBrzRTCsyTDjXrxUgEMtpxynJYmJoBJ4VAybwVARgvL7Oik0okCodnKpVKX7P0leiVMb0VvbJT+upznK4vh0GIeQwwQStJkHQD3MwsCALTJRG7Qrdrj5m/djgYaIa0hlkRdJk26XEgC9txurccBtVW3IudBImmZuACUP+ZlIDBt9FKcubYNTcAH/X0RYM1E7utJPlqe+uZzPxUcEkiSS4sTT95n15Mud0xWC0o2PAWOCdK3KYZlFxfM+tHOcnMzNr1es18ug+cgsVjP4yBU/Ppfrter1m/+l0+zYygML1xRVHU7TSb1cSzBzoBzszsH+AMdJJ49jrNZjWKou6wBnwOzcyndBpNbuueURR1Dw8Pq35p9cc5p/Dy9Dypt7jXrtdGwQECS9NPhr6Gq6txUzNigE6zydLK6lTw12/KT4FGFEUfJX2YJNONq5tVs4ODA7sD/DnwJ/BoADZuE3tHFs12dna6d4C/BI6AlbyzI8ii2TTw12/KK33gb2cdXsNZoAntbZC2SeO4c9592k/5eNQbiwvFd1kJuFGwLJr1wSPg/SwpvyFBHufOeXcFeAlE97U/uCxOY+P3b+Bn4B3Q+L8EdJfD4a+/AbC4UBzPxiPg3wlHZquB28Cn2IuR9x3gr3uV4DbwfvSDOvi4uFA8BDZmIRHkjHpS9Ht9iRqd8+5G3g05mAGcQbsdiX5QJ428G7Kygo8XYdb1/K4NWVmjzkNge2sz84bs+ELmpDDLtqWsNZBXgvmw8CTtpWVMT7x5YWBjLARnwZfKQNYN2U2LPvrh+5nBt7c2M2/It9bArCTKR8eZN+SJ13AScPnoODeRdqNenH+wul5w2gUr2WUjMFAt8bZ/0axX/wNnv4H8vTFb1QAAAABJRU5ErkJggg=="));}.bk-tool-icon-poly-edit{background-image:var(--bokeh-icon-poly-edit, url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gELFi46qJmxxAAABV9JREFUWMOdl19vFFUYxn9n9u9sCyylUIzWUoMQBAWCMdEEIt6xIRQSLIEKtvHe6AcA4yeQb7CAUNJy0daLeomJN8SEULAC2kBBapBKoLvbmdl/c14vdmY7u91tF95kknPOnHmf95znPc97Ro2OTeBbdjFDT3c32ZxVHUOE9kSMB0/m6ExuoJn1H+ur6Y+OTfD50SMN5168OgrAlyf7CfuD+z7+iDs3p8hkLUQ0iFQ/yFl5Nm/qonfHVva+s32Zw9GxCYILsZ08tpNfBhbs+1YN4OH9+7huGdECSBVfqUosbsllfmauBqiR+cCNwOr7AEo8pPHJnymXykhg5fUWjoQpl0vVvhZhbSzGoUOHqgBlt6B6uruj2Zy1E9jo0fhfeyL2x4Mnc8VErK0KUEOB64JSyptfG4RSytsJjUJVxw2lsFy3urL9nx1Qd25ObctkrVMi+jQivd7U2ZyV/3Hzpq7h3h1b/7p9Y0o8v8rwAbTWrGpSocN/FGDlbAI0Rl23PCBan0Ok158H9Ipwzi25A/Mzc9Gl/BYx/E4kYqC1NKRARNAaDCNUM27Z+Zr+ouXs0q4+LSLBHPYCFkTkC6uU39kwCdsS7WRKmaYUiAhdnZ3MPX2K4+QjQI+C94A93rMzm8ltMwyDeDzWjMZeEb2pYQDdW3vITU2jtUZ5QThOPgm8C7wP7J15OPsBsB3oWpGnVWisCeDS1VHj4vBI92+/3tgB7Ab2AruAXiDBK5oIOkhtkEYRNRuJhObrd8Dl9ewf4D5wG7hVLpen29vb5wzD+BrkbBMaL3d1dk5nsrnlFDTTFWAWmAZueWD3gCemGde2k2fw1Al1YXhEvjozoO49eczdqekrWmsc2zlrmvEKOGoW1GUjFLqSk2KpJrCLwyMCPAP+BO54QL8DM6YZX/ClsP9YnwKkXnIBP4jdIpJRpdJTCYdMwwi98KU0Hjc/dDILNyUcwTCWdOSMJ0TRmBktGRhLugu0xyLk7CIqVNm+0bGJptl1YXikD0grpY4Rjc4a8Fbgdab/6OGbAJeCUuyJnnHmZH9pbSyGuBXV8NUwlUpR1EWyixmSyTWEwqGlJ2Swbo2JXbAAfgDGgGQA9I1A9t1tlq0AxrXxn0ilUpw4fhQqYkH/sT41OTnJJwf2s6FjI5mshdYa7bqVR2uezr9MJmJt14FvGrh/O9D+e6UkM/xyCuCqEKCYnJyUTKFQrZDHjxzGshwWLQcRsOz8Hi85P23id0ug/XilAMLBmm4tPGdoaKjSH5+oAGrhwvBI9SjZTn4QSK9yenoD7dlrExPoJlXW8G8ytpNHxRKk02lGxsdRKFwXLNvx5yY94HQLGhGk4LFCYQSqaE0AwWM1eOoEbR0dKBSW7bC4mKuffxs4D/wCLKwQQPAUzIkslfp6cVomROWSolh0GjldAM4nzDi2k9/i5UAzC9aKfwNJ3zgJg9YEvN6+C7SHgKm69+sD7RfNnKTTaZRPQfAut4oFV//IS7gkcB34VlVo8kGzphlfB+DU+TfNGBpZtRastvrvARJmfMF28ge9sc2B9/PNnCilMIDwK6y8/ow/Ai4kvILTljAXvDvEvrqKSUs60KolzPjBxspavQD2tKqCAGF/Ba+xE/Wbilu54wZV8NEKF5fXzQHl/bh4hUsE0WAXSlDMYcQSrQXgCmsTseXHsJkNnjqBFGwKJaHsKlxtUHYVhbLCzr1kaOA4bcn1y1Swmb+iLpJKpVrfgdpfsiVVCYcgluwgnU7jEgJ4s5UkLFtWYyHyEg0/N1q1tmQH+YXnAMFr97Nmv3p+0QsHQRsF8qpBOE5+rb9Nkaj50tVQKjqh4OU3GNL/1/So3vuUgbAAAAAASUVORK5CYII="));}.bk-tool-icon-line-edit{background-image:var(--bokeh-icon-line-edit, url("data:image/png;base64, "));}.bk-tool-icon-settings{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-settings);-webkit-mask-image:var(--bokeh-icon-settings);}.bk-tool-icon-unknown{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-unknown);-webkit-mask-image:var(--bokeh-icon-unknown);}.bk-tool-icon-fullscreen{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-fullscreen);-webkit-mask-image:var(--bokeh-icon-fullscreen);}.bk-tool-icon-chevron-up{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-up);-webkit-mask-image:var(--bokeh-icon-chevron-up);}.bk-tool-icon-chevron-down{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-down);-webkit-mask-image:var(--bokeh-icon-chevron-down);}.bk-tool-icon-chevron-left{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-left);-webkit-mask-image:var(--bokeh-icon-chevron-left);}.bk-tool-icon-chevron-right{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-right);-webkit-mask-image:var(--bokeh-icon-chevron-right);}.bk-tool-icon-caret-up{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-caret-up);-webkit-mask-image:var(--bokeh-icon-caret-up);}.bk-tool-icon-caret-down{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-caret-down);-webkit-mask-image:var(--bokeh-icon-caret-down);}.bk-tool-icon-caret-left{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-caret-left);-webkit-mask-image:var(--bokeh-icon-caret-left);}.bk-tool-icon-caret-right{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-caret-right);-webkit-mask-image:var(--bokeh-icon-caret-right);}.bk-tool-icon-see-on{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-see-on);-webkit-mask-image:var(--bokeh-icon-see-on);}.bk-tool-icon-see-off{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-see-off);-webkit-mask-image:var(--bokeh-icon-see-off);}.bk-tool-icon-x-grip{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-y-grip);-webkit-mask-image:var(--bokeh-icon-y-grip);}.bk-tool-icon-y-grip{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-y-grip);-webkit-mask-image:var(--bokeh-icon-y-grip);}.bk-tool-icon-arrow-down-to-bar{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-arrow-down-to-bar);-webkit-mask-image:var(--bokeh-icon-arrow-down-to-bar);}.bk-tool-icon-arrow-up-from-bar{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-arrow-up-from-bar);-webkit-mask-image:var(--bokeh-icon-arrow-up-from-bar);}.bk-tool-icon-minimize{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-minimize);-webkit-mask-image:var(--bokeh-icon-minimize);}.bk-tool-icon-maximize{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-maximize);-webkit-mask-image:var(--bokeh-icon-maximize);}.bk-tool-icon-pin{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-pin);-webkit-mask-image:var(--bokeh-icon-pin);}.bk-tool-icon-unpin{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-unpin);-webkit-mask-image:var(--bokeh-icon-unpin);}.bk-tool-icon-pointer{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-hand-pointing);-webkit-mask-image:var(--bokeh-icon-hand-pointing);}.bk-tool-icon-delete{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-trash);-webkit-mask-image:var(--bokeh-icon-trash);}.bk-tool-icon-bold{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-bold);-webkit-mask-image:var(--bokeh-icon-bold);}.bk-tool-icon-italic{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-italic);-webkit-mask-image:var(--bokeh-icon-italic);}.bk-tool-icon-text-align-left{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-text-align-left);-webkit-mask-image:var(--bokeh-icon-text-align-left);}.bk-tool-icon-text-align-center{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-text-align-center);-webkit-mask-image:var(--bokeh-icon-text-align-center);}.bk-tool-icon-text-align-right{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-text-align-right);-webkit-mask-image:var(--bokeh-icon-text-align-right);}.bk-tool-icon-check{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-check);-webkit-mask-image:var(--bokeh-icon-check);}.bk-tool-icon-square{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-square);-webkit-mask-image:var(--bokeh-icon-square);}.bk-tool-icon-square-check{background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-square-check);-webkit-mask-image:var(--bokeh-icon-square-check);}:host{--bokeh-icon-question-mark:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M8%208a3.5%203%200%200%201%203.5%20-3h1a3.5%203%200%200%201%203.5%203a3%203%200%200%201%20-2%203a3%204%200%200%200%20-2%204%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2212%22%20y1%3D%2219%22%20x2%3D%2212%22%20y2%3D%2219.01%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-help:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Ccircle%20cx%3D%2212%22%20cy%3D%2212%22%20r%3D%229%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2212%22%20y1%3D%2217%22%20x2%3D%2212%22%20y2%3D%2217.01%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2013.5a1.5%201.5%200%200%201%201%20-1.5a2.6%202.6%200%201%200%20-3%20-4%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-x:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M18%206l-12%2012%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%206l12%2012%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-settings:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M10.325%204.317c.426%20-1.756%202.924%20-1.756%203.35%200a1.724%201.724%200%200%200%202.573%201.066c1.543%20-.94%203.31%20.826%202.37%202.37a1.724%201.724%200%200%200%201.065%202.572c1.756%20.426%201.756%202.924%200%203.35a1.724%201.724%200%200%200%20-1.066%202.573c.94%201.543%20-.826%203.31%20-2.37%202.37a1.724%201.724%200%200%200%20-2.572%201.065c-.426%201.756%20-2.924%201.756%20-3.35%200a1.724%201.724%200%200%200%20-2.573%20-1.066c-1.543%20.94%20-3.31%20-.826%20-2.37%20-2.37a1.724%201.724%200%200%200%20-1.065%20-2.572c-1.756%20-.426%20-1.756%20-2.924%200%20-3.35a1.724%201.724%200%200%200%201.066%20-2.573c-.94%20-1.543%20.826%20-3.31%202.37%20-2.37c1%20.608%202.296%20.07%202.572%20-1.065z%22%3E%3C%2Fpath%3E%0A%20%20%3Ccircle%20cx%3D%2212%22%20cy%3D%2212%22%20r%3D%223%22%3E%3C%2Fcircle%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-unknown:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M14%203v4a1%201%200%200%200%201%201h4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M17%2021h-10a2%202%200%200%201%20-2%20-2v-14a2%202%200%200%201%202%20-2h7l5%205v11a2%202%200%200%201%20-2%202z%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2017v.01%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2014a1.5%201.5%200%201%200%20-1.14%20-2.474%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-fullscreen:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Crect%20x%3D%223%22%20y%3D%2216%22%20width%3D%225%22%20height%3D%225%22%20rx%3D%221%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M4%2012v-6a2%202%200%200%201%202%20-2h12a2%202%200%200%201%202%202v12a2%202%200%200%201%20-2%202h-6%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%208h4v4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M16%208l-5%205%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-save:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%2017v2a2%202%200%200%200%202%202h12a2%202%200%200%200%202%20-2v-2%22%20%2F%3E%0A%20%20%3Cpolyline%20points%3D%227%2011%2012%2016%2017%2011%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2212%22%20y1%3D%224%22%20x2%3D%2212%22%20y2%3D%2216%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-copy:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Crect%20x%3D%228%22%20y%3D%228%22%20width%3D%2212%22%20height%3D%2212%22%20rx%3D%222%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M16%208v-2a2%202%200%200%200%20-2%20-2h-8a2%202%200%200%200%20-2%202v8a2%202%200%200%200%202%202h2%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-tap-select:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cline%20x1%3D%223%22%20y1%3D%2212%22%20x2%3D%226%22%20y2%3D%2212%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2212%22%20y1%3D%223%22%20x2%3D%2212%22%20y2%3D%226%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%227.8%22%20y1%3D%227.8%22%20x2%3D%225.6%22%20y2%3D%225.6%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2216.2%22%20y1%3D%227.8%22%20x2%3D%2218.4%22%20y2%3D%225.6%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%227.8%22%20y1%3D%2216.2%22%20x2%3D%225.6%22%20y2%3D%2218.4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2012l9%203l-4%202l-2%204l-3%20-9%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-lasso-select:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4.028%2013.252c-.657%20-.972%20-1.028%20-2.078%20-1.028%20-3.252c0%20-3.866%204.03%20-7%209%20-7s9%203.134%209%207s-4.03%207%20-9%207c-1.913%200%20-3.686%20-.464%20-5.144%20-1.255%22%20%2F%3E%0A%20%20%3Ccircle%20cx%3D%225%22%20cy%3D%2215%22%20r%3D%222%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M5%2017c0%201.42%20.316%202.805%201%204%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-pan:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M18%209l3%203l-3%203%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%209l-3%203l3%203%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M9%2018l3%203l3%20-3%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%206l-3%20-3l-3%203%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2012h9%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%2012h9%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2012v9%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%203v9%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-x-pan:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M18%209l3%203l-3%203%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%209l-3%203l3%203%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2012h9%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%2012h9%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-y-pan:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M9%2018l3%203l3%20-3%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%206l-3%20-3l-3%203%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2012v9%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%203v9%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-box-select:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%206v-1a1%201%200%200%201%201%20-1h1m5%200h2m5%200h1a1%201%200%200%201%201%201v1m0%205v2m0%205v1a1%201%200%200%201%20-1%201h-1m-5%200h-2m-5%200h-1a1%201%200%200%201%20-1%20-1v-1m0%20-5v-2%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-x-box-select:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M17%2013l-4%204%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M13%2013l4%204%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M4%206v-1a1%201%200%200%201%201%20-1h1m5%200h2m5%200h1a1%201%200%200%201%201%201v1m0%205v2m0%205v1a1%201%200%200%201%20-1%201h-1m-5%200h-2m-5%200h-1a1%201%200%200%201%20-1%20-1v-1m0%20-5v-2%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-y-box-select:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M13%2013l2%202l2%20-2%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%2015v2.5%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M4%206v-1a1%201%200%200%201%201%20-1h1m5%200h2m5%200h1a1%201%200%200%201%201%201v1m0%205v2m0%205v1a1%201%200%200%201%20-1%201h-1m-5%200h-2m-5%200h-1a1%201%200%200%201%20-1%20-1v-1m0%20-5v-2%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-box-zoom:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Ccircle%20cx%3D%2215%22%20cy%3D%2215%22%20r%3D%225%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M22%2022l-3%20-3%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%2018h-1a2%202%200%200%201%20-2%20-2v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%2011v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%206v-1a2%202%200%200%201%202%20-2h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M10%203h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%203h1a2%202%200%200%201%202%202v1%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-x-box-zoom:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M17%2013l-4%204%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M13%2013l4%204%22%20%2F%3E%0A%20%20%3Ccircle%20cx%3D%2215%22%20cy%3D%2215%22%20r%3D%225%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M22%2022l-3%20-3%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%2018h-1a2%202%200%200%201%20-2%20-2v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%2011v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%206v-1a2%202%200%200%201%202%20-2h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M10%203h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%203h1a2%202%200%200%201%202%202v1%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-y-box-zoom:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M13%2013l2%202l2%20-2%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%2015v2.5%22%20%2F%3E%0A%20%20%3Ccircle%20cx%3D%2215%22%20cy%3D%2215%22%20r%3D%225%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M22%2022l-3%20-3%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%2018h-1a2%202%200%200%201%20-2%20-2v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%2011v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%206v-1a2%202%200%200%201%202%20-2h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M10%203h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%203h1a2%202%200%200%201%202%202v1%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-auto-box-zoom:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Ccircle%20cx%3D%2215%22%20cy%3D%2215%22%20r%3D%225%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M22%2022l-3%20-3%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%2018h-1a2%202%200%200%201%20-2%20-2v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%2011v-1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M3%206v-1a2%202%200%200%201%202%20-2h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M10%203h1%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%203h1a2%202%200%200%201%202%202v1%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-zoom-in:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Ccircle%20cx%3D%2210%22%20cy%3D%2210%22%20r%3D%227%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%227%22%20y1%3D%2210%22%20x2%3D%2213%22%20y2%3D%2210%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2210%22%20y1%3D%227%22%20x2%3D%2210%22%20y2%3D%2213%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2221%22%20y1%3D%2221%22%20x2%3D%2215%22%20y2%3D%2215%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-zoom-out:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Ccircle%20cx%3D%2210%22%20cy%3D%2210%22%20r%3D%227%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%227%22%20y1%3D%2210%22%20x2%3D%2213%22%20y2%3D%2210%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%2221%22%20y1%3D%2221%22%20x2%3D%2215%22%20y2%3D%2215%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-undo:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M9%2013l-4%20-4l4%20-4m-4%204h11a4%204%200%200%201%200%208h-1%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-redo:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M15%2013l4%20-4l-4%20-4m4%204h-11a4%204%200%200%200%200%208h1%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-reset:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M20%2011a8.1%208.1%200%200%200%20-15.5%20-2m-.5%20-4v4h4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M4%2013a8.1%208.1%200%200%200%2015.5%202m.5%204v-4h-4%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-hover:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M12%2020l-3%20-3h-2a3%203%200%200%201%20-3%20-3v-6a3%203%200%200%201%203%20-3h10a3%203%200%200%201%203%203v6a3%203%200%200%201%20-3%203h-2l-3%203%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%228%22%20y1%3D%229%22%20x2%3D%2216%22%20y2%3D%229%22%20%2F%3E%0A%20%20%3Cline%20x1%3D%228%22%20y1%3D%2213%22%20x2%3D%2214%22%20y2%3D%2213%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-crosshair:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Ccircle%20cx%3D%2212%22%20cy%3D%2212%22%20r%3D%229%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M20%2012h-4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M4%2012h4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%2020v-4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M12%204v4%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-chevron-up:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpolyline%20points%3D%226%2015%2012%209%2018%2015%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-chevron-down:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpolyline%20points%3D%226%209%2012%2015%2018%209%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-chevron-left:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpolyline%20points%3D%2215%206%209%2012%2015%2018%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-chevron-right:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpolyline%20points%3D%229%206%2015%2012%209%2018%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-caret-up:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22currentColor%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M18%2015l-6%20-6l-6%206h12%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-caret-down:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22currentColor%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M18%2015l-6%20-6l-6%206h12%22%20transform%3D%22rotate(180%2012%2012)%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-caret-left:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22currentColor%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M18%2015l-6%20-6l-6%206h12%22%20transform%3D%22rotate(270%2012%2012)%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-caret-right:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22currentColor%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M18%2015l-6%20-6l-6%206h12%22%20transform%3D%22rotate(90%2012%2012)%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-see-on:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Ccircle%20cx%3D%2212%22%20cy%3D%2212%22%20r%3D%222%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M22%2012c-2.667%204.667%20-6%207%20-10%207s-7.333%20-2.333%20-10%20-7c2.667%20-4.667%206%20-7%2010%20-7s7.333%202.333%2010%207%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-see-off:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cline%20x1%3D%223%22%20y1%3D%223%22%20x2%3D%2221%22%20y2%3D%2221%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M10.584%2010.587a2%202%200%200%200%202.828%202.83%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M9.363%205.365a9.466%209.466%200%200%201%202.637%20-.365c4%200%207.333%202.333%2010%207c-.778%201.361%20-1.612%202.524%20-2.503%203.488m-2.14%201.861c-1.631%201.1%20-3.415%201.651%20-5.357%201.651c-4%200%20-7.333%20-2.333%20-10%20-7c1.369%20-2.395%202.913%20-4.175%204.632%20-5.341%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-cursor-rotate:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%3E%0A%20%20%3Cg%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%20%20%3Cg%20stroke-width%3D%224%22%20stroke%3D%22white%22%3E%0A%20%20%20%20%20%20%3Cpath%20d%3D%22M9%204.55a8%208%200%200%201%206%2014.9m0%20-4.45v5h5%22%20%2F%3E%0A%20%20%20%20%3C%2Fg%3E%0A%20%20%20%20%3Cg%20stroke-width%3D%222%22%20stroke%3D%22black%22%3E%0A%20%20%20%20%20%20%3Cpath%20d%3D%22M9%204.55a8%208%200%200%201%206%2014.9m0%20-4.45v5h5%22%20%2F%3E%0A%20%20%20%20%3C%2Fg%3E%0A%20%20%3C%2Fg%3E%0A%3C%2Fsvg%3E%0A") 12 12, auto;--bokeh-icon-x-grip:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M5%209m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M5%2015m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M12%209m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M12%2015m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M19%209m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M19%2015m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-y-grip:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M9%205m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M9%2012m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M9%2019m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M15%205m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M15%2012m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M15%2019m-1%200a1%201%200%201%200%202%200a1%201%200%201%200%20-2%200%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-arrow-down-to-bar:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%2020l16%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M12%2014l0%20-10%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M12%2014l4%20-4%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M12%2014l-4%20-4%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-arrow-up-from-bar:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M12%204l0%2010%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M12%204l4%204%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M12%204l-4%204%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M4%2020l16%200%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-minimize:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M15%2019v-2a2%202%200%200%201%202%20-2h2%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M15%205v2a2%202%200%200%200%202%202h2%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M5%2015h2a2%202%200%200%201%202%202v2%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M5%209h2a2%202%200%200%200%202%20-2v-2%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-maximize:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%208v-2a2%202%200%200%201%202%20-2h2%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M4%2016v2a2%202%200%200%200%202%202h2%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M16%204h2a2%202%200%200%201%202%202v2%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M16%2020h2a2%202%200%200%200%202%20-2v-2%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-pin:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M15%204.5l-4%204l-4%201.5l-1.5%201.5l7%207l1.5%20-1.5l1.5%20-4l4%20-4%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M9%2015l-4.5%204.5%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M14.5%204l5.5%205.5%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-unpin:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M3%203l18%2018%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M15%204.5l-3.249%203.249m-2.57%201.433l-2.181%20.818l-1.5%201.5l7%207l1.5%20-1.5l.82%20-2.186m1.43%20-2.563l3.25%20-3.251%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M9%2015l-4.5%204.5%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M14.5%204l5.5%205.5%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-hand-pointing:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M8%2013v-8.5a1.5%201.5%200%200%201%203%200v7.5%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M11%2011.5v-2a1.5%201.5%200%201%201%203%200v2.5%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M14%2010.5a1.5%201.5%200%200%201%203%200v1.5%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M17%2011.5a1.5%201.5%200%200%201%203%200v4.5a6%206%200%200%201%20-6%206h-2h.208a6%206%200%200%201%20-5.012%20-2.7a69.74%2069.74%200%200%201%20-.196%20-.3c-.312%20-.479%20-1.407%20-2.388%20-3.286%20-5.728a1.5%201.5%200%200%201%20.536%20-2.022a1.867%201.867%200%200%201%202.28%20.28l1.47%201.47%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-trash:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%207l16%200%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M10%2011l0%206%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M14%2011l0%206%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M5%207l1%2012a2%202%200%200%200%202%202h8a2%202%200%200%200%202%20-2l1%20-12%22%3E%3C%2Fpath%3E%0A%20%20%3Cpath%20d%3D%22M9%207v-3a1%201%200%200%201%201%20-1h4a1%201%200%200%201%201%201v3%22%3E%3C%2Fpath%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-bold:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M7%205h6a3.5%203.5%200%200%201%200%207h-6z%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M13%2012h1a3.5%203.5%200%200%201%200%207h-7v-7%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-italic:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M11%205l6%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M7%2019l6%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M14%205l-4%2014%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-text-align-left:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%206l16%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M4%2012l10%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M4%2018l14%200%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-text-align-center:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%206l16%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M8%2012l8%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%2018l12%200%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-text-align-right:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M4%206l16%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M10%2012l10%200%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M6%2018l14%200%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-check:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M5%2012l5%205l10%20-10%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-square:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M3%203m0%202a2%202%200%200%201%202%20-2h14a2%202%200%200%201%202%202v14a2%202%200%200%201%20-2%202h-14a2%202%200%200%201%20-2%20-2z%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");--bokeh-icon-square-check:url("data:image/svg+xml,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%2224%22%20height%3D%2224%22%20viewBox%3D%220%200%2024%2024%22%20stroke-width%3D%222%22%20stroke%3D%22currentColor%22%20fill%3D%22none%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%0A%20%20%3Cpath%20d%3D%22M3%203m0%202a2%202%200%200%201%202%20-2h14a2%202%200%200%201%202%202v14a2%202%200%200%201%20-2%202h-14a2%202%200%200%201%20-2%20-2z%22%20%2F%3E%0A%20%20%3Cpath%20d%3D%22M9%2012l2%202l4%20-4%22%20%2F%3E%0A%3C%2Fsvg%3E%0A");}';
  class MenuView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "_menu_views", /* @__PURE__ */ new Map());
      mt(this, "prevent_hide");
      mt(this, "_open", !1);
      mt(this, "_item_click", (H) => {
        var V;
        H.disabled || ((V = H.action) == null || V.execute(this.model, { item: H }), this.hide());
      });
      mt(this, "_on_mousedown", (H) => {
        var V;
        H.composedPath().includes(this.el) || (((V = this.prevent_hide) == null ? void 0 : V.call(this, H)) ?? !1) || this.hide();
      });
      mt(this, "_on_keydown", (H) => {
        H.key == "Escape" && this.hide();
      });
      mt(this, "_on_blur", () => {
        this.hide();
      });
    }
    *children() {
      yield* super.children(), yield* this._menu_views.values();
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const H = this.model.items.map((V) => V instanceof ActionItem ? V.menu : null).filter(isNotNull);
      await build_views(this._menu_views, H, { parent: this });
    }
    get is_open() {
      return this._open;
    }
    remove() {
      this._unlisten(), remove_views(this._menu_views), super.remove();
    }
    _listen() {
      document.addEventListener("mousedown", this._on_mousedown), document.addEventListener("keydown", this._on_keydown), window.addEventListener("blur", this._on_blur);
    }
    _unlisten() {
      document.removeEventListener("mousedown", this._on_mousedown), document.removeEventListener("keydown", this._on_keydown), window.removeEventListener("blur", this._on_blur);
    }
    stylesheets() {
      return [...super.stylesheets(), menus_css$1, icons_css];
    }
    render() {
      super.render();
      const H = (() => {
        const { reversed: V, items: q } = this.model;
        return V ? reversed(q) : q;
      })();
      for (const V of H)
        if (V instanceof DividerItem) {
          const q = div({ class: divider$3 });
          this.shadow_el.append(q);
        } else if (V instanceof ActionItem) {
          const q = div({ class: check }), G = div({ class: icon$1 }), J = div({ class: label$1 }, V.label), Z = div({ class: shortcut }, V.shortcut), te = div({ class: chevron$1 }), { icon: ne } = V;
          if (ne != null) {
            if (ne.startsWith("data:image")) {
              const oe = `url("${encodeURI(ne)}")`;
              G.style.backgroundImage = oe;
            } else if (ne.startsWith("--"))
              G.style.backgroundImage = `var(${ne})`;
            else if (ne.startsWith(".")) {
              const oe = ne.substring(1);
              G.classList.add(oe);
            } else if (ToolIcon.valid(ne)) {
              const oe = `bk-tool-icon-${ne.replace(/_/g, "-")}`;
              G.classList.add(oe);
            }
          }
          const ee = div({ class: item$1, title: V.tooltip, tabIndex: 0 }, q, G, J, Z, te);
          ee.classList.toggle(menu$1, V.menu != null), ee.classList.toggle(disabled$3, V.disabled), V instanceof CheckableItem && (ee.classList.add(checkable), ee.classList.toggle(checked, V.checked)), ee.addEventListener("click", () => {
            this._item_click(V);
          }), ee.addEventListener("keydown", (oe) => {
            oe.key == "Enter" && this._item_click(V);
          });
          const { menu: ae } = V;
          ae != null && (ee.addEventListener("pointerenter", () => {
            this._menu_views.get(ae)._show_submenu(ee);
          }), ee.addEventListener("pointerleave", () => {
            this._menu_views.get(ae).hide();
          })), this.shadow_el.append(ee);
        }
    }
    _show_submenu(H) {
      if (this.model.items.length == 0) {
        this.hide();
        return;
      }
      this.render(), H.append(this.el);
      const { style: V } = this.el;
      V.left = "100%", V.top = "0", this._listen(), this._open = !0;
    }
    show(H) {
      if (this.model.items.length == 0) {
        this.hide();
        return;
      }
      const { parent: V } = this;
      if (V == null) {
        this.hide();
        return;
      }
      this.render(), (V.el.shadowRoot ?? V.el).append(this.el);
      const { style: G } = this.el;
      G.left = px$1(H.x), G.top = px$1(H.y), this._listen(), this._open = !0;
    }
    hide() {
      this._open && (this._open = !1, this._unlisten(), this.el.remove());
    }
  }
  mt(MenuView, "__name__", "MenuView");
  const O3 = class O3 extends UIElement {
    constructor(_) {
      super(_);
    }
  };
  mt(O3, "__name__", "Menu"), O3.prototype.default_view = MenuView, O3.define(({ Bool: _, List: H, Ref: V }) => ({
    items: [H(V(MenuItem)), []],
    reversed: [_, !1]
  }));
  let Menu = O3;
  const bg = class bg extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(bg, "__name__", "RendererGroup"), bg.define(({ Bool: _ }) => ({
    visible: [_, !0]
  }));
  let RendererGroup = bg;
  class RendererView extends View$2 {
    constructor() {
      super(...arguments);
      mt(this, "visuals");
      mt(this, "_context_menu", null);
      mt(this, "_coordinates");
      mt(this, "_custom_coordinates", null);
    }
    get context_menu() {
      return this._context_menu;
    }
    get coordinates() {
      const { _coordinates: H } = this;
      return H ?? (this._coordinates = this._initialize_coordinates());
    }
    set coordinates(H) {
      this._custom_coordinates = H;
    }
    initialize() {
      super.initialize(), this.visuals = new Visuals(this);
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { context_menu: H } = this.model;
      H != null && (this._context_menu = await build_view(H, { parent: this.plot_view }));
    }
    remove() {
      var H;
      (H = this._context_menu) == null || H.remove(), super.remove();
    }
    connect_signals() {
      super.connect_signals();
      const { group: H } = this.model;
      H != null && this.on_change(H.properties.visible, () => {
        this.model.visible = H.visible;
      });
      const { x_range_name: V, y_range_name: q } = this.model.properties;
      this.on_change([V, q], () => delete this._coordinates), this.connect(this.plot_view.frame.change, () => delete this._coordinates);
    }
    _initialize_coordinates() {
      if (this._custom_coordinates != null)
        return this._custom_coordinates;
      const { coordinates: H } = this.model, { frame: V } = this.plot_view;
      if (H != null)
        return H.get_transform(V);
      {
        const { x_range_name: q, y_range_name: G } = this.model, J = V.x_scales.get(q), Z = V.y_scales.get(G);
        return assert$2(J != null, `missing '${q}' range`), assert$2(Z != null, `missing '${G}' range`), new CoordinateTransform$1(J, Z);
      }
    }
    get plot_view() {
      return this.parent;
    }
    get plot_model() {
      return this.parent.model;
    }
    get layer() {
      const { overlays: H, primary: V } = this.canvas;
      return this.model.level == "overlay" ? H : V;
    }
    get canvas() {
      return this.plot_view.canvas_view;
    }
    request_render() {
      this.request_paint();
    }
    request_paint() {
      this.plot_view.request_paint(this);
    }
    request_layout() {
      this.plot_view.request_layout();
    }
    notify_finished() {
      this.plot_view.notify_finished();
    }
    notify_finished_after_paint() {
      this.plot_view.notify_finished_after_paint();
    }
    get needs_clip() {
      return !1;
    }
    get has_webgl() {
      return !1;
    }
    /*
    get visible(): boolean {
      const {visible, group} = this.model
      return !visible ? false : (group?.visible ?? true)
    }
    */
    get displayed() {
      return this.model.visible;
    }
    render() {
      this.update_geometry(), this.compute_geometry(), this.displayed && this._render(), this._has_finished = !0;
    }
    renderer_view(H) {
    }
    /**
     * Geometry setup that doesn't change between paints.
     */
    update_geometry() {
    }
    /**
     * Geometry setup that changes between paints.
     */
    compute_geometry() {
    }
    resolve_frame() {
      return this.plot_view.frame;
    }
    resolve_canvas() {
      return this.plot_view.canvas;
    }
    resolve_plot() {
      return this.plot_view;
    }
    resolve_symbol(H) {
      const V = this;
      if ("bbox" in V && V.bbox instanceof BBox$1) {
        const q = V.bbox.resolve(H.symbol), { offset: G } = H;
        if (isNumber$3(q))
          return q + G;
        {
          const { x: J, y: Z } = q;
          return { x: J + G, y: Z + G };
        }
      } else
        return { x: NaN, y: NaN };
    }
    get attribution() {
      return null;
    }
  }
  mt(RendererView, "__name__", "RendererView");
  var k0;
  let Renderer$1 = (k0 = class extends Model$1 {
    constructor(_) {
      super(_);
    }
  }, mt(k0, "__name__", "Renderer"), k0.define(({ Bool: _, Str: H, Ref: V, Nullable: q }) => ({
    group: [q(V(RendererGroup)), null],
    level: [RenderLevel, "image"],
    visible: [_, !0],
    x_range_name: [H, "default"],
    y_range_name: [H, "default"],
    coordinates: [q(V(CoordinateMapping)), null],
    propagate_hover: [_, !1],
    context_menu: [q(V(Menu)), null]
  })), k0);
  class DOMNodeView extends DOMView {
  }
  mt(DOMNodeView, "__name__", "DOMNodeView");
  class DOMNode extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(DOMNode, "__name__", "DOMNode"), mt(DOMNode, "__module__", "bokeh.models.dom");
  const ElementLike$1 = Or$1(Ref(UIElement), Ref(DOMNode));
  class CompositeRendererView extends RendererView {
    constructor() {
      super(...arguments);
      mt(this, "_renderer_views", /* @__PURE__ */ new Map());
      mt(this, "_element_views", /* @__PURE__ */ new Map());
      mt(this, "_has_rendered_elements", !1);
    }
    get renderer_views() {
      return this.model.renderers.map((H) => this._renderer_views.get(H));
    }
    get element_views() {
      return this.model.elements.map((H) => this._element_views.get(H));
    }
    *children() {
      yield* super.children(), yield* this.renderer_views, yield* this.element_views;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._build_renderers(), await this._build_elements();
    }
    async _build_renderers() {
      return await build_views(this._renderer_views, this.model.renderers, { parent: this.plot_view });
    }
    async _build_elements() {
      return await build_views(this._element_views, this.model.elements, { parent: this.plot_view });
    }
    async _update_renderers() {
      await this._build_renderers();
    }
    async _update_elements() {
      const { created: H } = await this._build_elements(), V = new Set(H);
      for (const q of this.element_views)
        q.el.remove();
      for (const q of this.element_views)
        V.has(q) ? q.render_to(this.plot_view.shadow_el) : this.plot_view.shadow_el.append(q.el);
    }
    remove() {
      remove_views(this._renderer_views), remove_views(this._element_views), super.remove();
    }
    connect_signals() {
      super.connect_signals();
      const { renderers: H, elements: V } = this.model.properties;
      this.on_change(H, async () => {
        await this._update_renderers();
      }), this.on_change(V, async () => {
        await this._update_elements();
      });
    }
    render() {
      if (!this._has_rendered_elements) {
        for (const V of this.element_views)
          V.render_to(this.plot_view.shadow_el);
        this._has_rendered_elements = !0;
      }
      super.render();
      const { displayed: H } = this;
      for (const V of this.element_views)
        V.reposition(H);
    }
    has_finished() {
      if (!super.has_finished())
        return !1;
      for (const H of this.renderer_views)
        if (!H.has_finished())
          return !1;
      for (const H of this.element_views)
        if (!H.has_finished())
          return !1;
      return !0;
    }
    serializable_state() {
      const { children: H, ...V } = super.serializable_state();
      return {
        ...V,
        children: [
          ...H ?? [],
          ...this.renderer_views.map((q) => q.serializable_state()),
          ...this.element_views.map((q) => q.serializable_state())
        ]
      };
    }
  }
  mt(CompositeRendererView, "__name__", "CompositeRendererView");
  const Tg = class Tg extends Renderer$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Tg, "__name__", "CompositeRenderer"), Tg.define(({ List: _, Ref: H }) => ({
    renderers: [_(H(Renderer$1)), []],
    elements: [_(ElementLike$1), []]
  }));
  let CompositeRenderer = Tg;
  class AnnotationView extends CompositeRendererView {
    constructor() {
      super(...arguments);
      mt(this, "layout");
      mt(this, "panel");
      mt(this, "bbox");
    }
    get_size() {
      if (this.displayed) {
        const { width: H, height: V } = this._get_size();
        return { width: Math.round(H), height: Math.round(V) };
      } else
        return { width: 0, height: 0 };
    }
    _get_size() {
      throw new Error("not implemented");
    }
    connect_signals() {
      super.connect_signals();
      const H = this.model.properties;
      this.on_change(H.visible, () => {
        this.layout != null && (this.layout.visible = this.model.visible, this.plot_view.request_layout());
      });
    }
    get needs_clip() {
      return this.layout == null;
    }
    serializable_state() {
      var q, G;
      const H = super.serializable_state(), V = ((q = this.bbox) == null ? void 0 : q.round()) ?? ((G = this.layout) == null ? void 0 : G.bbox);
      return V == null ? H : { ...H, bbox: V };
    }
  }
  mt(AnnotationView, "__name__", "AnnotationView");
  var M0;
  let Annotation$1 = (M0 = class extends CompositeRenderer {
    constructor(_) {
      super(_);
    }
  }, mt(M0, "__name__", "Annotation"), M0.override({
    level: "annotation"
  }), M0);
  const OpaqueIndices = Arrayable(Int$1), MultiIndices = Mapping(Int$1, OpaqueIndices), pp = class pp extends Model$1 {
    constructor(_) {
      super(_);
    }
    get_view() {
      return this.view;
    }
    get selected_glyph() {
      return this.selected_glyphs.length > 0 ? this.selected_glyphs[0] : null;
    }
    add_to_selected_glyphs(_) {
      this.selected_glyphs.push(_);
    }
    update(_, H = !0, V = "replace") {
      switch (V) {
        case "replace": {
          this.update_through_replacement(_);
          break;
        }
        case "toggle": {
          this.update_through_toggle(_);
          break;
        }
        case "append": {
          this.update_through_union(_);
          break;
        }
        case "intersect": {
          this.update_through_intersection(_);
          break;
        }
        case "subtract": {
          this.update_through_subtraction(_);
          break;
        }
        case "xor": {
          this.update_through_symmetric_difference(_);
          break;
        }
      }
    }
    // TODO `size` wouldn't be needed if `indices` was an instance of
    // `Indices` class, instead of an array of numbers. Then also we
    // could just call `.invert()` on the class.
    invert(_) {
      const H = new Set(this.indices), V = [];
      for (let q = 0; q < _; q++)
        H.has(q) || V.push(q);
      this.indices = V;
    }
    clear() {
      this.indices = [], this.line_indices = [], this.multiline_indices = /* @__PURE__ */ new Map(), this.image_indices = [], this.view = null, this.selected_glyphs = [];
    }
    map(_) {
      return new pp({
        ...this.attributes,
        indices: map$4(this.indices, _),
        // NOTE: line_indices don't support subset indexing
        multiline_indices: new Map(map$4([...this.multiline_indices.entries()], ([H, V]) => [_(H), V])),
        image_indices: this.image_indices.map((H) => ({ ...H, index: _(H.index) }))
      });
    }
    is_empty() {
      return this.indices.length == 0 && this.line_indices.length == 0 && this.image_indices.length == 0;
    }
    _union_image_indices(..._) {
      const H = /* @__PURE__ */ new Map(), V = /* @__PURE__ */ new Map(), q = [];
      for (const G of _)
        for (const J of G) {
          const { index: Z, i: te, j: ne } = J, ee = H.get(Z), ae = V.get(Z);
          ee != null && ae != null ? (!ee.has(te) || !ae.has(ne)) && (q.push(J), ee.add(te), ae.add(ne)) : (q.push(J), H.set(Z, /* @__PURE__ */ new Set([te])), V.set(Z, /* @__PURE__ */ new Set([ne])));
        }
      return q;
    }
    update_through_replacement(_) {
      this.indices = _.indices, this.line_indices = _.line_indices, this.multiline_indices = _.multiline_indices, this.image_indices = _.image_indices, this.view = _.view, this.selected_glyphs = _.selected_glyphs;
    }
    update_through_toggle(_) {
      this.indices = difference$1(_.indices, this.indices), this.selected_glyphs = union$3(_.selected_glyphs, this.selected_glyphs), this.line_indices = union$3(_.line_indices, this.line_indices), this.image_indices = this._union_image_indices(this.image_indices, _.image_indices), this.view = _.view, this.multiline_indices = merge$4(_.multiline_indices, this.multiline_indices);
    }
    update_through_union(_) {
      this.indices = union$3(this.indices, _.indices), this.selected_glyphs = union$3(_.selected_glyphs, this.selected_glyphs), this.line_indices = union$3(_.line_indices, this.line_indices), this.image_indices = this._union_image_indices(this.image_indices, _.image_indices), this.view = _.view, this.multiline_indices = merge$4(_.multiline_indices, this.multiline_indices);
    }
    update_through_intersection(_) {
      this.indices = intersection$1(this.indices, _.indices), this.selected_glyphs = union$3(_.selected_glyphs, this.selected_glyphs), this.line_indices = union$3(_.line_indices, this.line_indices), this.image_indices = this._union_image_indices(this.image_indices, _.image_indices), this.view = _.view, this.multiline_indices = merge$4(_.multiline_indices, this.multiline_indices);
    }
    update_through_subtraction(_) {
      this.indices = difference$1(this.indices, _.indices), this.selected_glyphs = union$3(_.selected_glyphs, this.selected_glyphs), this.line_indices = union$3(_.line_indices, this.line_indices), this.image_indices = this._union_image_indices(this.image_indices, _.image_indices), this.view = _.view, this.multiline_indices = merge$4(_.multiline_indices, this.multiline_indices);
    }
    update_through_symmetric_difference(_) {
      this.indices = symmetric_difference(this.indices, _.indices), this.selected_glyphs = union$3(_.selected_glyphs, this.selected_glyphs), this.line_indices = union$3(_.line_indices, this.line_indices), this.image_indices = this._union_image_indices(this.image_indices, _.image_indices), this.view = _.view, this.multiline_indices = merge$4(_.multiline_indices, this.multiline_indices);
    }
  };
  mt(pp, "__name__", "Selection"), pp.define(({ Int: _, List: H, Struct: V }) => ({
    indices: [OpaqueIndices, []],
    line_indices: [OpaqueIndices, []],
    multiline_indices: [MultiIndices, /* @__PURE__ */ new Map()],
    image_indices: [H(V({ index: _, i: _, j: _, flat_index: _ })), []]
  })), pp.internal(({ List: _, AnyRef: H, Nullable: V }) => ({
    selected_glyphs: [_(H()), []],
    view: [V(H()), null]
  }));
  let Selection = pp;
  function is_GlyphRendererView(O) {
    return O.model.type == "GlyphRenderer";
  }
  function is_GraphRendererView(O) {
    return O.model.type == "GraphRenderer";
  }
  class SelectionManager {
    constructor(_) {
      mt(this, "source");
      mt(this, "inspectors", /* @__PURE__ */ new Map());
      this.source = _;
    }
    select(_, H, V, q = "replace") {
      const G = [], J = [];
      for (const te of _)
        is_GlyphRendererView(te) ? G.push(te) : is_GraphRendererView(te) && J.push(te);
      let Z = !1;
      for (const te of J) {
        const ne = te.model.selection_policy.hit_test(H, te);
        Z = Z || te.model.selection_policy.do_selection(ne, te.model, V, q);
      }
      if (G.length > 0) {
        const te = this.source.selection_policy.hit_test(H, G);
        Z = Z || this.source.selection_policy.do_selection(te, this.source, V, q);
      }
      return Z;
    }
    inspect(_, H) {
      let V = !1;
      if (is_GlyphRendererView(_)) {
        const q = _.hit_test(H);
        if (q != null) {
          V = !q.is_empty();
          const G = this.get_or_create_inspector(_.model);
          G.update(q, !0, "replace"), this.source.setv({ inspected: G }, { silent: !0 }), this.source.inspect.emit([_.model, { geometry: H }]);
        }
      } else if (is_GraphRendererView(_)) {
        const q = _.model.inspection_policy.hit_test(H, _);
        V = _.model.inspection_policy.do_inspection(q, H, _, !1, "replace");
      }
      return V;
    }
    invert(_) {
      const H = this.source.get_length();
      H != null && (this.source.selected.invert(H), _ != null && this.get_or_create_inspector(_.model).invert(H));
    }
    clear(_) {
      this.source.selected.clear(), _ != null && this.get_or_create_inspector(_.model).clear();
    }
    get_or_create_inspector(_) {
      let H = this.inspectors.get(_);
      return H == null && (H = new Selection(), this.inspectors.set(_, H)), H;
    }
  }
  mt(SelectionManager, "__name__", "SelectionManager");
  class SelectionPolicy extends Model$1 {
    do_selection(_, H, V, q) {
      return _ == null ? !1 : (H.selected.update(_, V, q), H._select.emit(), !H.selected.is_empty());
    }
  }
  mt(SelectionPolicy, "__name__", "SelectionPolicy");
  class IntersectRenderers extends SelectionPolicy {
    hit_test(_, H) {
      const V = [];
      for (const q of H) {
        const G = q.hit_test(_);
        G != null && V.push(G);
      }
      if (V.length > 0) {
        const q = V[0];
        for (const G of V)
          q.update_through_intersection(G);
        return q;
      } else
        return null;
    }
  }
  mt(IntersectRenderers, "__name__", "IntersectRenderers");
  class UnionRenderers extends SelectionPolicy {
    hit_test(_, H) {
      const V = [];
      for (const q of H) {
        const G = q.hit_test(_);
        G != null && V.push(G);
      }
      if (V.length > 0) {
        const q = V[0];
        for (const G of V)
          q.update_through_union(G);
        return q;
      } else
        return null;
    }
  }
  mt(UnionRenderers, "__name__", "UnionRenderers");
  const xg = class xg extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(xg, "__name__", "DataSource"), xg.define(({ Ref: _ }) => ({
    selected: [_(Selection), () => new Selection(), { readonly: !0 }]
  }));
  let DataSource = xg;
  const I3 = class I3 extends DataSource {
    constructor(H) {
      super(H);
      mt(this, "_select");
      mt(this, "inspect");
      mt(this, "selection_manager", new SelectionManager(this));
    }
    get_array(H) {
      const V = dict(this.data);
      let q = V.get(H);
      return q == null ? V.set(H, q = []) : isArray$3(q) || V.set(H, q = Array.from(q)), q;
    }
    initialize() {
      super.initialize(), this._select = new Signal0(this, "select"), this.inspect = new Signal(this, "inspect");
    }
    get inferred_defaults() {
      const H = /* @__PURE__ */ new Map();
      for (const [V, q] of entries$2(this.data)) {
        const G = (() => {
          if (is_NDArray(q))
            switch (q.dtype) {
              case "bool":
                return !1;
              case "uint8":
              case "int8":
              case "uint16":
              case "int16":
              case "uint32":
              case "int32":
              case "float32":
              case "float64":
                return 0;
              case "object":
                return null;
            }
          else if (isArray$3(q) && q.length != 0) {
            const [J] = q;
            if (J === null)
              return null;
            if (isBoolean$4(J))
              return !1;
            if (isNumber$3(J))
              return 0;
            if (isString$2(J))
              return "";
            if (isArray$3(J))
              return [];
          }
        })();
        G !== void 0 && H.set(V, G);
      }
      return H;
    }
    get(H) {
      const V = this.get_column(H);
      return assert$2(V != null, `unknown column '${H}' in ${this}`), V;
    }
    set(H, V) {
      dict(this.data).set(H, V);
    }
    get_column(H) {
      return dict(this.data).get(H) ?? null;
    }
    columns() {
      return keys$5(this.data);
    }
    get_length(H = !0) {
      const V = uniq(values$3(this.data).map((q) => is_NDArray(q) ? q.shape[0] : q.length));
      switch (V.length) {
        case 0:
          return null;
        case 1:
          return V[0];
        default: {
          const q = "data source has columns of inconsistent lengths";
          if (H)
            return logger$1.warn(q), V.sort()[0];
          throw new Error(q);
        }
      }
    }
    get length() {
      return this.get_length() ?? 0;
    }
    clear() {
      const H = clone$1(this.data), V = dict(H);
      for (const [q, G] of V) {
        const J = new G.constructor(0);
        V.set(q, J);
      }
      this.data = H;
    }
    stream(H, V, { sync: q } = {}) {
      this.stream_to(this.properties.data, H, V, { sync: q });
    }
    patch(H, { sync: V } = {}) {
      this.patch_to(this.properties.data, H, { sync: V });
    }
  };
  mt(I3, "__name__", "ColumnarDataSource"), I3.define(({ Ref: H, Dict: V, Unknown: q }) => ({
    default_values: [V(q), {}],
    selection_policy: [H(SelectionPolicy), () => new UnionRenderers()]
  })), I3.internal(({ AnyRef: H }) => ({
    inspected: [H(), () => new Selection()]
  }));
  let ColumnarDataSource = I3;
  const wg = class wg extends ColumnarDataSource {
    constructor(_) {
      super(_);
    }
  };
  mt(wg, "__name__", "ColumnDataSource"), wg.define(({ Unknown: _, Dict: H, Arrayable: V }) => ({
    data: [H(V(_)), {}]
  }));
  let ColumnDataSource = wg;
  function globals(O) {
    O("EPSG:4326", "+title=WGS 84 (long/lat) +proj=longlat +ellps=WGS84 +datum=WGS84 +units=degrees"), O("EPSG:4269", "+title=NAD83 (long/lat) +proj=longlat +a=6378137.0 +b=6356752.31414036 +ellps=GRS80 +datum=NAD83 +units=degrees"), O("EPSG:3857", "+title=WGS 84 / Pseudo-Mercator +proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs"), O.WGS84 = O["EPSG:4326"], O["EPSG:3785"] = O["EPSG:3857"], O.GOOGLE = O["EPSG:3857"], O["EPSG:900913"] = O["EPSG:3857"], O["EPSG:102113"] = O["EPSG:3857"];
  }
  var PJD_3PARAM = 1, PJD_7PARAM = 2, PJD_GRIDSHIFT = 3, PJD_WGS84 = 4, PJD_NODATUM = 5, SRS_WGS84_SEMIMAJOR = 6378137, SRS_WGS84_SEMIMINOR = 6356752314e-3, SRS_WGS84_ESQUARED = 0.0066943799901413165, SEC_TO_RAD = 484813681109536e-20, HALF_PI = Math.PI / 2, SIXTH = 0.16666666666666666, RA4 = 0.04722222222222222, RA6 = 0.022156084656084655, EPSLN = 1e-10, D2R$1 = 0.017453292519943295, R2D = 57.29577951308232, FORTPI = Math.PI / 4, TWO_PI = Math.PI * 2, SPI = 3.14159265359, exports$3 = {};
  exports$3.greenwich = 0;
  exports$3.lisbon = -9.131906111111;
  exports$3.paris = 2.337229166667;
  exports$3.bogota = -74.080916666667;
  exports$3.madrid = -3.687938888889;
  exports$3.rome = 12.452333333333;
  exports$3.bern = 7.439583333333;
  exports$3.jakarta = 106.807719444444;
  exports$3.ferro = -17.666666666667;
  exports$3.brussels = 4.367975;
  exports$3.stockholm = 18.058277777778;
  exports$3.athens = 23.7163375;
  exports$3.oslo = 10.722916666667;
  const units = {
    ft: { to_meter: 0.3048 },
    "us-ft": { to_meter: 1200 / 3937 }
  };
  var ignoredChar = /[\s_\-\/\(\)]/g;
  function match$1(O, _) {
    if (O[_])
      return O[_];
    for (var H = Object.keys(O), V = _.toLowerCase().replace(ignoredChar, ""), q = -1, G, J; ++q < H.length; )
      if (G = H[q], J = G.toLowerCase().replace(ignoredChar, ""), J === V)
        return O[G];
  }
  function projStr(O) {
    var _ = {}, H = O.split("+").map(function(Z) {
      return Z.trim();
    }).filter(function(Z) {
      return Z;
    }).reduce(function(Z, te) {
      var ne = te.split("=");
      return ne.push(!0), Z[ne[0].toLowerCase()] = ne[1], Z;
    }, {}), V, q, G, J = {
      proj: "projName",
      datum: "datumCode",
      rf: function(Z) {
        _.rf = parseFloat(Z);
      },
      lat_0: function(Z) {
        _.lat0 = Z * D2R$1;
      },
      lat_1: function(Z) {
        _.lat1 = Z * D2R$1;
      },
      lat_2: function(Z) {
        _.lat2 = Z * D2R$1;
      },
      lat_ts: function(Z) {
        _.lat_ts = Z * D2R$1;
      },
      lon_0: function(Z) {
        _.long0 = Z * D2R$1;
      },
      lon_1: function(Z) {
        _.long1 = Z * D2R$1;
      },
      lon_2: function(Z) {
        _.long2 = Z * D2R$1;
      },
      alpha: function(Z) {
        _.alpha = parseFloat(Z) * D2R$1;
      },
      gamma: function(Z) {
        _.rectified_grid_angle = parseFloat(Z);
      },
      lonc: function(Z) {
        _.longc = Z * D2R$1;
      },
      x_0: function(Z) {
        _.x0 = parseFloat(Z);
      },
      y_0: function(Z) {
        _.y0 = parseFloat(Z);
      },
      k_0: function(Z) {
        _.k0 = parseFloat(Z);
      },
      k: function(Z) {
        _.k0 = parseFloat(Z);
      },
      a: function(Z) {
        _.a = parseFloat(Z);
      },
      b: function(Z) {
        _.b = parseFloat(Z);
      },
      r: function(Z) {
        _.a = _.b = parseFloat(Z);
      },
      r_a: function() {
        _.R_A = !0;
      },
      zone: function(Z) {
        _.zone = parseInt(Z, 10);
      },
      south: function() {
        _.utmSouth = !0;
      },
      towgs84: function(Z) {
        _.datum_params = Z.split(",").map(function(te) {
          return parseFloat(te);
        });
      },
      to_meter: function(Z) {
        _.to_meter = parseFloat(Z);
      },
      units: function(Z) {
        _.units = Z;
        var te = match$1(units, Z);
        te && (_.to_meter = te.to_meter);
      },
      from_greenwich: function(Z) {
        _.from_greenwich = Z * D2R$1;
      },
      pm: function(Z) {
        var te = match$1(exports$3, Z);
        _.from_greenwich = (te || parseFloat(Z)) * D2R$1;
      },
      nadgrids: function(Z) {
        Z === "@null" ? _.datumCode = "none" : _.nadgrids = Z;
      },
      axis: function(Z) {
        var te = "ewnsud";
        Z.length === 3 && te.indexOf(Z.substr(0, 1)) !== -1 && te.indexOf(Z.substr(1, 1)) !== -1 && te.indexOf(Z.substr(2, 1)) !== -1 && (_.axis = Z);
      },
      approx: function() {
        _.approx = !0;
      }
    };
    for (V in H)
      q = H[V], V in J ? (G = J[V], typeof G == "function" ? G(q) : _[G] = q) : _[V] = q;
    return typeof _.datumCode == "string" && _.datumCode !== "WGS84" && (_.datumCode = _.datumCode.toLowerCase()), _;
  }
  var NEUTRAL = 1, KEYWORD = 2, NUMBER = 3, QUOTED = 4, AFTERQUOTE = 5, ENDED = -1, whitespace$1 = /\s/, latin = /[A-Za-z]/, keyword$1 = /[A-Za-z84_]/, endThings = /[,\]]/, digets = /[\d\.E\-\+]/;
  function Parser$2(O) {
    if (typeof O != "string")
      throw new Error("not a string");
    this.text = O.trim(), this.level = 0, this.place = 0, this.root = null, this.stack = [], this.currentObject = null, this.state = NEUTRAL;
  }
  Parser$2.prototype.readCharicter = function() {
    var O = this.text[this.place++];
    if (this.state !== QUOTED)
      for (; whitespace$1.test(O); ) {
        if (this.place >= this.text.length)
          return;
        O = this.text[this.place++];
      }
    switch (this.state) {
      case NEUTRAL:
        return this.neutral(O);
      case KEYWORD:
        return this.keyword(O);
      case QUOTED:
        return this.quoted(O);
      case AFTERQUOTE:
        return this.afterquote(O);
      case NUMBER:
        return this.number(O);
      case ENDED:
        return;
    }
  };
  Parser$2.prototype.afterquote = function(O) {
    if (O === '"') {
      this.word += '"', this.state = QUOTED;
      return;
    }
    if (endThings.test(O)) {
      this.word = this.word.trim(), this.afterItem(O);
      return;
    }
    throw new Error(`havn't handled "` + O + '" in afterquote yet, index ' + this.place);
  };
  Parser$2.prototype.afterItem = function(O) {
    if (O === ",") {
      this.word !== null && this.currentObject.push(this.word), this.word = null, this.state = NEUTRAL;
      return;
    }
    if (O === "]") {
      this.level--, this.word !== null && (this.currentObject.push(this.word), this.word = null), this.state = NEUTRAL, this.currentObject = this.stack.pop(), this.currentObject || (this.state = ENDED);
      return;
    }
  };
  Parser$2.prototype.number = function(O) {
    if (digets.test(O)) {
      this.word += O;
      return;
    }
    if (endThings.test(O)) {
      this.word = parseFloat(this.word), this.afterItem(O);
      return;
    }
    throw new Error(`havn't handled "` + O + '" in number yet, index ' + this.place);
  };
  Parser$2.prototype.quoted = function(O) {
    if (O === '"') {
      this.state = AFTERQUOTE;
      return;
    }
    this.word += O;
  };
  Parser$2.prototype.keyword = function(O) {
    if (keyword$1.test(O)) {
      this.word += O;
      return;
    }
    if (O === "[") {
      var _ = [];
      _.push(this.word), this.level++, this.root === null ? this.root = _ : this.currentObject.push(_), this.stack.push(this.currentObject), this.currentObject = _, this.state = NEUTRAL;
      return;
    }
    if (endThings.test(O)) {
      this.afterItem(O);
      return;
    }
    throw new Error(`havn't handled "` + O + '" in keyword yet, index ' + this.place);
  };
  Parser$2.prototype.neutral = function(O) {
    if (latin.test(O)) {
      this.word = O, this.state = KEYWORD;
      return;
    }
    if (O === '"') {
      this.word = "", this.state = QUOTED;
      return;
    }
    if (digets.test(O)) {
      this.word = O, this.state = NUMBER;
      return;
    }
    if (endThings.test(O)) {
      this.afterItem(O);
      return;
    }
    throw new Error(`havn't handled "` + O + '" in neutral yet, index ' + this.place);
  };
  Parser$2.prototype.output = function() {
    for (; this.place < this.text.length; )
      this.readCharicter();
    if (this.state === ENDED)
      return this.root;
    throw new Error('unable to parse string "' + this.text + '". State is ' + this.state);
  };
  function parseString(O) {
    var _ = new Parser$2(O);
    return _.output();
  }
  function mapit(O, _, H) {
    Array.isArray(_) && (H.unshift(_), _ = null);
    var V = _ ? {} : O, q = H.reduce(function(G, J) {
      return sExpr(J, G), G;
    }, V);
    _ && (O[_] = q);
  }
  function sExpr(O, _) {
    if (!Array.isArray(O)) {
      _[O] = !0;
      return;
    }
    var H = O.shift();
    if (H === "PARAMETER" && (H = O.shift()), O.length === 1) {
      if (Array.isArray(O[0])) {
        _[H] = {}, sExpr(O[0], _[H]);
        return;
      }
      _[H] = O[0];
      return;
    }
    if (!O.length) {
      _[H] = !0;
      return;
    }
    if (H === "TOWGS84") {
      _[H] = O;
      return;
    }
    if (H === "AXIS") {
      H in _ || (_[H] = []), _[H].push(O);
      return;
    }
    Array.isArray(H) || (_[H] = {});
    var V;
    switch (H) {
      case "UNIT":
      case "PRIMEM":
      case "VERT_DATUM":
        _[H] = {
          name: O[0].toLowerCase(),
          convert: O[1]
        }, O.length === 3 && sExpr(O[2], _[H]);
        return;
      case "SPHEROID":
      case "ELLIPSOID":
        _[H] = {
          name: O[0],
          a: O[1],
          rf: O[2]
        }, O.length === 4 && sExpr(O[3], _[H]);
        return;
      case "PROJECTEDCRS":
      case "PROJCRS":
      case "GEOGCS":
      case "GEOCCS":
      case "PROJCS":
      case "LOCAL_CS":
      case "GEODCRS":
      case "GEODETICCRS":
      case "GEODETICDATUM":
      case "EDATUM":
      case "ENGINEERINGDATUM":
      case "VERT_CS":
      case "VERTCRS":
      case "VERTICALCRS":
      case "COMPD_CS":
      case "COMPOUNDCRS":
      case "ENGINEERINGCRS":
      case "ENGCRS":
      case "FITTED_CS":
      case "LOCAL_DATUM":
      case "DATUM":
        O[0] = ["name", O[0]], mapit(_, H, O);
        return;
      default:
        for (V = -1; ++V < O.length; )
          if (!Array.isArray(O[V]))
            return sExpr(O, _[H]);
        return mapit(_, H, O);
    }
  }
  var D2R = 0.017453292519943295;
  function rename(O, _) {
    var H = _[0], V = _[1];
    !(H in O) && V in O && (O[H] = O[V], _.length === 3 && (O[H] = _[2](O[H])));
  }
  function d2r(O) {
    return O * D2R;
  }
  function cleanWKT(O) {
    if (O.type === "GEOGCS" ? O.projName = "longlat" : O.type === "LOCAL_CS" ? (O.projName = "identity", O.local = !0) : typeof O.PROJECTION == "object" ? O.projName = Object.keys(O.PROJECTION)[0] : O.projName = O.PROJECTION, O.AXIS) {
      for (var _ = "", H = 0, V = O.AXIS.length; H < V; ++H) {
        var q = [O.AXIS[H][0].toLowerCase(), O.AXIS[H][1].toLowerCase()];
        q[0].indexOf("north") !== -1 || (q[0] === "y" || q[0] === "lat") && q[1] === "north" ? _ += "n" : q[0].indexOf("south") !== -1 || (q[0] === "y" || q[0] === "lat") && q[1] === "south" ? _ += "s" : q[0].indexOf("east") !== -1 || (q[0] === "x" || q[0] === "lon") && q[1] === "east" ? _ += "e" : (q[0].indexOf("west") !== -1 || (q[0] === "x" || q[0] === "lon") && q[1] === "west") && (_ += "w");
      }
      _.length === 2 && (_ += "u"), _.length === 3 && (O.axis = _);
    }
    O.UNIT && (O.units = O.UNIT.name.toLowerCase(), O.units === "metre" && (O.units = "meter"), O.UNIT.convert && (O.type === "GEOGCS" ? O.DATUM && O.DATUM.SPHEROID && (O.to_meter = O.UNIT.convert * O.DATUM.SPHEROID.a) : O.to_meter = O.UNIT.convert));
    var G = O.GEOGCS;
    O.type === "GEOGCS" && (G = O), G && (G.DATUM ? O.datumCode = G.DATUM.name.toLowerCase() : O.datumCode = G.name.toLowerCase(), O.datumCode.slice(0, 2) === "d_" && (O.datumCode = O.datumCode.slice(2)), (O.datumCode === "new_zealand_geodetic_datum_1949" || O.datumCode === "new_zealand_1949") && (O.datumCode = "nzgd49"), (O.datumCode === "wgs_1984" || O.datumCode === "world_geodetic_system_1984") && (O.PROJECTION === "Mercator_Auxiliary_Sphere" && (O.sphere = !0), O.datumCode = "wgs84"), O.datumCode.slice(-6) === "_ferro" && (O.datumCode = O.datumCode.slice(0, -6)), O.datumCode.slice(-8) === "_jakarta" && (O.datumCode = O.datumCode.slice(0, -8)), ~O.datumCode.indexOf("belge") && (O.datumCode = "rnb72"), G.DATUM && G.DATUM.SPHEROID && (O.ellps = G.DATUM.SPHEROID.name.replace("_19", "").replace(/[Cc]larke\_18/, "clrk"), O.ellps.toLowerCase().slice(0, 13) === "international" && (O.ellps = "intl"), O.a = G.DATUM.SPHEROID.a, O.rf = parseFloat(G.DATUM.SPHEROID.rf, 10)), G.DATUM && G.DATUM.TOWGS84 && (O.datum_params = G.DATUM.TOWGS84), ~O.datumCode.indexOf("osgb_1936") && (O.datumCode = "osgb36"), ~O.datumCode.indexOf("osni_1952") && (O.datumCode = "osni52"), (~O.datumCode.indexOf("tm65") || ~O.datumCode.indexOf("geodetic_datum_of_1965")) && (O.datumCode = "ire65"), O.datumCode === "ch1903+" && (O.datumCode = "ch1903"), ~O.datumCode.indexOf("israel") && (O.datumCode = "isr93")), O.b && !isFinite(O.b) && (O.b = O.a);
    function J(ne) {
      var ee = O.to_meter || 1;
      return ne * ee;
    }
    var Z = function(ne) {
      return rename(O, ne);
    }, te = [
      ["standard_parallel_1", "Standard_Parallel_1"],
      ["standard_parallel_1", "Latitude of 1st standard parallel"],
      ["standard_parallel_2", "Standard_Parallel_2"],
      ["standard_parallel_2", "Latitude of 2nd standard parallel"],
      ["false_easting", "False_Easting"],
      ["false_easting", "False easting"],
      ["false-easting", "Easting at false origin"],
      ["false_northing", "False_Northing"],
      ["false_northing", "False northing"],
      ["false_northing", "Northing at false origin"],
      ["central_meridian", "Central_Meridian"],
      ["central_meridian", "Longitude of natural origin"],
      ["central_meridian", "Longitude of false origin"],
      ["latitude_of_origin", "Latitude_Of_Origin"],
      ["latitude_of_origin", "Central_Parallel"],
      ["latitude_of_origin", "Latitude of natural origin"],
      ["latitude_of_origin", "Latitude of false origin"],
      ["scale_factor", "Scale_Factor"],
      ["k0", "scale_factor"],
      ["latitude_of_center", "Latitude_Of_Center"],
      ["latitude_of_center", "Latitude_of_center"],
      ["lat0", "latitude_of_center", d2r],
      ["longitude_of_center", "Longitude_Of_Center"],
      ["longitude_of_center", "Longitude_of_center"],
      ["longc", "longitude_of_center", d2r],
      ["x0", "false_easting", J],
      ["y0", "false_northing", J],
      ["long0", "central_meridian", d2r],
      ["lat0", "latitude_of_origin", d2r],
      ["lat0", "standard_parallel_1", d2r],
      ["lat1", "standard_parallel_1", d2r],
      ["lat2", "standard_parallel_2", d2r],
      ["azimuth", "Azimuth"],
      ["alpha", "azimuth", d2r],
      ["srsCode", "name"]
    ];
    te.forEach(Z), !O.long0 && O.longc && (O.projName === "Albers_Conic_Equal_Area" || O.projName === "Lambert_Azimuthal_Equal_Area") && (O.long0 = O.longc), !O.lat_ts && O.lat1 && (O.projName === "Stereographic_South_Pole" || O.projName === "Polar Stereographic (variant B)") ? (O.lat0 = d2r(O.lat1 > 0 ? 90 : -90), O.lat_ts = O.lat1) : !O.lat_ts && O.lat0 && O.projName === "Polar_Stereographic" && (O.lat_ts = O.lat0, O.lat0 = d2r(O.lat0 > 0 ? 90 : -90));
  }
  function wkt(O) {
    var _ = parseString(O), H = _.shift(), V = _.shift();
    _.unshift(["name", V]), _.unshift(["type", H]);
    var q = {};
    return sExpr(_, q), cleanWKT(q), q;
  }
  function defs(O) {
    var _ = this;
    if (arguments.length === 2) {
      var H = arguments[1];
      typeof H == "string" ? H.charAt(0) === "+" ? defs[O] = projStr(arguments[1]) : defs[O] = wkt(arguments[1]) : defs[O] = H;
    } else if (arguments.length === 1) {
      if (Array.isArray(O))
        return O.map(function(V) {
          Array.isArray(V) ? defs.apply(_, V) : defs(V);
        });
      if (typeof O == "string") {
        if (O in defs)
          return defs[O];
      } else "EPSG" in O ? defs["EPSG:" + O.EPSG] = O : "ESRI" in O ? defs["ESRI:" + O.ESRI] = O : "IAU2000" in O ? defs["IAU2000:" + O.IAU2000] = O : console.log(O);
      return;
    }
  }
  globals(defs);
  function testObj(O) {
    return typeof O == "string";
  }
  function testDef(O) {
    return O in defs;
  }
  var codeWords = ["PROJECTEDCRS", "PROJCRS", "GEOGCS", "GEOCCS", "PROJCS", "LOCAL_CS", "GEODCRS", "GEODETICCRS", "GEODETICDATUM", "ENGCRS", "ENGINEERINGCRS"];
  function testWKT(O) {
    return codeWords.some(function(_) {
      return O.indexOf(_) > -1;
    });
  }
  var codes$1 = ["3857", "900913", "3785", "102113"];
  function checkMercator(O) {
    var _ = match$1(O, "authority");
    if (_) {
      var H = match$1(_, "epsg");
      return H && codes$1.indexOf(H) > -1;
    }
  }
  function checkProjStr(O) {
    var _ = match$1(O, "extension");
    if (_)
      return match$1(_, "proj4");
  }
  function testProj(O) {
    return O[0] === "+";
  }
  function parse$7(O) {
    if (testObj(O)) {
      if (testDef(O))
        return defs[O];
      if (testWKT(O)) {
        var _ = wkt(O);
        if (checkMercator(_))
          return defs["EPSG:3857"];
        var H = checkProjStr(_);
        return H ? projStr(H) : _;
      }
      if (testProj(O))
        return projStr(O);
    } else
      return O;
  }
  function extend$4(O, _) {
    O = O || {};
    var H, V;
    if (!_)
      return O;
    for (V in _)
      H = _[V], H !== void 0 && (O[V] = H);
    return O;
  }
  function msfnz(O, _, H) {
    var V = O * _;
    return H / Math.sqrt(1 - V * V);
  }
  function sign$2(O) {
    return O < 0 ? -1 : 1;
  }
  function adjust_lon(O) {
    return Math.abs(O) <= SPI ? O : O - sign$2(O) * TWO_PI;
  }
  function tsfnz(O, _, H) {
    var V = O * H, q = 0.5 * O;
    return V = Math.pow((1 - V) / (1 + V), q), Math.tan(0.5 * (HALF_PI - _)) / V;
  }
  function phi2z(O, _) {
    for (var H = 0.5 * O, V, q, G = HALF_PI - 2 * Math.atan(_), J = 0; J <= 15; J++)
      if (V = O * Math.sin(G), q = HALF_PI - 2 * Math.atan(_ * Math.pow((1 - V) / (1 + V), H)) - G, G += q, Math.abs(q) <= 1e-10)
        return G;
    return -9999;
  }
  function init$2() {
    var O = this.b / this.a;
    this.es = 1 - O * O, "x0" in this || (this.x0 = 0), "y0" in this || (this.y0 = 0), this.e = Math.sqrt(this.es), this.lat_ts ? this.sphere ? this.k0 = Math.cos(this.lat_ts) : this.k0 = msfnz(this.e, Math.sin(this.lat_ts), Math.cos(this.lat_ts)) : this.k0 || (this.k ? this.k0 = this.k : this.k0 = 1);
  }
  function forward(O) {
    var _ = O.x, H = O.y;
    if (H * R2D > 90 && H * R2D < -90 && _ * R2D > 180 && _ * R2D < -180)
      return null;
    var V, q;
    if (Math.abs(Math.abs(H) - HALF_PI) <= EPSLN)
      return null;
    if (this.sphere)
      V = this.x0 + this.a * this.k0 * adjust_lon(_ - this.long0), q = this.y0 + this.a * this.k0 * Math.log(Math.tan(FORTPI + 0.5 * H));
    else {
      var G = Math.sin(H), J = tsfnz(this.e, H, G);
      V = this.x0 + this.a * this.k0 * adjust_lon(_ - this.long0), q = this.y0 - this.a * this.k0 * Math.log(J);
    }
    return O.x = V, O.y = q, O;
  }
  function inverse(O) {
    var _ = O.x - this.x0, H = O.y - this.y0, V, q;
    if (this.sphere)
      q = HALF_PI - 2 * Math.atan(Math.exp(-H / (this.a * this.k0)));
    else {
      var G = Math.exp(-H / (this.a * this.k0));
      if (q = phi2z(this.e, G), q === -9999)
        return null;
    }
    return V = adjust_lon(this.long0 + _ / (this.a * this.k0)), O.x = V, O.y = q, O;
  }
  var names$2 = ["Mercator", "Popular Visualisation Pseudo Mercator", "Mercator_1SP", "Mercator_Auxiliary_Sphere", "merc"];
  const merc = {
    init: init$2,
    forward,
    inverse,
    names: names$2
  };
  function init$1() {
  }
  function identity$8(O) {
    return O;
  }
  var names$1 = ["longlat", "identity"];
  const longlat = {
    init: init$1,
    forward: identity$8,
    inverse: identity$8,
    names: names$1
  };
  var projs = [merc, longlat], names = {}, projStore = [];
  function add$6(O, _) {
    var H = projStore.length;
    return O.names ? (projStore[H] = O, O.names.forEach(function(V) {
      names[V.toLowerCase()] = H;
    }), this) : (console.log(_), !0);
  }
  function get$6(O) {
    if (!O)
      return !1;
    var _ = O.toLowerCase();
    if (typeof names[_] < "u" && projStore[names[_]])
      return projStore[names[_]];
  }
  function start() {
    projs.forEach(add$6);
  }
  const projections$1 = {
    start,
    add: add$6,
    get: get$6
  };
  var exports$2 = {};
  exports$2.MERIT = {
    a: 6378137,
    rf: 298.257,
    ellipseName: "MERIT 1983"
  };
  exports$2.SGS85 = {
    a: 6378136,
    rf: 298.257,
    ellipseName: "Soviet Geodetic System 85"
  };
  exports$2.GRS80 = {
    a: 6378137,
    rf: 298.257222101,
    ellipseName: "GRS 1980(IUGG, 1980)"
  };
  exports$2.IAU76 = {
    a: 6378140,
    rf: 298.257,
    ellipseName: "IAU 1976"
  };
  exports$2.airy = {
    a: 6377563396e-3,
    b: 635625691e-2,
    ellipseName: "Airy 1830"
  };
  exports$2.APL4 = {
    a: 6378137,
    rf: 298.25,
    ellipseName: "Appl. Physics. 1965"
  };
  exports$2.NWL9D = {
    a: 6378145,
    rf: 298.25,
    ellipseName: "Naval Weapons Lab., 1965"
  };
  exports$2.mod_airy = {
    a: 6377340189e-3,
    b: 6356034446e-3,
    ellipseName: "Modified Airy"
  };
  exports$2.andrae = {
    a: 637710443e-2,
    rf: 300,
    ellipseName: "Andrae 1876 (Den., Iclnd.)"
  };
  exports$2.aust_SA = {
    a: 6378160,
    rf: 298.25,
    ellipseName: "Australian Natl & S. Amer. 1969"
  };
  exports$2.GRS67 = {
    a: 6378160,
    rf: 298.247167427,
    ellipseName: "GRS 67(IUGG 1967)"
  };
  exports$2.bessel = {
    a: 6377397155e-3,
    rf: 299.1528128,
    ellipseName: "Bessel 1841"
  };
  exports$2.bess_nam = {
    a: 6377483865e-3,
    rf: 299.1528128,
    ellipseName: "Bessel 1841 (Namibia)"
  };
  exports$2.clrk66 = {
    a: 63782064e-1,
    b: 63565838e-1,
    ellipseName: "Clarke 1866"
  };
  exports$2.clrk80 = {
    a: 6378249145e-3,
    rf: 293.4663,
    ellipseName: "Clarke 1880 mod."
  };
  exports$2.clrk80ign = {
    a: 63782492e-1,
    b: 6356515,
    rf: 293.4660213,
    ellipseName: "Clarke 1880 (IGN)"
  };
  exports$2.clrk58 = {
    a: 6378293645208759e-9,
    rf: 294.2606763692654,
    ellipseName: "Clarke 1858"
  };
  exports$2.CPM = {
    a: 63757387e-1,
    rf: 334.29,
    ellipseName: "Comm. des Poids et Mesures 1799"
  };
  exports$2.delmbr = {
    a: 6376428,
    rf: 311.5,
    ellipseName: "Delambre 1810 (Belgium)"
  };
  exports$2.engelis = {
    a: 637813605e-2,
    rf: 298.2566,
    ellipseName: "Engelis 1985"
  };
  exports$2.evrst30 = {
    a: 6377276345e-3,
    rf: 300.8017,
    ellipseName: "Everest 1830"
  };
  exports$2.evrst48 = {
    a: 6377304063e-3,
    rf: 300.8017,
    ellipseName: "Everest 1948"
  };
  exports$2.evrst56 = {
    a: 6377301243e-3,
    rf: 300.8017,
    ellipseName: "Everest 1956"
  };
  exports$2.evrst69 = {
    a: 6377295664e-3,
    rf: 300.8017,
    ellipseName: "Everest 1969"
  };
  exports$2.evrstSS = {
    a: 6377298556e-3,
    rf: 300.8017,
    ellipseName: "Everest (Sabah & Sarawak)"
  };
  exports$2.fschr60 = {
    a: 6378166,
    rf: 298.3,
    ellipseName: "Fischer (Mercury Datum) 1960"
  };
  exports$2.fschr60m = {
    a: 6378155,
    rf: 298.3,
    ellipseName: "Fischer 1960"
  };
  exports$2.fschr68 = {
    a: 6378150,
    rf: 298.3,
    ellipseName: "Fischer 1968"
  };
  exports$2.helmert = {
    a: 6378200,
    rf: 298.3,
    ellipseName: "Helmert 1906"
  };
  exports$2.hough = {
    a: 6378270,
    rf: 297,
    ellipseName: "Hough"
  };
  exports$2.intl = {
    a: 6378388,
    rf: 297,
    ellipseName: "International 1909 (Hayford)"
  };
  exports$2.kaula = {
    a: 6378163,
    rf: 298.24,
    ellipseName: "Kaula 1961"
  };
  exports$2.lerch = {
    a: 6378139,
    rf: 298.257,
    ellipseName: "Lerch 1979"
  };
  exports$2.mprts = {
    a: 6397300,
    rf: 191,
    ellipseName: "Maupertius 1738"
  };
  exports$2.new_intl = {
    a: 63781575e-1,
    b: 63567722e-1,
    ellipseName: "New International 1967"
  };
  exports$2.plessis = {
    a: 6376523,
    rf: 6355863,
    ellipseName: "Plessis 1817 (France)"
  };
  exports$2.krass = {
    a: 6378245,
    rf: 298.3,
    ellipseName: "Krassovsky, 1942"
  };
  exports$2.SEasia = {
    a: 6378155,
    b: 63567733205e-4,
    ellipseName: "Southeast Asia"
  };
  exports$2.walbeck = {
    a: 6376896,
    b: 63558348467e-4,
    ellipseName: "Walbeck"
  };
  exports$2.WGS60 = {
    a: 6378165,
    rf: 298.3,
    ellipseName: "WGS 60"
  };
  exports$2.WGS66 = {
    a: 6378145,
    rf: 298.25,
    ellipseName: "WGS 66"
  };
  exports$2.WGS7 = {
    a: 6378135,
    rf: 298.26,
    ellipseName: "WGS 72"
  };
  var WGS84 = exports$2.WGS84 = {
    a: 6378137,
    rf: 298.257223563,
    ellipseName: "WGS 84"
  };
  exports$2.sphere = {
    a: 6370997,
    b: 6370997,
    ellipseName: "Normal Sphere (r=6370997)"
  };
  function eccentricity(O, _, H, V) {
    var q = O * O, G = _ * _, J = (q - G) / q, Z = 0;
    V ? (O *= 1 - J * (SIXTH + J * (RA4 + J * RA6)), q = O * O, J = 0) : Z = Math.sqrt(J);
    var te = (q - G) / G;
    return {
      es: J,
      e: Z,
      ep2: te
    };
  }
  function sphere(O, _, H, V, q) {
    if (!O) {
      var G = match$1(exports$2, V);
      G || (G = WGS84), O = G.a, _ = G.b, H = G.rf;
    }
    return H && !_ && (_ = (1 - 1 / H) * O), (H === 0 || Math.abs(O - _) < EPSLN) && (q = !0, _ = O), {
      a: O,
      b: _,
      rf: H,
      sphere: q
    };
  }
  var exports$1 = {};
  exports$1.wgs84 = {
    towgs84: "0,0,0",
    ellipse: "WGS84",
    datumName: "WGS84"
  };
  exports$1.ch1903 = {
    towgs84: "674.374,15.056,405.346",
    ellipse: "bessel",
    datumName: "swiss"
  };
  exports$1.ggrs87 = {
    towgs84: "-199.87,74.79,246.62",
    ellipse: "GRS80",
    datumName: "Greek_Geodetic_Reference_System_1987"
  };
  exports$1.nad83 = {
    towgs84: "0,0,0",
    ellipse: "GRS80",
    datumName: "North_American_Datum_1983"
  };
  exports$1.nad27 = {
    nadgrids: "@conus,@alaska,@ntv2_0.gsb,@ntv1_can.dat",
    ellipse: "clrk66",
    datumName: "North_American_Datum_1927"
  };
  exports$1.potsdam = {
    towgs84: "598.1,73.7,418.2,0.202,0.045,-2.455,6.7",
    ellipse: "bessel",
    datumName: "Potsdam Rauenberg 1950 DHDN"
  };
  exports$1.carthage = {
    towgs84: "-263.0,6.0,431.0",
    ellipse: "clark80",
    datumName: "Carthage 1934 Tunisia"
  };
  exports$1.hermannskogel = {
    towgs84: "577.326,90.129,463.919,5.137,1.474,5.297,2.4232",
    ellipse: "bessel",
    datumName: "Hermannskogel"
  };
  exports$1.militargeographische_institut = {
    towgs84: "577.326,90.129,463.919,5.137,1.474,5.297,2.4232",
    ellipse: "bessel",
    datumName: "Militar-Geographische Institut"
  };
  exports$1.osni52 = {
    towgs84: "482.530,-130.596,564.557,-1.042,-0.214,-0.631,8.15",
    ellipse: "airy",
    datumName: "Irish National"
  };
  exports$1.ire65 = {
    towgs84: "482.530,-130.596,564.557,-1.042,-0.214,-0.631,8.15",
    ellipse: "mod_airy",
    datumName: "Ireland 1965"
  };
  exports$1.rassadiran = {
    towgs84: "-133.63,-157.5,-158.62",
    ellipse: "intl",
    datumName: "Rassadiran"
  };
  exports$1.nzgd49 = {
    towgs84: "59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993",
    ellipse: "intl",
    datumName: "New Zealand Geodetic Datum 1949"
  };
  exports$1.osgb36 = {
    towgs84: "446.448,-125.157,542.060,0.1502,0.2470,0.8421,-20.4894",
    ellipse: "airy",
    datumName: "Airy 1830"
  };
  exports$1.s_jtsk = {
    towgs84: "589,76,480",
    ellipse: "bessel",
    datumName: "S-JTSK (Ferro)"
  };
  exports$1.beduaram = {
    towgs84: "-106,-87,188",
    ellipse: "clrk80",
    datumName: "Beduaram"
  };
  exports$1.gunung_segara = {
    towgs84: "-403,684,41",
    ellipse: "bessel",
    datumName: "Gunung Segara Jakarta"
  };
  exports$1.rnb72 = {
    towgs84: "106.869,-52.2978,103.724,-0.33657,0.456955,-1.84218,1",
    ellipse: "intl",
    datumName: "Reseau National Belge 1972"
  };
  function datum$1(O, _, H, V, q, G, J) {
    var Z = {};
    return O === void 0 || O === "none" ? Z.datum_type = PJD_NODATUM : Z.datum_type = PJD_WGS84, _ && (Z.datum_params = _.map(parseFloat), (Z.datum_params[0] !== 0 || Z.datum_params[1] !== 0 || Z.datum_params[2] !== 0) && (Z.datum_type = PJD_3PARAM), Z.datum_params.length > 3 && (Z.datum_params[3] !== 0 || Z.datum_params[4] !== 0 || Z.datum_params[5] !== 0 || Z.datum_params[6] !== 0) && (Z.datum_type = PJD_7PARAM, Z.datum_params[3] *= SEC_TO_RAD, Z.datum_params[4] *= SEC_TO_RAD, Z.datum_params[5] *= SEC_TO_RAD, Z.datum_params[6] = Z.datum_params[6] / 1e6 + 1)), J && (Z.datum_type = PJD_GRIDSHIFT, Z.grids = J), Z.a = H, Z.b = V, Z.es = q, Z.ep2 = G, Z;
  }
  var loadedNadgrids = {};
  function getNadgrids(O) {
    if (O === void 0)
      return null;
    var _ = O.split(",");
    return _.map(parseNadgridString);
  }
  function parseNadgridString(O) {
    if (O.length === 0)
      return null;
    var _ = O[0] === "@";
    return _ && (O = O.slice(1)), O === "null" ? { name: "null", mandatory: !_, grid: null, isNull: !0 } : {
      name: O,
      mandatory: !_,
      grid: loadedNadgrids[O] || null,
      isNull: !1
    };
  }
  function Projection$2(O, _) {
    if (!(this instanceof Projection$2))
      return new Projection$2(O);
    _ = _ || function(ne) {
      if (ne)
        throw ne;
    };
    var H = parse$7(O);
    if (typeof H != "object") {
      _("Could not parse to valid json: " + O);
      return;
    }
    var V = Projection$2.projections.get(H.projName);
    if (!V) {
      _("Could not get projection name from: " + O);
      return;
    }
    if (H.datumCode && H.datumCode !== "none") {
      var q = match$1(exports$1, H.datumCode);
      q && (H.datum_params = H.datum_params || (q.towgs84 ? q.towgs84.split(",") : null), H.ellps = q.ellipse, H.datumName = q.datumName ? q.datumName : H.datumCode);
    }
    H.k0 = H.k0 || 1, H.axis = H.axis || "enu", H.ellps = H.ellps || "wgs84", H.lat1 = H.lat1 || H.lat0;
    var G = sphere(H.a, H.b, H.rf, H.ellps, H.sphere), J = eccentricity(G.a, G.b, G.rf, H.R_A), Z = getNadgrids(H.nadgrids), te = H.datum || datum$1(
      H.datumCode,
      H.datum_params,
      G.a,
      G.b,
      J.es,
      J.ep2,
      Z
    );
    extend$4(this, H), extend$4(this, V), this.a = G.a, this.b = G.b, this.rf = G.rf, this.sphere = G.sphere, this.es = J.es, this.e = J.e, this.ep2 = J.ep2, this.datum = te, this.init(), _(null, this);
  }
  Projection$2.projections = projections$1;
  Projection$2.projections.start();
  function compareDatums(O, _) {
    return O.datum_type !== _.datum_type || O.a !== _.a || Math.abs(O.es - _.es) > 5e-11 ? !1 : O.datum_type === PJD_3PARAM ? O.datum_params[0] === _.datum_params[0] && O.datum_params[1] === _.datum_params[1] && O.datum_params[2] === _.datum_params[2] : O.datum_type === PJD_7PARAM ? O.datum_params[0] === _.datum_params[0] && O.datum_params[1] === _.datum_params[1] && O.datum_params[2] === _.datum_params[2] && O.datum_params[3] === _.datum_params[3] && O.datum_params[4] === _.datum_params[4] && O.datum_params[5] === _.datum_params[5] && O.datum_params[6] === _.datum_params[6] : !0;
  }
  function geodeticToGeocentric(O, _, H) {
    var V = O.x, q = O.y, G = O.z ? O.z : 0, J, Z, te, ne;
    if (q < -HALF_PI && q > -1.001 * HALF_PI)
      q = -HALF_PI;
    else if (q > HALF_PI && q < 1.001 * HALF_PI)
      q = HALF_PI;
    else {
      if (q < -HALF_PI)
        return { x: -1 / 0, y: -1 / 0, z: O.z };
      if (q > HALF_PI)
        return { x: 1 / 0, y: 1 / 0, z: O.z };
    }
    return V > Math.PI && (V -= 2 * Math.PI), Z = Math.sin(q), ne = Math.cos(q), te = Z * Z, J = H / Math.sqrt(1 - _ * te), {
      x: (J + G) * ne * Math.cos(V),
      y: (J + G) * ne * Math.sin(V),
      z: (J * (1 - _) + G) * Z
    };
  }
  function geocentricToGeodetic(O, _, H, V) {
    var q = 1e-12, G = q * q, J = 30, Z, te, ne, ee, ae, oe, se, ce, le, ue, de, fe, he, me = O.x, pe = O.y, ye = O.z ? O.z : 0, ge, ve, be;
    if (Z = Math.sqrt(me * me + pe * pe), te = Math.sqrt(me * me + pe * pe + ye * ye), Z / H < q) {
      if (ge = 0, te / H < q)
        return ve = HALF_PI, be = -V, {
          x: O.x,
          y: O.y,
          z: O.z
        };
    } else
      ge = Math.atan2(pe, me);
    ne = ye / te, ee = Z / te, ae = 1 / Math.sqrt(1 - _ * (2 - _) * ee * ee), ce = ee * (1 - _) * ae, le = ne * ae, he = 0;
    do
      he++, se = H / Math.sqrt(1 - _ * le * le), be = Z * ce + ye * le - se * (1 - _ * le * le), oe = _ * se / (se + be), ae = 1 / Math.sqrt(1 - oe * (2 - oe) * ee * ee), ue = ee * (1 - oe) * ae, de = ne * ae, fe = de * ce - ue * le, ce = ue, le = de;
    while (fe * fe > G && he < J);
    return ve = Math.atan(de / Math.abs(ue)), {
      x: ge,
      y: ve,
      z: be
    };
  }
  function geocentricToWgs84(O, _, H) {
    if (_ === PJD_3PARAM)
      return {
        x: O.x + H[0],
        y: O.y + H[1],
        z: O.z + H[2]
      };
    if (_ === PJD_7PARAM) {
      var V = H[0], q = H[1], G = H[2], J = H[3], Z = H[4], te = H[5], ne = H[6];
      return {
        x: ne * (O.x - te * O.y + Z * O.z) + V,
        y: ne * (te * O.x + O.y - J * O.z) + q,
        z: ne * (-Z * O.x + J * O.y + O.z) + G
      };
    }
  }
  function geocentricFromWgs84(O, _, H) {
    if (_ === PJD_3PARAM)
      return {
        x: O.x - H[0],
        y: O.y - H[1],
        z: O.z - H[2]
      };
    if (_ === PJD_7PARAM) {
      var V = H[0], q = H[1], G = H[2], J = H[3], Z = H[4], te = H[5], ne = H[6], ee = (O.x - V) / ne, ae = (O.y - q) / ne, oe = (O.z - G) / ne;
      return {
        x: ee + te * ae - Z * oe,
        y: -te * ee + ae + J * oe,
        z: Z * ee - J * ae + oe
      };
    }
  }
  function checkParams(O) {
    return O === PJD_3PARAM || O === PJD_7PARAM;
  }
  function datum_transform(O, _, H) {
    if (compareDatums(O, _) || O.datum_type === PJD_NODATUM || _.datum_type === PJD_NODATUM)
      return H;
    var V = O.a, q = O.es;
    if (O.datum_type === PJD_GRIDSHIFT) {
      var G = applyGridShift(O, !1, H);
      if (G !== 0)
        return;
      V = SRS_WGS84_SEMIMAJOR, q = SRS_WGS84_ESQUARED;
    }
    var J = _.a, Z = _.b, te = _.es;
    if (_.datum_type === PJD_GRIDSHIFT && (J = SRS_WGS84_SEMIMAJOR, Z = SRS_WGS84_SEMIMINOR, te = SRS_WGS84_ESQUARED), q === te && V === J && !checkParams(O.datum_type) && !checkParams(_.datum_type))
      return H;
    if (H = geodeticToGeocentric(H, q, V), checkParams(O.datum_type) && (H = geocentricToWgs84(H, O.datum_type, O.datum_params)), checkParams(_.datum_type) && (H = geocentricFromWgs84(H, _.datum_type, _.datum_params)), H = geocentricToGeodetic(H, te, J, Z), _.datum_type === PJD_GRIDSHIFT) {
      var ne = applyGridShift(_, !0, H);
      if (ne !== 0)
        return;
    }
    return H;
  }
  function applyGridShift(O, _, H) {
    if (O.grids === null || O.grids.length === 0)
      return console.log("Grid shift grids not found"), -1;
    var V = { x: -H.x, y: H.y }, q = { x: Number.NaN, y: Number.NaN }, G = [];
    e:
      for (var J = 0; J < O.grids.length; J++) {
        var Z = O.grids[J];
        if (G.push(Z.name), Z.isNull) {
          q = V;
          break;
        }
        if (Z.mandatory, Z.grid === null) {
          if (Z.mandatory)
            return console.log("Unable to find mandatory grid '" + Z.name + "'"), -1;
          continue;
        }
        for (var te = Z.grid.subgrids, ne = 0, ee = te.length; ne < ee; ne++) {
          var ae = te[ne], oe = (Math.abs(ae.del[1]) + Math.abs(ae.del[0])) / 1e4, se = ae.ll[0] - oe, ce = ae.ll[1] - oe, le = ae.ll[0] + (ae.lim[0] - 1) * ae.del[0] + oe, ue = ae.ll[1] + (ae.lim[1] - 1) * ae.del[1] + oe;
          if (!(ce > V.y || se > V.x || ue < V.y || le < V.x) && (q = applySubgridShift(V, _, ae), !isNaN(q.x)))
            break e;
        }
      }
    return isNaN(q.x) ? (console.log("Failed to find a grid shift table for location '" + -V.x * R2D + " " + V.y * R2D + " tried: '" + G + "'"), -1) : (H.x = -q.x, H.y = q.y, 0);
  }
  function applySubgridShift(O, _, H) {
    var V = { x: Number.NaN, y: Number.NaN };
    if (isNaN(O.x))
      return V;
    var q = { x: O.x, y: O.y };
    q.x -= H.ll[0], q.y -= H.ll[1], q.x = adjust_lon(q.x - Math.PI) + Math.PI;
    var G = nadInterpolate(q, H);
    if (_) {
      if (isNaN(G.x))
        return V;
      G.x = q.x - G.x, G.y = q.y - G.y;
      var J = 9, Z = 1e-12, te, ne;
      do {
        if (ne = nadInterpolate(G, H), isNaN(ne.x)) {
          console.log("Inverse grid shift iteration failed, presumably at grid edge.  Using first approximation.");
          break;
        }
        te = { x: q.x - (ne.x + G.x), y: q.y - (ne.y + G.y) }, G.x += te.x, G.y += te.y;
      } while (J-- && Math.abs(te.x) > Z && Math.abs(te.y) > Z);
      if (J < 0)
        return console.log("Inverse grid shift iterator failed to converge."), V;
      V.x = adjust_lon(G.x + H.ll[0]), V.y = G.y + H.ll[1];
    } else
      isNaN(G.x) || (V.x = O.x + G.x, V.y = O.y + G.y);
    return V;
  }
  function nadInterpolate(O, _) {
    var H = { x: O.x / _.del[0], y: O.y / _.del[1] }, V = { x: Math.floor(H.x), y: Math.floor(H.y) }, q = { x: H.x - 1 * V.x, y: H.y - 1 * V.y }, G = { x: Number.NaN, y: Number.NaN }, J;
    if (V.x < 0 || V.x >= _.lim[0] || V.y < 0 || V.y >= _.lim[1])
      return G;
    J = V.y * _.lim[0] + V.x;
    var Z = { x: _.cvs[J][0], y: _.cvs[J][1] };
    J++;
    var te = { x: _.cvs[J][0], y: _.cvs[J][1] };
    J += _.lim[0];
    var ne = { x: _.cvs[J][0], y: _.cvs[J][1] };
    J--;
    var ee = { x: _.cvs[J][0], y: _.cvs[J][1] }, ae = q.x * q.y, oe = q.x * (1 - q.y), se = (1 - q.x) * (1 - q.y), ce = (1 - q.x) * q.y;
    return G.x = se * Z.x + oe * te.x + ce * ee.x + ae * ne.x, G.y = se * Z.y + oe * te.y + ce * ee.y + ae * ne.y, G;
  }
  function adjust_axis(O, _, H) {
    var V = H.x, q = H.y, G = H.z || 0, J, Z, te, ne = {};
    for (te = 0; te < 3; te++)
      if (!(_ && te === 2 && H.z === void 0))
        switch (te === 0 ? (J = V, "ew".indexOf(O.axis[te]) !== -1 ? Z = "x" : Z = "y") : te === 1 ? (J = q, "ns".indexOf(O.axis[te]) !== -1 ? Z = "y" : Z = "x") : (J = G, Z = "z"), O.axis[te]) {
          case "e":
            ne[Z] = J;
            break;
          case "w":
            ne[Z] = -J;
            break;
          case "n":
            ne[Z] = J;
            break;
          case "s":
            ne[Z] = -J;
            break;
          case "u":
            H[Z] !== void 0 && (ne.z = J);
            break;
          case "d":
            H[Z] !== void 0 && (ne.z = -J);
            break;
          default:
            return null;
        }
    return ne;
  }
  function toPoint(O) {
    var _ = {
      x: O[0],
      y: O[1]
    };
    return O.length > 2 && (_.z = O[2]), O.length > 3 && (_.m = O[3]), _;
  }
  function checkSanity(O) {
    checkCoord(O.x), checkCoord(O.y);
  }
  function checkCoord(O) {
    if (typeof Number.isFinite == "function") {
      if (Number.isFinite(O))
        return;
      throw new TypeError("coordinates must be finite numbers");
    }
    if (typeof O != "number" || O !== O || !isFinite(O))
      throw new TypeError("coordinates must be finite numbers");
  }
  function checkNotWGS(O, _) {
    return (O.datum.datum_type === PJD_3PARAM || O.datum.datum_type === PJD_7PARAM || O.datum.datum_type === PJD_GRIDSHIFT) && _.datumCode !== "WGS84" || (_.datum.datum_type === PJD_3PARAM || _.datum.datum_type === PJD_7PARAM || _.datum.datum_type === PJD_GRIDSHIFT) && O.datumCode !== "WGS84";
  }
  function transform$4(O, _, H, V) {
    var q;
    Array.isArray(H) ? H = toPoint(H) : H = {
      x: H.x,
      y: H.y,
      z: H.z,
      m: H.m
    };
    var G = H.z !== void 0;
    if (checkSanity(H), O.datum && _.datum && checkNotWGS(O, _) && (q = new Projection$2("WGS84"), H = transform$4(O, q, H, V), O = q), V && O.axis !== "enu" && (H = adjust_axis(O, !1, H)), O.projName === "longlat")
      H = {
        x: H.x * D2R$1,
        y: H.y * D2R$1,
        z: H.z || 0
      };
    else if (O.to_meter && (H = {
      x: H.x * O.to_meter,
      y: H.y * O.to_meter,
      z: H.z || 0
    }), H = O.inverse(H), !H)
      return;
    if (O.from_greenwich && (H.x += O.from_greenwich), H = datum_transform(O.datum, _.datum, H), !!H)
      return _.from_greenwich && (H = {
        x: H.x - _.from_greenwich,
        y: H.y,
        z: H.z || 0
      }), _.projName === "longlat" ? H = {
        x: H.x * R2D,
        y: H.y * R2D,
        z: H.z || 0
      } : (H = _.forward(H), _.to_meter && (H = {
        x: H.x / _.to_meter,
        y: H.y / _.to_meter,
        z: H.z || 0
      })), V && _.axis !== "enu" ? adjust_axis(_, !0, H) : (H && !G && delete H.z, H);
  }
  var wgs84$1 = Projection$2("WGS84");
  function transformer$4(O, _, H, V) {
    var q, G, J;
    return Array.isArray(H) ? (q = transform$4(O, _, H, V) || { x: NaN, y: NaN }, H.length > 2 ? typeof O.name < "u" && O.name === "geocent" || typeof _.name < "u" && _.name === "geocent" ? typeof q.z == "number" ? [q.x, q.y, q.z].concat(H.slice(3)) : [q.x, q.y, H[2]].concat(H.slice(3)) : [q.x, q.y].concat(H.slice(2)) : [q.x, q.y]) : (G = transform$4(O, _, H, V), J = Object.keys(H), J.length === 2 || J.forEach(function(Z) {
      if (typeof O.name < "u" && O.name === "geocent" || typeof _.name < "u" && _.name === "geocent") {
        if (Z === "x" || Z === "y" || Z === "z")
          return;
      } else if (Z === "x" || Z === "y")
        return;
      G[Z] = H[Z];
    }), G);
  }
  function checkProj(O) {
    return O instanceof Projection$2 ? O : O.oProj ? O.oProj : Projection$2(O);
  }
  function proj4(O, _, H) {
    O = checkProj(O);
    var V = !1, q;
    return typeof _ > "u" ? (_ = O, O = wgs84$1, V = !0) : (typeof _.x < "u" || Array.isArray(_)) && (H = _, _ = O, O = wgs84$1, V = !0), _ = checkProj(_), H ? transformer$4(O, _, H) : (q = {
      forward: function(G, J) {
        return transformer$4(O, _, G, J);
      },
      inverse: function(G, J) {
        return transformer$4(_, O, G, J);
      }
    }, V && (q.oProj = _), q);
  }
  const mercator = new Projection$2("GOOGLE"), wgs84 = new Projection$2("WGS84"), _wgs84_mercator = proj4(wgs84, mercator), wgs84_mercator = {
    compute(O, _) {
      return isFinite(O) && isFinite(_) ? _wgs84_mercator.forward([O, _]) : [NaN, NaN];
    },
    invert(O, _) {
      return isFinite(O) && isFinite(_) ? _wgs84_mercator.inverse([O, _]) : [NaN, NaN];
    }
  }, mercator_bounds = {
    lon: [-2002637639e-2, 2002637639e-2],
    lat: [-200489661e-1, 200489661e-1]
  }, latlon_bounds = {
    lon: [-180, 180],
    lat: [-85.06, 85.06]
  }, { min: min$9, max: max$g } = Math;
  function clip_mercator(O, _, H) {
    const [V, q] = mercator_bounds[H];
    return [max$g(O, V), min$9(_, q)];
  }
  function in_bounds(O, _) {
    const [H, V] = latlon_bounds[_];
    return H < O && O < V;
  }
  var inplace;
  (function(O) {
    function _(V, q, G, J) {
      const Z = min$9(V.length, q.length);
      G = G ?? V, J = J ?? q;
      for (let te = 0; te < Z; te++) {
        const ne = V[te], ee = q[te], [ae, oe] = wgs84_mercator.compute(ne, ee);
        G[te] = ae, J[te] = oe;
      }
    }
    O.project_xy = _;
    function H(V, q, G, J) {
      const Z = min$9(V.length, q.length);
      G = G ?? V, J = J ?? q;
      for (let te = 0; te < Z; te++)
        _(V[te], q[te], G[te], J[te]);
    }
    O.project_xsys = H;
  })(inplace || (inplace = {}));
  function project_xy(O, _) {
    const H = min$9(O.length, _.length), V = infer_type(O, _), q = new V(H), G = new V(H);
    return inplace.project_xy(O, _, q, G), [q, G];
  }
  class DataAnnotationView extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "_initial_set_data", !1);
    }
    connect_signals() {
      super.connect_signals();
      const H = () => {
        this.set_data(this.model.source), this._rerender();
      };
      this.connect(this.model.change, H), this.connect(this.model.source.streaming, H), this.connect(this.model.source.patching, H), this.connect(this.model.source.change, H);
    }
    _rerender() {
      this.request_render();
    }
    set_data(H) {
      const V = this;
      for (const q of this.model)
        if (q instanceof VectorSpec || q instanceof ScalarSpec)
          if (q instanceof BaseCoordinateSpec) {
            const G = q.array(H);
            V[`_${q.attr}`] = G;
          } else {
            const G = q.uniform(H);
            V[`${q.attr}`] = G;
          }
      this.plot_model.use_map && (V._x != null && inplace.project_xy(V._x, V._y), V._xs != null && inplace.project_xsys(V._xs, V._ys));
      for (const q of this.visuals)
        q.update();
    }
    _render() {
      this._initial_set_data || (this.set_data(this.model.source), this._initial_set_data = !0), this.map_data(), this.paint(this.layer.ctx);
    }
  }
  mt(DataAnnotationView, "__name__", "DataAnnotationView");
  const Sg = class Sg extends Annotation$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Sg, "__name__", "DataAnnotation"), Sg.define(({ Ref: _ }) => ({
    source: [_(ColumnarDataSource), () => new ColumnDataSource()]
  }));
  let DataAnnotation = Sg;
  class MarkingView extends View$2 {
    constructor() {
      super(...arguments);
      mt(this, "visuals");
      mt(this, "size");
    }
    initialize() {
      super.initialize(), this.visuals = new Visuals(this);
    }
    request_render() {
      this.parent.request_render();
    }
    get canvas() {
      return this.parent.canvas;
    }
    set_data(H, V) {
      const q = this;
      for (const G of this.model) {
        if (!(G instanceof VectorSpec || G instanceof ScalarSpec))
          continue;
        const J = G.uniform(H).select(V);
        q[`${G.attr}`] = J;
      }
    }
  }
  mt(MarkingView, "__name__", "MarkingView");
  const Eg = class Eg extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Eg, "__name__", "Marking"), Eg.define(({}) => ({}));
  let Marking = Eg;
  class ArrowHeadView extends MarkingView {
  }
  mt(ArrowHeadView, "__name__", "ArrowHeadView");
  const Ag = class Ag extends Marking {
    constructor(_) {
      super(_);
    }
  };
  mt(Ag, "__name__", "ArrowHead"), Ag.define(() => ({
    size: [NumberSpec, 25]
  }));
  let ArrowHead = Ag;
  class OpenHeadView extends ArrowHeadView {
    clip(_, H) {
      this.visuals.line.set_vectorize(_, H);
      const V = this.size.get(H);
      _.moveTo(0.5 * V, V), _.lineTo(0.5 * V, -2), _.lineTo(-0.5 * V, -2), _.lineTo(-0.5 * V, V), _.lineTo(0, 0), _.lineTo(0.5 * V, V);
    }
    render(_, H) {
      const V = this.size.get(H);
      _.beginPath(), _.moveTo(0.5 * V, V), _.lineTo(0, 0), _.lineTo(-0.5 * V, V), this.visuals.line.apply(_, H);
    }
  }
  mt(OpenHeadView, "__name__", "OpenHeadView");
  const D3 = class D3 extends ArrowHead {
    constructor(_) {
      super(_);
    }
  };
  mt(D3, "__name__", "OpenHead"), D3.prototype.default_view = OpenHeadView, D3.mixins(LineVector$1);
  let OpenHead = D3;
  class NormalHeadView extends ArrowHeadView {
    clip(_, H) {
      this.visuals.line.set_vectorize(_, H);
      const V = this.size.get(H);
      _.moveTo(0.5 * V, V), _.lineTo(0.5 * V, -2), _.lineTo(-0.5 * V, -2), _.lineTo(-0.5 * V, V), _.lineTo(0.5 * V, V);
    }
    render(_, H) {
      const V = this.size.get(H);
      _.beginPath(), _.moveTo(0.5 * V, V), _.lineTo(0, 0), _.lineTo(-0.5 * V, V), _.closePath(), this.visuals.fill.apply(_, H), this.visuals.line.apply(_, H);
    }
  }
  mt(NormalHeadView, "__name__", "NormalHeadView");
  const mp = class mp extends ArrowHead {
    constructor(_) {
      super(_);
    }
  };
  mt(mp, "__name__", "NormalHead"), mp.prototype.default_view = NormalHeadView, mp.mixins([LineVector$1, FillVector$1]), mp.override({
    fill_color: "black"
  });
  let NormalHead = mp;
  class VeeHeadView extends ArrowHeadView {
    clip(_, H) {
      this.visuals.line.set_vectorize(_, H);
      const V = this.size.get(H);
      _.moveTo(0.5 * V, V), _.lineTo(0.5 * V, -2), _.lineTo(-0.5 * V, -2), _.lineTo(-0.5 * V, V), _.lineTo(0, 0.5 * V), _.lineTo(0.5 * V, V);
    }
    render(_, H) {
      const V = this.size.get(H);
      _.beginPath(), _.moveTo(0.5 * V, V), _.lineTo(0, 0), _.lineTo(-0.5 * V, V), _.lineTo(0, 0.5 * V), _.closePath(), this.visuals.fill.apply(_, H), this.visuals.line.apply(_, H);
    }
  }
  mt(VeeHeadView, "__name__", "VeeHeadView");
  const gp = class gp extends ArrowHead {
    constructor(_) {
      super(_);
    }
  };
  mt(gp, "__name__", "VeeHead"), gp.prototype.default_view = VeeHeadView, gp.mixins([LineVector$1, FillVector$1]), gp.override({
    fill_color: "black"
  });
  let VeeHead = gp;
  class TeeHeadView extends ArrowHeadView {
    render(_, H) {
      const V = this.size.get(H);
      _.beginPath(), _.moveTo(0.5 * V, 0), _.lineTo(-0.5 * V, 0), this.visuals.line.apply(_, H);
    }
    clip(_, H) {
    }
  }
  mt(TeeHeadView, "__name__", "TeeHeadView");
  const N3 = class N3 extends ArrowHead {
    constructor(_) {
      super(_);
    }
  };
  mt(N3, "__name__", "TeeHead"), N3.prototype.default_view = TeeHeadView, N3.mixins(LineVector$1);
  let TeeHead = N3;
  class ArrowView extends DataAnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "start");
      mt(this, "end");
      mt(this, "_x_start");
      mt(this, "_y_start");
      mt(this, "_x_end");
      mt(this, "_y_end");
      mt(this, "_sx_start");
      mt(this, "_sy_start");
      mt(this, "_sx_end");
      mt(this, "_sy_end");
      mt(this, "_angles");
    }
    *children() {
      yield* super.children();
      const { start: H, end: V } = this;
      H != null && (yield H), V != null && (yield V);
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { start: H, end: V } = this.model;
      H != null && (this.start = await build_view(H, { parent: this })), V != null && (this.end = await build_view(V, { parent: this }));
    }
    set_data(H) {
      var q, G;
      super.set_data(H);
      const V = BitSet.all_set(this._x_start.length);
      (q = this.start) == null || q.set_data(H, V), (G = this.end) == null || G.set_data(H, V);
    }
    remove() {
      var H, V;
      (H = this.start) == null || H.remove(), (V = this.end) == null || V.remove(), super.remove();
    }
    map_data() {
      const { frame: H } = this.plot_view, [V, q] = (() => {
        switch (this.model.start_units) {
          case "canvas":
            return [
              new ScreenArray(this._x_start),
              new ScreenArray(this._y_start)
            ];
          case "screen":
            return [
              H.bbox.xview.v_compute(this._x_start),
              H.bbox.yview.v_compute(this._y_start)
            ];
          case "data":
            return [
              this.coordinates.x_scale.v_compute(this._x_start),
              this.coordinates.y_scale.v_compute(this._y_start)
            ];
        }
      })(), [G, J] = (() => {
        switch (this.model.end_units) {
          case "canvas":
            return [
              new ScreenArray(this._x_end),
              new ScreenArray(this._y_end)
            ];
          case "screen":
            return [
              H.bbox.xview.v_compute(this._x_end),
              H.bbox.yview.v_compute(this._y_end)
            ];
          case "data":
            return [
              this.coordinates.x_scale.v_compute(this._x_end),
              this.coordinates.y_scale.v_compute(this._y_end)
            ];
        }
      })();
      this._sx_start = V, this._sy_start = q, this._sx_end = G, this._sy_end = J;
      const Z = V.length, te = this._angles = new ScreenArray(Z);
      for (let ne = 0; ne < Z; ne++)
        te[ne] = Math.PI / 2 + atan2$2([V[ne], q[ne]], [G[ne], J[ne]]);
    }
    paint(H) {
      const { start: V, end: q } = this, { _sx_start: G, _sy_start: J, _sx_end: Z, _sy_end: te, _angles: ne } = this, { x: ee, y: ae, width: oe, height: se } = this.plot_view.frame.bbox;
      for (let ce = 0, le = G.length; ce < le; ce++)
        q != null && (H.save(), H.translate(Z[ce], te[ce]), H.rotate(ne[ce]), q.render(H, ce), H.restore()), V != null && (H.save(), H.translate(G[ce], J[ce]), H.rotate(ne[ce] + Math.PI), V.render(H, ce), H.restore()), this.visuals.line.doit && (H.save(), (V != null || q != null) && (H.beginPath(), H.rect(ee, ae, oe, se), q != null && (H.save(), H.translate(Z[ce], te[ce]), H.rotate(ne[ce]), q.clip(H, ce), H.restore()), V != null && (H.save(), H.translate(G[ce], J[ce]), H.rotate(ne[ce] + Math.PI), V.clip(H, ce), H.restore()), H.closePath(), H.clip()), H.beginPath(), H.moveTo(G[ce], J[ce]), H.lineTo(Z[ce], te[ce]), this.visuals.line.apply(H, ce), H.restore());
    }
  }
  mt(ArrowView, "__name__", "ArrowView");
  const yp = class yp extends DataAnnotation {
    constructor(_) {
      super(_);
    }
  };
  mt(yp, "__name__", "Arrow"), yp.prototype.default_view = ArrowView, yp.mixins(LineVector$1), yp.define(({ Ref: _, Nullable: H }) => ({
    x_start: [XCoordinateSpec, { field: "x_start" }],
    y_start: [YCoordinateSpec, { field: "y_start" }],
    start_units: [CoordinateUnits, "data"],
    start: [H(_(ArrowHead)), null],
    x_end: [XCoordinateSpec, { field: "x_end" }],
    y_end: [YCoordinateSpec, { field: "y_end" }],
    end_units: [CoordinateUnits, "data"],
    end: [H(_(ArrowHead)), () => new OpenHead()]
  }));
  let Arrow = yp;
  const { min: min$8, max: max$f } = Math, a2 = class a2 {
    constructor(_ = {}) {
      mt(this, "width");
      mt(this, "height");
      this.width = _.width != null ? _.width : 0, this.height = _.height != null ? _.height : 0;
    }
    bounded_to({ width: _, height: H }) {
      return new a2({
        width: this.width == 1 / 0 && _ != null ? _ : this.width,
        height: this.height == 1 / 0 && H != null ? H : this.height
      });
    }
    expanded_to({ width: _, height: H }) {
      return new a2({
        width: _ != 1 / 0 ? max$f(this.width, _) : this.width,
        height: H != 1 / 0 ? max$f(this.height, H) : this.height
      });
    }
    expand_to({ width: _, height: H }) {
      this.width = max$f(this.width, _), this.height = max$f(this.height, H);
    }
    narrowed_to({ width: _, height: H }) {
      return new a2({
        width: min$8(this.width, _),
        height: min$8(this.height, H)
      });
    }
    narrow_to({ width: _, height: H }) {
      this.width = min$8(this.width, _), this.height = min$8(this.height, H);
    }
    grow_by({ left: _, right: H, top: V, bottom: q }) {
      const G = this.width + _ + H, J = this.height + V + q;
      return new a2({ width: G, height: J });
    }
    shrink_by({ left: _, right: H, top: V, bottom: q }) {
      const G = max$f(this.width - _ - H, 0), J = max$f(this.height - V - q, 0);
      return new a2({ width: G, height: J });
    }
    map(_, H) {
      return new a2({
        width: _(this.width),
        height: (H ?? _)(this.height)
      });
    }
  };
  mt(a2, "__name__", "Sizeable");
  let Sizeable = a2;
  const SizingPolicy = Enum("fixed", "fit", "min", "max"), { abs: abs$c, min: min$7, max: max$e, round: round$4 } = Math;
  class Layoutable {
    constructor() {
      mt(this, "absolute", !1);
      mt(this, "position", { left: 0, top: 0 });
      mt(this, "_bbox", new BBox$1());
      mt(this, "_inner_bbox", new BBox$1());
      mt(this, "_sizing", null);
      mt(this, "_dirty", !1);
      mt(this, "_handlers", []);
    }
    *[Symbol.iterator]() {
    }
    get bbox() {
      return this._bbox;
    }
    get inner_bbox() {
      return this._inner_bbox;
    }
    get sizing() {
      return assert$2(this._sizing != null), this._sizing;
    }
    set dirty(_) {
      this._dirty = _;
    }
    get dirty() {
      return this._dirty;
    }
    get visible() {
      return this.sizing.visible;
    }
    set visible(_) {
      this.sizing.visible != _ && (this.sizing.visible = _, this._dirty = !0);
    }
    set_sizing(_ = {}) {
      const H = _.width_policy ?? "fit", V = _.width, q = _.min_width, G = _.max_width, J = _.height_policy ?? "fit", Z = _.height, te = _.min_height, ne = _.max_height, ee = _.aspect, ae = _.margin ?? { top: 0, right: 0, bottom: 0, left: 0 }, oe = _.visible ?? !0, se = _.halign ?? "start", ce = _.valign ?? "start";
      this._sizing = {
        width_policy: H,
        min_width: q,
        width: V,
        max_width: G,
        height_policy: J,
        min_height: te,
        height: Z,
        max_height: ne,
        aspect: ee,
        margin: ae,
        visible: oe,
        halign: se,
        valign: ce,
        size: { width: V, height: Z }
      }, this._init();
    }
    _init() {
    }
    _set_geometry(_, H) {
      this._bbox = _, this._inner_bbox = H;
    }
    set_geometry(_, H) {
      const { fixup_geometry: V } = this;
      V != null && ([_, H] = V(_, H)), this._set_geometry(_, H ?? _);
      for (const q of this._handlers)
        q(this._bbox, this._inner_bbox);
    }
    on_resize(_) {
      this._handlers.push(_);
    }
    is_width_expanding() {
      return this.sizing.width_policy == "max";
    }
    is_height_expanding() {
      return this.sizing.height_policy == "max";
    }
    apply_aspect(_, { width: H, height: V }) {
      const { aspect: q } = this.sizing;
      if (q != null) {
        const { width_policy: G, height_policy: J } = this.sizing, Z = (te, ne) => {
          const ee = { max: 4, fit: 3, min: 2, fixed: 1 };
          return ee[te] > ee[ne];
        };
        if (G != "fixed" && J != "fixed")
          if (G == J) {
            const te = H, ne = round$4(H / q), ee = round$4(V * q), ae = V, oe = abs$c(_.width - te) + abs$c(_.height - ne), se = abs$c(_.width - ee) + abs$c(_.height - ae);
            oe <= se ? (H = te, V = ne) : (H = ee, V = ae);
          } else Z(G, J) ? V = round$4(H / q) : H = round$4(V * q);
        else G == "fixed" ? V = round$4(H / q) : J == "fixed" && (H = round$4(V * q));
      }
      return { width: H, height: V };
    }
    measure(_) {
      if (this._sizing == null && this.set_sizing(), !this.sizing.visible)
        return { width: 0, height: 0 };
      const H = (ee) => this.sizing.width_policy == "fixed" && this.sizing.width != null ? this.sizing.width : ee, V = (ee) => this.sizing.height_policy == "fixed" && this.sizing.height != null ? this.sizing.height : ee, q = new Sizeable(_).shrink_by(this.sizing.margin).map(H, V), G = this._measure(q), J = this.clip_size(G, q), Z = H(J.width), te = V(J.height), ne = this.apply_aspect(q, { width: Z, height: te });
      return { ...G, ...ne };
    }
    compute(_ = {}) {
      const H = this.measure({
        width: _.width != null && this.is_width_expanding() ? _.width : 1 / 0,
        height: _.height != null && this.is_height_expanding() ? _.height : 1 / 0
      }), { width: V, height: q } = H, { left: G, top: J } = this.position, Z = new BBox$1({ left: G, top: J, width: V, height: q });
      let te;
      if (H.inner != null) {
        const { left: ne, top: ee, right: ae, bottom: oe } = H.inner;
        te = new BBox$1({ left: ne, top: ee, right: V - ae, bottom: q - oe });
      }
      this.set_geometry(Z, te);
    }
    get xview() {
      return this.bbox.xview;
    }
    get yview() {
      return this.bbox.yview;
    }
    clip_size(_, H) {
      function V(q, G, J, Z) {
        return J == null ? J = 0 : isNumber$3(J) || (J = round$4(J.percent * G)), Z == null ? Z = 1 / 0 : isNumber$3(Z) || (Z = round$4(Z.percent * G)), max$e(J, min$7(q, Z));
      }
      return {
        width: V(_.width, H.width, this.sizing.min_width, this.sizing.max_width),
        height: V(_.height, H.height, this.sizing.min_height, this.sizing.max_height)
      };
    }
    has_size_changed() {
      const { _dirty: _ } = this;
      return this._dirty = !1, _;
    }
  }
  mt(Layoutable, "__name__", "Layoutable");
  class ContentLayoutable extends Layoutable {
    _measure(_) {
      const H = this._content_size(), V = _.bounded_to(this.sizing.size).bounded_to(H), q = (() => {
        switch (this.sizing.width_policy) {
          case "fixed":
            return this.sizing.width != null ? this.sizing.width : H.width;
          case "min":
            return H.width;
          case "fit":
            return V.width;
          case "max":
            return max$e(H.width, V.width);
        }
      })(), G = (() => {
        switch (this.sizing.height_policy) {
          case "fixed":
            return this.sizing.height != null ? this.sizing.height : H.height;
          case "min":
            return H.height;
          case "fit":
            return V.height;
          case "max":
            return max$e(H.height, V.height);
        }
      })();
      return { width: q, height: G };
    }
  }
  mt(ContentLayoutable, "__name__", "ContentLayoutable");
  class TextLayout extends ContentLayoutable {
    constructor(H) {
      super();
      mt(this, "text");
      this.text = H;
    }
    _content_size() {
      return new Sizeable(this.text.size());
    }
  }
  mt(TextLayout, "__name__", "TextLayout");
  class FixedLayout extends ContentLayoutable {
    constructor(H = {}) {
      super();
      mt(this, "size");
      this.size = H;
    }
    _content_size() {
      return new Sizeable(this.size);
    }
  }
  mt(FixedLayout, "__name__", "FixedLayout");
  const pi2 = Math.PI / 2, _angle_lookup = {
    above: {
      parallel: 0,
      normal: -pi2,
      horizontal: 0,
      vertical: -pi2
    },
    below: {
      parallel: 0,
      normal: pi2,
      horizontal: 0,
      vertical: pi2
    },
    left: {
      parallel: -pi2,
      normal: 0,
      horizontal: 0,
      vertical: -pi2
    },
    right: {
      parallel: pi2,
      normal: 0,
      horizontal: 0,
      vertical: pi2
    }
  }, _vertical_align_lookup = {
    above: {
      parallel: "bottom",
      normal: "center",
      horizontal: "bottom",
      vertical: "center"
    },
    below: {
      parallel: "top",
      normal: "center",
      horizontal: "top",
      vertical: "center"
    },
    left: {
      parallel: "bottom",
      normal: "center",
      horizontal: "center",
      vertical: "bottom"
    },
    right: {
      parallel: "bottom",
      normal: "center",
      horizontal: "center",
      vertical: "bottom"
    }
  }, _align_lookup = {
    above: {
      parallel: "center",
      normal: "left",
      horizontal: "center",
      vertical: "left"
    },
    below: {
      parallel: "center",
      normal: "left",
      horizontal: "center",
      vertical: "left"
    },
    left: {
      parallel: "center",
      normal: "right",
      horizontal: "right",
      vertical: "center"
    },
    right: {
      parallel: "center",
      normal: "left",
      horizontal: "left",
      vertical: "center"
    }
  }, _align_lookup_negative = {
    above: "right",
    below: "left",
    left: "right",
    right: "left"
  }, _align_lookup_positive = {
    above: "left",
    below: "right",
    left: "right",
    right: "left"
  };
  class SidePanel {
    constructor(_, H) {
      mt(this, "side");
      mt(this, "face");
      mt(this, "dimension");
      mt(this, "orientation");
      mt(this, "is_horizontal");
      mt(this, "is_vertical");
      mt(this, "normals");
      this.side = _, this.face = (() => {
        if (H != null && H != "auto")
          return H;
        switch (this.side) {
          case "left":
          case "above":
            return "front";
          case "right":
          case "below":
            return "back";
        }
      })(), this.dimension = this.side == "above" || this.side == "below" ? 0 : 1, this.orientation = this.dimension == 0 ? "horizontal" : "vertical", this.is_horizontal = this.dimension == 0, this.is_vertical = this.dimension == 1, this.normals = (() => {
        const V = this.face == "front" ? -1 : 1;
        switch (this.side) {
          case "left":
            return [V, 0];
          case "right":
            return [V, 0];
          case "above":
            return [0, V];
          case "below":
            return [0, V];
        }
      })();
    }
    get face_adjusted_side() {
      const { side: _, face: H } = this;
      switch (_) {
        case "left":
        case "right":
          return H == "front" ? "left" : "right";
        case "above":
        case "below":
          return H == "front" ? "above" : "below";
      }
    }
    get_label_text_heuristics(_) {
      const H = this.face_adjusted_side;
      return isString$2(_) ? {
        vertical_align: _vertical_align_lookup[H][_],
        align: _align_lookup[H][_]
      } : {
        vertical_align: "center",
        align: (_ < 0 ? _align_lookup_negative : _align_lookup_positive)[H]
      };
    }
    get_label_angle_heuristic(_) {
      if (isString$2(_)) {
        const H = this.face_adjusted_side;
        return _angle_lookup[H][_];
      } else
        return -_;
    }
  }
  mt(SidePanel, "__name__", "SidePanel");
  class SideLayout extends ContentLayoutable {
    constructor(H, V, q = !1) {
      super();
      mt(this, "panel");
      mt(this, "get_size");
      mt(this, "rotate");
      this.panel = H, this.get_size = V, this.rotate = q, this.panel.is_horizontal ? this.set_sizing({ width_policy: "max", height_policy: "fixed" }) : this.set_sizing({ width_policy: "fixed", height_policy: "max" });
    }
    _content_size() {
      const { width: H, height: V } = this.get_size();
      return !this.rotate || this.panel.is_horizontal ? new Sizeable({ width: H, height: V }) : new Sizeable({ width: V, height: H });
    }
    has_size_changed() {
      const { width: H, height: V } = this._content_size();
      return this.panel.is_horizontal ? this.bbox.height != V : this.bbox.width != H;
    }
  }
  mt(SideLayout, "__name__", "SideLayout");
  class BaseTextView extends View$2 {
  }
  mt(BaseTextView, "__name__", "BaseTextView");
  const Qg = class Qg extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Qg, "__name__", "BaseText"), Qg.define(({ Str: _ }) => ({
    text: [_]
  }));
  let BaseText = Qg;
  async function load_image(O, _) {
    return new ImageLoader(O, _).promise;
  }
  class ImageLoader {
    constructor(_, H = {}) {
      mt(this, "image", new Image());
      mt(this, "promise");
      mt(this, "_finished", !1);
      const { attempts: V = 1, timeout: q = 1 } = H, G = (() => {
        if (_ instanceof ArrayBuffer) {
          const J = new Blob([_], { type: "image/png" });
          return URL.createObjectURL(J);
        } else
          return _;
      })();
      this.promise = new Promise((J, Z) => {
        this.image.crossOrigin = "anonymous";
        let te = 0;
        this.image.onerror = () => {
          var ne;
          if (++te == V) {
            const ee = `unable to load ${G} image after ${V} attempts`;
            if (logger$1.warn(ee), this.image.crossOrigin != null)
              logger$1.warn(`attempting to load ${G} without a cross origin policy`), this.image.crossOrigin = null, te = 0;
            else {
              (ne = H.failed) == null || ne.call(H);
              return;
            }
          }
          setTimeout(() => this.image.src = G, q);
        }, this.image.onload = () => {
          var ne;
          this._finished = !0, (ne = H.loaded) == null || ne.call(H, this.image), J(this.image);
        }, this.image.src = G;
      });
    }
    get finished() {
      return this._finished;
    }
  }
  mt(ImageLoader, "__name__", "ImageLoader");
  const has_OffscreenCanvas = (() => {
    try {
      return typeof OffscreenCanvas < "u" && new OffscreenCanvas(0, 0).getContext("2d") != null;
    } catch {
      return !1;
    }
  })(), _offscreen_canvas = has_OffscreenCanvas ? (O, _) => new OffscreenCanvas(O, _) : (O, _) => {
    const H = document.createElement("canvas");
    return H.width = O, H.height = _, H;
  }, _native_font_metrics = (() => {
    const _ = _offscreen_canvas(0, 0).getContext("2d");
    return assert$2(_ != null, "can't obtain 2d context"), (H) => {
      _.font = H;
      const V = _.measureText("M"), q = _.measureText("x"), G = _.measureText("ÅŚg|"), J = G.fontBoundingBoxAscent, Z = G.fontBoundingBoxDescent;
      if (is_defined(J) && is_defined(Z))
        return {
          height: J + Z,
          ascent: J,
          descent: Z,
          cap_height: V.actualBoundingBoxAscent,
          x_height: q.actualBoundingBoxAscent
        };
      const te = G.actualBoundingBoxAscent, ne = G.actualBoundingBoxDescent;
      if (is_defined(te) && is_defined(ne))
        return {
          height: te + ne,
          ascent: te,
          descent: ne,
          cap_height: V.actualBoundingBoxAscent,
          x_height: q.actualBoundingBoxAscent
        };
      unreachable();
    };
  })(), _internal_font_metrics = (() => {
    const O = document.createElement("canvas"), _ = O.getContext("2d");
    let H = -1, V = -1;
    return (q, G = 1) => {
      _.font = q;
      const { width: J } = _.measureText("M"), Z = J * G, te = Math.ceil(Z), ne = Math.ceil(2 * Z), ee = Math.ceil(1.5 * Z);
      H < te && (H = te, O.width = te), V < ne && (V = ne, O.height = ne), _.save(), _.scale(G, G), _.fillStyle = "#f00", _.fillRect(0, 0, te, ne);
      const ae = (me) => {
        let pe = 0;
        for (let ye = 0; ye <= ee; ye++)
          for (let ge = 0; ge < te; ge++, pe += 4)
            if (me[pe] != 255)
              return ee - ye;
        return 0;
      }, oe = (me) => {
        let pe = me.length - 4;
        for (let ye = ne; ye >= ee; ye--)
          for (let ge = 0; ge < te; ge++, pe -= 4)
            if (me[pe] != 255)
              return ye - ee;
        return 0;
      };
      _.font = q, _.fillStyle = "#000";
      for (const me of "xa")
        _.fillText(me, 0, ee / G);
      const { data: se } = _.getImageData(0, 0, te, ne), ce = ae(se) / G;
      for (const me of "ASQ")
        _.fillText(me, 0, ee / G);
      const { data: le } = _.getImageData(0, 0, te, ne), ue = ae(le) / G;
      for (const me of "ÅŚgy")
        _.fillText(me, 0, ee / G);
      const { data: de } = _.getImageData(0, 0, te, ne), fe = ae(de) / G, he = oe(de) / G;
      return _.restore(), { height: fe + he, ascent: fe, cap_height: ue, x_height: ce, descent: he };
    };
  })(), _font_metrics = (() => {
    try {
      return _native_font_metrics("normal 10px sans-serif"), _native_font_metrics;
    } catch {
      return _internal_font_metrics;
    }
  })(), _metrics_cache = /* @__PURE__ */ new Map();
  function font_metrics(O) {
    let _ = _metrics_cache.get(O);
    if (_ == null) {
      const H = document.fonts.check(O);
      _ = { font: _font_metrics(O) }, H && _metrics_cache.set(O, _);
    }
    return _.font;
  }
  function parse_css_font_size(O) {
    const _ = O.match(/^\s*(\d+(\.\d+)?)(\w+)\s*$/);
    if (_ != null) {
      const [, H, , V] = _, q = Number(H);
      if (isFinite(q))
        return { value: q, unit: V };
    }
    return null;
  }
  function parse_css_length(O) {
    const _ = O.match(/^\s*(-?\d+(\.\d+)?)(\w+)\s*$/);
    if (_ != null) {
      const [, H, , V] = _, q = Number(H);
      if (isFinite(q))
        return { value: q, unit: V };
    }
    return null;
  }
  const text_width = (() => {
    const _ = document.createElement("canvas").getContext("2d");
    let H = "";
    return (V, q) => (q != H && (H = q, _.font = q), _.measureText(V).width);
  })();
  class GraphicsBox {
    constructor() {
      mt(this, "_position", { sx: 0, sy: 0 });
      mt(this, "angle");
      mt(this, "width");
      mt(this, "height");
      mt(this, "padding");
      mt(this, "font_size_scale", 1);
      mt(this, "text_height_metric");
      mt(this, "align", "left");
      mt(this, "_base_font_size", 13);
      // the same as :host's font-size (13px)
      mt(this, "_x_anchor", "left");
      mt(this, "_y_anchor", "center");
    }
    set base_font_size(_) {
      _ != null && (this._base_font_size = _);
    }
    get base_font_size() {
      return this._base_font_size;
    }
    set position(_) {
      this._position = _;
    }
    get position() {
      return this._position;
    }
    infer_text_height() {
      return "ascent_descent";
    }
    bbox() {
      const { p0: _, p1: H, p2: V, p3: q } = this.rect(), G = Math.min(_.x, H.x, V.x, q.x), J = Math.min(_.y, H.y, V.y, q.y), Z = Math.max(_.x, H.x, V.x, q.x), te = Math.max(_.y, H.y, V.y, q.y);
      return new BBox$1({ left: G, right: Z, top: J, bottom: te });
    }
    size() {
      const { width: _, height: H } = this._size(), { angle: V } = this;
      if (V == null || V == 0)
        return { width: _, height: H };
      {
        const q = Math.cos(Math.abs(V)), G = Math.sin(Math.abs(V));
        return {
          width: Math.abs(_ * q + H * G),
          height: Math.abs(_ * G + H * q)
        };
      }
    }
    rect() {
      const _ = this._rect(), { angle: H } = this;
      if (H == null || H == 0)
        return _;
      {
        const { sx: V, sy: q } = this.position, G = new AffineTransform();
        return G.translate(V, q), G.rotate(H), G.translate(-V, -q), G.apply_rect(_);
      }
    }
    paint_rect(_) {
      const { p0: H, p1: V, p2: q, p3: G } = this.rect();
      _.save(), _.strokeStyle = "red", _.lineWidth = 1, _.beginPath();
      const { round: J } = Math;
      _.moveTo(J(H.x), J(H.y)), _.lineTo(J(V.x), J(V.y)), _.lineTo(J(q.x), J(q.y)), _.lineTo(J(G.x), J(G.y)), _.closePath(), _.stroke(), _.restore();
    }
    paint_bbox(_) {
      const { x: H, y: V, width: q, height: G } = this.bbox();
      _.save(), _.strokeStyle = "blue", _.lineWidth = 1, _.beginPath();
      const { round: J } = Math;
      _.moveTo(J(H), J(V)), _.lineTo(J(H), J(V + G)), _.lineTo(J(H + q), J(V + G)), _.lineTo(J(H + q), J(V)), _.closePath(), _.stroke(), _.restore();
    }
  }
  mt(GraphicsBox, "__name__", "GraphicsBox");
  class TextBox extends GraphicsBox {
    constructor({ text: H }) {
      super();
      mt(this, "text");
      mt(this, "color");
      mt(this, "outline_color");
      mt(this, "font");
      mt(this, "line_height");
      //padding: Padding
      mt(this, "_visual_align", "left");
      this.text = H;
    }
    set visuals(H) {
      const V = H.color, q = H.alpha, G = H.outline_color, J = H.font_style;
      let Z = H.font_size;
      const te = H.font, { font_size_scale: ne, base_font_size: ee } = this, ae = parse_css_font_size(Z);
      if (ae != null) {
        let { value: le, unit: ue } = ae;
        le *= ne, ue == "em" && ee != 0 && (le *= ee, ue = "px"), Z = `${le}${ue}`;
      }
      const oe = `${J} ${Z} ${te}`;
      this.font = oe, this.color = color2css(V, q), this.outline_color = color2css(G, q), this.line_height = H.line_height;
      const se = H.align;
      this._visual_align = se, this._x_anchor = se;
      const ce = H.baseline;
      this._y_anchor = (() => {
        switch (ce) {
          case "top":
            return "top";
          case "middle":
            return "center";
          case "bottom":
            return "bottom";
          default:
            return "baseline";
        }
      })();
    }
    infer_text_height() {
      if (this.text.includes(`
`))
        return "ascent_descent";
      {
        let V = function(q) {
          for (const G of new Set(q))
            if (!("0" <= G && G <= "9"))
              switch (G) {
                case ",":
                case ".":
                case "+":
                case "-":
                case "−":
                case "e":
                  continue;
                default:
                  return !1;
              }
          return !0;
        };
        var H = V;
        return V(this.text) ? "cap" : "ascent_descent";
      }
    }
    _text_line(H) {
      const V = this.text_height_metric ?? this.infer_text_height(), q = (() => {
        switch (V) {
          case "x":
          case "x_descent":
            return H.x_height;
          case "cap":
          case "cap_descent":
            return H.cap_height;
          case "ascent":
          case "ascent_descent":
            return H.ascent;
        }
      })(), G = (() => {
        switch (V) {
          case "x":
          case "cap":
          case "ascent":
            return 0;
          case "x_descent":
          case "cap_descent":
          case "ascent_descent":
            return H.descent;
        }
      })();
      return { height: q + G, ascent: q, descent: G };
    }
    get nlines() {
      return this.text.split(`
`).length;
    }
    _size() {
      var le, ue;
      const { font: H } = this, V = font_metrics(H), q = (this.line_height - 1) * V.height, G = this.text == "", J = this.text.split(`
`), Z = J.length, te = J.map((de) => text_width(de, H)), ee = this._text_line(V).height * Z, ae = ((le = this.width) == null ? void 0 : le.unit) == "%" ? this.width.value : 1, oe = ((ue = this.height) == null ? void 0 : ue.unit) == "%" ? this.height.value : 1, se = max$j(te) * ae, ce = G ? 0 : (ee + q * (Z - 1)) * oe;
      return { width: se, height: ce, metrics: V };
    }
    _computed_position(H, V, q) {
      const { width: G, height: J } = H, { sx: Z, sy: te, x_anchor: ne = this._x_anchor, y_anchor: ee = this._y_anchor } = this.position, ae = Z - (() => {
        if (isNumber$3(ne))
          return ne * G;
        switch (ne) {
          case "left":
            return 0;
          case "center":
            return 0.5 * G;
          case "right":
            return G;
        }
      })(), oe = te - (() => {
        if (isNumber$3(ee))
          return ee * J;
        switch (ee) {
          case "top":
            return 0;
          case "center":
            return 0.5 * J;
          case "bottom":
            return J;
          case "baseline":
            if (q == 1)
              switch (this.text_height_metric ?? this.infer_text_height()) {
                case "x":
                case "x_descent":
                  return V.x_height;
                case "cap":
                case "cap_descent":
                  return V.cap_height;
                case "ascent":
                case "ascent_descent":
                  return V.ascent;
              }
            else
              return 0.5 * J;
        }
      })();
      return { x: ae, y: oe };
    }
    _rect() {
      const { width: H, height: V, metrics: q } = this._size(), G = this.text.split(`
`).length, { x: J, y: Z } = this._computed_position({ width: H, height: V }, q, G);
      return new BBox$1({ x: J, y: Z, width: H, height: V }).rect;
    }
    paint(H) {
      var pe, ye;
      const { font: V } = this, q = font_metrics(V), G = (this.line_height - 1) * q.height, J = this.text.split(`
`), Z = J.length, te = J.map((ge) => text_width(ge, V)), ne = this._text_line(q), ee = ne.height * Z, ae = ((pe = this.width) == null ? void 0 : pe.unit) == "%" ? this.width.value : 1, oe = ((ye = this.height) == null ? void 0 : ye.unit) == "%" ? this.height.value : 1, se = max$j(te) * ae, ce = (ee + G * (Z - 1)) * oe;
      H.save(), H.fillStyle = this.color, H.strokeStyle = this.outline_color, H.font = this.font, H.textAlign = "left", H.textBaseline = "alphabetic";
      const { sx: le, sy: ue } = this.position, { align: de } = this, { angle: fe } = this;
      fe != null && fe != 0 && (H.translate(le, ue), H.rotate(fe), H.translate(-le, -ue));
      let { x: he, y: me } = this._computed_position({ width: se, height: ce }, q, Z);
      if (de == "justify")
        for (let ge = 0; ge < Z; ge++) {
          let ve = he;
          const Ce = J[ge].split(" "), Se = Ce.length, we = Ce.map((xe) => text_width(xe, V)), Ee = (se - sum$3(we)) / (Se - 1);
          for (let xe = 0; xe < Se; xe++)
            H.fillText(Ce[xe], ve, me), H.strokeText(Ce[xe], ve, me), ve += we[xe] + Ee;
          me += /*heights[i]*/
          ne.height + G;
        }
      else
        for (let ge = 0; ge < Z; ge++) {
          const ve = he + (() => {
            switch (de == "auto" ? this._visual_align : de) {
              case "left":
                return 0;
              case "center":
                return 0.5 * (se - te[ge]);
              case "right":
                return se - te[ge];
            }
          })(), be = J[ge], Ce = me + /*ascents[i]*/
          ne.ascent;
          H.fillText(be, ve, Ce), H.strokeText(be, ve, Ce), me += /*heights[i]*/
          ne.height + G;
        }
      H.restore();
    }
  }
  mt(TextBox, "__name__", "TextBox");
  class BaseExpo extends GraphicsBox {
    constructor(H, V) {
      super();
      mt(this, "base");
      mt(this, "expo");
      this.base = H, this.expo = V;
    }
    get children() {
      return [this.base, this.expo];
    }
    set base_font_size(H) {
      super.base_font_size = H, this.base.base_font_size = H, this.expo.base_font_size = H;
    }
    set position(H) {
      this._position = H;
      const V = this.base.size(), q = this.expo.size(), G = this._shift_scale() * V.height, J = Math.max(V.height, G + q.height);
      this.base.position = {
        sx: 0,
        x_anchor: "left",
        sy: J,
        y_anchor: "bottom"
      }, this.expo.position = {
        sx: V.width,
        x_anchor: "left",
        sy: G,
        y_anchor: "bottom"
      };
    }
    get position() {
      return this._position;
    }
    set visuals(H) {
      this.expo.font_size_scale = 0.7, this.base.visuals = H, this.expo.visuals = H;
    }
    _shift_scale() {
      if (this.base instanceof TextBox && this.base.nlines == 1) {
        const { x_height: H, cap_height: V } = font_metrics(this.base.font);
        return H / V;
      } else
        return 2 / 3;
    }
    infer_text_height() {
      return this.base.infer_text_height();
    }
    _rect() {
      const H = this.base.bbox(), V = this.expo.bbox(), q = H.union(V), { x: G, y: J } = this._computed_position();
      return q.translate(G, J).rect;
    }
    _size() {
      const H = this.base.size(), V = this.expo.size();
      return {
        width: H.width + V.width,
        height: Math.max(H.height, this._shift_scale() * H.height + V.height)
      };
    }
    paint(H) {
      H.save();
      const { angle: V } = this;
      if (V != null && V != 0) {
        const { sx: J, sy: Z } = this.position;
        H.translate(J, Z), H.rotate(V), H.translate(-J, -Z);
      }
      const { x: q, y: G } = this._computed_position();
      H.translate(q, G), this.base.paint(H), this.expo.paint(H), H.restore();
    }
    // paint_rect ...
    paint_bbox(H) {
      super.paint_bbox(H);
      const { x: V, y: q } = this._computed_position();
      H.save(), H.translate(V, q);
      for (const G of this.children)
        G.paint_bbox(H);
      H.restore();
    }
    _computed_position() {
      const { width: H, height: V } = this._size(), { sx: q, sy: G, x_anchor: J = this._x_anchor, y_anchor: Z = this._y_anchor } = this.position, te = q - (() => {
        if (isNumber$3(J))
          return J * H;
        switch (J) {
          case "left":
            return 0;
          case "center":
            return 0.5 * H;
          case "right":
            return H;
        }
      })(), ne = G - (() => {
        if (isNumber$3(Z))
          return Z * V;
        switch (Z) {
          case "top":
            return 0;
          case "center":
            return 0.5 * V;
          case "bottom":
            return V;
          case "baseline":
            return 0.5 * V;
        }
      })();
      return { x: te, y: ne };
    }
  }
  mt(BaseExpo, "__name__", "BaseExpo");
  class GraphicsBoxes {
    constructor(_) {
      mt(this, "items");
      this.items = _;
    }
    set base_font_size(_) {
      for (const H of this.items)
        H.base_font_size = _;
    }
    get length() {
      return this.items.length;
    }
    set visuals(_) {
      for (const q of this.items)
        q.visuals = _;
      const H = { x: 0, cap: 1, ascent: 2, x_descent: 3, cap_descent: 4, ascent_descent: 5 }, V = max_by(this.items.map((q) => q.infer_text_height()), (q) => H[q]);
      for (const q of this.items)
        q.text_height_metric = V;
    }
    set angle(_) {
      for (const H of this.items)
        H.angle = _;
    }
    max_size() {
      let _ = 0, H = 0;
      for (const V of this.items) {
        const q = V.size();
        _ = Math.max(_, q.width), H = Math.max(H, q.height);
      }
      return { width: _, height: H };
    }
  }
  mt(GraphicsBoxes, "__name__", "GraphicsBoxes");
  function is_ModuleError(O) {
    return O instanceof Error && "code" in O;
  }
  async function load_module(O) {
    try {
      return await O;
    } catch (_) {
      if (is_ModuleError(_) && _.code === "MODULE_NOT_FOUND")
        return null;
      throw _;
    }
  }
  class MathJaxProvider {
    constructor() {
      mt(this, "ready", new Signal0(this, "ready"));
      mt(this, "status", "not_started");
    }
  }
  mt(MathJaxProvider, "__name__", "MathJaxProvider");
  class BundleProvider extends MathJaxProvider {
    constructor() {
      super(...arguments);
      mt(this, "_mathjax");
    }
    get MathJax() {
      return this._mathjax;
    }
    async fetch() {
      this.status = "loading";
      try {
        const H = await load_module(Promise.resolve().then(() => index$2));
        this.status = H == null ? "failed" : "loaded", this._mathjax = H, this.ready.emit();
      } catch {
        this.status = "failed";
      }
    }
  }
  mt(BundleProvider, "__name__", "BundleProvider");
  const default_provider = new BundleProvider();
  class MathTextView extends BaseTextView {
    constructor() {
      super(...arguments);
      mt(this, "valign");
      mt(this, "angle");
      mt(this, "_position", { sx: 0, sy: 0 });
      // Align does nothing, needed to maintain compatibility with TextBox,
      // to align you need to use TeX Macros.
      // http://docs.mathjax.org/en/latest/input/tex/macros/index.html?highlight=align
      mt(this, "align", "left");
      mt(this, "_x_anchor", "left");
      mt(this, "_y_anchor", "center");
      mt(this, "_base_font_size", 13);
      mt(this, "font_size_scale", 1);
      mt(this, "font");
      mt(this, "color");
      mt(this, "svg_image", null);
      mt(this, "svg_element");
      mt(this, "width");
      mt(this, "height");
    }
    graphics() {
      return this;
    }
    // Same for infer_text_height
    infer_text_height() {
      return "ascent_descent";
    }
    // the same as :host's font-size (13px)
    set base_font_size(H) {
      H != null && (this._base_font_size = H);
    }
    get base_font_size() {
      return this._base_font_size;
    }
    _rect() {
      const { width: H, height: V } = this._size(), { x: q, y: G } = this._computed_position();
      return new BBox$1({ x: q, y: G, width: H, height: V }).rect;
    }
    set position(H) {
      this._position = H;
    }
    get position() {
      return this._position;
    }
    get text() {
      return this.model.text;
    }
    get provider() {
      return default_provider;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), this.provider.status == "not_started" && await this.provider.fetch();
    }
    connect_signals() {
      super.connect_signals(), this.on_change(this.model.properties.text, () => this.load_image());
    }
    set visuals(H) {
      const V = H.color, q = H.alpha, G = H.font_style;
      let J = H.font_size;
      const Z = H.font, { font_size_scale: te, _base_font_size: ne } = this, ee = parse_css_font_size(J);
      if (ee != null) {
        let { value: ce, unit: le } = ee;
        ce *= te, le == "em" && ne != 0 && (ce *= ne, le = "px"), J = `${ce}${le}`;
      }
      const ae = `${G} ${J} ${Z}`;
      this.font = ae, this.color = color2css(V, q);
      const oe = H.align;
      this._x_anchor = oe;
      const se = H.baseline;
      this._y_anchor = (() => {
        switch (se) {
          case "top":
            return "top";
          case "middle":
            return "center";
          case "bottom":
            return "bottom";
          default:
            return "baseline";
        }
      })();
    }
    /**
     * Calculates position of element after considering
     * anchor and dimensions
     */
    _computed_position() {
      const { width: H, height: V } = this._size(), { sx: q, sy: G, x_anchor: J = this._x_anchor, y_anchor: Z = this._y_anchor } = this.position, te = font_metrics(this.font), ne = q - (() => {
        if (isNumber$3(J))
          return J * H;
        switch (J) {
          case "left":
            return 0;
          case "center":
            return 0.5 * H;
          case "right":
            return H;
        }
      })(), ee = G - (() => {
        if (isNumber$3(Z))
          return Z * V;
        switch (Z) {
          case "top":
            return te.height > V ? V - (-this.valign - te.descent) - te.height : 0;
          case "center":
            return 0.5 * V;
          case "bottom":
            return te.height > V ? V + te.descent + this.valign : V;
          case "baseline":
            return 0.5 * V;
        }
      })();
      return { x: ne, y: ee };
    }
    /**
     * Uses the width, height and given angle to calculate the size
    */
    size() {
      const { width: H, height: V } = this._size(), { angle: q } = this;
      if (q == null || q == 0)
        return { width: H, height: V };
      {
        const G = Math.cos(Math.abs(q)), J = Math.sin(Math.abs(q));
        return {
          width: Math.abs(H * G + V * J),
          height: Math.abs(H * J + V * G)
        };
      }
    }
    get_image_dimensions() {
      var G;
      const H = font_metrics(this.font), V = (G = this.svg_element.getAttribute("style")) == null ? void 0 : G.split(";");
      if (V != null) {
        const J = /* @__PURE__ */ new Map();
        V.forEach((te) => {
          const [ne, ee] = te.split(":");
          ne.trim() != "" && J.set(ne.trim(), ee.trim());
        });
        const Z = parse_css_length(J.get("vertical-align"));
        (Z == null ? void 0 : Z.unit) == "ex" ? this.valign = Z.value * H.x_height : (Z == null ? void 0 : Z.unit) == "px" && (this.valign = Z.value);
      }
      const q = (() => {
        const J = this.svg_element.getAttribute("width"), Z = this.svg_element.getAttribute("height");
        return {
          width: J != null && J.endsWith("ex") ? parseFloat(J) : 1,
          height: Z != null && Z.endsWith("ex") ? parseFloat(Z) : 1
        };
      })();
      return {
        width: H.x_height * q.width,
        height: H.x_height * q.height
      };
    }
    get truncated_text() {
      return this.model.text.length > 6 ? `${this.model.text.substring(0, 6)}...` : this.model.text;
    }
    _size() {
      var Z, te;
      if (this.svg_image == null)
        return this.provider.status == "failed" || this.provider.status == "not_started" ? {
          width: text_width(this.truncated_text, this.font),
          height: font_metrics(this.font).height
        } : { width: this._base_font_size, height: this._base_font_size };
      const H = font_metrics(this.font);
      let { width: V, height: q } = this.get_image_dimensions();
      q = Math.max(q, H.height);
      const G = ((Z = this.width) == null ? void 0 : Z.unit) == "%" ? this.width.value : 1, J = ((te = this.height) == null ? void 0 : te.unit) == "%" ? this.height.value : 1;
      return { width: V * G, height: q * J };
    }
    bbox() {
      const { p0: H, p1: V, p2: q, p3: G } = this.rect(), J = Math.min(H.x, V.x, q.x, G.x), Z = Math.min(H.y, V.y, q.y, G.y), te = Math.max(H.x, V.x, q.x, G.x), ne = Math.max(H.y, V.y, q.y, G.y);
      return new BBox$1({ left: J, right: te, top: Z, bottom: ne });
    }
    rect() {
      const H = this._rect(), { angle: V } = this;
      if (V == null || V == 0)
        return H;
      {
        const { sx: q, sy: G } = this.position, J = new AffineTransform();
        return J.translate(q, G), J.rotate(V), J.translate(-q, -G), J.apply_rect(H);
      }
    }
    paint_rect(H) {
      const { p0: V, p1: q, p2: G, p3: J } = this.rect();
      H.save(), H.strokeStyle = "red", H.lineWidth = 1, H.beginPath();
      const { round: Z } = Math;
      H.moveTo(Z(V.x), Z(V.y)), H.lineTo(Z(q.x), Z(q.y)), H.lineTo(Z(G.x), Z(G.y)), H.lineTo(Z(J.x), Z(J.y)), H.closePath(), H.stroke(), H.restore();
    }
    paint_bbox(H) {
      const { x: V, y: q, width: G, height: J } = this.bbox();
      H.save(), H.strokeStyle = "blue", H.lineWidth = 1, H.beginPath();
      const { round: Z } = Math;
      H.moveTo(Z(V), Z(q)), H.lineTo(Z(V), Z(q + J)), H.lineTo(Z(V + G), Z(q + J)), H.lineTo(Z(V + G), Z(q)), H.closePath(), H.stroke(), H.restore();
    }
    async request_image() {
      if (this.provider.MathJax == null)
        return;
      const H = this._process_text();
      if (H == null) {
        this._has_finished = !0;
        return;
      }
      const V = H.children[0];
      this.svg_element = V, V.setAttribute("font", this.font), V.setAttribute("stroke", this.color);
      const q = V.outerHTML, G = `data:image/svg+xml;utf-8,${encodeURIComponent(q)}`;
      this.svg_image = await load_image(G);
    }
    async load_image() {
      await this.request_image(), this.parent.request_layout();
    }
    /**
     * Takes a Canvas' Context2d and if the image has already
     * been loaded draws the image in it otherwise draws the model's text.
    */
    paint(H) {
      this.svg_image == null && ((this.provider.status == "not_started" || this.provider.status == "loading") && this.provider.ready.connect(() => this.load_image()), this.provider.status == "loaded" && this.load_image()), H.save();
      const { sx: V, sy: q } = this.position, { angle: G } = this;
      G != null && G != 0 && (H.translate(V, q), H.rotate(G), H.translate(-V, -q));
      const { x: J, y: Z } = this._computed_position();
      if (this.svg_image != null) {
        const { width: te, height: ne } = this.get_image_dimensions();
        H.drawImage(this.svg_image, J, Z, te, ne);
      } else (this.provider.status == "failed" || this.provider.status == "not_started") && (H.fillStyle = this.color, H.font = this.font, H.textAlign = "left", H.textBaseline = "alphabetic", H.fillText(this.truncated_text, J, Z + font_metrics(this.font).ascent));
      H.restore(), !this._has_finished && (this.provider.status == "failed" || this.svg_image != null) && (this._has_finished = !0, this.parent.notify_finished_after_paint());
    }
  }
  mt(MathTextView, "__name__", "MathTextView");
  class MathText extends BaseText {
    constructor(_) {
      super(_);
    }
  }
  mt(MathText, "__name__", "MathText");
  class AsciiView extends MathTextView {
    // TODO: Color ascii
    get styled_text() {
      return this.text;
    }
    _process_text() {
    }
    _size() {
      return {
        width: text_width(this.text, this.font),
        height: font_metrics(this.font).height
      };
    }
    paint(_) {
      _.save();
      const { sx: H, sy: V } = this.position, { angle: q } = this;
      q != null && q != 0 && (_.translate(H, V), _.rotate(q), _.translate(-H, -V));
      const { x: G, y: J } = this._computed_position();
      _.fillStyle = this.color, _.font = this.font, _.textAlign = "left", _.textBaseline = "alphabetic", _.fillText(this.text, G, J + font_metrics(this.font).ascent), _.restore(), this._has_finished = !0, this.parent.notify_finished_after_paint();
    }
  }
  mt(AsciiView, "__name__", "AsciiView");
  const Cg = class Cg extends MathText {
    constructor(_) {
      super(_);
    }
  };
  mt(Cg, "__name__", "Ascii"), Cg.prototype.default_view = AsciiView;
  let Ascii = Cg;
  class MathMLView extends MathTextView {
    get styled_text() {
      let _ = this.text.trim(), H = _.match(/<math(.*?[^?])?>/s);
      return H == null ? this.text.trim() : (_ = insert_text_on_position(_, _.indexOf(H[0]) + H[0].length, `<mstyle displaystyle="true" mathcolor="${color2hexrgb(this.color)}" ${this.font.includes("bold") ? 'mathvariant="bold"' : ""}>`), H = _.match(/<\/[^>]*?math.*?>/s), H == null ? this.text.trim() : insert_text_on_position(_, _.indexOf(H[0]), "</mstyle>"));
    }
    _process_text() {
      var H;
      const _ = font_metrics(this.font);
      return (H = this.provider.MathJax) == null ? void 0 : H.mathml2svg(this.styled_text, {
        em: this.base_font_size,
        ex: _.x_height
      });
    }
  }
  mt(MathMLView, "__name__", "MathMLView");
  const Lg = class Lg extends MathText {
    constructor(_) {
      super(_);
    }
  };
  mt(Lg, "__name__", "MathML"), Lg.prototype.default_view = MathMLView;
  let MathML = Lg;
  class TeXView extends MathTextView {
    get styled_text() {
      const [_, H, V] = color2rgba(this.color);
      return `\\color[RGB]{${_}, ${H}, ${V}} ${this.font.includes("bold") ? `\\pmb{${this.text}}` : this.text}`;
    }
    _process_text() {
      var H;
      const _ = font_metrics(this.font);
      return (H = this.provider.MathJax) == null ? void 0 : H.tex2svg(this.styled_text, {
        display: !this.model.inline,
        em: this.base_font_size,
        ex: _.x_height
      }, to_object(this.model.macros));
    }
  }
  mt(TeXView, "__name__", "TeXView");
  const P3 = class P3 extends MathText {
    constructor(_) {
      super(_);
    }
  };
  mt(P3, "__name__", "TeX"), P3.prototype.default_view = TeXView, P3.define(({ Bool: _, Float: H, Str: V, Dict: q, Tuple: G, Or: J }) => ({
    macros: [q(J(V, G(V, H))), {}],
    inline: [_, !1]
  }));
  let TeX = P3;
  class PlainTextView extends BaseTextView {
    initialize() {
      super.initialize(), this._has_finished = !0;
    }
    graphics() {
      return new TextBox({ text: this.model.text });
    }
  }
  mt(PlainTextView, "__name__", "PlainTextView");
  const kg = class kg extends BaseText {
    constructor(_) {
      super(_);
    }
  };
  mt(kg, "__name__", "PlainText"), kg.prototype.default_view = PlainTextView;
  let PlainText = kg;
  const delimiters$2 = [
    { start: "$$", end: "$$", inline: !1 },
    { start: "\\[", end: "\\]", inline: !1 },
    { start: "\\(", end: "\\)", inline: !0 }
  ];
  function add_backslash(O) {
    return O.split("").map((_) => `\\${_}`).join("");
  }
  function parse_delimited_string(O) {
    const _ = [];
    for (const H of delimiters$2) {
      const V = add_backslash(H.start), q = add_backslash(H.end), G = new RegExp(`${V}([^]*?)${q}`, "gm");
      for (const J of O.matchAll(G)) {
        const { index: Z } = J;
        if (Z != null) {
          const [te, ne] = J;
          _.push({ index: Z, outer: te, inner: ne, delim: H });
        }
      }
    }
    if (_.length > 0) {
      if (_.length == 1) {
        const [q] = _, G = q.index, J = q.outer.length;
        if (G == 0 && J == O.length) {
          const Z = q.inner;
          return new TeX({ text: Z, inline: q.delim.inline });
        }
      }
      let H = "", V = 0;
      for (const q of sort_by(_, (G) => G.index)) {
        const G = q.index;
        V <= G && (H += G != 0 ? `\\text{${O.slice(V, G)}}${q.inner}` : `${q.inner}`, V = G + q.outer.length);
      }
      return H += V < O.length ? `\\text{${O.slice(V)}}` : "", new TeX({ text: H, inline: !1 });
    } else
      return new PlainText({ text: O });
  }
  const Length = NonNegative(Int$1), XY = (O) => PartialStruct({ x: O, y: O }), LRTB$1 = (O) => PartialStruct({ left: O, right: O, top: O, bottom: O }), Anchor$1 = Or$1(Anchor$2, Tuple(Or$1(Align, HAlign, Percent), Or$1(Align, VAlign, Percent))), TextAnchor = Or$1(Anchor$1, Auto), Padding$2 = Or$1(Length, Tuple(Length, Length), XY(Length), Tuple(Length, Length, Length, Length), LRTB$1(Length)), BorderRadius = Or$1(Length, Tuple(Length, Length, Length, Length), PartialStruct({
    top_left: Length,
    top_right: Length,
    bottom_right: Length,
    bottom_left: Length
  })), Index$2 = NonNegative(Int$1), Span$4 = NonNegative(Int$1), GridChild = (O) => Tuple(Ref(O), Index$2, Index$2, Opt(Span$4), Opt(Span$4)), GridSpacing = Or$1(Length, Tuple(Length, Length)), TrackAlign = Enum("start", "center", "end", "auto"), TrackSize = Str, TrackSizing = PartialStruct({ size: TrackSize, align: TrackAlign }), TrackSizingLike = Or$1(TrackSize, TrackSizing), TracksSizing = Or$1(TrackSizingLike, List$2(TrackSizingLike), Mapping(Int$1, TrackSizingLike));
  function anchor(O) {
    if (isString$2(O))
      switch (O) {
        case "top_left":
          return { x: 0, y: 0 };
        case "top":
        case "top_center":
          return { x: 0.5, y: 0 };
        case "top_right":
          return { x: 1, y: 0 };
        case "right":
        case "center_right":
          return { x: 1, y: 0.5 };
        case "bottom_right":
          return { x: 1, y: 1 };
        case "bottom":
        case "bottom_center":
          return { x: 0.5, y: 1 };
        case "bottom_left":
          return { x: 0, y: 1 };
        case "left":
        case "center_left":
          return { x: 0, y: 0.5 };
        case "center":
        case "center_center":
          return { x: 0.5, y: 0.5 };
      }
    else {
      const _ = (() => {
        const [V] = O;
        switch (V) {
          case "start":
          case "left":
            return 0;
          case "center":
            return 0.5;
          case "end":
          case "right":
            return 1;
          default:
            return V;
        }
      })(), H = (() => {
        const [, V] = O;
        switch (V) {
          case "start":
          case "top":
            return 0;
          case "center":
            return 0.5;
          case "end":
          case "bottom":
            return 1;
          default:
            return V;
        }
      })();
      return { x: _, y: H };
    }
  }
  function text_anchor(O, _, H) {
    if (O != "auto")
      return anchor(O);
    {
      const V = (() => {
        switch (_) {
          case "left":
            return "start";
          case "center":
            return "center";
          case "right":
            return "end";
        }
      })(), q = (() => {
        switch (H) {
          case "alphabetic":
          case "ideographic":
          case "hanging":
            return "center";
          case "top":
            return "start";
          case "middle":
            return "center";
          case "bottom":
            return "end";
        }
      })();
      return anchor([V, q]);
    }
  }
  function padding$2(O) {
    if (isNumber$3(O))
      return { left: O, right: O, top: O, bottom: O };
    if (isPlainObject(O))
      if ("x" in O || "y" in O) {
        const { x: _ = 0, y: H = 0 } = O;
        return { left: _, right: _, top: H, bottom: H };
      } else if ("left" in O || "right" in O || "top" in O || "bottom" in O) {
        const { left: _ = 0, right: H = 0, top: V = 0, bottom: q = 0 } = O;
        return { left: _, right: H, top: V, bottom: q };
      } else
        unreachable();
    else if (O.length == 2) {
      const [_ = 0, H = 0] = O;
      return { left: _, right: _, top: H, bottom: H };
    } else {
      const [_ = 0, H = 0, V = 0, q = 0] = O;
      return { left: _, right: H, top: V, bottom: q };
    }
  }
  function border_radius(O) {
    if (isNumber$3(O))
      return {
        top_left: O,
        top_right: O,
        bottom_right: O,
        bottom_left: O
      };
    if (isPlainObject(O))
      return {
        top_left: O.top_left ?? 0,
        top_right: O.top_right ?? 0,
        bottom_right: O.bottom_right ?? 0,
        bottom_left: O.bottom_left ?? 0
      };
    {
      const [_ = 0, H = 0, V = 0, q = 0] = O;
      return { top_left: _, top_right: H, bottom_right: V, bottom_left: q };
    }
  }
  function round_rect(O, _, H) {
    let { top_left: V, top_right: q, bottom_right: G, bottom_left: J } = H;
    if (V != 0 || q != 0 || G != 0 || J != 0) {
      const { left: Z, right: te, top: ne, bottom: ee, width: ae, height: oe } = _, se = Math.min(ae / (V + q), oe / (q + G), ae / (G + J), oe / (V + J));
      se < 1 && (V *= se, q *= se, G *= se, J *= se), O.moveTo(Z + V, ne), O.lineTo(te - q, ne), q != 0 && O.arcTo(te, ne, te, ne + q, q), O.lineTo(te, ee - G), G != 0 && O.arcTo(te, ee, te - G, ee, G), O.lineTo(Z + J, ee), J != 0 && O.arcTo(Z, ee, Z, ee - J, J), O.lineTo(Z, ne + V), V != 0 && O.arcTo(Z, ne, Z + V, ne, V), O.closePath();
    } else {
      const { left: Z, top: te, width: ne, height: ee } = _;
      O.rect(Z, te, ne, ee);
    }
  }
  var J_;
  let TextAnnotationView$1 = (J_ = class extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "_text_view");
      mt(this, "_text_box");
      mt(this, "_rect");
    }
    *children() {
      yield* super.children(), yield this._text_view;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._init_text();
    }
    async _init_text() {
      const { text: H } = this.model, V = isString$2(H) ? parse_delimited_string(H) : H;
      this._text_view = await build_view(V, { parent: this });
    }
    update_layout() {
      const { panel: H } = this;
      H != null ? this.layout = new SideLayout(H, () => this.get_size(), !1) : this.layout = void 0;
    }
    connect_signals() {
      super.connect_signals();
      const { text: H } = this.model.properties;
      this.on_change(H, async () => {
        this._text_view.remove(), await this._init_text();
      }), this.connect(this.model.change, () => this.request_render());
    }
    remove() {
      this._text_view.remove(), super.remove();
    }
    has_finished() {
      return !(!super.has_finished() || !this._text_view.has_finished());
    }
    get displayed() {
      return super.displayed && this._text_view.model.text != "" && this.visuals.text.doit;
    }
    get padding() {
      return padding$2(this.model.padding);
    }
    get border_radius() {
      return border_radius(this.model.border_radius);
    }
    _get_size() {
      if (!this.displayed)
        return { width: 0, height: 0 };
      const H = this._text_view.graphics();
      H.angle = this.angle, H.align = "auto", H.visuals = this.visuals.text.values();
      const V = H.size(), { padding: q } = this, G = V.width + q.left + q.right, J = V.height + q.top + q.bottom;
      return { width: G, height: J };
    }
    compute_geometry() {
      super.compute_geometry();
      const H = this._text_view.graphics();
      H.position = { sx: 0, sy: 0, x_anchor: "left", y_anchor: "top" }, H.angle = 0, H.align = "auto", H.visuals = this.visuals.text.values();
      const V = H.size(), { sx: q, sy: G } = this.origin, { anchor: J, padding: Z, border_radius: te, angle: ne } = this, ee = V.width + Z.left + Z.right, ae = V.height + Z.top + Z.bottom;
      this._text_box = H, this._rect = { sx: q, sy: G, width: ee, height: ae, angle: ne, anchor: J, padding: Z, border_radius: te };
    }
    _render() {
      this.compute_geometry();
      const { ctx: H } = this.layer, { sx: V, sy: q, width: G, height: J, angle: Z, anchor: te, padding: ne, border_radius: ee } = this._rect, ae = this._text_box, oe = te.x * G, se = te.y * J;
      H.translate(V, q), H.rotate(Z), H.translate(-oe, -se);
      const { background_fill: ce, background_hatch: le, border_line: ue, text: de } = this.visuals;
      if (ce.doit || le.doit || ue.doit) {
        H.beginPath();
        const fe = new BBox$1({ x: 0, y: 0, width: G, height: J });
        round_rect(H, fe, ee), ce.apply(H), le.apply(H), ue.apply(H);
      }
      if (de.doit) {
        const { left: fe, top: he } = ne;
        H.translate(fe, he), ae.paint(H), H.translate(-fe, -he);
      }
      H.translate(oe, se), H.rotate(-Z), H.translate(-V, -q);
    }
  }, mt(J_, "__name__", "TextAnnotationView"), J_);
  var c2;
  let TextAnnotation$1 = (c2 = class extends Annotation$1 {
    constructor(_) {
      super(_);
    }
  }, mt(c2, "__name__", "TextAnnotation"), c2.mixins([
    Text$4,
    ["border_", Line$4],
    ["background_", Fill$2],
    ["background_", Hatch$1]
  ]), c2.define(({ Str: _, Or: H, Ref: V }) => ({
    text: [H(_, V(BaseText)), ""],
    padding: [Padding$2, 0],
    border_radius: [BorderRadius, 0]
  })), c2.override({
    border_line_color: null,
    background_fill_color: null,
    background_hatch_color: null
  }), c2);
  class TitleView extends TextAnnotationView$1 {
    _get_position() {
      const _ = this.model.offset, H = this.model.standoff / 2, { align: V, vertical_align: q } = this.model;
      let G, J;
      const { bbox: Z } = this.layout;
      switch (this.panel.side) {
        case "above":
        case "below": {
          switch (q) {
            case "top":
              J = Z.top + H;
              break;
            case "middle":
              J = Z.vcenter;
              break;
            case "bottom":
              J = Z.bottom - H;
              break;
          }
          switch (V) {
            case "left":
              G = Z.left + _;
              break;
            case "center":
              G = Z.hcenter;
              break;
            case "right":
              G = Z.right - _;
              break;
          }
          break;
        }
        case "left": {
          switch (q) {
            case "top":
              G = Z.left + H;
              break;
            case "middle":
              G = Z.hcenter;
              break;
            case "bottom":
              G = Z.right - H;
              break;
          }
          switch (V) {
            case "left":
              J = Z.bottom - _;
              break;
            case "center":
              J = Z.vcenter;
              break;
            case "right":
              J = Z.top + _;
              break;
          }
          break;
        }
        case "right": {
          switch (q) {
            case "top":
              G = Z.right - H;
              break;
            case "middle":
              G = Z.hcenter;
              break;
            case "bottom":
              G = Z.left + H;
              break;
          }
          switch (V) {
            case "left":
              J = Z.top + _;
              break;
            case "center":
              J = Z.vcenter;
              break;
            case "right":
              J = Z.bottom - _;
              break;
          }
          break;
        }
      }
      return { sx: G, sy: J };
    }
    get anchor() {
      const { align: _, vertical_align: H } = this.model;
      return text_anchor("auto", _, H);
    }
    get origin() {
      return this._get_position();
    }
    get angle() {
      return this.panel.get_label_angle_heuristic("parallel");
    }
    _get_size() {
      const _ = (q) => q == 0 ? 0 : 2 + q + this.model.standoff, { width: H, height: V } = super._get_size();
      return this.panel.is_horizontal ? { width: H, height: _(V) } : { width: _(H), height: V };
    }
  }
  mt(TitleView, "__name__", "TitleView");
  const vp = class vp extends TextAnnotation$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(vp, "__name__", "Title"), vp.prototype.default_view = TitleView, vp.define(({ Float: _ }) => ({
    vertical_align: [VerticalAlign, "bottom"],
    align: [TextAlign, "left"],
    offset: [_, 0],
    standoff: [_, 10]
  })), vp.override({
    text_font_size: "13px",
    text_font_style: "bold",
    text_line_height: 1
  });
  let Title = vp;
  class CartesianFrame {
    constructor(_, H, V, q, G = {}, J = {}, Z = {}, te = {}) {
      mt(this, "in_x_scale");
      mt(this, "in_y_scale");
      mt(this, "x_range");
      mt(this, "y_range");
      mt(this, "extra_x_ranges");
      mt(this, "extra_y_ranges");
      mt(this, "extra_x_scales");
      mt(this, "extra_y_scales");
      mt(this, "_bbox", new BBox$1());
      mt(this, "change", new Signal0(this, "change"));
      mt(this, "_x_target");
      mt(this, "_y_target");
      mt(this, "_x_ranges");
      mt(this, "_y_ranges");
      mt(this, "_x_scales");
      mt(this, "_y_scales");
      this.in_x_scale = _, this.in_y_scale = H, this.x_range = V, this.y_range = q, this.extra_x_ranges = G, this.extra_y_ranges = J, this.extra_x_scales = Z, this.extra_y_scales = te, assert$2(_.properties.source_range.is_unset && _.properties.target_range.is_unset), assert$2(H.properties.source_range.is_unset && H.properties.target_range.is_unset), this._configure_scales();
    }
    get bbox() {
      return this._bbox;
    }
    _get_ranges(_, H) {
      return new Map([...entries$2(H), ["default", _]]);
    }
    /*protected*/
    _get_scales(_, H, V, q) {
      const G = new Map([...entries$2(H), ["default", _]]), J = /* @__PURE__ */ new Map();
      for (const [Z, te] of V) {
        const ne = te instanceof FactorRange, ee = _ instanceof CategoricalScale;
        if (ne != ee)
          throw new Error(`Range ${te.type} is incompatible is Scale ${_.type}`);
        _ instanceof LogScale && te instanceof DataRange1d && (te.scale_hint = "log");
        const oe = (G.get(Z) ?? _).clone();
        oe.setv({ source_range: te, target_range: q }), J.set(Z, oe);
      }
      return J;
    }
    _configure_frame_ranges() {
      const { bbox: _ } = this;
      this._x_target = new Range1d({ start: _.left, end: _.right }), this._y_target = new Range1d({ start: _.bottom, end: _.top });
    }
    _configure_scales() {
      this._configure_frame_ranges(), this._x_ranges = this._get_ranges(this.x_range, this.extra_x_ranges), this._y_ranges = this._get_ranges(this.y_range, this.extra_y_ranges), this._x_scales = this._get_scales(this.in_x_scale, this.extra_x_scales, this._x_ranges, this._x_target), this._y_scales = this._get_scales(this.in_y_scale, this.extra_y_scales, this._y_ranges, this._y_target);
    }
    configure_scales() {
      this._configure_scales(), this.change.emit();
    }
    _update_scales() {
      this._configure_frame_ranges();
      for (const [, _] of this._x_scales)
        _.target_range = this._x_target;
      for (const [, _] of this._y_scales)
        _.target_range = this._y_target;
    }
    set_geometry(_) {
      this._bbox = _, this._update_scales();
    }
    get x_target() {
      return this._x_target;
    }
    get y_target() {
      return this._y_target;
    }
    get x_ranges() {
      return this._x_ranges;
    }
    get y_ranges() {
      return this._y_ranges;
    }
    get ranges() {
      return /* @__PURE__ */ new Set([...this.x_ranges.values(), ...this.y_ranges.values()]);
    }
    get x_scales() {
      return this._x_scales;
    }
    get y_scales() {
      return this._y_scales;
    }
    get scales() {
      return /* @__PURE__ */ new Set([...this.x_scales.values(), ...this.y_scales.values()]);
    }
    get x_scale() {
      return this._x_scales.get("default");
    }
    get y_scale() {
      return this._y_scales.get("default");
    }
    // TODO remove this when CartesianFrameView is implemented
    resolve_symbol(_) {
      const V = this.bbox.resolve(_.symbol), { offset: q } = _;
      if (isNumber$3(V))
        return V + q;
      {
        const { x: G, y: J } = V;
        return { x: G + q, y: J + q };
      }
    }
  }
  mt(CartesianFrame, "__name__", "CartesianFrame");
  class GuideRendererView extends RendererView {
  }
  mt(GuideRendererView, "__name__", "GuideRendererView");
  const Mg = class Mg extends Renderer$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Mg, "__name__", "GuideRenderer"), Mg.override({
    level: "guide"
  });
  let GuideRenderer = Mg;
  class Ticker extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(Ticker, "__name__", "Ticker");
  class TickFormatter extends Model$1 {
    constructor(_) {
      super(_);
    }
    format_graphics(_, H) {
      return this.doFormat(_, H).map((V) => new TextBox({ text: V }));
    }
    compute(_, H) {
      return this.doFormat([_], H ?? { loc: 0 })[0];
    }
    v_compute(_, H) {
      return this.doFormat(_, H ?? { loc: 0 });
    }
  }
  mt(TickFormatter, "__name__", "TickFormatter");
  class LabelingPolicy extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(LabelingPolicy, "__name__", "LabelingPolicy");
  class AllLabels extends LabelingPolicy {
    constructor(_) {
      super(_);
    }
    filter(_, H, V) {
      return _;
    }
  }
  mt(AllLabels, "__name__", "AllLabels");
  const Rg = class Rg extends LabelingPolicy {
    constructor(_) {
      super(_);
    }
    filter(_, H, V) {
      const { min_distance: q } = this;
      let G = null;
      for (const J of _)
        G != null && V(G, J) < q ? _.unset(J) : G = J;
      return _;
    }
  };
  mt(Rg, "__name__", "NoOverlap"), Rg.define(({ Float: _ }) => ({
    min_distance: [_, 5]
  }));
  let NoOverlap = Rg;
  const Og = class Og extends LabelingPolicy {
    constructor(_) {
      super(_);
    }
    get names() {
      return keys$5(this.args);
    }
    get values() {
      return values$3(this.args);
    }
    get func() {
      const _ = use_strict(this.code);
      return new GeneratorFunction("indices", "bboxes", "distance", ...this.names, _);
    }
    filter(_, H, V) {
      const q = /* @__PURE__ */ Object.create(null), G = this.func.call(q, _, H, V, ...this.values);
      let J = G.next();
      if ((J.done ?? !1) && J.value !== void 0) {
        const { value: Z } = J;
        return Z instanceof BitSet ? Z : Z === void 0 ? _ : isIterable$2(Z) ? BitSet.from_indices(_.size, Z) : BitSet.all_unset(_.size);
      } else {
        const Z = [];
        do
          Z.push(J.value), J = G.next();
        while (!(J.done ?? !1));
        return BitSet.from_indices(_.size, Z);
      }
    }
  };
  mt(Og, "__name__", "CustomLabelingPolicy"), Og.define(({ Unknown: _, Str: H, Dict: V }) => ({
    args: [V(_), {}],
    code: [H, ""]
  }));
  let CustomLabelingPolicy = Og;
  const LabelOverrides = Or$1(Dict(Or$1(Str, Ref(BaseText))), Mapping(Or$1(Str, Float$1), Or$1(Str, Ref(BaseText)))), { abs: abs$b } = Math;
  class AxisView extends GuideRendererView {
    constructor() {
      super(...arguments);
      mt(this, "layout");
      mt(this, "_panel");
      /*private*/
      mt(this, "_axis_label_view", null);
      /*private*/
      mt(this, "_major_label_views", /* @__PURE__ */ new Map());
    }
    get panel() {
      return this._panel;
    }
    set panel(H) {
      this._panel = new SidePanel(H.side, this.model.face);
    }
    get bbox() {
      if (this.layout != null && this.model.fixed_location == null)
        return this.layout.bbox;
      if (this.is_renderable) {
        const { extents: H } = this, V = Math.round(H.tick + H.tick_label + H.axis_label);
        let { sx0: q, sy0: G, sx1: J, sy1: Z } = this.rule_scoords;
        const { dimension: te, face: ne } = this;
        return te == 0 ? ne == "front" ? G -= V : Z += V : ne == "front" ? q -= V : J += V, BBox$1.from_lrtb({ left: q, top: G, right: J, bottom: Z });
      } else
        return new BBox$1();
    }
    *children() {
      yield* super.children(), this._axis_label_view != null && (yield this._axis_label_view), yield* this._major_label_views.values();
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._init_axis_label(), await this._init_major_labels();
    }
    async _init_axis_label() {
      const { axis_label: H } = this.model;
      if (H != null) {
        const V = isString$2(H) ? parse_delimited_string(H) : H;
        this._axis_label_view = await build_view(V, { parent: this });
      } else
        this._axis_label_view = null;
    }
    async _init_major_labels() {
      for (const [H, V] of dict(this.model.major_label_overrides)) {
        const q = isString$2(V) ? parse_delimited_string(V) : V;
        this._major_label_views.set(H, await build_view(q, { parent: this }));
      }
    }
    update_layout() {
      this.layout = new SideLayout(this.panel, () => this.get_size(), !0), this.layout.on_resize(() => this._coordinates = void 0);
    }
    get_size() {
      const { visible: H, fixed_location: V } = this.model;
      if (H && V == null && this.is_renderable) {
        const { extents: q } = this;
        return { width: 0, height: Math.round(q.tick + q.tick_label + q.axis_label) };
      } else
        return { width: 0, height: 0 };
    }
    get is_renderable() {
      const [H, V] = this.ranges;
      return H.is_valid && V.is_valid && H.span > 0 && V.span > 0;
    }
    _render() {
      var G;
      if (!this.is_renderable)
        return;
      const { tick_coords: H, extents: V } = this, q = this.layer.ctx;
      q.save(), this._draw_background(q, V), this._draw_rule(q, V), this._draw_major_ticks(q, V, H), this._draw_minor_ticks(q, V, H), this._draw_major_labels(q, V, H), this._draw_axis_label(q, V, H), (G = this._paint) == null || G.call(this, q, V, H), q.restore();
    }
    connect_signals() {
      super.connect_signals();
      const { axis_label: H, major_label_overrides: V } = this.model.properties;
      this.on_change(H, async () => {
        var q;
        (q = this._axis_label_view) == null || q.remove(), await this._init_axis_label();
      }), this.on_change(V, async () => {
        for (const q of this._major_label_views.values())
          q.remove();
        await this._init_major_labels();
      }), this.connect(this.model.change, () => this.plot_view.request_layout());
    }
    get needs_clip() {
      return this.model.fixed_location != null;
    }
    // drawing sub functions -----------------------------------------------------
    _draw_background(H, V) {
      if (!this.visuals.background_fill.doit)
        return;
      H.beginPath();
      const { x: q, y: G, width: J, height: Z } = this.bbox;
      H.rect(q, G, J, Z), this.visuals.background_fill.apply(H);
    }
    _draw_rule(H, V) {
      if (!this.visuals.axis_line.doit)
        return;
      const { sx0: q, sy0: G, sx1: J, sy1: Z } = this.rule_scoords;
      H.beginPath(), H.moveTo(q, G), H.lineTo(J, Z), this.visuals.axis_line.apply(H);
    }
    _draw_major_ticks(H, V, q) {
      const G = this.model.major_tick_in, J = this.model.major_tick_out, Z = this.visuals.major_tick_line;
      this._draw_ticks(H, q.major, G, J, Z);
    }
    _draw_minor_ticks(H, V, q) {
      const G = this.model.minor_tick_in, J = this.model.minor_tick_out, Z = this.visuals.minor_tick_line;
      this._draw_ticks(H, q.minor, G, J, Z);
    }
    _draw_major_labels(H, V, q) {
      const G = q.major, J = this.compute_labels(G[this.dimension]), Z = this.model.major_label_orientation, te = V.tick + this.model.major_label_standoff, ne = this.visuals.major_label_text;
      this._draw_oriented_labels(H, J, G, Z, te, ne);
    }
    _axis_label_extent() {
      if (this._axis_label_view == null)
        return 0;
      const H = this._axis_label_view.graphics(), V = 3, q = this.model.axis_label_orientation;
      H.visuals = this.visuals.axis_label_text.values(), H.angle = this.panel.get_label_angle_heuristic(q), H.base_font_size = this.plot_view.base_font_size;
      const G = H.size(), J = this.dimension == 0 ? G.height : G.width, Z = this.model.axis_label_standoff;
      return J > 0 ? Z + J + V : 0;
    }
    _draw_axis_label(H, V, q) {
      if (this._axis_label_view == null)
        return;
      const [
        G,
        J
        /* TODO, x_anchor, y_anchor*/
      ] = (() => {
        const { bbox: le } = this, { side: ue, face: de } = this.panel, [fe] = this.ranges, { axis_label_align: he } = this.model;
        switch (ue) {
          case "above":
          case "below": {
            const [me, pe] = (() => {
              switch (he) {
                case "start":
                  return fe.is_reversed ? [le.right, "right"] : [le.left, "left"];
                case "center":
                  return [le.hcenter, "center"];
                case "end":
                  return fe.is_reversed ? [le.left, "left"] : [le.right, "right"];
              }
            })(), [ye, ge] = de == "front" ? [le.bottom, "bottom"] : [le.top, "top"];
            return [me, ye, pe, ge];
          }
          case "left":
          case "right": {
            const [me, pe] = (() => {
              switch (he) {
                case "start":
                  return fe.is_reversed ? [le.top, "top"] : [le.bottom, "bottom"];
                case "center":
                  return [le.vcenter, "center"];
                case "end":
                  return fe.is_reversed ? [le.bottom, "bottom"] : [le.top, "top"];
              }
            })(), [ye, ge] = de == "front" ? [le.right, "right"] : [le.left, "left"];
            return [ye, me, ge, pe];
          }
        }
      })(), [Z, te] = this.normals, ne = this.model.axis_label_orientation, ee = V.tick + V.tick_label + this.model.axis_label_standoff, { vertical_align: ae, align: oe } = this.panel.get_label_text_heuristics(ne), se = {
        sx: G + Z * ee,
        sy: J + te * ee,
        x_anchor: oe,
        y_anchor: ae
      }, ce = this._axis_label_view.graphics();
      ce.visuals = this.visuals.axis_label_text.values(), ce.angle = this.panel.get_label_angle_heuristic(ne), ce.base_font_size = this.plot_view.base_font_size, ce.position = se, ce.align = oe, ce.paint(H);
    }
    _draw_ticks(H, V, q, G, J) {
      if (!J.doit)
        return;
      const [Z, te] = this.scoords(V), [ne, ee] = this.normals, [ae, oe] = this.offsets, [se, ce] = [ne * (ae - q), ee * (oe - q)], [le, ue] = [ne * (ae + G), ee * (oe + G)];
      J.set_value(H), H.beginPath();
      for (let de = 0; de < Z.length; de++) {
        const fe = Math.round(Z[de] + le), he = Math.round(te[de] + ue), me = Math.round(Z[de] + se), pe = Math.round(te[de] + ce);
        H.moveTo(fe, he), H.lineTo(me, pe);
      }
      H.stroke();
    }
    _draw_oriented_labels(H, V, q, G, J, Z) {
      if (!Z.doit || V.length == 0)
        return;
      const [te, ne] = this.scoords(q), [ee, ae] = this.offsets, [oe, se] = this.normals, ce = oe * (ee + J), le = se * (ae + J), { vertical_align: ue, align: de } = this.panel.get_label_text_heuristics(G), fe = this.panel.get_label_angle_heuristic(G);
      V.visuals = Z.values(), V.angle = fe, V.base_font_size = this.plot_view.base_font_size;
      for (let Se = 0; Se < V.length; Se++) {
        const we = V.items[Se];
        we.position = {
          sx: te[Se] + ce,
          sy: ne[Se] + le,
          x_anchor: de,
          y_anchor: ue
        }, we instanceof TextBox && (we.align = de);
      }
      const he = V.length, me = BitSet.all_set(he), { items: pe } = V, ye = pe.map((Se) => Se.bbox()), ge = (() => {
        const [Se] = this.ranges;
        return Se.is_reversed ? this.dimension == 0 ? (we, Ee) => ye[we].left - ye[Ee].right : (we, Ee) => ye[Ee].top - ye[we].bottom : this.dimension == 0 ? (we, Ee) => ye[Ee].left - ye[we].right : (we, Ee) => ye[we].top - ye[Ee].bottom;
      })(), { major_label_policy: ve } = this.model, be = ve.filter(me, ye, ge), Ce = [...be.ones()];
      if (Ce.length != 0) {
        const Se = this.canvas.bbox, we = (Qe) => {
          const ke = ye[Qe];
          if (ke.left < 0) {
            const Me = -ke.left, { position: Re } = pe[Qe];
            pe[Qe].position = { ...Re, sx: Re.sx + Me };
          } else if (ke.right > Se.width) {
            const Me = ke.right - Se.width, { position: Re } = pe[Qe];
            pe[Qe].position = { ...Re, sx: Re.sx - Me };
          }
        }, Ee = (Qe) => {
          const ke = ye[Qe];
          if (ke.top < 0) {
            const Me = -ke.top, { position: Re } = pe[Qe];
            pe[Qe].position = { ...Re, sy: Re.sy + Me };
          } else if (ke.bottom > Se.height) {
            const Me = ke.bottom - Se.height, { position: Re } = pe[Qe];
            pe[Qe].position = { ...Re, sy: Re.sy - Me };
          }
        }, xe = Ce[0], Le = Ce[Ce.length - 1];
        this.dimension == 0 ? (we(xe), we(Le)) : (Ee(xe), Ee(Le));
      }
      for (const Se of be)
        pe[Se].paint(H);
    }
    // extents sub functions -----------------------------------------------------
    /*protected*/
    _tick_extent() {
      const { major: H, minor: V } = this.tick_coords, q = this.dimension;
      return Math.max(H[q].length == 0 ? 0 : this.model.major_tick_out, V[q].length == 0 ? 0 : this.model.minor_tick_out);
    }
    _tick_label_extents() {
      const H = this.tick_coords.major, V = this.compute_labels(H[this.dimension]), q = this.model.major_label_orientation, G = this.model.major_label_standoff, J = this.visuals.major_label_text;
      return [this._oriented_labels_extent(V, q, G, J)];
    }
    get extents() {
      const H = this._tick_label_extents();
      return {
        tick: this._tick_extent(),
        tick_labels: H,
        tick_label: sum$3(H),
        axis_label: this._axis_label_extent()
      };
    }
    _oriented_labels_extent(H, V, q, G) {
      if (H.length == 0 || !G.doit)
        return 0;
      const J = this.panel.get_label_angle_heuristic(V);
      H.visuals = G.values(), H.angle = J, H.base_font_size = this.plot_view.base_font_size;
      const Z = H.max_size(), te = this.dimension == 0 ? Z.height : Z.width;
      return te > 0 ? q + te + 3 : 0;
    }
    // {{{ TODO: state
    get normals() {
      return this.panel.normals;
    }
    get dimension() {
      return this.panel.dimension;
    }
    compute_labels(H) {
      const V = this.model.formatter.format_graphics(H, this), { _major_label_views: q } = this, G = /* @__PURE__ */ new Set();
      for (let J = 0; J < H.length; J++) {
        const Z = q.get(H[J]);
        Z != null && (G.add(Z), V[J] = Z.graphics());
      }
      for (const J of this._major_label_views.values())
        G.has(J) || (J._has_finished = !0);
      return new GraphicsBoxes(V);
    }
    scoords(H) {
      const [V, q] = H, [G, J] = this.coordinates.map_to_screen(V, q);
      if (this.model.fixed_location != null)
        return [[...G], [...J]];
      {
        const { bbox: Z } = this, { face: te } = this.panel;
        if (this.panel.is_vertical) {
          const ne = te == "front" ? Z.right : Z.left;
          return [repeat$1(ne, G.length), [...J]];
        } else {
          const ne = te == "front" ? Z.bottom : Z.top;
          return [[...G], repeat$1(ne, J.length)];
        }
      }
    }
    // TODO Remove this.
    get offsets() {
      return [0, 0];
    }
    get ranges() {
      const H = this.dimension, V = 1 - H, { ranges: q } = this.coordinates;
      return [q[H], q[V]];
    }
    get computed_bounds() {
      const [H] = this.ranges, V = this.model.bounds, q = [H.min, H.max];
      if (V == "auto")
        return [H.min, H.max];
      {
        let G, J;
        const [Z, te] = V, [ne, ee] = q, { min: ae, max: oe } = Math;
        return abs$b(Z - te) > abs$b(ne - ee) ? (G = oe(ae(Z, te), ne), J = ae(oe(Z, te), ee)) : (G = ae(Z, te), J = oe(Z, te)), [G, J];
      }
    }
    get rule_coords() {
      const H = this.dimension, V = 1 - H, [q] = this.ranges, [G, J] = this.computed_bounds, Z = new Array(2), te = new Array(2), ne = [Z, te];
      return ne[H][0] = Math.max(G, q.min), ne[H][1] = Math.min(J, q.max), ne[H][0] > ne[H][1] && (ne[H][0] = ne[H][1] = NaN), ne[V][0] = this.loc, ne[V][1] = this.loc, ne;
    }
    get rule_scoords() {
      const [[H, V], [q, G]] = this.scoords(this.rule_coords);
      return {
        sx0: Math.round(H),
        sy0: Math.round(q),
        sx1: Math.round(V),
        sy1: Math.round(G)
      };
    }
    get tick_coords() {
      const H = this.dimension, V = 1 - H, [q] = this.ranges, [G, J] = this.computed_bounds, Z = this.model.ticker.get_ticks(G, J, q, this.loc), te = Z.major, ne = Z.minor, oe = [[], []], le = [[], []], [ue, de] = [q.min, q.max];
      for (let fe = 0; fe < te.length; fe++)
        te[fe] < ue || te[fe] > de || (oe[H].push(te[fe]), oe[V].push(this.loc));
      for (let fe = 0; fe < ne.length; fe++)
        ne[fe] < ue || ne[fe] > de || (le[H].push(ne[fe]), le[V].push(this.loc));
      return {
        major: oe,
        minor: le
      };
    }
    get loc() {
      const { fixed_location: H } = this.model;
      if (H != null) {
        if (isNumber$3(H))
          return H;
        const [, q] = this.ranges;
        if (q instanceof FactorRange)
          return q.synthetic(H);
        unreachable();
      }
      const [, V] = this.ranges;
      switch (this.panel.side) {
        case "left":
        case "below":
          return V.start;
        case "right":
        case "above":
          return V.end;
      }
    }
    get face() {
      return this.panel.face;
    }
    // }}}
    serializable_state() {
      return {
        ...super.serializable_state(),
        bbox: this.bbox
      };
    }
    remove() {
      var H;
      (H = this._axis_label_view) == null || H.remove();
      for (const V of this._major_label_views.values())
        V.remove();
      super.remove();
    }
    has_finished() {
      if (!super.has_finished() || this._axis_label_view != null && !this._axis_label_view.has_finished())
        return !1;
      for (const H of this._major_label_views.values())
        if (!H.has_finished())
          return !1;
      return !0;
    }
  }
  mt(AxisView, "__name__", "AxisView");
  const n0 = class n0 extends GuideRenderer {
    constructor(_) {
      super(_);
    }
  };
  mt(n0, "__name__", "Axis"), n0.prototype.default_view = AxisView, n0.mixins([
    ["axis_", Line$4],
    ["major_tick_", Line$4],
    ["minor_tick_", Line$4],
    ["major_label_", Text$4],
    ["axis_label_", Text$4],
    ["background_", Fill$2]
  ]), n0.define(({ Any: _, Int: H, Float: V, Str: q, Ref: G, Tuple: J, Or: Z, Nullable: te, Auto: ne, Enum: ee }) => ({
    dimension: [Z(ee(0, 1), ne), "auto"],
    face: [Z(Face, ne), "auto"],
    bounds: [Z(J(V, V), ne), "auto"],
    ticker: [G(Ticker)],
    formatter: [G(TickFormatter)],
    axis_label: [te(Z(q, G(BaseText))), null],
    axis_label_standoff: [H, 5],
    axis_label_orientation: [Z(LabelOrientation, V), "parallel"],
    axis_label_align: [Align, "center"],
    major_label_standoff: [H, 5],
    major_label_orientation: [Z(LabelOrientation, V), "horizontal"],
    major_label_overrides: [LabelOverrides, /* @__PURE__ */ new Map()],
    major_label_policy: [G(LabelingPolicy), () => new AllLabels()],
    major_tick_in: [V, 2],
    major_tick_out: [V, 6],
    minor_tick_in: [V, 0],
    minor_tick_out: [V, 4],
    fixed_location: [te(Z(V, _)), null]
  })), n0.override({
    axis_line_color: "black",
    major_tick_line_color: "black",
    minor_tick_line_color: "black",
    major_label_text_font_size: "11px",
    major_label_text_align: "center",
    // XXX: remove
    major_label_text_baseline: "alphabetic",
    // XXX: remove
    axis_label_text_font_size: "13px",
    axis_label_text_font_style: "italic",
    background_fill_color: null
  });
  let Axis = n0;
  class ContinuousAxisView extends AxisView {
  }
  mt(ContinuousAxisView, "__name__", "ContinuousAxisView");
  class ContinuousAxis extends Axis {
    constructor(_) {
      super(_);
    }
  }
  mt(ContinuousAxis, "__name__", "ContinuousAxis");
  function unicode_replace(O) {
    let _ = "";
    for (const H of O)
      H == "-" ? _ += "−" : _ += H;
    return _;
  }
  const Ig = class Ig extends TickFormatter {
    constructor(H) {
      super(H);
      mt(this, "last_precision", 3);
    }
    get scientific_limit_low() {
      return 10 ** this.power_limit_low;
    }
    get scientific_limit_high() {
      return 10 ** this.power_limit_high;
    }
    _need_sci(H) {
      if (!this.use_scientific)
        return !1;
      const { scientific_limit_high: V } = this, { scientific_limit_low: q } = this, G = H.length < 2 ? 0 : Math.abs(H[1] - H[0]) / 1e4;
      for (const J of H) {
        const Z = Math.abs(J);
        if (!(Z <= G) && (Z >= V || Z <= q))
          return !0;
      }
      return !1;
    }
    _format_with_precision(H, V, q) {
      return V ? H.map((G) => unicode_replace(G.toExponential(q))) : H.map((G) => unicode_replace(to_fixed(G, q)));
    }
    _auto_precision(H, V) {
      const q = new Array(H.length), G = this.last_precision <= 15;
      e: for (let J = this.last_precision; G ? J <= 15 : J >= 1; G ? J++ : J--)
        if (V) {
          q[0] = H[0].toExponential(J);
          for (let Z = 1; Z < H.length; Z++)
            if (q[Z] == q[Z - 1])
              continue e;
          this.last_precision = J;
          break;
        } else {
          q[0] = to_fixed(H[0], J);
          for (let Z = 1; Z < H.length; Z++)
            if (q[Z] = to_fixed(H[Z], J), q[Z] == q[Z - 1])
              continue e;
          this.last_precision = J;
          break;
        }
      return this.last_precision;
    }
    doFormat(H, V) {
      if (H.length == 0)
        return [];
      const q = this._need_sci(H), G = this.precision == "auto" ? this._auto_precision(H, q) : this.precision;
      return this._format_with_precision(H, q, G);
    }
  };
  mt(Ig, "__name__", "BasicTickFormatter"), Ig.define(({ Bool: H, Int: V, Auto: q, Or: G }) => ({
    precision: [G(V, q), "auto"],
    use_scientific: [H, !0],
    power_limit_high: [V, 5],
    power_limit_low: [V, -3]
  }));
  let BasicTickFormatter = Ig;
  const Dg = class Dg extends Ticker {
    constructor(_) {
      super(_);
    }
    get_ticks(_, H, V, q) {
      return this.get_ticks_no_defaults(_, H, q, this.desired_num_ticks);
    }
    // The version of get_ticks() that does the work (and the version that
    // should be overridden in subclasses).
    get_ticks_no_defaults(_, H, V, q) {
      const G = this.get_interval(_, H, q), J = Math.floor(_ / G), Z = Math.ceil(H / G);
      let te;
      !isFinite(J) || !isFinite(Z) ? te = [] : te = range$9(J, Z + 1);
      const ne = te.map((oe) => oe * G).filter((oe) => _ <= oe && oe <= H), ee = this.num_minor_ticks, ae = [];
      if (ee > 0 && ne.length > 0) {
        const oe = G / ee, se = range$9(0, ee).map((ce) => ce * oe);
        for (const ce of se.slice(1)) {
          const le = ne[0] - ce;
          _ <= le && le <= H && ae.push(le);
        }
        for (const ce of ne)
          for (const le of se) {
            const ue = ce + le;
            _ <= ue && ue <= H && ae.push(ue);
          }
      }
      return {
        major: ne,
        minor: ae
      };
    }
    // Returns the interval size that would produce exactly the number of
    // desired ticks.  (In general we won't use exactly this interval, because
    // we want the ticks to be round numbers.)
    get_ideal_interval(_, H, V) {
      return (H - _) / V;
    }
  };
  mt(Dg, "__name__", "ContinuousTicker"), Dg.define(({ Int: _ }) => ({
    num_minor_ticks: [_, 5],
    desired_num_ticks: [_, 6]
  }));
  let ContinuousTicker = Dg;
  const Ng = class Ng extends ContinuousTicker {
    constructor(_) {
      super(_);
    }
    get_min_interval() {
      return this.min_interval;
    }
    get_max_interval() {
      return this.max_interval ?? 1 / 0;
    }
    // These arguments control the range of possible intervals. The interval I
    // returned by get_interval() will be the one that most closely matches the
    // desired number of ticks, subject to the following constraints:
    // I = (M * B^N), where
    // M is a member of mantissas,
    // B is base,
    // and N is an integer;
    // and min_interval <= I <= max_interval.
    get extended_mantissas() {
      const _ = nth(this.mantissas, -1) / this.base, H = nth(this.mantissas, 0) * this.base;
      return [_, ...this.mantissas, H];
    }
    get base_factor() {
      return this.get_min_interval() == 0 ? 1 : this.get_min_interval();
    }
    get_interval(_, H, V) {
      const q = H - _, G = this.get_ideal_interval(_, H, V), J = Math.floor(log$7(G / this.base_factor, this.base)), Z = this.base ** J * this.base_factor, te = this.extended_mantissas, ne = te.map((oe) => Math.abs(V - q / (oe * Z))), ae = te[argmin(ne)] * Z;
      return clamp$1(ae, this.get_min_interval(), this.get_max_interval());
    }
  };
  mt(Ng, "__name__", "AdaptiveTicker"), Ng.define(({ Float: _, List: H, Nullable: V }) => ({
    base: [_, 10],
    mantissas: [H(_), [1, 2, 5]],
    min_interval: [_, 0],
    max_interval: [V(_), null]
  }));
  let AdaptiveTicker = Ng;
  class BasicTicker extends AdaptiveTicker {
    constructor(_) {
      super(_);
    }
  }
  mt(BasicTicker, "__name__", "BasicTicker");
  class LinearAxisView extends ContinuousAxisView {
  }
  mt(LinearAxisView, "__name__", "LinearAxisView");
  const $3 = class $3 extends ContinuousAxis {
    constructor(_) {
      super(_);
    }
  };
  mt($3, "__name__", "LinearAxis"), $3.prototype.default_view = LinearAxisView, $3.override({
    ticker: () => new BasicTicker(),
    formatter: () => new BasicTickFormatter()
  });
  let LinearAxis = $3;
  class CategoricalTicker extends Ticker {
    constructor(_) {
      super(_);
    }
    get_ticks(_, H, V, q) {
      const G = this._collect(V.factors, V, _, H), J = this._collect(V.tops ?? [], V, _, H), Z = this._collect(V.mids ?? [], V, _, H);
      return {
        major: G,
        minor: [],
        tops: J,
        mids: Z
      };
    }
    _collect(_, H, V, q) {
      const G = [];
      for (const J of _) {
        const Z = H.synthetic(J);
        Z > V && Z < q && G.push(J);
      }
      return G;
    }
  }
  mt(CategoricalTicker, "__name__", "CategoricalTicker");
  const Pg = class Pg extends ContinuousTicker {
    constructor(_) {
      super(_);
    }
    // The tickers should be in order of increasing interval size; specifically,
    // if S comes before T, then it should be the case that
    // S.get_max_interval() < T.get_min_interval().
    // FIXME Enforce this automatically.
    get min_intervals() {
      return this.tickers.map((_) => _.get_min_interval());
    }
    get max_intervals() {
      return this.tickers.map((_) => _.get_max_interval());
    }
    get_min_interval() {
      return this.min_intervals[0];
    }
    get_max_interval() {
      return this.max_intervals[0];
    }
    get_best_ticker(_, H, V) {
      const q = H - _, G = this.get_ideal_interval(_, H, V), J = [
        sorted_index(this.min_intervals, G) - 1,
        sorted_index(this.max_intervals, G)
      ], te = [
        this.min_intervals[J[0]],
        this.max_intervals[J[1]]
      ].map((ee) => Math.abs(V - q / ee));
      let ne;
      if (is_empty$1(te.filter((ee) => !isNaN(ee))))
        ne = this.tickers[0];
      else {
        const ee = argmin(te), ae = J[ee];
        ne = this.tickers[ae];
      }
      return ne;
    }
    get_interval(_, H, V) {
      return this.get_best_ticker(_, H, V).get_interval(_, H, V);
    }
    get_ticks_no_defaults(_, H, V, q) {
      return this.get_best_ticker(_, H, q).get_ticks_no_defaults(_, H, V, q);
    }
  };
  mt(Pg, "__name__", "CompositeTicker"), Pg.define(({ List: _, Ref: H }) => ({
    tickers: [_(H(ContinuousTicker)), []]
  }));
  let CompositeTicker = Pg;
  class BaseSingleIntervalTicker extends ContinuousTicker {
    constructor(_) {
      super(_);
    }
    get_interval(_, H, V) {
      return this.interval;
    }
    get_min_interval() {
      return this.interval;
    }
    get_max_interval() {
      return this.interval;
    }
  }
  mt(BaseSingleIntervalTicker, "__name__", "BaseSingleIntervalTicker");
  const $g = class $g extends BaseSingleIntervalTicker {
    constructor(H) {
      super(H);
      mt(this, "interval");
    }
  };
  mt($g, "__name__", "SingleIntervalTicker"), $g.define(({ Float: H }) => ({
    interval: [H]
  }));
  let SingleIntervalTicker = $g;
  const ONE_MILLI = 1, ONE_SECOND = 1e3, ONE_MINUTE = 60 * ONE_SECOND, ONE_HOUR = 60 * ONE_MINUTE, ONE_DAY = 24 * ONE_HOUR, ONE_MONTH = 30 * ONE_DAY, ONE_YEAR = 365 * ONE_DAY;
  function copy_date(O) {
    return new Date(O.getTime());
  }
  function last_month_no_later_than(O) {
    const _ = copy_date(O);
    return _.setUTCDate(1), _.setUTCHours(0), _.setUTCMinutes(0), _.setUTCSeconds(0), _.setUTCMilliseconds(0), _;
  }
  function last_year_no_later_than(O) {
    const _ = last_month_no_later_than(O);
    return _.setUTCMonth(0), _;
  }
  function date_range_by_month(O, _) {
    const H = last_month_no_later_than(new Date(O)), V = last_month_no_later_than(new Date(_));
    V.setUTCMonth(V.getUTCMonth() + 1);
    const q = [], G = H;
    for (; q.push(copy_date(G)), G.setUTCMonth(G.getUTCMonth() + 1), !(G > V); )
      ;
    return q;
  }
  const F3 = class F3 extends BaseSingleIntervalTicker {
    constructor(H) {
      super(H);
      mt(this, "interval");
    }
    initialize() {
      super.initialize();
      const H = this.days;
      H.length > 1 ? this.interval = (H[1] - H[0]) * ONE_DAY : this.interval = 31 * ONE_DAY;
    }
    get_ticks_no_defaults(H, V, q, G) {
      const J = date_range_by_month(H, V), Z = this.days, te = (oe, se) => {
        const ce = oe.getUTCMonth(), le = [];
        for (const ue of Z) {
          const de = copy_date(oe);
          de.setUTCDate(ue), new Date(de.getTime() + se / 2).getUTCMonth() == ce && le.push(de);
        }
        return le;
      };
      return {
        major: concat$1(J.map((oe) => te(oe, this.interval))).map((oe) => oe.getTime()).filter((oe) => H <= oe && oe <= V),
        minor: []
      };
    }
  };
  mt(F3, "__name__", "DaysTicker"), F3.define(({ Int: H, List: V }) => ({
    days: [V(H), []]
  })), F3.override({
    num_minor_ticks: 0
  });
  let DaysTicker = F3;
  function date_range_by_year(O, _) {
    const H = last_year_no_later_than(new Date(O)), V = last_year_no_later_than(new Date(_));
    V.setUTCFullYear(V.getUTCFullYear() + 1);
    const q = [], G = H;
    for (; q.push(copy_date(G)), G.setUTCFullYear(G.getUTCFullYear() + 1), !(G > V); )
      ;
    return q;
  }
  const Fg = class Fg extends BaseSingleIntervalTicker {
    constructor(H) {
      super(H);
      mt(this, "interval");
    }
    initialize() {
      super.initialize();
      const H = this.months;
      H.length > 1 ? this.interval = (H[1] - H[0]) * ONE_MONTH : this.interval = 12 * ONE_MONTH;
    }
    get_ticks_no_defaults(H, V, q, G) {
      const J = date_range_by_year(H, V), Z = this.months, te = (oe) => Z.map((se) => {
        const ce = copy_date(oe);
        return ce.setUTCMonth(se), ce;
      });
      return {
        major: concat$1(J.map(te)).map((oe) => oe.getTime()).filter((oe) => H <= oe && oe <= V),
        minor: []
      };
    }
  };
  mt(Fg, "__name__", "MonthsTicker"), Fg.define(({ Int: H, List: V }) => ({
    months: [V(H), []]
  }));
  let MonthsTicker = Fg;
  class YearsTicker extends BaseSingleIntervalTicker {
    constructor(H) {
      super(H);
      mt(this, "interval", ONE_YEAR);
      mt(this, "basic_ticker", new BasicTicker({ num_minor_ticks: 0 }));
    }
    get_ticks_no_defaults(H, V, q, G) {
      const J = last_year_no_later_than(new Date(H)).getUTCFullYear(), Z = last_year_no_later_than(new Date(V)).getUTCFullYear();
      return {
        major: this.basic_ticker.get_ticks_no_defaults(J, Z, q, G).major.map((ae) => Date.UTC(ae, 0, 1)).filter((ae) => H <= ae && ae <= V),
        minor: []
      };
    }
  }
  mt(YearsTicker, "__name__", "YearsTicker");
  const Bg = class Bg extends CompositeTicker {
    constructor(_) {
      super(_);
    }
  };
  mt(Bg, "__name__", "DatetimeTicker"), Bg.override({
    num_minor_ticks: 0,
    tickers: () => [
      // Sub-second.
      new AdaptiveTicker({
        mantissas: [1, 2, 5],
        base: 10,
        min_interval: 0,
        max_interval: 500 * ONE_MILLI,
        num_minor_ticks: 0
      }),
      // Seconds, minutes.
      new AdaptiveTicker({
        mantissas: [1, 2, 5, 10, 15, 20, 30],
        base: 60,
        min_interval: ONE_SECOND,
        max_interval: 30 * ONE_MINUTE,
        num_minor_ticks: 0
      }),
      // Hours.
      new AdaptiveTicker({
        mantissas: [1, 2, 4, 6, 8, 12],
        base: 24,
        min_interval: ONE_HOUR,
        max_interval: 12 * ONE_HOUR,
        num_minor_ticks: 0
      }),
      // Days.
      new DaysTicker({ days: range$9(1, 32) }),
      new DaysTicker({ days: range$9(1, 31, 3) }),
      new DaysTicker({ days: [1, 8, 15, 22] }),
      new DaysTicker({ days: [1, 15] }),
      // Months.
      new MonthsTicker({ months: range$9(0, 12, 1) }),
      new MonthsTicker({ months: range$9(0, 12, 2) }),
      new MonthsTicker({ months: range$9(0, 12, 4) }),
      new MonthsTicker({ months: range$9(0, 12, 6) }),
      // Years
      new YearsTicker({})
    ]
  });
  let DatetimeTicker = Bg;
  const Hg = class Hg extends ContinuousTicker {
    constructor(_) {
      super(_);
    }
    get_ticks_no_defaults(_, H, V, q) {
      return {
        major: [...this.ticks],
        minor: [...this.minor_ticks]
      };
    }
    // XXX: whatever, because FixedTicker needs to fulfill the interface somehow
    get_interval(_, H, V) {
      return 0;
    }
    get_min_interval() {
      return 0;
    }
    get_max_interval() {
      return 0;
    }
  };
  mt(Hg, "__name__", "FixedTicker"), Hg.define(({ Float: _, Arrayable: H }) => ({
    ticks: [H(_), []],
    minor_ticks: [H(_), []]
  }));
  let FixedTicker = Hg;
  const zg = class zg extends AdaptiveTicker {
    constructor(_) {
      super(_);
    }
    get_ticks_no_defaults(_, H, V, q) {
      const G = this.num_minor_ticks, J = [], Z = this.base, te = Math.log(_) / Math.log(Z), ne = Math.log(H) / Math.log(Z), ee = ne - te;
      let ae;
      if (!isFinite(ee) || ee == 0)
        ae = [];
      else if (ee < 2) {
        const oe = this.get_interval(_, H, q), se = Math.floor(_ / oe), ce = Math.ceil(H / oe);
        if (ae = range$9(se, ce + 1).filter((le) => le != 0).map((le) => le * oe).filter((le) => _ <= le && le <= H), G > 0 && ae.length > 0) {
          const le = oe / G, ue = range$9(0, G).map((de) => de * le);
          for (const de of ue.slice(1))
            J.push(ae[0] - de);
          for (const de of ae)
            for (const fe of ue)
              J.push(de + fe);
        }
      } else {
        const oe = Math.ceil(te * 0.999999), se = Math.floor(ne * 1.000001), ce = Math.ceil((se - oe) / 9);
        if (ae = range$9(oe - 1, se + 1, ce).map((le) => Z ** le), G > 0 && ae.length > 0) {
          const le = Z ** ce / G, ue = range$9(1, G + 1).map((de) => de * le);
          for (const de of ue)
            J.push(ae[0] / de);
          J.push(ae[0]);
          for (const de of ae)
            for (const fe of ue)
              J.push(de * fe);
        }
      }
      return {
        major: ae.filter((oe) => _ <= oe && oe <= H),
        minor: J.filter((oe) => _ <= oe && oe <= H)
      };
    }
  };
  mt(zg, "__name__", "LogTicker"), zg.override({
    mantissas: [1, 5]
  });
  let LogTicker = zg;
  const Vg = class Vg extends BasicTicker {
    constructor(_) {
      super(_);
    }
    get_ticks_no_defaults(_, H, V, q) {
      if (this.dimension == null)
        throw new Error(`${this}.dimension wasn't configured`);
      return [_, H] = clip_mercator(_, H, this.dimension), this.dimension == "lon" ? this._get_ticks_lon(_, H, V, q) : this._get_ticks_lat(_, H, V, q);
    }
    _get_ticks_lon(_, H, V, q) {
      const [G] = wgs84_mercator.invert(_, V), [J, Z] = wgs84_mercator.invert(H, V), te = super.get_ticks_no_defaults(G, J, V, q), ne = [];
      for (const ae of te.major)
        if (in_bounds(ae, "lon")) {
          const [oe] = wgs84_mercator.compute(ae, Z);
          ne.push(oe);
        }
      const ee = [];
      for (const ae of te.minor)
        if (in_bounds(ae, "lon")) {
          const [oe] = wgs84_mercator.compute(ae, Z);
          ee.push(oe);
        }
      return { major: ne, minor: ee };
    }
    _get_ticks_lat(_, H, V, q) {
      const [, G] = wgs84_mercator.invert(V, _), [J, Z] = wgs84_mercator.invert(V, H), te = super.get_ticks_no_defaults(G, Z, V, q), ne = [];
      for (const ae of te.major)
        if (in_bounds(ae, "lat")) {
          const [, oe] = wgs84_mercator.compute(J, ae);
          ne.push(oe);
        }
      const ee = [];
      for (const ae of te.minor)
        if (in_bounds(ae, "lat")) {
          const [, oe] = wgs84_mercator.compute(J, ae);
          ee.push(oe);
        }
      return { major: ne, minor: ee };
    }
  };
  mt(Vg, "__name__", "MercatorTicker"), Vg.define(({ Nullable: _ }) => ({
    dimension: [_(LatLon), null]
  }));
  let MercatorTicker = Vg;
  class Mapper extends Transform$1 {
    constructor(_) {
      super(_);
    }
    compute(_) {
      throw new Error("mapping single values is not supported");
    }
  }
  mt(Mapper, "__name__", "Mapper");
  function _convert_color(O) {
    return encode_rgba(color2rgba(O));
  }
  function _convert_palette(O) {
    const _ = new Uint32Array(O.length);
    for (let H = 0, V = O.length; H < V; H++)
      _[H] = _convert_color(O[H]);
    return _;
  }
  const Ug = class Ug extends Mapper {
    constructor(H) {
      super(H);
      mt(this, "metrics_change");
    }
    initialize() {
      super.initialize(), this.metrics_change = new Signal0(this, "metrics_change");
    }
    v_compute(H) {
      const V = new Array(H.length);
      return this._v_compute(H, V, this.palette, this._colors((q) => q)), V;
    }
    get rgba_mapper() {
      const H = this, V = _convert_palette(this.palette), q = this._colors(_convert_color);
      return {
        v_compute(G) {
          const J = is_NDArray(G) && G.dimension == 3 ? G.shape[2] : 1, Z = new ColorArray(G.length / J);
          return H._v_compute_uint32(G, Z, V, q), new Uint8ClampedArray(to_big_endian(Z).buffer);
        }
      };
    }
    _colors(H) {
      return { nan_color: H(this.nan_color) };
    }
    _v_compute_uint32(H, V, q, G) {
      this._v_compute(H, V, q, G);
    }
  };
  mt(Ug, "__name__", "ColorMapper"), Ug.define(({ Color: H, List: V }) => ({
    palette: [V(H)],
    nan_color: [H, "gray"]
  }));
  let ColorMapper = Ug;
  var Uy, qy;
  class DataRendererView extends (qy = RendererView, Uy = auto_ranged, qy) {
    constructor() {
      super(...arguments);
      mt(this, Uy, !0);
    }
    get xscale() {
      return this.coordinates.x_scale;
    }
    get yscale() {
      return this.coordinates.y_scale;
    }
    bounds() {
      return this.glyph_view.bounds();
    }
    log_bounds() {
      return this.glyph_view.log_bounds();
    }
  }
  mt(DataRendererView, "__name__", "DataRendererView");
  const qg = class qg extends Renderer$1 {
    constructor(_) {
      super(_);
    }
    get selection_manager() {
      return this.get_selection_manager();
    }
  };
  mt(qg, "__name__", "DataRenderer"), qg.override({
    level: "glyph"
  });
  let DataRenderer = qg;
  var jy;
  const bp = class bp {
    constructor(_, H) {
      mt(this, "offsets");
      mt(this, "data");
      this.offsets = _, this.data = H;
    }
    [(jy = Symbol.toStringTag, equals)](_, H) {
      return H.arrays(this.offsets, _.offsets) && H.arrays(this.data, _.data);
    }
    get length() {
      return this.offsets.length;
    }
    clone() {
      return new bp(this.offsets.slice(), this.data.slice());
    }
    static from(_, H) {
      const V = _.length;
      let q = 0;
      const G = (() => {
        const Z = new Uint32Array(V);
        for (let te = 0; te < V; te++) {
          const ne = _[te].length;
          Z[te] = q, q += ne;
        }
        return q < 256 ? new Uint8Array(Z) : q < 65536 ? new Uint16Array(Z) : Z;
      })(), J = new H(q);
      for (let Z = 0; Z < V; Z++)
        J.set(_[Z], G[Z]);
      return new bp(G, J);
    }
    *[Symbol.iterator]() {
      const { offsets: _, length: H } = this;
      for (let V = 0; V < H; V++)
        yield this.data.subarray(_[V], _[V + 1]);
    }
    _check_bounds(_) {
      assert$2(0 <= _ && _ < this.length, `Out of bounds: 0 <= ${_} < ${this.length}`);
    }
    get(_) {
      this._check_bounds(_);
      const { offsets: H } = this;
      return this.data.subarray(H[_], H[_ + 1]);
    }
    set(_, H) {
      this._check_bounds(_), this.data.set(H, this.offsets[_]);
    }
  };
  mt(bp, "__name__", "RaggedArray"), mt(bp, jy, "RaggedArray");
  let RaggedArray = bp;
  class FlatQueue {
    constructor() {
      this.ids = [], this.values = [], this.length = 0;
    }
    clear() {
      this.length = 0;
    }
    push(_, H) {
      let V = this.length++;
      for (; V > 0; ) {
        const q = V - 1 >> 1, G = this.values[q];
        if (H >= G) break;
        this.ids[V] = this.ids[q], this.values[V] = G, V = q;
      }
      this.ids[V] = _, this.values[V] = H;
    }
    pop() {
      if (this.length === 0) return;
      const _ = this.ids[0];
      if (this.length--, this.length > 0) {
        const H = this.ids[0] = this.ids[this.length], V = this.values[0] = this.values[this.length], q = this.length >> 1;
        let G = 0;
        for (; G < q; ) {
          let J = (G << 1) + 1;
          const Z = J + 1;
          let te = this.ids[J], ne = this.values[J];
          const ee = this.values[Z];
          if (Z < this.length && ee < ne && (J = Z, te = this.ids[Z], ne = ee), ne >= V) break;
          this.ids[G] = te, this.values[G] = ne, G = J;
        }
        this.ids[G] = H, this.values[G] = V;
      }
      return _;
    }
    peek() {
      if (this.length !== 0)
        return this.ids[0];
    }
    peekValue() {
      if (this.length !== 0)
        return this.values[0];
    }
    shrink() {
      this.ids.length = this.values.length = this.length;
    }
  }
  const ARRAY_TYPES = [Int8Array, Uint8Array, Uint8ClampedArray, Int16Array, Uint16Array, Int32Array, Uint32Array, Float32Array, Float64Array], VERSION$1 = 3;
  class Flatbush {
    /**
     * Recreate a Flatbush index from raw `ArrayBuffer` or `SharedArrayBuffer` data.
     * @param {ArrayBuffer | SharedArrayBuffer} data
     * @param {number} [byteOffset=0] byte offset to the start of the Flatbush buffer in the referenced ArrayBuffer.
     * @returns {Flatbush} index
     */
    static from(_, H = 0) {
      if (H % 8 !== 0)
        throw new Error("byteOffset must be 8-byte aligned.");
      if (!_ || _.byteLength === void 0 || _.buffer)
        throw new Error("Data must be an instance of ArrayBuffer or SharedArrayBuffer.");
      const [V, q] = new Uint8Array(_, H + 0, 2);
      if (V !== 251)
        throw new Error("Data does not appear to be in a Flatbush format.");
      const G = q >> 4;
      if (G !== VERSION$1)
        throw new Error(`Got v${G} data when expected v${VERSION$1}.`);
      const J = ARRAY_TYPES[q & 15];
      if (!J)
        throw new Error("Unrecognized array type.");
      const [Z] = new Uint16Array(_, H + 2, 1), [te] = new Uint32Array(_, H + 4, 1);
      return new Flatbush(te, Z, J, void 0, _, H);
    }
    /**
     * Create a Flatbush index that will hold a given number of items.
     * @param {number} numItems
     * @param {number} [nodeSize=16] Size of the tree node (16 by default).
     * @param {TypedArrayConstructor} [ArrayType=Float64Array] The array type used for coordinates storage (`Float64Array` by default).
     * @param {ArrayBufferConstructor | SharedArrayBufferConstructor} [ArrayBufferType=ArrayBuffer] The array buffer type used to store data (`ArrayBuffer` by default).
     * @param {ArrayBuffer | SharedArrayBuffer} [data] (Only used internally)
     * @param {number} [byteOffset=0] (Only used internally)
     */
    constructor(_, H = 16, V = Float64Array, q = ArrayBuffer, G, J = 0) {
      if (_ === void 0) throw new Error("Missing required argument: numItems.");
      if (isNaN(_) || _ <= 0) throw new Error(`Unexpected numItems value: ${_}.`);
      this.numItems = +_, this.nodeSize = Math.min(Math.max(+H, 2), 65535), this.byteOffset = J;
      let Z = _, te = Z;
      this._levelBounds = [Z * 4];
      do
        Z = Math.ceil(Z / this.nodeSize), te += Z, this._levelBounds.push(te * 4);
      while (Z !== 1);
      this.ArrayType = V, this.IndexArrayType = te < 16384 ? Uint16Array : Uint32Array;
      const ne = ARRAY_TYPES.indexOf(this.ArrayType), ee = te * 4 * this.ArrayType.BYTES_PER_ELEMENT;
      if (ne < 0)
        throw new Error(`Unexpected typed array class: ${V}.`);
      G && G.byteLength !== void 0 && !G.buffer ? (this.data = G, this._boxes = new this.ArrayType(this.data, J + 8, te * 4), this._indices = new this.IndexArrayType(this.data, J + 8 + ee, te), this._pos = te * 4, this.minX = this._boxes[this._pos - 4], this.minY = this._boxes[this._pos - 3], this.maxX = this._boxes[this._pos - 2], this.maxY = this._boxes[this._pos - 1]) : (this.data = new q(8 + ee + te * this.IndexArrayType.BYTES_PER_ELEMENT), this._boxes = new this.ArrayType(this.data, 8, te * 4), this._indices = new this.IndexArrayType(this.data, 8 + ee, te), this._pos = 0, this.minX = 1 / 0, this.minY = 1 / 0, this.maxX = -1 / 0, this.maxY = -1 / 0, new Uint8Array(this.data, 0, 2).set([251, (VERSION$1 << 4) + ne]), new Uint16Array(this.data, 2, 1)[0] = H, new Uint32Array(this.data, 4, 1)[0] = _), this._queue = new FlatQueue();
    }
    /**
     * Add a given rectangle to the index.
     * @param {number} minX
     * @param {number} minY
     * @param {number} maxX
     * @param {number} maxY
     * @returns {number} A zero-based, incremental number that represents the newly added rectangle.
     */
    add(_, H, V = _, q = H) {
      const G = this._pos >> 2, J = this._boxes;
      return this._indices[G] = G, J[this._pos++] = _, J[this._pos++] = H, J[this._pos++] = V, J[this._pos++] = q, _ < this.minX && (this.minX = _), H < this.minY && (this.minY = H), V > this.maxX && (this.maxX = V), q > this.maxY && (this.maxY = q), G;
    }
    /** Perform indexing of the added rectangles. */
    finish() {
      if (this._pos >> 2 !== this.numItems)
        throw new Error(`Added ${this._pos >> 2} items when expected ${this.numItems}.`);
      const _ = this._boxes;
      if (this.numItems <= this.nodeSize) {
        _[this._pos++] = this.minX, _[this._pos++] = this.minY, _[this._pos++] = this.maxX, _[this._pos++] = this.maxY;
        return;
      }
      const H = this.maxX - this.minX || 1, V = this.maxY - this.minY || 1, q = new Uint32Array(this.numItems), G = 65535;
      for (let J = 0, Z = 0; J < this.numItems; J++) {
        const te = _[Z++], ne = _[Z++], ee = _[Z++], ae = _[Z++], oe = Math.floor(G * ((te + ee) / 2 - this.minX) / H), se = Math.floor(G * ((ne + ae) / 2 - this.minY) / V);
        q[J] = hilbert(oe, se);
      }
      sort$1(q, _, this._indices, 0, this.numItems - 1, this.nodeSize);
      for (let J = 0, Z = 0; J < this._levelBounds.length - 1; J++) {
        const te = this._levelBounds[J];
        for (; Z < te; ) {
          const ne = Z;
          let ee = _[Z++], ae = _[Z++], oe = _[Z++], se = _[Z++];
          for (let ce = 1; ce < this.nodeSize && Z < te; ce++)
            ee = Math.min(ee, _[Z++]), ae = Math.min(ae, _[Z++]), oe = Math.max(oe, _[Z++]), se = Math.max(se, _[Z++]);
          this._indices[this._pos >> 2] = ne, _[this._pos++] = ee, _[this._pos++] = ae, _[this._pos++] = oe, _[this._pos++] = se;
        }
      }
    }
    /**
     * Search the index by a bounding box.
     * @param {number} minX
     * @param {number} minY
     * @param {number} maxX
     * @param {number} maxY
     * @param {(index: number) => boolean} [filterFn] An optional function for filtering the results.
     * @returns {number[]} An array of indices of items intersecting or touching the given bounding box.
     */
    search(_, H, V, q, G) {
      if (this._pos !== this._boxes.length)
        throw new Error("Data not yet indexed - call index.finish().");
      let J = this._boxes.length - 4;
      const Z = [], te = [];
      for (; J !== void 0; ) {
        const ne = Math.min(J + this.nodeSize * 4, upperBound$1(J, this._levelBounds));
        for (let ee = J; ee < ne; ee += 4) {
          if (V < this._boxes[ee] || q < this._boxes[ee + 1] || _ > this._boxes[ee + 2] || H > this._boxes[ee + 3]) continue;
          const ae = this._indices[ee >> 2] | 0;
          J >= this.numItems * 4 ? Z.push(ae) : (G === void 0 || G(ae)) && te.push(ae);
        }
        J = Z.pop();
      }
      return te;
    }
    /**
     * Search items in order of distance from the given point.
     * @param {number} x
     * @param {number} y
     * @param {number} [maxResults=Infinity]
     * @param {number} [maxDistance=Infinity]
     * @param {(index: number) => boolean} [filterFn] An optional function for filtering the results.
     * @returns {number[]} An array of indices of items found.
     */
    neighbors(_, H, V = 1 / 0, q = 1 / 0, G) {
      if (this._pos !== this._boxes.length)
        throw new Error("Data not yet indexed - call index.finish().");
      let J = this._boxes.length - 4;
      const Z = this._queue, te = [], ne = q * q;
      e: for (; J !== void 0; ) {
        const ee = Math.min(J + this.nodeSize * 4, upperBound$1(J, this._levelBounds));
        for (let ae = J; ae < ee; ae += 4) {
          const oe = this._indices[ae >> 2] | 0, se = axisDist(_, this._boxes[ae], this._boxes[ae + 2]), ce = axisDist(H, this._boxes[ae + 1], this._boxes[ae + 3]), le = se * se + ce * ce;
          le > ne || (J >= this.numItems * 4 ? Z.push(oe << 1, le) : (G === void 0 || G(oe)) && Z.push((oe << 1) + 1, le));
        }
        for (; Z.length && Z.peek() & 1; )
          if (Z.peekValue() > ne || (te.push(Z.pop() >> 1), te.length === V)) break e;
        J = Z.length ? Z.pop() >> 1 : void 0;
      }
      return Z.clear(), te;
    }
  }
  function axisDist(O, _, H) {
    return O < _ ? _ - O : O <= H ? 0 : O - H;
  }
  function upperBound$1(O, _) {
    let H = 0, V = _.length - 1;
    for (; H < V; ) {
      const q = H + V >> 1;
      _[q] > O ? V = q : H = q + 1;
    }
    return _[H];
  }
  function sort$1(O, _, H, V, q, G) {
    if (Math.floor(V / G) >= Math.floor(q / G)) return;
    const J = O[V + q >> 1];
    let Z = V - 1, te = q + 1;
    for (; ; ) {
      do
        Z++;
      while (O[Z] < J);
      do
        te--;
      while (O[te] > J);
      if (Z >= te) break;
      swap$2(O, _, H, Z, te);
    }
    sort$1(O, _, H, V, te, G), sort$1(O, _, H, te + 1, q, G);
  }
  function swap$2(O, _, H, V, q) {
    const G = O[V];
    O[V] = O[q], O[q] = G;
    const J = 4 * V, Z = 4 * q, te = _[J], ne = _[J + 1], ee = _[J + 2], ae = _[J + 3];
    _[J] = _[Z], _[J + 1] = _[Z + 1], _[J + 2] = _[Z + 2], _[J + 3] = _[Z + 3], _[Z] = te, _[Z + 1] = ne, _[Z + 2] = ee, _[Z + 3] = ae;
    const oe = H[V];
    H[V] = H[q], H[q] = oe;
  }
  function hilbert(O, _) {
    let H = O ^ _, V = 65535 ^ H, q = 65535 ^ (O | _), G = O & (_ ^ 65535), J = H | V >> 1, Z = H >> 1 ^ H, te = q >> 1 ^ V & G >> 1 ^ q, ne = H & q >> 1 ^ G >> 1 ^ G;
    H = J, V = Z, q = te, G = ne, J = H & H >> 2 ^ V & V >> 2, Z = H & V >> 2 ^ V & (H ^ V) >> 2, te ^= H & q >> 2 ^ V & G >> 2, ne ^= V & q >> 2 ^ (H ^ V) & G >> 2, H = J, V = Z, q = te, G = ne, J = H & H >> 4 ^ V & V >> 4, Z = H & V >> 4 ^ V & (H ^ V) >> 4, te ^= H & q >> 4 ^ V & G >> 4, ne ^= V & q >> 4 ^ (H ^ V) & G >> 4, H = J, V = Z, q = te, G = ne, te ^= H & q >> 8 ^ V & G >> 8, ne ^= V & q >> 8 ^ (H ^ V) & G >> 8, H = te ^ te >> 1, V = ne ^ ne >> 1;
    let ee = O ^ _, ae = V | 65535 ^ (ee | H);
    return ee = (ee | ee << 8) & 16711935, ee = (ee | ee << 4) & 252645135, ee = (ee | ee << 2) & 858993459, ee = (ee | ee << 1) & 1431655765, ae = (ae | ae << 8) & 16711935, ae = (ae | ae << 4) & 252645135, ae = (ae | ae << 2) & 858993459, ae = (ae | ae << 1) & 1431655765, (ae << 1 | ee) >>> 0;
  }
  function upperBound(O, _) {
    let H = 0, V = _.length - 1;
    for (; H < V; ) {
      const q = H + V >> 1;
      _[q] > O ? V = q : H = q + 1;
    }
    return _[H];
  }
  class _FlatBush extends Flatbush {
    get boxes() {
      return this._boxes;
    }
    search_indices(_, H, V, q) {
      if (this._pos !== this._boxes.length)
        throw new Error("Data not yet indexed - call index.finish().");
      let G = this._boxes.length - 4;
      const J = [], Z = new BitSet(this.numItems);
      for (; G !== void 0; ) {
        const te = Math.min(G + this.nodeSize * 4, upperBound(G, this._levelBounds));
        for (let ne = G; ne < te; ne += 4) {
          const ee = this._indices[ne >> 2] | 0, ae = this._boxes[ne + 0], oe = this._boxes[ne + 1], se = this._boxes[ne + 2], ce = this._boxes[ne + 3];
          V < ae || q < oe || _ > se || H > ce || (G < this.numItems * 4 ? Z.set(ee) : J.push(ee));
        }
        G = J.pop();
      }
      return Z;
    }
  }
  mt(_FlatBush, "__name__", "_FlatBush");
  class SpatialIndex {
    constructor(_) {
      mt(this, "index", null);
      _ > 0 && (this.index = new _FlatBush(_));
    }
    add_rect(_, H, V, q) {
      var G;
      isFinite(_ + H + V + q) ? (G = this.index) == null || G.add(_, H, V, q) : this.add_empty();
    }
    add_point(_, H) {
      var V;
      isFinite(_ + H) ? (V = this.index) == null || V.add(_, H, _, H) : this.add_empty();
    }
    add_empty() {
      var _;
      (_ = this.index) == null || _.add(1 / 0, 1 / 0, -1 / 0, -1 / 0);
    }
    finish() {
      var _;
      (_ = this.index) == null || _.finish();
    }
    _normalize(_) {
      let { x0: H, y0: V, x1: q, y1: G } = _;
      return H > q && ([H, q] = [q, H]), V > G && ([V, G] = [G, V]), { x0: H, y0: V, x1: q, y1: G };
    }
    get bbox() {
      if (this.index == null)
        return empty$2();
      {
        const { minX: _, minY: H, maxX: V, maxY: q } = this.index;
        return { x0: _, y0: H, x1: V, y1: q };
      }
    }
    indices(_) {
      if (this.index == null)
        return new BitSet(0);
      {
        const { x0: H, y0: V, x1: q, y1: G } = this._normalize(_);
        return this.index.search_indices(H, V, q, G);
      }
    }
    bounds(_) {
      const H = empty$2();
      if (this.index == null)
        return H;
      const { boxes: V } = this.index;
      for (const q of this.indices(_)) {
        const G = V[4 * q + 0], J = V[4 * q + 1], Z = V[4 * q + 2], te = V[4 * q + 3];
        G >= _.x0 && G < H.x0 && (H.x0 = G), Z <= _.x1 && Z > H.x1 && (H.x1 = Z), J >= _.y0 && J < H.y0 && (H.y0 = J), te <= _.y1 && te > H.y1 && (H.y1 = te);
      }
      return H;
    }
  }
  mt(SpatialIndex, "__name__", "SpatialIndex");
  class DecorationView extends View$2 {
    constructor() {
      super(...arguments);
      mt(this, "visuals");
      mt(this, "marking");
    }
    *children() {
      yield* super.children(), yield this.marking;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), this.marking = await build_view(this.model.marking, { parent: this.parent });
    }
  }
  mt(DecorationView, "__name__", "DecorationView");
  var Q2;
  let Decoration$1 = (Q2 = class extends Model$1 {
    constructor(_) {
      super(_);
    }
  }, mt(Q2, "__name__", "Decoration"), Q2.prototype.default_view = DecorationView, Q2.define(({ Enum: _, Ref: H }) => ({
    marking: [H(Marking)],
    node: [_("start", "middle", "end")]
  })), Q2);
  const { abs: abs$a, ceil: ceil$2 } = Math, inherit = Symbol("inherit");
  class GlyphView extends View$2 {
    constructor() {
      super(...arguments);
      mt(this, "visuals");
      /** @internal */
      mt(this, "glglyph");
      mt(this, "_index", null);
      mt(this, "_data_size", null);
      mt(this, "_nohit_warned", /* @__PURE__ */ new Set());
      mt(this, "decorations", /* @__PURE__ */ new Map());
      mt(this, "_base", null);
    }
    get renderer() {
      return this.parent;
    }
    get has_webgl() {
      return this.glglyph != null;
    }
    get index() {
      const { _index: H } = this;
      if (H != null)
        return H;
      throw new Error(`${this}.index_data() wasn't called`);
    }
    get data_size() {
      const { base: H } = this;
      if (H != null)
        return H.data_size;
      {
        const { _data_size: V } = this;
        if (V != null)
          return V;
        throw new Error(`${this}.set_data() wasn't called`);
      }
    }
    initialize() {
      super.initialize(), this.visuals = new Visuals(this);
    }
    *children() {
      yield* super.children(), yield* this.decorations.values();
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await build_views(this.decorations, this.model.decorations, { parent: this.parent });
      const { webgl: H } = this.canvas;
      if (H != null && this.load_glglyph != null) {
        const V = await this.load_glglyph();
        this.glglyph = new V(H.regl_wrapper, this);
      }
    }
    request_render() {
      this.parent.request_render();
    }
    get canvas() {
      return this.renderer.parent.canvas_view;
    }
    render(H, V, q) {
      if (this.glglyph != null)
        this.glglyph.render(H, V, this.base ?? this);
      else {
        if (this.canvas.webgl != null && settings.force_webgl)
          throw new Error(`${this} doesn't support webgl rendering`);
        this._render(H, V, q);
      }
    }
    has_finished() {
      return !0;
    }
    notify_finished() {
      this.renderer.notify_finished();
    }
    _bounds(H) {
      return H;
    }
    bounds() {
      return this._bounds(this.index.bbox);
    }
    log_bounds() {
      const { x0: H, x1: V } = this.index.bounds(positive_x()), { y0: q, y1: G } = this.index.bounds(positive_y());
      return this._bounds({ x0: H, y0: q, x1: V, y1: G });
    }
    get_anchor_point(H, V, [q, G]) {
      switch (H) {
        case "center":
        case "center_center": {
          const [J, Z] = this.scenterxy(V, q, G);
          return { x: J, y: Z };
        }
        default:
          return null;
      }
    }
    sdist(H, V, q, G = "edge", J = !1) {
      const Z = V.length, te = new ScreenArray(Z), ne = H.s_compute;
      if (G == "center")
        for (let ee = 0; ee < Z; ee++) {
          const ae = V[ee], oe = q.get(ee) / 2, se = ne(ae - oe), ce = ne(ae + oe);
          te[ee] = abs$a(ce - se);
        }
      else
        for (let ee = 0; ee < Z; ee++) {
          const ae = V[ee], oe = ne(ae), se = ne(ae + q.get(ee));
          te[ee] = abs$a(se - oe);
        }
      return J && inplace_map(te, (ee) => ceil$2(ee)), te;
    }
    draw_legend_for_index(H, V, q) {
    }
    hit_test(H) {
      const V = (() => {
        var q, G, J, Z;
        switch (H.type) {
          case "point":
            return (q = this._hit_point) == null ? void 0 : q.call(this, H);
          case "span":
            return (G = this._hit_span) == null ? void 0 : G.call(this, H);
          case "rect":
            return (J = this._hit_rect) == null ? void 0 : J.call(this, H);
          case "poly":
            return (Z = this._hit_poly) == null ? void 0 : Z.call(this, H);
        }
      })();
      return V ?? (this._nohit_warned.has(H.type) || (logger$1.debug(`'${H.type}' selection not available for ${this.model.type}`), this._nohit_warned.add(H.type)), null);
    }
    _hit_rect_against_index(H) {
      const { sx0: V, sx1: q, sy0: G, sy1: J } = H, [Z, te] = this.renderer.coordinates.x_scale.r_invert(V, q), [ne, ee] = this.renderer.coordinates.y_scale.r_invert(G, J), ae = [...this.index.indices({ x0: Z, x1: te, y0: ne, y1: ee })];
      return new Selection({ indices: ae });
    }
    _project_xy(H, V, q, G) {
      const J = this._is_inherited(H), Z = this._is_inherited(q);
      if (!J && !Z)
        inplace.project_xy(V, G);
      else if (!J || !Z) {
        const [te, ne] = project_xy(V, G);
        this._define_attr(H, te), this._define_attr(q, ne);
      }
    }
    _project_data() {
    }
    *_iter_visuals() {
      for (const H of this.visuals)
        for (const V of H)
          (V instanceof VectorSpec || V instanceof ScalarSpec) && (yield V);
    }
    get base() {
      return this._base;
    }
    set_base(H) {
      H != this && H instanceof this.constructor ? this._base = H : this._base = null;
    }
    _define_or_inherit_attr(H, V) {
      const q = V();
      q === inherit ? this._inherit_attr(H) : this._define_attr(H, q);
    }
    _define_attr(H, V) {
      Object.defineProperty(this, H, {
        configurable: !0,
        enumerable: !0,
        value: V
      }), this._define_inherited(H, !1);
    }
    _inherit_attr(H) {
      const { base: V } = this;
      assert$2(V != null), this._inherit_from(H, V);
    }
    _inherit_from(H, V) {
      Object.defineProperty(this, H, {
        configurable: !0,
        enumerable: !0,
        get() {
          return V[H];
        }
      }), this._define_inherited(H, !0);
    }
    _define_inherited(H, V) {
      Object.defineProperty(this, `inherited_${H}`, {
        configurable: !0,
        enumerable: !0,
        value: V
      });
    }
    _can_inherit_from(H, V) {
      if (V == null)
        return !1;
      const q = V.model.property(H.attr), G = H.get_value(), J = q.get_value();
      try {
        return is_equal(G, J);
      } catch (Z) {
        if (Z instanceof EqNotImplemented)
          return !1;
        throw Z;
      }
    }
    _is_inherited(H) {
      const V = isString$2(H) ? H : H.attr;
      return this[`inherited_${V}`];
    }
    set_visuals(H, V) {
      var q;
      for (const G of this._iter_visuals()) {
        const { base: J } = this;
        if (J != null && this._can_inherit_from(G, J))
          this._inherit_from(G.attr, J);
        else {
          const Z = G.uniform(H).select(V);
          this._define_attr(G.attr, Z);
        }
      }
      for (const G of this.visuals)
        G.update();
      (q = this.glglyph) == null || q.set_visuals_changed();
    }
    _transform_array(H, V) {
      const { x_source: q, y_source: G } = this.renderer.coordinates, J = H.dimension == "x" ? q : G;
      if (J instanceof FactorRange) {
        if (H instanceof CoordinateSpec)
          V = J.v_synthetic(V);
        else if (H instanceof CoordinateSeqSpec)
          for (let te = 0; te < V.length; te++)
            V[te] = J.v_synthetic(V[te]);
      }
      let Z;
      return H instanceof CoordinateSeqSpec ? Z = RaggedArray.from(V, Float64Array) : (H instanceof CoordinateSeqSeqSeqSpec, Z = V), Z;
    }
    async set_data(H, V, q) {
      var Z;
      const G = new Set(this._iter_visuals()), { base: J } = this;
      this._data_size = V.count;
      for (const te of this.model)
        if ((te instanceof VectorSpec || te instanceof ScalarSpec) && !G.has(te))
          if (J != null && this._can_inherit_from(te, J))
            this._inherit_from(te.attr, J), (te instanceof DistanceSpec || te instanceof ScreenSizeSpec) && this._inherit_from(`max_${te.attr}`, J);
          else if (te instanceof BaseCoordinateSpec) {
            const ne = this._transform_array(te, V.select(te.array(H)));
            this._define_attr(te.attr, ne);
          } else {
            const ne = te.uniform(H).select(V);
            if (this._define_attr(te.attr, ne), te instanceof DistanceSpec || te instanceof ScreenSizeSpec) {
              const ee = max$i(ne);
              this._define_attr(`max_${te.attr}`, ee);
            }
          }
      this.renderer.plot_view.model.use_map && this._project_data(), this._set_data(q ?? null), await this._set_lazy_data(q ?? null);
      for (const te of this.decorations.values())
        te.marking.set_data(H, V);
      (Z = this.glglyph) == null || Z.set_data_changed(), J == null && this.index_data();
    }
    _set_data(H) {
    }
    async _set_lazy_data(H) {
    }
    /**
     * Any data transformations that require visuals.
     */
    after_visuals() {
    }
    async after_lazy_visuals() {
    }
    get _index_size() {
      return this.data_size;
    }
    index_data() {
      const H = new SpatialIndex(this._index_size);
      this._index_data(H), H.finish(), this._index = H;
    }
    mask_data() {
      return this._mask_data == null ? BitSet.all_set(this.data_size) : this._mask_data();
    }
    map_data() {
      var J;
      const { x_scale: H, y_scale: V } = this.renderer.coordinates, { base: q } = this, G = (Z) => {
        const te = Z.dimension == "x" ? H : V, ne = this[Z.attr];
        return ne instanceof RaggedArray ? new RaggedArray(ne.offsets, te.v_compute(ne.data)) : te.v_compute(ne);
      };
      for (const Z of this.model)
        if (Z instanceof BaseCoordinateSpec)
          if (q != null && this._is_inherited(Z))
            this._inherit_from(`s${Z.attr}`, q);
          else {
            const te = G(Z);
            this._define_attr(`s${Z.attr}`, te);
          }
      this._map_data(), (J = this.glglyph) == null || J.set_data_mapped();
    }
    // This is where specs not included in coords are computed, e.g. radius.
    _map_data() {
    }
  }
  mt(GlyphView, "__name__", "GlyphView");
  const jg = class jg extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(jg, "__name__", "Glyph"), jg.define(({ List: _, Ref: H }) => ({
    decorations: [_(H(Decoration$1)), []]
  }));
  let Glyph = jg;
  class XYGlyphView extends GlyphView {
    _project_data() {
      this._project_xy("x", this.x, "y", this.y);
    }
    _index_data(_) {
      const { x: H, y: V, data_size: q } = this;
      for (let G = 0; G < q; G++) {
        const J = H[G], Z = V[G];
        _.add_point(J, Z);
      }
    }
    scenterxy(_) {
      return [this.sx[_], this.sy[_]];
    }
  }
  mt(XYGlyphView, "__name__", "XYGlyphView");
  const Gg = class Gg extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Gg, "__name__", "XYGlyph"), Gg.define(() => ({
    x: [XCoordinateSpec, { field: "x" }],
    y: [YCoordinateSpec, { field: "y" }]
  }));
  let XYGlyph = Gg;
  function point_in_poly(O, _, H, V) {
    let q = !1, G = H[H.length - 1], J = V[V.length - 1];
    for (let Z = 0; Z < H.length; Z++) {
      const te = H[Z], ne = V[Z];
      J < _ != ne < _ && G + (_ - J) / (ne - J) * (te - G) < O && (q = !q), G = te, J = ne;
    }
    return q;
  }
  function point_in_ellipse(O, _, H, V, q, G, J) {
    const Z = (Math.cos(H) / q) ** 2 + (Math.sin(H) / V) ** 2, te = 2 * Math.cos(H) * Math.sin(H) * ((1 / q) ** 2 - (1 / V) ** 2), ne = (Math.cos(H) / V) ** 2 + (Math.sin(H) / q) ** 2;
    return Z * (O - G) ** 2 + te * (O - G) * (_ - J) + ne * (_ - J) ** 2 <= 1;
  }
  function dist_2_pts(O, _) {
    return (O.x - _.x) ** 2 + (O.y - _.y) ** 2;
  }
  function dist_to_segment_squared(O, _, H) {
    const V = dist_2_pts(_, H);
    if (V == 0)
      return dist_2_pts(O, _);
    const q = ((O.x - _.x) * (H.x - _.x) + (O.y - _.y) * (H.y - _.y)) / V;
    if (q < 0)
      return dist_2_pts(O, _);
    if (q > 1)
      return dist_2_pts(O, H);
    const G = { x: _.x + q * (H.x - _.x), y: _.y + q * (H.y - _.y) };
    return dist_2_pts(O, G);
  }
  function dist_to_segment(O, _, H) {
    return Math.sqrt(dist_to_segment_squared(O, _, H));
  }
  function check_2_segments_intersect(O, _, H, V, q, G, J, Z) {
    const te = (Z - G) * (H - O) - (J - q) * (V - _);
    if (te == 0)
      return { hit: !1, x: null, y: null };
    {
      let ne = _ - G, ee = O - q;
      const ae = (J - q) * ne - (Z - G) * ee, oe = (H - O) * ne - (V - _) * ee;
      ne = ae / te, ee = oe / te;
      const se = O + ne * (H - O), ce = _ + ne * (V - _);
      return { hit: ne > 0 && ne < 1 && ee > 0 && ee < 1, x: se, y: ce };
    }
  }
  function generic_line_scalar_legend(O, _, { x0: H, x1: V, y0: q, y1: G }) {
    _.save(), _.beginPath(), _.moveTo(H, (q + G) / 2), _.lineTo(V, (q + G) / 2), O.line.apply(_), _.restore();
  }
  function generic_line_vector_legend(O, _, { x0: H, x1: V, y0: q, y1: G }, J) {
    _.save(), _.beginPath(), _.moveTo(H, (q + G) / 2), _.lineTo(V, (q + G) / 2), O.line.apply(_, J), _.restore();
  }
  function generic_area_scalar_legend(O, _, { x0: H, x1: V, y0: q, y1: G }) {
    var ce, le;
    const Z = Math.abs(V - H) * 0.1, ne = Math.abs(G - q) * 0.1, ee = H + Z, ae = V - Z, oe = q + ne, se = G - ne;
    _.beginPath(), _.rect(ee, oe, ae - ee, se - oe), O.fill.apply(_), (ce = O.hatch) == null || ce.apply(_), (le = O.line) == null || le.apply(_);
  }
  function generic_area_vector_legend(O, _, { x0: H, x1: V, y0: q, y1: G }, J) {
    var le, ue;
    const te = Math.abs(V - H) * 0.1, ee = Math.abs(G - q) * 0.1, ae = H + te, oe = V - te, se = q + ee, ce = G - ee;
    _.beginPath(), _.rect(ae, se, oe - ae, ce - se), O.fill.apply(_, J), (le = O.hatch) == null || le.apply(_, J), (ue = O.line) == null || ue.apply(_, J);
  }
  function line_interpolation(O, _, H, V, q, G) {
    const { sx: J, sy: Z } = _;
    let te, ne, ee, ae;
    _.type == "point" ? ([ee, ae] = O.yscale.r_invert(Z - 1, Z + 1), [te, ne] = O.xscale.r_invert(J - 1, J + 1)) : _.direction == "v" ? ([ee, ae] = O.yscale.r_invert(Z, Z), [te, ne] = [Math.min(H - 1, q - 1), Math.max(H + 1, q + 1)]) : ([te, ne] = O.xscale.r_invert(J, J), [ee, ae] = [Math.min(V - 1, G - 1), Math.max(V + 1, G + 1)]);
    const { x: oe, y: se } = check_2_segments_intersect(te, ee, ne, ae, H, V, q, G);
    return [oe, se];
  }
  var ey;
  let LineView$1 = (ey = class extends XYGlyphView {
    async load_glglyph() {
      const { LineGL: _ } = await Promise.resolve().then(() => line_gl);
      return _;
    }
    _render(_, H, V) {
      const { sx: q, sy: G } = { ...this, ...V }, J = this.parent.nonselection_glyph == this;
      let Z = null;
      const te = (ee) => Z != null && ee - Z != 1;
      let ne = !0;
      _.beginPath();
      for (const ee of H) {
        const ae = q[ee], oe = G[ee];
        J && !ne && Z != null && ee - Z > 1 && isFinite(q[Z + 1] + G[Z + 1]) && _.lineTo(q[Z + 1], G[Z + 1]), isFinite(ae + oe) ? (ne || te(ee) ? (J && ee > 0 && isFinite(q[ee - 1] + G[ee - 1]) ? (_.moveTo(q[ee - 1], G[ee - 1]), _.lineTo(ae, oe)) : _.moveTo(ae, oe), ne = !1) : _.lineTo(ae, oe), Z = ee) : ne = !0;
      }
      if (J && !ne && Z != null) {
        const ee = q.length;
        Z < ee - 1 && isFinite(q[Z + 1] + G[Z + 1]) && _.lineTo(q[Z + 1], G[Z + 1]);
      }
      this.visuals.line.set_value(_), _.stroke();
    }
    _hit_point(_) {
      const H = new Selection(), V = { x: _.sx, y: _.sy };
      let q = 9999;
      const G = Math.max(2, this.line_width.value / 2);
      for (let J = 0, Z = this.sx.length - 1; J < Z; J++) {
        const te = { x: this.sx[J], y: this.sy[J] }, ne = { x: this.sx[J + 1], y: this.sy[J + 1] }, ee = dist_to_segment(V, te, ne);
        ee < G && ee < q && (q = ee, H.add_to_selected_glyphs(this.model), H.view = this, H.line_indices = [J]);
      }
      return H;
    }
    _hit_span(_) {
      const { sx: H, sy: V } = _;
      let q, G;
      _.direction == "v" ? (q = this.renderer.yscale.invert(V), G = this.y) : (q = this.renderer.xscale.invert(H), G = this.x);
      const J = [];
      for (let te = 0, ne = G.length - 1; te < ne; te++) {
        const ee = G[te], ae = G[te + 1];
        (ee <= q && q <= ae || ae <= q && q <= ee) && J.push(te);
      }
      const Z = new Selection();
      return J.length != 0 && (Z.add_to_selected_glyphs(this.model), Z.view = this, Z.line_indices = J), Z;
    }
    get_interpolation_hit(_, H) {
      const [V, q, G, J] = [this.x[_], this.y[_], this.x[_ + 1], this.y[_ + 1]];
      return line_interpolation(this.renderer, H, V, q, G, J);
    }
    draw_legend_for_index(_, H, V) {
      generic_line_scalar_legend(this.visuals, _, H);
    }
  }, mt(ey, "__name__", "LineView"), ey);
  var L2;
  let Line$2 = (L2 = class extends XYGlyph {
    constructor(_) {
      super(_);
    }
  }, mt(L2, "__name__", "Line"), L2.prototype.default_view = LineView$1, L2.mixins(LineScalar$1), L2);
  class PatchView extends XYGlyphView {
    _render(_, H, V) {
      const { sx: q, sy: G } = { ...this, ...V };
      let J = !0;
      _.beginPath();
      for (const Z of H) {
        const te = q[Z], ne = G[Z];
        isFinite(te + ne) ? J ? (_.moveTo(te, ne), J = !1) : _.lineTo(te, ne) : (_.closePath(), J = !0);
      }
      _.closePath(), this.visuals.fill.apply(_), this.visuals.hatch.apply(_), this.visuals.line.apply(_);
    }
    draw_legend_for_index(_, H, V) {
      generic_area_scalar_legend(this.visuals, _, H);
    }
    _hit_point(_) {
      const H = new Selection();
      return point_in_poly(_.sx, _.sy, this.sx, this.sy) && (H.add_to_selected_glyphs(this.model), H.view = this), H;
    }
  }
  mt(PatchView, "__name__", "PatchView");
  const B3 = class B3 extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(B3, "__name__", "Patch"), B3.prototype.default_view = PatchView, B3.mixins([LineScalar$1, FillScalar$1, HatchScalar$1]);
  let Patch = B3;
  class AreaView extends GlyphView {
    draw_legend_for_index(_, H, V) {
      generic_area_scalar_legend(this.visuals, _, H);
    }
  }
  mt(AreaView, "__name__", "AreaView");
  const Wg = class Wg extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Wg, "__name__", "Area"), Wg.mixins([FillScalar$1, HatchScalar$1]);
  let Area = Wg;
  function flip_step_mode(O) {
    switch (O) {
      case "before":
        return "after";
      case "after":
        return "before";
      case "center":
        return "center";
    }
  }
  class HAreaStepView extends AreaView {
    _index_data(_) {
      const { min: H, max: V } = Math, { x1: q, x2: G, y: J } = this;
      for (let Z = 0; Z < this.data_size; Z++) {
        const te = q[Z], ne = G[Z], ee = J[Z];
        _.add_rect(H(te, ne), ee, V(te, ne), ee);
      }
    }
    _step_path(_, H, V, q, G, J) {
      let Z = V[G], te = q[G];
      const ne = G < J ? 1 : -1;
      for (let ee = G + ne; ee != J; ee += ne) {
        switch (H) {
          case "before": {
            _.lineTo(V[ee], te), _.lineTo(V[ee], q[ee]);
            break;
          }
          case "after": {
            _.lineTo(Z, q[ee]), _.lineTo(V[ee], q[ee]);
            break;
          }
          case "center": {
            const ae = (te + q[ee]) / 2;
            _.lineTo(Z, ae), _.lineTo(V[ee], ae), _.lineTo(V[ee], q[ee]);
            break;
          }
        }
        Z = V[ee], te = q[ee];
      }
    }
    _render(_, H, V) {
      const { sx1: q, sx2: G, sy: J } = { ...this, ...V }, Z = this.model.step_mode, te = flip_step_mode(this.model.step_mode);
      _.beginPath(), _.moveTo(q[0], J[0]), this._step_path(_, Z, q, J, 0, J.length), this._step_path(_, te, G, J, J.length, -1), _.closePath(), this.visuals.fill.apply(_), this.visuals.hatch.apply(_);
    }
    scenterxy(_) {
      const H = (this.sx1[_] + this.sx2[_]) / 2, V = this.sy[_];
      return [H, V];
    }
    _line_selection_for(_) {
      return new Selection({ line_indices: [_], selected_glyphs: [this.model], view: this });
    }
    _hit_point_before(_) {
      const { sy: H, sx1: V, sx2: q } = this;
      for (let G = 1; G < this.data_size; G++) {
        const J = [V[G], V[G], q[G], q[G]], Z = [H[G - 1], H[G], H[G], H[G - 1]];
        if (point_in_poly(_.sx, _.sy, J, Z))
          return this._line_selection_for(G);
      }
      return new Selection();
    }
    _hit_point_after(_) {
      const { sy: H, sx1: V, sx2: q } = this;
      for (let G = 0; G < this.data_size - 1; G++) {
        const J = [V[G], V[G], q[G], q[G]], Z = [H[G], H[G + 1], H[G + 1], H[G]];
        if (point_in_poly(_.sx, _.sy, J, Z))
          return this._line_selection_for(G);
      }
      return new Selection();
    }
    _hit_point_center(_) {
      const { sy: H, sx1: V, sx2: q } = this;
      for (let G = 0; G < this.data_size; G++) {
        const J = (H[G - 1] + H[G]) / 2, Z = (H[G] + H[G + 1]) / 2, te = [V[G], V[G], q[G], q[G]], ne = G == 0 ? [H[G], Z, Z, H[G]] : G == this.data_size - 1 ? [J, H[G], H[G], J] : [J, Z, Z, J];
        if (point_in_poly(_.sx, _.sy, te, ne))
          return this._line_selection_for(G);
      }
      return new Selection();
    }
    _hit_point(_) {
      switch (this.model.step_mode) {
        case "before":
          return this._hit_point_before(_);
        case "after":
          return this._hit_point_after(_);
        case "center":
          return this._hit_point_center(_);
      }
    }
  }
  mt(HAreaStepView, "__name__", "HAreaStepView");
  const H3 = class H3 extends Area {
    constructor(_) {
      super(_);
    }
  };
  mt(H3, "__name__", "HAreaStep"), H3.prototype.default_view = HAreaStepView, H3.define(({}) => ({
    x1: [XCoordinateSpec, { field: "x1" }],
    x2: [XCoordinateSpec, { field: "x2" }],
    y: [YCoordinateSpec, { field: "y" }],
    step_mode: [StepMode, "before"]
  }));
  let HAreaStep = H3;
  class HAreaView extends AreaView {
    _index_data(_) {
      const { min: H, max: V } = Math, { data_size: q } = this;
      for (let G = 0; G < q; G++) {
        const J = this.x1[G], Z = this.x2[G], te = this.y[G];
        _.add_rect(H(J, Z), te, V(J, Z), te);
      }
    }
    _render(_, H, V) {
      const { sx1: q, sx2: G, sy: J } = { ...this, ...V };
      _.beginPath();
      for (let Z = 0, te = q.length; Z < te; Z++)
        _.lineTo(q[Z], J[Z]);
      for (let Z = G.length - 1; Z >= 0; Z--)
        _.lineTo(G[Z], J[Z]);
      _.closePath(), this.visuals.fill.apply(_), this.visuals.hatch.apply(_);
    }
    _hit_point(_) {
      const H = this.sy.length, V = new Selection();
      for (let q = 0, G = H - 1; q < G; q++) {
        const J = [this.sx1[q], this.sx1[q + 1], this.sx2[q + 1], this.sx2[q]], Z = [this.sy[q], this.sy[q + 1], this.sy[q + 1], this.sy[q]];
        if (point_in_poly(_.sx, _.sy, J, Z)) {
          V.add_to_selected_glyphs(this.model), V.view = this, V.line_indices = [q];
          break;
        }
      }
      return V;
    }
    scenterxy(_) {
      const H = (this.sx1[_] + this.sx2[_]) / 2, V = this.sy[_];
      return [H, V];
    }
  }
  mt(HAreaView, "__name__", "HAreaView");
  const z3 = class z3 extends Area {
    constructor(_) {
      super(_);
    }
  };
  mt(z3, "__name__", "HArea"), z3.prototype.default_view = HAreaView, z3.define(({}) => ({
    x1: [XCoordinateSpec, { field: "x1" }],
    x2: [XCoordinateSpec, { field: "x2" }],
    y: [YCoordinateSpec, { field: "y" }]
  }));
  let HArea = z3;
  class VAreaStepView extends AreaView {
    _index_data(_) {
      const { min: H, max: V } = Math, { x: q, y1: G, y2: J } = this;
      for (let Z = 0; Z < this.data_size; Z++) {
        const te = q[Z], ne = G[Z], ee = J[Z];
        _.add_rect(te, H(ne, ee), te, V(ne, ee));
      }
    }
    _step_path(_, H, V, q, G, J) {
      let Z = V[G], te = q[G];
      const ne = G < J ? 1 : -1;
      for (let ee = G + ne; ee != J; ee += ne) {
        switch (H) {
          case "before": {
            _.lineTo(Z, q[ee]), _.lineTo(V[ee], q[ee]);
            break;
          }
          case "after": {
            _.lineTo(V[ee], te), _.lineTo(V[ee], q[ee]);
            break;
          }
          case "center": {
            const ae = (Z + V[ee]) / 2;
            _.lineTo(ae, te), _.lineTo(ae, q[ee]), _.lineTo(V[ee], q[ee]);
            break;
          }
        }
        Z = V[ee], te = q[ee];
      }
    }
    _render(_, H, V) {
      const { sx: q, sy1: G, sy2: J } = { ...this, ...V }, Z = this.model.step_mode, te = flip_step_mode(this.model.step_mode);
      _.beginPath(), _.moveTo(q[0], G[0]), this._step_path(_, Z, q, G, 0, q.length), this._step_path(_, te, q, J, q.length, -1), _.closePath(), this.visuals.fill.apply(_), this.visuals.hatch.apply(_);
    }
    scenterxy(_) {
      const H = this.sx[_], V = (this.sy1[_] + this.sy2[_]) / 2;
      return [H, V];
    }
    _line_selection_for(_) {
      return new Selection({ line_indices: [_], selected_glyphs: [this.model], view: this });
    }
    _hit_point_before(_) {
      const { sx: H, sy1: V, sy2: q } = this;
      for (let G = 1; G < this.data_size; G++) {
        const J = [H[G - 1], H[G], H[G], H[G - 1]], Z = [V[G], V[G], q[G], q[G]];
        if (point_in_poly(_.sx, _.sy, J, Z))
          return this._line_selection_for(G);
      }
      return new Selection();
    }
    _hit_point_after(_) {
      const { sx: H, sy1: V, sy2: q } = this;
      for (let G = 0; G < this.data_size - 1; G++) {
        const J = [H[G], H[G + 1], H[G + 1], H[G]], Z = [V[G], V[G], q[G], q[G]];
        if (point_in_poly(_.sx, _.sy, J, Z))
          return this._line_selection_for(G);
      }
      return new Selection();
    }
    _hit_point_center(_) {
      const { sx: H, sy1: V, sy2: q } = this;
      for (let G = 0; G < this.data_size; G++) {
        const J = (H[G - 1] + H[G]) / 2, Z = (H[G] + H[G + 1]) / 2, te = G == 0 ? [H[G], Z, Z, H[G]] : G == this.data_size - 1 ? [J, H[G], H[G], J] : [J, Z, Z, J], ne = [V[G], V[G], q[G], q[G]];
        if (point_in_poly(_.sx, _.sy, te, ne))
          return this._line_selection_for(G);
      }
      return new Selection();
    }
    _hit_point(_) {
      switch (this.model.step_mode) {
        case "before":
          return this._hit_point_before(_);
        case "after":
          return this._hit_point_after(_);
        case "center":
          return this._hit_point_center(_);
      }
    }
  }
  mt(VAreaStepView, "__name__", "VAreaStepView");
  const V3 = class V3 extends Area {
    constructor(_) {
      super(_);
    }
  };
  mt(V3, "__name__", "VAreaStep"), V3.prototype.default_view = VAreaStepView, V3.define(({}) => ({
    x: [XCoordinateSpec, { field: "x" }],
    y1: [YCoordinateSpec, { field: "y1" }],
    y2: [YCoordinateSpec, { field: "y2" }],
    step_mode: [StepMode, "before"]
  }));
  let VAreaStep = V3;
  class VAreaView extends AreaView {
    _index_data(_) {
      const { min: H, max: V } = Math, { data_size: q } = this;
      for (let G = 0; G < q; G++) {
        const J = this.x[G], Z = this.y1[G], te = this.y2[G];
        _.add_rect(J, H(Z, te), J, V(Z, te));
      }
    }
    _render(_, H, V) {
      const { sx: q, sy1: G, sy2: J } = { ...this, ...V };
      _.beginPath();
      for (let Z = 0, te = G.length; Z < te; Z++)
        _.lineTo(q[Z], G[Z]);
      for (let Z = J.length - 1; Z >= 0; Z--)
        _.lineTo(q[Z], J[Z]);
      _.closePath(), this.visuals.fill.apply(_), this.visuals.hatch.apply(_);
    }
    scenterxy(_) {
      const H = this.sx[_], V = (this.sy1[_] + this.sy2[_]) / 2;
      return [H, V];
    }
    _hit_point(_) {
      const H = this.sx.length, V = new Selection();
      for (let q = 0, G = H - 1; q < G; q++) {
        const J = [this.sx[q], this.sx[q + 1], this.sx[q + 1], this.sx[q]], Z = [this.sy1[q], this.sy1[q + 1], this.sy2[q + 1], this.sy2[q]];
        if (point_in_poly(_.sx, _.sy, J, Z)) {
          V.add_to_selected_glyphs(this.model), V.view = this, V.line_indices = [q];
          break;
        }
      }
      return V;
    }
  }
  mt(VAreaView, "__name__", "VAreaView");
  const U3 = class U3 extends Area {
    constructor(_) {
      super(_);
    }
  };
  mt(U3, "__name__", "VArea"), U3.prototype.default_view = VAreaView, U3.define(({}) => ({
    x: [XCoordinateSpec, { field: "x" }],
    y1: [YCoordinateSpec, { field: "y1" }],
    y2: [YCoordinateSpec, { field: "y2" }]
  }));
  let VArea = U3;
  var ty;
  let Filter$1 = (ty = class extends Model$1 {
    constructor(_) {
      super(_);
    }
  }, mt(ty, "__name__", "Filter"), ty);
  class AllIndices extends Filter$1 {
    constructor(_) {
      super(_);
    }
    compute_indices(_) {
      const H = _.get_length() ?? 1;
      return BitSet.all_set(H);
    }
  }
  mt(AllIndices, "__name__", "AllIndices");
  const Yg = class Yg extends Filter$1 {
    constructor(_) {
      super(_);
    }
    connect_signals() {
      super.connect_signals();
      const _ = () => {
        this.change.emit();
      }, H = (G) => {
        for (const J of G)
          this.connect(J.change, _);
      }, V = (G) => {
        for (const J of G)
          this.disconnect(J.change, _);
      };
      let q = (() => {
        const { operands: G } = this.properties;
        return G.is_unset ? [] : G.get_value();
      })();
      H(q), this.on_change(this.properties.operands, () => {
        V(q), q = this.operands, H(q);
      });
    }
    compute_indices(_) {
      const { operands: H } = this;
      if (H.length == 0) {
        const V = _.get_length() ?? 1;
        return BitSet.all_set(V);
      } else {
        const [V, ...q] = H.map((G) => G.compute_indices(_));
        for (const G of q)
          this._inplace_op(V, G);
        return V;
      }
    }
  };
  mt(Yg, "__name__", "CompositeFilter"), Yg.define(({ List: _, Ref: H }) => ({
    operands: [_(H(Filter$1))]
  }));
  let CompositeFilter = Yg;
  class IntersectionFilter extends CompositeFilter {
    constructor(_) {
      super(_);
    }
    _inplace_op(_, H) {
      _.intersect(H);
    }
  }
  mt(IntersectionFilter, "__name__", "IntersectionFilter");
  class CDSViewView extends View$2 {
    initialize() {
      super.initialize(), this.compute_indices();
    }
    connect_signals() {
      super.connect_signals();
      const _ = () => {
        this.compute_indices();
      }, H = (te) => {
        this.connect(te.change, _);
      }, V = (te) => {
        this.disconnect(te.change, _);
      };
      let { filter: q } = this.model;
      H(q), this.on_change(this.model.properties.filter, () => {
        V(q), q = this.model.filter, H(q), _();
      });
      const G = (te) => {
        this.connect(te.change, _), this.connect(te.streaming, _), this.connect(te.patching, _), this.connect(te.properties.data.change, _);
      }, J = (te) => {
        this.disconnect(te.change, _), this.disconnect(te.streaming, _), this.disconnect(te.patching, _), this.disconnect(te.properties.data.change, _);
      };
      let Z = this.parent.data_source.get_value();
      G(Z), this.on_change(this.parent.data_source, () => {
        J(Z), Z = this.parent.data_source.get_value(), G(Z), _();
      });
    }
    compute_indices() {
      const _ = this.parent.data_source.get_value(), H = _.get_length() ?? 1, V = BitSet.all_set(H), q = this.model.filter.compute_indices(_);
      V.intersect(q), this.model.indices = V, this.model._indices_map_to_subset();
    }
  }
  mt(CDSViewView, "__name__", "CDSViewView");
  const Tp = class Tp extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "_indices");
    }
    _indices_map_to_subset() {
      this._indices = [...this.indices], this.indices_map = /* @__PURE__ */ new Map();
      const { _indices: H, indices_map: V } = this, q = H.length;
      for (let G = 0; G < q; G++)
        V.set(H[G], G);
    }
    convert_selection_from_subset(H) {
      return H.map((V) => this._indices[V]);
    }
    convert_selection_to_subset(H) {
      return H.map((V) => this.indices_map.get(V));
    }
    convert_indices_from_subset(H) {
      return H.map((V) => this._indices[V]);
    }
    /** @deprecated */
    get filters() {
      const { filter: H } = this;
      return H instanceof IntersectionFilter ? H.operands : H instanceof AllIndices ? [] : [H];
    }
    /** @deprecated */
    set filters(H) {
      H.length == 0 ? this.filter = new AllIndices() : H.length == 1 ? this.filter = H[0] : this.filter = new IntersectionFilter({ operands: H });
    }
  };
  mt(Tp, "__name__", "CDSView"), Tp.prototype.default_view = CDSViewView, Tp.define(({ Ref: H }) => ({
    filter: [H(Filter$1), () => new AllIndices()]
  })), Tp.internal(({ Int: H, Mapping: V, Ref: q, Nullable: G }) => ({
    indices: [q(BitSet)],
    indices_map: [V(H, H), /* @__PURE__ */ new Map()],
    masked: [G(q(BitSet)), null]
  }));
  let CDSView = Tp;
  const selection_defaults = {
    fill: {},
    line: {}
  }, decimated_defaults = {
    fill: { fill_alpha: 0.3, fill_color: "grey" },
    line: { line_alpha: 0.3, line_color: "grey" }
  }, nonselection_defaults = {
    fill: { fill_alpha: 0.2 },
    line: {}
  }, muted_defaults = {
    fill: { fill_alpha: 0.2 },
    line: {}
  };
  class GlyphRendererView extends DataRendererView {
    constructor() {
      super(...arguments);
      mt(this, "cds_view");
      mt(this, "glyph");
      mt(this, "selection_glyph");
      mt(this, "nonselection_glyph");
      mt(this, "hover_glyph");
      mt(this, "muted_glyph");
      mt(this, "decimated_glyph");
      mt(this, "all_indices");
      mt(this, "decimated");
      mt(this, "last_dtrender");
      mt(this, "_previous_inspected");
    }
    get glyph_view() {
      return this.glyph;
    }
    *children() {
      yield* super.children(), yield this.cds_view, yield this.glyph, yield this.selection_glyph, yield this.nonselection_glyph, this.hover_glyph != null && (yield this.hover_glyph), yield this.muted_glyph, yield this.decimated_glyph;
    }
    get data_source() {
      return this.model.properties.data_source;
    }
    async lazy_initialize() {
      var se;
      await super.lazy_initialize(), this.cds_view = await build_view(this.model.view, { parent: this });
      const H = this.model.glyph;
      this.glyph = await this.build_glyph_view(H);
      const V = "fill" in this.glyph.visuals, q = "line" in this.glyph.visuals, G = { ...H.attributes };
      delete G.id;
      function J(ce) {
        const le = clone$1(G);
        return V && extend$5(le, ce.fill), q && extend$5(le, ce.line), new H.constructor(le);
      }
      function Z(ce, le) {
        return le instanceof Glyph ? le : le == "auto" ? J(ce) : J({ fill: {}, line: {} });
      }
      let { selection_glyph: te, nonselection_glyph: ne, hover_glyph: ee, muted_glyph: ae } = this.model;
      te = Z(selection_defaults, te), this.selection_glyph = await this.build_glyph_view(te), ne = Z(nonselection_defaults, ne), this.nonselection_glyph = await this.build_glyph_view(ne), ee != null && (this.hover_glyph = await this.build_glyph_view(ee)), ae = Z(muted_defaults, ae), this.muted_glyph = await this.build_glyph_view(ae);
      const oe = Z(decimated_defaults, "auto");
      this.decimated_glyph = await this.build_glyph_view(oe), this.selection_glyph.set_base(this.glyph), this.nonselection_glyph.set_base(this.glyph), (se = this.hover_glyph) == null || se.set_base(this.glyph), this.muted_glyph.set_base(this.glyph), this.decimated_glyph.set_base(this.glyph), await this.set_data();
    }
    async build_glyph_view(H) {
      return build_view(H, { parent: this });
    }
    remove() {
      var H;
      this.cds_view.remove(), this.glyph.remove(), this.selection_glyph.remove(), this.nonselection_glyph.remove(), (H = this.hover_glyph) == null || H.remove(), this.muted_glyph.remove(), this.decimated_glyph.remove(), super.remove();
    }
    connect_signals() {
      super.connect_signals();
      const H = () => this.request_render(), V = () => this.update_data();
      this.connect(this.model.change, H), this.connect(this.glyph.model.change, V), this.connect(this.selection_glyph.model.change, V), this.connect(this.nonselection_glyph.model.change, V), this.hover_glyph != null && this.connect(this.hover_glyph.model.change, V), this.connect(this.muted_glyph.model.change, V), this.connect(this.decimated_glyph.model.change, V), this.connect(this.model.data_source.change, V), this.connect(this.model.data_source.streaming, V), this.connect(this.model.data_source.patching, (te) => this.update_data(te)), this.connect(this.model.data_source.selected.change, H), this.connect(this.model.data_source._select, H), this.hover_glyph != null && this.connect(this.model.data_source.inspect, () => {
        const { inspected: te } = this.model.data_source, ne = {
          indices: te.indices,
          line_indices: te.line_indices,
          multiline_indices: te.multiline_indices,
          image_indices: te.image_indices,
          selected_glyphs: te.selected_glyphs
        };
        is_equal(this._previous_inspected, ne) || (this._previous_inspected = ne, H());
      }), this.connect(this.model.properties.view.change, async () => {
        this.cds_view.remove(), this.cds_view = await build_view(this.model.view, { parent: this }), await V();
      }), this.connect(this.model.view.properties.indices.change, V), this.connect(this.model.view.properties.masked.change, async () => await this.set_visuals()), this.connect(this.model.properties.visible.change, () => this.plot_view.invalidate_dataranges = !0);
      const { x_ranges: q, y_ranges: G } = this.plot_view.frame;
      for (const [, te] of q)
        te instanceof FactorRange && this.connect(te.invalidate_synthetic, V);
      for (const [, te] of G)
        te instanceof FactorRange && this.connect(te.invalidate_synthetic, V);
      const { transformchange: J, exprchange: Z } = this.model.glyph;
      this.connect(J, V), this.connect(Z, V);
    }
    _update_masked_indices() {
      const H = this.glyph.mask_data();
      return this.model.view.masked = H, H;
    }
    async update_data(H) {
      await this.set_data(H), this.request_render();
    }
    // in case of partial updates like patching, the list of indices that actually
    // changed may be passed as the "indices" parameter to afford any optional optimizations
    async set_data(H) {
      var Z;
      const V = this.model.data_source;
      this.all_indices = this.model.view.indices;
      const { all_indices: q } = this;
      await this.glyph.set_data(V, q, H), await this.decimated_glyph.set_data(V, q, H), await this.selection_glyph.set_data(V, q, H), await this.nonselection_glyph.set_data(V, q, H), await ((Z = this.hover_glyph) == null ? void 0 : Z.set_data(V, q, H)), await this.muted_glyph.set_data(V, q, H), await this.set_visuals(), this._update_masked_indices();
      const { lod_factor: G } = this.plot_model, J = this.all_indices.count;
      this.decimated = new BitSet(J);
      for (let te = 0; te < J; te += G)
        this.decimated.set(te);
      this.plot_view.invalidate_dataranges = !0;
    }
    async set_visuals() {
      var q, G, J;
      const H = this.model.data_source, { all_indices: V } = this;
      this.glyph.set_visuals(H, V), this.glyph.after_visuals(), await this.glyph.after_lazy_visuals(), this.decimated_glyph.set_visuals(H, V), this.decimated_glyph.after_visuals(), await this.decimated_glyph.after_lazy_visuals(), this.selection_glyph.set_visuals(H, V), this.selection_glyph.after_visuals(), await this.selection_glyph.after_lazy_visuals(), this.nonselection_glyph.set_visuals(H, V), this.nonselection_glyph.after_visuals(), await this.nonselection_glyph.after_lazy_visuals(), (q = this.hover_glyph) == null || q.set_visuals(H, V), (G = this.hover_glyph) == null || G.after_visuals(), await ((J = this.hover_glyph) == null ? void 0 : J.after_lazy_visuals()), this.muted_glyph.set_visuals(H, V), this.muted_glyph.after_visuals(), await this.muted_glyph.after_lazy_visuals();
    }
    get has_webgl() {
      return this.glyph.has_webgl;
    }
    _render() {
      var le;
      const H = this.has_webgl;
      this.glyph.map_data(), this.decimated_glyph.map_data(), this.selection_glyph.map_data(), this.nonselection_glyph.map_data(), (le = this.hover_glyph) == null || le.map_data(), this.muted_glyph.map_data();
      const V = [...this.all_indices];
      let q = [...this._update_masked_indices()];
      const { ctx: G } = this.layer;
      G.save();
      const { selected: J } = this.model.data_source, Z = J.is_empty() ? [] : this.glyph instanceof LineView$1 && J.selected_glyph === this.glyph.model ? this.model.view.convert_indices_from_subset(q) : J.indices, { inspected: te } = this.model.data_source;
      this._previous_inspected = {
        indices: te.indices,
        line_indices: te.line_indices,
        multiline_indices: te.multiline_indices,
        image_indices: te.image_indices,
        selected_glyphs: te.selected_glyphs
      };
      const ne = new Set(te.is_empty() ? [] : te.selected_glyph != null ? this.model.view.convert_indices_from_subset(q) : te.indices.length > 0 ? te.indices : Object.keys(te.multiline_indices).map((ue) => parseInt(ue))), ee = filter$2(q, (ue) => ne.has(V[ue])), { lod_threshold: ae } = this.plot_model;
      let oe, se, ce;
      if (this.model.document != null && this.model.document.interactive_duration() > 0 && !H && ae != null && V.length > ae ? (q = [...this.decimated], oe = this.decimated_glyph, se = this.decimated_glyph, ce = this.selection_glyph) : (oe = this.model.muted ? this.muted_glyph : this.glyph, se = this.nonselection_glyph, ce = this.selection_glyph), this.hover_glyph != null && ee.length != 0) {
        const ue = new Set(q);
        for (const de of ee)
          ue.delete(de);
        q = [...ue];
      }
      if (Z.length == 0)
        if (this.glyph instanceof LineView$1)
          this.hover_glyph != null && ee.length != 0 ? this.hover_glyph.render(G, this.model.view.convert_indices_from_subset(ee)) : oe.render(G, V);
        else if (this.glyph instanceof PatchView || this.glyph instanceof HAreaView || this.glyph instanceof VAreaView || this.glyph instanceof VAreaStepView || this.glyph instanceof HAreaStepView)
          if (te.selected_glyphs.length == 0 || this.hover_glyph == null)
            oe.render(G, V);
          else
            for (const ue of te.selected_glyphs)
              ue == this.glyph.model && this.hover_glyph.render(G, V);
        else
          oe.render(G, q), this.hover_glyph != null && ee.length != 0 && this.hover_glyph.render(G, ee);
      else {
        const ue = new Set(Z), de = new Array(), fe = new Array();
        if (this.glyph instanceof LineView$1)
          for (const he of V)
            ue.has(he) ? de.push(he) : fe.push(he);
        else
          for (const he of q)
            ue.has(V[he]) ? de.push(he) : fe.push(he);
        se.render(G, fe), ce.render(G, de), this.hover_glyph != null && (this.glyph instanceof LineView$1 ? this.hover_glyph.render(G, this.model.view.convert_indices_from_subset(ee)) : this.hover_glyph.render(G, ee));
      }
      G.restore();
    }
    get_reference_point(H, V) {
      if (H != null) {
        const q = this.model.data_source.get_column(H);
        if (q != null) {
          for (const [G, J] of this.model.view.indices_map)
            if (q[G] == V)
              return J;
        }
      }
      return 0;
    }
    draw_legend(H, V, q, G, J, Z, te, ne) {
      if (this.glyph.data_size == 0)
        return;
      const ee = (() => {
        if (ne == null)
          return this.get_reference_point(Z, te);
        {
          const { indices_map: ae } = this.model.view;
          return ae.get(ne);
        }
      })();
      ee != null && this.glyph.draw_legend_for_index(H, { x0: V, x1: q, y0: G, y1: J }, ee);
    }
    hit_test(H) {
      if (!this.model.visible)
        return null;
      const V = this.glyph.hit_test(H);
      return V == null ? null : this.model.view.convert_selection_from_subset(V);
    }
  }
  mt(GlyphRendererView, "__name__", "GlyphRendererView");
  const q3 = class q3 extends DataRenderer {
    constructor(_) {
      super(_);
    }
    get_selection_manager() {
      return this.data_source.selection_manager;
    }
    add_decoration(_, H) {
      const V = new Decoration$1({ marking: _, node: H }), q = [
        this.glyph,
        this.selection_glyph,
        this.nonselection_glyph,
        this.hover_glyph,
        this.muted_glyph
      ];
      for (const G of q)
        G instanceof Glyph && (G.decorations = [...G.decorations, V]);
      return V;
    }
  };
  mt(q3, "__name__", "GlyphRenderer"), q3.prototype.default_view = GlyphRendererView, q3.define(({ Bool: _, Auto: H, Or: V, Ref: q, Null: G, Nullable: J }) => ({
    data_source: [q(ColumnarDataSource)],
    view: [q(CDSView), () => new CDSView()],
    glyph: [q(Glyph)],
    hover_glyph: [J(q(Glyph)), null],
    nonselection_glyph: [V(q(Glyph), H, G), "auto"],
    selection_glyph: [V(q(Glyph), H, G), "auto"],
    muted_glyph: [V(q(Glyph), H, G), "auto"],
    muted: [_, !1]
  }));
  let GlyphRenderer = q3;
  const Xg = class Xg extends ColorMapper {
    constructor(H) {
      super(H);
      mt(this, "MatricsType");
      mt(this, "_scan_data", null);
    }
    connect_signals() {
      super.connect_signals();
      const H = () => {
        for (const [ne] of this.domain)
          this.connect(ne.view.change, () => this.update_data()), this.connect(ne.data_source.selected.change, () => this.update_data());
      }, { high: V, low: q, high_color: G, low_color: J, palette: Z, nan_color: te } = this.properties;
      this.on_change([V, q, G, J, Z, te], () => this.update_data()), this.connect(this.properties.domain.change, () => H()), H();
    }
    update_data() {
      const { domain: H, palette: V } = this, q = [...this._collect(H)];
      this._scan_data = this.scan(q, V.length), this.metrics_change.emit(), this.change.emit();
    }
    get metrics() {
      return this._scan_data == null && this.update_data(), this._scan_data;
    }
    *_collect(H) {
      for (const [V, q] of H)
        for (const G of isArray$3(q) ? q : [q]) {
          if (V.view.properties.indices.is_unset)
            continue;
          const J = V.data_source.get_column(G);
          if (J == null)
            continue;
          let Z = V.view.indices.select(J);
          const te = V.view.masked, ne = V.data_source.selected.indices;
          let ee;
          if (te != null && ne.length > 0 ? ee = intersection$1([...te], ne) : te != null ? ee = [...te] : ne.length > 0 && (ee = ne), ee != null && (Z = map$4(ee, (ae) => Z[ae])), Z.length > 0 && !isNumber$3(Z[0]))
            for (const ae of Z)
              yield* ae;
          else
            yield* Z;
        }
    }
    _v_compute(H, V, q, G) {
      const { nan_color: J } = G;
      let { low_color: Z, high_color: te } = G;
      Z == null && (Z = q[0]), te == null && (te = q[q.length - 1]);
      const { domain: ne } = this, ee = is_empty$1(ne) ? H : [...this._collect(ne)];
      this._scan_data = this.scan(ee, q.length), this.metrics_change.emit();
      for (let ae = 0, oe = H.length; ae < oe; ae++) {
        const se = H[ae];
        isNaN(se) ? V[ae] = J : V[ae] = this.cmap(se, q, Z, te);
      }
    }
    _colors(H) {
      return {
        ...super._colors(H),
        low_color: this.low_color != null ? H(this.low_color) : void 0,
        high_color: this.high_color != null ? H(this.high_color) : void 0
      };
    }
    cmap(H, V, q, G) {
      const J = this.value_to_index(H, V.length);
      return J < 0 ? q : J >= V.length ? G : V[J];
    }
  };
  mt(Xg, "__name__", "ContinuousColorMapper"), Xg.define(({ Float: H, Str: V, Ref: q, Color: G, Or: J, Tuple: Z, List: te, Nullable: ne }) => ({
    high: [ne(H), null],
    low: [ne(H), null],
    high_color: [ne(G), null],
    low_color: [ne(G), null],
    domain: [te(Z(q(GlyphRenderer), J(V, te(V)))), []]
  }));
  let ContinuousColorMapper = Xg;
  class ScanningColorMapper extends ContinuousColorMapper {
    constructor(_) {
      super(_);
    }
    index_to_value(_) {
      return this._scan_data.binning[_];
    }
    value_to_index(_, H) {
      const V = this._scan_data;
      return _ < V.binning[0] ? -1 : _ > V.binning[V.binning.length - 1] ? H : left_edge_index(_, V.binning);
    }
  }
  mt(ScanningColorMapper, "__name__", "ScanningColorMapper");
  const Zg = class Zg extends Ticker {
    constructor(_) {
      super(_);
    }
    get_ticks(_, H, V, q) {
      const { binning: G } = this.mapper.metrics, J = Math.max(0, left_edge_index(_, G)), Z = Math.min(left_edge_index(H, G) + 1, G.length - 1), te = [];
      for (let se = J; se <= Z; se++)
        te.push(G[se]);
      const { num_major_ticks: ne } = this, ee = [], ae = ne == "auto" ? te.length : ne, oe = Math.max(1, Math.floor(te.length / ae));
      for (let se = 0; se < te.length; se += oe)
        ee.push(te[se]);
      return {
        major: ee,
        minor: []
      };
    }
  };
  mt(Zg, "__name__", "BinnedTicker"), Zg.define(({ Float: _, Ref: H, Or: V, Auto: q }) => ({
    mapper: [H(ScanningColorMapper)],
    num_major_ticks: [V(_, q), 8]
  }));
  let BinnedTicker = Zg;
  class CategoricalTickFormatter extends TickFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H) {
      return copy$3(_);
    }
  }
  mt(CategoricalTickFormatter, "__name__", "CategoricalTickFormatter");
  var numbro = { exports: {} };
  /*!
   * numbro.js
   * version : 1.6.2
   * author : Företagsplatsen AB
   * license : MIT
   * http://www.foretagsplatsen.se
   */
  var hasRequiredNumbro;
  function requireNumbro() {
    return hasRequiredNumbro || (hasRequiredNumbro = 1, function(O) {
      var _, H = "1.6.2", V = {}, q = V, G = "en-US", J = null, Z = "0,0", te = {
        delimiters: {
          thousands: ",",
          decimal: "."
        },
        abbreviations: {
          thousand: "k",
          million: "m",
          billion: "b",
          trillion: "t"
        },
        ordinal: function(pe) {
          var ye = pe % 10;
          return ~~(pe % 100 / 10) === 1 ? "th" : ye === 1 ? "st" : ye === 2 ? "nd" : ye === 3 ? "rd" : "th";
        },
        currency: {
          symbol: "$",
          position: "prefix"
        },
        defaults: {
          currencyFormat: ",0000 a"
        },
        formats: {
          fourDigits: "0000 a",
          fullWithTwoDecimals: "$ ,0.00",
          fullWithTwoDecimalsNoCurrency: ",0.00"
        }
      };
      function ne(pe) {
        this._value = pe;
      }
      function ee(pe) {
        var ye, ge = "";
        for (ye = 0; ye < pe; ye++)
          ge += "0";
        return ge;
      }
      function ae(pe, ye) {
        var ge, ve, be, Ce, Se;
        return Se = pe.toString(), ge = Se.split("e")[0], Ce = Se.split("e")[1], ve = ge.split(".")[0], be = ge.split(".")[1] || "", Se = ve + be + ee(Ce - be.length), ye > 0 && (Se += "." + ee(ye)), Se;
      }
      function oe(pe, ye, ge, ve) {
        var be = Math.pow(10, ye), Ce, Se;
        return pe.toFixed(0).search("e") > -1 ? Se = ae(pe, ye) : Se = (ge(pe * be) / be).toFixed(ye), ve && (Ce = new RegExp("0{1," + ve + "}$"), Se = Se.replace(Ce, "")), Se;
      }
      function se(pe, ye, ge) {
        var ve;
        return ye.indexOf("$") > -1 ? ve = ce(pe, ye, ge) : ye.indexOf("%") > -1 ? ve = le(pe, ye, ge) : ye.indexOf(":") > -1 ? ve = ue(pe) : ve = de(pe, ye, ge), ve;
      }
      function ce(pe, ye, ge) {
        var ve = ye, be = ve.indexOf("$"), Ce = ve.indexOf("("), Se = ve.indexOf("+"), we = ve.indexOf("-"), Ee = "", xe = "", Le, Qe;
        if (ve.indexOf("$") === -1 ? V[G].currency.position === "infix" ? (xe = V[G].currency.symbol, V[G].currency.spaceSeparated && (xe = " " + xe + " ")) : V[G].currency.spaceSeparated && (Ee = " ") : ve.indexOf(" $") > -1 ? (Ee = " ", ve = ve.replace(" $", "")) : ve.indexOf("$ ") > -1 ? (Ee = " ", ve = ve.replace("$ ", "")) : ve = ve.replace("$", ""), Qe = de(pe, ve, ge, xe), ye.indexOf("$") === -1)
          switch (V[G].currency.position) {
            case "postfix":
              Qe.indexOf(")") > -1 ? (Qe = Qe.split(""), Qe.splice(-1, 0, Ee + V[G].currency.symbol), Qe = Qe.join("")) : Qe = Qe + Ee + V[G].currency.symbol;
              break;
            case "infix":
              break;
            case "prefix":
              Qe.indexOf("(") > -1 || Qe.indexOf("-") > -1 ? (Qe = Qe.split(""), Le = Math.max(Ce, we) + 1, Qe.splice(Le, 0, V[G].currency.symbol + Ee), Qe = Qe.join("")) : Qe = V[G].currency.symbol + Ee + Qe;
              break;
            default:
              throw Error('Currency position should be among ["prefix", "infix", "postfix"]');
          }
        else
          be <= 1 ? Qe.indexOf("(") > -1 || Qe.indexOf("+") > -1 || Qe.indexOf("-") > -1 ? (Qe = Qe.split(""), Le = 1, (be < Ce || be < Se || be < we) && (Le = 0), Qe.splice(Le, 0, V[G].currency.symbol + Ee), Qe = Qe.join("")) : Qe = V[G].currency.symbol + Ee + Qe : Qe.indexOf(")") > -1 ? (Qe = Qe.split(""), Qe.splice(-1, 0, Ee + V[G].currency.symbol), Qe = Qe.join("")) : Qe = Qe + Ee + V[G].currency.symbol;
        return Qe;
      }
      function le(pe, ye, ge) {
        var ve = "", be;
        return pe = pe * 100, ye.indexOf(" %") > -1 ? (ve = " ", ye = ye.replace(" %", "")) : ye = ye.replace("%", ""), be = de(pe, ye, ge), be.indexOf(")") > -1 ? (be = be.split(""), be.splice(-1, 0, ve + "%"), be = be.join("")) : be = be + ve + "%", be;
      }
      function ue(pe) {
        var ye = Math.floor(pe / 60 / 60), ge = Math.floor((pe - ye * 60 * 60) / 60), ve = Math.round(pe - ye * 60 * 60 - ge * 60);
        return ye + ":" + (ge < 10 ? "0" + ge : ge) + ":" + (ve < 10 ? "0" + ve : ve);
      }
      function de(pe, ye, ge, ve) {
        var be = !1, Ce = !1, Se = !1, we = "", Ee, xe = !1, Le = !1, Qe = !1, ke = !1, Me = !1, Re = "", De = "", Pe = Math.abs(pe), $e = ["B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"], qe = ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"], Xe, Ve, Ze, Ge, ze, We, at, ct, et, dt, Je, it, st, gt = "", bt = !1, yt = !1, St, Lt, Rt, vt = "", $t;
        if (pe === 0 && J !== null)
          return J;
        if (!isFinite(pe))
          return "" + pe;
        if (ye.indexOf("{") === 0) {
          var Pt = ye.indexOf("}");
          if (Pt === -1)
            throw Error('Format should also contain a "}"');
          Je = ye.slice(1, Pt), ye = ye.slice(Pt + 1);
        } else
          Je = "";
        if (ye.indexOf("}") === ye.length - 1) {
          var Xt = ye.indexOf("{");
          if (Xt === -1)
            throw Error('Format should also contain a "{"');
          it = ye.slice(Xt + 1, -1), ye = ye.slice(0, Xt + 1);
        } else
          it = "";
        var Nt;
        if (ye.indexOf(".") === -1 ? Nt = ye.match(/([0-9]+).*/) : Nt = ye.match(/([0-9]+)\..*/), $t = Nt === null ? -1 : Nt[1].length, ye.indexOf("-") !== -1 && (bt = !0), ye.indexOf("(") > -1 ? (be = !0, ye = ye.slice(1, -1)) : ye.indexOf("+") > -1 && (Ce = !0, ye = ye.replace(/\+/g, "")), ye.indexOf("a") > -1) {
          if (et = ye.split(".")[0].match(/[0-9]+/g) || ["0"], et = parseInt(et[0], 10), xe = ye.indexOf("aK") >= 0, Le = ye.indexOf("aM") >= 0, Qe = ye.indexOf("aB") >= 0, ke = ye.indexOf("aT") >= 0, Me = xe || Le || Qe || ke, ye.indexOf(" a") > -1 ? (we = " ", ye = ye.replace(" a", "")) : ye = ye.replace("a", ""), Ge = Math.floor(Math.log(Pe) / Math.LN10) + 1, We = Ge % 3, We = We === 0 ? 3 : We, et && Pe !== 0 && (ze = Math.floor(Math.log(Pe) / Math.LN10) + 1 - et, at = 3 * ~~((Math.min(et, Ge) - We) / 3), Pe = Pe / Math.pow(10, at), ye.indexOf(".") === -1 && et > 3))
            for (ye += "[.]", Lt = ze === 0 ? 0 : 3 * ~~(ze / 3) - ze, Lt = Lt < 0 ? Lt + 3 : Lt, Ee = 0; Ee < Lt; Ee++)
              ye += "0";
          Math.floor(Math.log(Math.abs(pe)) / Math.LN10) + 1 !== et && (Pe >= Math.pow(10, 12) && !Me || ke ? (we = we + V[G].abbreviations.trillion, pe = pe / Math.pow(10, 12)) : Pe < Math.pow(10, 12) && Pe >= Math.pow(10, 9) && !Me || Qe ? (we = we + V[G].abbreviations.billion, pe = pe / Math.pow(10, 9)) : Pe < Math.pow(10, 9) && Pe >= Math.pow(10, 6) && !Me || Le ? (we = we + V[G].abbreviations.million, pe = pe / Math.pow(10, 6)) : (Pe < Math.pow(10, 6) && Pe >= Math.pow(10, 3) && !Me || xe) && (we = we + V[G].abbreviations.thousand, pe = pe / Math.pow(10, 3)));
        }
        if (ye.indexOf("b") > -1) {
          for (ye.indexOf(" b") > -1 ? (Re = " ", ye = ye.replace(" b", "")) : ye = ye.replace("b", ""), Ze = 0; Ze <= $e.length; Ze++)
            if (Xe = Math.pow(1024, Ze), Ve = Math.pow(1024, Ze + 1), pe >= Xe && pe < Ve) {
              Re = Re + $e[Ze], Xe > 0 && (pe = pe / Xe);
              break;
            }
        }
        if (ye.indexOf("d") > -1) {
          for (ye.indexOf(" d") > -1 ? (Re = " ", ye = ye.replace(" d", "")) : ye = ye.replace("d", ""), Ze = 0; Ze <= qe.length; Ze++)
            if (Xe = Math.pow(1e3, Ze), Ve = Math.pow(1e3, Ze + 1), pe >= Xe && pe < Ve) {
              Re = Re + qe[Ze], Xe > 0 && (pe = pe / Xe);
              break;
            }
        }
        if (ye.indexOf("o") > -1 && (ye.indexOf(" o") > -1 ? (De = " ", ye = ye.replace(" o", "")) : ye = ye.replace("o", ""), V[G].ordinal && (De = De + V[G].ordinal(pe))), ye.indexOf("[.]") > -1 && (Se = !0, ye = ye.replace("[.]", ".")), ct = pe.toString().split(".")[0], dt = ye.split(".")[1], st = ye.indexOf(","), dt) {
          if (dt.indexOf("*") !== -1 ? gt = oe(pe, pe.toString().split(".")[1].length, ge) : dt.indexOf("[") > -1 ? (dt = dt.replace("]", ""), dt = dt.split("["), gt = oe(
            pe,
            dt[0].length + dt[1].length,
            ge,
            dt[1].length
          )) : gt = oe(pe, dt.length, ge), ct = gt.split(".")[0], gt.split(".")[1].length) {
            var Gt = ve ? we + ve : V[G].delimiters.decimal;
            gt = Gt + gt.split(".")[1];
          } else
            gt = "";
          Se && Number(gt.slice(1)) === 0 && (gt = "");
        } else
          ct = oe(pe, null, ge);
        return ct.indexOf("-") > -1 && (ct = ct.slice(1), yt = !0), ct.length < $t && (ct = new Array($t - ct.length + 1).join("0") + ct), st > -1 && (ct = ct.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1" + V[G].delimiters.thousands)), ye.indexOf(".") === 0 && (ct = ""), St = ye.indexOf("("), Rt = ye.indexOf("-"), St < Rt ? vt = (be && yt ? "(" : "") + (bt && yt || !be && yt ? "-" : "") : vt = (bt && yt || !be && yt ? "-" : "") + (be && yt ? "(" : ""), Je + vt + (!yt && Ce && pe !== 0 ? "+" : "") + ct + gt + (De || "") + (we && !ve ? we : "") + (Re || "") + (be && yt ? ")" : "") + it;
      }
      _ = function(pe) {
        return _.isNumbro(pe) ? pe = pe.value() : pe === 0 || typeof pe > "u" ? pe = 0 : Number(pe) || (pe = _.fn.unformat(pe)), new ne(Number(pe));
      }, _.version = H, _.isNumbro = function(pe) {
        return pe instanceof ne;
      }, _.setLanguage = function(pe, ye) {
        console.warn("`setLanguage` is deprecated since version 1.6.0. Use `setCulture` instead");
        var ge = pe, ve = pe.split("-")[0], be = null;
        q[ge] || (Object.keys(q).forEach(function(Ce) {
          !be && Ce.split("-")[0] === ve && (be = Ce);
        }), ge = be || ye || "en-US"), he(ge);
      }, _.setCulture = function(pe, ye) {
        var ge = pe, ve = pe.split("-")[1], be = null;
        V[ge] || (ve && Object.keys(V).forEach(function(Ce) {
          !be && Ce.split("-")[1] === ve && (be = Ce);
        }), ge = be || ye || "en-US"), he(ge);
      }, _.language = function(pe, ye) {
        if (console.warn("`language` is deprecated since version 1.6.0. Use `culture` instead"), !pe)
          return G;
        if (pe && !ye) {
          if (!q[pe])
            throw new Error("Unknown language : " + pe);
          he(pe);
        }
        return (ye || !q[pe]) && fe(pe, ye), _;
      }, _.culture = function(pe, ye) {
        if (!pe)
          return G;
        if (pe && !ye) {
          if (!V[pe])
            throw new Error("Unknown culture : " + pe);
          he(pe);
        }
        return (ye || !V[pe]) && fe(pe, ye), _;
      }, _.languageData = function(pe) {
        if (console.warn("`languageData` is deprecated since version 1.6.0. Use `cultureData` instead"), !pe)
          return q[G];
        if (!q[pe])
          throw new Error("Unknown language : " + pe);
        return q[pe];
      }, _.cultureData = function(pe) {
        if (!pe)
          return V[G];
        if (!V[pe])
          throw new Error("Unknown culture : " + pe);
        return V[pe];
      }, _.culture("en-US", te), _.languages = function() {
        return console.warn("`languages` is deprecated since version 1.6.0. Use `cultures` instead"), q;
      }, _.cultures = function() {
        return V;
      }, _.zeroFormat = function(pe) {
        J = typeof pe == "string" ? pe : null;
      }, _.defaultFormat = function(pe) {
        Z = typeof pe == "string" ? pe : "0.0";
      }, _.defaultCurrencyFormat = function(pe) {
      }, _.validate = function(pe, ye) {
        var ge, ve, be, Ce, Se, we, Ee, xe;
        if (typeof pe != "string" && (pe += "", console.warn && console.warn("Numbro.js: Value is not string. It has been co-erced to: ", pe)), pe = pe.trim(), pe.match(/^\d+$/))
          return !0;
        if (pe === "")
          return !1;
        try {
          Ee = _.cultureData(ye);
        } catch {
          Ee = _.cultureData(_.culture());
        }
        return be = Ee.currency.symbol, Se = Ee.abbreviations, ge = Ee.delimiters.decimal, Ee.delimiters.thousands === "." ? ve = "\\." : ve = Ee.delimiters.thousands, xe = pe.match(/^[^\d]+/), xe !== null && (pe = pe.substr(1), xe[0] !== be) || (xe = pe.match(/[^\d]+$/), xe !== null && (pe = pe.slice(0, -1), xe[0] !== Se.thousand && xe[0] !== Se.million && xe[0] !== Se.billion && xe[0] !== Se.trillion)) ? !1 : (we = new RegExp(ve + "{2}"), pe.match(/[^\d.,]/g) ? !1 : (Ce = pe.split(ge), Ce.length > 2 ? !1 : Ce.length < 2 ? !!Ce[0].match(/^\d+.*\d$/) && !Ce[0].match(we) : Ce[0].length === 1 ? !!Ce[0].match(/^\d+$/) && !Ce[0].match(we) && !!Ce[1].match(/^\d+$/) : !!Ce[0].match(/^\d+.*\d$/) && !Ce[0].match(we) && !!Ce[1].match(/^\d+$/)));
      };
      function fe(pe, ye) {
        V[pe] = ye;
      }
      function he(pe) {
        G = pe;
        var ye = V[pe].defaults;
        ye && ye.format && _.defaultFormat(ye.format), ye && ye.currencyFormat && _.defaultCurrencyFormat(ye.currencyFormat);
      }
      function me(pe, ye, ge, ve) {
        return ge != null && ge !== _.culture() && _.setCulture(ge), se(
          Number(pe),
          ye ?? Z,
          ve ?? Math.round
        );
      }
      O.exports = { format: me };
    }(numbro)), numbro.exports;
  }
  var numbroExports = requireNumbro(), sprintf$1 = {}, hasRequiredSprintf;
  function requireSprintf() {
    return hasRequiredSprintf || (hasRequiredSprintf = 1, function(O) {
      (function() {
        var _ = {
          not_string: /[^s]/,
          not_bool: /[^t]/,
          not_type: /[^T]/,
          not_primitive: /[^v]/,
          number: /[diefg]/,
          numeric_arg: /[bcdiefguxX]/,
          json: /[j]/,
          not_json: /[^j]/,
          text: /^[^\x25]+/,
          modulo: /^\x25{2}/,
          placeholder: /^\x25(?:([1-9]\d*)\$|\(([^)]+)\))?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-gijostTuvxX])/,
          key: /^([a-z_][a-z_\d]*)/i,
          key_access: /^\.([a-z_][a-z_\d]*)/i,
          index_access: /^\[(\d+)\]/,
          sign: /^[+-]/
        };
        function H(Z) {
          return q(J(Z), arguments);
        }
        function V(Z, te) {
          return H.apply(null, [Z].concat(te || []));
        }
        function q(Z, te) {
          var ne = 1, ee = Z.length, ae, oe = "", se, ce, le, ue, de, fe, he, me;
          for (se = 0; se < ee; se++)
            if (typeof Z[se] == "string")
              oe += Z[se];
            else if (typeof Z[se] == "object") {
              if (le = Z[se], le.keys)
                for (ae = te[ne], ce = 0; ce < le.keys.length; ce++) {
                  if (ae == null)
                    throw new Error(H('[sprintf] Cannot access property "%s" of undefined value "%s"', le.keys[ce], le.keys[ce - 1]));
                  ae = ae[le.keys[ce]];
                }
              else le.param_no ? ae = te[le.param_no] : ae = te[ne++];
              if (_.not_type.test(le.type) && _.not_primitive.test(le.type) && ae instanceof Function && (ae = ae()), _.numeric_arg.test(le.type) && typeof ae != "number" && isNaN(ae))
                throw new TypeError(H("[sprintf] expecting number but found %T", ae));
              switch (_.number.test(le.type) && (he = ae >= 0), le.type) {
                case "b":
                  ae = parseInt(ae, 10).toString(2);
                  break;
                case "c":
                  ae = String.fromCharCode(parseInt(ae, 10));
                  break;
                case "d":
                case "i":
                  ae = parseInt(ae, 10);
                  break;
                case "j":
                  ae = JSON.stringify(ae, null, le.width ? parseInt(le.width) : 0);
                  break;
                case "e":
                  ae = le.precision ? parseFloat(ae).toExponential(le.precision) : parseFloat(ae).toExponential();
                  break;
                case "f":
                  ae = le.precision ? parseFloat(ae).toFixed(le.precision) : parseFloat(ae);
                  break;
                case "g":
                  ae = le.precision ? String(Number(ae.toPrecision(le.precision))) : parseFloat(ae);
                  break;
                case "o":
                  ae = (parseInt(ae, 10) >>> 0).toString(8);
                  break;
                case "s":
                  ae = String(ae), ae = le.precision ? ae.substring(0, le.precision) : ae;
                  break;
                case "t":
                  ae = String(!!ae), ae = le.precision ? ae.substring(0, le.precision) : ae;
                  break;
                case "T":
                  ae = Object.prototype.toString.call(ae).slice(8, -1).toLowerCase(), ae = le.precision ? ae.substring(0, le.precision) : ae;
                  break;
                case "u":
                  ae = parseInt(ae, 10) >>> 0;
                  break;
                case "v":
                  ae = ae.valueOf(), ae = le.precision ? ae.substring(0, le.precision) : ae;
                  break;
                case "x":
                  ae = (parseInt(ae, 10) >>> 0).toString(16);
                  break;
                case "X":
                  ae = (parseInt(ae, 10) >>> 0).toString(16).toUpperCase();
                  break;
              }
              _.json.test(le.type) ? oe += ae : (_.number.test(le.type) && (!he || le.sign) ? (me = he ? "+" : "-", ae = ae.toString().replace(_.sign, "")) : me = "", de = le.pad_char ? le.pad_char === "0" ? "0" : le.pad_char.charAt(1) : " ", fe = le.width - (me + ae).length, ue = le.width && fe > 0 ? de.repeat(fe) : "", oe += le.align ? me + ae + ue : de === "0" ? me + ue + ae : ue + me + ae);
            }
          return oe;
        }
        var G = /* @__PURE__ */ Object.create(null);
        function J(Z) {
          if (G[Z])
            return G[Z];
          for (var te = Z, ne, ee = [], ae = 0; te; ) {
            if ((ne = _.text.exec(te)) !== null)
              ee.push(ne[0]);
            else if ((ne = _.modulo.exec(te)) !== null)
              ee.push("%");
            else if ((ne = _.placeholder.exec(te)) !== null) {
              if (ne[2]) {
                ae |= 1;
                var oe = [], se = ne[2], ce = [];
                if ((ce = _.key.exec(se)) !== null)
                  for (oe.push(ce[1]); (se = se.substring(ce[0].length)) !== ""; )
                    if ((ce = _.key_access.exec(se)) !== null)
                      oe.push(ce[1]);
                    else if ((ce = _.index_access.exec(se)) !== null)
                      oe.push(ce[1]);
                    else
                      throw new SyntaxError("[sprintf] failed to parse named argument key");
                else
                  throw new SyntaxError("[sprintf] failed to parse named argument key");
                ne[2] = oe;
              } else
                ae |= 2;
              if (ae === 3)
                throw new Error("[sprintf] mixing positional and named placeholders is not (yet) supported");
              ee.push(
                {
                  placeholder: ne[0],
                  param_no: ne[1],
                  keys: ne[2],
                  sign: ne[3],
                  pad_char: ne[4],
                  align: ne[5],
                  width: ne[6],
                  precision: ne[7],
                  type: ne[8]
                }
              );
            } else
              throw new SyntaxError("[sprintf] unexpected placeholder");
            te = te.substring(ne[0].length);
          }
          return G[Z] = ee;
        }
        O.sprintf = H, O.vsprintf = V, typeof window < "u" && (window.sprintf = H, window.vsprintf = V);
      })();
    }(sprintf$1)), sprintf$1;
  }
  var sprintfExports = requireSprintf(), timezone = { exports: {} }, hasRequiredTimezone;
  function requireTimezone() {
    return hasRequiredTimezone || (hasRequiredTimezone = 1, function(O) {
      (function(_) {
        O.exports ? O.exports = _() : this.tz = _();
      })(function() {
        function _(se, ce, le) {
          var ue, de = ce.day[1];
          do
            ue = new Date(Date.UTC(le, ce.month, Math.abs(de++)));
          while (ce.day[0] < 7 && ue.getUTCDay() != ce.day[0]);
          return ue = {
            clock: ce.clock,
            sort: ue.getTime(),
            rule: ce,
            save: ce.save * 6e4,
            offset: se.offset
          }, ue[ue.clock] = ue.sort + ce.time * 6e4, ue.posix ? ue.wallclock = ue[ue.clock] + (se.offset + ce.saved) : ue.posix = ue[ue.clock] - (se.offset + ce.saved), ue;
        }
        function H(se, ce, le) {
          var ue, de, fe, he, me = se[se.zone], pe = [], ye, ge, ve, be = new Date(le).getUTCFullYear(), Ce = 1;
          for (ue = 1, de = me.length; ue < de && !(me[ue][ce] <= le); ue++) ;
          if (fe = me[ue], fe.rules) {
            for (ge = se[fe.rules], ve = be + 1; ve >= be - Ce; --ve)
              for (ue = 0, de = ge.length; ue < de; ue++)
                ge[ue].from <= ve && ve <= ge[ue].to ? pe.push(_(fe, ge[ue], ve)) : ge[ue].to < ve && Ce == 1 && (Ce = ve - ge[ue].to);
            for (pe.sort(function(Se, we) {
              return Se.sort - we.sort;
            }), ue = 0, de = pe.length; ue < de; ue++)
              le >= pe[ue][ce] && pe[ue][pe[ue].clock] > fe[pe[ue].clock] && (he = pe[ue]);
          }
          return he && ((ye = /^(.*)\/(.*)$/.exec(fe.format)) ? he.abbrev = ye[he.save ? 2 : 1] : he.abbrev = fe.format.replace(/%s/, he.rule.letter)), he || fe;
        }
        function V(se, ce) {
          return se.zone == "UTC" ? ce : (se.entry = H(se, "posix", ce), ce + se.entry.offset + se.entry.save);
        }
        function q(se, ce) {
          if (se.zone == "UTC") return ce;
          var le, ue;
          return se.entry = le = H(se, "wallclock", ce), ue = ce - le.wallclock, 0 < ue && ue < le.save ? null : ce - le.offset - le.save;
        }
        function G(se, ce, le) {
          var ue = +(le[1] + 1), de = le[2] * ue, fe = te.indexOf(le[3].toLowerCase()), he;
          if (fe > 9)
            ce += de * ee[fe - 10];
          else {
            if (he = new Date(V(se, ce)), fe < 7)
              for (; de; )
                he.setUTCDate(he.getUTCDate() + ue), he.getUTCDay() == fe && (de -= ue);
            else fe == 7 ? he.setUTCFullYear(he.getUTCFullYear() + de) : fe == 8 ? he.setUTCMonth(he.getUTCMonth() + de) : he.setUTCDate(he.getUTCDate() + de);
            (ce = q(se, he.getTime())) == null && (ce = q(se, he.getTime() + 864e5 * ue) - 864e5 * ue);
          }
          return ce;
        }
        function J(se) {
          if (!se.length) return "1.0.23";
          var ce = Object.create(this), le = [], ue, de, fe, he, me;
          for (ue = 0; ue < se.length; ue++)
            if (he = se[ue], Array.isArray(he))
              !ue && !isNaN(he[1]) ? me = he : he.splice.apply(se, [ue--, 1].concat(he));
            else if (isNaN(he)) {
              if (fe = typeof he, fe == "string")
                ~he.indexOf("%") ? ce.format = he : !ue && he == "*" ? me = he : !ue && (fe = /^(\d{4})-(\d{2})-(\d{2})(?:[T\s](\d{2}):(\d{2})(?::(\d{2})(?:\.(\d+))?)?(Z|(([+-])(\d{2}(:\d{2}){0,2})))?)?$/.exec(he)) ? (me = [], me.push.apply(me, fe.slice(1, 8)), fe[9] ? (me.push(fe[10] + 1), me.push.apply(me, fe[11].split(/:/))) : fe[8] && me.push(1)) : /^\w{2,3}_\w{2}$/.test(he) ? ce.locale = he : (fe = ne.exec(he)) ? le.push(fe) : ce.zone = he;
              else if (fe == "function") {
                if (fe = he.call(ce)) return fe;
              } else if (/^\w{2,3}_\w{2}$/.test(he.name))
                ce[he.name] = he;
              else if (he.zones) {
                for (fe in he.zones) ce[fe] = he.zones[fe];
                for (fe in he.rules) ce[fe] = he.rules[fe];
              }
            } else ue || (me = he);
          if (ce[ce.locale] || delete ce.locale, ce[ce.zone] || delete ce.zone, me != null) {
            if (me == "*")
              me = ce.clock();
            else if (Array.isArray(me)) {
              for (fe = [], de = !me[7], ue = 0; ue < 11; ue++) fe[ue] = +(me[ue] || 0);
              --fe[1], me = Date.UTC.apply(Date.UTC, fe) + -fe[7] * (fe[8] * 36e5 + fe[9] * 6e4 + fe[10] * 1e3);
            } else
              me = Math.floor(me);
            if (!isNaN(me)) {
              if (de && (me = q(ce, me)), me == null) return me;
              for (ue = 0, de = le.length; ue < de; ue++)
                me = G(ce, me, le[ue]);
              return ce.format ? (fe = new Date(V(ce, me)), ce.format.replace(
                /%([-0_^]?)(:{0,3})(\d*)(.)/g,
                function(pe, ye, ge, ve, be) {
                  var Ce, Se = "0", we;
                  if (Ce = ce[be]) {
                    for (pe = String(Ce.call(ce, fe, me, ye, ge.length)), (ye || Ce.style) == "_" && (Se = " "), we = ye == "-" ? 0 : Ce.pad || 0; pe.length < we; ) pe = Se + pe;
                    for (we = ye == "-" ? 0 : ve || Ce.pad; pe.length < we; ) pe = Se + pe;
                    be == "N" && we < pe.length && (pe = pe.slice(0, we)), ye == "^" && (pe = pe.toUpperCase());
                  }
                  return pe;
                }
              )) : me;
            }
          }
          return function() {
            return ce.convert(arguments);
          };
        }
        var Z = {
          clock: function() {
            return +/* @__PURE__ */ new Date();
          },
          zone: "UTC",
          entry: { abbrev: "UTC", offset: 0, save: 0 },
          UTC: 1,
          z: function(se, ce, le, ue) {
            var de = this.entry.offset + this.entry.save, fe = Math.abs(de / 1e3), he = [], me = 3600, pe, ye;
            for (pe = 0; pe < 3; pe++)
              he.push(("0" + Math.floor(fe / me)).slice(-2)), fe %= me, me /= 60;
            return le == "^" && !de ? "Z" : (le == "^" && (ue = 3), ue == 3 ? (ye = he.join(":"), ye = ye.replace(/:00$/, ""), le != "^" && (ye = ye.replace(/:00$/, ""))) : ue ? (ye = he.slice(0, ue + 1).join(":"), le == "^" && (ye = ye.replace(/:00$/, ""))) : ye = he.slice(0, 2).join(""), ye = (de < 0 ? "-" : "+") + ye, ye = ye.replace(/([-+])(0)/, { _: " $1", "-": "$1" }[le] || "$1$2"), ye);
          },
          "%": function(se) {
            return "%";
          },
          n: function(se) {
            return `
`;
          },
          t: function(se) {
            return "	";
          },
          U: function(se) {
            return ae(se, 0);
          },
          W: function(se) {
            return ae(se, 1);
          },
          V: function(se) {
            return oe(se)[0];
          },
          G: function(se) {
            return oe(se)[1];
          },
          g: function(se) {
            return oe(se)[1] % 100;
          },
          j: function(se) {
            return Math.floor((se.getTime() - Date.UTC(se.getUTCFullYear(), 0)) / 864e5) + 1;
          },
          s: function(se) {
            return Math.floor(se.getTime() / 1e3);
          },
          C: function(se) {
            return Math.floor(se.getUTCFullYear() / 100);
          },
          N: function(se) {
            return se.getTime() % 1e3 * 1e6;
          },
          m: function(se) {
            return se.getUTCMonth() + 1;
          },
          Y: function(se) {
            return se.getUTCFullYear();
          },
          y: function(se) {
            return se.getUTCFullYear() % 100;
          },
          H: function(se) {
            return se.getUTCHours();
          },
          M: function(se) {
            return se.getUTCMinutes();
          },
          S: function(se) {
            return se.getUTCSeconds();
          },
          e: function(se) {
            return se.getUTCDate();
          },
          d: function(se) {
            return se.getUTCDate();
          },
          u: function(se) {
            return se.getUTCDay() || 7;
          },
          w: function(se) {
            return se.getUTCDay();
          },
          l: function(se) {
            return se.getUTCHours() % 12 || 12;
          },
          I: function(se) {
            return se.getUTCHours() % 12 || 12;
          },
          k: function(se) {
            return se.getUTCHours();
          },
          Z: function(se) {
            return this.entry.abbrev;
          },
          a: function(se) {
            return this[this.locale].day.abbrev[se.getUTCDay()];
          },
          A: function(se) {
            return this[this.locale].day.full[se.getUTCDay()];
          },
          h: function(se) {
            return this[this.locale].month.abbrev[se.getUTCMonth()];
          },
          b: function(se) {
            return this[this.locale].month.abbrev[se.getUTCMonth()];
          },
          B: function(se) {
            return this[this.locale].month.full[se.getUTCMonth()];
          },
          P: function(se) {
            return this[this.locale].meridiem[Math.floor(se.getUTCHours() / 12)].toLowerCase();
          },
          p: function(se) {
            return this[this.locale].meridiem[Math.floor(se.getUTCHours() / 12)];
          },
          R: function(se, ce) {
            return this.convert([ce, "%H:%M"]);
          },
          T: function(se, ce) {
            return this.convert([ce, "%H:%M:%S"]);
          },
          D: function(se, ce) {
            return this.convert([ce, "%m/%d/%y"]);
          },
          F: function(se, ce) {
            return this.convert([ce, "%Y-%m-%d"]);
          },
          x: function(se, ce) {
            return this.convert([ce, this[this.locale].date]);
          },
          r: function(se, ce) {
            return this.convert([ce, this[this.locale].time12 || "%I:%M:%S"]);
          },
          X: function(se, ce) {
            return this.convert([ce, this[this.locale].time24]);
          },
          c: function(se, ce) {
            return this.convert([ce, this[this.locale].dateTime]);
          },
          convert: J,
          locale: "en_US",
          en_US: {
            date: "%m/%d/%Y",
            time24: "%I:%M:%S %p",
            time12: "%I:%M:%S %p",
            dateTime: "%a %d %b %Y %I:%M:%S %p %Z",
            meridiem: ["AM", "PM"],
            month: {
              abbrev: "Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec".split("|"),
              full: "January|February|March|April|May|June|July|August|September|October|November|December".split("|")
            },
            day: {
              abbrev: "Sun|Mon|Tue|Wed|Thu|Fri|Sat".split("|"),
              full: "Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday".split("|")
            }
          }
        }, te = "Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|year|month|day|hour|minute|second|millisecond", ne = new RegExp("^\\s*([+-])(\\d+)\\s+(" + te + ")s?\\s*$", "i"), ee = [36e5, 6e4, 1e3, 1];
        te = te.toLowerCase().split("|"), "delmHMSUWVgCIky".replace(/./g, function(se) {
          Z[se].pad = 2;
        }), Z.N.pad = 9, Z.j.pad = 3, Z.k.style = "_", Z.l.style = "_", Z.e.style = "_";
        function ae(se, ce) {
          var le, ue, de;
          return ue = new Date(Date.UTC(se.getUTCFullYear(), 0)), le = Math.floor((se.getTime() - ue.getTime()) / 864e5), ue.getUTCDay() == ce ? de = 0 : (de = 7 - ue.getUTCDay() + ce, de == 8 && (de = 1)), le >= de ? Math.floor((le - de) / 7) + 1 : 0;
        }
        function oe(se) {
          var ce, le, ue;
          return le = se.getUTCFullYear(), ce = new Date(Date.UTC(le, 0)).getUTCDay(), ue = ae(se, 1) + (ce > 1 && ce <= 4 ? 1 : 0), ue ? ue == 53 && !(ce == 4 || ce == 3 && new Date(le, 1, 29).getDate() == 29) ? [1, se.getUTCFullYear() + 1] : [ue, se.getUTCFullYear()] : (le = se.getUTCFullYear() - 1, ce = new Date(Date.UTC(le, 0)).getUTCDay(), ue = ce == 4 || ce == 3 && new Date(le, 1, 29).getDate() == 29 ? 53 : 52, [ue, se.getUTCFullYear() - 1]);
        }
        return function() {
          return Z.convert(arguments);
        };
      });
    }(timezone)), timezone.exports;
  }
  var timezoneExports = requireTimezone();
  const tz = /* @__PURE__ */ getDefaultExportFromCjs$1(timezoneExports), FormatterType = Enum("numeral", "printf", "datetime"), DEFAULT_FORMATTERS = {
    numeral: (O, _, H) => numbroExports.format(O, _),
    datetime: (O, _, H) => tz(O, _),
    printf: (O, _, H) => sprintf(_, O)
  };
  function sprintf(O, ..._) {
    return sprintfExports.sprintf(O, ..._);
  }
  function basic_formatter(O, _, H) {
    if (isNumber$3(O)) {
      const V = (() => {
        switch (!1) {
          case Math.floor(O) != O:
            return "%d";
          case (!(Math.abs(O) > 0.1) || !(Math.abs(O) < 1e3)):
            return "%0.3f";
          default:
            return "%0.3e";
        }
      })();
      return sprintf(V, O);
    } else
      return `${O}`;
  }
  function get_formatter(O, _, H) {
    if (_ == null)
      return basic_formatter;
    if (H != null) {
      const V = dict(H).get(O);
      if (V != null) {
        if (isString$2(V)) {
          if (V in DEFAULT_FORMATTERS)
            return DEFAULT_FORMATTERS[V];
          throw new Error(`Unknown tooltip field formatter type '${V}'`);
        }
        return function(q, G, J) {
          return V.format(q, G, J);
        };
      }
    }
    return DEFAULT_FORMATTERS.numeral;
  }
  const MISSING = "???";
  function _get_special_value(O, _) {
    return O in _ ? _[O] : (logger$1.warn(`unknown special variable '$${O}'`), MISSING);
  }
  function _get_column_value(O, _, H) {
    const V = _.get_column(O);
    if (V == null || H == null)
      return null;
    if (isNumber$3(H))
      return V[H];
    const q = V[H.index];
    return isTypedArray$2(q) || isArray$3(q) ? isArray$3(q[0]) ? q[H.j][H.i] : is_NDArray(q) && q.dimension == 3 ? q.slice(H.flat_index * q.shape[2], (H.flat_index + 1) * q.shape[2]) : q[H.flat_index] : q;
  }
  function get_value(O, _, H, V, q) {
    switch (O) {
      case "$":
        return _get_special_value(_, q);
      case "@":
        return _get_column_value(_, H, V);
    }
  }
  function replace_placeholders(O, _, H, V, q = {}, G) {
    let J, Z;
    return isString$2(O) ? (J = O, Z = !1) : (J = O.html, Z = !0), J = J.replace(/@\$name/g, (te) => `@{${q.name}}`), J = process_placeholders(J, (te, ne, ee, ae, oe) => {
      const se = get_value(te, ne, _, H, q);
      if (ee == "safe")
        return Z = !0, se == null ? MISSING : isNumber$3(se) && isNaN(se) ? "NaN" : `${se}`;
      {
        const ce = se == null ? MISSING : isNumber$3(se) && isNaN(se) ? "NaN" : `${get_formatter(oe, ee, V)(se, ee ?? "", q)}`;
        return G != null ? G(ce) : ce;
      }
    }), Z ? [...new DOMParser().parseFromString(J, "text/html").body.childNodes] : J;
  }
  const regex = /((?:[$@][\p{Letter}\p{Number}_]+)|(?:[$@]\{(?:[^{}]+)\}))(?:\{([^{}]+)\})?/gu;
  function process_placeholders(O, _) {
    let H = 0;
    return O.replace(regex, (V, q, G) => {
      const J = q[0], Z = q.substring(1).replace(/^{/, "").replace(/}$/, "").trim();
      return _(J, Z, G, H++, q) ?? MISSING;
    });
  }
  const resolution_order = [
    "microseconds",
    "milliseconds",
    "seconds",
    "minsec",
    "minutes",
    "hourmin",
    "hours",
    "days",
    "months",
    "years"
  ], tm_index_for_resolution = /* @__PURE__ */ new Map();
  for (const O of resolution_order)
    tm_index_for_resolution.set(O, 0);
  tm_index_for_resolution.set("seconds", 5);
  tm_index_for_resolution.set("minsec", 4);
  tm_index_for_resolution.set("minutes", 4);
  tm_index_for_resolution.set("hourmin", 3);
  tm_index_for_resolution.set("hours", 3);
  function _get_resolution(O, _) {
    const H = O * 1.1 * 1e3, V = _ * 1e3;
    return H < ONE_MILLI ? "microseconds" : H < ONE_SECOND ? "milliseconds" : H < ONE_MINUTE ? V >= ONE_MINUTE ? "minsec" : "seconds" : H < ONE_HOUR ? V >= ONE_HOUR ? "hourmin" : "minutes" : H < ONE_DAY ? "hours" : H < ONE_MONTH ? "days" : H < ONE_YEAR ? "months" : "years";
  }
  function _mktime(O) {
    return tz(O, "%Y %m %d %H %M %S").split(/\s+/).map((_) => parseInt(_, 10));
  }
  function _strftime(O, _) {
    const H = sprintf("$1%06d", _us(O));
    return _ = _.replace(/((^|[^%])(%%)*)%f/, H), _.indexOf("%") == -1 ? _ : tz(O, _);
  }
  function _us(O) {
    let _ = Math.round(O / 1e3 % 1 * 1e6);
    return O < 0 && (_ = (1e6 + _) % 1e6), _;
  }
  const j3 = class j3 extends TickFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H, V) {
      if (_.length == 0)
        return [];
      const q = Math.abs(_[_.length - 1] - _[0]) / 1e3, G = q / (_.length - 1), J = is_undefined(V) ? _get_resolution(G, q) : V, Z = [];
      for (const [te, ne] of enumerate(_)) {
        const ee = this._compute_label(te, J), ae = this._add_context(te, ee, ne, _.length, J);
        Z.push(ae);
      }
      return Z;
    }
    _compute_label(_, H) {
      const V = _strftime(_, this[H]), q = _mktime(_), G = resolution_order.indexOf(H);
      let J = V, Z = !1, te = G, ne = H;
      for (; q[tm_index_for_resolution.get(resolution_order[te])] == 0 && (te += 1, te != resolution_order.length); ) {
        if ((H == "minsec" || H == "hourmin") && !Z)
          if (H == "minsec" && q[4] == 0 && q[5] != 0 || H == "hourmin" && q[3] == 0 && q[4] != 0) {
            ne = resolution_order[G - 1], J = _strftime(_, this[ne]);
            break;
          } else
            Z = !0;
        ne = resolution_order[te], J = _strftime(_, this[ne]);
      }
      const ee = ne, { strip_leading_zeros: ae } = this;
      if (isBoolean$4(ae) && ae || isArray$3(ae) && ae.includes(ee)) {
        const oe = J.replace(/^0+/g, "");
        return oe != J && !Number.isInteger(Number(oe[0])) ? `0${oe}` : oe;
      }
      return J;
    }
    _add_context(_, H, V, q, G) {
      const J = this.context_location, Z = this.context_which;
      if (this.context == null)
        return H;
      if (Z == "start" && V == 0 || Z == "end" && V == q - 1 || Z == "center" && V == Math.floor(q / 2) || Z == "all") {
        const te = isString$2(this.context) ? _strftime(_, this.context) : this.context.doFormat([_], { loc: 0 }, G)[0];
        if (te == "")
          return H;
        switch (J) {
          case "above":
            return `${te}
${H}`;
          case "below":
            return `${H}
${te}`;
          case "left":
            return `${te} ${H}`;
          case "right":
            return `${H} ${te}`;
        }
      }
      return H;
    }
  };
  mt(j3, "__name__", "DatetimeTickFormatter"), j3.define(({ Bool: _, Nullable: H, Or: V, Ref: q, Str: G, Arrayable: J }) => ({
    microseconds: [G, "%fus"],
    milliseconds: [G, "%3Nms"],
    seconds: [G, "%Ss"],
    minsec: [G, ":%M:%S"],
    minutes: [G, ":%M"],
    hourmin: [G, "%H:%M"],
    hours: [G, "%Hh"],
    days: [G, "%m/%d"],
    months: [G, "%m/%Y"],
    years: [G, "%Y"],
    strip_leading_zeros: [V(_, J(ResolutionType)), !1],
    context: [H(V(G, q(j3))), null],
    context_which: [ContextWhich, "start"],
    context_location: [Location, "below"]
  }));
  let DatetimeTickFormatter = j3;
  const Kg = class Kg extends TickFormatter {
    constructor(_) {
      super(_);
    }
    get names() {
      return keys$5(this.args);
    }
    get values() {
      return values$3(this.args);
    }
    /*protected*/
    _make_func() {
      const _ = use_strict(this.code);
      return new Function("tick", "index", "ticks", ...this.names, _);
    }
    doFormat(_, H) {
      const V = {}, q = this._make_func().bind(V);
      return _.map((G, J, Z) => `${q(G, J, Z, ...this.values)}`);
    }
  };
  mt(Kg, "__name__", "CustomJSTickFormatter"), Kg.define(({ Unknown: _, Str: H, Dict: V }) => ({
    args: [V(_), {}],
    code: [H, ""]
  }));
  let CustomJSTickFormatter = Kg;
  const { abs: abs$9, log: log$5, round: round$3 } = Math, Jg = class Jg extends TickFormatter {
    constructor(H) {
      super(H);
      mt(this, "basic_formatter");
    }
    initialize() {
      super.initialize(), this.basic_formatter = new BasicTickFormatter();
    }
    format_graphics(H, V) {
      var J;
      if (H.length == 0)
        return [];
      const q = ((J = this.ticker) == null ? void 0 : J.base) ?? 10, G = this._exponents(H, q);
      return G == null ? this.basic_formatter.format_graphics(H, V) : G.map((Z) => {
        if (abs$9(Z) < this.min_exponent) {
          const te = new TextBox({ text: unicode_replace(`${q ** Z}`) }), ne = new TextBox({ text: "" });
          return new BaseExpo(te, ne);
        } else {
          const te = new TextBox({ text: unicode_replace(`${q}`) }), ne = new TextBox({ text: unicode_replace(`${Z}`) });
          return new BaseExpo(te, ne);
        }
      });
    }
    _exponents(H, V) {
      let q = null;
      const G = [];
      for (const J of H) {
        const Z = round$3(log$5(J) / log$5(V));
        if (q != Z)
          q = Z, G.push(Z);
        else
          return null;
      }
      return G;
    }
    doFormat(H, V) {
      var J;
      if (H.length == 0)
        return [];
      const q = ((J = this.ticker) == null ? void 0 : J.base) ?? 10, G = this._exponents(H, q);
      return G == null ? this.basic_formatter.doFormat(H, V) : G.map((Z) => abs$9(Z) < this.min_exponent ? unicode_replace(`${q ** Z}`) : unicode_replace(`${q}^${Z}`));
    }
  };
  mt(Jg, "__name__", "LogTickFormatter"), Jg.define(({ Int: H, Ref: V, Nullable: q }) => ({
    ticker: [q(V(LogTicker)), null],
    min_exponent: [H, 0]
  }));
  let LogTickFormatter = Jg;
  const t5 = class t5 extends BasicTickFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H) {
      if (this.dimension == null)
        throw new Error("MercatorTickFormatter.dimension not configured");
      if (_.length == 0)
        return [];
      const V = _.length, q = new Array(V);
      if (this.dimension == "lon")
        for (let G = 0; G < V; G++) {
          const [J] = wgs84_mercator.invert(_[G], H.loc);
          q[G] = J;
        }
      else
        for (let G = 0; G < V; G++) {
          const [, J] = wgs84_mercator.invert(H.loc, _[G]);
          q[G] = J;
        }
      return super.doFormat(q, H);
    }
  };
  mt(t5, "__name__", "MercatorTickFormatter"), t5.define(({ Nullable: _ }) => ({
    dimension: [_(LatLon), null]
  }));
  let MercatorTickFormatter = t5;
  const n5 = class n5 extends TickFormatter {
    constructor(_) {
      super(_);
    }
    get _rounding_fn() {
      switch (this.rounding) {
        case "round":
        case "nearest":
          return Math.round;
        case "floor":
        case "rounddown":
          return Math.floor;
        case "ceil":
        case "roundup":
          return Math.ceil;
      }
    }
    doFormat(_, H) {
      const { format: V, language: q, _rounding_fn: G } = this;
      return _.map((J) => numbroExports.format(J, V, q, G));
    }
  };
  mt(n5, "__name__", "NumeralTickFormatter"), n5.define(({ Str: _ }) => ({
    // TODO (bev) all of these could be tightened up
    format: [_, "0,0"],
    language: [_, "en"],
    rounding: [RoundingFunction, "round"]
  }));
  let NumeralTickFormatter = n5;
  const r5 = class r5 extends TickFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H) {
      return _.map((V) => sprintf(this.format, V));
    }
  };
  mt(r5, "__name__", "PrintfTickFormatter"), r5.define(({ Str: _ }) => ({
    format: [_, "%s"]
  }));
  let PrintfTickFormatter = r5;
  const i5 = class i5 extends Scale$2 {
    constructor(_) {
      super(_);
    }
    initialize() {
      super.initialize();
      const { source_range: _, target_range: H } = this.properties;
      !_.is_unset && !H.is_unset && (this.linear_scale = new LinearScale({
        source_range: _.get_value(),
        target_range: H.get_value()
      }));
    }
    connect_signals() {
      super.connect_signals();
      const { source_range: _, target_range: H } = this.properties;
      this.on_change([_, H], () => {
        this.linear_scale = new LinearScale({
          source_range: this.source_range,
          target_range: this.target_range
        });
      });
    }
    get s_compute() {
      throw new Error("not implemented");
    }
    get s_invert() {
      throw new Error("not implemented");
    }
    compute(_) {
      return _;
    }
    v_compute(_) {
      const { binning: H } = this, { start: V, end: q } = this.source_range, G = V, J = q, Z = H.length, te = (q - V) / (Z - 1), ne = new Float64Array(Z);
      for (let ae = 0; ae < Z; ae++)
        ne[ae] = V + ae * te;
      const ee = map$4(_, (ae) => {
        if (ae < G)
          return G;
        if (ae > J)
          return J;
        const oe = left_edge_index(ae, H);
        if (oe == -1)
          return G;
        if (oe >= Z - 1)
          return J;
        const se = H[oe], ce = H[oe + 1], le = (ae - se) / (ce - se), ue = ne[oe], de = ne[oe + 1];
        return ue + le * (de - ue);
      });
      return this.linear_scale.v_compute(ee);
    }
    invert(_) {
      return _;
    }
    v_invert(_) {
      return new Float64Array(_);
    }
  };
  mt(i5, "__name__", "LinearInterpolationScale"), i5.internal(({ Float: _, Arrayable: H, Ref: V }) => ({
    binning: [H(_)],
    linear_scale: [V(LinearScale)]
  }));
  let LinearInterpolationScale = i5;
  const { max: max$d, round: round$2 } = Math;
  var ny;
  let Stack$5 = (ny = class extends Layoutable {
    constructor() {
      super(...arguments);
      mt(this, "children", []);
    }
    *[Symbol.iterator]() {
      yield* this.children;
    }
  }, mt(ny, "__name__", "Stack"), ny);
  class HStack extends Stack$5 {
    _measure(_) {
      let H = 0, V = 0;
      for (const q of this.children) {
        const G = q.measure({ width: 0, height: 0 });
        H += G.width, V = max$d(V, G.height);
      }
      return { width: H, height: V };
    }
    _set_geometry(_, H) {
      if (super._set_geometry(_, H), _.is_empty)
        for (const V of this.children)
          V.set_geometry(new BBox$1());
      else {
        const V = this.absolute ? _.top : 0;
        let q = this.absolute ? _.left : 0;
        const { height: G } = _;
        for (const J of this.children) {
          const { width: Z } = J.measure({ width: 0, height: 0 });
          J.set_geometry(new BBox$1({ left: q, width: Z, top: V, height: G })), q += Z;
        }
      }
    }
  }
  mt(HStack, "__name__", "HStack");
  class VStack extends Stack$5 {
    _measure(_) {
      let H = 0, V = 0;
      for (const q of this.children) {
        const G = q.measure({ width: 0, height: 0 });
        H = max$d(H, G.width), V += G.height;
      }
      return { width: H, height: V };
    }
    _set_geometry(_, H) {
      if (super._set_geometry(_, H), _.is_empty)
        for (const V of this.children)
          V.set_geometry(new BBox$1());
      else {
        const V = this.absolute ? _.left : 0;
        let q = this.absolute ? _.top : 0;
        const { width: G } = _;
        for (const J of this.children) {
          const { height: Z } = J.measure({ width: 0, height: 0 });
          J.set_geometry(new BBox$1({ top: q, height: Z, left: V, width: G })), q += Z;
        }
      }
    }
  }
  mt(VStack, "__name__", "VStack");
  class NodeLayout extends Layoutable {
    constructor() {
      super(...arguments);
      mt(this, "children", []);
    }
    *[Symbol.iterator]() {
      yield* this.children;
    }
    _measure(H) {
      const { width_policy: V, height_policy: q } = this.sizing, { min: G, max: J } = Math;
      let Z = 0, te = 0;
      for (const ae of this.children) {
        const { width: oe, height: se } = ae.measure(H);
        Z = J(Z, oe), te = J(te, se);
      }
      const ne = (() => {
        const { width: ae } = this.sizing;
        if (H.width == 1 / 0)
          return V == "fixed" ? ae ?? Z : Z;
        switch (V) {
          case "fixed":
            return ae ?? Z;
          case "min":
            return (
              /*width != null ? min(viewport.width, width) :*/
              Z
            );
          case "fit":
            return ae != null ? G(H.width, ae) : H.width;
          case "max":
            return ae != null ? J(H.width, ae) : H.width;
        }
      })(), ee = (() => {
        const { height: ae } = this.sizing;
        if (H.height == 1 / 0)
          return q == "fixed" ? ae ?? te : te;
        switch (q) {
          case "fixed":
            return ae ?? te;
          case "min":
            return (
              /*height != null ? min(viewport.height, height) :*/
              te
            );
          case "fit":
            return ae != null ? G(H.height, ae) : H.height;
          case "max":
            return ae != null ? J(H.height, ae) : H.height;
        }
      })();
      return { width: ne, height: ee };
    }
    _set_geometry(H, V) {
      if (super._set_geometry(H, V), H.is_empty)
        for (const q of this.children)
          q.set_geometry(new BBox$1());
      else {
        const q = this.absolute ? H : H.relative(), { left: G, right: J, top: Z, bottom: te } = q, ne = round$2(q.vcenter), ee = round$2(q.hcenter);
        for (const ae of this.children) {
          const { margin: oe, halign: se = "start", valign: ce = "start" } = ae.sizing, { width: le, height: ue, inner: de } = ae.measure(H), fe = (() => {
            switch (`${ce}_${se}`) {
              case "start_start":
                return new BBox$1({ left: G + oe.left, top: Z + oe.top, width: le, height: ue });
              case "start_center":
                return new BBox$1({ hcenter: ee, top: Z + oe.top, width: le, height: ue });
              case "start_end":
                return new BBox$1({ right: J - oe.right, top: Z + oe.top, width: le, height: ue });
              case "center_start":
                return new BBox$1({ left: G + oe.left, vcenter: ne, width: le, height: ue });
              case "center_center":
                return new BBox$1({ hcenter: ee, vcenter: ne, width: le, height: ue });
              case "center_end":
                return new BBox$1({ right: J - oe.right, vcenter: ne, width: le, height: ue });
              case "end_start":
                return new BBox$1({ left: G + oe.left, bottom: te - oe.bottom, width: le, height: ue });
              case "end_center":
                return new BBox$1({ hcenter: ee, bottom: te - oe.bottom, width: le, height: ue });
              case "end_end":
                return new BBox$1({ right: J - oe.right, bottom: te - oe.bottom, width: le, height: ue });
            }
          })(), he = de == null ? fe : new BBox$1({
            left: fe.left + de.left,
            top: fe.top + de.top,
            right: fe.right - de.right,
            bottom: fe.bottom - de.bottom
          });
          ae.set_geometry(fe, he);
        }
      }
    }
  }
  mt(NodeLayout, "__name__", "NodeLayout");
  const { max: max$c, round: round$1 } = Math;
  class DefaultMap {
    constructor(_) {
      mt(this, "def");
      mt(this, "_map", /* @__PURE__ */ new Map());
      this.def = _;
    }
    get(_) {
      let H = this._map.get(_);
      return H === void 0 && (H = this.def(), this._map.set(_, H)), H;
    }
    apply(_, H) {
      const V = this.get(_);
      this._map.set(_, H(V));
    }
  }
  mt(DefaultMap, "__name__", "DefaultMap");
  const a5 = class a5 {
    constructor() {
      mt(this, "_items", []);
      mt(this, "_nrows", 0);
      mt(this, "_ncols", 0);
    }
    get size() {
      return this._items.length;
    }
    get nrows() {
      return this._nrows;
    }
    get ncols() {
      return this._ncols;
    }
    add(_, H) {
      const { r1: V, c1: q } = _;
      this._nrows = max$c(this._nrows, V + 1), this._ncols = max$c(this._ncols, q + 1), this._items.push({ span: _, data: H });
    }
    at(_, H) {
      return this._items.filter(({ span: q }) => q.r0 <= _ && _ <= q.r1 && q.c0 <= H && H <= q.c1).map(({ data: q }) => q);
    }
    row(_) {
      return this._items.filter(({ span: V }) => V.r0 <= _ && _ <= V.r1).map(({ data: V }) => V);
    }
    col(_) {
      return this._items.filter(({ span: V }) => V.c0 <= _ && _ <= V.c1).map(({ data: V }) => V);
    }
    *[Symbol.iterator]() {
      yield* this._items;
    }
    foreach(_) {
      for (const { span: H, data: V } of this._items)
        _(H, V);
    }
    map(_) {
      const H = new a5();
      for (const { span: V, data: q } of this._items)
        H.add(V, _(V, q));
      return H;
    }
  };
  mt(a5, "__name__", "Container");
  let Container = a5;
  var ry;
  let Grid$1 = (ry = class extends Layoutable {
    constructor(H = []) {
      super();
      mt(this, "items");
      mt(this, "rows", "auto");
      mt(this, "cols", "auto");
      mt(this, "spacing", 0);
      mt(this, "_state");
      this.items = H;
    }
    *[Symbol.iterator]() {
      for (const { layout: H } of this.items)
        yield H;
    }
    is_width_expanding() {
      if (super.is_width_expanding())
        return !0;
      if (this.sizing.width_policy == "fixed")
        return !1;
      const { cols: H } = this._state;
      return some$2(H, (V) => V.policy == "max");
    }
    is_height_expanding() {
      if (super.is_height_expanding())
        return !0;
      if (this.sizing.height_policy == "fixed")
        return !1;
      const { rows: H } = this._state;
      return some$2(H, (V) => V.policy == "max");
    }
    _init() {
      super._init();
      const H = new Container();
      for (const { layout: ne, row: ee, col: ae, row_span: oe = 1, col_span: se = 1 } of this.items)
        if (ne.sizing.visible) {
          const ce = ee, le = ae, ue = ee + oe - 1, de = ae + se - 1;
          H.add({ r0: ce, c0: le, r1: ue, c1: de }, ne);
        }
      const { nrows: V, ncols: q } = H, G = new Array(V);
      for (let ne = 0; ne < V; ne++) {
        const ee = (() => {
          const oe = isPlainObject(this.rows) ? this.rows[ne] ?? this.rows["*"] : this.rows;
          return oe == null ? { policy: "auto" } : isNumber$3(oe) ? { policy: "fixed", height: oe } : isString$2(oe) ? { policy: oe } : oe;
        })(), ae = ee.align ?? "auto";
        ee.policy == "fixed" ? G[ne] = { policy: "fixed", height: ee.height, align: ae } : ee.policy == "min" ? G[ne] = { policy: "min", align: ae } : ee.policy == "fit" || ee.policy == "max" ? G[ne] = { policy: ee.policy, flex: ee.flex ?? 1, align: ae } : some$2(H.row(ne), (oe) => oe.is_height_expanding()) ? G[ne] = { policy: "max", flex: 1, align: ae } : G[ne] = { policy: "min", align: ae };
      }
      const J = new Array(q);
      for (let ne = 0; ne < q; ne++) {
        const ee = (() => {
          const oe = isPlainObject(this.cols) ? this.cols[ne] ?? this.cols["*"] : this.cols;
          return oe == null ? { policy: "auto" } : isNumber$3(oe) ? { policy: "fixed", width: oe } : isString$2(oe) ? { policy: oe } : oe;
        })(), ae = ee.align ?? "auto";
        ee.policy == "fixed" ? J[ne] = { policy: "fixed", width: ee.width, align: ae } : ee.policy == "min" ? J[ne] = { policy: "min", align: ae } : ee.policy == "fit" || ee.policy == "max" ? J[ne] = { policy: ee.policy, flex: ee.flex ?? 1, align: ae } : some$2(H.col(ne), (oe) => oe.is_width_expanding()) ? J[ne] = { policy: "max", flex: 1, align: ae } : J[ne] = { policy: "min", align: ae };
      }
      const [Z, te] = isNumber$3(this.spacing) ? [this.spacing, this.spacing] : this.spacing;
      this._state = { items: H, nrows: V, ncols: q, rows: G, cols: J, rspacing: Z, cspacing: te };
    }
    _measure_totals(H, V) {
      const { nrows: q, ncols: G, rspacing: J, cspacing: Z } = this._state;
      return {
        height: sum$3(H) + (q - 1) * J,
        width: sum$3(V) + (G - 1) * Z
      };
    }
    _measure_cells(H) {
      const { items: V, nrows: q, ncols: G, rows: J, cols: Z, rspacing: te, cspacing: ne } = this._state, ee = new Array(q);
      for (let ce = 0; ce < q; ce++) {
        const le = J[ce];
        ee[ce] = le.policy == "fixed" ? le.height : 0;
      }
      const ae = new Array(G);
      for (let ce = 0; ce < G; ce++) {
        const le = Z[ce];
        ae[ce] = le.policy == "fixed" ? le.width : 0;
      }
      const oe = new Container();
      return V.foreach((ce, le) => {
        const { r0: ue, c0: de, r1: fe, c1: he } = ce, me = (fe - ue) * te, pe = (he - de) * ne;
        let ye = 0;
        for (let we = ue; we <= fe; we++)
          ye += H(we, de).height;
        ye += me;
        let ge = 0;
        for (let we = de; we <= he; we++)
          ge += H(ue, we).width;
        ge += pe;
        const ve = le.measure({ width: ge, height: ye });
        oe.add(ce, { layout: le, size_hint: ve });
        const be = new Sizeable(ve).grow_by(le.sizing.margin);
        be.height -= me, be.width -= pe;
        const Ce = [];
        for (let we = ue; we <= fe; we++) {
          const Ee = J[we];
          Ee.policy == "fixed" ? be.height -= Ee.height : Ce.push(we);
        }
        if (be.height > 0) {
          const we = round$1(be.height / Ce.length);
          for (const Ee of Ce)
            ee[Ee] = max$c(ee[Ee], we);
        }
        const Se = [];
        for (let we = de; we <= he; we++) {
          const Ee = Z[we];
          Ee.policy == "fixed" ? be.width -= Ee.width : Se.push(we);
        }
        if (be.width > 0) {
          const we = round$1(be.width / Se.length);
          for (const Ee of Se)
            ae[Ee] = max$c(ae[Ee], we);
        }
      }), { size: this._measure_totals(ee, ae), row_heights: ee, col_widths: ae, size_hints: oe };
    }
    _measure_grid(H) {
      const { nrows: V, ncols: q, rows: G, cols: J, rspacing: Z, cspacing: te } = this._state, ne = this._measure_cells((fe, he) => {
        const me = G[fe], pe = J[he];
        return {
          width: pe.policy == "fixed" ? pe.width : 1 / 0,
          height: me.policy == "fixed" ? me.height : 1 / 0
        };
      });
      let ee;
      this.sizing.height_policy == "fixed" && this.sizing.height != null ? ee = this.sizing.height : H.height != 1 / 0 && this.is_height_expanding() ? ee = H.height : ee = ne.size.height;
      let ae = 0;
      for (let fe = 0; fe < V; fe++) {
        const he = G[fe];
        he.policy == "fit" || he.policy == "max" ? ae += he.flex : ee -= ne.row_heights[fe];
      }
      if (ee -= (V - 1) * Z, ae != 0 && ee > 0)
        for (let fe = 0; fe < V; fe++) {
          const he = G[fe];
          if (he.policy == "fit" || he.policy == "max") {
            const me = round$1(ee * (he.flex / ae));
            ee -= me, ne.row_heights[fe] = me, ae -= he.flex;
          }
        }
      else if (ee < 0) {
        let fe = 0;
        for (let me = 0; me < V; me++)
          G[me].policy != "fixed" && fe++;
        let he = -ee;
        for (let me = 0; me < V; me++)
          if (G[me].policy != "fixed") {
            const ye = ne.row_heights[me], ge = round$1(he / fe);
            ne.row_heights[me] = max$c(ye - ge, 0), he -= ge > ye ? ye : ge, fe--;
          }
      }
      let oe;
      this.sizing.width_policy == "fixed" && this.sizing.width != null ? oe = this.sizing.width : H.width != 1 / 0 && this.is_width_expanding() ? oe = H.width : oe = ne.size.width;
      let se = 0;
      for (let fe = 0; fe < q; fe++) {
        const he = J[fe];
        he.policy == "fit" || he.policy == "max" ? se += he.flex : oe -= ne.col_widths[fe];
      }
      if (oe -= (q - 1) * te, se != 0 && oe > 0)
        for (let fe = 0; fe < q; fe++) {
          const he = J[fe];
          if (he.policy == "fit" || he.policy == "max") {
            const me = round$1(oe * (he.flex / se));
            oe -= me, ne.col_widths[fe] = me, se -= he.flex;
          }
        }
      else if (oe < 0) {
        let fe = 0;
        for (let me = 0; me < q; me++)
          J[me].policy != "fixed" && fe++;
        let he = -oe;
        for (let me = 0; me < q; me++)
          if (J[me].policy != "fixed") {
            const ye = ne.col_widths[me], ge = round$1(he / fe);
            ne.col_widths[me] = max$c(ye - ge, 0), he -= ge > ye ? ye : ge, fe--;
          }
      }
      const { row_heights: ce, col_widths: le, size_hints: ue } = this._measure_cells((fe, he) => ({
        width: ne.col_widths[he],
        height: ne.row_heights[fe]
      }));
      return { size: this._measure_totals(ce, le), row_heights: ce, col_widths: le, size_hints: ue };
    }
    _measure(H) {
      const { size: V } = this._measure_grid(H);
      return V;
    }
    _set_geometry(H, V) {
      super._set_geometry(H, V);
      const { nrows: q, ncols: G, rspacing: J, cspacing: Z } = this._state, { row_heights: te, col_widths: ne, size_hints: ee } = this._measure_grid(H), ae = this._state.rows.map((fe, he) => ({ ...fe, top: 0, height: te[he], get bottom() {
        return this.top + this.height;
      } })), oe = this._state.cols.map((fe, he) => ({ ...fe, left: 0, width: ne[he], get right() {
        return this.left + this.width;
      } })), se = ee.map((fe, he) => ({ ...he, outer: new BBox$1(), inner: new BBox$1() }));
      for (let fe = 0, he = this.absolute ? H.top : this.position.top; fe < q; fe++) {
        const me = ae[fe];
        me.top = he, he += me.height + J;
      }
      for (let fe = 0, he = this.absolute ? H.left : this.position.left; fe < G; fe++) {
        const me = oe[fe];
        me.left = he, he += me.width + Z;
      }
      function ce(fe, he) {
        let me = (he - fe) * Z;
        for (let pe = fe; pe <= he; pe++)
          me += oe[pe].width;
        return me;
      }
      function le(fe, he) {
        let me = (he - fe) * J;
        for (let pe = fe; pe <= he; pe++)
          me += ae[pe].height;
        return me;
      }
      se.foreach(({ r0: fe, c0: he, r1: me, c1: pe }, ye) => {
        const { layout: ge, size_hint: ve } = ye, { sizing: be } = ge, { width: Ce, height: Se } = ve, we = {
          width: ce(he, pe),
          height: le(fe, me)
        }, Ee = he == pe && oe[he].align != "auto" ? oe[he].align : be.halign, xe = fe == me && ae[fe].align != "auto" ? ae[fe].align : be.valign;
        let Le = oe[he].left;
        Ee == "start" ? Le += be.margin.left : Ee == "center" ? Le += round$1((we.width - Ce) / 2) : Ee == "end" && (Le += we.width - be.margin.right - Ce);
        let Qe = ae[fe].top;
        xe == "start" ? Qe += be.margin.top : xe == "center" ? Qe += round$1((we.height - Se) / 2) : xe == "end" && (Qe += we.height - be.margin.bottom - Se), ye.outer = new BBox$1({ left: Le, top: Qe, width: Ce, height: Se });
      });
      const ue = ae.map(() => ({
        start: new DefaultMap(() => 0),
        end: new DefaultMap(() => 0)
      })), de = oe.map(() => ({
        start: new DefaultMap(() => 0),
        end: new DefaultMap(() => 0)
      }));
      se.foreach(({ r0: fe, c0: he, r1: me, c1: pe }, { size_hint: ye, outer: ge }) => {
        const { inner: ve } = ye;
        ve != null && (ue[fe].start.apply(ge.top, (be) => max$c(be, ve.top)), ue[me].end.apply(ae[me].bottom - ge.bottom, (be) => max$c(be, ve.bottom)), de[he].start.apply(ge.left, (be) => max$c(be, ve.left)), de[pe].end.apply(oe[pe].right - ge.right, (be) => max$c(be, ve.right)));
      }), se.foreach(({ r0: fe, c0: he, r1: me, c1: pe }, ye) => {
        const { size_hint: ge, outer: ve } = ye, be = (Ce) => {
          const Se = this.absolute ? ve : ve.relative(), we = Se.left + Ce.left, Ee = Se.top + Ce.top, xe = Se.right - Ce.right, Le = Se.bottom - Ce.bottom;
          return new BBox$1({ left: we, top: Ee, right: xe, bottom: Le });
        };
        if (ge.inner != null) {
          let Ce = be(ge.inner);
          const Se = ue[fe].start.get(ve.top), we = ue[me].end.get(ae[me].bottom - ve.bottom), Ee = de[he].start.get(ve.left), xe = de[pe].end.get(oe[pe].right - ve.right);
          try {
            Ce = be({ top: Se, bottom: we, left: Ee, right: xe });
          } catch {
          }
          ye.inner = Ce;
        } else
          ye.inner = ve;
      }), se.foreach((fe, { layout: he, outer: me, inner: pe }) => {
        he.set_geometry(me, pe);
      });
    }
  }, mt(ry, "__name__", "Grid"), ry);
  var iy;
  let Row$2 = (iy = class extends Grid$1 {
    constructor(_) {
      super(), this.items = _.map((H, V) => ({ layout: H, row: 0, col: V })), this.rows = "fit";
    }
  }, mt(iy, "__name__", "Row"), iy);
  var ay;
  let Column$2 = (ay = class extends Grid$1 {
    constructor(_) {
      super(), this.items = _.map((H, V) => ({ layout: H, row: V, col: 0 })), this.cols = "fit";
    }
  }, mt(ay, "__name__", "Column"), ay);
  class BorderLayout extends Layoutable {
    constructor() {
      super(...arguments);
      mt(this, "top_panel");
      mt(this, "bottom_panel");
      mt(this, "left_panel");
      mt(this, "right_panel");
      mt(this, "center_panel");
      mt(this, "inner_top_panel");
      mt(this, "inner_bottom_panel");
      mt(this, "inner_left_panel");
      mt(this, "inner_right_panel");
      mt(this, "aligns", { left: !0, right: !0, top: !0, bottom: !0 });
      mt(this, "min_border", { left: 0, top: 0, right: 0, bottom: 0 });
      mt(this, "padding", { left: 0, top: 0, right: 0, bottom: 0 });
      mt(this, "center_border_width", 0);
    }
    *[Symbol.iterator]() {
      yield this.top_panel, yield this.bottom_panel, yield this.left_panel, yield this.right_panel, yield this.center_panel;
    }
    _measure(H) {
      H = new Sizeable({
        width: this.sizing.width_policy == "fixed" || H.width == 1 / 0 ? this.sizing.width : H.width,
        height: this.sizing.height_policy == "fixed" || H.height == 1 / 0 ? this.sizing.height : H.height
      });
      const V = this.left_panel.measure({ width: 0, height: H.height }), q = Math.max(V.width, this.min_border.left) + this.padding.left, G = this.right_panel.measure({ width: 0, height: H.height }), J = Math.max(G.width, this.min_border.right) + this.padding.right, Z = this.top_panel.measure({ width: H.width, height: 0 }), te = Math.max(Z.height, this.min_border.top) + this.padding.top, ne = this.bottom_panel.measure({ width: H.width, height: 0 }), ee = Math.max(ne.height, this.min_border.bottom) + this.padding.bottom, ae = new Sizeable(H).shrink_by({ left: q, right: J, top: te, bottom: ee }), oe = this.center_panel.measure(ae), se = q + oe.width + J, ce = te + oe.height + ee, le = (() => {
        const { width_policy: ue, height_policy: de } = this.center_panel.sizing;
        return {
          ...this.aligns,
          fixed_width: ue == "fixed",
          fixed_height: de == "fixed"
        };
      })();
      return { width: se, height: ce, inner: { left: q, right: J, top: te, bottom: ee }, align: le };
    }
    _set_geometry(H, V) {
      var q, G, J, Z;
      if (super._set_geometry(H, V), this.sizing.visible) {
        this.center_panel.set_geometry(V);
        const te = this.left_panel.measure({ width: 0, height: H.height }), ne = this.right_panel.measure({ width: 0, height: H.height }), ee = this.top_panel.measure({ width: H.width, height: 0 }), ae = this.bottom_panel.measure({ width: H.width, height: 0 }), { left: oe, top: se, right: ce, bottom: le } = V;
        this.top_panel.set_geometry(new BBox$1({ left: oe, right: ce, bottom: se, height: ee.height })), this.bottom_panel.set_geometry(new BBox$1({ left: oe, right: ce, top: le, height: ae.height })), this.left_panel.set_geometry(new BBox$1({ top: se, bottom: le, right: oe, width: te.width })), this.right_panel.set_geometry(new BBox$1({ top: se, bottom: le, left: ce, width: ne.width }));
        const ue = V.shrink_by(this.center_border_width);
        if (this.inner_top_panel != null) {
          const { left: de, right: fe, top: he, width: me } = ue, pe = this.inner_top_panel.measure({ width: me, height: 0 });
          this.inner_top_panel.set_geometry(new BBox$1({ left: de, right: fe, top: he, height: pe.height }));
        }
        if (this.inner_bottom_panel != null) {
          const { left: de, right: fe, bottom: he, width: me } = ue, pe = this.inner_bottom_panel.measure({ width: me, height: 0 });
          this.inner_bottom_panel.set_geometry(new BBox$1({ left: de, right: fe, bottom: he, height: pe.height }));
        }
        if (this.inner_left_panel != null) {
          const { top: de, bottom: fe, left: he, height: me } = ue, pe = this.inner_left_panel.measure({ width: 0, height: me });
          this.inner_left_panel.set_geometry(new BBox$1({ top: de, bottom: fe, left: he, width: pe.width }));
        }
        if (this.inner_right_panel != null) {
          const { top: de, bottom: fe, right: he, height: me } = ue, pe = this.inner_right_panel.measure({ width: 0, height: me });
          this.inner_right_panel.set_geometry(new BBox$1({ top: de, bottom: fe, right: he, width: pe.width }));
        }
      } else
        this.center_panel.set_geometry(new BBox$1()), this.top_panel.set_geometry(new BBox$1()), this.bottom_panel.set_geometry(new BBox$1()), this.left_panel.set_geometry(new BBox$1()), this.right_panel.set_geometry(new BBox$1()), (q = this.inner_top_panel) == null || q.set_geometry(new BBox$1()), (G = this.inner_bottom_panel) == null || G.set_geometry(new BBox$1()), (J = this.inner_left_panel) == null || J.set_geometry(new BBox$1()), (Z = this.inner_right_panel) == null || Z.set_geometry(new BBox$1());
    }
  }
  mt(BorderLayout, "__name__", "BorderLayout");
  const MINOR_DIM = 25, MAJOR_DIM_MIN_SCALAR = 0.3, MAJOR_DIM_MAX_SCALAR = 0.8;
  class BaseColorBarView extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "_frame");
      mt(this, "_axis");
      mt(this, "_axis_view");
      mt(this, "_title");
      mt(this, "_title_view");
      mt(this, "_ticker");
      mt(this, "_formatter");
      mt(this, "_inner_layout");
      mt(this, "_major_range");
      mt(this, "_major_scale");
      mt(this, "_minor_range");
      mt(this, "_minor_scale");
      mt(this, "_orientation");
    }
    get orientation() {
      return this._orientation;
    }
    *children() {
      yield* super.children(), yield this._axis_view, yield this._title_view;
    }
    initialize() {
      super.initialize();
      const { ticker: H, formatter: V } = this.model;
      this._ticker = H != "auto" ? H : this._create_ticker(), this._formatter = V != "auto" ? V : this._create_formatter(), this._major_range = this._create_major_range(), this._major_scale = this._create_major_scale(), this._minor_range = new Range1d({ start: 0, end: 1 }), this._minor_scale = new LinearScale(), this._frame = new CartesianFrame(this._major_scale, this._minor_scale, this._major_range, this._minor_range), this._axis = this._create_axis(), this._apply_axis_properties(), this._title = new Title(), this._apply_title_properties();
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const H = this, V = {
        get parent() {
          return H.parent;
        },
        get root() {
          return H.root;
        },
        get frame() {
          return H._frame;
        },
        get canvas_view() {
          return H.parent.canvas_view;
        },
        request_layout() {
          H.layout.dirty = !0, H.parent.request_layout();
        },
        request_paint() {
          H.parent.request_paint(H);
        },
        request_render() {
          H.request_paint();
        },
        notify_finished_after_paint() {
          H.parent.notify_finished_after_paint();
        }
      };
      this._axis_view = await build_view(this._axis, { parent: V }), this._title_view = await build_view(this._title, { parent: V });
    }
    remove() {
      this._title_view.remove(), this._axis_view.remove(), super.remove();
    }
    _apply_axis_properties() {
      const H = {
        ticker: this._ticker,
        formatter: this._formatter,
        major_label_standoff: this.model.label_standoff,
        axis_line_color: null,
        major_tick_in: this.model.major_tick_in,
        major_tick_out: this.model.major_tick_out,
        minor_tick_in: this.model.minor_tick_in,
        minor_tick_out: this.model.minor_tick_out,
        major_label_overrides: this.model.major_label_overrides,
        major_label_policy: this.model.major_label_policy,
        // TODO: this needs strict typing
        ...attrs_of(this.model, "major_label_", Text$4, !0),
        ...attrs_of(this.model, "major_tick_", Line$4, !0),
        ...attrs_of(this.model, "minor_tick_", Line$4, !0)
      };
      this._axis.setv(H);
    }
    _apply_title_properties() {
      const H = {
        text: this.model.title ?? "",
        standoff: this.model.title_standoff,
        // TODO: this needs strict typing
        ...attrs_of(this.model, "title_", Text$4, !1)
      };
      this._title.setv(H);
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => {
        this._apply_title_properties(), this._apply_axis_properties();
      }), this.connect(this._ticker.change, () => this.request_render()), this.connect(this._formatter.change, () => this.request_render());
    }
    _update_frame() {
      const [H, V, q, G] = this.orientation == "horizontal" ? [this._major_scale, this._minor_scale, this._major_range, this._minor_range] : [this._minor_scale, this._major_scale, this._minor_range, this._major_range];
      this._frame.in_x_scale = H, this._frame.in_y_scale = V, this._frame.x_range = q, this._frame.y_range = G, this._frame.configure_scales();
    }
    update_layout() {
      const { location: H, width: V, height: q, padding: G, margin: J } = this.model, [Z, te] = (() => {
        if (isString$2(H))
          switch (H) {
            case "top_left":
              return ["start", "start"];
            case "top":
            case "top_center":
              return ["start", "center"];
            case "top_right":
              return ["start", "end"];
            case "bottom_left":
              return ["end", "start"];
            case "bottom":
            case "bottom_center":
              return ["end", "center"];
            case "bottom_right":
              return ["end", "end"];
            case "left":
            case "center_left":
              return ["center", "start"];
            case "center":
            case "center_center":
              return ["center", "center"];
            case "right":
            case "center_right":
              return ["center", "end"];
          }
        else
          return ["end", "start"];
      })(), ne = this._orientation = (() => {
        const { orientation: we } = this.model;
        return we == "auto" ? this.panel != null ? this.panel.is_horizontal ? "horizontal" : "vertical" : te == "start" || te == "end" || /*halign == "center" &&*/
        Z == "center" ? "vertical" : "horizontal" : we;
      })();
      this._update_frame();
      const ee = new NodeLayout(), ae = new VStack(), oe = new VStack(), se = new HStack(), ce = new HStack();
      ee.absolute = !0, ae.absolute = !0, oe.absolute = !0, se.absolute = !0, ce.absolute = !0, ee.on_resize((we) => this._frame.set_geometry(we));
      const le = new BorderLayout();
      this._inner_layout = le, le.absolute = !0, le.center_panel = ee, le.top_panel = ae, le.bottom_panel = oe, le.left_panel = se, le.right_panel = ce;
      const ue = { left: G, right: G, top: G, bottom: G }, de = (() => {
        if (this.panel == null) {
          if (isString$2(H))
            return { left: J, right: J, top: J, bottom: J };
          {
            const [we, Ee] = H;
            return { left: we, right: J, top: J, bottom: Ee };
          }
        } else if (isString$2(H)) {
          le.fixup_geometry = (we, Ee) => {
            const xe = we;
            if (ne == "horizontal") {
              const { top: Le, width: Qe, height: ke } = we;
              if (te == "end") {
                const { right: Me } = this.layout.bbox;
                we = new BBox$1({ right: Me, top: Le, width: Qe, height: ke });
              } else if (te == "center") {
                const { hcenter: Me } = this.layout.bbox;
                we = new BBox$1({ hcenter: Math.round(Me), top: Le, width: Qe, height: ke });
              }
            } else {
              const { left: Le, width: Qe, height: ke } = we;
              if (Z == "end") {
                const { bottom: Me } = this.layout.bbox;
                we = new BBox$1({ left: Le, bottom: Me, width: Qe, height: ke });
              } else if (Z == "center") {
                const { vcenter: Me } = this.layout.bbox;
                we = new BBox$1({ left: Le, vcenter: Math.round(Me), width: Qe, height: ke });
              }
            }
            if (Ee != null) {
              const Le = we.left - xe.left, Qe = we.top - xe.top, { left: ke, top: Me, width: Re, height: De } = Ee;
              Ee = new BBox$1({ left: ke + Le, top: Me + Qe, width: Re, height: De });
            }
            return [we, Ee];
          };
          return;
        } else {
          const [we, Ee] = H;
          return le.fixup_geometry = (xe, Le) => {
            const Qe = xe, ke = this.layout.bbox, { width: Me, height: Re } = xe;
            if (xe = new BBox$1({ left: ke.left + we, bottom: ke.bottom - Ee, width: Me, height: Re }), Le != null) {
              const De = xe.left - Qe.left, Pe = xe.top - Qe.top, { left: $e, top: qe, width: Xe, height: Ve } = Le;
              Le = new BBox$1({ left: $e + De, top: qe + Pe, width: Xe, height: Ve });
            }
            return [xe, Le];
          }, { left: we, right: 0, top: 0, bottom: Ee };
        }
      })();
      le.padding = ue;
      let fe, he, me, pe;
      if (this.panel != null)
        fe = "max", he = void 0, me = void 0, pe = void 0;
      else if ((ne == "horizontal" ? V : q) == "auto") {
        fe = "fixed";
        const we = this._get_major_size_factor();
        we != null && (he = we * MINOR_DIM), me = { percent: MAJOR_DIM_MIN_SCALAR }, pe = { percent: MAJOR_DIM_MAX_SCALAR };
      } else
        fe = "fit", he = void 0;
      if (ne == "horizontal") {
        const we = V == "auto" ? void 0 : V, Ee = q == "auto" ? MINOR_DIM : q;
        le.set_sizing({
          width_policy: fe,
          height_policy: "min",
          width: he,
          min_width: me,
          max_width: pe,
          halign: te,
          valign: Z,
          margin: de
        }), le.center_panel.set_sizing({ width_policy: V == "auto" ? "fit" : "fixed", height_policy: "fixed", width: we, height: Ee });
      } else {
        const we = V == "auto" ? MINOR_DIM : V, Ee = q == "auto" ? void 0 : q;
        le.set_sizing({
          width_policy: "min",
          height_policy: fe,
          height: he,
          min_height: me,
          max_height: pe,
          halign: te,
          valign: Z,
          margin: de
        }), le.center_panel.set_sizing({ width_policy: "fixed", height_policy: q == "auto" ? "fit" : "fixed", width: we, height: Ee });
      }
      ae.set_sizing({ width_policy: "fit", height_policy: "min" }), oe.set_sizing({ width_policy: "fit", height_policy: "min" }), se.set_sizing({ width_policy: "min", height_policy: "fit" }), ce.set_sizing({ width_policy: "min", height_policy: "fit" });
      const { _title_view: ye } = this;
      ne == "horizontal" ? (ye.panel = new SidePanel("above"), ye.update_layout(), ae.children.push(ye.layout)) : (ye.panel = new SidePanel("left"), ye.update_layout(), se.children.push(ye.layout));
      const { panel: ge } = this, ve = ge != null && ne == ge.orientation ? ge.side : ne == "horizontal" ? "below" : "right", be = (() => {
        switch (ve) {
          case "above":
            return ae;
          case "below":
            return oe;
          case "left":
            return se;
          case "right":
            return ce;
        }
      })(), { _axis_view: Ce } = this;
      if (Ce.panel = new SidePanel(ve), Ce.update_layout(), Ce.layout != null && be.children.push(Ce.layout), this.panel != null) {
        const we = new Grid$1([{ layout: le, row: 0, col: 0 }]);
        we.absolute = !0, ne == "horizontal" ? we.set_sizing({ width_policy: "max", height_policy: "min" }) : we.set_sizing({ width_policy: "min", height_policy: "max" }), this.layout = we;
      } else
        this.layout = this._inner_layout;
      const { visible: Se } = this.model;
      this.layout.sizing.visible = Se;
    }
    _create_axis() {
      return new LinearAxis();
    }
    _create_formatter() {
      return new BasicTickFormatter();
    }
    _create_major_range() {
      return new Range1d({ start: 0, end: 1 });
    }
    _create_major_scale() {
      return new LinearScale();
    }
    _create_ticker() {
      return new BasicTicker();
    }
    _get_major_size_factor() {
      return null;
    }
    _render() {
      const { ctx: H } = this.layer;
      H.save(), this._paint_bbox(H, this._inner_layout.bbox), this._paint_colors(H, this._inner_layout.center_panel.bbox), this._title_view.render(), this._axis_view.render(), H.restore();
    }
    _paint_bbox(H, V) {
      const { x: q, y: G } = V;
      let { width: J, height: Z } = V;
      q + J >= this.parent.canvas_view.bbox.width && (J -= 1), G + Z >= this.parent.canvas_view.bbox.height && (Z -= 1), H.save(), this.visuals.background_fill.doit && (this.visuals.background_fill.set_value(H), H.fillRect(q, G, J, Z)), this.visuals.border_line.doit && (this.visuals.border_line.set_value(H), H.strokeRect(q, G, J, Z)), H.restore();
    }
    serializable_state() {
      const { children: H = [], ...V } = super.serializable_state();
      return H.push(this._title_view.serializable_state()), H.push(this._axis_view.serializable_state()), { ...V, children: H };
    }
  }
  mt(BaseColorBarView, "__name__", "BaseColorBarView");
  const xp = class xp extends Annotation$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(xp, "__name__", "BaseColorBar"), xp.mixins([
    ["major_label_", Text$4],
    ["title_", Text$4],
    ["major_tick_", Line$4],
    ["minor_tick_", Line$4],
    ["border_", Line$4],
    ["bar_", Line$4],
    ["background_", Fill$2]
  ]), xp.define(({ Alpha: _, Float: H, Str: V, Tuple: q, Or: G, Ref: J, Auto: Z, Nullable: te }) => ({
    location: [G(Anchor$2, q(H, H)), "top_right"],
    orientation: [G(Orientation, Z), "auto"],
    title: [te(G(V, J(BaseText))), null],
    title_standoff: [H, 2],
    width: [G(H, Z), "auto"],
    height: [G(H, Z), "auto"],
    scale_alpha: [_, 1],
    ticker: [G(J(Ticker), Z), "auto"],
    formatter: [G(J(TickFormatter), Z), "auto"],
    major_label_overrides: [LabelOverrides, /* @__PURE__ */ new Map()],
    major_label_policy: [J(LabelingPolicy), () => new NoOverlap()],
    label_standoff: [H, 5],
    margin: [H, 30],
    padding: [H, 10],
    major_tick_in: [H, 5],
    major_tick_out: [H, 0],
    minor_tick_in: [H, 0],
    minor_tick_out: [H, 0]
  })), xp.override({
    background_fill_color: "#ffffff",
    background_fill_alpha: 0.95,
    bar_line_color: null,
    border_line_color: null,
    major_label_text_font_size: "11px",
    major_tick_line_color: "#ffffff",
    minor_tick_line_color: null,
    title_text_font_size: "13px",
    title_text_font_style: "italic"
  });
  let BaseColorBar = xp;
  class UpperLowerView extends DataAnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "_lower");
      mt(this, "_upper");
      mt(this, "_base");
      mt(this, "_lower_sx");
      mt(this, "_lower_sy");
      mt(this, "_upper_sx");
      mt(this, "_upper_sy");
    }
    map_data() {
      const { frame: H } = this.plot_view, V = this.model.dimension, q = this.coordinates.x_scale, G = this.coordinates.y_scale, J = V == "height" ? G : q, Z = V == "height" ? q : G, te = V == "height" ? H.bbox.yview : H.bbox.xview, ne = V == "height" ? H.bbox.xview : H.bbox.yview, ee = (() => {
        switch (this.model.properties.lower.units) {
          case "canvas":
            return new ScreenArray(this._lower);
          case "screen":
            return te.v_compute(this._lower);
          case "data":
            return J.v_compute(this._lower);
        }
      })(), ae = (() => {
        switch (this.model.properties.upper.units) {
          case "canvas":
            return new ScreenArray(this._upper);
          case "screen":
            return te.v_compute(this._upper);
          case "data":
            return J.v_compute(this._upper);
        }
      })(), oe = (() => {
        switch (this.model.properties.base.units) {
          case "canvas":
            return new ScreenArray(this._base);
          case "screen":
            return ne.v_compute(this._base);
          case "data":
            return Z.v_compute(this._base);
        }
      })(), [se, ce] = V == "height" ? [1, 0] : [0, 1], le = [ee, oe], ue = [ae, oe];
      this._lower_sx = le[se], this._lower_sy = le[ce], this._upper_sx = ue[se], this._upper_sy = ue[ce];
    }
  }
  mt(UpperLowerView, "__name__", "UpperLowerView");
  class XOrYCoordinateSpec extends CoordinateSpec {
    constructor() {
      super(...arguments);
      mt(this, "_value", unset);
    }
    get dimension() {
      return this.obj.dimension == "width" ? "x" : "y";
    }
    // XXX: a hack to make a coordinate & unit spec
    get units() {
      return this._value === unset ? "data" : this._value.units ?? "data";
    }
  }
  mt(XOrYCoordinateSpec, "__name__", "XOrYCoordinateSpec");
  const o5 = class o5 extends DataAnnotation {
    constructor(_) {
      super(_);
    }
  };
  mt(o5, "__name__", "UpperLower"), o5.define(() => ({
    dimension: [Dimension$1, "height"],
    lower: [XOrYCoordinateSpec, { field: "lower" }],
    upper: [XOrYCoordinateSpec, { field: "upper" }],
    base: [XOrYCoordinateSpec, { field: "base" }]
  }));
  let UpperLower = o5;
  class BandView extends UpperLowerView {
    paint(_) {
      _.beginPath(), _.moveTo(this._lower_sx[0], this._lower_sy[0]);
      for (let H = 0, V = this._lower_sx.length; H < V; H++)
        _.lineTo(this._lower_sx[H], this._lower_sy[H]);
      for (let H = this._upper_sx.length - 1; H >= 0; H--)
        _.lineTo(this._upper_sx[H], this._upper_sy[H]);
      _.closePath(), this.visuals.fill.apply(_), _.beginPath(), _.moveTo(this._lower_sx[0], this._lower_sy[0]);
      for (let H = 0, V = this._lower_sx.length; H < V; H++)
        _.lineTo(this._lower_sx[H], this._lower_sy[H]);
      this.visuals.line.apply(_), _.beginPath(), _.moveTo(this._upper_sx[0], this._upper_sy[0]);
      for (let H = 0, V = this._upper_sx.length; H < V; H++)
        _.lineTo(this._upper_sx[H], this._upper_sy[H]);
      this.visuals.line.apply(_);
    }
  }
  mt(BandView, "__name__", "BandView");
  var u2;
  let Band$1 = (u2 = class extends UpperLower {
    constructor(_) {
      super(_);
    }
  }, mt(u2, "__name__", "Band"), u2.prototype.default_view = BandView, u2.mixins([Line$4, Fill$2]), u2.override({
    fill_color: "#fff9ba",
    fill_alpha: 0.4,
    line_color: "#cccccc",
    line_alpha: 0.3
  }), u2);
  const Corner = Enum("top_left", "top_right", "bottom_left", "bottom_right"), Edge = Enum("left", "right", "top", "bottom");
  Enum(...Corner, ...Edge, "area");
  const Resizable = Enum("none", "left", "right", "top", "bottom", "x", "y", "all"), Movable = Enum("none", "x", "y", "both"), Limit = Nullable(Or$1(Float$1, Ref(Node$4))), EDGE_TOLERANCE$1 = 2.5, { abs: abs$8 } = Math;
  var Gy, Wy;
  class BoxAnnotationView extends (Wy = AnnotationView, Gy = auto_ranged, Wy) {
    constructor() {
      super(...arguments);
      mt(this, "bbox", new BBox$1());
      mt(this, Gy, !0);
      mt(this, "_pan_state", null);
      mt(this, "_pinch_state", null);
      mt(this, "_is_hovered", !1);
    }
    serializable_state() {
      return { ...super.serializable_state(), bbox: this.bbox.round() };
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.request_render());
    }
    bounds() {
      const { left: H, left_units: V, right: q, right_units: G, top: J, top_units: Z, bottom: te, bottom_units: ne } = this.model, ee = V == "data" && !(H instanceof Coordinate), ae = G == "data" && !(q instanceof Coordinate), oe = Z == "data" && !(J instanceof Coordinate), se = ne == "data" && !(te instanceof Coordinate), [ce, le] = ee && ae ? H <= q ? [H, q] : [q, H] : ee ? [H, H] : ae ? [q, q] : [NaN, NaN], [ue, de] = oe && se ? J <= te ? [J, te] : [te, J] : oe ? [J, J] : se ? [te, te] : [NaN, NaN];
      return { x0: ce, x1: le, y0: ue, y1: de };
    }
    log_bounds() {
      return empty$2();
    }
    get mappers() {
      function H(ae, oe, se, ce) {
        switch (ae) {
          case "canvas":
            return ce;
          case "screen":
            return se;
          case "data":
            return oe;
        }
      }
      const V = this.model, { x_scale: q, y_scale: G } = this.coordinates, { x_view: J, y_view: Z } = this.plot_view.frame.bbox, { x_screen: te, y_screen: ne } = this.plot_view.canvas.bbox;
      return {
        left: H(V.left_units, q, J, te),
        right: H(V.right_units, q, J, te),
        top: H(V.top_units, G, Z, ne),
        bottom: H(V.bottom_units, G, Z, ne)
      };
    }
    get border_radius() {
      return border_radius(this.model.border_radius);
    }
    compute_geometry() {
      super.compute_geometry();
      const H = (te, ne, ee) => ne instanceof Coordinate ? this.resolve_as_scalar(ne, te) : ee.compute(ne), { left: V, right: q, top: G, bottom: J } = this.model, { mappers: Z } = this;
      this.bbox = BBox$1.from_lrtb({
        left: H("x", V, Z.left),
        right: H("x", q, Z.right),
        top: H("y", G, Z.top),
        bottom: H("y", J, Z.bottom)
      });
    }
    _render() {
      if (!this.bbox.is_valid)
        return;
      const { ctx: H } = this.layer;
      H.save(), H.beginPath(), round_rect(H, this.bbox, this.border_radius);
      const { _is_hovered: V, visuals: q } = this, G = V && q.hover_fill.doit ? q.hover_fill : q.fill, J = V && q.hover_hatch.doit ? q.hover_hatch : q.hatch, Z = V && q.hover_line.doit ? q.hover_line : q.line;
      G.apply(H), J.apply(H), Z.apply(H), H.restore();
    }
    interactive_bbox() {
      const H = this.model.line_width + EDGE_TOLERANCE$1;
      return this.bbox.grow_by(H);
    }
    interactive_hit(H, V) {
      return !this.model.visible || !this.model.editable ? !1 : this.interactive_bbox().contains(H, V);
    }
    _hit_test(H, V) {
      const { left: q, right: G, bottom: J, top: Z } = this.bbox, te = Math.max(EDGE_TOLERANCE$1, this.model.line_width / 2), ne = abs$8(q - H), ee = abs$8(G - H), ae = abs$8(Z - V), oe = abs$8(J - V), se = ne < te && ne < ee, ce = ee < te && ee < ne, le = ae < te && ae < oe, ue = oe < te && oe < ae;
      return le && se ? "top_left" : le && ce ? "top_right" : ue && se ? "bottom_left" : ue && ce ? "bottom_right" : se ? "left" : ce ? "right" : le ? "top" : ue ? "bottom" : this.bbox.contains(H, V) ? "area" : null;
    }
    get resizable() {
      const { resizable: H } = this.model;
      return {
        left: H == "left" || H == "x" || H == "all",
        right: H == "right" || H == "x" || H == "all",
        top: H == "top" || H == "y" || H == "all",
        bottom: H == "bottom" || H == "y" || H == "all"
      };
    }
    _can_hit(H) {
      const { left: V, right: q, top: G, bottom: J } = this.resizable;
      switch (H) {
        case "top_left":
          return G && V;
        case "top_right":
          return G && q;
        case "bottom_left":
          return J && V;
        case "bottom_right":
          return J && q;
        case "left":
          return V;
        case "right":
          return q;
        case "top":
          return G;
        case "bottom":
          return J;
        case "area":
          return this.model.movable != "none";
      }
    }
    on_pan_start(H) {
      if (this.model.visible && this.model.editable) {
        const { sx: V, sy: q } = H, G = this._hit_test(V, q);
        if (G != null && this._can_hit(G))
          return this._pan_state = {
            bbox: this.bbox.clone(),
            target: G
          }, this.model.pan.emit(["pan:start", H.modifiers]), !0;
      }
      return !1;
    }
    on_pan(H) {
      assert$2(this._pan_state != null);
      const { mappers: V } = this, q = (pe, ye, ge) => ye instanceof Coordinate ? this.resolve_as_scalar(ye, pe) : ye == null ? NaN : ge.compute(ye), G = BBox$1.from_lrtb({
        left: q("x", this.model.left_limit, V.left),
        right: q("x", this.model.right_limit, V.right),
        top: q("y", this.model.top_limit, V.top),
        bottom: q("y", this.model.bottom_limit, V.bottom)
      }), [J, Z, te, ne] = (() => {
        const { dx: pe, dy: ye } = H, { target: ge } = this._pan_state, { symmetric: ve } = this.model, [be, Ce] = ve ? [-pe, -ye] : [0, 0];
        switch (ge) {
          // corners
          case "top_left":
            return [pe, be, ye, Ce];
          case "top_right":
            return [be, pe, ye, Ce];
          case "bottom_left":
            return [pe, be, Ce, ye];
          case "bottom_right":
            return [be, pe, Ce, ye];
          // edges
          case "left":
            return [pe, be, 0, 0];
          case "right":
            return [be, pe, 0, 0];
          case "top":
            return [0, 0, ye, Ce];
          case "bottom":
            return [0, 0, Ce, ye];
          // area
          case "area":
            switch (this.model.movable) {
              case "both":
                return [pe, pe, ye, ye];
              case "x":
                return [pe, pe, 0, 0];
              case "y":
                return [0, 0, ye, ye];
              case "none":
                return [0, 0, 0, 0];
            }
        }
      })(), ee = (() => {
        const pe = ($e, qe) => min$b([$e, qe]), ye = ($e) => $e < 0 ? -1 : $e > 0 ? 1 : 0, { bbox: ge } = this._pan_state;
        let { left: ve, right: be, left_sign: Ce, right_sign: Se } = (() => {
          const $e = ge.left + J, qe = ge.right + Z, Xe = ye(J), Ve = ye(Z);
          return $e <= qe ? { left: $e, right: qe, left_sign: Xe, right_sign: Ve } : { left: qe, right: $e, left_sign: Ve, right_sign: Xe };
        })(), { top: we, bottom: Ee, top_sign: xe, bottom_sign: Le } = (() => {
          const $e = ge.top + te, qe = ge.bottom + ne, Xe = ye(te), Ve = ye(ne);
          return $e <= qe ? { top: $e, bottom: qe, top_sign: Xe, bottom_sign: Ve } : { top: qe, bottom: $e, top_sign: Ve, bottom_sign: Xe };
        })();
        const Qe = ve - G.left, ke = G.right - be, Me = pe(Qe < 0 ? Qe : NaN, ke < 0 ? ke : NaN);
        isFinite(Me) && Me < 0 && (ve += -Ce * -Me, be += -Se * -Me);
        const Re = we - G.top, De = G.bottom - Ee, Pe = pe(Re < 0 ? Re : NaN, De < 0 ? De : NaN);
        return isFinite(Pe) && Pe < 0 && (we += -xe * -Pe, Ee += -Le * -Pe), BBox$1.from_lrtb({ left: ve, right: be, top: we, bottom: Ee });
      })(), { min_width: ae, min_height: oe, max_width: se, max_height: ce } = this.model, { left: le, right: ue, top: de, bottom: fe } = this.model, he = {
        left: V.left.invert(ee.left),
        right: V.right.invert(ee.right),
        top: V.top.invert(ee.top),
        bottom: V.bottom.invert(ee.bottom)
      };
      if (0 < ae || se < 1 / 0) {
        if (J != 0 && Z == 0) {
          const pe = he.right - se, ye = he.right - ae;
          he.left = clamp$1(he.left, pe, ye);
        } else if (J == 0 && Z != 0) {
          const pe = he.left + ae, ye = he.left + se;
          he.right = clamp$1(he.right, pe, ye);
        }
      }
      if (0 < oe || ce < 1 / 0) {
        if (te != 0 && ne == 0) {
          const pe = he.bottom + ce, ye = he.bottom + oe;
          he.top = clamp$1(he.top, pe, ye);
        } else if (te == 0 && ne != 0) {
          const pe = he.top - oe, ye = he.top - ce;
          he.bottom = clamp$1(he.bottom, pe, ye);
        }
      }
      const me = {
        left: le instanceof Coordinate ? le : he.left,
        right: ue instanceof Coordinate ? ue : he.right,
        top: de instanceof Coordinate ? de : he.top,
        bottom: fe instanceof Coordinate ? fe : he.bottom
      };
      this.model.update(me), this.model.pan.emit(["pan", H.modifiers]);
    }
    on_pan_end(H) {
      this._pan_state = null, this.model.pan.emit(["pan:end", H.modifiers]);
    }
    on_pinch_start(H) {
      if (this.model.visible && this.model.editable && this.model.resizable != "none") {
        const { sx: V, sy: q } = H;
        if (this.bbox.contains(V, q))
          return this._pinch_state = {
            bbox: this.bbox.clone()
          }, this.model.pan.emit(["pan:start", H.modifiers]), !0;
      }
      return !1;
    }
    on_pinch(H) {
      assert$2(this._pinch_state != null);
      const V = (() => {
        const { scale: G } = H, { bbox: J } = this._pinch_state, { left: Z, top: te, right: ne, bottom: ee, width: ae, height: oe } = J, se = ae * (G - 1), ce = oe * (G - 1), { resizable: le } = this, ue = le.left ? -se / 2 : 0, de = le.right ? se / 2 : 0, fe = le.top ? -ce / 2 : 0, he = le.bottom ? ce / 2 : 0;
        return BBox$1.from_lrtb({
          left: Z + ue,
          right: ne + de,
          top: te + fe,
          bottom: ee + he
        });
      })(), q = (() => {
        const { left: G, right: J, top: Z, bottom: te } = this.model, { mappers: ne } = this;
        return {
          left: G instanceof Coordinate ? G : ne.left.invert(V.left),
          right: J instanceof Coordinate ? J : ne.right.invert(V.right),
          top: Z instanceof Coordinate ? Z : ne.top.invert(V.top),
          bottom: te instanceof Coordinate ? te : ne.bottom.invert(V.bottom)
        };
      })();
      this.model.update(q), this.model.pan.emit(["pan", H.modifiers]);
    }
    on_pinch_end(H) {
      this._pinch_state = null, this.model.pan.emit(["pan:end", H.modifiers]);
    }
    get _has_hover() {
      const { hover_line: H, hover_fill: V, hover_hatch: q } = this.visuals;
      return H.doit || V.doit || q.doit;
    }
    on_enter(H) {
      const { _has_hover: V } = this;
      return V && (this._is_hovered = !0, this.request_paint()), V;
    }
    on_move(H) {
    }
    on_leave(H) {
      this._has_hover && (this._is_hovered = !1, this.request_paint());
    }
    cursor(H, V) {
      var G;
      const q = ((G = this._pan_state) == null ? void 0 : G.target) ?? this._hit_test(H, V);
      if (q == null || !this._can_hit(q))
        return null;
      switch (q) {
        case "top_left":
          return this.model.tl_cursor;
        case "top_right":
          return this.model.tr_cursor;
        case "bottom_left":
          return this.model.bl_cursor;
        case "bottom_right":
          return this.model.br_cursor;
        case "left":
        case "right":
          return this.model.ew_cursor;
        case "top":
        case "bottom":
          return this.model.ns_cursor;
        case "area":
          switch (this.model.movable) {
            case "both":
              return this.model.in_cursor;
            case "x":
              return this.model.ew_cursor;
            case "y":
              return this.model.ns_cursor;
            case "none":
              return null;
          }
      }
    }
  }
  mt(BoxAnnotationView, "__name__", "BoxAnnotationView");
  const T2 = class T2 extends Annotation$1 {
    constructor(H) {
      super(H);
      mt(this, "pan", new Signal(this, "pan"));
    }
    update({ left: H, right: V, top: q, bottom: G }) {
      this.setv({ left: H, right: V, top: q, bottom: G, visible: !0 });
    }
    clear() {
      this.visible = !1;
    }
  };
  mt(T2, "__name__", "BoxAnnotation"), T2.prototype.default_view = BoxAnnotationView, T2.mixins([
    Line$4,
    Fill$2,
    Hatch$1,
    ["hover_", Line$4],
    ["hover_", Fill$2],
    ["hover_", Hatch$1]
  ]), T2.define(({ Bool: H, Float: V, Ref: q, Or: G, NonNegative: J, Positive: Z }) => ({
    top: [G(V, q(Coordinate)), () => new Node$4({ target: "frame", symbol: "top" })],
    bottom: [G(V, q(Coordinate)), () => new Node$4({ target: "frame", symbol: "bottom" })],
    left: [G(V, q(Coordinate)), () => new Node$4({ target: "frame", symbol: "left" })],
    right: [G(V, q(Coordinate)), () => new Node$4({ target: "frame", symbol: "right" })],
    top_units: [CoordinateUnits, "data"],
    bottom_units: [CoordinateUnits, "data"],
    left_units: [CoordinateUnits, "data"],
    right_units: [CoordinateUnits, "data"],
    top_limit: [Limit, null],
    bottom_limit: [Limit, null],
    left_limit: [Limit, null],
    right_limit: [Limit, null],
    min_width: [J(V), 0],
    min_height: [J(V), 0],
    max_width: [Z(V), 1 / 0],
    max_height: [Z(V), 1 / 0],
    border_radius: [BorderRadius, 0],
    editable: [H, !1],
    resizable: [Resizable, "all"],
    movable: [Movable, "both"],
    symmetric: [H, !1]
  })), T2.internal(({ Str: H }) => ({
    tl_cursor: [H, "nwse-resize"],
    tr_cursor: [H, "nesw-resize"],
    bl_cursor: [H, "nesw-resize"],
    br_cursor: [H, "nwse-resize"],
    ew_cursor: [H, "ew-resize"],
    ns_cursor: [H, "ns-resize"],
    in_cursor: [H, "move"]
  })), T2.override({
    fill_color: "#fff9ba",
    fill_alpha: 0.4,
    line_color: "#cccccc",
    line_alpha: 0.3,
    hover_fill_color: null,
    hover_fill_alpha: 0.4,
    hover_line_color: null,
    hover_line_alpha: 0.3
  });
  let BoxAnnotation = T2;
  class CategoricalAxisView extends AxisView {
    _paint(_, H, V) {
      this._draw_group_separators(_, H, V);
    }
    _draw_group_separators(_, H, V) {
      const [q] = this.ranges, [G, J] = this.computed_bounds;
      if (q.tops == null || q.tops.length < 2 || !this.visuals.separator_line.doit)
        return;
      const Z = this.dimension, te = 1 - Z, ne = [[], []];
      let ee = 0;
      for (let oe = 0; oe < q.tops.length - 1; oe++) {
        let se, ce;
        for (let ue = ee; ue < q.factors.length; ue++)
          if (q.factors[ue][0] == q.tops[oe + 1]) {
            [se, ce] = [q.factors[ue - 1], q.factors[ue]], ee = ue;
            break;
          }
        const le = (q.synthetic(se) + q.synthetic(ce)) / 2;
        le > G && le < J && (ne[Z].push(le), ne[te].push(this.loc));
      }
      const ae = this.extents.tick_label;
      this._draw_ticks(_, ne, -3, ae - 6, this.visuals.separator_line);
    }
    _draw_major_labels(_, H, V) {
      const q = this._get_factor_info();
      let G = H.tick + this.model.major_label_standoff;
      for (let J = 0; J < q.length; J++) {
        const [Z, te, ne, ee] = q[J];
        this._draw_oriented_labels(_, Z, te, ne, G, ee), G += H.tick_labels[J];
      }
    }
    _tick_label_extents() {
      const _ = this._get_factor_info(), H = [];
      for (const [V, , q, G] of _) {
        const J = this._oriented_labels_extent(V, q, this.model.major_label_standoff, G);
        H.push(J);
      }
      return H;
    }
    _get_factor_info() {
      const [_] = this.ranges, [H, V] = this.computed_bounds, q = this.loc, G = this.model.ticker.get_ticks(H, V, _, q), J = this.tick_coords, Z = [], te = (ee) => new GraphicsBoxes(ee.map((ae) => isString$2(ae) ? new TextBox({ text: ae }) : ae)), ne = (ee) => te(this.model.formatter.doFormat(ee, this));
      if (_.levels == 1) {
        const ee = G.major, ae = ne(ee);
        Z.push([ae, J.major, this.model.major_label_orientation, this.visuals.major_label_text]);
      } else if (_.levels == 2) {
        const ee = G.major.map((oe) => oe[1]), ae = ne(ee);
        Z.push([ae, J.major, this.model.major_label_orientation, this.visuals.major_label_text]), Z.push([te(G.tops), J.tops, this.model.group_label_orientation, this.visuals.group_text]);
      } else if (_.levels == 3) {
        const ee = G.major.map((se) => se[2]), ae = ne(ee), oe = G.mids.map((se) => se[1]);
        Z.push([ae, J.major, this.model.major_label_orientation, this.visuals.major_label_text]), Z.push([te(oe), J.mids, this.model.subgroup_label_orientation, this.visuals.subgroup_text]), Z.push([te(G.tops), J.tops, this.model.group_label_orientation, this.visuals.group_text]);
      }
      return Z;
    }
    get tick_coords() {
      const _ = this.dimension, H = 1 - _, [V] = this.ranges, [q, G] = this.computed_bounds, J = this.model.ticker.get_ticks(q, G, V, this.loc), Z = {
        major: [[], []],
        mids: [[], []],
        tops: [[], []],
        minor: [[], []]
      };
      return Z.major[_] = J.major, Z.major[H] = J.major.map(() => this.loc), V.levels == 3 && (Z.mids[_] = J.mids, Z.mids[H] = J.mids.map(() => this.loc)), V.levels > 1 && (Z.tops[_] = J.tops, Z.tops[H] = J.tops.map(() => this.loc)), Z;
    }
  }
  mt(CategoricalAxisView, "__name__", "CategoricalAxisView");
  const r0 = class r0 extends Axis {
    constructor(_) {
      super(_);
    }
  };
  mt(r0, "__name__", "CategoricalAxis"), r0.prototype.default_view = CategoricalAxisView, r0.mixins([
    ["separator_", Line$4],
    ["group_", Text$4],
    ["subgroup_", Text$4]
  ]), r0.define(({ Float: _, Or: H }) => ({
    group_label_orientation: [H(LabelOrientation, _), "parallel"],
    subgroup_label_orientation: [H(LabelOrientation, _), "parallel"]
  })), r0.override({
    ticker: () => new CategoricalTicker(),
    formatter: () => new CategoricalTickFormatter(),
    separator_line_color: "lightgrey",
    separator_line_width: 2,
    group_text_font_style: "bold",
    group_text_font_size: "11px",
    group_text_color: "grey",
    subgroup_text_font_style: "bold",
    subgroup_text_font_size: "11px"
  });
  let CategoricalAxis = r0;
  class DatetimeAxisView extends ContinuousAxisView {
  }
  mt(DatetimeAxisView, "__name__", "DatetimeAxisView");
  const G3 = class G3 extends ContinuousAxis {
    constructor(_) {
      super(_);
    }
  };
  mt(G3, "__name__", "DatetimeAxis"), G3.prototype.default_view = DatetimeAxisView, G3.override({
    ticker: () => new DatetimeTicker(),
    formatter: () => new DatetimeTickFormatter()
  });
  let DatetimeAxis = G3;
  class LogAxisView extends ContinuousAxisView {
  }
  mt(LogAxisView, "__name__", "LogAxisView");
  const W3 = class W3 extends ContinuousAxis {
    constructor(_) {
      super(_);
    }
  };
  mt(W3, "__name__", "LogAxis"), W3.prototype.default_view = LogAxisView, W3.override({
    ticker: () => new LogTicker(),
    formatter: () => new LogTickFormatter()
  });
  let LogAxis = W3;
  class MercatorAxisView extends AxisView {
  }
  mt(MercatorAxisView, "__name__", "MercatorAxisView");
  const Y3 = class Y3 extends LinearAxis {
    constructor(_) {
      super(_);
    }
  };
  mt(Y3, "__name__", "MercatorAxis"), Y3.prototype.default_view = MercatorAxisView, Y3.override({
    ticker: () => new MercatorTicker({ dimension: "lat" }),
    formatter: () => new MercatorTickFormatter({ dimension: "lat" })
  });
  let MercatorAxis = Y3;
  function _cat_equals(O, _) {
    if (O.length != _.length)
      return !1;
    const H = O.length;
    for (let V = 0; V < H; V++)
      if (O[V] !== _[V])
        return !1;
    return !0;
  }
  function cat_v_compute(O, _, H, V, q, G, J) {
    let Z = 0;
    for (const te of O) {
      let ne;
      if (!isArray$3(te))
        ne = index_of(_, te);
      else {
        const ae = te.slice(q, G ?? void 0);
        ae.length == 1 ? ne = index_of(_, ae[0]) : ne = find_index(_, (oe) => _cat_equals(oe, ae));
      }
      const ee = ne in H ? H[ne] : J;
      V[Z++] = ee;
    }
  }
  const s5 = class s5 extends ColorMapper {
    constructor(_) {
      super(_);
    }
    _v_compute(_, H, V, { nan_color: q }) {
      cat_v_compute(_, this.factors, V, H, this.start, this.end, q);
    }
  };
  mt(s5, "__name__", "CategoricalColorMapper"), s5.define(({ Float: _, Nullable: H }) => ({
    factors: [FactorSeq],
    start: [_, 0],
    end: [H(_), null]
  }));
  let CategoricalColorMapper = s5;
  const l5 = class l5 extends Mapper {
    constructor(_) {
      super(_);
    }
    v_compute(_) {
      const H = new Array(_.length);
      return cat_v_compute(_, this.factors, this.markers, H, this.start, this.end, this.default_value), H;
    }
  };
  mt(l5, "__name__", "CategoricalMarkerMapper"), l5.define(({ Float: _, List: H, Nullable: V }) => ({
    factors: [FactorSeq],
    markers: [H(MarkerType)],
    start: [_, 0],
    end: [V(_), null],
    default_value: [MarkerType, "circle"]
  }));
  let CategoricalMarkerMapper = l5;
  const c5 = class c5 extends Mapper {
    constructor(_) {
      super(_);
    }
    v_compute(_) {
      const H = new Array(_.length);
      return cat_v_compute(_, this.factors, this.patterns, H, this.start, this.end, this.default_value), H;
    }
  };
  mt(c5, "__name__", "CategoricalPatternMapper"), c5.define(({ Float: _, List: H, Nullable: V }) => ({
    factors: [FactorSeq],
    patterns: [H(HatchPatternType)],
    start: [_, 0],
    end: [V(_), null],
    default_value: [HatchPatternType, " "]
  }));
  let CategoricalPatternMapper = c5;
  class LinearColorMapper extends ContinuousColorMapper {
    constructor(_) {
      super(_);
    }
    scan(_, H) {
      const V = this.low != null ? this.low : min$b(_), q = this.high != null ? this.high : max$j(_), G = 1 / (q - V), J = 1 / H;
      return { max: q, min: V, norm_factor: G, normed_interval: J };
    }
    index_to_value(_) {
      const H = this._scan_data;
      return H.min + H.normed_interval * _ / H.norm_factor;
    }
    value_to_index(_, H) {
      const V = this._scan_data;
      if (_ == V.max)
        return H - 1;
      const q = (_ - V.min) * V.norm_factor, G = Math.floor(q / V.normed_interval);
      return clamp$1(G, -1, H);
    }
  }
  mt(LinearColorMapper, "__name__", "LinearColorMapper");
  class LogColorMapper extends ContinuousColorMapper {
    constructor(_) {
      super(_);
    }
    scan(_, H) {
      const V = this.low != null ? this.low : min$b(_), q = this.high != null ? this.high : max$j(_), G = H / Math.log(q / V);
      return { max: q, min: V, scale: G };
    }
    index_to_value(_) {
      const H = this._scan_data;
      return H.min * Math.exp(_ / H.scale);
    }
    value_to_index(_, H) {
      const V = this._scan_data;
      if (_ == V.max)
        return H - 1;
      if (_ > V.max)
        return H;
      if (_ < V.min)
        return -1;
      const q = Math.log(_ / V.min), G = Math.floor(q * V.scale);
      return clamp$1(G, -1, H);
    }
  }
  mt(LogColorMapper, "__name__", "LogColorMapper");
  const u5 = class u5 extends ScanningColorMapper {
    constructor(_) {
      super(_);
    }
    // Public for unit tests
    /*protected*/
    scan(_, H) {
      const V = this.low != null ? this.low : min$b(_), q = this.high != null ? this.high : max$j(_), G = this.bins, J = linspace(V, q, G + 1), Z = bin_counts(_, J);
      let te = 0;
      for (let he = 0; he < G; he++)
        Z[he] != 0 && te++;
      const ne = new Array(te + 1), ee = new Array(te + 1);
      for (let he = 0, me = 1; he < G; he++)
        Z[he] != 0 && (ne[me] = Z[he], ee[me] = (J[he] + J[he + 1]) / 2, me++);
      ne[0] = 0, ee[0] = 2 * ee[1] - ee[te];
      const ae = cumsum(ne), oe = ae[1], se = ae[te] - oe;
      for (let he = 1; he <= te; he++)
        ae[he] = (ae[he] - oe) / se;
      ae[0] = -1;
      let { rescale_discrete_levels: ce } = this, le = 0;
      if (ce) {
        const he = te, me = -0.5 / 98, pe = 1.5 - 2 * me, ye = me * he + pe;
        ye > 1 ? le = 1 - ye : ce = !1;
      }
      const ue = linspace(le, 1, H + 1), de = interpolate$2(ue, ae, ee);
      let fe = !1;
      if (ce) {
        const he = sorted_index(de, V);
        V < de[he] && he > 0 && (de[he - 1] = V), fe = !0;
      } else
        de[0] = V;
      return de[de.length - 1] = q, { min: V, max: q, binning: de, force_low_cutoff: fe };
    }
  };
  mt(u5, "__name__", "EqHistColorMapper"), u5.define(({ Bool: _, Int: H }) => ({
    bins: [H, 256 * 256],
    rescale_discrete_levels: [_, !1]
  }));
  let EqHistColorMapper = u5;
  class StackColorMapper extends ColorMapper {
    constructor(_) {
      super(_);
    }
  }
  mt(StackColorMapper, "__name__", "StackColorMapper");
  const d5 = class d5 extends StackColorMapper {
    constructor(_) {
      super(_);
    }
    // Weighted mix of colors.
    _mix_colors(_, H, V, q) {
      if (isNaN(q))
        return H;
      let G = 0, J = 0, Z = 0, te = 0;
      const ne = V.length;
      if (q != 0)
        for (let ee = 0; ee < ne; ee++) {
          if (isNaN(V[ee]))
            continue;
          const ae = V[ee] / q;
          G += _[ee * 4] * ae, J += _[ee * 4 + 1] * ae, Z += _[ee * 4 + 2] * ae, te += _[ee * 4 + 3] * ae;
        }
      else {
        let ee = 0;
        for (let ae = 0; ae < ne; ae++)
          V[ae] == 0 && (G += _[ae * 4], J += _[ae * 4 + 1], Z += _[ae * 4 + 2], te += _[ae * 4 + 3], ee++);
        G /= ee, J /= ee, Z /= ee, te /= ee;
      }
      return encode_rgba([byte(G), byte(J), byte(Z), byte(te)]);
    }
    _v_compute(_, H, V, q) {
      unreachable();
    }
    _v_compute_uint32(_, H, V, q) {
      const G = H.length, J = V.length, Z = _.length / G;
      assert$2(Z == J, `Expected ${Z} not ${J} colors in palette`);
      const te = new RGBAArray(J * 4);
      for (let ue = 0; ue < J; ue++) {
        const [de, fe, he, me] = decode_rgba(V[ue]);
        te[ue * 4] = de, te[ue * 4 + 1] = fe, te[ue * 4 + 2] = he, te[ue * 4 + 3] = me;
      }
      const ne = this.color_baseline, ee = ne ?? min$b(_), { nan_color: ae } = q, oe = new Array(G), se = new Array(J);
      for (let ue = 0; ue < G; ue++) {
        let de = NaN;
        for (let fe = 0; fe < J; fe++) {
          const he = ue * J + fe, me = ee == 0 ? _[he] : Math.max(_[he] - ee, 0);
          se[fe] = me, isNaN(me) || (isNaN(de) ? de = me : de += me);
        }
        H[ue] = this._mix_colors(te, ae, se, de), oe[ue] = de + ee * J;
      }
      const ce = _convert_palette(this.alpha_mapper.palette), le = new Uint32Array(G);
      this.alpha_mapper._v_compute(oe, le, ce, q);
      for (let ue = 0; ue < G; ue++) {
        const de = byte((H[ue] & 255) * (le[ue] & 255) / 255);
        H[ue] = H[ue] & 4294967040 | de;
      }
    }
  };
  mt(d5, "__name__", "WeightedStackColorMapper"), d5.define(({ List: _, Nullable: H, Float: V, Ref: q, Str: G }) => ({
    alpha_mapper: [q(ContinuousColorMapper)],
    color_baseline: [H(V), null],
    stack_labels: [H(_(G)), null]
  }));
  let WeightedStackColorMapper = d5;
  class ColorBarView extends BaseColorBarView {
    constructor() {
      super(...arguments);
      mt(this, "_image");
      // Indices of displayed colors corresponding to low and high display cutoffs.
      mt(this, "_index_low");
      mt(this, "_index_high");
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.color_mapper.change, async () => {
        this._title_view.remove(), this._axis_view.remove(), this.initialize(), await this.lazy_initialize(), this.plot_view.invalidate_layout();
      }), this.connect(this.model.color_mapper.metrics_change, () => this._metrics_changed()), this.connect(this.model.properties.display_low.change, () => this._metrics_changed()), this.connect(this.model.properties.display_high.change, () => this._metrics_changed());
    }
    get color_mapper() {
      let H = this.model.color_mapper;
      return H instanceof WeightedStackColorMapper && (H = H.alpha_mapper), H;
    }
    update_layout() {
      super.update_layout(), this._set_canvas_image();
    }
    _create_axis() {
      const { color_mapper: H } = this;
      return H instanceof CategoricalColorMapper ? new CategoricalAxis() : H instanceof LogColorMapper ? new LogAxis() : new LinearAxis();
    }
    _create_formatter() {
      const { color_mapper: H } = this;
      return this._ticker instanceof LogTicker ? new LogTickFormatter() : H instanceof CategoricalColorMapper ? new CategoricalTickFormatter() : new BasicTickFormatter();
    }
    _create_major_range() {
      const { color_mapper: H } = this;
      if (H instanceof CategoricalColorMapper)
        return new FactorRange({ factors: H.factors });
      if (H instanceof ContinuousColorMapper) {
        const { min: V, max: q } = this._continuous_metrics(H);
        return new Range1d({ start: V, end: q });
      } else
        unreachable();
    }
    _create_major_scale() {
      const { color_mapper: H } = this;
      if (H instanceof LinearColorMapper)
        return new LinearScale();
      if (H instanceof LogColorMapper)
        return new LogScale();
      if (H instanceof ScanningColorMapper)
        return new LinearInterpolationScale({ binning: this._scanning_binning(H) });
      if (H instanceof CategoricalColorMapper)
        return new CategoricalScale();
      unreachable();
    }
    _create_ticker() {
      const { color_mapper: H } = this;
      return H instanceof LogColorMapper ? new LogTicker() : H instanceof ScanningColorMapper ? new BinnedTicker({ mapper: H }) : H instanceof CategoricalColorMapper ? new CategoricalTicker() : new BasicTicker();
    }
    // Return min and max metrics of ContinuousColorMapper, modified to account
    // for low and high display cutoffs.
    _continuous_metrics(H) {
      const { display_low: V, display_high: q } = this.model;
      let { min: G, max: J } = H.metrics;
      if (q != null && V != null && q < V)
        return this._index_low = 0, this._index_high = -1, { min: NaN, max: NaN };
      if (this._index_high = null, q != null) {
        const Z = H.palette.length, te = H.value_to_index(q, Z);
        te < Z - 1 && (this._index_high = te, J = H.index_to_value(te + 1));
      }
      if (this._index_low = null, V != null) {
        const Z = H.value_to_index(V, H.palette.length);
        Z > 0 && (this._index_low = Z, G = H.index_to_value(Z));
      }
      return { min: G, max: J };
    }
    _get_major_size_factor() {
      return this.color_mapper.palette.length;
    }
    _metrics_changed() {
      const H = this._major_range, V = this._major_scale, { color_mapper: q } = this;
      if (q instanceof ScanningColorMapper && V instanceof LinearInterpolationScale) {
        const G = this._scanning_binning(q);
        V.binning = G;
        const J = this.orientation == "vertical", Z = J ? this._frame.y_scale : this._frame.x_scale;
        if (Z instanceof LinearInterpolationScale) {
          Z.binning = G;
          const te = J ? this._frame.y_range : this._frame.x_range;
          te instanceof Range1d && (te.start = G[0], te.end = G[G.length - 1]);
        }
      } else if (q instanceof ContinuousColorMapper && H instanceof Range1d) {
        const { min: G, max: J } = this._continuous_metrics(q);
        H.setv({ start: G, end: J });
      }
      this._set_canvas_image(), this.plot_view.request_layout();
    }
    _paint_colors(H, V) {
      const { x: q, y: G, width: J, height: Z } = V;
      H.save(), H.globalAlpha = this.model.scale_alpha, this._image != null && (H.imageSmoothingEnabled = !1, H.drawImage(this._image, q, G, J, Z)), this.visuals.bar_line.doit && (this.visuals.bar_line.set_value(H), H.strokeRect(q, G, J, Z)), H.restore();
    }
    // Return binning array of ScanningColorMapper, modified to account for low
    // and high display cutoffs.
    _scanning_binning(H) {
      let { binning: V, force_low_cutoff: q } = H.metrics;
      const { display_high: G } = this.model;
      let { display_low: J } = this.model;
      if (q && (J == null || H.metrics.min > J) && (J = H.metrics.min), G != null && J != null && G < J)
        return this._index_low = 0, this._index_high = -1, [NaN];
      if (this._index_high = null, G != null) {
        const Z = H.value_to_index(G, V.length);
        Z < V.length - 1 && (this._index_high = Z);
      }
      if (this._index_low = null, J != null) {
        const Z = H.value_to_index(J, V.length);
        Z > 0 && (this._index_low = Z);
      }
      if (this._index_low != null || this._index_high != null) {
        const Z = this._index_low != null ? this._index_low : 0, ne = (this._index_high != null ? this._index_high + 1 : V.length - 1) - Z + 1;
        if (ne > 0) {
          const ee = new Array(ne);
          for (let ae = 0; ae < ne; ae++)
            ee[ae] = V[ae + Z];
          V = ee;
        } else
          V = [NaN];
      }
      return V;
    }
    _set_canvas_image() {
      const { orientation: H } = this;
      let { palette: V } = this.color_mapper;
      if ((this._index_high != null || this._index_low != null) && (V = V.slice(this._index_low != null ? this._index_low : 0, this._index_high != null ? this._index_high + 1 : V.length)), V.length < 1) {
        this._image = null;
        return;
      }
      H == "vertical" && (V = reversed(V));
      const [q, G] = H == "vertical" ? [1, V.length] : [V.length, 1], J = this._image = document.createElement("canvas");
      J.width = q, J.height = G;
      const Z = J.getContext("2d"), te = Z.getImageData(0, 0, q, G), ee = new LinearColorMapper({ palette: V }).rgba_mapper.v_compute(range$9(0, V.length));
      te.data.set(ee), Z.putImageData(te, 0, 0);
    }
  }
  mt(ColorBarView, "__name__", "ColorBarView");
  const X3 = class X3 extends BaseColorBar {
    constructor(_) {
      super(_);
    }
  };
  mt(X3, "__name__", "ColorBar"), X3.prototype.default_view = ColorBarView, X3.define(({ Nullable: _, Float: H, Ref: V }) => ({
    color_mapper: [V(ColorMapper)],
    display_low: [_(H), null],
    display_high: [_(H), null]
  }));
  let ColorBar = X3;
  class ContourColorBarView extends BaseColorBarView {
    constructor() {
      super(...arguments);
      mt(this, "_fill_view");
      mt(this, "_line_view");
    }
    *children() {
      yield* super.children(), yield this._fill_view, yield this._line_view;
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { fill_renderer: H, line_renderer: V } = this.model;
      this._fill_view = await build_view(H, { parent: this.parent }), this._line_view = await build_view(V, { parent: this.parent });
    }
    remove() {
      this._fill_view.remove(), this._line_view.remove(), super.remove();
    }
    _create_major_range() {
      const H = this.model.levels;
      return H.length > 0 ? new Range1d({ start: H[0], end: H[H.length - 1] }) : new Range1d({ start: 0, end: 1 });
    }
    _paint_colors(H, V) {
      const q = this.orientation == "vertical", G = this.model.levels, J = this._major_scale;
      J.source_range = this._major_range, q ? J.target_range = new Range1d({ start: V.bottom, end: V.top }) : J.target_range = new Range1d({ start: V.left, end: V.right });
      const Z = J.v_compute(G), te = this._fill_view.glyph, ne = te.data_size;
      if (ne > 0) {
        assert$2(G.length == ne + 1, "Inconsistent number of filled contour levels"), H.save();
        for (let oe = 0; oe < ne; oe++)
          H.beginPath(), q ? H.rect(V.left, Z[oe], V.width, Z[oe + 1] - Z[oe]) : H.rect(Z[oe], V.top, Z[oe + 1] - Z[oe], V.height), te.visuals.fill.apply(H, oe), te.visuals.hatch.apply(H, oe);
        H.restore();
      }
      const ee = this._line_view.glyph, ae = ee.data_size;
      if (ae > 0) {
        assert$2(G.length == ae, "Inconsistent number of line contour levels"), H.save();
        for (let oe = 0; oe < ae; oe++)
          H.beginPath(), q ? (H.moveTo(V.left, Z[oe]), H.lineTo(V.right, Z[oe])) : (H.moveTo(Z[oe], V.bottom), H.lineTo(Z[oe], V.top)), ee.visuals.line.set_vectorize(H, oe), H.stroke();
        H.restore();
      }
    }
  }
  mt(ContourColorBarView, "__name__", "ContourColorBarView");
  const Z3 = class Z3 extends BaseColorBar {
    constructor(_) {
      super(_);
    }
  };
  mt(Z3, "__name__", "ContourColorBar"), Z3.prototype.default_view = ContourColorBarView, Z3.define(({ List: _, Float: H, Ref: V }) => ({
    fill_renderer: [V(GlyphRenderer)],
    line_renderer: [V(GlyphRenderer)],
    levels: [_(H), []]
  }));
  let ContourColorBar = Z3;
  function xy(O, _) {
    return { x: O, y: _ };
  }
  class LabelView extends TextAnnotationView$1 {
    constructor() {
      super(...arguments);
      mt(this, "_pan_state", null);
    }
    get mappers() {
      function H(oe, se, ce, le) {
        switch (oe) {
          case "canvas":
            return le;
          case "screen":
            return ce;
          case "data":
            return se;
        }
      }
      const V = this.model, q = this.layout ?? this.plot_view.frame, { x_scale: G, y_scale: J } = this.coordinates, { x_view: Z, y_view: te } = q.bbox, { x_screen: ne, y_screen: ee } = this.plot_view.canvas.bbox;
      return {
        x: H(V.x_units, G, Z, ne),
        y: H(V.y_units, J, te, ee)
      };
    }
    get anchor() {
      const { align: H, baseline: V } = this.visuals.text.values();
      return text_anchor(this.model.anchor, H, V);
    }
    get angle() {
      const { angle: H, angle_units: V, direction: q } = this.model;
      return compute_angle(H, V, q);
    }
    get origin() {
      const { mappers: H } = this, { x: V, y: q, x_offset: G, y_offset: J } = this.model, Z = (ee, ae, oe) => ae instanceof Coordinate ? this.resolve_as_scalar(ae, ee) : oe.compute(ae), te = Z("x", V, H.x) + G, ne = Z("y", q, H.y) - J;
      return { sx: te, sy: ne };
    }
    interactive_hit(H, V) {
      return !this.model.visible || !this.model.editable ? !1 : this._hit_test(H, V) == "area";
    }
    _hit_test(H, V) {
      const { sx: q, sy: G, anchor: J, angle: Z, width: te, height: ne } = this._rect, { x: ee, y: ae } = rotate_around(xy(H, V), xy(q, G), -Z), oe = q - J.x * te, se = G - J.y * ne, ce = oe + te, le = se + ne;
      return oe <= ee && ee <= ce && se <= ae && ae <= le ? "area" : null;
    }
    _can_hit(H) {
      return !0;
    }
    on_pan_start(H) {
      if (this.model.visible && this.model.editable) {
        const { sx: V, sy: q } = H, G = this._hit_test(V, q);
        if (G != null && this._can_hit(G))
          return this._pan_state = {
            angle: this.angle,
            base: { sx: V, sy: q },
            target: G,
            action: "rotate"
          }, this.model.pan.emit(["pan:start", H.modifiers]), !0;
      }
      return !1;
    }
    on_pan(H) {
      assert$2(this._pan_state != null);
      const { dx: V, dy: q } = H, { angle: G, base: J } = this._pan_state, { origin: Z } = this, te = atan2$2([Z.sx, Z.sy], [J.sx, J.sy]), ee = atan2$2([Z.sx, Z.sy], [J.sx + V, J.sy + q]) - te, oe = (G + ee) % (2 * Math.PI), { angle_units: se, direction: ce } = this.model;
      this.model.angle = invert_angle(oe, se, ce), this.model.pan.emit(["pan", H.modifiers]);
    }
    on_pan_end(H) {
      this._pan_state = null, this.model.pan.emit(["pan:end", H.modifiers]);
    }
    cursor(H, V) {
      var G;
      const q = ((G = this._pan_state) == null ? void 0 : G.target) ?? this._hit_test(H, V);
      return q == null || !this._can_hit(q) ? null : "var(--bokeh-cursor-rotate)";
    }
  }
  mt(LabelView, "__name__", "LabelView");
  var k2;
  let Label$3 = (k2 = class extends TextAnnotation$1 {
    constructor(H) {
      super(H);
      mt(this, "pan", new Signal(this, "pan"));
    }
  }, mt(k2, "__name__", "Label"), k2.prototype.default_view = LabelView, k2.define(({ Bool: H, Float: V, Angle: q, Or: G, Ref: J }) => ({
    anchor: [TextAnchor, "auto"],
    x: [G(V, J(Coordinate))],
    y: [G(V, J(Coordinate))],
    x_units: [CoordinateUnits, "data"],
    y_units: [CoordinateUnits, "data"],
    x_offset: [V, 0],
    y_offset: [V, 0],
    angle: [q, 0],
    angle_units: [AngleUnits, "rad"],
    direction: [Direction$1, "anticlock"],
    editable: [H, !1]
  })), k2);
  class LabelSetView extends DataAnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "_x");
      mt(this, "_y");
      mt(this, "sx");
      mt(this, "sy");
      mt(this, "text");
      mt(this, "angle");
      mt(this, "x_offset");
      mt(this, "y_offset");
    }
    map_data() {
      const { x_scale: H, y_scale: V } = this.coordinates, q = this.layout != null ? this.layout : this.plot_view.frame;
      this.sx = (() => {
        switch (this.model.x_units) {
          case "canvas":
            return new ScreenArray(this._x);
          case "screen":
            return q.bbox.xview.v_compute(this._x);
          case "data":
            return H.v_compute(this._x);
        }
      })(), this.sy = (() => {
        switch (this.model.y_units) {
          case "canvas":
            return new ScreenArray(this._y);
          case "screen":
            return q.bbox.yview.v_compute(this._y);
          case "data":
            return V.v_compute(this._y);
        }
      })();
    }
    paint() {
      const { ctx: H } = this.layer;
      for (let V = 0, q = this.text.length; V < q; V++) {
        const G = this.x_offset.get(V), J = this.y_offset.get(V), Z = this.sx[V] + G, te = this.sy[V] - J, ne = this.angle.get(V), ee = this.text.get(V);
        !isFinite(Z + te + ne) || ee == null || this._paint(H, V, `${ee}`, Z, te, ne);
      }
    }
    _paint(H, V, q, G, J, Z) {
      const te = new TextBox({ text: q });
      te.angle = Z, te.position = { sx: G, sy: J }, te.visuals = this.visuals.text.values(V);
      const { background_fill: ne, border_line: ee } = this.visuals;
      if (ne.doit || ee.doit) {
        const { p0: ae, p1: oe, p2: se, p3: ce } = te.rect();
        H.beginPath(), H.moveTo(ae.x, ae.y), H.lineTo(oe.x, oe.y), H.lineTo(se.x, se.y), H.lineTo(ce.x, ce.y), H.closePath(), this.visuals.background_fill.apply(H, V), this.visuals.border_line.apply(H, V);
      }
      this.visuals.text.doit && te.paint(H);
    }
  }
  mt(LabelSetView, "__name__", "LabelSetView");
  const i0 = class i0 extends DataAnnotation {
    constructor(_) {
      super(_);
    }
  };
  mt(i0, "__name__", "LabelSet"), i0.prototype.default_view = LabelSetView, i0.mixins([
    TextVector$1,
    ["border_", LineVector$1],
    ["background_", FillVector$1]
  ]), i0.define(() => ({
    x: [XCoordinateSpec, { field: "x" }],
    y: [YCoordinateSpec, { field: "y" }],
    x_units: [CoordinateUnits, "data"],
    y_units: [CoordinateUnits, "data"],
    text: [NullStringSpec, { field: "text" }],
    angle: [AngleSpec, 0],
    x_offset: [NumberSpec, { value: 0 }],
    y_offset: [NumberSpec, { value: 0 }]
  })), i0.override({
    background_fill_color: null,
    border_line_color: null
  });
  let LabelSet = i0;
  const f5 = class f5 extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "legend");
    }
    /*protected*/
    _check_data_sources_on_renderers() {
      if (this.get_field_from_label_prop() != null) {
        if (this.renderers.length < 1)
          return !1;
        const V = this.renderers[0].data_source;
        for (const q of this.renderers)
          if (q.data_source != V)
            return !1;
      }
      return !0;
    }
    /*protected*/
    _check_field_label_on_data_source() {
      const H = this.get_field_from_label_prop();
      if (H != null) {
        if (this.renderers.length < 1)
          return !1;
        const V = this.renderers[0].data_source;
        if (!includes(V.columns(), H))
          return !1;
      }
      return !0;
    }
    initialize() {
      super.initialize(), this.legend = null, this.connect(this.change, () => {
        var q;
        return (q = this.legend) == null ? void 0 : q.item_change.emit();
      }), this._check_data_sources_on_renderers() || logger$1.error("Non matching data sources on legend item renderers"), this._check_field_label_on_data_source() || logger$1.error(`Bad column name on label: ${this.label}`);
    }
    get_field_from_label_prop() {
      const { label: H } = this;
      return isField$1(H) ? H.field : null;
    }
    get_labels_list_from_label_prop() {
      if (!this.visible)
        return [];
      const { index: H } = this;
      if (H != null && this.renderers.every((q) => !q.view.indices_map.has(H)))
        return [];
      if (isValue(this.label)) {
        const { value: q } = this.label;
        return q != null ? [q] : [];
      }
      const V = this.get_field_from_label_prop();
      if (V != null) {
        let q;
        if (this.renderers.length != 0)
          q = this.renderers[0].data_source;
        else
          return ["No source found"];
        if (q instanceof ColumnarDataSource) {
          const G = q.get_column(V);
          return G != null ? uniq(Array.from(G)) : ["Invalid field"];
        }
      }
      return [];
    }
  };
  mt(f5, "__name__", "LegendItem"), f5.define(({ Bool: H, Int: V, List: q, Ref: G, Nullable: J }) => ({
    label: [NullStringSpec, null],
    renderers: [q(G(GlyphRenderer)), []],
    index: [J(V), null],
    visible: [H, !0]
  }));
  let LegendItem = f5;
  const { max: max$b, ceil: ceil$1 } = Math;
  class LegendEntry extends ContentLayoutable {
    constructor(H, V, q, G) {
      super();
      mt(this, "item");
      mt(this, "label");
      mt(this, "text");
      mt(this, "settings");
      this.item = H, this.label = V, this.text = q, this.settings = G;
    }
    get field() {
      return this.item.get_field_from_label_prop();
    }
    _content_size() {
      const H = this.text.size(), { glyph_width: V, glyph_height: q, label_standoff: G, label_width: J, label_height: Z } = this.settings, te = V + G + max$b(H.width, J), ne = max$b(q, H.height, Z);
      return new Sizeable({ width: te, height: ne });
    }
  }
  mt(LegendEntry, "__name__", "LegendEntry");
  class LegendView extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "bbox", new BBox$1());
      mt(this, "grid");
      mt(this, "border_box");
      mt(this, "title_panel");
    }
    _get_size() {
      const { width: H, height: V } = this.bbox, { margin: q } = this.model;
      return {
        width: H + 2 * q,
        height: V + 2 * q
      };
    }
    update_layout() {
      this.update_geometry();
      const { panel: H } = this;
      H != null ? this.layout = new SideLayout(H, () => this.get_size()) : this.layout = void 0;
    }
    connect_signals() {
      super.connect_signals();
      const H = () => this.request_render();
      this.connect(this.model.change, H), this.connect(this.model.item_change, H);
    }
    get padding() {
      return this.model.border_line_color != null ? this.model.padding : 0;
    }
    update_geometry() {
      super.update_geometry();
      const { spacing: H, orientation: V } = this.model, q = V == "vertical", { padding: G } = this, J = G, Z = G, { title: te } = this.model, ne = new TextBox({ text: te ?? "" });
      ne.position = { sx: 0, sy: 0, x_anchor: "left", y_anchor: "top" }, ne.visuals = this.visuals.title_text.values();
      const ee = new SidePanel(this.model.title_location);
      ne.angle = ee.get_label_angle_heuristic("parallel");
      const ae = [];
      for (const ye of this.model.items) {
        ye.legend = this.model;
        const ge = ye.get_labels_list_from_label_prop();
        for (const ve of ge) {
          const be = new TextBox({ text: `${ve}` });
          be.position = { sx: 0, sy: 0, x_anchor: "left", y_anchor: "center" }, be.visuals = this.visuals.label_text.values();
          const Ce = new LegendEntry(ye, ve, be, this.model);
          Ce.set_sizing({ visible: ye.visible }), ae.push({ layout: Ce, row: 0, col: 0 });
        }
      }
      const { ncols: oe, nrows: se } = (() => {
        let { ncols: ye, nrows: ge } = this.model;
        const ve = ae.length;
        return q ? (ge != "auto" || (ye != "auto" ? ge = ceil$1(ve / ye) : ge = 1 / 0), ye = 1 / 0) : (ye != "auto" || (ge != "auto" ? ye = ceil$1(ve / ge) : ye = 1 / 0), ge = 1 / 0), { ncols: ye, nrows: ge };
      })();
      let ce = 0, le = 0;
      for (const ye of ae)
        ye.row = ce, ye.col = le, q ? (ce += 1, ce >= se && (ce = 0, le += 1)) : (le += 1, le >= oe && (le = 0, ce += 1));
      const ue = new Grid$1(ae);
      this.grid = ue, ue.spacing = H, ue.set_sizing();
      const de = new TextLayout(ne);
      this.title_panel = de;
      const fe = ne.text != "" && this.visuals.title_text.doit;
      de.set_sizing({ visible: fe });
      const he = (() => {
        if (!fe)
          return new Column$2([ue]);
        switch (this.model.title_location) {
          case "above":
            return new Column$2([de, ue]);
          case "below":
            return new Column$2([ue, de]);
          case "left":
            return new Row$2([de, ue]);
          case "right":
            return new Row$2([ue, de]);
        }
      })();
      this.border_box = he, he.position = { left: J, top: Z }, he.spacing = this.model.title_standoff, he.set_sizing(), he.compute();
      const me = G + he.bbox.width + G, pe = G + he.bbox.height + G;
      this.bbox = new BBox$1({ left: 0, top: 0, width: me, height: pe });
    }
    compute_geometry() {
      super.compute_geometry();
      const { margin: H, location: V } = this.model, { width: q, height: G } = this.bbox, J = this.layout != null ? this.layout : this.plot_view.frame, [Z, te] = J.bbox.ranges;
      let ne, ee;
      if (isString$2(V))
        switch (V) {
          case "top_left":
            ne = Z.start + H, ee = te.start + H;
            break;
          case "top":
          case "top_center":
            ne = (Z.end + Z.start) / 2 - q / 2, ee = te.start + H;
            break;
          case "top_right":
            ne = Z.end - H - q, ee = te.start + H;
            break;
          case "bottom_right":
            ne = Z.end - H - q, ee = te.end - H - G;
            break;
          case "bottom":
          case "bottom_center":
            ne = (Z.end + Z.start) / 2 - q / 2, ee = te.end - H - G;
            break;
          case "bottom_left":
            ne = Z.start + H, ee = te.end - H - G;
            break;
          case "left":
          case "center_left":
            ne = Z.start + H, ee = (te.end + te.start) / 2 - G / 2;
            break;
          case "center":
          case "center_center":
            ne = (Z.end + Z.start) / 2 - q / 2, ee = (te.end + te.start) / 2 - G / 2;
            break;
          case "right":
          case "center_right":
            ne = Z.end - H - q, ee = (te.end + te.start) / 2 - G / 2;
            break;
        }
      else {
        const [ae, oe] = V;
        ne = J.bbox.xview.compute(ae), ee = J.bbox.yview.compute(oe) - G;
      }
      this.bbox = new BBox$1({ left: ne, top: ee, width: q, height: G });
    }
    interactive_hit(H, V) {
      return this.bbox.contains(H, V);
    }
    _hit_test(H, V) {
      const { left: q, top: G } = this.bbox;
      H -= q + this.grid.bbox.left, V -= G + this.grid.bbox.top;
      for (const J of this.grid)
        if (J.bbox.contains(H, V))
          return { type: "entry", entry: J };
      return null;
    }
    cursor(H, V) {
      return this.model.click_policy == "none" ? null : this._hit_test(H, V) != null ? "pointer" : null;
    }
    on_hit(H, V) {
      const q = (() => {
        switch (this.model.click_policy) {
          case "hide":
            return (J) => J.visible = !J.visible;
          case "mute":
            return (J) => J.muted = !J.muted;
          case "none":
            return (J) => {
            };
        }
      })(), G = this._hit_test(H, V);
      if (G != null) {
        const { renderers: J } = G.entry.item;
        for (const Z of J)
          q(Z);
        return !0;
      }
      return !1;
    }
    _render() {
      if (this.update_geometry(), this.compute_geometry(), this.model.items.length == 0 || !some$2(this.model.items, (V) => V.visible))
        return;
      const { ctx: H } = this.layer;
      H.save(), this._draw_legend_box(H), this._draw_legend_items(H), this._draw_title(H), H.restore();
    }
    _draw_legend_box(H) {
      const { x: V, y: q, width: G, height: J } = this.bbox;
      H.beginPath(), H.rect(V, q, G, J), this.visuals.background_fill.apply(H), this.visuals.border_line.apply(H);
    }
    _draw_title(H) {
      const { title: V } = this.model;
      if (V == null || V.length == 0 || !this.visuals.title_text.doit)
        return;
      const { left: q, top: G } = this.bbox;
      switch (H.save(), H.translate(q, G), H.translate(this.title_panel.bbox.left, this.title_panel.bbox.top), this.model.title_location) {
        case "left": {
          H.translate(0, this.title_panel.bbox.height);
          break;
        }
        case "right": {
          H.translate(this.title_panel.bbox.width, 0);
          break;
        }
      }
      this.title_panel.text.paint(H), H.restore();
    }
    _draw_legend_items(H) {
      const V = (() => {
        switch (this.model.click_policy) {
          case "none":
            return (Z) => !0;
          case "hide":
            return (Z) => every$2(Z.renderers, (te) => te.visible);
          case "mute":
            return (Z) => every$2(Z.renderers, (te) => !te.muted);
        }
      })(), q = (Z, te, ne) => {
        if (!this.visuals.item_background_fill.doit)
          return !1;
        switch (this.model.item_background_policy) {
          case "every":
            return !0;
          case "even":
            return te % 2 == 0 == (ne % 2 == 0);
          case "odd":
            return te % 2 == 0 != (ne % 2 == 0);
          case "none":
            return !1;
        }
      }, { left: G, top: J } = this.bbox;
      H.translate(G, J), H.translate(this.grid.bbox.left, this.grid.bbox.top);
      for (const [{ layout: Z, row: te, col: ne }, ee] of enumerate(this.grid.items)) {
        const { bbox: ae, text: oe, item: se, label: ce, field: le, settings: ue } = Z, { glyph_width: de, glyph_height: fe, label_standoff: he } = ue, { left: me, top: pe, width: ye, height: ge } = ae;
        H.translate(me, pe), q(ee, te, ne) && (H.beginPath(), H.rect(0, 0, ye, ge), this.visuals.item_background_fill.apply(H));
        const ve = ge / 2, be = 0, Ce = ve - fe / 2, Se = be + de, we = Ce + fe;
        for (const Ee of se.renderers) {
          const xe = this.plot_view.renderer_view(Ee);
          xe == null || xe.draw_legend(H, be, Se, Ce, we, le, ce, se.index);
        }
        H.translate(Se + he, ve), oe.paint(H), H.translate(-Se - he, -ve), V(se) || (H.beginPath(), H.rect(0, 0, ye, ge), this.visuals.inactive_fill.set_value(H), H.fill()), H.translate(-me, -pe);
      }
      H.translate(-this.grid.bbox.left, -this.grid.bbox.top), H.translate(-G, -J);
    }
  }
  mt(LegendView, "__name__", "LegendView");
  const a0 = class a0 extends Annotation$1 {
    constructor(H) {
      super(H);
      mt(this, "item_change");
    }
    initialize() {
      super.initialize(), this.item_change = new Signal0(this, "item_change");
    }
  };
  mt(a0, "__name__", "Legend"), a0.prototype.default_view = LegendView, a0.mixins([
    ["label_", Text$4],
    ["title_", Text$4],
    ["inactive_", Fill$2],
    ["border_", Line$4],
    ["background_", Fill$2],
    ["item_background_", Fill$2]
  ]), a0.define(({ Float: H, Int: V, Str: q, List: G, Tuple: J, Or: Z, Ref: te, Nullable: ne, Positive: ee, Auto: ae }) => ({
    orientation: [Orientation, "vertical"],
    ncols: [Z(ee(V), ae), "auto"],
    nrows: [Z(ee(V), ae), "auto"],
    location: [Z(LegendLocation, J(H, H)), "top_right"],
    title: [ne(q), null],
    title_location: [Location, "above"],
    title_standoff: [H, 5],
    label_standoff: [H, 5],
    glyph_height: [H, 20],
    glyph_width: [H, 20],
    label_height: [H, 20],
    label_width: [H, 20],
    margin: [H, 10],
    padding: [H, 10],
    spacing: [H, 3],
    items: [G(te(LegendItem)), []],
    click_policy: [LegendClickPolicy, "none"],
    item_background_policy: [AlternationPolicy, "none"]
  })), a0.override({
    border_line_color: "#e5e5e5",
    border_line_alpha: 0.5,
    border_line_width: 1,
    background_fill_color: "#ffffff",
    background_fill_alpha: 0.95,
    item_background_fill_color: "#f1f1f1",
    item_background_fill_alpha: 0.8,
    inactive_fill_color: "white",
    inactive_fill_alpha: 0.7,
    label_text_font_size: "13px",
    label_text_baseline: "middle",
    title_text_font_size: "13px",
    title_text_font_style: "italic"
  });
  let Legend = a0;
  var R0;
  let Polygon$1 = (R0 = class {
    constructor(_ = [], H = []) {
      mt(this, "xs");
      mt(this, "ys");
      this.xs = _, this.ys = H, assert$2(_.length == H.length);
    }
    clone() {
      return new R0(this.xs.slice(), this.ys.slice());
    }
    [Symbol.iterator]() {
      return this.nodes();
    }
    *nodes() {
      const { xs: _, ys: H, n: V } = this;
      for (let q = 0; q < V; q++)
        yield [_[q], H[q], q];
    }
    *edges() {
      const { xs: _, ys: H, n: V } = this;
      for (let q = 1; q < V; q++) {
        const G = { x: _[q - 1], y: H[q - 1] }, J = { x: _[q], y: H[q] };
        yield [G, J, q - 1];
      }
      if (V >= 3) {
        const q = { x: _[V - 1], y: H[V - 1] }, G = { x: _[0], y: H[0] };
        yield [q, G, V - 1];
      }
    }
    contains(_, H) {
      return point_in_poly(_, H, this.xs, this.ys);
    }
    get bbox() {
      const [_, H, V, q] = minmax2(this.xs, this.ys);
      return new BBox$1({ x0: _, x1: H, y0: V, y1: q });
    }
    get n() {
      return this.xs.length;
    }
    translate(_, H, ...V) {
      const q = this.clone(), { xs: G, ys: J, n: Z } = q;
      if (V.length != 0)
        for (const te of V) {
          const ne = te % Z;
          G[ne] += _, J[ne] += H;
        }
      else
        for (let te = 0; te < Z; te++)
          G[te] += _, J[te] += H;
      return q;
    }
  }, mt(R0, "__name__", "Polygon"), R0);
  var Yy, Xy;
  class PolyAnnotationView extends (Xy = AnnotationView, Yy = auto_ranged, Xy) {
    constructor() {
      super(...arguments);
      mt(this, "poly", new Polygon$1());
      mt(this, Yy, !0);
      mt(this, "_pan_state", null);
      mt(this, "_is_hovered", !1);
    }
    serializable_state() {
      return { ...super.serializable_state(), bbox: this.poly.bbox.round() };
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.request_render());
    }
    bounds() {
      const { xs_units: H, ys_units: V } = this.model;
      if (H == "data" && V == "data") {
        const { xs: q, ys: G } = this.model, [J, Z, te, ne] = minmax2(q, G);
        return { x0: J, x1: Z, y0: te, y1: ne };
      } else
        return empty$2();
    }
    log_bounds() {
      return empty$2();
    }
    _mappers() {
      const H = (oe, se, ce, le) => {
        switch (oe) {
          case "canvas":
            return le;
          case "screen":
            return ce;
          case "data":
            return se;
        }
      }, V = this.model, { frame: q, canvas: G } = this.plot_view, { x_scale: J, y_scale: Z } = q, { x_view: te, y_view: ne } = q.bbox, { x_screen: ee, y_screen: ae } = G.bbox;
      return {
        x: H(V.xs_units, J, te, ee),
        y: H(V.ys_units, Z, ne, ae)
      };
    }
    _render() {
      const { xs: H, ys: V } = this.model;
      assert$2(H.length == V.length), this.poly = (() => {
        const { x: ee, y: ae } = this._mappers();
        return new Polygon$1(ee.v_compute(H), ae.v_compute(V));
      })();
      const { ctx: q } = this.layer;
      q.beginPath();
      for (const [ee, ae] of this.poly)
        q.lineTo(ee, ae);
      const { _is_hovered: G, visuals: J } = this, Z = G && J.hover_fill.doit ? J.hover_fill : J.fill, te = G && J.hover_hatch.doit ? J.hover_hatch : J.hatch, ne = G && J.hover_line.doit ? J.hover_line : J.line;
      this.poly.n >= 3 && (q.closePath(), Z.apply(q), te.apply(q)), ne.apply(q);
    }
    interactive_hit(H, V) {
      return !this.model.visible || !this.model.editable ? !1 : this.poly.contains(H, V);
    }
    _hit_test(H, V) {
      const { abs: q } = Math, J = Math.max(2.5, this.model.line_width / 2);
      for (const [ee, ae, oe] of this.poly)
        if (q(ee - H) < J && q(ae - V) < J)
          return { type: "node", i: oe };
      const Z = { x: H, y: V };
      let te = null, ne = 1 / 0;
      for (const [ee, ae, oe] of this.poly.edges()) {
        const se = dist_to_segment(Z, ee, ae);
        se < J && se < ne && (ne = se, te = oe);
      }
      return te != null ? { type: "edge", i: te } : this.poly.contains(H, V) ? { type: "area" } : null;
    }
    _can_hit(H) {
      return !0;
    }
    on_pan_start(H) {
      if (this.model.visible && this.model.editable) {
        const { sx: V, sy: q } = H, G = this._hit_test(V, q);
        if (G != null && this._can_hit(G))
          return this._pan_state = {
            poly: this.poly.clone(),
            target: G
          }, this.model.pan.emit(["pan:start", H.modifiers]), !0;
      }
      return !1;
    }
    on_pan(H) {
      assert$2(this._pan_state != null);
      const V = (() => {
        const { poly: te, target: ne } = this._pan_state, { dx: ee, dy: ae } = H;
        switch (ne.type) {
          case "node": {
            const { i: oe } = ne;
            return te.translate(ee, ae, oe);
          }
          case "edge": {
            const { i: oe } = ne;
            return te.translate(ee, ae, oe, oe + 1);
          }
          case "area":
            return te.translate(ee, ae);
        }
      })(), { x: q, y: G } = this._mappers(), J = q.v_invert(V.xs), Z = G.v_invert(V.ys);
      this.model.update({ xs: J, ys: Z }), this.model.pan.emit(["pan", H.modifiers]);
    }
    on_pan_end(H) {
      this._pan_state = null, this.model.pan.emit(["pan:end", H.modifiers]);
    }
    get _has_hover() {
      const { hover_line: H, hover_fill: V, hover_hatch: q } = this.visuals;
      return H.doit || V.doit || q.doit;
    }
    on_enter(H) {
      const { _has_hover: V } = this;
      return V && (this._is_hovered = !0, this.request_paint()), V;
    }
    on_move(H) {
    }
    on_leave(H) {
      this._has_hover && (this._is_hovered = !1, this.request_paint());
    }
    cursor(H, V) {
      var G;
      const q = ((G = this._pan_state) == null ? void 0 : G.target) ?? this._hit_test(H, V);
      if (q == null || !this._can_hit(q))
        return null;
      switch (q.type) {
        case "node":
          return "move";
        case "edge":
          return "move";
        case "area":
          return "move";
      }
    }
  }
  mt(PolyAnnotationView, "__name__", "PolyAnnotationView");
  const o0 = class o0 extends Annotation$1 {
    constructor(H) {
      super(H);
      mt(this, "pan", new Signal(this, "pan"));
    }
    update({ xs: H, ys: V }) {
      this.setv({ xs: H.slice(), ys: V.slice(), visible: !0 });
    }
    clear() {
      this.setv({ xs: [], ys: [], visible: !1 });
    }
  };
  mt(o0, "__name__", "PolyAnnotation"), o0.prototype.default_view = PolyAnnotationView, o0.mixins([
    Line$4,
    Fill$2,
    Hatch$1,
    ["hover_", Line$4],
    ["hover_", Fill$2],
    ["hover_", Hatch$1]
  ]), o0.define(({ Bool: H, Float: V, Arrayable: q }) => ({
    xs: [q(V), []],
    ys: [q(V), []],
    xs_units: [CoordinateUnits, "data"],
    ys_units: [CoordinateUnits, "data"],
    editable: [H, !1]
  })), o0.override({
    fill_color: "#fff9ba",
    fill_alpha: 0.4,
    line_color: "#cccccc",
    line_alpha: 0.3,
    hover_fill_color: null,
    hover_fill_alpha: 0.4,
    hover_line_color: null,
    hover_line_alpha: 0.3
  });
  let PolyAnnotation = o0;
  const { min: min$6 } = Math, h5 = class h5 extends Model$1 {
    constructor(_) {
      super(_);
    }
    compute(_, H, V) {
      const q = (() => {
        const { include: se, exclude: ce } = this, le = entries$2(this.get_basis()).map(([ue, [de, fe, he]]) => ({ name: ue, factor: de, tex_name: fe, long_name: he })).filter(({ name: ue }) => (se == null || se.includes(ue)) && !ce.includes(ue));
        return sort_by(le, ({ factor: ue }) => ue);
      })(), { ticks: G } = this, J = q.find(({ name: se }) => se == H);
      assert$2(J != null);
      const Z = _ * J.factor, [te, ne] = (() => {
        const se = bisect_right_by(q, Z, ({ factor: ce }) => ce);
        if (se > 0) {
          const { name: ce, factor: le } = q[se - 1], ue = Z / le;
          return [ce, ue];
        } else
          return [H, Z];
      })();
      V = V ?? G.length == 0;
      const ee = (() => {
        if (V)
          return ne;
        {
          const se = bisect_right(G, ne);
          return G[min$6(se, G.length - 1)];
        }
      })(), oe = ee * (Z / ne) / _ / J.factor;
      return {
        new_value: ee,
        new_unit: te,
        scale_factor: oe,
        exact: V
      };
    }
  };
  mt(h5, "__name__", "Dimensional"), h5.define(({ Nullable: _, List: H, Str: V, Float: q }) => ({
    ticks: [H(q)],
    include: [_(H(V)), null],
    exclude: [H(V), []]
  }));
  let Dimensional = h5;
  const p5 = class p5 extends Dimensional {
    constructor(_) {
      super(_);
    }
    get_basis() {
      return this.basis;
    }
  };
  mt(p5, "__name__", "CustomDimensional"), p5.define(({ Dict: _, Tuple: H, Float: V, Str: q, Or: G }) => ({
    basis: [_(G(H(V, q), H(V, q, q)))]
  }));
  let CustomDimensional = p5;
  const s0 = class s0 extends Dimensional {
    constructor(_) {
      super(_);
    }
    get_basis() {
      const { base_unit: _, full_unit: H } = this, V = {};
      for (const [q, G, J, Z] of s0._metric_basis) {
        const te = `${q}${_}`, ne = `${J}${_}`, ee = H != null ? `${Z}${H}` : void 0;
        V[te] = [G, ne, ee];
      }
      return V;
    }
  };
  mt(s0, "__name__", "Metric"), s0.define(({ Str: _, Nullable: H }) => ({
    base_unit: [_],
    full_unit: [H(_), null]
  })), s0.override({
    ticks: [1, 2, 5, 10, 15, 20, 25, 50, 75, 100, 125, 150, 200, 250, 500, 750]
  }), mt(s0, "_metric_basis", [
    ["Q", 1e30, "Q", "quetta"],
    ["R", 1e27, "R", "ronna"],
    ["Y", 1e24, "Y", "yotta"],
    ["Z", 1e21, "Z", "zetta"],
    ["E", 1e18, "E", "exa"],
    ["P", 1e15, "P", "peta"],
    ["T", 1e12, "T", "tera"],
    ["G", 1e9, "G", "giga"],
    ["M", 1e6, "M", "mega"],
    ["k", 1e3, "k", "kilo"],
    ["h", 100, "h", "hecto"],
    ["", 1, "", ""],
    ["d", 0.1, "d", "deci"],
    ["c", 0.01, "c", "centi"],
    ["m", 1e-3, "m", "milli"],
    ["µ", 1e-6, "\\mu", "micro"],
    ["n", 1e-9, "n", "nano"],
    ["p", 1e-12, "p", "pico"],
    ["f", 1e-15, "f", "femto"],
    ["a", 1e-18, "a", "atto"],
    ["z", 1e-21, "z", "zepto"],
    ["y", 1e-24, "y", "yocto"],
    ["r", 1e-27, "r", "ronto"],
    ["q", 1e-30, "q", "quecto"]
  ]);
  let Metric = s0;
  class ReciprocalMetric extends Metric {
    constructor(_) {
      super(_);
    }
    get_basis() {
      const _ = super.get_basis(), H = {};
      for (const [V, [q, G]] of entries$2(_))
        H[`${V}⁻1`] = [q ** -1, `${G}^{-1}`];
      return H;
    }
  }
  mt(ReciprocalMetric, "__name__", "ReciprocalMetric");
  const m5 = class m5 extends Metric {
    constructor(_) {
      super(_);
    }
  };
  mt(m5, "__name__", "MetricLength"), m5.override({
    base_unit: "m",
    exclude: ["dm", "hm"]
  });
  let MetricLength = m5;
  const g5 = class g5 extends ReciprocalMetric {
    constructor(_) {
      super(_);
    }
  };
  mt(g5, "__name__", "ReciprocalMetricLength"), g5.override({
    base_unit: "m",
    exclude: ["dm", "hm"]
  });
  let ReciprocalMetricLength = g5;
  const _5 = class _5 extends CustomDimensional {
    constructor(_) {
      super(_);
    }
  };
  mt(_5, "__name__", "ImperialLength"), _5.override({
    basis: {
      in: [1 / 12, "in", "inch"],
      ft: [1, "ft", "foot"],
      yd: [3, "yd", "yard"],
      ch: [66, "ch", "chain"],
      fur: [660, "fur", "furlong"],
      mi: [5280, "mi", "mile"],
      lea: [15840, "lea", "league"]
    },
    ticks: [1, 3, 6, 12, 60]
  });
  let ImperialLength = _5;
  const y5 = class y5 extends CustomDimensional {
    constructor(_) {
      super(_);
    }
  };
  mt(y5, "__name__", "Angular"), y5.override({
    basis: {
      "°": [1, "^\\circ", "degree"],
      "'": [1 / 60, "^\\prime", "minute"],
      "''": [1 / 3600, "^{\\prime\\prime}", "second"]
    },
    ticks: [1, 3, 6, 12, 60, 120, 240, 360]
  });
  let Angular = y5;
  const { round } = Math, LengthSizing = Enum("adaptive", "exact");
  class ScaleBarView extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "bbox", new BBox$1());
      mt(this, "label_layout");
      mt(this, "title_layout");
      mt(this, "axis_layout");
      mt(this, "box_layout");
      mt(this, "axis");
      mt(this, "axis_view");
      mt(this, "axis_scale");
      mt(this, "cross_scale");
      mt(this, "range");
    }
    _get_size() {
      const { width: H, height: V } = this.bbox, { margin: q } = this.model;
      return {
        width: H + 2 * q,
        height: V + 2 * q
      };
    }
    initialize() {
      super.initialize();
      const { ticker: H } = this.model;
      this.axis = new LinearAxis({
        ticker: H,
        ...attrs_of(this.model, "bar_", Line$4, "axis_")
      }), this.range = (() => {
        const { range: V, orientation: q } = this.model;
        if (V == "auto") {
          const { frame: G } = this.parent;
          switch (q) {
            case "horizontal":
              return G.x_range;
            case "vertical":
              return G.y_range;
          }
        } else
          return V;
      })();
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const H = (() => {
        const V = new Range1d(), q = new Range1d(), G = new Range1d(), J = new Range1d();
        return this.axis_scale = new LinearScale({ source_range: V, target_range: q }), this.cross_scale = new LinearScale({ source_range: G, target_range: J }), this.model.orientation == "horizontal" ? new CoordinateTransform$1(this.axis_scale, this.cross_scale) : new CoordinateTransform$1(this.cross_scale, this.axis_scale);
      })();
      this.axis_view = await build_view(this.axis, { parent: this.plot_view }), this.axis_view.coordinates = H, this.axis_view.panel = new SidePanel(this.model.orientation == "horizontal" ? "below" : "right"), this.axis_view.update_layout();
    }
    remove() {
      this.axis_view.remove(), super.remove();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => {
        this.request_render();
      }), this.connect(this.range.change, () => {
        this.request_render();
      });
    }
    update_layout() {
      this.update_geometry();
      const { panel: H } = this;
      H != null ? this.layout = new SideLayout(H, () => this.get_size()) : this.layout = void 0;
    }
    update_geometry() {
      super.update_geometry();
    }
    get horizontal() {
      return this.model.orientation == "horizontal";
    }
    text_layout(H) {
      const { text: V, location: q, align: G, visuals: J } = H, { orientation: Z } = this.model, te = new TextBox({ text: V }), ne = new SidePanel(q);
      te.visuals = J.values();
      const ee = (() => {
        switch (q) {
          case "above":
          case "below":
            return "horizontal";
          default:
            return Z;
        }
      })();
      te.angle = ne.get_label_angle_heuristic(ee), te.base_font_size = this.plot_view.base_font_size, te.position = {
        sx: 0,
        sy: 0,
        x_anchor: "left",
        y_anchor: "top"
      }, te.align = "auto";
      const ae = new TextLayout(te);
      ae.absolute = !0;
      const oe = Z == "horizontal", se = oe ? G : void 0, ce = oe ? void 0 : G;
      return ae.set_sizing({
        width_policy: "min",
        height_policy: "min",
        visible: V != "" && J.doit,
        halign: se,
        valign: ce
      }), ae;
    }
    compute_geometry() {
      super.compute_geometry();
      const { orientation: H, bar_length: V, length_sizing: q, padding: G, margin: J, location: Z } = this.model, { border_line: te, bar_line: ne } = this.visuals, ee = ne.line_width.get_value(), ae = te.line_width.get_value(), { frame: oe } = this.parent, se = H == "horizontal" ? oe.bbox.width : oe.bbox.height, ce = 0 <= V && V <= 1 ? V : clamp$1(V / se, 0, 1), { new_value: le, new_unit: ue, scale_factor: de, exact: fe } = (() => {
        const { unit: Me, dimensional: Re } = this.model, De = this.range.span * ce;
        return Re.compute(De, Me, q == "exact");
      })(), he = se * ce, me = round(he * de), pe = (() => {
        const { label: Me } = this.model;
        return process_placeholders(Me, (Re, De, Pe) => {
          switch (De) {
            case "value":
              return fe ? Pe != null ? sprintf(Pe, le) : le.toFixed(2) : `${le}`;
            case "unit":
              switch (Pe ?? "short") {
                case "short":
                  return ue;
              }
            default:
              return null;
          }
        });
      })();
      this.label_layout = this.text_layout({
        text: pe,
        location: this.model.label_location,
        align: this.model.label_align,
        visuals: this.visuals.label_text
      }), this.title_layout = this.text_layout({
        text: this.model.title,
        location: this.model.title_location,
        align: this.model.title_align,
        visuals: this.visuals.title_text
      });
      const ye = H == "horizontal" ? { width: me, height: ee } : { width: ee, height: me }, ge = this.axis_view.layout;
      assert$2(ge != null), this.axis_layout = ge, ge.absolute = !0, H == "horizontal" ? ge.set_sizing({
        width_policy: "fixed",
        width: ye.width,
        height_policy: "min",
        valign: "center"
      }) : ge.set_sizing({
        width_policy: "min",
        height_policy: "fixed",
        height: ye.height,
        halign: "center"
      }), this.box_layout = (() => {
        const Me = {
          above: [],
          below: [],
          left: [],
          right: []
        };
        function Re(Xe, Ve) {
          const Ze = new FixedLayout();
          return Ze.absolute = !0, Ze.set_sizing(Xe == "left" || Xe == "right" ? { width_policy: "fixed", width: Ve } : { height_policy: "fixed", height: Ve }), Ze;
        }
        function De(Xe, Ve, Ze) {
          Xe.visible && Me[Ve].push(Re(Ve, Ze), Xe);
        }
        De(this.label_layout, this.model.label_location, this.model.label_standoff), De(this.title_layout, this.model.title_location, this.model.title_standoff);
        const Pe = Me.above.length, $e = Me.left.length, qe = [
          { layout: ge, row: Pe, col: $e }
        ];
        for (const [Xe, Ve] of enumerate(Me.above))
          qe.push({ layout: Xe, row: Pe - Ve - 1, col: $e });
        for (const [Xe, Ve] of enumerate(Me.below))
          qe.push({ layout: Xe, row: Pe + Ve + 1, col: $e });
        for (const [Xe, Ve] of enumerate(Me.left))
          qe.push({ layout: Xe, row: Pe, col: $e - Ve - 1 });
        for (const [Xe, Ve] of enumerate(Me.right))
          qe.push({ layout: Xe, row: Pe, col: $e + Ve + 1 });
        return new Grid$1(qe);
      })();
      const { box_layout: ve } = this;
      ve.absolute = !0, ve.position = { left: G, top: G }, ve.set_sizing(), ve.compute();
      const [be, Ce] = (() => {
        const { x_range: Me, y_range: Re } = this.axis_view.bbox;
        return H == "horizontal" ? [Me, Re] : [Re, Me];
      })();
      this.axis_scale.source_range.end = le, this.axis_scale.target_range.setv(be), this.cross_scale.source_range.end = 1, this.cross_scale.target_range.setv(Ce);
      const Se = ae + G + ve.bbox.width + G + ae, we = ae + G + ve.bbox.height + G + ae, Ee = this.layout != null ? this.layout : this.plot_view.frame, [xe, Le] = Ee.bbox.ranges, { sx: Qe, sy: ke } = (() => {
        switch (Z) {
          case "top_left":
            return {
              sx: xe.start + J,
              sy: Le.start + J
            };
          case "top":
          case "top_center":
            return {
              sx: (xe.end + xe.start) / 2 - Se / 2,
              sy: Le.start + J
            };
          case "top_right":
            return {
              sx: xe.end - J - Se,
              sy: Le.start + J
            };
          case "bottom_right":
            return {
              sx: xe.end - J - Se,
              sy: Le.end - J - we
            };
          case "bottom":
          case "bottom_center":
            return {
              sx: (xe.end + xe.start) / 2 - Se / 2,
              sy: Le.end - J - we
            };
          case "bottom_left":
            return {
              sx: xe.start + J,
              sy: Le.end - J - we
            };
          case "left":
          case "center_left":
            return {
              sx: xe.start + J,
              sy: (Le.end + Le.start) / 2 - we / 2
            };
          case "center":
          case "center_center":
            return {
              sx: (xe.end + xe.start) / 2 - Se / 2,
              sy: (Le.end + Le.start) / 2 - we / 2
            };
          case "right":
          case "center_right":
            return {
              sx: xe.end - J - Se,
              sy: (Le.end + Le.start) / 2 - we / 2
            };
        }
      })();
      this.bbox = new BBox$1({ left: Qe, top: ke, width: Se, height: we });
    }
    _draw_box(H) {
      const { width: V, height: q } = this.bbox;
      H.beginPath(), H.rect(0, 0, V, q), this.visuals.background_fill.apply(H), this.visuals.background_hatch.apply(H), this.visuals.border_line.apply(H);
    }
    _draw_axis(H) {
      this.axis_view.render();
    }
    _draw_text(H, V, q) {
      const { bbox: G } = V, [J, Z] = (() => {
        const { orientation: ee } = this.model, ae = ee == "horizontal";
        switch (q) {
          case "left":
            return ae ? [0, 0] : [0, G.height];
          case "right":
            return ae ? [0, 0] : [G.width, 0];
          case "above":
            return [0, 0];
          case "below":
            return [0, 0];
        }
      })(), { left: te, top: ne } = G.translate(J, Z);
      H.translate(te, ne), V.text.paint(H), H.translate(-te, -ne);
    }
    _draw_label(H) {
      this._draw_text(H, this.label_layout, this.model.label_location);
    }
    _draw_title(H) {
      this._draw_text(H, this.title_layout, this.model.title_location);
    }
    _render() {
      this.update_geometry(), this.compute_geometry();
      const { ctx: H } = this.layer, { left: V, top: q } = this.bbox;
      H.translate(V, q), this.box_layout.visible && this._draw_box(H), this.axis_layout.visible && this._draw_axis(H), this.label_layout.visible && this._draw_label(H), this.title_layout.visible && this._draw_title(H), H.translate(-V, -q);
    }
  }
  mt(ScaleBarView, "__name__", "ScaleBarView");
  const l0 = class l0 extends Annotation$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(l0, "__name__", "ScaleBar"), l0.prototype.default_view = ScaleBarView, l0.mixins([
    ["bar_", Line$4],
    ["label_", Text$4],
    ["title_", Text$4],
    ["border_", Line$4],
    ["background_", Fill$2],
    ["background_", Hatch$1]
  ]), l0.define(({ NonNegative: _, Float: H, Str: V, Ref: q, Or: G, Auto: J }) => ({
    range: [G(q(Range$2), J), "auto"],
    unit: [V, "m"],
    dimensional: [q(Dimensional), () => new MetricLength()],
    orientation: [Orientation, "horizontal"],
    bar_length: [_(H), 0.2],
    length_sizing: [LengthSizing, "adaptive"],
    location: [Anchor$2, "top_right"],
    label: [V, "@{value} @{unit}"],
    label_align: [Align, "center"],
    label_location: [Location, "below"],
    label_standoff: [H, 5],
    title: [V, ""],
    title_align: [Align, "center"],
    title_location: [Location, "above"],
    title_standoff: [H, 5],
    margin: [H, 10],
    padding: [H, 10],
    ticker: [q(Ticker), () => new FixedTicker({ ticks: [] })]
  })), l0.override({
    bar_line_width: 2,
    border_line_color: "#e5e5e5",
    border_line_alpha: 0.5,
    border_line_width: 1,
    background_fill_color: "#ffffff",
    background_fill_alpha: 0.95,
    label_text_font_size: "13px",
    label_text_baseline: "middle",
    title_text_font_size: "13px",
    title_text_font_style: "italic"
  });
  let ScaleBar = l0;
  class SlopeView extends AnnotationView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.request_render());
    }
    _render() {
      const { gradient: _, y_intercept: H } = this.model;
      if (_ == null || H == null)
        return;
      const { frame: V } = this.plot_view, q = this.coordinates.x_scale, G = this.coordinates.y_scale, [J, Z, te, ne] = (() => {
        if (_ == 0) {
          const ae = G.compute(H), oe = ae, se = V.bbox.left, ce = V.bbox.right;
          return [se, ce, ae, oe];
        } else {
          const ae = V.bbox.top, oe = V.bbox.bottom, se = G.invert(ae), ce = G.invert(oe), le = (se - H) / _, ue = (ce - H) / _, de = q.compute(le), fe = q.compute(ue);
          return de <= fe ? [de, fe, ae, oe] : [fe, de, oe, ae];
        }
      })(), { ctx: ee } = this.layer;
      if (ee.save(), this.visuals.above_fill.doit || this.visuals.above_hatch.doit) {
        const { left: ae, right: oe, top: se, bottom: ce } = V.bbox;
        ee.beginPath(), ee.moveTo(J, te), ee.lineTo(J, te), ee.lineTo(Z, ne), ee.lineTo(Z, ne), te <= ne ? (Z < oe && ee.lineTo(oe, ce), ee.lineTo(oe, se), ee.lineTo(ae, se)) : (ee.lineTo(oe, se), ee.lineTo(ae, se), J > ae && ee.lineTo(ae, ce)), ee.closePath(), this.visuals.above_fill.apply(ee), this.visuals.above_hatch.apply(ee);
      }
      if (this.visuals.below_fill.doit || this.visuals.below_hatch.doit) {
        const { left: ae, right: oe, top: se, bottom: ce } = V.bbox;
        ee.beginPath(), ee.moveTo(J, te), ee.lineTo(J, te), ee.lineTo(Z, ne), te <= ne ? (ee.lineTo(oe, ce), ee.lineTo(ae, ce), J > ae && ee.lineTo(ae, se)) : (Z < oe && ee.lineTo(oe, se), ee.lineTo(oe, ce), ee.lineTo(ae, ce)), ee.closePath(), this.visuals.below_fill.apply(ee), this.visuals.below_hatch.apply(ee);
      }
      ee.beginPath(), ee.moveTo(J, te), ee.lineTo(Z, ne), this.visuals.line.apply(ee), ee.restore();
    }
  }
  mt(SlopeView, "__name__", "SlopeView");
  const c0 = class c0 extends Annotation$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(c0, "__name__", "Slope"), c0.prototype.default_view = SlopeView, c0.mixins([
    Line$4,
    ["above_", Fill$2],
    ["above_", Hatch$1],
    ["below_", Fill$2],
    ["below_", Hatch$1]
  ]), c0.define(({ Float: _, Nullable: H }) => ({
    gradient: [H(_), null],
    y_intercept: [H(_), null]
  })), c0.override({
    line_color: "black",
    above_fill_color: null,
    above_fill_alpha: 0.4,
    below_fill_color: null,
    below_fill_alpha: 0.4
  });
  let Slope = c0;
  const EDGE_TOLERANCE = 2.5;
  var M2;
  let Line$1 = (M2 = class {
    constructor(_, H) {
      mt(this, "p0");
      mt(this, "p1");
      this.p0 = _, this.p1 = H;
    }
    clone() {
      return new M2({ ...this.p0 }, { ...this.p1 });
    }
    hit_test(_, H = EDGE_TOLERANCE) {
      return dist_to_segment(_, this.p0, this.p1) < H;
    }
    translate(_, H) {
      const { p0: V, p1: q } = this, G = { x: V.x + _, y: V.y + H }, J = { x: q.x + _, y: q.y + H };
      return new M2(G, J);
    }
  }, mt(M2, "__name__", "Line"), M2);
  var oy;
  let SpanView$1 = (oy = class extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "line");
      mt(this, "_pan_state", null);
      mt(this, "_is_hovered", !1);
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.plot_view.request_paint(this));
    }
    _render() {
      const { location: H, location_units: V } = this.model;
      if (H == null)
        return;
      function q(he, me, pe, ye, ge) {
        switch (me) {
          case "canvas":
            return ge.compute(he);
          case "screen":
            return ye.compute(he);
          case "data":
            return pe.compute(he);
        }
      }
      const { frame: G, canvas: J } = this.plot_view, { x_scale: Z, y_scale: te } = this.coordinates;
      let ne, ee, ae, oe;
      this.model.dimension == "width" ? (ae = q(H, V, te, G.bbox.yview, J.bbox.y_screen), ee = G.bbox.left, oe = G.bbox.width, ne = this.model.line_width) : (ae = G.bbox.top, ee = q(H, V, Z, G.bbox.xview, J.bbox.y_screen), oe = this.model.line_width, ne = G.bbox.height);
      const se = { x: ee, y: ae }, ce = { x: ee + oe, y: ae + ne };
      this.line = new Line$1(se, ce);
      const { _is_hovered: le, visuals: ue } = this, de = le && ue.hover_line.doit ? ue.hover_line : ue.line, { ctx: fe } = this.layer;
      fe.save(), fe.beginPath(), this.visuals.line.set_value(fe), fe.moveTo(ee, ae), this.model.dimension == "width" ? fe.lineTo(ee + oe, ae) : fe.lineTo(ee, ae + ne), de.apply(fe), fe.restore();
    }
    interactive_hit(H, V) {
      return !this.model.visible || !this.model.editable ? !1 : this._hit_test(H, V) != null;
    }
    _hit_test(H, V) {
      const q = Math.max(EDGE_TOLERANCE, this.model.line_width / 2);
      return this.line.hit_test({ x: H, y: V }, q) ? "edge" : null;
    }
    _can_hit(H) {
      return !0;
    }
    on_pan_start(H) {
      if (this.model.visible && this.model.editable) {
        const { sx: V, sy: q } = H, G = this._hit_test(V, q);
        if (G != null && this._can_hit(G))
          return this._pan_state = {
            line: this.line.clone(),
            target: G
          }, this.model.pan.emit(["pan:start", H.modifiers]), !0;
      }
      return !1;
    }
    on_pan(H) {
      assert$2(this._pan_state != null);
      function V(J, Z, te, ne, ee) {
        switch (Z) {
          case "canvas":
            return ee.invert(J);
          case "screen":
            return ne.invert(J);
          case "data":
            return te.invert(J);
        }
      }
      const q = (() => {
        const { dx: J, dy: Z } = H, { line: te } = this._pan_state;
        return this.model.dimension == "width" ? te.translate(0, Z).p0.y : te.translate(J, 0).p0.x;
      })(), G = (() => {
        const { location_units: J } = this.model, { frame: Z, canvas: te } = this.plot_view, { x_scale: ne, y_scale: ee } = this.coordinates;
        return this.model.dimension == "width" ? V(q, J, ee, Z.bbox.yview, te.bbox.y_screen) : V(q, J, ne, Z.bbox.xview, te.bbox.y_screen);
      })();
      this.model.location = G, this.model.pan.emit(["pan", H.modifiers]);
    }
    on_pan_end(H) {
      this._pan_state = null, this.model.pan.emit(["pan:end", H.modifiers]);
    }
    get _has_hover() {
      const { hover_line: H } = this.visuals;
      return H.doit;
    }
    on_enter(H) {
      const { _has_hover: V } = this;
      return V && (this._is_hovered = !0, this.request_paint()), V;
    }
    on_move(H) {
    }
    on_leave(H) {
      this._has_hover && (this._is_hovered = !1, this.request_paint());
    }
    cursor(H, V) {
      var G;
      const q = ((G = this._pan_state) == null ? void 0 : G.target) ?? this._hit_test(H, V);
      return q == null || !this._can_hit(q) ? null : this.model.dimension == "width" ? "ns-resize" : "ew-resize";
    }
  }, mt(oy, "__name__", "SpanView"), oy);
  var Gh;
  let Span$3 = (Gh = class extends Annotation$1 {
    constructor(H) {
      super(H);
      mt(this, "pan", new Signal(this, "pan"));
    }
  }, mt(Gh, "__name__", "Span"), Gh.prototype.default_view = SpanView$1, Gh.mixins([
    Line$4,
    ["hover_", Line$4]
  ]), Gh.define(({ Bool: H, Float: V, Nullable: q }) => ({
    location: [q(V), null],
    location_units: [CoordinateUnits, "data"],
    dimension: [Dimension$1, "width"],
    editable: [H, !1]
  })), Gh.override({
    line_color: "black",
    hover_line_color: null,
    hover_line_alpha: 0.3
  }), Gh);
  class ToolView extends View$2 {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.active.change, () => {
        this.model.active ? this.activate() : this.deactivate();
      });
    }
    get overlays() {
      return [];
    }
    // activate is triggered by toolbar ui actions
    activate() {
    }
    // deactivate is triggered by toolbar ui actions
    deactivate() {
    }
  }
  mt(ToolView, "__name__", "ToolView");
  const wp = class wp extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "tool_name");
      mt(this, "tool_icon");
      // GestureTool {{{
      mt(this, "event_type");
      // }}}
      mt(this, "button_view");
      /** @prototype */
      mt(this, "_known_aliases");
    }
    get event_role() {
      const { event_type: H } = this;
      return isString$2(H) ? H : "multi";
    }
    get event_types() {
      const { event_type: H } = this;
      return H == null ? [] : isString$2(H) ? [H] : H;
    }
    get tooltip() {
      return this.description ?? this.tool_name;
    }
    get computed_icon() {
      const { icon: H, tool_icon: V } = this;
      return H ?? (V != null ? `.${V}` : void 0);
    }
    get menu() {
      return null;
    }
    // utility function to get limits along both dimensions, given
    // optional dimensional constraints
    _get_dim_limits([H, V], [q, G], J, Z) {
      const te = J.bbox.h_range;
      let ne;
      Z == "width" || Z == "both" ? (ne = [min$b([H, q]), max$j([H, q])], ne = [max$j([ne[0], te.start]), min$b([ne[1], te.end])]) : ne = [te.start, te.end];
      const ee = J.bbox.v_range;
      let ae;
      return Z == "height" || Z == "both" ? (ae = [min$b([V, G]), max$j([V, G])], ae = [max$j([ae[0], ee.start]), min$b([ae[1], ee.end])]) : ae = [ee.start, ee.end], [ne, ae];
    }
    // utility function to return a tool name, modified
    // by the active dimensions. Used by tools that have dimensions
    _get_dim_tooltip(H) {
      const { description: V, tool_name: q } = this;
      return V ?? (H == "both" ? q : H == "auto" ? `${q} (either x, y or both dimensions)` : `${q} (${H == "width" ? "x" : "y"}-axis)`);
    }
    static register_alias(H, V) {
      this.prototype._known_aliases.set(H, V);
    }
    static from_string(H) {
      const V = this.prototype._known_aliases.get(H);
      if (V != null)
        return V();
      {
        const q = [...this.prototype._known_aliases.keys()];
        throw new Error(`unexpected tool name '${H}', possible tools are ${q.join(", ")}`);
      }
    }
  };
  mt(wp, "__name__", "Tool"), wp.prototype._known_aliases = /* @__PURE__ */ new Map(), wp.define(({ Bool: H, Str: V, Regex: q, Nullable: G, Or: J }) => ({
    icon: [G(J(ToolIcon, q(/^--/), q(/^\./), q(/^data:image/))), null],
    description: [G(V), null],
    visible: [H, !0]
  })), wp.internal(({ Bool: H }) => ({
    active: [H, !1],
    disabled: [H, !1]
  }));
  let Tool = wp;
  const v5 = class v5 extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "do");
    }
    // Operates all the tools given only one button
    get underlying() {
      return this.tools[0];
    }
    tool_button() {
      const H = this.tools[0].tool_button();
      return H.tool = this, H;
    }
    get event_type() {
      return this.tools[0].event_type;
    }
    get event_role() {
      return this.tools[0].event_role;
    }
    get event_types() {
      return this.tools[0].event_types;
    }
    get default_order() {
      return this.tools[0].default_order;
    }
    get tooltip() {
      return this.tools[0].tooltip;
    }
    get tool_name() {
      return this.tools[0].tool_name;
    }
    get computed_icon() {
      return this.tools[0].computed_icon;
    }
    get toggleable() {
      const H = this.tools[0];
      return "toggleable" in H && H.toggleable;
    }
    get visible() {
      return this.tools[0].visible;
    }
    initialize() {
      super.initialize(), this.do = new Signal0(this, "do");
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.do, () => this.doit()), this.connect(this.properties.active.change, () => this.set_active());
      for (const H of this.tools)
        this.connect(H.properties.active.change, () => {
          this.active = H.active;
        });
    }
    doit() {
      for (const H of this.tools)
        H.do.emit();
    }
    set_active() {
      for (const H of this.tools)
        H.active = this.active;
    }
    get menu() {
      const { menu: H } = this.tools[0];
      if (H == null)
        return null;
      const V = [];
      for (const [q, G] of enumerate(H))
        if (q == null)
          V.push(null);
        else {
          const J = () => {
            var Z, te, ne;
            for (const ee of this.tools)
              (ne = (te = (Z = ee.menu) == null ? void 0 : Z[G]) == null ? void 0 : te.handler) == null || ne.call(te);
          };
          V.push({ ...q, handler: J });
        }
      return V;
    }
  };
  mt(v5, "__name__", "ToolProxy"), v5.define(({ Bool: H, List: V, Ref: q }) => ({
    tools: [V(q(Tool)), []],
    active: [H, (G) => some$1(G.tools, (J) => J.active)],
    disabled: [H, !1]
  }));
  let ToolProxy = v5;
  const Wf = class Wf {
    constructor(_, H, V = {}) {
      mt(this, "hit_area");
      mt(this, "handlers");
      mt(this, "must_be_target");
      mt(this, "phase", "idle");
      mt(this, "pointers", /* @__PURE__ */ new Map());
      mt(this, "press_timer", null);
      mt(this, "tap_timestamp", -1 / 0);
      mt(this, "last_scale", null);
      mt(this, "last_rotation", null);
      this.hit_area = _, this.handlers = H, this.must_be_target = V.must_be_target ?? !1, this._pointer_over = this._pointer_over.bind(this), this._pointer_out = this._pointer_out.bind(this), this._pointer_down = this._pointer_down.bind(this), this._pointer_move = this._pointer_move.bind(this), this._pointer_up = this._pointer_up.bind(this), this._pointer_cancel = this._pointer_cancel.bind(this);
    }
    connect_signals() {
      this.hit_area.addEventListener("pointerover", this._pointer_over), this.hit_area.addEventListener("pointerout", this._pointer_out), this.hit_area.addEventListener("pointerdown", this._pointer_down), this.hit_area.addEventListener("pointermove", this._pointer_move), this.hit_area.addEventListener("pointerup", this._pointer_up), this.hit_area.addEventListener("pointercancel", this._pointer_cancel);
    }
    disconnect_signals() {
      this.hit_area.removeEventListener("pointerover", this._pointer_over), this.hit_area.removeEventListener("pointerout", this._pointer_out), this.hit_area.removeEventListener("pointerdown", this._pointer_down), this.hit_area.removeEventListener("pointermove", this._pointer_move), this.hit_area.removeEventListener("pointerup", this._pointer_up), this.hit_area.removeEventListener("pointercancel", this._pointer_cancel);
    }
    remove() {
      this.disconnect_signals();
    }
    _self_is_target(_) {
      return _.composedPath()[0] == this.hit_area;
    }
    _is_event_target(_) {
      return !this.must_be_target || this._self_is_target(_);
    }
    /*rad*/
    get _is_multi_gesture() {
      return this.pointers.size >= 2;
    }
    _within_threshold(_) {
      const { dx: H, dy: V } = this._movement(_);
      return H ** 2 + V ** 2 <= Wf.move_threshold ** 2;
    }
    get _any_movement() {
      return [...this.pointers.values()].some((_) => !this._within_threshold(_));
    }
    _start_timeout() {
      assert$2(this.press_timer == null), this.press_timer = setTimeout(() => this._pointer_timeout(), Wf.press_threshold);
    }
    _cancel_timeout() {
      const { press_timer: _ } = this;
      _ != null && (clearTimeout(_), this.press_timer = null);
    }
    _pointer_timeout() {
      assert$2(this.phase == "started"), assert$2(!this._is_multi_gesture), this.phase = "pressing", this.press_timer = null;
      const [_] = this.pointers.values();
      this.on_press(_.init);
    }
    _pointer_over(_) {
      this._is_event_target(_) && _.isPrimary && this.on_enter(_);
    }
    _pointer_out(_) {
      this._is_event_target(_) && _.isPrimary && this.on_leave(_);
    }
    _pointer_down(_) {
      if (this._is_event_target(_) && !this._is_multi_gesture && !this.pointers.has(_.pointerId) && !(_.isPrimary && _.pointerType == "mouse" && _.buttons != MouseButton.Left))
        switch (this.pointers.set(_.pointerId, { init: _, last: _ }), this.hit_area.setPointerCapture(_.pointerId), this.phase) {
          case "idle": {
            this.phase = "started", this._start_timeout();
            break;
          }
          case "started": {
            this._cancel_timeout();
            break;
          }
        }
    }
    _pointer_move(_) {
      if (!this._is_event_target(_))
        return;
      _.isPrimary && this.on_move(_);
      const H = this.pointers.get(_.pointerId);
      if (H != null)
        switch (H.last = _, this.phase) {
          case "idle":
            unreachable();
          case "started":
          case "transitional": {
            if (!this._any_movement)
              return;
            if (this._cancel_timeout(), this._is_multi_gesture) {
              const [V, q] = this.pointers.values(), G = this._scale(V, q), J = this._rotation(V, q);
              Math.abs(G - 1) > Wf.pinch_threshold ? (this.phase = "pinching", this.on_pinch_start(V.init, q.init, 1), this.on_pinch(V.last, q.last, G), this.last_scale = G) : Math.abs(J) > Wf.rotate_threshold && (this.phase = "rotating", this.on_rotate_start(V.init, q.init, 0), this.on_rotate(q.last, q.last, J), this.last_rotation = J);
            } else {
              this.phase = "panning";
              const [V] = this.pointers.values(), { dx: q, dy: G } = this._movement(V);
              this.on_pan_start(V.init, 0, 0), this.on_pan(V.last, q, G);
            }
            break;
          }
          case "pressing":
            break;
          case "panning": {
            const [V] = this.pointers.values(), { dx: q, dy: G } = this._movement(V);
            this.on_pan(_, q, G);
            break;
          }
          case "pinching": {
            const [V, q] = this.pointers.values(), G = this._scale(V, q);
            G != this.last_scale && (this.on_pinch(V.last, q.last, G), this.last_scale = G);
            break;
          }
          case "rotating": {
            const [V, q] = this.pointers.values(), G = this._rotation(V, q);
            G != this.last_rotation && (this.on_rotate(V.last, q.last, G), this.last_rotation = G);
            break;
          }
        }
    }
    _pointer_up(_) {
      if (!this._is_event_target(_))
        return;
      const H = this.pointers.get(_.pointerId);
      if (H != null) {
        switch (H.last = _, this._cancel_timeout(), this.phase) {
          case "idle":
            unreachable();
          case "started": {
            const [V] = this.pointers.values(), { tap_timestamp: q } = this;
            V.last.timeStamp - q < Wf.doubletap_threshold ? (this.tap_timestamp = -1 / 0, this.on_doubletap(V.last)) : (this.tap_timestamp = V.last.timeStamp, this.on_tap(V.last)), this.phase = "idle";
            break;
          }
          case "transitional": {
            this.phase = "idle";
            break;
          }
          case "pressing": {
            const [V] = this.pointers.values();
            this.on_pressup(V.last), this.phase = "idle";
            break;
          }
          case "panning": {
            const [V] = this.pointers.values(), { dx: q, dy: G } = this._movement(V);
            this.on_pan_end(_, q, G), this.phase = "idle";
            break;
          }
          case "pinching": {
            const [V, q] = this.pointers.values(), G = this._scale(V, q);
            this.on_pinch_end(V.last, q.last, G), this.phase = "transitional", this.last_scale = null;
            break;
          }
          case "rotating": {
            const [V, q] = this.pointers.values(), G = this._rotation(V, q);
            this.on_rotate_end(V.last, q.last, G), this.phase = "transitional", this.last_rotation = null;
            break;
          }
        }
        if (this.pointers.delete(_.pointerId), this.phase == "transitional") {
          const [V] = this.pointers.values();
          V.init = V.last;
        }
      }
    }
    _pointer_cancel(_) {
      if (this.pointers.has(_.pointerId)) {
        switch (this._cancel_timeout(), this.phase) {
          case "idle":
            unreachable();
          case "started":
          case "pressing":
          case "transitional": {
            this.phase = "idle";
            break;
          }
          case "panning": {
            const [H] = this.pointers.values(), { dx: V, dy: q } = this._movement(H);
            this.on_pan_end(_, V, q), this.phase = "idle";
            break;
          }
          case "pinching": {
            const [H, V] = this.pointers.values(), q = this._scale(H, V);
            this.on_pinch_end(H.last, V.last, q), this.phase = "transitional", this.last_scale = null;
            break;
          }
          case "rotating": {
            const [H, V] = this.pointers.values(), q = this._rotation(H, V);
            this.on_rotate_end(H.last, V.last, q), this.phase = "transitional", this.last_rotation = null;
            break;
          }
        }
        if (this.pointers.delete(_.pointerId), this.phase == "transitional") {
          const [H] = this.pointers.values();
          H.init = H.last;
        }
      }
    }
    on_tap(_) {
      const { on_tap: H } = this.handlers;
      H != null && H(this._tap_event("tap", _));
    }
    on_doubletap(_) {
      const { on_doubletap: H } = this.handlers;
      H != null && H(this._tap_event("double_tap", _));
    }
    on_press(_) {
      const { on_press: H } = this.handlers;
      H != null && H(this._tap_event("press", _));
    }
    on_pressup(_) {
      const { on_pressup: H } = this.handlers;
      H != null && H(this._tap_event("press_up", _));
    }
    on_enter(_) {
      const { on_enter: H } = this.handlers;
      H != null && H(this._move_event("enter", _));
    }
    on_move(_) {
      const { on_move: H } = this.handlers;
      H != null && H(this._move_event("move", _));
    }
    on_leave(_) {
      const { on_leave: H } = this.handlers;
      H != null && H(this._move_event("leave", _));
    }
    on_pan_start(_, H, V) {
      const { on_pan_start: q } = this.handlers;
      q != null && q(this._pan_event("pan_start", _, H, V));
    }
    on_pan(_, H, V) {
      const { on_pan: q } = this.handlers;
      q != null && q(this._pan_event("pan", _, H, V));
    }
    on_pan_end(_, H, V) {
      const { on_pan_end: q } = this.handlers;
      q != null && q(this._pan_event("pan_end", _, H, V));
    }
    on_pinch_start(_, H, V) {
      const { on_pinch_start: q } = this.handlers;
      q != null && q(this._pinch_event("pinch_start", _, H, V));
    }
    on_pinch(_, H, V) {
      const { on_pinch: q } = this.handlers;
      q != null && q(this._pinch_event("pinch", _, H, V));
    }
    on_pinch_end(_, H, V) {
      const { on_pinch_end: q } = this.handlers;
      q != null && q(this._pinch_event("pinch_end", _, H, V));
    }
    on_rotate_start(_, H, V) {
      const { on_rotate_start: q } = this.handlers;
      q != null && q(this._rotate_event("rotate_start", _, H, V));
    }
    on_rotate(_, H, V) {
      const { on_rotate: q } = this.handlers;
      q != null && q(this._rotate_event("rotate", _, H, V));
    }
    on_rotate_end(_, H, V) {
      const { on_rotate_end: q } = this.handlers;
      q != null && q(this._rotate_event("rotate_end", _, H, V));
    }
    _get_sxy(_) {
      const { pageX: H, pageY: V } = _, { left: q, top: G } = offset_bbox(this.hit_area);
      return {
        sx: H - q,
        sy: V - G
      };
    }
    _get_modifiers(_) {
      return {
        shift: _.shiftKey,
        ctrl: _.ctrlKey,
        alt: _.altKey
      };
    }
    _tap_event(_, H) {
      return {
        type: _,
        ...this._get_sxy(H),
        modifiers: this._get_modifiers(H),
        native: H
      };
    }
    _move_event(_, H) {
      return {
        type: _,
        ...this._get_sxy(H),
        modifiers: this._get_modifiers(H),
        native: H
      };
    }
    _pan_event(_, H, V, q) {
      return {
        type: _,
        ...this._get_sxy(H),
        dx: V,
        dy: q,
        modifiers: this._get_modifiers(H),
        native: H
      };
    }
    _pinch_event(_, H, V, q) {
      const { sx: G, sy: J } = this._get_sxy(H), { sx: Z, sy: te } = this._get_sxy(V);
      return {
        type: _,
        sx: (G + Z) / 2,
        sy: (J + te) / 2,
        scale: q,
        modifiers: this._get_modifiers(H),
        native: H
      };
    }
    _rotate_event(_, H, V, q) {
      const { sx: G, sy: J } = this._get_sxy(H), { sx: Z, sy: te } = this._get_sxy(V);
      return {
        type: _,
        sx: (G + Z) / 2,
        sy: (J + te) / 2,
        rotation: q,
        modifiers: this._get_modifiers(H),
        native: H
      };
    }
    _movement(_) {
      return {
        dx: _.last.x - _.init.x,
        dy: _.last.y - _.init.y
      };
    }
    _distance(_, H) {
      const V = H.x - _.x, q = H.y - _.y;
      return Math.sqrt(V ** 2 + q ** 2);
    }
    _angle(_, H) {
      const V = H.x - _.x, q = H.y - _.y;
      return Math.atan2(q, V) * 180 / Math.PI;
    }
    _scale(_, H) {
      return this._distance(_.last, H.last) / this._distance(_.init, H.init);
    }
    _rotation(_, H) {
      return this._angle(H.last, _.last) + this._angle(H.init, _.init);
    }
  };
  mt(Wf, "__name__", "UIGestures"), mt(Wf, "move_threshold", 5), /*px*/
  mt(Wf, "press_threshold", 300), /*ms*/
  mt(Wf, "doubletap_threshold", 300), /*ms*/
  mt(Wf, "pinch_threshold", 0), /*unit less*/
  mt(Wf, "rotate_threshold", 0);
  let UIGestures = Wf;
  const menu_icon = "bk-menu-icon", horizontal$2 = "bk-horizontal", vertical$2 = "bk-vertical", divider$2 = "bk-divider", active$4 = "bk-active", menus_css = ".bk-menu-icon{width:28px;height:28px;mask-size:60% 60%;mask-position:center center;mask-repeat:no-repeat;-webkit-mask-size:60% 60%;-webkit-mask-position:center center;-webkit-mask-repeat:no-repeat;background-size:60%;background-color:transparent;background-repeat:no-repeat;background-position:center center;}:host{position:absolute;display:inline-flex;flex-wrap:nowrap;user-select:none;-webkit-user-select:none;width:auto;height:auto;z-index:var(--bokeh-top-level);cursor:pointer;font-size:var(--font-size);background-color:#fff;border:1px solid #ccc;border-radius:var(--border-radius);box-shadow:2px 4px 8px rgba(0, 0, 0, 0.175);}:host(.bk-horizontal){flex-direction:row;}:host(.bk-vertical){flex-direction:column;}.bk-divider{cursor:default;overflow:hidden;background-color:#e5e5e5;}:host(.bk-horizontal) > .bk-divider{width:1px;margin:5px 0;}:host(.bk-vertical) > .bk-divider{height:1px;margin:0 5px;}:host > :not(.bk-divider){border:1px solid transparent;--active-tool-highlight:#26aae1;}:host > :not(.bk-divider).bk-active{border-color:var(--active-tool-highlight);}:host > :not(.bk-divider):hover{background-color:#f9f9f9;}:host > :not(.bk-divider):focus,:host > :not(.bk-divider):focus-visible{outline:1px dotted var(--active-tool-highlight);outline-offset:-1px;}:host > :not(.bk-divider)::-moz-focus-inner{border:0;}:host(.bk-horizontal) > :not(.bk-divider):first-child{border-top-left-radius:var(--border-radius);border-bottom-left-radius:var(--border-radius);}:host(.bk-horizontal) > :not(.bk-divider):last-child{border-top-right-radius:var(--border-radius);border-bottom-right-radius:var(--border-radius);}:host(.bk-vertical) > :not(.bk-divider):first-child{border-top-left-radius:var(--border-radius);border-top-right-radius:var(--border-radius);}:host(.bk-vertical) > :not(.bk-divider):last-child{border-bottom-left-radius:var(--border-radius);border-bottom-right-radius:var(--border-radius);}", menus = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    active: active$4,
    default: menus_css,
    divider: divider$2,
    horizontal: horizontal$2,
    menu_icon,
    vertical: vertical$2
  }, Symbol.toStringTag, { value: "Module" }));
  class ContextMenu {
    constructor(_, H) {
      mt(this, "items");
      mt(this, "el", div());
      mt(this, "shadow_el");
      mt(this, "_open", !1);
      mt(this, "target");
      mt(this, "orientation");
      mt(this, "reversed");
      mt(this, "prevent_hide");
      mt(this, "extra_styles");
      mt(this, "entry_handler");
      mt(this, "class_list");
      mt(this, "_item_click", (_, H) => {
        var V, q;
        (V = this.entry_handler) == null || V.call(this, _, H), (q = _.handler) == null || q.call(_), this.hide();
      });
      mt(this, "_on_mousedown", (_) => {
        var H;
        _.composedPath().includes(this.el) || (((H = this.prevent_hide) == null ? void 0 : H.call(this, _)) ?? !1) || this.hide();
      });
      mt(this, "_on_keydown", (_) => {
        _.key == "Escape" && this.hide();
      });
      mt(this, "_on_blur", () => {
        this.hide();
      });
      this.items = _, this.target = H.target, this.orientation = H.orientation ?? "vertical", this.reversed = H.reversed ?? !1, this.prevent_hide = H.prevent_hide, this.extra_styles = H.extra_styles ?? [], this.entry_handler = H.entry_handler, this.shadow_el = this.el.attachShadow({ mode: "open" }), this.class_list = new ClassList(this.el.classList);
    }
    get is_open() {
      return this._open;
    }
    get can_open() {
      return this.items.length != 0;
    }
    remove() {
      this._unlisten(), remove$2(this.el);
    }
    _listen() {
      document.addEventListener("mousedown", this._on_mousedown), document.addEventListener("keydown", this._on_keydown), window.addEventListener("blur", this._on_blur);
    }
    _unlisten() {
      document.removeEventListener("mousedown", this._on_mousedown), document.removeEventListener("keydown", this._on_keydown), window.removeEventListener("blur", this._on_blur);
    }
    _position(_) {
      const H = (() => {
        if ("left_of" in _) {
          const { left: J, top: Z } = _.left_of.getBoundingClientRect();
          return { right: J, top: Z };
        }
        if ("right_of" in _) {
          const { top: J, right: Z } = _.right_of.getBoundingClientRect();
          return { left: Z, top: J };
        }
        if ("below" in _) {
          const { left: J, bottom: Z } = _.below.getBoundingClientRect();
          return { left: J, top: Z };
        }
        if ("above" in _) {
          const { left: J, top: Z } = _.above.getBoundingClientRect();
          return { left: J, bottom: Z };
        }
        return _;
      })(), V = this.el.offsetParent ?? document.body, q = (() => {
        const J = V.getBoundingClientRect(), Z = getComputedStyle(V);
        return {
          left: J.left - parseFloat(Z.marginLeft),
          right: J.right + parseFloat(Z.marginRight),
          top: J.top - parseFloat(Z.marginTop),
          bottom: J.bottom + parseFloat(Z.marginBottom)
        };
      })(), { style: G } = this.el;
      G.left = H.left != null ? `${H.left - q.left}px` : "auto", G.top = H.top != null ? `${H.top - q.top}px` : "auto", G.right = H.right != null ? `${q.right - H.right}px` : "auto", G.bottom = H.bottom != null ? `${q.bottom - H.bottom}px` : "auto";
    }
    stylesheets() {
      return [
        base_css,
        /*...super.stylesheets(), */
        menus_css,
        icons_css,
        ...this.extra_styles
      ];
    }
    empty() {
      empty$1(this.shadow_el), this.class_list.clear();
    }
    render() {
      var H;
      this.empty();
      for (const V of this.stylesheets())
        (isString$2(V) ? new InlineStyleSheet$1(V) : V).install(this.shadow_el);
      this.class_list.add(menus[this.orientation]);
      const _ = this.reversed ? reversed(this.items) : this.items;
      for (const [V, q] of enumerate(_)) {
        let G;
        if (V == null)
          G = div({ class: divider$2 });
        else {
          if (V.if != null && !V.if())
            continue;
          if (V.custom != null)
            G = V.custom;
          else {
            const J = V.icon != null ? div({ class: [menu_icon, V.icon] }) : null, Z = [((H = V.active) == null ? void 0 : H.call(V)) ?? !1 ? active$4 : null, V.class];
            G = div({ class: Z, title: V.tooltip, tabIndex: 0 }, J, V.label, V.content), G.addEventListener("click", () => {
              this._item_click(V, q);
            }), G.addEventListener("keydown", (te) => {
              te.key == "Enter" && this._item_click(V, q);
            });
          }
        }
        this.shadow_el.appendChild(G);
      }
    }
    show(_) {
      if (this.items.length != 0 && !this._open) {
        if (this.render(), this.shadow_el.children.length == 0)
          return;
        (this.target.shadowRoot ?? this.target).appendChild(this.el), this._position(_ ?? { left: 0, top: 0 }), this._listen(), this._open = !0;
      }
    }
    hide() {
      this._open && (this._open = !1, this._unlisten(), remove$2(this.el));
    }
    toggle(_) {
      this._open ? this.hide() : this.show(_);
    }
  }
  mt(ContextMenu, "__name__", "ContextMenu");
  const tool_icon = "bk-tool-icon", disabled$2 = "bk-disabled", tool_chevron = "bk-tool-chevron", above$3 = "bk-above", below$4 = "bk-below", left$3 = "bk-left", right$3 = "bk-right", active$3 = "bk-active", tool_button_css = ":host{--button-width:30px;--button-height:30px;--button-color:lightgray;--button-border:2px;--active-tool-highlight:#26aae1;--active-tool-border:var(--button-border) solid transparent;}:host{position:relative;width:var(--button-width);height:var(--button-height);cursor:pointer;user-select:none;-webkit-user-select:none;touch-action:none;}.bk-tool-icon{position:relative;top:calc(var(--button-border)/2);width:calc(var(--button-width) - var(--button-border));height:calc(var(--button-height) - var(--button-border));mask-size:60% 60%;mask-position:center center;mask-repeat:no-repeat;-webkit-mask-size:60% 60%;-webkit-mask-position:center center;-webkit-mask-repeat:no-repeat;background-size:60% 60%;background-origin:border-box;background-position:center center;background-repeat:no-repeat;}:host(.bk-disabled) .bk-tool-icon{background-color:var(--bokeh-icon-color-disabled);cursor:not-allowed;}.bk-tool-chevron{position:absolute;visibility:hidden;width:8px;height:8px;mask-size:100% 100%;mask-position:center center;mask-repeat:no-repeat;-webkit-mask-size:100% 100%;-webkit-mask-position:center center;-webkit-mask-repeat:no-repeat;}:host(:hover) .bk-tool-chevron{visibility:visible;}:host(.bk-above) .bk-tool-chevron{right:0;bottom:0;background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-down);-webkit-mask-image:var(--bokeh-icon-chevron-down);}:host(.bk-below) .bk-tool-chevron{right:0;top:0;background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-up);-webkit-mask-image:var(--bokeh-icon-chevron-up);}:host(.bk-left) .bk-tool-chevron{right:0;bottom:0;background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-right);-webkit-mask-image:var(--bokeh-icon-chevron-right);}:host(.bk-right) .bk-tool-chevron{left:0;bottom:0;background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-chevron-left);-webkit-mask-image:var(--bokeh-icon-chevron-left);}:host(:hover){background-color:rgba(192, 192, 192, 0.15);}:host(:focus),:host(:focus-visible){outline:1px dotted var(--active-tool-highlight);outline-offset:-1px;}:host::-moz-focus-inner{border:0;}:host(.bk-above){border-bottom:var(--active-tool-border);}:host(.bk-above.bk-active){border-bottom-color:var(--active-tool-highlight);}:host(.bk-below){border-top:var(--active-tool-border);}:host(.bk-below.bk-active){border-top-color:var(--active-tool-highlight);}:host(.bk-right){border-left:var(--active-tool-border);}:host(.bk-right.bk-active){border-left-color:var(--active-tool-highlight);}:host(.bk-left){border-right:var(--active-tool-border);}:host(.bk-left.bk-active){border-right-color:var(--active-tool-highlight);}", tool_button = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    above: above$3,
    active: active$3,
    below: below$4,
    default: tool_button_css,
    disabled: disabled$2,
    left: left$3,
    right: right$3,
    tool_chevron,
    tool_icon
  }, Symbol.toStringTag, { value: "Module" }));
  class ToolButtonView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "_menu");
      mt(this, "_ui_gestures");
    }
    initialize() {
      super.initialize();
      const { location: H } = this.parent.model, V = H == "left" || H == "above", q = this.parent.model.horizontal ? "vertical" : "horizontal", G = this.model.tool.menu ?? [];
      this._menu = new ContextMenu(V ? reversed(G) : G, {
        target: this.parent.el,
        orientation: q,
        prevent_hide: (J) => J.composedPath().includes(this.el)
      }), this._ui_gestures = new UIGestures(this.el, {
        on_tap: (J) => {
          if (this._menu.is_open) {
            this._menu.hide();
            return;
          }
          J.native.composedPath().includes(this.el) && this._clicked();
        },
        on_press: () => {
          this._pressed();
        }
      }), this.el.addEventListener("keydown", (J) => {
        switch (J.key) {
          case "Enter": {
            this._clicked();
            break;
          }
          case " ": {
            this._pressed();
            break;
          }
        }
      });
    }
    connect_signals() {
      super.connect_signals(), this._ui_gestures.connect_signals(), this.connect(this.model.change, () => this.render()), this.connect(this.model.tool.change, () => this.render());
    }
    remove() {
      this._ui_gestures.remove(), this._menu.remove(), super.remove();
    }
    stylesheets() {
      return [...super.stylesheets(), tool_button_css, icons_css];
    }
    render() {
      super.render(), this.class_list.add(tool_button[this.parent.model.location]), this.model.tool.disabled && this.class_list.add(disabled$2);
      const H = div({ class: tool_icon });
      this.shadow_el.appendChild(H);
      const V = this.model.icon ?? this.model.tool.computed_icon;
      if (V != null) {
        if (V.startsWith("data:image")) {
          const G = `url("${encodeURI(V)}")`;
          H.style.backgroundImage = G;
        } else if (V.startsWith("--"))
          H.style.backgroundImage = `var(${V})`;
        else if (V.startsWith(".")) {
          const G = V.substring(1);
          H.classList.add(G);
        } else if (ToolIcon.valid(V)) {
          const G = `bk-tool-icon-${V.replace(/_/g, "-")}`;
          H.classList.add(G);
        }
      }
      if (this.model.tool.menu != null) {
        const G = div({ class: tool_chevron });
        this.shadow_el.appendChild(G);
      }
      const q = this.model.tooltip ?? this.model.tool.tooltip;
      this.el.title = q, this.el.tabIndex = 0;
    }
    _pressed() {
      const H = (() => {
        switch (this.parent.model.location) {
          case "right":
            return { left_of: this.el };
          case "left":
            return { right_of: this.el };
          case "above":
            return { below: this.el };
          case "below":
            return { above: this.el };
        }
      })();
      this._menu.toggle(H);
    }
  }
  mt(ToolButtonView, "__name__", "ToolButtonView");
  const b5 = class b5 extends UIElement {
    constructor(_) {
      super(_);
    }
  };
  mt(b5, "__name__", "ToolButton"), b5.define(({ Str: _, Regex: H, Ref: V, Nullable: q, Or: G }) => ({
    tool: [G(V(Tool), V(ToolProxy))],
    icon: [q(G(ToolIcon, H(/^--/), H(/^\./), H(/^data:image/))), null],
    tooltip: [q(_), null]
  }));
  let ToolButton = b5;
  class OnOffButtonView extends ToolButtonView {
    _toggle_active() {
      this.class_list.toggle(active$3, this.model.tool.active);
    }
    connect_signals() {
      super.connect_signals();
      const { active: _ } = this.model.tool.properties;
      this.on_change(_, () => {
        this._toggle_active();
      });
    }
    render() {
      super.render(), this._toggle_active();
    }
    _clicked() {
      const { active: _ } = this.model.tool;
      this.model.tool.active = !_;
    }
  }
  mt(OnOffButtonView, "__name__", "OnOffButtonView");
  const T5 = class T5 extends ToolButton {
    constructor(_) {
      super(_);
    }
  };
  mt(T5, "__name__", "OnOffButton"), T5.prototype.default_view = OnOffButtonView;
  let OnOffButton = T5;
  class GestureToolView extends ToolView {
    get plot_view() {
      return this.parent;
    }
  }
  mt(GestureToolView, "__name__", "GestureToolView");
  class GestureTool extends Tool {
    constructor(_) {
      super(_);
    }
    tool_button() {
      return new OnOffButton({ tool: this });
    }
  }
  mt(GestureTool, "__name__", "GestureTool");
  class InspectToolView extends ToolView {
    get plot_view() {
      return this.parent;
    }
  }
  mt(InspectToolView, "__name__", "InspectToolView");
  const K3 = class K3 extends Tool {
    constructor(H) {
      super(H);
      mt(this, "event_type", "move");
    }
    tool_button() {
      return new OnOffButton({ tool: this });
    }
  };
  mt(K3, "__name__", "InspectTool"), K3.define(() => ({
    toggleable: [new PropertyAlias("visible")]
  })), K3.override({
    active: !0
  });
  let InspectTool = K3;
  class ClickButtonView extends ToolButtonView {
    _clicked() {
      this.model.tool.do.emit(void 0);
    }
  }
  mt(ClickButtonView, "__name__", "ClickButtonView");
  const x5 = class x5 extends ToolButton {
    constructor(_) {
      super(_);
    }
  };
  mt(x5, "__name__", "ClickButton"), x5.prototype.default_view = ClickButtonView;
  let ClickButton = x5;
  class ActionToolView extends ToolView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.do, (_) => this.doit(_));
    }
  }
  mt(ActionToolView, "__name__", "ActionToolView");
  class ActionTool extends Tool {
    constructor(H) {
      super(H);
      mt(this, "do", new Signal(this, "do"));
    }
    tool_button() {
      return new ClickButton({ tool: this });
    }
  }
  mt(ActionTool, "__name__", "ActionTool");
  class HelpToolView extends ActionToolView {
    doit() {
      window.open(this.model.redirect);
    }
  }
  mt(HelpToolView, "__name__", "HelpToolView");
  const x2 = class x2 extends ActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Help");
      mt(this, "tool_icon", tool_icon_help);
    }
  };
  mt(x2, "__name__", "HelpTool"), x2.prototype.default_view = HelpToolView, x2.define(({ Str: H }) => ({
    redirect: [H, "https://docs.bokeh.org/en/latest/docs/user_guide/interaction/tools.html"]
  })), x2.override({
    description: "Click the question mark to learn more about Bokeh plot tools."
  }), x2.register_alias("help", () => new x2());
  let HelpTool = x2;
  const inner$2 = "bk-inner", hidden = "bk-hidden", logo$1 = "bk-logo", above$2 = "bk-above", below$3 = "bk-below", left$2 = "bk-left", right$2 = "bk-right", divider$1 = "bk-divider", tool_overflow = "bk-tool-overflow", horizontal$1 = "bk-horizontal", vertical$1 = "bk-vertical", toolbars_css = ':host{--button-width:30px;--button-height:30px;--button-color:lightgray;}:host{display:flex;flex-wrap:nowrap;align-items:center;user-select:none;-webkit-user-select:none;}:host(.bk-inner){background-color:white;opacity:0.8;}:host(.bk-hidden){visibility:hidden;opacity:0;transition:visibility 0.3s linear, opacity 0.3s linear;}.bk-logo{flex-shrink:0;}:host(.bk-above),:host(.bk-below){flex-direction:row;justify-content:flex-end;}:host(.bk-above) .bk-logo,:host(.bk-below) .bk-logo{order:1;margin-left:5px;margin-right:0px;}:host(.bk-left),:host(.bk-right){flex-direction:column;justify-content:flex-start;}:host(.bk-left) .bk-logo,:host(.bk-right) .bk-logo{order:0;margin-bottom:5px;margin-top:0px;}.bk-divider{content:" ";display:inline-block;background-color:var(--button-color);}:host(.bk-above) .bk-divider,:host(.bk-below) .bk-divider{height:10px;width:1px;}:host(.bk-left) .bk-divider,:host(.bk-right) .bk-divider{height:1px;width:10px;}.bk-tool-overflow{color:gray;display:flex;align-items:center;}.bk-tool-overflow:hover{background-color:rgba(192, 192, 192, 0.15);}.bk-tool-overflow:focus,.bk-tool-overflow:focus-visible{outline:1px dotted var(--active-tool-highlight);outline-offset:-1px;}.bk-tool-overflow::-moz-focus-inner{border:0;}:host(.bk-above) .bk-tool-overflow,:host(.bk-below) .bk-tool-overflow,:host(.bk-horizontal) .bk-tool-overflow{width:calc(var(--button-width)/2);height:var(--button-height);flex-direction:row;}:host(.bk-left) .bk-tool-overflow,:host(.bk-right) .bk-tool-overflow,:host(.bk-vertical) .bk-tool-overflow{width:var(--button-width);height:calc(var(--button-height)/2);flex-direction:column;}', toolbars = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    above: above$2,
    below: below$3,
    default: toolbars_css,
    divider: divider$1,
    hidden,
    horizontal: horizontal$1,
    inner: inner$2,
    left: left$2,
    logo: logo$1,
    right: right$2,
    tool_overflow,
    vertical: vertical$1
  }, Symbol.toStringTag, { value: "Module" })), logo = "bk-logo", grey = "bk-grey", logo_small = "bk-logo-small", logos_css = ".bk-logo{margin:5px;position:relative;display:block;background-repeat:no-repeat;}.bk-logo.bk-grey{filter:grayscale(100%);}.bk-logo-small{width:20px;height:20px;background-image:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAALEgAACxIB0t1+/AAAABx0RVh0U29mdHdhcmUAQWRvYmUgRmlyZXdvcmtzIENTNui8sowAAAOkSURBVDiNjZRtaJVlGMd/1/08zzln5zjP1LWcU9N0NkN8m2CYjpgQYQXqSs0I84OLIC0hkEKoPtiH3gmKoiJDU7QpLgoLjLIQCpEsNJ1vqUOdO7ppbuec5+V+rj4ctwzd8IIbbi6u+8f1539dt3A78eXC7QizUF7gyV1fD1Yqg4JWz84yffhm0qkFqBogB9rM8tZdtwVsPUhWhGcFJngGeWrPzHm5oaMmkfEg1usvLFyc8jLRqDOMru7AyC8saQr7GG7f5fvDeH7Ej8CM66nIF+8yngt6HWaKh7k49Soy9nXurCi1o3qUbS3zWfrYeQDTB/Qj6kX6Ybhw4B+bOYoLKCC9H3Nu/leUTZ1JdRWkkn2ldcCamzrcf47KKXdAJllSlxAOkRgyHsGC/zRday5Qld9DyoM4/q/rUoy/CXh3jzOu3bHUVZeU+DEn8FInkPBFlu3+nW3Nw0mk6vCDiWg8CeJaxEwuHS3+z5RgY+YBR6V1Z1nxSOfoaPa4LASWxxdNp+VWTk7+4vzaou8v8PN+xo+KY2xsw6une2frhw05CTYOmQvsEhjhWjn0bmXPjpE1+kplmmkP3suftwTubK9Vq22qKmrBhpY4jvd5afdRA3wGjFAgcnTK2s4hY0/GPNIb0nErGMCRxWOOX64Z8RAC4oCXdklmEvcL8o0BfkNK4lUg9HTl+oPlQxdNo3Mg4Nv175e/1LDGzZen30MEjRUtmXSfiTVu1kK8W4txyV6BMKlbgk3lMwYCiusNy9fVfvvwMxv8Ynl6vxoByANLTWplvuj/nF9m2+PDtt1eiHPBr1oIfhCChQMBw6Aw0UulqTKZdfVvfG7VcfIqLG9bcldL/+pdWTLxLUy8Qq38heUIjh4XlzZxzQm19lLFlr8vdQ97rjZVOLf8nclzckbcD4wxXMidpX30sFd37Fv/GtwwhzhxGVAprjbg0gCAEeIgwCZyTV2Z1REEW8O4py0wsjeloKoMr6iCY6dP92H6Vw/oTyICIthibxjm/DfN9lVz8IqtqKYLUXfoKVMVQVVJOElGjrnnUt9T9wbgp8AyYKaGlqingHZU/uG2NTZSVqwHQTWkx9hxjkpWDaCg6Ckj5qebgBVbT3V3NNXMSiWSDdGV3hrtzla7J+duwPOToIg42ChPQOQjspnSlp1V+Gjdged7+8UN5CRAV7a5EdFNwCjEaBR27b3W890TE7g24NAP/mMDXRWrGoFPQI9ls/MWO2dWFAar/xcOIImbbpA3zgAAAABJRU5ErkJggg==);}";
  class ToolbarView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "_tool_button_views", /* @__PURE__ */ new Map());
      mt(this, "_tool_buttons");
      mt(this, "_items", []);
      mt(this, "_overflow_menu");
      mt(this, "_overflow_el");
      mt(this, "_visible", null);
    }
    get tool_buttons() {
      return this._tool_buttons.flat();
    }
    get overflow_el() {
      return this._overflow_el;
    }
    get visible() {
      return this.model.visible ? !this.model.autohide || (this._visible ?? !1) : !1;
    }
    *children() {
      yield* super.children(), yield* this._tool_button_views.values();
    }
    has_finished() {
      if (!super.has_finished())
        return !1;
      for (const H of this._tool_button_views.values())
        if (!H.has_finished())
          return !1;
      return !0;
    }
    initialize() {
      super.initialize();
      const { location: H } = this.model, V = H == "left" || H == "above", q = this.model.horizontal ? "vertical" : "horizontal";
      this._overflow_menu = new ContextMenu([], {
        target: this.el,
        orientation: q,
        reversed: V,
        prevent_hide: (G) => G.composedPath().includes(this._overflow_el)
      });
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._build_tool_button_views();
    }
    connect_signals() {
      super.connect_signals();
      const { buttons: H, tools: V, location: q, autohide: G } = this.model.properties;
      this.on_change([H, V], async () => {
        await this._build_tool_button_views(), this.render();
      }), this.on_change(q, () => {
        this.render();
      }), this.on_change(G, () => {
        this._on_visible_change();
      });
    }
    stylesheets() {
      return [...super.stylesheets(), toolbars_css, logos_css, icons_css];
    }
    remove() {
      remove_views(this._tool_button_views), super.remove();
    }
    async _build_tool_button_views() {
      this._tool_buttons = (() => {
        const { buttons: H } = this.model;
        return H == "auto" ? [
          ...values$3(this.model.gestures).map((G) => G.tools),
          this.model.actions,
          this.model.inspectors,
          this.model.auxiliaries
        ].map((G) => G.filter((J) => J.visible).map((J) => J.tool_button())) : split(H, null);
      })(), await build_views(this._tool_button_views, this._tool_buttons.flat(), { parent: this });
    }
    set_visibility(H) {
      H != this._visible && (this._visible = H, this._on_visible_change());
    }
    _on_visible_change() {
      this.el.classList.toggle(hidden, !this.visible);
    }
    _after_resize() {
      super._after_resize(), this._after_render();
    }
    render() {
      super.render(), this.el.classList.add(toolbars[this.model.location]), this.el.classList.toggle(inner$2, this.model.inner), this._on_visible_change();
      const { horizontal: H } = this.model;
      this._overflow_el = div({ class: tool_overflow, tabIndex: 0 }, H ? "⋮" : "⋯");
      const V = () => {
        const Z = (() => {
          switch (this.model.location) {
            case "right":
              return { left_of: this._overflow_el };
            case "left":
              return { right_of: this._overflow_el };
            case "above":
              return { below: this._overflow_el };
            case "below":
              return { above: this._overflow_el };
          }
        })();
        this._overflow_menu.toggle(Z);
      };
      if (this._overflow_el.addEventListener("click", () => {
        V();
      }), this._overflow_el.addEventListener("keydown", (Z) => {
        Z.key == "Enter" && V();
      }), this._items = [], this.model.logo != null) {
        const Z = this.model.logo === "grey" ? grey : null, te = a$2({ href: "https://bokeh.org/", target: "_blank", class: [logo, logo_small, Z] });
        this._items.push(te), this.shadow_el.appendChild(te);
      }
      for (const [, Z] of this._tool_button_views)
        Z.render(), Z.after_render();
      const G = this._tool_buttons.map((Z) => Z.map((te) => this._tool_button_views.get(te).el)).filter((Z) => Z.length != 0), J = () => div({ class: divider$1 });
      for (const Z of join$2(G, J))
        this._items.push(Z), this.shadow_el.appendChild(Z);
    }
    _after_render() {
      super._after_render(), clear$1(this._overflow_menu.items), this.shadow_el.contains(this._overflow_el) && this.shadow_el.removeChild(this._overflow_el);
      for (const te of this._items)
        this.shadow_el.contains(te) || this.shadow_el.append(te);
      const { horizontal: H } = this.model, V = 15, { bbox: q } = this, G = H ? right$2 : above$2;
      let J = 0, Z = !1;
      for (const te of this._items)
        if (Z)
          this.shadow_el.removeChild(te), this._overflow_menu.items.push({ custom: te, class: G });
        else {
          const { width: ne, height: ee } = te.getBoundingClientRect();
          J += H ? ne : ee, Z = H ? J > q.width - V : J > q.height - V, Z && (this.shadow_el.removeChild(te), this.shadow_el.appendChild(this._overflow_el), this._overflow_menu.items.push({ custom: te, class: G }));
        }
    }
  }
  mt(ToolbarView, "__name__", "ToolbarView");
  const GestureToolLike = Or$1(Ref(GestureTool), Ref(ToolProxy)), GestureEntry = Struct$1({
    tools: List$2(GestureToolLike),
    active: Nullable(GestureToolLike)
  }), GesturesMap = Struct$1({
    pan: GestureEntry,
    scroll: GestureEntry,
    pinch: GestureEntry,
    rotate: GestureEntry,
    move: GestureEntry,
    tap: GestureEntry,
    doubletap: GestureEntry,
    press: GestureEntry,
    pressup: GestureEntry,
    multi: GestureEntry
  }), Drag = Tool, Inspection = Tool, Scroll = Tool, Tap = Tool;
  function create_gesture_map() {
    return {
      pan: { tools: [], active: null },
      scroll: { tools: [], active: null },
      pinch: { tools: [], active: null },
      rotate: { tools: [], active: null },
      move: { tools: [], active: null },
      tap: { tools: [], active: null },
      doubletap: { tools: [], active: null },
      press: { tools: [], active: null },
      pressup: { tools: [], active: null },
      multi: { tools: [], active: null }
    };
  }
  const Sp = class Sp extends UIElement {
    constructor(_) {
      super(_);
    }
    get horizontal() {
      return this.location == "above" || this.location == "below";
    }
    get vertical() {
      return this.location == "left" || this.location == "right";
    }
    connect_signals() {
      super.connect_signals();
      const { tools: _, active_drag: H, active_inspect: V, active_scroll: q, active_tap: G, active_multi: J } = this.properties;
      this.on_change([_, H, V, q, G, J], () => {
        this._init_tools(), this._activate_tools();
      });
    }
    initialize() {
      super.initialize(), this._init_tools(), this._activate_tools();
    }
    _init_tools() {
      const _ = /* @__PURE__ */ new Set();
      function H(te, ne) {
        const ee = (te instanceof ToolProxy ? te.underlying : te) instanceof ne;
        return ee && _.add(te), ee;
      }
      const V = this.tools.filter((te) => H(te, InspectTool));
      this.inspectors = V;
      const q = this.tools.filter((te) => H(te, HelpTool));
      this.help = q;
      const G = this.tools.filter((te) => H(te, ActionTool));
      this.actions = G;
      const J = create_gesture_map();
      for (const te of this.tools)
        H(te, GestureTool) && J[te.event_role].tools.push(te);
      for (const te of typed_keys(J)) {
        const ne = this.gestures[te];
        ne.tools = sort_by(J[te].tools, (ee) => ee.default_order), ne.active != null && every$2(ne.tools, (ee) => {
          var ae;
          return ee.id != ((ae = ne.active) == null ? void 0 : ae.id);
        }) && (ne.active = null);
      }
      const Z = this.tools.filter((te) => !_.has(te));
      this.auxiliaries = Z;
    }
    _activate_tools() {
      if (this.active_inspect != "auto") if (this.active_inspect == null)
        for (const q of this.inspectors)
          q.active = !1;
      else if (isArray$3(this.active_inspect)) {
        const q = intersection$1(this.active_inspect, this.inspectors);
        q.length != this.active_inspect.length && (this.active_inspect = q);
        for (const G of this.inspectors)
          includes(this.active_inspect, G) || (G.active = !1);
      } else {
        let q = !1;
        for (const G of this.inspectors)
          G != this.active_inspect ? G.active = !1 : q = !0;
        q || (this.active_inspect = null);
      }
      const _ = (q) => {
        q.active ? this._active_change(q) : q.active = !0;
      };
      for (const q of values$3(this.gestures))
        for (const G of q.tools)
          this.connect(G.properties.active.change, () => this._active_change(G));
      function H(q) {
        switch (q) {
          case "tap":
            return "active_tap";
          case "pan":
            return "active_drag";
          case "pinch":
          case "scroll":
            return "active_scroll";
          case "multi":
            return "active_multi";
          default:
            return null;
        }
      }
      function V(q) {
        return q == "tap" || q == "pan";
      }
      for (const [q, G] of entries$2(this.gestures)) {
        const J = q, Z = H(J);
        if (Z != null) {
          const te = this[Z];
          if (te == "auto")
            G.tools.length != 0 && V(J) && _(G.tools[0]);
          else if (te != null)
            includes(this.tools, te) ? _(te) : this[Z] = null;
          else {
            this.gestures[J].active = null;
            for (const ne of this.gestures[J].tools)
              ne.active = !1;
          }
        }
      }
    }
    _active_change(_) {
      const { event_types: H } = _;
      for (const V of H)
        if (_.active) {
          const q = this.gestures[V].active;
          q != null && _ != q && (logger$1.debug(`Toolbar: deactivating tool: ${q} for event type '${V}'`), q.active = !1), this.gestures[V].active = _, logger$1.debug(`Toolbar: activating tool: ${_} for event type '${V}'`);
        } else
          this.gestures[V].active = null;
    }
  };
  mt(Sp, "__name__", "Toolbar"), Sp.prototype.default_view = ToolbarView, Sp.define(({ Bool: _, List: H, Or: V, Ref: q, Nullable: G, Auto: J }) => ({
    tools: [H(V(q(Tool), q(ToolProxy))), []],
    logo: [G(Logo), "normal"],
    autohide: [_, !1],
    active_drag: [G(V(q(Drag), J)), "auto"],
    active_inspect: [G(V(q(Inspection), H(q(Inspection)), J)), "auto"],
    active_scroll: [G(V(q(Scroll), J)), "auto"],
    active_tap: [G(V(q(Tap), J)), "auto"],
    active_multi: [G(V(q(GestureTool), J)), "auto"]
  })), Sp.internal(({ List: _, Bool: H, Ref: V, Or: q, Null: G, Auto: J }) => ({
    buttons: [q(_(q(V(ToolButton), G)), J), "auto"],
    location: [Location, "right"],
    inner: [H, !1],
    gestures: [GesturesMap, create_gesture_map],
    actions: [_(q(V(ActionTool), V(ToolProxy))), []],
    inspectors: [_(q(V(InspectTool), V(ToolProxy))), []],
    auxiliaries: [_(q(V(Tool), V(ToolProxy))), []],
    help: [_(q(V(HelpTool), V(ToolProxy))), []]
  }));
  let Toolbar = Sp;
  class ToolbarPanelView extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "toolbar_view");
      mt(this, "el", div());
      mt(this, "_previous_bbox", new BBox$1());
    }
    update_layout() {
      this.layout = new SideLayout(this.panel, () => this.get_size(), !0);
    }
    has_finished() {
      return super.has_finished() && this.toolbar_view.has_finished();
    }
    *children() {
      yield* super.children(), yield this.toolbar_view;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), this.toolbar_view = await build_view(this.model.toolbar, { parent: this.canvas });
    }
    connect_signals() {
      super.connect_signals(), this.plot_view.mouseenter.connect(() => {
        this.toolbar_view.set_visibility(!0);
      }), this.plot_view.mouseleave.connect(() => {
        this.toolbar_view.set_visibility(!1);
      });
    }
    remove() {
      this.toolbar_view.remove(), remove$2(this.el), super.remove();
    }
    _render() {
      display(this.el);
      const { bbox: H } = this.layout;
      if (!this._previous_bbox.equals(H)) {
        position$1(this.el, H), this._previous_bbox = H, empty$1(this.el), this.el.style.position = "absolute";
        const { style: V } = this.toolbar_view.el;
        this.toolbar_view.model.horizontal ? (V.width = "100%", V.height = "unset") : (V.width = "unset", V.height = "100%"), this.toolbar_view.render(), this.plot_view.canvas_view.add_event(this.el), this.el.appendChild(this.toolbar_view.el), this.toolbar_view.after_render();
      }
      this.model.visible || undisplay(this.el);
    }
    _get_size() {
      const { tools: H, logo: V } = this.model.toolbar;
      return {
        width: H.length * 30 + (V != null ? 25 : 0) + 15,
        // TODO: approximate, use a proper layout instead.
        height: 30
      };
    }
  }
  mt(ToolbarPanelView, "__name__", "ToolbarPanelView");
  const J3 = class J3 extends Annotation$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(J3, "__name__", "ToolbarPanel"), J3.prototype.default_view = ToolbarPanelView, J3.define(({ Ref: _ }) => ({
    toolbar: [_(Toolbar)]
  }));
  let ToolbarPanel = J3;
  class WhiskerView extends UpperLowerView {
    constructor() {
      super(...arguments);
      mt(this, "lower_head");
      mt(this, "upper_head");
    }
    *children() {
      yield* super.children();
      const { lower_head: H, upper_head: V } = this;
      H != null && (yield H), V != null && (yield V);
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { lower_head: H, upper_head: V } = this.model;
      H != null && (this.lower_head = await build_view(H, { parent: this })), V != null && (this.upper_head = await build_view(V, { parent: this }));
    }
    set_data(H) {
      var q, G;
      super.set_data(H);
      const V = BitSet.all_set(this._lower.length);
      (q = this.lower_head) == null || q.set_data(H, V), (G = this.upper_head) == null || G.set_data(H, V);
    }
    paint(H) {
      if (this.visuals.line.doit)
        for (let q = 0, G = this._lower_sx.length; q < G; q++)
          H.beginPath(), H.moveTo(this._lower_sx[q], this._lower_sy[q]), H.lineTo(this._upper_sx[q], this._upper_sy[q]), this.visuals.line.apply(H, q);
      const V = this.model.dimension == "height" ? 0 : Math.PI / 2;
      if (this.lower_head != null)
        for (let q = 0, G = this._lower_sx.length; q < G; q++)
          H.save(), H.translate(this._lower_sx[q], this._lower_sy[q]), H.rotate(V + Math.PI), this.lower_head.render(H, q), H.restore();
      if (this.upper_head != null)
        for (let q = 0, G = this._upper_sx.length; q < G; q++)
          H.save(), H.translate(this._upper_sx[q], this._upper_sy[q]), H.rotate(V), this.upper_head.render(H, q), H.restore();
    }
  }
  mt(WhiskerView, "__name__", "WhiskerView");
  const u0 = class u0 extends UpperLower {
    constructor(_) {
      super(_);
    }
  };
  mt(u0, "__name__", "Whisker"), u0.prototype.default_view = WhiskerView, u0.mixins(LineVector$1), u0.define(({ Ref: _, Nullable: H }) => ({
    lower_head: [H(_(ArrowHead)), () => new TeeHead({ size: 10 })],
    upper_head: [H(_(ArrowHead)), () => new TeeHead({ size: 10 })]
  })), u0.override({
    level: "underlay"
  });
  let Whisker = u0;
  class TextAnnotationView extends AnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "el");
    }
    update_layout() {
      const { panel: H } = this;
      H != null ? this.layout = new SideLayout(H, () => this.get_size(), !0) : this.layout = void 0;
    }
    initialize() {
      super.initialize(), this.el = div(), this.plot_view.canvas_view.add_overlay(this.el);
    }
    remove() {
      remove$2(this.el), super.remove();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.render());
    }
    render() {
      this.model.visible || undisplay(this.el), super.render();
    }
    get padding() {
      return padding$2(this.model.padding);
    }
    get border_radius() {
      return border_radius(this.model.border_radius);
    }
    _paint(H, V, q, G, J) {
      const { el: Z } = this;
      undisplay(Z), Z.textContent = V, this.visuals.text.set_value(H), Z.style.position = "absolute", Z.style.left = `${q}px`, Z.style.top = `${G}px`, Z.style.color = H.fillStyle, Z.style.webkitTextStroke = `1px ${H.strokeStyle}`, Z.style.font = H.font, Z.style.lineHeight = "normal", Z.style.whiteSpace = "pre", Z.style.padding = (() => {
        const { left: se, right: ce, top: le, bottom: ue } = this.padding;
        return `${le}px ${ce}px ${ue}px ${se}px`;
      })(), Z.style.borderRadius = (() => {
        const { top_left: se, top_right: ce, bottom_right: le, bottom_left: ue } = this.border_radius;
        return `${se}px ${ce}px ${le}px ${ue}px`;
      })();
      const [te, ne] = (() => {
        switch (this.visuals.text.text_align.get_value()) {
          case "left":
            return ["left", "0%"];
          case "center":
            return ["center", "-50%"];
          case "right":
            return ["right", "-100%"];
        }
      })(), [ee, ae] = (() => {
        switch (this.visuals.text.text_baseline.get_value()) {
          case "top":
            return ["top", "0%"];
          case "middle":
            return ["center", "-50%"];
          case "bottom":
            return ["bottom", "-100%"];
          default:
            return ["center", "-50%"];
        }
      })();
      let oe = `translate(${ne}, ${ae})`;
      J != 0 && (oe += `rotate(${J}rad)`), Z.style.transformOrigin = `${te} ${ee}`, Z.style.transform = oe, this.layout == null, this.visuals.background_fill.doit && (this.visuals.background_fill.set_value(H), Z.style.backgroundColor = H.fillStyle), this.visuals.border_line.doit && (this.visuals.border_line.set_value(H), Z.style.borderStyle = H.getLineDash().length < 2 ? "solid" : "dashed", Z.style.borderWidth = `${H.lineWidth}px`, Z.style.borderColor = H.strokeStyle), display(Z);
    }
  }
  mt(TextAnnotationView, "__name__", "TextAnnotationView");
  const w5 = class w5 extends Annotation$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(w5, "__name__", "TextAnnotation"), w5.define(() => ({
    padding: [Padding$2, 0],
    border_radius: [BorderRadius, 0]
  }));
  let TextAnnotation = w5;
  class HTMLLabelView extends TextAnnotationView {
    update_layout() {
      const { panel: _ } = this;
      _ != null ? this.layout = new SideLayout(_, () => this.get_size(), !1) : this.layout = void 0;
    }
    // XXX: this needs to use CSS computed styles
    _get_size() {
      const { text: _ } = this.model, H = new TextBox({ text: _ }), { angle: V, angle_units: q } = this.model;
      H.angle = compute_angle(V, q), H.visuals = this.visuals.text.values();
      const G = H.size(), { padding: J } = this, Z = G.width + J.left + J.right, te = G.height + J.top + J.bottom;
      return { width: Z, height: te };
    }
    _render() {
      const { angle: _, angle_units: H } = this.model, V = compute_angle(_, H), q = this.layout != null ? this.layout : this.plot_view.frame, G = this.coordinates.x_scale, J = this.coordinates.y_scale;
      let Z = (() => {
        switch (this.model.x_units) {
          case "canvas":
            return this.model.x;
          case "screen":
            return q.bbox.xview.compute(this.model.x);
          case "data":
            return G.compute(this.model.x);
        }
      })(), te = (() => {
        switch (this.model.y_units) {
          case "canvas":
            return this.model.y;
          case "screen":
            return q.bbox.yview.compute(this.model.y);
          case "data":
            return J.compute(this.model.y);
        }
      })();
      Z += this.model.x_offset, te -= this.model.y_offset, this._paint(this.layer.ctx, this.model.text, Z, te, V);
    }
  }
  mt(HTMLLabelView, "__name__", "HTMLLabelView");
  const d0 = class d0 extends TextAnnotation {
    constructor(_) {
      super(_);
    }
  };
  mt(d0, "__name__", "HTMLLabel"), d0.prototype.default_view = HTMLLabelView, d0.mixins([
    Text$4,
    ["border_", Line$4],
    ["background_", Fill$2],
    ["background_", Hatch$1]
  ]), d0.define(({ Float: _, Str: H, Angle: V }) => ({
    x: [_],
    x_units: [CoordinateUnits, "data"],
    y: [_],
    y_units: [CoordinateUnits, "data"],
    text: [H, ""],
    angle: [V, 0],
    angle_units: [AngleUnits, "rad"],
    x_offset: [_, 0],
    y_offset: [_, 0]
  })), d0.override({
    background_fill_color: null,
    background_hatch_color: null,
    border_line_color: null
  });
  let HTMLLabel = d0;
  class HTMLLabelSetView extends DataAnnotationView {
    constructor() {
      super(...arguments);
      mt(this, "_x");
      mt(this, "_y");
      mt(this, "sx");
      mt(this, "sy");
      mt(this, "text");
      mt(this, "angle");
      mt(this, "x_offset");
      mt(this, "y_offset");
      mt(this, "els", []);
    }
    set_data(H) {
      super.set_data(H), this.els.forEach((V) => remove$2(V)), this.els = [];
      for (const V of this.text) {
        const q = div({ style: { display: "none" } });
        this.plot_view.canvas_view.add_overlay(q), this.els.push(q);
      }
    }
    remove() {
      this.els.forEach((H) => remove$2(H)), this.els = [], super.remove();
    }
    _rerender() {
      this.render();
    }
    map_data() {
      const { x_scale: H, y_scale: V } = this.coordinates, q = this.layout != null ? this.layout : this.plot_view.frame;
      this.sx = (() => {
        switch (this.model.x_units) {
          case "canvas":
            return new ScreenArray(this._x);
          case "screen":
            return q.bbox.xview.v_compute(this._x);
          case "data":
            return H.v_compute(this._x);
        }
      })(), this.sy = (() => {
        switch (this.model.y_units) {
          case "canvas":
            return new ScreenArray(this._y);
          case "screen":
            return q.bbox.yview.v_compute(this._y);
          case "data":
            return V.v_compute(this._y);
        }
      })();
    }
    paint() {
      const { ctx: H } = this.layer;
      for (let V = 0, q = this.text.length; V < q; V++) {
        const G = this.x_offset.get(V), J = this.y_offset.get(V), Z = this.sx[V] + G, te = this.sy[V] - J, ne = this.angle.get(V), ee = this.text.get(V);
        !isFinite(Z + te + ne) || ee == null || this._paint(H, V, ee, Z, te, ne);
      }
    }
    _paint(H, V, q, G, J, Z) {
      assert$2(V in this.els);
      const te = this.els[V];
      te.textContent = q, this.visuals.text.set_vectorize(H, V), te.style.position = "absolute", te.style.left = `${G}px`, te.style.top = `${J}px`, te.style.color = H.fillStyle, te.style.webkitTextStroke = `1px ${H.strokeStyle}`, te.style.font = H.font, te.style.lineHeight = "normal", te.style.whiteSpace = "pre";
      const [ne, ee] = (() => {
        switch (this.visuals.text.text_align.get(V)) {
          case "left":
            return ["left", "0%"];
          case "center":
            return ["center", "-50%"];
          case "right":
            return ["right", "-100%"];
        }
      })(), [ae, oe] = (() => {
        switch (this.visuals.text.text_baseline.get(V)) {
          case "top":
            return ["top", "0%"];
          case "middle":
            return ["center", "-50%"];
          case "bottom":
            return ["bottom", "-100%"];
          default:
            return ["center", "-50%"];
        }
      })();
      let se = `translate(${ee}, ${oe})`;
      Z != 0 && (se += `rotate(${Z}rad)`), te.style.transformOrigin = `${ne} ${ae}`, te.style.transform = se, this.layout == null, this.visuals.background_fill.doit && (this.visuals.background_fill.set_vectorize(H, V), te.style.backgroundColor = H.fillStyle), this.visuals.border_line.doit && (this.visuals.border_line.set_vectorize(H, V), te.style.borderStyle = H.getLineDash().length < 2 ? "solid" : "dashed", te.style.borderWidth = `${H.lineWidth}px`, te.style.borderColor = H.strokeStyle), display(te);
    }
  }
  mt(HTMLLabelSetView, "__name__", "HTMLLabelSetView");
  const f0 = class f0 extends DataAnnotation {
    constructor(_) {
      super(_);
    }
  };
  mt(f0, "__name__", "HTMLLabelSet"), f0.prototype.default_view = HTMLLabelSetView, f0.mixins([
    TextVector$1,
    ["border_", LineVector$1],
    ["background_", FillVector$1]
  ]), f0.define(() => ({
    x: [XCoordinateSpec, { field: "x" }],
    y: [YCoordinateSpec, { field: "y" }],
    x_units: [CoordinateUnits, "data"],
    y_units: [CoordinateUnits, "data"],
    text: [NullStringSpec, { field: "text" }],
    angle: [AngleSpec, 0],
    x_offset: [NumberSpec, { value: 0 }],
    y_offset: [NumberSpec, { value: 0 }]
  })), f0.override({
    background_fill_color: null,
    border_line_color: null
  });
  let HTMLLabelSet = f0;
  class HTMLTitleView extends TextAnnotationView {
    _get_location() {
      const _ = this.model.offset, H = this.model.standoff / 2;
      let V, q;
      const { bbox: G } = this.layout;
      switch (this.panel.side) {
        case "above":
        case "below": {
          switch (this.model.vertical_align) {
            case "top":
              q = G.top + H;
              break;
            case "middle":
              q = G.vcenter;
              break;
            case "bottom":
              q = G.bottom - H;
              break;
          }
          switch (this.model.align) {
            case "left":
              V = G.left + _;
              break;
            case "center":
              V = G.hcenter;
              break;
            case "right":
              V = G.right - _;
              break;
          }
          break;
        }
        case "left": {
          switch (this.model.vertical_align) {
            case "top":
              V = G.left + H;
              break;
            case "middle":
              V = G.hcenter;
              break;
            case "bottom":
              V = G.right - H;
              break;
          }
          switch (this.model.align) {
            case "left":
              q = G.bottom - _;
              break;
            case "center":
              q = G.vcenter;
              break;
            case "right":
              q = G.top + _;
              break;
          }
          break;
        }
        case "right": {
          switch (this.model.vertical_align) {
            case "top":
              V = G.right - H;
              break;
            case "middle":
              V = G.hcenter;
              break;
            case "bottom":
              V = G.left + H;
              break;
          }
          switch (this.model.align) {
            case "left":
              q = G.top + _;
              break;
            case "center":
              q = G.vcenter;
              break;
            case "right":
              q = G.bottom - _;
              break;
          }
          break;
        }
      }
      return [V, q];
    }
    _render() {
      const { text: _ } = this.model;
      if (_.length == 0)
        return;
      this.model.text_baseline = this.model.vertical_align, this.model.text_align = this.model.align;
      const [H, V] = this._get_location(), q = this.panel.get_label_angle_heuristic("parallel");
      this._paint(this.layer.ctx, _, H, V, q);
    }
    // XXX: this needs to use CSS computed styles
    _get_size() {
      const { text: _ } = this.model, H = new TextBox({ text: _ });
      H.visuals = this.visuals.text.values();
      const V = H.size(), { padding: q } = this, G = V.width + q.left + q.right, J = V.height + q.top + q.bottom;
      return { width: G, height: J == 0 ? 0 : 2 + J + this.model.standoff };
    }
  }
  mt(HTMLTitleView, "__name__", "HTMLTitleView");
  const o2 = class o2 extends TextAnnotation {
    constructor(_) {
      super(_);
    }
  };
  mt(o2, "__name__", "HTMLTitle"), o2.prototype.default_view = HTMLTitleView, o2.mixins([
    Text$4,
    ["border_", Line$4],
    ["background_", Fill$2],
    ["background_", Hatch$1]
  ]), o2.define(({ Float: _, Str: H }) => ({
    text: [H, ""],
    vertical_align: [VerticalAlign, "bottom"],
    align: [TextAlign, "left"],
    offset: [_, 0],
    standoff: [_, 10]
  })), o2.prototype._props.text_align.options.internal = !0, o2.prototype._props.text_baseline.options.internal = !0, o2.override({
    text_font_size: "13px",
    text_font_style: "bold",
    text_line_height: 1,
    background_fill_color: null,
    background_hatch_color: null,
    border_line_color: null
  });
  let HTMLTitle = o2;
  const _CustomJS = class _CustomJS extends Callback {
    constructor(_) {
      super(_);
      mt(this, "_state", null);
    }
    connect_signals() {
      super.connect_signals();
      const { args: _, code: H, module: V } = this.properties;
      this.on_change([_, H, V], () => this._state = null);
    }
    async _compile_module() {
      const url = URL.createObjectURL(new Blob([this.code], { type: "text/javascript" }));
      try {
        const module = await eval(`import("${url}")`);
        return isFunction$3(module.default) ? module.default : (logger$1.warn("custom ES module didn't export a default function"), () => {
        });
      } finally {
        URL.revokeObjectURL(url);
      }
    }
    async _compile_function() {
      const [_ = [], H = []] = unzip(entries$2(this.args)), V = use_strict(this.code), q = new Function(..._, "cb_obj", "cb_data", "cb_context", V);
      return function(...G) {
        return q.call(this, ...H, ...G);
      };
    }
    _is_es_module(_) {
      return _.split(`
`).some((H) => H.trimStart().startsWith("export default"));
    }
    async _compile() {
      const _ = this.module == "auto" ? this._is_es_module(this.code) : this.module;
      return _ ? { func: await this._compile_module(), module: _ } : { func: await this._compile_function(), module: _ };
    }
    async state() {
      return this._state == null && (this._state = await this._compile()), this._state;
    }
    async execute(_, H = {}) {
      const { func: V, module: q } = await this.state(), G = { index: index$4 };
      return q ? V(to_object(this.args), _, H, G) : V.call(_, _, H, G);
    }
  };
  mt(_CustomJS, "__name__", "CustomJS"), _CustomJS.define(({ Unknown: _, Str: H, Dict: V, Auto: q, Or: G, Bool: J }) => ({
    args: [V(_), {}],
    code: [H],
    module: [G(q, J), "auto"]
  }));
  let CustomJS = _CustomJS;
  const S5 = class S5 extends Callback {
    constructor(_) {
      super(_);
    }
    navigate(_) {
      this.same_tab ? window.location.href = _ : window.open(_);
    }
    execute(_, { source: H }) {
      const V = (G) => {
        const J = replace_placeholders(this.url, H, G, void 0, void 0, encodeURI);
        if (!isString$2(J))
          throw new Error("HTML output is not supported in this context");
        this.navigate(J);
      }, { selected: q } = H;
      for (const G of q.indices)
        V(G);
      for (const G of q.line_indices)
        V(G);
    }
  };
  mt(S5, "__name__", "OpenURL"), S5.define(({ Bool: _, Str: H }) => ({
    url: [H, "http://"],
    same_tab: [_, !1]
  }));
  let OpenURL = S5;
  const E5 = class E5 extends Callback {
    constructor(_) {
      super(_);
    }
    execute() {
      const { obj: _, attr: H, value: V } = this;
      H in _.properties ? _.setv({ [H]: V }) : logger$1.error(`${_.type}.${H} is not a property`);
    }
  };
  mt(E5, "__name__", "SetValue"), E5.define(({ Str: _, Unknown: H, Ref: V }) => ({
    obj: [V(HasProps)],
    attr: [_],
    value: [H]
  }));
  let SetValue = E5;
  const A5 = class A5 extends Callback {
    constructor(_) {
      super(_);
    }
    execute() {
      const { target: _ } = this;
      _.visible = !_.visible;
    }
  };
  mt(A5, "__name__", "ToggleVisibility"), A5.define(({ Ref: _ }) => ({
    target: [_(UIElement)]
  }));
  let ToggleVisibility = A5;
  var sy;
  let TextView$2 = (sy = class extends DOMNodeView {
    render() {
      this.el.textContent = this.model.content;
    }
    // TODO This shouldn't be here.
    after_render() {
      this.finish();
    }
    _createElement() {
      return document.createTextNode("");
    }
  }, mt(sy, "__name__", "TextView"), sy);
  var R2;
  let Text$2 = (R2 = class extends DOMNode {
    constructor(_) {
      super(_);
    }
  }, mt(R2, "__name__", "Text"), R2.prototype.default_view = TextView$2, R2.define(({ Str: _ }) => ({
    content: [_, ""]
  })), R2);
  const interacting = "bk-interacting", inner$1 = "bk-inner", header$1 = "bk-header", content$2 = "bk-content", collapsed = "bk-collapsed", minimized = "bk-minimized", footer = "bk-footer", grip = "bk-grip", title$1 = "bk-title", controls = "bk-controls", ctrl = "bk-ctrl", pin = "bk-pin", pinned = "bk-pinned", collapse = "bk-collapse", minimize = "bk-minimize", maximize = "bk-maximize", maximized = "bk-maximized", close$3 = "bk-close", handle = "bk-handle", resize_top = "bk-resize-top", resize_bottom = "bk-resize-bottom", resize_left = "bk-resize-left", resize_right = "bk-resize-right", resize_top_left = "bk-resize-top-left", resize_top_right = "bk-resize-top-right", resize_bottom_left = "bk-resize-bottom-left", resize_bottom_right = "bk-resize-bottom-right", dialogs_css = ":host{--bokeh-bg-color:white;--bokeh-border-color:#e5e5e5;--bokeh-shadow-color:#e5e5e5;--bokeh-ctrl-size:16px;--bokeh-ctrl-color:gray;}:host{position:fixed;left:200px;top:200px;width:600px;height:600px;width:80vw;height:60vh;}:host(.bk-interacting){opacity:0.9;}.bk-inner{position:relative;display:flex;flex-direction:column;flex-wrap:nowrap;width:100%;height:100%;overflow:hidden;border-radius:4px;background-color:var(--bokeh-bg-color);border:1px solid var(--bokeh-border-color);box-shadow:5px 5px 10px var(--bokeh-shadow-color);}.bk-header{position:relative;display:flex;flex:0;gap:1em;padding:5px;background-color:lightgray;}.bk-content{position:relative;display:flex;flex:1;overflow:auto;}:host(.bk-collapsed) .bk-content,:host(.bk-minimized) .bk-content,:host(.bk-collapsed) .bk-footer,:host(.bk-minimized) .bk-footer{display:none;}.bk-footer{position:relative;display:flex;flex:0;}.bk-grip{width:var(--bokeh-ctrl-size);height:var(--bokeh-ctrl-size);background-color:var(--bokeh-ctrl-color);background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-y-grip);-webkit-mask-image:var(--bokeh-icon-y-grip);mask-size:100% 100%;-webkit-mask-size:100% 100%;mask-position:center center;-webkit-mask-position:center center;mask-repeat:no-repeat;-webkit-mask-repeat:no-repeat;}.bk-title{position:relative;display:flex;flex:1;cursor:move;white-space:nowrap;}.bk-controls{position:relative;display:flex;flex:0;}.bk-ctrl{width:var(--bokeh-ctrl-size);height:var(--bokeh-ctrl-size);cursor:pointer;mask-size:100% 100%;-webkit-mask-size:100% 100%;mask-position:center center;-webkit-mask-position:center center;mask-repeat:no-repeat;-webkit-mask-repeat:no-repeat;background-color:var(--bokeh-ctrl-color);}.bk-ctrl:hover{background-color:red;}.bk-pin{mask-image:var(--bokeh-icon-pin);-webkit-mask-image:var(--bokeh-icon-pin);}:host(.bk-pinned) .bk-pin{mask-image:var(--bokeh-icon-unpin);-webkit-mask-image:var(--bokeh-icon-unpin);}.bk-collapse{mask-image:var(--bokeh-icon-chevron-up);-webkit-mask-image:var(--bokeh-icon-chevron-up);}:host(.bk-collapsed) .bk-collapse{mask-image:var(--bokeh-icon-chevron-down);-webkit-mask-image:var(--bokeh-icon-chevron-down);}.bk-minimize{mask-image:var(--bokeh-icon-arrow-down-to-bar);-webkit-mask-image:var(--bokeh-icon-arrow-down-to-bar);}:host(.bk-minimized) .bk-minimize{mask-image:var(--bokeh-icon-arrow-up-from-bar);-webkit-mask-image:var(--bokeh-icon-arrow-up-from-bar);}.bk-maximize{mask-image:var(--bokeh-icon-maximize);-webkit-mask-image:var(--bokeh-icon-maximize);}:host(.bk-maximized) .bk-maximize{mask-image:var(--bokeh-icon-minimize);-webkit-mask-image:var(--bokeh-icon-minimize);}.bk-close{mask-image:var(--bokeh-icon-x);-webkit-mask-image:var(--bokeh-icon-x);}:host{--resize-radius:3px;--resize-depth:calc(2*var(--resize-radius));}:host(.bk-minimized){position:relative;}:host(.bk-minimized) .bk-handle,:host(.bk-maximized) .bk-handle{display:none;}:host(.bk-minimized) .bk-title,:host(.bk-maximized) .bk-title{cursor:default;}.bk-resize-top{position:absolute;top:0;left:var(--resize-radius);width:calc(100% - var(--resize-depth));height:var(--resize-depth);transform:translate(0, -50%);cursor:ns-resize;}.bk-resize-bottom{position:absolute;bottom:0;left:var(--resize-radius);width:calc(100% - var(--resize-depth));height:var(--resize-depth);transform:translate(0, 50%);cursor:ns-resize;}.bk-resize-left{position:absolute;left:0;top:var(--resize-radius);width:var(--resize-depth);height:calc(100% - var(--resize-depth));transform:translate(-50%, 0);cursor:ew-resize;}.bk-resize-right{position:absolute;right:0;top:var(--resize-radius);width:var(--resize-depth);height:calc(100% - var(--resize-depth));transform:translate(50%, 0);cursor:ew-resize;}.bk-resize-top-left{position:absolute;top:0;left:0;width:var(--resize-depth);height:var(--resize-depth);transform:translate(-50%, -50%);cursor:nw-resize;}.bk-resize-top-right{position:absolute;top:0;right:0;width:var(--resize-depth);height:var(--resize-depth);transform:translate(50%, -50%);cursor:ne-resize;}.bk-resize-bottom-left{position:absolute;bottom:0;left:0;width:var(--resize-depth);height:var(--resize-depth);transform:translate(-50%, 50%);cursor:sw-resize;}.bk-resize-bottom-right{position:absolute;bottom:0;right:0;width:var(--resize-depth);height:var(--resize-depth);transform:translate(50%, 50%);cursor:se-resize;}";
  Or$1(Ref(UIElement), Ref(DOMNode));
  const _stacking_order = [], _minimization_area = (() => {
    const O = div(), _ = O.attachShadow({ mode: "open" });
    return new InlineStyleSheet$1(`
:host {
  display: flex;
  flex-direction: column;
  flex-wrap: nowrap;
  position: fixed;
  left: 0;
  bottom: 0;
  width: max-content;
  height: max-content;
}
:host:empty {
  display: none;
}
`).install(_), dom_ready().then(() => document.body.append(O)), O;
  })();
  class DialogView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "_title");
      mt(this, "_content");
      mt(this, "_position", new InlineStyleSheet$1());
      mt(this, "_stacking", new InlineStyleSheet$1());
      mt(this, "_has_rendered", !1);
      mt(this, "_handles");
      mt(this, "_pin_el");
      mt(this, "_collapse_el");
      mt(this, "_minimize_el");
      mt(this, "_maximize_el");
      mt(this, "_close_el");
      mt(this, "_pinned", !1);
      mt(this, "_normal_bbox", null);
      mt(this, "_collapsed", !1);
      mt(this, "_minimized", !1);
      mt(this, "_maximized", !1);
    }
    *children() {
      yield* super.children(), yield this._title, yield this._content;
    }
    stylesheets() {
      return [...super.stylesheets(), dialogs_css, icons_css, this._position, this._stacking];
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const H = (() => {
        const { title: q } = this.model;
        return isString$2(q) || q == null ? new Text$2({ content: q ?? "" }) : q;
      })(), V = (() => {
        const { content: q } = this.model;
        return isString$2(q) ? new Text$2({ content: q }) : q;
      })();
      this._title = await build_view(H, { parent: this }), this._content = await build_view(V, { parent: this });
    }
    connect_signals() {
      super.connect_signals();
      const { visible: H } = this.model.properties;
      this.connect(H.change, () => this._toggle(this.model.visible));
    }
    remove() {
      remove$3(_stacking_order, this), this._content.remove(), this._title.remove(), super.remove();
    }
    _reposition(H) {
      this._position.replace(":host", {
        left: "left" in H ? px$1(H.left) : "unset",
        right: "right" in H ? px$1(H.right) : "unset",
        top: "top" in H ? px$1(H.top) : "unset",
        bottom: "bottom" in H ? px$1(H.bottom) : "unset",
        width: "width" in H ? px$1(H.width) : "unset",
        height: "height" in H ? px$1(H.height) : "unset"
      }), this.update_bbox();
    }
    render() {
      super.render(), this._title.render_to(null), this._content.render_to(null);
      const H = div({ class: inner$1 });
      this.shadow_el.append(H);
      const V = div({ class: header$1 }), q = div({ class: content$2 }, this._content.el), G = div({ class: footer });
      H.append(V), H.append(q), H.append(G);
      const J = div({ class: grip }), Z = div({ class: title$1 }, J, this._title.el), te = div({ class: controls });
      V.append(Z, te);
      const ne = div({ class: [ctrl, pin], title: "Pin" });
      ne.addEventListener("click", () => this.pin()), this._pin_el = ne;
      const ee = div({ class: [ctrl, collapse], title: "Collapse" });
      ee.addEventListener("click", () => this.collapse()), this._collapse_el = ee;
      const ae = div({ class: [ctrl, minimize], title: "Minimize" });
      ae.addEventListener("click", () => this.minimize()), this._minimize_el = ae;
      const oe = div({ class: [ctrl, maximize], title: "Maximize" });
      oe.addEventListener("click", () => this.maximize()), this._maximize_el = oe;
      const se = div({ class: [ctrl, close$3], title: "Close" });
      se.addEventListener("click", () => this.close()), this._close_el = se, this.model.pinnable && te.append(ne), this.model.collapsible && te.append(ee), this.model.minimizable && te.append(ae), this.model.maximizable && te.append(oe), this.model.closable && te.append(se);
      const ce = this._handles = {
        // area
        area: Z,
        // edges
        top: div({ class: [handle, resize_top] }),
        bottom: div({ class: [handle, resize_bottom] }),
        left: div({ class: [handle, resize_left] }),
        right: div({ class: [handle, resize_right] }),
        // corners
        top_left: div({ class: [handle, resize_top_left] }),
        top_right: div({ class: [handle, resize_top_right] }),
        bottom_left: div({ class: [handle, resize_bottom_left] }),
        bottom_right: div({ class: [handle, resize_bottom_right] })
      };
      this.shadow_el.append(ce.top, ce.bottom, ce.left, ce.right, ce.top_left, ce.top_right, ce.bottom_left, ce.bottom_right);
      let le = null;
      const ue = () => {
        le = null, document.removeEventListener("pointermove", de), document.removeEventListener("pointerup", fe), document.removeEventListener("keydown", he), this.el.classList.remove(interacting);
      }, de = (me) => {
        assert$2(le != null), me.preventDefault(), this.el.classList.add(interacting);
        const pe = me.x - le.xy.x, ye = me.y - le.xy.y, { target: ge, bbox: ve } = le, be = this._move_bbox(ge, ve, pe, ye);
        this._reposition(be);
      }, fe = (me) => {
        assert$2(le != null), me.preventDefault(), ue();
      }, he = (me) => {
        if (me.key == "Escape") {
          assert$2(le != null), me.preventDefault();
          const { left: pe, top: ye, width: ge, height: ve } = le.bbox;
          this._reposition({ left: pe, top: ye, width: ge, height: ve }), ue();
        }
      };
      this.el.addEventListener("pointerdown", (me) => {
        assert$2(le == null), this.bring_to_front();
        const pe = this._hit_target(me.composedPath());
        if (pe == null || !this._can_hit(pe))
          return;
        me.preventDefault();
        const { x: ye, y: ge } = me;
        le = {
          bbox: bounding_box(this.el),
          xy: { x: ye, y: ge },
          target: pe
        }, document.addEventListener("pointermove", de), document.addEventListener("pointerup", fe), document.addEventListener("keydown", he), this._handles[pe].setPointerCapture(me.pointerId);
      }), Z.addEventListener("wheel", (me) => {
        const pe = me.deltaY;
        (pe < 0 && !this._collapsed || pe > 0 && this._collapsed) && (me.preventDefault(), me.stopPropagation(), this.collapse());
      }), this._has_rendered = !0;
    }
    get resizable() {
      const { resizable: H } = this.model;
      return {
        left: H == "left" || H == "x" || H == "all",
        right: H == "right" || H == "x" || H == "all",
        top: H == "top" || H == "y" || H == "all",
        bottom: H == "bottom" || H == "y" || H == "all"
      };
    }
    _hit_target(H) {
      const { _handles: V } = this;
      for (const q of H)
        switch (q) {
          case V.area:
            return "area";
          case V.top:
            return "top";
          case V.bottom:
            return "bottom";
          case V.left:
            return "left";
          case V.right:
            return "right";
          case V.top_left:
            return "top_left";
          case V.top_right:
            return "top_right";
          case V.bottom_left:
            return "bottom_left";
          case V.bottom_right:
            return "bottom_right";
        }
      return null;
    }
    _can_hit(H) {
      if (this._minimized || this._maximized)
        return !1;
      const { left: V, right: q, top: G, bottom: J } = this.resizable;
      switch (H) {
        case "top_left":
          return G && V;
        case "top_right":
          return G && q;
        case "bottom_left":
          return J && V;
        case "bottom_right":
          return J && q;
        case "left":
          return V;
        case "right":
          return q;
        case "top":
          return G;
        case "bottom":
          return J;
        case "area":
          return this.model.movable != "none";
      }
    }
    _move_bbox(H, V, q, G) {
      const J = (we, Ee) => Ee instanceof Coordinate ? this.resolve_as_scalar(Ee, we) : NaN, Z = BBox$1.from_lrtb({
        left: J("x", this.model.left_limit),
        right: J("x", this.model.right_limit),
        top: J("y", this.model.top_limit),
        bottom: J("y", this.model.bottom_limit)
      }), [te, ne, ee, ae] = (() => {
        const { symmetric: we } = this.model, [Ee, xe] = we ? [-q, -G] : [0, 0];
        switch (H) {
          // corners
          case "top_left":
            return [q, Ee, G, xe];
          case "top_right":
            return [Ee, q, G, xe];
          case "bottom_left":
            return [q, Ee, xe, G];
          case "bottom_right":
            return [Ee, q, xe, G];
          // edges
          case "left":
            return [q, Ee, 0, 0];
          case "right":
            return [Ee, q, 0, 0];
          case "top":
            return [0, 0, G, xe];
          case "bottom":
            return [0, 0, xe, G];
          // area
          case "area":
            switch (this.model.movable) {
              case "both":
                return [q, q, G, G];
              case "x":
                return [q, q, 0, 0];
              case "y":
                return [0, 0, G, G];
              case "none":
                return [0, 0, 0, 0];
            }
        }
      })(), oe = (we, Ee) => min$b([we, Ee]), se = (we) => we < 0 ? -1 : we > 0 ? 1 : 0;
      let { left: ce, right: le, left_sign: ue, right_sign: de } = (() => {
        const we = V.left + te, Ee = V.right + ne, xe = se(te), Le = se(ne);
        return we <= Ee ? { left: we, right: Ee, left_sign: xe, right_sign: Le } : { left: Ee, right: we, left_sign: Le, right_sign: xe };
      })(), { top: fe, bottom: he, top_sign: me, bottom_sign: pe } = (() => {
        const we = V.top + ee, Ee = V.bottom + ae, xe = se(ee), Le = se(ae);
        return we <= Ee ? { top: we, bottom: Ee, top_sign: xe, bottom_sign: Le } : { top: Ee, bottom: we, top_sign: Le, bottom_sign: xe };
      })();
      const ye = ce - Z.left, ge = Z.right - le, ve = oe(ye < 0 ? ye : NaN, ge < 0 ? ge : NaN);
      isFinite(ve) && ve < 0 && (ce += -ue * -ve, le += -de * -ve);
      const be = fe - Z.top, Ce = Z.bottom - he, Se = oe(be < 0 ? be : NaN, Ce < 0 ? Ce : NaN);
      return isFinite(Se) && Se < 0 && (fe += -me * -Se, he += -pe * -Se), BBox$1.from_lrtb({ left: ce, right: le, top: fe, bottom: he });
    }
    pin() {
      const { _pinned: H } = this;
      for (const V of _stacking_order)
        V == this ? this._pin(!H) : V._pin(!1);
      H || this.bring_to_front();
    }
    _pin(H) {
      this._pinned != H && (this._pinned = H, this.el.classList.toggle(pinned, this._pinned), this._pin_el.title = this._pinned ? "Unpin" : "Pin");
    }
    collapse() {
      const H = (() => {
        if (this._collapsed) {
          const { _normal_bbox: V } = this;
          return assert$2(V != null), this._normal_bbox = null, V;
        } else {
          this._minimize(!1), this._maximize(!1), this._normal_bbox == null && (this._normal_bbox = bounding_box(this.el));
          const { left: V, top: q, width: G } = this._normal_bbox;
          return { left: V, top: q, width: G, height: "max-content" };
        }
      })();
      this._reposition(H), this._collapse(!this._collapsed);
    }
    _collapse(H) {
      this._collapsed != H && (this._collapsed = H, this.el.classList.toggle(collapsed, this._collapsed), this._collapse_el.title = this._collapsed ? "Restore" : "Collapse");
    }
    minimize() {
      const H = (() => {
        if (this._minimized) {
          const { _normal_bbox: V } = this;
          return assert$2(V != null), this._normal_bbox = null, V;
        } else
          return this._pin(!1), this._collapse(!1), this._maximize(!1), this._normal_bbox == null && (this._normal_bbox = bounding_box(this.el)), { width: "auto", height: "max-content" };
      })();
      this._reposition(H), this._minimize(!this._minimized);
    }
    _minimize(H) {
      this._minimized != H && (this._minimized = H, (H ? _minimization_area.shadowRoot ?? _minimization_area : document.body).append(this.el), this.el.classList.toggle(minimized, this._minimized), this._minimize_el.title = this._minimized ? "Restore" : "Minimize");
    }
    maximize() {
      const H = (() => {
        if (this._maximized) {
          const { _normal_bbox: V } = this;
          return assert$2(V != null), this._normal_bbox = null, V;
        } else
          return this._collapse(!1), this._minimize(!1), this._normal_bbox == null && (this._normal_bbox = bounding_box(this.el)), { left: 0, top: 0, width: "100%", height: "100%" };
      })();
      this._reposition(H), this._maximize(!this._maximized);
    }
    _maximize(H) {
      this._maximized != H && (this._maximized = H, this.el.classList.toggle(maximized, this._maximized), this._maximize_el.title = this._maximized ? "Restore" : "Maximize");
    }
    restore() {
      this._collapse(!1), this._minimize(!1), this._maximize(!1);
      const { _normal_bbox: H } = this;
      H != null && (this._reposition(H), this._normal_bbox = null);
    }
    _toggle(H) {
      H ? (this._has_rendered || this.render(), this.el.isConnected || (_stacking_order.push(this), document.body.append(this.el)), this.bring_to_front()) : (remove$3(_stacking_order, this), this.el.remove());
    }
    open() {
      this.model.setv({ visible: !0 }, { check_eq: !1 });
    }
    close() {
      switch (this.model.close_action) {
        case "hide": {
          this.model.visible = !1;
          break;
        }
        case "destroy": {
          this.remove();
          break;
        }
      }
    }
    bring_to_front() {
      const H = find$4(_stacking_order, (V) => V._pinned);
      H != null && remove$3(_stacking_order, H), remove$3(_stacking_order, this), _stacking_order.push(this), H != null && _stacking_order.push(H);
      for (const [V, q] of enumerate(_stacking_order))
        V._stacking.replace(":host", {
          "z-index": `${q}`
        });
    }
  }
  mt(DialogView, "__name__", "DialogView");
  const em = class em extends UIElement {
    constructor(_) {
      super(_);
    }
  };
  mt(em, "__name__", "Dialog"), em.prototype.default_view = DialogView, em.define(({ Bool: _, Str: H, Ref: V, Or: q, Nullable: G, Enum: J }) => ({
    title: [G(q(H, V(DOMNode), V(UIElement))), null],
    content: [q(H, V(DOMNode), V(UIElement))],
    pinnable: [_, !0],
    collapsible: [_, !0],
    minimizable: [_, !0],
    maximizable: [_, !0],
    closable: [_, !0],
    close_action: [J("hide", "destroy"), "destroy"],
    resizable: [Resizable, "all"],
    movable: [Movable, "both"],
    symmetric: [_, !1],
    top_limit: [Limit, null],
    bottom_limit: [Limit, null],
    left_limit: [Limit, null],
    right_limit: [Limit, null]
  }));
  let Dialog = em;
  const Q5 = class Q5 extends Callback {
    constructor(_) {
      super(_);
    }
    async execute() {
      var V;
      const { dialog: _ } = this, H = (V = _.document) == null ? void 0 : V.views_manager;
      if (H != null) {
        let q = H.find_one(_);
        q == null && (q = await H.build_view(_)), q.open();
      }
    }
  };
  mt(Q5, "__name__", "OpenDialog"), Q5.define(({ Ref: _ }) => ({
    dialog: [_(Dialog)]
  }));
  let OpenDialog = Q5;
  const C5 = class C5 extends Callback {
    constructor(_) {
      super(_);
    }
    async execute() {
      var H, V, q;
      const { dialog: _ } = this;
      (q = (V = (H = _.document) == null ? void 0 : H.views_manager) == null ? void 0 : V.find_one(_)) == null || q.close();
    }
  };
  mt(C5, "__name__", "CloseDialog"), C5.define(({ Ref: _ }) => ({
    dialog: [_(Dialog)]
  }));
  let CloseDialog = C5;
  /*!
   * jQuery Mousewheel 3.1.13
   *
   * Copyright jQuery Foundation and other contributors
   * Released under the MIT license
   * http://jquery.org/license
   */
  function fontSize$1(O) {
    const _ = getComputedStyle(O).fontSize, H = parseInt(_, 10);
    return isNaN(H) ? null : H;
  }
  function lineHeight$1(O) {
    const _ = O.offsetParent ?? document.body;
    return fontSize$1(_) ?? fontSize$1(O) ?? 16;
  }
  function pageHeight(O) {
    return O.clientHeight;
  }
  function getDeltaY(O) {
    let _ = -O.deltaY;
    if (O.target instanceof HTMLElement)
      switch (O.deltaMode) {
        case O.DOM_DELTA_LINE:
          _ *= lineHeight$1(O.target);
          break;
        case O.DOM_DELTA_PAGE:
          _ *= pageHeight(O.target);
          break;
      }
    return _;
  }
  function is_Moveable(O) {
    return isObject$3(O) && "on_enter" in O && "on_move" in O && "on_leave" in O;
  }
  function is_Pannable(O) {
    return isObject$3(O) && "on_pan_start" in O && "on_pan" in O && "on_pan_end" in O;
  }
  function is_Pinchable(O) {
    return isObject$3(O) && "on_pinch_start" in O && "on_pinch" in O && "on_pinch_end" in O;
  }
  function is_Rotatable(O) {
    return isObject$3(O) && "on_rotate_start" in O && "on_rotate" in O && "on_rotate_end" in O;
  }
  class UIEventBus {
    constructor(_) {
      mt(this, "canvas_view");
      mt(this, "pan_start", new Signal(this, "pan:start"));
      mt(this, "pan", new Signal(this, "pan"));
      mt(this, "pan_end", new Signal(this, "pan:end"));
      mt(this, "pinch_start", new Signal(this, "pinch:start"));
      mt(this, "pinch", new Signal(this, "pinch"));
      mt(this, "pinch_end", new Signal(this, "pinch:end"));
      mt(this, "rotate_start", new Signal(this, "rotate:start"));
      mt(this, "rotate", new Signal(this, "rotate"));
      mt(this, "rotate_end", new Signal(this, "rotate:end"));
      mt(this, "tap", new Signal(this, "tap"));
      mt(this, "doubletap", new Signal(this, "doubletap"));
      mt(this, "press", new Signal(this, "press"));
      mt(this, "pressup", new Signal(this, "pressup"));
      mt(this, "move_enter", new Signal(this, "move:enter"));
      mt(this, "move", new Signal(this, "move"));
      mt(this, "move_exit", new Signal(this, "move:exit"));
      mt(this, "scroll", new Signal(this, "scroll"));
      mt(this, "keydown", new Signal(this, "keydown"));
      mt(this, "keyup", new Signal(this, "keyup"));
      mt(this, "hit_area");
      mt(this, "ui_gestures");
      mt(this, "_prev_move", null);
      mt(this, "_curr_pan", null);
      mt(this, "_curr_pinch", null);
      mt(this, "_curr_rotate", null);
      mt(this, "_current_pan_view", null);
      mt(this, "_current_pinch_view", null);
      mt(this, "_current_rotate_view", null);
      mt(this, "_current_move_view", null);
      this.canvas_view = _, this.hit_area = _.events_el, this.on_tap = this.on_tap.bind(this), this.on_doubletap = this.on_doubletap.bind(this), this.on_press = this.on_press.bind(this), this.on_pressup = this.on_pressup.bind(this), this.on_enter = this.on_enter.bind(this), this.on_move = this.on_move.bind(this), this.on_leave = this.on_leave.bind(this), this.on_pan_start = this.on_pan_start.bind(this), this.on_pan = this.on_pan.bind(this), this.on_pan_end = this.on_pan_end.bind(this), this.on_pinch_start = this.on_pinch_start.bind(this), this.on_pinch = this.on_pinch.bind(this), this.on_pinch_end = this.on_pinch_end.bind(this), this.on_rotate_start = this.on_rotate_start.bind(this), this.on_rotate = this.on_rotate.bind(this), this.on_rotate_end = this.on_rotate_end.bind(this), this.ui_gestures = new UIGestures(this.hit_area, this, { must_be_target: !0 }), this.ui_gestures.connect_signals(), this.on_context_menu = this.on_context_menu.bind(this), this.on_mouse_wheel = this.on_mouse_wheel.bind(this), this.on_key_down = this.on_key_down.bind(this), this.on_key_up = this.on_key_up.bind(this), this.hit_area.addEventListener("contextmenu", this.on_context_menu), this.hit_area.addEventListener("wheel", this.on_mouse_wheel), document.addEventListener("keydown", this.on_key_down), document.addEventListener("keyup", this.on_key_up);
    }
    remove() {
      this.ui_gestures.remove(), this.hit_area.removeEventListener("contextmenu", this.on_context_menu), this.hit_area.removeEventListener("wheel", this.on_mouse_wheel), document.removeEventListener("keydown", this.on_key_down), document.removeEventListener("keyup", this.on_key_up);
    }
    register_tool(_) {
      const { model: H } = _, V = (q) => (G) => {
        (G.tool == null || G.tool == H) && q.call(_, G.e);
      };
      _._pan_start != null && _.connect(this.pan_start, V(_._pan_start)), _._pan != null && _.connect(this.pan, V(_._pan)), _._pan_end != null && _.connect(this.pan_end, V(_._pan_end)), _._pinch_start != null && _.connect(this.pinch_start, V(_._pinch_start)), _._pinch != null && _.connect(this.pinch, V(_._pinch)), _._pinch_end != null && _.connect(this.pinch_end, V(_._pinch_end)), _._rotate_start != null && _.connect(this.rotate_start, V(_._rotate_start)), _._rotate != null && _.connect(this.rotate, V(_._rotate)), _._rotate_end != null && _.connect(this.rotate_end, V(_._rotate_end)), _._move_enter != null && _.connect(this.move_enter, V(_._move_enter)), _._move != null && _.connect(this.move, V(_._move)), _._move_exit != null && _.connect(this.move_exit, V(_._move_exit)), _._tap != null && _.connect(this.tap, V(_._tap)), _._doubletap != null && _.connect(this.doubletap, V(_._doubletap)), _._press != null && _.connect(this.press, V(_._press)), _._pressup != null && _.connect(this.pressup, V(_._pressup)), _._scroll != null && _.connect(this.scroll, V(_._scroll)), _._keydown != null && _.connect(this.keydown, V(_._keydown)), _._keyup != null && _.connect(this.keyup, V(_._keyup));
    }
    hit_test_renderers(_, H, V) {
      var G;
      const q = _.get_renderer_views();
      for (const J of reversed(q))
        if (((G = J.interactive_hit) == null ? void 0 : G.call(J, H, V)) ?? !1)
          return J;
      return null;
    }
    set_cursor(_) {
      this.hit_area.style.cursor = _ ?? "default";
    }
    hit_test_frame(_, H, V) {
      return _.frame.bbox.contains(H, V);
    }
    hit_test_plot(_, H) {
      for (const V of this.canvas_view.plot_views)
        if (V.bbox.relative().contains(_, H))
          return V;
      return null;
    }
    _trigger(_, H) {
      var ne;
      if (!this.hit_area.isConnected)
        return;
      const { sx: V, sy: q, native: G } = H, Z = this.hit_test_plot(V, q), te = (ee) => {
        const [ae, oe] = [V, q];
        return { ...H, sx: ae, sy: oe };
      };
      if (H.type == "pan_start" || H.type == "pan" || H.type == "pan_end") {
        let ee;
        if (H.type == "pan_start" && Z != null ? (this._curr_pan = { plot_view: Z }, ee = Z) : H.type == "pan" && this._curr_pan != null ? ee = this._curr_pan.plot_view : H.type == "pan_end" && this._curr_pan != null ? (ee = this._curr_pan.plot_view, this._curr_pan = null) : ee = null, ee != null) {
          const ae = te();
          this.__trigger(ee, _, ae, G);
        }
      } else if (H.type == "pinch_start" || H.type == "pinch" || H.type == "pinch_end") {
        let ee;
        if (H.type == "pinch_start" && Z != null ? (this._curr_pinch = { plot_view: Z }, ee = Z) : H.type == "pinch" && this._curr_pinch != null ? ee = this._curr_pinch.plot_view : H.type == "pinch_end" && this._curr_pinch != null ? (ee = this._curr_pinch.plot_view, this._curr_pinch = null) : ee = null, ee != null) {
          const ae = te();
          this.__trigger(ee, _, ae, G);
        }
      } else if (H.type == "rotate_start" || H.type == "rotate" || H.type == "rotate_end") {
        let ee;
        if (H.type == "rotate_start" && Z != null ? (this._curr_rotate = { plot_view: Z }, ee = Z) : H.type == "rotate" && this._curr_rotate != null ? ee = this._curr_rotate.plot_view : H.type == "rotate_end" && this._curr_rotate != null ? (ee = this._curr_rotate.plot_view, this._curr_rotate = null) : ee = null, ee != null) {
          const ae = te();
          this.__trigger(ee, _, ae, G);
        }
      } else if (H.type == "enter" || H.type == "move" || H.type == "leave") {
        const ee = (ne = this._prev_move) == null ? void 0 : ne.plot_view;
        if (ee != null && (H.type == "leave" || ee != Z)) {
          const { sx: ae, sy: oe } = te();
          this.__trigger(ee, this.move_exit, { type: "leave", sx: ae, sy: oe, modifiers: { shift: !1, ctrl: !1, alt: !1 }, native: G }, G);
        }
        if (Z != null && (H.type == "enter" || ee != Z)) {
          const { sx: ae, sy: oe } = te();
          this.__trigger(Z, this.move_enter, { type: "enter", sx: ae, sy: oe, modifiers: { shift: !1, ctrl: !1, alt: !1 }, native: G }, G);
        }
        if (Z != null && H.type == "move") {
          const ae = te();
          this.__trigger(Z, _, ae, G);
        }
        this._prev_move = { sx: V, sy: q, plot_view: Z };
      } else if (Z != null) {
        const ee = te();
        this.__trigger(Z, _, ee, G);
      }
    }
    __trigger(_, H, V, q) {
      var ee, ae, oe;
      const G = _.model.toolbar.gestures, J = H.name, Z = J.split(":")[0], te = this.hit_test_renderers(_, V.sx, V.sy);
      if (Z == "pan")
        if (this._current_pan_view == null) {
          if (te != null && J == "pan:start" && is_Pannable(te) && te.on_pan_start(V)) {
            this._current_pan_view = te, q.preventDefault();
            return;
          }
        } else {
          J == "pan" ? this._current_pan_view.on_pan(V) : J == "pan:end" && (this._current_pan_view.on_pan_end(V), this._current_pan_view = null), q.preventDefault();
          return;
        }
      else if (Z == "pinch")
        if (this._current_pinch_view == null) {
          if (te != null && J == "pinch:start" && is_Pinchable(te) && te.on_pinch_start(V)) {
            this._current_pinch_view = te, q.preventDefault();
            return;
          }
        } else {
          J == "pinch" ? this._current_pinch_view.on_pinch(V) : J == "pinch:end" && (this._current_pinch_view.on_pinch_end(V), this._current_pinch_view = null), q.preventDefault();
          return;
        }
      else if (Z == "rotate")
        if (this._current_rotate_view == null) {
          if (te != null && J == "rotate:start" && is_Rotatable(te) && te.on_rotate_start(V)) {
            this._current_rotate_view = te, q.preventDefault();
            return;
          }
        } else {
          J == "rotate" ? this._current_rotate_view.on_rotate(V) : J == "rotate:end" && (this._current_rotate_view.on_rotate_end(V), this._current_rotate_view = null), q.preventDefault();
          return;
        }
      else Z == "move" && (this._current_move_view == te ? (ee = this._current_move_view) == null || ee.on_move(V) : ((ae = this._current_move_view) == null || ae.on_leave(V), this._current_move_view = null, te != null && is_Moveable(te) && te.on_enter(V) && (this._current_move_view = te)));
      function ne(se) {
        if (se != null) {
          const ce = se instanceof ToolProxy ? se.tools[0] : se;
          return _.tool_views.get(ce) ?? null;
        } else
          return null;
      }
      switch (Z) {
        case "move": {
          const se = G.move.active;
          se != null && this.trigger(H, V, se);
          const ce = _.model.toolbar.inspectors.filter((ue) => ue.active), le = (() => {
            const ue = this._current_pan_view ?? this._current_pinch_view ?? this._current_rotate_view ?? this._current_move_view ?? te ?? ne(se);
            if (ue != null) {
              const de = ue.cursor(V.sx, V.sy);
              if (de != null)
                return de;
            }
            return this.hit_test_frame(_, V.sx, V.sy) && !is_empty$1(ce) ? "crosshair" : null;
          })();
          this.set_cursor(le), te != null && !te.model.propagate_hover && !is_empty$1(ce) && (H = this.move_exit), ce.map((ue) => this.trigger(H, V, ue));
          break;
        }
        case "tap": {
          const se = q.composedPath();
          if (se.length != 0 && se[0] != this.hit_area)
            return;
          if ((oe = te == null ? void 0 : te.on_hit) == null || oe.call(te, V.sx, V.sy), this.hit_test_frame(_, V.sx, V.sy)) {
            const ce = G.tap.active;
            ce != null && this.trigger(H, V, ce);
          }
          break;
        }
        case "doubletap": {
          if (this.hit_test_frame(_, V.sx, V.sy)) {
            const se = G.doubletap.active ?? G.tap.active;
            se != null && this.trigger(H, V, se);
          }
          break;
        }
        case "press": {
          if (this.hit_test_frame(_, V.sx, V.sy)) {
            const se = G.press.active ?? G.tap.active;
            se != null && this.trigger(H, V, se);
          }
          break;
        }
        case "pinch": {
          const se = G.pinch.active ?? G.scroll.active;
          se != null && (q.preventDefault(), q.stopPropagation(), this.trigger(H, V, se));
          break;
        }
        case "scroll": {
          const se = G.scroll.active;
          se != null && (q.preventDefault(), q.stopPropagation(), this.trigger(H, V, se));
          break;
        }
        case "pan": {
          const se = G.pan.active;
          se != null && (q.preventDefault(), q.stopPropagation(), this.trigger(H, V, se));
          break;
        }
        default: {
          const se = G[Z].active;
          se != null && this.trigger(H, V, se);
        }
      }
      this._trigger_bokeh_event(_, V);
    }
    trigger(_, H, V = null) {
      _.emit({ tool: V, e: H });
    }
    /*protected*/
    _trigger_bokeh_event(_, H) {
      const V = (() => {
        const { sx: q, sy: G, modifiers: J } = H, Z = _.frame.x_scale.invert(q), te = _.frame.y_scale.invert(G);
        switch (H.type) {
          case "wheel":
            return new MouseWheel(q, G, Z, te, H.delta, J);
          case "enter":
            return new MouseEnter(q, G, Z, te, J);
          case "move":
            return new MouseMove(q, G, Z, te, J);
          case "leave":
            return new MouseLeave(q, G, Z, te, J);
          case "tap":
            return new Tap$1(q, G, Z, te, J);
          case "double_tap":
            return new DoubleTap(q, G, Z, te, J);
          case "press":
            return new Press(q, G, Z, te, J);
          case "press_up":
            return new PressUp(q, G, Z, te, J);
          case "pan_start":
            return new PanStart(q, G, Z, te, J);
          case "pan":
            return new Pan(q, G, Z, te, H.dx, H.dy, J);
          case "pan_end":
            return new PanEnd(q, G, Z, te, J);
          case "pinch_start":
            return new PinchStart(q, G, Z, te, J);
          case "pinch":
            return new Pinch(q, G, Z, te, H.scale, J);
          case "pinch_end":
            return new PinchEnd(q, G, Z, te, J);
          case "rotate_start":
            return new RotateStart(q, G, Z, te, J);
          case "rotate":
            return new Rotate(q, G, Z, te, H.rotation, J);
          case "rotate_end":
            return new RotateEnd(q, G, Z, te, J);
          default:
            return null;
        }
      })();
      V != null && _.model.trigger_event(V);
    }
    _get_sxy(_) {
      const { pageX: H, pageY: V } = _, { left: q, top: G } = offset_bbox(this.hit_area);
      return {
        sx: H - q,
        sy: V - G
      };
    }
    _get_modifiers(_) {
      return {
        shift: _.shiftKey,
        ctrl: _.ctrlKey,
        alt: _.altKey
      };
    }
    _scroll_event(_) {
      return {
        type: _.type,
        ...this._get_sxy(_),
        delta: getDeltaY(_),
        modifiers: this._get_modifiers(_),
        native: _
      };
    }
    _key_event(_) {
      return {
        type: _.type,
        key: _.key,
        modifiers: this._get_modifiers(_),
        native: _
      };
    }
    on_tap(_) {
      this._trigger(this.tap, _);
    }
    on_doubletap(_) {
      this._trigger(this.doubletap, _);
    }
    on_press(_) {
      this._trigger(this.press, _);
    }
    on_pressup(_) {
      this._trigger(this.pressup, _);
    }
    on_enter(_) {
      this._trigger(this.move_enter, _);
    }
    on_move(_) {
      this._trigger(this.move, _);
    }
    on_leave(_) {
      this._trigger(this.move_exit, _);
    }
    on_pan_start(_) {
      this._trigger(this.pan_start, _);
    }
    on_pan(_) {
      this._trigger(this.pan, _);
    }
    on_pan_end(_) {
      this._trigger(this.pan_end, _);
    }
    on_pinch_start(_) {
      this._trigger(this.pinch_start, _);
    }
    on_pinch(_) {
      this._trigger(this.pinch, _);
    }
    on_pinch_end(_) {
      this._trigger(this.pinch_end, _);
    }
    on_rotate_start(_) {
      this._trigger(this.rotate_start, _);
    }
    on_rotate(_) {
      this._trigger(this.rotate, _);
    }
    on_rotate_end(_) {
      this._trigger(this.rotate_end, _);
    }
    on_mouse_wheel(_) {
      this._trigger(this.scroll, this._scroll_event(_));
    }
    on_context_menu(_) {
    }
    on_key_down(_) {
      this.trigger(this.keydown, this._key_event(_));
    }
    on_key_up(_) {
      this.trigger(this.keyup, this._key_event(_));
    }
  }
  mt(UIEventBus, "__name__", "UIEventBus");
  const layer$1 = "bk-layer", events$3 = "bk-events", canvas_css = ".bk-layer{position:absolute;top:0;left:0;width:100%;height:100%;}.bk-events{touch-action:none;}";
  async function init_webgl() {
    const O = document.createElement("canvas"), _ = O.getContext("webgl", { alpha: !0, antialias: !1, depth: !1, premultipliedAlpha: !0 });
    if (_ != null) {
      const H = await load_module(Promise.resolve().then(() => index$3));
      if (H != null) {
        const V = H.get_regl(_);
        if (V.has_webgl)
          return { canvas: O, regl_wrapper: V };
        logger$1.trace("WebGL is supported, but not the required extensions");
      } else
        logger$1.trace("WebGL is supported, but bokehjs(.min).js bundle is not available");
    } else
      logger$1.trace("WebGL is not supported");
    return null;
  }
  const global_webgl = /* @__PURE__ */ (() => {
    let O;
    return async () => O !== void 0 ? O : O = await init_webgl();
  })();
  class CanvasView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "webgl", null);
      mt(this, "underlays_el");
      mt(this, "primary");
      mt(this, "overlays");
      mt(this, "overlays_el");
      mt(this, "events_el");
      mt(this, "ui_event_bus");
      mt(this, "_size", new InlineStyleSheet$1());
      mt(this, "plot_views", []);
    }
    initialize() {
      super.initialize(), this.underlays_el = div({ class: layer$1 }), this.primary = this.create_layer(), this.overlays = this.create_layer(), this.overlays_el = div({ class: layer$1 }), this.events_el = div({ class: [layer$1, events$3] }), this.ui_event_bus = new UIEventBus(this);
    }
    get layers() {
      return [
        this.underlays_el,
        this.primary,
        this.overlays,
        this.overlays_el,
        this.events_el
      ];
    }
    async lazy_initialize() {
      if (await super.lazy_initialize(), this.model.output_backend == "webgl" && (this.webgl = await global_webgl(), settings.force_webgl && this.webgl == null))
        throw new Error("webgl is not available");
    }
    remove() {
      this.ui_event_bus.remove(), super.remove();
    }
    stylesheets() {
      return [...super.stylesheets(), canvas_css, icons_css, this._size];
    }
    render() {
      super.render();
      const H = [
        this.underlays_el,
        this.primary.el,
        this.overlays.el,
        this.overlays_el,
        this.events_el
      ];
      append$2(this.shadow_el, ...H);
    }
    add_underlay(H) {
      this.underlays_el.appendChild(H);
    }
    add_overlay(H) {
      this.overlays_el.appendChild(H);
    }
    add_event(H) {
      this.events_el.appendChild(H);
    }
    get pixel_ratio() {
      return this.primary.pixel_ratio;
    }
    _update_bbox() {
      const H = super._update_bbox();
      if (H) {
        const { width: V, height: q } = this.bbox;
        this._size.replace(`.${layer$1}`, {
          width: px$1(V),
          height: px$1(q)
        }), this.primary.resize(V, q), this.overlays.resize(V, q);
      }
      return H;
    }
    after_resize() {
      this.plot_views.length != 0 ? this.finish() : super.after_resize();
    }
    _after_resize() {
      super._after_resize();
      const { width: H, height: V } = this.bbox;
      this.primary.resize(H, V), this.overlays.resize(H, V);
    }
    resize() {
      this._update_bbox(), this._after_resize();
    }
    prepare_webgl(H) {
      const { webgl: V } = this;
      if (V != null) {
        const { width: q, height: G } = this.bbox;
        V.canvas.width = this.pixel_ratio * q, V.canvas.height = this.pixel_ratio * G;
        const [J, Z, te, ne] = H, { xview: ee, yview: ae } = this.bbox, oe = ee.compute(J), se = ae.compute(Z + ne), ce = this.pixel_ratio;
        V.regl_wrapper.set_scissor(ce * oe, ce * se, ce * te, ce * ne), this._clear_webgl();
      }
    }
    blit_webgl(H) {
      const { webgl: V } = this;
      if (V != null && V.canvas.width * V.canvas.height > 0) {
        if (logger$1.debug("Blitting WebGL canvas"), H.restore(), H.drawImage(V.canvas, 0, 0), H.save(), this.model.hidpi) {
          const q = this.pixel_ratio;
          H.scale(q, q), H.translate(0.5, 0.5);
        }
        this._clear_webgl();
      }
    }
    _clear_webgl() {
      const { webgl: H } = this;
      if (H != null) {
        const { regl_wrapper: V, canvas: q } = H;
        V.clear(q.width, q.height);
      }
    }
    compose() {
      const H = this.create_layer(), { width: V, height: q } = this.bbox;
      return H.resize(V, q), H.ctx.drawImage(this.primary.canvas, 0, 0), H.ctx.drawImage(this.overlays.canvas, 0, 0), H;
    }
    create_layer() {
      const { output_backend: H, hidpi: V } = this.model;
      return new CanvasLayer(H, V);
    }
    to_blob() {
      return this.compose().to_blob();
    }
  }
  mt(CanvasView, "__name__", "CanvasView");
  var O2;
  let Canvas$1 = (O2 = class extends UIElement {
    constructor(_) {
      super(_);
    }
  }, mt(O2, "__name__", "Canvas"), O2.prototype.default_view = CanvasView, O2.define(({ Bool: _ }) => ({
    hidpi: [_, !0],
    output_backend: [OutputBackend, "canvas"]
  })), O2);
  var cy;
  let Expression$2 = (cy = class extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "_result");
    }
    initialize() {
      super.initialize(), this._result = /* @__PURE__ */ new Map();
    }
    v_compute(H) {
      let V = this._result.get(H);
      return (V === void 0 || H.changed_for(this)) && (V = this._v_compute(H), this._result.set(H, V)), V;
    }
  }, mt(cy, "__name__", "Expression"), cy);
  class ScalarExpression extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "_result");
    }
    initialize() {
      super.initialize(), this._result = /* @__PURE__ */ new Map();
    }
    compute(H) {
      let V = this._result.get(H);
      return (V === void 0 || H.changed_for(this)) && (V = this._compute(H), this._result.set(H, V)), V;
    }
  }
  mt(ScalarExpression, "__name__", "ScalarExpression");
  const L5 = class L5 extends Expression$2 {
    constructor(_) {
      super(_);
    }
    connect_signals() {
      super.connect_signals();
      for (const _ of values$3(this.args))
        _ instanceof HasProps && _.change.connect(() => {
          this._result.clear(), this.change.emit();
        });
    }
    get names() {
      return keys$5(this.args);
    }
    get values() {
      return values$3(this.args);
    }
    get func() {
      const _ = use_strict(this.code);
      return new GeneratorFunction(...this.names, _);
    }
    _v_compute(_) {
      const H = this.func.apply(_, this.values);
      let V = H.next();
      if ((V.done ?? !1) && V.value !== void 0) {
        const { value: q } = V;
        return isArray$3(q) || isTypedArray$2(q) ? q : isIterable$2(q) ? [...q] : repeat$1(q, _.length);
      } else {
        const q = [];
        do
          q.push(V.value), V = H.next();
        while (!(V.done ?? !1));
        return q;
      }
    }
  };
  mt(L5, "__name__", "CustomJSExpr"), L5.define(({ Unknown: _, Str: H, Dict: V }) => ({
    args: [V(_), {}],
    code: [H, ""]
  }));
  let CustomJSExpr = L5;
  var O0;
  let Stack$4 = (O0 = class extends Expression$2 {
    constructor(_) {
      super(_);
    }
    _v_compute(_) {
      const H = _.get_length() ?? 0, V = new Float64Array(H);
      for (const q of this.fields) {
        const G = dict(_.data).get(q);
        if (G != null) {
          const J = Math.min(H, G.length);
          for (let Z = 0; Z < J; Z++)
            V[Z] += G[Z];
        }
      }
      return V;
    }
  }, mt(O0, "__name__", "Stack"), O0.define(({ Str: _, List: H }) => ({
    fields: [H(_), []]
  })), O0);
  const k5 = class k5 extends Expression$2 {
    constructor(_) {
      super(_);
    }
    _v_compute(_) {
      const H = new Float64Array(_.get_length() ?? 0), V = dict(_.data).get(this.field) ?? [], q = this.include_zero ? 1 : 0;
      H[0] = this.include_zero ? 0 : V[0];
      for (let G = 1; G < H.length; G++)
        H[G] = H[G - 1] + V[G - q];
      return H;
    }
  };
  mt(k5, "__name__", "CumSum"), k5.define(({ Bool: _, Str: H }) => ({
    field: [H],
    include_zero: [_, !1]
  }));
  let CumSum = k5;
  const M5 = class M5 extends ScalarExpression {
    constructor(_) {
      super(_);
    }
    _compute(_) {
      const H = dict(_.data).get(this.field) ?? [];
      return Math.min(this.initial, min$b(H));
    }
  };
  mt(M5, "__name__", "Minimum"), M5.define(({ Float: _, Str: H }) => ({
    field: [H],
    initial: [_, 1 / 0]
  }));
  let Minimum = M5;
  const R5 = class R5 extends ScalarExpression {
    constructor(_) {
      super(_);
    }
    _compute(_) {
      const H = dict(_.data).get(this.field) ?? [];
      return Math.max(this.initial, max$j(H));
    }
  };
  mt(R5, "__name__", "Maximum"), R5.define(({ Float: _, Str: H }) => ({
    field: [H],
    initial: [_, -1 / 0]
  }));
  let Maximum = R5;
  class CoordinateTransform extends Expression$2 {
    constructor(_) {
      super(_);
    }
    get x() {
      return new XComponent({ transform: this });
    }
    get y() {
      return new YComponent({ transform: this });
    }
  }
  mt(CoordinateTransform, "__name__", "CoordinateTransform");
  const O5 = class O5 extends Expression$2 {
    constructor(_) {
      super(_);
    }
  };
  mt(O5, "__name__", "XYComponent"), O5.define(({ Ref: _ }) => ({
    transform: [_(CoordinateTransform)]
  }));
  let XYComponent = O5;
  class XComponent extends XYComponent {
    constructor(_) {
      super(_);
    }
    _v_compute(_) {
      return this.transform.v_compute(_).x;
    }
  }
  mt(XComponent, "__name__", "XComponent");
  class YComponent extends XYComponent {
    constructor(_) {
      super(_);
    }
    _v_compute(_) {
      return this.transform.v_compute(_).y;
    }
  }
  mt(YComponent, "__name__", "YComponent");
  const I5 = class I5 extends CoordinateTransform {
    constructor(_) {
      super(_);
    }
    _v_compute(_) {
      const H = this.properties.radius.uniform(_), V = this.properties.angle.uniform(_), q = this.direction == "anticlock" ? -1 : 1, G = Math.min(H.length, V.length), J = new Float64Array(G), Z = new Float64Array(G);
      for (let te = 0; te < G; te++) {
        const ne = H.get(te), ee = V.get(te) * q;
        J[te] = ne * Math.cos(ee), Z[te] = ne * Math.sin(ee);
      }
      return { x: J, y: Z };
    }
  };
  mt(I5, "__name__", "PolarTransform"), I5.define(({}) => ({
    radius: [DistanceSpec, { field: "radius" }],
    angle: [AngleSpec, { field: "angle" }],
    direction: [Direction$1, "anticlock"]
  }));
  let PolarTransform = I5;
  const D5 = class D5 extends Filter$1 {
    constructor(_) {
      super(_);
    }
    compute_indices(_) {
      const H = _.get_length() ?? 1, { booleans: V } = this;
      return V == null ? BitSet.all_set(H) : BitSet.from_booleans(H, V);
    }
  };
  mt(D5, "__name__", "BooleanFilter"), D5.define(({ Bool: _, Iterable: H, Nullable: V }) => ({
    booleans: [V(H(_)), null]
  }));
  let BooleanFilter = D5;
  const N5 = class N5 extends Filter$1 {
    constructor(_) {
      super(_);
    }
    get names() {
      return keys$5(this.args);
    }
    get values() {
      return values$3(this.args);
    }
    get func() {
      const _ = use_strict(this.code);
      return new Function(...this.names, "source", _);
    }
    compute_indices(_) {
      const H = _.get_length() ?? 1, V = this.func(...this.values, _);
      if (V == null)
        return BitSet.all_set(H);
      if (isArrayOf(V, isInteger$2))
        return BitSet.from_indices(H, V);
      if (isArrayOf(V, isBoolean$4))
        return BitSet.from_booleans(H, V);
      throw new Error(`expect an array of integers or booleans, or null, got ${V}`);
    }
  };
  mt(N5, "__name__", "CustomJSFilter"), N5.define(({ Unknown: _, Str: H, Dict: V }) => ({
    args: [V(_), {}],
    code: [H, ""]
  }));
  let CustomJSFilter = N5;
  const P5 = class P5 extends Filter$1 {
    constructor(_) {
      super(_);
    }
    compute_indices(_) {
      const H = _.get_column(this.column_name), V = _.get_length() ?? 1;
      if (H == null)
        return logger$1.warn(`${this}: groupby column '${this.column_name}' not found in the data source`), BitSet.all_set(V);
      {
        const q = new BitSet(V, 0);
        for (let G = 0; G < q.size; G++)
          H[G] === this.group && q.set(G);
        return q;
      }
    }
  };
  mt(P5, "__name__", "GroupFilter"), P5.define(({ Str: _ }) => ({
    column_name: [_],
    group: [_]
  }));
  let GroupFilter = P5;
  const $5 = class $5 extends Filter$1 {
    constructor(_) {
      super(_);
    }
    compute_indices(_) {
      const H = _.get_length() ?? 1, { indices: V } = this;
      return V == null ? BitSet.all_set(H) : BitSet.from_indices(H, V);
    }
  };
  mt($5, "__name__", "IndexFilter"), $5.define(({ Int: _, Iterable: H, Nullable: V }) => ({
    indices: [V(H(_)), null]
  }));
  let IndexFilter = $5;
  const F5 = class F5 extends Filter$1 {
    constructor(_) {
      super(_);
    }
    connect_signals() {
      super.connect_signals();
      const _ = () => {
        this.change.emit();
      }, H = (G) => {
        for (const J of G)
          this.connect(J.change, _);
      }, V = (G) => {
        for (const J of G)
          this.disconnect(J.change, _);
      };
      let q = (() => {
        const { operand: G } = this.properties;
        return G.is_unset ? [] : [G.get_value()];
      })();
      H(q), this.on_change(this.properties.operand, () => {
        V(q), q = [this.operand], H(q);
      });
    }
    compute_indices(_) {
      const H = this.operand.compute_indices(_);
      return H.invert(), H;
    }
  };
  mt(F5, "__name__", "InversionFilter"), F5.define(({ Ref: _ }) => ({
    operand: [_(Filter$1)]
  }));
  let InversionFilter = F5;
  class UnionFilter extends CompositeFilter {
    constructor(_) {
      super(_);
    }
    _inplace_op(_, H) {
      _.add(H);
    }
  }
  mt(UnionFilter, "__name__", "UnionFilter");
  class DifferenceFilter extends CompositeFilter {
    constructor(_) {
      super(_);
    }
    _inplace_op(_, H) {
      _.subtract(H);
    }
  }
  mt(DifferenceFilter, "__name__", "DifferenceFilter");
  class SymmetricDifferenceFilter extends CompositeFilter {
    constructor(_) {
      super(_);
    }
    _inplace_op(_, H) {
      _.symmetric_subtract(H);
    }
  }
  mt(SymmetricDifferenceFilter, "__name__", "SymmetricDifferenceFilter");
  class AnnularWedgeView extends XYGlyphView {
    async load_glglyph() {
      const { AnnularWedgeGL: _ } = await Promise.resolve().then(() => annular_wedge);
      return _;
    }
    _map_data() {
      this._define_or_inherit_attr("sinner_radius", () => this.model.properties.inner_radius.units == "data" ? this.inherited_x && this.inherited_inner_radius ? inherit : this.sdist(this.renderer.xscale, this.x, this.inner_radius) : this.inherited_inner_radius ? inherit : to_screen(this.inner_radius)), this._define_or_inherit_attr("souter_radius", () => this.model.properties.outer_radius.units == "data" ? this.inherited_x && this.inherited_outer_radius ? inherit : this.sdist(this.renderer.xscale, this.x, this.outer_radius) : this.inherited_outer_radius ? inherit : to_screen(this.outer_radius)), this._define_or_inherit_attr("max_souter_radius", () => max$j(this.souter_radius));
    }
    _render(_, H, V) {
      const { sx: q, sy: G, start_angle: J, end_angle: Z, sinner_radius: te, souter_radius: ne } = { ...this, ...V }, ee = this.model.direction == "anticlock";
      for (const ae of H) {
        const oe = q[ae], se = G[ae], ce = te[ae], le = ne[ae], ue = J.get(ae), de = Z.get(ae);
        if (!isFinite(oe + se + ce + le + ue + de))
          continue;
        const fe = de - ue;
        _.translate(oe, se), _.rotate(ue), _.beginPath(), _.moveTo(le, 0), _.arc(0, 0, le, 0, fe, ee), _.rotate(fe), _.lineTo(ce, 0), _.arc(0, 0, ce, 0, -fe, !ee), _.closePath(), _.rotate(-fe - ue), _.translate(-oe, -se), this.visuals.fill.apply(_, ae), this.visuals.hatch.apply(_, ae), this.visuals.line.apply(_, ae);
      }
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = this.renderer.xscale.invert(H), G = this.renderer.yscale.invert(V), J = H - this.max_souter_radius, Z = H + this.max_souter_radius, [te, ne] = this.renderer.xscale.r_invert(J, Z), ee = V - this.max_souter_radius, ae = V + this.max_souter_radius, [oe, se] = this.renderer.yscale.r_invert(ee, ae), ce = [];
      for (const de of this.index.indices({ x0: te, x1: ne, y0: oe, y1: se })) {
        const fe = this.souter_radius[de] ** 2, he = this.sinner_radius[de] ** 2, [me, pe] = this.renderer.xscale.r_compute(q, this.x[de]), [ye, ge] = this.renderer.yscale.r_compute(G, this.y[de]), ve = (me - pe) ** 2 + (ye - ge) ** 2;
        ve <= fe && ve >= he && ce.push(de);
      }
      const le = this.model.direction == "anticlock", ue = [];
      for (const de of ce) {
        const fe = Math.atan2(V - this.sy[de], H - this.sx[de]);
        (Math.abs(this.start_angle.get(de) - this.end_angle.get(de)) >= 2 * Math.PI || angle_between(-fe, -this.start_angle.get(de), -this.end_angle.get(de), le)) && ue.push(de);
      }
      return new Selection({ indices: ue });
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
    scenterxy(_) {
      const H = (this.sinner_radius[_] + this.souter_radius[_]) / 2, V = (this.start_angle.get(_) + this.end_angle.get(_)) / 2, q = this.sx[_] + H * Math.cos(V), G = this.sy[_] + H * Math.sin(V);
      return [q, G];
    }
  }
  mt(AnnularWedgeView, "__name__", "AnnularWedgeView");
  const Ep = class Ep extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Ep, "__name__", "AnnularWedge"), Ep.prototype.default_view = AnnularWedgeView, Ep.mixins([LineVector$1, FillVector$1, HatchVector$1]), Ep.define(({}) => ({
    direction: [Direction$1, "anticlock"],
    inner_radius: [DistanceSpec, { field: "inner_radius" }],
    outer_radius: [DistanceSpec, { field: "outer_radius" }],
    start_angle: [AngleSpec, { field: "start_angle" }],
    end_angle: [AngleSpec, { field: "end_angle" }]
  }));
  let AnnularWedge = Ep;
  class AnnulusView extends XYGlyphView {
    async load_glglyph() {
      const { AnnulusGL: _ } = await Promise.resolve().then(() => annulus);
      return _;
    }
    _map_data() {
      this._define_or_inherit_attr("sinner_radius", () => this.model.properties.inner_radius.units == "data" ? this.inherited_x && this.inherited_inner_radius ? inherit : this.sdist(this.renderer.xscale, this.x, this.inner_radius) : this.inherited_inner_radius ? inherit : to_screen(this.inner_radius)), this._define_or_inherit_attr("souter_radius", () => this.model.properties.outer_radius.units == "data" ? this.inherited_x && this.inherited_outer_radius ? inherit : this.sdist(this.renderer.xscale, this.x, this.outer_radius) : this.inherited_outer_radius ? inherit : to_screen(this.outer_radius));
    }
    _render(_, H, V) {
      const { sx: q, sy: G, sinner_radius: J, souter_radius: Z } = { ...this, ...V };
      for (const te of H) {
        const ne = q[te], ee = G[te], ae = J[te], oe = Z[te];
        isFinite(ne + ee + ae + oe) && (_.beginPath(), _.arc(ne, ee, ae, 0, 2 * Math.PI, !0), _.moveTo(ne + oe, ee), _.arc(ne, ee, oe, 2 * Math.PI, 0, !1), this.visuals.fill.apply(_, te), this.visuals.hatch.apply(_, te), this.visuals.line.apply(_, te));
      }
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = this.renderer.xscale.invert(H), G = this.renderer.yscale.invert(V);
      let J, Z, te, ne;
      if (this.model.properties.outer_radius.units == "data")
        J = q - this.max_outer_radius, te = q + this.max_outer_radius, Z = G - this.max_outer_radius, ne = G + this.max_outer_radius;
      else {
        const ae = H - this.max_outer_radius, oe = H + this.max_outer_radius;
        [J, te] = this.renderer.xscale.r_invert(ae, oe);
        const se = V - this.max_outer_radius, ce = V + this.max_outer_radius;
        [Z, ne] = this.renderer.yscale.r_invert(se, ce);
      }
      const ee = [];
      for (const ae of this.index.indices({ x0: J, x1: te, y0: Z, y1: ne })) {
        const oe = this.souter_radius[ae] ** 2, se = this.sinner_radius[ae] ** 2, [ce, le] = this.renderer.xscale.r_compute(q, this.x[ae]), [ue, de] = this.renderer.yscale.r_compute(G, this.y[ae]), fe = (ce - le) ** 2 + (ue - de) ** 2;
        fe <= oe && fe >= se && ee.push(ae);
      }
      return new Selection({ indices: ee });
    }
    draw_legend_for_index(_, { x0: H, y0: V, x1: q, y1: G }, J) {
      const Z = J + 1, te = new Array(Z);
      te[J] = (H + q) / 2;
      const ne = new Array(Z);
      ne[J] = (V + G) / 2;
      const ee = Math.min(Math.abs(q - H), Math.abs(G - V)) * 0.5, ae = new Array(Z);
      ae[J] = ee * 0.4;
      const oe = new Array(Z);
      oe[J] = ee * 0.8, this._render(_, [J], { sx: te, sy: ne, sinner_radius: ae, souter_radius: oe });
    }
  }
  mt(AnnulusView, "__name__", "AnnulusView");
  const Ap = class Ap extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Ap, "__name__", "Annulus"), Ap.prototype.default_view = AnnulusView, Ap.mixins([LineVector$1, FillVector$1, HatchVector$1]), Ap.define(({}) => ({
    inner_radius: [DistanceSpec, { field: "inner_radius" }],
    outer_radius: [DistanceSpec, { field: "outer_radius" }]
  }));
  let Annulus = Ap;
  class ArcView extends XYGlyphView {
    _map_data() {
      this._define_or_inherit_attr("sradius", () => this.model.properties.radius.units == "data" ? this.inherited_x && this.inherited_radius ? inherit : this.sdist(this.renderer.xscale, this.x, this.radius) : this.inherited_radius ? inherit : to_screen(this.radius));
    }
    _render(_, H, V) {
      if (!this.visuals.line.doit)
        return;
      const { sx: q, sy: G, sradius: J, start_angle: Z, end_angle: te } = { ...this, ...V }, ne = this.model.direction == "anticlock";
      for (const ee of H) {
        const ae = q[ee], oe = G[ee], se = J[ee], ce = Z.get(ee), le = te.get(ee);
        isFinite(ae + oe + se + ce + le) && (this._render_decorations(_, ee, ae, oe, se, ce, le, ne), _.beginPath(), _.arc(ae, oe, se, ce, le, ne), this.visuals.line.apply(_, ee));
      }
    }
    _render_decorations(_, H, V, q, G, J, Z, te) {
      const { sin: ne, cos: ee, PI: ae } = Math;
      for (const oe of this.decorations.values()) {
        if (_.save(), oe.model.node == "start") {
          const se = G * ee(J) + V, ce = G * ne(J) + q;
          _.translate(se, ce), _.rotate(J + ae);
        } else if (oe.model.node == "end") {
          const se = G * Math.cos(Z) + V, ce = G * Math.sin(Z) + q;
          _.translate(se, ce), _.rotate(Z);
        }
        oe.marking.render(_, H), _.restore();
      }
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(ArcView, "__name__", "ArcView");
  const Qp = class Qp extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Qp, "__name__", "Arc"), Qp.prototype.default_view = ArcView, Qp.mixins(LineVector$1), Qp.define(({}) => ({
    direction: [Direction$1, "anticlock"],
    radius: [DistanceSpec, { field: "radius" }],
    start_angle: [AngleSpec, { field: "start_angle" }],
    end_angle: [AngleSpec, { field: "end_angle" }]
  }));
  let Arc = Qp;
  const { abs: abs$7, sqrt: sqrt$5, min: min$5, max: max$a } = Math;
  function qbb(O, _, H, V, q, G) {
    function J(ae, oe, se) {
      if (oe == (ae + se) / 2)
        return [ae, se];
      {
        const ce = (ae - oe) / (ae - 2 * oe + se), le = ae * (1 - ce) ** 2 + 2 * oe * (1 - ce) * ce + se * ce ** 2;
        return [min$5(ae, se, le), max$a(ae, se, le)];
      }
    }
    const [Z, te] = J(O, H, q), [ne, ee] = J(_, V, G);
    return {
      x0: Z,
      x1: te,
      y0: ne,
      y1: ee
    };
  }
  function cbb(O, _, H, V, q, G, J, Z) {
    const te = J, ne = Z;
    J = H, Z = V;
    const ee = q, ae = G, oe = [];
    for (let pe = 0; pe <= 2; pe++) {
      let ye, ge, ve;
      if (pe == 0 ? (ge = 6 * O - 12 * J + 6 * ee, ye = -3 * O + 9 * J - 9 * ee + 3 * te, ve = 3 * J - 3 * O) : (ge = 6 * _ - 12 * Z + 6 * ae, ye = -3 * _ + 9 * Z - 9 * ae + 3 * ne, ve = 3 * Z - 3 * _), abs$7(ye) < 1e-12) {
        if (abs$7(ge) < 1e-12)
          continue;
        const Ee = -ve / ge;
        0 < Ee && Ee < 1 && oe.push(Ee);
        continue;
      }
      const be = ge ** 2 - 4 * ve * ye, Ce = sqrt$5(be);
      if (be < 0)
        continue;
      const Se = (-ge + Ce) / (2 * ye);
      0 < Se && Se < 1 && oe.push(Se);
      const we = (-ge - Ce) / (2 * ye);
      0 < we && we < 1 && oe.push(we);
    }
    const se = oe.length;
    let ce = se;
    const le = Array(se + 2), ue = Array(se + 2);
    for (; ce-- > 0; ) {
      const pe = oe[ce], ye = 1 - pe, ge = ye ** 3 * O + 3 * ye ** 2 * pe * J + 3 * ye * pe ** 2 * ee + pe ** 3 * te, ve = ye ** 3 * _ + 3 * ye ** 2 * pe * Z + 3 * ye * pe ** 2 * ae + pe ** 3 * ne;
      le[ce] = ge, ue[ce] = ve;
    }
    le[se] = O, ue[se] = _, le[se + 1] = te, ue[se + 1] = ne;
    const [de, fe, he, me] = minmax2(le, ue);
    return { x0: de, x1: fe, y0: he, y1: me };
  }
  class BezierView extends GlyphView {
    _project_data() {
      this._project_xy("x0", this.x0, "y0", this.y0), this._project_xy("x1", this.x1, "y1", this.y1);
    }
    _index_data(_) {
      const { data_size: H, x0: V, y0: q, x1: G, y1: J, cx0: Z, cy0: te, cx1: ne, cy1: ee } = this;
      for (let ae = 0; ae < H; ae++) {
        const oe = V[ae], se = q[ae], ce = G[ae], le = J[ae], ue = Z[ae], de = te[ae], fe = ne[ae], he = ee[ae];
        if (!isFinite(oe + ce + se + le + ue + de + fe + he))
          _.add_empty();
        else {
          const { x0: me, y0: pe, x1: ye, y1: ge } = cbb(oe, se, ue, de, fe, he, ce, le);
          _.add_rect(me, pe, ye, ge);
        }
      }
    }
    _render(_, H, V) {
      if (!this.visuals.line.doit)
        return;
      const { sx0: q, sy0: G, sx1: J, sy1: Z, scx0: te, scy0: ne, scx1: ee, scy1: ae } = { ...this, ...V };
      for (const oe of H) {
        const se = q[oe], ce = G[oe], le = J[oe], ue = Z[oe], de = te[oe], fe = ne[oe], he = ee[oe], me = ae[oe];
        isFinite(se + ce + le + ue + de + fe + he + me) && (_.beginPath(), _.moveTo(se, ce), _.bezierCurveTo(de, fe, he, me, le, ue), this.visuals.line.apply(_, oe));
      }
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
    scenterxy() {
      throw new Error(`${this}.scenterxy() is not implemented`);
    }
  }
  mt(BezierView, "__name__", "BezierView");
  const Cp = class Cp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Cp, "__name__", "Bezier"), Cp.prototype.default_view = BezierView, Cp.define(({}) => ({
    x0: [XCoordinateSpec, { field: "x0" }],
    y0: [YCoordinateSpec, { field: "y0" }],
    x1: [XCoordinateSpec, { field: "x1" }],
    y1: [YCoordinateSpec, { field: "y1" }],
    cx0: [XCoordinateSpec, { field: "cx0" }],
    cy0: [YCoordinateSpec, { field: "cy0" }],
    cx1: [XCoordinateSpec, { field: "cx1" }],
    cy1: [YCoordinateSpec, { field: "cy1" }]
  })), Cp.mixins(LineVector$1);
  let Bezier = Cp;
  class LRTBView extends GlyphView {
    async load_glglyph() {
      const { LRTBGL: _ } = await Promise.resolve().then(() => lrtb);
      return _;
    }
    get_anchor_point(_, H, V) {
      const q = Math.min(this.sleft[H], this.sright[H]), G = Math.max(this.sright[H], this.sleft[H]), J = Math.min(this.stop[H], this.sbottom[H]), Z = Math.max(this.sbottom[H], this.stop[H]);
      switch (_) {
        case "top_left":
          return { x: q, y: J };
        case "top":
        case "top_center":
          return { x: (q + G) / 2, y: J };
        case "top_right":
          return { x: G, y: J };
        case "bottom_left":
          return { x: q, y: Z };
        case "bottom":
        case "bottom_center":
          return { x: (q + G) / 2, y: Z };
        case "bottom_right":
          return { x: G, y: Z };
        case "left":
        case "center_left":
          return { x: q, y: (J + Z) / 2 };
        case "center":
        case "center_center":
          return { x: (q + G) / 2, y: (J + Z) / 2 };
        case "right":
        case "center_right":
          return { x: G, y: (J + Z) / 2 };
      }
    }
    _set_data(_) {
      super._set_data(_), this.border_radius = border_radius(this.model.border_radius);
    }
    _index_data(_) {
      const { min: H, max: V } = Math, { data_size: q } = this;
      for (let G = 0; G < q; G++) {
        const { l: J, r: Z, t: te, b: ne } = this._lrtb(G);
        _.add_rect(H(J, Z), H(te, ne), V(Z, J), V(te, ne));
      }
    }
    _render(_, H, V) {
      const { sleft: q, sright: G, stop: J, sbottom: Z, border_radius: te } = { ...this, ...V };
      for (const ne of H) {
        const ee = q[ne], ae = J[ne], oe = G[ne], se = Z[ne];
        if (!isFinite(ee + ae + oe + se))
          continue;
        _.beginPath();
        const ce = BBox$1.from_lrtb({ left: ee, right: oe, top: ae, bottom: se });
        round_rect(_, ce, te), this.visuals.fill.apply(_, ne), this.visuals.hatch.apply(_, ne), this.visuals.line.apply(_, ne);
      }
    }
    // We need to clamp the endpoints inside the viewport, because various browser canvas
    // implementations have issues drawing rects with enpoints far outside the viewport
    _clamp_to_viewport() {
      const { min: _, max: H } = Math, { h_range: V, v_range: q } = this.renderer.plot_view.frame.bbox;
      this.inherited_sleft || inplace_map(this.sleft, (G) => H(G, V.start)), this.inherited_sright || inplace_map(this.sright, (G) => _(G, V.end)), this.inherited_stop || inplace_map(this.stop, (G) => H(G, q.start)), this.inherited_sbottom || inplace_map(this.sbottom, (G) => _(G, q.end));
    }
    _hit_rect(_) {
      return this._hit_rect_against_index(_);
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = this.renderer.xscale.invert(H), G = this.renderer.yscale.invert(V), J = [...this.index.indices({ x0: q, y0: G, x1: q, y1: G })];
      return new Selection({ indices: J });
    }
    _hit_span(_) {
      const { sx: H, sy: V } = _;
      let q;
      if (_.direction == "v") {
        const G = this.renderer.yscale.invert(V), J = this.renderer.plot_view.frame.bbox.h_range, [Z, te] = this.renderer.xscale.r_invert(J.start, J.end);
        q = [...this.index.indices({ x0: Z, y0: G, x1: te, y1: G })];
      } else {
        const G = this.renderer.xscale.invert(H), J = this.renderer.plot_view.frame.bbox.v_range, [Z, te] = this.renderer.yscale.r_invert(J.start, J.end);
        q = [...this.index.indices({ x0: G, y0: Z, x1: G, y1: te })];
      }
      return new Selection({ indices: q });
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(LRTBView, "__name__", "LRTBView");
  const tm = class tm extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(tm, "__name__", "LRTB"), tm.mixins([LineVector$1, FillVector$1, HatchVector$1]), tm.define(() => ({
    border_radius: [BorderRadius, 0]
  }));
  let LRTB = tm;
  class BlockView extends LRTBView {
    scenterxy(_) {
      const H = this.sleft[_] / 2 + this.sright[_] / 2, V = this.stop[_] / 2 + this.sbottom[_] / 2;
      return [H, V];
    }
    _lrtb(_) {
      const H = this.x[_], V = this.y[_], q = this.width.get(_), G = this.height.get(_), [J, Z] = minmax(H, H + q), [te, ne] = minmax(V, V + G);
      return { l: J, r: Z, t: ne, b: te };
    }
    _map_data() {
      const { sx: _, sy: H } = this, V = _.length;
      if (this.inherited_x && this.inherited_width)
        this._inherit_attr("sleft"), this._inherit_attr("sright");
      else {
        const q = this.sdist(this.renderer.xscale, this.x, this.width, "edge"), G = new ScreenArray(V), J = new ScreenArray(V);
        for (let Z = 0; Z < V; Z++)
          G[Z] = _[Z], J[Z] = _[Z] + q[Z];
        this._define_attr("sleft", G), this._define_attr("sright", J);
      }
      if (this.inherited_y && this.inherited_height)
        this._inherit_attr("stop"), this._inherit_attr("sbottom");
      else {
        const q = this.sdist(this.renderer.yscale, this.y, this.height, "edge"), G = new ScreenArray(V), J = new ScreenArray(V);
        for (let Z = 0; Z < V; Z++)
          G[Z] = H[Z] - q[Z], J[Z] = H[Z];
        this._define_attr("stop", G), this._define_attr("sbottom", J);
      }
      this._clamp_to_viewport();
    }
  }
  mt(BlockView, "__name__", "BlockView");
  var I2;
  let Block$1 = (I2 = class extends LRTB {
    constructor(_) {
      super(_);
    }
  }, mt(I2, "__name__", "Block"), I2.prototype.default_view = BlockView, I2.define(({}) => ({
    x: [XCoordinateSpec, { field: "x" }],
    y: [YCoordinateSpec, { field: "y" }],
    width: [DistanceSpec, { value: 1 }],
    height: [DistanceSpec, { value: 1 }]
  })), I2);
  class CircleView extends XYGlyphView {
    async load_glglyph() {
      const { CircleGL: _ } = await Promise.resolve().then(() => circle$2);
      return _;
    }
    _index_data(_) {
      const { x: H, y: V, radius: q, data_size: G } = this;
      for (let J = 0; J < G; J++) {
        const Z = H[J], te = V[J], ne = q.get(J);
        _.add_rect(Z - ne, te - ne, Z + ne, te + ne);
      }
    }
    _map_data() {
      this._define_or_inherit_attr("sradius", () => {
        if (this.model.properties.radius.units == "data") {
          const _ = () => this.sdist(this.renderer.xscale, this.x, this.radius), H = () => this.sdist(this.renderer.yscale, this.y, this.radius), { radius_dimension: V } = this.model;
          switch (V) {
            case "x":
              return this.inherited_x && this.inherited_radius ? inherit : _();
            case "y":
              return this.inherited_y && this.inherited_radius ? inherit : H();
            case "min":
            case "max":
              return this.inherited_x && this.inherited_y && this.inherited_radius ? inherit : elementwise(_(), H(), Math[V]);
          }
        } else
          return this.inherited_sradius ? inherit : to_screen(this.radius);
      });
    }
    _mask_data() {
      const { frame: _ } = this.renderer.plot_view, H = _.x_target, V = _.y_target;
      let q, G;
      return this.model.properties.radius.units == "data" ? (q = H.map((J) => this.renderer.xscale.invert(J)).widen(this.max_radius), G = V.map((J) => this.renderer.yscale.invert(J)).widen(this.max_radius)) : (q = H.widen(this.max_radius).map((J) => this.renderer.xscale.invert(J)), G = V.widen(this.max_radius).map((J) => this.renderer.yscale.invert(J))), this.index.indices({
        x0: q.start,
        x1: q.end,
        y0: G.start,
        y1: G.end
      });
    }
    _render(_, H, V) {
      const { sx: q, sy: G, sradius: J } = { ...this, ...V };
      for (const Z of H) {
        const te = q[Z], ne = G[Z], ee = J[Z];
        isFinite(te + ne + ee) && (_.beginPath(), _.arc(te, ne, ee, 0, 2 * Math.PI, !1), this.visuals.fill.apply(_, Z), this.visuals.hatch.apply(_, Z), this.visuals.line.apply(_, Z));
      }
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = this.renderer.xscale.invert(H), G = this.renderer.yscale.invert(V), { hit_dilation: J } = this.model, [Z, te, ne, ee] = (() => {
        if (this.model.properties.radius.units == "data") {
          const se = this.max_radius * J, ce = q - se, le = q + se, ue = G - se, de = G + se;
          return [ce, le, ue, de];
        } else {
          const se = this.max_radius * J, ce = H - se, le = H + se, ue = V - se, de = V + se, [fe, he] = this.renderer.xscale.r_invert(ce, le), [me, pe] = this.renderer.yscale.r_invert(ue, de);
          return [fe, he, me, pe];
        }
      })(), ae = this.index.indices({ x0: Z, x1: te, y0: ne, y1: ee }), oe = [];
      if (this.model.properties.radius.units == "data")
        for (const se of ae) {
          const ce = (this.sradius[se] * J) ** 2, [le, ue] = this.renderer.xscale.r_compute(q, this.x[se]), [de, fe] = this.renderer.yscale.r_compute(G, this.y[se]);
          (le - ue) ** 2 + (de - fe) ** 2 <= ce && oe.push(se);
        }
      else
        for (const se of ae) {
          const ce = (this.sradius[se] * J) ** 2;
          (this.sx[se] - H) ** 2 + (this.sy[se] - V) ** 2 <= ce && oe.push(se);
        }
      return new Selection({ indices: oe });
    }
    _hit_span(_) {
      const { sx: H, sy: V } = _, q = this.bounds(), [G, J, Z, te] = (() => {
        const ee = this.max_radius;
        if (_.direction == "h") {
          const ae = H - ee, oe = H + ee, [se, ce] = this.renderer.xscale.r_invert(ae, oe), { y0: le, y1: ue } = q;
          return [se, ce, le, ue];
        } else {
          const ae = V - ee, oe = V + ee, { x0: se, x1: ce } = q, [le, ue] = this.renderer.yscale.r_invert(ae, oe);
          return [se, ce, le, ue];
        }
      })(), ne = [...this.index.indices({ x0: G, x1: J, y0: Z, y1: te })];
      return new Selection({ indices: ne });
    }
    _hit_rect(_) {
      const { sx0: H, sx1: V, sy0: q, sy1: G } = _, [J, Z] = this.renderer.xscale.r_invert(H, V), [te, ne] = this.renderer.yscale.r_invert(q, G), ee = this.index.indices({ x0: J, x1: Z, y0: te, y1: ne }), ae = [];
      for (const oe of ee) {
        const se = this.sx[oe], ce = this.sy[oe];
        H <= se && se <= V && q <= ce && ce <= G && ae.push(oe);
      }
      return new Selection({ indices: ae });
    }
    _hit_poly(_) {
      const { sx: H, sy: V } = _, q = (() => {
        const [J, Z, te, ne] = minmax2(H, V), [ee, ae] = this.renderer.xscale.r_invert(J, Z), [oe, se] = this.renderer.yscale.r_invert(te, ne);
        return this.index.indices({ x0: ee, x1: ae, y0: oe, y1: se });
      })(), G = [];
      for (const J of q)
        point_in_poly(this.sx[J], this.sy[J], H, V) && G.push(J);
      return new Selection({ indices: G });
    }
    // circle does not inherit from marker (since it also accepts radius) so we
    // must supply a draw_legend for it  here
    draw_legend_for_index(_, { x0: H, y0: V, x1: q, y1: G }, J) {
      const Z = J + 1, te = new Array(Z);
      te[J] = (H + q) / 2;
      const ne = new Array(Z);
      ne[J] = (V + G) / 2;
      const ee = new Array(Z);
      ee[J] = Math.min(Math.abs(q - H), Math.abs(G - V)) * 0.2, this._render(_, [J], { sx: te, sy: ne, sradius: ee });
    }
  }
  mt(CircleView, "__name__", "CircleView");
  const Lp = class Lp extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Lp, "__name__", "Circle"), Lp.prototype.default_view = CircleView, Lp.mixins([LineVector$1, FillVector$1, HatchVector$1]), Lp.define(({ Float: _ }) => ({
    radius: [DistanceSpec, { field: "radius" }],
    radius_dimension: [RadiusDimension, "x"],
    hit_dilation: [_, 1]
  }));
  let Circle = Lp;
  class CenterRotatableView extends XYGlyphView {
    get max_w2() {
      return this.model.properties.width.units == "data" ? this.max_width / 2 : 0;
    }
    get max_h2() {
      return this.model.properties.height.units == "data" ? this.max_height / 2 : 0;
    }
    _bounds({ x0: _, x1: H, y0: V, y1: q }) {
      const { max_w2: G, max_h2: J } = this;
      return {
        x0: _ - G,
        x1: H + G,
        y0: V - J,
        y1: q + J
      };
    }
  }
  mt(CenterRotatableView, "__name__", "CenterRotatableView");
  const nm = class nm extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(nm, "__name__", "CenterRotatable"), nm.mixins([LineVector$1, FillVector$1, HatchVector$1]), nm.define(({}) => ({
    angle: [AngleSpec, 0],
    width: [DistanceSpec, { field: "width" }],
    height: [DistanceSpec, { field: "height" }]
  }));
  let CenterRotatable = nm;
  class EllipseView extends CenterRotatableView {
    _map_data() {
      this._define_or_inherit_attr("swidth", () => this.model.properties.width.units == "data" ? this.inherited_x && this.inherited_width ? inherit : this.sdist(this.renderer.xscale, this.x, this.width, "center") : this.inherited_width ? inherit : to_screen(this.width)), this._define_or_inherit_attr("sheight", () => this.model.properties.height.units == "data" ? this.inherited_y && this.inherited_height ? inherit : this.sdist(this.renderer.yscale, this.y, this.height, "center") : this.inherited_height ? inherit : to_screen(this.height));
    }
    _render(_, H, V) {
      const { sx: q, sy: G, swidth: J, sheight: Z, angle: te } = { ...this, ...V };
      for (const ne of H) {
        const ee = q[ne], ae = G[ne], oe = J[ne], se = Z[ne], ce = te.get(ne);
        isFinite(ee + ae + oe + se + ce) && (_.beginPath(), _.ellipse(ee, ae, oe / 2, se / 2, ce, 0, 2 * Math.PI), this.visuals.fill.apply(_, ne), this.visuals.hatch.apply(_, ne), this.visuals.line.apply(_, ne));
      }
    }
    _hit_point(_) {
      let H, V, q, G, J, Z, te, ne, ee;
      const { sx: ae, sy: oe } = _, se = this.renderer.xscale.invert(ae), ce = this.renderer.yscale.invert(oe);
      this.model.properties.width.units == "data" ? (H = se - this.max_width, V = se + this.max_width) : (Z = ae - this.max_width, te = ae + this.max_width, [H, V] = this.renderer.xscale.r_invert(Z, te)), this.model.properties.height.units == "data" ? (q = ce - this.max_height, G = ce + this.max_height) : (ne = oe - this.max_height, ee = oe + this.max_height, [q, G] = this.renderer.yscale.r_invert(ne, ee));
      const le = this.index.indices({ x0: H, x1: V, y0: q, y1: G }), ue = [];
      for (const de of le)
        J = point_in_ellipse(ae, oe, this.angle.get(de), this.sheight[de] / 2, this.swidth[de] / 2, this.sx[de], this.sy[de]), J && ue.push(de);
      return new Selection({ indices: ue });
    }
    draw_legend_for_index(_, { x0: H, y0: V, x1: q, y1: G }, J) {
      const Z = J + 1, te = new Array(Z);
      te[J] = (H + q) / 2;
      const ne = new Array(Z);
      ne[J] = (V + G) / 2;
      const ee = this.swidth[J] / this.sheight[J], ae = Math.min(Math.abs(q - H), Math.abs(G - V)) * 0.8, oe = new Array(Z), se = new Array(Z);
      ee > 1 ? (oe[J] = ae, se[J] = ae / ee) : (oe[J] = ae * ee, se[J] = ae);
      const ce = new UniformScalar(0, Z);
      this._render(_, [J], { sx: te, sy: ne, swidth: oe, sheight: se, angle: ce });
    }
  }
  mt(EllipseView, "__name__", "EllipseView");
  const B5 = class B5 extends CenterRotatable {
    constructor(_) {
      super(_);
    }
  };
  mt(B5, "__name__", "Ellipse"), B5.prototype.default_view = EllipseView;
  let Ellipse = B5;
  class HBarView extends LRTBView {
    scenterxy(_) {
      const H = (this.sleft[_] + this.sright[_]) / 2, V = this.sy[_];
      return [H, V];
    }
    _lrtb(_) {
      const H = this.left[_], V = this.right[_], q = this.y[_], G = this.height.get(_) / 2, J = Math.min(H, V), Z = Math.max(H, V), te = q + G, ne = q - G;
      return { l: J, r: Z, t: te, b: ne };
    }
    _map_data() {
      if (this.inherited_y && this.inherited_height)
        this._inherit_attr("sheight"), this._inherit_attr("stop"), this._inherit_attr("sbottom");
      else {
        const _ = this.sdist(this.renderer.yscale, this.y, this.height, "center"), { sy: H } = this, V = this.sy.length, q = new ScreenArray(V), G = new ScreenArray(V);
        for (let J = 0; J < V; J++) {
          const Z = H[J], te = _[J];
          q[J] = Z - te / 2, G[J] = Z + te / 2;
        }
        this._define_attr("sheight", _), this._define_attr("stop", q), this._define_attr("sbottom", G);
      }
      this._clamp_to_viewport();
    }
  }
  mt(HBarView, "__name__", "HBarView");
  const im = class im extends LRTB {
    constructor(_) {
      super(_);
    }
  };
  mt(im, "__name__", "HBar"), im.prototype.default_view = HBarView, im.define(({}) => ({
    left: [XCoordinateSpec, { value: 0 }],
    y: [YCoordinateSpec, { field: "y" }],
    height: [DistanceSpec, { value: 1 }],
    right: [XCoordinateSpec, { field: "right" }]
  }));
  let HBar = im;
  class HexTileView extends GlyphView {
    async load_glglyph() {
      const { HexTileGL: _ } = await Promise.resolve().then(() => hex_tile);
      return _;
    }
    scenterxy(_) {
      const H = this.sx[_], V = this.sy[_];
      return [H, V];
    }
    _set_data() {
      const { orientation: _, size: H, aspect_scale: V } = this.model, { q, r: G } = this, J = this.q.length, Z = new Float64Array(J), te = new Float64Array(J), ne = Math.sqrt(3);
      if (_ == "pointytop")
        for (let ee = 0; ee < J; ee++) {
          const ae = q.get(ee), oe = G.get(ee) / 2;
          Z[ee] = H * ne * (ae + oe) / V, te[ee] = -3 * H * oe;
        }
      else
        for (let ee = 0; ee < J; ee++) {
          const ae = q.get(ee) / 2, oe = G.get(ee);
          Z[ee] = 3 * H * ae, te[ee] = -H * ne * (oe + ae) * V;
        }
      this._define_attr("x", Z), this._define_attr("y", te);
    }
    _project_data() {
      this._project_xy("x", this.x, "y", this.y);
    }
    _index_data(_) {
      let H = this.model.size, V = Math.sqrt(3) * H / 2;
      this.model.orientation == "flattop" ? ([V, H] = [H, V], H *= this.model.aspect_scale) : V /= this.model.aspect_scale;
      const { x: q, y: G, data_size: J } = this;
      for (let Z = 0; Z < J; Z++) {
        const te = q[Z], ne = G[Z];
        _.add_rect(te - V, ne - H, te + V, ne + H);
      }
    }
    // overriding map_data instead of _map_data because the default automatic mappings
    // for other glyphs (with cartesian coordinates) is not useful
    map_data() {
      var G;
      const [_, H] = this.renderer.coordinates.map_to_screen(this.x, this.y), [V, q] = this._get_unscaled_vertices();
      this._define_attr("sx", _), this._define_attr("sy", H), this._define_attr("svx", V), this._define_attr("svy", q), (G = this.glglyph) == null || G.set_data_mapped();
    }
    _get_unscaled_vertices() {
      const _ = this.model.size, H = this.model.aspect_scale;
      if (this.model.orientation == "pointytop") {
        const V = this.renderer.yscale, q = this.renderer.xscale, G = Math.abs(V.compute(0) - V.compute(_)), J = Math.sqrt(3) / 2 * Math.abs(q.compute(0) - q.compute(_)) / H, Z = G / 2, te = [0, -J, -J, 0, J, J], ne = [G, Z, -Z, -G, -Z, Z];
        return [te, ne];
      } else {
        const V = this.renderer.xscale, q = this.renderer.yscale, G = Math.abs(V.compute(0) - V.compute(_)), J = Math.sqrt(3) / 2 * Math.abs(q.compute(0) - q.compute(_)) * H, Z = G / 2, te = [G, Z, -Z, -G, -Z, Z], ne = [0, -J, -J, 0, J, J];
        return [te, ne];
      }
    }
    _render(_, H, V) {
      const { sx: q, sy: G, svx: J, svy: Z, scale: te } = { ...this, ...V };
      for (const ne of H) {
        const ee = q[ne], ae = G[ne], oe = te.get(ne);
        if (isFinite(ee + ae + oe)) {
          _.translate(ee, ae), _.beginPath();
          for (let se = 0; se < 6; se++)
            _.lineTo(J[se] * oe, Z[se] * oe);
          _.closePath(), _.translate(-ee, -ae), this.visuals.fill.apply(_, ne), this.visuals.hatch.apply(_, ne), this.visuals.line.apply(_, ne);
        }
      }
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = this.renderer.xscale.invert(H), G = this.renderer.yscale.invert(V), J = this.index.indices({ x0: q, y0: G, x1: q, y1: G }), Z = [];
      for (const te of J)
        point_in_poly(H - this.sx[te], V - this.sy[te], this.svx, this.svy) && Z.push(te);
      return new Selection({ indices: Z });
    }
    _hit_span(_) {
      const H = (() => {
        if (_.direction == "v") {
          const { sy: q } = _, G = this.renderer.yscale.invert(q), J = this.renderer.plot_view.frame.bbox.h_range, [Z, te] = this.renderer.xscale.r_invert(J.start, J.end);
          return { x0: Z, y0: G, x1: te, y1: G };
        } else {
          const { sx: q } = _, G = this.renderer.xscale.invert(q), J = this.renderer.plot_view.frame.bbox.v_range, [Z, te] = this.renderer.yscale.r_invert(J.start, J.end);
          return { x0: G, y0: Z, x1: G, y1: te };
        }
      })(), V = [...this.index.indices(H)];
      return new Selection({ indices: V });
    }
    _hit_rect(_) {
      const { sx0: H, sx1: V, sy0: q, sy1: G } = _, [J, Z] = this.renderer.xscale.r_invert(H, V), [te, ne] = this.renderer.yscale.r_invert(q, G), ee = [...this.index.indices({ x0: J, x1: Z, y0: te, y1: ne })];
      return new Selection({ indices: ee });
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(HexTileView, "__name__", "HexTileView");
  const h0 = class h0 extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(h0, "__name__", "HexTile"), h0.prototype.default_view = HexTileView, h0.mixins([LineVector$1, FillVector$1, HatchVector$1]), h0.define(({ Float: _ }) => ({
    r: [NumberSpec, { field: "r" }],
    q: [NumberSpec, { field: "q" }],
    scale: [NumberSpec, 1],
    size: [_, 1],
    aspect_scale: [_, 1],
    orientation: [HexTileOrientation, "pointytop"]
  })), h0.override({ line_color: null });
  let HexTile = h0;
  const { abs: abs$6, max: max$9 } = Math, UNUSED$3 = 0;
  class HSpanView extends GlyphView {
    after_visuals() {
      super.after_visuals(), this.max_line_width = max$i(this.line_width);
    }
    _index_data(_) {
      for (const H of this.y)
        _.add_point(UNUSED$3, H);
    }
    _bounds(_) {
      const { y0: H, y1: V } = _;
      return { x0: NaN, x1: NaN, y0: H, y1: V };
    }
    _map_data() {
      super._map_data();
      const { round: _ } = Math;
      if (!this.inherited_sy) {
        const H = map$4(this.sy, (V) => _(V));
        this._define_attr("sy", H);
      }
    }
    scenterxy(_) {
      const { hcenter: H } = this.renderer.plot_view.frame.bbox;
      return [H, this.sy[_]];
    }
    _render(_, H, V) {
      const { sy: q } = { ...this, ...V }, { left: G, right: J } = this.renderer.plot_view.frame.bbox;
      for (const Z of H) {
        const te = q[Z];
        isFinite(te) && (_.beginPath(), _.moveTo(G, te), _.lineTo(J, te), this.visuals.line.apply(_, Z));
      }
    }
    _get_candidates(_, H) {
      const { max_line_width: V } = this, [q, G] = this.renderer.yscale.r_invert(_ - V, (H ?? _) + V);
      return this.index.indices({ x0: 0, x1: 0, y0: q, y1: G });
    }
    _find_spans(_, H) {
      const { sy: V, line_width: q } = this, G = [];
      for (const J of _) {
        const Z = V[J], te = q.get(J);
        H(Z, te) && G.push(J);
      }
      return G;
    }
    _hit_point(_) {
      const { sy: H } = _, V = this._get_candidates(H), q = this._find_spans(V, (G, J) => abs$6(G - H) <= max$9(
        J / 2,
        2
        /*px*/
      ));
      return new Selection({ indices: q });
    }
    _hit_span(_) {
      const H = (() => {
        if (_.direction == "v")
          return range$9(0, this.data_size);
        {
          const { sy: V } = _, q = this._get_candidates(V);
          return this._find_spans(q, (G, J) => abs$6(G - V) <= max$9(
            J / 2,
            2
            /*px*/
          ));
        }
      })();
      return new Selection({ indices: H });
    }
    _hit_rect(_) {
      const H = (() => {
        const { sy0: V, sy1: q } = _, G = this._get_candidates(V, q);
        return this._find_spans(G, (J, Z) => V - Z / 2 <= J && J <= q + Z / 2);
      })();
      return new Selection({ indices: H });
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(HSpanView, "__name__", "HSpanView");
  const kp = class kp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(kp, "__name__", "HSpan"), kp.prototype.default_view = HSpanView, kp.mixins([LineVector$1]), kp.define(() => ({
    y: [YCoordinateSpec, { field: "y" }]
  }));
  let HSpan = kp;
  const UNUSED$2 = 0;
  class HStripView extends GlyphView {
    async lazy_initialize() {
      await super.lazy_initialize();
      const { webgl: _ } = this.renderer.plot_view.canvas_view;
      if (_ != null && _.regl_wrapper.has_webgl) {
        const { LRTBGL: H } = await Promise.resolve().then(() => lrtb);
        this.glglyph = new H(_.regl_wrapper, this);
      }
    }
    get sleft() {
      const { left: _ } = this.renderer.plot_view.frame.bbox, H = this.data_size, V = new ScreenArray(H);
      return V.fill(_), V;
    }
    get sright() {
      const { right: _ } = this.renderer.plot_view.frame.bbox, H = this.data_size, V = new ScreenArray(H);
      return V.fill(_), V;
    }
    get stop() {
      return this.sy0;
    }
    get sbottom() {
      return this.sy1;
    }
    _set_data(_) {
      super._set_data(_);
      const { abs: H } = Math, { max: V, map: q, zip: G } = iter, { y0: J, y1: Z } = this;
      if (this.inherited_y0 && this.inherited_y1)
        this._inherit_attr("max_height");
      else {
        const te = V(q(G(J, Z), ([ne, ee]) => H(ne - ee)));
        this._define_attr("max_height", te);
      }
    }
    _index_data(_) {
      const { y0: H, y1: V, data_size: q } = this;
      for (let G = 0; G < q; G++) {
        const J = H[G], Z = V[G];
        _.add_rect(UNUSED$2, J, UNUSED$2, Z);
      }
    }
    _bounds(_) {
      const { y0: H, y1: V } = _;
      return { x0: NaN, x1: NaN, y0: H, y1: V };
    }
    _map_data() {
      super._map_data();
      const { round: _ } = Math;
      if (!this.inherited_sy0) {
        const H = map$4(this.sy0, (V) => _(V));
        this._define_attr("sy0", H);
      }
      if (!this.inherited_sy1) {
        const H = map$4(this.sy1, (V) => _(V));
        this._define_attr("sy1", H);
      }
    }
    scenterxy(_) {
      const { hcenter: H } = this.renderer.plot_view.frame.bbox;
      return [H, (this.sy0[_] + this.sy1[_]) / 2];
    }
    _render(_, H, V) {
      const { sy0: q, sy1: G } = { ...this, ...V }, { left: J, right: Z, width: te } = this.renderer.plot_view.frame.bbox;
      for (const ne of H) {
        const ee = q[ne], ae = G[ne];
        isFinite(ee + ae) && (_.beginPath(), _.rect(J, ee, te, ae - ee), this.visuals.fill.apply(_, ne), this.visuals.hatch.apply(_, ne), _.beginPath(), _.moveTo(J, ee), _.lineTo(Z, ee), _.moveTo(J, ae), _.lineTo(Z, ae), this.visuals.line.apply(_, ne));
      }
    }
    _get_candidates(_, H) {
      const { max_height: V } = this, [q, G] = this.renderer.yscale.r_invert(_, H ?? _), J = q - V, Z = G + V;
      return this.index.indices({ x0: 0, x1: 0, y0: J, y1: Z });
    }
    _find_strips(_, H) {
      function V(Z, te) {
        return Z <= te ? H(Z, te) : H(te, Z);
      }
      const { sy0: q, sy1: G } = this, J = [];
      for (const Z of _) {
        const te = q[Z], ne = G[Z];
        V(te, ne) && J.push(Z);
      }
      return J;
    }
    _hit_point(_) {
      const { sy: H } = _, V = this._get_candidates(H), q = this._find_strips(V, (G, J) => G <= H && H <= J);
      return new Selection({ indices: q });
    }
    _hit_span(_) {
      const H = (() => {
        if (_.direction == "v")
          return range$9(0, this.data_size);
        {
          const { sy: V } = _, q = this._get_candidates(V);
          return this._find_strips(q, (G, J) => G <= V && V <= J);
        }
      })();
      return new Selection({ indices: H });
    }
    _hit_rect(_) {
      const H = (() => {
        const { sy0: V, sy1: q } = _, G = this._get_candidates(V, q);
        return this._find_strips(G, (J, Z) => V <= J && J <= q && V <= Z && Z <= q);
      })();
      return new Selection({ indices: H });
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(HStripView, "__name__", "HStripView");
  const Mp = class Mp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Mp, "__name__", "HStrip"), Mp.prototype.default_view = HStripView, Mp.mixins([LineVector$1, FillVector$1, HatchVector$1]), Mp.define(() => ({
    y0: [YCoordinateSpec, { field: "y0" }],
    y1: [YCoordinateSpec, { field: "y1" }]
  }));
  let HStrip = Mp;
  class ImageBaseView extends XYGlyphView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.global_alpha.change, () => this.renderer.request_render());
    }
    get image_dimension() {
      return 2;
    }
    get xy_scale() {
      switch (this.model.origin) {
        case "bottom_left":
          return { x: 1, y: -1 };
        case "top_left":
          return { x: 1, y: 1 };
        case "bottom_right":
          return { x: -1, y: -1 };
        case "top_right":
          return { x: -1, y: 1 };
      }
    }
    get xy_offset() {
      switch (this.model.origin) {
        case "bottom_left":
          return { x: 0, y: 1 };
        case "top_left":
          return { x: 0, y: 0 };
        case "bottom_right":
          return { x: 1, y: 1 };
        case "top_right":
          return { x: 1, y: 0 };
      }
    }
    get xy_anchor() {
      return anchor(this.model.anchor);
    }
    get xy_sign() {
      const _ = this.renderer.xscale.source_range, H = this.renderer.yscale.source_range;
      return {
        x: _.is_reversed ? -1 : 1,
        y: H.is_reversed ? -1 : 1
      };
    }
    _render(_, H, V) {
      const { image_data: q, sx: G, sy: J, sdw: Z, sdh: te } = { ...this, ...V }, { xy_sign: ne, xy_scale: ee, xy_offset: ae, xy_anchor: oe } = this;
      if (assert$2(q != null), _.save(), _.imageSmoothingEnabled = !1, this.visuals.image.doit)
        for (const se of H) {
          const ce = q[se], le = G[se], ue = J[se], de = Z[se], fe = te[se];
          if (ce == null || !isFinite(le + ue + de + fe))
            continue;
          const he = ne.x * oe.x * de, me = ne.y * oe.y * fe;
          _.save(), _.translate(le - he, ue - me), _.scale(ne.x * ee.x, ne.y * ee.y), this.visuals.image.set_vectorize(_, se), _.drawImage(ce, -ae.x * de, -ae.y * fe, de, fe), _.restore();
        }
      _.restore();
    }
    get _can_inherit_image_data() {
      return this.inherited_image;
    }
    _set_data(_) {
      const H = this.data_size;
      if (this._can_inherit_image_data)
        this._inherit_attr("image_data"), this._inherit_attr("image_width"), this._inherit_attr("image_height");
      else {
        (typeof this.image_data > "u" || this.image_data.length != H) && (this._define_attr("image_data", new Array(H).fill(null)), this._define_attr("image_width", new Uint32Array(H)), this._define_attr("image_height", new Uint32Array(H)));
        const { image_dimension: V } = this;
        for (let q = 0; q < H; q++) {
          if (_ != null && !_.includes(q))
            continue;
          const G = this.image.get(q);
          assert$2(G.dimension == V, `expected a ${V}D array, not ${G.dimension}D`);
          const [J, Z] = G.shape;
          this.image_width[q] = Z, this.image_height[q] = J;
          const te = this._flat_img_to_buf8(G);
          this._set_image_data_from_buffer(q, te);
        }
      }
    }
    _index_data(_) {
      const { data_size: H } = this;
      for (let V = 0; V < H; V++) {
        const [q, G, J, Z] = this._lrtb(V);
        _.add_rect(q, Z, G, J);
      }
    }
    _lrtb(_) {
      const H = this.dw.get(_), V = this.dh.get(_), q = this.x[_], G = this.y[_], { xy_anchor: J } = this, [Z, te] = [q - J.x * H, q + (1 - J.x) * H], [ne, ee] = [G + J.y * V, G - (1 - J.y) * V], [ae, oe] = Z <= te ? [Z, te] : [te, Z], [se, ce] = ne <= ee ? [ne, ee] : [ee, ne];
      return [ae, oe, ce, se];
    }
    _get_or_create_canvas(_) {
      assert$2(this.image_data != null);
      const H = this.image_data[_];
      if (H != null && H.width == this.image_width[_] && H.height == this.image_height[_])
        return H;
      {
        const V = document.createElement("canvas");
        return V.width = this.image_width[_], V.height = this.image_height[_], V;
      }
    }
    _set_image_data_from_buffer(_, H) {
      assert$2(this.image_data != null);
      const V = this._get_or_create_canvas(_), q = V.getContext("2d"), G = q.getImageData(0, 0, this.image_width[_], this.image_height[_]);
      G.data.set(H), q.putImageData(G, 0, 0), this.image_data[_] = V;
    }
    _map_data() {
      this._define_or_inherit_attr("sdw", () => this.model.properties.dw.units == "data" ? this.inherited_x && this.inherited_dw ? inherit : this.sdist(this.renderer.xscale, this.x, this.dw, "edge", this.model.dilate) : this.inherited_dw ? inherit : to_screen(this.dw)), this._define_or_inherit_attr("sdh", () => this.model.properties.dh.units == "data" ? this.inherited_y && this.inherited_dh ? inherit : this.sdist(this.renderer.yscale, this.y, this.dh, "edge", this.model.dilate) : this.inherited_dh ? inherit : to_screen(this.dh));
    }
    _image_index(_, H, V) {
      const [q, G, J, Z] = this._lrtb(_), te = this.image_width[_], ne = this.image_height[_], ee = (G - q) / te, ae = (J - Z) / ne, oe = Math.floor((H - q) / ee), se = Math.floor((V - Z) / ae);
      return { index: _, i: oe, j: se, flat_index: se * te + oe };
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = this.renderer.xscale.invert(H), G = this.renderer.yscale.invert(V), J = this.index.indices({ x0: q, x1: q, y0: G, y1: G }), Z = new Selection(), te = [];
      for (const ne of J)
        isFinite(H) && isFinite(V) && (te.push(ne), Z.image_indices.push(this._image_index(ne, q, G)));
      return Z.indices = te, Z;
    }
  }
  mt(ImageBaseView, "__name__", "ImageBaseView");
  const am = class am extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(am, "__name__", "ImageBase"), am.mixins(ImageVector$1), am.define(({ Bool: _ }) => ({
    image: [NDArraySpec, { field: "image" }],
    dw: [DistanceSpec, { field: "dw" }],
    dh: [DistanceSpec, { field: "dh" }],
    dilate: [_, !1],
    origin: [ImageOrigin, "bottom_left"],
    anchor: [Anchor$1, "bottom_left"]
  }));
  let ImageBase = am;
  class ImageView extends ImageBaseView {
    async load_glglyph() {
      const { ImageGL: _ } = await Promise.resolve().then(() => image$2);
      return _;
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.color_mapper.change, () => this._update_image());
    }
    _update_image() {
      this.glglyph != null && this.glglyph.set_image_changed(), this.image_data != null && (this._set_data(null), this.renderer.request_render());
    }
    get _can_inherit_image_data() {
      return super._can_inherit_image_data && this._can_inherit_from(this.model.properties.color_mapper, this.base);
    }
    _flat_img_to_buf8(_) {
      return this.model.color_mapper.rgba_mapper.v_compute(_);
    }
  }
  mt(ImageView, "__name__", "ImageView");
  const Greys9 = () => ["#000000", "#252525", "#525252", "#737373", "#969696", "#bdbdbd", "#d9d9d9", "#f0f0f0", "#ffffff"];
  var D2;
  let Image$1 = (D2 = class extends ImageBase {
    constructor(_) {
      super(_);
    }
  }, mt(D2, "__name__", "Image"), D2.prototype.default_view = ImageView, D2.define(({ Ref: _ }) => ({
    color_mapper: [_(ColorMapper), () => new LinearColorMapper({ palette: Greys9() })]
  })), D2);
  class ImageRGBAView extends ImageBaseView {
    async load_glglyph() {
      const { ImageGL: _ } = await Promise.resolve().then(() => image$2);
      return _;
    }
    _flat_img_to_buf8(_) {
      const H = isTypedArray$2(_) ? _ : new Uint32Array(_);
      return new Uint8ClampedArray(H.buffer);
    }
  }
  mt(ImageRGBAView, "__name__", "ImageRGBAView");
  const H5 = class H5 extends ImageBase {
    constructor(_) {
      super(_);
    }
  };
  mt(H5, "__name__", "ImageRGBA"), H5.prototype.default_view = ImageRGBAView;
  let ImageRGBA = H5;
  class ImageStackView extends ImageBaseView {
    async load_glglyph() {
      const { ImageGL: _ } = await Promise.resolve().then(() => image$2);
      return _;
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.color_mapper.change, () => this._update_image());
    }
    get image_dimension() {
      return 3;
    }
    _update_image() {
      this.glglyph != null && this.glglyph.set_image_changed(), this.image_data != null && (this._set_data(null), this.renderer.request_render());
    }
    get _can_inherit_image_data() {
      return super._can_inherit_image_data && this._can_inherit_from(this.model.properties.color_mapper, this.base);
    }
    _flat_img_to_buf8(_) {
      return this.model.color_mapper.rgba_mapper.v_compute(_);
    }
  }
  mt(ImageStackView, "__name__", "ImageStackView");
  const om = class om extends ImageBase {
    constructor(_) {
      super(_);
    }
  };
  mt(om, "__name__", "ImageStack"), om.prototype.default_view = ImageStackView, om.define(({ Ref: _ }) => ({
    color_mapper: [_(StackColorMapper)]
  }));
  let ImageStack = om;
  class ImageURLView extends XYGlyphView {
    constructor() {
      super(...arguments);
      mt(this, "_images_rendered", !1);
      mt(this, "_bounds_rect");
      mt(this, "anchor");
      /*protected*/
      mt(this, "image", new Array(0));
      mt(this, "loaders");
      mt(this, "resolved");
      mt(this, "_set_data_iteration", 0);
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.global_alpha.change, () => this.renderer.request_render());
    }
    _index_data(H) {
      const { data_size: V } = this;
      for (let q = 0; q < V; q++)
        H.add_empty();
    }
    _set_data() {
      if (this.inherited_url)
        return;
      this._set_data_iteration++;
      const { url: H } = this, V = H.length;
      this.image = resize$1(this.image, V, null), this.loaders = new Array(V).fill(null), this.resolved = new BitSet(V);
      const { retry_attempts: q, retry_timeout: G } = this.model, { _set_data_iteration: J } = this;
      for (let me = 0; me < V; me++) {
        const pe = H.get(me);
        if (pe == "")
          continue;
        const ye = new ImageLoader(pe, {
          loaded: (ge) => {
            this._set_data_iteration == J && !this.resolved.get(me) && (this.resolved.set(me), this.image[me] = ge, this.loaders[me] = null, this.renderer.request_render());
          },
          failed: () => {
            this._set_data_iteration == J && (this.resolved.set(me), this.loaders[me] = null, this.image[me] != null && (this.image[me] = null, this.renderer.request_render()));
          },
          attempts: q + 1,
          timeout: G
        });
        this.loaders[me] = ye;
      }
      const Z = this.model.properties.w.units == "data", te = this.model.properties.h.units == "data", ne = this.data_size, ee = new ScreenArray(Z ? 2 * ne : ne), ae = new ScreenArray(te ? 2 * ne : ne);
      this.anchor = anchor(this.model.anchor);
      const { x: oe, y: se } = this.anchor;
      function ce(me, pe) {
        const ye = me - oe * pe;
        return [ye, ye + pe];
      }
      function le(me, pe) {
        const ye = me + se * pe;
        return [ye, ye - pe];
      }
      if (Z)
        for (let me = 0; me < ne; me++)
          [ee[me], ee[ne + me]] = ce(this.x[me], this.w.get(me) ?? 0);
      else
        ee.set(this.x, 0);
      if (te)
        for (let me = 0; me < ne; me++)
          [ae[me], ae[ne + me]] = le(this.y[me], this.h.get(me) ?? 0);
      else
        ae.set(this.y, 0);
      const [ue, de, fe, he] = minmax2(ee, ae);
      this._bounds_rect = { x0: ue, x1: de, y0: fe, y1: he };
    }
    has_finished() {
      return super.has_finished() && this._images_rendered;
    }
    _map_data() {
      const H = () => this.w.map((q) => q ?? NaN), V = () => this.h.map((q) => q ?? NaN);
      this._define_or_inherit_attr("sw", () => this.model.properties.w.units == "data" ? this.inherited_x && this.inherited_w ? inherit : this.sdist(this.renderer.xscale, this.x, H(), "edge", this.model.dilate) : this.inherited_w ? inherit : to_screen(H())), this._define_or_inherit_attr("sh", () => this.model.properties.h.units == "data" ? this.inherited_y && this.inherited_h ? inherit : this.sdist(this.renderer.yscale, this.y, V(), "edge", this.model.dilate) : this.inherited_h ? inherit : to_screen(V()));
    }
    _render(H, V, q) {
      const { sx: G, sy: J, sw: Z, sh: te, angle: ne, global_alpha: ee } = { ...this, ...q }, { image: ae, loaders: oe, resolved: se } = this, { frame: ce } = this.renderer.plot_view, { left: le, top: ue, width: de, height: fe } = ce.bbox;
      H.beginPath(), H.rect(le + 1, ue + 1, de - 2, fe - 2), H.clip();
      let he = !0;
      for (const me of V) {
        const pe = oe[me];
        if (!isFinite(G[me] + J[me] + ne.get(me) + ee.get(me)))
          continue;
        se.get(me) || (pe != null && pe.image.complete ? (ae[me] = pe.image, oe[me] = null, se.set(me)) : he = !1);
        const ye = ae[me];
        ye != null && (ye.naturalWidth == 0 && ye.naturalHeight == 0 || this._render_image(H, me, ye, G, J, Z, te, ne, ee));
      }
      he && !this._images_rendered && (this._images_rendered = !0, this.notify_finished());
    }
    _render_image(H, V, q, G, J, Z, te, ne, ee) {
      isFinite(Z[V]) || (Z[V] = q.width), isFinite(te[V]) || (te[V] = q.height);
      const ae = Z[V], oe = te[V], { anchor: se } = this, ce = se.x * ae, le = se.y * oe, ue = G[V] - ce, de = J[V] - le, fe = ne.get(V), he = ee.get(V);
      H.save(), H.globalAlpha = he;
      const me = ae / 2, pe = oe / 2;
      fe != 0 ? (H.translate(ue, de), H.translate(me, pe), H.rotate(fe), H.translate(-me, -pe), H.drawImage(q, 0, 0, ae, oe), H.translate(me, pe), H.rotate(-fe), H.translate(-me, -pe), H.translate(-ue, -de)) : H.drawImage(q, ue, de, ae, oe), H.restore();
    }
    bounds() {
      return this._bounds_rect;
    }
  }
  mt(ImageURLView, "__name__", "ImageURLView");
  const sm = class sm extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(sm, "__name__", "ImageURL"), sm.prototype.default_view = ImageURLView, sm.define(({ Bool: _, Int: H }) => ({
    url: [StringSpec, { field: "url" }],
    anchor: [Anchor$2, "top_left"],
    global_alpha: [NumberSpec, { value: 1 }],
    angle: [AngleSpec, 0],
    w: [NullDistanceSpec, null],
    h: [NullDistanceSpec, null],
    dilate: [_, !1],
    retry_attempts: [H, 0],
    retry_timeout: [H, 0]
  }));
  let ImageURL = sm;
  class TextAnchorSpec extends DataSpec {
  }
  mt(TextAnchorSpec, "__name__", "TextAnchorSpec");
  var uy;
  let TextView$1 = (uy = class extends XYGlyphView {
    async _build_labels() {
      const { text: _ } = this.base ?? this;
      this.labels = Array.from(_, (H) => {
        if (H == null)
          return null;
        {
          const V = `${H}`;
          return new TextBox({ text: V });
        }
      });
    }
    async _set_lazy_data() {
      await this._build_labels();
    }
    after_visuals() {
      super.after_visuals();
      const _ = this.data_size, { anchor: H } = this.base ?? this, { padding: V, border_radius: q } = this.model, { text_align: G, text_baseline: J } = this.visuals.text;
      if (H.is_Scalar() && H.value != "auto")
        this.anchor_ = new UniformScalar(anchor(H.value), _);
      else if (H.is_Scalar() && G.is_Scalar() && J.is_Scalar())
        this.anchor_ = new UniformScalar(text_anchor(H.value, G.value, J.value), _);
      else {
        const ae = new Array(_);
        for (let oe = 0; oe < _; oe++) {
          const se = H.get(oe), ce = G.get(oe), le = J.get(oe);
          ae[oe] = text_anchor(se, ce, le);
        }
        this.anchor_ = new UniformVector(ae);
      }
      this.padding = padding$2(V), this.border_radius = border_radius(q), this.swidth = new Float32Array(_), this.sheight = new Float32Array(_);
      const { left: Z, right: te, top: ne, bottom: ee } = this.padding;
      for (const [ae, oe] of enumerate(this.labels)) {
        if (ae == null)
          continue;
        ae.visuals = this.visuals.text.values(oe), ae.position = { sx: 0, sy: 0, x_anchor: "left", y_anchor: "top" }, ae.align = "auto";
        const se = ae.size(), ce = Z + se.width + te, le = ne + se.height + ee;
        this.swidth[oe] = ce, this.sheight[oe] = le;
      }
    }
    _render(_, H, V) {
      const { sx: q, sy: G, x_offset: J, y_offset: Z, angle: te } = { ...this, ...V }, { text: ne, background_fill: ee, background_hatch: ae, border_line: oe } = this.visuals, { anchor_: se, border_radius: ce, padding: le } = this, { labels: ue, swidth: de, sheight: fe } = this;
      for (const he of H) {
        const me = q[he] + J.get(he), pe = G[he] + Z.get(he), ye = te.get(he), ge = ue[he];
        if (!isFinite(me + pe + ye) || ge == null)
          continue;
        const ve = de[he], be = fe[he], Ce = se.get(he), Se = Ce.x * ve, we = Ce.y * be;
        if (_.translate(me, pe), _.rotate(ye), _.translate(-Se, -we), ee.v_doit(he) || ae.v_doit(he) || oe.v_doit(he)) {
          _.beginPath();
          const Ee = new BBox$1({ x: 0, y: 0, width: ve, height: be });
          round_rect(_, Ee, ce), ee.apply(_, he), ae.apply(_, he), oe.apply(_, he);
        }
        if (ne.v_doit(he)) {
          const { left: Ee, top: xe } = le;
          _.translate(Ee, xe), ge.paint(_), _.translate(-Ee, -xe);
        }
        _.translate(Se, we), _.rotate(-ye), _.translate(-me, -pe);
      }
    }
    _hit_point(_) {
      const H = { x: _.sx, y: _.sy }, { sx: V, sy: q, x_offset: G, y_offset: J, angle: Z, labels: te } = this, { anchor_: ne } = this, { swidth: ee, sheight: ae } = this, oe = this.data_size, se = [];
      for (let ce = 0; ce < oe; ce++) {
        const le = V[ce] + G.get(ce), ue = q[ce] + J.get(ce), de = Z.get(ce), fe = te[ce];
        if (!isFinite(le + ue + de) || fe == null)
          continue;
        const he = ee[ce], me = ae[ce], pe = ne.get(ce), ye = pe.x * he, ge = pe.y * me, { x: ve, y: be } = rotate_around(H, { x: le, y: ue }, -de), Ce = le - ye, Se = ue - ge, we = Ce + he, Ee = Se + me;
        Ce <= ve && ve <= we && Se <= be && be <= Ee && se.push(ce);
      }
      return new Selection({ indices: se });
    }
    rect_i(_) {
      const { sx: H, sy: V, x_offset: q, y_offset: G, angle: J, labels: Z } = this, { anchor_: te } = this, { swidth: ne, sheight: ee } = this, ae = H[_] + q.get(_), oe = V[_] + G.get(_), se = J.get(_), ce = Z[_];
      if (!isFinite(ae + oe + se) || ce == null)
        return {
          p0: { x: NaN, y: NaN },
          p1: { x: NaN, y: NaN },
          p2: { x: NaN, y: NaN },
          p3: { x: NaN, y: NaN }
        };
      const le = ne[_], ue = ee[_], de = te.get(_), fe = de.x * le, he = de.y * ue, me = new BBox$1({
        x: ae - fe,
        y: oe - he,
        width: le,
        height: ue
      }), { rect: pe } = me;
      if (se == 0)
        return pe;
      {
        const ye = new AffineTransform();
        return ye.rotate_around(ae, oe, se), ye.apply_rect(pe);
      }
    }
    scenterxy(_) {
      const { p0: H, p1: V, p2: q, p3: G } = this.rect_i(_), J = (H.x + V.x + q.x + G.x) / 4, Z = (H.y + V.y + q.y + G.y) / 4;
      return [J, Z];
    }
  }, mt(uy, "__name__", "TextView"), uy);
  var Wh;
  let Text$1 = (Wh = class extends XYGlyph {
    constructor(_) {
      super(_);
    }
  }, mt(Wh, "__name__", "Text"), Wh.prototype.default_view = TextView$1, Wh.mixins([
    TextVector$1,
    ["border_", LineVector$1],
    ["background_", FillVector$1],
    ["background_", HatchVector$1]
  ]), Wh.define(() => ({
    text: [NullStringSpec, { field: "text" }],
    angle: [AngleSpec, 0],
    x_offset: [NumberSpec, 0],
    y_offset: [NumberSpec, 0],
    anchor: [TextAnchorSpec, { value: "auto" }],
    padding: [Padding$2, 0],
    border_radius: [BorderRadius, 0]
  })), Wh.override({
    border_line_color: null,
    background_fill_color: null,
    background_hatch_color: null
  }), Wh);
  class MathTextGlyphView extends TextView$1 {
    constructor() {
      super(...arguments);
      mt(this, "_label_views", /* @__PURE__ */ new Map());
    }
    remove() {
      remove_views(this._label_views), super.remove();
    }
    *children() {
      yield* super.children(), yield* this._label_views.values();
    }
    has_finished() {
      if (!super.has_finished())
        return !1;
      for (const H of this._label_views.values())
        if (!H.has_finished())
          return !1;
      return !0;
    }
    async _build_labels() {
      const { text: H } = this.base ?? this, V = Array.from(H, (q) => q == null ? null : this._build_label(q));
      await build_views(this._label_views, V.filter(non_null), { parent: this.renderer }), this.labels = V.map((q) => q == null ? null : this._label_views.get(q).graphics());
    }
    async after_lazy_visuals() {
      await super.after_lazy_visuals();
      const H = [...this._label_views.values()].map((Z) => Z instanceof MathTextView ? Z.request_image() : null);
      await Promise.allSettled(H);
      const { left: V, right: q, top: G, bottom: J } = this.padding;
      for (const [Z, te] of enumerate(this.labels)) {
        if (Z == null || !(Z instanceof MathTextView))
          continue;
        const ne = Z.size(), ee = V + ne.width + q, ae = G + ne.height + J;
        this.swidth[te] = ee, this.sheight[te] = ae;
      }
    }
  }
  mt(MathTextGlyphView, "__name__", "MathTextGlyphView");
  class MathTextGlyph extends Text$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(MathTextGlyph, "__name__", "MathTextGlyph");
  class MathMLGlyphView extends MathTextGlyphView {
    _build_label(_) {
      return new MathML({ text: _ });
    }
  }
  mt(MathMLGlyphView, "__name__", "MathMLGlyphView");
  const z5 = class z5 extends MathTextGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(z5, "__name__", "MathMLGlyph"), z5.prototype.default_view = MathMLGlyphView;
  let MathMLGlyph = z5;
  class MultiLineView extends GlyphView {
    async load_glglyph() {
      const { MultiLineGL: _ } = await Promise.resolve().then(() => multi_line);
      return _;
    }
    _project_data() {
      this._project_xy("xs", this.xs.data, "ys", this.ys.data);
    }
    _index_data(_) {
      const { data_size: H } = this;
      for (let V = 0; V < H; V++) {
        const q = this.xs.get(V), G = this.ys.get(V), [J, Z, te, ne] = minmax2(q, G);
        _.add_rect(J, te, Z, ne);
      }
    }
    _render(_, H, V) {
      const { sxs: q, sys: G } = { ...this, ...V };
      for (const J of H) {
        const Z = q.get(J), te = G.get(J), ne = Math.min(Z.length, te.length);
        let ee = !0;
        _.beginPath();
        for (let ae = 0; ae < ne; ae++) {
          const oe = Z[ae], se = te[ae];
          isFinite(oe + se) ? ee ? (_.moveTo(oe, se), ee = !1) : _.lineTo(oe, se) : ee = !0;
        }
        this.visuals.line.set_vectorize(_, J), _.stroke();
      }
    }
    _hit_point(_) {
      const H = { x: _.sx, y: _.sy };
      let V = 9999;
      const q = /* @__PURE__ */ new Map();
      for (let G = 0, J = this.sxs.length; G < J; G++) {
        const Z = Math.max(2, this.line_width.get(G) / 2), te = this.sxs.get(G), ne = this.sys.get(G);
        let ee = null;
        for (let ae = 0, oe = te.length - 1; ae < oe; ae++) {
          const se = { x: te[ae], y: ne[ae] }, ce = { x: te[ae + 1], y: ne[ae + 1] }, le = dist_to_segment(H, se, ce);
          le < Z && le < V && (V = le, ee = [ae]);
        }
        ee != null && q.set(G, ee);
      }
      return new Selection({
        indices: [...q.keys()],
        multiline_indices: q
      });
    }
    _hit_span(_) {
      const { sx: H, sy: V } = _;
      let q, G;
      _.direction == "v" ? (q = this.renderer.yscale.invert(V), G = this.ys) : (q = this.renderer.xscale.invert(H), G = this.xs);
      const J = /* @__PURE__ */ new Map();
      for (let Z = 0, te = G.length; Z < te; Z++) {
        const ne = G.get(Z), ee = [];
        for (let ae = 0, oe = ne.length - 1; ae < oe; ae++)
          ne[ae] <= q && q <= ne[ae + 1] && ee.push(ae);
        ee.length > 0 && J.set(Z, ee);
      }
      return new Selection({
        indices: [...J.keys()],
        multiline_indices: J
      });
    }
    get_interpolation_hit(_, H, V) {
      const q = this.xs.get(_), G = this.ys.get(_), J = q[H], Z = G[H], te = q[H + 1], ne = G[H + 1];
      return line_interpolation(this.renderer, V, J, Z, te, ne);
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
    scenterxy() {
      throw new Error(`${this}.scenterxy() is not implemented`);
    }
  }
  mt(MultiLineView, "__name__", "MultiLineView");
  const Rp = class Rp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Rp, "__name__", "MultiLine"), Rp.prototype.default_view = MultiLineView, Rp.define(({}) => ({
    xs: [XCoordinateSeqSpec, { field: "xs" }],
    ys: [YCoordinateSeqSpec, { field: "ys" }]
  })), Rp.mixins(LineVector$1);
  let MultiLine = Rp;
  class MultiPolygonsView extends GlyphView {
    constructor() {
      super(...arguments);
      mt(this, "_hole_index");
    }
    _project_data() {
    }
    _index_data(H) {
      const { min: V, max: q } = Math, { data_size: G } = this;
      for (let J = 0; J < G; J++) {
        const Z = this.xs[J], te = this.ys[J];
        if (Z.length == 0 || te.length == 0) {
          H.add_empty();
          continue;
        }
        let ne = 1 / 0, ee = -1 / 0, ae = 1 / 0, oe = -1 / 0;
        for (let se = 0, ce = Z.length; se < ce; se++) {
          const le = Z[se][0], ue = te[se][0];
          if (le.length != 0 && ue.length != 0) {
            const [de, fe, he, me] = minmax2(le, ue);
            ne = V(ne, de), ee = q(ee, fe), ae = V(ae, he), oe = q(oe, me);
          }
        }
        H.add_rect(ne, ae, ee, oe);
      }
      this._hole_index = this._index_hole_data();
    }
    _index_hole_data() {
      const { min: H, max: V } = Math, { data_size: q } = this, G = new SpatialIndex(q);
      for (let J = 0; J < q; J++) {
        const Z = this.xs[J], te = this.ys[J];
        if (Z.length == 0 || te.length == 0) {
          G.add_empty();
          continue;
        }
        let ne = 1 / 0, ee = -1 / 0, ae = 1 / 0, oe = -1 / 0;
        for (let se = 0, ce = Z.length; se < ce; se++) {
          const le = Z[se], ue = te[se];
          if (le.length > 1 && ue.length > 1)
            for (let de = 1, fe = le.length; de < fe; de++) {
              const [he, me, pe, ye] = minmax2(le[de], ue[de]);
              ne = H(ne, he), ee = V(ee, me), ae = H(ae, pe), oe = V(oe, ye);
            }
        }
        G.add_rect(ne, ae, ee, oe);
      }
      return G.finish(), G;
    }
    _mask_data() {
      const { x_range: H, y_range: V } = this.renderer.plot_view.frame;
      return this.index.indices({
        x0: H.min,
        x1: H.max,
        y0: V.min,
        y1: V.max
      });
    }
    _render(H, V, q) {
      if (!this.visuals.fill.doit && !this.visuals.line.doit)
        return;
      const { sxs: G, sys: J } = { ...this, ...q };
      for (const Z of V) {
        H.beginPath();
        const te = G[Z], ne = J[Z], ee = Math.min(te.length, ne.length);
        for (let ae = 0; ae < ee; ae++) {
          const oe = te[ae], se = ne[ae], ce = Math.min(oe.length, se.length);
          for (let le = 0; le < ce; le++) {
            const ue = oe[le], de = se[le], fe = Math.min(ue.length, de.length);
            for (let he = 0; he < fe; he++) {
              const me = ue[he], pe = de[he];
              he == 0 ? H.moveTo(me, pe) : H.lineTo(me, pe);
            }
            H.closePath();
          }
        }
        this.visuals.fill.apply(H, Z, "evenodd"), this.visuals.hatch.apply(H, Z, "evenodd"), this.visuals.line.apply(H, Z);
      }
    }
    _hit_poly(H) {
      const { sx: V, sy: q, greedy: G = !1 } = H, J = (() => {
        const te = this.renderer.xscale.v_invert(V), ne = this.renderer.yscale.v_invert(q), [ee, ae, oe, se] = minmax2(te, ne);
        return this.index.indices({ x0: ee, x1: ae, y0: oe, y1: se });
      })(), Z = [];
      for (const te of J) {
        const ne = this.sxs[te], ee = this.sys[te];
        let ae = !G;
        const oe = ne.length;
        for (let se = 0; se < oe; se++) {
          const ce = ne[se][0], le = ee[se][0], ue = ce.length;
          for (let de = 0; de < ue; de++) {
            const fe = ce[de], he = le[de];
            if (point_in_poly(fe, he, V, q)) {
              if (G) {
                ae = !0;
                break;
              }
            } else if (!G) {
              ae = !1;
              break;
            }
          }
          if (G) {
            if (ae)
              break;
          } else if (!ae)
            break;
        }
        ae && Z.push(te);
      }
      return new Selection({ indices: Z });
    }
    _hit_rect(H) {
      const { sx0: V, sx1: q, sy0: G, sy1: J, greedy: Z } = H, te = [V, q, q, V], ne = [G, G, J, J];
      return this._hit_poly({ type: "poly", sx: te, sy: ne, greedy: Z });
    }
    _hit_point(H) {
      const { sx: V, sy: q } = H, G = this.renderer.xscale.invert(V), J = this.renderer.yscale.invert(q), Z = this.index.indices({ x0: G, y0: J, x1: G, y1: J }), te = this._hole_index.indices({ x0: G, y0: J, x1: G, y1: J }), ne = [];
      for (const ee of Z) {
        const ae = this.sxs[ee], oe = this.sys[ee];
        for (let se = 0, ce = ae.length; se < ce; se++) {
          const le = ae[se].length;
          if (point_in_poly(V, q, ae[se][0], oe[se][0])) {
            if (le == 1)
              ne.push(ee);
            else if (!te.get(ee))
              ne.push(ee);
            else if (le > 1) {
              let ue = !1;
              for (let de = 1; de < le; de++) {
                const fe = ae[se][de], he = oe[se][de];
                if (point_in_poly(V, q, fe, he)) {
                  ue = !0;
                  break;
                } else
                  continue;
              }
              ue || ne.push(ee);
            }
          }
        }
      }
      return new Selection({ indices: ne });
    }
    _get_snap_coord(H) {
      return sum$3(H) / H.length;
    }
    scenterxy(H, V, q) {
      if (this.sxs[H].length == 1) {
        const G = this._get_snap_coord(this.sxs[H][0][0]), J = this._get_snap_coord(this.sys[H][0][0]);
        return [G, J];
      } else {
        const G = this.sxs[H], J = this.sys[H];
        for (let Z = 0, te = G.length; Z < te; Z++)
          if (point_in_poly(V, q, G[Z][0], J[Z][0])) {
            const ne = this._get_snap_coord(G[Z][0]), ee = this._get_snap_coord(J[Z][0]);
            return [ne, ee];
          }
      }
      unreachable();
    }
    map_data() {
      if (this.inherited_xs && this.inherited_ys)
        this._inherit_attr("sxs"), this._inherit_attr("sys");
      else {
        const { xs: H, ys: V } = this, q = H.length, G = new Array(q), J = new Array(q);
        for (let Z = 0; Z < q; Z++) {
          const te = H[Z].length;
          G[Z] = new Array(te), J[Z] = new Array(te);
          for (let ne = 0; ne < te; ne++) {
            const ee = H[Z][ne].length;
            G[Z][ne] = new Array(ee), J[Z][ne] = new Array(ee);
            for (let ae = 0; ae < ee; ae++) {
              const [oe, se] = this.renderer.coordinates.map_to_screen(H[Z][ne][ae], V[Z][ne][ae]);
              G[Z][ne][ae] = oe, J[Z][ne][ae] = se;
            }
          }
        }
        this._define_attr("sxs", G), this._define_attr("sys", J);
      }
    }
    draw_legend_for_index(H, V, q) {
      generic_area_vector_legend(this.visuals, H, V, q);
    }
  }
  mt(MultiPolygonsView, "__name__", "MultiPolygonsView");
  const Op = class Op extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Op, "__name__", "MultiPolygons"), Op.prototype.default_view = MultiPolygonsView, Op.define(({}) => ({
    xs: [XCoordinateSeqSeqSeqSpec, { field: "xs" }],
    ys: [YCoordinateSeqSeqSeqSpec, { field: "ys" }]
  })), Op.mixins([LineVector$1, FillVector$1, HatchVector$1]);
  let MultiPolygons = Op;
  class PatchesView extends GlyphView {
    _project_data() {
      this._project_xy("xs", this.xs.data, "ys", this.ys.data);
    }
    _index_data(_) {
      const { data_size: H } = this;
      for (let V = 0; V < H; V++) {
        const q = this.xs.get(V), G = this.ys.get(V), [J, Z, te, ne] = minmax2(q, G);
        _.add_rect(J, te, Z, ne);
      }
    }
    _mask_data() {
      const { x_range: _, y_range: H } = this.renderer.plot_view.frame;
      return this.index.indices({
        x0: _.min,
        x1: _.max,
        y0: H.min,
        y1: H.max
      });
    }
    _render(_, H, V) {
      const { sxs: q, sys: G } = { ...this, ...V };
      for (const J of H) {
        const Z = q.get(J), te = G.get(J);
        let ne = !0;
        _.beginPath();
        const ee = Math.min(Z.length, te.length);
        for (let ae = 0; ae < ee; ae++) {
          const oe = Z[ae], se = te[ae];
          isFinite(oe + se) ? ne ? (_.moveTo(oe, se), ne = !1) : _.lineTo(oe, se) : (_.closePath(), ne = !0);
        }
        _.closePath(), this.visuals.fill.apply(_, J), this.visuals.hatch.apply(_, J), this.visuals.line.apply(_, J);
      }
    }
    _hit_poly(_) {
      const { sx: H, sy: V, greedy: q = !1 } = _, G = (() => {
        const Z = this.renderer.xscale.v_invert(H), te = this.renderer.yscale.v_invert(V), [ne, ee, ae, oe] = minmax2(Z, te);
        return this.index.indices({ x0: ne, x1: ee, y0: ae, y1: oe });
      })(), J = [];
      for (const Z of G) {
        const te = this.sxs.get(Z), ne = this.sys.get(Z), ee = te.length;
        if (ee == 0)
          continue;
        let ae = !q;
        for (let oe = 0; oe < ee; oe++) {
          const se = te[oe], ce = ne[oe];
          if (point_in_poly(se, ce, H, V)) {
            if (q) {
              ae = !0;
              break;
            }
          } else if (!q) {
            ae = !1;
            break;
          }
        }
        ae && J.push(Z);
      }
      return new Selection({ indices: J });
    }
    _hit_rect(_) {
      const { sx0: H, sx1: V, sy0: q, sy1: G, greedy: J } = _, Z = [H, V, V, H], te = [q, q, G, G];
      return this._hit_poly({ type: "poly", sx: Z, sy: te, greedy: J });
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = this.renderer.xscale.invert(H), G = this.renderer.yscale.invert(V), J = this.index.indices({ x0: q, y0: G, x1: q, y1: G }), Z = [];
      for (const te of J) {
        const ne = this.sxs.get(te), ee = this.sys.get(te), ae = ne.length;
        for (let oe = 0, se = 0; ; se++) {
          if (isNaN(ne[se]) || se == ae) {
            const ce = ne.subarray(oe, se), le = ee.subarray(oe, se);
            if (point_in_poly(H, V, ce, le)) {
              Z.push(te);
              break;
            }
            oe = se + 1;
          }
          if (se == ae)
            break;
        }
      }
      return new Selection({ indices: Z });
    }
    _get_snap_coord(_) {
      return sum$3(_) / _.length;
    }
    scenterxy(_, H, V) {
      const q = this.sxs.get(_), G = this.sys.get(_), J = q.length;
      let Z = !1;
      for (let te = 0, ne = 0; ; ne++) {
        const ee = isNaN(q[ne]);
        if (Z = Z || ee, ne == J && !Z) {
          const ae = this._get_snap_coord(q), oe = this._get_snap_coord(G);
          return [ae, oe];
        }
        if (ee || ne == J) {
          const ae = q.subarray(te, ne), oe = G.subarray(te, ne);
          if (point_in_poly(H, V, ae, oe)) {
            const se = this._get_snap_coord(ae), ce = this._get_snap_coord(oe);
            return [se, ce];
          }
          te = ne + 1;
        }
        if (ne == J)
          break;
      }
      unreachable();
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(PatchesView, "__name__", "PatchesView");
  const Ip = class Ip extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Ip, "__name__", "Patches"), Ip.prototype.default_view = PatchesView, Ip.define(({}) => ({
    xs: [XCoordinateSeqSpec, { field: "xs" }],
    ys: [YCoordinateSeqSpec, { field: "ys" }]
  })), Ip.mixins([LineVector$1, FillVector$1, HatchVector$1]);
  let Patches = Ip;
  class QuadView extends LRTBView {
    scenterxy(_) {
      const H = this.sleft[_] / 2 + this.sright[_] / 2, V = this.stop[_] / 2 + this.sbottom[_] / 2;
      return [H, V];
    }
    _lrtb(_) {
      const H = this.left[_], V = this.right[_], q = this.top[_], G = this.bottom[_];
      return { l: H, r: V, t: q, b: G };
    }
  }
  mt(QuadView, "__name__", "QuadView");
  var N2;
  let Quad$1 = (N2 = class extends LRTB {
    constructor(_) {
      super(_);
    }
  }, mt(N2, "__name__", "Quad"), N2.prototype.default_view = QuadView, N2.define(({}) => ({
    right: [XCoordinateSpec, { field: "right" }],
    bottom: [YCoordinateSpec, { field: "bottom" }],
    left: [XCoordinateSpec, { field: "left" }],
    top: [YCoordinateSpec, { field: "top" }]
  })), N2);
  class QuadraticView extends GlyphView {
    _project_data() {
      this._project_xy("x0", this.x0, "y0", this.y0), this._project_xy("x1", this.x1, "y1", this.y1);
    }
    _index_data(_) {
      const { x0: H, x1: V, y0: q, y1: G, cx: J, cy: Z, data_size: te } = this;
      for (let ne = 0; ne < te; ne++) {
        const ee = H[ne], ae = V[ne], oe = q[ne], se = G[ne], ce = J[ne], le = Z[ne];
        if (!isFinite(ee + ae + oe + se + ce + le))
          _.add_empty();
        else {
          const { x0: ue, y0: de, x1: fe, y1: he } = qbb(ee, oe, ce, le, ae, se);
          _.add_rect(ue, de, fe, he);
        }
      }
    }
    _render(_, H, V) {
      if (!this.visuals.line.doit)
        return;
      const { sx0: q, sy0: G, sx1: J, sy1: Z, scx: te, scy: ne } = { ...this, ...V };
      for (const ee of H) {
        const ae = q[ee], oe = G[ee], se = J[ee], ce = Z[ee], le = te[ee], ue = ne[ee];
        isFinite(ae + oe + se + ce + le + ue) && (_.beginPath(), _.moveTo(ae, oe), _.quadraticCurveTo(le, ue, se, ce), this.visuals.line.apply(_, ee));
      }
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
    scenterxy() {
      throw new Error(`${this}.scenterxy() is not implemented`);
    }
  }
  mt(QuadraticView, "__name__", "QuadraticView");
  const Dp = class Dp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Dp, "__name__", "Quadratic"), Dp.prototype.default_view = QuadraticView, Dp.define(({}) => ({
    x0: [XCoordinateSpec, { field: "x0" }],
    y0: [YCoordinateSpec, { field: "y0" }],
    x1: [XCoordinateSpec, { field: "x1" }],
    y1: [YCoordinateSpec, { field: "y1" }],
    cx: [XCoordinateSpec, { field: "cx" }],
    cy: [YCoordinateSpec, { field: "cy" }]
  })), Dp.mixins(LineVector$1);
  let Quadratic = Dp;
  class RayView extends XYGlyphView {
    _map_data() {
      if (this._define_or_inherit_attr("slength", () => this.model.properties.length.units == "data" ? this.inherited_x && this.inherited_length ? inherit : this.sdist(this.renderer.xscale, this.x, this.length) : this.inherited_length ? inherit : to_screen(this.length)), !this.inherited_slength) {
        const { width: _, height: H } = this.renderer.plot_view.frame.bbox, V = 2 * (_ + H), { slength: q } = this, G = q.length;
        for (let J = 0; J < G; J++)
          q[J] == 0 && (q[J] = V);
      }
    }
    _render(_, H, V) {
      if (!this.visuals.line.doit)
        return;
      const { sx: q, sy: G, slength: J, angle: Z } = { ...this, ...V };
      for (const te of H) {
        const ne = q[te], ee = G[te], ae = Z.get(te), oe = J[te];
        isFinite(ne + ee + ae + oe) && (_.translate(ne, ee), _.rotate(ae), _.beginPath(), _.moveTo(0, 0), _.lineTo(oe, 0), this.visuals.line.apply(_, te), _.rotate(-ae), _.translate(-ne, -ee));
      }
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(RayView, "__name__", "RayView");
  const Np = class Np extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Np, "__name__", "Ray"), Np.prototype.default_view = RayView, Np.mixins(LineVector$1), Np.define(({}) => ({
    length: [DistanceSpec, 0],
    angle: [AngleSpec, 0]
  }));
  let Ray = Np;
  const { abs: abs$5, sqrt: sqrt$4 } = Math;
  class RectView extends CenterRotatableView {
    async load_glglyph() {
      const { RectGL: _ } = await Promise.resolve().then(() => rect$2);
      return _;
    }
    _set_data(_) {
      super._set_data(_), this.border_radius = border_radius(this.model.border_radius);
    }
    _map_data() {
      const _ = this.data_size;
      if (this.inherited_x && this.inherited_width)
        this._inherit_attr("swidth"), this._inherit_attr("sx0");
      else {
        let H, V;
        if (this.model.properties.width.units == "data")
          [H, V] = this._map_dist_corner_for_data_side_length(this.x, this.width, this.renderer.xscale);
        else {
          H = to_screen(this.width), V = new ScreenArray(_);
          const { sx: q } = this;
          for (let G = 0; G < _; G++)
            V[G] = q[G] - H[G] / 2;
        }
        this._define_attr("swidth", H), this._define_attr("sx0", V);
      }
      if (this.inherited_y && this.inherited_height)
        this._inherit_attr("sheight"), this._inherit_attr("sy1");
      else {
        let H, V;
        if (this.model.properties.height.units == "data")
          [H, V] = this._map_dist_corner_for_data_side_length(this.y, this.height, this.renderer.yscale);
        else {
          H = to_screen(this.height), V = new ScreenArray(_);
          const { sy: q } = this;
          for (let G = 0; G < _; G++)
            V[G] = q[G] - H[G] / 2;
        }
        this._define_attr("sheight", H), this._define_attr("sy1", V);
      }
      if (this.inherited_swidth && this.inherited_sheight)
        this._inherit_attr("max_x2_ddist"), this._inherit_attr("max_y2_ddist");
      else {
        const { sx0: H, sy1: V, swidth: q, sheight: G } = this, J = new ScreenArray(_);
        for (let ae = 0; ae < _; ae++) {
          const oe = q[ae], se = G[ae];
          J[ae] = sqrt$4(oe ** 2 + se ** 2) / 2;
        }
        const Z = new ScreenArray(_), te = new ScreenArray(_);
        for (let ae = 0; ae < _; ae++)
          Z[ae] = H[ae] + q[ae] / 2, te[ae] = V[ae] + G[ae] / 2;
        const ne = max$j(this._ddist(0, Z, J)), ee = max$j(this._ddist(1, te, J));
        this._define_attr("max_x2_ddist", ne), this._define_attr("max_y2_ddist", ee);
      }
    }
    _render(_, H, V) {
      const { sx: q, sy: G, sx0: J, sy1: Z, swidth: te, sheight: ne, angle: ee, border_radius: ae } = { ...this, ...V };
      for (const oe of H) {
        const se = q[oe], ce = G[oe], le = J[oe], ue = Z[oe], de = te[oe], fe = ne[oe], he = ee.get(oe);
        if (isFinite(se + ce + le + ue + de + fe + he) && !(de == 0 || fe == 0)) {
          if (_.beginPath(), he != 0) {
            _.translate(se, ce), _.rotate(he);
            const me = new BBox$1({ x: -de / 2, y: -fe / 2, width: de, height: fe });
            round_rect(_, me, ae), _.rotate(-he), _.translate(-se, -ce);
          } else {
            const me = new BBox$1({ x: le, y: ue, width: de, height: fe });
            round_rect(_, me, ae);
          }
          this.visuals.fill.apply(_, oe), this.visuals.hatch.apply(_, oe), this.visuals.line.apply(_, oe);
        }
      }
    }
    _hit_rect(_) {
      return this._hit_rect_against_index(_);
    }
    _hit_point(_) {
      const H = { x: _.sx, y: _.sy }, V = this.renderer.xscale.invert(H.x), q = this.renderer.yscale.invert(H.y), G = this.index.indices({
        x0: V - this.max_x2_ddist,
        x1: V + this.max_x2_ddist,
        y0: q - this.max_y2_ddist,
        y1: q + this.max_y2_ddist
      }), { sx: J, sy: Z, sx0: te, sy1: ne, swidth: ee, sheight: ae, angle: oe } = this, se = [];
      for (const ce of G) {
        const le = J[ce], ue = Z[ce], de = te[ce], fe = ne[ce], he = ee[ce], me = ae[ce], pe = oe.get(ce), ye = rotate_around(H, { x: le, y: ue }, -pe), ge = ye.x - de, ve = ye.y - fe;
        0 <= ge && ge <= he && 0 <= ve && ve <= me && se.push(ce);
      }
      return new Selection({ indices: se });
    }
    _map_dist_corner_for_data_side_length(_, H, V) {
      const q = _.length, G = new Float64Array(q), J = new Float64Array(q);
      for (let ae = 0; ae < q; ae++) {
        const oe = _[ae], se = H.get(ae) / 2;
        G[ae] = oe - se, J[ae] = oe + se;
      }
      const Z = V.v_compute(G), te = V.v_compute(J), ne = this.sdist(V, G, H, "edge", this.model.dilate);
      let ee = Z;
      for (let ae = 0; ae < q; ae++) {
        const oe = Z[ae], se = te[ae];
        if (!isNaN(oe + se) && oe != se) {
          ee = oe < se ? Z : te;
          break;
        }
      }
      return [ne, ee];
    }
    _ddist(_, H, V) {
      const q = infer_type(H, V), G = _ == 0 ? this.renderer.xscale : this.renderer.yscale, J = H, Z = J.length, te = new q(Z);
      for (let se = 0; se < Z; se++)
        te[se] = J[se] + V[se];
      const ne = G.v_invert(J), ee = G.v_invert(te), ae = ne.length, oe = new q(ae);
      for (let se = 0; se < ae; se++)
        oe[se] = abs$5(ee[se] - ne[se]);
      return oe;
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(RectView, "__name__", "RectView");
  const lm = class lm extends CenterRotatable {
    constructor(_) {
      super(_);
    }
  };
  mt(lm, "__name__", "Rect"), lm.prototype.default_view = RectView, lm.define(({ Bool: _ }) => ({
    border_radius: [BorderRadius, 0],
    dilate: [_, !1]
  }));
  let Rect = lm;
  class MarkerView extends XYGlyphView {
    constructor() {
      super(...arguments);
      mt(this, "_render_one");
    }
    _render(H, V, q) {
      const { sx: G, sy: J, size: Z, angle: te } = { ...this, ...q };
      for (const ne of V) {
        const ee = G[ne], ae = J[ne], oe = Z.get(ne), se = te.get(ne);
        if (!isFinite(ee + ae + oe + se))
          continue;
        const ce = oe / 2;
        H.beginPath(), H.translate(ee, ae), se != 0 && H.rotate(se), this._render_one(H, ne, ce, this.visuals), se != 0 && H.rotate(-se), H.translate(-ee, -ae);
      }
    }
    _mask_data() {
      const { x_target: H, y_target: V } = this.renderer.plot_view.frame, q = H.widen(this.max_size).map((J) => this.renderer.xscale.invert(J)), G = V.widen(this.max_size).map((J) => this.renderer.yscale.invert(J));
      return this.index.indices({
        x0: q.start,
        x1: q.end,
        y0: G.start,
        y1: G.end
      });
    }
    _hit_point(H) {
      const { sx: V, sy: q } = H, { max_size: G } = this, { hit_dilation: J } = this.model, Z = V - G * J, te = V + G * J, [ne, ee] = this.renderer.xscale.r_invert(Z, te), ae = q - G * J, oe = q + G * J, [se, ce] = this.renderer.yscale.r_invert(ae, oe), le = this.index.indices({ x0: ne, x1: ee, y0: se, y1: ce }), ue = [];
      for (const de of le) {
        const fe = this.size.get(de) / 2 * J;
        Math.abs(this.sx[de] - V) <= fe && Math.abs(this.sy[de] - q) <= fe && ue.push(de);
      }
      return new Selection({ indices: ue });
    }
    _hit_span(H) {
      const { sx: V, sy: q } = H, G = this.bounds(), J = this.max_size / 2, [Z, te, ne, ee] = (() => {
        if (H.direction == "h") {
          const { y0: oe, y1: se } = G, ce = V - J, le = V + J, [ue, de] = this.renderer.xscale.r_invert(ce, le);
          return [ue, de, oe, se];
        } else {
          const { x0: oe, x1: se } = G, ce = q - J, le = q + J, [ue, de] = this.renderer.yscale.r_invert(ce, le);
          return [oe, se, ue, de];
        }
      })(), ae = [...this.index.indices({ x0: Z, x1: te, y0: ne, y1: ee })];
      return new Selection({ indices: ae });
    }
    _hit_rect(H) {
      const { sx0: V, sx1: q, sy0: G, sy1: J } = H, [Z, te] = this.renderer.xscale.r_invert(V, q), [ne, ee] = this.renderer.yscale.r_invert(G, J), ae = [...this.index.indices({ x0: Z, x1: te, y0: ne, y1: ee })];
      return new Selection({ indices: ae });
    }
    _hit_poly(H) {
      const { sx: V, sy: q } = H, G = (() => {
        const Z = this.renderer.xscale.v_invert(V), te = this.renderer.yscale.v_invert(q), [ne, ee, ae, oe] = minmax2(Z, te);
        return this.index.indices({ x0: ne, x1: ee, y0: ae, y1: oe });
      })(), J = [];
      for (const Z of G)
        point_in_poly(this.sx[Z], this.sy[Z], V, q) && J.push(Z);
      return new Selection({ indices: J });
    }
    _get_legend_args({ x0: H, x1: V, y0: q, y1: G }, J) {
      const Z = J + 1, te = new Array(Z), ne = new Array(Z);
      te[J] = (H + V) / 2, ne[J] = (q + G) / 2;
      const ee = Math.min(Math.abs(V - H), Math.abs(G - q)) * 0.4, ae = new UniformScalar(ee, Z), oe = new UniformScalar(0, Z);
      return { sx: te, sy: ne, size: ae, angle: oe };
    }
    draw_legend_for_index(H, { x0: V, x1: q, y0: G, y1: J }, Z) {
      const te = this._get_legend_args({ x0: V, x1: q, y0: G, y1: J }, Z);
      this._render(H, [Z], te);
    }
  }
  mt(MarkerView, "__name__", "MarkerView");
  const cm = class cm extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(cm, "__name__", "Marker"), cm.mixins([LineVector$1, FillVector$1, HatchVector$1]), cm.define(({ Float: _ }) => ({
    size: [ScreenSizeSpec, { value: 4 }],
    angle: [AngleSpec, 0],
    hit_dilation: [_, 1]
  }));
  let Marker = cm;
  const SQ3 = Math.sqrt(3), SQ5 = Math.sqrt(5), c36 = (SQ5 + 1) / 4, s36 = Math.sqrt((5 - SQ5) / 8), c72 = (SQ5 - 1) / 4, s72 = Math.sqrt((5 + SQ5) / 8);
  function _one_line(O, _) {
    O.moveTo(-_, 0), O.lineTo(_, 0);
  }
  function _one_x(O, _) {
    O.rotate(Math.PI / 4), _one_cross(O, _), O.rotate(-Math.PI / 4);
  }
  function _one_y(O, _) {
    const H = _ * SQ3, V = H / 3;
    O.moveTo(-H / 2, -V), O.lineTo(0, 0), O.lineTo(H / 2, -V), O.lineTo(0, 0), O.lineTo(0, _);
  }
  function _one_cross(O, _) {
    O.moveTo(0, _), O.lineTo(0, -_), O.moveTo(-_, 0), O.lineTo(_, 0);
  }
  function _one_dot(O, _) {
    O.beginPath(), O.arc(0, 0, _ / 4, 0, 2 * Math.PI, !1), O.closePath();
  }
  function _one_diamond(O, _) {
    O.moveTo(0, _), O.lineTo(_ / 1.5, 0), O.lineTo(0, -_), O.lineTo(-_ / 1.5, 0), O.closePath();
  }
  function _one_hex(O, _) {
    const H = _ / 2, V = SQ3 * H;
    O.moveTo(_, 0), O.lineTo(H, -V), O.lineTo(-H, -V), O.lineTo(-_, 0), O.lineTo(-H, V), O.lineTo(H, V), O.closePath();
  }
  function _one_star(O, _) {
    const H = Math.sqrt(5 - 2 * SQ5) * _;
    O.moveTo(0, -_), O.lineTo(H * c72, -_ + H * s72), O.lineTo(H * (1 + c72), -_ + H * s72), O.lineTo(H * (1 + c72 - c36), -_ + H * (s72 + s36)), O.lineTo(H * (1 + 2 * c72 - c36), -_ + H * (2 * s72 + s36)), O.lineTo(0, -_ + H * 2 * s72), O.lineTo(-H * (1 + 2 * c72 - c36), -_ + H * (2 * s72 + s36)), O.lineTo(-H * (1 + c72 - c36), -_ + H * (s72 + s36)), O.lineTo(-H * (1 + c72), -_ + H * s72), O.lineTo(-H * c72, -_ + H * s72), O.closePath();
  }
  function _one_tri(O, _) {
    const H = _ * SQ3, V = H / 3;
    O.moveTo(-_, V), O.lineTo(_, V), O.lineTo(0, V - H), O.closePath();
  }
  function asterisk(O, _, H, V) {
    _one_cross(O, H), _one_x(O, H), V.line.apply(O, _);
  }
  function circle$3(O, _, H, V) {
    O.arc(0, 0, H, 0, 2 * Math.PI, !1), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function circle_cross(O, _, H, V) {
    O.arc(0, 0, H, 0, 2 * Math.PI, !1), V.fill.apply(O, _), V.hatch.apply(O, _), _one_cross(O, H), V.line.apply(O, _);
  }
  function circle_dot(O, _, H, V) {
    circle$3(O, _, H, V), dot(O, _, H, V);
  }
  function circle_y(O, _, H, V) {
    O.arc(0, 0, H, 0, 2 * Math.PI, !1), V.fill.apply(O, _), V.hatch.apply(O, _), _one_y(O, H), V.line.apply(O, _);
  }
  function circle_x(O, _, H, V) {
    O.arc(0, 0, H, 0, 2 * Math.PI, !1), V.fill.apply(O, _), V.hatch.apply(O, _), _one_x(O, H), V.line.apply(O, _);
  }
  function cross$1(O, _, H, V) {
    _one_cross(O, H), V.line.apply(O, _);
  }
  function diamond(O, _, H, V) {
    _one_diamond(O, H), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function diamond_cross(O, _, H, V) {
    _one_diamond(O, H), V.fill.apply(O, _), V.hatch.apply(O, _), O.moveTo(0, H), O.lineTo(0, -H), O.moveTo(-H / 1.5, 0), O.lineTo(H / 1.5, 0), V.line.apply(O, _);
  }
  function diamond_dot(O, _, H, V) {
    diamond(O, _, H, V), dot(O, _, H, V);
  }
  function dot(O, _, H, V) {
    _one_dot(O, H), V.line.set_vectorize(O, _), O.fillStyle = O.strokeStyle, O.fill();
  }
  function hex$1(O, _, H, V) {
    _one_hex(O, H), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function hex_dot(O, _, H, V) {
    hex$1(O, _, H, V), dot(O, _, H, V);
  }
  function inverted_triangle(O, _, H, V) {
    O.rotate(Math.PI), _one_tri(O, H), O.rotate(-Math.PI), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function plus(O, _, H, V) {
    const q = 3 * H / 8, G = H, J = [q, q, G, G, q, q, -q, -q, -G, -G, -q, -q], Z = [G, q, q, -q, -q, -G, -G, -q, -q, q, q, G];
    O.beginPath();
    for (let te = 0; te < 12; te++)
      O.lineTo(J[te], Z[te]);
    O.closePath(), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function square$1(O, _, H, V) {
    const q = 2 * H;
    O.rect(-H, -H, q, q), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function square_pin(O, _, H, V) {
    const q = 3 * H / 8;
    O.moveTo(-H, -H), O.quadraticCurveTo(0, -q, H, -H), O.quadraticCurveTo(q, 0, H, H), O.quadraticCurveTo(0, q, -H, H), O.quadraticCurveTo(-q, 0, -H, -H), O.closePath(), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function square_cross(O, _, H, V) {
    const q = 2 * H;
    O.rect(-H, -H, q, q), V.fill.apply(O, _), V.hatch.apply(O, _), _one_cross(O, H), V.line.apply(O, _);
  }
  function square_dot(O, _, H, V) {
    square$1(O, _, H, V), dot(O, _, H, V);
  }
  function square_x(O, _, H, V) {
    const q = 2 * H;
    O.rect(-H, -H, q, q), V.fill.apply(O, _), V.hatch.apply(O, _), O.moveTo(-H, H), O.lineTo(H, -H), O.moveTo(-H, -H), O.lineTo(H, H), V.line.apply(O, _);
  }
  function star(O, _, H, V) {
    _one_star(O, H), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function star_dot(O, _, H, V) {
    star(O, _, H, V), dot(O, _, H, V);
  }
  function triangle(O, _, H, V) {
    _one_tri(O, H), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function triangle_dot(O, _, H, V) {
    triangle(O, _, H, V), dot(O, _, H, V);
  }
  function triangle_pin(O, _, H, V) {
    const q = H * SQ3, G = q / 3, J = 3 * G / 8;
    O.moveTo(-H, G), O.quadraticCurveTo(0, J, H, G), O.quadraticCurveTo(SQ3 * J / 2, J / 2, 0, G - q), O.quadraticCurveTo(-SQ3 * J / 2, J / 2, -H, G), O.closePath(), V.fill.apply(O, _), V.hatch.apply(O, _), V.line.apply(O, _);
  }
  function dash(O, _, H, V) {
    _one_line(O, H), V.line.apply(O, _);
  }
  function x$5(O, _, H, V) {
    _one_x(O, H), V.line.apply(O, _);
  }
  function y$4(O, _, H, V) {
    _one_y(O, H), V.line.apply(O, _);
  }
  const marker_funcs = {
    asterisk,
    circle: circle$3,
    circle_cross,
    circle_dot,
    circle_y,
    circle_x,
    cross: cross$1,
    diamond,
    diamond_dot,
    diamond_cross,
    dot,
    hex: hex$1,
    hex_dot,
    inverted_triangle,
    plus,
    square: square$1,
    square_cross,
    square_dot,
    square_pin,
    square_x,
    star,
    star_dot,
    triangle,
    triangle_dot,
    triangle_pin,
    dash,
    x: x$5,
    y: y$4
  };
  class ScatterView extends MarkerView {
    async load_glglyph() {
      const { MultiMarkerGL: _ } = await Promise.resolve().then(() => multi_marker);
      return _;
    }
    _render(_, H, V) {
      const { sx: q, sy: G, size: J, angle: Z, marker: te } = { ...this, ...V };
      for (const ne of H) {
        const ee = q[ne], ae = G[ne], oe = J.get(ne), se = Z.get(ne), ce = te.get(ne);
        if (!isFinite(ee + ae + oe + se) || ce == null)
          continue;
        const le = oe / 2;
        _.beginPath(), _.translate(ee, ae), se != 0 && _.rotate(se), marker_funcs[ce](_, ne, le, this.visuals), se != 0 && _.rotate(-se), _.translate(-ee, -ae);
      }
    }
    draw_legend_for_index(_, { x0: H, x1: V, y0: q, y1: G }, J) {
      const Z = J + 1, te = this.marker.get(J), ne = {
        ...this._get_legend_args({ x0: H, x1: V, y0: q, y1: G }, J),
        marker: new UniformScalar(te, Z)
      };
      this._render(_, [J], ne);
    }
  }
  mt(ScatterView, "__name__", "ScatterView");
  const um = class um extends Marker {
    constructor(_) {
      super(_);
    }
  };
  mt(um, "__name__", "Scatter"), um.prototype.default_view = ScatterView, um.define(() => ({
    marker: [MarkerSpec, { value: "circle" }]
  }));
  let Scatter = um;
  class SegmentView extends GlyphView {
    _project_data() {
      this._project_xy("x0", this.x0, "y0", this.y0), this._project_xy("x1", this.x1, "y1", this.y1);
    }
    _index_data(_) {
      const { min: H, max: V } = Math, { x0: q, x1: G, y0: J, y1: Z, data_size: te } = this;
      for (let ne = 0; ne < te; ne++) {
        const ee = q[ne], ae = G[ne], oe = J[ne], se = Z[ne];
        _.add_rect(H(ee, ae), H(oe, se), V(ee, ae), V(oe, se));
      }
    }
    _render(_, H, V) {
      if (!this.visuals.line.doit)
        return;
      const { sx0: q, sy0: G, sx1: J, sy1: Z } = { ...this, ...V };
      for (const te of H) {
        const ne = q[te], ee = G[te], ae = J[te], oe = Z[te];
        isFinite(ne + ee + ae + oe) && (this._render_decorations(_, te, ne, ee, ae, oe), _.beginPath(), _.moveTo(ne, ee), _.lineTo(ae, oe), this.visuals.line.apply(_, te));
      }
    }
    _render_decorations(_, H, V, q, G, J) {
      const { PI: Z } = Math, te = atan2$2([V, q], [G, J]) + Z / 2;
      for (const ne of this.decorations.values())
        _.save(), ne.model.node == "start" ? (_.translate(V, q), _.rotate(te + Z)) : ne.model.node == "end" && (_.translate(G, J), _.rotate(te)), ne.marking.render(_, H), _.restore();
    }
    _hit_point(_) {
      const { sx: H, sy: V } = _, q = { x: H, y: V }, G = 2, [J, Z] = this.renderer.xscale.r_invert(H - G, H + G), [te, ne] = this.renderer.yscale.r_invert(V - G, V + G), ee = this.index.indices({ x0: J, y0: te, x1: Z, y1: ne }), ae = [];
      for (const oe of ee) {
        const se = Math.max(2, this.line_width.get(oe) / 2) ** 2, ce = { x: this.sx0[oe], y: this.sy0[oe] }, le = { x: this.sx1[oe], y: this.sy1[oe] };
        dist_to_segment_squared(q, ce, le) < se && ae.push(oe);
      }
      return new Selection({ indices: ae });
    }
    _hit_span(_) {
      const [H, V] = this.renderer.plot_view.frame.bbox.ranges, { sx: q, sy: G } = _;
      let J, Z, te;
      _.direction == "v" ? (te = this.renderer.yscale.invert(G), [J, Z] = [this.y0, this.y1]) : (te = this.renderer.xscale.invert(q), [J, Z] = [this.x0, this.x1]);
      const ne = [], [ee, ae] = this.renderer.xscale.r_invert(H.start, H.end), [oe, se] = this.renderer.yscale.r_invert(V.start, V.end), ce = this.index.indices({ x0: ee, y0: oe, x1: ae, y1: se });
      for (const le of ce) {
        (J[le] <= te && te <= Z[le] || Z[le] <= te && te <= J[le]) && ne.push(le);
        const ue = 1.5 + this.line_width.get(le) / 2;
        J[le] == Z[le] && (_.direction == "h" ? Math.abs(this.sx0[le] - q) <= ue && ne.push(le) : Math.abs(this.sy0[le] - G) <= ue && ne.push(le));
      }
      return new Selection({ indices: ne });
    }
    scenterxy(_) {
      const H = this.sx0[_] / 2 + this.sx1[_] / 2, V = this.sy0[_] / 2 + this.sy1[_] / 2;
      return [H, V];
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(SegmentView, "__name__", "SegmentView");
  const Pp = class Pp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Pp, "__name__", "Segment"), Pp.prototype.default_view = SegmentView, Pp.define(({}) => ({
    x0: [XCoordinateSpec, { field: "x0" }],
    y0: [YCoordinateSpec, { field: "y0" }],
    x1: [XCoordinateSpec, { field: "x1" }],
    y1: [YCoordinateSpec, { field: "y1" }]
  })), Pp.mixins(LineVector$1);
  let Segment = Pp;
  function catmullrom_spline(O, _, H = 10, V = 0.5, q = !1) {
    assert$2(O.length == _.length);
    const G = O.length, J = q ? G + 1 : G, Z = infer_type(O, _), te = new Z(J + 2), ne = new Z(J + 2);
    te.set(O, 1), ne.set(_, 1), q ? (te[0] = O[G - 1], ne[0] = _[G - 1], te[J] = O[0], ne[J] = _[0], te[J + 1] = O[1], ne[J + 1] = _[1]) : (te[0] = O[0], ne[0] = _[0], te[J + 1] = O[G - 1], ne[J + 1] = _[G - 1]);
    const ee = new Z(4 * (H + 1));
    for (let se = 0, ce = 0; se <= H; se++) {
      const le = se / H, ue = le ** 2, de = le * ue;
      ee[ce++] = 2 * de - 3 * ue + 1, ee[ce++] = -2 * de + 3 * ue, ee[ce++] = de - 2 * ue + le, ee[ce++] = de - ue;
    }
    const ae = new Z((J - 1) * (H + 1)), oe = new Z((J - 1) * (H + 1));
    for (let se = 1, ce = 0; se < J; se++) {
      const le = (te[se + 1] - te[se - 1]) * V, ue = (ne[se + 1] - ne[se - 1]) * V, de = (te[se + 2] - te[se]) * V, fe = (ne[se + 2] - ne[se]) * V;
      for (let he = 0; he <= 4 * H; ce++) {
        const me = ee[he++], pe = ee[he++], ye = ee[he++], ge = ee[he++];
        ae[ce] = me * te[se] + pe * te[se + 1] + ye * le + ge * de, oe[ce] = me * ne[se] + pe * ne[se + 1] + ye * ue + ge * fe;
      }
    }
    return [ae, oe];
  }
  class SplineView extends XYGlyphView {
    _set_data() {
      const { tension: _, closed: H } = this.model, [V, q] = catmullrom_spline(this.x, this.y, 20, _, H);
      this._define_attr("xt", V), this._define_attr("yt", q);
    }
    _map_data() {
      const { x_scale: _, y_scale: H } = this.renderer.coordinates, V = _.v_compute(this.xt), q = H.v_compute(this.yt);
      this._define_attr("sxt", V), this._define_attr("syt", q);
    }
    _render(_, H, V) {
      const { sxt: q, syt: G } = { ...this, ...V };
      let J = !0;
      _.beginPath();
      const Z = q.length;
      for (let te = 0; te < Z; te++) {
        const ne = q[te], ee = G[te];
        isFinite(ne + ee) ? J ? (_.moveTo(ne, ee), J = !1) : _.lineTo(ne, ee) : J = !0;
      }
      this.visuals.line.set_value(_), _.stroke();
    }
  }
  mt(SplineView, "__name__", "SplineView");
  const $p = class $p extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt($p, "__name__", "Spline"), $p.prototype.default_view = SplineView, $p.mixins(LineScalar$1), $p.define(({ Bool: _, Float: H }) => ({
    tension: [H, 0.5],
    closed: [_, !1]
  }));
  let Spline = $p;
  class StepView extends XYGlyphView {
    async load_glglyph() {
      const { StepGL: _ } = await Promise.resolve().then(() => step);
      return _;
    }
    _render(_, H, V) {
      const q = H.length;
      if (q < 2)
        return;
      const { sx: G, sy: J } = { ...this, ...V }, Z = this.model.mode;
      this.visuals.line.set_value(_);
      let te = !1, ne = !1;
      const ee = H[0];
      let ae = isFinite(G[ee] + J[ee]);
      Z == "center" && (te = this._render_xy(_, te, ae ? G[ee] : NaN, J[ee]));
      for (const oe of H) {
        const se = isFinite(G[oe + 1] + J[oe + 1]);
        switch (Z) {
          case "before":
            te = this._render_xy(_, te, ae ? G[oe] : NaN, J[oe]), oe < G.length - 1 && (te = this._render_xy(_, te, ae && se ? G[oe] : NaN, J[oe + 1]));
            break;
          case "after":
            te = this._render_xy(_, te, ae ? G[oe] : NaN, J[oe]), oe < G.length - 1 && (te = this._render_xy(_, te, ae && se ? G[oe + 1] : NaN, J[oe]));
            break;
          case "center":
            if (ae && se) {
              const ce = (G[oe] + G[oe + 1]) / 2;
              te = this._render_xy(_, te, ce, J[oe]), te = this._render_xy(_, te, ce, J[oe + 1]);
            } else
              ne && (te = this._render_xy(_, te, ae ? G[oe] : NaN, J[oe])), te = this._render_xy(_, te, se ? G[oe + 1] : NaN, J[oe + 1]);
            break;
          default:
            unreachable();
        }
        ne = ae, ae = se;
      }
      if (te) {
        const oe = H[q - 1];
        this._render_xy(_, te, ae ? G[oe] : NaN, J[oe]) && _.stroke();
      }
    }
    _render_xy(_, H, V, q) {
      return isFinite(V + q) ? H ? _.lineTo(V, q) : (_.beginPath(), _.moveTo(V, q), H = !0) : H && (_.stroke(), H = !1), H;
    }
    draw_legend_for_index(_, H, V) {
      generic_line_scalar_legend(this.visuals, _, H);
    }
  }
  mt(StepView, "__name__", "StepView");
  var d2;
  let Step$1 = (d2 = class extends XYGlyph {
    constructor(_) {
      super(_);
    }
  }, mt(d2, "__name__", "Step"), d2.prototype.default_view = StepView, d2.mixins(LineScalar$1), d2.define(() => ({
    mode: [StepMode, "before"]
  })), d2);
  const DisplayMode = Or$1(Enum("inline", "block"), Auto);
  class TeXGlyphView extends MathTextGlyphView {
    _build_label(_) {
      const { macros: H, display: V } = this.model;
      if (V == "auto") {
        const q = parse_delimited_string(_);
        return q instanceof TeX && (q.macros = H), q;
      } else
        return new TeX({ text: _, macros: H, inline: V == "inline" });
    }
  }
  mt(TeXGlyphView, "__name__", "TeXGlyphView");
  const dm = class dm extends MathTextGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(dm, "__name__", "TeXGlyph"), dm.prototype.default_view = TeXGlyphView, dm.define(({ Float: _, Str: H, Dict: V, Tuple: q, Or: G }) => ({
    macros: [V(G(H, q(H, _))), {}],
    display: [DisplayMode, "auto"]
  }));
  let TeXGlyph = dm;
  class VBarView extends LRTBView {
    scenterxy(_) {
      const H = this.sx[_], V = (this.stop[_] + this.sbottom[_]) / 2;
      return [H, V];
    }
    _lrtb(_) {
      const H = this.width.get(_) / 2, V = this.x[_], q = this.top[_], G = this.bottom[_], J = V - H, Z = V + H, te = Math.max(q, G), ne = Math.min(q, G);
      return { l: J, r: Z, t: te, b: ne };
    }
    _map_data() {
      if (this.inherited_x && this.inherited_width)
        this._inherit_attr("swidth"), this._inherit_attr("sleft"), this._inherit_attr("sright");
      else {
        const _ = this.sdist(this.renderer.xscale, this.x, this.width, "center"), { sx: H } = this, V = H.length, q = new ScreenArray(V), G = new ScreenArray(V);
        for (let J = 0; J < V; J++) {
          const Z = H[J], te = _[J];
          q[J] = Z - te / 2, G[J] = Z + te / 2;
        }
        this._define_attr("swidth", _), this._define_attr("sleft", q), this._define_attr("sright", G);
      }
      this._clamp_to_viewport();
    }
  }
  mt(VBarView, "__name__", "VBarView");
  const fm = class fm extends LRTB {
    constructor(_) {
      super(_);
    }
  };
  mt(fm, "__name__", "VBar"), fm.prototype.default_view = VBarView, fm.define(({}) => ({
    x: [XCoordinateSpec, { field: "x" }],
    bottom: [YCoordinateSpec, { value: 0 }],
    width: [DistanceSpec, { value: 1 }],
    top: [YCoordinateSpec, { field: "top" }]
  }));
  let VBar = fm;
  const { abs: abs$4, max: max$8 } = Math, UNUSED$1 = 0;
  class VSpanView extends GlyphView {
    after_visuals() {
      super.after_visuals(), this.max_line_width = max$i(this.line_width);
    }
    _index_data(_) {
      for (const H of this.x)
        _.add_point(H, UNUSED$1);
    }
    _bounds(_) {
      const { x0: H, x1: V } = _;
      return { x0: H, x1: V, y0: NaN, y1: NaN };
    }
    _map_data() {
      super._map_data();
      const { round: _ } = Math;
      if (!this.inherited_sx) {
        const H = map$4(this.sx, (V) => _(V));
        this._define_attr("sx", H);
      }
    }
    scenterxy(_) {
      const { vcenter: H } = this.renderer.plot_view.frame.bbox;
      return [this.sx[_], H];
    }
    _render(_, H, V) {
      const { sx: q } = { ...this, ...V }, { top: G, bottom: J } = this.renderer.plot_view.frame.bbox;
      for (const Z of H) {
        const te = q[Z];
        isFinite(te) && (_.beginPath(), _.moveTo(te, G), _.lineTo(te, J), this.visuals.line.apply(_, Z));
      }
    }
    _get_candidates(_, H) {
      const { max_line_width: V } = this, [q, G] = this.renderer.xscale.r_invert(_ - V, (H ?? _) + V);
      return this.index.indices({ x0: q, x1: G, y0: 0, y1: 0 });
    }
    _find_spans(_, H) {
      const { sx: V, line_width: q } = this, G = [];
      for (const J of _) {
        const Z = V[J], te = q.get(J);
        H(Z, te) && G.push(J);
      }
      return G;
    }
    _hit_point(_) {
      const { sx: H } = _, V = this._get_candidates(H), q = this._find_spans(V, (G, J) => abs$4(G - H) <= max$8(
        J,
        2
        /*px*/
      ));
      return new Selection({ indices: q });
    }
    _hit_span(_) {
      const H = (() => {
        if (_.direction == "h")
          return range$9(0, this.data_size);
        {
          const { sx: V } = _, q = this._get_candidates(V);
          return this._find_spans(q, (G, J) => abs$4(G - V) <= max$8(
            J / 2,
            2
            /*px*/
          ));
        }
      })();
      return new Selection({ indices: H });
    }
    _hit_rect(_) {
      const H = (() => {
        const { sx0: V, sx1: q } = _, G = this._get_candidates(V, q);
        return this._find_spans(G, (J, Z) => V - Z / 2 <= J && J <= q + Z / 2);
      })();
      return new Selection({ indices: H });
    }
    draw_legend_for_index(_, H, V) {
      generic_line_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(VSpanView, "__name__", "VSpanView");
  const Fp = class Fp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Fp, "__name__", "VSpan"), Fp.prototype.default_view = VSpanView, Fp.mixins([LineVector$1]), Fp.define(() => ({
    x: [XCoordinateSpec, { field: "x" }]
  }));
  let VSpan = Fp;
  const UNUSED = 0;
  class VStripView extends GlyphView {
    async lazy_initialize() {
      await super.lazy_initialize();
      const { webgl: _ } = this.renderer.plot_view.canvas_view;
      if (_ != null && _.regl_wrapper.has_webgl) {
        const { LRTBGL: H } = await Promise.resolve().then(() => lrtb);
        this.glglyph = new H(_.regl_wrapper, this);
      }
    }
    get sleft() {
      return this.sx0;
    }
    get sright() {
      return this.sx1;
    }
    get stop() {
      const { top: _ } = this.renderer.plot_view.frame.bbox, H = this.data_size, V = new ScreenArray(H);
      return V.fill(_), V;
    }
    get sbottom() {
      const { bottom: _ } = this.renderer.plot_view.frame.bbox, H = this.data_size, V = new ScreenArray(H);
      return V.fill(_), V;
    }
    _set_data(_) {
      super._set_data(_);
      const { abs: H } = Math, { max: V, map: q, zip: G } = iter, { x0: J, x1: Z } = this;
      if (this.inherited_x0 && this.inherited_x1)
        this._inherit_attr("max_width");
      else {
        const te = V(q(G(J, Z), ([ne, ee]) => H(ne - ee)));
        this._define_attr("max_width", te);
      }
    }
    _index_data(_) {
      const { x0: H, x1: V, data_size: q } = this;
      for (let G = 0; G < q; G++) {
        const J = H[G], Z = V[G];
        _.add_rect(J, UNUSED, Z, UNUSED);
      }
    }
    _bounds(_) {
      const { x0: H, x1: V } = _;
      return { x0: H, x1: V, y0: NaN, y1: NaN };
    }
    _map_data() {
      super._map_data();
      const { round: _ } = Math;
      if (!this.inherited_sx0) {
        const H = map$4(this.sx0, (V) => _(V));
        this._define_attr("sx0", H);
      }
      if (!this.inherited_sx1) {
        const H = map$4(this.sx1, (V) => _(V));
        this._define_attr("sx1", H);
      }
    }
    scenterxy(_) {
      const { vcenter: H } = this.renderer.plot_view.frame.bbox;
      return [(this.sx0[_] + this.sx1[_]) / 2, H];
    }
    _render(_, H, V) {
      const { sx0: q, sx1: G } = { ...this, ...V }, { top: J, bottom: Z, height: te } = this.renderer.plot_view.frame.bbox;
      for (const ne of H) {
        const ee = q[ne], ae = G[ne];
        isFinite(ee + ae) && (_.beginPath(), _.rect(ee, J, ae - ee, te), this.visuals.fill.apply(_, ne), this.visuals.hatch.apply(_, ne), _.beginPath(), _.moveTo(ee, J), _.lineTo(ee, Z), _.moveTo(ae, J), _.lineTo(ae, Z), this.visuals.line.apply(_, ne));
      }
    }
    _get_candidates(_, H) {
      const { max_width: V } = this, [q, G] = this.renderer.xscale.r_invert(_, H ?? _), J = q - V, Z = G + V;
      return this.index.indices({ x0: J, x1: Z, y0: 0, y1: 0 });
    }
    _find_strips(_, H) {
      function V(Z, te) {
        return Z <= te ? H(Z, te) : H(te, Z);
      }
      const { sx0: q, sx1: G } = this, J = [];
      for (const Z of _) {
        const te = q[Z], ne = G[Z];
        V(te, ne) && J.push(Z);
      }
      return J;
    }
    _hit_point(_) {
      const { sx: H } = _, V = this._get_candidates(H), q = this._find_strips(V, (G, J) => G <= H && H <= J);
      return new Selection({ indices: q });
    }
    _hit_span(_) {
      const H = (() => {
        if (_.direction == "h")
          return range$9(0, this.data_size);
        {
          const { sx: V } = _, q = this._get_candidates(V);
          return this._find_strips(q, (G, J) => G <= V && V <= J);
        }
      })();
      return new Selection({ indices: H });
    }
    _hit_rect(_) {
      const H = (() => {
        const { sx0: V, sx1: q } = _, G = this._get_candidates(V, q);
        return this._find_strips(G, (J, Z) => V <= J && J <= q && V <= Z && Z <= q);
      })();
      return new Selection({ indices: H });
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
  }
  mt(VStripView, "__name__", "VStripView");
  const Bp = class Bp extends Glyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Bp, "__name__", "VStrip"), Bp.prototype.default_view = VStripView, Bp.mixins([LineVector$1, FillVector$1, HatchVector$1]), Bp.define(() => ({
    x0: [XCoordinateSpec, { field: "x0" }],
    x1: [XCoordinateSpec, { field: "x1" }]
  }));
  let VStrip = Bp;
  class WedgeView extends XYGlyphView {
    async load_glglyph() {
      const { WedgeGL: _ } = await Promise.resolve().then(() => wedge);
      return _;
    }
    _map_data() {
      this._define_or_inherit_attr("sradius", () => this.model.properties.radius.units == "data" ? this.inherited_x && this.inherited_radius ? inherit : this.sdist(this.renderer.xscale, this.x, this.radius) : this.inherited_radius ? inherit : to_screen(this.radius)), this._define_or_inherit_attr("max_sradius", () => max$j(this.sradius));
    }
    _render(_, H, V) {
      const { sx: q, sy: G, sradius: J, start_angle: Z, end_angle: te } = { ...this, ...V }, ne = this.model.direction == "anticlock";
      for (const ee of H) {
        const ae = q[ee], oe = G[ee], se = J[ee], ce = Z.get(ee), le = te.get(ee);
        isFinite(ae + oe + se + ce + le) && (_.beginPath(), _.arc(ae, oe, se, ce, le, ne), _.lineTo(ae, oe), _.closePath(), this.visuals.fill.apply(_, ee), this.visuals.hatch.apply(_, ee), this.visuals.line.apply(_, ee));
      }
    }
    _hit_point(_) {
      let H, V, q, G, J;
      const { sx: Z, sy: te } = _, ne = this.renderer.xscale.invert(Z), ee = this.renderer.yscale.invert(te);
      V = Z - this.max_sradius, q = Z + this.max_sradius;
      const [ae, oe] = this.renderer.xscale.r_invert(V, q);
      G = te - this.max_sradius, J = te + this.max_sradius;
      const [se, ce] = this.renderer.yscale.r_invert(G, J), le = [];
      for (const fe of this.index.indices({ x0: ae, x1: oe, y0: se, y1: ce })) {
        const he = this.sradius[fe] ** 2;
        [V, q] = this.renderer.xscale.r_compute(ne, this.x[fe]), [G, J] = this.renderer.yscale.r_compute(ee, this.y[fe]), H = (V - q) ** 2 + (G - J) ** 2, H <= he && le.push(fe);
      }
      const ue = this.model.direction == "anticlock", de = [];
      for (const fe of le) {
        const he = Math.atan2(te - this.sy[fe], Z - this.sx[fe]);
        (Math.abs(this.start_angle.get(fe) - this.end_angle.get(fe)) >= 2 * Math.PI || angle_between(-he, -this.start_angle.get(fe), -this.end_angle.get(fe), ue)) && de.push(fe);
      }
      return new Selection({ indices: de });
    }
    draw_legend_for_index(_, H, V) {
      generic_area_vector_legend(this.visuals, _, H, V);
    }
    scenterxy(_) {
      const H = this.sradius[_] / 2, V = (this.start_angle.get(_) + this.end_angle.get(_)) / 2, q = this.sx[_] + H * Math.cos(V), G = this.sy[_] + H * Math.sin(V);
      return [q, G];
    }
  }
  mt(WedgeView, "__name__", "WedgeView");
  const Hp = class Hp extends XYGlyph {
    constructor(_) {
      super(_);
    }
  };
  mt(Hp, "__name__", "Wedge"), Hp.prototype.default_view = WedgeView, Hp.mixins([LineVector$1, FillVector$1, HatchVector$1]), Hp.define(({}) => ({
    direction: [Direction$1, "anticlock"],
    radius: [DistanceSpec, { field: "radius" }],
    start_angle: [AngleSpec, { field: "start_angle" }],
    end_angle: [AngleSpec, { field: "end_angle" }]
  }));
  let Wedge = Hp;
  class GraphHitTestPolicy extends Model$1 {
    constructor(_) {
      super(_);
    }
    _hit_test(_, H, V) {
      if (!H.model.visible)
        return null;
      const q = V.glyph.hit_test(_);
      return q == null ? null : V.model.view.convert_selection_from_subset(q);
    }
  }
  mt(GraphHitTestPolicy, "__name__", "GraphHitTestPolicy");
  class EdgesOnly extends GraphHitTestPolicy {
    constructor(_) {
      super(_);
    }
    hit_test(_, H) {
      return this._hit_test(_, H, H.edge_view);
    }
    do_selection(_, H, V, q) {
      if (_ == null)
        return !1;
      const G = H.edge_renderer.data_source.selected;
      return G.update(_, V, q), H.edge_renderer.data_source._select.emit(), !G.is_empty();
    }
    do_inspection(_, H, V, q, G) {
      if (_ == null)
        return !1;
      const { edge_renderer: J } = V.model, Z = J.get_selection_manager().get_or_create_inspector(V.edge_view.model);
      return Z.update(_, q, G), V.edge_view.model.data_source.setv({ inspected: Z }, { silent: !0 }), V.edge_view.model.data_source.inspect.emit([V.edge_view.model, { geometry: H }]), !Z.is_empty();
    }
  }
  mt(EdgesOnly, "__name__", "EdgesOnly");
  class NodesOnly extends GraphHitTestPolicy {
    constructor(_) {
      super(_);
    }
    hit_test(_, H) {
      return this._hit_test(_, H, H.node_view);
    }
    do_selection(_, H, V, q) {
      if (_ == null)
        return !1;
      const G = H.node_renderer.data_source.selected;
      return G.update(_, V, q), H.node_renderer.data_source._select.emit(), !G.is_empty();
    }
    do_inspection(_, H, V, q, G) {
      if (_ == null)
        return !1;
      const { node_renderer: J } = V.model, Z = J.get_selection_manager().get_or_create_inspector(V.node_view.model);
      return Z.update(_, q, G), V.node_view.model.data_source.setv({ inspected: Z }, { silent: !0 }), V.node_view.model.data_source.inspect.emit([V.node_view.model, { geometry: H }]), !Z.is_empty();
    }
  }
  mt(NodesOnly, "__name__", "NodesOnly");
  class NodesAndLinkedEdges extends GraphHitTestPolicy {
    constructor(_) {
      super(_);
    }
    hit_test(_, H) {
      return this._hit_test(_, H, H.node_view);
    }
    get_linked_edges(_, H, V) {
      const G = dict(_.data).get("index") ?? [], J = (() => {
        switch (V) {
          case "selection":
            return map$4(_.selected.indices, (se) => G[se]);
          case "inspection":
            return map$4(_.inspected.indices, (se) => G[se]);
        }
      })(), Z = dict(H.data), te = Z.get("start") ?? [], ne = Z.get("end") ?? [], ee = [], ae = te.length;
      for (let se = 0; se < ae; se++)
        (contains$6(J, te[se]) || contains$6(J, ne[se])) && ee.push(se);
      const oe = new Selection();
      for (const se of ee)
        oe.multiline_indices.set(se, [0]);
      return oe.indices = ee, oe;
    }
    do_selection(_, H, V, q) {
      if (_ == null)
        return !1;
      const G = H.node_renderer.data_source.selected;
      G.update(_, V, q);
      const J = H.edge_renderer.data_source.selected, Z = this.get_linked_edges(H.node_renderer.data_source, H.edge_renderer.data_source, "selection");
      return J.update(Z, V, q), H.node_renderer.data_source._select.emit(), !G.is_empty();
    }
    do_inspection(_, H, V, q, G) {
      if (_ == null)
        return !1;
      const J = V.node_view.model.data_source.selection_manager.get_or_create_inspector(V.node_view.model);
      J.update(_, q, G), V.node_view.model.data_source.setv({ inspected: J }, { silent: !0 });
      const Z = V.edge_view.model.data_source.selection_manager.get_or_create_inspector(V.edge_view.model), te = this.get_linked_edges(V.node_view.model.data_source, V.edge_view.model.data_source, "inspection");
      return Z.update(te, q, G), V.edge_view.model.data_source.setv({ inspected: Z }, { silent: !0 }), V.node_view.model.data_source.inspect.emit([V.node_view.model, { geometry: H }]), !J.is_empty();
    }
  }
  mt(NodesAndLinkedEdges, "__name__", "NodesAndLinkedEdges");
  class EdgesAndLinkedNodes extends GraphHitTestPolicy {
    constructor(_) {
      super(_);
    }
    hit_test(_, H) {
      return this._hit_test(_, H, H.edge_view);
    }
    get_linked_nodes(_, H, V) {
      const q = (() => {
        switch (V) {
          case "selection":
            return H.selected.indices;
          case "inspection":
            return H.inspected.indices;
        }
      })(), G = dict(H.data), J = G.get("start") ?? [], Z = G.get("end") ?? [], te = [];
      for (const oe of q)
        te.push(J[oe], Z[oe]);
      const ee = dict(_.data).get("index") ?? [], ae = uniq(te).map((oe) => index_of(ee, oe));
      return new Selection({ indices: ae });
    }
    do_selection(_, H, V, q) {
      if (_ == null)
        return !1;
      const G = H.edge_renderer.data_source.selected;
      G.update(_, V, q);
      const J = H.node_renderer.data_source.selected, Z = this.get_linked_nodes(H.node_renderer.data_source, H.edge_renderer.data_source, "selection");
      return J.update(Z, V, q), H.edge_renderer.data_source._select.emit(), !G.is_empty();
    }
    do_inspection(_, H, V, q, G) {
      if (_ == null)
        return !1;
      const J = V.edge_view.model.data_source.selection_manager.get_or_create_inspector(V.edge_view.model);
      J.update(_, q, G), V.edge_view.model.data_source.setv({ inspected: J }, { silent: !0 });
      const Z = V.node_view.model.data_source.selection_manager.get_or_create_inspector(V.node_view.model), te = this.get_linked_nodes(V.node_view.model.data_source, V.edge_view.model.data_source, "inspection");
      return Z.update(te, q, G), V.node_view.model.data_source.setv({ inspected: Z }, { silent: !0 }), V.edge_view.model.data_source.inspect.emit([V.edge_view.model, { geometry: H }]), !J.is_empty();
    }
  }
  mt(EdgesAndLinkedNodes, "__name__", "EdgesAndLinkedNodes");
  class NodesAndAdjacentNodes extends GraphHitTestPolicy {
    constructor(_) {
      super(_);
    }
    hit_test(_, H) {
      return this._hit_test(_, H, H.node_view);
    }
    get_adjacent_nodes(_, H, V) {
      const G = dict(_.data).get("index") ?? [], J = (() => {
        switch (V) {
          case "selection":
            return map$4(_.selected.indices, (se) => G[se]);
          case "inspection":
            return map$4(_.inspected.indices, (se) => G[se]);
        }
      })(), Z = dict(H.data), te = Z.get("start") ?? [], ne = Z.get("end") ?? [], ee = [], ae = [];
      for (let se = 0; se < te.length; se++)
        contains$6(J, te[se]) && (ee.push(ne[se]), ae.push(te[se])), contains$6(J, ne[se]) && (ee.push(te[se]), ae.push(ne[se]));
      for (let se = 0; se < ae.length; se++)
        ee.push(ae[se]);
      const oe = uniq(ee).map((se) => index_of(G, se));
      return new Selection({ indices: oe });
    }
    do_selection(_, H, V, q) {
      if (_ == null)
        return !1;
      const G = H.node_renderer.data_source.selected;
      G.update(_, V, q);
      const J = this.get_adjacent_nodes(H.node_renderer.data_source, H.edge_renderer.data_source, "selection");
      return J.is_empty() || G.update(J, V, q), H.node_renderer.data_source._select.emit(), !G.is_empty();
    }
    do_inspection(_, H, V, q, G) {
      if (_ == null)
        return !1;
      const J = V.node_view.model.data_source.selection_manager.get_or_create_inspector(V.node_view.model);
      J.update(_, q, G), V.node_view.model.data_source.setv({ inspected: J }, { silent: !0 });
      const Z = this.get_adjacent_nodes(V.node_view.model.data_source, V.edge_view.model.data_source, "inspection");
      return Z.is_empty() || (J.update(Z, q, G), V.node_view.model.data_source.setv({ inspected: J }, { silent: !0 })), V.node_view.model.data_source.inspect.emit([V.node_view.model, { geometry: H }]), !J.is_empty();
    }
  }
  mt(NodesAndAdjacentNodes, "__name__", "NodesAndAdjacentNodes");
  class LayoutProvider extends Model$1 {
    constructor(_) {
      super(_);
    }
    get node_coordinates() {
      return new NodeCoordinates({ layout: this });
    }
    get edge_coordinates() {
      return new EdgeCoordinates({ layout: this });
    }
  }
  mt(LayoutProvider, "__name__", "LayoutProvider");
  const V5 = class V5 extends CoordinateTransform {
    constructor(_) {
      super(_);
    }
  };
  mt(V5, "__name__", "GraphCoordinates"), V5.define(({ Ref: _ }) => ({
    layout: [_(LayoutProvider)]
  }));
  let GraphCoordinates = V5;
  class NodeCoordinates extends GraphCoordinates {
    constructor(_) {
      super(_);
    }
    _v_compute(_) {
      const [H, V] = this.layout.get_node_coordinates(_);
      return { x: H, y: V };
    }
  }
  mt(NodeCoordinates, "__name__", "NodeCoordinates");
  class EdgeCoordinates extends GraphCoordinates {
    constructor(_) {
      super(_);
    }
    _v_compute(_) {
      const [H, V] = this.layout.get_edge_coordinates(_);
      return { x: H, y: V };
    }
  }
  mt(EdgeCoordinates, "__name__", "EdgeCoordinates");
  const GraphLayout = Or$1(Dict(Arrayable(Float$1)), Mapping(Or$1(Int$1, Str), Arrayable(Float$1))), U5 = class U5 extends LayoutProvider {
    constructor(_) {
      super(_);
    }
    get_node_coordinates(_) {
      const V = dict(_.data).get("index") ?? [], q = V.length, G = new Float64Array(q), J = new Float64Array(q), Z = dict(this.graph_layout);
      for (let te = 0; te < q; te++) {
        const ne = V[te], [ee, ae] = Z.get(ne) ?? [NaN, NaN];
        G[te] = ee, J[te] = ae;
      }
      return [G, J];
    }
    get_edge_coordinates(_) {
      const H = dict(_.data), V = H.get("start") ?? [], q = H.get("end") ?? [], G = Math.min(V.length, q.length), J = [], Z = [], te = H.get("xs"), ne = H.get("ys"), ee = te != null && ne != null, ae = dict(this.graph_layout);
      for (let oe = 0; oe < G; oe++) {
        const se = ae.has(V[oe]) && ae.has(q[oe]);
        if (ee && se)
          J.push(te[oe]), Z.push(ne[oe]);
        else {
          const ce = ae.get(V[oe]) ?? [NaN, NaN], le = ae.get(q[oe]) ?? [NaN, NaN];
          J.push([ce[0], le[0]]), Z.push([ce[1], le[1]]);
        }
      }
      return [J, Z];
    }
  };
  mt(U5, "__name__", "StaticLayoutProvider"), U5.define(() => ({
    graph_layout: [GraphLayout, /* @__PURE__ */ new Map()]
    // TODO: length == 2
  }));
  let StaticLayoutProvider = U5;
  class GridView extends GuideRendererView {
    _render() {
      const _ = this.layer.ctx;
      _.save(), this._draw_regions(_), this._draw_minor_grids(_), this._draw_grids(_), _.restore();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.request_render());
    }
    _draw_regions(_) {
      if (!this.visuals.band_fill.doit && !this.visuals.band_hatch.doit)
        return;
      const [H, V] = this.grid_coords("major", !1);
      for (let q = 0; q < H.length - 1; q++) {
        if (q % 2 != 1)
          continue;
        const [G, J] = this.coordinates.map_to_screen(H[q], V[q]), [Z, te] = this.coordinates.map_to_screen(H[q + 1], V[q + 1]);
        _.beginPath(), _.rect(G[0], J[0], Z[1] - G[0], te[1] - J[0]), this.visuals.band_fill.apply(_), this.visuals.band_hatch.apply(_);
      }
    }
    _draw_grids(_) {
      if (!this.visuals.grid_line.doit)
        return;
      const [H, V] = this.grid_coords("major");
      this._draw_grid_helper(_, this.visuals.grid_line, H, V);
    }
    _draw_minor_grids(_) {
      if (!this.visuals.minor_grid_line.doit)
        return;
      const [H, V] = this.grid_coords("minor");
      this._draw_grid_helper(_, this.visuals.minor_grid_line, H, V);
    }
    _draw_grid_helper(_, H, V, q) {
      H.set_value(_), _.beginPath();
      for (let G = 0; G < V.length; G++) {
        const [J, Z] = this.coordinates.map_to_screen(V[G], q[G]);
        _.moveTo(Math.round(J[0]), Math.round(Z[0]));
        for (let te = 1; te < J.length; te++)
          _.lineTo(Math.round(J[te]), Math.round(Z[te]));
      }
      _.stroke();
    }
    // {{{ TODO: state
    ranges() {
      const _ = this.model.dimension, H = 1 - _, { ranges: V } = this.coordinates;
      return [V[_], V[H]];
    }
    computed_bounds() {
      const [_] = this.ranges(), H = this.model.bounds, V = [_.min, _.max];
      let q, G;
      if (isArray$3(H))
        q = Math.min(H[0], H[1]), G = Math.max(H[0], H[1]), q < V[0] && (q = V[0]), G > V[1] && (G = V[1]);
      else {
        [q, G] = V;
        for (const J of this.plot_view.axis_views)
          J.dimension == this.model.dimension && J.model.x_range_name == this.model.x_range_name && J.model.y_range_name == this.model.y_range_name && ([q, G] = J.computed_bounds);
      }
      return [q, G];
    }
    grid_coords(_, H = !0) {
      const V = this.model.dimension, q = 1 - V, [G, J] = this.ranges(), [Z, te] = (() => {
        const [ue, de] = this.computed_bounds();
        return [Math.min(ue, de), Math.max(ue, de)];
      })(), ne = [[], []], ee = this.model.get_ticker();
      if (ee == null)
        return ne;
      const ae = ee.get_ticks(Z, te, G, J.min)[_], oe = G.min, se = G.max, [ce, le] = (() => {
        const { cross_bounds: ue } = this.model;
        return ue == "auto" ? [J.min, J.max] : ue;
      })();
      H || (ae[0] != oe && ae.splice(0, 0, oe), ae[ae.length - 1] != se && ae.push(se));
      for (let ue = 0; ue < ae.length; ue++) {
        if ((ae[ue] == oe || ae[ue] == se) && H)
          continue;
        const de = [], fe = [], he = 2;
        for (let me = 0; me < he; me++) {
          const pe = ce + (le - ce) / (he - 1) * me;
          de.push(ae[ue]), fe.push(pe);
        }
        ne[V].push(de), ne[q].push(fe);
      }
      return ne;
    }
  }
  mt(GridView, "__name__", "GridView");
  const m0 = class m0 extends GuideRenderer {
    constructor(_) {
      super(_);
    }
    get_ticker() {
      return this.ticker != null ? this.ticker : this.axis != null ? this.axis.ticker : null;
    }
  };
  mt(m0, "__name__", "Grid"), m0.prototype.default_view = GridView, m0.mixins([
    ["grid_", Line$4],
    ["minor_grid_", Line$4],
    ["band_", Fill$2],
    ["band_", Hatch$1]
  ]), m0.define(({ Float: _, Auto: H, Enum: V, Ref: q, Tuple: G, Or: J, Nullable: Z }) => ({
    bounds: [J(G(_, _), H), "auto"],
    cross_bounds: [J(G(_, _), H), "auto"],
    dimension: [V(0, 1), 0],
    axis: [Z(q(Axis)), null],
    ticker: [Z(q(Ticker)), null]
  })), m0.override({
    level: "underlay",
    band_fill_color: null,
    band_fill_alpha: 0,
    grid_line_color: "#e5e5e5",
    minor_grid_line_color: null
  });
  let Grid = m0;
  const HTMLRef = Or$1(Ref(DOMNode), Ref(UIElement)), HTMLMarkup = Str;
  class HTMLView extends DOMNodeView {
    constructor() {
      super(...arguments);
      mt(this, "_refs", /* @__PURE__ */ new Map());
    }
    get refs() {
      const { html: H, refs: V } = this.model;
      return [
        ...isArray$3(H) ? H.filter((q) => !isString$2(q)) : [],
        ...V
      ];
    }
    *children() {
      yield* super.children(), yield* this._refs.values();
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await build_views(this._refs, this.refs);
    }
    remove() {
      remove_views(this._refs), super.remove();
    }
    render() {
      empty$1(this.el);
      const H = (() => {
        const { html: q } = this.model;
        return isArray$3(q) ? q.map((G) => isString$2(G) ? G : `<ref id="${G.id}"></ref>`).join("") : q;
      })(), V = isString$2(H) ? this.parse_html(H) : [H];
      this.el.append(...V), this.finish();
    }
    parse_html(H) {
      const q = new DOMParser().parseFromString(H, "text/html"), G = q.createNodeIterator(q, NodeFilter.SHOW_ELEMENT, (Z) => Z.nodeName.toLowerCase() == "ref" ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_REJECT);
      let J;
      e: for (; (J = G.nextNode()) != null; ) {
        assert$2(J instanceof Element);
        const Z = J.getAttribute("id");
        if (Z != null) {
          for (const [ne, ee] of this._refs)
            if (ne.id == Z) {
              ee.render(), J.replaceWith(ee.el);
              continue e;
            }
          J.replaceWith(span$1(`<not found: id=${Z}>`));
          continue;
        }
        const te = J.getAttribute("name");
        if (te != null) {
          for (const [ne, ee] of this._refs)
            if (ne.name == te) {
              ee.render(), J.replaceWith(ee.el);
              continue e;
            }
          J.replaceWith(span$1(`<not found: name=${te}>`));
          continue;
        }
      }
      return [...q.body.childNodes];
    }
  }
  mt(HTMLView, "__name__", "HTMLView");
  const hm = class hm extends DOMNode {
    constructor(_) {
      super(_);
    }
  };
  mt(hm, "__name__", "HTML"), hm.prototype.default_view = HTMLView, hm.define(({ Node: _, List: H, Or: V }) => ({
    html: [V(_, HTMLMarkup, H(V(HTMLMarkup, HTMLRef)))],
    refs: [H(HTMLRef), []]
  }));
  let HTML = hm;
  const ElementLike = Or$1(Ref(UIElement), Ref(DOMNode), Ref(HTML));
  class PaneView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "_element_views", /* @__PURE__ */ new Map());
    }
    get elements() {
      return this.model.elements;
    }
    get element_views() {
      return this.elements.map((H) => this._element_views.get(H));
    }
    *children() {
      yield* super.children(), yield* this.element_views;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._build_elements();
    }
    async _build_elements() {
      return await build_views(this._element_views, this.elements, { parent: this });
    }
    async _update_elements() {
      const { created: H } = await this._build_elements(), V = new Set(H);
      for (const q of this.element_views)
        q.el.remove();
      for (const q of this.element_views)
        V.has(q) ? q.render_to(this.shadow_el) : this.shadow_el.append(q.el);
    }
    remove() {
      remove_views(this._element_views), super.remove();
    }
    connect_signals() {
      super.connect_signals();
      const { elements: H } = this.model.properties;
      this.on_change(H, async () => {
        await this._update_elements();
      });
    }
    render() {
      super.render();
      for (const H of this.element_views)
        H.render_to(this.shadow_el);
    }
    has_finished() {
      if (!super.has_finished())
        return !1;
      for (const H of this.element_views)
        if (!H.has_finished())
          return !1;
      return !0;
    }
    serializable_state() {
      const { children: H, ...V } = super.serializable_state();
      return {
        ...V,
        children: [...H ?? [], ...this.element_views.map((q) => q.serializable_state())]
      };
    }
  }
  mt(PaneView, "__name__", "PaneView");
  const pm = class pm extends UIElement {
    constructor(_) {
      super(_);
    }
  };
  mt(pm, "__name__", "Pane"), pm.prototype.default_view = PaneView, pm.define(({ List: _ }) => ({
    elements: [_(ElementLike), []]
  }));
  let Pane = pm;
  const xh = class xh extends PaneView {
    constructor() {
      super(...arguments);
      mt(this, "_child_views", /* @__PURE__ */ new Map());
      mt(this, "layout");
      mt(this, "mouseenter", new Signal(this, "mouseenter"));
      mt(this, "mouseleave", new Signal(this, "mouseleave"));
      mt(this, "disabled", new Signal(this, "disabled"));
      mt(this, "_resized", !1);
      mt(this, "_auto_width", "fit-content");
      mt(this, "_auto_height", "fit-content");
      mt(this, "_layout_computed", !1);
      mt(this, "_was_built", !1);
    }
    get is_layout_root() {
      return this.is_root || !(this.parent instanceof xh);
    }
    _after_resize() {
      this._resized = !0, super._after_resize(), this.is_layout_root && !this._was_built ? (logger$1.warn(`${this} wasn't built properly`), this.render_to(null)) : this.compute_layout();
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this.build_child_views();
    }
    remove() {
      for (const H of this.child_views)
        H.remove();
      this._child_views.clear(), super.remove();
    }
    connect_signals() {
      super.connect_signals(), this.el.addEventListener("mouseenter", (V) => {
        this.mouseenter.emit(V);
      }), this.el.addEventListener("mouseleave", (V) => {
        this.mouseleave.emit(V);
      }), this.parent instanceof xh && this.connect(this.parent.disabled, (V) => {
        this.disabled.emit(V || this.model.disabled);
      });
      const H = this.model.properties;
      this.on_change(H.disabled, () => {
        this.disabled.emit(this.model.disabled);
      }), this.on_change([
        H.css_classes,
        H.stylesheets,
        H.width,
        H.height,
        H.min_width,
        H.min_height,
        H.max_width,
        H.max_height,
        H.margin,
        H.width_policy,
        H.height_policy,
        H.flow_mode,
        H.sizing_mode,
        H.aspect_ratio,
        H.visible
      ], () => this.invalidate_layout());
    }
    *children() {
      yield* super.children(), yield* this.child_views;
    }
    get child_views() {
      return this.child_models.map((H) => this._child_views.get(H)).filter(isNotNull);
    }
    get layoutable_views() {
      return this.child_views.filter((H) => H instanceof xh);
    }
    async build_child_views() {
      const { created: H, removed: V } = await build_views(this._child_views, this.child_models, { parent: this });
      for (const q of V)
        this._resize_observer.unobserve(q.el);
      for (const q of H)
        this._resize_observer.observe(q.el, { box: "border-box" });
      return H;
    }
    render() {
      super.render();
      for (const H of this.child_views)
        H.render(), this.shadow_el.appendChild(H.el);
    }
    _update_children() {
    }
    async update_children() {
      const H = await this.build_child_views(), V = new Set(H);
      for (const q of this.child_views)
        q.el.remove();
      for (const q of this.child_views)
        V.has(q) ? q.render_to(this.shadow_el) : this.shadow_el.append(q.el);
      this._update_children(), this.invalidate_layout();
    }
    _intrinsic_display() {
      return { inner: this.model.flow_mode, outer: "flow" };
    }
    _update_layout() {
      function H(ge, ve, be, Ce) {
        switch (ge) {
          case "auto":
            return ve != null ? px$1(ve) : be;
          case "fixed":
            return ve != null ? px$1(ve) : "fit-content";
          case "fit":
            return "fit-content";
          case "min":
            return "min-content";
          case "max":
            return Ce == null ? "100%" : `calc(100% - ${Ce})`;
        }
      }
      function V(ge) {
        const { inner: ve, outer: be } = ge;
        switch (`${ve} ${be}`) {
          case "block flow":
            return "block";
          case "inline flow":
            return "inline";
          case "block flow-root":
            return "flow-root";
          case "inline flow-root":
            return "inline-block";
          case "block flex":
            return "flex";
          case "inline flex":
            return "inline-flex";
          case "block grid":
            return "grid";
          case "inline grid":
            return "inline-grid";
          case "block table":
            return "table";
          case "inline table":
            return "inline-table";
          default:
            unreachable();
        }
      }
      function q(ge) {
        return isNumber$3(ge) ? px$1(ge) : `${ge.percent}%`;
      }
      const G = {}, J = this._intrinsic_display();
      G.display = V(J);
      const Z = this.box_sizing(), { width_policy: te, height_policy: ne, width: ee, height: ae, aspect_ratio: oe } = Z, se = (() => {
        if (oe == "auto") {
          if (ee != null && ae != null)
            return ee / ae;
        } else if (isNumber$3(oe))
          return oe;
        return null;
      })();
      oe == "auto" ? ee != null && ae != null ? G.aspect_ratio = `${ee} / ${ae}` : G.aspect_ratio = "auto" : isNumber$3(oe) && (G.aspect_ratio = `${oe}`);
      const { margin: ce } = this.model, le = (() => {
        if (ce != null) {
          if (isNumber$3(ce))
            return G.margin = px$1(ce), { width: px$1(2 * ce), height: px$1(2 * ce) };
          if (ce.length == 2) {
            const [ge, ve] = ce;
            return G.margin = `${px$1(ge)} ${px$1(ve)}`, { width: px$1(2 * ve), height: px$1(2 * ge) };
          } else {
            const [ge, ve, be, Ce] = ce;
            return G.margin = `${px$1(ge)} ${px$1(ve)} ${px$1(be)} ${px$1(Ce)}`, { width: px$1(Ce + ve), height: px$1(ge + be) };
          }
        } else
          return { width: null, height: null };
      })(), [ue, de] = (() => {
        const ge = H(te, ee, this._auto_width, le.width), ve = H(ne, ae, this._auto_height, le.height);
        if (oe != null) {
          if (te != ne)
            return te == "fixed" ? [ge, "auto"] : ne == "fixed" ? ["auto", ve] : te == "max" ? [ge, "auto"] : ne == "max" ? ["auto", ve] : ["auto", "auto"];
          if (te != "fixed" && ne != "fixed" && se != null)
            return se >= 1 ? [ge, "auto"] : ["auto", ve];
        }
        return [ge, ve];
      })();
      G.width = ue, G.height = de;
      const { min_width: fe, max_width: he } = this.model, { min_height: me, max_height: pe } = this.model;
      G.min_width = fe == null ? "0px" : q(fe), G.min_height = me == null ? "0px" : q(me), this.is_layout_root ? (he != null && (G.max_width = q(he)), pe != null && (G.max_height = q(pe))) : (he != null ? G.max_width = `min(${q(he)}, 100%)` : te != "fixed" && (G.max_width = "100%"), pe != null ? G.max_height = `min(${q(pe)}, 100%)` : ne != "fixed" && (G.max_height = "100%"));
      const { resizable: ye } = this.model;
      if (ye !== !1) {
        const ge = (() => {
          switch (ye) {
            case "width":
              return "horizontal";
            case "height":
              return "vertical";
            case !0:
            case "both":
              return "both";
          }
        })();
        G.resize = ge, G.overflow = "auto";
      }
      this.style.append(":host", G);
    }
    update_layout() {
      this.update_style();
      for (const H of this.layoutable_views)
        H.update_layout();
      this._update_layout();
    }
    get is_managed() {
      return this.parent instanceof xh;
    }
    /**
     * Update CSS layout with computed values from canvas layout.
     * This can be done more frequently than `_update_layout()`.
     */
    _measure_layout() {
    }
    measure_layout() {
      for (const H of this.layoutable_views)
        H.measure_layout();
      this._measure_layout();
    }
    compute_layout() {
      this.parent instanceof xh ? this.parent.compute_layout() : (this.measure_layout(), this.update_bbox(), this._compute_layout(), this.after_layout()), this._layout_computed = !0;
    }
    _compute_layout() {
      if (this.layout != null) {
        this.layout.compute(this.bbox.size);
        for (const H of this.layoutable_views)
          H.layout == null ? H._compute_layout() : H._propagate_layout();
      } else
        for (const H of this.layoutable_views)
          H._compute_layout();
    }
    _propagate_layout() {
      for (const H of this.layoutable_views)
        H.layout == null && H._compute_layout();
    }
    update_bbox() {
      for (const V of this.layoutable_views)
        V.update_bbox();
      const H = super.update_bbox();
      return this.layout != null && (this.layout.visible = this.is_displayed), H;
    }
    _after_layout() {
    }
    after_layout() {
      for (const H of this.layoutable_views)
        H.after_layout();
      this._after_layout();
    }
    render_to(H) {
      H != null && H.appendChild(this.el), this.render(), this.r_after_render(), this._was_built = !0, this.notify_finished();
    }
    r_after_render() {
      for (const H of this.child_views)
        H instanceof xh ? H.r_after_render() : H.after_render();
      this.after_render();
    }
    after_render() {
      this._after_render(), this.is_managed || this.invalidate_layout(), this._has_finished || (this.is_displayed ? defer().then(() => {
        this._resized || this.finish();
      }) : this.finish());
    }
    invalidate_layout() {
      this.parent instanceof xh ? this.parent.invalidate_layout() : (this.update_layout(), this.compute_layout());
    }
    invalidate_render() {
      this.render(), this.invalidate_layout();
    }
    has_finished() {
      if (!super.has_finished() || this.is_layout_root && !this._layout_computed)
        return !1;
      for (const H of this.child_views)
        if (!H.has_finished())
          return !1;
      return !0;
    }
    box_sizing() {
      let { width_policy: H, height_policy: V, aspect_ratio: q } = this.model;
      const { sizing_mode: G } = this.model;
      if (G != null)
        if (G == "inherit") {
          if (this.parent instanceof xh) {
            const ee = this.parent.box_sizing();
            H = ee.width_policy, V = ee.height_policy, q == null && (q = ee.aspect_ratio);
          }
        } else if (G == "fixed")
          H = V = "fixed";
        else if (G == "stretch_both")
          H = V = "max";
        else if (G == "stretch_width")
          H = "max";
        else if (G == "stretch_height")
          V = "max";
        else
          switch (q == null && (q = "auto"), G) {
            case "scale_width":
              H = "max", V = "min";
              break;
            case "scale_height":
              H = "min", V = "max";
              break;
            case "scale_both":
              H = "max", V = "max";
              break;
          }
      const [J, Z] = (() => {
        const { align: ee } = this.model;
        return ee == "auto" ? [void 0, void 0] : isArray$3(ee) ? ee : [ee, ee];
      })(), { width: te, height: ne } = this.model;
      return {
        width_policy: H,
        height_policy: V,
        width: te,
        height: ne,
        aspect_ratio: q,
        halign: J,
        valign: Z
      };
    }
    export(H = "auto", V = !0) {
      const q = (() => {
        switch (H) {
          case "auto":
          // TODO: actually infer the best type
          case "png":
            return "canvas";
          case "svg":
            return "svg";
        }
      })(), G = new CanvasLayer(q, V), { x: J, y: Z, width: te, height: ne } = this.bbox;
      G.resize(te, ne);
      const ee = getComputedStyle(this.el).backgroundColor;
      G.ctx.fillStyle = ee, G.ctx.fillRect(J, Z, te, ne);
      for (const ae of this.child_views) {
        const oe = ae.export(H, V), { x: se, y: ce } = ae.bbox.scale(G.pixel_ratio);
        G.ctx.drawImage(oe.canvas, se, ce);
      }
      return G;
    }
    serializable_state() {
      return {
        ...super.serializable_state(),
        children: this.child_views.map((H) => H.serializable_state())
      };
    }
  };
  mt(xh, "__name__", "LayoutDOMView");
  let LayoutDOMView = xh;
  const q5 = class q5 extends Pane {
    constructor(_) {
      super(_);
    }
  };
  mt(q5, "__name__", "LayoutDOM"), q5.define((_) => {
    const { Bool: H, Float: V, Auto: q, Tuple: G, Or: J, Null: Z, Nullable: te } = _, ne = G(V, V), ee = G(V, V, V, V);
    return {
      width: [te(V), null],
      height: [te(V), null],
      min_width: [te(V), null],
      min_height: [te(V), null],
      max_width: [te(V), null],
      max_height: [te(V), null],
      margin: [te(J(V, ne, ee)), null],
      width_policy: [J(SizingPolicy, q), "auto"],
      height_policy: [J(SizingPolicy, q), "auto"],
      aspect_ratio: [J(V, q, Z), null],
      flow_mode: [FlowMode, "block"],
      sizing_mode: [te(SizingMode), null],
      disabled: [H, !1],
      align: [J(Align, G(Align, Align), q), "auto"],
      resizable: [J(H, Dimensions), !1]
    };
  });
  let LayoutDOM = q5;
  const { max: max$7 } = Math;
  class GridAlignmentLayout extends Layoutable {
    constructor(H) {
      super();
      mt(this, "children");
      this.children = H;
    }
    _measure(H) {
      return { width: 0, height: 0 };
    }
    compute(H = {}) {
      const { width: V, height: q } = H;
      assert$2(V != null && q != null);
      const G = { width: V, height: q }, J = new BBox$1({ left: 0, top: 0, width: V, height: q });
      let Z;
      if (G.inner != null) {
        const { left: te, top: ne, right: ee, bottom: ae } = G.inner;
        Z = new BBox$1({ left: te, top: ne, right: V - ee, bottom: q - ae });
      }
      this.set_geometry(J, Z);
    }
    _set_geometry(H, V) {
      super._set_geometry(H, V);
      const q = this.children.map((Z, te) => {
        const { layout: ne, bbox: ee } = te;
        assert$2(ne != null);
        const ae = ne.measure(ee);
        return { child: te, layout: ne, bbox: ee, size_hint: ae };
      }), G = Array(q.nrows).fill(null).map(() => ({ top: 0, bottom: 0 })), J = Array(q.ncols).fill(null).map(() => ({ left: 0, right: 0 }));
      q.foreach(({ r0: Z, c0: te, r1: ne, c1: ee }, { size_hint: ae }) => {
        const { inner: oe } = ae;
        oe != null && (J[te].left = max$7(J[te].left, oe.left), J[ee].right = max$7(J[ee].right, oe.right), G[Z].top = max$7(G[Z].top, oe.top), G[ne].bottom = max$7(G[ne].bottom, oe.bottom));
      }), q.foreach(({ r0: Z, c0: te, r1: ne, c1: ee }, { layout: ae, size_hint: oe, bbox: se }) => {
        const ce = se, le = oe.inner == null ? void 0 : (() => {
          const { inner: ue, align: de } = oe, fe = (de == null ? void 0 : de.left) ?? !0, he = (de == null ? void 0 : de.right) ?? !0, me = (de == null ? void 0 : de.top) ?? !0, pe = (de == null ? void 0 : de.bottom) ?? !0, ye = (de == null ? void 0 : de.fixed_width) ?? !1, ge = (de == null ? void 0 : de.fixed_height) ?? !1, { left: ve, right: be } = (() => {
            if (ye) {
              const xe = ce.width - ue.right - ue.left;
              if (fe) {
                const Le = J[te].left, Qe = ce.width - (Le + xe);
                return { left: Le, right: Qe };
              } else if (he) {
                const Le = J[ee].right;
                return { left: ce.width - (Le + xe), right: Le };
              } else {
                const Le = ue.left, Qe = ue.right;
                return { left: Le, right: Qe };
              }
            } else {
              const xe = fe ? J[te].left : ue.left, Le = he ? J[ee].right : ue.right;
              return { left: xe, right: Le };
            }
          })(), { top: Ce, bottom: Se } = (() => {
            if (ge) {
              const xe = ce.height - ue.bottom - ue.top;
              if (me) {
                const Le = G[Z].top, Qe = ce.height - (Le + xe);
                return { top: Le, bottom: Qe };
              } else if (pe) {
                const Le = G[ne].bottom;
                return { top: ce.height - (Le + xe), bottom: Le };
              } else {
                const Le = ue.top, Qe = ue.bottom;
                return { top: Le, bottom: Qe };
              }
            } else {
              const xe = me ? G[Z].top : ue.top, Le = pe ? G[ne].bottom : ue.bottom;
              return { top: xe, bottom: Le };
            }
          })(), { width: we, height: Ee } = ce;
          return BBox$1.from_lrtb({ left: ve, top: Ce, right: we - be, bottom: Ee - Se });
        })();
        ae.set_geometry(ce, le);
      });
    }
  }
  mt(GridAlignmentLayout, "__name__", "GridAlignmentLayout");
  class FlexBoxView extends LayoutDOMView {
    connect_signals() {
      super.connect_signals();
      const { children: _ } = this.model.properties;
      this.on_change(_, () => this.update_children());
    }
    get child_models() {
      return this.model.children;
    }
    _intrinsic_display() {
      return { inner: this.model.flow_mode, outer: "flex" };
    }
    _update_layout() {
      super._update_layout(), this.style.append(":host", {
        flex_direction: this._direction,
        gap: px$1(this.model.spacing)
      });
      const _ = new Container();
      let H = 0, V = 0;
      for (const q of this.child_views) {
        if (!(q instanceof LayoutDOMView))
          continue;
        const G = q.box_sizing(), J = (() => {
          const te = this._direction == "row" ? G.width_policy : G.height_policy, ne = this._direction == "row" ? G.width : G.height, ee = ne != null ? px$1(ne) : "auto";
          switch (te) {
            case "auto":
            case "fixed":
              return `0 0 ${ee}`;
            case "fit":
              return "1 1 auto";
            case "min":
              return "0 1 auto";
            case "max":
              return "1 0 0px";
          }
        })(), Z = (() => {
          switch (this._direction == "row" ? G.height_policy : G.width_policy) {
            case "auto":
            case "fixed":
            case "fit":
            case "min":
              return this._direction == "row" ? G.valign : G.halign;
            case "max":
              return "stretch";
          }
        })();
        q.style.append(":host", { flex: J, align_self: Z }), this._direction == "row" ? G.height_policy == "max" && q.style.append(":host", { height: "auto" }) : G.width_policy == "max" && q.style.append(":host", { width: "auto" }), q.layout != null && (_.add({ r0: H, c0: V, r1: H + 1, c1: V + 1 }, q), this._direction == "row" ? V += 1 : H += 1);
      }
      _.size != 0 ? (this.layout = new GridAlignmentLayout(_), this.layout.set_sizing()) : delete this.layout;
    }
  }
  mt(FlexBoxView, "__name__", "FlexBoxView");
  const j5 = class j5 extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(j5, "__name__", "FlexBox"), j5.define(({ Float: _, List: H, Ref: V }) => ({
    children: [H(V(UIElement)), []],
    spacing: [_, 0]
  }));
  let FlexBox = j5;
  class ColumnView extends FlexBoxView {
    constructor() {
      super(...arguments);
      mt(this, "_direction", "column");
    }
  }
  mt(ColumnView, "__name__", "ColumnView");
  var I0;
  let Column$1 = (I0 = class extends FlexBox {
    constructor(_) {
      super(_);
    }
  }, mt(I0, "__name__", "Column"), I0.prototype.default_view = ColumnView, I0);
  const { max: max$6 } = Math;
  class CSSGridBoxView extends LayoutDOMView {
    connect_signals() {
      super.connect_signals();
      const { spacing: _ } = this.model.properties;
      this.on_change(_, () => this.invalidate_layout());
    }
    get child_models() {
      return this._children.map(([_]) => _);
    }
    _intrinsic_display() {
      return { inner: this.model.flow_mode, outer: "grid" };
    }
    _update_layout() {
      super._update_layout();
      const _ = {}, [H, V] = (() => {
        const { spacing: se } = this.model;
        return isNumber$3(se) ? [se, se] : se;
      })();
      _.row_gap = px$1(H), _.column_gap = px$1(V);
      let q = 0, G = 0;
      const J = new Container();
      for (const [[, se, ce, le = 1, ue = 1], de] of enumerate(this._children)) {
        const fe = this.child_views[de];
        q = max$6(q, se + le), G = max$6(G, ce + ue);
        const he = {};
        if (he.grid_row_start = `${se + 1}`, he.grid_row_end = `span ${le}`, he.grid_column_start = `${ce + 1}`, he.grid_column_end = `span ${ue}`, fe.style.append(":host", he), fe instanceof LayoutDOMView && fe.layout != null) {
          const me = se, pe = ce, ye = se + le - 1, ge = ce + ue - 1;
          J.add({ r0: me, c0: pe, r1: ye, c1: ge }, fe);
        }
      }
      const { _rows: Z, _cols: te } = this;
      Z instanceof Map ? q = max$6(q, ...Z.keys()) : isArray$3(Z) && (q = max$6(q, Z.length)), te instanceof Map ? G = max$6(G, ...te.keys()) : isArray$3(te) && (G = max$6(G, te.length));
      function ne(se, ce) {
        if (se instanceof Map)
          for (const [le, ue] of se.entries())
            isString$2(ue) ? ce[le].size = ue : ce[le] = ue;
        else if (isArray$3(se))
          for (const [le, ue] of enumerate(se))
            isString$2(le) ? ce[ue].size = le : ce[ue] = le;
        else if (se != null)
          for (const le of ce)
            isString$2(se) ? le.size = se : (le.size = se.size, le.align = se.align);
      }
      const ee = Array(q).fill(null).map(() => ({})), ae = Array(G).fill(null).map(() => ({}));
      ne(Z, ee), ne(te, ae);
      for (const [[, se, ce], le] of enumerate(this._children)) {
        const ue = this.child_views[le], { halign: de, valign: fe } = ue.box_sizing();
        ue.style.append(":host", {
          justify_self: de ?? ae[ce].align,
          align_self: fe ?? ee[se].align
        });
      }
      const oe = "auto";
      _.grid_template_rows = ee.map(({ size: se }) => se ?? oe).join(" "), _.grid_template_columns = ae.map(({ size: se }) => se ?? oe).join(" "), this.style.append(":host", _), J.size != 0 ? (this.layout = new GridAlignmentLayout(J), this.layout.set_sizing()) : delete this.layout;
    }
  }
  mt(CSSGridBoxView, "__name__", "CSSGridBoxView");
  const G5 = class G5 extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(G5, "__name__", "CSSGridBox"), G5.define(() => ({
    spacing: [GridSpacing, 0]
  }));
  let CSSGridBox = G5;
  class GridBoxView extends CSSGridBoxView {
    connect_signals() {
      super.connect_signals();
      const { children: _, rows: H, cols: V } = this.model.properties;
      this.on_change(_, () => this.update_children()), this.on_change([H, V], () => this.invalidate_layout());
    }
    get _children() {
      return this.model.children;
    }
    get _rows() {
      return this.model.rows;
    }
    get _cols() {
      return this.model.cols;
    }
  }
  mt(GridBoxView, "__name__", "GridBoxView");
  const mm = class mm extends CSSGridBox {
    constructor(_) {
      super(_);
    }
  };
  mt(mm, "__name__", "GridBox"), mm.prototype.default_view = GridBoxView, mm.define(({ List: _, Nullable: H }) => ({
    children: [_(GridChild(UIElement)), []],
    rows: [H(TracksSizing), null],
    cols: [H(TracksSizing), null]
  }));
  let GridBox = mm;
  const group_box_css = "legend{display:flex;gap:0.5em;padding:0 calc(var(--padding-horizontal) / 2);}fieldset{border:1px solid #ccc;}";
  class GroupBoxView extends LayoutDOMView {
    constructor() {
      super(...arguments);
      mt(this, "checkbox_el");
      mt(this, "fieldset_el");
    }
    stylesheets() {
      return [...super.stylesheets(), group_box_css];
    }
    connect_signals() {
      super.connect_signals();
      const { child: H } = this.model.properties;
      this.on_change(H, () => this.update_children());
      const { checkable: V, disabled: q } = this.model.properties;
      this.on_change(V, () => {
        display(this.checkbox_el, this.model.checkable);
      }), this.on_change(q, () => {
        this.checkbox_el.checked = !this.model.disabled;
      });
    }
    get child_models() {
      return [this.model.child];
    }
    render() {
      super.render();
      const { checkable: H, disabled: V, title: q } = this.model;
      this.checkbox_el = input$1({ type: "checkbox", checked: !V }), this.checkbox_el.addEventListener("change", () => {
        this.model.disabled = !this.checkbox_el.checked;
      }), display(this.checkbox_el, H);
      const G = legend({}, this.checkbox_el, q), J = this.child_views.map((Z) => Z.el);
      this.fieldset_el = fieldset({}, G, ...J), this.shadow_el.appendChild(this.fieldset_el);
    }
    _update_children() {
      const H = this.child_views.map((V) => V.el);
      this.fieldset_el.append(...H);
    }
  }
  mt(GroupBoxView, "__name__", "GroupBoxView");
  const gm = class gm extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(gm, "__name__", "GroupBox"), gm.prototype.default_view = GroupBoxView, gm.define(({ Bool: _, Str: H, Nullable: V, Ref: q }) => ({
    title: [V(H), null],
    child: [q(UIElement)],
    checkable: [_, !1]
  }));
  let GroupBox = gm;
  const HBoxChild = Struct$1({ child: Ref(UIElement), col: Opt(Index$2), span: Opt(Span$4) });
  class HBoxView extends CSSGridBoxView {
    connect_signals() {
      super.connect_signals();
      const { children: _, cols: H } = this.model.properties;
      this.on_change(_, () => this.update_children()), this.on_change(H, () => this.invalidate_layout());
    }
    get _children() {
      return this.model.children.map(({ child: _, col: H, span: V }, q) => [_, 0, H ?? q, 1, V ?? 1]);
    }
    get _rows() {
      return null;
    }
    get _cols() {
      return this.model.cols;
    }
  }
  mt(HBoxView, "__name__", "HBoxView");
  const ym = class ym extends CSSGridBox {
    constructor(_) {
      super(_);
    }
  };
  mt(ym, "__name__", "HBox"), ym.prototype.default_view = HBoxView, ym.define(({ List: _, Nullable: H }) => ({
    children: [_(HBoxChild), []],
    cols: [H(TracksSizing), null]
  }));
  let HBox = ym;
  class RowView extends FlexBoxView {
    constructor() {
      super(...arguments);
      mt(this, "_direction", "row");
    }
  }
  mt(RowView, "__name__", "RowView");
  var D0;
  let Row$1 = (D0 = class extends FlexBox {
    constructor(_) {
      super(_);
    }
  }, mt(D0, "__name__", "Row"), D0.prototype.default_view = RowView, D0);
  class ScrollBoxView extends LayoutDOMView {
    stylesheets() {
      return [...super.stylesheets()];
    }
    connect_signals() {
      super.connect_signals();
      const { child: _, horizontal_scrollbar: H, vertical_scrollbar: V } = this.model.properties;
      this.on_change(_, () => this.update_children()), this.on_change([H, V], () => this.invalidate_layout());
    }
    get child_models() {
      return [this.model.child];
    }
    _update_layout() {
      super._update_layout();
      function _(q) {
        switch (q) {
          case "auto":
            return "auto";
          case "visible":
            return "scroll";
          case "hidden":
            return "hidden";
        }
      }
      const { horizontal_scrollbar: H, vertical_scrollbar: V } = this.model;
      this.style.append(":host", {
        overflow_x: _(H),
        overflow_y: _(V)
      });
    }
  }
  mt(ScrollBoxView, "__name__", "ScrollBoxView");
  const vm = class vm extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(vm, "__name__", "ScrollBox"), vm.prototype.default_view = ScrollBoxView, vm.define(({ Ref: _ }) => ({
    child: [_(UIElement)],
    horizontal_scrollbar: [ScrollbarPolicy, "auto"],
    vertical_scrollbar: [ScrollbarPolicy, "auto"]
  }));
  let ScrollBox = vm;
  class SpacerView extends LayoutDOMView {
    constructor() {
      super(...arguments);
      mt(this, "_auto_width", "auto");
      mt(this, "_auto_height", "auto");
    }
    get child_models() {
      return [];
    }
  }
  mt(SpacerView, "__name__", "SpacerView");
  const W5 = class W5 extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(W5, "__name__", "Spacer"), W5.prototype.default_view = SpacerView;
  let Spacer = W5;
  const Y5 = class Y5 extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Y5, "__name__", "Selector"), Y5.define(({ Str: _ }) => ({
    query: [_]
  }));
  let Selector = Y5;
  const non_interactive = "bk-non-interactive", arrow = "bk-arrow", show_arrow = "bk-show-arrow", left$1 = "bk-left", right$1 = "bk-right", above$1 = "bk-above", below$2 = "bk-below", tooltip_content = "bk-tooltip-content", tooltip_row_label = "bk-tooltip-row-label", tooltip_row_value = "bk-tooltip-row-value", tooltip_color_block = "bk-tooltip-color-block", closable = "bk-closable", close$2 = "bk-close", tooltips_css = ':host{--tooltip-border:#e5e5e5;--tooltip-color:white;--tooltip-text:#2f2f2f;--tooltip-arrow-color:#909599;--tooltip-arrow-width:10px;--tooltip-arrow-height:10px;--tooltip-arrow-half-width:7px;--tooltip-arrow-half-height:7px;}:host{width:max-content;font-weight:300;font-size:var(--font-size);position:fixed;padding:5px;border:1px solid var(--tooltip-border);color:var(--tooltip-text);background-color:var(--tooltip-color);opacity:0.95;z-index:var(--bokeh-top-level);}:host(.bk-non-interactive){pointer-events:none;}.bk-arrow{pointer-events:none;position:fixed;width:0;height:0;content:" ";border-style:solid;border-color:transparent;}:host(:not(.bk-show-arrow)) .bk-arrow{display:none;}:host(.bk-left) .bk-arrow,:host(.bk-right) .bk-arrow{border-width:var(--tooltip-arrow-half-height) 0 var(--tooltip-arrow-half-height) 0;}:host(.bk-above) .bk-arrow,:host(.bk-below) .bk-arrow{border-width:0 var(--tooltip-arrow-half-width) 0 var(--tooltip-arrow-half-width);}:host(.bk-left) .bk-arrow{transform:translate(0%, -50%);border-right-width:var(--tooltip-arrow-width);border-right-color:var(--tooltip-arrow-color);}:host(.bk-right) .bk-arrow{transform:translate(-100%, -50%);border-left-width:var(--tooltip-arrow-width);border-left-color:var(--tooltip-arrow-color);}:host(.bk-above) .bk-arrow{transform:translate(-50%, 0%);border-bottom-width:var(--tooltip-arrow-height);border-bottom-color:var(--tooltip-arrow-color);}:host(.bk-below) .bk-arrow{transform:translate(-50%, -100%);border-top-width:var(--tooltip-arrow-height);border-top-color:var(--tooltip-arrow-color);}.bk-tooltip-content > div:not(:first-child){margin-top:5px;border-top:var(--tooltip-border) 1px dashed;}.bk-tooltip-row-label{text-align:right;color:#26aae1;}.bk-tooltip-row-value{color:none;}.bk-tooltip-color-block{width:12px;height:12px;margin-left:5px;margin-right:5px;outline:#dddddd solid 1px;display:inline-block;}:host(:not(.bk-closable)) .bk-close{display:none;}.bk-close{position:absolute;top:2px;right:2px;width:12px;height:12px;cursor:pointer;background-color:gray;mask-image:var(--bokeh-icon-x);mask-size:contain;mask-repeat:no-repeat;-webkit-mask-image:var(--bokeh-icon-x);-webkit-mask-size:contain;-webkit-mask-repeat:no-repeat;}.bk-close:hover{background-color:red;}', NativeNode = globalThis.Node;
  class TooltipView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "arrow_el");
      mt(this, "content_el");
      mt(this, "_observer");
      mt(this, "_target");
      mt(this, "_element_view", null);
      mt(this, "_scroll_listener");
    }
    get target() {
      return this._target;
    }
    set target(H) {
      this._target = H;
    }
    _init_target() {
      const { target: H } = this.model, V = (() => {
        var q;
        if (H instanceof UIElement)
          return ((q = this.owner.find_one(H)) == null ? void 0 : q.el) ?? null;
        if (H instanceof Selector)
          return H.find_one(document);
        if (H instanceof NativeNode)
          return H;
        {
          const { parent: G } = this;
          return G instanceof DOMElementView$1 ? G.el : null;
        }
      })();
      V instanceof Element ? this._target = V : (logger$1.warn(`unable to resolve target '${H}' for '${this}'`), this._target = document.body);
    }
    initialize() {
      super.initialize(), this._init_target();
    }
    *children() {
      yield* super.children(), this._element_view != null && (yield this._element_view);
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { content: H } = this.model;
      H instanceof Model$1 && (this._element_view = await build_view(H, { parent: this })), this.render();
    }
    connect_signals() {
      super.connect_signals(), this._observer = new ResizeObserver(() => {
        this._reposition();
      }), this._observer.observe(this.target);
      let H = !1;
      document.addEventListener("scroll", this._scroll_listener = () => {
        H || (requestAnimationFrame(() => {
          this._reposition(), H = !1;
        }), H = !0);
      }, { capture: !0 });
      const { target: V, content: q, closable: G, interactive: J, position: Z, attachment: te, visible: ne } = this.model.properties;
      this.on_change(V, () => {
        this._init_target(), this._observer.disconnect(), this._observer.observe(this.target), this.render();
      }), this.on_change([q, G, J], () => this.render()), this.on_change([Z, te, ne], () => this._reposition());
    }
    disconnect_signals() {
      this._scroll_listener != null && (document.removeEventListener("scroll", this._scroll_listener, { capture: !0 }), delete this._scroll_listener), super.disconnect_signals();
    }
    remove() {
      var H;
      (H = this._element_view) == null || H.remove(), this._observer.disconnect(), super.remove();
    }
    stylesheets() {
      return [...super.stylesheets(), tooltips_css, icons_css];
    }
    get content() {
      const { content: H } = this.model;
      return isString$2(H) ? document.createTextNode(H) : H instanceof Model$1 ? (assert$2(this._element_view != null), this._element_view.el) : H;
    }
    render() {
      var V;
      super.render(), (V = this._element_view) == null || V.render_to(null), this.arrow_el = div({ class: [arrow] }), this.content_el = div({ class: tooltip_content }, this.content), this.shadow_el.append(this.arrow_el, this.content_el), this.class_list.toggle(closable, this.model.closable);
      const H = div({ class: close$2 });
      this.shadow_el.append(H), H.addEventListener("click", () => {
        this.model.visible = !1;
      }), this.el.classList.toggle(show_arrow, this.model.show_arrow), this.el.classList.toggle(non_interactive, !this.model.interactive), this._reposition();
    }
    _anchor_to_align(H) {
      switch (H) {
        case "top_left":
          return ["top", "left"];
        case "top":
        case "top_center":
          return ["top", "center"];
        case "top_right":
          return ["top", "right"];
        case "left":
        case "center_left":
          return ["center", "left"];
        case "center":
        case "center_center":
          return ["center", "center"];
        case "right":
        case "center_right":
          return ["center", "right"];
        case "bottom_left":
          return ["bottom", "left"];
        case "bottom":
        case "bottom_center":
          return ["bottom", "center"];
        case "bottom_right":
          return ["bottom", "right"];
      }
    }
    _reposition() {
      const { position: H, visible: V } = this.model;
      if (H == null || !V) {
        this.el.remove();
        return;
      }
      (document.body.shadowRoot ?? document.body).append(this.el);
      const q = bounding_box(this.target), [G, J] = (() => {
        if (isString$2(H)) {
          const [oe, se] = this._anchor_to_align(H), ce = (() => {
            switch (se) {
              case "left":
                return q.left;
              case "center":
                return q.hcenter;
              case "right":
                return q.right;
            }
          })(), le = (() => {
            switch (oe) {
              case "top":
                return q.top;
              case "center":
                return q.vcenter;
              case "bottom":
                return q.bottom;
            }
          })();
          return [ce, le];
        } else if (isArray$3(H)) {
          const [oe, se] = H;
          return [q.left + oe, q.top + se];
        } else {
          const { x: oe, y: se } = this.resolve_as_xy(H);
          return [q.left + oe, q.top + se];
        }
      })(), Z = new BBox$1({
        x: 0,
        y: 0,
        width: window.innerWidth,
        height: window.innerHeight
      }), te = box_size(this.arrow_el), ne = (() => {
        const oe = (() => {
          const { attachment: ue } = this.model;
          if (ue == "auto") {
            if (isString$2(H)) {
              const [de, fe] = this._anchor_to_align(H);
              if (fe != "center")
                return fe == "left" ? "left" : "right";
              if (de != "center")
                return de == "top" ? "above" : "below";
            }
            return "horizontal";
          } else
            return ue;
        })(), se = box_size(this.el), ce = se.width + te.width, le = se.height + te.height;
        switch (oe) {
          case "horizontal":
            return G < q.hcenter ? G + ce <= Z.right ? "right" : "left" : G - ce >= Z.left ? "left" : "right";
          case "vertical":
            return J < q.vcenter ? J + le <= Z.bottom ? "below" : "above" : J - le >= Z.top ? "above" : "below";
          default:
            return oe;
        }
      })();
      this.class_list.remove(right$1, left$1, above$1, below$2), this.class_list.add((() => {
        switch (ne) {
          case "left":
            return right$1;
          case "right":
            return left$1;
          case "above":
            return below$2;
          case "below":
            return above$1;
        }
      })()), this.arrow_el.style.left = `${G}px`, this.arrow_el.style.top = `${J}px`;
      const { left: ee, top: ae } = (() => {
        const { width: oe, height: se } = box_size(this.el);
        function ce(ue) {
          return ue < Z.top ? Z.top : ue + se > Z.bottom ? Z.bottom - se : ue;
        }
        function le(ue) {
          return ue < Z.left ? Z.left : ue + oe > Z.right ? Z.right - oe : ue;
        }
        switch (ne) {
          case "left":
            return {
              left: G - oe - te.width,
              top: ce(J - se / 2)
            };
          case "right":
            return {
              left: G + te.width,
              top: ce(J - se / 2)
            };
          case "above":
            return {
              left: le(G - oe / 2),
              top: J - se - te.height
            };
          case "below":
            return {
              left: le(G - oe / 2),
              top: J + te.height
            };
        }
      })();
      this.el.style.top = `${ae}px`, this.el.style.left = `${ee}px`;
    }
  }
  mt(TooltipView, "__name__", "TooltipView");
  const zp = class zp extends UIElement {
    constructor(_) {
      super(_);
    }
    show({ x: _, y: H }) {
      this.setv({ position: [_, H], visible: !0 }, { check_eq: !1 });
    }
    clear() {
      this.position = null;
    }
  };
  mt(zp, "__name__", "Tooltip"), zp.prototype.default_view = TooltipView, zp.define(({ Bool: _, Float: H, Str: V, Tuple: q, Or: G, Ref: J, Nullable: Z, Auto: te }) => ({
    target: [G(J(UIElement), J(Selector), J(NativeNode), te), "auto"],
    position: [Z(G(Anchor$2, q(H, H), J(Coordinate))), null],
    content: [G(V, J(DOMNode), J(UIElement), J(NativeNode))],
    attachment: [G(TooltipAttachment, te), "auto"],
    show_arrow: [_, !0],
    closable: [_, !1],
    interactive: [_, !0]
  })), zp.override({
    visible: !1
  });
  let Tooltip = zp;
  const X5 = class X5 extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(X5, "__name__", "TabPanel"), X5.define(({ Bool: _, Str: H, Ref: V, Nullable: q }) => ({
    title: [H, ""],
    tooltip: [q(V(Tooltip)), null],
    child: [V(UIElement)],
    closable: [_, !1],
    disabled: [_, !1]
  }));
  let TabPanel = X5;
  const above = "bk-above", below$1 = "bk-below", left = "bk-left", right = "bk-right", header = "bk-header", tab = "bk-tab", active$2 = "bk-active", close$1 = "bk-close", disabled$1 = "bk-disabled", tabs_css = ':host{display:grid;}:host(.bk-above){grid-template:"header" max-content "stack" 1fr / 1fr;}:host(.bk-below){grid-template:"stack" 1fr "header" max-content / 1fr;}:host(.bk-left){grid-template:"header stack" 1fr / max-content 1fr;}:host(.bk-right){grid-template:"stack header" 1fr / 1fr max-content;}.bk-header{grid-area:"header";display:flex;flex-wrap:nowrap;align-items:stretch;user-select:none;-webkit-user-select:none;}:host(.bk-above) .bk-header,:host(.bk-below) .bk-header{flex-direction:row;}:host(.bk-left) .bk-header,:host(.bk-right) .bk-header{flex-direction:column;}:host(.bk-above) .bk-header{border-bottom:1px solid #e6e6e6;}:host(.bk-right) .bk-header{border-left:1px solid #e6e6e6;}:host(.bk-below) .bk-header{border-top:1px solid #e6e6e6;}:host(.bk-left) .bk-header{border-right:1px solid #e6e6e6;}.bk-tab{padding:4px 8px;border:solid transparent;outline:0;outline-offset:-5px;white-space:nowrap;cursor:pointer;text-align:center;}.bk-tab:hover{background-color:#f2f2f2;}.bk-tab:focus,.bk-tab:active{outline:1px dotted #ccc;}.bk-tab.bk-active{color:#4d4d4d;background-color:white;border-color:#e6e6e6;}.bk-tab .bk-close{margin-left:10px;}.bk-tab.bk-disabled{cursor:not-allowed;pointer-events:none;opacity:0.65;}:host(.bk-above) .bk-tab{border-width:3px 1px 0px 1px;border-radius:var(--border-radius) var(--border-radius) 0 0;}:host(.bk-right) .bk-tab{border-width:1px 3px 1px 0px;border-radius:0 var(--border-radius) var(--border-radius) 0;}:host(.bk-below) .bk-tab{border-width:0px 1px 3px 1px;border-radius:0 0 var(--border-radius) var(--border-radius);}:host(.bk-left) .bk-tab{border-width:1px 0px 1px 3px;border-radius:var(--border-radius) 0 0 var(--border-radius);}.bk-close{display:inline-block;vertical-align:middle;width:14px;height:14px;cursor:pointer;background-color:gray;mask-image:var(--bokeh-icon-x);mask-size:contain;mask-repeat:no-repeat;-webkit-mask-image:var(--bokeh-icon-x);-webkit-mask-size:contain;-webkit-mask-repeat:no-repeat;}.bk-close:hover{background-color:red;}', tabs = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    above,
    active: active$2,
    below: below$1,
    close: close$1,
    default: tabs_css,
    disabled: disabled$1,
    header,
    left,
    right,
    tab
  }, Symbol.toStringTag, { value: "Module" }));
  class TabsView extends LayoutDOMView {
    constructor() {
      super(...arguments);
      mt(this, "tooltip_views", /* @__PURE__ */ new Map());
      mt(this, "header_el");
      mt(this, "header_els");
    }
    connect_signals() {
      super.connect_signals();
      const { tabs: H, active: V } = this.model.properties;
      this.on_change(H, async () => {
        this._update_headers(), await this.update_children();
      }), this.on_change(V, () => {
        this.update_active();
      });
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { tabs: H } = this.model, V = H.map((q) => q.tooltip).filter((q) => q != null);
      await build_views(this.tooltip_views, V, { parent: this });
    }
    stylesheets() {
      return [...super.stylesheets(), tabs_css, icons_css];
    }
    get child_models() {
      return this.model.tabs.map((H) => H.child);
    }
    _intrinsic_display() {
      return { inner: this.model.flow_mode, outer: "grid" };
    }
    _update_layout() {
      super._update_layout();
      const H = this.model.tabs_location;
      this.class_list.remove([...Location].map((q) => tabs[q])), this.class_list.add(tabs[H]);
      const V = new Container();
      for (const q of this.child_views)
        q.style.append(":host", { grid_area: "stack" }), q instanceof LayoutDOMView && q.layout != null && V.add({ r0: 0, c0: 0, r1: 1, c1: 1 }, q);
      V.size != 0 ? (this.layout = new GridAlignmentLayout(V), this.layout.set_sizing()) : delete this.layout;
    }
    _after_layout() {
      super._after_layout();
      const { child_views: H } = this;
      for (const q of H)
        hide(q.el);
      const { active: V } = this.model;
      if (V in H) {
        const q = H[V];
        show(q.el);
      }
    }
    render() {
      super.render(), this.header_el = div({ class: header }), this.shadow_el.append(this.header_el), this._update_headers();
    }
    _update_headers() {
      const { active: H } = this.model, V = this.model.tabs.map((q, G) => {
        const J = div({ class: [tab, G == H ? active$2 : null], tabIndex: 0 }, q.title);
        J.addEventListener("click", (te) => {
          this.model.disabled || te.target == te.currentTarget && this.change_active(G);
        });
        const Z = q.tooltip != null ? this.tooltip_views.get(q.tooltip) : null;
        if (Z != null) {
          Z.model.target = J;
          const te = (ne) => {
            Z.model.visible = ne;
          };
          J.addEventListener("mouseenter", () => {
            te(!0);
          }), J.addEventListener("mouseleave", () => {
            te(!1);
          });
        }
        if (q.closable) {
          const te = div({ class: close$1 });
          te.addEventListener("click", (ne) => {
            if (ne.target == ne.currentTarget) {
              this.model.tabs = remove_at(this.model.tabs, G);
              const ee = this.model.tabs.length;
              this.model.active > ee - 1 && (this.model.active = ee - 1);
            }
          }), J.appendChild(te);
        }
        return (this.model.disabled || q.disabled) && J.classList.add(disabled$1), J;
      });
      this.header_els = V, empty$1(this.header_el), this.header_el.append(...V);
    }
    change_active(H) {
      H != this.model.active && (this.model.active = H);
    }
    update_active() {
      const H = this.model.active, { header_els: V } = this;
      for (const G of V)
        G.classList.remove(active$2);
      H in V && V[H].classList.add(active$2);
      const { child_views: q } = this;
      for (const G of q)
        hide(G.el);
      H in q && show(q[H].el);
    }
  }
  mt(TabsView, "__name__", "TabsView");
  const bm = class bm extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(bm, "__name__", "Tabs"), bm.prototype.default_view = TabsView, bm.define(({ Int: _, List: H, Ref: V }) => ({
    tabs: [H(V(TabPanel)), []],
    tabs_location: [Location, "above"],
    active: [_, 0]
  }));
  let Tabs = bm;
  const VBoxChild = Struct$1({ child: Ref(UIElement), row: Opt(Index$2), span: Opt(Span$4) });
  class VBoxView extends CSSGridBoxView {
    connect_signals() {
      super.connect_signals();
      const { children: _, rows: H } = this.model.properties;
      this.on_change(_, () => this.update_children()), this.on_change(H, () => this.invalidate_layout());
    }
    get _children() {
      return this.model.children.map(({ child: _, row: H, span: V }, q) => [_, H ?? q, 0, V ?? 1, 1]);
    }
    get _rows() {
      return this.model.rows;
    }
    get _cols() {
      return null;
    }
  }
  mt(VBoxView, "__name__", "VBoxView");
  const Tm = class Tm extends CSSGridBox {
    constructor(_) {
      super(_);
    }
  };
  mt(Tm, "__name__", "VBox"), Tm.prototype.default_view = VBoxView, Tm.define(({ List: _, Nullable: H }) => ({
    children: [_(VBoxChild), []],
    rows: [H(TracksSizing), null]
  }));
  let VBox = Tm;
  const Z5 = class Z5 extends Transform$1 {
    constructor(_) {
      super(_);
    }
    get names() {
      return keys$5(this.args);
    }
    get values() {
      return values$3(this.args);
    }
    _make_transform(_, H) {
      return new Function(...this.names, _, use_strict(H));
    }
    get scalar_transform() {
      return this._make_transform("x", this.func);
    }
    get vector_transform() {
      return this._make_transform("xs", this.v_func);
    }
    compute(_) {
      return this.scalar_transform(...this.values, _);
    }
    v_compute(_) {
      return this.vector_transform(...this.values, _);
    }
  };
  mt(Z5, "__name__", "CustomJSTransform"), Z5.define(({ Unknown: _, Str: H, Dict: V }) => ({
    args: [V(_), {}],
    func: [H, ""],
    v_func: [H, ""]
  }));
  let CustomJSTransform = Z5;
  const K5 = class K5 extends Transform$1 {
    constructor(_) {
      super(_);
    }
    v_compute(_) {
      let H;
      this.range instanceof FactorRange ? H = this.range.v_synthetic(_) : isArrayableOf(_, isNumber$3) ? H = _ : unreachable();
      const V = new (infer_type(H))(H.length);
      for (let q = 0; q < H.length; q++) {
        const G = H[q];
        V[q] = this._compute(G);
      }
      return V;
    }
    compute(_) {
      if (this.range instanceof FactorRange)
        return this._compute(this.range.synthetic(_));
      if (isNumber$3(_))
        return this._compute(_);
      unreachable();
    }
  };
  mt(K5, "__name__", "RangeTransform"), K5.define(({ Ref: _, Nullable: H }) => ({
    range: [H(_(Range$2)), null]
  }));
  let RangeTransform = K5;
  const J5 = class J5 extends RangeTransform {
    constructor(_) {
      super(_);
    }
    _compute(_) {
      return _ + this.value;
    }
  };
  mt(J5, "__name__", "Dodge"), J5.define(({ Float: _ }) => ({
    value: [_, 0]
  }));
  let Dodge = J5;
  const e6 = class e6 extends Transform$1 {
    constructor(H) {
      super(H);
      mt(this, "_x_sorted");
      mt(this, "_y_sorted");
      mt(this, "_sorted_dirty", !0);
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.change, () => this._sorted_dirty = !0);
    }
    v_compute(H) {
      const V = infer_type(H), q = new V(H.length);
      for (let G = 0; G < H.length; G++) {
        const J = H[G];
        q[G] = this.compute(J);
      }
      return q;
    }
    sort(H = !1) {
      if (!this._sorted_dirty)
        return;
      let V, q;
      if (isString$2(this.x) && isString$2(this.y) && this.data != null) {
        const te = this.data.columns();
        if (!includes(te, this.x))
          throw new Error("The x parameter does not correspond to a valid column name defined in the data parameter");
        if (!includes(te, this.y))
          throw new Error("The y parameter does not correspond to a valid column name defined in the data parameter");
        V = this.data.get_column(this.x), q = this.data.get_column(this.y);
      } else if (isArray$3(this.x) && isArray$3(this.y))
        V = this.x, q = this.y;
      else
        throw new Error("parameters 'x' and 'y' must be both either string fields or arrays");
      if (V.length !== q.length)
        throw new Error("The length for x and y do not match");
      if (V.length < 2)
        throw new Error("x and y must have at least two elements to support interpolation");
      const G = V.length, J = new Uint32Array(G);
      for (let te = 0; te < G; te++)
        J[te] = te;
      const Z = H ? -1 : 1;
      J.sort((te, ne) => Z * (V[te] - V[ne])), this._x_sorted = new (infer_type(V))(G), this._y_sorted = new (infer_type(q))(G);
      for (let te = 0; te < G; te++)
        this._x_sorted[te] = V[J[te]], this._y_sorted[te] = q[J[te]];
      this._sorted_dirty = !1;
    }
  };
  mt(e6, "__name__", "Interpolator"), e6.define(({ Bool: H, Float: V, Str: q, Ref: G, List: J, Or: Z, Nullable: te }) => ({
    x: [Z(q, J(V))],
    y: [Z(q, J(V))],
    data: [te(G(ColumnarDataSource)), null],
    clip: [H, !0]
  }));
  let Interpolator = e6;
  class RandomGenerator extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(RandomGenerator, "__name__", "RandomGenerator");
  const xm = class xm extends RangeTransform {
    constructor(H) {
      super(H);
      mt(this, "_previous_offsets", null);
      mt(this, "_generator");
    }
    initialize() {
      var H;
      super.initialize(), this._generator = ((H = this.random_generator) == null ? void 0 : H.generator()) ?? new SystemRandom();
    }
    v_compute(H) {
      const V = this.range instanceof FactorRange ? this.range.v_synthetic(H) : H, q = (() => {
        var J;
        const G = V.length;
        return ((J = this._previous_offsets) == null ? void 0 : J.length) != G && (this._previous_offsets = this._v_compute(G)), this._previous_offsets;
      })();
      return map$4(q, (G, J) => G + V[J]);
    }
    _compute() {
      const { mean: H, width: V } = this;
      switch (this.distribution) {
        case "uniform":
          return this._generator.uniform(H, V);
        case "normal":
          return this._generator.normal(H, V);
      }
    }
    _v_compute(H) {
      const { mean: V, width: q } = this;
      switch (this.distribution) {
        case "uniform":
          return this._generator.uniforms(V, q, H);
        case "normal":
          return this._generator.normals(V, q, H);
      }
    }
  };
  mt(xm, "__name__", "Jitter"), xm.define(({ Float: H }) => ({
    mean: [H, 0],
    width: [H, 1],
    distribution: [Distribution, "uniform"]
  })), xm.internal(({ Nullable: H, Ref: V }) => ({
    random_generator: [H(V(RandomGenerator)), null]
  }));
  let Jitter = xm;
  class LinearInterpolator extends Interpolator {
    constructor(_) {
      super(_);
    }
    compute(_) {
      if (this.sort(!1), this.clip) {
        if (_ < this._x_sorted[0] || _ > this._x_sorted[this._x_sorted.length - 1])
          return NaN;
      } else {
        if (_ < this._x_sorted[0])
          return this._y_sorted[0];
        if (_ > this._x_sorted[this._x_sorted.length - 1])
          return this._y_sorted[this._y_sorted.length - 1];
      }
      if (_ == this._x_sorted[0])
        return this._y_sorted[0];
      const H = find_last_index(this._x_sorted, (Z) => Z < _), V = this._x_sorted[H], q = this._x_sorted[H + 1], G = this._y_sorted[H], J = this._y_sorted[H + 1];
      return G + (_ - V) / (q - V) * (J - G);
    }
  }
  mt(LinearInterpolator, "__name__", "LinearInterpolator");
  const t6 = class t6 extends Interpolator {
    constructor(_) {
      super(_);
    }
    compute(_) {
      if (this.sort(!1), this.clip) {
        if (_ < this._x_sorted[0] || _ > this._x_sorted[this._x_sorted.length - 1])
          return NaN;
      } else {
        if (_ < this._x_sorted[0])
          return this._y_sorted[0];
        if (_ > this._x_sorted[this._x_sorted.length - 1])
          return this._y_sorted[this._y_sorted.length - 1];
      }
      let H;
      switch (this.mode) {
        case "after": {
          H = find_last_index(this._x_sorted, (V) => _ >= V);
          break;
        }
        case "before": {
          H = find_index(this._x_sorted, (V) => _ <= V);
          break;
        }
        case "center": {
          const V = map$4(this._x_sorted, (G) => Math.abs(G - _)), q = min$b(V);
          H = find_index(V, (G) => q === G);
          break;
        }
        default:
          throw new Error(`unknown mode: ${this.mode}`);
      }
      return H != -1 ? this._y_sorted[H] : NaN;
    }
  };
  mt(t6, "__name__", "StepInterpolator"), t6.define(() => ({
    mode: [StepMode, "after"]
  }));
  let StepInterpolator = t6;
  const panels_css = ":host{position:absolute;background-color:white;}";
  class PanelView extends PaneView {
    stylesheets() {
      return [...super.stylesheets(), panels_css];
    }
    connect_signals() {
      super.connect_signals();
      const { position: _, anchor: H, width: V, height: q, elements: G } = this.model.properties;
      this.on_change([H, V, q, G], () => this.reposition()), this.on_transitive_change(_, () => this.reposition());
    }
    reposition(_) {
      var ae;
      super.reposition(_);
      const { position: H, visible: V, anchor: q, elements: G } = this.model;
      if (_ == !1 || !V || G.length == 0) {
        this.el.remove();
        return;
      }
      const { x: J, y: Z } = this.resolve_as_xy(H);
      if (!isFinite(J + Z)) {
        this.el.remove();
        return;
      }
      const te = ((ae = this.parent) == null ? void 0 : ae.el) ?? document.body, ne = te.shadowRoot ?? te;
      this.el.isConnected || ne.append(this.el), this.el.style.left = px$1(J), this.el.style.top = px$1(Z);
      const ee = anchor(q);
      this.el.style.transform = `translate(${-100 * ee.x}%, ${-100 * ee.y}%)`;
    }
  }
  mt(PanelView, "__name__", "PanelView");
  const wm = class wm extends Pane {
    constructor(_) {
      super(_);
    }
  };
  mt(wm, "__name__", "Panel"), wm.prototype.default_view = PanelView, wm.define(({ Ref: _, Or: H, Auto: V, Int: q }) => ({
    position: [_(Coordinate)],
    anchor: [Anchor$1, "top_left"],
    width: [H(V, q, _(Node$4)), "auto"],
    height: [H(V, q, _(Node$4)), "auto"]
  }));
  let Panel = wm;
  class DOMElementView extends DOMNodeView {
    constructor() {
      super(...arguments);
      mt(this, "child_views", /* @__PURE__ */ new Map());
    }
    *children() {
      yield* super.children(), yield* this.child_views.values();
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const H = this.model.children.filter((V) => !isString$2(V));
      await build_views(this.child_views, H, { parent: this });
    }
    remove() {
      remove_views(this.child_views), super.remove();
    }
    render() {
      empty$1(this.el), apply_styles(this.el.style, this.model.style);
      for (const H of this.model.children)
        if (isString$2(H)) {
          const V = document.createTextNode(H);
          this.el.appendChild(V);
        } else
          this.child_views.get(H).render_to(this.el);
      this.finish();
    }
  }
  mt(DOMElementView, "__name__", "DOMElementView");
  const n6 = class n6 extends DOMNode {
    constructor(_) {
      super(_);
    }
  };
  mt(n6, "__name__", "DOMElement"), n6.define(({ Str: _, List: H, Or: V, Ref: q }) => ({
    style: [StylesLike, {}],
    children: [H(V(_, q(DOMNode), q(UIElement))), []]
  }));
  let DOMElement = n6;
  class SpanView extends DOMElementView {
  }
  mt(SpanView, "__name__", "SpanView"), mt(SpanView, "tag_name", "span");
  var N0;
  let Span$2 = (N0 = class extends DOMElement {
  }, mt(N0, "__name__", "Span"), N0.prototype.default_view = SpanView, N0);
  var Y4;
  let DivView$1 = (Y4 = class extends DOMElementView {
  }, mt(Y4, "__name__", "DivView"), mt(Y4, "tag_name", "div"), Y4);
  var P0;
  let Div$1 = (P0 = class extends DOMElement {
  }, mt(P0, "__name__", "Div"), P0.prototype.default_view = DivView$1, P0);
  class TableView extends DOMElementView {
  }
  mt(TableView, "__name__", "TableView"), mt(TableView, "tag_name", "table");
  var $0;
  let Table$1 = ($0 = class extends DOMElement {
  }, mt($0, "__name__", "Table"), $0.prototype.default_view = TableView, $0);
  class TableRowView extends DOMElementView {
  }
  mt(TableRowView, "__name__", "TableRowView"), mt(TableRowView, "tag_name", "tr");
  const r6 = class r6 extends DOMElement {
  };
  mt(r6, "__name__", "TableRow"), r6.prototype.default_view = TableRowView;
  let TableRow = r6;
  function throttle$1(O, _) {
    let H = null, V = null, q = 0, G = !1, J;
    const Z = function() {
      return new Promise((te, ne) => {
        J = te;
        const ee = function() {
          q = Date.now(), H = null, V = null, G = !1;
          try {
            O(), te();
          } catch (se) {
            ne(se);
          }
        }, ae = Date.now(), oe = _ - (ae - q);
        oe <= 0 && !G ? (H != null && clearTimeout(H), G = !0, V = requestAnimationFrame(ee)) : H == null && !G ? H = setTimeout(() => V = requestAnimationFrame(ee), oe) : te();
      });
    };
    return Z.stop = function() {
      H != null && clearTimeout(H), V != null && cancelAnimationFrame(V), J();
    }, Z;
  }
  class RangeManager {
    constructor(_) {
      mt(this, "parent");
      mt(this, "invalidate_dataranges", !0);
      this.parent = _;
    }
    get frame() {
      return this.parent.frame;
    }
    update(_, H = {}) {
      const V = /* @__PURE__ */ new Map();
      for (const [q, G] of _.xrs)
        V.set(q, G);
      for (const [q, G] of _.yrs)
        V.set(q, G);
      (H.scrolling ?? !1) && this._update_ranges_together(V), this._update_ranges_individually(V, H);
    }
    ranges() {
      const _ = /* @__PURE__ */ new Set(), H = /* @__PURE__ */ new Set();
      for (const V of this.frame.x_ranges.values())
        _.add(V);
      for (const V of this.frame.y_ranges.values())
        H.add(V);
      for (const V of this.parent.model.data_renderers) {
        const { coordinates: q } = V;
        q != null && (_.add(q.x_source), H.add(q.y_source));
      }
      return {
        x_ranges: [..._],
        y_ranges: [...H]
      };
    }
    reset() {
      const { x_ranges: _, y_ranges: H } = this.ranges();
      for (const V of _)
        V.reset();
      for (const V of H)
        V.reset();
      this.update_dataranges();
    }
    _update_dataranges(_) {
      const H = /* @__PURE__ */ new Map(), V = /* @__PURE__ */ new Map();
      let q = !1;
      for (const [, ee] of _.x_ranges)
        ee instanceof DataRange1d && ee.scale_hint == "log" && (q = !0);
      for (const [, ee] of _.y_ranges)
        ee instanceof DataRange1d && ee.scale_hint == "log" && (q = !0);
      for (const ee of this.parent.auto_ranged_renderers) {
        const ae = ee.bounds();
        if (H.set(ee.model, ae), q) {
          const oe = ee.log_bounds();
          V.set(ee.model, oe);
        }
      }
      let G = !1, J = !1;
      const Z = _.x_target.span, te = _.y_target.span;
      let ne;
      this.parent.model.match_aspect !== !1 && Z != 0 && te != 0 && (ne = 1 / this.parent.model.aspect_scale * (Z / te));
      for (const [, ee] of _.x_ranges) {
        if (ee instanceof DataRange1d) {
          const ae = ee.scale_hint == "log" ? V : H;
          ee.update(ae, 0, this.parent, ne), ee.follow != null && (G = !0);
        }
        ee.bounds != null && (J = !0);
      }
      for (const [, ee] of _.y_ranges) {
        if (ee instanceof DataRange1d) {
          const ae = ee.scale_hint == "log" ? V : H;
          ee.update(ae, 1, this.parent, ne), ee.follow != null && (G = !0);
        }
        ee.bounds != null && (J = !0);
      }
      if (G && J) {
        logger$1.warn("Follow enabled so bounds are unset.");
        for (const [, ee] of _.x_ranges)
          ee.bounds = null;
        for (const [, ee] of _.y_ranges)
          ee.bounds = null;
      }
    }
    update_dataranges() {
      this._update_dataranges(this.frame);
      for (const _ of this.parent.auto_ranged_renderers) {
        const { coordinates: H } = _.model;
        H != null && this._update_dataranges(H);
      }
      this.compute_initial() != null && (this.invalidate_dataranges = !1);
    }
    compute_initial() {
      let _ = !0;
      const { x_ranges: H, y_ranges: V } = this.frame, q = /* @__PURE__ */ new Map(), G = /* @__PURE__ */ new Map();
      for (const [, J] of H) {
        const { start: Z, end: te } = J;
        if (isNaN(Z + te)) {
          _ = !1;
          break;
        }
        q.set(J, { start: Z, end: te });
      }
      if (_)
        for (const [, J] of V) {
          const { start: Z, end: te } = J;
          if (isNaN(Z + te)) {
            _ = !1;
            break;
          }
          G.set(J, { start: Z, end: te });
        }
      return _ ? { xrs: q, yrs: G } : (logger$1.warn("could not set initial ranges"), null);
    }
    _update_ranges_together(_) {
      let H = 1;
      for (const [V, q] of _)
        H = Math.min(H, this._get_weight_to_constrain_interval(V, q));
      if (H < 1)
        for (const [V, q] of _)
          q.start = H * q.start + (1 - H) * V.start, q.end = H * q.end + (1 - H) * V.end;
    }
    _update_ranges_individually(_, H = {}) {
      const V = H.panning ?? !1, q = H.scrolling ?? !1, G = H.maintain_focus ?? !1;
      let J = !1;
      for (const [Z, te] of _) {
        if (!q) {
          const ne = this._get_weight_to_constrain_interval(Z, te);
          ne < 1 && (te.start = ne * te.start + (1 - ne) * Z.start, te.end = ne * te.end + (1 - ne) * Z.end);
        }
        if (Z.bounds != null) {
          const [ne, ee] = Z.computed_bounds, ae = Math.abs(te.end - te.start);
          Z.is_reversed ? (ne > te.end && (J = !0, te.end = ne, (V || q) && (te.start = ne + ae)), ee < te.start && (J = !0, te.start = ee, (V || q) && (te.end = ee - ae))) : (ne > te.start && (J = !0, te.start = ne, (V || q) && (te.end = ne + ae)), ee < te.end && (J = !0, te.end = ee, (V || q) && (te.start = ee - ae)));
        }
      }
      if (!(q && J && G))
        for (const [Z, te] of _)
          Z.have_updated_interactively = !0, (Z.start != te.start || Z.end != te.end) && Z.setv(te);
    }
    _get_weight_to_constrain_interval(_, H) {
      const { min_interval: V } = _;
      let { max_interval: q } = _;
      if (_.bounds != null && _.bounds != "auto") {
        const [J, Z] = _.bounds;
        if (J != null && Z != null) {
          const te = Math.abs(Z - J);
          q = q != null ? Math.min(q, te) : te;
        }
      }
      let G = 1;
      if (V != null || q != null) {
        const J = Math.abs(_.end - _.start), Z = Math.abs(H.end - H.start);
        V != null && V > 0 && Z < V && (G = (J - V) / (J - Z)), q != null && q > 0 && Z > q && (G = (q - J) / (Z - J)), G = Math.max(0, Math.min(1, G));
      }
      return G;
    }
  }
  mt(RangeManager, "__name__", "RangeManager");
  class StateManager {
    constructor(_, H) {
      mt(this, "parent");
      mt(this, "initial_state");
      mt(this, "changed");
      mt(this, "history", []);
      mt(this, "index", -1);
      this.parent = _, this.initial_state = H, this.changed = new Signal0(this.parent, "state_changed");
    }
    _do_state_change(_) {
      const H = _ in this.history ? this.history[_].state : this.initial_state;
      return H.range != null && this.parent.update_range(H.range), H.selection != null && this.parent.update_selection(H.selection), H;
    }
    peek() {
      return this.can_undo ? this.history[this.index] : null;
    }
    push(_, H) {
      const { history: V, index: q } = this, G = q in V ? V[q].state : {}, J = { ...this.initial_state, ...G, ...H };
      this.history = this.history.slice(0, this.index + 1), this.history.push({ type: _, state: J }), this.index = this.history.length - 1, this.changed.emit();
    }
    clear() {
      this.history = [], this.index = -1, this.changed.emit();
    }
    undo() {
      if (this.can_undo) {
        this.index -= 1;
        const _ = this._do_state_change(this.index);
        return this.changed.emit(), _;
      }
      return null;
    }
    redo() {
      if (this.can_redo) {
        this.index += 1;
        const _ = this._do_state_change(this.index);
        return this.changed.emit(), _;
      }
      return null;
    }
    get can_undo() {
      return this.index >= 0;
    }
    get can_redo() {
      return this.index < this.history.length - 1;
    }
  }
  mt(StateManager, "__name__", "StateManager");
  const plots_css = ':host{display:grid;grid-template-areas:".    above  .    " "left center right" ".    below  .    ";grid-template-rows:0px 1fr 0px;grid-template-columns:0px 1fr 0px;}.bk-Canvas{grid-row-start:1;grid-row-end:span 3;grid-column-start:1;grid-column-end:span 3;}', attribution_css = ":host{display:flex;flex-direction:column;flex-wrap:nowrap;padding:2px;background-color:rgba(255, 255, 255, 0.5);font-size:9px;line-height:1.05;white-space:nowrap;max-width:var(--max-width);}:host > *{overflow:hidden;text-overflow:ellipsis;text-align:right;}a{color:black;}", { max: max$5 } = Math;
  class PlotView extends LayoutDOMView {
    constructor() {
      super(...arguments);
      mt(this, "visuals");
      mt(this, "frame");
      mt(this, "_render_count", 0);
      mt(this, "_canvas");
      mt(this, "canvas_view");
      mt(this, "repainted", new Signal0(this, "repainted"));
      mt(this, "_computed_style", new InlineStyleSheet$1());
      mt(this, "_title");
      mt(this, "_toolbar");
      mt(this, "_attribution");
      mt(this, "_outer_bbox", new BBox$1());
      mt(this, "_inner_bbox", new BBox$1());
      mt(this, "_needs_paint", !0);
      mt(this, "_invalidated_painters", /* @__PURE__ */ new Set());
      mt(this, "_invalidate_all", !0);
      mt(this, "_state_manager");
      mt(this, "_range_manager");
      mt(this, "_is_paused");
      mt(this, "lod_started");
      mt(this, "_initial_state");
      mt(this, "throttled_paint");
      mt(this, "computed_renderers");
      /*protected*/
      mt(this, "renderer_views", /* @__PURE__ */ new Map());
      /*protected*/
      mt(this, "tool_views", /* @__PURE__ */ new Map());
      mt(this, "_needs_notify", !1);
    }
    get canvas() {
      return this.canvas_view;
    }
    stylesheets() {
      return [...super.stylesheets(), plots_css, this._computed_style];
    }
    get toolbar_panel() {
      return this._toolbar != null ? this.renderer_view(this._toolbar) : void 0;
    }
    get state() {
      return this._state_manager;
    }
    set invalidate_dataranges(H) {
      this._range_manager.invalidate_dataranges = H;
    }
    get computed_renderer_views() {
      return this.computed_renderers.map((H) => this.renderer_views.get(H));
    }
    renderer_view(H) {
      const V = this.renderer_views.get(H);
      if (V == null)
        for (const [, q] of this.renderer_views) {
          const G = q.renderer_view(H);
          if (G != null)
            return G;
        }
      return V;
    }
    get auto_ranged_renderers() {
      return this.model.renderers.map((H) => this.renderer_view(H)).filter(is_auto_ranged);
    }
    get base_font_size() {
      const H = getComputedStyle(this.el).fontSize, V = parse_css_font_size(H);
      if (V != null) {
        const { value: q, unit: G } = V;
        if (G == "px")
          return q;
      }
      return null;
    }
    *children() {
      yield* super.children(), yield* this.renderer_views.values(), yield* this.tool_views.values();
    }
    get is_paused() {
      return this._is_paused != null && this._is_paused !== 0;
    }
    get child_models() {
      return [];
    }
    pause() {
      this._is_paused == null ? this._is_paused = 1 : this._is_paused += 1;
    }
    unpause(H = !1) {
      if (this._is_paused == null)
        throw new Error("wasn't paused");
      this._is_paused = Math.max(this._is_paused - 1, 0), this._is_paused == 0 && !H && this.request_paint("everything");
    }
    notify_finished_after_paint() {
      this._needs_notify = !0;
    }
    // TODO: this needs to be removed
    request_render() {
      this.request_paint("everything");
    }
    request_paint(H) {
      this.invalidate_painters(H), this.schedule_paint();
    }
    invalidate_painters(H) {
      if (H == "everything")
        this._invalidate_all = !0;
      else if (isArray$3(H))
        for (const V of H) {
          const q = V instanceof RendererView ? V : this.renderer_view(V);
          this._invalidated_painters.add(q);
        }
      else {
        const V = H instanceof RendererView ? H : this.renderer_view(H);
        this._invalidated_painters.add(V);
      }
    }
    schedule_paint() {
      if (!this.is_paused) {
        const H = this.throttled_paint();
        this._ready = this._ready.then(() => H);
      }
    }
    request_layout() {
      this.request_paint("everything");
    }
    reset() {
      this.model.reset_policy == "standard" && (this.state.clear(), this.reset_range(), this.reset_selection()), this.model.trigger_event(new Reset());
    }
    remove() {
      for (const H of this.frame.ranges.values())
        H.plots.delete(this);
      remove_views(this.renderer_views), remove_views(this.tool_views), super.remove();
    }
    get_context_menu(H) {
      var G;
      const { x: V, y: q } = H;
      for (const J of reversed([...this.renderer_views.values()]))
        if (J.context_menu != null && ((G = J.interactive_hit) == null ? void 0 : G.call(J, V, q)) == !0)
          return J.context_menu;
      return super.get_context_menu(H);
    }
    initialize() {
      this.pause(), super.initialize(), this.lod_started = !1, this.visuals = new Visuals(this), this._initial_state = {
        selection: /* @__PURE__ */ new Map()
        // XXX: initial selection?
      }, this.frame = new CartesianFrame(this.model.x_scale, this.model.y_scale, this.model.x_range, this.model.y_range, this.model.extra_x_ranges, this.model.extra_y_ranges, this.model.extra_x_scales, this.model.extra_y_scales);
      for (const ne of this.frame.ranges.values())
        ne.plots.add(this);
      this._range_manager = new RangeManager(this), this._state_manager = new StateManager(this, this._initial_state), this.throttled_paint = throttle$1(() => {
        this.is_destroyed || this.repaint();
      }, 1e3 / 60);
      const { title_location: H, title: V } = this.model;
      H != null && V != null && (this._title = V instanceof Title ? V : new Title({ text: V }));
      const { toolbar_location: q, toolbar_inner: G, toolbar: J } = this.model;
      q != null && (this._toolbar = new ToolbarPanel({ toolbar: J }), J.location = q, J.inner = G);
      const { hidpi: Z, output_backend: te } = this.model;
      this._canvas = new Canvas$1({ hidpi: Z, output_backend: te }), this._attribution = new Panel({
        position: new Node$4({ target: "frame", symbol: "bottom_right" }),
        anchor: "bottom_right",
        elements: [],
        css_variables: {
          "--max-width": new Node$4({ target: "frame", symbol: "width" })
        },
        stylesheets: [attribution_css]
      });
    }
    get elements() {
      return [this._canvas, this._attribution, ...super.elements];
    }
    async lazy_initialize() {
      await super.lazy_initialize(), this.canvas_view = this._element_views.get(this._canvas), this.canvas_view.plot_views = [this], await this.build_tool_views(), await this.build_renderer_views(), this._range_manager.update_dataranges();
    }
    box_sizing() {
      const { width_policy: H, height_policy: V, ...q } = super.box_sizing(), { frame_width: G, frame_height: J } = this.model;
      return {
        ...q,
        width_policy: G != null && H == "auto" ? "fit" : H,
        height_policy: J != null && V == "auto" ? "fit" : V
      };
    }
    _intrinsic_display() {
      return { inner: this.model.flow_mode, outer: "grid" };
    }
    _update_layout() {
      super._update_layout(), this._invalidate_all = !0, this._needs_paint = !0;
      const H = new BorderLayout(), { frame_align: V } = this.model;
      if (H.aligns = (() => {
        if (isBoolean$4(V))
          return { left: V, right: V, top: V, bottom: V };
        {
          const { left: we = !0, right: Ee = !0, top: xe = !0, bottom: Le = !0 } = V;
          return { left: we, right: Ee, top: xe, bottom: Le };
        }
      })(), H.set_sizing({ width_policy: "max", height_policy: "max" }), this.visuals.outline_line.doit) {
        const we = this.visuals.outline_line.line_width.get_value();
        H.center_border_width = we;
      }
      const q = copy$3(this.model.above), G = copy$3(this.model.below), J = copy$3(this.model.left), Z = copy$3(this.model.right), te = [], ne = [], ee = [], ae = [], oe = (we, Ee = !1) => {
        switch (we) {
          case "above":
            return Ee ? te : q;
          case "below":
            return Ee ? ne : G;
          case "left":
            return Ee ? ee : J;
          case "right":
            return Ee ? ae : Z;
        }
      }, { title_location: se } = this.model;
      if (se != null && this._title != null && oe(se).push(this._title), this._toolbar != null) {
        const { location: we } = this._toolbar.toolbar;
        if (this.model.toolbar_inner)
          oe(we, !0).push(this._toolbar);
        else {
          const Ee = oe(we);
          let xe = !0;
          if (this.model.toolbar_sticky)
            for (let Le = 0; Le < Ee.length; Le++) {
              const Qe = Ee[Le];
              if (Qe instanceof Title) {
                we == "above" || we == "below" ? Ee[Le] = [Qe, this._toolbar] : Ee[Le] = [this._toolbar, Qe], xe = !1;
                break;
              }
            }
          xe && Ee.push(this._toolbar);
        }
      }
      const ce = (we, Ee) => {
        var Le;
        const xe = this.renderer_view(Ee);
        return xe.panel = new SidePanel(we), (Le = xe.update_layout) == null || Le.call(xe), xe.layout;
      }, le = (we, Ee) => {
        const xe = we == "above" || we == "below", Le = [];
        for (const Qe of Ee)
          if (isArray$3(Qe)) {
            const ke = Qe.map((Re) => {
              const De = ce(we, Re);
              if (De != null) {
                if (Re instanceof ToolbarPanel) {
                  const Pe = xe ? "width_policy" : "height_policy";
                  De.set_sizing({ ...De.sizing, [Pe]: "min" });
                }
                return De;
              }
            }).filter((Re) => Re != null);
            let Me;
            xe ? (Me = new Row$2(ke), Me.set_sizing({ width_policy: "max", height_policy: "min" })) : (Me = new Column$2(ke), Me.set_sizing({ width_policy: "min", height_policy: "max" })), Me.absolute = !0, Le.push(Me);
          } else {
            const ke = ce(we, Qe);
            ke != null && Le.push(ke);
          }
        return Le;
      }, ue = this.model.min_border ?? 0;
      H.min_border = {
        left: this.model.min_border_left ?? ue,
        top: this.model.min_border_top ?? ue,
        right: this.model.min_border_right ?? ue,
        bottom: this.model.min_border_bottom ?? ue
      };
      const de = new NodeLayout(), fe = new VStack(), he = new VStack(), me = new HStack(), pe = new HStack(), ye = new VStack(), ge = new VStack(), ve = new HStack(), be = new HStack();
      de.absolute = !0, fe.absolute = !0, he.absolute = !0, me.absolute = !0, pe.absolute = !0, ye.absolute = !0, ge.absolute = !0, ve.absolute = !0, be.absolute = !0, de.children = this.model.center.filter((we) => we instanceof Annotation$1).map((we) => {
        var xe;
        const Ee = this.renderer_view(we);
        return (xe = Ee.update_layout) == null || xe.call(Ee), Ee.layout;
      }).filter((we) => we != null);
      const { frame_width: Ce, frame_height: Se } = this.model;
      de.set_sizing({
        ...Ce != null ? { width_policy: "fixed", width: Ce } : { width_policy: "fit" },
        ...Se != null ? { height_policy: "fixed", height: Se } : { height_policy: "fit" }
      }), de.on_resize((we) => this.frame.set_geometry(we)), fe.children = reversed(le("above", q)), he.children = le("below", G), me.children = reversed(le("left", J)), pe.children = le("right", Z), ye.children = le("above", te), ge.children = le("below", ne), ve.children = le("left", ee), be.children = le("right", ae), fe.set_sizing({
        width_policy: "fit",
        height_policy: "min"
        /*, min_height: layout.min_border.top*/
      }), he.set_sizing({
        width_policy: "fit",
        height_policy: "min"
        /*, min_height: layout.min_width.bottom*/
      }), me.set_sizing({
        width_policy: "min",
        height_policy: "fit"
        /*, min_width: layout.min_width.left*/
      }), pe.set_sizing({
        width_policy: "min",
        height_policy: "fit"
        /*, min_width: layout.min_width.right*/
      }), ye.set_sizing({ width_policy: "fit", height_policy: "min" }), ge.set_sizing({ width_policy: "fit", height_policy: "min" }), ve.set_sizing({ width_policy: "min", height_policy: "fit" }), be.set_sizing({ width_policy: "min", height_policy: "fit" }), H.center_panel = de, H.top_panel = fe, H.bottom_panel = he, H.left_panel = me, H.right_panel = pe, ye.children.length != 0 && (H.inner_top_panel = ye), ge.children.length != 0 && (H.inner_bottom_panel = ge), ve.children.length != 0 && (H.inner_left_panel = ve), be.children.length != 0 && (H.inner_right_panel = be), this.layout = H;
    }
    _measure_layout() {
      const { frame_width: H, frame_height: V } = this.model, q = {
        width: H == null ? "1fr" : px$1(H),
        height: V == null ? "1fr" : px$1(V)
      }, { layout: G } = this, J = G.top_panel.measure({ width: 1 / 0, height: 1 / 0 }), Z = G.bottom_panel.measure({ width: 1 / 0, height: 1 / 0 }), te = G.left_panel.measure({ width: 1 / 0, height: 1 / 0 }), ne = G.right_panel.measure({ width: 1 / 0, height: 1 / 0 }), ee = max$5(J.height, G.min_border.top), ae = max$5(Z.height, G.min_border.bottom), oe = max$5(te.width, G.min_border.left), se = max$5(ne.width, G.min_border.right);
      this._computed_style.replace(`
      :host {
        grid-template-rows: ${ee}px ${q.height} ${ae}px;
        grid-template-columns: ${oe}px ${q.width} ${se}px;
      }
    `);
    }
    get axis_views() {
      const H = [];
      for (const [, V] of this.renderer_views)
        V instanceof AxisView && H.push(V);
      return H;
    }
    update_range(H, V) {
      this.pause(), this._range_manager.update(H, V), this.unpause();
    }
    reset_range() {
      this.pause(), this._range_manager.reset(), this.unpause(), this.trigger_ranges_update_event();
    }
    trigger_ranges_update_event(H = []) {
      const { x_ranges: V, y_ranges: q } = this._range_manager.ranges(), G = [...V, ...q, ...H], J = new Set(G.flatMap((Z) => [...Z.plots]));
      for (const Z of J) {
        const { x_range: te, y_range: ne } = Z.model, ee = new RangesUpdate(te.start, te.end, ne.start, ne.end);
        Z.model.trigger_event(ee);
      }
    }
    get_selection() {
      const H = /* @__PURE__ */ new Map();
      for (const V of this.model.data_renderers) {
        const { selected: q } = V.selection_manager.source;
        H.set(V, q);
      }
      return H;
    }
    update_selection(H) {
      for (const V of this.model.data_renderers) {
        const q = V.selection_manager.source;
        if (H != null) {
          const G = H.get(V);
          G != null && q.selected.update(G, !0);
        } else
          q.selection_manager.clear();
      }
    }
    reset_selection() {
      this.update_selection(null);
    }
    _invalidate_layout_if_needed() {
      (() => {
        var V;
        for (const q of this.model.side_panels) {
          const G = this.renderer_views.get(q);
          if (((V = G.layout) == null ? void 0 : V.has_size_changed()) ?? !1)
            return this.invalidate_painters(G), !0;
        }
        return !1;
      })() && this.compute_layout();
    }
    get_renderer_views() {
      return this.computed_renderers.map((H) => this.renderer_views.get(H));
    }
    *_compute_renderers() {
      const { above: H, below: V, left: q, right: G, center: J, renderers: Z } = this.model;
      yield* Z, yield* H, yield* V, yield* q, yield* G, yield* J, this._title != null && (yield this._title), this._toolbar != null && (yield this._toolbar);
      for (const [, te] of this.tool_views)
        yield* te.overlays;
    }
    _update_attribution() {
      const V = [
        ...this.model.attribution,
        ...this.computed_renderer_views.map((q) => q == null ? void 0 : q.attribution)
        // TODO race condition again
      ].filter(isNotNull).map((q) => isString$2(q) ? new Div$1({ children: [q] }) : q);
      this._attribution.elements = V;
    }
    async build_renderer_views() {
      this.computed_renderers = [...this._compute_renderers()], await build_views(this.renderer_views, this.computed_renderers, { parent: this }), this._update_attribution();
    }
    async build_tool_views() {
      const H = flat_map(this.model.toolbar.tools, (q) => q instanceof ToolProxy ? q.tools : [q]), { created: V } = await build_views(this.tool_views, [...H], { parent: this });
      V.map((q) => this.canvas_view.ui_event_bus.register_tool(q));
    }
    connect_signals() {
      super.connect_signals();
      const { extra_x_ranges: H, extra_y_ranges: V, extra_x_scales: q, extra_y_scales: G } = this.model.properties;
      this.on_change([H, V, q, G], () => {
        this.frame.x_range = this.model.x_range, this.frame.y_range = this.model.y_range, this.frame.in_x_scale = this.model.x_scale, this.frame.in_y_scale = this.model.y_scale, this.frame.extra_x_ranges = this.model.extra_x_ranges, this.frame.extra_y_ranges = this.model.extra_y_ranges, this.frame.extra_x_scales = this.model.extra_x_scales, this.frame.extra_y_scales = this.model.extra_y_scales, this.frame.configure_scales();
      });
      const { above: J, below: Z, left: te, right: ne, center: ee, renderers: ae } = this.model.properties, oe = [J, Z, te, ne, ee];
      this.on_change(ae, async () => {
        await this.build_renderer_views();
      }), this.on_change(oe, async () => {
        await this.build_renderer_views(), this.invalidate_layout();
      }), this.connect(this.model.toolbar.properties.tools.change, async () => {
        await this.build_tool_views(), await this.build_renderer_views();
      });
      const { x_ranges: se, y_ranges: ce } = this.frame;
      for (const [, de] of se)
        this.connect(de.change, () => {
          this.request_paint("everything");
        });
      for (const [, de] of ce)
        this.connect(de.change, () => {
          this.request_paint("everything");
        });
      this.connect(this.model.change, () => this.request_paint("everything")), this.connect(this.model.reset, () => this.reset());
      const { toolbar_location: le } = this.model.properties;
      this.on_change(le, async () => {
        const { toolbar_location: de } = this.model;
        if (this._toolbar != null)
          de != null ? this._toolbar.toolbar.location = de : (this._toolbar = void 0, await this.build_renderer_views());
        else if (de != null) {
          const { toolbar: fe, toolbar_inner: he } = this.model;
          this._toolbar = new ToolbarPanel({ toolbar: fe }), fe.location = de, fe.inner = he, await this.build_renderer_views();
        }
        this.invalidate_layout();
      });
      const { hold_render: ue } = this.model.properties;
      this.on_change(ue, () => this._hold_render_changed());
    }
    has_finished() {
      if (!super.has_finished())
        return !1;
      if (this.model.visible) {
        for (const [, H] of this.renderer_views)
          if (!H.has_finished())
            return !1;
      }
      return !0;
    }
    _after_layout() {
      var se;
      super._after_layout(), this.unpause(!0);
      const H = this.layout.left_panel.bbox, V = this.layout.right_panel.bbox, q = this.layout.center_panel.bbox, G = this.layout.top_panel.bbox, J = this.layout.bottom_panel.bbox, { bbox: Z } = this, te = G.bottom, ne = Z.height - J.top, ee = H.right, ae = Z.width - V.left;
      this.canvas.style.replace(`
      .bk-layer.bk-events {
        display: grid;
        grid-template-areas:
          ".    above  .    "
          "left center right"
          ".    below  .    ";
        grid-template-rows: ${px$1(te)} ${px$1(q.height)} ${px$1(ne)};
        grid-template-columns: ${px$1(ee)} ${px$1(q.width)} ${px$1(ae)};
      }
    `);
      for (const [, ce] of this.renderer_views)
        ce instanceof AnnotationView && ((se = ce.after_layout) == null || se.call(ce));
      this.model.setv({
        inner_width: Math.round(this.frame.bbox.width),
        inner_height: Math.round(this.frame.bbox.height),
        outer_width: Math.round(this.bbox.width),
        outer_height: Math.round(this.bbox.height)
      }, { no_change: !0 }), this.model.match_aspect && (this.pause(), this._range_manager.update_dataranges(), this.unpause(!0)), this._outer_bbox.equals(this.bbox) || (this.canvas_view.resize(), this._outer_bbox = this.bbox, this._invalidate_all = !0, this._needs_paint = !0);
      const { inner_bbox: oe } = this.layout;
      this._inner_bbox.equals(oe) || (this._inner_bbox = oe, this._invalidate_all = !0, this._needs_paint = !0), this._needs_paint && this.paint();
    }
    repaint() {
      this._invalidate_layout_if_needed(), this.paint();
    }
    paint() {
      if (!this.is_paused) {
        if (this.is_displayed)
          logger$1.trace(`${this.toString()}.paint()`), this._actual_paint();
        else
          for (const H of this.computed_renderer_views)
            H.mark_finished();
        this._needs_notify && (this._needs_notify = !1, this.notify_finished());
      }
    }
    _actual_paint() {
      logger$1.trace(`${this.toString()}._actual_paint ${this._render_count} start`);
      const { document: H } = this.model;
      if (H != null) {
        const te = H.interactive_duration();
        te >= 0 && te < this.model.lod_interval ? setTimeout(() => {
          H.interactive_duration() > this.model.lod_timeout && H.interactive_stop(), this.request_paint("everything");
        }, this.model.lod_timeout) : H.interactive_stop();
      }
      this._range_manager.invalidate_dataranges && (this._range_manager.update_dataranges(), this._invalidate_layout_if_needed());
      let V = !1, q = !1;
      if (this._invalidate_all)
        V = !0, q = !0;
      else
        for (const te of this._invalidated_painters) {
          const { level: ne } = te.model;
          if (ne != "overlay" ? V = !0 : q = !0, V && q)
            break;
        }
      this._invalidated_painters.clear(), this._invalidate_all = !1;
      const G = [
        this.frame.bbox.left,
        this.frame.bbox.top,
        this.frame.bbox.width,
        this.frame.bbox.height
      ], { primary: J, overlays: Z } = this.canvas_view;
      V && (J.prepare(), this.canvas_view.prepare_webgl(G), this._paint_empty(J.ctx, G), this._paint_outline(J.ctx, G), this._paint_levels(J.ctx, "image", G, !0), this._paint_levels(J.ctx, "underlay", G, !0), this._paint_levels(J.ctx, "glyph", G, !0), this._paint_levels(J.ctx, "guide", G, !1), this._paint_levels(J.ctx, "annotation", G, !1), J.finish()), (q || settings.wireframe) && (Z.prepare(), this._paint_levels(Z.ctx, "overlay", G, !1), settings.wireframe && this.paint_layout(Z.ctx, this.layout), Z.finish()), this._initial_state.range == null && (this._initial_state.range = this._range_manager.compute_initial() ?? void 0);
      for (const te of this.element_views)
        te.reposition();
      this._needs_paint = !1, this.repainted.emit(), logger$1.trace(`${this.toString()}._actual_paint ${this._render_count} end`), this._render_count++;
    }
    _paint_levels(H, V, q, G) {
      for (const J of this.computed_renderers) {
        if (J.level != V)
          continue;
        const Z = this.renderer_views.get(J);
        H.save(), (G || Z.needs_clip) && (H.beginPath(), H.rect(...q), H.clip()), Z.render(), H.restore(), Z.has_webgl && this.canvas_view.blit_webgl(H);
      }
    }
    paint_layout(H, V) {
      const { x: q, y: G, width: J, height: Z } = V.bbox;
      H.strokeStyle = "blue", H.strokeRect(q, G, J, Z);
      for (const te of V)
        H.save(), V.absolute || H.translate(q, G), this.paint_layout(H, te), H.restore();
    }
    _paint_empty(H, V) {
      const [q, G, J, Z] = [0, 0, this.bbox.width, this.bbox.height], [te, ne, ee, ae] = V;
      this.visuals.border_fill.doit && (H.save(), H.beginPath(), H.rect(q, G, J, Z), H.rect(te, ne, ee, ae), H.clip("evenodd"), H.beginPath(), H.rect(q, G, J, Z), this.visuals.border_fill.apply(H), H.restore()), this.visuals.background_fill.doit && (this.visuals.background_fill.set_value(H), H.fillRect(te, ne, ee, ae));
    }
    _paint_outline(H, V) {
      if (this.visuals.outline_line.doit) {
        H.save(), this.visuals.outline_line.set_value(H);
        let [q, G, J, Z] = V;
        q + J == this.bbox.width && (J -= 1), G + Z == this.bbox.height && (Z -= 1), H.strokeRect(q, G, J, Z), H.restore();
      }
    }
    export(H = "auto", V = !0) {
      const q = (() => {
        switch (H) {
          case "auto":
            return this.canvas_view.model.output_backend;
          case "png":
            return "canvas";
          case "svg":
            return "svg";
        }
      })(), G = new CanvasLayer(q, V), { width: J, height: Z } = this.bbox;
      if (G.resize(J, Z), J != 0 && Z != 0) {
        const { canvas: te } = this.canvas_view.compose();
        G.ctx.drawImage(te, 0, 0);
      }
      return G;
    }
    serializable_state() {
      const { children: H, ...V } = super.serializable_state(), q = this.get_renderer_views().filter((J) => J.model.syncable).map((J) => J.serializable_state()).filter((J) => J.bbox != null), G = { type: "CartesianFrame", bbox: this.frame.bbox };
      return { ...V, children: [...H ?? [], G, ...q] };
    }
    _hold_render_changed() {
      this.model.hold_render ? this.pause() : this.unpause();
    }
    resolve_frame() {
      return this.frame;
    }
    resolve_canvas() {
      return this.canvas;
    }
    resolve_plot() {
      return this;
    }
    resolve_xy(H) {
      const { x: V, y: q } = H, G = this.frame.x_scale.compute(V), J = this.frame.y_scale.compute(q);
      return this.frame.bbox.contains(G, J) ? { x: G, y: J } : { x: NaN, y: NaN };
    }
    resolve_indexed(H) {
      const { index: V, renderer: q } = H, G = this.renderer_view(q);
      if (G != null && G.has_finished()) {
        const [J, Z] = G.glyph.scenterxy(V, NaN, NaN);
        if (this.frame.bbox.contains(J, Z))
          return { x: J, y: Z };
      }
      return { x: NaN, y: NaN };
    }
  }
  mt(PlotView, "__name__", "PlotView");
  const g0 = class g0 extends LayoutDOM {
    constructor(H) {
      super(H);
      mt(this, "use_map", !1);
      mt(this, "reset", new Signal0(this, "reset"));
    }
    add_layout(H, V = "center") {
      const q = this.properties[V].get_value();
      this.setv({ [V]: [...q, H] });
    }
    remove_layout(H) {
      const V = (q) => {
        remove_by(q, (G) => G == H);
      };
      V(this.left), V(this.right), V(this.above), V(this.below), V(this.center);
    }
    get data_renderers() {
      return this.renderers.filter((H) => H instanceof DataRenderer);
    }
    add_renderers(...H) {
      this.renderers = [...this.renderers, ...H];
    }
    add_glyph(H, V = new ColumnDataSource(), q = {}) {
      const G = new GlyphRenderer({ ...q, data_source: V, glyph: H });
      return this.add_renderers(G), G;
    }
    add_tools(...H) {
      const V = H.map((q) => q instanceof Tool ? q : Tool.from_string(q));
      this.toolbar.tools = [...this.toolbar.tools, ...V];
    }
    remove_tools(...H) {
      this.toolbar.tools = [...difference(new Set(this.toolbar.tools), new Set(H))];
    }
    get panels() {
      return [...this.side_panels, ...this.center];
    }
    get side_panels() {
      const { above: H, below: V, left: q, right: G } = this;
      return concat$1([H, V, q, G]);
    }
  };
  mt(g0, "__name__", "Plot"), g0.prototype.default_view = PlotView, g0.mixins([
    ["outline_", Line$4],
    ["background_", Fill$2],
    ["border_", Fill$2]
  ]), g0.define(({ Bool: H, Float: V, Str: q, List: G, Dict: J, Or: Z, Ref: te, Null: ne, Nullable: ee, Struct: ae, Opt: oe }) => ({
    toolbar: [te(Toolbar), () => new Toolbar()],
    toolbar_location: [ee(Location), "right"],
    toolbar_sticky: [H, !0],
    toolbar_inner: [H, !1],
    frame_width: [ee(V), null],
    frame_height: [ee(V), null],
    frame_align: [Z(H, ae({ left: oe(H), right: oe(H), top: oe(H), bottom: oe(H) })), !0],
    // revise this when https://github.com/microsoft/TypeScript/pull/42425 is merged
    title: [Z(te(Title), q, ne), "", {
      convert: (se) => isString$2(se) ? new Title({ text: se }) : se
    }],
    title_location: [ee(Location), "above"],
    above: [G(Z(te(Annotation$1), te(Axis))), []],
    below: [G(Z(te(Annotation$1), te(Axis))), []],
    left: [G(Z(te(Annotation$1), te(Axis))), []],
    right: [G(Z(te(Annotation$1), te(Axis))), []],
    center: [G(Z(te(Annotation$1), te(Grid))), []],
    renderers: [G(te(Renderer$1)), []],
    x_range: [te(Range$2), () => new DataRange1d()],
    y_range: [te(Range$2), () => new DataRange1d()],
    x_scale: [te(Scale$2), () => new LinearScale()],
    y_scale: [te(Scale$2), () => new LinearScale()],
    extra_x_ranges: [J(te(Range$2)), {}],
    extra_y_ranges: [J(te(Range$2)), {}],
    extra_x_scales: [J(te(Scale$2)), {}],
    extra_y_scales: [J(te(Scale$2)), {}],
    lod_factor: [V, 10],
    lod_interval: [V, 300],
    lod_threshold: [ee(V), 2e3],
    lod_timeout: [V, 500],
    hidpi: [H, !0],
    output_backend: [OutputBackend, "canvas"],
    min_border: [ee(V), 5],
    min_border_top: [ee(V), null],
    min_border_left: [ee(V), null],
    min_border_bottom: [ee(V), null],
    min_border_right: [ee(V), null],
    inner_width: [V, unset, { readonly: !0 }],
    inner_height: [V, unset, { readonly: !0 }],
    outer_width: [V, unset, { readonly: !0 }],
    outer_height: [V, unset, { readonly: !0 }],
    match_aspect: [H, !1],
    aspect_scale: [V, 1],
    reset_policy: [ResetPolicy, "standard"],
    hold_render: [H, !1],
    attribution: [G(Z(q, te(HTML))), []]
  })), g0.override({
    width: 600,
    height: 600,
    outline_line_color: "#e5e5e5",
    border_fill_color: "#ffffff",
    background_fill_color: "#ffffff"
  });
  let Plot = g0;
  function has_maps_API() {
    return typeof google < "u" && typeof google.maps < "u";
  }
  const gmaps_ready = new Signal0({}, "gmaps_ready"), load_google_api = function(O, _) {
    window._bokeh_gmaps_callback = () => gmaps_ready.emit();
    const H = encodeURIComponent, V = document.createElement("script");
    V.type = "text/javascript", V.src = `https://maps.googleapis.com/maps/api/js?v=${H(_)}&key=${H(O)}&callback=_bokeh_gmaps_callback&loading=async`, document.body.appendChild(V);
  };
  class GMapPlotView extends PlotView {
    constructor() {
      super(...arguments);
      mt(this, "_tiles_loaded");
      mt(this, "zoom_count");
      mt(this, "initial_zoom");
      mt(this, "initial_lat");
      mt(this, "initial_lng");
      mt(this, "map_el");
      mt(this, "map");
      mt(this, "map_types");
      mt(this, "_api_key");
    }
    initialize() {
      super.initialize(), this._tiles_loaded = !1, this.zoom_count = 0;
      const { zoom: H, lat: V, lng: q } = this.model.map_options;
      this.initial_zoom = H, this.initial_lat = V, this.initial_lng = q;
      const G = new TextDecoder("utf-8");
      this._api_key = G.decode(this.model.api_key), this._api_key == "" && logger$1.error("api_key is required. See https://developers.google.com/maps/documentation/javascript/get-api-key for more information on how to obtain your own.");
    }
    async lazy_initialize() {
      if (await super.lazy_initialize(), this.map_el = div({ style: { position: "absolute" } }), this.canvas_view.add_underlay(this.map_el), has_maps_API())
        this._build_map();
      else {
        if (typeof window._bokeh_gmaps_callback > "u") {
          const { api_version: H } = this.model;
          load_google_api(this._api_key, H);
        }
        gmaps_ready.connect(() => {
          this._build_map(), this.request_paint("everything");
        });
      }
    }
    remove() {
      remove$2(this.map_el), super.remove();
    }
    update_range(H, V) {
      if (H == null)
        this.map.setCenter({ lat: this.initial_lat, lng: this.initial_lng }), this.map.setOptions({ zoom: this.initial_zoom }), super.reset_range();
      else if (H.sdx != null || H.sdy != null)
        this.map.panBy(H.sdx ?? 0, H.sdy ?? 0), super.update_range(H, V);
      else if (H.factor != null) {
        if (this.zoom_count !== 10) {
          this.zoom_count += 1;
          return;
        }
        this.zoom_count = 0, this.pause(), super.update_range(H, V);
        const q = H.factor < 0 ? -1 : 1, G = this.map.getZoom(), J = this.map.getBounds();
        if (G != null && J != null) {
          const Z = G + q;
          if (Z >= 2) {
            this.map.setZoom(Z);
            const [te, ne] = this._get_projected_bounds(J);
            ne - te < 0 && this.map.setZoom(G);
          }
        }
        this.unpause();
      }
      this._set_bokeh_ranges();
    }
    _build_map() {
      const { maps: H } = google;
      this.map_types = {
        satellite: H.MapTypeId.SATELLITE,
        terrain: H.MapTypeId.TERRAIN,
        roadmap: H.MapTypeId.ROADMAP,
        hybrid: H.MapTypeId.HYBRID
      };
      const V = this.model.map_options, q = {
        center: new H.LatLng(V.lat, V.lng),
        zoom: V.zoom,
        disableDefaultUI: !0,
        mapTypeId: this.map_types[V.map_type],
        scaleControl: V.scale_control,
        tilt: V.tilt
      };
      V.styles != null && (q.styles = JSON.parse(V.styles)), this.map = new H.Map(this.map_el, q), H.event.addListener(this.map, "idle", () => this._set_bokeh_ranges()), H.event.addListener(this.map, "bounds_changed", () => this._set_bokeh_ranges()), H.event.addListenerOnce(this.map, "tilesloaded", () => this._render_finished()), this.connect(this.model.properties.map_options.change, () => this._update_options()), this.connect(this.model.map_options.properties.styles.change, () => this._update_styling()), this.connect(this.model.map_options.properties.lat.change, () => this._update_center("lat")), this.connect(this.model.map_options.properties.lng.change, () => this._update_center("lng")), this.connect(this.model.map_options.properties.zoom.change, () => this._update_zoom()), this.connect(this.model.map_options.properties.map_type.change, () => this._update_map_type()), this.connect(this.model.map_options.properties.scale_control.change, () => this._update_scale_control()), this.connect(this.model.map_options.properties.tilt.change, () => this._update_tilt());
    }
    _render_finished() {
      this._tiles_loaded = !0, this.notify_finished();
    }
    has_finished() {
      return super.has_finished() && this._tiles_loaded === !0;
    }
    _get_latlon_bounds(H) {
      const V = H.getNorthEast(), q = H.getSouthWest(), G = q.lng(), J = V.lng(), Z = q.lat(), te = V.lat();
      return [G, J, Z, te];
    }
    _get_projected_bounds(H) {
      const [V, q, G, J] = this._get_latlon_bounds(H), [Z, te] = wgs84_mercator.compute(V, G), [ne, ee] = wgs84_mercator.compute(q, J);
      return [Z, ne, te, ee];
    }
    _set_bokeh_ranges() {
      const H = this.map.getBounds();
      if (H != null) {
        const [V, q, G, J] = this._get_projected_bounds(H);
        this.frame.x_range.setv({ start: V, end: q }), this.frame.y_range.setv({ start: G, end: J });
      }
    }
    _update_center(H) {
      var q;
      const V = (q = this.map.getCenter()) == null ? void 0 : q.toJSON();
      V != null && (V[H] = this.model.map_options[H], this.map.setCenter(V), this._set_bokeh_ranges());
    }
    _update_map_type() {
      this.map.setOptions({ mapTypeId: this.map_types[this.model.map_options.map_type] });
    }
    _update_scale_control() {
      this.map.setOptions({ scaleControl: this.model.map_options.scale_control });
    }
    _update_tilt() {
      this.map.setOptions({ tilt: this.model.map_options.tilt });
    }
    _update_options() {
      this._update_styling(), this._update_center("lat"), this._update_center("lng"), this._update_zoom(), this._update_map_type();
    }
    _update_styling() {
      const { styles: H } = this.model.map_options;
      this.map.setOptions({ styles: H != null ? JSON.parse(H) : null });
    }
    _update_zoom() {
      this.map.setOptions({ zoom: this.model.map_options.zoom }), this._set_bokeh_ranges();
    }
    _after_layout() {
      super._after_layout();
      const { left: H, top: V, width: q, height: G } = this.frame.bbox;
      this.map_el.style.top = `${V}px`, this.map_el.style.left = `${H}px`, this.map_el.style.width = `${q}px`, this.map_el.style.height = `${G}px`;
    }
  }
  mt(GMapPlotView, "__name__", "GMapPlotView");
  const i6 = class i6 extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(i6, "__name__", "MapOptions"), i6.define(({ Int: _, Float: H }) => ({
    lat: [H],
    lng: [H],
    zoom: [_, 12]
  }));
  let MapOptions = i6;
  const a6 = class a6 extends MapOptions {
    constructor(_) {
      super(_);
    }
  };
  mt(a6, "__name__", "GMapOptions"), a6.define(({ Bool: _, Int: H, Str: V, Nullable: q }) => ({
    map_type: [MapType, "roadmap"],
    scale_control: [_, !1],
    styles: [q(V), null],
    tilt: [H, 45]
  }));
  let GMapOptions = a6;
  const Vp = class Vp extends Plot {
    constructor(H) {
      super(H);
      mt(this, "use_map", !0);
    }
  };
  mt(Vp, "__name__", "GMapPlot"), Vp.prototype.default_view = GMapPlotView, Vp.define(({ Str: H, Bytes: V, Ref: q }) => ({
    map_options: [q(GMapOptions)],
    api_key: [V],
    api_version: [H, "weekly"]
  })), Vp.override({
    x_range: () => new Range1d(),
    y_range: () => new Range1d(),
    background_fill_alpha: 0
  });
  let GMapPlot = Vp;
  class GMapView extends GMapPlotView {
    // TODO: remove this before bokeh 3.0 and update *.blf files
    serializable_state() {
      return { ...super.serializable_state(), type: "GMapPlot" };
    }
  }
  mt(GMapView, "__name__", "GMapView");
  const o6 = class o6 extends GMapPlot {
    constructor(_) {
      super(_);
    }
  };
  mt(o6, "__name__", "GMap"), o6.prototype.default_view = GMapView;
  let GMap = o6;
  class GridPlotView extends LayoutDOMView {
    constructor() {
      super(...arguments);
      mt(this, "_grid_box");
      mt(this, "_tool_views", /* @__PURE__ */ new Map());
    }
    get toolbar_view() {
      return this.child_views.find((H) => H.model == this.model.toolbar);
    }
    get grid_box_view() {
      return this.child_views.find((H) => H.model == this._grid_box);
    }
    _update_location() {
      const H = this.model.toolbar_location;
      H == null ? this.model.toolbar.visible = !1 : this.model.toolbar.setv({ visible: !0, location: H });
    }
    initialize() {
      super.initialize(), this._update_location();
      const { children: H, rows: V, cols: q, spacing: G } = this.model;
      this._grid_box = new GridBox({ children: H, rows: V, cols: q, spacing: G, sizing_mode: "inherit" });
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this.build_tool_views();
    }
    connect_signals() {
      super.connect_signals();
      const { toolbar: H, toolbar_location: V, children: q, rows: G, cols: J, spacing: Z } = this.model.properties;
      this.on_change(V, async () => {
        this._update_location(), this.invalidate_layout();
      }), this.on_change([H, q, G, J, Z], async () => {
        await this.update_children();
      }), this.on_change(this.model.toolbar.properties.tools, async () => {
        await this.build_tool_views();
      }), this.mouseenter.connect(() => {
        this.toolbar_view.set_visibility(!0);
      }), this.mouseleave.connect(() => {
        this.toolbar_view.set_visibility(!1);
      });
    }
    remove() {
      remove_views(this._tool_views), super.remove();
    }
    async build_tool_views() {
      const H = this.model.toolbar.tools.filter((V) => V instanceof ActionTool);
      await build_views(this._tool_views, H, { parent: this });
    }
    *children() {
      yield* super.children(), yield* this._tool_views.values();
    }
    get child_models() {
      return [this.model.toolbar, this._grid_box];
    }
    _intrinsic_display() {
      return { inner: this.model.flow_mode, outer: "flex" };
    }
    _update_layout() {
      super._update_layout();
      const { location: H } = this.model.toolbar, V = (() => {
        switch (H) {
          case "above":
            return "column";
          case "below":
            return "column-reverse";
          case "left":
            return "row";
          case "right":
            return "row-reverse";
        }
      })();
      this.style.append(":host", { flex_direction: V });
    }
  }
  mt(GridPlotView, "__name__", "GridPlotView");
  const Sm = class Sm extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(Sm, "__name__", "GridPlot"), Sm.prototype.default_view = GridPlotView, Sm.define(({ List: _, Ref: H, Nullable: V }) => ({
    toolbar: [H(Toolbar), () => new Toolbar()],
    toolbar_location: [V(Location), "above"],
    children: [_(GridChild(LayoutDOM)), []],
    rows: [V(TracksSizing), null],
    cols: [V(TracksSizing), null],
    spacing: [GridSpacing, 0]
  }));
  let GridPlot = Sm;
  class FigureView extends PlotView {
    // TODO: remove this before bokeh 3.0 and update *.blf files
    serializable_state() {
      return { ...super.serializable_state(), type: "Plot" };
    }
  }
  mt(FigureView, "__name__", "FigureView");
  var F0;
  let Figure$1 = (F0 = class extends Plot {
    constructor(_) {
      super(_);
    }
  }, mt(F0, "__name__", "Figure"), F0.prototype.default_view = FigureView, F0);
  const s6 = class s6 extends RandomGenerator {
    constructor(_) {
      super(_);
    }
    generator() {
      return new LCGRandom(this.seed ?? Date.now());
    }
  };
  mt(s6, "__name__", "ParkMillerLCG"), s6.define(({ Int: _, Nullable: H }) => ({
    seed: [H(_), null]
  }));
  let ParkMillerLCG = s6;
  class ContourRendererView extends DataRendererView {
    constructor() {
      super(...arguments);
      mt(this, "fill_view");
      mt(this, "line_view");
    }
    *children() {
      yield* super.children(), yield this.fill_view, yield this.line_view;
    }
    get glyph_view() {
      return this.fill_view.glyph.data_size > 0 ? this.fill_view.glyph : this.line_view.glyph;
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { parent: H } = this, { fill_renderer: V, line_renderer: q } = this.model;
      this.fill_view = await build_view(V, { parent: H }), this.line_view = await build_view(q, { parent: H });
    }
    remove() {
      this.fill_view.remove(), this.line_view.remove(), super.remove();
    }
    _render() {
      this.fill_view.render(), this.line_view.render();
    }
    renderer_view(H) {
      if (H instanceof GlyphRenderer) {
        if (H == this.fill_view.model)
          return this.fill_view;
        if (H == this.line_view.model)
          return this.line_view;
      }
      return super.renderer_view(H);
    }
  }
  mt(ContourRendererView, "__name__", "ContourRendererView");
  const Em = class Em extends DataRenderer {
    constructor(_) {
      super(_);
    }
    get_selection_manager() {
      return this.fill_renderer.data_source.selection_manager;
    }
  };
  mt(Em, "__name__", "ContourRenderer"), Em.prototype.default_view = ContourRendererView, Em.define(({ List: _, Float: H, Ref: V }) => ({
    fill_renderer: [V(GlyphRenderer)],
    line_renderer: [V(GlyphRenderer)],
    levels: [_(H), []]
  }));
  let ContourRenderer = Em;
  class GraphRendererView extends DataRendererView {
    constructor() {
      super(...arguments);
      mt(this, "edge_view");
      mt(this, "node_view");
    }
    get glyph_view() {
      return this.node_view.glyph;
    }
    *children() {
      yield* super.children(), yield this.edge_view, yield this.node_view;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), this.apply_coordinates();
      const { parent: H } = this, { edge_renderer: V, node_renderer: q } = this.model;
      this.edge_view = await build_view(V, { parent: H }), this.node_view = await build_view(q, { parent: H });
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.layout_provider.change, async () => {
        this.apply_coordinates(), await this.edge_view.set_data(), await this.node_view.set_data(), this.request_render();
      });
    }
    apply_coordinates() {
      const { edge_renderer: H, node_renderer: V } = this.model, q = this.model.layout_provider.edge_coordinates, G = this.model.layout_provider.node_coordinates, J = { expr: q.x }, Z = { expr: q.y }, te = { expr: G.x }, ne = { expr: G.y }, ee = [
        H.glyph,
        H.hover_glyph,
        H.muted_glyph,
        H.selection_glyph,
        H.nonselection_glyph
      ], ae = [
        V.glyph,
        V.hover_glyph,
        V.muted_glyph,
        V.selection_glyph,
        V.nonselection_glyph
      ];
      for (const oe of ee)
        if (!(oe == null || oe == "auto")) {
          if (!(oe instanceof MultiLine || oe instanceof Patches)) {
            logger$1.warn(`${this}.edge_renderer only supports MultiLine and Patches glyphs`);
            continue;
          }
          oe.properties.xs.internal = !0, oe.properties.ys.internal = !0, oe.xs = J, oe.ys = Z;
        }
      for (const oe of ae)
        if (!(oe == null || oe == "auto")) {
          if (!(oe instanceof XYGlyph)) {
            logger$1.warn(`${this}.node_renderer only supports XY glyphs`);
            continue;
          }
          oe.properties.x.internal = !0, oe.properties.y.internal = !0, oe.x = te, oe.y = ne;
        }
    }
    remove() {
      this.edge_view.remove(), this.node_view.remove(), super.remove();
    }
    _render() {
      this.edge_view.render(), this.node_view.render();
    }
    get has_webgl() {
      return this.edge_view.has_webgl || this.node_view.has_webgl;
    }
    renderer_view(H) {
      if (H instanceof GlyphRenderer) {
        if (H == this.edge_view.model)
          return this.edge_view;
        if (H == this.node_view.model)
          return this.node_view;
      }
      return super.renderer_view(H);
    }
  }
  mt(GraphRendererView, "__name__", "GraphRendererView");
  const Am = class Am extends DataRenderer {
    constructor(_) {
      super(_);
    }
    get_selection_manager() {
      return this.node_renderer.data_source.selection_manager;
    }
  };
  mt(Am, "__name__", "GraphRenderer"), Am.prototype.default_view = GraphRendererView, Am.define(({ Ref: _ }) => ({
    layout_provider: [_(LayoutProvider)],
    node_renderer: [_(GlyphRenderer)],
    edge_renderer: [_(GlyphRenderer)],
    selection_policy: [_(GraphHitTestPolicy), () => new NodesOnly()],
    inspection_policy: [_(GraphHitTestPolicy), () => new NodesOnly()]
  }));
  let GraphRenderer = Am;
  class ByID extends Selector {
    constructor(_) {
      super(_);
    }
    find_one(_) {
      return _.querySelector(`#${this.query}`);
    }
  }
  mt(ByID, "__name__", "ByID");
  class ByClass extends Selector {
    constructor(_) {
      super(_);
    }
    find_one(_) {
      return _.querySelector(`.${this.query}`);
    }
  }
  mt(ByClass, "__name__", "ByClass");
  class ByCSS extends Selector {
    constructor(_) {
      super(_);
    }
    find_one(_) {
      return _.querySelector(this.query);
    }
  }
  mt(ByCSS, "__name__", "ByCSS");
  class ByXPath extends Selector {
    constructor(_) {
      super(_);
    }
    find_one(_) {
      return document.evaluate(this.query, _).iterateNext();
    }
  }
  mt(ByXPath, "__name__", "ByXPath");
  const l6 = class l6 extends ColumnDataSource {
    constructor(_) {
      super(_);
    }
    get_column(_) {
      return dict(this.data).get(_) ?? [];
    }
    get_length() {
      return super.get_length() ?? 0;
    }
    initialize() {
      super.initialize(), this.setup();
    }
    async load_data(_, H, V) {
      const { adapter: q } = this;
      let G;
      switch (q != null ? G = await execute(q, this, { response: _ }) : G = _, H) {
        case "replace":
          break;
        case "append": {
          const J = dict(this.data), Z = dict(G);
          for (const te of this.columns()) {
            const ne = Array.from(J.get(te) ?? []), ee = Array.from(Z.get(te) ?? []), ae = ne.concat(ee);
            Z.set(te, V != null ? ae.slice(-V) : ae);
          }
          break;
        }
      }
      this.data = G;
    }
  };
  mt(l6, "__name__", "WebDataSource"), l6.define(({ Any: _, Int: H, Str: V, Nullable: q }) => ({
    max_size: [q(H), null],
    mode: [UpdateMode, "replace"],
    adapter: [q(
      _
      /*TODO*/
    ), null],
    data_url: [V]
  }));
  let WebDataSource = l6;
  class ServerSentDataSource extends WebDataSource {
    constructor(H) {
      super(H);
      mt(this, "initialized", !1);
    }
    setup() {
      if (!this.initialized) {
        this.initialized = !0;
        const H = new EventSource(this.data_url);
        H.onmessage = async (V) => {
          await this.load_data(JSON.parse(V.data), this.mode, this.max_size ?? void 0);
        };
      }
    }
  }
  mt(ServerSentDataSource, "__name__", "ServerSentDataSource");
  const c6 = class c6 extends WebDataSource {
    constructor(H) {
      super(H);
      mt(this, "interval", null);
      mt(this, "initialized", !1);
    }
    destroy() {
      this.interval != null && clearInterval(this.interval), super.destroy();
    }
    setup() {
      if (!this.initialized && (this.initialized = !0, this.get_data(this.mode), this.polling_interval != null)) {
        const H = () => this.get_data(this.mode, this.max_size, this.if_modified);
        this.interval = setInterval(H, this.polling_interval);
      }
    }
    get_data(H, V = null, q = !1) {
      const G = this.prepare_request();
      G.addEventListener("load", () => this.do_load(G, H, V ?? void 0)), G.addEventListener("error", () => this.do_error(G)), G.send();
    }
    prepare_request() {
      const H = new XMLHttpRequest();
      H.open(this.method, this.data_url, !0), H.withCredentials = !1, H.setRequestHeader("Content-Type", this.content_type);
      for (const [V, q] of entries$2(this.http_headers))
        H.setRequestHeader(V, q);
      return H;
    }
    async do_load(H, V, q) {
      if (H.status == 200) {
        const G = JSON.parse(H.responseText);
        await this.load_data(G, V, q);
      }
    }
    do_error(H) {
      logger$1.error(`Failed to fetch JSON from ${this.data_url} with code ${H.status}`);
    }
  };
  mt(c6, "__name__", "AjaxDataSource"), c6.define(({ Bool: H, Int: V, Str: q, Dict: G, Nullable: J }) => ({
    polling_interval: [J(V), null],
    content_type: [q, "application/json"],
    http_headers: [G(q), {}],
    method: [HTTPMethod, "POST"],
    if_modified: [H, !1]
  }));
  let AjaxDataSource = c6;
  function orNaN(O) {
    return O ?? NaN;
  }
  const Qm = class Qm extends ColumnarDataSource {
    constructor(_) {
      super(_);
    }
    initialize() {
      super.initialize(), this._update_data();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.properties.geojson.change, () => this._update_data());
    }
    _update_data() {
      this.data = this.geojson_to_column_data();
    }
    _get_new_list_array(_) {
      return range$9(0, _).map((H) => []);
    }
    _get_new_nan_array(_) {
      return range$9(0, _).map((H) => NaN);
    }
    _add_properties(_, H, V, q) {
      const G = _.properties ?? {}, J = dict(H);
      for (const [Z, te] of dict(G))
        J.has(Z) || (H[Z] = this._get_new_nan_array(q)), H[Z][V] = orNaN(te);
    }
    _add_geometry(_, H, V) {
      function q(G, J) {
        return G.concat([[NaN, NaN, NaN]]).concat(J);
      }
      switch (_.type) {
        case "Point": {
          const [G, J, Z] = _.coordinates;
          H.x[V] = G, H.y[V] = J, H.z[V] = orNaN(Z);
          break;
        }
        case "LineString": {
          const { coordinates: G } = _;
          for (let J = 0; J < G.length; J++) {
            const [Z, te, ne] = G[J];
            H.xs[V][J] = Z, H.ys[V][J] = te, H.zs[V][J] = orNaN(ne);
          }
          break;
        }
        case "Polygon": {
          _.coordinates.length > 1 && logger$1.warn("Bokeh does not support Polygons with holes in, only exterior ring used.");
          const G = _.coordinates[0];
          for (let J = 0; J < G.length; J++) {
            const [Z, te, ne] = G[J];
            H.xs[V][J] = Z, H.ys[V][J] = te, H.zs[V][J] = orNaN(ne);
          }
          break;
        }
        case "MultiPoint": {
          logger$1.warn("MultiPoint not supported in Bokeh");
          break;
        }
        case "MultiLineString": {
          const G = _.coordinates.reduce(q);
          for (let J = 0; J < G.length; J++) {
            const [Z, te, ne] = G[J];
            H.xs[V][J] = Z, H.ys[V][J] = te, H.zs[V][J] = orNaN(ne);
          }
          break;
        }
        case "MultiPolygon": {
          const G = [];
          for (const Z of _.coordinates)
            Z.length > 1 && logger$1.warn("Bokeh does not support Polygons with holes in, only exterior ring used."), G.push(Z[0]);
          const J = G.reduce(q);
          for (let Z = 0; Z < J.length; Z++) {
            const [te, ne, ee] = J[Z];
            H.xs[V][Z] = te, H.ys[V][Z] = ne, H.zs[V][Z] = orNaN(ee);
          }
          break;
        }
        default:
          throw new Error(`Invalid GeoJSON geometry type: ${_.type}`);
      }
    }
    geojson_to_column_data() {
      const _ = JSON.parse(this.geojson);
      let H;
      switch (_.type) {
        case "GeometryCollection": {
          if (is_undefined(_.geometries))
            throw new Error("No geometries found in GeometryCollection");
          if (_.geometries.length === 0)
            throw new Error("geojson.geometries must have one or more items");
          H = _.geometries;
          break;
        }
        case "FeatureCollection": {
          if (is_undefined(_.features))
            throw new Error("No features found in FeaturesCollection");
          if (_.features.length == 0)
            throw new Error("geojson.features must have one or more items");
          H = _.features;
          break;
        }
        default:
          throw new Error("Bokeh only supports type GeometryCollection and FeatureCollection at top level");
      }
      let V = 0;
      for (const J of H) {
        const Z = J.type === "Feature" ? J.geometry : J;
        Z.type == "GeometryCollection" ? V += Z.geometries.length : V += 1;
      }
      const q = {
        x: this._get_new_nan_array(V),
        y: this._get_new_nan_array(V),
        z: this._get_new_nan_array(V),
        xs: this._get_new_list_array(V),
        ys: this._get_new_list_array(V),
        zs: this._get_new_list_array(V)
      };
      let G = 0;
      for (const J of H) {
        const Z = J.type == "Feature" ? J.geometry : J;
        if (Z.type == "GeometryCollection")
          for (const te of Z.geometries)
            this._add_geometry(te, q, G), J.type === "Feature" && this._add_properties(J, q, G, V), G += 1;
        else
          this._add_geometry(Z, q, G), J.type === "Feature" && this._add_properties(J, q, G, V), G += 1;
      }
      return q;
    }
  };
  mt(Qm, "__name__", "GeoJSONDataSource"), Qm.define(({ Str: _ }) => ({
    geojson: [_]
  })), Qm.internal(({ Unknown: _, Dict: H, Arrayable: V }) => ({
    data: [H(V(_)), {}]
  }));
  let GeoJSONDataSource = Qm;
  const u6 = class u6 extends Model$1 {
    constructor(H) {
      super(H);
      mt(this, "tiles");
    }
    initialize() {
      super.initialize(), this.tiles = /* @__PURE__ */ new Map(), this._normalize_case();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.change, () => this._clear_cache());
    }
    string_lookup_replace(H, V) {
      let q = H;
      for (const [G, J] of entries$2(V))
        q = q.replace(`{${G}}`, J);
      return q;
    }
    _normalize_case() {
      const H = this.url.replace("{x}", "{X}").replace("{y}", "{Y}").replace("{z}", "{Z}").replace("{q}", "{Q}").replace("{xmin}", "{XMIN}").replace("{ymin}", "{YMIN}").replace("{xmax}", "{XMAX}").replace("{ymax}", "{YMAX}");
      this.url = H;
    }
    _clear_cache() {
      this.tiles = /* @__PURE__ */ new Map();
    }
    tile_xyz_to_key(H, V, q) {
      return `${H}:${V}:${q}`;
    }
    key_to_tile_xyz(H) {
      const [V, q, G] = H.split(":").map((J) => parseInt(J));
      return [V, q, G];
    }
    sort_tiles_from_center(H, V) {
      const [q, G, J, Z] = V, te = (J - q) / 2 + q, ne = (Z - G) / 2 + G;
      H.sort(function(ee, ae) {
        const oe = Math.sqrt((te - ee[0]) ** 2 + (ne - ee[1]) ** 2), se = Math.sqrt((te - ae[0]) ** 2 + (ne - ae[1]) ** 2);
        return oe - se;
      });
    }
    get_image_url(H, V, q) {
      return this.string_lookup_replace(this.url, this.extra_url_vars).replace("{X}", H.toString()).replace("{Y}", V.toString()).replace("{Z}", q.toString());
    }
  };
  mt(u6, "__name__", "TileSource"), u6.define(({
    Float: H,
    Str: V,
    Dict: q,
    Nullable: G
    /*, Null, Or, Ref*/
  }) => ({
    url: [V, ""],
    tile_size: [H, 256],
    max_zoom: [H, 30],
    min_zoom: [H, 0],
    extra_url_vars: [q(V), {}],
    attribution: [V, ""],
    // Or(Str, Ref(DOMElement), Null), null ],
    x_origin_offset: [H],
    y_origin_offset: [H],
    initial_resolution: [G(H), null]
  }));
  let TileSource = u6;
  function meters_to_geographic(O, _) {
    return wgs84_mercator.invert(O, _);
  }
  function meters_extent_to_geographic(O) {
    const [_, H, V, q] = O, [G, J] = meters_to_geographic(_, H), [Z, te] = meters_to_geographic(V, q);
    return [G, J, Z, te];
  }
  const Cm = class Cm extends TileSource {
    constructor(H) {
      super(H);
      mt(this, "_resolutions");
    }
    initialize() {
      super.initialize(), this._resolutions = range$9(this.min_zoom, this.max_zoom + 1).map((H) => this.get_resolution(H));
    }
    _computed_initial_resolution() {
      return this.initial_resolution != null ? this.initial_resolution : 2 * Math.PI * 6378137 / this.tile_size;
    }
    is_valid_tile(H, V, q) {
      return !(!this.wrap_around && (H < 0 || H >= 2 ** q) || V < 0 || V >= 2 ** q);
    }
    parent_by_tile_xyz(H, V, q) {
      const G = this.tile_xyz_to_quadkey(H, V, q), J = G.substring(0, G.length - 1);
      return this.quadkey_to_tile_xyz(J);
    }
    get_resolution(H) {
      return this._computed_initial_resolution() / 2 ** H;
    }
    get_resolution_by_extent(H, V, q) {
      const G = (H[2] - H[0]) / q, J = (H[3] - H[1]) / V;
      return [G, J];
    }
    get_level_by_extent(H, V, q) {
      const G = (H[2] - H[0]) / q, J = (H[3] - H[1]) / V, Z = Math.max(G, J);
      let te = 0;
      for (const ne of this._resolutions) {
        if (Z > ne) {
          if (te == 0)
            return 0;
          if (te > 0)
            return te - 1;
        }
        te += 1;
      }
      return te - 1;
    }
    get_closest_level_by_extent(H, V, q) {
      const G = (H[2] - H[0]) / q, J = (H[3] - H[1]) / V, Z = Math.max(G, J), te = this._resolutions.reduce(function(ne, ee) {
        return Math.abs(ee - Z) < Math.abs(ne - Z) ? ee : ne;
      });
      return this._resolutions.indexOf(te);
    }
    snap_to_zoom_level(H, V, q, G) {
      const [J, Z, te, ne] = H, ee = this._resolutions[G];
      let ae = q * ee, oe = V * ee;
      if (!this.snap_to_zoom) {
        const le = (te - J) / ae, ue = (ne - Z) / oe;
        le > ue ? (ae = te - J, oe = oe * le) : (ae = ae * ue, oe = ne - Z);
      }
      const se = (ae - (te - J)) / 2, ce = (oe - (ne - Z)) / 2;
      return [J - se, Z - ce, te + se, ne + ce];
    }
    tms_to_wmts(H, V, q) {
      return [H, 2 ** q - 1 - V, q];
    }
    wmts_to_tms(H, V, q) {
      return [H, 2 ** q - 1 - V, q];
    }
    pixels_to_meters(H, V, q) {
      const G = this.get_resolution(q), J = H * G - this.x_origin_offset, Z = V * G - this.y_origin_offset;
      return [J, Z];
    }
    meters_to_pixels(H, V, q) {
      const G = this.get_resolution(q), J = (H + this.x_origin_offset) / G, Z = (V + this.y_origin_offset) / G;
      return [J, Z];
    }
    pixels_to_tile(H, V) {
      let q = Math.ceil(H / this.tile_size);
      q = q === 0 ? q : q - 1;
      const G = Math.max(Math.ceil(V / this.tile_size) - 1, 0);
      return [q, G];
    }
    pixels_to_raster(H, V, q) {
      const G = this.tile_size << q;
      return [H, G - V];
    }
    meters_to_tile(H, V, q) {
      const [G, J] = this.meters_to_pixels(H, V, q);
      return this.pixels_to_tile(G, J);
    }
    get_tile_meter_bounds(H, V, q) {
      const [G, J] = this.pixels_to_meters(H * this.tile_size, V * this.tile_size, q), [Z, te] = this.pixels_to_meters((H + 1) * this.tile_size, (V + 1) * this.tile_size, q);
      return [G, J, Z, te];
    }
    get_tile_geographic_bounds(H, V, q) {
      const G = this.get_tile_meter_bounds(H, V, q), [J, Z, te, ne] = meters_extent_to_geographic(G);
      return [J, Z, te, ne];
    }
    get_tiles_by_extent(H, V, q = 1) {
      const [G, J, Z, te] = H;
      let [ne, ee] = this.meters_to_tile(G, J, V), [ae, oe] = this.meters_to_tile(Z, te, V);
      ne -= q, ee -= q, ae += q, oe += q;
      const se = [];
      for (let ce = oe; ce >= ee; ce--)
        for (let le = ne; le <= ae; le++)
          this.is_valid_tile(le, ce, V) && se.push([le, ce, V, this.get_tile_meter_bounds(le, ce, V)]);
      return this.sort_tiles_from_center(se, [ne, ee, ae, oe]), se;
    }
    quadkey_to_tile_xyz(H) {
      let V = 0, q = 0;
      const G = H.length;
      for (let J = G; J > 0; J--) {
        const Z = H.charAt(G - J), te = 1 << J - 1;
        switch (Z) {
          case "0":
            continue;
          case "1":
            V |= te;
            break;
          case "2":
            q |= te;
            break;
          case "3":
            V |= te, q |= te;
            break;
          default:
            throw new TypeError(`Invalid Quadkey: ${H}`);
        }
      }
      return [V, q, G];
    }
    tile_xyz_to_quadkey(H, V, q) {
      let G = "";
      for (let J = q; J > 0; J--) {
        const Z = 1 << J - 1;
        let te = 0;
        H & Z && (te += 1), V & Z && (te += 2), G += te.toString();
      }
      return G;
    }
    children_by_tile_xyz(H, V, q) {
      const G = this.tile_xyz_to_quadkey(H, V, q), J = [];
      for (let Z = 0; Z <= 3; Z++) {
        const [te, ne, ee] = this.quadkey_to_tile_xyz(G + Z.toString()), ae = this.get_tile_meter_bounds(te, ne, ee);
        J.push([te, ne, ee, ae]);
      }
      return J;
    }
    get_closest_parent_by_tile_xyz(H, V, q) {
      const G = this.calculate_world_x_by_tile_xyz(H, V, q);
      [H, V, q] = this.normalize_xyz(H, V, q);
      let J = this.tile_xyz_to_quadkey(H, V, q);
      for (; J.length > 0; )
        if (J = J.substring(0, J.length - 1), [H, V, q] = this.quadkey_to_tile_xyz(J), [H, V, q] = this.denormalize_xyz(H, V, q, G), this.tiles.has(this.tile_xyz_to_key(H, V, q)))
          return [H, V, q];
      return [0, 0, 0];
    }
    normalize_xyz(H, V, q) {
      if (this.wrap_around) {
        const G = 2 ** q;
        return [(H % G + G) % G, V, q];
      } else
        return [H, V, q];
    }
    denormalize_xyz(H, V, q, G) {
      return [H + G * 2 ** q, V, q];
    }
    denormalize_meters(H, V, q, G) {
      return [H + G * 2 * Math.PI * 6378137, V];
    }
    calculate_world_x_by_tile_xyz(H, V, q) {
      return Math.floor(H / 2 ** q);
    }
  };
  mt(Cm, "__name__", "MercatorTileSource"), Cm.define(({ Bool: H }) => ({
    snap_to_zoom: [H, !1],
    wrap_around: [H, !0]
  })), Cm.override({
    x_origin_offset: 2003750834e-2,
    y_origin_offset: 2003750834e-2,
    initial_resolution: 156543.03392804097
  });
  let MercatorTileSource = Cm;
  const d6 = class d6 extends MercatorTileSource {
    constructor(_) {
      super(_);
    }
    get_image_url(_, H, V) {
      const q = this.string_lookup_replace(this.url, this.extra_url_vars);
      let G, J, Z, te;
      return this.use_latlon ? [J, te, G, Z] = this.get_tile_geographic_bounds(_, H, V) : [J, te, G, Z] = this.get_tile_meter_bounds(_, H, V), q.replace("{XMIN}", J.toString()).replace("{YMIN}", te.toString()).replace("{XMAX}", G.toString()).replace("{YMAX}", Z.toString());
    }
  };
  mt(d6, "__name__", "BBoxTileSource"), d6.define(({ Bool: _ }) => ({
    use_latlon: [_, !1]
  }));
  let BBoxTileSource = d6;
  class QUADKEYTileSource extends MercatorTileSource {
    constructor(_) {
      super(_);
    }
    get_image_url(_, H, V) {
      const q = this.string_lookup_replace(this.url, this.extra_url_vars), [G, J, Z] = this.tms_to_wmts(_, H, V), te = this.tile_xyz_to_quadkey(G, J, Z);
      return q.replace("{Q}", te);
    }
  }
  mt(QUADKEYTileSource, "__name__", "QUADKEYTileSource");
  class WMTSTileSource extends MercatorTileSource {
    constructor(_) {
      super(_);
    }
    get_image_url(_, H, V) {
      const q = this.string_lookup_replace(this.url, this.extra_url_vars), [G, J, Z] = this.tms_to_wmts(_, H, V);
      return q.replace("{X}", G.toString()).replace("{Y}", J.toString()).replace("{Z}", Z.toString());
    }
  }
  mt(WMTSTileSource, "__name__", "WMTSTileSource");
  class TileRendererView extends RendererView {
    constructor() {
      super(...arguments);
      mt(this, "_tiles", null);
      mt(this, "extent");
      mt(this, "initial_extent");
      mt(this, "_last_height");
      mt(this, "_last_width");
      mt(this, "map_initialized");
      mt(this, "render_timer");
      mt(this, "prefetch_timer");
    }
    mark_finished() {
      super.mark_finished(), this._tiles = [];
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.request_render()), this.connect(this.model.tile_source.change, () => this.request_render());
    }
    get_extent() {
      const { x_range: H, y_range: V } = this, q = H.start, G = V.start, J = H.end, Z = V.end;
      return assert$2(isFinite(q)), assert$2(isFinite(G)), assert$2(isFinite(J)), assert$2(isFinite(Z)), [q, G, J, Z];
    }
    get map_plot() {
      return this.plot_model;
    }
    get map_canvas() {
      return this.layer.ctx;
    }
    get map_frame() {
      return this.plot_view.frame;
    }
    get x_range() {
      return this.map_plot.x_range;
    }
    get y_range() {
      return this.map_plot.y_range;
    }
    _set_data() {
      this.extent = this.get_extent(), this._last_height = void 0, this._last_width = void 0;
    }
    get attribution() {
      return new HTML({ html: [this.model.tile_source.attribution] });
    }
    _map_data() {
      this.initial_extent = this.get_extent();
      const H = this.model.tile_source.get_level_by_extent(this.initial_extent, this.map_frame.bbox.height, this.map_frame.bbox.width), V = this.model.tile_source.snap_to_zoom_level(this.initial_extent, this.map_frame.bbox.height, this.map_frame.bbox.width, H);
      this.x_range.start = V[0], this.y_range.start = V[1], this.x_range.end = V[2], this.y_range.end = V[3], this.x_range instanceof Range1d && (this.x_range.reset_start = V[0], this.x_range.reset_end = V[2]), this.y_range instanceof Range1d && (this.y_range.reset_start = V[1], this.y_range.reset_end = V[3]);
    }
    _create_tile(H, V, q, G, J = !1) {
      const Z = this.model.tile_source.tile_xyz_to_quadkey(H, V, q), te = this.model.tile_source.tile_xyz_to_key(H, V, q);
      if (this.model.tile_source.tiles.has(te))
        return;
      const [ne, ee, ae] = this.model.tile_source.normalize_xyz(H, V, q), oe = this.model.tile_source.get_image_url(ne, ee, ae), se = {
        img: void 0,
        tile_coords: [H, V, q],
        normalized_coords: [ne, ee, ae],
        quadkey: Z,
        cache_key: te,
        bounds: G,
        loaded: !1,
        finished: !1,
        x_coord: G[0],
        y_coord: G[3]
      };
      this.model.tile_source.tiles.set(te, se), this._tiles == null && (this._tiles = []), this._tiles.push(se), new ImageLoader(oe, {
        loaded: (ce) => {
          Object.assign(se, { img: ce, loaded: !0 }), J ? (se.finished = !0, this.notify_finished()) : this.request_render();
        },
        failed() {
          se.finished = !0;
        }
      });
    }
    _enforce_aspect_ratio() {
      if (this._last_height !== this.map_frame.bbox.height || this._last_width !== this.map_frame.bbox.width) {
        const H = this.get_extent(), V = this.model.tile_source.get_level_by_extent(H, this.map_frame.bbox.height, this.map_frame.bbox.width), q = this.model.tile_source.snap_to_zoom_level(H, this.map_frame.bbox.height, this.map_frame.bbox.width, V);
        this.x_range.setv({ start: q[0], end: q[2] }), this.y_range.setv({ start: q[1], end: q[3] }), this.extent = q, this._last_height = this.map_frame.bbox.height, this._last_width = this.map_frame.bbox.width;
      }
    }
    has_finished() {
      if (!super.has_finished() || this._tiles == null)
        return !1;
      for (const H of this._tiles)
        if (!H.finished)
          return !1;
      return !0;
    }
    _render() {
      this.map_initialized == null && (this._set_data(), this._map_data(), this.map_initialized = !0), this._enforce_aspect_ratio(), this._update(), this.prefetch_timer != null && clearTimeout(this.prefetch_timer), this.prefetch_timer = setTimeout(this._prefetch_tiles.bind(this), 500), this.has_finished() && this.notify_finished();
    }
    _draw_tile(H) {
      const V = this.model.tile_source.tiles.get(H);
      if (V != null && V.loaded) {
        const [[q], [G]] = this.coordinates.map_to_screen([V.bounds[0]], [V.bounds[3]]), [[J], [Z]] = this.coordinates.map_to_screen([V.bounds[2]], [V.bounds[1]]), te = J - q, ne = Z - G, ee = q, ae = G, oe = this.map_canvas.imageSmoothingEnabled;
        this.map_canvas.imageSmoothingEnabled = this.model.smoothing, this.map_canvas.drawImage(V.img, ee, ae, te, ne), this.map_canvas.imageSmoothingEnabled = oe, V.finished = !0;
      }
    }
    _set_rect() {
      const H = this.plot_model.outline_line_width, V = this.map_frame.bbox.left + H / 2, q = this.map_frame.bbox.top + H / 2, G = this.map_frame.bbox.width - H, J = this.map_frame.bbox.height - H;
      this.map_canvas.rect(V, q, G, J), this.map_canvas.clip();
    }
    _render_tiles(H) {
      this.map_canvas.save(), this._set_rect(), this.map_canvas.globalAlpha = this.model.alpha;
      for (const V of H)
        this._draw_tile(V);
      this.map_canvas.restore();
    }
    _prefetch_tiles() {
      const { tile_source: H } = this.model, V = this.get_extent(), q = this.map_frame.bbox.height, G = this.map_frame.bbox.width, J = this.model.tile_source.get_level_by_extent(V, q, G), Z = this.model.tile_source.get_tiles_by_extent(V, J);
      for (let te = 0, ne = Math.min(10, Z.length); te < ne; te++) {
        const [ee, ae, oe] = Z[te], se = this.model.tile_source.children_by_tile_xyz(ee, ae, oe);
        for (const ce of se) {
          const [le, ue, de, fe] = ce;
          H.tiles.has(H.tile_xyz_to_key(le, ue, de)) || this._create_tile(le, ue, de, fe, !0);
        }
      }
    }
    _fetch_tiles(H) {
      for (const V of H) {
        const [q, G, J, Z] = V;
        this._create_tile(q, G, J, Z);
      }
    }
    _update() {
      const { tile_source: H } = this.model, { min_zoom: V } = H, { max_zoom: q } = H;
      let G = this.get_extent();
      const J = this.extent[2] - this.extent[0] < G[2] - G[0], Z = this.map_frame.bbox.height, te = this.map_frame.bbox.width;
      let ne = H.get_level_by_extent(G, Z, te), ee = !1;
      ne < V ? (G = this.extent, ne = V, ee = !0) : ne > q && (G = this.extent, ne = q, ee = !0), ee && (this.x_range.setv({ start: G[0], end: G[2] }), this.y_range.setv({ start: G[1], end: G[3] })), this.extent = G;
      const ae = H.get_tiles_by_extent(G, ne), oe = [], se = [], ce = [], le = [];
      for (const ue of ae) {
        const [de, fe, he] = ue, me = H.tile_xyz_to_key(de, fe, he), pe = H.tiles.get(me);
        if (pe != null && pe.loaded)
          se.push(me);
        else if (this.model.render_parents) {
          const [ye, ge, ve] = H.get_closest_parent_by_tile_xyz(de, fe, he), be = H.tile_xyz_to_key(ye, ge, ve), Ce = H.tiles.get(be);
          if (Ce != null && Ce.loaded && !includes(ce, be) && ce.push(be), J) {
            const Se = H.children_by_tile_xyz(de, fe, he);
            for (const [we, Ee, xe] of Se) {
              const Le = H.tile_xyz_to_key(we, Ee, xe);
              H.tiles.has(Le) && le.push(Le);
            }
          }
        }
        pe == null && oe.push(ue);
      }
      this._render_tiles(ce), this._render_tiles(le), this._render_tiles(se), this.render_timer != null && clearTimeout(this.render_timer), this.render_timer = setTimeout(() => this._fetch_tiles(oe), 65);
    }
  }
  mt(TileRendererView, "__name__", "TileRendererView");
  const Up = class Up extends Renderer$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Up, "__name__", "TileRenderer"), Up.prototype.default_view = TileRendererView, Up.define(({ Bool: _, Float: H, Ref: V }) => ({
    alpha: [H, 1],
    smoothing: [_, !0],
    tile_source: [V(TileSource), () => new WMTSTileSource()],
    render_parents: [_, !0]
  })), Up.override({
    level: "image"
  });
  let TileRenderer = Up;
  class TMSTileSource extends MercatorTileSource {
    constructor(_) {
      super(_);
    }
    get_image_url(_, H, V) {
      return this.string_lookup_replace(this.url, this.extra_url_vars).replace("{X}", _.toString()).replace("{Y}", H.toString()).replace("{Z}", V.toString());
    }
  }
  mt(TMSTileSource, "__name__", "TMSTileSource");
  const f6 = class f6 extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(f6, "__name__", "Texture"), f6.define(() => ({
    repetition: [TextureRepetition, "repeat"]
  }));
  let Texture = f6;
  const h6 = class h6 extends Texture {
    constructor(_) {
      super(_);
    }
    get func() {
      const _ = use_strict(this.code);
      return new Function("ctx", "color", "scale", "weight", _);
    }
    get_pattern(_, H, V) {
      const q = document.createElement("canvas");
      q.width = H, q.height = H;
      const G = q.getContext("2d");
      return this.func.call(this, G, _, H, V), q;
    }
  };
  mt(h6, "__name__", "CanvasTexture"), h6.define(({ Str: _ }) => ({
    code: [_]
  }));
  let CanvasTexture = h6;
  const p6 = class p6 extends Texture {
    constructor(H) {
      super(H);
      mt(this, "_loader");
    }
    initialize() {
      super.initialize(), this._loader = new ImageLoader(this.url);
    }
    get_pattern(H, V, q) {
      const { _loader: G } = this;
      return this._loader.finished ? G.image : G.promise;
    }
  };
  mt(p6, "__name__", "ImageURLTexture"), p6.define(({ Str: H }) => ({
    url: [H]
  }));
  let ImageURLTexture = p6;
  class IconView extends UIElementView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => this.render());
    }
  }
  mt(IconView, "__name__", "IconView");
  const m6 = class m6 extends UIElement {
    constructor(_) {
      super(_);
    }
  };
  mt(m6, "__name__", "Icon"), m6.define(({ Float: _, Or: H, CSSLength: V }) => ({
    size: [H(_, V), "1em"]
  }));
  let Icon = m6;
  class BuiltinIconView extends IconView {
    constructor() {
      super(...arguments);
      mt(this, "_style", new InlineStyleSheet$1());
    }
    stylesheets() {
      return [...super.stylesheets(), icons_css, this._style];
    }
    render() {
      super.render();
      const H = `var(--bokeh-icon-${this.model.icon_name})`, V = color2css(this.model.color), q = (() => {
        const { size: G } = this.model;
        return isNumber$3(G) ? `${G}px` : G;
      })();
      this._style.replace(`
      :host {
        display: inline-block;
        vertical-align: middle;
        width: ${q};
        height: ${q};
        background-color: ${V};
        mask-image: ${H};
        mask-size: contain;
        mask-repeat: no-repeat;
        -webkit-mask-image: ${H};
        -webkit-mask-size: contain;
        -webkit-mask-repeat: no-repeat;
      }
    `);
    }
  }
  mt(BuiltinIconView, "__name__", "BuiltinIconView");
  const Lm = class Lm extends Icon {
    constructor(_) {
      super(_);
    }
  };
  mt(Lm, "__name__", "BuiltinIcon"), Lm.prototype.default_view = BuiltinIconView, Lm.define(({ Str: _, Color: H }) => ({
    icon_name: [_],
    color: [H, "gray"]
  }));
  let BuiltinIcon = Lm;
  class SVGIconView extends IconView {
    constructor() {
      super(...arguments);
      mt(this, "_style", new InlineStyleSheet$1());
    }
    stylesheets() {
      return [...super.stylesheets(), this._style];
    }
    render() {
      super.render();
      const H = (() => {
        const { size: G } = this.model;
        return isNumber$3(G) ? `${G}px` : G;
      })();
      this._style.replace(`
      :host {
        display: inline-block;
        vertical-align: middle;
      }
      :host svg {
        width: ${H};
        height: ${H};
      }
    `);
      const q = new DOMParser().parseFromString(this.model.svg, "image/svg+xml");
      this.shadow_el.append(q.documentElement);
    }
  }
  mt(SVGIconView, "__name__", "SVGIconView");
  const km = class km extends Icon {
    constructor(_) {
      super(_);
    }
  };
  mt(km, "__name__", "SVGIcon"), km.prototype.default_view = SVGIconView, km.define(({ Str: _ }) => ({
    svg: [_]
  }));
  let SVGIcon = km;
  const $f = class $f extends IconView {
    constructor() {
      super(...arguments);
      mt(this, "_tabler", new ImportedStyleSheet$1(`${$f._url}/tabler-icons.min.css`));
      mt(this, "_style", new InlineStyleSheet$1());
    }
    stylesheets() {
      return [...super.stylesheets(), $f._fonts, this._tabler, this._style];
    }
    render() {
      super.render();
      const H = (() => {
        const { size: q } = this.model;
        return isNumber$3(q) ? `${q}px` : q;
      })();
      this._style.replace(`
      :host {
        display: inline-block;
        vertical-align: middle;
        font-size: ${H};
      }
    `);
      const V = span$1({ class: ["ti", `ti-${this.model.icon_name}`] });
      this.shadow_el.appendChild(V);
    }
  };
  mt($f, "__name__", "TablerIconView"), mt($f, "_url", "https://cdn.jsdelivr.net/npm/@tabler/icons-webfont@latest"), mt($f, "_fonts", new GlobalInlineStyleSheet$1(`    /*!
    * Tabler Icons 1.68.0 by tabler - https://tabler.io
    * License - https://github.com/tabler/tabler-icons/blob/master/LICENSE
    */
  @font-face {
    font-family: "tabler-icons";
    font-style: normal;
    font-weight: 400;
    src: url("${$f._url}/fonts/tabler-icons.eot");
    src: url("${$f._url}/fonts/tabler-icons.eot?#iefix") format("embedded-opentype"),
         url("${$f._url}/fonts/tabler-icons.woff2") format("woff2"),
         url("${$f._url}/fonts/tabler-icons.woff") format("woff"),
         url("${$f._url}/fonts/tabler-icons.ttf") format("truetype"),
         url("${$f._url}/fonts/tabler-icons.svg#tabler-icons") format("svg");
  }

  @media screen and (-webkit-min-device-pixel-ratio: 0) {
    @font-face {
      font-family: "tabler-icons";
      src: url("${$f._url}/fonts/tabler-icons.svg#tabler-icons") format("svg");
    }
  }
`));
  let TablerIconView = $f;
  const Mm = class Mm extends Icon {
    constructor(_) {
      super(_);
    }
  };
  mt(Mm, "__name__", "TablerIcon"), Mm.prototype.default_view = TablerIconView, Mm.define(({ Str: _ }) => ({
    icon_name: [_]
  }));
  let TablerIcon = Mm;
  const examiner_css = `:root{--common-padding:3px;--common-outline:#1a73e8 solid 1px;--panel-bg-color:#f1f3f4;--panel-border-color:#cacdd1;}.ref{cursor:pointer;}.monospace{font-family:var(--mono-font);}.hidden{display:none !important;}.col{display:flex;flex-direction:column;}.row{display:flex;flex-direction:row;}.toolbar{display:flex;flex-direction:row;gap:1em;background-color:var(--panel-bg-color);border-bottom:1px solid var(--panel-border-color);padding:var(--common-padding);}.checkbox{display:flex;flex-direction:row;align-items:center;gap:0.25em;}.filter:focus{outline:var(--common-outline);}.examiner{height:100%;display:flex;border:1px solid var(--panel-border-color);}.models-panel{display:flex;flex-direction:column;border-right:1px solid var(--panel-border-color);}.props-panel{display:flex;flex-direction:column;height:100%;overflow:auto;}.watches-panel{display:flex;flex-direction:column;border-bottom:1px solid var(--panel-border-color);}.models-list{display:flex;flex-direction:column;height:min-content;padding:var(--common-padding);overflow-x:hidden;overflow-y:auto;}.props-list{display:grid;grid-template-columns:min-content min-content 1fr 1fr;column-gap:1em;padding:var(--common-padding);}.watches-list{display:grid;grid-template-columns:1fr 1fr;column-gap:1em;padding:var(--common-padding);}.nothing{grid-column:1 / span 2;font-style:italic;text-align:center;}.prop{display:contents;}.prop.closed{display:none;}.prop > *{opacity:0.6;}.prop.dirty > *{opacity:1;}.model-ref{display:flex;align-items:center;flex-direction:row;font-family:var(--mono-font);}.model-ref:focus-visible{outline:var(--common-outline);}.model-ref:hover{background-color:#e2e2e2;}.model-ref.active{background-color:#c8e0ee;}.tag{margin-left:1em;padding:0 4px;font-size:60%;border-width:1px;border-style:solid;border-radius:4px;color:#202124;background-color:#f1f3f4;border-color:#cacdd1;}.expander{margin:0 2px;display:inline-block;vertical-align:middle;background-color:#5f6368;--open-image:url('data:image/svg+xml;utf8,    <svg xmlns="http://www.w3.org/2000/svg" width="6" height="6">      <path d="M 0 1 L 6 1 3 5 Z"/>    </svg>');--closed-image:url('data:image/svg+xml;utf8,    <svg xmlns="http://www.w3.org/2000/svg" width="6" height="6">      <path d="M 1 0 L 5 3 1 6 Z"/>    </svg>');}.expander{width:6px;height:6px;mask-image:var(--open-image);-webkit-mask-image:var(--open-image);}.expander.closed{width:6px;height:6px;mask-image:var(--closed-image);-webkit-mask-image:var(--closed-image);}.base{grid-column:1 / span 4;color:#5f6368;cursor:pointer;}.prop-attr,.prop-conns,.prop-kind,.prop-value{display:flex;flex-direction:row;align-items:center;overflow:hidden;white-space:nowrap;text-overflow:ellipsis;font-family:var(--mono-font);}.prop-attr > input[type="checkbox"]{visibility:hidden;margin-right:0.25em;}.prop-attr > input[type="checkbox"]:checked,.prop-attr:hover > input{visibility:visible;}.prop-attr:focus-visible{outline:var(--common-outline);}`, nullish = "bk-nullish", token = "bk-token", boolean = "bk-boolean", number$8 = "bk-number", string$3 = "bk-string", symbol$2 = "bk-symbol", model = "bk-model", attr$6 = "bk-attr", array$6 = "bk-array", object$2 = "bk-object", iterable = "bk-iterable", pretty_css = ".bk-nullish{color:#7724c1;}.bk-token{color:#881280;}.bk-boolean{color:#007500;}.bk-number{color:#1a1aa6;}.bk-string{color:#994500;}.bk-symbol{color:#c80000;}.bk-model{color:initial;}.bk-attr{color:#c80000;}.bk-array{color:initial;}.bk-object{color:initial;}.bk-iterable{color:initial;}";
  class HTMLPrinter {
    constructor(_, H = 5, V = 3) {
      mt(this, "click");
      mt(this, "max_items");
      mt(this, "max_depth");
      mt(this, "visited", /* @__PURE__ */ new WeakSet());
      mt(this, "depth", 0);
      this.click = _, this.max_items = H, this.max_depth = V;
    }
    to_html(_) {
      if (isObject$3(_)) {
        if (this.visited.has(_))
          return span$1("<circular>");
        this.visited.add(_);
      }
      return _ == null ? this.null() : isBoolean$4(_) ? this.boolean(_) : isNumber$3(_) ? this.number(_) : isString$2(_) ? this.string(_) : isSymbol$1(_) ? this.symbol(_) : _ instanceof Model$1 ? this.model(_) : _ instanceof Property$1 ? this.property(_) : isPlainObject(_) ? this.object(_) : isArray$3(_) ? this.array(_) : isIterable$2(_) ? this.iterable(_) : span$1(to_string(_));
    }
    null() {
      return span$1({ class: nullish }, "null");
    }
    token(_) {
      return span$1({ class: token }, _);
    }
    boolean(_) {
      return span$1({ class: boolean }, `${_}`);
    }
    number(_) {
      return span$1({ class: number$8 }, `${_}`);
    }
    string(_) {
      const H = _.includes("'"), V = _.includes('"'), q = H && V ? `\`${_.replace(/`/g, "\\`")}\`` : V ? `'${_}'` : `"${_}"`;
      return span$1({ class: string$3 }, q);
    }
    symbol(_) {
      return span$1({ class: symbol$2 }, _.toString());
    }
    array(_) {
      const H = this.token, V = [];
      let q = 0;
      for (const G of _)
        if (V.push(this.to_html(G)), q++ > this.max_items) {
          V.push(span$1("…"));
          break;
        }
      return span$1({ class: array$6 }, H("["), ...interleave$1(V, () => H(", ")), H("]"));
    }
    iterable(_) {
      const H = this.token, V = Object(_)[Symbol.toStringTag] ?? "Object", q = this.array([..._]);
      return span$1({ class: iterable }, `${V}`, H("("), q, H(")"));
    }
    object(_) {
      const H = this.token, V = [];
      let q = 0;
      for (const [G, J] of entries$2(_))
        if (V.push(span$1(`${G}`, H(": "), this.to_html(J))), q++ > this.max_items) {
          V.push(span$1("…"));
          break;
        }
      return span$1({ class: object$2 }, H("{"), ...interleave$1(V, () => H(", ")), H("}"));
    }
    model(_) {
      const H = this.token, V = span$1({ class: model }, _.constructor.__qualified__, H("("), this.to_html(_.id), H(")")), { click: q } = this;
      return q != null && (V.classList.add("ref"), V.addEventListener("click", () => q(_))), V;
    }
    property(_) {
      const H = this.model(_.obj), V = span$1({ class: attr$6 }, _.attr);
      return span$1(H, this.token("."), V);
    }
  }
  mt(HTMLPrinter, "__name__", "HTMLPrinter");
  class ExaminerView extends UIElementView {
    constructor() {
      super(...arguments);
      mt(this, "prev_listener", null);
      mt(this, "watched_props", /* @__PURE__ */ new Set());
    }
    stylesheets() {
      return [...super.stylesheets(), pretty_css, examiner_css];
    }
    render() {
      super.render(), this.prev_listener != null && diagnostics.disconnect(this.prev_listener);
      const H = [], V = [], q = [], G = /* @__PURE__ */ new WeakMap(), J = (Se) => {
        if (!(Se instanceof Property$1))
          return;
        function we(xe) {
          const Le = G.get(xe);
          Le != null && Le.cancel();
          const Qe = xe.animate([
            { backgroundColor: "#def189" },
            { backgroundColor: "initial" }
          ], { duration: 2e3 });
          G.set(xe, Qe);
        }
        function Ee(xe, Le, Qe) {
          Le.classList.toggle("dirty", xe.dirty), empty$1(Qe);
          const ke = xe.is_unset ? span$1("unset") : ye(xe.get_value());
          Qe.appendChild(ke), we(Qe);
        }
        for (const [xe, Le] of H)
          xe == Se.obj && we(Le);
        for (const [xe, Le] of V)
          if (xe == Se) {
            const [, , , Qe] = Le.children;
            Ee(xe, Le, Qe);
            break;
          }
        for (const [xe, Le] of q)
          if (xe == Se) {
            const [, Qe] = Le.children;
            Ee(xe, Le, Qe);
            break;
          }
      };
      diagnostics.connect(J);
      const Z = (() => {
        const Se = input$1({ class: "filter", type: "text", placeholder: "Filter" });
        return Se.addEventListener("keyup", () => {
          const we = Se.value;
          for (const [Ee, xe] of H) {
            const Le = Ee.constructor.__qualified__.includes(we);
            xe.classList.toggle("hidden", !Le);
          }
        }), div({ class: "toolbar" }, Se);
      })(), te = input$1({ type: "checkbox", checked: !0 }), ne = input$1({ type: "checkbox", checked: !0 }), ee = () => {
        for (const [Se, we] of V) {
          const Ee = te.checked, xe = ne.checked, Le = !Se.dirty && !Ee || Se.internal && !xe;
          we.classList.toggle("hidden", Le);
        }
      };
      te.addEventListener("change", () => ee()), ne.addEventListener("change", () => ee());
      const ae = (() => {
        const Se = input$1({ class: "filter", type: "text", placeholder: "Filter" }), we = span$1({ class: "checkbox" }, input$1({ type: "checkbox", checked: !0 }), span$1("Group")), Ee = span$1({ class: "checkbox" }, te, span$1("Initial?")), xe = span$1({ class: "checkbox" }, ne, span$1("Internal?"));
        return Se.addEventListener("keyup", () => {
          const Le = Se.value;
          for (const [Qe, ke] of V) {
            const Me = Qe.attr.includes(Le);
            ke.classList.toggle("hidden", !Me);
          }
        }), div({ class: "toolbar" }, Se, we, Ee, xe);
      })(), oe = (() => {
        const Se = input$1({ class: "filter", type: "text", placeholder: "Filter" });
        return Se.addEventListener("keyup", () => {
          const we = Se.value;
          for (const [Ee, xe] of q) {
            const Le = Ee.attr.includes(we);
            xe.classList.toggle("hidden", !Le);
          }
        }), div({ class: "toolbar" }, Se);
      })(), se = div({ class: "models-list" }), ce = div({ class: "props-list" }), le = div({ class: "watches-list" }), ue = div({ class: "models-panel" }, Z, se), de = div({ class: "props-panel" }, ae, ce), fe = div({ class: "watches-panel" }, oe, le), he = div({ class: "col", style: { width: "100%" } }, fe, de), me = div({ class: "examiner" }, ue, he);
      function pe(Se) {
        Se instanceof Model$1 && ve(Se);
      }
      function ye(Se) {
        return new HTMLPrinter(pe).to_html(Se);
      }
      const ge = (Se, we) => {
        clear$1(H), empty$1(se);
        const Ee = we != null ? new Set(we.roots()) : /* @__PURE__ */ new Set();
        for (const xe of Se) {
          const Le = Ee.has(xe) ? span$1({ class: "tag" }, "root") : null, Qe = span$1({ class: "model-ref", tabIndex: 0 }, ye(xe), Le);
          Qe.addEventListener("keydown", (ke) => {
            ke.key == "Enter" && ve(xe);
          }), H.push([xe, Qe]), se.appendChild(Qe);
        }
      }, ve = (Se) => {
        clear$1(V), empty$1(ce);
        for (const [xe, Le] of H)
          Le.classList.toggle("active", Se == xe);
        const we = (() => {
          const xe = [];
          let Le = Object.getPrototypeOf(Se);
          do
            xe.push([Le.constructor, keys$5(Le._props)]), Le = Object.getPrototypeOf(Le);
          while (Le.constructor != HasProps);
          xe.reverse();
          const Qe = [];
          for (const [, ke] of xe)
            ke.splice(0, Qe.length), Qe.push(...ke);
          return xe;
        })(), Ee = receivers_for_sender.get(Se) ?? [];
        for (const [xe, Le] of we) {
          if (Le.length == 0)
            continue;
          const Qe = span$1({ class: ["expander"] }), ke = div({ class: "base" }, Qe, "inherited from", " ", span$1({ class: "monospace" }, xe.__qualified__));
          ce.appendChild(ke);
          const Me = [];
          for (const Re of Le) {
            const De = Se.property(Re), Pe = De.kind.toString(), $e = De.is_unset ? span$1("unset") : ye(De.get_value()), qe = De.internal ? span$1({ class: "tag" }, "internal") : null, Xe = Ee.filter((bt) => bt.signal == De.change).length, Ve = Xe != 0 ? span$1({ class: "tag" }, `${Xe}`) : null, Ze = this.watched_props.has(De), Ge = input$1({ type: "checkbox", checked: Ze }), ze = div({ class: "prop-attr", tabIndex: 0 }, Ge, span$1({ class: "attr" }, Re), qe), We = div({ class: "prop-conns" }, Ve), at = div({ class: "prop-kind" }, Pe), ct = div({ class: "prop-value" }, $e), et = De.dirty ? "dirty" : null, dt = De.internal ? "internal" : null, Je = te.checked, it = ne.checked, st = !De.dirty && !Je || De.internal && !it ? "hidden" : null, gt = div({ class: ["prop", et, dt, st] }, ze, We, at, ct);
            Me.push(gt), V.push([De, gt]), ce.appendChild(gt), Ge.addEventListener("change", () => {
              this.watched_props[Ge.checked ? "add" : "delete"](De), be();
            });
          }
          ke.addEventListener("click", () => {
            Qe.classList.toggle("closed");
            for (const Re of Me)
              Re.classList.toggle("closed");
          });
        }
      }, be = () => {
        if (clear$1(q), empty$1(le), this.watched_props.size == 0) {
          const Se = div({ class: "nothing" }, "No watched properties");
          le.appendChild(Se);
        } else
          for (const Se of this.watched_props) {
            const we = span$1(ye(Se)), Ee = span$1(Se.is_unset ? span$1("unset") : ye(Se.get_value())), xe = div({ class: ["prop", Se.dirty ? "dirty" : null] }, we, Ee);
            q.push([Se, xe]), le.appendChild(xe);
          }
      };
      this.shadow_el.appendChild(me);
      const { target: Ce } = this.model;
      if (Ce != null) {
        const Se = Ce.references(), { document: we } = Ce;
        ge(Se, we), ve(Ce);
      } else {
        const { document: Se } = this.model;
        if (Se != null) {
          const we = Se._all_models.values();
          ge(we, Se);
          const Ee = Se.roots();
          if (Ee.length != 0) {
            const [xe] = Ee;
            ve(xe);
          }
        }
      }
      be();
    }
  }
  mt(ExaminerView, "__name__", "ExaminerView");
  const Rm = class Rm extends UIElement {
    constructor(_) {
      super(_);
    }
  };
  mt(Rm, "__name__", "Examiner"), Rm.prototype.default_view = ExaminerView, Rm.define(({ Ref: _, Nullable: H }) => ({
    target: [H(_(HasProps)), null]
  }));
  let Examiner = Rm;
  class CopyToolView extends ActionToolView {
    async copy() {
      const _ = await this.parent.export().to_blob(), H = new ClipboardItem({ [_.type]: _ });
      await navigator.clipboard.write([H]);
    }
    doit() {
      this.copy();
    }
  }
  mt(CopyToolView, "__name__", "CopyToolView");
  const qp = class qp extends ActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Copy");
      mt(this, "tool_icon", tool_icon_copy);
    }
  };
  mt(qp, "__name__", "CopyTool"), qp.prototype.default_view = CopyToolView, qp.register_alias("copy", () => new qp());
  let CopyTool = qp;
  class CustomActionView extends ActionToolView {
    doit() {
      const { callback: _ } = this.model;
      _ != null && execute(_, this.model);
    }
  }
  mt(CustomActionView, "__name__", "CustomActionView");
  const jp = class jp extends ActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Custom Action");
      mt(this, "tool_icon", tool_icon_unknown);
    }
  };
  mt(jp, "__name__", "CustomAction"), jp.prototype.default_view = CustomActionView, jp.define(({ Any: H, Nullable: V }) => ({
    callback: [V(
      H
      /*TODO*/
    ), null]
  })), jp.override({
    description: "Perform a Custom Action"
  });
  let CustomAction = jp;
  const request_fullscreen = typeof Element.prototype.webkitRequestFullscreen < "u" ? (O, _) => O.webkitRequestFullscreen(_) : (O, _) => O.requestFullscreen(_);
  class FullscreenToolView extends ActionToolView {
    async fullscreen() {
      document.fullscreenElement != null ? await document.exitFullscreen() : await request_fullscreen(this.parent.el);
    }
    doit() {
      this.fullscreen();
    }
  }
  mt(FullscreenToolView, "__name__", "FullscreenToolView");
  const Gp = class Gp extends ActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Fullscreen");
      mt(this, "tool_icon", tool_icon_fullscreen);
    }
  };
  mt(Gp, "__name__", "FullscreenTool"), Gp.prototype.default_view = FullscreenToolView, Gp.register_alias("fullscreen", () => new Gp());
  let FullscreenTool = Gp;
  class ExamineToolView extends ActionToolView {
    constructor() {
      super(...arguments);
      mt(this, "_dialog");
    }
    *children() {
      yield* super.children(), yield this._dialog;
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const H = this.parent.model, V = new HTMLPrinter(), q = new Dialog({
        stylesheets: [pretty_css],
        title: new HTML({ html: div("Examine ", V.to_html(H)) }),
        content: new Examiner({ target: H }),
        visible: !1,
        close_action: "hide"
      });
      this._dialog = await build_view(q, { parent: this.parent });
    }
    doit() {
      this._dialog.open();
    }
  }
  mt(ExamineToolView, "__name__", "ExamineToolView");
  const Wp = class Wp extends ActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Examine");
      mt(this, "tool_icon", tool_icon_settings);
    }
    // TODO: better icon
  };
  mt(Wp, "__name__", "ExamineTool"), Wp.prototype.default_view = ExamineToolView, Wp.register_alias("examine", () => new Wp());
  let ExamineTool = Wp;
  class PlotActionToolView extends ActionToolView {
    get plot_view() {
      return this.parent;
    }
  }
  mt(PlotActionToolView, "__name__", "PlotActionToolView");
  class PlotActionTool extends ActionTool {
    constructor(_) {
      super(_);
    }
  }
  mt(PlotActionTool, "__name__", "PlotActionTool");
  class RedoToolView extends PlotActionToolView {
    connect_signals() {
      super.connect_signals(), this.connect(this.plot_view.state.changed, () => this.model.disabled = !this.plot_view.state.can_redo);
    }
    doit() {
      const _ = this.plot_view.state.redo();
      (_ == null ? void 0 : _.range) != null && this.plot_view.trigger_ranges_update_event();
    }
  }
  mt(RedoToolView, "__name__", "RedoToolView");
  const _0 = class _0 extends PlotActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Redo");
      mt(this, "tool_icon", tool_icon_redo);
    }
  };
  mt(_0, "__name__", "RedoTool"), _0.prototype.default_view = RedoToolView, _0.override({
    disabled: !0
  }), _0.register_alias("redo", () => new _0());
  let RedoTool = _0;
  class ResetToolView extends PlotActionToolView {
    doit() {
      this.plot_view.reset();
    }
  }
  mt(ResetToolView, "__name__", "ResetToolView");
  const Yp = class Yp extends PlotActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Reset");
      mt(this, "tool_icon", tool_icon_reset);
    }
  };
  mt(Yp, "__name__", "ResetTool"), Yp.prototype.default_view = ResetToolView, Yp.register_alias("reset", () => new Yp());
  let ResetTool = Yp;
  class SaveToolView extends ActionToolView {
    async copy() {
      const _ = await this.parent.export().to_blob(), H = new ClipboardItem({ [_.type]: _ });
      await navigator.clipboard.write([H]);
    }
    async save(_) {
      const H = await this.parent.export().to_blob(), V = document.createElement("a");
      V.href = URL.createObjectURL(H), V.download = _, V.target = "_blank", V.dispatchEvent(new MouseEvent("click"));
    }
    doit(_ = "save") {
      switch (_) {
        case "save": {
          const H = this.model.filename ?? prompt("Enter filename", "bokeh_plot");
          H != null && this.save(H);
          break;
        }
        case "copy": {
          this.copy();
          break;
        }
      }
    }
  }
  mt(SaveToolView, "__name__", "SaveToolView");
  const v0 = class v0 extends ActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Save");
      mt(this, "tool_icon", tool_icon_save);
    }
    get menu() {
      return [
        {
          icon: "bk-tool-icon-copy",
          tooltip: "Copy image to clipboard",
          if: () => typeof ClipboardItem < "u",
          handler: () => {
            this.do.emit("copy");
          }
        }
      ];
    }
  };
  mt(v0, "__name__", "SaveTool"), v0.prototype.default_view = SaveToolView, v0.define(({ Str: H, Nullable: V }) => ({
    filename: [V(H), null]
  })), v0.register_alias("save", () => new v0());
  let SaveTool = v0;
  class UndoToolView extends PlotActionToolView {
    connect_signals() {
      super.connect_signals(), this.connect(this.plot_view.state.changed, () => this.model.disabled = !this.plot_view.state.can_undo);
    }
    doit() {
      const _ = this.plot_view.state.undo();
      (_ == null ? void 0 : _.range) != null && this.plot_view.trigger_ranges_update_event();
    }
  }
  mt(UndoToolView, "__name__", "UndoToolView");
  const b0 = class b0 extends PlotActionTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Undo");
      mt(this, "tool_icon", tool_icon_undo);
    }
  };
  mt(b0, "__name__", "UndoTool"), b0.prototype.default_view = UndoToolView, b0.override({
    disabled: !0
  }), b0.register_alias("undo", () => new b0());
  let UndoTool = b0;
  function scale_interval(O, _, H) {
    const [V, q] = minmax(O.start, O.end), G = H ?? (q + V) / 2, J = V - (V - G) * _, Z = q - (q - G) * _;
    return [J, Z];
  }
  function rescale(O, _, H) {
    const V = /* @__PURE__ */ new Map();
    for (const q of O) {
      const [G, J] = scale_interval(q.target_range, _, H), [Z, te] = q.r_invert(G, J);
      V.set(q.source_range, { start: Z, end: te });
    }
    return V;
  }
  function scale_range(O, _, H, V, q, G = !0, J = !0, Z) {
    const ne = rescale(O, G ? q : 0, Z == null ? void 0 : Z.x), ae = rescale(_, J ? q : 0, Z == null ? void 0 : Z.y);
    return { xrs: ne, yrs: ae, factor: q };
  }
  class ZoomBaseToolView extends PlotActionToolView {
    doit() {
      var ce;
      const { dimensions: _ } = this.model, H = _ == "width" || _ == "both", V = _ == "height" || _ == "both", { frame: q } = this.plot_view, { x_target: G, y_target: J } = q, Z = new Map(q.x_scales), te = new Map(q.y_scales), { renderers: ne } = this.model;
      if (ne != "auto") {
        const le = /* @__PURE__ */ new Set(), ue = /* @__PURE__ */ new Set();
        for (const de of ne)
          de.coordinates == null && (le.add(de.x_range_name), ue.add(de.y_range_name));
        for (const de of Z.keys())
          le.has(de) || Z.delete(de);
        for (const de of te.keys())
          ue.has(de) || te.delete(de);
      }
      const ee = [...Z.values()], ae = [...te.values()], oe = ne != "auto" ? ne : this.plot_view.model.data_renderers;
      for (const le of oe) {
        if (le.coordinates == null)
          continue;
        const ue = this.plot_view.renderer_view(le);
        assert$2(ue != null);
        const de = (me, pe) => {
          const { level: ye } = this.model;
          for (let ge = 0; ge < ye; ge++)
            if (me instanceof CompositeScale)
              me = me.source_scale;
            else {
              logger$1.warn(`can't reach sub-coordinate level ${ye} for ${me} in ${pe} dimension; stopped at ${ge}`);
              break;
            }
          return me instanceof CompositeScale ? me.target_scale : me;
        }, { x_scale: fe, y_scale: he } = ue.coordinates;
        ee.push(de(fe, "x")), ae.push(de(he, "y"));
      }
      const se = scale_range(ee, ae, G, J, this.factor, H, V);
      this.plot_view.state.push("zoom_out", { range: se }), this.plot_view.update_range(se, { scrolling: !0, maintain_focus: this.model.maintain_focus }), (ce = this.model.document) == null || ce.interactive_start(this.plot_view.model), this.plot_view.trigger_ranges_update_event();
    }
  }
  mt(ZoomBaseToolView, "__name__", "ZoomBaseToolView");
  const g6 = class g6 extends PlotActionTool {
    constructor(_) {
      super(_);
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimensions);
    }
  };
  mt(g6, "__name__", "ZoomBaseTool"), g6.define(({ Percent: _, Or: H, List: V, Ref: q, Auto: G, NonNegative: J, Int: Z }) => ({
    factor: [_, 0.1],
    dimensions: [Dimensions, "both"],
    renderers: [H(V(q(DataRenderer)), G), "auto"],
    level: [J(Z), 0]
  }));
  let ZoomBaseTool = g6;
  class ZoomInToolView extends ZoomBaseToolView {
    get factor() {
      return this.model.factor;
    }
  }
  mt(ZoomInToolView, "__name__", "ZoomInToolView");
  const qh = class qh extends ZoomBaseTool {
    constructor(H) {
      super(H);
      mt(this, "maintain_focus", !0);
      mt(this, "tool_name", "Zoom In");
      mt(this, "tool_icon", tool_icon_zoom_in);
    }
  };
  mt(qh, "__name__", "ZoomInTool"), qh.prototype.default_view = ZoomInToolView, qh.register_alias("zoom_in", () => new qh({ dimensions: "both" })), qh.register_alias("xzoom_in", () => new qh({ dimensions: "width" })), qh.register_alias("yzoom_in", () => new qh({ dimensions: "height" }));
  let ZoomInTool = qh;
  class ZoomOutToolView extends ZoomBaseToolView {
    get factor() {
      const { factor: _ } = this.model;
      return -_ / (1 - _);
    }
  }
  mt(ZoomOutToolView, "__name__", "ZoomOutToolView");
  const wh = class wh extends ZoomBaseTool {
    constructor(H) {
      super(H);
      mt(this, "maintain_focus");
      mt(this, "tool_name", "Zoom Out");
      mt(this, "tool_icon", tool_icon_zoom_out);
    }
  };
  mt(wh, "__name__", "ZoomOutTool"), wh.prototype.default_view = ZoomOutToolView, wh.define(({ Bool: H }) => ({
    maintain_focus: [H, !0]
  })), wh.register_alias("zoom_out", () => new wh({ dimensions: "both" })), wh.register_alias("xzoom_out", () => new wh({ dimensions: "width" })), wh.register_alias("yzoom_out", () => new wh({ dimensions: "height" }));
  let ZoomOutTool = wh;
  class EditToolView extends GestureToolView {
    constructor() {
      super(...arguments);
      mt(this, "_basepoint");
      mt(this, "_mouse_in_frame", !0);
    }
    _select_mode(H) {
      const { shift: V, ctrl: q } = H.modifiers;
      if (!V && !q)
        return "replace";
      if (V && !q)
        return "append";
      if (!V && q)
        return "intersect";
      if (V && q)
        return "subtract";
      unreachable();
    }
    _move_enter(H) {
      this._mouse_in_frame = !0;
    }
    _move_exit(H) {
      this._mouse_in_frame = !1;
    }
    _map_drag(H, V, q) {
      if (!this.plot_view.frame.bbox.contains(H, V))
        return null;
      const J = this.plot_view.renderer_view(q);
      if (J == null)
        return null;
      const Z = J.coordinates.x_scale.invert(H), te = J.coordinates.y_scale.invert(V);
      return [Z, te];
    }
    _delete_selected(H) {
      const V = H.data_source, q = V.selected.indices;
      q.sort();
      for (const G of V.columns()) {
        const J = V.get_array(G);
        for (let Z = 0; Z < q.length; Z++) {
          const te = q[Z];
          J.splice(te - Z, 1);
        }
      }
      this._emit_cds_changes(V);
    }
    _pop_glyphs(H, V) {
      const q = H.columns();
      if (V == 0 || q.length == 0)
        return;
      const G = dict(H.data);
      for (const J of q) {
        let Z = H.get_array(J);
        const te = Z.length - V + 1;
        te < 1 || (isArray$3(Z) || (Z = Array.from(Z), G.set(J, Z)), Z.splice(0, te));
      }
    }
    _emit_cds_changes(H, V = !0, q = !0, G = !0) {
      if (q && H.selection_manager.clear(), V && H.change.emit(), G) {
        const { data: J } = H;
        H.setv({ data: J }, { check_eq: !1 });
      }
    }
    _drag_points(H, V, q = "both") {
      if (this._basepoint == null)
        return;
      const [G, J] = this._basepoint;
      for (const Z of V) {
        const te = this._map_drag(G, J, Z), ne = this._map_drag(H.sx, H.sy, Z);
        if (ne == null || te == null)
          continue;
        const [ee, ae] = ne, [oe, se] = te, [ce, le] = [ee - oe, ae - se], ue = Z.glyph, de = Z.data_source, fe = dict(de.data), [he, me] = [ue.x.field, ue.y.field];
        for (const pe of de.selected.indices) {
          if (he && (q == "width" || q == "both")) {
            const ye = fe.get(he) ?? [];
            ye[pe] += ce;
          }
          if (me && (q == "height" || q == "both")) {
            const ye = fe.get(me) ?? [];
            ye[pe] += le;
          }
        }
        de.change.emit();
      }
      this._basepoint = [H.sx, H.sy];
    }
    _pad_empty_columns(H, V) {
      const { inferred_defaults: q } = H, G = dict(H.default_values), J = dict(this.model.default_overrides);
      for (const Z of H.columns())
        if (!includes(V, Z)) {
          const te = J.has(Z) ? J.get(Z) : G.has(Z) ? G.get(Z) : q.has(Z) ? q.get(Z) : this.model.empty_value;
          H.get_array(Z).push(te);
        }
    }
    _select_event(H, V, q) {
      const G = this.plot_view.frame, { sx: J, sy: Z } = H;
      if (!G.bbox.contains(J, Z))
        return [];
      const te = { type: "point", sx: J, sy: Z }, ne = [];
      for (const ee of q) {
        const ae = ee.get_selection_manager(), oe = ee.data_source, se = this.plot_view.renderer_view(ee);
        se != null && (ae.select([se], te, !0, V) && ne.push(ee), oe.properties.selected.change.emit());
      }
      return ne;
    }
  }
  mt(EditToolView, "__name__", "EditToolView");
  const _6 = class _6 extends GestureTool {
    constructor(_) {
      super(_);
    }
  };
  mt(_6, "__name__", "EditTool"), _6.define(({ Unknown: _, Dict: H }) => ({
    default_overrides: [H(_), {}],
    empty_value: [_, 0]
  }));
  let EditTool = _6;
  class BoxEditToolView extends EditToolView {
    constructor() {
      super(...arguments);
      mt(this, "_draw_basepoint");
      mt(this, "_recent_renderers", []);
    }
    _tap(H) {
      this._draw_basepoint != null || this._basepoint != null || (this._recent_renderers = this._select_event(H, this._select_mode(H), this.model.renderers));
    }
    _keyup(H) {
      if (!(!this.model.active || !this._mouse_in_frame))
        for (const V of this.model.renderers)
          H.key == "Backspace" ? this._delete_selected(V) : H.key == "Escape" && V.data_source.selection_manager.clear();
    }
    _set_extent([H, V], [q, G], J, Z = !1) {
      const te = this._recent_renderers[0] ?? this.model.renderers[0], ne = this.plot_view.renderer_view(te);
      if (ne == null)
        return;
      const { glyph: ee } = te, ae = te.data_source, oe = dict(ae.data), [se, ce] = ne.coordinates.x_scale.r_invert(H, V), [le, ue] = ne.coordinates.y_scale.r_invert(q, G), de = (() => {
        if (ee instanceof Rect) {
          const { x: fe, y: he, width: me, height: pe } = ee;
          if (isField$1(fe) && isField$1(he) && isField$1(me) && isField$1(pe))
            return {
              [fe.field]: (se + ce) / 2,
              [he.field]: (le + ue) / 2,
              [me.field]: ce - se,
              [pe.field]: ue - le
            };
        } else if (ee instanceof Block$1) {
          const { x: fe, y: he, width: me, height: pe } = ee;
          if (isField$1(fe) && isField$1(he) && isField$1(me) && isField$1(pe))
            return {
              [fe.field]: se,
              [he.field]: le,
              [me.field]: ce - se,
              [pe.field]: ue - le
            };
        } else if (ee instanceof Quad$1) {
          const { right: fe, bottom: he, left: me, top: pe } = ee;
          if (isField$1(fe) && isField$1(he) && isField$1(me) && isField$1(pe))
            return {
              [fe.field]: ce,
              [he.field]: le,
              [me.field]: se,
              [pe.field]: ue
            };
        } else if (ee instanceof HBar) {
          const { left: fe, y: he, height: me, right: pe } = ee;
          if (isField$1(fe) && isField$1(he) && isField$1(me) && isField$1(pe))
            return {
              [fe.field]: se,
              [he.field]: (le + ue) / 2,
              [me.field]: ue - le,
              [pe.field]: ce
            };
        } else if (ee instanceof VBar) {
          const { x: fe, bottom: he, width: me, top: pe } = ee;
          if (isField$1(fe) && isField$1(he) && isField$1(me) && isField$1(pe))
            return {
              [fe.field]: (se + ce) / 2,
              [he.field]: le,
              [me.field]: ce - se,
              [pe.field]: ue
            };
        } else if (ee instanceof HStrip) {
          const { y0: fe, y1: he } = ee;
          if (isField$1(fe) && isField$1(he))
            return {
              [fe.field]: le,
              [he.field]: ue
            };
        } else if (ee instanceof VStrip) {
          const { x0: fe, x1: he } = ee;
          if (isField$1(fe) && isField$1(he))
            return {
              [fe.field]: se,
              [he.field]: ce
            };
        } else
          unreachable(`'${ee.type}' is not supported"`);
        return null;
      })();
      if (de != null) {
        if (J) {
          this._pop_glyphs(ae, this.model.num_objects);
          for (const [fe, he] of entries$2(de))
            ae.get_array(fe).push(he);
          this._pad_empty_columns(ae, keys$5(de));
        } else {
          const fe = ae.get_length();
          if (fe == null)
            return;
          const he = fe - 1;
          for (const [me, pe] of entries$2(de))
            oe.get(me)[he] = pe;
        }
        this._emit_cds_changes(ae, !0, !1, Z);
      }
    }
    _update_box(H, V = !1, q = !1) {
      if (this._draw_basepoint == null)
        return;
      const G = [H.sx, H.sy], J = this.plot_view.frame, Z = this.model.dimensions, [te, ne] = this.model._get_dim_limits(this._draw_basepoint, G, J, Z);
      this._set_extent(te, ne, V, q);
    }
    _press(H) {
      this.model.active && (this._draw_basepoint != null ? (this._update_box(H, !1, !0), this._draw_basepoint = null) : (this._draw_basepoint = [H.sx, H.sy], this._select_event(H, "append", this.model.renderers), this._update_box(H, !0, !1)));
    }
    _move(H) {
      this._update_box(H, !1, !1);
    }
    _pan_start(H) {
      if (H.modifiers.shift) {
        if (this._draw_basepoint != null)
          return;
        this._draw_basepoint = [H.sx, H.sy], this._update_box(H, !0, !1);
      } else {
        if (this._basepoint != null)
          return;
        this._recent_renderers = this._select_event(H, "append", this.model.renderers), this._basepoint = [H.sx, H.sy];
      }
    }
    _pan(H, V = !1, q = !1) {
      if (H.modifiers.shift) {
        if (this._draw_basepoint == null)
          return;
        this._update_box(H, V, q);
      } else {
        if (this._basepoint == null)
          return;
        this._drag_points(H, this.model.renderers);
      }
    }
    _drag_points(H, V, q = "both") {
      if (this._basepoint == null)
        return;
      const [G, J] = this._basepoint;
      for (const Z of V) {
        const te = this._map_drag(G, J, Z), ne = this._map_drag(H.sx, H.sy, Z);
        if (ne == null || te == null)
          continue;
        const [ee, ae] = ne, [oe, se] = te, ce = q == "width" || q == "both" ? ee - oe : 0, le = q == "height" || q == "both" ? ae - se : 0, { glyph: ue } = Z, de = Z.data_source, fe = dict(de.data), he = {};
        if (ue instanceof XYGlyph) {
          const { x: me, y: pe } = ue;
          isField$1(me) && (he[me.field] = ce), isField$1(pe) && (he[pe.field] = le);
        } else if (ue instanceof Block$1) {
          const { x: me, y: pe } = ue;
          isField$1(me) && (he[me.field] = ce), isField$1(pe) && (he[pe.field] = le);
        } else if (ue instanceof Quad$1) {
          const { right: me, bottom: pe, left: ye, top: ge } = ue;
          isField$1(ye) && isField$1(me) && (he[ye.field] = ce, he[me.field] = ce), isField$1(ge) && isField$1(pe) && (he[ge.field] = le, he[pe.field] = le);
        } else if (ue instanceof HBar) {
          const { left: me, right: pe, y: ye } = ue;
          isField$1(me) && isField$1(pe) && (he[me.field] = ce, he[pe.field] = ce), isField$1(ye) && (he[ye.field] = le);
        } else if (ue instanceof VBar) {
          const { x: me, top: pe, bottom: ye } = ue;
          isField$1(me) && (he[me.field] = ce), isField$1(pe) && isField$1(ye) && (he[pe.field] = le, he[ye.field] = le);
        } else if (ue instanceof HStrip) {
          const { y0: me, y1: pe } = ue;
          isField$1(me) && isField$1(pe) && (he[me.field] = le, he[pe.field] = le);
        } else if (ue instanceof VStrip) {
          const { x0: me, x1: pe } = ue;
          isField$1(me) && isField$1(pe) && (he[me.field] = ce, he[pe.field] = ce);
        } else
          unreachable(`'${ue.type}' is not supported"`);
        for (const me of de.selected.indices)
          for (const [pe, ye] of entries$2(he)) {
            const ge = fe.get(pe) ?? [];
            ge[me] += ye;
          }
        de.change.emit();
      }
      this._basepoint = [H.sx, H.sy];
    }
    _pan_end(H) {
      if (this._pan(H, !1, !0), H.modifiers.shift)
        this._draw_basepoint = null;
      else {
        this._basepoint = null;
        for (const V of this.model.renderers)
          this._emit_cds_changes(V.data_source, !1, !0, !0);
      }
    }
  }
  mt(BoxEditToolView, "__name__", "BoxEditToolView");
  const Om = class Om extends EditTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Box Edit Tool");
      mt(this, "tool_icon", tool_icon_box_edit);
      mt(this, "event_type", ["tap", "press", "pan", "move"]);
      mt(this, "default_order", 1);
    }
  };
  mt(Om, "__name__", "BoxEditTool"), Om.prototype.default_view = BoxEditToolView, Om.define(({ Int: H, List: V, Ref: q }) => ({
    dimensions: [Dimensions, "both"],
    num_objects: [H, 0],
    renderers: [V(q(GlyphRenderer)), []]
  }));
  let BoxEditTool = Om;
  class FreehandDrawToolView extends EditToolView {
    _draw(_, H, V = !1) {
      if (!this.model.active)
        return;
      const q = this.model.renderers[0], G = this._map_drag(_.sx, _.sy, q);
      if (G == null)
        return;
      const [J, Z] = G, te = q.data_source, ne = dict(te.data), ee = q.glyph, [ae, oe] = [ee.xs.field, ee.ys.field];
      if (H == "new")
        this._pop_glyphs(te, this.model.num_objects), ae && te.get_array(ae).push([J]), oe && te.get_array(oe).push([Z]), this._pad_empty_columns(te, [ae, oe]);
      else if (H == "add") {
        if (ae) {
          const se = ne.get(ae) ?? [], ce = se.length - 1;
          let le = te.get_array(ae)[ce];
          isArray$3(le) || (le = Array.from(le), se[ce] = le), le.push(J);
        }
        if (oe) {
          const se = ne.get(oe) ?? [], ce = se.length - 1;
          let le = te.get_array(oe)[ce];
          isArray$3(le) || (le = Array.from(le), se[ce] = le), le.push(Z);
        }
      }
      this._emit_cds_changes(te, !0, !0, V);
    }
    _pan_start(_) {
      this._draw(_, "new");
    }
    _pan(_) {
      this._draw(_, "add");
    }
    _pan_end(_) {
      this._draw(_, "add", !0);
    }
    _tap(_) {
      this._select_event(_, this._select_mode(_), this.model.renderers);
    }
    _keyup(_) {
      if (!(!this.model.active || !this._mouse_in_frame))
        for (const H of this.model.renderers)
          _.key == "Escape" ? H.data_source.selection_manager.clear() : _.key == "Backspace" && this._delete_selected(H);
    }
  }
  mt(FreehandDrawToolView, "__name__", "FreehandDrawToolView");
  const T0 = class T0 extends EditTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Freehand Draw Tool");
      mt(this, "tool_icon", tool_icon_freehand_draw);
      mt(this, "event_type", ["pan", "tap"]);
      mt(this, "default_order", 3);
    }
  };
  mt(T0, "__name__", "FreehandDrawTool"), T0.prototype.default_view = FreehandDrawToolView, T0.define(({ Int: H, List: V, Ref: q }) => ({
    num_objects: [H, 0],
    renderers: [V(q(GlyphRenderer)), []]
  })), T0.register_alias("freehand_draw", () => new T0());
  let FreehandDrawTool = T0;
  class LineToolView extends EditToolView {
    _set_intersection(_, H) {
      const V = this.model.intersection_renderer.glyph, q = this.model.intersection_renderer.data_source, G = dict(q.data), [J, Z] = [V.x.field, V.y.field];
      J && (isArray$3(_) ? G.set(J, _) : V.x = { value: _ }), Z && (isArray$3(H) ? G.set(Z, H) : V.y = { value: H }), this._emit_cds_changes(q, !0, !0, !1);
    }
    _hide_intersections() {
      this._set_intersection([], []);
    }
  }
  mt(LineToolView, "__name__", "LineToolView");
  const y6 = class y6 extends EditTool {
    constructor(_) {
      super(_);
    }
  };
  mt(y6, "__name__", "LineTool"), y6.define(({ AnyRef: _ }) => ({
    intersection_renderer: [_()]
  }));
  let LineTool = y6;
  class LineEditToolView extends LineToolView {
    constructor() {
      super(...arguments);
      mt(this, "_selected_renderer");
      mt(this, "_drawing", !1);
    }
    _press(H) {
      if (!this.model.active)
        return;
      const V = this.model.renderers;
      for (const q of V)
        this._select_event(H, "replace", [q]).length == 1 && (this._selected_renderer = q);
      this._show_intersections(), this._update_line_cds();
    }
    _show_intersections() {
      if (!this.model.active || this._selected_renderer == null)
        return;
      if (this.model.renderers.length == 0) {
        this._set_intersection([], []), this._selected_renderer = null, this._drawing = !1;
        return;
      }
      const V = this._selected_renderer.data_source, q = this._selected_renderer.glyph, [G, J] = [q.x.field, q.y.field], Z = V.get_array(G), te = V.get_array(J);
      this._set_intersection(Z, te);
    }
    _tap(H) {
      const V = this.model.intersection_renderer;
      if (this._map_drag(H.sx, H.sy, V) == null)
        return;
      if (this._drawing && this._selected_renderer != null) {
        const J = this._select_mode(H);
        if (this._select_event(H, J, [V]).length == 0)
          return;
      }
      const G = this._select_mode(H);
      this._select_event(H, G, [V]), this._select_event(H, G, this.model.renderers);
    }
    _update_line_cds() {
      if (this._selected_renderer == null)
        return;
      const H = this.model.intersection_renderer.glyph, V = this.model.intersection_renderer.data_source, q = dict(V.data), [G, J] = [H.x.field, H.y.field];
      if (G && J) {
        const Z = q.get(G), te = q.get(J);
        Z != null && dict(this._selected_renderer.data_source.data).set(G, Z), te != null && dict(this._selected_renderer.data_source.data).set(J, te);
      }
      this._emit_cds_changes(this._selected_renderer.data_source, !0, !0, !1);
    }
    _pan_start(H) {
      this._select_event(H, "append", [this.model.intersection_renderer]), this._basepoint = [H.sx, H.sy];
    }
    _pan(H) {
      this._basepoint != null && (this._drag_points(H, [this.model.intersection_renderer], this.model.dimensions), this._selected_renderer != null && this._selected_renderer.data_source.change.emit());
    }
    _pan_end(H) {
      this._basepoint != null && (this._drag_points(H, [this.model.intersection_renderer]), this._emit_cds_changes(this.model.intersection_renderer.data_source, !1, !0, !0), this._selected_renderer != null && this._emit_cds_changes(this._selected_renderer.data_source), this._basepoint = null);
    }
    activate() {
      this._drawing = !0;
    }
    deactivate() {
      this._selected_renderer != null && (this._drawing && (this._drawing = !1), this._hide_intersections());
    }
  }
  mt(LineEditToolView, "__name__", "LineEditToolView");
  const Im = class Im extends LineTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Line Edit Tool");
      mt(this, "tool_icon", tool_icon_line_edit);
      mt(this, "event_type", ["tap", "press", "pan", "move"]);
      mt(this, "default_order", 4);
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimensions);
    }
  };
  mt(Im, "__name__", "LineEditTool"), Im.prototype.default_view = LineEditToolView, Im.define(({ List: H, Ref: V }) => ({
    dimensions: [Dimensions, "both"],
    renderers: [H(V(GlyphRenderer)), []]
  }));
  let LineEditTool = Im;
  class PointDrawToolView extends EditToolView {
    _tap(_) {
      if (this._select_event(_, this._select_mode(_), this.model.renderers).length != 0 || !this.model.add)
        return;
      const V = this.model.renderers[0], q = this._map_drag(_.sx, _.sy, V);
      if (q == null)
        return;
      const G = V.glyph, J = V.data_source, [Z, te] = [G.x.field, G.y.field], [ne, ee] = q;
      this._pop_glyphs(J, this.model.num_objects), Z && J.get_array(Z).push(ne), te && J.get_array(te).push(ee), this._pad_empty_columns(J, [Z, te]);
      const { data: ae } = J;
      J.setv({ data: ae }, { check_eq: !1 });
    }
    _keyup(_) {
      if (!(!this.model.active || !this._mouse_in_frame))
        for (const H of this.model.renderers)
          _.key == "Backspace" ? this._delete_selected(H) : _.key == "Escape" && H.data_source.selection_manager.clear();
    }
    _pan_start(_) {
      this.model.drag && (this._select_event(_, "append", this.model.renderers), this._basepoint = [_.sx, _.sy]);
    }
    _pan(_) {
      !this.model.drag || this._basepoint == null || this._drag_points(_, this.model.renderers);
    }
    _pan_end(_) {
      if (this.model.drag) {
        this._pan(_);
        for (const H of this.model.renderers)
          this._emit_cds_changes(H.data_source, !1, !0, !0);
        this._basepoint = null;
      }
    }
  }
  mt(PointDrawToolView, "__name__", "PointDrawToolView");
  const Dm = class Dm extends EditTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Point Draw Tool");
      mt(this, "tool_icon", tool_icon_point_draw);
      mt(this, "event_type", ["tap", "pan", "move"]);
      mt(this, "default_order", 2);
    }
  };
  mt(Dm, "__name__", "PointDrawTool"), Dm.prototype.default_view = PointDrawToolView, Dm.define(({ Bool: H, Int: V, List: q, Ref: G }) => ({
    add: [H, !0],
    drag: [H, !0],
    num_objects: [V, 0],
    renderers: [q(G(GlyphRenderer)), []]
  }));
  let PointDrawTool = Dm;
  class PolyToolView extends EditToolView {
    _set_vertices(_, H) {
      const { vertex_renderer: V } = this.model;
      assert$2(V != null);
      const q = V.glyph, G = V.data_source, [J, Z] = [q.x.field, q.y.field], te = dict(G.data);
      J && (isArray$3(_) ? te.set(J, _) : q.x = { value: _ }), Z && (isArray$3(H) ? te.set(Z, H) : q.y = { value: H }), this._emit_cds_changes(G, !0, !0, !1);
    }
    _hide_vertices() {
      this._set_vertices([], []);
    }
    _snap_to_vertex(_, H, V) {
      if (this.model.vertex_renderer != null) {
        const q = this._select_event(_, "replace", [this.model.vertex_renderer]), G = this.model.vertex_renderer.data_source, J = this.model.vertex_renderer.glyph, [Z, te] = [J.x.field, J.y.field];
        if (q.length != 0) {
          const ne = G.selected.indices[0], ee = dict(G.data);
          Z && (H = ee.get(Z)[ne]), te && (V = ee.get(te)[ne]), G.selection_manager.clear();
        }
      }
      return [H, V];
    }
  }
  mt(PolyToolView, "__name__", "PolyToolView");
  const v6 = class v6 extends EditTool {
    constructor(_) {
      super(_);
    }
  };
  mt(v6, "__name__", "PolyTool"), v6.define(({ AnyRef: _, Nullable: H }) => ({
    vertex_renderer: [H(_()), null]
  }));
  let PolyTool = v6;
  class PolyDrawToolView extends PolyToolView {
    constructor() {
      super(...arguments);
      mt(this, "_drawing", !1);
      mt(this, "_initialized", !1);
    }
    _tap(H) {
      this._drawing ? this._draw(H, "add", !0) : this._select_event(H, this._select_mode(H), this.model.renderers);
    }
    _draw(H, V, q = !1) {
      const G = this.model.renderers[0], J = this._map_drag(H.sx, H.sy, G);
      if (this._initialized || this.activate(), J == null)
        return;
      const [Z, te] = this._snap_to_vertex(H, ...J), ne = G.data_source, ee = dict(ne.data), ae = G.glyph, [oe, se] = [ae.xs.field, ae.ys.field];
      if (V == "new")
        this._pop_glyphs(ne, this.model.num_objects), oe && ne.get_array(oe).push([Z, Z]), se && ne.get_array(se).push([te, te]), this._pad_empty_columns(ne, [oe, se]);
      else if (V == "edit") {
        if (oe) {
          const ce = ee.get(oe) ?? [], le = ce[ce.length - 1];
          le[le.length - 1] = Z;
        }
        if (se) {
          const ce = ee.get(se) ?? [], le = ce[ce.length - 1];
          le[le.length - 1] = te;
        }
      } else if (V == "add") {
        if (oe) {
          const ce = ee.get(oe) ?? [], le = ce.length - 1;
          let ue = ne.get_array(oe)[le];
          const de = ue[ue.length - 1];
          ue[ue.length - 1] = Z, isArray$3(ue) || (ue = Array.from(ue), ce[le] = ue), ue.push(de);
        }
        if (se) {
          const ce = ee.get(se) ?? [], le = ce.length - 1;
          let ue = ne.get_array(se)[le];
          const de = ue[ue.length - 1];
          ue[ue.length - 1] = te, isArray$3(ue) || (ue = Array.from(ue), ce[le] = ue), ue.push(de);
        }
      }
      this._emit_cds_changes(ne, !0, !1, q);
    }
    _show_vertices() {
      if (!this.model.active)
        return;
      const H = [], V = [];
      for (let q = 0; q < this.model.renderers.length; q++) {
        const G = this.model.renderers[q], J = G.data_source, Z = G.glyph, [te, ne] = [Z.xs.field, Z.ys.field];
        if (te)
          for (const ee of J.get_array(te))
            H.push(...ee);
        if (ne)
          for (const ee of J.get_array(ne))
            V.push(...ee);
        this._drawing && q == this.model.renderers.length - 1 && (H.splice(H.length - 1, 1), V.splice(V.length - 1, 1));
      }
      this._set_vertices(H, V);
    }
    _press(H) {
      this.model.active && (this._drawing ? (this._drawing = !1, this._draw(H, "edit", !0)) : (this._drawing = !0, this._draw(H, "new", !0)));
    }
    _move(H) {
      this._drawing && this._draw(H, "edit");
    }
    _remove() {
      const H = this.model.renderers[0], V = H.data_source, q = dict(V.data), G = H.glyph, [J, Z] = [G.xs.field, G.ys.field];
      if (J) {
        const ne = (q.get(J) ?? []).length - 1, ee = V.get_array(J)[ne];
        ee.splice(ee.length - 1, 1);
      }
      if (Z) {
        const ne = (q.get(Z) ?? []).length - 1, ee = V.get_array(Z)[ne];
        ee.splice(ee.length - 1, 1);
      }
      this._emit_cds_changes(V);
    }
    _keyup(H) {
      if (!(!this.model.active || !this._mouse_in_frame))
        for (const V of this.model.renderers)
          H.key == "Backspace" ? this._delete_selected(V) : H.key == "Escape" && (this._drawing && (this._remove(), this._drawing = !1), V.data_source.selection_manager.clear());
    }
    _pan_start(H) {
      this.model.drag && (this._select_event(H, "append", this.model.renderers), this._basepoint = [H.sx, H.sy]);
    }
    _pan(H) {
      if (this._basepoint == null || !this.model.drag)
        return;
      const [V, q] = this._basepoint;
      for (const G of this.model.renderers) {
        const J = this._map_drag(V, q, G), Z = this._map_drag(H.sx, H.sy, G);
        if (Z == null || J == null)
          continue;
        const te = G.data_source, ne = G.glyph, [ee, ae] = [ne.xs.field, ne.ys.field];
        if (!ee && !ae)
          continue;
        const [oe, se] = Z, [ce, le] = J, [ue, de] = [oe - ce, se - le], fe = dict(te.data);
        for (const he of te.selected.indices) {
          let me, pe, ye;
          ee && (pe = (fe.get(ee) ?? [])[he]), ae ? (ye = (fe.get(ae) ?? [])[he], me = ye.length) : me = pe.length;
          for (let ge = 0; ge < me; ge++)
            pe && (pe[ge] += ue), ye && (ye[ge] += de);
        }
        te.change.emit();
      }
      this._basepoint = [H.sx, H.sy];
    }
    _pan_end(H) {
      if (this.model.drag) {
        this._pan(H);
        for (const V of this.model.renderers)
          this._emit_cds_changes(V.data_source);
        this._basepoint = null;
      }
    }
    activate() {
      if (!(this.model.vertex_renderer == null || !this.model.active)) {
        if (this._show_vertices(), !this._initialized)
          for (const H of this.model.renderers) {
            const V = H.data_source;
            V.connect(V.properties.data.change, () => this._show_vertices());
          }
        this._initialized = !0;
      }
    }
    deactivate() {
      this._drawing && (this._remove(), this._drawing = !1), this.model.vertex_renderer != null && this._hide_vertices();
    }
  }
  mt(PolyDrawToolView, "__name__", "PolyDrawToolView");
  const Nm = class Nm extends PolyTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Polygon Draw Tool");
      mt(this, "tool_icon", tool_icon_poly_draw);
      mt(this, "event_type", ["pan", "tap", "press", "move"]);
      mt(this, "default_order", 3);
    }
  };
  mt(Nm, "__name__", "PolyDrawTool"), Nm.prototype.default_view = PolyDrawToolView, Nm.define(({ Bool: H, Int: V, List: q, Ref: G }) => ({
    drag: [H, !0],
    num_objects: [V, 0],
    renderers: [q(G(GlyphRenderer)), []]
  }));
  let PolyDrawTool = Nm;
  class PolyEditToolView extends PolyToolView {
    constructor() {
      super(...arguments);
      mt(this, "_selected_renderer");
      mt(this, "_drawing", !1);
      mt(this, "_cur_index", null);
    }
    _press(H) {
      if (this.model.vertex_renderer == null || !this.model.active)
        return;
      const V = this._map_drag(H.sx, H.sy, this.model.vertex_renderer);
      if (V == null)
        return;
      const [q, G] = V, J = this._select_event(H, "replace", [this.model.vertex_renderer]), Z = this.model.vertex_renderer.data_source, te = this.model.vertex_renderer.glyph, [ne, ee] = [te.x.field, te.y.field];
      if (J.length != 0 && this._selected_renderer != null) {
        const ae = Z.selected.indices[0];
        this._drawing ? (this._drawing = !1, Z.selection_manager.clear()) : (Z.selected.indices = [ae + 1], ne && Z.get_array(ne).splice(ae + 1, 0, q), ee && Z.get_array(ee).splice(ae + 1, 0, G), this._drawing = !0), Z.change.emit(), this._emit_cds_changes(this._selected_renderer.data_source);
      } else
        this._show_vertices(H);
    }
    _show_vertices(H) {
      if (!this.model.active || this.model.renderers.length == 0)
        return;
      const V = this.model.renderers[0], q = () => this._update_vertices(V), G = V.data_source, J = this._select_event(H, "replace", this.model.renderers);
      if (J.length == 0) {
        this._set_vertices([], []), this._selected_renderer = null, this._drawing = !1, this._cur_index = null, G.disconnect(G.properties.data.change, q);
        return;
      }
      G.connect(G.properties.data.change, q), this._cur_index = J[0].data_source.selected.indices[0], this._update_vertices(J[0]);
    }
    _update_vertices(H) {
      const V = H.glyph, q = dict(H.data_source.data), G = this._cur_index, [J, Z] = [V.xs.field, V.ys.field];
      if (this._drawing || G == null && (J || Z))
        return;
      let te, ne;
      if (J && G != null) {
        const ee = q.get(J) ?? [];
        te = ee[G], isArray$3(te) || (ee[G] = te = Array.from(te));
      } else
        te = V.xs.value;
      if (Z && G != null) {
        const ee = q.get(Z) ?? [];
        ne = ee[G], isArray$3(ne) || (ee[G] = ne = Array.from(ne));
      } else
        ne = V.ys.value;
      this._selected_renderer = H, this._set_vertices(te, ne);
    }
    _move(H) {
      if (this._drawing && this._selected_renderer != null) {
        const V = this.model.vertex_renderer;
        if (V == null)
          return;
        const q = V.data_source, G = dict(q.data), J = V.glyph, Z = this._map_drag(H.sx, H.sy, V);
        if (Z == null)
          return;
        let [te, ne] = Z;
        const ee = q.selected.indices;
        [te, ne] = this._snap_to_vertex(H, te, ne), q.selected.indices = ee;
        const [ae, oe] = [J.x.field, J.y.field], se = ee[0];
        ae && (G.get(ae)[se] = te), oe && (G.get(oe)[se] = ne), q.change.emit(), this._selected_renderer.data_source.change.emit();
      }
    }
    _tap(H) {
      const V = this.model.vertex_renderer;
      if (V == null)
        return;
      const q = this._map_drag(H.sx, H.sy, V);
      if (q == null)
        return;
      if (this._drawing && this._selected_renderer != null) {
        let [J, Z] = q;
        const te = V.data_source, ne = V.glyph, [ee, ae] = [ne.x.field, ne.y.field], oe = te.selected.indices;
        [J, Z] = this._snap_to_vertex(H, J, Z);
        const se = oe[0];
        if (te.selected.indices = [se + 1], ee) {
          const ce = te.get_array(ee), le = ce[se];
          ce[se] = J, ce.splice(se + 1, 0, le);
        }
        if (ae) {
          const ce = te.get_array(ae), le = ce[se];
          ce[se] = Z, ce.splice(se + 1, 0, le);
        }
        te.change.emit(), this._emit_cds_changes(this._selected_renderer.data_source, !0, !1, !0);
        return;
      }
      const G = this._select_mode(H);
      this._select_event(H, G, [V]), this._select_event(H, G, this.model.renderers);
    }
    _remove_vertex() {
      if (!this._drawing || this._selected_renderer == null)
        return;
      const H = this.model.vertex_renderer;
      if (H == null)
        return;
      const V = H.data_source, q = H.glyph, G = V.selected.indices[0], [J, Z] = [q.x.field, q.y.field];
      J && V.get_array(J).splice(G, 1), Z && V.get_array(Z).splice(G, 1), V.change.emit(), this._emit_cds_changes(this._selected_renderer.data_source);
    }
    _pan_start(H) {
      this.model.vertex_renderer != null && (this._select_event(H, "append", [this.model.vertex_renderer]), this._basepoint = [H.sx, H.sy]);
    }
    _pan(H) {
      this._basepoint != null && this.model.vertex_renderer != null && (this._drag_points(H, [this.model.vertex_renderer]), this._selected_renderer != null && this._selected_renderer.data_source.change.emit());
    }
    _pan_end(H) {
      this._basepoint != null && this.model.vertex_renderer != null && (this._drag_points(H, [this.model.vertex_renderer]), this._emit_cds_changes(this.model.vertex_renderer.data_source, !1, !0, !0), this._selected_renderer != null && this._emit_cds_changes(this._selected_renderer.data_source), this._basepoint = null);
    }
    _keyup(H) {
      if (!this.model.active || !this._mouse_in_frame)
        return;
      let V;
      if (this._selected_renderer != null) {
        const { vertex_renderer: q } = this.model;
        V = q != null ? [q] : [];
      } else
        V = this.model.renderers;
      for (const q of V)
        H.key == "Backspace" ? (this._delete_selected(q), this._selected_renderer != null && this._emit_cds_changes(this._selected_renderer.data_source)) : H.key == "Escape" && (this._drawing ? (this._remove_vertex(), this._drawing = !1) : this._selected_renderer != null && this._hide_vertices(), q.data_source.selection_manager.clear());
    }
    deactivate() {
      this._selected_renderer != null && (this._drawing && (this._remove_vertex(), this._drawing = !1), this._hide_vertices());
    }
  }
  mt(PolyEditToolView, "__name__", "PolyEditToolView");
  const Pm = class Pm extends PolyTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Poly Edit Tool");
      mt(this, "tool_icon", tool_icon_poly_edit);
      mt(this, "event_type", ["tap", "press", "pan", "move"]);
      mt(this, "default_order", 4);
    }
  };
  mt(Pm, "__name__", "PolyEditTool"), Pm.prototype.default_view = PolyEditToolView, Pm.define(({ List: H, Ref: V }) => ({
    renderers: [H(V(GlyphRenderer)), []]
  }));
  let PolyEditTool = Pm;
  const b6 = class b6 extends GestureToolView {
    connect_signals() {
      super.connect_signals(), this.model.invert.connect(() => this._invert_selection()), this.model.clear.connect(() => this._clear_selection());
    }
    get computed_renderers() {
      const { renderers: _ } = this.model, H = this.plot_view.model.data_renderers;
      return compute_renderers(_, H);
    }
    _computed_renderers_by_data_source() {
      const _ = /* @__PURE__ */ new Map();
      for (const H of this.computed_renderers) {
        let V;
        if (H instanceof GlyphRenderer)
          V = H.data_source;
        else if (H instanceof GraphRenderer)
          V = H.node_renderer.data_source;
        else
          continue;
        const q = _.get(V) ?? [];
        _.set(V, [...q, H]);
      }
      return _;
    }
    _clear_overlay() {
    }
    _clear_other_overlays() {
      for (const _ of this.plot_view.tool_views.values())
        _ instanceof b6 && _ != this && _._clear_overlay();
    }
    _clear_selection() {
      const { computed_renderers: _ } = this, H = uniq(_.map((V) => V.selection_manager));
      for (const V of H)
        V.clear();
      this.plot_view.request_paint(_);
    }
    _invert_selection() {
      const { computed_renderers: _ } = this, H = uniq(_.map((V) => V.selection_manager));
      for (const V of H)
        V.invert();
      this.plot_view.request_paint(_);
    }
    _select_mode(_) {
      const { shift: H, ctrl: V } = _;
      if (!H && !V)
        return this.model.mode;
      if (H && !V)
        return "append";
      if (!H && V)
        return "intersect";
      if (H && V)
        return "subtract";
      unreachable();
    }
    _keyup(_) {
      this.model.active && _.key == "Escape" && this._clear_selection();
    }
    _emit_selection_event(_, H = !0) {
      const { x_scale: V, y_scale: q } = this.plot_view.frame, G = (() => {
        switch (_.type) {
          case "point": {
            const { sx: J, sy: Z } = _, te = V.invert(J), ne = q.invert(Z);
            return { ..._, x: te, y: ne };
          }
          case "span": {
            const { sx: J, sy: Z } = _, te = V.invert(J), ne = q.invert(Z);
            return { ..._, x: te, y: ne };
          }
          case "rect": {
            const { sx0: J, sx1: Z, sy0: te, sy1: ne } = _, [ee, ae] = V.r_invert(J, Z), [oe, se] = q.r_invert(te, ne);
            return { ..._, x0: ee, y0: oe, x1: ae, y1: se };
          }
          case "poly": {
            const { sx: J, sy: Z } = _, te = V.v_invert(J), ne = q.v_invert(Z);
            return { ..._, x: te, y: ne };
          }
        }
      })();
      this.plot_view.model.trigger_event(new SelectionGeometry(G, H));
    }
  };
  mt(b6, "__name__", "SelectToolView");
  let SelectToolView = b6;
  const T6 = class T6 extends GestureTool {
    constructor(H) {
      super(H);
      mt(this, "invert", new Signal0(this, "invert"));
      mt(this, "clear", new Signal0(this, "clear"));
    }
    get menu() {
      return [
        {
          icon: tool_icon_replace_mode,
          tooltip: "Replace the current selection",
          active: () => this.mode == "replace",
          handler: () => {
            this.mode = "replace", this.active = !0;
          }
        },
        {
          icon: tool_icon_append_mode,
          tooltip: "Append to the current selection (Shift)",
          active: () => this.mode == "append",
          handler: () => {
            this.mode = "append", this.active = !0;
          }
        },
        {
          icon: tool_icon_intersect_mode,
          tooltip: "Intersect with the current selection (Ctrl)",
          active: () => this.mode == "intersect",
          handler: () => {
            this.mode = "intersect", this.active = !0;
          }
        },
        {
          icon: tool_icon_subtract_mode,
          tooltip: "Subtract from the current selection (Shift+Ctrl)",
          active: () => this.mode == "subtract",
          handler: () => {
            this.mode = "subtract", this.active = !0;
          }
        },
        {
          icon: tool_icon_xor_mode,
          tooltip: "Symmetric difference with the current selection",
          active: () => this.mode == "xor",
          handler: () => {
            this.mode = "xor", this.active = !0;
          }
        },
        null,
        {
          icon: tool_icon_invert_selection,
          tooltip: "Invert the current selection",
          handler: () => {
            this.invert.emit();
          }
        },
        {
          icon: tool_icon_clear_selection,
          tooltip: "Clear the current selection and/or selection overlay (Esc)",
          handler: () => {
            this.clear.emit();
          }
        }
      ];
    }
  };
  mt(T6, "__name__", "SelectTool"), T6.define(({ List: H, Ref: V, Or: q, Auto: G }) => ({
    renderers: [q(H(V(DataRenderer)), G), "auto"]
  }));
  let SelectTool = T6;
  class RegionSelectToolView extends SelectToolView {
    get overlays() {
      return [...super.overlays, this.model.overlay];
    }
    _is_continuous(_) {
      return this.model.continuous != _.alt;
    }
    _select(_, H, V) {
      const q = this._computed_renderers_by_data_source();
      for (const [, G] of q) {
        const J = G[0].get_selection_manager(), Z = [];
        for (const te of G) {
          const ne = this.plot_view.renderer_view(te);
          ne != null && Z.push(ne);
        }
        J.select(Z, _, H, V);
      }
      this._emit_selection_event(_, H);
    }
    _clear_overlay() {
      super._clear_overlay(), this.model.overlay.clear();
    }
  }
  mt(RegionSelectToolView, "__name__", "RegionSelectToolView");
  const x6 = class x6 extends SelectTool {
    constructor(_) {
      super(_);
    }
  };
  mt(x6, "__name__", "RegionSelectTool"), x6.define(({ Bool: _ }) => ({
    mode: [RegionSelectionMode, "replace"],
    continuous: [_, !1],
    persistent: [_, !1],
    greedy: [_, !1]
  }));
  let RegionSelectTool = x6;
  class BoxSelectToolView extends RegionSelectToolView {
    constructor() {
      super(...arguments);
      mt(this, "_base_point");
    }
    connect_signals() {
      super.connect_signals();
      const { pan: H } = this.model.overlay;
      this.connect(H, ([q, G]) => {
        if (q == "pan" && this._is_continuous(G) || q == "pan:end") {
          const { left: J, top: Z, right: te, bottom: ne } = this.model.overlay;
          if (!(J instanceof Coordinate) && !(Z instanceof Coordinate) && !(te instanceof Coordinate) && !(ne instanceof Coordinate)) {
            const ee = this._compute_lrtb({ left: J, right: te, top: Z, bottom: ne });
            this._do_select([ee.left, ee.right], [ee.top, ee.bottom], !1, this._select_mode(G));
          }
        }
      });
      const { active: V } = this.model.properties;
      this.on_change(V, () => {
        !this.model.active && !this.model.persistent && this._clear_overlay();
      });
    }
    _compute_limits(H) {
      const V = this.plot_view.frame, q = this.model.dimensions;
      let G = this._base_point;
      if (this.model.origin == "center") {
        const [J, Z] = G, [te, ne] = H;
        G = [J - (te - J), Z - (ne - Z)];
      }
      return this.model._get_dim_limits(G, H, V, q);
    }
    _mappers() {
      const H = (oe, se, ce, le) => {
        switch (oe) {
          case "canvas":
            return le;
          case "screen":
            return ce;
          case "data":
            return se;
        }
      }, { overlay: V } = this.model, { frame: q, canvas: G } = this.plot_view, { x_scale: J, y_scale: Z } = q, { x_view: te, y_view: ne } = q.bbox, { x_screen: ee, y_screen: ae } = G.bbox;
      return {
        left: H(V.left_units, J, te, ee),
        right: H(V.right_units, J, te, ee),
        top: H(V.top_units, Z, ne, ae),
        bottom: H(V.bottom_units, Z, ne, ae)
      };
    }
    _compute_lrtb({ left: H, right: V, top: q, bottom: G }) {
      const J = this._mappers();
      return {
        left: J.left.compute(H),
        right: J.right.compute(V),
        top: J.top.compute(q),
        bottom: J.bottom.compute(G)
      };
    }
    _invert_lrtb({ left: H, right: V, top: q, bottom: G }) {
      const J = this._mappers();
      return {
        left: J.left.invert(H),
        right: J.right.invert(V),
        top: J.top.invert(q),
        bottom: J.bottom.invert(G)
      };
    }
    _pan_start(H) {
      const { sx: V, sy: q } = H, { frame: G } = this.plot_view;
      G.bbox.contains(V, q) && (this._clear_other_overlays(), this._base_point = [V, q]);
    }
    _pan(H) {
      if (this._base_point == null)
        return;
      const { sx: V, sy: q } = H, [G, J] = this._compute_limits([V, q]), [[Z, te], [ne, ee]] = [G, J];
      this.model.overlay.update(this._invert_lrtb({ left: Z, right: te, top: ne, bottom: ee })), this._is_continuous(H.modifiers) && this._do_select(G, J, !1, this._select_mode(H.modifiers));
    }
    _pan_end(H) {
      if (this._base_point == null)
        return;
      const { sx: V, sy: q } = H, [G, J] = this._compute_limits([V, q]);
      this._do_select(G, J, !0, this._select_mode(H.modifiers)), this.model.persistent || this._clear_overlay(), this._base_point = null, this.plot_view.state.push("box_select", { selection: this.plot_view.get_selection() });
    }
    get _is_selecting() {
      return this._base_point != null;
    }
    _stop() {
      this._clear_overlay(), this._base_point = null;
    }
    _keyup(H) {
      if (this.model.active) {
        if (H.key == "Escape") {
          if (this._is_selecting) {
            this._stop();
            return;
          }
          if (this.model.overlay.visible) {
            this._clear_overlay();
            return;
          }
        }
        super._keyup(H);
      }
    }
    _clear_selection() {
      this.model.overlay.visible ? this._clear_overlay() : super._clear_selection();
    }
    _do_select([H, V], [q, G], J, Z = "replace") {
      const { greedy: te } = this.model, ne = { type: "rect", sx0: H, sx1: V, sy0: q, sy1: G, greedy: te };
      this._select(ne, J, Z);
    }
  }
  mt(BoxSelectToolView, "__name__", "BoxSelectToolView");
  const DEFAULT_BOX_OVERLAY$1 = () => new BoxAnnotation({
    syncable: !1,
    level: "overlay",
    visible: !1,
    editable: !0,
    left: NaN,
    right: NaN,
    top: NaN,
    bottom: NaN,
    top_units: "data",
    left_units: "data",
    bottom_units: "data",
    right_units: "data",
    fill_color: "lightgrey",
    fill_alpha: 0.5,
    line_color: "black",
    line_alpha: 1,
    line_width: 2,
    line_dash: [4, 4]
  }), Sh = class Sh extends RegionSelectTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Box Select");
      mt(this, "event_type", "pan");
      mt(this, "default_order", 30);
    }
    initialize() {
      super.initialize();
      const [H, V] = (() => {
        switch (this.dimensions) {
          case "width":
            return ["x", "x"];
          case "height":
            return ["y", "y"];
          case "both":
            return ["all", "both"];
        }
      })(), q = this.origin == "center";
      this.overlay.setv({ resizable: H, movable: V, symmetric: q });
    }
    get computed_icon() {
      const H = super.computed_icon;
      if (H != null)
        return H;
      switch (this.dimensions) {
        case "both":
          return `.${tool_icon_box_select}`;
        case "width":
          return `.${tool_icon_x_box_select}`;
        case "height":
          return `.${tool_icon_y_box_select}`;
      }
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimensions);
    }
  };
  mt(Sh, "__name__", "BoxSelectTool"), Sh.prototype.default_view = BoxSelectToolView, Sh.define(({ Ref: H }) => ({
    dimensions: [Dimensions, "both"],
    overlay: [H(BoxAnnotation), DEFAULT_BOX_OVERLAY$1],
    origin: [BoxOrigin, "corner"]
  })), Sh.register_alias("box_select", () => new Sh()), Sh.register_alias("xbox_select", () => new Sh({ dimensions: "width" })), Sh.register_alias("ybox_select", () => new Sh({ dimensions: "height" }));
  let BoxSelectTool = Sh;
  class BoxZoomToolView extends GestureToolView {
    constructor() {
      super(...arguments);
      mt(this, "_base_point", null);
    }
    get overlays() {
      return [...super.overlays, this.model.overlay];
    }
    _match_aspect([H, V], [q, G], J) {
      const Z = J.bbox.aspect, te = J.bbox.h_range.end, ne = J.bbox.h_range.start, ee = J.bbox.v_range.end, ae = J.bbox.v_range.start;
      let oe = Math.abs(H - q), se = Math.abs(V - G);
      const ce = se == 0 ? 0 : oe / se, [le] = ce >= Z ? [1, ce / Z] : [Z / ce, 1];
      let ue, de;
      H <= q ? (ue = H, de = H + oe * le, de > te && (de = te)) : (de = H, ue = H - oe * le, ue < ne && (ue = ne)), oe = Math.abs(de - ue);
      let fe, he;
      return V <= G ? (he = V, fe = V + oe / Z, fe > ee && (fe = ee)) : (fe = V, he = V - oe / Z, he < ae && (he = ae)), se = Math.abs(fe - he), H <= q ? de = H + Z * se : ue = H - Z * se, [[ue, de], [he, fe]];
    }
    _compute_limits(H, V) {
      const { frame: q } = this.plot_view;
      if (this.model.origin == "center") {
        const [J, Z] = H, [te, ne] = V;
        H = [J - (te - J), Z - (ne - Z)];
      }
      const G = (() => {
        const { dimensions: J } = this.model;
        if (J == "auto") {
          const [Z, te] = H, [ne, ee] = V, ae = Math.abs(Z - ne), oe = Math.abs(te - ee), se = 5;
          return ae < se && oe > se ? "height" : ae > se && oe < se ? "width" : "both";
        } else
          return J;
      })();
      return this.model.match_aspect && G == "both" ? this._match_aspect(H, V, q) : this.model._get_dim_limits(H, V, q, G);
    }
    _pan_start(H) {
      const { sx: V, sy: q } = H;
      this.plot_view.frame.bbox.contains(V, q) && (this._base_point = [V, q]);
    }
    _pan(H) {
      if (this._base_point == null)
        return;
      const [[V, q], [G, J]] = this._compute_limits(this._base_point, [H.sx, H.sy]);
      this.model.overlay.update({ left: V, right: q, top: G, bottom: J });
    }
    _pan_end(H) {
      if (this._base_point == null)
        return;
      const [V, q] = this._compute_limits(this._base_point, [H.sx, H.sy]);
      this._update(V, q), this._stop();
    }
    _stop() {
      this.model.overlay.clear(), this._base_point = null;
    }
    _keydown(H) {
      H.key == "Escape" && this._stop();
    }
    _doubletap(H) {
      var q;
      const { state: V } = this.plot_view;
      ((q = V.peek()) == null ? void 0 : q.type) == "box_zoom" && V.undo();
    }
    _update([H, V], [q, G]) {
      if (Math.abs(V - H) <= 5 || Math.abs(G - q) <= 5)
        return;
      const { x_scales: J, y_scales: Z } = this.plot_view.frame, te = /* @__PURE__ */ new Map();
      for (const [, ae] of J) {
        const [oe, se] = ae.r_invert(H, V);
        te.set(ae.source_range, { start: oe, end: se });
      }
      const ne = /* @__PURE__ */ new Map();
      for (const [, ae] of Z) {
        const [oe, se] = ae.r_invert(q, G);
        ne.set(ae.source_range, { start: oe, end: se });
      }
      const ee = { xrs: te, yrs: ne };
      this.plot_view.state.push("box_zoom", { range: ee }), this.plot_view.update_range(ee), this.plot_view.trigger_ranges_update_event();
    }
  }
  mt(BoxZoomToolView, "__name__", "BoxZoomToolView");
  const DEFAULT_BOX_OVERLAY = () => new BoxAnnotation({
    syncable: !1,
    level: "overlay",
    visible: !1,
    editable: !1,
    left: NaN,
    right: NaN,
    top: NaN,
    bottom: NaN,
    top_units: "canvas",
    left_units: "canvas",
    bottom_units: "canvas",
    right_units: "canvas",
    fill_color: "lightgrey",
    fill_alpha: 0.5,
    line_color: "black",
    line_alpha: 1,
    line_width: 2,
    line_dash: [4, 4]
  }), Yf = class Yf extends GestureTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Box Zoom");
      mt(this, "event_type", ["pan", "doubletap"]);
      mt(this, "default_order", 20);
    }
    get event_role() {
      return "pan";
    }
    get computed_icon() {
      const H = super.computed_icon;
      if (H != null)
        return H;
      switch (this.dimensions) {
        case "both":
          return `.${tool_icon_box_zoom}`;
        case "width":
          return `.${tool_icon_x_box_zoom}`;
        case "height":
          return `.${tool_icon_y_box_zoom}`;
        case "auto":
          return `.${tool_icon_auto_box_zoom}`;
      }
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimensions);
    }
    get menu() {
      return [
        {
          icon: tool_icon_box_zoom,
          tooltip: "Box zoom in both dimensions",
          active: () => this.dimensions == "both",
          handler: () => {
            this.dimensions = "both", this.active = !0;
          }
        },
        {
          icon: tool_icon_x_box_zoom,
          tooltip: "Box zoom in x-dimension",
          active: () => this.dimensions == "width",
          handler: () => {
            this.dimensions = "width", this.active = !0;
          }
        },
        {
          icon: tool_icon_y_box_zoom,
          tooltip: "Box zoom in y-dimension",
          active: () => this.dimensions == "height",
          handler: () => {
            this.dimensions = "height", this.active = !0;
          }
        },
        {
          icon: tool_icon_auto_box_zoom,
          tooltip: "Automatic mode (box zoom in x, y or both dimensions, depending on the mouse gesture)",
          active: () => this.dimensions == "auto",
          handler: () => {
            this.dimensions = "auto", this.active = !0;
          }
        }
      ];
    }
  };
  mt(Yf, "__name__", "BoxZoomTool"), Yf.prototype.default_view = BoxZoomToolView, Yf.define(({ Bool: H, Ref: V, Or: q, Auto: G }) => ({
    dimensions: [q(Dimensions, G), "both"],
    overlay: [V(BoxAnnotation), DEFAULT_BOX_OVERLAY],
    match_aspect: [H, !1],
    origin: [BoxOrigin, "corner"]
  })), Yf.register_alias("box_zoom", () => new Yf({ dimensions: "both" })), Yf.register_alias("xbox_zoom", () => new Yf({ dimensions: "width" })), Yf.register_alias("ybox_zoom", () => new Yf({ dimensions: "height" })), Yf.register_alias("auto_box_zoom", () => new Yf({ dimensions: "auto" }));
  let BoxZoomTool = Yf;
  class PolySelectToolView extends RegionSelectToolView {
    constructor() {
      super(...arguments);
      mt(this, "_is_selecting", !1);
    }
    _mappers() {
      const H = (oe, se, ce, le) => {
        switch (oe) {
          case "canvas":
            return le;
          case "screen":
            return ce;
          case "data":
            return se;
        }
      }, { overlay: V } = this.model, { frame: q, canvas: G } = this.plot_view, { x_scale: J, y_scale: Z } = q, { x_view: te, y_view: ne } = q.bbox, { x_screen: ee, y_screen: ae } = G.bbox;
      return {
        x: H(V.xs_units, J, te, ee),
        y: H(V.ys_units, Z, ne, ae)
      };
    }
    _v_compute(H, V) {
      const { x: q, y: G } = this._mappers();
      return [q.v_compute(H), G.v_compute(V)];
    }
    _v_invert(H, V) {
      const { x: q, y: G } = this._mappers();
      return [q.v_invert(H), G.v_invert(V)];
    }
    connect_signals() {
      super.connect_signals();
      const { pan: H } = this.model.overlay;
      this.connect(H, ([q, G]) => {
        if (q == "pan" && this._is_continuous(G) || q == "pan:end" && !this._is_selecting) {
          const { xs: J, ys: Z } = this.model.overlay, [te, ne] = this._v_compute(J, Z);
          this._do_select(te, ne, !1, this._select_mode(G));
        }
      });
      const { active: V } = this.model.properties;
      this.on_change(V, () => {
        !this.model.active && !this.model.persistent && this._clear_overlay();
      });
    }
    _tap(H) {
      const { sx: V, sy: q } = H, { frame: G } = this.plot_view;
      if (!G.bbox.contains(V, q))
        return;
      this._clear_other_overlays();
      const [J, Z] = (() => {
        if (this._is_selecting) {
          const { xs: ee, ys: ae } = this.model.overlay, [oe, se] = this._v_compute(ee, ae);
          return [[...oe], [...se]];
        } else
          return this._is_selecting = !0, [[], []];
      })();
      J.push(V), Z.push(q);
      const [te, ne] = this._v_invert(J, Z);
      this.model.overlay.update({ xs: te, ys: ne }), this._is_continuous(H.modifiers) && this._do_select(J, Z, !0, this._select_mode(H.modifiers));
    }
    _finish_selection(H) {
      this._is_selecting = !1;
      const { xs: V, ys: q } = this.model.overlay, [G, J] = this._v_compute(V, q);
      this._do_select(G, J, !0, this._select_mode(H)), this.plot_view.state.push("poly_select", { selection: this.plot_view.get_selection() }), this.model.persistent || this._clear_overlay();
    }
    _press(H) {
      this._finish_selection(H.modifiers);
    }
    _keyup(H) {
      if (this.model.active) {
        if (H.key == "Enter") {
          this._finish_selection(H.modifiers);
          return;
        }
        if (H.key == "Escape" && this.model.overlay.visible) {
          this._clear_overlay();
          return;
        }
        super._keyup(H);
      }
    }
    _clear_selection() {
      this.model.overlay.visible ? this._clear_overlay() : (this._is_selecting = !1, super._clear_selection());
    }
    _clear_overlay() {
      this._is_selecting = !1, super._clear_overlay();
    }
    _do_select(H, V, q, G) {
      const { greedy: J } = this.model, Z = { type: "poly", sx: H, sy: V, greedy: J };
      this._select(Z, q, G);
    }
  }
  mt(PolySelectToolView, "__name__", "PolySelectToolView");
  const DEFAULT_POLY_OVERLAY = () => new PolyAnnotation({
    syncable: !1,
    level: "overlay",
    visible: !1,
    editable: !0,
    xs_units: "data",
    ys_units: "data",
    fill_color: "lightgrey",
    fill_alpha: 0.5,
    line_color: "black",
    line_alpha: 1,
    line_width: 2,
    line_dash: [4, 4]
  }), w0 = class w0 extends RegionSelectTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Poly Select");
      mt(this, "tool_icon", tool_icon_polygon_select);
      mt(this, "event_type", "tap");
      mt(this, "default_order", 11);
    }
  };
  mt(w0, "__name__", "PolySelectTool"), w0.prototype.default_view = PolySelectToolView, w0.define(({ Ref: H }) => ({
    overlay: [H(PolyAnnotation), DEFAULT_POLY_OVERLAY]
  })), w0.register_alias("poly_select", () => new w0());
  let PolySelectTool = w0;
  class LassoSelectToolView extends RegionSelectToolView {
    constructor() {
      super(...arguments);
      mt(this, "_is_selecting", !1);
    }
    _mappers() {
      const H = (oe, se, ce, le) => {
        switch (oe) {
          case "canvas":
            return le;
          case "screen":
            return ce;
          case "data":
            return se;
        }
      }, { overlay: V } = this.model, { frame: q, canvas: G } = this.plot_view, { x_scale: J, y_scale: Z } = q, { x_view: te, y_view: ne } = q.bbox, { x_screen: ee, y_screen: ae } = G.bbox;
      return {
        x: H(V.xs_units, J, te, ee),
        y: H(V.ys_units, Z, ne, ae)
      };
    }
    _v_compute(H, V) {
      const { x: q, y: G } = this._mappers();
      return [q.v_compute(H), G.v_compute(V)];
    }
    _v_invert(H, V) {
      const { x: q, y: G } = this._mappers();
      return [q.v_invert(H), G.v_invert(V)];
    }
    connect_signals() {
      super.connect_signals();
      const { pan: H } = this.model.overlay;
      this.connect(H, ([q, G]) => {
        if (q == "pan" && this._is_continuous(G) || q == "pan:end") {
          const { xs: J, ys: Z } = this.model.overlay, [te, ne] = this._v_compute(J, Z);
          this._do_select(te, ne, !1, this._select_mode(G));
        }
      });
      const { active: V } = this.model.properties;
      this.on_change(V, () => {
        !this.model.active && !this.model.persistent && this._clear_overlay();
      });
    }
    _pan_start(H) {
      const { sx: V, sy: q } = H, { frame: G } = this.plot_view;
      if (!G.bbox.contains(V, q))
        return;
      this._clear_other_overlays(), this._is_selecting = !0;
      const [J, Z] = this._v_invert([V], [q]);
      this.model.overlay.update({ xs: J, ys: Z });
    }
    _pan(H) {
      assert$2(this._is_selecting);
      const [V, q] = (() => {
        const { xs: ne, ys: ee } = this.model.overlay, [ae, oe] = this._v_compute(ne, ee);
        return [[...ae], [...oe]];
      })(), [G, J] = this.plot_view.frame.bbox.clip(H.sx, H.sy);
      V.push(G), q.push(J);
      const [Z, te] = this._v_invert(V, q);
      this.model.overlay.update({ xs: Z, ys: te }), this._is_continuous(H.modifiers) && this._do_select(V, q, !1, this._select_mode(H.modifiers));
    }
    _pan_end(H) {
      assert$2(this._is_selecting), this._is_selecting = !1;
      const { xs: V, ys: q } = this.model.overlay, [G, J] = this._v_compute(V, q);
      this._do_select(G, J, !0, this._select_mode(H.modifiers)), this.plot_view.state.push("lasso_select", { selection: this.plot_view.get_selection() }), this.model.persistent || this._clear_overlay();
    }
    _keyup(H) {
      if (this.model.active) {
        if (H.key == "Escape" && this.model.overlay.visible) {
          this._clear_overlay();
          return;
        }
        super._keyup(H);
      }
    }
    _clear_selection() {
      this.model.overlay.visible ? this._clear_overlay() : super._clear_selection();
    }
    _do_select(H, V, q, G) {
      const { greedy: J } = this.model, Z = { type: "poly", sx: H, sy: V, greedy: J };
      this._select(Z, q, G);
    }
  }
  mt(LassoSelectToolView, "__name__", "LassoSelectToolView");
  const w2 = class w2 extends RegionSelectTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Lasso Select");
      mt(this, "tool_icon", tool_icon_lasso_select);
      mt(this, "event_type", "pan");
      mt(this, "default_order", 12);
    }
  };
  mt(w2, "__name__", "LassoSelectTool"), w2.prototype.default_view = LassoSelectToolView, w2.define(({ Ref: H }) => ({
    overlay: [H(PolyAnnotation), DEFAULT_POLY_OVERLAY]
  })), w2.override({
    continuous: !0
  }), w2.register_alias("lasso_select", () => new w2());
  let LassoSelectTool = w2;
  function update_ranges(O, _, H) {
    const V = /* @__PURE__ */ new Map();
    for (const [, q] of O) {
      const [G, J] = q.r_invert(_, H);
      V.set(q.source_range, { start: G, end: J });
    }
    return V;
  }
  class PanToolView extends GestureToolView {
    constructor() {
      super(...arguments);
      mt(this, "last_dx");
      mt(this, "last_dy");
      mt(this, "v_axis_only");
      mt(this, "h_axis_only");
      mt(this, "pan_info");
    }
    cursor(H, V) {
      const q = this.plot_view.axis_views.find((G) => G.bbox.contains(H, V));
      if (q != null)
        switch (q.dimension) {
          case 0:
            return "ew-resize";
          case 1:
            return "ns-resize";
        }
      else return this.plot_view.frame.bbox.contains(H, V) ? "move" : super.cursor(H, V);
    }
    _pan_start(H) {
      var J;
      this.last_dx = 0, this.last_dy = 0;
      const { sx: V, sy: q } = H, G = this.plot_view.frame.bbox;
      if (!G.contains(V, q)) {
        const Z = G.h_range, te = G.v_range;
        (V < Z.start || V > Z.end) && (this.v_axis_only = !0), (q < te.start || q > te.end) && (this.h_axis_only = !0);
      }
      (J = this.model.document) == null || J.interactive_start(this.plot_view.model);
    }
    _pan(H) {
      var V;
      this._update(H.dx, H.dy), (V = this.model.document) == null || V.interactive_start(this.plot_view.model);
    }
    _pan_end(H) {
      this.h_axis_only = !1, this.v_axis_only = !1, this.pan_info != null && this.plot_view.state.push("pan", { range: this.pan_info }), this.plot_view.trigger_ranges_update_event();
    }
    _update(H, V) {
      const q = this.plot_view.frame, G = H - this.last_dx, J = V - this.last_dy, Z = q.bbox.h_range, te = Z.start - G, ne = Z.end - G, ee = q.bbox.v_range, ae = ee.start - J, oe = ee.end - J, se = this.model.dimensions;
      let ce, le, ue;
      (se == "width" || se == "both") && !this.v_axis_only ? (ce = te, le = ne, ue = -G) : (ce = Z.start, le = Z.end, ue = 0);
      let de, fe, he;
      (se == "height" || se == "both") && !this.h_axis_only ? (de = ae, fe = oe, he = -J) : (de = ee.start, fe = ee.end, he = 0), this.last_dx = H, this.last_dy = V;
      const { x_scales: me, y_scales: pe } = q, ye = update_ranges(me, ce, le), ge = update_ranges(pe, de, fe);
      this.pan_info = { xrs: ye, yrs: ge, sdx: ue, sdy: he }, this.plot_view.update_range(this.pan_info, { panning: !0 });
    }
  }
  mt(PanToolView, "__name__", "PanToolView");
  const Eh = class Eh extends GestureTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Pan");
      mt(this, "event_type", "pan");
      mt(this, "default_order", 10);
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimensions);
    }
    get computed_icon() {
      const H = super.computed_icon;
      if (H != null)
        return H;
      switch (this.dimensions) {
        case "both":
          return `.${tool_icon_pan}`;
        case "width":
          return `.${tool_icon_x_pan}`;
        case "height":
          return `.${tool_icon_y_pan}`;
      }
    }
    get menu() {
      return [
        {
          icon: tool_icon_pan,
          tooltip: "Pan in both dimensions",
          active: () => this.dimensions == "both",
          handler: () => {
            this.dimensions = "both", this.active = !0;
          }
        },
        {
          icon: tool_icon_x_pan,
          tooltip: "Pan in x-dimension",
          active: () => this.dimensions == "width",
          handler: () => {
            this.dimensions = "width", this.active = !0;
          }
        },
        {
          icon: tool_icon_y_pan,
          tooltip: "Pan in y-dimension",
          active: () => this.dimensions == "height",
          handler: () => {
            this.dimensions = "height", this.active = !0;
          }
        }
      ];
    }
  };
  mt(Eh, "__name__", "PanTool"), Eh.prototype.default_view = PanToolView, Eh.define(() => ({
    dimensions: [Dimensions, "both"]
  })), Eh.register_alias("pan", () => new Eh({ dimensions: "both" })), Eh.register_alias("xpan", () => new Eh({ dimensions: "width" })), Eh.register_alias("ypan", () => new Eh({ dimensions: "height" }));
  let PanTool = Eh;
  class RangeToolView extends ToolView {
    get overlays() {
      return [...super.overlays, this.model.overlay];
    }
    initialize() {
      super.initialize(), this.model.update_overlay_from_ranges();
    }
    connect_signals() {
      super.connect_signals();
      const _ = () => this.model.update_overlay_from_ranges();
      this.on_transitive_change(this.model.properties.x_range, _), this.on_transitive_change(this.model.properties.y_range, _), this.model.overlay.pan.connect(([G, J]) => {
        if (G == "pan")
          this.model.update_ranges_from_overlay();
        else if (G == "pan:end") {
          const Z = [this.model.x_range, this.model.y_range].filter(non_null);
          this.parent.trigger_ranges_update_event(Z);
        }
      });
      const { active: H, x_interaction: V, y_interaction: q } = this.model.properties;
      this.on_change([H, V, q], () => {
        this.model.update_constraints();
      });
    }
  }
  mt(RangeToolView, "__name__", "RangeToolView");
  const DEFAULT_RANGE_OVERLAY = () => new BoxAnnotation({
    syncable: !1,
    level: "overlay",
    visible: !0,
    editable: !0,
    propagate_hover: !0,
    left: NaN,
    right: NaN,
    top: NaN,
    bottom: NaN,
    left_limit: Node$4.frame.left,
    right_limit: Node$4.frame.right,
    top_limit: Node$4.frame.top,
    bottom_limit: Node$4.frame.bottom,
    fill_color: "lightgrey",
    fill_alpha: 0.5,
    line_color: "black",
    line_alpha: 1,
    line_width: 0.5,
    line_dash: [2, 2]
  }), Xp = class Xp extends Tool {
    constructor(H) {
      super(H);
      mt(this, "_nodes", Node$4.frame.freeze());
      mt(this, "tool_name", "Range Tool");
      mt(this, "tool_icon", tool_icon_range);
    }
    initialize() {
      super.initialize(), this.update_constraints();
    }
    update_constraints() {
      this.overlay.editable = this.active;
      const H = this.x_range != null && this.x_interaction, V = this.y_range != null && this.y_interaction;
      H && V ? (this.overlay.movable = "both", this.overlay.resizable = "all") : H ? (this.overlay.movable = "x", this.overlay.resizable = "x") : V ? (this.overlay.movable = "y", this.overlay.resizable = "y") : (this.overlay.movable = "none", this.overlay.resizable = "none");
      const { x_range: q, y_range: G } = this;
      q != null && (this.overlay.min_width = q.min_interval ?? 0, this.overlay.max_width = q.max_interval ?? 1 / 0), G != null && (this.overlay.min_height = G.min_interval ?? 0, this.overlay.max_height = G.max_interval ?? 1 / 0);
    }
    update_ranges_from_overlay() {
      const { left: H, right: V, top: q, bottom: G } = this.overlay, { x_range: J, y_range: Z } = this, te = /* @__PURE__ */ new Set(), ne = /* @__PURE__ */ new Map(), ee = /* @__PURE__ */ new Map();
      if (J != null && this.x_interaction) {
        assert$2(isNumber$3(H) && isNumber$3(V)), ne.set(J, { start: H, end: V });
        for (const ae of J.plots)
          te.add(ae);
      }
      if (Z != null && this.y_interaction) {
        assert$2(isNumber$3(G) && isNumber$3(q)), ee.set(Z, { start: G, end: q });
        for (const ae of Z.plots)
          te.add(ae);
      }
      for (const ae of te)
        ae.update_range({ xrs: ne, yrs: ee }, { panning: !0, scrolling: !0 });
    }
    update_overlay_from_ranges() {
      const { x_range: H, y_range: V } = this, q = H != null, G = V != null;
      this.overlay.update({
        left: q ? H.start : this._nodes.left,
        right: q ? H.end : this._nodes.right,
        top: G ? V.end : this._nodes.top,
        bottom: G ? V.start : this._nodes.bottom
      }), !q && !G && (logger$1.warn("RangeTool not configured with any Ranges."), this.overlay.clear());
    }
    tool_button() {
      return new OnOffButton({ tool: this });
    }
  };
  mt(Xp, "__name__", "RangeTool"), Xp.prototype.default_view = RangeToolView, Xp.define(({ Bool: H, Ref: V, Nullable: q }) => ({
    x_range: [q(V(Range$2)), null],
    y_range: [q(V(Range$2)), null],
    x_interaction: [H, !0],
    y_interaction: [H, !0],
    overlay: [V(BoxAnnotation), DEFAULT_RANGE_OVERLAY]
  })), Xp.override({
    active: !0
  });
  let RangeTool = Xp;
  const Modifiers = PartialStruct({ shift: Bool$3, ctrl: Bool$3, alt: Bool$3 });
  class TapToolView extends SelectToolView {
    _tap(_) {
      this.model.gesture == "tap" && this._handle_tap(_);
    }
    _doubletap(_) {
      this.model.gesture == "doubletap" && this._handle_tap(_);
    }
    _handle_tap(_) {
      const { modifiers: H } = this.model;
      if (H.shift != null && H.shift != _.modifiers.shift || H.ctrl != null && H.ctrl != _.modifiers.ctrl || H.alt != null && H.alt != _.modifiers.alt)
        return;
      const { sx: V, sy: q } = _, { frame: G } = this.plot_view;
      if (!G.bbox.contains(V, q))
        return;
      this._clear_other_overlays();
      const J = { type: "point", sx: V, sy: q };
      this.model.behavior == "select" ? this._select(J, !0, this._select_mode(_.modifiers)) : this._inspect(J, _.modifiers);
    }
    _select(_, H, V) {
      const q = this._computed_renderers_by_data_source();
      for (const [, G] of q) {
        const J = G[0].get_selection_manager(), Z = G.map((ne) => this.plot_view.renderer_view(ne)).filter(non_null);
        if (J.select(Z, _, H, V)) {
          const [ne] = Z;
          this._emit_callback(ne, _, J.source);
        }
      }
      this._emit_selection_event(_), this.plot_view.state.push("tap", { selection: this.plot_view.get_selection() });
    }
    _inspect(_, H) {
      for (const V of this.computed_renderers) {
        const q = this.plot_view.renderer_view(V);
        if (q == null)
          continue;
        const G = V.get_selection_manager();
        G.inspect(q, _) && this._emit_callback(q, _, G.source, H);
      }
    }
    _emit_callback(_, H, V, q) {
      const { callback: G } = this.model;
      if (G != null) {
        const J = _.coordinates.x_scale.invert(H.sx), Z = _.coordinates.y_scale.invert(H.sy), te = {
          geometries: { ...H, x: J, y: Z },
          source: V,
          event: { modifiers: q }
        };
        execute(G, this.model, te);
      }
    }
  }
  mt(TapToolView, "__name__", "TapToolView");
  const Ah = class Ah extends SelectTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Tap");
      mt(this, "tool_icon", tool_icon_tap_select);
      mt(this, "event_type", "tap");
      mt(this, "default_order", 10);
    }
    get menu() {
      const H = super.menu;
      return H == null ? null : prepend$1(H, {
        icon: tool_icon_toggle_mode,
        tooltip: "Toggle the current selection",
        active: () => this.mode == "toggle",
        handler: () => {
          this.mode = "toggle", this.active = !0;
        }
      });
    }
  };
  mt(Ah, "__name__", "TapTool"), Ah.prototype.default_view = TapToolView, Ah.define(({ Any: H, Nullable: V }) => ({
    mode: [SelectionMode, "toggle"],
    behavior: [TapBehavior, "select"],
    gesture: [TapGesture, "tap"],
    modifiers: [Modifiers, {}],
    callback: [V(
      H
      /*TODO*/
    ), null]
  })), Ah.register_alias("click", () => new Ah({ behavior: "inspect" })), Ah.register_alias("tap", () => new Ah()), Ah.register_alias("doubletap", () => new Ah({ gesture: "doubletap" }));
  let TapTool = Ah;
  class WheelPanToolView extends GestureToolView {
    _scroll(_) {
      let H = this.model.speed * _.delta;
      H > 0.9 ? H = 0.9 : H < -0.9 && (H = -0.9), this._update_ranges(H);
    }
    _update_ranges(_) {
      var fe;
      const { frame: H } = this.plot_view, V = H.bbox.h_range, q = H.bbox.v_range, [G, J] = [V.start, V.end], [Z, te] = [q.start, q.end];
      let ne, ee, ae, oe;
      switch (this.model.dimension) {
        case "height": {
          const he = Math.abs(te - Z);
          ne = G, ee = J, ae = Z - he * _, oe = te - he * _;
          break;
        }
        case "width": {
          const he = Math.abs(J - G);
          ne = G - he * _, ee = J - he * _, ae = Z, oe = te;
          break;
        }
      }
      const { x_scales: se, y_scales: ce } = H, le = update_ranges(se, ne, ee), ue = update_ranges(ce, ae, oe), de = { xrs: le, yrs: ue, factor: _ };
      this.plot_view.state.push("wheel_pan", { range: de }), this.plot_view.update_range(de, { scrolling: !0 }), (fe = this.model.document) == null || fe.interactive_start(this.plot_view.model, () => this.plot_view.trigger_ranges_update_event());
    }
  }
  mt(WheelPanToolView, "__name__", "WheelPanToolView");
  const jh = class jh extends GestureTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Wheel Pan");
      mt(this, "tool_icon", tool_icon_wheel_pan);
      mt(this, "event_type", "scroll");
      mt(this, "default_order", 12);
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimension);
    }
  };
  mt(jh, "__name__", "WheelPanTool"), jh.prototype.default_view = WheelPanToolView, jh.define(() => ({
    dimension: [Dimension$1, "width"]
  })), jh.internal(({ Float: H }) => ({
    speed: [H, 1 / 1e3]
  })), jh.register_alias("xwheel_pan", () => new jh({ dimension: "width" })), jh.register_alias("ywheel_pan", () => new jh({ dimension: "height" }));
  let WheelPanTool = jh;
  const ZoomTogether = Enum("none", "cross", "all"), Renderers = Or$1(List$2(Ref(DataRenderer)), Auto);
  class WheelZoomToolView extends GestureToolView {
    _scroll(_) {
      const { sx: H, sy: V, delta: q } = _;
      this.zoom(H, V, q);
    }
    _pinch(_) {
      const { sx: H, sy: V, scale: q } = _, G = q >= 1 ? (q - 1) * 20 : -20 / q;
      this.zoom(H, V, G);
    }
    zoom(_, H, V) {
      var Ee;
      const q = this.plot_view.axis_views.find((xe) => xe.bbox.contains(_, H));
      if (q != null && !this.model.zoom_on_axis)
        return;
      const { frame: G } = this.plot_view;
      if (q == null && !G.bbox.contains(_, H))
        return;
      const [J, Z] = (() => {
        const xe = [...G.x_scales.values()], Le = [...G.y_scales.values()];
        if (q == null)
          return [xe, Le];
        {
          const { zoom_together: Qe } = this.model;
          if (Qe == "all")
            return q.dimension == 0 ? [xe, []] : [[], Le];
          {
            const { x_scale: ke, y_scale: Me } = q.coordinates;
            switch (Qe) {
              case "cross":
                return [[ke], [Me]];
              case "none":
                return q.dimension == 0 ? [[ke], []] : [[], [Me]];
            }
          }
        }
      })(), te = new Set(J), ne = new Set(Z), ee = /* @__PURE__ */ new Set(), ae = /* @__PURE__ */ new Set(), { renderers: oe } = this.model, se = oe != "auto" ? oe : this.plot_view.model.data_renderers;
      for (const xe of se) {
        if (xe.coordinates == null)
          continue;
        const Le = this.plot_view.renderer_view(xe);
        assert$2(Le != null);
        const { x_scale: Qe, y_scale: ke } = Le.coordinates;
        Qe instanceof CompositeScale && te.has(Qe.target_scale) && ee.add(Qe), ke instanceof CompositeScale && ne.has(ke.target_scale) && ae.add(ke);
      }
      const [ce, le] = oe == "auto" ? [
        /* @__PURE__ */ new Set([...te, ...ee]),
        /* @__PURE__ */ new Set([...ne, ...ae])
      ] : [
        ee,
        ae
      ], ue = { x: !1, y: !1 }, de = (xe, Le) => {
        const { level: Qe } = this.model;
        for (let ke = 0; ke < Qe; ke++)
          if (xe instanceof CompositeScale)
            ue[Le] = !0, xe = xe.source_scale;
          else {
            logger$1.warn(`can't reach sub-coordinate level ${Qe} for ${xe} in ${Le} dimension; stopped at ${ke}`);
            break;
          }
        return xe instanceof CompositeScale ? xe.target_scale : xe;
      }, fe = /* @__PURE__ */ new Set(), he = /* @__PURE__ */ new Set();
      for (const xe of ce)
        fe.add(de(xe, "x"));
      for (const xe of le)
        he.add(de(xe, "y"));
      const me = (() => {
        const xe = ue.x ? null : _, Le = ue.y ? null : H;
        return q != null ? q.dimension == 0 ? { x: xe, y: null } : { x: null, y: Le } : { x: xe, y: Le };
      })(), pe = this.model.dimensions, ye = pe == "width" || pe == "both", ge = pe == "height" || pe == "both", { x_target: ve, y_target: be } = G, Ce = this.model.speed * V, Se = scale_range(fe, he, ve, be, Ce, ye, ge, me);
      this.plot_view.state.push("wheel_zoom", { range: Se });
      const { maintain_focus: we } = this.model;
      this.plot_view.update_range(Se, { scrolling: !0, maintain_focus: we }), (Ee = this.model.document) == null || Ee.interactive_start(this.plot_view.model, () => this.plot_view.trigger_ranges_update_event());
    }
  }
  mt(WheelZoomToolView, "__name__", "WheelZoomToolView");
  const Qh = class Qh extends GestureTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Wheel Zoom");
      mt(this, "tool_icon", tool_icon_wheel_zoom);
      mt(this, "event_type", "scroll");
      mt(this, "default_order", 10);
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimensions);
    }
  };
  mt(Qh, "__name__", "WheelZoomTool"), Qh.prototype.default_view = WheelZoomToolView, Qh.define(({ Bool: H, Float: V, NonNegative: q, Int: G }) => ({
    dimensions: [Dimensions, "both"],
    renderers: [Renderers, "auto"],
    level: [q(G), 0],
    maintain_focus: [H, !0],
    zoom_on_axis: [H, !0],
    zoom_together: [ZoomTogether, "all"],
    speed: [V, 1 / 600]
  })), Qh.register_alias("wheel_zoom", () => new Qh({ dimensions: "both" })), Qh.register_alias("xwheel_zoom", () => new Qh({ dimensions: "width" })), Qh.register_alias("ywheel_zoom", () => new Qh({ dimensions: "height" }));
  let WheelZoomTool = Qh;
  class CrosshairToolView extends InspectToolView {
    constructor() {
      super(...arguments);
      mt(this, "_spans");
    }
    get overlays() {
      return [...super.overlays, ...this._spans];
    }
    initialize() {
      super.initialize(), this._update_overlays();
    }
    connect_signals() {
      super.connect_signals();
      const { overlay: H, dimensions: V, line_color: q, line_width: G, line_alpha: J } = this.model.properties;
      this.on_change([H, V, q, G, J], () => {
        this._update_overlays();
      });
    }
    _update_overlays() {
      const { overlay: H } = this.model;
      if (H == "auto") {
        let te = function(ne) {
          return new Span$3({
            dimension: ne,
            location_units: "canvas",
            level: "overlay",
            line_color: G,
            line_width: Z,
            line_alpha: J
          });
        };
        var V = te;
        const { dimensions: q, line_color: G, line_alpha: J, line_width: Z } = this.model;
        switch (q) {
          case "width": {
            this._spans = [te("width")];
            break;
          }
          case "height": {
            this._spans = [te("height")];
            break;
          }
          case "both": {
            this._spans = [te("width"), te("height")];
            break;
          }
        }
      } else isArray$3(H) ? this._spans = [...H] : this._spans = [H];
    }
    _move(H) {
      if (!this.model.active)
        return;
      const { sx: V, sy: q } = H;
      this.plot_view.frame.bbox.contains(V, q) ? this._update_spans(V, q) : this._update_spans(NaN, NaN);
    }
    _move_exit(H) {
      this._update_spans(NaN, NaN);
    }
    _update_spans(H, V) {
      const { frame: q } = this.plot_view;
      function G(J, Z, te) {
        const { dimension: ne } = J;
        switch (J.location_units) {
          case "canvas":
            return ne == "width" ? te : Z;
          case "screen": {
            const { xview: ee, yview: ae } = q.bbox;
            return ne == "width" ? ae.invert(te) : ee.invert(Z);
          }
          case "data": {
            const { x_scale: ee, y_scale: ae } = q;
            return ne == "width" ? ae.invert(te) : ee.invert(Z);
          }
        }
      }
      for (const J of this._spans)
        J.location = G(J, H, V);
    }
  }
  mt(CrosshairToolView, "__name__", "CrosshairToolView");
  const S0 = class S0 extends InspectTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Crosshair");
      mt(this, "tool_icon", tool_icon_crosshair);
    }
    get tooltip() {
      return this._get_dim_tooltip(this.dimensions);
    }
  };
  mt(S0, "__name__", "CrosshairTool"), S0.prototype.default_view = CrosshairToolView, S0.define(({ Alpha: H, Float: V, Color: q, Auto: G, Tuple: J, Ref: Z, Or: te }) => ({
    overlay: [te(G, Z(Span$3), J(Z(Span$3), Z(Span$3))), "auto"],
    dimensions: [Dimensions, "both"],
    line_color: [q, "black"],
    line_width: [V, 1],
    line_alpha: [H, 1]
  })), S0.register_alias("crosshair", () => new S0());
  let CrosshairTool = S0;
  const w6 = class w6 extends Model$1 {
    constructor(_) {
      super(_);
    }
    get values() {
      return values$3(this.args);
    }
    /*protected*/
    _make_code(_, H, V, q) {
      return new Function(...keys$5(this.args), _, H, V, use_strict(q));
    }
    format(_, H, V) {
      return this._make_code("value", "format", "special_vars", this.code)(...this.values, _, H, V);
    }
  };
  mt(w6, "__name__", "CustomJSHover"), w6.define(({ Unknown: _, Str: H, Dict: V }) => ({
    args: [V(_), {}],
    code: [H, ""]
  }));
  let CustomJSHover = w6;
  class ActionView extends View$2 {
  }
  mt(ActionView, "__name__", "ActionView");
  class Action extends Model$1 {
    constructor(_) {
      super(_);
    }
  }
  mt(Action, "__name__", "Action"), mt(Action, "__module__", "bokeh.models.dom");
  class PlaceholderView extends DOMNodeView {
    render() {
    }
  }
  mt(PlaceholderView, "__name__", "PlaceholderView"), mt(PlaceholderView, "tag_name", "span");
  class Placeholder extends DOMNode {
    constructor(_) {
      super(_);
    }
  }
  mt(Placeholder, "__name__", "Placeholder");
  class TemplateView extends DOMElementView {
    constructor() {
      super(...arguments);
      mt(this, "action_views", /* @__PURE__ */ new Map());
    }
    *children() {
      yield* super.children(), yield* this.action_views.values();
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await build_views(this.action_views, this.model.actions, { parent: this });
    }
    remove() {
      remove_views(this.action_views), super.remove();
    }
    update(H, V, q = {}) {
      function G(J) {
        for (const Z of J.child_views.values())
          Z instanceof PlaceholderView ? Z.update(H, V, q) : Z instanceof DOMElementView && G(Z);
      }
      G(this);
      for (const J of this.action_views.values())
        J.update(H, V, q);
    }
  }
  mt(TemplateView, "__name__", "TemplateView"), mt(TemplateView, "tag_name", "div");
  const $m = class $m extends DOMElement {
  };
  mt($m, "__name__", "Template"), $m.prototype.default_view = TemplateView, $m.define(({ List: _, Ref: H }) => ({
    actions: [_(H(Action)), []]
  }));
  let Template = $m;
  function _nearest_line_hit(O, _, H, V) {
    const q = { x: H[O], y: V[O] }, G = { x: H[O + 1], y: V[O + 1] }, { sx: J, sy: Z } = _, [te, ne] = function() {
      if (_.type == "span")
        return _.direction == "h" ? [Math.abs(q.x - J), Math.abs(G.x - J)] : [Math.abs(q.y - Z), Math.abs(G.y - Z)];
      const ee = { x: J, y: Z }, ae = dist_2_pts(q, ee), oe = dist_2_pts(G, ee);
      return [ae, oe];
    }();
    return te < ne ? [[q.x, q.y], O] : [[G.x, G.y], O + 1];
  }
  function _line_hit(O, _, H) {
    return [[O[H], _[H]], H];
  }
  class HoverToolView extends InspectToolView {
    constructor() {
      super(...arguments);
      mt(this, "_current_sxy", null);
      mt(this, "ttmodels", /* @__PURE__ */ new Map());
      mt(this, "_ttviews", /* @__PURE__ */ new Map());
      mt(this, "_template_el");
      mt(this, "_template_view");
    }
    *children() {
      yield* super.children(), yield* this._ttviews.values(), this._template_view != null && (yield this._template_view);
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._update_ttmodels();
      const { tooltips: H } = this.model;
      H instanceof Template && (this._template_view = await build_view(H, { parent: this.plot_view.canvas }), this._template_view.render());
    }
    remove() {
      var H;
      (H = this._template_view) == null || H.remove(), remove_views(this._ttviews), super.remove();
    }
    connect_signals() {
      super.connect_signals();
      const H = this.plot_view.model.properties.renderers, { renderers: V, tooltips: q } = this.model.properties;
      this.on_change(q, () => delete this._template_el), this.on_change([H, V, q], async () => await this._update_ttmodels()), this.connect(this.plot_view.repainted, () => {
        if (this.model.active && this._current_sxy != null) {
          const [G, J] = this._current_sxy;
          this._inspect(G, J);
        }
      });
    }
    async _update_ttmodels() {
      const { ttmodels: H } = this;
      H.clear();
      const { tooltips: V } = this.model;
      if (V == null)
        return;
      const { computed_renderers: q } = this;
      for (const Z of q) {
        const te = new Tooltip({
          content: document.createElement("div"),
          attachment: this.model.attachment,
          show_arrow: this.model.show_arrow,
          interactive: !1,
          visible: !0,
          position: null
        });
        Z instanceof GlyphRenderer ? H.set(Z, te) : Z instanceof GraphRenderer && (H.set(Z.node_renderer, te), H.set(Z.edge_renderer, te));
      }
      await build_views(this._ttviews, [...H.values()], { parent: this.plot_view });
      const G = [...function* () {
        for (const Z of q)
          Z instanceof GlyphRenderer ? yield Z : Z instanceof GraphRenderer && (yield Z.node_renderer, yield Z.edge_renderer);
      }()], J = this._slots.get(this.update);
      if (J != null) {
        const Z = new Set(G.map((te) => te.data_source));
        Signal.disconnect_receiver(this, J, Z);
      }
      for (const Z of G)
        this.connect(Z.data_source.inspect, this.update);
    }
    get computed_renderers() {
      const { renderers: H } = this.model, V = this.plot_view.model.data_renderers;
      return compute_renderers(H, V);
    }
    _clear() {
      this._inspect(1 / 0, 1 / 0);
      for (const [, H] of this.ttmodels)
        H.clear();
    }
    _move(H) {
      if (!this.model.active)
        return;
      const { sx: V, sy: q } = H;
      this.plot_view.frame.bbox.contains(V, q) ? (this._current_sxy = [V, q], this._inspect(V, q)) : this._clear();
    }
    _move_exit() {
      this._current_sxy = null, this._clear();
    }
    _inspect(H, V) {
      const q = this.model.mode == "mouse" ? { type: "point", sx: H, sy: V } : { type: "span", direction: this.model.mode == "vline" ? "h" : "v", sx: H, sy: V };
      for (const G of this.computed_renderers) {
        const J = G.get_selection_manager(), Z = this.plot_view.renderer_view(G);
        Z != null && J.inspect(Z, q);
      }
      this._emit_callback(q);
    }
    _update(H, V, q) {
      var me, pe;
      const G = H.get_selection_manager(), J = G.inspectors.get(H), Z = H.view.convert_selection_to_subset(J);
      if (J.is_empty() && J.view == null) {
        q.clear();
        return;
      }
      const te = G.source, ne = this.plot_view.renderer_view(H);
      if (ne == null)
        return;
      const { sx: ee, sy: ae } = V, oe = ne.coordinates.x_scale, se = ne.coordinates.y_scale, ce = oe.invert(ee), le = se.invert(ae), { glyph: ue } = ne, de = [];
      if (ue instanceof PatchView) {
        const [ye, ge] = [ee, ae], [ve, be] = [ce, le], Ce = {
          index: null,
          glyph_view: ue,
          type: ue.model.type,
          x: ce,
          y: le,
          sx: ee,
          sy: ae,
          snap_x: ve,
          snap_y: be,
          snap_sx: ye,
          snap_sy: ge,
          name: H.name
        }, Se = this._render_tooltips(te, Ce);
        de.push([ye, ge, Se]);
      } else if (ue instanceof HAreaStepView || ue instanceof HAreaView || ue instanceof VAreaStepView || ue instanceof VAreaView)
        for (const ye of Z.line_indices) {
          const [ge, ve] = [ce, le], [be, Ce] = [ee, ae], Se = {
            index: ye,
            glyph_view: ue,
            type: ue.model.type,
            x: ce,
            y: le,
            sx: ee,
            sy: ae,
            snap_x: ge,
            snap_y: ve,
            snap_sx: be,
            snap_sy: Ce,
            name: H.name,
            indices: Z.line_indices
          }, we = this._render_tooltips(te, Se);
          de.push([be, Ce, we]);
        }
      else if (ue instanceof LineView$1) {
        const { line_policy: ye } = this.model;
        for (const ge of Z.line_indices) {
          const [[ve, be], [Ce, Se], we] = (() => {
            const { x: Le, y: Qe } = ue;
            switch (ye) {
              case "interp": {
                const [ke, Me] = ue.get_interpolation_hit(ge, V), Re = [oe.compute(ke), se.compute(Me)];
                return [[ke, Me], Re, ge];
              }
              case "prev": {
                const [ke, Me] = _line_hit(ue.sx, ue.sy, ge);
                return [[Le[ge + 1], Qe[ge + 1]], ke, Me];
              }
              case "next": {
                const [ke, Me] = _line_hit(ue.sx, ue.sy, ge + 1);
                return [[Le[ge + 1], Qe[ge + 1]], ke, Me];
              }
              case "nearest": {
                const [ke, Me] = _nearest_line_hit(ge, V, ue.sx, ue.sy);
                return [[Le[Me], Qe[Me]], ke, Me];
              }
              case "none": {
                const ke = ne.coordinates.x_scale, Me = ne.coordinates.y_scale, Re = ke.invert(ee), De = Me.invert(ae);
                return [[Re, De], [ee, ae], ge];
              }
            }
          })(), Ee = {
            index: we,
            glyph_view: ue,
            type: ue.model.type,
            x: ce,
            y: le,
            sx: ee,
            sy: ae,
            snap_x: ve,
            snap_y: be,
            snap_sx: Ce,
            snap_sy: Se,
            name: H.name,
            indices: Z.line_indices
          }, xe = this._render_tooltips(te, Ee);
          de.push([Ce, Se, xe]);
        }
      } else if (ue instanceof ImageBaseView)
        for (const ye of J.image_indices) {
          const [ge, ve] = [ee, ae], [be, Ce] = [ce, le], Se = {
            index: ye.index,
            glyph_view: ue,
            type: ue.model.type,
            x: ce,
            y: le,
            sx: ee,
            sy: ae,
            snap_x: be,
            snap_y: Ce,
            snap_sx: ge,
            snap_sy: ve,
            name: H.name,
            image_index: ye
          }, we = this._render_tooltips(te, Se);
          de.push([ge, ve, we]);
        }
      else
        for (const ye of Z.indices)
          if (ue instanceof MultiLineView && Z.multiline_indices.size != 0) {
            const { line_policy: ge } = this.model;
            for (const ve of Z.multiline_indices.get(ye) ?? []) {
              const [[be, Ce], [Se, we], Ee] = (() => {
                if (ge == "interp") {
                  const [Re, De] = ue.get_interpolation_hit(ye, ve, V), Pe = [oe.compute(Re), se.compute(De)];
                  return [[Re, De], Pe, ve];
                }
                const [ke, Me] = [ue.xs.get(ye), ue.ys.get(ye)];
                if (ge == "prev") {
                  const [Re, De] = _line_hit(ue.sxs.get(ye), ue.sys.get(ye), ve);
                  return [[ke[ve], Me[ve]], Re, De];
                }
                if (ge == "next") {
                  const [Re, De] = _line_hit(ue.sxs.get(ye), ue.sys.get(ye), ve + 1);
                  return [[ke[ve], Me[ve]], Re, De];
                }
                if (ge == "nearest") {
                  const [Re, De] = _nearest_line_hit(ve, V, ue.sxs.get(ye), ue.sys.get(ye));
                  return [[ke[De], Me[De]], Re, De];
                }
                unreachable();
              })(), Le = {
                index: H.view.convert_indices_from_subset([ye])[0],
                glyph_view: ue,
                type: ue.model.type,
                x: ce,
                y: le,
                sx: ee,
                sy: ae,
                snap_x: be,
                snap_y: Ce,
                snap_sx: Se,
                snap_sy: we,
                name: H.name,
                indices: Z.multiline_indices,
                segment_index: Ee
              }, Qe = this._render_tooltips(te, Le);
              de.push([Se, we, Qe]);
            }
          } else {
            const ge = (me = ue.x) == null ? void 0 : me[ye], ve = (pe = ue.y) == null ? void 0 : pe[ye], { point_policy: be, anchor: Ce } = this.model, [Se, we] = function() {
              if (be == "snap_to_data") {
                const Qe = ue.get_anchor_point(Ce, ye, [ee, ae]);
                if (Qe != null)
                  return [Qe.x, Qe.y];
                const ke = ue.get_anchor_point("center", ye, [ee, ae]);
                return ke != null ? [ke.x, ke.y] : [ee, ae];
              }
              return [ee, ae];
            }(), xe = {
              index: H.view.convert_indices_from_subset([ye])[0],
              glyph_view: ue,
              type: ue.model.type,
              x: ce,
              y: le,
              sx: ee,
              sy: ae,
              snap_x: ge,
              snap_y: ve,
              snap_sx: Se,
              snap_sy: we,
              name: H.name,
              indices: Z.indices
            }, Le = this._render_tooltips(te, xe);
            de.push([Se, we, Le]);
          }
      const { bbox: fe } = this.plot_view.frame, he = de.filter(([ye, ge]) => fe.contains(ye, ge));
      if (he.length == 0)
        q.clear();
      else {
        const { content: ye } = q;
        assert$2(ye instanceof Element), empty$1(ye);
        for (const [, , be] of he)
          be != null && ye.appendChild(be);
        const [ge, ve] = he[he.length - 1];
        q.show({ x: ge, y: ve });
      }
    }
    update([H, { geometry: V }]) {
      if (!this.model.active || !(V.type == "point" || V.type == "span") || this.model.muted_policy == "ignore" && H.muted)
        return;
      const q = this.ttmodels.get(H);
      is_undefined(q) || this._update(H, V, q);
    }
    _emit_callback(H) {
      const { callback: V } = this.model;
      if (V != null)
        for (const q of this.computed_renderers) {
          if (!(q instanceof GlyphRenderer))
            continue;
          const G = this.plot_view.renderer_view(q);
          if (G == null)
            continue;
          const { x_scale: J, y_scale: Z } = G.coordinates, te = J.invert(H.sx), ne = Z.invert(H.sy), ee = q.data_source.inspected;
          execute(V, this.model, {
            geometry: { x: te, y: ne, ...H },
            renderer: q,
            index: ee
          });
        }
    }
    _create_template(H) {
      const V = div({ style: { display: "table", borderSpacing: "2px" } });
      for (const [q] of H) {
        const G = div({ style: { display: "table-row" } });
        V.appendChild(G);
        const J = div({ style: { display: "table-cell" }, class: tooltip_row_label }, q.length != 0 ? `${q}: ` : "");
        G.appendChild(J);
        const Z = span$1();
        Z.dataset.value = "";
        const te = span$1({ class: tooltip_color_block }, " ");
        te.dataset.swatch = "", undisplay(te);
        const ne = div({ style: { display: "table-cell" }, class: tooltip_row_value }, Z, te);
        G.appendChild(ne);
      }
      return V;
    }
    _render_template(H, V, q, G) {
      const J = H.cloneNode(!0), Z = is_undefined(G.image_index) ? G.index : G.image_index, te = J.querySelectorAll("[data-value]"), ne = J.querySelectorAll("[data-swatch]"), ee = /\$color(\[.*\])?:(\w*)/, ae = /\$swatch:(\w*)/;
      for (const [[, oe], se] of enumerate(V)) {
        const ce = oe.match(ae), le = oe.match(ee);
        if (ce == null && le == null) {
          const ue = replace_placeholders(oe.replace("$~", "$data_"), q, Z, this.model.formatters, G);
          if (isString$2(ue))
            te[se].textContent = ue;
          else
            for (const de of ue)
              te[se].appendChild(de);
          continue;
        }
        if (ce != null) {
          const [, ue] = ce, de = q.get_column(ue);
          if (de == null)
            te[se].textContent = `${ue} unknown`;
          else {
            const fe = isNumber$3(Z) ? de[Z] : null;
            fe != null && (ne[se].style.backgroundColor = color2css(fe), display(ne[se]));
          }
        }
        if (le != null) {
          const [, ue = "", de] = le, fe = q.get_column(de);
          if (fe == null) {
            te[se].textContent = `${de} unknown`;
            continue;
          }
          const he = ue.indexOf("hex") >= 0, me = ue.indexOf("swatch") >= 0, pe = isNumber$3(Z) ? fe[Z] : null;
          if (pe == null) {
            te[se].textContent = "(null)";
            continue;
          }
          te[se].textContent = he ? color2hex(pe) : color2css(pe), me && (ne[se].style.backgroundColor = color2css(pe), display(ne[se]));
        }
      }
      return J;
    }
    _render_tooltips(H, V) {
      const { tooltips: q } = this.model, G = V.index;
      if (isString$2(q)) {
        const J = replace_placeholders({ html: q }, H, G, this.model.formatters, V);
        return div(J);
      }
      if (isFunction$3(q))
        return q(H, V);
      if (q instanceof Template)
        return this._template_view.update(H, G, V), this._template_view.el;
      if (q != null) {
        const J = this._template_el ?? (this._template_el = this._create_template(q));
        return this._render_template(J, q, H, V);
      }
      return null;
    }
  }
  mt(HoverToolView, "__name__", "HoverToolView");
  const E0 = class E0 extends InspectTool {
    constructor(H) {
      super(H);
      mt(this, "tool_name", "Hover");
      mt(this, "tool_icon", tool_icon_hover);
    }
  };
  mt(E0, "__name__", "HoverTool"), E0.prototype.default_view = HoverToolView, E0.define(({ Any: H, Bool: V, Str: q, List: G, Tuple: J, Dict: Z, Or: te, Ref: ne, Func: ee, Auto: ae, Nullable: oe }) => ({
    tooltips: [oe(te(ne(Template), q, G(J(q, q)), ee())), [
      ["index", "$index"],
      ["data (x, y)", "($x, $y)"],
      ["screen (x, y)", "($sx, $sy)"]
    ]],
    formatters: [Z(te(ne(CustomJSHover), FormatterType)), {}],
    renderers: [te(G(ne(DataRenderer)), ae), "auto"],
    mode: [HoverMode, "mouse"],
    muted_policy: [MutedPolicy, "show"],
    point_policy: [PointPolicy, "snap_to_data"],
    line_policy: [LinePolicy, "nearest"],
    show_arrow: [V, !0],
    anchor: [Anchor$2, "center"],
    attachment: [TooltipAttachment, "horizontal"],
    callback: [oe(
      H
      /*TODO*/
    ), null]
  })), E0.register_alias("hover", () => new E0());
  let HoverTool = E0;
  const AllModels = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    ActionItem,
    ActionTool,
    AdaptiveTicker,
    AjaxDataSource,
    AllIndices,
    AllLabels,
    Angular,
    Annotation: Annotation$1,
    AnnularWedge,
    Annulus,
    Arc,
    Arrow,
    ArrowHead,
    Ascii,
    Axis,
    BBoxTileSource,
    Band: Band$1,
    BaseColorBar,
    BasicTickFormatter,
    BasicTicker,
    Bezier,
    BinnedTicker,
    Block: Block$1,
    BooleanFilter,
    BoxAnnotation,
    BoxEditTool,
    BoxSelectTool,
    BoxZoomTool,
    BuiltinIcon,
    ByCSS,
    ByClass,
    ByID,
    ByXPath,
    CDSView,
    Canvas: Canvas$1,
    CanvasTexture,
    CartesianFrame,
    CategoricalAxis,
    CategoricalColorMapper,
    CategoricalMarkerMapper,
    CategoricalPatternMapper,
    CategoricalScale,
    CategoricalTickFormatter,
    CategoricalTicker,
    CheckableItem,
    Circle,
    ClickButton,
    CloseDialog,
    ColorBar,
    ColorMapper,
    Column: Column$1,
    ColumnDataSource,
    ColumnarDataSource,
    CompositeScale,
    CompositeTicker,
    ContinuousAxis,
    ContinuousColorMapper,
    ContinuousScale,
    ContinuousTicker,
    ContourColorBar,
    ContourRenderer,
    CoordinateMapping,
    CopyTool,
    CrosshairTool,
    CumSum,
    CustomAction,
    CustomJS,
    CustomJSExpr,
    CustomJSFilter,
    CustomJSHover,
    CustomJSTickFormatter,
    CustomJSTransform,
    CustomLabelingPolicy,
    DataRange,
    DataRange1d,
    DataSource,
    DatetimeAxis,
    DatetimeTickFormatter,
    DatetimeTicker,
    DaysTicker,
    Decoration: Decoration$1,
    Dialog,
    DifferenceFilter,
    DividerItem,
    Dodge,
    EdgeCoordinates,
    EdgesAndLinkedNodes,
    EdgesOnly,
    EditTool,
    Ellipse,
    EqHistColorMapper,
    ExamineTool,
    Examiner,
    Expression: Expression$2,
    FactorRange,
    Figure: Figure$1,
    Filter: Filter$1,
    FixedTicker,
    FlexBox,
    FreehandDrawTool,
    FullscreenTool,
    GMap,
    GMapOptions,
    GMapPlot,
    GeoJSONDataSource,
    GestureTool,
    Glyph,
    GlyphRenderer,
    GraphCoordinates,
    GraphHitTestPolicy,
    GraphLayout,
    GraphRenderer,
    Grid,
    GridBox,
    GridPlot,
    GroupBox,
    GroupFilter,
    GuideRenderer,
    HArea,
    HAreaStep,
    HBar,
    HBox,
    HSpan,
    HStrip,
    HTMLLabel,
    HTMLLabelSet,
    HTMLTitle,
    HelpTool,
    HexTile,
    HoverTool,
    Image: Image$1,
    ImageRGBA,
    ImageStack,
    ImageURL,
    ImageURLTexture,
    ImperialLength,
    IndexFilter,
    Indexed,
    InspectTool,
    Interpolator,
    IntersectRenderers,
    IntersectionFilter,
    InversionFilter,
    Jitter,
    Label: Label$3,
    LabelSet,
    LabelingPolicy,
    LassoSelectTool,
    LayoutDOM,
    LayoutProvider,
    Legend,
    LegendItem,
    Line: Line$2,
    LineEditTool,
    LinearAxis,
    LinearColorMapper,
    LinearInterpolationScale,
    LinearInterpolator,
    LinearScale,
    LogAxis,
    LogColorMapper,
    LogScale,
    LogTickFormatter,
    LogTicker,
    MapOptions,
    Marking,
    MathML,
    MathMLGlyph,
    MathText,
    Maximum,
    Menu,
    MercatorAxis,
    MercatorTickFormatter,
    MercatorTicker,
    MercatorTileSource,
    Metric,
    MetricLength,
    Minimum,
    MonthsTicker,
    MultiLine,
    MultiPolygons,
    NoOverlap,
    Node: Node$4,
    NodeCoordinates,
    NodesAndAdjacentNodes,
    NodesAndLinkedEdges,
    NodesOnly,
    NormalHead,
    NumeralTickFormatter,
    OnOffButton,
    OpenDialog,
    OpenHead,
    OpenURL,
    PanTool,
    Pane,
    Panel,
    ParkMillerLCG,
    Patch,
    Patches,
    PlainText,
    Plot,
    PointDrawTool,
    PolarTransform,
    PolyAnnotation,
    PolyDrawTool,
    PolyEditTool,
    PolySelectTool,
    PolyTool,
    PrintfTickFormatter,
    QUADKEYTileSource,
    Quad: Quad$1,
    Quadratic,
    Range: Range$2,
    Range1d,
    RangeTool,
    Ray,
    ReciprocalMetric,
    ReciprocalMetricLength,
    Rect,
    RedoTool,
    Renderer: Renderer$1,
    RendererGroup,
    ResetTool,
    Row: Row$1,
    SVGIcon,
    SaveTool,
    ScalarExpression,
    Scale: Scale$2,
    ScaleBar,
    ScanningColorMapper,
    Scatter,
    ScrollBox,
    Segment,
    SelectTool,
    Selection,
    SelectionPolicy,
    ServerSentDataSource,
    SetValue,
    SingleIntervalTicker,
    Slope,
    Spacer,
    Span: Span$3,
    Spline,
    Stack: Stack$4,
    StackColorMapper,
    StaticLayoutProvider,
    Step: Step$1,
    StepInterpolator,
    SymmetricDifferenceFilter,
    TMSTileSource,
    TabPanel,
    TablerIcon,
    Tabs,
    TapTool,
    TeX,
    TeXGlyph,
    TeeHead,
    Text: Text$1,
    TextAnnotation: TextAnnotation$1,
    Texture,
    TickFormatter,
    Ticker,
    TileRenderer,
    TileSource,
    Title,
    ToggleVisibility,
    Tool,
    ToolButton,
    ToolProxy,
    Toolbar,
    ToolbarPanel,
    Tooltip,
    Transform: Transform$1,
    UIElement,
    UndoTool,
    UnionFilter,
    UnionRenderers,
    VArea,
    VAreaStep,
    VBar,
    VBox,
    VSpan,
    VStrip,
    VeeHead,
    WMTSTileSource,
    WebDataSource,
    Wedge,
    WeightedStackColorMapper,
    WheelPanTool,
    WheelZoomTool,
    Whisker,
    XComponent,
    XY: XY$1,
    YComponent,
    YearsTicker,
    ZoomInTool,
    ZoomOutTool
  }, Symbol.toStringTag, { value: "Module" }));
  class ValueRefView extends PlaceholderView {
    update(_, H, V) {
      const q = _get_column_value(this.model.field, _, H), G = q == null ? "???" : `${q}`;
      this.el.textContent = G;
    }
  }
  mt(ValueRefView, "__name__", "ValueRefView");
  const Fm = class Fm extends Placeholder {
    constructor(_) {
      super(_);
    }
  };
  mt(Fm, "__name__", "ValueRef"), Fm.prototype.default_view = ValueRefView, Fm.define(({ Str: _ }) => ({
    field: [_]
  }));
  let ValueRef = Fm;
  class ColorRefView extends ValueRefView {
    constructor() {
      super(...arguments);
      mt(this, "value_el");
      mt(this, "swatch_el");
    }
    render() {
      super.render(), this.value_el = span$1(), this.swatch_el = span$1({ class: tooltip_color_block }, " "), this.el.appendChild(this.value_el), this.el.appendChild(this.swatch_el);
    }
    update(H, V, q) {
      const G = _get_column_value(this.model.field, H, V), J = G == null ? "???" : `${G}`;
      this.el.textContent = J;
    }
  }
  mt(ColorRefView, "__name__", "ColorRefView");
  const Bm = class Bm extends ValueRef {
    constructor(_) {
      super(_);
    }
  };
  mt(Bm, "__name__", "ColorRef"), Bm.prototype.default_view = ColorRefView, Bm.define(({ Bool: _ }) => ({
    hex: [_, !0],
    swatch: [_, !0]
  }));
  let ColorRef = Bm;
  class IndexView extends PlaceholderView {
    update(_, H, V) {
      this.el.textContent = H == null ? "(null)" : H.toString();
    }
  }
  mt(IndexView, "__name__", "IndexView");
  var B0;
  let Index$1 = (B0 = class extends Placeholder {
    constructor(_) {
      super(_);
    }
  }, mt(B0, "__name__", "Index"), B0.prototype.default_view = IndexView, B0);
  class ToggleGroupView extends ActionView {
    update(_, H, V) {
      for (const [q, G] of enumerate(this.model.groups))
        q.visible = H == G;
    }
  }
  mt(ToggleGroupView, "__name__", "ToggleGroupView");
  const Hm = class Hm extends Action {
    constructor(_) {
      super(_);
    }
  };
  mt(Hm, "__name__", "ToggleGroup"), Hm.prototype.default_view = ToggleGroupView, Hm.define(({ List: _, Ref: H }) => ({
    groups: [_(H(RendererGroup)), []]
  }));
  let ToggleGroup = Hm;
  class ValueOfView extends DOMNodeView {
    connect_signals() {
      super.connect_signals();
      const { obj: _, attr: H } = this.model;
      H in _.properties && this.on_change(_.properties[H], () => this.render());
    }
    render() {
      empty$1(this.el), this.el.style.display = "contents";
      const _ = (() => {
        const { obj: H, attr: V } = this.model;
        if (V in H.properties) {
          const q = H.properties[V].get_value();
          return to_string(q);
        } else
          return `<not found: ${H.type}.${V}>`;
      })();
      this.el.textContent = _;
    }
  }
  mt(ValueOfView, "__name__", "ValueOfView");
  const zm = class zm extends DOMNode {
    constructor(_) {
      super(_);
    }
  };
  mt(zm, "__name__", "ValueOf"), zm.prototype.default_view = ValueOfView, zm.define(({ Str: _, Ref: H }) => ({
    obj: [H(HasProps)],
    attr: [_]
  }));
  let ValueOf = zm;
  const DOMModels = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    Action,
    ColorRef,
    DOMElement,
    DOMNode,
    Div: Div$1,
    GlobalImportedStyleSheet,
    GlobalInlineStyleSheet,
    HTML,
    ImportedStyleSheet,
    Index: Index$1,
    InlineStyleSheet,
    Placeholder,
    Span: Span$2,
    Styles: Styles$1,
    Table: Table$1,
    TableRow,
    Template,
    Text: Text$2,
    ToggleGroup,
    ValueOf,
    ValueRef
  }, Symbol.toStringTag, { value: "Module" }));
  register_models(AllModels);
  register_models(DOMModels);
  var regl$1 = { exports: {} }, regl = regl$1.exports, hasRequiredRegl;
  function requireRegl() {
    return hasRequiredRegl || (hasRequiredRegl = 1, function(O, _) {
      (function(H, V) {
        O.exports = V();
      })(regl, function() {
        var H = function(pr) {
          return pr instanceof Uint8Array || pr instanceof Uint16Array || pr instanceof Uint32Array || pr instanceof Int8Array || pr instanceof Int16Array || pr instanceof Int32Array || pr instanceof Float32Array || pr instanceof Float64Array || pr instanceof Uint8ClampedArray;
        }, V = function(pr, oi) {
          for (var ar = Object.keys(oi), Sa = 0; Sa < ar.length; ++Sa)
            pr[ar[Sa]] = oi[ar[Sa]];
          return pr;
        }, q = `
`;
        function G(pr) {
          return typeof atob < "u" ? atob(pr) : "base64:" + pr;
        }
        function J(pr) {
          var oi = new Error("(regl) " + pr);
          throw console.error(oi), oi;
        }
        function Z(pr, oi) {
          pr || J(oi);
        }
        function te(pr) {
          return pr ? ": " + pr : "";
        }
        function ne(pr, oi, ar) {
          pr in oi || J("unknown parameter (" + pr + ")" + te(ar) + ". possible values: " + Object.keys(oi).join());
        }
        function ee(pr, oi) {
          H(pr) || J(
            "invalid parameter type" + te(oi) + ". must be a typed array"
          );
        }
        function ae(pr, oi) {
          switch (oi) {
            case "number":
              return typeof pr == "number";
            case "object":
              return typeof pr == "object";
            case "string":
              return typeof pr == "string";
            case "boolean":
              return typeof pr == "boolean";
            case "function":
              return typeof pr == "function";
            case "undefined":
              return typeof pr > "u";
            case "symbol":
              return typeof pr == "symbol";
          }
        }
        function oe(pr, oi, ar) {
          ae(pr, oi) || J(
            "invalid parameter type" + te(ar) + ". expected " + oi + ", got " + typeof pr
          );
        }
        function se(pr, oi) {
          pr >= 0 && (pr | 0) === pr || J("invalid parameter type, (" + pr + ")" + te(oi) + ". must be a nonnegative integer");
        }
        function ce(pr, oi, ar) {
          oi.indexOf(pr) < 0 && J("invalid value" + te(ar) + ". must be one of: " + oi);
        }
        var le = [
          "gl",
          "canvas",
          "container",
          "attributes",
          "pixelRatio",
          "extensions",
          "optionalExtensions",
          "profile",
          "onDone"
        ];
        function ue(pr) {
          Object.keys(pr).forEach(function(oi) {
            le.indexOf(oi) < 0 && J('invalid regl constructor argument "' + oi + '". must be one of ' + le);
          });
        }
        function de(pr, oi) {
          for (pr = pr + ""; pr.length < oi; )
            pr = " " + pr;
          return pr;
        }
        function fe() {
          this.name = "unknown", this.lines = [], this.index = {}, this.hasErrors = !1;
        }
        function he(pr, oi) {
          this.number = pr, this.line = oi, this.errors = [];
        }
        function me(pr, oi, ar) {
          this.file = pr, this.line = oi, this.message = ar;
        }
        function pe() {
          var pr = new Error(), oi = (pr.stack || pr).toString(), ar = /compileProcedure.*\n\s*at.*\((.*)\)/.exec(oi);
          if (ar)
            return ar[1];
          var Sa = /compileProcedure.*\n\s*at\s+(.*)(\n|$)/.exec(oi);
          return Sa ? Sa[1] : "unknown";
        }
        function ye() {
          var pr = new Error(), oi = (pr.stack || pr).toString(), ar = /at REGLCommand.*\n\s+at.*\((.*)\)/.exec(oi);
          if (ar)
            return ar[1];
          var Sa = /at REGLCommand.*\n\s+at\s+(.*)\n/.exec(oi);
          return Sa ? Sa[1] : "unknown";
        }
        function ge(pr, oi) {
          var ar = pr.split(`
`), Sa = 1, Qn = 0, za = {
            unknown: new fe(),
            0: new fe()
          };
          za.unknown.name = za[0].name = oi || pe(), za.unknown.lines.push(new he(0, ""));
          for (var En = 0; En < ar.length; ++En) {
            var Qo = ar[En], lo = /^\s*#\s*(\w+)\s+(.+)\s*$/.exec(Qo);
            if (lo)
              switch (lo[1]) {
                case "line":
                  var Eo = /(\d+)(\s+\d+)?/.exec(lo[2]);
                  Eo && (Sa = Eo[1] | 0, Eo[2] && (Qn = Eo[2] | 0, Qn in za || (za[Qn] = new fe())));
                  break;
                case "define":
                  var bo = /SHADER_NAME(_B64)?\s+(.*)$/.exec(lo[2]);
                  bo && (za[Qn].name = bo[1] ? G(bo[2]) : bo[2]);
                  break;
              }
            za[Qn].lines.push(new he(Sa++, Qo));
          }
          return Object.keys(za).forEach(function(Ho) {
            var Co = za[Ho];
            Co.lines.forEach(function(Ao) {
              Co.index[Ao.number] = Ao;
            });
          }), za;
        }
        function ve(pr) {
          var oi = [];
          return pr.split(`
`).forEach(function(ar) {
            if (!(ar.length < 5)) {
              var Sa = /^ERROR:\s+(\d+):(\d+):\s*(.*)$/.exec(ar);
              Sa ? oi.push(new me(
                Sa[1] | 0,
                Sa[2] | 0,
                Sa[3].trim()
              )) : ar.length > 0 && oi.push(new me("unknown", 0, ar));
            }
          }), oi;
        }
        function be(pr, oi) {
          oi.forEach(function(ar) {
            var Sa = pr[ar.file];
            if (Sa) {
              var Qn = Sa.index[ar.line];
              if (Qn) {
                Qn.errors.push(ar), Sa.hasErrors = !0;
                return;
              }
            }
            pr.unknown.hasErrors = !0, pr.unknown.lines[0].errors.push(ar);
          });
        }
        function Ce(pr, oi, ar, Sa, Qn) {
          if (!pr.getShaderParameter(oi, pr.COMPILE_STATUS)) {
            var za = pr.getShaderInfoLog(oi), En = Sa === pr.FRAGMENT_SHADER ? "fragment" : "vertex";
            ke(ar, "string", En + " shader source must be a string", Qn);
            var Qo = ge(ar, Qn), lo = ve(za);
            be(Qo, lo), Object.keys(Qo).forEach(function(Eo) {
              var bo = Qo[Eo];
              if (!bo.hasErrors)
                return;
              var Ho = [""], Co = [""];
              function Ao(Ro, Qa) {
                Ho.push(Ro), Co.push(Qa || "");
              }
              Ao("file number " + Eo + ": " + bo.name + `
`, "color:red;text-decoration:underline;font-weight:bold"), bo.lines.forEach(function(Ro) {
                if (Ro.errors.length > 0) {
                  Ao(de(Ro.number, 4) + "|  ", "background-color:yellow; font-weight:bold"), Ao(Ro.line + q, "color:red; background-color:yellow; font-weight:bold");
                  var Qa = 0;
                  Ro.errors.forEach(function(Ka) {
                    var qo = Ka.message, Ss = /^\s*'(.*)'\s*:\s*(.*)$/.exec(qo);
                    if (Ss) {
                      var ho = Ss[1];
                      switch (qo = Ss[2], ho) {
                        case "assign":
                          ho = "=";
                          break;
                      }
                      Qa = Math.max(Ro.line.indexOf(ho, Qa), 0);
                    } else
                      Qa = 0;
                    Ao(de("| ", 6)), Ao(de("^^^", Qa + 3) + q, "font-weight:bold"), Ao(de("| ", 6)), Ao(qo + q, "font-weight:bold");
                  }), Ao(de("| ", 6) + q);
                } else
                  Ao(de(Ro.number, 4) + "|  "), Ao(Ro.line + q, "color:red");
              }), typeof document < "u" && !window.chrome ? (Co[0] = Ho.join("%c"), console.log.apply(console, Co)) : console.log(Ho.join(""));
            }), Z.raise("Error compiling " + En + " shader, " + Qo[0].name);
          }
        }
        function Se(pr, oi, ar, Sa, Qn) {
          if (!pr.getProgramParameter(oi, pr.LINK_STATUS)) {
            var za = pr.getProgramInfoLog(oi), En = ge(ar, Qn), Qo = ge(Sa, Qn), lo = 'Error linking program with vertex shader, "' + Qo[0].name + '", and fragment shader "' + En[0].name + '"';
            typeof document < "u" ? console.log(
              "%c" + lo + q + "%c" + za,
              "color:red;text-decoration:underline;font-weight:bold",
              "color:red"
            ) : console.log(lo + q + za), Z.raise(lo);
          }
        }
        function we(pr) {
          pr._commandRef = pe();
        }
        function Ee(pr, oi, ar, Sa) {
          we(pr);
          function Qn(lo) {
            return lo ? Sa.id(lo) : 0;
          }
          pr._fragId = Qn(pr.static.frag), pr._vertId = Qn(pr.static.vert);
          function za(lo, Eo) {
            Object.keys(Eo).forEach(function(bo) {
              lo[Sa.id(bo)] = !0;
            });
          }
          var En = pr._uniformSet = {};
          za(En, oi.static), za(En, oi.dynamic);
          var Qo = pr._attributeSet = {};
          za(Qo, ar.static), za(Qo, ar.dynamic), pr._hasCount = "count" in pr.static || "count" in pr.dynamic || "elements" in pr.static || "elements" in pr.dynamic;
        }
        function xe(pr, oi) {
          var ar = ye();
          J(pr + " in command " + (oi || pe()) + (ar === "unknown" ? "" : " called from " + ar));
        }
        function Le(pr, oi, ar) {
          pr || xe(oi, ar || pe());
        }
        function Qe(pr, oi, ar, Sa) {
          pr in oi || xe(
            "unknown parameter (" + pr + ")" + te(ar) + ". possible values: " + Object.keys(oi).join(),
            Sa || pe()
          );
        }
        function ke(pr, oi, ar, Sa) {
          ae(pr, oi) || xe(
            "invalid parameter type" + te(ar) + ". expected " + oi + ", got " + typeof pr,
            Sa || pe()
          );
        }
        function Me(pr) {
          pr();
        }
        function Re(pr, oi, ar) {
          pr.texture ? ce(
            pr.texture._texture.internalformat,
            oi,
            "unsupported texture format for attachment"
          ) : ce(
            pr.renderbuffer._renderbuffer.format,
            ar,
            "unsupported renderbuffer format for attachment"
          );
        }
        var De = 33071, Pe = 9728, $e = 9984, qe = 9985, Xe = 9986, Ve = 9987, Ze = 5120, Ge = 5121, ze = 5122, We = 5123, at = 5124, ct = 5125, et = 5126, dt = 32819, Je = 32820, it = 33635, st = 34042, gt = 36193, bt = {};
        bt[Ze] = bt[Ge] = 1, bt[ze] = bt[We] = bt[gt] = bt[it] = bt[dt] = bt[Je] = 2, bt[at] = bt[ct] = bt[et] = bt[st] = 4;
        function yt(pr, oi) {
          return pr === Je || pr === dt || pr === it ? 2 : pr === st ? 4 : bt[pr] * oi;
        }
        function St(pr) {
          return !(pr & pr - 1) && !!pr;
        }
        function Lt(pr, oi, ar) {
          var Sa, Qn = oi.width, za = oi.height, En = oi.channels;
          Z(
            Qn > 0 && Qn <= ar.maxTextureSize && za > 0 && za <= ar.maxTextureSize,
            "invalid texture shape"
          ), (pr.wrapS !== De || pr.wrapT !== De) && Z(
            St(Qn) && St(za),
            "incompatible wrap mode for texture, both width and height must be power of 2"
          ), oi.mipmask === 1 ? Qn !== 1 && za !== 1 && Z(
            pr.minFilter !== $e && pr.minFilter !== Xe && pr.minFilter !== qe && pr.minFilter !== Ve,
            "min filter requires mipmap"
          ) : (Z(
            St(Qn) && St(za),
            "texture must be a square power of 2 to support mipmapping"
          ), Z(
            oi.mipmask === (Qn << 1) - 1,
            "missing or incomplete mipmap data"
          )), oi.type === et && (ar.extensions.indexOf("oes_texture_float_linear") < 0 && Z(
            pr.minFilter === Pe && pr.magFilter === Pe,
            "filter not supported, must enable oes_texture_float_linear"
          ), Z(
            !pr.genMipmaps,
            "mipmap generation not supported with float textures"
          ));
          var Qo = oi.images;
          for (Sa = 0; Sa < 16; ++Sa)
            if (Qo[Sa]) {
              var lo = Qn >> Sa, Eo = za >> Sa;
              Z(oi.mipmask & 1 << Sa, "missing mipmap data");
              var bo = Qo[Sa];
              if (Z(
                bo.width === lo && bo.height === Eo,
                "invalid shape for mip images"
              ), Z(
                bo.format === oi.format && bo.internalformat === oi.internalformat && bo.type === oi.type,
                "incompatible type for mip image"
              ), !bo.compressed) if (bo.data) {
                var Ho = Math.ceil(yt(bo.type, En) * lo / bo.unpackAlignment) * bo.unpackAlignment;
                Z(
                  bo.data.byteLength === Ho * Eo,
                  "invalid data for image, buffer size is inconsistent with image format"
                );
              } else bo.element || bo.copy;
            } else pr.genMipmaps || Z((oi.mipmask & 1 << Sa) === 0, "extra mipmap data");
          oi.compressed && Z(
            !pr.genMipmaps,
            "mipmap generation for compressed images not supported"
          );
        }
        function Rt(pr, oi, ar, Sa) {
          var Qn = pr.width, za = pr.height, En = pr.channels;
          Z(
            Qn > 0 && Qn <= Sa.maxTextureSize && za > 0 && za <= Sa.maxTextureSize,
            "invalid texture shape"
          ), Z(
            Qn === za,
            "cube map must be square"
          ), Z(
            oi.wrapS === De && oi.wrapT === De,
            "wrap mode not supported by cube map"
          );
          for (var Qo = 0; Qo < ar.length; ++Qo) {
            var lo = ar[Qo];
            Z(
              lo.width === Qn && lo.height === za,
              "inconsistent cube map face shape"
            ), oi.genMipmaps && (Z(
              !lo.compressed,
              "can not generate mipmap for compressed textures"
            ), Z(
              lo.mipmask === 1,
              "can not specify mipmaps and generate mipmaps"
            ));
            for (var Eo = lo.images, bo = 0; bo < 16; ++bo) {
              var Ho = Eo[bo];
              if (Ho) {
                var Co = Qn >> bo, Ao = za >> bo;
                Z(lo.mipmask & 1 << bo, "missing mipmap data"), Z(
                  Ho.width === Co && Ho.height === Ao,
                  "invalid shape for mip images"
                ), Z(
                  Ho.format === pr.format && Ho.internalformat === pr.internalformat && Ho.type === pr.type,
                  "incompatible type for mip image"
                ), Ho.compressed || (Ho.data ? Z(
                  Ho.data.byteLength === Co * Ao * Math.max(yt(Ho.type, En), Ho.unpackAlignment),
                  "invalid data for image, buffer size is inconsistent with image format"
                ) : Ho.element || Ho.copy);
              }
            }
          }
        }
        var vt = V(Z, {
          optional: Me,
          raise: J,
          commandRaise: xe,
          command: Le,
          parameter: ne,
          commandParameter: Qe,
          constructor: ue,
          type: oe,
          commandType: ke,
          isTypedArray: ee,
          nni: se,
          oneOf: ce,
          shaderError: Ce,
          linkError: Se,
          callSite: ye,
          saveCommandRef: we,
          saveDrawInfo: Ee,
          framebufferFormat: Re,
          guessCommand: pe,
          texture2D: Lt,
          textureCube: Rt
        }), $t = 0, Pt = 0, Xt = 5, Nt = 6;
        function Gt(pr, oi) {
          this.id = $t++, this.type = pr, this.data = oi;
        }
        function Ft(pr) {
          return pr.replace(/\\/g, "\\\\").replace(/"/g, '\\"');
        }
        function Tt(pr) {
          if (pr.length === 0)
            return [];
          var oi = pr.charAt(0), ar = pr.charAt(pr.length - 1);
          if (pr.length > 1 && oi === ar && (oi === '"' || oi === "'"))
            return ['"' + Ft(pr.substr(1, pr.length - 2)) + '"'];
          var Sa = /\[(false|true|null|\d+|'[^']*'|"[^"]*")\]/.exec(pr);
          if (Sa)
            return Tt(pr.substr(0, Sa.index)).concat(Tt(Sa[1])).concat(Tt(pr.substr(Sa.index + Sa[0].length)));
          var Qn = pr.split(".");
          if (Qn.length === 1)
            return ['"' + Ft(pr) + '"'];
          for (var za = [], En = 0; En < Qn.length; ++En)
            za = za.concat(Tt(Qn[En]));
          return za;
        }
        function It(pr) {
          return "[" + Tt(pr).join("][") + "]";
        }
        function Ct(pr, oi) {
          return new Gt(pr, It(oi + ""));
        }
        function Kt(pr) {
          return typeof pr == "function" && !pr._reglType || pr instanceof Gt;
        }
        function Vt(pr, oi) {
          if (typeof pr == "function")
            return new Gt(Pt, pr);
          if (typeof pr == "number" || typeof pr == "boolean")
            return new Gt(Xt, pr);
          if (Array.isArray(pr))
            return new Gt(Nt, pr.map(function(ar, Sa) {
              return Vt(ar, oi + "[" + Sa + "]");
            }));
          if (pr instanceof Gt)
            return pr;
          vt(!1, "invalid option type in uniform " + oi);
        }
        var Yt = {
          DynamicVariable: Gt,
          define: Ct,
          isDynamic: Kt,
          unbox: Vt,
          accessor: It
        }, sn = {
          next: typeof requestAnimationFrame == "function" ? function(pr) {
            return requestAnimationFrame(pr);
          } : function(pr) {
            return setTimeout(pr, 16);
          },
          cancel: typeof cancelAnimationFrame == "function" ? function(pr) {
            return cancelAnimationFrame(pr);
          } : clearTimeout
        }, hn = typeof performance < "u" && performance.now ? function() {
          return performance.now();
        } : function() {
          return +/* @__PURE__ */ new Date();
        };
        function Qt() {
          var pr = { "": 0 }, oi = [""];
          return {
            id: function(ar) {
              var Sa = pr[ar];
              return Sa || (Sa = pr[ar] = oi.length, oi.push(ar), Sa);
            },
            str: function(ar) {
              return oi[ar];
            }
          };
        }
        function Ut(pr, oi, ar) {
          var Sa = document.createElement("canvas");
          V(Sa.style, {
            border: 0,
            margin: 0,
            padding: 0,
            top: 0,
            left: 0,
            width: "100%",
            height: "100%"
          }), pr.appendChild(Sa), pr === document.body && (Sa.style.position = "absolute", V(pr.style, {
            margin: 0,
            padding: 0
          }));
          function Qn() {
            var Qo = window.innerWidth, lo = window.innerHeight;
            if (pr !== document.body) {
              var Eo = Sa.getBoundingClientRect();
              Qo = Eo.right - Eo.left, lo = Eo.bottom - Eo.top;
            }
            Sa.width = ar * Qo, Sa.height = ar * lo;
          }
          var za;
          pr !== document.body && typeof ResizeObserver == "function" ? (za = new ResizeObserver(function() {
            setTimeout(Qn);
          }), za.observe(pr)) : window.addEventListener("resize", Qn, !1);
          function En() {
            za ? za.disconnect() : window.removeEventListener("resize", Qn), pr.removeChild(Sa);
          }
          return Qn(), {
            canvas: Sa,
            onDestroy: En
          };
        }
        function Wt(pr, oi) {
          function ar(Sa) {
            try {
              return pr.getContext(Sa, oi);
            } catch {
              return null;
            }
          }
          return ar("webgl") || ar("experimental-webgl") || ar("webgl-experimental");
        }
        function rn(pr) {
          return typeof pr.nodeName == "string" && typeof pr.appendChild == "function" && typeof pr.getBoundingClientRect == "function";
        }
        function vn(pr) {
          return typeof pr.drawArrays == "function" || typeof pr.drawElements == "function";
        }
        function dn(pr) {
          return typeof pr == "string" ? pr.split() : (vt(Array.isArray(pr), "invalid extension array"), pr);
        }
        function Ht(pr) {
          return typeof pr == "string" ? (vt(typeof document < "u", "not supported outside of DOM"), document.querySelector(pr)) : pr;
        }
        function jt(pr) {
          var oi = pr || {}, ar, Sa, Qn, za, En = {}, Qo = [], lo = [], Eo = typeof window > "u" ? 1 : window.devicePixelRatio, bo = !1, Ho = function(Ro) {
            Ro && vt.raise(Ro);
          }, Co = function() {
          };
          if (typeof oi == "string" ? (vt(
            typeof document < "u",
            "selector queries only supported in DOM enviroments"
          ), ar = document.querySelector(oi), vt(ar, "invalid query string for element")) : typeof oi == "object" ? rn(oi) ? ar = oi : vn(oi) ? (za = oi, Qn = za.canvas) : (vt.constructor(oi), "gl" in oi ? za = oi.gl : "canvas" in oi ? Qn = Ht(oi.canvas) : "container" in oi && (Sa = Ht(oi.container)), "attributes" in oi && (En = oi.attributes, vt.type(En, "object", "invalid context attributes")), "extensions" in oi && (Qo = dn(oi.extensions)), "optionalExtensions" in oi && (lo = dn(oi.optionalExtensions)), "onDone" in oi && (vt.type(
            oi.onDone,
            "function",
            "invalid or missing onDone callback"
          ), Ho = oi.onDone), "profile" in oi && (bo = !!oi.profile), "pixelRatio" in oi && (Eo = +oi.pixelRatio, vt(Eo > 0, "invalid pixel ratio"))) : vt.raise("invalid arguments to regl"), ar && (ar.nodeName.toLowerCase() === "canvas" ? Qn = ar : Sa = ar), !za) {
            if (!Qn) {
              vt(
                typeof document < "u",
                "must manually specify webgl context outside of DOM environments"
              );
              var Ao = Ut(Sa || document.body, Ho, Eo);
              if (!Ao)
                return null;
              Qn = Ao.canvas, Co = Ao.onDestroy;
            }
            En.premultipliedAlpha === void 0 && (En.premultipliedAlpha = !0), za = Wt(Qn, En);
          }
          return za ? {
            gl: za,
            canvas: Qn,
            container: Sa,
            extensions: Qo,
            optionalExtensions: lo,
            pixelRatio: Eo,
            profile: bo,
            onDone: Ho,
            onDestroy: Co
          } : (Co(), Ho("webgl not supported, try upgrading your browser or graphics drivers http://get.webgl.org"), null);
        }
        function cn(pr, oi) {
          var ar = {};
          function Sa(En) {
            vt.type(En, "string", "extension name must be string");
            var Qo = En.toLowerCase(), lo;
            try {
              lo = ar[Qo] = pr.getExtension(Qo);
            } catch {
            }
            return !!lo;
          }
          for (var Qn = 0; Qn < oi.extensions.length; ++Qn) {
            var za = oi.extensions[Qn];
            if (!Sa(za))
              return oi.onDestroy(), oi.onDone('"' + za + '" extension is not supported by the current WebGL context, try upgrading your system or a different browser'), null;
          }
          return oi.optionalExtensions.forEach(Sa), {
            extensions: ar,
            restore: function() {
              Object.keys(ar).forEach(function(En) {
                if (ar[En] && !Sa(En))
                  throw new Error("(regl): error restoring extension " + En);
              });
            }
          };
        }
        function wn(pr, oi) {
          for (var ar = Array(pr), Sa = 0; Sa < pr; ++Sa)
            ar[Sa] = oi(Sa);
          return ar;
        }
        var Rn = 5120, Pn = 5121, Ie = 5122, rt = 5123, ft = 5124, pt = 5125, wt = 5126;
        function Bt(pr) {
          for (var oi = 16; oi <= 1 << 28; oi *= 16)
            if (pr <= oi)
              return oi;
          return 0;
        }
        function At(pr) {
          var oi, ar;
          return oi = (pr > 65535) << 4, pr >>>= oi, ar = (pr > 255) << 3, pr >>>= ar, oi |= ar, ar = (pr > 15) << 2, pr >>>= ar, oi |= ar, ar = (pr > 3) << 1, pr >>>= ar, oi |= ar, oi | pr >> 1;
        }
        function Dt() {
          var pr = wn(8, function() {
            return [];
          });
          function oi(za) {
            var En = Bt(za), Qo = pr[At(En) >> 2];
            return Qo.length > 0 ? Qo.pop() : new ArrayBuffer(En);
          }
          function ar(za) {
            pr[At(za.byteLength) >> 2].push(za);
          }
          function Sa(za, En) {
            var Qo = null;
            switch (za) {
              case Rn:
                Qo = new Int8Array(oi(En), 0, En);
                break;
              case Pn:
                Qo = new Uint8Array(oi(En), 0, En);
                break;
              case Ie:
                Qo = new Int16Array(oi(2 * En), 0, En);
                break;
              case rt:
                Qo = new Uint16Array(oi(2 * En), 0, En);
                break;
              case ft:
                Qo = new Int32Array(oi(4 * En), 0, En);
                break;
              case pt:
                Qo = new Uint32Array(oi(4 * En), 0, En);
                break;
              case wt:
                Qo = new Float32Array(oi(4 * En), 0, En);
                break;
              default:
                return null;
            }
            return Qo.length !== En ? Qo.subarray(0, En) : Qo;
          }
          function Qn(za) {
            ar(za.buffer);
          }
          return {
            alloc: oi,
            free: ar,
            allocType: Sa,
            freeType: Qn
          };
        }
        var Ot = Dt();
        Ot.zero = Dt();
        var un = 3408, yn = 3410, _n = 3411, gn = 3412, bn = 3413, an = 3414, Zt = 3415, Mn = 33901, Ln = 33902, In = 3379, kn = 3386, er = 34921, jn = 36347, ur = 36348, Kn = 35661, Jn = 35660, Lr = 34930, mr = 36349, Ar = 34076, Yr = 34024, jr = 7936, Mr = 7937, Wr = 7938, Bi = 35724, gi = 34047, Zr = 36063, fi = 34852, ri = 3553, bi = 34067, Zi = 34069, $i = 33984, Br = 6408, Pr = 5126, ei = 5121, Ci = 36160, oa = 36053, ka = 36064, aa = 16384, hi = function(pr, oi) {
          var ar = 1;
          oi.ext_texture_filter_anisotropic && (ar = pr.getParameter(gi));
          var Sa = 1, Qn = 1;
          oi.webgl_draw_buffers && (Sa = pr.getParameter(fi), Qn = pr.getParameter(Zr));
          var za = !!oi.oes_texture_float;
          if (za) {
            var En = pr.createTexture();
            pr.bindTexture(ri, En), pr.texImage2D(ri, 0, Br, 1, 1, 0, Br, Pr, null);
            var Qo = pr.createFramebuffer();
            if (pr.bindFramebuffer(Ci, Qo), pr.framebufferTexture2D(Ci, ka, ri, En, 0), pr.bindTexture(ri, null), pr.checkFramebufferStatus(Ci) !== oa) za = !1;
            else {
              pr.viewport(0, 0, 1, 1), pr.clearColor(1, 0, 0, 1), pr.clear(aa);
              var lo = Ot.allocType(Pr, 4);
              pr.readPixels(0, 0, 1, 1, Br, Pr, lo), pr.getError() ? za = !1 : (pr.deleteFramebuffer(Qo), pr.deleteTexture(En), za = lo[0] === 1), Ot.freeType(lo);
            }
          }
          var Eo = typeof navigator < "u" && (/MSIE/.test(navigator.userAgent) || /Trident\//.test(navigator.appVersion) || /Edge/.test(navigator.userAgent)), bo = !0;
          if (!Eo) {
            var Ho = pr.createTexture(), Co = Ot.allocType(ei, 36);
            pr.activeTexture($i), pr.bindTexture(bi, Ho), pr.texImage2D(Zi, 0, Br, 3, 3, 0, Br, ei, Co), Ot.freeType(Co), pr.bindTexture(bi, null), pr.deleteTexture(Ho), bo = !pr.getError();
          }
          return {
            // drawing buffer bit depth
            colorBits: [
              pr.getParameter(yn),
              pr.getParameter(_n),
              pr.getParameter(gn),
              pr.getParameter(bn)
            ],
            depthBits: pr.getParameter(an),
            stencilBits: pr.getParameter(Zt),
            subpixelBits: pr.getParameter(un),
            // supported extensions
            extensions: Object.keys(oi).filter(function(Ao) {
              return !!oi[Ao];
            }),
            // max aniso samples
            maxAnisotropic: ar,
            // max draw buffers
            maxDrawbuffers: Sa,
            maxColorAttachments: Qn,
            // point and line size ranges
            pointSizeDims: pr.getParameter(Mn),
            lineWidthDims: pr.getParameter(Ln),
            maxViewportDims: pr.getParameter(kn),
            maxCombinedTextureUnits: pr.getParameter(Kn),
            maxCubeMapSize: pr.getParameter(Ar),
            maxRenderbufferSize: pr.getParameter(Yr),
            maxTextureUnits: pr.getParameter(Lr),
            maxTextureSize: pr.getParameter(In),
            maxAttributes: pr.getParameter(er),
            maxVertexUniforms: pr.getParameter(jn),
            maxVertexTextureUnits: pr.getParameter(Jn),
            maxVaryingVectors: pr.getParameter(ur),
            maxFragmentUniforms: pr.getParameter(mr),
            // vendor info
            glsl: pr.getParameter(Bi),
            renderer: pr.getParameter(Mr),
            vendor: pr.getParameter(jr),
            version: pr.getParameter(Wr),
            // quirks
            readFloat: za,
            npotTextureCube: bo
          };
        };
        function Gr(pr) {
          return !!pr && typeof pr == "object" && Array.isArray(pr.shape) && Array.isArray(pr.stride) && typeof pr.offset == "number" && pr.shape.length === pr.stride.length && (Array.isArray(pr.data) || H(pr.data));
        }
        var Ca = function(pr) {
          return Object.keys(pr).map(function(oi) {
            return pr[oi];
          });
        }, Sr = {
          shape: la,
          flatten: Da
        };
        function Dr(pr, oi, ar) {
          for (var Sa = 0; Sa < oi; ++Sa)
            ar[Sa] = pr[Sa];
        }
        function si(pr, oi, ar, Sa) {
          for (var Qn = 0, za = 0; za < oi; ++za)
            for (var En = pr[za], Qo = 0; Qo < ar; ++Qo)
              Sa[Qn++] = En[Qo];
        }
        function zr(pr, oi, ar, Sa, Qn, za) {
          for (var En = za, Qo = 0; Qo < oi; ++Qo)
            for (var lo = pr[Qo], Eo = 0; Eo < ar; ++Eo)
              for (var bo = lo[Eo], Ho = 0; Ho < Sa; ++Ho)
                Qn[En++] = bo[Ho];
        }
        function di(pr, oi, ar, Sa, Qn) {
          for (var za = 1, En = ar + 1; En < oi.length; ++En)
            za *= oi[En];
          var Qo = oi[ar];
          if (oi.length - ar === 4) {
            var lo = oi[ar + 1], Eo = oi[ar + 2], bo = oi[ar + 3];
            for (En = 0; En < Qo; ++En)
              zr(pr[En], lo, Eo, bo, Sa, Qn), Qn += za;
          } else
            for (En = 0; En < Qo; ++En)
              di(pr[En], oi, ar + 1, Sa, Qn), Qn += za;
        }
        function Da(pr, oi, ar, Sa) {
          var Qn = 1;
          if (oi.length)
            for (var za = 0; za < oi.length; ++za)
              Qn *= oi[za];
          else
            Qn = 0;
          var En = Sa || Ot.allocType(ar, Qn);
          switch (oi.length) {
            case 0:
              break;
            case 1:
              Dr(pr, oi[0], En);
              break;
            case 2:
              si(pr, oi[0], oi[1], En);
              break;
            case 3:
              zr(pr, oi[0], oi[1], oi[2], En, 0);
              break;
            default:
              di(pr, oi, 0, En, 0);
          }
          return En;
        }
        function la(pr) {
          for (var oi = [], ar = pr; ar.length; ar = ar[0])
            oi.push(ar.length);
          return oi;
        }
        var wa = {
          "[object Int8Array]": 5120,
          "[object Int16Array]": 5122,
          "[object Int32Array]": 5124,
          "[object Uint8Array]": 5121,
          "[object Uint8ClampedArray]": 5121,
          "[object Uint16Array]": 5123,
          "[object Uint32Array]": 5125,
          "[object Float32Array]": 5126,
          "[object Float64Array]": 5121,
          "[object ArrayBuffer]": 5121
        }, Wa = 5120, Qr = 5122, Ur = 5124, rr = 5121, vr = 5123, Rr = 5125, Vr = 5126, Xr = 5126, Jr = {
          int8: Wa,
          int16: Qr,
          int32: Ur,
          uint8: rr,
          uint16: vr,
          uint32: Rr,
          float: Vr,
          float32: Xr
        }, ti = 35048, Hr = 35040, Er = {
          dynamic: ti,
          stream: Hr,
          static: 35044
        }, gr = Sr.flatten, $r = Sr.shape, li = 35044, Qi = 35040, Ai = 5121, Li = 5126, Ni = [];
        Ni[5120] = 1, Ni[5122] = 2, Ni[5124] = 4, Ni[5121] = 1, Ni[5123] = 2, Ni[5125] = 4, Ni[5126] = 4;
        function Pi(pr) {
          return wa[Object.prototype.toString.call(pr)] | 0;
        }
        function Oi(pr, oi) {
          for (var ar = 0; ar < oi.length; ++ar)
            pr[ar] = oi[ar];
        }
        function ki(pr, oi, ar, Sa, Qn, za, En) {
          for (var Qo = 0, lo = 0; lo < ar; ++lo)
            for (var Eo = 0; Eo < Sa; ++Eo)
              pr[Qo++] = oi[Qn * lo + za * Eo + En];
        }
        function Mi(pr, oi, ar, Sa) {
          var Qn = 0, za = {};
          function En(Qa) {
            this.id = Qn++, this.buffer = pr.createBuffer(), this.type = Qa, this.usage = li, this.byteLength = 0, this.dimension = 1, this.dtype = Ai, this.persistentData = null, ar.profile && (this.stats = { size: 0 });
          }
          En.prototype.bind = function() {
            pr.bindBuffer(this.type, this.buffer);
          }, En.prototype.destroy = function() {
            Co(this);
          };
          var Qo = [];
          function lo(Qa, Ka) {
            var qo = Qo.pop();
            return qo || (qo = new En(Qa)), qo.bind(), Ho(qo, Ka, Qi, 0, 1, !1), qo;
          }
          function Eo(Qa) {
            Qo.push(Qa);
          }
          function bo(Qa, Ka, qo) {
            Qa.byteLength = Ka.byteLength, pr.bufferData(Qa.type, Ka, qo);
          }
          function Ho(Qa, Ka, qo, Ss, ho, hs) {
            var Bo;
            if (Qa.usage = qo, Array.isArray(Ka)) {
              if (Qa.dtype = Ss || Li, Ka.length > 0) {
                var Vs;
                if (Array.isArray(Ka[0])) {
                  Bo = $r(Ka);
                  for (var vo = 1, go = 1; go < Bo.length; ++go)
                    vo *= Bo[go];
                  Qa.dimension = vo, Vs = gr(Ka, Bo, Qa.dtype), bo(Qa, Vs, qo), hs ? Qa.persistentData = Vs : Ot.freeType(Vs);
                } else if (typeof Ka[0] == "number") {
                  Qa.dimension = ho;
                  var Qs = Ot.allocType(Qa.dtype, Ka.length);
                  Oi(Qs, Ka), bo(Qa, Qs, qo), hs ? Qa.persistentData = Qs : Ot.freeType(Qs);
                } else H(Ka[0]) ? (Qa.dimension = Ka[0].length, Qa.dtype = Ss || Pi(Ka[0]) || Li, Vs = gr(
                  Ka,
                  [Ka.length, Ka[0].length],
                  Qa.dtype
                ), bo(Qa, Vs, qo), hs ? Qa.persistentData = Vs : Ot.freeType(Vs)) : vt.raise("invalid buffer data");
              }
            } else if (H(Ka))
              Qa.dtype = Ss || Pi(Ka), Qa.dimension = ho, bo(Qa, Ka, qo), hs && (Qa.persistentData = new Uint8Array(new Uint8Array(Ka.buffer)));
            else if (Gr(Ka)) {
              Bo = Ka.shape;
              var us = Ka.stride, Po = Ka.offset, Jo = 0, ps = 0, wl = 0, ol = 0;
              Bo.length === 1 ? (Jo = Bo[0], ps = 1, wl = us[0], ol = 0) : Bo.length === 2 ? (Jo = Bo[0], ps = Bo[1], wl = us[0], ol = us[1]) : vt.raise("invalid shape"), Qa.dtype = Ss || Pi(Ka.data) || Li, Qa.dimension = ps;
              var xs = Ot.allocType(Qa.dtype, Jo * ps);
              ki(
                xs,
                Ka.data,
                Jo,
                ps,
                wl,
                ol,
                Po
              ), bo(Qa, xs, qo), hs ? Qa.persistentData = xs : Ot.freeType(xs);
            } else Ka instanceof ArrayBuffer ? (Qa.dtype = Ai, Qa.dimension = ho, bo(Qa, Ka, qo), hs && (Qa.persistentData = new Uint8Array(new Uint8Array(Ka)))) : vt.raise("invalid buffer data");
          }
          function Co(Qa) {
            oi.bufferCount--, Sa(Qa);
            var Ka = Qa.buffer;
            vt(Ka, "buffer must not be deleted already"), pr.deleteBuffer(Ka), Qa.buffer = null, delete za[Qa.id];
          }
          function Ao(Qa, Ka, qo, Ss) {
            oi.bufferCount++;
            var ho = new En(Ka);
            za[ho.id] = ho;
            function hs(vo) {
              var go = li, Qs = null, us = 0, Po = 0, Jo = 1;
              return Array.isArray(vo) || H(vo) || Gr(vo) || vo instanceof ArrayBuffer ? Qs = vo : typeof vo == "number" ? us = vo | 0 : vo && (vt.type(
                vo,
                "object",
                "buffer arguments must be an object, a number or an array"
              ), "data" in vo && (vt(
                Qs === null || Array.isArray(Qs) || H(Qs) || Gr(Qs),
                "invalid data for buffer"
              ), Qs = vo.data), "usage" in vo && (vt.parameter(vo.usage, Er, "invalid buffer usage"), go = Er[vo.usage]), "type" in vo && (vt.parameter(vo.type, Jr, "invalid buffer type"), Po = Jr[vo.type]), "dimension" in vo && (vt.type(vo.dimension, "number", "invalid dimension"), Jo = vo.dimension | 0), "length" in vo && (vt.nni(us, "buffer length must be a nonnegative integer"), us = vo.length | 0)), ho.bind(), Qs ? Ho(ho, Qs, go, Po, Jo, Ss) : (us && pr.bufferData(ho.type, us, go), ho.dtype = Po || Ai, ho.usage = go, ho.dimension = Jo, ho.byteLength = us), ar.profile && (ho.stats.size = ho.byteLength * Ni[ho.dtype]), hs;
            }
            function Bo(vo, go) {
              vt(
                go + vo.byteLength <= ho.byteLength,
                "invalid buffer subdata call, buffer is too small.  Can't write data of size " + vo.byteLength + " starting from offset " + go + " to a buffer of size " + ho.byteLength
              ), pr.bufferSubData(ho.type, go, vo);
            }
            function Vs(vo, go) {
              var Qs = (go || 0) | 0, us;
              if (ho.bind(), H(vo) || vo instanceof ArrayBuffer)
                Bo(vo, Qs);
              else if (Array.isArray(vo)) {
                if (vo.length > 0)
                  if (typeof vo[0] == "number") {
                    var Po = Ot.allocType(ho.dtype, vo.length);
                    Oi(Po, vo), Bo(Po, Qs), Ot.freeType(Po);
                  } else if (Array.isArray(vo[0]) || H(vo[0])) {
                    us = $r(vo);
                    var Jo = gr(vo, us, ho.dtype);
                    Bo(Jo, Qs), Ot.freeType(Jo);
                  } else
                    vt.raise("invalid buffer data");
              } else if (Gr(vo)) {
                us = vo.shape;
                var ps = vo.stride, wl = 0, ol = 0, xs = 0, ns = 0;
                us.length === 1 ? (wl = us[0], ol = 1, xs = ps[0], ns = 0) : us.length === 2 ? (wl = us[0], ol = us[1], xs = ps[0], ns = ps[1]) : vt.raise("invalid shape");
                var ll = Array.isArray(vo.data) ? ho.dtype : Pi(vo.data), Ws = Ot.allocType(ll, wl * ol);
                ki(
                  Ws,
                  vo.data,
                  wl,
                  ol,
                  xs,
                  ns,
                  vo.offset
                ), Bo(Ws, Qs), Ot.freeType(Ws);
              } else
                vt.raise("invalid data for buffer subdata");
              return hs;
            }
            return qo || hs(Qa), hs._reglType = "buffer", hs._buffer = ho, hs.subdata = Vs, ar.profile && (hs.stats = ho.stats), hs.destroy = function() {
              Co(ho);
            }, hs;
          }
          function Ro() {
            Ca(za).forEach(function(Qa) {
              Qa.buffer = pr.createBuffer(), pr.bindBuffer(Qa.type, Qa.buffer), pr.bufferData(
                Qa.type,
                Qa.persistentData || Qa.byteLength,
                Qa.usage
              );
            });
          }
          return ar.profile && (oi.getTotalBufferSize = function() {
            var Qa = 0;
            return Object.keys(za).forEach(function(Ka) {
              Qa += za[Ka].stats.size;
            }), Qa;
          }), {
            create: Ao,
            createStream: lo,
            destroyStream: Eo,
            clear: function() {
              Ca(za).forEach(Co), Qo.forEach(Co);
            },
            getBuffer: function(Qa) {
              return Qa && Qa._buffer instanceof En ? Qa._buffer : null;
            },
            restore: Ro,
            _initBuffer: Ho
          };
        }
        var Vi = 0, ua = 0, Ki = 1, Fi = 1, na = 4, nn = 4, Tn = {
          points: Vi,
          point: ua,
          lines: Ki,
          line: Fi,
          triangles: na,
          triangle: nn,
          "line loop": 2,
          "line strip": 3,
          "triangle strip": 5,
          "triangle fan": 6
        }, Dn = 0, Fn = 1, fr = 4, dr = 5120, yr = 5121, ai = 5122, Fr = 5123, Ti = 5124, Si = 5125, Ei = 34963, Fa = 35040, $o = 35044;
        function Gs(pr, oi, ar, Sa) {
          var Qn = {}, za = 0, En = {
            uint8: yr,
            uint16: Fr
          };
          oi.oes_element_index_uint && (En.uint32 = Si);
          function Qo(Ro) {
            this.id = za++, Qn[this.id] = this, this.buffer = Ro, this.primType = fr, this.vertCount = 0, this.type = 0;
          }
          Qo.prototype.bind = function() {
            this.buffer.bind();
          };
          var lo = [];
          function Eo(Ro) {
            var Qa = lo.pop();
            return Qa || (Qa = new Qo(ar.create(
              null,
              Ei,
              !0,
              !1
            )._buffer)), Ho(Qa, Ro, Fa, -1, -1, 0, 0), Qa;
          }
          function bo(Ro) {
            lo.push(Ro);
          }
          function Ho(Ro, Qa, Ka, qo, Ss, ho, hs) {
            Ro.buffer.bind();
            var Bo;
            if (Qa) {
              var Vs = hs;
              !hs && (!H(Qa) || Gr(Qa) && !H(Qa.data)) && (Vs = oi.oes_element_index_uint ? Si : Fr), ar._initBuffer(
                Ro.buffer,
                Qa,
                Ka,
                Vs,
                3
              );
            } else
              pr.bufferData(Ei, ho, Ka), Ro.buffer.dtype = Bo || yr, Ro.buffer.usage = Ka, Ro.buffer.dimension = 3, Ro.buffer.byteLength = ho;
            if (Bo = hs, !hs) {
              switch (Ro.buffer.dtype) {
                case yr:
                case dr:
                  Bo = yr;
                  break;
                case Fr:
                case ai:
                  Bo = Fr;
                  break;
                case Si:
                case Ti:
                  Bo = Si;
                  break;
                default:
                  vt.raise("unsupported type for element array");
              }
              Ro.buffer.dtype = Bo;
            }
            Ro.type = Bo, vt(
              Bo !== Si || !!oi.oes_element_index_uint,
              "32 bit element buffers not supported, enable oes_element_index_uint first"
            );
            var vo = Ss;
            vo < 0 && (vo = Ro.buffer.byteLength, Bo === Fr ? vo >>= 1 : Bo === Si && (vo >>= 2)), Ro.vertCount = vo;
            var go = qo;
            if (qo < 0) {
              go = fr;
              var Qs = Ro.buffer.dimension;
              Qs === 1 && (go = Dn), Qs === 2 && (go = Fn), Qs === 3 && (go = fr);
            }
            Ro.primType = go;
          }
          function Co(Ro) {
            Sa.elementsCount--, vt(Ro.buffer !== null, "must not double destroy elements"), delete Qn[Ro.id], Ro.buffer.destroy(), Ro.buffer = null;
          }
          function Ao(Ro, Qa) {
            var Ka = ar.create(null, Ei, !0), qo = new Qo(Ka._buffer);
            Sa.elementsCount++;
            function Ss(ho) {
              if (!ho)
                Ka(), qo.primType = fr, qo.vertCount = 0, qo.type = yr;
              else if (typeof ho == "number")
                Ka(ho), qo.primType = fr, qo.vertCount = ho | 0, qo.type = yr;
              else {
                var hs = null, Bo = $o, Vs = -1, vo = -1, go = 0, Qs = 0;
                Array.isArray(ho) || H(ho) || Gr(ho) ? hs = ho : (vt.type(ho, "object", "invalid arguments for elements"), "data" in ho && (hs = ho.data, vt(
                  Array.isArray(hs) || H(hs) || Gr(hs),
                  "invalid data for element buffer"
                )), "usage" in ho && (vt.parameter(
                  ho.usage,
                  Er,
                  "invalid element buffer usage"
                ), Bo = Er[ho.usage]), "primitive" in ho && (vt.parameter(
                  ho.primitive,
                  Tn,
                  "invalid element buffer primitive"
                ), Vs = Tn[ho.primitive]), "count" in ho && (vt(
                  typeof ho.count == "number" && ho.count >= 0,
                  "invalid vertex count for elements"
                ), vo = ho.count | 0), "type" in ho && (vt.parameter(
                  ho.type,
                  En,
                  "invalid buffer type"
                ), Qs = En[ho.type]), "length" in ho ? go = ho.length | 0 : (go = vo, Qs === Fr || Qs === ai ? go *= 2 : (Qs === Si || Qs === Ti) && (go *= 4))), Ho(
                  qo,
                  hs,
                  Bo,
                  Vs,
                  vo,
                  go,
                  Qs
                );
              }
              return Ss;
            }
            return Ss(Ro), Ss._reglType = "elements", Ss._elements = qo, Ss.subdata = function(ho, hs) {
              return Ka.subdata(ho, hs), Ss;
            }, Ss.destroy = function() {
              Co(qo);
            }, Ss;
          }
          return {
            create: Ao,
            createStream: Eo,
            destroyStream: bo,
            getElements: function(Ro) {
              return typeof Ro == "function" && Ro._elements instanceof Qo ? Ro._elements : null;
            },
            clear: function() {
              Ca(Qn).forEach(Co);
            }
          };
        }
        var Al = new Float32Array(1), ul = new Uint32Array(Al.buffer), gc = 5123;
        function Gl(pr) {
          for (var oi = Ot.allocType(gc, pr.length), ar = 0; ar < pr.length; ++ar)
            if (isNaN(pr[ar]))
              oi[ar] = 65535;
            else if (pr[ar] === 1 / 0)
              oi[ar] = 31744;
            else if (pr[ar] === -1 / 0)
              oi[ar] = 64512;
            else {
              Al[0] = pr[ar];
              var Sa = ul[0], Qn = Sa >>> 31 << 15, za = (Sa << 1 >>> 24) - 127, En = Sa >> 13 & 1023;
              if (za < -24)
                oi[ar] = Qn;
              else if (za < -14) {
                var Qo = -14 - za;
                oi[ar] = Qn + (En + 1024 >> Qo);
              } else za > 15 ? oi[ar] = Qn + 31744 : oi[ar] = Qn + (za + 15 << 10) + En;
            }
          return oi;
        }
        function Sl(pr) {
          return Array.isArray(pr) || H(pr);
        }
        var ls = function(pr) {
          return !(pr & pr - 1) && !!pr;
        }, vs = 34467, Vl = 3553, Ll = 34067, Xl = 34069, yc = 6408, gu = 6406, Nc = 6407, Cu = 6409, zc = 6410, Kc = 32854, zi = 32855, Fu = 36194, d1 = 32819, Gu = 32820, Ku = 33635, I1 = 34042, f1 = 6402, D1 = 34041, Hl = 35904, Kl = 35906, uu = 36193, Hs = 33776, b1 = 33777, Wl = 33778, zl = 33779, os = 35986, jo = 35987, uo = 34798, so = 35840, So = 35841, Jc = 35842, No = 35843, ro = 36196, Io = 5121, bs = 5123, ws = 5125, zo = 5126, Yo = 10242, Ls = 10243, Ms = 10497, vi = 33071, Ri = 33648, ta = 10240, da = 10241, Ji = 9728, Pa = 9729, Ga = 9984, Ha = 9985, Ba = 9986, ao = 9987, io = 33170, To = 4352, Mo = 4353, po = 4354, Vo = 34046, Xo = 3317, Go = 37440, es = 37441, Ts = 37443, Fo = 37444, Oo = 33984, gs = [
          Ga,
          Ba,
          Ha,
          ao
        ], Bu = [
          0,
          Cu,
          zc,
          Nc,
          yc
        ], Ql = {};
        Ql[Cu] = Ql[gu] = Ql[f1] = 1, Ql[D1] = Ql[zc] = 2, Ql[Nc] = Ql[Hl] = 3, Ql[yc] = Ql[Kl] = 4;
        function dl(pr) {
          return "[object " + pr + "]";
        }
        var h1 = dl("HTMLCanvasElement"), z1 = dl("OffscreenCanvas"), J1 = dl("CanvasRenderingContext2D"), Lu = dl("ImageBitmap"), M1 = dl("HTMLImageElement"), ku = dl("HTMLVideoElement"), Wu = Object.keys(wa).concat([
          h1,
          z1,
          J1,
          Lu,
          M1,
          ku
        ]), T1 = [];
        T1[Io] = 1, T1[zo] = 4, T1[uu] = 2, T1[bs] = 2, T1[ws] = 4;
        var kl = [];
        kl[Kc] = 2, kl[zi] = 2, kl[Fu] = 2, kl[D1] = 4, kl[Hs] = 0.5, kl[b1] = 0.5, kl[Wl] = 1, kl[zl] = 1, kl[os] = 0.5, kl[jo] = 1, kl[uo] = 1, kl[so] = 0.5, kl[So] = 0.25, kl[Jc] = 0.5, kl[No] = 0.25, kl[ro] = 0.5;
        function Yl(pr) {
          return Array.isArray(pr) && (pr.length === 0 || typeof pr[0] == "number");
        }
        function i1(pr) {
          if (!Array.isArray(pr))
            return !1;
          var oi = pr.length;
          return !(oi === 0 || !Sl(pr[0]));
        }
        function ic(pr) {
          return Object.prototype.toString.call(pr);
        }
        function iu(pr) {
          return ic(pr) === h1;
        }
        function xu(pr) {
          return ic(pr) === z1;
        }
        function vc(pr) {
          return ic(pr) === J1;
        }
        function Zl(pr) {
          return ic(pr) === Lu;
        }
        function Ec(pr) {
          return ic(pr) === M1;
        }
        function wu(pr) {
          return ic(pr) === ku;
        }
        function Pc(pr) {
          if (!pr)
            return !1;
          var oi = ic(pr);
          return Wu.indexOf(oi) >= 0 ? !0 : Yl(pr) || i1(pr) || Gr(pr);
        }
        function du(pr) {
          return wa[Object.prototype.toString.call(pr)] | 0;
        }
        function Ac(pr, oi) {
          var ar = oi.length;
          switch (pr.type) {
            case Io:
            case bs:
            case ws:
            case zo:
              var Sa = Ot.allocType(pr.type, ar);
              Sa.set(oi), pr.data = Sa;
              break;
            case uu:
              pr.data = Gl(oi);
              break;
            default:
              vt.raise("unsupported texture type, must specify a typed array");
          }
        }
        function yu(pr, oi) {
          return Ot.allocType(
            pr.type === uu ? zo : pr.type,
            oi
          );
        }
        function au(pr, oi) {
          pr.type === uu ? (pr.data = Gl(oi), Ot.freeType(oi)) : pr.data = oi;
        }
        function Qc(pr, oi, ar, Sa, Qn, za) {
          for (var En = pr.width, Qo = pr.height, lo = pr.channels, Eo = En * Qo * lo, bo = yu(pr, Eo), Ho = 0, Co = 0; Co < Qo; ++Co)
            for (var Ao = 0; Ao < En; ++Ao)
              for (var Ro = 0; Ro < lo; ++Ro)
                bo[Ho++] = oi[ar * Ao + Sa * Co + Qn * Ro + za];
          au(pr, bo);
        }
        function Cc(pr, oi, ar, Sa, Qn, za) {
          var En;
          if (typeof kl[pr] < "u" ? En = kl[pr] : En = Ql[pr] * T1[oi], za && (En *= 6), Qn) {
            for (var Qo = 0, lo = ar; lo >= 1; )
              Qo += En * lo * lo, lo /= 2;
            return Qo;
          } else
            return En * ar * Sa;
        }
        function Ol(pr, oi, ar, Sa, Qn, za, En) {
          var Qo = {
            "don't care": To,
            "dont care": To,
            nice: po,
            fast: Mo
          }, lo = {
            repeat: Ms,
            clamp: vi,
            mirror: Ri
          }, Eo = {
            nearest: Ji,
            linear: Pa
          }, bo = V({
            mipmap: ao,
            "nearest mipmap nearest": Ga,
            "linear mipmap nearest": Ha,
            "nearest mipmap linear": Ba,
            "linear mipmap linear": ao
          }, Eo), Ho = {
            none: 0,
            browser: Fo
          }, Co = {
            uint8: Io,
            rgba4: d1,
            rgb565: Ku,
            "rgb5 a1": Gu
          }, Ao = {
            alpha: gu,
            luminance: Cu,
            "luminance alpha": zc,
            rgb: Nc,
            rgba: yc,
            rgba4: Kc,
            "rgb5 a1": zi,
            rgb565: Fu
          }, Ro = {};
          oi.ext_srgb && (Ao.srgb = Hl, Ao.srgba = Kl), oi.oes_texture_float && (Co.float32 = Co.float = zo), oi.oes_texture_half_float && (Co.float16 = Co["half float"] = uu), oi.webgl_depth_texture && (V(Ao, {
            depth: f1,
            "depth stencil": D1
          }), V(Co, {
            uint16: bs,
            uint32: ws,
            "depth stencil": I1
          })), oi.webgl_compressed_texture_s3tc && V(Ro, {
            "rgb s3tc dxt1": Hs,
            "rgba s3tc dxt1": b1,
            "rgba s3tc dxt3": Wl,
            "rgba s3tc dxt5": zl
          }), oi.webgl_compressed_texture_atc && V(Ro, {
            "rgb atc": os,
            "rgba atc explicit alpha": jo,
            "rgba atc interpolated alpha": uo
          }), oi.webgl_compressed_texture_pvrtc && V(Ro, {
            "rgb pvrtc 4bppv1": so,
            "rgb pvrtc 2bppv1": So,
            "rgba pvrtc 4bppv1": Jc,
            "rgba pvrtc 2bppv1": No
          }), oi.webgl_compressed_texture_etc1 && (Ro["rgb etc1"] = ro);
          var Qa = Array.prototype.slice.call(
            pr.getParameter(vs)
          );
          Object.keys(Ro).forEach(function(Xi) {
            var Za = Ro[Xi];
            Qa.indexOf(Za) >= 0 && (Ao[Xi] = Za);
          });
          var Ka = Object.keys(Ao);
          ar.textureFormats = Ka;
          var qo = [];
          Object.keys(Ao).forEach(function(Xi) {
            var Za = Ao[Xi];
            qo[Za] = Xi;
          });
          var Ss = [];
          Object.keys(Co).forEach(function(Xi) {
            var Za = Co[Xi];
            Ss[Za] = Xi;
          });
          var ho = [];
          Object.keys(Eo).forEach(function(Xi) {
            var Za = Eo[Xi];
            ho[Za] = Xi;
          });
          var hs = [];
          Object.keys(bo).forEach(function(Xi) {
            var Za = bo[Xi];
            hs[Za] = Xi;
          });
          var Bo = [];
          Object.keys(lo).forEach(function(Xi) {
            var Za = lo[Xi];
            Bo[Za] = Xi;
          });
          var Vs = Ka.reduce(function(Xi, Za) {
            var ja = Ao[Za];
            return ja === Cu || ja === gu || ja === Cu || ja === zc || ja === f1 || ja === D1 || oi.ext_srgb && (ja === Hl || ja === Kl) ? Xi[ja] = ja : ja === zi || Za.indexOf("rgba") >= 0 ? Xi[ja] = yc : Xi[ja] = Nc, Xi;
          }, {});
          function vo() {
            this.internalformat = yc, this.format = yc, this.type = Io, this.compressed = !1, this.premultiplyAlpha = !1, this.flipY = !1, this.unpackAlignment = 1, this.colorSpace = Fo, this.width = 0, this.height = 0, this.channels = 0;
          }
          function go(Xi, Za) {
            Xi.internalformat = Za.internalformat, Xi.format = Za.format, Xi.type = Za.type, Xi.compressed = Za.compressed, Xi.premultiplyAlpha = Za.premultiplyAlpha, Xi.flipY = Za.flipY, Xi.unpackAlignment = Za.unpackAlignment, Xi.colorSpace = Za.colorSpace, Xi.width = Za.width, Xi.height = Za.height, Xi.channels = Za.channels;
          }
          function Qs(Xi, Za) {
            if (!(typeof Za != "object" || !Za)) {
              if ("premultiplyAlpha" in Za && (vt.type(
                Za.premultiplyAlpha,
                "boolean",
                "invalid premultiplyAlpha"
              ), Xi.premultiplyAlpha = Za.premultiplyAlpha), "flipY" in Za && (vt.type(
                Za.flipY,
                "boolean",
                "invalid texture flip"
              ), Xi.flipY = Za.flipY), "alignment" in Za && (vt.oneOf(
                Za.alignment,
                [1, 2, 4, 8],
                "invalid texture unpack alignment"
              ), Xi.unpackAlignment = Za.alignment), "colorSpace" in Za && (vt.parameter(
                Za.colorSpace,
                Ho,
                "invalid colorSpace"
              ), Xi.colorSpace = Ho[Za.colorSpace]), "type" in Za) {
                var ja = Za.type;
                vt(
                  oi.oes_texture_float || !(ja === "float" || ja === "float32"),
                  "you must enable the OES_texture_float extension in order to use floating point textures."
                ), vt(
                  oi.oes_texture_half_float || !(ja === "half float" || ja === "float16"),
                  "you must enable the OES_texture_half_float extension in order to use 16-bit floating point textures."
                ), vt(
                  oi.webgl_depth_texture || !(ja === "uint16" || ja === "uint32" || ja === "depth stencil"),
                  "you must enable the WEBGL_depth_texture extension in order to use depth/stencil textures."
                ), vt.parameter(
                  ja,
                  Co,
                  "invalid texture type"
                ), Xi.type = Co[ja];
              }
              var Ds = Xi.width, rc = Xi.height, Di = Xi.channels, yi = !1;
              "shape" in Za ? (vt(
                Array.isArray(Za.shape) && Za.shape.length >= 2,
                "shape must be an array"
              ), Ds = Za.shape[0], rc = Za.shape[1], Za.shape.length === 3 && (Di = Za.shape[2], vt(Di > 0 && Di <= 4, "invalid number of channels"), yi = !0), vt(Ds >= 0 && Ds <= ar.maxTextureSize, "invalid width"), vt(rc >= 0 && rc <= ar.maxTextureSize, "invalid height")) : ("radius" in Za && (Ds = rc = Za.radius, vt(Ds >= 0 && Ds <= ar.maxTextureSize, "invalid radius")), "width" in Za && (Ds = Za.width, vt(Ds >= 0 && Ds <= ar.maxTextureSize, "invalid width")), "height" in Za && (rc = Za.height, vt(rc >= 0 && rc <= ar.maxTextureSize, "invalid height")), "channels" in Za && (Di = Za.channels, vt(Di > 0 && Di <= 4, "invalid number of channels"), yi = !0)), Xi.width = Ds | 0, Xi.height = rc | 0, Xi.channels = Di | 0;
              var ca = !1;
              if ("format" in Za) {
                var Ma = Za.format;
                vt(
                  oi.webgl_depth_texture || !(Ma === "depth" || Ma === "depth stencil"),
                  "you must enable the WEBGL_depth_texture extension in order to use depth/stencil textures."
                ), vt.parameter(
                  Ma,
                  Ao,
                  "invalid texture format"
                );
                var Ia = Xi.internalformat = Ao[Ma];
                Xi.format = Vs[Ia], Ma in Co && ("type" in Za || (Xi.type = Co[Ma])), Ma in Ro && (Xi.compressed = !0), ca = !0;
              }
              !yi && ca ? Xi.channels = Ql[Xi.format] : yi && !ca ? Xi.channels !== Bu[Xi.format] && (Xi.format = Xi.internalformat = Bu[Xi.channels]) : ca && yi && vt(
                Xi.channels === Ql[Xi.format],
                "number of channels inconsistent with specified format"
              );
            }
          }
          function us(Xi) {
            pr.pixelStorei(Go, Xi.flipY), pr.pixelStorei(es, Xi.premultiplyAlpha), pr.pixelStorei(Ts, Xi.colorSpace), pr.pixelStorei(Xo, Xi.unpackAlignment);
          }
          function Po() {
            vo.call(this), this.xOffset = 0, this.yOffset = 0, this.data = null, this.needsFree = !1, this.element = null, this.needsCopy = !1;
          }
          function Jo(Xi, Za) {
            var ja = null;
            if (Pc(Za) ? ja = Za : Za && (vt.type(Za, "object", "invalid pixel data type"), Qs(Xi, Za), "x" in Za && (Xi.xOffset = Za.x | 0), "y" in Za && (Xi.yOffset = Za.y | 0), Pc(Za.data) && (ja = Za.data)), vt(
              !Xi.compressed || ja instanceof Uint8Array,
              "compressed texture data must be stored in a uint8array"
            ), Za.copy) {
              vt(!ja, "can not specify copy and data field for the same texture");
              var Ds = Qn.viewportWidth, rc = Qn.viewportHeight;
              Xi.width = Xi.width || Ds - Xi.xOffset, Xi.height = Xi.height || rc - Xi.yOffset, Xi.needsCopy = !0, vt(
                Xi.xOffset >= 0 && Xi.xOffset < Ds && Xi.yOffset >= 0 && Xi.yOffset < rc && Xi.width > 0 && Xi.width <= Ds && Xi.height > 0 && Xi.height <= rc,
                "copy texture read out of bounds"
              );
            } else if (!ja)
              Xi.width = Xi.width || 1, Xi.height = Xi.height || 1, Xi.channels = Xi.channels || 4;
            else if (H(ja))
              Xi.channels = Xi.channels || 4, Xi.data = ja, !("type" in Za) && Xi.type === Io && (Xi.type = du(ja));
            else if (Yl(ja))
              Xi.channels = Xi.channels || 4, Ac(Xi, ja), Xi.alignment = 1, Xi.needsFree = !0;
            else if (Gr(ja)) {
              var Di = ja.data;
              !Array.isArray(Di) && Xi.type === Io && (Xi.type = du(Di));
              var yi = ja.shape, ca = ja.stride, Ma, Ia, ba, fa, Ea, Hi;
              yi.length === 3 ? (ba = yi[2], Hi = ca[2]) : (vt(yi.length === 2, "invalid ndarray pixel data, must be 2 or 3D"), ba = 1, Hi = 1), Ma = yi[0], Ia = yi[1], fa = ca[0], Ea = ca[1], Xi.alignment = 1, Xi.width = Ma, Xi.height = Ia, Xi.channels = ba, Xi.format = Xi.internalformat = Bu[ba], Xi.needsFree = !0, Qc(Xi, Di, fa, Ea, Hi, ja.offset);
            } else if (iu(ja) || xu(ja) || vc(ja))
              iu(ja) || xu(ja) ? Xi.element = ja : Xi.element = ja.canvas, Xi.width = Xi.element.width, Xi.height = Xi.element.height, Xi.channels = 4;
            else if (Zl(ja))
              Xi.element = ja, Xi.width = ja.width, Xi.height = ja.height, Xi.channels = 4;
            else if (Ec(ja))
              Xi.element = ja, Xi.width = ja.naturalWidth, Xi.height = ja.naturalHeight, Xi.channels = 4;
            else if (wu(ja))
              Xi.element = ja, Xi.width = ja.videoWidth, Xi.height = ja.videoHeight, Xi.channels = 4;
            else if (i1(ja)) {
              var ji = Xi.width || ja[0].length, qi = Xi.height || ja.length, Ua = Xi.channels;
              Sl(ja[0][0]) ? Ua = Ua || ja[0][0].length : Ua = Ua || 1;
              for (var La = Sr.shape(ja), yo = 1, xo = 0; xo < La.length; ++xo)
                yo *= La[xo];
              var rs = yu(Xi, yo);
              Sr.flatten(ja, La, "", rs), au(Xi, rs), Xi.alignment = 1, Xi.width = ji, Xi.height = qi, Xi.channels = Ua, Xi.format = Xi.internalformat = Bu[Ua], Xi.needsFree = !0;
            }
            Xi.type === zo ? vt(
              ar.extensions.indexOf("oes_texture_float") >= 0,
              "oes_texture_float extension not enabled"
            ) : Xi.type === uu && vt(
              ar.extensions.indexOf("oes_texture_half_float") >= 0,
              "oes_texture_half_float extension not enabled"
            );
          }
          function ps(Xi, Za, ja) {
            var Ds = Xi.element, rc = Xi.data, Di = Xi.internalformat, yi = Xi.format, ca = Xi.type, Ma = Xi.width, Ia = Xi.height;
            us(Xi), Ds ? pr.texImage2D(Za, ja, yi, yi, ca, Ds) : Xi.compressed ? pr.compressedTexImage2D(Za, ja, Di, Ma, Ia, 0, rc) : Xi.needsCopy ? (Sa(), pr.copyTexImage2D(
              Za,
              ja,
              yi,
              Xi.xOffset,
              Xi.yOffset,
              Ma,
              Ia,
              0
            )) : pr.texImage2D(Za, ja, yi, Ma, Ia, 0, yi, ca, rc || null);
          }
          function wl(Xi, Za, ja, Ds, rc) {
            var Di = Xi.element, yi = Xi.data, ca = Xi.internalformat, Ma = Xi.format, Ia = Xi.type, ba = Xi.width, fa = Xi.height;
            us(Xi), Di ? pr.texSubImage2D(
              Za,
              rc,
              ja,
              Ds,
              Ma,
              Ia,
              Di
            ) : Xi.compressed ? pr.compressedTexSubImage2D(
              Za,
              rc,
              ja,
              Ds,
              ca,
              ba,
              fa,
              yi
            ) : Xi.needsCopy ? (Sa(), pr.copyTexSubImage2D(
              Za,
              rc,
              ja,
              Ds,
              Xi.xOffset,
              Xi.yOffset,
              ba,
              fa
            )) : pr.texSubImage2D(
              Za,
              rc,
              ja,
              Ds,
              ba,
              fa,
              Ma,
              Ia,
              yi
            );
          }
          var ol = [];
          function xs() {
            return ol.pop() || new Po();
          }
          function ns(Xi) {
            Xi.needsFree && Ot.freeType(Xi.data), Po.call(Xi), ol.push(Xi);
          }
          function ll() {
            vo.call(this), this.genMipmaps = !1, this.mipmapHint = To, this.mipmask = 0, this.images = Array(16);
          }
          function Ws(Xi, Za, ja) {
            var Ds = Xi.images[0] = xs();
            Xi.mipmask = 1, Ds.width = Xi.width = Za, Ds.height = Xi.height = ja, Ds.channels = Xi.channels = 4;
          }
          function ec(Xi, Za) {
            var ja = null;
            if (Pc(Za))
              ja = Xi.images[0] = xs(), go(ja, Xi), Jo(ja, Za), Xi.mipmask = 1;
            else if (Qs(Xi, Za), Array.isArray(Za.mipmap))
              for (var Ds = Za.mipmap, rc = 0; rc < Ds.length; ++rc)
                ja = Xi.images[rc] = xs(), go(ja, Xi), ja.width >>= rc, ja.height >>= rc, Jo(ja, Ds[rc]), Xi.mipmask |= 1 << rc;
            else
              ja = Xi.images[0] = xs(), go(ja, Xi), Jo(ja, Za), Xi.mipmask = 1;
            go(Xi, Xi.images[0]), Xi.compressed && (Xi.internalformat === Hs || Xi.internalformat === b1 || Xi.internalformat === Wl || Xi.internalformat === zl) && vt(
              Xi.width % 4 === 0 && Xi.height % 4 === 0,
              "for compressed texture formats, mipmap level 0 must have width and height that are a multiple of 4"
            );
          }
          function s1(Xi, Za) {
            for (var ja = Xi.images, Ds = 0; Ds < ja.length; ++Ds) {
              if (!ja[Ds])
                return;
              ps(ja[Ds], Za, Ds);
            }
          }
          var n1 = [];
          function Js() {
            var Xi = n1.pop() || new ll();
            vo.call(Xi), Xi.mipmask = 0;
            for (var Za = 0; Za < 16; ++Za)
              Xi.images[Za] = null;
            return Xi;
          }
          function N1(Xi) {
            for (var Za = Xi.images, ja = 0; ja < Za.length; ++ja)
              Za[ja] && ns(Za[ja]), Za[ja] = null;
            n1.push(Xi);
          }
          function Xu() {
            this.minFilter = Ji, this.magFilter = Ji, this.wrapS = vi, this.wrapT = vi, this.anisotropic = 1, this.genMipmaps = !1, this.mipmapHint = To;
          }
          function Q1(Xi, Za) {
            if ("min" in Za) {
              var ja = Za.min;
              vt.parameter(ja, bo), Xi.minFilter = bo[ja], gs.indexOf(Xi.minFilter) >= 0 && !("faces" in Za) && (Xi.genMipmaps = !0);
            }
            if ("mag" in Za) {
              var Ds = Za.mag;
              vt.parameter(Ds, Eo), Xi.magFilter = Eo[Ds];
            }
            var rc = Xi.wrapS, Di = Xi.wrapT;
            if ("wrap" in Za) {
              var yi = Za.wrap;
              typeof yi == "string" ? (vt.parameter(yi, lo), rc = Di = lo[yi]) : Array.isArray(yi) && (vt.parameter(yi[0], lo), vt.parameter(yi[1], lo), rc = lo[yi[0]], Di = lo[yi[1]]);
            } else {
              if ("wrapS" in Za) {
                var ca = Za.wrapS;
                vt.parameter(ca, lo), rc = lo[ca];
              }
              if ("wrapT" in Za) {
                var Ma = Za.wrapT;
                vt.parameter(Ma, lo), Di = lo[Ma];
              }
            }
            if (Xi.wrapS = rc, Xi.wrapT = Di, "anisotropic" in Za) {
              var Ia = Za.anisotropic;
              vt(
                typeof Ia == "number" && Ia >= 1 && Ia <= ar.maxAnisotropic,
                "aniso samples must be between 1 and "
              ), Xi.anisotropic = Za.anisotropic;
            }
            if ("mipmap" in Za) {
              var ba = !1;
              switch (typeof Za.mipmap) {
                case "string":
                  vt.parameter(
                    Za.mipmap,
                    Qo,
                    "invalid mipmap hint"
                  ), Xi.mipmapHint = Qo[Za.mipmap], Xi.genMipmaps = !0, ba = !0;
                  break;
                case "boolean":
                  ba = Xi.genMipmaps = Za.mipmap;
                  break;
                case "object":
                  vt(Array.isArray(Za.mipmap), "invalid mipmap type"), Xi.genMipmaps = !1, ba = !0;
                  break;
                default:
                  vt.raise("invalid mipmap type");
              }
              ba && !("min" in Za) && (Xi.minFilter = Ga);
            }
          }
          function Qu(Xi, Za) {
            pr.texParameteri(Za, da, Xi.minFilter), pr.texParameteri(Za, ta, Xi.magFilter), pr.texParameteri(Za, Yo, Xi.wrapS), pr.texParameteri(Za, Ls, Xi.wrapT), oi.ext_texture_filter_anisotropic && pr.texParameteri(Za, Vo, Xi.anisotropic), Xi.genMipmaps && (pr.hint(io, Xi.mipmapHint), pr.generateMipmap(Za));
          }
          var r1 = 0, ud = {}, Ed = ar.maxTextureUnits, R1 = Array(Ed).map(function() {
            return null;
          });
          function jl(Xi) {
            vo.call(this), this.mipmask = 0, this.internalformat = yc, this.id = r1++, this.refCount = 1, this.target = Xi, this.texture = pr.createTexture(), this.unit = -1, this.bindCount = 0, this.texInfo = new Xu(), En.profile && (this.stats = { size: 0 });
          }
          function Ad(Xi) {
            pr.activeTexture(Oo), pr.bindTexture(Xi.target, Xi.texture);
          }
          function Fc() {
            var Xi = R1[0];
            Xi ? pr.bindTexture(Xi.target, Xi.texture) : pr.bindTexture(Vl, null);
          }
          function yl(Xi) {
            var Za = Xi.texture;
            vt(Za, "must not double destroy texture");
            var ja = Xi.unit, Ds = Xi.target;
            ja >= 0 && (pr.activeTexture(Oo + ja), pr.bindTexture(Ds, null), R1[ja] = null), pr.deleteTexture(Za), Xi.texture = null, Xi.params = null, Xi.pixels = null, Xi.refCount = 0, delete ud[Xi.id], za.textureCount--;
          }
          V(jl.prototype, {
            bind: function() {
              var Xi = this;
              Xi.bindCount += 1;
              var Za = Xi.unit;
              if (Za < 0) {
                for (var ja = 0; ja < Ed; ++ja) {
                  var Ds = R1[ja];
                  if (Ds) {
                    if (Ds.bindCount > 0)
                      continue;
                    Ds.unit = -1;
                  }
                  R1[ja] = Xi, Za = ja;
                  break;
                }
                Za >= Ed && vt.raise("insufficient number of texture units"), En.profile && za.maxTextureUnits < Za + 1 && (za.maxTextureUnits = Za + 1), Xi.unit = Za, pr.activeTexture(Oo + Za), pr.bindTexture(Xi.target, Xi.texture);
              }
              return Za;
            },
            unbind: function() {
              this.bindCount -= 1;
            },
            decRef: function() {
              --this.refCount <= 0 && yl(this);
            }
          });
          function cc(Xi, Za) {
            var ja = new jl(Vl);
            ud[ja.id] = ja, za.textureCount++;
            function Ds(yi, ca) {
              var Ma = ja.texInfo;
              Xu.call(Ma);
              var Ia = Js();
              return typeof yi == "number" ? typeof ca == "number" ? Ws(Ia, yi | 0, ca | 0) : Ws(Ia, yi | 0, yi | 0) : yi ? (vt.type(yi, "object", "invalid arguments to regl.texture"), Q1(Ma, yi), ec(Ia, yi)) : Ws(Ia, 1, 1), Ma.genMipmaps && (Ia.mipmask = (Ia.width << 1) - 1), ja.mipmask = Ia.mipmask, go(ja, Ia), vt.texture2D(Ma, Ia, ar), ja.internalformat = Ia.internalformat, Ds.width = Ia.width, Ds.height = Ia.height, Ad(ja), s1(Ia, Vl), Qu(Ma, Vl), Fc(), N1(Ia), En.profile && (ja.stats.size = Cc(
                ja.internalformat,
                ja.type,
                Ia.width,
                Ia.height,
                Ma.genMipmaps,
                !1
              )), Ds.format = qo[ja.internalformat], Ds.type = Ss[ja.type], Ds.mag = ho[Ma.magFilter], Ds.min = hs[Ma.minFilter], Ds.wrapS = Bo[Ma.wrapS], Ds.wrapT = Bo[Ma.wrapT], Ds;
            }
            function rc(yi, ca, Ma, Ia) {
              vt(!!yi, "must specify image data");
              var ba = ca | 0, fa = Ma | 0, Ea = Ia | 0, Hi = xs();
              return go(Hi, ja), Hi.width = 0, Hi.height = 0, Jo(Hi, yi), Hi.width = Hi.width || (ja.width >> Ea) - ba, Hi.height = Hi.height || (ja.height >> Ea) - fa, vt(
                ja.type === Hi.type && ja.format === Hi.format && ja.internalformat === Hi.internalformat,
                "incompatible format for texture.subimage"
              ), vt(
                ba >= 0 && fa >= 0 && ba + Hi.width <= ja.width && fa + Hi.height <= ja.height,
                "texture.subimage write out of bounds"
              ), vt(
                ja.mipmask & 1 << Ea,
                "missing mipmap data"
              ), vt(
                Hi.data || Hi.element || Hi.needsCopy,
                "missing image data"
              ), Ad(ja), wl(Hi, Vl, ba, fa, Ea), Fc(), ns(Hi), Ds;
            }
            function Di(yi, ca) {
              var Ma = yi | 0, Ia = ca | 0 || Ma;
              if (Ma === ja.width && Ia === ja.height)
                return Ds;
              Ds.width = ja.width = Ma, Ds.height = ja.height = Ia, Ad(ja);
              for (var ba = 0; ja.mipmask >> ba; ++ba) {
                var fa = Ma >> ba, Ea = Ia >> ba;
                if (!fa || !Ea) break;
                pr.texImage2D(
                  Vl,
                  ba,
                  ja.format,
                  fa,
                  Ea,
                  0,
                  ja.format,
                  ja.type,
                  null
                );
              }
              return Fc(), En.profile && (ja.stats.size = Cc(
                ja.internalformat,
                ja.type,
                Ma,
                Ia,
                !1,
                !1
              )), Ds;
            }
            return Ds(Xi, Za), Ds.subimage = rc, Ds.resize = Di, Ds._reglType = "texture2d", Ds._texture = ja, En.profile && (Ds.stats = ja.stats), Ds.destroy = function() {
              ja.decRef();
            }, Ds;
          }
          function Bc(Xi, Za, ja, Ds, rc, Di) {
            var yi = new jl(Ll);
            ud[yi.id] = yi, za.cubeCount++;
            var ca = new Array(6);
            function Ma(fa, Ea, Hi, ji, qi, Ua) {
              var La, yo = yi.texInfo;
              for (Xu.call(yo), La = 0; La < 6; ++La)
                ca[La] = Js();
              if (typeof fa == "number" || !fa) {
                var xo = fa | 0 || 1;
                for (La = 0; La < 6; ++La)
                  Ws(ca[La], xo, xo);
              } else if (typeof fa == "object")
                if (Ea)
                  ec(ca[0], fa), ec(ca[1], Ea), ec(ca[2], Hi), ec(ca[3], ji), ec(ca[4], qi), ec(ca[5], Ua);
                else if (Q1(yo, fa), Qs(yi, fa), "faces" in fa) {
                  var rs = fa.faces;
                  for (vt(
                    Array.isArray(rs) && rs.length === 6,
                    "cube faces must be a length 6 array"
                  ), La = 0; La < 6; ++La)
                    vt(
                      typeof rs[La] == "object" && !!rs[La],
                      "invalid input for cube map face"
                    ), go(ca[La], yi), ec(ca[La], rs[La]);
                } else
                  for (La = 0; La < 6; ++La)
                    ec(ca[La], fa);
              else
                vt.raise("invalid arguments to cube map");
              for (go(yi, ca[0]), vt.optional(function() {
                ar.npotTextureCube || vt(ls(yi.width) && ls(yi.height), "your browser does not support non power or two texture dimensions");
              }), yo.genMipmaps ? yi.mipmask = (ca[0].width << 1) - 1 : yi.mipmask = ca[0].mipmask, vt.textureCube(yi, yo, ca, ar), yi.internalformat = ca[0].internalformat, Ma.width = ca[0].width, Ma.height = ca[0].height, Ad(yi), La = 0; La < 6; ++La)
                s1(ca[La], Xl + La);
              for (Qu(yo, Ll), Fc(), En.profile && (yi.stats.size = Cc(
                yi.internalformat,
                yi.type,
                Ma.width,
                Ma.height,
                yo.genMipmaps,
                !0
              )), Ma.format = qo[yi.internalformat], Ma.type = Ss[yi.type], Ma.mag = ho[yo.magFilter], Ma.min = hs[yo.minFilter], Ma.wrapS = Bo[yo.wrapS], Ma.wrapT = Bo[yo.wrapT], La = 0; La < 6; ++La)
                N1(ca[La]);
              return Ma;
            }
            function Ia(fa, Ea, Hi, ji, qi) {
              vt(!!Ea, "must specify image data"), vt(typeof fa == "number" && fa === (fa | 0) && fa >= 0 && fa < 6, "invalid face");
              var Ua = Hi | 0, La = ji | 0, yo = qi | 0, xo = xs();
              return go(xo, yi), xo.width = 0, xo.height = 0, Jo(xo, Ea), xo.width = xo.width || (yi.width >> yo) - Ua, xo.height = xo.height || (yi.height >> yo) - La, vt(
                yi.type === xo.type && yi.format === xo.format && yi.internalformat === xo.internalformat,
                "incompatible format for texture.subimage"
              ), vt(
                Ua >= 0 && La >= 0 && Ua + xo.width <= yi.width && La + xo.height <= yi.height,
                "texture.subimage write out of bounds"
              ), vt(
                yi.mipmask & 1 << yo,
                "missing mipmap data"
              ), vt(
                xo.data || xo.element || xo.needsCopy,
                "missing image data"
              ), Ad(yi), wl(xo, Xl + fa, Ua, La, yo), Fc(), ns(xo), Ma;
            }
            function ba(fa) {
              var Ea = fa | 0;
              if (Ea !== yi.width) {
                Ma.width = yi.width = Ea, Ma.height = yi.height = Ea, Ad(yi);
                for (var Hi = 0; Hi < 6; ++Hi)
                  for (var ji = 0; yi.mipmask >> ji; ++ji)
                    pr.texImage2D(
                      Xl + Hi,
                      ji,
                      yi.format,
                      Ea >> ji,
                      Ea >> ji,
                      0,
                      yi.format,
                      yi.type,
                      null
                    );
                return Fc(), En.profile && (yi.stats.size = Cc(
                  yi.internalformat,
                  yi.type,
                  Ma.width,
                  Ma.height,
                  !1,
                  !0
                )), Ma;
              }
            }
            return Ma(Xi, Za, ja, Ds, rc, Di), Ma.subimage = Ia, Ma.resize = ba, Ma._reglType = "textureCube", Ma._texture = yi, En.profile && (Ma.stats = yi.stats), Ma.destroy = function() {
              yi.decRef();
            }, Ma;
          }
          function U1() {
            for (var Xi = 0; Xi < Ed; ++Xi)
              pr.activeTexture(Oo + Xi), pr.bindTexture(Vl, null), R1[Xi] = null;
            Ca(ud).forEach(yl), za.cubeCount = 0, za.textureCount = 0;
          }
          En.profile && (za.getTotalTextureSize = function() {
            var Xi = 0;
            return Object.keys(ud).forEach(function(Za) {
              Xi += ud[Za].stats.size;
            }), Xi;
          });
          function uf() {
            for (var Xi = 0; Xi < Ed; ++Xi) {
              var Za = R1[Xi];
              Za && (Za.bindCount = 0, Za.unit = -1, R1[Xi] = null);
            }
            Ca(ud).forEach(function(ja) {
              ja.texture = pr.createTexture(), pr.bindTexture(ja.target, ja.texture);
              for (var Ds = 0; Ds < 32; ++Ds)
                if (ja.mipmask & 1 << Ds)
                  if (ja.target === Vl)
                    pr.texImage2D(
                      Vl,
                      Ds,
                      ja.internalformat,
                      ja.width >> Ds,
                      ja.height >> Ds,
                      0,
                      ja.internalformat,
                      ja.type,
                      null
                    );
                  else
                    for (var rc = 0; rc < 6; ++rc)
                      pr.texImage2D(
                        Xl + rc,
                        Ds,
                        ja.internalformat,
                        ja.width >> Ds,
                        ja.height >> Ds,
                        0,
                        ja.internalformat,
                        ja.type,
                        null
                      );
              Qu(ja.texInfo, ja.target);
            });
          }
          function df() {
            for (var Xi = 0; Xi < Ed; ++Xi) {
              var Za = R1[Xi];
              Za && (Za.bindCount = 0, Za.unit = -1, R1[Xi] = null), pr.activeTexture(Oo + Xi), pr.bindTexture(Vl, null), pr.bindTexture(Ll, null);
            }
          }
          return {
            create2D: cc,
            createCube: Bc,
            clear: U1,
            getTexture: function(Xi) {
              return null;
            },
            restore: uf,
            refresh: df
          };
        }
        var uc = 36161, cs = 32854, Mu = 32855, Hu = 36194, ou = 33189, Gc = 36168, Su = 34041, Lc = 35907, Do = 34836, Ru = 34842, dc = 34843, vl = [];
        vl[cs] = 2, vl[Mu] = 2, vl[Hu] = 2, vl[ou] = 2, vl[Gc] = 1, vl[Su] = 4, vl[Lc] = 4, vl[Do] = 16, vl[Ru] = 8, vl[dc] = 6;
        function Eu(pr, oi, ar) {
          return vl[pr] * oi * ar;
        }
        var fu = function(pr, oi, ar, Sa, Qn) {
          var za = {
            rgba4: cs,
            rgb565: Hu,
            "rgb5 a1": Mu,
            depth: ou,
            stencil: Gc,
            "depth stencil": Su
          };
          oi.ext_srgb && (za.srgba = Lc), oi.ext_color_buffer_half_float && (za.rgba16f = Ru, za.rgb16f = dc), oi.webgl_color_buffer_float && (za.rgba32f = Do);
          var En = [];
          Object.keys(za).forEach(function(Ao) {
            var Ro = za[Ao];
            En[Ro] = Ao;
          });
          var Qo = 0, lo = {};
          function Eo(Ao) {
            this.id = Qo++, this.refCount = 1, this.renderbuffer = Ao, this.format = cs, this.width = 0, this.height = 0, Qn.profile && (this.stats = { size: 0 });
          }
          Eo.prototype.decRef = function() {
            --this.refCount <= 0 && bo(this);
          };
          function bo(Ao) {
            var Ro = Ao.renderbuffer;
            vt(Ro, "must not double destroy renderbuffer"), pr.bindRenderbuffer(uc, null), pr.deleteRenderbuffer(Ro), Ao.renderbuffer = null, Ao.refCount = 0, delete lo[Ao.id], Sa.renderbufferCount--;
          }
          function Ho(Ao, Ro) {
            var Qa = new Eo(pr.createRenderbuffer());
            lo[Qa.id] = Qa, Sa.renderbufferCount++;
            function Ka(Ss, ho) {
              var hs = 0, Bo = 0, Vs = cs;
              if (typeof Ss == "object" && Ss) {
                var vo = Ss;
                if ("shape" in vo) {
                  var go = vo.shape;
                  vt(
                    Array.isArray(go) && go.length >= 2,
                    "invalid renderbuffer shape"
                  ), hs = go[0] | 0, Bo = go[1] | 0;
                } else
                  "radius" in vo && (hs = Bo = vo.radius | 0), "width" in vo && (hs = vo.width | 0), "height" in vo && (Bo = vo.height | 0);
                "format" in vo && (vt.parameter(
                  vo.format,
                  za,
                  "invalid renderbuffer format"
                ), Vs = za[vo.format]);
              } else typeof Ss == "number" ? (hs = Ss | 0, typeof ho == "number" ? Bo = ho | 0 : Bo = hs) : Ss ? vt.raise("invalid arguments to renderbuffer constructor") : hs = Bo = 1;
              if (vt(
                hs > 0 && Bo > 0 && hs <= ar.maxRenderbufferSize && Bo <= ar.maxRenderbufferSize,
                "invalid renderbuffer size"
              ), !(hs === Qa.width && Bo === Qa.height && Vs === Qa.format))
                return Ka.width = Qa.width = hs, Ka.height = Qa.height = Bo, Qa.format = Vs, pr.bindRenderbuffer(uc, Qa.renderbuffer), pr.renderbufferStorage(uc, Vs, hs, Bo), vt(
                  pr.getError() === 0,
                  "invalid render buffer format"
                ), Qn.profile && (Qa.stats.size = Eu(Qa.format, Qa.width, Qa.height)), Ka.format = En[Qa.format], Ka;
            }
            function qo(Ss, ho) {
              var hs = Ss | 0, Bo = ho | 0 || hs;
              return hs === Qa.width && Bo === Qa.height || (vt(
                hs > 0 && Bo > 0 && hs <= ar.maxRenderbufferSize && Bo <= ar.maxRenderbufferSize,
                "invalid renderbuffer size"
              ), Ka.width = Qa.width = hs, Ka.height = Qa.height = Bo, pr.bindRenderbuffer(uc, Qa.renderbuffer), pr.renderbufferStorage(uc, Qa.format, hs, Bo), vt(
                pr.getError() === 0,
                "invalid render buffer format"
              ), Qn.profile && (Qa.stats.size = Eu(
                Qa.format,
                Qa.width,
                Qa.height
              ))), Ka;
            }
            return Ka(Ao, Ro), Ka.resize = qo, Ka._reglType = "renderbuffer", Ka._renderbuffer = Qa, Qn.profile && (Ka.stats = Qa.stats), Ka.destroy = function() {
              Qa.decRef();
            }, Ka;
          }
          Qn.profile && (Sa.getTotalRenderbufferSize = function() {
            var Ao = 0;
            return Object.keys(lo).forEach(function(Ro) {
              Ao += lo[Ro].stats.size;
            }), Ao;
          });
          function Co() {
            Ca(lo).forEach(function(Ao) {
              Ao.renderbuffer = pr.createRenderbuffer(), pr.bindRenderbuffer(uc, Ao.renderbuffer), pr.renderbufferStorage(uc, Ao.format, Ao.width, Ao.height);
            }), pr.bindRenderbuffer(uc, null);
          }
          return {
            create: Ho,
            clear: function() {
              Ca(lo).forEach(bo);
            },
            restore: Co
          };
        }, ac = 36160, bc = 36161, kc = 3553, fc = 34069, oc = 36064, ss = 36096, su = 36128, Tl = 33306, sc = 36053, hc = 36054, Vc = 36055, Rc = 36057, Zo = 36061, Ou = 36193, Uc = 5121, El = 5126, eu = 6407, Wc = 6408, hu = 6402, gd = [
          eu,
          Wc
        ], Jl = [];
        Jl[Wc] = 4, Jl[eu] = 3;
        var Yc = [];
        Yc[Uc] = 1, Yc[El] = 4, Yc[Ou] = 2;
        var nc = 32854, Ko = 32855, ks = 36194, qs = 33189, a1 = 36168, $l = 34041, o1 = 35907, $c = 34836, lu = 34842, Ul = 34843, zs = [
          nc,
          Ko,
          ks,
          o1,
          lu,
          Ul,
          $c
        ], Os = {};
        Os[sc] = "complete", Os[hc] = "incomplete attachment", Os[Rc] = "incomplete dimensions", Os[Vc] = "incomplete, missing attachment", Os[Zo] = "unsupported";
        function lc(pr, oi, ar, Sa, Qn, za) {
          var En = {
            cur: null,
            next: null,
            dirty: !1,
            setFBO: null
          }, Qo = ["rgba"], lo = ["rgba4", "rgb565", "rgb5 a1"];
          oi.ext_srgb && lo.push("srgba"), oi.ext_color_buffer_half_float && lo.push("rgba16f", "rgb16f"), oi.webgl_color_buffer_float && lo.push("rgba32f");
          var Eo = ["uint8"];
          oi.oes_texture_half_float && Eo.push("half float", "float16"), oi.oes_texture_float && Eo.push("float", "float32");
          function bo(Po, Jo, ps) {
            this.target = Po, this.texture = Jo, this.renderbuffer = ps;
            var wl = 0, ol = 0;
            Jo ? (wl = Jo.width, ol = Jo.height) : ps && (wl = ps.width, ol = ps.height), this.width = wl, this.height = ol;
          }
          function Ho(Po) {
            Po && (Po.texture && Po.texture._texture.decRef(), Po.renderbuffer && Po.renderbuffer._renderbuffer.decRef());
          }
          function Co(Po, Jo, ps) {
            if (Po)
              if (Po.texture) {
                var wl = Po.texture._texture, ol = Math.max(1, wl.width), xs = Math.max(1, wl.height);
                vt(
                  ol === Jo && xs === ps,
                  "inconsistent width/height for supplied texture"
                ), wl.refCount += 1;
              } else {
                var ns = Po.renderbuffer._renderbuffer;
                vt(
                  ns.width === Jo && ns.height === ps,
                  "inconsistent width/height for renderbuffer"
                ), ns.refCount += 1;
              }
          }
          function Ao(Po, Jo) {
            Jo && (Jo.texture ? pr.framebufferTexture2D(
              ac,
              Po,
              Jo.target,
              Jo.texture._texture.texture,
              0
            ) : pr.framebufferRenderbuffer(
              ac,
              Po,
              bc,
              Jo.renderbuffer._renderbuffer.renderbuffer
            ));
          }
          function Ro(Po) {
            var Jo = kc, ps = null, wl = null, ol = Po;
            typeof Po == "object" && (ol = Po.data, "target" in Po && (Jo = Po.target | 0)), vt.type(ol, "function", "invalid attachment data");
            var xs = ol._reglType;
            return xs === "texture2d" ? (ps = ol, vt(Jo === kc)) : xs === "textureCube" ? (ps = ol, vt(
              Jo >= fc && Jo < fc + 6,
              "invalid cube map target"
            )) : xs === "renderbuffer" ? (wl = ol, Jo = bc) : vt.raise("invalid regl object for attachment"), new bo(Jo, ps, wl);
          }
          function Qa(Po, Jo, ps, wl, ol) {
            if (ps) {
              var xs = Sa.create2D({
                width: Po,
                height: Jo,
                format: wl,
                type: ol
              });
              return xs._texture.refCount = 0, new bo(kc, xs, null);
            } else {
              var ns = Qn.create({
                width: Po,
                height: Jo,
                format: wl
              });
              return ns._renderbuffer.refCount = 0, new bo(bc, null, ns);
            }
          }
          function Ka(Po) {
            return Po && (Po.texture || Po.renderbuffer);
          }
          function qo(Po, Jo, ps) {
            Po && (Po.texture ? Po.texture.resize(Jo, ps) : Po.renderbuffer && Po.renderbuffer.resize(Jo, ps), Po.width = Jo, Po.height = ps);
          }
          var Ss = 0, ho = {};
          function hs() {
            this.id = Ss++, ho[this.id] = this, this.framebuffer = pr.createFramebuffer(), this.width = 0, this.height = 0, this.colorAttachments = [], this.depthAttachment = null, this.stencilAttachment = null, this.depthStencilAttachment = null;
          }
          function Bo(Po) {
            Po.colorAttachments.forEach(Ho), Ho(Po.depthAttachment), Ho(Po.stencilAttachment), Ho(Po.depthStencilAttachment);
          }
          function Vs(Po) {
            var Jo = Po.framebuffer;
            vt(Jo, "must not double destroy framebuffer"), pr.deleteFramebuffer(Jo), Po.framebuffer = null, za.framebufferCount--, delete ho[Po.id];
          }
          function vo(Po) {
            var Jo;
            pr.bindFramebuffer(ac, Po.framebuffer);
            var ps = Po.colorAttachments;
            for (Jo = 0; Jo < ps.length; ++Jo)
              Ao(oc + Jo, ps[Jo]);
            for (Jo = ps.length; Jo < ar.maxColorAttachments; ++Jo)
              pr.framebufferTexture2D(
                ac,
                oc + Jo,
                kc,
                null,
                0
              );
            pr.framebufferTexture2D(
              ac,
              Tl,
              kc,
              null,
              0
            ), pr.framebufferTexture2D(
              ac,
              ss,
              kc,
              null,
              0
            ), pr.framebufferTexture2D(
              ac,
              su,
              kc,
              null,
              0
            ), Ao(ss, Po.depthAttachment), Ao(su, Po.stencilAttachment), Ao(Tl, Po.depthStencilAttachment);
            var wl = pr.checkFramebufferStatus(ac);
            !pr.isContextLost() && wl !== sc && vt.raise("framebuffer configuration not supported, status = " + Os[wl]), pr.bindFramebuffer(ac, En.next ? En.next.framebuffer : null), En.cur = En.next, pr.getError();
          }
          function go(Po, Jo) {
            var ps = new hs();
            za.framebufferCount++;
            function wl(xs, ns) {
              var ll;
              vt(
                En.next !== ps,
                "can not update framebuffer which is currently in use"
              );
              var Ws = 0, ec = 0, s1 = !0, n1 = !0, Js = null, N1 = !0, Xu = "rgba", Q1 = "uint8", Qu = 1, r1 = null, ud = null, Ed = null, R1 = !1;
              if (typeof xs == "number")
                Ws = xs | 0, ec = ns | 0 || Ws;
              else if (!xs)
                Ws = ec = 1;
              else {
                vt.type(xs, "object", "invalid arguments for framebuffer");
                var jl = xs;
                if ("shape" in jl) {
                  var Ad = jl.shape;
                  vt(
                    Array.isArray(Ad) && Ad.length >= 2,
                    "invalid shape for framebuffer"
                  ), Ws = Ad[0], ec = Ad[1];
                } else
                  "radius" in jl && (Ws = ec = jl.radius), "width" in jl && (Ws = jl.width), "height" in jl && (ec = jl.height);
                ("color" in jl || "colors" in jl) && (Js = jl.color || jl.colors, Array.isArray(Js) && vt(
                  Js.length === 1 || oi.webgl_draw_buffers,
                  "multiple render targets not supported"
                )), Js || ("colorCount" in jl && (Qu = jl.colorCount | 0, vt(Qu > 0, "invalid color buffer count")), "colorTexture" in jl && (N1 = !!jl.colorTexture, Xu = "rgba4"), "colorType" in jl && (Q1 = jl.colorType, N1 ? (vt(
                  oi.oes_texture_float || !(Q1 === "float" || Q1 === "float32"),
                  "you must enable OES_texture_float in order to use floating point framebuffer objects"
                ), vt(
                  oi.oes_texture_half_float || !(Q1 === "half float" || Q1 === "float16"),
                  "you must enable OES_texture_half_float in order to use 16-bit floating point framebuffer objects"
                )) : Q1 === "half float" || Q1 === "float16" ? (vt(
                  oi.ext_color_buffer_half_float,
                  "you must enable EXT_color_buffer_half_float to use 16-bit render buffers"
                ), Xu = "rgba16f") : (Q1 === "float" || Q1 === "float32") && (vt(
                  oi.webgl_color_buffer_float,
                  "you must enable WEBGL_color_buffer_float in order to use 32-bit floating point renderbuffers"
                ), Xu = "rgba32f"), vt.oneOf(Q1, Eo, "invalid color type")), "colorFormat" in jl && (Xu = jl.colorFormat, Qo.indexOf(Xu) >= 0 ? N1 = !0 : lo.indexOf(Xu) >= 0 ? N1 = !1 : vt.optional(function() {
                  N1 ? vt.oneOf(
                    jl.colorFormat,
                    Qo,
                    "invalid color format for texture"
                  ) : vt.oneOf(
                    jl.colorFormat,
                    lo,
                    "invalid color format for renderbuffer"
                  );
                }))), ("depthTexture" in jl || "depthStencilTexture" in jl) && (R1 = !!(jl.depthTexture || jl.depthStencilTexture), vt(
                  !R1 || oi.webgl_depth_texture,
                  "webgl_depth_texture extension not supported"
                )), "depth" in jl && (typeof jl.depth == "boolean" ? s1 = jl.depth : (r1 = jl.depth, n1 = !1)), "stencil" in jl && (typeof jl.stencil == "boolean" ? n1 = jl.stencil : (ud = jl.stencil, s1 = !1)), "depthStencil" in jl && (typeof jl.depthStencil == "boolean" ? s1 = n1 = jl.depthStencil : (Ed = jl.depthStencil, s1 = !1, n1 = !1));
              }
              var Fc = null, yl = null, cc = null, Bc = null;
              if (Array.isArray(Js))
                Fc = Js.map(Ro);
              else if (Js)
                Fc = [Ro(Js)];
              else
                for (Fc = new Array(Qu), ll = 0; ll < Qu; ++ll)
                  Fc[ll] = Qa(
                    Ws,
                    ec,
                    N1,
                    Xu,
                    Q1
                  );
              vt(
                oi.webgl_draw_buffers || Fc.length <= 1,
                "you must enable the WEBGL_draw_buffers extension in order to use multiple color buffers."
              ), vt(
                Fc.length <= ar.maxColorAttachments,
                "too many color attachments, not supported"
              ), Ws = Ws || Fc[0].width, ec = ec || Fc[0].height, r1 ? yl = Ro(r1) : s1 && !n1 && (yl = Qa(
                Ws,
                ec,
                R1,
                "depth",
                "uint32"
              )), ud ? cc = Ro(ud) : n1 && !s1 && (cc = Qa(
                Ws,
                ec,
                !1,
                "stencil",
                "uint8"
              )), Ed ? Bc = Ro(Ed) : !r1 && !ud && n1 && s1 && (Bc = Qa(
                Ws,
                ec,
                R1,
                "depth stencil",
                "depth stencil"
              )), vt(
                !!r1 + !!ud + !!Ed <= 1,
                "invalid framebuffer configuration, can specify exactly one depth/stencil attachment"
              );
              var U1 = null;
              for (ll = 0; ll < Fc.length; ++ll)
                if (Co(Fc[ll], Ws, ec), vt(
                  !Fc[ll] || Fc[ll].texture && gd.indexOf(Fc[ll].texture._texture.format) >= 0 || Fc[ll].renderbuffer && zs.indexOf(Fc[ll].renderbuffer._renderbuffer.format) >= 0,
                  "framebuffer color attachment " + ll + " is invalid"
                ), Fc[ll] && Fc[ll].texture) {
                  var uf = Jl[Fc[ll].texture._texture.format] * Yc[Fc[ll].texture._texture.type];
                  U1 === null ? U1 = uf : vt(
                    U1 === uf,
                    "all color attachments much have the same number of bits per pixel."
                  );
                }
              return Co(yl, Ws, ec), vt(
                !yl || yl.texture && yl.texture._texture.format === hu || yl.renderbuffer && yl.renderbuffer._renderbuffer.format === qs,
                "invalid depth attachment for framebuffer object"
              ), Co(cc, Ws, ec), vt(
                !cc || cc.renderbuffer && cc.renderbuffer._renderbuffer.format === a1,
                "invalid stencil attachment for framebuffer object"
              ), Co(Bc, Ws, ec), vt(
                !Bc || Bc.texture && Bc.texture._texture.format === $l || Bc.renderbuffer && Bc.renderbuffer._renderbuffer.format === $l,
                "invalid depth-stencil attachment for framebuffer object"
              ), Bo(ps), ps.width = Ws, ps.height = ec, ps.colorAttachments = Fc, ps.depthAttachment = yl, ps.stencilAttachment = cc, ps.depthStencilAttachment = Bc, wl.color = Fc.map(Ka), wl.depth = Ka(yl), wl.stencil = Ka(cc), wl.depthStencil = Ka(Bc), wl.width = ps.width, wl.height = ps.height, vo(ps), wl;
            }
            function ol(xs, ns) {
              vt(
                En.next !== ps,
                "can not resize a framebuffer which is currently in use"
              );
              var ll = Math.max(xs | 0, 1), Ws = Math.max(ns | 0 || ll, 1);
              if (ll === ps.width && Ws === ps.height)
                return wl;
              for (var ec = ps.colorAttachments, s1 = 0; s1 < ec.length; ++s1)
                qo(ec[s1], ll, Ws);
              return qo(ps.depthAttachment, ll, Ws), qo(ps.stencilAttachment, ll, Ws), qo(ps.depthStencilAttachment, ll, Ws), ps.width = wl.width = ll, ps.height = wl.height = Ws, vo(ps), wl;
            }
            return wl(Po, Jo), V(wl, {
              resize: ol,
              _reglType: "framebuffer",
              _framebuffer: ps,
              destroy: function() {
                Vs(ps), Bo(ps);
              },
              use: function(xs) {
                En.setFBO({
                  framebuffer: wl
                }, xs);
              }
            });
          }
          function Qs(Po) {
            var Jo = Array(6);
            function ps(ol) {
              var xs;
              vt(
                Jo.indexOf(En.next) < 0,
                "can not update framebuffer which is currently in use"
              );
              var ns = {
                color: null
              }, ll = 0, Ws = null, ec = "rgba", s1 = "uint8", n1 = 1;
              if (typeof ol == "number")
                ll = ol | 0;
              else if (!ol)
                ll = 1;
              else {
                vt.type(ol, "object", "invalid arguments for framebuffer");
                var Js = ol;
                if ("shape" in Js) {
                  var N1 = Js.shape;
                  vt(
                    Array.isArray(N1) && N1.length >= 2,
                    "invalid shape for framebuffer"
                  ), vt(
                    N1[0] === N1[1],
                    "cube framebuffer must be square"
                  ), ll = N1[0];
                } else
                  "radius" in Js && (ll = Js.radius | 0), "width" in Js ? (ll = Js.width | 0, "height" in Js && vt(Js.height === ll, "must be square")) : "height" in Js && (ll = Js.height | 0);
                ("color" in Js || "colors" in Js) && (Ws = Js.color || Js.colors, Array.isArray(Ws) && vt(
                  Ws.length === 1 || oi.webgl_draw_buffers,
                  "multiple render targets not supported"
                )), Ws || ("colorCount" in Js && (n1 = Js.colorCount | 0, vt(n1 > 0, "invalid color buffer count")), "colorType" in Js && (vt.oneOf(
                  Js.colorType,
                  Eo,
                  "invalid color type"
                ), s1 = Js.colorType), "colorFormat" in Js && (ec = Js.colorFormat, vt.oneOf(
                  Js.colorFormat,
                  Qo,
                  "invalid color format for texture"
                ))), "depth" in Js && (ns.depth = Js.depth), "stencil" in Js && (ns.stencil = Js.stencil), "depthStencil" in Js && (ns.depthStencil = Js.depthStencil);
              }
              var Xu;
              if (Ws)
                if (Array.isArray(Ws))
                  for (Xu = [], xs = 0; xs < Ws.length; ++xs)
                    Xu[xs] = Ws[xs];
                else
                  Xu = [Ws];
              else {
                Xu = Array(n1);
                var Q1 = {
                  radius: ll,
                  format: ec,
                  type: s1
                };
                for (xs = 0; xs < n1; ++xs)
                  Xu[xs] = Sa.createCube(Q1);
              }
              for (ns.color = Array(Xu.length), xs = 0; xs < Xu.length; ++xs) {
                var Qu = Xu[xs];
                vt(
                  typeof Qu == "function" && Qu._reglType === "textureCube",
                  "invalid cube map"
                ), ll = ll || Qu.width, vt(
                  Qu.width === ll && Qu.height === ll,
                  "invalid cube map shape"
                ), ns.color[xs] = {
                  target: fc,
                  data: Xu[xs]
                };
              }
              for (xs = 0; xs < 6; ++xs) {
                for (var r1 = 0; r1 < Xu.length; ++r1)
                  ns.color[r1].target = fc + xs;
                xs > 0 && (ns.depth = Jo[0].depth, ns.stencil = Jo[0].stencil, ns.depthStencil = Jo[0].depthStencil), Jo[xs] ? Jo[xs](ns) : Jo[xs] = go(ns);
              }
              return V(ps, {
                width: ll,
                height: ll,
                color: Xu
              });
            }
            function wl(ol) {
              var xs, ns = ol | 0;
              if (vt(
                ns > 0 && ns <= ar.maxCubeMapSize,
                "invalid radius for cube fbo"
              ), ns === ps.width)
                return ps;
              var ll = ps.color;
              for (xs = 0; xs < ll.length; ++xs)
                ll[xs].resize(ns);
              for (xs = 0; xs < 6; ++xs)
                Jo[xs].resize(ns);
              return ps.width = ps.height = ns, ps;
            }
            return ps(Po), V(ps, {
              faces: Jo,
              resize: wl,
              _reglType: "framebufferCube",
              destroy: function() {
                Jo.forEach(function(ol) {
                  ol.destroy();
                });
              }
            });
          }
          function us() {
            En.cur = null, En.next = null, En.dirty = !0, Ca(ho).forEach(function(Po) {
              Po.framebuffer = pr.createFramebuffer(), vo(Po);
            });
          }
          return V(En, {
            getFramebuffer: function(Po) {
              if (typeof Po == "function" && Po._reglType === "framebuffer") {
                var Jo = Po._framebuffer;
                if (Jo instanceof hs)
                  return Jo;
              }
              return null;
            },
            create: go,
            createCube: Qs,
            clear: function() {
              Ca(ho).forEach(Vs);
            },
            restore: us
          });
        }
        var xc = 5126, Iu = 34962, tu = 34963, w1 = [
          "attributes",
          "elements",
          "offset",
          "count",
          "primitive",
          "instances"
        ];
        function $n() {
          this.state = 0, this.x = 0, this.y = 0, this.z = 0, this.w = 0, this.buffer = null, this.size = 0, this.normalized = !1, this.type = xc, this.offset = 0, this.stride = 0, this.divisor = 0;
        }
        function zn(pr, oi, ar, Sa, Qn, za, En) {
          for (var Qo = ar.maxAttributes, lo = new Array(Qo), Eo = 0; Eo < Qo; ++Eo)
            lo[Eo] = new $n();
          var bo = 0, Ho = {}, Co = {
            Record: $n,
            scope: {},
            state: lo,
            currentVAO: null,
            targetVAO: null,
            restore: Ro() ? Bo : function() {
            },
            createVAO: Vs,
            getVAO: Ka,
            destroyBuffer: Ao,
            setVAO: Ro() ? qo : Ss,
            clear: Ro() ? ho : function() {
            }
          };
          function Ao(vo) {
            for (var go = 0; go < lo.length; ++go) {
              var Qs = lo[go];
              Qs.buffer === vo && (pr.disableVertexAttribArray(go), Qs.buffer = null);
            }
          }
          function Ro() {
            return oi.oes_vertex_array_object;
          }
          function Qa() {
            return oi.angle_instanced_arrays;
          }
          function Ka(vo) {
            return typeof vo == "function" && vo._vao ? vo._vao : null;
          }
          function qo(vo) {
            if (vo !== Co.currentVAO) {
              var go = Ro();
              vo ? go.bindVertexArrayOES(vo.vao) : go.bindVertexArrayOES(null), Co.currentVAO = vo;
            }
          }
          function Ss(vo) {
            if (vo !== Co.currentVAO) {
              if (vo)
                vo.bindAttrs();
              else {
                for (var go = Qa(), Qs = 0; Qs < lo.length; ++Qs) {
                  var us = lo[Qs];
                  us.buffer ? (pr.enableVertexAttribArray(Qs), us.buffer.bind(), pr.vertexAttribPointer(Qs, us.size, us.type, us.normalized, us.stride, us.offfset), go && us.divisor && go.vertexAttribDivisorANGLE(Qs, us.divisor)) : (pr.disableVertexAttribArray(Qs), pr.vertexAttrib4f(Qs, us.x, us.y, us.z, us.w));
                }
                En.elements ? pr.bindBuffer(tu, En.elements.buffer.buffer) : pr.bindBuffer(tu, null);
              }
              Co.currentVAO = vo;
            }
          }
          function ho() {
            Ca(Ho).forEach(function(vo) {
              vo.destroy();
            });
          }
          function hs() {
            this.id = ++bo, this.attributes = [], this.elements = null, this.ownsElements = !1, this.count = 0, this.offset = 0, this.instances = -1, this.primitive = 4;
            var vo = Ro();
            vo ? this.vao = vo.createVertexArrayOES() : this.vao = null, Ho[this.id] = this, this.buffers = [];
          }
          hs.prototype.bindAttrs = function() {
            for (var vo = Qa(), go = this.attributes, Qs = 0; Qs < go.length; ++Qs) {
              var us = go[Qs];
              us.buffer ? (pr.enableVertexAttribArray(Qs), pr.bindBuffer(Iu, us.buffer.buffer), pr.vertexAttribPointer(Qs, us.size, us.type, us.normalized, us.stride, us.offset), vo && us.divisor && vo.vertexAttribDivisorANGLE(Qs, us.divisor)) : (pr.disableVertexAttribArray(Qs), pr.vertexAttrib4f(Qs, us.x, us.y, us.z, us.w));
            }
            for (var Po = go.length; Po < Qo; ++Po)
              pr.disableVertexAttribArray(Po);
            var Jo = za.getElements(this.elements);
            Jo ? pr.bindBuffer(tu, Jo.buffer.buffer) : pr.bindBuffer(tu, null);
          }, hs.prototype.refresh = function() {
            var vo = Ro();
            vo && (vo.bindVertexArrayOES(this.vao), this.bindAttrs(), Co.currentVAO = null, vo.bindVertexArrayOES(null));
          }, hs.prototype.destroy = function() {
            if (this.vao) {
              var vo = Ro();
              this === Co.currentVAO && (Co.currentVAO = null, vo.bindVertexArrayOES(null)), vo.deleteVertexArrayOES(this.vao), this.vao = null;
            }
            this.ownsElements && (this.elements.destroy(), this.elements = null, this.ownsElements = !1), Ho[this.id] && (delete Ho[this.id], Sa.vaoCount -= 1);
          };
          function Bo() {
            var vo = Ro();
            vo && Ca(Ho).forEach(function(go) {
              go.refresh();
            });
          }
          function Vs(vo) {
            var go = new hs();
            Sa.vaoCount += 1;
            function Qs(us) {
              var Po;
              if (Array.isArray(us))
                Po = us, go.elements && go.ownsElements && go.elements.destroy(), go.elements = null, go.ownsElements = !1, go.offset = 0, go.count = 0, go.instances = -1, go.primitive = 4;
              else {
                if (vt(typeof us == "object", "invalid arguments for create vao"), vt("attributes" in us, "must specify attributes for vao"), us.elements) {
                  var Jo = us.elements;
                  go.ownsElements ? typeof Jo == "function" && Jo._reglType === "elements" ? (go.elements.destroy(), go.ownsElements = !1) : (go.elements(Jo), go.ownsElements = !1) : za.getElements(us.elements) ? (go.elements = us.elements, go.ownsElements = !1) : (go.elements = za.create(us.elements), go.ownsElements = !0);
                } else
                  go.elements = null, go.ownsElements = !1;
                Po = us.attributes, go.offset = 0, go.count = -1, go.instances = -1, go.primitive = 4, go.elements && (go.count = go.elements._elements.vertCount, go.primitive = go.elements._elements.primType), "offset" in us && (go.offset = us.offset | 0), "count" in us && (go.count = us.count | 0), "instances" in us && (go.instances = us.instances | 0), "primitive" in us && (vt(us.primitive in Tn, "bad primitive type: " + us.primitive), go.primitive = Tn[us.primitive]), vt.optional(() => {
                  for (var s1 = Object.keys(us), n1 = 0; n1 < s1.length; ++n1)
                    vt(w1.indexOf(s1[n1]) >= 0, 'invalid option for vao: "' + s1[n1] + '" valid options are ' + w1);
                }), vt(Array.isArray(Po), "attributes must be an array");
              }
              vt(Po.length < Qo, "too many attributes"), vt(Po.length > 0, "must specify at least one attribute");
              var ps = {}, wl = go.attributes;
              wl.length = Po.length;
              for (var ol = 0; ol < Po.length; ++ol) {
                var xs = Po[ol], ns = wl[ol] = new $n(), ll = xs.data || xs;
                if (Array.isArray(ll) || H(ll) || Gr(ll)) {
                  var Ws;
                  go.buffers[ol] && (Ws = go.buffers[ol], H(ll) && Ws._buffer.byteLength >= ll.byteLength ? Ws.subdata(ll) : (Ws.destroy(), go.buffers[ol] = null)), go.buffers[ol] || (Ws = go.buffers[ol] = Qn.create(xs, Iu, !1, !0)), ns.buffer = Qn.getBuffer(Ws), ns.size = ns.buffer.dimension | 0, ns.normalized = !1, ns.type = ns.buffer.dtype, ns.offset = 0, ns.stride = 0, ns.divisor = 0, ns.state = 1, ps[ol] = 1;
                } else Qn.getBuffer(xs) ? (ns.buffer = Qn.getBuffer(xs), ns.size = ns.buffer.dimension | 0, ns.normalized = !1, ns.type = ns.buffer.dtype, ns.offset = 0, ns.stride = 0, ns.divisor = 0, ns.state = 1) : Qn.getBuffer(xs.buffer) ? (ns.buffer = Qn.getBuffer(xs.buffer), ns.size = (+xs.size || ns.buffer.dimension) | 0, ns.normalized = !!xs.normalized || !1, "type" in xs ? (vt.parameter(xs.type, Jr, "invalid buffer type"), ns.type = Jr[xs.type]) : ns.type = ns.buffer.dtype, ns.offset = (xs.offset || 0) | 0, ns.stride = (xs.stride || 0) | 0, ns.divisor = (xs.divisor || 0) | 0, ns.state = 1, vt(ns.size >= 1 && ns.size <= 4, "size must be between 1 and 4"), vt(ns.offset >= 0, "invalid offset"), vt(ns.stride >= 0 && ns.stride <= 255, "stride must be between 0 and 255"), vt(ns.divisor >= 0, "divisor must be positive"), vt(!ns.divisor || !!oi.angle_instanced_arrays, "ANGLE_instanced_arrays must be enabled to use divisor")) : "x" in xs ? (vt(ol > 0, "first attribute must not be a constant"), ns.x = +xs.x || 0, ns.y = +xs.y || 0, ns.z = +xs.z || 0, ns.w = +xs.w || 0, ns.state = 2) : vt(!1, "invalid attribute spec for location " + ol);
              }
              for (var ec = 0; ec < go.buffers.length; ++ec)
                !ps[ec] && go.buffers[ec] && (go.buffers[ec].destroy(), go.buffers[ec] = null);
              return go.refresh(), Qs;
            }
            return Qs.destroy = function() {
              for (var us = 0; us < go.buffers.length; ++us)
                go.buffers[us] && go.buffers[us].destroy();
              go.buffers.length = 0, go.ownsElements && (go.elements.destroy(), go.elements = null, go.ownsElements = !1), go.destroy();
            }, Qs._vao = go, Qs._reglType = "vao", Qs(vo);
          }
          return Co;
        }
        var Gn = 35632, Ke = 35633, Fe = 35718, Te = 35721;
        function Ae(pr, oi, ar, Sa) {
          var Qn = {}, za = {};
          function En(Qa, Ka, qo, Ss) {
            this.name = Qa, this.id = Ka, this.location = qo, this.info = Ss;
          }
          function Qo(Qa, Ka) {
            for (var qo = 0; qo < Qa.length; ++qo)
              if (Qa[qo].id === Ka.id) {
                Qa[qo].location = Ka.location;
                return;
              }
            Qa.push(Ka);
          }
          function lo(Qa, Ka, qo) {
            var Ss = Qa === Gn ? Qn : za, ho = Ss[Ka];
            if (!ho) {
              var hs = oi.str(Ka);
              ho = pr.createShader(Qa), pr.shaderSource(ho, hs), pr.compileShader(ho), vt.shaderError(pr, ho, hs, Qa, qo), Ss[Ka] = ho;
            }
            return ho;
          }
          var Eo = {}, bo = [], Ho = 0;
          function Co(Qa, Ka) {
            this.id = Ho++, this.fragId = Qa, this.vertId = Ka, this.program = null, this.uniforms = [], this.attributes = [], this.refCount = 1, Sa.profile && (this.stats = {
              uniformsCount: 0,
              attributesCount: 0
            });
          }
          function Ao(Qa, Ka, qo) {
            var Ss, ho, hs = lo(Gn, Qa.fragId), Bo = lo(Ke, Qa.vertId), Vs = Qa.program = pr.createProgram();
            if (pr.attachShader(Vs, hs), pr.attachShader(Vs, Bo), qo)
              for (Ss = 0; Ss < qo.length; ++Ss) {
                var vo = qo[Ss];
                pr.bindAttribLocation(Vs, vo[0], vo[1]);
              }
            pr.linkProgram(Vs), vt.linkError(
              pr,
              Vs,
              oi.str(Qa.fragId),
              oi.str(Qa.vertId),
              Ka
            );
            var go = pr.getProgramParameter(Vs, Fe);
            Sa.profile && (Qa.stats.uniformsCount = go);
            var Qs = Qa.uniforms;
            for (Ss = 0; Ss < go; ++Ss)
              if (ho = pr.getActiveUniform(Vs, Ss), ho) {
                if (ho.size > 1)
                  for (var us = 0; us < ho.size; ++us) {
                    var Po = ho.name.replace("[0]", "[" + us + "]");
                    Qo(Qs, new En(
                      Po,
                      oi.id(Po),
                      pr.getUniformLocation(Vs, Po),
                      ho
                    ));
                  }
                var Jo = ho.name;
                ho.size > 1 && (Jo = Jo.replace("[0]", "")), Qo(Qs, new En(
                  Jo,
                  oi.id(Jo),
                  pr.getUniformLocation(Vs, Jo),
                  ho
                ));
              }
            var ps = pr.getProgramParameter(Vs, Te);
            Sa.profile && (Qa.stats.attributesCount = ps);
            var wl = Qa.attributes;
            for (Ss = 0; Ss < ps; ++Ss)
              ho = pr.getActiveAttrib(Vs, Ss), ho && Qo(wl, new En(
                ho.name,
                oi.id(ho.name),
                pr.getAttribLocation(Vs, ho.name),
                ho
              ));
          }
          Sa.profile && (ar.getMaxUniformsCount = function() {
            var Qa = 0;
            return bo.forEach(function(Ka) {
              Ka.stats.uniformsCount > Qa && (Qa = Ka.stats.uniformsCount);
            }), Qa;
          }, ar.getMaxAttributesCount = function() {
            var Qa = 0;
            return bo.forEach(function(Ka) {
              Ka.stats.attributesCount > Qa && (Qa = Ka.stats.attributesCount);
            }), Qa;
          });
          function Ro() {
            Qn = {}, za = {};
            for (var Qa = 0; Qa < bo.length; ++Qa)
              Ao(bo[Qa], null, bo[Qa].attributes.map(function(Ka) {
                return [Ka.location, Ka.name];
              }));
          }
          return {
            clear: function() {
              var Qa = pr.deleteShader.bind(pr);
              Ca(Qn).forEach(Qa), Qn = {}, Ca(za).forEach(Qa), za = {}, bo.forEach(function(Ka) {
                pr.deleteProgram(Ka.program);
              }), bo.length = 0, Eo = {}, ar.shaderCount = 0;
            },
            program: function(Qa, Ka, qo, Ss) {
              vt.command(Qa >= 0, "missing vertex shader", qo), vt.command(Ka >= 0, "missing fragment shader", qo);
              var ho = Eo[Ka];
              ho || (ho = Eo[Ka] = {});
              var hs = ho[Qa];
              if (hs && (hs.refCount++, !Ss))
                return hs;
              var Bo = new Co(Ka, Qa);
              return ar.shaderCount++, Ao(Bo, qo, Ss), hs || (ho[Qa] = Bo), bo.push(Bo), V(Bo, {
                destroy: function() {
                  if (Bo.refCount--, Bo.refCount <= 0) {
                    pr.deleteProgram(Bo.program);
                    var Vs = bo.indexOf(Bo);
                    bo.splice(Vs, 1), ar.shaderCount--;
                  }
                  ho[Bo.vertId].refCount <= 0 && (pr.deleteShader(za[Bo.vertId]), delete za[Bo.vertId], delete Eo[Bo.fragId][Bo.vertId]), Object.keys(Eo[Bo.fragId]).length || (pr.deleteShader(Qn[Bo.fragId]), delete Qn[Bo.fragId], delete Eo[Bo.fragId]);
                }
              });
            },
            restore: Ro,
            shader: lo,
            frag: -1,
            vert: -1
          };
        }
        var Oe = 6408, Be = 5121, Ue = 3333, nt = 5126;
        function ht(pr, oi, ar, Sa, Qn, za, En) {
          function Qo(bo) {
            var Ho;
            oi.next === null ? (vt(
              Qn.preserveDrawingBuffer,
              'you must create a webgl context with "preserveDrawingBuffer":true in order to read pixels from the drawing buffer'
            ), Ho = Be) : (vt(
              oi.next.colorAttachments[0].texture !== null,
              "You cannot read from a renderbuffer"
            ), Ho = oi.next.colorAttachments[0].texture._texture.type, vt.optional(function() {
              za.oes_texture_float ? (vt(
                Ho === Be || Ho === nt,
                "Reading from a framebuffer is only allowed for the types 'uint8' and 'float'"
              ), Ho === nt && vt(En.readFloat, "Reading 'float' values is not permitted in your browser. For a fallback, please see: https://www.npmjs.com/package/glsl-read-float")) : vt(
                Ho === Be,
                "Reading from a framebuffer is only allowed for the type 'uint8'"
              );
            }));
            var Co = 0, Ao = 0, Ro = Sa.framebufferWidth, Qa = Sa.framebufferHeight, Ka = null;
            H(bo) ? Ka = bo : bo && (vt.type(bo, "object", "invalid arguments to regl.read()"), Co = bo.x | 0, Ao = bo.y | 0, vt(
              Co >= 0 && Co < Sa.framebufferWidth,
              "invalid x offset for regl.read"
            ), vt(
              Ao >= 0 && Ao < Sa.framebufferHeight,
              "invalid y offset for regl.read"
            ), Ro = (bo.width || Sa.framebufferWidth - Co) | 0, Qa = (bo.height || Sa.framebufferHeight - Ao) | 0, Ka = bo.data || null), Ka && (Ho === Be ? vt(
              Ka instanceof Uint8Array,
              "buffer must be 'Uint8Array' when reading from a framebuffer of type 'uint8'"
            ) : Ho === nt && vt(
              Ka instanceof Float32Array,
              "buffer must be 'Float32Array' when reading from a framebuffer of type 'float'"
            )), vt(
              Ro > 0 && Ro + Co <= Sa.framebufferWidth,
              "invalid width for read pixels"
            ), vt(
              Qa > 0 && Qa + Ao <= Sa.framebufferHeight,
              "invalid height for read pixels"
            ), ar();
            var qo = Ro * Qa * 4;
            return Ka || (Ho === Be ? Ka = new Uint8Array(qo) : Ho === nt && (Ka = Ka || new Float32Array(qo))), vt.isTypedArray(Ka, "data buffer for regl.read() must be a typedarray"), vt(Ka.byteLength >= qo, "data buffer for regl.read() too small"), pr.pixelStorei(Ue, 4), pr.readPixels(
              Co,
              Ao,
              Ro,
              Qa,
              Oe,
              Ho,
              Ka
            ), Ka;
          }
          function lo(bo) {
            var Ho;
            return oi.setFBO({
              framebuffer: bo.framebuffer
            }, function() {
              Ho = Qo(bo);
            }), Ho;
          }
          function Eo(bo) {
            return !bo || !("framebuffer" in bo) ? Qo(bo) : lo(bo);
          }
          return Eo;
        }
        function Et(pr) {
          return Array.prototype.slice.call(pr);
        }
        function Mt(pr) {
          return Et(pr).join("");
        }
        function qt() {
          var pr = 0, oi = [], ar = [];
          function Sa(Ho) {
            for (var Co = 0; Co < ar.length; ++Co)
              if (ar[Co] === Ho)
                return oi[Co];
            var Ao = "g" + pr++;
            return oi.push(Ao), ar.push(Ho), Ao;
          }
          function Qn() {
            var Ho = [];
            function Co() {
              Ho.push.apply(Ho, Et(arguments));
            }
            var Ao = [];
            function Ro() {
              var Qa = "v" + pr++;
              return Ao.push(Qa), arguments.length > 0 && (Ho.push(Qa, "="), Ho.push.apply(Ho, Et(arguments)), Ho.push(";")), Qa;
            }
            return V(Co, {
              def: Ro,
              toString: function() {
                return Mt([
                  Ao.length > 0 ? "var " + Ao.join(",") + ";" : "",
                  Mt(Ho)
                ]);
              }
            });
          }
          function za() {
            var Ho = Qn(), Co = Qn(), Ao = Ho.toString, Ro = Co.toString;
            function Qa(Ka, qo) {
              Co(Ka, qo, "=", Ho.def(Ka, qo), ";");
            }
            return V(function() {
              Ho.apply(Ho, Et(arguments));
            }, {
              def: Ho.def,
              entry: Ho,
              exit: Co,
              save: Qa,
              set: function(Ka, qo, Ss) {
                Qa(Ka, qo), Ho(Ka, qo, "=", Ss, ";");
              },
              toString: function() {
                return Ao() + Ro();
              }
            });
          }
          function En() {
            var Ho = Mt(arguments), Co = za(), Ao = za(), Ro = Co.toString, Qa = Ao.toString;
            return V(Co, {
              then: function() {
                return Co.apply(Co, Et(arguments)), this;
              },
              else: function() {
                return Ao.apply(Ao, Et(arguments)), this;
              },
              toString: function() {
                var Ka = Qa();
                return Ka && (Ka = "else{" + Ka + "}"), Mt([
                  "if(",
                  Ho,
                  "){",
                  Ro(),
                  "}",
                  Ka
                ]);
              }
            });
          }
          var Qo = Qn(), lo = {};
          function Eo(Ho, Co) {
            var Ao = [];
            function Ro() {
              var ho = "a" + Ao.length;
              return Ao.push(ho), ho;
            }
            Co = Co || 0;
            for (var Qa = 0; Qa < Co; ++Qa)
              Ro();
            var Ka = za(), qo = Ka.toString, Ss = lo[Ho] = V(Ka, {
              arg: Ro,
              toString: function() {
                return Mt([
                  "function(",
                  Ao.join(),
                  "){",
                  qo(),
                  "}"
                ]);
              }
            });
            return Ss;
          }
          function bo() {
            var Ho = [
              '"use strict";',
              Qo,
              "return {"
            ];
            Object.keys(lo).forEach(function(Ro) {
              Ho.push('"', Ro, '":', lo[Ro].toString(), ",");
            }), Ho.push("}");
            var Co = Mt(Ho).replace(/;/g, `;
`).replace(/}/g, `}
`).replace(/{/g, `{
`), Ao = Function.apply(null, oi.concat(Co));
            return Ao.apply(null, ar);
          }
          return {
            global: Qo,
            link: Sa,
            block: Qn,
            proc: Eo,
            scope: za,
            cond: En,
            compile: bo
          };
        }
        var tn = "xyzw".split(""), xn = 5121, An = 1, On = 2, Bn = 0, qn = 1, Vn = 2, sr = 3, Un = 4, wr = 5, Cr = 6, Tr = "dither", ii = "blend.enable", wi = "blend.color", Wi = "blend.equation", Yi = "blend.func", ra = "depth.enable", ha = "depth.func", ya = "depth.range", va = "depth.mask", qa = "colorMask", Aa = "cull.enable", Xa = "cull.face", Oa = "frontFace", Ya = "lineWidth", Wo = "polygonOffset.enable", Ja = "polygonOffset.offset", Uo = "sample.alpha", ys = "sample.enable", il = "sample.coverage", Is = "stencil.enable", ml = "stencil.mask", nl = "stencil.func", Il = "stencil.opFront", Fl = "stencil.opBack", qc = "scissor.enable", ql = "scissor.box", jc = "viewport", Dl = "profile", Ml = "framebuffer", pc = "vert", hl = "frag", xl = "elements", Bl = "primitive", al = "count", Oc = "offset", nu = "instances", Ic = "vao", $s = "Width", Xc = "Height", wc = Ml + $s, zu = Ml + Xc, Zc = jc + $s, Ju = jc + Xc, ru = "drawingBuffer", Bd = ru + $s, tf = ru + Xc, wd = [
          Yi,
          Wi,
          nl,
          Il,
          Fl,
          il,
          jc,
          ql,
          Ja
        ], od = 34962, yd = 34963, Ef = 35632, Md = 35633, nf = 3553, Vd = 34067, S1 = 2884, Af = 3042, Pd = 3024, Au = 2960, Vu = 2929, Yh = 3089, Qf = 32823, vd = 32926, Ff = 32928, Uu = 5126, rf = 35664, Ud = 35665, $d = 35666, Sd = 5124, Yd = 35667, af = 35668, of = 35669, Bf = 35670, qd = 35671, Cf = 35672, V1 = 35673, jd = 35674, E1 = 35675, sd = 35676, Xd = 35678, ld = 35680, Ch = 4, sf = 1028, Zd = 1029, Lf = 2304, uh = 2305, Xf = 32775, Lh = 32776, p1 = 519, cd = 7680, dh = 0, kh = 1, Mh = 32774, h2 = 513, Rh = 36160, Zf = 36064, Rd = {
          0: 0,
          1: 1,
          zero: 0,
          one: 1,
          "src color": 768,
          "one minus src color": 769,
          "src alpha": 770,
          "one minus src alpha": 771,
          "dst color": 774,
          "one minus dst color": 775,
          "dst alpha": 772,
          "one minus dst alpha": 773,
          "constant color": 32769,
          "one minus constant color": 32770,
          "constant alpha": 32771,
          "one minus constant alpha": 32772,
          "src alpha saturate": 776
        }, ed = [
          "constant color, constant alpha",
          "one minus constant color, constant alpha",
          "constant color, one minus constant alpha",
          "one minus constant color, one minus constant alpha",
          "constant alpha, constant color",
          "constant alpha, one minus constant color",
          "one minus constant alpha, constant color",
          "one minus constant alpha, one minus constant color"
        ], Hf = {
          never: 512,
          less: 513,
          "<": 513,
          equal: 514,
          "=": 514,
          "==": 514,
          "===": 514,
          lequal: 515,
          "<=": 515,
          greater: 516,
          ">": 516,
          notequal: 517,
          "!=": 517,
          "!==": 517,
          gequal: 518,
          ">=": 518,
          always: 519
        }, Od = {
          0: 0,
          zero: 0,
          keep: 7680,
          replace: 7681,
          increment: 7682,
          decrement: 7683,
          "increment wrap": 34055,
          "decrement wrap": 34056,
          invert: 5386
        }, Oh = {
          frag: Ef,
          vert: Md
        }, Yu = {
          cw: Lf,
          ccw: uh
        };
        function kf(pr) {
          return Array.isArray(pr) || H(pr) || Gr(pr);
        }
        function Mf(pr) {
          return pr.sort(function(oi, ar) {
            return oi === jc ? -1 : ar === jc ? 1 : oi < ar ? -1 : 1;
          });
        }
        function e1(pr, oi, ar, Sa) {
          this.thisDep = pr, this.contextDep = oi, this.propDep = ar, this.append = Sa;
        }
        function Id(pr) {
          return pr && !(pr.thisDep || pr.contextDep || pr.propDep);
        }
        function cu(pr) {
          return new e1(!1, !1, !1, pr);
        }
        function pu(pr, oi) {
          var ar = pr.type;
          if (ar === Bn) {
            var Sa = pr.data.length;
            return new e1(
              !0,
              Sa >= 1,
              Sa >= 2,
              oi
            );
          } else if (ar === Un) {
            var Qn = pr.data;
            return new e1(
              Qn.thisDep,
              Qn.contextDep,
              Qn.propDep,
              oi
            );
          } else {
            if (ar === wr)
              return new e1(
                !1,
                !1,
                !1,
                oi
              );
            if (ar === Cr) {
              for (var za = !1, En = !1, Qo = !1, lo = 0; lo < pr.data.length; ++lo) {
                var Eo = pr.data[lo];
                if (Eo.type === qn)
                  Qo = !0;
                else if (Eo.type === Vn)
                  En = !0;
                else if (Eo.type === sr)
                  za = !0;
                else if (Eo.type === Bn) {
                  za = !0;
                  var bo = Eo.data;
                  bo >= 1 && (En = !0), bo >= 2 && (Qo = !0);
                } else Eo.type === Un && (za = za || Eo.data.thisDep, En = En || Eo.data.contextDep, Qo = Qo || Eo.data.propDep);
              }
              return new e1(
                za,
                En,
                Qo,
                oi
              );
            } else
              return new e1(
                ar === sr,
                ar === Vn,
                ar === qn,
                oi
              );
          }
        }
        var gf = new e1(!1, !1, !1, function() {
        });
        function lf(pr, oi, ar, Sa, Qn, za, En, Qo, lo, Eo, bo, Ho, Co, Ao, Ro) {
          var Qa = Eo.Record, Ka = {
            add: 32774,
            subtract: 32778,
            "reverse subtract": 32779
          };
          ar.ext_blend_minmax && (Ka.min = Xf, Ka.max = Lh);
          var qo = ar.angle_instanced_arrays, Ss = ar.webgl_draw_buffers, ho = ar.oes_vertex_array_object, hs = {
            dirty: !0,
            profile: Ro.profile
          }, Bo = {}, Vs = [], vo = {}, go = {};
          function Qs(Di) {
            return Di.replace(".", "_");
          }
          function us(Di, yi, ca) {
            var Ma = Qs(Di);
            Vs.push(Di), Bo[Ma] = hs[Ma] = !!ca, vo[Ma] = yi;
          }
          function Po(Di, yi, ca) {
            var Ma = Qs(Di);
            Vs.push(Di), Array.isArray(ca) ? (hs[Ma] = ca.slice(), Bo[Ma] = ca.slice()) : hs[Ma] = Bo[Ma] = ca, go[Ma] = yi;
          }
          us(Tr, Pd), us(ii, Af), Po(wi, "blendColor", [0, 0, 0, 0]), Po(
            Wi,
            "blendEquationSeparate",
            [Mh, Mh]
          ), Po(
            Yi,
            "blendFuncSeparate",
            [kh, dh, kh, dh]
          ), us(ra, Vu, !0), Po(ha, "depthFunc", h2), Po(ya, "depthRange", [0, 1]), Po(va, "depthMask", !0), Po(qa, qa, [!0, !0, !0, !0]), us(Aa, S1), Po(Xa, "cullFace", Zd), Po(Oa, Oa, uh), Po(Ya, Ya, 1), us(Wo, Qf), Po(Ja, "polygonOffset", [0, 0]), us(Uo, vd), us(ys, Ff), Po(il, "sampleCoverage", [1, !1]), us(Is, Au), Po(ml, "stencilMask", -1), Po(nl, "stencilFunc", [p1, 0, -1]), Po(
            Il,
            "stencilOpSeparate",
            [sf, cd, cd, cd]
          ), Po(
            Fl,
            "stencilOpSeparate",
            [Zd, cd, cd, cd]
          ), us(qc, Yh), Po(
            ql,
            "scissor",
            [0, 0, pr.drawingBufferWidth, pr.drawingBufferHeight]
          ), Po(
            jc,
            jc,
            [0, 0, pr.drawingBufferWidth, pr.drawingBufferHeight]
          );
          var Jo = {
            gl: pr,
            context: Co,
            strings: oi,
            next: Bo,
            current: hs,
            draw: Ho,
            elements: za,
            buffer: Qn,
            shader: bo,
            attributes: Eo.state,
            vao: Eo,
            uniforms: lo,
            framebuffer: Qo,
            extensions: ar,
            timer: Ao,
            isBufferArgs: kf
          }, ps = {
            primTypes: Tn,
            compareFuncs: Hf,
            blendFuncs: Rd,
            blendEquations: Ka,
            stencilOps: Od,
            glTypes: Jr,
            orientationType: Yu
          };
          vt.optional(function() {
            Jo.isArrayLike = Sl;
          }), Ss && (ps.backBuffer = [Zd], ps.drawBuffer = wn(Sa.maxDrawbuffers, function(Di) {
            return Di === 0 ? [0] : wn(Di, function(yi) {
              return Zf + yi;
            });
          }));
          var wl = 0;
          function ol() {
            var Di = qt(), yi = Di.link, ca = Di.global;
            Di.id = wl++, Di.batchId = "0";
            var Ma = yi(Jo), Ia = Di.shared = {
              props: "a0"
            };
            Object.keys(Jo).forEach(function(ji) {
              Ia[ji] = ca.def(Ma, ".", ji);
            }), vt.optional(function() {
              Di.CHECK = yi(vt), Di.commandStr = vt.guessCommand(), Di.command = yi(Di.commandStr), Di.assert = function(ji, qi, Ua) {
                ji(
                  "if(!(",
                  qi,
                  "))",
                  this.CHECK,
                  ".commandRaise(",
                  yi(Ua),
                  ",",
                  this.command,
                  ");"
                );
              }, ps.invalidBlendCombinations = ed;
            });
            var ba = Di.next = {}, fa = Di.current = {};
            Object.keys(go).forEach(function(ji) {
              Array.isArray(hs[ji]) && (ba[ji] = ca.def(Ia.next, ".", ji), fa[ji] = ca.def(Ia.current, ".", ji));
            });
            var Ea = Di.constants = {};
            Object.keys(ps).forEach(function(ji) {
              Ea[ji] = ca.def(JSON.stringify(ps[ji]));
            }), Di.invoke = function(ji, qi) {
              switch (qi.type) {
                case Bn:
                  var Ua = [
                    "this",
                    Ia.context,
                    Ia.props,
                    Di.batchId
                  ];
                  return ji.def(
                    yi(qi.data),
                    ".call(",
                    Ua.slice(0, Math.max(qi.data.length + 1, 4)),
                    ")"
                  );
                case qn:
                  return ji.def(Ia.props, qi.data);
                case Vn:
                  return ji.def(Ia.context, qi.data);
                case sr:
                  return ji.def("this", qi.data);
                case Un:
                  return qi.data.append(Di, ji), qi.data.ref;
                case wr:
                  return qi.data.toString();
                case Cr:
                  return qi.data.map(function(La) {
                    return Di.invoke(ji, La);
                  });
              }
            }, Di.attribCache = {};
            var Hi = {};
            return Di.scopeAttrib = function(ji) {
              var qi = oi.id(ji);
              if (qi in Hi)
                return Hi[qi];
              var Ua = Eo.scope[qi];
              Ua || (Ua = Eo.scope[qi] = new Qa());
              var La = Hi[qi] = yi(Ua);
              return La;
            }, Di;
          }
          function xs(Di) {
            var yi = Di.static, ca = Di.dynamic, Ma;
            if (Dl in yi) {
              var Ia = !!yi[Dl];
              Ma = cu(function(fa, Ea) {
                return Ia;
              }), Ma.enable = Ia;
            } else if (Dl in ca) {
              var ba = ca[Dl];
              Ma = pu(ba, function(fa, Ea) {
                return fa.invoke(Ea, ba);
              });
            }
            return Ma;
          }
          function ns(Di, yi) {
            var ca = Di.static, Ma = Di.dynamic;
            if (Ml in ca) {
              var Ia = ca[Ml];
              return Ia ? (Ia = Qo.getFramebuffer(Ia), vt.command(Ia, "invalid framebuffer object"), cu(function(fa, Ea) {
                var Hi = fa.link(Ia), ji = fa.shared;
                Ea.set(
                  ji.framebuffer,
                  ".next",
                  Hi
                );
                var qi = ji.context;
                return Ea.set(
                  qi,
                  "." + wc,
                  Hi + ".width"
                ), Ea.set(
                  qi,
                  "." + zu,
                  Hi + ".height"
                ), Hi;
              })) : cu(function(fa, Ea) {
                var Hi = fa.shared;
                Ea.set(
                  Hi.framebuffer,
                  ".next",
                  "null"
                );
                var ji = Hi.context;
                return Ea.set(
                  ji,
                  "." + wc,
                  ji + "." + Bd
                ), Ea.set(
                  ji,
                  "." + zu,
                  ji + "." + tf
                ), "null";
              });
            } else if (Ml in Ma) {
              var ba = Ma[Ml];
              return pu(ba, function(fa, Ea) {
                var Hi = fa.invoke(Ea, ba), ji = fa.shared, qi = ji.framebuffer, Ua = Ea.def(
                  qi,
                  ".getFramebuffer(",
                  Hi,
                  ")"
                );
                vt.optional(function() {
                  fa.assert(
                    Ea,
                    "!" + Hi + "||" + Ua,
                    "invalid framebuffer object"
                  );
                }), Ea.set(
                  qi,
                  ".next",
                  Ua
                );
                var La = ji.context;
                return Ea.set(
                  La,
                  "." + wc,
                  Ua + "?" + Ua + ".width:" + La + "." + Bd
                ), Ea.set(
                  La,
                  "." + zu,
                  Ua + "?" + Ua + ".height:" + La + "." + tf
                ), Ua;
              });
            } else
              return null;
          }
          function ll(Di, yi, ca) {
            var Ma = Di.static, Ia = Di.dynamic;
            function ba(Hi) {
              if (Hi in Ma) {
                var ji = Ma[Hi];
                vt.commandType(ji, "object", "invalid " + Hi, ca.commandStr);
                var qi = !0, Ua = ji.x | 0, La = ji.y | 0, yo, xo;
                return "width" in ji ? (yo = ji.width | 0, vt.command(yo >= 0, "invalid " + Hi, ca.commandStr)) : qi = !1, "height" in ji ? (xo = ji.height | 0, vt.command(xo >= 0, "invalid " + Hi, ca.commandStr)) : qi = !1, new e1(
                  !qi && yi && yi.thisDep,
                  !qi && yi && yi.contextDep,
                  !qi && yi && yi.propDep,
                  function(sl, Cs) {
                    var no = sl.shared.context, fo = yo;
                    "width" in ji || (fo = Cs.def(no, ".", wc, "-", Ua));
                    var Ys = xo;
                    return "height" in ji || (Ys = Cs.def(no, ".", zu, "-", La)), [Ua, La, fo, Ys];
                  }
                );
              } else if (Hi in Ia) {
                var rs = Ia[Hi], Us = pu(rs, function(sl, Cs) {
                  var no = sl.invoke(Cs, rs);
                  vt.optional(function() {
                    sl.assert(
                      Cs,
                      no + "&&typeof " + no + '==="object"',
                      "invalid " + Hi
                    );
                  });
                  var fo = sl.shared.context, Ys = Cs.def(no, ".x|0"), Rs = Cs.def(no, ".y|0"), js = Cs.def(
                    '"width" in ',
                    no,
                    "?",
                    no,
                    ".width|0:",
                    "(",
                    fo,
                    ".",
                    wc,
                    "-",
                    Ys,
                    ")"
                  ), Mc = Cs.def(
                    '"height" in ',
                    no,
                    "?",
                    no,
                    ".height|0:",
                    "(",
                    fo,
                    ".",
                    zu,
                    "-",
                    Rs,
                    ")"
                  );
                  return vt.optional(function() {
                    sl.assert(
                      Cs,
                      js + ">=0&&" + Mc + ">=0",
                      "invalid " + Hi
                    );
                  }), [Ys, Rs, js, Mc];
                });
                return yi && (Us.thisDep = Us.thisDep || yi.thisDep, Us.contextDep = Us.contextDep || yi.contextDep, Us.propDep = Us.propDep || yi.propDep), Us;
              } else return yi ? new e1(
                yi.thisDep,
                yi.contextDep,
                yi.propDep,
                function(sl, Cs) {
                  var no = sl.shared.context;
                  return [
                    0,
                    0,
                    Cs.def(no, ".", wc),
                    Cs.def(no, ".", zu)
                  ];
                }
              ) : null;
            }
            var fa = ba(jc);
            if (fa) {
              var Ea = fa;
              fa = new e1(
                fa.thisDep,
                fa.contextDep,
                fa.propDep,
                function(Hi, ji) {
                  var qi = Ea.append(Hi, ji), Ua = Hi.shared.context;
                  return ji.set(
                    Ua,
                    "." + Zc,
                    qi[2]
                  ), ji.set(
                    Ua,
                    "." + Ju,
                    qi[3]
                  ), qi;
                }
              );
            }
            return {
              viewport: fa,
              scissor_box: ba(ql)
            };
          }
          function Ws(Di, yi) {
            var ca = Di.static, Ma = typeof ca[hl] == "string" && typeof ca[pc] == "string";
            if (Ma) {
              if (Object.keys(yi.dynamic).length > 0)
                return null;
              var Ia = yi.static, ba = Object.keys(Ia);
              if (ba.length > 0 && typeof Ia[ba[0]] == "number") {
                for (var fa = [], Ea = 0; Ea < ba.length; ++Ea)
                  vt(typeof Ia[ba[Ea]] == "number", "must specify all vertex attribute locations when using vaos"), fa.push([Ia[ba[Ea]] | 0, ba[Ea]]);
                return fa;
              }
            }
            return null;
          }
          function ec(Di, yi, ca) {
            var Ma = Di.static, Ia = Di.dynamic;
            function ba(qi) {
              if (qi in Ma) {
                var Ua = oi.id(Ma[qi]);
                vt.optional(function() {
                  bo.shader(Oh[qi], Ua, vt.guessCommand());
                });
                var La = cu(function() {
                  return Ua;
                });
                return La.id = Ua, La;
              } else if (qi in Ia) {
                var yo = Ia[qi];
                return pu(yo, function(xo, rs) {
                  var Us = xo.invoke(rs, yo), sl = rs.def(xo.shared.strings, ".id(", Us, ")");
                  return vt.optional(function() {
                    rs(
                      xo.shared.shader,
                      ".shader(",
                      Oh[qi],
                      ",",
                      sl,
                      ",",
                      xo.command,
                      ");"
                    );
                  }), sl;
                });
              }
              return null;
            }
            var fa = ba(hl), Ea = ba(pc), Hi = null, ji;
            return Id(fa) && Id(Ea) ? (Hi = bo.program(Ea.id, fa.id, null, ca), ji = cu(function(qi, Ua) {
              return qi.link(Hi);
            })) : ji = new e1(
              fa && fa.thisDep || Ea && Ea.thisDep,
              fa && fa.contextDep || Ea && Ea.contextDep,
              fa && fa.propDep || Ea && Ea.propDep,
              function(qi, Ua) {
                var La = qi.shared.shader, yo;
                fa ? yo = fa.append(qi, Ua) : yo = Ua.def(La, ".", hl);
                var xo;
                Ea ? xo = Ea.append(qi, Ua) : xo = Ua.def(La, ".", pc);
                var rs = La + ".program(" + xo + "," + yo;
                return vt.optional(function() {
                  rs += "," + qi.command;
                }), Ua.def(rs + ")");
              }
            ), {
              frag: fa,
              vert: Ea,
              progVar: ji,
              program: Hi
            };
          }
          function s1(Di, yi) {
            var ca = Di.static, Ma = Di.dynamic, Ia = {}, ba = !1;
            function fa() {
              if (Ic in ca) {
                var Cs = ca[Ic];
                return Cs !== null && Eo.getVAO(Cs) === null && (Cs = Eo.createVAO(Cs)), ba = !0, Ia.vao = Cs, cu(function(fo) {
                  var Ys = Eo.getVAO(Cs);
                  return Ys ? fo.link(Ys) : "null";
                });
              } else if (Ic in Ma) {
                ba = !0;
                var no = Ma[Ic];
                return pu(no, function(fo, Ys) {
                  var Rs = fo.invoke(Ys, no);
                  return Ys.def(fo.shared.vao + ".getVAO(" + Rs + ")");
                });
              }
              return null;
            }
            var Ea = fa(), Hi = !1;
            function ji() {
              if (xl in ca) {
                var Cs = ca[xl];
                if (Ia.elements = Cs, kf(Cs)) {
                  var no = Ia.elements = za.create(Cs, !0);
                  Cs = za.getElements(no), Hi = !0;
                } else Cs && (Cs = za.getElements(Cs), Hi = !0, vt.command(Cs, "invalid elements", yi.commandStr));
                var fo = cu(function(Rs, js) {
                  if (Cs) {
                    var Mc = Rs.link(Cs);
                    return Rs.ELEMENTS = Mc, Mc;
                  }
                  return Rs.ELEMENTS = null, null;
                });
                return fo.value = Cs, fo;
              } else if (xl in Ma) {
                Hi = !0;
                var Ys = Ma[xl];
                return pu(Ys, function(Rs, js) {
                  var Mc = Rs.shared, dd = Mc.isBufferArgs, nh = Mc.elements, Vf = Rs.invoke(js, Ys), vf = js.def("null"), Uf = js.def(dd, "(", Vf, ")"), rh = Rs.cond(Uf).then(vf, "=", nh, ".createStream(", Vf, ");").else(vf, "=", nh, ".getElements(", Vf, ");");
                  return vt.optional(function() {
                    Rs.assert(
                      rh.else,
                      "!" + Vf + "||" + vf,
                      "invalid elements"
                    );
                  }), js.entry(rh), js.exit(
                    Rs.cond(Uf).then(nh, ".destroyStream(", vf, ");")
                  ), Rs.ELEMENTS = vf, vf;
                });
              } else if (ba)
                return new e1(
                  Ea.thisDep,
                  Ea.contextDep,
                  Ea.propDep,
                  function(Rs, js) {
                    return js.def(Rs.shared.vao + ".currentVAO?" + Rs.shared.elements + ".getElements(" + Rs.shared.vao + ".currentVAO.elements):null");
                  }
                );
              return null;
            }
            var qi = ji();
            function Ua() {
              if (Bl in ca) {
                var Cs = ca[Bl];
                return Ia.primitive = Cs, vt.commandParameter(Cs, Tn, "invalid primitve", yi.commandStr), cu(function(fo, Ys) {
                  return Tn[Cs];
                });
              } else if (Bl in Ma) {
                var no = Ma[Bl];
                return pu(no, function(fo, Ys) {
                  var Rs = fo.constants.primTypes, js = fo.invoke(Ys, no);
                  return vt.optional(function() {
                    fo.assert(
                      Ys,
                      js + " in " + Rs,
                      "invalid primitive, must be one of " + Object.keys(Tn)
                    );
                  }), Ys.def(Rs, "[", js, "]");
                });
              } else {
                if (Hi)
                  return Id(qi) ? qi.value ? cu(function(fo, Ys) {
                    return Ys.def(fo.ELEMENTS, ".primType");
                  }) : cu(function() {
                    return Ch;
                  }) : new e1(
                    qi.thisDep,
                    qi.contextDep,
                    qi.propDep,
                    function(fo, Ys) {
                      var Rs = fo.ELEMENTS;
                      return Ys.def(Rs, "?", Rs, ".primType:", Ch);
                    }
                  );
                if (ba)
                  return new e1(
                    Ea.thisDep,
                    Ea.contextDep,
                    Ea.propDep,
                    function(fo, Ys) {
                      return Ys.def(fo.shared.vao + ".currentVAO?" + fo.shared.vao + ".currentVAO.primitive:" + Ch);
                    }
                  );
              }
              return null;
            }
            function La(Cs, no) {
              if (Cs in ca) {
                var fo = ca[Cs] | 0;
                return no ? Ia.offset = fo : Ia.instances = fo, vt.command(!no || fo >= 0, "invalid " + Cs, yi.commandStr), cu(function(Rs, js) {
                  return no && (Rs.OFFSET = fo), fo;
                });
              } else if (Cs in Ma) {
                var Ys = Ma[Cs];
                return pu(Ys, function(Rs, js) {
                  var Mc = Rs.invoke(js, Ys);
                  return no && (Rs.OFFSET = Mc, vt.optional(function() {
                    Rs.assert(
                      js,
                      Mc + ">=0",
                      "invalid " + Cs
                    );
                  })), Mc;
                });
              } else if (no) {
                if (Hi)
                  return cu(function(Rs, js) {
                    return Rs.OFFSET = 0, 0;
                  });
                if (ba)
                  return new e1(
                    Ea.thisDep,
                    Ea.contextDep,
                    Ea.propDep,
                    function(Rs, js) {
                      return js.def(Rs.shared.vao + ".currentVAO?" + Rs.shared.vao + ".currentVAO.offset:0");
                    }
                  );
              } else if (ba)
                return new e1(
                  Ea.thisDep,
                  Ea.contextDep,
                  Ea.propDep,
                  function(Rs, js) {
                    return js.def(Rs.shared.vao + ".currentVAO?" + Rs.shared.vao + ".currentVAO.instances:-1");
                  }
                );
              return null;
            }
            var yo = La(Oc, !0);
            function xo() {
              if (al in ca) {
                var Cs = ca[al] | 0;
                return Ia.count = Cs, vt.command(
                  typeof Cs == "number" && Cs >= 0,
                  "invalid vertex count",
                  yi.commandStr
                ), cu(function() {
                  return Cs;
                });
              } else if (al in Ma) {
                var no = Ma[al];
                return pu(no, function(js, Mc) {
                  var dd = js.invoke(Mc, no);
                  return vt.optional(function() {
                    js.assert(
                      Mc,
                      "typeof " + dd + '==="number"&&' + dd + ">=0&&" + dd + "===(" + dd + "|0)",
                      "invalid vertex count"
                    );
                  }), dd;
                });
              } else if (Hi)
                if (Id(qi)) {
                  if (qi)
                    return yo ? new e1(
                      yo.thisDep,
                      yo.contextDep,
                      yo.propDep,
                      function(js, Mc) {
                        var dd = Mc.def(
                          js.ELEMENTS,
                          ".vertCount-",
                          js.OFFSET
                        );
                        return vt.optional(function() {
                          js.assert(
                            Mc,
                            dd + ">=0",
                            "invalid vertex offset/element buffer too small"
                          );
                        }), dd;
                      }
                    ) : cu(function(js, Mc) {
                      return Mc.def(js.ELEMENTS, ".vertCount");
                    });
                  var fo = cu(function() {
                    return -1;
                  });
                  return vt.optional(function() {
                    fo.MISSING = !0;
                  }), fo;
                } else {
                  var Ys = new e1(
                    qi.thisDep || yo.thisDep,
                    qi.contextDep || yo.contextDep,
                    qi.propDep || yo.propDep,
                    function(js, Mc) {
                      var dd = js.ELEMENTS;
                      return js.OFFSET ? Mc.def(
                        dd,
                        "?",
                        dd,
                        ".vertCount-",
                        js.OFFSET,
                        ":-1"
                      ) : Mc.def(dd, "?", dd, ".vertCount:-1");
                    }
                  );
                  return vt.optional(function() {
                    Ys.DYNAMIC = !0;
                  }), Ys;
                }
              else if (ba) {
                var Rs = new e1(
                  Ea.thisDep,
                  Ea.contextDep,
                  Ea.propDep,
                  function(js, Mc) {
                    return Mc.def(js.shared.vao, ".currentVAO?", js.shared.vao, ".currentVAO.count:-1");
                  }
                );
                return Rs;
              }
              return null;
            }
            var rs = Ua(), Us = xo(), sl = La(nu, !1);
            return {
              elements: qi,
              primitive: rs,
              count: Us,
              instances: sl,
              offset: yo,
              vao: Ea,
              vaoActive: ba,
              elementsActive: Hi,
              // static draw props
              static: Ia
            };
          }
          function n1(Di, yi) {
            var ca = Di.static, Ma = Di.dynamic, Ia = {};
            return Vs.forEach(function(ba) {
              var fa = Qs(ba);
              function Ea(Hi, ji) {
                if (ba in ca) {
                  var qi = Hi(ca[ba]);
                  Ia[fa] = cu(function() {
                    return qi;
                  });
                } else if (ba in Ma) {
                  var Ua = Ma[ba];
                  Ia[fa] = pu(Ua, function(La, yo) {
                    return ji(La, yo, La.invoke(yo, Ua));
                  });
                }
              }
              switch (ba) {
                case Aa:
                case ii:
                case Tr:
                case Is:
                case ra:
                case qc:
                case Wo:
                case Uo:
                case ys:
                case va:
                  return Ea(
                    function(Hi) {
                      return vt.commandType(Hi, "boolean", ba, yi.commandStr), Hi;
                    },
                    function(Hi, ji, qi) {
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          "typeof " + qi + '==="boolean"',
                          "invalid flag " + ba,
                          Hi.commandStr
                        );
                      }), qi;
                    }
                  );
                case ha:
                  return Ea(
                    function(Hi) {
                      return vt.commandParameter(Hi, Hf, "invalid " + ba, yi.commandStr), Hf[Hi];
                    },
                    function(Hi, ji, qi) {
                      var Ua = Hi.constants.compareFuncs;
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          qi + " in " + Ua,
                          "invalid " + ba + ", must be one of " + Object.keys(Hf)
                        );
                      }), ji.def(Ua, "[", qi, "]");
                    }
                  );
                case ya:
                  return Ea(
                    function(Hi) {
                      return vt.command(
                        Sl(Hi) && Hi.length === 2 && typeof Hi[0] == "number" && typeof Hi[1] == "number" && Hi[0] <= Hi[1],
                        "depth range is 2d array",
                        yi.commandStr
                      ), Hi;
                    },
                    function(Hi, ji, qi) {
                      vt.optional(function() {
                        Hi.assert(
                          ji,
                          Hi.shared.isArrayLike + "(" + qi + ")&&" + qi + ".length===2&&typeof " + qi + '[0]==="number"&&typeof ' + qi + '[1]==="number"&&' + qi + "[0]<=" + qi + "[1]",
                          "depth range must be a 2d array"
                        );
                      });
                      var Ua = ji.def("+", qi, "[0]"), La = ji.def("+", qi, "[1]");
                      return [Ua, La];
                    }
                  );
                case Yi:
                  return Ea(
                    function(Hi) {
                      vt.commandType(Hi, "object", "blend.func", yi.commandStr);
                      var ji = "srcRGB" in Hi ? Hi.srcRGB : Hi.src, qi = "srcAlpha" in Hi ? Hi.srcAlpha : Hi.src, Ua = "dstRGB" in Hi ? Hi.dstRGB : Hi.dst, La = "dstAlpha" in Hi ? Hi.dstAlpha : Hi.dst;
                      return vt.commandParameter(ji, Rd, fa + ".srcRGB", yi.commandStr), vt.commandParameter(qi, Rd, fa + ".srcAlpha", yi.commandStr), vt.commandParameter(Ua, Rd, fa + ".dstRGB", yi.commandStr), vt.commandParameter(La, Rd, fa + ".dstAlpha", yi.commandStr), vt.command(
                        ed.indexOf(ji + ", " + Ua) === -1,
                        "unallowed blending combination (srcRGB, dstRGB) = (" + ji + ", " + Ua + ")",
                        yi.commandStr
                      ), [
                        Rd[ji],
                        Rd[Ua],
                        Rd[qi],
                        Rd[La]
                      ];
                    },
                    function(Hi, ji, qi) {
                      var Ua = Hi.constants.blendFuncs;
                      vt.optional(function() {
                        Hi.assert(
                          ji,
                          qi + "&&typeof " + qi + '==="object"',
                          "invalid blend func, must be an object"
                        );
                      });
                      function La(no, fo) {
                        var Ys = ji.def(
                          '"',
                          no,
                          fo,
                          '" in ',
                          qi,
                          "?",
                          qi,
                          ".",
                          no,
                          fo,
                          ":",
                          qi,
                          ".",
                          no
                        );
                        return vt.optional(function() {
                          Hi.assert(
                            ji,
                            Ys + " in " + Ua,
                            "invalid " + ba + "." + no + fo + ", must be one of " + Object.keys(Rd)
                          );
                        }), Ys;
                      }
                      var yo = La("src", "RGB"), xo = La("dst", "RGB");
                      vt.optional(function() {
                        var no = Hi.constants.invalidBlendCombinations;
                        Hi.assert(
                          ji,
                          no + ".indexOf(" + yo + '+", "+' + xo + ") === -1 ",
                          "unallowed blending combination for (srcRGB, dstRGB)"
                        );
                      });
                      var rs = ji.def(Ua, "[", yo, "]"), Us = ji.def(Ua, "[", La("src", "Alpha"), "]"), sl = ji.def(Ua, "[", xo, "]"), Cs = ji.def(Ua, "[", La("dst", "Alpha"), "]");
                      return [rs, sl, Us, Cs];
                    }
                  );
                case Wi:
                  return Ea(
                    function(Hi) {
                      if (typeof Hi == "string")
                        return vt.commandParameter(Hi, Ka, "invalid " + ba, yi.commandStr), [
                          Ka[Hi],
                          Ka[Hi]
                        ];
                      if (typeof Hi == "object")
                        return vt.commandParameter(
                          Hi.rgb,
                          Ka,
                          ba + ".rgb",
                          yi.commandStr
                        ), vt.commandParameter(
                          Hi.alpha,
                          Ka,
                          ba + ".alpha",
                          yi.commandStr
                        ), [
                          Ka[Hi.rgb],
                          Ka[Hi.alpha]
                        ];
                      vt.commandRaise("invalid blend.equation", yi.commandStr);
                    },
                    function(Hi, ji, qi) {
                      var Ua = Hi.constants.blendEquations, La = ji.def(), yo = ji.def(), xo = Hi.cond("typeof ", qi, '==="string"');
                      return vt.optional(function() {
                        function rs(Us, sl, Cs) {
                          Hi.assert(
                            Us,
                            Cs + " in " + Ua,
                            "invalid " + sl + ", must be one of " + Object.keys(Ka)
                          );
                        }
                        rs(xo.then, ba, qi), Hi.assert(
                          xo.else,
                          qi + "&&typeof " + qi + '==="object"',
                          "invalid " + ba
                        ), rs(xo.else, ba + ".rgb", qi + ".rgb"), rs(xo.else, ba + ".alpha", qi + ".alpha");
                      }), xo.then(
                        La,
                        "=",
                        yo,
                        "=",
                        Ua,
                        "[",
                        qi,
                        "];"
                      ), xo.else(
                        La,
                        "=",
                        Ua,
                        "[",
                        qi,
                        ".rgb];",
                        yo,
                        "=",
                        Ua,
                        "[",
                        qi,
                        ".alpha];"
                      ), ji(xo), [La, yo];
                    }
                  );
                case wi:
                  return Ea(
                    function(Hi) {
                      return vt.command(
                        Sl(Hi) && Hi.length === 4,
                        "blend.color must be a 4d array",
                        yi.commandStr
                      ), wn(4, function(ji) {
                        return +Hi[ji];
                      });
                    },
                    function(Hi, ji, qi) {
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          Hi.shared.isArrayLike + "(" + qi + ")&&" + qi + ".length===4",
                          "blend.color must be a 4d array"
                        );
                      }), wn(4, function(Ua) {
                        return ji.def("+", qi, "[", Ua, "]");
                      });
                    }
                  );
                case ml:
                  return Ea(
                    function(Hi) {
                      return vt.commandType(Hi, "number", fa, yi.commandStr), Hi | 0;
                    },
                    function(Hi, ji, qi) {
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          "typeof " + qi + '==="number"',
                          "invalid stencil.mask"
                        );
                      }), ji.def(qi, "|0");
                    }
                  );
                case nl:
                  return Ea(
                    function(Hi) {
                      vt.commandType(Hi, "object", fa, yi.commandStr);
                      var ji = Hi.cmp || "keep", qi = Hi.ref || 0, Ua = "mask" in Hi ? Hi.mask : -1;
                      return vt.commandParameter(ji, Hf, ba + ".cmp", yi.commandStr), vt.commandType(qi, "number", ba + ".ref", yi.commandStr), vt.commandType(Ua, "number", ba + ".mask", yi.commandStr), [
                        Hf[ji],
                        qi,
                        Ua
                      ];
                    },
                    function(Hi, ji, qi) {
                      var Ua = Hi.constants.compareFuncs;
                      vt.optional(function() {
                        function rs() {
                          Hi.assert(
                            ji,
                            Array.prototype.join.call(arguments, ""),
                            "invalid stencil.func"
                          );
                        }
                        rs(qi + "&&typeof ", qi, '==="object"'), rs(
                          '!("cmp" in ',
                          qi,
                          ")||(",
                          qi,
                          ".cmp in ",
                          Ua,
                          ")"
                        );
                      });
                      var La = ji.def(
                        '"cmp" in ',
                        qi,
                        "?",
                        Ua,
                        "[",
                        qi,
                        ".cmp]",
                        ":",
                        cd
                      ), yo = ji.def(qi, ".ref|0"), xo = ji.def(
                        '"mask" in ',
                        qi,
                        "?",
                        qi,
                        ".mask|0:-1"
                      );
                      return [La, yo, xo];
                    }
                  );
                case Il:
                case Fl:
                  return Ea(
                    function(Hi) {
                      vt.commandType(Hi, "object", fa, yi.commandStr);
                      var ji = Hi.fail || "keep", qi = Hi.zfail || "keep", Ua = Hi.zpass || "keep";
                      return vt.commandParameter(ji, Od, ba + ".fail", yi.commandStr), vt.commandParameter(qi, Od, ba + ".zfail", yi.commandStr), vt.commandParameter(Ua, Od, ba + ".zpass", yi.commandStr), [
                        ba === Fl ? Zd : sf,
                        Od[ji],
                        Od[qi],
                        Od[Ua]
                      ];
                    },
                    function(Hi, ji, qi) {
                      var Ua = Hi.constants.stencilOps;
                      vt.optional(function() {
                        Hi.assert(
                          ji,
                          qi + "&&typeof " + qi + '==="object"',
                          "invalid " + ba
                        );
                      });
                      function La(yo) {
                        return vt.optional(function() {
                          Hi.assert(
                            ji,
                            '!("' + yo + '" in ' + qi + ")||(" + qi + "." + yo + " in " + Ua + ")",
                            "invalid " + ba + "." + yo + ", must be one of " + Object.keys(Od)
                          );
                        }), ji.def(
                          '"',
                          yo,
                          '" in ',
                          qi,
                          "?",
                          Ua,
                          "[",
                          qi,
                          ".",
                          yo,
                          "]:",
                          cd
                        );
                      }
                      return [
                        ba === Fl ? Zd : sf,
                        La("fail"),
                        La("zfail"),
                        La("zpass")
                      ];
                    }
                  );
                case Ja:
                  return Ea(
                    function(Hi) {
                      vt.commandType(Hi, "object", fa, yi.commandStr);
                      var ji = Hi.factor | 0, qi = Hi.units | 0;
                      return vt.commandType(ji, "number", fa + ".factor", yi.commandStr), vt.commandType(qi, "number", fa + ".units", yi.commandStr), [ji, qi];
                    },
                    function(Hi, ji, qi) {
                      vt.optional(function() {
                        Hi.assert(
                          ji,
                          qi + "&&typeof " + qi + '==="object"',
                          "invalid " + ba
                        );
                      });
                      var Ua = ji.def(qi, ".factor|0"), La = ji.def(qi, ".units|0");
                      return [Ua, La];
                    }
                  );
                case Xa:
                  return Ea(
                    function(Hi) {
                      var ji = 0;
                      return Hi === "front" ? ji = sf : Hi === "back" && (ji = Zd), vt.command(!!ji, fa, yi.commandStr), ji;
                    },
                    function(Hi, ji, qi) {
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          qi + '==="front"||' + qi + '==="back"',
                          "invalid cull.face"
                        );
                      }), ji.def(qi, '==="front"?', sf, ":", Zd);
                    }
                  );
                case Ya:
                  return Ea(
                    function(Hi) {
                      return vt.command(
                        typeof Hi == "number" && Hi >= Sa.lineWidthDims[0] && Hi <= Sa.lineWidthDims[1],
                        "invalid line width, must be a positive number between " + Sa.lineWidthDims[0] + " and " + Sa.lineWidthDims[1],
                        yi.commandStr
                      ), Hi;
                    },
                    function(Hi, ji, qi) {
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          "typeof " + qi + '==="number"&&' + qi + ">=" + Sa.lineWidthDims[0] + "&&" + qi + "<=" + Sa.lineWidthDims[1],
                          "invalid line width"
                        );
                      }), qi;
                    }
                  );
                case Oa:
                  return Ea(
                    function(Hi) {
                      return vt.commandParameter(Hi, Yu, fa, yi.commandStr), Yu[Hi];
                    },
                    function(Hi, ji, qi) {
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          qi + '==="cw"||' + qi + '==="ccw"',
                          "invalid frontFace, must be one of cw,ccw"
                        );
                      }), ji.def(qi + '==="cw"?' + Lf + ":" + uh);
                    }
                  );
                case qa:
                  return Ea(
                    function(Hi) {
                      return vt.command(
                        Sl(Hi) && Hi.length === 4,
                        "color.mask must be length 4 array",
                        yi.commandStr
                      ), Hi.map(function(ji) {
                        return !!ji;
                      });
                    },
                    function(Hi, ji, qi) {
                      return vt.optional(function() {
                        Hi.assert(
                          ji,
                          Hi.shared.isArrayLike + "(" + qi + ")&&" + qi + ".length===4",
                          "invalid color.mask"
                        );
                      }), wn(4, function(Ua) {
                        return "!!" + qi + "[" + Ua + "]";
                      });
                    }
                  );
                case il:
                  return Ea(
                    function(Hi) {
                      vt.command(typeof Hi == "object" && Hi, fa, yi.commandStr);
                      var ji = "value" in Hi ? Hi.value : 1, qi = !!Hi.invert;
                      return vt.command(
                        typeof ji == "number" && ji >= 0 && ji <= 1,
                        "sample.coverage.value must be a number between 0 and 1",
                        yi.commandStr
                      ), [ji, qi];
                    },
                    function(Hi, ji, qi) {
                      vt.optional(function() {
                        Hi.assert(
                          ji,
                          qi + "&&typeof " + qi + '==="object"',
                          "invalid sample.coverage"
                        );
                      });
                      var Ua = ji.def(
                        '"value" in ',
                        qi,
                        "?+",
                        qi,
                        ".value:1"
                      ), La = ji.def("!!", qi, ".invert");
                      return [Ua, La];
                    }
                  );
              }
            }), Ia;
          }
          function Js(Di, yi) {
            var ca = Di.static, Ma = Di.dynamic, Ia = {};
            return Object.keys(ca).forEach(function(ba) {
              var fa = ca[ba], Ea;
              if (typeof fa == "number" || typeof fa == "boolean")
                Ea = cu(function() {
                  return fa;
                });
              else if (typeof fa == "function") {
                var Hi = fa._reglType;
                Hi === "texture2d" || Hi === "textureCube" ? Ea = cu(function(ji) {
                  return ji.link(fa);
                }) : Hi === "framebuffer" || Hi === "framebufferCube" ? (vt.command(
                  fa.color.length > 0,
                  'missing color attachment for framebuffer sent to uniform "' + ba + '"',
                  yi.commandStr
                ), Ea = cu(function(ji) {
                  return ji.link(fa.color[0]);
                })) : vt.commandRaise('invalid data for uniform "' + ba + '"', yi.commandStr);
              } else Sl(fa) ? Ea = cu(function(ji) {
                var qi = ji.global.def(
                  "[",
                  wn(fa.length, function(Ua) {
                    return vt.command(
                      typeof fa[Ua] == "number" || typeof fa[Ua] == "boolean",
                      "invalid uniform " + ba,
                      ji.commandStr
                    ), fa[Ua];
                  }),
                  "]"
                );
                return qi;
              }) : vt.commandRaise('invalid or missing data for uniform "' + ba + '"', yi.commandStr);
              Ea.value = fa, Ia[ba] = Ea;
            }), Object.keys(Ma).forEach(function(ba) {
              var fa = Ma[ba];
              Ia[ba] = pu(fa, function(Ea, Hi) {
                return Ea.invoke(Hi, fa);
              });
            }), Ia;
          }
          function N1(Di, yi) {
            var ca = Di.static, Ma = Di.dynamic, Ia = {};
            return Object.keys(ca).forEach(function(ba) {
              var fa = ca[ba], Ea = oi.id(ba), Hi = new Qa();
              if (kf(fa))
                Hi.state = An, Hi.buffer = Qn.getBuffer(
                  Qn.create(fa, od, !1, !0)
                ), Hi.type = 0;
              else {
                var ji = Qn.getBuffer(fa);
                if (ji)
                  Hi.state = An, Hi.buffer = ji, Hi.type = 0;
                else if (vt.command(
                  typeof fa == "object" && fa,
                  "invalid data for attribute " + ba,
                  yi.commandStr
                ), "constant" in fa) {
                  var qi = fa.constant;
                  Hi.buffer = "null", Hi.state = On, typeof qi == "number" ? Hi.x = qi : (vt.command(
                    Sl(qi) && qi.length > 0 && qi.length <= 4,
                    "invalid constant for attribute " + ba,
                    yi.commandStr
                  ), tn.forEach(function(sl, Cs) {
                    Cs < qi.length && (Hi[sl] = qi[Cs]);
                  }));
                } else {
                  kf(fa.buffer) ? ji = Qn.getBuffer(
                    Qn.create(fa.buffer, od, !1, !0)
                  ) : ji = Qn.getBuffer(fa.buffer), vt.command(!!ji, 'missing buffer for attribute "' + ba + '"', yi.commandStr);
                  var Ua = fa.offset | 0;
                  vt.command(
                    Ua >= 0,
                    'invalid offset for attribute "' + ba + '"',
                    yi.commandStr
                  );
                  var La = fa.stride | 0;
                  vt.command(
                    La >= 0 && La < 256,
                    'invalid stride for attribute "' + ba + '", must be integer betweeen [0, 255]',
                    yi.commandStr
                  );
                  var yo = fa.size | 0;
                  vt.command(
                    !("size" in fa) || yo > 0 && yo <= 4,
                    'invalid size for attribute "' + ba + '", must be 1,2,3,4',
                    yi.commandStr
                  );
                  var xo = !!fa.normalized, rs = 0;
                  "type" in fa && (vt.commandParameter(
                    fa.type,
                    Jr,
                    "invalid type for attribute " + ba,
                    yi.commandStr
                  ), rs = Jr[fa.type]);
                  var Us = fa.divisor | 0;
                  vt.optional(function() {
                    "divisor" in fa && (vt.command(
                      Us === 0 || qo,
                      'cannot specify divisor for attribute "' + ba + '", instancing not supported',
                      yi.commandStr
                    ), vt.command(
                      Us >= 0,
                      'invalid divisor for attribute "' + ba + '"',
                      yi.commandStr
                    ));
                    var sl = yi.commandStr, Cs = [
                      "buffer",
                      "offset",
                      "divisor",
                      "normalized",
                      "type",
                      "size",
                      "stride"
                    ];
                    Object.keys(fa).forEach(function(no) {
                      vt.command(
                        Cs.indexOf(no) >= 0,
                        'unknown parameter "' + no + '" for attribute pointer "' + ba + '" (valid parameters are ' + Cs + ")",
                        sl
                      );
                    });
                  }), Hi.buffer = ji, Hi.state = An, Hi.size = yo, Hi.normalized = xo, Hi.type = rs || ji.dtype, Hi.offset = Ua, Hi.stride = La, Hi.divisor = Us;
                }
              }
              Ia[ba] = cu(function(sl, Cs) {
                var no = sl.attribCache;
                if (Ea in no)
                  return no[Ea];
                var fo = {
                  isStream: !1
                };
                return Object.keys(Hi).forEach(function(Ys) {
                  fo[Ys] = Hi[Ys];
                }), Hi.buffer && (fo.buffer = sl.link(Hi.buffer), fo.type = fo.type || fo.buffer + ".dtype"), no[Ea] = fo, fo;
              });
            }), Object.keys(Ma).forEach(function(ba) {
              var fa = Ma[ba];
              function Ea(Hi, ji) {
                var qi = Hi.invoke(ji, fa), Ua = Hi.shared, La = Hi.constants, yo = Ua.isBufferArgs, xo = Ua.buffer;
                vt.optional(function() {
                  Hi.assert(
                    ji,
                    qi + "&&(typeof " + qi + '==="object"||typeof ' + qi + '==="function")&&(' + yo + "(" + qi + ")||" + xo + ".getBuffer(" + qi + ")||" + xo + ".getBuffer(" + qi + ".buffer)||" + yo + "(" + qi + '.buffer)||("constant" in ' + qi + "&&(typeof " + qi + '.constant==="number"||' + Ua.isArrayLike + "(" + qi + ".constant))))",
                    'invalid dynamic attribute "' + ba + '"'
                  );
                });
                var rs = {
                  isStream: ji.def(!1)
                }, Us = new Qa();
                Us.state = An, Object.keys(Us).forEach(function(fo) {
                  rs[fo] = ji.def("" + Us[fo]);
                });
                var sl = rs.buffer, Cs = rs.type;
                ji(
                  "if(",
                  yo,
                  "(",
                  qi,
                  ")){",
                  rs.isStream,
                  "=true;",
                  sl,
                  "=",
                  xo,
                  ".createStream(",
                  od,
                  ",",
                  qi,
                  ");",
                  Cs,
                  "=",
                  sl,
                  ".dtype;",
                  "}else{",
                  sl,
                  "=",
                  xo,
                  ".getBuffer(",
                  qi,
                  ");",
                  "if(",
                  sl,
                  "){",
                  Cs,
                  "=",
                  sl,
                  ".dtype;",
                  '}else if("constant" in ',
                  qi,
                  "){",
                  rs.state,
                  "=",
                  On,
                  ";",
                  "if(typeof " + qi + '.constant === "number"){',
                  rs[tn[0]],
                  "=",
                  qi,
                  ".constant;",
                  tn.slice(1).map(function(fo) {
                    return rs[fo];
                  }).join("="),
                  "=0;",
                  "}else{",
                  tn.map(function(fo, Ys) {
                    return rs[fo] + "=" + qi + ".constant.length>" + Ys + "?" + qi + ".constant[" + Ys + "]:0;";
                  }).join(""),
                  "}}else{",
                  "if(",
                  yo,
                  "(",
                  qi,
                  ".buffer)){",
                  sl,
                  "=",
                  xo,
                  ".createStream(",
                  od,
                  ",",
                  qi,
                  ".buffer);",
                  "}else{",
                  sl,
                  "=",
                  xo,
                  ".getBuffer(",
                  qi,
                  ".buffer);",
                  "}",
                  Cs,
                  '="type" in ',
                  qi,
                  "?",
                  La.glTypes,
                  "[",
                  qi,
                  ".type]:",
                  sl,
                  ".dtype;",
                  rs.normalized,
                  "=!!",
                  qi,
                  ".normalized;"
                );
                function no(fo) {
                  ji(rs[fo], "=", qi, ".", fo, "|0;");
                }
                return no("size"), no("offset"), no("stride"), no("divisor"), ji("}}"), ji.exit(
                  "if(",
                  rs.isStream,
                  "){",
                  xo,
                  ".destroyStream(",
                  sl,
                  ");",
                  "}"
                ), rs;
              }
              Ia[ba] = pu(fa, Ea);
            }), Ia;
          }
          function Xu(Di) {
            var yi = Di.static, ca = Di.dynamic, Ma = {};
            return Object.keys(yi).forEach(function(Ia) {
              var ba = yi[Ia];
              Ma[Ia] = cu(function(fa, Ea) {
                return typeof ba == "number" || typeof ba == "boolean" ? "" + ba : fa.link(ba);
              });
            }), Object.keys(ca).forEach(function(Ia) {
              var ba = ca[Ia];
              Ma[Ia] = pu(ba, function(fa, Ea) {
                return fa.invoke(Ea, ba);
              });
            }), Ma;
          }
          function Q1(Di, yi, ca, Ma, Ia) {
            var ba = Di.static, fa = Di.dynamic;
            vt.optional(function() {
              var no = [
                Ml,
                pc,
                hl,
                xl,
                Bl,
                Oc,
                al,
                nu,
                Dl,
                Ic
              ].concat(Vs);
              function fo(Ys) {
                Object.keys(Ys).forEach(function(Rs) {
                  vt.command(
                    no.indexOf(Rs) >= 0,
                    'unknown parameter "' + Rs + '"',
                    Ia.commandStr
                  );
                });
              }
              fo(ba), fo(fa);
            });
            var Ea = Ws(Di, yi), Hi = ns(Di), ji = ll(Di, Hi, Ia), qi = s1(Di, Ia), Ua = n1(Di, Ia), La = ec(Di, Ia, Ea);
            function yo(no) {
              var fo = ji[no];
              fo && (Ua[no] = fo);
            }
            yo(jc), yo(Qs(ql));
            var xo = Object.keys(Ua).length > 0, rs = {
              framebuffer: Hi,
              draw: qi,
              shader: La,
              state: Ua,
              dirty: xo,
              scopeVAO: null,
              drawVAO: null,
              useVAO: !1,
              attributes: {}
            };
            if (rs.profile = xs(Di), rs.uniforms = Js(ca, Ia), rs.drawVAO = rs.scopeVAO = qi.vao, !rs.drawVAO && La.program && !Ea && ar.angle_instanced_arrays && qi.static.elements) {
              var Us = !0, sl = La.program.attributes.map(function(no) {
                var fo = yi.static[no];
                return Us = Us && !!fo, fo;
              });
              if (Us && sl.length > 0) {
                var Cs = Eo.getVAO(Eo.createVAO({
                  attributes: sl,
                  elements: qi.static.elements
                }));
                rs.drawVAO = new e1(null, null, null, function(no, fo) {
                  return no.link(Cs);
                }), rs.useVAO = !0;
              }
            }
            return Ea ? rs.useVAO = !0 : rs.attributes = N1(yi, Ia), rs.context = Xu(Ma), rs;
          }
          function Qu(Di, yi, ca) {
            var Ma = Di.shared, Ia = Ma.context, ba = Di.scope();
            Object.keys(ca).forEach(function(fa) {
              yi.save(Ia, "." + fa);
              var Ea = ca[fa], Hi = Ea.append(Di, yi);
              Array.isArray(Hi) ? ba(Ia, ".", fa, "=[", Hi.join(), "];") : ba(Ia, ".", fa, "=", Hi, ";");
            }), yi(ba);
          }
          function r1(Di, yi, ca, Ma) {
            var Ia = Di.shared, ba = Ia.gl, fa = Ia.framebuffer, Ea;
            Ss && (Ea = yi.def(Ia.extensions, ".webgl_draw_buffers"));
            var Hi = Di.constants, ji = Hi.drawBuffer, qi = Hi.backBuffer, Ua;
            ca ? Ua = ca.append(Di, yi) : Ua = yi.def(fa, ".next"), Ma || yi("if(", Ua, "!==", fa, ".cur){"), yi(
              "if(",
              Ua,
              "){",
              ba,
              ".bindFramebuffer(",
              Rh,
              ",",
              Ua,
              ".framebuffer);"
            ), Ss && yi(
              Ea,
              ".drawBuffersWEBGL(",
              ji,
              "[",
              Ua,
              ".colorAttachments.length]);"
            ), yi(
              "}else{",
              ba,
              ".bindFramebuffer(",
              Rh,
              ",null);"
            ), Ss && yi(Ea, ".drawBuffersWEBGL(", qi, ");"), yi(
              "}",
              fa,
              ".cur=",
              Ua,
              ";"
            ), Ma || yi("}");
          }
          function ud(Di, yi, ca) {
            var Ma = Di.shared, Ia = Ma.gl, ba = Di.current, fa = Di.next, Ea = Ma.current, Hi = Ma.next, ji = Di.cond(Ea, ".dirty");
            Vs.forEach(function(qi) {
              var Ua = Qs(qi);
              if (!(Ua in ca.state)) {
                var La, yo;
                if (Ua in fa) {
                  La = fa[Ua], yo = ba[Ua];
                  var xo = wn(hs[Ua].length, function(Us) {
                    return ji.def(La, "[", Us, "]");
                  });
                  ji(Di.cond(xo.map(function(Us, sl) {
                    return Us + "!==" + yo + "[" + sl + "]";
                  }).join("||")).then(
                    Ia,
                    ".",
                    go[Ua],
                    "(",
                    xo,
                    ");",
                    xo.map(function(Us, sl) {
                      return yo + "[" + sl + "]=" + Us;
                    }).join(";"),
                    ";"
                  ));
                } else {
                  La = ji.def(Hi, ".", Ua);
                  var rs = Di.cond(La, "!==", Ea, ".", Ua);
                  ji(rs), Ua in vo ? rs(
                    Di.cond(La).then(Ia, ".enable(", vo[Ua], ");").else(Ia, ".disable(", vo[Ua], ");"),
                    Ea,
                    ".",
                    Ua,
                    "=",
                    La,
                    ";"
                  ) : rs(
                    Ia,
                    ".",
                    go[Ua],
                    "(",
                    La,
                    ");",
                    Ea,
                    ".",
                    Ua,
                    "=",
                    La,
                    ";"
                  );
                }
              }
            }), Object.keys(ca.state).length === 0 && ji(Ea, ".dirty=false;"), yi(ji);
          }
          function Ed(Di, yi, ca, Ma) {
            var Ia = Di.shared, ba = Di.current, fa = Ia.current, Ea = Ia.gl;
            Mf(Object.keys(ca)).forEach(function(Hi) {
              var ji = ca[Hi];
              if (!(Ma && !Ma(ji))) {
                var qi = ji.append(Di, yi);
                if (vo[Hi]) {
                  var Ua = vo[Hi];
                  Id(ji) ? qi ? yi(Ea, ".enable(", Ua, ");") : yi(Ea, ".disable(", Ua, ");") : yi(Di.cond(qi).then(Ea, ".enable(", Ua, ");").else(Ea, ".disable(", Ua, ");")), yi(fa, ".", Hi, "=", qi, ";");
                } else if (Sl(qi)) {
                  var La = ba[Hi];
                  yi(
                    Ea,
                    ".",
                    go[Hi],
                    "(",
                    qi,
                    ");",
                    qi.map(function(yo, xo) {
                      return La + "[" + xo + "]=" + yo;
                    }).join(";"),
                    ";"
                  );
                } else
                  yi(
                    Ea,
                    ".",
                    go[Hi],
                    "(",
                    qi,
                    ");",
                    fa,
                    ".",
                    Hi,
                    "=",
                    qi,
                    ";"
                  );
              }
            });
          }
          function R1(Di, yi) {
            qo && (Di.instancing = yi.def(
              Di.shared.extensions,
              ".angle_instanced_arrays"
            ));
          }
          function jl(Di, yi, ca, Ma, Ia) {
            var ba = Di.shared, fa = Di.stats, Ea = ba.current, Hi = ba.timer, ji = ca.profile;
            function qi() {
              return typeof performance > "u" ? "Date.now()" : "performance.now()";
            }
            var Ua, La;
            function yo(no) {
              Ua = yi.def(), no(Ua, "=", qi(), ";"), typeof Ia == "string" ? no(fa, ".count+=", Ia, ";") : no(fa, ".count++;"), Ao && (Ma ? (La = yi.def(), no(La, "=", Hi, ".getNumPendingQueries();")) : no(Hi, ".beginQuery(", fa, ");"));
            }
            function xo(no) {
              no(fa, ".cpuTime+=", qi(), "-", Ua, ";"), Ao && (Ma ? no(
                Hi,
                ".pushScopeStats(",
                La,
                ",",
                Hi,
                ".getNumPendingQueries(),",
                fa,
                ");"
              ) : no(Hi, ".endQuery();"));
            }
            function rs(no) {
              var fo = yi.def(Ea, ".profile");
              yi(Ea, ".profile=", no, ";"), yi.exit(Ea, ".profile=", fo, ";");
            }
            var Us;
            if (ji) {
              if (Id(ji)) {
                ji.enable ? (yo(yi), xo(yi.exit), rs("true")) : rs("false");
                return;
              }
              Us = ji.append(Di, yi), rs(Us);
            } else
              Us = yi.def(Ea, ".profile");
            var sl = Di.block();
            yo(sl), yi("if(", Us, "){", sl, "}");
            var Cs = Di.block();
            xo(Cs), yi.exit("if(", Us, "){", Cs, "}");
          }
          function Ad(Di, yi, ca, Ma, Ia) {
            var ba = Di.shared;
            function fa(Hi) {
              switch (Hi) {
                case rf:
                case Yd:
                case qd:
                  return 2;
                case Ud:
                case af:
                case Cf:
                  return 3;
                case $d:
                case of:
                case V1:
                  return 4;
                default:
                  return 1;
              }
            }
            function Ea(Hi, ji, qi) {
              var Ua = ba.gl, La = yi.def(Hi, ".location"), yo = yi.def(ba.attributes, "[", La, "]"), xo = qi.state, rs = qi.buffer, Us = [
                qi.x,
                qi.y,
                qi.z,
                qi.w
              ], sl = [
                "buffer",
                "normalized",
                "offset",
                "stride"
              ];
              function Cs() {
                yi(
                  "if(!",
                  yo,
                  ".buffer){",
                  Ua,
                  ".enableVertexAttribArray(",
                  La,
                  ");}"
                );
                var fo = qi.type, Ys;
                if (qi.size ? Ys = yi.def(qi.size, "||", ji) : Ys = ji, yi(
                  "if(",
                  yo,
                  ".type!==",
                  fo,
                  "||",
                  yo,
                  ".size!==",
                  Ys,
                  "||",
                  sl.map(function(js) {
                    return yo + "." + js + "!==" + qi[js];
                  }).join("||"),
                  "){",
                  Ua,
                  ".bindBuffer(",
                  od,
                  ",",
                  rs,
                  ".buffer);",
                  Ua,
                  ".vertexAttribPointer(",
                  [
                    La,
                    Ys,
                    fo,
                    qi.normalized,
                    qi.stride,
                    qi.offset
                  ],
                  ");",
                  yo,
                  ".type=",
                  fo,
                  ";",
                  yo,
                  ".size=",
                  Ys,
                  ";",
                  sl.map(function(js) {
                    return yo + "." + js + "=" + qi[js] + ";";
                  }).join(""),
                  "}"
                ), qo) {
                  var Rs = qi.divisor;
                  yi(
                    "if(",
                    yo,
                    ".divisor!==",
                    Rs,
                    "){",
                    Di.instancing,
                    ".vertexAttribDivisorANGLE(",
                    [La, Rs],
                    ");",
                    yo,
                    ".divisor=",
                    Rs,
                    ";}"
                  );
                }
              }
              function no() {
                yi(
                  "if(",
                  yo,
                  ".buffer){",
                  Ua,
                  ".disableVertexAttribArray(",
                  La,
                  ");",
                  yo,
                  ".buffer=null;",
                  "}if(",
                  tn.map(function(fo, Ys) {
                    return yo + "." + fo + "!==" + Us[Ys];
                  }).join("||"),
                  "){",
                  Ua,
                  ".vertexAttrib4f(",
                  La,
                  ",",
                  Us,
                  ");",
                  tn.map(function(fo, Ys) {
                    return yo + "." + fo + "=" + Us[Ys] + ";";
                  }).join(""),
                  "}"
                );
              }
              xo === An ? Cs() : xo === On ? no() : (yi("if(", xo, "===", An, "){"), Cs(), yi("}else{"), no(), yi("}"));
            }
            Ma.forEach(function(Hi) {
              var ji = Hi.name, qi = ca.attributes[ji], Ua;
              if (qi) {
                if (!Ia(qi))
                  return;
                Ua = qi.append(Di, yi);
              } else {
                if (!Ia(gf))
                  return;
                var La = Di.scopeAttrib(ji);
                vt.optional(function() {
                  Di.assert(
                    yi,
                    La + ".state",
                    "missing attribute " + ji
                  );
                }), Ua = {}, Object.keys(new Qa()).forEach(function(yo) {
                  Ua[yo] = yi.def(La, ".", yo);
                });
              }
              Ea(
                Di.link(Hi),
                fa(Hi.info.type),
                Ua
              );
            });
          }
          function Fc(Di, yi, ca, Ma, Ia, ba) {
            for (var fa = Di.shared, Ea = fa.gl, Hi = {}, ji, qi = 0; qi < Ma.length; ++qi) {
              var Ua = Ma[qi], La = Ua.name, yo = Ua.info.type, xo = Ua.info.size, rs = ca.uniforms[La];
              if (xo > 1) {
                if (!rs)
                  continue;
                var Us = La.replace("[0]", "");
                if (Hi[Us])
                  continue;
                Hi[Us] = 1;
              }
              var sl = Di.link(Ua), Cs = sl + ".location", no;
              if (rs) {
                if (!Ia(rs))
                  continue;
                if (Id(rs)) {
                  var fo = rs.value;
                  if (vt.command(
                    fo !== null && typeof fo < "u",
                    'missing uniform "' + La + '"',
                    Di.commandStr
                  ), yo === Xd || yo === ld) {
                    vt.command(
                      typeof fo == "function" && (yo === Xd && (fo._reglType === "texture2d" || fo._reglType === "framebuffer") || yo === ld && (fo._reglType === "textureCube" || fo._reglType === "framebufferCube")),
                      "invalid texture for uniform " + La,
                      Di.commandStr
                    );
                    var Ys = Di.link(fo._texture || fo.color[0]._texture);
                    yi(Ea, ".uniform1i(", Cs, ",", Ys + ".bind());"), yi.exit(Ys, ".unbind();");
                  } else if (yo === jd || yo === E1 || yo === sd) {
                    vt.optional(function() {
                      vt.command(
                        Sl(fo),
                        "invalid matrix for uniform " + La,
                        Di.commandStr
                      ), vt.command(
                        yo === jd && fo.length === 4 || yo === E1 && fo.length === 9 || yo === sd && fo.length === 16,
                        "invalid length for matrix uniform " + La,
                        Di.commandStr
                      );
                    });
                    var Rs = Di.global.def("new Float32Array([" + Array.prototype.slice.call(fo) + "])"), js = 2;
                    yo === E1 ? js = 3 : yo === sd && (js = 4), yi(
                      Ea,
                      ".uniformMatrix",
                      js,
                      "fv(",
                      Cs,
                      ",false,",
                      Rs,
                      ");"
                    );
                  } else {
                    switch (yo) {
                      case Uu:
                        xo === 1 ? vt.commandType(fo, "number", "uniform " + La, Di.commandStr) : vt.command(
                          Sl(fo) && fo.length === xo,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "1f";
                        break;
                      case rf:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 2 === 0 && fo.length <= xo * 2,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "2f";
                        break;
                      case Ud:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 3 === 0 && fo.length <= xo * 3,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "3f";
                        break;
                      case $d:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 4 === 0 && fo.length <= xo * 4,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "4f";
                        break;
                      case Bf:
                        xo === 1 ? vt.commandType(fo, "boolean", "uniform " + La, Di.commandStr) : vt.command(
                          Sl(fo) && fo.length === xo,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "1i";
                        break;
                      case Sd:
                        xo === 1 ? vt.commandType(fo, "number", "uniform " + La, Di.commandStr) : vt.command(
                          Sl(fo) && fo.length === xo,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "1i";
                        break;
                      case qd:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 2 === 0 && fo.length <= xo * 2,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "2i";
                        break;
                      case Yd:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 2 === 0 && fo.length <= xo * 2,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "2i";
                        break;
                      case Cf:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 3 === 0 && fo.length <= xo * 3,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "3i";
                        break;
                      case af:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 3 === 0 && fo.length <= xo * 3,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "3i";
                        break;
                      case V1:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 4 === 0 && fo.length <= xo * 4,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "4i";
                        break;
                      case of:
                        vt.command(
                          Sl(fo) && fo.length && fo.length % 4 === 0 && fo.length <= xo * 4,
                          "uniform " + La,
                          Di.commandStr
                        ), ji = "4i";
                        break;
                    }
                    xo > 1 ? (ji += "v", fo = Di.global.def("[" + Array.prototype.slice.call(fo) + "]")) : fo = Sl(fo) ? Array.prototype.slice.call(fo) : fo, yi(
                      Ea,
                      ".uniform",
                      ji,
                      "(",
                      Cs,
                      ",",
                      fo,
                      ");"
                    );
                  }
                  continue;
                } else
                  no = rs.append(Di, yi);
              } else {
                if (!Ia(gf))
                  continue;
                no = yi.def(fa.uniforms, "[", oi.id(La), "]");
              }
              yo === Xd ? (vt(!Array.isArray(no), "must specify a scalar prop for textures"), yi(
                "if(",
                no,
                "&&",
                no,
                '._reglType==="framebuffer"){',
                no,
                "=",
                no,
                ".color[0];",
                "}"
              )) : yo === ld && (vt(!Array.isArray(no), "must specify a scalar prop for cube maps"), yi(
                "if(",
                no,
                "&&",
                no,
                '._reglType==="framebufferCube"){',
                no,
                "=",
                no,
                ".color[0];",
                "}"
              )), vt.optional(function() {
                function Kd(P1, Nh) {
                  Di.assert(
                    yi,
                    P1,
                    'bad data or missing for uniform "' + La + '".  ' + Nh
                  );
                }
                function mh(P1, Nh) {
                  Nh === 1 && vt(!Array.isArray(no), "must not specify an array type for uniform"), Kd(
                    "Array.isArray(" + no + ") && typeof " + no + '[0]===" ' + P1 + '" || typeof ' + no + '==="' + P1 + '"',
                    "invalid type, expected " + P1
                  );
                }
                function Jd(P1, Nh, qf) {
                  Array.isArray(no) ? vt(no.length && no.length % P1 === 0 && no.length <= P1 * qf, "must have length of " + (qf === 1 ? "" : "n * ") + P1) : Kd(
                    fa.isArrayLike + "(" + no + ")&&" + no + ".length && " + no + ".length % " + P1 + " === 0 && " + no + ".length<=" + P1 * qf,
                    "invalid vector, should have length of " + (qf === 1 ? "" : "n * ") + P1,
                    Di.commandStr
                  );
                }
                function Dh(P1) {
                  vt(!Array.isArray(no), "must not specify a value type"), Kd(
                    "typeof " + no + '==="function"&&' + no + '._reglType==="texture' + (P1 === nf ? "2d" : "Cube") + '"',
                    "invalid texture type",
                    Di.commandStr
                  );
                }
                switch (yo) {
                  case Sd:
                    mh("number", xo);
                    break;
                  case Yd:
                    Jd(2, "number", xo);
                    break;
                  case af:
                    Jd(3, "number", xo);
                    break;
                  case of:
                    Jd(4, "number", xo);
                    break;
                  case Uu:
                    mh("number", xo);
                    break;
                  case rf:
                    Jd(2, "number", xo);
                    break;
                  case Ud:
                    Jd(3, "number", xo);
                    break;
                  case $d:
                    Jd(4, "number", xo);
                    break;
                  case Bf:
                    mh("boolean", xo);
                    break;
                  case qd:
                    Jd(2, "boolean", xo);
                    break;
                  case Cf:
                    Jd(3, "boolean", xo);
                    break;
                  case V1:
                    Jd(4, "boolean", xo);
                    break;
                  case jd:
                    Jd(4, "number", xo);
                    break;
                  case E1:
                    Jd(9, "number", xo);
                    break;
                  case sd:
                    Jd(16, "number", xo);
                    break;
                  case Xd:
                    Dh(nf);
                    break;
                  case ld:
                    Dh(Vd);
                    break;
                }
              });
              var Mc = 1;
              switch (yo) {
                case Xd:
                case ld:
                  var dd = yi.def(no, "._texture");
                  yi(Ea, ".uniform1i(", Cs, ",", dd, ".bind());"), yi.exit(dd, ".unbind();");
                  continue;
                case Sd:
                case Bf:
                  ji = "1i";
                  break;
                case Yd:
                case qd:
                  ji = "2i", Mc = 2;
                  break;
                case af:
                case Cf:
                  ji = "3i", Mc = 3;
                  break;
                case of:
                case V1:
                  ji = "4i", Mc = 4;
                  break;
                case Uu:
                  ji = "1f";
                  break;
                case rf:
                  ji = "2f", Mc = 2;
                  break;
                case Ud:
                  ji = "3f", Mc = 3;
                  break;
                case $d:
                  ji = "4f", Mc = 4;
                  break;
                case jd:
                  ji = "Matrix2fv";
                  break;
                case E1:
                  ji = "Matrix3fv";
                  break;
                case sd:
                  ji = "Matrix4fv";
                  break;
              }
              if (ji.indexOf("Matrix") === -1 && xo > 1 && (ji += "v", Mc = 1), ji.charAt(0) === "M") {
                yi(Ea, ".uniform", ji, "(", Cs, ",");
                var nh = Math.pow(yo - jd + 2, 2), Vf = Di.global.def("new Float32Array(", nh, ")");
                Array.isArray(no) ? yi(
                  "false,(",
                  wn(nh, function(Kd) {
                    return Vf + "[" + Kd + "]=" + no[Kd];
                  }),
                  ",",
                  Vf,
                  ")"
                ) : yi(
                  "false,(Array.isArray(",
                  no,
                  ")||",
                  no,
                  " instanceof Float32Array)?",
                  no,
                  ":(",
                  wn(nh, function(Kd) {
                    return Vf + "[" + Kd + "]=" + no + "[" + Kd + "]";
                  }),
                  ",",
                  Vf,
                  ")"
                ), yi(");");
              } else if (Mc > 1) {
                for (var vf = [], Uf = [], rh = 0; rh < Mc; ++rh)
                  Array.isArray(no) ? Uf.push(no[rh]) : Uf.push(yi.def(no + "[" + rh + "]")), ba && vf.push(yi.def());
                ba && yi("if(!", Di.batchId, "||", vf.map(function(Kd, mh) {
                  return Kd + "!==" + Uf[mh];
                }).join("||"), "){", vf.map(function(Kd, mh) {
                  return Kd + "=" + Uf[mh] + ";";
                }).join("")), yi(Ea, ".uniform", ji, "(", Cs, ",", Uf.join(","), ");"), ba && yi("}");
              } else {
                if (vt(!Array.isArray(no), "uniform value must not be an array"), ba) {
                  var B2 = yi.def();
                  yi(
                    "if(!",
                    Di.batchId,
                    "||",
                    B2,
                    "!==",
                    no,
                    "){",
                    B2,
                    "=",
                    no,
                    ";"
                  );
                }
                yi(Ea, ".uniform", ji, "(", Cs, ",", no, ");"), ba && yi("}");
              }
            }
          }
          function yl(Di, yi, ca, Ma) {
            var Ia = Di.shared, ba = Ia.gl, fa = Ia.draw, Ea = Ma.draw;
            function Hi() {
              var Ys = Ea.elements, Rs, js = yi;
              return Ys ? ((Ys.contextDep && Ma.contextDynamic || Ys.propDep) && (js = ca), Rs = Ys.append(Di, js), Ea.elementsActive && js(
                "if(" + Rs + ")" + ba + ".bindBuffer(" + yd + "," + Rs + ".buffer.buffer);"
              )) : (Rs = js.def(), js(
                Rs,
                "=",
                fa,
                ".",
                xl,
                ";",
                "if(",
                Rs,
                "){",
                ba,
                ".bindBuffer(",
                yd,
                ",",
                Rs,
                ".buffer.buffer);}",
                "else if(",
                Ia.vao,
                ".currentVAO){",
                Rs,
                "=",
                Di.shared.elements + ".getElements(" + Ia.vao,
                ".currentVAO.elements);",
                ho ? "" : "if(" + Rs + ")" + ba + ".bindBuffer(" + yd + "," + Rs + ".buffer.buffer);",
                "}"
              )), Rs;
            }
            function ji() {
              var Ys = Ea.count, Rs, js = yi;
              return Ys ? ((Ys.contextDep && Ma.contextDynamic || Ys.propDep) && (js = ca), Rs = Ys.append(Di, js), vt.optional(function() {
                Ys.MISSING && Di.assert(yi, "false", "missing vertex count"), Ys.DYNAMIC && Di.assert(js, Rs + ">=0", "missing vertex count");
              })) : (Rs = js.def(fa, ".", al), vt.optional(function() {
                Di.assert(js, Rs + ">=0", "missing vertex count");
              })), Rs;
            }
            var qi = Hi();
            function Ua(Ys) {
              var Rs = Ea[Ys];
              return Rs ? Rs.contextDep && Ma.contextDynamic || Rs.propDep ? Rs.append(Di, ca) : Rs.append(Di, yi) : yi.def(fa, ".", Ys);
            }
            var La = Ua(Bl), yo = Ua(Oc), xo = ji();
            if (typeof xo == "number") {
              if (xo === 0)
                return;
            } else
              ca("if(", xo, "){"), ca.exit("}");
            var rs, Us;
            qo && (rs = Ua(nu), Us = Di.instancing);
            var sl = qi + ".type", Cs = Ea.elements && Id(Ea.elements) && !Ea.vaoActive;
            function no() {
              function Ys() {
                ca(Us, ".drawElementsInstancedANGLE(", [
                  La,
                  xo,
                  sl,
                  yo + "<<((" + sl + "-" + xn + ")>>1)",
                  rs
                ], ");");
              }
              function Rs() {
                ca(
                  Us,
                  ".drawArraysInstancedANGLE(",
                  [La, yo, xo, rs],
                  ");"
                );
              }
              qi && qi !== "null" ? Cs ? Ys() : (ca("if(", qi, "){"), Ys(), ca("}else{"), Rs(), ca("}")) : Rs();
            }
            function fo() {
              function Ys() {
                ca(ba + ".drawElements(" + [
                  La,
                  xo,
                  sl,
                  yo + "<<((" + sl + "-" + xn + ")>>1)"
                ] + ");");
              }
              function Rs() {
                ca(ba + ".drawArrays(" + [La, yo, xo] + ");");
              }
              qi && qi !== "null" ? Cs ? Ys() : (ca("if(", qi, "){"), Ys(), ca("}else{"), Rs(), ca("}")) : Rs();
            }
            qo && (typeof rs != "number" || rs >= 0) ? typeof rs == "string" ? (ca("if(", rs, ">0){"), no(), ca("}else if(", rs, "<0){"), fo(), ca("}")) : no() : fo();
          }
          function cc(Di, yi, ca, Ma, Ia) {
            var ba = ol(), fa = ba.proc("body", Ia);
            return vt.optional(function() {
              ba.commandStr = yi.commandStr, ba.command = ba.link(yi.commandStr);
            }), qo && (ba.instancing = fa.def(
              ba.shared.extensions,
              ".angle_instanced_arrays"
            )), Di(ba, fa, ca, Ma), ba.compile().body;
          }
          function Bc(Di, yi, ca, Ma) {
            R1(Di, yi), ca.useVAO ? ca.drawVAO ? yi(Di.shared.vao, ".setVAO(", ca.drawVAO.append(Di, yi), ");") : yi(Di.shared.vao, ".setVAO(", Di.shared.vao, ".targetVAO);") : (yi(Di.shared.vao, ".setVAO(null);"), Ad(Di, yi, ca, Ma.attributes, function() {
              return !0;
            })), Fc(Di, yi, ca, Ma.uniforms, function() {
              return !0;
            }, !1), yl(Di, yi, yi, ca);
          }
          function U1(Di, yi) {
            var ca = Di.proc("draw", 1);
            R1(Di, ca), Qu(Di, ca, yi.context), r1(Di, ca, yi.framebuffer), ud(Di, ca, yi), Ed(Di, ca, yi.state), jl(Di, ca, yi, !1, !0);
            var Ma = yi.shader.progVar.append(Di, ca);
            if (ca(Di.shared.gl, ".useProgram(", Ma, ".program);"), yi.shader.program)
              Bc(Di, ca, yi, yi.shader.program);
            else {
              ca(Di.shared.vao, ".setVAO(null);");
              var Ia = Di.global.def("{}"), ba = ca.def(Ma, ".id"), fa = ca.def(Ia, "[", ba, "]");
              ca(
                Di.cond(fa).then(fa, ".call(this,a0);").else(
                  fa,
                  "=",
                  Ia,
                  "[",
                  ba,
                  "]=",
                  Di.link(function(Ea) {
                    return cc(Bc, Di, yi, Ea, 1);
                  }),
                  "(",
                  Ma,
                  ");",
                  fa,
                  ".call(this,a0);"
                )
              );
            }
            Object.keys(yi.state).length > 0 && ca(Di.shared.current, ".dirty=true;"), Di.shared.vao && ca(Di.shared.vao, ".setVAO(null);");
          }
          function uf(Di, yi, ca, Ma) {
            Di.batchId = "a1", R1(Di, yi);
            function Ia() {
              return !0;
            }
            Ad(Di, yi, ca, Ma.attributes, Ia), Fc(Di, yi, ca, Ma.uniforms, Ia, !1), yl(Di, yi, yi, ca);
          }
          function df(Di, yi, ca, Ma) {
            R1(Di, yi);
            var Ia = ca.contextDep, ba = yi.def(), fa = "a0", Ea = "a1", Hi = yi.def();
            Di.shared.props = Hi, Di.batchId = ba;
            var ji = Di.scope(), qi = Di.scope();
            yi(
              ji.entry,
              "for(",
              ba,
              "=0;",
              ba,
              "<",
              Ea,
              ";++",
              ba,
              "){",
              Hi,
              "=",
              fa,
              "[",
              ba,
              "];",
              qi,
              "}",
              ji.exit
            );
            function Ua(sl) {
              return sl.contextDep && Ia || sl.propDep;
            }
            function La(sl) {
              return !Ua(sl);
            }
            if (ca.needsContext && Qu(Di, qi, ca.context), ca.needsFramebuffer && r1(Di, qi, ca.framebuffer), Ed(Di, qi, ca.state, Ua), ca.profile && Ua(ca.profile) && jl(Di, qi, ca, !1, !0), Ma)
              ca.useVAO ? ca.drawVAO ? Ua(ca.drawVAO) ? qi(Di.shared.vao, ".setVAO(", ca.drawVAO.append(Di, qi), ");") : ji(Di.shared.vao, ".setVAO(", ca.drawVAO.append(Di, ji), ");") : ji(Di.shared.vao, ".setVAO(", Di.shared.vao, ".targetVAO);") : (ji(Di.shared.vao, ".setVAO(null);"), Ad(Di, ji, ca, Ma.attributes, La), Ad(Di, qi, ca, Ma.attributes, Ua)), Fc(Di, ji, ca, Ma.uniforms, La, !1), Fc(Di, qi, ca, Ma.uniforms, Ua, !0), yl(Di, ji, qi, ca);
            else {
              var yo = Di.global.def("{}"), xo = ca.shader.progVar.append(Di, qi), rs = qi.def(xo, ".id"), Us = qi.def(yo, "[", rs, "]");
              qi(
                Di.shared.gl,
                ".useProgram(",
                xo,
                ".program);",
                "if(!",
                Us,
                "){",
                Us,
                "=",
                yo,
                "[",
                rs,
                "]=",
                Di.link(function(sl) {
                  return cc(
                    uf,
                    Di,
                    ca,
                    sl,
                    2
                  );
                }),
                "(",
                xo,
                ");}",
                Us,
                ".call(this,a0[",
                ba,
                "],",
                ba,
                ");"
              );
            }
          }
          function Xi(Di, yi) {
            var ca = Di.proc("batch", 2);
            Di.batchId = "0", R1(Di, ca);
            var Ma = !1, Ia = !0;
            Object.keys(yi.context).forEach(function(yo) {
              Ma = Ma || yi.context[yo].propDep;
            }), Ma || (Qu(Di, ca, yi.context), Ia = !1);
            var ba = yi.framebuffer, fa = !1;
            ba ? (ba.propDep ? Ma = fa = !0 : ba.contextDep && Ma && (fa = !0), fa || r1(Di, ca, ba)) : r1(Di, ca, null), yi.state.viewport && yi.state.viewport.propDep && (Ma = !0);
            function Ea(yo) {
              return yo.contextDep && Ma || yo.propDep;
            }
            ud(Di, ca, yi), Ed(Di, ca, yi.state, function(yo) {
              return !Ea(yo);
            }), (!yi.profile || !Ea(yi.profile)) && jl(Di, ca, yi, !1, "a1"), yi.contextDep = Ma, yi.needsContext = Ia, yi.needsFramebuffer = fa;
            var Hi = yi.shader.progVar;
            if (Hi.contextDep && Ma || Hi.propDep)
              df(
                Di,
                ca,
                yi,
                null
              );
            else {
              var ji = Hi.append(Di, ca);
              if (ca(Di.shared.gl, ".useProgram(", ji, ".program);"), yi.shader.program)
                df(
                  Di,
                  ca,
                  yi,
                  yi.shader.program
                );
              else {
                ca(Di.shared.vao, ".setVAO(null);");
                var qi = Di.global.def("{}"), Ua = ca.def(ji, ".id"), La = ca.def(qi, "[", Ua, "]");
                ca(
                  Di.cond(La).then(La, ".call(this,a0,a1);").else(
                    La,
                    "=",
                    qi,
                    "[",
                    Ua,
                    "]=",
                    Di.link(function(yo) {
                      return cc(df, Di, yi, yo, 2);
                    }),
                    "(",
                    ji,
                    ");",
                    La,
                    ".call(this,a0,a1);"
                  )
                );
              }
            }
            Object.keys(yi.state).length > 0 && ca(Di.shared.current, ".dirty=true;"), Di.shared.vao && ca(Di.shared.vao, ".setVAO(null);");
          }
          function Za(Di, yi) {
            var ca = Di.proc("scope", 3);
            Di.batchId = "a2";
            var Ma = Di.shared, Ia = Ma.current;
            Qu(Di, ca, yi.context), yi.framebuffer && yi.framebuffer.append(Di, ca), Mf(Object.keys(yi.state)).forEach(function(fa) {
              var Ea = yi.state[fa], Hi = Ea.append(Di, ca);
              Sl(Hi) ? Hi.forEach(function(ji, qi) {
                ca.set(Di.next[fa], "[" + qi + "]", ji);
              }) : ca.set(Ma.next, "." + fa, Hi);
            }), jl(Di, ca, yi, !0, !0), [xl, Oc, al, nu, Bl].forEach(
              function(fa) {
                var Ea = yi.draw[fa];
                Ea && ca.set(Ma.draw, "." + fa, "" + Ea.append(Di, ca));
              }
            ), Object.keys(yi.uniforms).forEach(function(fa) {
              var Ea = yi.uniforms[fa].append(Di, ca);
              Array.isArray(Ea) && (Ea = "[" + Ea.join() + "]"), ca.set(
                Ma.uniforms,
                "[" + oi.id(fa) + "]",
                Ea
              );
            }), Object.keys(yi.attributes).forEach(function(fa) {
              var Ea = yi.attributes[fa].append(Di, ca), Hi = Di.scopeAttrib(fa);
              Object.keys(new Qa()).forEach(function(ji) {
                ca.set(Hi, "." + ji, Ea[ji]);
              });
            }), yi.scopeVAO && ca.set(Ma.vao, ".targetVAO", yi.scopeVAO.append(Di, ca));
            function ba(fa) {
              var Ea = yi.shader[fa];
              Ea && ca.set(Ma.shader, "." + fa, Ea.append(Di, ca));
            }
            ba(pc), ba(hl), Object.keys(yi.state).length > 0 && (ca(Ia, ".dirty=true;"), ca.exit(Ia, ".dirty=true;")), ca("a1(", Di.shared.context, ",a0,", Di.batchId, ");");
          }
          function ja(Di) {
            if (!(typeof Di != "object" || Sl(Di))) {
              for (var yi = Object.keys(Di), ca = 0; ca < yi.length; ++ca)
                if (Yt.isDynamic(Di[yi[ca]]))
                  return !0;
              return !1;
            }
          }
          function Ds(Di, yi, ca) {
            var Ma = yi.static[ca];
            if (!Ma || !ja(Ma))
              return;
            var Ia = Di.global, ba = Object.keys(Ma), fa = !1, Ea = !1, Hi = !1, ji = Di.global.def("{}");
            ba.forEach(function(Ua) {
              var La = Ma[Ua];
              if (Yt.isDynamic(La)) {
                typeof La == "function" && (La = Ma[Ua] = Yt.unbox(La));
                var yo = pu(La, null);
                fa = fa || yo.thisDep, Hi = Hi || yo.propDep, Ea = Ea || yo.contextDep;
              } else {
                switch (Ia(ji, ".", Ua, "="), typeof La) {
                  case "number":
                    Ia(La);
                    break;
                  case "string":
                    Ia('"', La, '"');
                    break;
                  case "object":
                    Array.isArray(La) && Ia("[", La.join(), "]");
                    break;
                  default:
                    Ia(Di.link(La));
                    break;
                }
                Ia(";");
              }
            });
            function qi(Ua, La) {
              ba.forEach(function(yo) {
                var xo = Ma[yo];
                if (Yt.isDynamic(xo)) {
                  var rs = Ua.invoke(La, xo);
                  La(ji, ".", yo, "=", rs, ";");
                }
              });
            }
            yi.dynamic[ca] = new Yt.DynamicVariable(Un, {
              thisDep: fa,
              contextDep: Ea,
              propDep: Hi,
              ref: ji,
              append: qi
            }), delete yi.static[ca];
          }
          function rc(Di, yi, ca, Ma, Ia) {
            var ba = ol();
            ba.stats = ba.link(Ia), Object.keys(yi.static).forEach(function(Ea) {
              Ds(ba, yi, Ea);
            }), wd.forEach(function(Ea) {
              Ds(ba, Di, Ea);
            });
            var fa = Q1(Di, yi, ca, Ma, ba);
            return U1(ba, fa), Za(ba, fa), Xi(ba, fa), V(ba.compile(), {
              destroy: function() {
                fa.shader.program.destroy();
              }
            });
          }
          return {
            next: Bo,
            current: hs,
            procs: function() {
              var Di = ol(), yi = Di.proc("poll"), ca = Di.proc("refresh"), Ma = Di.block();
              yi(Ma), ca(Ma);
              var Ia = Di.shared, ba = Ia.gl, fa = Ia.next, Ea = Ia.current;
              Ma(Ea, ".dirty=false;"), r1(Di, yi), r1(Di, ca, null, !0);
              var Hi;
              qo && (Hi = Di.link(qo)), ar.oes_vertex_array_object && ca(Di.link(ar.oes_vertex_array_object), ".bindVertexArrayOES(null);");
              for (var ji = 0; ji < Sa.maxAttributes; ++ji) {
                var qi = ca.def(Ia.attributes, "[", ji, "]"), Ua = Di.cond(qi, ".buffer");
                Ua.then(
                  ba,
                  ".enableVertexAttribArray(",
                  ji,
                  ");",
                  ba,
                  ".bindBuffer(",
                  od,
                  ",",
                  qi,
                  ".buffer.buffer);",
                  ba,
                  ".vertexAttribPointer(",
                  ji,
                  ",",
                  qi,
                  ".size,",
                  qi,
                  ".type,",
                  qi,
                  ".normalized,",
                  qi,
                  ".stride,",
                  qi,
                  ".offset);"
                ).else(
                  ba,
                  ".disableVertexAttribArray(",
                  ji,
                  ");",
                  ba,
                  ".vertexAttrib4f(",
                  ji,
                  ",",
                  qi,
                  ".x,",
                  qi,
                  ".y,",
                  qi,
                  ".z,",
                  qi,
                  ".w);",
                  qi,
                  ".buffer=null;"
                ), ca(Ua), qo && ca(
                  Hi,
                  ".vertexAttribDivisorANGLE(",
                  ji,
                  ",",
                  qi,
                  ".divisor);"
                );
              }
              return ca(
                Di.shared.vao,
                ".currentVAO=null;",
                Di.shared.vao,
                ".setVAO(",
                Di.shared.vao,
                ".targetVAO);"
              ), Object.keys(vo).forEach(function(La) {
                var yo = vo[La], xo = Ma.def(fa, ".", La), rs = Di.block();
                rs(
                  "if(",
                  xo,
                  "){",
                  ba,
                  ".enable(",
                  yo,
                  ")}else{",
                  ba,
                  ".disable(",
                  yo,
                  ")}",
                  Ea,
                  ".",
                  La,
                  "=",
                  xo,
                  ";"
                ), ca(rs), yi(
                  "if(",
                  xo,
                  "!==",
                  Ea,
                  ".",
                  La,
                  "){",
                  rs,
                  "}"
                );
              }), Object.keys(go).forEach(function(La) {
                var yo = go[La], xo = hs[La], rs, Us, sl = Di.block();
                if (sl(ba, ".", yo, "("), Sl(xo)) {
                  var Cs = xo.length;
                  rs = Di.global.def(fa, ".", La), Us = Di.global.def(Ea, ".", La), sl(
                    wn(Cs, function(no) {
                      return rs + "[" + no + "]";
                    }),
                    ");",
                    wn(Cs, function(no) {
                      return Us + "[" + no + "]=" + rs + "[" + no + "];";
                    }).join("")
                  ), yi(
                    "if(",
                    wn(Cs, function(no) {
                      return rs + "[" + no + "]!==" + Us + "[" + no + "]";
                    }).join("||"),
                    "){",
                    sl,
                    "}"
                  );
                } else
                  rs = Ma.def(fa, ".", La), Us = Ma.def(Ea, ".", La), sl(
                    rs,
                    ");",
                    Ea,
                    ".",
                    La,
                    "=",
                    rs,
                    ";"
                  ), yi(
                    "if(",
                    rs,
                    "!==",
                    Us,
                    "){",
                    sl,
                    "}"
                  );
                ca(sl);
              }), Di.compile();
            }(),
            compile: rc
          };
        }
        function yf() {
          return {
            vaoCount: 0,
            bufferCount: 0,
            elementsCount: 0,
            framebufferCount: 0,
            shaderCount: 0,
            textureCount: 0,
            cubeCount: 0,
            renderbufferCount: 0,
            maxTextureUnits: 0
          };
        }
        var Ih = 34918, Kf = 34919, zf = 35007, cf = function(pr, oi) {
          if (!oi.ext_disjoint_timer_query)
            return null;
          var ar = [];
          function Sa() {
            return ar.pop() || oi.ext_disjoint_timer_query.createQueryEXT();
          }
          function Qn(qo) {
            ar.push(qo);
          }
          var za = [];
          function En(qo) {
            var Ss = Sa();
            oi.ext_disjoint_timer_query.beginQueryEXT(zf, Ss), za.push(Ss), Ao(za.length - 1, za.length, qo);
          }
          function Qo() {
            oi.ext_disjoint_timer_query.endQueryEXT(zf);
          }
          function lo() {
            this.startQueryIndex = -1, this.endQueryIndex = -1, this.sum = 0, this.stats = null;
          }
          var Eo = [];
          function bo() {
            return Eo.pop() || new lo();
          }
          function Ho(qo) {
            Eo.push(qo);
          }
          var Co = [];
          function Ao(qo, Ss, ho) {
            var hs = bo();
            hs.startQueryIndex = qo, hs.endQueryIndex = Ss, hs.sum = 0, hs.stats = ho, Co.push(hs);
          }
          var Ro = [], Qa = [];
          function Ka() {
            var qo, Ss, ho = za.length;
            if (ho !== 0) {
              Qa.length = Math.max(Qa.length, ho + 1), Ro.length = Math.max(Ro.length, ho + 1), Ro[0] = 0, Qa[0] = 0;
              var hs = 0;
              for (qo = 0, Ss = 0; Ss < za.length; ++Ss) {
                var Bo = za[Ss];
                oi.ext_disjoint_timer_query.getQueryObjectEXT(Bo, Kf) ? (hs += oi.ext_disjoint_timer_query.getQueryObjectEXT(Bo, Ih), Qn(Bo)) : za[qo++] = Bo, Ro[Ss + 1] = hs, Qa[Ss + 1] = qo;
              }
              for (za.length = qo, qo = 0, Ss = 0; Ss < Co.length; ++Ss) {
                var Vs = Co[Ss], vo = Vs.startQueryIndex, go = Vs.endQueryIndex;
                Vs.sum += Ro[go] - Ro[vo];
                var Qs = Qa[vo], us = Qa[go];
                us === Qs ? (Vs.stats.gpuTime += Vs.sum / 1e6, Ho(Vs)) : (Vs.startQueryIndex = Qs, Vs.endQueryIndex = us, Co[qo++] = Vs);
              }
              Co.length = qo;
            }
          }
          return {
            beginQuery: En,
            endQuery: Qo,
            pushScopeStats: Ao,
            update: Ka,
            getNumPendingQueries: function() {
              return za.length;
            },
            clear: function() {
              ar.push.apply(ar, za);
              for (var qo = 0; qo < ar.length; qo++)
                oi.ext_disjoint_timer_query.deleteQueryEXT(ar[qo]);
              za.length = 0, ar.length = 0;
            },
            restore: function() {
              za.length = 0, ar.length = 0;
            }
          };
        }, Xh = 16384, P2 = 256, Jf = 1024, eh = 34962, td = "webglcontextlost", Rf = "webglcontextrestored", fh = 1, $2 = 2, F2 = 3;
        function hh(pr, oi) {
          for (var ar = 0; ar < pr.length; ++ar)
            if (pr[ar] === oi)
              return ar;
          return -1;
        }
        function bd(pr) {
          var oi = jt(pr);
          if (!oi)
            return null;
          var ar = oi.gl, Sa = ar.getContextAttributes(), Qn = ar.isContextLost(), za = cn(ar, oi);
          if (!za)
            return null;
          var En = Qt(), Qo = yf(), lo = za.extensions, Eo = cf(ar, lo), bo = hn(), Ho = ar.drawingBufferWidth, Co = ar.drawingBufferHeight, Ao = {
            tick: 0,
            time: 0,
            viewportWidth: Ho,
            viewportHeight: Co,
            framebufferWidth: Ho,
            framebufferHeight: Co,
            drawingBufferWidth: Ho,
            drawingBufferHeight: Co,
            pixelRatio: oi.pixelRatio
          }, Ro = {}, Qa = {
            elements: null,
            primitive: 4,
            // GL_TRIANGLES
            count: -1,
            offset: 0,
            instances: -1
          }, Ka = hi(ar, lo), qo = Mi(
            ar,
            Qo,
            oi,
            hs
          ), Ss = Gs(ar, lo, qo, Qo), ho = zn(
            ar,
            lo,
            Ka,
            Qo,
            qo,
            Ss,
            Qa
          );
          function hs(yl) {
            return ho.destroyBuffer(yl);
          }
          var Bo = Ae(ar, En, Qo, oi), Vs = Ol(
            ar,
            lo,
            Ka,
            function() {
              Qs.procs.poll();
            },
            Ao,
            Qo,
            oi
          ), vo = fu(ar, lo, Ka, Qo, oi), go = lc(
            ar,
            lo,
            Ka,
            Vs,
            vo,
            Qo
          ), Qs = lf(
            ar,
            En,
            lo,
            Ka,
            qo,
            Ss,
            Vs,
            go,
            Ro,
            ho,
            Bo,
            Qa,
            Ao,
            Eo,
            oi
          ), us = ht(
            ar,
            go,
            Qs.procs.poll,
            Ao,
            Sa,
            lo,
            Ka
          ), Po = Qs.next, Jo = ar.canvas, ps = [], wl = [], ol = [], xs = [oi.onDestroy], ns = null;
          function ll() {
            if (ps.length === 0) {
              Eo && Eo.update(), ns = null;
              return;
            }
            ns = sn.next(ll), Ed();
            for (var yl = ps.length - 1; yl >= 0; --yl) {
              var cc = ps[yl];
              cc && cc(Ao, null, 0);
            }
            ar.flush(), Eo && Eo.update();
          }
          function Ws() {
            !ns && ps.length > 0 && (ns = sn.next(ll));
          }
          function ec() {
            ns && (sn.cancel(ll), ns = null);
          }
          function s1(yl) {
            yl.preventDefault(), Qn = !0, ec(), wl.forEach(function(cc) {
              cc();
            });
          }
          function n1(yl) {
            ar.getError(), Qn = !1, za.restore(), Bo.restore(), qo.restore(), Vs.restore(), vo.restore(), go.restore(), ho.restore(), Eo && Eo.restore(), Qs.procs.refresh(), Ws(), ol.forEach(function(cc) {
              cc();
            });
          }
          Jo && (Jo.addEventListener(td, s1, !1), Jo.addEventListener(Rf, n1, !1));
          function Js() {
            ps.length = 0, ec(), Jo && (Jo.removeEventListener(td, s1), Jo.removeEventListener(Rf, n1)), Bo.clear(), go.clear(), vo.clear(), ho.clear(), Vs.clear(), Ss.clear(), qo.clear(), Eo && Eo.clear(), xs.forEach(function(yl) {
              yl();
            });
          }
          function N1(yl) {
            vt(!!yl, "invalid args to regl({...})"), vt.type(yl, "object", "invalid args to regl({...})");
            function cc(Ia) {
              var ba = V({}, Ia);
              delete ba.uniforms, delete ba.attributes, delete ba.context, delete ba.vao, "stencil" in ba && ba.stencil.op && (ba.stencil.opBack = ba.stencil.opFront = ba.stencil.op, delete ba.stencil.op);
              function fa(Ea) {
                if (Ea in ba) {
                  var Hi = ba[Ea];
                  delete ba[Ea], Object.keys(Hi).forEach(function(ji) {
                    ba[Ea + "." + ji] = Hi[ji];
                  });
                }
              }
              return fa("blend"), fa("depth"), fa("cull"), fa("stencil"), fa("polygonOffset"), fa("scissor"), fa("sample"), "vao" in Ia && (ba.vao = Ia.vao), ba;
            }
            function Bc(Ia, ba) {
              var fa = {}, Ea = {};
              return Object.keys(Ia).forEach(function(Hi) {
                var ji = Ia[Hi];
                if (Yt.isDynamic(ji)) {
                  Ea[Hi] = Yt.unbox(ji, Hi);
                  return;
                } else if (ba && Array.isArray(ji)) {
                  for (var qi = 0; qi < ji.length; ++qi)
                    if (Yt.isDynamic(ji[qi])) {
                      Ea[Hi] = Yt.unbox(ji, Hi);
                      return;
                    }
                }
                fa[Hi] = ji;
              }), {
                dynamic: Ea,
                static: fa
              };
            }
            var U1 = Bc(yl.context || {}, !0), uf = Bc(yl.uniforms || {}, !0), df = Bc(yl.attributes || {}, !1), Xi = Bc(cc(yl), !1), Za = {
              gpuTime: 0,
              cpuTime: 0,
              count: 0
            }, ja = Qs.compile(Xi, df, uf, U1, Za), Ds = ja.draw, rc = ja.batch, Di = ja.scope, yi = [];
            function ca(Ia) {
              for (; yi.length < Ia; )
                yi.push(null);
              return yi;
            }
            function Ma(Ia, ba) {
              var fa;
              if (Qn && vt.raise("context lost"), typeof Ia == "function")
                return Di.call(this, null, Ia, 0);
              if (typeof ba == "function")
                if (typeof Ia == "number")
                  for (fa = 0; fa < Ia; ++fa)
                    Di.call(this, null, ba, fa);
                else if (Array.isArray(Ia))
                  for (fa = 0; fa < Ia.length; ++fa)
                    Di.call(this, Ia[fa], ba, fa);
                else
                  return Di.call(this, Ia, ba, 0);
              else if (typeof Ia == "number") {
                if (Ia > 0)
                  return rc.call(this, ca(Ia | 0), Ia | 0);
              } else if (Array.isArray(Ia)) {
                if (Ia.length)
                  return rc.call(this, Ia, Ia.length);
              } else
                return Ds.call(this, Ia);
            }
            return V(Ma, {
              stats: Za,
              destroy: function() {
                ja.destroy();
              }
            });
          }
          var Xu = go.setFBO = N1({
            framebuffer: Yt.define.call(null, fh, "framebuffer")
          });
          function Q1(yl, cc) {
            var Bc = 0;
            Qs.procs.poll();
            var U1 = cc.color;
            U1 && (ar.clearColor(+U1[0] || 0, +U1[1] || 0, +U1[2] || 0, +U1[3] || 0), Bc |= Xh), "depth" in cc && (ar.clearDepth(+cc.depth), Bc |= P2), "stencil" in cc && (ar.clearStencil(cc.stencil | 0), Bc |= Jf), vt(!!Bc, "called regl.clear with no buffer specified"), ar.clear(Bc);
          }
          function Qu(yl) {
            if (vt(
              typeof yl == "object" && yl,
              "regl.clear() takes an object as input"
            ), "framebuffer" in yl)
              if (yl.framebuffer && yl.framebuffer_reglType === "framebufferCube")
                for (var cc = 0; cc < 6; ++cc)
                  Xu(V({
                    framebuffer: yl.framebuffer.faces[cc]
                  }, yl), Q1);
              else
                Xu(yl, Q1);
            else
              Q1(null, yl);
          }
          function r1(yl) {
            vt.type(yl, "function", "regl.frame() callback must be a function"), ps.push(yl);
            function cc() {
              var Bc = hh(ps, yl);
              vt(Bc >= 0, "cannot cancel a frame twice");
              function U1() {
                var uf = hh(ps, U1);
                ps[uf] = ps[ps.length - 1], ps.length -= 1, ps.length <= 0 && ec();
              }
              ps[Bc] = U1;
            }
            return Ws(), {
              cancel: cc
            };
          }
          function ud() {
            var yl = Po.viewport, cc = Po.scissor_box;
            yl[0] = yl[1] = cc[0] = cc[1] = 0, Ao.viewportWidth = Ao.framebufferWidth = Ao.drawingBufferWidth = yl[2] = cc[2] = ar.drawingBufferWidth, Ao.viewportHeight = Ao.framebufferHeight = Ao.drawingBufferHeight = yl[3] = cc[3] = ar.drawingBufferHeight;
          }
          function Ed() {
            Ao.tick += 1, Ao.time = jl(), ud(), Qs.procs.poll();
          }
          function R1() {
            Vs.refresh(), ud(), Qs.procs.refresh(), Eo && Eo.update();
          }
          function jl() {
            return (hn() - bo) / 1e3;
          }
          R1();
          function Ad(yl, cc) {
            vt.type(cc, "function", "listener callback must be a function");
            var Bc;
            switch (yl) {
              case "frame":
                return r1(cc);
              case "lost":
                Bc = wl;
                break;
              case "restore":
                Bc = ol;
                break;
              case "destroy":
                Bc = xs;
                break;
              default:
                vt.raise("invalid event, must be one of frame,lost,restore,destroy");
            }
            return Bc.push(cc), {
              cancel: function() {
                for (var U1 = 0; U1 < Bc.length; ++U1)
                  if (Bc[U1] === cc) {
                    Bc[U1] = Bc[Bc.length - 1], Bc.pop();
                    return;
                  }
              }
            };
          }
          var Fc = V(N1, {
            // Clear current FBO
            clear: Qu,
            // Short cuts for dynamic variables
            prop: Yt.define.bind(null, fh),
            context: Yt.define.bind(null, $2),
            this: Yt.define.bind(null, F2),
            // executes an empty draw command
            draw: N1({}),
            // Resources
            buffer: function(yl) {
              return qo.create(yl, eh, !1, !1);
            },
            elements: function(yl) {
              return Ss.create(yl, !1);
            },
            texture: Vs.create2D,
            cube: Vs.createCube,
            renderbuffer: vo.create,
            framebuffer: go.create,
            framebufferCube: go.createCube,
            vao: ho.createVAO,
            // Expose context attributes
            attributes: Sa,
            // Frame rendering
            frame: r1,
            on: Ad,
            // System limits
            limits: Ka,
            hasExtension: function(yl) {
              return Ka.extensions.indexOf(yl.toLowerCase()) >= 0;
            },
            // Read pixels
            read: us,
            // Destroy regl and all associated resources
            destroy: Js,
            // Direct GL state manipulation
            _gl: ar,
            _refresh: R1,
            poll: function() {
              Ed(), Eo && Eo.update();
            },
            // Current time
            now: jl,
            // regl Statistics Information
            stats: Qo
          });
          return oi.onDone(null, Fc), Fc;
        }
        return bd;
      });
    }(regl$1)), regl$1.exports;
  }
  var reglExports = requireRegl();
  const createRegl = /* @__PURE__ */ getDefaultExportFromCjs$1(reglExports);
  function gcd2(O, _) {
    let H, V;
    O > _ ? (H = O, V = _) : (H = _, V = O);
    let q = H % V;
    for (; q != 0; )
      H = V, V = q, q = H % V;
    return V;
  }
  function gcd(O) {
    let _ = O[0];
    for (let H = 1; H < O.length; H++)
      _ = gcd2(_, O[H]);
    return _;
  }
  function is_pow_2(O) {
    return (O & O - 1) == 0 && O != 0;
  }
  class DashCache {
    constructor(_) {
      mt(this, "_regl");
      // Needed to create textures.
      mt(this, "_map");
      this._regl = _, this._map = /* @__PURE__ */ new Map();
    }
    _create_texture(_) {
      const H = _.length;
      let V = 0;
      const q = [];
      let G = 0, J = 0;
      for (let fe = 0; fe < H; fe++)
        V += _[fe], q.push(_[fe] + _[(fe + 1) % H]), fe % 2 == 0 ? J = Math.max(J, _[fe]) : G = Math.min(G, -_[fe]);
      G *= 0.5, J *= 0.5;
      const Z = gcd(q), te = [0];
      for (let fe = 0; fe < H; fe++)
        te.push(te[fe] + _[fe]);
      const ne = 2 * V / Z, ee = is_pow_2(ne), ae = ee ? ne : 128, oe = 0.5 * Z * ne / ae;
      let se;
      if (ee) {
        if (se = 0.5 * _[0], oe < se) {
          const fe = Math.floor(se / oe);
          se -= fe * oe;
        }
      } else
        se = 0;
      const ce = se - 0.5 * oe, le = new Uint8Array(ae);
      let ue = 0;
      for (let fe = 0; fe < ae; fe++) {
        const he = se + fe * oe;
        he > te[ue + 1] && ue++;
        const me = _[ue], pe = te[ue] + 0.5 * me;
        let ye = 0.5 * me - Math.abs(he - pe);
        ue % 2 == 1 && (ye = -ye), le[fe] = Math.round(255 * (ye - G) / (J - G));
      }
      const de = this._regl.texture({
        shape: [ae, 1, 1],
        data: le,
        wrapS: "repeat",
        format: "alpha",
        type: "uint8",
        mag: "linear",
        min: "linear"
      });
      return [[V, ce, G, J], de];
    }
    _get_key(_) {
      return _.join(",");
    }
    _get_or_create(_) {
      const H = this._get_key(_);
      let V = this._map.get(H);
      if (V == null) {
        const q = gcd(_);
        if (q > 1) {
          _ = map$4(_, (te) => te / q), V = this._get_or_create(_);
          const [G, J, Z] = V;
          V = [G, J, q], this._map.set(H, V);
        } else {
          const [G, J] = this._create_texture(_);
          V = [G, J, q], this._map.set(H, V);
        }
      }
      return V;
    }
    get(_) {
      return _.length % 2 == 1 && (_ = concat$1([_, _])), this._get_or_create(_);
    }
  }
  mt(DashCache, "__name__", "DashCache");
  const shader$7 = `
precision mediump float;

attribute vec2 a_position;
varying vec2 v_tex_coords;

void main()
{
  gl_Position = vec4(a_position.x, a_position.y, 0.0, 1.0);
  v_tex_coords = 0.5*(1.0 + a_position);
}
`, shader$6 = `
precision mediump float;

uniform sampler2D u_framebuffer_tex;
varying vec2 v_tex_coords;

void main()
{
  gl_FragColor = texture2D(u_framebuffer_tex, v_tex_coords);
}
`, shader$5 = `
precision mediump float;

attribute vec2 a_position;
attribute vec4 a_bounds;

uniform vec2 u_canvas_size;

varying vec2 v_tex_coords;

void main()
{
  v_tex_coords = vec2(a_position.x < 0.0 ? 0.0 : 1.0, a_position.y < 0.0 ? 0.0 : 1.0);

  float x = a_position.x < 0.0 ? a_bounds[0] : a_bounds[2];
  float y = a_position.y < 0.0 ? a_bounds[1] : a_bounds[3];
  vec2 xy = vec2(x, y);

  vec2 pos = xy + 0.5;  // Bokeh's offset.
  pos /= u_canvas_size;  // in 0..1
  gl_Position = vec4(2.0*pos.x - 1.0, 1.0 - 2.0*pos.y, 0.0, 1.0);
}
`, shader$4 = `
precision mediump float;

uniform sampler2D u_tex;
uniform float u_global_alpha;

varying vec2 v_tex_coords;

void main()
{
  vec4 color = texture2D(u_tex, v_tex_coords);
  float alpha = color.a*u_global_alpha;
  gl_FragColor = vec4(color.rgb*alpha, alpha);  // Premultiplied alpha.
}
`, shader$3 = `
precision mediump float;

const int butt_cap   = 0;
const int round_cap  = 1;
const int square_cap = 2;

const int miter_join = 0;
const int round_join = 1;
const int bevel_join = 2;

attribute vec2 a_position;
attribute vec2 a_point_prev;
attribute vec2 a_point_start;
attribute vec2 a_point_end;
attribute vec2 a_point_next;
attribute float a_show_prev;
attribute float a_show_curr;
attribute float a_show_next;
attribute float a_linewidth;
attribute vec4 a_line_color;
attribute float a_line_cap;
attribute float a_line_join;
#ifdef DASHED
attribute float a_length_so_far;
attribute vec4 a_dash_tex_info;
attribute float a_dash_scale;
attribute float a_dash_offset;
#endif

uniform vec2 u_canvas_size;
uniform float u_antialias;
uniform float u_miter_limit;

varying float v_linewidth;
varying vec4 v_line_color;
varying float v_line_cap;
varying float v_line_join;
varying float v_segment_length;
varying vec2 v_coords;
varying float v_flags;  // Boolean flags
varying float v_cos_turn_angle_start;
varying float v_cos_turn_angle_end;
#ifdef DASHED
varying float v_length_so_far;
varying vec4 v_dash_tex_info;
varying float v_dash_scale;
varying float v_dash_offset;
#endif

#define SMALL 1e-6

float cross_z(in vec2 v0, in vec2 v1)
{
    return v0.x*v1.y - v0.y*v1.x;
}

vec2 right_vector(in vec2 v)
{
    return vec2(v.y, -v.x);
}

// Calculate cos/sin turn angle with adjacent segment, and unit normal vector to right
float calc_turn_angle(in bool has_cap, in vec2 segment_right, in vec2 other_right, out vec2 point_right, out float sin_turn_angle)
{
    float cos_turn_angle;
    vec2 diff = segment_right + other_right;
    float len = length(diff);
    if (has_cap || len < SMALL) {
        point_right = segment_right;
        cos_turn_angle = -1.0;  // Turns back on itself.
        sin_turn_angle = 0.0;
    }
    else {
        point_right = diff / len;
        cos_turn_angle = dot(segment_right, other_right);   // cos zero at +/-pi/2, +ve angle is turn right
        sin_turn_angle = cross_z(segment_right, other_right);
    }
    return cos_turn_angle;
}

// If miter too large use bevel join instead
bool miter_too_large(in int join_type, in float cos_turn_angle)
{
    float cos_half_angle_sqr = 0.5*(1.0 + cos_turn_angle);  // Trig identity
    return join_type == miter_join && cos_half_angle_sqr < 1.0 / (u_miter_limit*u_miter_limit);
}

vec2 normalize_check_len(in vec2 vec, in float len)
{
    if (abs(len) < SMALL)
        return vec2(1.0, 0.0);
    else
        return vec / len;
}

vec2 normalize_check(in vec2 vec)
{
    return normalize_check_len(vec, length(vec));
}

void main()
{
    if (a_show_curr < 0.5) {
        // Line segment has non-finite value at one or both ends, do not render.
        gl_Position = vec4(-2.0, -2.0, 0.0, 1.0);
        return;
    }

    int join_type = int(a_line_join + 0.5);
    int cap_type = int(a_line_cap + 0.5);

    v_linewidth = a_linewidth;
    v_line_color = a_line_color;
    if (v_linewidth < 1.0) {
        // Linewidth less than 1 is implemented as 1 but with reduced alpha.
        v_line_color.a *= v_linewidth;
        v_linewidth = 1.0;
    }

    float halfwidth = 0.5*(v_linewidth + u_antialias);

    vec2 segment_along = a_point_end - a_point_start;
    v_segment_length = length(a_point_end - a_point_start);
    segment_along = normalize_check_len(segment_along, v_segment_length); // unit vector.
    vec2 segment_right = right_vector(segment_along);  // unit vector.
    vec2 xy;

    // in screen coords
    vec2 prev_along = normalize_check(a_point_start - a_point_prev);
    vec2 prev_right = right_vector(prev_along);
    vec2 next_right = right_vector(normalize_check(a_point_next - a_point_end));

    v_coords.y = a_position.y*halfwidth;  // Overwritten later for join points.

    // Start and end cap properties
    bool has_start_cap = a_show_prev < 0.5;
    bool has_end_cap = a_show_next < 0.5;

    // Start and end join properties
    vec2 point_right_start, point_right_end;
    float sin_turn_angle_start, sin_turn_angle_end;
    v_cos_turn_angle_start = calc_turn_angle(has_start_cap, segment_right, prev_right, point_right_start, sin_turn_angle_start);
    v_cos_turn_angle_end = calc_turn_angle(has_end_cap, segment_right, next_right, point_right_end, sin_turn_angle_end);
    float sign_turn_right_start = sin_turn_angle_start >= 0.0 ? 1.0 : -1.0;

    bool miter_too_large_start = !has_start_cap && miter_too_large(join_type, v_cos_turn_angle_start);
    bool miter_too_large_end = !has_end_cap && miter_too_large(join_type, v_cos_turn_angle_end);

    float sign_at_start = -sign(a_position.x);  // +ve at segment start, -ve end.
    vec2 point = sign_at_start > 0.0 ? a_point_start : a_point_end;

    if ( (has_start_cap && sign_at_start > 0.0) ||
         (has_end_cap && sign_at_start < 0.0) ) {
        // Cap.
        xy = point - segment_right*(halfwidth*a_position.y);
        if (cap_type == butt_cap)
            xy -= sign_at_start*0.5*u_antialias*segment_along;
        else
            xy -= sign_at_start*halfwidth*segment_along;
    }
    else if (sign_at_start > 0.0) {
        vec2 inside_point = a_point_start + segment_right*(sign_turn_right_start*halfwidth);
        vec2 prev_outside_point = a_point_start - prev_right*(sign_turn_right_start*halfwidth);

        // join at start.
        if (join_type == round_join || join_type == bevel_join || miter_too_large_start) {
            if (v_cos_turn_angle_start <= 0.0) {  // |turn_angle| > 90 degrees
                xy = a_point_start - segment_right*(halfwidth*a_position.y) - halfwidth*segment_along;
            }
            else {
                if (a_position.x < -1.5) {
                    xy = prev_outside_point;
                    v_coords.y = -dot(xy - a_point_start, segment_right);
                }
                else if (a_position.y*sign_turn_right_start > 0.0) {  // outside corner of turn
                    float d = halfwidth*abs(sin_turn_angle_start);
                    xy = a_point_start - segment_right*(halfwidth*a_position.y) - d*segment_along;
                }
                else {  // inside corner of turn
                    xy = inside_point;
                }
            }
        }
        else {  // miter join
            if (a_position.x < -1.5) {
                xy = prev_outside_point;
                v_coords.y = -dot(xy - a_point_start, segment_right);
            }
            else if (a_position.y*sign_turn_right_start > 0.0) {  // outside corner of turn
                float tan_half_turn_angle = (1.0-v_cos_turn_angle_start) / sin_turn_angle_start;  // Trig identity
                float d = sign_turn_right_start*halfwidth*tan_half_turn_angle;
                xy = a_point_start - segment_right*(halfwidth*a_position.y) - d*segment_along;
            }
            else {  // inside corner if turn
                xy = inside_point;
            }
        }
    }
    else {
        xy = point - segment_right*(halfwidth*a_position.y);
    }

    vec2 pos = xy + 0.5;  // Bokeh's offset.
    pos /= u_canvas_size;  // in 0..1
    gl_Position = vec4(2.0*pos.x - 1.0, 1.0 - 2.0*pos.y, 0.0, 1.0);

    bool turn_right_start = sin_turn_angle_start >= 0.0;
    bool turn_right_end = sin_turn_angle_end >= 0.0;

    v_coords.x = dot(xy - a_point_start, segment_along);
    v_flags = float(int(has_start_cap) +
                    2*int(has_end_cap) +
                    4*int(miter_too_large_start) +
                    8*int(miter_too_large_end) +
                    16*int(turn_right_start) +
                    32*int(turn_right_end));

    v_line_cap = a_line_cap;
    v_line_join = a_line_join;

#ifdef DASHED
    v_length_so_far = a_length_so_far;
    v_dash_tex_info = a_dash_tex_info;
    v_dash_scale = a_dash_scale;
    v_dash_offset = a_dash_offset;
#endif
}
`, shader$2 = `
precision mediump float;

const int butt_cap   = 0;
const int round_cap  = 1;
const int square_cap = 2;

const int miter_join = 0;
const int round_join = 1;
const int bevel_join = 2;

uniform float u_antialias;
#ifdef DASHED
uniform sampler2D u_dash_tex;
#endif

varying float v_linewidth;
varying vec4 v_line_color;
varying float v_line_cap;
varying float v_line_join;
varying float v_segment_length;
varying vec2 v_coords;
varying float v_flags;
varying float v_cos_turn_angle_start;
varying float v_cos_turn_angle_end;
#ifdef DASHED
varying float v_length_so_far;
varying vec4 v_dash_tex_info;
varying float v_dash_scale;
varying float v_dash_offset;
#endif

#define ONE_MINUS_SMALL (1.0 - 1e-6)

float cross_z(in vec2 v0, in vec2 v1)
{
    return v0.x*v1.y - v0.y*v1.x;
}

vec2 right_vector(in vec2 v)
{
    return vec2(v.y, -v.x);
}

float bevel_join_distance(in vec2 coords, in vec2 other_right, in float sign_turn_right)
{
    // other_right is unit vector facing right of the other (previous or next) segment, in coord reference frame
    float hw = 0.5*v_linewidth;  // Not including antialiasing
    if (other_right.y >= ONE_MINUS_SMALL) {  // other_right.y is -cos(turn_angle)
        // 180 degree turn.
        return abs(hw - v_coords.x);
    }
    else {
        const vec2 segment_right = vec2(0.0, -1.0);
        // corner_right is unit vector bisecting corner facing right, in coord reference frame
        vec2 corner_right = normalize(other_right + segment_right);
        vec2 outside_point = (-hw*sign_turn_right)*segment_right;
        return hw + sign_turn_right*dot(outside_point - coords, corner_right);
    }
}

float cap(in int cap_type, in float x, in float y)
{
    // x is distance along segment in direction away from end of segment,
    // y is distance across segment.
    if (cap_type == butt_cap)
        return max(0.5*v_linewidth - x, abs(y));
    else if (cap_type == square_cap)
        return max(-x, abs(y));
    else  // cap_type == round_cap
        return distance(vec2(min(x, 0.0), y), vec2(0.0, 0.0));
}

float distance_to_alpha(in float dist)
{
    return 1.0 - smoothstep(0.5*(v_linewidth - u_antialias),
                            0.5*(v_linewidth + u_antialias), dist);
}

vec2 turn_angle_to_right_vector(in float cos_turn_angle, in float sign_turn_right)
{
    float sin_turn_angle = sign_turn_right*sqrt(1.0 - cos_turn_angle*cos_turn_angle);
    return vec2(sin_turn_angle, -cos_turn_angle);
}

#ifdef DASHED
float dash_distance(in float x)
{
    // x is in direction of v_coords.x, i.e. along segment.
    float tex_length = v_dash_tex_info.x;
    float tex_offset = v_dash_tex_info.y;
    float tex_dist_min = v_dash_tex_info.z;
    float tex_dist_max = v_dash_tex_info.w;

    // Apply offset.
    x += v_length_so_far - v_dash_scale*tex_offset + v_dash_offset;

    // Interpolate within texture to obtain distance to dash.
    float dist = texture2D(u_dash_tex,
                           vec2(x / (tex_length*v_dash_scale), 0.0)).a;

    // Scale distance within min and max limits.
    dist = tex_dist_min + dist*(tex_dist_max - tex_dist_min);

    return v_dash_scale*dist;
}

mat2 rotation_matrix(in vec2 other_right)
{
    float sin_angle = other_right.x;
    float cos_angle = -other_right.y;
    return mat2(cos_angle, -sin_angle, sin_angle, cos_angle);
}
#endif

void main()
{
    int join_type = int(v_line_join + 0.5);
    int cap_type = int(v_line_cap + 0.5);
    float halfwidth = 0.5*(v_linewidth + u_antialias);
    float half_antialias = 0.5*u_antialias;

    // Extract flags.
    int flags = int(v_flags + 0.5);
    bool turn_right_end = (flags / 32 > 0);
    float sign_turn_right_end = turn_right_end ? 1.0 : -1.0;
    flags -= 32*int(turn_right_end);
    bool turn_right_start = (flags / 16 > 0);
    float sign_turn_right_start = turn_right_start ? 1.0 : -1.0;
    flags -= 16*int(turn_right_start);
    bool miter_too_large_end = (flags / 8 > 0);
    flags -= 8*int(miter_too_large_end);
    bool miter_too_large_start = (flags / 4 > 0);
    flags -= 4*int(miter_too_large_start);
    bool has_end_cap = (flags / 2 > 0);
    flags -= 2*int(has_end_cap);
    bool has_start_cap = flags > 0;

    // Unit vectors to right of previous and next segments in coord reference frame
    vec2 prev_right = turn_angle_to_right_vector(v_cos_turn_angle_start, sign_turn_right_start);
    vec2 next_right = turn_angle_to_right_vector(v_cos_turn_angle_end, sign_turn_right_end);

    float dist = v_coords.y;  // For straight segment, and miter join.

    // Along-segment coords with respect to end of segment, facing inwards
    vec2 end_coords = vec2(v_segment_length, 0.0) - v_coords;

    if (v_coords.x <= half_antialias) {
        // At start of segment, either cap or join.
        if (has_start_cap)
            dist = cap(cap_type, v_coords.x, v_coords.y);
        else if (join_type == round_join) {
            if (v_coords.x <= 0.0)
                dist = distance(v_coords, vec2(0.0, 0.0));
        }
        else {  // bevel or miter join
            if (join_type == bevel_join || miter_too_large_start)
                dist = max(abs(dist), bevel_join_distance(v_coords, prev_right, sign_turn_right_start));
            float prev_sideways_dist = -sign_turn_right_start*dot(v_coords, prev_right);
            dist = max(abs(dist), prev_sideways_dist);
        }
    }

    if (end_coords.x <= half_antialias) {
        if (has_end_cap) {
            dist = max(abs(dist), cap(cap_type, end_coords.x, v_coords.y));
        }
        else if (join_type == bevel_join || miter_too_large_end) {
            // Bevel join at end impacts half antialias distance
            dist = max(abs(dist), bevel_join_distance(end_coords, next_right, sign_turn_right_end));
        }
    }

    float alpha = distance_to_alpha(abs(dist));

#ifdef DASHED
    if (v_dash_tex_info.x >= 0.0) {
        // Dashes in straight segments (outside of joins) are easily calculated.
        dist = dash_distance(v_coords.x);

        vec2 prev_coords = rotation_matrix(prev_right)*v_coords;
        float start_dash_distance = dash_distance(0.0);

        if (!has_start_cap && cap_type == butt_cap) {
            // Outer of start join rendered solid color or not at all depending on whether corner
            // point is in dash or gap, with antialiased ends.
            bool outer_solid = start_dash_distance >= 0.0 && v_coords.x < half_antialias && prev_coords.x > -half_antialias;
            if (outer_solid) {
                // Within solid outer region, antialiased at ends
                float half_aa_dist = dash_distance(half_antialias);
                if (half_aa_dist > 0.0)  // Next dash near, do not want antialiased gap
                    dist = half_aa_dist - v_coords.x + half_antialias;
                else
                    dist = start_dash_distance - v_coords.x;

                half_aa_dist = dash_distance(-half_antialias);
                if (half_aa_dist > 0.0)  // Prev dash nearm do not want antialiased gap
                    dist = min(dist, half_aa_dist + prev_coords.x + half_antialias);
                else
                    dist = min(dist, start_dash_distance + prev_coords.x);
            }
            else {
                // Outer not rendered, antialias ends.
                if (v_coords.x < half_antialias)
                    dist = min(0.0, dash_distance(half_antialias) - half_antialias) + v_coords.x;

                if (prev_coords.x > -half_antialias && prev_coords.x <= half_antialias) {
                    // Antialias from end of previous segment into join
                    float prev_dist = min(0.0, dash_distance(-half_antialias) - half_antialias) - prev_coords.x;
                    // Consider width of previous segment
                    prev_dist = min(prev_dist, 0.5*v_linewidth - abs(prev_coords.y));
                    dist = max(dist, prev_dist);
                }
            }
        }

        if (!has_end_cap && cap_type == butt_cap && end_coords.x < half_antialias) {
            float end_dash_distance = dash_distance(v_segment_length);
            bool increasing = end_dash_distance >= 0.0 && sign_turn_right_end*v_coords.y < 0.0;
            if (!increasing) {
                float half_aa_dist = dash_distance(v_segment_length - half_antialias);
                dist = min(0.0, half_aa_dist - half_antialias) + end_coords.x;
            }
        }

        dist = cap(cap_type, dist, v_coords.y);

        float dash_alpha = distance_to_alpha(dist);
        alpha = min(alpha, dash_alpha);
    }
#endif

    alpha = v_line_color.a*alpha;
    gl_FragColor = vec4(v_line_color.rgb*alpha, alpha);  // Premultiplied alpha.
}
`, shader$1 = `
precision mediump float;

attribute vec2 a_position;
attribute vec2 a_center;
attribute float a_width;   // or radius or outer_radius
attribute float a_height;  // or inner_radius
attribute float a_angle;   // or start_angle
attribute float a_aux;     // or end_angle
attribute float a_linewidth;
attribute vec4 a_line_color;
attribute vec4 a_fill_color;
attribute float a_line_cap;
attribute float a_line_join;
attribute float a_show;

#ifdef HATCH
attribute float a_hatch_pattern;
attribute float a_hatch_scale;
attribute float a_hatch_weight;
attribute vec4 a_hatch_color;
#endif

uniform vec2 u_canvas_size;
uniform float u_antialias;

#ifdef MULTI_MARKER
uniform float u_size_hint;
#endif

#ifdef USE_ROUND_RECT
uniform vec4 u_border_radius;
varying vec4 v_border_radius;
#endif

#ifdef USE_ANNULAR_WEDGE
varying float v_outer_radius;
varying float v_inner_radius;
varying float v_start_angle;
varying float v_end_angle;
#endif

#ifdef USE_ANNULUS
varying float v_outer_radius;
varying float v_inner_radius;
#endif

#ifdef USE_WEDGE
varying float v_radius;
varying float v_start_angle;
varying float v_end_angle;
#endif

#ifdef USE_CIRCLE
varying float v_radius;
#endif

varying float v_linewidth;
varying vec2 v_size; // 2D size for rects compared to 1D for markers.
varying vec4 v_line_color;
varying vec4 v_fill_color;
varying float v_line_cap;
varying float v_line_join;
varying vec2 v_coords;

#ifdef HATCH
varying float v_hatch_pattern;
varying float v_hatch_scale;
varying float v_hatch_weight;
varying vec4 v_hatch_color;
varying vec2 v_hatch_coords;
#endif

#ifdef MULTI_MARKER

#define M_DASH         1
#define M_DOT          2
#define M_DIAMOND      3
#define M_HEX          4
#define M_SQUARE_PIN   5
#define M_TRIANGLE     6
#define M_TRIANGLE_PIN 7
#define M_STAR         8

vec2 enclosing_size() {
  // Need extra size of (v_linewidth+u_antialias) if edge of marker parallel to
  // edge of bounding box.  If symmetric spike towards edge then multiply by
  // 1/cos(theta) where theta is angle between spike and bbox edges.
  int size_hint = int(u_size_hint + 0.5);
  if (size_hint == M_DASH)
    return vec2(v_size.x + v_linewidth + u_antialias,
                           v_linewidth + u_antialias);
  else if (size_hint == M_DOT)
    return 0.25*v_size + u_antialias;
  else if (size_hint == M_DIAMOND)
    return vec2(v_size.x*(2.0/3.0) + (v_linewidth + u_antialias)*1.20185,
                v_size.y + (v_linewidth + u_antialias)*1.80278);
  else if (size_hint == M_HEX)
    return v_size + (v_linewidth + u_antialias)*vec2(2.0/sqrt(3.0), 1.0);
  else if (size_hint == M_SQUARE_PIN)  // Square pin
    return v_size + (v_linewidth + u_antialias)*3.1;
  else if (size_hint == M_TRIANGLE)
    return vec2(v_size.x + (v_linewidth + u_antialias)*sqrt(3.0),
                v_size.y*(2.0/sqrt(3.0)) + (v_linewidth + u_antialias)*2.0);
  else if (size_hint == M_TRIANGLE_PIN)
    return v_size + (v_linewidth + u_antialias)*vec2(4.8, 6.0);
  else if (size_hint == M_STAR)
    return vec2(v_size.x*0.95106 + (v_linewidth + u_antialias)*3.0,
                v_size.y + (v_linewidth + u_antialias)*3.2);
  else
    return v_size + v_linewidth + u_antialias;
}
#else
vec2 enclosing_size() {
  return v_size + v_linewidth + u_antialias;
}
#endif

void main()
{
#if defined(USE_RECT) || defined(USE_ROUND_RECT) || defined(USE_HEX_TILE)
  v_size = vec2(a_width, a_height);
#elif defined(USE_ANNULUS) || defined(USE_ANNULAR_WEDGE) || defined(USE_WEDGE)
  v_size = vec2(2.0*a_width, 2.0*a_width);
#else
  v_size = vec2(a_width, a_width);
#endif

  if (a_show < 0.5 || v_size.x < 0.0 || v_size.y < 0.0 || (v_size.x == 0.0 && v_size.y == 0.0)) {
    // Do not show this marker.
    gl_Position = vec4(-2.0, -2.0, 0.0, 1.0);
    return;
  }

#ifdef USE_ANNULAR_WEDGE
  v_outer_radius = a_width;
  v_inner_radius = a_height;
  v_start_angle = a_angle;
  v_end_angle = a_aux;
#endif

#ifdef USE_ANNULUS
  v_outer_radius = a_width;
  v_inner_radius = a_height;
#endif

#ifdef USE_WEDGE
  v_radius = a_width;
  v_start_angle = a_angle;
  v_end_angle = a_aux;
#endif

#ifdef USE_CIRCLE
  v_radius = 0.5*a_width;
#endif

#ifdef USE_ROUND_RECT
  // Scale corner radii if they are too large, the same as canvas
  // https://html.spec.whatwg.org/multipage/canvas.html#dom-context-2d-roundrect
  // Order of border_radius is top_left, top_right, bottom_right, bottom_left
  const vec2 unit2 = vec2(1.0, 1.0);
  float scale = min(v_size.x / max(dot(u_border_radius.xy, unit2), dot(u_border_radius.zw, unit2)),
                    v_size.y / max(dot(u_border_radius.yz, unit2), dot(u_border_radius.wx, unit2)));
  v_border_radius = u_border_radius*min(scale, 1.0);
#endif

  v_linewidth = a_linewidth;
  v_line_color = a_line_color;
  v_fill_color = a_fill_color;
  v_line_cap = a_line_cap;
  v_line_join = a_line_join;

  if (v_linewidth < 1.0) {
    // Linewidth less than 1 is implemented as 1 but with reduced alpha.
    v_line_color.a *= v_linewidth;
    v_linewidth = 1.0;
  }

#ifdef HATCH
  v_hatch_pattern = a_hatch_pattern;
  v_hatch_scale = a_hatch_scale;
  v_hatch_weight = a_hatch_weight;
  v_hatch_color = a_hatch_color;
#endif

  // Coordinates in rotated frame with respect to center of marker, used for
  // distance functions in fragment shader.
  v_coords = a_position*enclosing_size();

#if defined(USE_CIRCLE) || defined(USE_ANNULUS) || defined(USE_ANNULAR_WEDGE) || defined(USE_WEDGE)
  vec2 pos = a_center + v_coords;
#else
  float c = cos(-a_angle);
  float s = sin(-a_angle);
  mat2 rotation = mat2(c, -s, s, c);

  vec2 pos = a_center + rotation*v_coords;
#endif

#ifdef HATCH
  // Coordinates for hatching in unrotated frame of reference.
  v_hatch_coords = pos - 0.5;
#endif

  pos += 0.5; // Make up for Bokeh's offset.
  pos /= u_canvas_size; // 0 to 1.
  gl_Position = vec4(2.0*pos.x - 1.0, 1.0 - 2.0*pos.y, 0.0, 1.0);
}
`, shader = `
precision mediump float;

const float SQRT2 = sqrt(2.0);
const float SQRT3 = sqrt(3.0);
const float PI = 3.14159265358979323846;

const int butt_cap = 0;
const int round_cap = 1;
const int square_cap = 2;

const int miter_join = 0;
const int round_join = 1;
const int bevel_join = 2;

#ifdef HATCH
const int hatch_dot = 1;
const int hatch_ring = 2;
const int hatch_horizontal_line = 3;
const int hatch_vertical_line = 4;
const int hatch_cross = 5;
const int hatch_horizontal_dash = 6;
const int hatch_vertical_dash = 7;
const int hatch_spiral = 8;
const int hatch_right_diagonal_line = 9;
const int hatch_left_diagonal_line = 10;
const int hatch_diagonal_cross = 11;
const int hatch_right_diagonal_dash = 12;
const int hatch_left_diagonal_dash = 13;
const int hatch_horizontal_wave = 14;
const int hatch_vertical_wave = 15;
const int hatch_criss_cross = 16;
#endif

uniform float u_antialias;

varying vec2 v_coords;
varying vec2 v_size;

#ifdef USE_ANNULAR_WEDGE
varying float v_outer_radius;
varying float v_inner_radius;
varying float v_start_angle;
varying float v_end_angle;
#endif

#ifdef USE_ANNULUS
varying float v_outer_radius;
varying float v_inner_radius;
#endif

#ifdef USE_WEDGE
varying float v_radius;
varying float v_start_angle;
varying float v_end_angle;
#endif

#ifdef USE_CIRCLE
varying float v_radius;
#endif

#ifdef USE_ROUND_RECT
varying vec4 v_border_radius;
#endif

varying float v_linewidth;
varying vec4 v_line_color;
varying vec4 v_fill_color;
varying float v_line_cap;
varying float v_line_join;

#ifdef HATCH
varying float v_hatch_pattern;
varying float v_hatch_scale;
varying float v_hatch_weight;
varying vec4 v_hatch_color;
varying vec2 v_hatch_coords;
#endif

// Lines within the marker (dot, cross, x and y) are added at the end as they are
// on top of the fill rather than astride it.
#if defined(USE_CIRCLE_DOT) || defined(USE_DIAMOND_DOT) || defined(USE_DOT) ||     defined(USE_HEX_DOT) || defined(USE_SQUARE_DOT) || defined(USE_STAR_DOT) ||     defined(USE_TRIANGLE_DOT)
  #define APPEND_DOT
#endif

#if defined(USE_CIRCLE_CROSS) || defined(USE_SQUARE_CROSS)
  #define APPEND_CROSS
#endif

#ifdef USE_DIAMOND_CROSS
  #define APPEND_CROSS_2
#endif

#ifdef USE_CIRCLE_X
  #define APPEND_X
  #define APPEND_X_LEN (0.5*v_size.x)
#endif

#ifdef USE_SQUARE_X
  #define APPEND_X
  #define APPEND_X_LEN (v_size.x/SQRT2)
#endif

#ifdef USE_CIRCLE_Y
  #define APPEND_Y
#endif

#if defined(USE_ASTERISK) || defined(USE_CROSS) || defined(USE_DASH) ||     defined(USE_DOT) || defined(USE_X) || defined(USE_Y)
  // No fill.
  #define LINE_ONLY
#endif

#if defined(LINE_ONLY) || defined(APPEND_CROSS) || defined(APPEND_CROSS_2) ||     defined(APPEND_X) || defined(APPEND_Y)
float end_cap_distance(in vec2 p, in vec2 end_point, in vec2 unit_direction, in int line_cap)
{
  vec2 offset = p - end_point;
  if (line_cap == butt_cap)
    return dot(offset, unit_direction) + 0.5*v_linewidth;
  else if (line_cap == square_cap)
    return dot(offset, unit_direction);
  else if (line_cap == round_cap && dot(offset, unit_direction) > 0.0)
    return length(offset);
  else
    // Default is outside of line and should be -0.5*(v_linewidth+u_antialias) or less,
    // so here avoid the multiplication.
    return -v_linewidth-u_antialias;
}
#endif

#if !(defined(LINE_ONLY) || defined(USE_SQUARE_PIN) || defined(USE_TRIANGLE_PIN))
// For line join at a vec2 corner where 2 line segments meet, consider bevel points which are the 2
// points obtained by moving half a linewidth away from the corner point in the directions normal to
// the line segments.  The line through these points is the bevel line, characterised by a vec2
// unit_normal and offset distance from the corner point.  Edge of bevel join straddles this line,
// round join occurs outside of this line centred on the corner point.  In general
//   offset = (linewidth/2)*sin(alpha/2)
// where alpha is the angle between the 2 line segments at the corner.
float line_join_distance_no_miter(
  in vec2 p, in vec2 corner, in vec2 unit_normal, in float offset, in int line_join)
{
  // Simplified version of line_join_distance ignoring miter which most markers do implicitly
  // as they are composed of straight line segments.
  float dist_outside = dot((p - corner), unit_normal) - offset;

  if (line_join == bevel_join && dist_outside > -0.5*u_antialias)
    return dist_outside + 0.5*v_linewidth;
  else if (dist_outside > 0.0)  // round_join
    return distance(p, corner);
  else
    // Default is outside of line and should be -0.5*(v_linewidth+u_antialias) or less,
    // so here avoid the multiplication.
    return -v_linewidth-u_antialias;
}
#endif

#if defined(USE_SQUARE_PIN) || defined(USE_TRIANGLE_PIN)
// Line join distance including miter but only one-sided check as assuming use of symmetry in
// calling function.
float line_join_distance_incl_miter(
  in vec2 p, in vec2 corner, in vec2 unit_normal, in float offset, in int line_join,
  vec2 miter_unit_normal)
{
  float dist_outside = dot((p - corner), unit_normal) - offset;

  if (line_join == miter_join && dist_outside > 0.0)
    return dot((p - corner), miter_unit_normal);
  else if (line_join == bevel_join && dist_outside > -0.5*u_antialias)
    return dist_outside + 0.5*v_linewidth;
  else if (dist_outside > 0.0)  // round_join
    return distance(p, corner);
  else
    return -v_linewidth-u_antialias;
}
#endif

#if defined(APPEND_CROSS) || defined(APPEND_X) || defined(USE_ASTERISK) ||     defined(USE_CROSS) || defined(USE_X)
float one_cross(in vec2 p, in int line_cap, in float len)
{
  p = abs(p);
  p = (p.y > p.x) ? p.yx : p.xy;
  float dist = p.y;
  float end_dist = end_cap_distance(p, vec2(len, 0.0), vec2(1.0, 0.0), line_cap);
  return max(dist, end_dist);
}
#endif

#ifdef APPEND_CROSS_2
float one_cross_2(in vec2 p, in int line_cap, in vec2 lengths)
{
  // Cross with different length in x and y directions.
  p = abs(p);
  bool switch_xy = (p.y > p.x);
  p = switch_xy ? p.yx : p.xy;
  float len = switch_xy ? lengths.y : lengths.x;
  float dist = p.y;
  float end_dist = end_cap_distance(p, vec2(len, 0.0), vec2(1.0, 0.0), line_cap);
  return max(dist, end_dist);
}
#endif

#if defined(APPEND_Y) || defined(USE_Y)
float one_y(in vec2 p, in int line_cap, in float len)
{
  p = vec2(abs(p.x), -p.y);

  // End point of line to right is (1/2, 1/3)*len*SQRT3.
  // Unit vector along line is (1/2, 1/3)*k where k = 6/SQRT13.
  const float k = 6.0/sqrt(13.0);
  vec2 unit_along = vec2(0.5*k, k/3.0);
  vec2 end_point = vec2(0.5*len*SQRT3, len*SQRT3/3.0);
  float dist = max(abs(dot(p, vec2(-unit_along.y, unit_along.x))),
                   end_cap_distance(p, end_point, unit_along, line_cap));

  if (p.y < 0.0) {
    // Vertical line.
    float vert_dist = max(p.x,
                          end_cap_distance(p, vec2(0.0, -len), vec2(0.0, -1.0), line_cap));
    dist = min(dist, vert_dist);
  }
  return dist;
}
#endif

// One marker_distance function per marker type.
// Distance is zero on edge of marker, +ve outside and -ve inside.

#ifdef USE_ASTERISK
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  vec2 p_diag = vec2((p.x + p.y)/SQRT2, (p.x - p.y)/SQRT2);
  float len = 0.5*v_size.x;
  return min(one_cross(p, line_cap, len),  // cross
             one_cross(p_diag, line_cap, len));  // x
}
#endif

#if defined(USE_ANNULUS) || defined(USE_WEDGE) || defined(USE_ANNULAR_WEDGE)
float merge(in float d1, in float d2)
{
  return min(d1, d2);
}

float intersect(in float d1, in float d2)
{
  return max(d1, d2);
}

float subtract(in float d1, in float d2)
{
  return max(d1, -d2);
}

float circle(in vec2 p, in float radius)
{
  return length(p) - radius;
}

float segment_square(in vec2 p, in vec2 q) {
  vec2 v = p - q*clamp(dot(p, q)/dot(q, q), 0.0, 1.0);
  return dot(v, v);
}

vec2 xy(in float angle)
{
  return vec2(cos(angle), sin(angle));
}

float cross_z(in vec2 v0, in vec2 v1)
{
    return v0.x*v1.y - v0.y*v1.x;
}

// From https://www.shadertoy.com/view/wldXWB (MIT licensed)
float wedge(in vec2 p, in float r, in float start_angle, in float end_angle)
{
    vec2 a = r*xy(start_angle);
    vec2 b = r*xy(end_angle);

    // distance
    float d = sqrt(merge(segment_square(p, a), segment_square(p, b)));

    // sign
    float s;
    if (cross_z(a, b) < 0.0) {
        s =  sign(max(cross_z(a, p), cross_z(p, b)));
    } else {
        s = -sign(max(cross_z(p, a), cross_z(b, p)));
    }

    return s*d;
}

float annulus(in vec2 p, in float outer_radius, in float inner_radius)
{
  float outer = circle(p, outer_radius);
  float inner = circle(p, inner_radius);

  return subtract(outer, inner);
}
#endif

#if defined(USE_ANNULUS)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  return annulus(p, v_outer_radius, v_inner_radius);
}
#endif

#if defined(USE_WEDGE)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  return intersect(
    circle(p, v_radius),
    wedge(p, v_radius, v_start_angle, v_end_angle));
}
#endif

#if defined(USE_ANNULAR_WEDGE)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  return intersect(
    annulus(p, v_outer_radius, v_inner_radius),
    wedge(p, v_outer_radius, v_start_angle, v_end_angle));
}
#endif

#if defined(USE_CIRCLE) || defined(USE_CIRCLE_CROSS) || defined(USE_CIRCLE_DOT) ||     defined(USE_CIRCLE_X) || defined(USE_CIRCLE_Y)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  return length(p) - 0.5*v_size.x;
}
#endif

#ifdef USE_CROSS
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  return one_cross(p, line_cap, 0.5*v_size.x);
}
#endif

#ifdef USE_DASH
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  p = abs(p);
  float dist = p.y;
  float end_dist = end_cap_distance(p, vec2(0.5*v_size.x, 0.0), vec2(1.0, 0.0), line_cap);
  return max(dist, end_dist);
}
#endif

#if defined(USE_DIAMOND) || defined(USE_DIAMOND_CROSS) || defined(USE_DIAMOND_DOT)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  // Only need to consider +ve quadrant, the 2 end points are (2r/3, 0) and (0, r)
  // where r = radius = v_size.x/2.
  // Line has outward-facing unit normal vec2(1, 2/3)/k where k = SQRT13/3
  // hence vec2(3, 2)/SQRT13, and distance from origin of 2r/(3k) = 2r/SQRT13.
  p = abs(p);
  float r = 0.5*v_size.x;
  const float SQRT13 = sqrt(13.0);
  float dist = dot(p, vec2(3.0, 2.0))/SQRT13 - 2.0*r/SQRT13;

  if (line_join != miter_join) {
    dist = max(dist, line_join_distance_no_miter(
      p, vec2(0.0, r), vec2(0.0, 1.0), v_linewidth/SQRT13, line_join));

    dist = max(dist, line_join_distance_no_miter(
      p, vec2(r*2.0/3.0, 0.0), vec2(1.0, 0.0), v_linewidth*(1.5/SQRT13), line_join));
  }

  return dist;
}
#endif

#ifdef USE_DOT
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Dot is always appended.
  return v_linewidth+u_antialias;
}
#endif

#if defined(USE_HEX_TILE) || defined(USE_HEX) || defined(USE_HEX_DOT)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // A regular hexagon has v_size.x == v.size_y = r where r is the length of
  // each of the 3 sides of the 6 equilateral triangles that comprise the hex.
  // Only consider +ve quadrant, the 3 corners are at (0, h), (rx/2, h), (rx, 0)
  // where rx = 0.5*v_size.x, ry = 0.5*v_size.y and h = ry*SQRT3/2.
  // Sloping line has outward normal vec2(h, rx/2).  Length of this is
  // len = sqrt(h**2 + rx**2/4) to give unit normal (h, rx/2)/len and distance
  // from origin of this line is rx*h/len.
  p = abs(p);
  float rx = v_size.x/2.0;
  float h = v_size.y*(SQRT3/4.0);
  float len_normal = sqrt(h*h + 0.25*rx*rx);
  vec2 unit_normal = vec2(h, 0.5*rx) / len_normal;
  float dist = max(dot(p, unit_normal) - rx*h/len_normal,  // Distance from sloping line.
                   p.y - h);  // Distance from horizontal line.

  if (line_join != miter_join) {
    dist = max(dist, line_join_distance_no_miter(
      p, vec2(rx, 0.0), vec2(1.0, 0.0), 0.5*v_linewidth*unit_normal.x, line_join));

    unit_normal = normalize(unit_normal + vec2(0.0, 1.0));  // At (rx/2, h) corner.
    dist = max(dist, line_join_distance_no_miter(
      p, vec2(0.5*rx, h), unit_normal, 0.5*v_linewidth*unit_normal.y, line_join));
  }
  return dist;
}
#endif

#ifdef USE_PLUS
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  // Only need to consider one octant, the +ve quadrant with x >= y.
  p = abs(p);
  p = (p.y > p.x) ? p.yx : p.xy;

  // 3 corners are (r, 0), (r, 3r/8) and (3r/8, 3r/8).
  float r = 0.5*v_size.x;
  p = p - vec2(r, 0.375*r);  // Distance with respect to outside corner
  float dist = max(p.x, p.y);

  if (line_join != miter_join) {
    // Outside corner
    dist = max(dist, line_join_distance_no_miter(
      p, vec2(0.0, 0.0), vec2(1.0/SQRT2, 1.0/SQRT2), v_linewidth/(2.0*SQRT2), line_join));

    // Inside corner
    dist = min(dist, -line_join_distance_no_miter(
      p, vec2(-5.0*r/8.0, 0.0), vec2(-1.0/SQRT2, -1.0/SQRT2), v_linewidth/(2.0*SQRT2), line_join));
  }

  return dist;
}
#endif

#if defined(USE_ROUND_RECT)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  vec2 halfsize = v_size/2.0;
  vec2 p2 = abs(p) - halfsize;  // Offset from corner
  float dist = max(p2.x, p2.y);

  if (line_join != miter_join) {
    dist = max(dist, line_join_distance_no_miter(
      p2, vec2(0.0, 0.0), vec2(1.0/SQRT2, 1.0/SQRT2), v_linewidth/(2.0*SQRT2), line_join));
  }

  // Need to consider distance to all 4 corners
  // Order of border_radius is top_left, top_right, bottom_right, bottom_left
  vec4 border_radius = v_border_radius;
  vec4 xsign = vec4(-1.0, 1.0, 1.0, -1.0);
  vec4 ysign = vec4(-1.0, -1.0, 1.0, 1.0);
  for (int i = 0; i < 4; i++) {
    float radius = border_radius.x;
    p2 = p*vec2(xsign.x, ysign.x);  // In +ve quadrant
    vec2 offset = p2 - halfsize + radius;
    if (min(radius, min(offset.x, offset.y)) > 0.0) {
      dist = max(dist, length(offset) - radius);
    }
    // Swizzle
    border_radius.xyzw = border_radius.yzwx;
    xsign.xyzw = xsign.yzwx;
    ysign.xyzw = ysign.yzwx;
  }

  return dist;
}
#endif

#if defined(USE_RECT) || defined(USE_SQUARE) || defined(USE_SQUARE_CROSS) || defined(USE_SQUARE_DOT) || defined(USE_SQUARE_X)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  vec2 p2 = abs(p) - v_size/2.0;  // Offset from corner
  float dist = max(p2.x, p2.y);

  if (line_join != miter_join) {
    dist = max(dist, line_join_distance_no_miter(
      p2, vec2(0.0, 0.0), vec2(1.0/SQRT2, 1.0/SQRT2), v_linewidth/(2.0*SQRT2), line_join));
  }

  return dist;
}
#endif

#ifdef USE_SQUARE_PIN
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  p = abs(p);
  p = (p.y > p.x) ? p.yx : p.xy;
  // p is in octant between y=0 and y=x.
  // Quadratic bezier curve passes through (r, r), (11r/16, 0) and (r, -r).
  // Circular arc that passes through the same points has center at
  // x = r + 231r/160 = 2.44275r and y = 0 and hence radius is
  // x - 11r/16 = 1.75626 precisely.
  float r = 0.5*v_size.x;
  float center_x = r*2.44375;
  float radius = r*1.75626;
  float dist = radius - distance(p, vec2(center_x, 0.0));

  // Magic number is 0.5*sin(atan(8/5) - pi/4)
  dist = max(dist, line_join_distance_incl_miter(
    p, vec2(r, r), vec2(1.0/SQRT2, 1.0/SQRT2), v_linewidth*0.1124297533493792, line_join,
    vec2(8.0/sqrt(89.0), -5.0/sqrt(89.0))));

  return dist;
}
#endif

#if defined(USE_STAR) || defined(USE_STAR_DOT)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  const float SQRT5 = sqrt(5.0);
  const float COS72 = 0.25*(SQRT5 - 1.0);
  const float SIN72 = sqrt((5.0+SQRT5) / 8.0);

  float angle = atan(p.x, p.y);  // In range -pi to +pi clockwise from +y direction.
  angle = mod(angle, 0.4*PI) - 0.2*PI;  // In range -pi/5 to +pi/5 clockwise from +y direction.
  p = length(p)*vec2(cos(angle), abs(sin(angle)));  // (x,y) in pi/10 (36 degree) sector.

  // 2 corners are at (r, 0) and (r-a*SIN72, a*COS72) where a = r sqrt(5-2*sqrt(5)).
  // Line has outward-facing unit normal vec2(COS72, SIN72) and distance from
  // origin of dot(vec2(r, 0), vec2(COS72, SIN72)) = r*COS72
  float r = 0.5*v_size.x;
  float a = r*sqrt(5.0 - 2.0*SQRT5);
  float dist = dot(p, vec2(COS72, SIN72)) - r*COS72;

  if (line_join != miter_join) {
    // Outside corner
    dist = max(dist, line_join_distance_no_miter(
      p, vec2(r, 0.0), vec2(1.0, 0.0), v_linewidth*(0.5*COS72), line_join));

    // Inside corner
    const float COS36 = sqrt(0.5 + COS72/2.0);
    const float SIN36 = sqrt(0.5 - COS72/2.0);
    dist = min(dist, -line_join_distance_no_miter(
      p, vec2(r-a*SIN72, a*COS72), vec2(-COS36, -SIN36), v_linewidth*(0.5*COS36), line_join));
  }

  return dist;
}
#endif

#if defined(USE_TRIANGLE) || defined(USE_TRIANGLE_DOT) || defined(USE_INVERTED_TRIANGLE)
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  // For normal triangle 3 corners are at (-r, a), (r, a), (0, a-h)=(0, -2h/3)
  // given r = radius = v_size.x/2, h = SQRT3*r, a = h/3.
  // Sloping line has outward-facing unit normal vec2(h, -r)/2r = vec2(SQRT3, -1)/2
  // and distance from origin of a.  Horizontal line has outward-facing unit normal
  // vec2(0, 1) and distance from origin of a.
  float r = 0.5*v_size.x;
  float a = r*SQRT3/3.0;

  // Only need to consider +ve x.
#ifdef USE_INVERTED_TRIANGLE
  p = vec2(abs(p.x), -p.y);
#else
  p = vec2(abs(p.x), p.y);
#endif

  float dist = max(0.5*dot(p, vec2(SQRT3, -1.0)) - a,  // Distance from sloping line.
                   p.y - a);  // Distance from horizontal line.

  if (line_join != miter_join) {
    dist = max(dist, line_join_distance_no_miter(
      p, vec2(0.0, -(2.0/SQRT3)*r), vec2(0.0, -1.0), v_linewidth*0.25, line_join));

    dist = max(dist, line_join_distance_no_miter(
      p, vec2(r, a), vec2(SQRT3/2.0, 0.5), v_linewidth*0.25, line_join));
  }

  return dist;
}
#endif

#ifdef USE_TRIANGLE_PIN
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  float angle = atan(p.x, -p.y);  // In range -pi to +pi.
  angle = mod(angle, PI*2.0/3.0) - PI/3.0;  // In range -pi/3 to pi/3.
  p = length(p)*vec2(cos(angle), abs(sin(angle)));  // (x,y) in range 0 to pi/3.
  // Quadratic bezier curve passes through (a, r), ((a+b)/2, 0) and (a, -r) where
  // a = r/SQRT3, b = 3a/8 = r SQRT3/8.  Circular arc that passes through the same points has
  // center at (a+x, 0) and radius x+c where c = (a-b)/2 and x = (r**2 - c**2) / (2c).
  // Ignore r factor until the end so can use const.
  const float a = 1.0/SQRT3;
  const float b = SQRT3/8.0;
  const float c = (a-b)/2.0;
  const float x = (1.0 - c*c) / (2.0*c);
  const float center_x = x + a;
  const float radius = x + c;
  float r = 0.5*v_size.x;
  float dist = r*radius - distance(p, vec2(r*center_x, 0.0));

  // Magic number is 0.5*sin(atan(8*sqrt(3)/5) - pi/3)
  dist = max(dist, line_join_distance_incl_miter(
    p, vec2(a*r, r), vec2(0.5, 0.5*SQRT3), v_linewidth*0.0881844526878324, line_join,
    vec2(8.0*SQRT3, -5.0)/sqrt(217.0)));

  return dist;
}
#endif

#ifdef USE_X
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  p = vec2((p.x + p.y)/SQRT2, (p.x - p.y)/SQRT2);
  return one_cross(p, line_cap, 0.5*v_size.x);
}
#endif

#ifdef USE_Y
float marker_distance(in vec2 p, in int line_cap, in int line_join)
{
  // Assuming v_size.x == v.size_y
  return one_y(p, line_cap, 0.5*v_size.x);
}
#endif

// Convert distance from edge of marker to fraction in range 0 to 1, depending
// on antialiasing width.
float distance_to_fraction(in float dist)
{
  return 1.0 - smoothstep(-0.5*u_antialias, 0.5*u_antialias, dist);
}

// Return fraction from 0 (no fill color) to 1 (full fill color).
float fill_fraction(in float dist)
{
  return distance_to_fraction(dist);
}

// Return fraction in range 0 (no line color) to 1 (full line color).
float line_fraction(in float dist)
{
  return distance_to_fraction(abs(dist) - 0.5*v_linewidth);
}

// Return fraction (in range 0 to 1) of a color, with premultiplied alpha.
vec4 fractional_color(in vec4 color, in float fraction)
{
  color.a *= fraction;
  color.rgb *= color.a;
  return color;
}

// Blend colors that have premultiplied alpha.
vec4 blend_colors(in vec4 src, in vec4 dest)
{
  return (1.0 - src.a)*dest + src;
}

#ifdef APPEND_DOT
float dot_fraction(in vec2 p)
{
  // Assuming v_size.x == v_size.y
  float radius = 0.125*v_size.x;
  float dot_distance = max(length(p) - radius, -0.5*u_antialias);
  return fill_fraction(dot_distance);
}
#endif

#ifdef HATCH
// Wrap coordinate(s) by removing integer part to give distance from center of
// repeat, in the range -0.5 to +0.5.
float wrap(in float x)
{
  return fract(x) - 0.5;
}

vec2 wrap(in vec2 xy)
{
  return fract(xy) - 0.5;
}

// Return fraction from 0 (no hatch color) to 1 (full hatch color).
float hatch_fraction(in vec2 coords, in int hatch_pattern)
{
  float scale = v_hatch_scale; // Hatch repeat distance.

  // Coordinates and linewidth/halfwidth are scaled to hatch repeat distance.
  coords = coords / scale;
  float halfwidth = 0.5*v_hatch_weight / scale; // Half the hatch linewidth.

  // Default is to return fraction of zero, i.e. no pattern.
  float dist = u_antialias;

  if (hatch_pattern == hatch_dot) {
    const float dot_radius = 0.25;
    dist = length(wrap(coords)) - dot_radius;
  }
  else if (hatch_pattern == hatch_ring) {
    const float ring_radius = 0.25;
    dist = abs(length(wrap(coords)) - ring_radius) - halfwidth;
  }
  else if (hatch_pattern == hatch_horizontal_line) {
    dist = abs(wrap(coords.y)) - halfwidth;
  }
  else if (hatch_pattern == hatch_vertical_line) {
    dist = abs(wrap(coords.x)) - halfwidth;
  }
  else if (hatch_pattern == hatch_cross) {
    dist = min(abs(wrap(coords.x)), abs(wrap(coords.y))) - halfwidth;
  }
  else if (hatch_pattern == hatch_horizontal_dash) {
    // Dashes have square caps.
    const float halflength = 0.25;
    dist = max(abs(wrap(coords.y)),
               abs(wrap(coords.x) + 0.25) - halflength) - halfwidth;
  }
  else if (hatch_pattern == hatch_vertical_dash) {
    const float halflength = 0.25;
    dist = max(abs(wrap(coords.x)),
               abs(wrap(coords.y) + 0.25) - halflength) - halfwidth;
  }
  else if (hatch_pattern == hatch_spiral) {
    vec2 wrap2 = wrap(coords);
    float angle = wrap(atan(wrap2.y, wrap2.x) / (2.0*PI));
    // Canvas spiral radius increases by scale*pi/15 each rotation.
    const float dr = PI/15.0;
    float radius = length(wrap2);
    // At any angle, spiral lines are equally spaced dr apart.
    // Find distance to nearest of these lines.
    float frac = fract((radius - dr*angle) / dr); // 0 to 1.
    dist = dr*(abs(frac - 0.5));
    dist = min(dist, radius) - halfwidth; // Consider center point also.
  }
  else if (hatch_pattern == hatch_right_diagonal_line) {
    dist = abs(wrap(2.0*coords.x + coords.y))/sqrt(5.0) - halfwidth;
  }
  else if (hatch_pattern == hatch_left_diagonal_line) {
    dist = abs(wrap(2.0*coords.x - coords.y))/sqrt(5.0) - halfwidth;
  }
  else if (hatch_pattern == hatch_diagonal_cross) {
    coords = vec2(coords.x + coords.y + 0.5, coords.x - coords.y + 0.5);
    dist = min(abs(wrap(coords.x)), abs(wrap(coords.y))) / SQRT2 - halfwidth;
  }
  else if (hatch_pattern == hatch_right_diagonal_dash) {
    float across = coords.x + coords.y + 0.5;
    dist = abs(wrap(across)) / SQRT2; // Distance to nearest solid line.

    across = floor(across); // Offset for dash.
    float along = wrap(0.5*(coords.x - coords.y + across));
    const float halflength = 0.25;
    along = abs(along) - halflength; // Distance along line.

    dist = max(dist, along) - halfwidth;
  }
  else if (hatch_pattern == hatch_left_diagonal_dash) {
    float across = coords.x - coords.y + 0.5;
    dist = abs(wrap(across)) / SQRT2; // Distance to nearest solid line.

    across = floor(across); // Offset for dash.
    float along = wrap(0.5*(coords.x + coords.y + across));
    const float halflength = 0.25;
    along = abs(along) - halflength; // Distance along line.

    dist = max(dist, along) - halfwidth;
  }
  else if (hatch_pattern == hatch_horizontal_wave) {
    float wrapx = wrap(coords.x);
    float wrapy = wrap(coords.y - 0.25 + abs(wrapx));
    dist = abs(wrapy) / SQRT2 - halfwidth;
  }
  else if (hatch_pattern == hatch_vertical_wave) {
    float wrapy = wrap(coords.y);
    float wrapx = wrap(coords.x - 0.25 + abs(wrapy));
    dist = abs(wrapx) / SQRT2 - halfwidth;
  }
  else if (hatch_pattern == hatch_criss_cross) {
    float plus = min(abs(wrap(coords.x)), abs(wrap(coords.y)));

    coords = vec2(coords.x + coords.y + 0.5, coords.x - coords.y + 0.5);
    float X = min(abs(wrap(coords.x)), abs(wrap(coords.y))) / SQRT2;

    dist = min(plus, X) - halfwidth;
  }

  return distance_to_fraction(dist*scale);
}
#endif

void main()
{
  int line_cap = int(v_line_cap + 0.5);
  int line_join = int(v_line_join + 0.5);
#ifdef HATCH
  int hatch_pattern = int(v_hatch_pattern + 0.5);
#endif

  float dist = marker_distance(v_coords, line_cap, line_join);

#ifdef LINE_ONLY
  vec4 color = vec4(0.0, 0.0, 0.0, 0.0);
#else
  float fill_frac = fill_fraction(dist);
  vec4 color = fractional_color(v_fill_color, fill_frac);
#endif

#if defined(HATCH) && !defined(LINE_ONLY)
  if (hatch_pattern > 0 && fill_frac > 0.0) {
    float hatch_frac = hatch_fraction(v_hatch_coords, hatch_pattern);
    vec4 hatch_color = fractional_color(v_hatch_color, hatch_frac*fill_frac);
    color = blend_colors(hatch_color, color);
  }
#endif

  float line_frac = line_fraction(dist);

#ifdef APPEND_DOT
  line_frac = max(line_frac, dot_fraction(v_coords));
#endif
#ifdef APPEND_CROSS
  line_frac = max(line_frac, line_fraction(one_cross(v_coords, line_cap, 0.5*v_size.x)));
#endif
#ifdef APPEND_CROSS_2
  vec2 lengths = vec2(v_size.x/3.0, v_size.x/2.0);
  line_frac = max(line_frac, line_fraction(one_cross_2(v_coords, line_cap, lengths)));
#endif
#ifdef APPEND_X
  vec2 p = vec2((v_coords.x + v_coords.y)/SQRT2, (v_coords.x - v_coords.y)/SQRT2);
  line_frac = max(line_frac, line_fraction(one_cross(p, line_cap, APPEND_X_LEN)));
#endif
#ifdef APPEND_Y
  line_frac = max(line_frac, line_fraction(one_y(v_coords, line_cap, 0.5*v_size.x)));
#endif

  if (line_frac > 0.0) {
    vec4 line_color = fractional_color(v_line_color, line_frac);
    color = blend_colors(line_color, color);
  }

  gl_FragColor = color;
}
`;
  let regl_wrapper = null;
  function get_regl(O) {
    return regl_wrapper == null && (regl_wrapper = new ReglWrapper(O)), regl_wrapper;
  }
  class ReglWrapper {
    constructor(_) {
      mt(this, "_regl");
      mt(this, "_regl_available");
      mt(this, "_dash_cache");
      // Drawing functions.
      mt(this, "_accumulate");
      mt(this, "_image");
      mt(this, "_solid_line");
      mt(this, "_dashed_line");
      mt(this, "_marker_no_hatch_map", /* @__PURE__ */ new Map());
      mt(this, "_marker_hatch_map", /* @__PURE__ */ new Map());
      // Static Buffers/Elements
      mt(this, "_line_geometry");
      mt(this, "_line_triangles");
      mt(this, "_rect_geometry");
      mt(this, "_rect_triangles");
      // WebGL state variables.
      mt(this, "_scissor");
      mt(this, "_viewport");
      // WebGL framebuffer used to accumulate glyph rendering before single blit to Canvas.
      mt(this, "_framebuffer");
      mt(this, "_framebuffer_texture");
      try {
        this._regl = createRegl({
          gl: _,
          extensions: [
            "ANGLE_instanced_arrays",
            "EXT_blend_minmax"
          ]
        }), this._regl_available = !0, this._line_geometry = this._regl.buffer({
          usage: "static",
          type: "float",
          data: [[-2, 0], [-1, -1], [1, -1], [1, 1], [-1, 1]]
        }), this._line_triangles = this._regl.elements({
          usage: "static",
          primitive: "triangle fan",
          data: [0, 1, 2, 3, 4]
        }), this._rect_geometry = this._regl.buffer({
          usage: "static",
          type: "float",
          data: [[-1, -1], [1, -1], [1, 1], [-1, 1]]
        }), this._rect_triangles = this._regl.elements({
          usage: "static",
          primitive: "triangle fan",
          data: [0, 1, 2, 3]
        });
      } catch {
        this._regl_available = !1;
      }
    }
    // Create and return ReGL Buffer.
    buffer(_) {
      return this._regl.buffer(_);
    }
    clear(_, H) {
      this._viewport = { x: 0, y: 0, width: _, height: H }, this._regl.clear({ color: [0, 0, 0, 0] });
    }
    clear_framebuffer(_) {
      this._regl.clear({ color: [0, 0, 0, 0], framebuffer: _ });
    }
    get framebuffer_and_texture() {
      const { _regl: _ } = this, { _gl: H } = _, V = {
        height: H.drawingBufferHeight,
        width: H.drawingBufferWidth
      };
      return this._framebuffer_texture == null ? this._framebuffer_texture = _.texture(V) : this._framebuffer_texture(V), this._framebuffer == null && (this._framebuffer = _.framebuffer({
        // Auto-sizes to size of texture.
        color: this._framebuffer_texture,
        depth: !1,
        stencil: !1
      })), [this._framebuffer, this._framebuffer_texture];
    }
    get has_webgl() {
      return this._regl_available;
    }
    get scissor() {
      return this._scissor;
    }
    set_scissor(_, H, V, q) {
      this._scissor = { x: _, y: H, width: V, height: q };
    }
    texture(_) {
      return this._regl.texture(_);
    }
    get viewport() {
      return this._viewport;
    }
    accumulate() {
      return this._accumulate == null && (this._accumulate = regl_accumulate(this._regl, this._rect_geometry, this._rect_triangles)), this._accumulate;
    }
    dashed_line() {
      return this._dashed_line == null && (this._dashed_line = regl_dashed_line(this._regl, this._line_geometry, this._line_triangles)), this._dashed_line;
    }
    get_dash(_) {
      return this._dash_cache == null && (this._dash_cache = new DashCache(this._regl)), this._dash_cache.get(_);
    }
    image() {
      return this._image == null && (this._image = regl_image(this._regl, this._rect_geometry, this._rect_triangles)), this._image;
    }
    marker_no_hatch(_) {
      let H = this._marker_no_hatch_map.get(_);
      return H == null && (H = regl_marker(this._regl, _), this._marker_no_hatch_map.set(_, H)), H;
    }
    marker_hatch(_) {
      let H = this._marker_hatch_map.get(_);
      return H == null && (H = regl_marker_hatch(this._regl, _), this._marker_hatch_map.set(_, H)), H;
    }
    solid_line() {
      return this._solid_line == null && (this._solid_line = regl_solid_line(this._regl, this._line_geometry, this._line_triangles)), this._solid_line;
    }
  }
  mt(ReglWrapper, "__name__", "ReglWrapper");
  function regl_accumulate(O, _, H) {
    const V = {
      vert: shader$7,
      frag: shader$6,
      attributes: {
        a_position: {
          buffer: _,
          divisor: 0
        }
      },
      uniforms: {
        u_framebuffer_tex: O.prop("framebuffer_tex")
      },
      elements: H,
      blend: {
        enable: !0,
        func: {
          srcRGB: "one",
          srcAlpha: "one",
          dstRGB: "one minus src alpha",
          dstAlpha: "one minus src alpha"
        }
      },
      depth: { enable: !1 },
      scissor: {
        enable: !0,
        box: O.prop("scissor")
      },
      viewport: O.prop("viewport")
    };
    return O(V);
  }
  function regl_image(O, _, H) {
    const V = {
      vert: shader$5,
      frag: shader$4,
      attributes: {
        a_position: {
          buffer: _,
          divisor: 0
        },
        a_bounds(q, G) {
          return G.bounds.to_attribute_config();
        }
      },
      uniforms: {
        u_canvas_size: O.prop("canvas_size"),
        u_tex: O.prop("tex"),
        u_global_alpha: O.prop("global_alpha")
      },
      elements: H,
      blend: {
        enable: !0,
        func: {
          srcRGB: "one",
          srcAlpha: "one",
          dstRGB: "one minus src alpha",
          dstAlpha: "one minus src alpha"
        }
      },
      depth: { enable: !1 },
      scissor: {
        enable: !0,
        box: O.prop("scissor")
      },
      viewport: O.prop("viewport")
    };
    return O(V);
  }
  function regl_solid_line(O, _, H) {
    const V = {
      vert: shader$3,
      frag: shader$2,
      attributes: {
        a_position: {
          buffer: _,
          divisor: 0
        },
        a_point_prev(q, G) {
          return G.points.to_attribute_config(G.point_offset);
        },
        a_point_start(q, G) {
          return G.points.to_attribute_config(G.point_offset + 2);
        },
        a_point_end(q, G) {
          return G.points.to_attribute_config(G.point_offset + 4);
        },
        a_point_next(q, G) {
          return G.points.to_attribute_config(G.point_offset + 6);
        },
        a_show_prev(q, G) {
          return G.show.to_attribute_config(G.point_offset / 2 - G.line_offset);
        },
        a_show_curr(q, G) {
          return G.show.to_attribute_config(G.point_offset / 2 - G.line_offset + 1);
        },
        a_show_next(q, G) {
          return G.show.to_attribute_config(G.point_offset / 2 - G.line_offset + 2);
        },
        a_linewidth(q, G) {
          return G.linewidth.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_line_color(q, G) {
          return G.line_color.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_line_cap(q, G) {
          return G.line_cap.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_line_join(q, G) {
          return G.line_join.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        }
      },
      uniforms: {
        u_canvas_size: O.prop("canvas_size"),
        u_antialias: O.prop("antialias"),
        u_miter_limit: O.prop("miter_limit")
      },
      elements: H,
      instances: O.prop("nsegments"),
      blend: {
        enable: !0,
        equation: "max",
        func: {
          srcRGB: 1,
          srcAlpha: 1,
          dstRGB: 1,
          dstAlpha: 1
        }
      },
      depth: { enable: !1 },
      framebuffer: O.prop("framebuffer"),
      scissor: {
        enable: !0,
        box: O.prop("scissor")
      },
      viewport: O.prop("viewport")
    };
    return O(V);
  }
  function regl_dashed_line(O, _, H) {
    const V = {
      vert: `#define DASHED
${shader$3}
`,
      frag: `#define DASHED
${shader$2}
`,
      attributes: {
        a_position: {
          buffer: _,
          divisor: 0
        },
        a_point_prev(q, G) {
          return G.points.to_attribute_config(G.point_offset);
        },
        a_point_start(q, G) {
          return G.points.to_attribute_config(G.point_offset + 2);
        },
        a_point_end(q, G) {
          return G.points.to_attribute_config(G.point_offset + 4);
        },
        a_point_next(q, G) {
          return G.points.to_attribute_config(G.point_offset + 6);
        },
        a_show_prev(q, G) {
          return G.show.to_attribute_config(G.point_offset / 2 - G.line_offset);
        },
        a_show_curr(q, G) {
          return G.show.to_attribute_config(G.point_offset / 2 - G.line_offset + 1);
        },
        a_show_next(q, G) {
          return G.show.to_attribute_config(G.point_offset / 2 - G.line_offset + 2);
        },
        a_linewidth(q, G) {
          return G.linewidth.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_line_color(q, G) {
          return G.line_color.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_line_cap(q, G) {
          return G.line_cap.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_line_join(q, G) {
          return G.line_join.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_length_so_far(q, G) {
          return G.length_so_far.to_attribute_config(G.point_offset / 2 - 3 * G.line_offset);
        },
        a_dash_tex_info(q, G) {
          return G.dash_tex_info.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_dash_scale(q, G) {
          return G.dash_scale.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        },
        a_dash_offset(q, G) {
          return G.dash_offset.to_attribute_config_nested(G.line_offset, G.nsegments + 3);
        }
      },
      uniforms: {
        u_canvas_size: O.prop("canvas_size"),
        u_antialias: O.prop("antialias"),
        u_miter_limit: O.prop("miter_limit"),
        u_dash_tex: O.prop("dash_tex")
      },
      elements: H,
      instances: O.prop("nsegments"),
      blend: {
        enable: !0,
        equation: "max",
        func: {
          srcRGB: 1,
          srcAlpha: 1,
          dstRGB: 1,
          dstAlpha: 1
        }
      },
      depth: { enable: !1 },
      framebuffer: O.prop("framebuffer"),
      scissor: {
        enable: !0,
        box: O.prop("scissor")
      },
      viewport: O.prop("viewport")
    };
    return O(V);
  }
  function regl_marker(O, _, H = [], V = [], q) {
    const G = H.map((te) => `#define ${te}`).join(`
`), J = V.map((te) => `#define ${te}`).join(`
`), Z = {
      vert: `${G}
#define MULTI_MARKER
#define USE_${_.toUpperCase()}
${shader$1}
`,
      frag: `${J}
#define USE_${_.toUpperCase()}
${shader}
`,
      attributes: {
        a_position: {
          buffer: O.buffer([[-0.5, -0.5], [-0.5, 0.5], [0.5, 0.5], [0.5, -0.5]]),
          divisor: 0
        },
        a_center(te, ne) {
          return ne.center.to_attribute_config(0, ne.nmarkers);
        },
        a_width(te, ne) {
          return ne.width.to_attribute_config(0, ne.nmarkers);
        },
        a_height(te, ne) {
          return ne.height.to_attribute_config(0, ne.nmarkers);
        },
        a_angle(te, ne) {
          return ne.angle.to_attribute_config(0, ne.nmarkers);
        },
        a_aux(te, ne) {
          return ne.aux.to_attribute_config(0, ne.nmarkers);
        },
        a_linewidth(te, ne) {
          return ne.linewidth.to_attribute_config(0, ne.nmarkers);
        },
        a_line_color(te, ne) {
          return ne.line_color.to_attribute_config(0, ne.nmarkers);
        },
        a_fill_color(te, ne) {
          return ne.fill_color.to_attribute_config(0, ne.nmarkers);
        },
        a_line_cap(te, ne) {
          return ne.line_cap.to_attribute_config(0, ne.nmarkers);
        },
        a_line_join(te, ne) {
          return ne.line_join.to_attribute_config(0, ne.nmarkers);
        },
        a_show(te, ne) {
          return ne.show.to_attribute_config(0, ne.nmarkers);
        },
        ...q
      },
      uniforms: {
        u_canvas_size: O.prop("canvas_size"),
        u_antialias: O.prop("antialias"),
        u_size_hint: O.prop("size_hint"),
        u_border_radius: O.prop("border_radius")
      },
      count: 4,
      primitive: "triangle fan",
      instances: O.prop("nmarkers"),
      blend: {
        enable: !0,
        func: {
          srcRGB: "one",
          srcAlpha: "one",
          dstRGB: "one minus src alpha",
          dstAlpha: "one minus src alpha"
        }
      },
      depth: { enable: !1 },
      scissor: {
        enable: !0,
        box: O.prop("scissor")
      },
      viewport: O.prop("viewport")
    };
    return O(Z);
  }
  function regl_marker_hatch(O, _) {
    return regl_marker(O, _, ["HATCH"], ["HATCH"], {
      a_hatch_pattern(V, q) {
        return q.hatch_pattern.to_attribute_config(0, q.nmarkers);
      },
      a_hatch_scale(V, q) {
        return q.hatch_scale.to_attribute_config(0, q.nmarkers);
      },
      a_hatch_weight(V, q) {
        return q.hatch_weight.to_attribute_config(0, q.nmarkers);
      },
      a_hatch_color(V, q) {
        return q.hatch_color.to_attribute_config(0, q.nmarkers);
      }
    });
  }
  class BaseGLGlyph {
    constructor(_, H) {
      mt(this, "regl_wrapper");
      mt(this, "glyph");
      mt(this, "nvertices", 0);
      mt(this, "size_changed", !1);
      mt(this, "data_changed", !1);
      mt(this, "data_mapped", !1);
      mt(this, "visuals_changed", !1);
      this.regl_wrapper = _, this.glyph = H;
    }
    set_data_changed() {
      const { data_size: _ } = this.glyph;
      _ != this.nvertices && (this.nvertices = _, this.size_changed = !0), this.data_changed = !0;
    }
    set_data_mapped() {
      this.data_mapped = !0;
    }
    set_visuals_changed() {
      this.visuals_changed = !0;
    }
    render(_, H, V) {
      if (H.length == 0)
        return;
      const { width: q, height: G } = this.glyph.renderer.plot_view.canvas_view.webgl.canvas, { pixel_ratio: J } = this.glyph.renderer.plot_view.canvas_view, Z = {
        pixel_ratio: J,
        // Needed to scale antialiasing
        width: q / J,
        height: G / J
      };
      this.draw(H, V, Z);
    }
  }
  mt(BaseGLGlyph, "__name__", "BaseGLGlyph");
  function interleave(O, _, H, V, q) {
    for (let G = 0; G < H; G++) {
      const J = O[G], Z = _[G];
      isFinite(J + Z) ? (q[2 * G] = J, q[2 * G + 1] = Z) : (q[2 * G] = V, q[2 * G + 1] = V);
    }
  }
  const cap_lookup = { butt: 0, round: 1, square: 2 }, join_lookup = { miter: 0, round: 1, bevel: 2 }, hatch_pattern_lookup = {
    blank: 0,
    dot: 1,
    ring: 2,
    horizontal_line: 3,
    vertical_line: 4,
    cross: 5,
    horizontal_dash: 6,
    vertical_dash: 7,
    spiral: 8,
    right_diagonal_line: 9,
    left_diagonal_line: 10,
    diagonal_cross: 11,
    right_diagonal_dash: 12,
    left_diagonal_dash: 13,
    horizontal_wave: 14,
    vertical_wave: 15,
    criss_cross: 16
  };
  function hatch_pattern_to_index(O) {
    return hatch_pattern_lookup[hatch_aliases[O] ?? O] ?? 0;
  }
  function marker_type_to_size_hint(O) {
    switch (O) {
      case "dash":
        return 1;
      case "dot":
        return 2;
      case "diamond":
      case "diamond_cross":
      case "diamond_dot":
        return 3;
      case "hex":
      case "hex_tile":
        return 4;
      case "square_pin":
        return 5;
      case "inverted_triangle":
      case "triangle":
      case "triangle_dot":
        return 6;
      case "triangle_pin":
        return 7;
      case "star":
      case "star_dot":
        return 8;
      default:
        return 0;
    }
  }
  class WrappedBuffer {
    constructor(_, H = 1) {
      mt(this, "regl_wrapper");
      mt(this, "buffer");
      mt(this, "array");
      mt(this, "is_scalar");
      // Number of buffer elements per rendered primitive, e.g. for RGBA buffers this is 4
      // as a single color is 4 x uint8 = 32-bit in total.
      mt(this, "elements_per_primitive");
      this.regl_wrapper = _, this.is_scalar = !0, this.elements_per_primitive = H;
    }
    // Return array if already know it exists and is the correct length.
    get_array() {
      return assert$2(this.array != null, "WrappedBuffer not yet initialised"), this.array;
    }
    // Return array of correct size, creating it if necessary.
    // Must call update() when finished setting the array values.
    get_sized_array(_) {
      return (this.array == null || this.array.length != _) && (this.array = this.new_array(_)), this.array;
    }
    is_normalized() {
      return !1;
    }
    get length() {
      return this.array != null ? this.array.length : 0;
    }
    set_from_array(_) {
      const H = _.length, V = this.get_sized_array(H);
      for (let q = 0; q < H; q++)
        V[q] = _[q];
      this.update();
    }
    set_from_prop(_) {
      const H = _.is_Scalar() ? 1 : _.length, V = this.get_sized_array(H);
      for (let q = 0; q < H; q++)
        V[q] = _.get(q);
      this.update(_.is_Scalar());
    }
    set_from_scalar(_) {
      this.get_sized_array(1).fill(_), this.update(!0);
    }
    // Return a ReGL AttributeConfig that corresponds to one value for each glyph
    // or the same value for a number of glyphs.  A buffer passed to ReGL for
    // instanced rendering can be used for multiple rendering calls and the
    // important attributes for this are the offset (in bytes) into the buffer
    // and the divisor, which is the number of instances rendered before the
    // offset is advanced to the next buffer element.
    // to_attribute_config() is used for the common case of a single render call
    // per buffer with visual properties that are either scalar or vector.
    // Visual properties of scatter markers are an good example, and scalar_divisor
    // would be the number of markers rendered.
    to_attribute_config(_ = 0, H = 1) {
      return {
        buffer: this.buffer,
        divisor: this.is_scalar ? H : 1,
        normalized: this.is_normalized(),
        offset: _ * this.bytes_per_element()
      };
    }
    // to_attribute_config_nested() is used for the more complicated case in
    // which the vectorisation is nested, such as rendering multi_lines where
    // each visual property has a single buffer that is used multiple times, once
    // for each of the constituent lines.  Vector properties are therefore
    // constant for each constituent line (composed of multiple rendered
    // instances) but change between lines.
    to_attribute_config_nested(_ = 0, H = 1) {
      return {
        buffer: this.buffer,
        divisor: H * this.elements_per_primitive,
        normalized: this.is_normalized(),
        offset: this.is_scalar ? 0 : _ * this.bytes_per_element() * this.elements_per_primitive
      };
    }
    // Update ReGL buffer with data contained in array in preparation for passing
    // it to the GPU.  This function must be called after get_sized_array().
    update(_ = !1) {
      this.buffer == null ? this.buffer = this.regl_wrapper.buffer({
        usage: "dynamic",
        data: this.array
      }) : this.buffer({ data: this.array }), this.is_scalar = _;
    }
  }
  mt(WrappedBuffer, "__name__", "WrappedBuffer");
  class Float32Buffer extends WrappedBuffer {
    bytes_per_element() {
      return Float32Array.BYTES_PER_ELEMENT;
    }
    new_array(_) {
      return new Float32Array(_);
    }
  }
  mt(Float32Buffer, "__name__", "Float32Buffer");
  class Uint8Buffer extends WrappedBuffer {
    bytes_per_element() {
      return Uint8Array.BYTES_PER_ELEMENT;
    }
    new_array(_) {
      return new Uint8Array(_);
    }
    set_from_color(_, H) {
      const V = _.is_Scalar() && H.is_Scalar(), q = V ? 1 : _.length, G = this.get_sized_array(4 * q);
      for (let J = 0; J < q; J++) {
        const [Z, te, ne, ee] = color2rgba(_.get(J), H.get(J));
        G[4 * J] = Z, G[4 * J + 1] = te, G[4 * J + 2] = ne, G[4 * J + 3] = ee;
      }
      this.update(V);
    }
    set_from_hatch_pattern(_) {
      const H = _.is_Scalar() ? 1 : _.length, V = this.get_sized_array(H);
      for (let q = 0; q < H; q++)
        V[q] = hatch_pattern_to_index(_.get(q));
      this.update(_.is_Scalar());
    }
    set_from_line_cap(_) {
      const H = _.is_Scalar() ? 1 : _.length, V = this.get_sized_array(H);
      for (let q = 0; q < H; q++)
        V[q] = cap_lookup[_.get(q)];
      this.update(_.is_Scalar());
    }
    set_from_line_join(_) {
      const H = _.is_Scalar() ? 1 : _.length, V = this.get_sized_array(H);
      for (let q = 0; q < H; q++)
        V[q] = join_lookup[_.get(q)];
      this.update(_.is_Scalar());
    }
  }
  mt(Uint8Buffer, "__name__", "Uint8Buffer");
  class NormalizedUint8Buffer extends Uint8Buffer {
    is_normalized() {
      return !0;
    }
  }
  mt(NormalizedUint8Buffer, "__name__", "NormalizedUint8Buffer");
  class BaseMarkerGL extends BaseGLGlyph {
    constructor() {
      super(...arguments);
      mt(this, "_antialias", 1.5);
      // data properties
      mt(this, "_centers", new Float32Buffer(this.regl_wrapper));
      mt(this, "_widths", new Float32Buffer(this.regl_wrapper));
      mt(this, "_heights", new Float32Buffer(this.regl_wrapper));
      mt(this, "_angles", new Float32Buffer(this.regl_wrapper));
      mt(this, "_auxs", new Float32Buffer(this.regl_wrapper));
      // used by RectGL
      mt(this, "_border_radius", [0, 0, 0, 0]);
      mt(this, "_border_radius_nonzero", !1);
      // indices properties
      mt(this, "_show", new Uint8Buffer(this.regl_wrapper));
      mt(this, "_show_all", !1);
      // visual properties
      mt(this, "_linewidths", new Float32Buffer(this.regl_wrapper));
      mt(this, "_line_caps", new Uint8Buffer(this.regl_wrapper));
      mt(this, "_line_joins", new Uint8Buffer(this.regl_wrapper));
      mt(this, "_line_rgba", new NormalizedUint8Buffer(this.regl_wrapper, 4));
      mt(this, "_fill_rgba", new NormalizedUint8Buffer(this.regl_wrapper, 4));
      // Only needed if have hatch pattern, either all or none of the buffers are set.
      mt(this, "_have_hatch", !1);
      mt(this, "_hatch_patterns", new Uint8Buffer(this.regl_wrapper));
      mt(this, "_hatch_scales", new Float32Buffer(this.regl_wrapper));
      mt(this, "_hatch_weights", new Float32Buffer(this.regl_wrapper));
      mt(this, "_hatch_rgba", new NormalizedUint8Buffer(this.regl_wrapper, 4));
      mt(this, "_did_set_once", !1);
    }
    marker_props(H) {
      return {
        width: H._widths,
        height: H._heights,
        angle: H._angles,
        aux: H._auxs,
        border_radius: H._border_radius
      };
    }
    get line_props() {
      return {
        linewidth: this._linewidths,
        line_color: this._line_rgba,
        line_cap: this._line_caps,
        line_join: this._line_joins
      };
    }
    get fill_props() {
      return {
        fill_color: this._fill_rgba
      };
    }
    get hatch_props() {
      return {
        hatch_pattern: this._hatch_patterns,
        hatch_scale: this._hatch_scales,
        hatch_weight: this._hatch_weights,
        hatch_color: this._hatch_rgba
      };
    }
    _draw_one_marker_type(H, V, q) {
      const G = {
        scissor: this.regl_wrapper.scissor,
        viewport: this.regl_wrapper.viewport,
        canvas_size: [V.width, V.height],
        size_hint: marker_type_to_size_hint(H),
        nmarkers: q.nvertices,
        antialias: this._antialias / V.pixel_ratio,
        show: this._show,
        center: q._centers,
        ...this.marker_props(q),
        ...this.line_props,
        ...this.fill_props
      };
      if (this._have_hatch) {
        const J = { ...G, ...this.hatch_props };
        this.regl_wrapper.marker_hatch(H)(J);
      } else
        this.regl_wrapper.marker_no_hatch(H)(G);
    }
    set_data() {
      this._did_set_once || (this._did_set_once = !0, this._set_once()), this._set_data();
    }
    _set_once() {
    }
    _set_visuals() {
      const { line: H, fill: V, hatch: q } = this._get_visuals();
      this._linewidths.set_from_prop(H.line_width), this._line_caps.set_from_line_cap(H.line_cap), this._line_joins.set_from_line_join(H.line_join), this._line_rgba.set_from_color(H.line_color, H.line_alpha), this._fill_rgba.set_from_color(V.fill_color, V.fill_alpha), this._have_hatch = q.doit, this._have_hatch && (this._hatch_patterns.set_from_hatch_pattern(q.hatch_pattern), this._hatch_scales.set_from_prop(q.hatch_scale), this._hatch_weights.set_from_prop(q.hatch_weight), this._hatch_rgba.set_from_color(q.hatch_color, q.hatch_alpha));
    }
  }
  mt(BaseMarkerGL, "__name__", "BaseMarkerGL"), // Avoiding use of nan or inf to represent missing data in webgl as shaders may
  // have reduced floating point precision. So here using a large-ish negative
  // value instead.
  mt(BaseMarkerGL, "missing_point", -1e4);
  class SingleMarkerGL extends BaseMarkerGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    _get_visuals() {
      return this.glyph.visuals;
    }
    draw(H, V, q) {
      this._draw_impl(H, q, V.glglyph);
    }
    _draw_impl(H, V, q) {
      (q.data_changed || q.data_mapped) && (q.set_data(), q.data_changed = !1, q.data_mapped = !1), this.visuals_changed && (this._set_visuals(), this.visuals_changed = !1);
      const G = q.nvertices, J = this._show.length, Z = this._show.get_sized_array(G);
      if (H.length < G) {
        this._show_all = !1, Z.fill(0);
        for (let te = 0; te < H.length; te++)
          Z[H[te]] = 255;
      } else (!this._show_all || J != G) && (this._show_all = !0, Z.fill(255));
      this._show.update(), this._draw_one_marker_type(q.marker_type, V, q);
    }
  }
  mt(SingleMarkerGL, "__name__", "SingleMarkerGL");
  class SXSYGlyphGL extends SingleMarkerGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    _set_data() {
      const H = this.nvertices, V = this._centers.get_sized_array(2 * H);
      interleave(this.glyph.sx, this.glyph.sy, H, SingleMarkerGL.missing_point, V), this._centers.update();
    }
  }
  mt(SXSYGlyphGL, "__name__", "SXSYGlyphGL");
  class AnnularWedgeGL extends SXSYGlyphGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    get marker_type() {
      return "annular_wedge";
    }
    get outer_radius() {
      return this._widths;
    }
    get inner_radius() {
      return this._heights;
    }
    get start_angle() {
      return this._angles;
    }
    get end_angle() {
      return this._auxs;
    }
    _set_data() {
      super._set_data(), this.outer_radius.set_from_array(this.glyph.souter_radius), this.inner_radius.set_from_array(this.glyph.sinner_radius), this.glyph.model.direction == "anticlock" ? (this.start_angle.set_from_prop(this.glyph.start_angle), this.end_angle.set_from_prop(this.glyph.end_angle)) : (this.start_angle.set_from_prop(this.glyph.end_angle), this.end_angle.set_from_prop(this.glyph.start_angle));
    }
  }
  mt(AnnularWedgeGL, "__name__", "AnnularWedgeGL");
  const annular_wedge = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    AnnularWedgeGL
  }, Symbol.toStringTag, { value: "Module" }));
  class AnnulusGL extends SXSYGlyphGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    get marker_type() {
      return "annulus";
    }
    get outer_radius() {
      return this._widths;
    }
    get inner_radius() {
      return this._heights;
    }
    _set_data() {
      super._set_data(), this.outer_radius.set_from_array(this.glyph.souter_radius), this.inner_radius.set_from_array(this.glyph.sinner_radius);
    }
    _set_once() {
      super._set_once(), this._angles.set_from_scalar(0), this._auxs.set_from_scalar(0);
    }
  }
  mt(AnnulusGL, "__name__", "AnnulusGL");
  const annulus = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    AnnulusGL
  }, Symbol.toStringTag, { value: "Module" }));
  class BaseLineGL extends BaseGLGlyph {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      mt(this, "_antialias", 1.5);
      // Make this larger to test antialiasing at edges.
      mt(this, "_miter_limit", 10);
      // Threshold for miters to be replaced by bevels.
      // data properties
      mt(this, "_points");
      mt(this, "_show");
      // Applies to segments not points.
      // visual properties
      mt(this, "_linewidth", new Float32Buffer(this.regl_wrapper));
      mt(this, "_line_color", new NormalizedUint8Buffer(this.regl_wrapper, 4));
      mt(this, "_line_cap", new Uint8Buffer(this.regl_wrapper));
      mt(this, "_line_join", new Uint8Buffer(this.regl_wrapper));
      mt(this, "_is_dashed", !1);
      // visual properties that are only used if line is dashed.
      mt(this, "_length_so_far");
      // Depends on both data and visuals.
      mt(this, "_dash_tex", []);
      mt(this, "_dash_tex_info");
      mt(this, "_dash_scale");
      mt(this, "_dash_offset");
      this.glyph = V;
    }
    _draw_single(H, V, q, G, J, Z, te = null) {
      const ne = {
        scissor: this.regl_wrapper.scissor,
        viewport: this.regl_wrapper.viewport,
        canvas_size: [V.width, V.height],
        antialias: this._antialias / V.pixel_ratio,
        miter_limit: this._miter_limit,
        points: H._points,
        show: te ?? H._show,
        nsegments: J,
        linewidth: this._linewidth,
        line_color: this._line_color,
        line_cap: this._line_cap,
        line_join: this._line_join,
        framebuffer: Z,
        point_offset: G,
        line_offset: q
      };
      if (this._is_dashed && this._dash_tex[q] != null) {
        const ee = {
          ...ne,
          length_so_far: H._length_so_far,
          dash_tex: this._dash_tex[q],
          dash_tex_info: this._dash_tex_info,
          dash_scale: this._dash_scale,
          dash_offset: this._dash_offset
        };
        this.regl_wrapper.dashed_line()(ee);
      } else
        this.regl_wrapper.solid_line()(ne);
    }
    _set_length_single(H, V, q) {
      const G = H.length;
      let J = 0;
      for (let Z = 0; Z < G; Z++)
        H[Z] = J, q[Z + 1] == 1 ? J += Math.sqrt((V[2 * Z + 4] - V[2 * Z + 2]) ** 2 + (V[2 * Z + 5] - V[2 * Z + 3]) ** 2) : J = 0;
    }
    _set_points_single(H, V, q) {
      const G = H.length / 2 - 2, J = G > 2 && V[0] == V[G - 1] && q[0] == q[G - 1] && isFinite(V[0] + q[0]);
      for (let Z = 1; Z < G + 1; Z++)
        H[2 * Z] = V[Z - 1], H[2 * Z + 1] = q[Z - 1];
      J ? (H[0] = H[2 * G - 2], H[1] = H[2 * G - 1], H[2 * G + 2] = H[4], H[2 * G + 3] = H[5]) : (H[0] = 0, H[1] = 0, H[2 * G + 2] = 0, H[2 * G + 3] = 0);
    }
    _set_show_single(H, V) {
      const q = V.length / 2 - 2;
      let G = isFinite(V[2] + V[3]);
      for (let Z = 1; Z < q; Z++) {
        const te = isFinite(V[2 * Z + 2] + V[2 * Z + 3]);
        H[Z] = G && te ? 1 : 0, G = te;
      }
      q > 2 && V[0] == V[2 * q - 2] && V[1] == V[2 * q - 1] ? (H[0] = H[q - 1], H[q] = H[1]) : (H[0] = 0, H[q] = 0);
    }
    _set_visuals() {
      const H = this._get_visuals();
      this._line_color.set_from_color(H.line_color, H.line_alpha), this._linewidth.set_from_prop(H.line_width), this._line_cap.set_from_line_cap(H.line_cap), this._line_join.set_from_line_join(H.line_join);
      const { line_dash: V } = H;
      if (this._is_dashed = !(V.is_Scalar() && V.get(0).length == 0), this._is_dashed) {
        this._dash_offset == null && (this._dash_offset = new Float32Buffer(this.regl_wrapper)), this._dash_offset.set_from_prop(H.line_dash_offset);
        const q = V.length;
        this._dash_tex_info == null && (this._dash_tex_info = new Float32Buffer(this.regl_wrapper, 4));
        const G = this._dash_tex_info.get_sized_array(4 * q);
        this._dash_scale == null && (this._dash_scale = new Float32Buffer(this.regl_wrapper));
        const J = this._dash_scale.get_sized_array(q);
        for (let Z = 0; Z < q; Z++) {
          const te = resolve_line_dash(V.get(Z));
          if (te.length > 0) {
            const [ne, ee, ae] = this.regl_wrapper.get_dash(te);
            this._dash_tex.push(ee);
            for (let oe = 0; oe < 4; oe++)
              G[4 * Z + oe] = ne[oe];
            J[Z] = ae;
          } else
            this._dash_tex.push(null), G.fill(0, 4 * Z, 4 * (Z + 1)), J[Z] = 0;
        }
        this._dash_tex_info.update(), this._dash_scale.update();
      }
    }
  }
  mt(BaseLineGL, "__name__", "BaseLineGL");
  class CircleGL extends SXSYGlyphGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    get marker_type() {
      return "circle";
    }
    // TODO: should be 'radius'
    get size() {
      return this._widths;
    }
    _set_data() {
      super._set_data(), this.size.set_from_array(mul(this.glyph.sradius, 2));
    }
    _set_once() {
      super._set_once(), this._heights.set_from_scalar(0), this._angles.set_from_scalar(0);
    }
  }
  mt(CircleGL, "__name__", "CircleGL");
  const circle$2 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    CircleGL
  }, Symbol.toStringTag, { value: "Module" }));
  class HexTileGL extends SXSYGlyphGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    get marker_type() {
      return "hex_tile";
    }
    _set_data() {
      super._set_data(), this.glyph.model.orientation == "pointytop" ? (this._angles.set_from_scalar(0.5 * Math.PI), this._widths.set_from_scalar(this.glyph.svy[0] * 2), this._heights.set_from_scalar(this.glyph.svx[4] * 4 / Math.sqrt(3))) : (this._angles.set_from_scalar(0), this._widths.set_from_scalar(this.glyph.svx[0] * 2), this._heights.set_from_scalar(this.glyph.svy[4] * 4 / Math.sqrt(3)));
    }
    _set_once() {
      super._set_once(), this._auxs.set_from_scalar(0);
    }
  }
  mt(HexTileGL, "__name__", "HexTileGL");
  const hex_tile = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    HexTileGL
  }, Symbol.toStringTag, { value: "Module" }));
  class ImageGL extends BaseGLGlyph {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      // data properties
      mt(this, "_tex", []);
      mt(this, "_bounds", []);
      // image_changed is separate from data_changed as it can occur through changed colormapping.
      mt(this, "_image_changed", !1);
      this.glyph = V;
    }
    draw(H, V, q) {
      const G = V.glglyph;
      (G.data_changed || G.data_mapped) && G._set_data(), (G._image_changed || G.data_changed) && G._set_image(), G.data_changed = !1, G.data_mapped = !1, G._image_changed = !1;
      const { global_alpha: Z } = this.glyph.visuals.image;
      for (const te of H) {
        if (G._tex[te] == null || G._bounds[te] == null)
          continue;
        const ne = {
          scissor: this.regl_wrapper.scissor,
          viewport: this.regl_wrapper.viewport,
          canvas_size: [q.width, q.height],
          bounds: G._bounds[te],
          tex: G._tex[te],
          global_alpha: Z.get(te)
        };
        this.regl_wrapper.image()(ne);
      }
    }
    set_image_changed() {
      this._image_changed = !0;
    }
    _set_data() {
      const { image: H } = this.glyph, V = H.length;
      this._bounds.length != V && (this._bounds = Array(V).fill(null));
      for (let q = 0; q < V; q++) {
        const { sx: G, sy: J, sdw: Z, sdh: te, xy_anchor: ne, xy_scale: ee, xy_sign: ae } = this.glyph, oe = G[q], se = J[q], ce = Z[q], le = te[q];
        if (!isFinite(oe + se + ce + le)) {
          this._bounds[q] = null;
          continue;
        }
        this._bounds[q] == null && (this._bounds[q] = new Float32Buffer(this.regl_wrapper));
        const ue = this._bounds[q].get_sized_array(4);
        ue[0] = G[q] + Z[q] * (0.5 * (1 - ee.x) - ne.x) * ae.x, ue[1] = J[q] + te[q] * (0.5 * (1 - ee.y) - ne.y) * ae.y, ue[2] = ue[0] + Z[q] * ee.x * ae.x, ue[3] = ue[1] + te[q] * ee.y * ae.y, this._bounds[q].update();
      }
    }
    _set_image() {
      const { image: H, image_data: V } = this.glyph, q = H.length;
      assert$2(V != null), this._tex.length != q && (this._tex = Array(q).fill(null));
      for (let G = 0; G < q; G++) {
        const J = V[G];
        if (J == null) {
          this._tex[G] = null;
          continue;
        }
        const Z = {
          width: J.width,
          height: J.height,
          data: J,
          format: "rgba",
          type: "uint8"
        };
        this._tex[G] == null ? this._tex[G] = this.regl_wrapper.texture(Z) : this._tex[G](Z);
      }
    }
  }
  mt(ImageGL, "__name__", "ImageGL");
  const image$2 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    ImageGL
  }, Symbol.toStringTag, { value: "Module" }));
  class SingleLineGL extends BaseLineGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    _draw_impl(H, V, q) {
      this.visuals_changed && (this._set_visuals(), this.visuals_changed = !1);
      const G = q.data_changed || q.data_mapped;
      G && q._set_data(q.data_changed), (G && q._is_dashed || this._is_dashed) && q._set_length(), G && (q.data_changed = !1, q.data_mapped = !1);
      const J = this._get_show_buffer(H, q), te = q._points.length / 2 - 2 - 1;
      this._draw_single(q, V, 0, 0, te, null, J);
    }
    _set_data(H) {
      const V = this._set_data_points();
      if (H) {
        const q = V.length / 2 - 2;
        this._show == null && (this._show = new Uint8Buffer(this.regl_wrapper));
        const G = this._show.get_sized_array(q + 1);
        this._set_show_single(G, V), this._show.update();
      }
    }
    _set_length() {
      const H = this._points.get_array(), V = this._show.get_array(), q = H.length / 2 - 2;
      this._length_so_far == null && (this._length_so_far = new Float32Buffer(this.regl_wrapper));
      const G = this._length_so_far.get_sized_array(q - 1);
      this._set_length_single(G, H, V), this._length_so_far.update();
    }
  }
  mt(SingleLineGL, "__name__", "SingleLineGL");
  class LineGL extends SingleLineGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    draw(H, V, q) {
      this._draw_impl(H, q, V.glglyph);
    }
    _get_show_buffer(H, V) {
      const q = V._show;
      let G = q;
      if (H.length != q.length - 1) {
        const J = this.glyph.parent.nonselection_glyph == this.glyph, Z = q.length, te = q.get_sized_array(Z);
        this._show == null && (this._show = new Uint8Buffer(this.regl_wrapper));
        const ne = this._show.get_sized_array(Z);
        ne.fill(0);
        let ee = H[0];
        J && ee > 0 && (ne[ee] = te[ee]);
        for (let ae = 1; ae < H.length; ae++) {
          const oe = H[ae];
          oe == ee + 1 ? ne[oe] = te[oe] : J && (ne[ee + 1] = te[ee + 1], ne[oe] = te[oe]), ee = oe;
        }
        J && ee != Z - 2 && (ne[ee + 1] = te[ee + 1]), this._show.update(), G = this._show;
      }
      return G;
    }
    _get_visuals() {
      return this.glyph.visuals.line;
    }
    _set_data_points() {
      const H = this.glyph.sx, V = this.glyph.sy, q = H.length;
      this._points == null && (this._points = new Float32Buffer(this.regl_wrapper));
      const G = this._points.get_sized_array((q + 2) * 2);
      return this._set_points_single(G, H, V), this._points.update(), G;
    }
  }
  mt(LineGL, "__name__", "LineGL");
  const line_gl = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    LineGL
  }, Symbol.toStringTag, { value: "Module" })), { abs: abs$3 } = Math;
  class LRTBGL extends SingleMarkerGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    get marker_type() {
      return this._border_radius_nonzero ? "round_rect" : "rect";
    }
    _set_data() {
      const H = this.nvertices, V = this._centers.get_sized_array(H * 2), q = this._widths.get_sized_array(H), G = this._heights.get_sized_array(H), { sleft: J, sright: Z, stop: te, sbottom: ne } = this.glyph, { missing_point: ee } = SingleMarkerGL;
      for (let ae = 0; ae < H; ae++) {
        const oe = J[ae], se = Z[ae], ce = te[ae], le = ne[ae];
        isFinite(oe + se + ce + le) ? (V[2 * ae] = (oe + se) / 2, V[2 * ae + 1] = (ce + le) / 2, q[ae] = abs$3(se - oe), G[ae] = abs$3(ce - le)) : (V[2 * ae] = ee, V[2 * ae + 1] = ee, q[ae] = ee, G[ae] = ee);
      }
      if (this._centers.update(), this._widths.update(), this._heights.update(), this._angles.set_from_scalar(0), this.glyph.border_radius != null) {
        const { top_left: ae, top_right: oe, bottom_right: se, bottom_left: ce } = this.glyph.border_radius;
        this._border_radius = [ae, oe, se, ce], this._border_radius_nonzero = Math.max(...this._border_radius) > 0;
      } else
        this._border_radius = [0, 0, 0, 0], this._border_radius_nonzero = !1;
    }
    _set_once() {
      super._set_once(), this._auxs.set_from_scalar(0);
    }
  }
  mt(LRTBGL, "__name__", "LRTBGL");
  const lrtb = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    LRTBGL
  }, Symbol.toStringTag, { value: "Module" }));
  class MultiLineGL extends BaseLineGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    draw(H, V, q) {
      this.visuals_changed && (this._set_visuals(), this.visuals_changed = !1);
      const G = V.glglyph, J = G.data_changed || G.data_mapped;
      J && G._set_data(G.data_changed), (J && G._is_dashed || this._is_dashed) && G._set_length(), J && (G.data_changed = !1, G.data_mapped = !1);
      const { data_size: Z } = this.glyph;
      let te = null, ne = null;
      Z > 1 && ([te, ne] = this.regl_wrapper.framebuffer_and_texture);
      let ee = 0, ae = -1;
      for (const oe of H) {
        for (let le = ae + 1; le < oe; le++) {
          const ue = V.sxs.get(le).length;
          ee += (ue + 2) * 2;
        }
        const se = V.sxs.get(oe).length, ce = se - 1;
        if (te != null && this.regl_wrapper.clear_framebuffer(te), this._draw_single(G, q, oe, ee, ce, te), te != null) {
          const le = {
            scissor: this.regl_wrapper.scissor,
            viewport: this.regl_wrapper.viewport,
            framebuffer_tex: ne
          };
          this.regl_wrapper.accumulate()(le);
        }
        ee += (se + 2) * 2, ae = oe;
      }
    }
    _get_visuals() {
      return this.glyph.visuals.line;
    }
    _set_data(H) {
      const V = this.glyph.data_size, q = this.glyph.sxs.data.length;
      this._points == null && (this._points = new Float32Buffer(this.regl_wrapper));
      const G = this._points.get_sized_array((q + 2 * V) * 2);
      let J = 0;
      for (let Z = 0; Z < V; Z++) {
        const te = this.glyph.sxs.get(Z), ne = this.glyph.sys.get(Z), ee = te.length, ae = G.subarray(J, J + (ee + 2) * 2);
        this._set_points_single(ae, te, ne), J += (ee + 2) * 2;
      }
      if (this._points.update(), H) {
        this._show == null && (this._show = new Uint8Buffer(this.regl_wrapper));
        const Z = this._show.get_sized_array(q + V);
        let te = 0, ne = 0;
        for (let ee = 0; ee < V; ee++) {
          const oe = this.glyph.sxs.get(ee).length, se = G.subarray(te, te + (oe + 2) * 2), ce = Z.subarray(ne, ne + oe + 1);
          this._set_show_single(ce, se), te += (oe + 2) * 2, ne += oe + 1;
        }
        this._show.update();
      }
    }
    _set_length() {
      const H = this.glyph.data_size, V = this.glyph.sxs.data.length, q = this._points.get_array(), G = this._show.get_array();
      this._length_so_far == null && (this._length_so_far = new Float32Buffer(this.regl_wrapper));
      const J = this._length_so_far.get_sized_array(V - H);
      let Z = 0, te = 0, ne = 0;
      for (let ee = 0; ee < H; ee++) {
        const oe = this.glyph.sxs.get(ee).length, se = oe - 1, ce = q.subarray(Z, Z + (oe + 2) * 2), le = G.subarray(te, te + oe + 1), ue = J.subarray(ne, ne + se);
        this._set_length_single(ue, ce, le), Z += (oe + 2) * 2, te += oe + 1, ne += se;
      }
      this._length_so_far.update();
    }
  }
  mt(MultiLineGL, "__name__", "MultiLineGL");
  const multi_line = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    MultiLineGL
  }, Symbol.toStringTag, { value: "Module" }));
  class MultiMarkerGL extends BaseMarkerGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      // data properties, either all or none are set.
      mt(this, "_marker_types");
      mt(this, "_unique_marker_types");
      this.glyph = V;
    }
    draw(H, V, q) {
      const G = V.glglyph;
      (G.data_changed || G.data_mapped) && (G.set_data(), G.data_changed = !1, G.data_mapped = !1), this.visuals_changed && (this._set_visuals(), this.visuals_changed = !1);
      const J = G.nvertices, Z = G._unique_marker_types.length;
      for (const te of G._unique_marker_types) {
        if (te == null)
          continue;
        let ne = J;
        const ee = this._show.length, ae = this._show.get_sized_array(J);
        if (Z > 1 || H.length < J) {
          this._show_all = !1, ae.fill(0), ne = 0;
          for (const oe of H)
            (Z == 1 || G._marker_types.get(oe) == te) && (ae[oe] = 255, ne++);
        } else (!this._show_all || ee != J) && (this._show_all = !0, ae.fill(255));
        this._show.update(), ne != 0 && this._draw_one_marker_type(te, q, G);
      }
    }
    _get_visuals() {
      return this.glyph.visuals;
    }
    _set_data() {
      const H = this.nvertices, V = this._centers.get_sized_array(2 * H);
      interleave(this.glyph.sx, this.glyph.sy, H, BaseMarkerGL.missing_point, V), this._centers.update(), this._widths.set_from_prop(this.glyph.size), this._angles.set_from_prop(this.glyph.angle), this._marker_types = this.glyph.marker, this._unique_marker_types = [...new Set(this._marker_types)];
    }
    _set_once() {
      super._set_once(), this._heights.set_from_scalar(0), this._auxs.set_from_scalar(0);
    }
  }
  mt(MultiMarkerGL, "__name__", "MultiMarkerGL");
  const multi_marker = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    MultiMarkerGL
  }, Symbol.toStringTag, { value: "Module" }));
  class RectGL extends SXSYGlyphGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    get marker_type() {
      return this._border_radius_nonzero ? "round_rect" : "rect";
    }
    _set_data() {
      super._set_data(), this._widths.set_from_array(this.glyph.swidth), this._heights.set_from_array(this.glyph.sheight), this._angles.set_from_prop(this.glyph.angle);
      const { top_left: H, top_right: V, bottom_right: q, bottom_left: G } = this.glyph.border_radius;
      this._border_radius = [H, V, q, G], this._border_radius_nonzero = Math.max(...this._border_radius) > 0;
    }
    _set_once() {
      super._set_once(), this._auxs.set_from_scalar(0);
    }
  }
  mt(RectGL, "__name__", "RectGL");
  const rect$2 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    RectGL
  }, Symbol.toStringTag, { value: "Module" }));
  class StepGL extends SingleLineGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    draw(H, V, q) {
      this._draw_impl(H, q, V.glglyph);
    }
    _get_show_buffer(H, V) {
      return V._show;
    }
    _get_visuals() {
      return this.glyph.visuals.line;
    }
    _set_data_points() {
      const H = this.glyph.sx, V = this.glyph.sy, q = this.glyph.model.mode;
      let G = H.length;
      const J = G > 2 && H[0] == H[G - 1] && V[0] == V[G - 1] && isFinite(H[0]) && isFinite(V[0]), Z = q == "center" ? 2 * G : 2 * G - 1;
      this._points == null && (this._points = new Float32Buffer(this.regl_wrapper));
      const te = this._points.get_sized_array((Z + 2) * 2);
      let ne = isFinite(H[0] + V[0]), ee = 2;
      q == "center" && (te[ee++] = ne ? H[0] : NaN, te[ee++] = V[0]);
      for (let ae = 0; ae < G - 1; ae++) {
        const oe = isFinite(H[ae + 1] + V[ae + 1]);
        switch (q) {
          case "before":
            te[ee++] = ne ? H[ae] : NaN, te[ee++] = V[ae], ae < G - 1 && (te[ee++] = ne && oe ? H[ae] : NaN, te[ee++] = V[ae + 1]);
            break;
          case "after":
            te[ee++] = ne ? H[ae] : NaN, te[ee++] = V[ae], ae < G - 1 && (te[ee++] = ne && oe ? H[ae + 1] : NaN, te[ee++] = V[ae]);
            break;
          case "center":
            if (ne && oe) {
              const se = (H[ae] + H[ae + 1]) / 2;
              te[ee++] = se, te[ee++] = V[ae], te[ee++] = se, te[ee++] = V[ae + 1];
            } else
              te[ee++] = ne ? H[ae] : NaN, te[ee++] = V[ae], te[ee++] = oe ? H[ae + 1] : NaN, te[ee++] = V[ae + 1];
            break;
          default:
            unreachable();
        }
        ne = oe;
      }
      return te[ee++] = ne ? H[G - 1] : NaN, te[ee++] = ne ? V[G - 1] : NaN, assert$2(ee == Z * 2 + 2), G = Z, J ? (te[0] = te[2 * G - 2], te[1] = te[2 * G - 1], te[2 * G + 2] = te[4], te[2 * G + 3] = te[5]) : (te[0] = 0, te[1] = 0, te[2 * G + 2] = 0, te[2 * G + 3] = 0), this._points.update(), te;
    }
  }
  mt(StepGL, "__name__", "StepGL");
  const step = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    StepGL
  }, Symbol.toStringTag, { value: "Module" }));
  class WedgeGL extends SXSYGlyphGL {
    constructor(H, V) {
      super(H, V);
      mt(this, "glyph");
      this.glyph = V;
    }
    get marker_type() {
      return "wedge";
    }
    get radius() {
      return this._widths;
    }
    get start_angle() {
      return this._angles;
    }
    get end_angle() {
      return this._auxs;
    }
    _set_data() {
      super._set_data(), this.radius.set_from_array(this.glyph.sradius), this.glyph.model.direction == "anticlock" ? (this.start_angle.set_from_prop(this.glyph.start_angle), this.end_angle.set_from_prop(this.glyph.end_angle)) : (this.start_angle.set_from_prop(this.glyph.end_angle), this.end_angle.set_from_prop(this.glyph.start_angle));
    }
    _set_once() {
      super._set_once(), this._heights.set_from_scalar(0);
    }
  }
  mt(WedgeGL, "__name__", "WedgeGL");
  const wedge = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    WedgeGL
  }, Symbol.toStringTag, { value: "Module" })), index$3 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    AnnularWedgeGL,
    AnnulusGL,
    BaseLineGL,
    BaseMarkerGL,
    CircleGL,
    HexTileGL,
    ImageGL,
    LRTBGL,
    LineGL,
    MultiLineGL,
    MultiMarkerGL,
    RectGL,
    SingleLineGL,
    SingleMarkerGL,
    StepGL,
    WedgeGL,
    get_regl
  }, Symbol.toStringTag, { value: "Module" }));
  var np;
  (function(O) {
    O.pi = Math.PI;
    function _(Se, we, Ee = 1) {
      const xe = range$9(Se, we, Ee);
      return ndarray(xe, { shape: [xe.length], dtype: "float64" });
    }
    O.arange = _;
    function H(Se, we, Ee = 100) {
      const xe = linspace(Se, we, Ee);
      return ndarray(xe, { shape: [xe.length], dtype: "float64" });
    }
    O.linspace = H;
    function V(Se) {
      return G(Se) / Se.length;
    }
    O.mean = V;
    function q(Se) {
      const we = V(Se);
      return Math.sqrt(G(map$4(Se, (Ee) => (Ee - we) ** 2)) / Se.length);
    }
    O.std = q;
    function G(Se) {
      return sum$3(Se);
    }
    O.sum = G;
    function J(Se) {
      const we = Se.length - 1, Ee = new Float64Array(we);
      for (let xe = 0; xe < we; xe++)
        Ee[xe] = Se[xe + 1] - Se[xe];
      return ndarray(Ee.buffer, { shape: [we], dtype: "float64" });
    }
    O.diff = J;
    function Z(Se) {
      return isNumber$3(Se) ? Math.sin(Se) : is_Floating(Se) ? Math.sin(Se[float]()) : map$4(Se, (we) => Math.sin(we));
    }
    O.sin = Z;
    function te(Se) {
      return isNumber$3(Se) ? Math.cos(Se) : is_Floating(Se) ? Math.cos(Se[float]()) : map$4(Se, (we) => Math.cos(we));
    }
    O.cos = te;
    function ne(Se) {
      return isNumber$3(Se) ? Math.exp(Se) : is_Floating(Se) ? Math.exp(Se[float]()) : map$4(Se, (we) => Math.exp(we));
    }
    O.exp = ne;
    function ee(Se) {
      return isNumber$3(Se) ? Math.sqrt(Se) : is_Floating(Se) ? Math.sqrt(Se[float]()) : map$4(Se, (we) => Math.sqrt(we));
    }
    O.sqrt = ee;
    function ae(Se) {
      return isNumber$3(Se) ? factorial(Se) : is_Floating(Se) ? factorial(Se[float]()) : map$4(Se, factorial);
    }
    O.factorial = ae;
    function oe(Se) {
      const we = hermite(Se);
      return (Ee) => isNumber$3(Ee) ? eval_poly(we, Ee) : is_Floating(Ee) ? eval_poly(we, Ee[float]()) : map$4(Ee, (xe) => eval_poly(we, xe));
    }
    O.hermite = oe;
    function se(Se) {
      return isNumber$3(Se) ? +Se : is_Floating(Se) ? +Se[float]() : map$4(Se, (we) => +we);
    }
    O.pos = se;
    function ce(Se) {
      return isNumber$3(Se) ? -Se : is_Floating(Se) ? -Se[float]() : map$4(Se, (we) => -we);
    }
    O.neg = ce;
    function le(Se, we) {
      const Ee = is_Floating(Se) ? Se[float]() : Se, xe = is_Floating(we) ? we[float]() : we, Le = isNumber$3(Ee), Qe = isNumber$3(xe);
      if (Le && Qe)
        return Ee + xe;
      if (Le && !Qe)
        return map$4(xe, (ke) => Ee + ke);
      if (!Le && Qe)
        return map$4(Ee, (ke) => ke + xe);
      if (is_NDArray(Ee) && is_NDArray(xe)) {
        if (is_equal(Ee.shape, xe.shape) && Ee.dtype == xe.dtype)
          return map$4(Ee, (ke, Me) => ke + xe[Me]);
        throw new Error("shape or dtype mismatch");
      } else
        throw new Error("not implemented");
    }
    O.add = le;
    function ue(Se, we) {
      const Ee = is_Floating(Se) ? Se[float]() : Se, xe = is_Floating(we) ? we[float]() : we, Le = isNumber$3(Ee), Qe = isNumber$3(xe);
      if (Le && Qe)
        return Ee - xe;
      if (Le && !Qe)
        return map$4(xe, (ke) => Ee - ke);
      if (!Le && Qe)
        return map$4(Ee, (ke) => ke - xe);
      if (is_NDArray(Ee) && is_NDArray(xe)) {
        if (is_equal(Ee.shape, xe.shape) && Ee.dtype == xe.dtype)
          return map$4(Ee, (ke, Me) => ke - xe[Me]);
        throw new Error("shape or dtype mismatch");
      } else
        throw new Error("not implemented");
    }
    O.sub = ue;
    function de(Se, we) {
      const Ee = is_Floating(Se) ? Se[float]() : Se, xe = is_Floating(we) ? we[float]() : we, Le = isNumber$3(Ee), Qe = isNumber$3(xe);
      if (Le && Qe)
        return Ee * xe;
      if (Le && !Qe)
        return map$4(xe, (ke) => Ee * ke);
      if (!Le && Qe)
        return map$4(Ee, (ke) => ke * xe);
      if (is_NDArray(Ee) && is_NDArray(xe)) {
        if (is_equal(Ee.shape, xe.shape) && Ee.dtype == xe.dtype)
          return map$4(Ee, (ke, Me) => ke * xe[Me]);
        throw new Error("shape or dtype mismatch");
      } else
        throw new Error("not implemented");
    }
    O.mul = de;
    function fe(Se, we) {
      const Ee = is_Floating(Se) ? Se[float]() : Se, xe = is_Floating(we) ? we[float]() : we, Le = isNumber$3(Ee), Qe = isNumber$3(xe);
      if (Le && Qe)
        return Ee / xe;
      if (Le && !Qe)
        return map$4(xe, (ke) => Ee / ke);
      if (!Le && Qe)
        return map$4(Ee, (ke) => ke / xe);
      if (is_NDArray(Ee) && is_NDArray(xe)) {
        if (is_equal(Ee.shape, xe.shape) && Ee.dtype == xe.dtype)
          return map$4(Ee, (ke, Me) => ke / xe[Me]);
        throw new Error("shape or dtype mismatch");
      } else
        throw new Error("not implemented");
    }
    O.div = fe;
    function he(Se, we) {
      const Ee = is_Floating(Se) ? Se[float]() : Se, xe = is_Floating(we) ? we[float]() : we, Le = isNumber$3(Ee), Qe = isNumber$3(xe);
      if (Le && Qe)
        return Ee ** xe;
      if (Le && !Qe)
        return map$4(xe, (ke) => Ee ** ke);
      if (!Le && Qe)
        return map$4(Ee, (ke) => ke ** xe);
      if (is_NDArray(Ee) && is_NDArray(xe)) {
        if (is_equal(Ee.shape, xe.shape) && Ee.dtype == xe.dtype)
          return map$4(Ee, (ke, Me) => ke ** xe[Me]);
        throw new Error("shape or dtype mismatch");
      } else
        throw new Error("not implemented");
    }
    O.pow = he;
    function me(Se, we, Ee) {
      const xe = is_Floating(Se) ? Se[float]() : Se, Le = is_Floating(we) ? we[float]() : we, Qe = isNumber$3(xe), ke = isNumber$3(Le), Me = (Re) => Re ? 1 : 0;
      if (Qe && ke)
        return Me(xe >= Le);
      if (Qe && !ke)
        return map$4(Le, (Re) => Me(Ee(xe, Re)));
      if (!Qe && ke)
        return map$4(xe, (Re) => Me(Ee(Re, Le)));
      if (is_NDArray(xe) && is_NDArray(Le)) {
        if (is_equal(xe.shape, Le.shape) && xe.dtype == Le.dtype)
          return map$4(xe, (Re, De) => Me(Ee(Re, Le[De])));
        throw new Error("shape or dtype mismatch");
      } else
        throw new Error("not implemented");
    }
    function pe(Se, we) {
      return me(Se, we, (Ee, xe) => Ee >= xe);
    }
    O.ge = pe;
    function ye(Se, we) {
      return me(Se, we, (Ee, xe) => Ee <= xe);
    }
    O.le = ye;
    function ge(Se, we) {
      return me(Se, we, (Ee, xe) => Ee > xe);
    }
    O.gt = ge;
    function ve(Se, we) {
      return me(Se, we, (Ee, xe) => Ee < xe);
    }
    O.lt = ve;
    function be(Se, we, Ee) {
      const xe = is_Floating(we) ? we[float]() : we, Le = is_Floating(Ee) ? Ee[float]() : Ee, Qe = isNumber$3(xe), ke = isNumber$3(Le), Me = (() => {
        if (Qe && ke)
          return (Re) => Re != 0 ? xe : Le;
        if (Qe && !ke)
          return (Re, De) => Re != 0 ? xe : Le[De];
        if (!Qe && ke)
          return (Re, De) => Re != 0 ? xe[De] : Le;
        if (is_NDArray(xe) && is_NDArray(Le)) {
          if (is_equal(xe.shape, Le.shape) && xe.dtype == Le.dtype)
            return (Re, De) => Re != 0 ? xe[De] : Le[De];
          throw new Error("shape or dtype mismatch");
        } else
          throw new Error("not implemented");
      })();
      return map$4(Se, Me);
    }
    O.where = be;
    function Ce(Se, we) {
      const { density: Ee, bins: xe } = we, Le = ndarray(xe, { dtype: "float64", shape: [xe.length] }), Qe = ndarray(bin_counts(Se, Le), { dtype: "float64", shape: [Le.length - 1] });
      return Ee ? [fe(fe(Qe, J(Le)), G(Qe)), Le] : [Qe, Le];
    }
    O.histogram = Ce, function(Se) {
      class we {
        constructor(Le) {
          mt(this, "_random");
          this._random = new Random(Le ?? Date.now());
        }
        normal(Le, Qe, ke) {
          const Me = this._random.normals(Le, Qe, ke);
          return ndarray(Me.buffer, { shape: [ke], dtype: "float64" });
        }
      }
      mt(we, "__name__", "RandomGenerator"), Se.RandomGenerator = we;
      function Ee(xe) {
        return new we(xe);
      }
      Se.default_rng = Ee;
    }(O.random || (O.random = {}));
  })(np || (np = {}));
  function extend$3(O, ..._) {
    for (const H of _)
      for (const V of Object.getOwnPropertyNames(H.prototype))
        V != "constructor" && Object.defineProperty(O.prototype, V, Object.getOwnPropertyDescriptor(H.prototype, V) ?? /* @__PURE__ */ Object.create(null));
  }
  class GlyphAPI {
    annular_wedge(..._) {
      return this._glyph(AnnularWedge, ["x", "y", "inner_radius", "outer_radius", "start_angle", "end_angle"], _);
    }
    annulus(..._) {
      return this._glyph(Annulus, ["x", "y", "inner_radius", "outer_radius"], _);
    }
    arc(..._) {
      return this._glyph(Arc, ["x", "y", "radius", "start_angle", "end_angle"], _);
    }
    bezier(..._) {
      return this._glyph(Bezier, ["x0", "y0", "x1", "y1", "cx0", "cy0", "cx1", "cy1"], _);
    }
    block(..._) {
      return this._glyph(Block$1, ["x", "y", "width", "height"], _);
    }
    circle(..._) {
      return this._glyph(Circle, ["x", "y", "radius"], _);
    }
    ellipse(..._) {
      return this._glyph(Ellipse, ["x", "y", "width", "height"], _);
    }
    harea(..._) {
      return this._glyph(HArea, ["x1", "x2", "y"], _);
    }
    harea_step(..._) {
      return this._glyph(HAreaStep, ["x1", "x2", "y", "step_mode"], _);
    }
    hbar(..._) {
      return this._glyph(HBar, ["y", "height", "right", "left"], _);
    }
    hspan(..._) {
      return this._glyph(HSpan, ["y"], _);
    }
    hstrip(..._) {
      return this._glyph(HStrip, ["y0", "y1"], _);
    }
    hex_tile(..._) {
      return this._glyph(HexTile, ["q", "r"], _);
    }
    image(..._) {
      return this._glyph(Image$1, ["color_mapper", "image", "x", "y", "dw", "dh"], _);
    }
    image_stack(..._) {
      return this._glyph(ImageStack, ["color_mapper", "image", "x", "y", "dw", "dh"], _);
    }
    image_rgba(..._) {
      return this._glyph(ImageRGBA, ["image", "x", "y", "dw", "dh"], _);
    }
    image_url(..._) {
      return this._glyph(ImageURL, ["url", "x", "y", "w", "h"], _);
    }
    line(..._) {
      return this._glyph(Line$2, ["x", "y"], _);
    }
    mathml(..._) {
      return this._glyph(MathMLGlyph, ["x", "y", "text"], _);
    }
    multi_line(..._) {
      return this._glyph(MultiLine, ["xs", "ys"], _);
    }
    multi_polygons(..._) {
      return this._glyph(MultiPolygons, ["xs", "ys"], _);
    }
    patch(..._) {
      return this._glyph(Patch, ["x", "y"], _);
    }
    patches(..._) {
      return this._glyph(Patches, ["xs", "ys"], _);
    }
    quad(..._) {
      return this._glyph(Quad$1, ["left", "right", "bottom", "top"], _);
    }
    quadratic(..._) {
      return this._glyph(Quadratic, ["x0", "y0", "x1", "y1", "cx", "cy"], _);
    }
    ray(..._) {
      return this._glyph(Ray, ["x", "y", "length"], _);
    }
    rect(..._) {
      return this._glyph(Rect, ["x", "y", "width", "height"], _);
    }
    segment(..._) {
      return this._glyph(Segment, ["x0", "y0", "x1", "y1"], _);
    }
    spline(..._) {
      return this._glyph(Spline, ["x", "y"], _);
    }
    step(..._) {
      return this._glyph(Step$1, ["x", "y", "mode"], _);
    }
    tex(..._) {
      return this._glyph(TeXGlyph, ["x", "y", "text"], _);
    }
    text(..._) {
      return this._glyph(Text$1, ["x", "y", "text"], _);
    }
    varea(..._) {
      return this._glyph(VArea, ["x", "y1", "y2"], _);
    }
    varea_step(..._) {
      return this._glyph(VAreaStep, ["x", "y1", "y2", "step_mode"], _);
    }
    vbar(..._) {
      return this._glyph(VBar, ["x", "width", "top", "bottom"], _);
    }
    vspan(..._) {
      return this._glyph(VSpan, ["x"], _);
    }
    vstrip(..._) {
      return this._glyph(VStrip, ["x0", "x1"], _);
    }
    wedge(..._) {
      return this._glyph(Wedge, ["x", "y", "radius", "start_angle", "end_angle"], _);
    }
    _scatter(_, H) {
      return this._glyph(Scatter, ["x", "y"], _, H != null ? { marker: H } : void 0);
    }
    scatter(..._) {
      return this._scatter(_);
    }
    /** @deprecated */
    asterisk(..._) {
      return this._scatter(_, "asterisk");
    }
    /** @deprecated */
    circle_cross(..._) {
      return this._scatter(_, "circle_cross");
    }
    /** @deprecated */
    circle_dot(..._) {
      return this._scatter(_, "circle_dot");
    }
    /** @deprecated */
    circle_x(..._) {
      return this._scatter(_, "circle_x");
    }
    /** @deprecated */
    circle_y(..._) {
      return this._scatter(_, "circle_y");
    }
    /** @deprecated */
    cross(..._) {
      return this._scatter(_, "cross");
    }
    /** @deprecated */
    dash(..._) {
      return this._scatter(_, "dash");
    }
    /** @deprecated */
    diamond(..._) {
      return this._scatter(_, "diamond");
    }
    /** @deprecated */
    diamond_cross(..._) {
      return this._scatter(_, "diamond_cross");
    }
    /** @deprecated */
    diamond_dot(..._) {
      return this._scatter(_, "diamond_dot");
    }
    /** @deprecated */
    dot(..._) {
      return this._scatter(_, "dot");
    }
    /** @deprecated */
    hex(..._) {
      return this._scatter(_, "hex");
    }
    /** @deprecated */
    hex_dot(..._) {
      return this._scatter(_, "hex_dot");
    }
    /** @deprecated */
    inverted_triangle(..._) {
      return this._scatter(_, "inverted_triangle");
    }
    /** @deprecated */
    plus(..._) {
      return this._scatter(_, "plus");
    }
    /** @deprecated */
    square(..._) {
      return this._scatter(_, "square");
    }
    /** @deprecated */
    square_cross(..._) {
      return this._scatter(_, "square_cross");
    }
    /** @deprecated */
    square_dot(..._) {
      return this._scatter(_, "square_dot");
    }
    /** @deprecated */
    square_pin(..._) {
      return this._scatter(_, "square_pin");
    }
    /** @deprecated */
    square_x(..._) {
      return this._scatter(_, "square_x");
    }
    /** @deprecated */
    star(..._) {
      return this._scatter(_, "star");
    }
    /** @deprecated */
    star_dot(..._) {
      return this._scatter(_, "star_dot");
    }
    /** @deprecated */
    triangle(..._) {
      return this._scatter(_, "triangle");
    }
    /** @deprecated */
    triangle_dot(..._) {
      return this._scatter(_, "triangle_dot");
    }
    /** @deprecated */
    triangle_pin(..._) {
      return this._scatter(_, "triangle_pin");
    }
    /** @deprecated */
    x(..._) {
      return this._scatter(_, "x");
    }
    /** @deprecated */
    y(..._) {
      return this._scatter(_, "y");
    }
  }
  mt(GlyphAPI, "__name__", "GlyphAPI");
  const _default_tools = ["pan", "wheel_zoom", "box_zoom", "save", "reset", "help"], _default_color = "#1f77b4", _default_alpha = 1;
  class ModelProxy {
    constructor(_) {
      mt(this, "models");
      this.models = _;
      const H = /* @__PURE__ */ new Map();
      for (const V of _)
        for (const q of V) {
          const { attr: G } = q;
          H.has(G) || H.set(G, []), H.get(G).push(q);
        }
      for (const [V, q] of H)
        Object.defineProperty(this, V, {
          get() {
            throw new Error("only setting values is supported");
          },
          set(G) {
            for (const J of q)
              J.obj.setv({ [V]: G });
            return this;
          }
        });
    }
    each(_) {
      let H = 0;
      for (const V of this.models)
        _(V, H++);
    }
    *[Symbol.iterator]() {
      yield* this.models;
    }
  }
  mt(ModelProxy, "__name__", "ModelProxy");
  class SubFigure extends GlyphAPI {
    constructor(H, V) {
      super();
      mt(this, "coordinates");
      mt(this, "parent");
      this.coordinates = H, this.parent = V;
    }
    _glyph(H, V, q, G) {
      const { coordinates: J } = this;
      return this.parent._glyph(H, V, q, { coordinates: J, ...G });
    }
  }
  mt(SubFigure, "__name__", "SubFigure");
  const S2 = class S2 extends Figure$1 {
    get xaxes() {
      return [...this.below, ...this.above].filter((_) => _ instanceof Axis);
    }
    get yaxes() {
      return [...this.left, ...this.right].filter((_) => _ instanceof Axis);
    }
    get axes() {
      return [...this.below, ...this.above, ...this.left, ...this.right].filter((_) => _ instanceof Axis);
    }
    get xaxis() {
      return new ModelProxy(this.xaxes);
    }
    get yaxis() {
      return new ModelProxy(this.yaxes);
    }
    get axis() {
      return new ModelProxy(this.axes);
    }
    get xgrids() {
      return this.center.filter((_) => _ instanceof Grid && _.dimension == 0);
    }
    get ygrids() {
      return this.center.filter((_) => _ instanceof Grid && _.dimension == 1);
    }
    get grids() {
      return this.center.filter((_) => _ instanceof Grid);
    }
    get xgrid() {
      return new ModelProxy(this.xgrids);
    }
    get ygrid() {
      return new ModelProxy(this.ygrids);
    }
    get grid() {
      return new ModelProxy(this.grids);
    }
    get legend() {
      const _ = this.panels.filter((H) => H instanceof Legend);
      if (_.length == 0) {
        const H = new Legend();
        return this.add_layout(H), H;
      } else {
        const [H] = _;
        return H;
      }
    }
    constructor(_ = {}) {
      _ = { ..._ };
      const H = _.x_axis_type === void 0 ? "auto" : _.x_axis_type, V = _.y_axis_type === void 0 ? "auto" : _.y_axis_type;
      delete _.x_axis_type, delete _.y_axis_type;
      const q = _.x_minor_ticks ?? "auto", G = _.y_minor_ticks ?? "auto";
      delete _.x_minor_ticks, delete _.y_minor_ticks;
      const J = _.x_axis_location === void 0 ? "below" : _.x_axis_location, Z = _.y_axis_location === void 0 ? "left" : _.y_axis_location;
      delete _.x_axis_location, delete _.y_axis_location;
      const te = _.x_axis_label ?? "", ne = _.y_axis_label ?? "";
      delete _.x_axis_label, delete _.y_axis_label;
      const ee = S2._get_range(_.x_range), ae = S2._get_range(_.y_range);
      delete _.x_range, delete _.y_range;
      const oe = _.x_scale ?? S2._get_scale(ee, H), se = _.y_scale ?? S2._get_scale(ae, V);
      delete _.x_scale, delete _.y_scale;
      const { active_drag: ce, active_inspect: le, active_scroll: ue, active_tap: de, active_multi: fe } = _;
      delete _.active_drag, delete _.active_inspect, delete _.active_scroll, delete _.active_tap, delete _.active_multi;
      const he = (() => {
        const { tools: pe, toolbar: ye } = _;
        if (pe != null) {
          if (ye != null)
            throw new Error("'tools' and 'toolbar' can't be used together");
          return delete _.tools, isString$2(pe) ? pe.split(",").map((ge) => ge.trim()).filter((ge) => ge.length > 0) : pe;
        } else
          return ye != null ? null : _default_tools;
      })();
      super({ ..._, x_range: ee, y_range: ae, x_scale: oe, y_scale: se }), this._process_axis_and_grid(H, J, q, te, ee, 0), this._process_axis_and_grid(V, Z, G, ne, ae, 1);
      const me = /* @__PURE__ */ new Map();
      if (he != null) {
        const pe = he.map((ye) => {
          if (ye instanceof Tool)
            return ye;
          {
            const ge = Tool.from_string(ye);
            return me.set(ye, ge), ge;
          }
        });
        this.add_tools(...pe);
      }
      if (isString$2(ce) && ce != "auto") {
        const pe = me.get(ce);
        pe != null && (this.toolbar.active_drag = pe);
      } else ce !== void 0 && (this.toolbar.active_drag = ce);
      if (isString$2(le) && le != "auto") {
        const pe = me.get(le);
        pe != null && (this.toolbar.active_inspect = pe);
      } else le !== void 0 && (this.toolbar.active_inspect = le);
      if (isString$2(ue) && ue != "auto") {
        const pe = me.get(ue);
        pe != null && (this.toolbar.active_scroll = pe);
      } else ue !== void 0 && (this.toolbar.active_scroll = ue);
      if (isString$2(de) && de != "auto") {
        const pe = me.get(de);
        pe != null && (this.toolbar.active_tap = pe);
      } else de !== void 0 && (this.toolbar.active_tap = de);
      if (isString$2(fe) && fe != "auto") {
        const pe = me.get(fe);
        pe instanceof GestureTool && (this.toolbar.active_multi = pe);
      } else fe !== void 0 && (this.toolbar.active_multi = fe);
    }
    get coordinates() {
      return null;
    }
    subplot(_) {
      const H = new CoordinateMapping(_);
      return new SubFigure(H, this);
    }
    _pop_visuals(_, H, V = "", q = {}, G = {}) {
      const J = function(ue) {
        const de = ue.split("_", 2);
        return de.length == 2 ? de : de.concat([""]);
      }, Z = function(ue) {
        const [de, fe] = J(ue);
        return includes(["line", "fill", "hatch", "text", "global"], de) && fe !== "";
      };
      q = { ...q };
      const te = {}, ne = dict(H), ee = dict(_.prototype._props), ae = dict(q), oe = dict(te), se = dict(G);
      ae.has("text_color") || (q.text_color = "black"), ae.has("hatch_color") || (q.hatch_color = "black"), oe.has("color") || (te.color = _default_color), oe.has("alpha") || (te.alpha = _default_alpha);
      const ce = {}, le = /* @__PURE__ */ new Set();
      for (const ue of keys$5(_.prototype._props))
        if (Z(ue)) {
          const de = J(ue)[1];
          ne.has(V + ue) ? (ce[ue] = H[V + ue], delete H[V + ue]) : !ee.has(de) && ne.has(V + de) ? ce[ue] = H[V + de] : se.has(de) ? ce[ue] = G[de] : ae.has(ue) ? ce[ue] = q[ue] : oe.has(de) && (ce[ue] = te[de]), ee.has(de) || le.add(de);
        }
      for (const ue of le)
        delete H[V + ue];
      return ce;
    }
    _find_uniq_name(_, H) {
      let V = 1;
      for (; ; ) {
        const q = `${H}__${V}`;
        if (_.has(q))
          V += 1;
        else
          return q;
      }
    }
    _fixup_values(_, H, V) {
      const q = /* @__PURE__ */ new Set(), G = dict(_.prototype._props);
      for (const [J, Z] of entries$2(V)) {
        const te = G.get(J);
        if (te != null) {
          if (te.type.prototype instanceof VectorSpec) {
            if (Z != null)
              if (isArray$3(Z) || is_NDArray(Z)) {
                let ne;
                H.has(J) ? H.get(J) !== Z ? (ne = this._find_uniq_name(H, J), H.set(ne, Z)) : ne = J : (ne = J, H.set(ne, Z)), V[J] = { field: ne };
              } else (isNumber$3(Z) || isString$2(Z)) && (V[J] = { value: Z });
            if (te.type.prototype instanceof UnitsSpec) {
              const ne = `${J}_units`, ee = V[ne];
              ee !== void 0 && (V[J] = { ...V[J], units: ee }, q.delete(ne), delete V[ne]);
            }
          }
        } else
          q.add(J);
      }
      return q;
    }
    _glyph(_, H, V, q = {}) {
      let G;
      if (V.length == 0)
        G = {};
      else if (V.length == 1)
        G = { ...V[0] };
      else {
        V.length == H.length ? G = {} : G = { ...V[V.length - 1] };
        for (const [Qe, ke] of enumerate(H))
          G[Qe] = V[ke];
      }
      G = { ...G, ...q };
      const J = (() => {
        const { source: Qe } = G;
        return Qe == null ? new ColumnDataSource() : Qe instanceof ColumnarDataSource ? Qe : new ColumnDataSource({ data: Qe });
      })(), Z = clone$1(J.data);
      delete G.source;
      const { view: te } = G;
      delete G.view;
      const ne = G.legend;
      delete G.legend;
      const ee = G.legend_label;
      delete G.legend_label;
      const ae = G.legend_field;
      delete G.legend_field;
      const oe = G.legend_group;
      if (delete G.legend_group, [ne, ee, ae, oe].filter((Qe) => Qe != null).length > 1)
        throw new Error("only one of legend, legend_label, legend_field, legend_group can be specified");
      const se = G.name;
      delete G.name;
      const ce = G.level;
      delete G.level;
      const le = G.visible;
      delete G.visible;
      const ue = G.x_range_name;
      delete G.x_range_name;
      const de = G.y_range_name;
      delete G.y_range_name;
      const fe = G.coordinates;
      delete G.coordinates;
      const he = this._pop_visuals(_, G), me = this._pop_visuals(_, G, "nonselection_", he, { alpha: 0.1 }), pe = this._pop_visuals(_, G, "selection_", he), ye = this._pop_visuals(_, G, "hover_", he), ge = this._pop_visuals(_, G, "muted_", he, { alpha: 0.2 }), ve = dict(Z);
      this._fixup_values(_, ve, he), this._fixup_values(_, ve, me), this._fixup_values(_, ve, pe), this._fixup_values(_, ve, ye), this._fixup_values(_, ve, ge), this._fixup_values(_, ve, G), J.data = Z;
      const be = (Qe, ke, Me) => new Qe({ ...ke, ...Me }), Ce = be(_, G, he), Se = is_empty(me) ? "auto" : be(_, G, me), we = is_empty(pe) ? "auto" : be(_, G, pe), Ee = is_empty(ye) ? void 0 : be(_, G, ye), xe = is_empty(ge) ? "auto" : be(_, G, ge), Le = new GlyphRenderer({
        data_source: J,
        view: te,
        glyph: Ce,
        nonselection_glyph: Se,
        selection_glyph: we,
        hover_glyph: Ee,
        muted_glyph: xe,
        name: se,
        level: ce,
        visible: le,
        x_range_name: ue,
        y_range_name: de,
        coordinates: fe
      });
      return ee != null && this._handle_legend_label(ee, this.legend, Le), ae != null && this._handle_legend_field(ae, this.legend, Le), oe != null && this._handle_legend_group(oe, this.legend, Le), this.add_renderers(Le), Le;
    }
    static _get_range(_) {
      if (_ == null)
        return new DataRange1d();
      if (_ instanceof Range$2)
        return _;
      if (isArray$3(_))
        if (isArrayOf(_, isString$2)) {
          const H = _;
          return new FactorRange({ factors: H });
        } else {
          const [H, V] = _;
          return new Range1d({ start: H, end: V });
        }
      throw new Error(`unable to determine proper range for: '${_}'`);
    }
    static _get_scale(_, H) {
      if (_ instanceof DataRange1d || _ instanceof Range1d)
        switch (H) {
          case null:
          case "auto":
          case "linear":
          case "datetime":
          case "mercator":
            return new LinearScale();
          case "log":
            return new LogScale();
        }
      if (_ instanceof FactorRange)
        return new CategoricalScale();
      throw new Error(`unable to determine proper scale for: '${_}'`);
    }
    _process_axis_and_grid(_, H, V, q, G, J) {
      const Z = this._get_axis(_, G, J);
      if (Z != null) {
        Z instanceof LogAxis && (J == 0 ? this.x_scale = new LogScale() : this.y_scale = new LogScale()), Z.ticker instanceof ContinuousTicker && (Z.ticker.num_minor_ticks = this._get_num_minor_ticks(Z, V)), Z.axis_label = q, H != null && this.add_layout(Z, H);
        const te = new Grid({ dimension: J, ticker: Z.ticker });
        this.add_layout(te);
      }
    }
    _get_axis(_, H, V) {
      switch (_) {
        case null:
          return null;
        case "linear":
          return new LinearAxis();
        case "log":
          return new LogAxis();
        case "datetime":
          return new DatetimeAxis();
        case "mercator": {
          const q = new MercatorAxis(), G = V == 0 ? "lon" : "lat";
          return q.ticker.dimension = G, q.formatter.dimension = G, q;
        }
        case "auto":
          return H instanceof FactorRange ? new CategoricalAxis() : new LinearAxis();
        // TODO: return DatetimeAxis (Date type)
        default:
          throw new Error("shouldn't have happened");
      }
    }
    _get_num_minor_ticks(_, H) {
      if (isNumber$3(H)) {
        if (H <= 1)
          throw new Error("num_minor_ticks must be > 1");
        return H;
      } else return H == null ? 0 : _ instanceof LogAxis ? 10 : 5;
    }
    _update_legend(_, H) {
      const { legend: V } = this;
      let q = !1;
      for (const G of V.items)
        if (G.label != null && is_equal(G.label, _)) {
          const J = G.label;
          if ("value" in J) {
            G.renderers.push(H), q = !0;
            break;
          }
          if ("field" in J && H.data_source == G.renderers[0].data_source) {
            G.renderers.push(H), q = !0;
            break;
          }
        }
      if (!q) {
        const G = new LegendItem({ label: _, renderers: [H] });
        V.items.push(G);
      }
    }
    _handle_legend_label(_, H, V) {
      const q = { value: _ }, G = this._find_legend_item(q, H);
      if (G != null)
        G.renderers.push(V);
      else {
        const J = new LegendItem({ label: q, renderers: [V] });
        H.items.push(J);
      }
    }
    _handle_legend_field(_, H, V) {
      const q = { field: _ }, G = this._find_legend_item(q, H);
      if (G != null)
        G.renderers.push(V);
      else {
        const J = new LegendItem({ label: q, renderers: [V] });
        H.items.push(J);
      }
    }
    _handle_legend_group(_, H, V) {
      const q = dict(V.data_source.data);
      if (!q.has(_))
        throw new Error(`column to be grouped does not exist in glyph data source: ${_}`);
      const G = q.get(_) ?? [], J = uniq(G).sort();
      for (const Z of J) {
        const te = { value: `${Z}` }, ne = G.indexOf(Z), ee = new LegendItem({ label: te, renderers: [V], index: ne });
        H.items.push(ee);
      }
    }
    _find_legend_item(_, H) {
      const V = new Comparator();
      for (const q of H.items)
        if (V.eq(q.label, _))
          return q;
      return null;
    }
  };
  mt(S2, "__name__", "Figure"), extend$3(S2, GlyphAPI);
  let Figure = S2;
  class ThemedAttrs {
    constructor(_, H) {
      mt(this, "type");
      mt(this, "attrs");
      mt(this, "defaults");
      this.type = _, this.attrs = H, this.defaults = new Map(entries$2(H));
    }
  }
  mt(ThemedAttrs, "__name__", "ThemedAttrs");
  function themed(O, _) {
    return new ThemedAttrs(O, _);
  }
  class Theme {
    constructor(_) {
      mt(this, "attrs");
      this.attrs = _;
    }
    get(_, H) {
      const V = _ instanceof HasProps ? _.constructor : _;
      for (const { type: q, defaults: G } of this.attrs)
        if (V == q || V.prototype instanceof q)
          return G.get(H);
    }
  }
  mt(Theme, "__name__", "Theme");
  new Theme([
    themed(Plot, {
      background_fill_color: "#20262b",
      border_fill_color: "#15191c",
      outline_line_color: "#e0e0e0",
      outline_line_alpha: 0.25
    }),
    themed(Grid, {
      grid_line_color: "#e0e0e0",
      grid_line_alpha: 0.25
    }),
    themed(Axis, {
      major_tick_line_alpha: 0,
      major_tick_line_color: "#e0e0e0",
      minor_tick_line_alpha: 0,
      minor_tick_line_color: "#e0e0e0",
      axis_line_alpha: 0,
      axis_line_color: "#e0e0e0",
      major_label_text_color: "#e0e0e0",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      axis_label_standoff: 10,
      axis_label_text_color: "#e0e0e0",
      axis_label_text_font: "Helvetica",
      axis_label_text_font_size: "1.25em",
      axis_label_text_font_style: "normal"
    }),
    themed(Legend, {
      spacing: 8,
      glyph_width: 15,
      label_standoff: 8,
      label_text_color: "#e0e0e0",
      label_text_font: "Helvetica",
      label_text_font_size: "1.025em",
      border_line_alpha: 0,
      background_fill_alpha: 0.25,
      background_fill_color: "#20262b"
    }),
    themed(BaseColorBar, {
      title_text_color: "#e0e0e0",
      title_text_font: "Helvetica",
      title_text_font_size: "1.025em",
      title_text_font_style: "normal",
      major_label_text_color: "#e0e0e0",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      background_fill_color: "#15191c",
      major_tick_line_alpha: 0,
      bar_line_alpha: 0
    }),
    themed(Title, {
      text_color: "#e0e0e0",
      text_font: "Helvetica",
      text_font_size: "1.15em"
    })
  ]);
  new Theme([
    themed(Axis, {
      major_tick_line_alpha: 0,
      major_tick_line_color: "#5b5b5b",
      minor_tick_line_alpha: 0,
      minor_tick_line_color: "#5b5b5b",
      axis_line_alpha: 0,
      axis_line_color: "#5b5b5b",
      major_label_text_color: "#5b5b5b",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      axis_label_standoff: 10,
      axis_label_text_color: "#5b5b5b",
      axis_label_text_font: "Helvetica",
      axis_label_text_font_size: "1.25em",
      axis_label_text_font_style: "normal"
    }),
    themed(Legend, {
      spacing: 8,
      glyph_width: 15,
      label_standoff: 8,
      label_text_color: "#5b5b5b",
      label_text_font: "Helvetica",
      label_text_font_size: "1.025em",
      border_line_alpha: 0,
      background_fill_alpha: 0.25
    }),
    themed(BaseColorBar, {
      title_text_color: "#5b5b5b",
      title_text_font: "Helvetica",
      title_text_font_size: "1.025em",
      title_text_font_style: "normal",
      major_label_text_color: "#5b5b5b",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      major_tick_line_alpha: 0,
      bar_line_alpha: 0
    }),
    themed(Title, {
      text_color: "#5b5b5b",
      text_font: "Helvetica",
      text_font_size: "1.15em"
    })
  ]);
  new Theme([
    themed(Axis, {
      major_tick_in: 0,
      major_tick_out: 3,
      major_tick_line_alpha: 0.25,
      major_tick_line_color: "#5b5b5b",
      minor_tick_line_alpha: 0.25,
      minor_tick_line_color: "#5b5b5b",
      axis_line_alpha: 1,
      axis_line_color: "#5b5b5b",
      major_label_text_color: "#5b5b5b",
      major_label_text_font: "Calibri Light",
      major_label_text_font_size: "0.95em",
      major_label_text_font_style: "bold",
      axis_label_standoff: 10,
      axis_label_text_color: "#5b5b5b",
      axis_label_text_font: "Calibri Light",
      axis_label_text_font_size: "1.15em",
      axis_label_text_font_style: "bold"
    }),
    themed(Legend, {
      spacing: 8,
      glyph_width: 15,
      label_standoff: 8,
      label_text_color: "#5b5b5b",
      label_text_font: "Calibri Light",
      label_text_font_size: "0.95em",
      label_text_font_style: "bold",
      border_line_alpha: 0,
      background_fill_alpha: 0.25
    }),
    themed(BaseColorBar, {
      title_text_color: "#5b5b5b",
      title_text_font: "Calibri Light",
      title_text_font_size: "1.15em",
      title_text_font_style: "bold",
      major_label_text_color: "#5b5b5b",
      major_label_text_font: "Calibri Light",
      major_label_text_font_size: "0.95em",
      major_label_text_font_style: "bold",
      major_tick_line_alpha: 0,
      bar_line_alpha: 0
    }),
    themed(Title, {
      text_color: "#5b5b5b",
      text_font: "Calibri Light",
      text_font_size: "1.25em",
      text_font_style: "bold"
    })
  ]);
  new Theme([
    themed(Plot, {
      background_fill_color: "#000000",
      border_fill_color: "#ffffff",
      outline_line_color: "#000000",
      outline_line_alpha: 0.25
    }),
    themed(Grid, {
      grid_line_color: "#e0e0e0",
      grid_line_alpha: 0.25
    }),
    themed(Axis, {
      major_tick_line_alpha: 0,
      major_tick_line_color: "#000000",
      minor_tick_line_alpha: 0,
      minor_tick_line_color: "#000000",
      axis_line_alpha: 0,
      axis_line_color: "#000000",
      major_label_text_color: "#000000",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      axis_label_standoff: 10,
      axis_label_text_color: "#000000",
      axis_label_text_font: "Helvetica",
      axis_label_text_font_size: "1.25em",
      axis_label_text_font_style: "normal"
    }),
    themed(Legend, {
      spacing: 8,
      glyph_width: 15,
      label_standoff: 8,
      label_text_color: "#ffffff",
      label_text_font: "Helvetica",
      label_text_font_size: "1.025em",
      border_line_alpha: 0,
      background_fill_alpha: 0.25,
      background_fill_color: "#000000"
    }),
    themed(BaseColorBar, {
      title_text_color: "#e0e0e0",
      title_text_font: "Helvetica",
      title_text_font_size: "1.025em",
      title_text_font_style: "normal",
      major_label_text_color: "#e0e0e0",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      background_fill_color: "#15191c",
      major_tick_line_alpha: 0,
      bar_line_alpha: 0
    }),
    themed(Title, {
      text_color: "#000000",
      text_font: "Helvetica",
      text_font_size: "1.15em"
    })
  ]);
  new Theme([
    themed(Plot, {
      background_fill_color: "#2C001e",
      border_fill_color: "#15191c",
      outline_line_color: "#e0e0e0",
      outline_line_alpha: 0.25
    }),
    themed(Grid, {
      grid_line_color: "#e0e0e0",
      grid_line_alpha: 0.25
    }),
    themed(Axis, {
      major_tick_line_alpha: 0,
      major_tick_line_color: "#e0e0e0",
      minor_tick_line_alpha: 0,
      minor_tick_line_color: "#e0e0e0",
      axis_line_alpha: 0,
      axis_line_color: "#e0e0e0",
      major_label_text_color: "#e0e0e0",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      axis_label_standoff: 10,
      axis_label_text_color: "#e0e0e0",
      axis_label_text_font: "Helvetica",
      axis_label_text_font_size: "1.25em",
      axis_label_text_font_style: "normal"
    }),
    themed(Legend, {
      spacing: 8,
      glyph_width: 15,
      label_standoff: 8,
      label_text_color: "#e0e0e0",
      label_text_font: "Helvetica",
      label_text_font_size: "1.025em",
      border_line_alpha: 0,
      background_fill_alpha: 0.25,
      background_fill_color: "#2C001e"
    }),
    themed(BaseColorBar, {
      title_text_color: "#e0e0e0",
      title_text_font: "Helvetica",
      title_text_font_size: "1.025em",
      title_text_font_style: "normal",
      major_label_text_color: "#e0e0e0",
      major_label_text_font: "Helvetica",
      major_label_text_font_size: "1.025em",
      background_fill_color: "#15191c",
      major_tick_line_alpha: 0,
      bar_line_alpha: 0
    }),
    themed(Title, {
      text_color: "#e0e0e0",
      text_font: "Helvetica",
      text_font_size: "1.15em"
    })
  ]);
  const unary_ops = dict({
    "-": 1,
    "!": 1,
    "~": 1,
    "+": 1
  }), binary_ops = dict({
    "||": 1,
    "&&": 2,
    "|": 3,
    "^": 4,
    "&": 5,
    "==": 6,
    "!=": 6,
    "<": 7,
    ">": 7,
    "<=": 7,
    ">=": 7,
    "<<": 8,
    ">>": 8,
    "+": 9,
    "-": 9,
    "*": 10,
    "/": 10,
    "%": 10,
    "**": 11
  });
  dict({
    true: !0,
    false: !1,
    null: null
  });
  function max_key_len(O) {
    return Math.max(0, ...[...O.keys()].map((_) => _.length));
  }
  max_key_len(unary_ops);
  max_key_len(binary_ops);
  var dy;
  let WidgetView$1 = (dy = class extends LayoutDOMView {
    get child_models() {
      return [];
    }
    get provider() {
      return default_provider;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), this.provider.status == "not_started" && await this.provider.fetch();
    }
    _after_layout() {
      super._after_layout(), this.provider.status == "loading" && (this._has_finished = !1);
    }
    process_tex(_) {
      if (this.provider.MathJax == null)
        return _;
      const H = this.provider.MathJax.find_tex(_), V = [];
      let q = 0;
      for (const G of H)
        V.push(_.slice(q, G.start.n)), V.push(this.provider.MathJax.tex2svg(G.math, { display: G.display }).outerHTML), q = G.end.n;
      return q < _.length && V.push(_.slice(q)), V.join("");
    }
    contains_tex_string(_) {
      return this.provider.MathJax == null ? !1 : this.provider.MathJax.find_tex(_).length > 0;
    }
  }, mt(dy, "__name__", "WidgetView"), dy);
  const S6 = class S6 extends LayoutDOM {
    constructor(_) {
      super(_);
    }
  };
  mt(S6, "__name__", "Widget"), S6.override({
    margin: 5
  });
  let Widget = S6;
  class ControlView extends WidgetView$1 {
    connect_signals() {
      super.connect_signals(), this.connect(this.disabled, (_) => {
        for (const H of this.controls())
          toggle_attribute(H, "disabled", _);
      });
    }
  }
  mt(ControlView, "__name__", "ControlView");
  class Control extends Widget {
    constructor(_) {
      super(_);
    }
  }
  mt(Control, "__name__", "Control");
  const btn = "bk-btn", active$1 = "bk-active", btn_default = "bk-btn-default", btn_primary = "bk-btn-primary", btn_success = "bk-btn-success", btn_warning = "bk-btn-warning", btn_danger = "bk-btn-danger", btn_light = "bk-btn-light", btn_group = "bk-btn-group", vertical = "bk-vertical", horizontal = "bk-horizontal", dropdown_toggle = "bk-dropdown-toggle", buttons_css = ".bk-btn,::file-selector-button{height:100%;display:inline-block;text-align:center;vertical-align:middle;white-space:nowrap;cursor:pointer;padding:var(--padding-vertical) var(--padding-horizontal);font-size:var(--font-size);border:1px solid transparent;border-radius:var(--border-radius);outline:0;outline-offset:-5px;user-select:none;-webkit-user-select:none;}.bk-btn:hover,::file-selector-button:hover,.bk-btn:focus,::file-selector-button:focus{text-decoration:none;}.bk-btn:active,::file-selector-button:active,.bk-active.bk-btn,.bk-active::file-selector-button{background-image:none;box-shadow:inset 0 3px 5px rgba(0, 0, 0, 0.125);}.bk-btn[disabled]{cursor:not-allowed;pointer-events:none;opacity:0.65;box-shadow:none;}::file-selector-button{color:#333;background-color:#fff;border-color:#ccc;}::file-selector-button:hover{background-color:#f5f5f5;border-color:#b8b8b8;}.bk-active::file-selector-button{background-color:#ebebeb;border-color:#adadad;}::file-selector-button[disabled],::file-selector-button[disabled]:hover,::file-selector-button[disabled]:focus,::file-selector-button[disabled]:active,.bk-active::file-selector-button[disabled]{background-color:#e6e6e6;border-color:#ccc;}::file-selector-button:focus,::file-selector-button:active{outline:1px dotted #ccc;}.bk-btn-default{color:#333;background-color:#fff;border-color:#ccc;}.bk-btn-default:hover{background-color:#f5f5f5;border-color:#b8b8b8;}.bk-active.bk-btn-default{background-color:#ebebeb;border-color:#adadad;}.bk-btn-default[disabled],.bk-btn-default[disabled]:hover,.bk-btn-default[disabled]:focus,.bk-btn-default[disabled]:active,.bk-active.bk-btn-default[disabled]{background-color:#e6e6e6;border-color:#ccc;}.bk-btn-default:focus,.bk-btn-default:active{outline:1px dotted #ccc;}.bk-btn-primary{color:#fff;background-color:#428bca;border-color:#357ebd;}.bk-btn-primary:hover{background-color:#3681c1;border-color:#2c699e;}.bk-active.bk-btn-primary{background-color:#3276b1;border-color:#285e8e;}.bk-btn-primary[disabled],.bk-btn-primary[disabled]:hover,.bk-btn-primary[disabled]:focus,.bk-btn-primary[disabled]:active,.bk-active.bk-btn-primary[disabled]{background-color:#506f89;border-color:#357ebd;}.bk-btn-primary:focus,.bk-btn-primary:active{outline:1px dotted #ccc;}.bk-btn-success{color:#fff;background-color:#5cb85c;border-color:#4cae4c;}.bk-btn-success:hover{background-color:#4eb24e;border-color:#409240;}.bk-active.bk-btn-success{background-color:#47a447;border-color:#398439;}.bk-btn-success[disabled],.bk-btn-success[disabled]:hover,.bk-btn-success[disabled]:focus,.bk-btn-success[disabled]:active,.bk-active.bk-btn-success[disabled]{background-color:#667b66;border-color:#4cae4c;}.bk-btn-success:focus,.bk-btn-success:active{outline:1px dotted #ccc;}.bk-btn-warning{color:#fff;background-color:#f0ad4e;border-color:#eea236;}.bk-btn-warning:hover{background-color:#eea43b;border-color:#e89014;}.bk-active.bk-btn-warning{background-color:#ed9c28;border-color:#d58512;}.bk-btn-warning[disabled],.bk-btn-warning[disabled]:hover,.bk-btn-warning[disabled]:focus,.bk-btn-warning[disabled]:active,.bk-active.bk-btn-warning[disabled]{background-color:#c89143;border-color:#eea236;}.bk-btn-warning:focus,.bk-btn-warning:active{outline:1px dotted #ccc;}.bk-btn-danger{color:#fff;background-color:#d9534f;border-color:#d43f3a;}.bk-btn-danger:hover{background-color:#d5433e;border-color:#bd2d29;}.bk-active.bk-btn-danger{background-color:#d2322d;border-color:#ac2925;}.bk-btn-danger[disabled],.bk-btn-danger[disabled]:hover,.bk-btn-danger[disabled]:focus,.bk-btn-danger[disabled]:active,.bk-active.bk-btn-danger[disabled]{background-color:#a55350;border-color:#d43f3a;}.bk-btn-danger:focus,.bk-btn-danger:active{outline:1px dotted #ccc;}.bk-btn-light{color:#333;background-color:#fff;border-color:#ccc;border-color:transparent;}.bk-btn-light:hover{background-color:#f5f5f5;border-color:#b8b8b8;}.bk-active.bk-btn-light{background-color:#ebebeb;border-color:#adadad;}.bk-btn-light[disabled],.bk-btn-light[disabled]:hover,.bk-btn-light[disabled]:focus,.bk-btn-light[disabled]:active,.bk-active.bk-btn-light[disabled]{background-color:#e6e6e6;border-color:#ccc;}.bk-btn-light:focus,.bk-btn-light:active{outline:1px dotted #ccc;}.bk-btn-group{height:100%;display:flex;flex-wrap:nowrap;align-items:center;}.bk-btn-group:not(.bk-vertical),.bk-btn-group.bk-horizontal{flex-direction:row;}.bk-btn-group.bk-vertical{flex-direction:column;}.bk-btn-group > .bk-btn{flex-grow:1;}.bk-btn-group:not(.bk-vertical) > .bk-btn + .bk-btn{margin-left:-1px;}.bk-btn-group.bk-vertical > .bk-btn + .bk-btn{margin-top:-1px;}.bk-btn-group:not(.bk-vertical) > .bk-btn:first-child:not(:last-child){border-bottom-right-radius:0;border-top-right-radius:0;}.bk-btn-group.bk-vertical > .bk-btn:first-child:not(:last-child){border-bottom-left-radius:0;border-bottom-right-radius:0;}.bk-btn-group:not(.bk-vertical) > .bk-btn:not(:first-child):last-child{border-bottom-left-radius:0;border-top-left-radius:0;}.bk-btn-group.bk-vertical > .bk-btn:not(:first-child):last-child{border-top-left-radius:0;border-top-right-radius:0;}.bk-btn-group > .bk-btn:not(:first-child):not(:last-child){border-radius:0;}.bk-btn-group.bk-vertical > .bk-btn{width:100%;}.bk-btn-group .bk-dropdown-toggle{flex:0 0 0;padding:var(--padding-vertical) calc(var(--padding-horizontal)/2);}", buttons = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    active: active$1,
    btn,
    btn_danger,
    btn_default,
    btn_group,
    btn_light,
    btn_primary,
    btn_success,
    btn_warning,
    default: buttons_css,
    dropdown_toggle,
    horizontal,
    vertical
  }, Symbol.toStringTag, { value: "Module" }));
  class AbstractButtonView extends ControlView {
    constructor() {
      super(...arguments);
      mt(this, "label_view");
      mt(this, "icon_view");
      mt(this, "button_el");
      mt(this, "group_el");
    }
    *controls() {
      yield this.button_el;
    }
    *children() {
      yield* super.children(), this.label_view != null && (yield this.label_view), this.icon_view != null && (yield this.icon_view);
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._rebuild_label(), await this._rebuild_icon();
    }
    async _rebuild_label() {
      var V;
      (V = this.label_view) == null || V.remove();
      const H = (() => {
        const { label: q } = this.model;
        return isString$2(q) ? new Text$2({ content: q }) : q;
      })();
      this.label_view = await this.owner.build_view(H, this);
    }
    async _rebuild_icon() {
      var V;
      (V = this.icon_view) == null || V.remove();
      const { icon: H } = this.model;
      H != null && (this.icon_view = await build_view(H, { parent: this }));
    }
    connect_signals() {
      super.connect_signals();
      const { label: H, icon: V, button_type: q, disabled: G } = this.model.properties;
      this.on_transitive_change(H, async () => {
        await this._rebuild_label(), this.render();
      }), this.on_transitive_change(V, async () => {
        await this._rebuild_icon(), this.render();
      }), this.on_change([q, G], () => {
        this.render();
      });
    }
    remove() {
      var H, V;
      (H = this.label_view) == null || H.remove(), (V = this.icon_view) == null || V.remove(), super.remove();
    }
    stylesheets() {
      return [...super.stylesheets(), buttons_css];
    }
    _render_button(...H) {
      return button({
        type: "button",
        disabled: this.model.disabled,
        class: [btn, buttons[`btn_${this.model.button_type}`]]
      }, ...H);
    }
    render() {
      var H, V;
      if (super.render(), (H = this.label_view) == null || H.render_to(null), this.button_el = this._render_button((V = this.label_view) == null ? void 0 : V.el), this.button_el.addEventListener("click", () => this.click()), this.icon_view != null) {
        const q = this.model.label != "" ? nbsp() : text$4("");
        prepend(this.button_el, this.icon_view.el, q), this.icon_view.render_to(null);
      }
      this.group_el = div({ class: btn_group }, this.button_el), this.shadow_el.append(this.group_el);
    }
    click() {
    }
  }
  mt(AbstractButtonView, "__name__", "AbstractButtonView");
  const E6 = class E6 extends Control {
    constructor(_) {
      super(_);
    }
  };
  mt(E6, "__name__", "AbstractButton"), E6.define(({ Str: _, Ref: H, Or: V, Nullable: q }) => ({
    label: [V(H(DOMNode), _), "Button"],
    icon: [q(H(Icon)), null],
    button_type: [ButtonType, "default"]
  }));
  let AbstractButton = E6;
  const input = "bk-input", disabled = "bk-disabled", input_group = "bk-input-group", inline = "bk-inline", description$3 = "bk-description", icon = "bk-icon", opaque = "bk-opaque", inputs_css = `:host{--input-min-height:calc(var(--line-height-computed) + 2*var(--padding-vertical) + 2px);}.bk-input{position:relative;display:inline-block;width:100%;flex-grow:1;min-height:var(--input-min-height);padding:0 var(--padding-horizontal);background-color:#fff;border:1px solid #ccc;border-radius:var(--border-radius);}.bk-input:focus{border-color:#66afe9;outline:0;box-shadow:inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(102, 175, 233, 0.6);}.bk-input::placeholder,.bk-input:-ms-input-placeholder,.bk-input::-moz-placeholder,.bk-input::-webkit-input-placeholder{color:#999;opacity:1;}.bk-input[disabled],.bk-input.bk-disabled{cursor:not-allowed;background-color:#eee;opacity:1;}.bk-input-container{width:100%;height:100%;display:flex;flex-direction:row;flex-wrap:nowrap;}.bk-input-container .bk-input-prefix,.bk-input-container .bk-input-suffix{display:flex;align-items:center;flex:0 1 0;border:1px solid #ccc;border-radius:var(--border-radius);padding:0 var(--padding-horizontal);background-color:#e6e6e6;}.bk-input-container .bk-input-prefix{border-right:none;border-top-right-radius:0;border-bottom-right-radius:0;}.bk-input-container .bk-input-suffix{border-left:none;border-top-left-radius:0;border-bottom-left-radius:0;}.bk-input-container .bk-input{flex:1 0 0;}.bk-input-container .bk-input:not(:first-child){border-top-left-radius:0;border-bottom-left-radius:0;}.bk-input-container .bk-input:not(:last-child){border-top-right-radius:0;border-bottom-right-radius:0;}input[type=file].bk-input{padding-left:0;}input[type=file]::file-selector-button{box-sizing:inherit;font-family:inherit;font-size:inherit;line-height:inherit;}select:not([multiple]).bk-input,select:not([size]).bk-input{height:auto;appearance:none;-webkit-appearance:none;background-image:url('data:image/svg+xml;utf8,<svg version="1.1" viewBox="0 0 25 20" xmlns="http://www.w3.org/2000/svg"><path d="M 0,0 25,0 12.5,20 Z" fill="black" /></svg>');background-position:right 0.5em center;background-size:8px 6px;background-repeat:no-repeat;padding-right:calc(var(--padding-horizontal) + 8px);}option{padding:0;}select[multiple].bk-input,select[size].bk-input,textarea.bk-input{height:auto;}.bk-input-group{position:relative;width:100%;height:100%;display:inline-flex;flex-wrap:nowrap;align-items:start;flex-direction:column;white-space:nowrap;}.bk-input-group.bk-inline{flex-direction:row;}.bk-input-group.bk-inline > *:not(:first-child){margin-left:5px;}.bk-input-group > .bk-spin-wrapper{display:inherit;width:inherit;height:inherit;position:relative;overflow:hidden;padding:0;vertical-align:middle;}.bk-input-group > .bk-spin-wrapper input{padding-right:20px;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn{position:absolute;display:block;height:50%;min-height:0;min-width:0;width:30px;padding:0;margin:0;right:0;border:none;background:none;cursor:pointer;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn:before{content:"";display:inline-block;transform:translateY(-50%);border-left:5px solid transparent;border-right:5px solid transparent;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn.bk-spin-btn-up{top:0;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn.bk-spin-btn-up:before{border-bottom:5px solid black;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn.bk-spin-btn-up:disabled:before{border-bottom-color:grey;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn.bk-spin-btn-down{bottom:0;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn.bk-spin-btn-down:before{border-top:5px solid black;}.bk-input-group > .bk-spin-wrapper > .bk-spin-btn.bk-spin-btn-down:disabled:before{border-top-color:grey;}.bk-description{position:relative;display:inline-block;margin-left:0.25em;vertical-align:middle;margin-top:-2px;cursor:pointer;}.bk-description > .bk-icon{opacity:0.5;width:18px;height:18px;background-color:gray;mask-image:var(--bokeh-icon-help);mask-size:contain;mask-repeat:no-repeat;-webkit-mask-image:var(--bokeh-icon-help);-webkit-mask-size:contain;-webkit-mask-repeat:no-repeat;}label:hover > .bk-description > .bk-icon,.bk-icon.bk-opaque{opacity:1;}`;
  class InputWidgetView extends ControlView {
    constructor() {
      super(...arguments);
      mt(this, "title");
      mt(this, "description", null);
      mt(this, "input_el");
      mt(this, "title_el");
      mt(this, "desc_el", null);
      mt(this, "group_el");
    }
    *controls() {
      yield this.input_el;
    }
    *children() {
      yield* super.children();
      const { title: H, description: V } = this;
      H instanceof View$2 && (yield H), V instanceof View$2 && (yield V);
    }
    async lazy_initialize() {
      await super.lazy_initialize(), await this._build_title(), await this._build_description();
    }
    remove() {
      const { title: H, description: V } = this;
      H instanceof View$2 && H.remove(), V instanceof View$2 && V.remove(), super.remove();
    }
    connect_signals() {
      super.connect_signals();
      const { title: H, description: V } = this.model.properties;
      this.on_change(H, async () => {
        await this._build_title(), this.render();
      }), this.on_change(V, async () => {
        await this._build_description(), this.render();
      });
    }
    stylesheets() {
      return [...super.stylesheets(), inputs_css, icons_css];
    }
    render() {
      super.render(), this.desc_el = this._build_description_el(), this.title_el = this._build_title_el();
      const H = this._render_input();
      this.input_el.id = "input", this.group_el = div({ class: input_group }, this.title_el, H), this.shadow_el.append(this.group_el);
    }
    _build_description_el() {
      const { description: H } = this;
      if (H == null)
        return null;
      {
        const V = div({ class: icon }), q = div({ class: description$3 }, V);
        if (isString$2(H))
          q.title = H;
        else {
          H.model.target == "auto" && (H.target = q);
          let G = !1;
          const J = (Z) => {
            H.model.setv({
              visible: Z,
              closable: G
            }), V.classList.toggle(opaque, Z && G);
          };
          this.on_change(H.model.properties.visible, () => {
            const { visible: Z } = H.model;
            Z || (G = !1), J(Z);
          }), q.addEventListener("mouseenter", () => {
            J(!0);
          }), q.addEventListener("mouseleave", () => {
            G || J(!1);
          }), document.addEventListener("mousedown", (Z) => {
            const te = Z.composedPath();
            te.includes(H.el) || (te.includes(q) ? (G = !G, J(G)) : (G = !1, J(!1)));
          }), window.addEventListener("blur", () => {
            G = !1, J(!1);
          });
        }
        return q;
      }
    }
    async _build_title() {
      const { title: H } = this.model;
      H instanceof HTML ? this.title = await build_view(H, { parent: this }) : this.title = H;
    }
    async _build_description() {
      const { description: H } = this.model;
      H instanceof Tooltip ? this.description = await build_view(H, { parent: this }) : this.description = H;
    }
    _build_title_el() {
      const { title: H } = this, V = H instanceof HTMLView ? (H.render(), H.el) : H, q = H == "" ? "none" : "";
      return label$2({ for: "input", style: { display: q } }, V, this.desc_el);
    }
    change_input() {
    }
  }
  mt(InputWidgetView, "__name__", "InputWidgetView");
  const A6 = class A6 extends Control {
    constructor(_) {
      super(_);
    }
  };
  mt(A6, "__name__", "InputWidget"), A6.define(({ Str: _, Nullable: H, Or: V, Ref: q }) => ({
    title: [V(_, q(HTML)), ""],
    description: [H(V(_, q(Tooltip))), null]
  }));
  let InputWidget = A6;
  class TextLikeInputView extends InputWidgetView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.value.change, () => this.input_el.value = this.model.value), this.connect(this.model.properties.value_input.change, () => this.input_el.value = this.model.value_input), this.connect(this.model.properties.disabled.change, () => this.input_el.disabled = this.model.disabled), this.connect(this.model.properties.placeholder.change, () => this.input_el.placeholder = this.model.placeholder), this.connect(this.model.properties.max_length.change, () => {
        const { max_length: _ } = this.model;
        _ != null ? this.input_el.maxLength = _ : this.input_el.removeAttribute("maxLength");
      });
    }
    render() {
      super.render();
      const { input_el: _ } = this;
      _.value = this.model.value, _.disabled = this.model.disabled, _.placeholder = this.model.placeholder, this.model.max_length != null && (_.maxLength = this.model.max_length), _.addEventListener("change", () => this.change_input()), _.addEventListener("input", () => this.change_input_value());
    }
    change_input() {
      this.model.value = this.input_el.value, super.change_input();
    }
    change_input_value() {
      this.model.value_input = this.input_el.value, super.change_input();
    }
  }
  mt(TextLikeInputView, "__name__", "TextLikeInputView");
  const Q6 = class Q6 extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(Q6, "__name__", "TextLikeInput"), Q6.define(({ Int: _, Str: H, Nullable: V }) => ({
    value: [H, ""],
    value_input: [H, ""],
    placeholder: [H, ""],
    max_length: [V(_), null]
  }));
  let TextLikeInput = Q6;
  class TextInputView extends TextLikeInputView {
    connect_signals() {
      super.connect_signals();
      const { prefix: _, suffix: H } = this.model.properties;
      this.on_change([_, H], () => this.render());
    }
    _render_input() {
      this.input_el = input$1({ type: "text", class: input });
      const { prefix: _, suffix: H } = this.model, V = _ != null ? div({ class: "bk-input-prefix" }, _) : null, q = H != null ? div({ class: "bk-input-suffix" }, H) : null;
      return div({ class: "bk-input-container" }, V, this.input_el, q);
    }
    render() {
      super.render(), this.input_el.addEventListener("keyup", (_) => this._keyup(_));
    }
    _keyup(_) {
      _.key == "Enter" && !_.shiftKey && !_.ctrlKey && !_.altKey && this.model.trigger_event(new ValueSubmit(this.input_el.value));
    }
  }
  mt(TextInputView, "__name__", "TextInputView");
  const Vm = class Vm extends TextLikeInput {
    constructor(_) {
      super(_);
    }
  };
  mt(Vm, "__name__", "TextInput"), Vm.prototype.default_view = TextInputView, Vm.define(({ Str: _, Nullable: H }) => ({
    prefix: [H(_), null],
    suffix: [H(_), null]
  }));
  let TextInput = Vm;
  const menu = "bk-menu", below = "bk-below", divider = "bk-divider", active = "bk-active", dropdown_css = ":host{position:relative;}.bk-menu{position:absolute;left:0;width:100%;z-index:var(--bokeh-top-level);cursor:pointer;font-size:var(--font-size);background-color:#fff;border:1px solid #ccc;border-radius:var(--border-radius);box-shadow:0 6px 12px rgba(0, 0, 0, 0.175);}.bk-menu.bk-above{bottom:100%;}.bk-menu.bk-below{top:100%;}.bk-menu > .bk-divider{height:1px;margin:calc(var(--line-height-computed)/2 - 1px) 0;overflow:hidden;background-color:#e5e5e5;}.bk-menu > :not(.bk-divider){padding:var(--padding-vertical) var(--padding-horizontal);}.bk-menu > :not(.bk-divider):hover,.bk-menu > :not(.bk-divider).bk-active{background-color:#e6e6e6;}", SearchStrategy = Enum("starts_with", "includes");
  class AutocompleteInputView extends TextInputView {
    constructor() {
      super(...arguments);
      mt(this, "_open", !1);
      mt(this, "_last_value", "");
      mt(this, "_hover_index", 0);
      mt(this, "menu");
    }
    stylesheets() {
      return [...super.stylesheets(), dropdown_css];
    }
    render() {
      super.render(), this.input_el.addEventListener("focusin", () => this._toggle_menu()), this.menu = div({ class: [menu, below] }), this.menu.addEventListener("click", (H) => this._menu_click(H)), this.menu.addEventListener("mouseover", (H) => this._menu_hover(H)), this.shadow_el.appendChild(this.menu), undisplay(this.menu);
    }
    change_input() {
      this._open && this.menu.children.length > 0 ? (this.model.value = this.menu.children[this._hover_index].textContent, this.input_el.focus(), this._hide_menu()) : this.model.restrict || super.change_input();
    }
    _update_completions(H) {
      var G;
      empty$1(this.menu);
      const { max_completions: V } = this.model, q = V != null ? take(H, V) : H;
      for (const J of q) {
        const Z = div(J);
        this.menu.append(Z);
      }
      (G = this.menu.firstElementChild) == null || G.classList.add(active);
    }
    compute_completions(H) {
      const V = (() => {
        const { case_sensitive: Z } = this.model;
        return Z ? (te) => te : (te) => te.toLowerCase();
      })(), q = (() => {
        switch (this.model.search_strategy) {
          case "starts_with":
            return (Z, te) => Z.startsWith(te);
          case "includes":
            return (Z, te) => Z.includes(te);
        }
      })(), G = V(H), J = [];
      for (const Z of this.model.completions) {
        const te = V(Z);
        q(te, G) && J.push(Z);
      }
      return J;
    }
    _toggle_menu() {
      const { value: H } = this.input_el;
      if (H.length < this.model.min_characters) {
        this._hide_menu();
        return;
      }
      const V = this.compute_completions(H);
      this._update_completions(V), V.length == 0 ? this._hide_menu() : this._show_menu();
    }
    _show_menu() {
      if (!this._open) {
        this._open = !0, this._hover_index = 0, this._last_value = this.model.value, display(this.menu);
        const H = (V) => {
          V.composedPath().includes(this.el) || (document.removeEventListener("click", H), this._hide_menu());
        };
        document.addEventListener("click", H);
      }
    }
    _hide_menu() {
      this._open && (this._open = !1, undisplay(this.menu));
    }
    _menu_click(H) {
      H.target != H.currentTarget && H.target instanceof Element && (this.model.value = H.target.textContent, this.input_el.focus(), this._hide_menu());
    }
    _menu_hover(H) {
      if (H.target != H.currentTarget && H.target instanceof Element) {
        for (let V = 0; V < this.menu.children.length; V++)
          if (this.menu.children[V].textContent == H.target.textContent) {
            this._bump_hover(V);
            break;
          }
      }
    }
    _bump_hover(H) {
      const V = this.menu.children.length;
      this._open && V > 0 && (this.menu.children[this._hover_index].classList.remove(active), this._hover_index = clamp$1(H, 0, V - 1), this.menu.children[this._hover_index].classList.add(active));
    }
    _keyup(H) {
      switch (super._keyup(H), H.key) {
        case "Enter": {
          this.change_input();
          break;
        }
        case "Escape": {
          this._hide_menu();
          break;
        }
        case "ArrowUp": {
          this._bump_hover(this._hover_index - 1);
          break;
        }
        case "ArrowDown": {
          this._bump_hover(this._hover_index + 1);
          break;
        }
        default:
          this._toggle_menu();
      }
    }
  }
  mt(AutocompleteInputView, "__name__", "AutocompleteInputView");
  const Um = class Um extends TextInput {
    constructor(_) {
      super(_);
    }
  };
  mt(Um, "__name__", "AutocompleteInput"), Um.prototype.default_view = AutocompleteInputView, Um.define(({ Bool: _, Int: H, Str: V, List: q, NonNegative: G, Positive: J, Nullable: Z }) => ({
    completions: [q(V), []],
    min_characters: [G(H), 2],
    max_completions: [Z(J(H)), null],
    case_sensitive: [_, !0],
    restrict: [_, !0],
    search_strategy: [SearchStrategy, "starts_with"]
  }));
  let AutocompleteInput = Um;
  class ButtonView extends AbstractButtonView {
    click() {
      this.model.trigger_event(new ButtonClick()), super.click();
    }
  }
  mt(ButtonView, "__name__", "ButtonView");
  const qm = class qm extends AbstractButton {
    constructor(_) {
      super(_);
    }
    on_click(_) {
      this.on_event(ButtonClick, _);
    }
  };
  mt(qm, "__name__", "Button"), qm.prototype.default_view = ButtonView, qm.override({
    label: "Button"
  });
  let Button = qm;
  class OrientedControlView extends ControlView {
  }
  mt(OrientedControlView, "__name__", "OrientedControlView");
  const C6 = class C6 extends Control {
    constructor(_) {
      super(_);
    }
  };
  mt(C6, "__name__", "OrientedControl"), C6.define(() => ({
    orientation: [Orientation, "horizontal"]
  }));
  let OrientedControl = C6;
  class ToggleButtonGroupView extends OrientedControlView {
    constructor() {
      super(...arguments);
      mt(this, "_buttons");
    }
    *controls() {
      yield* this._buttons;
    }
    connect_signals() {
      super.connect_signals();
      const H = this.model.properties;
      this.on_change(H.button_type, () => this.render()), this.on_change(H.labels, () => this.render()), this.on_change(H.active, () => this._update_active());
    }
    stylesheets() {
      return [...super.stylesheets(), buttons_css];
    }
    render() {
      super.render(), this._buttons = this.model.labels.map((q, G) => {
        const J = button({
          class: [btn, buttons[`btn_${this.model.button_type}`]],
          disabled: this.model.disabled
        }, q);
        return J.addEventListener("click", () => {
          this.change_active(G), this.model.trigger_event(new ButtonClick());
        }), J;
      }), this._update_active();
      const H = this.model.orientation == "horizontal" ? horizontal : vertical, V = div({ class: [btn_group, H] }, this._buttons);
      this.shadow_el.appendChild(V);
    }
  }
  mt(ToggleButtonGroupView, "__name__", "ToggleButtonGroupView");
  const L6 = class L6 extends OrientedControl {
    constructor(_) {
      super(_);
    }
  };
  mt(L6, "__name__", "ToggleButtonGroup"), L6.define(({ Str: _, List: H }) => ({
    labels: [H(_), []],
    button_type: [ButtonType, "default"]
  }));
  let ToggleButtonGroup = L6;
  class CheckboxButtonGroupView extends ToggleButtonGroupView {
    get active() {
      return new Set(this.model.active);
    }
    change_active(_) {
      const { active: H } = this;
      H.has(_) ? H.delete(_) : H.add(_), this.model.active = [...H].sort();
    }
    _update_active() {
      const { active: _ } = this;
      this._buttons.forEach((H, V) => {
        H.classList.toggle(active$1, _.has(V));
      });
    }
  }
  mt(CheckboxButtonGroupView, "__name__", "CheckboxButtonGroupView");
  const jm = class jm extends ToggleButtonGroup {
    constructor(_) {
      super(_);
    }
  };
  mt(jm, "__name__", "CheckboxButtonGroup"), jm.prototype.default_view = CheckboxButtonGroupView, jm.define(({ Int: _, List: H }) => ({
    active: [H(_), []]
  }));
  let CheckboxButtonGroup = jm;
  const checkbox_css = 'input[type="checkbox"],input[type="radio"]{margin:0;}input[type="checkbox"] + *,input[type="radio"] + *{position:relative;top:-2px;margin-left:3px;}';
  class ToggleInputGroupView extends ControlView {
    constructor() {
      super(...arguments);
      mt(this, "_inputs");
    }
    *controls() {
      yield* this._inputs;
    }
    connect_signals() {
      super.connect_signals();
      const { labels: H, inline: V } = this.model.properties;
      this.on_change([H, V], () => this.render());
    }
    stylesheets() {
      return [...super.stylesheets(), inputs_css, checkbox_css];
    }
  }
  mt(ToggleInputGroupView, "__name__", "ToggleInputGroupView");
  const k6 = class k6 extends Control {
    constructor(_) {
      super(_);
    }
  };
  mt(k6, "__name__", "ToggleInputGroup"), k6.define(({ Bool: _, Str: H, List: V }) => ({
    labels: [V(H), []],
    inline: [_, !1]
  }));
  let ToggleInputGroup = k6;
  class CheckboxGroupView extends ToggleInputGroupView {
    get active() {
      return new Set(this.model.active);
    }
    connect_signals() {
      super.connect_signals();
      const { active: _ } = this.model.properties;
      this.on_change(_, () => {
        const { active: H } = this;
        for (const [V, q] of enumerate(this._inputs))
          V.checked = H.has(q);
      });
    }
    render() {
      super.render();
      const _ = div({ class: [input_group, this.model.inline ? inline : null] });
      this.shadow_el.appendChild(_);
      const { active: H, labels: V } = this.model;
      this._inputs = [];
      for (let q = 0; q < V.length; q++) {
        const G = input$1({ type: "checkbox", value: `${q}` });
        G.addEventListener("change", () => this.change_active(q)), this._inputs.push(G), this.model.disabled && (G.disabled = !0), includes(H, q) && (G.checked = !0);
        const J = label$2(G, span$1(V[q]));
        _.appendChild(J);
      }
    }
    change_active(_) {
      const { active: H } = this;
      H.has(_) ? H.delete(_) : H.add(_), this.model.active = [...H].sort();
    }
  }
  mt(CheckboxGroupView, "__name__", "CheckboxGroupView");
  const Gm = class Gm extends ToggleInputGroup {
    constructor(_) {
      super(_);
    }
  };
  mt(Gm, "__name__", "CheckboxGroup"), Gm.prototype.default_view = CheckboxGroupView, Gm.define(({ Int: _, List: H }) => ({
    active: [H(_), []]
  }));
  let CheckboxGroup = Gm;
  class ToggleInputView extends WidgetView$1 {
    connect_signals() {
      super.connect_signals();
      const { active: _, disabled: H } = this.model.properties;
      this.on_change(_, () => this._update_active()), this.on_change(H, () => this._update_disabled());
    }
    _toggle_active() {
      this.model.disabled || (this.model.active = !this.model.active);
    }
  }
  mt(ToggleInputView, "__name__", "ToggleInputView");
  const M6 = class M6 extends Widget {
    constructor(_) {
      super(_);
    }
  };
  mt(M6, "__name__", "ToggleInput"), M6.define(({ Bool: _ }) => ({
    active: [_, !1]
  }));
  let ToggleInput = M6;
  class CheckboxView extends ToggleInputView {
    constructor() {
      super(...arguments);
      mt(this, "checkbox_el");
      mt(this, "label_el");
    }
    stylesheets() {
      return [...super.stylesheets(), checkbox_css];
    }
    connect_signals() {
      super.connect_signals();
      const { label: H } = this.model.properties;
      this.on_change(H, () => this._update_label());
    }
    render() {
      super.render(), this.checkbox_el = input$1({ type: "checkbox" }), this.label_el = span$1(this.model.label), this.checkbox_el.addEventListener("change", () => this._toggle_active()), this._update_active(), this._update_disabled(), this.shadow_el.append(this.checkbox_el, this.label_el);
    }
    _update_active() {
      this.checkbox_el.checked = this.model.active;
    }
    _update_disabled() {
      this.checkbox_el.toggleAttribute("disabled", this.model.disabled);
    }
    _update_label() {
      this.label_el.textContent = this.model.label;
    }
  }
  mt(CheckboxView, "__name__", "CheckboxView");
  const Wm = class Wm extends ToggleInput {
    constructor(_) {
      super(_);
    }
  };
  mt(Wm, "__name__", "Checkbox"), Wm.prototype.default_view = CheckboxView, Wm.define(({ Str: _ }) => ({
    label: [_, ""]
  }));
  let Checkbox = Wm;
  class ColorPickerView extends InputWidgetView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.name.change, () => this.input_el.name = this.model.name ?? ""), this.connect(this.model.properties.color.change, () => this.input_el.value = color2hexrgb(this.model.color)), this.connect(this.model.properties.disabled.change, () => this.input_el.disabled = this.model.disabled);
    }
    _render_input() {
      return this.input_el = input$1({
        type: "color",
        class: input,
        name: this.model.name,
        value: this.model.color,
        disabled: this.model.disabled
      });
    }
    render() {
      super.render(), this.input_el.addEventListener("change", () => this.change_input());
    }
    change_input() {
      this.model.color = this.input_el.value, super.change_input();
    }
  }
  mt(ColorPickerView, "__name__", "ColorPickerView");
  const Ym = class Ym extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(Ym, "__name__", "ColorPicker"), Ym.prototype.default_view = ColorPickerView, Ym.define(({ Color: _ }) => ({
    color: [_, "#000000"]
  }));
  let ColorPicker = Ym;
  var HOOKS = [
    "onChange",
    "onClose",
    "onDayCreate",
    "onDestroy",
    "onKeyDown",
    "onMonthChange",
    "onOpen",
    "onParseConfig",
    "onReady",
    "onValueUpdate",
    "onYearChange",
    "onPreCalendarPosition"
  ], defaults$5 = {
    _disable: [],
    allowInput: !1,
    allowInvalidPreload: !1,
    altFormat: "F j, Y",
    altInput: !1,
    altInputClass: "form-control input",
    animate: typeof window == "object" && window.navigator.userAgent.indexOf("MSIE") === -1,
    ariaDateFormat: "F j, Y",
    autoFillDefaultTime: !0,
    clickOpens: !0,
    closeOnSelect: !0,
    conjunction: ", ",
    dateFormat: "Y-m-d",
    defaultHour: 12,
    defaultMinute: 0,
    defaultSeconds: 0,
    disable: [],
    disableMobile: !1,
    enableSeconds: !1,
    enableTime: !1,
    errorHandler: function(O) {
      return typeof console < "u" && console.warn(O);
    },
    getWeek: function(O) {
      var _ = new Date(O.getTime());
      _.setHours(0, 0, 0, 0), _.setDate(_.getDate() + 3 - (_.getDay() + 6) % 7);
      var H = new Date(_.getFullYear(), 0, 4);
      return 1 + Math.round(((_.getTime() - H.getTime()) / 864e5 - 3 + (H.getDay() + 6) % 7) / 7);
    },
    hourIncrement: 1,
    ignoredFocusElements: [],
    inline: !1,
    locale: "default",
    minuteIncrement: 5,
    mode: "single",
    monthSelectorType: "dropdown",
    nextArrow: "<svg version='1.1' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' viewBox='0 0 17 17'><g></g><path d='M13.207 8.472l-7.854 7.854-0.707-0.707 7.146-7.146-7.146-7.148 0.707-0.707 7.854 7.854z' /></svg>",
    noCalendar: !1,
    now: /* @__PURE__ */ new Date(),
    onChange: [],
    onClose: [],
    onDayCreate: [],
    onDestroy: [],
    onKeyDown: [],
    onMonthChange: [],
    onOpen: [],
    onParseConfig: [],
    onReady: [],
    onValueUpdate: [],
    onYearChange: [],
    onPreCalendarPosition: [],
    plugins: [],
    position: "auto",
    positionElement: void 0,
    prevArrow: "<svg version='1.1' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' viewBox='0 0 17 17'><g></g><path d='M5.207 8.471l7.146 7.147-0.707 0.707-7.853-7.854 7.854-7.853 0.707 0.707-7.147 7.146z' /></svg>",
    shorthandCurrentMonth: !1,
    showMonths: 1,
    static: !1,
    time_24hr: !1,
    weekNumbers: !1,
    wrap: !1
  }, english = {
    weekdays: {
      shorthand: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
      longhand: [
        "Sunday",
        "Monday",
        "Tuesday",
        "Wednesday",
        "Thursday",
        "Friday",
        "Saturday"
      ]
    },
    months: {
      shorthand: [
        "Jan",
        "Feb",
        "Mar",
        "Apr",
        "May",
        "Jun",
        "Jul",
        "Aug",
        "Sep",
        "Oct",
        "Nov",
        "Dec"
      ],
      longhand: [
        "January",
        "February",
        "March",
        "April",
        "May",
        "June",
        "July",
        "August",
        "September",
        "October",
        "November",
        "December"
      ]
    },
    daysInMonth: [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31],
    firstDayOfWeek: 0,
    ordinal: function(O) {
      var _ = O % 100;
      if (_ > 3 && _ < 21)
        return "th";
      switch (_ % 10) {
        case 1:
          return "st";
        case 2:
          return "nd";
        case 3:
          return "rd";
        default:
          return "th";
      }
    },
    rangeSeparator: " to ",
    weekAbbreviation: "Wk",
    scrollTitle: "Scroll to increment",
    toggleTitle: "Click to toggle",
    amPM: ["AM", "PM"],
    yearAriaLabel: "Year",
    monthAriaLabel: "Month",
    hourAriaLabel: "Hour",
    minuteAriaLabel: "Minute",
    time_24hr: !1
  }, pad$4 = function(O, _) {
    return _ === void 0 && (_ = 2), ("000" + O).slice(_ * -1);
  }, int = function(O) {
    return O === !0 ? 1 : 0;
  };
  function debounce$4(O, _) {
    var H;
    return function() {
      var V = this, q = arguments;
      clearTimeout(H), H = setTimeout(function() {
        return O.apply(V, q);
      }, _);
    };
  }
  var arrayify = function(O) {
    return O instanceof Array ? O : [O];
  };
  function toggleClass(O, _, H) {
    if (H === !0)
      return O.classList.add(_);
    O.classList.remove(_);
  }
  function createElement(O, _, H) {
    var V = window.document.createElement(O);
    return _ = _ || "", H = H || "", V.className = _, H !== void 0 && (V.textContent = H), V;
  }
  function clearNode(O) {
    for (; O.firstChild; )
      O.removeChild(O.firstChild);
  }
  function findParent(O, _) {
    if (_(O))
      return O;
    if (O.parentNode)
      return findParent(O.parentNode, _);
  }
  function createNumberInput(O, _) {
    var H = createElement("div", "numInputWrapper"), V = createElement("input", "numInput " + O), q = createElement("span", "arrowUp"), G = createElement("span", "arrowDown");
    if (navigator.userAgent.indexOf("MSIE 9.0") === -1 ? V.type = "number" : (V.type = "text", V.pattern = "\\d*"), _ !== void 0)
      for (var J in _)
        V.setAttribute(J, _[J]);
    return H.appendChild(V), H.appendChild(q), H.appendChild(G), H;
  }
  function getEventTarget(O) {
    try {
      if (typeof O.composedPath == "function") {
        var _ = O.composedPath();
        return _[0];
      }
      return O.target;
    } catch {
      return O.target;
    }
  }
  var doNothing = function() {
  }, monthToStr = function(O, _, H) {
    return H.months[_ ? "shorthand" : "longhand"][O];
  }, revFormat = {
    D: doNothing,
    F: function(O, _, H) {
      O.setMonth(H.months.longhand.indexOf(_));
    },
    G: function(O, _) {
      O.setHours((O.getHours() >= 12 ? 12 : 0) + parseFloat(_));
    },
    H: function(O, _) {
      O.setHours(parseFloat(_));
    },
    J: function(O, _) {
      O.setDate(parseFloat(_));
    },
    K: function(O, _, H) {
      O.setHours(O.getHours() % 12 + 12 * int(new RegExp(H.amPM[1], "i").test(_)));
    },
    M: function(O, _, H) {
      O.setMonth(H.months.shorthand.indexOf(_));
    },
    S: function(O, _) {
      O.setSeconds(parseFloat(_));
    },
    U: function(O, _) {
      return new Date(parseFloat(_) * 1e3);
    },
    W: function(O, _, H) {
      var V = parseInt(_), q = new Date(O.getFullYear(), 0, 2 + (V - 1) * 7, 0, 0, 0, 0);
      return q.setDate(q.getDate() - q.getDay() + H.firstDayOfWeek), q;
    },
    Y: function(O, _) {
      O.setFullYear(parseFloat(_));
    },
    Z: function(O, _) {
      return new Date(_);
    },
    d: function(O, _) {
      O.setDate(parseFloat(_));
    },
    h: function(O, _) {
      O.setHours((O.getHours() >= 12 ? 12 : 0) + parseFloat(_));
    },
    i: function(O, _) {
      O.setMinutes(parseFloat(_));
    },
    j: function(O, _) {
      O.setDate(parseFloat(_));
    },
    l: doNothing,
    m: function(O, _) {
      O.setMonth(parseFloat(_) - 1);
    },
    n: function(O, _) {
      O.setMonth(parseFloat(_) - 1);
    },
    s: function(O, _) {
      O.setSeconds(parseFloat(_));
    },
    u: function(O, _) {
      return new Date(parseFloat(_));
    },
    w: doNothing,
    y: function(O, _) {
      O.setFullYear(2e3 + parseFloat(_));
    }
  }, tokenRegex = {
    D: "",
    F: "",
    G: "(\\d\\d|\\d)",
    H: "(\\d\\d|\\d)",
    J: "(\\d\\d|\\d)\\w+",
    K: "",
    M: "",
    S: "(\\d\\d|\\d)",
    U: "(.+)",
    W: "(\\d\\d|\\d)",
    Y: "(\\d{4})",
    Z: "(.+)",
    d: "(\\d\\d|\\d)",
    h: "(\\d\\d|\\d)",
    i: "(\\d\\d|\\d)",
    j: "(\\d\\d|\\d)",
    l: "",
    m: "(\\d\\d|\\d)",
    n: "(\\d\\d|\\d)",
    s: "(\\d\\d|\\d)",
    u: "(.+)",
    w: "(\\d\\d|\\d)",
    y: "(\\d{2})"
  }, formats$2 = {
    Z: function(O) {
      return O.toISOString();
    },
    D: function(O, _, H) {
      return _.weekdays.shorthand[formats$2.w(O, _, H)];
    },
    F: function(O, _, H) {
      return monthToStr(formats$2.n(O, _, H) - 1, !1, _);
    },
    G: function(O, _, H) {
      return pad$4(formats$2.h(O, _, H));
    },
    H: function(O) {
      return pad$4(O.getHours());
    },
    J: function(O, _) {
      return _.ordinal !== void 0 ? O.getDate() + _.ordinal(O.getDate()) : O.getDate();
    },
    K: function(O, _) {
      return _.amPM[int(O.getHours() > 11)];
    },
    M: function(O, _) {
      return monthToStr(O.getMonth(), !0, _);
    },
    S: function(O) {
      return pad$4(O.getSeconds());
    },
    U: function(O) {
      return O.getTime() / 1e3;
    },
    W: function(O, _, H) {
      return H.getWeek(O);
    },
    Y: function(O) {
      return pad$4(O.getFullYear(), 4);
    },
    d: function(O) {
      return pad$4(O.getDate());
    },
    h: function(O) {
      return O.getHours() % 12 ? O.getHours() % 12 : 12;
    },
    i: function(O) {
      return pad$4(O.getMinutes());
    },
    j: function(O) {
      return O.getDate();
    },
    l: function(O, _) {
      return _.weekdays.longhand[O.getDay()];
    },
    m: function(O) {
      return pad$4(O.getMonth() + 1);
    },
    n: function(O) {
      return O.getMonth() + 1;
    },
    s: function(O) {
      return O.getSeconds();
    },
    u: function(O) {
      return O.getTime();
    },
    w: function(O) {
      return O.getDay();
    },
    y: function(O) {
      return String(O.getFullYear()).substring(2);
    }
  }, createDateFormatter = function(O) {
    var _ = O.config, H = _ === void 0 ? defaults$5 : _, V = O.l10n, q = V === void 0 ? english : V, G = O.isMobile, J = G === void 0 ? !1 : G;
    return function(Z, te, ne) {
      var ee = ne || q;
      return H.formatDate !== void 0 && !J ? H.formatDate(Z, te, ee) : te.split("").map(function(ae, oe, se) {
        return formats$2[ae] && se[oe - 1] !== "\\" ? formats$2[ae](Z, ee, H) : ae !== "\\" ? ae : "";
      }).join("");
    };
  }, createDateParser = function(O) {
    var _ = O.config, H = _ === void 0 ? defaults$5 : _, V = O.l10n, q = V === void 0 ? english : V;
    return function(G, J, Z, te) {
      if (!(G !== 0 && !G)) {
        var ne = te || q, ee, ae = G;
        if (G instanceof Date)
          ee = new Date(G.getTime());
        else if (typeof G != "string" && G.toFixed !== void 0)
          ee = new Date(G);
        else if (typeof G == "string") {
          var oe = J || (H || defaults$5).dateFormat, se = String(G).trim();
          if (se === "today")
            ee = /* @__PURE__ */ new Date(), Z = !0;
          else if (H && H.parseDate)
            ee = H.parseDate(G, oe);
          else if (/Z$/.test(se) || /GMT$/.test(se))
            ee = new Date(G);
          else {
            for (var ce = void 0, le = [], ue = 0, de = 0, fe = ""; ue < oe.length; ue++) {
              var he = oe[ue], me = he === "\\", pe = oe[ue - 1] === "\\" || me;
              if (tokenRegex[he] && !pe) {
                fe += tokenRegex[he];
                var ye = new RegExp(fe).exec(G);
                ye && (ce = !0) && le[he !== "Y" ? "push" : "unshift"]({
                  fn: revFormat[he],
                  val: ye[++de]
                });
              } else me || (fe += ".");
            }
            ee = !H || !H.noCalendar ? new Date((/* @__PURE__ */ new Date()).getFullYear(), 0, 1, 0, 0, 0, 0) : new Date((/* @__PURE__ */ new Date()).setHours(0, 0, 0, 0)), le.forEach(function(ge) {
              var ve = ge.fn, be = ge.val;
              return ee = ve(ee, be, ne) || ee;
            }), ee = ce ? ee : void 0;
          }
        }
        if (!(ee instanceof Date && !isNaN(ee.getTime()))) {
          H.errorHandler(new Error("Invalid date provided: " + ae));
          return;
        }
        return Z === !0 && ee.setHours(0, 0, 0, 0), ee;
      }
    };
  };
  function compareDates(O, _, H) {
    return H === void 0 && (H = !0), H !== !1 ? new Date(O.getTime()).setHours(0, 0, 0, 0) - new Date(_.getTime()).setHours(0, 0, 0, 0) : O.getTime() - _.getTime();
  }
  var isBetween = function(O, _, H) {
    return O > Math.min(_, H) && O < Math.max(_, H);
  }, calculateSecondsSinceMidnight = function(O, _, H) {
    return O * 3600 + _ * 60 + H;
  }, parseSeconds$1 = function(O) {
    var _ = Math.floor(O / 3600), H = (O - _ * 3600) / 60;
    return [_, H, O - _ * 3600 - H * 60];
  }, duration = {
    DAY: 864e5
  };
  function getDefaultHours(O) {
    var _ = O.defaultHour, H = O.defaultMinute, V = O.defaultSeconds;
    if (O.minDate !== void 0) {
      var q = O.minDate.getHours(), G = O.minDate.getMinutes(), J = O.minDate.getSeconds();
      _ < q && (_ = q), _ === q && H < G && (H = G), _ === q && H === G && V < J && (V = O.minDate.getSeconds());
    }
    if (O.maxDate !== void 0) {
      var Z = O.maxDate.getHours(), te = O.maxDate.getMinutes();
      _ = Math.min(_, Z), _ === Z && (H = Math.min(te, H)), _ === Z && H === te && (V = O.maxDate.getSeconds());
    }
    return { hours: _, minutes: H, seconds: V };
  }
  typeof Object.assign != "function" && (Object.assign = function(O) {
    for (var _ = [], H = 1; H < arguments.length; H++)
      _[H - 1] = arguments[H];
    if (!O)
      throw TypeError("Cannot convert undefined or null to object");
    for (var V = function(Z) {
      Z && Object.keys(Z).forEach(function(te) {
        return O[te] = Z[te];
      });
    }, q = 0, G = _; q < G.length; q++) {
      var J = G[q];
      V(J);
    }
    return O;
  });
  var __assign$1 = function() {
    return __assign$1 = Object.assign || function(O) {
      for (var _, H = 1, V = arguments.length; H < V; H++) {
        _ = arguments[H];
        for (var q in _) Object.prototype.hasOwnProperty.call(_, q) && (O[q] = _[q]);
      }
      return O;
    }, __assign$1.apply(this, arguments);
  }, __spreadArrays$1 = function() {
    for (var O = 0, _ = 0, H = arguments.length; _ < H; _++) O += arguments[_].length;
    for (var V = Array(O), q = 0, _ = 0; _ < H; _++)
      for (var G = arguments[_], J = 0, Z = G.length; J < Z; J++, q++)
        V[q] = G[J];
    return V;
  }, DEBOUNCED_CHANGE_MS = 300;
  function FlatpickrInstance(O, _) {
    var H = {
      config: __assign$1(__assign$1({}, defaults$5), flatpickr.defaultConfig),
      l10n: english
    };
    H.parseDate = createDateParser({ config: H.config, l10n: H.l10n }), H._handlers = [], H.pluginElements = [], H.loadedPlugins = [], H._bind = le, H._setHoursFromDate = oe, H._positionCalendar = yt, H.changeMonth = Pe, H.changeYear = Ge, H.clear = $e, H.close = qe, H.onMouseOver = et, H._createElement = createElement, H.createDay = ye, H.destroy = Xe, H.isEnabled = ze, H.jumpToDate = fe, H.updateValue = rn, H.open = Je, H.redraw = Rt, H.set = Xt, H.setDate = Gt, H.toggle = Vt;
    function V() {
      H.utils = {
        getDaysInMonth: function(Ht, jt) {
          return Ht === void 0 && (Ht = H.currentMonth), jt === void 0 && (jt = H.currentYear), Ht === 1 && (jt % 4 === 0 && jt % 100 !== 0 || jt % 400 === 0) ? 29 : H.l10n.daysInMonth[Ht];
        }
      };
    }
    function q() {
      H.element = H.input = O, H.isOpen = !1, st(), bt(), It(), Tt(), V(), H.isMobile || pe(), de(), (H.selectedDates.length || H.config.noCalendar) && (H.config.enableTime && oe(H.config.noCalendar ? H.latestSelectedDateObj : void 0), rn(!1)), Z();
      var Ht = /^((?!chrome|android).)*safari/i.test(navigator.userAgent);
      !H.isMobile && Ht && yt(), Yt("onReady");
    }
    function G() {
      var Ht;
      return ((Ht = H.calendarContainer) === null || Ht === void 0 ? void 0 : Ht.getRootNode()).activeElement || document.activeElement;
    }
    function J(Ht) {
      return Ht.bind(H);
    }
    function Z() {
      var Ht = H.config;
      Ht.weekNumbers === !1 && Ht.showMonths === 1 || Ht.noCalendar !== !0 && window.requestAnimationFrame(function() {
        if (H.calendarContainer !== void 0 && (H.calendarContainer.style.visibility = "hidden", H.calendarContainer.style.display = "block"), H.daysContainer !== void 0) {
          var jt = (H.days.offsetWidth + 1) * Ht.showMonths;
          H.daysContainer.style.width = jt + "px", H.calendarContainer.style.width = jt + (H.weekWrapper !== void 0 ? H.weekWrapper.offsetWidth : 0) + "px", H.calendarContainer.style.removeProperty("visibility"), H.calendarContainer.style.removeProperty("display");
        }
      });
    }
    function te(Ht) {
      if (H.selectedDates.length === 0) {
        var jt = H.config.minDate === void 0 || compareDates(/* @__PURE__ */ new Date(), H.config.minDate) >= 0 ? /* @__PURE__ */ new Date() : new Date(H.config.minDate.getTime()), cn = getDefaultHours(H.config);
        jt.setHours(cn.hours, cn.minutes, cn.seconds, jt.getMilliseconds()), H.selectedDates = [jt], H.latestSelectedDateObj = jt;
      }
      Ht !== void 0 && Ht.type !== "blur" && dn(Ht);
      var wn = H._input.value;
      ae(), rn(), H._input.value !== wn && H._debouncedChange();
    }
    function ne(Ht, jt) {
      return Ht % 12 + 12 * int(jt === H.l10n.amPM[1]);
    }
    function ee(Ht) {
      switch (Ht % 24) {
        case 0:
        case 12:
          return 12;
        default:
          return Ht % 12;
      }
    }
    function ae() {
      if (!(H.hourElement === void 0 || H.minuteElement === void 0)) {
        var Ht = (parseInt(H.hourElement.value.slice(-2), 10) || 0) % 24, jt = (parseInt(H.minuteElement.value, 10) || 0) % 60, cn = H.secondElement !== void 0 ? (parseInt(H.secondElement.value, 10) || 0) % 60 : 0;
        H.amPM !== void 0 && (Ht = ne(Ht, H.amPM.textContent));
        var wn = H.config.minTime !== void 0 || H.config.minDate && H.minDateHasTime && H.latestSelectedDateObj && compareDates(H.latestSelectedDateObj, H.config.minDate, !0) === 0, Rn = H.config.maxTime !== void 0 || H.config.maxDate && H.maxDateHasTime && H.latestSelectedDateObj && compareDates(H.latestSelectedDateObj, H.config.maxDate, !0) === 0;
        if (H.config.maxTime !== void 0 && H.config.minTime !== void 0 && H.config.minTime > H.config.maxTime) {
          var Pn = calculateSecondsSinceMidnight(H.config.minTime.getHours(), H.config.minTime.getMinutes(), H.config.minTime.getSeconds()), Ie = calculateSecondsSinceMidnight(H.config.maxTime.getHours(), H.config.maxTime.getMinutes(), H.config.maxTime.getSeconds()), rt = calculateSecondsSinceMidnight(Ht, jt, cn);
          if (rt > Ie && rt < Pn) {
            var ft = parseSeconds$1(Pn);
            Ht = ft[0], jt = ft[1], cn = ft[2];
          }
        } else {
          if (Rn) {
            var pt = H.config.maxTime !== void 0 ? H.config.maxTime : H.config.maxDate;
            Ht = Math.min(Ht, pt.getHours()), Ht === pt.getHours() && (jt = Math.min(jt, pt.getMinutes())), jt === pt.getMinutes() && (cn = Math.min(cn, pt.getSeconds()));
          }
          if (wn) {
            var wt = H.config.minTime !== void 0 ? H.config.minTime : H.config.minDate;
            Ht = Math.max(Ht, wt.getHours()), Ht === wt.getHours() && jt < wt.getMinutes() && (jt = wt.getMinutes()), jt === wt.getMinutes() && (cn = Math.max(cn, wt.getSeconds()));
          }
        }
        se(Ht, jt, cn);
      }
    }
    function oe(Ht) {
      var jt = Ht || H.latestSelectedDateObj;
      jt && jt instanceof Date && se(jt.getHours(), jt.getMinutes(), jt.getSeconds());
    }
    function se(Ht, jt, cn) {
      H.latestSelectedDateObj !== void 0 && H.latestSelectedDateObj.setHours(Ht % 24, jt, cn || 0, 0), !(!H.hourElement || !H.minuteElement || H.isMobile) && (H.hourElement.value = pad$4(H.config.time_24hr ? Ht : (12 + Ht) % 12 + 12 * int(Ht % 12 === 0)), H.minuteElement.value = pad$4(jt), H.amPM !== void 0 && (H.amPM.textContent = H.l10n.amPM[int(Ht >= 12)]), H.secondElement !== void 0 && (H.secondElement.value = pad$4(cn)));
    }
    function ce(Ht) {
      var jt = getEventTarget(Ht), cn = parseInt(jt.value) + (Ht.delta || 0);
      (cn / 1e3 > 1 || Ht.key === "Enter" && !/[^\d]/.test(cn.toString())) && Ge(cn);
    }
    function le(Ht, jt, cn, wn) {
      if (jt instanceof Array)
        return jt.forEach(function(Rn) {
          return le(Ht, Rn, cn, wn);
        });
      if (Ht instanceof Array)
        return Ht.forEach(function(Rn) {
          return le(Rn, jt, cn, wn);
        });
      Ht.addEventListener(jt, cn, wn), H._handlers.push({
        remove: function() {
          return Ht.removeEventListener(jt, cn, wn);
        }
      });
    }
    function ue() {
      Yt("onChange");
    }
    function de() {
      if (H.config.wrap && ["open", "close", "toggle", "clear"].forEach(function(cn) {
        Array.prototype.forEach.call(H.element.querySelectorAll("[data-" + cn + "]"), function(wn) {
          return le(wn, "click", H[cn]);
        });
      }), H.isMobile) {
        Kt();
        return;
      }
      var Ht = debounce$4(dt, 50);
      if (H._debouncedChange = debounce$4(ue, DEBOUNCED_CHANGE_MS), H.daysContainer && !/iPhone|iPad|iPod/i.test(navigator.userAgent) && le(H.daysContainer, "mouseover", function(cn) {
        H.config.mode === "range" && et(getEventTarget(cn));
      }), le(H._input, "keydown", ct), H.calendarContainer !== void 0 && le(H.calendarContainer, "keydown", ct), !H.config.inline && !H.config.static && le(window, "resize", Ht), window.ontouchstart !== void 0 ? le(window.document, "touchstart", Ze) : le(window.document, "mousedown", Ze), le(window.document, "focus", Ze, { capture: !0 }), H.config.clickOpens === !0 && (le(H._input, "focus", H.open), le(H._input, "click", H.open)), H.daysContainer !== void 0 && (le(H.monthNav, "click", vn), le(H.monthNav, ["keyup", "increment"], ce), le(H.daysContainer, "click", $t)), H.timeContainer !== void 0 && H.minuteElement !== void 0 && H.hourElement !== void 0) {
        var jt = function(cn) {
          return getEventTarget(cn).select();
        };
        le(H.timeContainer, ["increment"], te), le(H.timeContainer, "blur", te, { capture: !0 }), le(H.timeContainer, "click", he), le([H.hourElement, H.minuteElement], ["focus", "click"], jt), H.secondElement !== void 0 && le(H.secondElement, "focus", function() {
          return H.secondElement && H.secondElement.select();
        }), H.amPM !== void 0 && le(H.amPM, "click", function(cn) {
          te(cn);
        });
      }
      H.config.allowInput && le(H._input, "blur", at);
    }
    function fe(Ht, jt) {
      var cn = Ht !== void 0 ? H.parseDate(Ht) : H.latestSelectedDateObj || (H.config.minDate && H.config.minDate > H.now ? H.config.minDate : H.config.maxDate && H.config.maxDate < H.now ? H.config.maxDate : H.now), wn = H.currentYear, Rn = H.currentMonth;
      try {
        cn !== void 0 && (H.currentYear = cn.getFullYear(), H.currentMonth = cn.getMonth());
      } catch (Pn) {
        Pn.message = "Invalid date supplied: " + cn, H.config.errorHandler(Pn);
      }
      jt && H.currentYear !== wn && (Yt("onYearChange"), Ee()), jt && (H.currentYear !== wn || H.currentMonth !== Rn) && Yt("onMonthChange"), H.redraw();
    }
    function he(Ht) {
      var jt = getEventTarget(Ht);
      ~jt.className.indexOf("arrow") && me(Ht, jt.classList.contains("arrowUp") ? 1 : -1);
    }
    function me(Ht, jt, cn) {
      var wn = Ht && getEventTarget(Ht), Rn = cn || wn && wn.parentNode && wn.parentNode.firstChild, Pn = sn("increment");
      Pn.delta = jt, Rn && Rn.dispatchEvent(Pn);
    }
    function pe() {
      var Ht = window.document.createDocumentFragment();
      if (H.calendarContainer = createElement("div", "flatpickr-calendar"), H.calendarContainer.tabIndex = -1, !H.config.noCalendar) {
        if (Ht.appendChild(Qe()), H.innerContainer = createElement("div", "flatpickr-innerContainer"), H.config.weekNumbers) {
          var jt = De(), cn = jt.weekWrapper, wn = jt.weekNumbers;
          H.innerContainer.appendChild(cn), H.weekNumbers = wn, H.weekWrapper = cn;
        }
        H.rContainer = createElement("div", "flatpickr-rContainer"), H.rContainer.appendChild(Me()), H.daysContainer || (H.daysContainer = createElement("div", "flatpickr-days"), H.daysContainer.tabIndex = -1), we(), H.rContainer.appendChild(H.daysContainer), H.innerContainer.appendChild(H.rContainer), Ht.appendChild(H.innerContainer);
      }
      H.config.enableTime && Ht.appendChild(ke()), toggleClass(H.calendarContainer, "rangeMode", H.config.mode === "range"), toggleClass(H.calendarContainer, "animate", H.config.animate === !0), toggleClass(H.calendarContainer, "multiMonth", H.config.showMonths > 1), H.calendarContainer.appendChild(Ht);
      var Rn = H.config.appendTo !== void 0 && H.config.appendTo.nodeType !== void 0;
      if ((H.config.inline || H.config.static) && (H.calendarContainer.classList.add(H.config.inline ? "inline" : "static"), H.config.inline && (!Rn && H.element.parentNode ? H.element.parentNode.insertBefore(H.calendarContainer, H._input.nextSibling) : H.config.appendTo !== void 0 && H.config.appendTo.appendChild(H.calendarContainer)), H.config.static)) {
        var Pn = createElement("div", "flatpickr-wrapper");
        H.element.parentNode && H.element.parentNode.insertBefore(Pn, H.element), Pn.appendChild(H.element), H.altInput && Pn.appendChild(H.altInput), Pn.appendChild(H.calendarContainer);
      }
      !H.config.static && !H.config.inline && (H.config.appendTo !== void 0 ? H.config.appendTo : window.document.body).appendChild(H.calendarContainer);
    }
    function ye(Ht, jt, cn, wn) {
      var Rn = ze(jt, !0), Pn = createElement("span", Ht, jt.getDate().toString());
      return Pn.dateObj = jt, Pn.$i = wn, Pn.setAttribute("aria-label", H.formatDate(jt, H.config.ariaDateFormat)), Ht.indexOf("hidden") === -1 && compareDates(jt, H.now) === 0 && (H.todayDateElem = Pn, Pn.classList.add("today"), Pn.setAttribute("aria-current", "date")), Rn ? (Pn.tabIndex = -1, hn(jt) && (Pn.classList.add("selected"), H.selectedDateElem = Pn, H.config.mode === "range" && (toggleClass(Pn, "startRange", H.selectedDates[0] && compareDates(jt, H.selectedDates[0], !0) === 0), toggleClass(Pn, "endRange", H.selectedDates[1] && compareDates(jt, H.selectedDates[1], !0) === 0), Ht === "nextMonthDay" && Pn.classList.add("inRange")))) : Pn.classList.add("flatpickr-disabled"), H.config.mode === "range" && Qt(jt) && !hn(jt) && Pn.classList.add("inRange"), H.weekNumbers && H.config.showMonths === 1 && Ht !== "prevMonthDay" && wn % 7 === 6 && H.weekNumbers.insertAdjacentHTML("beforeend", "<span class='flatpickr-day'>" + H.config.getWeek(jt) + "</span>"), Yt("onDayCreate", Pn), Pn;
    }
    function ge(Ht) {
      Ht.focus(), H.config.mode === "range" && et(Ht);
    }
    function ve(Ht) {
      for (var jt = Ht > 0 ? 0 : H.config.showMonths - 1, cn = Ht > 0 ? H.config.showMonths : -1, wn = jt; wn != cn; wn += Ht)
        for (var Rn = H.daysContainer.children[wn], Pn = Ht > 0 ? 0 : Rn.children.length - 1, Ie = Ht > 0 ? Rn.children.length : -1, rt = Pn; rt != Ie; rt += Ht) {
          var ft = Rn.children[rt];
          if (ft.className.indexOf("hidden") === -1 && ze(ft.dateObj))
            return ft;
        }
    }
    function be(Ht, jt) {
      for (var cn = Ht.className.indexOf("Month") === -1 ? Ht.dateObj.getMonth() : H.currentMonth, wn = jt > 0 ? H.config.showMonths : -1, Rn = jt > 0 ? 1 : -1, Pn = cn - H.currentMonth; Pn != wn; Pn += Rn)
        for (var Ie = H.daysContainer.children[Pn], rt = cn - H.currentMonth === Pn ? Ht.$i + jt : jt < 0 ? Ie.children.length - 1 : 0, ft = Ie.children.length, pt = rt; pt >= 0 && pt < ft && pt != (jt > 0 ? ft : -1); pt += Rn) {
          var wt = Ie.children[pt];
          if (wt.className.indexOf("hidden") === -1 && ze(wt.dateObj) && Math.abs(Ht.$i - pt) >= Math.abs(jt))
            return ge(wt);
        }
      H.changeMonth(Rn), Ce(ve(Rn), 0);
    }
    function Ce(Ht, jt) {
      var cn = G(), wn = We(cn || document.body), Rn = Ht !== void 0 ? Ht : wn ? cn : H.selectedDateElem !== void 0 && We(H.selectedDateElem) ? H.selectedDateElem : H.todayDateElem !== void 0 && We(H.todayDateElem) ? H.todayDateElem : ve(jt > 0 ? 1 : -1);
      Rn === void 0 ? H._input.focus() : wn ? be(Rn, jt) : ge(Rn);
    }
    function Se(Ht, jt) {
      for (var cn = (new Date(Ht, jt, 1).getDay() - H.l10n.firstDayOfWeek + 7) % 7, wn = H.utils.getDaysInMonth((jt - 1 + 12) % 12, Ht), Rn = H.utils.getDaysInMonth(jt, Ht), Pn = window.document.createDocumentFragment(), Ie = H.config.showMonths > 1, rt = Ie ? "prevMonthDay hidden" : "prevMonthDay", ft = Ie ? "nextMonthDay hidden" : "nextMonthDay", pt = wn + 1 - cn, wt = 0; pt <= wn; pt++, wt++)
        Pn.appendChild(ye("flatpickr-day " + rt, new Date(Ht, jt - 1, pt), pt, wt));
      for (pt = 1; pt <= Rn; pt++, wt++)
        Pn.appendChild(ye("flatpickr-day", new Date(Ht, jt, pt), pt, wt));
      for (var Bt = Rn + 1; Bt <= 42 - cn && (H.config.showMonths === 1 || wt % 7 !== 0); Bt++, wt++)
        Pn.appendChild(ye("flatpickr-day " + ft, new Date(Ht, jt + 1, Bt % Rn), Bt, wt));
      var At = createElement("div", "dayContainer");
      return At.appendChild(Pn), At;
    }
    function we() {
      if (H.daysContainer !== void 0) {
        clearNode(H.daysContainer), H.weekNumbers && clearNode(H.weekNumbers);
        for (var Ht = document.createDocumentFragment(), jt = 0; jt < H.config.showMonths; jt++) {
          var cn = new Date(H.currentYear, H.currentMonth, 1);
          cn.setMonth(H.currentMonth + jt), Ht.appendChild(Se(cn.getFullYear(), cn.getMonth()));
        }
        H.daysContainer.appendChild(Ht), H.days = H.daysContainer.firstChild, H.config.mode === "range" && H.selectedDates.length === 1 && et();
      }
    }
    function Ee() {
      if (!(H.config.showMonths > 1 || H.config.monthSelectorType !== "dropdown")) {
        var Ht = function(wn) {
          return H.config.minDate !== void 0 && H.currentYear === H.config.minDate.getFullYear() && wn < H.config.minDate.getMonth() ? !1 : !(H.config.maxDate !== void 0 && H.currentYear === H.config.maxDate.getFullYear() && wn > H.config.maxDate.getMonth());
        };
        H.monthsDropdownContainer.tabIndex = -1, H.monthsDropdownContainer.innerHTML = "";
        for (var jt = 0; jt < 12; jt++)
          if (Ht(jt)) {
            var cn = createElement("option", "flatpickr-monthDropdown-month");
            cn.value = new Date(H.currentYear, jt).getMonth().toString(), cn.textContent = monthToStr(jt, H.config.shorthandCurrentMonth, H.l10n), cn.tabIndex = -1, H.currentMonth === jt && (cn.selected = !0), H.monthsDropdownContainer.appendChild(cn);
          }
      }
    }
    function xe() {
      var Ht = createElement("div", "flatpickr-month"), jt = window.document.createDocumentFragment(), cn;
      H.config.showMonths > 1 || H.config.monthSelectorType === "static" ? cn = createElement("span", "cur-month") : (H.monthsDropdownContainer = createElement("select", "flatpickr-monthDropdown-months"), H.monthsDropdownContainer.setAttribute("aria-label", H.l10n.monthAriaLabel), le(H.monthsDropdownContainer, "change", function(Ie) {
        var rt = getEventTarget(Ie), ft = parseInt(rt.value, 10);
        H.changeMonth(ft - H.currentMonth), Yt("onMonthChange");
      }), Ee(), cn = H.monthsDropdownContainer);
      var wn = createNumberInput("cur-year", { tabindex: "-1" }), Rn = wn.getElementsByTagName("input")[0];
      Rn.setAttribute("aria-label", H.l10n.yearAriaLabel), H.config.minDate && Rn.setAttribute("min", H.config.minDate.getFullYear().toString()), H.config.maxDate && (Rn.setAttribute("max", H.config.maxDate.getFullYear().toString()), Rn.disabled = !!H.config.minDate && H.config.minDate.getFullYear() === H.config.maxDate.getFullYear());
      var Pn = createElement("div", "flatpickr-current-month");
      return Pn.appendChild(cn), Pn.appendChild(wn), jt.appendChild(Pn), Ht.appendChild(jt), {
        container: Ht,
        yearElement: Rn,
        monthElement: cn
      };
    }
    function Le() {
      clearNode(H.monthNav), H.monthNav.appendChild(H.prevMonthNav), H.config.showMonths && (H.yearElements = [], H.monthElements = []);
      for (var Ht = H.config.showMonths; Ht--; ) {
        var jt = xe();
        H.yearElements.push(jt.yearElement), H.monthElements.push(jt.monthElement), H.monthNav.appendChild(jt.container);
      }
      H.monthNav.appendChild(H.nextMonthNav);
    }
    function Qe() {
      return H.monthNav = createElement("div", "flatpickr-months"), H.yearElements = [], H.monthElements = [], H.prevMonthNav = createElement("span", "flatpickr-prev-month"), H.prevMonthNav.innerHTML = H.config.prevArrow, H.nextMonthNav = createElement("span", "flatpickr-next-month"), H.nextMonthNav.innerHTML = H.config.nextArrow, Le(), Object.defineProperty(H, "_hidePrevMonthArrow", {
        get: function() {
          return H.__hidePrevMonthArrow;
        },
        set: function(Ht) {
          H.__hidePrevMonthArrow !== Ht && (toggleClass(H.prevMonthNav, "flatpickr-disabled", Ht), H.__hidePrevMonthArrow = Ht);
        }
      }), Object.defineProperty(H, "_hideNextMonthArrow", {
        get: function() {
          return H.__hideNextMonthArrow;
        },
        set: function(Ht) {
          H.__hideNextMonthArrow !== Ht && (toggleClass(H.nextMonthNav, "flatpickr-disabled", Ht), H.__hideNextMonthArrow = Ht);
        }
      }), H.currentYearElement = H.yearElements[0], Ut(), H.monthNav;
    }
    function ke() {
      H.calendarContainer.classList.add("hasTime"), H.config.noCalendar && H.calendarContainer.classList.add("noCalendar");
      var Ht = getDefaultHours(H.config);
      H.timeContainer = createElement("div", "flatpickr-time"), H.timeContainer.tabIndex = -1;
      var jt = createElement("span", "flatpickr-time-separator", ":"), cn = createNumberInput("flatpickr-hour", {
        "aria-label": H.l10n.hourAriaLabel
      });
      H.hourElement = cn.getElementsByTagName("input")[0];
      var wn = createNumberInput("flatpickr-minute", {
        "aria-label": H.l10n.minuteAriaLabel
      });
      if (H.minuteElement = wn.getElementsByTagName("input")[0], H.hourElement.tabIndex = H.minuteElement.tabIndex = -1, H.hourElement.value = pad$4(H.latestSelectedDateObj ? H.latestSelectedDateObj.getHours() : H.config.time_24hr ? Ht.hours : ee(Ht.hours)), H.minuteElement.value = pad$4(H.latestSelectedDateObj ? H.latestSelectedDateObj.getMinutes() : Ht.minutes), H.hourElement.setAttribute("step", H.config.hourIncrement.toString()), H.minuteElement.setAttribute("step", H.config.minuteIncrement.toString()), H.hourElement.setAttribute("min", H.config.time_24hr ? "0" : "1"), H.hourElement.setAttribute("max", H.config.time_24hr ? "23" : "12"), H.hourElement.setAttribute("maxlength", "2"), H.minuteElement.setAttribute("min", "0"), H.minuteElement.setAttribute("max", "59"), H.minuteElement.setAttribute("maxlength", "2"), H.timeContainer.appendChild(cn), H.timeContainer.appendChild(jt), H.timeContainer.appendChild(wn), H.config.time_24hr && H.timeContainer.classList.add("time24hr"), H.config.enableSeconds) {
        H.timeContainer.classList.add("hasSeconds");
        var Rn = createNumberInput("flatpickr-second");
        H.secondElement = Rn.getElementsByTagName("input")[0], H.secondElement.value = pad$4(H.latestSelectedDateObj ? H.latestSelectedDateObj.getSeconds() : Ht.seconds), H.secondElement.setAttribute("step", H.minuteElement.getAttribute("step")), H.secondElement.setAttribute("min", "0"), H.secondElement.setAttribute("max", "59"), H.secondElement.setAttribute("maxlength", "2"), H.timeContainer.appendChild(createElement("span", "flatpickr-time-separator", ":")), H.timeContainer.appendChild(Rn);
      }
      return H.config.time_24hr || (H.amPM = createElement("span", "flatpickr-am-pm", H.l10n.amPM[int((H.latestSelectedDateObj ? H.hourElement.value : H.config.defaultHour) > 11)]), H.amPM.title = H.l10n.toggleTitle, H.amPM.tabIndex = -1, H.timeContainer.appendChild(H.amPM)), H.timeContainer;
    }
    function Me() {
      H.weekdayContainer ? clearNode(H.weekdayContainer) : H.weekdayContainer = createElement("div", "flatpickr-weekdays");
      for (var Ht = H.config.showMonths; Ht--; ) {
        var jt = createElement("div", "flatpickr-weekdaycontainer");
        H.weekdayContainer.appendChild(jt);
      }
      return Re(), H.weekdayContainer;
    }
    function Re() {
      if (H.weekdayContainer) {
        var Ht = H.l10n.firstDayOfWeek, jt = __spreadArrays$1(H.l10n.weekdays.shorthand);
        Ht > 0 && Ht < jt.length && (jt = __spreadArrays$1(jt.splice(Ht, jt.length), jt.splice(0, Ht)));
        for (var cn = H.config.showMonths; cn--; )
          H.weekdayContainer.children[cn].innerHTML = `
      <span class='flatpickr-weekday'>
        ` + jt.join("</span><span class='flatpickr-weekday'>") + `
      </span>
      `;
      }
    }
    function De() {
      H.calendarContainer.classList.add("hasWeeks");
      var Ht = createElement("div", "flatpickr-weekwrapper");
      Ht.appendChild(createElement("span", "flatpickr-weekday", H.l10n.weekAbbreviation));
      var jt = createElement("div", "flatpickr-weeks");
      return Ht.appendChild(jt), {
        weekWrapper: Ht,
        weekNumbers: jt
      };
    }
    function Pe(Ht, jt) {
      jt === void 0 && (jt = !0);
      var cn = jt ? Ht : Ht - H.currentMonth;
      cn < 0 && H._hidePrevMonthArrow === !0 || cn > 0 && H._hideNextMonthArrow === !0 || (H.currentMonth += cn, (H.currentMonth < 0 || H.currentMonth > 11) && (H.currentYear += H.currentMonth > 11 ? 1 : -1, H.currentMonth = (H.currentMonth + 12) % 12, Yt("onYearChange"), Ee()), we(), Yt("onMonthChange"), Ut());
    }
    function $e(Ht, jt) {
      if (Ht === void 0 && (Ht = !0), jt === void 0 && (jt = !0), H.input.value = "", H.altInput !== void 0 && (H.altInput.value = ""), H.mobileInput !== void 0 && (H.mobileInput.value = ""), H.selectedDates = [], H.latestSelectedDateObj = void 0, jt === !0 && (H.currentYear = H._initialDate.getFullYear(), H.currentMonth = H._initialDate.getMonth()), H.config.enableTime === !0) {
        var cn = getDefaultHours(H.config), wn = cn.hours, Rn = cn.minutes, Pn = cn.seconds;
        se(wn, Rn, Pn);
      }
      H.redraw(), Ht && Yt("onChange");
    }
    function qe() {
      H.isOpen = !1, H.isMobile || (H.calendarContainer !== void 0 && H.calendarContainer.classList.remove("open"), H._input !== void 0 && H._input.classList.remove("active")), Yt("onClose");
    }
    function Xe() {
      H.config !== void 0 && Yt("onDestroy");
      for (var Ht = H._handlers.length; Ht--; )
        H._handlers[Ht].remove();
      if (H._handlers = [], H.mobileInput)
        H.mobileInput.parentNode && H.mobileInput.parentNode.removeChild(H.mobileInput), H.mobileInput = void 0;
      else if (H.calendarContainer && H.calendarContainer.parentNode)
        if (H.config.static && H.calendarContainer.parentNode) {
          var jt = H.calendarContainer.parentNode;
          if (jt.lastChild && jt.removeChild(jt.lastChild), jt.parentNode) {
            for (; jt.firstChild; )
              jt.parentNode.insertBefore(jt.firstChild, jt);
            jt.parentNode.removeChild(jt);
          }
        } else
          H.calendarContainer.parentNode.removeChild(H.calendarContainer);
      H.altInput && (H.input.type = "text", H.altInput.parentNode && H.altInput.parentNode.removeChild(H.altInput), delete H.altInput), H.input && (H.input.type = H.input._type, H.input.classList.remove("flatpickr-input"), H.input.removeAttribute("readonly")), [
        "_showTimeInput",
        "latestSelectedDateObj",
        "_hideNextMonthArrow",
        "_hidePrevMonthArrow",
        "__hideNextMonthArrow",
        "__hidePrevMonthArrow",
        "isMobile",
        "isOpen",
        "selectedDateElem",
        "minDateHasTime",
        "maxDateHasTime",
        "days",
        "daysContainer",
        "_input",
        "_positionElement",
        "innerContainer",
        "rContainer",
        "monthNav",
        "todayDateElem",
        "calendarContainer",
        "weekdayContainer",
        "prevMonthNav",
        "nextMonthNav",
        "monthsDropdownContainer",
        "currentMonthElement",
        "currentYearElement",
        "navigationCurrentMonth",
        "selectedDateElem",
        "config"
      ].forEach(function(cn) {
        try {
          delete H[cn];
        } catch {
        }
      });
    }
    function Ve(Ht) {
      return H.calendarContainer.contains(Ht);
    }
    function Ze(Ht) {
      if (H.isOpen && !H.config.inline) {
        var jt = getEventTarget(Ht), cn = Ve(jt), wn = jt === H.input || jt === H.altInput || H.element.contains(jt) || Ht.path && Ht.path.indexOf && (~Ht.path.indexOf(H.input) || ~Ht.path.indexOf(H.altInput)), Rn = !wn && !cn && !Ve(Ht.relatedTarget), Pn = !H.config.ignoredFocusElements.some(function(Ie) {
          return Ie.contains(jt);
        });
        Rn && Pn && (H.config.allowInput && H.setDate(H._input.value, !1, H.config.altInput ? H.config.altFormat : H.config.dateFormat), H.timeContainer !== void 0 && H.minuteElement !== void 0 && H.hourElement !== void 0 && H.input.value !== "" && H.input.value !== void 0 && te(), H.close(), H.config && H.config.mode === "range" && H.selectedDates.length === 1 && H.clear(!1));
      }
    }
    function Ge(Ht) {
      if (!(!Ht || H.config.minDate && Ht < H.config.minDate.getFullYear() || H.config.maxDate && Ht > H.config.maxDate.getFullYear())) {
        var jt = Ht, cn = H.currentYear !== jt;
        H.currentYear = jt || H.currentYear, H.config.maxDate && H.currentYear === H.config.maxDate.getFullYear() ? H.currentMonth = Math.min(H.config.maxDate.getMonth(), H.currentMonth) : H.config.minDate && H.currentYear === H.config.minDate.getFullYear() && (H.currentMonth = Math.max(H.config.minDate.getMonth(), H.currentMonth)), cn && (H.redraw(), Yt("onYearChange"), Ee());
      }
    }
    function ze(Ht, jt) {
      var cn;
      jt === void 0 && (jt = !0);
      var wn = H.parseDate(Ht, void 0, jt);
      if (H.config.minDate && wn && compareDates(wn, H.config.minDate, jt !== void 0 ? jt : !H.minDateHasTime) < 0 || H.config.maxDate && wn && compareDates(wn, H.config.maxDate, jt !== void 0 ? jt : !H.maxDateHasTime) > 0)
        return !1;
      if (!H.config.enable && H.config.disable.length === 0)
        return !0;
      if (wn === void 0)
        return !1;
      for (var Rn = !!H.config.enable, Pn = (cn = H.config.enable) !== null && cn !== void 0 ? cn : H.config.disable, Ie = 0, rt = void 0; Ie < Pn.length; Ie++) {
        if (rt = Pn[Ie], typeof rt == "function" && rt(wn))
          return Rn;
        if (rt instanceof Date && wn !== void 0 && rt.getTime() === wn.getTime())
          return Rn;
        if (typeof rt == "string") {
          var ft = H.parseDate(rt, void 0, !0);
          return ft && ft.getTime() === wn.getTime() ? Rn : !Rn;
        } else if (typeof rt == "object" && wn !== void 0 && rt.from && rt.to && wn.getTime() >= rt.from.getTime() && wn.getTime() <= rt.to.getTime())
          return Rn;
      }
      return !Rn;
    }
    function We(Ht) {
      return H.daysContainer !== void 0 ? Ht.className.indexOf("hidden") === -1 && Ht.className.indexOf("flatpickr-disabled") === -1 && H.daysContainer.contains(Ht) : !1;
    }
    function at(Ht) {
      var jt = Ht.target === H._input, cn = H._input.value.trimEnd() !== Wt();
      jt && cn && !(Ht.relatedTarget && Ve(Ht.relatedTarget)) && H.setDate(H._input.value, !0, Ht.target === H.altInput ? H.config.altFormat : H.config.dateFormat);
    }
    function ct(Ht) {
      var jt = getEventTarget(Ht), cn = H.config.wrap ? O.contains(jt) : jt === H._input, wn = H.config.allowInput, Rn = H.isOpen && (!wn || !cn), Pn = H.config.inline && cn && !wn;
      if (Ht.keyCode === 13 && cn) {
        if (wn)
          return H.setDate(H._input.value, !0, jt === H.altInput ? H.config.altFormat : H.config.dateFormat), H.close(), jt.blur();
        H.open();
      } else if (Ve(jt) || Rn || Pn) {
        var Ie = !!H.timeContainer && H.timeContainer.contains(jt);
        switch (Ht.keyCode) {
          case 13:
            Ie ? (Ht.preventDefault(), te(), vt()) : $t(Ht);
            break;
          case 27:
            Ht.preventDefault(), vt();
            break;
          case 8:
          case 46:
            cn && !H.config.allowInput && (Ht.preventDefault(), H.clear());
            break;
          case 37:
          case 39:
            if (!Ie && !cn) {
              Ht.preventDefault();
              var rt = G();
              if (H.daysContainer !== void 0 && (wn === !1 || rt && We(rt))) {
                var ft = Ht.keyCode === 39 ? 1 : -1;
                Ht.ctrlKey ? (Ht.stopPropagation(), Pe(ft), Ce(ve(1), 0)) : Ce(void 0, ft);
              }
            } else H.hourElement && H.hourElement.focus();
            break;
          case 38:
          case 40:
            Ht.preventDefault();
            var pt = Ht.keyCode === 40 ? 1 : -1;
            H.daysContainer && jt.$i !== void 0 || jt === H.input || jt === H.altInput ? Ht.ctrlKey ? (Ht.stopPropagation(), Ge(H.currentYear - pt), Ce(ve(1), 0)) : Ie || Ce(void 0, pt * 7) : jt === H.currentYearElement ? Ge(H.currentYear - pt) : H.config.enableTime && (!Ie && H.hourElement && H.hourElement.focus(), te(Ht), H._debouncedChange());
            break;
          case 9:
            if (Ie) {
              var wt = [
                H.hourElement,
                H.minuteElement,
                H.secondElement,
                H.amPM
              ].concat(H.pluginElements).filter(function(Dt) {
                return Dt;
              }), Bt = wt.indexOf(jt);
              if (Bt !== -1) {
                var At = wt[Bt + (Ht.shiftKey ? -1 : 1)];
                Ht.preventDefault(), (At || H._input).focus();
              }
            } else !H.config.noCalendar && H.daysContainer && H.daysContainer.contains(jt) && Ht.shiftKey && (Ht.preventDefault(), H._input.focus());
            break;
        }
      }
      if (H.amPM !== void 0 && jt === H.amPM)
        switch (Ht.key) {
          case H.l10n.amPM[0].charAt(0):
          case H.l10n.amPM[0].charAt(0).toLowerCase():
            H.amPM.textContent = H.l10n.amPM[0], ae(), rn();
            break;
          case H.l10n.amPM[1].charAt(0):
          case H.l10n.amPM[1].charAt(0).toLowerCase():
            H.amPM.textContent = H.l10n.amPM[1], ae(), rn();
            break;
        }
      (cn || Ve(jt)) && Yt("onKeyDown", Ht);
    }
    function et(Ht, jt) {
      if (jt === void 0 && (jt = "flatpickr-day"), !(H.selectedDates.length !== 1 || Ht && (!Ht.classList.contains(jt) || Ht.classList.contains("flatpickr-disabled")))) {
        for (var cn = Ht ? Ht.dateObj.getTime() : H.days.firstElementChild.dateObj.getTime(), wn = H.parseDate(H.selectedDates[0], void 0, !0).getTime(), Rn = Math.min(cn, H.selectedDates[0].getTime()), Pn = Math.max(cn, H.selectedDates[0].getTime()), Ie = !1, rt = 0, ft = 0, pt = Rn; pt < Pn; pt += duration.DAY)
          ze(new Date(pt), !0) || (Ie = Ie || pt > Rn && pt < Pn, pt < wn && (!rt || pt > rt) ? rt = pt : pt > wn && (!ft || pt < ft) && (ft = pt));
        var wt = Array.from(H.rContainer.querySelectorAll("*:nth-child(-n+" + H.config.showMonths + ") > ." + jt));
        wt.forEach(function(Bt) {
          var At = Bt.dateObj, Dt = At.getTime(), Ot = rt > 0 && Dt < rt || ft > 0 && Dt > ft;
          if (Ot) {
            Bt.classList.add("notAllowed"), ["inRange", "startRange", "endRange"].forEach(function(un) {
              Bt.classList.remove(un);
            });
            return;
          } else if (Ie && !Ot)
            return;
          ["startRange", "inRange", "endRange", "notAllowed"].forEach(function(un) {
            Bt.classList.remove(un);
          }), Ht !== void 0 && (Ht.classList.add(cn <= H.selectedDates[0].getTime() ? "startRange" : "endRange"), wn < cn && Dt === wn ? Bt.classList.add("startRange") : wn > cn && Dt === wn && Bt.classList.add("endRange"), Dt >= rt && (ft === 0 || Dt <= ft) && isBetween(Dt, wn, cn) && Bt.classList.add("inRange"));
        });
      }
    }
    function dt() {
      H.isOpen && !H.config.static && !H.config.inline && yt();
    }
    function Je(Ht, jt) {
      if (jt === void 0 && (jt = H._positionElement), H.isMobile === !0) {
        if (Ht) {
          Ht.preventDefault();
          var cn = getEventTarget(Ht);
          cn && cn.blur();
        }
        H.mobileInput !== void 0 && (H.mobileInput.focus(), H.mobileInput.click()), Yt("onOpen");
        return;
      } else if (H._input.disabled || H.config.inline)
        return;
      var wn = H.isOpen;
      H.isOpen = !0, wn || (H.calendarContainer.classList.add("open"), H._input.classList.add("active"), Yt("onOpen"), yt(jt)), H.config.enableTime === !0 && H.config.noCalendar === !0 && H.config.allowInput === !1 && (Ht === void 0 || !H.timeContainer.contains(Ht.relatedTarget)) && setTimeout(function() {
        return H.hourElement.select();
      }, 50);
    }
    function it(Ht) {
      return function(jt) {
        var cn = H.config["_" + Ht + "Date"] = H.parseDate(jt, H.config.dateFormat), wn = H.config["_" + (Ht === "min" ? "max" : "min") + "Date"];
        cn !== void 0 && (H[Ht === "min" ? "minDateHasTime" : "maxDateHasTime"] = cn.getHours() > 0 || cn.getMinutes() > 0 || cn.getSeconds() > 0), H.selectedDates && (H.selectedDates = H.selectedDates.filter(function(Rn) {
          return ze(Rn);
        }), !H.selectedDates.length && Ht === "min" && oe(cn), rn()), H.daysContainer && (Rt(), cn !== void 0 ? H.currentYearElement[Ht] = cn.getFullYear().toString() : H.currentYearElement.removeAttribute(Ht), H.currentYearElement.disabled = !!wn && cn !== void 0 && wn.getFullYear() === cn.getFullYear());
      };
    }
    function st() {
      var Ht = [
        "wrap",
        "weekNumbers",
        "allowInput",
        "allowInvalidPreload",
        "clickOpens",
        "time_24hr",
        "enableTime",
        "noCalendar",
        "altInput",
        "shorthandCurrentMonth",
        "inline",
        "static",
        "enableSeconds",
        "disableMobile"
      ], jt = __assign$1(__assign$1({}, JSON.parse(JSON.stringify(O.dataset || {}))), _), cn = {};
      H.config.parseDate = jt.parseDate, H.config.formatDate = jt.formatDate, Object.defineProperty(H.config, "enable", {
        get: function() {
          return H.config._enable;
        },
        set: function(wt) {
          H.config._enable = Ft(wt);
        }
      }), Object.defineProperty(H.config, "disable", {
        get: function() {
          return H.config._disable;
        },
        set: function(wt) {
          H.config._disable = Ft(wt);
        }
      });
      var wn = jt.mode === "time";
      if (!jt.dateFormat && (jt.enableTime || wn)) {
        var Rn = flatpickr.defaultConfig.dateFormat || defaults$5.dateFormat;
        cn.dateFormat = jt.noCalendar || wn ? "H:i" + (jt.enableSeconds ? ":S" : "") : Rn + " H:i" + (jt.enableSeconds ? ":S" : "");
      }
      if (jt.altInput && (jt.enableTime || wn) && !jt.altFormat) {
        var Pn = flatpickr.defaultConfig.altFormat || defaults$5.altFormat;
        cn.altFormat = jt.noCalendar || wn ? "h:i" + (jt.enableSeconds ? ":S K" : " K") : Pn + (" h:i" + (jt.enableSeconds ? ":S" : "") + " K");
      }
      Object.defineProperty(H.config, "minDate", {
        get: function() {
          return H.config._minDate;
        },
        set: it("min")
      }), Object.defineProperty(H.config, "maxDate", {
        get: function() {
          return H.config._maxDate;
        },
        set: it("max")
      });
      var Ie = function(wt) {
        return function(Bt) {
          H.config[wt === "min" ? "_minTime" : "_maxTime"] = H.parseDate(Bt, "H:i:S");
        };
      };
      Object.defineProperty(H.config, "minTime", {
        get: function() {
          return H.config._minTime;
        },
        set: Ie("min")
      }), Object.defineProperty(H.config, "maxTime", {
        get: function() {
          return H.config._maxTime;
        },
        set: Ie("max")
      }), jt.mode === "time" && (H.config.noCalendar = !0, H.config.enableTime = !0), Object.assign(H.config, cn, jt);
      for (var rt = 0; rt < Ht.length; rt++)
        H.config[Ht[rt]] = H.config[Ht[rt]] === !0 || H.config[Ht[rt]] === "true";
      HOOKS.filter(function(wt) {
        return H.config[wt] !== void 0;
      }).forEach(function(wt) {
        H.config[wt] = arrayify(H.config[wt] || []).map(J);
      }), H.isMobile = !H.config.disableMobile && !H.config.inline && H.config.mode === "single" && !H.config.disable.length && !H.config.enable && !H.config.weekNumbers && /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
      for (var rt = 0; rt < H.config.plugins.length; rt++) {
        var ft = H.config.plugins[rt](H) || {};
        for (var pt in ft)
          HOOKS.indexOf(pt) > -1 ? H.config[pt] = arrayify(ft[pt]).map(J).concat(H.config[pt]) : typeof jt[pt] > "u" && (H.config[pt] = ft[pt]);
      }
      jt.altInputClass || (H.config.altInputClass = gt().className + " " + H.config.altInputClass), Yt("onParseConfig");
    }
    function gt() {
      return H.config.wrap ? O.querySelector("[data-input]") : O;
    }
    function bt() {
      typeof H.config.locale != "object" && typeof flatpickr.l10ns[H.config.locale] > "u" && H.config.errorHandler(new Error("flatpickr: invalid locale " + H.config.locale)), H.l10n = __assign$1(__assign$1({}, flatpickr.l10ns.default), typeof H.config.locale == "object" ? H.config.locale : H.config.locale !== "default" ? flatpickr.l10ns[H.config.locale] : void 0), tokenRegex.D = "(" + H.l10n.weekdays.shorthand.join("|") + ")", tokenRegex.l = "(" + H.l10n.weekdays.longhand.join("|") + ")", tokenRegex.M = "(" + H.l10n.months.shorthand.join("|") + ")", tokenRegex.F = "(" + H.l10n.months.longhand.join("|") + ")", tokenRegex.K = "(" + H.l10n.amPM[0] + "|" + H.l10n.amPM[1] + "|" + H.l10n.amPM[0].toLowerCase() + "|" + H.l10n.amPM[1].toLowerCase() + ")";
      var Ht = __assign$1(__assign$1({}, _), JSON.parse(JSON.stringify(O.dataset || {})));
      Ht.time_24hr === void 0 && flatpickr.defaultConfig.time_24hr === void 0 && (H.config.time_24hr = H.l10n.time_24hr), H.formatDate = createDateFormatter(H), H.parseDate = createDateParser({ config: H.config, l10n: H.l10n });
    }
    function yt(Ht) {
      if (typeof H.config.position == "function")
        return void H.config.position(H, Ht);
      if (H.calendarContainer !== void 0) {
        Yt("onPreCalendarPosition");
        var jt = Ht || H._positionElement, cn = Array.prototype.reduce.call(H.calendarContainer.children, function(In, kn) {
          return In + kn.offsetHeight;
        }, 0), wn = H.calendarContainer.offsetWidth, Rn = H.config.position.split(" "), Pn = Rn[0], Ie = Rn.length > 1 ? Rn[1] : null, rt = jt.getBoundingClientRect(), ft = window.innerHeight - rt.bottom, pt = Pn === "above" || Pn !== "below" && ft < cn && rt.top > cn, wt = window.pageYOffset + rt.top + (pt ? -cn - 2 : jt.offsetHeight + 2);
        if (toggleClass(H.calendarContainer, "arrowTop", !pt), toggleClass(H.calendarContainer, "arrowBottom", pt), !H.config.inline) {
          var Bt = window.pageXOffset + rt.left, At = !1, Dt = !1;
          Ie === "center" ? (Bt -= (wn - rt.width) / 2, At = !0) : Ie === "right" && (Bt -= wn - rt.width, Dt = !0), toggleClass(H.calendarContainer, "arrowLeft", !At && !Dt), toggleClass(H.calendarContainer, "arrowCenter", At), toggleClass(H.calendarContainer, "arrowRight", Dt);
          var Ot = window.document.body.offsetWidth - (window.pageXOffset + rt.right), un = Bt + wn > window.document.body.offsetWidth, yn = Ot + wn > window.document.body.offsetWidth;
          if (toggleClass(H.calendarContainer, "rightMost", un), !H.config.static)
            if (H.calendarContainer.style.top = wt + "px", !un)
              H.calendarContainer.style.left = Bt + "px", H.calendarContainer.style.right = "auto";
            else if (!yn)
              H.calendarContainer.style.left = "auto", H.calendarContainer.style.right = Ot + "px";
            else {
              var _n = St();
              if (_n === void 0)
                return;
              var gn = window.document.body.offsetWidth, bn = Math.max(0, gn / 2 - wn / 2), an = ".flatpickr-calendar.centerMost:before", Zt = ".flatpickr-calendar.centerMost:after", Mn = _n.cssRules.length, Ln = "{left:" + rt.left + "px;right:auto;}";
              toggleClass(H.calendarContainer, "rightMost", !1), toggleClass(H.calendarContainer, "centerMost", !0), _n.insertRule(an + "," + Zt + Ln, Mn), H.calendarContainer.style.left = bn + "px", H.calendarContainer.style.right = "auto";
            }
        }
      }
    }
    function St() {
      for (var Ht = null, jt = 0; jt < document.styleSheets.length; jt++) {
        var cn = document.styleSheets[jt];
        if (cn.cssRules) {
          try {
            cn.cssRules;
          } catch {
            continue;
          }
          Ht = cn;
          break;
        }
      }
      return Ht ?? Lt();
    }
    function Lt() {
      var Ht = document.createElement("style");
      return document.head.appendChild(Ht), Ht.sheet;
    }
    function Rt() {
      H.config.noCalendar || H.isMobile || (Ee(), Ut(), we());
    }
    function vt() {
      H._input.focus(), window.navigator.userAgent.indexOf("MSIE") !== -1 || navigator.msMaxTouchPoints !== void 0 ? setTimeout(H.close, 0) : H.close();
    }
    function $t(Ht) {
      Ht.preventDefault(), Ht.stopPropagation();
      var jt = function(wt) {
        return wt.classList && wt.classList.contains("flatpickr-day") && !wt.classList.contains("flatpickr-disabled") && !wt.classList.contains("notAllowed");
      }, cn = findParent(getEventTarget(Ht), jt);
      if (cn !== void 0) {
        var wn = cn, Rn = H.latestSelectedDateObj = new Date(wn.dateObj.getTime()), Pn = (Rn.getMonth() < H.currentMonth || Rn.getMonth() > H.currentMonth + H.config.showMonths - 1) && H.config.mode !== "range";
        if (H.selectedDateElem = wn, H.config.mode === "single")
          H.selectedDates = [Rn];
        else if (H.config.mode === "multiple") {
          var Ie = hn(Rn);
          Ie ? H.selectedDates.splice(parseInt(Ie), 1) : H.selectedDates.push(Rn);
        } else H.config.mode === "range" && (H.selectedDates.length === 2 && H.clear(!1, !1), H.latestSelectedDateObj = Rn, H.selectedDates.push(Rn), compareDates(Rn, H.selectedDates[0], !0) !== 0 && H.selectedDates.sort(function(wt, Bt) {
          return wt.getTime() - Bt.getTime();
        }));
        if (ae(), Pn) {
          var rt = H.currentYear !== Rn.getFullYear();
          H.currentYear = Rn.getFullYear(), H.currentMonth = Rn.getMonth(), rt && (Yt("onYearChange"), Ee()), Yt("onMonthChange");
        }
        if (Ut(), we(), rn(), !Pn && H.config.mode !== "range" && H.config.showMonths === 1 ? ge(wn) : H.selectedDateElem !== void 0 && H.hourElement === void 0 && H.selectedDateElem && H.selectedDateElem.focus(), H.hourElement !== void 0 && H.hourElement !== void 0 && H.hourElement.focus(), H.config.closeOnSelect) {
          var ft = H.config.mode === "single" && !H.config.enableTime, pt = H.config.mode === "range" && H.selectedDates.length === 2 && !H.config.enableTime;
          (ft || pt) && vt();
        }
        ue();
      }
    }
    var Pt = {
      locale: [bt, Re],
      showMonths: [Le, Z, Me],
      minDate: [fe],
      maxDate: [fe],
      positionElement: [Ct],
      clickOpens: [
        function() {
          H.config.clickOpens === !0 ? (le(H._input, "focus", H.open), le(H._input, "click", H.open)) : (H._input.removeEventListener("focus", H.open), H._input.removeEventListener("click", H.open));
        }
      ]
    };
    function Xt(Ht, jt) {
      if (Ht !== null && typeof Ht == "object") {
        Object.assign(H.config, Ht);
        for (var cn in Ht)
          Pt[cn] !== void 0 && Pt[cn].forEach(function(wn) {
            return wn();
          });
      } else
        H.config[Ht] = jt, Pt[Ht] !== void 0 ? Pt[Ht].forEach(function(wn) {
          return wn();
        }) : HOOKS.indexOf(Ht) > -1 && (H.config[Ht] = arrayify(jt));
      H.redraw(), rn(!0);
    }
    function Nt(Ht, jt) {
      var cn = [];
      if (Ht instanceof Array)
        cn = Ht.map(function(wn) {
          return H.parseDate(wn, jt);
        });
      else if (Ht instanceof Date || typeof Ht == "number")
        cn = [H.parseDate(Ht, jt)];
      else if (typeof Ht == "string")
        switch (H.config.mode) {
          case "single":
          case "time":
            cn = [H.parseDate(Ht, jt)];
            break;
          case "multiple":
            cn = Ht.split(H.config.conjunction).map(function(wn) {
              return H.parseDate(wn, jt);
            });
            break;
          case "range":
            cn = Ht.split(H.l10n.rangeSeparator).map(function(wn) {
              return H.parseDate(wn, jt);
            });
            break;
        }
      else
        H.config.errorHandler(new Error("Invalid date supplied: " + JSON.stringify(Ht)));
      H.selectedDates = H.config.allowInvalidPreload ? cn : cn.filter(function(wn) {
        return wn instanceof Date && ze(wn, !1);
      }), H.config.mode === "range" && H.selectedDates.sort(function(wn, Rn) {
        return wn.getTime() - Rn.getTime();
      });
    }
    function Gt(Ht, jt, cn) {
      if (jt === void 0 && (jt = !1), cn === void 0 && (cn = H.config.dateFormat), Ht !== 0 && !Ht || Ht instanceof Array && Ht.length === 0)
        return H.clear(jt);
      Nt(Ht, cn), H.latestSelectedDateObj = H.selectedDates[H.selectedDates.length - 1], H.redraw(), fe(void 0, jt), oe(), H.selectedDates.length === 0 && H.clear(!1), rn(jt), jt && Yt("onChange");
    }
    function Ft(Ht) {
      return Ht.slice().map(function(jt) {
        return typeof jt == "string" || typeof jt == "number" || jt instanceof Date ? H.parseDate(jt, void 0, !0) : jt && typeof jt == "object" && jt.from && jt.to ? {
          from: H.parseDate(jt.from, void 0),
          to: H.parseDate(jt.to, void 0)
        } : jt;
      }).filter(function(jt) {
        return jt;
      });
    }
    function Tt() {
      H.selectedDates = [], H.now = H.parseDate(H.config.now) || /* @__PURE__ */ new Date();
      var Ht = H.config.defaultDate || ((H.input.nodeName === "INPUT" || H.input.nodeName === "TEXTAREA") && H.input.placeholder && H.input.value === H.input.placeholder ? null : H.input.value);
      Ht && Nt(Ht, H.config.dateFormat), H._initialDate = H.selectedDates.length > 0 ? H.selectedDates[0] : H.config.minDate && H.config.minDate.getTime() > H.now.getTime() ? H.config.minDate : H.config.maxDate && H.config.maxDate.getTime() < H.now.getTime() ? H.config.maxDate : H.now, H.currentYear = H._initialDate.getFullYear(), H.currentMonth = H._initialDate.getMonth(), H.selectedDates.length > 0 && (H.latestSelectedDateObj = H.selectedDates[0]), H.config.minTime !== void 0 && (H.config.minTime = H.parseDate(H.config.minTime, "H:i")), H.config.maxTime !== void 0 && (H.config.maxTime = H.parseDate(H.config.maxTime, "H:i")), H.minDateHasTime = !!H.config.minDate && (H.config.minDate.getHours() > 0 || H.config.minDate.getMinutes() > 0 || H.config.minDate.getSeconds() > 0), H.maxDateHasTime = !!H.config.maxDate && (H.config.maxDate.getHours() > 0 || H.config.maxDate.getMinutes() > 0 || H.config.maxDate.getSeconds() > 0);
    }
    function It() {
      if (H.input = gt(), !H.input) {
        H.config.errorHandler(new Error("Invalid input element specified"));
        return;
      }
      H.input._type = H.input.type, H.input.type = "text", H.input.classList.add("flatpickr-input"), H._input = H.input, H.config.altInput && (H.altInput = createElement(H.input.nodeName, H.config.altInputClass), H._input = H.altInput, H.altInput.placeholder = H.input.placeholder, H.altInput.disabled = H.input.disabled, H.altInput.required = H.input.required, H.altInput.tabIndex = H.input.tabIndex, H.altInput.type = "text", H.input.setAttribute("type", "hidden"), !H.config.static && H.input.parentNode && H.input.parentNode.insertBefore(H.altInput, H.input.nextSibling)), H.config.allowInput || H._input.setAttribute("readonly", "readonly"), Ct();
    }
    function Ct() {
      H._positionElement = H.config.positionElement || H._input;
    }
    function Kt() {
      var Ht = H.config.enableTime ? H.config.noCalendar ? "time" : "datetime-local" : "date";
      H.mobileInput = createElement("input", H.input.className + " flatpickr-mobile"), H.mobileInput.tabIndex = 1, H.mobileInput.type = Ht, H.mobileInput.disabled = H.input.disabled, H.mobileInput.required = H.input.required, H.mobileInput.placeholder = H.input.placeholder, H.mobileFormatStr = Ht === "datetime-local" ? "Y-m-d\\TH:i:S" : Ht === "date" ? "Y-m-d" : "H:i:S", H.selectedDates.length > 0 && (H.mobileInput.defaultValue = H.mobileInput.value = H.formatDate(H.selectedDates[0], H.mobileFormatStr)), H.config.minDate && (H.mobileInput.min = H.formatDate(H.config.minDate, "Y-m-d")), H.config.maxDate && (H.mobileInput.max = H.formatDate(H.config.maxDate, "Y-m-d")), H.input.getAttribute("step") && (H.mobileInput.step = String(H.input.getAttribute("step"))), H.input.type = "hidden", H.altInput !== void 0 && (H.altInput.type = "hidden");
      try {
        H.input.parentNode && H.input.parentNode.insertBefore(H.mobileInput, H.input.nextSibling);
      } catch {
      }
      le(H.mobileInput, "change", function(jt) {
        H.setDate(getEventTarget(jt).value, !1, H.mobileFormatStr), Yt("onChange"), Yt("onClose");
      });
    }
    function Vt(Ht) {
      if (H.isOpen === !0)
        return H.close();
      H.open(Ht);
    }
    function Yt(Ht, jt) {
      if (H.config !== void 0) {
        var cn = H.config[Ht];
        if (cn !== void 0 && cn.length > 0)
          for (var wn = 0; cn[wn] && wn < cn.length; wn++)
            cn[wn](H.selectedDates, H.input.value, H, jt);
        Ht === "onChange" && (H.input.dispatchEvent(sn("change")), H.input.dispatchEvent(sn("input")));
      }
    }
    function sn(Ht) {
      var jt = document.createEvent("Event");
      return jt.initEvent(Ht, !0, !0), jt;
    }
    function hn(Ht) {
      for (var jt = 0; jt < H.selectedDates.length; jt++) {
        var cn = H.selectedDates[jt];
        if (cn instanceof Date && compareDates(cn, Ht) === 0)
          return "" + jt;
      }
      return !1;
    }
    function Qt(Ht) {
      return H.config.mode !== "range" || H.selectedDates.length < 2 ? !1 : compareDates(Ht, H.selectedDates[0]) >= 0 && compareDates(Ht, H.selectedDates[1]) <= 0;
    }
    function Ut() {
      H.config.noCalendar || H.isMobile || !H.monthNav || (H.yearElements.forEach(function(Ht, jt) {
        var cn = new Date(H.currentYear, H.currentMonth, 1);
        cn.setMonth(H.currentMonth + jt), H.config.showMonths > 1 || H.config.monthSelectorType === "static" ? H.monthElements[jt].textContent = monthToStr(cn.getMonth(), H.config.shorthandCurrentMonth, H.l10n) + " " : H.monthsDropdownContainer.value = cn.getMonth().toString(), Ht.value = cn.getFullYear().toString();
      }), H._hidePrevMonthArrow = H.config.minDate !== void 0 && (H.currentYear === H.config.minDate.getFullYear() ? H.currentMonth <= H.config.minDate.getMonth() : H.currentYear < H.config.minDate.getFullYear()), H._hideNextMonthArrow = H.config.maxDate !== void 0 && (H.currentYear === H.config.maxDate.getFullYear() ? H.currentMonth + 1 > H.config.maxDate.getMonth() : H.currentYear > H.config.maxDate.getFullYear()));
    }
    function Wt(Ht) {
      var jt = Ht || (H.config.altInput ? H.config.altFormat : H.config.dateFormat);
      return H.selectedDates.map(function(cn) {
        return H.formatDate(cn, jt);
      }).filter(function(cn, wn, Rn) {
        return H.config.mode !== "range" || H.config.enableTime || Rn.indexOf(cn) === wn;
      }).join(H.config.mode !== "range" ? H.config.conjunction : H.l10n.rangeSeparator);
    }
    function rn(Ht) {
      Ht === void 0 && (Ht = !0), H.mobileInput !== void 0 && H.mobileFormatStr && (H.mobileInput.value = H.latestSelectedDateObj !== void 0 ? H.formatDate(H.latestSelectedDateObj, H.mobileFormatStr) : ""), H.input.value = Wt(H.config.dateFormat), H.altInput !== void 0 && (H.altInput.value = Wt(H.config.altFormat)), Ht !== !1 && Yt("onValueUpdate");
    }
    function vn(Ht) {
      var jt = getEventTarget(Ht), cn = H.prevMonthNav.contains(jt), wn = H.nextMonthNav.contains(jt);
      cn || wn ? Pe(cn ? -1 : 1) : H.yearElements.indexOf(jt) >= 0 ? jt.select() : jt.classList.contains("arrowUp") ? H.changeYear(H.currentYear + 1) : jt.classList.contains("arrowDown") && H.changeYear(H.currentYear - 1);
    }
    function dn(Ht) {
      Ht.preventDefault();
      var jt = Ht.type === "keydown", cn = getEventTarget(Ht), wn = cn;
      H.amPM !== void 0 && cn === H.amPM && (H.amPM.textContent = H.l10n.amPM[int(H.amPM.textContent === H.l10n.amPM[0])]);
      var Rn = parseFloat(wn.getAttribute("min")), Pn = parseFloat(wn.getAttribute("max")), Ie = parseFloat(wn.getAttribute("step")), rt = parseInt(wn.value, 10), ft = Ht.delta || (jt ? Ht.which === 38 ? 1 : -1 : 0), pt = rt + Ie * ft;
      if (typeof wn.value < "u" && wn.value.length === 2) {
        var wt = wn === H.hourElement, Bt = wn === H.minuteElement;
        pt < Rn ? (pt = Pn + pt + int(!wt) + (int(wt) && int(!H.amPM)), Bt && me(void 0, -1, H.hourElement)) : pt > Pn && (pt = wn === H.hourElement ? pt - Pn - int(!H.amPM) : Rn, Bt && me(void 0, 1, H.hourElement)), H.amPM && wt && (Ie === 1 ? pt + rt === 23 : Math.abs(pt - rt) > Ie) && (H.amPM.textContent = H.l10n.amPM[int(H.amPM.textContent === H.l10n.amPM[0])]), wn.value = pad$4(pt);
      }
    }
    return q(), H;
  }
  function _flatpickr(O, _) {
    for (var H = Array.prototype.slice.call(O).filter(function(J) {
      return J instanceof HTMLElement;
    }), V = [], q = 0; q < H.length; q++) {
      var G = H[q];
      try {
        if (G.getAttribute("data-fp-omit") !== null)
          continue;
        G._flatpickr !== void 0 && (G._flatpickr.destroy(), G._flatpickr = void 0), G._flatpickr = FlatpickrInstance(G, _ || {}), V.push(G._flatpickr);
      } catch (J) {
        console.error(J);
      }
    }
    return V.length === 1 ? V[0] : V;
  }
  typeof HTMLElement < "u" && typeof HTMLCollection < "u" && typeof NodeList < "u" && (HTMLCollection.prototype.flatpickr = NodeList.prototype.flatpickr = function(O) {
    return _flatpickr(this, O);
  }, HTMLElement.prototype.flatpickr = function(O) {
    return _flatpickr([this], O);
  });
  var flatpickr = function(O, _) {
    return typeof O == "string" ? _flatpickr(window.document.querySelectorAll(O), _) : O instanceof Node ? _flatpickr([O], _) : _flatpickr(O, _);
  };
  flatpickr.defaultConfig = {};
  flatpickr.l10ns = {
    en: __assign$1({}, english),
    default: __assign$1({}, english)
  };
  flatpickr.localize = function(O) {
    flatpickr.l10ns.default = __assign$1(__assign$1({}, flatpickr.l10ns.default), O);
  };
  flatpickr.setDefaults = function(O) {
    flatpickr.defaultConfig = __assign$1(__assign$1({}, flatpickr.defaultConfig), O);
  };
  flatpickr.parseDate = createDateParser({});
  flatpickr.formatDate = createDateFormatter({});
  flatpickr.compareDates = compareDates;
  typeof jQuery < "u" && typeof jQuery.fn < "u" && (jQuery.fn.flatpickr = function(O) {
    return _flatpickr(this, O);
  });
  Date.prototype.fp_incr = function(O) {
    return new Date(this.getFullYear(), this.getMonth(), this.getDate() + (typeof O == "string" ? parseInt(O, 10) : O));
  };
  typeof window < "u" && (window.flatpickr = flatpickr);
  const flatpickr_css = `.flatpickr-calendar{background:transparent;opacity:0;display:none;text-align:center;visibility:hidden;padding:0;-webkit-animation:none;animation:none;direction:ltr;border:0;font-size:14px;line-height:24px;border-radius:5px;position:absolute;width:307.875px;-webkit-box-sizing:border-box;box-sizing:border-box;-ms-touch-action:manipulation;touch-action:manipulation;background:#fff;-webkit-box-shadow:1px 0 0 #e6e6e6, -1px 0 0 #e6e6e6, 0 1px 0 #e6e6e6, 0 -1px 0 #e6e6e6, 0 3px 13px rgba(0, 0, 0, 0.08);box-shadow:1px 0 0 #e6e6e6, -1px 0 0 #e6e6e6, 0 1px 0 #e6e6e6, 0 -1px 0 #e6e6e6, 0 3px 13px rgba(0, 0, 0, 0.08);}.flatpickr-calendar.open,.flatpickr-calendar.inline{opacity:1;max-height:640px;visibility:visible;}.flatpickr-calendar.open{display:inline-block;z-index:99999;}.flatpickr-calendar.animate.open{-webkit-animation:fpFadeInDown 300ms cubic-bezier(0.23, 1, 0.32, 1);animation:fpFadeInDown 300ms cubic-bezier(0.23, 1, 0.32, 1);}.flatpickr-calendar.inline{display:block;position:relative;top:2px;}.flatpickr-calendar.static{position:absolute;top:calc(100% + 2px);}.flatpickr-calendar.static.open{z-index:999;display:block;}.flatpickr-calendar.multiMonth .flatpickr-days .dayContainer:nth-child(n+1) .flatpickr-day.inRange:nth-child(7n+7){-webkit-box-shadow:none !important;box-shadow:none !important;}.flatpickr-calendar.multiMonth .flatpickr-days .dayContainer:nth-child(n+2) .flatpickr-day.inRange:nth-child(7n+1){-webkit-box-shadow:-2px 0 0 #e6e6e6, 5px 0 0 #e6e6e6;box-shadow:-2px 0 0 #e6e6e6, 5px 0 0 #e6e6e6;}.flatpickr-calendar .hasWeeks .dayContainer,.flatpickr-calendar .hasTime .dayContainer{border-bottom:0;border-bottom-right-radius:0;border-bottom-left-radius:0;}.flatpickr-calendar .hasWeeks .dayContainer{border-left:0;}.flatpickr-calendar.hasTime .flatpickr-time{height:40px;border-top:1px solid #e6e6e6;}.flatpickr-calendar.noCalendar.hasTime .flatpickr-time{height:auto;}.flatpickr-calendar:before,.flatpickr-calendar:after{position:absolute;display:block;pointer-events:none;border:solid transparent;content:'';height:0;width:0;left:22px;}.flatpickr-calendar.rightMost:before,.flatpickr-calendar.arrowRight:before,.flatpickr-calendar.rightMost:after,.flatpickr-calendar.arrowRight:after{left:auto;right:22px;}.flatpickr-calendar.arrowCenter:before,.flatpickr-calendar.arrowCenter:after{left:50%;right:50%;}.flatpickr-calendar:before{border-width:5px;margin:0 -5px;}.flatpickr-calendar:after{border-width:4px;margin:0 -4px;}.flatpickr-calendar.arrowTop:before,.flatpickr-calendar.arrowTop:after{bottom:100%;}.flatpickr-calendar.arrowTop:before{border-bottom-color:#e6e6e6;}.flatpickr-calendar.arrowTop:after{border-bottom-color:#fff;}.flatpickr-calendar.arrowBottom:before,.flatpickr-calendar.arrowBottom:after{top:100%;}.flatpickr-calendar.arrowBottom:before{border-top-color:#e6e6e6;}.flatpickr-calendar.arrowBottom:after{border-top-color:#fff;}.flatpickr-calendar:focus{outline:0;}.flatpickr-wrapper{position:relative;display:inline-block;}.flatpickr-months{display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;}.flatpickr-months .flatpickr-month{background:transparent;color:rgba(0, 0, 0, 0.9);fill:rgba(0, 0, 0, 0.9);height:34px;line-height:1;text-align:center;position:relative;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;overflow:hidden;-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;}.flatpickr-months .flatpickr-prev-month,.flatpickr-months .flatpickr-next-month{-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;text-decoration:none;cursor:pointer;position:absolute;top:0;height:34px;padding:10px;z-index:3;color:rgba(0, 0, 0, 0.9);fill:rgba(0, 0, 0, 0.9);}.flatpickr-months .flatpickr-prev-month.flatpickr-disabled,.flatpickr-months .flatpickr-next-month.flatpickr-disabled{display:none;}.flatpickr-months .flatpickr-prev-month i,.flatpickr-months .flatpickr-next-month i{position:relative;}.flatpickr-months .flatpickr-prev-month.flatpickr-prev-month,.flatpickr-months .flatpickr-next-month.flatpickr-prev-month{left:0;}.flatpickr-months .flatpickr-prev-month.flatpickr-next-month,.flatpickr-months .flatpickr-next-month.flatpickr-next-month{right:0;}.flatpickr-months .flatpickr-prev-month:hover,.flatpickr-months .flatpickr-next-month:hover{color:#959ea9;}.flatpickr-months .flatpickr-prev-month:hover svg,.flatpickr-months .flatpickr-next-month:hover svg{fill:#f64747;}.flatpickr-months .flatpickr-prev-month svg,.flatpickr-months .flatpickr-next-month svg{width:14px;height:14px;}.flatpickr-months .flatpickr-prev-month svg path,.flatpickr-months .flatpickr-next-month svg path{-webkit-transition:fill 0.1s;transition:fill 0.1s;fill:inherit;}.numInputWrapper{position:relative;height:auto;}.numInputWrapper input,.numInputWrapper span{display:inline-block;}.numInputWrapper input{width:100%;}.numInputWrapper input::-ms-clear{display:none;}.numInputWrapper input::-webkit-outer-spin-button,.numInputWrapper input::-webkit-inner-spin-button{margin:0;-webkit-appearance:none;}.numInputWrapper span{position:absolute;right:0;width:14px;padding:0 4px 0 2px;height:50%;line-height:50%;opacity:0;cursor:pointer;border:1px solid rgba(57, 57, 57, 0.15);-webkit-box-sizing:border-box;box-sizing:border-box;}.numInputWrapper span:hover{background:rgba(0, 0, 0, 0.1);}.numInputWrapper span:active{background:rgba(0, 0, 0, 0.2);}.numInputWrapper span:after{display:block;content:"";position:absolute;}.numInputWrapper span.arrowUp{top:0;border-bottom:0;}.numInputWrapper span.arrowUp:after{border-left:4px solid transparent;border-right:4px solid transparent;border-bottom:4px solid rgba(57, 57, 57, 0.6);top:26%;}.numInputWrapper span.arrowDown{top:50%;}.numInputWrapper span.arrowDown:after{border-left:4px solid transparent;border-right:4px solid transparent;border-top:4px solid rgba(57, 57, 57, 0.6);top:40%;}.numInputWrapper span svg{width:inherit;height:auto;}.numInputWrapper span svg path{fill:rgba(0, 0, 0, 0.5);}.numInputWrapper:hover{background:rgba(0, 0, 0, 0.05);}.numInputWrapper:hover span{opacity:1;}.flatpickr-current-month{font-size:135%;line-height:inherit;font-weight:300;color:inherit;position:absolute;width:75%;left:12.5%;padding:7.48px 0 0 0;line-height:1;height:34px;display:inline-block;text-align:center;-webkit-transform:translate3d(0px, 0px, 0px);transform:translate3d(0px, 0px, 0px);}.flatpickr-current-month span.cur-month{font-family:inherit;font-weight:700;color:inherit;display:inline-block;margin-left:0.5ch;padding:0;}.flatpickr-current-month span.cur-month:hover{background:rgba(0, 0, 0, 0.05);}.flatpickr-current-month .numInputWrapper{width:6ch;width:7ch\0;display:inline-block;}.flatpickr-current-month .numInputWrapper span.arrowUp:after{border-bottom-color:rgba(0, 0, 0, 0.9);}.flatpickr-current-month .numInputWrapper span.arrowDown:after{border-top-color:rgba(0, 0, 0, 0.9);}.flatpickr-current-month input.cur-year{background:transparent;-webkit-box-sizing:border-box;box-sizing:border-box;color:inherit;cursor:text;padding:0 0 0 0.5ch;margin:0;display:inline-block;font-size:inherit;font-family:inherit;font-weight:300;line-height:inherit;height:auto;border:0;border-radius:0;vertical-align:initial;-webkit-appearance:textfield;-moz-appearance:textfield;appearance:textfield;}.flatpickr-current-month input.cur-year:focus{outline:0;}.flatpickr-current-month input.cur-year[disabled],.flatpickr-current-month input.cur-year[disabled]:hover{font-size:100%;color:rgba(0, 0, 0, 0.5);background:transparent;pointer-events:none;}.flatpickr-current-month .flatpickr-monthDropdown-months{appearance:menulist;background:transparent;border:none;border-radius:0;box-sizing:border-box;color:inherit;cursor:pointer;font-size:inherit;font-family:inherit;font-weight:300;height:auto;line-height:inherit;margin:-1px 0 0 0;outline:none;padding:0 0 0 0.5ch;position:relative;vertical-align:initial;-webkit-box-sizing:border-box;-webkit-appearance:menulist;-moz-appearance:menulist;width:auto;}.flatpickr-current-month .flatpickr-monthDropdown-months:focus,.flatpickr-current-month .flatpickr-monthDropdown-months:active{outline:none;}.flatpickr-current-month .flatpickr-monthDropdown-months:hover{background:rgba(0, 0, 0, 0.05);}.flatpickr-current-month .flatpickr-monthDropdown-months .flatpickr-monthDropdown-month{background-color:transparent;outline:none;padding:0;}.flatpickr-weekdays{background:transparent;text-align:center;overflow:hidden;width:100%;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-ms-flex-align:center;align-items:center;height:28px;}.flatpickr-weekdays .flatpickr-weekdaycontainer{display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;}span.flatpickr-weekday{cursor:default;font-size:90%;background:transparent;color:rgba(0, 0, 0, 0.54);line-height:1;margin:0;text-align:center;display:block;-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;font-weight:bolder;}.dayContainer,.flatpickr-weeks{padding:1px 0 0 0;}.flatpickr-days{position:relative;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-ms-flex-align:start;align-items:flex-start;width:307.875px;}.flatpickr-days:focus{outline:0;}.dayContainer{padding:0;outline:0;text-align:left;width:307.875px;min-width:307.875px;max-width:307.875px;-webkit-box-sizing:border-box;box-sizing:border-box;display:inline-block;display:-ms-flexbox;display:-webkit-box;display:-webkit-flex;display:flex;-webkit-flex-wrap:wrap;flex-wrap:wrap;-ms-flex-wrap:wrap;-ms-flex-pack:justify;-webkit-justify-content:space-around;justify-content:space-around;-webkit-transform:translate3d(0px, 0px, 0px);transform:translate3d(0px, 0px, 0px);opacity:1;}.dayContainer + .dayContainer{-webkit-box-shadow:-1px 0 0 #e6e6e6;box-shadow:-1px 0 0 #e6e6e6;}.flatpickr-day{background:none;border:1px solid transparent;border-radius:150px;-webkit-box-sizing:border-box;box-sizing:border-box;color:#393939;cursor:pointer;font-weight:400;width:14.2857143%;-webkit-flex-basis:14.2857143%;-ms-flex-preferred-size:14.2857143%;flex-basis:14.2857143%;max-width:39px;height:39px;line-height:39px;margin:0;display:inline-block;position:relative;-webkit-box-pack:center;-webkit-justify-content:center;-ms-flex-pack:center;justify-content:center;text-align:center;}.flatpickr-day.inRange,.flatpickr-day.prevMonthDay.inRange,.flatpickr-day.nextMonthDay.inRange,.flatpickr-day.today.inRange,.flatpickr-day.prevMonthDay.today.inRange,.flatpickr-day.nextMonthDay.today.inRange,.flatpickr-day:hover,.flatpickr-day.prevMonthDay:hover,.flatpickr-day.nextMonthDay:hover,.flatpickr-day:focus,.flatpickr-day.prevMonthDay:focus,.flatpickr-day.nextMonthDay:focus{cursor:pointer;outline:0;background:#e6e6e6;border-color:#e6e6e6;}.flatpickr-day.today{border-color:#959ea9;}.flatpickr-day.today:hover,.flatpickr-day.today:focus{border-color:#959ea9;background:#959ea9;color:#fff;}.flatpickr-day.selected,.flatpickr-day.startRange,.flatpickr-day.endRange,.flatpickr-day.selected.inRange,.flatpickr-day.startRange.inRange,.flatpickr-day.endRange.inRange,.flatpickr-day.selected:focus,.flatpickr-day.startRange:focus,.flatpickr-day.endRange:focus,.flatpickr-day.selected:hover,.flatpickr-day.startRange:hover,.flatpickr-day.endRange:hover,.flatpickr-day.selected.prevMonthDay,.flatpickr-day.startRange.prevMonthDay,.flatpickr-day.endRange.prevMonthDay,.flatpickr-day.selected.nextMonthDay,.flatpickr-day.startRange.nextMonthDay,.flatpickr-day.endRange.nextMonthDay{background:#569ff7;-webkit-box-shadow:none;box-shadow:none;color:#fff;border-color:#569ff7;}.flatpickr-day.selected.startRange,.flatpickr-day.startRange.startRange,.flatpickr-day.endRange.startRange{border-radius:50px 0 0 50px;}.flatpickr-day.selected.endRange,.flatpickr-day.startRange.endRange,.flatpickr-day.endRange.endRange{border-radius:0 50px 50px 0;}.flatpickr-day.selected.startRange + .endRange:not(:nth-child(7n+1)),.flatpickr-day.startRange.startRange + .endRange:not(:nth-child(7n+1)),.flatpickr-day.endRange.startRange + .endRange:not(:nth-child(7n+1)){-webkit-box-shadow:-10px 0 0 #569ff7;box-shadow:-10px 0 0 #569ff7;}.flatpickr-day.selected.startRange.endRange,.flatpickr-day.startRange.startRange.endRange,.flatpickr-day.endRange.startRange.endRange{border-radius:50px;}.flatpickr-day.inRange{border-radius:0;-webkit-box-shadow:-5px 0 0 #e6e6e6, 5px 0 0 #e6e6e6;box-shadow:-5px 0 0 #e6e6e6, 5px 0 0 #e6e6e6;}.flatpickr-day.flatpickr-disabled,.flatpickr-day.flatpickr-disabled:hover,.flatpickr-day.prevMonthDay,.flatpickr-day.nextMonthDay,.flatpickr-day.notAllowed,.flatpickr-day.notAllowed.prevMonthDay,.flatpickr-day.notAllowed.nextMonthDay{color:rgba(57, 57, 57, 0.3);background:transparent;border-color:transparent;cursor:default;}.flatpickr-day.flatpickr-disabled,.flatpickr-day.flatpickr-disabled:hover{cursor:not-allowed;color:rgba(57, 57, 57, 0.1);}.flatpickr-day.week.selected{border-radius:0;-webkit-box-shadow:-5px 0 0 #569ff7, 5px 0 0 #569ff7;box-shadow:-5px 0 0 #569ff7, 5px 0 0 #569ff7;}.flatpickr-day.hidden{visibility:hidden;}.rangeMode .flatpickr-day{margin-top:1px;}.flatpickr-weekwrapper{float:left;}.flatpickr-weekwrapper .flatpickr-weeks{padding:0 12px;-webkit-box-shadow:1px 0 0 #e6e6e6;box-shadow:1px 0 0 #e6e6e6;}.flatpickr-weekwrapper .flatpickr-weekday{float:none;width:100%;line-height:28px;}.flatpickr-weekwrapper span.flatpickr-day,.flatpickr-weekwrapper span.flatpickr-day:hover{display:block;width:100%;max-width:none;color:rgba(57, 57, 57, 0.3);background:transparent;cursor:default;border:none;}.flatpickr-innerContainer{display:block;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-sizing:border-box;box-sizing:border-box;overflow:hidden;}.flatpickr-rContainer{display:inline-block;padding:0;-webkit-box-sizing:border-box;box-sizing:border-box;}.flatpickr-time{text-align:center;outline:0;display:block;height:0;line-height:40px;max-height:40px;-webkit-box-sizing:border-box;box-sizing:border-box;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;}.flatpickr-time:after{content:"";display:table;clear:both;}.flatpickr-time .numInputWrapper{-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;width:40%;height:40px;float:left;}.flatpickr-time .numInputWrapper span.arrowUp:after{border-bottom-color:#393939;}.flatpickr-time .numInputWrapper span.arrowDown:after{border-top-color:#393939;}.flatpickr-time.hasSeconds .numInputWrapper{width:26%;}.flatpickr-time.time24hr .numInputWrapper{width:49%;}.flatpickr-time input{background:transparent;-webkit-box-shadow:none;box-shadow:none;border:0;border-radius:0;text-align:center;margin:0;padding:0;height:inherit;line-height:inherit;color:#393939;font-size:14px;position:relative;-webkit-box-sizing:border-box;box-sizing:border-box;-webkit-appearance:textfield;-moz-appearance:textfield;appearance:textfield;}.flatpickr-time input.flatpickr-hour{font-weight:bold;}.flatpickr-time input.flatpickr-minute,.flatpickr-time input.flatpickr-second{font-weight:400;}.flatpickr-time input:focus{outline:0;border:0;}.flatpickr-time .flatpickr-time-separator,.flatpickr-time .flatpickr-am-pm{height:inherit;float:left;line-height:inherit;color:#393939;font-weight:bold;width:2%;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-webkit-align-self:center;-ms-flex-item-align:center;align-self:center;}.flatpickr-time .flatpickr-am-pm{outline:0;width:18%;cursor:pointer;text-align:center;font-weight:400;}.flatpickr-time input:hover,.flatpickr-time .flatpickr-am-pm:hover,.flatpickr-time input:focus,.flatpickr-time .flatpickr-am-pm:focus{background:#eee;}.flatpickr-input[readonly]{cursor:pointer;}@-webkit-keyframes fpFadeInDown{from{opacity:0;-webkit-transform:translate3d(0, -20px, 0);transform:translate3d(0, -20px, 0);}to{opacity:1;-webkit-transform:translate3d(0, 0, 0);transform:translate3d(0, 0, 0);}}@keyframes fpFadeInDown{from{opacity:0;-webkit-transform:translate3d(0, -20px, 0);transform:translate3d(0, -20px, 0);}to{opacity:1;-webkit-transform:translate3d(0, 0, 0);transform:translate3d(0, 0, 0);}}.flatpickr-calendar{background:transparent;opacity:0;display:none;text-align:center;visibility:hidden;padding:0;-webkit-animation:none;animation:none;direction:ltr;border:0;font-size:14px;line-height:24px;border-radius:5px;position:absolute;width:307.875px;-webkit-box-sizing:border-box;box-sizing:border-box;-ms-touch-action:manipulation;touch-action:manipulation;-webkit-box-shadow:0 3px 13px rgba(0, 0, 0, 0.08);box-shadow:0 3px 13px rgba(0, 0, 0, 0.08);}.flatpickr-calendar.open,.flatpickr-calendar.inline{opacity:1;max-height:640px;visibility:visible;}.flatpickr-calendar.open{display:inline-block;z-index:99999;}.flatpickr-calendar.animate.open{-webkit-animation:fpFadeInDown 300ms cubic-bezier(0.23, 1, 0.32, 1);animation:fpFadeInDown 300ms cubic-bezier(0.23, 1, 0.32, 1);}.flatpickr-calendar.inline{display:block;position:relative;top:2px;}.flatpickr-calendar.static{position:absolute;top:calc(100% + 2px);}.flatpickr-calendar.static.open{z-index:999;display:block;}.flatpickr-calendar.multiMonth .flatpickr-days .dayContainer:nth-child(n+1) .flatpickr-day.inRange:nth-child(7n+7){-webkit-box-shadow:none !important;box-shadow:none !important;}.flatpickr-calendar.multiMonth .flatpickr-days .dayContainer:nth-child(n+2) .flatpickr-day.inRange:nth-child(7n+1){-webkit-box-shadow:-2px 0 0 #e6e6e6, 5px 0 0 #e6e6e6;box-shadow:-2px 0 0 #e6e6e6, 5px 0 0 #e6e6e6;}.flatpickr-calendar .hasWeeks .dayContainer,.flatpickr-calendar .hasTime .dayContainer{border-bottom:0;border-bottom-right-radius:0;border-bottom-left-radius:0;}.flatpickr-calendar .hasWeeks .dayContainer{border-left:0;}.flatpickr-calendar.hasTime .flatpickr-time{height:40px;border-top:1px solid #eceef1;}.flatpickr-calendar.hasTime .flatpickr-innerContainer{border-bottom:0;}.flatpickr-calendar.hasTime .flatpickr-time{border:1px solid #eceef1;}.flatpickr-calendar.noCalendar.hasTime .flatpickr-time{height:auto;}.flatpickr-calendar:before,.flatpickr-calendar:after{position:absolute;display:block;pointer-events:none;border:solid transparent;content:'';height:0;width:0;left:22px;}.flatpickr-calendar.rightMost:before,.flatpickr-calendar.arrowRight:before,.flatpickr-calendar.rightMost:after,.flatpickr-calendar.arrowRight:after{left:auto;right:22px;}.flatpickr-calendar.arrowCenter:before,.flatpickr-calendar.arrowCenter:after{left:50%;right:50%;}.flatpickr-calendar:before{border-width:5px;margin:0 -5px;}.flatpickr-calendar:after{border-width:4px;margin:0 -4px;}.flatpickr-calendar.arrowTop:before,.flatpickr-calendar.arrowTop:after{bottom:100%;}.flatpickr-calendar.arrowTop:before{border-bottom-color:#eceef1;}.flatpickr-calendar.arrowTop:after{border-bottom-color:#eceef1;}.flatpickr-calendar.arrowBottom:before,.flatpickr-calendar.arrowBottom:after{top:100%;}.flatpickr-calendar.arrowBottom:before{border-top-color:#eceef1;}.flatpickr-calendar.arrowBottom:after{border-top-color:#eceef1;}.flatpickr-calendar:focus{outline:0;}.flatpickr-wrapper{position:relative;display:inline-block;}.flatpickr-months{display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;}.flatpickr-months .flatpickr-month{border-radius:5px 5px 0 0;background:#eceef1;color:#5a6171;fill:#5a6171;height:34px;line-height:1;text-align:center;position:relative;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;overflow:hidden;-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;}.flatpickr-months .flatpickr-prev-month,.flatpickr-months .flatpickr-next-month{-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;text-decoration:none;cursor:pointer;position:absolute;top:0;height:34px;padding:10px;z-index:3;color:#5a6171;fill:#5a6171;}.flatpickr-months .flatpickr-prev-month.flatpickr-disabled,.flatpickr-months .flatpickr-next-month.flatpickr-disabled{display:none;}.flatpickr-months .flatpickr-prev-month i,.flatpickr-months .flatpickr-next-month i{position:relative;}.flatpickr-months .flatpickr-prev-month.flatpickr-prev-month,.flatpickr-months .flatpickr-next-month.flatpickr-prev-month{left:0;}.flatpickr-months .flatpickr-prev-month.flatpickr-next-month,.flatpickr-months .flatpickr-next-month.flatpickr-next-month{right:0;}.flatpickr-months .flatpickr-prev-month:hover,.flatpickr-months .flatpickr-next-month:hover{color:#bbb;}.flatpickr-months .flatpickr-prev-month:hover svg,.flatpickr-months .flatpickr-next-month:hover svg{fill:#f64747;}.flatpickr-months .flatpickr-prev-month svg,.flatpickr-months .flatpickr-next-month svg{width:14px;height:14px;}.flatpickr-months .flatpickr-prev-month svg path,.flatpickr-months .flatpickr-next-month svg path{-webkit-transition:fill 0.1s;transition:fill 0.1s;fill:inherit;}.numInputWrapper{position:relative;height:auto;}.numInputWrapper input,.numInputWrapper span{display:inline-block;}.numInputWrapper input{width:100%;}.numInputWrapper input::-ms-clear{display:none;}.numInputWrapper input::-webkit-outer-spin-button,.numInputWrapper input::-webkit-inner-spin-button{margin:0;-webkit-appearance:none;}.numInputWrapper span{position:absolute;right:0;width:14px;padding:0 4px 0 2px;height:50%;line-height:50%;opacity:0;cursor:pointer;border:1px solid rgba(72, 72, 72, 0.15);-webkit-box-sizing:border-box;box-sizing:border-box;}.numInputWrapper span:hover{background:rgba(0, 0, 0, 0.1);}.numInputWrapper span:active{background:rgba(0, 0, 0, 0.2);}.numInputWrapper span:after{display:block;content:"";position:absolute;}.numInputWrapper span.arrowUp{top:0;border-bottom:0;}.numInputWrapper span.arrowUp:after{border-left:4px solid transparent;border-right:4px solid transparent;border-bottom:4px solid rgba(72, 72, 72, 0.6);top:26%;}.numInputWrapper span.arrowDown{top:50%;}.numInputWrapper span.arrowDown:after{border-left:4px solid transparent;border-right:4px solid transparent;border-top:4px solid rgba(72, 72, 72, 0.6);top:40%;}.numInputWrapper span svg{width:inherit;height:auto;}.numInputWrapper span svg path{fill:rgba(90, 97, 113, 0.5);}.numInputWrapper:hover{background:rgba(0, 0, 0, 0.05);}.numInputWrapper:hover span{opacity:1;}.flatpickr-current-month{font-size:135%;line-height:inherit;font-weight:300;color:inherit;position:absolute;width:75%;left:12.5%;padding:7.48px 0 0 0;line-height:1;height:34px;display:inline-block;text-align:center;-webkit-transform:translate3d(0px, 0px, 0px);transform:translate3d(0px, 0px, 0px);}.flatpickr-current-month span.cur-month{font-family:inherit;font-weight:700;color:inherit;display:inline-block;margin-left:0.5ch;padding:0;}.flatpickr-current-month span.cur-month:hover{background:rgba(0, 0, 0, 0.05);}.flatpickr-current-month .numInputWrapper{width:6ch;width:7ch\0;display:inline-block;}.flatpickr-current-month .numInputWrapper span.arrowUp:after{border-bottom-color:#5a6171;}.flatpickr-current-month .numInputWrapper span.arrowDown:after{border-top-color:#5a6171;}.flatpickr-current-month input.cur-year{background:transparent;-webkit-box-sizing:border-box;box-sizing:border-box;color:inherit;cursor:text;padding:0 0 0 0.5ch;margin:0;display:inline-block;font-size:inherit;font-family:inherit;font-weight:300;line-height:inherit;height:auto;border:0;border-radius:0;vertical-align:initial;-webkit-appearance:textfield;-moz-appearance:textfield;appearance:textfield;}.flatpickr-current-month input.cur-year:focus{outline:0;}.flatpickr-current-month input.cur-year[disabled],.flatpickr-current-month input.cur-year[disabled]:hover{font-size:100%;color:rgba(90, 97, 113, 0.5);background:transparent;pointer-events:none;}.flatpickr-current-month .flatpickr-monthDropdown-months{appearance:menulist;background:#eceef1;border:none;border-radius:0;box-sizing:border-box;color:inherit;cursor:pointer;font-size:inherit;font-family:inherit;font-weight:300;height:auto;line-height:inherit;margin:-1px 0 0 0;outline:none;padding:0 0 0 0.5ch;position:relative;vertical-align:initial;-webkit-box-sizing:border-box;-webkit-appearance:menulist;-moz-appearance:menulist;width:auto;}.flatpickr-current-month .flatpickr-monthDropdown-months:focus,.flatpickr-current-month .flatpickr-monthDropdown-months:active{outline:none;}.flatpickr-current-month .flatpickr-monthDropdown-months:hover{background:rgba(0, 0, 0, 0.05);}.flatpickr-current-month .flatpickr-monthDropdown-months .flatpickr-monthDropdown-month{background-color:#eceef1;outline:none;padding:0;}.flatpickr-weekdays{background:#eceef1;text-align:center;overflow:hidden;width:100%;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-align:center;-webkit-align-items:center;-ms-flex-align:center;align-items:center;height:28px;}.flatpickr-weekdays .flatpickr-weekdaycontainer{display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;}span.flatpickr-weekday{cursor:default;font-size:90%;background:#eceef1;color:#5a6171;line-height:1;margin:0;text-align:center;display:block;-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;font-weight:bolder;}.dayContainer,.flatpickr-weeks{padding:1px 0 0 0;}.flatpickr-days{position:relative;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-align:start;-webkit-align-items:flex-start;-ms-flex-align:start;align-items:flex-start;width:307.875px;border-left:1px solid #eceef1;border-right:1px solid #eceef1;}.flatpickr-days:focus{outline:0;}.dayContainer{padding:0;outline:0;text-align:left;width:307.875px;min-width:307.875px;max-width:307.875px;-webkit-box-sizing:border-box;box-sizing:border-box;display:inline-block;display:-ms-flexbox;display:-webkit-box;display:-webkit-flex;display:flex;-webkit-flex-wrap:wrap;flex-wrap:wrap;-ms-flex-wrap:wrap;-ms-flex-pack:justify;-webkit-justify-content:space-around;justify-content:space-around;-webkit-transform:translate3d(0px, 0px, 0px);transform:translate3d(0px, 0px, 0px);opacity:1;}.dayContainer + .dayContainer{-webkit-box-shadow:-1px 0 0 #eceef1;box-shadow:-1px 0 0 #eceef1;}.flatpickr-day{background:none;border:1px solid transparent;border-radius:150px;-webkit-box-sizing:border-box;box-sizing:border-box;color:#484848;cursor:pointer;font-weight:400;width:14.2857143%;-webkit-flex-basis:14.2857143%;-ms-flex-preferred-size:14.2857143%;flex-basis:14.2857143%;max-width:39px;height:39px;line-height:39px;margin:0;display:inline-block;position:relative;-webkit-box-pack:center;-webkit-justify-content:center;-ms-flex-pack:center;justify-content:center;text-align:center;}.flatpickr-day.inRange,.flatpickr-day.prevMonthDay.inRange,.flatpickr-day.nextMonthDay.inRange,.flatpickr-day.today.inRange,.flatpickr-day.prevMonthDay.today.inRange,.flatpickr-day.nextMonthDay.today.inRange,.flatpickr-day:hover,.flatpickr-day.prevMonthDay:hover,.flatpickr-day.nextMonthDay:hover,.flatpickr-day:focus,.flatpickr-day.prevMonthDay:focus,.flatpickr-day.nextMonthDay:focus{cursor:pointer;outline:0;background:#e2e2e2;border-color:#e2e2e2;}.flatpickr-day.today{border-color:#bbb;}.flatpickr-day.today:hover,.flatpickr-day.today:focus{border-color:#bbb;background:#bbb;color:#fff;}.flatpickr-day.selected,.flatpickr-day.startRange,.flatpickr-day.endRange,.flatpickr-day.selected.inRange,.flatpickr-day.startRange.inRange,.flatpickr-day.endRange.inRange,.flatpickr-day.selected:focus,.flatpickr-day.startRange:focus,.flatpickr-day.endRange:focus,.flatpickr-day.selected:hover,.flatpickr-day.startRange:hover,.flatpickr-day.endRange:hover,.flatpickr-day.selected.prevMonthDay,.flatpickr-day.startRange.prevMonthDay,.flatpickr-day.endRange.prevMonthDay,.flatpickr-day.selected.nextMonthDay,.flatpickr-day.startRange.nextMonthDay,.flatpickr-day.endRange.nextMonthDay{background:#ff5a5f;-webkit-box-shadow:none;box-shadow:none;color:#fff;border-color:#ff5a5f;}.flatpickr-day.selected.startRange,.flatpickr-day.startRange.startRange,.flatpickr-day.endRange.startRange{border-radius:50px 0 0 50px;}.flatpickr-day.selected.endRange,.flatpickr-day.startRange.endRange,.flatpickr-day.endRange.endRange{border-radius:0 50px 50px 0;}.flatpickr-day.selected.startRange + .endRange:not(:nth-child(7n+1)),.flatpickr-day.startRange.startRange + .endRange:not(:nth-child(7n+1)),.flatpickr-day.endRange.startRange + .endRange:not(:nth-child(7n+1)){-webkit-box-shadow:-10px 0 0 #ff5a5f;box-shadow:-10px 0 0 #ff5a5f;}.flatpickr-day.selected.startRange.endRange,.flatpickr-day.startRange.startRange.endRange,.flatpickr-day.endRange.startRange.endRange{border-radius:50px;}.flatpickr-day.inRange{border-radius:0;-webkit-box-shadow:-5px 0 0 #e2e2e2, 5px 0 0 #e2e2e2;box-shadow:-5px 0 0 #e2e2e2, 5px 0 0 #e2e2e2;}.flatpickr-day.flatpickr-disabled,.flatpickr-day.flatpickr-disabled:hover,.flatpickr-day.prevMonthDay,.flatpickr-day.nextMonthDay,.flatpickr-day.notAllowed,.flatpickr-day.notAllowed.prevMonthDay,.flatpickr-day.notAllowed.nextMonthDay{color:rgba(72, 72, 72, 0.3);background:transparent;border-color:transparent;cursor:default;}.flatpickr-day.flatpickr-disabled,.flatpickr-day.flatpickr-disabled:hover{cursor:not-allowed;color:rgba(72, 72, 72, 0.1);}.flatpickr-day.week.selected{border-radius:0;-webkit-box-shadow:-5px 0 0 #ff5a5f, 5px 0 0 #ff5a5f;box-shadow:-5px 0 0 #ff5a5f, 5px 0 0 #ff5a5f;}.flatpickr-day.hidden{visibility:hidden;}.rangeMode .flatpickr-day{margin-top:1px;}.flatpickr-weekwrapper{float:left;}.flatpickr-weekwrapper .flatpickr-weeks{padding:0 12px;border-left:1px solid #eceef1;}.flatpickr-weekwrapper .flatpickr-weekday{float:none;width:100%;line-height:28px;}.flatpickr-weekwrapper span.flatpickr-day,.flatpickr-weekwrapper span.flatpickr-day:hover{display:block;width:100%;max-width:none;color:rgba(72, 72, 72, 0.3);background:transparent;cursor:default;border:none;}.flatpickr-innerContainer{display:block;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;-webkit-box-sizing:border-box;box-sizing:border-box;overflow:hidden;background:#fff;border-bottom:1px solid #eceef1;}.flatpickr-rContainer{display:inline-block;padding:0;-webkit-box-sizing:border-box;box-sizing:border-box;}.flatpickr-time{text-align:center;outline:0;display:block;height:0;line-height:40px;max-height:40px;-webkit-box-sizing:border-box;box-sizing:border-box;overflow:hidden;display:-webkit-box;display:-webkit-flex;display:-ms-flexbox;display:flex;background:#fff;border-radius:0 0 5px 5px;}.flatpickr-time:after{content:"";display:table;clear:both;}.flatpickr-time .numInputWrapper{-webkit-box-flex:1;-webkit-flex:1;-ms-flex:1;flex:1;width:40%;height:40px;float:left;}.flatpickr-time .numInputWrapper span.arrowUp:after{border-bottom-color:#484848;}.flatpickr-time .numInputWrapper span.arrowDown:after{border-top-color:#484848;}.flatpickr-time.hasSeconds .numInputWrapper{width:26%;}.flatpickr-time.time24hr .numInputWrapper{width:49%;}.flatpickr-time input{background:transparent;-webkit-box-shadow:none;box-shadow:none;border:0;border-radius:0;text-align:center;margin:0;padding:0;height:inherit;line-height:inherit;color:#484848;font-size:14px;position:relative;-webkit-box-sizing:border-box;box-sizing:border-box;-webkit-appearance:textfield;-moz-appearance:textfield;appearance:textfield;}.flatpickr-time input.flatpickr-hour{font-weight:bold;}.flatpickr-time input.flatpickr-minute,.flatpickr-time input.flatpickr-second{font-weight:400;}.flatpickr-time input:focus{outline:0;border:0;}.flatpickr-time .flatpickr-time-separator,.flatpickr-time .flatpickr-am-pm{height:inherit;float:left;line-height:inherit;color:#484848;font-weight:bold;width:2%;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-webkit-align-self:center;-ms-flex-item-align:center;align-self:center;}.flatpickr-time .flatpickr-am-pm{outline:0;width:18%;cursor:pointer;text-align:center;font-weight:400;}.flatpickr-time input:hover,.flatpickr-time .flatpickr-am-pm:hover,.flatpickr-time input:focus,.flatpickr-time .flatpickr-am-pm:focus{background:#eaeaea;}.flatpickr-input[readonly]{cursor:pointer;}@-webkit-keyframes fpFadeInDown{from{opacity:0;-webkit-transform:translate3d(0, -20px, 0);transform:translate3d(0, -20px, 0);}to{opacity:1;-webkit-transform:translate3d(0, 0, 0);transform:translate3d(0, 0, 0);}}@keyframes fpFadeInDown{from{opacity:0;-webkit-transform:translate3d(0, -20px, 0);transform:translate3d(0, -20px, 0);}to{opacity:1;-webkit-transform:translate3d(0, 0, 0);transform:translate3d(0, 0, 0);}}span.flatpickr-day.selected{font-weight:bold;}`;
  class PickerBaseView extends InputWidgetView {
    constructor() {
      super(...arguments);
      mt(this, "_picker");
    }
    get picker() {
      return assert$2(this._picker != null), this._picker;
    }
    *controls() {
      yield this.picker.altInput ?? this.input_el;
    }
    remove() {
      var H;
      (H = this._picker) == null || H.destroy(), super.remove();
    }
    stylesheets() {
      return [...super.stylesheets(), flatpickr_css];
    }
    connect_signals() {
      super.connect_signals();
      const { inline: H } = this.model.properties;
      this.connect(H.change, () => this.picker.set("inline", this.model.inline));
    }
    get flatpickr_options() {
      return {
        appendTo: this.group_el,
        inline: this.model.inline,
        position: this._position.bind(this),
        onChange: (H) => {
          this._on_change(H), this.change_input();
        }
      };
    }
    _render_input() {
      return this.input_el = input$1({ type: "text", class: input, disabled: this.model.disabled });
    }
    render() {
      var V;
      super.render(), (V = this._picker) == null || V.destroy();
      const H = this.flatpickr_options;
      this._picker = flatpickr(this.input_el, H);
    }
    // https://github.com/flatpickr/flatpickr/pull/2362
    _position(H, V) {
      const q = V ?? H._positionElement, G = [...H.calendarContainer.children].reduce((me, pe) => me + bounding_box(pe).height, 0), J = H.calendarContainer.offsetWidth, Z = this.model.position.split(" "), te = Z[0], ne = Z.length > 1 ? Z[1] : null, ee = {
        top: q.offsetTop,
        bottom: q.offsetTop + q.offsetHeight,
        left: q.offsetLeft,
        right: q.offsetLeft + q.offsetWidth,
        width: q.offsetWidth
      }, ae = window.innerHeight - ee.bottom, oe = te === "above" || te !== "below" && ae < G && ee.top > G, se = H.config.appendTo != null ? ee.top + (oe ? -G - 2 : q.offsetHeight + 2) : window.scrollY + ee.top + (oe ? -G - 2 : q.offsetHeight + 2);
      if (H.calendarContainer.classList.toggle("arrowTop", !oe), H.calendarContainer.classList.toggle("arrowBottom", oe), H.config.inline)
        return;
      let ce = window.scrollX + ee.left, le = !1, ue = !1;
      ne === "center" ? (ce -= (J - ee.width) / 2, le = !0) : ne === "right" && (ce -= J - ee.width, ue = !0), H.calendarContainer.classList.toggle("arrowLeft", !le && !ue), H.calendarContainer.classList.toggle("arrowCenter", le), H.calendarContainer.classList.toggle("arrowRight", ue);
      const de = window.document.body.offsetWidth - (window.scrollX + ee.right), fe = ce + J > window.document.body.offsetWidth, he = de + J > window.document.body.offsetWidth;
      if (H.calendarContainer.classList.toggle("rightMost", fe), !H.config.static)
        if (H.calendarContainer.style.top = `${se}px`, !fe)
          H.calendarContainer.style.left = `${ce}px`, H.calendarContainer.style.right = "auto";
        else if (!he)
          H.calendarContainer.style.left = "auto", H.calendarContainer.style.right = `${de}px`;
        else {
          const me = this.shadow_el.styleSheets[0], pe = window.document.body.offsetWidth, ye = Math.max(0, pe / 2 - J / 2), ge = ".flatpickr-calendar.centerMost:before", ve = ".flatpickr-calendar.centerMost:after", be = me.cssRules.length, Ce = `{left:${ee.left}px;right:auto;}`;
          H.calendarContainer.classList.toggle("rightMost", !1), H.calendarContainer.classList.toggle("centerMost", !0), me.insertRule(`${ge},${ve}${Ce}`, be), H.calendarContainer.style.left = `${ye}px`, H.calendarContainer.style.right = "auto";
        }
    }
  }
  mt(PickerBaseView, "__name__", "PickerBaseView");
  const R6 = class R6 extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(R6, "__name__", "PickerBase"), R6.define(({ Bool: _ }) => ({
    position: [CalendarPosition, "auto"],
    inline: [_, !1]
  }));
  let PickerBase = R6;
  const DateLike = Or$1(Ref(Date), Str, Float$1), DateLikeList = List$2(Or$1(DateLike, Tuple(DateLike, DateLike), Struct$1({ from: DateLike, to: DateLike })));
  class BaseDatePickerView extends PickerBaseView {
    _format_date(_) {
      const { picker: H } = this;
      return H.formatDate(_, H.config.dateFormat);
    }
    connect_signals() {
      super.connect_signals();
      const { value: _, min_date: H, max_date: V, disabled_dates: q, enabled_dates: G, date_format: J } = this.model.properties;
      this.connect(_.change, () => {
        const { value: Z } = this.model;
        Z != null ? this.picker.setDate(Z) : this.picker.clear();
      }), this.connect(H.change, () => this.picker.set("minDate", this.model.min_date)), this.connect(V.change, () => this.picker.set("maxDate", this.model.max_date)), this.connect(q.change, () => {
        const { disabled_dates: Z } = this.model;
        this.picker.set("disable", Z != null ? this._convert_date_list(Z) : []);
      }), this.connect(G.change, () => {
        const { enabled_dates: Z } = this.model;
        Z != null ? this.picker.set("enable", this._convert_date_list(Z)) : (this.picker.config._enable = void 0, this.picker.redraw(), this.picker.updateValue(!0));
      }), this.connect(J.change, () => this.picker.set("altFormat", this.model.date_format));
    }
    get flatpickr_options() {
      const { value: _, min_date: H, max_date: V, disabled_dates: q, enabled_dates: G, date_format: J } = this.model, Z = super.flatpickr_options;
      return Z.altInput = !0, Z.altFormat = J, Z.dateFormat = "Y-m-d", _ != null && (Z.defaultDate = _), H != null && (Z.minDate = H), V != null && (Z.maxDate = V), q != null && (Z.disable = this._convert_date_list(q)), G != null && (Z.enable = this._convert_date_list(G)), Z;
    }
    _convert_date_list(_) {
      const H = [];
      for (const V of _)
        if (isArray$3(V)) {
          const [q, G] = V;
          H.push({ from: q, to: G });
        } else
          H.push(V);
      return H;
    }
  }
  mt(BaseDatePickerView, "__name__", "BaseDatePickerView");
  const O6 = class O6 extends PickerBase {
    constructor(_) {
      super(_);
    }
  };
  mt(O6, "__name__", "BaseDatePicker"), O6.define(({ Nullable: _ }) => ({
    min_date: [_(DateLike), null],
    max_date: [_(DateLike), null],
    disabled_dates: [_(DateLikeList), null],
    enabled_dates: [_(DateLikeList), null],
    date_format: [Str, "Y-m-d"]
  }));
  let BaseDatePicker = O6;
  class DatePickerView extends BaseDatePickerView {
    get flatpickr_options() {
      return {
        ...super.flatpickr_options,
        mode: "single"
      };
    }
    _on_change(_) {
      assert$2(_.length <= 1), this.model.value = (() => {
        if (_.length == 0)
          return null;
        {
          const [H] = _;
          return this._format_date(H);
        }
      })();
    }
  }
  mt(DatePickerView, "__name__", "DatePickerView");
  const Xm = class Xm extends BaseDatePicker {
    constructor(_) {
      super(_);
    }
  };
  mt(Xm, "__name__", "DatePicker"), Xm.prototype.default_view = DatePickerView, Xm.define(({ Nullable: _ }) => ({
    value: [_(DateLike), null]
  }));
  let DatePicker = Xm;
  class DateRangePickerView extends BaseDatePickerView {
    get flatpickr_options() {
      return {
        ...super.flatpickr_options,
        mode: "range"
      };
    }
    _on_change(_) {
      switch (_.length) {
        case 0:
          this.model.value = null;
          break;
        case 1:
          break;
        case 2: {
          const [H, V] = _, q = this._format_date(H), G = this._format_date(V);
          this.model.value = [q, G];
          break;
        }
        default:
          assert$2(!1, "invalid length");
      }
    }
  }
  mt(DateRangePickerView, "__name__", "DateRangePickerView");
  const Zm = class Zm extends BaseDatePicker {
    constructor(_) {
      super(_);
    }
  };
  mt(Zm, "__name__", "DateRangePicker"), Zm.prototype.default_view = DateRangePickerView, Zm.define(({ Tuple: _, Nullable: H }) => ({
    value: [H(_(DateLike, DateLike)), null]
  }));
  let DateRangePicker = Zm;
  class BaseDatetimePickerView extends BaseDatePickerView {
    connect_signals() {
      super.connect_signals();
      const { value: _, hour_increment: H, minute_increment: V, second_increment: q, seconds: G, clock: J } = this.model.properties;
      this.connect(_.change, () => {
        const { value: Z } = this.model;
        Z != null ? this.picker.setDate(Z) : this.picker.clear();
      }), this.connect(H.change, () => this.picker.set("hourIncrement", this.model.hour_increment)), this.connect(V.change, () => this.picker.set("minuteIncrement", this.model.minute_increment)), this.connect(q.change, () => this._update_second_increment()), this.connect(G.change, () => this.picker.set("enableSeconds", this.model.seconds)), this.connect(J.change, () => this.picker.set("time_24hr", this.model.clock == "24h"));
    }
    get flatpickr_options() {
      const { hour_increment: _, minute_increment: H, seconds: V, clock: q } = this.model, G = super.flatpickr_options;
      return G.enableTime = !0, G.dateFormat = "Y-m-dTH:i:S", G.hourIncrement = _, G.minuteIncrement = H, G.enableSeconds = V, G.time_24hr = q == "24h", G;
    }
    render() {
      super.render(), this._update_second_increment();
    }
    _update_second_increment() {
      var H;
      const { second_increment: _ } = this.model;
      (H = this.picker.secondElement) == null || H.setAttribute("step", _.toString());
    }
  }
  mt(BaseDatetimePickerView, "__name__", "BaseDatetimePickerView");
  const Km = class Km extends BaseDatePicker {
    constructor(_) {
      super(_);
    }
  };
  mt(Km, "__name__", "BaseDatetimePicker"), Km.define(({ Bool: _, Positive: H, Int: V }) => ({
    hour_increment: [H(V), 1],
    minute_increment: [H(V), 1],
    second_increment: [H(V), 1],
    seconds: [_, !1],
    clock: [Clock, "24h"]
  })), Km.override({
    date_format: "Y-m-d H:i"
  });
  let BaseDatetimePicker = Km;
  class DatetimePickerView extends BaseDatetimePickerView {
    get flatpickr_options() {
      return {
        ...super.flatpickr_options,
        mode: "single"
      };
    }
    _on_change(_) {
      assert$2(_.length <= 1), this.model.value = (() => {
        if (_.length == 0)
          return null;
        {
          const [H] = _;
          return this._format_date(H);
        }
      })();
    }
  }
  mt(DatetimePickerView, "__name__", "DatetimePickerView");
  const Jm = class Jm extends BaseDatetimePicker {
    constructor(_) {
      super(_);
    }
  };
  mt(Jm, "__name__", "DatetimePicker"), Jm.prototype.default_view = DatetimePickerView, Jm.define(({ Nullable: _ }) => ({
    value: [_(DateLike), null]
  }));
  let DatetimePicker = Jm;
  class DatetimeRangePickerView extends BaseDatetimePickerView {
    get flatpickr_options() {
      return {
        ...super.flatpickr_options,
        mode: "range"
      };
    }
    _on_change(_) {
      switch (_.length) {
        case 0:
          this.model.value = null;
          break;
        case 1:
          break;
        case 2: {
          const [H, V] = _, q = this._format_date(H), G = this._format_date(V);
          this.model.value = [q, G];
          break;
        }
        default:
          assert$2(!1, "invalid length");
      }
    }
  }
  mt(DatetimeRangePickerView, "__name__", "DatetimeRangePickerView");
  const e4 = class e4 extends BaseDatetimePicker {
    constructor(_) {
      super(_);
    }
  };
  mt(e4, "__name__", "DatetimeRangePicker"), e4.prototype.default_view = DatetimeRangePickerView, e4.define(({ Nullable: _, Tuple: H }) => ({
    value: [_(H(DateLike, DateLike)), null]
  }));
  let DatetimeRangePicker = e4;
  const clearfix = "bk-clearfix", clearfix_css = '.bk-clearfix:before,.bk-clearfix:after{content:"";display:table;}.bk-clearfix:after{clear:both;}';
  class MarkupView extends WidgetView$1 {
    constructor() {
      super(...arguments);
      mt(this, "markup_el");
      mt(this, "_auto_width", "fit-content");
      mt(this, "_auto_height", "auto");
    }
    async lazy_initialize() {
      await super.lazy_initialize(), (this.provider.status == "not_started" || this.provider.status == "loading") && this.provider.ready.connect(() => {
        this.contains_tex_string(this.model.text) && this.rerender();
      });
    }
    has_math_disabled() {
      return this.model.disable_math || !this.contains_tex_string(this.model.text);
    }
    rerender() {
      this.render();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => {
        this.rerender();
      });
    }
    stylesheets() {
      return [...super.stylesheets(), clearfix_css, "p { margin: 0; }"];
    }
    render() {
      super.render(), this.markup_el = div({ class: clearfix, style: { display: "inline-block" } }), this.shadow_el.appendChild(this.markup_el), (this.provider.status == "failed" || this.provider.status == "loaded") && (this._has_finished = !0);
    }
  }
  mt(MarkupView, "__name__", "MarkupView");
  const I6 = class I6 extends Widget {
    constructor(_) {
      super(_);
    }
  };
  mt(I6, "__name__", "Markup"), I6.define(({ Bool: _, Str: H }) => ({
    text: [H, ""],
    disable_math: [_, !1]
  }));
  let Markup = I6;
  class DivView extends MarkupView {
    render() {
      super.render(), this.model.render_as_text ? this.markup_el.textContent = this.model.text : this.markup_el.innerHTML = this.has_math_disabled() ? this.model.text : this.process_tex(this.model.text);
    }
  }
  mt(DivView, "__name__", "DivView");
  const t4 = class t4 extends Markup {
    constructor(_) {
      super(_);
    }
  };
  mt(t4, "__name__", "Div"), t4.prototype.default_view = DivView, t4.define(({ Bool: _ }) => ({
    render_as_text: [_, !1]
  }));
  let Div = t4;
  const caret = "bk-caret", down = "bk-down", carets_css = ":host{--caret-width:4px;}.bk-caret{display:inline-block;vertical-align:middle;width:0;height:0;margin:0 5px;}.bk-caret.bk-down{border-top:var(--caret-width) solid;}.bk-caret.bk-up{border-bottom:var(--caret-width) solid;}.bk-caret.bk-down,.bk-caret.bk-up{border-right:var(--caret-width) solid transparent;border-left:var(--caret-width) solid transparent;}.bk-caret.bk-left{border-right:var(--caret-width) solid;}.bk-caret.bk-right{border-left:var(--caret-width) solid;}.bk-caret.bk-left,.bk-caret.bk-right{border-top:var(--caret-width) solid transparent;border-bottom:var(--caret-width) solid transparent;}";
  class DropdownView extends AbstractButtonView {
    constructor() {
      super(...arguments);
      mt(this, "_open", !1);
      mt(this, "menu");
    }
    stylesheets() {
      return [...super.stylesheets(), dropdown_css, carets_css];
    }
    render() {
      super.render();
      const H = div({ class: [caret, down] });
      if (!this.model.is_split)
        this.button_el.appendChild(H);
      else {
        const q = this._render_button(H);
        q.classList.add(dropdown_toggle), q.addEventListener("click", () => this._toggle_menu()), this.group_el.appendChild(q);
      }
      const V = this.model.menu.map((q, G) => {
        if (q == null)
          return div({ class: divider });
        {
          const J = isString$2(q) ? q : q[0], Z = div(J);
          return Z.addEventListener("click", () => this._item_click(G)), Z;
        }
      });
      this.menu = div({ class: [menu, below] }, V), this.shadow_el.appendChild(this.menu), undisplay(this.menu);
    }
    _show_menu() {
      if (!this._open) {
        this._open = !0, display(this.menu);
        const H = (V) => {
          V.composedPath().includes(this.el) || (document.removeEventListener("click", H), this._hide_menu());
        };
        document.addEventListener("click", H);
      }
    }
    _hide_menu() {
      this._open && (this._open = !1, undisplay(this.menu));
    }
    _toggle_menu() {
      this._open ? this._hide_menu() : this._show_menu();
    }
    click() {
      this.model.is_split ? (this._hide_menu(), this.model.trigger_event(new ButtonClick()), super.click()) : this._toggle_menu();
    }
    _item_click(H) {
      this._hide_menu();
      const V = this.model.menu[H];
      if (V != null) {
        const q = isString$2(V) ? V : V[1];
        isString$2(q) ? this.model.trigger_event(new MenuItemClick(q)) : execute(q, this.model, { index: H });
      }
    }
  }
  mt(DropdownView, "__name__", "DropdownView");
  const Zp = class Zp extends AbstractButton {
    constructor(_) {
      super(_);
    }
    get is_split() {
      return this.split;
    }
  };
  mt(Zp, "__name__", "Dropdown"), Zp.prototype.default_view = DropdownView, Zp.define(({ Null: _, Bool: H, Str: V, List: q, Tuple: G, Or: J }) => ({
    split: [H, !1],
    menu: [q(J(V, G(V, J(
      V
      /*TODO*/
    )), _)), []]
  })), Zp.override({
    label: "Dropdown"
  });
  let Dropdown = Zp;
  class FileInputView extends InputWidgetView {
    stylesheets() {
      return [...super.stylesheets(), buttons_css];
    }
    _render_input() {
      const { multiple: _, disabled: H } = this.model, V = (() => {
        const { accept: q } = this.model;
        return isString$2(q) ? q : q.join(",");
      })();
      return this.input_el = input$1({ type: "file", class: input, multiple: _, accept: V, disabled: H });
    }
    render() {
      super.render(), this.input_el.addEventListener("change", async () => {
        const { files: _ } = this.input_el;
        _ != null && await this.load_files(_);
      });
    }
    async load_files(_) {
      const H = [], V = [], q = [];
      for (const te of _) {
        const ne = await this._read_file(te), [, ee = "", , ae = ""] = ne.split(/[:;,]/, 4);
        H.push(ae), V.push(te.name), q.push(ee);
      }
      const [G, J, Z] = this.model.multiple ? [H, V, q] : _.length != 0 ? [H[0], V[0], q[0]] : ["", "", ""];
      this.model.setv({ value: G, filename: J, mime_type: Z });
    }
    _read_file(_) {
      return new Promise((H, V) => {
        const q = new FileReader();
        q.onload = () => {
          const { result: G } = q;
          G != null ? H(G) : V(q.error ?? new Error(`unable to read '${_.name}'`));
        }, q.readAsDataURL(_);
      });
    }
  }
  mt(FileInputView, "__name__", "FileInputView");
  const n4 = class n4 extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(n4, "__name__", "FileInput"), n4.prototype.default_view = FileInputView, n4.define(({ Bool: _, Str: H, List: V, Or: q }) => ({
    value: [q(H, V(H)), unset, { readonly: !0 }],
    mime_type: [q(H, V(H)), unset, { readonly: !0 }],
    filename: [q(H, V(H)), unset, { readonly: !0 }],
    accept: [q(H, V(H)), ""],
    multiple: [_, !1]
  }));
  let FileInput = n4;
  class HelpButtonView extends AbstractButtonView {
    constructor() {
      super(...arguments);
      mt(this, "tooltip");
    }
    *children() {
      yield* super.children(), yield this.tooltip;
    }
    async lazy_initialize() {
      await super.lazy_initialize();
      const { tooltip: H } = this.model;
      this.tooltip = await build_view(H, { parent: this });
    }
    remove() {
      this.tooltip.remove(), super.remove();
    }
    render() {
      super.render();
      let H = !1;
      const V = (q) => {
        this.tooltip.model.setv({
          visible: q,
          closable: H
        });
      };
      this.on_change(this.tooltip.model.properties.visible, () => {
        const { visible: q } = this.tooltip.model;
        q || (H = !1), V(q);
      }), this.el.addEventListener("mouseenter", () => {
        V(!0);
      }), this.el.addEventListener("mouseleave", () => {
        H || V(!1);
      }), document.addEventListener("mousedown", (q) => {
        const G = q.composedPath();
        G.includes(this.tooltip.el) || (G.includes(this.el) ? (H = !H, V(H)) : (H = !1, V(!1)));
      }), window.addEventListener("blur", () => {
        H = !1, V(!1);
      });
    }
  }
  mt(HelpButtonView, "__name__", "HelpButtonView");
  const Kp = class Kp extends AbstractButton {
    constructor(_) {
      super(_);
    }
  };
  mt(Kp, "__name__", "HelpButton"), Kp.prototype.default_view = HelpButtonView, Kp.define(({ Ref: _ }) => ({
    tooltip: [_(Tooltip)]
  })), Kp.override({
    label: "",
    icon: () => new BuiltinIcon({ icon_name: "help", size: 18 }),
    button_type: "default"
  });
  let HelpButton = Kp;
  var choices = { exports: {} };
  /*! choices.js v10.2.0 | © 2022 Josh Johnson | https://github.com/jshjohnson/Choices#readme */
  var hasRequiredChoices;
  function requireChoices() {
    return hasRequiredChoices || (hasRequiredChoices = 1, function(O, _) {
      (function(V, q) {
        O.exports = q();
      })(window, function() {
        return (
          /******/
          function() {
            var H = {
              /***/
              282: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.clearChoices = Z.activateChoices = Z.filterChoices = Z.addChoice = void 0;
                  var ne = te(883), ee = function(ce) {
                    var le = ce.value, ue = ce.label, de = ce.id, fe = ce.groupId, he = ce.disabled, me = ce.elementId, pe = ce.customProperties, ye = ce.placeholder, ge = ce.keyCode;
                    return {
                      type: ne.ACTION_TYPES.ADD_CHOICE,
                      value: le,
                      label: ue,
                      id: de,
                      groupId: fe,
                      disabled: he,
                      elementId: me,
                      customProperties: pe,
                      placeholder: ye,
                      keyCode: ge
                    };
                  };
                  Z.addChoice = ee;
                  var ae = function(ce) {
                    return {
                      type: ne.ACTION_TYPES.FILTER_CHOICES,
                      results: ce
                    };
                  };
                  Z.filterChoices = ae;
                  var oe = function(ce) {
                    return ce === void 0 && (ce = !0), {
                      type: ne.ACTION_TYPES.ACTIVATE_CHOICES,
                      active: ce
                    };
                  };
                  Z.activateChoices = oe;
                  var se = function() {
                    return {
                      type: ne.ACTION_TYPES.CLEAR_CHOICES
                    };
                  };
                  Z.clearChoices = se;
                }
              ),
              /***/
              783: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.addGroup = void 0;
                  var ne = te(883), ee = function(ae) {
                    var oe = ae.value, se = ae.id, ce = ae.active, le = ae.disabled;
                    return {
                      type: ne.ACTION_TYPES.ADD_GROUP,
                      value: oe,
                      id: se,
                      active: ce,
                      disabled: le
                    };
                  };
                  Z.addGroup = ee;
                }
              ),
              /***/
              464: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.highlightItem = Z.removeItem = Z.addItem = void 0;
                  var ne = te(883), ee = function(se) {
                    var ce = se.value, le = se.label, ue = se.id, de = se.choiceId, fe = se.groupId, he = se.customProperties, me = se.placeholder, pe = se.keyCode;
                    return {
                      type: ne.ACTION_TYPES.ADD_ITEM,
                      value: ce,
                      label: le,
                      id: ue,
                      choiceId: de,
                      groupId: fe,
                      customProperties: he,
                      placeholder: me,
                      keyCode: pe
                    };
                  };
                  Z.addItem = ee;
                  var ae = function(se, ce) {
                    return {
                      type: ne.ACTION_TYPES.REMOVE_ITEM,
                      id: se,
                      choiceId: ce
                    };
                  };
                  Z.removeItem = ae;
                  var oe = function(se, ce) {
                    return {
                      type: ne.ACTION_TYPES.HIGHLIGHT_ITEM,
                      id: se,
                      highlighted: ce
                    };
                  };
                  Z.highlightItem = oe;
                }
              ),
              /***/
              137: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.setIsLoading = Z.resetTo = Z.clearAll = void 0;
                  var ne = te(883), ee = function() {
                    return {
                      type: ne.ACTION_TYPES.CLEAR_ALL
                    };
                  };
                  Z.clearAll = ee;
                  var ae = function(se) {
                    return {
                      type: ne.ACTION_TYPES.RESET_TO,
                      state: se
                    };
                  };
                  Z.resetTo = ae;
                  var oe = function(se) {
                    return {
                      type: ne.ACTION_TYPES.SET_IS_LOADING,
                      isLoading: se
                    };
                  };
                  Z.setIsLoading = oe;
                }
              ),
              /***/
              373: (
                /***/
                function(J, Z, te) {
                  var ne = this && this.__spreadArray || function(Se, we, Ee) {
                    if (Ee || arguments.length === 2) for (var xe = 0, Le = we.length, Qe; xe < Le; xe++)
                      (Qe || !(xe in we)) && (Qe || (Qe = Array.prototype.slice.call(we, 0, xe)), Qe[xe] = we[xe]);
                    return Se.concat(Qe || Array.prototype.slice.call(we));
                  }, ee = this && this.__importDefault || function(Se) {
                    return Se && Se.__esModule ? Se : {
                      default: Se
                    };
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ae = ee(te(996)), oe = ee(te(221)), se = te(282), ce = te(783), le = te(464), ue = te(137), de = te(520), fe = te(883), he = te(789), me = te(799), pe = te(655), ye = ee(te(744)), ge = ee(te(686)), ve = "-ms-scroll-limit" in document.documentElement.style && "-ms-ime-align" in document.documentElement.style, be = {}, Ce = (
                    /** @class */
                    function() {
                      function Se(we, Ee) {
                        we === void 0 && (we = "[data-choice]"), Ee === void 0 && (Ee = {});
                        var xe = this;
                        Ee.allowHTML === void 0 && console.warn("Deprecation warning: allowHTML will default to false in a future release. To render HTML in Choices, you will need to set it to true. Setting allowHTML will suppress this message."), this.config = ae.default.all(
                          [he.DEFAULT_CONFIG, Se.defaults.options, Ee],
                          // When merging array configs, replace with a copy of the userConfig array,
                          // instead of concatenating with the default array
                          {
                            arrayMerge: function(Pe, $e) {
                              return ne([], $e, !0);
                            }
                          }
                        );
                        var Le = (0, me.diff)(this.config, he.DEFAULT_CONFIG);
                        Le.length && console.warn("Unknown config option(s) passed", Le.join(", "));
                        var Qe = typeof we == "string" ? document.querySelector(we) : we;
                        if (!(Qe instanceof HTMLInputElement || Qe instanceof HTMLSelectElement))
                          throw TypeError("Expected one of the following types text|select-one|select-multiple");
                        if (this._isTextElement = Qe.type === fe.TEXT_TYPE, this._isSelectOneElement = Qe.type === fe.SELECT_ONE_TYPE, this._isSelectMultipleElement = Qe.type === fe.SELECT_MULTIPLE_TYPE, this._isSelectElement = this._isSelectOneElement || this._isSelectMultipleElement, this.config.searchEnabled = this._isSelectMultipleElement || this.config.searchEnabled, ["auto", "always"].includes("".concat(this.config.renderSelectedChoices)) || (this.config.renderSelectedChoices = "auto"), Ee.addItemFilter && typeof Ee.addItemFilter != "function") {
                          var ke = Ee.addItemFilter instanceof RegExp ? Ee.addItemFilter : new RegExp(Ee.addItemFilter);
                          this.config.addItemFilter = ke.test.bind(ke);
                        }
                        if (this._isTextElement ? this.passedElement = new de.WrappedInput({
                          element: Qe,
                          classNames: this.config.classNames,
                          delimiter: this.config.delimiter
                        }) : this.passedElement = new de.WrappedSelect({
                          element: Qe,
                          classNames: this.config.classNames,
                          template: function(Pe) {
                            return xe._templates.option(Pe);
                          }
                        }), this.initialised = !1, this._store = new ye.default(), this._initialState = pe.defaultState, this._currentState = pe.defaultState, this._prevState = pe.defaultState, this._currentValue = "", this._canSearch = !!this.config.searchEnabled, this._isScrollingOnIe = !1, this._highlightPosition = 0, this._wasTap = !0, this._placeholderValue = this._generatePlaceholderValue(), this._baseId = (0, me.generateId)(this.passedElement.element, "choices-"), this._direction = this.passedElement.dir, !this._direction) {
                          var Me = window.getComputedStyle(this.passedElement.element).direction, Re = window.getComputedStyle(document.documentElement).direction;
                          Me !== Re && (this._direction = Me);
                        }
                        if (this._idNames = {
                          itemChoice: "item-choice"
                        }, this._isSelectElement && (this._presetGroups = this.passedElement.optionGroups, this._presetOptions = this.passedElement.options), this._presetChoices = this.config.choices, this._presetItems = this.config.items, this.passedElement.value && this._isTextElement) {
                          var De = this.passedElement.value.split(this.config.delimiter);
                          this._presetItems = this._presetItems.concat(De);
                        }
                        if (this.passedElement.options && this.passedElement.options.forEach(function(Pe) {
                          xe._presetChoices.push({
                            value: Pe.value,
                            label: Pe.innerHTML,
                            selected: !!Pe.selected,
                            disabled: Pe.disabled || Pe.parentNode.disabled,
                            placeholder: Pe.value === "" || Pe.hasAttribute("placeholder"),
                            customProperties: (0, me.parseCustomProperties)(Pe.dataset.customProperties)
                          });
                        }), this._render = this._render.bind(this), this._onFocus = this._onFocus.bind(this), this._onBlur = this._onBlur.bind(this), this._onKeyUp = this._onKeyUp.bind(this), this._onKeyDown = this._onKeyDown.bind(this), this._onClick = this._onClick.bind(this), this._onTouchMove = this._onTouchMove.bind(this), this._onTouchEnd = this._onTouchEnd.bind(this), this._onMouseDown = this._onMouseDown.bind(this), this._onMouseOver = this._onMouseOver.bind(this), this._onFormReset = this._onFormReset.bind(this), this._onSelectKey = this._onSelectKey.bind(this), this._onEnterKey = this._onEnterKey.bind(this), this._onEscapeKey = this._onEscapeKey.bind(this), this._onDirectionKey = this._onDirectionKey.bind(this), this._onDeleteKey = this._onDeleteKey.bind(this), this.passedElement.isActive) {
                          this.config.silent || console.warn("Trying to initialise Choices on element already initialised", {
                            element: we
                          }), this.initialised = !0;
                          return;
                        }
                        this.init();
                      }
                      return Object.defineProperty(Se, "defaults", {
                        get: function() {
                          return Object.preventExtensions({
                            get options() {
                              return be;
                            },
                            get templates() {
                              return ge.default;
                            }
                          });
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Se.prototype.init = function() {
                        if (!this.initialised) {
                          this._createTemplates(), this._createElements(), this._createStructure(), this._store.subscribe(this._render), this._render(), this._addEventListeners();
                          var we = !this.config.addItems || this.passedElement.element.hasAttribute("disabled");
                          we && this.disable(), this.initialised = !0;
                          var Ee = this.config.callbackOnInit;
                          Ee && typeof Ee == "function" && Ee.call(this);
                        }
                      }, Se.prototype.destroy = function() {
                        this.initialised && (this._removeEventListeners(), this.passedElement.reveal(), this.containerOuter.unwrap(this.passedElement.element), this.clearStore(), this._isSelectElement && (this.passedElement.options = this._presetOptions), this._templates = ge.default, this.initialised = !1);
                      }, Se.prototype.enable = function() {
                        return this.passedElement.isDisabled && this.passedElement.enable(), this.containerOuter.isDisabled && (this._addEventListeners(), this.input.enable(), this.containerOuter.enable()), this;
                      }, Se.prototype.disable = function() {
                        return this.passedElement.isDisabled || this.passedElement.disable(), this.containerOuter.isDisabled || (this._removeEventListeners(), this.input.disable(), this.containerOuter.disable()), this;
                      }, Se.prototype.highlightItem = function(we, Ee) {
                        if (Ee === void 0 && (Ee = !0), !we || !we.id)
                          return this;
                        var xe = we.id, Le = we.groupId, Qe = Le === void 0 ? -1 : Le, ke = we.value, Me = ke === void 0 ? "" : ke, Re = we.label, De = Re === void 0 ? "" : Re, Pe = Qe >= 0 ? this._store.getGroupById(Qe) : null;
                        return this._store.dispatch((0, le.highlightItem)(xe, !0)), Ee && this.passedElement.triggerEvent(fe.EVENTS.highlightItem, {
                          id: xe,
                          value: Me,
                          label: De,
                          groupValue: Pe && Pe.value ? Pe.value : null
                        }), this;
                      }, Se.prototype.unhighlightItem = function(we) {
                        if (!we || !we.id)
                          return this;
                        var Ee = we.id, xe = we.groupId, Le = xe === void 0 ? -1 : xe, Qe = we.value, ke = Qe === void 0 ? "" : Qe, Me = we.label, Re = Me === void 0 ? "" : Me, De = Le >= 0 ? this._store.getGroupById(Le) : null;
                        return this._store.dispatch((0, le.highlightItem)(Ee, !1)), this.passedElement.triggerEvent(fe.EVENTS.highlightItem, {
                          id: Ee,
                          value: ke,
                          label: Re,
                          groupValue: De && De.value ? De.value : null
                        }), this;
                      }, Se.prototype.highlightAll = function() {
                        var we = this;
                        return this._store.items.forEach(function(Ee) {
                          return we.highlightItem(Ee);
                        }), this;
                      }, Se.prototype.unhighlightAll = function() {
                        var we = this;
                        return this._store.items.forEach(function(Ee) {
                          return we.unhighlightItem(Ee);
                        }), this;
                      }, Se.prototype.removeActiveItemsByValue = function(we) {
                        var Ee = this;
                        return this._store.activeItems.filter(function(xe) {
                          return xe.value === we;
                        }).forEach(function(xe) {
                          return Ee._removeItem(xe);
                        }), this;
                      }, Se.prototype.removeActiveItems = function(we) {
                        var Ee = this;
                        return this._store.activeItems.filter(function(xe) {
                          var Le = xe.id;
                          return Le !== we;
                        }).forEach(function(xe) {
                          return Ee._removeItem(xe);
                        }), this;
                      }, Se.prototype.removeHighlightedItems = function(we) {
                        var Ee = this;
                        return we === void 0 && (we = !1), this._store.highlightedActiveItems.forEach(function(xe) {
                          Ee._removeItem(xe), we && Ee._triggerChange(xe.value);
                        }), this;
                      }, Se.prototype.showDropdown = function(we) {
                        var Ee = this;
                        return this.dropdown.isActive ? this : (requestAnimationFrame(function() {
                          Ee.dropdown.show(), Ee.containerOuter.open(Ee.dropdown.distanceFromTopWindow), !we && Ee._canSearch && Ee.input.focus(), Ee.passedElement.triggerEvent(fe.EVENTS.showDropdown, {});
                        }), this);
                      }, Se.prototype.hideDropdown = function(we) {
                        var Ee = this;
                        return this.dropdown.isActive ? (requestAnimationFrame(function() {
                          Ee.dropdown.hide(), Ee.containerOuter.close(), !we && Ee._canSearch && (Ee.input.removeActiveDescendant(), Ee.input.blur()), Ee.passedElement.triggerEvent(fe.EVENTS.hideDropdown, {});
                        }), this) : this;
                      }, Se.prototype.getValue = function(we) {
                        we === void 0 && (we = !1);
                        var Ee = this._store.activeItems.reduce(function(xe, Le) {
                          var Qe = we ? Le.value : Le;
                          return xe.push(Qe), xe;
                        }, []);
                        return this._isSelectOneElement ? Ee[0] : Ee;
                      }, Se.prototype.setValue = function(we) {
                        var Ee = this;
                        return this.initialised ? (we.forEach(function(xe) {
                          return Ee._setChoiceOrItem(xe);
                        }), this) : this;
                      }, Se.prototype.setChoiceByValue = function(we) {
                        var Ee = this;
                        if (!this.initialised || this._isTextElement)
                          return this;
                        var xe = Array.isArray(we) ? we : [we];
                        return xe.forEach(function(Le) {
                          return Ee._findAndSelectChoiceByValue(Le);
                        }), this;
                      }, Se.prototype.setChoices = function(we, Ee, xe, Le) {
                        var Qe = this;
                        if (we === void 0 && (we = []), Ee === void 0 && (Ee = "value"), xe === void 0 && (xe = "label"), Le === void 0 && (Le = !1), !this.initialised)
                          throw new ReferenceError("setChoices was called on a non-initialized instance of Choices");
                        if (!this._isSelectElement)
                          throw new TypeError("setChoices can't be used with INPUT based Choices");
                        if (typeof Ee != "string" || !Ee)
                          throw new TypeError("value parameter must be a name of 'value' field in passed objects");
                        if (Le && this.clearChoices(), typeof we == "function") {
                          var ke = we(this);
                          if (typeof Promise == "function" && ke instanceof Promise)
                            return new Promise(function(Me) {
                              return requestAnimationFrame(Me);
                            }).then(function() {
                              return Qe._handleLoadingState(!0);
                            }).then(function() {
                              return ke;
                            }).then(function(Me) {
                              return Qe.setChoices(Me, Ee, xe, Le);
                            }).catch(function(Me) {
                              Qe.config.silent || console.error(Me);
                            }).then(function() {
                              return Qe._handleLoadingState(!1);
                            }).then(function() {
                              return Qe;
                            });
                          if (!Array.isArray(ke))
                            throw new TypeError(".setChoices first argument function must return either array of choices or Promise, got: ".concat(typeof ke));
                          return this.setChoices(ke, Ee, xe, !1);
                        }
                        if (!Array.isArray(we))
                          throw new TypeError(".setChoices must be called either with array of choices with a function resulting into Promise of array of choices");
                        return this.containerOuter.removeLoadingState(), this._startLoading(), we.forEach(function(Me) {
                          if (Me.choices)
                            Qe._addGroup({
                              id: Me.id ? parseInt("".concat(Me.id), 10) : null,
                              group: Me,
                              valueKey: Ee,
                              labelKey: xe
                            });
                          else {
                            var Re = Me;
                            Qe._addChoice({
                              value: Re[Ee],
                              label: Re[xe],
                              isSelected: !!Re.selected,
                              isDisabled: !!Re.disabled,
                              placeholder: !!Re.placeholder,
                              customProperties: Re.customProperties
                            });
                          }
                        }), this._stopLoading(), this;
                      }, Se.prototype.clearChoices = function() {
                        return this._store.dispatch((0, se.clearChoices)()), this;
                      }, Se.prototype.clearStore = function() {
                        return this._store.dispatch((0, ue.clearAll)()), this;
                      }, Se.prototype.clearInput = function() {
                        var we = !this._isSelectOneElement;
                        return this.input.clear(we), !this._isTextElement && this._canSearch && (this._isSearching = !1, this._store.dispatch((0, se.activateChoices)(!0))), this;
                      }, Se.prototype._render = function() {
                        if (!this._store.isLoading()) {
                          this._currentState = this._store.state;
                          var we = this._currentState.choices !== this._prevState.choices || this._currentState.groups !== this._prevState.groups || this._currentState.items !== this._prevState.items, Ee = this._isSelectElement, xe = this._currentState.items !== this._prevState.items;
                          we && (Ee && this._renderChoices(), xe && this._renderItems(), this._prevState = this._currentState);
                        }
                      }, Se.prototype._renderChoices = function() {
                        var we = this, Ee = this._store, xe = Ee.activeGroups, Le = Ee.activeChoices, Qe = document.createDocumentFragment();
                        if (this.choiceList.clear(), this.config.resetScrollPosition && requestAnimationFrame(function() {
                          return we.choiceList.scrollToTop();
                        }), xe.length >= 1 && !this._isSearching) {
                          var ke = Le.filter(function($e) {
                            return $e.placeholder === !0 && $e.groupId === -1;
                          });
                          ke.length >= 1 && (Qe = this._createChoicesFragment(ke, Qe)), Qe = this._createGroupsFragment(xe, Le, Qe);
                        } else Le.length >= 1 && (Qe = this._createChoicesFragment(Le, Qe));
                        if (Qe.childNodes && Qe.childNodes.length > 0) {
                          var Me = this._store.activeItems, Re = this._canAddItem(Me, this.input.value);
                          if (Re.response)
                            this.choiceList.append(Qe), this._highlightChoice();
                          else {
                            var De = this._getTemplate("notice", Re.notice);
                            this.choiceList.append(De);
                          }
                        } else {
                          var Pe = void 0, De = void 0;
                          this._isSearching ? (De = typeof this.config.noResultsText == "function" ? this.config.noResultsText() : this.config.noResultsText, Pe = this._getTemplate("notice", De, "no-results")) : (De = typeof this.config.noChoicesText == "function" ? this.config.noChoicesText() : this.config.noChoicesText, Pe = this._getTemplate("notice", De, "no-choices")), this.choiceList.append(Pe);
                        }
                      }, Se.prototype._renderItems = function() {
                        var we = this._store.activeItems || [];
                        this.itemList.clear();
                        var Ee = this._createItemsFragment(we);
                        Ee.childNodes && this.itemList.append(Ee);
                      }, Se.prototype._createGroupsFragment = function(we, Ee, xe) {
                        var Le = this;
                        xe === void 0 && (xe = document.createDocumentFragment());
                        var Qe = function(ke) {
                          return Ee.filter(function(Me) {
                            return Le._isSelectOneElement ? Me.groupId === ke.id : Me.groupId === ke.id && (Le.config.renderSelectedChoices === "always" || !Me.selected);
                          });
                        };
                        return this.config.shouldSort && we.sort(this.config.sorter), we.forEach(function(ke) {
                          var Me = Qe(ke);
                          if (Me.length >= 1) {
                            var Re = Le._getTemplate("choiceGroup", ke);
                            xe.appendChild(Re), Le._createChoicesFragment(Me, xe, !0);
                          }
                        }), xe;
                      }, Se.prototype._createChoicesFragment = function(we, Ee, xe) {
                        var Le = this;
                        Ee === void 0 && (Ee = document.createDocumentFragment()), xe === void 0 && (xe = !1);
                        var Qe = this.config, ke = Qe.renderSelectedChoices, Me = Qe.searchResultLimit, Re = Qe.renderChoiceLimit, De = this._isSearching ? me.sortByScore : this.config.sorter, Pe = function(We) {
                          var at = ke === "auto" ? Le._isSelectOneElement || !We.selected : !0;
                          if (at) {
                            var ct = Le._getTemplate("choice", We, Le.config.itemSelectText);
                            Ee.appendChild(ct);
                          }
                        }, $e = we;
                        ke === "auto" && !this._isSelectOneElement && ($e = we.filter(function(We) {
                          return !We.selected;
                        }));
                        var qe = $e.reduce(function(We, at) {
                          return at.placeholder ? We.placeholderChoices.push(at) : We.normalChoices.push(at), We;
                        }, {
                          placeholderChoices: [],
                          normalChoices: []
                        }), Xe = qe.placeholderChoices, Ve = qe.normalChoices;
                        (this.config.shouldSort || this._isSearching) && Ve.sort(De);
                        var Ze = $e.length, Ge = this._isSelectOneElement ? ne(ne([], Xe, !0), Ve, !0) : Ve;
                        this._isSearching ? Ze = Me : Re && Re > 0 && !xe && (Ze = Re);
                        for (var ze = 0; ze < Ze; ze += 1)
                          Ge[ze] && Pe(Ge[ze]);
                        return Ee;
                      }, Se.prototype._createItemsFragment = function(we, Ee) {
                        var xe = this;
                        Ee === void 0 && (Ee = document.createDocumentFragment());
                        var Le = this.config, Qe = Le.shouldSortItems, ke = Le.sorter, Me = Le.removeItemButton;
                        Qe && !this._isSelectOneElement && we.sort(ke), this._isTextElement ? this.passedElement.value = we.map(function(De) {
                          var Pe = De.value;
                          return Pe;
                        }).join(this.config.delimiter) : this.passedElement.options = we;
                        var Re = function(De) {
                          var Pe = xe._getTemplate("item", De, Me);
                          Ee.appendChild(Pe);
                        };
                        return we.forEach(Re), Ee;
                      }, Se.prototype._triggerChange = function(we) {
                        we != null && this.passedElement.triggerEvent(fe.EVENTS.change, {
                          value: we
                        });
                      }, Se.prototype._selectPlaceholderChoice = function(we) {
                        this._addItem({
                          value: we.value,
                          label: we.label,
                          choiceId: we.id,
                          groupId: we.groupId,
                          placeholder: we.placeholder
                        }), this._triggerChange(we.value);
                      }, Se.prototype._handleButtonAction = function(we, Ee) {
                        if (!(!we || !Ee || !this.config.removeItems || !this.config.removeItemButton)) {
                          var xe = Ee.parentNode && Ee.parentNode.dataset.id, Le = xe && we.find(function(Qe) {
                            return Qe.id === parseInt(xe, 10);
                          });
                          Le && (this._removeItem(Le), this._triggerChange(Le.value), this._isSelectOneElement && this._store.placeholderChoice && this._selectPlaceholderChoice(this._store.placeholderChoice));
                        }
                      }, Se.prototype._handleItemAction = function(we, Ee, xe) {
                        var Le = this;
                        if (xe === void 0 && (xe = !1), !(!we || !Ee || !this.config.removeItems || this._isSelectOneElement)) {
                          var Qe = Ee.dataset.id;
                          we.forEach(function(ke) {
                            ke.id === parseInt("".concat(Qe), 10) && !ke.highlighted ? Le.highlightItem(ke) : !xe && ke.highlighted && Le.unhighlightItem(ke);
                          }), this.input.focus();
                        }
                      }, Se.prototype._handleChoiceAction = function(we, Ee) {
                        if (!(!we || !Ee)) {
                          var xe = Ee.dataset.id, Le = xe && this._store.getChoiceById(xe);
                          if (Le) {
                            var Qe = we[0] && we[0].keyCode ? we[0].keyCode : void 0, ke = this.dropdown.isActive;
                            if (Le.keyCode = Qe, this.passedElement.triggerEvent(fe.EVENTS.choice, {
                              choice: Le
                            }), !Le.selected && !Le.disabled) {
                              var Me = this._canAddItem(we, Le.value);
                              Me.response && (this._addItem({
                                value: Le.value,
                                label: Le.label,
                                choiceId: Le.id,
                                groupId: Le.groupId,
                                customProperties: Le.customProperties,
                                placeholder: Le.placeholder,
                                keyCode: Le.keyCode
                              }), this._triggerChange(Le.value));
                            }
                            this.clearInput(), ke && this._isSelectOneElement && (this.hideDropdown(!0), this.containerOuter.focus());
                          }
                        }
                      }, Se.prototype._handleBackspace = function(we) {
                        if (!(!this.config.removeItems || !we)) {
                          var Ee = we[we.length - 1], xe = we.some(function(Le) {
                            return Le.highlighted;
                          });
                          this.config.editItems && !xe && Ee ? (this.input.value = Ee.value, this.input.setWidth(), this._removeItem(Ee), this._triggerChange(Ee.value)) : (xe || this.highlightItem(Ee, !1), this.removeHighlightedItems(!0));
                        }
                      }, Se.prototype._startLoading = function() {
                        this._store.dispatch((0, ue.setIsLoading)(!0));
                      }, Se.prototype._stopLoading = function() {
                        this._store.dispatch((0, ue.setIsLoading)(!1));
                      }, Se.prototype._handleLoadingState = function(we) {
                        we === void 0 && (we = !0);
                        var Ee = this.itemList.getChild(".".concat(this.config.classNames.placeholder));
                        we ? (this.disable(), this.containerOuter.addLoadingState(), this._isSelectOneElement ? Ee ? Ee.innerHTML = this.config.loadingText : (Ee = this._getTemplate("placeholder", this.config.loadingText), Ee && this.itemList.append(Ee)) : this.input.placeholder = this.config.loadingText) : (this.enable(), this.containerOuter.removeLoadingState(), this._isSelectOneElement ? Ee && (Ee.innerHTML = this._placeholderValue || "") : this.input.placeholder = this._placeholderValue || "");
                      }, Se.prototype._handleSearch = function(we) {
                        if (this.input.isFocussed) {
                          var Ee = this._store.choices, xe = this.config, Le = xe.searchFloor, Qe = xe.searchChoices, ke = Ee.some(function(Re) {
                            return !Re.active;
                          });
                          if (we !== null && typeof we < "u" && we.length >= Le) {
                            var Me = Qe ? this._searchChoices(we) : 0;
                            this.passedElement.triggerEvent(fe.EVENTS.search, {
                              value: we,
                              resultCount: Me
                            });
                          } else ke && (this._isSearching = !1, this._store.dispatch((0, se.activateChoices)(!0)));
                        }
                      }, Se.prototype._canAddItem = function(we, Ee) {
                        var xe = !0, Le = typeof this.config.addItemText == "function" ? this.config.addItemText(Ee) : this.config.addItemText;
                        if (!this._isSelectOneElement) {
                          var Qe = (0, me.existsInArray)(we, Ee);
                          this.config.maxItemCount > 0 && this.config.maxItemCount <= we.length && (xe = !1, Le = typeof this.config.maxItemText == "function" ? this.config.maxItemText(this.config.maxItemCount) : this.config.maxItemText), !this.config.duplicateItemsAllowed && Qe && xe && (xe = !1, Le = typeof this.config.uniqueItemText == "function" ? this.config.uniqueItemText(Ee) : this.config.uniqueItemText), this._isTextElement && this.config.addItems && xe && typeof this.config.addItemFilter == "function" && !this.config.addItemFilter(Ee) && (xe = !1, Le = typeof this.config.customAddItemText == "function" ? this.config.customAddItemText(Ee) : this.config.customAddItemText);
                        }
                        return {
                          response: xe,
                          notice: Le
                        };
                      }, Se.prototype._searchChoices = function(we) {
                        var Ee = typeof we == "string" ? we.trim() : we, xe = typeof this._currentValue == "string" ? this._currentValue.trim() : this._currentValue;
                        if (Ee.length < 1 && Ee === "".concat(xe, " "))
                          return 0;
                        var Le = this._store.searchableChoices, Qe = Ee, ke = Object.assign(this.config.fuseOptions, {
                          keys: ne([], this.config.searchFields, !0),
                          includeMatches: !0
                        }), Me = new oe.default(Le, ke), Re = Me.search(Qe);
                        return this._currentValue = Ee, this._highlightPosition = 0, this._isSearching = !0, this._store.dispatch((0, se.filterChoices)(Re)), Re.length;
                      }, Se.prototype._addEventListeners = function() {
                        var we = document.documentElement;
                        we.addEventListener("touchend", this._onTouchEnd, !0), this.containerOuter.element.addEventListener("keydown", this._onKeyDown, !0), this.containerOuter.element.addEventListener("mousedown", this._onMouseDown, !0), we.addEventListener("click", this._onClick, {
                          passive: !0
                        }), we.addEventListener("touchmove", this._onTouchMove, {
                          passive: !0
                        }), this.dropdown.element.addEventListener("mouseover", this._onMouseOver, {
                          passive: !0
                        }), this._isSelectOneElement && (this.containerOuter.element.addEventListener("focus", this._onFocus, {
                          passive: !0
                        }), this.containerOuter.element.addEventListener("blur", this._onBlur, {
                          passive: !0
                        })), this.input.element.addEventListener("keyup", this._onKeyUp, {
                          passive: !0
                        }), this.input.element.addEventListener("focus", this._onFocus, {
                          passive: !0
                        }), this.input.element.addEventListener("blur", this._onBlur, {
                          passive: !0
                        }), this.input.element.form && this.input.element.form.addEventListener("reset", this._onFormReset, {
                          passive: !0
                        }), this.input.addEventListeners();
                      }, Se.prototype._removeEventListeners = function() {
                        var we = document.documentElement;
                        we.removeEventListener("touchend", this._onTouchEnd, !0), this.containerOuter.element.removeEventListener("keydown", this._onKeyDown, !0), this.containerOuter.element.removeEventListener("mousedown", this._onMouseDown, !0), we.removeEventListener("click", this._onClick), we.removeEventListener("touchmove", this._onTouchMove), this.dropdown.element.removeEventListener("mouseover", this._onMouseOver), this._isSelectOneElement && (this.containerOuter.element.removeEventListener("focus", this._onFocus), this.containerOuter.element.removeEventListener("blur", this._onBlur)), this.input.element.removeEventListener("keyup", this._onKeyUp), this.input.element.removeEventListener("focus", this._onFocus), this.input.element.removeEventListener("blur", this._onBlur), this.input.element.form && this.input.element.form.removeEventListener("reset", this._onFormReset), this.input.removeEventListeners();
                      }, Se.prototype._onKeyDown = function(we) {
                        var Ee = we.keyCode, xe = this._store.activeItems, Le = this.input.isFocussed, Qe = this.dropdown.isActive, ke = this.itemList.hasChildren(), Me = String.fromCharCode(Ee), Re = /[^\x00-\x1F]/.test(Me), De = fe.KEY_CODES.BACK_KEY, Pe = fe.KEY_CODES.DELETE_KEY, $e = fe.KEY_CODES.ENTER_KEY, qe = fe.KEY_CODES.A_KEY, Xe = fe.KEY_CODES.ESC_KEY, Ve = fe.KEY_CODES.UP_KEY, Ze = fe.KEY_CODES.DOWN_KEY, Ge = fe.KEY_CODES.PAGE_UP_KEY, ze = fe.KEY_CODES.PAGE_DOWN_KEY;
                        switch (!this._isTextElement && !Qe && Re && (this.showDropdown(), this.input.isFocussed || (this.input.value += we.key.toLowerCase())), Ee) {
                          case qe:
                            return this._onSelectKey(we, ke);
                          case $e:
                            return this._onEnterKey(we, xe, Qe);
                          case Xe:
                            return this._onEscapeKey(Qe);
                          case Ve:
                          case Ge:
                          case Ze:
                          case ze:
                            return this._onDirectionKey(we, Qe);
                          case Pe:
                          case De:
                            return this._onDeleteKey(we, xe, Le);
                        }
                      }, Se.prototype._onKeyUp = function(we) {
                        var Ee = we.target, xe = we.keyCode, Le = this.input.value, Qe = this._store.activeItems, ke = this._canAddItem(Qe, Le), Me = fe.KEY_CODES.BACK_KEY, Re = fe.KEY_CODES.DELETE_KEY;
                        if (this._isTextElement) {
                          var De = ke.notice && Le;
                          if (De) {
                            var Pe = this._getTemplate("notice", ke.notice);
                            this.dropdown.element.innerHTML = Pe.outerHTML, this.showDropdown(!0);
                          } else
                            this.hideDropdown(!0);
                        } else {
                          var $e = xe === Me || xe === Re, qe = $e && Ee && !Ee.value, Xe = !this._isTextElement && this._isSearching, Ve = this._canSearch && ke.response;
                          qe && Xe ? (this._isSearching = !1, this._store.dispatch((0, se.activateChoices)(!0))) : Ve && this._handleSearch(this.input.rawValue);
                        }
                        this._canSearch = this.config.searchEnabled;
                      }, Se.prototype._onSelectKey = function(we, Ee) {
                        var xe = we.ctrlKey, Le = we.metaKey, Qe = xe || Le;
                        if (Qe && Ee) {
                          this._canSearch = !1;
                          var ke = this.config.removeItems && !this.input.value && this.input.element === document.activeElement;
                          ke && this.highlightAll();
                        }
                      }, Se.prototype._onEnterKey = function(we, Ee, xe) {
                        var Le = we.target, Qe = fe.KEY_CODES.ENTER_KEY, ke = Le && Le.hasAttribute("data-button");
                        if (this._isTextElement && Le && Le.value) {
                          var Me = this.input.value, Re = this._canAddItem(Ee, Me);
                          Re.response && (this.hideDropdown(!0), this._addItem({
                            value: Me
                          }), this._triggerChange(Me), this.clearInput());
                        }
                        if (ke && (this._handleButtonAction(Ee, Le), we.preventDefault()), xe) {
                          var De = this.dropdown.getChild(".".concat(this.config.classNames.highlightedState));
                          De && (Ee[0] && (Ee[0].keyCode = Qe), this._handleChoiceAction(Ee, De)), we.preventDefault();
                        } else this._isSelectOneElement && (this.showDropdown(), we.preventDefault());
                      }, Se.prototype._onEscapeKey = function(we) {
                        we && (this.hideDropdown(!0), this.containerOuter.focus());
                      }, Se.prototype._onDirectionKey = function(we, Ee) {
                        var xe = we.keyCode, Le = we.metaKey, Qe = fe.KEY_CODES.DOWN_KEY, ke = fe.KEY_CODES.PAGE_UP_KEY, Me = fe.KEY_CODES.PAGE_DOWN_KEY;
                        if (Ee || this._isSelectOneElement) {
                          this.showDropdown(), this._canSearch = !1;
                          var Re = xe === Qe || xe === Me ? 1 : -1, De = Le || xe === Me || xe === ke, Pe = "[data-choice-selectable]", $e = void 0;
                          if (De)
                            Re > 0 ? $e = this.dropdown.element.querySelector("".concat(Pe, ":last-of-type")) : $e = this.dropdown.element.querySelector(Pe);
                          else {
                            var qe = this.dropdown.element.querySelector(".".concat(this.config.classNames.highlightedState));
                            qe ? $e = (0, me.getAdjacentEl)(qe, Pe, Re) : $e = this.dropdown.element.querySelector(Pe);
                          }
                          $e && ((0, me.isScrolledIntoView)($e, this.choiceList.element, Re) || this.choiceList.scrollToChildElement($e, Re), this._highlightChoice($e)), we.preventDefault();
                        }
                      }, Se.prototype._onDeleteKey = function(we, Ee, xe) {
                        var Le = we.target;
                        !this._isSelectOneElement && !Le.value && xe && (this._handleBackspace(Ee), we.preventDefault());
                      }, Se.prototype._onTouchMove = function() {
                        this._wasTap && (this._wasTap = !1);
                      }, Se.prototype._onTouchEnd = function(we) {
                        var Ee = (we || we.touches[0]).target, xe = this._wasTap && this.containerOuter.element.contains(Ee);
                        if (xe) {
                          var Le = Ee === this.containerOuter.element || Ee === this.containerInner.element;
                          Le && (this._isTextElement ? this.input.focus() : this._isSelectMultipleElement && this.showDropdown()), we.stopPropagation();
                        }
                        this._wasTap = !0;
                      }, Se.prototype._onMouseDown = function(we) {
                        var Ee = we.target;
                        if (Ee instanceof HTMLElement) {
                          if (ve && this.choiceList.element.contains(Ee)) {
                            var xe = this.choiceList.element.firstElementChild, Le = this._direction === "ltr" ? we.offsetX >= xe.offsetWidth : we.offsetX < xe.offsetLeft;
                            this._isScrollingOnIe = Le;
                          }
                          if (Ee !== this.input.element) {
                            var Qe = Ee.closest("[data-button],[data-item],[data-choice]");
                            if (Qe instanceof HTMLElement) {
                              var ke = we.shiftKey, Me = this._store.activeItems, Re = Qe.dataset;
                              "button" in Re ? this._handleButtonAction(Me, Qe) : "item" in Re ? this._handleItemAction(Me, Qe, ke) : "choice" in Re && this._handleChoiceAction(Me, Qe);
                            }
                            we.preventDefault();
                          }
                        }
                      }, Se.prototype._onMouseOver = function(we) {
                        var Ee = we.target;
                        Ee instanceof HTMLElement && "choice" in Ee.dataset && this._highlightChoice(Ee);
                      }, Se.prototype._onClick = function(we) {
                        var Ee = we.target, xe = this.containerOuter.element.contains(Ee);
                        if (xe)
                          !this.dropdown.isActive && !this.containerOuter.isDisabled ? this._isTextElement ? document.activeElement !== this.input.element && this.input.focus() : (this.showDropdown(), this.containerOuter.focus()) : this._isSelectOneElement && Ee !== this.input.element && !this.dropdown.element.contains(Ee) && this.hideDropdown();
                        else {
                          var Le = this._store.highlightedActiveItems.length > 0;
                          Le && this.unhighlightAll(), this.containerOuter.removeFocusState(), this.hideDropdown(!0);
                        }
                      }, Se.prototype._onFocus = function(we) {
                        var Ee, xe = this, Le = we.target, Qe = Le && this.containerOuter.element.contains(Le);
                        if (Qe) {
                          var ke = (Ee = {}, Ee[fe.TEXT_TYPE] = function() {
                            Le === xe.input.element && xe.containerOuter.addFocusState();
                          }, Ee[fe.SELECT_ONE_TYPE] = function() {
                            xe.containerOuter.addFocusState(), Le === xe.input.element && xe.showDropdown(!0);
                          }, Ee[fe.SELECT_MULTIPLE_TYPE] = function() {
                            Le === xe.input.element && (xe.showDropdown(!0), xe.containerOuter.addFocusState());
                          }, Ee);
                          ke[this.passedElement.element.type]();
                        }
                      }, Se.prototype._onBlur = function(we) {
                        var Ee, xe = this, Le = we.target, Qe = Le && this.containerOuter.element.contains(Le);
                        if (Qe && !this._isScrollingOnIe) {
                          var ke = this._store.activeItems, Me = ke.some(function(De) {
                            return De.highlighted;
                          }), Re = (Ee = {}, Ee[fe.TEXT_TYPE] = function() {
                            Le === xe.input.element && (xe.containerOuter.removeFocusState(), Me && xe.unhighlightAll(), xe.hideDropdown(!0));
                          }, Ee[fe.SELECT_ONE_TYPE] = function() {
                            xe.containerOuter.removeFocusState(), (Le === xe.input.element || Le === xe.containerOuter.element && !xe._canSearch) && xe.hideDropdown(!0);
                          }, Ee[fe.SELECT_MULTIPLE_TYPE] = function() {
                            Le === xe.input.element && (xe.containerOuter.removeFocusState(), xe.hideDropdown(!0), Me && xe.unhighlightAll());
                          }, Ee);
                          Re[this.passedElement.element.type]();
                        } else
                          this._isScrollingOnIe = !1, this.input.element.focus();
                      }, Se.prototype._onFormReset = function() {
                        this._store.dispatch((0, ue.resetTo)(this._initialState));
                      }, Se.prototype._highlightChoice = function(we) {
                        var Ee = this;
                        we === void 0 && (we = null);
                        var xe = Array.from(this.dropdown.element.querySelectorAll("[data-choice-selectable]"));
                        if (xe.length) {
                          var Le = we, Qe = Array.from(this.dropdown.element.querySelectorAll(".".concat(this.config.classNames.highlightedState)));
                          Qe.forEach(function(ke) {
                            ke.classList.remove(Ee.config.classNames.highlightedState), ke.setAttribute("aria-selected", "false");
                          }), Le ? this._highlightPosition = xe.indexOf(Le) : (xe.length > this._highlightPosition ? Le = xe[this._highlightPosition] : Le = xe[xe.length - 1], Le || (Le = xe[0])), Le.classList.add(this.config.classNames.highlightedState), Le.setAttribute("aria-selected", "true"), this.passedElement.triggerEvent(fe.EVENTS.highlightChoice, {
                            el: Le
                          }), this.dropdown.isActive && (this.input.setActiveDescendant(Le.id), this.containerOuter.setActiveDescendant(Le.id));
                        }
                      }, Se.prototype._addItem = function(we) {
                        var Ee = we.value, xe = we.label, Le = xe === void 0 ? null : xe, Qe = we.choiceId, ke = Qe === void 0 ? -1 : Qe, Me = we.groupId, Re = Me === void 0 ? -1 : Me, De = we.customProperties, Pe = De === void 0 ? {} : De, $e = we.placeholder, qe = $e === void 0 ? !1 : $e, Xe = we.keyCode, Ve = Xe === void 0 ? -1 : Xe, Ze = typeof Ee == "string" ? Ee.trim() : Ee, Ge = this._store.items, ze = Le || Ze, We = ke || -1, at = Re >= 0 ? this._store.getGroupById(Re) : null, ct = Ge ? Ge.length + 1 : 1;
                        this.config.prependValue && (Ze = this.config.prependValue + Ze.toString()), this.config.appendValue && (Ze += this.config.appendValue.toString()), this._store.dispatch((0, le.addItem)({
                          value: Ze,
                          label: ze,
                          id: ct,
                          choiceId: We,
                          groupId: Re,
                          customProperties: Pe,
                          placeholder: qe,
                          keyCode: Ve
                        })), this._isSelectOneElement && this.removeActiveItems(ct), this.passedElement.triggerEvent(fe.EVENTS.addItem, {
                          id: ct,
                          value: Ze,
                          label: ze,
                          customProperties: Pe,
                          groupValue: at && at.value ? at.value : null,
                          keyCode: Ve
                        });
                      }, Se.prototype._removeItem = function(we) {
                        var Ee = we.id, xe = we.value, Le = we.label, Qe = we.customProperties, ke = we.choiceId, Me = we.groupId, Re = Me && Me >= 0 ? this._store.getGroupById(Me) : null;
                        !Ee || !ke || (this._store.dispatch((0, le.removeItem)(Ee, ke)), this.passedElement.triggerEvent(fe.EVENTS.removeItem, {
                          id: Ee,
                          value: xe,
                          label: Le,
                          customProperties: Qe,
                          groupValue: Re && Re.value ? Re.value : null
                        }));
                      }, Se.prototype._addChoice = function(we) {
                        var Ee = we.value, xe = we.label, Le = xe === void 0 ? null : xe, Qe = we.isSelected, ke = Qe === void 0 ? !1 : Qe, Me = we.isDisabled, Re = Me === void 0 ? !1 : Me, De = we.groupId, Pe = De === void 0 ? -1 : De, $e = we.customProperties, qe = $e === void 0 ? {} : $e, Xe = we.placeholder, Ve = Xe === void 0 ? !1 : Xe, Ze = we.keyCode, Ge = Ze === void 0 ? -1 : Ze;
                        if (!(typeof Ee > "u" || Ee === null)) {
                          var ze = this._store.choices, We = Le || Ee, at = ze ? ze.length + 1 : 1, ct = "".concat(this._baseId, "-").concat(this._idNames.itemChoice, "-").concat(at);
                          this._store.dispatch((0, se.addChoice)({
                            id: at,
                            groupId: Pe,
                            elementId: ct,
                            value: Ee,
                            label: We,
                            disabled: Re,
                            customProperties: qe,
                            placeholder: Ve,
                            keyCode: Ge
                          })), ke && this._addItem({
                            value: Ee,
                            label: We,
                            choiceId: at,
                            customProperties: qe,
                            placeholder: Ve,
                            keyCode: Ge
                          });
                        }
                      }, Se.prototype._addGroup = function(we) {
                        var Ee = this, xe = we.group, Le = we.id, Qe = we.valueKey, ke = Qe === void 0 ? "value" : Qe, Me = we.labelKey, Re = Me === void 0 ? "label" : Me, De = (0, me.isType)("Object", xe) ? xe.choices : Array.from(xe.getElementsByTagName("OPTION")), Pe = Le || Math.floor((/* @__PURE__ */ new Date()).valueOf() * Math.random()), $e = xe.disabled ? xe.disabled : !1;
                        if (De) {
                          this._store.dispatch((0, ce.addGroup)({
                            value: xe.label,
                            id: Pe,
                            active: !0,
                            disabled: $e
                          }));
                          var qe = function(Xe) {
                            var Ve = Xe.disabled || Xe.parentNode && Xe.parentNode.disabled;
                            Ee._addChoice({
                              value: Xe[ke],
                              label: (0, me.isType)("Object", Xe) ? Xe[Re] : Xe.innerHTML,
                              isSelected: Xe.selected,
                              isDisabled: Ve,
                              groupId: Pe,
                              customProperties: Xe.customProperties,
                              placeholder: Xe.placeholder
                            });
                          };
                          De.forEach(qe);
                        } else
                          this._store.dispatch((0, ce.addGroup)({
                            value: xe.label,
                            id: xe.id,
                            active: !1,
                            disabled: xe.disabled
                          }));
                      }, Se.prototype._getTemplate = function(we) {
                        for (var Ee, xe = [], Le = 1; Le < arguments.length; Le++)
                          xe[Le - 1] = arguments[Le];
                        return (Ee = this._templates[we]).call.apply(Ee, ne([this, this.config], xe, !1));
                      }, Se.prototype._createTemplates = function() {
                        var we = this.config.callbackOnCreateTemplates, Ee = {};
                        we && typeof we == "function" && (Ee = we.call(this, me.strToEl)), this._templates = (0, ae.default)(ge.default, Ee);
                      }, Se.prototype._createElements = function() {
                        this.containerOuter = new de.Container({
                          element: this._getTemplate("containerOuter", this._direction, this._isSelectElement, this._isSelectOneElement, this.config.searchEnabled, this.passedElement.element.type, this.config.labelId),
                          classNames: this.config.classNames,
                          type: this.passedElement.element.type,
                          position: this.config.position
                        }), this.containerInner = new de.Container({
                          element: this._getTemplate("containerInner"),
                          classNames: this.config.classNames,
                          type: this.passedElement.element.type,
                          position: this.config.position
                        }), this.input = new de.Input({
                          element: this._getTemplate("input", this._placeholderValue),
                          classNames: this.config.classNames,
                          type: this.passedElement.element.type,
                          preventPaste: !this.config.paste
                        }), this.choiceList = new de.List({
                          element: this._getTemplate("choiceList", this._isSelectOneElement)
                        }), this.itemList = new de.List({
                          element: this._getTemplate("itemList", this._isSelectOneElement)
                        }), this.dropdown = new de.Dropdown({
                          element: this._getTemplate("dropdown"),
                          classNames: this.config.classNames,
                          type: this.passedElement.element.type
                        });
                      }, Se.prototype._createStructure = function() {
                        this.passedElement.conceal(), this.containerInner.wrap(this.passedElement.element), this.containerOuter.wrap(this.containerInner.element), this._isSelectOneElement ? this.input.placeholder = this.config.searchPlaceholderValue || "" : this._placeholderValue && (this.input.placeholder = this._placeholderValue, this.input.setWidth()), this.containerOuter.element.appendChild(this.containerInner.element), this.containerOuter.element.appendChild(this.dropdown.element), this.containerInner.element.appendChild(this.itemList.element), this._isTextElement || this.dropdown.element.appendChild(this.choiceList.element), this._isSelectOneElement ? this.config.searchEnabled && this.dropdown.element.insertBefore(this.input.element, this.dropdown.element.firstChild) : this.containerInner.element.appendChild(this.input.element), this._isSelectElement && (this._highlightPosition = 0, this._isSearching = !1, this._startLoading(), this._presetGroups.length ? this._addPredefinedGroups(this._presetGroups) : this._addPredefinedChoices(this._presetChoices), this._stopLoading()), this._isTextElement && this._addPredefinedItems(this._presetItems);
                      }, Se.prototype._addPredefinedGroups = function(we) {
                        var Ee = this, xe = this.passedElement.placeholderOption;
                        xe && xe.parentNode && xe.parentNode.tagName === "SELECT" && this._addChoice({
                          value: xe.value,
                          label: xe.innerHTML,
                          isSelected: xe.selected,
                          isDisabled: xe.disabled,
                          placeholder: !0
                        }), we.forEach(function(Le) {
                          return Ee._addGroup({
                            group: Le,
                            id: Le.id || null
                          });
                        });
                      }, Se.prototype._addPredefinedChoices = function(we) {
                        var Ee = this;
                        this.config.shouldSort && we.sort(this.config.sorter);
                        var xe = we.some(function(Qe) {
                          return Qe.selected;
                        }), Le = we.findIndex(function(Qe) {
                          return Qe.disabled === void 0 || !Qe.disabled;
                        });
                        we.forEach(function(Qe, ke) {
                          var Me = Qe.value, Re = Me === void 0 ? "" : Me, De = Qe.label, Pe = Qe.customProperties, $e = Qe.placeholder;
                          if (Ee._isSelectElement)
                            if (Qe.choices)
                              Ee._addGroup({
                                group: Qe,
                                id: Qe.id || null
                              });
                            else {
                              var qe = Ee._isSelectOneElement && !xe && ke === Le, Xe = qe ? !0 : Qe.selected, Ve = Qe.disabled;
                              Ee._addChoice({
                                value: Re,
                                label: De,
                                isSelected: !!Xe,
                                isDisabled: !!Ve,
                                placeholder: !!$e,
                                customProperties: Pe
                              });
                            }
                          else
                            Ee._addChoice({
                              value: Re,
                              label: De,
                              isSelected: !!Qe.selected,
                              isDisabled: !!Qe.disabled,
                              placeholder: !!Qe.placeholder,
                              customProperties: Pe
                            });
                        });
                      }, Se.prototype._addPredefinedItems = function(we) {
                        var Ee = this;
                        we.forEach(function(xe) {
                          typeof xe == "object" && xe.value && Ee._addItem({
                            value: xe.value,
                            label: xe.label,
                            choiceId: xe.id,
                            customProperties: xe.customProperties,
                            placeholder: xe.placeholder
                          }), typeof xe == "string" && Ee._addItem({
                            value: xe
                          });
                        });
                      }, Se.prototype._setChoiceOrItem = function(we) {
                        var Ee = this, xe = (0, me.getType)(we).toLowerCase(), Le = {
                          object: function() {
                            we.value && (Ee._isTextElement ? Ee._addItem({
                              value: we.value,
                              label: we.label,
                              choiceId: we.id,
                              customProperties: we.customProperties,
                              placeholder: we.placeholder
                            }) : Ee._addChoice({
                              value: we.value,
                              label: we.label,
                              isSelected: !0,
                              isDisabled: !1,
                              customProperties: we.customProperties,
                              placeholder: we.placeholder
                            }));
                          },
                          string: function() {
                            Ee._isTextElement ? Ee._addItem({
                              value: we
                            }) : Ee._addChoice({
                              value: we,
                              label: we,
                              isSelected: !0,
                              isDisabled: !1
                            });
                          }
                        };
                        Le[xe]();
                      }, Se.prototype._findAndSelectChoiceByValue = function(we) {
                        var Ee = this, xe = this._store.choices, Le = xe.find(function(Qe) {
                          return Ee.config.valueComparer(Qe.value, we);
                        });
                        Le && !Le.selected && this._addItem({
                          value: Le.value,
                          label: Le.label,
                          choiceId: Le.id,
                          groupId: Le.groupId,
                          customProperties: Le.customProperties,
                          placeholder: Le.placeholder,
                          keyCode: Le.keyCode
                        });
                      }, Se.prototype._generatePlaceholderValue = function() {
                        if (this._isSelectElement && this.passedElement.placeholderOption) {
                          var we = this.passedElement.placeholderOption;
                          return we ? we.text : null;
                        }
                        var Ee = this.config, xe = Ee.placeholder, Le = Ee.placeholderValue, Qe = this.passedElement.element.dataset;
                        if (xe) {
                          if (Le)
                            return Le;
                          if (Qe.placeholder)
                            return Qe.placeholder;
                        }
                        return null;
                      }, Se;
                    }()
                  );
                  Z.default = Ce;
                }
              ),
              /***/
              613: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ne = te(799), ee = te(883), ae = (
                    /** @class */
                    function() {
                      function oe(se) {
                        var ce = se.element, le = se.type, ue = se.classNames, de = se.position;
                        this.element = ce, this.classNames = ue, this.type = le, this.position = de, this.isOpen = !1, this.isFlipped = !1, this.isFocussed = !1, this.isDisabled = !1, this.isLoading = !1, this._onFocus = this._onFocus.bind(this), this._onBlur = this._onBlur.bind(this);
                      }
                      return oe.prototype.addEventListeners = function() {
                        this.element.addEventListener("focus", this._onFocus), this.element.addEventListener("blur", this._onBlur);
                      }, oe.prototype.removeEventListeners = function() {
                        this.element.removeEventListener("focus", this._onFocus), this.element.removeEventListener("blur", this._onBlur);
                      }, oe.prototype.shouldFlip = function(se) {
                        if (typeof se != "number")
                          return !1;
                        var ce = !1;
                        return this.position === "auto" ? ce = !window.matchMedia("(min-height: ".concat(se + 1, "px)")).matches : this.position === "top" && (ce = !0), ce;
                      }, oe.prototype.setActiveDescendant = function(se) {
                        this.element.setAttribute("aria-activedescendant", se);
                      }, oe.prototype.removeActiveDescendant = function() {
                        this.element.removeAttribute("aria-activedescendant");
                      }, oe.prototype.open = function(se) {
                        this.element.classList.add(this.classNames.openState), this.element.setAttribute("aria-expanded", "true"), this.isOpen = !0, this.shouldFlip(se) && (this.element.classList.add(this.classNames.flippedState), this.isFlipped = !0);
                      }, oe.prototype.close = function() {
                        this.element.classList.remove(this.classNames.openState), this.element.setAttribute("aria-expanded", "false"), this.removeActiveDescendant(), this.isOpen = !1, this.isFlipped && (this.element.classList.remove(this.classNames.flippedState), this.isFlipped = !1);
                      }, oe.prototype.focus = function() {
                        this.isFocussed || this.element.focus();
                      }, oe.prototype.addFocusState = function() {
                        this.element.classList.add(this.classNames.focusState);
                      }, oe.prototype.removeFocusState = function() {
                        this.element.classList.remove(this.classNames.focusState);
                      }, oe.prototype.enable = function() {
                        this.element.classList.remove(this.classNames.disabledState), this.element.removeAttribute("aria-disabled"), this.type === ee.SELECT_ONE_TYPE && this.element.setAttribute("tabindex", "0"), this.isDisabled = !1;
                      }, oe.prototype.disable = function() {
                        this.element.classList.add(this.classNames.disabledState), this.element.setAttribute("aria-disabled", "true"), this.type === ee.SELECT_ONE_TYPE && this.element.setAttribute("tabindex", "-1"), this.isDisabled = !0;
                      }, oe.prototype.wrap = function(se) {
                        (0, ne.wrap)(se, this.element);
                      }, oe.prototype.unwrap = function(se) {
                        this.element.parentNode && (this.element.parentNode.insertBefore(se, this.element), this.element.parentNode.removeChild(this.element));
                      }, oe.prototype.addLoadingState = function() {
                        this.element.classList.add(this.classNames.loadingState), this.element.setAttribute("aria-busy", "true"), this.isLoading = !0;
                      }, oe.prototype.removeLoadingState = function() {
                        this.element.classList.remove(this.classNames.loadingState), this.element.removeAttribute("aria-busy"), this.isLoading = !1;
                      }, oe.prototype._onFocus = function() {
                        this.isFocussed = !0;
                      }, oe.prototype._onBlur = function() {
                        this.isFocussed = !1;
                      }, oe;
                    }()
                  );
                  Z.default = ae;
                }
              ),
              /***/
              217: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var te = (
                    /** @class */
                    function() {
                      function ne(ee) {
                        var ae = ee.element, oe = ee.type, se = ee.classNames;
                        this.element = ae, this.classNames = se, this.type = oe, this.isActive = !1;
                      }
                      return Object.defineProperty(ne.prototype, "distanceFromTopWindow", {
                        /**
                         * Bottom position of dropdown in viewport coordinates
                         */
                        get: function() {
                          return this.element.getBoundingClientRect().bottom;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), ne.prototype.getChild = function(ee) {
                        return this.element.querySelector(ee);
                      }, ne.prototype.show = function() {
                        return this.element.classList.add(this.classNames.activeState), this.element.setAttribute("aria-expanded", "true"), this.isActive = !0, this;
                      }, ne.prototype.hide = function() {
                        return this.element.classList.remove(this.classNames.activeState), this.element.setAttribute("aria-expanded", "false"), this.isActive = !1, this;
                      }, ne;
                    }()
                  );
                  Z.default = te;
                }
              ),
              /***/
              520: (
                /***/
                function(J, Z, te) {
                  var ne = this && this.__importDefault || function(ue) {
                    return ue && ue.__esModule ? ue : {
                      default: ue
                    };
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.WrappedSelect = Z.WrappedInput = Z.List = Z.Input = Z.Container = Z.Dropdown = void 0;
                  var ee = ne(te(217));
                  Z.Dropdown = ee.default;
                  var ae = ne(te(613));
                  Z.Container = ae.default;
                  var oe = ne(te(11));
                  Z.Input = oe.default;
                  var se = ne(te(624));
                  Z.List = se.default;
                  var ce = ne(te(541));
                  Z.WrappedInput = ce.default;
                  var le = ne(te(982));
                  Z.WrappedSelect = le.default;
                }
              ),
              /***/
              11: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ne = te(799), ee = te(883), ae = (
                    /** @class */
                    function() {
                      function oe(se) {
                        var ce = se.element, le = se.type, ue = se.classNames, de = se.preventPaste;
                        this.element = ce, this.type = le, this.classNames = ue, this.preventPaste = de, this.isFocussed = this.element.isEqualNode(document.activeElement), this.isDisabled = ce.disabled, this._onPaste = this._onPaste.bind(this), this._onInput = this._onInput.bind(this), this._onFocus = this._onFocus.bind(this), this._onBlur = this._onBlur.bind(this);
                      }
                      return Object.defineProperty(oe.prototype, "placeholder", {
                        set: function(se) {
                          this.element.placeholder = se;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(oe.prototype, "value", {
                        get: function() {
                          return (0, ne.sanitise)(this.element.value);
                        },
                        set: function(se) {
                          this.element.value = se;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(oe.prototype, "rawValue", {
                        get: function() {
                          return this.element.value;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), oe.prototype.addEventListeners = function() {
                        this.element.addEventListener("paste", this._onPaste), this.element.addEventListener("input", this._onInput, {
                          passive: !0
                        }), this.element.addEventListener("focus", this._onFocus, {
                          passive: !0
                        }), this.element.addEventListener("blur", this._onBlur, {
                          passive: !0
                        });
                      }, oe.prototype.removeEventListeners = function() {
                        this.element.removeEventListener("input", this._onInput), this.element.removeEventListener("paste", this._onPaste), this.element.removeEventListener("focus", this._onFocus), this.element.removeEventListener("blur", this._onBlur);
                      }, oe.prototype.enable = function() {
                        this.element.removeAttribute("disabled"), this.isDisabled = !1;
                      }, oe.prototype.disable = function() {
                        this.element.setAttribute("disabled", ""), this.isDisabled = !0;
                      }, oe.prototype.focus = function() {
                        this.isFocussed || this.element.focus();
                      }, oe.prototype.blur = function() {
                        this.isFocussed && this.element.blur();
                      }, oe.prototype.clear = function(se) {
                        return se === void 0 && (se = !0), this.element.value && (this.element.value = ""), se && this.setWidth(), this;
                      }, oe.prototype.setWidth = function() {
                        var se = this.element, ce = se.style, le = se.value, ue = se.placeholder;
                        ce.minWidth = "".concat(ue.length + 1, "ch"), ce.width = "".concat(le.length + 1, "ch");
                      }, oe.prototype.setActiveDescendant = function(se) {
                        this.element.setAttribute("aria-activedescendant", se);
                      }, oe.prototype.removeActiveDescendant = function() {
                        this.element.removeAttribute("aria-activedescendant");
                      }, oe.prototype._onInput = function() {
                        this.type !== ee.SELECT_ONE_TYPE && this.setWidth();
                      }, oe.prototype._onPaste = function(se) {
                        this.preventPaste && se.preventDefault();
                      }, oe.prototype._onFocus = function() {
                        this.isFocussed = !0;
                      }, oe.prototype._onBlur = function() {
                        this.isFocussed = !1;
                      }, oe;
                    }()
                  );
                  Z.default = ae;
                }
              ),
              /***/
              624: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ne = te(883), ee = (
                    /** @class */
                    function() {
                      function ae(oe) {
                        var se = oe.element;
                        this.element = se, this.scrollPos = this.element.scrollTop, this.height = this.element.offsetHeight;
                      }
                      return ae.prototype.clear = function() {
                        this.element.innerHTML = "";
                      }, ae.prototype.append = function(oe) {
                        this.element.appendChild(oe);
                      }, ae.prototype.getChild = function(oe) {
                        return this.element.querySelector(oe);
                      }, ae.prototype.hasChildren = function() {
                        return this.element.hasChildNodes();
                      }, ae.prototype.scrollToTop = function() {
                        this.element.scrollTop = 0;
                      }, ae.prototype.scrollToChildElement = function(oe, se) {
                        var ce = this;
                        if (oe) {
                          var le = this.element.offsetHeight, ue = this.element.scrollTop + le, de = oe.offsetHeight, fe = oe.offsetTop + de, he = se > 0 ? this.element.scrollTop + fe - ue : oe.offsetTop;
                          requestAnimationFrame(function() {
                            ce._animateScroll(he, se);
                          });
                        }
                      }, ae.prototype._scrollDown = function(oe, se, ce) {
                        var le = (ce - oe) / se, ue = le > 1 ? le : 1;
                        this.element.scrollTop = oe + ue;
                      }, ae.prototype._scrollUp = function(oe, se, ce) {
                        var le = (oe - ce) / se, ue = le > 1 ? le : 1;
                        this.element.scrollTop = oe - ue;
                      }, ae.prototype._animateScroll = function(oe, se) {
                        var ce = this, le = ne.SCROLLING_SPEED, ue = this.element.scrollTop, de = !1;
                        se > 0 ? (this._scrollDown(ue, le, oe), ue < oe && (de = !0)) : (this._scrollUp(ue, le, oe), ue > oe && (de = !0)), de && requestAnimationFrame(function() {
                          ce._animateScroll(oe, se);
                        });
                      }, ae;
                    }()
                  );
                  Z.default = ee;
                }
              ),
              /***/
              730: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ne = te(799), ee = (
                    /** @class */
                    function() {
                      function ae(oe) {
                        var se = oe.element, ce = oe.classNames;
                        if (this.element = se, this.classNames = ce, !(se instanceof HTMLInputElement) && !(se instanceof HTMLSelectElement))
                          throw new TypeError("Invalid element passed");
                        this.isDisabled = !1;
                      }
                      return Object.defineProperty(ae.prototype, "isActive", {
                        get: function() {
                          return this.element.dataset.choice === "active";
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ae.prototype, "dir", {
                        get: function() {
                          return this.element.dir;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ae.prototype, "value", {
                        get: function() {
                          return this.element.value;
                        },
                        set: function(oe) {
                          this.element.value = oe;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), ae.prototype.conceal = function() {
                        this.element.classList.add(this.classNames.input), this.element.hidden = !0, this.element.tabIndex = -1;
                        var oe = this.element.getAttribute("style");
                        oe && this.element.setAttribute("data-choice-orig-style", oe), this.element.setAttribute("data-choice", "active");
                      }, ae.prototype.reveal = function() {
                        this.element.classList.remove(this.classNames.input), this.element.hidden = !1, this.element.removeAttribute("tabindex");
                        var oe = this.element.getAttribute("data-choice-orig-style");
                        oe ? (this.element.removeAttribute("data-choice-orig-style"), this.element.setAttribute("style", oe)) : this.element.removeAttribute("style"), this.element.removeAttribute("data-choice"), this.element.value = this.element.value;
                      }, ae.prototype.enable = function() {
                        this.element.removeAttribute("disabled"), this.element.disabled = !1, this.isDisabled = !1;
                      }, ae.prototype.disable = function() {
                        this.element.setAttribute("disabled", ""), this.element.disabled = !0, this.isDisabled = !0;
                      }, ae.prototype.triggerEvent = function(oe, se) {
                        (0, ne.dispatchEvent)(this.element, oe, se);
                      }, ae;
                    }()
                  );
                  Z.default = ee;
                }
              ),
              /***/
              541: (
                /***/
                function(J, Z, te) {
                  var ne = this && this.__extends || /* @__PURE__ */ function() {
                    var se = function(ce, le) {
                      return se = Object.setPrototypeOf || {
                        __proto__: []
                      } instanceof Array && function(ue, de) {
                        ue.__proto__ = de;
                      } || function(ue, de) {
                        for (var fe in de) Object.prototype.hasOwnProperty.call(de, fe) && (ue[fe] = de[fe]);
                      }, se(ce, le);
                    };
                    return function(ce, le) {
                      if (typeof le != "function" && le !== null) throw new TypeError("Class extends value " + String(le) + " is not a constructor or null");
                      se(ce, le);
                      function ue() {
                        this.constructor = ce;
                      }
                      ce.prototype = le === null ? Object.create(le) : (ue.prototype = le.prototype, new ue());
                    };
                  }(), ee = this && this.__importDefault || function(se) {
                    return se && se.__esModule ? se : {
                      default: se
                    };
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ae = ee(te(730)), oe = (
                    /** @class */
                    function(se) {
                      ne(ce, se);
                      function ce(le) {
                        var ue = le.element, de = le.classNames, fe = le.delimiter, he = se.call(this, {
                          element: ue,
                          classNames: de
                        }) || this;
                        return he.delimiter = fe, he;
                      }
                      return Object.defineProperty(ce.prototype, "value", {
                        get: function() {
                          return this.element.value;
                        },
                        set: function(le) {
                          this.element.setAttribute("value", le), this.element.value = le;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), ce;
                    }(ae.default)
                  );
                  Z.default = oe;
                }
              ),
              /***/
              982: (
                /***/
                function(J, Z, te) {
                  var ne = this && this.__extends || /* @__PURE__ */ function() {
                    var se = function(ce, le) {
                      return se = Object.setPrototypeOf || {
                        __proto__: []
                      } instanceof Array && function(ue, de) {
                        ue.__proto__ = de;
                      } || function(ue, de) {
                        for (var fe in de) Object.prototype.hasOwnProperty.call(de, fe) && (ue[fe] = de[fe]);
                      }, se(ce, le);
                    };
                    return function(ce, le) {
                      if (typeof le != "function" && le !== null) throw new TypeError("Class extends value " + String(le) + " is not a constructor or null");
                      se(ce, le);
                      function ue() {
                        this.constructor = ce;
                      }
                      ce.prototype = le === null ? Object.create(le) : (ue.prototype = le.prototype, new ue());
                    };
                  }(), ee = this && this.__importDefault || function(se) {
                    return se && se.__esModule ? se : {
                      default: se
                    };
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ae = ee(te(730)), oe = (
                    /** @class */
                    function(se) {
                      ne(ce, se);
                      function ce(le) {
                        var ue = le.element, de = le.classNames, fe = le.template, he = se.call(this, {
                          element: ue,
                          classNames: de
                        }) || this;
                        return he.template = fe, he;
                      }
                      return Object.defineProperty(ce.prototype, "placeholderOption", {
                        get: function() {
                          return this.element.querySelector('option[value=""]') || // Backward compatibility layer for the non-standard placeholder attribute supported in older versions.
                          this.element.querySelector("option[placeholder]");
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "optionGroups", {
                        get: function() {
                          return Array.from(this.element.getElementsByTagName("OPTGROUP"));
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "options", {
                        get: function() {
                          return Array.from(this.element.options);
                        },
                        set: function(le) {
                          var ue = this, de = document.createDocumentFragment(), fe = function(he) {
                            var me = ue.template(he);
                            de.appendChild(me);
                          };
                          le.forEach(function(he) {
                            return fe(he);
                          }), this.appendDocFragment(de);
                        },
                        enumerable: !1,
                        configurable: !0
                      }), ce.prototype.appendDocFragment = function(le) {
                        this.element.innerHTML = "", this.element.appendChild(le);
                      }, ce;
                    }(ae.default)
                  );
                  Z.default = oe;
                }
              ),
              /***/
              883: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.SCROLLING_SPEED = Z.SELECT_MULTIPLE_TYPE = Z.SELECT_ONE_TYPE = Z.TEXT_TYPE = Z.KEY_CODES = Z.ACTION_TYPES = Z.EVENTS = void 0, Z.EVENTS = {
                    showDropdown: "showDropdown",
                    hideDropdown: "hideDropdown",
                    change: "change",
                    choice: "choice",
                    search: "search",
                    addItem: "addItem",
                    removeItem: "removeItem",
                    highlightItem: "highlightItem",
                    highlightChoice: "highlightChoice",
                    unhighlightItem: "unhighlightItem"
                  }, Z.ACTION_TYPES = {
                    ADD_CHOICE: "ADD_CHOICE",
                    FILTER_CHOICES: "FILTER_CHOICES",
                    ACTIVATE_CHOICES: "ACTIVATE_CHOICES",
                    CLEAR_CHOICES: "CLEAR_CHOICES",
                    ADD_GROUP: "ADD_GROUP",
                    ADD_ITEM: "ADD_ITEM",
                    REMOVE_ITEM: "REMOVE_ITEM",
                    HIGHLIGHT_ITEM: "HIGHLIGHT_ITEM",
                    CLEAR_ALL: "CLEAR_ALL",
                    RESET_TO: "RESET_TO",
                    SET_IS_LOADING: "SET_IS_LOADING"
                  }, Z.KEY_CODES = {
                    BACK_KEY: 46,
                    DELETE_KEY: 8,
                    ENTER_KEY: 13,
                    A_KEY: 65,
                    ESC_KEY: 27,
                    UP_KEY: 38,
                    DOWN_KEY: 40,
                    PAGE_UP_KEY: 33,
                    PAGE_DOWN_KEY: 34
                  }, Z.TEXT_TYPE = "text", Z.SELECT_ONE_TYPE = "select-one", Z.SELECT_MULTIPLE_TYPE = "select-multiple", Z.SCROLLING_SPEED = 4;
                }
              ),
              /***/
              789: (
                /***/
                function(J, Z, te) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.DEFAULT_CONFIG = Z.DEFAULT_CLASSNAMES = void 0;
                  var ne = te(799);
                  Z.DEFAULT_CLASSNAMES = {
                    containerOuter: "choices",
                    containerInner: "choices__inner",
                    input: "choices__input",
                    inputCloned: "choices__input--cloned",
                    list: "choices__list",
                    listItems: "choices__list--multiple",
                    listSingle: "choices__list--single",
                    listDropdown: "choices__list--dropdown",
                    item: "choices__item",
                    itemSelectable: "choices__item--selectable",
                    itemDisabled: "choices__item--disabled",
                    itemChoice: "choices__item--choice",
                    placeholder: "choices__placeholder",
                    group: "choices__group",
                    groupHeading: "choices__heading",
                    button: "choices__button",
                    activeState: "is-active",
                    focusState: "is-focused",
                    openState: "is-open",
                    disabledState: "is-disabled",
                    highlightedState: "is-highlighted",
                    selectedState: "is-selected",
                    flippedState: "is-flipped",
                    loadingState: "is-loading",
                    noResults: "has-no-results",
                    noChoices: "has-no-choices"
                  }, Z.DEFAULT_CONFIG = {
                    items: [],
                    choices: [],
                    silent: !1,
                    renderChoiceLimit: -1,
                    maxItemCount: -1,
                    addItems: !0,
                    addItemFilter: null,
                    removeItems: !0,
                    removeItemButton: !1,
                    editItems: !1,
                    allowHTML: !0,
                    duplicateItemsAllowed: !0,
                    delimiter: ",",
                    paste: !0,
                    searchEnabled: !0,
                    searchChoices: !0,
                    searchFloor: 1,
                    searchResultLimit: 4,
                    searchFields: ["label", "value"],
                    position: "auto",
                    resetScrollPosition: !0,
                    shouldSort: !0,
                    shouldSortItems: !1,
                    sorter: ne.sortByAlpha,
                    placeholder: !0,
                    placeholderValue: null,
                    searchPlaceholderValue: null,
                    prependValue: null,
                    appendValue: null,
                    renderSelectedChoices: "auto",
                    loadingText: "Loading...",
                    noResultsText: "No results found",
                    noChoicesText: "No choices to choose from",
                    itemSelectText: "Press to select",
                    uniqueItemText: "Only unique values can be added",
                    customAddItemText: "Only values matching specific conditions can be added",
                    addItemText: function(ee) {
                      return 'Press Enter to add <b>"'.concat((0, ne.sanitise)(ee), '"</b>');
                    },
                    maxItemText: function(ee) {
                      return "Only ".concat(ee, " values can be added");
                    },
                    valueComparer: function(ee, ae) {
                      return ee === ae;
                    },
                    fuseOptions: {
                      includeScore: !0
                    },
                    labelId: "",
                    callbackOnInit: null,
                    callbackOnCreateTemplates: null,
                    classNames: Z.DEFAULT_CLASSNAMES
                  };
                }
              ),
              /***/
              18: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              978: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              948: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              359: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              285: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              533: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              187: (
                /***/
                function(J, Z, te) {
                  var ne = this && this.__createBinding || (Object.create ? function(ae, oe, se, ce) {
                    ce === void 0 && (ce = se);
                    var le = Object.getOwnPropertyDescriptor(oe, se);
                    (!le || ("get" in le ? !oe.__esModule : le.writable || le.configurable)) && (le = {
                      enumerable: !0,
                      get: function() {
                        return oe[se];
                      }
                    }), Object.defineProperty(ae, ce, le);
                  } : function(ae, oe, se, ce) {
                    ce === void 0 && (ce = se), ae[ce] = oe[se];
                  }), ee = this && this.__exportStar || function(ae, oe) {
                    for (var se in ae) se !== "default" && !Object.prototype.hasOwnProperty.call(oe, se) && ne(oe, ae, se);
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), ee(te(18), Z), ee(te(978), Z), ee(te(948), Z), ee(te(359), Z), ee(te(285), Z), ee(te(533), Z), ee(te(287), Z), ee(te(132), Z), ee(te(837), Z), ee(te(598), Z), ee(te(369), Z), ee(te(37), Z), ee(te(47), Z), ee(te(923), Z), ee(te(876), Z);
                }
              ),
              /***/
              287: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              132: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              837: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              598: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              37: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              369: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              47: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              923: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              876: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                }
              ),
              /***/
              799: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.parseCustomProperties = Z.diff = Z.cloneObject = Z.existsInArray = Z.dispatchEvent = Z.sortByScore = Z.sortByAlpha = Z.strToEl = Z.sanitise = Z.isScrolledIntoView = Z.getAdjacentEl = Z.wrap = Z.isType = Z.getType = Z.generateId = Z.generateChars = Z.getRandomNumber = void 0;
                  var te = function(ve, be) {
                    return Math.floor(Math.random() * (be - ve) + ve);
                  };
                  Z.getRandomNumber = te;
                  var ne = function(ve) {
                    return Array.from({
                      length: ve
                    }, function() {
                      return (0, Z.getRandomNumber)(0, 36).toString(36);
                    }).join("");
                  };
                  Z.generateChars = ne;
                  var ee = function(ve, be) {
                    var Ce = ve.id || ve.name && "".concat(ve.name, "-").concat((0, Z.generateChars)(2)) || (0, Z.generateChars)(4);
                    return Ce = Ce.replace(/(:|\.|\[|\]|,)/g, ""), Ce = "".concat(be, "-").concat(Ce), Ce;
                  };
                  Z.generateId = ee;
                  var ae = function(ve) {
                    return Object.prototype.toString.call(ve).slice(8, -1);
                  };
                  Z.getType = ae;
                  var oe = function(ve, be) {
                    return be != null && (0, Z.getType)(be) === ve;
                  };
                  Z.isType = oe;
                  var se = function(ve, be) {
                    return be === void 0 && (be = document.createElement("div")), ve.parentNode && (ve.nextSibling ? ve.parentNode.insertBefore(be, ve.nextSibling) : ve.parentNode.appendChild(be)), be.appendChild(ve);
                  };
                  Z.wrap = se;
                  var ce = function(ve, be, Ce) {
                    Ce === void 0 && (Ce = 1);
                    for (var Se = "".concat(Ce > 0 ? "next" : "previous", "ElementSibling"), we = ve[Se]; we; ) {
                      if (we.matches(be))
                        return we;
                      we = we[Se];
                    }
                    return we;
                  };
                  Z.getAdjacentEl = ce;
                  var le = function(ve, be, Ce) {
                    if (Ce === void 0 && (Ce = 1), !ve)
                      return !1;
                    var Se;
                    return Ce > 0 ? Se = be.scrollTop + be.offsetHeight >= ve.offsetTop + ve.offsetHeight : Se = ve.offsetTop >= be.scrollTop, Se;
                  };
                  Z.isScrolledIntoView = le;
                  var ue = function(ve) {
                    return typeof ve != "string" ? ve : ve.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/"/g, "&quot;");
                  };
                  Z.sanitise = ue, Z.strToEl = function() {
                    var ve = document.createElement("div");
                    return function(be) {
                      var Ce = be.trim();
                      ve.innerHTML = Ce;
                      for (var Se = ve.children[0]; ve.firstChild; )
                        ve.removeChild(ve.firstChild);
                      return Se;
                    };
                  }();
                  var de = function(ve, be) {
                    var Ce = ve.value, Se = ve.label, we = Se === void 0 ? Ce : Se, Ee = be.value, xe = be.label, Le = xe === void 0 ? Ee : xe;
                    return we.localeCompare(Le, [], {
                      sensitivity: "base",
                      ignorePunctuation: !0,
                      numeric: !0
                    });
                  };
                  Z.sortByAlpha = de;
                  var fe = function(ve, be) {
                    var Ce = ve.score, Se = Ce === void 0 ? 0 : Ce, we = be.score, Ee = we === void 0 ? 0 : we;
                    return Se - Ee;
                  };
                  Z.sortByScore = fe;
                  var he = function(ve, be, Ce) {
                    Ce === void 0 && (Ce = null);
                    var Se = new CustomEvent(be, {
                      detail: Ce,
                      bubbles: !0,
                      cancelable: !0
                    });
                    return ve.dispatchEvent(Se);
                  };
                  Z.dispatchEvent = he;
                  var me = function(ve, be, Ce) {
                    return Ce === void 0 && (Ce = "value"), ve.some(function(Se) {
                      return typeof be == "string" ? Se[Ce] === be.trim() : Se[Ce] === be;
                    });
                  };
                  Z.existsInArray = me;
                  var pe = function(ve) {
                    return JSON.parse(JSON.stringify(ve));
                  };
                  Z.cloneObject = pe;
                  var ye = function(ve, be) {
                    var Ce = Object.keys(ve).sort(), Se = Object.keys(be).sort();
                    return Ce.filter(function(we) {
                      return Se.indexOf(we) < 0;
                    });
                  };
                  Z.diff = ye;
                  var ge = function(ve) {
                    if (typeof ve < "u")
                      try {
                        return JSON.parse(ve);
                      } catch {
                        return ve;
                      }
                    return {};
                  };
                  Z.parseCustomProperties = ge;
                }
              ),
              /***/
              273: (
                /***/
                function(J, Z) {
                  var te = this && this.__spreadArray || function(ee, ae, oe) {
                    if (oe || arguments.length === 2) for (var se = 0, ce = ae.length, le; se < ce; se++)
                      (le || !(se in ae)) && (le || (le = Array.prototype.slice.call(ae, 0, se)), le[se] = ae[se]);
                    return ee.concat(le || Array.prototype.slice.call(ae));
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.defaultState = void 0, Z.defaultState = [];
                  function ne(ee, ae) {
                    switch (ee === void 0 && (ee = Z.defaultState), ae === void 0 && (ae = {}), ae.type) {
                      case "ADD_CHOICE": {
                        var oe = ae, se = {
                          id: oe.id,
                          elementId: oe.elementId,
                          groupId: oe.groupId,
                          value: oe.value,
                          label: oe.label || oe.value,
                          disabled: oe.disabled || !1,
                          selected: !1,
                          active: !0,
                          score: 9999,
                          customProperties: oe.customProperties,
                          placeholder: oe.placeholder || !1
                        };
                        return te(te([], ee, !0), [se], !1);
                      }
                      case "ADD_ITEM": {
                        var ce = ae;
                        return ce.choiceId > -1 ? ee.map(function(fe) {
                          var he = fe;
                          return he.id === parseInt("".concat(ce.choiceId), 10) && (he.selected = !0), he;
                        }) : ee;
                      }
                      case "REMOVE_ITEM": {
                        var le = ae;
                        return le.choiceId && le.choiceId > -1 ? ee.map(function(fe) {
                          var he = fe;
                          return he.id === parseInt("".concat(le.choiceId), 10) && (he.selected = !1), he;
                        }) : ee;
                      }
                      case "FILTER_CHOICES": {
                        var ue = ae;
                        return ee.map(function(fe) {
                          var he = fe;
                          return he.active = ue.results.some(function(me) {
                            var pe = me.item, ye = me.score;
                            return pe.id === he.id ? (he.score = ye, !0) : !1;
                          }), he;
                        });
                      }
                      case "ACTIVATE_CHOICES": {
                        var de = ae;
                        return ee.map(function(fe) {
                          var he = fe;
                          return he.active = de.active, he;
                        });
                      }
                      case "CLEAR_CHOICES":
                        return Z.defaultState;
                      default:
                        return ee;
                    }
                  }
                  Z.default = ne;
                }
              ),
              /***/
              871: (
                /***/
                function(J, Z) {
                  var te = this && this.__spreadArray || function(ee, ae, oe) {
                    if (oe || arguments.length === 2) for (var se = 0, ce = ae.length, le; se < ce; se++)
                      (le || !(se in ae)) && (le || (le = Array.prototype.slice.call(ae, 0, se)), le[se] = ae[se]);
                    return ee.concat(le || Array.prototype.slice.call(ae));
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.defaultState = void 0, Z.defaultState = [];
                  function ne(ee, ae) {
                    switch (ee === void 0 && (ee = Z.defaultState), ae === void 0 && (ae = {}), ae.type) {
                      case "ADD_GROUP": {
                        var oe = ae;
                        return te(te([], ee, !0), [{
                          id: oe.id,
                          value: oe.value,
                          active: oe.active,
                          disabled: oe.disabled
                        }], !1);
                      }
                      case "CLEAR_CHOICES":
                        return [];
                      default:
                        return ee;
                    }
                  }
                  Z.default = ne;
                }
              ),
              /***/
              655: (
                /***/
                function(J, Z, te) {
                  var ne = this && this.__importDefault || function(fe) {
                    return fe && fe.__esModule ? fe : {
                      default: fe
                    };
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.defaultState = void 0;
                  var ee = te(791), ae = ne(te(52)), oe = ne(te(871)), se = ne(te(273)), ce = ne(te(502)), le = te(799);
                  Z.defaultState = {
                    groups: [],
                    items: [],
                    choices: [],
                    loading: !1
                  };
                  var ue = (0, ee.combineReducers)({
                    items: ae.default,
                    groups: oe.default,
                    choices: se.default,
                    loading: ce.default
                  }), de = function(fe, he) {
                    var me = fe;
                    if (he.type === "CLEAR_ALL")
                      me = Z.defaultState;
                    else if (he.type === "RESET_TO")
                      return (0, le.cloneObject)(he.state);
                    return ue(me, he);
                  };
                  Z.default = de;
                }
              ),
              /***/
              52: (
                /***/
                function(J, Z) {
                  var te = this && this.__spreadArray || function(ee, ae, oe) {
                    if (oe || arguments.length === 2) for (var se = 0, ce = ae.length, le; se < ce; se++)
                      (le || !(se in ae)) && (le || (le = Array.prototype.slice.call(ae, 0, se)), le[se] = ae[se]);
                    return ee.concat(le || Array.prototype.slice.call(ae));
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.defaultState = void 0, Z.defaultState = [];
                  function ne(ee, ae) {
                    switch (ee === void 0 && (ee = Z.defaultState), ae === void 0 && (ae = {}), ae.type) {
                      case "ADD_ITEM": {
                        var oe = ae, se = te(te([], ee, !0), [{
                          id: oe.id,
                          choiceId: oe.choiceId,
                          groupId: oe.groupId,
                          value: oe.value,
                          label: oe.label,
                          active: !0,
                          highlighted: !1,
                          customProperties: oe.customProperties,
                          placeholder: oe.placeholder || !1,
                          keyCode: null
                        }], !1);
                        return se.map(function(le) {
                          var ue = le;
                          return ue.highlighted = !1, ue;
                        });
                      }
                      case "REMOVE_ITEM":
                        return ee.map(function(le) {
                          var ue = le;
                          return ue.id === ae.id && (ue.active = !1), ue;
                        });
                      case "HIGHLIGHT_ITEM": {
                        var ce = ae;
                        return ee.map(function(le) {
                          var ue = le;
                          return ue.id === ce.id && (ue.highlighted = ce.highlighted), ue;
                        });
                      }
                      default:
                        return ee;
                    }
                  }
                  Z.default = ne;
                }
              ),
              /***/
              502: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  }), Z.defaultState = void 0, Z.defaultState = !1;
                  var te = function(ne, ee) {
                    switch (ne === void 0 && (ne = Z.defaultState), ee === void 0 && (ee = {}), ee.type) {
                      case "SET_IS_LOADING":
                        return ee.isLoading;
                      default:
                        return ne;
                    }
                  };
                  Z.default = te;
                }
              ),
              /***/
              744: (
                /***/
                function(J, Z, te) {
                  var ne = this && this.__spreadArray || function(ce, le, ue) {
                    if (ue || arguments.length === 2) for (var de = 0, fe = le.length, he; de < fe; de++)
                      (he || !(de in le)) && (he || (he = Array.prototype.slice.call(le, 0, de)), he[de] = le[de]);
                    return ce.concat(he || Array.prototype.slice.call(le));
                  }, ee = this && this.__importDefault || function(ce) {
                    return ce && ce.__esModule ? ce : {
                      default: ce
                    };
                  };
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var ae = te(791), oe = ee(te(655)), se = (
                    /** @class */
                    function() {
                      function ce() {
                        this._store = (0, ae.createStore)(oe.default, window.__REDUX_DEVTOOLS_EXTENSION__ && window.__REDUX_DEVTOOLS_EXTENSION__());
                      }
                      return ce.prototype.subscribe = function(le) {
                        this._store.subscribe(le);
                      }, ce.prototype.dispatch = function(le) {
                        this._store.dispatch(le);
                      }, Object.defineProperty(ce.prototype, "state", {
                        /**
                         * Get store object (wrapping Redux method)
                         */
                        get: function() {
                          return this._store.getState();
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "items", {
                        /**
                         * Get items from store
                         */
                        get: function() {
                          return this.state.items;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "activeItems", {
                        /**
                         * Get active items from store
                         */
                        get: function() {
                          return this.items.filter(function(le) {
                            return le.active === !0;
                          });
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "highlightedActiveItems", {
                        /**
                         * Get highlighted items from store
                         */
                        get: function() {
                          return this.items.filter(function(le) {
                            return le.active && le.highlighted;
                          });
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "choices", {
                        /**
                         * Get choices from store
                         */
                        get: function() {
                          return this.state.choices;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "activeChoices", {
                        /**
                         * Get active choices from store
                         */
                        get: function() {
                          return this.choices.filter(function(le) {
                            return le.active === !0;
                          });
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "selectableChoices", {
                        /**
                         * Get selectable choices from store
                         */
                        get: function() {
                          return this.choices.filter(function(le) {
                            return le.disabled !== !0;
                          });
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "searchableChoices", {
                        /**
                         * Get choices that can be searched (excluding placeholders)
                         */
                        get: function() {
                          return this.selectableChoices.filter(function(le) {
                            return le.placeholder !== !0;
                          });
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "placeholderChoice", {
                        /**
                         * Get placeholder choice from store
                         */
                        get: function() {
                          return ne([], this.choices, !0).reverse().find(function(le) {
                            return le.placeholder === !0;
                          });
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "groups", {
                        /**
                         * Get groups from store
                         */
                        get: function() {
                          return this.state.groups;
                        },
                        enumerable: !1,
                        configurable: !0
                      }), Object.defineProperty(ce.prototype, "activeGroups", {
                        /**
                         * Get active groups from store
                         */
                        get: function() {
                          var le = this, ue = le.groups, de = le.choices;
                          return ue.filter(function(fe) {
                            var he = fe.active === !0 && fe.disabled === !1, me = de.some(function(pe) {
                              return pe.active === !0 && pe.disabled === !1;
                            });
                            return he && me;
                          }, []);
                        },
                        enumerable: !1,
                        configurable: !0
                      }), ce.prototype.isLoading = function() {
                        return this.state.loading;
                      }, ce.prototype.getChoiceById = function(le) {
                        return this.activeChoices.find(function(ue) {
                          return ue.id === parseInt(le, 10);
                        });
                      }, ce.prototype.getGroupById = function(le) {
                        return this.groups.find(function(ue) {
                          return ue.id === le;
                        });
                      }, ce;
                    }()
                  );
                  Z.default = se;
                }
              ),
              /***/
              686: (
                /***/
                function(J, Z) {
                  Object.defineProperty(Z, "__esModule", {
                    value: !0
                  });
                  var te = {
                    containerOuter: function(ne, ee, ae, oe, se, ce, le) {
                      var ue = ne.classNames.containerOuter, de = Object.assign(document.createElement("div"), {
                        className: ue
                      });
                      return de.dataset.type = ce, ee && (de.dir = ee), oe && (de.tabIndex = 0), ae && (de.setAttribute("role", se ? "combobox" : "listbox"), se && de.setAttribute("aria-autocomplete", "list")), de.setAttribute("aria-haspopup", "true"), de.setAttribute("aria-expanded", "false"), le && de.setAttribute("aria-labelledby", le), de;
                    },
                    containerInner: function(ne) {
                      var ee = ne.classNames.containerInner;
                      return Object.assign(document.createElement("div"), {
                        className: ee
                      });
                    },
                    itemList: function(ne, ee) {
                      var ae = ne.classNames, oe = ae.list, se = ae.listSingle, ce = ae.listItems;
                      return Object.assign(document.createElement("div"), {
                        className: "".concat(oe, " ").concat(ee ? se : ce)
                      });
                    },
                    placeholder: function(ne, ee) {
                      var ae, oe = ne.allowHTML, se = ne.classNames.placeholder;
                      return Object.assign(document.createElement("div"), (ae = {
                        className: se
                      }, ae[oe ? "innerHTML" : "innerText"] = ee, ae));
                    },
                    item: function(ne, ee, ae) {
                      var oe, se, ce = ne.allowHTML, le = ne.classNames, ue = le.item, de = le.button, fe = le.highlightedState, he = le.itemSelectable, me = le.placeholder, pe = ee.id, ye = ee.value, ge = ee.label, ve = ee.customProperties, be = ee.active, Ce = ee.disabled, Se = ee.highlighted, we = ee.placeholder, Ee = Object.assign(document.createElement("div"), (oe = {
                        className: ue
                      }, oe[ce ? "innerHTML" : "innerText"] = ge, oe));
                      if (Object.assign(Ee.dataset, {
                        item: "",
                        id: pe,
                        value: ye,
                        customProperties: ve
                      }), be && Ee.setAttribute("aria-selected", "true"), Ce && Ee.setAttribute("aria-disabled", "true"), we && Ee.classList.add(me), Ee.classList.add(Se ? fe : he), ae) {
                        Ce && Ee.classList.remove(he), Ee.dataset.deletable = "";
                        var xe = "Remove item", Le = Object.assign(document.createElement("button"), (se = {
                          type: "button",
                          className: de
                        }, se[ce ? "innerHTML" : "innerText"] = xe, se));
                        Le.setAttribute("aria-label", "".concat(xe, ": '").concat(ye, "'")), Le.dataset.button = "", Ee.appendChild(Le);
                      }
                      return Ee;
                    },
                    choiceList: function(ne, ee) {
                      var ae = ne.classNames.list, oe = Object.assign(document.createElement("div"), {
                        className: ae
                      });
                      return ee || oe.setAttribute("aria-multiselectable", "true"), oe.setAttribute("role", "listbox"), oe;
                    },
                    choiceGroup: function(ne, ee) {
                      var ae, oe = ne.allowHTML, se = ne.classNames, ce = se.group, le = se.groupHeading, ue = se.itemDisabled, de = ee.id, fe = ee.value, he = ee.disabled, me = Object.assign(document.createElement("div"), {
                        className: "".concat(ce, " ").concat(he ? ue : "")
                      });
                      return me.setAttribute("role", "group"), Object.assign(me.dataset, {
                        group: "",
                        id: de,
                        value: fe
                      }), he && me.setAttribute("aria-disabled", "true"), me.appendChild(Object.assign(document.createElement("div"), (ae = {
                        className: le
                      }, ae[oe ? "innerHTML" : "innerText"] = fe, ae))), me;
                    },
                    choice: function(ne, ee, ae) {
                      var oe, se = ne.allowHTML, ce = ne.classNames, le = ce.item, ue = ce.itemChoice, de = ce.itemSelectable, fe = ce.selectedState, he = ce.itemDisabled, me = ce.placeholder, pe = ee.id, ye = ee.value, ge = ee.label, ve = ee.groupId, be = ee.elementId, Ce = ee.disabled, Se = ee.selected, we = ee.placeholder, Ee = Object.assign(document.createElement("div"), (oe = {
                        id: be
                      }, oe[se ? "innerHTML" : "innerText"] = ge, oe.className = "".concat(le, " ").concat(ue), oe));
                      return Se && Ee.classList.add(fe), we && Ee.classList.add(me), Ee.setAttribute("role", ve && ve > 0 ? "treeitem" : "option"), Object.assign(Ee.dataset, {
                        choice: "",
                        id: pe,
                        value: ye,
                        selectText: ae
                      }), Ce ? (Ee.classList.add(he), Ee.dataset.choiceDisabled = "", Ee.setAttribute("aria-disabled", "true")) : (Ee.classList.add(de), Ee.dataset.choiceSelectable = ""), Ee;
                    },
                    input: function(ne, ee) {
                      var ae = ne.classNames, oe = ae.input, se = ae.inputCloned, ce = Object.assign(document.createElement("input"), {
                        type: "search",
                        name: "search_terms",
                        className: "".concat(oe, " ").concat(se),
                        autocomplete: "off",
                        autocapitalize: "off",
                        spellcheck: !1
                      });
                      return ce.setAttribute("role", "textbox"), ce.setAttribute("aria-autocomplete", "list"), ce.setAttribute("aria-label", ee), ce;
                    },
                    dropdown: function(ne) {
                      var ee = ne.classNames, ae = ee.list, oe = ee.listDropdown, se = document.createElement("div");
                      return se.classList.add(ae, oe), se.setAttribute("aria-expanded", "false"), se;
                    },
                    notice: function(ne, ee, ae) {
                      var oe, se = ne.allowHTML, ce = ne.classNames, le = ce.item, ue = ce.itemChoice, de = ce.noResults, fe = ce.noChoices;
                      ae === void 0 && (ae = "");
                      var he = [le, ue];
                      return ae === "no-choices" ? he.push(fe) : ae === "no-results" && he.push(de), Object.assign(document.createElement("div"), (oe = {}, oe[se ? "innerHTML" : "innerText"] = ee, oe.className = he.join(" "), oe));
                    },
                    option: function(ne) {
                      var ee = ne.label, ae = ne.value, oe = ne.customProperties, se = ne.active, ce = ne.disabled, le = new Option(ee, ae, !1, se);
                      return oe && (le.dataset.customProperties = "".concat(oe)), le.disabled = !!ce, le;
                    }
                  };
                  Z.default = te;
                }
              ),
              /***/
              996: (
                /***/
                function(J) {
                  var Z = function(be) {
                    return te(be) && !ne(be);
                  };
                  function te(ve) {
                    return !!ve && typeof ve == "object";
                  }
                  function ne(ve) {
                    var be = Object.prototype.toString.call(ve);
                    return be === "[object RegExp]" || be === "[object Date]" || oe(ve);
                  }
                  var ee = typeof Symbol == "function" && Symbol.for, ae = ee ? Symbol.for("react.element") : 60103;
                  function oe(ve) {
                    return ve.$$typeof === ae;
                  }
                  function se(ve) {
                    return Array.isArray(ve) ? [] : {};
                  }
                  function ce(ve, be) {
                    return be.clone !== !1 && be.isMergeableObject(ve) ? ye(se(ve), ve, be) : ve;
                  }
                  function le(ve, be, Ce) {
                    return ve.concat(be).map(function(Se) {
                      return ce(Se, Ce);
                    });
                  }
                  function ue(ve, be) {
                    if (!be.customMerge)
                      return ye;
                    var Ce = be.customMerge(ve);
                    return typeof Ce == "function" ? Ce : ye;
                  }
                  function de(ve) {
                    return Object.getOwnPropertySymbols ? Object.getOwnPropertySymbols(ve).filter(function(be) {
                      return ve.propertyIsEnumerable(be);
                    }) : [];
                  }
                  function fe(ve) {
                    return Object.keys(ve).concat(de(ve));
                  }
                  function he(ve, be) {
                    try {
                      return be in ve;
                    } catch {
                      return !1;
                    }
                  }
                  function me(ve, be) {
                    return he(ve, be) && !(Object.hasOwnProperty.call(ve, be) && Object.propertyIsEnumerable.call(ve, be));
                  }
                  function pe(ve, be, Ce) {
                    var Se = {};
                    return Ce.isMergeableObject(ve) && fe(ve).forEach(function(we) {
                      Se[we] = ce(ve[we], Ce);
                    }), fe(be).forEach(function(we) {
                      me(ve, we) || (he(ve, we) && Ce.isMergeableObject(be[we]) ? Se[we] = ue(we, Ce)(ve[we], be[we], Ce) : Se[we] = ce(be[we], Ce));
                    }), Se;
                  }
                  function ye(ve, be, Ce) {
                    Ce = Ce || {}, Ce.arrayMerge = Ce.arrayMerge || le, Ce.isMergeableObject = Ce.isMergeableObject || Z, Ce.cloneUnlessOtherwiseSpecified = ce;
                    var Se = Array.isArray(be), we = Array.isArray(ve), Ee = Se === we;
                    return Ee ? Se ? Ce.arrayMerge(ve, be, Ce) : pe(ve, be, Ce) : ce(be, Ce);
                  }
                  ye.all = function(be, Ce) {
                    if (!Array.isArray(be))
                      throw new Error("first argument should be an array");
                    return be.reduce(function(Se, we) {
                      return ye(Se, we, Ce);
                    }, {});
                  };
                  var ge = ye;
                  J.exports = ge;
                }
              ),
              /***/
              221: (
                /***/
                function(J, Z, te) {
                  te.r(Z), te.d(Z, {
                    /* harmony export */
                    default: function() {
                      return (
                        /* binding */
                        vn
                      );
                    }
                    /* harmony export */
                  });
                  function ne(dn) {
                    return Array.isArray ? Array.isArray(dn) : he(dn) === "[object Array]";
                  }
                  function ee(dn) {
                    if (typeof dn == "string")
                      return dn;
                    let Ht = dn + "";
                    return Ht == "0" && 1 / dn == -1 / 0 ? "-0" : Ht;
                  }
                  function ae(dn) {
                    return dn == null ? "" : ee(dn);
                  }
                  function oe(dn) {
                    return typeof dn == "string";
                  }
                  function se(dn) {
                    return typeof dn == "number";
                  }
                  function ce(dn) {
                    return dn === !0 || dn === !1 || ue(dn) && he(dn) == "[object Boolean]";
                  }
                  function le(dn) {
                    return typeof dn == "object";
                  }
                  function ue(dn) {
                    return le(dn) && dn !== null;
                  }
                  function de(dn) {
                    return dn != null;
                  }
                  function fe(dn) {
                    return !dn.trim().length;
                  }
                  function he(dn) {
                    return dn == null ? dn === void 0 ? "[object Undefined]" : "[object Null]" : Object.prototype.toString.call(dn);
                  }
                  const me = "Incorrect 'index' type", pe = (dn) => `Invalid value for key ${dn}`, ye = (dn) => `Pattern length exceeds max of ${dn}.`, ge = (dn) => `Missing ${dn} property in key`, ve = (dn) => `Property 'weight' in key '${dn}' must be a positive integer`, be = Object.prototype.hasOwnProperty;
                  class Ce {
                    constructor(Ht) {
                      this._keys = [], this._keyMap = {};
                      let jt = 0;
                      Ht.forEach((cn) => {
                        let wn = Se(cn);
                        jt += wn.weight, this._keys.push(wn), this._keyMap[wn.id] = wn, jt += wn.weight;
                      }), this._keys.forEach((cn) => {
                        cn.weight /= jt;
                      });
                    }
                    get(Ht) {
                      return this._keyMap[Ht];
                    }
                    keys() {
                      return this._keys;
                    }
                    toJSON() {
                      return JSON.stringify(this._keys);
                    }
                  }
                  function Se(dn) {
                    let Ht = null, jt = null, cn = null, wn = 1, Rn = null;
                    if (oe(dn) || ne(dn))
                      cn = dn, Ht = we(dn), jt = Ee(dn);
                    else {
                      if (!be.call(dn, "name"))
                        throw new Error(ge("name"));
                      const Pn = dn.name;
                      if (cn = Pn, be.call(dn, "weight") && (wn = dn.weight, wn <= 0))
                        throw new Error(ve(Pn));
                      Ht = we(Pn), jt = Ee(Pn), Rn = dn.getFn;
                    }
                    return { path: Ht, id: jt, weight: wn, src: cn, getFn: Rn };
                  }
                  function we(dn) {
                    return ne(dn) ? dn : dn.split(".");
                  }
                  function Ee(dn) {
                    return ne(dn) ? dn.join(".") : dn;
                  }
                  function xe(dn, Ht) {
                    let jt = [], cn = !1;
                    const wn = (Rn, Pn, Ie) => {
                      if (de(Rn))
                        if (!Pn[Ie])
                          jt.push(Rn);
                        else {
                          let rt = Pn[Ie];
                          const ft = Rn[rt];
                          if (!de(ft))
                            return;
                          if (Ie === Pn.length - 1 && (oe(ft) || se(ft) || ce(ft)))
                            jt.push(ae(ft));
                          else if (ne(ft)) {
                            cn = !0;
                            for (let pt = 0, wt = ft.length; pt < wt; pt += 1)
                              wn(ft[pt], Pn, Ie + 1);
                          } else Pn.length && wn(ft, Pn, Ie + 1);
                        }
                    };
                    return wn(dn, oe(Ht) ? Ht.split(".") : Ht, 0), cn ? jt : jt[0];
                  }
                  var Re = {
                    ...{
                      // When `true`, the algorithm continues searching to the end of the input even if a perfect
                      // match is found before the end of the same input.
                      isCaseSensitive: !1,
                      // When true, the matching function will continue to the end of a search pattern even if
                      includeScore: !1,
                      // List of properties that will be searched. This also supports nested properties.
                      keys: [],
                      // Whether to sort the result list, by score
                      shouldSort: !0,
                      // Default sort function: sort by ascending score, ascending index
                      sortFn: (dn, Ht) => dn.score === Ht.score ? dn.idx < Ht.idx ? -1 : 1 : dn.score < Ht.score ? -1 : 1
                    },
                    ...{
                      // Whether the matches should be included in the result set. When `true`, each record in the result
                      // set will include the indices of the matched characters.
                      // These can consequently be used for highlighting purposes.
                      includeMatches: !1,
                      // When `true`, the matching function will continue to the end of a search pattern even if
                      // a perfect match has already been located in the string.
                      findAllMatches: !1,
                      // Minimum number of characters that must be matched before a result is considered a match
                      minMatchCharLength: 1
                    },
                    ...{
                      // Approximately where in the text is the pattern expected to be found?
                      location: 0,
                      // At what point does the match algorithm give up. A threshold of '0.0' requires a perfect match
                      // (of both letters and location), a threshold of '1.0' would match anything.
                      threshold: 0.6,
                      // Determines how close the match must be to the fuzzy location (specified above).
                      // An exact letter match which is 'distance' characters away from the fuzzy location
                      // would score as a complete mismatch. A distance of '0' requires the match be at
                      // the exact location specified, a threshold of '1000' would require a perfect match
                      // to be within 800 characters of the fuzzy location to be found using a 0.8 threshold.
                      distance: 100
                    },
                    ...{
                      // When `true`, it enables the use of unix-like search commands
                      useExtendedSearch: !1,
                      // The get function to use when fetching an object's properties.
                      // The default will search nested paths *ie foo.bar.baz*
                      getFn: xe,
                      // When `true`, search will ignore `location` and `distance`, so it won't matter
                      // where in the string the pattern appears.
                      // More info: https://fusejs.io/concepts/scoring-theory.html#fuzziness-score
                      ignoreLocation: !1,
                      // When `true`, the calculation for the relevance score (used for sorting) will
                      // ignore the field-length norm.
                      // More info: https://fusejs.io/concepts/scoring-theory.html#field-length-norm
                      ignoreFieldNorm: !1,
                      // The weight to determine how much field length norm effects scoring.
                      fieldNormWeight: 1
                    }
                  };
                  const De = /[^ ]+/g;
                  function Pe(dn = 1, Ht = 3) {
                    const jt = /* @__PURE__ */ new Map(), cn = Math.pow(10, Ht);
                    return {
                      get(wn) {
                        const Rn = wn.match(De).length;
                        if (jt.has(Rn))
                          return jt.get(Rn);
                        const Pn = 1 / Math.pow(Rn, 0.5 * dn), Ie = parseFloat(Math.round(Pn * cn) / cn);
                        return jt.set(Rn, Ie), Ie;
                      },
                      clear() {
                        jt.clear();
                      }
                    };
                  }
                  class $e {
                    constructor({
                      getFn: Ht = Re.getFn,
                      fieldNormWeight: jt = Re.fieldNormWeight
                    } = {}) {
                      this.norm = Pe(jt, 3), this.getFn = Ht, this.isCreated = !1, this.setIndexRecords();
                    }
                    setSources(Ht = []) {
                      this.docs = Ht;
                    }
                    setIndexRecords(Ht = []) {
                      this.records = Ht;
                    }
                    setKeys(Ht = []) {
                      this.keys = Ht, this._keysMap = {}, Ht.forEach((jt, cn) => {
                        this._keysMap[jt.id] = cn;
                      });
                    }
                    create() {
                      this.isCreated || !this.docs.length || (this.isCreated = !0, oe(this.docs[0]) ? this.docs.forEach((Ht, jt) => {
                        this._addString(Ht, jt);
                      }) : this.docs.forEach((Ht, jt) => {
                        this._addObject(Ht, jt);
                      }), this.norm.clear());
                    }
                    // Adds a doc to the end of the index
                    add(Ht) {
                      const jt = this.size();
                      oe(Ht) ? this._addString(Ht, jt) : this._addObject(Ht, jt);
                    }
                    // Removes the doc at the specified index of the index
                    removeAt(Ht) {
                      this.records.splice(Ht, 1);
                      for (let jt = Ht, cn = this.size(); jt < cn; jt += 1)
                        this.records[jt].i -= 1;
                    }
                    getValueForItemAtKeyId(Ht, jt) {
                      return Ht[this._keysMap[jt]];
                    }
                    size() {
                      return this.records.length;
                    }
                    _addString(Ht, jt) {
                      if (!de(Ht) || fe(Ht))
                        return;
                      let cn = {
                        v: Ht,
                        i: jt,
                        n: this.norm.get(Ht)
                      };
                      this.records.push(cn);
                    }
                    _addObject(Ht, jt) {
                      let cn = { i: jt, $: {} };
                      this.keys.forEach((wn, Rn) => {
                        let Pn = wn.getFn ? wn.getFn(Ht) : this.getFn(Ht, wn.path);
                        if (de(Pn)) {
                          if (ne(Pn)) {
                            let Ie = [];
                            const rt = [{ nestedArrIndex: -1, value: Pn }];
                            for (; rt.length; ) {
                              const { nestedArrIndex: ft, value: pt } = rt.pop();
                              if (de(pt))
                                if (oe(pt) && !fe(pt)) {
                                  let wt = {
                                    v: pt,
                                    i: ft,
                                    n: this.norm.get(pt)
                                  };
                                  Ie.push(wt);
                                } else ne(pt) && pt.forEach((wt, Bt) => {
                                  rt.push({
                                    nestedArrIndex: Bt,
                                    value: wt
                                  });
                                });
                            }
                            cn.$[Rn] = Ie;
                          } else if (oe(Pn) && !fe(Pn)) {
                            let Ie = {
                              v: Pn,
                              n: this.norm.get(Pn)
                            };
                            cn.$[Rn] = Ie;
                          }
                        }
                      }), this.records.push(cn);
                    }
                    toJSON() {
                      return {
                        keys: this.keys,
                        records: this.records
                      };
                    }
                  }
                  function qe(dn, Ht, { getFn: jt = Re.getFn, fieldNormWeight: cn = Re.fieldNormWeight } = {}) {
                    const wn = new $e({ getFn: jt, fieldNormWeight: cn });
                    return wn.setKeys(dn.map(Se)), wn.setSources(Ht), wn.create(), wn;
                  }
                  function Xe(dn, { getFn: Ht = Re.getFn, fieldNormWeight: jt = Re.fieldNormWeight } = {}) {
                    const { keys: cn, records: wn } = dn, Rn = new $e({ getFn: Ht, fieldNormWeight: jt });
                    return Rn.setKeys(cn), Rn.setIndexRecords(wn), Rn;
                  }
                  function Ve(dn, {
                    errors: Ht = 0,
                    currentLocation: jt = 0,
                    expectedLocation: cn = 0,
                    distance: wn = Re.distance,
                    ignoreLocation: Rn = Re.ignoreLocation
                  } = {}) {
                    const Pn = Ht / dn.length;
                    if (Rn)
                      return Pn;
                    const Ie = Math.abs(cn - jt);
                    return wn ? Pn + Ie / wn : Ie ? 1 : Pn;
                  }
                  function Ze(dn = [], Ht = Re.minMatchCharLength) {
                    let jt = [], cn = -1, wn = -1, Rn = 0;
                    for (let Pn = dn.length; Rn < Pn; Rn += 1) {
                      let Ie = dn[Rn];
                      Ie && cn === -1 ? cn = Rn : !Ie && cn !== -1 && (wn = Rn - 1, wn - cn + 1 >= Ht && jt.push([cn, wn]), cn = -1);
                    }
                    return dn[Rn - 1] && Rn - cn >= Ht && jt.push([cn, Rn - 1]), jt;
                  }
                  const Ge = 32;
                  function ze(dn, Ht, jt, {
                    location: cn = Re.location,
                    distance: wn = Re.distance,
                    threshold: Rn = Re.threshold,
                    findAllMatches: Pn = Re.findAllMatches,
                    minMatchCharLength: Ie = Re.minMatchCharLength,
                    includeMatches: rt = Re.includeMatches,
                    ignoreLocation: ft = Re.ignoreLocation
                  } = {}) {
                    if (Ht.length > Ge)
                      throw new Error(ye(Ge));
                    const pt = Ht.length, wt = dn.length, Bt = Math.max(0, Math.min(cn, wt));
                    let At = Rn, Dt = Bt;
                    const Ot = Ie > 1 || rt, un = Ot ? Array(wt) : [];
                    let yn;
                    for (; (yn = dn.indexOf(Ht, Dt)) > -1; ) {
                      let Mn = Ve(Ht, {
                        currentLocation: yn,
                        expectedLocation: Bt,
                        distance: wn,
                        ignoreLocation: ft
                      });
                      if (At = Math.min(Mn, At), Dt = yn + pt, Ot) {
                        let Ln = 0;
                        for (; Ln < pt; )
                          un[yn + Ln] = 1, Ln += 1;
                      }
                    }
                    Dt = -1;
                    let _n = [], gn = 1, bn = pt + wt;
                    const an = 1 << pt - 1;
                    for (let Mn = 0; Mn < pt; Mn += 1) {
                      let Ln = 0, In = bn;
                      for (; Ln < In; )
                        Ve(Ht, {
                          errors: Mn,
                          currentLocation: Bt + In,
                          expectedLocation: Bt,
                          distance: wn,
                          ignoreLocation: ft
                        }) <= At ? Ln = In : bn = In, In = Math.floor((bn - Ln) / 2 + Ln);
                      bn = In;
                      let kn = Math.max(1, Bt - In + 1), er = Pn ? wt : Math.min(Bt + In, wt) + pt, jn = Array(er + 2);
                      jn[er + 1] = (1 << Mn) - 1;
                      for (let Kn = er; Kn >= kn; Kn -= 1) {
                        let Jn = Kn - 1, Lr = jt[dn.charAt(Jn)];
                        if (Ot && (un[Jn] = +!!Lr), jn[Kn] = (jn[Kn + 1] << 1 | 1) & Lr, Mn && (jn[Kn] |= (_n[Kn + 1] | _n[Kn]) << 1 | 1 | _n[Kn + 1]), jn[Kn] & an && (gn = Ve(Ht, {
                          errors: Mn,
                          currentLocation: Jn,
                          expectedLocation: Bt,
                          distance: wn,
                          ignoreLocation: ft
                        }), gn <= At)) {
                          if (At = gn, Dt = Jn, Dt <= Bt)
                            break;
                          kn = Math.max(1, 2 * Bt - Dt);
                        }
                      }
                      if (Ve(Ht, {
                        errors: Mn + 1,
                        currentLocation: Bt,
                        expectedLocation: Bt,
                        distance: wn,
                        ignoreLocation: ft
                      }) > At)
                        break;
                      _n = jn;
                    }
                    const Zt = {
                      isMatch: Dt >= 0,
                      // Count exact matches (those with a score of 0) to be "almost" exact
                      score: Math.max(1e-3, gn)
                    };
                    if (Ot) {
                      const Mn = Ze(un, Ie);
                      Mn.length ? rt && (Zt.indices = Mn) : Zt.isMatch = !1;
                    }
                    return Zt;
                  }
                  function We(dn) {
                    let Ht = {};
                    for (let jt = 0, cn = dn.length; jt < cn; jt += 1) {
                      const wn = dn.charAt(jt);
                      Ht[wn] = (Ht[wn] || 0) | 1 << cn - jt - 1;
                    }
                    return Ht;
                  }
                  class at {
                    constructor(Ht, {
                      location: jt = Re.location,
                      threshold: cn = Re.threshold,
                      distance: wn = Re.distance,
                      includeMatches: Rn = Re.includeMatches,
                      findAllMatches: Pn = Re.findAllMatches,
                      minMatchCharLength: Ie = Re.minMatchCharLength,
                      isCaseSensitive: rt = Re.isCaseSensitive,
                      ignoreLocation: ft = Re.ignoreLocation
                    } = {}) {
                      if (this.options = {
                        location: jt,
                        threshold: cn,
                        distance: wn,
                        includeMatches: Rn,
                        findAllMatches: Pn,
                        minMatchCharLength: Ie,
                        isCaseSensitive: rt,
                        ignoreLocation: ft
                      }, this.pattern = rt ? Ht : Ht.toLowerCase(), this.chunks = [], !this.pattern.length)
                        return;
                      const pt = (Bt, At) => {
                        this.chunks.push({
                          pattern: Bt,
                          alphabet: We(Bt),
                          startIndex: At
                        });
                      }, wt = this.pattern.length;
                      if (wt > Ge) {
                        let Bt = 0;
                        const At = wt % Ge, Dt = wt - At;
                        for (; Bt < Dt; )
                          pt(this.pattern.substr(Bt, Ge), Bt), Bt += Ge;
                        if (At) {
                          const Ot = wt - Ge;
                          pt(this.pattern.substr(Ot), Ot);
                        }
                      } else
                        pt(this.pattern, 0);
                    }
                    searchIn(Ht) {
                      const { isCaseSensitive: jt, includeMatches: cn } = this.options;
                      if (jt || (Ht = Ht.toLowerCase()), this.pattern === Ht) {
                        let Dt = {
                          isMatch: !0,
                          score: 0
                        };
                        return cn && (Dt.indices = [[0, Ht.length - 1]]), Dt;
                      }
                      const {
                        location: wn,
                        distance: Rn,
                        threshold: Pn,
                        findAllMatches: Ie,
                        minMatchCharLength: rt,
                        ignoreLocation: ft
                      } = this.options;
                      let pt = [], wt = 0, Bt = !1;
                      this.chunks.forEach(({ pattern: Dt, alphabet: Ot, startIndex: un }) => {
                        const { isMatch: yn, score: _n, indices: gn } = ze(Ht, Dt, Ot, {
                          location: wn + un,
                          distance: Rn,
                          threshold: Pn,
                          findAllMatches: Ie,
                          minMatchCharLength: rt,
                          includeMatches: cn,
                          ignoreLocation: ft
                        });
                        yn && (Bt = !0), wt += _n, yn && gn && (pt = [...pt, ...gn]);
                      });
                      let At = {
                        isMatch: Bt,
                        score: Bt ? wt / this.chunks.length : 1
                      };
                      return Bt && cn && (At.indices = pt), At;
                    }
                  }
                  class ct {
                    constructor(Ht) {
                      this.pattern = Ht;
                    }
                    static isMultiMatch(Ht) {
                      return et(Ht, this.multiRegex);
                    }
                    static isSingleMatch(Ht) {
                      return et(Ht, this.singleRegex);
                    }
                    search() {
                    }
                  }
                  function et(dn, Ht) {
                    const jt = dn.match(Ht);
                    return jt ? jt[1] : null;
                  }
                  class dt extends ct {
                    constructor(Ht) {
                      super(Ht);
                    }
                    static get type() {
                      return "exact";
                    }
                    static get multiRegex() {
                      return /^="(.*)"$/;
                    }
                    static get singleRegex() {
                      return /^=(.*)$/;
                    }
                    search(Ht) {
                      const jt = Ht === this.pattern;
                      return {
                        isMatch: jt,
                        score: jt ? 0 : 1,
                        indices: [0, this.pattern.length - 1]
                      };
                    }
                  }
                  class Je extends ct {
                    constructor(Ht) {
                      super(Ht);
                    }
                    static get type() {
                      return "inverse-exact";
                    }
                    static get multiRegex() {
                      return /^!"(.*)"$/;
                    }
                    static get singleRegex() {
                      return /^!(.*)$/;
                    }
                    search(Ht) {
                      const cn = Ht.indexOf(this.pattern) === -1;
                      return {
                        isMatch: cn,
                        score: cn ? 0 : 1,
                        indices: [0, Ht.length - 1]
                      };
                    }
                  }
                  class it extends ct {
                    constructor(Ht) {
                      super(Ht);
                    }
                    static get type() {
                      return "prefix-exact";
                    }
                    static get multiRegex() {
                      return /^\^"(.*)"$/;
                    }
                    static get singleRegex() {
                      return /^\^(.*)$/;
                    }
                    search(Ht) {
                      const jt = Ht.startsWith(this.pattern);
                      return {
                        isMatch: jt,
                        score: jt ? 0 : 1,
                        indices: [0, this.pattern.length - 1]
                      };
                    }
                  }
                  class st extends ct {
                    constructor(Ht) {
                      super(Ht);
                    }
                    static get type() {
                      return "inverse-prefix-exact";
                    }
                    static get multiRegex() {
                      return /^!\^"(.*)"$/;
                    }
                    static get singleRegex() {
                      return /^!\^(.*)$/;
                    }
                    search(Ht) {
                      const jt = !Ht.startsWith(this.pattern);
                      return {
                        isMatch: jt,
                        score: jt ? 0 : 1,
                        indices: [0, Ht.length - 1]
                      };
                    }
                  }
                  class gt extends ct {
                    constructor(Ht) {
                      super(Ht);
                    }
                    static get type() {
                      return "suffix-exact";
                    }
                    static get multiRegex() {
                      return /^"(.*)"\$$/;
                    }
                    static get singleRegex() {
                      return /^(.*)\$$/;
                    }
                    search(Ht) {
                      const jt = Ht.endsWith(this.pattern);
                      return {
                        isMatch: jt,
                        score: jt ? 0 : 1,
                        indices: [Ht.length - this.pattern.length, Ht.length - 1]
                      };
                    }
                  }
                  class bt extends ct {
                    constructor(Ht) {
                      super(Ht);
                    }
                    static get type() {
                      return "inverse-suffix-exact";
                    }
                    static get multiRegex() {
                      return /^!"(.*)"\$$/;
                    }
                    static get singleRegex() {
                      return /^!(.*)\$$/;
                    }
                    search(Ht) {
                      const jt = !Ht.endsWith(this.pattern);
                      return {
                        isMatch: jt,
                        score: jt ? 0 : 1,
                        indices: [0, Ht.length - 1]
                      };
                    }
                  }
                  class yt extends ct {
                    constructor(Ht, {
                      location: jt = Re.location,
                      threshold: cn = Re.threshold,
                      distance: wn = Re.distance,
                      includeMatches: Rn = Re.includeMatches,
                      findAllMatches: Pn = Re.findAllMatches,
                      minMatchCharLength: Ie = Re.minMatchCharLength,
                      isCaseSensitive: rt = Re.isCaseSensitive,
                      ignoreLocation: ft = Re.ignoreLocation
                    } = {}) {
                      super(Ht), this._bitapSearch = new at(Ht, {
                        location: jt,
                        threshold: cn,
                        distance: wn,
                        includeMatches: Rn,
                        findAllMatches: Pn,
                        minMatchCharLength: Ie,
                        isCaseSensitive: rt,
                        ignoreLocation: ft
                      });
                    }
                    static get type() {
                      return "fuzzy";
                    }
                    static get multiRegex() {
                      return /^"(.*)"$/;
                    }
                    static get singleRegex() {
                      return /^(.*)$/;
                    }
                    search(Ht) {
                      return this._bitapSearch.searchIn(Ht);
                    }
                  }
                  class St extends ct {
                    constructor(Ht) {
                      super(Ht);
                    }
                    static get type() {
                      return "include";
                    }
                    static get multiRegex() {
                      return /^'"(.*)"$/;
                    }
                    static get singleRegex() {
                      return /^'(.*)$/;
                    }
                    search(Ht) {
                      let jt = 0, cn;
                      const wn = [], Rn = this.pattern.length;
                      for (; (cn = Ht.indexOf(this.pattern, jt)) > -1; )
                        jt = cn + Rn, wn.push([cn, jt - 1]);
                      const Pn = !!wn.length;
                      return {
                        isMatch: Pn,
                        score: Pn ? 0 : 1,
                        indices: wn
                      };
                    }
                  }
                  const Lt = [
                    dt,
                    St,
                    it,
                    st,
                    bt,
                    gt,
                    Je,
                    yt
                  ], Rt = Lt.length, vt = / +(?=(?:[^\"]*\"[^\"]*\")*[^\"]*$)/, $t = "|";
                  function Pt(dn, Ht = {}) {
                    return dn.split($t).map((jt) => {
                      let cn = jt.trim().split(vt).filter((Rn) => Rn && !!Rn.trim()), wn = [];
                      for (let Rn = 0, Pn = cn.length; Rn < Pn; Rn += 1) {
                        const Ie = cn[Rn];
                        let rt = !1, ft = -1;
                        for (; !rt && ++ft < Rt; ) {
                          const pt = Lt[ft];
                          let wt = pt.isMultiMatch(Ie);
                          wt && (wn.push(new pt(wt, Ht)), rt = !0);
                        }
                        if (!rt)
                          for (ft = -1; ++ft < Rt; ) {
                            const pt = Lt[ft];
                            let wt = pt.isSingleMatch(Ie);
                            if (wt) {
                              wn.push(new pt(wt, Ht));
                              break;
                            }
                          }
                      }
                      return wn;
                    });
                  }
                  const Xt = /* @__PURE__ */ new Set([yt.type, St.type]);
                  class Nt {
                    constructor(Ht, {
                      isCaseSensitive: jt = Re.isCaseSensitive,
                      includeMatches: cn = Re.includeMatches,
                      minMatchCharLength: wn = Re.minMatchCharLength,
                      ignoreLocation: Rn = Re.ignoreLocation,
                      findAllMatches: Pn = Re.findAllMatches,
                      location: Ie = Re.location,
                      threshold: rt = Re.threshold,
                      distance: ft = Re.distance
                    } = {}) {
                      this.query = null, this.options = {
                        isCaseSensitive: jt,
                        includeMatches: cn,
                        minMatchCharLength: wn,
                        findAllMatches: Pn,
                        ignoreLocation: Rn,
                        location: Ie,
                        threshold: rt,
                        distance: ft
                      }, this.pattern = jt ? Ht : Ht.toLowerCase(), this.query = Pt(this.pattern, this.options);
                    }
                    static condition(Ht, jt) {
                      return jt.useExtendedSearch;
                    }
                    searchIn(Ht) {
                      const jt = this.query;
                      if (!jt)
                        return {
                          isMatch: !1,
                          score: 1
                        };
                      const { includeMatches: cn, isCaseSensitive: wn } = this.options;
                      Ht = wn ? Ht : Ht.toLowerCase();
                      let Rn = 0, Pn = [], Ie = 0;
                      for (let rt = 0, ft = jt.length; rt < ft; rt += 1) {
                        const pt = jt[rt];
                        Pn.length = 0, Rn = 0;
                        for (let wt = 0, Bt = pt.length; wt < Bt; wt += 1) {
                          const At = pt[wt], { isMatch: Dt, indices: Ot, score: un } = At.search(Ht);
                          if (Dt) {
                            if (Rn += 1, Ie += un, cn) {
                              const yn = At.constructor.type;
                              Xt.has(yn) ? Pn = [...Pn, ...Ot] : Pn.push(Ot);
                            }
                          } else {
                            Ie = 0, Rn = 0, Pn.length = 0;
                            break;
                          }
                        }
                        if (Rn) {
                          let wt = {
                            isMatch: !0,
                            score: Ie / Rn
                          };
                          return cn && (wt.indices = Pn), wt;
                        }
                      }
                      return {
                        isMatch: !1,
                        score: 1
                      };
                    }
                  }
                  const Gt = [];
                  function Ft(...dn) {
                    Gt.push(...dn);
                  }
                  function Tt(dn, Ht) {
                    for (let jt = 0, cn = Gt.length; jt < cn; jt += 1) {
                      let wn = Gt[jt];
                      if (wn.condition(dn, Ht))
                        return new wn(dn, Ht);
                    }
                    return new at(dn, Ht);
                  }
                  const It = {
                    AND: "$and",
                    OR: "$or"
                  }, Ct = {
                    PATH: "$path",
                    PATTERN: "$val"
                  }, Kt = (dn) => !!(dn[It.AND] || dn[It.OR]), Vt = (dn) => !!dn[Ct.PATH], Yt = (dn) => !ne(dn) && le(dn) && !Kt(dn), sn = (dn) => ({
                    [It.AND]: Object.keys(dn).map((Ht) => ({
                      [Ht]: dn[Ht]
                    }))
                  });
                  function hn(dn, Ht, { auto: jt = !0 } = {}) {
                    const cn = (wn) => {
                      let Rn = Object.keys(wn);
                      const Pn = Vt(wn);
                      if (!Pn && Rn.length > 1 && !Kt(wn))
                        return cn(sn(wn));
                      if (Yt(wn)) {
                        const rt = Pn ? wn[Ct.PATH] : Rn[0], ft = Pn ? wn[Ct.PATTERN] : wn[rt];
                        if (!oe(ft))
                          throw new Error(pe(rt));
                        const pt = {
                          keyId: Ee(rt),
                          pattern: ft
                        };
                        return jt && (pt.searcher = Tt(ft, Ht)), pt;
                      }
                      let Ie = {
                        children: [],
                        operator: Rn[0]
                      };
                      return Rn.forEach((rt) => {
                        const ft = wn[rt];
                        ne(ft) && ft.forEach((pt) => {
                          Ie.children.push(cn(pt));
                        });
                      }), Ie;
                    };
                    return Kt(dn) || (dn = sn(dn)), cn(dn);
                  }
                  function Qt(dn, { ignoreFieldNorm: Ht = Re.ignoreFieldNorm }) {
                    dn.forEach((jt) => {
                      let cn = 1;
                      jt.matches.forEach(({ key: wn, norm: Rn, score: Pn }) => {
                        const Ie = wn ? wn.weight : null;
                        cn *= Math.pow(
                          Pn === 0 && Ie ? Number.EPSILON : Pn,
                          (Ie || 1) * (Ht ? 1 : Rn)
                        );
                      }), jt.score = cn;
                    });
                  }
                  function Ut(dn, Ht) {
                    const jt = dn.matches;
                    Ht.matches = [], de(jt) && jt.forEach((cn) => {
                      if (!de(cn.indices) || !cn.indices.length)
                        return;
                      const { indices: wn, value: Rn } = cn;
                      let Pn = {
                        indices: wn,
                        value: Rn
                      };
                      cn.key && (Pn.key = cn.key.src), cn.idx > -1 && (Pn.refIndex = cn.idx), Ht.matches.push(Pn);
                    });
                  }
                  function Wt(dn, Ht) {
                    Ht.score = dn.score;
                  }
                  function rn(dn, Ht, {
                    includeMatches: jt = Re.includeMatches,
                    includeScore: cn = Re.includeScore
                  } = {}) {
                    const wn = [];
                    return jt && wn.push(Ut), cn && wn.push(Wt), dn.map((Rn) => {
                      const { idx: Pn } = Rn, Ie = {
                        item: Ht[Pn],
                        refIndex: Pn
                      };
                      return wn.length && wn.forEach((rt) => {
                        rt(Rn, Ie);
                      }), Ie;
                    });
                  }
                  class vn {
                    constructor(Ht, jt = {}, cn) {
                      this.options = { ...Re, ...jt }, this.options.useExtendedSearch, this._keyStore = new Ce(this.options.keys), this.setCollection(Ht, cn);
                    }
                    setCollection(Ht, jt) {
                      if (this._docs = Ht, jt && !(jt instanceof $e))
                        throw new Error(me);
                      this._myIndex = jt || qe(this.options.keys, this._docs, {
                        getFn: this.options.getFn,
                        fieldNormWeight: this.options.fieldNormWeight
                      });
                    }
                    add(Ht) {
                      de(Ht) && (this._docs.push(Ht), this._myIndex.add(Ht));
                    }
                    remove(Ht = () => !1) {
                      const jt = [];
                      for (let cn = 0, wn = this._docs.length; cn < wn; cn += 1) {
                        const Rn = this._docs[cn];
                        Ht(Rn, cn) && (this.removeAt(cn), cn -= 1, wn -= 1, jt.push(Rn));
                      }
                      return jt;
                    }
                    removeAt(Ht) {
                      this._docs.splice(Ht, 1), this._myIndex.removeAt(Ht);
                    }
                    getIndex() {
                      return this._myIndex;
                    }
                    search(Ht, { limit: jt = -1 } = {}) {
                      const {
                        includeMatches: cn,
                        includeScore: wn,
                        shouldSort: Rn,
                        sortFn: Pn,
                        ignoreFieldNorm: Ie
                      } = this.options;
                      let rt = oe(Ht) ? oe(this._docs[0]) ? this._searchStringList(Ht) : this._searchObjectList(Ht) : this._searchLogical(Ht);
                      return Qt(rt, { ignoreFieldNorm: Ie }), Rn && rt.sort(Pn), se(jt) && jt > -1 && (rt = rt.slice(0, jt)), rn(rt, this._docs, {
                        includeMatches: cn,
                        includeScore: wn
                      });
                    }
                    _searchStringList(Ht) {
                      const jt = Tt(Ht, this.options), { records: cn } = this._myIndex, wn = [];
                      return cn.forEach(({ v: Rn, i: Pn, n: Ie }) => {
                        if (!de(Rn))
                          return;
                        const { isMatch: rt, score: ft, indices: pt } = jt.searchIn(Rn);
                        rt && wn.push({
                          item: Rn,
                          idx: Pn,
                          matches: [{ score: ft, value: Rn, norm: Ie, indices: pt }]
                        });
                      }), wn;
                    }
                    _searchLogical(Ht) {
                      const jt = hn(Ht, this.options), cn = (Ie, rt, ft) => {
                        if (!Ie.children) {
                          const { keyId: wt, searcher: Bt } = Ie, At = this._findMatches({
                            key: this._keyStore.get(wt),
                            value: this._myIndex.getValueForItemAtKeyId(rt, wt),
                            searcher: Bt
                          });
                          return At && At.length ? [
                            {
                              idx: ft,
                              item: rt,
                              matches: At
                            }
                          ] : [];
                        }
                        const pt = [];
                        for (let wt = 0, Bt = Ie.children.length; wt < Bt; wt += 1) {
                          const At = Ie.children[wt], Dt = cn(At, rt, ft);
                          if (Dt.length)
                            pt.push(...Dt);
                          else if (Ie.operator === It.AND)
                            return [];
                        }
                        return pt;
                      }, wn = this._myIndex.records, Rn = {}, Pn = [];
                      return wn.forEach(({ $: Ie, i: rt }) => {
                        if (de(Ie)) {
                          let ft = cn(jt, Ie, rt);
                          ft.length && (Rn[rt] || (Rn[rt] = { idx: rt, item: Ie, matches: [] }, Pn.push(Rn[rt])), ft.forEach(({ matches: pt }) => {
                            Rn[rt].matches.push(...pt);
                          }));
                        }
                      }), Pn;
                    }
                    _searchObjectList(Ht) {
                      const jt = Tt(Ht, this.options), { keys: cn, records: wn } = this._myIndex, Rn = [];
                      return wn.forEach(({ $: Pn, i: Ie }) => {
                        if (!de(Pn))
                          return;
                        let rt = [];
                        cn.forEach((ft, pt) => {
                          rt.push(
                            ...this._findMatches({
                              key: ft,
                              value: Pn[pt],
                              searcher: jt
                            })
                          );
                        }), rt.length && Rn.push({
                          idx: Ie,
                          item: Pn,
                          matches: rt
                        });
                      }), Rn;
                    }
                    _findMatches({ key: Ht, value: jt, searcher: cn }) {
                      if (!de(jt))
                        return [];
                      let wn = [];
                      if (ne(jt))
                        jt.forEach(({ v: Rn, i: Pn, n: Ie }) => {
                          if (!de(Rn))
                            return;
                          const { isMatch: rt, score: ft, indices: pt } = cn.searchIn(Rn);
                          rt && wn.push({
                            score: ft,
                            key: Ht,
                            value: Rn,
                            idx: Pn,
                            norm: Ie,
                            indices: pt
                          });
                        });
                      else {
                        const { v: Rn, n: Pn } = jt, { isMatch: Ie, score: rt, indices: ft } = cn.searchIn(Rn);
                        Ie && wn.push({ score: rt, key: Ht, value: Rn, norm: Pn, indices: ft });
                      }
                      return wn;
                    }
                  }
                  vn.version = "6.6.2", vn.createIndex = qe, vn.parseIndex = Xe, vn.config = Re, vn.parseQuery = hn, Ft(Nt);
                }
              ),
              /***/
              791: (
                /***/
                function(J, Z, te) {
                  te.r(Z), te.d(Z, {
                    __DO_NOT_USE__ActionTypes: function() {
                      return (
                        /* binding */
                        fe
                      );
                    },
                    applyMiddleware: function() {
                      return (
                        /* binding */
                        Se
                      );
                    },
                    bindActionCreators: function() {
                      return (
                        /* binding */
                        be
                      );
                    },
                    combineReducers: function() {
                      return (
                        /* binding */
                        ge
                      );
                    },
                    compose: function() {
                      return (
                        /* binding */
                        Ce
                      );
                    },
                    createStore: function() {
                      return (
                        /* binding */
                        me
                      );
                    },
                    legacy_createStore: function() {
                      return (
                        /* binding */
                        pe
                      );
                    }
                  });
                  function ne(we) {
                    "@babel/helpers - typeof";
                    return ne = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(Ee) {
                      return typeof Ee;
                    } : function(Ee) {
                      return Ee && typeof Symbol == "function" && Ee.constructor === Symbol && Ee !== Symbol.prototype ? "symbol" : typeof Ee;
                    }, ne(we);
                  }
                  function ee(we, Ee) {
                    if (ne(we) !== "object" || we === null) return we;
                    var xe = we[Symbol.toPrimitive];
                    if (xe !== void 0) {
                      var Le = xe.call(we, Ee || "default");
                      if (ne(Le) !== "object") return Le;
                      throw new TypeError("@@toPrimitive must return a primitive value.");
                    }
                    return (Ee === "string" ? String : Number)(we);
                  }
                  function ae(we) {
                    var Ee = ee(we, "string");
                    return ne(Ee) === "symbol" ? Ee : String(Ee);
                  }
                  function oe(we, Ee, xe) {
                    return Ee = ae(Ee), Ee in we ? Object.defineProperty(we, Ee, {
                      value: xe,
                      enumerable: !0,
                      configurable: !0,
                      writable: !0
                    }) : we[Ee] = xe, we;
                  }
                  function se(we, Ee) {
                    var xe = Object.keys(we);
                    if (Object.getOwnPropertySymbols) {
                      var Le = Object.getOwnPropertySymbols(we);
                      Ee && (Le = Le.filter(function(Qe) {
                        return Object.getOwnPropertyDescriptor(we, Qe).enumerable;
                      })), xe.push.apply(xe, Le);
                    }
                    return xe;
                  }
                  function ce(we) {
                    for (var Ee = 1; Ee < arguments.length; Ee++) {
                      var xe = arguments[Ee] != null ? arguments[Ee] : {};
                      Ee % 2 ? se(Object(xe), !0).forEach(function(Le) {
                        oe(we, Le, xe[Le]);
                      }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(we, Object.getOwnPropertyDescriptors(xe)) : se(Object(xe)).forEach(function(Le) {
                        Object.defineProperty(we, Le, Object.getOwnPropertyDescriptor(xe, Le));
                      });
                    }
                    return we;
                  }
                  function le(we) {
                    return "Minified Redux error #" + we + "; visit https://redux.js.org/Errors?code=" + we + " for the full message or use the non-minified dev environment for full errors. ";
                  }
                  var ue = function() {
                    return typeof Symbol == "function" && Symbol.observable || "@@observable";
                  }(), de = function() {
                    return Math.random().toString(36).substring(7).split("").join(".");
                  }, fe = {
                    INIT: "@@redux/INIT" + de(),
                    REPLACE: "@@redux/REPLACE" + de(),
                    PROBE_UNKNOWN_ACTION: function() {
                      return "@@redux/PROBE_UNKNOWN_ACTION" + de();
                    }
                  };
                  function he(we) {
                    if (typeof we != "object" || we === null) return !1;
                    for (var Ee = we; Object.getPrototypeOf(Ee) !== null; )
                      Ee = Object.getPrototypeOf(Ee);
                    return Object.getPrototypeOf(we) === Ee;
                  }
                  function me(we, Ee, xe) {
                    var Le;
                    if (typeof Ee == "function" && typeof xe == "function" || typeof xe == "function" && typeof arguments[3] == "function")
                      throw new Error(le(0));
                    if (typeof Ee == "function" && typeof xe > "u" && (xe = Ee, Ee = void 0), typeof xe < "u") {
                      if (typeof xe != "function")
                        throw new Error(le(1));
                      return xe(me)(we, Ee);
                    }
                    if (typeof we != "function")
                      throw new Error(le(2));
                    var Qe = we, ke = Ee, Me = [], Re = Me, De = !1;
                    function Pe() {
                      Re === Me && (Re = Me.slice());
                    }
                    function $e() {
                      if (De)
                        throw new Error(le(3));
                      return ke;
                    }
                    function qe(Ge) {
                      if (typeof Ge != "function")
                        throw new Error(le(4));
                      if (De)
                        throw new Error(le(5));
                      var ze = !0;
                      return Pe(), Re.push(Ge), function() {
                        if (ze) {
                          if (De)
                            throw new Error(le(6));
                          ze = !1, Pe();
                          var at = Re.indexOf(Ge);
                          Re.splice(at, 1), Me = null;
                        }
                      };
                    }
                    function Xe(Ge) {
                      if (!he(Ge))
                        throw new Error(le(7));
                      if (typeof Ge.type > "u")
                        throw new Error(le(8));
                      if (De)
                        throw new Error(le(9));
                      try {
                        De = !0, ke = Qe(ke, Ge);
                      } finally {
                        De = !1;
                      }
                      for (var ze = Me = Re, We = 0; We < ze.length; We++) {
                        var at = ze[We];
                        at();
                      }
                      return Ge;
                    }
                    function Ve(Ge) {
                      if (typeof Ge != "function")
                        throw new Error(le(10));
                      Qe = Ge, Xe({
                        type: fe.REPLACE
                      });
                    }
                    function Ze() {
                      var Ge, ze = qe;
                      return Ge = {
                        /**
                         * The minimal observable subscription method.
                         * @param {Object} observer Any object that can be used as an observer.
                         * The observer object should have a `next` method.
                         * @returns {subscription} An object with an `unsubscribe` method that can
                         * be used to unsubscribe the observable from the store, and prevent further
                         * emission of values from the observable.
                         */
                        subscribe: function(at) {
                          if (typeof at != "object" || at === null)
                            throw new Error(le(11));
                          function ct() {
                            at.next && at.next($e());
                          }
                          ct();
                          var et = ze(ct);
                          return {
                            unsubscribe: et
                          };
                        }
                      }, Ge[ue] = function() {
                        return this;
                      }, Ge;
                    }
                    return Xe({
                      type: fe.INIT
                    }), Le = {
                      dispatch: Xe,
                      subscribe: qe,
                      getState: $e,
                      replaceReducer: Ve
                    }, Le[ue] = Ze, Le;
                  }
                  var pe = me;
                  function ye(we) {
                    Object.keys(we).forEach(function(Ee) {
                      var xe = we[Ee], Le = xe(void 0, {
                        type: fe.INIT
                      });
                      if (typeof Le > "u")
                        throw new Error(le(12));
                      if (typeof xe(void 0, {
                        type: fe.PROBE_UNKNOWN_ACTION()
                      }) > "u")
                        throw new Error(le(13));
                    });
                  }
                  function ge(we) {
                    for (var Ee = Object.keys(we), xe = {}, Le = 0; Le < Ee.length; Le++) {
                      var Qe = Ee[Le];
                      typeof we[Qe] == "function" && (xe[Qe] = we[Qe]);
                    }
                    var ke = Object.keys(xe), Me;
                    try {
                      ye(xe);
                    } catch (Re) {
                      Me = Re;
                    }
                    return function(De, Pe) {
                      if (De === void 0 && (De = {}), Me)
                        throw Me;
                      for (var $e = !1, qe = {}, Xe = 0; Xe < ke.length; Xe++) {
                        var Ve = ke[Xe], Ze = xe[Ve], Ge = De[Ve], ze = Ze(Ge, Pe);
                        if (typeof ze > "u")
                          throw Pe && Pe.type, new Error(le(14));
                        qe[Ve] = ze, $e = $e || ze !== Ge;
                      }
                      return $e = $e || ke.length !== Object.keys(De).length, $e ? qe : De;
                    };
                  }
                  function ve(we, Ee) {
                    return function() {
                      return Ee(we.apply(this, arguments));
                    };
                  }
                  function be(we, Ee) {
                    if (typeof we == "function")
                      return ve(we, Ee);
                    if (typeof we != "object" || we === null)
                      throw new Error(le(16));
                    var xe = {};
                    for (var Le in we) {
                      var Qe = we[Le];
                      typeof Qe == "function" && (xe[Le] = ve(Qe, Ee));
                    }
                    return xe;
                  }
                  function Ce() {
                    for (var we = arguments.length, Ee = new Array(we), xe = 0; xe < we; xe++)
                      Ee[xe] = arguments[xe];
                    return Ee.length === 0 ? function(Le) {
                      return Le;
                    } : Ee.length === 1 ? Ee[0] : Ee.reduce(function(Le, Qe) {
                      return function() {
                        return Le(Qe.apply(void 0, arguments));
                      };
                    });
                  }
                  function Se() {
                    for (var we = arguments.length, Ee = new Array(we), xe = 0; xe < we; xe++)
                      Ee[xe] = arguments[xe];
                    return function(Le) {
                      return function() {
                        var Qe = Le.apply(void 0, arguments), ke = function() {
                          throw new Error(le(15));
                        }, Me = {
                          getState: Qe.getState,
                          dispatch: function() {
                            return ke.apply(void 0, arguments);
                          }
                        }, Re = Ee.map(function(De) {
                          return De(Me);
                        });
                        return ke = Ce.apply(void 0, Re)(Qe.dispatch), ce(ce({}, Qe), {}, {
                          dispatch: ke
                        });
                      };
                    };
                  }
                }
              )
              /******/
            }, V = {};
            function q(J) {
              var Z = V[J];
              if (Z !== void 0)
                return Z.exports;
              var te = V[J] = {
                /******/
                // no module.id needed
                /******/
                // no module.loaded needed
                /******/
                exports: {}
                /******/
              };
              return H[J].call(te.exports, te, te.exports, q), te.exports;
            }
            (function() {
              q.n = function(J) {
                var Z = J && J.__esModule ? (
                  /******/
                  function() {
                    return J.default;
                  }
                ) : (
                  /******/
                  function() {
                    return J;
                  }
                );
                return q.d(Z, { a: Z }), Z;
              };
            })(), function() {
              q.d = function(J, Z) {
                for (var te in Z)
                  q.o(Z, te) && !q.o(J, te) && Object.defineProperty(J, te, { enumerable: !0, get: Z[te] });
              };
            }(), function() {
              q.o = function(J, Z) {
                return Object.prototype.hasOwnProperty.call(J, Z);
              };
            }(), function() {
              q.r = function(J) {
                typeof Symbol < "u" && Symbol.toStringTag && Object.defineProperty(J, Symbol.toStringTag, { value: "Module" }), Object.defineProperty(J, "__esModule", { value: !0 });
              };
            }();
            var G = {};
            return function() {
              var J = q(373), Z = /* @__PURE__ */ q.n(J);
              q(187), q(883), q(789), q(686), G.default = Z();
            }(), G = G.default, G;
          }()
        );
      });
    }(choices)), choices.exports;
  }
  var choicesExports = requireChoices();
  const Choices = /* @__PURE__ */ getDefaultExportFromCjs$1(choicesExports), choices_css = '.choices{position:relative;overflow:hidden;margin-bottom:24px;font-size:16px;}.choices:focus{outline:none;}.choices:last-child{margin-bottom:0;}.choices.is-open{overflow:visible;}.choices.is-disabled .choices__inner,.choices.is-disabled .choices__input{background-color:#eaeaea;cursor:not-allowed;-webkit-user-select:none;user-select:none;}.choices.is-disabled .choices__item{cursor:not-allowed;}.choices [hidden]{display:none !important;}.choices[data-type*=select-one]{cursor:pointer;}.choices[data-type*=select-one] .choices__inner{padding-bottom:7.5px;}.choices[data-type*=select-one] .choices__input{display:block;width:100%;padding:10px;border-bottom:1px solid #ddd;background-color:#fff;margin:0;}.choices[data-type*=select-one] .choices__button{background-image:url("data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjEiIGhlaWdodD0iMjEiIHZpZXdCb3g9IjAgMCAyMSAyMSIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj48ZyBmaWxsPSIjMDAwIiBmaWxsLXJ1bGU9ImV2ZW5vZGQiPjxwYXRoIGQ9Ik0yLjU5Mi4wNDRsMTguMzY0IDE4LjM2NC0yLjU0OCAyLjU0OEwuMDQ0IDIuNTkyeiIvPjxwYXRoIGQ9Ik0wIDE4LjM2NEwxOC4zNjQgMGwyLjU0OCAyLjU0OEwyLjU0OCAyMC45MTJ6Ii8+PC9nPjwvc3ZnPg==");padding:0;background-size:8px;position:absolute;top:50%;right:0;margin-top:-10px;margin-right:25px;height:20px;width:20px;border-radius:10em;opacity:0.25;}.choices[data-type*=select-one] .choices__button:hover,.choices[data-type*=select-one] .choices__button:focus{opacity:1;}.choices[data-type*=select-one] .choices__button:focus{box-shadow:0 0 0 2px #00bcd4;}.choices[data-type*=select-one] .choices__item[data-value=""] .choices__button{display:none;}.choices[data-type*=select-one]::after{content:"";height:0;width:0;border-style:solid;border-color:#333 transparent transparent transparent;border-width:5px;position:absolute;right:11.5px;top:50%;margin-top:-2.5px;pointer-events:none;}.choices[data-type*=select-one].is-open::after{border-color:transparent transparent #333 transparent;margin-top:-7.5px;}.choices[data-type*=select-one][dir=rtl]::after{left:11.5px;right:auto;}.choices[data-type*=select-one][dir=rtl] .choices__button{right:auto;left:0;margin-left:25px;margin-right:0;}.choices[data-type*=select-multiple] .choices__inner,.choices[data-type*=text] .choices__inner{cursor:text;}.choices[data-type*=select-multiple] .choices__button,.choices[data-type*=text] .choices__button{position:relative;display:inline-block;margin-top:0;margin-right:-4px;margin-bottom:0;margin-left:8px;padding-left:16px;border-left:1px solid #008fa1;background-image:url("data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjEiIGhlaWdodD0iMjEiIHZpZXdCb3g9IjAgMCAyMSAyMSIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj48ZyBmaWxsPSIjRkZGIiBmaWxsLXJ1bGU9ImV2ZW5vZGQiPjxwYXRoIGQ9Ik0yLjU5Mi4wNDRsMTguMzY0IDE4LjM2NC0yLjU0OCAyLjU0OEwuMDQ0IDIuNTkyeiIvPjxwYXRoIGQ9Ik0wIDE4LjM2NEwxOC4zNjQgMGwyLjU0OCAyLjU0OEwyLjU0OCAyMC45MTJ6Ii8+PC9nPjwvc3ZnPg==");background-size:8px;width:8px;line-height:1;opacity:0.75;border-radius:0;}.choices[data-type*=select-multiple] .choices__button:hover,.choices[data-type*=select-multiple] .choices__button:focus,.choices[data-type*=text] .choices__button:hover,.choices[data-type*=text] .choices__button:focus{opacity:1;}.choices__inner{display:inline-block;vertical-align:top;width:100%;background-color:#f9f9f9;padding:7.5px 7.5px 3.75px;border:1px solid #ddd;border-radius:2.5px;font-size:14px;min-height:44px;overflow:hidden;}.is-focused .choices__inner,.is-open .choices__inner{border-color:#b7b7b7;}.is-open .choices__inner{border-radius:2.5px 2.5px 0 0;}.is-flipped.is-open .choices__inner{border-radius:0 0 2.5px 2.5px;}.choices__list{margin:0;padding-left:0;list-style:none;}.choices__list--single{display:inline-block;padding:4px 16px 4px 4px;width:100%;}[dir=rtl] .choices__list--single{padding-right:4px;padding-left:16px;}.choices__list--single .choices__item{width:100%;}.choices__list--multiple{display:inline;}.choices__list--multiple .choices__item{display:inline-block;vertical-align:middle;border-radius:20px;padding:4px 10px;font-size:12px;font-weight:500;margin-right:3.75px;margin-bottom:3.75px;background-color:#00bcd4;border:1px solid #00a5bb;color:#fff;word-break:break-all;box-sizing:border-box;}.choices__list--multiple .choices__item[data-deletable]{padding-right:5px;}[dir=rtl] .choices__list--multiple .choices__item{margin-right:0;margin-left:3.75px;}.choices__list--multiple .choices__item.is-highlighted{background-color:#00a5bb;border:1px solid #008fa1;}.is-disabled .choices__list--multiple .choices__item{background-color:#aaaaaa;border:1px solid #919191;}.choices__list--dropdown,.choices__list[aria-expanded]{visibility:hidden;z-index:1;position:absolute;width:100%;background-color:#fff;border:1px solid #ddd;top:100%;margin-top:-1px;border-bottom-left-radius:2.5px;border-bottom-right-radius:2.5px;overflow:hidden;word-break:break-all;will-change:visibility;}.is-active.choices__list--dropdown,.is-active.choices__list[aria-expanded]{visibility:visible;}.is-open .choices__list--dropdown,.is-open .choices__list[aria-expanded]{border-color:#b7b7b7;}.is-flipped .choices__list--dropdown,.is-flipped .choices__list[aria-expanded]{top:auto;bottom:100%;margin-top:0;margin-bottom:-1px;border-radius:0.25rem 0.25rem 0 0;}.choices__list--dropdown .choices__list,.choices__list[aria-expanded] .choices__list{position:relative;max-height:300px;overflow:auto;-webkit-overflow-scrolling:touch;will-change:scroll-position;}.choices__list--dropdown .choices__item,.choices__list[aria-expanded] .choices__item{position:relative;padding:10px;font-size:14px;}[dir=rtl] .choices__list--dropdown .choices__item,[dir=rtl] .choices__list[aria-expanded] .choices__item{text-align:right;}@media (min-width: 640px){.choices__list--dropdown .choices__item--selectable,.choices__list[aria-expanded] .choices__item--selectable{padding-right:100px;}.choices__list--dropdown .choices__item--selectable::after,.choices__list[aria-expanded] .choices__item--selectable::after{content:attr(data-select-text);font-size:12px;opacity:0;position:absolute;right:10px;top:50%;transform:translateY(-50%);}[dir=rtl] .choices__list--dropdown .choices__item--selectable,[dir=rtl] .choices__list[aria-expanded] .choices__item--selectable{text-align:right;padding-left:100px;padding-right:10px;}[dir=rtl] .choices__list--dropdown .choices__item--selectable::after,[dir=rtl] .choices__list[aria-expanded] .choices__item--selectable::after{right:auto;left:10px;}}.choices__list--dropdown .choices__item--selectable.is-highlighted,.choices__list[aria-expanded] .choices__item--selectable.is-highlighted{background-color:#f2f2f2;}.choices__list--dropdown .choices__item--selectable.is-highlighted::after,.choices__list[aria-expanded] .choices__item--selectable.is-highlighted::after{opacity:0.5;}.choices__item{cursor:default;}.choices__item--selectable{cursor:pointer;}.choices__item--disabled{cursor:not-allowed;-webkit-user-select:none;user-select:none;opacity:0.5;}.choices__heading{font-weight:600;font-size:12px;padding:10px;border-bottom:1px solid #f7f7f7;color:gray;}.choices__button{text-indent:-9999px;-webkit-appearance:none;appearance:none;border:0;background-color:transparent;background-repeat:no-repeat;background-position:center;cursor:pointer;}.choices__button:focus{outline:none;}.choices__input{display:inline-block;vertical-align:baseline;background-color:#f9f9f9;font-size:14px;margin-bottom:5px;border:0;border-radius:0;max-width:100%;padding:4px 0 4px 2px;}.choices__input:focus{outline:0;}.choices__input::-webkit-search-decoration,.choices__input::-webkit-search-cancel-button,.choices__input::-webkit-search-results-button,.choices__input::-webkit-search-results-decoration{display:none;}.choices__input::-ms-clear,.choices__input::-ms-reveal{display:none;width:0;height:0;}[dir=rtl] .choices__input{padding-right:2px;padding-left:0;}.choices__placeholder{opacity:0.5;}.choices{width:100%;}.choices{box-sizing:border-box;}.choices *,.choices *:before,.choices *:after{box-sizing:inherit;}input[type="search"]{margin:0;}.choices__inner .choices__item.light{background-color:rgba(0, 126, 255, 0.08);border-radius:5px;border:1px solid rgba(0, 126, 255, 0.24);color:#007eff;}.choices__inner .choices__item.solid{background-color:#1f77b4;border:none;border-radius:5px;color:white;}.choices__inner .choices__item.solid .is-highlighted{background-color:#1f77b4;border:none;}.choices__input{background-color:transparent;}.choices__inner{background:transparent;border:1px solid darkgray;border-radius:5px;min-height:0;padding:calc(var(--padding-vertical) / 2) var(--padding-horizontal);}.choices__list{white-space:initial;}.choices__list--dropdown,.choices__list[aria-expanded]{z-index:var(--bokeh-top-level);}.choices__list--dropdown .choices__item--selectable,.choices__list[aria-expanded] .choices__item--selectable{padding-right:0;}.choices[data-type*=select-multiple] .choices__button.light{background-image:url(data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjEiIGhlaWdodD0iMjEiIHZpZXdCb3g9IjAgMCAyMSAyMSIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj48ZyBmaWxsPSIjMDA3ZWZmIiBmaWxsLXJ1bGU9ImV2ZW5vZGQiPjxwYXRoIGQ9Ik0yLjU5Mi4wNDRsMTguMzY0IDE4LjM2NC0yLjU0OCAyLjU0OEwuMDQ0IDIuNTkyeiIvPjxwYXRoIGQ9Ik0wIDE4LjM2NEwxOC4zNjQgMGwyLjU0OCAyLjU0OEwyLjU0OCAyMC45MTJ6Ii8+PC9nPjwvc3ZnPg==);}.choices[data-type*=select-multiple] .choices__button.solid{background-image:url(data:image/svg+xml;base64,PHN2ZyB3aWR0aD0iMjEiIGhlaWdodD0iMjEiIHZpZXdCb3g9IjAgMCAyMSAyMSIgeG1sbnM9Imh0dHA6Ly93d3cudzMub3JnLzIwMDAvc3ZnIj48ZyBmaWxsPSIjZmZmZmZmIiBmaWxsLXJ1bGU9ImV2ZW5vZGQiPjxwYXRoIGQ9Ik0yLjU5Mi4wNDRsMTguMzY0IDE4LjM2NC0yLjU0OCAyLjU0OEwuMDQ0IDIuNTkyeiIvPjxwYXRoIGQ9Ik0wIDE4LjM2NEwxOC4zNjQgMGwyLjU0OCAyLjU0OEwyLjU0OCAyMC45MTJ6Ii8+PC9nPjwvc3ZnPg==);border-left:1px solid white;opacity:1;}';
  function retarget(O) {
    return Object.defineProperty(O, "target", {
      get: () => O.composedPath()[0] ?? null,
      configurable: !0
    }), O;
  }
  class OurChoices extends Choices {
    _onFocus(_) {
      super._onFocus(retarget(_));
    }
    _onBlur(_) {
      super._onBlur(retarget(_));
    }
    _onKeyUp(_) {
      super._onKeyUp(retarget(_));
    }
    _onKeyDown(_) {
      super._onKeyDown(retarget(_));
    }
    _onClick(_) {
      super._onClick(retarget(_));
    }
    _onTouchEnd(_) {
      super._onTouchEnd(retarget(_));
    }
    _onMouseDown(_) {
      super._onMouseDown(retarget(_));
    }
    _onMouseOver(_) {
      super._onMouseOver(retarget(_));
    }
  }
  mt(OurChoices, "__name__", "OurChoices");
  class MultiChoiceView extends InputWidgetView {
    constructor() {
      super(...arguments);
      mt(this, "choice_el");
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.disabled.change, () => this.set_disabled());
      const { value: H, max_items: V, option_limit: q, search_option_limit: G, delete_button: J, placeholder: Z, options: te, name: ne, title: ee } = this.model.properties;
      this.on_change([V, q, G, J, Z, te, ne, ee], () => this.render()), this.on_change(H, () => {
        is_equal(this.model.value, this._current_values) || this.render();
      });
    }
    stylesheets() {
      return [...super.stylesheets(), choices_css];
    }
    _render_input() {
      return this.input_el = select$1({
        multiple: !0,
        class: input,
        name: this.model.name,
        disabled: this.model.disabled
      });
    }
    render() {
      super.render();
      const H = new Set(this.model.value), V = this.model.options.map((te) => {
        let ne, ee;
        return isString$2(te) ? ne = ee = te : [ne, ee] = te, { value: ne, label: ee, selected: H.has(ne) };
      }), q = this.model.solid ? "solid" : "light", G = `choices__item ${q}`, J = `choices__button ${q}`, Z = {
        choices: V,
        itemSelectText: "",
        duplicateItemsAllowed: !1,
        shouldSort: !1,
        removeItemButton: this.model.delete_button,
        classNames: { item: G, button: J },
        // XXX: missing typings
        placeholderValue: this.model.placeholder,
        maxItemCount: this.model.max_items ?? -1,
        renderChoiceLimit: this.model.option_limit ?? -1,
        searchResultLimit: this.model.search_option_limit ?? 4
      };
      this.choice_el = new OurChoices(this.input_el, Z), this.input_el.addEventListener("change", () => this.change_input());
    }
    set_disabled() {
      this.model.disabled ? this.choice_el.disable() : this.choice_el.enable();
    }
    get _current_values() {
      return this.choice_el.getValue().map((V) => V.value);
    }
    change_input() {
      this.model.value = this._current_values, super.change_input();
    }
  }
  mt(MultiChoiceView, "__name__", "MultiChoiceView");
  const r4 = class r4 extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(r4, "__name__", "MultiChoice"), r4.prototype.default_view = MultiChoiceView, r4.define(({ Bool: _, Int: H, Str: V, List: q, Tuple: G, Or: J, Nullable: Z }) => ({
    value: [q(V), []],
    options: [q(J(V, G(V, V))), []],
    max_items: [Z(H), null],
    delete_button: [_, !0],
    placeholder: [Z(V), null],
    option_limit: [Z(H), null],
    search_option_limit: [Z(H), null],
    solid: [_, !0]
  }));
  let MultiChoice = r4;
  class MultiSelectView extends InputWidgetView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.value.change, () => this.render_selection()), this.connect(this.model.properties.options.change, () => this.render()), this.connect(this.model.properties.name.change, () => this.render()), this.connect(this.model.properties.title.change, () => this.render()), this.connect(this.model.properties.size.change, () => this.render()), this.connect(this.model.properties.disabled.change, () => this.render());
    }
    _render_input() {
      const _ = this.model.options.map((H) => {
        let V, q;
        return isString$2(H) ? V = q = H : [V, q] = H, option({ value: V }, q);
      });
      return this.input_el = select$1({
        multiple: !0,
        class: input,
        name: this.model.name,
        disabled: this.model.disabled
      }, _), this.input_el.addEventListener("change", () => this.change_input()), this.input_el;
    }
    render() {
      super.render(), this.render_selection();
    }
    render_selection() {
      const _ = new Set(this.model.value);
      for (const H of this.shadow_el.querySelectorAll("option"))
        H.selected = _.has(H.value);
      this.input_el.size = this.model.size;
    }
    change_input() {
      const _ = this.shadow_el.querySelector("select:focus") != null, H = [];
      for (const V of this.shadow_el.querySelectorAll("option"))
        V.selected && H.push(V.value);
      this.model.value = H, super.change_input(), _ && this.input_el.focus();
    }
  }
  mt(MultiSelectView, "__name__", "MultiSelectView");
  const i4 = class i4 extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(i4, "__name__", "MultiSelect"), i4.prototype.default_view = MultiSelectView, i4.define(({ Int: _, Str: H, List: V, Tuple: q, Or: G }) => ({
    value: [V(H), []],
    options: [V(G(H, q(H, H))), []],
    size: [_, 4]
    // 4 is the HTML default
  }));
  let MultiSelect = i4;
  class MultipleDatePickerView extends BaseDatePickerView {
    get flatpickr_options() {
      return {
        ...super.flatpickr_options,
        mode: "multiple",
        conjunction: this.model.separator
      };
    }
    _on_change(_) {
      this.model.value = _.map((H) => this._format_date(H));
    }
  }
  mt(MultipleDatePickerView, "__name__", "MultipleDatePickerView");
  const a4 = class a4 extends BaseDatePicker {
    constructor(_) {
      super(_);
    }
  };
  mt(a4, "__name__", "MultipleDatePicker"), a4.prototype.default_view = MultipleDatePickerView, a4.define(({ Str: _, List: H }) => ({
    value: [H(DateLike), []],
    separator: [_, ", "]
  }));
  let MultipleDatePicker = a4;
  class MultipleDatetimePickerView extends BaseDatetimePickerView {
    get flatpickr_options() {
      return {
        ...super.flatpickr_options,
        mode: "multiple",
        conjunction: this.model.separator
      };
    }
    _on_change(_) {
      this.model.value = _.map((H) => this._format_date(H));
    }
  }
  mt(MultipleDatetimePickerView, "__name__", "MultipleDatetimePickerView");
  const o4 = class o4 extends BaseDatetimePicker {
    constructor(_) {
      super(_);
    }
  };
  mt(o4, "__name__", "MultipleDatetimePicker"), o4.prototype.default_view = MultipleDatetimePickerView, o4.define(({ Str: _, List: H }) => ({
    value: [H(DateLike), []],
    separator: [_, ", "]
  }));
  let MultipleDatetimePicker = o4;
  const int_regex = /^[-+]?\d*$/, float_regex = /^[-+]?\d*\.?\d*(?:(?:\d|\d.)[eE][-+]?)*\d*$/;
  class NumericInputView extends InputWidgetView {
    constructor() {
      super(...arguments);
      mt(this, "old_value");
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.name.change, () => this.input_el.name = this.model.name ?? ""), this.connect(this.model.properties.value.change, () => {
        this.input_el.value = this.format_value, this.old_value = this.input_el.value;
      }), this.connect(this.model.properties.low.change, () => {
        const { value: H, low: V, high: q } = this.model;
        V != null && q != null && assert$2(V <= q, "Invalid bounds, low must be inferior to high"), H != null && V != null && H < V && (this.model.value = V);
      }), this.connect(this.model.properties.high.change, () => {
        const { value: H, low: V, high: q } = this.model;
        V != null && q != null && assert$2(q >= V, "Invalid bounds, high must be superior to low"), H != null && q != null && H > q && (this.model.value = q);
      }), this.connect(this.model.properties.high.change, () => this.input_el.placeholder = this.model.placeholder), this.connect(this.model.properties.disabled.change, () => this.input_el.disabled = this.model.disabled), this.connect(this.model.properties.placeholder.change, () => this.input_el.placeholder = this.model.placeholder);
    }
    get format_value() {
      return this.model.value != null ? this.model.pretty(this.model.value) : "";
    }
    _set_input_filter(H) {
      this.input_el.addEventListener("input", () => {
        const { selectionStart: V, selectionEnd: q } = this.input_el;
        if (H(this.input_el.value))
          this.old_value = this.input_el.value;
        else {
          const G = this.old_value.length - this.input_el.value.length;
          this.input_el.value = this.old_value, V != null && q != null && this.input_el.setSelectionRange(V - 1, q + G);
        }
      });
    }
    _render_input() {
      return this.input_el = input$1({
        type: "text",
        class: input,
        name: this.model.name,
        value: this.format_value,
        disabled: this.model.disabled,
        placeholder: this.model.placeholder
      });
    }
    render() {
      super.render(), this.old_value = this.format_value, this.set_input_filter(), this.input_el.addEventListener("change", () => this.change_input()), this.input_el.addEventListener("focusout", () => this.input_el.value = this.format_value);
    }
    set_input_filter() {
      const H = this.model.mode == "int" ? int_regex : float_regex;
      this._set_input_filter((V) => H.test(V));
    }
    bound_value(H) {
      let V = H;
      const { low: q, high: G } = this.model;
      return V = q != null ? Math.max(q, V) : V, V = G != null ? Math.min(G, V) : V, V;
    }
    get value() {
      let H = this.input_el.value != "" ? Number(this.input_el.value) : null;
      return H != null && (H = this.bound_value(H)), H;
    }
    change_input() {
      this.value == null ? this.model.value = null : Number.isNaN(this.value) || (this.model.value = this.value);
    }
  }
  mt(NumericInputView, "__name__", "NumericInputView");
  const s4 = class s4 extends InputWidget {
    constructor(_) {
      super(_);
    }
    _formatter(_, H) {
      return isString$2(H) ? numbroExports.format(_, H) : H.doFormat([_], { loc: 0 })[0];
    }
    pretty(_) {
      return this.format != null ? this._formatter(_, this.format) : `${_}`;
    }
  };
  mt(s4, "__name__", "NumericInput"), s4.prototype.default_view = NumericInputView, s4.define(({ Float: _, Str: H, Enum: V, Ref: q, Or: G, Nullable: J }) => ({
    value: [J(_), null],
    placeholder: [H, ""],
    mode: [V("int", "float"), "int"],
    format: [J(G(H, q(TickFormatter))), null],
    low: [J(_), null],
    high: [J(_), null]
  }));
  let NumericInput = s4;
  const panes_css = ":host{position:absolute;top:100%;min-width:100%;width:auto;height:auto;user-select:none;-webkit-user-select:none;z-index:var(--bokeh-top-level);cursor:pointer;font-size:var(--font-size);background-color:#fff;border:1px solid #ccc;border-radius:var(--border-radius);box-shadow:2px 4px 8px rgba(0, 0, 0, 0.175);}";
  class DropPane {
    constructor(_, H) {
      mt(this, "contents");
      mt(this, "el", div());
      mt(this, "shadow_el");
      mt(this, "_open", !1);
      mt(this, "target");
      mt(this, "orientation");
      mt(this, "reversed");
      mt(this, "prevent_hide");
      mt(this, "extra_stylesheets");
      mt(this, "class_list");
      mt(this, "_on_mousedown", (_) => {
        if (_.composedPath().includes(this.el))
          return;
        const { prevent_hide: H } = this;
        if (H instanceof HTMLElement) {
          if (_.composedPath().includes(H))
            return;
        } else if (H != null && H(_))
          return;
        this.hide();
      });
      mt(this, "_on_keydown", (_) => {
        switch (_.key) {
          case "Escape": {
            this.hide();
            break;
          }
        }
      });
      mt(this, "_on_blur", () => {
        this.hide();
      });
      this.contents = _, this.target = H.target, this.prevent_hide = H.prevent_hide, this.extra_stylesheets = H.extra_stylesheets ?? [], this.shadow_el = this.el.attachShadow({ mode: "open" }), this.class_list = new ClassList(this.el.classList);
    }
    get is_open() {
      return this._open;
    }
    remove() {
      this._unlisten(), remove$2(this.el);
    }
    _listen() {
      document.addEventListener("mousedown", this._on_mousedown), document.addEventListener("keydown", this._on_keydown), window.addEventListener("blur", this._on_blur);
    }
    _unlisten() {
      document.removeEventListener("mousedown", this._on_mousedown), document.removeEventListener("keydown", this._on_keydown), window.removeEventListener("blur", this._on_blur);
    }
    stylesheets() {
      return [
        base_css,
        /*...super.stylesheets(), */
        panes_css,
        ...this.extra_stylesheets
      ];
    }
    empty() {
      empty$1(this.shadow_el), this.class_list.clear();
    }
    render() {
      this.empty();
      for (const _ of this.stylesheets())
        (isString$2(_) ? new InlineStyleSheet$1(_) : _).install(this.shadow_el);
      this.shadow_el.append(...this.contents);
    }
    show() {
      this._open || (this.render(), (this.target.shadowRoot ?? this.target).appendChild(this.el), this._listen(), this._open = !0);
    }
    hide() {
      this._open && (this._open = !1, this._unlisten(), remove$2(this.el));
    }
    toggle() {
      this._open ? this.hide() : this.show();
    }
  }
  mt(DropPane, "__name__", "DropPane");
  const value_input = "bk-value-input", value$3 = "bk-value", chevron = "bk-chevron", palette_select_css = ".bk-value-input{display:flex;flex-direction:row;flex-wrap:nowrap;align-items:center;gap:1em;cursor:pointer;}.bk-value{flex-grow:1;}.bk-chevron{width:16px;height:16px;mask-size:100% 100%;-webkit-mask-size:100% 100%;mask-position:center center;-webkit-mask-position:center center;mask-repeat:no-repeat;-webkit-mask-repeat:no-repeat;}", entry$2 = "bk-entry", item = "bk-item", palette_select_item_css = ".bk-entry{display:flex;flex-direction:row;flex-wrap:nowrap;align-items:center;gap:0.5em;}.bk-item{--active-tool-highlight:#26aae1;border:1px solid transparent;}.bk-item.bk-active{border-color:var(--active-tool-highlight);}.bk-item:hover{background-color:#f9f9f9;}.bk-item:focus,.bk-item:focus-visible{outline:1px dotted var(--active-tool-highlight);outline-offset:-1px;}.bk-item::-moz-focus-inner{border:0;}", palette_select_pane_css = ":host{--number-of-columns:1;padding:5px;display:grid;grid-template-columns:repeat(var(--number-of-columns), 1fr);gap:0.25em;}", Item$1 = Tuple(Str, Arrayable(Color$1));
  class PaletteSelectView extends InputWidgetView {
    constructor() {
      super(...arguments);
      mt(this, "_value_el");
      mt(this, "_pane");
    }
    stylesheets() {
      return [...super.stylesheets(), palette_select_css, palette_select_item_css, icons_css];
    }
    connect_signals() {
      super.connect_signals();
      const { value: H, items: V, ncols: q, swatch_width: G, swatch_height: J } = this.model.properties;
      this.on_change([V, G, J], () => this.render()), this.on_change(H, () => this._update_value()), this.on_change(q, () => this._update_ncols());
    }
    _update_value() {
      empty$1(this._value_el);
      const H = this._render_value();
      H != null && this._value_el.append(H);
    }
    _update_ncols() {
      const { ncols: H } = this.model;
      this._pane.el.style.setProperty("--number-of-columns", `${H}`);
    }
    _render_image(H) {
      const [V, q] = H, { swatch_width: G, swatch_height: J } = this.model, Z = canvas({ width: `${G}`, height: `${J}` }), te = Z.getContext("2d"), ee = 100 / q.length;
      for (const [ae, oe] of enumerate(q)) {
        te.beginPath(), te.rect(oe * ee, 0, ee, 20);
        const se = color2css(ae);
        te.strokeStyle = se, te.fillStyle = se, te.fill(), te.stroke();
      }
      return Z;
    }
    _render_item(H) {
      const [V, q] = H, { swatch_width: G, swatch_height: J } = this.model, Z = q.length, te = linspace(0, 100, Z + 1), ne = [];
      for (const [oe, se] of enumerate(q)) {
        const [ce, le] = [te[se], te[se + 1]];
        ne.push(`${color2css(oe)} ${ce}% ${le}%`);
      }
      const ee = div();
      return ee.style.background = `linear-gradient(to right, ${ne.join(", ")})`, ee.style.width = px$1(G), J == "auto" ? ee.style.alignSelf = "stretch" : ee.style.height = px$1(J), div({ class: entry$2 }, ee, V);
    }
    _render_value() {
      const { value: H, items: V } = this.model, q = V.find(([G]) => G == H);
      return q != null ? this._render_item(q) : null;
    }
    _render_input() {
      this._value_el = div({ class: [value$3, entry$2] }, this._render_value());
      const H = div({ class: [chevron, tool_icon_chevron_down] }), V = div({ class: [input, value_input] }, this._value_el, H);
      return this.model.disabled ? V.classList.add(disabled) : V.tabIndex = 0, this.input_el = V, this.input_el;
    }
    render() {
      super.render();
      const H = [];
      for (const [q, G] of enumerate(this.model.items)) {
        const J = this._render_item(q), Z = div({ class: item, tabIndex: 0 }, J);
        Z.addEventListener("pointerup", () => {
          this.select(q);
        }), Z.addEventListener("keyup", (ne) => {
          switch (ne.key) {
            case "Enter": {
              this.select(q);
              break;
            }
            case "Escape": {
              this.hide();
              break;
            }
          }
        });
        const te = (ne) => {
          const { items: ee } = this.model, ae = cycle(G + ne, 0, ee.length - 1);
          H[ae].focus();
        };
        Z.addEventListener("keydown", (ne) => {
          const ee = (() => {
            switch (ne.key) {
              case "ArrowUp":
                return -this.model.ncols;
              case "ArrowDown":
                return +this.model.ncols;
              case "ArrowLeft":
                return -1;
              case "ArrowRight":
                return 1;
              default:
                return null;
            }
          })();
          ee != null && (ne.preventDefault(), te(ee));
        }), H.push(Z);
      }
      this._pane = new DropPane(H, {
        target: this.group_el,
        prevent_hide: this.input_el,
        extra_stylesheets: [palette_select_item_css, palette_select_pane_css]
      }), this._update_ncols(), this.input_el.addEventListener("pointerup", () => {
        this.toggle();
      }), this.input_el.addEventListener("keyup", (q) => {
        switch (q.key) {
          case "Enter": {
            this.toggle();
            break;
          }
          case "Escape": {
            this.hide();
            break;
          }
        }
      });
      const V = (q) => {
        const { items: G, value: J } = this.model, Z = G.findIndex(([te]) => J == te);
        if (Z != -1) {
          const te = cycle(Z + q, 0, G.length - 1);
          this.select(G[te]);
        }
      };
      this.input_el.addEventListener("keydown", (q) => {
        const G = (() => {
          switch (q.key) {
            case "ArrowUp":
              return -1;
            case "ArrowDown":
              return 1;
            default:
              return null;
          }
        })();
        G != null && (q.preventDefault(), V(G));
      });
    }
    select(H) {
      this.hide();
      const [V] = H;
      this.model.value = V, super.change_input(), this.input_el.focus();
    }
    toggle() {
      this.model.disabled || this._pane.toggle();
    }
    hide() {
      this._pane.hide();
    }
  }
  mt(PaletteSelectView, "__name__", "PaletteSelectView");
  const l4 = class l4 extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(l4, "__name__", "PaletteSelect"), l4.prototype.default_view = PaletteSelectView, l4.define(({ Int: _, Str: H, List: V, NonNegative: q, Positive: G, Or: J, Auto: Z }) => ({
    value: [H],
    items: [V(Item$1)],
    swatch_width: [q(_), 100],
    swatch_height: [J(Z, q(_)), "auto"],
    ncols: [G(_), 1]
  }));
  let PaletteSelect = l4;
  class ParagraphView extends MarkupView {
    render() {
      super.render();
      const _ = p$1({ style: { margin: 0 } });
      this.has_math_disabled() ? _.textContent = this.model.text : _.innerHTML = this.process_tex(this.model.text), this.markup_el.appendChild(_);
    }
  }
  mt(ParagraphView, "__name__", "ParagraphView");
  const D6 = class D6 extends Markup {
    constructor(_) {
      super(_);
    }
  };
  mt(D6, "__name__", "Paragraph"), D6.prototype.default_view = ParagraphView;
  let Paragraph = D6;
  const password_input_css = ":host{--toggle-size:14px;--toggle-padding:4px;--toggle-width:calc(var(--toggle-size) + 2*var(--toggle-padding));}.bk-input{padding-right:max();}.bk-toggle{position:absolute;right:0;top:0;width:var(--toggle-width);height:100%;padding:0 var(--toggle-padding);background-color:var(--bokeh-icon-color);mask-image:var(--bokeh-icon-see-off);-webkit-mask-image:var(--bokeh-icon-see-off);mask-size:var(--toggle-size) var(--toggle-size);-webkit-mask-size:var(--toggle-size) var(--toggle-size);mask-position:center center;-webkit-mask-position:center center;mask-repeat:no-repeat;-webkit-mask-repeat:no-repeat;cursor:pointer;}.bk-toggle.bk-visible{mask-image:var(--bokeh-icon-see-on);-webkit-mask-image:var(--bokeh-icon-see-on);}";
  class PasswordInputView extends TextInputView {
    constructor() {
      super(...arguments);
      mt(this, "toggle_el");
    }
    stylesheets() {
      return [...super.stylesheets(), password_input_css, icons_css];
    }
    render() {
      super.render(), this.input_el.type = "password", this.toggle_el = div({ class: "bk-toggle" }), this.toggle_el.addEventListener("click", () => {
        const { input_el: H, toggle_el: V } = this, q = H.type == "text";
        V.classList.toggle("bk-visible", !q), H.type = q ? "password" : "text";
      }), this.shadow_el.append(this.toggle_el);
    }
  }
  mt(PasswordInputView, "__name__", "PasswordInputView");
  const N6 = class N6 extends TextInput {
    constructor(_) {
      super(_);
    }
  };
  mt(N6, "__name__", "PasswordInput"), N6.prototype.default_view = PasswordInputView;
  let PasswordInput = N6;
  class PreTextView extends MarkupView {
    render() {
      super.render();
      const _ = pre({ style: { overflow: "auto" } }, this.model.text);
      this.markup_el.appendChild(_);
    }
  }
  mt(PreTextView, "__name__", "PreTextView");
  const P6 = class P6 extends Markup {
    constructor(_) {
      super(_);
    }
  };
  mt(P6, "__name__", "PreText"), P6.prototype.default_view = PreTextView;
  let PreText = P6;
  class RadioButtonGroupView extends ToggleButtonGroupView {
    change_active(_) {
      this.model.active !== _ && (this.model.active = _);
    }
    _update_active() {
      const { active: _ } = this.model;
      this._buttons.forEach((H, V) => {
        H.classList.toggle(active$1, _ === V);
      });
    }
  }
  mt(RadioButtonGroupView, "__name__", "RadioButtonGroupView");
  const c4 = class c4 extends ToggleButtonGroup {
    constructor(_) {
      super(_);
    }
  };
  mt(c4, "__name__", "RadioButtonGroup"), c4.prototype.default_view = RadioButtonGroupView, c4.define(({ Int: _, Nullable: H }) => ({
    active: [H(_), null]
  }));
  let RadioButtonGroup = c4;
  class RadioGroupView extends ToggleInputGroupView {
    connect_signals() {
      super.connect_signals();
      const { active: _ } = this.model.properties;
      this.on_change(_, () => {
        const { active: H } = this.model;
        for (const [V, q] of enumerate(this._inputs))
          V.checked = H == q;
      });
    }
    render() {
      super.render();
      const _ = div({ class: [input_group, this.model.inline ? inline : null] });
      this.shadow_el.appendChild(_);
      const H = unique_id(), { active: V, labels: q } = this.model;
      this._inputs = [];
      for (let G = 0; G < q.length; G++) {
        const J = input$1({ type: "radio", name: H, value: `${G}` });
        J.addEventListener("change", () => this.change_active(G)), this._inputs.push(J), this.model.disabled && (J.disabled = !0), G == V && (J.checked = !0);
        const Z = label$2(J, span$1(q[G]));
        _.appendChild(Z);
      }
    }
    change_active(_) {
      this.model.active = _;
    }
  }
  mt(RadioGroupView, "__name__", "RadioGroupView");
  const u4 = class u4 extends ToggleInputGroup {
    constructor(_) {
      super(_);
    }
  };
  mt(u4, "__name__", "RadioGroup"), u4.prototype.default_view = RadioGroupView, u4.define(({ Int: _, Nullable: H }) => ({
    active: [H(_), null]
  }));
  let RadioGroup = u4;
  const Value$1 = Unknown, Label$2 = Str, Options$2 = List$2(Or$1(Label$2, Tuple(Value$1, Label$2))), OptionsGroups = Dict(Options$2), NotSelected = "";
  class SelectView extends InputWidgetView {
    constructor() {
      super(...arguments);
      mt(this, "_known_values", /* @__PURE__ */ new Map());
    }
    connect_signals() {
      super.connect_signals();
      const { value: H, options: V } = this.model.properties;
      this.on_change(H, () => {
        this._update_value();
      }), this.on_change(V, () => {
        empty$1(this.input_el), append$2(this.input_el, ...this.options_el()), this._update_value();
      });
    }
    options_el() {
      const { _known_values: H } = this;
      H.clear();
      function V(G) {
        return G.map((J) => {
          let Z, te;
          return isString$2(J) ? Z = te = J : [Z, te] = J, H.set(Z, te), option({ value: te }, te);
        });
      }
      const { options: q } = this.model;
      return isArray$3(q) ? V(q) : entries$2(q).map(([G, J]) => optgroup({ label: G }, V(J)));
    }
    _render_input() {
      return this.input_el = select$1({
        class: input,
        name: this.model.name,
        disabled: this.model.disabled
      }, this.options_el()), this.input_el.addEventListener("change", () => this.change_input()), this.input_el;
    }
    render() {
      super.render(), this._update_value();
    }
    change_input() {
      const H = this.input_el.value, V = [...this._known_values].find(([G, J]) => H == J), q = (() => {
        if (V == null)
          return NotSelected;
        {
          const [G, J] = V;
          return G;
        }
      })();
      this.model.value = q, super.change_input();
    }
    _update_value() {
      const { value: H } = this.model, V = this._known_values.get(H);
      V !== void 0 ? this.input_el.value = V : (this.input_el.removeAttribute("value"), this.input_el.selectedIndex = -1);
    }
  }
  mt(SelectView, "__name__", "SelectView");
  const d4 = class d4 extends InputWidget {
    constructor(_) {
      super(_);
    }
  };
  mt(d4, "__name__", "Select"), d4.prototype.default_view = SelectView, d4.define(() => ({
    value: [Value$1, NotSelected],
    options: [Or$1(Options$2, OptionsGroups), []]
  }));
  let Select = d4;
  const { min: min$4, max: max$4 } = Math;
  function debounce$3(O, _, H = !1) {
    let V;
    return function(...q) {
      const G = this, J = function() {
        V = void 0, H || O.apply(G, q);
      }, Z = H && V === void 0;
      V !== void 0 && clearTimeout(V), V = setTimeout(J, _), Z && O.apply(G, q);
    };
  }
  class SpinnerView extends NumericInputView {
    constructor() {
      super(...arguments);
      mt(this, "wrapper_el");
      mt(this, "btn_up_el");
      mt(this, "btn_down_el");
      mt(this, "_handles");
      mt(this, "_counter");
      mt(this, "_interval");
    }
    *buttons() {
      yield this.btn_up_el, yield this.btn_down_el;
    }
    initialize() {
      super.initialize(), this._handles = { interval: void 0, timeout: void 0 }, this._interval = 200;
    }
    connect_signals() {
      super.connect_signals();
      const H = this.model.properties;
      this.on_change(H.disabled, () => {
        for (const V of this.buttons())
          toggle_attribute(V, "disabled", this.model.disabled);
      });
    }
    _render_input() {
      super._render_input(), this.btn_up_el = button({ class: "bk-spin-btn bk-spin-btn-up" }), this.btn_down_el = button({ class: "bk-spin-btn bk-spin-btn-down" });
      const { input_el: H, btn_up_el: V, btn_down_el: q } = this;
      return this.wrapper_el = div({ class: "bk-spin-wrapper" }, H, V, q), this.wrapper_el;
    }
    render() {
      super.render();
      for (const H of this.buttons())
        toggle_attribute(H, "disabled", this.model.disabled), H.addEventListener("mousedown", (V) => this._btn_mouse_down(V)), H.addEventListener("mouseup", () => this._btn_mouse_up()), H.addEventListener("mouseleave", () => this._btn_mouse_leave());
      this.input_el.addEventListener("keydown", (H) => {
        this._input_key_down(H);
      }), this.input_el.addEventListener("keyup", () => {
        this.model.value_throttled = this.model.value;
      }), this.input_el.addEventListener("wheel", (H) => {
        this._input_mouse_wheel(H);
      }), this.input_el.addEventListener("wheel", debounce$3(() => {
        this.model.value_throttled = this.model.value;
      }, this.model.wheel_wait, !1));
    }
    remove() {
      this._stop_incrementation(), super.remove();
    }
    _start_incrementation(H) {
      clearInterval(this._handles.interval), this._counter = 0;
      const { step: V } = this.model, q = (G) => {
        if (this._counter += 1, this._counter % 5 == 0) {
          const J = Math.floor(this._counter / 5);
          J < 10 ? (clearInterval(this._handles.interval), this._handles.interval = setInterval(() => q(G), this._interval / (J + 1))) : J >= 10 && J <= 13 && (clearInterval(this._handles.interval), this._handles.interval = setInterval(() => q(G * 2), this._interval / 10));
        }
        this.increment(G);
      };
      this._handles.interval = setInterval(() => q(H * V), this._interval);
    }
    _stop_incrementation() {
      clearTimeout(this._handles.timeout), this._handles.timeout = void 0, clearInterval(this._handles.interval), this._handles.interval = void 0, this.model.value_throttled = this.model.value;
    }
    _btn_mouse_down(H) {
      H.preventDefault();
      const V = H.currentTarget === this.btn_up_el ? 1 : -1;
      this.increment(V * this.model.step), this.input_el.focus(), this._handles.timeout = setTimeout(() => this._start_incrementation(V), this._interval);
    }
    _btn_mouse_up() {
      this._stop_incrementation();
    }
    _btn_mouse_leave() {
      this._stop_incrementation();
    }
    _input_mouse_wheel(H) {
      if (document.activeElement === this.input_el) {
        H.preventDefault();
        const V = H.deltaY > 0 ? -1 : 1;
        this.increment(V * this.model.step);
      }
    }
    _input_key_down(H) {
      switch (H.key) {
        case "ArrowUp":
          return H.preventDefault(), this.increment(this.model.step);
        case "ArrowDown":
          return H.preventDefault(), this.increment(-this.model.step);
        case "PageUp":
          return H.preventDefault(), this.increment(this.model.page_step_multiplier * this.model.step);
        case "PageDown":
          return H.preventDefault(), this.increment(-this.model.page_step_multiplier * this.model.step);
      }
    }
    increment(H) {
      const { low: V, high: q } = this.model;
      this.model.value == null ? H > 0 ? this.model.value = V ?? (q != null ? min$4(0, q) : 0) : H < 0 && (this.model.value = q ?? (V != null ? max$4(V, 0) : 0)) : this.model.value = this.bound_value(this.model.value + H);
    }
    change_input() {
      super.change_input(), this.model.value_throttled = this.model.value;
    }
    bound_value(H) {
      const { low: V, high: q } = this.model;
      return V != null && H < V ? this.model.value ?? 0 : q != null && H > q ? this.model.value ?? 0 : H;
    }
  }
  mt(SpinnerView, "__name__", "SpinnerView");
  const Jp = class Jp extends NumericInput {
    constructor(_) {
      super(_);
    }
  };
  mt(Jp, "__name__", "Spinner"), Jp.prototype.default_view = SpinnerView, Jp.define(({ Float: _, Nullable: H }) => ({
    value_throttled: [H(_), unset, { readonly: !0 }],
    step: [_, 1],
    page_step_multiplier: [_, 10],
    wheel_wait: [_, 100]
  })), Jp.override({
    mode: "float"
  });
  let Spinner = Jp;
  const switch_css = ":host{cursor:pointer;}:host(.disabled){cursor:default;}:host{--switch-size:16px;--bar-height:10px;}.body{width:100%;height:var(--switch-size);}.bar{position:relative;top:calc(50% - var(--bar-height)/2);height:var(--bar-height);border-radius:calc(var(--bar-height)/2);background-color:#e5e5e5;transition-property:background-color;}.knob{position:absolute;top:0;left:0;width:var(--switch-size);height:var(--switch-size);border-radius:8px;background-color:#adadad;transition-property:left, background-color;}:host(.active) .bar{background-color:#c2d5f7;}:host(.active) .knob{left:calc(100% - var(--switch-size));background-color:#3b80f0;}";
  class SwitchView extends ToggleInputView {
    constructor() {
      super(...arguments);
      mt(this, "knob_el");
      mt(this, "bar_el");
    }
    stylesheets() {
      return [...super.stylesheets(), switch_css];
    }
    connect_signals() {
      super.connect_signals(), this.el.addEventListener("keydown", (H) => {
        switch (H.key) {
          case "Enter":
          case " ": {
            H.preventDefault(), this._toggle_active();
            break;
          }
        }
      }), this.el.addEventListener("click", () => this._toggle_active());
    }
    render() {
      super.render(), this.bar_el = div({ class: "bar" }), this.knob_el = div({ class: "knob", tabIndex: 0 });
      const H = div({ class: "body" }, this.bar_el, this.knob_el);
      this._update_active(), this._update_disabled(), this.shadow_el.appendChild(H);
    }
    _update_active() {
      this.el.classList.toggle("active", this.model.active);
    }
    _update_disabled() {
      this.el.classList.toggle("disabled", this.model.disabled);
    }
  }
  mt(SwitchView, "__name__", "SwitchView");
  const f4 = class f4 extends ToggleInput {
    constructor(_) {
      super(_);
    }
  };
  mt(f4, "__name__", "Switch"), f4.prototype.default_view = SwitchView, f4.override({
    width: 32
  });
  let Switch = f4;
  class TextAreaInputView extends TextLikeInputView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.rows.change, () => this.input_el.rows = this.model.rows), this.connect(this.model.properties.cols.change, () => this.input_el.cols = this.model.cols);
    }
    _render_input() {
      return this.input_el = textarea({ class: input });
    }
    render() {
      super.render(), this.input_el.cols = this.model.cols, this.input_el.rows = this.model.rows;
    }
  }
  mt(TextAreaInputView, "__name__", "TextAreaInputView");
  const e3 = class e3 extends TextLikeInput {
    constructor(_) {
      super(_);
    }
  };
  mt(e3, "__name__", "TextAreaInput"), e3.prototype.default_view = TextAreaInputView, e3.define(({ Int: _ }) => ({
    cols: [_, 20],
    rows: [_, 2]
  })), e3.override({
    max_length: 500
  });
  let TextAreaInput = e3;
  const TimeLike = Or$1(Str, Float$1);
  class TimePickerView extends PickerBaseView {
    _format_time(_) {
      const { picker: H } = this;
      return H.formatDate(_, H.config.dateFormat);
    }
    connect_signals() {
      super.connect_signals();
      const { value: _, min_time: H, max_time: V, time_format: q, hour_increment: G, minute_increment: J, second_increment: Z, seconds: te, clock: ne } = this.model.properties;
      this.connect(_.change, () => {
        const { value: ee } = this.model;
        ee != null ? this.picker.setDate(ee) : this.picker.clear();
      }), this.connect(H.change, () => this.picker.set("minTime", this.model.min_time)), this.connect(V.change, () => this.picker.set("maxTime", this.model.max_time)), this.connect(q.change, () => this.picker.set("altFormat", this.model.time_format)), this.connect(G.change, () => this.picker.set("hourIncrement", this.model.hour_increment)), this.connect(J.change, () => this.picker.set("minuteIncrement", this.model.minute_increment)), this.connect(Z.change, () => this._update_second_increment()), this.connect(te.change, () => this.picker.set("enableSeconds", this.model.seconds)), this.connect(ne.change, () => this.picker.set("time_24hr", this.model.clock == "24h"));
    }
    get flatpickr_options() {
      const { value: _, min_time: H, max_time: V, time_format: q, hour_increment: G, minute_increment: J, seconds: Z, clock: te } = this.model, ne = super.flatpickr_options;
      return ne.enableTime = !0, ne.noCalendar = !0, ne.altInput = !0, ne.altFormat = q, ne.dateFormat = "H:i:S", ne.hourIncrement = G, ne.minuteIncrement = J, ne.enableSeconds = Z, ne.time_24hr = te == "24h", _ != null && (ne.defaultDate = _), H != null && (ne.minTime = H), V != null && (ne.maxTime = V), ne;
    }
    render() {
      super.render(), this._update_second_increment();
    }
    _update_second_increment() {
      var H;
      const { second_increment: _ } = this.model;
      (H = this.picker.secondElement) == null || H.setAttribute("step", _.toString());
    }
    _on_change(_) {
      assert$2(_.length <= 1), this.model.value = (() => {
        if (_.length == 0)
          return null;
        {
          const [H] = _;
          return this._format_time(H);
        }
      })();
    }
  }
  mt(TimePickerView, "__name__", "TimePickerView");
  const h4 = class h4 extends PickerBase {
    constructor(_) {
      super(_);
    }
  };
  mt(h4, "__name__", "TimePicker"), h4.prototype.default_view = TimePickerView, h4.define(({ Bool: _, Str: H, Nullable: V, Positive: q, Int: G }) => ({
    value: [V(TimeLike), null],
    min_time: [V(TimeLike), null],
    max_time: [V(TimeLike), null],
    time_format: [H, "H:i"],
    hour_increment: [q(G), 1],
    minute_increment: [q(G), 1],
    second_increment: [q(G), 1],
    seconds: [_, !1],
    clock: [Clock, "24h"]
  }));
  let TimePicker = h4;
  class ToggleView extends AbstractButtonView {
    connect_signals() {
      super.connect_signals(), this.connect(this.model.properties.active.change, () => this._update_active());
    }
    render() {
      super.render(), this._update_active();
    }
    click() {
      this.model.active = !this.model.active, this.model.trigger_event(new ButtonClick()), super.click();
    }
    _update_active() {
      this.button_el.classList.toggle(active$1, this.model.active);
    }
  }
  mt(ToggleView, "__name__", "ToggleView");
  const n3 = class n3 extends AbstractButton {
    constructor(_) {
      super(_);
    }
  };
  mt(n3, "__name__", "Toggle"), n3.prototype.default_view = ToggleView, n3.define(({ Bool: _ }) => ({
    active: [_, !1]
  })), n3.override({
    label: "Toggle"
  });
  let Toggle = n3;
  var PipsMode;
  (function(O) {
    O.Range = "range", O.Steps = "steps", O.Positions = "positions", O.Count = "count", O.Values = "values";
  })(PipsMode || (PipsMode = {}));
  var PipsType;
  (function(O) {
    O[O.None = -1] = "None", O[O.NoValue = 0] = "NoValue", O[O.LargeValue = 1] = "LargeValue", O[O.SmallValue = 2] = "SmallValue";
  })(PipsType || (PipsType = {}));
  function isValidFormatter(O) {
    return isValidPartialFormatter(O) && typeof O.from == "function";
  }
  function isValidPartialFormatter(O) {
    return typeof O == "object" && typeof O.to == "function";
  }
  function removeElement(O) {
    O.parentElement.removeChild(O);
  }
  function isSet(O) {
    return O != null;
  }
  function preventDefault(O) {
    O.preventDefault();
  }
  function unique$1(O) {
    return O.filter(function(_) {
      return this[_] ? !1 : this[_] = !0;
    }, {});
  }
  function closest(O, _) {
    return Math.round(O / _) * _;
  }
  function offset$4(O, _) {
    var H = O.getBoundingClientRect(), V = O.ownerDocument, q = V.documentElement, G = getPageOffset(V);
    return /webkit.*Chrome.*Mobile/i.test(navigator.userAgent) && (G.x = 0), _ ? H.top + G.y - q.clientTop : H.left + G.x - q.clientLeft;
  }
  function isNumeric$1(O) {
    return typeof O == "number" && !isNaN(O) && isFinite(O);
  }
  function addClassFor(O, _, H) {
    H > 0 && (addClass(O, _), setTimeout(function() {
      removeClass(O, _);
    }, H));
  }
  function limit(O) {
    return Math.max(Math.min(O, 100), 0);
  }
  function asArray$2(O) {
    return Array.isArray(O) ? O : [O];
  }
  function countDecimals(O) {
    O = String(O);
    var _ = O.split(".");
    return _.length > 1 ? _[1].length : 0;
  }
  function addClass(O, _) {
    O.classList && !/\s/.test(_) ? O.classList.add(_) : O.className += " " + _;
  }
  function removeClass(O, _) {
    O.classList && !/\s/.test(_) ? O.classList.remove(_) : O.className = O.className.replace(new RegExp("(^|\\b)" + _.split(" ").join("|") + "(\\b|$)", "gi"), " ");
  }
  function hasClass(O, _) {
    return O.classList ? O.classList.contains(_) : new RegExp("\\b" + _ + "\\b").test(O.className);
  }
  function getPageOffset(O) {
    var _ = window.pageXOffset !== void 0, H = (O.compatMode || "") === "CSS1Compat", V = _ ? window.pageXOffset : H ? O.documentElement.scrollLeft : O.body.scrollLeft, q = _ ? window.pageYOffset : H ? O.documentElement.scrollTop : O.body.scrollTop;
    return {
      x: V,
      y: q
    };
  }
  function getActions() {
    return window.navigator.pointerEnabled ? {
      start: "pointerdown",
      move: "pointermove",
      end: "pointerup"
    } : window.navigator.msPointerEnabled ? {
      start: "MSPointerDown",
      move: "MSPointerMove",
      end: "MSPointerUp"
    } : {
      start: "mousedown touchstart",
      move: "mousemove touchmove",
      end: "mouseup touchend"
    };
  }
  function getSupportsPassive() {
    var O = !1;
    try {
      var _ = Object.defineProperty({}, "passive", {
        get: function() {
          O = !0;
        }
      });
      window.addEventListener("test", null, _);
    } catch {
    }
    return O;
  }
  function getSupportsTouchActionNone() {
    return window.CSS && CSS.supports && CSS.supports("touch-action", "none");
  }
  function subRangeRatio(O, _) {
    return 100 / (_ - O);
  }
  function fromPercentage(O, _, H) {
    return _ * 100 / (O[H + 1] - O[H]);
  }
  function toPercentage(O, _) {
    return fromPercentage(O, O[0] < 0 ? _ + Math.abs(O[0]) : _ - O[0], 0);
  }
  function isPercentage(O, _) {
    return _ * (O[1] - O[0]) / 100 + O[0];
  }
  function getJ(O, _) {
    for (var H = 1; O >= _[H]; )
      H += 1;
    return H;
  }
  function toStepping(O, _, H) {
    if (H >= O.slice(-1)[0])
      return 100;
    var V = getJ(H, O), q = O[V - 1], G = O[V], J = _[V - 1], Z = _[V];
    return J + toPercentage([q, G], H) / subRangeRatio(J, Z);
  }
  function fromStepping(O, _, H) {
    if (H >= 100)
      return O.slice(-1)[0];
    var V = getJ(H, _), q = O[V - 1], G = O[V], J = _[V - 1], Z = _[V];
    return isPercentage([q, G], (H - J) * subRangeRatio(J, Z));
  }
  function getStep(O, _, H, V) {
    if (V === 100)
      return V;
    var q = getJ(V, O), G = O[q - 1], J = O[q];
    return H ? V - G > (J - G) / 2 ? J : G : _[q - 1] ? O[q - 1] + closest(V - O[q - 1], _[q - 1]) : V;
  }
  var Spectrum = (
    /** @class */
    function() {
      function O(_, H, V) {
        this.xPct = [], this.xVal = [], this.xSteps = [], this.xNumSteps = [], this.xHighestCompleteStep = [], this.xSteps = [V || !1], this.xNumSteps = [!1], this.snap = H;
        var q, G = [];
        for (Object.keys(_).forEach(function(J) {
          G.push([asArray$2(_[J]), J]);
        }), G.sort(function(J, Z) {
          return J[0][0] - Z[0][0];
        }), q = 0; q < G.length; q++)
          this.handleEntryPoint(G[q][1], G[q][0]);
        for (this.xNumSteps = this.xSteps.slice(0), q = 0; q < this.xNumSteps.length; q++)
          this.handleStepPoint(q, this.xNumSteps[q]);
      }
      return O.prototype.getDistance = function(_) {
        for (var H = [], V = 0; V < this.xNumSteps.length - 1; V++)
          H[V] = fromPercentage(this.xVal, _, V);
        return H;
      }, O.prototype.getAbsoluteDistance = function(_, H, V) {
        var q = 0;
        if (_ < this.xPct[this.xPct.length - 1])
          for (; _ > this.xPct[q + 1]; )
            q++;
        else _ === this.xPct[this.xPct.length - 1] && (q = this.xPct.length - 2);
        !V && _ === this.xPct[q + 1] && q++, H === null && (H = []);
        var G, J = 1, Z = H[q], te = 0, ne = 0, ee = 0, ae = 0;
        for (V ? G = (_ - this.xPct[q]) / (this.xPct[q + 1] - this.xPct[q]) : G = (this.xPct[q + 1] - _) / (this.xPct[q + 1] - this.xPct[q]); Z > 0; )
          te = this.xPct[q + 1 + ae] - this.xPct[q + ae], H[q + ae] * J + 100 - G * 100 > 100 ? (ne = te * G, J = (Z - 100 * G) / H[q + ae], G = 1) : (ne = H[q + ae] * te / 100 * J, J = 0), V ? (ee = ee - ne, this.xPct.length + ae >= 1 && ae--) : (ee = ee + ne, this.xPct.length - ae >= 1 && ae++), Z = H[q + ae] * J;
        return _ + ee;
      }, O.prototype.toStepping = function(_) {
        return _ = toStepping(this.xVal, this.xPct, _), _;
      }, O.prototype.fromStepping = function(_) {
        return fromStepping(this.xVal, this.xPct, _);
      }, O.prototype.getStep = function(_) {
        return _ = getStep(this.xPct, this.xSteps, this.snap, _), _;
      }, O.prototype.getDefaultStep = function(_, H, V) {
        var q = getJ(_, this.xPct);
        return (_ === 100 || H && _ === this.xPct[q - 1]) && (q = Math.max(q - 1, 1)), (this.xVal[q] - this.xVal[q - 1]) / V;
      }, O.prototype.getNearbySteps = function(_) {
        var H = getJ(_, this.xPct);
        return {
          stepBefore: {
            startValue: this.xVal[H - 2],
            step: this.xNumSteps[H - 2],
            highestStep: this.xHighestCompleteStep[H - 2]
          },
          thisStep: {
            startValue: this.xVal[H - 1],
            step: this.xNumSteps[H - 1],
            highestStep: this.xHighestCompleteStep[H - 1]
          },
          stepAfter: {
            startValue: this.xVal[H],
            step: this.xNumSteps[H],
            highestStep: this.xHighestCompleteStep[H]
          }
        };
      }, O.prototype.countStepDecimals = function() {
        var _ = this.xNumSteps.map(countDecimals);
        return Math.max.apply(null, _);
      }, O.prototype.hasNoSize = function() {
        return this.xVal[0] === this.xVal[this.xVal.length - 1];
      }, O.prototype.convert = function(_) {
        return this.getStep(this.toStepping(_));
      }, O.prototype.handleEntryPoint = function(_, H) {
        var V;
        if (_ === "min" ? V = 0 : _ === "max" ? V = 100 : V = parseFloat(_), !isNumeric$1(V) || !isNumeric$1(H[0]))
          throw new Error("noUiSlider: 'range' value isn't numeric.");
        this.xPct.push(V), this.xVal.push(H[0]);
        var q = Number(H[1]);
        V ? this.xSteps.push(isNaN(q) ? !1 : q) : isNaN(q) || (this.xSteps[0] = q), this.xHighestCompleteStep.push(0);
      }, O.prototype.handleStepPoint = function(_, H) {
        if (H) {
          if (this.xVal[_] === this.xVal[_ + 1]) {
            this.xSteps[_] = this.xHighestCompleteStep[_] = this.xVal[_];
            return;
          }
          this.xSteps[_] = fromPercentage([this.xVal[_], this.xVal[_ + 1]], H, 0) / subRangeRatio(this.xPct[_], this.xPct[_ + 1]);
          var V = (this.xVal[_ + 1] - this.xVal[_]) / this.xNumSteps[_], q = Math.ceil(Number(V.toFixed(3)) - 1), G = this.xVal[_] + this.xNumSteps[_] * q;
          this.xHighestCompleteStep[_] = G;
        }
      }, O;
    }()
  ), defaultFormatter$1 = {
    to: function(O) {
      return O === void 0 ? "" : O.toFixed(2);
    },
    from: Number
  }, cssClasses = {
    target: "target",
    base: "base",
    origin: "origin",
    handle: "handle",
    handleLower: "handle-lower",
    handleUpper: "handle-upper",
    touchArea: "touch-area",
    horizontal: "horizontal",
    vertical: "vertical",
    background: "background",
    connect: "connect",
    connects: "connects",
    ltr: "ltr",
    rtl: "rtl",
    textDirectionLtr: "txt-dir-ltr",
    textDirectionRtl: "txt-dir-rtl",
    draggable: "draggable",
    drag: "state-drag",
    tap: "state-tap",
    active: "active",
    tooltip: "tooltip",
    pips: "pips",
    pipsHorizontal: "pips-horizontal",
    pipsVertical: "pips-vertical",
    marker: "marker",
    markerHorizontal: "marker-horizontal",
    markerVertical: "marker-vertical",
    markerNormal: "marker-normal",
    markerLarge: "marker-large",
    markerSub: "marker-sub",
    value: "value",
    valueHorizontal: "value-horizontal",
    valueVertical: "value-vertical",
    valueNormal: "value-normal",
    valueLarge: "value-large",
    valueSub: "value-sub"
  }, INTERNAL_EVENT_NS = {
    tooltips: ".__tooltips",
    aria: ".__aria"
  };
  function testStep(O, _) {
    if (!isNumeric$1(_))
      throw new Error("noUiSlider: 'step' is not numeric.");
    O.singleStep = _;
  }
  function testKeyboardPageMultiplier(O, _) {
    if (!isNumeric$1(_))
      throw new Error("noUiSlider: 'keyboardPageMultiplier' is not numeric.");
    O.keyboardPageMultiplier = _;
  }
  function testKeyboardMultiplier(O, _) {
    if (!isNumeric$1(_))
      throw new Error("noUiSlider: 'keyboardMultiplier' is not numeric.");
    O.keyboardMultiplier = _;
  }
  function testKeyboardDefaultStep(O, _) {
    if (!isNumeric$1(_))
      throw new Error("noUiSlider: 'keyboardDefaultStep' is not numeric.");
    O.keyboardDefaultStep = _;
  }
  function testRange(O, _) {
    if (typeof _ != "object" || Array.isArray(_))
      throw new Error("noUiSlider: 'range' is not an object.");
    if (_.min === void 0 || _.max === void 0)
      throw new Error("noUiSlider: Missing 'min' or 'max' in 'range'.");
    O.spectrum = new Spectrum(_, O.snap || !1, O.singleStep);
  }
  function testStart(O, _) {
    if (_ = asArray$2(_), !Array.isArray(_) || !_.length)
      throw new Error("noUiSlider: 'start' option is incorrect.");
    O.handles = _.length, O.start = _;
  }
  function testSnap(O, _) {
    if (typeof _ != "boolean")
      throw new Error("noUiSlider: 'snap' option must be a boolean.");
    O.snap = _;
  }
  function testAnimate(O, _) {
    if (typeof _ != "boolean")
      throw new Error("noUiSlider: 'animate' option must be a boolean.");
    O.animate = _;
  }
  function testAnimationDuration(O, _) {
    if (typeof _ != "number")
      throw new Error("noUiSlider: 'animationDuration' option must be a number.");
    O.animationDuration = _;
  }
  function testConnect(O, _) {
    var H = [!1], V;
    if (_ === "lower" ? _ = [!0, !1] : _ === "upper" && (_ = [!1, !0]), _ === !0 || _ === !1) {
      for (V = 1; V < O.handles; V++)
        H.push(_);
      H.push(!1);
    } else {
      if (!Array.isArray(_) || !_.length || _.length !== O.handles + 1)
        throw new Error("noUiSlider: 'connect' option doesn't match handle count.");
      H = _;
    }
    O.connect = H;
  }
  function testOrientation(O, _) {
    switch (_) {
      case "horizontal":
        O.ort = 0;
        break;
      case "vertical":
        O.ort = 1;
        break;
      default:
        throw new Error("noUiSlider: 'orientation' option is invalid.");
    }
  }
  function testMargin(O, _) {
    if (!isNumeric$1(_))
      throw new Error("noUiSlider: 'margin' option must be numeric.");
    _ !== 0 && (O.margin = O.spectrum.getDistance(_));
  }
  function testLimit(O, _) {
    if (!isNumeric$1(_))
      throw new Error("noUiSlider: 'limit' option must be numeric.");
    if (O.limit = O.spectrum.getDistance(_), !O.limit || O.handles < 2)
      throw new Error("noUiSlider: 'limit' option is only supported on linear sliders with 2 or more handles.");
  }
  function testPadding(O, _) {
    var H;
    if (!isNumeric$1(_) && !Array.isArray(_))
      throw new Error("noUiSlider: 'padding' option must be numeric or array of exactly 2 numbers.");
    if (Array.isArray(_) && !(_.length === 2 || isNumeric$1(_[0]) || isNumeric$1(_[1])))
      throw new Error("noUiSlider: 'padding' option must be numeric or array of exactly 2 numbers.");
    if (_ !== 0) {
      for (Array.isArray(_) || (_ = [_, _]), O.padding = [O.spectrum.getDistance(_[0]), O.spectrum.getDistance(_[1])], H = 0; H < O.spectrum.xNumSteps.length - 1; H++)
        if (O.padding[0][H] < 0 || O.padding[1][H] < 0)
          throw new Error("noUiSlider: 'padding' option must be a positive number(s).");
      var V = _[0] + _[1], q = O.spectrum.xVal[0], G = O.spectrum.xVal[O.spectrum.xVal.length - 1];
      if (V / (G - q) > 1)
        throw new Error("noUiSlider: 'padding' option must not exceed 100% of the range.");
    }
  }
  function testDirection(O, _) {
    switch (_) {
      case "ltr":
        O.dir = 0;
        break;
      case "rtl":
        O.dir = 1;
        break;
      default:
        throw new Error("noUiSlider: 'direction' option was not recognized.");
    }
  }
  function testBehaviour(O, _) {
    if (typeof _ != "string")
      throw new Error("noUiSlider: 'behaviour' must be a string containing options.");
    var H = _.indexOf("tap") >= 0, V = _.indexOf("drag") >= 0, q = _.indexOf("fixed") >= 0, G = _.indexOf("snap") >= 0, J = _.indexOf("hover") >= 0, Z = _.indexOf("unconstrained") >= 0, te = _.indexOf("invert-connects") >= 0, ne = _.indexOf("drag-all") >= 0, ee = _.indexOf("smooth-steps") >= 0;
    if (q) {
      if (O.handles !== 2)
        throw new Error("noUiSlider: 'fixed' behaviour must be used with 2 handles");
      testMargin(O, O.start[1] - O.start[0]);
    }
    if (te && O.handles !== 2)
      throw new Error("noUiSlider: 'invert-connects' behaviour must be used with 2 handles");
    if (Z && (O.margin || O.limit))
      throw new Error("noUiSlider: 'unconstrained' behaviour cannot be used with margin or limit");
    O.events = {
      tap: H || G,
      drag: V,
      dragAll: ne,
      smoothSteps: ee,
      fixed: q,
      snap: G,
      hover: J,
      unconstrained: Z,
      invertConnects: te
    };
  }
  function testTooltips(O, _) {
    if (_ !== !1)
      if (_ === !0 || isValidPartialFormatter(_)) {
        O.tooltips = [];
        for (var H = 0; H < O.handles; H++)
          O.tooltips.push(_);
      } else {
        if (_ = asArray$2(_), _.length !== O.handles)
          throw new Error("noUiSlider: must pass a formatter for all handles.");
        _.forEach(function(V) {
          if (typeof V != "boolean" && !isValidPartialFormatter(V))
            throw new Error("noUiSlider: 'tooltips' must be passed a formatter or 'false'.");
        }), O.tooltips = _;
      }
  }
  function testHandleAttributes(O, _) {
    if (_.length !== O.handles)
      throw new Error("noUiSlider: must pass a attributes for all handles.");
    O.handleAttributes = _;
  }
  function testAriaFormat(O, _) {
    if (!isValidPartialFormatter(_))
      throw new Error("noUiSlider: 'ariaFormat' requires 'to' method.");
    O.ariaFormat = _;
  }
  function testFormat(O, _) {
    if (!isValidFormatter(_))
      throw new Error("noUiSlider: 'format' requires 'to' and 'from' methods.");
    O.format = _;
  }
  function testKeyboardSupport(O, _) {
    if (typeof _ != "boolean")
      throw new Error("noUiSlider: 'keyboardSupport' option must be a boolean.");
    O.keyboardSupport = _;
  }
  function testDocumentElement(O, _) {
    O.documentElement = _;
  }
  function testCssPrefix(O, _) {
    if (typeof _ != "string" && _ !== !1)
      throw new Error("noUiSlider: 'cssPrefix' must be a string or `false`.");
    O.cssPrefix = _;
  }
  function testCssClasses(O, _) {
    if (typeof _ != "object")
      throw new Error("noUiSlider: 'cssClasses' must be an object.");
    typeof O.cssPrefix == "string" ? (O.cssClasses = {}, Object.keys(_).forEach(function(H) {
      O.cssClasses[H] = O.cssPrefix + _[H];
    })) : O.cssClasses = _;
  }
  function testOptions(O) {
    var _ = {
      margin: null,
      limit: null,
      padding: null,
      animate: !0,
      animationDuration: 300,
      ariaFormat: defaultFormatter$1,
      format: defaultFormatter$1
    }, H = {
      step: { r: !1, t: testStep },
      keyboardPageMultiplier: { r: !1, t: testKeyboardPageMultiplier },
      keyboardMultiplier: { r: !1, t: testKeyboardMultiplier },
      keyboardDefaultStep: { r: !1, t: testKeyboardDefaultStep },
      start: { r: !0, t: testStart },
      connect: { r: !0, t: testConnect },
      direction: { r: !0, t: testDirection },
      snap: { r: !1, t: testSnap },
      animate: { r: !1, t: testAnimate },
      animationDuration: { r: !1, t: testAnimationDuration },
      range: { r: !0, t: testRange },
      orientation: { r: !1, t: testOrientation },
      margin: { r: !1, t: testMargin },
      limit: { r: !1, t: testLimit },
      padding: { r: !1, t: testPadding },
      behaviour: { r: !0, t: testBehaviour },
      ariaFormat: { r: !1, t: testAriaFormat },
      format: { r: !1, t: testFormat },
      tooltips: { r: !1, t: testTooltips },
      keyboardSupport: { r: !0, t: testKeyboardSupport },
      documentElement: { r: !1, t: testDocumentElement },
      cssPrefix: { r: !0, t: testCssPrefix },
      cssClasses: { r: !0, t: testCssClasses },
      handleAttributes: { r: !1, t: testHandleAttributes }
    }, V = {
      connect: !1,
      direction: "ltr",
      behaviour: "tap",
      orientation: "horizontal",
      keyboardSupport: !0,
      cssPrefix: "noUi-",
      cssClasses,
      keyboardPageMultiplier: 5,
      keyboardMultiplier: 1,
      keyboardDefaultStep: 10
    };
    O.format && !O.ariaFormat && (O.ariaFormat = O.format), Object.keys(H).forEach(function(te) {
      if (!isSet(O[te]) && V[te] === void 0) {
        if (H[te].r)
          throw new Error("noUiSlider: '" + te + "' is required.");
        return;
      }
      H[te].t(_, isSet(O[te]) ? O[te] : V[te]);
    }), _.pips = O.pips;
    var q = document.createElement("div"), G = q.style.msTransform !== void 0, J = q.style.transform !== void 0;
    _.transformRule = J ? "transform" : G ? "msTransform" : "webkitTransform";
    var Z = [
      ["left", "top"],
      ["right", "bottom"]
    ];
    return _.style = Z[_.dir][_.ort], _;
  }
  function scope(O, _, H) {
    var V = getActions(), q = getSupportsTouchActionNone(), G = q && getSupportsPassive(), J = O, Z, te, ne, ee, ae, oe, se = _.spectrum, ce = [], le = [], ue = [], de = 0, fe = {}, he = !1, me = O.ownerDocument, pe = _.documentElement || me.documentElement, ye = me.body, ge = me.dir === "rtl" || _.ort === 1 ? 0 : 100;
    function ve(jt, cn) {
      var wn = me.createElement("div");
      return cn && addClass(wn, cn), jt.appendChild(wn), wn;
    }
    function be(jt, cn) {
      var wn = ve(jt, _.cssClasses.origin), Rn = ve(wn, _.cssClasses.handle);
      if (ve(Rn, _.cssClasses.touchArea), Rn.setAttribute("data-handle", String(cn)), _.keyboardSupport && (Rn.setAttribute("tabindex", "0"), Rn.addEventListener("keydown", function(Ie) {
        return bt(Ie, cn);
      })), _.handleAttributes !== void 0) {
        var Pn = _.handleAttributes[cn];
        Object.keys(Pn).forEach(function(Ie) {
          Rn.setAttribute(Ie, Pn[Ie]);
        });
      }
      return Rn.setAttribute("role", "slider"), Rn.setAttribute("aria-orientation", _.ort ? "vertical" : "horizontal"), cn === 0 ? addClass(Rn, _.cssClasses.handleLower) : cn === _.handles - 1 && addClass(Rn, _.cssClasses.handleUpper), wn.handle = Rn, wn;
    }
    function Ce(jt, cn) {
      return cn ? ve(jt, _.cssClasses.connect) : !1;
    }
    function Se(jt, cn) {
      te = ve(cn, _.cssClasses.connects), ne = [], ee = [], ee.push(Ce(te, jt[0]));
      for (var wn = 0; wn < _.handles; wn++)
        ne.push(be(cn, wn)), ue[wn] = wn, ee.push(Ce(te, jt[wn + 1]));
    }
    function we(jt) {
      addClass(jt, _.cssClasses.target), _.dir === 0 ? addClass(jt, _.cssClasses.ltr) : addClass(jt, _.cssClasses.rtl), _.ort === 0 ? addClass(jt, _.cssClasses.horizontal) : addClass(jt, _.cssClasses.vertical);
      var cn = getComputedStyle(jt).direction;
      return cn === "rtl" ? addClass(jt, _.cssClasses.textDirectionRtl) : addClass(jt, _.cssClasses.textDirectionLtr), ve(jt, _.cssClasses.base);
    }
    function Ee(jt, cn) {
      return !_.tooltips || !_.tooltips[cn] ? !1 : ve(jt.firstChild, _.cssClasses.tooltip);
    }
    function xe() {
      return J.hasAttribute("disabled");
    }
    function Le(jt) {
      var cn = ne[jt];
      return cn.hasAttribute("disabled");
    }
    function Qe(jt) {
      jt != null ? (ne[jt].setAttribute("disabled", ""), ne[jt].handle.removeAttribute("tabindex")) : (J.setAttribute("disabled", ""), ne.forEach(function(cn) {
        cn.handle.removeAttribute("tabindex");
      }));
    }
    function ke(jt) {
      jt != null ? (ne[jt].removeAttribute("disabled"), ne[jt].handle.setAttribute("tabindex", "0")) : (J.removeAttribute("disabled"), ne.forEach(function(cn) {
        cn.removeAttribute("disabled"), cn.handle.setAttribute("tabindex", "0");
      }));
    }
    function Me() {
      oe && (Rt("update" + INTERNAL_EVENT_NS.tooltips), oe.forEach(function(jt) {
        jt && removeElement(jt);
      }), oe = null);
    }
    function Re() {
      Me(), oe = ne.map(Ee), St("update" + INTERNAL_EVENT_NS.tooltips, function(jt, cn, wn) {
        if (!(!oe || !_.tooltips) && oe[cn] !== !1) {
          var Rn = jt[cn];
          _.tooltips[cn] !== !0 && (Rn = _.tooltips[cn].to(wn[cn])), oe[cn].innerHTML = Rn;
        }
      });
    }
    function De() {
      Rt("update" + INTERNAL_EVENT_NS.aria), St("update" + INTERNAL_EVENT_NS.aria, function(jt, cn, wn, Rn, Pn) {
        ue.forEach(function(Ie) {
          var rt = ne[Ie], ft = $t(le, Ie, 0, !0, !0, !0), pt = $t(le, Ie, 100, !0, !0, !0), wt = Pn[Ie], Bt = String(_.ariaFormat.to(wn[Ie]));
          ft = se.fromStepping(ft).toFixed(1), pt = se.fromStepping(pt).toFixed(1), wt = se.fromStepping(wt).toFixed(1), rt.children[0].setAttribute("aria-valuemin", ft), rt.children[0].setAttribute("aria-valuemax", pt), rt.children[0].setAttribute("aria-valuenow", wt), rt.children[0].setAttribute("aria-valuetext", Bt);
        });
      });
    }
    function Pe(jt) {
      if (jt.mode === PipsMode.Range || jt.mode === PipsMode.Steps)
        return se.xVal;
      if (jt.mode === PipsMode.Count) {
        if (jt.values < 2)
          throw new Error("noUiSlider: 'values' (>= 2) required for mode 'count'.");
        for (var cn = jt.values - 1, wn = 100 / cn, Rn = []; cn--; )
          Rn[cn] = cn * wn;
        return Rn.push(100), $e(Rn, jt.stepped);
      }
      return jt.mode === PipsMode.Positions ? $e(jt.values, jt.stepped) : jt.mode === PipsMode.Values ? jt.stepped ? jt.values.map(function(Pn) {
        return se.fromStepping(se.getStep(se.toStepping(Pn)));
      }) : jt.values : [];
    }
    function $e(jt, cn) {
      return jt.map(function(wn) {
        return se.fromStepping(cn ? se.getStep(wn) : wn);
      });
    }
    function qe(jt) {
      function cn(wt, Bt) {
        return Number((wt + Bt).toFixed(7));
      }
      var wn = Pe(jt), Rn = {}, Pn = se.xVal[0], Ie = se.xVal[se.xVal.length - 1], rt = !1, ft = !1, pt = 0;
      return wn = unique$1(wn.slice().sort(function(wt, Bt) {
        return wt - Bt;
      })), wn[0] !== Pn && (wn.unshift(Pn), rt = !0), wn[wn.length - 1] !== Ie && (wn.push(Ie), ft = !0), wn.forEach(function(wt, Bt) {
        var At, Dt, Ot, un = wt, yn = wn[Bt + 1], _n, gn, bn, an, Zt, Mn, Ln, In = jt.mode === PipsMode.Steps;
        for (In && (At = se.xNumSteps[Bt]), At || (At = yn - un), yn === void 0 && (yn = un), At = Math.max(At, 1e-7), Dt = un; Dt <= yn; Dt = cn(Dt, At)) {
          for (_n = se.toStepping(Dt), gn = _n - pt, Zt = gn / (jt.density || 1), Mn = Math.round(Zt), Ln = gn / Mn, Ot = 1; Ot <= Mn; Ot += 1)
            bn = pt + Ot * Ln, Rn[bn.toFixed(5)] = [se.fromStepping(bn), 0];
          an = wn.indexOf(Dt) > -1 ? PipsType.LargeValue : In ? PipsType.SmallValue : PipsType.NoValue, !Bt && rt && Dt !== yn && (an = 0), Dt === yn && ft || (Rn[_n.toFixed(5)] = [Dt, an]), pt = _n;
        }
      }), Rn;
    }
    function Xe(jt, cn, wn) {
      var Rn, Pn, Ie = me.createElement("div"), rt = (Rn = {}, Rn[PipsType.None] = "", Rn[PipsType.NoValue] = _.cssClasses.valueNormal, Rn[PipsType.LargeValue] = _.cssClasses.valueLarge, Rn[PipsType.SmallValue] = _.cssClasses.valueSub, Rn), ft = (Pn = {}, Pn[PipsType.None] = "", Pn[PipsType.NoValue] = _.cssClasses.markerNormal, Pn[PipsType.LargeValue] = _.cssClasses.markerLarge, Pn[PipsType.SmallValue] = _.cssClasses.markerSub, Pn), pt = [_.cssClasses.valueHorizontal, _.cssClasses.valueVertical], wt = [_.cssClasses.markerHorizontal, _.cssClasses.markerVertical];
      addClass(Ie, _.cssClasses.pips), addClass(Ie, _.ort === 0 ? _.cssClasses.pipsHorizontal : _.cssClasses.pipsVertical);
      function Bt(Dt, Ot) {
        var un = Ot === _.cssClasses.value, yn = un ? pt : wt, _n = un ? rt : ft;
        return Ot + " " + yn[_.ort] + " " + _n[Dt];
      }
      function At(Dt, Ot, un) {
        if (un = cn ? cn(Ot, un) : un, un !== PipsType.None) {
          var yn = ve(Ie, !1);
          yn.className = Bt(un, _.cssClasses.marker), yn.style[_.style] = Dt + "%", un > PipsType.NoValue && (yn = ve(Ie, !1), yn.className = Bt(un, _.cssClasses.value), yn.setAttribute("data-value", String(Ot)), yn.style[_.style] = Dt + "%", yn.innerHTML = String(wn.to(Ot)));
        }
      }
      return Object.keys(jt).forEach(function(Dt) {
        At(Dt, jt[Dt][0], jt[Dt][1]);
      }), Ie;
    }
    function Ve() {
      ae && (removeElement(ae), ae = null);
    }
    function Ze(jt) {
      Ve();
      var cn = qe(jt), wn = jt.filter, Rn = jt.format || {
        to: function(Pn) {
          return String(Math.round(Pn));
        }
      };
      return ae = J.appendChild(Xe(cn, wn, Rn)), ae;
    }
    function Ge() {
      var jt = Z.getBoundingClientRect(), cn = "offset" + ["Width", "Height"][_.ort];
      return _.ort === 0 ? jt.width || Z[cn] : jt.height || Z[cn];
    }
    function ze(jt, cn, wn, Rn) {
      var Pn = function(rt) {
        var ft = We(rt, Rn.pageOffset, Rn.target || cn);
        if (!ft || xe() && !Rn.doNotReject || hasClass(J, _.cssClasses.tap) && !Rn.doNotReject || jt === V.start && ft.buttons !== void 0 && ft.buttons > 1 || Rn.hover && ft.buttons)
          return !1;
        G || ft.preventDefault(), ft.calcPoint = ft.points[_.ort], wn(ft, Rn);
      }, Ie = [];
      return jt.split(" ").forEach(function(rt) {
        cn.addEventListener(rt, Pn, G ? { passive: !0 } : !1), Ie.push([rt, Pn]);
      }), Ie;
    }
    function We(jt, cn, wn) {
      var Rn = jt.type.indexOf("touch") === 0, Pn = jt.type.indexOf("mouse") === 0, Ie = jt.type.indexOf("pointer") === 0, rt = 0, ft = 0;
      if (jt.type.indexOf("MSPointer") === 0 && (Ie = !0), jt.type === "mousedown" && !jt.buttons && !jt.touches)
        return !1;
      if (Rn) {
        var pt = function(At) {
          var Dt = At.target;
          return Dt === wn || wn.contains(Dt) || jt.composed && jt.composedPath().shift() === wn;
        };
        if (jt.type === "touchstart") {
          var wt = Array.prototype.filter.call(jt.touches, pt);
          if (wt.length > 1)
            return !1;
          rt = wt[0].pageX, ft = wt[0].pageY;
        } else {
          var Bt = Array.prototype.find.call(jt.changedTouches, pt);
          if (!Bt)
            return !1;
          rt = Bt.pageX, ft = Bt.pageY;
        }
      }
      return cn = cn || getPageOffset(me), (Pn || Ie) && (rt = jt.clientX + cn.x, ft = jt.clientY + cn.y), jt.pageOffset = cn, jt.points = [rt, ft], jt.cursor = Pn || Ie, jt;
    }
    function at(jt) {
      var cn = jt - offset$4(Z, _.ort), wn = cn * 100 / Ge();
      return wn = limit(wn), _.dir ? 100 - wn : wn;
    }
    function ct(jt) {
      var cn = 100, wn = !1;
      return ne.forEach(function(Rn, Pn) {
        if (!Le(Pn)) {
          var Ie = le[Pn], rt = Math.abs(Ie - jt), ft = rt === 100 && cn === 100, pt = rt < cn, wt = rt <= cn && jt > Ie;
          (pt || wt || ft) && (wn = Pn, cn = rt);
        }
      }), wn;
    }
    function et(jt, cn) {
      jt.type === "mouseout" && jt.target.nodeName === "HTML" && jt.relatedTarget === null && Je(jt, cn);
    }
    function dt(jt, cn) {
      if (navigator.appVersion.indexOf("MSIE 9") === -1 && jt.buttons === 0 && cn.buttonsProperty !== 0)
        return Je(jt, cn);
      var wn = (_.dir ? -1 : 1) * (jt.calcPoint - cn.startCalcPoint), Rn = wn * 100 / cn.baseSize;
      Xt(wn > 0, Rn, cn.locations, cn.handleNumbers, cn.connect);
    }
    function Je(jt, cn) {
      cn.handle && (removeClass(cn.handle, _.cssClasses.active), de -= 1), cn.listeners.forEach(function(wn) {
        pe.removeEventListener(wn[0], wn[1]);
      }), de === 0 && (removeClass(J, _.cssClasses.drag), Ft(), jt.cursor && (ye.style.cursor = "", ye.removeEventListener("selectstart", preventDefault))), _.events.smoothSteps && (cn.handleNumbers.forEach(function(wn) {
        Tt(wn, le[wn], !0, !0, !1, !1);
      }), cn.handleNumbers.forEach(function(wn) {
        vt("update", wn);
      })), cn.handleNumbers.forEach(function(wn) {
        vt("change", wn), vt("set", wn), vt("end", wn);
      });
    }
    function it(jt, cn) {
      if (!cn.handleNumbers.some(Le)) {
        var wn;
        if (cn.handleNumbers.length === 1) {
          var Rn = ne[cn.handleNumbers[0]];
          wn = Rn.children[0], de += 1, addClass(wn, _.cssClasses.active);
        }
        jt.stopPropagation();
        var Pn = [], Ie = ze(V.move, pe, dt, {
          // The event target has changed so we need to propagate the original one so that we keep
          // relying on it to extract target touches.
          target: jt.target,
          handle: wn,
          connect: cn.connect,
          listeners: Pn,
          startCalcPoint: jt.calcPoint,
          baseSize: Ge(),
          pageOffset: jt.pageOffset,
          handleNumbers: cn.handleNumbers,
          buttonsProperty: jt.buttons,
          locations: le.slice()
        }), rt = ze(V.end, pe, Je, {
          target: jt.target,
          handle: wn,
          listeners: Pn,
          doNotReject: !0,
          handleNumbers: cn.handleNumbers
        }), ft = ze("mouseout", pe, et, {
          target: jt.target,
          handle: wn,
          listeners: Pn,
          doNotReject: !0,
          handleNumbers: cn.handleNumbers
        });
        Pn.push.apply(Pn, Ie.concat(rt, ft)), jt.cursor && (ye.style.cursor = getComputedStyle(jt.target).cursor, ne.length > 1 && addClass(J, _.cssClasses.drag), ye.addEventListener("selectstart", preventDefault, !1)), cn.handleNumbers.forEach(function(pt) {
          vt("start", pt);
        });
      }
    }
    function st(jt) {
      jt.stopPropagation();
      var cn = at(jt.calcPoint), wn = ct(cn);
      wn !== !1 && (_.events.snap || addClassFor(J, _.cssClasses.tap, _.animationDuration), Tt(wn, cn, !0, !0), Ft(), vt("slide", wn, !0), vt("update", wn, !0), _.events.snap ? it(jt, { handleNumbers: [wn] }) : (vt("change", wn, !0), vt("set", wn, !0)));
    }
    function gt(jt) {
      var cn = at(jt.calcPoint), wn = se.getStep(cn), Rn = se.fromStepping(wn);
      Object.keys(fe).forEach(function(Pn) {
        Pn.split(".")[0] === "hover" && fe[Pn].forEach(function(Ie) {
          Ie.call(Ht, Rn);
        });
      });
    }
    function bt(jt, cn) {
      if (xe() || Le(cn))
        return !1;
      var wn = ["Left", "Right"], Rn = ["Down", "Up"], Pn = ["PageDown", "PageUp"], Ie = ["Home", "End"];
      _.dir && !_.ort ? wn.reverse() : _.ort && !_.dir && (Rn.reverse(), Pn.reverse());
      var rt = jt.key.replace("Arrow", ""), ft = rt === Pn[0], pt = rt === Pn[1], wt = rt === Rn[0] || rt === wn[0] || ft, Bt = rt === Rn[1] || rt === wn[1] || pt, At = rt === Ie[0], Dt = rt === Ie[1];
      if (!wt && !Bt && !At && !Dt)
        return !0;
      jt.preventDefault();
      var Ot;
      if (Bt || wt) {
        var un = wt ? 0 : 1, yn = Qt(cn), _n = yn[un];
        if (_n === null)
          return !1;
        _n === !1 && (_n = se.getDefaultStep(le[cn], wt, _.keyboardDefaultStep)), pt || ft ? _n *= _.keyboardPageMultiplier : _n *= _.keyboardMultiplier, _n = Math.max(_n, 1e-7), _n = (wt ? -1 : 1) * _n, Ot = ce[cn] + _n;
      } else Dt ? Ot = _.spectrum.xVal[_.spectrum.xVal.length - 1] : Ot = _.spectrum.xVal[0];
      return Tt(cn, se.toStepping(Ot), !0, !0), vt("slide", cn), vt("update", cn), vt("change", cn), vt("set", cn), !1;
    }
    function yt(jt) {
      jt.fixed || ne.forEach(function(cn, wn) {
        ze(V.start, cn.children[0], it, {
          handleNumbers: [wn]
        });
      }), jt.tap && ze(V.start, Z, st, {}), jt.hover && ze(V.move, Z, gt, {
        hover: !0
      }), jt.drag && ee.forEach(function(cn, wn) {
        if (!(cn === !1 || wn === 0 || wn === ee.length - 1)) {
          var Rn = ne[wn - 1], Pn = ne[wn], Ie = [cn], rt = [Rn, Pn], ft = [wn - 1, wn];
          addClass(cn, _.cssClasses.draggable), jt.fixed && (Ie.push(Rn.children[0]), Ie.push(Pn.children[0])), jt.dragAll && (rt = ne, ft = ue), Ie.forEach(function(pt) {
            ze(V.start, pt, it, {
              handles: rt,
              handleNumbers: ft,
              connect: cn
            });
          });
        }
      });
    }
    function St(jt, cn) {
      fe[jt] = fe[jt] || [], fe[jt].push(cn), jt.split(".")[0] === "update" && ne.forEach(function(wn, Rn) {
        vt("update", Rn);
      });
    }
    function Lt(jt) {
      return jt === INTERNAL_EVENT_NS.aria || jt === INTERNAL_EVENT_NS.tooltips;
    }
    function Rt(jt) {
      var cn = jt && jt.split(".")[0], wn = cn ? jt.substring(cn.length) : jt;
      Object.keys(fe).forEach(function(Rn) {
        var Pn = Rn.split(".")[0], Ie = Rn.substring(Pn.length);
        (!cn || cn === Pn) && (!wn || wn === Ie) && (!Lt(Ie) || wn === Ie) && delete fe[Rn];
      });
    }
    function vt(jt, cn, wn) {
      Object.keys(fe).forEach(function(Rn) {
        var Pn = Rn.split(".")[0];
        jt === Pn && fe[Rn].forEach(function(Ie) {
          Ie.call(
            // Use the slider public API as the scope ('this')
            Ht,
            // Return values as array, so arg_1[arg_2] is always valid.
            ce.map(_.format.to),
            // Handle index, 0 or 1
            cn,
            // Un-formatted slider values
            ce.slice(),
            // Event is fired by tap, true or false
            wn || !1,
            // Left offset of the handle, in relation to the slider
            le.slice(),
            // add the slider public API to an accessible parameter when this is unavailable
            Ht
          );
        });
      });
    }
    function $t(jt, cn, wn, Rn, Pn, Ie, rt) {
      var ft;
      return ne.length > 1 && !_.events.unconstrained && (Rn && cn > 0 && (ft = se.getAbsoluteDistance(jt[cn - 1], _.margin, !1), wn = Math.max(wn, ft)), Pn && cn < ne.length - 1 && (ft = se.getAbsoluteDistance(jt[cn + 1], _.margin, !0), wn = Math.min(wn, ft))), ne.length > 1 && _.limit && (Rn && cn > 0 && (ft = se.getAbsoluteDistance(jt[cn - 1], _.limit, !1), wn = Math.min(wn, ft)), Pn && cn < ne.length - 1 && (ft = se.getAbsoluteDistance(jt[cn + 1], _.limit, !0), wn = Math.max(wn, ft))), _.padding && (cn === 0 && (ft = se.getAbsoluteDistance(0, _.padding[0], !1), wn = Math.max(wn, ft)), cn === ne.length - 1 && (ft = se.getAbsoluteDistance(100, _.padding[1], !0), wn = Math.min(wn, ft))), rt || (wn = se.getStep(wn)), wn = limit(wn), wn === jt[cn] && !Ie ? !1 : wn;
    }
    function Pt(jt, cn) {
      var wn = _.ort;
      return (wn ? cn : jt) + ", " + (wn ? jt : cn);
    }
    function Xt(jt, cn, wn, Rn, Pn) {
      var Ie = wn.slice(), rt = Rn[0], ft = _.events.smoothSteps, pt = [!jt, jt], wt = [jt, !jt];
      Rn = Rn.slice(), jt && Rn.reverse(), Rn.length > 1 ? Rn.forEach(function(At, Dt) {
        var Ot = $t(Ie, At, Ie[At] + cn, pt[Dt], wt[Dt], !1, ft);
        Ot === !1 ? cn = 0 : (cn = Ot - Ie[At], Ie[At] = Ot);
      }) : pt = wt = [!0];
      var Bt = !1;
      Rn.forEach(function(At, Dt) {
        Bt = Tt(At, wn[At] + cn, pt[Dt], wt[Dt], !1, ft) || Bt;
      }), Bt && (Rn.forEach(function(At) {
        vt("update", At), vt("slide", At);
      }), Pn != null && vt("drag", rt));
    }
    function Nt(jt, cn) {
      return _.dir ? 100 - jt - cn : jt;
    }
    function Gt(jt, cn) {
      le[jt] = cn, ce[jt] = se.fromStepping(cn);
      var wn = Nt(cn, 0) - ge, Rn = "translate(" + Pt(wn + "%", "0") + ")";
      if (ne[jt].style[_.transformRule] = Rn, _.events.invertConnects && le.length > 1) {
        var Pn = le.every(function(Ie, rt, ft) {
          return rt === 0 || Ie >= ft[rt - 1];
        });
        if (he !== !Pn) {
          vn();
          return;
        }
      }
      It(jt), It(jt + 1), he && (It(jt - 1), It(jt + 2));
    }
    function Ft() {
      ue.forEach(function(jt) {
        var cn = le[jt] > 50 ? -1 : 1, wn = 3 + (ne.length + cn * jt);
        ne[jt].style.zIndex = String(wn);
      });
    }
    function Tt(jt, cn, wn, Rn, Pn, Ie) {
      return Pn || (cn = $t(le, jt, cn, wn, Rn, !1, Ie)), cn === !1 ? !1 : (Gt(jt, cn), !0);
    }
    function It(jt) {
      if (ee[jt]) {
        var cn = le.slice();
        he && cn.sort(function(ft, pt) {
          return ft - pt;
        });
        var wn = 0, Rn = 100;
        jt !== 0 && (wn = cn[jt - 1]), jt !== ee.length - 1 && (Rn = cn[jt]);
        var Pn = Rn - wn, Ie = "translate(" + Pt(Nt(wn, Pn) + "%", "0") + ")", rt = "scale(" + Pt(Pn / 100, "1") + ")";
        ee[jt].style[_.transformRule] = Ie + " " + rt;
      }
    }
    function Ct(jt, cn) {
      return jt === null || jt === !1 || jt === void 0 || (typeof jt == "number" && (jt = String(jt)), jt = _.format.from(jt), jt !== !1 && (jt = se.toStepping(jt)), jt === !1 || isNaN(jt)) ? le[cn] : jt;
    }
    function Kt(jt, cn, wn) {
      var Rn = asArray$2(jt), Pn = le[0] === void 0;
      cn = cn === void 0 ? !0 : cn, _.animate && !Pn && addClassFor(J, _.cssClasses.tap, _.animationDuration), ue.forEach(function(ft) {
        Tt(ft, Ct(Rn[ft], ft), !0, !1, wn);
      });
      var Ie = ue.length === 1 ? 0 : 1;
      if (Pn && se.hasNoSize() && (wn = !0, le[0] = 0, ue.length > 1)) {
        var rt = 100 / (ue.length - 1);
        ue.forEach(function(ft) {
          le[ft] = ft * rt;
        });
      }
      for (; Ie < ue.length; ++Ie)
        ue.forEach(function(ft) {
          Tt(ft, le[ft], !0, !0, wn);
        });
      Ft(), ue.forEach(function(ft) {
        vt("update", ft), Rn[ft] !== null && cn && vt("set", ft);
      });
    }
    function Vt(jt) {
      Kt(_.start, jt);
    }
    function Yt(jt, cn, wn, Rn) {
      if (jt = Number(jt), !(jt >= 0 && jt < ue.length))
        throw new Error("noUiSlider: invalid handle number, got: " + jt);
      Tt(jt, Ct(cn, jt), !0, !0, Rn), vt("update", jt), wn && vt("set", jt);
    }
    function sn(jt) {
      if (jt === void 0 && (jt = !1), jt)
        return ce.length === 1 ? ce[0] : ce.slice(0);
      var cn = ce.map(_.format.to);
      return cn.length === 1 ? cn[0] : cn;
    }
    function hn() {
      for (Rt(INTERNAL_EVENT_NS.aria), Rt(INTERNAL_EVENT_NS.tooltips), Object.keys(_.cssClasses).forEach(function(jt) {
        removeClass(J, _.cssClasses[jt]);
      }); J.firstChild; )
        J.removeChild(J.firstChild);
      delete J.noUiSlider;
    }
    function Qt(jt) {
      var cn = le[jt], wn = se.getNearbySteps(cn), Rn = ce[jt], Pn = wn.thisStep.step, Ie = null;
      if (_.snap)
        return [
          Rn - wn.stepBefore.startValue || null,
          wn.stepAfter.startValue - Rn || null
        ];
      Pn !== !1 && Rn + Pn > wn.stepAfter.startValue && (Pn = wn.stepAfter.startValue - Rn), Rn > wn.thisStep.startValue ? Ie = wn.thisStep.step : wn.stepBefore.step === !1 ? Ie = !1 : Ie = Rn - wn.stepBefore.highestStep, cn === 100 ? Pn = null : cn === 0 && (Ie = null);
      var rt = se.countStepDecimals();
      return Pn !== null && Pn !== !1 && (Pn = Number(Pn.toFixed(rt))), Ie !== null && Ie !== !1 && (Ie = Number(Ie.toFixed(rt))), [Ie, Pn];
    }
    function Ut() {
      return ue.map(Qt);
    }
    function Wt(jt, cn) {
      var wn = sn(), Rn = [
        "margin",
        "limit",
        "padding",
        "range",
        "animate",
        "snap",
        "step",
        "format",
        "pips",
        "tooltips",
        "connect"
      ];
      Rn.forEach(function(Ie) {
        jt[Ie] !== void 0 && (H[Ie] = jt[Ie]);
      });
      var Pn = testOptions(H);
      Rn.forEach(function(Ie) {
        jt[Ie] !== void 0 && (_[Ie] = Pn[Ie]);
      }), se = Pn.spectrum, _.margin = Pn.margin, _.limit = Pn.limit, _.padding = Pn.padding, _.pips ? Ze(_.pips) : Ve(), _.tooltips ? Re() : Me(), le = [], Kt(isSet(jt.start) ? jt.start : wn, cn), jt.connect && rn();
    }
    function rn() {
      for (; te.firstChild; )
        te.removeChild(te.firstChild);
      for (var jt = 0; jt <= _.handles; jt++)
        ee[jt] = Ce(te, _.connect[jt]), It(jt);
      yt({ drag: _.events.drag, fixed: !0 });
    }
    function vn() {
      he = !he, testConnect(
        _,
        // inverse the connect boolean array
        _.connect.map(function(jt) {
          return !jt;
        })
      ), rn();
    }
    function dn() {
      Z = we(J), Se(_.connect, Z), yt(_.events), Kt(_.start), _.pips && Ze(_.pips), _.tooltips && Re(), De();
    }
    dn();
    var Ht = {
      destroy: hn,
      steps: Ut,
      on: St,
      off: Rt,
      get: sn,
      set: Kt,
      setHandle: Yt,
      reset: Vt,
      disable: Qe,
      enable: ke,
      // Exposed for unit testing, don't use this in your application.
      __moveHandles: function(jt, cn, wn) {
        Xt(jt, cn, le, wn);
      },
      options: H,
      updateOptions: Wt,
      target: J,
      removePips: Ve,
      removeTooltips: Me,
      getPositions: function() {
        return le.slice();
      },
      getTooltips: function() {
        return oe;
      },
      getOrigins: function() {
        return ne;
      },
      pips: Ze
      // Issue #594
    };
    return Ht;
  }
  function initialize$2(O, _) {
    if (!O || !O.nodeName)
      throw new Error("noUiSlider: create requires a single element, got: " + O);
    if (O.noUiSlider)
      throw new Error("noUiSlider: Slider was already initialized.");
    var H = testOptions(_), V = scope(O, H, _);
    return O.noUiSlider = V, V;
  }
  const noUiSlider = {
    // Exposed for unit testing, don't use this in your application.
    __spectrum: Spectrum,
    // A reference to the default classes, allows global changes.
    // Use the cssClasses option for changes to one slider.
    cssClasses,
    create: initialize$2
  }, slider_title = "bk-slider-title", slider_value = "bk-slider-value", sliders_css = ".bk-slider-title{white-space:nowrap;}.bk-slider-value{font-weight:600;}", nouislider_css = '.noUi-target,.noUi-target *{-webkit-touch-callout:none;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);-webkit-user-select:none;-ms-touch-action:none;touch-action:none;-ms-user-select:none;-moz-user-select:none;user-select:none;-moz-box-sizing:border-box;box-sizing:border-box;}.noUi-target{position:relative;}.noUi-base,.noUi-connects{width:100%;height:100%;position:relative;z-index:1;}.noUi-connects{overflow:hidden;z-index:0;}.noUi-connect,.noUi-origin{will-change:transform;position:absolute;z-index:1;top:0;right:0;height:100%;width:100%;-ms-transform-origin:0 0;-webkit-transform-origin:0 0;-webkit-transform-style:preserve-3d;transform-origin:0 0;transform-style:flat;}.noUi-txt-dir-rtl.noUi-horizontal .noUi-origin{left:0;right:auto;}.noUi-vertical .noUi-origin{top:-100%;width:0;}.noUi-horizontal .noUi-origin{height:0;}.noUi-handle{-webkit-backface-visibility:hidden;backface-visibility:hidden;position:absolute;}.noUi-touch-area{height:100%;width:100%;}.noUi-state-tap .noUi-connect,.noUi-state-tap .noUi-origin{-webkit-transition:transform 0.3s;transition:transform 0.3s;}.noUi-state-drag *{cursor:inherit !important;}.noUi-horizontal{height:18px;}.noUi-horizontal .noUi-handle{width:34px;height:28px;right:-17px;top:-6px;}.noUi-vertical{width:18px;}.noUi-vertical .noUi-handle{width:28px;height:34px;right:-6px;bottom:-17px;}.noUi-txt-dir-rtl.noUi-horizontal .noUi-handle{left:-17px;right:auto;}.noUi-target{background:#FAFAFA;border-radius:4px;border:1px solid #D3D3D3;box-shadow:inset 0 1px 1px #F0F0F0, 0 3px 6px -5px #BBB;}.noUi-connects{border-radius:3px;}.noUi-connect{background:#3FB8AF;}.noUi-draggable{cursor:ew-resize;}.noUi-vertical .noUi-draggable{cursor:ns-resize;}.noUi-handle{border:1px solid #D9D9D9;border-radius:3px;background:#FFF;cursor:default;box-shadow:inset 0 0 1px #FFF, inset 0 1px 7px #EBEBEB, 0 3px 6px -3px #BBB;}.noUi-active{box-shadow:inset 0 0 1px #FFF, inset 0 1px 7px #DDD, 0 3px 6px -3px #BBB;}.noUi-handle:before,.noUi-handle:after{content:"";display:block;position:absolute;height:14px;width:1px;background:#E8E7E6;left:14px;top:6px;}.noUi-handle:after{left:17px;}.noUi-vertical .noUi-handle:before,.noUi-vertical .noUi-handle:after{width:14px;height:1px;left:6px;top:14px;}.noUi-vertical .noUi-handle:after{top:17px;}[disabled] .noUi-connect{background:#B8B8B8;}[disabled].noUi-target,[disabled].noUi-handle,[disabled] .noUi-handle{cursor:not-allowed;}.noUi-pips,.noUi-pips *{-moz-box-sizing:border-box;box-sizing:border-box;}.noUi-pips{position:absolute;color:#999;}.noUi-value{position:absolute;white-space:nowrap;text-align:center;}.noUi-value-sub{color:#ccc;font-size:10px;}.noUi-marker{position:absolute;background:#CCC;}.noUi-marker-sub{background:#AAA;}.noUi-marker-large{background:#AAA;}.noUi-pips-horizontal{padding:10px 0;height:80px;top:100%;left:0;width:100%;}.noUi-value-horizontal{-webkit-transform:translate(-50%, 50%);transform:translate(-50%, 50%);}.noUi-rtl .noUi-value-horizontal{-webkit-transform:translate(50%, 50%);transform:translate(50%, 50%);}.noUi-marker-horizontal.noUi-marker{margin-left:-1px;width:2px;height:5px;}.noUi-marker-horizontal.noUi-marker-sub{height:10px;}.noUi-marker-horizontal.noUi-marker-large{height:15px;}.noUi-pips-vertical{padding:0 10px;height:100%;top:0;left:100%;}.noUi-value-vertical{-webkit-transform:translate(0, -50%);transform:translate(0, -50%);padding-left:25px;}.noUi-rtl .noUi-value-vertical{-webkit-transform:translate(0, 50%);transform:translate(0, 50%);}.noUi-marker-vertical.noUi-marker{width:5px;height:2px;margin-top:-1px;}.noUi-marker-vertical.noUi-marker-sub{width:10px;}.noUi-marker-vertical.noUi-marker-large{width:15px;}.noUi-tooltip{display:block;position:absolute;border:1px solid #D9D9D9;border-radius:3px;background:#fff;color:#000;padding:5px;text-align:center;white-space:nowrap;}.noUi-horizontal .noUi-tooltip{-webkit-transform:translate(-50%, 0);transform:translate(-50%, 0);left:50%;bottom:120%;}.noUi-vertical .noUi-tooltip{-webkit-transform:translate(0, -50%);transform:translate(0, -50%);top:50%;right:120%;}.noUi-horizontal .noUi-origin > .noUi-tooltip{-webkit-transform:translate(50%, 0);transform:translate(50%, 0);left:auto;bottom:10px;}.noUi-vertical .noUi-origin > .noUi-tooltip{-webkit-transform:translate(0, -18px);transform:translate(0, -18px);top:auto;right:28px;}.noUi-handle{cursor:grab;cursor:-webkit-grab;}.noUi-handle.noUi-active{cursor:grabbing;cursor:-webkit-grabbing;}.noUi-handle:after,.noUi-handle:before{display:none;}.noUi-tooltip{display:none;white-space:nowrap;}.noUi-handle:hover .noUi-tooltip{display:block;}:host{--slider-size:10px;--handle-width:14px;--handle-height:18px;--handle-right:calc(-1*var(--handle-width)/2);--handle-top:calc(-1*(var(--handle-height) - var(--slider-size))/2 - 1px);--slider-margin:calc((var(--handle-height) - var(--slider-size))/2 + 1px);}.noUi-horizontal{width:100%;height:var(--slider-size);}.noUi-vertical{width:var(--slider-size);height:100%;}.noUi-horizontal .noUi-handle{width:var(--handle-width);height:var(--handle-height);right:var(--handle-right);top:var(--handle-top);}.noUi-vertical .noUi-handle{width:var(--handle-height);height:var(--handle-width);right:var(--handle-top);top:var(--handle-right);}.noUi-target.noUi-horizontal{margin:var(--slider-margin) 0px;}.noUi-target.noUi-vertical{margin:0px var(--slider-margin);}';
  class AbstractSliderView extends OrientedControlView {
    constructor() {
      super(...arguments);
      mt(this, "behaviour");
      mt(this, "connected", !1);
      mt(this, "group_el");
      mt(this, "slider_el");
      mt(this, "title_el");
      mt(this, "_auto_width", "auto");
      mt(this, "_auto_height", "auto");
      mt(this, "_noUiSlider");
    }
    *controls() {
      yield this.slider_el;
    }
    get _steps() {
      return this._noUiSlider.steps;
    }
    _update_slider() {
      this._noUiSlider.updateOptions(this._calc_to(), !0);
    }
    connect_signals() {
      super.connect_signals();
      const { direction: H, orientation: V, tooltips: q } = this.model.properties;
      this.on_change([H, V, q], () => this.render());
      const { bar_color: G } = this.model.properties;
      this.on_change(G, () => {
        this._set_bar_color();
      });
      const { value: J, title: Z, show_value: te } = this.model.properties;
      this.on_change([J, Z, te], () => this._update_title()), this.on_change(J, () => this._update_slider());
    }
    stylesheets() {
      return [...super.stylesheets(), nouislider_css, sliders_css];
    }
    _update_title() {
      empty$1(this.title_el);
      const H = this.model.title == null || this.model.title.length == 0 && !this.model.show_value;
      if (this.title_el.style.display = H ? "none" : "", !H) {
        const { title: V } = this.model;
        if (V != null && V.length > 0 && (this.contains_tex_string(V) ? this.title_el.innerHTML = `${this.process_tex(V)}: ` : this.title_el.textContent = `${V}: `), this.model.show_value) {
          const { start: q } = this._calc_to(), G = q.map((J) => this.pretty(J)).join(" .. ");
          this.title_el.appendChild(span$1({ class: slider_value }, G));
        }
      }
    }
    _set_bar_color() {
      if (this.connected !== !1 && !this.model.disabled && this.slider_el != null) {
        const H = this.slider_el.querySelector(".noUi-connect");
        H.style.backgroundColor = color2css(this.model.bar_color);
      }
    }
    render() {
      super.render();
      let H;
      if (this.model.tooltips) {
        const V = {
          to: (G) => this.pretty(G)
        }, { start: q } = this._calc_to();
        H = repeat$1(V, q.length);
      } else
        H = null;
      if (this.slider_el == null) {
        this.slider_el = div(), this._noUiSlider = noUiSlider.create(this.slider_el, {
          ...this._calc_to(),
          behaviour: this.behaviour,
          connect: this.connected,
          tooltips: H ?? !1,
          orientation: this.model.orientation,
          direction: this.model.direction
        }), this._noUiSlider.on("slide", (q, G, J) => this._slide(J)), this._noUiSlider.on("change", (q, G, J) => this._change(J));
        const V = (q, G) => {
          if (H == null || this.slider_el == null)
            return;
          const Z = this.slider_el.querySelectorAll(".noUi-handle")[q].querySelector(".noUi-tooltip");
          Z.style.display = G ? "block" : "";
        };
        this._noUiSlider.on("start", () => this._toggle_user_select(!1)), this._noUiSlider.on("end", () => this._toggle_user_select(!0)), this._noUiSlider.on("start", (q, G) => V(G, !0)), this._noUiSlider.on("end", (q, G) => V(G, !1));
      } else
        this._update_slider();
      this._set_bar_color(), this.model.disabled ? this.slider_el.setAttribute("disabled", "true") : this.slider_el.removeAttribute("disabled"), this.title_el = div({ class: slider_title }), this._update_title(), this.group_el = div({ class: input_group }, this.title_el, this.slider_el), this.shadow_el.appendChild(this.group_el), this._has_finished = !0;
    }
    _toggle_user_select(H) {
      const { style: V } = document.body, q = H ? "" : "none";
      V.userSelect = q, V.webkitUserSelect = q;
    }
    _slide(H) {
      this.model.value = this._calc_from(H);
    }
    _change(H) {
      const V = this._calc_from(H);
      this.model.setv({ value: V, value_throttled: V });
    }
  }
  mt(AbstractSliderView, "__name__", "AbstractSliderView");
  const p4 = class p4 extends OrientedControl {
    constructor(_) {
      super(_);
    }
  };
  mt(p4, "__name__", "AbstractSlider"), p4.define(({ Unknown: _, Bool: H, Str: V, Color: q, Enum: G, Nullable: J }) => ({
    title: [J(V), ""],
    show_value: [H, !0],
    value: [_],
    value_throttled: [_, unset, { readonly: !0 }],
    direction: [G("ltr", "rtl"), "ltr"],
    tooltips: [H, !0],
    bar_color: [q, "#e6e6e6"]
  })), p4.override({
    width: 300
    // sliders don't have any intrinsic width
  });
  let AbstractSlider = p4;
  class CategoricalSliderView extends AbstractSliderView {
    constructor() {
      super(...arguments);
      mt(this, "behaviour", "tap");
    }
    connect_signals() {
      super.connect_signals();
      const { categories: H } = this.model.properties;
      this.on_change([H], () => this._update_slider());
    }
    _calc_to() {
      const { categories: H } = this.model;
      return {
        range: {
          min: 0,
          max: H.length - 1
        },
        start: [this.model.value],
        step: 1,
        format: {
          to: (V) => H[V],
          from: (V) => H.indexOf(V)
        }
      };
    }
    _calc_from([H]) {
      const { categories: V } = this.model;
      return V[H | 0];
    }
    pretty(H) {
      return isNumber$3(H) ? this.model.categories[H] : H;
    }
  }
  mt(CategoricalSliderView, "__name__", "CategoricalSliderView");
  const m4 = class m4 extends AbstractSlider {
    constructor(_) {
      super(_);
    }
  };
  mt(m4, "__name__", "CategoricalSlider"), m4.prototype.default_view = CategoricalSliderView, m4.define(({ List: _, Str: H }) => ({
    categories: [_(H)]
  }));
  let CategoricalSlider = m4;
  class BaseNumericalSliderView extends AbstractSliderView {
    connect_signals() {
      super.connect_signals();
      const { start: _, end: H, step: V } = this.model.properties;
      this.on_change([_, H, V], () => this._update_slider());
    }
    pretty(_) {
      return this._formatter(_, this.model.format);
    }
  }
  mt(BaseNumericalSliderView, "__name__", "BaseNumericalSliderView");
  const $6 = class $6 extends AbstractSlider {
    constructor(_) {
      super(_);
    }
  };
  mt($6, "__name__", "BaseNumericalSlider"), $6.define(({ Float: _, Str: H, Or: V, Ref: q }) => ({
    start: [_],
    end: [_],
    step: [_, 1],
    format: [V(H, q(TickFormatter))]
  }));
  let BaseNumericalSlider = $6;
  class NumericalRangeSliderView extends BaseNumericalSliderView {
    _calc_to() {
      return {
        range: {
          min: this.model.start,
          max: this.model.end
        },
        start: this.model.value,
        step: this.model.step
      };
    }
    _calc_from(_) {
      return _;
    }
  }
  mt(NumericalRangeSliderView, "__name__", "NumericalRangeSliderView");
  class NumericalRangeSlider extends BaseNumericalSlider {
    constructor(_) {
      super(_);
    }
  }
  mt(NumericalRangeSlider, "__name__", "NumericalRangeSlider");
  class DateRangeSliderView extends NumericalRangeSliderView {
    constructor() {
      super(...arguments);
      mt(this, "behaviour", "drag");
      mt(this, "connected", [!1, !0, !1]);
    }
    _calc_to() {
      const H = super._calc_to();
      return H.step *= 864e5, H;
    }
    _formatter(H, V) {
      return isString$2(V) ? tz(H, V) : V.compute(H);
    }
  }
  mt(DateRangeSliderView, "__name__", "DateRangeSliderView");
  const g4 = class g4 extends NumericalRangeSlider {
    constructor(_) {
      super(_);
    }
  };
  mt(g4, "__name__", "DateRangeSlider"), g4.prototype.default_view = DateRangeSliderView, g4.override({
    format: "%d %b %Y"
  });
  let DateRangeSlider = g4;
  class NumericalSliderView extends BaseNumericalSliderView {
    _calc_to() {
      const { start: _, end: H, value: V, step: q } = this.model;
      return {
        range: {
          min: _,
          max: H
        },
        start: [V],
        step: q
      };
    }
    _calc_from([_]) {
      return Number.isInteger(this.model.start) && Number.isInteger(this.model.end) && Number.isInteger(this.model.step) ? Math.round(_) : _;
    }
  }
  mt(NumericalSliderView, "__name__", "NumericalSliderView");
  class NumericalSlider extends BaseNumericalSlider {
    constructor(_) {
      super(_);
    }
  }
  mt(NumericalSlider, "__name__", "NumericalSlider");
  class DateSliderView extends NumericalSliderView {
    constructor() {
      super(...arguments);
      mt(this, "behaviour", "tap");
      mt(this, "connected", [!0, !1]);
    }
    _calc_to() {
      const H = super._calc_to();
      return H.step *= 864e5, H;
    }
    _formatter(H, V) {
      return isString$2(V) ? tz(H, V) : V.compute(H);
    }
  }
  mt(DateSliderView, "__name__", "DateSliderView");
  const _4 = class _4 extends NumericalSlider {
    constructor(_) {
      super(_);
    }
  };
  mt(_4, "__name__", "DateSlider"), _4.prototype.default_view = DateSliderView, _4.override({
    format: "%d %b %Y"
  });
  let DateSlider = _4;
  class DatetimeRangeSliderView extends NumericalRangeSliderView {
    constructor() {
      super(...arguments);
      mt(this, "behaviour", "drag");
      mt(this, "connected", [!1, !0, !1]);
    }
    _formatter(H, V) {
      return isString$2(V) ? tz(H, V) : V.compute(H);
    }
  }
  mt(DatetimeRangeSliderView, "__name__", "DatetimeRangeSliderView");
  const y4 = class y4 extends NumericalRangeSlider {
    constructor(_) {
      super(_);
    }
  };
  mt(y4, "__name__", "DatetimeRangeSlider"), y4.prototype.default_view = DatetimeRangeSliderView, y4.override({
    format: "%d %b %Y %H:%M:%S",
    step: 36e5
    // 1 hour
  });
  let DatetimeRangeSlider = y4;
  class RangeSliderView extends NumericalRangeSliderView {
    constructor() {
      super(...arguments);
      mt(this, "behaviour", "drag");
      mt(this, "connected", [!1, !0, !1]);
    }
    _formatter(H, V) {
      return isString$2(V) ? numbroExports.format(H, V) : V.compute(H);
    }
  }
  mt(RangeSliderView, "__name__", "RangeSliderView");
  const b4 = class b4 extends NumericalRangeSlider {
    constructor(_) {
      super(_);
    }
  };
  mt(b4, "__name__", "RangeSlider"), b4.prototype.default_view = RangeSliderView, b4.override({
    format: "0[.]00"
  });
  let RangeSlider = b4;
  class SliderView extends NumericalSliderView {
    constructor() {
      super(...arguments);
      mt(this, "behaviour", "tap");
      mt(this, "connected", [!0, !1]);
    }
    _formatter(H, V) {
      return isString$2(V) ? numbroExports.format(H, V) : V.compute(H);
    }
  }
  mt(SliderView, "__name__", "SliderView");
  const T4 = class T4 extends NumericalSlider {
    constructor(_) {
      super(_);
    }
  };
  mt(T4, "__name__", "Slider"), T4.prototype.default_view = SliderView, T4.override({
    format: "0[.]00"
  });
  let Slider = T4;
  const Widgets = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    AbstractButton,
    AutocompleteInput,
    Button,
    CategoricalSlider,
    Checkbox,
    CheckboxButtonGroup,
    CheckboxGroup,
    ColorPicker,
    DatePicker,
    DateRangePicker,
    DateRangeSlider,
    DateSlider,
    DatetimePicker,
    DatetimeRangePicker,
    DatetimeRangeSlider,
    Div,
    Dropdown,
    FileInput,
    HelpButton,
    InputWidget,
    Markup,
    MultiChoice,
    MultiSelect,
    MultipleDatePicker,
    MultipleDatetimePicker,
    NumericInput,
    PaletteSelect,
    Paragraph,
    PasswordInput,
    PreText,
    RadioButtonGroup,
    RadioGroup,
    RangeSlider,
    Select,
    Slider,
    Spinner,
    Switch,
    TextAreaInput,
    TextInput,
    TimePicker,
    Toggle,
    Widget
  }, Symbol.toStringTag, { value: "Module" }));
  register_models(Widgets);
  const DTINDEX_NAME = "__bkdt_internal_index__", data_table = "bk-data-table", cell_select = "bk-cell-select", cell_index = "bk-cell-index", header_index = "bk-header-index", cell_editor = "bk-cell-editor", tables_css = ':host{--data-table-font-size:var(--font-size);}.bk-data-table{box-sizing:content-box;width:100%;height:100%;font-size:var(--data-table-font-size);}.bk-data-table input[type="checkbox"]{margin-left:4px;margin-right:4px;}.bk-cell-special-defaults{border-right-color:silver;border-right-style:solid;background:#f5f5f5;}.bk-cell-select{border-right-color:silver;border-right-style:solid;background:#f5f5f5;}.slick-cell.bk-cell-index{border-right-color:silver;border-right-style:solid;background:#f5f5f5;text-align:right;background:#f0f0f0;color:#909090;}.bk-header-index .slick-column-name{float:right;}.slick-row.selected .bk-cell-index{background-color:transparent;}.slick-row.odd{background:#f0f0f0;}.slick-cell{padding-left:4px;padding-right:4px;border-right-color:transparent;border:0.25px solid transparent;}.slick-cell .bk{line-height:inherit;}.slick-cell.active{border-style:dashed;}.slick-cell.selected{background-color:#F0F8FF;}.slick-cell.editable{padding-left:0;padding-right:0;}.bk-cell-editor{display:contents;}.bk-cell-editor input,.bk-cell-editor select{width:100%;height:100%;border:0;margin:0;padding:0;outline:0;background:transparent;vertical-align:baseline;}.bk-cell-editor input{padding-left:4px;padding-right:4px;}.bk-cell-editor-completion{font-size:var(--data-table-font-size);}';
  class CellEditorView extends DOMComponentView {
    constructor(H) {
      const { model: V, parent: q } = H.column;
      super({ model: V, parent: q, ...H });
      mt(this, "defaultValue");
      mt(this, "inputEl");
      mt(this, "args");
      this.args = H, this.initialize(), this.render();
    }
    get emptyValue() {
      return null;
    }
    initialize() {
      super.initialize(), this.inputEl = this._createInput(), this.defaultValue = null;
    }
    async lazy_initialize() {
      throw new Error("unsupported");
    }
    css_classes() {
      return super.css_classes().concat(cell_editor);
    }
    render() {
      this.args.container.append(this.el), this.shadow_el.appendChild(this.inputEl), this.renderEditor(), this.disableNavigation();
    }
    renderEditor() {
    }
    disableNavigation() {
      this.inputEl.addEventListener("keydown", (H) => {
        switch (H.key) {
          case "ArrowLeft":
          case "ArrowRight":
          case "ArrowUp":
          case "ArrowDown":
          case "PageUp":
          case "PageDown": {
            H.stopImmediatePropagation();
            break;
          }
        }
      });
    }
    destroy() {
      this.remove();
    }
    focus() {
      this.inputEl.focus();
    }
    show() {
    }
    hide() {
    }
    position() {
    }
    getValue() {
      return this.inputEl.value;
    }
    setValue(H) {
      this.inputEl.value = H;
    }
    serializeValue() {
      return this.getValue();
    }
    isValueChanged() {
      return !(this.getValue() == "" && this.defaultValue == null) && this.getValue() !== this.defaultValue;
    }
    applyValue(H, V) {
      const q = this.args.grid.getData(), G = q.index.indexOf(H[DTINDEX_NAME]);
      q.setField(G, this.args.column.field, V);
    }
    loadValue(H) {
      const V = H[this.args.column.field];
      this.defaultValue = V ?? this.emptyValue, this.setValue(this.defaultValue);
    }
    validateValue(H) {
      if (this.args.column.validator) {
        const V = this.args.column.validator(H);
        if (!V.valid)
          return V;
      }
      return { valid: !0, msg: null };
    }
    validate() {
      return this.validateValue(this.getValue());
    }
  }
  mt(CellEditorView, "__name__", "CellEditorView");
  class CellEditor extends Model$1 {
  }
  mt(CellEditor, "__name__", "CellEditor");
  class StringEditorView extends CellEditorView {
    get emptyValue() {
      return "";
    }
    _createInput() {
      return input$1({ type: "text" });
    }
    renderEditor() {
      this.inputEl.focus(), this.inputEl.select();
    }
    loadValue(_) {
      super.loadValue(_), this.inputEl.defaultValue = this.defaultValue, this.inputEl.select();
    }
  }
  mt(StringEditorView, "__name__", "StringEditorView");
  const x4 = class x4 extends CellEditor {
  };
  mt(x4, "__name__", "StringEditor"), x4.prototype.default_view = StringEditorView, x4.define(({ Str: _, List: H }) => ({
    completions: [H(_), []]
  }));
  let StringEditor = x4;
  class TextEditorView extends CellEditorView {
    _createInput() {
      return textarea();
    }
    renderEditor() {
      this.inputEl.focus(), this.inputEl.select();
    }
  }
  mt(TextEditorView, "__name__", "TextEditorView");
  var H0;
  let TextEditor$1 = (H0 = class extends CellEditor {
  }, mt(H0, "__name__", "TextEditor"), H0.prototype.default_view = TextEditorView, H0);
  class SelectEditorView extends CellEditorView {
    _createInput() {
      return select$1();
    }
    renderEditor() {
      for (const _ of this.model.options)
        this.inputEl.appendChild(option({ value: _ }, _));
      this.focus();
    }
  }
  mt(SelectEditorView, "__name__", "SelectEditorView");
  const w4 = class w4 extends CellEditor {
  };
  mt(w4, "__name__", "SelectEditor"), w4.prototype.default_view = SelectEditorView, w4.define(({ Str: _, List: H }) => ({
    options: [H(_), []]
  }));
  let SelectEditor = w4;
  class PercentEditorView extends CellEditorView {
    _createInput() {
      return input$1({ type: "text" });
    }
  }
  mt(PercentEditorView, "__name__", "PercentEditorView");
  const F6 = class F6 extends CellEditor {
  };
  mt(F6, "__name__", "PercentEditor"), F6.prototype.default_view = PercentEditorView;
  let PercentEditor = F6;
  class CheckboxEditorView extends CellEditorView {
    _createInput() {
      return input$1({ type: "checkbox" });
    }
    renderEditor() {
      this.focus();
    }
    loadValue(_) {
      this.defaultValue = !!_[this.args.column.field], this.inputEl.checked = this.defaultValue;
    }
    serializeValue() {
      return this.inputEl.checked;
    }
  }
  mt(CheckboxEditorView, "__name__", "CheckboxEditorView");
  const B6 = class B6 extends CellEditor {
  };
  mt(B6, "__name__", "CheckboxEditor"), B6.prototype.default_view = CheckboxEditorView;
  let CheckboxEditor = B6;
  class IntEditorView extends CellEditorView {
    _createInput() {
      return input$1({ type: "text" });
    }
    renderEditor() {
      this.inputEl.focus(), this.inputEl.select();
    }
    remove() {
      super.remove();
    }
    serializeValue() {
      const _ = parseInt(this.getValue(), 10);
      return isNaN(_) ? 0 : _;
    }
    loadValue(_) {
      super.loadValue(_), this.inputEl.defaultValue = this.defaultValue, this.inputEl.select();
    }
    validateValue(_) {
      return isString$2(_) && (_ = Number(_)), isInteger$2(_) ? super.validateValue(_) : { valid: !1, msg: "Please enter a valid integer" };
    }
  }
  mt(IntEditorView, "__name__", "IntEditorView");
  const S4 = class S4 extends CellEditor {
  };
  mt(S4, "__name__", "IntEditor"), S4.prototype.default_view = IntEditorView, S4.define(({ Int: _ }) => ({
    step: [_, 1]
  }));
  let IntEditor = S4;
  class NumberEditorView extends CellEditorView {
    _createInput() {
      return input$1({ type: "text" });
    }
    renderEditor() {
      this.inputEl.focus(), this.inputEl.select();
    }
    remove() {
      super.remove();
    }
    serializeValue() {
      const _ = parseFloat(this.getValue());
      return isNaN(_) ? 0 : _;
    }
    loadValue(_) {
      super.loadValue(_), this.inputEl.defaultValue = this.defaultValue, this.inputEl.select();
    }
    validateValue(_) {
      return isNaN(_) ? { valid: !1, msg: "Please enter a valid number" } : super.validateValue(_);
    }
  }
  mt(NumberEditorView, "__name__", "NumberEditorView");
  const E4 = class E4 extends CellEditor {
  };
  mt(E4, "__name__", "NumberEditor"), E4.prototype.default_view = NumberEditorView, E4.define(({ Float: _ }) => ({
    step: [_, 0.01]
  }));
  let NumberEditor = E4;
  class TimeEditorView extends CellEditorView {
    _createInput() {
      return input$1({ type: "text" });
    }
  }
  mt(TimeEditorView, "__name__", "TimeEditorView");
  const H6 = class H6 extends CellEditor {
  };
  mt(H6, "__name__", "TimeEditor"), H6.prototype.default_view = TimeEditorView;
  let TimeEditor = H6;
  class DateEditorView extends CellEditorView {
    _createInput() {
      return input$1({ type: "text" });
    }
    get emptyValue() {
      return /* @__PURE__ */ new Date();
    }
    renderEditor() {
      this.inputEl.focus(), this.inputEl.select();
    }
    destroy() {
      super.destroy();
    }
    show() {
      super.show();
    }
    hide() {
      super.hide();
    }
    position() {
      return super.position();
    }
    getValue() {
    }
    setValue(_) {
    }
  }
  mt(DateEditorView, "__name__", "DateEditorView");
  const z6 = class z6 extends CellEditor {
  };
  mt(z6, "__name__", "DateEditor"), z6.prototype.default_view = DateEditorView;
  let DateEditor = z6;
  var underscore_template, hasRequiredUnderscore_template;
  function requireUnderscore_template() {
    if (hasRequiredUnderscore_template) return underscore_template;
    hasRequiredUnderscore_template = 1;
    var O = {}, _ = Array.prototype, H = Object.prototype, V = _.slice, q = H.toString, G = H.hasOwnProperty, J = _.forEach, Z = Object.keys, te = Array.isArray, ne = function() {
    }, ee = ne.each = ne.forEach = function(ue, de, fe) {
      if (ue != null) {
        if (J && ue.forEach === J)
          ue.forEach(de, fe);
        else if (ue.length === +ue.length) {
          for (var he = 0, me = ue.length; he < me; he++)
            if (de.call(fe, ue[he], he, ue) === O) return;
        } else
          for (var pe = ne.keys(ue), he = 0, me = pe.length; he < me; he++)
            if (de.call(fe, ue[pe[he]], pe[he], ue) === O) return;
      }
    };
    ne.keys = Z || function(ue) {
      if (ue !== Object(ue)) throw new TypeError("Invalid object");
      var de = [];
      for (var fe in ue) ne.has(ue, fe) && de.push(fe);
      return de;
    }, ne.defaults = function(ue) {
      return ee(V.call(arguments, 1), function(de) {
        if (de)
          for (var fe in de)
            ue[fe] === void 0 && (ue[fe] = de[fe]);
      }), ue;
    }, ne.isArray = te || function(ue) {
      return q.call(ue) === "[object Array]";
    }, ne.has = function(ue, de) {
      if (!ne.isArray(de))
        return ue != null && G.call(ue, de);
      for (var fe = de.length, he = 0; he < fe; he++) {
        var me = de[he];
        if (ue == null || !G.call(ue, me))
          return !1;
        ue = ue[me];
      }
      return !!fe;
    };
    var ae = {
      escape: {
        "&": "&amp;",
        "<": "&lt;",
        ">": "&gt;",
        '"': "&quot;",
        "'": "&#x27;"
      }
    }, oe = {
      escape: new RegExp("[" + ne.keys(ae.escape).join("") + "]", "g")
    };
    ne.each(["escape"], function(ue) {
      ne[ue] = function(de) {
        return de == null ? "" : ("" + de).replace(oe[ue], function(fe) {
          return ae[ue][fe];
        });
      };
    }), ne.templateSettings = {
      evaluate: /<%([\s\S]+?)%>/g,
      interpolate: /<%=([\s\S]+?)%>/g,
      escape: /<%-([\s\S]+?)%>/g
    };
    var se = /(.)^/, ce = {
      "'": "'",
      "\\": "\\",
      "\r": "r",
      "\n": "n",
      "	": "t",
      "\u2028": "u2028",
      "\u2029": "u2029"
    }, le = /\\|'|\r|\n|\t|\u2028|\u2029/g;
    return ne.template = function(ue, de, fe) {
      var he;
      fe = ne.defaults({}, fe, ne.templateSettings);
      var me = new RegExp([
        (fe.escape || se).source,
        (fe.interpolate || se).source,
        (fe.evaluate || se).source
      ].join("|") + "|$", "g"), pe = 0, ye = "__p+='";
      ue.replace(me, function(ve, be, Ce, Se, we) {
        return ye += ue.slice(pe, we).replace(le, function(Ee) {
          return "\\" + ce[Ee];
        }), be && (ye += `'+
((__t=(` + be + `))==null?'':_.escape(__t))+
'`), Ce && (ye += `'+
((__t=(` + Ce + `))==null?'':__t)+
'`), Se && (ye += `';
` + Se + `
__p+='`), pe = we + ve.length, ve;
      }), ye += `';
`, fe.variable || (ye = `with(obj||{}){
` + ye + `}
`), ye = `var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
` + ye + `return __p;
`;
      try {
        he = new Function(fe.variable || "obj", "_", ye);
      } catch (ve) {
        throw ve.source = ye, ve;
      }
      if (de) return he(de, ne);
      var ge = function(ve) {
        return he.call(this, ve, ne);
      };
      return ge.source = "function(" + (fe.variable || "obj") + `){
` + ye + "}", ge;
    }, underscore_template = ne, underscore_template;
  }
  var lib, hasRequiredLib;
  function requireLib() {
    if (hasRequiredLib) return lib;
    hasRequiredLib = 1;
    var O = requireUnderscore_template(), _ = O.template;
    function H(V, q, G) {
      return _(V, q, G);
    }
    return H._ = O, lib = H, (typeof window < "u" || typeof navigator < "u") && (window.UnderscoreTemplate = H), lib;
  }
  var libExports = requireLib();
  class CellFormatter extends Model$1 {
    constructor(_) {
      super(_);
    }
    doFormat(_, H, V, q, G) {
      return V == null ? "" : `${V}`.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
    }
  }
  mt(CellFormatter, "__name__", "CellFormatter");
  const V6 = class V6 extends CellFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H, V, q, G) {
      const { font_style: J, text_align: Z, text_color: te, background_color: ne } = this, ee = div(V == null ? "" : `${V}`);
      let ae;
      switch (isValue(J) ? ae = J.value : isField$1(J) ? ae = G[J.field] : isExpr$2(J) || unreachable(), ae) {
        case "normal":
          break;
        case "italic":
          ee.style.fontStyle = "italic";
          break;
        case "bold":
          ee.style.fontWeight = "bold";
          break;
        case "bold italic":
          ee.style.fontStyle = "italic", ee.style.fontWeight = "bold";
          break;
      }
      if (isValue(Z) ? ee.style.textAlign = Z.value : isField$1(Z) ? ee.style.textAlign = G[Z.field] : isExpr$2(Z) || unreachable(), isValue(te))
        te.value != null && (ee.style.color = color2css(te.value));
      else if (isField$1(te))
        if (te.transform != null && te.transform instanceof ColorMapper) {
          const oe = te.transform.rgba_mapper.v_compute([G[te.field]]), [se, ce, le, ue] = oe;
          ee.style.color = rgba2css([se, ce, le, ue]);
        } else
          ee.style.color = color2css(G[te.field]);
      else isExpr$2(te) || unreachable();
      if (isValue(ne))
        ne.value != null && (ee.style.backgroundColor = color2css(ne.value));
      else if (isField$1(ne))
        if (ne.transform != null && ne.transform instanceof ColorMapper) {
          const oe = ne.transform.rgba_mapper.v_compute([G[ne.field]]), [se, ce, le, ue] = oe;
          ee.style.backgroundColor = rgba2css([se, ce, le, ue]);
        } else
          ee.style.backgroundColor = color2css(G[ne.field]);
      else isExpr$2(ne) || unreachable();
      return ee.outerHTML;
    }
  };
  mt(V6, "__name__", "StringFormatter"), V6.define(({ Str: _ }) => ({
    font_style: [FontStyleSpec, { value: "normal" }],
    text_align: [TextAlignSpec, { value: "left" }],
    text_color: [ColorSpec, null],
    background_color: [ColorSpec, null],
    nan_format: [_, "-"]
  }));
  let StringFormatter = V6;
  const U6 = class U6 extends StringFormatter {
    constructor(_) {
      super(_);
    }
    get scientific_limit_low() {
      return 10 ** this.power_limit_low;
    }
    get scientific_limit_high() {
      return 10 ** this.power_limit_high;
    }
    doFormat(_, H, V, q, G) {
      const J = Math.abs(V) <= this.scientific_limit_low || Math.abs(V) >= this.scientific_limit_high;
      let Z = this.precision;
      return Z < 1 && (Z = 1), V == null || isNaN(V) ? V = this.nan_format : V == 0 ? V = to_fixed(V, 1) : J ? V = V.toExponential(Z) : V = to_fixed(V, Z), super.doFormat(_, H, V, q, G);
    }
  };
  mt(U6, "__name__", "ScientificFormatter"), U6.define(({ Float: _ }) => ({
    precision: [_, 10],
    power_limit_high: [_, 5],
    power_limit_low: [_, -3]
  }));
  let ScientificFormatter = U6;
  const q6 = class q6 extends StringFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H, V, q, G) {
      const { format: J, language: Z, nan_format: te } = this, ne = (() => {
        switch (this.rounding) {
          case "round":
          case "nearest":
            return Math.round;
          case "floor":
          case "rounddown":
            return Math.floor;
          case "ceil":
          case "roundup":
            return Math.ceil;
        }
      })();
      return V == null || isNaN(V) ? V = te : V = numbroExports.format(V, J, Z, ne), super.doFormat(_, H, V, q, G);
    }
  };
  mt(q6, "__name__", "NumberFormatter"), q6.define(({ Str: _ }) => ({
    format: [_, "0,0"],
    language: [_, "en"],
    rounding: [RoundingFunction, "round"]
  }));
  let NumberFormatter = q6;
  const j6 = class j6 extends CellFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H, V, q, G) {
      return V ? i$2({ class: this.icon }).outerHTML : "";
    }
  };
  mt(j6, "__name__", "BooleanFormatter"), j6.define(({ Str: _ }) => ({
    icon: [_, "check"]
  }));
  let BooleanFormatter = j6;
  const G6 = class G6 extends StringFormatter {
    constructor(_) {
      super(_);
    }
    getFormat() {
      switch (this.format) {
        case "ATOM":
        case "W3C":
        case "RFC-3339":
        case "ISO-8601":
          return "%Y-%m-%d";
        case "COOKIE":
          return "%a, %d %b %Y";
        case "RFC-850":
          return "%A, %d-%b-%y";
        case "RFC-1123":
        case "RFC-2822":
          return "%a, %e %b %Y";
        case "RSS":
        case "RFC-822":
        case "RFC-1036":
          return "%a, %e %b %y";
        case "TIMESTAMP":
          return;
        default:
          return this.format;
      }
    }
    doFormat(_, H, V, q, G) {
      function J(ee) {
        const oe = /Z$|[+-]\d\d((:?)\d\d)?$/.test(ee) ? ee : `${ee}Z`;
        return new Date(oe).getTime();
      }
      const Z = (() => {
        if (V == null || isNumber$3(V))
          return V;
        if (isString$2(V)) {
          const ee = Number(V);
          return isNaN(ee) ? J(V) : ee;
        } else return V instanceof Date ? V.valueOf() : Number(V);
      })(), te = -9223372036854776, ne = Z == null || isNaN(Z) || Z == te ? this.nan_format : tz(Z, this.getFormat());
      return super.doFormat(_, H, ne, q, G);
    }
  };
  mt(G6, "__name__", "DateFormatter"), G6.define(({ Str: _ }) => ({
    format: [_, "ISO-8601"]
  }));
  let DateFormatter = G6;
  const W6 = class W6 extends CellFormatter {
    constructor(_) {
      super(_);
    }
    doFormat(_, H, V, q, G) {
      const { template: J } = this;
      if (V == null)
        return "";
      {
        const Z = libExports._.template(J), te = { ...G, value: V };
        return Z(te);
      }
    }
  };
  mt(W6, "__name__", "HTMLTemplateFormatter"), W6.define(({ Str: _ }) => ({
    template: [_, "<%= value %>"]
  }));
  let HTMLTemplateFormatter = W6;
  var jquery$1 = { exports: {} };
  /*!
   * jQuery JavaScript Library v3.7.1
   * https://jquery.com/
   *
   * Copyright OpenJS Foundation and other contributors
   * Released under the MIT license
   * https://jquery.org/license
   *
   * Date: 2023-08-28T13:37Z
   */
  var jquery = jquery$1.exports, hasRequiredJquery;
  function requireJquery() {
    return hasRequiredJquery || (hasRequiredJquery = 1, function(O) {
      (function(_, H) {
        O.exports = _.document ? H(_, !0) : function(V) {
          if (!V.document)
            throw new Error("jQuery requires a window with a document");
          return H(V);
        };
      })(typeof window < "u" ? window : jquery, function(_, H) {
        var V = [], q = Object.getPrototypeOf, G = V.slice, J = V.flat ? function(nn) {
          return V.flat.call(nn);
        } : function(nn) {
          return V.concat.apply([], nn);
        }, Z = V.push, te = V.indexOf, ne = {}, ee = ne.toString, ae = ne.hasOwnProperty, oe = ae.toString, se = oe.call(Object), ce = {}, le = function(Tn) {
          return typeof Tn == "function" && typeof Tn.nodeType != "number" && typeof Tn.item != "function";
        }, ue = function(Tn) {
          return Tn != null && Tn === Tn.window;
        }, de = _.document, fe = {
          type: !0,
          src: !0,
          nonce: !0,
          noModule: !0
        };
        function he(nn, Tn, Dn) {
          Dn = Dn || de;
          var Fn, fr, dr = Dn.createElement("script");
          if (dr.text = nn, Tn)
            for (Fn in fe)
              fr = Tn[Fn] || Tn.getAttribute && Tn.getAttribute(Fn), fr && dr.setAttribute(Fn, fr);
          Dn.head.appendChild(dr).parentNode.removeChild(dr);
        }
        function me(nn) {
          return nn == null ? nn + "" : typeof nn == "object" || typeof nn == "function" ? ne[ee.call(nn)] || "object" : typeof nn;
        }
        var pe = "3.7.1", ye = /HTML$/i, ge = function(nn, Tn) {
          return new ge.fn.init(nn, Tn);
        };
        ge.fn = ge.prototype = {
          // The current version of jQuery being used
          jquery: pe,
          constructor: ge,
          // The default length of a jQuery object is 0
          length: 0,
          toArray: function() {
            return G.call(this);
          },
          // Get the Nth element in the matched element set OR
          // Get the whole matched element set as a clean array
          get: function(nn) {
            return nn == null ? G.call(this) : nn < 0 ? this[nn + this.length] : this[nn];
          },
          // Take an array of elements and push it onto the stack
          // (returning the new matched element set)
          pushStack: function(nn) {
            var Tn = ge.merge(this.constructor(), nn);
            return Tn.prevObject = this, Tn;
          },
          // Execute a callback for every element in the matched set.
          each: function(nn) {
            return ge.each(this, nn);
          },
          map: function(nn) {
            return this.pushStack(ge.map(this, function(Tn, Dn) {
              return nn.call(Tn, Dn, Tn);
            }));
          },
          slice: function() {
            return this.pushStack(G.apply(this, arguments));
          },
          first: function() {
            return this.eq(0);
          },
          last: function() {
            return this.eq(-1);
          },
          even: function() {
            return this.pushStack(ge.grep(this, function(nn, Tn) {
              return (Tn + 1) % 2;
            }));
          },
          odd: function() {
            return this.pushStack(ge.grep(this, function(nn, Tn) {
              return Tn % 2;
            }));
          },
          eq: function(nn) {
            var Tn = this.length, Dn = +nn + (nn < 0 ? Tn : 0);
            return this.pushStack(Dn >= 0 && Dn < Tn ? [this[Dn]] : []);
          },
          end: function() {
            return this.prevObject || this.constructor();
          },
          // For internal use only.
          // Behaves like an Array's method, not like a jQuery method.
          push: Z,
          sort: V.sort,
          splice: V.splice
        }, ge.extend = ge.fn.extend = function() {
          var nn, Tn, Dn, Fn, fr, dr, yr = arguments[0] || {}, ai = 1, Fr = arguments.length, Ti = !1;
          for (typeof yr == "boolean" && (Ti = yr, yr = arguments[ai] || {}, ai++), typeof yr != "object" && !le(yr) && (yr = {}), ai === Fr && (yr = this, ai--); ai < Fr; ai++)
            if ((nn = arguments[ai]) != null)
              for (Tn in nn)
                Fn = nn[Tn], !(Tn === "__proto__" || yr === Fn) && (Ti && Fn && (ge.isPlainObject(Fn) || (fr = Array.isArray(Fn))) ? (Dn = yr[Tn], fr && !Array.isArray(Dn) ? dr = [] : !fr && !ge.isPlainObject(Dn) ? dr = {} : dr = Dn, fr = !1, yr[Tn] = ge.extend(Ti, dr, Fn)) : Fn !== void 0 && (yr[Tn] = Fn));
          return yr;
        }, ge.extend({
          // Unique for each copy of jQuery on the page
          expando: "jQuery" + (pe + Math.random()).replace(/\D/g, ""),
          // Assume jQuery is ready without the ready module
          isReady: !0,
          error: function(nn) {
            throw new Error(nn);
          },
          noop: function() {
          },
          isPlainObject: function(nn) {
            var Tn, Dn;
            return !nn || ee.call(nn) !== "[object Object]" ? !1 : (Tn = q(nn), Tn ? (Dn = ae.call(Tn, "constructor") && Tn.constructor, typeof Dn == "function" && oe.call(Dn) === se) : !0);
          },
          isEmptyObject: function(nn) {
            var Tn;
            for (Tn in nn)
              return !1;
            return !0;
          },
          // Evaluates a script in a provided context; falls back to the global one
          // if not specified.
          globalEval: function(nn, Tn, Dn) {
            he(nn, { nonce: Tn && Tn.nonce }, Dn);
          },
          each: function(nn, Tn) {
            var Dn, Fn = 0;
            if (ve(nn))
              for (Dn = nn.length; Fn < Dn && Tn.call(nn[Fn], Fn, nn[Fn]) !== !1; Fn++)
                ;
            else
              for (Fn in nn)
                if (Tn.call(nn[Fn], Fn, nn[Fn]) === !1)
                  break;
            return nn;
          },
          // Retrieve the text value of an array of DOM nodes
          text: function(nn) {
            var Tn, Dn = "", Fn = 0, fr = nn.nodeType;
            if (!fr)
              for (; Tn = nn[Fn++]; )
                Dn += ge.text(Tn);
            return fr === 1 || fr === 11 ? nn.textContent : fr === 9 ? nn.documentElement.textContent : fr === 3 || fr === 4 ? nn.nodeValue : Dn;
          },
          // results is for internal usage only
          makeArray: function(nn, Tn) {
            var Dn = Tn || [];
            return nn != null && (ve(Object(nn)) ? ge.merge(
              Dn,
              typeof nn == "string" ? [nn] : nn
            ) : Z.call(Dn, nn)), Dn;
          },
          inArray: function(nn, Tn, Dn) {
            return Tn == null ? -1 : te.call(Tn, nn, Dn);
          },
          isXMLDoc: function(nn) {
            var Tn = nn && nn.namespaceURI, Dn = nn && (nn.ownerDocument || nn).documentElement;
            return !ye.test(Tn || Dn && Dn.nodeName || "HTML");
          },
          // Support: Android <=4.0 only, PhantomJS 1 only
          // push.apply(_, arraylike) throws on ancient WebKit
          merge: function(nn, Tn) {
            for (var Dn = +Tn.length, Fn = 0, fr = nn.length; Fn < Dn; Fn++)
              nn[fr++] = Tn[Fn];
            return nn.length = fr, nn;
          },
          grep: function(nn, Tn, Dn) {
            for (var Fn, fr = [], dr = 0, yr = nn.length, ai = !Dn; dr < yr; dr++)
              Fn = !Tn(nn[dr], dr), Fn !== ai && fr.push(nn[dr]);
            return fr;
          },
          // arg is for internal usage only
          map: function(nn, Tn, Dn) {
            var Fn, fr, dr = 0, yr = [];
            if (ve(nn))
              for (Fn = nn.length; dr < Fn; dr++)
                fr = Tn(nn[dr], dr, Dn), fr != null && yr.push(fr);
            else
              for (dr in nn)
                fr = Tn(nn[dr], dr, Dn), fr != null && yr.push(fr);
            return J(yr);
          },
          // A global GUID counter for objects
          guid: 1,
          // jQuery.support is not used in Core but other projects attach their
          // properties to it so it needs to exist.
          support: ce
        }), typeof Symbol == "function" && (ge.fn[Symbol.iterator] = V[Symbol.iterator]), ge.each(
          "Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),
          function(nn, Tn) {
            ne["[object " + Tn + "]"] = Tn.toLowerCase();
          }
        );
        function ve(nn) {
          var Tn = !!nn && "length" in nn && nn.length, Dn = me(nn);
          return le(nn) || ue(nn) ? !1 : Dn === "array" || Tn === 0 || typeof Tn == "number" && Tn > 0 && Tn - 1 in nn;
        }
        function be(nn, Tn) {
          return nn.nodeName && nn.nodeName.toLowerCase() === Tn.toLowerCase();
        }
        var Ce = V.pop, Se = V.sort, we = V.splice, Ee = "[\\x20\\t\\r\\n\\f]", xe = new RegExp(
          "^" + Ee + "+|((?:^|[^\\\\])(?:\\\\.)*)" + Ee + "+$",
          "g"
        );
        ge.contains = function(nn, Tn) {
          var Dn = Tn && Tn.parentNode;
          return nn === Dn || !!(Dn && Dn.nodeType === 1 && // Support: IE 9 - 11+
          // IE doesn't have `contains` on SVG.
          (nn.contains ? nn.contains(Dn) : nn.compareDocumentPosition && nn.compareDocumentPosition(Dn) & 16));
        };
        var Le = /([\0-\x1f\x7f]|^-?\d)|^-$|[^\x80-\uFFFF\w-]/g;
        function Qe(nn, Tn) {
          return Tn ? nn === "\0" ? "�" : nn.slice(0, -1) + "\\" + nn.charCodeAt(nn.length - 1).toString(16) + " " : "\\" + nn;
        }
        ge.escapeSelector = function(nn) {
          return (nn + "").replace(Le, Qe);
        };
        var ke = de, Me = Z;
        (function() {
          var nn, Tn, Dn, Fn, fr, dr = Me, yr, ai, Fr, Ti, Si, Ei = ge.expando, Fa = 0, $o = 0, Gs = Kl(), Al = Kl(), ul = Kl(), gc = Kl(), Gl = function(vi, Ri) {
            return vi === Ri && (fr = !0), 0;
          }, Sl = "checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped", ls = "(?:\\\\[\\da-fA-F]{1,6}" + Ee + "?|\\\\[^\\r\\n\\f]|[\\w-]|[^\0-\\x7f])+", vs = "\\[" + Ee + "*(" + ls + ")(?:" + Ee + // Operator (capture 2)
          "*([*^$|!~]?=)" + Ee + // "Attribute values must be CSS identifiers [capture 5] or strings [capture 3 or capture 4]"
          `*(?:'((?:\\\\.|[^\\\\'])*)'|"((?:\\\\.|[^\\\\"])*)"|(` + ls + "))|)" + Ee + "*\\]", Vl = ":(" + ls + `)(?:\\((('((?:\\\\.|[^\\\\'])*)'|"((?:\\\\.|[^\\\\"])*)")|((?:\\\\.|[^\\\\()[\\]]|` + vs + ")*)|.*)\\)|)", Ll = new RegExp(Ee + "+", "g"), Xl = new RegExp("^" + Ee + "*," + Ee + "*"), yc = new RegExp("^" + Ee + "*([>+~]|" + Ee + ")" + Ee + "*"), gu = new RegExp(Ee + "|>"), Nc = new RegExp(Vl), Cu = new RegExp("^" + ls + "$"), zc = {
            ID: new RegExp("^#(" + ls + ")"),
            CLASS: new RegExp("^\\.(" + ls + ")"),
            TAG: new RegExp("^(" + ls + "|[*])"),
            ATTR: new RegExp("^" + vs),
            PSEUDO: new RegExp("^" + Vl),
            CHILD: new RegExp(
              "^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(" + Ee + "*(even|odd|(([+-]|)(\\d*)n|)" + Ee + "*(?:([+-]|)" + Ee + "*(\\d+)|))" + Ee + "*\\)|)",
              "i"
            ),
            bool: new RegExp("^(?:" + Sl + ")$", "i"),
            // For use in libraries implementing .is()
            // We use this for POS matching in `select`
            needsContext: new RegExp("^" + Ee + "*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(" + Ee + "*((?:-\\d)?\\d*)" + Ee + "*\\)|)(?=[^-]|$)", "i")
          }, Kc = /^(?:input|select|textarea|button)$/i, zi = /^h\d$/i, Fu = /^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/, d1 = /[+~]/, Gu = new RegExp("\\\\[\\da-fA-F]{1,6}" + Ee + "?|\\\\([^\\r\\n\\f])", "g"), Ku = function(vi, Ri) {
            var ta = "0x" + vi.slice(1) - 65536;
            return Ri || (ta < 0 ? String.fromCharCode(ta + 65536) : String.fromCharCode(ta >> 10 | 55296, ta & 1023 | 56320));
          }, I1 = function() {
            uo();
          }, f1 = No(
            function(vi) {
              return vi.disabled === !0 && be(vi, "fieldset");
            },
            { dir: "parentNode", next: "legend" }
          );
          function D1() {
            try {
              return yr.activeElement;
            } catch {
            }
          }
          try {
            dr.apply(
              V = G.call(ke.childNodes),
              ke.childNodes
            ), V[ke.childNodes.length].nodeType;
          } catch {
            dr = {
              apply: function(Ri, ta) {
                Me.apply(Ri, G.call(ta));
              },
              call: function(Ri) {
                Me.apply(Ri, G.call(arguments, 1));
              }
            };
          }
          function Hl(vi, Ri, ta, da) {
            var Ji, Pa, Ga, Ha, Ba, ao, io, To = Ri && Ri.ownerDocument, Mo = Ri ? Ri.nodeType : 9;
            if (ta = ta || [], typeof vi != "string" || !vi || Mo !== 1 && Mo !== 9 && Mo !== 11)
              return ta;
            if (!da && (uo(Ri), Ri = Ri || yr, Fr)) {
              if (Mo !== 11 && (Ba = Fu.exec(vi)))
                if (Ji = Ba[1]) {
                  if (Mo === 9)
                    if (Ga = Ri.getElementById(Ji)) {
                      if (Ga.id === Ji)
                        return dr.call(ta, Ga), ta;
                    } else
                      return ta;
                  else if (To && (Ga = To.getElementById(Ji)) && Hl.contains(Ri, Ga) && Ga.id === Ji)
                    return dr.call(ta, Ga), ta;
                } else {
                  if (Ba[2])
                    return dr.apply(ta, Ri.getElementsByTagName(vi)), ta;
                  if ((Ji = Ba[3]) && Ri.getElementsByClassName)
                    return dr.apply(ta, Ri.getElementsByClassName(Ji)), ta;
                }
              if (!gc[vi + " "] && (!Ti || !Ti.test(vi))) {
                if (io = vi, To = Ri, Mo === 1 && (gu.test(vi) || yc.test(vi))) {
                  for (To = d1.test(vi) && jo(Ri.parentNode) || Ri, (To != Ri || !ce.scope) && ((Ha = Ri.getAttribute("id")) ? Ha = ge.escapeSelector(Ha) : Ri.setAttribute("id", Ha = Ei)), ao = So(vi), Pa = ao.length; Pa--; )
                    ao[Pa] = (Ha ? "#" + Ha : ":scope") + " " + Jc(ao[Pa]);
                  io = ao.join(",");
                }
                try {
                  return dr.apply(
                    ta,
                    To.querySelectorAll(io)
                  ), ta;
                } catch {
                  gc(vi, !0);
                } finally {
                  Ha === Ei && Ri.removeAttribute("id");
                }
              }
            }
            return Ms(vi.replace(xe, "$1"), Ri, ta, da);
          }
          function Kl() {
            var vi = [];
            function Ri(ta, da) {
              return vi.push(ta + " ") > Tn.cacheLength && delete Ri[vi.shift()], Ri[ta + " "] = da;
            }
            return Ri;
          }
          function uu(vi) {
            return vi[Ei] = !0, vi;
          }
          function Hs(vi) {
            var Ri = yr.createElement("fieldset");
            try {
              return !!vi(Ri);
            } catch {
              return !1;
            } finally {
              Ri.parentNode && Ri.parentNode.removeChild(Ri), Ri = null;
            }
          }
          function b1(vi) {
            return function(Ri) {
              return be(Ri, "input") && Ri.type === vi;
            };
          }
          function Wl(vi) {
            return function(Ri) {
              return (be(Ri, "input") || be(Ri, "button")) && Ri.type === vi;
            };
          }
          function zl(vi) {
            return function(Ri) {
              return "form" in Ri ? Ri.parentNode && Ri.disabled === !1 ? "label" in Ri ? "label" in Ri.parentNode ? Ri.parentNode.disabled === vi : Ri.disabled === vi : Ri.isDisabled === vi || // Where there is no isDisabled, check manually
              Ri.isDisabled !== !vi && f1(Ri) === vi : Ri.disabled === vi : "label" in Ri ? Ri.disabled === vi : !1;
            };
          }
          function os(vi) {
            return uu(function(Ri) {
              return Ri = +Ri, uu(function(ta, da) {
                for (var Ji, Pa = vi([], ta.length, Ri), Ga = Pa.length; Ga--; )
                  ta[Ji = Pa[Ga]] && (ta[Ji] = !(da[Ji] = ta[Ji]));
              });
            });
          }
          function jo(vi) {
            return vi && typeof vi.getElementsByTagName < "u" && vi;
          }
          function uo(vi) {
            var Ri, ta = vi ? vi.ownerDocument || vi : ke;
            return ta == yr || ta.nodeType !== 9 || !ta.documentElement || (yr = ta, ai = yr.documentElement, Fr = !ge.isXMLDoc(yr), Si = ai.matches || ai.webkitMatchesSelector || ai.msMatchesSelector, ai.msMatchesSelector && // Support: IE 11+, Edge 17 - 18+
            // IE/Edge sometimes throw a "Permission denied" error when strict-comparing
            // two documents; shallow comparisons work.
            // eslint-disable-next-line eqeqeq
            ke != yr && (Ri = yr.defaultView) && Ri.top !== Ri && Ri.addEventListener("unload", I1), ce.getById = Hs(function(da) {
              return ai.appendChild(da).id = ge.expando, !yr.getElementsByName || !yr.getElementsByName(ge.expando).length;
            }), ce.disconnectedMatch = Hs(function(da) {
              return Si.call(da, "*");
            }), ce.scope = Hs(function() {
              return yr.querySelectorAll(":scope");
            }), ce.cssHas = Hs(function() {
              try {
                return yr.querySelector(":has(*,:jqfake)"), !1;
              } catch {
                return !0;
              }
            }), ce.getById ? (Tn.filter.ID = function(da) {
              var Ji = da.replace(Gu, Ku);
              return function(Pa) {
                return Pa.getAttribute("id") === Ji;
              };
            }, Tn.find.ID = function(da, Ji) {
              if (typeof Ji.getElementById < "u" && Fr) {
                var Pa = Ji.getElementById(da);
                return Pa ? [Pa] : [];
              }
            }) : (Tn.filter.ID = function(da) {
              var Ji = da.replace(Gu, Ku);
              return function(Pa) {
                var Ga = typeof Pa.getAttributeNode < "u" && Pa.getAttributeNode("id");
                return Ga && Ga.value === Ji;
              };
            }, Tn.find.ID = function(da, Ji) {
              if (typeof Ji.getElementById < "u" && Fr) {
                var Pa, Ga, Ha, Ba = Ji.getElementById(da);
                if (Ba) {
                  if (Pa = Ba.getAttributeNode("id"), Pa && Pa.value === da)
                    return [Ba];
                  for (Ha = Ji.getElementsByName(da), Ga = 0; Ba = Ha[Ga++]; )
                    if (Pa = Ba.getAttributeNode("id"), Pa && Pa.value === da)
                      return [Ba];
                }
                return [];
              }
            }), Tn.find.TAG = function(da, Ji) {
              return typeof Ji.getElementsByTagName < "u" ? Ji.getElementsByTagName(da) : Ji.querySelectorAll(da);
            }, Tn.find.CLASS = function(da, Ji) {
              if (typeof Ji.getElementsByClassName < "u" && Fr)
                return Ji.getElementsByClassName(da);
            }, Ti = [], Hs(function(da) {
              var Ji;
              ai.appendChild(da).innerHTML = "<a id='" + Ei + "' href='' disabled='disabled'></a><select id='" + Ei + "-\r\\' disabled='disabled'><option selected=''></option></select>", da.querySelectorAll("[selected]").length || Ti.push("\\[" + Ee + "*(?:value|" + Sl + ")"), da.querySelectorAll("[id~=" + Ei + "-]").length || Ti.push("~="), da.querySelectorAll("a#" + Ei + "+*").length || Ti.push(".#.+[+~]"), da.querySelectorAll(":checked").length || Ti.push(":checked"), Ji = yr.createElement("input"), Ji.setAttribute("type", "hidden"), da.appendChild(Ji).setAttribute("name", "D"), ai.appendChild(da).disabled = !0, da.querySelectorAll(":disabled").length !== 2 && Ti.push(":enabled", ":disabled"), Ji = yr.createElement("input"), Ji.setAttribute("name", ""), da.appendChild(Ji), da.querySelectorAll("[name='']").length || Ti.push("\\[" + Ee + "*name" + Ee + "*=" + Ee + `*(?:''|"")`);
            }), ce.cssHas || Ti.push(":has"), Ti = Ti.length && new RegExp(Ti.join("|")), Gl = function(da, Ji) {
              if (da === Ji)
                return fr = !0, 0;
              var Pa = !da.compareDocumentPosition - !Ji.compareDocumentPosition;
              return Pa || (Pa = (da.ownerDocument || da) == (Ji.ownerDocument || Ji) ? da.compareDocumentPosition(Ji) : (
                // Otherwise we know they are disconnected
                1
              ), Pa & 1 || !ce.sortDetached && Ji.compareDocumentPosition(da) === Pa ? da === yr || da.ownerDocument == ke && Hl.contains(ke, da) ? -1 : Ji === yr || Ji.ownerDocument == ke && Hl.contains(ke, Ji) ? 1 : Fn ? te.call(Fn, da) - te.call(Fn, Ji) : 0 : Pa & 4 ? -1 : 1);
            }), yr;
          }
          Hl.matches = function(vi, Ri) {
            return Hl(vi, null, null, Ri);
          }, Hl.matchesSelector = function(vi, Ri) {
            if (uo(vi), Fr && !gc[Ri + " "] && (!Ti || !Ti.test(Ri)))
              try {
                var ta = Si.call(vi, Ri);
                if (ta || ce.disconnectedMatch || // As well, disconnected nodes are said to be in a document
                // fragment in IE 9
                vi.document && vi.document.nodeType !== 11)
                  return ta;
              } catch {
                gc(Ri, !0);
              }
            return Hl(Ri, yr, null, [vi]).length > 0;
          }, Hl.contains = function(vi, Ri) {
            return (vi.ownerDocument || vi) != yr && uo(vi), ge.contains(vi, Ri);
          }, Hl.attr = function(vi, Ri) {
            (vi.ownerDocument || vi) != yr && uo(vi);
            var ta = Tn.attrHandle[Ri.toLowerCase()], da = ta && ae.call(Tn.attrHandle, Ri.toLowerCase()) ? ta(vi, Ri, !Fr) : void 0;
            return da !== void 0 ? da : vi.getAttribute(Ri);
          }, Hl.error = function(vi) {
            throw new Error("Syntax error, unrecognized expression: " + vi);
          }, ge.uniqueSort = function(vi) {
            var Ri, ta = [], da = 0, Ji = 0;
            if (fr = !ce.sortStable, Fn = !ce.sortStable && G.call(vi, 0), Se.call(vi, Gl), fr) {
              for (; Ri = vi[Ji++]; )
                Ri === vi[Ji] && (da = ta.push(Ji));
              for (; da--; )
                we.call(vi, ta[da], 1);
            }
            return Fn = null, vi;
          }, ge.fn.uniqueSort = function() {
            return this.pushStack(ge.uniqueSort(G.apply(this)));
          }, Tn = ge.expr = {
            // Can be adjusted by the user
            cacheLength: 50,
            createPseudo: uu,
            match: zc,
            attrHandle: {},
            find: {},
            relative: {
              ">": { dir: "parentNode", first: !0 },
              " ": { dir: "parentNode" },
              "+": { dir: "previousSibling", first: !0 },
              "~": { dir: "previousSibling" }
            },
            preFilter: {
              ATTR: function(vi) {
                return vi[1] = vi[1].replace(Gu, Ku), vi[3] = (vi[3] || vi[4] || vi[5] || "").replace(Gu, Ku), vi[2] === "~=" && (vi[3] = " " + vi[3] + " "), vi.slice(0, 4);
              },
              CHILD: function(vi) {
                return vi[1] = vi[1].toLowerCase(), vi[1].slice(0, 3) === "nth" ? (vi[3] || Hl.error(vi[0]), vi[4] = +(vi[4] ? vi[5] + (vi[6] || 1) : 2 * (vi[3] === "even" || vi[3] === "odd")), vi[5] = +(vi[7] + vi[8] || vi[3] === "odd")) : vi[3] && Hl.error(vi[0]), vi;
              },
              PSEUDO: function(vi) {
                var Ri, ta = !vi[6] && vi[2];
                return zc.CHILD.test(vi[0]) ? null : (vi[3] ? vi[2] = vi[4] || vi[5] || "" : ta && Nc.test(ta) && // Get excess from tokenize (recursively)
                (Ri = So(ta, !0)) && // advance to the next closing parenthesis
                (Ri = ta.indexOf(")", ta.length - Ri) - ta.length) && (vi[0] = vi[0].slice(0, Ri), vi[2] = ta.slice(0, Ri)), vi.slice(0, 3));
              }
            },
            filter: {
              TAG: function(vi) {
                var Ri = vi.replace(Gu, Ku).toLowerCase();
                return vi === "*" ? function() {
                  return !0;
                } : function(ta) {
                  return be(ta, Ri);
                };
              },
              CLASS: function(vi) {
                var Ri = Gs[vi + " "];
                return Ri || (Ri = new RegExp("(^|" + Ee + ")" + vi + "(" + Ee + "|$)")) && Gs(vi, function(ta) {
                  return Ri.test(
                    typeof ta.className == "string" && ta.className || typeof ta.getAttribute < "u" && ta.getAttribute("class") || ""
                  );
                });
              },
              ATTR: function(vi, Ri, ta) {
                return function(da) {
                  var Ji = Hl.attr(da, vi);
                  return Ji == null ? Ri === "!=" : Ri ? (Ji += "", Ri === "=" ? Ji === ta : Ri === "!=" ? Ji !== ta : Ri === "^=" ? ta && Ji.indexOf(ta) === 0 : Ri === "*=" ? ta && Ji.indexOf(ta) > -1 : Ri === "$=" ? ta && Ji.slice(-ta.length) === ta : Ri === "~=" ? (" " + Ji.replace(Ll, " ") + " ").indexOf(ta) > -1 : Ri === "|=" ? Ji === ta || Ji.slice(0, ta.length + 1) === ta + "-" : !1) : !0;
                };
              },
              CHILD: function(vi, Ri, ta, da, Ji) {
                var Pa = vi.slice(0, 3) !== "nth", Ga = vi.slice(-4) !== "last", Ha = Ri === "of-type";
                return da === 1 && Ji === 0 ? (
                  // Shortcut for :nth-*(n)
                  function(Ba) {
                    return !!Ba.parentNode;
                  }
                ) : function(Ba, ao, io) {
                  var To, Mo, po, Vo, Xo, Go = Pa !== Ga ? "nextSibling" : "previousSibling", es = Ba.parentNode, Ts = Ha && Ba.nodeName.toLowerCase(), Fo = !io && !Ha, Oo = !1;
                  if (es) {
                    if (Pa) {
                      for (; Go; ) {
                        for (po = Ba; po = po[Go]; )
                          if (Ha ? be(po, Ts) : po.nodeType === 1)
                            return !1;
                        Xo = Go = vi === "only" && !Xo && "nextSibling";
                      }
                      return !0;
                    }
                    if (Xo = [Ga ? es.firstChild : es.lastChild], Ga && Fo) {
                      for (Mo = es[Ei] || (es[Ei] = {}), To = Mo[vi] || [], Vo = To[0] === Fa && To[1], Oo = Vo && To[2], po = Vo && es.childNodes[Vo]; po = ++Vo && po && po[Go] || // Fallback to seeking `elem` from the start
                      (Oo = Vo = 0) || Xo.pop(); )
                        if (po.nodeType === 1 && ++Oo && po === Ba) {
                          Mo[vi] = [Fa, Vo, Oo];
                          break;
                        }
                    } else if (Fo && (Mo = Ba[Ei] || (Ba[Ei] = {}), To = Mo[vi] || [], Vo = To[0] === Fa && To[1], Oo = Vo), Oo === !1)
                      for (; (po = ++Vo && po && po[Go] || (Oo = Vo = 0) || Xo.pop()) && !((Ha ? be(po, Ts) : po.nodeType === 1) && ++Oo && (Fo && (Mo = po[Ei] || (po[Ei] = {}), Mo[vi] = [Fa, Oo]), po === Ba)); )
                        ;
                    return Oo -= Ji, Oo === da || Oo % da === 0 && Oo / da >= 0;
                  }
                };
              },
              PSEUDO: function(vi, Ri) {
                var ta, da = Tn.pseudos[vi] || Tn.setFilters[vi.toLowerCase()] || Hl.error("unsupported pseudo: " + vi);
                return da[Ei] ? da(Ri) : da.length > 1 ? (ta = [vi, vi, "", Ri], Tn.setFilters.hasOwnProperty(vi.toLowerCase()) ? uu(function(Ji, Pa) {
                  for (var Ga, Ha = da(Ji, Ri), Ba = Ha.length; Ba--; )
                    Ga = te.call(Ji, Ha[Ba]), Ji[Ga] = !(Pa[Ga] = Ha[Ba]);
                }) : function(Ji) {
                  return da(Ji, 0, ta);
                }) : da;
              }
            },
            pseudos: {
              // Potentially complex pseudos
              not: uu(function(vi) {
                var Ri = [], ta = [], da = Ls(vi.replace(xe, "$1"));
                return da[Ei] ? uu(function(Ji, Pa, Ga, Ha) {
                  for (var Ba, ao = da(Ji, null, Ha, []), io = Ji.length; io--; )
                    (Ba = ao[io]) && (Ji[io] = !(Pa[io] = Ba));
                }) : function(Ji, Pa, Ga) {
                  return Ri[0] = Ji, da(Ri, null, Ga, ta), Ri[0] = null, !ta.pop();
                };
              }),
              has: uu(function(vi) {
                return function(Ri) {
                  return Hl(vi, Ri).length > 0;
                };
              }),
              contains: uu(function(vi) {
                return vi = vi.replace(Gu, Ku), function(Ri) {
                  return (Ri.textContent || ge.text(Ri)).indexOf(vi) > -1;
                };
              }),
              // "Whether an element is represented by a :lang() selector
              // is based solely on the element's language value
              // being equal to the identifier C,
              // or beginning with the identifier C immediately followed by "-".
              // The matching of C against the element's language value is performed case-insensitively.
              // The identifier C does not have to be a valid language name."
              // https://www.w3.org/TR/selectors/#lang-pseudo
              lang: uu(function(vi) {
                return Cu.test(vi || "") || Hl.error("unsupported lang: " + vi), vi = vi.replace(Gu, Ku).toLowerCase(), function(Ri) {
                  var ta;
                  do
                    if (ta = Fr ? Ri.lang : Ri.getAttribute("xml:lang") || Ri.getAttribute("lang"))
                      return ta = ta.toLowerCase(), ta === vi || ta.indexOf(vi + "-") === 0;
                  while ((Ri = Ri.parentNode) && Ri.nodeType === 1);
                  return !1;
                };
              }),
              // Miscellaneous
              target: function(vi) {
                var Ri = _.location && _.location.hash;
                return Ri && Ri.slice(1) === vi.id;
              },
              root: function(vi) {
                return vi === ai;
              },
              focus: function(vi) {
                return vi === D1() && yr.hasFocus() && !!(vi.type || vi.href || ~vi.tabIndex);
              },
              // Boolean properties
              enabled: zl(!1),
              disabled: zl(!0),
              checked: function(vi) {
                return be(vi, "input") && !!vi.checked || be(vi, "option") && !!vi.selected;
              },
              selected: function(vi) {
                return vi.parentNode && vi.parentNode.selectedIndex, vi.selected === !0;
              },
              // Contents
              empty: function(vi) {
                for (vi = vi.firstChild; vi; vi = vi.nextSibling)
                  if (vi.nodeType < 6)
                    return !1;
                return !0;
              },
              parent: function(vi) {
                return !Tn.pseudos.empty(vi);
              },
              // Element/input types
              header: function(vi) {
                return zi.test(vi.nodeName);
              },
              input: function(vi) {
                return Kc.test(vi.nodeName);
              },
              button: function(vi) {
                return be(vi, "input") && vi.type === "button" || be(vi, "button");
              },
              text: function(vi) {
                var Ri;
                return be(vi, "input") && vi.type === "text" && // Support: IE <10 only
                // New HTML5 attribute values (e.g., "search") appear
                // with elem.type === "text"
                ((Ri = vi.getAttribute("type")) == null || Ri.toLowerCase() === "text");
              },
              // Position-in-collection
              first: os(function() {
                return [0];
              }),
              last: os(function(vi, Ri) {
                return [Ri - 1];
              }),
              eq: os(function(vi, Ri, ta) {
                return [ta < 0 ? ta + Ri : ta];
              }),
              even: os(function(vi, Ri) {
                for (var ta = 0; ta < Ri; ta += 2)
                  vi.push(ta);
                return vi;
              }),
              odd: os(function(vi, Ri) {
                for (var ta = 1; ta < Ri; ta += 2)
                  vi.push(ta);
                return vi;
              }),
              lt: os(function(vi, Ri, ta) {
                var da;
                for (ta < 0 ? da = ta + Ri : ta > Ri ? da = Ri : da = ta; --da >= 0; )
                  vi.push(da);
                return vi;
              }),
              gt: os(function(vi, Ri, ta) {
                for (var da = ta < 0 ? ta + Ri : ta; ++da < Ri; )
                  vi.push(da);
                return vi;
              })
            }
          }, Tn.pseudos.nth = Tn.pseudos.eq;
          for (nn in { radio: !0, checkbox: !0, file: !0, password: !0, image: !0 })
            Tn.pseudos[nn] = b1(nn);
          for (nn in { submit: !0, reset: !0 })
            Tn.pseudos[nn] = Wl(nn);
          function so() {
          }
          so.prototype = Tn.filters = Tn.pseudos, Tn.setFilters = new so();
          function So(vi, Ri) {
            var ta, da, Ji, Pa, Ga, Ha, Ba, ao = Al[vi + " "];
            if (ao)
              return Ri ? 0 : ao.slice(0);
            for (Ga = vi, Ha = [], Ba = Tn.preFilter; Ga; ) {
              (!ta || (da = Xl.exec(Ga))) && (da && (Ga = Ga.slice(da[0].length) || Ga), Ha.push(Ji = [])), ta = !1, (da = yc.exec(Ga)) && (ta = da.shift(), Ji.push({
                value: ta,
                // Cast descendant combinators to space
                type: da[0].replace(xe, " ")
              }), Ga = Ga.slice(ta.length));
              for (Pa in Tn.filter)
                (da = zc[Pa].exec(Ga)) && (!Ba[Pa] || (da = Ba[Pa](da))) && (ta = da.shift(), Ji.push({
                  value: ta,
                  type: Pa,
                  matches: da
                }), Ga = Ga.slice(ta.length));
              if (!ta)
                break;
            }
            return Ri ? Ga.length : Ga ? Hl.error(vi) : (
              // Cache the tokens
              Al(vi, Ha).slice(0)
            );
          }
          function Jc(vi) {
            for (var Ri = 0, ta = vi.length, da = ""; Ri < ta; Ri++)
              da += vi[Ri].value;
            return da;
          }
          function No(vi, Ri, ta) {
            var da = Ri.dir, Ji = Ri.next, Pa = Ji || da, Ga = ta && Pa === "parentNode", Ha = $o++;
            return Ri.first ? (
              // Check against closest ancestor/preceding element
              function(Ba, ao, io) {
                for (; Ba = Ba[da]; )
                  if (Ba.nodeType === 1 || Ga)
                    return vi(Ba, ao, io);
                return !1;
              }
            ) : (
              // Check against all ancestor/preceding elements
              function(Ba, ao, io) {
                var To, Mo, po = [Fa, Ha];
                if (io) {
                  for (; Ba = Ba[da]; )
                    if ((Ba.nodeType === 1 || Ga) && vi(Ba, ao, io))
                      return !0;
                } else
                  for (; Ba = Ba[da]; )
                    if (Ba.nodeType === 1 || Ga)
                      if (Mo = Ba[Ei] || (Ba[Ei] = {}), Ji && be(Ba, Ji))
                        Ba = Ba[da] || Ba;
                      else {
                        if ((To = Mo[Pa]) && To[0] === Fa && To[1] === Ha)
                          return po[2] = To[2];
                        if (Mo[Pa] = po, po[2] = vi(Ba, ao, io))
                          return !0;
                      }
                return !1;
              }
            );
          }
          function ro(vi) {
            return vi.length > 1 ? function(Ri, ta, da) {
              for (var Ji = vi.length; Ji--; )
                if (!vi[Ji](Ri, ta, da))
                  return !1;
              return !0;
            } : vi[0];
          }
          function Io(vi, Ri, ta) {
            for (var da = 0, Ji = Ri.length; da < Ji; da++)
              Hl(vi, Ri[da], ta);
            return ta;
          }
          function bs(vi, Ri, ta, da, Ji) {
            for (var Pa, Ga = [], Ha = 0, Ba = vi.length, ao = Ri != null; Ha < Ba; Ha++)
              (Pa = vi[Ha]) && (!ta || ta(Pa, da, Ji)) && (Ga.push(Pa), ao && Ri.push(Ha));
            return Ga;
          }
          function ws(vi, Ri, ta, da, Ji, Pa) {
            return da && !da[Ei] && (da = ws(da)), Ji && !Ji[Ei] && (Ji = ws(Ji, Pa)), uu(function(Ga, Ha, Ba, ao) {
              var io, To, Mo, po, Vo = [], Xo = [], Go = Ha.length, es = Ga || Io(
                Ri || "*",
                Ba.nodeType ? [Ba] : Ba,
                []
              ), Ts = vi && (Ga || !Ri) ? bs(es, Vo, vi, Ba, ao) : es;
              if (ta ? (po = Ji || (Ga ? vi : Go || da) ? (
                // ...intermediate processing is necessary
                []
              ) : (
                // ...otherwise use results directly
                Ha
              ), ta(Ts, po, Ba, ao)) : po = Ts, da)
                for (io = bs(po, Xo), da(io, [], Ba, ao), To = io.length; To--; )
                  (Mo = io[To]) && (po[Xo[To]] = !(Ts[Xo[To]] = Mo));
              if (Ga) {
                if (Ji || vi) {
                  if (Ji) {
                    for (io = [], To = po.length; To--; )
                      (Mo = po[To]) && io.push(Ts[To] = Mo);
                    Ji(null, po = [], io, ao);
                  }
                  for (To = po.length; To--; )
                    (Mo = po[To]) && (io = Ji ? te.call(Ga, Mo) : Vo[To]) > -1 && (Ga[io] = !(Ha[io] = Mo));
                }
              } else
                po = bs(
                  po === Ha ? po.splice(Go, po.length) : po
                ), Ji ? Ji(null, Ha, po, ao) : dr.apply(Ha, po);
            });
          }
          function zo(vi) {
            for (var Ri, ta, da, Ji = vi.length, Pa = Tn.relative[vi[0].type], Ga = Pa || Tn.relative[" "], Ha = Pa ? 1 : 0, Ba = No(function(To) {
              return To === Ri;
            }, Ga, !0), ao = No(function(To) {
              return te.call(Ri, To) > -1;
            }, Ga, !0), io = [function(To, Mo, po) {
              var Vo = !Pa && (po || Mo != Dn) || ((Ri = Mo).nodeType ? Ba(To, Mo, po) : ao(To, Mo, po));
              return Ri = null, Vo;
            }]; Ha < Ji; Ha++)
              if (ta = Tn.relative[vi[Ha].type])
                io = [No(ro(io), ta)];
              else {
                if (ta = Tn.filter[vi[Ha].type].apply(null, vi[Ha].matches), ta[Ei]) {
                  for (da = ++Ha; da < Ji && !Tn.relative[vi[da].type]; da++)
                    ;
                  return ws(
                    Ha > 1 && ro(io),
                    Ha > 1 && Jc(
                      // If the preceding token was a descendant combinator, insert an implicit any-element `*`
                      vi.slice(0, Ha - 1).concat({ value: vi[Ha - 2].type === " " ? "*" : "" })
                    ).replace(xe, "$1"),
                    ta,
                    Ha < da && zo(vi.slice(Ha, da)),
                    da < Ji && zo(vi = vi.slice(da)),
                    da < Ji && Jc(vi)
                  );
                }
                io.push(ta);
              }
            return ro(io);
          }
          function Yo(vi, Ri) {
            var ta = Ri.length > 0, da = vi.length > 0, Ji = function(Pa, Ga, Ha, Ba, ao) {
              var io, To, Mo, po = 0, Vo = "0", Xo = Pa && [], Go = [], es = Dn, Ts = Pa || da && Tn.find.TAG("*", ao), Fo = Fa += es == null ? 1 : Math.random() || 0.1, Oo = Ts.length;
              for (ao && (Dn = Ga == yr || Ga || ao); Vo !== Oo && (io = Ts[Vo]) != null; Vo++) {
                if (da && io) {
                  for (To = 0, !Ga && io.ownerDocument != yr && (uo(io), Ha = !Fr); Mo = vi[To++]; )
                    if (Mo(io, Ga || yr, Ha)) {
                      dr.call(Ba, io);
                      break;
                    }
                  ao && (Fa = Fo);
                }
                ta && ((io = !Mo && io) && po--, Pa && Xo.push(io));
              }
              if (po += Vo, ta && Vo !== po) {
                for (To = 0; Mo = Ri[To++]; )
                  Mo(Xo, Go, Ga, Ha);
                if (Pa) {
                  if (po > 0)
                    for (; Vo--; )
                      Xo[Vo] || Go[Vo] || (Go[Vo] = Ce.call(Ba));
                  Go = bs(Go);
                }
                dr.apply(Ba, Go), ao && !Pa && Go.length > 0 && po + Ri.length > 1 && ge.uniqueSort(Ba);
              }
              return ao && (Fa = Fo, Dn = es), Xo;
            };
            return ta ? uu(Ji) : Ji;
          }
          function Ls(vi, Ri) {
            var ta, da = [], Ji = [], Pa = ul[vi + " "];
            if (!Pa) {
              for (Ri || (Ri = So(vi)), ta = Ri.length; ta--; )
                Pa = zo(Ri[ta]), Pa[Ei] ? da.push(Pa) : Ji.push(Pa);
              Pa = ul(
                vi,
                Yo(Ji, da)
              ), Pa.selector = vi;
            }
            return Pa;
          }
          function Ms(vi, Ri, ta, da) {
            var Ji, Pa, Ga, Ha, Ba, ao = typeof vi == "function" && vi, io = !da && So(vi = ao.selector || vi);
            if (ta = ta || [], io.length === 1) {
              if (Pa = io[0] = io[0].slice(0), Pa.length > 2 && (Ga = Pa[0]).type === "ID" && Ri.nodeType === 9 && Fr && Tn.relative[Pa[1].type]) {
                if (Ri = (Tn.find.ID(
                  Ga.matches[0].replace(Gu, Ku),
                  Ri
                ) || [])[0], Ri)
                  ao && (Ri = Ri.parentNode);
                else return ta;
                vi = vi.slice(Pa.shift().value.length);
              }
              for (Ji = zc.needsContext.test(vi) ? 0 : Pa.length; Ji-- && (Ga = Pa[Ji], !Tn.relative[Ha = Ga.type]); )
                if ((Ba = Tn.find[Ha]) && (da = Ba(
                  Ga.matches[0].replace(Gu, Ku),
                  d1.test(Pa[0].type) && jo(Ri.parentNode) || Ri
                ))) {
                  if (Pa.splice(Ji, 1), vi = da.length && Jc(Pa), !vi)
                    return dr.apply(ta, da), ta;
                  break;
                }
            }
            return (ao || Ls(vi, io))(
              da,
              Ri,
              !Fr,
              ta,
              !Ri || d1.test(vi) && jo(Ri.parentNode) || Ri
            ), ta;
          }
          ce.sortStable = Ei.split("").sort(Gl).join("") === Ei, uo(), ce.sortDetached = Hs(function(vi) {
            return vi.compareDocumentPosition(yr.createElement("fieldset")) & 1;
          }), ge.find = Hl, ge.expr[":"] = ge.expr.pseudos, ge.unique = ge.uniqueSort, Hl.compile = Ls, Hl.select = Ms, Hl.setDocument = uo, Hl.tokenize = So, Hl.escape = ge.escapeSelector, Hl.getText = ge.text, Hl.isXML = ge.isXMLDoc, Hl.selectors = ge.expr, Hl.support = ge.support, Hl.uniqueSort = ge.uniqueSort;
        })();
        var Re = function(nn, Tn, Dn) {
          for (var Fn = [], fr = Dn !== void 0; (nn = nn[Tn]) && nn.nodeType !== 9; )
            if (nn.nodeType === 1) {
              if (fr && ge(nn).is(Dn))
                break;
              Fn.push(nn);
            }
          return Fn;
        }, De = function(nn, Tn) {
          for (var Dn = []; nn; nn = nn.nextSibling)
            nn.nodeType === 1 && nn !== Tn && Dn.push(nn);
          return Dn;
        }, Pe = ge.expr.match.needsContext, $e = /^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i;
        function qe(nn, Tn, Dn) {
          return le(Tn) ? ge.grep(nn, function(Fn, fr) {
            return !!Tn.call(Fn, fr, Fn) !== Dn;
          }) : Tn.nodeType ? ge.grep(nn, function(Fn) {
            return Fn === Tn !== Dn;
          }) : typeof Tn != "string" ? ge.grep(nn, function(Fn) {
            return te.call(Tn, Fn) > -1 !== Dn;
          }) : ge.filter(Tn, nn, Dn);
        }
        ge.filter = function(nn, Tn, Dn) {
          var Fn = Tn[0];
          return Dn && (nn = ":not(" + nn + ")"), Tn.length === 1 && Fn.nodeType === 1 ? ge.find.matchesSelector(Fn, nn) ? [Fn] : [] : ge.find.matches(nn, ge.grep(Tn, function(fr) {
            return fr.nodeType === 1;
          }));
        }, ge.fn.extend({
          find: function(nn) {
            var Tn, Dn, Fn = this.length, fr = this;
            if (typeof nn != "string")
              return this.pushStack(ge(nn).filter(function() {
                for (Tn = 0; Tn < Fn; Tn++)
                  if (ge.contains(fr[Tn], this))
                    return !0;
              }));
            for (Dn = this.pushStack([]), Tn = 0; Tn < Fn; Tn++)
              ge.find(nn, fr[Tn], Dn);
            return Fn > 1 ? ge.uniqueSort(Dn) : Dn;
          },
          filter: function(nn) {
            return this.pushStack(qe(this, nn || [], !1));
          },
          not: function(nn) {
            return this.pushStack(qe(this, nn || [], !0));
          },
          is: function(nn) {
            return !!qe(
              this,
              // If this is a positional/relative selector, check membership in the returned set
              // so $("p:first").is("p:last") won't return true for a doc with two "p".
              typeof nn == "string" && Pe.test(nn) ? ge(nn) : nn || [],
              !1
            ).length;
          }
        });
        var Xe, Ve = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/, Ze = ge.fn.init = function(nn, Tn, Dn) {
          var Fn, fr;
          if (!nn)
            return this;
          if (Dn = Dn || Xe, typeof nn == "string")
            if (nn[0] === "<" && nn[nn.length - 1] === ">" && nn.length >= 3 ? Fn = [null, nn, null] : Fn = Ve.exec(nn), Fn && (Fn[1] || !Tn))
              if (Fn[1]) {
                if (Tn = Tn instanceof ge ? Tn[0] : Tn, ge.merge(this, ge.parseHTML(
                  Fn[1],
                  Tn && Tn.nodeType ? Tn.ownerDocument || Tn : de,
                  !0
                )), $e.test(Fn[1]) && ge.isPlainObject(Tn))
                  for (Fn in Tn)
                    le(this[Fn]) ? this[Fn](Tn[Fn]) : this.attr(Fn, Tn[Fn]);
                return this;
              } else
                return fr = de.getElementById(Fn[2]), fr && (this[0] = fr, this.length = 1), this;
            else return !Tn || Tn.jquery ? (Tn || Dn).find(nn) : this.constructor(Tn).find(nn);
          else {
            if (nn.nodeType)
              return this[0] = nn, this.length = 1, this;
            if (le(nn))
              return Dn.ready !== void 0 ? Dn.ready(nn) : (
                // Execute immediately if ready is not present
                nn(ge)
              );
          }
          return ge.makeArray(nn, this);
        };
        Ze.prototype = ge.fn, Xe = ge(de);
        var Ge = /^(?:parents|prev(?:Until|All))/, ze = {
          children: !0,
          contents: !0,
          next: !0,
          prev: !0
        };
        ge.fn.extend({
          has: function(nn) {
            var Tn = ge(nn, this), Dn = Tn.length;
            return this.filter(function() {
              for (var Fn = 0; Fn < Dn; Fn++)
                if (ge.contains(this, Tn[Fn]))
                  return !0;
            });
          },
          closest: function(nn, Tn) {
            var Dn, Fn = 0, fr = this.length, dr = [], yr = typeof nn != "string" && ge(nn);
            if (!Pe.test(nn)) {
              for (; Fn < fr; Fn++)
                for (Dn = this[Fn]; Dn && Dn !== Tn; Dn = Dn.parentNode)
                  if (Dn.nodeType < 11 && (yr ? yr.index(Dn) > -1 : (
                    // Don't pass non-elements to jQuery#find
                    Dn.nodeType === 1 && ge.find.matchesSelector(Dn, nn)
                  ))) {
                    dr.push(Dn);
                    break;
                  }
            }
            return this.pushStack(dr.length > 1 ? ge.uniqueSort(dr) : dr);
          },
          // Determine the position of an element within the set
          index: function(nn) {
            return nn ? typeof nn == "string" ? te.call(ge(nn), this[0]) : te.call(
              this,
              // If it receives a jQuery object, the first element is used
              nn.jquery ? nn[0] : nn
            ) : this[0] && this[0].parentNode ? this.first().prevAll().length : -1;
          },
          add: function(nn, Tn) {
            return this.pushStack(
              ge.uniqueSort(
                ge.merge(this.get(), ge(nn, Tn))
              )
            );
          },
          addBack: function(nn) {
            return this.add(
              nn == null ? this.prevObject : this.prevObject.filter(nn)
            );
          }
        });
        function We(nn, Tn) {
          for (; (nn = nn[Tn]) && nn.nodeType !== 1; )
            ;
          return nn;
        }
        ge.each({
          parent: function(nn) {
            var Tn = nn.parentNode;
            return Tn && Tn.nodeType !== 11 ? Tn : null;
          },
          parents: function(nn) {
            return Re(nn, "parentNode");
          },
          parentsUntil: function(nn, Tn, Dn) {
            return Re(nn, "parentNode", Dn);
          },
          next: function(nn) {
            return We(nn, "nextSibling");
          },
          prev: function(nn) {
            return We(nn, "previousSibling");
          },
          nextAll: function(nn) {
            return Re(nn, "nextSibling");
          },
          prevAll: function(nn) {
            return Re(nn, "previousSibling");
          },
          nextUntil: function(nn, Tn, Dn) {
            return Re(nn, "nextSibling", Dn);
          },
          prevUntil: function(nn, Tn, Dn) {
            return Re(nn, "previousSibling", Dn);
          },
          siblings: function(nn) {
            return De((nn.parentNode || {}).firstChild, nn);
          },
          children: function(nn) {
            return De(nn.firstChild);
          },
          contents: function(nn) {
            return nn.contentDocument != null && // Support: IE 11+
            // <object> elements with no `data` attribute has an object
            // `contentDocument` with a `null` prototype.
            q(nn.contentDocument) ? nn.contentDocument : (be(nn, "template") && (nn = nn.content || nn), ge.merge([], nn.childNodes));
          }
        }, function(nn, Tn) {
          ge.fn[nn] = function(Dn, Fn) {
            var fr = ge.map(this, Tn, Dn);
            return nn.slice(-5) !== "Until" && (Fn = Dn), Fn && typeof Fn == "string" && (fr = ge.filter(Fn, fr)), this.length > 1 && (ze[nn] || ge.uniqueSort(fr), Ge.test(nn) && fr.reverse()), this.pushStack(fr);
          };
        });
        var at = /[^\x20\t\r\n\f]+/g;
        function ct(nn) {
          var Tn = {};
          return ge.each(nn.match(at) || [], function(Dn, Fn) {
            Tn[Fn] = !0;
          }), Tn;
        }
        ge.Callbacks = function(nn) {
          nn = typeof nn == "string" ? ct(nn) : ge.extend({}, nn);
          var Tn, Dn, Fn, fr, dr = [], yr = [], ai = -1, Fr = function() {
            for (fr = fr || nn.once, Fn = Tn = !0; yr.length; ai = -1)
              for (Dn = yr.shift(); ++ai < dr.length; )
                dr[ai].apply(Dn[0], Dn[1]) === !1 && nn.stopOnFalse && (ai = dr.length, Dn = !1);
            nn.memory || (Dn = !1), Tn = !1, fr && (Dn ? dr = [] : dr = "");
          }, Ti = {
            // Add a callback or a collection of callbacks to the list
            add: function() {
              return dr && (Dn && !Tn && (ai = dr.length - 1, yr.push(Dn)), function Si(Ei) {
                ge.each(Ei, function(Fa, $o) {
                  le($o) ? (!nn.unique || !Ti.has($o)) && dr.push($o) : $o && $o.length && me($o) !== "string" && Si($o);
                });
              }(arguments), Dn && !Tn && Fr()), this;
            },
            // Remove a callback from the list
            remove: function() {
              return ge.each(arguments, function(Si, Ei) {
                for (var Fa; (Fa = ge.inArray(Ei, dr, Fa)) > -1; )
                  dr.splice(Fa, 1), Fa <= ai && ai--;
              }), this;
            },
            // Check if a given callback is in the list.
            // If no argument is given, return whether or not list has callbacks attached.
            has: function(Si) {
              return Si ? ge.inArray(Si, dr) > -1 : dr.length > 0;
            },
            // Remove all callbacks from the list
            empty: function() {
              return dr && (dr = []), this;
            },
            // Disable .fire and .add
            // Abort any current/pending executions
            // Clear all callbacks and values
            disable: function() {
              return fr = yr = [], dr = Dn = "", this;
            },
            disabled: function() {
              return !dr;
            },
            // Disable .fire
            // Also disable .add unless we have memory (since it would have no effect)
            // Abort any pending executions
            lock: function() {
              return fr = yr = [], !Dn && !Tn && (dr = Dn = ""), this;
            },
            locked: function() {
              return !!fr;
            },
            // Call all callbacks with the given context and arguments
            fireWith: function(Si, Ei) {
              return fr || (Ei = Ei || [], Ei = [Si, Ei.slice ? Ei.slice() : Ei], yr.push(Ei), Tn || Fr()), this;
            },
            // Call all the callbacks with the given arguments
            fire: function() {
              return Ti.fireWith(this, arguments), this;
            },
            // To know if the callbacks have already been called at least once
            fired: function() {
              return !!Fn;
            }
          };
          return Ti;
        };
        function et(nn) {
          return nn;
        }
        function dt(nn) {
          throw nn;
        }
        function Je(nn, Tn, Dn, Fn) {
          var fr;
          try {
            nn && le(fr = nn.promise) ? fr.call(nn).done(Tn).fail(Dn) : nn && le(fr = nn.then) ? fr.call(nn, Tn, Dn) : Tn.apply(void 0, [nn].slice(Fn));
          } catch (dr) {
            Dn.apply(void 0, [dr]);
          }
        }
        ge.extend({
          Deferred: function(nn) {
            var Tn = [
              // action, add listener, callbacks,
              // ... .then handlers, argument index, [final state]
              [
                "notify",
                "progress",
                ge.Callbacks("memory"),
                ge.Callbacks("memory"),
                2
              ],
              [
                "resolve",
                "done",
                ge.Callbacks("once memory"),
                ge.Callbacks("once memory"),
                0,
                "resolved"
              ],
              [
                "reject",
                "fail",
                ge.Callbacks("once memory"),
                ge.Callbacks("once memory"),
                1,
                "rejected"
              ]
            ], Dn = "pending", Fn = {
              state: function() {
                return Dn;
              },
              always: function() {
                return fr.done(arguments).fail(arguments), this;
              },
              catch: function(dr) {
                return Fn.then(null, dr);
              },
              // Keep pipe for back-compat
              pipe: function() {
                var dr = arguments;
                return ge.Deferred(function(yr) {
                  ge.each(Tn, function(ai, Fr) {
                    var Ti = le(dr[Fr[4]]) && dr[Fr[4]];
                    fr[Fr[1]](function() {
                      var Si = Ti && Ti.apply(this, arguments);
                      Si && le(Si.promise) ? Si.promise().progress(yr.notify).done(yr.resolve).fail(yr.reject) : yr[Fr[0] + "With"](
                        this,
                        Ti ? [Si] : arguments
                      );
                    });
                  }), dr = null;
                }).promise();
              },
              then: function(dr, yr, ai) {
                var Fr = 0;
                function Ti(Si, Ei, Fa, $o) {
                  return function() {
                    var Gs = this, Al = arguments, ul = function() {
                      var Gl, Sl;
                      if (!(Si < Fr)) {
                        if (Gl = Fa.apply(Gs, Al), Gl === Ei.promise())
                          throw new TypeError("Thenable self-resolution");
                        Sl = Gl && // Support: Promises/A+ section 2.3.4
                        // https://promisesaplus.com/#point-64
                        // Only check objects and functions for thenability
                        (typeof Gl == "object" || typeof Gl == "function") && Gl.then, le(Sl) ? $o ? Sl.call(
                          Gl,
                          Ti(Fr, Ei, et, $o),
                          Ti(Fr, Ei, dt, $o)
                        ) : (Fr++, Sl.call(
                          Gl,
                          Ti(Fr, Ei, et, $o),
                          Ti(Fr, Ei, dt, $o),
                          Ti(
                            Fr,
                            Ei,
                            et,
                            Ei.notifyWith
                          )
                        )) : (Fa !== et && (Gs = void 0, Al = [Gl]), ($o || Ei.resolveWith)(Gs, Al));
                      }
                    }, gc = $o ? ul : function() {
                      try {
                        ul();
                      } catch (Gl) {
                        ge.Deferred.exceptionHook && ge.Deferred.exceptionHook(
                          Gl,
                          gc.error
                        ), Si + 1 >= Fr && (Fa !== dt && (Gs = void 0, Al = [Gl]), Ei.rejectWith(Gs, Al));
                      }
                    };
                    Si ? gc() : (ge.Deferred.getErrorHook ? gc.error = ge.Deferred.getErrorHook() : ge.Deferred.getStackHook && (gc.error = ge.Deferred.getStackHook()), _.setTimeout(gc));
                  };
                }
                return ge.Deferred(function(Si) {
                  Tn[0][3].add(
                    Ti(
                      0,
                      Si,
                      le(ai) ? ai : et,
                      Si.notifyWith
                    )
                  ), Tn[1][3].add(
                    Ti(
                      0,
                      Si,
                      le(dr) ? dr : et
                    )
                  ), Tn[2][3].add(
                    Ti(
                      0,
                      Si,
                      le(yr) ? yr : dt
                    )
                  );
                }).promise();
              },
              // Get a promise for this deferred
              // If obj is provided, the promise aspect is added to the object
              promise: function(dr) {
                return dr != null ? ge.extend(dr, Fn) : Fn;
              }
            }, fr = {};
            return ge.each(Tn, function(dr, yr) {
              var ai = yr[2], Fr = yr[5];
              Fn[yr[1]] = ai.add, Fr && ai.add(
                function() {
                  Dn = Fr;
                },
                // rejected_callbacks.disable
                // fulfilled_callbacks.disable
                Tn[3 - dr][2].disable,
                // rejected_handlers.disable
                // fulfilled_handlers.disable
                Tn[3 - dr][3].disable,
                // progress_callbacks.lock
                Tn[0][2].lock,
                // progress_handlers.lock
                Tn[0][3].lock
              ), ai.add(yr[3].fire), fr[yr[0]] = function() {
                return fr[yr[0] + "With"](this === fr ? void 0 : this, arguments), this;
              }, fr[yr[0] + "With"] = ai.fireWith;
            }), Fn.promise(fr), nn && nn.call(fr, fr), fr;
          },
          // Deferred helper
          when: function(nn) {
            var Tn = arguments.length, Dn = Tn, Fn = Array(Dn), fr = G.call(arguments), dr = ge.Deferred(), yr = function(ai) {
              return function(Fr) {
                Fn[ai] = this, fr[ai] = arguments.length > 1 ? G.call(arguments) : Fr, --Tn || dr.resolveWith(Fn, fr);
              };
            };
            if (Tn <= 1 && (Je(
              nn,
              dr.done(yr(Dn)).resolve,
              dr.reject,
              !Tn
            ), dr.state() === "pending" || le(fr[Dn] && fr[Dn].then)))
              return dr.then();
            for (; Dn--; )
              Je(fr[Dn], yr(Dn), dr.reject);
            return dr.promise();
          }
        });
        var it = /^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;
        ge.Deferred.exceptionHook = function(nn, Tn) {
          _.console && _.console.warn && nn && it.test(nn.name) && _.console.warn(
            "jQuery.Deferred exception: " + nn.message,
            nn.stack,
            Tn
          );
        }, ge.readyException = function(nn) {
          _.setTimeout(function() {
            throw nn;
          });
        };
        var st = ge.Deferred();
        ge.fn.ready = function(nn) {
          return st.then(nn).catch(function(Tn) {
            ge.readyException(Tn);
          }), this;
        }, ge.extend({
          // Is the DOM ready to be used? Set to true once it occurs.
          isReady: !1,
          // A counter to track how many items to wait for before
          // the ready event fires. See trac-6781
          readyWait: 1,
          // Handle when the DOM is ready
          ready: function(nn) {
            (nn === !0 ? --ge.readyWait : ge.isReady) || (ge.isReady = !0, !(nn !== !0 && --ge.readyWait > 0) && st.resolveWith(de, [ge]));
          }
        }), ge.ready.then = st.then;
        function gt() {
          de.removeEventListener("DOMContentLoaded", gt), _.removeEventListener("load", gt), ge.ready();
        }
        de.readyState === "complete" || de.readyState !== "loading" && !de.documentElement.doScroll ? _.setTimeout(ge.ready) : (de.addEventListener("DOMContentLoaded", gt), _.addEventListener("load", gt));
        var bt = function(nn, Tn, Dn, Fn, fr, dr, yr) {
          var ai = 0, Fr = nn.length, Ti = Dn == null;
          if (me(Dn) === "object") {
            fr = !0;
            for (ai in Dn)
              bt(nn, Tn, ai, Dn[ai], !0, dr, yr);
          } else if (Fn !== void 0 && (fr = !0, le(Fn) || (yr = !0), Ti && (yr ? (Tn.call(nn, Fn), Tn = null) : (Ti = Tn, Tn = function(Si, Ei, Fa) {
            return Ti.call(ge(Si), Fa);
          })), Tn))
            for (; ai < Fr; ai++)
              Tn(
                nn[ai],
                Dn,
                yr ? Fn : Fn.call(nn[ai], ai, Tn(nn[ai], Dn))
              );
          return fr ? nn : Ti ? Tn.call(nn) : Fr ? Tn(nn[0], Dn) : dr;
        }, yt = /^-ms-/, St = /-([a-z])/g;
        function Lt(nn, Tn) {
          return Tn.toUpperCase();
        }
        function Rt(nn) {
          return nn.replace(yt, "ms-").replace(St, Lt);
        }
        var vt = function(nn) {
          return nn.nodeType === 1 || nn.nodeType === 9 || !+nn.nodeType;
        };
        function $t() {
          this.expando = ge.expando + $t.uid++;
        }
        $t.uid = 1, $t.prototype = {
          cache: function(nn) {
            var Tn = nn[this.expando];
            return Tn || (Tn = {}, vt(nn) && (nn.nodeType ? nn[this.expando] = Tn : Object.defineProperty(nn, this.expando, {
              value: Tn,
              configurable: !0
            }))), Tn;
          },
          set: function(nn, Tn, Dn) {
            var Fn, fr = this.cache(nn);
            if (typeof Tn == "string")
              fr[Rt(Tn)] = Dn;
            else
              for (Fn in Tn)
                fr[Rt(Fn)] = Tn[Fn];
            return fr;
          },
          get: function(nn, Tn) {
            return Tn === void 0 ? this.cache(nn) : (
              // Always use camelCase key (gh-2257)
              nn[this.expando] && nn[this.expando][Rt(Tn)]
            );
          },
          access: function(nn, Tn, Dn) {
            return Tn === void 0 || Tn && typeof Tn == "string" && Dn === void 0 ? this.get(nn, Tn) : (this.set(nn, Tn, Dn), Dn !== void 0 ? Dn : Tn);
          },
          remove: function(nn, Tn) {
            var Dn, Fn = nn[this.expando];
            if (Fn !== void 0) {
              if (Tn !== void 0)
                for (Array.isArray(Tn) ? Tn = Tn.map(Rt) : (Tn = Rt(Tn), Tn = Tn in Fn ? [Tn] : Tn.match(at) || []), Dn = Tn.length; Dn--; )
                  delete Fn[Tn[Dn]];
              (Tn === void 0 || ge.isEmptyObject(Fn)) && (nn.nodeType ? nn[this.expando] = void 0 : delete nn[this.expando]);
            }
          },
          hasData: function(nn) {
            var Tn = nn[this.expando];
            return Tn !== void 0 && !ge.isEmptyObject(Tn);
          }
        };
        var Pt = new $t(), Xt = new $t(), Nt = /^(?:\{[\w\W]*\}|\[[\w\W]*\])$/, Gt = /[A-Z]/g;
        function Ft(nn) {
          return nn === "true" ? !0 : nn === "false" ? !1 : nn === "null" ? null : nn === +nn + "" ? +nn : Nt.test(nn) ? JSON.parse(nn) : nn;
        }
        function Tt(nn, Tn, Dn) {
          var Fn;
          if (Dn === void 0 && nn.nodeType === 1)
            if (Fn = "data-" + Tn.replace(Gt, "-$&").toLowerCase(), Dn = nn.getAttribute(Fn), typeof Dn == "string") {
              try {
                Dn = Ft(Dn);
              } catch {
              }
              Xt.set(nn, Tn, Dn);
            } else
              Dn = void 0;
          return Dn;
        }
        ge.extend({
          hasData: function(nn) {
            return Xt.hasData(nn) || Pt.hasData(nn);
          },
          data: function(nn, Tn, Dn) {
            return Xt.access(nn, Tn, Dn);
          },
          removeData: function(nn, Tn) {
            Xt.remove(nn, Tn);
          },
          // TODO: Now that all calls to _data and _removeData have been replaced
          // with direct calls to dataPriv methods, these can be deprecated.
          _data: function(nn, Tn, Dn) {
            return Pt.access(nn, Tn, Dn);
          },
          _removeData: function(nn, Tn) {
            Pt.remove(nn, Tn);
          }
        }), ge.fn.extend({
          data: function(nn, Tn) {
            var Dn, Fn, fr, dr = this[0], yr = dr && dr.attributes;
            if (nn === void 0) {
              if (this.length && (fr = Xt.get(dr), dr.nodeType === 1 && !Pt.get(dr, "hasDataAttrs"))) {
                for (Dn = yr.length; Dn--; )
                  yr[Dn] && (Fn = yr[Dn].name, Fn.indexOf("data-") === 0 && (Fn = Rt(Fn.slice(5)), Tt(dr, Fn, fr[Fn])));
                Pt.set(dr, "hasDataAttrs", !0);
              }
              return fr;
            }
            return typeof nn == "object" ? this.each(function() {
              Xt.set(this, nn);
            }) : bt(this, function(ai) {
              var Fr;
              if (dr && ai === void 0)
                return Fr = Xt.get(dr, nn), Fr !== void 0 || (Fr = Tt(dr, nn), Fr !== void 0) ? Fr : void 0;
              this.each(function() {
                Xt.set(this, nn, ai);
              });
            }, null, Tn, arguments.length > 1, null, !0);
          },
          removeData: function(nn) {
            return this.each(function() {
              Xt.remove(this, nn);
            });
          }
        }), ge.extend({
          queue: function(nn, Tn, Dn) {
            var Fn;
            if (nn)
              return Tn = (Tn || "fx") + "queue", Fn = Pt.get(nn, Tn), Dn && (!Fn || Array.isArray(Dn) ? Fn = Pt.access(nn, Tn, ge.makeArray(Dn)) : Fn.push(Dn)), Fn || [];
          },
          dequeue: function(nn, Tn) {
            Tn = Tn || "fx";
            var Dn = ge.queue(nn, Tn), Fn = Dn.length, fr = Dn.shift(), dr = ge._queueHooks(nn, Tn), yr = function() {
              ge.dequeue(nn, Tn);
            };
            fr === "inprogress" && (fr = Dn.shift(), Fn--), fr && (Tn === "fx" && Dn.unshift("inprogress"), delete dr.stop, fr.call(nn, yr, dr)), !Fn && dr && dr.empty.fire();
          },
          // Not public - generate a queueHooks object, or return the current one
          _queueHooks: function(nn, Tn) {
            var Dn = Tn + "queueHooks";
            return Pt.get(nn, Dn) || Pt.access(nn, Dn, {
              empty: ge.Callbacks("once memory").add(function() {
                Pt.remove(nn, [Tn + "queue", Dn]);
              })
            });
          }
        }), ge.fn.extend({
          queue: function(nn, Tn) {
            var Dn = 2;
            return typeof nn != "string" && (Tn = nn, nn = "fx", Dn--), arguments.length < Dn ? ge.queue(this[0], nn) : Tn === void 0 ? this : this.each(function() {
              var Fn = ge.queue(this, nn, Tn);
              ge._queueHooks(this, nn), nn === "fx" && Fn[0] !== "inprogress" && ge.dequeue(this, nn);
            });
          },
          dequeue: function(nn) {
            return this.each(function() {
              ge.dequeue(this, nn);
            });
          },
          clearQueue: function(nn) {
            return this.queue(nn || "fx", []);
          },
          // Get a promise resolved when queues of a certain type
          // are emptied (fx is the type by default)
          promise: function(nn, Tn) {
            var Dn, Fn = 1, fr = ge.Deferred(), dr = this, yr = this.length, ai = function() {
              --Fn || fr.resolveWith(dr, [dr]);
            };
            for (typeof nn != "string" && (Tn = nn, nn = void 0), nn = nn || "fx"; yr--; )
              Dn = Pt.get(dr[yr], nn + "queueHooks"), Dn && Dn.empty && (Fn++, Dn.empty.add(ai));
            return ai(), fr.promise(Tn);
          }
        });
        var It = /[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source, Ct = new RegExp("^(?:([+-])=|)(" + It + ")([a-z%]*)$", "i"), Kt = ["Top", "Right", "Bottom", "Left"], Vt = de.documentElement, Yt = function(nn) {
          return ge.contains(nn.ownerDocument, nn);
        }, sn = { composed: !0 };
        Vt.getRootNode && (Yt = function(nn) {
          return ge.contains(nn.ownerDocument, nn) || nn.getRootNode(sn) === nn.ownerDocument;
        });
        var hn = function(nn, Tn) {
          return nn = Tn || nn, nn.style.display === "none" || nn.style.display === "" && // Otherwise, check computed style
          // Support: Firefox <=43 - 45
          // Disconnected elements can have computed display: none, so first confirm that elem is
          // in the document.
          Yt(nn) && ge.css(nn, "display") === "none";
        };
        function Qt(nn, Tn, Dn, Fn) {
          var fr, dr, yr = 20, ai = Fn ? function() {
            return Fn.cur();
          } : function() {
            return ge.css(nn, Tn, "");
          }, Fr = ai(), Ti = Dn && Dn[3] || (ge.cssNumber[Tn] ? "" : "px"), Si = nn.nodeType && (ge.cssNumber[Tn] || Ti !== "px" && +Fr) && Ct.exec(ge.css(nn, Tn));
          if (Si && Si[3] !== Ti) {
            for (Fr = Fr / 2, Ti = Ti || Si[3], Si = +Fr || 1; yr--; )
              ge.style(nn, Tn, Si + Ti), (1 - dr) * (1 - (dr = ai() / Fr || 0.5)) <= 0 && (yr = 0), Si = Si / dr;
            Si = Si * 2, ge.style(nn, Tn, Si + Ti), Dn = Dn || [];
          }
          return Dn && (Si = +Si || +Fr || 0, fr = Dn[1] ? Si + (Dn[1] + 1) * Dn[2] : +Dn[2], Fn && (Fn.unit = Ti, Fn.start = Si, Fn.end = fr)), fr;
        }
        var Ut = {};
        function Wt(nn) {
          var Tn, Dn = nn.ownerDocument, Fn = nn.nodeName, fr = Ut[Fn];
          return fr || (Tn = Dn.body.appendChild(Dn.createElement(Fn)), fr = ge.css(Tn, "display"), Tn.parentNode.removeChild(Tn), fr === "none" && (fr = "block"), Ut[Fn] = fr, fr);
        }
        function rn(nn, Tn) {
          for (var Dn, Fn, fr = [], dr = 0, yr = nn.length; dr < yr; dr++)
            Fn = nn[dr], Fn.style && (Dn = Fn.style.display, Tn ? (Dn === "none" && (fr[dr] = Pt.get(Fn, "display") || null, fr[dr] || (Fn.style.display = "")), Fn.style.display === "" && hn(Fn) && (fr[dr] = Wt(Fn))) : Dn !== "none" && (fr[dr] = "none", Pt.set(Fn, "display", Dn)));
          for (dr = 0; dr < yr; dr++)
            fr[dr] != null && (nn[dr].style.display = fr[dr]);
          return nn;
        }
        ge.fn.extend({
          show: function() {
            return rn(this, !0);
          },
          hide: function() {
            return rn(this);
          },
          toggle: function(nn) {
            return typeof nn == "boolean" ? nn ? this.show() : this.hide() : this.each(function() {
              hn(this) ? ge(this).show() : ge(this).hide();
            });
          }
        });
        var vn = /^(?:checkbox|radio)$/i, dn = /<([a-z][^\/\0>\x20\t\r\n\f]*)/i, Ht = /^$|^module$|\/(?:java|ecma)script/i;
        (function() {
          var nn = de.createDocumentFragment(), Tn = nn.appendChild(de.createElement("div")), Dn = de.createElement("input");
          Dn.setAttribute("type", "radio"), Dn.setAttribute("checked", "checked"), Dn.setAttribute("name", "t"), Tn.appendChild(Dn), ce.checkClone = Tn.cloneNode(!0).cloneNode(!0).lastChild.checked, Tn.innerHTML = "<textarea>x</textarea>", ce.noCloneChecked = !!Tn.cloneNode(!0).lastChild.defaultValue, Tn.innerHTML = "<option></option>", ce.option = !!Tn.lastChild;
        })();
        var jt = {
          // XHTML parsers do not magically insert elements in the
          // same way that tag soup parsers do. So we cannot shorten
          // this by omitting <tbody> or other required elements.
          thead: [1, "<table>", "</table>"],
          col: [2, "<table><colgroup>", "</colgroup></table>"],
          tr: [2, "<table><tbody>", "</tbody></table>"],
          td: [3, "<table><tbody><tr>", "</tr></tbody></table>"],
          _default: [0, "", ""]
        };
        jt.tbody = jt.tfoot = jt.colgroup = jt.caption = jt.thead, jt.th = jt.td, ce.option || (jt.optgroup = jt.option = [1, "<select multiple='multiple'>", "</select>"]);
        function cn(nn, Tn) {
          var Dn;
          return typeof nn.getElementsByTagName < "u" ? Dn = nn.getElementsByTagName(Tn || "*") : typeof nn.querySelectorAll < "u" ? Dn = nn.querySelectorAll(Tn || "*") : Dn = [], Tn === void 0 || Tn && be(nn, Tn) ? ge.merge([nn], Dn) : Dn;
        }
        function wn(nn, Tn) {
          for (var Dn = 0, Fn = nn.length; Dn < Fn; Dn++)
            Pt.set(
              nn[Dn],
              "globalEval",
              !Tn || Pt.get(Tn[Dn], "globalEval")
            );
        }
        var Rn = /<|&#?\w+;/;
        function Pn(nn, Tn, Dn, Fn, fr) {
          for (var dr, yr, ai, Fr, Ti, Si, Ei = Tn.createDocumentFragment(), Fa = [], $o = 0, Gs = nn.length; $o < Gs; $o++)
            if (dr = nn[$o], dr || dr === 0)
              if (me(dr) === "object")
                ge.merge(Fa, dr.nodeType ? [dr] : dr);
              else if (!Rn.test(dr))
                Fa.push(Tn.createTextNode(dr));
              else {
                for (yr = yr || Ei.appendChild(Tn.createElement("div")), ai = (dn.exec(dr) || ["", ""])[1].toLowerCase(), Fr = jt[ai] || jt._default, yr.innerHTML = Fr[1] + ge.htmlPrefilter(dr) + Fr[2], Si = Fr[0]; Si--; )
                  yr = yr.lastChild;
                ge.merge(Fa, yr.childNodes), yr = Ei.firstChild, yr.textContent = "";
              }
          for (Ei.textContent = "", $o = 0; dr = Fa[$o++]; ) {
            if (Fn && ge.inArray(dr, Fn) > -1) {
              fr && fr.push(dr);
              continue;
            }
            if (Ti = Yt(dr), yr = cn(Ei.appendChild(dr), "script"), Ti && wn(yr), Dn)
              for (Si = 0; dr = yr[Si++]; )
                Ht.test(dr.type || "") && Dn.push(dr);
          }
          return Ei;
        }
        var Ie = /^([^.]*)(?:\.(.+)|)/;
        function rt() {
          return !0;
        }
        function ft() {
          return !1;
        }
        function pt(nn, Tn, Dn, Fn, fr, dr) {
          var yr, ai;
          if (typeof Tn == "object") {
            typeof Dn != "string" && (Fn = Fn || Dn, Dn = void 0);
            for (ai in Tn)
              pt(nn, ai, Dn, Fn, Tn[ai], dr);
            return nn;
          }
          if (Fn == null && fr == null ? (fr = Dn, Fn = Dn = void 0) : fr == null && (typeof Dn == "string" ? (fr = Fn, Fn = void 0) : (fr = Fn, Fn = Dn, Dn = void 0)), fr === !1)
            fr = ft;
          else if (!fr)
            return nn;
          return dr === 1 && (yr = fr, fr = function(Fr) {
            return ge().off(Fr), yr.apply(this, arguments);
          }, fr.guid = yr.guid || (yr.guid = ge.guid++)), nn.each(function() {
            ge.event.add(this, Tn, fr, Fn, Dn);
          });
        }
        ge.event = {
          global: {},
          add: function(nn, Tn, Dn, Fn, fr) {
            var dr, yr, ai, Fr, Ti, Si, Ei, Fa, $o, Gs, Al, ul = Pt.get(nn);
            if (vt(nn))
              for (Dn.handler && (dr = Dn, Dn = dr.handler, fr = dr.selector), fr && ge.find.matchesSelector(Vt, fr), Dn.guid || (Dn.guid = ge.guid++), (Fr = ul.events) || (Fr = ul.events = /* @__PURE__ */ Object.create(null)), (yr = ul.handle) || (yr = ul.handle = function(gc) {
                return typeof ge < "u" && ge.event.triggered !== gc.type ? ge.event.dispatch.apply(nn, arguments) : void 0;
              }), Tn = (Tn || "").match(at) || [""], Ti = Tn.length; Ti--; )
                ai = Ie.exec(Tn[Ti]) || [], $o = Al = ai[1], Gs = (ai[2] || "").split(".").sort(), $o && (Ei = ge.event.special[$o] || {}, $o = (fr ? Ei.delegateType : Ei.bindType) || $o, Ei = ge.event.special[$o] || {}, Si = ge.extend({
                  type: $o,
                  origType: Al,
                  data: Fn,
                  handler: Dn,
                  guid: Dn.guid,
                  selector: fr,
                  needsContext: fr && ge.expr.match.needsContext.test(fr),
                  namespace: Gs.join(".")
                }, dr), (Fa = Fr[$o]) || (Fa = Fr[$o] = [], Fa.delegateCount = 0, (!Ei.setup || Ei.setup.call(nn, Fn, Gs, yr) === !1) && nn.addEventListener && nn.addEventListener($o, yr)), Ei.add && (Ei.add.call(nn, Si), Si.handler.guid || (Si.handler.guid = Dn.guid)), fr ? Fa.splice(Fa.delegateCount++, 0, Si) : Fa.push(Si), ge.event.global[$o] = !0);
          },
          // Detach an event or set of events from an element
          remove: function(nn, Tn, Dn, Fn, fr) {
            var dr, yr, ai, Fr, Ti, Si, Ei, Fa, $o, Gs, Al, ul = Pt.hasData(nn) && Pt.get(nn);
            if (!(!ul || !(Fr = ul.events))) {
              for (Tn = (Tn || "").match(at) || [""], Ti = Tn.length; Ti--; ) {
                if (ai = Ie.exec(Tn[Ti]) || [], $o = Al = ai[1], Gs = (ai[2] || "").split(".").sort(), !$o) {
                  for ($o in Fr)
                    ge.event.remove(nn, $o + Tn[Ti], Dn, Fn, !0);
                  continue;
                }
                for (Ei = ge.event.special[$o] || {}, $o = (Fn ? Ei.delegateType : Ei.bindType) || $o, Fa = Fr[$o] || [], ai = ai[2] && new RegExp("(^|\\.)" + Gs.join("\\.(?:.*\\.|)") + "(\\.|$)"), yr = dr = Fa.length; dr--; )
                  Si = Fa[dr], (fr || Al === Si.origType) && (!Dn || Dn.guid === Si.guid) && (!ai || ai.test(Si.namespace)) && (!Fn || Fn === Si.selector || Fn === "**" && Si.selector) && (Fa.splice(dr, 1), Si.selector && Fa.delegateCount--, Ei.remove && Ei.remove.call(nn, Si));
                yr && !Fa.length && ((!Ei.teardown || Ei.teardown.call(nn, Gs, ul.handle) === !1) && ge.removeEvent(nn, $o, ul.handle), delete Fr[$o]);
              }
              ge.isEmptyObject(Fr) && Pt.remove(nn, "handle events");
            }
          },
          dispatch: function(nn) {
            var Tn, Dn, Fn, fr, dr, yr, ai = new Array(arguments.length), Fr = ge.event.fix(nn), Ti = (Pt.get(this, "events") || /* @__PURE__ */ Object.create(null))[Fr.type] || [], Si = ge.event.special[Fr.type] || {};
            for (ai[0] = Fr, Tn = 1; Tn < arguments.length; Tn++)
              ai[Tn] = arguments[Tn];
            if (Fr.delegateTarget = this, !(Si.preDispatch && Si.preDispatch.call(this, Fr) === !1)) {
              for (yr = ge.event.handlers.call(this, Fr, Ti), Tn = 0; (fr = yr[Tn++]) && !Fr.isPropagationStopped(); )
                for (Fr.currentTarget = fr.elem, Dn = 0; (dr = fr.handlers[Dn++]) && !Fr.isImmediatePropagationStopped(); )
                  (!Fr.rnamespace || dr.namespace === !1 || Fr.rnamespace.test(dr.namespace)) && (Fr.handleObj = dr, Fr.data = dr.data, Fn = ((ge.event.special[dr.origType] || {}).handle || dr.handler).apply(fr.elem, ai), Fn !== void 0 && (Fr.result = Fn) === !1 && (Fr.preventDefault(), Fr.stopPropagation()));
              return Si.postDispatch && Si.postDispatch.call(this, Fr), Fr.result;
            }
          },
          handlers: function(nn, Tn) {
            var Dn, Fn, fr, dr, yr, ai = [], Fr = Tn.delegateCount, Ti = nn.target;
            if (Fr && // Support: IE <=9
            // Black-hole SVG <use> instance trees (trac-13180)
            Ti.nodeType && // Support: Firefox <=42
            // Suppress spec-violating clicks indicating a non-primary pointer button (trac-3861)
            // https://www.w3.org/TR/DOM-Level-3-Events/#event-type-click
            // Support: IE 11 only
            // ...but not arrow key "clicks" of radio inputs, which can have `button` -1 (gh-2343)
            !(nn.type === "click" && nn.button >= 1)) {
              for (; Ti !== this; Ti = Ti.parentNode || this)
                if (Ti.nodeType === 1 && !(nn.type === "click" && Ti.disabled === !0)) {
                  for (dr = [], yr = {}, Dn = 0; Dn < Fr; Dn++)
                    Fn = Tn[Dn], fr = Fn.selector + " ", yr[fr] === void 0 && (yr[fr] = Fn.needsContext ? ge(fr, this).index(Ti) > -1 : ge.find(fr, this, null, [Ti]).length), yr[fr] && dr.push(Fn);
                  dr.length && ai.push({ elem: Ti, handlers: dr });
                }
            }
            return Ti = this, Fr < Tn.length && ai.push({ elem: Ti, handlers: Tn.slice(Fr) }), ai;
          },
          addProp: function(nn, Tn) {
            Object.defineProperty(ge.Event.prototype, nn, {
              enumerable: !0,
              configurable: !0,
              get: le(Tn) ? function() {
                if (this.originalEvent)
                  return Tn(this.originalEvent);
              } : function() {
                if (this.originalEvent)
                  return this.originalEvent[nn];
              },
              set: function(Dn) {
                Object.defineProperty(this, nn, {
                  enumerable: !0,
                  configurable: !0,
                  writable: !0,
                  value: Dn
                });
              }
            });
          },
          fix: function(nn) {
            return nn[ge.expando] ? nn : new ge.Event(nn);
          },
          special: {
            load: {
              // Prevent triggered image.load events from bubbling to window.load
              noBubble: !0
            },
            click: {
              // Utilize native event to ensure correct state for checkable inputs
              setup: function(nn) {
                var Tn = this || nn;
                return vn.test(Tn.type) && Tn.click && be(Tn, "input") && wt(Tn, "click", !0), !1;
              },
              trigger: function(nn) {
                var Tn = this || nn;
                return vn.test(Tn.type) && Tn.click && be(Tn, "input") && wt(Tn, "click"), !0;
              },
              // For cross-browser consistency, suppress native .click() on links
              // Also prevent it if we're currently inside a leveraged native-event stack
              _default: function(nn) {
                var Tn = nn.target;
                return vn.test(Tn.type) && Tn.click && be(Tn, "input") && Pt.get(Tn, "click") || be(Tn, "a");
              }
            },
            beforeunload: {
              postDispatch: function(nn) {
                nn.result !== void 0 && nn.originalEvent && (nn.originalEvent.returnValue = nn.result);
              }
            }
          }
        };
        function wt(nn, Tn, Dn) {
          if (!Dn) {
            Pt.get(nn, Tn) === void 0 && ge.event.add(nn, Tn, rt);
            return;
          }
          Pt.set(nn, Tn, !1), ge.event.add(nn, Tn, {
            namespace: !1,
            handler: function(Fn) {
              var fr, dr = Pt.get(this, Tn);
              if (Fn.isTrigger & 1 && this[Tn]) {
                if (dr)
                  (ge.event.special[Tn] || {}).delegateType && Fn.stopPropagation();
                else if (dr = G.call(arguments), Pt.set(this, Tn, dr), this[Tn](), fr = Pt.get(this, Tn), Pt.set(this, Tn, !1), dr !== fr)
                  return Fn.stopImmediatePropagation(), Fn.preventDefault(), fr;
              } else dr && (Pt.set(this, Tn, ge.event.trigger(
                dr[0],
                dr.slice(1),
                this
              )), Fn.stopPropagation(), Fn.isImmediatePropagationStopped = rt);
            }
          });
        }
        ge.removeEvent = function(nn, Tn, Dn) {
          nn.removeEventListener && nn.removeEventListener(Tn, Dn);
        }, ge.Event = function(nn, Tn) {
          if (!(this instanceof ge.Event))
            return new ge.Event(nn, Tn);
          nn && nn.type ? (this.originalEvent = nn, this.type = nn.type, this.isDefaultPrevented = nn.defaultPrevented || nn.defaultPrevented === void 0 && // Support: Android <=2.3 only
          nn.returnValue === !1 ? rt : ft, this.target = nn.target && nn.target.nodeType === 3 ? nn.target.parentNode : nn.target, this.currentTarget = nn.currentTarget, this.relatedTarget = nn.relatedTarget) : this.type = nn, Tn && ge.extend(this, Tn), this.timeStamp = nn && nn.timeStamp || Date.now(), this[ge.expando] = !0;
        }, ge.Event.prototype = {
          constructor: ge.Event,
          isDefaultPrevented: ft,
          isPropagationStopped: ft,
          isImmediatePropagationStopped: ft,
          isSimulated: !1,
          preventDefault: function() {
            var nn = this.originalEvent;
            this.isDefaultPrevented = rt, nn && !this.isSimulated && nn.preventDefault();
          },
          stopPropagation: function() {
            var nn = this.originalEvent;
            this.isPropagationStopped = rt, nn && !this.isSimulated && nn.stopPropagation();
          },
          stopImmediatePropagation: function() {
            var nn = this.originalEvent;
            this.isImmediatePropagationStopped = rt, nn && !this.isSimulated && nn.stopImmediatePropagation(), this.stopPropagation();
          }
        }, ge.each({
          altKey: !0,
          bubbles: !0,
          cancelable: !0,
          changedTouches: !0,
          ctrlKey: !0,
          detail: !0,
          eventPhase: !0,
          metaKey: !0,
          pageX: !0,
          pageY: !0,
          shiftKey: !0,
          view: !0,
          char: !0,
          code: !0,
          charCode: !0,
          key: !0,
          keyCode: !0,
          button: !0,
          buttons: !0,
          clientX: !0,
          clientY: !0,
          offsetX: !0,
          offsetY: !0,
          pointerId: !0,
          pointerType: !0,
          screenX: !0,
          screenY: !0,
          targetTouches: !0,
          toElement: !0,
          touches: !0,
          which: !0
        }, ge.event.addProp), ge.each({ focus: "focusin", blur: "focusout" }, function(nn, Tn) {
          function Dn(Fn) {
            if (de.documentMode) {
              var fr = Pt.get(this, "handle"), dr = ge.event.fix(Fn);
              dr.type = Fn.type === "focusin" ? "focus" : "blur", dr.isSimulated = !0, fr(Fn), dr.target === dr.currentTarget && fr(dr);
            } else
              ge.event.simulate(
                Tn,
                Fn.target,
                ge.event.fix(Fn)
              );
          }
          ge.event.special[nn] = {
            // Utilize native event if possible so blur/focus sequence is correct
            setup: function() {
              var Fn;
              if (wt(this, nn, !0), de.documentMode)
                Fn = Pt.get(this, Tn), Fn || this.addEventListener(Tn, Dn), Pt.set(this, Tn, (Fn || 0) + 1);
              else
                return !1;
            },
            trigger: function() {
              return wt(this, nn), !0;
            },
            teardown: function() {
              var Fn;
              if (de.documentMode)
                Fn = Pt.get(this, Tn) - 1, Fn ? Pt.set(this, Tn, Fn) : (this.removeEventListener(Tn, Dn), Pt.remove(this, Tn));
              else
                return !1;
            },
            // Suppress native focus or blur if we're currently inside
            // a leveraged native-event stack
            _default: function(Fn) {
              return Pt.get(Fn.target, nn);
            },
            delegateType: Tn
          }, ge.event.special[Tn] = {
            setup: function() {
              var Fn = this.ownerDocument || this.document || this, fr = de.documentMode ? this : Fn, dr = Pt.get(fr, Tn);
              dr || (de.documentMode ? this.addEventListener(Tn, Dn) : Fn.addEventListener(nn, Dn, !0)), Pt.set(fr, Tn, (dr || 0) + 1);
            },
            teardown: function() {
              var Fn = this.ownerDocument || this.document || this, fr = de.documentMode ? this : Fn, dr = Pt.get(fr, Tn) - 1;
              dr ? Pt.set(fr, Tn, dr) : (de.documentMode ? this.removeEventListener(Tn, Dn) : Fn.removeEventListener(nn, Dn, !0), Pt.remove(fr, Tn));
            }
          };
        }), ge.each({
          mouseenter: "mouseover",
          mouseleave: "mouseout",
          pointerenter: "pointerover",
          pointerleave: "pointerout"
        }, function(nn, Tn) {
          ge.event.special[nn] = {
            delegateType: Tn,
            bindType: Tn,
            handle: function(Dn) {
              var Fn, fr = this, dr = Dn.relatedTarget, yr = Dn.handleObj;
              return (!dr || dr !== fr && !ge.contains(fr, dr)) && (Dn.type = yr.origType, Fn = yr.handler.apply(this, arguments), Dn.type = Tn), Fn;
            }
          };
        }), ge.fn.extend({
          on: function(nn, Tn, Dn, Fn) {
            return pt(this, nn, Tn, Dn, Fn);
          },
          one: function(nn, Tn, Dn, Fn) {
            return pt(this, nn, Tn, Dn, Fn, 1);
          },
          off: function(nn, Tn, Dn) {
            var Fn, fr;
            if (nn && nn.preventDefault && nn.handleObj)
              return Fn = nn.handleObj, ge(nn.delegateTarget).off(
                Fn.namespace ? Fn.origType + "." + Fn.namespace : Fn.origType,
                Fn.selector,
                Fn.handler
              ), this;
            if (typeof nn == "object") {
              for (fr in nn)
                this.off(fr, Tn, nn[fr]);
              return this;
            }
            return (Tn === !1 || typeof Tn == "function") && (Dn = Tn, Tn = void 0), Dn === !1 && (Dn = ft), this.each(function() {
              ge.event.remove(this, nn, Dn, Tn);
            });
          }
        });
        var Bt = /<script|<style|<link/i, At = /checked\s*(?:[^=]|=\s*.checked.)/i, Dt = /^\s*<!\[CDATA\[|\]\]>\s*$/g;
        function Ot(nn, Tn) {
          return be(nn, "table") && be(Tn.nodeType !== 11 ? Tn : Tn.firstChild, "tr") && ge(nn).children("tbody")[0] || nn;
        }
        function un(nn) {
          return nn.type = (nn.getAttribute("type") !== null) + "/" + nn.type, nn;
        }
        function yn(nn) {
          return (nn.type || "").slice(0, 5) === "true/" ? nn.type = nn.type.slice(5) : nn.removeAttribute("type"), nn;
        }
        function _n(nn, Tn) {
          var Dn, Fn, fr, dr, yr, ai, Fr;
          if (Tn.nodeType === 1) {
            if (Pt.hasData(nn) && (dr = Pt.get(nn), Fr = dr.events, Fr)) {
              Pt.remove(Tn, "handle events");
              for (fr in Fr)
                for (Dn = 0, Fn = Fr[fr].length; Dn < Fn; Dn++)
                  ge.event.add(Tn, fr, Fr[fr][Dn]);
            }
            Xt.hasData(nn) && (yr = Xt.access(nn), ai = ge.extend({}, yr), Xt.set(Tn, ai));
          }
        }
        function gn(nn, Tn) {
          var Dn = Tn.nodeName.toLowerCase();
          Dn === "input" && vn.test(nn.type) ? Tn.checked = nn.checked : (Dn === "input" || Dn === "textarea") && (Tn.defaultValue = nn.defaultValue);
        }
        function bn(nn, Tn, Dn, Fn) {
          Tn = J(Tn);
          var fr, dr, yr, ai, Fr, Ti, Si = 0, Ei = nn.length, Fa = Ei - 1, $o = Tn[0], Gs = le($o);
          if (Gs || Ei > 1 && typeof $o == "string" && !ce.checkClone && At.test($o))
            return nn.each(function(Al) {
              var ul = nn.eq(Al);
              Gs && (Tn[0] = $o.call(this, Al, ul.html())), bn(ul, Tn, Dn, Fn);
            });
          if (Ei && (fr = Pn(Tn, nn[0].ownerDocument, !1, nn, Fn), dr = fr.firstChild, fr.childNodes.length === 1 && (fr = dr), dr || Fn)) {
            for (yr = ge.map(cn(fr, "script"), un), ai = yr.length; Si < Ei; Si++)
              Fr = fr, Si !== Fa && (Fr = ge.clone(Fr, !0, !0), ai && ge.merge(yr, cn(Fr, "script"))), Dn.call(nn[Si], Fr, Si);
            if (ai)
              for (Ti = yr[yr.length - 1].ownerDocument, ge.map(yr, yn), Si = 0; Si < ai; Si++)
                Fr = yr[Si], Ht.test(Fr.type || "") && !Pt.access(Fr, "globalEval") && ge.contains(Ti, Fr) && (Fr.src && (Fr.type || "").toLowerCase() !== "module" ? ge._evalUrl && !Fr.noModule && ge._evalUrl(Fr.src, {
                  nonce: Fr.nonce || Fr.getAttribute("nonce")
                }, Ti) : he(Fr.textContent.replace(Dt, ""), Fr, Ti));
          }
          return nn;
        }
        function an(nn, Tn, Dn) {
          for (var Fn, fr = Tn ? ge.filter(Tn, nn) : nn, dr = 0; (Fn = fr[dr]) != null; dr++)
            !Dn && Fn.nodeType === 1 && ge.cleanData(cn(Fn)), Fn.parentNode && (Dn && Yt(Fn) && wn(cn(Fn, "script")), Fn.parentNode.removeChild(Fn));
          return nn;
        }
        ge.extend({
          htmlPrefilter: function(nn) {
            return nn;
          },
          clone: function(nn, Tn, Dn) {
            var Fn, fr, dr, yr, ai = nn.cloneNode(!0), Fr = Yt(nn);
            if (!ce.noCloneChecked && (nn.nodeType === 1 || nn.nodeType === 11) && !ge.isXMLDoc(nn))
              for (yr = cn(ai), dr = cn(nn), Fn = 0, fr = dr.length; Fn < fr; Fn++)
                gn(dr[Fn], yr[Fn]);
            if (Tn)
              if (Dn)
                for (dr = dr || cn(nn), yr = yr || cn(ai), Fn = 0, fr = dr.length; Fn < fr; Fn++)
                  _n(dr[Fn], yr[Fn]);
              else
                _n(nn, ai);
            return yr = cn(ai, "script"), yr.length > 0 && wn(yr, !Fr && cn(nn, "script")), ai;
          },
          cleanData: function(nn) {
            for (var Tn, Dn, Fn, fr = ge.event.special, dr = 0; (Dn = nn[dr]) !== void 0; dr++)
              if (vt(Dn)) {
                if (Tn = Dn[Pt.expando]) {
                  if (Tn.events)
                    for (Fn in Tn.events)
                      fr[Fn] ? ge.event.remove(Dn, Fn) : ge.removeEvent(Dn, Fn, Tn.handle);
                  Dn[Pt.expando] = void 0;
                }
                Dn[Xt.expando] && (Dn[Xt.expando] = void 0);
              }
          }
        }), ge.fn.extend({
          detach: function(nn) {
            return an(this, nn, !0);
          },
          remove: function(nn) {
            return an(this, nn);
          },
          text: function(nn) {
            return bt(this, function(Tn) {
              return Tn === void 0 ? ge.text(this) : this.empty().each(function() {
                (this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9) && (this.textContent = Tn);
              });
            }, null, nn, arguments.length);
          },
          append: function() {
            return bn(this, arguments, function(nn) {
              if (this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9) {
                var Tn = Ot(this, nn);
                Tn.appendChild(nn);
              }
            });
          },
          prepend: function() {
            return bn(this, arguments, function(nn) {
              if (this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9) {
                var Tn = Ot(this, nn);
                Tn.insertBefore(nn, Tn.firstChild);
              }
            });
          },
          before: function() {
            return bn(this, arguments, function(nn) {
              this.parentNode && this.parentNode.insertBefore(nn, this);
            });
          },
          after: function() {
            return bn(this, arguments, function(nn) {
              this.parentNode && this.parentNode.insertBefore(nn, this.nextSibling);
            });
          },
          empty: function() {
            for (var nn, Tn = 0; (nn = this[Tn]) != null; Tn++)
              nn.nodeType === 1 && (ge.cleanData(cn(nn, !1)), nn.textContent = "");
            return this;
          },
          clone: function(nn, Tn) {
            return nn = nn ?? !1, Tn = Tn ?? nn, this.map(function() {
              return ge.clone(this, nn, Tn);
            });
          },
          html: function(nn) {
            return bt(this, function(Tn) {
              var Dn = this[0] || {}, Fn = 0, fr = this.length;
              if (Tn === void 0 && Dn.nodeType === 1)
                return Dn.innerHTML;
              if (typeof Tn == "string" && !Bt.test(Tn) && !jt[(dn.exec(Tn) || ["", ""])[1].toLowerCase()]) {
                Tn = ge.htmlPrefilter(Tn);
                try {
                  for (; Fn < fr; Fn++)
                    Dn = this[Fn] || {}, Dn.nodeType === 1 && (ge.cleanData(cn(Dn, !1)), Dn.innerHTML = Tn);
                  Dn = 0;
                } catch {
                }
              }
              Dn && this.empty().append(Tn);
            }, null, nn, arguments.length);
          },
          replaceWith: function() {
            var nn = [];
            return bn(this, arguments, function(Tn) {
              var Dn = this.parentNode;
              ge.inArray(this, nn) < 0 && (ge.cleanData(cn(this)), Dn && Dn.replaceChild(Tn, this));
            }, nn);
          }
        }), ge.each({
          appendTo: "append",
          prependTo: "prepend",
          insertBefore: "before",
          insertAfter: "after",
          replaceAll: "replaceWith"
        }, function(nn, Tn) {
          ge.fn[nn] = function(Dn) {
            for (var Fn, fr = [], dr = ge(Dn), yr = dr.length - 1, ai = 0; ai <= yr; ai++)
              Fn = ai === yr ? this : this.clone(!0), ge(dr[ai])[Tn](Fn), Z.apply(fr, Fn.get());
            return this.pushStack(fr);
          };
        });
        var Zt = new RegExp("^(" + It + ")(?!px)[a-z%]+$", "i"), Mn = /^--/, Ln = function(nn) {
          var Tn = nn.ownerDocument.defaultView;
          return (!Tn || !Tn.opener) && (Tn = _), Tn.getComputedStyle(nn);
        }, In = function(nn, Tn, Dn) {
          var Fn, fr, dr = {};
          for (fr in Tn)
            dr[fr] = nn.style[fr], nn.style[fr] = Tn[fr];
          Fn = Dn.call(nn);
          for (fr in Tn)
            nn.style[fr] = dr[fr];
          return Fn;
        }, kn = new RegExp(Kt.join("|"), "i");
        (function() {
          function nn() {
            if (Ti) {
              Fr.style.cssText = "position:absolute;left:-11111px;width:60px;margin-top:1px;padding:0;border:0", Ti.style.cssText = "position:relative;display:block;box-sizing:border-box;overflow:scroll;margin:auto;border:1px;padding:1px;width:60%;top:1%", Vt.appendChild(Fr).appendChild(Ti);
              var Si = _.getComputedStyle(Ti);
              Dn = Si.top !== "1%", ai = Tn(Si.marginLeft) === 12, Ti.style.right = "60%", dr = Tn(Si.right) === 36, Fn = Tn(Si.width) === 36, Ti.style.position = "absolute", fr = Tn(Ti.offsetWidth / 3) === 12, Vt.removeChild(Fr), Ti = null;
            }
          }
          function Tn(Si) {
            return Math.round(parseFloat(Si));
          }
          var Dn, Fn, fr, dr, yr, ai, Fr = de.createElement("div"), Ti = de.createElement("div");
          Ti.style && (Ti.style.backgroundClip = "content-box", Ti.cloneNode(!0).style.backgroundClip = "", ce.clearCloneStyle = Ti.style.backgroundClip === "content-box", ge.extend(ce, {
            boxSizingReliable: function() {
              return nn(), Fn;
            },
            pixelBoxStyles: function() {
              return nn(), dr;
            },
            pixelPosition: function() {
              return nn(), Dn;
            },
            reliableMarginLeft: function() {
              return nn(), ai;
            },
            scrollboxSize: function() {
              return nn(), fr;
            },
            // Support: IE 9 - 11+, Edge 15 - 18+
            // IE/Edge misreport `getComputedStyle` of table rows with width/height
            // set in CSS while `offset*` properties report correct values.
            // Behavior in IE 9 is more subtle than in newer versions & it passes
            // some versions of this test; make sure not to make it pass there!
            //
            // Support: Firefox 70+
            // Only Firefox includes border widths
            // in computed dimensions. (gh-4529)
            reliableTrDimensions: function() {
              var Si, Ei, Fa, $o;
              return yr == null && (Si = de.createElement("table"), Ei = de.createElement("tr"), Fa = de.createElement("div"), Si.style.cssText = "position:absolute;left:-11111px;border-collapse:separate", Ei.style.cssText = "box-sizing:content-box;border:1px solid", Ei.style.height = "1px", Fa.style.height = "9px", Fa.style.display = "block", Vt.appendChild(Si).appendChild(Ei).appendChild(Fa), $o = _.getComputedStyle(Ei), yr = parseInt($o.height, 10) + parseInt($o.borderTopWidth, 10) + parseInt($o.borderBottomWidth, 10) === Ei.offsetHeight, Vt.removeChild(Si)), yr;
            }
          }));
        })();
        function er(nn, Tn, Dn) {
          var Fn, fr, dr, yr, ai = Mn.test(Tn), Fr = nn.style;
          return Dn = Dn || Ln(nn), Dn && (yr = Dn.getPropertyValue(Tn) || Dn[Tn], ai && yr && (yr = yr.replace(xe, "$1") || void 0), yr === "" && !Yt(nn) && (yr = ge.style(nn, Tn)), !ce.pixelBoxStyles() && Zt.test(yr) && kn.test(Tn) && (Fn = Fr.width, fr = Fr.minWidth, dr = Fr.maxWidth, Fr.minWidth = Fr.maxWidth = Fr.width = yr, yr = Dn.width, Fr.width = Fn, Fr.minWidth = fr, Fr.maxWidth = dr)), yr !== void 0 ? (
            // Support: IE <=9 - 11 only
            // IE returns zIndex value as an integer.
            yr + ""
          ) : yr;
        }
        function jn(nn, Tn) {
          return {
            get: function() {
              if (nn()) {
                delete this.get;
                return;
              }
              return (this.get = Tn).apply(this, arguments);
            }
          };
        }
        var ur = ["Webkit", "Moz", "ms"], Kn = de.createElement("div").style, Jn = {};
        function Lr(nn) {
          for (var Tn = nn[0].toUpperCase() + nn.slice(1), Dn = ur.length; Dn--; )
            if (nn = ur[Dn] + Tn, nn in Kn)
              return nn;
        }
        function mr(nn) {
          var Tn = ge.cssProps[nn] || Jn[nn];
          return Tn || (nn in Kn ? nn : Jn[nn] = Lr(nn) || nn);
        }
        var Ar = /^(none|table(?!-c[ea]).+)/, Yr = { position: "absolute", visibility: "hidden", display: "block" }, jr = {
          letterSpacing: "0",
          fontWeight: "400"
        };
        function Mr(nn, Tn, Dn) {
          var Fn = Ct.exec(Tn);
          return Fn ? (
            // Guard against undefined "subtract", e.g., when used as in cssHooks
            Math.max(0, Fn[2] - (Dn || 0)) + (Fn[3] || "px")
          ) : Tn;
        }
        function Wr(nn, Tn, Dn, Fn, fr, dr) {
          var yr = Tn === "width" ? 1 : 0, ai = 0, Fr = 0, Ti = 0;
          if (Dn === (Fn ? "border" : "content"))
            return 0;
          for (; yr < 4; yr += 2)
            Dn === "margin" && (Ti += ge.css(nn, Dn + Kt[yr], !0, fr)), Fn ? (Dn === "content" && (Fr -= ge.css(nn, "padding" + Kt[yr], !0, fr)), Dn !== "margin" && (Fr -= ge.css(nn, "border" + Kt[yr] + "Width", !0, fr))) : (Fr += ge.css(nn, "padding" + Kt[yr], !0, fr), Dn !== "padding" ? Fr += ge.css(nn, "border" + Kt[yr] + "Width", !0, fr) : ai += ge.css(nn, "border" + Kt[yr] + "Width", !0, fr));
          return !Fn && dr >= 0 && (Fr += Math.max(0, Math.ceil(
            nn["offset" + Tn[0].toUpperCase() + Tn.slice(1)] - dr - Fr - ai - 0.5
            // If offsetWidth/offsetHeight is unknown, then we can't determine content-box scroll gutter
            // Use an explicit zero to avoid NaN (gh-3964)
          )) || 0), Fr + Ti;
        }
        function Bi(nn, Tn, Dn) {
          var Fn = Ln(nn), fr = !ce.boxSizingReliable() || Dn, dr = fr && ge.css(nn, "boxSizing", !1, Fn) === "border-box", yr = dr, ai = er(nn, Tn, Fn), Fr = "offset" + Tn[0].toUpperCase() + Tn.slice(1);
          if (Zt.test(ai)) {
            if (!Dn)
              return ai;
            ai = "auto";
          }
          return (!ce.boxSizingReliable() && dr || // Support: IE 10 - 11+, Edge 15 - 18+
          // IE/Edge misreport `getComputedStyle` of table rows with width/height
          // set in CSS while `offset*` properties report correct values.
          // Interestingly, in some cases IE 9 doesn't suffer from this issue.
          !ce.reliableTrDimensions() && be(nn, "tr") || // Fall back to offsetWidth/offsetHeight when value is "auto"
          // This happens for inline elements with no explicit setting (gh-3571)
          ai === "auto" || // Support: Android <=4.1 - 4.3 only
          // Also use offsetWidth/offsetHeight for misreported inline dimensions (gh-3602)
          !parseFloat(ai) && ge.css(nn, "display", !1, Fn) === "inline") && // Make sure the element is visible & connected
          nn.getClientRects().length && (dr = ge.css(nn, "boxSizing", !1, Fn) === "border-box", yr = Fr in nn, yr && (ai = nn[Fr])), ai = parseFloat(ai) || 0, ai + Wr(
            nn,
            Tn,
            Dn || (dr ? "border" : "content"),
            yr,
            Fn,
            // Provide the current computed size to request scroll gutter calculation (gh-3589)
            ai
          ) + "px";
        }
        ge.extend({
          // Add in style property hooks for overriding the default
          // behavior of getting and setting a style property
          cssHooks: {
            opacity: {
              get: function(nn, Tn) {
                if (Tn) {
                  var Dn = er(nn, "opacity");
                  return Dn === "" ? "1" : Dn;
                }
              }
            }
          },
          // Don't automatically add "px" to these possibly-unitless properties
          cssNumber: {
            animationIterationCount: !0,
            aspectRatio: !0,
            borderImageSlice: !0,
            columnCount: !0,
            flexGrow: !0,
            flexShrink: !0,
            fontWeight: !0,
            gridArea: !0,
            gridColumn: !0,
            gridColumnEnd: !0,
            gridColumnStart: !0,
            gridRow: !0,
            gridRowEnd: !0,
            gridRowStart: !0,
            lineHeight: !0,
            opacity: !0,
            order: !0,
            orphans: !0,
            scale: !0,
            widows: !0,
            zIndex: !0,
            zoom: !0,
            // SVG-related
            fillOpacity: !0,
            floodOpacity: !0,
            stopOpacity: !0,
            strokeMiterlimit: !0,
            strokeOpacity: !0
          },
          // Add in properties whose names you wish to fix before
          // setting or getting the value
          cssProps: {},
          // Get and set the style property on a DOM Node
          style: function(nn, Tn, Dn, Fn) {
            if (!(!nn || nn.nodeType === 3 || nn.nodeType === 8 || !nn.style)) {
              var fr, dr, yr, ai = Rt(Tn), Fr = Mn.test(Tn), Ti = nn.style;
              if (Fr || (Tn = mr(ai)), yr = ge.cssHooks[Tn] || ge.cssHooks[ai], Dn !== void 0) {
                if (dr = typeof Dn, dr === "string" && (fr = Ct.exec(Dn)) && fr[1] && (Dn = Qt(nn, Tn, fr), dr = "number"), Dn == null || Dn !== Dn)
                  return;
                dr === "number" && !Fr && (Dn += fr && fr[3] || (ge.cssNumber[ai] ? "" : "px")), !ce.clearCloneStyle && Dn === "" && Tn.indexOf("background") === 0 && (Ti[Tn] = "inherit"), (!yr || !("set" in yr) || (Dn = yr.set(nn, Dn, Fn)) !== void 0) && (Fr ? Ti.setProperty(Tn, Dn) : Ti[Tn] = Dn);
              } else
                return yr && "get" in yr && (fr = yr.get(nn, !1, Fn)) !== void 0 ? fr : Ti[Tn];
            }
          },
          css: function(nn, Tn, Dn, Fn) {
            var fr, dr, yr, ai = Rt(Tn), Fr = Mn.test(Tn);
            return Fr || (Tn = mr(ai)), yr = ge.cssHooks[Tn] || ge.cssHooks[ai], yr && "get" in yr && (fr = yr.get(nn, !0, Dn)), fr === void 0 && (fr = er(nn, Tn, Fn)), fr === "normal" && Tn in jr && (fr = jr[Tn]), Dn === "" || Dn ? (dr = parseFloat(fr), Dn === !0 || isFinite(dr) ? dr || 0 : fr) : fr;
          }
        }), ge.each(["height", "width"], function(nn, Tn) {
          ge.cssHooks[Tn] = {
            get: function(Dn, Fn, fr) {
              if (Fn)
                return Ar.test(ge.css(Dn, "display")) && // Support: Safari 8+
                // Table columns in Safari have non-zero offsetWidth & zero
                // getBoundingClientRect().width unless display is changed.
                // Support: IE <=11 only
                // Running getBoundingClientRect on a disconnected node
                // in IE throws an error.
                (!Dn.getClientRects().length || !Dn.getBoundingClientRect().width) ? In(Dn, Yr, function() {
                  return Bi(Dn, Tn, fr);
                }) : Bi(Dn, Tn, fr);
            },
            set: function(Dn, Fn, fr) {
              var dr, yr = Ln(Dn), ai = !ce.scrollboxSize() && yr.position === "absolute", Fr = ai || fr, Ti = Fr && ge.css(Dn, "boxSizing", !1, yr) === "border-box", Si = fr ? Wr(
                Dn,
                Tn,
                fr,
                Ti,
                yr
              ) : 0;
              return Ti && ai && (Si -= Math.ceil(
                Dn["offset" + Tn[0].toUpperCase() + Tn.slice(1)] - parseFloat(yr[Tn]) - Wr(Dn, Tn, "border", !1, yr) - 0.5
              )), Si && (dr = Ct.exec(Fn)) && (dr[3] || "px") !== "px" && (Dn.style[Tn] = Fn, Fn = ge.css(Dn, Tn)), Mr(Dn, Fn, Si);
            }
          };
        }), ge.cssHooks.marginLeft = jn(
          ce.reliableMarginLeft,
          function(nn, Tn) {
            if (Tn)
              return (parseFloat(er(nn, "marginLeft")) || nn.getBoundingClientRect().left - In(nn, { marginLeft: 0 }, function() {
                return nn.getBoundingClientRect().left;
              })) + "px";
          }
        ), ge.each({
          margin: "",
          padding: "",
          border: "Width"
        }, function(nn, Tn) {
          ge.cssHooks[nn + Tn] = {
            expand: function(Dn) {
              for (var Fn = 0, fr = {}, dr = typeof Dn == "string" ? Dn.split(" ") : [Dn]; Fn < 4; Fn++)
                fr[nn + Kt[Fn] + Tn] = dr[Fn] || dr[Fn - 2] || dr[0];
              return fr;
            }
          }, nn !== "margin" && (ge.cssHooks[nn + Tn].set = Mr);
        }), ge.fn.extend({
          css: function(nn, Tn) {
            return bt(this, function(Dn, Fn, fr) {
              var dr, yr, ai = {}, Fr = 0;
              if (Array.isArray(Fn)) {
                for (dr = Ln(Dn), yr = Fn.length; Fr < yr; Fr++)
                  ai[Fn[Fr]] = ge.css(Dn, Fn[Fr], !1, dr);
                return ai;
              }
              return fr !== void 0 ? ge.style(Dn, Fn, fr) : ge.css(Dn, Fn);
            }, nn, Tn, arguments.length > 1);
          }
        });
        function gi(nn, Tn, Dn, Fn, fr) {
          return new gi.prototype.init(nn, Tn, Dn, Fn, fr);
        }
        ge.Tween = gi, gi.prototype = {
          constructor: gi,
          init: function(nn, Tn, Dn, Fn, fr, dr) {
            this.elem = nn, this.prop = Dn, this.easing = fr || ge.easing._default, this.options = Tn, this.start = this.now = this.cur(), this.end = Fn, this.unit = dr || (ge.cssNumber[Dn] ? "" : "px");
          },
          cur: function() {
            var nn = gi.propHooks[this.prop];
            return nn && nn.get ? nn.get(this) : gi.propHooks._default.get(this);
          },
          run: function(nn) {
            var Tn, Dn = gi.propHooks[this.prop];
            return this.options.duration ? this.pos = Tn = ge.easing[this.easing](
              nn,
              this.options.duration * nn,
              0,
              1,
              this.options.duration
            ) : this.pos = Tn = nn, this.now = (this.end - this.start) * Tn + this.start, this.options.step && this.options.step.call(this.elem, this.now, this), Dn && Dn.set ? Dn.set(this) : gi.propHooks._default.set(this), this;
          }
        }, gi.prototype.init.prototype = gi.prototype, gi.propHooks = {
          _default: {
            get: function(nn) {
              var Tn;
              return nn.elem.nodeType !== 1 || nn.elem[nn.prop] != null && nn.elem.style[nn.prop] == null ? nn.elem[nn.prop] : (Tn = ge.css(nn.elem, nn.prop, ""), !Tn || Tn === "auto" ? 0 : Tn);
            },
            set: function(nn) {
              ge.fx.step[nn.prop] ? ge.fx.step[nn.prop](nn) : nn.elem.nodeType === 1 && (ge.cssHooks[nn.prop] || nn.elem.style[mr(nn.prop)] != null) ? ge.style(nn.elem, nn.prop, nn.now + nn.unit) : nn.elem[nn.prop] = nn.now;
            }
          }
        }, gi.propHooks.scrollTop = gi.propHooks.scrollLeft = {
          set: function(nn) {
            nn.elem.nodeType && nn.elem.parentNode && (nn.elem[nn.prop] = nn.now);
          }
        }, ge.easing = {
          linear: function(nn) {
            return nn;
          },
          swing: function(nn) {
            return 0.5 - Math.cos(nn * Math.PI) / 2;
          },
          _default: "swing"
        }, ge.fx = gi.prototype.init, ge.fx.step = {};
        var Zr, fi, ri = /^(?:toggle|show|hide)$/, bi = /queueHooks$/;
        function Zi() {
          fi && (de.hidden === !1 && _.requestAnimationFrame ? _.requestAnimationFrame(Zi) : _.setTimeout(Zi, ge.fx.interval), ge.fx.tick());
        }
        function $i() {
          return _.setTimeout(function() {
            Zr = void 0;
          }), Zr = Date.now();
        }
        function Br(nn, Tn) {
          var Dn, Fn = 0, fr = { height: nn };
          for (Tn = Tn ? 1 : 0; Fn < 4; Fn += 2 - Tn)
            Dn = Kt[Fn], fr["margin" + Dn] = fr["padding" + Dn] = nn;
          return Tn && (fr.opacity = fr.width = nn), fr;
        }
        function Pr(nn, Tn, Dn) {
          for (var Fn, fr = (oa.tweeners[Tn] || []).concat(oa.tweeners["*"]), dr = 0, yr = fr.length; dr < yr; dr++)
            if (Fn = fr[dr].call(Dn, Tn, nn))
              return Fn;
        }
        function ei(nn, Tn, Dn) {
          var Fn, fr, dr, yr, ai, Fr, Ti, Si, Ei = "width" in Tn || "height" in Tn, Fa = this, $o = {}, Gs = nn.style, Al = nn.nodeType && hn(nn), ul = Pt.get(nn, "fxshow");
          Dn.queue || (yr = ge._queueHooks(nn, "fx"), yr.unqueued == null && (yr.unqueued = 0, ai = yr.empty.fire, yr.empty.fire = function() {
            yr.unqueued || ai();
          }), yr.unqueued++, Fa.always(function() {
            Fa.always(function() {
              yr.unqueued--, ge.queue(nn, "fx").length || yr.empty.fire();
            });
          }));
          for (Fn in Tn)
            if (fr = Tn[Fn], ri.test(fr)) {
              if (delete Tn[Fn], dr = dr || fr === "toggle", fr === (Al ? "hide" : "show"))
                if (fr === "show" && ul && ul[Fn] !== void 0)
                  Al = !0;
                else
                  continue;
              $o[Fn] = ul && ul[Fn] || ge.style(nn, Fn);
            }
          if (Fr = !ge.isEmptyObject(Tn), !(!Fr && ge.isEmptyObject($o))) {
            Ei && nn.nodeType === 1 && (Dn.overflow = [Gs.overflow, Gs.overflowX, Gs.overflowY], Ti = ul && ul.display, Ti == null && (Ti = Pt.get(nn, "display")), Si = ge.css(nn, "display"), Si === "none" && (Ti ? Si = Ti : (rn([nn], !0), Ti = nn.style.display || Ti, Si = ge.css(nn, "display"), rn([nn]))), (Si === "inline" || Si === "inline-block" && Ti != null) && ge.css(nn, "float") === "none" && (Fr || (Fa.done(function() {
              Gs.display = Ti;
            }), Ti == null && (Si = Gs.display, Ti = Si === "none" ? "" : Si)), Gs.display = "inline-block")), Dn.overflow && (Gs.overflow = "hidden", Fa.always(function() {
              Gs.overflow = Dn.overflow[0], Gs.overflowX = Dn.overflow[1], Gs.overflowY = Dn.overflow[2];
            })), Fr = !1;
            for (Fn in $o)
              Fr || (ul ? "hidden" in ul && (Al = ul.hidden) : ul = Pt.access(nn, "fxshow", { display: Ti }), dr && (ul.hidden = !Al), Al && rn([nn], !0), Fa.done(function() {
                Al || rn([nn]), Pt.remove(nn, "fxshow");
                for (Fn in $o)
                  ge.style(nn, Fn, $o[Fn]);
              })), Fr = Pr(Al ? ul[Fn] : 0, Fn, Fa), Fn in ul || (ul[Fn] = Fr.start, Al && (Fr.end = Fr.start, Fr.start = 0));
          }
        }
        function Ci(nn, Tn) {
          var Dn, Fn, fr, dr, yr;
          for (Dn in nn)
            if (Fn = Rt(Dn), fr = Tn[Fn], dr = nn[Dn], Array.isArray(dr) && (fr = dr[1], dr = nn[Dn] = dr[0]), Dn !== Fn && (nn[Fn] = dr, delete nn[Dn]), yr = ge.cssHooks[Fn], yr && "expand" in yr) {
              dr = yr.expand(dr), delete nn[Fn];
              for (Dn in dr)
                Dn in nn || (nn[Dn] = dr[Dn], Tn[Dn] = fr);
            } else
              Tn[Fn] = fr;
        }
        function oa(nn, Tn, Dn) {
          var Fn, fr, dr = 0, yr = oa.prefilters.length, ai = ge.Deferred().always(function() {
            delete Fr.elem;
          }), Fr = function() {
            if (fr)
              return !1;
            for (var Ei = Zr || $i(), Fa = Math.max(0, Ti.startTime + Ti.duration - Ei), $o = Fa / Ti.duration || 0, Gs = 1 - $o, Al = 0, ul = Ti.tweens.length; Al < ul; Al++)
              Ti.tweens[Al].run(Gs);
            return ai.notifyWith(nn, [Ti, Gs, Fa]), Gs < 1 && ul ? Fa : (ul || ai.notifyWith(nn, [Ti, 1, 0]), ai.resolveWith(nn, [Ti]), !1);
          }, Ti = ai.promise({
            elem: nn,
            props: ge.extend({}, Tn),
            opts: ge.extend(!0, {
              specialEasing: {},
              easing: ge.easing._default
            }, Dn),
            originalProperties: Tn,
            originalOptions: Dn,
            startTime: Zr || $i(),
            duration: Dn.duration,
            tweens: [],
            createTween: function(Ei, Fa) {
              var $o = ge.Tween(
                nn,
                Ti.opts,
                Ei,
                Fa,
                Ti.opts.specialEasing[Ei] || Ti.opts.easing
              );
              return Ti.tweens.push($o), $o;
            },
            stop: function(Ei) {
              var Fa = 0, $o = Ei ? Ti.tweens.length : 0;
              if (fr)
                return this;
              for (fr = !0; Fa < $o; Fa++)
                Ti.tweens[Fa].run(1);
              return Ei ? (ai.notifyWith(nn, [Ti, 1, 0]), ai.resolveWith(nn, [Ti, Ei])) : ai.rejectWith(nn, [Ti, Ei]), this;
            }
          }), Si = Ti.props;
          for (Ci(Si, Ti.opts.specialEasing); dr < yr; dr++)
            if (Fn = oa.prefilters[dr].call(Ti, nn, Si, Ti.opts), Fn)
              return le(Fn.stop) && (ge._queueHooks(Ti.elem, Ti.opts.queue).stop = Fn.stop.bind(Fn)), Fn;
          return ge.map(Si, Pr, Ti), le(Ti.opts.start) && Ti.opts.start.call(nn, Ti), Ti.progress(Ti.opts.progress).done(Ti.opts.done, Ti.opts.complete).fail(Ti.opts.fail).always(Ti.opts.always), ge.fx.timer(
            ge.extend(Fr, {
              elem: nn,
              anim: Ti,
              queue: Ti.opts.queue
            })
          ), Ti;
        }
        ge.Animation = ge.extend(oa, {
          tweeners: {
            "*": [function(nn, Tn) {
              var Dn = this.createTween(nn, Tn);
              return Qt(Dn.elem, nn, Ct.exec(Tn), Dn), Dn;
            }]
          },
          tweener: function(nn, Tn) {
            le(nn) ? (Tn = nn, nn = ["*"]) : nn = nn.match(at);
            for (var Dn, Fn = 0, fr = nn.length; Fn < fr; Fn++)
              Dn = nn[Fn], oa.tweeners[Dn] = oa.tweeners[Dn] || [], oa.tweeners[Dn].unshift(Tn);
          },
          prefilters: [ei],
          prefilter: function(nn, Tn) {
            Tn ? oa.prefilters.unshift(nn) : oa.prefilters.push(nn);
          }
        }), ge.speed = function(nn, Tn, Dn) {
          var Fn = nn && typeof nn == "object" ? ge.extend({}, nn) : {
            complete: Dn || !Dn && Tn || le(nn) && nn,
            duration: nn,
            easing: Dn && Tn || Tn && !le(Tn) && Tn
          };
          return ge.fx.off ? Fn.duration = 0 : typeof Fn.duration != "number" && (Fn.duration in ge.fx.speeds ? Fn.duration = ge.fx.speeds[Fn.duration] : Fn.duration = ge.fx.speeds._default), (Fn.queue == null || Fn.queue === !0) && (Fn.queue = "fx"), Fn.old = Fn.complete, Fn.complete = function() {
            le(Fn.old) && Fn.old.call(this), Fn.queue && ge.dequeue(this, Fn.queue);
          }, Fn;
        }, ge.fn.extend({
          fadeTo: function(nn, Tn, Dn, Fn) {
            return this.filter(hn).css("opacity", 0).show().end().animate({ opacity: Tn }, nn, Dn, Fn);
          },
          animate: function(nn, Tn, Dn, Fn) {
            var fr = ge.isEmptyObject(nn), dr = ge.speed(Tn, Dn, Fn), yr = function() {
              var ai = oa(this, ge.extend({}, nn), dr);
              (fr || Pt.get(this, "finish")) && ai.stop(!0);
            };
            return yr.finish = yr, fr || dr.queue === !1 ? this.each(yr) : this.queue(dr.queue, yr);
          },
          stop: function(nn, Tn, Dn) {
            var Fn = function(fr) {
              var dr = fr.stop;
              delete fr.stop, dr(Dn);
            };
            return typeof nn != "string" && (Dn = Tn, Tn = nn, nn = void 0), Tn && this.queue(nn || "fx", []), this.each(function() {
              var fr = !0, dr = nn != null && nn + "queueHooks", yr = ge.timers, ai = Pt.get(this);
              if (dr)
                ai[dr] && ai[dr].stop && Fn(ai[dr]);
              else
                for (dr in ai)
                  ai[dr] && ai[dr].stop && bi.test(dr) && Fn(ai[dr]);
              for (dr = yr.length; dr--; )
                yr[dr].elem === this && (nn == null || yr[dr].queue === nn) && (yr[dr].anim.stop(Dn), fr = !1, yr.splice(dr, 1));
              (fr || !Dn) && ge.dequeue(this, nn);
            });
          },
          finish: function(nn) {
            return nn !== !1 && (nn = nn || "fx"), this.each(function() {
              var Tn, Dn = Pt.get(this), Fn = Dn[nn + "queue"], fr = Dn[nn + "queueHooks"], dr = ge.timers, yr = Fn ? Fn.length : 0;
              for (Dn.finish = !0, ge.queue(this, nn, []), fr && fr.stop && fr.stop.call(this, !0), Tn = dr.length; Tn--; )
                dr[Tn].elem === this && dr[Tn].queue === nn && (dr[Tn].anim.stop(!0), dr.splice(Tn, 1));
              for (Tn = 0; Tn < yr; Tn++)
                Fn[Tn] && Fn[Tn].finish && Fn[Tn].finish.call(this);
              delete Dn.finish;
            });
          }
        }), ge.each(["toggle", "show", "hide"], function(nn, Tn) {
          var Dn = ge.fn[Tn];
          ge.fn[Tn] = function(Fn, fr, dr) {
            return Fn == null || typeof Fn == "boolean" ? Dn.apply(this, arguments) : this.animate(Br(Tn, !0), Fn, fr, dr);
          };
        }), ge.each({
          slideDown: Br("show"),
          slideUp: Br("hide"),
          slideToggle: Br("toggle"),
          fadeIn: { opacity: "show" },
          fadeOut: { opacity: "hide" },
          fadeToggle: { opacity: "toggle" }
        }, function(nn, Tn) {
          ge.fn[nn] = function(Dn, Fn, fr) {
            return this.animate(Tn, Dn, Fn, fr);
          };
        }), ge.timers = [], ge.fx.tick = function() {
          var nn, Tn = 0, Dn = ge.timers;
          for (Zr = Date.now(); Tn < Dn.length; Tn++)
            nn = Dn[Tn], !nn() && Dn[Tn] === nn && Dn.splice(Tn--, 1);
          Dn.length || ge.fx.stop(), Zr = void 0;
        }, ge.fx.timer = function(nn) {
          ge.timers.push(nn), ge.fx.start();
        }, ge.fx.interval = 13, ge.fx.start = function() {
          fi || (fi = !0, Zi());
        }, ge.fx.stop = function() {
          fi = null;
        }, ge.fx.speeds = {
          slow: 600,
          fast: 200,
          // Default speed
          _default: 400
        }, ge.fn.delay = function(nn, Tn) {
          return nn = ge.fx && ge.fx.speeds[nn] || nn, Tn = Tn || "fx", this.queue(Tn, function(Dn, Fn) {
            var fr = _.setTimeout(Dn, nn);
            Fn.stop = function() {
              _.clearTimeout(fr);
            };
          });
        }, function() {
          var nn = de.createElement("input"), Tn = de.createElement("select"), Dn = Tn.appendChild(de.createElement("option"));
          nn.type = "checkbox", ce.checkOn = nn.value !== "", ce.optSelected = Dn.selected, nn = de.createElement("input"), nn.value = "t", nn.type = "radio", ce.radioValue = nn.value === "t";
        }();
        var ka, aa = ge.expr.attrHandle;
        ge.fn.extend({
          attr: function(nn, Tn) {
            return bt(this, ge.attr, nn, Tn, arguments.length > 1);
          },
          removeAttr: function(nn) {
            return this.each(function() {
              ge.removeAttr(this, nn);
            });
          }
        }), ge.extend({
          attr: function(nn, Tn, Dn) {
            var Fn, fr, dr = nn.nodeType;
            if (!(dr === 3 || dr === 8 || dr === 2)) {
              if (typeof nn.getAttribute > "u")
                return ge.prop(nn, Tn, Dn);
              if ((dr !== 1 || !ge.isXMLDoc(nn)) && (fr = ge.attrHooks[Tn.toLowerCase()] || (ge.expr.match.bool.test(Tn) ? ka : void 0)), Dn !== void 0) {
                if (Dn === null) {
                  ge.removeAttr(nn, Tn);
                  return;
                }
                return fr && "set" in fr && (Fn = fr.set(nn, Dn, Tn)) !== void 0 ? Fn : (nn.setAttribute(Tn, Dn + ""), Dn);
              }
              return fr && "get" in fr && (Fn = fr.get(nn, Tn)) !== null ? Fn : (Fn = ge.find.attr(nn, Tn), Fn ?? void 0);
            }
          },
          attrHooks: {
            type: {
              set: function(nn, Tn) {
                if (!ce.radioValue && Tn === "radio" && be(nn, "input")) {
                  var Dn = nn.value;
                  return nn.setAttribute("type", Tn), Dn && (nn.value = Dn), Tn;
                }
              }
            }
          },
          removeAttr: function(nn, Tn) {
            var Dn, Fn = 0, fr = Tn && Tn.match(at);
            if (fr && nn.nodeType === 1)
              for (; Dn = fr[Fn++]; )
                nn.removeAttribute(Dn);
          }
        }), ka = {
          set: function(nn, Tn, Dn) {
            return Tn === !1 ? ge.removeAttr(nn, Dn) : nn.setAttribute(Dn, Dn), Dn;
          }
        }, ge.each(ge.expr.match.bool.source.match(/\w+/g), function(nn, Tn) {
          var Dn = aa[Tn] || ge.find.attr;
          aa[Tn] = function(Fn, fr, dr) {
            var yr, ai, Fr = fr.toLowerCase();
            return dr || (ai = aa[Fr], aa[Fr] = yr, yr = Dn(Fn, fr, dr) != null ? Fr : null, aa[Fr] = ai), yr;
          };
        });
        var hi = /^(?:input|select|textarea|button)$/i, Gr = /^(?:a|area)$/i;
        ge.fn.extend({
          prop: function(nn, Tn) {
            return bt(this, ge.prop, nn, Tn, arguments.length > 1);
          },
          removeProp: function(nn) {
            return this.each(function() {
              delete this[ge.propFix[nn] || nn];
            });
          }
        }), ge.extend({
          prop: function(nn, Tn, Dn) {
            var Fn, fr, dr = nn.nodeType;
            if (!(dr === 3 || dr === 8 || dr === 2))
              return (dr !== 1 || !ge.isXMLDoc(nn)) && (Tn = ge.propFix[Tn] || Tn, fr = ge.propHooks[Tn]), Dn !== void 0 ? fr && "set" in fr && (Fn = fr.set(nn, Dn, Tn)) !== void 0 ? Fn : nn[Tn] = Dn : fr && "get" in fr && (Fn = fr.get(nn, Tn)) !== null ? Fn : nn[Tn];
          },
          propHooks: {
            tabIndex: {
              get: function(nn) {
                var Tn = ge.find.attr(nn, "tabindex");
                return Tn ? parseInt(Tn, 10) : hi.test(nn.nodeName) || Gr.test(nn.nodeName) && nn.href ? 0 : -1;
              }
            }
          },
          propFix: {
            for: "htmlFor",
            class: "className"
          }
        }), ce.optSelected || (ge.propHooks.selected = {
          get: function(nn) {
            var Tn = nn.parentNode;
            return Tn && Tn.parentNode && Tn.parentNode.selectedIndex, null;
          },
          set: function(nn) {
            var Tn = nn.parentNode;
            Tn && (Tn.selectedIndex, Tn.parentNode && Tn.parentNode.selectedIndex);
          }
        }), ge.each([
          "tabIndex",
          "readOnly",
          "maxLength",
          "cellSpacing",
          "cellPadding",
          "rowSpan",
          "colSpan",
          "useMap",
          "frameBorder",
          "contentEditable"
        ], function() {
          ge.propFix[this.toLowerCase()] = this;
        });
        function Ca(nn) {
          var Tn = nn.match(at) || [];
          return Tn.join(" ");
        }
        function Sr(nn) {
          return nn.getAttribute && nn.getAttribute("class") || "";
        }
        function Dr(nn) {
          return Array.isArray(nn) ? nn : typeof nn == "string" ? nn.match(at) || [] : [];
        }
        ge.fn.extend({
          addClass: function(nn) {
            var Tn, Dn, Fn, fr, dr, yr;
            return le(nn) ? this.each(function(ai) {
              ge(this).addClass(nn.call(this, ai, Sr(this)));
            }) : (Tn = Dr(nn), Tn.length ? this.each(function() {
              if (Fn = Sr(this), Dn = this.nodeType === 1 && " " + Ca(Fn) + " ", Dn) {
                for (dr = 0; dr < Tn.length; dr++)
                  fr = Tn[dr], Dn.indexOf(" " + fr + " ") < 0 && (Dn += fr + " ");
                yr = Ca(Dn), Fn !== yr && this.setAttribute("class", yr);
              }
            }) : this);
          },
          removeClass: function(nn) {
            var Tn, Dn, Fn, fr, dr, yr;
            return le(nn) ? this.each(function(ai) {
              ge(this).removeClass(nn.call(this, ai, Sr(this)));
            }) : arguments.length ? (Tn = Dr(nn), Tn.length ? this.each(function() {
              if (Fn = Sr(this), Dn = this.nodeType === 1 && " " + Ca(Fn) + " ", Dn) {
                for (dr = 0; dr < Tn.length; dr++)
                  for (fr = Tn[dr]; Dn.indexOf(" " + fr + " ") > -1; )
                    Dn = Dn.replace(" " + fr + " ", " ");
                yr = Ca(Dn), Fn !== yr && this.setAttribute("class", yr);
              }
            }) : this) : this.attr("class", "");
          },
          toggleClass: function(nn, Tn) {
            var Dn, Fn, fr, dr, yr = typeof nn, ai = yr === "string" || Array.isArray(nn);
            return le(nn) ? this.each(function(Fr) {
              ge(this).toggleClass(
                nn.call(this, Fr, Sr(this), Tn),
                Tn
              );
            }) : typeof Tn == "boolean" && ai ? Tn ? this.addClass(nn) : this.removeClass(nn) : (Dn = Dr(nn), this.each(function() {
              if (ai)
                for (dr = ge(this), fr = 0; fr < Dn.length; fr++)
                  Fn = Dn[fr], dr.hasClass(Fn) ? dr.removeClass(Fn) : dr.addClass(Fn);
              else (nn === void 0 || yr === "boolean") && (Fn = Sr(this), Fn && Pt.set(this, "__className__", Fn), this.setAttribute && this.setAttribute(
                "class",
                Fn || nn === !1 ? "" : Pt.get(this, "__className__") || ""
              ));
            }));
          },
          hasClass: function(nn) {
            var Tn, Dn, Fn = 0;
            for (Tn = " " + nn + " "; Dn = this[Fn++]; )
              if (Dn.nodeType === 1 && (" " + Ca(Sr(Dn)) + " ").indexOf(Tn) > -1)
                return !0;
            return !1;
          }
        });
        var si = /\r/g;
        ge.fn.extend({
          val: function(nn) {
            var Tn, Dn, Fn, fr = this[0];
            return arguments.length ? (Fn = le(nn), this.each(function(dr) {
              var yr;
              this.nodeType === 1 && (Fn ? yr = nn.call(this, dr, ge(this).val()) : yr = nn, yr == null ? yr = "" : typeof yr == "number" ? yr += "" : Array.isArray(yr) && (yr = ge.map(yr, function(ai) {
                return ai == null ? "" : ai + "";
              })), Tn = ge.valHooks[this.type] || ge.valHooks[this.nodeName.toLowerCase()], (!Tn || !("set" in Tn) || Tn.set(this, yr, "value") === void 0) && (this.value = yr));
            })) : fr ? (Tn = ge.valHooks[fr.type] || ge.valHooks[fr.nodeName.toLowerCase()], Tn && "get" in Tn && (Dn = Tn.get(fr, "value")) !== void 0 ? Dn : (Dn = fr.value, typeof Dn == "string" ? Dn.replace(si, "") : Dn ?? "")) : void 0;
          }
        }), ge.extend({
          valHooks: {
            option: {
              get: function(nn) {
                var Tn = ge.find.attr(nn, "value");
                return Tn ?? // Support: IE <=10 - 11 only
                // option.text throws exceptions (trac-14686, trac-14858)
                // Strip and collapse whitespace
                // https://html.spec.whatwg.org/#strip-and-collapse-whitespace
                Ca(ge.text(nn));
              }
            },
            select: {
              get: function(nn) {
                var Tn, Dn, Fn, fr = nn.options, dr = nn.selectedIndex, yr = nn.type === "select-one", ai = yr ? null : [], Fr = yr ? dr + 1 : fr.length;
                for (dr < 0 ? Fn = Fr : Fn = yr ? dr : 0; Fn < Fr; Fn++)
                  if (Dn = fr[Fn], (Dn.selected || Fn === dr) && // Don't return options that are disabled or in a disabled optgroup
                  !Dn.disabled && (!Dn.parentNode.disabled || !be(Dn.parentNode, "optgroup"))) {
                    if (Tn = ge(Dn).val(), yr)
                      return Tn;
                    ai.push(Tn);
                  }
                return ai;
              },
              set: function(nn, Tn) {
                for (var Dn, Fn, fr = nn.options, dr = ge.makeArray(Tn), yr = fr.length; yr--; )
                  Fn = fr[yr], (Fn.selected = ge.inArray(ge.valHooks.option.get(Fn), dr) > -1) && (Dn = !0);
                return Dn || (nn.selectedIndex = -1), dr;
              }
            }
          }
        }), ge.each(["radio", "checkbox"], function() {
          ge.valHooks[this] = {
            set: function(nn, Tn) {
              if (Array.isArray(Tn))
                return nn.checked = ge.inArray(ge(nn).val(), Tn) > -1;
            }
          }, ce.checkOn || (ge.valHooks[this].get = function(nn) {
            return nn.getAttribute("value") === null ? "on" : nn.value;
          });
        });
        var zr = _.location, di = { guid: Date.now() }, Da = /\?/;
        ge.parseXML = function(nn) {
          var Tn, Dn;
          if (!nn || typeof nn != "string")
            return null;
          try {
            Tn = new _.DOMParser().parseFromString(nn, "text/xml");
          } catch {
          }
          return Dn = Tn && Tn.getElementsByTagName("parsererror")[0], (!Tn || Dn) && ge.error("Invalid XML: " + (Dn ? ge.map(Dn.childNodes, function(Fn) {
            return Fn.textContent;
          }).join(`
`) : nn)), Tn;
        };
        var la = /^(?:focusinfocus|focusoutblur)$/, wa = function(nn) {
          nn.stopPropagation();
        };
        ge.extend(ge.event, {
          trigger: function(nn, Tn, Dn, Fn) {
            var fr, dr, yr, ai, Fr, Ti, Si, Ei, Fa = [Dn || de], $o = ae.call(nn, "type") ? nn.type : nn, Gs = ae.call(nn, "namespace") ? nn.namespace.split(".") : [];
            if (dr = Ei = yr = Dn = Dn || de, !(Dn.nodeType === 3 || Dn.nodeType === 8) && !la.test($o + ge.event.triggered) && ($o.indexOf(".") > -1 && (Gs = $o.split("."), $o = Gs.shift(), Gs.sort()), Fr = $o.indexOf(":") < 0 && "on" + $o, nn = nn[ge.expando] ? nn : new ge.Event($o, typeof nn == "object" && nn), nn.isTrigger = Fn ? 2 : 3, nn.namespace = Gs.join("."), nn.rnamespace = nn.namespace ? new RegExp("(^|\\.)" + Gs.join("\\.(?:.*\\.|)") + "(\\.|$)") : null, nn.result = void 0, nn.target || (nn.target = Dn), Tn = Tn == null ? [nn] : ge.makeArray(Tn, [nn]), Si = ge.event.special[$o] || {}, !(!Fn && Si.trigger && Si.trigger.apply(Dn, Tn) === !1))) {
              if (!Fn && !Si.noBubble && !ue(Dn)) {
                for (ai = Si.delegateType || $o, la.test(ai + $o) || (dr = dr.parentNode); dr; dr = dr.parentNode)
                  Fa.push(dr), yr = dr;
                yr === (Dn.ownerDocument || de) && Fa.push(yr.defaultView || yr.parentWindow || _);
              }
              for (fr = 0; (dr = Fa[fr++]) && !nn.isPropagationStopped(); )
                Ei = dr, nn.type = fr > 1 ? ai : Si.bindType || $o, Ti = (Pt.get(dr, "events") || /* @__PURE__ */ Object.create(null))[nn.type] && Pt.get(dr, "handle"), Ti && Ti.apply(dr, Tn), Ti = Fr && dr[Fr], Ti && Ti.apply && vt(dr) && (nn.result = Ti.apply(dr, Tn), nn.result === !1 && nn.preventDefault());
              return nn.type = $o, !Fn && !nn.isDefaultPrevented() && (!Si._default || Si._default.apply(Fa.pop(), Tn) === !1) && vt(Dn) && Fr && le(Dn[$o]) && !ue(Dn) && (yr = Dn[Fr], yr && (Dn[Fr] = null), ge.event.triggered = $o, nn.isPropagationStopped() && Ei.addEventListener($o, wa), Dn[$o](), nn.isPropagationStopped() && Ei.removeEventListener($o, wa), ge.event.triggered = void 0, yr && (Dn[Fr] = yr)), nn.result;
            }
          },
          // Piggyback on a donor event to simulate a different one
          // Used only for `focus(in | out)` events
          simulate: function(nn, Tn, Dn) {
            var Fn = ge.extend(
              new ge.Event(),
              Dn,
              {
                type: nn,
                isSimulated: !0
              }
            );
            ge.event.trigger(Fn, null, Tn);
          }
        }), ge.fn.extend({
          trigger: function(nn, Tn) {
            return this.each(function() {
              ge.event.trigger(nn, Tn, this);
            });
          },
          triggerHandler: function(nn, Tn) {
            var Dn = this[0];
            if (Dn)
              return ge.event.trigger(nn, Tn, Dn, !0);
          }
        });
        var Wa = /\[\]$/, Qr = /\r?\n/g, Ur = /^(?:submit|button|image|reset|file)$/i, rr = /^(?:input|select|textarea|keygen)/i;
        function vr(nn, Tn, Dn, Fn) {
          var fr;
          if (Array.isArray(Tn))
            ge.each(Tn, function(dr, yr) {
              Dn || Wa.test(nn) ? Fn(nn, yr) : vr(
                nn + "[" + (typeof yr == "object" && yr != null ? dr : "") + "]",
                yr,
                Dn,
                Fn
              );
            });
          else if (!Dn && me(Tn) === "object")
            for (fr in Tn)
              vr(nn + "[" + fr + "]", Tn[fr], Dn, Fn);
          else
            Fn(nn, Tn);
        }
        ge.param = function(nn, Tn) {
          var Dn, Fn = [], fr = function(dr, yr) {
            var ai = le(yr) ? yr() : yr;
            Fn[Fn.length] = encodeURIComponent(dr) + "=" + encodeURIComponent(ai ?? "");
          };
          if (nn == null)
            return "";
          if (Array.isArray(nn) || nn.jquery && !ge.isPlainObject(nn))
            ge.each(nn, function() {
              fr(this.name, this.value);
            });
          else
            for (Dn in nn)
              vr(Dn, nn[Dn], Tn, fr);
          return Fn.join("&");
        }, ge.fn.extend({
          serialize: function() {
            return ge.param(this.serializeArray());
          },
          serializeArray: function() {
            return this.map(function() {
              var nn = ge.prop(this, "elements");
              return nn ? ge.makeArray(nn) : this;
            }).filter(function() {
              var nn = this.type;
              return this.name && !ge(this).is(":disabled") && rr.test(this.nodeName) && !Ur.test(nn) && (this.checked || !vn.test(nn));
            }).map(function(nn, Tn) {
              var Dn = ge(this).val();
              return Dn == null ? null : Array.isArray(Dn) ? ge.map(Dn, function(Fn) {
                return { name: Tn.name, value: Fn.replace(Qr, `\r
`) };
              }) : { name: Tn.name, value: Dn.replace(Qr, `\r
`) };
            }).get();
          }
        });
        var Rr = /%20/g, Vr = /#.*$/, Xr = /([?&])_=[^&]*/, Jr = /^(.*?):[ \t]*([^\r\n]*)$/mg, ti = /^(?:about|app|app-storage|.+-extension|file|res|widget):$/, Hr = /^(?:GET|HEAD)$/, Er = /^\/\//, gr = {}, $r = {}, li = "*/".concat("*"), Qi = de.createElement("a");
        Qi.href = zr.href;
        function Ai(nn) {
          return function(Tn, Dn) {
            typeof Tn != "string" && (Dn = Tn, Tn = "*");
            var Fn, fr = 0, dr = Tn.toLowerCase().match(at) || [];
            if (le(Dn))
              for (; Fn = dr[fr++]; )
                Fn[0] === "+" ? (Fn = Fn.slice(1) || "*", (nn[Fn] = nn[Fn] || []).unshift(Dn)) : (nn[Fn] = nn[Fn] || []).push(Dn);
          };
        }
        function Li(nn, Tn, Dn, Fn) {
          var fr = {}, dr = nn === $r;
          function yr(ai) {
            var Fr;
            return fr[ai] = !0, ge.each(nn[ai] || [], function(Ti, Si) {
              var Ei = Si(Tn, Dn, Fn);
              if (typeof Ei == "string" && !dr && !fr[Ei])
                return Tn.dataTypes.unshift(Ei), yr(Ei), !1;
              if (dr)
                return !(Fr = Ei);
            }), Fr;
          }
          return yr(Tn.dataTypes[0]) || !fr["*"] && yr("*");
        }
        function Ni(nn, Tn) {
          var Dn, Fn, fr = ge.ajaxSettings.flatOptions || {};
          for (Dn in Tn)
            Tn[Dn] !== void 0 && ((fr[Dn] ? nn : Fn || (Fn = {}))[Dn] = Tn[Dn]);
          return Fn && ge.extend(!0, nn, Fn), nn;
        }
        function Pi(nn, Tn, Dn) {
          for (var Fn, fr, dr, yr, ai = nn.contents, Fr = nn.dataTypes; Fr[0] === "*"; )
            Fr.shift(), Fn === void 0 && (Fn = nn.mimeType || Tn.getResponseHeader("Content-Type"));
          if (Fn) {
            for (fr in ai)
              if (ai[fr] && ai[fr].test(Fn)) {
                Fr.unshift(fr);
                break;
              }
          }
          if (Fr[0] in Dn)
            dr = Fr[0];
          else {
            for (fr in Dn) {
              if (!Fr[0] || nn.converters[fr + " " + Fr[0]]) {
                dr = fr;
                break;
              }
              yr || (yr = fr);
            }
            dr = dr || yr;
          }
          if (dr)
            return dr !== Fr[0] && Fr.unshift(dr), Dn[dr];
        }
        function Oi(nn, Tn, Dn, Fn) {
          var fr, dr, yr, ai, Fr, Ti = {}, Si = nn.dataTypes.slice();
          if (Si[1])
            for (yr in nn.converters)
              Ti[yr.toLowerCase()] = nn.converters[yr];
          for (dr = Si.shift(); dr; )
            if (nn.responseFields[dr] && (Dn[nn.responseFields[dr]] = Tn), !Fr && Fn && nn.dataFilter && (Tn = nn.dataFilter(Tn, nn.dataType)), Fr = dr, dr = Si.shift(), dr) {
              if (dr === "*")
                dr = Fr;
              else if (Fr !== "*" && Fr !== dr) {
                if (yr = Ti[Fr + " " + dr] || Ti["* " + dr], !yr) {
                  for (fr in Ti)
                    if (ai = fr.split(" "), ai[1] === dr && (yr = Ti[Fr + " " + ai[0]] || Ti["* " + ai[0]], yr)) {
                      yr === !0 ? yr = Ti[fr] : Ti[fr] !== !0 && (dr = ai[0], Si.unshift(ai[1]));
                      break;
                    }
                }
                if (yr !== !0)
                  if (yr && nn.throws)
                    Tn = yr(Tn);
                  else
                    try {
                      Tn = yr(Tn);
                    } catch (Ei) {
                      return {
                        state: "parsererror",
                        error: yr ? Ei : "No conversion from " + Fr + " to " + dr
                      };
                    }
              }
            }
          return { state: "success", data: Tn };
        }
        ge.extend({
          // Counter for holding the number of active queries
          active: 0,
          // Last-Modified header cache for next request
          lastModified: {},
          etag: {},
          ajaxSettings: {
            url: zr.href,
            type: "GET",
            isLocal: ti.test(zr.protocol),
            global: !0,
            processData: !0,
            async: !0,
            contentType: "application/x-www-form-urlencoded; charset=UTF-8",
            /*
            timeout: 0,
            data: null,
            dataType: null,
            username: null,
            password: null,
            cache: null,
            throws: false,
            traditional: false,
            headers: {},
            */
            accepts: {
              "*": li,
              text: "text/plain",
              html: "text/html",
              xml: "application/xml, text/xml",
              json: "application/json, text/javascript"
            },
            contents: {
              xml: /\bxml\b/,
              html: /\bhtml/,
              json: /\bjson\b/
            },
            responseFields: {
              xml: "responseXML",
              text: "responseText",
              json: "responseJSON"
            },
            // Data converters
            // Keys separate source (or catchall "*") and destination types with a single space
            converters: {
              // Convert anything to text
              "* text": String,
              // Text to html (true = no transformation)
              "text html": !0,
              // Evaluate text as a json expression
              "text json": JSON.parse,
              // Parse text as xml
              "text xml": ge.parseXML
            },
            // For options that shouldn't be deep extended:
            // you can add your own custom options here if
            // and when you create one that shouldn't be
            // deep extended (see ajaxExtend)
            flatOptions: {
              url: !0,
              context: !0
            }
          },
          // Creates a full fledged settings object into target
          // with both ajaxSettings and settings fields.
          // If target is omitted, writes into ajaxSettings.
          ajaxSetup: function(nn, Tn) {
            return Tn ? (
              // Building a settings object
              Ni(Ni(nn, ge.ajaxSettings), Tn)
            ) : (
              // Extending ajaxSettings
              Ni(ge.ajaxSettings, nn)
            );
          },
          ajaxPrefilter: Ai(gr),
          ajaxTransport: Ai($r),
          // Main method
          ajax: function(nn, Tn) {
            typeof nn == "object" && (Tn = nn, nn = void 0), Tn = Tn || {};
            var Dn, Fn, fr, dr, yr, ai, Fr, Ti, Si, Ei, Fa = ge.ajaxSetup({}, Tn), $o = Fa.context || Fa, Gs = Fa.context && ($o.nodeType || $o.jquery) ? ge($o) : ge.event, Al = ge.Deferred(), ul = ge.Callbacks("once memory"), gc = Fa.statusCode || {}, Gl = {}, Sl = {}, ls = "canceled", vs = {
              readyState: 0,
              // Builds headers hashtable if needed
              getResponseHeader: function(Ll) {
                var Xl;
                if (Fr) {
                  if (!dr)
                    for (dr = {}; Xl = Jr.exec(fr); )
                      dr[Xl[1].toLowerCase() + " "] = (dr[Xl[1].toLowerCase() + " "] || []).concat(Xl[2]);
                  Xl = dr[Ll.toLowerCase() + " "];
                }
                return Xl == null ? null : Xl.join(", ");
              },
              // Raw string
              getAllResponseHeaders: function() {
                return Fr ? fr : null;
              },
              // Caches the header
              setRequestHeader: function(Ll, Xl) {
                return Fr == null && (Ll = Sl[Ll.toLowerCase()] = Sl[Ll.toLowerCase()] || Ll, Gl[Ll] = Xl), this;
              },
              // Overrides response content-type header
              overrideMimeType: function(Ll) {
                return Fr == null && (Fa.mimeType = Ll), this;
              },
              // Status-dependent callbacks
              statusCode: function(Ll) {
                var Xl;
                if (Ll)
                  if (Fr)
                    vs.always(Ll[vs.status]);
                  else
                    for (Xl in Ll)
                      gc[Xl] = [gc[Xl], Ll[Xl]];
                return this;
              },
              // Cancel the request
              abort: function(Ll) {
                var Xl = Ll || ls;
                return Dn && Dn.abort(Xl), Vl(0, Xl), this;
              }
            };
            if (Al.promise(vs), Fa.url = ((nn || Fa.url || zr.href) + "").replace(Er, zr.protocol + "//"), Fa.type = Tn.method || Tn.type || Fa.method || Fa.type, Fa.dataTypes = (Fa.dataType || "*").toLowerCase().match(at) || [""], Fa.crossDomain == null) {
              ai = de.createElement("a");
              try {
                ai.href = Fa.url, ai.href = ai.href, Fa.crossDomain = Qi.protocol + "//" + Qi.host != ai.protocol + "//" + ai.host;
              } catch {
                Fa.crossDomain = !0;
              }
            }
            if (Fa.data && Fa.processData && typeof Fa.data != "string" && (Fa.data = ge.param(Fa.data, Fa.traditional)), Li(gr, Fa, Tn, vs), Fr)
              return vs;
            Ti = ge.event && Fa.global, Ti && ge.active++ === 0 && ge.event.trigger("ajaxStart"), Fa.type = Fa.type.toUpperCase(), Fa.hasContent = !Hr.test(Fa.type), Fn = Fa.url.replace(Vr, ""), Fa.hasContent ? Fa.data && Fa.processData && (Fa.contentType || "").indexOf("application/x-www-form-urlencoded") === 0 && (Fa.data = Fa.data.replace(Rr, "+")) : (Ei = Fa.url.slice(Fn.length), Fa.data && (Fa.processData || typeof Fa.data == "string") && (Fn += (Da.test(Fn) ? "&" : "?") + Fa.data, delete Fa.data), Fa.cache === !1 && (Fn = Fn.replace(Xr, "$1"), Ei = (Da.test(Fn) ? "&" : "?") + "_=" + di.guid++ + Ei), Fa.url = Fn + Ei), Fa.ifModified && (ge.lastModified[Fn] && vs.setRequestHeader("If-Modified-Since", ge.lastModified[Fn]), ge.etag[Fn] && vs.setRequestHeader("If-None-Match", ge.etag[Fn])), (Fa.data && Fa.hasContent && Fa.contentType !== !1 || Tn.contentType) && vs.setRequestHeader("Content-Type", Fa.contentType), vs.setRequestHeader(
              "Accept",
              Fa.dataTypes[0] && Fa.accepts[Fa.dataTypes[0]] ? Fa.accepts[Fa.dataTypes[0]] + (Fa.dataTypes[0] !== "*" ? ", " + li + "; q=0.01" : "") : Fa.accepts["*"]
            );
            for (Si in Fa.headers)
              vs.setRequestHeader(Si, Fa.headers[Si]);
            if (Fa.beforeSend && (Fa.beforeSend.call($o, vs, Fa) === !1 || Fr))
              return vs.abort();
            if (ls = "abort", ul.add(Fa.complete), vs.done(Fa.success), vs.fail(Fa.error), Dn = Li($r, Fa, Tn, vs), !Dn)
              Vl(-1, "No Transport");
            else {
              if (vs.readyState = 1, Ti && Gs.trigger("ajaxSend", [vs, Fa]), Fr)
                return vs;
              Fa.async && Fa.timeout > 0 && (yr = _.setTimeout(function() {
                vs.abort("timeout");
              }, Fa.timeout));
              try {
                Fr = !1, Dn.send(Gl, Vl);
              } catch (Ll) {
                if (Fr)
                  throw Ll;
                Vl(-1, Ll);
              }
            }
            function Vl(Ll, Xl, yc, gu) {
              var Nc, Cu, zc, Kc, zi, Fu = Xl;
              Fr || (Fr = !0, yr && _.clearTimeout(yr), Dn = void 0, fr = gu || "", vs.readyState = Ll > 0 ? 4 : 0, Nc = Ll >= 200 && Ll < 300 || Ll === 304, yc && (Kc = Pi(Fa, vs, yc)), !Nc && ge.inArray("script", Fa.dataTypes) > -1 && ge.inArray("json", Fa.dataTypes) < 0 && (Fa.converters["text script"] = function() {
              }), Kc = Oi(Fa, Kc, vs, Nc), Nc ? (Fa.ifModified && (zi = vs.getResponseHeader("Last-Modified"), zi && (ge.lastModified[Fn] = zi), zi = vs.getResponseHeader("etag"), zi && (ge.etag[Fn] = zi)), Ll === 204 || Fa.type === "HEAD" ? Fu = "nocontent" : Ll === 304 ? Fu = "notmodified" : (Fu = Kc.state, Cu = Kc.data, zc = Kc.error, Nc = !zc)) : (zc = Fu, (Ll || !Fu) && (Fu = "error", Ll < 0 && (Ll = 0))), vs.status = Ll, vs.statusText = (Xl || Fu) + "", Nc ? Al.resolveWith($o, [Cu, Fu, vs]) : Al.rejectWith($o, [vs, Fu, zc]), vs.statusCode(gc), gc = void 0, Ti && Gs.trigger(
                Nc ? "ajaxSuccess" : "ajaxError",
                [vs, Fa, Nc ? Cu : zc]
              ), ul.fireWith($o, [vs, Fu]), Ti && (Gs.trigger("ajaxComplete", [vs, Fa]), --ge.active || ge.event.trigger("ajaxStop")));
            }
            return vs;
          },
          getJSON: function(nn, Tn, Dn) {
            return ge.get(nn, Tn, Dn, "json");
          },
          getScript: function(nn, Tn) {
            return ge.get(nn, void 0, Tn, "script");
          }
        }), ge.each(["get", "post"], function(nn, Tn) {
          ge[Tn] = function(Dn, Fn, fr, dr) {
            return le(Fn) && (dr = dr || fr, fr = Fn, Fn = void 0), ge.ajax(ge.extend({
              url: Dn,
              type: Tn,
              dataType: dr,
              data: Fn,
              success: fr
            }, ge.isPlainObject(Dn) && Dn));
          };
        }), ge.ajaxPrefilter(function(nn) {
          var Tn;
          for (Tn in nn.headers)
            Tn.toLowerCase() === "content-type" && (nn.contentType = nn.headers[Tn] || "");
        }), ge._evalUrl = function(nn, Tn, Dn) {
          return ge.ajax({
            url: nn,
            // Make this explicit, since user can override this through ajaxSetup (trac-11264)
            type: "GET",
            dataType: "script",
            cache: !0,
            async: !1,
            global: !1,
            // Only evaluate the response if it is successful (gh-4126)
            // dataFilter is not invoked for failure responses, so using it instead
            // of the default converter is kludgy but it works.
            converters: {
              "text script": function() {
              }
            },
            dataFilter: function(Fn) {
              ge.globalEval(Fn, Tn, Dn);
            }
          });
        }, ge.fn.extend({
          wrapAll: function(nn) {
            var Tn;
            return this[0] && (le(nn) && (nn = nn.call(this[0])), Tn = ge(nn, this[0].ownerDocument).eq(0).clone(!0), this[0].parentNode && Tn.insertBefore(this[0]), Tn.map(function() {
              for (var Dn = this; Dn.firstElementChild; )
                Dn = Dn.firstElementChild;
              return Dn;
            }).append(this)), this;
          },
          wrapInner: function(nn) {
            return le(nn) ? this.each(function(Tn) {
              ge(this).wrapInner(nn.call(this, Tn));
            }) : this.each(function() {
              var Tn = ge(this), Dn = Tn.contents();
              Dn.length ? Dn.wrapAll(nn) : Tn.append(nn);
            });
          },
          wrap: function(nn) {
            var Tn = le(nn);
            return this.each(function(Dn) {
              ge(this).wrapAll(Tn ? nn.call(this, Dn) : nn);
            });
          },
          unwrap: function(nn) {
            return this.parent(nn).not("body").each(function() {
              ge(this).replaceWith(this.childNodes);
            }), this;
          }
        }), ge.expr.pseudos.hidden = function(nn) {
          return !ge.expr.pseudos.visible(nn);
        }, ge.expr.pseudos.visible = function(nn) {
          return !!(nn.offsetWidth || nn.offsetHeight || nn.getClientRects().length);
        }, ge.ajaxSettings.xhr = function() {
          try {
            return new _.XMLHttpRequest();
          } catch {
          }
        };
        var ki = {
          // File protocol always yields status code 0, assume 200
          0: 200,
          // Support: IE <=9 only
          // trac-1450: sometimes IE returns 1223 when it should be 204
          1223: 204
        }, Mi = ge.ajaxSettings.xhr();
        ce.cors = !!Mi && "withCredentials" in Mi, ce.ajax = Mi = !!Mi, ge.ajaxTransport(function(nn) {
          var Tn, Dn;
          if (ce.cors || Mi && !nn.crossDomain)
            return {
              send: function(Fn, fr) {
                var dr, yr = nn.xhr();
                if (yr.open(
                  nn.type,
                  nn.url,
                  nn.async,
                  nn.username,
                  nn.password
                ), nn.xhrFields)
                  for (dr in nn.xhrFields)
                    yr[dr] = nn.xhrFields[dr];
                nn.mimeType && yr.overrideMimeType && yr.overrideMimeType(nn.mimeType), !nn.crossDomain && !Fn["X-Requested-With"] && (Fn["X-Requested-With"] = "XMLHttpRequest");
                for (dr in Fn)
                  yr.setRequestHeader(dr, Fn[dr]);
                Tn = function(ai) {
                  return function() {
                    Tn && (Tn = Dn = yr.onload = yr.onerror = yr.onabort = yr.ontimeout = yr.onreadystatechange = null, ai === "abort" ? yr.abort() : ai === "error" ? typeof yr.status != "number" ? fr(0, "error") : fr(
                      // File: protocol always yields status 0; see trac-8605, trac-14207
                      yr.status,
                      yr.statusText
                    ) : fr(
                      ki[yr.status] || yr.status,
                      yr.statusText,
                      // Support: IE <=9 only
                      // IE9 has no XHR2 but throws on binary (trac-11426)
                      // For XHR2 non-text, let the caller handle it (gh-2498)
                      (yr.responseType || "text") !== "text" || typeof yr.responseText != "string" ? { binary: yr.response } : { text: yr.responseText },
                      yr.getAllResponseHeaders()
                    ));
                  };
                }, yr.onload = Tn(), Dn = yr.onerror = yr.ontimeout = Tn("error"), yr.onabort !== void 0 ? yr.onabort = Dn : yr.onreadystatechange = function() {
                  yr.readyState === 4 && _.setTimeout(function() {
                    Tn && Dn();
                  });
                }, Tn = Tn("abort");
                try {
                  yr.send(nn.hasContent && nn.data || null);
                } catch (ai) {
                  if (Tn)
                    throw ai;
                }
              },
              abort: function() {
                Tn && Tn();
              }
            };
        }), ge.ajaxPrefilter(function(nn) {
          nn.crossDomain && (nn.contents.script = !1);
        }), ge.ajaxSetup({
          accepts: {
            script: "text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"
          },
          contents: {
            script: /\b(?:java|ecma)script\b/
          },
          converters: {
            "text script": function(nn) {
              return ge.globalEval(nn), nn;
            }
          }
        }), ge.ajaxPrefilter("script", function(nn) {
          nn.cache === void 0 && (nn.cache = !1), nn.crossDomain && (nn.type = "GET");
        }), ge.ajaxTransport("script", function(nn) {
          if (nn.crossDomain || nn.scriptAttrs) {
            var Tn, Dn;
            return {
              send: function(Fn, fr) {
                Tn = ge("<script>").attr(nn.scriptAttrs || {}).prop({ charset: nn.scriptCharset, src: nn.url }).on("load error", Dn = function(dr) {
                  Tn.remove(), Dn = null, dr && fr(dr.type === "error" ? 404 : 200, dr.type);
                }), de.head.appendChild(Tn[0]);
              },
              abort: function() {
                Dn && Dn();
              }
            };
          }
        });
        var Vi = [], ua = /(=)\?(?=&|$)|\?\?/;
        ge.ajaxSetup({
          jsonp: "callback",
          jsonpCallback: function() {
            var nn = Vi.pop() || ge.expando + "_" + di.guid++;
            return this[nn] = !0, nn;
          }
        }), ge.ajaxPrefilter("json jsonp", function(nn, Tn, Dn) {
          var Fn, fr, dr, yr = nn.jsonp !== !1 && (ua.test(nn.url) ? "url" : typeof nn.data == "string" && (nn.contentType || "").indexOf("application/x-www-form-urlencoded") === 0 && ua.test(nn.data) && "data");
          if (yr || nn.dataTypes[0] === "jsonp")
            return Fn = nn.jsonpCallback = le(nn.jsonpCallback) ? nn.jsonpCallback() : nn.jsonpCallback, yr ? nn[yr] = nn[yr].replace(ua, "$1" + Fn) : nn.jsonp !== !1 && (nn.url += (Da.test(nn.url) ? "&" : "?") + nn.jsonp + "=" + Fn), nn.converters["script json"] = function() {
              return dr || ge.error(Fn + " was not called"), dr[0];
            }, nn.dataTypes[0] = "json", fr = _[Fn], _[Fn] = function() {
              dr = arguments;
            }, Dn.always(function() {
              fr === void 0 ? ge(_).removeProp(Fn) : _[Fn] = fr, nn[Fn] && (nn.jsonpCallback = Tn.jsonpCallback, Vi.push(Fn)), dr && le(fr) && fr(dr[0]), dr = fr = void 0;
            }), "script";
        }), ce.createHTMLDocument = function() {
          var nn = de.implementation.createHTMLDocument("").body;
          return nn.innerHTML = "<form></form><form></form>", nn.childNodes.length === 2;
        }(), ge.parseHTML = function(nn, Tn, Dn) {
          if (typeof nn != "string")
            return [];
          typeof Tn == "boolean" && (Dn = Tn, Tn = !1);
          var Fn, fr, dr;
          return Tn || (ce.createHTMLDocument ? (Tn = de.implementation.createHTMLDocument(""), Fn = Tn.createElement("base"), Fn.href = de.location.href, Tn.head.appendChild(Fn)) : Tn = de), fr = $e.exec(nn), dr = !Dn && [], fr ? [Tn.createElement(fr[1])] : (fr = Pn([nn], Tn, dr), dr && dr.length && ge(dr).remove(), ge.merge([], fr.childNodes));
        }, ge.fn.load = function(nn, Tn, Dn) {
          var Fn, fr, dr, yr = this, ai = nn.indexOf(" ");
          return ai > -1 && (Fn = Ca(nn.slice(ai)), nn = nn.slice(0, ai)), le(Tn) ? (Dn = Tn, Tn = void 0) : Tn && typeof Tn == "object" && (fr = "POST"), yr.length > 0 && ge.ajax({
            url: nn,
            // If "type" variable is undefined, then "GET" method will be used.
            // Make value of this field explicit since
            // user can override it through ajaxSetup method
            type: fr || "GET",
            dataType: "html",
            data: Tn
          }).done(function(Fr) {
            dr = arguments, yr.html(Fn ? (
              // If a selector was specified, locate the right elements in a dummy div
              // Exclude scripts to avoid IE 'Permission Denied' errors
              ge("<div>").append(ge.parseHTML(Fr)).find(Fn)
            ) : (
              // Otherwise use the full result
              Fr
            ));
          }).always(Dn && function(Fr, Ti) {
            yr.each(function() {
              Dn.apply(this, dr || [Fr.responseText, Ti, Fr]);
            });
          }), this;
        }, ge.expr.pseudos.animated = function(nn) {
          return ge.grep(ge.timers, function(Tn) {
            return nn === Tn.elem;
          }).length;
        }, ge.offset = {
          setOffset: function(nn, Tn, Dn) {
            var Fn, fr, dr, yr, ai, Fr, Ti, Si = ge.css(nn, "position"), Ei = ge(nn), Fa = {};
            Si === "static" && (nn.style.position = "relative"), ai = Ei.offset(), dr = ge.css(nn, "top"), Fr = ge.css(nn, "left"), Ti = (Si === "absolute" || Si === "fixed") && (dr + Fr).indexOf("auto") > -1, Ti ? (Fn = Ei.position(), yr = Fn.top, fr = Fn.left) : (yr = parseFloat(dr) || 0, fr = parseFloat(Fr) || 0), le(Tn) && (Tn = Tn.call(nn, Dn, ge.extend({}, ai))), Tn.top != null && (Fa.top = Tn.top - ai.top + yr), Tn.left != null && (Fa.left = Tn.left - ai.left + fr), "using" in Tn ? Tn.using.call(nn, Fa) : Ei.css(Fa);
          }
        }, ge.fn.extend({
          // offset() relates an element's border box to the document origin
          offset: function(nn) {
            if (arguments.length)
              return nn === void 0 ? this : this.each(function(fr) {
                ge.offset.setOffset(this, nn, fr);
              });
            var Tn, Dn, Fn = this[0];
            if (Fn)
              return Fn.getClientRects().length ? (Tn = Fn.getBoundingClientRect(), Dn = Fn.ownerDocument.defaultView, {
                top: Tn.top + Dn.pageYOffset,
                left: Tn.left + Dn.pageXOffset
              }) : { top: 0, left: 0 };
          },
          // position() relates an element's margin box to its offset parent's padding box
          // This corresponds to the behavior of CSS absolute positioning
          position: function() {
            if (this[0]) {
              var nn, Tn, Dn, Fn = this[0], fr = { top: 0, left: 0 };
              if (ge.css(Fn, "position") === "fixed")
                Tn = Fn.getBoundingClientRect();
              else {
                for (Tn = this.offset(), Dn = Fn.ownerDocument, nn = Fn.offsetParent || Dn.documentElement; nn && (nn === Dn.body || nn === Dn.documentElement) && ge.css(nn, "position") === "static"; )
                  nn = nn.parentNode;
                nn && nn !== Fn && nn.nodeType === 1 && (fr = ge(nn).offset(), fr.top += ge.css(nn, "borderTopWidth", !0), fr.left += ge.css(nn, "borderLeftWidth", !0));
              }
              return {
                top: Tn.top - fr.top - ge.css(Fn, "marginTop", !0),
                left: Tn.left - fr.left - ge.css(Fn, "marginLeft", !0)
              };
            }
          },
          // This method will return documentElement in the following cases:
          // 1) For the element inside the iframe without offsetParent, this method will return
          //    documentElement of the parent window
          // 2) For the hidden or detached element
          // 3) For body or html element, i.e. in case of the html node - it will return itself
          //
          // but those exceptions were never presented as a real life use-cases
          // and might be considered as more preferable results.
          //
          // This logic, however, is not guaranteed and can change at any point in the future
          offsetParent: function() {
            return this.map(function() {
              for (var nn = this.offsetParent; nn && ge.css(nn, "position") === "static"; )
                nn = nn.offsetParent;
              return nn || Vt;
            });
          }
        }), ge.each({ scrollLeft: "pageXOffset", scrollTop: "pageYOffset" }, function(nn, Tn) {
          var Dn = Tn === "pageYOffset";
          ge.fn[nn] = function(Fn) {
            return bt(this, function(fr, dr, yr) {
              var ai;
              if (ue(fr) ? ai = fr : fr.nodeType === 9 && (ai = fr.defaultView), yr === void 0)
                return ai ? ai[Tn] : fr[dr];
              ai ? ai.scrollTo(
                Dn ? ai.pageXOffset : yr,
                Dn ? yr : ai.pageYOffset
              ) : fr[dr] = yr;
            }, nn, Fn, arguments.length);
          };
        }), ge.each(["top", "left"], function(nn, Tn) {
          ge.cssHooks[Tn] = jn(
            ce.pixelPosition,
            function(Dn, Fn) {
              if (Fn)
                return Fn = er(Dn, Tn), Zt.test(Fn) ? ge(Dn).position()[Tn] + "px" : Fn;
            }
          );
        }), ge.each({ Height: "height", Width: "width" }, function(nn, Tn) {
          ge.each({
            padding: "inner" + nn,
            content: Tn,
            "": "outer" + nn
          }, function(Dn, Fn) {
            ge.fn[Fn] = function(fr, dr) {
              var yr = arguments.length && (Dn || typeof fr != "boolean"), ai = Dn || (fr === !0 || dr === !0 ? "margin" : "border");
              return bt(this, function(Fr, Ti, Si) {
                var Ei;
                return ue(Fr) ? Fn.indexOf("outer") === 0 ? Fr["inner" + nn] : Fr.document.documentElement["client" + nn] : Fr.nodeType === 9 ? (Ei = Fr.documentElement, Math.max(
                  Fr.body["scroll" + nn],
                  Ei["scroll" + nn],
                  Fr.body["offset" + nn],
                  Ei["offset" + nn],
                  Ei["client" + nn]
                )) : Si === void 0 ? (
                  // Get width or height on the element, requesting but not forcing parseFloat
                  ge.css(Fr, Ti, ai)
                ) : (
                  // Set width or height on the element
                  ge.style(Fr, Ti, Si, ai)
                );
              }, Tn, yr ? fr : void 0, yr);
            };
          });
        }), ge.each([
          "ajaxStart",
          "ajaxStop",
          "ajaxComplete",
          "ajaxError",
          "ajaxSuccess",
          "ajaxSend"
        ], function(nn, Tn) {
          ge.fn[Tn] = function(Dn) {
            return this.on(Tn, Dn);
          };
        }), ge.fn.extend({
          bind: function(nn, Tn, Dn) {
            return this.on(nn, null, Tn, Dn);
          },
          unbind: function(nn, Tn) {
            return this.off(nn, null, Tn);
          },
          delegate: function(nn, Tn, Dn, Fn) {
            return this.on(Tn, nn, Dn, Fn);
          },
          undelegate: function(nn, Tn, Dn) {
            return arguments.length === 1 ? this.off(nn, "**") : this.off(Tn, nn || "**", Dn);
          },
          hover: function(nn, Tn) {
            return this.on("mouseenter", nn).on("mouseleave", Tn || nn);
          }
        }), ge.each(
          "blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),
          function(nn, Tn) {
            ge.fn[Tn] = function(Dn, Fn) {
              return arguments.length > 0 ? this.on(Tn, null, Dn, Fn) : this.trigger(Tn);
            };
          }
        );
        var Ki = /^[\s\uFEFF\xA0]+|([^\s\uFEFF\xA0])[\s\uFEFF\xA0]+$/g;
        ge.proxy = function(nn, Tn) {
          var Dn, Fn, fr;
          if (typeof Tn == "string" && (Dn = nn[Tn], Tn = nn, nn = Dn), !!le(nn))
            return Fn = G.call(arguments, 2), fr = function() {
              return nn.apply(Tn || this, Fn.concat(G.call(arguments)));
            }, fr.guid = nn.guid = nn.guid || ge.guid++, fr;
        }, ge.holdReady = function(nn) {
          nn ? ge.readyWait++ : ge.ready(!0);
        }, ge.isArray = Array.isArray, ge.parseJSON = JSON.parse, ge.nodeName = be, ge.isFunction = le, ge.isWindow = ue, ge.camelCase = Rt, ge.type = me, ge.now = Date.now, ge.isNumeric = function(nn) {
          var Tn = ge.type(nn);
          return (Tn === "number" || Tn === "string") && // parseFloat NaNs numeric-cast false positives ("")
          // ...but misinterprets leading-number strings, particularly hex literals ("0x...")
          // subtraction forces infinities to NaN
          !isNaN(nn - parseFloat(nn));
        }, ge.trim = function(nn) {
          return nn == null ? "" : (nn + "").replace(Ki, "$1");
        };
        var Fi = _.jQuery, na = _.$;
        return ge.noConflict = function(nn) {
          return _.$ === ge && (_.$ = na), nn && _.jQuery === ge && (_.jQuery = Fi), ge;
        }, typeof H > "u" && (_.jQuery = _.$ = ge), ge;
      });
    }(jquery$1)), jquery$1.exports;
  }
  var slick_jquery, hasRequiredSlick_jquery;
  function requireSlick_jquery() {
    return hasRequiredSlick_jquery || (hasRequiredSlick_jquery = 1, slick_jquery = typeof $ < "u" ? $ : requireJquery()), slick_jquery;
  }
  var slick_core, hasRequiredSlick_core;
  function requireSlick_core() {
    if (hasRequiredSlick_core) return slick_core;
    hasRequiredSlick_core = 1;
    var O = requireSlick_jquery();
    function _() {
      var ae = !1, oe = !1;
      this.stopPropagation = function() {
        ae = !0;
      }, this.isPropagationStopped = function() {
        return ae;
      }, this.stopImmediatePropagation = function() {
        oe = !0;
      }, this.isImmediatePropagationStopped = function() {
        return oe;
      };
    }
    function H() {
      var ae = [];
      this.subscribe = function(oe) {
        ae.push(oe);
      }, this.unsubscribe = function(oe) {
        for (var se = ae.length - 1; se >= 0; se--)
          ae[se] === oe && ae.splice(se, 1);
      }, this.notify = function(oe, se, ce) {
        se = se || new _(), ce = ce || this;
        for (var le, ue = 0; ue < ae.length && !(se.isPropagationStopped() || se.isImmediatePropagationStopped()); ue++)
          le = ae[ue].call(ce, se, oe);
        return le;
      };
    }
    function V() {
      var ae = [];
      this.subscribe = function(oe, se) {
        return ae.push({
          event: oe,
          handler: se
        }), oe.subscribe(se), this;
      }, this.unsubscribe = function(oe, se) {
        for (var ce = ae.length; ce--; )
          if (ae[ce].event === oe && ae[ce].handler === se) {
            ae.splice(ce, 1), oe.unsubscribe(se);
            return;
          }
        return this;
      }, this.unsubscribeAll = function() {
        for (var oe = ae.length; oe--; )
          ae[oe].event.unsubscribe(ae[oe].handler);
        return ae = [], this;
      };
    }
    function q(ae, oe, se, ce) {
      se === void 0 && ce === void 0 && (se = ae, ce = oe), this.fromRow = Math.min(ae, se), this.fromCell = Math.min(oe, ce), this.toRow = Math.max(ae, se), this.toCell = Math.max(oe, ce), this.isSingleRow = function() {
        return this.fromRow == this.toRow;
      }, this.isSingleCell = function() {
        return this.fromRow == this.toRow && this.fromCell == this.toCell;
      }, this.contains = function(le, ue) {
        return le >= this.fromRow && le <= this.toRow && ue >= this.fromCell && ue <= this.toCell;
      }, this.toString = function() {
        return this.isSingleCell() ? "(" + this.fromRow + ":" + this.fromCell + ")" : "(" + this.fromRow + ":" + this.fromCell + " - " + this.toRow + ":" + this.toCell + ")";
      };
    }
    function G() {
      this.__nonDataRow = !0;
    }
    function J() {
      this.__group = !0, this.level = 0, this.count = 0, this.value = null, this.title = null, this.collapsed = !1, this.selectChecked = !1, this.totals = null, this.rows = [], this.groups = null, this.groupingKey = null;
    }
    J.prototype = new G(), J.prototype.equals = function(ae) {
      return this.value === ae.value && this.count === ae.count && this.collapsed === ae.collapsed && this.title === ae.title;
    };
    function Z() {
      this.__groupTotals = !0, this.group = null, this.initialized = !1;
    }
    Z.prototype = new G();
    function te() {
      var ae = null;
      this.isActive = function(oe) {
        return oe ? ae === oe : ae !== null;
      }, this.activate = function(oe) {
        if (oe !== ae) {
          if (ae !== null)
            throw new Error("SlickGrid.EditorLock.activate: an editController is still active, can't activate another editController");
          if (!oe.commitCurrentEdit)
            throw new Error("SlickGrid.EditorLock.activate: editController must implement .commitCurrentEdit()");
          if (!oe.cancelCurrentEdit)
            throw new Error("SlickGrid.EditorLock.activate: editController must implement .cancelCurrentEdit()");
          ae = oe;
        }
      }, this.deactivate = function(oe) {
        if (ae) {
          if (ae !== oe)
            throw new Error("SlickGrid.EditorLock.deactivate: specified editController is not the currently active one");
          ae = null;
        }
      }, this.commitCurrentEdit = function() {
        return ae ? ae.commitCurrentEdit() : !0;
      }, this.cancelCurrentEdit = function() {
        return ae ? ae.cancelCurrentEdit() : !0;
      };
    }
    function ne(ae) {
      var oe = {};
      function se() {
        ce(ae);
      }
      function ce(ye) {
        ye.forEach(function(ge) {
          oe[ge.id] = ge, ge.columns && ce(ge.columns);
        });
      }
      function le(ye, ge) {
        return ye.filter(function(ve) {
          var be = ge.call(ve);
          return be && ve.columns && (ve.columns = le(ve.columns, ge)), be && (!ve.columns || ve.columns.length);
        });
      }
      function ue(ye, ge) {
        ye.sort(function(ve, be) {
          var Ce = de(ge.getColumnIndex(ve.id)), Se = de(ge.getColumnIndex(be.id));
          return Ce - Se;
        }).forEach(function(ve) {
          ve.columns && ue(ve.columns, ge);
        });
      }
      function de(ye) {
        return typeof ye > "u" ? -1 : ye;
      }
      function fe(ye) {
        if (ye.length)
          for (var ge in ye)
            return fe(ye[ge]);
        else return ye.columns ? 1 + fe(ye.columns) : 1;
      }
      function he(ye, ge, ve) {
        var be = [];
        if (ve = ve || 0, ge == ve)
          return ye.length && ye.forEach(function(Se) {
            Se.columns && (Se.extractColumns = function() {
              return me(Se);
            });
          }), ye;
        for (var Ce in ye)
          ye[Ce].columns && (be = be.concat(he(ye[Ce].columns, ge, ve + 1)));
        return be;
      }
      function me(ye) {
        var ge = [];
        if (ye.hasOwnProperty("length"))
          for (var ve = 0; ve < ye.length; ve++)
            ge = ge.concat(me(ye[ve]));
        else if (ye.hasOwnProperty("columns"))
          ge = ge.concat(me(ye.columns));
        else
          return ye;
        return ge;
      }
      function pe() {
        return O.extend(!0, [], ae);
      }
      se(), this.hasDepth = function() {
        for (var ye in ae)
          if (ae[ye].hasOwnProperty("columns"))
            return !0;
        return !1;
      }, this.getTreeColumns = function() {
        return ae;
      }, this.extractColumns = function() {
        return this.hasDepth() ? me(ae) : ae;
      }, this.getDepth = function() {
        return fe(ae);
      }, this.getColumnsInDepth = function(ye) {
        return he(ae, ye);
      }, this.getColumnsInGroup = function(ye) {
        return me(ye);
      }, this.visibleColumns = function() {
        return le(pe(), function() {
          return this.visible;
        });
      }, this.filter = function(ye) {
        return le(pe(), ye);
      }, this.reOrder = function(ye) {
        return ue(ae, ye);
      }, this.getById = function(ye) {
        return oe[ye];
      }, this.getInIds = function(ye) {
        return ye.map(function(ge) {
          return oe[ge];
        });
      };
    }
    var ee = "Map" in window ? window.Map : function() {
      var oe = {};
      this.get = function(se) {
        return oe[se];
      }, this.set = function(se, ce) {
        oe[se] = ce;
      }, this.has = function(se) {
        return se in oe;
      }, this.delete = function(se) {
        delete oe[se];
      };
    };
    return slick_core = {
      Event: H,
      EventData: _,
      EventHandler: V,
      Range: q,
      Map: ee,
      NonDataRow: G,
      Group: J,
      GroupTotals: Z,
      EditorLock: te,
      /***
       * A global singleton editor lock.
       * @class GlobalEditorLock
       * @static
       * @constructor
       */
      GlobalEditorLock: new te(),
      TreeColumns: ne,
      keyCode: {
        SPACE: 8,
        BACKSPACE: 8,
        DELETE: 46,
        DOWN: 40,
        END: 35,
        ENTER: 13,
        ESCAPE: 27,
        ESC: 27,
        HOME: 36,
        INSERT: 45,
        LEFT: 37,
        PAGE_DOWN: 34,
        PAGE_UP: 33,
        RIGHT: 39,
        TAB: 9,
        UP: 38,
        A: 65,
        C: 67,
        V: 86
      },
      preClickClassName: "slick-edit-preclick",
      GridAutosizeColsMode: {
        None: "NOA",
        LegacyOff: "LOF",
        LegacyForceFit: "LFF",
        IgnoreViewport: "IGV",
        FitColsToViewport: "FCV",
        FitViewportToCols: "FVC"
      },
      ColAutosizeMode: {
        Locked: "LCK",
        Guide: "GUI",
        Content: "CON",
        ContentIntelligent: "CTI"
      },
      RowSelectionMode: {
        FirstRow: "FS1",
        FirstNRows: "FSN",
        AllRows: "ALL",
        LastRow: "LS1"
      },
      ValueFilterMode: {
        None: "NONE",
        DeDuplicate: "DEDP",
        GetGreatestAndSub: "GR8T",
        GetLongestTextAndSub: "LNSB",
        GetLongestText: "LNSC"
      },
      WidthEvalMode: {
        CanvasTextSize: "CANV",
        HTML: "HTML"
      }
    }, slick_core;
  }
  var slick_rowselectionmodel, hasRequiredSlick_rowselectionmodel;
  function requireSlick_rowselectionmodel() {
    if (hasRequiredSlick_rowselectionmodel) return slick_rowselectionmodel;
    hasRequiredSlick_rowselectionmodel = 1;
    var O = requireSlick_jquery(), _ = requireSlick_core();
    function H(V) {
      var q, G = [], J = this, Z = new _.EventHandler(), te, ne, ee = {
        selectActiveRow: !0
      };
      function ae(ve) {
        ne = O.extend(!0, {}, ee, V), q = ve, Z.subscribe(
          q.onActiveCellChanged,
          se(pe)
        ), Z.subscribe(
          q.onKeyDown,
          se(ye)
        ), Z.subscribe(
          q.onClick,
          se(ge)
        );
      }
      function oe() {
        Z.unsubscribeAll();
      }
      function se(ve) {
        return function() {
          te || (te = !0, ve.apply(this, arguments), te = !1);
        };
      }
      function ce(ve) {
        for (var be = [], Ce = 0; Ce < ve.length; Ce++)
          for (var Se = ve[Ce].fromRow; Se <= ve[Ce].toRow; Se++)
            be.push(Se);
        return be;
      }
      function le(ve) {
        for (var be = [], Ce = q.getColumns().length - 1, Se = 0; Se < ve.length; Se++)
          be.push(new _.Range(ve[Se], 0, ve[Se], Ce));
        return be;
      }
      function ue(ve, be) {
        var Ce, Se = [];
        for (Ce = ve; Ce <= be; Ce++)
          Se.push(Ce);
        for (Ce = be; Ce < ve; Ce++)
          Se.push(Ce);
        return Se;
      }
      function de() {
        return ce(G);
      }
      function fe(ve) {
        he(le(ve));
      }
      function he(ve) {
        (!G || G.length === 0) && (!ve || ve.length === 0) || (G = ve, J.onSelectedRangesChanged.notify(G));
      }
      function me() {
        return G;
      }
      function pe(ve, be) {
        ne.selectActiveRow && be.row != null && he([new _.Range(be.row, 0, be.row, q.getColumns().length - 1)]);
      }
      function ye(ve) {
        var be = q.getActiveCell();
        if (q.getOptions().multiSelect && be && ve.shiftKey && !ve.ctrlKey && !ve.altKey && !ve.metaKey && (ve.which == _.keyCode.UP || ve.which == _.keyCode.DOWN)) {
          var Ce = de();
          Ce.sort(function(Le, Qe) {
            return Le - Qe;
          }), Ce.length || (Ce = [be.row]);
          var Se = Ce[0], we = Ce[Ce.length - 1], Ee;
          if (ve.which == _.keyCode.DOWN ? Ee = be.row < we || Se == we ? ++we : ++Se : Ee = be.row < we ? --we : --Se, Ee >= 0 && Ee < q.getDataLength()) {
            q.scrollRowIntoView(Ee);
            var xe = le(ue(Se, we));
            he(xe);
          }
          ve.preventDefault(), ve.stopPropagation();
        }
      }
      function ge(ve) {
        var be = q.getCellFromEvent(ve);
        if (!be || !q.canCellBeActive(be.row, be.cell) || !q.getOptions().multiSelect || !ve.ctrlKey && !ve.shiftKey && !ve.metaKey)
          return !1;
        var Ce = ce(G), Se = O.inArray(be.row, Ce);
        if (Se === -1 && (ve.ctrlKey || ve.metaKey))
          Ce.push(be.row), q.setActiveCell(be.row, be.cell);
        else if (Se !== -1 && (ve.ctrlKey || ve.metaKey))
          Ce = O.grep(Ce, function(ke, Me) {
            return ke !== be.row;
          }), q.setActiveCell(be.row, be.cell);
        else if (Ce.length && ve.shiftKey) {
          var we = Ce.pop(), Ee = Math.min(be.row, we), xe = Math.max(be.row, we);
          Ce = [];
          for (var Le = Ee; Le <= xe; Le++)
            Le !== we && Ce.push(Le);
          Ce.push(we), q.setActiveCell(be.row, be.cell);
        }
        var Qe = le(Ce);
        return he(Qe), ve.stopImmediatePropagation(), !0;
      }
      O.extend(this, {
        getSelectedRows: de,
        setSelectedRows: fe,
        getSelectedRanges: me,
        setSelectedRanges: he,
        init: ae,
        destroy: oe,
        pluginName: "RowSelectionModel",
        onSelectedRangesChanged: new _.Event()
      });
    }
    return slick_rowselectionmodel = {
      RowSelectionModel: H
    }, slick_rowselectionmodel;
  }
  var slick_rowselectionmodelExports = requireSlick_rowselectionmodel(), slick_checkboxselectcolumn, hasRequiredSlick_checkboxselectcolumn;
  function requireSlick_checkboxselectcolumn() {
    if (hasRequiredSlick_checkboxselectcolumn) return slick_checkboxselectcolumn;
    hasRequiredSlick_checkboxselectcolumn = 1;
    var O = requireSlick_jquery(), _ = requireSlick_core();
    function H(V) {
      var q, G = null, J = Ee(), Z = new _.EventHandler(), te = {}, ne = {
        columnId: "_checkbox_selector",
        cssClass: null,
        hideSelectAllCheckbox: !1,
        toolTip: "Select/Deselect All",
        width: 30,
        hideInColumnTitleRow: !1,
        hideInFilterHeaderRow: !0
      }, ee = !1, ae = O.extend(!0, {}, ne, V);
      typeof ae.selectableOverride == "function" && ke(ae.selectableOverride);
      function oe(Me) {
        q = Me, Z.subscribe(q.onSelectedRowsChanged, fe).subscribe(q.onClick, me).subscribe(q.onKeyDown, he), ae.hideInFilterHeaderRow || we(Me), ae.hideInColumnTitleRow || Z.subscribe(q.onHeaderClick, ve);
      }
      function se() {
        Z.unsubscribeAll();
      }
      function ce() {
        return ae;
      }
      function le(Me) {
        if (ae = O.extend(!0, {}, ae, Me), ae.hideSelectAllCheckbox)
          ue(), de();
        else if (ae.hideInColumnTitleRow ? ue() : (Qe(ee), Z.subscribe(q.onHeaderClick, ve)), ae.hideInFilterHeaderRow)
          de();
        else {
          var Re = O("#filter-checkbox-selectall-container");
          Re.show(), Re.find('input[type="checkbox"]').prop("checked", ee);
        }
      }
      function ue() {
        q.updateColumnHeader(ae.columnId, "", "");
      }
      function de() {
        O("#filter-checkbox-selectall-container").hide();
      }
      function fe(Me, Re) {
        var De = q.getSelectedRows(), Pe = {}, $e, qe, Xe, Ve = 0;
        if (typeof G == "function")
          for (Xe = 0; Xe < q.getDataLength(); Xe++) {
            var Ze = q.getDataItem(Xe);
            Le(qe, Ze, q) || Ve++;
          }
        var Ge = [];
        for (qe = 0; qe < De.length; qe++) {
          $e = De[qe];
          var ze = q.getDataItem($e);
          Le(qe, ze, q) ? (Pe[$e] = !0, Pe[$e] !== te[$e] && (q.invalidateRow($e), delete te[$e])) : Ge.push($e);
        }
        for (qe in te)
          q.invalidateRow(qe);
        if (te = Pe, q.render(), ee = De.length && De.length + Ve >= q.getDataLength(), !ae.hideInColumnTitleRow && !ae.hideSelectAllCheckbox && Qe(ee), !ae.hideInFilterHeaderRow) {
          var We = O("#header-filter-selector" + J);
          We.prop("checked", ee);
        }
        if (Ge.length > 0) {
          for (qe = 0; qe < Ge.length; qe++) {
            var at = De.indexOf(Ge[qe]);
            De.splice(at, 1);
          }
          q.setSelectedRows(De);
        }
      }
      function he(Me, Re) {
        Me.which == 32 && q.getColumns()[Re.cell].id === ae.columnId && ((!q.getEditorLock().isActive() || q.getEditorLock().commitCurrentEdit()) && pe(Re.row), Me.preventDefault(), Me.stopImmediatePropagation());
      }
      function me(Me, Re) {
        if (q.getColumns()[Re.cell].id === ae.columnId && O(Me.target).is(":checkbox")) {
          if (q.getEditorLock().isActive() && !q.getEditorLock().commitCurrentEdit()) {
            Me.preventDefault(), Me.stopImmediatePropagation();
            return;
          }
          pe(Re.row), Me.stopPropagation(), Me.stopImmediatePropagation();
        }
      }
      function pe(Me) {
        var Re = q.getDataItem(Me);
        Le(Me, Re, q) && (te[Me] ? q.setSelectedRows(O.grep(q.getSelectedRows(), function(De) {
          return De != Me;
        })) : q.setSelectedRows(q.getSelectedRows().concat(Me)), q.setActiveCell(Me, Ce()));
      }
      function ye(Me) {
        var Re, De = Me.length, Pe = [];
        for (Re = 0; Re < De; Re++)
          te[Me[Re]] || (Pe[Pe.length] = Me[Re]);
        q.setSelectedRows(q.getSelectedRows().concat(Pe));
      }
      function ge(Me) {
        var Re, De = Me.length, Pe = [];
        for (Re = 0; Re < De; Re++)
          te[Me[Re]] && (Pe[Pe.length] = Me[Re]);
        q.setSelectedRows(O.grep(q.getSelectedRows(), function($e) {
          return Pe.indexOf($e) < 0;
        }));
      }
      function ve(Me, Re) {
        if (Re.column.id == ae.columnId && O(Me.target).is(":checkbox")) {
          if (q.getEditorLock().isActive() && !q.getEditorLock().commitCurrentEdit()) {
            Me.preventDefault(), Me.stopImmediatePropagation();
            return;
          }
          if (O(Me.target).is(":checked")) {
            for (var De = [], Pe = 0; Pe < q.getDataLength(); Pe++) {
              var $e = q.getDataItem(Pe);
              !$e.__group && !$e.__groupTotals && Le(Pe, $e, q) && De.push(Pe);
            }
            q.setSelectedRows(De);
          } else
            q.setSelectedRows([]);
          Me.stopPropagation(), Me.stopImmediatePropagation();
        }
      }
      var be = null;
      function Ce() {
        if (be === null) {
          be = 0;
          for (var Me = q.getColumns(), Re = 0; Re < Me.length; Re++)
            Me[Re].id == ae.columnId && (be = Re);
        }
        return be;
      }
      function Se() {
        return {
          id: ae.columnId,
          name: ae.hideSelectAllCheckbox || ae.hideInColumnTitleRow ? "" : "<input id='header-selector" + J + "' type='checkbox'><label for='header-selector" + J + "'></label>",
          toolTip: ae.hideSelectAllCheckbox || ae.hideInColumnTitleRow ? "" : ae.toolTip,
          field: "sel",
          width: ae.width,
          resizable: !1,
          sortable: !1,
          cssClass: ae.cssClass,
          hideSelectAllCheckbox: ae.hideSelectAllCheckbox,
          formatter: xe
        };
      }
      function we(Me) {
        Z.subscribe(Me.onHeaderRowCellRendered, function(Re, De) {
          De.column.field === "sel" && (O(De.node).empty(), O("<span id='filter-checkbox-selectall-container'><input id='header-filter-selector" + J + "' type='checkbox'><label for='header-filter-selector" + J + "'></label></span>").appendTo(De.node).on("click", function(Pe) {
            ve(Pe, De);
          }));
        });
      }
      function Ee() {
        return Math.round(1e7 * Math.random());
      }
      function xe(Me, Re, De, Pe, $e, qe) {
        var Xe = Ee() + Me;
        return $e && Le(Me, $e, qe) ? te[Me] ? "<input id='selector" + Xe + "' type='checkbox' checked='checked'><label for='selector" + Xe + "'></label>" : "<input id='selector" + Xe + "' type='checkbox'><label for='selector" + Xe + "'></label>" : null;
      }
      function Le(Me, Re, De) {
        return typeof G == "function" ? G(Me, Re, De) : !0;
      }
      function Qe(Me) {
        Me ? q.updateColumnHeader(ae.columnId, "<input id='header-selector" + J + "' type='checkbox' checked='checked'><label for='header-selector" + J + "'></label>", ae.toolTip) : q.updateColumnHeader(ae.columnId, "<input id='header-selector" + J + "' type='checkbox'><label for='header-selector" + J + "'></label>", ae.toolTip);
      }
      function ke(Me) {
        G = Me;
      }
      O.extend(this, {
        init: oe,
        destroy: se,
        pluginName: "CheckboxSelectColumn",
        deSelectRows: ge,
        selectRows: ye,
        getColumnDefinition: Se,
        getOptions: ce,
        selectableOverride: ke,
        setOptions: le
      });
    }
    return slick_checkboxselectcolumn = {
      CheckboxSelectColumn: H
    }, slick_checkboxselectcolumn;
  }
  var slick_checkboxselectcolumnExports = requireSlick_checkboxselectcolumn(), slick_cellexternalcopymanager, hasRequiredSlick_cellexternalcopymanager;
  function requireSlick_cellexternalcopymanager() {
    if (hasRequiredSlick_cellexternalcopymanager) return slick_cellexternalcopymanager;
    hasRequiredSlick_cellexternalcopymanager = 1;
    var O = requireSlick_jquery(), _ = requireSlick_core(), H = _.keyCode;
    function V(q) {
      var G, J = this, Z, te = q || {}, ne = te.copiedCellStyleLayerKey || "copy-manager", ee = te.copiedCellStyle || "copied", ae = 0, oe = te.bodyElement || document.body, se = te.onCopyInit || null, ce = te.onCopySuccess || null;
      function le(Ce) {
        G = Ce, G.onKeyDown.subscribe(ye);
        var Se = Ce.getSelectionModel();
        if (!Se)
          throw new Error("Selection model is mandatory for this plugin. Please set a selection model on the grid before adding this plugin: grid.setSelectionModel(new Slick.CellSelectionModel())");
        Se.onSelectedRangesChanged.subscribe(function(we, Ee) {
          G.focus();
        });
      }
      function ue() {
        G.onKeyDown.unsubscribe(ye);
      }
      function de(Ce) {
        if (te.headerColumnValueExtractor) {
          var Se = te.headerColumnValueExtractor(Ce);
          if (Se)
            return Se;
        }
        return Ce.name;
      }
      function fe(Ce, Se, we) {
        if (te.dataItemColumnValueExtractor) {
          var Ee = te.dataItemColumnValueExtractor(Ce, Se);
          if (Ee)
            return Ee;
        }
        var xe = "";
        if (Se.editor) {
          var Le = {
            container: O("<p>"),
            // a dummy container
            column: Se,
            position: { top: 0, left: 0 },
            // a dummy position required by some editors
            grid: G,
            event: we
          }, Qe = new Se.editor(Le);
          Qe.loadValue(Ce), xe = Qe.serializeValue(), Qe.destroy();
        } else
          xe = Ce[Se.field];
        return xe;
      }
      function he(Ce, Se, we) {
        if (Se.denyPaste)
          return null;
        if (te.dataItemColumnValueSetter)
          return te.dataItemColumnValueSetter(Ce, Se, we);
        if (Se.editor) {
          var Ee = {
            container: O("body"),
            // a dummy container
            column: Se,
            position: { top: 0, left: 0 },
            // a dummy position required by some editors
            grid: G
          }, xe = new Se.editor(Ee);
          xe.loadValue(Ce), xe.applyValue(Ce, we), xe.destroy();
        } else
          Ce[Se.field] = we;
      }
      function me(Ce) {
        var Se = document.createElement("textarea");
        return Se.style.position = "absolute", Se.style.left = "-1000px", Se.style.top = document.body.scrollTop + "px", Se.value = Ce, oe.appendChild(Se), Se.select(), Se;
      }
      function pe(Ce, Se) {
        var we = Ce.getColumns(), Ee = Se.value, xe = Ee.split(/[\n\f\r]/);
        xe[xe.length - 1] === "" && xe.pop();
        var Le = [], Qe = 0;
        oe.removeChild(Se);
        for (var ke = 0; ke < xe.length; ke++)
          xe[ke] !== "" ? Le[Qe++] = xe[ke].split("	") : Le[Qe++] = [""];
        var Me = Ce.getActiveCell(), Re = Ce.getSelectionModel().getSelectedRanges(), De = Re && Re.length ? Re[0] : null, Pe = null, $e = null;
        if (De)
          Pe = De.fromRow, $e = De.fromCell;
        else if (Me)
          Pe = Me.row, $e = Me.cell;
        else
          return;
        var qe = !1, Xe = Le.length, Ve = Le.length ? Le[0].length : 0;
        Le.length == 1 && Le[0].length == 1 && De && (qe = !0, Xe = De.toRow - De.fromRow + 1, Ve = De.toCell - De.fromCell + 1);
        var Ze = Ce.getData().length - Pe, Ge = 0;
        if (Ze < Xe && te.newRowCreator) {
          var ze = Ce.getData();
          for (Ge = 1; Ge <= Xe - Ze; Ge++)
            ze.push({});
          Ce.setData(ze), Ce.render();
        }
        var We = Pe + Xe > Ce.getDataLength();
        if (te.newRowCreator && We) {
          var at = Pe + Xe - Ce.getDataLength();
          te.newRowCreator(at);
        }
        var ct = {
          isClipboardCommand: !0,
          clippedRange: Le,
          oldValues: [],
          cellExternalCopyManager: J,
          _options: te,
          setDataItemValueForColumn: he,
          markCopySelection: ge,
          oneCellToMultiple: qe,
          activeRow: Pe,
          activeCell: $e,
          destH: Xe,
          destW: Ve,
          maxDestY: Ce.getDataLength(),
          maxDestX: Ce.getColumns().length,
          h: 0,
          w: 0,
          execute: function() {
            this.h = 0;
            for (var et = 0; et < this.destH; et++) {
              this.oldValues[et] = [], this.w = 0, this.h++;
              for (var dt = 0; dt < this.destW; dt++) {
                this.w++;
                var Je = Pe + et, it = $e + dt;
                if (Je < this.maxDestY && it < this.maxDestX) {
                  Ce.getCellNode(Je, it);
                  var st = Ce.getDataItem(Je);
                  this.oldValues[et][dt] = st[we[it].field], qe ? this.setDataItemValueForColumn(st, we[it], Le[0][0]) : this.setDataItemValueForColumn(st, we[it], Le[et] ? Le[et][dt] : ""), Ce.updateCell(Je, it), Ce.onCellChange.notify({
                    row: Je,
                    cell: it,
                    item: st,
                    grid: Ce
                  });
                }
              }
            }
            var gt = {
              fromCell: $e,
              fromRow: Pe,
              toCell: $e + this.w - 1,
              toRow: Pe + this.h - 1
            };
            this.markCopySelection([gt]), Ce.getSelectionModel().setSelectedRanges([gt]), this.cellExternalCopyManager.onPasteCells.notify({ ranges: [gt] });
          },
          undo: function() {
            for (var et = 0; et < this.destH; et++)
              for (var dt = 0; dt < this.destW; dt++) {
                var Je = Pe + et, it = $e + dt;
                if (Je < this.maxDestY && it < this.maxDestX) {
                  Ce.getCellNode(Je, it);
                  var st = Ce.getDataItem(Je);
                  qe ? this.setDataItemValueForColumn(st, we[it], this.oldValues[0][0]) : this.setDataItemValueForColumn(st, we[it], this.oldValues[et][dt]), Ce.updateCell(Je, it), Ce.onCellChange.notify({
                    row: Je,
                    cell: it,
                    item: st,
                    grid: Ce
                  });
                }
              }
            var gt = {
              fromCell: $e,
              fromRow: Pe,
              toCell: $e + this.w - 1,
              toRow: Pe + this.h - 1
            };
            if (this.markCopySelection([gt]), Ce.getSelectionModel().setSelectedRanges([gt]), this.cellExternalCopyManager.onPasteCells.notify({ ranges: [gt] }), Ge > 1) {
              for (var bt = Ce.getData(); Ge > 1; Ge--)
                bt.splice(bt.length - 1, 1);
              Ce.setData(bt), Ce.render();
            }
          }
        };
        te.clipboardCommandHandler ? te.clipboardCommandHandler(ct) : ct.execute();
      }
      function ye(Ce, Se) {
        var we;
        if (!G.getEditorLock().isActive() || G.getOptions().autoEdit) {
          if (Ce.which == H.ESC && Z && (Ce.preventDefault(), ve(), J.onCopyCancelled.notify({ ranges: Z }), Z = null), (Ce.which === H.C || Ce.which === H.INSERT) && (Ce.ctrlKey || Ce.metaKey) && !Ce.shiftKey && (se && se.call(), we = G.getSelectionModel().getSelectedRanges(), we.length !== 0)) {
            Z = we, ge(we), J.onCopyCells.notify({ ranges: we });
            for (var Ee = G.getColumns(), xe = "", Le = 0; Le < we.length; Le++) {
              for (var Qe = we[Le], ke = [], Me = Qe.fromRow; Me < Qe.toRow + 1; Me++) {
                var Re = [], De = G.getDataItem(Me);
                if (ke.length === 0 && te.includeHeaderWhenCopying) {
                  for (var Pe = [], $e = Qe.fromCell; $e < Qe.toCell + 1; $e++)
                    Ee[$e].name.length > 0 && Pe.push(de(Ee[$e]));
                  ke.push(Pe.join("	"));
                }
                for (var $e = Qe.fromCell; $e < Qe.toCell + 1; $e++)
                  Re.push(fe(De, Ee[$e], Ce));
                ke.push(Re.join("	"));
              }
              xe += ke.join(`\r
`) + `\r
`;
            }
            if (window.clipboardData)
              return window.clipboardData.setData("Text", xe), !0;
            var qe = document.activeElement, Xe = me(xe);
            if (Xe.focus(), setTimeout(function() {
              oe.removeChild(Xe), qe ? qe.focus() : console.log("Not element to restore focus to after copy?");
            }, 100), ce) {
              var Ve = 0;
              we.length === 1 ? Ve = we[0].toRow + 1 - we[0].fromRow : Ve = we.length, ce.call(this, Ve);
            }
            return !1;
          }
          if (!te.readOnlyMode && (Ce.which === H.V && (Ce.ctrlKey || Ce.metaKey) && !Ce.shiftKey || Ce.which === H.INSERT && Ce.shiftKey && !Ce.ctrlKey)) {
            var Xe = me("");
            return setTimeout(function() {
              pe(G, Xe);
            }, 100), !1;
          }
        }
      }
      function ge(Ce) {
        ve();
        for (var Se = G.getColumns(), we = {}, Ee = 0; Ee < Ce.length; Ee++)
          for (var xe = Ce[Ee].fromRow; xe <= Ce[Ee].toRow; xe++) {
            we[xe] = {};
            for (var Le = Ce[Ee].fromCell; Le <= Ce[Ee].toCell && Le < Se.length; Le++)
              we[xe][Se[Le].id] = ee;
          }
        G.setCellCssStyles(ne, we), clearTimeout(ae), ae = setTimeout(function() {
          J.clearCopySelection();
        }, 2e3);
      }
      function ve() {
        G.removeCellCssStyles(ne);
      }
      function be(Ce) {
        te.includeHeaderWhenCopying = Ce;
      }
      O.extend(this, {
        init: le,
        destroy: ue,
        pluginName: "CellExternalCopyManager",
        clearCopySelection: ve,
        handleKeyDown: ye,
        onCopyCells: new _.Event(),
        onCopyCancelled: new _.Event(),
        onPasteCells: new _.Event(),
        setIncludeHeaderWhenCopying: be
      });
    }
    return slick_cellexternalcopymanager = {
      CellExternalCopyManager: V
    }, slick_cellexternalcopymanager;
  }
  var slick_cellexternalcopymanagerExports = requireSlick_cellexternalcopymanager(), slickgrid = { exports: {} };
  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation.
  
  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.
  
  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */
  var extendStatics = function(O, _) {
    return extendStatics = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(H, V) {
      H.__proto__ = V;
    } || function(H, V) {
      for (var q in V) V.hasOwnProperty(q) && (H[q] = V[q]);
    }, extendStatics(O, _);
  };
  function __extends$1(O, _) {
    extendStatics(O, _);
    function H() {
      this.constructor = O;
    }
    O.prototype = _ === null ? Object.create(_) : (H.prototype = _.prototype, new H());
  }
  var __assign = function() {
    return __assign = Object.assign || function(_) {
      for (var H, V = 1, q = arguments.length; V < q; V++) {
        H = arguments[V];
        for (var G in H) Object.prototype.hasOwnProperty.call(H, G) && (_[G] = H[G]);
      }
      return _;
    }, __assign.apply(this, arguments);
  };
  function __rest(O, _) {
    var H = {};
    for (var V in O) Object.prototype.hasOwnProperty.call(O, V) && _.indexOf(V) < 0 && (H[V] = O[V]);
    if (O != null && typeof Object.getOwnPropertySymbols == "function")
      for (var q = 0, V = Object.getOwnPropertySymbols(O); q < V.length; q++)
        _.indexOf(V[q]) < 0 && Object.prototype.propertyIsEnumerable.call(O, V[q]) && (H[V[q]] = O[V[q]]);
    return H;
  }
  function __decorate(O, _, H, V) {
    var q = arguments.length, G = q < 3 ? _ : V === null ? V = Object.getOwnPropertyDescriptor(_, H) : V, J;
    if (typeof Reflect == "object" && typeof Reflect.decorate == "function") G = Reflect.decorate(O, _, H, V);
    else for (var Z = O.length - 1; Z >= 0; Z--) (J = O[Z]) && (G = (q < 3 ? J(G) : q > 3 ? J(_, H, G) : J(_, H)) || G);
    return q > 3 && G && Object.defineProperty(_, H, G), G;
  }
  function __param(O, _) {
    return function(H, V) {
      _(H, V, O);
    };
  }
  function __metadata(O, _) {
    if (typeof Reflect == "object" && typeof Reflect.metadata == "function") return Reflect.metadata(O, _);
  }
  function __awaiter$1(O, _, H, V) {
    function q(G) {
      return G instanceof H ? G : new H(function(J) {
        J(G);
      });
    }
    return new (H || (H = Promise))(function(G, J) {
      function Z(ee) {
        try {
          ne(V.next(ee));
        } catch (ae) {
          J(ae);
        }
      }
      function te(ee) {
        try {
          ne(V.throw(ee));
        } catch (ae) {
          J(ae);
        }
      }
      function ne(ee) {
        ee.done ? G(ee.value) : q(ee.value).then(Z, te);
      }
      ne((V = V.apply(O, _ || [])).next());
    });
  }
  function __generator(O, _) {
    var H = { label: 0, sent: function() {
      if (G[0] & 1) throw G[1];
      return G[1];
    }, trys: [], ops: [] }, V, q, G, J;
    return J = { next: Z(0), throw: Z(1), return: Z(2) }, typeof Symbol == "function" && (J[Symbol.iterator] = function() {
      return this;
    }), J;
    function Z(ne) {
      return function(ee) {
        return te([ne, ee]);
      };
    }
    function te(ne) {
      if (V) throw new TypeError("Generator is already executing.");
      for (; H; ) try {
        if (V = 1, q && (G = ne[0] & 2 ? q.return : ne[0] ? q.throw || ((G = q.return) && G.call(q), 0) : q.next) && !(G = G.call(q, ne[1])).done) return G;
        switch (q = 0, G && (ne = [ne[0] & 2, G.value]), ne[0]) {
          case 0:
          case 1:
            G = ne;
            break;
          case 4:
            return H.label++, { value: ne[1], done: !1 };
          case 5:
            H.label++, q = ne[1], ne = [0];
            continue;
          case 7:
            ne = H.ops.pop(), H.trys.pop();
            continue;
          default:
            if (G = H.trys, !(G = G.length > 0 && G[G.length - 1]) && (ne[0] === 6 || ne[0] === 2)) {
              H = 0;
              continue;
            }
            if (ne[0] === 3 && (!G || ne[1] > G[0] && ne[1] < G[3])) {
              H.label = ne[1];
              break;
            }
            if (ne[0] === 6 && H.label < G[1]) {
              H.label = G[1], G = ne;
              break;
            }
            if (G && H.label < G[2]) {
              H.label = G[2], H.ops.push(ne);
              break;
            }
            G[2] && H.ops.pop(), H.trys.pop();
            continue;
        }
        ne = _.call(O, H);
      } catch (ee) {
        ne = [6, ee], q = 0;
      } finally {
        V = G = 0;
      }
      if (ne[0] & 5) throw ne[1];
      return { value: ne[0] ? ne[1] : void 0, done: !0 };
    }
  }
  function __createBinding(O, _, H, V) {
    V === void 0 && (V = H), O[V] = _[H];
  }
  function __exportStar(O, _) {
    for (var H in O) H !== "default" && !_.hasOwnProperty(H) && (_[H] = O[H]);
  }
  function __values$1(O) {
    var _ = typeof Symbol == "function" && Symbol.iterator, H = _ && O[_], V = 0;
    if (H) return H.call(O);
    if (O && typeof O.length == "number") return {
      next: function() {
        return O && V >= O.length && (O = void 0), { value: O && O[V++], done: !O };
      }
    };
    throw new TypeError(_ ? "Object is not iterable." : "Symbol.iterator is not defined.");
  }
  function __read(O, _) {
    var H = typeof Symbol == "function" && O[Symbol.iterator];
    if (!H) return O;
    var V = H.call(O), q, G = [], J;
    try {
      for (; (_ === void 0 || _-- > 0) && !(q = V.next()).done; ) G.push(q.value);
    } catch (Z) {
      J = { error: Z };
    } finally {
      try {
        q && !q.done && (H = V.return) && H.call(V);
      } finally {
        if (J) throw J.error;
      }
    }
    return G;
  }
  function __spread() {
    for (var O = [], _ = 0; _ < arguments.length; _++)
      O = O.concat(__read(arguments[_]));
    return O;
  }
  function __spreadArrays() {
    for (var O = 0, _ = 0, H = arguments.length; _ < H; _++) O += arguments[_].length;
    for (var V = Array(O), q = 0, _ = 0; _ < H; _++)
      for (var G = arguments[_], J = 0, Z = G.length; J < Z; J++, q++)
        V[q] = G[J];
    return V;
  }
  function __await$1(O) {
    return this instanceof __await$1 ? (this.v = O, this) : new __await$1(O);
  }
  function __asyncGenerator$1(O, _, H) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var V = H.apply(O, _ || []), q, G = [];
    return q = {}, J("next"), J("throw"), J("return"), q[Symbol.asyncIterator] = function() {
      return this;
    }, q;
    function J(oe) {
      V[oe] && (q[oe] = function(se) {
        return new Promise(function(ce, le) {
          G.push([oe, se, ce, le]) > 1 || Z(oe, se);
        });
      });
    }
    function Z(oe, se) {
      try {
        te(V[oe](se));
      } catch (ce) {
        ae(G[0][3], ce);
      }
    }
    function te(oe) {
      oe.value instanceof __await$1 ? Promise.resolve(oe.value.v).then(ne, ee) : ae(G[0][2], oe);
    }
    function ne(oe) {
      Z("next", oe);
    }
    function ee(oe) {
      Z("throw", oe);
    }
    function ae(oe, se) {
      oe(se), G.shift(), G.length && Z(G[0][0], G[0][1]);
    }
  }
  function __asyncDelegator$1(O) {
    var _, H;
    return _ = {}, V("next"), V("throw", function(q) {
      throw q;
    }), V("return"), _[Symbol.iterator] = function() {
      return this;
    }, _;
    function V(q, G) {
      _[q] = O[q] ? function(J) {
        return (H = !H) ? { value: __await$1(O[q](J)), done: q === "return" } : G ? G(J) : J;
      } : G;
    }
  }
  function __asyncValues$1(O) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var _ = O[Symbol.asyncIterator], H;
    return _ ? _.call(O) : (O = typeof __values$1 == "function" ? __values$1(O) : O[Symbol.iterator](), H = {}, V("next"), V("throw"), V("return"), H[Symbol.asyncIterator] = function() {
      return this;
    }, H);
    function V(G) {
      H[G] = O[G] && function(J) {
        return new Promise(function(Z, te) {
          J = O[G](J), q(Z, te, J.done, J.value);
        });
      };
    }
    function q(G, J, Z, te) {
      Promise.resolve(te).then(function(ne) {
        G({ value: ne, done: Z });
      }, J);
    }
  }
  function __makeTemplateObject(O, _) {
    return Object.defineProperty ? Object.defineProperty(O, "raw", { value: _ }) : O.raw = _, O;
  }
  function __importStar(O) {
    if (O && O.__esModule) return O;
    var _ = {};
    if (O != null) for (var H in O) Object.hasOwnProperty.call(O, H) && (_[H] = O[H]);
    return _.default = O, _;
  }
  function __importDefault(O) {
    return O && O.__esModule ? O : { default: O };
  }
  function __classPrivateFieldGet$1(O, _) {
    if (!_.has(O))
      throw new TypeError("attempted to get private field on non-instance");
    return _.get(O);
  }
  function __classPrivateFieldSet$1(O, _, H) {
    if (!_.has(O))
      throw new TypeError("attempted to set private field on non-instance");
    return _.set(O, H), H;
  }
  const tslib_es6 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    get __assign() {
      return __assign;
    },
    __asyncDelegator: __asyncDelegator$1,
    __asyncGenerator: __asyncGenerator$1,
    __asyncValues: __asyncValues$1,
    __await: __await$1,
    __awaiter: __awaiter$1,
    __classPrivateFieldGet: __classPrivateFieldGet$1,
    __classPrivateFieldSet: __classPrivateFieldSet$1,
    __createBinding,
    __decorate,
    __exportStar,
    __extends: __extends$1,
    __generator,
    __importDefault,
    __importStar,
    __makeTemplateObject,
    __metadata,
    __param,
    __read,
    __rest,
    __spread,
    __spreadArrays,
    __values: __values$1
  }, Symbol.toStringTag, { value: "Module" })), require$$0 = /* @__PURE__ */ getAugmentedNamespace(tslib_es6);
  var jquery_event_drag2_3_0 = {};
  /*!
   * jquery.event.drag - v 2.3.0
   * Copyright (c) 2010 Three Dub Media - http://threedubmedia.com
   * Open Source MIT License - http://threedubmedia.com/code/license
   */
  var hasRequiredJquery_event_drag2_3_0;
  function requireJquery_event_drag2_3_0() {
    if (hasRequiredJquery_event_drag2_3_0) return jquery_event_drag2_3_0;
    hasRequiredJquery_event_drag2_3_0 = 1;
    var O = requireSlick_jquery();
    O.fn.drag = function(G, J, Z) {
      var te = typeof G == "string" ? G : "", ne = O.isFunction(G) ? G : O.isFunction(J) ? J : null;
      return te.indexOf("drag") !== 0 && (te = "drag" + te), Z = (G == ne ? J : Z) || {}, ne ? this.on(te, Z, ne) : this.trigger(te);
    };
    var _ = O.event, H = _.special, V = H.drag = {
      // these are the default settings
      defaults: {
        which: 1,
        // mouse button pressed to start drag sequence
        distance: 0,
        // distance dragged before dragstart
        not: ":input",
        // selector to suppress dragging on target elements
        handle: null,
        // selector to match handle target elements
        relative: !1,
        // true to use "position", false to use "offset"
        drop: !0,
        // false to suppress drop events, true or selector to allow
        click: !1
        // false to suppress click events after dragend (no proxy)
      },
      // the key name for stored drag data
      datakey: "dragdata",
      // prevent bubbling for better performance
      noBubble: !0,
      // count bound related events
      add: function(G) {
        var J = O.data(this, V.datakey), Z = G.data || {};
        J.related += 1, O.each(V.defaults, function(te, ne) {
          Z[te] !== void 0 && (J[te] = Z[te]);
        });
      },
      // forget unbound related events
      remove: function() {
        O.data(this, V.datakey).related -= 1;
      },
      // configure interaction, capture settings
      setup: function() {
        if (!O.data(this, V.datakey)) {
          var G = O.extend({ related: 0 }, V.defaults);
          O.data(this, V.datakey, G), _.add(this, "touchstart mousedown", V.init, G), this.attachEvent && this.attachEvent("ondragstart", V.dontstart);
        }
      },
      // destroy configured interaction
      teardown: function() {
        var G = O.data(this, V.datakey) || {};
        G.related || (O.removeData(this, V.datakey), _.remove(this, "touchstart mousedown", V.init), V.textselect(!0), this.detachEvent && this.detachEvent("ondragstart", V.dontstart));
      },
      // initialize the interaction
      init: function(G) {
        if (!V.touched) {
          var J = G.data, Z;
          if (!(G.which != 0 && J.which > 0 && G.which != J.which)) {
            var te = O(G.target).attr("class") || "";
            if (!(O(G.target).is(J.not) || !te || te.toString().indexOf("slick") === -1) && !(J.handle && !O(G.target).closest(J.handle, G.currentTarget).length) && (V.touched = G.type == "touchstart" ? this : null, J.propagates = 1, J.mousedown = this, J.interactions = [V.interaction(this, J)], J.target = G.target, J.pageX = G.pageX, J.pageY = G.pageY, J.dragging = null, Z = V.hijack(G, "draginit", J), !!J.propagates && (Z = V.flatten(Z), Z && Z.length && (J.interactions = [], O.each(Z, function() {
              J.interactions.push(V.interaction(this, J));
            })), J.propagates = J.interactions.length, J.drop !== !1 && H.drop && H.drop.handler(G, J), V.textselect(!1), V.touched ? _.add(V.touched, "touchmove touchend", V.handler, J) : _.add(document, "mousemove mouseup", V.handler, J), !V.touched || J.live)))
              return !1;
          }
        }
      },
      // returns an interaction object
      interaction: function(G, J) {
        var Z = G && G.ownerDocument ? O(G)[J.relative ? "position" : "offset"]() || { top: 0, left: 0 } : { top: 0, left: 0 };
        return {
          drag: G,
          callback: new V.callback(),
          droppable: [],
          offset: Z
        };
      },
      // handle drag-releatd DOM events
      handler: function(G) {
        var J = G.data;
        switch (G.type) {
          // mousemove, check distance, start dragging
          case (!J.dragging && "touchmove"):
            G.preventDefault();
          case (!J.dragging && "mousemove"):
            if (Math.pow(G.pageX - J.pageX, 2) + Math.pow(G.pageY - J.pageY, 2) < Math.pow(J.distance, 2))
              break;
            G.target = J.target, V.hijack(G, "dragstart", J), J.propagates && (J.dragging = !0);
          // activate interaction
          // mousemove, dragging
          case "touchmove":
            G.preventDefault();
          case "mousemove":
            if (J.dragging) {
              if (V.hijack(G, "drag", J), J.propagates) {
                J.drop !== !1 && H.drop && H.drop.handler(G, J);
                break;
              }
              G.type = "mouseup";
            }
          // mouseup, stop dragging
          case "touchend":
          case "mouseup":
          default:
            V.touched ? _.remove(V.touched, "touchmove touchend", V.handler) : _.remove(document, "mousemove mouseup", V.handler), J.dragging && (J.drop !== !1 && H.drop && H.drop.handler(G, J), V.hijack(G, "dragend", J)), V.textselect(!0), J.click === !1 && J.dragging && O.data(J.mousedown, "suppress.click", (/* @__PURE__ */ new Date()).getTime() + 5), J.dragging = V.touched = !1;
            break;
        }
      },
      // re-use event object for custom events
      hijack: function(G, J, Z, te, ne) {
        if (Z) {
          var ee = { event: G.originalEvent, type: G.type }, ae = J.indexOf("drop") ? "drag" : "drop", oe, se = te || 0, ce, le, ue = isNaN(te) ? Z.interactions.length : te;
          G.type = J;
          var de = function() {
          };
          G.originalEvent = new O.Event(ee.event, {
            preventDefault: de,
            stopPropagation: de,
            stopImmediatePropagation: de
          }), Z.results = [];
          do
            if (ce = Z.interactions[se]) {
              if (J !== "dragend" && ce.cancelled)
                continue;
              le = V.properties(G, Z, ce), ce.results = [], O(ne || ce[ae] || Z.droppable).each(function(fe, he) {
                if (le.target = he, G.isPropagationStopped = function() {
                  return !1;
                }, oe = he ? _.dispatch.call(he, G, le) : null, oe === !1 ? (ae == "drag" && (ce.cancelled = !0, Z.propagates -= 1), J == "drop" && (ce[ae][fe] = null)) : J == "dropinit" && ce.droppable.push(V.element(oe) || he), J == "dragstart" && (ce.proxy = O(V.element(oe) || ce.drag)[0]), ce.results.push(oe), delete G.result, J !== "dropinit")
                  return oe;
              }), Z.results[se] = V.flatten(ce.results), J == "dropinit" && (ce.droppable = V.flatten(ce.droppable)), J == "dragstart" && !ce.cancelled && le.update();
            }
          while (++se < ue);
          return G.type = ee.type, G.originalEvent = ee.event, V.flatten(Z.results);
        }
      },
      // extend the callback object with drag/drop properties...
      properties: function(G, J, Z) {
        var te = Z.callback;
        return te.drag = Z.drag, te.proxy = Z.proxy || Z.drag, te.startX = J.pageX, te.startY = J.pageY, te.deltaX = G.pageX - J.pageX, te.deltaY = G.pageY - J.pageY, te.originalX = Z.offset.left, te.originalY = Z.offset.top, te.offsetX = te.originalX + te.deltaX, te.offsetY = te.originalY + te.deltaY, te.drop = V.flatten((Z.drop || []).slice()), te.available = V.flatten((Z.droppable || []).slice()), te;
      },
      // determine is the argument is an element or jquery instance
      element: function(G) {
        if (G && (G.jquery || G.nodeType == 1))
          return G;
      },
      // flatten nested jquery objects and arrays into a single dimension array
      flatten: function(G) {
        return O.map(G, function(J) {
          return J && J.jquery ? O.makeArray(J) : J && J.length ? V.flatten(J) : J;
        });
      },
      // toggles text selection attributes ON (true) or OFF (false)
      textselect: function(G) {
        O(document)[G ? "off" : "on"]("selectstart", V.dontstart).css("MozUserSelect", G ? "" : "none"), document.unselectable = G ? "off" : "on";
      },
      // suppress "selectstart" and "ondragstart" events
      dontstart: function() {
        return !1;
      },
      // a callback instance contructor
      callback: function() {
      }
    };
    V.callback.prototype = {
      update: function() {
        H.drop && this.available.length && O.each(this.available, function(G) {
          H.drop.locate(this, G);
        });
      }
    };
    var q = _.dispatch;
    return _.dispatch = function(G) {
      if (O.data(this, "suppress." + G.type) - (/* @__PURE__ */ new Date()).getTime() > 0) {
        O.removeData(this, "suppress." + G.type);
        return;
      }
      return q.apply(this, arguments);
    }, H.draginit = H.dragstart = H.dragend = V, jquery_event_drag2_3_0;
  }
  var jquery_event_drop2_3_0 = {};
  /*!
   * jquery.event.drop - v 2.3.0
   * Copyright (c) 2010 Three Dub Media - http://threedubmedia.com
   * Open Source MIT License - http://threedubmedia.com/code/license
   */
  var hasRequiredJquery_event_drop2_3_0;
  function requireJquery_event_drop2_3_0() {
    if (hasRequiredJquery_event_drop2_3_0) return jquery_event_drop2_3_0;
    hasRequiredJquery_event_drop2_3_0 = 1;
    var O = requireSlick_jquery();
    O.fn.drop = function(q, G, J) {
      var Z = typeof q == "string" ? q : "", te = O.isFunction(q) ? q : O.isFunction(G) ? G : null;
      return Z.indexOf("drop") !== 0 && (Z = "drop" + Z), J = (q == te ? G : J) || {}, te ? this.on(Z, J, te) : this.trigger(Z);
    }, O.drop = function(q) {
      q = q || {}, V.multi = q.multi === !0 ? 1 / 0 : q.multi === !1 ? 1 : isNaN(q.multi) ? V.multi : q.multi, V.delay = q.delay || V.delay, V.tolerance = O.isFunction(q.tolerance) ? q.tolerance : q.tolerance === null ? null : V.tolerance, V.mode = q.mode || V.mode || "intersect";
    };
    var _ = O.event, H = _.special, V = O.event.special.drop = {
      // these are the default settings
      multi: 1,
      // allow multiple drop winners per dragged element
      delay: 20,
      // async timeout delay
      mode: "overlap",
      // drop tolerance mode
      // internal cache
      targets: [],
      // the key name for stored drop data
      datakey: "dropdata",
      // prevent bubbling for better performance
      noBubble: !0,
      // count bound related events
      add: function(q) {
        var G = O.data(this, V.datakey);
        G.related += 1;
      },
      // forget unbound related events
      remove: function() {
        O.data(this, V.datakey).related -= 1;
      },
      // configure the interactions
      setup: function() {
        if (!O.data(this, V.datakey)) {
          var q = {
            related: 0,
            active: [],
            anyactive: 0,
            winner: 0,
            location: {}
          };
          O.data(this, V.datakey, q), V.targets.push(this);
        }
      },
      // destroy the configure interaction
      teardown: function() {
        var q = O.data(this, V.datakey) || {};
        if (!q.related) {
          O.removeData(this, V.datakey);
          var G = this;
          V.targets = O.grep(V.targets, function(J) {
            return J !== G;
          });
        }
      },
      // shared event handler
      handler: function(q, G) {
        var J;
        if (G)
          switch (q.type) {
            // draginit, from $.event.special.drag
            case "mousedown":
            // DROPINIT >>
            case "touchstart":
              J = O(V.targets), typeof G.drop == "string" && (J = J.filter(G.drop)), J.each(function() {
                var Z = O.data(this, V.datakey);
                Z.active = [], Z.anyactive = 0, Z.winner = 0;
              }), G.droppable = J, H.drag.hijack(q, "dropinit", G);
              break;
            // drag, from $.event.special.drag
            case "mousemove":
            // TOLERATE >>
            case "touchmove":
              V.event = q, V.timer || V.tolerate(G);
              break;
            // dragend, from $.event.special.drag
            case "mouseup":
            // DROP >> DROPEND >>
            case "touchend":
              V.timer = clearTimeout(V.timer), G.propagates && (H.drag.hijack(q, "drop", G), H.drag.hijack(q, "dropend", G));
              break;
          }
      },
      // returns the location positions of an element
      locate: function(q, G) {
        var J = O.data(q, V.datakey), Z = O(q), te = Z.length && !Z.is(document) ? Z.offset() : {}, ne = Z.outerHeight(), ee = Z.outerWidth(), ae = {
          elem: q,
          width: ee,
          height: ne,
          top: te.top,
          left: te.left,
          right: te.left + ee,
          bottom: te.top + ne
        };
        return J && (J.location = ae, J.index = G, J.elem = q), ae;
      },
      // test the location positions of an element against another OR an X,Y coord
      contains: function(q, G) {
        return (G[0] || G.left) >= q.left && (G[0] || G.right) <= q.right && (G[1] || G.top) >= q.top && (G[1] || G.bottom) <= q.bottom;
      },
      // stored tolerance modes
      modes: {
        // fn scope: "$.event.special.drop" object
        // target with mouse wins, else target with most overlap wins
        intersect: function(q, G, J) {
          return this.contains(J, [q.pageX, q.pageY]) ? (
            // check cursor
            1e9
          ) : this.modes.overlap.apply(this, arguments);
        },
        // target with most overlap wins
        overlap: function(q, G, J) {
          return Math.max(0, Math.min(J.bottom, G.bottom) - Math.max(J.top, G.top)) * Math.max(0, Math.min(J.right, G.right) - Math.max(J.left, G.left));
        },
        // proxy is completely contained within target bounds
        fit: function(q, G, J) {
          return this.contains(J, G) ? 1 : 0;
        },
        // center of the proxy is contained within target bounds
        middle: function(q, G, J) {
          return this.contains(J, [G.left + G.width * 0.5, G.top + G.height * 0.5]) ? 1 : 0;
        }
      },
      // sort drop target cache by by winner (dsc), then index (asc)
      sort: function(q, G) {
        return G.winner - q.winner || q.index - G.index;
      },
      // async, recursive tolerance execution
      tolerate: function(q) {
        var G, J, Z, te, ne, ee, ae, oe = 0, se, ce = q.interactions.length, le = [V.event.pageX, V.event.pageY], ue = V.tolerance || V.modes[V.mode];
        do
          if (se = q.interactions[oe]) {
            if (!se)
              return;
            se.drop = [], ne = [], ee = se.droppable.length, ue && (Z = V.locate(se.proxy)), G = 0;
            do
              if (ae = se.droppable[G]) {
                if (te = O.data(ae, V.datakey), J = te.location, !J) continue;
                te.winner = ue ? ue.call(V, V.event, Z, J) : V.contains(J, le) ? 1 : 0, ne.push(te);
              }
            while (++G < ee);
            ne.sort(V.sort), G = 0;
            do
              (te = ne[G]) && (te.winner && se.drop.length < V.multi ? (!te.active[oe] && !te.anyactive && (H.drag.hijack(V.event, "dropstart", q, oe, te.elem)[0] !== !1 ? (te.active[oe] = 1, te.anyactive += 1) : te.winner = 0), te.winner && se.drop.push(te.elem)) : te.active[oe] && te.anyactive == 1 && (H.drag.hijack(V.event, "dropend", q, oe, te.elem), te.active[oe] = 0, te.anyactive -= 1));
            while (++G < ee);
          }
        while (++oe < ce);
        V.last && le[0] == V.last.pageX && le[1] == V.last.pageY ? delete V.timer : V.timer = setTimeout(function() {
          V.tolerate(q);
        }, V.delay), V.last = V.event;
      }
    };
    return H.dropinit = H.dropstart = H.dropend = V, jquery_event_drop2_3_0;
  }
  /**
   * @license
   * (c) 2009-2016 Michael Leibman
   * michael{dot}leibman{at}gmail{dot}com
   * http://github.com/mleibman/slickgrid
   *
   * Distributed under MIT license.
   * All rights reserved.
   *
   * SlickGrid v2.4
   *
   * NOTES:
   *     Cell/row DOM manipulations are done directly bypassing jQuery's DOM manipulation methods.
   *     This increases the speed dramatically, but can only be done safely because there are no event handlers
   *     or data associated with any cell/row DOM nodes.  Cell editors must make sure they implement .destroy()
   *     and do proper cleanup.
   */
  var slick_grid, hasRequiredSlick_grid;
  function requireSlick_grid() {
    if (hasRequiredSlick_grid) return slick_grid;
    hasRequiredSlick_grid = 1;
    const jQuery = requireSlick_jquery(), $ = jQuery;
    var Slick = requireSlick_core(), scrollbarDimensions, maxSupportedCssHeight;
    function SlickGrid(container, data, columns, options) {
      $.fn.drag || requireJquery_event_drag2_3_0(), $.fn.drop || requireJquery_event_drop2_3_0();
      var defaults = {
        alwaysShowVerticalScroll: !1,
        alwaysAllowHorizontalScroll: !1,
        explicitInitialization: !1,
        rowHeight: 25,
        defaultColumnWidth: 80,
        enableAddRow: !1,
        leaveSpaceForNewRows: !1,
        editable: !1,
        autoEdit: !0,
        suppressActiveCellChangeOnEdit: !1,
        enableCellNavigation: !0,
        enableColumnReorder: !0,
        asyncEditorLoading: !1,
        asyncEditorLoadDelay: 100,
        forceFitColumns: !1,
        enableAsyncPostRender: !1,
        asyncPostRenderDelay: 50,
        enableAsyncPostRenderCleanup: !1,
        asyncPostRenderCleanupDelay: 40,
        autoHeight: !1,
        editorLock: Slick.GlobalEditorLock,
        showColumnHeader: !0,
        showHeaderRow: !1,
        headerRowHeight: 25,
        createFooterRow: !1,
        showFooterRow: !1,
        footerRowHeight: 25,
        createPreHeaderPanel: !1,
        showPreHeaderPanel: !1,
        preHeaderPanelHeight: 25,
        showTopPanel: !1,
        topPanelHeight: 25,
        formatterFactory: null,
        editorFactory: null,
        cellFlashingCssClass: "flashing",
        selectedCellCssClass: "selected",
        multiSelect: !0,
        enableTextSelectionOnCells: !1,
        dataItemColumnValueExtractor: null,
        frozenBottom: !1,
        frozenColumn: -1,
        frozenRow: -1,
        frozenRightViewportMinWidth: 100,
        fullWidthRows: !1,
        multiColumnSort: !1,
        numberedMultiColumnSort: !1,
        tristateMultiColumnSort: !1,
        sortColNumberInSeparateSpan: !1,
        defaultFormatter,
        forceSyncScrolling: !1,
        addNewRowCssClass: "new-row",
        preserveCopiedSelectionOnPaste: !1,
        showCellSelection: !0,
        viewportClass: null,
        minRowBuffer: 3,
        emulatePagingWhenScrolling: !0,
        // when scrolling off bottom of viewport, place new row at top of viewport
        editorCellNavOnLRKeys: !1,
        enableMouseWheelScrollHandler: !0,
        doPaging: !0,
        autosizeColsMode: Slick.GridAutosizeColsMode.LegacyOff,
        autosizeColPaddingPx: 4,
        autosizeTextAvgToMWidthRatio: 0.75,
        viewportSwitchToScrollModeWidthPercent: void 0,
        viewportMinWidthPx: void 0,
        viewportMaxWidthPx: void 0,
        suppressCssChangesOnHiddenInit: !1
      }, columnDefaults = {
        name: "",
        resizable: !0,
        sortable: !1,
        minWidth: 30,
        maxWidth: void 0,
        rerenderOnResize: !1,
        headerCssClass: null,
        defaultSortAsc: !0,
        focusable: !0,
        selectable: !0
      }, columnAutosizeDefaults = {
        ignoreHeaderText: !1,
        colValueArray: void 0,
        allowAddlPercent: void 0,
        formatterOverride: void 0,
        autosizeMode: Slick.ColAutosizeMode.ContentIntelligent,
        rowSelectionModeOnInit: void 0,
        rowSelectionMode: Slick.RowSelectionMode.FirstNRows,
        rowSelectionCount: 100,
        valueFilterMode: Slick.ValueFilterMode.None,
        widthEvalMode: Slick.WidthEvalMode.CanvasTextSize,
        sizeToRemaining: void 0,
        widthPx: void 0,
        colDataTypeOf: void 0
      }, th, h, ph, n, cj, page = 0, offset = 0, vScrollDir = 1, initialized = !1, $container, uid = "slickgrid_" + Math.round(1e6 * Math.random()), self = this, $focusSink, $focusSink2;
      $();
      var $headerScroller, $headers, $headerRow, $headerRowScroller, $headerRowSpacerL, $headerRowSpacerR, $footerRow, $footerRowScroller, $footerRowSpacerL, $footerRowSpacerR, $preHeaderPanel, $preHeaderPanelScroller, $preHeaderPanelSpacer, $preHeaderPanelR, $preHeaderPanelScrollerR, $topPanelScroller, $topPanel, $viewport, $canvas, $style, $boundAncestors, treeColumns, stylesheet, columnCssRulesL, columnCssRulesR, viewportH, viewportW, canvasWidth, canvasWidthL, canvasWidthR, headersWidth, headersWidthL, headersWidthR, viewportHasHScroll, viewportHasVScroll, headerColumnWidthDiff = 0, headerColumnHeightDiff = 0, cellWidthDiff = 0, cellHeightDiff = 0, jQueryNewWidthBehaviour = !1, absoluteColumnMinWidth, hasFrozenRows = !1, frozenRowsHeight = 0, actualFrozenRow = -1, paneTopH = 0, paneBottomH = 0, viewportTopH = 0, topPanelH = 0, headerRowH = 0, footerRowH = 0, tabbingDirection = 1, $activeCanvasNode, $activeViewportNode, activePosX, activeRow, activeCell, activeCellNode = null, currentEditor = null, serializedEditorValue, editController, rowsCache = {}, renderedRows = 0, numVisibleRows = 0, prevScrollTop = 0, scrollTop = 0, lastRenderedScrollTop = 0, lastRenderedScrollLeft = 0, prevScrollLeft = 0, scrollLeft = 0, selectionModel, selectedRows = [], plugins = [], cellCssClasses = {}, columnsById = {}, sortColumns = [], columnPosLeft = [], columnPosRight = [], pagingActive = !1, pagingIsLastPage = !1, scrollThrottle = ActionThrottle(render, 50), h_editorLoader = null, h_postrender = null, h_postrenderCleanup = null, postProcessedRows = {}, postProcessToRow = null, postProcessFromRow = null, postProcessedCleanupQueue = [], postProcessgroupId = 0, counter_rows_rendered = 0, counter_rows_removed = 0, $paneHeaderL, $paneHeaderR, $paneTopL, $paneTopR, $paneBottomL, $paneBottomR, $headerScrollerL, $headerScrollerR, $headerL, $headerR, $groupHeadersL, $groupHeadersR, $headerRowScrollerL, $headerRowScrollerR, $footerRowScrollerL, $footerRowScrollerR, $headerRowL, $headerRowR, $footerRowL, $footerRowR, $topPanelScrollerL, $topPanelScrollerR, $topPanelL, $topPanelR, $viewportTopL, $viewportTopR, $viewportBottomL, $viewportBottomR, $canvasTopL, $canvasTopR, $canvasBottomL, $canvasBottomR, $viewportScrollContainerX, $viewportScrollContainerY, $headerScrollContainer, $headerRowScrollContainer, $footerRowScrollContainer, cssShow = { position: "absolute", visibility: "hidden", display: "block" }, $hiddenParents, oldProps = [], columnResizeDragging = !1;
      function init() {
        if (container instanceof $ ? $container = container : $container = $(container), $container.length < 1)
          throw new Error("SlickGrid requires a valid container, " + container + " does not exist in the DOM.");
        if (maxSupportedCssHeight = maxSupportedCssHeight || getMaxSupportedCssHeight(), options = $.extend({}, defaults, options), validateAndEnforceOptions(), columnDefaults.width = options.defaultColumnWidth, options.suppressCssChangesOnHiddenInit || cacheCssForHiddenInit(), treeColumns = new Slick.TreeColumns(columns), columns = treeColumns.extractColumns(), updateColumnProps(), options.enableColumnReorder && !$.fn.sortable)
          throw new Error("SlickGrid's 'enableColumnReorder = true' option requires jquery-ui.sortable module to be loaded");
        if (editController = {
          commitCurrentEdit,
          cancelCurrentEdit
        }, $container.empty().css("overflow", "hidden").css("outline", 0).addClass(uid).addClass("ui-widget"), /relative|absolute|fixed/.test($container.css("position")) || $container.css("position", "relative"), $focusSink = $("<div tabIndex='0' hideFocus style='position:fixed;width:0;height:0;top:0;left:0;outline:0;'></div>").appendTo($container), $paneHeaderL = $("<div class='slick-pane slick-pane-header slick-pane-left' tabIndex='0' />").appendTo($container), $paneHeaderR = $("<div class='slick-pane slick-pane-header slick-pane-right' tabIndex='0' />").appendTo($container), $paneTopL = $("<div class='slick-pane slick-pane-top slick-pane-left' tabIndex='0' />").appendTo($container), $paneTopR = $("<div class='slick-pane slick-pane-top slick-pane-right' tabIndex='0' />").appendTo($container), $paneBottomL = $("<div class='slick-pane slick-pane-bottom slick-pane-left' tabIndex='0' />").appendTo($container), $paneBottomR = $("<div class='slick-pane slick-pane-bottom slick-pane-right' tabIndex='0' />").appendTo($container), options.createPreHeaderPanel && ($preHeaderPanelScroller = $("<div class='slick-preheader-panel ui-state-default' style='overflow:hidden;position:relative;' />").appendTo($paneHeaderL), $preHeaderPanel = $("<div />").appendTo($preHeaderPanelScroller), $preHeaderPanelSpacer = $("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($preHeaderPanelScroller), $preHeaderPanelScrollerR = $("<div class='slick-preheader-panel ui-state-default' style='overflow:hidden;position:relative;' />").appendTo($paneHeaderR), $preHeaderPanelR = $("<div />").appendTo($preHeaderPanelScrollerR), $("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($preHeaderPanelScrollerR), options.showPreHeaderPanel || ($preHeaderPanelScroller.hide(), $preHeaderPanelScrollerR.hide())), $headerScrollerL = $("<div class='slick-header ui-state-default slick-header-left' />").appendTo($paneHeaderL), $headerScrollerR = $("<div class='slick-header ui-state-default slick-header-right' />").appendTo($paneHeaderR), $headerScroller = $().add($headerScrollerL).add($headerScrollerR), treeColumns.hasDepth()) {
          $groupHeadersL = [], $groupHeadersR = [];
          for (var O = 0; O < treeColumns.getDepth() - 1; O++)
            $groupHeadersL[O] = $("<div class='slick-group-header-columns slick-group-header-columns-left' style='left:-1000px' />").appendTo($headerScrollerL), $groupHeadersR[O] = $("<div class='slick-group-header-columns slick-group-header-columns-right' style='left:-1000px' />").appendTo($headerScrollerR);
          $().add($groupHeadersL).add($groupHeadersR);
        }
        $headerL = $("<div class='slick-header-columns slick-header-columns-left' style='left:-1000px' />").appendTo($headerScrollerL), $headerR = $("<div class='slick-header-columns slick-header-columns-right' style='left:-1000px' />").appendTo($headerScrollerR), $headers = $().add($headerL).add($headerR), $headerRowScrollerL = $("<div class='slick-headerrow ui-state-default' />").appendTo($paneTopL), $headerRowScrollerR = $("<div class='slick-headerrow ui-state-default' />").appendTo($paneTopR), $headerRowScroller = $().add($headerRowScrollerL).add($headerRowScrollerR), $headerRowSpacerL = $("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($headerRowScrollerL), $headerRowSpacerR = $("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($headerRowScrollerR), $headerRowL = $("<div class='slick-headerrow-columns slick-headerrow-columns-left' />").appendTo($headerRowScrollerL), $headerRowR = $("<div class='slick-headerrow-columns slick-headerrow-columns-right' />").appendTo($headerRowScrollerR), $headerRow = $().add($headerRowL).add($headerRowR), $topPanelScrollerL = $("<div class='slick-top-panel-scroller ui-state-default' />").appendTo($paneTopL), $topPanelScrollerR = $("<div class='slick-top-panel-scroller ui-state-default' />").appendTo($paneTopR), $topPanelScroller = $().add($topPanelScrollerL).add($topPanelScrollerR), $topPanelL = $("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerL), $topPanelR = $("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerR), $topPanel = $().add($topPanelL).add($topPanelR), options.showColumnHeader || $headerScroller.hide(), options.showTopPanel || $topPanelScroller.hide(), options.showHeaderRow || $headerRowScroller.hide(), $viewportTopL = $("<div class='slick-viewport slick-viewport-top slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneTopL), $viewportTopR = $("<div class='slick-viewport slick-viewport-top slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneTopR), $viewportBottomL = $("<div class='slick-viewport slick-viewport-bottom slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneBottomL), $viewportBottomR = $("<div class='slick-viewport slick-viewport-bottom slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneBottomR), $viewport = $().add($viewportTopL).add($viewportTopR).add($viewportBottomL).add($viewportBottomR), $activeViewportNode = $viewportTopL, $canvasTopL = $("<div class='grid-canvas grid-canvas-top grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportTopL), $canvasTopR = $("<div class='grid-canvas grid-canvas-top grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportTopR), $canvasBottomL = $("<div class='grid-canvas grid-canvas-bottom grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportBottomL), $canvasBottomR = $("<div class='grid-canvas grid-canvas-bottom grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportBottomR), options.viewportClass && $viewport.toggleClass(options.viewportClass, !0), $canvas = $().add($canvasTopL).add($canvasTopR).add($canvasBottomL).add($canvasBottomR), scrollbarDimensions = scrollbarDimensions || measureScrollbar(), $activeCanvasNode = $canvasTopL, $preHeaderPanelSpacer && $preHeaderPanelSpacer.css("width", getCanvasWidth() + scrollbarDimensions.width + "px"), $headers.width(getHeadersWidth()), $headerRowSpacerL.css("width", getCanvasWidth() + scrollbarDimensions.width + "px"), $headerRowSpacerR.css("width", getCanvasWidth() + scrollbarDimensions.width + "px"), options.createFooterRow && ($footerRowScrollerR = $("<div class='slick-footerrow ui-state-default' />").appendTo($paneTopR), $footerRowScrollerL = $("<div class='slick-footerrow ui-state-default' />").appendTo($paneTopL), $footerRowScroller = $().add($footerRowScrollerL).add($footerRowScrollerR), $footerRowSpacerL = $("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width", getCanvasWidth() + scrollbarDimensions.width + "px").appendTo($footerRowScrollerL), $footerRowSpacerR = $("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width", getCanvasWidth() + scrollbarDimensions.width + "px").appendTo($footerRowScrollerR), $footerRowL = $("<div class='slick-footerrow-columns slick-footerrow-columns-left' />").appendTo($footerRowScrollerL), $footerRowR = $("<div class='slick-footerrow-columns slick-footerrow-columns-right' />").appendTo($footerRowScrollerR), $footerRow = $().add($footerRowL).add($footerRowR), options.showFooterRow || $footerRowScroller.hide()), $focusSink2 = $focusSink.clone().appendTo($container), options.explicitInitialization || finishInitialization();
      }
      function finishInitialization() {
        initialized || (initialized = !0, getViewportWidth(), getViewportHeight(), measureCellPaddingAndBorder(), disableSelection($headers), options.enableTextSelectionOnCells || $viewport.on("selectstart.ui", function(O) {
          return $(O.target).is("input,textarea");
        }), setFrozenOptions(), setPaneVisibility(), setScroller(), setOverflow(), updateColumnCaches(), createColumnHeaders(), createColumnGroupHeaders(), createColumnFooter(), setupColumnSort(), createCssRules(), resizeCanvas(), bindAncestorScrollEvents(), $container.on("resize.slickgrid", resizeCanvas), $viewport.on("scroll", handleScroll), jQuery.fn.mousewheel && options.enableMouseWheelScrollHandler && $viewport.on("mousewheel", handleMouseWheel), $headerScroller.on("contextmenu", handleHeaderContextMenu).on("click", handleHeaderClick).on("mouseenter", ".slick-header-column", handleHeaderMouseEnter).on("mouseleave", ".slick-header-column", handleHeaderMouseLeave), $headerRowScroller.on("scroll", handleHeaderRowScroll), options.createFooterRow && ($footerRow.on("contextmenu", handleFooterContextMenu).on("click", handleFooterClick), $footerRowScroller.on("scroll", handleFooterRowScroll)), options.createPreHeaderPanel && $preHeaderPanelScroller.on("scroll", handlePreHeaderPanelScroll), $focusSink.add($focusSink2).on("keydown", handleKeyDown), $canvas.on("keydown", handleKeyDown).on("click", handleClick).on("dblclick", handleDblClick).on("contextmenu", handleContextMenu).on("draginit", handleDragInit).on("dragstart", { distance: 3 }, handleDragStart).on("drag", handleDrag).on("dragend", handleDragEnd).on("mouseenter", ".slick-cell", handleMouseEnter).on("mouseleave", ".slick-cell", handleMouseLeave), options.suppressCssChangesOnHiddenInit || restoreCssFromHiddenInit());
      }
      function cacheCssForHiddenInit() {
        $hiddenParents = $container.parents().addBack().not(":visible"), $hiddenParents.each(function() {
          var O = {};
          for (var _ in cssShow)
            O[_] = this.style[_], this.style[_] = cssShow[_];
          oldProps.push(O);
        });
      }
      function restoreCssFromHiddenInit() {
        $hiddenParents.each(function(O) {
          var _ = oldProps[O];
          for (var H in cssShow)
            this.style[H] = _[H];
        });
      }
      function hasFrozenColumns() {
        return options.frozenColumn > -1;
      }
      function registerPlugin(O) {
        plugins.unshift(O), O.init(self);
      }
      function unregisterPlugin(O) {
        for (var _ = plugins.length; _ >= 0; _--)
          if (plugins[_] === O) {
            plugins[_].destroy && plugins[_].destroy(), plugins.splice(_, 1);
            break;
          }
      }
      function getPluginByName(O) {
        for (var _ = plugins.length - 1; _ >= 0; _--)
          if (plugins[_].pluginName === O)
            return plugins[_];
      }
      function setSelectionModel(O) {
        selectionModel && (selectionModel.onSelectedRangesChanged.unsubscribe(handleSelectedRangesChanged), selectionModel.destroy && selectionModel.destroy()), selectionModel = O, selectionModel && (selectionModel.init(self), selectionModel.onSelectedRangesChanged.subscribe(handleSelectedRangesChanged));
      }
      function getSelectionModel() {
        return selectionModel;
      }
      function getCanvasNode(O, _) {
        return _getContainerElement(getCanvases(), O, _);
      }
      function getActiveCanvasNode(O) {
        return setActiveCanvasNode(O), $activeCanvasNode[0];
      }
      function getCanvases() {
        return $canvas;
      }
      function setActiveCanvasNode(O) {
        O && ($activeCanvasNode = $(O.target).closest(".grid-canvas"));
      }
      function getViewportNode(O, _) {
        return _getContainerElement(getViewports(), O, _);
      }
      function getViewports() {
        return $viewport;
      }
      function getActiveViewportNode(O) {
        return setActiveViewportNode(O), $activeViewportNode[0];
      }
      function setActiveViewportNode(O) {
        O && ($activeViewportNode = $(O.target).closest(".slick-viewport"));
      }
      function _getContainerElement(O, _, H) {
        if (O) {
          _ || (_ = 0), H || (H = 0);
          var V = typeof _ == "number" ? _ : getColumnIndex(_), q = hasFrozenRows && H >= actualFrozenRow + (options.frozenBottom ? 0 : 1), G = hasFrozenColumns() && V > options.frozenColumn;
          return O[(q ? 2 : 0) + (G ? 1 : 0)];
        }
      }
      function measureScrollbar() {
        var O = $('<div class="' + $viewport.className + '" style="position:absolute; top:-10000px; left:-10000px; overflow:auto; width:100px; height:100px;"></div>').appendTo("body"), _ = $('<div style="width:200px; height:200px; overflow:auto;"></div>').appendTo(O), H = {
          width: O[0].offsetWidth - O[0].clientWidth,
          height: O[0].offsetHeight - O[0].clientHeight
        };
        return _.remove(), O.remove(), H;
      }
      function getHeadersWidth() {
        headersWidth = headersWidthL = headersWidthR = 0;
        for (var O = !options.autoHeight, _ = 0, H = columns.length; _ < H; _++) {
          var V = columns[_].width;
          options.frozenColumn > -1 && _ > options.frozenColumn ? headersWidthR += V : headersWidthL += V;
        }
        return O && (options.frozenColumn > -1 && _ > options.frozenColumn ? headersWidthR += scrollbarDimensions.width : headersWidthL += scrollbarDimensions.width), hasFrozenColumns() ? (headersWidthL = headersWidthL + 1e3, headersWidthR = Math.max(headersWidthR, viewportW) + headersWidthL, headersWidthR += scrollbarDimensions.width) : (headersWidthL += scrollbarDimensions.width, headersWidthL = Math.max(headersWidthL, viewportW) + 1e3), headersWidth = headersWidthL + headersWidthR, Math.max(headersWidth, viewportW) + 1e3;
      }
      function getHeadersWidthL() {
        return headersWidthL = 0, columns.forEach(function(O, _) {
          options.frozenColumn > -1 && _ > options.frozenColumn || (headersWidthL += O.width);
        }), hasFrozenColumns() ? headersWidthL += 1e3 : (headersWidthL += scrollbarDimensions.width, headersWidthL = Math.max(headersWidthL, viewportW) + 1e3), headersWidthL;
      }
      function getHeadersWidthR() {
        return headersWidthR = 0, columns.forEach(function(O, _) {
          options.frozenColumn > -1 && _ > options.frozenColumn && (headersWidthR += O.width);
        }), hasFrozenColumns() && (headersWidthR = Math.max(headersWidthR, viewportW) + getHeadersWidthL(), headersWidthR += scrollbarDimensions.width), headersWidthR;
      }
      function getCanvasWidth() {
        var O = viewportHasVScroll ? viewportW - scrollbarDimensions.width : viewportW, _ = columns.length;
        for (canvasWidthL = canvasWidthR = 0; _--; )
          hasFrozenColumns() && _ > options.frozenColumn ? canvasWidthR += columns[_].width : canvasWidthL += columns[_].width;
        var H = canvasWidthL + canvasWidthR;
        return options.fullWidthRows ? Math.max(H, O) : H;
      }
      function updateCanvasWidth(O) {
        var _ = canvasWidth, H = canvasWidthL, V = canvasWidthR, q;
        canvasWidth = getCanvasWidth(), q = canvasWidth !== _ || canvasWidthL !== H || canvasWidthR !== V, (q || hasFrozenColumns() || hasFrozenRows) && ($canvasTopL.width(canvasWidthL), getHeadersWidth(), $headerL.width(headersWidthL), $headerR.width(headersWidthR), hasFrozenColumns() ? ($canvasTopR.width(canvasWidthR), $paneHeaderL.width(canvasWidthL), $paneHeaderR.css("left", canvasWidthL), $paneHeaderR.css("width", viewportW - canvasWidthL), $paneTopL.width(canvasWidthL), $paneTopR.css("left", canvasWidthL), $paneTopR.css("width", viewportW - canvasWidthL), $headerRowScrollerL.width(canvasWidthL), $headerRowScrollerR.width(viewportW - canvasWidthL), $headerRowL.width(canvasWidthL), $headerRowR.width(canvasWidthR), options.createFooterRow && ($footerRowScrollerL.width(canvasWidthL), $footerRowScrollerR.width(viewportW - canvasWidthL), $footerRowL.width(canvasWidthL), $footerRowR.width(canvasWidthR)), options.createPreHeaderPanel && $preHeaderPanel.width(canvasWidth), $viewportTopL.width(canvasWidthL), $viewportTopR.width(viewportW - canvasWidthL), hasFrozenRows && ($paneBottomL.width(canvasWidthL), $paneBottomR.css("left", canvasWidthL), $viewportBottomL.width(canvasWidthL), $viewportBottomR.width(viewportW - canvasWidthL), $canvasBottomL.width(canvasWidthL), $canvasBottomR.width(canvasWidthR))) : ($paneHeaderL.width("100%"), $paneTopL.width("100%"), $headerRowScrollerL.width("100%"), $headerRowL.width(canvasWidth), options.createFooterRow && ($footerRowScrollerL.width("100%"), $footerRowL.width(canvasWidth)), options.createPreHeaderPanel && ($preHeaderPanel.width("100%"), $preHeaderPanel.width(canvasWidth)), $viewportTopL.width("100%"), hasFrozenRows && ($viewportBottomL.width("100%"), $canvasBottomL.width(canvasWidthL))), viewportHasHScroll = canvasWidth >= viewportW - scrollbarDimensions.width), $headerRowSpacerL.width(canvasWidth + (viewportHasVScroll ? scrollbarDimensions.width : 0)), $headerRowSpacerR.width(canvasWidth + (viewportHasVScroll ? scrollbarDimensions.width : 0)), options.createFooterRow && ($footerRowSpacerL.width(canvasWidth + (viewportHasVScroll ? scrollbarDimensions.width : 0)), $footerRowSpacerR.width(canvasWidth + (viewportHasVScroll ? scrollbarDimensions.width : 0))), (q || O) && applyColumnWidths();
      }
      function disableSelection(O) {
        O && O.jquery && O.attr("unselectable", "on").css("MozUserSelect", "none").on("selectstart.ui", function() {
          return !1;
        });
      }
      function getMaxSupportedCssHeight() {
        for (var O = 1e6, _ = navigator.userAgent.toLowerCase().match(/firefox/) ? 6e6 : 1e9, H = $("<div style='display:none' />").appendTo(document.body); ; ) {
          var V = O * 2;
          if (H.css("height", V), V > _ || H.height() !== V)
            break;
          O = V;
        }
        return H.remove(), O;
      }
      function getUID() {
        return uid;
      }
      function getHeaderColumnWidthDiff() {
        return headerColumnWidthDiff;
      }
      function getScrollbarDimensions() {
        return scrollbarDimensions;
      }
      function bindAncestorScrollEvents() {
        for (var O = hasFrozenRows && !options.frozenBottom ? $canvasBottomL[0] : $canvasTopL[0]; (O = O.parentNode) != document.body && O != null; )
          if (O == $viewportTopL[0] || O.scrollWidth != O.clientWidth || O.scrollHeight != O.clientHeight) {
            var _ = $(O);
            $boundAncestors ? $boundAncestors = $boundAncestors.add(_) : $boundAncestors = _, _.on("scroll." + uid, handleActiveCellPositionChange);
          }
      }
      function unbindAncestorScrollEvents() {
        $boundAncestors && ($boundAncestors.off("scroll." + uid), $boundAncestors = null);
      }
      function updateColumnHeader(O, _, H) {
        if (initialized) {
          var V = getColumnIndex(O);
          if (V != null) {
            var q = columns[V], G = $headers.children().eq(V);
            G && (_ !== void 0 && (columns[V].name = _), H !== void 0 && (columns[V].toolTip = H), trigger(self.onBeforeHeaderCellDestroy, {
              node: G[0],
              column: q,
              grid: self
            }), G.attr("title", H || "").children().eq(0).html(_), trigger(self.onHeaderCellRendered, {
              node: G[0],
              column: q,
              grid: self
            }));
          }
        }
      }
      function getHeader(O) {
        if (!O)
          return hasFrozenColumns() ? $headers : $headerL;
        var _ = getColumnIndex(O.id);
        return hasFrozenColumns() ? _ <= options.frozenColumn ? $headerL : $headerR : $headerL;
      }
      function getHeaderColumn(O) {
        var _ = typeof O == "number" ? O : getColumnIndex(O), H = hasFrozenColumns() ? _ <= options.frozenColumn ? $headerL : $headerR : $headerL, V = hasFrozenColumns() ? _ <= options.frozenColumn ? _ : _ - options.frozenColumn - 1 : _, q = H.children().eq(V);
        return q && q[0];
      }
      function getHeaderRow() {
        return hasFrozenColumns() ? $headerRow : $headerRow[0];
      }
      function getFooterRow() {
        return hasFrozenColumns() ? $footerRow : $footerRow[0];
      }
      function getPreHeaderPanel() {
        return $preHeaderPanel[0];
      }
      function getPreHeaderPanelRight() {
        return $preHeaderPanelR[0];
      }
      function getHeaderRowColumn(O) {
        var _ = typeof O == "number" ? O : getColumnIndex(O), H;
        hasFrozenColumns() ? _ <= options.frozenColumn ? H = $headerRowL : (H = $headerRowR, _ -= options.frozenColumn + 1) : H = $headerRowL;
        var V = H.children().eq(_);
        return V && V[0];
      }
      function getFooterRowColumn(O) {
        var _ = typeof O == "number" ? O : getColumnIndex(O), H;
        hasFrozenColumns() ? _ <= options.frozenColumn ? H = $footerRowL : (H = $footerRowR, _ -= options.frozenColumn + 1) : H = $footerRowL;
        var V = H && H.children().eq(_);
        return V && V[0];
      }
      function createColumnFooter() {
        if (options.createFooterRow) {
          $footerRow.find(".slick-footerrow-column").each(function() {
            var V = $(this).data("column");
            V && trigger(self.onBeforeFooterRowCellDestroy, {
              node: this,
              column: V,
              grid: self
            });
          }), $footerRowL.empty(), $footerRowR.empty();
          for (var O = 0; O < columns.length; O++) {
            var _ = columns[O], H = $("<div class='ui-state-default slick-footerrow-column l" + O + " r" + O + "'></div>").data("column", _).addClass(hasFrozenColumns() && O <= options.frozenColumn ? "frozen" : "").appendTo(hasFrozenColumns() && O > options.frozenColumn ? $footerRowR : $footerRowL);
            trigger(self.onFooterRowCellRendered, {
              node: H[0],
              column: _,
              grid: self
            });
          }
        }
      }
      function createColumnGroupHeaders() {
        var O = 0, _ = !1;
        if (treeColumns.hasDepth()) {
          for (var H = 0; H < $groupHeadersL.length; H++) {
            $groupHeadersL[H].empty(), $groupHeadersR[H].empty();
            var V = treeColumns.getColumnsInDepth(H);
            for (var q in V) {
              var G = V[q];
              O += G.extractColumns().length, hasFrozenColumns() && H === 0 && O - 1 === options.frozenColumn && (_ = !0), $("<div class='ui-state-default slick-group-header-column' />").html("<span class='slick-column-name'>" + G.name + "</span>").attr("id", "" + uid + G.id).attr("title", G.toolTip || "").data("column", G).addClass(G.headerCssClass || "").addClass(hasFrozenColumns() && O - 1 > options.frozenColumn ? "frozen" : "").appendTo(hasFrozenColumns() && O - 1 > options.frozenColumn ? $groupHeadersR[H] : $groupHeadersL[H]);
            }
            if (hasFrozenColumns() && H === 0 && !_) {
              $groupHeadersL[H].empty(), $groupHeadersR[H].empty(), alert("All columns of group should to be grouped!");
              break;
            }
          }
          applyColumnGroupHeaderWidths();
        }
      }
      function createColumnHeaders() {
        function O() {
          $(this).addClass("ui-state-hover");
        }
        function _() {
          $(this).removeClass("ui-state-hover");
        }
        $headers.find(".slick-header-column").each(function() {
          var ee = $(this).data("column");
          ee && trigger(self.onBeforeHeaderCellDestroy, {
            node: this,
            column: ee,
            grid: self
          });
        }), $headerL.empty(), $headerR.empty(), getHeadersWidth(), $headerL.width(headersWidthL), $headerR.width(headersWidthR), $headerRow.find(".slick-headerrow-column").each(function() {
          var ee = $(this).data("column");
          ee && trigger(self.onBeforeHeaderRowCellDestroy, {
            node: this,
            column: ee,
            grid: self
          });
        }), $headerRowL.empty(), $headerRowR.empty(), options.createFooterRow && ($footerRowL.find(".slick-footerrow-column").each(function() {
          var ee = $(this).data("column");
          ee && trigger(self.onBeforeFooterRowCellDestroy, {
            node: this,
            column: ee,
            grid: self
          });
        }), $footerRowL.empty(), hasFrozenColumns() && ($footerRowR.find(".slick-footerrow-column").each(function() {
          var ee = $(this).data("column");
          ee && trigger(self.onBeforeFooterRowCellDestroy, {
            node: this,
            column: ee,
            grid: self
          });
        }), $footerRowR.empty()));
        for (var H = 0; H < columns.length; H++) {
          var V = columns[H], q = hasFrozenColumns() ? H <= options.frozenColumn ? $headerL : $headerR : $headerL, G = hasFrozenColumns() ? H <= options.frozenColumn ? $headerRowL : $headerRowR : $headerRowL, J = $("<div class='ui-state-default slick-header-column' />").html("<span class='slick-column-name'>" + V.name + "</span>").width(V.width - headerColumnWidthDiff).attr("id", "" + uid + V.id).attr("title", V.toolTip || "").data("column", V).addClass(V.headerCssClass || "").addClass(hasFrozenColumns() && H <= options.frozenColumn ? "frozen" : "").appendTo(q);
          if ((options.enableColumnReorder || V.sortable) && J.on("mouseenter", O).on("mouseleave", _), V.hasOwnProperty("headerCellAttrs") && V.headerCellAttrs instanceof Object)
            for (var Z in V.headerCellAttrs)
              V.headerCellAttrs.hasOwnProperty(Z) && J.attr(Z, V.headerCellAttrs[Z]);
          if (V.sortable && (J.addClass("slick-header-sortable"), J.append("<span class='slick-sort-indicator" + (options.numberedMultiColumnSort && !options.sortColNumberInSeparateSpan ? " slick-sort-indicator-numbered" : "") + "' />"), options.numberedMultiColumnSort && options.sortColNumberInSeparateSpan && J.append("<span class='slick-sort-indicator-numbered' />")), trigger(self.onHeaderCellRendered, {
            node: J[0],
            column: V,
            grid: self
          }), options.showHeaderRow) {
            var te = $("<div class='ui-state-default slick-headerrow-column l" + H + " r" + H + "'></div>").data("column", V).addClass(hasFrozenColumns() && H <= options.frozenColumn ? "frozen" : "").appendTo(G);
            trigger(self.onHeaderRowCellRendered, {
              node: te[0],
              column: V,
              grid: self
            });
          }
          if (options.createFooterRow && options.showFooterRow) {
            var ne = $("<div class='ui-state-default slick-footerrow-column l" + H + " r" + H + "'></div>").data("column", V).appendTo($footerRow);
            trigger(self.onFooterRowCellRendered, {
              node: ne[0],
              column: V,
              grid: self
            });
          }
        }
        setSortColumns(sortColumns), setupColumnResize(), options.enableColumnReorder && (typeof options.enableColumnReorder == "function" ? options.enableColumnReorder(self, $headers, headerColumnWidthDiff, setColumns, setupColumnResize, columns, getColumnIndex, uid, trigger) : setupColumnReorder());
      }
      function setupColumnSort() {
        $headers.click(function(O) {
          if (!columnResizeDragging && (O.metaKey = O.metaKey || O.ctrlKey, !$(O.target).hasClass("slick-resizable-handle"))) {
            var _ = $(O.target).closest(".slick-header-column");
            if (_.length) {
              var H = _.data("column");
              if (H.sortable) {
                if (!getEditorLock().commitCurrentEdit())
                  return;
                for (var V = $.extend(!0, [], sortColumns), q = null, G = 0; G < sortColumns.length; G++)
                  if (sortColumns[G].columnId == H.id) {
                    q = sortColumns[G], q.sortAsc = !q.sortAsc;
                    break;
                  }
                var J = !!q;
                options.tristateMultiColumnSort ? (q || (q = { columnId: H.id, sortAsc: H.defaultSortAsc }), J && q.sortAsc && (sortColumns.splice(G, 1), q = null), options.multiColumnSort || (sortColumns = []), q && (!J || !options.multiColumnSort) && sortColumns.push(q)) : O.metaKey && options.multiColumnSort ? q && sortColumns.splice(G, 1) : ((!O.shiftKey && !O.metaKey || !options.multiColumnSort) && (sortColumns = []), q ? sortColumns.length === 0 && sortColumns.push(q) : (q = { columnId: H.id, sortAsc: H.defaultSortAsc }, sortColumns.push(q)));
                var Z;
                options.multiColumnSort ? Z = {
                  multiColumnSort: !0,
                  previousSortColumns: V,
                  sortCols: $.map(sortColumns, function(te) {
                    return { columnId: columns[getColumnIndex(te.columnId)].id, sortCol: columns[getColumnIndex(te.columnId)], sortAsc: te.sortAsc };
                  })
                } : Z = {
                  multiColumnSort: !1,
                  previousSortColumns: V,
                  columnId: sortColumns.length > 0 ? H.id : null,
                  sortCol: sortColumns.length > 0 ? H : null,
                  sortAsc: sortColumns.length > 0 ? sortColumns[0].sortAsc : !0
                }, trigger(self.onBeforeSort, Z, O) !== !1 && (setSortColumns(sortColumns), trigger(self.onSort, Z, O));
              }
            }
          }
        });
      }
      function currentPositionInHeader(O) {
        var _ = 0;
        return $headers.find(".slick-header-column").each(function(H) {
          if (this.id == O)
            return _ = H, !1;
        }), _;
      }
      function limitPositionInGroup(O) {
        var _, H = 0, V = 0;
        return treeColumns.getColumnsInDepth($groupHeadersL.length - 1).some(function(q) {
          return H = V, V += q.columns.length, q.columns.some(function(G) {
            return G.id === O && (_ = q), _;
          }), _;
        }), V--, {
          start: H,
          end: V,
          group: _
        };
      }
      function columnPositionValidInGroup(O) {
        var _ = currentPositionInHeader(O[0].id), H = limitPositionInGroup(O.data("column").id), V = H.start <= _ && _ <= H.end;
        return {
          limit: H,
          valid: V,
          message: V ? "" : 'Column "'.concat(O.text(), '" can be reordered only within the "', H.group.name, '" group!')
        };
      }
      function setupColumnReorder() {
        $headers.filter(":ui-sortable").sortable("destroy");
        var O = null;
        function _() {
          $viewportScrollContainerX[0].scrollLeft = $viewportScrollContainerX[0].scrollLeft + 10;
        }
        function H() {
          $viewportScrollContainerX[0].scrollLeft = $viewportScrollContainerX[0].scrollLeft - 10;
        }
        var V;
        $headers.sortable({
          containment: "parent",
          distance: 3,
          axis: "x",
          cursor: "default",
          tolerance: "intersection",
          helper: "clone",
          placeholder: "slick-sortable-placeholder ui-state-default slick-header-column",
          start: function(q, G) {
            G.placeholder.width(G.helper.outerWidth() - headerColumnWidthDiff), V = !hasFrozenColumns() || G.placeholder.offset().left + G.placeholder.width() > $viewportScrollContainerX.offset().left, $(G.helper).addClass("slick-header-column-active");
          },
          beforeStop: function(q, G) {
            $(G.helper).removeClass("slick-header-column-active");
          },
          sort: function(q, G) {
            V && q.originalEvent.pageX > $container[0].clientWidth ? O || (O = setInterval(
              _,
              100
            )) : V && q.originalEvent.pageX < $viewportScrollContainerX.offset().left ? O || (O = setInterval(
              H,
              100
            )) : (clearInterval(O), O = null);
          },
          stop: function(q, G) {
            var J = !1;
            clearInterval(O), O = null;
            var Z = null;
            if (treeColumns.hasDepth()) {
              var te = columnPositionValidInGroup(G.item);
              Z = te.limit, J = !te.valid, J && alert(te.message);
            }
            if (J || !getEditorLock().commitCurrentEdit()) {
              $(this).sortable("cancel");
              return;
            }
            var ne = $headerL.sortable("toArray");
            ne = ne.concat($headerR.sortable("toArray"));
            for (var ee = [], ae = 0; ae < ne.length; ae++)
              ee.push(columns[getColumnIndex(ne[ae].replace(uid, ""))]);
            setColumns(ee), trigger(self.onColumnsReordered, { impactedColumns: getImpactedColumns(Z) }), q.stopPropagation(), setupColumnResize();
          }
        });
      }
      function getImpactedColumns(O) {
        var _ = [];
        if (O)
          for (var H = O.start; H <= O.end; H++)
            _.push(columns[H]);
        else
          _ = columns;
        return _;
      }
      function setupColumnResize() {
        var O, _, H, V, q, G, J, Z, te, ne = 0;
        q = $headers.children(), q.find(".slick-resizable-handle").remove(), q.each(function(ee, ae) {
          ee >= columns.length || columns[ee].resizable && (Z === void 0 && (Z = ee), te = ee);
        }), Z !== void 0 && q.each(function(ee, ae) {
          ee >= columns.length || ee < Z || options.forceFitColumns && ee >= te || ($(ae), $("<div class='slick-resizable-handle' />").appendTo(ae).on("dragstart", function(oe, se) {
            if (!getEditorLock().commitCurrentEdit())
              return !1;
            V = oe.pageX, ne = 0, $(this).parent().addClass("slick-header-column-active");
            var ce = null, le = null;
            if (q.each(function(fe, he) {
              fe >= columns.length || (columns[fe].previousWidth = $(he).outerWidth());
            }), options.forceFitColumns)
              for (ce = 0, le = 0, O = ee + 1; O < columns.length; O++)
                H = columns[O], H.resizable && (le !== null && (H.maxWidth ? le += H.maxWidth - H.previousWidth : le = null), ce += H.previousWidth - Math.max(H.minWidth || 0, absoluteColumnMinWidth));
            var ue = 0, de = 0;
            for (O = 0; O <= ee; O++)
              H = columns[O], H.resizable && (de !== null && (H.maxWidth ? de += H.maxWidth - H.previousWidth : de = null), ue += H.previousWidth - Math.max(H.minWidth || 0, absoluteColumnMinWidth));
            ce === null && (ce = 1e5), ue === null && (ue = 1e5), le === null && (le = 1e5), de === null && (de = 1e5), J = V + Math.min(ce, de), G = V - Math.min(ue, le);
          }).on("drag", function(oe, se) {
            columnResizeDragging = !0;
            var ce, le = Math.min(J, Math.max(G, oe.pageX)) - V, ue, de = 0, fe = 0, he = viewportHasVScroll ? viewportW - scrollbarDimensions.width : viewportW;
            if (le < 0) {
              for (ue = le, O = ee; O >= 0; O--)
                H = columns[O], H.resizable && (ce = Math.max(H.minWidth || 0, absoluteColumnMinWidth), ue && H.previousWidth + ue < ce ? (ue += H.previousWidth - ce, H.width = ce) : (H.width = H.previousWidth + ue, ue = 0));
              for (_ = 0; _ <= ee; _++)
                H = columns[_], hasFrozenColumns() && _ > options.frozenColumn ? fe += H.width : de += H.width;
              if (options.forceFitColumns)
                for (ue = -le, O = ee + 1; O < columns.length; O++)
                  H = columns[O], H.resizable && (ue && H.maxWidth && H.maxWidth - H.previousWidth < ue ? (ue -= H.maxWidth - H.previousWidth, H.width = H.maxWidth) : (H.width = H.previousWidth + ue, ue = 0), hasFrozenColumns() && O > options.frozenColumn ? fe += H.width : de += H.width);
              else
                for (O = ee + 1; O < columns.length; O++)
                  H = columns[O], hasFrozenColumns() && O > options.frozenColumn ? fe += H.width : de += H.width;
              if (options.forceFitColumns)
                for (ue = -le, O = ee + 1; O < columns.length; O++)
                  H = columns[O], H.resizable && (ue && H.maxWidth && H.maxWidth - H.previousWidth < ue ? (ue -= H.maxWidth - H.previousWidth, H.width = H.maxWidth) : (H.width = H.previousWidth + ue, ue = 0));
            } else {
              for (ue = le, de = 0, fe = 0, O = ee; O >= 0; O--)
                if (H = columns[O], H.resizable)
                  if (ue && H.maxWidth && H.maxWidth - H.previousWidth < ue)
                    ue -= H.maxWidth - H.previousWidth, H.width = H.maxWidth;
                  else {
                    var me = H.previousWidth + ue, pe = canvasWidthL + ue;
                    hasFrozenColumns() && O <= options.frozenColumn ? (me > ne && pe < he - options.frozenRightViewportMinWidth && (ne = me), H.width = pe + options.frozenRightViewportMinWidth > he ? ne : me) : H.width = me, ue = 0;
                  }
              for (_ = 0; _ <= ee; _++)
                H = columns[_], hasFrozenColumns() && _ > options.frozenColumn ? fe += H.width : de += H.width;
              if (options.forceFitColumns)
                for (ue = -le, O = ee + 1; O < columns.length; O++)
                  H = columns[O], H.resizable && (ce = Math.max(H.minWidth || 0, absoluteColumnMinWidth), ue && H.previousWidth + ue < ce ? (ue += H.previousWidth - ce, H.width = ce) : (H.width = H.previousWidth + ue, ue = 0), hasFrozenColumns() && O > options.frozenColumn ? fe += H.width : de += H.width);
              else
                for (O = ee + 1; O < columns.length; O++)
                  H = columns[O], hasFrozenColumns() && O > options.frozenColumn ? fe += H.width : de += H.width;
            }
            hasFrozenColumns() && de != canvasWidthL && ($headerL.width(de + 1e3), $paneHeaderR.css("left", de)), applyColumnHeaderWidths(), applyColumnGroupHeaderWidths(), options.syncColumnCellResize && applyColumnWidths(), trigger(self.onColumnsDrag, {
              triggeredByColumn: $(this).parent().attr("id").replace(uid, ""),
              resizeHandle: $(this)
            });
          }).on("dragend", function(oe, se) {
            $(this).parent().removeClass("slick-header-column-active");
            var ce = $(this).parent().attr("id").replace(uid, "");
            trigger(self.onBeforeColumnsResize, { triggeredByColumn: ce }) === !0 && (applyColumnHeaderWidths(), applyColumnGroupHeaderWidths());
            var le;
            for (O = 0; O < columns.length; O++)
              H = columns[O], le = $(q[O]).outerWidth(), H.previousWidth !== le && H.rerenderOnResize && invalidateAllRows();
            updateCanvasWidth(!0), render(), trigger(self.onColumnsResized, { triggeredByColumn: ce }), setTimeout(function() {
              columnResizeDragging = !1;
            }, 300);
          }).on("dblclick", function() {
            var oe = $(this).parent().attr("id").replace(uid, "");
            trigger(self.onColumnsResizeDblClick, { triggeredByColumn: oe });
          }));
        });
      }
      function getVBoxDelta(O) {
        var _ = ["borderTopWidth", "borderBottomWidth", "paddingTop", "paddingBottom"], H = 0;
        return O && typeof O.css == "function" && $.each(_, function(V, q) {
          H += parseFloat(O.css(q)) || 0;
        }), H;
      }
      function setFrozenOptions() {
        if (options.frozenColumn = options.frozenColumn >= 0 && options.frozenColumn < columns.length ? parseInt(options.frozenColumn) : -1, options.frozenRow > -1) {
          hasFrozenRows = !0, frozenRowsHeight = options.frozenRow * options.rowHeight;
          var O = getDataLength();
          actualFrozenRow = options.frozenBottom ? O - options.frozenRow : options.frozenRow;
        } else
          hasFrozenRows = !1;
      }
      function setPaneVisibility() {
        hasFrozenColumns() ? ($paneHeaderR.show(), $paneTopR.show(), hasFrozenRows ? ($paneBottomL.show(), $paneBottomR.show()) : ($paneBottomR.hide(), $paneBottomL.hide())) : ($paneHeaderR.hide(), $paneTopR.hide(), $paneBottomR.hide(), hasFrozenRows ? $paneBottomL.show() : ($paneBottomR.hide(), $paneBottomL.hide()));
      }
      function setOverflow() {
        $viewportTopL.css({
          "overflow-x": hasFrozenColumns() ? hasFrozenRows && !options.alwaysAllowHorizontalScroll ? "hidden" : "scroll" : hasFrozenRows && !options.alwaysAllowHorizontalScroll ? "hidden" : "auto",
          "overflow-y": !hasFrozenColumns() && options.alwaysShowVerticalScroll ? "scroll" : hasFrozenColumns() ? "hidden" : hasFrozenRows ? "scroll" : "auto"
        }), $viewportTopR.css({
          "overflow-x": hasFrozenColumns() ? hasFrozenRows && !options.alwaysAllowHorizontalScroll ? "hidden" : "scroll" : hasFrozenRows && !options.alwaysAllowHorizontalScroll ? "hidden" : "auto",
          "overflow-y": options.alwaysShowVerticalScroll ? "scroll" : (hasFrozenColumns(), hasFrozenRows ? "scroll" : "auto")
        }), $viewportBottomL.css({
          "overflow-x": hasFrozenColumns() ? hasFrozenRows && !options.alwaysAllowHorizontalScroll ? "scroll" : "auto" : (hasFrozenRows && !options.alwaysAllowHorizontalScroll, "auto"),
          "overflow-y": !hasFrozenColumns() && options.alwaysShowVerticalScroll ? "scroll" : hasFrozenColumns() ? "hidden" : hasFrozenRows ? "scroll" : "auto"
        }), $viewportBottomR.css({
          "overflow-x": hasFrozenColumns() ? hasFrozenRows && !options.alwaysAllowHorizontalScroll ? "scroll" : "auto" : (hasFrozenRows && !options.alwaysAllowHorizontalScroll, "auto"),
          "overflow-y": options.alwaysShowVerticalScroll ? "scroll" : (hasFrozenColumns(), "auto")
        }), options.viewportClass && ($viewportTopL.toggleClass(options.viewportClass, !0), $viewportTopR.toggleClass(options.viewportClass, !0), $viewportBottomL.toggleClass(options.viewportClass, !0), $viewportBottomR.toggleClass(options.viewportClass, !0));
      }
      function setScroller() {
        hasFrozenColumns() ? ($headerScrollContainer = $headerScrollerR, $headerRowScrollContainer = $headerRowScrollerR, $footerRowScrollContainer = $footerRowScrollerR, hasFrozenRows ? options.frozenBottom ? ($viewportScrollContainerX = $viewportBottomR, $viewportScrollContainerY = $viewportTopR) : $viewportScrollContainerX = $viewportScrollContainerY = $viewportBottomR : $viewportScrollContainerX = $viewportScrollContainerY = $viewportTopR) : ($headerScrollContainer = $headerScrollerL, $headerRowScrollContainer = $headerRowScrollerL, $footerRowScrollContainer = $footerRowScrollerL, hasFrozenRows ? options.frozenBottom ? ($viewportScrollContainerX = $viewportBottomL, $viewportScrollContainerY = $viewportTopL) : $viewportScrollContainerX = $viewportScrollContainerY = $viewportBottomL : $viewportScrollContainerX = $viewportScrollContainerY = $viewportTopL);
      }
      function measureCellPaddingAndBorder() {
        var O, _ = ["borderLeftWidth", "borderRightWidth", "paddingLeft", "paddingRight"], H = ["borderTopWidth", "borderBottomWidth", "paddingTop", "paddingBottom"], V = $.fn.jquery.split(".");
        jQueryNewWidthBehaviour = V[0] == 1 && V[1] >= 8 || V[0] >= 2, O = $("<div class='ui-state-default slick-header-column' style='visibility:hidden'>-</div>").appendTo($headers), headerColumnWidthDiff = headerColumnHeightDiff = 0, O.css("box-sizing") != "border-box" && O.css("-moz-box-sizing") != "border-box" && O.css("-webkit-box-sizing") != "border-box" && ($.each(_, function(G, J) {
          headerColumnWidthDiff += parseFloat(O.css(J)) || 0;
        }), $.each(H, function(G, J) {
          headerColumnHeightDiff += parseFloat(O.css(J)) || 0;
        })), O.remove();
        var q = $("<div class='slick-row' />").appendTo($canvas);
        O = $("<div class='slick-cell' id='' style='visibility:hidden'>-</div>").appendTo(q), cellWidthDiff = cellHeightDiff = 0, O.css("box-sizing") != "border-box" && O.css("-moz-box-sizing") != "border-box" && O.css("-webkit-box-sizing") != "border-box" && ($.each(_, function(G, J) {
          cellWidthDiff += parseFloat(O.css(J)) || 0;
        }), $.each(H, function(G, J) {
          cellHeightDiff += parseFloat(O.css(J)) || 0;
        })), q.remove(), absoluteColumnMinWidth = Math.max(headerColumnWidthDiff, cellWidthDiff);
      }
      function createCssRules() {
        $style = $("<style type='text/css' rel='stylesheet' />"), $container[0].parentNode instanceof ShadowRoot ? $container[0].parentNode.insertBefore($style[0], $container[0]) : $style.appendTo($("head"));
        for (var O = options.rowHeight - cellHeightDiff, _ = [
          "." + uid + " .slick-group-header-column { left: 1000px; }",
          "." + uid + " .slick-header-column { left: 1000px; }",
          "." + uid + " .slick-top-panel { height:" + options.topPanelHeight + "px; }",
          "." + uid + " .slick-preheader-panel { height:" + options.preHeaderPanelHeight + "px; }",
          "." + uid + " .slick-headerrow-columns { height:" + options.headerRowHeight + "px; }",
          "." + uid + " .slick-footerrow-columns { height:" + options.footerRowHeight + "px; }",
          "." + uid + " .slick-cell { height:" + O + "px; }",
          "." + uid + " .slick-row { height:" + options.rowHeight + "px; }"
        ], H = 0; H < columns.length; H++)
          _.push("." + uid + " .l" + H + " { }"), _.push("." + uid + " .r" + H + " { }");
        $style[0].styleSheet ? $style[0].styleSheet.cssText = _.join(" ") : $style[0].appendChild(document.createTextNode(_.join(" ")));
      }
      function getColumnCssRules(O) {
        var _;
        if (!stylesheet) {
          var H;
          for ($container[0].parentNode instanceof ShadowRoot ? H = $container[0].parentNode.styleSheets : H = document.styleSheets, _ = 0; _ < H.length; _++)
            if ((H[_].ownerNode || H[_].owningElement) == $style[0]) {
              stylesheet = H[_];
              break;
            }
          if (!stylesheet)
            throw new Error("SlickGrid Cannot find stylesheet.");
          columnCssRulesL = [], columnCssRulesR = [];
          var V = stylesheet.cssRules || stylesheet.rules, q, G;
          for (_ = 0; _ < V.length; _++) {
            var J = V[_].selectorText;
            (q = /\.l\d+/.exec(J)) ? (G = parseInt(q[0].substr(2, q[0].length - 2), 10), columnCssRulesL[G] = V[_]) : (q = /\.r\d+/.exec(J)) && (G = parseInt(q[0].substr(2, q[0].length - 2), 10), columnCssRulesR[G] = V[_]);
          }
        }
        return {
          left: columnCssRulesL[O],
          right: columnCssRulesR[O]
        };
      }
      function removeCssRules() {
        $style.remove(), stylesheet = null;
      }
      function destroy(O) {
        getEditorLock().cancelCurrentEdit(), trigger(self.onBeforeDestroy, {});
        for (var _ = plugins.length; _--; )
          unregisterPlugin(plugins[_]);
        options.enableColumnReorder && $headers.filter(":ui-sortable").sortable("destroy"), unbindAncestorScrollEvents(), $container.off(".slickgrid"), removeCssRules(), $canvas.off(), $viewport.off(), $headerScroller.off(), $headerRowScroller.off(), $footerRow && $footerRow.off(), $footerRowScroller && $footerRowScroller.off(), $preHeaderPanelScroller && $preHeaderPanelScroller.off(), $focusSink.off(), $(".slick-resizable-handle").off(), $(".slick-header-column").off(), $container.empty().removeClass(uid), O && destroyAllElements();
      }
      function destroyAllElements() {
        $activeCanvasNode = null, $activeViewportNode = null, $boundAncestors = null, $canvas = null, $canvasTopL = null, $canvasTopR = null, $canvasBottomL = null, $canvasBottomR = null, $container = null, $focusSink = null, $focusSink2 = null, $groupHeadersL = null, $groupHeadersR = null, $headerL = null, $headerR = null, $headers = null, $headerRow = null, $headerRowL = null, $headerRowR = null, $headerRowSpacerL = null, $headerRowSpacerR = null, $headerRowScrollContainer = null, $headerRowScroller = null, $headerRowScrollerL = null, $headerRowScrollerR = null, $headerScrollContainer = null, $headerScroller = null, $headerScrollerL = null, $headerScrollerR = null, $hiddenParents = null, $footerRow = null, $footerRowL = null, $footerRowR = null, $footerRowSpacerL = null, $footerRowSpacerR = null, $footerRowScroller = null, $footerRowScrollerL = null, $footerRowScrollerR = null, $footerRowScrollContainer = null, $preHeaderPanel = null, $preHeaderPanelR = null, $preHeaderPanelScroller = null, $preHeaderPanelScrollerR = null, $preHeaderPanelSpacer = null, $topPanel = null, $topPanelScroller = null, $style = null, $topPanelScrollerL = null, $topPanelScrollerR = null, $topPanelL = null, $topPanelR = null, $paneHeaderL = null, $paneHeaderR = null, $paneTopL = null, $paneTopR = null, $paneBottomL = null, $paneBottomR = null, $viewport = null, $viewportTopL = null, $viewportTopR = null, $viewportBottomL = null, $viewportBottomR = null, $viewportScrollContainerX = null, $viewportScrollContainerY = null;
      }
      var canvas = null, canvas_context = null;
      function autosizeColumn(O, _) {
        var H = O;
        if (typeof O == "number")
          H = columns[O];
        else if (typeof O == "string")
          for (var V = 0; V < columns.length; V++)
            columns[V].Id === O && (H = columns[V]);
        var q = $(getCanvasNode(0, 0));
        getColAutosizeWidth(H, q, _);
      }
      function autosizeColumns(O, _) {
        if (O = O || options.autosizeColsMode, O === Slick.GridAutosizeColsMode.LegacyForceFit || O === Slick.GridAutosizeColsMode.LegacyOff) {
          legacyAutosizeColumns();
          return;
        }
        if (O !== Slick.GridAutosizeColsMode.None) {
          canvas = document.createElement("canvas"), canvas && canvas.getContext && (canvas_context = canvas.getContext("2d"));
          var H = $(getCanvasNode(0, 0)), V = viewportHasVScroll ? viewportW - scrollbarDimensions.width : viewportW, q, G, J, Z, te = 0, ne = 0, ee = 0, ae = 0, oe = 0;
          for (q = 0; q < columns.length; q++)
            G = columns[q], getColAutosizeWidth(G, H, _), oe += G.autoSize.autosizeMode === Slick.ColAutosizeMode.Locked ? G.width : 0, ae += G.autoSize.autosizeMode === Slick.ColAutosizeMode.Locked ? G.width : G.minWidth, te += G.autoSize.widthPx, ne += G.autoSize.sizeToRemaining ? 0 : G.autoSize.widthPx, ee += G.autoSize.sizeToRemaining && G.minWidth || 0;
          var se = te - ne;
          if (O === Slick.GridAutosizeColsMode.FitViewportToCols) {
            var ce = te + scrollbarDimensions.width;
            O = Slick.GridAutosizeColsMode.IgnoreViewport, options.viewportMaxWidthPx && ce > options.viewportMaxWidthPx ? (ce = options.viewportMaxWidthPx, O = Slick.GridAutosizeColsMode.FitColsToViewport) : options.viewportMinWidthPx && ce < options.viewportMinWidthPx && (ce = options.viewportMinWidthPx, O = Slick.GridAutosizeColsMode.FitColsToViewport), $container.width(ce);
          }
          if (O === Slick.GridAutosizeColsMode.FitColsToViewport)
            if (se > 0 && ne < V - ee)
              for (q = 0; q < columns.length; q++) {
                G = columns[q];
                var le = V - ne;
                G.autoSize.sizeToRemaining ? J = le * G.autoSize.widthPx / se : J = G.autoSize.widthPx, G.rerenderOnResize && G.width != J && (Z = !0), G.width = J;
              }
            else if (options.viewportSwitchToScrollModeWidthPercent && ne + ee > V * options.viewportSwitchToScrollModeWidthPercent / 100 || ae > V)
              O = Slick.GridAutosizeColsMode.IgnoreViewport;
            else {
              var ue = ne - oe, de = V - oe - ee;
              for (q = 0; q < columns.length; q++)
                G = columns[q], J = G.width, G.autoSize.autosizeMode !== Slick.ColAutosizeMode.Locked && (G.autoSize.sizeToRemaining ? J = G.minWidth : (J = de / ue * G.autoSize.widthPx, J < G.minWidth && (J = G.minWidth), ue -= G.autoSize.widthPx, de -= J)), G.rerenderOnResize && G.width != J && (Z = !0), G.width = J;
            }
          if (O === Slick.GridAutosizeColsMode.IgnoreViewport)
            for (q = 0; q < columns.length; q++)
              J = columns[q].autoSize.widthPx, columns[q].rerenderOnResize && columns[q].width != J && (Z = !0), columns[q].width = J;
          reRenderColumns(Z);
        }
      }
      function getColAutosizeWidth(O, _, H) {
        var V = O.autoSize;
        if (V.widthPx = O.width, !(V.autosizeMode === Slick.ColAutosizeMode.Locked || V.autosizeMode === Slick.ColAutosizeMode.Guide)) {
          var q = getDataLength();
          if (V.autosizeMode === Slick.ColAutosizeMode.ContentIntelligent) {
            var G = V.colDataTypeOf, J;
            if (q > 0) {
              var Z = getDataItem(0);
              Z && (J = Z[O.field], G = typeof J, G === "object" && (J instanceof Date && (G = "date"), typeof moment < "u" && J instanceof moment && (G = "moment")));
            }
            G === "boolean" && (V.colValueArray = [!0, !1]), G === "number" && (V.valueFilterMode = Slick.ValueFilterMode.GetGreatestAndSub, V.rowSelectionMode = Slick.RowSelectionMode.AllRows), G === "string" && (V.valueFilterMode = Slick.ValueFilterMode.GetLongestText, V.rowSelectionMode = Slick.RowSelectionMode.AllRows, V.allowAddlPercent = 5), G === "date" && (V.colValueArray = [new Date(2009, 8, 30, 12, 20, 20)]), G === "moment" && typeof moment < "u" && (V.colValueArray = [moment([2009, 8, 30, 12, 20, 20])]);
          }
          var te = getColContentSize(O, _, H), ne = V.allowAddlPercent ? 1 + V.allowAddlPercent / 100 : 1;
          te = te * ne + options.autosizeColPaddingPx, O.minWidth && te < O.minWidth && (te = O.minWidth), O.maxWidth && te > O.maxWidth && (te = O.maxWidth), V.widthPx = te;
        }
      }
      function getColContentSize(O, _, H) {
        var V = O.autoSize, q = 1, G, J, Z = 0, te = 0;
        if (V.ignoreHeaderText || (te = getColHeaderWidth(O)), V.colValueArray)
          return Z = getColWidth(O, _, V.colValueArray), Math.max(te, Z);
        var ne = getData();
        ne.getItems && (ne = ne.getItems());
        var ee = (H ? V.rowSelectionModeOnInit : void 0) || V.rowSelectionMode;
        if (ee === Slick.RowSelectionMode.FirstRow && (ne = ne.slice(0, 1)), ee === Slick.RowSelectionMode.LastRow && (ne = ne.slice(ne.length - 1, ne.length)), ee === Slick.RowSelectionMode.FirstNRows && (ne = ne.slice(0, V.rowSelectionCount)), V.valueFilterMode === Slick.ValueFilterMode.DeDuplicate) {
          var ae = {};
          for (G = 0, J = ne.length; G < J; G++)
            ae[ne[G][O.field]] = !0;
          if (Object.keys)
            ne = Object.keys(ae);
          else {
            ne = [];
            for (var G in ae) ne.push(G);
          }
        }
        if (V.valueFilterMode === Slick.ValueFilterMode.GetGreatestAndSub) {
          var oe, se, ce = 0;
          for (G = 0, J = ne.length; G < J; G++)
            oe = ne[G][O.field], Math.abs(oe) > ce && (se = oe, ce = Math.abs(oe));
          se = "" + se, se = Array(se.length + 1).join("9"), se = +se, ne = [se];
        }
        if (V.valueFilterMode === Slick.ValueFilterMode.GetLongestTextAndSub) {
          var oe, le = 0;
          for (G = 0, J = ne.length; G < J; G++)
            oe = ne[G][O.field], (oe || "").length > le && (le = oe.length);
          oe = Array(le + 1).join("m"), q = options.autosizeTextAvgToMWidthRatio, ne = [oe];
        }
        if (V.valueFilterMode === Slick.ValueFilterMode.GetLongestText) {
          var oe, le = 0, ue = 0;
          if (row.length) {
            for (G = 0, J = ne.length; G < J; G++)
              oe = ne[G][O.field], (oe || "").length > le && (le = oe.length, ue = G);
            oe = ne[ue][O.field];
          }
          ne = [oe];
        }
        return Z = getColWidth(O, _, ne) * q, Math.max(te, Z);
      }
      function getColWidth(O, _, H) {
        var V = getColumnIndex(O.id), q = $('<div class="slick-row ui-widget-content"></div>'), G = $('<div class="slick-cell"></div>');
        G.css({
          position: "absolute",
          visibility: "hidden",
          "text-overflow": "initial",
          "white-space": "nowrap"
        }), q.append(G), _.append(q);
        var J, Z = 0, te, ne, ee, ae;
        return canvas_context && O.autoSize.widthEvalMode === Slick.WidthEvalMode.CanvasTextSize ? (canvas_context.font = G.css("font-size") + " " + G.css("font-family"), $(H).each(function(oe, se) {
          ae = Array.isArray(se) ? se[O.field] : se, te = "" + ae, J = te ? canvas_context.measureText(te).width : 0, J > Z && (Z = J, ne = te);
        }), G.html(ne), J = G.outerWidth(), q.remove(), G = null, J) : ($(H).each(function(oe, se) {
          ae = Array.isArray(se) ? se[O.field] : se, O.formatterOverride ? ee = O.formatterOverride(oe, V, ae, O, se, self) : O.formatter ? ee = O.formatter(oe, V, ae, O, se, self) : ee = "" + ae, applyFormatResultToCellNode(ee, G[0]), J = G.outerWidth(), J > Z && (Z = J);
        }), q.remove(), G = null, Z);
      }
      function getColHeaderWidth(O) {
        var _ = 0, H = getUID() + O.id, V = document.getElementById(H), q = H + "_";
        if (V) {
          var G = V.cloneNode(!0);
          G.id = q, G.style.cssText = "position: absolute; visibility: hidden;right: auto;text-overflow: initial;white-space: nowrap;", V.parentNode.insertBefore(G, V), _ = G.offsetWidth, G.parentNode.removeChild(G);
        } else {
          var J = getHeader(O);
          V = $("<div class='ui-state-default slick-header-column' />").html("<span class='slick-column-name'>" + O.name + "</span>").attr("id", q).css({ position: "absolute", visibility: "hidden", right: "auto", "text-overflow:": "initial", "white-space": "nowrap" }).addClass(O.headerCssClass || "").appendTo(J), _ = V[0].offsetWidth, J[0].removeChild(V[0]);
        }
        return _;
      }
      function legacyAutosizeColumns() {
        var O, _, H = [], V = 0, q = 0, G, J = viewportHasVScroll ? viewportW - scrollbarDimensions.width : viewportW;
        for (O = 0; O < columns.length; O++)
          _ = columns[O], H.push(_.width), q += _.width, _.resizable && (V += _.width - Math.max(_.minWidth, absoluteColumnMinWidth));
        for (G = q; q > J && V; ) {
          var Z = (q - J) / V;
          for (O = 0; O < columns.length && q > J; O++) {
            _ = columns[O];
            var te = H[O];
            if (!(!_.resizable || te <= _.minWidth || te <= absoluteColumnMinWidth)) {
              var ne = Math.max(_.minWidth, absoluteColumnMinWidth), ee = Math.floor(Z * (te - ne)) || 1;
              ee = Math.min(ee, te - ne), q -= ee, V -= ee, H[O] -= ee;
            }
          }
          if (G <= q)
            break;
          G = q;
        }
        for (G = q; q < J; ) {
          var ae = J / q;
          for (O = 0; O < columns.length && q < J; O++) {
            _ = columns[O];
            var oe = H[O], se;
            !_.resizable || _.maxWidth <= oe ? se = 0 : se = Math.min(Math.floor(ae * oe) - oe, _.maxWidth - oe || 1e6) || 1, q += se, H[O] += q <= J ? se : 0;
          }
          if (G >= q)
            break;
          G = q;
        }
        var ce = !1;
        for (O = 0; O < columns.length; O++)
          columns[O].rerenderOnResize && columns[O].width != H[O] && (ce = !0), columns[O].width = H[O];
        reRenderColumns(ce);
      }
      function reRenderColumns(O) {
        applyColumnHeaderWidths(), applyColumnGroupHeaderWidths(), updateCanvasWidth(!0), trigger(self.onAutosizeColumns, { columns }), O && (invalidateAllRows(), render());
      }
      function trigger(O, _, H) {
        return H = H || new Slick.EventData(), _ = _ || {}, _.grid = self, O.notify(_, H, self);
      }
      function getEditorLock() {
        return options.editorLock;
      }
      function getEditController() {
        return editController;
      }
      function getColumnIndex(O) {
        return columnsById[O];
      }
      function applyColumnGroupHeaderWidths() {
        if (treeColumns.hasDepth())
          for (var O = $groupHeadersL.length - 1; O >= 0; O--)
            treeColumns.getColumnsInDepth(O), $().add($groupHeadersL[O]).add($groupHeadersR[O]).each(function(_) {
              var H = $(this), V = 0;
              H.width(_ === 0 ? getHeadersWidthL() : getHeadersWidthR()), H.children().each(function() {
                var q = $(this), G = $(this).data("column");
                G.width = 0, G.columns.forEach(function() {
                  var J = H.next().children(":eq(" + V++ + ")");
                  G.width += J.outerWidth();
                }), q.width(G.width - headerColumnWidthDiff);
              });
            });
      }
      function applyColumnHeaderWidths() {
        if (initialized) {
          for (var O, _ = 0, H = $headers.children(), V = columns.length; _ < V; _++)
            O = $(H[_]), jQueryNewWidthBehaviour ? O.outerWidth() !== columns[_].width && O.outerWidth(columns[_].width) : O.width() !== columns[_].width - headerColumnWidthDiff && O.width(columns[_].width - headerColumnWidthDiff);
          updateColumnCaches();
        }
      }
      function applyColumnWidths() {
        for (var O = 0, _, H, V = 0; V < columns.length; V++)
          _ = columns[V].width, H = getColumnCssRules(V), H.left.style.left = O + "px", H.right.style.right = (options.frozenColumn != -1 && V > options.frozenColumn ? canvasWidthR : canvasWidthL) - O - _ + "px", options.frozenColumn == V ? O = 0 : O += columns[V].width;
      }
      function setSortColumn(O, _) {
        setSortColumns([{ columnId: O, sortAsc: _ }]);
      }
      function setSortColumns(O) {
        sortColumns = O;
        var _ = options.numberedMultiColumnSort && sortColumns.length > 1, H = $headers.children();
        H.removeClass("slick-header-column-sorted").find(".slick-sort-indicator").removeClass("slick-sort-indicator-asc slick-sort-indicator-desc"), H.find(".slick-sort-indicator-numbered").text(""), $.each(sortColumns, function(V, q) {
          q.sortAsc == null && (q.sortAsc = !0);
          var G = getColumnIndex(q.columnId);
          G != null && (H.eq(G).addClass("slick-header-column-sorted").find(".slick-sort-indicator").addClass(q.sortAsc ? "slick-sort-indicator-asc" : "slick-sort-indicator-desc"), _ && H.eq(G).find(".slick-sort-indicator-numbered").text(V + 1));
        });
      }
      function getSortColumns() {
        return sortColumns;
      }
      function handleSelectedRangesChanged(O, _) {
        var H = selectedRows.slice(0);
        selectedRows = [];
        for (var V = {}, q = 0; q < _.length; q++)
          for (var G = _[q].fromRow; G <= _[q].toRow; G++) {
            V[G] || (selectedRows.push(G), V[G] = {});
            for (var J = _[q].fromCell; J <= _[q].toCell; J++)
              canCellBeSelected(G, J) && (V[G][columns[J].id] = options.selectedCellCssClass);
          }
        setCellCssStyles(options.selectedCellCssClass, V), simpleArrayEquals(H, selectedRows) || trigger(self.onSelectedRowsChanged, { rows: getSelectedRows(), previousSelectedRows: H }, O);
      }
      function simpleArrayEquals(O, _) {
        if (!Array.isArray(O) || !Array.isArray(_))
          return !0;
        const H = new Set(O), V = new Set(_);
        if (H.size != V.size)
          return !1;
        for (const q of H)
          if (!V.has(q))
            return !1;
        return !0;
      }
      function getColumns() {
        return columns;
      }
      function updateColumnCaches() {
        columnPosLeft = [], columnPosRight = [];
        for (var O = 0, _ = 0, H = columns.length; _ < H; _++)
          columnPosLeft[_] = O, columnPosRight[_] = O + columns[_].width, options.frozenColumn == _ ? O = 0 : O += columns[_].width;
      }
      function updateColumnProps() {
        columnsById = {};
        for (var O = 0; O < columns.length; O++) {
          columns[O].width && (columns[O].widthRequest = columns[O].width);
          var _ = columns[O] = $.extend({}, columnDefaults, columns[O]);
          _.autoSize = $.extend({}, columnAutosizeDefaults, _.autoSize), columnsById[_.id] = O, _.minWidth && _.width < _.minWidth && (_.width = _.minWidth), _.maxWidth && _.width > _.maxWidth && (_.width = _.maxWidth), _.resizable;
        }
      }
      function setColumns(O) {
        trigger(self.onBeforeSetColumns, { previousColumns: columns, newColumns: O, grid: self });
        var _ = new Slick.TreeColumns(O);
        _.hasDepth() ? (treeColumns = _, columns = treeColumns.extractColumns()) : columns = O, updateColumnProps(), updateColumnCaches(), initialized && (setPaneVisibility(), setOverflow(), invalidateAllRows(), createColumnHeaders(), createColumnGroupHeaders(), createColumnFooter(), removeCssRules(), createCssRules(), resizeCanvas(), updateCanvasWidth(), applyColumnHeaderWidths(), applyColumnWidths(), handleScroll());
      }
      function getOptions() {
        return options;
      }
      function setOptions(O, _, H, V) {
        if (getEditorLock().commitCurrentEdit()) {
          makeActiveCellNormal(), O.showColumnHeader !== void 0 && setColumnHeaderVisibility(O.showColumnHeader), options.enableAddRow !== O.enableAddRow && invalidateRow(getDataLength());
          var q = $.extend(!0, {}, options);
          if (options = $.extend(options, O), trigger(self.onSetOptions, { optionsBefore: q, optionsAfter: options }), validateAndEnforceOptions(), $viewport.css("overflow-y", options.autoHeight ? "hidden" : "auto"), _ || render(), setFrozenOptions(), setScroller(), V || setOverflow(), H || setColumns(treeColumns.extractColumns()), options.enableMouseWheelScrollHandler && $viewport && jQuery.fn.mousewheel) {
            var G = $._data($viewport[0], "events");
            (!G || !G.mousewheel) && $viewport.on("mousewheel", handleMouseWheel);
          } else options.enableMouseWheelScrollHandler === !1 && $viewport.off("mousewheel");
        }
      }
      function validateAndEnforceOptions() {
        options.autoHeight && (options.leaveSpaceForNewRows = !1), options.forceFitColumns && (options.autosizeColsMode = Slick.GridAutosizeColsMode.LegacyForceFit, console.log("forceFitColumns option is deprecated - use autosizeColsMode"));
      }
      function setData(O, _) {
        data = O, invalidateAllRows(), updateRowCount(), _ && scrollTo(0);
      }
      function getData() {
        return data;
      }
      function getDataLength() {
        return data.getLength ? data.getLength() : data && data.length || 0;
      }
      function getDataLengthIncludingAddNew() {
        return getDataLength() + (options.enableAddRow && (!pagingActive || pagingIsLastPage) ? 1 : 0);
      }
      function getDataItem(O) {
        return data.getItem ? data.getItem(O) : data[O];
      }
      function getTopPanel() {
        return $topPanel[0];
      }
      function setTopPanelVisibility(O, _) {
        var H = _ !== !1;
        options.showTopPanel != O && (options.showTopPanel = O, O ? H ? $topPanelScroller.slideDown("fast", resizeCanvas) : ($topPanelScroller.show(), resizeCanvas()) : H ? $topPanelScroller.slideUp("fast", resizeCanvas) : ($topPanelScroller.hide(), resizeCanvas()));
      }
      function setHeaderRowVisibility(O, _) {
        var H = _ !== !1;
        options.showHeaderRow != O && (options.showHeaderRow = O, O ? H ? $headerRowScroller.slideDown("fast", resizeCanvas) : ($headerRowScroller.show(), resizeCanvas()) : H ? $headerRowScroller.slideUp("fast", resizeCanvas) : ($headerRowScroller.hide(), resizeCanvas()));
      }
      function setColumnHeaderVisibility(O, _) {
        options.showColumnHeader != O && (options.showColumnHeader = O, O ? _ ? $headerScroller.slideDown("fast", resizeCanvas) : ($headerScroller.show(), resizeCanvas()) : _ ? $headerScroller.slideUp("fast", resizeCanvas) : ($headerScroller.hide(), resizeCanvas()));
      }
      function setFooterRowVisibility(O, _) {
        var H = _ !== !1;
        options.showFooterRow != O && (options.showFooterRow = O, O ? H ? $footerRowScroller.slideDown("fast", resizeCanvas) : ($footerRowScroller.show(), resizeCanvas()) : H ? $footerRowScroller.slideUp("fast", resizeCanvas) : ($footerRowScroller.hide(), resizeCanvas()));
      }
      function setPreHeaderPanelVisibility(O, _) {
        var H = _ !== !1;
        options.showPreHeaderPanel != O && (options.showPreHeaderPanel = O, O ? H ? ($preHeaderPanelScroller.slideDown("fast", resizeCanvas), $preHeaderPanelScrollerR.slideDown("fast", resizeCanvas)) : ($preHeaderPanelScroller.show(), $preHeaderPanelScrollerR.show(), resizeCanvas()) : H ? ($preHeaderPanelScroller.slideUp("fast", resizeCanvas), $preHeaderPanelScrollerR.slideUp("fast", resizeCanvas)) : ($preHeaderPanelScroller.hide(), $preHeaderPanelScrollerR.hide(), resizeCanvas()));
      }
      function getContainerNode() {
        return $container.get(0);
      }
      function getRowTop(O) {
        return options.rowHeight * O - offset;
      }
      function getRowFromPosition(O) {
        return Math.floor((O + offset) / options.rowHeight);
      }
      function scrollTo(O) {
        O = Math.max(O, 0), O = Math.min(O, th - $viewportScrollContainerY.height() + (viewportHasHScroll || hasFrozenColumns() ? scrollbarDimensions.height : 0));
        var _ = offset;
        page = Math.min(n - 1, Math.floor(O / ph)), offset = Math.round(page * cj);
        var H = O - offset;
        if (offset != _) {
          var V = getVisibleRange(H);
          cleanupRows(V), updateRowPositions();
        }
        prevScrollTop != H && (vScrollDir = prevScrollTop + _ < H + offset ? 1 : -1, lastRenderedScrollTop = scrollTop = prevScrollTop = H, hasFrozenColumns() && ($viewportTopL[0].scrollTop = H), hasFrozenRows && ($viewportBottomL[0].scrollTop = $viewportBottomR[0].scrollTop = H), $viewportScrollContainerY[0].scrollTop = H, trigger(self.onViewportChanged, {}));
      }
      function defaultFormatter(O, _, H, V, q, G) {
        return H == null ? "" : (H + "").replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
      }
      function getFormatter(O, _) {
        var H = data.getItemMetadata && data.getItemMetadata(O), V = H && H.columns && (H.columns[_.id] || H.columns[getColumnIndex(_.id)]);
        return V && V.formatter || H && H.formatter || _.formatter || options.formatterFactory && options.formatterFactory.getFormatter(_) || options.defaultFormatter;
      }
      function getEditor(O, _) {
        var H = columns[_], V = data.getItemMetadata && data.getItemMetadata(O), q = V && V.columns;
        return q && q[H.id] && q[H.id].editor !== void 0 ? q[H.id].editor : q && q[_] && q[_].editor !== void 0 ? q[_].editor : H.editor || options.editorFactory && options.editorFactory.getEditor(H);
      }
      function getDataItemValueForColumn(O, _) {
        return options.dataItemColumnValueExtractor ? options.dataItemColumnValueExtractor(O, _) : O[_.field];
      }
      function appendRowHtml(O, _, H, V, q) {
        var G = getDataItem(H), J = H < q && !G, Z = "slick-row" + (hasFrozenRows && H <= options.frozenRow ? " frozen" : "") + (J ? " loading" : "") + (H === activeRow && options.showCellSelection ? " active" : "") + (H % 2 == 1 ? " odd" : " even");
        G || (Z += " " + options.addNewRowCssClass);
        var te = data.getItemMetadata && data.getItemMetadata(H);
        te && te.cssClasses && (Z += " " + te.cssClasses);
        var ne = getFrozenRowOffset(H), ee = "<div class='ui-widget-content " + Z + "' style='top:" + (getRowTop(H) - ne) + "px'>";
        O.push(ee), hasFrozenColumns() && _.push(ee);
        for (var ae, oe, se = 0, ce = columns.length; se < ce; se++) {
          if (oe = columns[se], ae = 1, te && te.columns) {
            var le = te.columns[oe.id] || te.columns[se];
            ae = le && le.colspan || 1, ae === "*" && (ae = ce - se);
          }
          if (columnPosRight[Math.min(ce - 1, se + ae - 1)] > V.leftPx) {
            if (!oe.alwaysRenderColumn && columnPosLeft[se] > V.rightPx)
              break;
            hasFrozenColumns() && se > options.frozenColumn ? appendCellHtml(_, H, se, ae, G) : appendCellHtml(O, H, se, ae, G);
          } else (oe.alwaysRenderColumn || hasFrozenColumns() && se <= options.frozenColumn) && appendCellHtml(O, H, se, ae, G);
          ae > 1 && (se += ae - 1);
        }
        O.push("</div>"), hasFrozenColumns() && _.push("</div>");
      }
      function appendCellHtml(O, _, H, V, q) {
        var G = columns[H], J = "slick-cell l" + H + " r" + Math.min(columns.length - 1, H + V - 1) + (G.cssClass ? " " + G.cssClass : "");
        hasFrozenColumns() && H <= options.frozenColumn && (J += " frozen"), _ === activeRow && H === activeCell && options.showCellSelection && (J += " active");
        for (var Z in cellCssClasses)
          cellCssClasses[Z][_] && cellCssClasses[Z][_][G.id] && (J += " " + cellCssClasses[Z][_][G.id]);
        var te = null, ne = "";
        q && (te = getDataItemValueForColumn(q, G), ne = getFormatter(_, G)(_, H, te, G, q, self), ne == null && (ne = ""));
        var ee = trigger(self.onBeforeAppendCell, { row: _, cell: H, value: te, dataContext: q }) || "";
        ee += ne && ne.addClasses ? (ee ? " " : "") + ne.addClasses : "";
        var ae = ne && ne.toolTip ? "title='" + ne.toolTip + "'" : "", oe = "";
        if (G.hasOwnProperty("cellAttrs") && G.cellAttrs instanceof Object)
          for (var Z in G.cellAttrs)
            G.cellAttrs.hasOwnProperty(Z) && (oe += " " + Z + '="' + G.cellAttrs[Z] + '" ');
        O.push("<div class='" + J + (ee ? " " + ee : "") + "' " + ae + oe + ">"), q && O.push(Object.prototype.toString.call(ne) !== "[object Object]" ? ne : ne.text), O.push("</div>"), rowsCache[_].cellRenderQueue.push(H), rowsCache[_].cellColSpans[H] = V;
      }
      function cleanupRows(O) {
        for (var _ in rowsCache) {
          var H = !0;
          hasFrozenRows && (options.frozenBottom && _ >= actualFrozenRow || !options.frozenBottom && _ <= actualFrozenRow) && (H = !1), (_ = parseInt(_, 10)) !== activeRow && (_ < O.top || _ > O.bottom) && H && removeRowFromCache(_);
        }
        options.enableAsyncPostRenderCleanup && startPostProcessingCleanup();
      }
      function invalidate() {
        updateRowCount(), invalidateAllRows(), render();
      }
      function invalidateAllRows() {
        currentEditor && makeActiveCellNormal();
        for (var O in rowsCache)
          removeRowFromCache(O);
        options.enableAsyncPostRenderCleanup && startPostProcessingCleanup();
      }
      function queuePostProcessedRowForCleanup(O, _, H) {
        postProcessgroupId++;
        for (var V in _)
          _.hasOwnProperty(V) && postProcessedCleanupQueue.push({
            actionType: "C",
            groupId: postProcessgroupId,
            node: O.cellNodesByColumnIdx[V | 0],
            columnIdx: V | 0,
            rowIdx: H
          });
        postProcessedCleanupQueue.push({
          actionType: "R",
          groupId: postProcessgroupId,
          node: O.rowNode
        }), O.rowNode.detach();
      }
      function queuePostProcessedCellForCleanup(O, _, H) {
        postProcessedCleanupQueue.push({
          actionType: "C",
          groupId: postProcessgroupId,
          node: O,
          columnIdx: _,
          rowIdx: H
        }), $(O).detach();
      }
      function removeRowFromCache(O) {
        var _ = rowsCache[O];
        _ && (options.enableAsyncPostRenderCleanup && postProcessedRows[O] ? queuePostProcessedRowForCleanup(_, postProcessedRows[O], O) : _.rowNode.each(function() {
          this.parentElement.removeChild(this);
        }), delete rowsCache[O], delete postProcessedRows[O], renderedRows--, counter_rows_removed++);
      }
      function invalidateRows(O) {
        var _, H;
        if (!(!O || !O.length)) {
          for (vScrollDir = 0, H = O.length, _ = 0; _ < H; _++)
            currentEditor && activeRow === O[_] && makeActiveCellNormal(), rowsCache[O[_]] && removeRowFromCache(O[_]);
          options.enableAsyncPostRenderCleanup && startPostProcessingCleanup();
        }
      }
      function invalidateRow(O) {
        !O && O !== 0 || invalidateRows([O]);
      }
      function applyFormatResultToCellNode(O, _, H) {
        if (O == null && (O = ""), Object.prototype.toString.call(O) !== "[object Object]") {
          _.innerHTML = O;
          return;
        }
        _.innerHTML = O.text, O.removeClasses && !H && $(_).removeClass(O.removeClasses), O.addClasses && $(_).addClass(O.addClasses), O.toolTip && $(_).attr("title", O.toolTip);
      }
      function updateCell(O, _) {
        var H = getCellNode(O, _);
        if (H) {
          var V = columns[_], q = getDataItem(O);
          if (currentEditor && activeRow === O && activeCell === _)
            currentEditor.loadValue(q);
          else {
            var G = q ? getFormatter(O, V)(O, _, getDataItemValueForColumn(q, V), V, q, self) : "";
            applyFormatResultToCellNode(G, H), invalidatePostProcessingResults(O);
          }
        }
      }
      function updateRow(O) {
        var _ = rowsCache[O];
        if (_) {
          ensureCellNodesInRowsCache(O);
          var H, V = getDataItem(O);
          for (var q in _.cellNodesByColumnIdx)
            if (_.cellNodesByColumnIdx.hasOwnProperty(q)) {
              q = q | 0;
              var G = columns[q], J = _.cellNodesByColumnIdx[q][0];
              O === activeRow && q === activeCell && currentEditor ? currentEditor.loadValue(V) : V ? (H = getFormatter(O, G)(O, q, getDataItemValueForColumn(V, G), G, V, self), applyFormatResultToCellNode(H, J)) : J.innerHTML = "";
            }
          invalidatePostProcessingResults(O);
        }
      }
      function getViewportHeight() {
        if ((!options.autoHeight || options.frozenColumn != -1) && (topPanelH = options.showTopPanel ? options.topPanelHeight + getVBoxDelta($topPanelScroller) : 0, headerRowH = options.showHeaderRow ? options.headerRowHeight + getVBoxDelta($headerRowScroller) : 0, footerRowH = options.showFooterRow ? options.footerRowHeight + getVBoxDelta($footerRowScroller) : 0), options.autoHeight) {
          var O = $paneHeaderL.outerHeight();
          O += options.showHeaderRow ? options.headerRowHeight + getVBoxDelta($headerRowScroller) : 0, O += options.showFooterRow ? options.footerRowHeight + getVBoxDelta($footerRowScroller) : 0, O += getCanvasWidth() > viewportW ? scrollbarDimensions.height : 0, viewportH = options.rowHeight * getDataLengthIncludingAddNew() + (options.frozenColumn == -1 ? O : 0);
        } else {
          var _ = options.showColumnHeader ? parseFloat($.css($headerScroller[0], "height")) + getVBoxDelta($headerScroller) : 0, H = options.createPreHeaderPanel && options.showPreHeaderPanel ? options.preHeaderPanelHeight + getVBoxDelta($preHeaderPanelScroller) : 0;
          viewportH = parseFloat($.css($container[0], "height", !0)) - parseFloat($.css($container[0], "paddingTop", !0)) - parseFloat($.css($container[0], "paddingBottom", !0)) - _ - topPanelH - headerRowH - footerRowH - H;
        }
        return numVisibleRows = Math.ceil(viewportH / options.rowHeight), viewportH;
      }
      function getViewportWidth() {
        viewportW = parseFloat($container.width());
      }
      function resizeCanvas() {
        if (initialized) {
          paneTopH = 0, paneBottomH = 0, viewportTopH = 0, getViewportWidth(), getViewportHeight(), hasFrozenRows ? options.frozenBottom ? (paneTopH = viewportH - frozenRowsHeight - scrollbarDimensions.height, paneBottomH = frozenRowsHeight + scrollbarDimensions.height) : (paneTopH = frozenRowsHeight, paneBottomH = viewportH - frozenRowsHeight) : paneTopH = viewportH, paneTopH += topPanelH + headerRowH + footerRowH, hasFrozenColumns() && options.autoHeight && (paneTopH += scrollbarDimensions.height), viewportTopH = paneTopH - topPanelH - headerRowH - footerRowH, options.autoHeight && (hasFrozenColumns() && $container.height(
            paneTopH + parseFloat($.css($headerScrollerL[0], "height"))
          ), $paneTopL.css("position", "relative")), $paneTopL.css({
            top: $paneHeaderL.height() || (options.showHeaderRow ? options.headerRowHeight : 0) + (options.showPreHeaderPanel ? options.preHeaderPanelHeight : 0),
            height: paneTopH
          });
          var O = $paneTopL.position().top + paneTopH;
          options.autoHeight || $viewportTopL.height(viewportTopH), hasFrozenColumns() ? ($paneTopR.css({
            top: $paneHeaderL.height(),
            height: paneTopH
          }), $viewportTopR.height(viewportTopH), hasFrozenRows && ($paneBottomL.css({
            top: O,
            height: paneBottomH
          }), $paneBottomR.css({
            top: O,
            height: paneBottomH
          }), $viewportBottomR.height(paneBottomH))) : hasFrozenRows && ($paneBottomL.css({
            width: "100%",
            height: paneBottomH
          }), $paneBottomL.css("top", O)), hasFrozenRows ? ($viewportBottomL.height(paneBottomH), options.frozenBottom ? ($canvasBottomL.height(frozenRowsHeight), hasFrozenColumns() && $canvasBottomR.height(frozenRowsHeight)) : ($canvasTopL.height(frozenRowsHeight), hasFrozenColumns() && $canvasTopR.height(frozenRowsHeight))) : $viewportTopR.height(viewportTopH), (!scrollbarDimensions || !scrollbarDimensions.width) && (scrollbarDimensions = measureScrollbar()), options.autosizeColsMode === Slick.GridAutosizeColsMode.LegacyForceFit && autosizeColumns(), updateRowCount(), handleScroll(), lastRenderedScrollLeft = -1, render();
        }
      }
      function updatePagingStatusFromView(O) {
        pagingActive = O.pageSize !== 0, pagingIsLastPage = O.pageNum == O.totalPages - 1;
      }
      function updateRowCount() {
        if (initialized) {
          var O = getDataLength(), _ = getDataLengthIncludingAddNew(), H = 0, Z = hasFrozenRows && !options.frozenBottom ? $canvasBottomL.height() : $canvasTopL.height();
          if (hasFrozenRows)
            var H = getDataLength() - options.frozenRow;
          else
            var H = _ + (options.leaveSpaceForNewRows ? numVisibleRows - 1 : 0);
          var V = $viewportScrollContainerY.height(), q = viewportHasVScroll;
          viewportHasVScroll = options.alwaysShowVerticalScroll || !options.autoHeight && H * options.rowHeight > V, makeActiveCellNormal();
          var G = O - 1;
          for (var J in rowsCache)
            J > G && removeRowFromCache(J);
          options.enableAsyncPostRenderCleanup && startPostProcessingCleanup(), activeCellNode && activeRow > G && resetActiveCell();
          var Z = h;
          options.autoHeight ? h = options.rowHeight * H : (th = Math.max(options.rowHeight * H, V - scrollbarDimensions.height), th < maxSupportedCssHeight ? (h = ph = th, n = 1, cj = 0) : (h = maxSupportedCssHeight, ph = h / 100, n = Math.floor(th / ph), cj = (th - h) / (n - 1))), h !== Z && (hasFrozenRows && !options.frozenBottom ? ($canvasBottomL.css("height", h), hasFrozenColumns() && $canvasBottomR.css("height", h)) : ($canvasTopL.css("height", h), $canvasTopR.css("height", h)), scrollTop = $viewportScrollContainerY[0].scrollTop);
          var te = scrollTop + offset <= th - V;
          th == 0 || scrollTop == 0 ? page = offset = 0 : scrollTo(te ? scrollTop + offset : th - V), h != Z && options.autoHeight && resizeCanvas(), options.autosizeColsMode === Slick.GridAutosizeColsMode.LegacyForceFit && q != viewportHasVScroll && autosizeColumns(), updateCanvasWidth(!1);
        }
      }
      function getVisibleRange(O, _) {
        return O == null && (O = scrollTop), _ == null && (_ = scrollLeft), {
          top: getRowFromPosition(O),
          bottom: getRowFromPosition(O + viewportH) + 1,
          leftPx: _,
          rightPx: _ + viewportW
        };
      }
      function getRenderedRange(O, _) {
        var H = getVisibleRange(O, _), V = Math.round(viewportH / options.rowHeight), q = options.minRowBuffer;
        return vScrollDir == -1 ? (H.top -= V, H.bottom += q) : vScrollDir == 1 ? (H.top -= q, H.bottom += V) : (H.top -= q, H.bottom += q), H.top = Math.max(0, H.top), H.bottom = Math.min(getDataLengthIncludingAddNew() - 1, H.bottom), H.leftPx -= viewportW, H.rightPx += viewportW, H.leftPx = Math.max(0, H.leftPx), H.rightPx = Math.min(canvasWidth, H.rightPx), H;
      }
      function ensureCellNodesInRowsCache(O) {
        var _ = rowsCache[O];
        if (_ && _.cellRenderQueue.length)
          for (var H = _.rowNode.children().last(); _.cellRenderQueue.length; ) {
            var V = _.cellRenderQueue.pop();
            _.cellNodesByColumnIdx[V] = H, H = H.prev(), H.length === 0 && (H = $(_.rowNode[0]).children().last());
          }
      }
      function cleanUpCells(O, _) {
        if (!(hasFrozenRows && (options.frozenBottom && _ > actualFrozenRow || _ <= actualFrozenRow))) {
          var H = rowsCache[_], V = [];
          for (var q in H.cellNodesByColumnIdx)
            if (H.cellNodesByColumnIdx.hasOwnProperty(q) && (q = q | 0, !(q <= options.frozenColumn) && !(Array.isArray(columns) && columns[q] && columns[q].alwaysRenderColumn))) {
              var G = H.cellColSpans[q];
              (columnPosLeft[q] > O.rightPx || columnPosRight[Math.min(columns.length - 1, q + G - 1)] < O.leftPx) && (_ == activeRow && q == activeCell || V.push(q));
            }
          for (var J, Z; (J = V.pop()) != null; )
            Z = H.cellNodesByColumnIdx[J][0], options.enableAsyncPostRenderCleanup && postProcessedRows[_] && postProcessedRows[_][J] ? queuePostProcessedCellForCleanup(Z, J, _) : Z.parentElement.removeChild(Z), delete H.cellColSpans[J], delete H.cellNodesByColumnIdx[J], postProcessedRows[_] && delete postProcessedRows[_][J];
        }
      }
      function cleanUpAndRenderCells(O) {
        for (var _, H = [], V = [], q, G, J = O.top, Z = O.bottom; J <= Z; J++)
          if (_ = rowsCache[J], !!_) {
            ensureCellNodesInRowsCache(J), cleanUpCells(O, J), q = 0;
            var te = data.getItemMetadata && data.getItemMetadata(J);
            te = te && te.columns;
            for (var ne = getDataItem(J), ee = 0, ae = columns.length; ee < ae && !(columnPosLeft[ee] > O.rightPx); ee++) {
              if ((G = _.cellColSpans[ee]) != null) {
                ee += G > 1 ? G - 1 : 0;
                continue;
              }
              if (G = 1, te) {
                var oe = te[columns[ee].id] || te[ee];
                G = oe && oe.colspan || 1, G === "*" && (G = ae - ee);
              }
              columnPosRight[Math.min(ae - 1, ee + G - 1)] > O.leftPx && (appendCellHtml(H, J, ee, G, ne), q++), ee += G > 1 ? G - 1 : 0;
            }
            q && V.push(J);
          }
        if (H.length) {
          var se = document.createElement("div");
          se.innerHTML = H.join("");
          for (var ce, le; (ce = V.pop()) != null; ) {
            _ = rowsCache[ce];
            for (var ue; (ue = _.cellRenderQueue.pop()) != null; )
              le = se.lastChild, hasFrozenColumns() && ue > options.frozenColumn ? _.rowNode[1].appendChild(le) : _.rowNode[0].appendChild(le), _.cellNodesByColumnIdx[ue] = $(le);
          }
        }
      }
      function renderRows(O) {
        for (var _ = [], H = [], V = [], q = !1, G = getDataLength(), J = O.top, Z = O.bottom; J <= Z; J++)
          rowsCache[J] || hasFrozenRows && options.frozenBottom && J == getDataLength() || (renderedRows++, V.push(J), rowsCache[J] = {
            rowNode: null,
            // ColSpans of rendered cells (by column idx).
            // Can also be used for checking whether a cell has been rendered.
            cellColSpans: [],
            // Cell nodes (by column idx).  Lazy-populated by ensureCellNodesInRowsCache().
            cellNodesByColumnIdx: [],
            // Column indices of cell nodes that have been rendered, but not yet indexed in
            // cellNodesByColumnIdx.  These are in the same order as cell nodes added at the
            // end of the row.
            cellRenderQueue: []
          }, appendRowHtml(_, H, J, O, G), activeCellNode && activeRow === J && (q = !0), counter_rows_rendered++);
        if (V.length) {
          var te = document.createElement("div"), ne = document.createElement("div");
          te.innerHTML = _.join(""), ne.innerHTML = H.join("");
          for (var J = 0, Z = V.length; J < Z; J++)
            hasFrozenRows && V[J] >= actualFrozenRow ? hasFrozenColumns() ? (rowsCache[V[J]].rowNode = $().add($(te.firstChild)).add($(ne.firstChild)), $canvasBottomL[0].append(te.firstChild), $canvasBottomR[0].append(ne.firstChild)) : (rowsCache[V[J]].rowNode = $().add($(te.firstChild)), $canvasBottomL[0].append($(te.firstChild))) : hasFrozenColumns() ? (rowsCache[V[J]].rowNode = $().add($(te.firstChild)).add($(ne.firstChild)), $canvasTopL[0].append(te.firstChild), $canvasTopR[0].append(ne.firstChild)) : (rowsCache[V[J]].rowNode = $().add($(te.firstChild)), $canvasTopL[0].append(te.firstChild));
          q && (activeCellNode = getCellNode(activeRow, activeCell));
        }
      }
      function startPostProcessing() {
        options.enableAsyncPostRender && (clearTimeout(h_postrender), h_postrender = setTimeout(asyncPostProcessRows, options.asyncPostRenderDelay));
      }
      function startPostProcessingCleanup() {
        options.enableAsyncPostRenderCleanup && (clearTimeout(h_postrenderCleanup), h_postrenderCleanup = setTimeout(asyncPostProcessCleanupRows, options.asyncPostRenderCleanupDelay));
      }
      function invalidatePostProcessingResults(O) {
        for (var _ in postProcessedRows[O])
          postProcessedRows[O].hasOwnProperty(_) && (postProcessedRows[O][_] = "C");
        postProcessFromRow = Math.min(postProcessFromRow, O), postProcessToRow = Math.max(postProcessToRow, O), startPostProcessing();
      }
      function updateRowPositions() {
        for (var O in rowsCache) {
          var _ = O ? parseInt(O) : 0;
          rowsCache[_].rowNode[0].style.top = getRowTop(_) + "px";
        }
      }
      function render() {
        if (initialized) {
          scrollThrottle.dequeue();
          var O = getVisibleRange(), _ = getRenderedRange();
          if (cleanupRows(_), lastRenderedScrollLeft != scrollLeft) {
            if (hasFrozenRows) {
              var H = $.extend(!0, {}, _);
              options.frozenBottom ? (H.top = actualFrozenRow, H.bottom = getDataLength()) : (H.top = 0, H.bottom = options.frozenRow), cleanUpAndRenderCells(H);
            }
            cleanUpAndRenderCells(_);
          }
          renderRows(_), hasFrozenRows && (options.frozenBottom ? renderRows({
            top: actualFrozenRow,
            bottom: getDataLength() - 1,
            leftPx: _.leftPx,
            rightPx: _.rightPx
          }) : renderRows({
            top: 0,
            bottom: options.frozenRow - 1,
            leftPx: _.leftPx,
            rightPx: _.rightPx
          })), postProcessFromRow = O.top, postProcessToRow = Math.min(getDataLengthIncludingAddNew() - 1, O.bottom), startPostProcessing(), lastRenderedScrollTop = scrollTop, lastRenderedScrollLeft = scrollLeft, trigger(self.onRendered, { startRow: O.top, endRow: O.bottom, grid: self });
        }
      }
      function handleHeaderRowScroll() {
        var O = $headerRowScrollContainer[0].scrollLeft;
        O != $viewportScrollContainerX[0].scrollLeft && ($viewportScrollContainerX[0].scrollLeft = O);
      }
      function handleFooterRowScroll() {
        var O = $footerRowScrollContainer[0].scrollLeft;
        O != $viewportScrollContainerX[0].scrollLeft && ($viewportScrollContainerX[0].scrollLeft = O);
      }
      function handlePreHeaderPanelScroll() {
        handleElementScroll($preHeaderPanelScroller[0]);
      }
      function handleElementScroll(O) {
        var _ = O.scrollLeft;
        _ != $viewportScrollContainerX[0].scrollLeft && ($viewportScrollContainerX[0].scrollLeft = _);
      }
      function handleScroll() {
        return scrollTop = $viewportScrollContainerY[0].scrollTop, scrollLeft = $viewportScrollContainerX[0].scrollLeft, _handleScroll(!1);
      }
      function _handleScroll(O) {
        var _ = $viewportScrollContainerY[0].scrollHeight - $viewportScrollContainerY[0].clientHeight, H = $viewportScrollContainerY[0].scrollWidth - $viewportScrollContainerY[0].clientWidth;
        _ = Math.max(0, _), H = Math.max(0, H), scrollTop > _ && (scrollTop = _), scrollLeft > H && (scrollLeft = H);
        var V = Math.abs(scrollTop - prevScrollTop), q = Math.abs(scrollLeft - prevScrollLeft);
        if (q && (prevScrollLeft = scrollLeft, $viewportScrollContainerX[0].scrollLeft = scrollLeft, $headerScrollContainer[0].scrollLeft = scrollLeft, $topPanelScroller[0].scrollLeft = scrollLeft, $headerRowScrollContainer[0].scrollLeft = scrollLeft, options.createFooterRow && ($footerRowScrollContainer[0].scrollLeft = scrollLeft), options.createPreHeaderPanel && (hasFrozenColumns() ? $preHeaderPanelScrollerR[0].scrollLeft = scrollLeft : $preHeaderPanelScroller[0].scrollLeft = scrollLeft), hasFrozenColumns() ? hasFrozenRows && ($viewportTopR[0].scrollLeft = scrollLeft) : hasFrozenRows && ($viewportTopL[0].scrollLeft = scrollLeft)), V)
          if (vScrollDir = prevScrollTop < scrollTop ? 1 : -1, prevScrollTop = scrollTop, O && ($viewportScrollContainerY[0].scrollTop = scrollTop), hasFrozenColumns() && (hasFrozenRows && !options.frozenBottom ? $viewportBottomL[0].scrollTop = scrollTop : $viewportTopL[0].scrollTop = scrollTop), V < viewportH)
            scrollTo(scrollTop + offset);
          else {
            var G = offset;
            h == viewportH ? page = 0 : page = Math.min(n - 1, Math.floor(scrollTop * ((th - viewportH) / (h - viewportH)) * (1 / ph))), offset = Math.round(page * cj), G != offset && invalidateAllRows();
          }
        if (q || V) {
          var J = Math.abs(lastRenderedScrollLeft - scrollLeft), Z = Math.abs(lastRenderedScrollTop - scrollTop);
          (J > 20 || Z > 20) && (options.forceSyncScrolling || Z < viewportH && J < viewportW ? render() : scrollThrottle.enqueue(), trigger(self.onViewportChanged, {}));
        }
        return trigger(self.onScroll, { scrollLeft, scrollTop }), !!(q || V);
      }
      function ActionThrottle(O, _) {
        var H = !1, V = !1;
        function q() {
          H ? V = !0 : J();
        }
        function G() {
          V = !1;
        }
        function J() {
          H = !0, setTimeout(Z, _), O();
        }
        function Z() {
          V ? (G(), J()) : H = !1;
        }
        return {
          enqueue: q,
          dequeue: G
        };
      }
      function asyncPostProcessRows() {
        for (var O = getDataLength(); postProcessFromRow <= postProcessToRow; ) {
          var _ = vScrollDir >= 0 ? postProcessFromRow++ : postProcessToRow--, H = rowsCache[_];
          if (!(!H || _ >= O)) {
            postProcessedRows[_] || (postProcessedRows[_] = {}), ensureCellNodesInRowsCache(_);
            for (var V in H.cellNodesByColumnIdx)
              if (H.cellNodesByColumnIdx.hasOwnProperty(V)) {
                V = V | 0;
                var q = columns[V], G = postProcessedRows[_][V];
                if (q.asyncPostRender && G !== "R") {
                  var J = H.cellNodesByColumnIdx[V];
                  J && q.asyncPostRender(J, _, getDataItem(_), q, G === "C"), postProcessedRows[_][V] = "R";
                }
              }
            h_postrender = setTimeout(asyncPostProcessRows, options.asyncPostRenderDelay);
            return;
          }
        }
      }
      function asyncPostProcessCleanupRows() {
        if (postProcessedCleanupQueue.length > 0) {
          for (var O = postProcessedCleanupQueue[0].groupId; postProcessedCleanupQueue.length > 0 && postProcessedCleanupQueue[0].groupId == O; ) {
            var _ = postProcessedCleanupQueue.shift();
            if (_.actionType == "R" && $(_.node).remove(), _.actionType == "C") {
              var H = columns[_.columnIdx];
              H.asyncPostRenderCleanup && _.node && H.asyncPostRenderCleanup(_.node, _.rowIdx, H);
            }
          }
          h_postrenderCleanup = setTimeout(asyncPostProcessCleanupRows, options.asyncPostRenderCleanupDelay);
        }
      }
      function updateCellCssStylesOnRenderedRows(O, _) {
        var H, V, q, G;
        for (var J in rowsCache) {
          if (G = _ && _[J], q = O && O[J], G)
            for (V in G)
              (!q || G[V] != q[V]) && (H = getCellNode(J, getColumnIndex(V)), H && $(H).removeClass(G[V]));
          if (q)
            for (V in q)
              (!G || G[V] != q[V]) && (H = getCellNode(J, getColumnIndex(V)), H && $(H).addClass(q[V]));
        }
      }
      function addCellCssStyles(O, _) {
        if (cellCssClasses[O])
          throw new Error("SlickGrid addCellCssStyles: cell CSS hash with key '" + O + "' already exists.");
        cellCssClasses[O] = _, updateCellCssStylesOnRenderedRows(_, null), trigger(self.onCellCssStylesChanged, { key: O, hash: _, grid: self });
      }
      function removeCellCssStyles(O) {
        cellCssClasses[O] && (updateCellCssStylesOnRenderedRows(null, cellCssClasses[O]), delete cellCssClasses[O], trigger(self.onCellCssStylesChanged, { key: O, hash: null, grid: self }));
      }
      function setCellCssStyles(O, _) {
        var H = cellCssClasses[O];
        cellCssClasses[O] = _, updateCellCssStylesOnRenderedRows(_, H), trigger(self.onCellCssStylesChanged, { key: O, hash: _, grid: self });
      }
      function getCellCssStyles(O) {
        return cellCssClasses[O];
      }
      function flashCell(O, _, H) {
        H = H || 100;
        function V(G, J) {
          J && setTimeout(function() {
            G.queue(function() {
              G.toggleClass(options.cellFlashingCssClass).dequeue(), V(G, J - 1);
            });
          }, H);
        }
        if (rowsCache[O]) {
          var q = $(getCellNode(O, _));
          V(q, 4);
        }
      }
      function handleMouseWheel(O, _, H, V) {
        scrollTop = Math.max(0, $viewportScrollContainerY[0].scrollTop - V * options.rowHeight), scrollLeft = $viewportScrollContainerX[0].scrollLeft + H * 10;
        var q = _handleScroll(!0);
        q && O.preventDefault();
      }
      function handleDragInit(O, _) {
        var H = getCellFromEvent(O);
        if (!H || !cellExists(H.row, H.cell))
          return !1;
        var V = trigger(self.onDragInit, _, O);
        return O.isImmediatePropagationStopped() ? V : !1;
      }
      function handleDragStart(O, _) {
        var H = getCellFromEvent(O);
        if (!H || !cellExists(H.row, H.cell))
          return !1;
        var V = trigger(self.onDragStart, _, O);
        return O.isImmediatePropagationStopped() ? V : !1;
      }
      function handleDrag(O, _) {
        return trigger(self.onDrag, _, O);
      }
      function handleDragEnd(O, _) {
        trigger(self.onDragEnd, _, O);
      }
      function handleKeyDown(O) {
        trigger(self.onKeyDown, { row: activeRow, cell: activeCell }, O);
        var _ = O.isImmediatePropagationStopped(), H = Slick.keyCode;
        if (!_ && !O.shiftKey && !O.altKey) {
          if (options.editable && currentEditor && currentEditor.keyCaptureList && currentEditor.keyCaptureList.indexOf(O.which) > -1)
            return;
          O.which == H.HOME ? _ = O.ctrlKey ? navigateTop() : navigateRowStart() : O.which == H.END && (_ = O.ctrlKey ? navigateBottom() : navigateRowEnd());
        }
        if (!_)
          if (!O.shiftKey && !O.altKey && !O.ctrlKey) {
            if (options.editable && currentEditor && currentEditor.keyCaptureList && currentEditor.keyCaptureList.indexOf(O.which) > -1)
              return;
            if (O.which == H.ESCAPE) {
              if (!getEditorLock().isActive())
                return;
              cancelEditAndSetFocus();
            } else O.which == H.PAGE_DOWN ? (navigatePageDown(), _ = !0) : O.which == H.PAGE_UP ? (navigatePageUp(), _ = !0) : O.which == H.LEFT ? _ = navigateLeft() : O.which == H.RIGHT ? _ = navigateRight() : O.which == H.UP ? _ = navigateUp() : O.which == H.DOWN ? _ = navigateDown() : O.which == H.TAB ? _ = navigateNext() : O.which == H.ENTER && (options.editable && (currentEditor ? activeRow === getDataLength() ? navigateDown() : commitEditAndSetFocus() : getEditorLock().commitCurrentEdit() && makeActiveCellEditable(void 0, void 0, O)), _ = !0);
          } else O.which == H.TAB && O.shiftKey && !O.ctrlKey && !O.altKey && (_ = navigatePrev());
        if (_) {
          O.stopPropagation(), O.preventDefault();
          try {
            O.originalEvent.keyCode = 0;
          } catch {
          }
        }
      }
      function handleClick(O) {
        if (!currentEditor && (O.target != document.activeElement || $(O.target).hasClass("slick-cell"))) {
          var _ = getTextSelection();
          setFocus(), setTextSelection(_);
        }
        var H = getCellFromEvent(O);
        if (!(!H || currentEditor !== null && activeRow == H.row && activeCell == H.cell) && (trigger(self.onClick, { row: H.row, cell: H.cell }, O), !O.isImmediatePropagationStopped() && canCellBeActive(H.row, H.cell) && (!getEditorLock().isActive() || getEditorLock().commitCurrentEdit()))) {
          scrollRowIntoView(H.row, !1);
          var V = O.target && O.target.className === Slick.preClickClassName, q = columns[H.cell], G = !!(options.editable && q && q.editor && options.suppressActiveCellChangeOnEdit);
          setActiveCellInternal(getCellNode(H.row, H.cell), null, V, G, O);
        }
      }
      function handleContextMenu(O) {
        var _ = $(O.target).closest(".slick-cell", $canvas);
        _.length !== 0 && (activeCellNode === _[0] && currentEditor !== null || trigger(self.onContextMenu, {}, O));
      }
      function handleDblClick(O) {
        var _ = getCellFromEvent(O);
        !_ || currentEditor !== null && activeRow == _.row && activeCell == _.cell || (trigger(self.onDblClick, { row: _.row, cell: _.cell }, O), !O.isImmediatePropagationStopped() && options.editable && gotoCell(_.row, _.cell, !0, O));
      }
      function handleHeaderMouseEnter(O) {
        trigger(self.onHeaderMouseEnter, {
          column: $(this).data("column"),
          grid: self
        }, O);
      }
      function handleHeaderMouseLeave(O) {
        trigger(self.onHeaderMouseLeave, {
          column: $(this).data("column"),
          grid: self
        }, O);
      }
      function handleHeaderContextMenu(O) {
        var _ = $(O.target).closest(".slick-header-column", ".slick-header-columns"), H = _ && _.data("column");
        trigger(self.onHeaderContextMenu, { column: H }, O);
      }
      function handleHeaderClick(O) {
        if (!columnResizeDragging) {
          var _ = $(O.target).closest(".slick-header-column", ".slick-header-columns"), H = _ && _.data("column");
          H && trigger(self.onHeaderClick, { column: H }, O);
        }
      }
      function handleFooterContextMenu(O) {
        var _ = $(O.target).closest(".slick-footerrow-column", ".slick-footerrow-columns"), H = _ && _.data("column");
        trigger(self.onFooterContextMenu, { column: H }, O);
      }
      function handleFooterClick(O) {
        var _ = $(O.target).closest(".slick-footerrow-column", ".slick-footerrow-columns"), H = _ && _.data("column");
        trigger(self.onFooterClick, { column: H }, O);
      }
      function handleMouseEnter(O) {
        trigger(self.onMouseEnter, {}, O);
      }
      function handleMouseLeave(O) {
        trigger(self.onMouseLeave, {}, O);
      }
      function cellExists(O, _) {
        return !(O < 0 || O >= getDataLength() || _ < 0 || _ >= columns.length);
      }
      function getCellFromPoint(O, _) {
        for (var H = getRowFromPosition(_), V = 0, q = 0, G = 0; G < columns.length && q < O; G++)
          q += columns[G].width, V++;
        return V < 0 && (V = 0), { row: H, cell: V - 1 };
      }
      function getCellFromNode(O) {
        var _ = /l\d+/.exec(O.className);
        if (!_)
          throw new Error("SlickGrid getCellFromNode: cannot get cell - " + O.className);
        return parseInt(_[0].substr(1, _[0].length - 1), 10);
      }
      function getRowFromNode(O) {
        for (var _ in rowsCache)
          for (var H in rowsCache[_].rowNode)
            if (rowsCache[_].rowNode[H] === O)
              return _ ? parseInt(_) : 0;
        return null;
      }
      function getFrozenRowOffset(O) {
        var _ = hasFrozenRows ? options.frozenBottom ? O >= actualFrozenRow ? h < viewportTopH ? actualFrozenRow * options.rowHeight : h : 0 : O >= actualFrozenRow ? frozenRowsHeight : 0 : 0;
        return _;
      }
      function getCellFromEvent(O) {
        var _, H, V = $(O.target).closest(".slick-cell", $canvas);
        if (!V.length)
          return null;
        if (_ = getRowFromNode(V[0].parentNode), hasFrozenRows) {
          var q = V.parents(".grid-canvas").offset(), G = 0, J = V.parents(".grid-canvas-bottom").length;
          J && (G = options.frozenBottom ? $canvasTopL.height() : frozenRowsHeight), _ = getCellFromPoint(O.clientX - q.left, O.clientY - q.top + G + $(document).scrollTop()).row;
        }
        return H = getCellFromNode(V[0]), _ == null || H == null ? null : {
          row: _,
          cell: H
        };
      }
      function getCellNodeBox(O, _) {
        if (!cellExists(O, _))
          return null;
        for (var H = getFrozenRowOffset(O), V = getRowTop(O) - H, q = V + options.rowHeight - 1, G = 0, J = 0; J < _; J++)
          G += columns[J].width, options.frozenColumn == J && (G = 0);
        var Z = G + columns[_].width;
        return {
          top: V,
          left: G,
          bottom: q,
          right: Z
        };
      }
      function resetActiveCell() {
        setActiveCellInternal(null, !1);
      }
      function setFocus() {
        tabbingDirection == -1 ? $focusSink[0].focus() : $focusSink2[0].focus();
      }
      function scrollCellIntoView(O, _, H) {
        if (scrollRowIntoView(O, H), !(_ <= options.frozenColumn)) {
          var V = getColspan(O, _);
          internalScrollColumnIntoView(columnPosLeft[_], columnPosRight[_ + (V > 1 ? V - 1 : 0)]);
        }
      }
      function internalScrollColumnIntoView(O, _) {
        var H = scrollLeft + $viewportScrollContainerX.width();
        O < scrollLeft ? ($viewportScrollContainerX.scrollLeft(O), handleScroll(), render()) : _ > H && ($viewportScrollContainerX.scrollLeft(Math.min(O, _ - $viewportScrollContainerX[0].clientWidth)), handleScroll(), render());
      }
      function scrollColumnIntoView(O) {
        internalScrollColumnIntoView(columnPosLeft[O], columnPosRight[O]);
      }
      function setActiveCellInternal(O, _, H, V, q) {
        if (activeCellNode !== null && (makeActiveCellNormal(), $(activeCellNode).removeClass("active"), rowsCache[activeRow] && rowsCache[activeRow].rowNode.removeClass("active")), activeCellNode = O, activeCellNode != null) {
          var G = $(activeCellNode), J = G.offset(), Z = Math.floor(G.parents(".grid-canvas").offset().top), te = G.parents(".grid-canvas-bottom").length;
          hasFrozenRows && te && (Z -= options.frozenBottom ? $canvasTopL.height() : frozenRowsHeight);
          var ne = getCellFromPoint(J.left, Math.ceil(J.top) - Z);
          activeRow = ne.row, activeCell = activePosX = activeCell = activePosX = getCellFromNode(activeCellNode), _ == null && (_ = activeRow == getDataLength() || options.autoEdit), options.showCellSelection && (G.addClass("active"), rowsCache[activeRow] && rowsCache[activeRow].rowNode.addClass("active")), options.editable && _ && isCellPotentiallyEditable(activeRow, activeCell) && (clearTimeout(h_editorLoader), options.asyncEditorLoading ? h_editorLoader = setTimeout(function() {
            makeActiveCellEditable(void 0, H, q);
          }, options.asyncEditorLoadDelay) : makeActiveCellEditable(void 0, H, q));
        } else
          activeRow = activeCell = null;
        V || trigger(self.onActiveCellChanged, getActiveCell());
      }
      function clearTextSelection() {
        if (document.selection && document.selection.empty)
          try {
            document.selection.empty();
          } catch {
          }
        else if (window.getSelection) {
          var O = window.getSelection();
          O && O.removeAllRanges && O.removeAllRanges();
        }
      }
      function isCellPotentiallyEditable(O, _) {
        var H = getDataLength();
        return !(O < H && !getDataItem(O) || columns[_].cannotTriggerInsert && O >= H || !getEditor(O, _));
      }
      function makeActiveCellNormal() {
        if (currentEditor) {
          if (trigger(self.onBeforeCellEditorDestroy, { editor: currentEditor }), currentEditor.destroy(), currentEditor = null, activeCellNode) {
            var O = getDataItem(activeRow);
            if ($(activeCellNode).removeClass("editable invalid"), O) {
              var _ = columns[activeCell], H = getFormatter(activeRow, _), V = H(activeRow, activeCell, getDataItemValueForColumn(O, _), _, O, self);
              applyFormatResultToCellNode(V, activeCellNode), invalidatePostProcessingResults(activeRow);
            }
          }
          navigator.userAgent.toLowerCase().match(/msie/) && clearTextSelection(), getEditorLock().deactivate(editController);
        }
      }
      function makeActiveCellEditable(O, _, H) {
        if (activeCellNode) {
          if (!options.editable)
            throw new Error("SlickGrid makeActiveCellEditable : should never get called when options.editable is false");
          if (clearTimeout(h_editorLoader), !!isCellPotentiallyEditable(activeRow, activeCell)) {
            var V = columns[activeCell], q = getDataItem(activeRow);
            if (trigger(self.onBeforeEditCell, { row: activeRow, cell: activeCell, item: q, column: V, target: "grid" }) === !1) {
              setFocus();
              return;
            }
            getEditorLock().activate(editController), $(activeCellNode).addClass("editable");
            var G = O || getEditor(activeRow, activeCell);
            !O && !G.suppressClearOnEdit && (activeCellNode.innerHTML = "");
            var J = data.getItemMetadata && data.getItemMetadata(activeRow);
            J = J && J.columns;
            var Z = J && (J[V.id] || J[activeCell]);
            currentEditor = new G({
              grid: self,
              gridPosition: absBox($container[0]),
              position: absBox(activeCellNode),
              container: activeCellNode,
              column: V,
              columnMetaData: Z,
              item: q || {},
              event: H,
              commitChanges: commitEditAndSetFocus,
              cancelChanges: cancelEditAndSetFocus
            }), q && (currentEditor.loadValue(q), _ && currentEditor.preClick && currentEditor.preClick()), serializedEditorValue = currentEditor.serializeValue(), currentEditor.position && handleActiveCellPositionChange();
          }
        }
      }
      function commitEditAndSetFocus() {
        getEditorLock().commitCurrentEdit() && (setFocus(), options.autoEdit && navigateDown());
      }
      function cancelEditAndSetFocus() {
        getEditorLock().cancelCurrentEdit() && setFocus();
      }
      function absBox(O) {
        var _ = {
          top: O.offsetTop,
          left: O.offsetLeft,
          bottom: 0,
          right: 0,
          width: $(O).outerWidth(),
          height: $(O).outerHeight(),
          visible: !0
        };
        _.bottom = _.top + _.height, _.right = _.left + _.width;
        for (var H = O.offsetParent; (O = O.parentNode) != document.body && O != null; )
          _.visible && O.scrollHeight != O.offsetHeight && $(O).css("overflowY") != "visible" && (_.visible = _.bottom > O.scrollTop && _.top < O.scrollTop + O.clientHeight), _.visible && O.scrollWidth != O.offsetWidth && $(O).css("overflowX") != "visible" && (_.visible = _.right > O.scrollLeft && _.left < O.scrollLeft + O.clientWidth), _.left -= O.scrollLeft, _.top -= O.scrollTop, O === H && (_.left += O.offsetLeft, _.top += O.offsetTop, H = O.offsetParent), _.bottom = _.top + _.height, _.right = _.left + _.width;
        return _;
      }
      function getActiveCellPosition() {
        return absBox(activeCellNode);
      }
      function getGridPosition() {
        return absBox($container[0]);
      }
      function handleActiveCellPositionChange() {
        if (activeCellNode && (trigger(self.onActiveCellPositionChanged, {}), currentEditor)) {
          var O = getActiveCellPosition();
          currentEditor.show && currentEditor.hide && (O.visible ? currentEditor.show() : currentEditor.hide()), currentEditor.position && currentEditor.position(O);
        }
      }
      function getCellEditor() {
        return currentEditor;
      }
      function getActiveCell() {
        return activeCellNode ? { row: activeRow, cell: activeCell } : null;
      }
      function getActiveCellNode() {
        return activeCellNode;
      }
      function getTextSelection() {
        var O = null;
        if (window.getSelection) {
          var _ = window.getSelection();
          _.rangeCount > 0 && (O = _.getRangeAt(0));
        }
        return O;
      }
      function setTextSelection(O) {
        if (window.getSelection && O) {
          var _ = window.getSelection();
          _.removeAllRanges(), _.addRange(O);
        }
      }
      function scrollRowIntoView(O, _) {
        if (!hasFrozenRows || !options.frozenBottom && O > actualFrozenRow - 1 || options.frozenBottom && O < actualFrozenRow - 1) {
          var H = $viewportScrollContainerY.height(), V = hasFrozenRows && !options.frozenBottom ? O - options.frozenRow : O, q = V * options.rowHeight, G = (V + 1) * options.rowHeight - H + (viewportHasHScroll ? scrollbarDimensions.height : 0);
          (V + 1) * options.rowHeight > scrollTop + H + offset ? (scrollTo(_ ? q : G), render()) : V * options.rowHeight < scrollTop + offset && (scrollTo(_ ? G : q), render());
        }
      }
      function scrollRowToTop(O) {
        scrollTo(O * options.rowHeight), render();
      }
      function scrollPage(O) {
        var _ = O * numVisibleRows, H = scrollTop + options.rowHeight - 1;
        if (scrollTo((getRowFromPosition(H) + _) * options.rowHeight), render(), options.enableCellNavigation && activeRow != null) {
          var V = activeRow + _, q = getDataLengthIncludingAddNew();
          V >= q && (V = q - 1), V < 0 && (V = 0);
          for (var G = 0, J = null, Z = activePosX; G <= activePosX; )
            canCellBeActive(V, G) && (J = G), G += getColspan(V, G);
          J !== null ? (setActiveCellInternal(getCellNode(V, J)), activePosX = Z) : resetActiveCell();
        }
      }
      function navigatePageDown() {
        scrollPage(1);
      }
      function navigatePageUp() {
        scrollPage(-1);
      }
      function navigateTop() {
        navigateToRow(0);
      }
      function navigateBottom() {
        navigateToRow(getDataLength() - 1);
      }
      function navigateToRow(O) {
        var _ = getDataLength();
        if (!_) return !0;
        if (O < 0 ? O = 0 : O >= _ && (O = _ - 1), scrollCellIntoView(O, 0, !0), options.enableCellNavigation && activeRow != null) {
          for (var H = 0, V = null, q = activePosX; H <= activePosX; )
            canCellBeActive(O, H) && (V = H), H += getColspan(O, H);
          V !== null ? (setActiveCellInternal(getCellNode(O, V)), activePosX = q) : resetActiveCell();
        }
        return !0;
      }
      function getColspan(O, _) {
        var H = data.getItemMetadata && data.getItemMetadata(O);
        if (!H || !H.columns)
          return 1;
        var V = H.columns[columns[_].id] || H.columns[_], q = V && V.colspan;
        return q === "*" ? q = columns.length - _ : q = q || 1, q;
      }
      function findFirstFocusableCell(O) {
        for (var _ = 0; _ < columns.length; ) {
          if (canCellBeActive(O, _))
            return _;
          _ += getColspan(O, _);
        }
        return null;
      }
      function findLastFocusableCell(O) {
        for (var _ = 0, H = null; _ < columns.length; )
          canCellBeActive(O, _) && (H = _), _ += getColspan(O, _);
        return H;
      }
      function gotoRight(O, _, H) {
        if (_ >= columns.length)
          return null;
        do
          _ += getColspan(O, _);
        while (_ < columns.length && !canCellBeActive(O, _));
        return _ < columns.length ? {
          row: O,
          cell: _,
          posX: _
        } : null;
      }
      function gotoLeft(O, _, H) {
        if (_ <= 0)
          return null;
        var V = findFirstFocusableCell(O);
        if (V === null || V >= _)
          return null;
        for (var q = {
          row: O,
          cell: V,
          posX: V
        }, G; ; ) {
          if (G = gotoRight(q.row, q.cell, q.posX), !G)
            return null;
          if (G.cell >= _)
            return q;
          q = G;
        }
      }
      function gotoDown(O, _, H) {
        for (var V, q = getDataLengthIncludingAddNew(); ; ) {
          if (++O >= q)
            return null;
          for (V = _ = 0; _ <= H; )
            V = _, _ += getColspan(O, _);
          if (canCellBeActive(O, V))
            return {
              row: O,
              cell: V,
              posX: H
            };
        }
      }
      function gotoUp(O, _, H) {
        for (var V; ; ) {
          if (--O < 0)
            return null;
          for (V = _ = 0; _ <= H; )
            V = _, _ += getColspan(O, _);
          if (canCellBeActive(O, V))
            return {
              row: O,
              cell: V,
              posX: H
            };
        }
      }
      function gotoNext(O, _, H) {
        if (O == null && _ == null && (O = _ = 0, canCellBeActive(O, _)))
          return {
            row: O,
            cell: _,
            posX: _
          };
        var V = gotoRight(O, _);
        if (V)
          return V;
        var q = null, G = getDataLengthIncludingAddNew();
        for (O === G - 1 && O--; ++O < G; )
          if (q = findFirstFocusableCell(O), q !== null)
            return {
              row: O,
              cell: q,
              posX: q
            };
        return null;
      }
      function gotoPrev(O, _, H) {
        if (O == null && _ == null && (O = getDataLengthIncludingAddNew() - 1, _ = columns.length - 1, canCellBeActive(O, _)))
          return {
            row: O,
            cell: _,
            posX: _
          };
        for (var V, q; !V && (V = gotoLeft(O, _), !V); ) {
          if (--O < 0)
            return null;
          _ = 0, q = findLastFocusableCell(O), q !== null && (V = {
            row: O,
            cell: q,
            posX: q
          });
        }
        return V;
      }
      function gotoRowStart(O, _, H) {
        var V = findFirstFocusableCell(O);
        return V === null ? null : {
          row: O,
          cell: V,
          posX: V
        };
      }
      function gotoRowEnd(O, _, H) {
        var V = findLastFocusableCell(O);
        return V === null ? null : {
          row: O,
          cell: V,
          posX: V
        };
      }
      function navigateRight() {
        return navigate("right");
      }
      function navigateLeft() {
        return navigate("left");
      }
      function navigateDown() {
        return navigate("down");
      }
      function navigateUp() {
        return navigate("up");
      }
      function navigateNext() {
        return navigate("next");
      }
      function navigatePrev() {
        return navigate("prev");
      }
      function navigateRowStart() {
        return navigate("home");
      }
      function navigateRowEnd() {
        return navigate("end");
      }
      function navigate(O) {
        if (!options.enableCellNavigation || !activeCellNode && O != "prev" && O != "next")
          return !1;
        if (!getEditorLock().commitCurrentEdit())
          return !0;
        setFocus();
        var _ = {
          up: -1,
          down: 1,
          left: -1,
          right: 1,
          prev: -1,
          next: 1,
          home: -1,
          end: 1
        };
        tabbingDirection = _[O];
        var H = {
          up: gotoUp,
          down: gotoDown,
          left: gotoLeft,
          right: gotoRight,
          prev: gotoPrev,
          next: gotoNext,
          home: gotoRowStart,
          end: gotoRowEnd
        }, V = H[O], q = V(activeRow, activeCell, activePosX);
        if (q) {
          if (hasFrozenRows && options.frozenBottom & q.row == getDataLength())
            return;
          var G = q.row == getDataLength();
          return (!options.frozenBottom && q.row >= actualFrozenRow || options.frozenBottom && q.row < actualFrozenRow) && scrollCellIntoView(q.row, q.cell, !G && options.emulatePagingWhenScrolling), setActiveCellInternal(getCellNode(q.row, q.cell)), activePosX = q.posX, !0;
        } else
          return setActiveCellInternal(getCellNode(activeRow, activeCell)), !1;
      }
      function getCellNode(O, _) {
        if (rowsCache[O]) {
          ensureCellNodesInRowsCache(O);
          try {
            return rowsCache[O].cellNodesByColumnIdx.length > _ ? rowsCache[O].cellNodesByColumnIdx[_][0] : null;
          } catch {
            return rowsCache[O].cellNodesByColumnIdx[_];
          }
        }
        return null;
      }
      function setActiveCell(O, _, H, V, q) {
        initialized && (O > getDataLength() || O < 0 || _ >= columns.length || _ < 0 || options.enableCellNavigation && (scrollCellIntoView(O, _, !1), setActiveCellInternal(getCellNode(O, _), H, V, q)));
      }
      function setActiveRow(O, _, H) {
        initialized && (O > getDataLength() || O < 0 || _ >= columns.length || _ < 0 || (activeRow = O, H || scrollCellIntoView(O, _ || 0, !1)));
      }
      function canCellBeActive(O, _) {
        if (!options.enableCellNavigation || O >= getDataLengthIncludingAddNew() || O < 0 || _ >= columns.length || _ < 0)
          return !1;
        var H = data.getItemMetadata && data.getItemMetadata(O);
        if (H && typeof H.focusable < "u")
          return !!H.focusable;
        var V = H && H.columns;
        return V && V[columns[_].id] && typeof V[columns[_].id].focusable < "u" ? !!V[columns[_].id].focusable : V && V[_] && typeof V[_].focusable < "u" ? !!V[_].focusable : !!columns[_].focusable;
      }
      function canCellBeSelected(O, _) {
        if (O >= getDataLength() || O < 0 || _ >= columns.length || _ < 0)
          return !1;
        var H = data.getItemMetadata && data.getItemMetadata(O);
        if (H && typeof H.selectable < "u")
          return !!H.selectable;
        var V = H && H.columns && (H.columns[columns[_].id] || H.columns[_]);
        return V && typeof V.selectable < "u" ? !!V.selectable : !!columns[_].selectable;
      }
      function gotoCell(O, _, H, V) {
        if (initialized && canCellBeActive(O, _) && getEditorLock().commitCurrentEdit()) {
          scrollCellIntoView(O, _, !1);
          var q = getCellNode(O, _), G = columns[_], J = !!(options.editable && G && G.editor && options.suppressActiveCellChangeOnEdit);
          setActiveCellInternal(q, H || O === getDataLength() || options.autoEdit, null, J, V), currentEditor || setFocus();
        }
      }
      function commitCurrentEdit() {
        var O = getDataItem(activeRow), _ = columns[activeCell];
        if (currentEditor) {
          if (currentEditor.isValueChanged()) {
            var H = currentEditor.validate();
            if (H.valid) {
              if (activeRow < getDataLength()) {
                var V = {
                  row: activeRow,
                  cell: activeCell,
                  editor: currentEditor,
                  serializedValue: currentEditor.serializeValue(),
                  prevSerializedValue: serializedEditorValue,
                  execute: function() {
                    this.editor.applyValue(O, this.serializedValue), updateRow(this.row), trigger(self.onCellChange, {
                      command: "execute",
                      row: this.row,
                      cell: this.cell,
                      item: O,
                      column: _
                    });
                  },
                  undo: function() {
                    this.editor.applyValue(O, this.prevSerializedValue), updateRow(this.row), trigger(self.onCellChange, {
                      command: "undo",
                      row: this.row,
                      cell: this.cell,
                      item: O,
                      column: _
                    });
                  }
                };
                options.editCommandHandler ? (makeActiveCellNormal(), options.editCommandHandler(O, _, V)) : (V.execute(), makeActiveCellNormal());
              } else {
                var q = {};
                currentEditor.applyValue(q, currentEditor.serializeValue()), makeActiveCellNormal(), trigger(self.onAddNewRow, { item: q, column: _ });
              }
              return !getEditorLock().isActive();
            } else
              return $(activeCellNode).removeClass("invalid"), $(activeCellNode).width(), $(activeCellNode).addClass("invalid"), trigger(self.onValidationError, {
                editor: currentEditor,
                cellNode: activeCellNode,
                validationResults: H,
                row: activeRow,
                cell: activeCell,
                column: _
              }), currentEditor.focus(), !1;
          }
          makeActiveCellNormal();
        }
        return !0;
      }
      function cancelCurrentEdit() {
        return makeActiveCellNormal(), !0;
      }
      function rowsToRanges(O) {
        for (var _ = [], H = columns.length - 1, V = 0; V < O.length; V++)
          _.push(new Slick.Range(O[V], 0, O[V], H));
        return _;
      }
      function getSelectedRows() {
        if (!selectionModel)
          throw new Error("SlickGrid Selection model is not set");
        return selectedRows.slice(0);
      }
      function setSelectedRows(O) {
        if (!selectionModel)
          throw new Error("SlickGrid Selection model is not set");
        self && self.getEditorLock && !self.getEditorLock().isActive() && selectionModel.setSelectedRanges(rowsToRanges(O));
      }
      this.debug = function() {
        var O = "";
        O += `
counter_rows_rendered:  ` + counter_rows_rendered, O += `
counter_rows_removed:  ` + counter_rows_removed, O += `
renderedRows:  ` + renderedRows, O += `
numVisibleRows:  ` + numVisibleRows, O += `
maxSupportedCssHeight:  ` + maxSupportedCssHeight, O += `
n(umber of pages):  ` + n, O += `
(current) page:  ` + page, O += `
page height (ph):  ` + ph, O += `
vScrollDir:  ` + vScrollDir, alert(O);
      }, this.eval = function(expr) {
        return eval(expr);
      }, $.extend(this, {
        slickGridVersion: "2.4.41",
        // Events
        onScroll: new Slick.Event(),
        onBeforeSort: new Slick.Event(),
        onSort: new Slick.Event(),
        onHeaderMouseEnter: new Slick.Event(),
        onHeaderMouseLeave: new Slick.Event(),
        onHeaderContextMenu: new Slick.Event(),
        onHeaderClick: new Slick.Event(),
        onHeaderCellRendered: new Slick.Event(),
        onBeforeHeaderCellDestroy: new Slick.Event(),
        onHeaderRowCellRendered: new Slick.Event(),
        onFooterRowCellRendered: new Slick.Event(),
        onFooterContextMenu: new Slick.Event(),
        onFooterClick: new Slick.Event(),
        onBeforeHeaderRowCellDestroy: new Slick.Event(),
        onBeforeFooterRowCellDestroy: new Slick.Event(),
        onMouseEnter: new Slick.Event(),
        onMouseLeave: new Slick.Event(),
        onClick: new Slick.Event(),
        onDblClick: new Slick.Event(),
        onContextMenu: new Slick.Event(),
        onKeyDown: new Slick.Event(),
        onAddNewRow: new Slick.Event(),
        onBeforeAppendCell: new Slick.Event(),
        onValidationError: new Slick.Event(),
        onViewportChanged: new Slick.Event(),
        onColumnsReordered: new Slick.Event(),
        onColumnsDrag: new Slick.Event(),
        onColumnsResized: new Slick.Event(),
        onColumnsResizeDblClick: new Slick.Event(),
        onBeforeColumnsResize: new Slick.Event(),
        onCellChange: new Slick.Event(),
        onCompositeEditorChange: new Slick.Event(),
        onBeforeEditCell: new Slick.Event(),
        onBeforeCellEditorDestroy: new Slick.Event(),
        onBeforeDestroy: new Slick.Event(),
        onActiveCellChanged: new Slick.Event(),
        onActiveCellPositionChanged: new Slick.Event(),
        onDragInit: new Slick.Event(),
        onDragStart: new Slick.Event(),
        onDrag: new Slick.Event(),
        onDragEnd: new Slick.Event(),
        onSelectedRowsChanged: new Slick.Event(),
        onCellCssStylesChanged: new Slick.Event(),
        onAutosizeColumns: new Slick.Event(),
        onBeforeSetColumns: new Slick.Event(),
        onRendered: new Slick.Event(),
        onSetOptions: new Slick.Event(),
        // Methods
        registerPlugin,
        unregisterPlugin,
        getPluginByName,
        getColumns,
        setColumns,
        getColumnIndex,
        updateColumnHeader,
        setSortColumn,
        setSortColumns,
        getSortColumns,
        autosizeColumns,
        autosizeColumn,
        getOptions,
        setOptions,
        getData,
        getDataLength,
        getDataItem,
        setData,
        getSelectionModel,
        setSelectionModel,
        getSelectedRows,
        setSelectedRows,
        getContainerNode,
        updatePagingStatusFromView,
        applyFormatResultToCellNode,
        render,
        reRenderColumns,
        invalidate,
        invalidateRow,
        invalidateRows,
        invalidateAllRows,
        updateCell,
        updateRow,
        getViewport: getVisibleRange,
        getRenderedRange,
        resizeCanvas,
        updateRowCount,
        scrollRowIntoView,
        scrollRowToTop,
        scrollCellIntoView,
        scrollColumnIntoView,
        getCanvasNode,
        getUID,
        getHeaderColumnWidthDiff,
        getScrollbarDimensions,
        getHeadersWidth,
        getCanvasWidth,
        getCanvases,
        getActiveCanvasNode,
        setActiveCanvasNode,
        getViewportNode,
        getViewports,
        getActiveViewportNode,
        setActiveViewportNode,
        focus: setFocus,
        scrollTo,
        cacheCssForHiddenInit,
        restoreCssFromHiddenInit,
        getCellFromPoint,
        getCellFromEvent,
        getActiveCell,
        setActiveCell,
        setActiveRow,
        getActiveCellNode,
        getActiveCellPosition,
        resetActiveCell,
        editActiveCell: makeActiveCellEditable,
        getCellEditor,
        getCellNode,
        getCellNodeBox,
        canCellBeSelected,
        canCellBeActive,
        navigatePrev,
        navigateNext,
        navigateUp,
        navigateDown,
        navigateLeft,
        navigateRight,
        navigatePageUp,
        navigatePageDown,
        navigateTop,
        navigateBottom,
        navigateRowStart,
        navigateRowEnd,
        gotoCell,
        getTopPanel,
        setTopPanelVisibility,
        getPreHeaderPanel,
        getPreHeaderPanelLeft: getPreHeaderPanel,
        getPreHeaderPanelRight,
        setPreHeaderPanelVisibility,
        getHeader,
        getHeaderColumn,
        setHeaderRowVisibility,
        getHeaderRow,
        getHeaderRowColumn,
        setFooterRowVisibility,
        getFooterRow,
        getFooterRowColumn,
        getGridPosition,
        flashCell,
        addCellCssStyles,
        setCellCssStyles,
        removeCellCssStyles,
        getCellCssStyles,
        getFrozenRowOffset,
        setColumnHeaderVisibility,
        init: finishInitialization,
        destroy,
        // IEditor implementation
        getEditorLock,
        getEditController
      }), init();
    }
    return slick_grid = {
      Grid: SlickGrid
    }, slick_grid;
  }
  var slick_dataview, hasRequiredSlick_dataview;
  function requireSlick_dataview() {
    if (hasRequiredSlick_dataview) return slick_dataview;
    hasRequiredSlick_dataview = 1;
    var O = requireSlick_jquery(), _ = requireSlick_core();
    function H(ne) {
      var ee = this, ae = {
        groupItemMetadataProvider: null,
        inlineFilters: !1
      }, oe = "id", se = [], ce = [], le = new _.Map(), ue = null, de = null, fe = null, he = !1, me = !1, pe = new _.Map(), ye = !0, ge, ve, be = {}, Ce = {}, Se, we = [], Ee, xe, Le = [], Qe = null, ke = {
        getter: null,
        formatter: null,
        comparer: function(Br, Pr) {
          return Br.value === Pr.value ? 0 : Br.value > Pr.value ? 1 : -1;
        },
        predefinedValues: [],
        aggregators: [],
        aggregateEmpty: !1,
        aggregateCollapsed: !1,
        aggregateChildGroups: !1,
        collapsed: !1,
        displayTotalsRow: !0,
        lazyTotalsCalculation: !1
      }, Me = [], Re = [], De = [], Pe = ":|:", $e = null, qe = 0, Xe = 0, Ve = 0, Ze = new _.Event(), Ge = new _.Event(), ze = new _.Event(), We = new _.Event(), at = new _.Event(), ct = new _.Event(), et = new _.Event(), dt = new _.Event();
      ne = O.extend(!0, {}, ae, ne);
      function Je(Br) {
        he = !0, me = Br === !0;
      }
      function it() {
        var Br = me;
        me = !1, he = !1, Br && (yt(), Lt()), fi();
      }
      function st() {
        se = [], le = null, ue = null, de = null, fe = null, ve = null, Le = [], we = [], Ee = null, xe = null, Qe && Qe.onSelectedRowsChanged && Qe.onCellCssStylesChanged && (Qe.onSelectedRowsChanged.unsubscribe(), Qe.onCellCssStylesChanged.unsubscribe()), ee.onRowsOrCountChanged && ee.onRowsOrCountChanged.unsubscribe();
      }
      function gt(Br) {
        be = Br;
      }
      function bt(Br) {
        Se = Br;
      }
      function yt() {
        for (var Br, Pr, ei = 0, Ci = 0, oa = se.length; Ci < oa; Ci++) {
          if (Pr = se[Ci], Br = Pr[oe], Br === void 0)
            throw new Error("[SlickGrid DataView] Each data element must implement a unique 'id' property");
          pe.has(Br) ? le.delete(Br) : (se[ei] = Pr, le.set(Br, ei), ++ei);
        }
        se.length = ei, pe = new _.Map();
      }
      function St(Br) {
        if (!me) {
          Br = Br || 0;
          for (var Pr, ei = Br, Ci = se.length; ei < Ci; ei++) {
            if (Pr = se[ei][oe], Pr === void 0)
              throw new Error("[SlickGrid DataView] Each data element must implement a unique 'id' property");
            le.set(Pr, ei);
          }
        }
      }
      function Lt() {
        if (!me) {
          for (var Br, Pr = 0, ei = se.length; Pr < ei; Pr++)
            if (Br = se[Pr][oe], Br === void 0 || le.get(Br) !== Pr)
              throw new Error("[SlickGrid DataView] Each data element must implement a unique 'id' property");
        }
      }
      function Rt() {
        return se;
      }
      function vt() {
        return oe;
      }
      function $t(Br, Pr) {
        Pr !== void 0 && (oe = Pr), se = we = Br, Ze.notify({ idProperty: Pr, itemCount: se.length }, null, ee), le = new _.Map(), St(), Lt(), fi();
      }
      function Pt(Br) {
        at.notify(Xt(), null, ee) !== !1 && (Br.pageSize != null && (qe = Br.pageSize, Xe = qe ? Math.min(Xe, Math.max(0, Math.ceil(Ve / qe) - 1)) : 0), Br.pageNum != null && (Xe = Math.min(Br.pageNum, Math.max(0, Math.ceil(Ve / qe) - 1))), ct.notify(Xt(), null, ee), fi());
      }
      function Xt() {
        var Br = qe ? Math.max(1, Math.ceil(Ve / qe)) : 1;
        return { pageSize: qe, pageNum: Xe, totalRows: Ve, totalPages: Br, dataView: ee };
      }
      function Nt(Br, Pr) {
        ye = Pr, ve = Br, ge = null, Pr === !1 && se.reverse(), se.sort(Br), Pr === !1 && se.reverse(), le = new _.Map(), St(), fi();
      }
      function Gt(Br, Pr) {
        ye = Pr, ge = Br, ve = null;
        var ei = Object.prototype.toString;
        Object.prototype.toString = typeof Br == "function" ? Br : function() {
          return this[Br];
        }, Pr === !1 && se.reverse(), se.sort(), Object.prototype.toString = ei, Pr === !1 && se.reverse(), le = new _.Map(), St(), fi();
      }
      function Ft() {
        ve ? Nt(ve, ye) : ge && Gt(ge, ye);
      }
      function Tt() {
        return we;
      }
      function It() {
        return we.length;
      }
      function Ct() {
        return de;
      }
      function Kt(Br) {
        de = Br, ne.inlineFilters && (Ee = Ar(), xe = Yr()), fi();
      }
      function Vt() {
        return Me;
      }
      function Yt(Br) {
        ne.groupItemMetadataProvider || (ne.groupItemMetadataProvider = new _.Data.GroupItemMetadataProvider()), Re = [], De = [], Br = Br || [], Me = Br instanceof Array ? Br : [Br];
        for (var Pr = 0; Pr < Me.length; Pr++) {
          var ei = Me[Pr] = O.extend(!0, {}, ke, Me[Pr]);
          ei.getterIsAFn = typeof ei.getter == "function", ei.compiledAccumulators = [];
          for (var Ci = ei.aggregators.length; Ci--; )
            ei.compiledAccumulators[Ci] = mr(ei.aggregators[Ci]);
          De[Pr] = {};
        }
        fi();
      }
      function sn(Br, Pr, ei) {
        if (Br == null) {
          Yt([]);
          return;
        }
        Yt({
          getter: Br,
          formatter: Pr,
          comparer: ei
        });
      }
      function hn(Br, Pr) {
        if (!Me.length)
          throw new Error("[SlickGrid DataView] At least one grouping must be specified before calling setAggregators().");
        Me[0].aggregators = Br, Me[0].aggregateCollapsed = Pr, Yt(Me);
      }
      function Qt(Br) {
        return se[Br];
      }
      function Ut(Br) {
        return le.get(Br);
      }
      function Wt() {
        if (!ue) {
          ue = {};
          for (var Br = 0, Pr = ce.length; Br < Pr; Br++)
            ue[ce[Br][oe]] = Br;
        }
      }
      function rn(Br) {
        return Wt(), ue[Br[oe]];
      }
      function vn(Br) {
        return Wt(), ue[Br];
      }
      function dn(Br) {
        return se[le.get(Br)];
      }
      function Ht(Br) {
        var Pr = [];
        Wt();
        for (var ei = 0, Ci = Br.length; ei < Ci; ei++) {
          var oa = ue[Br[ei][oe]];
          oa != null && (Pr[Pr.length] = oa);
        }
        return Pr;
      }
      function jt(Br) {
        var Pr = [];
        Wt();
        for (var ei = 0, Ci = Br.length; ei < Ci; ei++) {
          var oa = ue[Br[ei]];
          oa != null && (Pr[Pr.length] = oa);
        }
        return Pr;
      }
      function cn(Br) {
        for (var Pr = [], ei = 0, Ci = Br.length; ei < Ci; ei++)
          Br[ei] < ce.length && (Pr[Pr.length] = ce[Br[ei]][oe]);
        return Pr;
      }
      function wn(Br, Pr) {
        if (!le.has(Br))
          throw new Error("[SlickGrid DataView] Invalid id");
        if (Br !== Pr[oe]) {
          var ei = Pr[oe];
          if (ei == null)
            throw new Error("[SlickGrid DataView] Cannot update item to associate with a null id");
          if (le.has(ei))
            throw new Error("[SlickGrid DataView] Cannot update item to associate with a non-unique id");
          le.set(ei, le.get(Br)), le.delete(Br), fe && fe[Br] && delete fe[Br], Br = ei;
        }
        se[le.get(Br)] = Pr, fe || (fe = {}), fe[Br] = !0;
      }
      function Rn(Br, Pr) {
        wn(Br, Pr), fi();
      }
      function Pn(Br, Pr) {
        if (Br.length !== Pr.length)
          throw new Error("[SlickGrid DataView] Mismatch on the length of ids and items provided to update");
        for (var ei = 0, Ci = Pr.length; ei < Ci; ei++)
          wn(Br[ei], Pr[ei]);
        fi();
      }
      function Ie(Br, Pr) {
        se.splice(Br, 0, Pr), St(Br), fi();
      }
      function rt(Br, Pr) {
        Array.prototype.splice.apply(se, [Br, 0].concat(Pr)), St(Br), fi();
      }
      function ft(Br) {
        se.push(Br), St(se.length - 1), fi();
      }
      function pt(Br) {
        se = se.concat(Br), St(se.length - Br.length), fi();
      }
      function wt(Br) {
        if (me)
          pe.set(Br, !0);
        else {
          var Pr = le.get(Br);
          if (Pr === void 0)
            throw new Error("[SlickGrid DataView] Invalid id");
          le.delete(Br), se.splice(Pr, 1), St(Pr), fi();
        }
      }
      function Bt(Br) {
        if (Br.length !== 0)
          if (me)
            for (var Pr = 0, ei = Br.length; Pr < ei; Pr++) {
              var Ci = Br[Pr], oa = le.get(Ci);
              if (oa === void 0)
                throw new Error("[SlickGrid DataView] Invalid id");
              pe.set(Ci, !0);
            }
          else {
            for (var ka = [], Pr = 0, ei = Br.length; Pr < ei; Pr++) {
              var Ci = Br[Pr], oa = le.get(Ci);
              if (oa === void 0)
                throw new Error("[SlickGrid DataView] Invalid id");
              le.delete(Ci), ka.push(oa);
            }
            ka.sort();
            for (var Pr = ka.length - 1; Pr >= 0; --Pr)
              se.splice(ka[Pr], 1);
            St(ka[0]), fi();
          }
      }
      function At(Br) {
        if (!ve)
          throw new Error("[SlickGrid DataView] sortedAddItem() requires a sort comparer, use sort()");
        Ie(Ot(Br), Br);
      }
      function Dt(Br, Pr) {
        if (!le.has(Br) || Br !== Pr[oe])
          throw new Error("[SlickGrid DataView] Invalid or non-matching id " + le.get(Br));
        if (!ve)
          throw new Error("[SlickGrid DataView] sortedUpdateItem() requires a sort comparer, use sort()");
        var ei = dn(Br);
        ve(ei, Pr) !== 0 ? (wt(Br), At(Pr)) : Rn(Br, Pr);
      }
      function Ot(Br) {
        for (var Pr = 0, ei = se.length; Pr < ei; ) {
          var Ci = Pr + ei >>> 1;
          ve(se[Ci], Br) === -1 ? Pr = Ci + 1 : ei = Ci;
        }
        return Pr;
      }
      function un() {
        return se.length;
      }
      function yn() {
        return ce.length;
      }
      function _n(Br) {
        var Pr = ce[Br];
        if (Pr && Pr.__group && Pr.totals && !Pr.totals.initialized) {
          var ei = Me[Pr.level];
          ei.displayTotalsRow || (jn(Pr.totals), Pr.title = ei.formatter ? ei.formatter(Pr) : Pr.value);
        } else Pr && Pr.__groupTotals && !Pr.initialized && jn(Pr);
        return Pr;
      }
      function gn(Br) {
        var Pr = ce[Br];
        return Pr === void 0 ? null : Pr.__group ? ne.groupItemMetadataProvider.getGroupRowMetadata(Pr) : Pr.__groupTotals ? ne.groupItemMetadataProvider.getTotalsRowMetadata(Pr) : null;
      }
      function bn(Br, Pr) {
        if (Br == null)
          for (var ei = 0; ei < Me.length; ei++)
            De[ei] = {}, Me[ei].collapsed = Pr, Pr === !0 ? dt.notify({ level: ei, groupingKey: null }) : et.notify({ level: ei, groupingKey: null });
        else
          De[Br] = {}, Me[Br].collapsed = Pr, Pr === !0 ? dt.notify({ level: Br, groupingKey: null }) : et.notify({ level: Br, groupingKey: null });
        fi();
      }
      function an(Br) {
        bn(Br, !0);
      }
      function Zt(Br) {
        bn(Br, !1);
      }
      function Mn(Br, Pr, ei) {
        De[Br][Pr] = Me[Br].collapsed ^ ei, fi();
      }
      function Ln(Br) {
        var Pr = Array.prototype.slice.call(arguments), ei = Pr[0], Ci, oa;
        Pr.length === 1 && ei.indexOf(Pe) !== -1 ? (Ci = ei, oa = ei.split(Pe).length - 1) : (Ci = Pr.join(Pe), oa = Pr.length - 1), Mn(oa, Ci, !0), dt.notify({ level: oa, groupingKey: Ci });
      }
      function In(Br) {
        var Pr = Array.prototype.slice.call(arguments), ei = Pr[0], Ci, oa;
        Pr.length === 1 && ei.indexOf(Pe) !== -1 ? (oa = ei.split(Pe).length - 1, Ci = ei) : (oa = Pr.length - 1, Ci = Pr.join(Pe)), Mn(oa, Ci, !1), et.notify({ level: oa, groupingKey: Ci });
      }
      function kn() {
        return Re;
      }
      function er(Br, Pr) {
        for (var ei, Ci, oa = [], ka = {}, aa, hi = Pr ? Pr.level + 1 : 0, Gr = Me[hi], Ca = 0, Sr = Gr.predefinedValues.length; Ca < Sr; Ca++)
          Ci = Gr.predefinedValues[Ca], ei = ka[Ci], ei || (ei = new _.Group(), ei.value = Ci, ei.level = hi, ei.groupingKey = (Pr ? Pr.groupingKey + Pe : "") + Ci, oa[oa.length] = ei, ka[Ci] = ei);
        for (var Ca = 0, Sr = Br.length; Ca < Sr; Ca++)
          aa = Br[Ca], Ci = Gr.getterIsAFn ? Gr.getter(aa) : aa[Gr.getter], ei = ka[Ci], ei || (ei = new _.Group(), ei.value = Ci, ei.level = hi, ei.groupingKey = (Pr ? Pr.groupingKey + Pe : "") + Ci, oa[oa.length] = ei, ka[Ci] = ei), ei.rows[ei.count++] = aa;
        if (hi < Me.length - 1)
          for (var Ca = 0; Ca < oa.length; Ca++)
            ei = oa[Ca], ei.groups = er(ei.rows, ei);
        return oa.length && Kn(oa, hi), oa.sort(Me[hi].comparer), oa;
      }
      function jn(Br) {
        var Pr = Br.group, ei = Me[Pr.level], Ci = Pr.level == Me.length, oa, ka = ei.aggregators.length;
        if (!Ci && ei.aggregateChildGroups)
          for (var aa = Pr.groups.length; aa--; )
            Pr.groups[aa].totals.initialized || jn(Pr.groups[aa].totals);
        for (; ka--; )
          oa = ei.aggregators[ka], oa.init(), !Ci && ei.aggregateChildGroups ? ei.compiledAccumulators[ka].call(oa, Pr.groups) : ei.compiledAccumulators[ka].call(oa, Pr.rows), oa.storeResult(Br);
        Br.initialized = !0;
      }
      function ur(Br) {
        var Pr = Me[Br.level], ei = new _.GroupTotals();
        ei.group = Br, Br.totals = ei, Pr.lazyTotalsCalculation || jn(ei);
      }
      function Kn(Br, Pr) {
        Pr = Pr || 0;
        for (var ei = Me[Pr], Ci = ei.collapsed, oa = De[Pr], ka = Br.length, aa; ka--; )
          aa = Br[ka], !(aa.collapsed && !ei.aggregateCollapsed) && (aa.groups && Kn(aa.groups, Pr + 1), ei.aggregators.length && (ei.aggregateEmpty || aa.rows.length || aa.groups && aa.groups.length) && ur(aa), aa.collapsed = Ci ^ oa[aa.groupingKey], aa.title = ei.formatter ? ei.formatter(aa) : aa.value);
      }
      function Jn(Br, Pr) {
        Pr = Pr || 0;
        for (var ei = Me[Pr], Ci = [], oa, ka = 0, aa, hi = 0, Gr = Br.length; hi < Gr; hi++) {
          if (aa = Br[hi], Ci[ka++] = aa, !aa.collapsed) {
            oa = aa.groups ? Jn(aa.groups, Pr + 1) : aa.rows;
            for (var Ca = 0, Sr = oa.length; Ca < Sr; Ca++)
              Ci[ka++] = oa[Ca];
          }
          aa.totals && ei.displayTotalsRow && (!aa.collapsed || ei.aggregateCollapsed) && (Ci[ka++] = aa.totals);
        }
        return Ci;
      }
      function Lr(Br) {
        var Pr = Br.toString(), ei = Pr.indexOf("function") >= 0, Ci = ei ? /^function[^(]*\(([^)]*)\)\s*{([\s\S]*)}$/ : /^[^(]*\(([^)]*)\)\s*{([\s\S]*)}$/, oa = Br.toString().match(Ci);
        return {
          params: oa[1].split(","),
          body: oa[2]
        };
      }
      function mr(Br) {
        if (Br.accumulate) {
          var Pr = Lr(Br.accumulate), ei = new Function(
            "_items",
            "for (var " + Pr.params[0] + ", _i=0, _il=_items.length; _i<_il; _i++) {" + Pr.params[0] + " = _items[_i]; " + Pr.body + "}"
          ), Ci = "compiledAccumulatorLoop";
          return ei.displayName = Ci, ei.name = jr(ei, Ci), ei;
        } else
          return function() {
          };
      }
      function Ar() {
        var Br = Lr(de), Pr = "{ continue _coreloop; }$1", ei = "{ _retval[_idx++] = $item$; continue _coreloop; }$1", Ci = Br.body.replace(/return false\s*([;}]|\}|$)/gi, Pr).replace(/return!1([;}]|\}|$)/gi, Pr).replace(/return true\s*([;}]|\}|$)/gi, ei).replace(/return!0([;}]|\}|$)/gi, ei).replace(
          /return ([^;}]+?)\s*([;}]|$)/gi,
          "{ if ($1) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"
        ), oa = [
          //"function(_items, _args) { ",
          "var _retval = [], _idx = 0; ",
          "var $item$, $args$ = _args; ",
          "_coreloop: ",
          "for (var _i = 0, _il = _items.length; _i < _il; _i++) { ",
          "$item$ = _items[_i]; ",
          "$filter$; ",
          "} ",
          "return _retval; "
          //"}"
        ].join("");
        oa = oa.replace(/\$filter\$/gi, Ci), oa = oa.replace(/\$item\$/gi, Br.params[0]), oa = oa.replace(/\$args\$/gi, Br.params[1]);
        var ka = new Function("_items,_args", oa), aa = "compiledFilter";
        return ka.displayName = aa, ka.name = jr(ka, aa), ka;
      }
      function Yr() {
        var Br = Lr(de), Pr = "{ continue _coreloop; }$1", ei = "{ _cache[_i] = true;_retval[_idx++] = $item$; continue _coreloop; }$1", Ci = Br.body.replace(/return false\s*([;}]|\}|$)/gi, Pr).replace(/return!1([;}]|\}|$)/gi, Pr).replace(/return true\s*([;}]|\}|$)/gi, ei).replace(/return!0([;}]|\}|$)/gi, ei).replace(
          /return ([^;}]+?)\s*([;}]|$)/gi,
          "{ if ((_cache[_i] = $1)) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"
        ), oa = [
          //"function(_items, _args, _cache) { ",
          "var _retval = [], _idx = 0; ",
          "var $item$, $args$ = _args; ",
          "_coreloop: ",
          "for (var _i = 0, _il = _items.length; _i < _il; _i++) { ",
          "$item$ = _items[_i]; ",
          "if (_cache[_i]) { ",
          "_retval[_idx++] = $item$; ",
          "continue _coreloop; ",
          "} ",
          "$filter$; ",
          "} ",
          "return _retval; "
          //"}"
        ].join("");
        oa = oa.replace(/\$filter\$/gi, Ci), oa = oa.replace(/\$item\$/gi, Br.params[0]), oa = oa.replace(/\$args\$/gi, Br.params[1]);
        var ka = new Function("_items,_args,_cache", oa), aa = "compiledFilterWithCaching";
        return ka.displayName = aa, ka.name = jr(ka, aa), ka;
      }
      function jr(Br, Pr) {
        try {
          Object.defineProperty(Br, "name", {
            writable: !0,
            value: Pr
          });
        } catch {
          Br.name = Pr;
        }
      }
      function Mr(Br, Pr) {
        for (var ei = [], Ci = 0, oa = 0, ka = Br.length; oa < ka; oa++)
          de(Br[oa], Pr) && (ei[Ci++] = Br[oa]);
        return ei;
      }
      function Wr(Br, Pr, ei) {
        for (var Ci = [], oa = 0, ka, aa = 0, hi = Br.length; aa < hi; aa++)
          ka = Br[aa], ei[aa] ? Ci[oa++] = ka : de(ka, Pr) && (Ci[oa++] = ka, ei[aa] = !0);
        return Ci;
      }
      function Bi(Br) {
        if (de) {
          var Pr = ne.inlineFilters ? Ee : Mr, ei = ne.inlineFilters ? xe : Wr;
          be.isFilterNarrowing ? we = Pr(we, Se) : be.isFilterExpanding ? we = ei(Br, Se, Le) : be.isFilterUnchanged || (we = Pr(Br, Se));
        } else
          we = qe ? Br : Br.concat();
        var Ci;
        return qe ? (we.length <= Xe * qe && (we.length === 0 ? Xe = 0 : Xe = Math.floor((we.length - 1) / qe)), Ci = we.slice(qe * Xe, qe * Xe + qe)) : Ci = we, { totalRows: we.length, rows: Ci };
      }
      function gi(Br, Pr) {
        var ei, Ci, oa, ka = [], aa = 0, hi = Math.max(Pr.length, Br.length);
        be && be.ignoreDiffsBefore && (aa = Math.max(
          0,
          Math.min(Pr.length, be.ignoreDiffsBefore)
        )), be && be.ignoreDiffsAfter && (hi = Math.min(
          Pr.length,
          Math.max(0, be.ignoreDiffsAfter)
        ));
        for (var Gr = aa, Ca = Br.length; Gr < hi; Gr++)
          Gr >= Ca ? ka[ka.length] = Gr : (ei = Pr[Gr], Ci = Br[Gr], (!ei || Me.length && (oa = ei.__nonDataRow || Ci.__nonDataRow) && ei.__group !== Ci.__group || ei.__group && !ei.equals(Ci) || oa && // no good way to compare totals since they are arbitrary DTOs
          // deep object comparison is pretty expensive
          // always considering them 'dirty' seems easier for the time being
          (ei.__groupTotals || Ci.__groupTotals) || ei[oe] != Ci[oe] || fe && fe[ei[oe]]) && (ka[ka.length] = Gr));
        return ka;
      }
      function Zr(Br) {
        ue = null, (be.isFilterNarrowing != Ce.isFilterNarrowing || be.isFilterExpanding != Ce.isFilterExpanding) && (Le = []);
        var Pr = Bi(Br);
        Ve = Pr.totalRows;
        var ei = Pr.rows;
        Re = [], Me.length && (Re = er(ei), Re.length && (ei = Jn(Re)));
        var Ci = gi(ce, ei);
        return ce = ei, Ci;
      }
      function fi() {
        if (!he) {
          var Br = O.extend(!0, {}, Xt()), Pr = ce.length, ei = Ve, Ci = Zr(se);
          qe && Ve < Xe * qe && (Xe = Math.max(0, Math.ceil(Ve / qe) - 1), Ci = Zr(se)), fe = null, Ce = be, be = {}, ei !== Ve && at.notify(Br, null, ee) !== !1 && ct.notify(Xt(), null, ee), Pr !== ce.length && Ge.notify({ previous: Pr, current: ce.length, itemCount: se.length, dataView: ee, callingOnRowsChanged: Ci.length > 0 }, null, ee), Ci.length > 0 && ze.notify({ rows: Ci, itemCount: se.length, dataView: ee, calledOnRowCountChanged: Pr !== ce.length }, null, ee), (Pr !== ce.length || Ci.length > 0) && We.notify({
            rowsDiff: Ci,
            previousRowCount: Pr,
            currentRowCount: ce.length,
            itemCount: se.length,
            rowCountChanged: Pr !== ce.length,
            rowsChanged: Ci.length > 0,
            dataView: ee
          }, null, ee);
        }
      }
      function ri(Br, Pr, ei) {
        var Ci = this;
        Qe = Br;
        var oa;
        $e = Ci.mapRowsToIds(Br.getSelectedRows());
        var ka = new _.Event();
        function aa(Gr) {
          $e.join(",") != Gr.join(",") && ($e = Gr, ka.notify({
            grid: Br,
            ids: $e,
            dataView: Ci
          }, new _.EventData(), Ci));
        }
        function hi() {
          if ($e.length > 0) {
            oa = !0;
            var Gr = Ci.mapIdsToRows($e);
            Pr || aa(Ci.mapRowsToIds(Gr)), Br.setSelectedRows(Gr), oa = !1;
          }
        }
        return Br.onSelectedRowsChanged.subscribe(function(Gr, Ca) {
          if (!oa) {
            var Sr = Ci.mapRowsToIds(Br.getSelectedRows());
            if (!ei || !Br.getOptions().multiSelect)
              aa(Sr);
            else {
              var Dr = O.grep($e, function(si) {
                return Ci.getRowById(si) === void 0;
              });
              aa(Dr.concat(Sr));
            }
          }
        }), this.onRowsOrCountChanged.subscribe(hi), ka;
      }
      function bi() {
        return $e;
      }
      function Zi() {
        var Br = [], Pr = bi();
        return Pr.forEach(function(ei) {
          Br.push(ee.getItemById(ei));
        }), Br;
      }
      function $i(Br, Pr) {
        var ei, Ci;
        oa(Br.getCellCssStyles(Pr));
        function oa(aa) {
          ei = {};
          for (var hi in aa) {
            var Gr = ce[hi][oe];
            ei[Gr] = aa[hi];
          }
        }
        function ka() {
          if (ei) {
            Ci = !0, Wt();
            var aa = {};
            for (var hi in ei) {
              var Gr = ue[hi];
              Gr != null && (aa[Gr] = ei[hi]);
            }
            Br.setCellCssStyles(Pr, aa), Ci = !1;
          }
        }
        Br.onCellCssStylesChanged.subscribe(function(aa, hi) {
          Ci || Pr == hi.key && (hi.hash ? oa(hi.hash) : (Br.onCellCssStylesChanged.unsubscribe(), ee.onRowsOrCountChanged.unsubscribe(ka)));
        }), this.onRowsOrCountChanged.subscribe(ka);
      }
      O.extend(this, {
        // methods
        beginUpdate: Je,
        endUpdate: it,
        destroy: st,
        setPagingOptions: Pt,
        getPagingInfo: Xt,
        getIdPropertyName: vt,
        getItems: Rt,
        setItems: $t,
        setFilter: Kt,
        getFilter: Ct,
        getFilteredItems: Tt,
        getFilteredItemCount: It,
        sort: Nt,
        fastSort: Gt,
        reSort: Ft,
        setGrouping: Yt,
        getGrouping: Vt,
        groupBy: sn,
        setAggregators: hn,
        collapseAllGroups: an,
        expandAllGroups: Zt,
        collapseGroup: Ln,
        expandGroup: In,
        getGroups: kn,
        getAllSelectedIds: bi,
        getAllSelectedItems: Zi,
        getIdxById: Ut,
        getRowByItem: rn,
        getRowById: vn,
        getItemById: dn,
        getItemByIdx: Qt,
        mapItemsToRows: Ht,
        mapRowsToIds: cn,
        mapIdsToRows: jt,
        setRefreshHints: gt,
        setFilterArgs: bt,
        refresh: fi,
        updateItem: Rn,
        updateItems: Pn,
        insertItem: Ie,
        insertItems: rt,
        addItem: ft,
        addItems: pt,
        deleteItem: wt,
        deleteItems: Bt,
        sortedAddItem: At,
        sortedUpdateItem: Dt,
        syncGridSelection: ri,
        syncGridCellCssStyles: $i,
        // data provider methods
        getItemCount: un,
        getLength: yn,
        getItem: _n,
        getItemMetadata: gn,
        // events
        onSetItemsCalled: Ze,
        onRowCountChanged: Ge,
        onRowsChanged: ze,
        onRowsOrCountChanged: We,
        onBeforePagingInfoChanged: at,
        onPagingInfoChanged: ct,
        onGroupExpanded: et,
        onGroupCollapsed: dt
      });
    }
    function V(ne) {
      this.field_ = ne, this.init = function() {
        this.count_ = 0, this.nonNullCount_ = 0, this.sum_ = 0;
      }, this.accumulate = function(ee) {
        var ae = ee[this.field_];
        this.count_++, ae != null && ae !== "" && !isNaN(ae) && (this.nonNullCount_++, this.sum_ += parseFloat(ae));
      }, this.storeResult = function(ee) {
        ee.avg || (ee.avg = {}), this.nonNullCount_ !== 0 && (ee.avg[this.field_] = this.sum_ / this.nonNullCount_);
      };
    }
    function q(ne) {
      this.field_ = ne, this.init = function() {
        this.min_ = null;
      }, this.accumulate = function(ee) {
        var ae = ee[this.field_];
        ae != null && ae !== "" && !isNaN(ae) && (this.min_ == null || ae < this.min_) && (this.min_ = ae);
      }, this.storeResult = function(ee) {
        ee.min || (ee.min = {}), ee.min[this.field_] = this.min_;
      };
    }
    function G(ne) {
      this.field_ = ne, this.init = function() {
        this.max_ = null;
      }, this.accumulate = function(ee) {
        var ae = ee[this.field_];
        ae != null && ae !== "" && !isNaN(ae) && (this.max_ == null || ae > this.max_) && (this.max_ = ae);
      }, this.storeResult = function(ee) {
        ee.max || (ee.max = {}), ee.max[this.field_] = this.max_;
      };
    }
    function J(ne) {
      this.field_ = ne, this.init = function() {
        this.sum_ = null;
      }, this.accumulate = function(ee) {
        var ae = ee[this.field_];
        ae != null && ae !== "" && !isNaN(ae) && (this.sum_ += parseFloat(ae));
      }, this.storeResult = function(ee) {
        ee.sum || (ee.sum = {}), ee.sum[this.field_] = this.sum_;
      };
    }
    function Z(ne) {
      this.field_ = ne, this.init = function() {
      }, this.storeResult = function(ee) {
        ee.count || (ee.count = {}), ee.count[this.field_] = ee.group.rows.length;
      };
    }
    var te = {
      Avg: V,
      Min: q,
      Max: G,
      Sum: J,
      Count: Z
    };
    return slick_dataview = {
      DataView: H,
      Aggregators: te,
      Data: { Aggregators: te }
    }, slick_dataview;
  }
  var slick_editors, hasRequiredSlick_editors;
  function requireSlick_editors() {
    if (hasRequiredSlick_editors) return slick_editors;
    hasRequiredSlick_editors = 1;
    var O = requireSlick_jquery(), _ = requireSlick_core();
    function H(oe) {
      var se, ce, le = this;
      this.args = oe, this.init = function() {
        var ue = oe.grid.getOptions().editorCellNavOnLRKeys;
        se = O("<INPUT type=text class='editor-text' />").appendTo(oe.container).on("keydown.nav", ue ? ee : ae).focus().select(), oe.compositeEditorOptions && se.on("change", function() {
          var de = oe.grid.getActiveCell();
          le.validate().valid && le.applyValue(le.args.item, le.serializeValue()), le.applyValue(le.args.compositeEditorOptions.formValues, le.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: de.row, cell: de.cell, item: le.args.item, column: le.args.column, formValues: le.args.compositeEditorOptions.formValues });
        });
      }, this.destroy = function() {
        se.remove();
      }, this.focus = function() {
        se.focus();
      }, this.getValue = function() {
        return se.val();
      }, this.setValue = function(ue) {
        se.val(ue);
      }, this.loadValue = function(ue) {
        ce = ue[oe.column.field] || "", se.val(ce), se[0].defaultValue = ce, se.select();
      }, this.serializeValue = function() {
        return se.val();
      }, this.applyValue = function(ue, de) {
        ue[oe.column.field] = de;
      }, this.isValueChanged = function() {
        return !(se.val() === "" && ce == null) && se.val() != ce;
      }, this.validate = function() {
        if (oe.column.validator) {
          var ue = oe.column.validator(se.val(), oe);
          if (!ue.valid)
            return ue;
        }
        return {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    function V(oe) {
      var se, ce, le = this;
      this.args = oe, this.init = function() {
        var ue = oe.grid.getOptions().editorCellNavOnLRKeys;
        se = O("<INPUT type=text class='editor-text' />").appendTo(oe.container).on("keydown.nav", ue ? ee : ae).focus().select(), oe.compositeEditorOptions && se.on("change", function() {
          var de = oe.grid.getActiveCell();
          le.validate().valid && le.applyValue(le.args.item, le.serializeValue()), le.applyValue(le.args.compositeEditorOptions.formValues, le.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: de.row, cell: de.cell, item: le.args.item, column: le.args.column, formValues: le.args.compositeEditorOptions.formValues });
        });
      }, this.destroy = function() {
        se.remove();
      }, this.focus = function() {
        se.focus();
      }, this.loadValue = function(ue) {
        ce = ue[oe.column.field], se.val(ce), se[0].defaultValue = ce, se.select();
      }, this.serializeValue = function() {
        return parseInt(se.val(), 10) || 0;
      }, this.applyValue = function(ue, de) {
        ue[oe.column.field] = de;
      }, this.isValueChanged = function() {
        return !(se.val() === "" && ce == null) && se.val() != ce;
      }, this.validate = function() {
        if (isNaN(se.val()))
          return {
            valid: !1,
            msg: "Please enter a valid integer"
          };
        if (oe.column.validator) {
          var ue = oe.column.validator(se.val(), oe);
          if (!ue.valid)
            return ue;
        }
        return {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    function q(oe) {
      var se, ce, le = this;
      this.args = oe, this.init = function() {
        var de = oe.grid.getOptions().editorCellNavOnLRKeys;
        se = O("<INPUT type=text class='editor-text' />").appendTo(oe.container).on("keydown.nav", de ? ee : ae).focus().select(), oe.compositeEditorOptions && se.on("change", function() {
          var fe = oe.grid.getActiveCell();
          le.validate().valid && le.applyValue(le.args.item, le.serializeValue()), le.applyValue(le.args.compositeEditorOptions.formValues, le.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: fe.row, cell: fe.cell, item: le.args.item, column: le.args.column, formValues: le.args.compositeEditorOptions.formValues });
        });
      }, this.destroy = function() {
        se.remove();
      }, this.focus = function() {
        se.focus();
      };
      function ue() {
        var de = oe.column.editorFixedDecimalPlaces;
        return typeof de > "u" && (de = q.DefaultDecimalPlaces), !de && de !== 0 ? null : de;
      }
      this.loadValue = function(de) {
        ce = de[oe.column.field];
        var fe = ue();
        fe !== null && (ce || ce === 0) && ce.toFixed && (ce = ce.toFixed(fe)), se.val(ce), se[0].defaultValue = ce, se.select();
      }, this.serializeValue = function() {
        var de = parseFloat(se.val());
        q.AllowEmptyValue ? !de && de !== 0 && (de = "") : de = de || 0;
        var fe = ue();
        return fe !== null && (de || de === 0) && de.toFixed && (de = parseFloat(de.toFixed(fe))), de;
      }, this.applyValue = function(de, fe) {
        de[oe.column.field] = fe;
      }, this.isValueChanged = function() {
        return !(se.val() === "" && ce == null) && se.val() != ce;
      }, this.validate = function() {
        if (isNaN(se.val()))
          return {
            valid: !1,
            msg: "Please enter a valid number"
          };
        if (oe.column.validator) {
          var de = oe.column.validator(se.val(), oe);
          if (!de.valid)
            return de;
        }
        return {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    q.DefaultDecimalPlaces = null, q.AllowEmptyValue = !1;
    function G(oe) {
      var se, ce, le = this, ue = !1;
      this.args = oe, this.init = function() {
        se = O("<INPUT type=text class='editor-text' />"), se.appendTo(oe.container), se.focus().select(), se.datepicker({
          showOn: "button",
          buttonImageOnly: !0,
          beforeShow: function() {
            ue = !0;
          },
          onClose: function() {
            if (ue = !1, oe.compositeEditorOptions) {
              var de = oe.grid.getActiveCell();
              le.validate().valid && le.applyValue(le.args.item, le.serializeValue()), le.applyValue(le.args.compositeEditorOptions.formValues, le.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: de.row, cell: de.cell, item: le.args.item, column: le.args.column, formValues: le.args.compositeEditorOptions.formValues });
            }
          }
        }), se.width(se.width() - (oe.compositeEditorOptions ? 28 : 18));
      }, this.destroy = function() {
        O.datepicker.dpDiv.stop(!0, !0), se.datepicker("hide"), se.datepicker("destroy"), se.remove();
      }, this.show = function() {
        ue && O.datepicker.dpDiv.stop(!0, !0).show();
      }, this.hide = function() {
        ue && O.datepicker.dpDiv.stop(!0, !0).hide();
      }, this.position = function(de) {
        ue && O.datepicker.dpDiv.css("top", de.top + 30).css("left", de.left);
      }, this.focus = function() {
        se.focus();
      }, this.loadValue = function(de) {
        ce = de[oe.column.field], se.val(ce), se[0].defaultValue = ce, se.select();
      }, this.serializeValue = function() {
        return se.val();
      }, this.applyValue = function(de, fe) {
        de[oe.column.field] = fe;
      }, this.isValueChanged = function() {
        return !(se.val() === "" && ce == null) && se.val() != ce;
      }, this.validate = function() {
        if (oe.column.validator) {
          var de = oe.column.validator(se.val(), oe);
          if (!de.valid)
            return de;
        }
        return {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    function J(oe) {
      var se, ce, le = this;
      this.args = oe, this.init = function() {
        se = O("<SELECT tabIndex='0' class='editor-yesno'><OPTION value='yes'>Yes</OPTION><OPTION value='no'>No</OPTION></SELECT>"), se.appendTo(oe.container), se.focus(), oe.compositeEditorOptions && se.on("change", function() {
          var ue = oe.grid.getActiveCell();
          le.validate().valid && le.applyValue(le.args.item, le.serializeValue()), le.applyValue(le.args.compositeEditorOptions.formValues, le.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: ue.row, cell: ue.cell, item: le.args.item, column: le.args.column, formValues: le.args.compositeEditorOptions.formValues });
        });
      }, this.destroy = function() {
        se.remove();
      }, this.focus = function() {
        se.focus();
      }, this.loadValue = function(ue) {
        se.val((ce = ue[oe.column.field]) ? "yes" : "no"), se.select();
      }, this.serializeValue = function() {
        return se.val() == "yes";
      }, this.applyValue = function(ue, de) {
        ue[oe.column.field] = de;
      }, this.isValueChanged = function() {
        return se.val() != ce;
      }, this.validate = function() {
        return {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    function Z(oe) {
      var se, ce, le = this;
      this.args = oe, this.init = function() {
        se = O("<INPUT type=checkbox value='true' class='editor-checkbox' hideFocus>"), se.appendTo(oe.container), se.focus(), oe.compositeEditorOptions && se.on("change", function() {
          var ue = oe.grid.getActiveCell();
          le.validate().valid && le.applyValue(le.args.item, le.serializeValue()), le.applyValue(le.args.compositeEditorOptions.formValues, le.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: ue.row, cell: ue.cell, item: le.args.item, column: le.args.column, formValues: le.args.compositeEditorOptions.formValues });
        });
      }, this.destroy = function() {
        se.remove();
      }, this.focus = function() {
        se.focus();
      }, this.loadValue = function(ue) {
        ce = !!ue[oe.column.field], ce ? se.prop("checked", !0) : se.prop("checked", !1);
      }, this.preClick = function() {
        se.prop("checked", !se.prop("checked"));
      }, this.serializeValue = function() {
        return se.prop("checked");
      }, this.applyValue = function(ue, de) {
        ue[oe.column.field] = de;
      }, this.isValueChanged = function() {
        return this.serializeValue() !== ce;
      }, this.validate = function() {
        return {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    function te(oe) {
      var se, ce, le, ue = this;
      this.args = oe, this.init = function() {
        se = O("<INPUT type=text class='editor-percentcomplete' />"), se.width(O(oe.container).innerWidth() - 25), se.appendTo(oe.container), ce = O("<div class='editor-percentcomplete-picker' />").appendTo(oe.container), ce.append("<div class='editor-percentcomplete-helper'><div class='editor-percentcomplete-wrapper'><div class='editor-percentcomplete-slider' /><div class='editor-percentcomplete-buttons' /></div></div>"), ce.find(".editor-percentcomplete-buttons").append("<button val=0>Not started</button><br/><button val=50>In Progress</button><br/><button val=100>Complete</button>"), se.focus().select(), ce.find(".editor-percentcomplete-slider").slider({
          orientation: "vertical",
          range: "min",
          value: le,
          slide: function(de, fe) {
            se.val(fe.value);
          },
          stop: function(de, fe) {
            if (oe.compositeEditorOptions) {
              var he = oe.grid.getActiveCell();
              ue.validate().valid && ue.applyValue(ue.args.item, ue.serializeValue()), ue.applyValue(ue.args.compositeEditorOptions.formValues, ue.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: he.row, cell: he.cell, item: ue.args.item, column: ue.args.column, formValues: ue.args.compositeEditorOptions.formValues });
            }
          }
        }), ce.find(".editor-percentcomplete-buttons button").on("click", function(de) {
          se.val(O(this).attr("val")), ce.find(".editor-percentcomplete-slider").slider("value", O(this).attr("val"));
        });
      }, this.destroy = function() {
        se.remove(), ce.remove();
      }, this.focus = function() {
        se.focus();
      }, this.loadValue = function(de) {
        se.val(le = de[oe.column.field]), se.select();
      }, this.serializeValue = function() {
        return parseInt(se.val(), 10) || 0;
      }, this.applyValue = function(de, fe) {
        de[oe.column.field] = fe;
      }, this.isValueChanged = function() {
        return !(se.val() === "" && le == null) && (parseInt(se.val(), 10) || 0) != le;
      }, this.validate = function() {
        return isNaN(parseInt(se.val(), 10)) ? {
          valid: !1,
          msg: "Please enter a valid positive number"
        } : {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    function ne(oe) {
      var se, ce, le, ue = this;
      this.args = oe, this.init = function() {
        var de = oe.compositeEditorOptions;
        oe.grid.getOptions().editorCellNavOnLRKeys;
        var fe = de ? oe.container : O("body");
        ce = O("<DIV class='slick-large-editor-text' style='z-index:10000;background:white;padding:5px;border:3px solid gray; border-radius:10px;'/>").appendTo(fe), de ? ce.css({ position: "relative", padding: 0, border: 0 }) : ce.css({ position: "absolute" }), se = O("<TEXTAREA hidefocus rows=5 style='background:white;width:250px;height:80px;border:0;outline:0'>").appendTo(ce), de ? se.on("change", function() {
          var he = oe.grid.getActiveCell();
          ue.validate().valid && ue.applyValue(ue.args.item, ue.serializeValue()), ue.applyValue(ue.args.compositeEditorOptions.formValues, ue.serializeValue()), oe.grid.onCompositeEditorChange.notify({ row: he.row, cell: he.cell, item: ue.args.item, column: ue.args.column, formValues: ue.args.compositeEditorOptions.formValues });
        }) : (O("<DIV style='text-align:right'><BUTTON>Save</BUTTON><BUTTON>Cancel</BUTTON></DIV>").appendTo(ce), ce.find("button:first").on("click", this.save), ce.find("button:last").on("click", this.cancel), se.on("keydown", this.handleKeyDown), ue.position(oe.position)), se.focus().select();
      }, this.handleKeyDown = function(de) {
        if (de.which == _.keyCode.ENTER && de.ctrlKey)
          ue.save();
        else if (de.which == _.keyCode.ESCAPE)
          de.preventDefault(), ue.cancel();
        else if (de.which == _.keyCode.TAB && de.shiftKey)
          de.preventDefault(), oe.grid.navigatePrev();
        else if (de.which == _.keyCode.TAB)
          de.preventDefault(), oe.grid.navigateNext();
        else if ((de.which == _.keyCode.LEFT || de.which == _.keyCode.RIGHT) && oe.grid.getOptions().editorCellNavOnLRKeys) {
          var fe = this.selectionStart, he = this.value.length;
          de.keyCode === _.keyCode.LEFT && fe === 0 && oe.grid.navigatePrev(), de.keyCode === _.keyCode.RIGHT && fe >= he - 1 && oe.grid.navigateNext();
        }
      }, this.save = function() {
        oe.commitChanges();
      }, this.cancel = function() {
        se.val(le), oe.cancelChanges();
      }, this.hide = function() {
        ce.hide();
      }, this.show = function() {
        ce.show();
      }, this.position = function(de) {
        ce.css("top", de.top - 5).css("left", de.left - 5);
      }, this.destroy = function() {
        ce.remove();
      }, this.focus = function() {
        se.focus();
      }, this.loadValue = function(de) {
        se.val(le = de[oe.column.field]), se.select();
      }, this.serializeValue = function() {
        return se.val();
      }, this.applyValue = function(de, fe) {
        de[oe.column.field] = fe;
      }, this.isValueChanged = function() {
        return !(se.val() === "" && le == null) && se.val() != le;
      }, this.validate = function() {
        if (oe.column.validator) {
          var de = oe.column.validator(se.val(), oe);
          if (!de.valid)
            return de;
        }
        return {
          valid: !0,
          msg: null
        };
      }, this.init();
    }
    function ee(oe) {
      var se = this.selectionStart, ce = this.value.length;
      (oe.keyCode === _.keyCode.LEFT && se > 0 || oe.keyCode === _.keyCode.RIGHT && se < ce - 1) && oe.stopImmediatePropagation();
    }
    function ae(oe) {
      (oe.keyCode === _.keyCode.LEFT || oe.keyCode === _.keyCode.RIGHT) && oe.stopImmediatePropagation();
    }
    return slick_editors = {
      Editors: {
        Text: H,
        Integer: V,
        Float: q,
        Date: G,
        YesNoSelect: J,
        Checkbox: Z,
        PercentComplete: te,
        LongText: ne
      }
    }, slick_editors;
  }
  var slick_formatters, hasRequiredSlick_formatters;
  function requireSlick_formatters() {
    if (hasRequiredSlick_formatters) return slick_formatters;
    hasRequiredSlick_formatters = 1, requireSlick_core();
    function O(G, J, Z, te, ne) {
      return Z == null || Z === "" ? "-" : Z < 50 ? "<span style='color:red;font-weight:bold;'>" + Z + "%</span>" : "<span style='color:green'>" + Z + "%</span>";
    }
    function _(G, J, Z, te, ne) {
      if (Z == null || Z === "")
        return "";
      var ee;
      return Z < 30 ? ee = "red" : Z < 70 ? ee = "silver" : ee = "green", "<span class='percent-complete-bar' style='background:" + ee + ";width:" + Z + "%'></span>";
    }
    function H(G, J, Z, te, ne) {
      return Z ? "Yes" : "No";
    }
    function V(G, J, Z, te, ne) {
      return '<img class="slick-edit-preclick" src="../images/' + (Z ? "CheckboxY" : "CheckboxN") + '.png">';
    }
    function q(G, J, Z, te, ne) {
      return Z ? "<img src='../images/tick.png'>" : "";
    }
    return slick_formatters = {
      Formatters: {
        PercentComplete: O,
        PercentCompleteBar: _,
        YesNo: H,
        Checkmark: q,
        Checkbox: V
      }
    }, slick_formatters;
  }
  var slick_remotemodel, hasRequiredSlick_remotemodel;
  function requireSlick_remotemodel() {
    if (hasRequiredSlick_remotemodel) return slick_remotemodel;
    hasRequiredSlick_remotemodel = 1;
    var O = requireSlick_jquery(), _ = requireSlick_core();
    function H() {
      var V = 50, q = { length: 0 }, G = "", J = null, Z = 1, te = null, ne = null, ee = new _.Event(), ae = new _.Event();
      function oe(me, pe) {
        for (var ye = me; ye <= pe; ye++)
          if (q[ye] == null || q[ye] == null)
            return !1;
        return !0;
      }
      function se() {
        for (var me in q)
          delete q[me];
        q.length = 0;
      }
      function ce(me, pe) {
        if (ne) {
          ne.abort();
          for (var ye = ne.fromPage; ye <= ne.toPage; ye++)
            q[ye * V] = void 0;
        }
        me < 0 && (me = 0), q.length > 0 && (pe = Math.min(pe, q.length - 1));
        for (var ge = Math.floor(me / V), ve = Math.floor(pe / V); q[ge * V] !== void 0 && ge < ve; )
          ge++;
        for (; q[ve * V] !== void 0 && ge < ve; )
          ve--;
        if (ge > ve || ge == ve && q[ge * V] !== void 0) {
          ae.notify({ from: me, to: pe });
          return;
        }
        var be = "http://octopart.com/api/v3/parts/search?apikey=68b25f31&include[]=short_description&show[]=uid&show[]=manufacturer&show[]=mpn&show[]=brand&show[]=octopart_url&show[]=short_description&q=" + G + "&start=" + ge * V + "&limit=" + ((ve - ge) * V + V);
        J != null && (be += "&sortby=" + J + (Z > 0 ? "+asc" : "+desc")), te != null && clearTimeout(te), te = setTimeout(function() {
          for (var Ce = ge; Ce <= ve; Ce++)
            q[Ce * V] = null;
          ee.notify({ from: me, to: pe }), ne = O.jsonp({
            url: be,
            callbackParameter: "callback",
            cache: !0,
            success: ue,
            error: function() {
              le(ge, ve);
            }
          }), ne.fromPage = ge, ne.toPage = ve;
        }, 50);
      }
      function le(me, pe) {
        alert("error loading pages " + me + " to " + pe);
      }
      function ue(me) {
        var pe = me.request.start, ye = pe + me.results.length;
        q.length = Math.min(parseInt(me.hits), 1e3);
        for (var ge = 0; ge < me.results.length; ge++) {
          var ve = me.results[ge].item;
          q[pe + ge] = ve, q[pe + ge].index = pe + ge;
        }
        ne = null, ae.notify({ from: pe, to: ye });
      }
      function de(me, pe) {
        for (var ye = me; ye <= pe; ye++)
          delete q[ye];
        ce(me, pe);
      }
      function fe(me, pe) {
        J = me, Z = pe, se();
      }
      function he(me) {
        G = me, se();
      }
      return {
        // properties
        data: q,
        // methods
        clear: se,
        isDataLoaded: oe,
        ensureData: ce,
        reloadData: de,
        setSort: fe,
        setSearch: he,
        // events
        onDataLoading: ee,
        onDataLoaded: ae
      };
    }
    return slick_remotemodel = {
      RemoteModel: H
    }, slick_remotemodel;
  }
  var slick_groupitemmetadataprovider, hasRequiredSlick_groupitemmetadataprovider;
  function requireSlick_groupitemmetadataprovider() {
    if (hasRequiredSlick_groupitemmetadataprovider) return slick_groupitemmetadataprovider;
    hasRequiredSlick_groupitemmetadataprovider = 1;
    var O = requireSlick_jquery(), _ = requireSlick_core();
    function H(V) {
      var q, G = {
        checkboxSelect: !1,
        checkboxSelectCssClass: "slick-group-select-checkbox",
        checkboxSelectPlugin: null,
        groupCssClass: "slick-group",
        groupTitleCssClass: "slick-group-title",
        totalsCssClass: "slick-group-totals",
        groupFocusable: !0,
        totalsFocusable: !1,
        toggleCssClass: "slick-group-toggle",
        toggleExpandedCssClass: "expanded",
        toggleCollapsedCssClass: "collapsed",
        enableExpandCollapse: !0,
        groupFormatter: ne,
        totalsFormatter: ee,
        includeHeaderTotals: !1
      }, J = O.extend(!0, {}, G, V);
      function Z() {
        return J;
      }
      function te(de) {
        O.extend(!0, J, de);
      }
      function ne(de, fe, he, me, pe, ye) {
        if (!J.enableExpandCollapse)
          return pe.title;
        var ge = pe.level * 15 + "px";
        return (J.checkboxSelect ? '<span class="' + J.checkboxSelectCssClass + " " + (pe.selectChecked ? "checked" : "unchecked") + '"></span>' : "") + "<span class='" + J.toggleCssClass + " " + (pe.collapsed ? J.toggleCollapsedCssClass : J.toggleExpandedCssClass) + "' style='margin-left:" + ge + "'></span><span class='" + J.groupTitleCssClass + "' level='" + pe.level + "'>" + pe.title + "</span>";
      }
      function ee(de, fe, he, me, pe, ye) {
        return me.groupTotalsFormatter && me.groupTotalsFormatter(pe, me, ye) || "";
      }
      function ae(de) {
        q = de, q.onClick.subscribe(se), q.onKeyDown.subscribe(ce);
      }
      function oe() {
        q && (q.onClick.unsubscribe(se), q.onKeyDown.unsubscribe(ce));
      }
      function se(de, fe) {
        var he = O(de.target), me = this.getDataItem(fe.row);
        if (me && me instanceof _.Group && he.hasClass(J.toggleCssClass)) {
          var pe = q.getRenderedRange();
          this.getData().setRefreshHints({
            ignoreDiffsBefore: pe.top,
            ignoreDiffsAfter: pe.bottom + 1
          }), me.collapsed ? this.getData().expandGroup(me.groupingKey) : this.getData().collapseGroup(me.groupingKey), de.stopImmediatePropagation(), de.preventDefault();
        }
        if (me && me instanceof _.Group && he.hasClass(J.checkboxSelectCssClass)) {
          me.selectChecked = !me.selectChecked, he.removeClass(me.selectChecked ? "unchecked" : "checked"), he.addClass(me.selectChecked ? "checked" : "unchecked");
          var ye = q.getData().mapItemsToRows(me.rows);
          (me.selectChecked ? J.checkboxSelectPlugin.selectRows : J.checkboxSelectPlugin.deSelectRows)(ye);
        }
      }
      function ce(de, fe) {
        if (J.enableExpandCollapse && de.which == _.keyCode.SPACE) {
          var he = this.getActiveCell();
          if (he) {
            var me = this.getDataItem(he.row);
            if (me && me instanceof _.Group) {
              var pe = q.getRenderedRange();
              this.getData().setRefreshHints({
                ignoreDiffsBefore: pe.top,
                ignoreDiffsAfter: pe.bottom + 1
              }), me.collapsed ? this.getData().expandGroup(me.groupingKey) : this.getData().collapseGroup(me.groupingKey), de.stopImmediatePropagation(), de.preventDefault();
            }
          }
        }
      }
      function le(de) {
        var fe = de && de.level;
        return {
          selectable: !1,
          focusable: J.groupFocusable,
          cssClasses: J.groupCssClass + " slick-group-level-" + fe,
          formatter: J.includeHeaderTotals && J.totalsFormatter,
          columns: {
            0: {
              colspan: J.includeHeaderTotals ? "1" : "*",
              formatter: J.groupFormatter,
              editor: null
            }
          }
        };
      }
      function ue(de) {
        var fe = de && de.group && de.group.level;
        return {
          selectable: !1,
          focusable: J.totalsFocusable,
          cssClasses: J.totalsCssClass + " slick-group-level-" + fe,
          formatter: J.totalsFormatter,
          editor: null
        };
      }
      return {
        init: ae,
        destroy: oe,
        getGroupRowMetadata: le,
        getTotalsRowMetadata: ue,
        getOptions: Z,
        setOptions: te
      };
    }
    return slick_groupitemmetadataprovider = {
      GroupItemMetadataProvider: H
    }, slick_groupitemmetadataprovider;
  }
  var hasRequiredSlickgrid;
  function requireSlickgrid() {
    return hasRequiredSlickgrid || (hasRequiredSlickgrid = 1, function(O) {
      var _ = require$$0;
      _.__exportStar(requireSlick_core(), O.exports), _.__exportStar(requireSlick_grid(), O.exports), _.__exportStar(requireSlick_dataview(), O.exports), _.__exportStar(requireSlick_editors(), O.exports), _.__exportStar(requireSlick_formatters(), O.exports), _.__exportStar(requireSlick_remotemodel(), O.exports), _.__exportStar(requireSlick_groupitemmetadataprovider(), O.exports);
    }(slickgrid)), slickgrid.exports;
  }
  var slickgridExports = requireSlickgrid();
  const Y6 = class Y6 extends Widget {
    constructor(_) {
      super(_);
    }
  };
  mt(Y6, "__name__", "TableWidget"), Y6.define(({ Ref: _ }) => ({
    source: [_(ColumnDataSource), () => new ColumnDataSource()],
    view: [_(CDSView), () => new CDSView()]
  }));
  let TableWidget = Y6;
  const X6 = class X6 extends Model$1 {
    constructor(_) {
      super(_);
    }
    toColumn() {
      return {
        id: unique_id(),
        field: this.field,
        name: this.title ?? this.field,
        width: this.width,
        formatter: this.formatter.doFormat.bind(this.formatter),
        model: this.editor,
        editor: this.editor.default_view,
        sortable: this.sortable,
        defaultSortAsc: this.default_sort == "ascending"
      };
    }
  };
  mt(X6, "__name__", "TableColumn"), X6.define(({ Bool: _, Float: H, Str: V, Nullable: q, Ref: G }) => ({
    field: [V],
    title: [q(V), null],
    width: [H, 300],
    formatter: [G(CellFormatter), () => new StringFormatter()],
    editor: [G(CellEditor), () => new StringEditor()],
    sortable: [_, !0],
    default_sort: [Sort, "ascending"],
    visible: [_, !0]
  }));
  let TableColumn = X6;
  const slickgrid_css = `.slick-header.ui-state-default,.slick-headerrow.ui-state-default,.slick-footerrow.ui-state-default,.slick-top-panel-scroller.ui-state-default,.slick-group-header.ui-state-default{width:100%;overflow:auto;position:relative;border-left:0px !important;}.slick-header.ui-state-default{overflow:inherit;}.slick-header::-webkit-scrollbar,.slick-headerrow::-webkit-scrollbar,.slick-footerrow::-webkit-scrollbar{display:none;}.slick-header-columns,.slick-headerrow-columns,.slick-footerrow-columns,.slick-group-header-columns{position:relative;white-space:nowrap;cursor:default;overflow:hidden;}.slick-header-column.ui-state-default,.slick-group-header-column.ui-state-default{position:relative;display:inline-block;box-sizing:content-box !important;overflow:hidden;-o-text-overflow:ellipsis;text-overflow:ellipsis;height:16px;line-height:16px;margin:0;padding:4px;border-right:1px solid silver;border-left:0px !important;border-top:0px !important;border-bottom:0px !important;float:left;}.slick-footerrow-column.ui-state-default{-o-text-overflow:ellipsis;text-overflow:ellipsis;margin:0;padding:4px;border-right:1px solid silver;border-left:0px;border-top:0px;border-bottom:0px;float:left;line-height:20px;vertical-align:middle;}.slick-headerrow-column.ui-state-default,.slick-footerrow-column.ui-state-default{padding:4px;}.slick-header-column-sorted{font-style:italic;}.slick-sort-indicator{display:inline-block;width:8px;height:5px;margin-left:4px;margin-top:6px;float:left;}.slick-sort-indicator-numbered{display:inline-block;width:8px;height:5px;margin-left:4px;margin-top:0;line-height:20px;float:left;font-family:Arial;font-style:normal;font-weight:bold;color:#6190CD;}.slick-sort-indicator-desc{background:url(images/sort-desc.gif);}.slick-sort-indicator-asc{background:url(images/sort-asc.gif);}.slick-resizable-handle{position:absolute;font-size:0.1px;display:block;cursor:col-resize;width:9px;right:-5px;top:0;height:100%;z-index:1;}.slick-sortable-placeholder{background:silver;}.grid-canvas{position:relative;outline:0;}.slick-row.ui-widget-content,.slick-row.ui-state-active{position:absolute;border:0px;width:100%;}.slick-cell,.slick-headerrow-column,.slick-footerrow-column{position:absolute;border:1px solid transparent;border-right:1px dotted silver;border-bottom-color:silver;overflow:hidden;-o-text-overflow:ellipsis;text-overflow:ellipsis;vertical-align:middle;z-index:1;padding:1px 2px 2px 1px;margin:0;white-space:nowrap;cursor:default;}.slick-cell,.slick-headerrow-column{border-bottom-color:silver;}.slick-footerrow-column{border-top-color:silver;}.slick-group-toggle{display:inline-block;}.slick-cell.highlighted{background:lightskyblue;background:rgba(0, 0, 255, 0.2);-webkit-transition:all 0.5s;-moz-transition:all 0.5s;-o-transition:all 0.5s;transition:all 0.5s;}.slick-cell.flashing{border:1px solid red !important;}.slick-cell.editable{z-index:11;overflow:visible;background:white;border-color:black;border-style:solid;}.slick-cell:focus{outline:none;}.slick-reorder-proxy{display:inline-block;background:blue;opacity:0.15;cursor:move;}.slick-reorder-guide{display:inline-block;height:2px;background:blue;opacity:0.7;}.slick-selection{z-index:10;position:absolute;border:2px dashed black;}.slick-pane{position:absolute;outline:0;overflow:hidden;width:100%;}.slick-pane-header{display:block;}.slick-header{overflow:hidden;position:relative;}.slick-headerrow{overflow:hidden;position:relative;}.slick-top-panel-scroller{overflow:hidden;position:relative;}.slick-top-panel{width:10000px;}.slick-viewport{position:relative;outline:0;width:100%;}.slick-header-columns{background:url('images/header-columns-bg.gif') repeat-x center bottom;border-bottom:1px solid silver;}.slick-header-column{background:url('images/header-columns-bg.gif') repeat-x center bottom;border-right:1px solid silver;}.slick-header-column:hover,.slick-header-column-active{background:white url('images/header-columns-over-bg.gif') repeat-x center bottom;}.slick-headerrow{background:#fafafa;}.slick-headerrow-column{background:#fafafa;border-bottom:0;height:100%;}.slick-row.ui-state-active{background:#F5F7D7;}.slick-row{position:absolute;background:white;border:0px;line-height:20px;}.slick-row.selected{z-index:10;background:#DFE8F6;}.slick-cell{padding-left:4px;padding-right:4px;}.slick-group{border-bottom:2px solid silver;}.slick-group-toggle{width:9px;height:9px;margin-right:5px;}.slick-group-toggle.expanded{background:url(images/collapse.gif) no-repeat center center;}.slick-group-toggle.collapsed{background:url(images/expand.gif) no-repeat center center;}.slick-group-totals{color:gray;background:white;}.slick-group-select-checkbox{width:13px;height:13px;margin:3px 10px 0 0;display:inline-block;}.slick-group-select-checkbox.checked{background:url(images/GrpCheckboxY.png) no-repeat center center;}.slick-group-select-checkbox.unchecked{background:url(images/GrpCheckboxN.png) no-repeat center center;}.slick-cell.selected{background-color:beige;}.slick-cell.active{border-color:gray;border-style:solid;}.slick-sortable-placeholder{background:silver !important;}.slick-row.odd{background:#fafafa;}.slick-row.ui-state-active{background:#F5F7D7;}.slick-row.loading{opacity:0.5;}.slick-cell.invalid{border-color:red;-moz-animation-duration:0.2s;-webkit-animation-duration:0.2s;-moz-animation-name:slickgrid-invalid-hilite;-webkit-animation-name:slickgrid-invalid-hilite;}@-moz-keyframes slickgrid-invalid-hilite{from{box-shadow:0 0 6px red;}to{box-shadow:none;}}@-webkit-keyframes slickgrid-invalid-hilite{from{box-shadow:0 0 6px red;}to{box-shadow:none;}}.slick-column-name,.slick-sort-indicator{display:inline-block;float:left;margin-bottom:100px;}.slick-header-button{display:inline-block;float:right;vertical-align:top;margin:1px;margin-bottom:100px;height:15px;width:15px;background-repeat:no-repeat;background-position:center center;cursor:pointer;}.slick-header-button-hidden{width:0;-webkit-transition:0.2s width;-ms-transition:0.2s width;transition:0.2s width;}.slick-header-column:hover > .slick-header-button{width:15px;}.slick-header-menubutton{position:absolute;right:0;top:0;bottom:0;width:14px;background-repeat:no-repeat;background-position:left center;background-image:url(../images/down.gif);cursor:pointer;display:none;border-left:thin ridge silver;}.slick-header-column:hover > .slick-header-menubutton,.slick-header-column-active .slick-header-menubutton{display:inline-block;}.slick-header-menu{position:absolute;display:inline-block;margin:0;padding:2px;cursor:default;}.slick-header-menuitem{list-style:none;margin:0;padding:0;cursor:pointer;display:block;}.slick-header-menuicon{display:inline-block;width:16px;height:16px;vertical-align:middle;margin-right:4px;background-repeat:no-repeat;background-position:center center;}.slick-header-menucontent{display:inline-block;vertical-align:middle;}.slick-header-menuitem-disabled{color:silver;}.slick-header-menuitem-hidden{display:none;}.slick-header-menuitem.slick-header-menuitem-divider{cursor:default;border:none;overflow:hidden;padding:0;height:1px;margin:8px 2px;background-color:#cecece;}.slick-header-menuitem-divider.slick-header-menuitem:hover{background-color:#cecece;}.slick-columnpicker{border:1px solid #718BB7;background:#f0f0f0;padding:6px;-moz-box-shadow:2px 2px 2px silver;-webkit-box-shadow:2px 2px 2px silver;box-shadow:2px 2px 2px silver;min-width:150px;cursor:default;position:absolute;z-index:20;overflow:auto;resize:both;}.slick-columnpicker > .close{float:right;}.slick-columnpicker .title{font-size:16px;width:60%;border-bottom:solid 1px #d6d6d6;margin-bottom:10px;}.slick-columnpicker li{list-style:none;margin:0;padding:0;background:none;}.slick-columnpicker input{margin:4px;}.slick-columnpicker li a{display:block;padding:4px;font-weight:bold;}.slick-columnpicker li a:hover{background:white;}.slick-columnpicker-list li.hidden{display:none;}.slick-pager{width:100%;height:26px;border:1px solid gray;border-top:0;background:url('../images/header-columns-bg.gif') repeat-x center bottom;vertical-align:middle;}.slick-pager .slick-pager-status{display:inline-block;padding:6px;}.slick-pager .ui-icon-container{display:inline-block;margin:2px;border-color:gray;}.slick-pager .slick-pager-nav{display:inline-block;float:left;padding:2px;}.slick-pager .slick-pager-settings{display:block;float:right;padding:2px;}.slick-pager .slick-pager-settings *{vertical-align:middle;}.slick-pager .slick-pager-settings a{padding:2px;text-decoration:underline;cursor:pointer;}.slick-header-columns{border-bottom:1px solid silver;background-image:none;}.slick-header-column{border-right:1px solid transparent;background-image:none;}.slick-header-column:last-of-type{border-right-color:transparent;}.slick-header-column:hover,.slick-header-column-active{background-color:#F0F8FF;background-image:none;}.slick-header-columns{background-image:none;}.slick-header-column{background-image:none;}.slick-header-column:hover,.slick-header-column-active{background-image:none;}.slick-group-toggle.expanded{background-image:url("data:image/gif;base64,R0lGODlhCQAJAPcAAAFGeoCAgNXz/+v5/+v6/+z5/+36//L7//X8//j9/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACQAJAAAIMwADCBxIUIDBgwIEChgwwECBAgQUFjBAkaJCABgxGlB4AGHCAAIQiBypEEECkScJqgwQEAA7");}.slick-group-toggle.collapsed{background-image:url("data:image/gif;base64,R0lGODlhCQAJAPcAAAFGeoCAgNXz/+v5/+v6/+z5/+36//L7//X8//j9/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAACQAJAAAIOAADCBxIUIDBgwIEChgwAECBAgQUFjAAQIABAwoBaNSIMYCAAwIqGlSIAEHFkiQTIBCgkqDLAAEBADs=");}.slick-group-select-checkbox.checked{background-image:url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA4AAAAOCAIAAACQKrqGAAAABGdBTUEAALGPC/xhBQAAAAlwSFlzAAAOwQAADsEBuJFr7QAAABl0RVh0U29mdHdhcmUAcGFpbnQubmV0IDQuMC4xNkRpr/UAAAEcSURBVChTjdI9S8NQFAbg/raQXVwCRRFE7GK7OXTwD+ikk066VF3a0ja0hQTyQdJrwNq0zrYSQRLEXMSWSlCIb8glqRcFD+9yz3nugXwU4n9XQqMoGjj36uBJsTwuaNo3EwBG4Yy7pe7Gv8YcvhJCGFVsjxsjxujj6OTSGlHv+U2WZUZbPWKOv1ZjT5a7pbIoiptbO5b73mwrjHa1B27l8VlTEIS1damlTnEE+EEN9/P8WrfH81qdAIGeXvTTmzltdCy46sEhxpKUINReZR9NnqZbr9puugxV3NjWh/k74WmmEdWhmUNy2jNmWRc6fZTVADCqao52u+DGWTACYNT3fRxwtatPufTNR4yCIGAUn5hS+vJHhWGY/ANx/A3tvdv+1tZmuwAAAABJRU5ErkJggg==");}.slick-group-select-checkbox.unchecked{background-image:url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA4AAAAOCAIAAACQKrqGAAAABGdBTUEAALGPC/xhBQAAAAlwSFlzAAAOwQAADsEBuJFr7QAAABl0RVh0U29mdHdhcmUAcGFpbnQubmV0IDQuMC4xNkRpr/UAAACXSURBVChT1dIxC4MwEAXg/v8/VOhQVDBNakV0KA6pxS4JhWRSIYPEJxwdDi1de7wleR+3JIf486w0hKCKRpSvvOhZcCmvNQBRuKqdah03U7UjNNH81rOaBYDo8SQaPX8JANFEaLaGBeAPaaY61rGksiN6TmR5H1j9CSoAosYYHLA7vTxYMvVEZa0liif23r93xjm3/oEYF8PiDn/I2FHCAAAAAElFTkSuQmCC");}.slick-sort-indicator-desc{background-image:url("data:image/gif;base64,R0lGODlhDQAFAIcAAGGQzUD/QOPu+wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAMAAAEALAAAAAANAAUAAAgeAAUAGEgQgIAACBEKLHgwYcKFBh1KFNhQosOKEgMCADs=");}.slick-sort-indicator-asc{background-image:url("data:image/gif;base64,R0lGODlhDQAFAIcAAGGQzUD/QOPu+wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAMAAAEALAAAAAANAAUAAAgbAAMIDABgoEGDABIeRJhQ4cKGEA8KmEiRosGAADs=");}.slick-header-menubutton{background-image:url("data:image/gif;base64,R0lGODlhDgAOAIABADtKYwAAACH5BAEAAAEALAAAAAAOAA4AAAISjI+py+0PHZgUsGobhTn6DxoFADs=");}.detailView-toggle.expand{background-image:url("data:image/gif;base64,R0lGODlhBQANAHAAACH5BAEAAAMALAAAAAAFAA0AgePu+wAAAGGQzQAAAAIQxD4gqJsCHxRttaMY2rz3AgA7");}.detailView-toggle.collapse{background-image:url("data:image/gif;base64,R0lGODlhDQAFAIcAAGGQzUD/QOPu+wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAMAAAEALAAAAAANAAUAAAgeAAUAGEgQgIAACBEKLHgwYcKFBh1KFNhQosOKEgMCADs=");}.slick-pager{background-image:none;}`, AutosizeModes = {
    fit_columns: "FCV",
    fit_viewport: "FVC",
    force_fit: "LFF",
    none: "NOA"
  };
  let _warned_not_reorderable = !1;
  class TableDataProvider {
    constructor(_, H) {
      mt(this, "index");
      mt(this, "source");
      mt(this, "view");
      this.init(_, H);
    }
    init(_, H) {
      if (DTINDEX_NAME in _.data)
        throw new Error(`special name ${DTINDEX_NAME} cannot be used as a data table column`);
      this.source = _, this.view = H, this.index = [...this.view.indices];
    }
    getLength() {
      return this.index.length;
    }
    getItem(_) {
      const H = {}, V = dict(this.source.data);
      for (const [q, G] of V) {
        const J = this.index[_], Z = is_NDArray(G) ? G.get(J) : G[J];
        H[q] = Z;
      }
      return H[DTINDEX_NAME] = this.index[_], H;
    }
    getField(_, H) {
      if (H == DTINDEX_NAME)
        return this.index[_];
      {
        const q = dict(this.source.data).get(H) ?? [], G = this.index[_];
        return is_NDArray(q) ? q.get(G) : q[G];
      }
    }
    setField(_, H, V) {
      const q = this.index[_], G = /* @__PURE__ */ new Map([
        [H, [[q, V]]]
      ]);
      this.source.patch(G);
    }
    getRecords() {
      return range$9(0, this.getLength()).map((_) => this.getItem(_));
    }
    getItems() {
      return this.getRecords();
    }
    slice(_, H, V = 1) {
      return H = H ?? this.getLength(), range$9(_, H, V).map((q) => this.getItem(q));
    }
    sort(_) {
      let H = _.map((G) => [G.sortCol.field, G.sortAsc ? 1 : -1]);
      H.length == 0 && (H = [[DTINDEX_NAME, 1]]);
      const V = this.getRecords(), q = this.index.slice();
      this.index.sort((G, J) => {
        for (const [Z, te] of H) {
          const ne = V[q.indexOf(G)][Z], ee = V[q.indexOf(J)][Z];
          if (ne !== ee) {
            if (isNumber$3(ne) && isNumber$3(ee))
              return te * (ne - ee || +isNaN(ne) - +isNaN(ee));
            {
              const ae = `${ne}`.localeCompare(`${ee}`);
              if (ae == 0)
                continue;
              return te * ae;
            }
          }
        }
        return 0;
      });
    }
  }
  mt(TableDataProvider, "__name__", "TableDataProvider");
  class DataTableView extends WidgetView$1 {
    constructor() {
      super(...arguments);
      mt(this, "cds_view");
      mt(this, "data");
      mt(this, "grid");
      mt(this, "_in_selection_update", !1);
      mt(this, "_width", null);
      mt(this, "_filtered_selection", []);
      mt(this, "wrapper_el");
    }
    get data_source() {
      return this.model.properties.source;
    }
    *children() {
      yield* super.children(), yield this.cds_view;
    }
    async lazy_initialize() {
      await super.lazy_initialize(), this.cds_view = await build_view(this.model.view, { parent: this });
    }
    remove() {
      this.cds_view.remove(), this.grid.destroy(), super.remove();
    }
    connect_signals() {
      super.connect_signals(), this.connect(this.model.change, () => {
        this.render(), this.after_render(), this.invalidate_layout();
      });
      for (const H of this.model.columns)
        this.connect(H.change, () => {
          this.render(), this.after_render(), this.invalidate_layout();
        });
      this.connect(this.model.view.change, () => this.updateGrid()), this.connect(this.model.source.selected.change, () => this.updateSelection()), this.connect(this.model.source.selected.properties.indices.change, () => this.updateSelection());
    }
    stylesheets() {
      return [...super.stylesheets(), slickgrid_css, tables_css];
    }
    _after_resize() {
      super._after_resize(), this.grid.resizeCanvas(), this.updateLayout(!0, !1);
    }
    _after_layout() {
      super._after_layout(), this.grid.resizeCanvas(), this.updateLayout(!0, !1);
    }
    box_sizing() {
      const H = super.box_sizing();
      return this.model.autosize_mode === "fit_viewport" && this._width != null && (H.width = this._width), H;
    }
    updateLayout(H, V) {
      const q = this.autosize;
      q === AutosizeModes.fit_columns || q === AutosizeModes.force_fit ? (H || this.grid.resizeCanvas(), this.grid.autosizeColumns()) : H && V && q === AutosizeModes.fit_viewport && this.invalidate_layout();
    }
    updateGrid() {
      if (this.data.init(this.model.source, this.model.view), this.model.sortable) {
        const H = this.grid.getColumns(), V = this.grid.getSortColumns().map((q) => ({
          sortCol: {
            field: H[this.grid.getColumnIndex(q.columnId)].field
          },
          sortAsc: q.sortAsc
        }));
        this.data.sort(V);
      }
      this._sync_selected_with_view(), this.updateSelection(), this.grid.invalidate(), this.updateLayout(!0, !0);
    }
    updateSelection() {
      if (this.model.selectable === !1 || this._in_selection_update)
        return;
      const { indices: H } = this.model.source.selected, V = sort_by(map$4(H, (J) => this.data.index.indexOf(J)), (J) => J);
      this._in_selection_update = !0;
      try {
        this.grid.setSelectedRows([...V]);
      } finally {
        this._in_selection_update = !1;
      }
      const q = this.grid.getViewport(), G = this.model.get_scroll_index(q, V);
      G != null && this.grid.scrollRowToTop(G);
    }
    newIndexColumn() {
      return {
        id: unique_id(),
        name: this.model.index_header,
        field: DTINDEX_NAME,
        width: this.model.index_width,
        behavior: "select",
        cannotTriggerInsert: !0,
        resizable: !1,
        selectable: !1,
        sortable: !0,
        cssClass: cell_index,
        headerCssClass: header_index
      };
    }
    get autosize() {
      let H;
      return this.model.fit_columns === !0 ? H = AutosizeModes.force_fit : this.model.fit_columns === !1 ? H = AutosizeModes.none : H = AutosizeModes[this.model.autosize_mode], H;
    }
    render() {
      super.render(), this.wrapper_el = div({ class: data_table }), this.shadow_el.appendChild(this.wrapper_el);
    }
    _after_render() {
      super._after_render();
      const H = this.model.columns.filter((oe) => oe.visible).map((oe) => ({ ...oe.toColumn(), parent: this }));
      let V = null;
      if (this.model.selectable == "checkbox" && (V = new slick_checkboxselectcolumnExports.CheckboxSelectColumn({ cssClass: cell_select }), H.unshift(V.getColumnDefinition())), this.model.index_position != null) {
        const oe = this.model.index_position, se = this.newIndexColumn();
        oe == -1 ? H.push(se) : oe < -1 ? H.splice(oe + 1, 0, se) : H.splice(oe, 0, se);
      }
      let { reorderable: q } = this.model;
      q && !(typeof $ < "u" && typeof $.fn < "u" && "sortable" in $.fn) && (_warned_not_reorderable || (logger$1.warn("jquery-ui is required to enable DataTable.reorderable"), _warned_not_reorderable = !0), q = !1);
      let G = -1, J = !1;
      const { frozen_rows: Z, frozen_columns: te } = this.model, ne = te == null ? -1 : te - 1;
      Z != null && (J = Z < 0, G = Math.abs(Z));
      const ee = {
        enableCellNavigation: this.model.selectable !== !1,
        enableColumnReorder: q,
        autosizeColsMode: this.autosize,
        multiColumnSort: this.model.sortable,
        editable: this.model.editable,
        autoEdit: this.model.auto_edit,
        autoHeight: !1,
        rowHeight: this.model.row_height,
        frozenColumn: ne,
        frozenRow: G,
        frozenBottom: J,
        explicitInitialization: !1
      }, ae = is_defined(this.grid);
      if (this.data = new TableDataProvider(this.model.source, this.model.view), this.grid = new slickgridExports.Grid(this.wrapper_el, this.data, H, ee), this.autosize == AutosizeModes.fit_viewport) {
        this.grid.autosizeColumns();
        let oe = 0;
        for (const se of H)
          oe += se.width ?? 0;
        this._width = Math.ceil(oe);
      }
      if (this.grid.onSort.subscribe((oe, se) => {
        if (!this.model.sortable)
          return;
        const ce = se.sortCols;
        ce != null && (this.data.sort(ce), this.grid.invalidate(), this.updateSelection(), this.grid.render(), this.model.header_row || this._hide_header(), this.model.update_sort_columns(ce));
      }), this.model.selectable !== !1) {
        this.grid.setSelectionModel(new slick_rowselectionmodelExports.RowSelectionModel({ selectActiveRow: V == null })), V != null && this.grid.registerPlugin(V);
        const oe = {
          dataItemColumnValueExtractor(se, ce) {
            let le = se[ce.field];
            return isString$2(le) && (le = le.replace(/\n/g, "\\n")), le;
          },
          includeHeaderWhenCopying: !1
        };
        this.grid.registerPlugin(new slick_cellexternalcopymanagerExports.CellExternalCopyManager(oe)), this.grid.onSelectedRowsChanged.subscribe((se, ce) => {
          this._in_selection_update || (this.model.source.selected.indices = ce.rows.map((le) => this.data.index[le]));
        }), this.updateSelection(), this.model.header_row || this._hide_header();
      }
      this.updateLayout(ae, !1);
    }
    _hide_header() {
      for (const H of this.shadow_el.querySelectorAll(".slick-header-columns"))
        H.style.height = "0px";
      this.grid.resizeCanvas();
    }
    get_selected_rows() {
      return this.grid.getSelectedRows();
    }
    _sync_selected_with_view() {
      const H = this.data.view.indices, { source: V } = this.data, q = filter$2(V.selected.indices, (J) => H.get(J)), G = new Set(filter$2(this._filtered_selection, (J) => H.get(J)));
      this._filtered_selection = [
        ...filter$2(this._filtered_selection, (J) => !G.has(J)),
        ...filter$2(V.selected.indices, (J) => !H.get(J))
      ], V.selected.indices = [
        ...G,
        ...q
      ];
    }
  }
  mt(DataTableView, "__name__", "DataTableView");
  var f2;
  let DataTable$1 = (f2 = class extends TableWidget {
    constructor(H) {
      super(H);
      mt(this, "_sort_columns", []);
    }
    get sort_columns() {
      return this._sort_columns;
    }
    update_sort_columns(H) {
      this._sort_columns = H.map(({ sortCol: V, sortAsc: q }) => ({ field: V.field, sortAsc: q }));
    }
    get_scroll_index(H, V) {
      return !this.scroll_to_selection || V.length == 0 || some$2(V, (q) => H.top <= q && q <= H.bottom) ? null : Math.max(0, Math.min(...V) - 1);
    }
  }, mt(f2, "__name__", "DataTable"), f2.prototype.default_view = DataTableView, f2.define(({ List: H, Bool: V, Int: q, Ref: G, Str: J, Enum: Z, Or: te, Nullable: ne }) => ({
    autosize_mode: [Z("fit_columns", "fit_viewport", "none", "force_fit"), "force_fit"],
    auto_edit: [V, !1],
    columns: [H(G(TableColumn)), []],
    fit_columns: [ne(V), null],
    frozen_columns: [ne(q), null],
    frozen_rows: [ne(q), null],
    sortable: [V, !0],
    reorderable: [V, !0],
    editable: [V, !1],
    selectable: [te(V, Z("checkbox")), !0],
    index_position: [ne(q), 0],
    index_header: [J, "#"],
    index_width: [q, 40],
    scroll_to_selection: [V, !0],
    header_row: [V, !0],
    row_height: [q, 25]
  })), f2.override({
    width: 600,
    height: 400
  }), f2);
  const { Avg, Min, Max, Sum } = slickgridExports.Data.Aggregators, Z6 = class Z6 extends Model$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Z6, "__name__", "RowAggregator"), Z6.define(({ Str: _ }) => ({
    field_: [_, ""]
  }));
  let RowAggregator = Z6;
  const avg = new Avg();
  class AvgAggregator extends RowAggregator {
    constructor() {
      super(...arguments);
      mt(this, "key", "avg");
      mt(this, "init", avg.init);
      mt(this, "accumulate", avg.accumulate);
      mt(this, "storeResult", avg.storeResult);
    }
  }
  mt(AvgAggregator, "__name__", "AvgAggregator");
  const min$3 = new Min();
  class MinAggregator extends RowAggregator {
    constructor() {
      super(...arguments);
      mt(this, "key", "min");
      mt(this, "init", min$3.init);
      mt(this, "accumulate", min$3.accumulate);
      mt(this, "storeResult", min$3.storeResult);
    }
  }
  mt(MinAggregator, "__name__", "MinAggregator");
  const max$3 = new Max();
  class MaxAggregator extends RowAggregator {
    constructor() {
      super(...arguments);
      mt(this, "key", "max");
      mt(this, "init", max$3.init);
      mt(this, "accumulate", max$3.accumulate);
      mt(this, "storeResult", max$3.storeResult);
    }
  }
  mt(MaxAggregator, "__name__", "MaxAggregator");
  const sum$2 = new Sum();
  class SumAggregator extends RowAggregator {
    constructor() {
      super(...arguments);
      mt(this, "key", "sum");
      mt(this, "init", sum$2.init);
      mt(this, "accumulate", sum$2.accumulate);
      mt(this, "storeResult", sum$2.storeResult);
    }
  }
  mt(SumAggregator, "__name__", "SumAggregator");
  function groupCellFormatter(O, _, H, V, q) {
    const { collapsed: G, level: J, title: Z } = q, te = span$1({
      class: `slick-group-toggle ${G ? "collapsed" : "expanded"}`,
      style: { "margin-left": `${J * 15}px` }
    }), ne = span$1({
      class: "slick-group-title",
      level: J
    }, Z);
    return `${te.outerHTML}${ne.outerHTML}`;
  }
  function indentFormatter(O, _) {
    return (H, V, q, G, J) => {
      const Z = span$1({
        class: "slick-group-toggle",
        style: { "margin-left": `${(_ ?? 0) * 15}px` }
      }), te = O != null ? O(H, V, q, G, J) : `${q}`;
      return `${Z.outerHTML}${te.replace(/^<div/, "<span").replace(/div>$/, "span>")}`;
    };
  }
  function handleGridClick(O, _) {
    const H = this.getDataItem(_.row);
    H instanceof slickgridExports.Group && O.target.classList.contains("slick-group-toggle") && (H.collapsed ? this.getData().expandGroup(H.groupingKey) : this.getData().collapseGroup(H.groupingKey), O.stopImmediatePropagation(), O.preventDefault(), this.invalidate(), this.render());
  }
  const K6 = class K6 extends Model$1 {
    constructor(_) {
      super(_);
    }
    get comparer() {
      return (_, H) => _.value === H.value ? 0 : _.value > H.value ? 1 : -1;
    }
  };
  mt(K6, "__name__", "GroupingInfo"), K6.define(({ Bool: _, Str: H, List: V, Ref: q }) => ({
    getter: [H, ""],
    aggregators: [V(q(RowAggregator)), []],
    collapsed: [_, !1]
  }));
  let GroupingInfo = K6;
  class DataCubeProvider extends TableDataProvider {
    constructor(H, V, q, G) {
      super(H, V);
      mt(this, "columns");
      mt(this, "groupingInfos");
      mt(this, "groupingDelimiter");
      mt(this, "toggledGroupsByLevel");
      mt(this, "rows");
      mt(this, "target");
      this.columns = q, this.groupingInfos = [], this.groupingDelimiter = ":|:", this.target = G;
    }
    setGrouping(H) {
      this.groupingInfos = H, this.toggledGroupsByLevel = H.map(() => ({})), this.refresh();
    }
    extractGroups(H, V) {
      const q = [], G = /* @__PURE__ */ new Map(), J = V != null ? V.level + 1 : 0, { comparer: Z, getter: te } = this.groupingInfos[J];
      for (const ne of H) {
        const ee = dict(this.source.data).get(te);
        assert$2(ee != null);
        const ae = ee[ne];
        let oe = G.get(ae);
        if (oe == null) {
          const se = V != null ? `${V.groupingKey}${this.groupingDelimiter}${ae}` : `${ae}`;
          oe = Object.assign(new slickgridExports.Group(), { value: ae, level: J, groupingKey: se }), q.push(oe), G.set(ae, oe);
        }
        oe.rows.push(ne);
      }
      if (J < this.groupingInfos.length - 1)
        for (const ne of q)
          ne.groups = this.extractGroups(ne.rows, ne);
      return q.sort(Z), q;
    }
    calculateTotals(H, V) {
      const q = { avg: {}, max: {}, min: {}, sum: {} }, G = dict(this.source.data), J = [...G.keys()], Z = H.rows.map((te) => J.reduce((ne, ee) => ({ ...ne, [ee]: G.get(ee)[te] }), {}));
      for (const te of V) {
        te.init();
        for (const ne of Z)
          te.accumulate(ne);
        te.storeResult(q);
      }
      return q;
    }
    addTotals(H, V = 0) {
      const { aggregators: q, collapsed: G } = this.groupingInfos[V], J = this.toggledGroupsByLevel[V];
      for (const Z of H)
        is_nullish(Z.groups) || this.addTotals(Z.groups, V + 1), q.length != 0 && Z.rows.length != 0 && (Z.totals = this.calculateTotals(Z, q)), Z.collapsed = G !== J[Z.groupingKey], Z.title = Z.value ? `${Z.value}` : "";
    }
    flattenedGroupedRows(H, V = 0) {
      const q = [];
      for (const G of H)
        if (q.push(G), !G.collapsed) {
          const J = is_nullish(G.groups) ? G.rows : this.flattenedGroupedRows(G.groups, V + 1);
          q.push(...J);
        }
      return q;
    }
    refresh() {
      const H = this.extractGroups(this.view.indices), q = dict(this.source.data).get(this.columns[0].field);
      assert$2(q != null), H.length != 0 && (this.addTotals(H), this.rows = this.flattenedGroupedRows(H), this.target.data = {
        row_indices: this.rows.map((G) => G instanceof slickgridExports.Group ? G.rows : G),
        labels: this.rows.map((G) => G instanceof slickgridExports.Group ? G.title : q[G])
      });
    }
    getLength() {
      return this.rows.length;
    }
    getItem(H) {
      const V = this.rows[H], q = dict(this.source.data);
      return V instanceof slickgridExports.Group ? V : [...q.keys()].reduce((G, J) => ({ ...G, [J]: q.get(J)[V] }), { [DTINDEX_NAME]: V });
    }
    getItemMetadata(H) {
      const V = this.rows[H], q = this.columns.slice(1), G = V instanceof slickgridExports.Group ? this.groupingInfos[V.level].aggregators : [];
      function J(Z) {
        const { field: te, formatter: ne } = Z, ee = G.find(({ field_: ae }) => ae === te);
        if (ee != null) {
          const { key: ae } = ee;
          return {
            formatter(oe, se, ce, le, ue) {
              return ne != null ? ne(oe, se, ue.totals[ae][te], le, ue) : "";
            }
          };
        }
        return {};
      }
      return V instanceof slickgridExports.Group ? {
        selectable: !1,
        focusable: !1,
        cssClasses: "slick-group",
        columns: [{ formatter: groupCellFormatter }, ...q.map(J)]
      } : {};
    }
    collapseGroup(H) {
      const V = H.split(this.groupingDelimiter).length - 1;
      this.toggledGroupsByLevel[V][H] = !this.groupingInfos[V].collapsed, this.refresh();
    }
    expandGroup(H) {
      const V = H.split(this.groupingDelimiter).length - 1;
      this.toggledGroupsByLevel[V][H] = this.groupingInfos[V].collapsed, this.refresh();
    }
  }
  mt(DataCubeProvider, "__name__", "DataCubeProvider");
  class DataCubeView extends DataTableView {
    _after_render() {
      super._after_render();
      const _ = {
        enableCellNavigation: this.model.selectable !== !1,
        enableColumnReorder: !1,
        autosizeColsMode: this.autosize,
        multiColumnSort: !1,
        editable: this.model.editable,
        autoEdit: this.model.auto_edit,
        rowHeight: this.model.row_height
      }, H = this.model.columns.map((V) => V.toColumn());
      H[0].formatter = indentFormatter(H[0].formatter, this.model.grouping.length), delete H[0].editor, this.data = new DataCubeProvider(this.model.source, this.model.view, H, this.model.target), this.data.setGrouping(this.model.grouping), this.el.style.width = `${this.model.width}px`, this.grid = new slickgridExports.Grid(this.wrapper_el, this.data, H, _), this.grid.onClick.subscribe(handleGridClick);
    }
  }
  mt(DataCubeView, "__name__", "DataCubeView");
  const Q4 = class Q4 extends DataTable$1 {
    constructor(_) {
      super(_);
    }
  };
  mt(Q4, "__name__", "DataCube"), Q4.prototype.default_view = DataCubeView, Q4.define(({ List: _, Ref: H }) => ({
    grouping: [_(H(GroupingInfo)), []],
    target: [H(ColumnDataSource)]
  }));
  let DataCube = Q4;
  const Tables = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    AvgAggregator,
    BooleanFormatter,
    CellEditor,
    CellEditorView,
    CellFormatter,
    CheckboxEditor,
    CheckboxEditorView,
    DataCube,
    DataTable: DataTable$1,
    DateEditor,
    DateEditorView,
    DateFormatter,
    GroupingInfo,
    HTMLTemplateFormatter,
    IntEditor,
    IntEditorView,
    MaxAggregator,
    MinAggregator,
    NumberEditor,
    NumberEditorView,
    NumberFormatter,
    PercentEditor,
    PercentEditorView,
    ScientificFormatter,
    SelectEditor,
    SelectEditorView,
    StringEditor,
    StringEditorView,
    StringFormatter,
    SumAggregator,
    TableColumn,
    TableWidget,
    TextEditor: TextEditor$1,
    TextEditorView,
    TimeEditor,
    TimeEditorView
  }, Symbol.toStringTag, { value: "Module" }));
  register_models(Tables);
  var mathjax = {}, version$5 = {}, hasRequiredVersion;
  function requireVersion() {
    return hasRequiredVersion || (hasRequiredVersion = 1, Object.defineProperty(version$5, "__esModule", { value: !0 }), version$5.VERSION = void 0, version$5.VERSION = "3.2.2"), version$5;
  }
  var HandlerList = {}, PrioritizedList = {}, hasRequiredPrioritizedList;
  function requirePrioritizedList() {
    if (hasRequiredPrioritizedList) return PrioritizedList;
    hasRequiredPrioritizedList = 1, Object.defineProperty(PrioritizedList, "__esModule", { value: !0 }), PrioritizedList.PrioritizedList = void 0;
    var O = function() {
      function _() {
        this.items = [], this.items = [];
      }
      return _.prototype[Symbol.iterator] = function() {
        var H = 0, V = this.items;
        return {
          next: function() {
            return { value: V[H++], done: H > V.length };
          }
        };
      }, _.prototype.add = function(H, V) {
        V === void 0 && (V = _.DEFAULTPRIORITY);
        var q = this.items.length;
        do
          q--;
        while (q >= 0 && V < this.items[q].priority);
        return this.items.splice(q + 1, 0, { item: H, priority: V }), H;
      }, _.prototype.remove = function(H) {
        var V = this.items.length;
        do
          V--;
        while (V >= 0 && this.items[V].item !== H);
        V >= 0 && this.items.splice(V, 1);
      }, _.DEFAULTPRIORITY = 5, _;
    }();
    return PrioritizedList.PrioritizedList = O, PrioritizedList;
  }
  var hasRequiredHandlerList;
  function requireHandlerList() {
    if (hasRequiredHandlerList) return HandlerList;
    hasRequiredHandlerList = 1;
    var O = HandlerList.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = HandlerList.__values || function(q) {
      var G = typeof Symbol == "function" && Symbol.iterator, J = G && q[G], Z = 0;
      if (J) return J.call(q);
      if (q && typeof q.length == "number") return {
        next: function() {
          return q && Z >= q.length && (q = void 0), { value: q && q[Z++], done: !q };
        }
      };
      throw new TypeError(G ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(HandlerList, "__esModule", { value: !0 }), HandlerList.HandlerList = void 0;
    var H = requirePrioritizedList(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return G.prototype.register = function(J) {
        return this.add(J, J.priority);
      }, G.prototype.unregister = function(J) {
        this.remove(J);
      }, G.prototype.handlesDocument = function(J) {
        var Z, te;
        try {
          for (var ne = _(this), ee = ne.next(); !ee.done; ee = ne.next()) {
            var ae = ee.value, oe = ae.item;
            if (oe.handlesDocument(J))
              return oe;
          }
        } catch (se) {
          Z = { error: se };
        } finally {
          try {
            ee && !ee.done && (te = ne.return) && te.call(ne);
          } finally {
            if (Z) throw Z.error;
          }
        }
        throw new Error("Can't find handler for document");
      }, G.prototype.document = function(J, Z) {
        return Z === void 0 && (Z = null), this.handlesDocument(J).create(J, Z);
      }, G;
    }(H.PrioritizedList);
    return HandlerList.HandlerList = V, HandlerList;
  }
  var Retries = {}, hasRequiredRetries;
  function requireRetries() {
    if (hasRequiredRetries) return Retries;
    hasRequiredRetries = 1, Object.defineProperty(Retries, "__esModule", { value: !0 }), Retries.retryAfter = Retries.handleRetriesFor = void 0;
    function O(H) {
      return new Promise(function V(q, G) {
        try {
          q(H());
        } catch (J) {
          J.retry && J.retry instanceof Promise ? J.retry.then(function() {
            return V(q, G);
          }).catch(function(Z) {
            return G(Z);
          }) : J.restart && J.restart.isCallback ? MathJax.Callback.After(function() {
            return V(q, G);
          }, J.restart) : G(J);
        }
      });
    }
    Retries.handleRetriesFor = O;
    function _(H) {
      var V = new Error("MathJax retry");
      throw V.retry = H, V;
    }
    return Retries.retryAfter = _, Retries;
  }
  var hasRequiredMathjax;
  function requireMathjax() {
    return hasRequiredMathjax || (hasRequiredMathjax = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.mathjax = void 0;
      var _ = requireVersion(), H = requireHandlerList(), V = requireRetries();
      O.mathjax = {
        version: _.VERSION,
        handlers: new H.HandlerList(),
        document: function(q, G) {
          return O.mathjax.handlers.document(q, G);
        },
        handleRetriesFor: V.handleRetriesFor,
        retryAfter: V.retryAfter,
        asyncLoad: null
      };
    }(mathjax)), mathjax;
  }
  var mathjaxExports = requireMathjax(), tex$2 = {}, InputJax = {}, Options$1 = {}, hasRequiredOptions;
  function requireOptions() {
    return hasRequiredOptions || (hasRequiredOptions = 1, function(O) {
      var _ = Options$1.__values || function(fe) {
        var he = typeof Symbol == "function" && Symbol.iterator, me = he && fe[he], pe = 0;
        if (me) return me.call(fe);
        if (fe && typeof fe.length == "number") return {
          next: function() {
            return fe && pe >= fe.length && (fe = void 0), { value: fe && fe[pe++], done: !fe };
          }
        };
        throw new TypeError(he ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, H = Options$1.__read || function(fe, he) {
        var me = typeof Symbol == "function" && fe[Symbol.iterator];
        if (!me) return fe;
        var pe = me.call(fe), ye, ge = [], ve;
        try {
          for (; (he === void 0 || he-- > 0) && !(ye = pe.next()).done; ) ge.push(ye.value);
        } catch (be) {
          ve = { error: be };
        } finally {
          try {
            ye && !ye.done && (me = pe.return) && me.call(pe);
          } finally {
            if (ve) throw ve.error;
          }
        }
        return ge;
      }, V = Options$1.__spreadArray || function(fe, he, me) {
        if (me || arguments.length === 2) for (var pe = 0, ye = he.length, ge; pe < ye; pe++)
          (ge || !(pe in he)) && (ge || (ge = Array.prototype.slice.call(he, 0, pe)), ge[pe] = he[pe]);
        return fe.concat(ge || Array.prototype.slice.call(he));
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.lookup = O.separateOptions = O.selectOptionsFromKeys = O.selectOptions = O.userOptions = O.defaultOptions = O.insert = O.copy = O.keys = O.makeArray = O.expandable = O.Expandable = O.OPTIONS = O.REMOVE = O.APPEND = O.isObject = void 0;
      var q = {}.constructor;
      function G(fe) {
        return typeof fe == "object" && fe !== null && (fe.constructor === q || fe.constructor === J);
      }
      O.isObject = G, O.APPEND = "[+]", O.REMOVE = "[-]", O.OPTIONS = {
        invalidOption: "warn",
        optionError: function(fe, he) {
          if (O.OPTIONS.invalidOption === "fatal")
            throw new Error(fe);
          console.warn("MathJax: " + fe);
        }
      };
      var J = /* @__PURE__ */ function() {
        function fe() {
        }
        return fe;
      }();
      O.Expandable = J;
      function Z(fe) {
        return Object.assign(Object.create(J.prototype), fe);
      }
      O.expandable = Z;
      function te(fe) {
        return Array.isArray(fe) ? fe : [fe];
      }
      O.makeArray = te;
      function ne(fe) {
        return fe ? Object.keys(fe).concat(Object.getOwnPropertySymbols(fe)) : [];
      }
      O.keys = ne;
      function ee(fe) {
        var he, me, pe = {};
        try {
          for (var ye = _(ne(fe)), ge = ye.next(); !ge.done; ge = ye.next()) {
            var ve = ge.value, be = Object.getOwnPropertyDescriptor(fe, ve), Ce = be.value;
            Array.isArray(Ce) ? be.value = ae([], Ce, !1) : G(Ce) && (be.value = ee(Ce)), be.enumerable && (pe[ve] = be);
          }
        } catch (Se) {
          he = { error: Se };
        } finally {
          try {
            ge && !ge.done && (me = ye.return) && me.call(ye);
          } finally {
            if (he) throw he.error;
          }
        }
        return Object.defineProperties(fe.constructor === J ? Z({}) : {}, pe);
      }
      O.copy = ee;
      function ae(fe, he, me) {
        var pe, ye;
        me === void 0 && (me = !0);
        var ge = function(Se) {
          if (me && fe[Se] === void 0 && fe.constructor !== J)
            return typeof Se == "symbol" && (Se = Se.toString()), O.OPTIONS.optionError('Invalid option "'.concat(Se, '" (no default value).'), Se), "continue";
          var we = he[Se], Ee = fe[Se];
          if (G(we) && Ee !== null && (typeof Ee == "object" || typeof Ee == "function")) {
            var xe = ne(we);
            Array.isArray(Ee) && (xe.length === 1 && (xe[0] === O.APPEND || xe[0] === O.REMOVE) && Array.isArray(we[xe[0]]) || xe.length === 2 && xe.sort().join(",") === O.APPEND + "," + O.REMOVE && Array.isArray(we[O.APPEND]) && Array.isArray(we[O.REMOVE])) ? (we[O.REMOVE] && (Ee = fe[Se] = Ee.filter(function(Le) {
              return we[O.REMOVE].indexOf(Le) < 0;
            })), we[O.APPEND] && (fe[Se] = V(V([], H(Ee), !1), H(we[O.APPEND]), !1))) : ae(Ee, we, me);
          } else Array.isArray(we) ? (fe[Se] = [], ae(fe[Se], we, !1)) : G(we) ? fe[Se] = ee(we) : fe[Se] = we;
        };
        try {
          for (var ve = _(ne(he)), be = ve.next(); !be.done; be = ve.next()) {
            var Ce = be.value;
            ge(Ce);
          }
        } catch (Se) {
          pe = { error: Se };
        } finally {
          try {
            be && !be.done && (ye = ve.return) && ye.call(ve);
          } finally {
            if (pe) throw pe.error;
          }
        }
        return fe;
      }
      O.insert = ae;
      function oe(fe) {
        for (var he = [], me = 1; me < arguments.length; me++)
          he[me - 1] = arguments[me];
        return he.forEach(function(pe) {
          return ae(fe, pe, !1);
        }), fe;
      }
      O.defaultOptions = oe;
      function se(fe) {
        for (var he = [], me = 1; me < arguments.length; me++)
          he[me - 1] = arguments[me];
        return he.forEach(function(pe) {
          return ae(fe, pe, !0);
        }), fe;
      }
      O.userOptions = se;
      function ce(fe) {
        for (var he, me, pe = [], ye = 1; ye < arguments.length; ye++)
          pe[ye - 1] = arguments[ye];
        var ge = {};
        try {
          for (var ve = _(pe), be = ve.next(); !be.done; be = ve.next()) {
            var Ce = be.value;
            fe.hasOwnProperty(Ce) && (ge[Ce] = fe[Ce]);
          }
        } catch (Se) {
          he = { error: Se };
        } finally {
          try {
            be && !be.done && (me = ve.return) && me.call(ve);
          } finally {
            if (he) throw he.error;
          }
        }
        return ge;
      }
      O.selectOptions = ce;
      function le(fe, he) {
        return ce.apply(void 0, V([fe], H(Object.keys(he)), !1));
      }
      O.selectOptionsFromKeys = le;
      function ue(fe) {
        for (var he, me, pe, ye, ge = [], ve = 1; ve < arguments.length; ve++)
          ge[ve - 1] = arguments[ve];
        var be = [];
        try {
          for (var Ce = _(ge), Se = Ce.next(); !Se.done; Se = Ce.next()) {
            var we = Se.value, Ee = {}, xe = {};
            try {
              for (var Le = (pe = void 0, _(Object.keys(fe || {}))), Qe = Le.next(); !Qe.done; Qe = Le.next()) {
                var ke = Qe.value;
                (we[ke] === void 0 ? xe : Ee)[ke] = fe[ke];
              }
            } catch (Me) {
              pe = { error: Me };
            } finally {
              try {
                Qe && !Qe.done && (ye = Le.return) && ye.call(Le);
              } finally {
                if (pe) throw pe.error;
              }
            }
            be.push(Ee), fe = xe;
          }
        } catch (Me) {
          he = { error: Me };
        } finally {
          try {
            Se && !Se.done && (me = Ce.return) && me.call(Ce);
          } finally {
            if (he) throw he.error;
          }
        }
        return be.unshift(fe), be;
      }
      O.separateOptions = ue;
      function de(fe, he, me) {
        return me === void 0 && (me = null), he.hasOwnProperty(fe) ? he[fe] : me;
      }
      O.lookup = de;
    }(Options$1)), Options$1;
  }
  var FunctionList = {}, hasRequiredFunctionList;
  function requireFunctionList() {
    if (hasRequiredFunctionList) return FunctionList;
    hasRequiredFunctionList = 1;
    var O = FunctionList.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = FunctionList.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = FunctionList.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    }, V = FunctionList.__spreadArray || function(J, Z, te) {
      if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
        (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
      return J.concat(ae || Array.prototype.slice.call(Z));
    };
    Object.defineProperty(FunctionList, "__esModule", { value: !0 }), FunctionList.FunctionList = void 0;
    var q = requirePrioritizedList(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.execute = function() {
        for (var te, ne, ee = [], ae = 0; ae < arguments.length; ae++)
          ee[ae] = arguments[ae];
        try {
          for (var oe = _(this), se = oe.next(); !se.done; se = oe.next()) {
            var ce = se.value, le = ce.item.apply(ce, V([], H(ee), !1));
            if (le === !1)
              return !1;
          }
        } catch (ue) {
          te = { error: ue };
        } finally {
          try {
            se && !se.done && (ne = oe.return) && ne.call(oe);
          } finally {
            if (te) throw te.error;
          }
        }
        return !0;
      }, Z.prototype.asyncExecute = function() {
        for (var te = [], ne = 0; ne < arguments.length; ne++)
          te[ne] = arguments[ne];
        var ee = -1, ae = this.items;
        return new Promise(function(oe, se) {
          (function ce() {
            for (var le; ++ee < ae.length; ) {
              var ue = (le = ae[ee]).item.apply(le, V([], H(te), !1));
              if (ue instanceof Promise) {
                ue.then(ce).catch(function(de) {
                  return se(de);
                });
                return;
              }
              if (ue === !1) {
                oe(!1);
                return;
              }
            }
            oe(!0);
          })();
        });
      }, Z;
    }(q.PrioritizedList);
    return FunctionList.FunctionList = G, FunctionList;
  }
  var hasRequiredInputJax;
  function requireInputJax() {
    if (hasRequiredInputJax) return InputJax;
    hasRequiredInputJax = 1, Object.defineProperty(InputJax, "__esModule", { value: !0 }), InputJax.AbstractInputJax = void 0;
    var O = requireOptions(), _ = requireFunctionList(), H = function() {
      function V(q) {
        q === void 0 && (q = {}), this.adaptor = null, this.mmlFactory = null;
        var G = this.constructor;
        this.options = (0, O.userOptions)((0, O.defaultOptions)({}, G.OPTIONS), q), this.preFilters = new _.FunctionList(), this.postFilters = new _.FunctionList();
      }
      return Object.defineProperty(V.prototype, "name", {
        get: function() {
          return this.constructor.NAME;
        },
        enumerable: !1,
        configurable: !0
      }), V.prototype.setAdaptor = function(q) {
        this.adaptor = q;
      }, V.prototype.setMmlFactory = function(q) {
        this.mmlFactory = q;
      }, V.prototype.initialize = function() {
      }, V.prototype.reset = function() {
      }, Object.defineProperty(V.prototype, "processStrings", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), V.prototype.findMath = function(q, G) {
        return [];
      }, V.prototype.executeFilters = function(q, G, J, Z) {
        var te = { math: G, document: J, data: Z };
        return q.execute(te), te.data;
      }, V.NAME = "generic", V.OPTIONS = {}, V;
    }();
    return InputJax.AbstractInputJax = H, InputJax;
  }
  var FindTeX = {}, FindMath = {}, hasRequiredFindMath;
  function requireFindMath() {
    if (hasRequiredFindMath) return FindMath;
    hasRequiredFindMath = 1, Object.defineProperty(FindMath, "__esModule", { value: !0 }), FindMath.AbstractFindMath = void 0;
    var O = requireOptions(), _ = function() {
      function H(V) {
        var q = this.constructor;
        this.options = (0, O.userOptions)((0, O.defaultOptions)({}, q.OPTIONS), V);
      }
      return H.OPTIONS = {}, H;
    }();
    return FindMath.AbstractFindMath = _, FindMath;
  }
  var string$2 = {}, hasRequiredString;
  function requireString() {
    if (hasRequiredString) return string$2;
    hasRequiredString = 1;
    var O = string$2.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    }, _ = string$2.__spreadArray || function(te, ne, ee) {
      if (ee || arguments.length === 2) for (var ae = 0, oe = ne.length, se; ae < oe; ae++)
        (se || !(ae in ne)) && (se || (se = Array.prototype.slice.call(ne, 0, ae)), se[ae] = ne[ae]);
      return te.concat(se || Array.prototype.slice.call(ne));
    };
    Object.defineProperty(string$2, "__esModule", { value: !0 }), string$2.split = string$2.isPercent = string$2.unicodeString = string$2.unicodeChars = string$2.quotePattern = string$2.sortLength = void 0;
    function H(te, ne) {
      return te.length !== ne.length ? ne.length - te.length : te === ne ? 0 : te < ne ? -1 : 1;
    }
    string$2.sortLength = H;
    function V(te) {
      return te.replace(/([\^$(){}+*?\-|\[\]\:\\])/g, "\\$1");
    }
    string$2.quotePattern = V;
    function q(te) {
      return Array.from(te).map(function(ne) {
        return ne.codePointAt(0);
      });
    }
    string$2.unicodeChars = q;
    function G(te) {
      return String.fromCodePoint.apply(String, _([], O(te), !1));
    }
    string$2.unicodeString = G;
    function J(te) {
      return !!te.match(/%\s*$/);
    }
    string$2.isPercent = J;
    function Z(te) {
      return te.trim().split(/\s+/);
    }
    return string$2.split = Z, string$2;
  }
  var MathItem = {}, hasRequiredMathItem;
  function requireMathItem() {
    return hasRequiredMathItem || (hasRequiredMathItem = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.newState = O.STATE = O.AbstractMathItem = O.protoItem = void 0;
      function _(q, G, J, Z, te, ne, ee) {
        ee === void 0 && (ee = null);
        var ae = {
          open: q,
          math: G,
          close: J,
          n: Z,
          start: { n: te },
          end: { n: ne },
          display: ee
        };
        return ae;
      }
      O.protoItem = _;
      var H = function() {
        function q(G, J, Z, te, ne) {
          Z === void 0 && (Z = !0), te === void 0 && (te = { i: 0, n: 0, delim: "" }), ne === void 0 && (ne = { i: 0, n: 0, delim: "" }), this.root = null, this.typesetRoot = null, this.metrics = {}, this.inputData = {}, this.outputData = {}, this._state = O.STATE.UNPROCESSED, this.math = G, this.inputJax = J, this.display = Z, this.start = te, this.end = ne, this.root = null, this.typesetRoot = null, this.metrics = {}, this.inputData = {}, this.outputData = {};
        }
        return Object.defineProperty(q.prototype, "isEscaped", {
          get: function() {
            return this.display === null;
          },
          enumerable: !1,
          configurable: !0
        }), q.prototype.render = function(G) {
          G.renderActions.renderMath(this, G);
        }, q.prototype.rerender = function(G, J) {
          J === void 0 && (J = O.STATE.RERENDER), this.state() >= J && this.state(J - 1), G.renderActions.renderMath(this, G, J);
        }, q.prototype.convert = function(G, J) {
          J === void 0 && (J = O.STATE.LAST), G.renderActions.renderConvert(this, G, J);
        }, q.prototype.compile = function(G) {
          this.state() < O.STATE.COMPILED && (this.root = this.inputJax.compile(this, G), this.state(O.STATE.COMPILED));
        }, q.prototype.typeset = function(G) {
          this.state() < O.STATE.TYPESET && (this.typesetRoot = G.outputJax[this.isEscaped ? "escaped" : "typeset"](this, G), this.state(O.STATE.TYPESET));
        }, q.prototype.updateDocument = function(G) {
        }, q.prototype.removeFromDocument = function(G) {
        }, q.prototype.setMetrics = function(G, J, Z, te, ne) {
          this.metrics = {
            em: G,
            ex: J,
            containerWidth: Z,
            lineWidth: te,
            scale: ne
          };
        }, q.prototype.state = function(G, J) {
          return G === void 0 && (G = null), J === void 0 && (J = !1), G != null && (G < O.STATE.INSERTED && this._state >= O.STATE.INSERTED && this.removeFromDocument(J), G < O.STATE.TYPESET && this._state >= O.STATE.TYPESET && (this.outputData = {}), G < O.STATE.COMPILED && this._state >= O.STATE.COMPILED && (this.inputData = {}), this._state = G), this._state;
        }, q.prototype.reset = function(G) {
          G === void 0 && (G = !1), this.state(O.STATE.UNPROCESSED, G);
        }, q;
      }();
      O.AbstractMathItem = H, O.STATE = {
        UNPROCESSED: 0,
        FINDMATH: 10,
        COMPILED: 20,
        CONVERT: 100,
        METRICS: 110,
        RERENDER: 125,
        TYPESET: 150,
        INSERTED: 200,
        LAST: 1e4
      };
      function V(q, G) {
        if (q in O.STATE)
          throw Error("State " + q + " already exists");
        O.STATE[q] = G;
      }
      O.newState = V;
    }(MathItem)), MathItem;
  }
  var hasRequiredFindTeX;
  function requireFindTeX() {
    if (hasRequiredFindTeX) return FindTeX;
    hasRequiredFindTeX = 1;
    var O = FindTeX.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = FindTeX.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    };
    Object.defineProperty(FindTeX, "__esModule", { value: !0 }), FindTeX.FindTeX = void 0;
    var H = requireFindMath(), V = requireString(), q = requireMathItem(), G = function(J) {
      O(Z, J);
      function Z(te) {
        var ne = J.call(this, te) || this;
        return ne.getPatterns(), ne;
      }
      return Z.prototype.getPatterns = function() {
        var te = this, ne = this.options, ee = [], ae = [], oe = [];
        this.end = {}, this.env = this.sub = 0;
        var se = 1;
        ne.inlineMath.forEach(function(ce) {
          return te.addPattern(ee, ce, !1);
        }), ne.displayMath.forEach(function(ce) {
          return te.addPattern(ee, ce, !0);
        }), ee.length && ae.push(ee.sort(V.sortLength).join("|")), ne.processEnvironments && (ae.push("\\\\begin\\s*\\{([^}]*)\\}"), this.env = se, se++), ne.processEscapes && oe.push("\\\\([\\\\$])"), ne.processRefs && oe.push("(\\\\(?:eq)?ref\\s*\\{[^}]*\\})"), oe.length && (ae.push("(" + oe.join("|") + ")"), this.sub = se), this.start = new RegExp(ae.join("|"), "g"), this.hasPatterns = ae.length > 0;
      }, Z.prototype.addPattern = function(te, ne, ee) {
        var ae = _(ne, 2), oe = ae[0], se = ae[1];
        te.push((0, V.quotePattern)(oe)), this.end[oe] = [se, ee, this.endPattern(se)];
      }, Z.prototype.endPattern = function(te, ne) {
        return new RegExp((ne || (0, V.quotePattern)(te)) + "|\\\\(?:[a-zA-Z]|.)|[{}]", "g");
      }, Z.prototype.findEnd = function(te, ne, ee, ae) {
        for (var oe = _(ae, 3), se = oe[0], ce = oe[1], le = oe[2], ue = le.lastIndex = ee.index + ee[0].length, de, fe = 0; de = le.exec(te); ) {
          if ((de[1] || de[0]) === se && fe === 0)
            return (0, q.protoItem)(ee[0], te.substr(ue, de.index - ue), de[0], ne, ee.index, de.index + de[0].length, ce);
          de[0] === "{" ? fe++ : de[0] === "}" && fe && fe--;
        }
        return null;
      }, Z.prototype.findMathInString = function(te, ne, ee) {
        var ae, oe;
        for (this.start.lastIndex = 0; ae = this.start.exec(ee); ) {
          if (ae[this.env] !== void 0 && this.env) {
            var se = "\\\\end\\s*(\\{" + (0, V.quotePattern)(ae[this.env]) + "\\})";
            oe = this.findEnd(ee, ne, ae, ["{" + ae[this.env] + "}", !0, this.endPattern(null, se)]), oe && (oe.math = oe.open + oe.math + oe.close, oe.open = oe.close = "");
          } else if (ae[this.sub] !== void 0 && this.sub) {
            var ce = ae[this.sub], se = ae.index + ae[this.sub].length;
            ce.length === 2 ? oe = (0, q.protoItem)("", ce.substr(1), "", ne, ae.index, se) : oe = (0, q.protoItem)("", ce, "", ne, ae.index, se, !1);
          } else
            oe = this.findEnd(ee, ne, ae, this.end[ae[0]]);
          oe && (te.push(oe), this.start.lastIndex = oe.end.n);
        }
      }, Z.prototype.findMath = function(te) {
        var ne = [];
        if (this.hasPatterns)
          for (var ee = 0, ae = te.length; ee < ae; ee++)
            this.findMathInString(ne, ee, te[ee]);
        return ne;
      }, Z.OPTIONS = {
        inlineMath: [
          ["\\(", "\\)"]
        ],
        displayMath: [
          ["$$", "$$"],
          ["\\[", "\\]"]
        ],
        processEscapes: !0,
        processEnvironments: !0,
        processRefs: !0
      }, Z;
    }(H.AbstractFindMath);
    return FindTeX.FindTeX = G, FindTeX;
  }
  var FilterUtil = {}, MmlNode = {}, Attributes = {}, hasRequiredAttributes;
  function requireAttributes() {
    return hasRequiredAttributes || (hasRequiredAttributes = 1, function(O) {
      var _ = Attributes.__values || function(V) {
        var q = typeof Symbol == "function" && Symbol.iterator, G = q && V[q], J = 0;
        if (G) return G.call(V);
        if (V && typeof V.length == "number") return {
          next: function() {
            return V && J >= V.length && (V = void 0), { value: V && V[J++], done: !V };
          }
        };
        throw new TypeError(q ? "Object is not iterable." : "Symbol.iterator is not defined.");
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.Attributes = O.INHERIT = void 0, O.INHERIT = "_inherit_";
      var H = function() {
        function V(q, G) {
          this.global = G, this.defaults = Object.create(G), this.inherited = Object.create(this.defaults), this.attributes = Object.create(this.inherited), Object.assign(this.defaults, q);
        }
        return V.prototype.set = function(q, G) {
          this.attributes[q] = G;
        }, V.prototype.setList = function(q) {
          Object.assign(this.attributes, q);
        }, V.prototype.get = function(q) {
          var G = this.attributes[q];
          return G === O.INHERIT && (G = this.global[q]), G;
        }, V.prototype.getExplicit = function(q) {
          if (this.attributes.hasOwnProperty(q))
            return this.attributes[q];
        }, V.prototype.getList = function() {
          for (var q, G, J = [], Z = 0; Z < arguments.length; Z++)
            J[Z] = arguments[Z];
          var te = {};
          try {
            for (var ne = _(J), ee = ne.next(); !ee.done; ee = ne.next()) {
              var ae = ee.value;
              te[ae] = this.get(ae);
            }
          } catch (oe) {
            q = { error: oe };
          } finally {
            try {
              ee && !ee.done && (G = ne.return) && G.call(ne);
            } finally {
              if (q) throw q.error;
            }
          }
          return te;
        }, V.prototype.setInherited = function(q, G) {
          this.inherited[q] = G;
        }, V.prototype.getInherited = function(q) {
          return this.inherited[q];
        }, V.prototype.getDefault = function(q) {
          return this.defaults[q];
        }, V.prototype.isSet = function(q) {
          return this.attributes.hasOwnProperty(q) || this.inherited.hasOwnProperty(q);
        }, V.prototype.hasDefault = function(q) {
          return q in this.defaults;
        }, V.prototype.getExplicitNames = function() {
          return Object.keys(this.attributes);
        }, V.prototype.getInheritedNames = function() {
          return Object.keys(this.inherited);
        }, V.prototype.getDefaultNames = function() {
          return Object.keys(this.defaults);
        }, V.prototype.getGlobalNames = function() {
          return Object.keys(this.global);
        }, V.prototype.getAllAttributes = function() {
          return this.attributes;
        }, V.prototype.getAllInherited = function() {
          return this.inherited;
        }, V.prototype.getAllDefaults = function() {
          return this.defaults;
        }, V.prototype.getAllGlobals = function() {
          return this.global;
        }, V;
      }();
      O.Attributes = H;
    }(Attributes)), Attributes;
  }
  var Node$3 = {}, hasRequiredNode;
  function requireNode() {
    if (hasRequiredNode) return Node$3;
    hasRequiredNode = 1;
    var O = Node$3.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = Node$3.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    }, H = Node$3.__values || function(G) {
      var J = typeof Symbol == "function" && Symbol.iterator, Z = J && G[J], te = 0;
      if (Z) return Z.call(G);
      if (G && typeof G.length == "number") return {
        next: function() {
          return G && te >= G.length && (G = void 0), { value: G && G[te++], done: !G };
        }
      };
      throw new TypeError(J ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(Node$3, "__esModule", { value: !0 }), Node$3.AbstractEmptyNode = Node$3.AbstractNode = void 0;
    var V = function() {
      function G(J, Z, te) {
        var ne, ee;
        Z === void 0 && (Z = {}), te === void 0 && (te = []), this.factory = J, this.parent = null, this.properties = {}, this.childNodes = [];
        try {
          for (var ae = H(Object.keys(Z)), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            this.setProperty(se, Z[se]);
          }
        } catch (ce) {
          ne = { error: ce };
        } finally {
          try {
            oe && !oe.done && (ee = ae.return) && ee.call(ae);
          } finally {
            if (ne) throw ne.error;
          }
        }
        te.length && this.setChildren(te);
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "unknown";
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.setProperty = function(J, Z) {
        this.properties[J] = Z;
      }, G.prototype.getProperty = function(J) {
        return this.properties[J];
      }, G.prototype.getPropertyNames = function() {
        return Object.keys(this.properties);
      }, G.prototype.getAllProperties = function() {
        return this.properties;
      }, G.prototype.removeProperty = function() {
        for (var J, Z, te = [], ne = 0; ne < arguments.length; ne++)
          te[ne] = arguments[ne];
        try {
          for (var ee = H(te), ae = ee.next(); !ae.done; ae = ee.next()) {
            var oe = ae.value;
            delete this.properties[oe];
          }
        } catch (se) {
          J = { error: se };
        } finally {
          try {
            ae && !ae.done && (Z = ee.return) && Z.call(ee);
          } finally {
            if (J) throw J.error;
          }
        }
      }, G.prototype.isKind = function(J) {
        return this.factory.nodeIsKind(this, J);
      }, G.prototype.setChildren = function(J) {
        var Z, te;
        this.childNodes = [];
        try {
          for (var ne = H(J), ee = ne.next(); !ee.done; ee = ne.next()) {
            var ae = ee.value;
            this.appendChild(ae);
          }
        } catch (oe) {
          Z = { error: oe };
        } finally {
          try {
            ee && !ee.done && (te = ne.return) && te.call(ne);
          } finally {
            if (Z) throw Z.error;
          }
        }
      }, G.prototype.appendChild = function(J) {
        return this.childNodes.push(J), J.parent = this, J;
      }, G.prototype.replaceChild = function(J, Z) {
        var te = this.childIndex(Z);
        return te !== null && (this.childNodes[te] = J, J.parent = this, Z.parent = null), J;
      }, G.prototype.removeChild = function(J) {
        var Z = this.childIndex(J);
        return Z !== null && (this.childNodes.splice(Z, 1), J.parent = null), J;
      }, G.prototype.childIndex = function(J) {
        var Z = this.childNodes.indexOf(J);
        return Z === -1 ? null : Z;
      }, G.prototype.copy = function() {
        var J, Z, te = this.factory.create(this.kind);
        te.properties = _({}, this.properties);
        try {
          for (var ne = H(this.childNodes || []), ee = ne.next(); !ee.done; ee = ne.next()) {
            var ae = ee.value;
            ae && te.appendChild(ae.copy());
          }
        } catch (oe) {
          J = { error: oe };
        } finally {
          try {
            ee && !ee.done && (Z = ne.return) && Z.call(ne);
          } finally {
            if (J) throw J.error;
          }
        }
        return te;
      }, G.prototype.findNodes = function(J) {
        var Z = [];
        return this.walkTree(function(te) {
          te.isKind(J) && Z.push(te);
        }), Z;
      }, G.prototype.walkTree = function(J, Z) {
        var te, ne;
        J(this, Z);
        try {
          for (var ee = H(this.childNodes), ae = ee.next(); !ae.done; ae = ee.next()) {
            var oe = ae.value;
            oe && oe.walkTree(J, Z);
          }
        } catch (se) {
          te = { error: se };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (te) throw te.error;
          }
        }
        return Z;
      }, G.prototype.toString = function() {
        return this.kind + "(" + this.childNodes.join(",") + ")";
      }, G;
    }();
    Node$3.AbstractNode = V;
    var q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.prototype.setChildren = function(Z) {
      }, J.prototype.appendChild = function(Z) {
        return Z;
      }, J.prototype.replaceChild = function(Z, te) {
        return te;
      }, J.prototype.childIndex = function(Z) {
        return null;
      }, J.prototype.walkTree = function(Z, te) {
        return Z(this, te), te;
      }, J.prototype.toString = function() {
        return this.kind;
      }, J;
    }(V);
    return Node$3.AbstractEmptyNode = q, Node$3;
  }
  var hasRequiredMmlNode;
  function requireMmlNode() {
    return hasRequiredMmlNode || (hasRequiredMmlNode = 1, function(O) {
      var _ = MmlNode.__extends || /* @__PURE__ */ function() {
        var ue = function(de, fe) {
          return ue = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(he, me) {
            he.__proto__ = me;
          } || function(he, me) {
            for (var pe in me) Object.prototype.hasOwnProperty.call(me, pe) && (he[pe] = me[pe]);
          }, ue(de, fe);
        };
        return function(de, fe) {
          if (typeof fe != "function" && fe !== null)
            throw new TypeError("Class extends value " + String(fe) + " is not a constructor or null");
          ue(de, fe);
          function he() {
            this.constructor = de;
          }
          de.prototype = fe === null ? Object.create(fe) : (he.prototype = fe.prototype, new he());
        };
      }(), H = MmlNode.__assign || function() {
        return H = Object.assign || function(ue) {
          for (var de, fe = 1, he = arguments.length; fe < he; fe++) {
            de = arguments[fe];
            for (var me in de) Object.prototype.hasOwnProperty.call(de, me) && (ue[me] = de[me]);
          }
          return ue;
        }, H.apply(this, arguments);
      }, V = MmlNode.__values || function(ue) {
        var de = typeof Symbol == "function" && Symbol.iterator, fe = de && ue[de], he = 0;
        if (fe) return fe.call(ue);
        if (ue && typeof ue.length == "number") return {
          next: function() {
            return ue && he >= ue.length && (ue = void 0), { value: ue && ue[he++], done: !ue };
          }
        };
        throw new TypeError(de ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, q = MmlNode.__read || function(ue, de) {
        var fe = typeof Symbol == "function" && ue[Symbol.iterator];
        if (!fe) return ue;
        var he = fe.call(ue), me, pe = [], ye;
        try {
          for (; (de === void 0 || de-- > 0) && !(me = he.next()).done; ) pe.push(me.value);
        } catch (ge) {
          ye = { error: ge };
        } finally {
          try {
            me && !me.done && (fe = he.return) && fe.call(he);
          } finally {
            if (ye) throw ye.error;
          }
        }
        return pe;
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.XMLNode = O.TextNode = O.AbstractMmlEmptyNode = O.AbstractMmlBaseNode = O.AbstractMmlLayoutNode = O.AbstractMmlTokenNode = O.AbstractMmlNode = O.indentAttributes = O.TEXCLASSNAMES = O.TEXCLASS = void 0;
      var G = requireAttributes(), J = requireNode();
      O.TEXCLASS = {
        ORD: 0,
        OP: 1,
        BIN: 2,
        REL: 3,
        OPEN: 4,
        CLOSE: 5,
        PUNCT: 6,
        INNER: 7,
        VCENTER: 8,
        NONE: -1
      }, O.TEXCLASSNAMES = ["ORD", "OP", "BIN", "REL", "OPEN", "CLOSE", "PUNCT", "INNER", "VCENTER"];
      var Z = ["", "thinmathspace", "mediummathspace", "thickmathspace"], te = [
        [0, -1, 2, 3, 0, 0, 0, 1],
        [-1, -1, 0, 3, 0, 0, 0, 1],
        [2, 2, 0, 0, 2, 0, 0, 2],
        [3, 3, 0, 0, 3, 0, 0, 3],
        [0, 0, 0, 0, 0, 0, 0, 0],
        [0, -1, 2, 3, 0, 0, 0, 1],
        [1, 1, 0, 1, 1, 1, 1, 1],
        [1, -1, 2, 3, 1, 0, 1, 1]
      ];
      O.indentAttributes = [
        "indentalign",
        "indentalignfirst",
        "indentshift",
        "indentshiftfirst"
      ];
      var ne = function(ue) {
        _(de, ue);
        function de(fe, he, me) {
          he === void 0 && (he = {}), me === void 0 && (me = []);
          var pe = ue.call(this, fe) || this;
          return pe.prevClass = null, pe.prevLevel = null, pe.texclass = null, pe.arity < 0 && (pe.childNodes = [fe.create("inferredMrow")], pe.childNodes[0].parent = pe), pe.setChildren(me), pe.attributes = new G.Attributes(fe.getNodeClass(pe.kind).defaults, fe.getNodeClass("math").defaults), pe.attributes.setList(he), pe;
        }
        return de.prototype.copy = function(fe) {
          var he, me, pe, ye;
          fe === void 0 && (fe = !1);
          var ge = this.factory.create(this.kind);
          if (ge.properties = H({}, this.properties), this.attributes) {
            var ve = this.attributes.getAllAttributes();
            try {
              for (var be = V(Object.keys(ve)), Ce = be.next(); !Ce.done; Ce = be.next()) {
                var Se = Ce.value;
                (Se !== "id" || fe) && ge.attributes.set(Se, ve[Se]);
              }
            } catch (Qe) {
              he = { error: Qe };
            } finally {
              try {
                Ce && !Ce.done && (me = be.return) && me.call(be);
              } finally {
                if (he) throw he.error;
              }
            }
          }
          if (this.childNodes && this.childNodes.length) {
            var we = this.childNodes;
            we.length === 1 && we[0].isInferred && (we = we[0].childNodes);
            try {
              for (var Ee = V(we), xe = Ee.next(); !xe.done; xe = Ee.next()) {
                var Le = xe.value;
                Le ? ge.appendChild(Le.copy()) : ge.childNodes.push(null);
              }
            } catch (Qe) {
              pe = { error: Qe };
            } finally {
              try {
                xe && !xe.done && (ye = Ee.return) && ye.call(Ee);
              } finally {
                if (pe) throw pe.error;
              }
            }
          }
          return ge;
        }, Object.defineProperty(de.prototype, "texClass", {
          get: function() {
            return this.texclass;
          },
          set: function(fe) {
            this.texclass = fe;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isToken", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isEmbellished", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isSpacelike", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "linebreakContainer", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "hasNewLine", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "arity", {
          get: function() {
            return 1 / 0;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isInferred", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "Parent", {
          get: function() {
            for (var fe = this.parent; fe && fe.notParent; )
              fe = fe.Parent;
            return fe;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "notParent", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.setChildren = function(fe) {
          return this.arity < 0 ? this.childNodes[0].setChildren(fe) : ue.prototype.setChildren.call(this, fe);
        }, de.prototype.appendChild = function(fe) {
          var he, me, pe = this;
          if (this.arity < 0)
            return this.childNodes[0].appendChild(fe), fe;
          if (fe.isInferred) {
            if (this.arity === 1 / 0)
              return fe.childNodes.forEach(function(Ce) {
                return ue.prototype.appendChild.call(pe, Ce);
              }), fe;
            var ye = fe;
            fe = this.factory.create("mrow"), fe.setChildren(ye.childNodes), fe.attributes = ye.attributes;
            try {
              for (var ge = V(ye.getPropertyNames()), ve = ge.next(); !ve.done; ve = ge.next()) {
                var be = ve.value;
                fe.setProperty(be, ye.getProperty(be));
              }
            } catch (Ce) {
              he = { error: Ce };
            } finally {
              try {
                ve && !ve.done && (me = ge.return) && me.call(ge);
              } finally {
                if (he) throw he.error;
              }
            }
          }
          return ue.prototype.appendChild.call(this, fe);
        }, de.prototype.replaceChild = function(fe, he) {
          return this.arity < 0 ? (this.childNodes[0].replaceChild(fe, he), fe) : ue.prototype.replaceChild.call(this, fe, he);
        }, de.prototype.core = function() {
          return this;
        }, de.prototype.coreMO = function() {
          return this;
        }, de.prototype.coreIndex = function() {
          return 0;
        }, de.prototype.childPosition = function() {
          for (var fe, he, me = this, pe = me.parent; pe && pe.notParent; )
            me = pe, pe = pe.parent;
          if (pe) {
            var ye = 0;
            try {
              for (var ge = V(pe.childNodes), ve = ge.next(); !ve.done; ve = ge.next()) {
                var be = ve.value;
                if (be === me)
                  return ye;
                ye++;
              }
            } catch (Ce) {
              fe = { error: Ce };
            } finally {
              try {
                ve && !ve.done && (he = ge.return) && he.call(ge);
              } finally {
                if (fe) throw fe.error;
              }
            }
          }
          return null;
        }, de.prototype.setTeXclass = function(fe) {
          return this.getPrevClass(fe), this.texClass != null ? this : fe;
        }, de.prototype.updateTeXclass = function(fe) {
          fe && (this.prevClass = fe.prevClass, this.prevLevel = fe.prevLevel, fe.prevClass = fe.prevLevel = null, this.texClass = fe.texClass);
        }, de.prototype.getPrevClass = function(fe) {
          fe && (this.prevClass = fe.texClass, this.prevLevel = fe.attributes.get("scriptlevel"));
        }, de.prototype.texSpacing = function() {
          var fe = this.prevClass != null ? this.prevClass : O.TEXCLASS.NONE, he = this.texClass || O.TEXCLASS.ORD;
          if (fe === O.TEXCLASS.NONE || he === O.TEXCLASS.NONE)
            return "";
          fe === O.TEXCLASS.VCENTER && (fe = O.TEXCLASS.ORD), he === O.TEXCLASS.VCENTER && (he = O.TEXCLASS.ORD);
          var me = te[fe][he];
          return (this.prevLevel > 0 || this.attributes.get("scriptlevel") > 0) && me >= 0 ? "" : Z[Math.abs(me)];
        }, de.prototype.hasSpacingAttributes = function() {
          return this.isEmbellished && this.coreMO().hasSpacingAttributes();
        }, de.prototype.setInheritedAttributes = function(fe, he, me, pe) {
          var ye, ge;
          fe === void 0 && (fe = {}), he === void 0 && (he = !1), me === void 0 && (me = 0), pe === void 0 && (pe = !1);
          var ve = this.attributes.getAllDefaults();
          try {
            for (var be = V(Object.keys(fe)), Ce = be.next(); !Ce.done; Ce = be.next()) {
              var Se = Ce.value;
              if (ve.hasOwnProperty(Se) || de.alwaysInherit.hasOwnProperty(Se)) {
                var we = q(fe[Se], 2), Ee = we[0], xe = we[1], Le = (de.noInherit[Ee] || {})[this.kind] || {};
                Le[Se] || this.attributes.setInherited(Se, xe);
              }
            }
          } catch (Re) {
            ye = { error: Re };
          } finally {
            try {
              Ce && !Ce.done && (ge = be.return) && ge.call(be);
            } finally {
              if (ye) throw ye.error;
            }
          }
          var Qe = this.attributes.getExplicit("displaystyle");
          Qe === void 0 && this.attributes.setInherited("displaystyle", he);
          var ke = this.attributes.getExplicit("scriptlevel");
          ke === void 0 && this.attributes.setInherited("scriptlevel", me), pe && this.setProperty("texprimestyle", pe);
          var Me = this.arity;
          if (Me >= 0 && Me !== 1 / 0 && (Me === 1 && this.childNodes.length === 0 || Me !== 1 && this.childNodes.length !== Me))
            if (Me < this.childNodes.length)
              this.childNodes = this.childNodes.slice(0, Me);
            else
              for (; this.childNodes.length < Me; )
                this.appendChild(this.factory.create("mrow"));
          this.setChildInheritedAttributes(fe, he, me, pe);
        }, de.prototype.setChildInheritedAttributes = function(fe, he, me, pe) {
          var ye, ge;
          try {
            for (var ve = V(this.childNodes), be = ve.next(); !be.done; be = ve.next()) {
              var Ce = be.value;
              Ce.setInheritedAttributes(fe, he, me, pe);
            }
          } catch (Se) {
            ye = { error: Se };
          } finally {
            try {
              be && !be.done && (ge = ve.return) && ge.call(ve);
            } finally {
              if (ye) throw ye.error;
            }
          }
        }, de.prototype.addInheritedAttributes = function(fe, he) {
          var me, pe, ye = H({}, fe);
          try {
            for (var ge = V(Object.keys(he)), ve = ge.next(); !ve.done; ve = ge.next()) {
              var be = ve.value;
              be !== "displaystyle" && be !== "scriptlevel" && be !== "style" && (ye[be] = [this.kind, he[be]]);
            }
          } catch (Ce) {
            me = { error: Ce };
          } finally {
            try {
              ve && !ve.done && (pe = ge.return) && pe.call(ge);
            } finally {
              if (me) throw me.error;
            }
          }
          return ye;
        }, de.prototype.inheritAttributesFrom = function(fe) {
          var he = fe.attributes, me = he.get("displaystyle"), pe = he.get("scriptlevel"), ye = he.isSet("mathsize") ? {
            mathsize: ["math", he.get("mathsize")]
          } : {}, ge = fe.getProperty("texprimestyle") || !1;
          this.setInheritedAttributes(ye, me, pe, ge);
        }, de.prototype.verifyTree = function(fe) {
          if (fe === void 0 && (fe = null), fe !== null) {
            this.verifyAttributes(fe);
            var he = this.arity;
            fe.checkArity && he >= 0 && he !== 1 / 0 && (he === 1 && this.childNodes.length === 0 || he !== 1 && this.childNodes.length !== he) && this.mError('Wrong number of children for "' + this.kind + '" node', fe, !0), this.verifyChildren(fe);
          }
        }, de.prototype.verifyAttributes = function(fe) {
          var he, me;
          if (fe.checkAttributes) {
            var pe = this.attributes, ye = [];
            try {
              for (var ge = V(pe.getExplicitNames()), ve = ge.next(); !ve.done; ve = ge.next()) {
                var be = ve.value;
                be.substr(0, 5) !== "data-" && pe.getDefault(be) === void 0 && !be.match(/^(?:class|style|id|(?:xlink:)?href)$/) && ye.push(be);
              }
            } catch (Ce) {
              he = { error: Ce };
            } finally {
              try {
                ve && !ve.done && (me = ge.return) && me.call(ge);
              } finally {
                if (he) throw he.error;
              }
            }
            ye.length && this.mError("Unknown attributes for " + this.kind + " node: " + ye.join(", "), fe);
          }
        }, de.prototype.verifyChildren = function(fe) {
          var he, me;
          try {
            for (var pe = V(this.childNodes), ye = pe.next(); !ye.done; ye = pe.next()) {
              var ge = ye.value;
              ge.verifyTree(fe);
            }
          } catch (ve) {
            he = { error: ve };
          } finally {
            try {
              ye && !ye.done && (me = pe.return) && me.call(pe);
            } finally {
              if (he) throw he.error;
            }
          }
        }, de.prototype.mError = function(fe, he, me) {
          if (me === void 0 && (me = !1), this.parent && this.parent.isKind("merror"))
            return null;
          var pe = this.factory.create("merror");
          if (pe.attributes.set("data-mjx-message", fe), he.fullErrors || me) {
            var ye = this.factory.create("mtext"), ge = this.factory.create("text");
            ge.setText(he.fullErrors ? fe : this.kind), ye.appendChild(ge), pe.appendChild(ye), this.parent.replaceChild(pe, this);
          } else
            this.parent.replaceChild(pe, this), pe.appendChild(this);
          return pe;
        }, de.defaults = {
          mathbackground: G.INHERIT,
          mathcolor: G.INHERIT,
          mathsize: G.INHERIT,
          dir: G.INHERIT
        }, de.noInherit = {
          mstyle: {
            mpadded: { width: !0, height: !0, depth: !0, lspace: !0, voffset: !0 },
            mtable: { width: !0, height: !0, depth: !0, align: !0 }
          },
          maligngroup: {
            mrow: { groupalign: !0 },
            mtable: { groupalign: !0 }
          }
        }, de.alwaysInherit = {
          scriptminsize: !0,
          scriptsizemultiplier: !0
        }, de.verifyDefaults = {
          checkArity: !0,
          checkAttributes: !1,
          fullErrors: !1,
          fixMmultiscripts: !0,
          fixMtables: !0
        }, de;
      }(J.AbstractNode);
      O.AbstractMmlNode = ne;
      var ee = function(ue) {
        _(de, ue);
        function de() {
          return ue !== null && ue.apply(this, arguments) || this;
        }
        return Object.defineProperty(de.prototype, "isToken", {
          get: function() {
            return !0;
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.getText = function() {
          var fe, he, me = "";
          try {
            for (var pe = V(this.childNodes), ye = pe.next(); !ye.done; ye = pe.next()) {
              var ge = ye.value;
              ge instanceof ce && (me += ge.getText());
            }
          } catch (ve) {
            fe = { error: ve };
          } finally {
            try {
              ye && !ye.done && (he = pe.return) && he.call(pe);
            } finally {
              if (fe) throw fe.error;
            }
          }
          return me;
        }, de.prototype.setChildInheritedAttributes = function(fe, he, me, pe) {
          var ye, ge;
          try {
            for (var ve = V(this.childNodes), be = ve.next(); !be.done; be = ve.next()) {
              var Ce = be.value;
              Ce instanceof ne && Ce.setInheritedAttributes(fe, he, me, pe);
            }
          } catch (Se) {
            ye = { error: Se };
          } finally {
            try {
              be && !be.done && (ge = ve.return) && ge.call(ve);
            } finally {
              if (ye) throw ye.error;
            }
          }
        }, de.prototype.walkTree = function(fe, he) {
          var me, pe;
          fe(this, he);
          try {
            for (var ye = V(this.childNodes), ge = ye.next(); !ge.done; ge = ye.next()) {
              var ve = ge.value;
              ve instanceof ne && ve.walkTree(fe, he);
            }
          } catch (be) {
            me = { error: be };
          } finally {
            try {
              ge && !ge.done && (pe = ye.return) && pe.call(ye);
            } finally {
              if (me) throw me.error;
            }
          }
          return he;
        }, de.defaults = H(H({}, ne.defaults), { mathvariant: "normal", mathsize: G.INHERIT }), de;
      }(ne);
      O.AbstractMmlTokenNode = ee;
      var ae = function(ue) {
        _(de, ue);
        function de() {
          return ue !== null && ue.apply(this, arguments) || this;
        }
        return Object.defineProperty(de.prototype, "isSpacelike", {
          get: function() {
            return this.childNodes[0].isSpacelike;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isEmbellished", {
          get: function() {
            return this.childNodes[0].isEmbellished;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "arity", {
          get: function() {
            return -1;
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.core = function() {
          return this.childNodes[0];
        }, de.prototype.coreMO = function() {
          return this.childNodes[0].coreMO();
        }, de.prototype.setTeXclass = function(fe) {
          return fe = this.childNodes[0].setTeXclass(fe), this.updateTeXclass(this.childNodes[0]), fe;
        }, de.defaults = ne.defaults, de;
      }(ne);
      O.AbstractMmlLayoutNode = ae;
      var oe = function(ue) {
        _(de, ue);
        function de() {
          return ue !== null && ue.apply(this, arguments) || this;
        }
        return Object.defineProperty(de.prototype, "isEmbellished", {
          get: function() {
            return this.childNodes[0].isEmbellished;
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.core = function() {
          return this.childNodes[0];
        }, de.prototype.coreMO = function() {
          return this.childNodes[0].coreMO();
        }, de.prototype.setTeXclass = function(fe) {
          var he, me;
          this.getPrevClass(fe), this.texClass = O.TEXCLASS.ORD;
          var pe = this.childNodes[0];
          pe ? this.isEmbellished || pe.isKind("mi") ? (fe = pe.setTeXclass(fe), this.updateTeXclass(this.core())) : (pe.setTeXclass(null), fe = this) : fe = this;
          try {
            for (var ye = V(this.childNodes.slice(1)), ge = ye.next(); !ge.done; ge = ye.next()) {
              var ve = ge.value;
              ve && ve.setTeXclass(null);
            }
          } catch (be) {
            he = { error: be };
          } finally {
            try {
              ge && !ge.done && (me = ye.return) && me.call(ye);
            } finally {
              if (he) throw he.error;
            }
          }
          return fe;
        }, de.defaults = ne.defaults, de;
      }(ne);
      O.AbstractMmlBaseNode = oe;
      var se = function(ue) {
        _(de, ue);
        function de() {
          return ue !== null && ue.apply(this, arguments) || this;
        }
        return Object.defineProperty(de.prototype, "isToken", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isEmbellished", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isSpacelike", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "linebreakContainer", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "hasNewLine", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "arity", {
          get: function() {
            return 0;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "isInferred", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "notParent", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "Parent", {
          get: function() {
            return this.parent;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "texClass", {
          get: function() {
            return O.TEXCLASS.NONE;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "prevClass", {
          get: function() {
            return O.TEXCLASS.NONE;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(de.prototype, "prevLevel", {
          get: function() {
            return 0;
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.hasSpacingAttributes = function() {
          return !1;
        }, Object.defineProperty(de.prototype, "attributes", {
          get: function() {
            return null;
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.core = function() {
          return this;
        }, de.prototype.coreMO = function() {
          return this;
        }, de.prototype.coreIndex = function() {
          return 0;
        }, de.prototype.childPosition = function() {
          return 0;
        }, de.prototype.setTeXclass = function(fe) {
          return fe;
        }, de.prototype.texSpacing = function() {
          return "";
        }, de.prototype.setInheritedAttributes = function(fe, he, me, pe) {
        }, de.prototype.inheritAttributesFrom = function(fe) {
        }, de.prototype.verifyTree = function(fe) {
        }, de.prototype.mError = function(fe, he, me) {
          return null;
        }, de;
      }(J.AbstractEmptyNode);
      O.AbstractMmlEmptyNode = se;
      var ce = function(ue) {
        _(de, ue);
        function de() {
          var fe = ue !== null && ue.apply(this, arguments) || this;
          return fe.text = "", fe;
        }
        return Object.defineProperty(de.prototype, "kind", {
          get: function() {
            return "text";
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.getText = function() {
          return this.text;
        }, de.prototype.setText = function(fe) {
          return this.text = fe, this;
        }, de.prototype.copy = function() {
          return this.factory.create(this.kind).setText(this.getText());
        }, de.prototype.toString = function() {
          return this.text;
        }, de;
      }(se);
      O.TextNode = ce;
      var le = function(ue) {
        _(de, ue);
        function de() {
          var fe = ue !== null && ue.apply(this, arguments) || this;
          return fe.xml = null, fe.adaptor = null, fe;
        }
        return Object.defineProperty(de.prototype, "kind", {
          get: function() {
            return "XML";
          },
          enumerable: !1,
          configurable: !0
        }), de.prototype.getXML = function() {
          return this.xml;
        }, de.prototype.setXML = function(fe, he) {
          return he === void 0 && (he = null), this.xml = fe, this.adaptor = he, this;
        }, de.prototype.getSerializedXML = function() {
          return this.adaptor.serializeXML(this.xml);
        }, de.prototype.copy = function() {
          return this.factory.create(this.kind).setXML(this.adaptor.clone(this.xml));
        }, de.prototype.toString = function() {
          return "XML data";
        }, de;
      }(se);
      O.XMLNode = le;
    }(MmlNode)), MmlNode;
  }
  var NodeUtil = {}, mo$2 = {}, OperatorDictionary = {}, hasRequiredOperatorDictionary;
  function requireOperatorDictionary() {
    return hasRequiredOperatorDictionary || (hasRequiredOperatorDictionary = 1, function(O) {
      var _ = OperatorDictionary.__values || function(G) {
        var J = typeof Symbol == "function" && Symbol.iterator, Z = J && G[J], te = 0;
        if (Z) return Z.call(G);
        if (G && typeof G.length == "number") return {
          next: function() {
            return G && te >= G.length && (G = void 0), { value: G && G[te++], done: !G };
          }
        };
        throw new TypeError(J ? "Object is not iterable." : "Symbol.iterator is not defined.");
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.OPTABLE = O.MMLSPACING = O.getRange = O.RANGES = O.MO = O.OPDEF = void 0;
      var H = requireMmlNode();
      function V(G, J, Z, te) {
        return Z === void 0 && (Z = H.TEXCLASS.BIN), te === void 0 && (te = null), [G, J, Z, te];
      }
      O.OPDEF = V, O.MO = {
        ORD: V(0, 0, H.TEXCLASS.ORD),
        ORD11: V(1, 1, H.TEXCLASS.ORD),
        ORD21: V(2, 1, H.TEXCLASS.ORD),
        ORD02: V(0, 2, H.TEXCLASS.ORD),
        ORD55: V(5, 5, H.TEXCLASS.ORD),
        NONE: V(0, 0, H.TEXCLASS.NONE),
        OP: V(1, 2, H.TEXCLASS.OP, { largeop: !0, movablelimits: !0, symmetric: !0 }),
        OPFIXED: V(1, 2, H.TEXCLASS.OP, { largeop: !0, movablelimits: !0 }),
        INTEGRAL: V(0, 1, H.TEXCLASS.OP, { largeop: !0, symmetric: !0 }),
        INTEGRAL2: V(1, 2, H.TEXCLASS.OP, { largeop: !0, symmetric: !0 }),
        BIN3: V(3, 3, H.TEXCLASS.BIN),
        BIN4: V(4, 4, H.TEXCLASS.BIN),
        BIN01: V(0, 1, H.TEXCLASS.BIN),
        BIN5: V(5, 5, H.TEXCLASS.BIN),
        TALLBIN: V(4, 4, H.TEXCLASS.BIN, { stretchy: !0 }),
        BINOP: V(4, 4, H.TEXCLASS.BIN, { largeop: !0, movablelimits: !0 }),
        REL: V(5, 5, H.TEXCLASS.REL),
        REL1: V(1, 1, H.TEXCLASS.REL, { stretchy: !0 }),
        REL4: V(4, 4, H.TEXCLASS.REL),
        RELSTRETCH: V(5, 5, H.TEXCLASS.REL, { stretchy: !0 }),
        RELACCENT: V(5, 5, H.TEXCLASS.REL, { accent: !0 }),
        WIDEREL: V(5, 5, H.TEXCLASS.REL, { accent: !0, stretchy: !0 }),
        OPEN: V(0, 0, H.TEXCLASS.OPEN, { fence: !0, stretchy: !0, symmetric: !0 }),
        CLOSE: V(0, 0, H.TEXCLASS.CLOSE, { fence: !0, stretchy: !0, symmetric: !0 }),
        INNER: V(0, 0, H.TEXCLASS.INNER),
        PUNCT: V(0, 3, H.TEXCLASS.PUNCT),
        ACCENT: V(0, 0, H.TEXCLASS.ORD, { accent: !0 }),
        WIDEACCENT: V(0, 0, H.TEXCLASS.ORD, { accent: !0, stretchy: !0 })
      }, O.RANGES = [
        [32, 127, H.TEXCLASS.REL, "mo"],
        [160, 191, H.TEXCLASS.ORD, "mo"],
        [192, 591, H.TEXCLASS.ORD, "mi"],
        [688, 879, H.TEXCLASS.ORD, "mo"],
        [880, 6688, H.TEXCLASS.ORD, "mi"],
        [6832, 6911, H.TEXCLASS.ORD, "mo"],
        [6912, 7615, H.TEXCLASS.ORD, "mi"],
        [7616, 7679, H.TEXCLASS.ORD, "mo"],
        [7680, 8191, H.TEXCLASS.ORD, "mi"],
        [8192, 8303, H.TEXCLASS.ORD, "mo"],
        [8304, 8351, H.TEXCLASS.ORD, "mo"],
        [8448, 8527, H.TEXCLASS.ORD, "mi"],
        [8528, 8591, H.TEXCLASS.ORD, "mn"],
        [8592, 8703, H.TEXCLASS.REL, "mo"],
        [8704, 8959, H.TEXCLASS.BIN, "mo"],
        [8960, 9215, H.TEXCLASS.ORD, "mo"],
        [9312, 9471, H.TEXCLASS.ORD, "mn"],
        [9472, 10223, H.TEXCLASS.ORD, "mo"],
        [10224, 10239, H.TEXCLASS.REL, "mo"],
        [10240, 10495, H.TEXCLASS.ORD, "mtext"],
        [10496, 10623, H.TEXCLASS.REL, "mo"],
        [10624, 10751, H.TEXCLASS.ORD, "mo"],
        [10752, 11007, H.TEXCLASS.BIN, "mo"],
        [11008, 11055, H.TEXCLASS.ORD, "mo"],
        [11056, 11087, H.TEXCLASS.REL, "mo"],
        [11088, 11263, H.TEXCLASS.ORD, "mo"],
        [11264, 11744, H.TEXCLASS.ORD, "mi"],
        [11776, 11903, H.TEXCLASS.ORD, "mo"],
        [11904, 12255, H.TEXCLASS.ORD, "mi", "normal"],
        [12272, 12351, H.TEXCLASS.ORD, "mo"],
        [12352, 42143, H.TEXCLASS.ORD, "mi", "normal"],
        [42192, 43055, H.TEXCLASS.ORD, "mi"],
        [43056, 43071, H.TEXCLASS.ORD, "mn"],
        [43072, 55295, H.TEXCLASS.ORD, "mi"],
        [63744, 64255, H.TEXCLASS.ORD, "mi", "normal"],
        [64256, 65023, H.TEXCLASS.ORD, "mi"],
        [65024, 65135, H.TEXCLASS.ORD, "mo"],
        [65136, 65791, H.TEXCLASS.ORD, "mi"],
        [65792, 65935, H.TEXCLASS.ORD, "mn"],
        [65936, 74751, H.TEXCLASS.ORD, "mi", "normal"],
        [74752, 74879, H.TEXCLASS.ORD, "mn"],
        [74880, 113823, H.TEXCLASS.ORD, "mi", "normal"],
        [113824, 119391, H.TEXCLASS.ORD, "mo"],
        [119648, 119679, H.TEXCLASS.ORD, "mn"],
        [119808, 120781, H.TEXCLASS.ORD, "mi"],
        [120782, 120831, H.TEXCLASS.ORD, "mn"],
        [122624, 129023, H.TEXCLASS.ORD, "mo"],
        [129024, 129279, H.TEXCLASS.REL, "mo"],
        [129280, 129535, H.TEXCLASS.ORD, "mo"],
        [131072, 195103, H.TEXCLASS.ORD, "mi", "normnal"]
      ];
      function q(G) {
        var J, Z, te = G.codePointAt(0);
        try {
          for (var ne = _(O.RANGES), ee = ne.next(); !ee.done; ee = ne.next()) {
            var ae = ee.value;
            if (te <= ae[1]) {
              if (te >= ae[0])
                return ae;
              break;
            }
          }
        } catch (oe) {
          J = { error: oe };
        } finally {
          try {
            ee && !ee.done && (Z = ne.return) && Z.call(ne);
          } finally {
            if (J) throw J.error;
          }
        }
        return null;
      }
      O.getRange = q, O.MMLSPACING = [
        [0, 0],
        [1, 2],
        [3, 3],
        [4, 4],
        [0, 0],
        [0, 0],
        [0, 3]
      ], O.OPTABLE = {
        prefix: {
          "(": O.MO.OPEN,
          "+": O.MO.BIN01,
          "-": O.MO.BIN01,
          "[": O.MO.OPEN,
          "{": O.MO.OPEN,
          "|": O.MO.OPEN,
          "||": [0, 0, H.TEXCLASS.BIN, { fence: !0, stretchy: !0, symmetric: !0 }],
          "|||": [0, 0, H.TEXCLASS.ORD, { fence: !0, stretchy: !0, symmetric: !0 }],
          "¬": O.MO.ORD21,
          "±": O.MO.BIN01,
          "‖": [0, 0, H.TEXCLASS.ORD, { fence: !0, stretchy: !0 }],
          "‘": [0, 0, H.TEXCLASS.OPEN, { fence: !0 }],
          "“": [0, 0, H.TEXCLASS.OPEN, { fence: !0 }],
          "ⅅ": O.MO.ORD21,
          "ⅆ": V(2, 0, H.TEXCLASS.ORD),
          "∀": O.MO.ORD21,
          "∂": O.MO.ORD21,
          "∃": O.MO.ORD21,
          "∄": O.MO.ORD21,
          "∇": O.MO.ORD21,
          "∏": O.MO.OP,
          "∐": O.MO.OP,
          "∑": O.MO.OP,
          "−": O.MO.BIN01,
          "∓": O.MO.BIN01,
          "√": [1, 1, H.TEXCLASS.ORD, { stretchy: !0 }],
          "∛": O.MO.ORD11,
          "∜": O.MO.ORD11,
          "∠": O.MO.ORD,
          "∡": O.MO.ORD,
          "∢": O.MO.ORD,
          "∫": O.MO.INTEGRAL,
          "∬": O.MO.INTEGRAL,
          "∭": O.MO.INTEGRAL,
          "∮": O.MO.INTEGRAL,
          "∯": O.MO.INTEGRAL,
          "∰": O.MO.INTEGRAL,
          "∱": O.MO.INTEGRAL,
          "∲": O.MO.INTEGRAL,
          "∳": O.MO.INTEGRAL,
          "⋀": O.MO.OP,
          "⋁": O.MO.OP,
          "⋂": O.MO.OP,
          "⋃": O.MO.OP,
          "⌈": O.MO.OPEN,
          "⌊": O.MO.OPEN,
          "〈": O.MO.OPEN,
          "❲": O.MO.OPEN,
          "⟦": O.MO.OPEN,
          "⟨": O.MO.OPEN,
          "⟪": O.MO.OPEN,
          "⟬": O.MO.OPEN,
          "⟮": O.MO.OPEN,
          "⦀": [0, 0, H.TEXCLASS.ORD, { fence: !0, stretchy: !0 }],
          "⦃": O.MO.OPEN,
          "⦅": O.MO.OPEN,
          "⦇": O.MO.OPEN,
          "⦉": O.MO.OPEN,
          "⦋": O.MO.OPEN,
          "⦍": O.MO.OPEN,
          "⦏": O.MO.OPEN,
          "⦑": O.MO.OPEN,
          "⦓": O.MO.OPEN,
          "⦕": O.MO.OPEN,
          "⦗": O.MO.OPEN,
          "⧼": O.MO.OPEN,
          "⨀": O.MO.OP,
          "⨁": O.MO.OP,
          "⨂": O.MO.OP,
          "⨃": O.MO.OP,
          "⨄": O.MO.OP,
          "⨅": O.MO.OP,
          "⨆": O.MO.OP,
          "⨇": O.MO.OP,
          "⨈": O.MO.OP,
          "⨉": O.MO.OP,
          "⨊": O.MO.OP,
          "⨋": O.MO.INTEGRAL2,
          "⨌": O.MO.INTEGRAL,
          "⨍": O.MO.INTEGRAL2,
          "⨎": O.MO.INTEGRAL2,
          "⨏": O.MO.INTEGRAL2,
          "⨐": O.MO.OP,
          "⨑": O.MO.OP,
          "⨒": O.MO.OP,
          "⨓": O.MO.OP,
          "⨔": O.MO.OP,
          "⨕": O.MO.INTEGRAL2,
          "⨖": O.MO.INTEGRAL2,
          "⨗": O.MO.INTEGRAL2,
          "⨘": O.MO.INTEGRAL2,
          "⨙": O.MO.INTEGRAL2,
          "⨚": O.MO.INTEGRAL2,
          "⨛": O.MO.INTEGRAL2,
          "⨜": O.MO.INTEGRAL2,
          "⫼": O.MO.OP,
          "⫿": O.MO.OP
        },
        postfix: {
          "!!": V(1, 0),
          "!": [1, 0, H.TEXCLASS.CLOSE, null],
          '"': O.MO.ACCENT,
          "&": O.MO.ORD,
          ")": O.MO.CLOSE,
          "++": V(0, 0),
          "--": V(0, 0),
          "..": V(0, 0),
          "...": O.MO.ORD,
          "'": O.MO.ACCENT,
          "]": O.MO.CLOSE,
          "^": O.MO.WIDEACCENT,
          _: O.MO.WIDEACCENT,
          "`": O.MO.ACCENT,
          "|": O.MO.CLOSE,
          "}": O.MO.CLOSE,
          "~": O.MO.WIDEACCENT,
          "||": [0, 0, H.TEXCLASS.BIN, { fence: !0, stretchy: !0, symmetric: !0 }],
          "|||": [0, 0, H.TEXCLASS.ORD, { fence: !0, stretchy: !0, symmetric: !0 }],
          "¨": O.MO.ACCENT,
          ª: O.MO.ACCENT,
          "¯": O.MO.WIDEACCENT,
          "°": O.MO.ORD,
          "²": O.MO.ACCENT,
          "³": O.MO.ACCENT,
          "´": O.MO.ACCENT,
          "¸": O.MO.ACCENT,
          "¹": O.MO.ACCENT,
          º: O.MO.ACCENT,
          "ˆ": O.MO.WIDEACCENT,
          "ˇ": O.MO.WIDEACCENT,
          "ˉ": O.MO.WIDEACCENT,
          "ˊ": O.MO.ACCENT,
          "ˋ": O.MO.ACCENT,
          "ˍ": O.MO.WIDEACCENT,
          "˘": O.MO.ACCENT,
          "˙": O.MO.ACCENT,
          "˚": O.MO.ACCENT,
          "˜": O.MO.WIDEACCENT,
          "˝": O.MO.ACCENT,
          "˷": O.MO.WIDEACCENT,
          "̂": O.MO.WIDEACCENT,
          "̑": O.MO.ACCENT,
          "϶": O.MO.REL,
          "‖": [0, 0, H.TEXCLASS.ORD, { fence: !0, stretchy: !0 }],
          "’": [0, 0, H.TEXCLASS.CLOSE, { fence: !0 }],
          "‚": O.MO.ACCENT,
          "‛": O.MO.ACCENT,
          "”": [0, 0, H.TEXCLASS.CLOSE, { fence: !0 }],
          "„": O.MO.ACCENT,
          "‟": O.MO.ACCENT,
          "′": O.MO.ORD,
          "″": O.MO.ACCENT,
          "‴": O.MO.ACCENT,
          "‵": O.MO.ACCENT,
          "‶": O.MO.ACCENT,
          "‷": O.MO.ACCENT,
          "‾": O.MO.WIDEACCENT,
          "⁗": O.MO.ACCENT,
          "⃛": O.MO.ACCENT,
          "⃜": O.MO.ACCENT,
          "⌉": O.MO.CLOSE,
          "⌋": O.MO.CLOSE,
          "〉": O.MO.CLOSE,
          "⎴": O.MO.WIDEACCENT,
          "⎵": O.MO.WIDEACCENT,
          "⏜": O.MO.WIDEACCENT,
          "⏝": O.MO.WIDEACCENT,
          "⏞": O.MO.WIDEACCENT,
          "⏟": O.MO.WIDEACCENT,
          "⏠": O.MO.WIDEACCENT,
          "⏡": O.MO.WIDEACCENT,
          "■": O.MO.BIN3,
          "□": O.MO.BIN3,
          "▪": O.MO.BIN3,
          "▫": O.MO.BIN3,
          "▭": O.MO.BIN3,
          "▮": O.MO.BIN3,
          "▯": O.MO.BIN3,
          "▰": O.MO.BIN3,
          "▱": O.MO.BIN3,
          "▲": O.MO.BIN4,
          "▴": O.MO.BIN4,
          "▶": O.MO.BIN4,
          "▷": O.MO.BIN4,
          "▸": O.MO.BIN4,
          "▼": O.MO.BIN4,
          "▾": O.MO.BIN4,
          "◀": O.MO.BIN4,
          "◁": O.MO.BIN4,
          "◂": O.MO.BIN4,
          "◄": O.MO.BIN4,
          "◅": O.MO.BIN4,
          "◆": O.MO.BIN4,
          "◇": O.MO.BIN4,
          "◈": O.MO.BIN4,
          "◉": O.MO.BIN4,
          "◌": O.MO.BIN4,
          "◍": O.MO.BIN4,
          "◎": O.MO.BIN4,
          "●": O.MO.BIN4,
          "◖": O.MO.BIN4,
          "◗": O.MO.BIN4,
          "◦": O.MO.BIN4,
          "♭": O.MO.ORD02,
          "♮": O.MO.ORD02,
          "♯": O.MO.ORD02,
          "❳": O.MO.CLOSE,
          "⟧": O.MO.CLOSE,
          "⟩": O.MO.CLOSE,
          "⟫": O.MO.CLOSE,
          "⟭": O.MO.CLOSE,
          "⟯": O.MO.CLOSE,
          "⦀": [0, 0, H.TEXCLASS.ORD, { fence: !0, stretchy: !0 }],
          "⦄": O.MO.CLOSE,
          "⦆": O.MO.CLOSE,
          "⦈": O.MO.CLOSE,
          "⦊": O.MO.CLOSE,
          "⦌": O.MO.CLOSE,
          "⦎": O.MO.CLOSE,
          "⦐": O.MO.CLOSE,
          "⦒": O.MO.CLOSE,
          "⦔": O.MO.CLOSE,
          "⦖": O.MO.CLOSE,
          "⦘": O.MO.CLOSE,
          "⧽": O.MO.CLOSE
        },
        infix: {
          "!=": O.MO.BIN4,
          "#": O.MO.ORD,
          $: O.MO.ORD,
          "%": [3, 3, H.TEXCLASS.ORD, null],
          "&&": O.MO.BIN4,
          "": O.MO.ORD,
          "*": O.MO.BIN3,
          "**": V(1, 1),
          "*=": O.MO.BIN4,
          "+": O.MO.BIN4,
          "+=": O.MO.BIN4,
          ",": [0, 3, H.TEXCLASS.PUNCT, { linebreakstyle: "after", separator: !0 }],
          "-": O.MO.BIN4,
          "-=": O.MO.BIN4,
          "->": O.MO.BIN5,
          ".": [0, 3, H.TEXCLASS.PUNCT, { separator: !0 }],
          "/": O.MO.ORD11,
          "//": V(1, 1),
          "/=": O.MO.BIN4,
          ":": [1, 2, H.TEXCLASS.REL, null],
          ":=": O.MO.BIN4,
          ";": [0, 3, H.TEXCLASS.PUNCT, { linebreakstyle: "after", separator: !0 }],
          "<": O.MO.REL,
          "<=": O.MO.BIN5,
          "<>": V(1, 1),
          "=": O.MO.REL,
          "==": O.MO.BIN4,
          ">": O.MO.REL,
          ">=": O.MO.BIN5,
          "?": [1, 1, H.TEXCLASS.CLOSE, null],
          "@": O.MO.ORD11,
          "\\": O.MO.ORD,
          "^": O.MO.ORD11,
          _: O.MO.ORD11,
          "|": [2, 2, H.TEXCLASS.ORD, { fence: !0, stretchy: !0, symmetric: !0 }],
          "||": [2, 2, H.TEXCLASS.BIN, { fence: !0, stretchy: !0, symmetric: !0 }],
          "|||": [2, 2, H.TEXCLASS.ORD, { fence: !0, stretchy: !0, symmetric: !0 }],
          "±": O.MO.BIN4,
          "·": O.MO.BIN4,
          "×": O.MO.BIN4,
          "÷": O.MO.BIN4,
          "ʹ": O.MO.ORD,
          "̀": O.MO.ACCENT,
          "́": O.MO.ACCENT,
          "̃": O.MO.WIDEACCENT,
          "̄": O.MO.ACCENT,
          "̆": O.MO.ACCENT,
          "̇": O.MO.ACCENT,
          "̈": O.MO.ACCENT,
          "̌": O.MO.ACCENT,
          "̲": O.MO.WIDEACCENT,
          "̸": O.MO.REL4,
          "―": [0, 0, H.TEXCLASS.ORD, { stretchy: !0 }],
          "‗": [0, 0, H.TEXCLASS.ORD, { stretchy: !0 }],
          "†": O.MO.BIN3,
          "‡": O.MO.BIN3,
          "•": O.MO.BIN4,
          "…": O.MO.INNER,
          "⁃": O.MO.BIN4,
          "⁄": O.MO.TALLBIN,
          "⁡": O.MO.NONE,
          "⁢": O.MO.NONE,
          "⁣": [0, 0, H.TEXCLASS.NONE, { linebreakstyle: "after", separator: !0 }],
          "⁤": O.MO.NONE,
          "⃗": O.MO.ACCENT,
          ℑ: O.MO.ORD,
          ℓ: O.MO.ORD,
          "℘": O.MO.ORD,
          ℜ: O.MO.ORD,
          "←": O.MO.WIDEREL,
          "↑": O.MO.RELSTRETCH,
          "→": O.MO.WIDEREL,
          "↓": O.MO.RELSTRETCH,
          "↔": O.MO.WIDEREL,
          "↕": O.MO.RELSTRETCH,
          "↖": O.MO.RELSTRETCH,
          "↗": O.MO.RELSTRETCH,
          "↘": O.MO.RELSTRETCH,
          "↙": O.MO.RELSTRETCH,
          "↚": O.MO.RELACCENT,
          "↛": O.MO.RELACCENT,
          "↜": O.MO.WIDEREL,
          "↝": O.MO.WIDEREL,
          "↞": O.MO.WIDEREL,
          "↟": O.MO.WIDEREL,
          "↠": O.MO.WIDEREL,
          "↡": O.MO.RELSTRETCH,
          "↢": O.MO.WIDEREL,
          "↣": O.MO.WIDEREL,
          "↤": O.MO.WIDEREL,
          "↥": O.MO.RELSTRETCH,
          "↦": O.MO.WIDEREL,
          "↧": O.MO.RELSTRETCH,
          "↨": O.MO.RELSTRETCH,
          "↩": O.MO.WIDEREL,
          "↪": O.MO.WIDEREL,
          "↫": O.MO.WIDEREL,
          "↬": O.MO.WIDEREL,
          "↭": O.MO.WIDEREL,
          "↮": O.MO.RELACCENT,
          "↯": O.MO.RELSTRETCH,
          "↰": O.MO.RELSTRETCH,
          "↱": O.MO.RELSTRETCH,
          "↲": O.MO.RELSTRETCH,
          "↳": O.MO.RELSTRETCH,
          "↴": O.MO.RELSTRETCH,
          "↵": O.MO.RELSTRETCH,
          "↶": O.MO.RELACCENT,
          "↷": O.MO.RELACCENT,
          "↸": O.MO.REL,
          "↹": O.MO.WIDEREL,
          "↺": O.MO.REL,
          "↻": O.MO.REL,
          "↼": O.MO.WIDEREL,
          "↽": O.MO.WIDEREL,
          "↾": O.MO.RELSTRETCH,
          "↿": O.MO.RELSTRETCH,
          "⇀": O.MO.WIDEREL,
          "⇁": O.MO.WIDEREL,
          "⇂": O.MO.RELSTRETCH,
          "⇃": O.MO.RELSTRETCH,
          "⇄": O.MO.WIDEREL,
          "⇅": O.MO.RELSTRETCH,
          "⇆": O.MO.WIDEREL,
          "⇇": O.MO.WIDEREL,
          "⇈": O.MO.RELSTRETCH,
          "⇉": O.MO.WIDEREL,
          "⇊": O.MO.RELSTRETCH,
          "⇋": O.MO.WIDEREL,
          "⇌": O.MO.WIDEREL,
          "⇍": O.MO.RELACCENT,
          "⇎": O.MO.RELACCENT,
          "⇏": O.MO.RELACCENT,
          "⇐": O.MO.WIDEREL,
          "⇑": O.MO.RELSTRETCH,
          "⇒": O.MO.WIDEREL,
          "⇓": O.MO.RELSTRETCH,
          "⇔": O.MO.WIDEREL,
          "⇕": O.MO.RELSTRETCH,
          "⇖": O.MO.RELSTRETCH,
          "⇗": O.MO.RELSTRETCH,
          "⇘": O.MO.RELSTRETCH,
          "⇙": O.MO.RELSTRETCH,
          "⇚": O.MO.WIDEREL,
          "⇛": O.MO.WIDEREL,
          "⇜": O.MO.WIDEREL,
          "⇝": O.MO.WIDEREL,
          "⇞": O.MO.REL,
          "⇟": O.MO.REL,
          "⇠": O.MO.WIDEREL,
          "⇡": O.MO.RELSTRETCH,
          "⇢": O.MO.WIDEREL,
          "⇣": O.MO.RELSTRETCH,
          "⇤": O.MO.WIDEREL,
          "⇥": O.MO.WIDEREL,
          "⇦": O.MO.WIDEREL,
          "⇧": O.MO.RELSTRETCH,
          "⇨": O.MO.WIDEREL,
          "⇩": O.MO.RELSTRETCH,
          "⇪": O.MO.RELSTRETCH,
          "⇫": O.MO.RELSTRETCH,
          "⇬": O.MO.RELSTRETCH,
          "⇭": O.MO.RELSTRETCH,
          "⇮": O.MO.RELSTRETCH,
          "⇯": O.MO.RELSTRETCH,
          "⇰": O.MO.WIDEREL,
          "⇱": O.MO.REL,
          "⇲": O.MO.REL,
          "⇳": O.MO.RELSTRETCH,
          "⇴": O.MO.RELACCENT,
          "⇵": O.MO.RELSTRETCH,
          "⇶": O.MO.WIDEREL,
          "⇷": O.MO.RELACCENT,
          "⇸": O.MO.RELACCENT,
          "⇹": O.MO.RELACCENT,
          "⇺": O.MO.RELACCENT,
          "⇻": O.MO.RELACCENT,
          "⇼": O.MO.RELACCENT,
          "⇽": O.MO.WIDEREL,
          "⇾": O.MO.WIDEREL,
          "⇿": O.MO.WIDEREL,
          "∁": V(1, 2, H.TEXCLASS.ORD),
          "∅": O.MO.ORD,
          "∆": O.MO.BIN3,
          "∈": O.MO.REL,
          "∉": O.MO.REL,
          "∊": O.MO.REL,
          "∋": O.MO.REL,
          "∌": O.MO.REL,
          "∍": O.MO.REL,
          "∎": O.MO.BIN3,
          "−": O.MO.BIN4,
          "∓": O.MO.BIN4,
          "∔": O.MO.BIN4,
          "∕": O.MO.TALLBIN,
          "∖": O.MO.BIN4,
          "∗": O.MO.BIN4,
          "∘": O.MO.BIN4,
          "∙": O.MO.BIN4,
          "∝": O.MO.REL,
          "∞": O.MO.ORD,
          "∟": O.MO.REL,
          "∣": O.MO.REL,
          "∤": O.MO.REL,
          "∥": O.MO.REL,
          "∦": O.MO.REL,
          "∧": O.MO.BIN4,
          "∨": O.MO.BIN4,
          "∩": O.MO.BIN4,
          "∪": O.MO.BIN4,
          "∴": O.MO.REL,
          "∵": O.MO.REL,
          "∶": O.MO.REL,
          "∷": O.MO.REL,
          "∸": O.MO.BIN4,
          "∹": O.MO.REL,
          "∺": O.MO.BIN4,
          "∻": O.MO.REL,
          "∼": O.MO.REL,
          "∽": O.MO.REL,
          "∽̱": O.MO.BIN3,
          "∾": O.MO.REL,
          "∿": O.MO.BIN3,
          "≀": O.MO.BIN4,
          "≁": O.MO.REL,
          "≂": O.MO.REL,
          "≂̸": O.MO.REL,
          "≃": O.MO.REL,
          "≄": O.MO.REL,
          "≅": O.MO.REL,
          "≆": O.MO.REL,
          "≇": O.MO.REL,
          "≈": O.MO.REL,
          "≉": O.MO.REL,
          "≊": O.MO.REL,
          "≋": O.MO.REL,
          "≌": O.MO.REL,
          "≍": O.MO.REL,
          "≎": O.MO.REL,
          "≎̸": O.MO.REL,
          "≏": O.MO.REL,
          "≏̸": O.MO.REL,
          "≐": O.MO.REL,
          "≑": O.MO.REL,
          "≒": O.MO.REL,
          "≓": O.MO.REL,
          "≔": O.MO.REL,
          "≕": O.MO.REL,
          "≖": O.MO.REL,
          "≗": O.MO.REL,
          "≘": O.MO.REL,
          "≙": O.MO.REL,
          "≚": O.MO.REL,
          "≛": O.MO.REL,
          "≜": O.MO.REL,
          "≝": O.MO.REL,
          "≞": O.MO.REL,
          "≟": O.MO.REL,
          "≠": O.MO.REL,
          "≡": O.MO.REL,
          "≢": O.MO.REL,
          "≣": O.MO.REL,
          "≤": O.MO.REL,
          "≥": O.MO.REL,
          "≦": O.MO.REL,
          "≦̸": O.MO.REL,
          "≧": O.MO.REL,
          "≨": O.MO.REL,
          "≩": O.MO.REL,
          "≪": O.MO.REL,
          "≪̸": O.MO.REL,
          "≫": O.MO.REL,
          "≫̸": O.MO.REL,
          "≬": O.MO.REL,
          "≭": O.MO.REL,
          "≮": O.MO.REL,
          "≯": O.MO.REL,
          "≰": O.MO.REL,
          "≱": O.MO.REL,
          "≲": O.MO.REL,
          "≳": O.MO.REL,
          "≴": O.MO.REL,
          "≵": O.MO.REL,
          "≶": O.MO.REL,
          "≷": O.MO.REL,
          "≸": O.MO.REL,
          "≹": O.MO.REL,
          "≺": O.MO.REL,
          "≻": O.MO.REL,
          "≼": O.MO.REL,
          "≽": O.MO.REL,
          "≾": O.MO.REL,
          "≿": O.MO.REL,
          "≿̸": O.MO.REL,
          "⊀": O.MO.REL,
          "⊁": O.MO.REL,
          "⊂": O.MO.REL,
          "⊂⃒": O.MO.REL,
          "⊃": O.MO.REL,
          "⊃⃒": O.MO.REL,
          "⊄": O.MO.REL,
          "⊅": O.MO.REL,
          "⊆": O.MO.REL,
          "⊇": O.MO.REL,
          "⊈": O.MO.REL,
          "⊉": O.MO.REL,
          "⊊": O.MO.REL,
          "⊋": O.MO.REL,
          "⊌": O.MO.BIN4,
          "⊍": O.MO.BIN4,
          "⊎": O.MO.BIN4,
          "⊏": O.MO.REL,
          "⊏̸": O.MO.REL,
          "⊐": O.MO.REL,
          "⊐̸": O.MO.REL,
          "⊑": O.MO.REL,
          "⊒": O.MO.REL,
          "⊓": O.MO.BIN4,
          "⊔": O.MO.BIN4,
          "⊕": O.MO.BIN4,
          "⊖": O.MO.BIN4,
          "⊗": O.MO.BIN4,
          "⊘": O.MO.BIN4,
          "⊙": O.MO.BIN4,
          "⊚": O.MO.BIN4,
          "⊛": O.MO.BIN4,
          "⊜": O.MO.BIN4,
          "⊝": O.MO.BIN4,
          "⊞": O.MO.BIN4,
          "⊟": O.MO.BIN4,
          "⊠": O.MO.BIN4,
          "⊡": O.MO.BIN4,
          "⊢": O.MO.REL,
          "⊣": O.MO.REL,
          "⊤": O.MO.ORD55,
          "⊥": O.MO.REL,
          "⊦": O.MO.REL,
          "⊧": O.MO.REL,
          "⊨": O.MO.REL,
          "⊩": O.MO.REL,
          "⊪": O.MO.REL,
          "⊫": O.MO.REL,
          "⊬": O.MO.REL,
          "⊭": O.MO.REL,
          "⊮": O.MO.REL,
          "⊯": O.MO.REL,
          "⊰": O.MO.REL,
          "⊱": O.MO.REL,
          "⊲": O.MO.REL,
          "⊳": O.MO.REL,
          "⊴": O.MO.REL,
          "⊵": O.MO.REL,
          "⊶": O.MO.REL,
          "⊷": O.MO.REL,
          "⊸": O.MO.REL,
          "⊹": O.MO.REL,
          "⊺": O.MO.BIN4,
          "⊻": O.MO.BIN4,
          "⊼": O.MO.BIN4,
          "⊽": O.MO.BIN4,
          "⊾": O.MO.BIN3,
          "⊿": O.MO.BIN3,
          "⋄": O.MO.BIN4,
          "⋅": O.MO.BIN4,
          "⋆": O.MO.BIN4,
          "⋇": O.MO.BIN4,
          "⋈": O.MO.REL,
          "⋉": O.MO.BIN4,
          "⋊": O.MO.BIN4,
          "⋋": O.MO.BIN4,
          "⋌": O.MO.BIN4,
          "⋍": O.MO.REL,
          "⋎": O.MO.BIN4,
          "⋏": O.MO.BIN4,
          "⋐": O.MO.REL,
          "⋑": O.MO.REL,
          "⋒": O.MO.BIN4,
          "⋓": O.MO.BIN4,
          "⋔": O.MO.REL,
          "⋕": O.MO.REL,
          "⋖": O.MO.REL,
          "⋗": O.MO.REL,
          "⋘": O.MO.REL,
          "⋙": O.MO.REL,
          "⋚": O.MO.REL,
          "⋛": O.MO.REL,
          "⋜": O.MO.REL,
          "⋝": O.MO.REL,
          "⋞": O.MO.REL,
          "⋟": O.MO.REL,
          "⋠": O.MO.REL,
          "⋡": O.MO.REL,
          "⋢": O.MO.REL,
          "⋣": O.MO.REL,
          "⋤": O.MO.REL,
          "⋥": O.MO.REL,
          "⋦": O.MO.REL,
          "⋧": O.MO.REL,
          "⋨": O.MO.REL,
          "⋩": O.MO.REL,
          "⋪": O.MO.REL,
          "⋫": O.MO.REL,
          "⋬": O.MO.REL,
          "⋭": O.MO.REL,
          "⋮": O.MO.ORD55,
          "⋯": O.MO.INNER,
          "⋰": O.MO.REL,
          "⋱": [5, 5, H.TEXCLASS.INNER, null],
          "⋲": O.MO.REL,
          "⋳": O.MO.REL,
          "⋴": O.MO.REL,
          "⋵": O.MO.REL,
          "⋶": O.MO.REL,
          "⋷": O.MO.REL,
          "⋸": O.MO.REL,
          "⋹": O.MO.REL,
          "⋺": O.MO.REL,
          "⋻": O.MO.REL,
          "⋼": O.MO.REL,
          "⋽": O.MO.REL,
          "⋾": O.MO.REL,
          "⋿": O.MO.REL,
          "⌅": O.MO.BIN3,
          "⌆": O.MO.BIN3,
          "⌢": O.MO.REL4,
          "⌣": O.MO.REL4,
          "〈": O.MO.OPEN,
          "〉": O.MO.CLOSE,
          "⎪": O.MO.ORD,
          "⎯": [0, 0, H.TEXCLASS.ORD, { stretchy: !0 }],
          "⎰": O.MO.OPEN,
          "⎱": O.MO.CLOSE,
          "─": O.MO.ORD,
          "△": O.MO.BIN4,
          "▵": O.MO.BIN4,
          "▹": O.MO.BIN4,
          "▽": O.MO.BIN4,
          "▿": O.MO.BIN4,
          "◃": O.MO.BIN4,
          "◯": O.MO.BIN3,
          "♠": O.MO.ORD,
          "♡": O.MO.ORD,
          "♢": O.MO.ORD,
          "♣": O.MO.ORD,
          "❘": O.MO.REL,
          "⟰": O.MO.RELSTRETCH,
          "⟱": O.MO.RELSTRETCH,
          "⟵": O.MO.WIDEREL,
          "⟶": O.MO.WIDEREL,
          "⟷": O.MO.WIDEREL,
          "⟸": O.MO.WIDEREL,
          "⟹": O.MO.WIDEREL,
          "⟺": O.MO.WIDEREL,
          "⟻": O.MO.WIDEREL,
          "⟼": O.MO.WIDEREL,
          "⟽": O.MO.WIDEREL,
          "⟾": O.MO.WIDEREL,
          "⟿": O.MO.WIDEREL,
          "⤀": O.MO.RELACCENT,
          "⤁": O.MO.RELACCENT,
          "⤂": O.MO.RELACCENT,
          "⤃": O.MO.RELACCENT,
          "⤄": O.MO.RELACCENT,
          "⤅": O.MO.RELACCENT,
          "⤆": O.MO.RELACCENT,
          "⤇": O.MO.RELACCENT,
          "⤈": O.MO.REL,
          "⤉": O.MO.REL,
          "⤊": O.MO.RELSTRETCH,
          "⤋": O.MO.RELSTRETCH,
          "⤌": O.MO.WIDEREL,
          "⤍": O.MO.WIDEREL,
          "⤎": O.MO.WIDEREL,
          "⤏": O.MO.WIDEREL,
          "⤐": O.MO.WIDEREL,
          "⤑": O.MO.RELACCENT,
          "⤒": O.MO.RELSTRETCH,
          "⤓": O.MO.RELSTRETCH,
          "⤔": O.MO.RELACCENT,
          "⤕": O.MO.RELACCENT,
          "⤖": O.MO.RELACCENT,
          "⤗": O.MO.RELACCENT,
          "⤘": O.MO.RELACCENT,
          "⤙": O.MO.RELACCENT,
          "⤚": O.MO.RELACCENT,
          "⤛": O.MO.RELACCENT,
          "⤜": O.MO.RELACCENT,
          "⤝": O.MO.RELACCENT,
          "⤞": O.MO.RELACCENT,
          "⤟": O.MO.RELACCENT,
          "⤠": O.MO.RELACCENT,
          "⤡": O.MO.RELSTRETCH,
          "⤢": O.MO.RELSTRETCH,
          "⤣": O.MO.REL,
          "⤤": O.MO.REL,
          "⤥": O.MO.REL,
          "⤦": O.MO.REL,
          "⤧": O.MO.REL,
          "⤨": O.MO.REL,
          "⤩": O.MO.REL,
          "⤪": O.MO.REL,
          "⤫": O.MO.REL,
          "⤬": O.MO.REL,
          "⤭": O.MO.REL,
          "⤮": O.MO.REL,
          "⤯": O.MO.REL,
          "⤰": O.MO.REL,
          "⤱": O.MO.REL,
          "⤲": O.MO.REL,
          "⤳": O.MO.RELACCENT,
          "⤴": O.MO.REL,
          "⤵": O.MO.REL,
          "⤶": O.MO.REL,
          "⤷": O.MO.REL,
          "⤸": O.MO.REL,
          "⤹": O.MO.REL,
          "⤺": O.MO.RELACCENT,
          "⤻": O.MO.RELACCENT,
          "⤼": O.MO.RELACCENT,
          "⤽": O.MO.RELACCENT,
          "⤾": O.MO.REL,
          "⤿": O.MO.REL,
          "⥀": O.MO.REL,
          "⥁": O.MO.REL,
          "⥂": O.MO.RELACCENT,
          "⥃": O.MO.RELACCENT,
          "⥄": O.MO.RELACCENT,
          "⥅": O.MO.RELACCENT,
          "⥆": O.MO.RELACCENT,
          "⥇": O.MO.RELACCENT,
          "⥈": O.MO.RELACCENT,
          "⥉": O.MO.REL,
          "⥊": O.MO.RELACCENT,
          "⥋": O.MO.RELACCENT,
          "⥌": O.MO.REL,
          "⥍": O.MO.REL,
          "⥎": O.MO.WIDEREL,
          "⥏": O.MO.RELSTRETCH,
          "⥐": O.MO.WIDEREL,
          "⥑": O.MO.RELSTRETCH,
          "⥒": O.MO.WIDEREL,
          "⥓": O.MO.WIDEREL,
          "⥔": O.MO.RELSTRETCH,
          "⥕": O.MO.RELSTRETCH,
          "⥖": O.MO.RELSTRETCH,
          "⥗": O.MO.RELSTRETCH,
          "⥘": O.MO.RELSTRETCH,
          "⥙": O.MO.RELSTRETCH,
          "⥚": O.MO.WIDEREL,
          "⥛": O.MO.WIDEREL,
          "⥜": O.MO.RELSTRETCH,
          "⥝": O.MO.RELSTRETCH,
          "⥞": O.MO.WIDEREL,
          "⥟": O.MO.WIDEREL,
          "⥠": O.MO.RELSTRETCH,
          "⥡": O.MO.RELSTRETCH,
          "⥢": O.MO.RELACCENT,
          "⥣": O.MO.REL,
          "⥤": O.MO.RELACCENT,
          "⥥": O.MO.REL,
          "⥦": O.MO.RELACCENT,
          "⥧": O.MO.RELACCENT,
          "⥨": O.MO.RELACCENT,
          "⥩": O.MO.RELACCENT,
          "⥪": O.MO.RELACCENT,
          "⥫": O.MO.RELACCENT,
          "⥬": O.MO.RELACCENT,
          "⥭": O.MO.RELACCENT,
          "⥮": O.MO.RELSTRETCH,
          "⥯": O.MO.RELSTRETCH,
          "⥰": O.MO.RELACCENT,
          "⥱": O.MO.RELACCENT,
          "⥲": O.MO.RELACCENT,
          "⥳": O.MO.RELACCENT,
          "⥴": O.MO.RELACCENT,
          "⥵": O.MO.RELACCENT,
          "⥶": O.MO.RELACCENT,
          "⥷": O.MO.RELACCENT,
          "⥸": O.MO.RELACCENT,
          "⥹": O.MO.RELACCENT,
          "⥺": O.MO.RELACCENT,
          "⥻": O.MO.RELACCENT,
          "⥼": O.MO.RELACCENT,
          "⥽": O.MO.RELACCENT,
          "⥾": O.MO.REL,
          "⥿": O.MO.REL,
          "⦁": O.MO.BIN3,
          "⦂": O.MO.BIN3,
          "⦙": O.MO.BIN3,
          "⦚": O.MO.BIN3,
          "⦛": O.MO.BIN3,
          "⦜": O.MO.BIN3,
          "⦝": O.MO.BIN3,
          "⦞": O.MO.BIN3,
          "⦟": O.MO.BIN3,
          "⦠": O.MO.BIN3,
          "⦡": O.MO.BIN3,
          "⦢": O.MO.BIN3,
          "⦣": O.MO.BIN3,
          "⦤": O.MO.BIN3,
          "⦥": O.MO.BIN3,
          "⦦": O.MO.BIN3,
          "⦧": O.MO.BIN3,
          "⦨": O.MO.BIN3,
          "⦩": O.MO.BIN3,
          "⦪": O.MO.BIN3,
          "⦫": O.MO.BIN3,
          "⦬": O.MO.BIN3,
          "⦭": O.MO.BIN3,
          "⦮": O.MO.BIN3,
          "⦯": O.MO.BIN3,
          "⦰": O.MO.BIN3,
          "⦱": O.MO.BIN3,
          "⦲": O.MO.BIN3,
          "⦳": O.MO.BIN3,
          "⦴": O.MO.BIN3,
          "⦵": O.MO.BIN3,
          "⦶": O.MO.BIN4,
          "⦷": O.MO.BIN4,
          "⦸": O.MO.BIN4,
          "⦹": O.MO.BIN4,
          "⦺": O.MO.BIN4,
          "⦻": O.MO.BIN4,
          "⦼": O.MO.BIN4,
          "⦽": O.MO.BIN4,
          "⦾": O.MO.BIN4,
          "⦿": O.MO.BIN4,
          "⧀": O.MO.REL,
          "⧁": O.MO.REL,
          "⧂": O.MO.BIN3,
          "⧃": O.MO.BIN3,
          "⧄": O.MO.BIN4,
          "⧅": O.MO.BIN4,
          "⧆": O.MO.BIN4,
          "⧇": O.MO.BIN4,
          "⧈": O.MO.BIN4,
          "⧉": O.MO.BIN3,
          "⧊": O.MO.BIN3,
          "⧋": O.MO.BIN3,
          "⧌": O.MO.BIN3,
          "⧍": O.MO.BIN3,
          "⧎": O.MO.REL,
          "⧏": O.MO.REL,
          "⧏̸": O.MO.REL,
          "⧐": O.MO.REL,
          "⧐̸": O.MO.REL,
          "⧑": O.MO.REL,
          "⧒": O.MO.REL,
          "⧓": O.MO.REL,
          "⧔": O.MO.REL,
          "⧕": O.MO.REL,
          "⧖": O.MO.BIN4,
          "⧗": O.MO.BIN4,
          "⧘": O.MO.BIN3,
          "⧙": O.MO.BIN3,
          "⧛": O.MO.BIN3,
          "⧜": O.MO.BIN3,
          "⧝": O.MO.BIN3,
          "⧞": O.MO.REL,
          "⧟": O.MO.BIN3,
          "⧠": O.MO.BIN3,
          "⧡": O.MO.REL,
          "⧢": O.MO.BIN4,
          "⧣": O.MO.REL,
          "⧤": O.MO.REL,
          "⧥": O.MO.REL,
          "⧦": O.MO.REL,
          "⧧": O.MO.BIN3,
          "⧨": O.MO.BIN3,
          "⧩": O.MO.BIN3,
          "⧪": O.MO.BIN3,
          "⧫": O.MO.BIN3,
          "⧬": O.MO.BIN3,
          "⧭": O.MO.BIN3,
          "⧮": O.MO.BIN3,
          "⧯": O.MO.BIN3,
          "⧰": O.MO.BIN3,
          "⧱": O.MO.BIN3,
          "⧲": O.MO.BIN3,
          "⧳": O.MO.BIN3,
          "⧴": O.MO.REL,
          "⧵": O.MO.BIN4,
          "⧶": O.MO.BIN4,
          "⧷": O.MO.BIN4,
          "⧸": O.MO.BIN3,
          "⧹": O.MO.BIN3,
          "⧺": O.MO.BIN3,
          "⧻": O.MO.BIN3,
          "⧾": O.MO.BIN4,
          "⧿": O.MO.BIN4,
          "⨝": O.MO.BIN3,
          "⨞": O.MO.BIN3,
          "⨟": O.MO.BIN3,
          "⨠": O.MO.BIN3,
          "⨡": O.MO.BIN3,
          "⨢": O.MO.BIN4,
          "⨣": O.MO.BIN4,
          "⨤": O.MO.BIN4,
          "⨥": O.MO.BIN4,
          "⨦": O.MO.BIN4,
          "⨧": O.MO.BIN4,
          "⨨": O.MO.BIN4,
          "⨩": O.MO.BIN4,
          "⨪": O.MO.BIN4,
          "⨫": O.MO.BIN4,
          "⨬": O.MO.BIN4,
          "⨭": O.MO.BIN4,
          "⨮": O.MO.BIN4,
          "⨯": O.MO.BIN4,
          "⨰": O.MO.BIN4,
          "⨱": O.MO.BIN4,
          "⨲": O.MO.BIN4,
          "⨳": O.MO.BIN4,
          "⨴": O.MO.BIN4,
          "⨵": O.MO.BIN4,
          "⨶": O.MO.BIN4,
          "⨷": O.MO.BIN4,
          "⨸": O.MO.BIN4,
          "⨹": O.MO.BIN4,
          "⨺": O.MO.BIN4,
          "⨻": O.MO.BIN4,
          "⨼": O.MO.BIN4,
          "⨽": O.MO.BIN4,
          "⨾": O.MO.BIN4,
          "⨿": O.MO.BIN4,
          "⩀": O.MO.BIN4,
          "⩁": O.MO.BIN4,
          "⩂": O.MO.BIN4,
          "⩃": O.MO.BIN4,
          "⩄": O.MO.BIN4,
          "⩅": O.MO.BIN4,
          "⩆": O.MO.BIN4,
          "⩇": O.MO.BIN4,
          "⩈": O.MO.BIN4,
          "⩉": O.MO.BIN4,
          "⩊": O.MO.BIN4,
          "⩋": O.MO.BIN4,
          "⩌": O.MO.BIN4,
          "⩍": O.MO.BIN4,
          "⩎": O.MO.BIN4,
          "⩏": O.MO.BIN4,
          "⩐": O.MO.BIN4,
          "⩑": O.MO.BIN4,
          "⩒": O.MO.BIN4,
          "⩓": O.MO.BIN4,
          "⩔": O.MO.BIN4,
          "⩕": O.MO.BIN4,
          "⩖": O.MO.BIN4,
          "⩗": O.MO.BIN4,
          "⩘": O.MO.BIN4,
          "⩙": O.MO.REL,
          "⩚": O.MO.BIN4,
          "⩛": O.MO.BIN4,
          "⩜": O.MO.BIN4,
          "⩝": O.MO.BIN4,
          "⩞": O.MO.BIN4,
          "⩟": O.MO.BIN4,
          "⩠": O.MO.BIN4,
          "⩡": O.MO.BIN4,
          "⩢": O.MO.BIN4,
          "⩣": O.MO.BIN4,
          "⩤": O.MO.BIN4,
          "⩥": O.MO.BIN4,
          "⩦": O.MO.REL,
          "⩧": O.MO.REL,
          "⩨": O.MO.REL,
          "⩩": O.MO.REL,
          "⩪": O.MO.REL,
          "⩫": O.MO.REL,
          "⩬": O.MO.REL,
          "⩭": O.MO.REL,
          "⩮": O.MO.REL,
          "⩯": O.MO.REL,
          "⩰": O.MO.REL,
          "⩱": O.MO.BIN4,
          "⩲": O.MO.BIN4,
          "⩳": O.MO.REL,
          "⩴": O.MO.REL,
          "⩵": O.MO.REL,
          "⩶": O.MO.REL,
          "⩷": O.MO.REL,
          "⩸": O.MO.REL,
          "⩹": O.MO.REL,
          "⩺": O.MO.REL,
          "⩻": O.MO.REL,
          "⩼": O.MO.REL,
          "⩽": O.MO.REL,
          "⩽̸": O.MO.REL,
          "⩾": O.MO.REL,
          "⩾̸": O.MO.REL,
          "⩿": O.MO.REL,
          "⪀": O.MO.REL,
          "⪁": O.MO.REL,
          "⪂": O.MO.REL,
          "⪃": O.MO.REL,
          "⪄": O.MO.REL,
          "⪅": O.MO.REL,
          "⪆": O.MO.REL,
          "⪇": O.MO.REL,
          "⪈": O.MO.REL,
          "⪉": O.MO.REL,
          "⪊": O.MO.REL,
          "⪋": O.MO.REL,
          "⪌": O.MO.REL,
          "⪍": O.MO.REL,
          "⪎": O.MO.REL,
          "⪏": O.MO.REL,
          "⪐": O.MO.REL,
          "⪑": O.MO.REL,
          "⪒": O.MO.REL,
          "⪓": O.MO.REL,
          "⪔": O.MO.REL,
          "⪕": O.MO.REL,
          "⪖": O.MO.REL,
          "⪗": O.MO.REL,
          "⪘": O.MO.REL,
          "⪙": O.MO.REL,
          "⪚": O.MO.REL,
          "⪛": O.MO.REL,
          "⪜": O.MO.REL,
          "⪝": O.MO.REL,
          "⪞": O.MO.REL,
          "⪟": O.MO.REL,
          "⪠": O.MO.REL,
          "⪡": O.MO.REL,
          "⪡̸": O.MO.REL,
          "⪢": O.MO.REL,
          "⪢̸": O.MO.REL,
          "⪣": O.MO.REL,
          "⪤": O.MO.REL,
          "⪥": O.MO.REL,
          "⪦": O.MO.REL,
          "⪧": O.MO.REL,
          "⪨": O.MO.REL,
          "⪩": O.MO.REL,
          "⪪": O.MO.REL,
          "⪫": O.MO.REL,
          "⪬": O.MO.REL,
          "⪭": O.MO.REL,
          "⪮": O.MO.REL,
          "⪯": O.MO.REL,
          "⪯̸": O.MO.REL,
          "⪰": O.MO.REL,
          "⪰̸": O.MO.REL,
          "⪱": O.MO.REL,
          "⪲": O.MO.REL,
          "⪳": O.MO.REL,
          "⪴": O.MO.REL,
          "⪵": O.MO.REL,
          "⪶": O.MO.REL,
          "⪷": O.MO.REL,
          "⪸": O.MO.REL,
          "⪹": O.MO.REL,
          "⪺": O.MO.REL,
          "⪻": O.MO.REL,
          "⪼": O.MO.REL,
          "⪽": O.MO.REL,
          "⪾": O.MO.REL,
          "⪿": O.MO.REL,
          "⫀": O.MO.REL,
          "⫁": O.MO.REL,
          "⫂": O.MO.REL,
          "⫃": O.MO.REL,
          "⫄": O.MO.REL,
          "⫅": O.MO.REL,
          "⫆": O.MO.REL,
          "⫇": O.MO.REL,
          "⫈": O.MO.REL,
          "⫉": O.MO.REL,
          "⫊": O.MO.REL,
          "⫋": O.MO.REL,
          "⫌": O.MO.REL,
          "⫍": O.MO.REL,
          "⫎": O.MO.REL,
          "⫏": O.MO.REL,
          "⫐": O.MO.REL,
          "⫑": O.MO.REL,
          "⫒": O.MO.REL,
          "⫓": O.MO.REL,
          "⫔": O.MO.REL,
          "⫕": O.MO.REL,
          "⫖": O.MO.REL,
          "⫗": O.MO.REL,
          "⫘": O.MO.REL,
          "⫙": O.MO.REL,
          "⫚": O.MO.REL,
          "⫛": O.MO.REL,
          "⫝": O.MO.REL,
          "⫝̸": O.MO.REL,
          "⫞": O.MO.REL,
          "⫟": O.MO.REL,
          "⫠": O.MO.REL,
          "⫡": O.MO.REL,
          "⫢": O.MO.REL,
          "⫣": O.MO.REL,
          "⫤": O.MO.REL,
          "⫥": O.MO.REL,
          "⫦": O.MO.REL,
          "⫧": O.MO.REL,
          "⫨": O.MO.REL,
          "⫩": O.MO.REL,
          "⫪": O.MO.REL,
          "⫫": O.MO.REL,
          "⫬": O.MO.REL,
          "⫭": O.MO.REL,
          "⫮": O.MO.REL,
          "⫯": O.MO.REL,
          "⫰": O.MO.REL,
          "⫱": O.MO.REL,
          "⫲": O.MO.REL,
          "⫳": O.MO.REL,
          "⫴": O.MO.BIN4,
          "⫵": O.MO.BIN4,
          "⫶": O.MO.BIN4,
          "⫷": O.MO.REL,
          "⫸": O.MO.REL,
          "⫹": O.MO.REL,
          "⫺": O.MO.REL,
          "⫻": O.MO.BIN4,
          "⫽": O.MO.BIN4,
          "⫾": O.MO.BIN3,
          "⭅": O.MO.RELSTRETCH,
          "⭆": O.MO.RELSTRETCH,
          "〈": O.MO.OPEN,
          "〉": O.MO.CLOSE,
          "︷": O.MO.WIDEACCENT,
          "︸": O.MO.WIDEACCENT
        }
      }, O.OPTABLE.infix["^"] = O.MO.WIDEREL, O.OPTABLE.infix._ = O.MO.WIDEREL, O.OPTABLE.infix["⫝̸"] = O.MO.REL;
    }(OperatorDictionary)), OperatorDictionary;
  }
  var hasRequiredMo$2;
  function requireMo$2() {
    if (hasRequiredMo$2) return mo$2;
    hasRequiredMo$2 = 1;
    var O = mo$2.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = mo$2.__assign || function() {
      return _ = Object.assign || function(te) {
        for (var ne, ee = 1, ae = arguments.length; ee < ae; ee++) {
          ne = arguments[ee];
          for (var oe in ne) Object.prototype.hasOwnProperty.call(ne, oe) && (te[oe] = ne[oe]);
        }
        return te;
      }, _.apply(this, arguments);
    }, H = mo$2.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    }, V = mo$2.__values || function(te) {
      var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
      if (ee) return ee.call(te);
      if (te && typeof te.length == "number") return {
        next: function() {
          return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
        }
      };
      throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mo$2, "__esModule", { value: !0 }), mo$2.MmlMo = void 0;
    var q = requireMmlNode(), G = requireOperatorDictionary(), J = requireString(), Z = function(te) {
      O(ne, te);
      function ne() {
        var ee = te !== null && te.apply(this, arguments) || this;
        return ee._texClass = null, ee.lspace = 5 / 18, ee.rspace = 5 / 18, ee;
      }
      return Object.defineProperty(ne.prototype, "texClass", {
        get: function() {
          if (this._texClass === null) {
            var ee = this.getText(), ae = H(this.handleExplicitForm(this.getForms()), 3), oe = ae[0], se = ae[1], ce = ae[2], le = this.constructor.OPTABLE, ue = le[oe][ee] || le[se][ee] || le[ce][ee];
            return ue ? ue[2] : q.TEXCLASS.REL;
          }
          return this._texClass;
        },
        set: function(ee) {
          this._texClass = ee;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "kind", {
        get: function() {
          return "mo";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "isEmbellished", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "hasNewLine", {
        get: function() {
          return this.attributes.get("linebreak") === "newline";
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.coreParent = function() {
        for (var ee = this, ae = this, oe = this.factory.getNodeClass("math"); ae && ae.isEmbellished && ae.coreMO() === this && !(ae instanceof oe); )
          ee = ae, ae = ae.parent;
        return ee;
      }, ne.prototype.coreText = function(ee) {
        if (!ee)
          return "";
        if (ee.isEmbellished)
          return ee.coreMO().getText();
        for (; ((ee.isKind("mrow") || ee.isKind("TeXAtom") && ee.texClass !== q.TEXCLASS.VCENTER || ee.isKind("mstyle") || ee.isKind("mphantom")) && ee.childNodes.length === 1 || ee.isKind("munderover")) && ee.childNodes[0]; )
          ee = ee.childNodes[0];
        return ee.isToken ? ee.getText() : "";
      }, ne.prototype.hasSpacingAttributes = function() {
        return this.attributes.isSet("lspace") || this.attributes.isSet("rspace");
      }, Object.defineProperty(ne.prototype, "isAccent", {
        get: function() {
          var ee = !1, ae = this.coreParent().parent;
          if (ae) {
            var oe = ae.isKind("mover") ? ae.childNodes[ae.over].coreMO() ? "accent" : "" : ae.isKind("munder") ? ae.childNodes[ae.under].coreMO() ? "accentunder" : "" : ae.isKind("munderover") ? this === ae.childNodes[ae.over].coreMO() ? "accent" : this === ae.childNodes[ae.under].coreMO() ? "accentunder" : "" : "";
            if (oe) {
              var se = ae.attributes.getExplicit(oe);
              ee = se !== void 0 ? ee : this.attributes.get("accent");
            }
          }
          return ee;
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.setTeXclass = function(ee) {
        var ae = this.attributes.getList("form", "fence"), oe = ae.form, se = ae.fence;
        return this.getProperty("texClass") === void 0 && (this.attributes.isSet("lspace") || this.attributes.isSet("rspace")) ? null : (se && this.texClass === q.TEXCLASS.REL && (oe === "prefix" && (this.texClass = q.TEXCLASS.OPEN), oe === "postfix" && (this.texClass = q.TEXCLASS.CLOSE)), this.adjustTeXclass(ee));
      }, ne.prototype.adjustTeXclass = function(ee) {
        var ae = this.texClass, oe = this.prevClass;
        if (ae === q.TEXCLASS.NONE)
          return ee;
        if (ee ? (ee.getProperty("autoOP") && (ae === q.TEXCLASS.BIN || ae === q.TEXCLASS.REL) && (oe = ee.texClass = q.TEXCLASS.ORD), oe = this.prevClass = ee.texClass || q.TEXCLASS.ORD, this.prevLevel = this.attributes.getInherited("scriptlevel")) : oe = this.prevClass = q.TEXCLASS.NONE, ae === q.TEXCLASS.BIN && (oe === q.TEXCLASS.NONE || oe === q.TEXCLASS.BIN || oe === q.TEXCLASS.OP || oe === q.TEXCLASS.REL || oe === q.TEXCLASS.OPEN || oe === q.TEXCLASS.PUNCT))
          this.texClass = q.TEXCLASS.ORD;
        else if (oe === q.TEXCLASS.BIN && (ae === q.TEXCLASS.REL || ae === q.TEXCLASS.CLOSE || ae === q.TEXCLASS.PUNCT))
          ee.texClass = this.prevClass = q.TEXCLASS.ORD;
        else if (ae === q.TEXCLASS.BIN) {
          for (var se = this, ce = this.parent; ce && ce.parent && ce.isEmbellished && (ce.childNodes.length === 1 || !ce.isKind("mrow") && ce.core() === se); )
            se = ce, ce = ce.parent;
          ce.childNodes[ce.childNodes.length - 1] === se && (this.texClass = q.TEXCLASS.ORD);
        }
        return this;
      }, ne.prototype.setInheritedAttributes = function(ee, ae, oe, se) {
        ee === void 0 && (ee = {}), ae === void 0 && (ae = !1), oe === void 0 && (oe = 0), se === void 0 && (se = !1), te.prototype.setInheritedAttributes.call(this, ee, ae, oe, se);
        var ce = this.getText();
        this.checkOperatorTable(ce), this.checkPseudoScripts(ce), this.checkPrimes(ce), this.checkMathAccent(ce);
      }, ne.prototype.checkOperatorTable = function(ee) {
        var ae, oe, se = H(this.handleExplicitForm(this.getForms()), 3), ce = se[0], le = se[1], ue = se[2];
        this.attributes.setInherited("form", ce);
        var de = this.constructor.OPTABLE, fe = de[ce][ee] || de[le][ee] || de[ue][ee];
        if (fe) {
          this.getProperty("texClass") === void 0 && (this.texClass = fe[2]);
          try {
            for (var he = V(Object.keys(fe[3] || {})), me = he.next(); !me.done; me = he.next()) {
              var pe = me.value;
              this.attributes.setInherited(pe, fe[3][pe]);
            }
          } catch (ve) {
            ae = { error: ve };
          } finally {
            try {
              me && !me.done && (oe = he.return) && oe.call(he);
            } finally {
              if (ae) throw ae.error;
            }
          }
          this.lspace = (fe[0] + 1) / 18, this.rspace = (fe[1] + 1) / 18;
        } else {
          var ye = (0, G.getRange)(ee);
          if (ye) {
            this.getProperty("texClass") === void 0 && (this.texClass = ye[2]);
            var ge = this.constructor.MMLSPACING[ye[2]];
            this.lspace = (ge[0] + 1) / 18, this.rspace = (ge[1] + 1) / 18;
          }
        }
      }, ne.prototype.getForms = function() {
        for (var ee = this, ae = this.parent, oe = this.Parent; oe && oe.isEmbellished; )
          ee = ae, ae = oe.parent, oe = oe.Parent;
        if (ae && ae.isKind("mrow") && ae.nonSpaceLength() !== 1) {
          if (ae.firstNonSpace() === ee)
            return ["prefix", "infix", "postfix"];
          if (ae.lastNonSpace() === ee)
            return ["postfix", "infix", "prefix"];
        }
        return ["infix", "prefix", "postfix"];
      }, ne.prototype.handleExplicitForm = function(ee) {
        if (this.attributes.isSet("form")) {
          var ae = this.attributes.get("form");
          ee = [ae].concat(ee.filter(function(oe) {
            return oe !== ae;
          }));
        }
        return ee;
      }, ne.prototype.checkPseudoScripts = function(ee) {
        var ae = this.constructor.pseudoScripts;
        if (ee.match(ae)) {
          var oe = this.coreParent().Parent, se = !oe || !(oe.isKind("msubsup") && !oe.isKind("msub"));
          this.setProperty("pseudoscript", se), se && (this.attributes.setInherited("lspace", 0), this.attributes.setInherited("rspace", 0));
        }
      }, ne.prototype.checkPrimes = function(ee) {
        var ae = this.constructor.primes;
        if (ee.match(ae)) {
          var oe = this.constructor.remapPrimes, se = (0, J.unicodeString)((0, J.unicodeChars)(ee).map(function(ce) {
            return oe[ce];
          }));
          this.setProperty("primes", se);
        }
      }, ne.prototype.checkMathAccent = function(ee) {
        var ae = this.Parent;
        if (!(this.getProperty("mathaccent") !== void 0 || !ae || !ae.isKind("munderover"))) {
          var oe = ae.childNodes[0];
          if (!(oe.isEmbellished && oe.coreMO() === this)) {
            var se = this.constructor.mathaccents;
            ee.match(se) && this.setProperty("mathaccent", !0);
          }
        }
      }, ne.defaults = _(_({}, q.AbstractMmlTokenNode.defaults), { form: "infix", fence: !1, separator: !1, lspace: "thickmathspace", rspace: "thickmathspace", stretchy: !1, symmetric: !1, maxsize: "infinity", minsize: "0em", largeop: !1, movablelimits: !1, accent: !1, linebreak: "auto", lineleading: "1ex", linebreakstyle: "before", indentalign: "auto", indentshift: "0", indenttarget: "", indentalignfirst: "indentalign", indentshiftfirst: "indentshift", indentalignlast: "indentalign", indentshiftlast: "indentshift" }), ne.MMLSPACING = G.MMLSPACING, ne.OPTABLE = G.OPTABLE, ne.pseudoScripts = new RegExp([
        "^[\"'*`",
        "ª",
        "°",
        "²-´",
        "¹",
        "º",
        "‘-‟",
        "′-‷⁗",
        "⁰ⁱ",
        "⁴-ⁿ",
        "₀-₎",
        "]+$"
      ].join("")), ne.primes = new RegExp([
        "^[\"'`",
        "‘-‟",
        "]+$"
      ].join("")), ne.remapPrimes = {
        34: 8243,
        39: 8242,
        96: 8245,
        8216: 8245,
        8217: 8242,
        8218: 8242,
        8219: 8245,
        8220: 8246,
        8221: 8243,
        8222: 8243,
        8223: 8246
      }, ne.mathaccents = new RegExp([
        "^[",
        "´́ˊ",
        "`̀ˋ",
        "¨̈",
        "~̃˜",
        "¯̄ˉ",
        "˘̆",
        "ˇ̌",
        "^̂ˆ",
        "→⃗",
        "˙̇",
        "˚̊",
        "⃛",
        "⃜",
        "]$"
      ].join("")), ne;
    }(q.AbstractMmlTokenNode);
    return mo$2.MmlMo = Z, mo$2;
  }
  var hasRequiredNodeUtil;
  function requireNodeUtil() {
    if (hasRequiredNodeUtil) return NodeUtil;
    hasRequiredNodeUtil = 1;
    var O = NodeUtil.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = NodeUtil.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    }, H = NodeUtil.__spreadArray || function(J, Z, te) {
      if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
        (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
      return J.concat(ae || Array.prototype.slice.call(Z));
    };
    Object.defineProperty(NodeUtil, "__esModule", { value: !0 });
    var V = requireMmlNode(), q = requireMo$2(), G;
    return function(J) {
      var Z = /* @__PURE__ */ new Map([
        ["autoOP", !0],
        ["fnOP", !0],
        ["movesupsub", !0],
        ["subsupOK", !0],
        ["texprimestyle", !0],
        ["useHeight", !0],
        ["variantForm", !0],
        ["withDelims", !0],
        ["mathaccent", !0],
        ["open", !0],
        ["close", !0]
      ]);
      function te(Ee) {
        return String.fromCodePoint(parseInt(Ee, 16));
      }
      J.createEntity = te;
      function ne(Ee) {
        return Ee.childNodes;
      }
      J.getChildren = ne;
      function ee(Ee) {
        return Ee.getText();
      }
      J.getText = ee;
      function ae(Ee, xe) {
        var Le, Qe;
        try {
          for (var ke = O(xe), Me = ke.next(); !Me.done; Me = ke.next()) {
            var Re = Me.value;
            Ee.appendChild(Re);
          }
        } catch (De) {
          Le = { error: De };
        } finally {
          try {
            Me && !Me.done && (Qe = ke.return) && Qe.call(ke);
          } finally {
            if (Le) throw Le.error;
          }
        }
      }
      J.appendChildren = ae;
      function oe(Ee, xe, Le) {
        Ee.attributes.set(xe, Le);
      }
      J.setAttribute = oe;
      function se(Ee, xe, Le) {
        Ee.setProperty(xe, Le);
      }
      J.setProperty = se;
      function ce(Ee, xe) {
        var Le, Qe;
        try {
          for (var ke = O(Object.keys(xe)), Me = ke.next(); !Me.done; Me = ke.next()) {
            var Re = Me.value, De = xe[Re];
            Re === "texClass" ? (Ee.texClass = De, Ee.setProperty(Re, De)) : Re === "movablelimits" ? (Ee.setProperty("movablelimits", De), (Ee.isKind("mo") || Ee.isKind("mstyle")) && Ee.attributes.set("movablelimits", De)) : Re === "inferred" || (Z.has(Re) ? Ee.setProperty(Re, De) : Ee.attributes.set(Re, De));
          }
        } catch (Pe) {
          Le = { error: Pe };
        } finally {
          try {
            Me && !Me.done && (Qe = ke.return) && Qe.call(ke);
          } finally {
            if (Le) throw Le.error;
          }
        }
      }
      J.setProperties = ce;
      function le(Ee, xe) {
        return Ee.getProperty(xe);
      }
      J.getProperty = le;
      function ue(Ee, xe) {
        return Ee.attributes.get(xe);
      }
      J.getAttribute = ue;
      function de(Ee) {
        for (var xe = [], Le = 1; Le < arguments.length; Le++)
          xe[Le - 1] = arguments[Le];
        Ee.removeProperty.apply(Ee, H([], _(xe), !1));
      }
      J.removeProperties = de;
      function fe(Ee, xe) {
        return Ee.childNodes[xe];
      }
      J.getChildAt = fe;
      function he(Ee, xe, Le) {
        var Qe = Ee.childNodes;
        Qe[xe] = Le, Le && (Le.parent = Ee);
      }
      J.setChild = he;
      function me(Ee, xe) {
        for (var Le = Ee.childNodes, Qe = 0; Qe < Le.length; Qe++)
          he(xe, Qe, Le[Qe]);
      }
      J.copyChildren = me;
      function pe(Ee, xe) {
        xe.attributes = Ee.attributes, ce(xe, Ee.getAllProperties());
      }
      J.copyAttributes = pe;
      function ye(Ee, xe) {
        return Ee.isKind(xe);
      }
      J.isType = ye;
      function ge(Ee) {
        return Ee.isEmbellished;
      }
      J.isEmbellished = ge;
      function ve(Ee) {
        return Ee.texClass;
      }
      J.getTexClass = ve;
      function be(Ee) {
        return Ee.coreMO();
      }
      J.getCoreMO = be;
      function Ce(Ee) {
        return Ee instanceof V.AbstractMmlNode || Ee instanceof V.AbstractMmlEmptyNode;
      }
      J.isNode = Ce;
      function Se(Ee) {
        return Ee.isInferred;
      }
      J.isInferred = Se;
      function we(Ee) {
        var xe, Le;
        if (!ye(Ee, "mo"))
          return null;
        var Qe = Ee, ke = Qe.getForms();
        try {
          for (var Me = O(ke), Re = Me.next(); !Re.done; Re = Me.next()) {
            var De = Re.value, Pe = q.MmlMo.OPTABLE[De][Qe.getText()];
            if (Pe)
              return Pe;
          }
        } catch ($e) {
          xe = { error: $e };
        } finally {
          try {
            Re && !Re.done && (Le = Me.return) && Le.call(Me);
          } finally {
            if (xe) throw xe.error;
          }
        }
        return null;
      }
      J.getForm = we;
    }(G || (G = {})), NodeUtil.default = G, NodeUtil;
  }
  var hasRequiredFilterUtil;
  function requireFilterUtil() {
    if (hasRequiredFilterUtil) return FilterUtil;
    hasRequiredFilterUtil = 1;
    var O = FilterUtil.__values || function(G) {
      var J = typeof Symbol == "function" && Symbol.iterator, Z = J && G[J], te = 0;
      if (Z) return Z.call(G);
      if (G && typeof G.length == "number") return {
        next: function() {
          return G && te >= G.length && (G = void 0), { value: G && G[te++], done: !G };
        }
      };
      throw new TypeError(J ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = FilterUtil.__importDefault || function(G) {
      return G && G.__esModule ? G : { default: G };
    };
    Object.defineProperty(FilterUtil, "__esModule", { value: !0 });
    var H = requireMmlNode(), V = _(requireNodeUtil()), q;
    return function(G) {
      G.cleanStretchy = function(ee) {
        var ae, oe, se = ee.data;
        try {
          for (var ce = O(se.getList("fixStretchy")), le = ce.next(); !le.done; le = ce.next()) {
            var ue = le.value;
            if (V.default.getProperty(ue, "fixStretchy")) {
              var de = V.default.getForm(ue);
              de && de[3] && de[3].stretchy && V.default.setAttribute(ue, "stretchy", !1);
              var fe = ue.parent;
              if (!V.default.getTexClass(ue) && (!de || !de[2])) {
                var he = se.nodeFactory.create("node", "TeXAtom", [ue]);
                fe.replaceChild(he, ue), he.inheritAttributesFrom(ue);
              }
              V.default.removeProperties(ue, "fixStretchy");
            }
          }
        } catch (me) {
          ae = { error: me };
        } finally {
          try {
            le && !le.done && (oe = ce.return) && oe.call(ce);
          } finally {
            if (ae) throw ae.error;
          }
        }
      }, G.cleanAttributes = function(ee) {
        var ae = ee.data.root;
        ae.walkTree(function(oe, se) {
          var ce, le, ue = oe.attributes;
          if (ue) {
            var de = new Set((ue.get("mjx-keep-attrs") || "").split(/ /));
            delete ue.getAllAttributes()["mjx-keep-attrs"];
            try {
              for (var fe = O(ue.getExplicitNames()), he = fe.next(); !he.done; he = fe.next()) {
                var me = he.value;
                !de.has(me) && ue.attributes[me] === oe.attributes.getInherited(me) && delete ue.attributes[me];
              }
            } catch (pe) {
              ce = { error: pe };
            } finally {
              try {
                he && !he.done && (le = fe.return) && le.call(fe);
              } finally {
                if (ce) throw ce.error;
              }
            }
          }
        }, {});
      }, G.combineRelations = function(ee) {
        var ae, oe, se, ce, le = [];
        try {
          for (var ue = O(ee.data.getList("mo")), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value;
            if (!(fe.getProperty("relationsCombined") || !fe.parent || fe.parent && !V.default.isType(fe.parent, "mrow") || V.default.getTexClass(fe) !== H.TEXCLASS.REL)) {
              for (var he = fe.parent, me = void 0, pe = he.childNodes, ye = pe.indexOf(fe) + 1, ge = V.default.getProperty(fe, "variantForm"); ye < pe.length && (me = pe[ye]) && V.default.isType(me, "mo") && V.default.getTexClass(me) === H.TEXCLASS.REL; )
                if (ge === V.default.getProperty(me, "variantForm") && Z(fe, me)) {
                  V.default.appendChildren(fe, V.default.getChildren(me)), J(["stretchy", "rspace"], fe, me);
                  try {
                    for (var ve = (se = void 0, O(me.getPropertyNames())), be = ve.next(); !be.done; be = ve.next()) {
                      var Ce = be.value;
                      fe.setProperty(Ce, me.getProperty(Ce));
                    }
                  } catch (Se) {
                    se = { error: Se };
                  } finally {
                    try {
                      be && !be.done && (ce = ve.return) && ce.call(ve);
                    } finally {
                      if (se) throw se.error;
                    }
                  }
                  pe.splice(ye, 1), le.push(me), me.parent = null, me.setProperty("relationsCombined", !0);
                } else {
                  fe.attributes.getExplicit("rspace") == null && V.default.setAttribute(fe, "rspace", "0pt"), me.attributes.getExplicit("lspace") == null && V.default.setAttribute(me, "lspace", "0pt");
                  break;
                }
              fe.attributes.setInherited("form", fe.getForms()[0]);
            }
          }
        } catch (Se) {
          ae = { error: Se };
        } finally {
          try {
            de && !de.done && (oe = ue.return) && oe.call(ue);
          } finally {
            if (ae) throw ae.error;
          }
        }
        ee.data.removeFromList("mo", le);
      };
      var J = function(ee, ae, oe) {
        var se = ae.attributes, ce = oe.attributes;
        ee.forEach(function(le) {
          var ue = ce.getExplicit(le);
          ue != null && se.set(le, ue);
        });
      }, Z = function(ee, ae) {
        var oe, se, ce = function(ye, ge) {
          var ve = ye.getExplicitNames();
          return ve.filter(function(be) {
            return be !== ge && (be !== "stretchy" || ye.getExplicit("stretchy"));
          });
        }, le = ee.attributes, ue = ae.attributes, de = ce(le, "lspace"), fe = ce(ue, "rspace");
        if (de.length !== fe.length)
          return !1;
        try {
          for (var he = O(de), me = he.next(); !me.done; me = he.next()) {
            var pe = me.value;
            if (le.getExplicit(pe) !== ue.getExplicit(pe))
              return !1;
          }
        } catch (ye) {
          oe = { error: ye };
        } finally {
          try {
            me && !me.done && (se = he.return) && se.call(he);
          } finally {
            if (oe) throw oe.error;
          }
        }
        return !0;
      }, te = function(ee, ae, oe) {
        var se, ce, le = [];
        try {
          for (var ue = O(ee.getList("m" + ae + oe)), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value, he = fe.childNodes;
            if (!(he[fe[ae]] && he[fe[oe]])) {
              var me = fe.parent, pe = he[fe[ae]] ? ee.nodeFactory.create("node", "m" + ae, [he[fe.base], he[fe[ae]]]) : ee.nodeFactory.create("node", "m" + oe, [he[fe.base], he[fe[oe]]]);
              V.default.copyAttributes(fe, pe), me ? me.replaceChild(pe, fe) : ee.root = pe, le.push(fe);
            }
          }
        } catch (ye) {
          se = { error: ye };
        } finally {
          try {
            de && !de.done && (ce = ue.return) && ce.call(ue);
          } finally {
            if (se) throw se.error;
          }
        }
        ee.removeFromList("m" + ae + oe, le);
      };
      G.cleanSubSup = function(ee) {
        var ae = ee.data;
        ae.error || (te(ae, "sub", "sup"), te(ae, "under", "over"));
      };
      var ne = function(ee, ae, oe) {
        var se, ce, le = [];
        try {
          for (var ue = O(ee.getList(ae)), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value;
            if (!fe.attributes.get("displaystyle")) {
              var he = fe.childNodes[fe.base], me = he.coreMO();
              if (he.getProperty("movablelimits") && !me.attributes.getExplicit("movablelimits")) {
                var pe = ee.nodeFactory.create("node", oe, fe.childNodes);
                V.default.copyAttributes(fe, pe), fe.parent ? fe.parent.replaceChild(pe, fe) : ee.root = pe, le.push(fe);
              }
            }
          }
        } catch (ye) {
          se = { error: ye };
        } finally {
          try {
            de && !de.done && (ce = ue.return) && ce.call(ue);
          } finally {
            if (se) throw se.error;
          }
        }
        ee.removeFromList(ae, le);
      };
      G.moveLimits = function(ee) {
        var ae = ee.data;
        ne(ae, "munderover", "msubsup"), ne(ae, "munder", "msub"), ne(ae, "mover", "msup");
      }, G.setInherited = function(ee) {
        ee.data.root.setInheritedAttributes({}, ee.math.display, 0, !1);
      };
    }(q || (q = {})), FilterUtil.default = q, FilterUtil;
  }
  var TexParser = {}, ParseUtil = {}, TexError = {}, hasRequiredTexError;
  function requireTexError() {
    if (hasRequiredTexError) return TexError;
    hasRequiredTexError = 1, Object.defineProperty(TexError, "__esModule", { value: !0 });
    var O = function() {
      function _(H, V) {
        for (var q = [], G = 2; G < arguments.length; G++)
          q[G - 2] = arguments[G];
        this.id = H, this.message = _.processString(V, q);
      }
      return _.processString = function(H, V) {
        for (var q = H.split(_.pattern), G = 1, J = q.length; G < J; G += 2) {
          var Z = q[G].charAt(0);
          if (Z >= "0" && Z <= "9")
            q[G] = V[parseInt(q[G], 10) - 1], typeof q[G] == "number" && (q[G] = q[G].toString());
          else if (Z === "{")
            if (Z = q[G].substr(1), Z >= "0" && Z <= "9")
              q[G] = V[parseInt(q[G].substr(1, q[G].length - 2), 10) - 1], typeof q[G] == "number" && (q[G] = q[G].toString());
            else {
              var te = q[G].match(/^\{([a-z]+):%(\d+)\|(.*)\}$/);
              te && (q[G] = "%" + q[G]);
            }
          q[G] == null && (q[G] = "???");
        }
        return q.join("");
      }, _.pattern = /%(\d+|\{\d+\}|\{[a-z]+:\%\d+(?:\|(?:%\{\d+\}|%.|[^\}])*)+\}|.)/g, _;
    }();
    return TexError.default = O, TexError;
  }
  var Entities = {}, AsyncLoad = {}, hasRequiredAsyncLoad;
  function requireAsyncLoad() {
    if (hasRequiredAsyncLoad) return AsyncLoad;
    hasRequiredAsyncLoad = 1, Object.defineProperty(AsyncLoad, "__esModule", { value: !0 }), AsyncLoad.asyncLoad = void 0;
    var O = requireMathjax();
    function _(H) {
      return O.mathjax.asyncLoad ? new Promise(function(V, q) {
        var G = O.mathjax.asyncLoad(H);
        G instanceof Promise ? G.then(function(J) {
          return V(J);
        }).catch(function(J) {
          return q(J);
        }) : V(G);
      }) : Promise.reject("Can't load '".concat(H, "': No asyncLoad method specified"));
    }
    return AsyncLoad.asyncLoad = _, AsyncLoad;
  }
  var hasRequiredEntities;
  function requireEntities() {
    return hasRequiredEntities || (hasRequiredEntities = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.numeric = O.translate = O.remove = O.add = O.entities = O.options = void 0;
      var _ = requireRetries(), H = requireAsyncLoad();
      O.options = {
        loadMissingEntities: !0
      }, O.entities = {
        ApplyFunction: "⁡",
        Backslash: "∖",
        Because: "∵",
        Breve: "˘",
        Cap: "⋒",
        CenterDot: "·",
        CircleDot: "⊙",
        CircleMinus: "⊖",
        CirclePlus: "⊕",
        CircleTimes: "⊗",
        Congruent: "≡",
        ContourIntegral: "∮",
        Coproduct: "∐",
        Cross: "⨯",
        Cup: "⋓",
        CupCap: "≍",
        Dagger: "‡",
        Del: "∇",
        Delta: "Δ",
        Diamond: "⋄",
        DifferentialD: "ⅆ",
        DotEqual: "≐",
        DoubleDot: "¨",
        DoubleRightTee: "⊨",
        DoubleVerticalBar: "∥",
        DownArrow: "↓",
        DownLeftVector: "↽",
        DownRightVector: "⇁",
        DownTee: "⊤",
        Downarrow: "⇓",
        Element: "∈",
        EqualTilde: "≂",
        Equilibrium: "⇌",
        Exists: "∃",
        ExponentialE: "ⅇ",
        FilledVerySmallSquare: "▪",
        ForAll: "∀",
        Gamma: "Γ",
        Gg: "⋙",
        GreaterEqual: "≥",
        GreaterEqualLess: "⋛",
        GreaterFullEqual: "≧",
        GreaterLess: "≷",
        GreaterSlantEqual: "⩾",
        GreaterTilde: "≳",
        Hacek: "ˇ",
        Hat: "^",
        HumpDownHump: "≎",
        HumpEqual: "≏",
        Im: "ℑ",
        ImaginaryI: "ⅈ",
        Integral: "∫",
        Intersection: "⋂",
        InvisibleComma: "⁣",
        InvisibleTimes: "⁢",
        Lambda: "Λ",
        Larr: "↞",
        LeftAngleBracket: "⟨",
        LeftArrow: "←",
        LeftArrowRightArrow: "⇆",
        LeftCeiling: "⌈",
        LeftDownVector: "⇃",
        LeftFloor: "⌊",
        LeftRightArrow: "↔",
        LeftTee: "⊣",
        LeftTriangle: "⊲",
        LeftTriangleEqual: "⊴",
        LeftUpVector: "↿",
        LeftVector: "↼",
        Leftarrow: "⇐",
        Leftrightarrow: "⇔",
        LessEqualGreater: "⋚",
        LessFullEqual: "≦",
        LessGreater: "≶",
        LessSlantEqual: "⩽",
        LessTilde: "≲",
        Ll: "⋘",
        Lleftarrow: "⇚",
        LongLeftArrow: "⟵",
        LongLeftRightArrow: "⟷",
        LongRightArrow: "⟶",
        Longleftarrow: "⟸",
        Longleftrightarrow: "⟺",
        Longrightarrow: "⟹",
        Lsh: "↰",
        MinusPlus: "∓",
        NestedGreaterGreater: "≫",
        NestedLessLess: "≪",
        NotDoubleVerticalBar: "∦",
        NotElement: "∉",
        NotEqual: "≠",
        NotExists: "∄",
        NotGreater: "≯",
        NotGreaterEqual: "≱",
        NotLeftTriangle: "⋪",
        NotLeftTriangleEqual: "⋬",
        NotLess: "≮",
        NotLessEqual: "≰",
        NotPrecedes: "⊀",
        NotPrecedesSlantEqual: "⋠",
        NotRightTriangle: "⋫",
        NotRightTriangleEqual: "⋭",
        NotSubsetEqual: "⊈",
        NotSucceeds: "⊁",
        NotSucceedsSlantEqual: "⋡",
        NotSupersetEqual: "⊉",
        NotTilde: "≁",
        NotVerticalBar: "∤",
        Omega: "Ω",
        OverBar: "‾",
        OverBrace: "⏞",
        PartialD: "∂",
        Phi: "Φ",
        Pi: "Π",
        PlusMinus: "±",
        Precedes: "≺",
        PrecedesEqual: "⪯",
        PrecedesSlantEqual: "≼",
        PrecedesTilde: "≾",
        Product: "∏",
        Proportional: "∝",
        Psi: "Ψ",
        Rarr: "↠",
        Re: "ℜ",
        ReverseEquilibrium: "⇋",
        RightAngleBracket: "⟩",
        RightArrow: "→",
        RightArrowLeftArrow: "⇄",
        RightCeiling: "⌉",
        RightDownVector: "⇂",
        RightFloor: "⌋",
        RightTee: "⊢",
        RightTeeArrow: "↦",
        RightTriangle: "⊳",
        RightTriangleEqual: "⊵",
        RightUpVector: "↾",
        RightVector: "⇀",
        Rightarrow: "⇒",
        Rrightarrow: "⇛",
        Rsh: "↱",
        Sigma: "Σ",
        SmallCircle: "∘",
        Sqrt: "√",
        Square: "□",
        SquareIntersection: "⊓",
        SquareSubset: "⊏",
        SquareSubsetEqual: "⊑",
        SquareSuperset: "⊐",
        SquareSupersetEqual: "⊒",
        SquareUnion: "⊔",
        Star: "⋆",
        Subset: "⋐",
        SubsetEqual: "⊆",
        Succeeds: "≻",
        SucceedsEqual: "⪰",
        SucceedsSlantEqual: "≽",
        SucceedsTilde: "≿",
        SuchThat: "∋",
        Sum: "∑",
        Superset: "⊃",
        SupersetEqual: "⊇",
        Supset: "⋑",
        Therefore: "∴",
        Theta: "Θ",
        Tilde: "∼",
        TildeEqual: "≃",
        TildeFullEqual: "≅",
        TildeTilde: "≈",
        UnderBar: "_",
        UnderBrace: "⏟",
        Union: "⋃",
        UnionPlus: "⊎",
        UpArrow: "↑",
        UpDownArrow: "↕",
        UpTee: "⊥",
        Uparrow: "⇑",
        Updownarrow: "⇕",
        Upsilon: "Υ",
        Vdash: "⊩",
        Vee: "⋁",
        VerticalBar: "∣",
        VerticalTilde: "≀",
        Vvdash: "⊪",
        Wedge: "⋀",
        Xi: "Ξ",
        amp: "&",
        acute: "´",
        aleph: "ℵ",
        alpha: "α",
        amalg: "⨿",
        and: "∧",
        ang: "∠",
        angmsd: "∡",
        angsph: "∢",
        ape: "≊",
        backprime: "‵",
        backsim: "∽",
        backsimeq: "⋍",
        beta: "β",
        beth: "ℶ",
        between: "≬",
        bigcirc: "◯",
        bigodot: "⨀",
        bigoplus: "⨁",
        bigotimes: "⨂",
        bigsqcup: "⨆",
        bigstar: "★",
        bigtriangledown: "▽",
        bigtriangleup: "△",
        biguplus: "⨄",
        blacklozenge: "⧫",
        blacktriangle: "▴",
        blacktriangledown: "▾",
        blacktriangleleft: "◂",
        bowtie: "⋈",
        boxdl: "┐",
        boxdr: "┌",
        boxminus: "⊟",
        boxplus: "⊞",
        boxtimes: "⊠",
        boxul: "┘",
        boxur: "└",
        bsol: "\\",
        bull: "•",
        cap: "∩",
        check: "✓",
        chi: "χ",
        circ: "ˆ",
        circeq: "≗",
        circlearrowleft: "↺",
        circlearrowright: "↻",
        circledR: "®",
        circledS: "Ⓢ",
        circledast: "⊛",
        circledcirc: "⊚",
        circleddash: "⊝",
        clubs: "♣",
        colon: ":",
        comp: "∁",
        ctdot: "⋯",
        cuepr: "⋞",
        cuesc: "⋟",
        cularr: "↶",
        cup: "∪",
        curarr: "↷",
        curlyvee: "⋎",
        curlywedge: "⋏",
        dagger: "†",
        daleth: "ℸ",
        ddarr: "⇊",
        deg: "°",
        delta: "δ",
        digamma: "ϝ",
        div: "÷",
        divideontimes: "⋇",
        dot: "˙",
        doteqdot: "≑",
        dotplus: "∔",
        dotsquare: "⊡",
        dtdot: "⋱",
        ecir: "≖",
        efDot: "≒",
        egs: "⪖",
        ell: "ℓ",
        els: "⪕",
        empty: "∅",
        epsi: "ε",
        epsiv: "ϵ",
        erDot: "≓",
        eta: "η",
        eth: "ð",
        flat: "♭",
        fork: "⋔",
        frown: "⌢",
        gEl: "⪌",
        gamma: "γ",
        gap: "⪆",
        gimel: "ℷ",
        gnE: "≩",
        gnap: "⪊",
        gne: "⪈",
        gnsim: "⋧",
        gt: ">",
        gtdot: "⋗",
        harrw: "↭",
        hbar: "ℏ",
        hellip: "…",
        hookleftarrow: "↩",
        hookrightarrow: "↪",
        imath: "ı",
        infin: "∞",
        intcal: "⊺",
        iota: "ι",
        jmath: "ȷ",
        kappa: "κ",
        kappav: "ϰ",
        lEg: "⪋",
        lambda: "λ",
        lap: "⪅",
        larrlp: "↫",
        larrtl: "↢",
        lbrace: "{",
        lbrack: "[",
        le: "≤",
        leftleftarrows: "⇇",
        leftthreetimes: "⋋",
        lessdot: "⋖",
        lmoust: "⎰",
        lnE: "≨",
        lnap: "⪉",
        lne: "⪇",
        lnsim: "⋦",
        longmapsto: "⟼",
        looparrowright: "↬",
        lowast: "∗",
        loz: "◊",
        lt: "<",
        ltimes: "⋉",
        ltri: "◃",
        macr: "¯",
        malt: "✠",
        mho: "℧",
        mu: "μ",
        multimap: "⊸",
        nLeftarrow: "⇍",
        nLeftrightarrow: "⇎",
        nRightarrow: "⇏",
        nVDash: "⊯",
        nVdash: "⊮",
        natur: "♮",
        nearr: "↗",
        nharr: "↮",
        nlarr: "↚",
        not: "¬",
        nrarr: "↛",
        nu: "ν",
        nvDash: "⊭",
        nvdash: "⊬",
        nwarr: "↖",
        omega: "ω",
        omicron: "ο",
        or: "∨",
        osol: "⊘",
        period: ".",
        phi: "φ",
        phiv: "ϕ",
        pi: "π",
        piv: "ϖ",
        prap: "⪷",
        precnapprox: "⪹",
        precneqq: "⪵",
        precnsim: "⋨",
        prime: "′",
        psi: "ψ",
        quot: '"',
        rarrtl: "↣",
        rbrace: "}",
        rbrack: "]",
        rho: "ρ",
        rhov: "ϱ",
        rightrightarrows: "⇉",
        rightthreetimes: "⋌",
        ring: "˚",
        rmoust: "⎱",
        rtimes: "⋊",
        rtri: "▹",
        scap: "⪸",
        scnE: "⪶",
        scnap: "⪺",
        scnsim: "⋩",
        sdot: "⋅",
        searr: "↘",
        sect: "§",
        sharp: "♯",
        sigma: "σ",
        sigmav: "ς",
        simne: "≆",
        smile: "⌣",
        spades: "♠",
        sub: "⊂",
        subE: "⫅",
        subnE: "⫋",
        subne: "⊊",
        supE: "⫆",
        supnE: "⫌",
        supne: "⊋",
        swarr: "↙",
        tau: "τ",
        theta: "θ",
        thetav: "ϑ",
        tilde: "˜",
        times: "×",
        triangle: "▵",
        triangleq: "≜",
        upsi: "υ",
        upuparrows: "⇈",
        veebar: "⊻",
        vellip: "⋮",
        weierp: "℘",
        xi: "ξ",
        yen: "¥",
        zeta: "ζ",
        zigrarr: "⇝",
        nbsp: " ",
        rsquo: "’",
        lsquo: "‘"
      };
      var V = {};
      function q(ne, ee) {
        Object.assign(O.entities, ne), V[ee] = !0;
      }
      O.add = q;
      function G(ne) {
        delete O.entities[ne];
      }
      O.remove = G;
      function J(ne) {
        return ne.replace(/&([a-z][a-z0-9]*|#(?:[0-9]+|x[0-9a-f]+));/ig, Z);
      }
      O.translate = J;
      function Z(ne, ee) {
        if (ee.charAt(0) === "#")
          return te(ee.slice(1));
        if (O.entities[ee])
          return O.entities[ee];
        if (O.options.loadMissingEntities) {
          var ae = ee.match(/^[a-zA-Z](fr|scr|opf)$/) ? RegExp.$1 : ee.charAt(0).toLowerCase();
          V[ae] || (V[ae] = !0, (0, _.retryAfter)((0, H.asyncLoad)("./util/entities/" + ae + ".js")));
        }
        return ne;
      }
      function te(ne) {
        var ee = ne.charAt(0) === "x" ? parseInt(ne.slice(1), 16) : parseInt(ne);
        return String.fromCodePoint(ee);
      }
      O.numeric = te;
    }(Entities)), Entities;
  }
  var hasRequiredParseUtil;
  function requireParseUtil() {
    if (hasRequiredParseUtil) return ParseUtil;
    hasRequiredParseUtil = 1;
    var O = ParseUtil.__read || function(ne, ee) {
      var ae = typeof Symbol == "function" && ne[Symbol.iterator];
      if (!ae) return ne;
      var oe = ae.call(ne), se, ce = [], le;
      try {
        for (; (ee === void 0 || ee-- > 0) && !(se = oe.next()).done; ) ce.push(se.value);
      } catch (ue) {
        le = { error: ue };
      } finally {
        try {
          se && !se.done && (ae = oe.return) && ae.call(oe);
        } finally {
          if (le) throw le.error;
        }
      }
      return ce;
    }, _ = ParseUtil.__values || function(ne) {
      var ee = typeof Symbol == "function" && Symbol.iterator, ae = ee && ne[ee], oe = 0;
      if (ae) return ae.call(ne);
      if (ne && typeof ne.length == "number") return {
        next: function() {
          return ne && oe >= ne.length && (ne = void 0), { value: ne && ne[oe++], done: !ne };
        }
      };
      throw new TypeError(ee ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = ParseUtil.__importDefault || function(ne) {
      return ne && ne.__esModule ? ne : { default: ne };
    };
    Object.defineProperty(ParseUtil, "__esModule", { value: !0 });
    var V = requireMmlNode(), q = H(requireNodeUtil()), G = H(requireTexParser()), J = H(requireTexError()), Z = requireEntities(), te;
    return function(ne) {
      var ee = 7.2, ae = 72, oe = {
        em: function(Ge) {
          return Ge;
        },
        ex: function(Ge) {
          return Ge * 0.43;
        },
        pt: function(Ge) {
          return Ge / 10;
        },
        pc: function(Ge) {
          return Ge * 1.2;
        },
        px: function(Ge) {
          return Ge * ee / ae;
        },
        in: function(Ge) {
          return Ge * ee;
        },
        cm: function(Ge) {
          return Ge * ee / 2.54;
        },
        mm: function(Ge) {
          return Ge * ee / 25.4;
        },
        mu: function(Ge) {
          return Ge / 18;
        }
      }, se = "([-+]?([.,]\\d+|\\d+([.,]\\d*)?))", ce = "(pt|em|ex|mu|px|mm|cm|in|pc)", le = RegExp("^\\s*" + se + "\\s*" + ce + "\\s*$"), ue = RegExp("^\\s*" + se + "\\s*" + ce + " ?");
      function de(Ge, ze) {
        ze === void 0 && (ze = !1);
        var We = Ge.match(ze ? ue : le);
        return We ? fe([We[1].replace(/,/, "."), We[4], We[0].length]) : [null, null, 0];
      }
      ne.matchDimen = de;
      function fe(Ge) {
        var ze = O(Ge, 3), We = ze[0], at = ze[1], ct = ze[2];
        if (at !== "mu")
          return [We, at, ct];
        var et = me(oe[at](parseFloat(We || "1")));
        return [et.slice(0, -2), "em", ct];
      }
      function he(Ge) {
        var ze = O(de(Ge), 2), We = ze[0], at = ze[1], ct = parseFloat(We || "1"), et = oe[at];
        return et ? et(ct) : 0;
      }
      ne.dimen2em = he;
      function me(Ge) {
        return Math.abs(Ge) < 6e-4 ? "0em" : Ge.toFixed(3).replace(/\.?0+$/, "") + "em";
      }
      ne.Em = me;
      function pe() {
        for (var Ge = [], ze = 0; ze < arguments.length; ze++)
          Ge[ze] = arguments[ze];
        return Ge.map(function(We) {
          return me(We);
        }).join(" ");
      }
      ne.cols = pe;
      function ye(Ge, ze, We, at, ct, et) {
        ct === void 0 && (ct = ""), et === void 0 && (et = "");
        var dt = Ge.nodeFactory, Je = dt.create("node", "mrow", [], { open: ze, close: at, texClass: V.TEXCLASS.INNER }), it;
        if (ct)
          it = new G.default("\\" + ct + "l" + ze, Ge.parser.stack.env, Ge).mml();
        else {
          var st = dt.create("text", ze);
          it = dt.create("node", "mo", [], { fence: !0, stretchy: !0, symmetric: !0, texClass: V.TEXCLASS.OPEN }, st);
        }
        if (q.default.appendChildren(Je, [it, We]), ct)
          it = new G.default("\\" + ct + "r" + at, Ge.parser.stack.env, Ge).mml();
        else {
          var gt = dt.create("text", at);
          it = dt.create("node", "mo", [], { fence: !0, stretchy: !0, symmetric: !0, texClass: V.TEXCLASS.CLOSE }, gt);
        }
        return et && it.attributes.set("mathcolor", et), q.default.appendChildren(Je, [it]), Je;
      }
      ne.fenced = ye;
      function ge(Ge, ze, We, at) {
        var ct = Ge.nodeFactory.create("node", "mrow", [], { open: ze, close: at, texClass: V.TEXCLASS.ORD });
        return ze && q.default.appendChildren(ct, [ve(Ge, ze, "l")]), q.default.isType(We, "mrow") ? q.default.appendChildren(ct, q.default.getChildren(We)) : q.default.appendChildren(ct, [We]), at && q.default.appendChildren(ct, [ve(Ge, at, "r")]), ct;
      }
      ne.fixedFence = ge;
      function ve(Ge, ze, We) {
        (ze === "{" || ze === "}") && (ze = "\\" + ze);
        var at = "{\\bigg" + We + " " + ze + "}", ct = "{\\big" + We + " " + ze + "}";
        return new G.default("\\mathchoice" + at + ct + ct + ct, {}, Ge).mml();
      }
      ne.mathPalette = ve;
      function be(Ge, ze) {
        for (var We = 0, at = ze.length; We < at; We++) {
          var ct = ze[We];
          if (ct && !q.default.isType(ct, "mspace") && (!q.default.isType(ct, "TeXAtom") || q.default.getChildren(ct)[0] && q.default.getChildren(q.default.getChildren(ct)[0]).length)) {
            if (q.default.isEmbellished(ct) || q.default.isType(ct, "TeXAtom") && q.default.getTexClass(ct) === V.TEXCLASS.REL) {
              var et = Ge.nodeFactory.create("node", "mi");
              ze.unshift(et);
            }
            break;
          }
        }
      }
      ne.fixInitialMO = be;
      function Ce(Ge, ze, We, at) {
        if (Ge.configuration.options.internalMath)
          return Ge.configuration.options.internalMath(Ge, ze, We, at);
        var ct = at || Ge.stack.env.font, et = ct ? { mathvariant: ct } : {}, dt = [], Je = 0, it = 0, st, gt, bt = "", yt = 0;
        if (ze.match(/\\?[${}\\]|\\\(|\\(eq)?ref\s*\{/)) {
          for (; Je < ze.length; )
            if (st = ze.charAt(Je++), st === "$")
              bt === "$" && yt === 0 ? (gt = Ge.create("node", "TeXAtom", [new G.default(ze.slice(it, Je - 1), {}, Ge.configuration).mml()]), dt.push(gt), bt = "", it = Je) : bt === "" && (it < Je - 1 && dt.push(Se(Ge, ze.slice(it, Je - 1), et)), bt = "$", it = Je);
            else if (st === "{" && bt !== "")
              yt++;
            else if (st === "}")
              if (bt === "}" && yt === 0) {
                var St = new G.default(ze.slice(it, Je), {}, Ge.configuration).mml();
                gt = Ge.create("node", "TeXAtom", [St], et), dt.push(gt), bt = "", it = Je;
              } else bt !== "" && yt && yt--;
            else if (st === "\\")
              if (bt === "" && ze.substr(Je).match(/^(eq)?ref\s*\{/)) {
                var Lt = RegExp["$&"].length;
                it < Je - 1 && dt.push(Se(Ge, ze.slice(it, Je - 1), et)), bt = "}", it = Je - 1, Je += Lt;
              } else
                st = ze.charAt(Je++), st === "(" && bt === "" ? (it < Je - 2 && dt.push(Se(Ge, ze.slice(it, Je - 2), et)), bt = ")", it = Je) : st === ")" && bt === ")" && yt === 0 ? (gt = Ge.create("node", "TeXAtom", [new G.default(ze.slice(it, Je - 2), {}, Ge.configuration).mml()]), dt.push(gt), bt = "", it = Je) : st.match(/[${}\\]/) && bt === "" && (Je--, ze = ze.substr(0, Je - 1) + ze.substr(Je));
          if (bt !== "")
            throw new J.default("MathNotTerminated", "Math not terminated in text box");
        }
        return it < ze.length && dt.push(Se(Ge, ze.slice(it), et)), We != null ? dt = [Ge.create("node", "mstyle", dt, { displaystyle: !1, scriptlevel: We })] : dt.length > 1 && (dt = [Ge.create("node", "mrow", dt)]), dt;
      }
      ne.internalMath = Ce;
      function Se(Ge, ze, We) {
        ze = ze.replace(/^\s+/, Z.entities.nbsp).replace(/\s+$/, Z.entities.nbsp);
        var at = Ge.create("text", ze);
        return Ge.create("node", "mtext", [], We, at);
      }
      ne.internalText = Se;
      function we(Ge, ze, We, at, ct) {
        if (ne.checkMovableLimits(ze), q.default.isType(ze, "munderover") && q.default.isEmbellished(ze)) {
          q.default.setProperties(q.default.getCoreMO(ze), { lspace: 0, rspace: 0 });
          var et = Ge.create("node", "mo", [], { rspace: 0 });
          ze = Ge.create("node", "mrow", [et, ze]);
        }
        var dt = Ge.create("node", "munderover", [ze]);
        q.default.setChild(dt, at === "over" ? dt.over : dt.under, We);
        var Je = dt;
        return ct && (Je = Ge.create("node", "TeXAtom", [dt], { texClass: V.TEXCLASS.OP, movesupsub: !0 })), q.default.setProperty(Je, "subsupOK", !0), Je;
      }
      ne.underOver = we;
      function Ee(Ge) {
        var ze = q.default.isType(Ge, "mo") ? q.default.getForm(Ge) : null;
        (q.default.getProperty(Ge, "movablelimits") || ze && ze[3] && ze[3].movablelimits) && q.default.setProperties(Ge, { movablelimits: !1 });
      }
      ne.checkMovableLimits = Ee;
      function xe(Ge) {
        if (typeof Ge != "string")
          return Ge;
        var ze = Ge.trim();
        return ze.match(/\\$/) && Ge.match(/ $/) && (ze += " "), ze;
      }
      ne.trimSpaces = xe;
      function Le(Ge, ze) {
        return ze = ne.trimSpaces(ze || ""), ze === "t" ? Ge.arraydef.align = "baseline 1" : ze === "b" ? Ge.arraydef.align = "baseline -1" : ze === "c" ? Ge.arraydef.align = "axis" : ze && (Ge.arraydef.align = ze), Ge;
      }
      ne.setArrayAlign = Le;
      function Qe(Ge, ze, We) {
        for (var at = "", ct = "", et = 0; et < We.length; ) {
          var dt = We.charAt(et++);
          if (dt === "\\")
            at += dt + We.charAt(et++);
          else if (dt === "#")
            if (dt = We.charAt(et++), dt === "#")
              at += dt;
            else {
              if (!dt.match(/[1-9]/) || parseInt(dt, 10) > ze.length)
                throw new J.default("IllegalMacroParam", "Illegal macro parameter reference");
              ct = ke(Ge, ke(Ge, ct, at), ze[parseInt(dt, 10) - 1]), at = "";
            }
          else
            at += dt;
        }
        return ke(Ge, ct, at);
      }
      ne.substituteArgs = Qe;
      function ke(Ge, ze, We) {
        if (We.match(/^[a-z]/i) && ze.match(/(^|[^\\])(\\\\)*\\[a-z]+$/i) && (ze += " "), ze.length + We.length > Ge.configuration.options.maxBuffer)
          throw new J.default("MaxBufferSize", "MathJax internal buffer size exceeded; is there a recursive macro call?");
        return ze + We;
      }
      ne.addArgs = ke;
      function Me(Ge, ze) {
        if (ze === void 0 && (ze = !0), !(++Ge.macroCount <= Ge.configuration.options.maxMacros))
          throw ze ? new J.default("MaxMacroSub1", "MathJax maximum macro substitution count exceeded; is here a recursive macro call?") : new J.default("MaxMacroSub2", "MathJax maximum substitution count exceeded; is there a recursive latex environment?");
      }
      ne.checkMaxMacros = Me;
      function Re(Ge) {
        if (Ge.stack.global.eqnenv)
          throw new J.default("ErroneousNestingEq", "Erroneous nesting of equation structures");
        Ge.stack.global.eqnenv = !0;
      }
      ne.checkEqnEnv = Re;
      function De(Ge, ze) {
        var We = Ge.copy(), at = ze.configuration;
        return We.walkTree(function(ct) {
          var et, dt;
          at.addNode(ct.kind, ct);
          var Je = (ct.getProperty("in-lists") || "").split(/,/);
          try {
            for (var it = _(Je), st = it.next(); !st.done; st = it.next()) {
              var gt = st.value;
              gt && at.addNode(gt, ct);
            }
          } catch (bt) {
            et = { error: bt };
          } finally {
            try {
              st && !st.done && (dt = it.return) && dt.call(it);
            } finally {
              if (et) throw et.error;
            }
          }
        }), We;
      }
      ne.copyNode = De;
      function Pe(Ge, ze, We) {
        return We;
      }
      ne.MmlFilterAttribute = Pe;
      function $e(Ge) {
        var ze = Ge.stack.env.font;
        return ze ? { mathvariant: ze } : {};
      }
      ne.getFontDef = $e;
      function qe(Ge, ze, We) {
        var at, ct;
        ze === void 0 && (ze = null), We === void 0 && (We = !1);
        var et = Xe(Ge);
        if (ze)
          try {
            for (var dt = _(Object.keys(et)), Je = dt.next(); !Je.done; Je = dt.next()) {
              var it = Je.value;
              if (!ze.hasOwnProperty(it)) {
                if (We)
                  throw new J.default("InvalidOption", "Invalid option: %1", it);
                delete et[it];
              }
            }
          } catch (st) {
            at = { error: st };
          } finally {
            try {
              Je && !Je.done && (ct = dt.return) && ct.call(dt);
            } finally {
              if (at) throw at.error;
            }
          }
        return et;
      }
      ne.keyvalOptions = qe;
      function Xe(Ge) {
        for (var ze, We, at = {}, ct = Ge, et, dt, Je; ct; )
          ze = O(Ze(ct, ["=", ","]), 3), dt = ze[0], et = ze[1], ct = ze[2], et === "=" ? (We = O(Ze(ct, [","]), 3), Je = We[0], et = We[1], ct = We[2], Je = Je === "false" || Je === "true" ? JSON.parse(Je) : Je, at[dt] = Je) : dt && (at[dt] = !0);
        return at;
      }
      function Ve(Ge, ze) {
        for (; ze > 0; )
          Ge = Ge.trim().slice(1, -1), ze--;
        return Ge.trim();
      }
      function Ze(Ge, ze) {
        for (var We = Ge.length, at = 0, ct = "", et = 0, dt = 0, Je = !0, it = !1; et < We; ) {
          var st = Ge[et++];
          switch (st) {
            case " ":
              break;
            case "{":
              Je ? dt++ : (it = !1, dt > at && (dt = at)), at++;
              break;
            case "}":
              at && at--, (Je || it) && (dt--, it = !0), Je = !1;
              break;
            default:
              if (!at && ze.indexOf(st) !== -1)
                return [it ? "true" : Ve(ct, dt), st, Ge.slice(et)];
              Je = !1, it = !1;
          }
          ct += st;
        }
        if (at)
          throw new J.default("ExtraOpenMissingClose", "Extra open brace or missing close brace");
        return [it ? "true" : Ve(ct, dt), "", Ge.slice(et)];
      }
    }(te || (te = {})), ParseUtil.default = te, ParseUtil;
  }
  var Stack$3 = {}, hasRequiredStack;
  function requireStack() {
    if (hasRequiredStack) return Stack$3;
    hasRequiredStack = 1;
    var O = Stack$3.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = Stack$3.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    }, H = Stack$3.__spreadArray || function(J, Z, te) {
      if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
        (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
      return J.concat(ae || Array.prototype.slice.call(Z));
    }, V = Stack$3.__importDefault || function(J) {
      return J && J.__esModule ? J : { default: J };
    };
    Object.defineProperty(Stack$3, "__esModule", { value: !0 });
    var q = V(requireNodeUtil()), G = function() {
      function J(Z, te, ne) {
        this._factory = Z, this._env = te, this.global = {}, this.stack = [], this.global = { isInner: ne }, this.stack = [this._factory.create("start", this.global)], te && (this.stack[0].env = te), this.env = this.stack[0].env;
      }
      return Object.defineProperty(J.prototype, "env", {
        get: function() {
          return this._env;
        },
        set: function(Z) {
          this._env = Z;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.Push = function() {
        for (var Z, te, ne = [], ee = 0; ee < arguments.length; ee++)
          ne[ee] = arguments[ee];
        try {
          for (var ae = O(ne), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            if (se) {
              var ce = q.default.isNode(se) ? this._factory.create("mml", se) : se;
              ce.global = this.global;
              var le = _(this.stack.length ? this.Top().checkItem(ce) : [null, !0], 2), ue = le[0], de = le[1];
              if (de) {
                if (ue) {
                  this.Pop(), this.Push.apply(this, H([], _(ue), !1));
                  continue;
                }
                this.stack.push(ce), ce.env ? (ce.copyEnv && Object.assign(ce.env, this.env), this.env = ce.env) : ce.env = this.env;
              }
            }
          }
        } catch (fe) {
          Z = { error: fe };
        } finally {
          try {
            oe && !oe.done && (te = ae.return) && te.call(ae);
          } finally {
            if (Z) throw Z.error;
          }
        }
      }, J.prototype.Pop = function() {
        var Z = this.stack.pop();
        return Z.isOpen || delete Z.env, this.env = this.stack.length ? this.Top().env : {}, Z;
      }, J.prototype.Top = function(Z) {
        return Z === void 0 && (Z = 1), this.stack.length < Z ? null : this.stack[this.stack.length - Z];
      }, J.prototype.Prev = function(Z) {
        var te = this.Top();
        return Z ? te.First : te.Pop();
      }, J.prototype.toString = function() {
        return `stack[
  ` + this.stack.join(`
  `) + `
]`;
      }, J;
    }();
    return Stack$3.default = G, Stack$3;
  }
  var hasRequiredTexParser;
  function requireTexParser() {
    if (hasRequiredTexParser) return TexParser;
    hasRequiredTexParser = 1;
    var O = TexParser.__values || function(ne) {
      var ee = typeof Symbol == "function" && Symbol.iterator, ae = ee && ne[ee], oe = 0;
      if (ae) return ae.call(ne);
      if (ne && typeof ne.length == "number") return {
        next: function() {
          return ne && oe >= ne.length && (ne = void 0), { value: ne && ne[oe++], done: !ne };
        }
      };
      throw new TypeError(ee ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = TexParser.__read || function(ne, ee) {
      var ae = typeof Symbol == "function" && ne[Symbol.iterator];
      if (!ae) return ne;
      var oe = ae.call(ne), se, ce = [], le;
      try {
        for (; (ee === void 0 || ee-- > 0) && !(se = oe.next()).done; ) ce.push(se.value);
      } catch (ue) {
        le = { error: ue };
      } finally {
        try {
          se && !se.done && (ae = oe.return) && ae.call(oe);
        } finally {
          if (le) throw le.error;
        }
      }
      return ce;
    }, H = TexParser.__spreadArray || function(ne, ee, ae) {
      if (ae || arguments.length === 2) for (var oe = 0, se = ee.length, ce; oe < se; oe++)
        (ce || !(oe in ee)) && (ce || (ce = Array.prototype.slice.call(ee, 0, oe)), ce[oe] = ee[oe]);
      return ne.concat(ce || Array.prototype.slice.call(ee));
    }, V = TexParser.__importDefault || function(ne) {
      return ne && ne.__esModule ? ne : { default: ne };
    };
    Object.defineProperty(TexParser, "__esModule", { value: !0 });
    var q = V(requireParseUtil()), G = V(requireStack()), J = V(requireTexError()), Z = requireMmlNode(), te = function() {
      function ne(ee, ae, oe) {
        var se, ce;
        this._string = ee, this.configuration = oe, this.macroCount = 0, this.i = 0, this.currentCS = "";
        var le = ae.hasOwnProperty("isInner"), ue = ae.isInner;
        delete ae.isInner;
        var de;
        if (ae) {
          de = {};
          try {
            for (var fe = O(Object.keys(ae)), he = fe.next(); !he.done; he = fe.next()) {
              var me = he.value;
              de[me] = ae[me];
            }
          } catch (pe) {
            se = { error: pe };
          } finally {
            try {
              he && !he.done && (ce = fe.return) && ce.call(fe);
            } finally {
              if (se) throw se.error;
            }
          }
        }
        this.configuration.pushParser(this), this.stack = new G.default(this.itemFactory, de, le ? ue : !0), this.Parse(), this.Push(this.itemFactory.create("stop"));
      }
      return Object.defineProperty(ne.prototype, "options", {
        get: function() {
          return this.configuration.options;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "itemFactory", {
        get: function() {
          return this.configuration.itemFactory;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "tags", {
        get: function() {
          return this.configuration.tags;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "string", {
        get: function() {
          return this._string;
        },
        set: function(ee) {
          this._string = ee;
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.parse = function(ee, ae) {
        return this.configuration.handlers.get(ee).parse(ae);
      }, ne.prototype.lookup = function(ee, ae) {
        return this.configuration.handlers.get(ee).lookup(ae);
      }, ne.prototype.contains = function(ee, ae) {
        return this.configuration.handlers.get(ee).contains(ae);
      }, ne.prototype.toString = function() {
        var ee, ae, oe = "";
        try {
          for (var se = O(Array.from(this.configuration.handlers.keys())), ce = se.next(); !ce.done; ce = se.next()) {
            var le = ce.value;
            oe += le + ": " + this.configuration.handlers.get(le) + `
`;
          }
        } catch (ue) {
          ee = { error: ue };
        } finally {
          try {
            ce && !ce.done && (ae = se.return) && ae.call(se);
          } finally {
            if (ee) throw ee.error;
          }
        }
        return oe;
      }, ne.prototype.Parse = function() {
        for (var ee; this.i < this.string.length; )
          ee = this.getCodePoint(), this.i += ee.length, this.parse("character", [this, ee]);
      }, ne.prototype.Push = function(ee) {
        ee instanceof Z.AbstractMmlNode && ee.isInferred ? this.PushAll(ee.childNodes) : this.stack.Push(ee);
      }, ne.prototype.PushAll = function(ee) {
        var ae, oe;
        try {
          for (var se = O(ee), ce = se.next(); !ce.done; ce = se.next()) {
            var le = ce.value;
            this.stack.Push(le);
          }
        } catch (ue) {
          ae = { error: ue };
        } finally {
          try {
            ce && !ce.done && (oe = se.return) && oe.call(se);
          } finally {
            if (ae) throw ae.error;
          }
        }
      }, ne.prototype.mml = function() {
        if (!this.stack.Top().isKind("mml"))
          return null;
        var ee = this.stack.Top().First;
        return this.configuration.popParser(), ee;
      }, ne.prototype.convertDelimiter = function(ee) {
        var ae = this.lookup("delimiter", ee);
        return ae ? ae.char : null;
      }, ne.prototype.getCodePoint = function() {
        var ee = this.string.codePointAt(this.i);
        return ee === void 0 ? "" : String.fromCodePoint(ee);
      }, ne.prototype.nextIsSpace = function() {
        return !!this.string.charAt(this.i).match(/\s/);
      }, ne.prototype.GetNext = function() {
        for (; this.nextIsSpace(); )
          this.i++;
        return this.getCodePoint();
      }, ne.prototype.GetCS = function() {
        var ee = this.string.slice(this.i).match(/^(([a-z]+) ?|[\uD800-\uDBFF].|.)/i);
        return ee ? (this.i += ee[0].length, ee[2] || ee[1]) : (this.i++, " ");
      }, ne.prototype.GetArgument = function(ee, ae) {
        switch (this.GetNext()) {
          case "":
            if (!ae)
              throw new J.default("MissingArgFor", "Missing argument for %1", this.currentCS);
            return null;
          case "}":
            if (!ae)
              throw new J.default("ExtraCloseMissingOpen", "Extra close brace or missing open brace");
            return null;
          case "\\":
            return this.i++, "\\" + this.GetCS();
          case "{":
            for (var oe = ++this.i, se = 1; this.i < this.string.length; )
              switch (this.string.charAt(this.i++)) {
                case "\\":
                  this.i++;
                  break;
                case "{":
                  se++;
                  break;
                case "}":
                  if (--se === 0)
                    return this.string.slice(oe, this.i - 1);
                  break;
              }
            throw new J.default("MissingCloseBrace", "Missing close brace");
        }
        var ce = this.getCodePoint();
        return this.i += ce.length, ce;
      }, ne.prototype.GetBrackets = function(ee, ae) {
        if (this.GetNext() !== "[")
          return ae;
        for (var oe = ++this.i, se = 0; this.i < this.string.length; )
          switch (this.string.charAt(this.i++)) {
            case "{":
              se++;
              break;
            case "\\":
              this.i++;
              break;
            case "}":
              if (se-- <= 0)
                throw new J.default("ExtraCloseLooking", "Extra close brace while looking for %1", "']'");
              break;
            case "]":
              if (se === 0)
                return this.string.slice(oe, this.i - 1);
              break;
          }
        throw new J.default("MissingCloseBracket", "Could not find closing ']' for argument to %1", this.currentCS);
      }, ne.prototype.GetDelimiter = function(ee, ae) {
        var oe = this.GetNext();
        if (this.i += oe.length, this.i <= this.string.length && (oe === "\\" ? oe += this.GetCS() : oe === "{" && ae && (this.i--, oe = this.GetArgument(ee).trim()), this.contains("delimiter", oe)))
          return this.convertDelimiter(oe);
        throw new J.default("MissingOrUnrecognizedDelim", "Missing or unrecognized delimiter for %1", this.currentCS);
      }, ne.prototype.GetDimen = function(ee) {
        if (this.GetNext() === "{") {
          var ae = this.GetArgument(ee), oe = _(q.default.matchDimen(ae), 2), se = oe[0], ce = oe[1];
          if (se)
            return se + ce;
        } else {
          var ae = this.string.slice(this.i), le = _(q.default.matchDimen(ae, !0), 3), se = le[0], ce = le[1], ue = le[2];
          if (se)
            return this.i += ue, se + ce;
        }
        throw new J.default("MissingDimOrUnits", "Missing dimension or its units for %1", this.currentCS);
      }, ne.prototype.GetUpTo = function(ee, ae) {
        for (; this.nextIsSpace(); )
          this.i++;
        for (var oe = this.i, se = 0; this.i < this.string.length; ) {
          var ce = this.i, le = this.GetNext();
          switch (this.i += le.length, le) {
            case "\\":
              le += this.GetCS();
              break;
            case "{":
              se++;
              break;
            case "}":
              if (se === 0)
                throw new J.default("ExtraCloseLooking", "Extra close brace while looking for %1", ae);
              se--;
              break;
          }
          if (se === 0 && le === ae)
            return this.string.slice(oe, ce);
        }
        throw new J.default("TokenNotFoundForCommand", "Could not find %1 for %2", ae, this.currentCS);
      }, ne.prototype.ParseArg = function(ee) {
        return new ne(this.GetArgument(ee), this.stack.env, this.configuration).mml();
      }, ne.prototype.ParseUpTo = function(ee, ae) {
        return new ne(this.GetUpTo(ee, ae), this.stack.env, this.configuration).mml();
      }, ne.prototype.GetDelimiterArg = function(ee) {
        var ae = q.default.trimSpaces(this.GetArgument(ee));
        if (ae === "")
          return null;
        if (this.contains("delimiter", ae))
          return ae;
        throw new J.default("MissingOrUnrecognizedDelim", "Missing or unrecognized delimiter for %1", this.currentCS);
      }, ne.prototype.GetStar = function() {
        var ee = this.GetNext() === "*";
        return ee && this.i++, ee;
      }, ne.prototype.create = function(ee) {
        for (var ae, oe = [], se = 1; se < arguments.length; se++)
          oe[se - 1] = arguments[se];
        return (ae = this.configuration.nodeFactory).create.apply(ae, H([ee], _(oe), !1));
      }, ne;
    }();
    return TexParser.default = te, TexParser;
  }
  var ParseOptions = {}, StackItemFactory = {}, StackItem = {}, hasRequiredStackItem;
  function requireStackItem() {
    if (hasRequiredStackItem) return StackItem;
    hasRequiredStackItem = 1;
    var O = StackItem.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = StackItem.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    }, H = StackItem.__spreadArray || function(te, ne, ee) {
      if (ee || arguments.length === 2) for (var ae = 0, oe = ne.length, se; ae < oe; ae++)
        (se || !(ae in ne)) && (se || (se = Array.prototype.slice.call(ne, 0, ae)), se[ae] = ne[ae]);
      return te.concat(se || Array.prototype.slice.call(ne));
    }, V = StackItem.__values || function(te) {
      var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
      if (ee) return ee.call(te);
      if (te && typeof te.length == "number") return {
        next: function() {
          return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
        }
      };
      throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, q = StackItem.__importDefault || function(te) {
      return te && te.__esModule ? te : { default: te };
    };
    Object.defineProperty(StackItem, "__esModule", { value: !0 }), StackItem.BaseItem = StackItem.MmlStack = void 0;
    var G = q(requireTexError()), J = function() {
      function te(ne) {
        this._nodes = ne;
      }
      return Object.defineProperty(te.prototype, "nodes", {
        get: function() {
          return this._nodes;
        },
        enumerable: !1,
        configurable: !0
      }), te.prototype.Push = function() {
        for (var ne, ee = [], ae = 0; ae < arguments.length; ae++)
          ee[ae] = arguments[ae];
        (ne = this._nodes).push.apply(ne, H([], _(ee), !1));
      }, te.prototype.Pop = function() {
        return this._nodes.pop();
      }, Object.defineProperty(te.prototype, "First", {
        get: function() {
          return this._nodes[this.Size() - 1];
        },
        set: function(ne) {
          this._nodes[this.Size() - 1] = ne;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(te.prototype, "Last", {
        get: function() {
          return this._nodes[0];
        },
        set: function(ne) {
          this._nodes[0] = ne;
        },
        enumerable: !1,
        configurable: !0
      }), te.prototype.Peek = function(ne) {
        return ne == null && (ne = 1), this._nodes.slice(this.Size() - ne);
      }, te.prototype.Size = function() {
        return this._nodes.length;
      }, te.prototype.Clear = function() {
        this._nodes = [];
      }, te.prototype.toMml = function(ne, ee) {
        return ne === void 0 && (ne = !0), this._nodes.length === 1 && !ee ? this.First : this.create("node", ne ? "inferredMrow" : "mrow", this._nodes, {});
      }, te.prototype.create = function(ne) {
        for (var ee, ae = [], oe = 1; oe < arguments.length; oe++)
          ae[oe - 1] = arguments[oe];
        return (ee = this.factory.configuration.nodeFactory).create.apply(ee, H([ne], _(ae), !1));
      }, te;
    }();
    StackItem.MmlStack = J;
    var Z = function(te) {
      O(ne, te);
      function ne(ee) {
        for (var ae = [], oe = 1; oe < arguments.length; oe++)
          ae[oe - 1] = arguments[oe];
        var se = te.call(this, ae) || this;
        return se.factory = ee, se.global = {}, se._properties = {}, se.isOpen && (se._env = {}), se;
      }
      return Object.defineProperty(ne.prototype, "kind", {
        get: function() {
          return "base";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "env", {
        get: function() {
          return this._env;
        },
        set: function(ee) {
          this._env = ee;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "copyEnv", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.getProperty = function(ee) {
        return this._properties[ee];
      }, ne.prototype.setProperty = function(ee, ae) {
        return this._properties[ee] = ae, this;
      }, Object.defineProperty(ne.prototype, "isOpen", {
        get: function() {
          return !1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "isClose", {
        get: function() {
          return !1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "isFinal", {
        get: function() {
          return !1;
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.isKind = function(ee) {
        return ee === this.kind;
      }, ne.prototype.checkItem = function(ee) {
        if (ee.isKind("over") && this.isOpen && (ee.setProperty("num", this.toMml(!1)), this.Clear()), ee.isKind("cell") && this.isOpen) {
          if (ee.getProperty("linebreak"))
            return ne.fail;
          throw new G.default("Misplaced", "Misplaced %1", ee.getName());
        }
        if (ee.isClose && this.getErrors(ee.kind)) {
          var ae = _(this.getErrors(ee.kind), 2), oe = ae[0], se = ae[1];
          throw new G.default(oe, se, ee.getName());
        }
        return ee.isFinal ? (this.Push(ee.First), ne.fail) : ne.success;
      }, ne.prototype.clearEnv = function() {
        var ee, ae;
        try {
          for (var oe = V(Object.keys(this.env)), se = oe.next(); !se.done; se = oe.next()) {
            var ce = se.value;
            delete this.env[ce];
          }
        } catch (le) {
          ee = { error: le };
        } finally {
          try {
            se && !se.done && (ae = oe.return) && ae.call(oe);
          } finally {
            if (ee) throw ee.error;
          }
        }
      }, ne.prototype.setProperties = function(ee) {
        return Object.assign(this._properties, ee), this;
      }, ne.prototype.getName = function() {
        return this.getProperty("name");
      }, ne.prototype.toString = function() {
        return this.kind + "[" + this.nodes.join("; ") + "]";
      }, ne.prototype.getErrors = function(ee) {
        var ae = this.constructor;
        return (ae.errors || {})[ee] || ne.errors[ee];
      }, ne.fail = [null, !1], ne.success = [null, !0], ne.errors = {
        end: ["MissingBeginExtraEnd", "Missing \\begin{%1} or extra \\end{%1}"],
        close: ["ExtraCloseMissingOpen", "Extra close brace or missing open brace"],
        right: ["MissingLeftExtraRight", "Missing \\left or extra \\right"],
        middle: ["ExtraMiddle", "Extra \\middle"]
      }, ne;
    }(J);
    return StackItem.BaseItem = Z, StackItem;
  }
  var Factory = {}, hasRequiredFactory;
  function requireFactory() {
    if (hasRequiredFactory) return Factory;
    hasRequiredFactory = 1;
    var O = Factory.__values || function(q) {
      var G = typeof Symbol == "function" && Symbol.iterator, J = G && q[G], Z = 0;
      if (J) return J.call(q);
      if (q && typeof q.length == "number") return {
        next: function() {
          return q && Z >= q.length && (q = void 0), { value: q && q[Z++], done: !q };
        }
      };
      throw new TypeError(G ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = Factory.__read || function(q, G) {
      var J = typeof Symbol == "function" && q[Symbol.iterator];
      if (!J) return q;
      var Z = J.call(q), te, ne = [], ee;
      try {
        for (; (G === void 0 || G-- > 0) && !(te = Z.next()).done; ) ne.push(te.value);
      } catch (ae) {
        ee = { error: ae };
      } finally {
        try {
          te && !te.done && (J = Z.return) && J.call(Z);
        } finally {
          if (ee) throw ee.error;
        }
      }
      return ne;
    }, H = Factory.__spreadArray || function(q, G, J) {
      if (J || arguments.length === 2) for (var Z = 0, te = G.length, ne; Z < te; Z++)
        (ne || !(Z in G)) && (ne || (ne = Array.prototype.slice.call(G, 0, Z)), ne[Z] = G[Z]);
      return q.concat(ne || Array.prototype.slice.call(G));
    };
    Object.defineProperty(Factory, "__esModule", { value: !0 }), Factory.AbstractFactory = void 0;
    var V = function() {
      function q(G) {
        var J, Z;
        G === void 0 && (G = null), this.defaultKind = "unknown", this.nodeMap = /* @__PURE__ */ new Map(), this.node = {}, G === null && (G = this.constructor.defaultNodes);
        try {
          for (var te = O(Object.keys(G)), ne = te.next(); !ne.done; ne = te.next()) {
            var ee = ne.value;
            this.setNodeClass(ee, G[ee]);
          }
        } catch (ae) {
          J = { error: ae };
        } finally {
          try {
            ne && !ne.done && (Z = te.return) && Z.call(te);
          } finally {
            if (J) throw J.error;
          }
        }
      }
      return q.prototype.create = function(G) {
        for (var J = [], Z = 1; Z < arguments.length; Z++)
          J[Z - 1] = arguments[Z];
        return (this.node[G] || this.node[this.defaultKind]).apply(void 0, H([], _(J), !1));
      }, q.prototype.setNodeClass = function(G, J) {
        this.nodeMap.set(G, J);
        var Z = this, te = this.nodeMap.get(G);
        this.node[G] = function() {
          for (var ne = [], ee = 0; ee < arguments.length; ee++)
            ne[ee] = arguments[ee];
          return new (te.bind.apply(te, H([void 0, Z], _(ne), !1)))();
        };
      }, q.prototype.getNodeClass = function(G) {
        return this.nodeMap.get(G);
      }, q.prototype.deleteNodeClass = function(G) {
        this.nodeMap.delete(G), delete this.node[G];
      }, q.prototype.nodeIsKind = function(G, J) {
        return G instanceof this.getNodeClass(J);
      }, q.prototype.getKinds = function() {
        return Array.from(this.nodeMap.keys());
      }, q.defaultNodes = {}, q;
    }();
    return Factory.AbstractFactory = V, Factory;
  }
  var hasRequiredStackItemFactory;
  function requireStackItemFactory() {
    if (hasRequiredStackItemFactory) return StackItemFactory;
    hasRequiredStackItemFactory = 1;
    var O = StackItemFactory.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _;
    Object.defineProperty(StackItemFactory, "__esModule", { value: !0 });
    var H = requireStackItem(), V = requireFactory(), q = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z;
    }(H.BaseItem), G = function(J) {
      O(Z, J);
      function Z() {
        var te = J !== null && J.apply(this, arguments) || this;
        return te.defaultKind = "dummy", te.configuration = null, te;
      }
      return Z.DefaultStackItems = (_ = {}, _[q.prototype.kind] = q, _), Z;
    }(V.AbstractFactory);
    return StackItemFactory.default = G, StackItemFactory;
  }
  var NodeFactory$1 = {}, hasRequiredNodeFactory$1;
  function requireNodeFactory$1() {
    if (hasRequiredNodeFactory$1) return NodeFactory$1;
    hasRequiredNodeFactory$1 = 1;
    var O = NodeFactory$1.__read || function(G, J) {
      var Z = typeof Symbol == "function" && G[Symbol.iterator];
      if (!Z) return G;
      var te = Z.call(G), ne, ee = [], ae;
      try {
        for (; (J === void 0 || J-- > 0) && !(ne = te.next()).done; ) ee.push(ne.value);
      } catch (oe) {
        ae = { error: oe };
      } finally {
        try {
          ne && !ne.done && (Z = te.return) && Z.call(te);
        } finally {
          if (ae) throw ae.error;
        }
      }
      return ee;
    }, _ = NodeFactory$1.__spreadArray || function(G, J, Z) {
      if (Z || arguments.length === 2) for (var te = 0, ne = J.length, ee; te < ne; te++)
        (ee || !(te in J)) && (ee || (ee = Array.prototype.slice.call(J, 0, te)), ee[te] = J[te]);
      return G.concat(ee || Array.prototype.slice.call(J));
    }, H = NodeFactory$1.__importDefault || function(G) {
      return G && G.__esModule ? G : { default: G };
    };
    Object.defineProperty(NodeFactory$1, "__esModule", { value: !0 }), NodeFactory$1.NodeFactory = void 0;
    var V = H(requireNodeUtil()), q = function() {
      function G() {
        this.mmlFactory = null, this.factory = {
          node: G.createNode,
          token: G.createToken,
          text: G.createText,
          error: G.createError
        };
      }
      return G.createNode = function(J, Z, te, ne, ee) {
        te === void 0 && (te = []), ne === void 0 && (ne = {});
        var ae = J.mmlFactory.create(Z);
        return ae.setChildren(te), ee && ae.appendChild(ee), V.default.setProperties(ae, ne), ae;
      }, G.createToken = function(J, Z, te, ne) {
        te === void 0 && (te = {}), ne === void 0 && (ne = "");
        var ee = J.create("text", ne);
        return J.create("node", Z, [], te, ee);
      }, G.createText = function(J, Z) {
        return Z == null ? null : J.mmlFactory.create("text").setText(Z);
      }, G.createError = function(J, Z) {
        var te = J.create("text", Z), ne = J.create("node", "mtext", [], {}, te), ee = J.create("node", "merror", [ne], { "data-mjx-error": Z });
        return ee;
      }, G.prototype.setMmlFactory = function(J) {
        this.mmlFactory = J;
      }, G.prototype.set = function(J, Z) {
        this.factory[J] = Z;
      }, G.prototype.setCreators = function(J) {
        for (var Z in J)
          this.set(Z, J[Z]);
      }, G.prototype.create = function(J) {
        for (var Z = [], te = 1; te < arguments.length; te++)
          Z[te - 1] = arguments[te];
        var ne = this.factory[J] || this.factory.node, ee = ne.apply(void 0, _([this, Z[0]], O(Z.slice(1)), !1));
        return J === "node" && this.configuration.addNode(Z[0], ee), ee;
      }, G.prototype.get = function(J) {
        return this.factory[J];
      }, G;
    }();
    return NodeFactory$1.NodeFactory = q, NodeFactory$1;
  }
  var hasRequiredParseOptions$1;
  function requireParseOptions$1() {
    if (hasRequiredParseOptions$1) return ParseOptions;
    hasRequiredParseOptions$1 = 1;
    var O = ParseOptions.__read || function(ne, ee) {
      var ae = typeof Symbol == "function" && ne[Symbol.iterator];
      if (!ae) return ne;
      var oe = ae.call(ne), se, ce = [], le;
      try {
        for (; (ee === void 0 || ee-- > 0) && !(se = oe.next()).done; ) ce.push(se.value);
      } catch (ue) {
        le = { error: ue };
      } finally {
        try {
          se && !se.done && (ae = oe.return) && ae.call(oe);
        } finally {
          if (le) throw le.error;
        }
      }
      return ce;
    }, _ = ParseOptions.__spreadArray || function(ne, ee, ae) {
      if (ae || arguments.length === 2) for (var oe = 0, se = ee.length, ce; oe < se; oe++)
        (ce || !(oe in ee)) && (ce || (ce = Array.prototype.slice.call(ee, 0, oe)), ce[oe] = ee[oe]);
      return ne.concat(ce || Array.prototype.slice.call(ee));
    }, H = ParseOptions.__values || function(ne) {
      var ee = typeof Symbol == "function" && Symbol.iterator, ae = ee && ne[ee], oe = 0;
      if (ae) return ae.call(ne);
      if (ne && typeof ne.length == "number") return {
        next: function() {
          return ne && oe >= ne.length && (ne = void 0), { value: ne && ne[oe++], done: !ne };
        }
      };
      throw new TypeError(ee ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, V = ParseOptions.__importDefault || function(ne) {
      return ne && ne.__esModule ? ne : { default: ne };
    };
    Object.defineProperty(ParseOptions, "__esModule", { value: !0 });
    var q = V(requireStackItemFactory()), G = requireNodeFactory$1(), J = V(requireNodeUtil()), Z = requireOptions(), te = function() {
      function ne(ee, ae) {
        ae === void 0 && (ae = []), this.options = {}, this.packageData = /* @__PURE__ */ new Map(), this.parsers = [], this.root = null, this.nodeLists = {}, this.error = !1, this.handlers = ee.handlers, this.nodeFactory = new G.NodeFactory(), this.nodeFactory.configuration = this, this.nodeFactory.setCreators(ee.nodes), this.itemFactory = new q.default(ee.items), this.itemFactory.configuration = this, Z.defaultOptions.apply(void 0, _([this.options], O(ae), !1)), (0, Z.defaultOptions)(this.options, ee.options);
      }
      return ne.prototype.pushParser = function(ee) {
        this.parsers.unshift(ee);
      }, ne.prototype.popParser = function() {
        this.parsers.shift();
      }, Object.defineProperty(ne.prototype, "parser", {
        get: function() {
          return this.parsers[0];
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.clear = function() {
        this.parsers = [], this.root = null, this.nodeLists = {}, this.error = !1, this.tags.resetTag();
      }, ne.prototype.addNode = function(ee, ae) {
        var oe = this.nodeLists[ee];
        if (oe || (oe = this.nodeLists[ee] = []), oe.push(ae), ae.kind !== ee) {
          var se = J.default.getProperty(ae, "in-lists") || "", ce = (se ? se.split(/,/) : []).concat(ee).join(",");
          J.default.setProperty(ae, "in-lists", ce);
        }
      }, ne.prototype.getList = function(ee) {
        var ae, oe, se = this.nodeLists[ee] || [], ce = [];
        try {
          for (var le = H(se), ue = le.next(); !ue.done; ue = le.next()) {
            var de = ue.value;
            this.inTree(de) && ce.push(de);
          }
        } catch (fe) {
          ae = { error: fe };
        } finally {
          try {
            ue && !ue.done && (oe = le.return) && oe.call(le);
          } finally {
            if (ae) throw ae.error;
          }
        }
        return this.nodeLists[ee] = ce, ce;
      }, ne.prototype.removeFromList = function(ee, ae) {
        var oe, se, ce = this.nodeLists[ee] || [];
        try {
          for (var le = H(ae), ue = le.next(); !ue.done; ue = le.next()) {
            var de = ue.value, fe = ce.indexOf(de);
            fe >= 0 && ce.splice(fe, 1);
          }
        } catch (he) {
          oe = { error: he };
        } finally {
          try {
            ue && !ue.done && (se = le.return) && se.call(le);
          } finally {
            if (oe) throw oe.error;
          }
        }
      }, ne.prototype.inTree = function(ee) {
        for (; ee && ee !== this.root; )
          ee = ee.parent;
        return !!ee;
      }, ne;
    }();
    return ParseOptions.default = te, ParseOptions;
  }
  var Tags = {}, hasRequiredTags;
  function requireTags() {
    return hasRequiredTags || (hasRequiredTags = 1, function(O) {
      var _ = Tags.__extends || /* @__PURE__ */ function() {
        var ee = function(ae, oe) {
          return ee = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(se, ce) {
            se.__proto__ = ce;
          } || function(se, ce) {
            for (var le in ce) Object.prototype.hasOwnProperty.call(ce, le) && (se[le] = ce[le]);
          }, ee(ae, oe);
        };
        return function(ae, oe) {
          if (typeof oe != "function" && oe !== null)
            throw new TypeError("Class extends value " + String(oe) + " is not a constructor or null");
          ee(ae, oe);
          function se() {
            this.constructor = ae;
          }
          ae.prototype = oe === null ? Object.create(oe) : (se.prototype = oe.prototype, new se());
        };
      }(), H = Tags.__values || function(ee) {
        var ae = typeof Symbol == "function" && Symbol.iterator, oe = ae && ee[ae], se = 0;
        if (oe) return oe.call(ee);
        if (ee && typeof ee.length == "number") return {
          next: function() {
            return ee && se >= ee.length && (ee = void 0), { value: ee && ee[se++], done: !ee };
          }
        };
        throw new TypeError(ae ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, V = Tags.__importDefault || function(ee) {
        return ee && ee.__esModule ? ee : { default: ee };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.TagsFactory = O.AllTags = O.NoTags = O.AbstractTags = O.TagInfo = O.Label = void 0;
      var q = V(requireTexParser()), G = /* @__PURE__ */ function() {
        function ee(ae, oe) {
          ae === void 0 && (ae = "???"), oe === void 0 && (oe = ""), this.tag = ae, this.id = oe;
        }
        return ee;
      }();
      O.Label = G;
      var J = /* @__PURE__ */ function() {
        function ee(ae, oe, se, ce, le, ue, de, fe) {
          ae === void 0 && (ae = ""), oe === void 0 && (oe = !1), se === void 0 && (se = !1), ce === void 0 && (ce = null), le === void 0 && (le = ""), ue === void 0 && (ue = ""), de === void 0 && (de = !1), fe === void 0 && (fe = ""), this.env = ae, this.taggable = oe, this.defaultTags = se, this.tag = ce, this.tagId = le, this.tagFormat = ue, this.noTag = de, this.labelId = fe;
        }
        return ee;
      }();
      O.TagInfo = J;
      var Z = function() {
        function ee() {
          this.counter = 0, this.allCounter = 0, this.configuration = null, this.ids = {}, this.allIds = {}, this.labels = {}, this.allLabels = {}, this.redo = !1, this.refUpdate = !1, this.currentTag = new J(), this.history = [], this.stack = [], this.enTag = function(ae, oe) {
            var se = this.configuration.nodeFactory, ce = se.create("node", "mtd", [ae]), le = se.create("node", "mlabeledtr", [oe, ce]), ue = se.create("node", "mtable", [le], {
              side: this.configuration.options.tagSide,
              minlabelspacing: this.configuration.options.tagIndent,
              displaystyle: !0
            });
            return ue;
          };
        }
        return ee.prototype.start = function(ae, oe, se) {
          this.currentTag && this.stack.push(this.currentTag), this.currentTag = new J(ae, oe, se);
        }, Object.defineProperty(ee.prototype, "env", {
          get: function() {
            return this.currentTag.env;
          },
          enumerable: !1,
          configurable: !0
        }), ee.prototype.end = function() {
          this.history.push(this.currentTag), this.currentTag = this.stack.pop();
        }, ee.prototype.tag = function(ae, oe) {
          this.currentTag.tag = ae, this.currentTag.tagFormat = oe ? ae : this.formatTag(ae), this.currentTag.noTag = !1;
        }, ee.prototype.notag = function() {
          this.tag("", !0), this.currentTag.noTag = !0;
        }, Object.defineProperty(ee.prototype, "noTag", {
          get: function() {
            return this.currentTag.noTag;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(ee.prototype, "label", {
          get: function() {
            return this.currentTag.labelId;
          },
          set: function(ae) {
            this.currentTag.labelId = ae;
          },
          enumerable: !1,
          configurable: !0
        }), ee.prototype.formatUrl = function(ae, oe) {
          return oe + "#" + encodeURIComponent(ae);
        }, ee.prototype.formatTag = function(ae) {
          return "(" + ae + ")";
        }, ee.prototype.formatId = function(ae) {
          return "mjx-eqn:" + ae.replace(/\s/g, "_");
        }, ee.prototype.formatNumber = function(ae) {
          return ae.toString();
        }, ee.prototype.autoTag = function() {
          this.currentTag.tag == null && (this.counter++, this.tag(this.formatNumber(this.counter), !1));
        }, ee.prototype.clearTag = function() {
          this.label = "", this.tag(null, !0), this.currentTag.tagId = "";
        }, ee.prototype.getTag = function(ae) {
          if (ae === void 0 && (ae = !1), ae)
            return this.autoTag(), this.makeTag();
          var oe = this.currentTag;
          return oe.taggable && !oe.noTag && (oe.defaultTags && this.autoTag(), oe.tag) ? this.makeTag() : null;
        }, ee.prototype.resetTag = function() {
          this.history = [], this.redo = !1, this.refUpdate = !1, this.clearTag();
        }, ee.prototype.reset = function(ae) {
          ae === void 0 && (ae = 0), this.resetTag(), this.counter = this.allCounter = ae, this.allLabels = {}, this.allIds = {};
        }, ee.prototype.startEquation = function(ae) {
          this.history = [], this.stack = [], this.clearTag(), this.currentTag = new J("", void 0, void 0), this.labels = {}, this.ids = {}, this.counter = this.allCounter, this.redo = !1;
          var oe = ae.inputData.recompile;
          oe && (this.refUpdate = !0, this.counter = oe.counter);
        }, ee.prototype.finishEquation = function(ae) {
          this.redo && (ae.inputData.recompile = {
            state: ae.state(),
            counter: this.allCounter
          }), this.refUpdate || (this.allCounter = this.counter), Object.assign(this.allIds, this.ids), Object.assign(this.allLabels, this.labels);
        }, ee.prototype.finalize = function(ae, oe) {
          if (!oe.display || this.currentTag.env || this.currentTag.tag == null)
            return ae;
          var se = this.makeTag(), ce = this.enTag(ae, se);
          return ce;
        }, ee.prototype.makeId = function() {
          this.currentTag.tagId = this.formatId(this.configuration.options.useLabelIds ? this.label || this.currentTag.tag : this.currentTag.tag);
        }, ee.prototype.makeTag = function() {
          this.makeId(), this.label && (this.labels[this.label] = new G(this.currentTag.tag, this.currentTag.tagId));
          var ae = new q.default("\\text{" + this.currentTag.tagFormat + "}", {}, this.configuration).mml();
          return this.configuration.nodeFactory.create("node", "mtd", [ae], { id: this.currentTag.tagId });
        }, ee;
      }();
      O.AbstractTags = Z;
      var te = function(ee) {
        _(ae, ee);
        function ae() {
          return ee !== null && ee.apply(this, arguments) || this;
        }
        return ae.prototype.autoTag = function() {
        }, ae.prototype.getTag = function() {
          return this.currentTag.tag ? ee.prototype.getTag.call(this) : null;
        }, ae;
      }(Z);
      O.NoTags = te;
      var ne = function(ee) {
        _(ae, ee);
        function ae() {
          return ee !== null && ee.apply(this, arguments) || this;
        }
        return ae.prototype.finalize = function(oe, se) {
          if (!se.display || this.history.find(function(le) {
            return le.taggable;
          }))
            return oe;
          var ce = this.getTag(!0);
          return this.enTag(oe, ce);
        }, ae;
      }(Z);
      O.AllTags = ne, function(ee) {
        var ae = /* @__PURE__ */ new Map([
          ["none", te],
          ["all", ne]
        ]), oe = "none";
        ee.OPTIONS = {
          tags: oe,
          tagSide: "right",
          tagIndent: "0.8em",
          useLabelIds: !0,
          ignoreDuplicateLabels: !1
        }, ee.add = function(se, ce) {
          ae.set(se, ce);
        }, ee.addTags = function(se) {
          var ce, le;
          try {
            for (var ue = H(Object.keys(se)), de = ue.next(); !de.done; de = ue.next()) {
              var fe = de.value;
              ee.add(fe, se[fe]);
            }
          } catch (he) {
            ce = { error: he };
          } finally {
            try {
              de && !de.done && (le = ue.return) && le.call(ue);
            } finally {
              if (ce) throw ce.error;
            }
          }
        }, ee.create = function(se) {
          var ce = ae.get(se) || ae.get(oe);
          if (!ce)
            throw Error("Unknown tags class");
          return new ce();
        }, ee.setDefault = function(se) {
          oe = se;
        }, ee.getDefault = function() {
          return ee.create(oe);
        };
      }(O.TagsFactory || (O.TagsFactory = {}));
    }(Tags)), Tags;
  }
  var Configuration$1 = {}, MapHandler = {}, hasRequiredMapHandler;
  function requireMapHandler() {
    return hasRequiredMapHandler || (hasRequiredMapHandler = 1, function(O) {
      var _ = MapHandler.__values || function(te) {
        var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
        if (ee) return ee.call(te);
        if (te && typeof te.length == "number") return {
          next: function() {
            return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
          }
        };
        throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, H = MapHandler.__read || function(te, ne) {
        var ee = typeof Symbol == "function" && te[Symbol.iterator];
        if (!ee) return te;
        var ae = ee.call(te), oe, se = [], ce;
        try {
          for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
        } catch (le) {
          ce = { error: le };
        } finally {
          try {
            oe && !oe.done && (ee = ae.return) && ee.call(ae);
          } finally {
            if (ce) throw ce.error;
          }
        }
        return se;
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.SubHandlers = O.SubHandler = O.MapHandler = void 0;
      var V = requirePrioritizedList(), q = requireFunctionList(), G;
      (function(te) {
        var ne = /* @__PURE__ */ new Map();
        te.register = function(ee) {
          ne.set(ee.name, ee);
        }, te.getMap = function(ee) {
          return ne.get(ee);
        };
      })(G = O.MapHandler || (O.MapHandler = {}));
      var J = function() {
        function te() {
          this._configuration = new V.PrioritizedList(), this._fallback = new q.FunctionList();
        }
        return te.prototype.add = function(ne, ee, ae) {
          var oe, se;
          ae === void 0 && (ae = V.PrioritizedList.DEFAULTPRIORITY);
          try {
            for (var ce = _(ne.slice().reverse()), le = ce.next(); !le.done; le = ce.next()) {
              var ue = le.value, de = G.getMap(ue);
              if (!de) {
                this.warn("Configuration " + ue + " not found! Omitted.");
                return;
              }
              this._configuration.add(de, ae);
            }
          } catch (fe) {
            oe = { error: fe };
          } finally {
            try {
              le && !le.done && (se = ce.return) && se.call(ce);
            } finally {
              if (oe) throw oe.error;
            }
          }
          ee && this._fallback.add(ee, ae);
        }, te.prototype.parse = function(ne) {
          var ee, ae;
          try {
            for (var oe = _(this._configuration), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value.item, le = ce.parse(ne);
              if (le)
                return le;
            }
          } catch (he) {
            ee = { error: he };
          } finally {
            try {
              se && !se.done && (ae = oe.return) && ae.call(oe);
            } finally {
              if (ee) throw ee.error;
            }
          }
          var ue = H(ne, 2), de = ue[0], fe = ue[1];
          Array.from(this._fallback)[0].item(de, fe);
        }, te.prototype.lookup = function(ne) {
          var ee = this.applicable(ne);
          return ee ? ee.lookup(ne) : null;
        }, te.prototype.contains = function(ne) {
          return !!this.applicable(ne);
        }, te.prototype.toString = function() {
          var ne, ee, ae = [];
          try {
            for (var oe = _(this._configuration), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value.item;
              ae.push(ce.name);
            }
          } catch (le) {
            ne = { error: le };
          } finally {
            try {
              se && !se.done && (ee = oe.return) && ee.call(oe);
            } finally {
              if (ne) throw ne.error;
            }
          }
          return ae.join(", ");
        }, te.prototype.applicable = function(ne) {
          var ee, ae;
          try {
            for (var oe = _(this._configuration), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value.item;
              if (ce.contains(ne))
                return ce;
            }
          } catch (le) {
            ee = { error: le };
          } finally {
            try {
              se && !se.done && (ae = oe.return) && ae.call(oe);
            } finally {
              if (ee) throw ee.error;
            }
          }
          return null;
        }, te.prototype.retrieve = function(ne) {
          var ee, ae;
          try {
            for (var oe = _(this._configuration), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value.item;
              if (ce.name === ne)
                return ce;
            }
          } catch (le) {
            ee = { error: le };
          } finally {
            try {
              se && !se.done && (ae = oe.return) && ae.call(oe);
            } finally {
              if (ee) throw ee.error;
            }
          }
          return null;
        }, te.prototype.warn = function(ne) {
          console.log("TexParser Warning: " + ne);
        }, te;
      }();
      O.SubHandler = J;
      var Z = function() {
        function te() {
          this.map = /* @__PURE__ */ new Map();
        }
        return te.prototype.add = function(ne, ee, ae) {
          var oe, se;
          ae === void 0 && (ae = V.PrioritizedList.DEFAULTPRIORITY);
          try {
            for (var ce = _(Object.keys(ne)), le = ce.next(); !le.done; le = ce.next()) {
              var ue = le.value, de = ue, fe = this.get(de);
              fe || (fe = new J(), this.set(de, fe)), fe.add(ne[de], ee[de], ae);
            }
          } catch (he) {
            oe = { error: he };
          } finally {
            try {
              le && !le.done && (se = ce.return) && se.call(ce);
            } finally {
              if (oe) throw oe.error;
            }
          }
        }, te.prototype.set = function(ne, ee) {
          this.map.set(ne, ee);
        }, te.prototype.get = function(ne) {
          return this.map.get(ne);
        }, te.prototype.retrieve = function(ne) {
          var ee, ae;
          try {
            for (var oe = _(this.map.values()), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value, le = ce.retrieve(ne);
              if (le)
                return le;
            }
          } catch (ue) {
            ee = { error: ue };
          } finally {
            try {
              se && !se.done && (ae = oe.return) && ae.call(oe);
            } finally {
              if (ee) throw ee.error;
            }
          }
          return null;
        }, te.prototype.keys = function() {
          return this.map.keys();
        }, te;
      }();
      O.SubHandlers = Z;
    }(MapHandler)), MapHandler;
  }
  var hasRequiredConfiguration;
  function requireConfiguration() {
    return hasRequiredConfiguration || (hasRequiredConfiguration = 1, function(O) {
      var _ = Configuration$1.__values || function(ae) {
        var oe = typeof Symbol == "function" && Symbol.iterator, se = oe && ae[oe], ce = 0;
        if (se) return se.call(ae);
        if (ae && typeof ae.length == "number") return {
          next: function() {
            return ae && ce >= ae.length && (ae = void 0), { value: ae && ae[ce++], done: !ae };
          }
        };
        throw new TypeError(oe ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, H = Configuration$1.__read || function(ae, oe) {
        var se = typeof Symbol == "function" && ae[Symbol.iterator];
        if (!se) return ae;
        var ce = se.call(ae), le, ue = [], de;
        try {
          for (; (oe === void 0 || oe-- > 0) && !(le = ce.next()).done; ) ue.push(le.value);
        } catch (fe) {
          de = { error: fe };
        } finally {
          try {
            le && !le.done && (se = ce.return) && se.call(ce);
          } finally {
            if (de) throw de.error;
          }
        }
        return ue;
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.ParserConfiguration = O.ConfigurationHandler = O.Configuration = void 0;
      var V = requireOptions(), q = requireMapHandler(), G = requireFunctionList(), J = requirePrioritizedList(), Z = requireTags(), te = function() {
        function ae(oe, se, ce, le, ue, de, fe, he, me, pe, ye, ge, ve) {
          se === void 0 && (se = {}), ce === void 0 && (ce = {}), le === void 0 && (le = {}), ue === void 0 && (ue = {}), de === void 0 && (de = {}), fe === void 0 && (fe = {}), he === void 0 && (he = []), me === void 0 && (me = []), pe === void 0 && (pe = null), ye === void 0 && (ye = null), this.name = oe, this.handler = se, this.fallback = ce, this.items = le, this.tags = ue, this.options = de, this.nodes = fe, this.preprocessors = he, this.postprocessors = me, this.initMethod = pe, this.configMethod = ye, this.priority = ge, this.parser = ve, this.handler = Object.assign({ character: [], delimiter: [], macro: [], environment: [] }, se);
        }
        return ae.makeProcessor = function(oe, se) {
          return Array.isArray(oe) ? oe : [oe, se];
        }, ae._create = function(oe, se) {
          var ce = this;
          se === void 0 && (se = {});
          var le = se.priority || J.PrioritizedList.DEFAULTPRIORITY, ue = se.init ? this.makeProcessor(se.init, le) : null, de = se.config ? this.makeProcessor(se.config, le) : null, fe = (se.preprocessors || []).map(function(pe) {
            return ce.makeProcessor(pe, le);
          }), he = (se.postprocessors || []).map(function(pe) {
            return ce.makeProcessor(pe, le);
          }), me = se.parser || "tex";
          return new ae(oe, se.handler || {}, se.fallback || {}, se.items || {}, se.tags || {}, se.options || {}, se.nodes || {}, fe, he, ue, de, le, me);
        }, ae.create = function(oe, se) {
          se === void 0 && (se = {});
          var ce = ae._create(oe, se);
          return ne.set(oe, ce), ce;
        }, ae.local = function(oe) {
          return oe === void 0 && (oe = {}), ae._create("", oe);
        }, Object.defineProperty(ae.prototype, "init", {
          get: function() {
            return this.initMethod ? this.initMethod[0] : null;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(ae.prototype, "config", {
          get: function() {
            return this.configMethod ? this.configMethod[0] : null;
          },
          enumerable: !1,
          configurable: !0
        }), ae;
      }();
      O.Configuration = te;
      var ne;
      (function(ae) {
        var oe = /* @__PURE__ */ new Map();
        ae.set = function(se, ce) {
          oe.set(se, ce);
        }, ae.get = function(se) {
          return oe.get(se);
        }, ae.keys = function() {
          return oe.keys();
        };
      })(ne = O.ConfigurationHandler || (O.ConfigurationHandler = {}));
      var ee = function() {
        function ae(oe, se) {
          var ce, le, ue, de;
          se === void 0 && (se = ["tex"]), this.initMethod = new G.FunctionList(), this.configMethod = new G.FunctionList(), this.configurations = new J.PrioritizedList(), this.parsers = [], this.handlers = new q.SubHandlers(), this.items = {}, this.tags = {}, this.options = {}, this.nodes = {}, this.parsers = se;
          try {
            for (var fe = _(oe.slice().reverse()), he = fe.next(); !he.done; he = fe.next()) {
              var me = he.value;
              this.addPackage(me);
            }
          } catch (Ce) {
            ce = { error: Ce };
          } finally {
            try {
              he && !he.done && (le = fe.return) && le.call(fe);
            } finally {
              if (ce) throw ce.error;
            }
          }
          try {
            for (var pe = _(this.configurations), ye = pe.next(); !ye.done; ye = pe.next()) {
              var ge = ye.value, ve = ge.item, be = ge.priority;
              this.append(ve, be);
            }
          } catch (Ce) {
            ue = { error: Ce };
          } finally {
            try {
              ye && !ye.done && (de = pe.return) && de.call(pe);
            } finally {
              if (ue) throw ue.error;
            }
          }
        }
        return ae.prototype.init = function() {
          this.initMethod.execute(this);
        }, ae.prototype.config = function(oe) {
          var se, ce;
          this.configMethod.execute(this, oe);
          try {
            for (var le = _(this.configurations), ue = le.next(); !ue.done; ue = le.next()) {
              var de = ue.value;
              this.addFilters(oe, de.item);
            }
          } catch (fe) {
            se = { error: fe };
          } finally {
            try {
              ue && !ue.done && (ce = le.return) && ce.call(le);
            } finally {
              if (se) throw se.error;
            }
          }
        }, ae.prototype.addPackage = function(oe) {
          var se = typeof oe == "string" ? oe : oe[0], ce = this.getPackage(se);
          ce && this.configurations.add(ce, typeof oe == "string" ? ce.priority : oe[1]);
        }, ae.prototype.add = function(oe, se, ce) {
          var le, ue;
          ce === void 0 && (ce = {});
          var de = this.getPackage(oe);
          this.append(de), this.configurations.add(de, de.priority), this.init();
          var fe = se.parseOptions;
          fe.nodeFactory.setCreators(de.nodes);
          try {
            for (var he = _(Object.keys(de.items)), me = he.next(); !me.done; me = he.next()) {
              var pe = me.value;
              fe.itemFactory.setNodeClass(pe, de.items[pe]);
            }
          } catch (ye) {
            le = { error: ye };
          } finally {
            try {
              me && !me.done && (ue = he.return) && ue.call(he);
            } finally {
              if (le) throw le.error;
            }
          }
          Z.TagsFactory.addTags(de.tags), (0, V.defaultOptions)(fe.options, de.options), (0, V.userOptions)(fe.options, ce), this.addFilters(se, de), de.config && de.config(this, se);
        }, ae.prototype.getPackage = function(oe) {
          var se = ne.get(oe);
          if (se && this.parsers.indexOf(se.parser) < 0)
            throw Error("Package ".concat(oe, " doesn't target the proper parser"));
          return se;
        }, ae.prototype.append = function(oe, se) {
          se = se || oe.priority, oe.initMethod && this.initMethod.add(oe.initMethod[0], oe.initMethod[1]), oe.configMethod && this.configMethod.add(oe.configMethod[0], oe.configMethod[1]), this.handlers.add(oe.handler, oe.fallback, se), Object.assign(this.items, oe.items), Object.assign(this.tags, oe.tags), (0, V.defaultOptions)(this.options, oe.options), Object.assign(this.nodes, oe.nodes);
        }, ae.prototype.addFilters = function(oe, se) {
          var ce, le, ue, de;
          try {
            for (var fe = _(se.preprocessors), he = fe.next(); !he.done; he = fe.next()) {
              var me = H(he.value, 2), pe = me[0], ye = me[1];
              oe.preFilters.add(pe, ye);
            }
          } catch (Se) {
            ce = { error: Se };
          } finally {
            try {
              he && !he.done && (le = fe.return) && le.call(fe);
            } finally {
              if (ce) throw ce.error;
            }
          }
          try {
            for (var ge = _(se.postprocessors), ve = ge.next(); !ve.done; ve = ge.next()) {
              var be = H(ve.value, 2), Ce = be[0], ye = be[1];
              oe.postFilters.add(Ce, ye);
            }
          } catch (Se) {
            ue = { error: Se };
          } finally {
            try {
              ve && !ve.done && (de = ge.return) && de.call(ge);
            } finally {
              if (ue) throw ue.error;
            }
          }
        }, ae;
      }();
      O.ParserConfiguration = ee;
    }(Configuration$1)), Configuration$1;
  }
  var BaseConfiguration = {}, SymbolMap = {}, _Symbol = {}, hasRequired_Symbol;
  function require_Symbol() {
    if (hasRequired_Symbol) return _Symbol;
    hasRequired_Symbol = 1, Object.defineProperty(_Symbol, "__esModule", { value: !0 }), _Symbol.Macro = _Symbol.Symbol = void 0;
    var O = function() {
      function H(V, q, G) {
        this._symbol = V, this._char = q, this._attributes = G;
      }
      return Object.defineProperty(H.prototype, "symbol", {
        get: function() {
          return this._symbol;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(H.prototype, "char", {
        get: function() {
          return this._char;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(H.prototype, "attributes", {
        get: function() {
          return this._attributes;
        },
        enumerable: !1,
        configurable: !0
      }), H;
    }();
    _Symbol.Symbol = O;
    var _ = function() {
      function H(V, q, G) {
        G === void 0 && (G = []), this._symbol = V, this._func = q, this._args = G;
      }
      return Object.defineProperty(H.prototype, "symbol", {
        get: function() {
          return this._symbol;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(H.prototype, "func", {
        get: function() {
          return this._func;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(H.prototype, "args", {
        get: function() {
          return this._args;
        },
        enumerable: !1,
        configurable: !0
      }), H;
    }();
    return _Symbol.Macro = _, _Symbol;
  }
  var hasRequiredSymbolMap;
  function requireSymbolMap() {
    if (hasRequiredSymbolMap) return SymbolMap;
    hasRequiredSymbolMap = 1;
    var O = SymbolMap.__extends || /* @__PURE__ */ function() {
      var le = function(ue, de) {
        return le = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(fe, he) {
          fe.__proto__ = he;
        } || function(fe, he) {
          for (var me in he) Object.prototype.hasOwnProperty.call(he, me) && (fe[me] = he[me]);
        }, le(ue, de);
      };
      return function(ue, de) {
        if (typeof de != "function" && de !== null)
          throw new TypeError("Class extends value " + String(de) + " is not a constructor or null");
        le(ue, de);
        function fe() {
          this.constructor = ue;
        }
        ue.prototype = de === null ? Object.create(de) : (fe.prototype = de.prototype, new fe());
      };
    }(), _ = SymbolMap.__read || function(le, ue) {
      var de = typeof Symbol == "function" && le[Symbol.iterator];
      if (!de) return le;
      var fe = de.call(le), he, me = [], pe;
      try {
        for (; (ue === void 0 || ue-- > 0) && !(he = fe.next()).done; ) me.push(he.value);
      } catch (ye) {
        pe = { error: ye };
      } finally {
        try {
          he && !he.done && (de = fe.return) && de.call(fe);
        } finally {
          if (pe) throw pe.error;
        }
      }
      return me;
    }, H = SymbolMap.__values || function(le) {
      var ue = typeof Symbol == "function" && Symbol.iterator, de = ue && le[ue], fe = 0;
      if (de) return de.call(le);
      if (le && typeof le.length == "number") return {
        next: function() {
          return le && fe >= le.length && (le = void 0), { value: le && le[fe++], done: !le };
        }
      };
      throw new TypeError(ue ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, V = SymbolMap.__spreadArray || function(le, ue, de) {
      if (de || arguments.length === 2) for (var fe = 0, he = ue.length, me; fe < he; fe++)
        (me || !(fe in ue)) && (me || (me = Array.prototype.slice.call(ue, 0, fe)), me[fe] = ue[fe]);
      return le.concat(me || Array.prototype.slice.call(ue));
    };
    Object.defineProperty(SymbolMap, "__esModule", { value: !0 }), SymbolMap.EnvironmentMap = SymbolMap.CommandMap = SymbolMap.MacroMap = SymbolMap.DelimiterMap = SymbolMap.CharacterMap = SymbolMap.AbstractParseMap = SymbolMap.RegExpMap = SymbolMap.AbstractSymbolMap = SymbolMap.parseResult = void 0;
    var q = require_Symbol(), G = requireMapHandler();
    function J(le) {
      return le === void 0 ? !0 : le;
    }
    SymbolMap.parseResult = J;
    var Z = function() {
      function le(ue, de) {
        this._name = ue, this._parser = de, G.MapHandler.register(this);
      }
      return Object.defineProperty(le.prototype, "name", {
        get: function() {
          return this._name;
        },
        enumerable: !1,
        configurable: !0
      }), le.prototype.parserFor = function(ue) {
        return this.contains(ue) ? this.parser : null;
      }, le.prototype.parse = function(ue) {
        var de = _(ue, 2), fe = de[0], he = de[1], me = this.parserFor(he), pe = this.lookup(he);
        return me && pe ? J(me(fe, pe)) : null;
      }, Object.defineProperty(le.prototype, "parser", {
        get: function() {
          return this._parser;
        },
        set: function(ue) {
          this._parser = ue;
        },
        enumerable: !1,
        configurable: !0
      }), le;
    }();
    SymbolMap.AbstractSymbolMap = Z;
    var te = function(le) {
      O(ue, le);
      function ue(de, fe, he) {
        var me = le.call(this, de, fe) || this;
        return me._regExp = he, me;
      }
      return ue.prototype.contains = function(de) {
        return this._regExp.test(de);
      }, ue.prototype.lookup = function(de) {
        return this.contains(de) ? de : null;
      }, ue;
    }(Z);
    SymbolMap.RegExpMap = te;
    var ne = function(le) {
      O(ue, le);
      function ue() {
        var de = le !== null && le.apply(this, arguments) || this;
        return de.map = /* @__PURE__ */ new Map(), de;
      }
      return ue.prototype.lookup = function(de) {
        return this.map.get(de);
      }, ue.prototype.contains = function(de) {
        return this.map.has(de);
      }, ue.prototype.add = function(de, fe) {
        this.map.set(de, fe);
      }, ue.prototype.remove = function(de) {
        this.map.delete(de);
      }, ue;
    }(Z);
    SymbolMap.AbstractParseMap = ne;
    var ee = function(le) {
      O(ue, le);
      function ue(de, fe, he) {
        var me, pe, ye = le.call(this, de, fe) || this;
        try {
          for (var ge = H(Object.keys(he)), ve = ge.next(); !ve.done; ve = ge.next()) {
            var be = ve.value, Ce = he[be], Se = _(typeof Ce == "string" ? [Ce, null] : Ce, 2), we = Se[0], Ee = Se[1], xe = new q.Symbol(be, we, Ee);
            ye.add(be, xe);
          }
        } catch (Le) {
          me = { error: Le };
        } finally {
          try {
            ve && !ve.done && (pe = ge.return) && pe.call(ge);
          } finally {
            if (me) throw me.error;
          }
        }
        return ye;
      }
      return ue;
    }(ne);
    SymbolMap.CharacterMap = ee;
    var ae = function(le) {
      O(ue, le);
      function ue() {
        return le !== null && le.apply(this, arguments) || this;
      }
      return ue.prototype.parse = function(de) {
        var fe = _(de, 2), he = fe[0], me = fe[1];
        return le.prototype.parse.call(this, [he, "\\" + me]);
      }, ue;
    }(ee);
    SymbolMap.DelimiterMap = ae;
    var oe = function(le) {
      O(ue, le);
      function ue(de, fe, he) {
        var me, pe, ye = le.call(this, de, null) || this;
        try {
          for (var ge = H(Object.keys(fe)), ve = ge.next(); !ve.done; ve = ge.next()) {
            var be = ve.value, Ce = fe[be], Se = _(typeof Ce == "string" ? [Ce] : Ce), we = Se[0], Ee = Se.slice(1), xe = new q.Macro(be, he[we], Ee);
            ye.add(be, xe);
          }
        } catch (Le) {
          me = { error: Le };
        } finally {
          try {
            ve && !ve.done && (pe = ge.return) && pe.call(ge);
          } finally {
            if (me) throw me.error;
          }
        }
        return ye;
      }
      return ue.prototype.parserFor = function(de) {
        var fe = this.lookup(de);
        return fe ? fe.func : null;
      }, ue.prototype.parse = function(de) {
        var fe = _(de, 2), he = fe[0], me = fe[1], pe = this.lookup(me), ye = this.parserFor(me);
        return !pe || !ye ? null : J(ye.apply(void 0, V([he, pe.symbol], _(pe.args), !1)));
      }, ue;
    }(ne);
    SymbolMap.MacroMap = oe;
    var se = function(le) {
      O(ue, le);
      function ue() {
        return le !== null && le.apply(this, arguments) || this;
      }
      return ue.prototype.parse = function(de) {
        var fe = _(de, 2), he = fe[0], me = fe[1], pe = this.lookup(me), ye = this.parserFor(me);
        if (!pe || !ye)
          return null;
        var ge = he.currentCS;
        he.currentCS = "\\" + me;
        var ve = ye.apply(void 0, V([he, "\\" + pe.symbol], _(pe.args), !1));
        return he.currentCS = ge, J(ve);
      }, ue;
    }(oe);
    SymbolMap.CommandMap = se;
    var ce = function(le) {
      O(ue, le);
      function ue(de, fe, he, me) {
        var pe = le.call(this, de, he, me) || this;
        return pe.parser = fe, pe;
      }
      return ue.prototype.parse = function(de) {
        var fe = _(de, 2), he = fe[0], me = fe[1], pe = this.lookup(me), ye = this.parserFor(me);
        return !pe || !ye ? null : J(this.parser(he, pe.symbol, ye, pe.args));
      }, ue;
    }(oe);
    return SymbolMap.EnvironmentMap = ce, SymbolMap;
  }
  var BaseItems = {}, hasRequiredBaseItems;
  function requireBaseItems() {
    if (hasRequiredBaseItems) return BaseItems;
    hasRequiredBaseItems = 1;
    var O = BaseItems.__extends || /* @__PURE__ */ function() {
      var Me = function(Re, De) {
        return Me = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Pe, $e) {
          Pe.__proto__ = $e;
        } || function(Pe, $e) {
          for (var qe in $e) Object.prototype.hasOwnProperty.call($e, qe) && (Pe[qe] = $e[qe]);
        }, Me(Re, De);
      };
      return function(Re, De) {
        if (typeof De != "function" && De !== null)
          throw new TypeError("Class extends value " + String(De) + " is not a constructor or null");
        Me(Re, De);
        function Pe() {
          this.constructor = Re;
        }
        Re.prototype = De === null ? Object.create(De) : (Pe.prototype = De.prototype, new Pe());
      };
    }(), _ = BaseItems.__read || function(Me, Re) {
      var De = typeof Symbol == "function" && Me[Symbol.iterator];
      if (!De) return Me;
      var Pe = De.call(Me), $e, qe = [], Xe;
      try {
        for (; (Re === void 0 || Re-- > 0) && !($e = Pe.next()).done; ) qe.push($e.value);
      } catch (Ve) {
        Xe = { error: Ve };
      } finally {
        try {
          $e && !$e.done && (De = Pe.return) && De.call(Pe);
        } finally {
          if (Xe) throw Xe.error;
        }
      }
      return qe;
    }, H = BaseItems.__spreadArray || function(Me, Re, De) {
      if (De || arguments.length === 2) for (var Pe = 0, $e = Re.length, qe; Pe < $e; Pe++)
        (qe || !(Pe in Re)) && (qe || (qe = Array.prototype.slice.call(Re, 0, Pe)), qe[Pe] = Re[Pe]);
      return Me.concat(qe || Array.prototype.slice.call(Re));
    }, V = BaseItems.__importDefault || function(Me) {
      return Me && Me.__esModule ? Me : { default: Me };
    };
    Object.defineProperty(BaseItems, "__esModule", { value: !0 }), BaseItems.EquationItem = BaseItems.EqnArrayItem = BaseItems.ArrayItem = BaseItems.DotsItem = BaseItems.NonscriptItem = BaseItems.NotItem = BaseItems.FnItem = BaseItems.MmlItem = BaseItems.CellItem = BaseItems.PositionItem = BaseItems.StyleItem = BaseItems.EndItem = BaseItems.BeginItem = BaseItems.RightItem = BaseItems.Middle = BaseItems.LeftItem = BaseItems.OverItem = BaseItems.SubsupItem = BaseItems.PrimeItem = BaseItems.CloseItem = BaseItems.OpenItem = BaseItems.StopItem = BaseItems.StartItem = void 0;
    var q = requireMapHandler(), G = requireEntities(), J = requireMmlNode(), Z = V(requireTexError()), te = V(requireParseUtil()), ne = V(requireNodeUtil()), ee = requireStackItem(), ae = function(Me) {
      O(Re, Me);
      function Re(De, Pe) {
        var $e = Me.call(this, De) || this;
        return $e.global = Pe, $e;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "start";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("stop")) {
          var Pe = this.toMml();
          return this.global.isInner || (Pe = this.factory.configuration.tags.finalize(Pe, this.env)), [[this.factory.create("mml", Pe)], !0];
        }
        return Me.prototype.checkItem.call(this, De);
      }, Re;
    }(ee.BaseItem);
    BaseItems.StartItem = ae;
    var oe = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "stop";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isClose", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re;
    }(ee.BaseItem);
    BaseItems.StopItem = oe;
    var se = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "open";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("close")) {
          var Pe = this.toMml(), $e = this.create("node", "TeXAtom", [Pe]);
          return [[this.factory.create("mml", $e)], !0];
        }
        return Me.prototype.checkItem.call(this, De);
      }, Re.errors = Object.assign(Object.create(ee.BaseItem.errors), {
        stop: [
          "ExtraOpenMissingClose",
          "Extra open brace or missing close brace"
        ]
      }), Re;
    }(ee.BaseItem);
    BaseItems.OpenItem = se;
    var ce = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "close";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isClose", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re;
    }(ee.BaseItem);
    BaseItems.CloseItem = ce;
    var le = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "prime";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        var Pe = _(this.Peek(2), 2), $e = Pe[0], qe = Pe[1];
        if (!ne.default.isType($e, "msubsup") || ne.default.isType($e, "msup")) {
          var Xe = this.create("node", "msup", [$e, qe]);
          return [[Xe, De], !0];
        }
        return ne.default.setChild($e, $e.sup, qe), [[$e, De], !0];
      }, Re;
    }(ee.BaseItem);
    BaseItems.PrimeItem = le;
    var ue = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "subsup";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("open") || De.isKind("left"))
          return ee.BaseItem.success;
        var Pe = this.First, $e = this.getProperty("position");
        if (De.isKind("mml")) {
          if (this.getProperty("primes"))
            if ($e !== 2)
              ne.default.setChild(Pe, 2, this.getProperty("primes"));
            else {
              ne.default.setProperty(this.getProperty("primes"), "variantForm", !0);
              var qe = this.create("node", "mrow", [this.getProperty("primes"), De.First]);
              De.First = qe;
            }
          ne.default.setChild(Pe, $e, De.First), this.getProperty("movesupsub") != null && ne.default.setProperty(Pe, "movesupsub", this.getProperty("movesupsub"));
          var Xe = this.factory.create("mml", Pe);
          return [[Xe], !0];
        }
        if (Me.prototype.checkItem.call(this, De)[1]) {
          var Ve = this.getErrors(["", "sub", "sup"][$e]);
          throw new (Z.default.bind.apply(Z.default, H([void 0, Ve[0], Ve[1]], _(Ve.splice(2)), !1)))();
        }
        return null;
      }, Re.errors = Object.assign(Object.create(ee.BaseItem.errors), {
        stop: [
          "MissingScript",
          "Missing superscript or subscript argument"
        ],
        sup: [
          "MissingOpenForSup",
          "Missing open brace for superscript"
        ],
        sub: [
          "MissingOpenForSub",
          "Missing open brace for subscript"
        ]
      }), Re;
    }(ee.BaseItem);
    BaseItems.SubsupItem = ue;
    var de = function(Me) {
      O(Re, Me);
      function Re(De) {
        var Pe = Me.call(this, De) || this;
        return Pe.setProperty("name", "\\over"), Pe;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "over";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isClose", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("over"))
          throw new Z.default("AmbiguousUseOf", "Ambiguous use of %1", De.getName());
        if (De.isClose) {
          var Pe = this.create("node", "mfrac", [this.getProperty("num"), this.toMml(!1)]);
          return this.getProperty("thickness") != null && ne.default.setAttribute(Pe, "linethickness", this.getProperty("thickness")), (this.getProperty("open") || this.getProperty("close")) && (ne.default.setProperty(Pe, "withDelims", !0), Pe = te.default.fixedFence(this.factory.configuration, this.getProperty("open"), Pe, this.getProperty("close"))), [[this.factory.create("mml", Pe), De], !0];
        }
        return Me.prototype.checkItem.call(this, De);
      }, Re.prototype.toString = function() {
        return "over[" + this.getProperty("num") + " / " + this.nodes.join("; ") + "]";
      }, Re;
    }(ee.BaseItem);
    BaseItems.OverItem = de;
    var fe = function(Me) {
      O(Re, Me);
      function Re(De, Pe) {
        var $e = Me.call(this, De) || this;
        return $e.setProperty("delim", Pe), $e;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "left";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("right"))
          return [[this.factory.create("mml", te.default.fenced(this.factory.configuration, this.getProperty("delim"), this.toMml(), De.getProperty("delim"), "", De.getProperty("color")))], !0];
        if (De.isKind("middle")) {
          var Pe = { stretchy: !0 };
          return De.getProperty("color") && (Pe.mathcolor = De.getProperty("color")), this.Push(this.create("node", "TeXAtom", [], { texClass: J.TEXCLASS.CLOSE }), this.create("token", "mo", Pe, De.getProperty("delim")), this.create("node", "TeXAtom", [], { texClass: J.TEXCLASS.OPEN })), this.env = {}, [[this], !0];
        }
        return Me.prototype.checkItem.call(this, De);
      }, Re.errors = Object.assign(Object.create(ee.BaseItem.errors), {
        stop: [
          "ExtraLeftMissingRight",
          "Extra \\left or missing \\right"
        ]
      }), Re;
    }(ee.BaseItem);
    BaseItems.LeftItem = fe;
    var he = function(Me) {
      O(Re, Me);
      function Re(De, Pe, $e) {
        var qe = Me.call(this, De) || this;
        return qe.setProperty("delim", Pe), $e && qe.setProperty("color", $e), qe;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "middle";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isClose", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re;
    }(ee.BaseItem);
    BaseItems.Middle = he;
    var me = function(Me) {
      O(Re, Me);
      function Re(De, Pe, $e) {
        var qe = Me.call(this, De) || this;
        return qe.setProperty("delim", Pe), $e && qe.setProperty("color", $e), qe;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "right";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isClose", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re;
    }(ee.BaseItem);
    BaseItems.RightItem = me;
    var pe = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "begin";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("end")) {
          if (De.getName() !== this.getName())
            throw new Z.default("EnvBadEnd", "\\begin{%1} ended with \\end{%2}", this.getName(), De.getName());
          return this.getProperty("end") ? ee.BaseItem.fail : [[this.factory.create("mml", this.toMml())], !0];
        }
        if (De.isKind("stop"))
          throw new Z.default("EnvMissingEnd", "Missing \\end{%1}", this.getName());
        return Me.prototype.checkItem.call(this, De);
      }, Re;
    }(ee.BaseItem);
    BaseItems.BeginItem = pe;
    var ye = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "end";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isClose", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re;
    }(ee.BaseItem);
    BaseItems.EndItem = ye;
    var ge = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "style";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (!De.isClose)
          return Me.prototype.checkItem.call(this, De);
        var Pe = this.create("node", "mstyle", this.nodes, this.getProperty("styles"));
        return [[this.factory.create("mml", Pe), De], !0];
      }, Re;
    }(ee.BaseItem);
    BaseItems.StyleItem = ge;
    var ve = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "position";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isClose)
          throw new Z.default("MissingBoxFor", "Missing box for %1", this.getName());
        if (De.isFinal) {
          var Pe = De.toMml();
          switch (this.getProperty("move")) {
            case "vertical":
              return Pe = this.create("node", "mpadded", [Pe], {
                height: this.getProperty("dh"),
                depth: this.getProperty("dd"),
                voffset: this.getProperty("dh")
              }), [[this.factory.create("mml", Pe)], !0];
            case "horizontal":
              return [[
                this.factory.create("mml", this.getProperty("left")),
                De,
                this.factory.create("mml", this.getProperty("right"))
              ], !0];
          }
        }
        return Me.prototype.checkItem.call(this, De);
      }, Re;
    }(ee.BaseItem);
    BaseItems.PositionItem = ve;
    var be = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "cell";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isClose", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re;
    }(ee.BaseItem);
    BaseItems.CellItem = be;
    var Ce = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "isFinal", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "mml";
        },
        enumerable: !1,
        configurable: !0
      }), Re;
    }(ee.BaseItem);
    BaseItems.MmlItem = Ce;
    var Se = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "fn";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        var Pe = this.First;
        if (Pe) {
          if (De.isOpen)
            return ee.BaseItem.success;
          if (!De.isKind("fn")) {
            var $e = De.First;
            if (!De.isKind("mml") || !$e)
              return [[Pe, De], !0];
            if (ne.default.isType($e, "mstyle") && $e.childNodes.length && ne.default.isType($e.childNodes[0].childNodes[0], "mspace") || ne.default.isType($e, "mspace"))
              return [[Pe, De], !0];
            ne.default.isEmbellished($e) && ($e = ne.default.getCoreMO($e));
            var qe = ne.default.getForm($e);
            if (qe != null && [0, 0, 1, 1, 0, 1, 1, 0, 0, 0][qe[2]])
              return [[Pe, De], !0];
          }
          var Xe = this.create("token", "mo", { texClass: J.TEXCLASS.NONE }, G.entities.ApplyFunction);
          return [[Pe, Xe, De], !0];
        }
        return Me.prototype.checkItem.apply(this, arguments);
      }, Re;
    }(ee.BaseItem);
    BaseItems.FnItem = Se;
    var we = function(Me) {
      O(Re, Me);
      function Re() {
        var De = Me !== null && Me.apply(this, arguments) || this;
        return De.remap = q.MapHandler.getMap("not_remap"), De;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "not";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        var Pe, $e, qe;
        if (De.isKind("open") || De.isKind("left"))
          return ee.BaseItem.success;
        if (De.isKind("mml") && (ne.default.isType(De.First, "mo") || ne.default.isType(De.First, "mi") || ne.default.isType(De.First, "mtext")) && (Pe = De.First, $e = ne.default.getText(Pe), $e.length === 1 && !ne.default.getProperty(Pe, "movesupsub") && ne.default.getChildren(Pe).length === 1))
          return this.remap.contains($e) ? (qe = this.create("text", this.remap.lookup($e).char), ne.default.setChild(Pe, 0, qe)) : (qe = this.create("text", "̸"), ne.default.appendChildren(Pe, [qe])), [[De], !0];
        qe = this.create("text", "⧸");
        var Xe = this.create("node", "mtext", [], {}, qe), Ve = this.create("node", "mpadded", [Xe], { width: 0 });
        return Pe = this.create("node", "TeXAtom", [Ve], { texClass: J.TEXCLASS.REL }), [[Pe, De], !0];
      }, Re;
    }(ee.BaseItem);
    BaseItems.NotItem = we;
    var Ee = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "nonscript";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("mml") && De.Size() === 1) {
          var Pe = De.First;
          if (Pe.isKind("mstyle") && Pe.notParent && (Pe = ne.default.getChildren(ne.default.getChildren(Pe)[0])[0]), Pe.isKind("mspace")) {
            if (Pe !== De.First) {
              var $e = this.create("node", "mrow", [De.Pop()]);
              De.Push($e);
            }
            this.factory.configuration.addNode("nonscript", De.First);
          }
        }
        return [[De], !0];
      }, Re;
    }(ee.BaseItem);
    BaseItems.NonscriptItem = Ee;
    var xe = function(Me) {
      O(Re, Me);
      function Re() {
        return Me !== null && Me.apply(this, arguments) || this;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "dots";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("open") || De.isKind("left"))
          return ee.BaseItem.success;
        var Pe = this.getProperty("ldots"), $e = De.First;
        if (De.isKind("mml") && ne.default.isEmbellished($e)) {
          var qe = ne.default.getTexClass(ne.default.getCoreMO($e));
          (qe === J.TEXCLASS.BIN || qe === J.TEXCLASS.REL) && (Pe = this.getProperty("cdots"));
        }
        return [[Pe, De], !0];
      }, Re;
    }(ee.BaseItem);
    BaseItems.DotsItem = xe;
    var Le = function(Me) {
      O(Re, Me);
      function Re() {
        var De = Me !== null && Me.apply(this, arguments) || this;
        return De.table = [], De.row = [], De.frame = [], De.hfill = [], De.arraydef = {}, De.dashed = !1, De;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "array";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "copyEnv", {
        get: function() {
          return !1;
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isClose && !De.isKind("over")) {
          if (De.getProperty("isEntry"))
            return this.EndEntry(), this.clearEnv(), ee.BaseItem.fail;
          if (De.getProperty("isCR"))
            return this.EndEntry(), this.EndRow(), this.clearEnv(), ee.BaseItem.fail;
          this.EndTable(), this.clearEnv();
          var Pe = this.factory.create("mml", this.createMml());
          if (this.getProperty("requireClose")) {
            if (De.isKind("close"))
              return [[Pe], !0];
            throw new Z.default("MissingCloseBrace", "Missing close brace");
          }
          return [[Pe, De], !0];
        }
        return Me.prototype.checkItem.call(this, De);
      }, Re.prototype.createMml = function() {
        var De = this.arraydef.scriptlevel;
        delete this.arraydef.scriptlevel;
        var Pe = this.create("node", "mtable", this.table, this.arraydef);
        return De && Pe.setProperty("scriptlevel", De), this.frame.length === 4 ? ne.default.setAttribute(Pe, "frame", this.dashed ? "dashed" : "solid") : this.frame.length && (this.arraydef.rowlines && (this.arraydef.rowlines = this.arraydef.rowlines.replace(/none( none)+$/, "none")), ne.default.setAttribute(Pe, "frame", ""), Pe = this.create("node", "menclose", [Pe], { notation: this.frame.join(" ") }), ((this.arraydef.columnlines || "none") !== "none" || (this.arraydef.rowlines || "none") !== "none") && ne.default.setAttribute(Pe, "data-padding", 0)), (this.getProperty("open") || this.getProperty("close")) && (Pe = te.default.fenced(this.factory.configuration, this.getProperty("open"), Pe, this.getProperty("close"))), Pe;
      }, Re.prototype.EndEntry = function() {
        var De = this.create("node", "mtd", this.nodes);
        this.hfill.length && (this.hfill[0] === 0 && ne.default.setAttribute(De, "columnalign", "right"), this.hfill[this.hfill.length - 1] === this.Size() && ne.default.setAttribute(De, "columnalign", ne.default.getAttribute(De, "columnalign") ? "center" : "left")), this.row.push(De), this.Clear(), this.hfill = [];
      }, Re.prototype.EndRow = function() {
        var De;
        this.getProperty("isNumbered") && this.row.length === 3 ? (this.row.unshift(this.row.pop()), De = this.create("node", "mlabeledtr", this.row)) : De = this.create("node", "mtr", this.row), this.table.push(De), this.row = [];
      }, Re.prototype.EndTable = function() {
        (this.Size() || this.row.length) && (this.EndEntry(), this.EndRow()), this.checkLines();
      }, Re.prototype.checkLines = function() {
        if (this.arraydef.rowlines) {
          var De = this.arraydef.rowlines.split(/ /);
          De.length === this.table.length ? (this.frame.push("bottom"), De.pop(), this.arraydef.rowlines = De.join(" ")) : De.length < this.table.length - 1 && (this.arraydef.rowlines += " none");
        }
        if (this.getProperty("rowspacing")) {
          for (var Pe = this.arraydef.rowspacing.split(/ /); Pe.length < this.table.length; )
            Pe.push(this.getProperty("rowspacing") + "em");
          this.arraydef.rowspacing = Pe.join(" ");
        }
      }, Re.prototype.addRowSpacing = function(De) {
        if (this.arraydef.rowspacing) {
          var Pe = this.arraydef.rowspacing.split(/ /);
          if (!this.getProperty("rowspacing")) {
            var $e = te.default.dimen2em(Pe[0]);
            this.setProperty("rowspacing", $e);
          }
          for (var qe = this.getProperty("rowspacing"); Pe.length < this.table.length; )
            Pe.push(te.default.Em(qe));
          Pe[this.table.length - 1] = te.default.Em(Math.max(0, qe + te.default.dimen2em(De))), this.arraydef.rowspacing = Pe.join(" ");
        }
      }, Re;
    }(ee.BaseItem);
    BaseItems.ArrayItem = Le;
    var Qe = function(Me) {
      O(Re, Me);
      function Re(De) {
        for (var Pe = [], $e = 1; $e < arguments.length; $e++)
          Pe[$e - 1] = arguments[$e];
        var qe = Me.call(this, De) || this;
        return qe.maxrow = 0, qe.factory.configuration.tags.start(Pe[0], Pe[2], Pe[1]), qe;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "eqnarray";
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.EndEntry = function() {
        this.row.length && te.default.fixInitialMO(this.factory.configuration, this.nodes);
        var De = this.create("node", "mtd", this.nodes);
        this.row.push(De), this.Clear();
      }, Re.prototype.EndRow = function() {
        this.row.length > this.maxrow && (this.maxrow = this.row.length);
        var De = "mtr", Pe = this.factory.configuration.tags.getTag();
        Pe && (this.row = [Pe].concat(this.row), De = "mlabeledtr"), this.factory.configuration.tags.clearTag();
        var $e = this.create("node", De, this.row);
        this.table.push($e), this.row = [];
      }, Re.prototype.EndTable = function() {
        Me.prototype.EndTable.call(this), this.factory.configuration.tags.end(), this.extendArray("columnalign", this.maxrow), this.extendArray("columnwidth", this.maxrow), this.extendArray("columnspacing", this.maxrow - 1);
      }, Re.prototype.extendArray = function(De, Pe) {
        if (this.arraydef[De]) {
          var $e = this.arraydef[De].split(/ /), qe = H([], _($e), !1);
          if (qe.length > 1) {
            for (; qe.length < Pe; )
              qe.push.apply(qe, H([], _($e), !1));
            this.arraydef[De] = qe.slice(0, Pe).join(" ");
          }
        }
      }, Re;
    }(Le);
    BaseItems.EqnArrayItem = Qe;
    var ke = function(Me) {
      O(Re, Me);
      function Re(De) {
        for (var Pe = [], $e = 1; $e < arguments.length; $e++)
          Pe[$e - 1] = arguments[$e];
        var qe = Me.call(this, De) || this;
        return qe.factory.configuration.tags.start("equation", !0, Pe[0]), qe;
      }
      return Object.defineProperty(Re.prototype, "kind", {
        get: function() {
          return "equation";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Re.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Re.prototype.checkItem = function(De) {
        if (De.isKind("end")) {
          var Pe = this.toMml(), $e = this.factory.configuration.tags.getTag();
          return this.factory.configuration.tags.end(), [[$e ? this.factory.configuration.tags.enTag(Pe, $e) : Pe, De], !0];
        }
        if (De.isKind("stop"))
          throw new Z.default("EnvMissingEnd", "Missing \\end{%1}", this.getName());
        return Me.prototype.checkItem.call(this, De);
      }, Re;
    }(ee.BaseItem);
    return BaseItems.EquationItem = ke, BaseItems;
  }
  var BaseMappings = {}, TexConstants = {}, hasRequiredTexConstants;
  function requireTexConstants() {
    return hasRequiredTexConstants || (hasRequiredTexConstants = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.TexConstant = void 0, function(_) {
        _.Variant = {
          NORMAL: "normal",
          BOLD: "bold",
          ITALIC: "italic",
          BOLDITALIC: "bold-italic",
          DOUBLESTRUCK: "double-struck",
          FRAKTUR: "fraktur",
          BOLDFRAKTUR: "bold-fraktur",
          SCRIPT: "script",
          BOLDSCRIPT: "bold-script",
          SANSSERIF: "sans-serif",
          BOLDSANSSERIF: "bold-sans-serif",
          SANSSERIFITALIC: "sans-serif-italic",
          SANSSERIFBOLDITALIC: "sans-serif-bold-italic",
          MONOSPACE: "monospace",
          INITIAL: "inital",
          TAILED: "tailed",
          LOOPED: "looped",
          STRETCHED: "stretched",
          CALLIGRAPHIC: "-tex-calligraphic",
          BOLDCALLIGRAPHIC: "-tex-bold-calligraphic",
          OLDSTYLE: "-tex-oldstyle",
          BOLDOLDSTYLE: "-tex-bold-oldstyle",
          MATHITALIC: "-tex-mathit"
        }, _.Form = {
          PREFIX: "prefix",
          INFIX: "infix",
          POSTFIX: "postfix"
        }, _.LineBreak = {
          AUTO: "auto",
          NEWLINE: "newline",
          NOBREAK: "nobreak",
          GOODBREAK: "goodbreak",
          BADBREAK: "badbreak"
        }, _.LineBreakStyle = {
          BEFORE: "before",
          AFTER: "after",
          DUPLICATE: "duplicate",
          INFIXLINBREAKSTYLE: "infixlinebreakstyle"
        }, _.IndentAlign = {
          LEFT: "left",
          CENTER: "center",
          RIGHT: "right",
          AUTO: "auto",
          ID: "id",
          INDENTALIGN: "indentalign"
        }, _.IndentShift = {
          INDENTSHIFT: "indentshift"
        }, _.LineThickness = {
          THIN: "thin",
          MEDIUM: "medium",
          THICK: "thick"
        }, _.Notation = {
          LONGDIV: "longdiv",
          ACTUARIAL: "actuarial",
          PHASORANGLE: "phasorangle",
          RADICAL: "radical",
          BOX: "box",
          ROUNDEDBOX: "roundedbox",
          CIRCLE: "circle",
          LEFT: "left",
          RIGHT: "right",
          TOP: "top",
          BOTTOM: "bottom",
          UPDIAGONALSTRIKE: "updiagonalstrike",
          DOWNDIAGONALSTRIKE: "downdiagonalstrike",
          VERTICALSTRIKE: "verticalstrike",
          HORIZONTALSTRIKE: "horizontalstrike",
          NORTHEASTARROW: "northeastarrow",
          MADRUWB: "madruwb",
          UPDIAGONALARROW: "updiagonalarrow"
        }, _.Align = {
          TOP: "top",
          BOTTOM: "bottom",
          CENTER: "center",
          BASELINE: "baseline",
          AXIS: "axis",
          LEFT: "left",
          RIGHT: "right"
        }, _.Lines = {
          NONE: "none",
          SOLID: "solid",
          DASHED: "dashed"
        }, _.Side = {
          LEFT: "left",
          RIGHT: "right",
          LEFTOVERLAP: "leftoverlap",
          RIGHTOVERLAP: "rightoverlap"
        }, _.Width = {
          AUTO: "auto",
          FIT: "fit"
        }, _.Actiontype = {
          TOGGLE: "toggle",
          STATUSLINE: "statusline",
          TOOLTIP: "tooltip",
          INPUT: "input"
        }, _.Overflow = {
          LINBREAK: "linebreak",
          SCROLL: "scroll",
          ELIDE: "elide",
          TRUNCATE: "truncate",
          SCALE: "scale"
        }, _.Unit = {
          EM: "em",
          EX: "ex",
          PX: "px",
          IN: "in",
          CM: "cm",
          MM: "mm",
          PT: "pt",
          PC: "pc"
        };
      }(O.TexConstant || (O.TexConstant = {}));
    }(TexConstants)), TexConstants;
  }
  var BaseMethods = {}, lengths = {}, hasRequiredLengths;
  function requireLengths() {
    return hasRequiredLengths || (hasRequiredLengths = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.px = O.emRounded = O.em = O.percent = O.length2em = O.MATHSPACE = O.RELUNITS = O.UNITS = O.BIGDIMEN = void 0, O.BIGDIMEN = 1e6, O.UNITS = {
        px: 1,
        in: 96,
        cm: 96 / 2.54,
        mm: 96 / 25.4
      }, O.RELUNITS = {
        em: 1,
        ex: 0.431,
        pt: 1 / 10,
        pc: 12 / 10,
        mu: 1 / 18
      }, O.MATHSPACE = {
        veryverythinmathspace: 1 / 18,
        verythinmathspace: 2 / 18,
        thinmathspace: 3 / 18,
        mediummathspace: 4 / 18,
        thickmathspace: 5 / 18,
        verythickmathspace: 6 / 18,
        veryverythickmathspace: 7 / 18,
        negativeveryverythinmathspace: -1 / 18,
        negativeverythinmathspace: -2 / 18,
        negativethinmathspace: -3 / 18,
        negativemediummathspace: -4 / 18,
        negativethickmathspace: -5 / 18,
        negativeverythickmathspace: -6 / 18,
        negativeveryverythickmathspace: -7 / 18,
        thin: 0.04,
        medium: 0.06,
        thick: 0.1,
        normal: 1,
        big: 2,
        small: 1 / Math.sqrt(2),
        infinity: O.BIGDIMEN
      };
      function _(J, Z, te, ne) {
        if (Z === void 0 && (Z = 0), te === void 0 && (te = 1), ne === void 0 && (ne = 16), typeof J != "string" && (J = String(J)), J === "" || J == null)
          return Z;
        if (O.MATHSPACE[J])
          return O.MATHSPACE[J];
        var ee = J.match(/^\s*([-+]?(?:\.\d+|\d+(?:\.\d*)?))?(pt|em|ex|mu|px|pc|in|mm|cm|%)?/);
        if (!ee)
          return Z;
        var ae = parseFloat(ee[1] || "1"), oe = ee[2];
        return O.UNITS.hasOwnProperty(oe) ? ae * O.UNITS[oe] / ne / te : O.RELUNITS.hasOwnProperty(oe) ? ae * O.RELUNITS[oe] : oe === "%" ? ae / 100 * Z : ae * Z;
      }
      O.length2em = _;
      function H(J) {
        return (100 * J).toFixed(1).replace(/\.?0+$/, "") + "%";
      }
      O.percent = H;
      function V(J) {
        return Math.abs(J) < 1e-3 ? "0" : J.toFixed(3).replace(/\.?0+$/, "") + "em";
      }
      O.em = V;
      function q(J, Z) {
        return Z === void 0 && (Z = 16), J = (Math.round(J * Z) + 0.05) / Z, Math.abs(J) < 1e-3 ? "0em" : J.toFixed(3).replace(/\.?0+$/, "") + "em";
      }
      O.emRounded = q;
      function G(J, Z, te) {
        return Z === void 0 && (Z = -O.BIGDIMEN), te === void 0 && (te = 16), J *= te, Z && J < Z && (J = Z), Math.abs(J) < 0.1 ? "0" : J.toFixed(1).replace(/\.0$/, "") + "px";
      }
      O.px = G;
    }(lengths)), lengths;
  }
  var hasRequiredBaseMethods;
  function requireBaseMethods() {
    if (hasRequiredBaseMethods) return BaseMethods;
    hasRequiredBaseMethods = 1;
    var O = BaseMethods.__assign || function() {
      return O = Object.assign || function(pe) {
        for (var ye, ge = 1, ve = arguments.length; ge < ve; ge++) {
          ye = arguments[ge];
          for (var be in ye) Object.prototype.hasOwnProperty.call(ye, be) && (pe[be] = ye[be]);
        }
        return pe;
      }, O.apply(this, arguments);
    }, _ = BaseMethods.__createBinding || (Object.create ? function(pe, ye, ge, ve) {
      ve === void 0 && (ve = ge);
      var be = Object.getOwnPropertyDescriptor(ye, ge);
      (!be || ("get" in be ? !ye.__esModule : be.writable || be.configurable)) && (be = { enumerable: !0, get: function() {
        return ye[ge];
      } }), Object.defineProperty(pe, ve, be);
    } : function(pe, ye, ge, ve) {
      ve === void 0 && (ve = ge), pe[ve] = ye[ge];
    }), H = BaseMethods.__setModuleDefault || (Object.create ? function(pe, ye) {
      Object.defineProperty(pe, "default", { enumerable: !0, value: ye });
    } : function(pe, ye) {
      pe.default = ye;
    }), V = BaseMethods.__importStar || function(pe) {
      if (pe && pe.__esModule) return pe;
      var ye = {};
      if (pe != null) for (var ge in pe) ge !== "default" && Object.prototype.hasOwnProperty.call(pe, ge) && _(ye, pe, ge);
      return H(ye, pe), ye;
    }, q = BaseMethods.__read || function(pe, ye) {
      var ge = typeof Symbol == "function" && pe[Symbol.iterator];
      if (!ge) return pe;
      var ve = ge.call(pe), be, Ce = [], Se;
      try {
        for (; (ye === void 0 || ye-- > 0) && !(be = ve.next()).done; ) Ce.push(be.value);
      } catch (we) {
        Se = { error: we };
      } finally {
        try {
          be && !be.done && (ge = ve.return) && ge.call(ve);
        } finally {
          if (Se) throw Se.error;
        }
      }
      return Ce;
    }, G = BaseMethods.__importDefault || function(pe) {
      return pe && pe.__esModule ? pe : { default: pe };
    };
    Object.defineProperty(BaseMethods, "__esModule", { value: !0 });
    var J = V(requireBaseItems()), Z = G(requireNodeUtil()), te = G(requireTexError()), ne = G(requireTexParser()), ee = requireTexConstants(), ae = G(requireParseUtil()), oe = requireMmlNode(), se = requireTags(), ce = requireLengths(), le = requireEntities(), ue = requireOptions(), de = {}, fe = 1.2 / 0.85, he = {
      fontfamily: 1,
      fontsize: 1,
      fontweight: 1,
      fontstyle: 1,
      color: 1,
      background: 1,
      id: 1,
      class: 1,
      href: 1,
      style: 1
    };
    de.Open = function(pe, ye) {
      pe.Push(pe.itemFactory.create("open"));
    }, de.Close = function(pe, ye) {
      pe.Push(pe.itemFactory.create("close"));
    }, de.Tilde = function(pe, ye) {
      pe.Push(pe.create("token", "mtext", {}, le.entities.nbsp));
    }, de.Space = function(pe, ye) {
    }, de.Superscript = function(pe, ye) {
      var ge;
      pe.GetNext().match(/\d/) && (pe.string = pe.string.substr(0, pe.i + 1) + " " + pe.string.substr(pe.i + 1));
      var ve, be, Ce = pe.stack.Top();
      Ce.isKind("prime") ? (ge = q(Ce.Peek(2), 2), be = ge[0], ve = ge[1], pe.stack.Pop()) : (be = pe.stack.Prev(), be || (be = pe.create("token", "mi", {}, "")));
      var Se = Z.default.getProperty(be, "movesupsub"), we = Z.default.isType(be, "msubsup") ? be.sup : be.over;
      if (Z.default.isType(be, "msubsup") && !Z.default.isType(be, "msup") && Z.default.getChildAt(be, be.sup) || Z.default.isType(be, "munderover") && !Z.default.isType(be, "mover") && Z.default.getChildAt(be, be.over) && !Z.default.getProperty(be, "subsupOK"))
        throw new te.default("DoubleExponent", "Double exponent: use braces to clarify");
      (!Z.default.isType(be, "msubsup") || Z.default.isType(be, "msup")) && (Se ? ((!Z.default.isType(be, "munderover") || Z.default.isType(be, "mover") || Z.default.getChildAt(be, be.over)) && (be = pe.create("node", "munderover", [be], { movesupsub: !0 })), we = be.over) : (be = pe.create("node", "msubsup", [be]), we = be.sup)), pe.Push(pe.itemFactory.create("subsup", be).setProperties({
        position: we,
        primes: ve,
        movesupsub: Se
      }));
    }, de.Subscript = function(pe, ye) {
      var ge;
      pe.GetNext().match(/\d/) && (pe.string = pe.string.substr(0, pe.i + 1) + " " + pe.string.substr(pe.i + 1));
      var ve, be, Ce = pe.stack.Top();
      Ce.isKind("prime") ? (ge = q(Ce.Peek(2), 2), be = ge[0], ve = ge[1], pe.stack.Pop()) : (be = pe.stack.Prev(), be || (be = pe.create("token", "mi", {}, "")));
      var Se = Z.default.getProperty(be, "movesupsub"), we = Z.default.isType(be, "msubsup") ? be.sub : be.under;
      if (Z.default.isType(be, "msubsup") && !Z.default.isType(be, "msup") && Z.default.getChildAt(be, be.sub) || Z.default.isType(be, "munderover") && !Z.default.isType(be, "mover") && Z.default.getChildAt(be, be.under) && !Z.default.getProperty(be, "subsupOK"))
        throw new te.default("DoubleSubscripts", "Double subscripts: use braces to clarify");
      (!Z.default.isType(be, "msubsup") || Z.default.isType(be, "msup")) && (Se ? ((!Z.default.isType(be, "munderover") || Z.default.isType(be, "mover") || Z.default.getChildAt(be, be.under)) && (be = pe.create("node", "munderover", [be], { movesupsub: !0 })), we = be.under) : (be = pe.create("node", "msubsup", [be]), we = be.sub)), pe.Push(pe.itemFactory.create("subsup", be).setProperties({
        position: we,
        primes: ve,
        movesupsub: Se
      }));
    }, de.Prime = function(pe, ye) {
      var ge = pe.stack.Prev();
      if (ge || (ge = pe.create("node", "mi")), Z.default.isType(ge, "msubsup") && !Z.default.isType(ge, "msup") && Z.default.getChildAt(ge, ge.sup))
        throw new te.default("DoubleExponentPrime", "Prime causes double exponent: use braces to clarify");
      var ve = "";
      pe.i--;
      do
        ve += le.entities.prime, pe.i++, ye = pe.GetNext();
      while (ye === "'" || ye === le.entities.rsquo);
      ve = ["", "′", "″", "‴", "⁗"][ve.length] || ve;
      var be = pe.create("token", "mo", { variantForm: !0 }, ve);
      pe.Push(pe.itemFactory.create("prime", ge, be));
    }, de.Comment = function(pe, ye) {
      for (; pe.i < pe.string.length && pe.string.charAt(pe.i) !== `
`; )
        pe.i++;
    }, de.Hash = function(pe, ye) {
      throw new te.default("CantUseHash1", "You can't use 'macro parameter character #' in math mode");
    }, de.MathFont = function(pe, ye, ge) {
      var ve = pe.GetArgument(ye), be = new ne.default(ve, O(O({}, pe.stack.env), { font: ge, multiLetterIdentifiers: /^[a-zA-Z]+/, noAutoOP: !0 }), pe.configuration).mml();
      pe.Push(pe.create("node", "TeXAtom", [be]));
    }, de.SetFont = function(pe, ye, ge) {
      pe.stack.env.font = ge;
    }, de.SetStyle = function(pe, ye, ge, ve, be) {
      pe.stack.env.style = ge, pe.stack.env.level = be, pe.Push(pe.itemFactory.create("style").setProperty("styles", { displaystyle: ve, scriptlevel: be }));
    }, de.SetSize = function(pe, ye, ge) {
      pe.stack.env.size = ge, pe.Push(pe.itemFactory.create("style").setProperty("styles", { mathsize: (0, ce.em)(ge) }));
    }, de.Spacer = function(pe, ye, ge) {
      var ve = pe.create("node", "mspace", [], { width: (0, ce.em)(ge) }), be = pe.create("node", "mstyle", [ve], { scriptlevel: 0 });
      pe.Push(be);
    }, de.LeftRight = function(pe, ye) {
      var ge = ye.substr(1);
      pe.Push(pe.itemFactory.create(ge, pe.GetDelimiter(ye), pe.stack.env.color));
    }, de.NamedFn = function(pe, ye, ge) {
      ge || (ge = ye.substr(1));
      var ve = pe.create("token", "mi", { texClass: oe.TEXCLASS.OP }, ge);
      pe.Push(pe.itemFactory.create("fn", ve));
    }, de.NamedOp = function(pe, ye, ge) {
      ge || (ge = ye.substr(1)), ge = ge.replace(/&thinsp;/, " ");
      var ve = pe.create("token", "mo", {
        movablelimits: !0,
        movesupsub: !0,
        form: ee.TexConstant.Form.PREFIX,
        texClass: oe.TEXCLASS.OP
      }, ge);
      pe.Push(ve);
    }, de.Limits = function(pe, ye, ge) {
      var ve = pe.stack.Prev(!0);
      if (!ve || Z.default.getTexClass(Z.default.getCoreMO(ve)) !== oe.TEXCLASS.OP && Z.default.getProperty(ve, "movesupsub") == null)
        throw new te.default("MisplacedLimits", "%1 is allowed only on operators", pe.currentCS);
      var be = pe.stack.Top(), Ce;
      Z.default.isType(ve, "munderover") && !ge ? (Ce = pe.create("node", "msubsup"), Z.default.copyChildren(ve, Ce), ve = be.Last = Ce) : Z.default.isType(ve, "msubsup") && ge && (Ce = pe.create("node", "munderover"), Z.default.copyChildren(ve, Ce), ve = be.Last = Ce), Z.default.setProperty(ve, "movesupsub", !!ge), Z.default.setProperties(Z.default.getCoreMO(ve), { movablelimits: !1 }), (Z.default.getAttribute(ve, "movablelimits") || Z.default.getProperty(ve, "movablelimits")) && Z.default.setProperties(ve, { movablelimits: !1 });
    }, de.Over = function(pe, ye, ge, ve) {
      var be = pe.itemFactory.create("over").setProperty("name", pe.currentCS);
      ge || ve ? (be.setProperty("open", ge), be.setProperty("close", ve)) : ye.match(/withdelims$/) && (be.setProperty("open", pe.GetDelimiter(ye)), be.setProperty("close", pe.GetDelimiter(ye))), ye.match(/^\\above/) ? be.setProperty("thickness", pe.GetDimen(ye)) : (ye.match(/^\\atop/) || ge || ve) && be.setProperty("thickness", 0), pe.Push(be);
    }, de.Frac = function(pe, ye) {
      var ge = pe.ParseArg(ye), ve = pe.ParseArg(ye), be = pe.create("node", "mfrac", [ge, ve]);
      pe.Push(be);
    }, de.Sqrt = function(pe, ye) {
      var ge = pe.GetBrackets(ye), ve = pe.GetArgument(ye);
      ve === "\\frac" && (ve += "{" + pe.GetArgument(ve) + "}{" + pe.GetArgument(ve) + "}");
      var be = new ne.default(ve, pe.stack.env, pe.configuration).mml();
      ge ? be = pe.create("node", "mroot", [be, me(pe, ge)]) : be = pe.create("node", "msqrt", [be]), pe.Push(be);
    };
    function me(pe, ye) {
      var ge = pe.stack.env, ve = ge.inRoot;
      ge.inRoot = !0;
      var be = new ne.default(ye, ge, pe.configuration), Ce = be.mml(), Se = be.stack.global;
      if (Se.leftRoot || Se.upRoot) {
        var we = {};
        Se.leftRoot && (we.width = Se.leftRoot), Se.upRoot && (we.voffset = Se.upRoot, we.height = Se.upRoot), Ce = pe.create("node", "mpadded", [Ce], we);
      }
      return ge.inRoot = ve, Ce;
    }
    return de.Root = function(pe, ye) {
      var ge = pe.GetUpTo(ye, "\\of"), ve = pe.ParseArg(ye), be = pe.create("node", "mroot", [ve, me(pe, ge)]);
      pe.Push(be);
    }, de.MoveRoot = function(pe, ye, ge) {
      if (!pe.stack.env.inRoot)
        throw new te.default("MisplacedMoveRoot", "%1 can appear only within a root", pe.currentCS);
      if (pe.stack.global[ge])
        throw new te.default("MultipleMoveRoot", "Multiple use of %1", pe.currentCS);
      var ve = pe.GetArgument(ye);
      if (!ve.match(/-?[0-9]+/))
        throw new te.default("IntegerArg", "The argument to %1 must be an integer", pe.currentCS);
      ve = parseInt(ve, 10) / 15 + "em", ve.substr(0, 1) !== "-" && (ve = "+" + ve), pe.stack.global[ge] = ve;
    }, de.Accent = function(pe, ye, ge, ve) {
      var be = pe.ParseArg(ye), Ce = O(O({}, ae.default.getFontDef(pe)), { accent: !0, mathaccent: !0 }), Se = Z.default.createEntity(ge), we = pe.create("token", "mo", Ce, Se), Ee = we;
      Z.default.setAttribute(Ee, "stretchy", !!ve);
      var xe = Z.default.isEmbellished(be) ? Z.default.getCoreMO(be) : be;
      (Z.default.isType(xe, "mo") || Z.default.getProperty(xe, "movablelimits")) && Z.default.setProperties(xe, { movablelimits: !1 });
      var Le = pe.create("node", "munderover");
      Z.default.setChild(Le, 0, be), Z.default.setChild(Le, 1, null), Z.default.setChild(Le, 2, Ee);
      var Qe = pe.create("node", "TeXAtom", [Le]);
      pe.Push(Qe);
    }, de.UnderOver = function(pe, ye, ge, ve) {
      var be = Z.default.createEntity(ge), Ce = pe.create("token", "mo", { stretchy: !0, accent: !0 }, be), Se = ye.charAt(1) === "o" ? "over" : "under", we = pe.ParseArg(ye);
      pe.Push(ae.default.underOver(pe, we, Ce, Se, ve));
    }, de.Overset = function(pe, ye) {
      var ge = pe.ParseArg(ye), ve = pe.ParseArg(ye);
      ae.default.checkMovableLimits(ve), ge.isKind("mo") && Z.default.setAttribute(ge, "accent", !1);
      var be = pe.create("node", "mover", [ve, ge]);
      pe.Push(be);
    }, de.Underset = function(pe, ye) {
      var ge = pe.ParseArg(ye), ve = pe.ParseArg(ye);
      ae.default.checkMovableLimits(ve), ge.isKind("mo") && Z.default.setAttribute(ge, "accent", !1);
      var be = pe.create("node", "munder", [ve, ge], { accentunder: !1 });
      pe.Push(be);
    }, de.Overunderset = function(pe, ye) {
      var ge = pe.ParseArg(ye), ve = pe.ParseArg(ye), be = pe.ParseArg(ye);
      ae.default.checkMovableLimits(be), ge.isKind("mo") && Z.default.setAttribute(ge, "accent", !1), ve.isKind("mo") && Z.default.setAttribute(ve, "accent", !1);
      var Ce = pe.create("node", "munderover", [be, ve, ge], { accent: !1, accentunder: !1 });
      pe.Push(Ce);
    }, de.TeXAtom = function(pe, ye, ge) {
      var ve = { texClass: ge }, be, Ce, Se;
      if (ge === oe.TEXCLASS.OP) {
        ve.movesupsub = ve.movablelimits = !0;
        var we = pe.GetArgument(ye), Ee = we.match(/^\s*\\rm\s+([a-zA-Z0-9 ]+)$/);
        Ee ? (ve.mathvariant = ee.TexConstant.Variant.NORMAL, Ce = pe.create("token", "mi", ve, Ee[1])) : (Se = new ne.default(we, pe.stack.env, pe.configuration).mml(), Ce = pe.create("node", "TeXAtom", [Se], ve)), be = pe.itemFactory.create("fn", Ce);
      } else
        Se = pe.ParseArg(ye), be = pe.create("node", "TeXAtom", [Se], ve);
      pe.Push(be);
    }, de.MmlToken = function(pe, ye) {
      var ge = pe.GetArgument(ye), ve = pe.GetBrackets(ye, "").replace(/^\s+/, ""), be = pe.GetArgument(ye), Ce = {}, Se = [], we;
      try {
        we = pe.create("node", ge);
      } catch {
        we = null;
      }
      if (!we || !we.isToken)
        throw new te.default("NotMathMLToken", "%1 is not a token element", ge);
      for (; ve !== ""; ) {
        var Ee = ve.match(/^([a-z]+)\s*=\s*('[^']*'|"[^"]*"|[^ ,]*)\s*,?\s*/i);
        if (!Ee)
          throw new te.default("InvalidMathMLAttr", "Invalid MathML attribute: %1", ve);
        if (!we.attributes.hasDefault(Ee[1]) && !he[Ee[1]])
          throw new te.default("UnknownAttrForElement", "%1 is not a recognized attribute for %2", Ee[1], ge);
        var xe = ae.default.MmlFilterAttribute(pe, Ee[1], Ee[2].replace(/^(['"])(.*)\1$/, "$2"));
        xe && (xe.toLowerCase() === "true" ? xe = !0 : xe.toLowerCase() === "false" && (xe = !1), Ce[Ee[1]] = xe, Se.push(Ee[1])), ve = ve.substr(Ee[0].length);
      }
      Se.length && (Ce["mjx-keep-attrs"] = Se.join(" "));
      var Le = pe.create("text", be);
      we.appendChild(Le), Z.default.setProperties(we, Ce), pe.Push(we);
    }, de.Strut = function(pe, ye) {
      var ge = pe.create("node", "mrow"), ve = pe.create("node", "mpadded", [ge], { height: "8.6pt", depth: "3pt", width: 0 });
      pe.Push(ve);
    }, de.Phantom = function(pe, ye, ge, ve) {
      var be = pe.create("node", "mphantom", [pe.ParseArg(ye)]);
      (ge || ve) && (be = pe.create("node", "mpadded", [be]), ve && (Z.default.setAttribute(be, "height", 0), Z.default.setAttribute(be, "depth", 0)), ge && Z.default.setAttribute(be, "width", 0));
      var Ce = pe.create("node", "TeXAtom", [be]);
      pe.Push(Ce);
    }, de.Smash = function(pe, ye) {
      var ge = ae.default.trimSpaces(pe.GetBrackets(ye, "")), ve = pe.create("node", "mpadded", [pe.ParseArg(ye)]);
      switch (ge) {
        case "b":
          Z.default.setAttribute(ve, "depth", 0);
          break;
        case "t":
          Z.default.setAttribute(ve, "height", 0);
          break;
        default:
          Z.default.setAttribute(ve, "height", 0), Z.default.setAttribute(ve, "depth", 0);
      }
      var be = pe.create("node", "TeXAtom", [ve]);
      pe.Push(be);
    }, de.Lap = function(pe, ye) {
      var ge = pe.create("node", "mpadded", [pe.ParseArg(ye)], { width: 0 });
      ye === "\\llap" && Z.default.setAttribute(ge, "lspace", "-1width");
      var ve = pe.create("node", "TeXAtom", [ge]);
      pe.Push(ve);
    }, de.RaiseLower = function(pe, ye) {
      var ge = pe.GetDimen(ye), ve = pe.itemFactory.create("position").setProperties({ name: pe.currentCS, move: "vertical" });
      ge.charAt(0) === "-" && (ge = ge.slice(1), ye = ye.substr(1) === "raise" ? "\\lower" : "\\raise"), ye === "\\lower" ? (ve.setProperty("dh", "-" + ge), ve.setProperty("dd", "+" + ge)) : (ve.setProperty("dh", "+" + ge), ve.setProperty("dd", "-" + ge)), pe.Push(ve);
    }, de.MoveLeftRight = function(pe, ye) {
      var ge = pe.GetDimen(ye), ve = ge.charAt(0) === "-" ? ge.slice(1) : "-" + ge;
      if (ye === "\\moveleft") {
        var be = ge;
        ge = ve, ve = be;
      }
      pe.Push(pe.itemFactory.create("position").setProperties({
        name: pe.currentCS,
        move: "horizontal",
        left: pe.create("node", "mspace", [], { width: ge }),
        right: pe.create("node", "mspace", [], { width: ve })
      }));
    }, de.Hskip = function(pe, ye) {
      var ge = pe.create("node", "mspace", [], { width: pe.GetDimen(ye) });
      pe.Push(ge);
    }, de.Nonscript = function(pe, ye) {
      pe.Push(pe.itemFactory.create("nonscript"));
    }, de.Rule = function(pe, ye, ge) {
      var ve = pe.GetDimen(ye), be = pe.GetDimen(ye), Ce = pe.GetDimen(ye), Se = { width: ve, height: be, depth: Ce };
      ge !== "blank" && (Se.mathbackground = pe.stack.env.color || "black");
      var we = pe.create("node", "mspace", [], Se);
      pe.Push(we);
    }, de.rule = function(pe, ye) {
      var ge = pe.GetBrackets(ye), ve = pe.GetDimen(ye), be = pe.GetDimen(ye), Ce = pe.create("node", "mspace", [], {
        width: ve,
        height: be,
        mathbackground: pe.stack.env.color || "black"
      });
      ge && (Ce = pe.create("node", "mpadded", [Ce], { voffset: ge }), ge.match(/^\-/) ? (Z.default.setAttribute(Ce, "height", ge), Z.default.setAttribute(Ce, "depth", "+" + ge.substr(1))) : Z.default.setAttribute(Ce, "height", "+" + ge)), pe.Push(Ce);
    }, de.MakeBig = function(pe, ye, ge, ve) {
      ve *= fe;
      var be = String(ve).replace(/(\.\d\d\d).+/, "$1") + "em", Ce = pe.GetDelimiter(ye, !0), Se = pe.create("token", "mo", {
        minsize: be,
        maxsize: be,
        fence: !0,
        stretchy: !0,
        symmetric: !0
      }, Ce), we = pe.create("node", "TeXAtom", [Se], { texClass: ge });
      pe.Push(we);
    }, de.BuildRel = function(pe, ye) {
      var ge = pe.ParseUpTo(ye, "\\over"), ve = pe.ParseArg(ye), be = pe.create("node", "munderover");
      Z.default.setChild(be, 0, ve), Z.default.setChild(be, 1, null), Z.default.setChild(be, 2, ge);
      var Ce = pe.create("node", "TeXAtom", [be], { texClass: oe.TEXCLASS.REL });
      pe.Push(Ce);
    }, de.HBox = function(pe, ye, ge, ve) {
      pe.PushAll(ae.default.internalMath(pe, pe.GetArgument(ye), ge, ve));
    }, de.FBox = function(pe, ye) {
      var ge = ae.default.internalMath(pe, pe.GetArgument(ye)), ve = pe.create("node", "menclose", ge, { notation: "box" });
      pe.Push(ve);
    }, de.FrameBox = function(pe, ye) {
      var ge = pe.GetBrackets(ye), ve = pe.GetBrackets(ye) || "c", be = ae.default.internalMath(pe, pe.GetArgument(ye));
      ge && (be = [pe.create("node", "mpadded", be, {
        width: ge,
        "data-align": (0, ue.lookup)(ve, { l: "left", r: "right" }, "center")
      })]);
      var Ce = pe.create("node", "TeXAtom", [pe.create("node", "menclose", be, { notation: "box" })], { texClass: oe.TEXCLASS.ORD });
      pe.Push(Ce);
    }, de.Not = function(pe, ye) {
      pe.Push(pe.itemFactory.create("not"));
    }, de.Dots = function(pe, ye) {
      var ge = Z.default.createEntity("2026"), ve = Z.default.createEntity("22EF"), be = pe.create("token", "mo", { stretchy: !1 }, ge), Ce = pe.create("token", "mo", { stretchy: !1 }, ve);
      pe.Push(pe.itemFactory.create("dots").setProperties({
        ldots: be,
        cdots: Ce
      }));
    }, de.Matrix = function(pe, ye, ge, ve, be, Ce, Se, we, Ee, xe) {
      var Le = pe.GetNext();
      if (Le === "")
        throw new te.default("MissingArgFor", "Missing argument for %1", pe.currentCS);
      Le === "{" ? pe.i++ : (pe.string = Le + "}" + pe.string.slice(pe.i + 1), pe.i = 0);
      var Qe = pe.itemFactory.create("array").setProperty("requireClose", !0);
      Qe.arraydef = {
        rowspacing: Se || "4pt",
        columnspacing: Ce || "1em"
      }, Ee && Qe.setProperty("isCases", !0), xe && (Qe.setProperty("isNumbered", !0), Qe.arraydef.side = xe), (ge || ve) && (Qe.setProperty("open", ge), Qe.setProperty("close", ve)), we === "D" && (Qe.arraydef.displaystyle = !0), be != null && (Qe.arraydef.columnalign = be), pe.Push(Qe);
    }, de.Entry = function(pe, ye) {
      pe.Push(pe.itemFactory.create("cell").setProperties({ isEntry: !0, name: ye }));
      var ge = pe.stack.Top(), ve = ge.getProperty("casesEnv"), be = ge.getProperty("isCases");
      if (!(!be && !ve)) {
        for (var Ce = pe.string, Se = 0, we = -1, Ee = pe.i, xe = Ce.length, Le = ve ? new RegExp("^\\\\end\\s*\\{".concat(ve.replace(/\*/, "\\*"), "\\}")) : null; Ee < xe; ) {
          var Qe = Ce.charAt(Ee);
          if (Qe === "{")
            Se++, Ee++;
          else if (Qe === "}")
            Se === 0 ? xe = 0 : (Se--, Se === 0 && we < 0 && (we = Ee - pe.i), Ee++);
          else {
            if (Qe === "&" && Se === 0)
              throw new te.default("ExtraAlignTab", "Extra alignment tab in \\cases text");
            if (Qe === "\\") {
              var ke = Ce.substr(Ee);
              ke.match(/^((\\cr)[^a-zA-Z]|\\\\)/) || Le && ke.match(Le) ? xe = 0 : Ee += 2;
            } else
              Ee++;
          }
        }
        var Me = Ce.substr(pe.i, Ee - pe.i);
        if (!Me.match(/^\s*\\text[^a-zA-Z]/) || we !== Me.replace(/\s+$/, "").length - 1) {
          var Re = ae.default.internalMath(pe, ae.default.trimSpaces(Me), 0);
          pe.PushAll(Re), pe.i = Ee;
        }
      }
    }, de.Cr = function(pe, ye) {
      pe.Push(pe.itemFactory.create("cell").setProperties({ isCR: !0, name: ye }));
    }, de.CrLaTeX = function(pe, ye, ge) {
      ge === void 0 && (ge = !1);
      var ve;
      if (!ge && (pe.string.charAt(pe.i) === "*" && pe.i++, pe.string.charAt(pe.i) === "[")) {
        var be = pe.GetBrackets(ye, ""), Ce = q(ae.default.matchDimen(be), 2), Se = Ce[0], we = Ce[1];
        if (be && !Se)
          throw new te.default("BracketMustBeDimension", "Bracket argument to %1 must be a dimension", pe.currentCS);
        ve = Se + we;
      }
      pe.Push(pe.itemFactory.create("cell").setProperties({ isCR: !0, name: ye, linebreak: !0 }));
      var Ee = pe.stack.Top(), xe;
      Ee instanceof J.ArrayItem ? ve && Ee.addRowSpacing(ve) : (ve && (xe = pe.create("node", "mspace", [], { depth: ve }), pe.Push(xe)), xe = pe.create("node", "mspace", [], { linebreak: ee.TexConstant.LineBreak.NEWLINE }), pe.Push(xe));
    }, de.HLine = function(pe, ye, ge) {
      ge == null && (ge = "solid");
      var ve = pe.stack.Top();
      if (!(ve instanceof J.ArrayItem) || ve.Size())
        throw new te.default("Misplaced", "Misplaced %1", pe.currentCS);
      if (!ve.table.length)
        ve.frame.push("top");
      else {
        for (var be = ve.arraydef.rowlines ? ve.arraydef.rowlines.split(/ /) : []; be.length < ve.table.length; )
          be.push("none");
        be[ve.table.length - 1] = ge, ve.arraydef.rowlines = be.join(" ");
      }
    }, de.HFill = function(pe, ye) {
      var ge = pe.stack.Top();
      if (ge instanceof J.ArrayItem)
        ge.hfill.push(ge.Size());
      else
        throw new te.default("UnsupportedHFill", "Unsupported use of %1", pe.currentCS);
    }, de.BeginEnd = function(pe, ye) {
      var ge = pe.GetArgument(ye);
      if (ge.match(/\\/i))
        throw new te.default("InvalidEnv", "Invalid environment name '%1'", ge);
      var ve = pe.configuration.handlers.get("environment").lookup(ge);
      if (ve && ye === "\\end") {
        if (!ve.args[0]) {
          var be = pe.itemFactory.create("end").setProperty("name", ge);
          pe.Push(be);
          return;
        }
        pe.stack.env.closing = ge;
      }
      ae.default.checkMaxMacros(pe, !1), pe.parse("environment", [pe, ge]);
    }, de.Array = function(pe, ye, ge, ve, be, Ce, Se, we, Ee) {
      be || (be = pe.GetArgument("\\begin{" + ye.getName() + "}"));
      var xe = ("c" + be).replace(/[^clr|:]/g, "").replace(/[^|:]([|:])+/g, "$1");
      be = be.replace(/[^clr]/g, "").split("").join(" "), be = be.replace(/l/g, "left").replace(/r/g, "right").replace(/c/g, "center");
      var Le = pe.itemFactory.create("array");
      return Le.arraydef = {
        columnalign: be,
        columnspacing: Ce || "1em",
        rowspacing: Se || "4pt"
      }, xe.match(/[|:]/) && (xe.charAt(0).match(/[|:]/) && (Le.frame.push("left"), Le.dashed = xe.charAt(0) === ":"), xe.charAt(xe.length - 1).match(/[|:]/) && Le.frame.push("right"), xe = xe.substr(1, xe.length - 2), Le.arraydef.columnlines = xe.split("").join(" ").replace(/[^|: ]/g, "none").replace(/\|/g, "solid").replace(/:/g, "dashed")), ge && Le.setProperty("open", pe.convertDelimiter(ge)), ve && Le.setProperty("close", pe.convertDelimiter(ve)), (we || "").charAt(1) === "'" && (Le.arraydef["data-cramped"] = !0, we = we.charAt(0)), we === "D" ? Le.arraydef.displaystyle = !0 : we && (Le.arraydef.displaystyle = !1), we === "S" && (Le.arraydef.scriptlevel = 1), Ee && (Le.arraydef.useHeight = !1), pe.Push(ye), Le;
    }, de.AlignedArray = function(pe, ye) {
      var ge = pe.GetBrackets("\\begin{" + ye.getName() + "}"), ve = de.Array(pe, ye);
      return ae.default.setArrayAlign(ve, ge);
    }, de.Equation = function(pe, ye, ge) {
      return pe.Push(ye), ae.default.checkEqnEnv(pe), pe.itemFactory.create("equation", ge).setProperty("name", ye.getName());
    }, de.EqnArray = function(pe, ye, ge, ve, be, Ce) {
      pe.Push(ye), ve && ae.default.checkEqnEnv(pe), be = be.replace(/[^clr]/g, "").split("").join(" "), be = be.replace(/l/g, "left").replace(/r/g, "right").replace(/c/g, "center");
      var Se = pe.itemFactory.create("eqnarray", ye.getName(), ge, ve, pe.stack.global);
      return Se.arraydef = {
        displaystyle: !0,
        columnalign: be,
        columnspacing: Ce || "1em",
        rowspacing: "3pt",
        side: pe.options.tagSide,
        minlabelspacing: pe.options.tagIndent
      }, Se;
    }, de.HandleNoTag = function(pe, ye) {
      pe.tags.notag();
    }, de.HandleLabel = function(pe, ye) {
      var ge = pe.GetArgument(ye);
      if (ge !== "" && !pe.tags.refUpdate) {
        if (pe.tags.label)
          throw new te.default("MultipleCommand", "Multiple %1", pe.currentCS);
        if (pe.tags.label = ge, (pe.tags.allLabels[ge] || pe.tags.labels[ge]) && !pe.options.ignoreDuplicateLabels)
          throw new te.default("MultipleLabel", "Label '%1' multiply defined", ge);
        pe.tags.labels[ge] = new se.Label();
      }
    }, de.HandleRef = function(pe, ye, ge) {
      var ve = pe.GetArgument(ye), be = pe.tags.allLabels[ve] || pe.tags.labels[ve];
      be || (pe.tags.refUpdate || (pe.tags.redo = !0), be = new se.Label());
      var Ce = be.tag;
      ge && (Ce = pe.tags.formatTag(Ce));
      var Se = pe.create("node", "mrow", ae.default.internalMath(pe, Ce), {
        href: pe.tags.formatUrl(be.id, pe.options.baseURL),
        class: "MathJax_ref"
      });
      pe.Push(Se);
    }, de.Macro = function(pe, ye, ge, ve, be) {
      if (ve) {
        var Ce = [];
        if (be != null) {
          var Se = pe.GetBrackets(ye);
          Ce.push(Se ?? be);
        }
        for (var we = Ce.length; we < ve; we++)
          Ce.push(pe.GetArgument(ye));
        ge = ae.default.substituteArgs(pe, Ce, ge);
      }
      pe.string = ae.default.addArgs(pe, ge, pe.string.slice(pe.i)), pe.i = 0, ae.default.checkMaxMacros(pe);
    }, de.MathChoice = function(pe, ye) {
      var ge = pe.ParseArg(ye), ve = pe.ParseArg(ye), be = pe.ParseArg(ye), Ce = pe.ParseArg(ye);
      pe.Push(pe.create("node", "MathChoice", [ge, ve, be, Ce]));
    }, BaseMethods.default = de, BaseMethods;
  }
  var ParseMethods = {}, hasRequiredParseMethods;
  function requireParseMethods() {
    if (hasRequiredParseMethods) return ParseMethods;
    hasRequiredParseMethods = 1;
    var O = ParseMethods.__read || function(Z, te) {
      var ne = typeof Symbol == "function" && Z[Symbol.iterator];
      if (!ne) return Z;
      var ee = ne.call(Z), ae, oe = [], se;
      try {
        for (; (te === void 0 || te-- > 0) && !(ae = ee.next()).done; ) oe.push(ae.value);
      } catch (ce) {
        se = { error: ce };
      } finally {
        try {
          ae && !ae.done && (ne = ee.return) && ne.call(ee);
        } finally {
          if (se) throw se.error;
        }
      }
      return oe;
    }, _ = ParseMethods.__spreadArray || function(Z, te, ne) {
      if (ne || arguments.length === 2) for (var ee = 0, ae = te.length, oe; ee < ae; ee++)
        (oe || !(ee in te)) && (oe || (oe = Array.prototype.slice.call(te, 0, ee)), oe[ee] = te[ee]);
      return Z.concat(oe || Array.prototype.slice.call(te));
    }, H = ParseMethods.__importDefault || function(Z) {
      return Z && Z.__esModule ? Z : { default: Z };
    };
    Object.defineProperty(ParseMethods, "__esModule", { value: !0 });
    var V = H(requireNodeUtil()), q = requireTexConstants(), G = H(requireParseUtil()), J;
    return function(Z) {
      function te(ue, de) {
        var fe = G.default.getFontDef(ue), he = ue.stack.env;
        he.multiLetterIdentifiers && he.font !== "" && (de = ue.string.substr(ue.i - 1).match(he.multiLetterIdentifiers)[0], ue.i += de.length - 1, fe.mathvariant === q.TexConstant.Variant.NORMAL && he.noAutoOP && de.length > 1 && (fe.autoOP = !1));
        var me = ue.create("token", "mi", fe, de);
        ue.Push(me);
      }
      Z.variable = te;
      function ne(ue, de) {
        var fe, he = ue.configuration.options.digits, me = ue.string.slice(ue.i - 1).match(he), pe = G.default.getFontDef(ue);
        me ? (fe = ue.create("token", "mn", pe, me[0].replace(/[{}]/g, "")), ue.i += me[0].length - 1) : fe = ue.create("token", "mo", pe, de), ue.Push(fe);
      }
      Z.digit = ne;
      function ee(ue, de) {
        var fe = ue.GetCS();
        ue.parse("macro", [ue, fe]);
      }
      Z.controlSequence = ee;
      function ae(ue, de) {
        var fe = de.attributes || { mathvariant: q.TexConstant.Variant.ITALIC }, he = ue.create("token", "mi", fe, de.char);
        ue.Push(he);
      }
      Z.mathchar0mi = ae;
      function oe(ue, de) {
        var fe = de.attributes || {};
        fe.stretchy = !1;
        var he = ue.create("token", "mo", fe, de.char);
        V.default.setProperty(he, "fixStretchy", !0), ue.configuration.addNode("fixStretchy", he), ue.Push(he);
      }
      Z.mathchar0mo = oe;
      function se(ue, de) {
        var fe = de.attributes || { mathvariant: q.TexConstant.Variant.NORMAL };
        ue.stack.env.font && (fe.mathvariant = ue.stack.env.font);
        var he = ue.create("token", "mi", fe, de.char);
        ue.Push(he);
      }
      Z.mathchar7 = se;
      function ce(ue, de) {
        var fe = de.attributes || {};
        fe = Object.assign({ fence: !1, stretchy: !1 }, fe);
        var he = ue.create("token", "mo", fe, de.char);
        ue.Push(he);
      }
      Z.delimiter = ce;
      function le(ue, de, fe, he) {
        var me = he[0], pe = ue.itemFactory.create("begin").setProperties({ name: de, end: me });
        pe = fe.apply(void 0, _([ue, pe], O(he.slice(1)), !1)), ue.Push(pe);
      }
      Z.environment = le;
    }(J || (J = {})), ParseMethods.default = J, ParseMethods;
  }
  var hasRequiredBaseMappings;
  function requireBaseMappings() {
    if (hasRequiredBaseMappings) return BaseMappings;
    hasRequiredBaseMappings = 1;
    var O = BaseMappings.__createBinding || (Object.create ? function(ae, oe, se, ce) {
      ce === void 0 && (ce = se);
      var le = Object.getOwnPropertyDescriptor(oe, se);
      (!le || ("get" in le ? !oe.__esModule : le.writable || le.configurable)) && (le = { enumerable: !0, get: function() {
        return oe[se];
      } }), Object.defineProperty(ae, ce, le);
    } : function(ae, oe, se, ce) {
      ce === void 0 && (ce = se), ae[ce] = oe[se];
    }), _ = BaseMappings.__setModuleDefault || (Object.create ? function(ae, oe) {
      Object.defineProperty(ae, "default", { enumerable: !0, value: oe });
    } : function(ae, oe) {
      ae.default = oe;
    }), H = BaseMappings.__importStar || function(ae) {
      if (ae && ae.__esModule) return ae;
      var oe = {};
      if (ae != null) for (var se in ae) se !== "default" && Object.prototype.hasOwnProperty.call(ae, se) && O(oe, ae, se);
      return _(oe, ae), oe;
    }, V = BaseMappings.__importDefault || function(ae) {
      return ae && ae.__esModule ? ae : { default: ae };
    };
    Object.defineProperty(BaseMappings, "__esModule", { value: !0 });
    var q = H(requireSymbolMap()), G = requireTexConstants(), J = V(requireBaseMethods()), Z = V(requireParseMethods()), te = V(requireParseUtil()), ne = requireMmlNode(), ee = requireLengths();
    return new q.RegExpMap("letter", Z.default.variable, /[a-z]/i), new q.RegExpMap("digit", Z.default.digit, /[0-9.,]/), new q.RegExpMap("command", Z.default.controlSequence, /^\\/), new q.MacroMap("special", {
      "{": "Open",
      "}": "Close",
      "~": "Tilde",
      "^": "Superscript",
      _: "Subscript",
      " ": "Space",
      "	": "Space",
      "\r": "Space",
      "\n": "Space",
      "'": "Prime",
      "%": "Comment",
      "&": "Entry",
      "#": "Hash",
      " ": "Space",
      "’": "Prime"
    }, J.default), new q.CharacterMap("mathchar0mi", Z.default.mathchar0mi, {
      alpha: "α",
      beta: "β",
      gamma: "γ",
      delta: "δ",
      epsilon: "ϵ",
      zeta: "ζ",
      eta: "η",
      theta: "θ",
      iota: "ι",
      kappa: "κ",
      lambda: "λ",
      mu: "μ",
      nu: "ν",
      xi: "ξ",
      omicron: "ο",
      pi: "π",
      rho: "ρ",
      sigma: "σ",
      tau: "τ",
      upsilon: "υ",
      phi: "ϕ",
      chi: "χ",
      psi: "ψ",
      omega: "ω",
      varepsilon: "ε",
      vartheta: "ϑ",
      varpi: "ϖ",
      varrho: "ϱ",
      varsigma: "ς",
      varphi: "φ",
      S: ["§", { mathvariant: G.TexConstant.Variant.NORMAL }],
      aleph: ["ℵ", { mathvariant: G.TexConstant.Variant.NORMAL }],
      hbar: ["ℏ", { variantForm: !0 }],
      imath: "ı",
      jmath: "ȷ",
      ell: "ℓ",
      wp: ["℘", { mathvariant: G.TexConstant.Variant.NORMAL }],
      Re: ["ℜ", { mathvariant: G.TexConstant.Variant.NORMAL }],
      Im: ["ℑ", { mathvariant: G.TexConstant.Variant.NORMAL }],
      partial: ["∂", { mathvariant: G.TexConstant.Variant.ITALIC }],
      infty: ["∞", { mathvariant: G.TexConstant.Variant.NORMAL }],
      prime: ["′", { variantForm: !0 }],
      emptyset: ["∅", { mathvariant: G.TexConstant.Variant.NORMAL }],
      nabla: ["∇", { mathvariant: G.TexConstant.Variant.NORMAL }],
      top: ["⊤", { mathvariant: G.TexConstant.Variant.NORMAL }],
      bot: ["⊥", { mathvariant: G.TexConstant.Variant.NORMAL }],
      angle: ["∠", { mathvariant: G.TexConstant.Variant.NORMAL }],
      triangle: ["△", { mathvariant: G.TexConstant.Variant.NORMAL }],
      backslash: ["∖", { mathvariant: G.TexConstant.Variant.NORMAL }],
      forall: ["∀", { mathvariant: G.TexConstant.Variant.NORMAL }],
      exists: ["∃", { mathvariant: G.TexConstant.Variant.NORMAL }],
      neg: ["¬", { mathvariant: G.TexConstant.Variant.NORMAL }],
      lnot: ["¬", { mathvariant: G.TexConstant.Variant.NORMAL }],
      flat: ["♭", { mathvariant: G.TexConstant.Variant.NORMAL }],
      natural: ["♮", { mathvariant: G.TexConstant.Variant.NORMAL }],
      sharp: ["♯", { mathvariant: G.TexConstant.Variant.NORMAL }],
      clubsuit: ["♣", { mathvariant: G.TexConstant.Variant.NORMAL }],
      diamondsuit: ["♢", { mathvariant: G.TexConstant.Variant.NORMAL }],
      heartsuit: ["♡", { mathvariant: G.TexConstant.Variant.NORMAL }],
      spadesuit: ["♠", { mathvariant: G.TexConstant.Variant.NORMAL }]
    }), new q.CharacterMap("mathchar0mo", Z.default.mathchar0mo, {
      surd: "√",
      coprod: ["∐", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      bigvee: ["⋁", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      bigwedge: ["⋀", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      biguplus: ["⨄", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      bigcap: ["⋂", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      bigcup: ["⋃", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      int: ["∫", { texClass: ne.TEXCLASS.OP }],
      intop: ["∫", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0,
        movablelimits: !0
      }],
      iint: ["∬", { texClass: ne.TEXCLASS.OP }],
      iiint: ["∭", { texClass: ne.TEXCLASS.OP }],
      prod: ["∏", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      sum: ["∑", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      bigotimes: ["⨂", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      bigoplus: ["⨁", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      bigodot: ["⨀", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      oint: ["∮", { texClass: ne.TEXCLASS.OP }],
      bigsqcup: ["⨆", {
        texClass: ne.TEXCLASS.OP,
        movesupsub: !0
      }],
      smallint: ["∫", { largeop: !1 }],
      triangleleft: "◃",
      triangleright: "▹",
      bigtriangleup: "△",
      bigtriangledown: "▽",
      wedge: "∧",
      land: "∧",
      vee: "∨",
      lor: "∨",
      cap: "∩",
      cup: "∪",
      ddagger: "‡",
      dagger: "†",
      sqcap: "⊓",
      sqcup: "⊔",
      uplus: "⊎",
      amalg: "⨿",
      diamond: "⋄",
      bullet: "∙",
      wr: "≀",
      div: "÷",
      divsymbol: "÷",
      odot: ["⊙", { largeop: !1 }],
      oslash: ["⊘", { largeop: !1 }],
      otimes: ["⊗", { largeop: !1 }],
      ominus: ["⊖", { largeop: !1 }],
      oplus: ["⊕", { largeop: !1 }],
      mp: "∓",
      pm: "±",
      circ: "∘",
      bigcirc: "◯",
      setminus: "∖",
      cdot: "⋅",
      ast: "∗",
      times: "×",
      star: "⋆",
      propto: "∝",
      sqsubseteq: "⊑",
      sqsupseteq: "⊒",
      parallel: "∥",
      mid: "∣",
      dashv: "⊣",
      vdash: "⊢",
      leq: "≤",
      le: "≤",
      geq: "≥",
      ge: "≥",
      lt: "<",
      gt: ">",
      succ: "≻",
      prec: "≺",
      approx: "≈",
      succeq: "⪰",
      preceq: "⪯",
      supset: "⊃",
      subset: "⊂",
      supseteq: "⊇",
      subseteq: "⊆",
      in: "∈",
      ni: "∋",
      notin: "∉",
      owns: "∋",
      gg: "≫",
      ll: "≪",
      sim: "∼",
      simeq: "≃",
      perp: "⊥",
      equiv: "≡",
      asymp: "≍",
      smile: "⌣",
      frown: "⌢",
      ne: "≠",
      neq: "≠",
      cong: "≅",
      doteq: "≐",
      bowtie: "⋈",
      models: "⊨",
      notChar: "⧸",
      Leftrightarrow: "⇔",
      Leftarrow: "⇐",
      Rightarrow: "⇒",
      leftrightarrow: "↔",
      leftarrow: "←",
      gets: "←",
      rightarrow: "→",
      to: ["→", { accent: !1 }],
      mapsto: "↦",
      leftharpoonup: "↼",
      leftharpoondown: "↽",
      rightharpoonup: "⇀",
      rightharpoondown: "⇁",
      nearrow: "↗",
      searrow: "↘",
      nwarrow: "↖",
      swarrow: "↙",
      rightleftharpoons: "⇌",
      hookrightarrow: "↪",
      hookleftarrow: "↩",
      longleftarrow: "⟵",
      Longleftarrow: "⟸",
      longrightarrow: "⟶",
      Longrightarrow: "⟹",
      Longleftrightarrow: "⟺",
      longleftrightarrow: "⟷",
      longmapsto: "⟼",
      ldots: "…",
      cdots: "⋯",
      vdots: "⋮",
      ddots: "⋱",
      dotsc: "…",
      dotsb: "⋯",
      dotsm: "⋯",
      dotsi: "⋯",
      dotso: "…",
      ldotp: [".", { texClass: ne.TEXCLASS.PUNCT }],
      cdotp: ["⋅", { texClass: ne.TEXCLASS.PUNCT }],
      colon: [":", { texClass: ne.TEXCLASS.PUNCT }]
    }), new q.CharacterMap("mathchar7", Z.default.mathchar7, {
      Gamma: "Γ",
      Delta: "Δ",
      Theta: "Θ",
      Lambda: "Λ",
      Xi: "Ξ",
      Pi: "Π",
      Sigma: "Σ",
      Upsilon: "Υ",
      Phi: "Φ",
      Psi: "Ψ",
      Omega: "Ω",
      _: "_",
      "#": "#",
      $: "$",
      "%": "%",
      "&": "&",
      And: "&"
    }), new q.DelimiterMap("delimiter", Z.default.delimiter, {
      "(": "(",
      ")": ")",
      "[": "[",
      "]": "]",
      "<": "⟨",
      ">": "⟩",
      "\\lt": "⟨",
      "\\gt": "⟩",
      "/": "/",
      "|": ["|", { texClass: ne.TEXCLASS.ORD }],
      ".": "",
      "\\\\": "\\",
      "\\lmoustache": "⎰",
      "\\rmoustache": "⎱",
      "\\lgroup": "⟮",
      "\\rgroup": "⟯",
      "\\arrowvert": "⏐",
      "\\Arrowvert": "‖",
      "\\bracevert": "⎪",
      "\\Vert": ["‖", { texClass: ne.TEXCLASS.ORD }],
      "\\|": ["‖", { texClass: ne.TEXCLASS.ORD }],
      "\\vert": ["|", { texClass: ne.TEXCLASS.ORD }],
      "\\uparrow": "↑",
      "\\downarrow": "↓",
      "\\updownarrow": "↕",
      "\\Uparrow": "⇑",
      "\\Downarrow": "⇓",
      "\\Updownarrow": "⇕",
      "\\backslash": "\\",
      "\\rangle": "⟩",
      "\\langle": "⟨",
      "\\rbrace": "}",
      "\\lbrace": "{",
      "\\}": "}",
      "\\{": "{",
      "\\rceil": "⌉",
      "\\lceil": "⌈",
      "\\rfloor": "⌋",
      "\\lfloor": "⌊",
      "\\lbrack": "[",
      "\\rbrack": "]"
    }), new q.CommandMap("macros", {
      displaystyle: ["SetStyle", "D", !0, 0],
      textstyle: ["SetStyle", "T", !1, 0],
      scriptstyle: ["SetStyle", "S", !1, 1],
      scriptscriptstyle: ["SetStyle", "SS", !1, 2],
      rm: ["SetFont", G.TexConstant.Variant.NORMAL],
      mit: ["SetFont", G.TexConstant.Variant.ITALIC],
      oldstyle: ["SetFont", G.TexConstant.Variant.OLDSTYLE],
      cal: ["SetFont", G.TexConstant.Variant.CALLIGRAPHIC],
      it: ["SetFont", G.TexConstant.Variant.MATHITALIC],
      bf: ["SetFont", G.TexConstant.Variant.BOLD],
      bbFont: ["SetFont", G.TexConstant.Variant.DOUBLESTRUCK],
      scr: ["SetFont", G.TexConstant.Variant.SCRIPT],
      frak: ["SetFont", G.TexConstant.Variant.FRAKTUR],
      sf: ["SetFont", G.TexConstant.Variant.SANSSERIF],
      tt: ["SetFont", G.TexConstant.Variant.MONOSPACE],
      mathrm: ["MathFont", G.TexConstant.Variant.NORMAL],
      mathup: ["MathFont", G.TexConstant.Variant.NORMAL],
      mathnormal: ["MathFont", ""],
      mathbf: ["MathFont", G.TexConstant.Variant.BOLD],
      mathbfup: ["MathFont", G.TexConstant.Variant.BOLD],
      mathit: ["MathFont", G.TexConstant.Variant.MATHITALIC],
      mathbfit: ["MathFont", G.TexConstant.Variant.BOLDITALIC],
      mathbb: ["MathFont", G.TexConstant.Variant.DOUBLESTRUCK],
      Bbb: ["MathFont", G.TexConstant.Variant.DOUBLESTRUCK],
      mathfrak: ["MathFont", G.TexConstant.Variant.FRAKTUR],
      mathbffrak: ["MathFont", G.TexConstant.Variant.BOLDFRAKTUR],
      mathscr: ["MathFont", G.TexConstant.Variant.SCRIPT],
      mathbfscr: ["MathFont", G.TexConstant.Variant.BOLDSCRIPT],
      mathsf: ["MathFont", G.TexConstant.Variant.SANSSERIF],
      mathsfup: ["MathFont", G.TexConstant.Variant.SANSSERIF],
      mathbfsf: ["MathFont", G.TexConstant.Variant.BOLDSANSSERIF],
      mathbfsfup: ["MathFont", G.TexConstant.Variant.BOLDSANSSERIF],
      mathsfit: ["MathFont", G.TexConstant.Variant.SANSSERIFITALIC],
      mathbfsfit: ["MathFont", G.TexConstant.Variant.SANSSERIFBOLDITALIC],
      mathtt: ["MathFont", G.TexConstant.Variant.MONOSPACE],
      mathcal: ["MathFont", G.TexConstant.Variant.CALLIGRAPHIC],
      mathbfcal: ["MathFont", G.TexConstant.Variant.BOLDCALLIGRAPHIC],
      symrm: ["MathFont", G.TexConstant.Variant.NORMAL],
      symup: ["MathFont", G.TexConstant.Variant.NORMAL],
      symnormal: ["MathFont", ""],
      symbf: ["MathFont", G.TexConstant.Variant.BOLD],
      symbfup: ["MathFont", G.TexConstant.Variant.BOLD],
      symit: ["MathFont", G.TexConstant.Variant.ITALIC],
      symbfit: ["MathFont", G.TexConstant.Variant.BOLDITALIC],
      symbb: ["MathFont", G.TexConstant.Variant.DOUBLESTRUCK],
      symfrak: ["MathFont", G.TexConstant.Variant.FRAKTUR],
      symbffrak: ["MathFont", G.TexConstant.Variant.BOLDFRAKTUR],
      symscr: ["MathFont", G.TexConstant.Variant.SCRIPT],
      symbfscr: ["MathFont", G.TexConstant.Variant.BOLDSCRIPT],
      symsf: ["MathFont", G.TexConstant.Variant.SANSSERIF],
      symsfup: ["MathFont", G.TexConstant.Variant.SANSSERIF],
      symbfsf: ["MathFont", G.TexConstant.Variant.BOLDSANSSERIF],
      symbfsfup: ["MathFont", G.TexConstant.Variant.BOLDSANSSERIF],
      symsfit: ["MathFont", G.TexConstant.Variant.SANSSERIFITALIC],
      symbfsfit: ["MathFont", G.TexConstant.Variant.SANSSERIFBOLDITALIC],
      symtt: ["MathFont", G.TexConstant.Variant.MONOSPACE],
      symcal: ["MathFont", G.TexConstant.Variant.CALLIGRAPHIC],
      symbfcal: ["MathFont", G.TexConstant.Variant.BOLDCALLIGRAPHIC],
      textrm: ["HBox", null, G.TexConstant.Variant.NORMAL],
      textup: ["HBox", null, G.TexConstant.Variant.NORMAL],
      textnormal: ["HBox"],
      textit: ["HBox", null, G.TexConstant.Variant.ITALIC],
      textbf: ["HBox", null, G.TexConstant.Variant.BOLD],
      textsf: ["HBox", null, G.TexConstant.Variant.SANSSERIF],
      texttt: ["HBox", null, G.TexConstant.Variant.MONOSPACE],
      tiny: ["SetSize", 0.5],
      Tiny: ["SetSize", 0.6],
      scriptsize: ["SetSize", 0.7],
      small: ["SetSize", 0.85],
      normalsize: ["SetSize", 1],
      large: ["SetSize", 1.2],
      Large: ["SetSize", 1.44],
      LARGE: ["SetSize", 1.73],
      huge: ["SetSize", 2.07],
      Huge: ["SetSize", 2.49],
      arcsin: "NamedFn",
      arccos: "NamedFn",
      arctan: "NamedFn",
      arg: "NamedFn",
      cos: "NamedFn",
      cosh: "NamedFn",
      cot: "NamedFn",
      coth: "NamedFn",
      csc: "NamedFn",
      deg: "NamedFn",
      det: "NamedOp",
      dim: "NamedFn",
      exp: "NamedFn",
      gcd: "NamedOp",
      hom: "NamedFn",
      inf: "NamedOp",
      ker: "NamedFn",
      lg: "NamedFn",
      lim: "NamedOp",
      liminf: ["NamedOp", "lim&thinsp;inf"],
      limsup: ["NamedOp", "lim&thinsp;sup"],
      ln: "NamedFn",
      log: "NamedFn",
      max: "NamedOp",
      min: "NamedOp",
      Pr: "NamedOp",
      sec: "NamedFn",
      sin: "NamedFn",
      sinh: "NamedFn",
      sup: "NamedOp",
      tan: "NamedFn",
      tanh: "NamedFn",
      limits: ["Limits", 1],
      nolimits: ["Limits", 0],
      overline: ["UnderOver", "2015"],
      underline: ["UnderOver", "2015"],
      overbrace: ["UnderOver", "23DE", 1],
      underbrace: ["UnderOver", "23DF", 1],
      overparen: ["UnderOver", "23DC"],
      underparen: ["UnderOver", "23DD"],
      overrightarrow: ["UnderOver", "2192"],
      underrightarrow: ["UnderOver", "2192"],
      overleftarrow: ["UnderOver", "2190"],
      underleftarrow: ["UnderOver", "2190"],
      overleftrightarrow: ["UnderOver", "2194"],
      underleftrightarrow: ["UnderOver", "2194"],
      overset: "Overset",
      underset: "Underset",
      overunderset: "Overunderset",
      stackrel: ["Macro", "\\mathrel{\\mathop{#2}\\limits^{#1}}", 2],
      stackbin: ["Macro", "\\mathbin{\\mathop{#2}\\limits^{#1}}", 2],
      over: "Over",
      overwithdelims: "Over",
      atop: "Over",
      atopwithdelims: "Over",
      above: "Over",
      abovewithdelims: "Over",
      brace: ["Over", "{", "}"],
      brack: ["Over", "[", "]"],
      choose: ["Over", "(", ")"],
      frac: "Frac",
      sqrt: "Sqrt",
      root: "Root",
      uproot: ["MoveRoot", "upRoot"],
      leftroot: ["MoveRoot", "leftRoot"],
      left: "LeftRight",
      right: "LeftRight",
      middle: "LeftRight",
      llap: "Lap",
      rlap: "Lap",
      raise: "RaiseLower",
      lower: "RaiseLower",
      moveleft: "MoveLeftRight",
      moveright: "MoveLeftRight",
      ",": ["Spacer", ee.MATHSPACE.thinmathspace],
      ":": ["Spacer", ee.MATHSPACE.mediummathspace],
      ">": ["Spacer", ee.MATHSPACE.mediummathspace],
      ";": ["Spacer", ee.MATHSPACE.thickmathspace],
      "!": ["Spacer", ee.MATHSPACE.negativethinmathspace],
      enspace: ["Spacer", 0.5],
      quad: ["Spacer", 1],
      qquad: ["Spacer", 2],
      thinspace: ["Spacer", ee.MATHSPACE.thinmathspace],
      negthinspace: ["Spacer", ee.MATHSPACE.negativethinmathspace],
      hskip: "Hskip",
      hspace: "Hskip",
      kern: "Hskip",
      mskip: "Hskip",
      mspace: "Hskip",
      mkern: "Hskip",
      rule: "rule",
      Rule: ["Rule"],
      Space: ["Rule", "blank"],
      nonscript: "Nonscript",
      big: ["MakeBig", ne.TEXCLASS.ORD, 0.85],
      Big: ["MakeBig", ne.TEXCLASS.ORD, 1.15],
      bigg: ["MakeBig", ne.TEXCLASS.ORD, 1.45],
      Bigg: ["MakeBig", ne.TEXCLASS.ORD, 1.75],
      bigl: ["MakeBig", ne.TEXCLASS.OPEN, 0.85],
      Bigl: ["MakeBig", ne.TEXCLASS.OPEN, 1.15],
      biggl: ["MakeBig", ne.TEXCLASS.OPEN, 1.45],
      Biggl: ["MakeBig", ne.TEXCLASS.OPEN, 1.75],
      bigr: ["MakeBig", ne.TEXCLASS.CLOSE, 0.85],
      Bigr: ["MakeBig", ne.TEXCLASS.CLOSE, 1.15],
      biggr: ["MakeBig", ne.TEXCLASS.CLOSE, 1.45],
      Biggr: ["MakeBig", ne.TEXCLASS.CLOSE, 1.75],
      bigm: ["MakeBig", ne.TEXCLASS.REL, 0.85],
      Bigm: ["MakeBig", ne.TEXCLASS.REL, 1.15],
      biggm: ["MakeBig", ne.TEXCLASS.REL, 1.45],
      Biggm: ["MakeBig", ne.TEXCLASS.REL, 1.75],
      mathord: ["TeXAtom", ne.TEXCLASS.ORD],
      mathop: ["TeXAtom", ne.TEXCLASS.OP],
      mathopen: ["TeXAtom", ne.TEXCLASS.OPEN],
      mathclose: ["TeXAtom", ne.TEXCLASS.CLOSE],
      mathbin: ["TeXAtom", ne.TEXCLASS.BIN],
      mathrel: ["TeXAtom", ne.TEXCLASS.REL],
      mathpunct: ["TeXAtom", ne.TEXCLASS.PUNCT],
      mathinner: ["TeXAtom", ne.TEXCLASS.INNER],
      vcenter: ["TeXAtom", ne.TEXCLASS.VCENTER],
      buildrel: "BuildRel",
      hbox: ["HBox", 0],
      text: "HBox",
      mbox: ["HBox", 0],
      fbox: "FBox",
      boxed: ["Macro", "\\fbox{$\\displaystyle{#1}$}", 1],
      framebox: "FrameBox",
      strut: "Strut",
      mathstrut: ["Macro", "\\vphantom{(}"],
      phantom: "Phantom",
      vphantom: ["Phantom", 1, 0],
      hphantom: ["Phantom", 0, 1],
      smash: "Smash",
      acute: ["Accent", "00B4"],
      grave: ["Accent", "0060"],
      ddot: ["Accent", "00A8"],
      tilde: ["Accent", "007E"],
      bar: ["Accent", "00AF"],
      breve: ["Accent", "02D8"],
      check: ["Accent", "02C7"],
      hat: ["Accent", "005E"],
      vec: ["Accent", "2192"],
      dot: ["Accent", "02D9"],
      widetilde: ["Accent", "007E", 1],
      widehat: ["Accent", "005E", 1],
      matrix: "Matrix",
      array: "Matrix",
      pmatrix: ["Matrix", "(", ")"],
      cases: [
        "Matrix",
        "{",
        "",
        "left left",
        null,
        ".1em",
        null,
        !0
      ],
      eqalign: [
        "Matrix",
        null,
        null,
        "right left",
        (0, ee.em)(ee.MATHSPACE.thickmathspace),
        ".5em",
        "D"
      ],
      displaylines: ["Matrix", null, null, "center", null, ".5em", "D"],
      cr: "Cr",
      "\\": "CrLaTeX",
      newline: ["CrLaTeX", !0],
      hline: ["HLine", "solid"],
      hdashline: ["HLine", "dashed"],
      eqalignno: [
        "Matrix",
        null,
        null,
        "right left",
        (0, ee.em)(ee.MATHSPACE.thickmathspace),
        ".5em",
        "D",
        null,
        "right"
      ],
      leqalignno: [
        "Matrix",
        null,
        null,
        "right left",
        (0, ee.em)(ee.MATHSPACE.thickmathspace),
        ".5em",
        "D",
        null,
        "left"
      ],
      hfill: "HFill",
      hfil: "HFill",
      hfilll: "HFill",
      bmod: ["Macro", '\\mmlToken{mo}[lspace="thickmathspace" rspace="thickmathspace"]{mod}'],
      pmod: ["Macro", "\\pod{\\mmlToken{mi}{mod}\\kern 6mu #1}", 1],
      mod: [
        "Macro",
        "\\mathchoice{\\kern18mu}{\\kern12mu}{\\kern12mu}{\\kern12mu}\\mmlToken{mi}{mod}\\,\\,#1",
        1
      ],
      pod: ["Macro", "\\mathchoice{\\kern18mu}{\\kern8mu}{\\kern8mu}{\\kern8mu}(#1)", 1],
      iff: ["Macro", "\\;\\Longleftrightarrow\\;"],
      skew: ["Macro", "{{#2{#3\\mkern#1mu}\\mkern-#1mu}{}}", 3],
      pmb: ["Macro", "\\rlap{#1}\\kern1px{#1}", 1],
      TeX: ["Macro", "T\\kern-.14em\\lower.5ex{E}\\kern-.115em X"],
      LaTeX: ["Macro", "L\\kern-.325em\\raise.21em{\\scriptstyle{A}}\\kern-.17em\\TeX"],
      " ": ["Macro", "\\text{ }"],
      not: "Not",
      dots: "Dots",
      space: "Tilde",
      " ": "Tilde",
      begin: "BeginEnd",
      end: "BeginEnd",
      label: "HandleLabel",
      ref: "HandleRef",
      nonumber: "HandleNoTag",
      mathchoice: "MathChoice",
      mmlToken: "MmlToken"
    }, J.default), new q.EnvironmentMap("environment", Z.default.environment, {
      array: ["AlignedArray"],
      equation: ["Equation", null, !0],
      eqnarray: [
        "EqnArray",
        null,
        !0,
        !0,
        "rcl",
        te.default.cols(0, ee.MATHSPACE.thickmathspace),
        ".5em"
      ]
    }, J.default), new q.CharacterMap("not_remap", null, {
      "←": "↚",
      "→": "↛",
      "↔": "↮",
      "⇐": "⇍",
      "⇒": "⇏",
      "⇔": "⇎",
      "∈": "∉",
      "∋": "∌",
      "∣": "∤",
      "∥": "∦",
      "∼": "≁",
      "~": "≁",
      "≃": "≄",
      "≅": "≇",
      "≈": "≉",
      "≍": "≭",
      "=": "≠",
      "≡": "≢",
      "<": "≮",
      ">": "≯",
      "≤": "≰",
      "≥": "≱",
      "≲": "≴",
      "≳": "≵",
      "≶": "≸",
      "≷": "≹",
      "≺": "⊀",
      "≻": "⊁",
      "⊂": "⊄",
      "⊃": "⊅",
      "⊆": "⊈",
      "⊇": "⊉",
      "⊢": "⊬",
      "⊨": "⊭",
      "⊩": "⊮",
      "⊫": "⊯",
      "≼": "⋠",
      "≽": "⋡",
      "⊑": "⋢",
      "⊒": "⋣",
      "⊲": "⋪",
      "⊳": "⋫",
      "⊴": "⋬",
      "⊵": "⋭",
      "∃": "∄"
    }), BaseMappings;
  }
  var hasRequiredBaseConfiguration;
  function requireBaseConfiguration() {
    if (hasRequiredBaseConfiguration) return BaseConfiguration;
    hasRequiredBaseConfiguration = 1;
    var O = BaseConfiguration.__extends || /* @__PURE__ */ function() {
      var me = function(pe, ye) {
        return me = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ge, ve) {
          ge.__proto__ = ve;
        } || function(ge, ve) {
          for (var be in ve) Object.prototype.hasOwnProperty.call(ve, be) && (ge[be] = ve[be]);
        }, me(pe, ye);
      };
      return function(pe, ye) {
        if (typeof ye != "function" && ye !== null)
          throw new TypeError("Class extends value " + String(ye) + " is not a constructor or null");
        me(pe, ye);
        function ge() {
          this.constructor = pe;
        }
        pe.prototype = ye === null ? Object.create(ye) : (ge.prototype = ye.prototype, new ge());
      };
    }(), _ = BaseConfiguration.__createBinding || (Object.create ? function(me, pe, ye, ge) {
      ge === void 0 && (ge = ye);
      var ve = Object.getOwnPropertyDescriptor(pe, ye);
      (!ve || ("get" in ve ? !pe.__esModule : ve.writable || ve.configurable)) && (ve = { enumerable: !0, get: function() {
        return pe[ye];
      } }), Object.defineProperty(me, ge, ve);
    } : function(me, pe, ye, ge) {
      ge === void 0 && (ge = ye), me[ge] = pe[ye];
    }), H = BaseConfiguration.__setModuleDefault || (Object.create ? function(me, pe) {
      Object.defineProperty(me, "default", { enumerable: !0, value: pe });
    } : function(me, pe) {
      me.default = pe;
    }), V = BaseConfiguration.__importStar || function(me) {
      if (me && me.__esModule) return me;
      var pe = {};
      if (me != null) for (var ye in me) ye !== "default" && Object.prototype.hasOwnProperty.call(me, ye) && _(pe, me, ye);
      return H(pe, me), pe;
    }, q = BaseConfiguration.__values || function(me) {
      var pe = typeof Symbol == "function" && Symbol.iterator, ye = pe && me[pe], ge = 0;
      if (ye) return ye.call(me);
      if (me && typeof me.length == "number") return {
        next: function() {
          return me && ge >= me.length && (me = void 0), { value: me && me[ge++], done: !me };
        }
      };
      throw new TypeError(pe ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, G = BaseConfiguration.__importDefault || function(me) {
      return me && me.__esModule ? me : { default: me };
    }, J;
    Object.defineProperty(BaseConfiguration, "__esModule", { value: !0 }), BaseConfiguration.BaseConfiguration = BaseConfiguration.BaseTags = BaseConfiguration.Other = void 0;
    var Z = requireConfiguration(), te = requireMapHandler(), ne = G(requireTexError()), ee = G(requireNodeUtil()), ae = requireSymbolMap(), oe = V(requireBaseItems()), se = requireTags();
    requireBaseMappings();
    var ce = requireOperatorDictionary();
    new ae.CharacterMap("remap", null, {
      "-": "−",
      "*": "∗",
      "`": "‘"
    });
    function le(me, pe) {
      var ye = me.stack.env.font, ge = ye ? { mathvariant: me.stack.env.font } : {}, ve = te.MapHandler.getMap("remap").lookup(pe), be = (0, ce.getRange)(pe), Ce = be ? be[3] : "mo", Se = me.create("token", Ce, ge, ve ? ve.char : pe);
      be[4] && Se.attributes.set("mathvariant", be[4]), Ce === "mo" && (ee.default.setProperty(Se, "fixStretchy", !0), me.configuration.addNode("fixStretchy", Se)), me.Push(Se);
    }
    BaseConfiguration.Other = le;
    function ue(me, pe) {
      throw new ne.default("UndefinedControlSequence", "Undefined control sequence %1", "\\" + pe);
    }
    function de(me, pe) {
      throw new ne.default("UnknownEnv", "Unknown environment '%1'", pe);
    }
    function fe(me) {
      var pe, ye, ge = me.data;
      try {
        for (var ve = q(ge.getList("nonscript")), be = ve.next(); !be.done; be = ve.next()) {
          var Ce = be.value;
          if (Ce.attributes.get("scriptlevel") > 0) {
            var Se = Ce.parent;
            if (Se.childNodes.splice(Se.childIndex(Ce), 1), ge.removeFromList(Ce.kind, [Ce]), Ce.isKind("mrow")) {
              var we = Ce.childNodes[0];
              ge.removeFromList("mstyle", [we]), ge.removeFromList("mspace", we.childNodes[0].childNodes);
            }
          } else Ce.isKind("mrow") && (Ce.parent.replaceChild(Ce.childNodes[0], Ce), ge.removeFromList("mrow", [Ce]));
        }
      } catch (Ee) {
        pe = { error: Ee };
      } finally {
        try {
          be && !be.done && (ye = ve.return) && ye.call(ve);
        } finally {
          if (pe) throw pe.error;
        }
      }
    }
    var he = function(me) {
      O(pe, me);
      function pe() {
        return me !== null && me.apply(this, arguments) || this;
      }
      return pe;
    }(se.AbstractTags);
    return BaseConfiguration.BaseTags = he, BaseConfiguration.BaseConfiguration = Z.Configuration.create("base", {
      handler: {
        character: ["command", "special", "letter", "digit"],
        delimiter: ["delimiter"],
        macro: ["delimiter", "macros", "mathchar0mi", "mathchar0mo", "mathchar7"],
        environment: ["environment"]
      },
      fallback: {
        character: le,
        macro: ue,
        environment: de
      },
      items: (J = {}, J[oe.StartItem.prototype.kind] = oe.StartItem, J[oe.StopItem.prototype.kind] = oe.StopItem, J[oe.OpenItem.prototype.kind] = oe.OpenItem, J[oe.CloseItem.prototype.kind] = oe.CloseItem, J[oe.PrimeItem.prototype.kind] = oe.PrimeItem, J[oe.SubsupItem.prototype.kind] = oe.SubsupItem, J[oe.OverItem.prototype.kind] = oe.OverItem, J[oe.LeftItem.prototype.kind] = oe.LeftItem, J[oe.Middle.prototype.kind] = oe.Middle, J[oe.RightItem.prototype.kind] = oe.RightItem, J[oe.BeginItem.prototype.kind] = oe.BeginItem, J[oe.EndItem.prototype.kind] = oe.EndItem, J[oe.StyleItem.prototype.kind] = oe.StyleItem, J[oe.PositionItem.prototype.kind] = oe.PositionItem, J[oe.CellItem.prototype.kind] = oe.CellItem, J[oe.MmlItem.prototype.kind] = oe.MmlItem, J[oe.FnItem.prototype.kind] = oe.FnItem, J[oe.NotItem.prototype.kind] = oe.NotItem, J[oe.NonscriptItem.prototype.kind] = oe.NonscriptItem, J[oe.DotsItem.prototype.kind] = oe.DotsItem, J[oe.ArrayItem.prototype.kind] = oe.ArrayItem, J[oe.EqnArrayItem.prototype.kind] = oe.EqnArrayItem, J[oe.EquationItem.prototype.kind] = oe.EquationItem, J),
      options: {
        maxMacros: 1e3,
        baseURL: typeof document > "u" || document.getElementsByTagName("base").length === 0 ? "" : String(document.location).replace(/#.*$/, "")
      },
      tags: {
        base: he
      },
      postprocessors: [[fe, -4]]
    }), BaseConfiguration;
  }
  var hasRequiredTex$2;
  function requireTex$2() {
    if (hasRequiredTex$2) return tex$2;
    hasRequiredTex$2 = 1;
    var O = tex$2.__extends || /* @__PURE__ */ function() {
      var le = function(ue, de) {
        return le = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(fe, he) {
          fe.__proto__ = he;
        } || function(fe, he) {
          for (var me in he) Object.prototype.hasOwnProperty.call(he, me) && (fe[me] = he[me]);
        }, le(ue, de);
      };
      return function(ue, de) {
        if (typeof de != "function" && de !== null)
          throw new TypeError("Class extends value " + String(de) + " is not a constructor or null");
        le(ue, de);
        function fe() {
          this.constructor = ue;
        }
        ue.prototype = de === null ? Object.create(de) : (fe.prototype = de.prototype, new fe());
      };
    }(), _ = tex$2.__assign || function() {
      return _ = Object.assign || function(le) {
        for (var ue, de = 1, fe = arguments.length; de < fe; de++) {
          ue = arguments[de];
          for (var he in ue) Object.prototype.hasOwnProperty.call(ue, he) && (le[he] = ue[he]);
        }
        return le;
      }, _.apply(this, arguments);
    }, H = tex$2.__read || function(le, ue) {
      var de = typeof Symbol == "function" && le[Symbol.iterator];
      if (!de) return le;
      var fe = de.call(le), he, me = [], pe;
      try {
        for (; (ue === void 0 || ue-- > 0) && !(he = fe.next()).done; ) me.push(he.value);
      } catch (ye) {
        pe = { error: ye };
      } finally {
        try {
          he && !he.done && (de = fe.return) && de.call(fe);
        } finally {
          if (pe) throw pe.error;
        }
      }
      return me;
    }, V = tex$2.__importDefault || function(le) {
      return le && le.__esModule ? le : { default: le };
    };
    Object.defineProperty(tex$2, "__esModule", { value: !0 }), tex$2.TeX = void 0;
    var q = requireInputJax(), G = requireOptions(), J = requireFindTeX(), Z = V(requireFilterUtil()), te = V(requireNodeUtil()), ne = V(requireTexParser()), ee = V(requireTexError()), ae = V(requireParseOptions$1()), oe = requireTags(), se = requireConfiguration();
    requireBaseConfiguration();
    var ce = function(le) {
      O(ue, le);
      function ue(de) {
        de === void 0 && (de = {});
        var fe = this, he = H((0, G.separateOptions)(de, ue.OPTIONS, J.FindTeX.OPTIONS), 3), me = he[0], pe = he[1], ye = he[2];
        fe = le.call(this, pe) || this, fe.findTeX = fe.options.FindTeX || new J.FindTeX(ye);
        var ge = fe.options.packages, ve = fe.configuration = ue.configure(ge), be = fe._parseOptions = new ae.default(ve, [fe.options, oe.TagsFactory.OPTIONS]);
        return (0, G.userOptions)(be.options, me), ve.config(fe), ue.tags(be, ve), fe.postFilters.add(Z.default.cleanSubSup, -6), fe.postFilters.add(Z.default.setInherited, -5), fe.postFilters.add(Z.default.moveLimits, -4), fe.postFilters.add(Z.default.cleanStretchy, -3), fe.postFilters.add(Z.default.cleanAttributes, -2), fe.postFilters.add(Z.default.combineRelations, -1), fe;
      }
      return ue.configure = function(de) {
        var fe = new se.ParserConfiguration(de, ["tex"]);
        return fe.init(), fe;
      }, ue.tags = function(de, fe) {
        oe.TagsFactory.addTags(fe.tags), oe.TagsFactory.setDefault(de.options.tags), de.tags = oe.TagsFactory.getDefault(), de.tags.configuration = de;
      }, ue.prototype.setMmlFactory = function(de) {
        le.prototype.setMmlFactory.call(this, de), this._parseOptions.nodeFactory.setMmlFactory(de);
      }, Object.defineProperty(ue.prototype, "parseOptions", {
        get: function() {
          return this._parseOptions;
        },
        enumerable: !1,
        configurable: !0
      }), ue.prototype.reset = function(de) {
        de === void 0 && (de = 0), this.parseOptions.tags.reset(de);
      }, ue.prototype.compile = function(de, fe) {
        this.parseOptions.clear(), this.executeFilters(this.preFilters, de, fe, this.parseOptions);
        var he = de.display;
        this.latex = de.math;
        var me;
        this.parseOptions.tags.startEquation(de);
        var pe;
        try {
          var ye = new ne.default(this.latex, { display: he, isInner: !1 }, this.parseOptions);
          me = ye.mml(), pe = ye.stack.global;
        } catch (ge) {
          if (!(ge instanceof ee.default))
            throw ge;
          this.parseOptions.error = !0, me = this.options.formatError(this, ge);
        }
        return me = this.parseOptions.nodeFactory.create("node", "math", [me]), pe != null && pe.indentalign && te.default.setAttribute(me, "indentalign", pe.indentalign), he && te.default.setAttribute(me, "display", "block"), this.parseOptions.tags.finishEquation(de), this.parseOptions.root = me, this.executeFilters(this.postFilters, de, fe, this.parseOptions), this.mathNode = this.parseOptions.root, this.mathNode;
      }, ue.prototype.findMath = function(de) {
        return this.findTeX.findMath(de);
      }, ue.prototype.formatError = function(de) {
        var fe = de.message.replace(/\n.*/, "");
        return this.parseOptions.nodeFactory.create("error", fe, de.id, this.latex);
      }, ue.NAME = "TeX", ue.OPTIONS = _(_({}, q.AbstractInputJax.OPTIONS), { FindTeX: null, packages: ["base"], digits: /^(?:[0-9]+(?:\{,\}[0-9]{3})*(?:\.[0-9]*)?|\.[0-9]+)/, maxBuffer: 5 * 1024, formatError: function(de, fe) {
        return de.formatError(fe);
      } }), ue;
    }(q.AbstractInputJax);
    return tex$2.TeX = ce, tex$2;
  }
  var texExports = requireTex$2(), mathml = {}, FindMathML = {}, hasRequiredFindMathML;
  function requireFindMathML() {
    if (hasRequiredFindMathML) return FindMathML;
    hasRequiredFindMathML = 1;
    var O = FindMathML.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = FindMathML.__values || function(G) {
      var J = typeof Symbol == "function" && Symbol.iterator, Z = J && G[J], te = 0;
      if (Z) return Z.call(G);
      if (G && typeof G.length == "number") return {
        next: function() {
          return G && te >= G.length && (G = void 0), { value: G && G[te++], done: !G };
        }
      };
      throw new TypeError(J ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(FindMathML, "__esModule", { value: !0 }), FindMathML.FindMathML = void 0;
    var H = requireFindMath(), V = "http://www.w3.org/1998/Math/MathML", q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.prototype.findMath = function(Z) {
        var te = /* @__PURE__ */ new Set();
        this.findMathNodes(Z, te), this.findMathPrefixed(Z, te);
        var ne = this.adaptor.root(this.adaptor.document);
        return this.adaptor.kind(ne) === "html" && te.size === 0 && this.findMathNS(Z, te), this.processMath(te);
      }, J.prototype.findMathNodes = function(Z, te) {
        var ne, ee;
        try {
          for (var ae = _(this.adaptor.tags(Z, "math")), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            te.add(se);
          }
        } catch (ce) {
          ne = { error: ce };
        } finally {
          try {
            oe && !oe.done && (ee = ae.return) && ee.call(ae);
          } finally {
            if (ne) throw ne.error;
          }
        }
      }, J.prototype.findMathPrefixed = function(Z, te) {
        var ne, ee, ae, oe, se = this.adaptor.root(this.adaptor.document);
        try {
          for (var ce = _(this.adaptor.allAttributes(se)), le = ce.next(); !le.done; le = ce.next()) {
            var ue = le.value;
            if (ue.name.substr(0, 6) === "xmlns:" && ue.value === V) {
              var de = ue.name.substr(6);
              try {
                for (var fe = (ae = void 0, _(this.adaptor.tags(Z, de + ":math"))), he = fe.next(); !he.done; he = fe.next()) {
                  var me = he.value;
                  te.add(me);
                }
              } catch (pe) {
                ae = { error: pe };
              } finally {
                try {
                  he && !he.done && (oe = fe.return) && oe.call(fe);
                } finally {
                  if (ae) throw ae.error;
                }
              }
            }
          }
        } catch (pe) {
          ne = { error: pe };
        } finally {
          try {
            le && !le.done && (ee = ce.return) && ee.call(ce);
          } finally {
            if (ne) throw ne.error;
          }
        }
      }, J.prototype.findMathNS = function(Z, te) {
        var ne, ee;
        try {
          for (var ae = _(this.adaptor.tags(Z, "math", V)), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            te.add(se);
          }
        } catch (ce) {
          ne = { error: ce };
        } finally {
          try {
            oe && !oe.done && (ee = ae.return) && ee.call(ae);
          } finally {
            if (ne) throw ne.error;
          }
        }
      }, J.prototype.processMath = function(Z) {
        var te, ne, ee = [];
        try {
          for (var ae = _(Array.from(Z)), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value, ce = this.adaptor.getAttribute(se, "display") === "block" || this.adaptor.getAttribute(se, "mode") === "display", le = { node: se, n: 0, delim: "" }, ue = { node: se, n: 0, delim: "" };
            ee.push({ math: this.adaptor.outerHTML(se), start: le, end: ue, display: ce });
          }
        } catch (de) {
          te = { error: de };
        } finally {
          try {
            oe && !oe.done && (ne = ae.return) && ne.call(ae);
          } finally {
            if (te) throw te.error;
          }
        }
        return ee;
      }, J.OPTIONS = {}, J;
    }(H.AbstractFindMath);
    return FindMathML.FindMathML = q, FindMathML;
  }
  var MathMLCompile = {}, hasRequiredMathMLCompile;
  function requireMathMLCompile() {
    if (hasRequiredMathMLCompile) return MathMLCompile;
    hasRequiredMathMLCompile = 1;
    var O = MathMLCompile.__assign || function() {
      return O = Object.assign || function(ne) {
        for (var ee, ae = 1, oe = arguments.length; ae < oe; ae++) {
          ee = arguments[ae];
          for (var se in ee) Object.prototype.hasOwnProperty.call(ee, se) && (ne[se] = ee[se]);
        }
        return ne;
      }, O.apply(this, arguments);
    }, _ = MathMLCompile.__createBinding || (Object.create ? function(ne, ee, ae, oe) {
      oe === void 0 && (oe = ae);
      var se = Object.getOwnPropertyDescriptor(ee, ae);
      (!se || ("get" in se ? !ee.__esModule : se.writable || se.configurable)) && (se = { enumerable: !0, get: function() {
        return ee[ae];
      } }), Object.defineProperty(ne, oe, se);
    } : function(ne, ee, ae, oe) {
      oe === void 0 && (oe = ae), ne[oe] = ee[ae];
    }), H = MathMLCompile.__setModuleDefault || (Object.create ? function(ne, ee) {
      Object.defineProperty(ne, "default", { enumerable: !0, value: ee });
    } : function(ne, ee) {
      ne.default = ee;
    }), V = MathMLCompile.__importStar || function(ne) {
      if (ne && ne.__esModule) return ne;
      var ee = {};
      if (ne != null) for (var ae in ne) ae !== "default" && Object.prototype.hasOwnProperty.call(ne, ae) && _(ee, ne, ae);
      return H(ee, ne), ee;
    }, q = MathMLCompile.__values || function(ne) {
      var ee = typeof Symbol == "function" && Symbol.iterator, ae = ee && ne[ee], oe = 0;
      if (ae) return ae.call(ne);
      if (ne && typeof ne.length == "number") return {
        next: function() {
          return ne && oe >= ne.length && (ne = void 0), { value: ne && ne[oe++], done: !ne };
        }
      };
      throw new TypeError(ee ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(MathMLCompile, "__esModule", { value: !0 }), MathMLCompile.MathMLCompile = void 0;
    var G = requireMmlNode(), J = requireOptions(), Z = V(requireEntities()), te = function() {
      function ne(ee) {
        ee === void 0 && (ee = {});
        var ae = this.constructor;
        this.options = (0, J.userOptions)((0, J.defaultOptions)({}, ae.OPTIONS), ee);
      }
      return ne.prototype.setMmlFactory = function(ee) {
        this.factory = ee;
      }, ne.prototype.compile = function(ee) {
        var ae = this.makeNode(ee);
        return ae.verifyTree(this.options.verify), ae.setInheritedAttributes({}, !1, 0, !1), ae.walkTree(this.markMrows), ae;
      }, ne.prototype.makeNode = function(ee) {
        var ae, oe, se = this.adaptor, ce = !1, le = se.kind(ee).replace(/^.*:/, ""), ue = se.getAttribute(ee, "data-mjx-texclass") || "";
        ue && (ue = this.filterAttribute("data-mjx-texclass", ue) || "");
        var de = ue && le === "mrow" ? "TeXAtom" : le;
        try {
          for (var fe = q(this.filterClassList(se.allClasses(ee))), he = fe.next(); !he.done; he = fe.next()) {
            var me = he.value;
            me.match(/^MJX-TeXAtom-/) && le === "mrow" ? (ue = me.substr(12), de = "TeXAtom") : me === "MJX-fixedlimits" && (ce = !0);
          }
        } catch (ye) {
          ae = { error: ye };
        } finally {
          try {
            he && !he.done && (oe = fe.return) && oe.call(fe);
          } finally {
            if (ae) throw ae.error;
          }
        }
        this.factory.getNodeClass(de) || this.error('Unknown node type "' + de + '"');
        var pe = this.factory.create(de);
        return de === "TeXAtom" && ue === "OP" && !ce && (pe.setProperty("movesupsub", !0), pe.attributes.setInherited("movablelimits", !0)), ue && (pe.texClass = G.TEXCLASS[ue], pe.setProperty("texClass", pe.texClass)), this.addAttributes(pe, ee), this.checkClass(pe, ee), this.addChildren(pe, ee), pe;
      }, ne.prototype.addAttributes = function(ee, ae) {
        var oe, se, ce = !1;
        try {
          for (var le = q(this.adaptor.allAttributes(ae)), ue = le.next(); !ue.done; ue = le.next()) {
            var de = ue.value, fe = de.name, he = this.filterAttribute(fe, de.value);
            if (!(he === null || fe === "xmlns")) {
              if (fe.substr(0, 9) === "data-mjx-")
                switch (fe.substr(9)) {
                  case "alternate":
                    ee.setProperty("variantForm", !0);
                    break;
                  case "variant":
                    ee.attributes.set("mathvariant", he), ce = !0;
                    break;
                  case "smallmatrix":
                    ee.setProperty("scriptlevel", 1), ee.setProperty("useHeight", !1);
                    break;
                  case "accent":
                    ee.setProperty("mathaccent", he === "true");
                    break;
                  case "auto-op":
                    ee.setProperty("autoOP", he === "true");
                    break;
                  case "script-align":
                    ee.setProperty("scriptalign", he);
                    break;
                }
              else if (fe !== "class") {
                var me = he.toLowerCase();
                me === "true" || me === "false" ? ee.attributes.set(fe, me === "true") : (!ce || fe !== "mathvariant") && ee.attributes.set(fe, he);
              }
            }
          }
        } catch (pe) {
          oe = { error: pe };
        } finally {
          try {
            ue && !ue.done && (se = le.return) && se.call(le);
          } finally {
            if (oe) throw oe.error;
          }
        }
      }, ne.prototype.filterAttribute = function(ee, ae) {
        return ae;
      }, ne.prototype.filterClassList = function(ee) {
        return ee;
      }, ne.prototype.addChildren = function(ee, ae) {
        var oe, se;
        if (ee.arity !== 0) {
          var ce = this.adaptor;
          try {
            for (var le = q(ce.childNodes(ae)), ue = le.next(); !ue.done; ue = le.next()) {
              var de = ue.value, fe = ce.kind(de);
              if (fe !== "#comment")
                if (fe === "#text")
                  this.addText(ee, de);
                else if (ee.isKind("annotation-xml"))
                  ee.appendChild(this.factory.create("XML").setXML(de, ce));
                else {
                  var he = ee.appendChild(this.makeNode(de));
                  he.arity === 0 && ce.childNodes(de).length && (this.options.fixMisplacedChildren ? this.addChildren(ee, de) : he.mError("There should not be children for " + he.kind + " nodes", this.options.verify, !0));
                }
            }
          } catch (me) {
            oe = { error: me };
          } finally {
            try {
              ue && !ue.done && (se = le.return) && se.call(le);
            } finally {
              if (oe) throw oe.error;
            }
          }
        }
      }, ne.prototype.addText = function(ee, ae) {
        var oe = this.adaptor.value(ae);
        (ee.isToken || ee.getProperty("isChars")) && ee.arity ? (ee.isToken && (oe = Z.translate(oe), oe = this.trimSpace(oe)), ee.appendChild(this.factory.create("text").setText(oe))) : oe.match(/\S/) && this.error('Unexpected text node "' + oe + '"');
      }, ne.prototype.checkClass = function(ee, ae) {
        var oe, se, ce = [];
        try {
          for (var le = q(this.filterClassList(this.adaptor.allClasses(ae))), ue = le.next(); !ue.done; ue = le.next()) {
            var de = ue.value;
            de.substr(0, 4) === "MJX-" ? de === "MJX-variant" ? ee.setProperty("variantForm", !0) : de.substr(0, 11) !== "MJX-TeXAtom" && ee.attributes.set("mathvariant", this.fixCalligraphic(de.substr(3))) : ce.push(de);
          }
        } catch (fe) {
          oe = { error: fe };
        } finally {
          try {
            ue && !ue.done && (se = le.return) && se.call(le);
          } finally {
            if (oe) throw oe.error;
          }
        }
        ce.length && ee.attributes.set("class", ce.join(" "));
      }, ne.prototype.fixCalligraphic = function(ee) {
        return ee.replace(/caligraphic/, "calligraphic");
      }, ne.prototype.markMrows = function(ee) {
        if (ee.isKind("mrow") && !ee.isInferred && ee.childNodes.length >= 2) {
          var ae = ee.childNodes[0], oe = ee.childNodes[ee.childNodes.length - 1];
          ae.isKind("mo") && ae.attributes.get("fence") && ae.attributes.get("stretchy") && oe.isKind("mo") && oe.attributes.get("fence") && oe.attributes.get("stretchy") && (ae.childNodes.length && ee.setProperty("open", ae.getText()), oe.childNodes.length && ee.setProperty("close", oe.getText()));
        }
      }, ne.prototype.trimSpace = function(ee) {
        return ee.replace(/[\t\n\r]/g, " ").replace(/^ +/, "").replace(/ +$/, "").replace(/  +/g, " ");
      }, ne.prototype.error = function(ee) {
        throw new Error(ee);
      }, ne.OPTIONS = {
        MmlFactory: null,
        fixMisplacedChildren: !0,
        verify: O({}, G.AbstractMmlNode.verifyDefaults),
        translateEntities: !0
      }, ne;
    }();
    return MathMLCompile.MathMLCompile = te, MathMLCompile;
  }
  var hasRequiredMathml;
  function requireMathml() {
    if (hasRequiredMathml) return mathml;
    hasRequiredMathml = 1;
    var O = mathml.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = mathml.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    };
    Object.defineProperty(mathml, "__esModule", { value: !0 }), mathml.MathML = void 0;
    var H = requireInputJax(), V = requireOptions(), q = requireFunctionList(), G = requireFindMathML(), J = requireMathMLCompile(), Z = function(te) {
      O(ne, te);
      function ne(ee) {
        ee === void 0 && (ee = {});
        var ae = this, oe = _((0, V.separateOptions)(ee, G.FindMathML.OPTIONS, J.MathMLCompile.OPTIONS), 3), se = oe[0], ce = oe[1], le = oe[2];
        return ae = te.call(this, se) || this, ae.findMathML = ae.options.FindMathML || new G.FindMathML(ce), ae.mathml = ae.options.MathMLCompile || new J.MathMLCompile(le), ae.mmlFilters = new q.FunctionList(), ae;
      }
      return ne.prototype.setAdaptor = function(ee) {
        te.prototype.setAdaptor.call(this, ee), this.findMathML.adaptor = ee, this.mathml.adaptor = ee;
      }, ne.prototype.setMmlFactory = function(ee) {
        te.prototype.setMmlFactory.call(this, ee), this.mathml.setMmlFactory(ee);
      }, Object.defineProperty(ne.prototype, "processStrings", {
        get: function() {
          return !1;
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.compile = function(ee, ae) {
        var oe = ee.start.node;
        if (!oe || !ee.end.node || this.options.forceReparse || this.adaptor.kind(oe) === "#text") {
          var se = this.executeFilters(this.preFilters, ee, ae, (ee.math || "<math></math>").trim()), ce = this.checkForErrors(this.adaptor.parse(se, "text/" + this.options.parseAs)), le = this.adaptor.body(ce);
          this.adaptor.childNodes(le).length !== 1 && this.error("MathML must consist of a single element"), oe = this.adaptor.remove(this.adaptor.firstChild(le)), this.adaptor.kind(oe).replace(/^[a-z]+:/, "") !== "math" && this.error("MathML must be formed by a <math> element, not <" + this.adaptor.kind(oe) + ">");
        }
        return oe = this.executeFilters(this.mmlFilters, ee, ae, oe), this.executeFilters(this.postFilters, ee, ae, this.mathml.compile(oe));
      }, ne.prototype.checkForErrors = function(ee) {
        var ae = this.adaptor.tags(this.adaptor.body(ee), "parsererror")[0];
        return ae && (this.adaptor.textContent(ae) === "" && this.error("Error processing MathML"), this.options.parseError.call(this, ae)), ee;
      }, ne.prototype.error = function(ee) {
        throw new Error(ee);
      }, ne.prototype.findMath = function(ee) {
        return this.findMathML.findMath(ee);
      }, ne.NAME = "MathML", ne.OPTIONS = (0, V.defaultOptions)({
        parseAs: "html",
        forceReparse: !1,
        FindMathML: null,
        MathMLCompile: null,
        parseError: function(ee) {
          this.error(this.adaptor.textContent(ee).replace(/\n.*/g, ""));
        }
      }, H.AbstractInputJax.OPTIONS), ne;
    }(H.AbstractInputJax);
    return mathml.MathML = Z, mathml;
  }
  var mathmlExports = requireMathml(), svg$2 = {}, OutputJax$1 = {}, OutputJax = {}, hasRequiredOutputJax$1;
  function requireOutputJax$1() {
    if (hasRequiredOutputJax$1) return OutputJax;
    hasRequiredOutputJax$1 = 1, Object.defineProperty(OutputJax, "__esModule", { value: !0 }), OutputJax.AbstractOutputJax = void 0;
    var O = requireOptions(), _ = requireFunctionList(), H = function() {
      function V(q) {
        q === void 0 && (q = {}), this.adaptor = null;
        var G = this.constructor;
        this.options = (0, O.userOptions)((0, O.defaultOptions)({}, G.OPTIONS), q), this.postFilters = new _.FunctionList();
      }
      return Object.defineProperty(V.prototype, "name", {
        get: function() {
          return this.constructor.NAME;
        },
        enumerable: !1,
        configurable: !0
      }), V.prototype.setAdaptor = function(q) {
        this.adaptor = q;
      }, V.prototype.initialize = function() {
      }, V.prototype.reset = function() {
      }, V.prototype.getMetrics = function(q) {
      }, V.prototype.styleSheet = function(q) {
        return null;
      }, V.prototype.pageElements = function(q) {
        return null;
      }, V.prototype.executeFilters = function(q, G, J, Z) {
        var te = { math: G, document: J, data: Z };
        return q.execute(te), te.data;
      }, V.NAME = "generic", V.OPTIONS = {}, V;
    }();
    return OutputJax.AbstractOutputJax = H, OutputJax;
  }
  var Styles = {}, hasRequiredStyles;
  function requireStyles() {
    if (hasRequiredStyles) return Styles;
    hasRequiredStyles = 1;
    var O = Styles.__values || function(fe) {
      var he = typeof Symbol == "function" && Symbol.iterator, me = he && fe[he], pe = 0;
      if (me) return me.call(fe);
      if (fe && typeof fe.length == "number") return {
        next: function() {
          return fe && pe >= fe.length && (fe = void 0), { value: fe && fe[pe++], done: !fe };
        }
      };
      throw new TypeError(he ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = Styles.__read || function(fe, he) {
      var me = typeof Symbol == "function" && fe[Symbol.iterator];
      if (!me) return fe;
      var pe = me.call(fe), ye, ge = [], ve;
      try {
        for (; (he === void 0 || he-- > 0) && !(ye = pe.next()).done; ) ge.push(ye.value);
      } catch (be) {
        ve = { error: be };
      } finally {
        try {
          ye && !ye.done && (me = pe.return) && me.call(pe);
        } finally {
          if (ve) throw ve.error;
        }
      }
      return ge;
    }, H = Styles.__spreadArray || function(fe, he, me) {
      if (me || arguments.length === 2) for (var pe = 0, ye = he.length, ge; pe < ye; pe++)
        (ge || !(pe in he)) && (ge || (ge = Array.prototype.slice.call(he, 0, pe)), ge[pe] = he[pe]);
      return fe.concat(ge || Array.prototype.slice.call(he));
    };
    Object.defineProperty(Styles, "__esModule", { value: !0 }), Styles.Styles = void 0;
    var V = ["top", "right", "bottom", "left"], q = ["width", "style", "color"];
    function G(fe) {
      for (var he = fe.split(/((?:'[^']*'|"[^"]*"|,[\s\n]|[^\s\n])*)/g), me = []; he.length > 1; )
        he.shift(), me.push(he.shift());
      return me;
    }
    function J(fe) {
      var he, me, pe = G(this.styles[fe]);
      pe.length === 0 && pe.push(""), pe.length === 1 && pe.push(pe[0]), pe.length === 2 && pe.push(pe[0]), pe.length === 3 && pe.push(pe[1]);
      try {
        for (var ye = O(de.connect[fe].children), ge = ye.next(); !ge.done; ge = ye.next()) {
          var ve = ge.value;
          this.setStyle(this.childName(fe, ve), pe.shift());
        }
      } catch (be) {
        he = { error: be };
      } finally {
        try {
          ge && !ge.done && (me = ye.return) && me.call(ye);
        } finally {
          if (he) throw he.error;
        }
      }
    }
    function Z(fe) {
      var he, me, pe = de.connect[fe].children, ye = [];
      try {
        for (var ge = O(pe), ve = ge.next(); !ve.done; ve = ge.next()) {
          var be = ve.value, Ce = this.styles[fe + "-" + be];
          if (!Ce) {
            delete this.styles[fe];
            return;
          }
          ye.push(Ce);
        }
      } catch (Se) {
        he = { error: Se };
      } finally {
        try {
          ve && !ve.done && (me = ge.return) && me.call(ge);
        } finally {
          if (he) throw he.error;
        }
      }
      ye[3] === ye[1] && (ye.pop(), ye[2] === ye[0] && (ye.pop(), ye[1] === ye[0] && ye.pop())), this.styles[fe] = ye.join(" ");
    }
    function te(fe) {
      var he, me;
      try {
        for (var pe = O(de.connect[fe].children), ye = pe.next(); !ye.done; ye = pe.next()) {
          var ge = ye.value;
          this.setStyle(this.childName(fe, ge), this.styles[fe]);
        }
      } catch (ve) {
        he = { error: ve };
      } finally {
        try {
          ye && !ye.done && (me = pe.return) && me.call(pe);
        } finally {
          if (he) throw he.error;
        }
      }
    }
    function ne(fe) {
      var he, me, pe = H([], _(de.connect[fe].children), !1), ye = this.styles[this.childName(fe, pe.shift())];
      try {
        for (var ge = O(pe), ve = ge.next(); !ve.done; ve = ge.next()) {
          var be = ve.value;
          if (this.styles[this.childName(fe, be)] !== ye) {
            delete this.styles[fe];
            return;
          }
        }
      } catch (Ce) {
        he = { error: Ce };
      } finally {
        try {
          ve && !ve.done && (me = ge.return) && me.call(ge);
        } finally {
          if (he) throw he.error;
        }
      }
      this.styles[fe] = ye;
    }
    var ee = {
      width: /^(?:[\d.]+(?:[a-z]+)|thin|medium|thick|inherit|initial|unset)$/,
      style: /^(?:none|hidden|dotted|dashed|solid|double|groove|ridge|inset|outset|inherit|initial|unset)$/
    };
    function ae(fe) {
      var he, me, pe, ye, ge = { width: "", style: "", color: "" };
      try {
        for (var ve = O(G(this.styles[fe])), be = ve.next(); !be.done; be = ve.next()) {
          var Ce = be.value;
          Ce.match(ee.width) && ge.width === "" ? ge.width = Ce : Ce.match(ee.style) && ge.style === "" ? ge.style = Ce : ge.color = Ce;
        }
      } catch (xe) {
        he = { error: xe };
      } finally {
        try {
          be && !be.done && (me = ve.return) && me.call(ve);
        } finally {
          if (he) throw he.error;
        }
      }
      try {
        for (var Se = O(de.connect[fe].children), we = Se.next(); !we.done; we = Se.next()) {
          var Ee = we.value;
          this.setStyle(this.childName(fe, Ee), ge[Ee]);
        }
      } catch (xe) {
        pe = { error: xe };
      } finally {
        try {
          we && !we.done && (ye = Se.return) && ye.call(Se);
        } finally {
          if (pe) throw pe.error;
        }
      }
    }
    function oe(fe) {
      var he, me, pe = [];
      try {
        for (var ye = O(de.connect[fe].children), ge = ye.next(); !ge.done; ge = ye.next()) {
          var ve = ge.value, be = this.styles[this.childName(fe, ve)];
          be && pe.push(be);
        }
      } catch (Ce) {
        he = { error: Ce };
      } finally {
        try {
          ge && !ge.done && (me = ye.return) && me.call(ye);
        } finally {
          if (he) throw he.error;
        }
      }
      pe.length ? this.styles[fe] = pe.join(" ") : delete this.styles[fe];
    }
    var se = {
      style: /^(?:normal|italic|oblique|inherit|initial|unset)$/,
      variant: new RegExp("^(?:" + [
        "normal|none",
        "inherit|initial|unset",
        "common-ligatures|no-common-ligatures",
        "discretionary-ligatures|no-discretionary-ligatures",
        "historical-ligatures|no-historical-ligatures",
        "contextual|no-contextual",
        "(?:stylistic|character-variant|swash|ornaments|annotation)\\([^)]*\\)",
        "small-caps|all-small-caps|petite-caps|all-petite-caps|unicase|titling-caps",
        "lining-nums|oldstyle-nums|proportional-nums|tabular-nums",
        "diagonal-fractions|stacked-fractions",
        "ordinal|slashed-zero",
        "jis78|jis83|jis90|jis04|simplified|traditional",
        "full-width|proportional-width",
        "ruby"
      ].join("|") + ")$"),
      weight: /^(?:normal|bold|bolder|lighter|[1-9]00|inherit|initial|unset)$/,
      stretch: new RegExp("^(?:" + [
        "normal",
        "(?:(?:ultra|extra|semi)-)?condensed",
        "(?:(?:semi|extra|ulta)-)?expanded",
        "inherit|initial|unset"
      ].join("|") + ")$"),
      size: new RegExp("^(?:" + [
        "xx-small|x-small|small|medium|large|x-large|xx-large|larger|smaller",
        "[d.]+%|[d.]+[a-z]+",
        "inherit|initial|unset"
      ].join("|") + ")(?:/(?:normal|[d.+](?:%|[a-z]+)?))?$")
    };
    function ce(fe) {
      var he, me, pe, ye, ge = G(this.styles[fe]), ve = {
        style: "",
        variant: [],
        weight: "",
        stretch: "",
        size: "",
        family: "",
        "line-height": ""
      };
      try {
        for (var be = O(ge), Ce = be.next(); !Ce.done; Ce = be.next()) {
          var Se = Ce.value;
          ve.family = Se;
          try {
            for (var we = (pe = void 0, O(Object.keys(se))), Ee = we.next(); !Ee.done; Ee = we.next()) {
              var xe = Ee.value;
              if ((Array.isArray(ve[xe]) || ve[xe] === "") && Se.match(se[xe]))
                if (xe === "size") {
                  var Le = _(Se.split(/\//), 2), Qe = Le[0], ke = Le[1];
                  ve[xe] = Qe, ke && (ve["line-height"] = ke);
                } else ve.size === "" && (Array.isArray(ve[xe]) ? ve[xe].push(Se) : ve[xe] = Se);
            }
          } catch (Me) {
            pe = { error: Me };
          } finally {
            try {
              Ee && !Ee.done && (ye = we.return) && ye.call(we);
            } finally {
              if (pe) throw pe.error;
            }
          }
        }
      } catch (Me) {
        he = { error: Me };
      } finally {
        try {
          Ce && !Ce.done && (me = be.return) && me.call(be);
        } finally {
          if (he) throw he.error;
        }
      }
      le(fe, ve), delete this.styles[fe];
    }
    function le(fe, he) {
      var me, pe;
      try {
        for (var ye = O(de.connect[fe].children), ge = ye.next(); !ge.done; ge = ye.next()) {
          var ve = ge.value, be = this.childName(fe, ve);
          if (Array.isArray(he[ve])) {
            var Ce = he[ve];
            Ce.length && (this.styles[be] = Ce.join(" "));
          } else he[ve] !== "" && (this.styles[be] = he[ve]);
        }
      } catch (Se) {
        me = { error: Se };
      } finally {
        try {
          ge && !ge.done && (pe = ye.return) && pe.call(ye);
        } finally {
          if (me) throw me.error;
        }
      }
    }
    function ue(fe) {
    }
    var de = function() {
      function fe(he) {
        he === void 0 && (he = ""), this.parse(he);
      }
      return Object.defineProperty(fe.prototype, "cssText", {
        get: function() {
          var he, me, pe = [];
          try {
            for (var ye = O(Object.keys(this.styles)), ge = ye.next(); !ge.done; ge = ye.next()) {
              var ve = ge.value, be = this.parentName(ve);
              this.styles[be] || pe.push(ve + ": " + this.styles[ve] + ";");
            }
          } catch (Ce) {
            he = { error: Ce };
          } finally {
            try {
              ge && !ge.done && (me = ye.return) && me.call(ye);
            } finally {
              if (he) throw he.error;
            }
          }
          return pe.join(" ");
        },
        enumerable: !1,
        configurable: !0
      }), fe.prototype.set = function(he, me) {
        for (he = this.normalizeName(he), this.setStyle(he, me), fe.connect[he] && !fe.connect[he].combine && (this.combineChildren(he), delete this.styles[he]); he.match(/-/) && (he = this.parentName(he), !!fe.connect[he]); )
          fe.connect[he].combine.call(this, he);
      }, fe.prototype.get = function(he) {
        return he = this.normalizeName(he), this.styles.hasOwnProperty(he) ? this.styles[he] : "";
      }, fe.prototype.setStyle = function(he, me) {
        this.styles[he] = me, fe.connect[he] && fe.connect[he].children && fe.connect[he].split.call(this, he), me === "" && delete this.styles[he];
      }, fe.prototype.combineChildren = function(he) {
        var me, pe, ye = this.parentName(he);
        try {
          for (var ge = O(fe.connect[he].children), ve = ge.next(); !ve.done; ve = ge.next()) {
            var be = ve.value, Ce = this.childName(ye, be);
            fe.connect[Ce].combine.call(this, Ce);
          }
        } catch (Se) {
          me = { error: Se };
        } finally {
          try {
            ve && !ve.done && (pe = ge.return) && pe.call(ge);
          } finally {
            if (me) throw me.error;
          }
        }
      }, fe.prototype.parentName = function(he) {
        var me = he.replace(/-[^-]*$/, "");
        return he === me ? "" : me;
      }, fe.prototype.childName = function(he, me) {
        return me.match(/-/) ? me : (fe.connect[he] && !fe.connect[he].combine && (me += he.replace(/.*-/, "-"), he = this.parentName(he)), he + "-" + me);
      }, fe.prototype.normalizeName = function(he) {
        return he.replace(/[A-Z]/g, function(me) {
          return "-" + me.toLowerCase();
        });
      }, fe.prototype.parse = function(he) {
        he === void 0 && (he = "");
        var me = this.constructor.pattern;
        this.styles = {};
        for (var pe = he.replace(me.comment, "").split(me.style); pe.length > 1; ) {
          var ye = _(pe.splice(0, 3), 3), ge = ye[0], ve = ye[1], be = ye[2];
          if (ge.match(/[^\s\n]/))
            return;
          this.set(ve, be);
        }
      }, fe.pattern = {
        style: /([-a-z]+)[\s\n]*:[\s\n]*((?:'[^']*'|"[^"]*"|\n|.)*?)[\s\n]*(?:;|$)/g,
        comment: /\/\*[^]*?\*\//g
      }, fe.connect = {
        padding: {
          children: V,
          split: J,
          combine: Z
        },
        border: {
          children: V,
          split: te,
          combine: ne
        },
        "border-top": {
          children: q,
          split: ae,
          combine: oe
        },
        "border-right": {
          children: q,
          split: ae,
          combine: oe
        },
        "border-bottom": {
          children: q,
          split: ae,
          combine: oe
        },
        "border-left": {
          children: q,
          split: ae,
          combine: oe
        },
        "border-width": {
          children: V,
          split: J,
          combine: null
        },
        "border-style": {
          children: V,
          split: J,
          combine: null
        },
        "border-color": {
          children: V,
          split: J,
          combine: null
        },
        font: {
          children: ["style", "variant", "weight", "stretch", "line-height", "size", "family"],
          split: ce,
          combine: ue
        }
      }, fe;
    }();
    return Styles.Styles = de, Styles;
  }
  var StyleList = {}, hasRequiredStyleList;
  function requireStyleList() {
    if (hasRequiredStyleList) return StyleList;
    hasRequiredStyleList = 1;
    var O = StyleList.__values || function(H) {
      var V = typeof Symbol == "function" && Symbol.iterator, q = V && H[V], G = 0;
      if (q) return q.call(H);
      if (H && typeof H.length == "number") return {
        next: function() {
          return H && G >= H.length && (H = void 0), { value: H && H[G++], done: !H };
        }
      };
      throw new TypeError(V ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(StyleList, "__esModule", { value: !0 }), StyleList.CssStyles = void 0;
    var _ = function() {
      function H(V) {
        V === void 0 && (V = null), this.styles = {}, this.addStyles(V);
      }
      return Object.defineProperty(H.prototype, "cssText", {
        get: function() {
          return this.getStyleString();
        },
        enumerable: !1,
        configurable: !0
      }), H.prototype.addStyles = function(V) {
        var q, G;
        if (V)
          try {
            for (var J = O(Object.keys(V)), Z = J.next(); !Z.done; Z = J.next()) {
              var te = Z.value;
              this.styles[te] || (this.styles[te] = {}), Object.assign(this.styles[te], V[te]);
            }
          } catch (ne) {
            q = { error: ne };
          } finally {
            try {
              Z && !Z.done && (G = J.return) && G.call(J);
            } finally {
              if (q) throw q.error;
            }
          }
      }, H.prototype.removeStyles = function() {
        for (var V, q, G = [], J = 0; J < arguments.length; J++)
          G[J] = arguments[J];
        try {
          for (var Z = O(G), te = Z.next(); !te.done; te = Z.next()) {
            var ne = te.value;
            delete this.styles[ne];
          }
        } catch (ee) {
          V = { error: ee };
        } finally {
          try {
            te && !te.done && (q = Z.return) && q.call(Z);
          } finally {
            if (V) throw V.error;
          }
        }
      }, H.prototype.clear = function() {
        this.styles = {};
      }, H.prototype.getStyleString = function() {
        return this.getStyleRules().join(`

`);
      }, H.prototype.getStyleRules = function() {
        var V, q, G = Object.keys(this.styles), J = new Array(G.length), Z = 0;
        try {
          for (var te = O(G), ne = te.next(); !ne.done; ne = te.next()) {
            var ee = ne.value;
            J[Z++] = ee + ` {
` + this.getStyleDefString(this.styles[ee]) + `
}`;
          }
        } catch (ae) {
          V = { error: ae };
        } finally {
          try {
            ne && !ne.done && (q = te.return) && q.call(te);
          } finally {
            if (V) throw V.error;
          }
        }
        return J;
      }, H.prototype.getStyleDefString = function(V) {
        var q, G, J = Object.keys(V), Z = new Array(J.length), te = 0;
        try {
          for (var ne = O(J), ee = ne.next(); !ee.done; ee = ne.next()) {
            var ae = ee.value;
            Z[te++] = "  " + ae + ": " + V[ae] + ";";
          }
        } catch (oe) {
          q = { error: oe };
        } finally {
          try {
            ee && !ee.done && (G = ne.return) && G.call(ne);
          } finally {
            if (q) throw q.error;
          }
        }
        return Z.join(`
`);
      }, H;
    }();
    return StyleList.CssStyles = _, StyleList;
  }
  var hasRequiredOutputJax;
  function requireOutputJax() {
    if (hasRequiredOutputJax) return OutputJax$1;
    hasRequiredOutputJax = 1;
    var O = OutputJax$1.__extends || /* @__PURE__ */ function() {
      var ae = function(oe, se) {
        return ae = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ce, le) {
          ce.__proto__ = le;
        } || function(ce, le) {
          for (var ue in le) Object.prototype.hasOwnProperty.call(le, ue) && (ce[ue] = le[ue]);
        }, ae(oe, se);
      };
      return function(oe, se) {
        if (typeof se != "function" && se !== null)
          throw new TypeError("Class extends value " + String(se) + " is not a constructor or null");
        ae(oe, se);
        function ce() {
          this.constructor = oe;
        }
        oe.prototype = se === null ? Object.create(se) : (ce.prototype = se.prototype, new ce());
      };
    }(), _ = OutputJax$1.__assign || function() {
      return _ = Object.assign || function(ae) {
        for (var oe, se = 1, ce = arguments.length; se < ce; se++) {
          oe = arguments[se];
          for (var le in oe) Object.prototype.hasOwnProperty.call(oe, le) && (ae[le] = oe[le]);
        }
        return ae;
      }, _.apply(this, arguments);
    }, H = OutputJax$1.__read || function(ae, oe) {
      var se = typeof Symbol == "function" && ae[Symbol.iterator];
      if (!se) return ae;
      var ce = se.call(ae), le, ue = [], de;
      try {
        for (; (oe === void 0 || oe-- > 0) && !(le = ce.next()).done; ) ue.push(le.value);
      } catch (fe) {
        de = { error: fe };
      } finally {
        try {
          le && !le.done && (se = ce.return) && se.call(ce);
        } finally {
          if (de) throw de.error;
        }
      }
      return ue;
    }, V = OutputJax$1.__values || function(ae) {
      var oe = typeof Symbol == "function" && Symbol.iterator, se = oe && ae[oe], ce = 0;
      if (se) return se.call(ae);
      if (ae && typeof ae.length == "number") return {
        next: function() {
          return ae && ce >= ae.length && (ae = void 0), { value: ae && ae[ce++], done: !ae };
        }
      };
      throw new TypeError(oe ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(OutputJax$1, "__esModule", { value: !0 }), OutputJax$1.CommonOutputJax = void 0;
    var q = requireOutputJax$1(), G = requireMathItem(), J = requireOptions(), Z = requireLengths(), te = requireStyles(), ne = requireStyleList(), ee = function(ae) {
      O(oe, ae);
      function oe(se, ce, le) {
        se === void 0 && (se = null), ce === void 0 && (ce = null), le === void 0 && (le = null);
        var ue = this, de = H((0, J.separateOptions)(se, le.OPTIONS), 2), fe = de[0], he = de[1];
        return ue = ae.call(this, fe) || this, ue.factory = ue.options.wrapperFactory || new ce(), ue.factory.jax = ue, ue.cssStyles = ue.options.cssStyles || new ne.CssStyles(), ue.font = ue.options.font || new le(he), ue.unknownCache = /* @__PURE__ */ new Map(), ue;
      }
      return oe.prototype.typeset = function(se, ce) {
        this.setDocument(ce);
        var le = this.createNode();
        return this.toDOM(se, le, ce), le;
      }, oe.prototype.createNode = function() {
        var se = this.constructor.NAME;
        return this.html("mjx-container", { class: "MathJax", jax: se });
      }, oe.prototype.setScale = function(se) {
        var ce = this.math.metrics.scale * this.options.scale;
        ce !== 1 && this.adaptor.setStyle(se, "fontSize", (0, Z.percent)(ce));
      }, oe.prototype.toDOM = function(se, ce, le) {
        le === void 0 && (le = null), this.setDocument(le), this.math = se, this.pxPerEm = se.metrics.ex / this.font.params.x_height, se.root.setTeXclass(null), this.setScale(ce), this.nodeMap = /* @__PURE__ */ new Map(), this.container = ce, this.processMath(se.root, ce), this.nodeMap = null, this.executeFilters(this.postFilters, se, le, ce);
      }, oe.prototype.getBBox = function(se, ce) {
        this.setDocument(ce), this.math = se, se.root.setTeXclass(null), this.nodeMap = /* @__PURE__ */ new Map();
        var le = this.factory.wrap(se.root).getOuterBBox();
        return this.nodeMap = null, le;
      }, oe.prototype.getMetrics = function(se) {
        var ce, le;
        this.setDocument(se);
        var ue = this.adaptor, de = this.getMetricMaps(se);
        try {
          for (var fe = V(se.math), he = fe.next(); !he.done; he = fe.next()) {
            var me = he.value, pe = ue.parent(me.start.node);
            if (me.state() < G.STATE.METRICS && pe) {
              var ye = de[me.display ? 1 : 0], ge = ye.get(pe), ve = ge.em, be = ge.ex, Ce = ge.containerWidth, Se = ge.lineWidth, we = ge.scale, Ee = ge.family;
              me.setMetrics(ve, be, Ce, Se, we), this.options.mtextInheritFont && (me.outputData.mtextFamily = Ee), this.options.merrorInheritFont && (me.outputData.merrorFamily = Ee), me.state(G.STATE.METRICS);
            }
          }
        } catch (xe) {
          ce = { error: xe };
        } finally {
          try {
            he && !he.done && (le = fe.return) && le.call(fe);
          } finally {
            if (ce) throw ce.error;
          }
        }
      }, oe.prototype.getMetricsFor = function(se, ce) {
        var le = this.options.mtextInheritFont || this.options.merrorInheritFont, ue = this.getTestElement(se, ce), de = this.measureMetrics(ue, le);
        return this.adaptor.remove(ue), de;
      }, oe.prototype.getMetricMaps = function(se) {
        var ce, le, ue, de, fe, he, me, pe, ye, ge, ve = this.adaptor, be = [/* @__PURE__ */ new Map(), /* @__PURE__ */ new Map()];
        try {
          for (var Ce = V(se.math), Se = Ce.next(); !Se.done; Se = Ce.next()) {
            var we = Se.value, Ee = ve.parent(we.start.node);
            if (Ee && we.state() < G.STATE.METRICS) {
              var xe = be[we.display ? 1 : 0];
              xe.has(Ee) || xe.set(Ee, this.getTestElement(Ee, we.display));
            }
          }
        } catch (Ze) {
          ce = { error: Ze };
        } finally {
          try {
            Se && !Se.done && (le = Ce.return) && le.call(Ce);
          } finally {
            if (ce) throw ce.error;
          }
        }
        var Le = this.options.mtextInheritFont || this.options.merrorInheritFont, Qe = [/* @__PURE__ */ new Map(), /* @__PURE__ */ new Map()];
        try {
          for (var ke = V(Qe.keys()), Me = ke.next(); !Me.done; Me = ke.next()) {
            var Re = Me.value;
            try {
              for (var De = (fe = void 0, V(be[Re].keys())), Pe = De.next(); !Pe.done; Pe = De.next()) {
                var Ee = Pe.value;
                Qe[Re].set(Ee, this.measureMetrics(be[Re].get(Ee), Le));
              }
            } catch (Ze) {
              fe = { error: Ze };
            } finally {
              try {
                Pe && !Pe.done && (he = De.return) && he.call(De);
              } finally {
                if (fe) throw fe.error;
              }
            }
          }
        } catch (Ze) {
          ue = { error: Ze };
        } finally {
          try {
            Me && !Me.done && (de = ke.return) && de.call(ke);
          } finally {
            if (ue) throw ue.error;
          }
        }
        try {
          for (var $e = V(Qe.keys()), qe = $e.next(); !qe.done; qe = $e.next()) {
            var Re = qe.value;
            try {
              for (var Xe = (ye = void 0, V(be[Re].values())), Ve = Xe.next(); !Ve.done; Ve = Xe.next()) {
                var Ee = Ve.value;
                ve.remove(Ee);
              }
            } catch (Ge) {
              ye = { error: Ge };
            } finally {
              try {
                Ve && !Ve.done && (ge = Xe.return) && ge.call(Xe);
              } finally {
                if (ye) throw ye.error;
              }
            }
          }
        } catch (Ze) {
          me = { error: Ze };
        } finally {
          try {
            qe && !qe.done && (pe = $e.return) && pe.call($e);
          } finally {
            if (me) throw me.error;
          }
        }
        return Qe;
      }, oe.prototype.getTestElement = function(se, ce) {
        var le = this.adaptor;
        if (!this.testInline) {
          this.testInline = this.html("mjx-test", { style: {
            display: "inline-block",
            width: "100%",
            "font-style": "normal",
            "font-weight": "normal",
            "font-size": "100%",
            "font-size-adjust": "none",
            "text-indent": 0,
            "text-transform": "none",
            "letter-spacing": "normal",
            "word-spacing": "normal",
            overflow: "hidden",
            height: "1px",
            "margin-right": "-1px"
          } }, [
            this.html("mjx-left-box", { style: {
              display: "inline-block",
              width: 0,
              float: "left"
            } }),
            this.html("mjx-ex-box", { style: {
              position: "absolute",
              overflow: "hidden",
              width: "1px",
              height: "60ex"
            } }),
            this.html("mjx-right-box", { style: {
              display: "inline-block",
              width: 0,
              float: "right"
            } })
          ]), this.testDisplay = le.clone(this.testInline), le.setStyle(this.testDisplay, "display", "table"), le.setStyle(this.testDisplay, "margin-right", ""), le.setStyle(le.firstChild(this.testDisplay), "display", "none");
          var ue = le.lastChild(this.testDisplay);
          le.setStyle(ue, "display", "table-cell"), le.setStyle(ue, "width", "10000em"), le.setStyle(ue, "float", "");
        }
        return le.append(se, le.clone(ce ? this.testDisplay : this.testInline));
      }, oe.prototype.measureMetrics = function(se, ce) {
        var le = this.adaptor, ue = ce ? le.fontFamily(se) : "", de = le.fontSize(se), fe = H(le.nodeSize(le.childNode(se, 1)), 2), he = fe[0], me = fe[1], pe = he ? me / 60 : de * this.options.exFactor, ye = he ? le.getStyle(se, "display") === "table" ? le.nodeSize(le.lastChild(se))[0] - 1 : le.nodeBBox(le.lastChild(se)).left - le.nodeBBox(le.firstChild(se)).left - 2 : 1e6, ge = Math.max(this.options.minScale, this.options.matchFontHeight ? pe / this.font.params.x_height / de : 1), ve = 1e6;
        return { em: de, ex: pe, containerWidth: ye, lineWidth: ve, scale: ge, family: ue };
      }, oe.prototype.styleSheet = function(se) {
        var ce, le;
        if (this.setDocument(se), this.cssStyles.clear(), this.cssStyles.addStyles(this.constructor.commonStyles), "getStyles" in se)
          try {
            for (var ue = V(se.getStyles()), de = ue.next(); !de.done; de = ue.next()) {
              var fe = de.value;
              this.cssStyles.addStyles(fe);
            }
          } catch (me) {
            ce = { error: me };
          } finally {
            try {
              de && !de.done && (le = ue.return) && le.call(ue);
            } finally {
              if (ce) throw ce.error;
            }
          }
        this.addWrapperStyles(this.cssStyles), this.addFontStyles(this.cssStyles);
        var he = this.html("style", { id: "MJX-styles" }, [this.text(`
` + this.cssStyles.cssText + `
`)]);
        return he;
      }, oe.prototype.addFontStyles = function(se) {
        se.addStyles(this.font.styles);
      }, oe.prototype.addWrapperStyles = function(se) {
        var ce, le;
        try {
          for (var ue = V(this.factory.getKinds()), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value;
            this.addClassStyles(this.factory.getNodeClass(fe), se);
          }
        } catch (he) {
          ce = { error: he };
        } finally {
          try {
            de && !de.done && (le = ue.return) && le.call(ue);
          } finally {
            if (ce) throw ce.error;
          }
        }
      }, oe.prototype.addClassStyles = function(se, ce) {
        ce.addStyles(se.styles);
      }, oe.prototype.setDocument = function(se) {
        se && (this.document = se, this.adaptor.document = se.document);
      }, oe.prototype.html = function(se, ce, le, ue) {
        return ce === void 0 && (ce = {}), le === void 0 && (le = []), this.adaptor.node(se, ce, le, ue);
      }, oe.prototype.text = function(se) {
        return this.adaptor.text(se);
      }, oe.prototype.fixed = function(se, ce) {
        return ce === void 0 && (ce = 3), Math.abs(se) < 6e-4 ? "0" : se.toFixed(ce).replace(/\.?0+$/, "");
      }, oe.prototype.measureText = function(se, ce, le) {
        le === void 0 && (le = ["", !1, !1]);
        var ue = this.unknownText(se, ce);
        if (ce === "-explicitFont") {
          var de = this.cssFontStyles(le);
          this.adaptor.setAttributes(ue, { style: de });
        }
        return this.measureTextNodeWithCache(ue, se, ce, le);
      }, oe.prototype.measureTextNodeWithCache = function(se, ce, le, ue) {
        ue === void 0 && (ue = ["", !1, !1]), le === "-explicitFont" && (le = [ue[0], ue[1] ? "T" : "F", ue[2] ? "T" : "F", ""].join("-")), this.unknownCache.has(le) || this.unknownCache.set(le, /* @__PURE__ */ new Map());
        var de = this.unknownCache.get(le), fe = de.get(ce);
        if (fe)
          return fe;
        var he = this.measureTextNode(se);
        return de.set(ce, he), he;
      }, oe.prototype.measureXMLnode = function(se) {
        var ce = this.adaptor, le = this.html("mjx-xml-block", { style: { display: "inline-block" } }, [ce.clone(se)]), ue = this.html("mjx-baseline", { style: { display: "inline-block", width: 0, height: 0 } }), de = {
          position: "absolute",
          display: "inline-block",
          "font-family": "initial",
          "line-height": "normal"
        }, fe = this.html("mjx-measure-xml", { style: de }, [ue, le]);
        ce.append(ce.parent(this.math.start.node), this.container), ce.append(this.container, fe);
        var he = this.math.metrics.em * this.math.metrics.scale, me = ce.nodeBBox(le), pe = me.left, ye = me.right, ge = me.bottom, ve = me.top, be = (ye - pe) / he, Ce = (ce.nodeBBox(ue).top - ve) / he, Se = (ge - ve) / he - Ce;
        return ce.remove(this.container), ce.remove(fe), { w: be, h: Ce, d: Se };
      }, oe.prototype.cssFontStyles = function(se, ce) {
        ce === void 0 && (ce = {});
        var le = H(se, 3), ue = le[0], de = le[1], fe = le[2];
        return ce["font-family"] = this.font.getFamily(ue), de && (ce["font-style"] = "italic"), fe && (ce["font-weight"] = "bold"), ce;
      }, oe.prototype.getFontData = function(se) {
        return se || (se = new te.Styles()), [
          this.font.getFamily(se.get("font-family")),
          se.get("font-style") === "italic",
          se.get("font-weight") === "bold"
        ];
      }, oe.NAME = "Common", oe.OPTIONS = _(_({}, q.AbstractOutputJax.OPTIONS), { scale: 1, minScale: 0.5, mtextInheritFont: !1, merrorInheritFont: !1, mtextFont: "", merrorFont: "serif", mathmlSpacing: !1, skipAttributes: {}, exFactor: 0.5, displayAlign: "center", displayIndent: "0", wrapperFactory: null, font: null, cssStyles: null }), oe.commonStyles = {}, oe;
    }(q.AbstractOutputJax);
    return OutputJax$1.CommonOutputJax = ee, OutputJax$1;
  }
  var WrapperFactory$2 = {}, WrapperFactory$1 = {}, WrapperFactory = {}, hasRequiredWrapperFactory$2;
  function requireWrapperFactory$2() {
    if (hasRequiredWrapperFactory$2) return WrapperFactory;
    hasRequiredWrapperFactory$2 = 1;
    var O = WrapperFactory.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = WrapperFactory.__read || function(G, J) {
      var Z = typeof Symbol == "function" && G[Symbol.iterator];
      if (!Z) return G;
      var te = Z.call(G), ne, ee = [], ae;
      try {
        for (; (J === void 0 || J-- > 0) && !(ne = te.next()).done; ) ee.push(ne.value);
      } catch (oe) {
        ae = { error: oe };
      } finally {
        try {
          ne && !ne.done && (Z = te.return) && Z.call(te);
        } finally {
          if (ae) throw ae.error;
        }
      }
      return ee;
    }, H = WrapperFactory.__spreadArray || function(G, J, Z) {
      if (Z || arguments.length === 2) for (var te = 0, ne = J.length, ee; te < ne; te++)
        (ee || !(te in J)) && (ee || (ee = Array.prototype.slice.call(J, 0, te)), ee[te] = J[te]);
      return G.concat(ee || Array.prototype.slice.call(J));
    };
    Object.defineProperty(WrapperFactory, "__esModule", { value: !0 }), WrapperFactory.AbstractWrapperFactory = void 0;
    var V = requireFactory(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.prototype.wrap = function(Z) {
        for (var te = [], ne = 1; ne < arguments.length; ne++)
          te[ne - 1] = arguments[ne];
        return this.create.apply(this, H([Z.kind, Z], _(te), !1));
      }, J;
    }(V.AbstractFactory);
    return WrapperFactory.AbstractWrapperFactory = q, WrapperFactory;
  }
  var hasRequiredWrapperFactory$1;
  function requireWrapperFactory$1() {
    if (hasRequiredWrapperFactory$1) return WrapperFactory$1;
    hasRequiredWrapperFactory$1 = 1;
    var O = WrapperFactory$1.__extends || /* @__PURE__ */ function() {
      var V = function(q, G) {
        return V = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(J, Z) {
          J.__proto__ = Z;
        } || function(J, Z) {
          for (var te in Z) Object.prototype.hasOwnProperty.call(Z, te) && (J[te] = Z[te]);
        }, V(q, G);
      };
      return function(q, G) {
        if (typeof G != "function" && G !== null)
          throw new TypeError("Class extends value " + String(G) + " is not a constructor or null");
        V(q, G);
        function J() {
          this.constructor = q;
        }
        q.prototype = G === null ? Object.create(G) : (J.prototype = G.prototype, new J());
      };
    }();
    Object.defineProperty(WrapperFactory$1, "__esModule", { value: !0 }), WrapperFactory$1.CommonWrapperFactory = void 0;
    var _ = requireWrapperFactory$2(), H = function(V) {
      O(q, V);
      function q() {
        var G = V !== null && V.apply(this, arguments) || this;
        return G.jax = null, G;
      }
      return Object.defineProperty(q.prototype, "Wrappers", {
        get: function() {
          return this.node;
        },
        enumerable: !1,
        configurable: !0
      }), q.defaultNodes = {}, q;
    }(_.AbstractWrapperFactory);
    return WrapperFactory$1.CommonWrapperFactory = H, WrapperFactory$1;
  }
  var Wrappers = {}, Wrapper$2 = {}, BBox = {}, hasRequiredBBox;
  function requireBBox() {
    if (hasRequiredBBox) return BBox;
    hasRequiredBBox = 1, Object.defineProperty(BBox, "__esModule", { value: !0 }), BBox.BBox = void 0;
    var O = requireLengths(), _ = function() {
      function H(V) {
        V === void 0 && (V = { w: 0, h: -O.BIGDIMEN, d: -O.BIGDIMEN }), this.w = V.w || 0, this.h = "h" in V ? V.h : -O.BIGDIMEN, this.d = "d" in V ? V.d : -O.BIGDIMEN, this.L = this.R = this.ic = this.sk = this.dx = 0, this.scale = this.rscale = 1, this.pwidth = "";
      }
      return H.zero = function() {
        return new H({ h: 0, d: 0, w: 0 });
      }, H.empty = function() {
        return new H();
      }, H.prototype.empty = function() {
        return this.w = 0, this.h = this.d = -O.BIGDIMEN, this;
      }, H.prototype.clean = function() {
        this.w === -O.BIGDIMEN && (this.w = 0), this.h === -O.BIGDIMEN && (this.h = 0), this.d === -O.BIGDIMEN && (this.d = 0);
      }, H.prototype.rescale = function(V) {
        this.w *= V, this.h *= V, this.d *= V;
      }, H.prototype.combine = function(V, q, G) {
        q === void 0 && (q = 0), G === void 0 && (G = 0);
        var J = V.rscale, Z = q + J * (V.w + V.L + V.R), te = G + J * V.h, ne = J * V.d - G;
        Z > this.w && (this.w = Z), te > this.h && (this.h = te), ne > this.d && (this.d = ne);
      }, H.prototype.append = function(V) {
        var q = V.rscale;
        this.w += q * (V.w + V.L + V.R), q * V.h > this.h && (this.h = q * V.h), q * V.d > this.d && (this.d = q * V.d);
      }, H.prototype.updateFrom = function(V) {
        this.h = V.h, this.d = V.d, this.w = V.w, V.pwidth && (this.pwidth = V.pwidth);
      }, H.fullWidth = "100%", H.StyleAdjust = [
        ["borderTopWidth", "h"],
        ["borderRightWidth", "w"],
        ["borderBottomWidth", "d"],
        ["borderLeftWidth", "w", 0],
        ["paddingTop", "h"],
        ["paddingRight", "w"],
        ["paddingBottom", "d"],
        ["paddingLeft", "w", 0]
      ], H;
    }();
    return BBox.BBox = _, BBox;
  }
  var Wrapper$1 = {}, Wrapper = {}, hasRequiredWrapper$2;
  function requireWrapper$2() {
    if (hasRequiredWrapper$2) return Wrapper;
    hasRequiredWrapper$2 = 1, Object.defineProperty(Wrapper, "__esModule", { value: !0 }), Wrapper.AbstractWrapper = void 0;
    var O = function() {
      function _(H, V) {
        this.factory = H, this.node = V;
      }
      return Object.defineProperty(_.prototype, "kind", {
        get: function() {
          return this.node.kind;
        },
        enumerable: !1,
        configurable: !0
      }), _.prototype.wrap = function(H) {
        return this.factory.wrap(H);
      }, _;
    }();
    return Wrapper.AbstractWrapper = O, Wrapper;
  }
  var FontData$1 = {}, hasRequiredFontData$1;
  function requireFontData$1() {
    if (hasRequiredFontData$1) return FontData$1;
    hasRequiredFontData$1 = 1;
    var O = FontData$1.__assign || function() {
      return O = Object.assign || function(J) {
        for (var Z, te = 1, ne = arguments.length; te < ne; te++) {
          Z = arguments[te];
          for (var ee in Z) Object.prototype.hasOwnProperty.call(Z, ee) && (J[ee] = Z[ee]);
        }
        return J;
      }, O.apply(this, arguments);
    }, _ = FontData$1.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    }, H = FontData$1.__spreadArray || function(J, Z, te) {
      if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
        (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
      return J.concat(ae || Array.prototype.slice.call(Z));
    }, V = FontData$1.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(FontData$1, "__esModule", { value: !0 }), FontData$1.FontData = FontData$1.NOSTRETCH = FontData$1.H = FontData$1.V = void 0;
    var q = requireOptions();
    FontData$1.V = 1, FontData$1.H = 2, FontData$1.NOSTRETCH = { dir: 0 };
    var G = function() {
      function J(Z) {
        var te, ne, ee, ae;
        Z === void 0 && (Z = null), this.variant = {}, this.delimiters = {}, this.cssFontMap = {}, this.remapChars = {}, this.skewIcFactor = 0.75;
        var oe = this.constructor;
        this.options = (0, q.userOptions)((0, q.defaultOptions)({}, oe.OPTIONS), Z), this.params = O({}, oe.defaultParams), this.sizeVariants = H([], _(oe.defaultSizeVariants), !1), this.stretchVariants = H([], _(oe.defaultStretchVariants), !1), this.cssFontMap = O({}, oe.defaultCssFonts);
        try {
          for (var se = V(Object.keys(this.cssFontMap)), ce = se.next(); !ce.done; ce = se.next()) {
            var le = ce.value;
            this.cssFontMap[le][0] === "unknown" && (this.cssFontMap[le][0] = this.options.unknownFamily);
          }
        } catch (he) {
          te = { error: he };
        } finally {
          try {
            ce && !ce.done && (ne = se.return) && ne.call(se);
          } finally {
            if (te) throw te.error;
          }
        }
        this.cssFamilyPrefix = oe.defaultCssFamilyPrefix, this.createVariants(oe.defaultVariants), this.defineDelimiters(oe.defaultDelimiters);
        try {
          for (var ue = V(Object.keys(oe.defaultChars)), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value;
            this.defineChars(fe, oe.defaultChars[fe]);
          }
        } catch (he) {
          ee = { error: he };
        } finally {
          try {
            de && !de.done && (ae = ue.return) && ae.call(ue);
          } finally {
            if (ee) throw ee.error;
          }
        }
        this.defineRemap("accent", oe.defaultAccentMap), this.defineRemap("mo", oe.defaultMoMap), this.defineRemap("mn", oe.defaultMnMap);
      }
      return J.charOptions = function(Z, te) {
        var ne = Z[te];
        return ne.length === 3 && (ne[3] = {}), ne[3];
      }, Object.defineProperty(J.prototype, "styles", {
        get: function() {
          return this._styles;
        },
        set: function(Z) {
          this._styles = Z;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.createVariant = function(Z, te, ne) {
        te === void 0 && (te = null), ne === void 0 && (ne = null);
        var ee = {
          linked: [],
          chars: te ? Object.create(this.variant[te].chars) : {}
        };
        ne && this.variant[ne] && (Object.assign(ee.chars, this.variant[ne].chars), this.variant[ne].linked.push(ee.chars), ee.chars = Object.create(ee.chars)), this.remapSmpChars(ee.chars, Z), this.variant[Z] = ee;
      }, J.prototype.remapSmpChars = function(Z, te) {
        var ne, ee, ae, oe, se = this.constructor;
        if (se.VariantSmp[te]) {
          var ce = se.SmpRemap, le = [null, null, se.SmpRemapGreekU, se.SmpRemapGreekL];
          try {
            for (var ue = V(se.SmpRanges), de = ue.next(); !de.done; de = ue.next()) {
              var fe = _(de.value, 3), he = fe[0], me = fe[1], pe = fe[2], ye = se.VariantSmp[te][he];
              if (ye) {
                for (var ge = me; ge <= pe; ge++)
                  if (ge !== 930) {
                    var ve = ye + ge - me;
                    Z[ge] = this.smpChar(ce[ve] || ve);
                  }
                if (le[he])
                  try {
                    for (var be = (ae = void 0, V(Object.keys(le[he]).map(function(Se) {
                      return parseInt(Se);
                    }))), Ce = be.next(); !Ce.done; Ce = be.next()) {
                      var ge = Ce.value;
                      Z[ge] = this.smpChar(ye + le[he][ge]);
                    }
                  } catch (Se) {
                    ae = { error: Se };
                  } finally {
                    try {
                      Ce && !Ce.done && (oe = be.return) && oe.call(be);
                    } finally {
                      if (ae) throw ae.error;
                    }
                  }
              }
            }
          } catch (Se) {
            ne = { error: Se };
          } finally {
            try {
              de && !de.done && (ee = ue.return) && ee.call(ue);
            } finally {
              if (ne) throw ne.error;
            }
          }
        }
        te === "bold" && (Z[988] = this.smpChar(120778), Z[989] = this.smpChar(120779));
      }, J.prototype.smpChar = function(Z) {
        return [, , , { smp: Z }];
      }, J.prototype.createVariants = function(Z) {
        var te, ne;
        try {
          for (var ee = V(Z), ae = ee.next(); !ae.done; ae = ee.next()) {
            var oe = ae.value;
            this.createVariant(oe[0], oe[1], oe[2]);
          }
        } catch (se) {
          te = { error: se };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (te) throw te.error;
          }
        }
      }, J.prototype.defineChars = function(Z, te) {
        var ne, ee, ae = this.variant[Z];
        Object.assign(ae.chars, te);
        try {
          for (var oe = V(ae.linked), se = oe.next(); !se.done; se = oe.next()) {
            var ce = se.value;
            Object.assign(ce, te);
          }
        } catch (le) {
          ne = { error: le };
        } finally {
          try {
            se && !se.done && (ee = oe.return) && ee.call(oe);
          } finally {
            if (ne) throw ne.error;
          }
        }
      }, J.prototype.defineDelimiters = function(Z) {
        Object.assign(this.delimiters, Z);
      }, J.prototype.defineRemap = function(Z, te) {
        this.remapChars.hasOwnProperty(Z) || (this.remapChars[Z] = {}), Object.assign(this.remapChars[Z], te);
      }, J.prototype.getDelimiter = function(Z) {
        return this.delimiters[Z];
      }, J.prototype.getSizeVariant = function(Z, te) {
        return this.delimiters[Z].variants && (te = this.delimiters[Z].variants[te]), this.sizeVariants[te];
      }, J.prototype.getStretchVariant = function(Z, te) {
        return this.stretchVariants[this.delimiters[Z].stretchv ? this.delimiters[Z].stretchv[te] : 0];
      }, J.prototype.getChar = function(Z, te) {
        return this.variant[Z].chars[te];
      }, J.prototype.getVariant = function(Z) {
        return this.variant[Z];
      }, J.prototype.getCssFont = function(Z) {
        return this.cssFontMap[Z] || ["serif", !1, !1];
      }, J.prototype.getFamily = function(Z) {
        return this.cssFamilyPrefix ? this.cssFamilyPrefix + ", " + Z : Z;
      }, J.prototype.getRemappedChar = function(Z, te) {
        var ne = this.remapChars[Z] || {};
        return ne[te];
      }, J.OPTIONS = {
        unknownFamily: "serif"
      }, J.JAX = "common", J.NAME = "", J.defaultVariants = [
        ["normal"],
        ["bold", "normal"],
        ["italic", "normal"],
        ["bold-italic", "italic", "bold"],
        ["double-struck", "bold"],
        ["fraktur", "normal"],
        ["bold-fraktur", "bold", "fraktur"],
        ["script", "italic"],
        ["bold-script", "bold-italic", "script"],
        ["sans-serif", "normal"],
        ["bold-sans-serif", "bold", "sans-serif"],
        ["sans-serif-italic", "italic", "sans-serif"],
        ["sans-serif-bold-italic", "bold-italic", "bold-sans-serif"],
        ["monospace", "normal"]
      ], J.defaultCssFonts = {
        normal: ["unknown", !1, !1],
        bold: ["unknown", !1, !0],
        italic: ["unknown", !0, !1],
        "bold-italic": ["unknown", !0, !0],
        "double-struck": ["unknown", !1, !0],
        fraktur: ["unknown", !1, !1],
        "bold-fraktur": ["unknown", !1, !0],
        script: ["cursive", !1, !1],
        "bold-script": ["cursive", !1, !0],
        "sans-serif": ["sans-serif", !1, !1],
        "bold-sans-serif": ["sans-serif", !1, !0],
        "sans-serif-italic": ["sans-serif", !0, !1],
        "sans-serif-bold-italic": ["sans-serif", !0, !0],
        monospace: ["monospace", !1, !1]
      }, J.defaultCssFamilyPrefix = "", J.VariantSmp = {
        bold: [119808, 119834, 120488, 120514, 120782],
        italic: [119860, 119886, 120546, 120572],
        "bold-italic": [119912, 119938, 120604, 120630],
        script: [119964, 119990],
        "bold-script": [120016, 120042],
        fraktur: [120068, 120094],
        "double-struck": [120120, 120146, , , 120792],
        "bold-fraktur": [120172, 120198],
        "sans-serif": [120224, 120250, , , 120802],
        "bold-sans-serif": [120276, 120302, 120662, 120688, 120812],
        "sans-serif-italic": [120328, 120354],
        "sans-serif-bold-italic": [120380, 120406, 120720, 120746],
        monospace: [120432, 120458, , , 120822]
      }, J.SmpRanges = [
        [0, 65, 90],
        [1, 97, 122],
        [2, 913, 937],
        [3, 945, 969],
        [4, 48, 57]
      ], J.SmpRemap = {
        119893: 8462,
        119965: 8492,
        119968: 8496,
        119969: 8497,
        119971: 8459,
        119972: 8464,
        119975: 8466,
        119976: 8499,
        119981: 8475,
        119994: 8495,
        119996: 8458,
        120004: 8500,
        120070: 8493,
        120075: 8460,
        120076: 8465,
        120085: 8476,
        120093: 8488,
        120122: 8450,
        120127: 8461,
        120133: 8469,
        120135: 8473,
        120136: 8474,
        120137: 8477,
        120145: 8484
      }, J.SmpRemapGreekU = {
        8711: 25,
        1012: 17
      }, J.SmpRemapGreekL = {
        977: 27,
        981: 29,
        982: 31,
        1008: 28,
        1009: 30,
        1013: 26,
        8706: 25
      }, J.defaultAccentMap = {
        768: "ˋ",
        769: "ˊ",
        770: "ˆ",
        771: "˜",
        772: "ˉ",
        774: "˘",
        775: "˙",
        776: "¨",
        778: "˚",
        780: "ˇ",
        8594: "⃗",
        8242: "'",
        8243: "''",
        8244: "'''",
        8245: "`",
        8246: "``",
        8247: "```",
        8279: "''''",
        8400: "↼",
        8401: "⇀",
        8406: "←",
        8417: "↔",
        8432: "*",
        8411: "...",
        8412: "....",
        8428: "⇁",
        8429: "↽",
        8430: "←",
        8431: "→"
      }, J.defaultMoMap = {
        45: "−"
      }, J.defaultMnMap = {
        45: "−"
      }, J.defaultParams = {
        x_height: 0.442,
        quad: 1,
        num1: 0.676,
        num2: 0.394,
        num3: 0.444,
        denom1: 0.686,
        denom2: 0.345,
        sup1: 0.413,
        sup2: 0.363,
        sup3: 0.289,
        sub1: 0.15,
        sub2: 0.247,
        sup_drop: 0.386,
        sub_drop: 0.05,
        delim1: 2.39,
        delim2: 1,
        axis_height: 0.25,
        rule_thickness: 0.06,
        big_op_spacing1: 0.111,
        big_op_spacing2: 0.167,
        big_op_spacing3: 0.2,
        big_op_spacing4: 0.6,
        big_op_spacing5: 0.1,
        surd_height: 0.075,
        scriptspace: 0.05,
        nulldelimiterspace: 0.12,
        delimiterfactor: 901,
        delimitershortfall: 0.3,
        min_rule_thickness: 1.25,
        separation_factor: 1.75,
        extra_ic: 0.033
      }, J.defaultDelimiters = {}, J.defaultChars = {}, J.defaultSizeVariants = [], J.defaultStretchVariants = [], J;
    }();
    return FontData$1.FontData = G, FontData$1;
  }
  var hasRequiredWrapper$1;
  function requireWrapper$1() {
    if (hasRequiredWrapper$1) return Wrapper$1;
    hasRequiredWrapper$1 = 1;
    var O = Wrapper$1.__extends || /* @__PURE__ */ function() {
      var de = function(fe, he) {
        return de = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(me, pe) {
          me.__proto__ = pe;
        } || function(me, pe) {
          for (var ye in pe) Object.prototype.hasOwnProperty.call(pe, ye) && (me[ye] = pe[ye]);
        }, de(fe, he);
      };
      return function(fe, he) {
        if (typeof he != "function" && he !== null)
          throw new TypeError("Class extends value " + String(he) + " is not a constructor or null");
        de(fe, he);
        function me() {
          this.constructor = fe;
        }
        fe.prototype = he === null ? Object.create(he) : (me.prototype = he.prototype, new me());
      };
    }(), _ = Wrapper$1.__createBinding || (Object.create ? function(de, fe, he, me) {
      me === void 0 && (me = he);
      var pe = Object.getOwnPropertyDescriptor(fe, he);
      (!pe || ("get" in pe ? !fe.__esModule : pe.writable || pe.configurable)) && (pe = { enumerable: !0, get: function() {
        return fe[he];
      } }), Object.defineProperty(de, me, pe);
    } : function(de, fe, he, me) {
      me === void 0 && (me = he), de[me] = fe[he];
    }), H = Wrapper$1.__setModuleDefault || (Object.create ? function(de, fe) {
      Object.defineProperty(de, "default", { enumerable: !0, value: fe });
    } : function(de, fe) {
      de.default = fe;
    }), V = Wrapper$1.__importStar || function(de) {
      if (de && de.__esModule) return de;
      var fe = {};
      if (de != null) for (var he in de) he !== "default" && Object.prototype.hasOwnProperty.call(de, he) && _(fe, de, he);
      return H(fe, de), fe;
    }, q = Wrapper$1.__values || function(de) {
      var fe = typeof Symbol == "function" && Symbol.iterator, he = fe && de[fe], me = 0;
      if (he) return he.call(de);
      if (de && typeof de.length == "number") return {
        next: function() {
          return de && me >= de.length && (de = void 0), { value: de && de[me++], done: !de };
        }
      };
      throw new TypeError(fe ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, G = Wrapper$1.__read || function(de, fe) {
      var he = typeof Symbol == "function" && de[Symbol.iterator];
      if (!he) return de;
      var me = he.call(de), pe, ye = [], ge;
      try {
        for (; (fe === void 0 || fe-- > 0) && !(pe = me.next()).done; ) ye.push(pe.value);
      } catch (ve) {
        ge = { error: ve };
      } finally {
        try {
          pe && !pe.done && (he = me.return) && he.call(me);
        } finally {
          if (ge) throw ge.error;
        }
      }
      return ye;
    }, J = Wrapper$1.__spreadArray || function(de, fe, he) {
      if (he || arguments.length === 2) for (var me = 0, pe = fe.length, ye; me < pe; me++)
        (ye || !(me in fe)) && (ye || (ye = Array.prototype.slice.call(fe, 0, me)), ye[me] = fe[me]);
      return de.concat(ye || Array.prototype.slice.call(fe));
    };
    Object.defineProperty(Wrapper$1, "__esModule", { value: !0 }), Wrapper$1.CommonWrapper = void 0;
    var Z = requireWrapper$2(), te = requireMmlNode(), ne = requireString(), ee = V(requireLengths()), ae = requireStyles(), oe = requireBBox(), se = requireFontData$1(), ce = 2 / 18;
    function le(de, fe) {
      return de ? fe < ce ? 0 : ce : fe;
    }
    var ue = function(de) {
      O(fe, de);
      function fe(he, me, pe) {
        pe === void 0 && (pe = null);
        var ye = de.call(this, he, me) || this;
        return ye.parent = null, ye.removedStyles = null, ye.styles = null, ye.variant = "", ye.bboxComputed = !1, ye.stretch = se.NOSTRETCH, ye.font = null, ye.parent = pe, ye.font = he.jax.font, ye.bbox = oe.BBox.zero(), ye.getStyles(), ye.getVariant(), ye.getScale(), ye.getSpace(), ye.childNodes = me.childNodes.map(function(ge) {
          var ve = ye.wrap(ge);
          return ve.bbox.pwidth && (me.notParent || me.isKind("math")) && (ye.bbox.pwidth = oe.BBox.fullWidth), ve;
        }), ye;
      }
      return Object.defineProperty(fe.prototype, "jax", {
        get: function() {
          return this.factory.jax;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(fe.prototype, "adaptor", {
        get: function() {
          return this.factory.jax.adaptor;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(fe.prototype, "metrics", {
        get: function() {
          return this.factory.jax.math.metrics;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(fe.prototype, "fixesPWidth", {
        get: function() {
          return !this.node.notParent && !this.node.isToken;
        },
        enumerable: !1,
        configurable: !0
      }), fe.prototype.wrap = function(he, me) {
        me === void 0 && (me = null);
        var pe = this.factory.wrap(he, me || this);
        return me && me.childNodes.push(pe), this.jax.nodeMap.set(he, pe), pe;
      }, fe.prototype.getBBox = function(he) {
        if (he === void 0 && (he = !0), this.bboxComputed)
          return this.bbox;
        var me = he ? this.bbox : oe.BBox.zero();
        return this.computeBBox(me), this.bboxComputed = he, me;
      }, fe.prototype.getOuterBBox = function(he) {
        var me, pe;
        he === void 0 && (he = !0);
        var ye = this.getBBox(he);
        if (!this.styles)
          return ye;
        var ge = new oe.BBox();
        Object.assign(ge, ye);
        try {
          for (var ve = q(oe.BBox.StyleAdjust), be = ve.next(); !be.done; be = ve.next()) {
            var Ce = G(be.value, 2), Se = Ce[0], we = Ce[1], Ee = this.styles.get(Se);
            Ee && (ge[we] += this.length2em(Ee, 1, ge.rscale));
          }
        } catch (xe) {
          me = { error: xe };
        } finally {
          try {
            be && !be.done && (pe = ve.return) && pe.call(ve);
          } finally {
            if (me) throw me.error;
          }
        }
        return ge;
      }, fe.prototype.computeBBox = function(he, me) {
        var pe, ye;
        me === void 0 && (me = !1), he.empty();
        try {
          for (var ge = q(this.childNodes), ve = ge.next(); !ve.done; ve = ge.next()) {
            var be = ve.value;
            he.append(be.getOuterBBox());
          }
        } catch (Ce) {
          pe = { error: Ce };
        } finally {
          try {
            ve && !ve.done && (ye = ge.return) && ye.call(ge);
          } finally {
            if (pe) throw pe.error;
          }
        }
        he.clean(), this.fixesPWidth && this.setChildPWidths(me) && this.computeBBox(he, !0);
      }, fe.prototype.setChildPWidths = function(he, me, pe) {
        var ye, ge;
        if (me === void 0 && (me = null), pe === void 0 && (pe = !0), he)
          return !1;
        pe && (this.bbox.pwidth = "");
        var ve = !1;
        try {
          for (var be = q(this.childNodes), Ce = be.next(); !Ce.done; Ce = be.next()) {
            var Se = Ce.value, we = Se.getOuterBBox();
            we.pwidth && Se.setChildPWidths(he, me === null ? we.w : me, pe) && (ve = !0);
          }
        } catch (Ee) {
          ye = { error: Ee };
        } finally {
          try {
            Ce && !Ce.done && (ge = be.return) && ge.call(be);
          } finally {
            if (ye) throw ye.error;
          }
        }
        return ve;
      }, fe.prototype.invalidateBBox = function() {
        this.bboxComputed && (this.bboxComputed = !1, this.parent && this.parent.invalidateBBox());
      }, fe.prototype.copySkewIC = function(he) {
        var me = this.childNodes[0];
        me != null && me.bbox.sk && (he.sk = me.bbox.sk), me != null && me.bbox.dx && (he.dx = me.bbox.dx);
        var pe = this.childNodes[this.childNodes.length - 1];
        pe != null && pe.bbox.ic && (he.ic = pe.bbox.ic, he.w += he.ic);
      }, fe.prototype.getStyles = function() {
        var he = this.node.attributes.getExplicit("style");
        if (he)
          for (var me = this.styles = new ae.Styles(he), pe = 0, ye = fe.removeStyles.length; pe < ye; pe++) {
            var ge = fe.removeStyles[pe];
            me.get(ge) && (this.removedStyles || (this.removedStyles = {}), this.removedStyles[ge] = me.get(ge), me.set(ge, ""));
          }
      }, fe.prototype.getVariant = function() {
        if (this.node.isToken) {
          var he = this.node.attributes, me = he.get("mathvariant");
          if (!he.getExplicit("mathvariant")) {
            var pe = he.getList("fontfamily", "fontweight", "fontstyle");
            if (this.removedStyles) {
              var ye = this.removedStyles;
              ye.fontFamily && (pe.family = ye.fontFamily), ye.fontWeight && (pe.weight = ye.fontWeight), ye.fontStyle && (pe.style = ye.fontStyle);
            }
            pe.fontfamily && (pe.family = pe.fontfamily), pe.fontweight && (pe.weight = pe.fontweight), pe.fontstyle && (pe.style = pe.fontstyle), pe.weight && pe.weight.match(/^\d+$/) && (pe.weight = parseInt(pe.weight) > 600 ? "bold" : "normal"), pe.family ? me = this.explicitVariant(pe.family, pe.weight, pe.style) : (this.node.getProperty("variantForm") && (me = "-tex-variant"), me = (fe.BOLDVARIANTS[pe.weight] || {})[me] || me, me = (fe.ITALICVARIANTS[pe.style] || {})[me] || me);
          }
          this.variant = me;
        }
      }, fe.prototype.explicitVariant = function(he, me, pe) {
        var ye = this.styles;
        return ye || (ye = this.styles = new ae.Styles()), ye.set("fontFamily", he), me && ye.set("fontWeight", me), pe && ye.set("fontStyle", pe), "-explicitFont";
      }, fe.prototype.getScale = function() {
        var he = 1, me = this.parent, pe = me ? me.bbox.scale : 1, ye = this.node.attributes, ge = Math.min(ye.get("scriptlevel"), 2), ve = ye.get("fontsize"), be = this.node.isToken || this.node.isKind("mstyle") ? ye.get("mathsize") : ye.getInherited("mathsize");
        if (ge !== 0) {
          he = Math.pow(ye.get("scriptsizemultiplier"), ge);
          var Ce = this.length2em(ye.get("scriptminsize"), 0.8, 1);
          he < Ce && (he = Ce);
        }
        this.removedStyles && this.removedStyles.fontSize && !ve && (ve = this.removedStyles.fontSize), ve && !ye.getExplicit("mathsize") && (be = ve), be !== "1" && (he *= this.length2em(be, 1, 1)), this.bbox.scale = he, this.bbox.rscale = he / pe;
      }, fe.prototype.getSpace = function() {
        var he = this.isTopEmbellished(), me = this.node.hasSpacingAttributes();
        this.jax.options.mathmlSpacing || me ? he && this.getMathMLSpacing() : this.getTeXSpacing(he, me);
      }, fe.prototype.getMathMLSpacing = function() {
        var he = this.node.coreMO(), me = he.coreParent(), pe = me.parent;
        if (!(!pe || !pe.isKind("mrow") || pe.childNodes.length === 1)) {
          var ye = he.attributes, ge = ye.get("scriptlevel") > 0;
          this.bbox.L = ye.isSet("lspace") ? Math.max(0, this.length2em(ye.get("lspace"))) : le(ge, he.lspace), this.bbox.R = ye.isSet("rspace") ? Math.max(0, this.length2em(ye.get("rspace"))) : le(ge, he.rspace);
          var ve = pe.childIndex(me);
          if (ve !== 0) {
            var be = pe.childNodes[ve - 1];
            if (be.isEmbellished) {
              var Ce = this.jax.nodeMap.get(be).getBBox();
              Ce.R && (this.bbox.L = Math.max(0, this.bbox.L - Ce.R));
            }
          }
        }
      }, fe.prototype.getTeXSpacing = function(he, me) {
        if (!me) {
          var pe = this.node.texSpacing();
          pe && (this.bbox.L = this.length2em(pe));
        }
        if (he || me) {
          var ye = this.node.coreMO().attributes;
          ye.isSet("lspace") && (this.bbox.L = Math.max(0, this.length2em(ye.get("lspace")))), ye.isSet("rspace") && (this.bbox.R = Math.max(0, this.length2em(ye.get("rspace"))));
        }
      }, fe.prototype.isTopEmbellished = function() {
        return this.node.isEmbellished && !(this.node.parent && this.node.parent.isEmbellished);
      }, fe.prototype.core = function() {
        return this.jax.nodeMap.get(this.node.core());
      }, fe.prototype.coreMO = function() {
        return this.jax.nodeMap.get(this.node.coreMO());
      }, fe.prototype.getText = function() {
        var he, me, pe = "";
        if (this.node.isToken)
          try {
            for (var ye = q(this.node.childNodes), ge = ye.next(); !ge.done; ge = ye.next()) {
              var ve = ge.value;
              ve instanceof te.TextNode && (pe += ve.getText());
            }
          } catch (be) {
            he = { error: be };
          } finally {
            try {
              ge && !ge.done && (me = ye.return) && me.call(ye);
            } finally {
              if (he) throw he.error;
            }
          }
        return pe;
      }, fe.prototype.canStretch = function(he) {
        if (this.stretch = se.NOSTRETCH, this.node.isEmbellished) {
          var me = this.core();
          me && me.node !== this.node && me.canStretch(he) && (this.stretch = me.stretch);
        }
        return this.stretch.dir !== 0;
      }, fe.prototype.getAlignShift = function() {
        var he, me = (he = this.node.attributes).getList.apply(he, J([], G(te.indentAttributes), !1)), pe = me.indentalign, ye = me.indentshift, ge = me.indentalignfirst, ve = me.indentshiftfirst;
        ge !== "indentalign" && (pe = ge), pe === "auto" && (pe = this.jax.options.displayAlign), ve !== "indentshift" && (ye = ve), ye === "auto" && (ye = this.jax.options.displayIndent, pe === "right" && !ye.match(/^\s*0[a-z]*\s*$/) && (ye = ("-" + ye.trim()).replace(/^--/, "")));
        var be = this.length2em(ye, this.metrics.containerWidth);
        return [pe, be];
      }, fe.prototype.getAlignX = function(he, me, pe) {
        return pe === "right" ? he - (me.w + me.R) * me.rscale : pe === "left" ? me.L * me.rscale : (he - me.w * me.rscale) / 2;
      }, fe.prototype.getAlignY = function(he, me, pe, ye, ge) {
        return ge === "top" ? he - pe : ge === "bottom" ? ye - me : ge === "center" ? (he - pe - (me - ye)) / 2 : 0;
      }, fe.prototype.getWrapWidth = function(he) {
        return this.childNodes[he].getBBox().w;
      }, fe.prototype.getChildAlign = function(he) {
        return "left";
      }, fe.prototype.percent = function(he) {
        return ee.percent(he);
      }, fe.prototype.em = function(he) {
        return ee.em(he);
      }, fe.prototype.px = function(he, me) {
        return me === void 0 && (me = -ee.BIGDIMEN), ee.px(he, me, this.metrics.em);
      }, fe.prototype.length2em = function(he, me, pe) {
        return me === void 0 && (me = 1), pe === void 0 && (pe = null), pe === null && (pe = this.bbox.scale), ee.length2em(he, me, pe, this.jax.pxPerEm);
      }, fe.prototype.unicodeChars = function(he, me) {
        me === void 0 && (me = this.variant);
        var pe = (0, ne.unicodeChars)(he), ye = this.font.getVariant(me);
        if (ye && ye.chars) {
          var ge = ye.chars;
          pe = pe.map(function(ve) {
            return ((ge[ve] || [])[3] || {}).smp || ve;
          });
        }
        return pe;
      }, fe.prototype.remapChars = function(he) {
        return he;
      }, fe.prototype.mmlText = function(he) {
        return this.node.factory.create("text").setText(he);
      }, fe.prototype.mmlNode = function(he, me, pe) {
        return me === void 0 && (me = {}), pe === void 0 && (pe = []), this.node.factory.create(he, me, pe);
      }, fe.prototype.createMo = function(he) {
        var me = this.node.factory, pe = me.create("text").setText(he), ye = me.create("mo", { stretchy: !0 }, [pe]);
        ye.inheritAttributesFrom(this.node);
        var ge = this.wrap(ye);
        return ge.parent = this, ge;
      }, fe.prototype.getVariantChar = function(he, me) {
        var pe = this.font.getChar(he, me) || [0, 0, 0, { unknown: !0 }];
        return pe.length === 3 && (pe[3] = {}), pe;
      }, fe.kind = "unknown", fe.styles = {}, fe.removeStyles = [
        "fontSize",
        "fontFamily",
        "fontWeight",
        "fontStyle",
        "fontVariant",
        "font"
      ], fe.skipAttributes = {
        fontfamily: !0,
        fontsize: !0,
        fontweight: !0,
        fontstyle: !0,
        color: !0,
        background: !0,
        class: !0,
        href: !0,
        style: !0,
        xmlns: !0
      }, fe.BOLDVARIANTS = {
        bold: {
          normal: "bold",
          italic: "bold-italic",
          fraktur: "bold-fraktur",
          script: "bold-script",
          "sans-serif": "bold-sans-serif",
          "sans-serif-italic": "sans-serif-bold-italic"
        },
        normal: {
          bold: "normal",
          "bold-italic": "italic",
          "bold-fraktur": "fraktur",
          "bold-script": "script",
          "bold-sans-serif": "sans-serif",
          "sans-serif-bold-italic": "sans-serif-italic"
        }
      }, fe.ITALICVARIANTS = {
        italic: {
          normal: "italic",
          bold: "bold-italic",
          "sans-serif": "sans-serif-italic",
          "bold-sans-serif": "sans-serif-bold-italic"
        },
        normal: {
          italic: "normal",
          "bold-italic": "bold",
          "sans-serif-italic": "sans-serif",
          "sans-serif-bold-italic": "bold-sans-serif"
        }
      }, fe;
    }(Z.AbstractWrapper);
    return Wrapper$1.CommonWrapper = ue, Wrapper$1;
  }
  var hasRequiredWrapper;
  function requireWrapper() {
    if (hasRequiredWrapper) return Wrapper$2;
    hasRequiredWrapper = 1;
    var O = Wrapper$2.__extends || /* @__PURE__ */ function() {
      var Z = function(te, ne) {
        return Z = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ee, ae) {
          ee.__proto__ = ae;
        } || function(ee, ae) {
          for (var oe in ae) Object.prototype.hasOwnProperty.call(ae, oe) && (ee[oe] = ae[oe]);
        }, Z(te, ne);
      };
      return function(te, ne) {
        if (typeof ne != "function" && ne !== null)
          throw new TypeError("Class extends value " + String(ne) + " is not a constructor or null");
        Z(te, ne);
        function ee() {
          this.constructor = te;
        }
        te.prototype = ne === null ? Object.create(ne) : (ee.prototype = ne.prototype, new ee());
      };
    }(), _ = Wrapper$2.__values || function(Z) {
      var te = typeof Symbol == "function" && Symbol.iterator, ne = te && Z[te], ee = 0;
      if (ne) return ne.call(Z);
      if (Z && typeof Z.length == "number") return {
        next: function() {
          return Z && ee >= Z.length && (Z = void 0), { value: Z && Z[ee++], done: !Z };
        }
      };
      throw new TypeError(te ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = Wrapper$2.__read || function(Z, te) {
      var ne = typeof Symbol == "function" && Z[Symbol.iterator];
      if (!ne) return Z;
      var ee = ne.call(Z), ae, oe = [], se;
      try {
        for (; (te === void 0 || te-- > 0) && !(ae = ee.next()).done; ) oe.push(ae.value);
      } catch (ce) {
        se = { error: ce };
      } finally {
        try {
          ae && !ae.done && (ne = ee.return) && ne.call(ee);
        } finally {
          if (se) throw se.error;
        }
      }
      return oe;
    };
    Object.defineProperty(Wrapper$2, "__esModule", { value: !0 }), Wrapper$2.SVGWrapper = void 0;
    var V = requireBBox(), q = requireWrapper$1(), G = requireSvg(), J = function(Z) {
      O(te, Z);
      function te() {
        var ne = Z !== null && Z.apply(this, arguments) || this;
        return ne.element = null, ne.dx = 0, ne;
      }
      return te.prototype.toSVG = function(ne) {
        this.addChildren(this.standardSVGnode(ne));
      }, te.prototype.addChildren = function(ne) {
        var ee, ae, oe = 0;
        try {
          for (var se = _(this.childNodes), ce = se.next(); !ce.done; ce = se.next()) {
            var le = ce.value;
            le.toSVG(ne);
            var ue = le.getOuterBBox();
            le.element && le.place(oe + ue.L * ue.rscale, 0), oe += (ue.L + ue.w + ue.R) * ue.rscale;
          }
        } catch (de) {
          ee = { error: de };
        } finally {
          try {
            ce && !ce.done && (ae = se.return) && ae.call(se);
          } finally {
            if (ee) throw ee.error;
          }
        }
      }, te.prototype.standardSVGnode = function(ne) {
        var ee = this.createSVGnode(ne);
        return this.handleStyles(), this.handleScale(), this.handleBorder(), this.handleColor(), this.handleAttributes(), ee;
      }, te.prototype.createSVGnode = function(ne) {
        this.element = this.svg("g", { "data-mml-node": this.node.kind });
        var ee = this.node.attributes.get("href");
        if (ee) {
          ne = this.adaptor.append(ne, this.svg("a", { href: ee }));
          var ae = this.getOuterBBox(), oe = ae.h, se = ae.d, ce = ae.w;
          this.adaptor.append(this.element, this.svg("rect", {
            "data-hitbox": !0,
            fill: "none",
            stroke: "none",
            "pointer-events": "all",
            width: this.fixed(ce),
            height: this.fixed(oe + se),
            y: this.fixed(-se)
          }));
        }
        return this.adaptor.append(ne, this.element), this.element;
      }, te.prototype.handleStyles = function() {
        var ne = this;
        if (this.styles) {
          var ee = this.styles.cssText;
          ee && this.adaptor.setAttribute(this.element, "style", ee), V.BBox.StyleAdjust.forEach(function(ae) {
            var oe = H(ae, 3), se = oe[0], ce = oe[2];
            if (ce === 0) {
              var le = ne.styles.get(se);
              le && (ne.dx += ne.length2em(le, 1, ne.bbox.rscale));
            }
          });
        }
      }, te.prototype.handleScale = function() {
        if (this.bbox.rscale !== 1) {
          var ne = "scale(" + this.fixed(this.bbox.rscale / 1e3, 3) + ")";
          this.adaptor.setAttribute(this.element, "transform", ne);
        }
      }, te.prototype.handleColor = function() {
        var ne, ee = this.adaptor, ae = this.node.attributes, oe = ae.getExplicit("mathcolor"), se = ae.getExplicit("color"), ce = ae.getExplicit("mathbackground"), le = ae.getExplicit("background"), ue = ((ne = this.styles) === null || ne === void 0 ? void 0 : ne.get("background-color")) || "";
        if ((oe || se) && (ee.setAttribute(this.element, "fill", oe || se), ee.setAttribute(this.element, "stroke", oe || se)), ce || le || ue) {
          var de = this.getOuterBBox(), fe = de.h, he = de.d, me = de.w, pe = this.svg("rect", {
            fill: ce || le || ue,
            x: this.fixed(-this.dx),
            y: this.fixed(-he),
            width: this.fixed(me),
            height: this.fixed(fe + he),
            "data-bgcolor": !0
          }), ye = ee.firstChild(this.element);
          ye ? ee.insert(pe, ye) : ee.append(this.element, pe);
        }
      }, te.prototype.handleBorder = function() {
        var ne, ee, ae, oe;
        if (this.styles) {
          var se = Array(4).fill(0), ce = Array(4), le = Array(4);
          try {
            for (var ue = _([["Top", 0], ["Right", 1], ["Bottom", 2], ["Left", 3]]), de = ue.next(); !de.done; de = ue.next()) {
              var fe = H(de.value, 2), he = fe[0], me = fe[1], pe = "border" + he, ye = this.styles.get(pe + "Width");
              ye && (se[me] = Math.max(0, this.length2em(ye, 1, this.bbox.rscale)), ce[me] = this.styles.get(pe + "Style") || "solid", le[me] = this.styles.get(pe + "Color") || "currentColor");
            }
          } catch (Ge) {
            ne = { error: Ge };
          } finally {
            try {
              de && !de.done && (ee = ue.return) && ee.call(ue);
            } finally {
              if (ne) throw ne.error;
            }
          }
          var ge = te.borderFuzz, ve = this.getOuterBBox(), be = H([ve.h + ge, ve.d + ge, ve.w + ge], 3), Ce = be[0], Se = be[1], we = be[2], Ee = [we, Ce], xe = [-ge, Ce], Le = [we, -Se], Qe = [-ge, -Se], ke = [we - se[1], Ce - se[0]], Me = [-ge + se[3], Ce - se[0]], Re = [we - se[1], -Se + se[2]], De = [-ge + se[3], -Se + se[2]], Pe = [
            [xe, Ee, ke, Me],
            [Le, Ee, ke, Re],
            [Qe, Le, Re, De],
            [Qe, xe, Me, De]
          ], $e = this.adaptor, qe = $e.firstChild(this.element);
          try {
            for (var Xe = _([0, 1, 2, 3]), Ve = Xe.next(); !Ve.done; Ve = Xe.next()) {
              var me = Ve.value;
              if (se[me]) {
                var Ze = Pe[me];
                ce[me] === "dashed" || ce[me] === "dotted" ? this.addBorderBroken(Ze, le[me], ce[me], se[me], me) : this.addBorderSolid(Ze, le[me], qe);
              }
            }
          } catch (Ge) {
            ae = { error: Ge };
          } finally {
            try {
              Ve && !Ve.done && (oe = Xe.return) && oe.call(Xe);
            } finally {
              if (ae) throw ae.error;
            }
          }
        }
      }, te.prototype.addBorderSolid = function(ne, ee, ae) {
        var oe = this, se = this.svg("polygon", {
          points: ne.map(function(ce) {
            var le = H(ce, 2), ue = le[0], de = le[1];
            return "".concat(oe.fixed(ue - oe.dx), ",").concat(oe.fixed(de));
          }).join(" "),
          stroke: "none",
          fill: ee
        });
        ae ? this.adaptor.insert(se, ae) : this.adaptor.append(this.element, se);
      }, te.prototype.addBorderBroken = function(ne, ee, ae, oe, se) {
        var ce = ae === "dotted", le = oe / 2, ue = H([[le, -le, -le, -le], [-le, le, -le, -le], [le, le, -le, le], [le, le, le, -le]][se], 4), de = ue[0], fe = ue[1], he = ue[2], me = ue[3], pe = H(ne, 2), ye = pe[0], ge = pe[1], ve = ye[0] + de - this.dx, be = ye[1] + fe, Ce = ge[0] + he - this.dx, Se = ge[1] + me, we = Math.abs(se % 2 ? Se - be : Ce - ve), Ee = Math.ceil(ce ? we / (2 * oe) : (we - oe) / (4 * oe)), xe = we / (4 * Ee + 1), Le = this.svg("line", {
          x1: this.fixed(ve),
          y1: this.fixed(be),
          x2: this.fixed(Ce),
          y2: this.fixed(Se),
          "stroke-width": this.fixed(oe),
          stroke: ee,
          "stroke-linecap": ce ? "round" : "square",
          "stroke-dasharray": ce ? [1, this.fixed(we / Ee - 2e-3)].join(" ") : [this.fixed(xe), this.fixed(3 * xe)].join(" ")
        }), Qe = this.adaptor, ke = Qe.firstChild(this.element);
        ke ? Qe.insert(Le, ke) : Qe.append(this.element, Le);
      }, te.prototype.handleAttributes = function() {
        var ne, ee, ae, oe, se = this.node.attributes, ce = se.getAllDefaults(), le = te.skipAttributes;
        try {
          for (var ue = _(se.getExplicitNames()), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value;
            (le[fe] === !1 || !(fe in ce) && !le[fe] && !this.adaptor.hasAttribute(this.element, fe)) && this.adaptor.setAttribute(this.element, fe, se.getExplicit(fe));
          }
        } catch (ge) {
          ne = { error: ge };
        } finally {
          try {
            de && !de.done && (ee = ue.return) && ee.call(ue);
          } finally {
            if (ne) throw ne.error;
          }
        }
        if (se.get("class")) {
          var he = se.get("class").trim().split(/ +/);
          try {
            for (var me = _(he), pe = me.next(); !pe.done; pe = me.next()) {
              var ye = pe.value;
              this.adaptor.addClass(this.element, ye);
            }
          } catch (ge) {
            ae = { error: ge };
          } finally {
            try {
              pe && !pe.done && (oe = me.return) && oe.call(me);
            } finally {
              if (ae) throw ae.error;
            }
          }
        }
      }, te.prototype.place = function(ne, ee, ae) {
        if (ae === void 0 && (ae = null), ne += this.dx, !!(ne || ee)) {
          ae || (ae = this.element, ee = this.handleId(ee));
          var oe = "translate(".concat(this.fixed(ne), ",").concat(this.fixed(ee), ")"), se = this.adaptor.getAttribute(ae, "transform") || "";
          this.adaptor.setAttribute(ae, "transform", oe + (se ? " " + se : ""));
        }
      }, te.prototype.handleId = function(ne) {
        if (!this.node.attributes || !this.node.attributes.get("id"))
          return ne;
        var ee = this.adaptor, ae = this.getBBox().h, oe = ee.childNodes(this.element);
        oe.forEach(function(ce) {
          return ee.remove(ce);
        });
        var se = this.svg("g", { "data-idbox": !0, transform: "translate(0,".concat(this.fixed(-ae), ")") }, oe);
        return ee.append(this.element, this.svg("text", { "data-id-align": !0 }, [this.text("")])), ee.append(this.element, se), ne + ae;
      }, te.prototype.firstChild = function() {
        var ne = this.adaptor, ee = ne.firstChild(this.element);
        return ee && ne.kind(ee) === "text" && ne.getAttribute(ee, "data-id-align") && (ee = ne.firstChild(ne.next(ee))), ee && ne.kind(ee) === "rect" && ne.getAttribute(ee, "data-hitbox") && (ee = ne.next(ee)), ee;
      }, te.prototype.placeChar = function(ne, ee, ae, oe, se) {
        var ce, le;
        se === void 0 && (se = null), se === null && (se = this.variant);
        var ue = ne.toString(16).toUpperCase(), de = H(this.getVariantChar(se, ne), 4), fe = de[2], he = de[3];
        if ("p" in he) {
          var me = he.p ? "M" + he.p + "Z" : "";
          this.place(ee, ae, this.adaptor.append(oe, this.charNode(se, ue, me)));
        } else if ("c" in he) {
          var pe = this.adaptor.append(oe, this.svg("g", { "data-c": ue }));
          this.place(ee, ae, pe), ee = 0;
          try {
            for (var ye = _(this.unicodeChars(he.c, se)), ge = ye.next(); !ge.done; ge = ye.next()) {
              var ve = ge.value;
              ee += this.placeChar(ve, ee, ae, pe, se);
            }
          } catch (Se) {
            ce = { error: Se };
          } finally {
            try {
              ge && !ge.done && (le = ye.return) && le.call(ye);
            } finally {
              if (ce) throw ce.error;
            }
          }
        } else if (he.unknown) {
          var be = String.fromCodePoint(ne), Ce = this.adaptor.append(oe, this.jax.unknownText(be, se));
          return this.place(ee, ae, Ce), this.jax.measureTextNodeWithCache(Ce, be, se).w;
        }
        return fe;
      }, te.prototype.charNode = function(ne, ee, ae) {
        var oe = this.jax.options.fontCache;
        return oe !== "none" ? this.useNode(ne, ee, ae) : this.pathNode(ee, ae);
      }, te.prototype.pathNode = function(ne, ee) {
        return this.svg("path", { "data-c": ne, d: ee });
      }, te.prototype.useNode = function(ne, ee, ae) {
        var oe = this.svg("use", { "data-c": ee }), se = "#" + this.jax.fontCache.cachePath(ne, ee, ae);
        return this.adaptor.setAttribute(oe, "href", se, G.XLINKNS), oe;
      }, te.prototype.drawBBox = function() {
        var ne = this.getBBox(), ee = ne.w, ae = ne.h, oe = ne.d, se = this.svg("g", { style: {
          opacity: 0.25
        } }, [
          this.svg("rect", {
            fill: "red",
            height: this.fixed(ae),
            width: this.fixed(ee)
          }),
          this.svg("rect", {
            fill: "green",
            height: this.fixed(oe),
            width: this.fixed(ee),
            y: this.fixed(-oe)
          })
        ]), ce = this.element || this.parent.element;
        this.adaptor.append(ce, se);
      }, te.prototype.html = function(ne, ee, ae) {
        return ee === void 0 && (ee = {}), ae === void 0 && (ae = []), this.jax.html(ne, ee, ae);
      }, te.prototype.svg = function(ne, ee, ae) {
        return ee === void 0 && (ee = {}), ae === void 0 && (ae = []), this.jax.svg(ne, ee, ae);
      }, te.prototype.text = function(ne) {
        return this.jax.text(ne);
      }, te.prototype.fixed = function(ne, ee) {
        return ee === void 0 && (ee = 1), this.jax.fixed(ne * 1e3, ee);
      }, te.kind = "unknown", te.borderFuzz = 5e-3, te;
    }(q.CommonWrapper);
    return Wrapper$2.SVGWrapper = J, Wrapper$2;
  }
  var math$3 = {}, math$2 = {}, hasRequiredMath$2;
  function requireMath$2() {
    if (hasRequiredMath$2) return math$2;
    hasRequiredMath$2 = 1;
    var O = math$2.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(math$2, "__esModule", { value: !0 }), math$2.CommonMathMixin = void 0;
    function _(H) {
      return function(V) {
        O(q, V);
        function q() {
          return V !== null && V.apply(this, arguments) || this;
        }
        return q.prototype.getWrapWidth = function(G) {
          return this.parent ? this.getBBox().w : this.metrics.containerWidth / this.jax.pxPerEm;
        }, q;
      }(H);
    }
    return math$2.CommonMathMixin = _, math$2;
  }
  var math$1 = {}, hasRequiredMath$1;
  function requireMath$1() {
    if (hasRequiredMath$1) return math$1;
    hasRequiredMath$1 = 1;
    var O = math$1.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = math$1.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(math$1, "__esModule", { value: !0 }), math$1.MmlMath = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "math";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.setChildInheritedAttributes = function(J, Z, te, ne) {
        this.attributes.get("mode") === "display" && this.attributes.setInherited("display", "block"), J = this.addInheritedAttributes(J, this.attributes.getAllAttributes()), Z = !!this.attributes.get("displaystyle") || !this.attributes.get("displaystyle") && this.attributes.get("display") === "block", this.attributes.setInherited("displaystyle", Z), te = this.attributes.get("scriptlevel") || this.constructor.defaults.scriptlevel, q.prototype.setChildInheritedAttributes.call(this, J, Z, te, ne);
      }, G.defaults = _(_({}, H.AbstractMmlLayoutNode.defaults), { mathvariant: "normal", mathsize: "normal", mathcolor: "", mathbackground: "transparent", dir: "ltr", scriptlevel: 0, displaystyle: !1, display: "inline", maxwidth: "", overflow: "linebreak", altimg: "", "altimg-width": "", "altimg-height": "", "altimg-valign": "", alttext: "", cdgroup: "", scriptsizemultiplier: 1 / Math.sqrt(2), scriptminsize: "8px", infixlinebreakstyle: "before", lineleading: "1ex", linebreakmultchar: "⁢", indentshift: "auto", indentalign: "auto", indenttarget: "", indentalignfirst: "indentalign", indentshiftfirst: "indentshift", indentalignlast: "indentalign", indentshiftlast: "indentshift" }), G;
    }(H.AbstractMmlLayoutNode);
    return math$1.MmlMath = V, math$1;
  }
  var hasRequiredMath;
  function requireMath() {
    if (hasRequiredMath) return math$3;
    hasRequiredMath = 1;
    var O = math$3.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = math$3.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    }, H = math$3.__values || function(te) {
      var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
      if (ee) return ee.call(te);
      if (te && typeof te.length == "number") return {
        next: function() {
          return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
        }
      };
      throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(math$3, "__esModule", { value: !0 }), math$3.SVGmath = void 0;
    var V = requireWrapper(), q = requireMath$2(), G = requireMath$1(), J = requireBBox(), Z = function(te) {
      O(ne, te);
      function ne() {
        return te !== null && te.apply(this, arguments) || this;
      }
      return ne.prototype.toSVG = function(ee) {
        te.prototype.toSVG.call(this, ee);
        var ae = this.adaptor, oe = this.node.attributes.get("display") === "block";
        oe && (ae.setAttribute(this.jax.container, "display", "true"), this.handleDisplay()), this.jax.document.options.internalSpeechTitles && this.handleSpeech();
      }, ne.prototype.handleDisplay = function() {
        var ee = _(this.getAlignShift(), 2), ae = ee[0], oe = ee[1];
        if (ae !== "center" && this.adaptor.setAttribute(this.jax.container, "justify", ae), this.bbox.pwidth === J.BBox.fullWidth) {
          if (this.adaptor.setAttribute(this.jax.container, "width", "full"), this.jax.table) {
            var se = this.jax.table.getOuterBBox(), ce = se.L, le = se.w, ue = se.R;
            ae === "right" ? ue = Math.max(ue || -oe, -oe) : ae === "left" ? ce = Math.max(ce || oe, oe) : ae === "center" && (le += 2 * Math.abs(oe)), this.jax.minwidth = Math.max(0, ce + le + ue);
          }
        } else
          this.jax.shift = oe;
      }, ne.prototype.handleSpeech = function() {
        var ee, ae, oe = this.adaptor, se = this.node.attributes, ce = se.get("aria-label") || se.get("data-semantic-speech");
        if (ce) {
          var le = this.getTitleID(), ue = this.svg("title", { id: le }, [this.text(ce)]);
          oe.insert(ue, oe.firstChild(this.element)), oe.setAttribute(this.element, "aria-labeledby", le), oe.removeAttribute(this.element, "aria-label");
          try {
            for (var de = H(this.childNodes[0].childNodes), fe = de.next(); !fe.done; fe = de.next()) {
              var he = fe.value;
              oe.setAttribute(he.element, "aria-hidden", "true");
            }
          } catch (me) {
            ee = { error: me };
          } finally {
            try {
              fe && !fe.done && (ae = de.return) && ae.call(de);
            } finally {
              if (ee) throw ee.error;
            }
          }
        }
      }, ne.prototype.getTitleID = function() {
        return "mjx-svg-title-" + String(this.jax.options.titleID++);
      }, ne.prototype.setChildPWidths = function(ee, ae, oe) {
        return ae === void 0 && (ae = null), te.prototype.setChildPWidths.call(this, ee, this.parent ? ae : this.metrics.containerWidth / this.jax.pxPerEm, !1);
      }, ne.kind = G.MmlMath.prototype.kind, ne.styles = {
        'mjx-container[jax="SVG"][display="true"]': {
          display: "block",
          "text-align": "center",
          margin: "1em 0"
        },
        'mjx-container[jax="SVG"][display="true"][width="full"]': {
          display: "flex"
        },
        'mjx-container[jax="SVG"][justify="left"]': {
          "text-align": "left"
        },
        'mjx-container[jax="SVG"][justify="right"]': {
          "text-align": "right"
        }
      }, ne;
    }((0, q.CommonMathMixin)(V.SVGWrapper));
    return math$3.SVGmath = Z, math$3;
  }
  var mrow$2 = {}, mrow$1 = {}, hasRequiredMrow$2;
  function requireMrow$2() {
    if (hasRequiredMrow$2) return mrow$1;
    hasRequiredMrow$2 = 1;
    var O = mrow$1.__extends || /* @__PURE__ */ function() {
      var Z = function(te, ne) {
        return Z = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ee, ae) {
          ee.__proto__ = ae;
        } || function(ee, ae) {
          for (var oe in ae) Object.prototype.hasOwnProperty.call(ae, oe) && (ee[oe] = ae[oe]);
        }, Z(te, ne);
      };
      return function(te, ne) {
        if (typeof ne != "function" && ne !== null)
          throw new TypeError("Class extends value " + String(ne) + " is not a constructor or null");
        Z(te, ne);
        function ee() {
          this.constructor = te;
        }
        te.prototype = ne === null ? Object.create(ne) : (ee.prototype = ne.prototype, new ee());
      };
    }(), _ = mrow$1.__read || function(Z, te) {
      var ne = typeof Symbol == "function" && Z[Symbol.iterator];
      if (!ne) return Z;
      var ee = ne.call(Z), ae, oe = [], se;
      try {
        for (; (te === void 0 || te-- > 0) && !(ae = ee.next()).done; ) oe.push(ae.value);
      } catch (ce) {
        se = { error: ce };
      } finally {
        try {
          ae && !ae.done && (ne = ee.return) && ne.call(ee);
        } finally {
          if (se) throw se.error;
        }
      }
      return oe;
    }, H = mrow$1.__spreadArray || function(Z, te, ne) {
      if (ne || arguments.length === 2) for (var ee = 0, ae = te.length, oe; ee < ae; ee++)
        (oe || !(ee in te)) && (oe || (oe = Array.prototype.slice.call(te, 0, ee)), oe[ee] = te[ee]);
      return Z.concat(oe || Array.prototype.slice.call(te));
    }, V = mrow$1.__values || function(Z) {
      var te = typeof Symbol == "function" && Symbol.iterator, ne = te && Z[te], ee = 0;
      if (ne) return ne.call(Z);
      if (Z && typeof Z.length == "number") return {
        next: function() {
          return Z && ee >= Z.length && (Z = void 0), { value: Z && Z[ee++], done: !Z };
        }
      };
      throw new TypeError(te ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mrow$1, "__esModule", { value: !0 }), mrow$1.CommonInferredMrowMixin = mrow$1.CommonMrowMixin = void 0;
    var q = requireBBox();
    function G(Z) {
      return function(te) {
        O(ne, te);
        function ne() {
          for (var ee, ae, oe = [], se = 0; se < arguments.length; se++)
            oe[se] = arguments[se];
          var ce = te.apply(this, H([], _(oe), !1)) || this;
          ce.stretchChildren();
          try {
            for (var le = V(ce.childNodes), ue = le.next(); !ue.done; ue = le.next()) {
              var de = ue.value;
              if (de.bbox.pwidth) {
                ce.bbox.pwidth = q.BBox.fullWidth;
                break;
              }
            }
          } catch (fe) {
            ee = { error: fe };
          } finally {
            try {
              ue && !ue.done && (ae = le.return) && ae.call(le);
            } finally {
              if (ee) throw ee.error;
            }
          }
          return ce;
        }
        return Object.defineProperty(ne.prototype, "fixesPWidth", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), ne.prototype.stretchChildren = function() {
          var ee, ae, oe, se, ce, le, ue = [];
          try {
            for (var de = V(this.childNodes), fe = de.next(); !fe.done; fe = de.next()) {
              var he = fe.value;
              he.canStretch(1) && ue.push(he);
            }
          } catch (Me) {
            ee = { error: Me };
          } finally {
            try {
              fe && !fe.done && (ae = de.return) && ae.call(de);
            } finally {
              if (ee) throw ee.error;
            }
          }
          var me = ue.length, pe = this.childNodes.length;
          if (me && pe > 1) {
            var ye = 0, ge = 0, ve = me > 1 && me === pe;
            try {
              for (var be = V(this.childNodes), Ce = be.next(); !Ce.done; Ce = be.next()) {
                var he = Ce.value, Se = he.stretch.dir === 0;
                if (ve || Se) {
                  var we = he.getOuterBBox(Se), Ee = we.h, xe = we.d, Le = we.rscale;
                  Ee *= Le, xe *= Le, Ee > ye && (ye = Ee), xe > ge && (ge = xe);
                }
              }
            } catch (Me) {
              oe = { error: Me };
            } finally {
              try {
                Ce && !Ce.done && (se = be.return) && se.call(be);
              } finally {
                if (oe) throw oe.error;
              }
            }
            try {
              for (var Qe = V(ue), ke = Qe.next(); !ke.done; ke = Qe.next()) {
                var he = ke.value;
                he.coreMO().getStretchedVariant([ye, ge]);
              }
            } catch (Me) {
              ce = { error: Me };
            } finally {
              try {
                ke && !ke.done && (le = Qe.return) && le.call(Qe);
              } finally {
                if (ce) throw ce.error;
              }
            }
          }
        }, ne;
      }(Z);
    }
    mrow$1.CommonMrowMixin = G;
    function J(Z) {
      return function(te) {
        O(ne, te);
        function ne() {
          return te !== null && te.apply(this, arguments) || this;
        }
        return ne.prototype.getScale = function() {
          this.bbox.scale = this.parent.bbox.scale, this.bbox.rscale = 1;
        }, ne;
      }(Z);
    }
    return mrow$1.CommonInferredMrowMixin = J, mrow$1;
  }
  var mrow = {}, hasRequiredMrow$1;
  function requireMrow$1() {
    if (hasRequiredMrow$1) return mrow;
    hasRequiredMrow$1 = 1;
    var O = mrow.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = mrow.__assign || function() {
      return _ = Object.assign || function(J) {
        for (var Z, te = 1, ne = arguments.length; te < ne; te++) {
          Z = arguments[te];
          for (var ee in Z) Object.prototype.hasOwnProperty.call(Z, ee) && (J[ee] = Z[ee]);
        }
        return J;
      }, _.apply(this, arguments);
    }, H = mrow.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mrow, "__esModule", { value: !0 }), mrow.MmlInferredMrow = mrow.MmlMrow = void 0;
    var V = requireMmlNode(), q = function(J) {
      O(Z, J);
      function Z() {
        var te = J !== null && J.apply(this, arguments) || this;
        return te._core = null, te;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "mrow";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "isSpacelike", {
        get: function() {
          var te, ne;
          try {
            for (var ee = H(this.childNodes), ae = ee.next(); !ae.done; ae = ee.next()) {
              var oe = ae.value;
              if (!oe.isSpacelike)
                return !1;
            }
          } catch (se) {
            te = { error: se };
          } finally {
            try {
              ae && !ae.done && (ne = ee.return) && ne.call(ee);
            } finally {
              if (te) throw te.error;
            }
          }
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "isEmbellished", {
        get: function() {
          var te, ne, ee = !1, ae = 0;
          try {
            for (var oe = H(this.childNodes), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value;
              if (ce) {
                if (ce.isEmbellished) {
                  if (ee)
                    return !1;
                  ee = !0, this._core = ae;
                } else if (!ce.isSpacelike)
                  return !1;
              }
              ae++;
            }
          } catch (le) {
            te = { error: le };
          } finally {
            try {
              se && !se.done && (ne = oe.return) && ne.call(oe);
            } finally {
              if (te) throw te.error;
            }
          }
          return ee;
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.core = function() {
        return !this.isEmbellished || this._core == null ? this : this.childNodes[this._core];
      }, Z.prototype.coreMO = function() {
        return !this.isEmbellished || this._core == null ? this : this.childNodes[this._core].coreMO();
      }, Z.prototype.nonSpaceLength = function() {
        var te, ne, ee = 0;
        try {
          for (var ae = H(this.childNodes), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            se && !se.isSpacelike && ee++;
          }
        } catch (ce) {
          te = { error: ce };
        } finally {
          try {
            oe && !oe.done && (ne = ae.return) && ne.call(ae);
          } finally {
            if (te) throw te.error;
          }
        }
        return ee;
      }, Z.prototype.firstNonSpace = function() {
        var te, ne;
        try {
          for (var ee = H(this.childNodes), ae = ee.next(); !ae.done; ae = ee.next()) {
            var oe = ae.value;
            if (oe && !oe.isSpacelike)
              return oe;
          }
        } catch (se) {
          te = { error: se };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (te) throw te.error;
          }
        }
        return null;
      }, Z.prototype.lastNonSpace = function() {
        for (var te = this.childNodes.length; --te >= 0; ) {
          var ne = this.childNodes[te];
          if (ne && !ne.isSpacelike)
            return ne;
        }
        return null;
      }, Z.prototype.setTeXclass = function(te) {
        var ne, ee, ae, oe;
        if (this.getProperty("open") != null || this.getProperty("close") != null) {
          this.getPrevClass(te), te = null;
          try {
            for (var se = H(this.childNodes), ce = se.next(); !ce.done; ce = se.next()) {
              var le = ce.value;
              te = le.setTeXclass(te);
            }
          } catch (fe) {
            ne = { error: fe };
          } finally {
            try {
              ce && !ce.done && (ee = se.return) && ee.call(se);
            } finally {
              if (ne) throw ne.error;
            }
          }
          this.texClass == null && (this.texClass = V.TEXCLASS.INNER);
        } else {
          try {
            for (var ue = H(this.childNodes), de = ue.next(); !de.done; de = ue.next()) {
              var le = de.value;
              te = le.setTeXclass(te);
            }
          } catch (fe) {
            ae = { error: fe };
          } finally {
            try {
              de && !de.done && (oe = ue.return) && oe.call(ue);
            } finally {
              if (ae) throw ae.error;
            }
          }
          this.childNodes[0] && this.updateTeXclass(this.childNodes[0]);
        }
        return te;
      }, Z.defaults = _({}, V.AbstractMmlNode.defaults), Z;
    }(V.AbstractMmlNode);
    mrow.MmlMrow = q;
    var G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "inferredMrow";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "isInferred", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "notParent", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.toString = function() {
        return "[" + this.childNodes.join(",") + "]";
      }, Z.defaults = q.defaults, Z;
    }(q);
    return mrow.MmlInferredMrow = G, mrow;
  }
  var hasRequiredMrow;
  function requireMrow() {
    if (hasRequiredMrow) return mrow$2;
    hasRequiredMrow = 1;
    var O = mrow$2.__extends || /* @__PURE__ */ function() {
      var Z = function(te, ne) {
        return Z = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ee, ae) {
          ee.__proto__ = ae;
        } || function(ee, ae) {
          for (var oe in ae) Object.prototype.hasOwnProperty.call(ae, oe) && (ee[oe] = ae[oe]);
        }, Z(te, ne);
      };
      return function(te, ne) {
        if (typeof ne != "function" && ne !== null)
          throw new TypeError("Class extends value " + String(ne) + " is not a constructor or null");
        Z(te, ne);
        function ee() {
          this.constructor = te;
        }
        te.prototype = ne === null ? Object.create(ne) : (ee.prototype = ne.prototype, new ee());
      };
    }();
    Object.defineProperty(mrow$2, "__esModule", { value: !0 }), mrow$2.SVGinferredMrow = mrow$2.SVGmrow = void 0;
    var _ = requireWrapper(), H = requireMrow$2(), V = requireMrow$2(), q = requireMrow$1(), G = function(Z) {
      O(te, Z);
      function te() {
        return Z !== null && Z.apply(this, arguments) || this;
      }
      return te.prototype.toSVG = function(ne) {
        var ee = this.node.isInferred ? this.element = ne : this.standardSVGnode(ne);
        this.addChildren(ee);
      }, te.kind = q.MmlMrow.prototype.kind, te;
    }((0, H.CommonMrowMixin)(_.SVGWrapper));
    mrow$2.SVGmrow = G;
    var J = function(Z) {
      O(te, Z);
      function te() {
        return Z !== null && Z.apply(this, arguments) || this;
      }
      return te.kind = q.MmlInferredMrow.prototype.kind, te;
    }((0, V.CommonInferredMrowMixin)(G));
    return mrow$2.SVGinferredMrow = J, mrow$2;
  }
  var mi$2 = {}, mi$1 = {}, hasRequiredMi$2;
  function requireMi$2() {
    if (hasRequiredMi$2) return mi$1;
    hasRequiredMi$2 = 1;
    var O = mi$1.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(mi$1, "__esModule", { value: !0 }), mi$1.CommonMiMixin = void 0;
    function _(H) {
      return function(V) {
        O(q, V);
        function q() {
          return V !== null && V.apply(this, arguments) || this;
        }
        return q.prototype.computeBBox = function(G, J) {
          V.prototype.computeBBox.call(this, G), this.copySkewIC(G);
        }, q;
      }(H);
    }
    return mi$1.CommonMiMixin = _, mi$1;
  }
  var mi = {}, hasRequiredMi$1;
  function requireMi$1() {
    if (hasRequiredMi$1) return mi;
    hasRequiredMi$1 = 1;
    var O = mi.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mi.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mi, "__esModule", { value: !0 }), mi.MmlMi = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mi";
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.setInheritedAttributes = function(J, Z, te, ne) {
        J === void 0 && (J = {}), Z === void 0 && (Z = !1), te === void 0 && (te = 0), ne === void 0 && (ne = !1), q.prototype.setInheritedAttributes.call(this, J, Z, te, ne);
        var ee = this.getText();
        ee.match(G.singleCharacter) && !J.mathvariant && this.attributes.setInherited("mathvariant", "italic");
      }, G.prototype.setTeXclass = function(J) {
        this.getPrevClass(J);
        var Z = this.getText();
        return Z.length > 1 && Z.match(G.operatorName) && this.attributes.get("mathvariant") === "normal" && this.getProperty("autoOP") === void 0 && this.getProperty("texClass") === void 0 && (this.texClass = H.TEXCLASS.OP, this.setProperty("autoOP", !0)), this;
      }, G.defaults = _({}, H.AbstractMmlTokenNode.defaults), G.operatorName = /^[a-z][a-z0-9]*$/i, G.singleCharacter = /^[\uD800-\uDBFF]?.[\u0300-\u036F\u1AB0-\u1ABE\u1DC0-\u1DFF\u20D0-\u20EF]*$/, G;
    }(H.AbstractMmlTokenNode);
    return mi.MmlMi = V, mi;
  }
  var hasRequiredMi;
  function requireMi() {
    if (hasRequiredMi) return mi$2;
    hasRequiredMi = 1;
    var O = mi$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(mi$2, "__esModule", { value: !0 }), mi$2.SVGmi = void 0;
    var _ = requireWrapper(), H = requireMi$2(), V = requireMi$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.kind = V.MmlMi.prototype.kind, J;
    }((0, H.CommonMiMixin)(_.SVGWrapper));
    return mi$2.SVGmi = q, mi$2;
  }
  var mo$1 = {}, mo = {}, hasRequiredMo$1;
  function requireMo$1() {
    if (hasRequiredMo$1) return mo;
    hasRequiredMo$1 = 1;
    var O = mo.__extends || /* @__PURE__ */ function() {
      var ee = function(ae, oe) {
        return ee = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(se, ce) {
          se.__proto__ = ce;
        } || function(se, ce) {
          for (var le in ce) Object.prototype.hasOwnProperty.call(ce, le) && (se[le] = ce[le]);
        }, ee(ae, oe);
      };
      return function(ae, oe) {
        if (typeof oe != "function" && oe !== null)
          throw new TypeError("Class extends value " + String(oe) + " is not a constructor or null");
        ee(ae, oe);
        function se() {
          this.constructor = ae;
        }
        ae.prototype = oe === null ? Object.create(oe) : (se.prototype = oe.prototype, new se());
      };
    }(), _ = mo.__assign || function() {
      return _ = Object.assign || function(ee) {
        for (var ae, oe = 1, se = arguments.length; oe < se; oe++) {
          ae = arguments[oe];
          for (var ce in ae) Object.prototype.hasOwnProperty.call(ae, ce) && (ee[ce] = ae[ce]);
        }
        return ee;
      }, _.apply(this, arguments);
    }, H = mo.__read || function(ee, ae) {
      var oe = typeof Symbol == "function" && ee[Symbol.iterator];
      if (!oe) return ee;
      var se = oe.call(ee), ce, le = [], ue;
      try {
        for (; (ae === void 0 || ae-- > 0) && !(ce = se.next()).done; ) le.push(ce.value);
      } catch (de) {
        ue = { error: de };
      } finally {
        try {
          ce && !ce.done && (oe = se.return) && oe.call(se);
        } finally {
          if (ue) throw ue.error;
        }
      }
      return le;
    }, V = mo.__spreadArray || function(ee, ae, oe) {
      if (oe || arguments.length === 2) for (var se = 0, ce = ae.length, le; se < ce; se++)
        (le || !(se in ae)) && (le || (le = Array.prototype.slice.call(ae, 0, se)), le[se] = ae[se]);
      return ee.concat(le || Array.prototype.slice.call(ae));
    }, q = mo.__values || function(ee) {
      var ae = typeof Symbol == "function" && Symbol.iterator, oe = ae && ee[ae], se = 0;
      if (oe) return oe.call(ee);
      if (ee && typeof ee.length == "number") return {
        next: function() {
          return ee && se >= ee.length && (ee = void 0), { value: ee && ee[se++], done: !ee };
        }
      };
      throw new TypeError(ae ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, G;
    Object.defineProperty(mo, "__esModule", { value: !0 }), mo.CommonMoMixin = mo.DirectionVH = void 0;
    var J = requireBBox(), Z = requireString(), te = requireFontData$1();
    mo.DirectionVH = (G = {}, G[1] = "v", G[2] = "h", G);
    function ne(ee) {
      return function(ae) {
        O(oe, ae);
        function oe() {
          for (var se = [], ce = 0; ce < arguments.length; ce++)
            se[ce] = arguments[ce];
          var le = ae.apply(this, V([], H(se), !1)) || this;
          return le.size = null, le.isAccent = le.node.isAccent, le;
        }
        return oe.prototype.computeBBox = function(se, ce) {
          if (this.protoBBox(se), this.node.attributes.get("symmetric") && this.stretch.dir !== 2) {
            var le = this.getCenterOffset(se);
            se.h += le, se.d -= le;
          }
          this.node.getProperty("mathaccent") && (this.stretch.dir === 0 || this.size >= 0) && (se.w = 0);
        }, oe.prototype.protoBBox = function(se) {
          var ce = this.stretch.dir !== 0;
          ce && this.size === null && this.getStretchedVariant([0]), !(ce && this.size < 0) && (ae.prototype.computeBBox.call(this, se), this.copySkewIC(se));
        }, oe.prototype.getAccentOffset = function() {
          var se = J.BBox.empty();
          return this.protoBBox(se), -se.w / 2;
        }, oe.prototype.getCenterOffset = function(se) {
          return se === void 0 && (se = null), se || (se = J.BBox.empty(), ae.prototype.computeBBox.call(this, se)), (se.h + se.d) / 2 + this.font.params.axis_height - se.h;
        }, oe.prototype.getVariant = function() {
          if (this.node.attributes.get("largeop")) {
            this.variant = this.node.attributes.get("displaystyle") ? "-largeop" : "-smallop";
            return;
          }
          if (!this.node.attributes.getExplicit("mathvariant") && this.node.getProperty("pseudoscript") === !1) {
            this.variant = "-tex-variant";
            return;
          }
          ae.prototype.getVariant.call(this);
        }, oe.prototype.canStretch = function(se) {
          if (this.stretch.dir !== 0)
            return this.stretch.dir === se;
          var ce = this.node.attributes;
          if (!ce.get("stretchy"))
            return !1;
          var le = this.getText();
          if (Array.from(le).length !== 1)
            return !1;
          var ue = this.font.getDelimiter(le.codePointAt(0));
          return this.stretch = ue && ue.dir === se ? ue : te.NOSTRETCH, this.stretch.dir !== 0;
        }, oe.prototype.getStretchedVariant = function(se, ce) {
          var le, ue;
          if (ce === void 0 && (ce = !1), this.stretch.dir !== 0) {
            var de = this.getWH(se), fe = this.getSize("minsize", 0), he = this.getSize("maxsize", 1 / 0), me = this.node.getProperty("mathaccent");
            de = Math.max(fe, Math.min(he, de));
            var pe = this.font.params.delimiterfactor / 1e3, ye = this.font.params.delimitershortfall, ge = fe || ce ? de : me ? Math.min(de / pe, de + ye) : Math.max(de * pe, de - ye), ve = this.stretch, be = ve.c || this.getText().codePointAt(0), Ce = 0;
            if (ve.sizes)
              try {
                for (var Se = q(ve.sizes), we = Se.next(); !we.done; we = Se.next()) {
                  var Ee = we.value;
                  if (Ee >= ge) {
                    me && Ce && Ce--, this.variant = this.font.getSizeVariant(be, Ce), this.size = Ce, ve.schar && ve.schar[Ce] && (this.stretch = _(_({}, this.stretch), { c: ve.schar[Ce] }));
                    return;
                  }
                  Ce++;
                }
              } catch (xe) {
                le = { error: xe };
              } finally {
                try {
                  we && !we.done && (ue = Se.return) && ue.call(Se);
                } finally {
                  if (le) throw le.error;
                }
              }
            ve.stretch ? (this.size = -1, this.invalidateBBox(), this.getStretchBBox(se, this.checkExtendedHeight(de, ve), ve)) : (this.variant = this.font.getSizeVariant(be, Ce - 1), this.size = Ce - 1);
          }
        }, oe.prototype.getSize = function(se, ce) {
          var le = this.node.attributes;
          return le.isSet(se) && (ce = this.length2em(le.get(se), 1, 1)), ce;
        }, oe.prototype.getWH = function(se) {
          if (se.length === 0)
            return 0;
          if (se.length === 1)
            return se[0];
          var ce = H(se, 2), le = ce[0], ue = ce[1], de = this.font.params.axis_height;
          return this.node.attributes.get("symmetric") ? 2 * Math.max(le - de, ue + de) : le + ue;
        }, oe.prototype.getStretchBBox = function(se, ce, le) {
          var ue;
          le.hasOwnProperty("min") && le.min > ce && (ce = le.min);
          var de = H(le.HDW, 3), fe = de[0], he = de[1], me = de[2];
          this.stretch.dir === 1 ? (ue = H(this.getBaseline(se, ce, le), 2), fe = ue[0], he = ue[1]) : me = ce, this.bbox.h = fe, this.bbox.d = he, this.bbox.w = me;
        }, oe.prototype.getBaseline = function(se, ce, le) {
          var ue = se.length === 2 && se[0] + se[1] === ce, de = this.node.attributes.get("symmetric"), fe = H(ue ? se : [ce, 0], 2), he = fe[0], me = fe[1], pe = H([he + me, 0], 2), ye = pe[0], ge = pe[1];
          if (de) {
            var ve = this.font.params.axis_height;
            ue && (ye = 2 * Math.max(he - ve, me + ve)), ge = ye / 2 - ve;
          } else if (ue)
            ge = me;
          else {
            var be = H(le.HDW || [0.75, 0.25], 2), Ce = be[0], Se = be[1];
            ge = Se * (ye / (Ce + Se));
          }
          return [ye - ge, ge];
        }, oe.prototype.checkExtendedHeight = function(se, ce) {
          if (ce.fullExt) {
            var le = H(ce.fullExt, 2), ue = le[0], de = le[1], fe = Math.ceil(Math.max(0, se - de) / ue);
            se = de + fe * ue;
          }
          return se;
        }, oe.prototype.remapChars = function(se) {
          var ce = this.node.getProperty("primes");
          if (ce)
            return (0, Z.unicodeChars)(ce);
          if (se.length === 1) {
            var le = this.node.coreParent().parent, ue = this.isAccent && !le.isKind("mrow"), de = ue ? "accent" : "mo", fe = this.font.getRemappedChar(de, se[0]);
            fe && (se = this.unicodeChars(fe, this.variant));
          }
          return se;
        }, oe;
      }(ee);
    }
    return mo.CommonMoMixin = ne, mo;
  }
  var hasRequiredMo;
  function requireMo() {
    if (hasRequiredMo) return mo$1;
    hasRequiredMo = 1;
    var O = mo$1.__extends || /* @__PURE__ */ function() {
      var ne = function(ee, ae) {
        return ne = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(oe, se) {
          oe.__proto__ = se;
        } || function(oe, se) {
          for (var ce in se) Object.prototype.hasOwnProperty.call(se, ce) && (oe[ce] = se[ce]);
        }, ne(ee, ae);
      };
      return function(ee, ae) {
        if (typeof ae != "function" && ae !== null)
          throw new TypeError("Class extends value " + String(ae) + " is not a constructor or null");
        ne(ee, ae);
        function oe() {
          this.constructor = ee;
        }
        ee.prototype = ae === null ? Object.create(ae) : (oe.prototype = ae.prototype, new oe());
      };
    }(), _ = mo$1.__values || function(ne) {
      var ee = typeof Symbol == "function" && Symbol.iterator, ae = ee && ne[ee], oe = 0;
      if (ae) return ae.call(ne);
      if (ne && typeof ne.length == "number") return {
        next: function() {
          return ne && oe >= ne.length && (ne = void 0), { value: ne && ne[oe++], done: !ne };
        }
      };
      throw new TypeError(ee ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = mo$1.__read || function(ne, ee) {
      var ae = typeof Symbol == "function" && ne[Symbol.iterator];
      if (!ae) return ne;
      var oe = ae.call(ne), se, ce = [], le;
      try {
        for (; (ee === void 0 || ee-- > 0) && !(se = oe.next()).done; ) ce.push(se.value);
      } catch (ue) {
        le = { error: ue };
      } finally {
        try {
          se && !se.done && (ae = oe.return) && ae.call(oe);
        } finally {
          if (le) throw le.error;
        }
      }
      return ce;
    };
    Object.defineProperty(mo$1, "__esModule", { value: !0 }), mo$1.SVGmo = void 0;
    var V = requireWrapper(), q = requireMo$1(), G = requireMo$2(), J = 0.1, Z = 0.1, te = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee.prototype.toSVG = function(ae) {
        var oe = this.node.attributes, se = oe.get("symmetric") && this.stretch.dir !== 2, ce = this.stretch.dir !== 0;
        ce && this.size === null && this.getStretchedVariant([]);
        var le = this.standardSVGnode(ae);
        if (ce && this.size < 0)
          this.stretchSVG();
        else {
          var ue = se || oe.get("largeop") ? this.fixed(this.getCenterOffset()) : "0", de = this.node.getProperty("mathaccent") ? this.fixed(this.getAccentOffset()) : "0";
          (ue !== "0" || de !== "0") && this.adaptor.setAttribute(le, "transform", "translate(".concat(de, " ").concat(ue, ")")), this.addChildren(le);
        }
      }, ee.prototype.stretchSVG = function() {
        var ae = this.stretch.stretch, oe = this.getStretchVariants(), se = this.getBBox();
        this.stretch.dir === 1 ? this.stretchVertical(ae, oe, se) : this.stretchHorizontal(ae, oe, se);
      }, ee.prototype.getStretchVariants = function() {
        var ae, oe, se = this.stretch.c || this.getText().codePointAt(0), ce = [];
        try {
          for (var le = _(this.stretch.stretch.keys()), ue = le.next(); !ue.done; ue = le.next()) {
            var de = ue.value;
            ce[de] = this.font.getStretchVariant(se, de);
          }
        } catch (fe) {
          ae = { error: fe };
        } finally {
          try {
            ue && !ue.done && (oe = le.return) && oe.call(le);
          } finally {
            if (ae) throw ae.error;
          }
        }
        return ce;
      }, ee.prototype.stretchVertical = function(ae, oe, se) {
        var ce = se.h, le = se.d, ue = se.w, de = this.addTop(ae[0], oe[0], ce, ue), fe = this.addBot(ae[2], oe[2], le, ue);
        if (ae.length === 4) {
          var he = H(this.addMidV(ae[3], oe[3], ue), 2), me = he[0], pe = he[1];
          this.addExtV(ae[1], oe[1], ce, 0, de, me, ue), this.addExtV(ae[1], oe[1], 0, le, pe, fe, ue);
        } else
          this.addExtV(ae[1], oe[1], ce, le, de, fe, ue);
      }, ee.prototype.stretchHorizontal = function(ae, oe, se) {
        var ce = se.w, le = this.addLeft(ae[0], oe[0]), ue = this.addRight(ae[2], oe[2], ce);
        if (ae.length === 4) {
          var de = H(this.addMidH(ae[3], oe[3], ce), 2), fe = de[0], he = de[1], me = ce / 2;
          this.addExtH(ae[1], oe[1], me, le, me - fe), this.addExtH(ae[1], oe[1], me, he - me, ue, me);
        } else
          this.addExtH(ae[1], oe[1], ce, le, ue);
      }, ee.prototype.getChar = function(ae, oe) {
        var se = this.font.getChar(oe, ae) || [0, 0, 0, null];
        return [se[0], se[1], se[2], se[3] || {}];
      }, ee.prototype.addGlyph = function(ae, oe, se, ce, le) {
        return le === void 0 && (le = null), this.placeChar(ae, se, ce, le || this.element, oe);
      }, ee.prototype.addTop = function(ae, oe, se, ce) {
        if (!ae)
          return 0;
        var le = H(this.getChar(ae, oe), 3), ue = le[0], de = le[1], fe = le[2];
        return this.addGlyph(ae, oe, (ce - fe) / 2, se - ue), ue + de;
      }, ee.prototype.addExtV = function(ae, oe, se, ce, le, ue, de) {
        var fe = this;
        if (ae) {
          le = Math.max(0, le - J), ue = Math.max(0, ue - J);
          var he = this.adaptor, me = H(this.getChar(ae, oe), 3), pe = me[0], ye = me[1], ge = me[2], ve = se + ce - le - ue, be = 1.5 * ve / (pe + ye), Ce = (be * (pe - ye) - ve) / 2;
          if (!(ve <= 0)) {
            var Se = this.svg("svg", {
              width: this.fixed(ge),
              height: this.fixed(ve),
              y: this.fixed(ue - ce),
              x: this.fixed((de - ge) / 2),
              viewBox: [0, Ce, ge, ve].map(function(Ee) {
                return fe.fixed(Ee);
              }).join(" ")
            });
            this.addGlyph(ae, oe, 0, 0, Se);
            var we = he.lastChild(Se);
            he.setAttribute(we, "transform", "scale(1,".concat(this.jax.fixed(be), ")")), he.append(this.element, Se);
          }
        }
      }, ee.prototype.addBot = function(ae, oe, se, ce) {
        if (!ae)
          return 0;
        var le = H(this.getChar(ae, oe), 3), ue = le[0], de = le[1], fe = le[2];
        return this.addGlyph(ae, oe, (ce - fe) / 2, de - se), ue + de;
      }, ee.prototype.addMidV = function(ae, oe, se) {
        if (!ae)
          return [0, 0];
        var ce = H(this.getChar(ae, oe), 3), le = ce[0], ue = ce[1], de = ce[2], fe = (ue - le) / 2 + this.font.params.axis_height;
        return this.addGlyph(ae, oe, (se - de) / 2, fe), [le + fe, ue - fe];
      }, ee.prototype.addLeft = function(ae, oe) {
        return ae ? this.addGlyph(ae, oe, 0, 0) : 0;
      }, ee.prototype.addExtH = function(ae, oe, se, ce, le, ue) {
        var de = this;
        if (ue === void 0 && (ue = 0), !!ae) {
          le = Math.max(0, le - Z), ce = Math.max(0, ce - Z);
          var fe = this.adaptor, he = H(this.getChar(ae, oe), 3), me = he[0], pe = he[1], ye = he[2], ge = se - ce - le, ve = me + pe + 2 * J, be = 1.5 * (ge / ye), Ce = -(pe + J);
          if (!(ge <= 0)) {
            var Se = this.svg("svg", {
              width: this.fixed(ge),
              height: this.fixed(ve),
              x: this.fixed(ue + ce),
              y: this.fixed(Ce),
              viewBox: [(be * ye - ge) / 2, Ce, ge, ve].map(function(Ee) {
                return de.fixed(Ee);
              }).join(" ")
            });
            this.addGlyph(ae, oe, 0, 0, Se);
            var we = fe.lastChild(Se);
            fe.setAttribute(we, "transform", "scale(" + this.jax.fixed(be) + ",1)"), fe.append(this.element, Se);
          }
        }
      }, ee.prototype.addRight = function(ae, oe, se) {
        if (!ae)
          return 0;
        var ce = this.getChar(ae, oe)[2];
        return this.addGlyph(ae, oe, se - ce, 0);
      }, ee.prototype.addMidH = function(ae, oe, se) {
        if (!ae)
          return [0, 0];
        var ce = this.getChar(ae, oe)[2];
        return this.addGlyph(ae, oe, (se - ce) / 2, 0), [(se - ce) / 2, (se + ce) / 2];
      }, ee.kind = G.MmlMo.prototype.kind, ee;
    }((0, q.CommonMoMixin)(V.SVGWrapper));
    return mo$1.SVGmo = te, mo$1;
  }
  var mn$2 = {}, mn$1 = {}, hasRequiredMn$2;
  function requireMn$2() {
    if (hasRequiredMn$2) return mn$1;
    hasRequiredMn$2 = 1;
    var O = mn$1.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(mn$1, "__esModule", { value: !0 }), mn$1.CommonMnMixin = void 0;
    function _(H) {
      return function(V) {
        O(q, V);
        function q() {
          return V !== null && V.apply(this, arguments) || this;
        }
        return q.prototype.remapChars = function(G) {
          if (G.length) {
            var J = this.font.getRemappedChar("mn", G[0]);
            if (J) {
              var Z = this.unicodeChars(J, this.variant);
              Z.length === 1 ? G[0] = Z[0] : G = Z.concat(G.slice(1));
            }
          }
          return G;
        }, q;
      }(H);
    }
    return mn$1.CommonMnMixin = _, mn$1;
  }
  var mn = {}, hasRequiredMn$1;
  function requireMn$1() {
    if (hasRequiredMn$1) return mn;
    hasRequiredMn$1 = 1;
    var O = mn.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mn.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mn, "__esModule", { value: !0 }), mn.MmlMn = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mn";
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _({}, H.AbstractMmlTokenNode.defaults), G;
    }(H.AbstractMmlTokenNode);
    return mn.MmlMn = V, mn;
  }
  var hasRequiredMn;
  function requireMn() {
    if (hasRequiredMn) return mn$2;
    hasRequiredMn = 1;
    var O = mn$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(mn$2, "__esModule", { value: !0 }), mn$2.SVGmn = void 0;
    var _ = requireWrapper(), H = requireMn$2(), V = requireMn$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.kind = V.MmlMn.prototype.kind, J;
    }((0, H.CommonMnMixin)(_.SVGWrapper));
    return mn$2.SVGmn = q, mn$2;
  }
  var ms$2 = {}, ms$1 = {}, hasRequiredMs$2;
  function requireMs$2() {
    if (hasRequiredMs$2) return ms$1;
    hasRequiredMs$2 = 1;
    var O = ms$1.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = ms$1.__read || function(q, G) {
      var J = typeof Symbol == "function" && q[Symbol.iterator];
      if (!J) return q;
      var Z = J.call(q), te, ne = [], ee;
      try {
        for (; (G === void 0 || G-- > 0) && !(te = Z.next()).done; ) ne.push(te.value);
      } catch (ae) {
        ee = { error: ae };
      } finally {
        try {
          te && !te.done && (J = Z.return) && J.call(Z);
        } finally {
          if (ee) throw ee.error;
        }
      }
      return ne;
    }, H = ms$1.__spreadArray || function(q, G, J) {
      if (J || arguments.length === 2) for (var Z = 0, te = G.length, ne; Z < te; Z++)
        (ne || !(Z in G)) && (ne || (ne = Array.prototype.slice.call(G, 0, Z)), ne[Z] = G[Z]);
      return q.concat(ne || Array.prototype.slice.call(G));
    };
    Object.defineProperty(ms$1, "__esModule", { value: !0 }), ms$1.CommonMsMixin = void 0;
    function V(q) {
      return function(G) {
        O(J, G);
        function J() {
          for (var Z = [], te = 0; te < arguments.length; te++)
            Z[te] = arguments[te];
          var ne = G.apply(this, H([], _(Z), !1)) || this, ee = ne.node.attributes, ae = ee.getList("lquote", "rquote");
          return ne.variant !== "monospace" && (!ee.isSet("lquote") && ae.lquote === '"' && (ae.lquote = "“"), !ee.isSet("rquote") && ae.rquote === '"' && (ae.rquote = "”")), ne.childNodes.unshift(ne.createText(ae.lquote)), ne.childNodes.push(ne.createText(ae.rquote)), ne;
        }
        return J.prototype.createText = function(Z) {
          var te = this.wrap(this.mmlText(Z));
          return te.parent = this, te;
        }, J;
      }(q);
    }
    return ms$1.CommonMsMixin = V, ms$1;
  }
  var ms = {}, hasRequiredMs$1;
  function requireMs$1() {
    if (hasRequiredMs$1) return ms;
    hasRequiredMs$1 = 1;
    var O = ms.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = ms.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(ms, "__esModule", { value: !0 }), ms.MmlMs = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "ms";
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _(_({}, H.AbstractMmlTokenNode.defaults), { lquote: '"', rquote: '"' }), G;
    }(H.AbstractMmlTokenNode);
    return ms.MmlMs = V, ms;
  }
  var hasRequiredMs;
  function requireMs() {
    if (hasRequiredMs) return ms$2;
    hasRequiredMs = 1;
    var O = ms$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(ms$2, "__esModule", { value: !0 }), ms$2.SVGms = void 0;
    var _ = requireWrapper(), H = requireMs$2(), V = requireMs$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.kind = V.MmlMs.prototype.kind, J;
    }((0, H.CommonMsMixin)(_.SVGWrapper));
    return ms$2.SVGms = q, ms$2;
  }
  var mtext$2 = {}, mtext$1 = {}, hasRequiredMtext$2;
  function requireMtext$2() {
    if (hasRequiredMtext$2) return mtext$1;
    hasRequiredMtext$2 = 1;
    var O = mtext$1.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(mtext$1, "__esModule", { value: !0 }), mtext$1.CommonMtextMixin = void 0;
    function _(H) {
      var V;
      return V = function(q) {
        O(G, q);
        function G() {
          return q !== null && q.apply(this, arguments) || this;
        }
        return G.prototype.getVariant = function() {
          var J = this.jax.options, Z = this.jax.math.outputData, te = (!!Z.merrorFamily || !!J.merrorFont) && this.node.Parent.isKind("merror");
          if (Z.mtextFamily || J.mtextFont || te) {
            var ne = this.node.attributes.get("mathvariant"), ee = this.constructor.INHERITFONTS[ne] || this.jax.font.getCssFont(ne), ae = ee[0] || (te ? Z.merrorFamily || J.merrorFont : Z.mtextFamily || J.mtextFont);
            this.variant = this.explicitVariant(ae, ee[2] ? "bold" : "", ee[1] ? "italic" : "");
            return;
          }
          q.prototype.getVariant.call(this);
        }, G;
      }(H), V.INHERITFONTS = {
        normal: ["", !1, !1],
        bold: ["", !1, !0],
        italic: ["", !0, !1],
        "bold-italic": ["", !0, !0]
      }, V;
    }
    return mtext$1.CommonMtextMixin = _, mtext$1;
  }
  var mtext = {}, hasRequiredMtext$1;
  function requireMtext$1() {
    if (hasRequiredMtext$1) return mtext;
    hasRequiredMtext$1 = 1;
    var O = mtext.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mtext.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mtext, "__esModule", { value: !0 }), mtext.MmlMtext = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mtext";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "isSpacelike", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _({}, H.AbstractMmlTokenNode.defaults), G;
    }(H.AbstractMmlTokenNode);
    return mtext.MmlMtext = V, mtext;
  }
  var hasRequiredMtext;
  function requireMtext() {
    if (hasRequiredMtext) return mtext$2;
    hasRequiredMtext = 1;
    var O = mtext$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(mtext$2, "__esModule", { value: !0 }), mtext$2.SVGmtext = void 0;
    var _ = requireWrapper(), H = requireMtext$2(), V = requireMtext$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.kind = V.MmlMtext.prototype.kind, J;
    }((0, H.CommonMtextMixin)(_.SVGWrapper));
    return mtext$2.SVGmtext = q, mtext$2;
  }
  var merror$1 = {}, merror = {}, hasRequiredMerror$1;
  function requireMerror$1() {
    if (hasRequiredMerror$1) return merror;
    hasRequiredMerror$1 = 1;
    var O = merror.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = merror.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(merror, "__esModule", { value: !0 }), merror.MmlMerror = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "merror";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return -1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _({}, H.AbstractMmlNode.defaults), G;
    }(H.AbstractMmlNode);
    return merror.MmlMerror = V, merror;
  }
  var hasRequiredMerror;
  function requireMerror() {
    if (hasRequiredMerror) return merror$1;
    hasRequiredMerror = 1;
    var O = merror$1.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }();
    Object.defineProperty(merror$1, "__esModule", { value: !0 }), merror$1.SVGmerror = void 0;
    var _ = requireWrapper(), H = requireMerror$1(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return G.prototype.toSVG = function(J) {
        var Z = this.standardSVGnode(J), te = this.getBBox(), ne = te.h, ee = te.d, ae = te.w;
        this.adaptor.append(this.element, this.svg("rect", {
          "data-background": !0,
          width: this.fixed(ae),
          height: this.fixed(ne + ee),
          y: this.fixed(-ee)
        }));
        var oe = this.node.attributes.get("title");
        oe && this.adaptor.append(this.element, this.svg("title", {}, [this.adaptor.text(oe)])), this.addChildren(Z);
      }, G.kind = H.MmlMerror.prototype.kind, G.styles = {
        'g[data-mml-node="merror"] > g': {
          fill: "red",
          stroke: "red"
        },
        'g[data-mml-node="merror"] > rect[data-background]': {
          fill: "yellow",
          stroke: "none"
        }
      }, G;
    }(_.SVGWrapper);
    return merror$1.SVGmerror = V, merror$1;
  }
  var mspace$2 = {}, mspace$1 = {}, hasRequiredMspace$2;
  function requireMspace$2() {
    if (hasRequiredMspace$2) return mspace$1;
    hasRequiredMspace$2 = 1;
    var O = mspace$1.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(mspace$1, "__esModule", { value: !0 }), mspace$1.CommonMspaceMixin = void 0;
    function _(H) {
      return function(V) {
        O(q, V);
        function q() {
          return V !== null && V.apply(this, arguments) || this;
        }
        return q.prototype.computeBBox = function(G, J) {
          var Z = this.node.attributes;
          G.w = this.length2em(Z.get("width"), 0), G.h = this.length2em(Z.get("height"), 0), G.d = this.length2em(Z.get("depth"), 0);
        }, q.prototype.handleVariant = function() {
        }, q;
      }(H);
    }
    return mspace$1.CommonMspaceMixin = _, mspace$1;
  }
  var mspace = {}, hasRequiredMspace$1;
  function requireMspace$1() {
    if (hasRequiredMspace$1) return mspace;
    hasRequiredMspace$1 = 1;
    var O = mspace.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mspace.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mspace, "__esModule", { value: !0 }), mspace.MmlMspace = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.NONE, J;
      }
      return G.prototype.setTeXclass = function(J) {
        return J;
      }, Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mspace";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return 0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "isSpacelike", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "hasNewline", {
        get: function() {
          var J = this.attributes;
          return J.getExplicit("width") == null && J.getExplicit("height") == null && J.getExplicit("depth") == null && J.get("linebreak") === "newline";
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _(_({}, H.AbstractMmlTokenNode.defaults), { width: "0em", height: "0ex", depth: "0ex", linebreak: "auto" }), G;
    }(H.AbstractMmlTokenNode);
    return mspace.MmlMspace = V, mspace;
  }
  var hasRequiredMspace;
  function requireMspace() {
    if (hasRequiredMspace) return mspace$2;
    hasRequiredMspace = 1;
    var O = mspace$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(mspace$2, "__esModule", { value: !0 }), mspace$2.SVGmspace = void 0;
    var _ = requireWrapper(), H = requireMspace$2(), V = requireMspace$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.kind = V.MmlMspace.prototype.kind, J;
    }((0, H.CommonMspaceMixin)(_.SVGWrapper));
    return mspace$2.SVGmspace = q, mspace$2;
  }
  var mpadded$2 = {}, mpadded$1 = {}, hasRequiredMpadded$2;
  function requireMpadded$2() {
    if (hasRequiredMpadded$2) return mpadded$1;
    hasRequiredMpadded$2 = 1;
    var O = mpadded$1.__extends || /* @__PURE__ */ function() {
      var V = function(q, G) {
        return V = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(J, Z) {
          J.__proto__ = Z;
        } || function(J, Z) {
          for (var te in Z) Object.prototype.hasOwnProperty.call(Z, te) && (J[te] = Z[te]);
        }, V(q, G);
      };
      return function(q, G) {
        if (typeof G != "function" && G !== null)
          throw new TypeError("Class extends value " + String(G) + " is not a constructor or null");
        V(q, G);
        function J() {
          this.constructor = q;
        }
        q.prototype = G === null ? Object.create(G) : (J.prototype = G.prototype, new J());
      };
    }(), _ = mpadded$1.__read || function(V, q) {
      var G = typeof Symbol == "function" && V[Symbol.iterator];
      if (!G) return V;
      var J = G.call(V), Z, te = [], ne;
      try {
        for (; (q === void 0 || q-- > 0) && !(Z = J.next()).done; ) te.push(Z.value);
      } catch (ee) {
        ne = { error: ee };
      } finally {
        try {
          Z && !Z.done && (G = J.return) && G.call(J);
        } finally {
          if (ne) throw ne.error;
        }
      }
      return te;
    };
    Object.defineProperty(mpadded$1, "__esModule", { value: !0 }), mpadded$1.CommonMpaddedMixin = void 0;
    function H(V) {
      return function(q) {
        O(G, q);
        function G() {
          return q !== null && q.apply(this, arguments) || this;
        }
        return G.prototype.getDimens = function() {
          var J = this.node.attributes.getList("width", "height", "depth", "lspace", "voffset"), Z = this.childNodes[0].getBBox(), te = Z.w, ne = Z.h, ee = Z.d, ae = te, oe = ne, se = ee, ce = 0, le = 0, ue = 0;
          J.width !== "" && (te = this.dimen(J.width, Z, "w", 0)), J.height !== "" && (ne = this.dimen(J.height, Z, "h", 0)), J.depth !== "" && (ee = this.dimen(J.depth, Z, "d", 0)), J.voffset !== "" && (le = this.dimen(J.voffset, Z)), J.lspace !== "" && (ce = this.dimen(J.lspace, Z));
          var de = this.node.attributes.get("data-align");
          return de && (ue = this.getAlignX(te, Z, de)), [oe, se, ae, ne - oe, ee - se, te - ae, ce, le, ue];
        }, G.prototype.dimen = function(J, Z, te, ne) {
          te === void 0 && (te = ""), ne === void 0 && (ne = null), J = String(J);
          var ee = J.match(/width|height|depth/), ae = ee ? Z[ee[0].charAt(0)] : te ? Z[te] : 0, oe = this.length2em(J, ae) || 0;
          return J.match(/^[-+]/) && te && (oe += ae), ne != null && (oe = Math.max(ne, oe)), oe;
        }, G.prototype.computeBBox = function(J, Z) {
          Z === void 0 && (Z = !1);
          var te = _(this.getDimens(), 6), ne = te[0], ee = te[1], ae = te[2], oe = te[3], se = te[4], ce = te[5];
          J.w = ae + ce, J.h = ne + oe, J.d = ee + se, this.setChildPWidths(Z, J.w);
        }, G.prototype.getWrapWidth = function(J) {
          return this.getBBox().w;
        }, G.prototype.getChildAlign = function(J) {
          return this.node.attributes.get("data-align") || "left";
        }, G;
      }(V);
    }
    return mpadded$1.CommonMpaddedMixin = H, mpadded$1;
  }
  var mpadded = {}, hasRequiredMpadded$1;
  function requireMpadded$1() {
    if (hasRequiredMpadded$1) return mpadded;
    hasRequiredMpadded$1 = 1;
    var O = mpadded.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mpadded.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mpadded, "__esModule", { value: !0 }), mpadded.MmlMpadded = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mpadded";
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _(_({}, H.AbstractMmlLayoutNode.defaults), { width: "", height: "", depth: "", lspace: 0, voffset: 0 }), G;
    }(H.AbstractMmlLayoutNode);
    return mpadded.MmlMpadded = V, mpadded;
  }
  var hasRequiredMpadded;
  function requireMpadded() {
    if (hasRequiredMpadded) return mpadded$2;
    hasRequiredMpadded = 1;
    var O = mpadded$2.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = mpadded$2.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    };
    Object.defineProperty(mpadded$2, "__esModule", { value: !0 }), mpadded$2.SVGmpadded = void 0;
    var H = requireWrapper(), V = requireMpadded$2(), q = requireMpadded$1(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.toSVG = function(te) {
        var ne = this.standardSVGnode(te), ee = _(this.getDimens(), 9), ae = ee[5], oe = ee[6], se = ee[7], ce = ee[8], le = this.node.attributes.get("data-align") || "left", ue = oe + ce - (ae < 0 && le !== "left" ? le === "center" ? ae / 2 : ae : 0);
        (ue || se) && (ne = this.adaptor.append(ne, this.svg("g")), this.place(ue, se, ne)), this.addChildren(ne);
      }, Z.kind = q.MmlMpadded.prototype.kind, Z;
    }((0, V.CommonMpaddedMixin)(H.SVGWrapper));
    return mpadded$2.SVGmpadded = G, mpadded$2;
  }
  var mphantom$1 = {}, mphantom = {}, hasRequiredMphantom$1;
  function requireMphantom$1() {
    if (hasRequiredMphantom$1) return mphantom;
    hasRequiredMphantom$1 = 1;
    var O = mphantom.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mphantom.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mphantom, "__esModule", { value: !0 }), mphantom.MmlMphantom = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mphantom";
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _({}, H.AbstractMmlLayoutNode.defaults), G;
    }(H.AbstractMmlLayoutNode);
    return mphantom.MmlMphantom = V, mphantom;
  }
  var hasRequiredMphantom;
  function requireMphantom() {
    if (hasRequiredMphantom) return mphantom$1;
    hasRequiredMphantom = 1;
    var O = mphantom$1.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }();
    Object.defineProperty(mphantom$1, "__esModule", { value: !0 }), mphantom$1.SVGmphantom = void 0;
    var _ = requireWrapper(), H = requireMphantom$1(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return G.prototype.toSVG = function(J) {
        this.standardSVGnode(J);
      }, G.kind = H.MmlMphantom.prototype.kind, G;
    }(_.SVGWrapper);
    return mphantom$1.SVGmphantom = V, mphantom$1;
  }
  var mfrac$2 = {}, mfrac$1 = {}, hasRequiredMfrac$2;
  function requireMfrac$2() {
    if (hasRequiredMfrac$2) return mfrac$1;
    hasRequiredMfrac$2 = 1;
    var O = mfrac$1.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mfrac$1.__read || function(q, G) {
      var J = typeof Symbol == "function" && q[Symbol.iterator];
      if (!J) return q;
      var Z = J.call(q), te, ne = [], ee;
      try {
        for (; (G === void 0 || G-- > 0) && !(te = Z.next()).done; ) ne.push(te.value);
      } catch (ae) {
        ee = { error: ae };
      } finally {
        try {
          te && !te.done && (J = Z.return) && J.call(Z);
        } finally {
          if (ee) throw ee.error;
        }
      }
      return ne;
    }, H = mfrac$1.__spreadArray || function(q, G, J) {
      if (J || arguments.length === 2) for (var Z = 0, te = G.length, ne; Z < te; Z++)
        (ne || !(Z in G)) && (ne || (ne = Array.prototype.slice.call(G, 0, Z)), ne[Z] = G[Z]);
      return q.concat(ne || Array.prototype.slice.call(G));
    };
    Object.defineProperty(mfrac$1, "__esModule", { value: !0 }), mfrac$1.CommonMfracMixin = void 0;
    function V(q) {
      return function(G) {
        O(J, G);
        function J() {
          for (var Z = [], te = 0; te < arguments.length; te++)
            Z[te] = arguments[te];
          var ne = G.apply(this, H([], _(Z), !1)) || this;
          if (ne.bevel = null, ne.pad = ne.node.getProperty("withDelims") ? 0 : ne.font.params.nulldelimiterspace, ne.node.attributes.get("bevelled")) {
            var ee = ne.getBevelData(ne.isDisplay()).H, ae = ne.bevel = ne.createMo("/");
            ae.node.attributes.set("symmetric", !0), ae.canStretch(1), ae.getStretchedVariant([ee], !0);
          }
          return ne;
        }
        return J.prototype.computeBBox = function(Z, te) {
          te === void 0 && (te = !1), Z.empty();
          var ne = this.node.attributes.getList("linethickness", "bevelled"), ee = ne.linethickness, ae = ne.bevelled, oe = this.isDisplay(), se = null;
          if (ae)
            this.getBevelledBBox(Z, oe);
          else {
            var ce = this.length2em(String(ee), 0.06);
            se = -2 * this.pad, ce === 0 ? this.getAtopBBox(Z, oe) : (this.getFractionBBox(Z, oe, ce), se -= 0.2), se += Z.w;
          }
          Z.clean(), this.setChildPWidths(te, se);
        }, J.prototype.getFractionBBox = function(Z, te, ne) {
          var ee = this.childNodes[0].getOuterBBox(), ae = this.childNodes[1].getOuterBBox(), oe = this.font.params, se = oe.axis_height, ce = this.getTUV(te, ne), le = ce.T, ue = ce.u, de = ce.v;
          Z.combine(ee, 0, se + le + Math.max(ee.d * ee.rscale, ue)), Z.combine(ae, 0, se - le - Math.max(ae.h * ae.rscale, de)), Z.w += 2 * this.pad + 0.2;
        }, J.prototype.getTUV = function(Z, te) {
          var ne = this.font.params, ee = ne.axis_height, ae = (Z ? 3.5 : 1.5) * te;
          return {
            T: (Z ? 3.5 : 1.5) * te,
            u: (Z ? ne.num1 : ne.num2) - ee - ae,
            v: (Z ? ne.denom1 : ne.denom2) + ee - ae
          };
        }, J.prototype.getAtopBBox = function(Z, te) {
          var ne = this.getUVQ(te), ee = ne.u, ae = ne.v, oe = ne.nbox, se = ne.dbox;
          Z.combine(oe, 0, ee), Z.combine(se, 0, -ae), Z.w += 2 * this.pad;
        }, J.prototype.getUVQ = function(Z) {
          var te = this.childNodes[0].getOuterBBox(), ne = this.childNodes[1].getOuterBBox(), ee = this.font.params, ae = _(Z ? [ee.num1, ee.denom1] : [ee.num3, ee.denom2], 2), oe = ae[0], se = ae[1], ce = (Z ? 7 : 3) * ee.rule_thickness, le = oe - te.d * te.scale - (ne.h * ne.scale - se);
          return le < ce && (oe += (ce - le) / 2, se += (ce - le) / 2, le = ce), { u: oe, v: se, q: le, nbox: te, dbox: ne };
        }, J.prototype.getBevelledBBox = function(Z, te) {
          var ne = this.getBevelData(te), ee = ne.u, ae = ne.v, oe = ne.delta, se = ne.nbox, ce = ne.dbox, le = this.bevel.getOuterBBox();
          Z.combine(se, 0, ee), Z.combine(le, Z.w - oe / 2, 0), Z.combine(ce, Z.w - oe / 2, ae);
        }, J.prototype.getBevelData = function(Z) {
          var te = this.childNodes[0].getOuterBBox(), ne = this.childNodes[1].getOuterBBox(), ee = Z ? 0.4 : 0.15, ae = Math.max(te.scale * (te.h + te.d), ne.scale * (ne.h + ne.d)) + 2 * ee, oe = this.font.params.axis_height, se = te.scale * (te.d - te.h) / 2 + oe + ee, ce = ne.scale * (ne.d - ne.h) / 2 + oe - ee;
          return { H: ae, delta: ee, u: se, v: ce, nbox: te, dbox: ne };
        }, J.prototype.canStretch = function(Z) {
          return !1;
        }, J.prototype.isDisplay = function() {
          var Z = this.node.attributes.getList("displaystyle", "scriptlevel"), te = Z.displaystyle, ne = Z.scriptlevel;
          return te && ne === 0;
        }, J.prototype.getWrapWidth = function(Z) {
          var te = this.node.attributes;
          if (te.get("bevelled"))
            return this.childNodes[Z].getOuterBBox().w;
          var ne = this.getBBox().w, ee = this.length2em(te.get("linethickness"));
          return ne - (ee ? 0.2 : 0) - 2 * this.pad;
        }, J.prototype.getChildAlign = function(Z) {
          var te = this.node.attributes;
          return te.get("bevelled") ? "left" : te.get(["numalign", "denomalign"][Z]);
        }, J;
      }(q);
    }
    return mfrac$1.CommonMfracMixin = V, mfrac$1;
  }
  var mfrac = {}, hasRequiredMfrac$1;
  function requireMfrac$1() {
    if (hasRequiredMfrac$1) return mfrac;
    hasRequiredMfrac$1 = 1;
    var O = mfrac.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = mfrac.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    }, H = mfrac.__values || function(G) {
      var J = typeof Symbol == "function" && Symbol.iterator, Z = J && G[J], te = 0;
      if (Z) return Z.call(G);
      if (G && typeof G.length == "number") return {
        next: function() {
          return G && te >= G.length && (G = void 0), { value: G && G[te++], done: !G };
        }
      };
      throw new TypeError(J ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mfrac, "__esModule", { value: !0 }), mfrac.MmlMfrac = void 0;
    var V = requireMmlNode(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return Object.defineProperty(J.prototype, "kind", {
        get: function() {
          return "mfrac";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "arity", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.setTeXclass = function(Z) {
        var te, ne;
        this.getPrevClass(Z);
        try {
          for (var ee = H(this.childNodes), ae = ee.next(); !ae.done; ae = ee.next()) {
            var oe = ae.value;
            oe.setTeXclass(null);
          }
        } catch (se) {
          te = { error: se };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (te) throw te.error;
          }
        }
        return this;
      }, J.prototype.setChildInheritedAttributes = function(Z, te, ne, ee) {
        (!te || ne > 0) && ne++, this.childNodes[0].setInheritedAttributes(Z, !1, ne, ee), this.childNodes[1].setInheritedAttributes(Z, !1, ne, !0);
      }, J.defaults = _(_({}, V.AbstractMmlBaseNode.defaults), { linethickness: "medium", numalign: "center", denomalign: "center", bevelled: !1 }), J;
    }(V.AbstractMmlBaseNode);
    return mfrac.MmlMfrac = q, mfrac;
  }
  var hasRequiredMfrac;
  function requireMfrac() {
    if (hasRequiredMfrac) return mfrac$2;
    hasRequiredMfrac = 1;
    var O = mfrac$2.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = mfrac$2.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    };
    Object.defineProperty(mfrac$2, "__esModule", { value: !0 }), mfrac$2.SVGmfrac = void 0;
    var H = requireWrapper(), V = requireMfrac$2(), q = requireMfrac$1(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.toSVG = function(te) {
        this.standardSVGnode(te);
        var ne = this.node.attributes.getList("linethickness", "bevelled"), ee = ne.linethickness, ae = ne.bevelled, oe = this.isDisplay();
        if (ae)
          this.makeBevelled(oe);
        else {
          var se = this.length2em(String(ee), 0.06);
          se === 0 ? this.makeAtop(oe) : this.makeFraction(oe, se);
        }
      }, Z.prototype.makeFraction = function(te, ne) {
        var ee = this.element, ae = this.node.attributes.getList("numalign", "denomalign"), oe = ae.numalign, se = ae.denomalign, ce = _(this.childNodes, 2), le = ce[0], ue = ce[1], de = le.getOuterBBox(), fe = ue.getOuterBBox(), he = this.font.params, me = he.axis_height, pe = 0.1, ye = this.node.getProperty("withDelims") ? 0 : he.nulldelimiterspace, ge = Math.max((de.L + de.w + de.R) * de.rscale, (fe.L + fe.w + fe.R) * fe.rscale), ve = this.getAlignX(ge, de, oe) + pe + ye, be = this.getAlignX(ge, fe, se) + pe + ye, Ce = this.getTUV(te, ne), Se = Ce.T, we = Ce.u, Ee = Ce.v;
        le.toSVG(ee), le.place(ve, me + Se + Math.max(de.d * de.rscale, we)), ue.toSVG(ee), ue.place(be, me - Se - Math.max(fe.h * fe.rscale, Ee)), this.adaptor.append(ee, this.svg("rect", {
          width: this.fixed(ge + 2 * pe),
          height: this.fixed(ne),
          x: this.fixed(ye),
          y: this.fixed(me - ne / 2)
        }));
      }, Z.prototype.makeAtop = function(te) {
        var ne = this.element, ee = this.node.attributes.getList("numalign", "denomalign"), ae = ee.numalign, oe = ee.denomalign, se = _(this.childNodes, 2), ce = se[0], le = se[1], ue = ce.getOuterBBox(), de = le.getOuterBBox(), fe = this.font.params, he = this.node.getProperty("withDelims") ? 0 : fe.nulldelimiterspace, me = Math.max((ue.L + ue.w + ue.R) * ue.rscale, (de.L + de.w + de.R) * de.rscale), pe = this.getAlignX(me, ue, ae) + he, ye = this.getAlignX(me, de, oe) + he, ge = this.getUVQ(te), ve = ge.u, be = ge.v;
        ce.toSVG(ne), ce.place(pe, ve), le.toSVG(ne), le.place(ye, -be);
      }, Z.prototype.makeBevelled = function(te) {
        var ne = this.element, ee = _(this.childNodes, 2), ae = ee[0], oe = ee[1], se = this.getBevelData(te), ce = se.u, le = se.v, ue = se.delta, de = se.nbox, fe = se.dbox, he = (de.L + de.w + de.R) * de.rscale;
        ae.toSVG(ne), this.bevel.toSVG(ne), oe.toSVG(ne), ae.place(de.L * de.rscale, ce), this.bevel.place(he - ue / 2, 0), oe.place(he + this.bevel.getOuterBBox().w + fe.L * fe.rscale - ue, le);
      }, Z.kind = q.MmlMfrac.prototype.kind, Z;
    }((0, V.CommonMfracMixin)(H.SVGWrapper));
    return mfrac$2.SVGmfrac = G, mfrac$2;
  }
  var msqrt$2 = {}, msqrt$1 = {}, hasRequiredMsqrt$2;
  function requireMsqrt$2() {
    if (hasRequiredMsqrt$2) return msqrt$1;
    hasRequiredMsqrt$2 = 1;
    var O = msqrt$1.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = msqrt$1.__read || function(G, J) {
      var Z = typeof Symbol == "function" && G[Symbol.iterator];
      if (!Z) return G;
      var te = Z.call(G), ne, ee = [], ae;
      try {
        for (; (J === void 0 || J-- > 0) && !(ne = te.next()).done; ) ee.push(ne.value);
      } catch (oe) {
        ae = { error: oe };
      } finally {
        try {
          ne && !ne.done && (Z = te.return) && Z.call(te);
        } finally {
          if (ae) throw ae.error;
        }
      }
      return ee;
    }, H = msqrt$1.__spreadArray || function(G, J, Z) {
      if (Z || arguments.length === 2) for (var te = 0, ne = J.length, ee; te < ne; te++)
        (ee || !(te in J)) && (ee || (ee = Array.prototype.slice.call(J, 0, te)), ee[te] = J[te]);
      return G.concat(ee || Array.prototype.slice.call(J));
    };
    Object.defineProperty(msqrt$1, "__esModule", { value: !0 }), msqrt$1.CommonMsqrtMixin = void 0;
    var V = requireBBox();
    function q(G) {
      return function(J) {
        O(Z, J);
        function Z() {
          for (var te = [], ne = 0; ne < arguments.length; ne++)
            te[ne] = arguments[ne];
          var ee = J.apply(this, H([], _(te), !1)) || this, ae = ee.createMo("√");
          ae.canStretch(1);
          var oe = ee.childNodes[ee.base].getOuterBBox(), se = oe.h, ce = oe.d, le = ee.font.params.rule_thickness, ue = ee.node.attributes.get("displaystyle") ? ee.font.params.x_height : le;
          return ee.surdH = se + ce + 2 * le + ue / 4, ae.getStretchedVariant([ee.surdH - ce, ce], !0), ee;
        }
        return Object.defineProperty(Z.prototype, "base", {
          get: function() {
            return 0;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(Z.prototype, "surd", {
          get: function() {
            return 1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(Z.prototype, "root", {
          get: function() {
            return null;
          },
          enumerable: !1,
          configurable: !0
        }), Z.prototype.createMo = function(te) {
          var ne = J.prototype.createMo.call(this, te);
          return this.childNodes.push(ne), ne;
        }, Z.prototype.computeBBox = function(te, ne) {
          ne === void 0 && (ne = !1);
          var ee = this.childNodes[this.surd].getBBox(), ae = new V.BBox(this.childNodes[this.base].getOuterBBox()), oe = this.getPQ(ee)[1], se = this.font.params.rule_thickness, ce = ae.h + oe + se, le = _(this.getRootDimens(ee, ce), 1), ue = le[0];
          te.h = ce + se, this.combineRootBBox(te, ee, ce), te.combine(ee, ue, ce - ee.h), te.combine(ae, ue + ee.w, 0), te.clean(), this.setChildPWidths(ne);
        }, Z.prototype.combineRootBBox = function(te, ne, ee) {
        }, Z.prototype.getPQ = function(te) {
          var ne = this.font.params.rule_thickness, ee = this.node.attributes.get("displaystyle") ? this.font.params.x_height : ne, ae = te.h + te.d > this.surdH ? (te.h + te.d - (this.surdH - 2 * ne - ee / 2)) / 2 : ne + ee / 4;
          return [ee, ae];
        }, Z.prototype.getRootDimens = function(te, ne) {
          return [0, 0, 0, 0];
        }, Z;
      }(G);
    }
    return msqrt$1.CommonMsqrtMixin = q, msqrt$1;
  }
  var msqrt = {}, hasRequiredMsqrt$1;
  function requireMsqrt$1() {
    if (hasRequiredMsqrt$1) return msqrt;
    hasRequiredMsqrt$1 = 1;
    var O = msqrt.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = msqrt.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(msqrt, "__esModule", { value: !0 }), msqrt.MmlMsqrt = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "msqrt";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return -1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.setTeXclass = function(J) {
        return this.getPrevClass(J), this.childNodes[0].setTeXclass(null), this;
      }, G.prototype.setChildInheritedAttributes = function(J, Z, te, ne) {
        this.childNodes[0].setInheritedAttributes(J, Z, te, !0);
      }, G.defaults = _({}, H.AbstractMmlNode.defaults), G;
    }(H.AbstractMmlNode);
    return msqrt.MmlMsqrt = V, msqrt;
  }
  var hasRequiredMsqrt;
  function requireMsqrt() {
    if (hasRequiredMsqrt) return msqrt$2;
    hasRequiredMsqrt = 1;
    var O = msqrt$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(msqrt$2, "__esModule", { value: !0 }), msqrt$2.SVGmsqrt = void 0;
    var _ = requireWrapper(), H = requireMsqrt$2(), V = requireMsqrt$1(), q = function(G) {
      O(J, G);
      function J() {
        var Z = G !== null && G.apply(this, arguments) || this;
        return Z.dx = 0, Z;
      }
      return J.prototype.toSVG = function(Z) {
        var te = this.childNodes[this.surd], ne = this.childNodes[this.base], ee = this.root ? this.childNodes[this.root] : null, ae = te.getBBox(), oe = ne.getOuterBBox(), se = this.getPQ(ae)[1], ce = this.font.params.rule_thickness * this.bbox.scale, le = oe.h + se + ce, ue = this.standardSVGnode(Z), de = this.adaptor.append(ue, this.svg("g"));
        this.addRoot(ue, ee, ae, le), te.toSVG(ue), te.place(this.dx, le - ae.h), ne.toSVG(de), ne.place(this.dx + ae.w, 0), this.adaptor.append(ue, this.svg("rect", {
          width: this.fixed(oe.w),
          height: this.fixed(ce),
          x: this.fixed(this.dx + ae.w),
          y: this.fixed(le - ce)
        }));
      }, J.prototype.addRoot = function(Z, te, ne, ee) {
      }, J.kind = V.MmlMsqrt.prototype.kind, J;
    }((0, H.CommonMsqrtMixin)(_.SVGWrapper));
    return msqrt$2.SVGmsqrt = q, msqrt$2;
  }
  var mroot$2 = {}, mroot$1 = {}, hasRequiredMroot$2;
  function requireMroot$2() {
    if (hasRequiredMroot$2) return mroot$1;
    hasRequiredMroot$2 = 1;
    var O = mroot$1.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(mroot$1, "__esModule", { value: !0 }), mroot$1.CommonMrootMixin = void 0;
    function _(H) {
      return function(V) {
        O(q, V);
        function q() {
          return V !== null && V.apply(this, arguments) || this;
        }
        return Object.defineProperty(q.prototype, "surd", {
          get: function() {
            return 2;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(q.prototype, "root", {
          get: function() {
            return 1;
          },
          enumerable: !1,
          configurable: !0
        }), q.prototype.combineRootBBox = function(G, J, Z) {
          var te = this.childNodes[this.root].getOuterBBox(), ne = this.getRootDimens(J, Z)[1];
          G.combine(te, 0, ne);
        }, q.prototype.getRootDimens = function(G, J) {
          var Z = this.childNodes[this.surd], te = this.childNodes[this.root].getOuterBBox(), ne = (Z.size < 0 ? 0.5 : 0.6) * G.w, ee = te.w, ae = te.rscale, oe = Math.max(ee, ne / ae), se = Math.max(0, oe - ee), ce = this.rootHeight(te, G, Z.size, J), le = oe * ae - ne;
          return [le, ce, se];
        }, q.prototype.rootHeight = function(G, J, Z, te) {
          var ne = J.h + J.d, ee = (Z < 0 ? 1.9 : 0.55 * ne) - (ne - te);
          return ee + Math.max(0, G.d * G.rscale);
        }, q;
      }(H);
    }
    return mroot$1.CommonMrootMixin = _, mroot$1;
  }
  var mroot = {}, hasRequiredMroot$1;
  function requireMroot$1() {
    if (hasRequiredMroot$1) return mroot;
    hasRequiredMroot$1 = 1;
    var O = mroot.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mroot.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mroot, "__esModule", { value: !0 }), mroot.MmlMroot = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mroot";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.setTeXclass = function(J) {
        return this.getPrevClass(J), this.childNodes[0].setTeXclass(null), this.childNodes[1].setTeXclass(null), this;
      }, G.prototype.setChildInheritedAttributes = function(J, Z, te, ne) {
        this.childNodes[0].setInheritedAttributes(J, Z, te, !0), this.childNodes[1].setInheritedAttributes(J, !1, te + 2, ne);
      }, G.defaults = _({}, H.AbstractMmlNode.defaults), G;
    }(H.AbstractMmlNode);
    return mroot.MmlMroot = V, mroot;
  }
  var hasRequiredMroot;
  function requireMroot() {
    if (hasRequiredMroot) return mroot$2;
    hasRequiredMroot = 1;
    var O = mroot$2.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = mroot$2.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    };
    Object.defineProperty(mroot$2, "__esModule", { value: !0 }), mroot$2.SVGmroot = void 0;
    var H = requireMsqrt(), V = requireMroot$2(), q = requireMroot$1(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.addRoot = function(te, ne, ee, ae) {
        ne.toSVG(te);
        var oe = _(this.getRootDimens(ee, ae), 3), se = oe[0], ce = oe[1], le = oe[2], ue = ne.getOuterBBox();
        ne.place(le * ue.rscale, ce), this.dx = se;
      }, Z.kind = q.MmlMroot.prototype.kind, Z;
    }((0, V.CommonMrootMixin)(H.SVGmsqrt));
    return mroot$2.SVGmroot = G, mroot$2;
  }
  var mfenced$2 = {}, mfenced$1 = {}, hasRequiredMfenced$2;
  function requireMfenced$2() {
    if (hasRequiredMfenced$2) return mfenced$1;
    hasRequiredMfenced$2 = 1;
    var O = mfenced$1.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = mfenced$1.__read || function(G, J) {
      var Z = typeof Symbol == "function" && G[Symbol.iterator];
      if (!Z) return G;
      var te = Z.call(G), ne, ee = [], ae;
      try {
        for (; (J === void 0 || J-- > 0) && !(ne = te.next()).done; ) ee.push(ne.value);
      } catch (oe) {
        ae = { error: oe };
      } finally {
        try {
          ne && !ne.done && (Z = te.return) && Z.call(te);
        } finally {
          if (ae) throw ae.error;
        }
      }
      return ee;
    }, H = mfenced$1.__spreadArray || function(G, J, Z) {
      if (Z || arguments.length === 2) for (var te = 0, ne = J.length, ee; te < ne; te++)
        (ee || !(te in J)) && (ee || (ee = Array.prototype.slice.call(J, 0, te)), ee[te] = J[te]);
      return G.concat(ee || Array.prototype.slice.call(J));
    }, V = mfenced$1.__values || function(G) {
      var J = typeof Symbol == "function" && Symbol.iterator, Z = J && G[J], te = 0;
      if (Z) return Z.call(G);
      if (G && typeof G.length == "number") return {
        next: function() {
          return G && te >= G.length && (G = void 0), { value: G && G[te++], done: !G };
        }
      };
      throw new TypeError(J ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mfenced$1, "__esModule", { value: !0 }), mfenced$1.CommonMfencedMixin = void 0;
    function q(G) {
      return function(J) {
        O(Z, J);
        function Z() {
          for (var te = [], ne = 0; ne < arguments.length; ne++)
            te[ne] = arguments[ne];
          var ee = J.apply(this, H([], _(te), !1)) || this;
          return ee.mrow = null, ee.createMrow(), ee.addMrowChildren(), ee;
        }
        return Z.prototype.createMrow = function() {
          var te = this.node.factory, ne = te.create("inferredMrow");
          ne.inheritAttributesFrom(this.node), this.mrow = this.wrap(ne), this.mrow.parent = this;
        }, Z.prototype.addMrowChildren = function() {
          var te, ne, ee = this.node, ae = this.mrow;
          this.addMo(ee.open), this.childNodes.length && ae.childNodes.push(this.childNodes[0]);
          var oe = 0;
          try {
            for (var se = V(this.childNodes.slice(1)), ce = se.next(); !ce.done; ce = se.next()) {
              var le = ce.value;
              this.addMo(ee.separators[oe++]), ae.childNodes.push(le);
            }
          } catch (ue) {
            te = { error: ue };
          } finally {
            try {
              ce && !ce.done && (ne = se.return) && ne.call(se);
            } finally {
              if (te) throw te.error;
            }
          }
          this.addMo(ee.close), ae.stretchChildren();
        }, Z.prototype.addMo = function(te) {
          if (te) {
            var ne = this.wrap(te);
            this.mrow.childNodes.push(ne), ne.parent = this.mrow;
          }
        }, Z.prototype.computeBBox = function(te, ne) {
          ne === void 0 && (ne = !1), te.updateFrom(this.mrow.getOuterBBox()), this.setChildPWidths(ne);
        }, Z;
      }(G);
    }
    return mfenced$1.CommonMfencedMixin = q, mfenced$1;
  }
  var mfenced = {}, hasRequiredMfenced$1;
  function requireMfenced$1() {
    if (hasRequiredMfenced$1) return mfenced;
    hasRequiredMfenced$1 = 1;
    var O = mfenced.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = mfenced.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    }, H = mfenced.__values || function(G) {
      var J = typeof Symbol == "function" && Symbol.iterator, Z = J && G[J], te = 0;
      if (Z) return Z.call(G);
      if (G && typeof G.length == "number") return {
        next: function() {
          return G && te >= G.length && (G = void 0), { value: G && G[te++], done: !G };
        }
      };
      throw new TypeError(J ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mfenced, "__esModule", { value: !0 }), mfenced.MmlMfenced = void 0;
    var V = requireMmlNode(), q = function(G) {
      O(J, G);
      function J() {
        var Z = G !== null && G.apply(this, arguments) || this;
        return Z.texclass = V.TEXCLASS.INNER, Z.separators = [], Z.open = null, Z.close = null, Z;
      }
      return Object.defineProperty(J.prototype, "kind", {
        get: function() {
          return "mfenced";
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.setTeXclass = function(Z) {
        this.getPrevClass(Z), this.open && (Z = this.open.setTeXclass(Z)), this.childNodes[0] && (Z = this.childNodes[0].setTeXclass(Z));
        for (var te = 1, ne = this.childNodes.length; te < ne; te++)
          this.separators[te - 1] && (Z = this.separators[te - 1].setTeXclass(Z)), this.childNodes[te] && (Z = this.childNodes[te].setTeXclass(Z));
        return this.close && (Z = this.close.setTeXclass(Z)), this.updateTeXclass(this.open), Z;
      }, J.prototype.setChildInheritedAttributes = function(Z, te, ne, ee) {
        var ae, oe;
        this.addFakeNodes();
        try {
          for (var se = H([this.open, this.close].concat(this.separators)), ce = se.next(); !ce.done; ce = se.next()) {
            var le = ce.value;
            le && le.setInheritedAttributes(Z, te, ne, ee);
          }
        } catch (ue) {
          ae = { error: ue };
        } finally {
          try {
            ce && !ce.done && (oe = se.return) && oe.call(se);
          } finally {
            if (ae) throw ae.error;
          }
        }
        G.prototype.setChildInheritedAttributes.call(this, Z, te, ne, ee);
      }, J.prototype.addFakeNodes = function() {
        var Z, te, ne = this.attributes.getList("open", "close", "separators"), ee = ne.open, ae = ne.close, oe = ne.separators;
        if (ee = ee.replace(/[ \t\n\r]/g, ""), ae = ae.replace(/[ \t\n\r]/g, ""), oe = oe.replace(/[ \t\n\r]/g, ""), ee && (this.open = this.fakeNode(ee, { fence: !0, form: "prefix" }, V.TEXCLASS.OPEN)), oe) {
          for (; oe.length < this.childNodes.length - 1; )
            oe += oe.charAt(oe.length - 1);
          var se = 0;
          try {
            for (var ce = H(this.childNodes.slice(1)), le = ce.next(); !le.done; le = ce.next()) {
              var ue = le.value;
              ue && this.separators.push(this.fakeNode(oe.charAt(se++)));
            }
          } catch (de) {
            Z = { error: de };
          } finally {
            try {
              le && !le.done && (te = ce.return) && te.call(ce);
            } finally {
              if (Z) throw Z.error;
            }
          }
        }
        ae && (this.close = this.fakeNode(ae, { fence: !0, form: "postfix" }, V.TEXCLASS.CLOSE));
      }, J.prototype.fakeNode = function(Z, te, ne) {
        te === void 0 && (te = {}), ne === void 0 && (ne = null);
        var ee = this.factory.create("text").setText(Z), ae = this.factory.create("mo", te, [ee]);
        return ae.texClass = ne, ae.parent = this, ae;
      }, J.defaults = _(_({}, V.AbstractMmlNode.defaults), { open: "(", close: ")", separators: "," }), J;
    }(V.AbstractMmlNode);
    return mfenced.MmlMfenced = q, mfenced;
  }
  var hasRequiredMfenced;
  function requireMfenced() {
    if (hasRequiredMfenced) return mfenced$2;
    hasRequiredMfenced = 1;
    var O = mfenced$2.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = mfenced$2.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mfenced$2, "__esModule", { value: !0 }), mfenced$2.SVGmfenced = void 0;
    var H = requireWrapper(), V = requireMfenced$2(), q = requireMfenced$1(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.toSVG = function(te) {
        var ne = this.standardSVGnode(te);
        this.setChildrenParent(this.mrow), this.mrow.toSVG(ne), this.setChildrenParent(this);
      }, Z.prototype.setChildrenParent = function(te) {
        var ne, ee;
        try {
          for (var ae = _(this.childNodes), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            se.parent = te;
          }
        } catch (ce) {
          ne = { error: ce };
        } finally {
          try {
            oe && !oe.done && (ee = ae.return) && ee.call(ae);
          } finally {
            if (ne) throw ne.error;
          }
        }
      }, Z.kind = q.MmlMfenced.prototype.kind, Z;
    }((0, V.CommonMfencedMixin)(H.SVGWrapper));
    return mfenced$2.SVGmfenced = G, mfenced$2;
  }
  var msubsup$2 = {}, scriptbase$1 = {}, scriptbase = {}, hasRequiredScriptbase$1;
  function requireScriptbase$1() {
    if (hasRequiredScriptbase$1) return scriptbase;
    hasRequiredScriptbase$1 = 1;
    var O = scriptbase.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = scriptbase.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    }, H = scriptbase.__spreadArray || function(J, Z, te) {
      if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
        (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
      return J.concat(ae || Array.prototype.slice.call(Z));
    }, V = scriptbase.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(scriptbase, "__esModule", { value: !0 }), scriptbase.CommonScriptbaseMixin = void 0;
    var q = requireMmlNode();
    function G(J) {
      var Z;
      return Z = function(te) {
        O(ne, te);
        function ne() {
          for (var ee = [], ae = 0; ae < arguments.length; ae++)
            ee[ae] = arguments[ae];
          var oe = te.apply(this, H([], _(ee), !1)) || this;
          oe.baseScale = 1, oe.baseIc = 0, oe.baseRemoveIc = !1, oe.baseIsChar = !1, oe.baseHasAccentOver = null, oe.baseHasAccentUnder = null, oe.isLineAbove = !1, oe.isLineBelow = !1, oe.isMathAccent = !1;
          var se = oe.baseCore = oe.getBaseCore();
          return se && (oe.setBaseAccentsFor(se), oe.baseScale = oe.getBaseScale(), oe.baseIc = oe.getBaseIc(), oe.baseIsChar = oe.isCharBase(), oe.isMathAccent = oe.baseIsChar && oe.scriptChild && !!oe.scriptChild.coreMO().node.getProperty("mathaccent"), oe.checkLineAccents(), oe.baseRemoveIc = !oe.isLineAbove && !oe.isLineBelow && (!oe.constructor.useIC || oe.isMathAccent)), oe;
        }
        return Object.defineProperty(ne.prototype, "baseChild", {
          get: function() {
            return this.childNodes[this.node.base];
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(ne.prototype, "scriptChild", {
          get: function() {
            return this.childNodes[1];
          },
          enumerable: !1,
          configurable: !0
        }), ne.prototype.getBaseCore = function() {
          for (var ee = this.getSemanticBase() || this.childNodes[0]; ee && (ee.childNodes.length === 1 && (ee.node.isKind("mrow") || ee.node.isKind("TeXAtom") && ee.node.texClass !== q.TEXCLASS.VCENTER || ee.node.isKind("mstyle") || ee.node.isKind("mpadded") || ee.node.isKind("mphantom") || ee.node.isKind("semantics")) || ee.node.isKind("munderover") && ee.isMathAccent); )
            this.setBaseAccentsFor(ee), ee = ee.childNodes[0];
          return ee || (this.baseHasAccentOver = this.baseHasAccentUnder = !1), ee || this.childNodes[0];
        }, ne.prototype.setBaseAccentsFor = function(ee) {
          ee.node.isKind("munderover") && (this.baseHasAccentOver === null && (this.baseHasAccentOver = !!ee.node.attributes.get("accent")), this.baseHasAccentUnder === null && (this.baseHasAccentUnder = !!ee.node.attributes.get("accentunder")));
        }, ne.prototype.getSemanticBase = function() {
          var ee = this.node.attributes.getExplicit("data-semantic-fencepointer");
          return this.getBaseFence(this.baseChild, ee);
        }, ne.prototype.getBaseFence = function(ee, ae) {
          var oe, se;
          if (!ee || !ee.node.attributes || !ae)
            return null;
          if (ee.node.attributes.getExplicit("data-semantic-id") === ae)
            return ee;
          try {
            for (var ce = V(ee.childNodes), le = ce.next(); !le.done; le = ce.next()) {
              var ue = le.value, de = this.getBaseFence(ue, ae);
              if (de)
                return de;
            }
          } catch (fe) {
            oe = { error: fe };
          } finally {
            try {
              le && !le.done && (se = ce.return) && se.call(ce);
            } finally {
              if (oe) throw oe.error;
            }
          }
          return null;
        }, ne.prototype.getBaseScale = function() {
          for (var ee = this.baseCore, ae = 1; ee && ee !== this; ) {
            var oe = ee.getOuterBBox();
            ae *= oe.rscale, ee = ee.parent;
          }
          return ae;
        }, ne.prototype.getBaseIc = function() {
          return this.baseCore.getOuterBBox().ic * this.baseScale;
        }, ne.prototype.getAdjustedIc = function() {
          var ee = this.baseCore.getOuterBBox();
          return (ee.ic ? 1.05 * ee.ic + 0.05 : 0) * this.baseScale;
        }, ne.prototype.isCharBase = function() {
          var ee = this.baseCore;
          return (ee.node.isKind("mo") && ee.size === null || ee.node.isKind("mi") || ee.node.isKind("mn")) && ee.bbox.rscale === 1 && Array.from(ee.getText()).length === 1;
        }, ne.prototype.checkLineAccents = function() {
          if (this.node.isKind("munderover"))
            if (this.node.isKind("mover"))
              this.isLineAbove = this.isLineAccent(this.scriptChild);
            else if (this.node.isKind("munder"))
              this.isLineBelow = this.isLineAccent(this.scriptChild);
            else {
              var ee = this;
              this.isLineAbove = this.isLineAccent(ee.overChild), this.isLineBelow = this.isLineAccent(ee.underChild);
            }
        }, ne.prototype.isLineAccent = function(ee) {
          var ae = ee.coreMO().node;
          return ae.isToken && ae.getText() === "―";
        }, ne.prototype.getBaseWidth = function() {
          var ee = this.baseChild.getOuterBBox();
          return ee.w * ee.rscale - (this.baseRemoveIc ? this.baseIc : 0) + this.font.params.extra_ic;
        }, ne.prototype.computeBBox = function(ee, ae) {
          ae === void 0 && (ae = !1);
          var oe = this.getBaseWidth(), se = _(this.getOffset(), 2), ce = se[0], le = se[1];
          ee.append(this.baseChild.getOuterBBox()), ee.combine(this.scriptChild.getOuterBBox(), oe + ce, le), ee.w += this.font.params.scriptspace, ee.clean(), this.setChildPWidths(ae);
        }, ne.prototype.getOffset = function() {
          return [0, 0];
        }, ne.prototype.baseCharZero = function(ee) {
          var ae = !!this.baseCore.node.attributes.get("largeop"), oe = this.baseScale;
          return this.baseIsChar && !ae && oe === 1 ? 0 : ee;
        }, ne.prototype.getV = function() {
          var ee = this.baseCore.getOuterBBox(), ae = this.scriptChild.getOuterBBox(), oe = this.font.params, se = this.length2em(this.node.attributes.get("subscriptshift"), oe.sub1);
          return Math.max(this.baseCharZero(ee.d * this.baseScale + oe.sub_drop * ae.rscale), se, ae.h * ae.rscale - 4 / 5 * oe.x_height);
        }, ne.prototype.getU = function() {
          var ee = this.baseCore.getOuterBBox(), ae = this.scriptChild.getOuterBBox(), oe = this.font.params, se = this.node.attributes.getList("displaystyle", "superscriptshift"), ce = this.node.getProperty("texprimestyle"), le = ce ? oe.sup3 : se.displaystyle ? oe.sup1 : oe.sup2, ue = this.length2em(se.superscriptshift, le);
          return Math.max(this.baseCharZero(ee.h * this.baseScale - oe.sup_drop * ae.rscale), ue, ae.d * ae.rscale + 1 / 4 * oe.x_height);
        }, ne.prototype.hasMovableLimits = function() {
          var ee = this.node.attributes.get("displaystyle"), ae = this.baseChild.coreMO().node;
          return !ee && !!ae.attributes.get("movablelimits");
        }, ne.prototype.getOverKU = function(ee, ae) {
          var oe = this.node.attributes.get("accent"), se = this.font.params, ce = ae.d * ae.rscale, le = se.rule_thickness * se.separation_factor, ue = this.baseHasAccentOver ? le : 0, de = this.isLineAbove ? 3 * se.rule_thickness : le, fe = (oe ? de : Math.max(se.big_op_spacing1, se.big_op_spacing3 - Math.max(0, ce))) - ue;
          return [fe, ee.h * ee.rscale + fe + ce];
        }, ne.prototype.getUnderKV = function(ee, ae) {
          var oe = this.node.attributes.get("accentunder"), se = this.font.params, ce = ae.h * ae.rscale, le = se.rule_thickness * se.separation_factor, ue = this.baseHasAccentUnder ? le : 0, de = this.isLineBelow ? 3 * se.rule_thickness : le, fe = (oe ? de : Math.max(se.big_op_spacing2, se.big_op_spacing4 - ce)) - ue;
          return [fe, -(ee.d * ee.rscale + fe + ce)];
        }, ne.prototype.getDeltaW = function(ee, ae) {
          var oe, se, ce, le;
          ae === void 0 && (ae = [0, 0, 0]);
          var ue = this.node.attributes.get("align"), de = ee.map(function(Ce) {
            return Ce.w * Ce.rscale;
          });
          de[0] -= this.baseRemoveIc && !this.baseCore.node.attributes.get("largeop") ? this.baseIc : 0;
          var fe = Math.max.apply(Math, H([], _(de), !1)), he = [], me = 0;
          try {
            for (var pe = V(de.keys()), ye = pe.next(); !ye.done; ye = pe.next()) {
              var ge = ye.value;
              he[ge] = (ue === "center" ? (fe - de[ge]) / 2 : ue === "right" ? fe - de[ge] : 0) + ae[ge], he[ge] < me && (me = -he[ge]);
            }
          } catch (Ce) {
            oe = { error: Ce };
          } finally {
            try {
              ye && !ye.done && (se = pe.return) && se.call(pe);
            } finally {
              if (oe) throw oe.error;
            }
          }
          if (me)
            try {
              for (var ve = V(he.keys()), be = ve.next(); !be.done; be = ve.next()) {
                var ge = be.value;
                he[ge] += me;
              }
            } catch (Ce) {
              ce = { error: Ce };
            } finally {
              try {
                be && !be.done && (le = ve.return) && le.call(ve);
              } finally {
                if (ce) throw ce.error;
              }
            }
          return [1, 2].map(function(Ce) {
            return he[Ce] += ee[Ce] ? ee[Ce].dx * ee[0].scale : 0;
          }), he;
        }, ne.prototype.getDelta = function(ee) {
          ee === void 0 && (ee = !1);
          var ae = this.node.attributes.get("accent"), oe = this.baseCore.getOuterBBox(), se = oe.sk, ce = oe.ic;
          return ((ae && !ee ? se : 0) + this.font.skewIcFactor * ce) * this.baseScale;
        }, ne.prototype.stretchChildren = function() {
          var ee, ae, oe, se, ce, le, ue = [];
          try {
            for (var de = V(this.childNodes), fe = de.next(); !fe.done; fe = de.next()) {
              var he = fe.value;
              he.canStretch(2) && ue.push(he);
            }
          } catch (Qe) {
            ee = { error: Qe };
          } finally {
            try {
              fe && !fe.done && (ae = de.return) && ae.call(de);
            } finally {
              if (ee) throw ee.error;
            }
          }
          var me = ue.length, pe = this.childNodes.length;
          if (me && pe > 1) {
            var ye = 0, ge = me > 1 && me === pe;
            try {
              for (var ve = V(this.childNodes), be = ve.next(); !be.done; be = ve.next()) {
                var he = be.value, Ce = he.stretch.dir === 0;
                if (ge || Ce) {
                  var Se = he.getOuterBBox(Ce), we = Se.w, Ee = Se.rscale;
                  we * Ee > ye && (ye = we * Ee);
                }
              }
            } catch (Qe) {
              oe = { error: Qe };
            } finally {
              try {
                be && !be.done && (se = ve.return) && se.call(ve);
              } finally {
                if (oe) throw oe.error;
              }
            }
            try {
              for (var xe = V(ue), Le = xe.next(); !Le.done; Le = xe.next()) {
                var he = Le.value;
                he.coreMO().getStretchedVariant([ye / he.bbox.rscale]);
              }
            } catch (Qe) {
              ce = { error: Qe };
            } finally {
              try {
                Le && !Le.done && (le = xe.return) && le.call(xe);
              } finally {
                if (ce) throw ce.error;
              }
            }
          }
        }, ne;
      }(J), Z.useIC = !0, Z;
    }
    return scriptbase.CommonScriptbaseMixin = G, scriptbase;
  }
  var hasRequiredScriptbase;
  function requireScriptbase() {
    if (hasRequiredScriptbase) return scriptbase$1;
    hasRequiredScriptbase = 1;
    var O = scriptbase$1.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = scriptbase$1.__read || function(G, J) {
      var Z = typeof Symbol == "function" && G[Symbol.iterator];
      if (!Z) return G;
      var te = Z.call(G), ne, ee = [], ae;
      try {
        for (; (J === void 0 || J-- > 0) && !(ne = te.next()).done; ) ee.push(ne.value);
      } catch (oe) {
        ae = { error: oe };
      } finally {
        try {
          ne && !ne.done && (Z = te.return) && Z.call(te);
        } finally {
          if (ae) throw ae.error;
        }
      }
      return ee;
    };
    Object.defineProperty(scriptbase$1, "__esModule", { value: !0 }), scriptbase$1.SVGscriptbase = void 0;
    var H = requireWrapper(), V = requireScriptbase$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.prototype.toSVG = function(Z) {
        var te = this.standardSVGnode(Z), ne = this.getBaseWidth(), ee = _(this.getOffset(), 2), ae = ee[0], oe = ee[1];
        this.baseChild.toSVG(te), this.scriptChild.toSVG(te), this.scriptChild.place(ne + ae, oe);
      }, J.kind = "scriptbase", J;
    }((0, V.CommonScriptbaseMixin)(H.SVGWrapper));
    return scriptbase$1.SVGscriptbase = q, scriptbase$1;
  }
  var msubsup$1 = {}, hasRequiredMsubsup$2;
  function requireMsubsup$2() {
    if (hasRequiredMsubsup$2) return msubsup$1;
    hasRequiredMsubsup$2 = 1;
    var O = msubsup$1.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = msubsup$1.__read || function(G, J) {
      var Z = typeof Symbol == "function" && G[Symbol.iterator];
      if (!Z) return G;
      var te = Z.call(G), ne, ee = [], ae;
      try {
        for (; (J === void 0 || J-- > 0) && !(ne = te.next()).done; ) ee.push(ne.value);
      } catch (oe) {
        ae = { error: oe };
      } finally {
        try {
          ne && !ne.done && (Z = te.return) && Z.call(te);
        } finally {
          if (ae) throw ae.error;
        }
      }
      return ee;
    };
    Object.defineProperty(msubsup$1, "__esModule", { value: !0 }), msubsup$1.CommonMsubsupMixin = msubsup$1.CommonMsupMixin = msubsup$1.CommonMsubMixin = void 0;
    function H(G) {
      var J;
      return J = function(Z) {
        O(te, Z);
        function te() {
          return Z !== null && Z.apply(this, arguments) || this;
        }
        return Object.defineProperty(te.prototype, "scriptChild", {
          get: function() {
            return this.childNodes[this.node.sub];
          },
          enumerable: !1,
          configurable: !0
        }), te.prototype.getOffset = function() {
          return [0, -this.getV()];
        }, te;
      }(G), J.useIC = !1, J;
    }
    msubsup$1.CommonMsubMixin = H;
    function V(G) {
      return function(J) {
        O(Z, J);
        function Z() {
          return J !== null && J.apply(this, arguments) || this;
        }
        return Object.defineProperty(Z.prototype, "scriptChild", {
          get: function() {
            return this.childNodes[this.node.sup];
          },
          enumerable: !1,
          configurable: !0
        }), Z.prototype.getOffset = function() {
          var te = this.getAdjustedIc() - (this.baseRemoveIc ? 0 : this.baseIc);
          return [te, this.getU()];
        }, Z;
      }(G);
    }
    msubsup$1.CommonMsupMixin = V;
    function q(G) {
      var J;
      return J = function(Z) {
        O(te, Z);
        function te() {
          var ne = Z !== null && Z.apply(this, arguments) || this;
          return ne.UVQ = null, ne;
        }
        return Object.defineProperty(te.prototype, "subChild", {
          get: function() {
            return this.childNodes[this.node.sub];
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(te.prototype, "supChild", {
          get: function() {
            return this.childNodes[this.node.sup];
          },
          enumerable: !1,
          configurable: !0
        }), te.prototype.computeBBox = function(ne, ee) {
          ee === void 0 && (ee = !1);
          var ae = this.baseChild.getOuterBBox(), oe = _([this.subChild.getOuterBBox(), this.supChild.getOuterBBox()], 2), se = oe[0], ce = oe[1];
          ne.empty(), ne.append(ae);
          var le = this.getBaseWidth(), ue = this.getAdjustedIc(), de = _(this.getUVQ(), 2), fe = de[0], he = de[1];
          ne.combine(se, le, he), ne.combine(ce, le + ue, fe), ne.w += this.font.params.scriptspace, ne.clean(), this.setChildPWidths(ee);
        }, te.prototype.getUVQ = function(ne, ee) {
          ne === void 0 && (ne = this.subChild.getOuterBBox()), ee === void 0 && (ee = this.supChild.getOuterBBox());
          var ae = this.baseCore.getOuterBBox();
          if (this.UVQ)
            return this.UVQ;
          var oe = this.font.params, se = 3 * oe.rule_thickness, ce = this.length2em(this.node.attributes.get("subscriptshift"), oe.sub2), le = this.baseCharZero(ae.d * this.baseScale + oe.sub_drop * ne.rscale), ue = _([this.getU(), Math.max(le, ce)], 2), de = ue[0], fe = ue[1], he = de - ee.d * ee.rscale - (ne.h * ne.rscale - fe);
          if (he < se) {
            fe += se - he;
            var me = 4 / 5 * oe.x_height - (de - ee.d * ee.rscale);
            me > 0 && (de += me, fe -= me);
          }
          return de = Math.max(this.length2em(this.node.attributes.get("superscriptshift"), de), de), fe = Math.max(this.length2em(this.node.attributes.get("subscriptshift"), fe), fe), he = de - ee.d * ee.rscale - (ne.h * ne.rscale - fe), this.UVQ = [de, -fe, he], this.UVQ;
        }, te;
      }(G), J.useIC = !1, J;
    }
    return msubsup$1.CommonMsubsupMixin = q, msubsup$1;
  }
  var msubsup = {}, hasRequiredMsubsup$1;
  function requireMsubsup$1() {
    if (hasRequiredMsubsup$1) return msubsup;
    hasRequiredMsubsup$1 = 1;
    var O = msubsup.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = msubsup.__assign || function() {
      return _ = Object.assign || function(J) {
        for (var Z, te = 1, ne = arguments.length; te < ne; te++) {
          Z = arguments[te];
          for (var ee in Z) Object.prototype.hasOwnProperty.call(Z, ee) && (J[ee] = Z[ee]);
        }
        return J;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(msubsup, "__esModule", { value: !0 }), msubsup.MmlMsup = msubsup.MmlMsub = msubsup.MmlMsubsup = void 0;
    var H = requireMmlNode(), V = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "msubsup";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "arity", {
        get: function() {
          return 3;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "base", {
        get: function() {
          return 0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "sub", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "sup", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.setChildInheritedAttributes = function(te, ne, ee, ae) {
        var oe = this.childNodes;
        oe[0].setInheritedAttributes(te, ne, ee, ae), oe[1].setInheritedAttributes(te, !1, ee + 1, ae || this.sub === 1), oe[2] && oe[2].setInheritedAttributes(te, !1, ee + 1, ae || this.sub === 2);
      }, Z.defaults = _(_({}, H.AbstractMmlBaseNode.defaults), { subscriptshift: "", superscriptshift: "" }), Z;
    }(H.AbstractMmlBaseNode);
    msubsup.MmlMsubsup = V;
    var q = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "msub";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "arity", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Z.defaults = _({}, V.defaults), Z;
    }(V);
    msubsup.MmlMsub = q;
    var G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "msup";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "arity", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "sup", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "sub", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Z.defaults = _({}, V.defaults), Z;
    }(V);
    return msubsup.MmlMsup = G, msubsup;
  }
  var hasRequiredMsubsup;
  function requireMsubsup() {
    if (hasRequiredMsubsup) return msubsup$2;
    hasRequiredMsubsup = 1;
    var O = msubsup$2.__extends || /* @__PURE__ */ function() {
      var ee = function(ae, oe) {
        return ee = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(se, ce) {
          se.__proto__ = ce;
        } || function(se, ce) {
          for (var le in ce) Object.prototype.hasOwnProperty.call(ce, le) && (se[le] = ce[le]);
        }, ee(ae, oe);
      };
      return function(ae, oe) {
        if (typeof oe != "function" && oe !== null)
          throw new TypeError("Class extends value " + String(oe) + " is not a constructor or null");
        ee(ae, oe);
        function se() {
          this.constructor = ae;
        }
        ae.prototype = oe === null ? Object.create(oe) : (se.prototype = oe.prototype, new se());
      };
    }(), _ = msubsup$2.__read || function(ee, ae) {
      var oe = typeof Symbol == "function" && ee[Symbol.iterator];
      if (!oe) return ee;
      var se = oe.call(ee), ce, le = [], ue;
      try {
        for (; (ae === void 0 || ae-- > 0) && !(ce = se.next()).done; ) le.push(ce.value);
      } catch (de) {
        ue = { error: de };
      } finally {
        try {
          ce && !ce.done && (oe = se.return) && oe.call(se);
        } finally {
          if (ue) throw ue.error;
        }
      }
      return le;
    };
    Object.defineProperty(msubsup$2, "__esModule", { value: !0 }), msubsup$2.SVGmsubsup = msubsup$2.SVGmsup = msubsup$2.SVGmsub = void 0;
    var H = requireScriptbase(), V = requireMsubsup$2(), q = requireMsubsup$2(), G = requireMsubsup$2(), J = requireMsubsup$1(), Z = function(ee) {
      O(ae, ee);
      function ae() {
        return ee !== null && ee.apply(this, arguments) || this;
      }
      return ae.kind = J.MmlMsub.prototype.kind, ae;
    }((0, V.CommonMsubMixin)(H.SVGscriptbase));
    msubsup$2.SVGmsub = Z;
    var te = function(ee) {
      O(ae, ee);
      function ae() {
        return ee !== null && ee.apply(this, arguments) || this;
      }
      return ae.kind = J.MmlMsup.prototype.kind, ae;
    }((0, q.CommonMsupMixin)(H.SVGscriptbase));
    msubsup$2.SVGmsup = te;
    var ne = function(ee) {
      O(ae, ee);
      function ae() {
        return ee !== null && ee.apply(this, arguments) || this;
      }
      return ae.prototype.toSVG = function(oe) {
        var se = this.standardSVGnode(oe), ce = _([this.baseChild, this.supChild, this.subChild], 3), le = ce[0], ue = ce[1], de = ce[2], fe = this.getBaseWidth(), he = this.getAdjustedIc(), me = _(this.getUVQ(), 2), pe = me[0], ye = me[1];
        le.toSVG(se), ue.toSVG(se), de.toSVG(se), de.place(fe, ye), ue.place(fe + he, pe);
      }, ae.kind = J.MmlMsubsup.prototype.kind, ae;
    }((0, G.CommonMsubsupMixin)(H.SVGscriptbase));
    return msubsup$2.SVGmsubsup = ne, msubsup$2;
  }
  var munderover$2 = {}, munderover$1 = {}, hasRequiredMunderover$2;
  function requireMunderover$2() {
    if (hasRequiredMunderover$2) return munderover$1;
    hasRequiredMunderover$2 = 1;
    var O = munderover$1.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = munderover$1.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    }, H = munderover$1.__spreadArray || function(J, Z, te) {
      if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
        (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
      return J.concat(ae || Array.prototype.slice.call(Z));
    };
    Object.defineProperty(munderover$1, "__esModule", { value: !0 }), munderover$1.CommonMunderoverMixin = munderover$1.CommonMoverMixin = munderover$1.CommonMunderMixin = void 0;
    function V(J) {
      return function(Z) {
        O(te, Z);
        function te() {
          for (var ne = [], ee = 0; ee < arguments.length; ee++)
            ne[ee] = arguments[ee];
          var ae = Z.apply(this, H([], _(ne), !1)) || this;
          return ae.stretchChildren(), ae;
        }
        return Object.defineProperty(te.prototype, "scriptChild", {
          get: function() {
            return this.childNodes[this.node.under];
          },
          enumerable: !1,
          configurable: !0
        }), te.prototype.computeBBox = function(ne, ee) {
          if (ee === void 0 && (ee = !1), this.hasMovableLimits()) {
            Z.prototype.computeBBox.call(this, ne, ee);
            return;
          }
          ne.empty();
          var ae = this.baseChild.getOuterBBox(), oe = this.scriptChild.getOuterBBox(), se = this.getUnderKV(ae, oe)[1], ce = this.isLineBelow ? 0 : this.getDelta(!0), le = _(this.getDeltaW([ae, oe], [0, -ce]), 2), ue = le[0], de = le[1];
          ne.combine(ae, ue, 0), ne.combine(oe, de, se), ne.d += this.font.params.big_op_spacing5, ne.clean(), this.setChildPWidths(ee);
        }, te;
      }(J);
    }
    munderover$1.CommonMunderMixin = V;
    function q(J) {
      return function(Z) {
        O(te, Z);
        function te() {
          for (var ne = [], ee = 0; ee < arguments.length; ee++)
            ne[ee] = arguments[ee];
          var ae = Z.apply(this, H([], _(ne), !1)) || this;
          return ae.stretchChildren(), ae;
        }
        return Object.defineProperty(te.prototype, "scriptChild", {
          get: function() {
            return this.childNodes[this.node.over];
          },
          enumerable: !1,
          configurable: !0
        }), te.prototype.computeBBox = function(ne) {
          if (this.hasMovableLimits()) {
            Z.prototype.computeBBox.call(this, ne);
            return;
          }
          ne.empty();
          var ee = this.baseChild.getOuterBBox(), ae = this.scriptChild.getOuterBBox();
          this.node.attributes.get("accent") && (ee.h = Math.max(ee.h, this.font.params.x_height * ee.scale));
          var oe = this.getOverKU(ee, ae)[1], se = this.isLineAbove ? 0 : this.getDelta(), ce = _(this.getDeltaW([ee, ae], [0, se]), 2), le = ce[0], ue = ce[1];
          ne.combine(ee, le, 0), ne.combine(ae, ue, oe), ne.h += this.font.params.big_op_spacing5, ne.clean();
        }, te;
      }(J);
    }
    munderover$1.CommonMoverMixin = q;
    function G(J) {
      return function(Z) {
        O(te, Z);
        function te() {
          for (var ne = [], ee = 0; ee < arguments.length; ee++)
            ne[ee] = arguments[ee];
          var ae = Z.apply(this, H([], _(ne), !1)) || this;
          return ae.stretchChildren(), ae;
        }
        return Object.defineProperty(te.prototype, "underChild", {
          get: function() {
            return this.childNodes[this.node.under];
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(te.prototype, "overChild", {
          get: function() {
            return this.childNodes[this.node.over];
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(te.prototype, "subChild", {
          get: function() {
            return this.underChild;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(te.prototype, "supChild", {
          get: function() {
            return this.overChild;
          },
          enumerable: !1,
          configurable: !0
        }), te.prototype.computeBBox = function(ne) {
          if (this.hasMovableLimits()) {
            Z.prototype.computeBBox.call(this, ne);
            return;
          }
          ne.empty();
          var ee = this.overChild.getOuterBBox(), ae = this.baseChild.getOuterBBox(), oe = this.underChild.getOuterBBox();
          this.node.attributes.get("accent") && (ae.h = Math.max(ae.h, this.font.params.x_height * ae.scale));
          var se = this.getOverKU(ae, ee)[1], ce = this.getUnderKV(ae, oe)[1], le = this.getDelta(), ue = _(this.getDeltaW([ae, oe, ee], [0, this.isLineBelow ? 0 : -le, this.isLineAbove ? 0 : le]), 3), de = ue[0], fe = ue[1], he = ue[2];
          ne.combine(ae, de, 0), ne.combine(ee, he, se), ne.combine(oe, fe, ce);
          var me = this.font.params.big_op_spacing5;
          ne.h += me, ne.d += me, ne.clean();
        }, te;
      }(J);
    }
    return munderover$1.CommonMunderoverMixin = G, munderover$1;
  }
  var munderover = {}, hasRequiredMunderover$1;
  function requireMunderover$1() {
    if (hasRequiredMunderover$1) return munderover;
    hasRequiredMunderover$1 = 1;
    var O = munderover.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = munderover.__assign || function() {
      return _ = Object.assign || function(J) {
        for (var Z, te = 1, ne = arguments.length; te < ne; te++) {
          Z = arguments[te];
          for (var ee in Z) Object.prototype.hasOwnProperty.call(Z, ee) && (J[ee] = Z[ee]);
        }
        return J;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(munderover, "__esModule", { value: !0 }), munderover.MmlMover = munderover.MmlMunder = munderover.MmlMunderover = void 0;
    var H = requireMmlNode(), V = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "munderover";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "arity", {
        get: function() {
          return 3;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "base", {
        get: function() {
          return 0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "under", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "over", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.setChildInheritedAttributes = function(te, ne, ee, ae) {
        var oe = this.childNodes;
        oe[0].setInheritedAttributes(te, ne, ee, ae || !!oe[this.over]);
        var se = !!(!ne && oe[0].coreMO().attributes.get("movablelimits")), ce = this.constructor.ACCENTS;
        oe[1].setInheritedAttributes(te, !1, this.getScriptlevel(ce[1], se, ee), ae || this.under === 1), this.setInheritedAccent(1, ce[1], ne, ee, ae, se), oe[2] && (oe[2].setInheritedAttributes(te, !1, this.getScriptlevel(ce[2], se, ee), ae || this.under === 2), this.setInheritedAccent(2, ce[2], ne, ee, ae, se));
      }, Z.prototype.getScriptlevel = function(te, ne, ee) {
        return (ne || !this.attributes.get(te)) && ee++, ee;
      }, Z.prototype.setInheritedAccent = function(te, ne, ee, ae, oe, se) {
        var ce = this.childNodes[te];
        if (this.attributes.getExplicit(ne) == null && ce.isEmbellished) {
          var le = ce.coreMO().attributes.get("accent");
          this.attributes.setInherited(ne, le), le !== this.attributes.getDefault(ne) && ce.setInheritedAttributes({}, ee, this.getScriptlevel(ne, se, ae), oe);
        }
      }, Z.defaults = _(_({}, H.AbstractMmlBaseNode.defaults), { accent: !1, accentunder: !1, align: "center" }), Z.ACCENTS = ["", "accentunder", "accent"], Z;
    }(H.AbstractMmlBaseNode);
    munderover.MmlMunderover = V;
    var q = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "munder";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "arity", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Z.defaults = _({}, V.defaults), Z;
    }(V);
    munderover.MmlMunder = q;
    var G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "mover";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "arity", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "over", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "under", {
        get: function() {
          return 2;
        },
        enumerable: !1,
        configurable: !0
      }), Z.defaults = _({}, V.defaults), Z.ACCENTS = ["", "accent", "accentunder"], Z;
    }(V);
    return munderover.MmlMover = G, munderover;
  }
  var hasRequiredMunderover;
  function requireMunderover() {
    if (hasRequiredMunderover) return munderover$2;
    hasRequiredMunderover = 1;
    var O = munderover$2.__extends || /* @__PURE__ */ function() {
      var ee = function(ae, oe) {
        return ee = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(se, ce) {
          se.__proto__ = ce;
        } || function(se, ce) {
          for (var le in ce) Object.prototype.hasOwnProperty.call(ce, le) && (se[le] = ce[le]);
        }, ee(ae, oe);
      };
      return function(ae, oe) {
        if (typeof oe != "function" && oe !== null)
          throw new TypeError("Class extends value " + String(oe) + " is not a constructor or null");
        ee(ae, oe);
        function se() {
          this.constructor = ae;
        }
        ae.prototype = oe === null ? Object.create(oe) : (se.prototype = oe.prototype, new se());
      };
    }(), _ = munderover$2.__read || function(ee, ae) {
      var oe = typeof Symbol == "function" && ee[Symbol.iterator];
      if (!oe) return ee;
      var se = oe.call(ee), ce, le = [], ue;
      try {
        for (; (ae === void 0 || ae-- > 0) && !(ce = se.next()).done; ) le.push(ce.value);
      } catch (de) {
        ue = { error: de };
      } finally {
        try {
          ce && !ce.done && (oe = se.return) && oe.call(se);
        } finally {
          if (ue) throw ue.error;
        }
      }
      return le;
    };
    Object.defineProperty(munderover$2, "__esModule", { value: !0 }), munderover$2.SVGmunderover = munderover$2.SVGmover = munderover$2.SVGmunder = void 0;
    var H = requireMsubsup(), V = requireMunderover$2(), q = requireMunderover$2(), G = requireMunderover$2(), J = requireMunderover$1(), Z = function(ee) {
      O(ae, ee);
      function ae() {
        return ee !== null && ee.apply(this, arguments) || this;
      }
      return ae.prototype.toSVG = function(oe) {
        if (this.hasMovableLimits()) {
          ee.prototype.toSVG.call(this, oe);
          return;
        }
        var se = this.standardSVGnode(oe), ce = _([this.baseChild, this.scriptChild], 2), le = ce[0], ue = ce[1], de = _([le.getOuterBBox(), ue.getOuterBBox()], 2), fe = de[0], he = de[1];
        le.toSVG(se), ue.toSVG(se);
        var me = this.isLineBelow ? 0 : this.getDelta(!0), pe = this.getUnderKV(fe, he)[1], ye = _(this.getDeltaW([fe, he], [0, -me]), 2), ge = ye[0], ve = ye[1];
        le.place(ge, 0), ue.place(ve, pe);
      }, ae.kind = J.MmlMunder.prototype.kind, ae;
    }((0, V.CommonMunderMixin)(H.SVGmsub));
    munderover$2.SVGmunder = Z;
    var te = function(ee) {
      O(ae, ee);
      function ae() {
        return ee !== null && ee.apply(this, arguments) || this;
      }
      return ae.prototype.toSVG = function(oe) {
        if (this.hasMovableLimits()) {
          ee.prototype.toSVG.call(this, oe);
          return;
        }
        var se = this.standardSVGnode(oe), ce = _([this.baseChild, this.scriptChild], 2), le = ce[0], ue = ce[1], de = _([le.getOuterBBox(), ue.getOuterBBox()], 2), fe = de[0], he = de[1];
        le.toSVG(se), ue.toSVG(se);
        var me = this.isLineAbove ? 0 : this.getDelta(), pe = this.getOverKU(fe, he)[1], ye = _(this.getDeltaW([fe, he], [0, me]), 2), ge = ye[0], ve = ye[1];
        le.place(ge, 0), ue.place(ve, pe);
      }, ae.kind = J.MmlMover.prototype.kind, ae;
    }((0, q.CommonMoverMixin)(H.SVGmsup));
    munderover$2.SVGmover = te;
    var ne = function(ee) {
      O(ae, ee);
      function ae() {
        return ee !== null && ee.apply(this, arguments) || this;
      }
      return ae.prototype.toSVG = function(oe) {
        if (this.hasMovableLimits()) {
          ee.prototype.toSVG.call(this, oe);
          return;
        }
        var se = this.standardSVGnode(oe), ce = _([this.baseChild, this.overChild, this.underChild], 3), le = ce[0], ue = ce[1], de = ce[2], fe = _([le.getOuterBBox(), ue.getOuterBBox(), de.getOuterBBox()], 3), he = fe[0], me = fe[1], pe = fe[2];
        le.toSVG(se), de.toSVG(se), ue.toSVG(se);
        var ye = this.getDelta(), ge = this.getOverKU(he, me)[1], ve = this.getUnderKV(he, pe)[1], be = _(this.getDeltaW([he, pe, me], [0, this.isLineBelow ? 0 : -ye, this.isLineAbove ? 0 : ye]), 3), Ce = be[0], Se = be[1], we = be[2];
        le.place(Ce, 0), de.place(Se, ve), ue.place(we, ge);
      }, ae.kind = J.MmlMunderover.prototype.kind, ae;
    }((0, G.CommonMunderoverMixin)(H.SVGmsubsup));
    return munderover$2.SVGmunderover = ne, munderover$2;
  }
  var mmultiscripts$2 = {}, mmultiscripts$1 = {}, hasRequiredMmultiscripts$2;
  function requireMmultiscripts$2() {
    return hasRequiredMmultiscripts$2 || (hasRequiredMmultiscripts$2 = 1, function(O) {
      var _ = mmultiscripts$1.__extends || /* @__PURE__ */ function() {
        var Z = function(te, ne) {
          return Z = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ee, ae) {
            ee.__proto__ = ae;
          } || function(ee, ae) {
            for (var oe in ae) Object.prototype.hasOwnProperty.call(ae, oe) && (ee[oe] = ae[oe]);
          }, Z(te, ne);
        };
        return function(te, ne) {
          if (typeof ne != "function" && ne !== null)
            throw new TypeError("Class extends value " + String(ne) + " is not a constructor or null");
          Z(te, ne);
          function ee() {
            this.constructor = te;
          }
          te.prototype = ne === null ? Object.create(ne) : (ee.prototype = ne.prototype, new ee());
        };
      }(), H = mmultiscripts$1.__read || function(Z, te) {
        var ne = typeof Symbol == "function" && Z[Symbol.iterator];
        if (!ne) return Z;
        var ee = ne.call(Z), ae, oe = [], se;
        try {
          for (; (te === void 0 || te-- > 0) && !(ae = ee.next()).done; ) oe.push(ae.value);
        } catch (ce) {
          se = { error: ce };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (se) throw se.error;
          }
        }
        return oe;
      }, V = mmultiscripts$1.__spreadArray || function(Z, te, ne) {
        if (ne || arguments.length === 2) for (var ee = 0, ae = te.length, oe; ee < ae; ee++)
          (oe || !(ee in te)) && (oe || (oe = Array.prototype.slice.call(te, 0, ee)), oe[ee] = te[ee]);
        return Z.concat(oe || Array.prototype.slice.call(te));
      }, q = mmultiscripts$1.__values || function(Z) {
        var te = typeof Symbol == "function" && Symbol.iterator, ne = te && Z[te], ee = 0;
        if (ne) return ne.call(Z);
        if (Z && typeof Z.length == "number") return {
          next: function() {
            return Z && ee >= Z.length && (Z = void 0), { value: Z && Z[ee++], done: !Z };
          }
        };
        throw new TypeError(te ? "Object is not iterable." : "Symbol.iterator is not defined.");
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.CommonMmultiscriptsMixin = O.ScriptNames = O.NextScript = void 0;
      var G = requireBBox();
      O.NextScript = {
        base: "subList",
        subList: "supList",
        supList: "subList",
        psubList: "psupList",
        psupList: "psubList"
      }, O.ScriptNames = ["sup", "sup", "psup", "psub"];
      function J(Z) {
        return function(te) {
          _(ne, te);
          function ne() {
            for (var ee = [], ae = 0; ae < arguments.length; ae++)
              ee[ae] = arguments[ae];
            var oe = te.apply(this, V([], H(ee), !1)) || this;
            return oe.scriptData = null, oe.firstPrescript = 0, oe.getScriptData(), oe;
          }
          return ne.prototype.combinePrePost = function(ee, ae) {
            var oe = new G.BBox(ee);
            return oe.combine(ae, 0, 0), oe;
          }, ne.prototype.computeBBox = function(ee, ae) {
            ae === void 0 && (ae = !1);
            var oe = this.font.params.scriptspace, se = this.scriptData, ce = this.combinePrePost(se.sub, se.psub), le = this.combinePrePost(se.sup, se.psup), ue = H(this.getUVQ(ce, le), 2), de = ue[0], fe = ue[1];
            if (ee.empty(), se.numPrescripts && (ee.combine(se.psup, oe, de), ee.combine(se.psub, oe, fe)), ee.append(se.base), se.numScripts) {
              var he = ee.w;
              ee.combine(se.sup, he, de), ee.combine(se.sub, he, fe), ee.w += oe;
            }
            ee.clean(), this.setChildPWidths(ae);
          }, ne.prototype.getScriptData = function() {
            var ee = this.scriptData = {
              base: null,
              sub: G.BBox.empty(),
              sup: G.BBox.empty(),
              psub: G.BBox.empty(),
              psup: G.BBox.empty(),
              numPrescripts: 0,
              numScripts: 0
            }, ae = this.getScriptBBoxLists();
            this.combineBBoxLists(ee.sub, ee.sup, ae.subList, ae.supList), this.combineBBoxLists(ee.psub, ee.psup, ae.psubList, ae.psupList), ee.base = ae.base[0], ee.numPrescripts = ae.psubList.length, ee.numScripts = ae.subList.length;
          }, ne.prototype.getScriptBBoxLists = function() {
            var ee, ae, oe = {
              base: [],
              subList: [],
              supList: [],
              psubList: [],
              psupList: []
            }, se = "base";
            try {
              for (var ce = q(this.childNodes), le = ce.next(); !le.done; le = ce.next()) {
                var ue = le.value;
                ue.node.isKind("mprescripts") ? se = "psubList" : (oe[se].push(ue.getOuterBBox()), se = O.NextScript[se]);
              }
            } catch (de) {
              ee = { error: de };
            } finally {
              try {
                le && !le.done && (ae = ce.return) && ae.call(ce);
              } finally {
                if (ee) throw ee.error;
              }
            }
            return this.firstPrescript = oe.subList.length + oe.supList.length + 2, this.padLists(oe.subList, oe.supList), this.padLists(oe.psubList, oe.psupList), oe;
          }, ne.prototype.padLists = function(ee, ae) {
            ee.length > ae.length && ae.push(G.BBox.empty());
          }, ne.prototype.combineBBoxLists = function(ee, ae, oe, se) {
            for (var ce = 0; ce < oe.length; ce++) {
              var le = H(this.getScaledWHD(oe[ce]), 3), ue = le[0], de = le[1], fe = le[2], he = H(this.getScaledWHD(se[ce]), 3), me = he[0], pe = he[1], ye = he[2], ge = Math.max(ue, me);
              ee.w += ge, ae.w += ge, de > ee.h && (ee.h = de), fe > ee.d && (ee.d = fe), pe > ae.h && (ae.h = pe), ye > ae.d && (ae.d = ye);
            }
          }, ne.prototype.getScaledWHD = function(ee) {
            var ae = ee.w, oe = ee.h, se = ee.d, ce = ee.rscale;
            return [ae * ce, oe * ce, se * ce];
          }, ne.prototype.getUVQ = function(ee, ae) {
            var oe;
            if (!this.UVQ) {
              var se = H([0, 0, 0], 3), ce = se[0], le = se[1], ue = se[2];
              ee.h === 0 && ee.d === 0 ? ce = this.getU() : ae.h === 0 && ae.d === 0 ? ce = -this.getV() : (oe = H(te.prototype.getUVQ.call(this, ee, ae), 3), ce = oe[0], le = oe[1], ue = oe[2]), this.UVQ = [ce, le, ue];
            }
            return this.UVQ;
          }, ne;
        }(Z);
      }
      O.CommonMmultiscriptsMixin = J;
    }(mmultiscripts$1)), mmultiscripts$1;
  }
  var mmultiscripts = {}, hasRequiredMmultiscripts$1;
  function requireMmultiscripts$1() {
    if (hasRequiredMmultiscripts$1) return mmultiscripts;
    hasRequiredMmultiscripts$1 = 1;
    var O = mmultiscripts.__extends || /* @__PURE__ */ function() {
      var Z = function(te, ne) {
        return Z = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ee, ae) {
          ee.__proto__ = ae;
        } || function(ee, ae) {
          for (var oe in ae) Object.prototype.hasOwnProperty.call(ae, oe) && (ee[oe] = ae[oe]);
        }, Z(te, ne);
      };
      return function(te, ne) {
        if (typeof ne != "function" && ne !== null)
          throw new TypeError("Class extends value " + String(ne) + " is not a constructor or null");
        Z(te, ne);
        function ee() {
          this.constructor = te;
        }
        te.prototype = ne === null ? Object.create(ne) : (ee.prototype = ne.prototype, new ee());
      };
    }(), _ = mmultiscripts.__assign || function() {
      return _ = Object.assign || function(Z) {
        for (var te, ne = 1, ee = arguments.length; ne < ee; ne++) {
          te = arguments[ne];
          for (var ae in te) Object.prototype.hasOwnProperty.call(te, ae) && (Z[ae] = te[ae]);
        }
        return Z;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mmultiscripts, "__esModule", { value: !0 }), mmultiscripts.MmlNone = mmultiscripts.MmlMprescripts = mmultiscripts.MmlMmultiscripts = void 0;
    var H = requireMmlNode(), V = requireMsubsup$1(), q = function(Z) {
      O(te, Z);
      function te() {
        return Z !== null && Z.apply(this, arguments) || this;
      }
      return Object.defineProperty(te.prototype, "kind", {
        get: function() {
          return "mmultiscripts";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(te.prototype, "arity", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), te.prototype.setChildInheritedAttributes = function(ne, ee, ae, oe) {
        this.childNodes[0].setInheritedAttributes(ne, ee, ae, oe);
        for (var se = !1, ce = 1, le = 0; ce < this.childNodes.length; ce++) {
          var ue = this.childNodes[ce];
          if (ue.isKind("mprescripts")) {
            if (!se && (se = !0, ce % 2 === 0)) {
              var de = this.factory.create("mrow");
              this.childNodes.splice(ce, 0, de), de.parent = this, ce++;
            }
          } else {
            var fe = oe || le % 2 === 0;
            ue.setInheritedAttributes(ne, !1, ae + 1, fe), le++;
          }
        }
        this.childNodes.length % 2 === (se ? 1 : 0) && (this.appendChild(this.factory.create("mrow")), this.childNodes[this.childNodes.length - 1].setInheritedAttributes(ne, !1, ae + 1, oe));
      }, te.prototype.verifyChildren = function(ne) {
        for (var ee = !1, ae = ne.fixMmultiscripts, oe = 0; oe < this.childNodes.length; oe++) {
          var se = this.childNodes[oe];
          se.isKind("mprescripts") && (ee ? se.mError(se.kind + " can only appear once in " + this.kind, ne, !0) : (ee = !0, oe % 2 === 0 && !ae && this.mError("There must be an equal number of prescripts of each type", ne)));
        }
        this.childNodes.length % 2 === (ee ? 1 : 0) && !ae && this.mError("There must be an equal number of scripts of each type", ne), Z.prototype.verifyChildren.call(this, ne);
      }, te.defaults = _({}, V.MmlMsubsup.defaults), te;
    }(V.MmlMsubsup);
    mmultiscripts.MmlMmultiscripts = q;
    var G = function(Z) {
      O(te, Z);
      function te() {
        return Z !== null && Z.apply(this, arguments) || this;
      }
      return Object.defineProperty(te.prototype, "kind", {
        get: function() {
          return "mprescripts";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(te.prototype, "arity", {
        get: function() {
          return 0;
        },
        enumerable: !1,
        configurable: !0
      }), te.prototype.verifyTree = function(ne) {
        Z.prototype.verifyTree.call(this, ne), this.parent && !this.parent.isKind("mmultiscripts") && this.mError(this.kind + " must be a child of mmultiscripts", ne, !0);
      }, te.defaults = _({}, H.AbstractMmlNode.defaults), te;
    }(H.AbstractMmlNode);
    mmultiscripts.MmlMprescripts = G;
    var J = function(Z) {
      O(te, Z);
      function te() {
        return Z !== null && Z.apply(this, arguments) || this;
      }
      return Object.defineProperty(te.prototype, "kind", {
        get: function() {
          return "none";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(te.prototype, "arity", {
        get: function() {
          return 0;
        },
        enumerable: !1,
        configurable: !0
      }), te.prototype.verifyTree = function(ne) {
        Z.prototype.verifyTree.call(this, ne), this.parent && !this.parent.isKind("mmultiscripts") && this.mError(this.kind + " must be a child of mmultiscripts", ne, !0);
      }, te.defaults = _({}, H.AbstractMmlNode.defaults), te;
    }(H.AbstractMmlNode);
    return mmultiscripts.MmlNone = J, mmultiscripts;
  }
  var hasRequiredMmultiscripts;
  function requireMmultiscripts() {
    if (hasRequiredMmultiscripts) return mmultiscripts$2;
    hasRequiredMmultiscripts = 1;
    var O = mmultiscripts$2.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = mmultiscripts$2.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    };
    Object.defineProperty(mmultiscripts$2, "__esModule", { value: !0 }), mmultiscripts$2.SVGmmultiscripts = mmultiscripts$2.AlignX = void 0;
    var H = requireMsubsup(), V = requireMmultiscripts$2(), q = requireMmultiscripts$1(), G = requireString();
    function J(te) {
      return {
        left: function(ne, ee) {
          return 0;
        },
        center: function(ne, ee) {
          return (ee - ne) / 2;
        },
        right: function(ne, ee) {
          return ee - ne;
        }
      }[te] || function(ne, ee) {
        return 0;
      };
    }
    mmultiscripts$2.AlignX = J;
    var Z = function(te) {
      O(ne, te);
      function ne() {
        return te !== null && te.apply(this, arguments) || this;
      }
      return ne.prototype.toSVG = function(ee) {
        var ae = this.standardSVGnode(ee), oe = this.scriptData, se = this.node.getProperty("scriptalign") || "right left", ce = _((0, G.split)(se + " " + se), 2), le = ce[0], ue = ce[1], de = this.combinePrePost(oe.sub, oe.psub), fe = this.combinePrePost(oe.sup, oe.psup), he = _(this.getUVQ(de, fe), 2), me = he[0], pe = he[1], ye = 0;
        oe.numPrescripts && (ye = this.addScripts(0.05, me, pe, this.firstPrescript, oe.numPrescripts, le));
        var ge = this.baseChild;
        ge.toSVG(ae), ge.place(ye, 0), ye += ge.getOuterBBox().w, oe.numScripts && this.addScripts(ye, me, pe, 1, oe.numScripts, ue);
      }, ne.prototype.addScripts = function(ee, ae, oe, se, ce, le) {
        var ue = this.adaptor, de = J(le), fe = ue.append(this.element, this.svg("g")), he = ue.append(this.element, this.svg("g"));
        this.place(ee, ae, fe), this.place(ee, oe, he);
        for (var me = se + 2 * ce, pe = 0; se < me; ) {
          var ye = _([this.childNodes[se++], this.childNodes[se++]], 2), ge = ye[0], ve = ye[1], be = _([ge.getOuterBBox(), ve.getOuterBBox()], 2), Ce = be[0], Se = be[1], we = _([Ce.rscale, Se.rscale], 2), Ee = we[0], xe = we[1], Le = Math.max(Ce.w * Ee, Se.w * xe);
          ge.toSVG(he), ve.toSVG(fe), ge.place(pe + de(Ce.w * Ee, Le), 0), ve.place(pe + de(Se.w * xe, Le), 0), pe += Le;
        }
        return ee + pe;
      }, ne.kind = q.MmlMmultiscripts.prototype.kind, ne;
    }((0, V.CommonMmultiscriptsMixin)(H.SVGmsubsup));
    return mmultiscripts$2.SVGmmultiscripts = Z, mmultiscripts$2;
  }
  var mtable$2 = {}, mtable$1 = {}, numeric = {}, hasRequiredNumeric;
  function requireNumeric() {
    if (hasRequiredNumeric) return numeric;
    hasRequiredNumeric = 1, Object.defineProperty(numeric, "__esModule", { value: !0 }), numeric.max = numeric.sum = void 0;
    function O(H) {
      return H.reduce(function(V, q) {
        return V + q;
      }, 0);
    }
    numeric.sum = O;
    function _(H) {
      return H.reduce(function(V, q) {
        return Math.max(V, q);
      }, 0);
    }
    return numeric.max = _, numeric;
  }
  var hasRequiredMtable$2;
  function requireMtable$2() {
    if (hasRequiredMtable$2) return mtable$1;
    hasRequiredMtable$2 = 1;
    var O = mtable$1.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = mtable$1.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    }, H = mtable$1.__spreadArray || function(te, ne, ee) {
      if (ee || arguments.length === 2) for (var ae = 0, oe = ne.length, se; ae < oe; ae++)
        (se || !(ae in ne)) && (se || (se = Array.prototype.slice.call(ne, 0, ae)), se[ae] = ne[ae]);
      return te.concat(se || Array.prototype.slice.call(ne));
    }, V = mtable$1.__values || function(te) {
      var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
      if (ee) return ee.call(te);
      if (te && typeof te.length == "number") return {
        next: function() {
          return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
        }
      };
      throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mtable$1, "__esModule", { value: !0 }), mtable$1.CommonMtableMixin = void 0;
    var q = requireBBox(), G = requireString(), J = requireNumeric();
    function Z(te) {
      return function(ne) {
        O(ee, ne);
        function ee() {
          for (var ae = [], oe = 0; oe < arguments.length; oe++)
            ae[oe] = arguments[oe];
          var se = ne.apply(this, H([], _(ae), !1)) || this;
          se.numCols = 0, se.numRows = 0, se.data = null, se.pwidthCells = [], se.pWidth = 0, se.numCols = (0, J.max)(se.tableRows.map(function(le) {
            return le.numCells;
          })), se.numRows = se.childNodes.length, se.hasLabels = se.childNodes.reduce(function(le, ue) {
            return le || ue.node.isKind("mlabeledtr");
          }, !1), se.findContainer(), se.isTop = !se.container || se.container.node.isKind("math") && !se.container.parent, se.isTop && (se.jax.table = se), se.getPercentageWidth();
          var ce = se.node.attributes;
          return se.frame = ce.get("frame") !== "none", se.fLine = se.frame && ce.get("frame") ? 0.07 : 0, se.fSpace = se.frame ? se.convertLengths(se.getAttributeArray("framespacing")) : [0, 0], se.cSpace = se.convertLengths(se.getColumnAttributes("columnspacing")), se.rSpace = se.convertLengths(se.getRowAttributes("rowspacing")), se.cLines = se.getColumnAttributes("columnlines").map(function(le) {
            return le === "none" ? 0 : 0.07;
          }), se.rLines = se.getRowAttributes("rowlines").map(function(le) {
            return le === "none" ? 0 : 0.07;
          }), se.cWidths = se.getColumnWidths(), se.stretchRows(), se.stretchColumns(), se;
        }
        return Object.defineProperty(ee.prototype, "tableRows", {
          get: function() {
            return this.childNodes;
          },
          enumerable: !1,
          configurable: !0
        }), ee.prototype.findContainer = function() {
          for (var ae = this, oe = ae.parent; oe && (oe.node.notParent || oe.node.isKind("mrow")); )
            ae = oe, oe = oe.parent;
          this.container = oe, this.containerI = ae.node.childPosition();
        }, ee.prototype.getPercentageWidth = function() {
          if (this.hasLabels)
            this.bbox.pwidth = q.BBox.fullWidth;
          else {
            var ae = this.node.attributes.get("width");
            (0, G.isPercent)(ae) && (this.bbox.pwidth = ae);
          }
        }, ee.prototype.stretchRows = function() {
          for (var ae = this.node.attributes.get("equalrows"), oe = ae ? this.getEqualRowHeight() : 0, se = ae ? this.getTableData() : { H: [0], D: [0] }, ce = se.H, le = se.D, ue = this.tableRows, de = 0; de < this.numRows; de++) {
            var fe = ae ? [(oe + ce[de] - le[de]) / 2, (oe - ce[de] + le[de]) / 2] : null;
            ue[de].stretchChildren(fe);
          }
        }, ee.prototype.stretchColumns = function() {
          for (var ae = 0; ae < this.numCols; ae++) {
            var oe = typeof this.cWidths[ae] == "number" ? this.cWidths[ae] : null;
            this.stretchColumn(ae, oe);
          }
        }, ee.prototype.stretchColumn = function(ae, oe) {
          var se, ce, le, ue, de, fe, he = [];
          try {
            for (var me = V(this.tableRows), pe = me.next(); !pe.done; pe = me.next()) {
              var ye = pe.value, ge = ye.getChild(ae);
              if (ge) {
                var ve = ge.childNodes[0];
                ve.stretch.dir === 0 && ve.canStretch(2) && he.push(ve);
              }
            }
          } catch (Me) {
            se = { error: Me };
          } finally {
            try {
              pe && !pe.done && (ce = me.return) && ce.call(me);
            } finally {
              if (se) throw se.error;
            }
          }
          var be = he.length, Ce = this.childNodes.length;
          if (be && Ce > 1) {
            if (oe === null) {
              oe = 0;
              var Se = be > 1 && be === Ce;
              try {
                for (var we = V(this.tableRows), Ee = we.next(); !Ee.done; Ee = we.next()) {
                  var ye = Ee.value, ge = ye.getChild(ae);
                  if (ge) {
                    var ve = ge.childNodes[0], xe = ve.stretch.dir === 0;
                    if (Se || xe) {
                      var Le = ve.getBBox(xe).w;
                      Le > oe && (oe = Le);
                    }
                  }
                }
              } catch (Me) {
                le = { error: Me };
              } finally {
                try {
                  Ee && !Ee.done && (ue = we.return) && ue.call(we);
                } finally {
                  if (le) throw le.error;
                }
              }
            }
            try {
              for (var Qe = V(he), ke = Qe.next(); !ke.done; ke = Qe.next()) {
                var ve = ke.value;
                ve.coreMO().getStretchedVariant([oe]);
              }
            } catch (Me) {
              de = { error: Me };
            } finally {
              try {
                ke && !ke.done && (fe = Qe.return) && fe.call(Qe);
              } finally {
                if (de) throw de.error;
              }
            }
          }
        }, ee.prototype.getTableData = function() {
          if (this.data)
            return this.data;
          for (var ae = new Array(this.numRows).fill(0), oe = new Array(this.numRows).fill(0), se = new Array(this.numCols).fill(0), ce = new Array(this.numRows), le = new Array(this.numRows), ue = [0], de = this.tableRows, fe = 0; fe < de.length; fe++) {
            for (var he = 0, me = de[fe], pe = me.node.attributes.get("rowalign"), ye = 0; ye < me.numCells; ye++) {
              var ge = me.getChild(ye);
              he = this.updateHDW(ge, ye, fe, pe, ae, oe, se, he), this.recordPWidthCell(ge, ye);
            }
            ce[fe] = ae[fe], le[fe] = oe[fe], me.labeled && (he = this.updateHDW(me.childNodes[0], 0, fe, pe, ae, oe, ue, he)), this.extendHD(fe, ae, oe, he), this.extendHD(fe, ce, le, he);
          }
          var ve = ue[0];
          return this.data = { H: ae, D: oe, W: se, NH: ce, ND: le, L: ve }, this.data;
        }, ee.prototype.updateHDW = function(ae, oe, se, ce, le, ue, de, fe) {
          var he = ae.getBBox(), me = he.h, pe = he.d, ye = he.w, ge = ae.parent.bbox.rscale;
          ae.parent.bbox.rscale !== 1 && (me *= ge, pe *= ge, ye *= ge), this.node.getProperty("useHeight") && (me < 0.75 && (me = 0.75), pe < 0.25 && (pe = 0.25));
          var ve = 0;
          return ce = ae.node.attributes.get("rowalign") || ce, ce !== "baseline" && ce !== "axis" && (ve = me + pe, me = pe = 0), me > le[se] && (le[se] = me), pe > ue[se] && (ue[se] = pe), ve > fe && (fe = ve), de && ye > de[oe] && (de[oe] = ye), fe;
        }, ee.prototype.extendHD = function(ae, oe, se, ce) {
          var le = (ce - (oe[ae] + se[ae])) / 2;
          le < 1e-5 || (oe[ae] += le, se[ae] += le);
        }, ee.prototype.recordPWidthCell = function(ae, oe) {
          ae.childNodes[0] && ae.childNodes[0].getBBox().pwidth && this.pwidthCells.push([ae, oe]);
        }, ee.prototype.computeBBox = function(ae, oe) {
          var se = this.getTableData(), ce = se.H, le = se.D, ue, de;
          if (this.node.attributes.get("equalrows")) {
            var fe = this.getEqualRowHeight();
            ue = (0, J.sum)([].concat(this.rLines, this.rSpace)) + fe * this.numRows;
          } else
            ue = (0, J.sum)(ce.concat(le, this.rLines, this.rSpace));
          ue += 2 * (this.fLine + this.fSpace[1]);
          var he = this.getComputedWidths();
          de = (0, J.sum)(he.concat(this.cLines, this.cSpace)) + 2 * (this.fLine + this.fSpace[0]);
          var me = this.node.attributes.get("width");
          me !== "auto" && (de = Math.max(this.length2em(me, 0) + 2 * this.fLine, de));
          var pe = _(this.getBBoxHD(ue), 2), ye = pe[0], ge = pe[1];
          ae.h = ye, ae.d = ge, ae.w = de;
          var ve = _(this.getBBoxLR(), 2), be = ve[0], Ce = ve[1];
          ae.L = be, ae.R = Ce, (0, G.isPercent)(me) || this.setColumnPWidths();
        }, ee.prototype.setChildPWidths = function(ae, oe, se) {
          var ce = this.node.attributes.get("width");
          if (!(0, G.isPercent)(ce))
            return !1;
          this.hasLabels || (this.bbox.pwidth = "", this.container.bbox.pwidth = "");
          var le = this.bbox, ue = le.w, de = le.L, fe = le.R, he = this.node.attributes.get("data-width-includes-label"), me = Math.max(ue, this.length2em(ce, Math.max(oe, de + ue + fe))) - (he ? de + fe : 0), pe = this.node.attributes.get("equalcolumns") ? Array(this.numCols).fill(this.percent(1 / Math.max(1, this.numCols))) : this.getColumnAttributes("columnwidth", 0);
          this.cWidths = this.getColumnWidthsFixed(pe, me);
          var ye = this.getComputedWidths();
          return this.pWidth = (0, J.sum)(ye.concat(this.cLines, this.cSpace)) + 2 * (this.fLine + this.fSpace[0]), this.isTop && (this.bbox.w = this.pWidth), this.setColumnPWidths(), this.pWidth !== ue && this.parent.invalidateBBox(), this.pWidth !== ue;
        }, ee.prototype.setColumnPWidths = function() {
          var ae, oe, se = this.cWidths;
          try {
            for (var ce = V(this.pwidthCells), le = ce.next(); !le.done; le = ce.next()) {
              var ue = _(le.value, 2), de = ue[0], fe = ue[1];
              de.setChildPWidths(!1, se[fe]) && (de.invalidateBBox(), de.getBBox());
            }
          } catch (he) {
            ae = { error: he };
          } finally {
            try {
              le && !le.done && (oe = ce.return) && oe.call(ce);
            } finally {
              if (ae) throw ae.error;
            }
          }
        }, ee.prototype.getBBoxHD = function(ae) {
          var oe = _(this.getAlignmentRow(), 2), se = oe[0], ce = oe[1];
          if (ce === null) {
            var le = this.font.params.axis_height, ue = ae / 2, de = {
              top: [0, ae],
              center: [ue, ue],
              bottom: [ae, 0],
              baseline: [ue, ue],
              axis: [ue + le, ue - le]
            };
            return de[se] || [ue, ue];
          } else {
            var fe = this.getVerticalPosition(ce, se);
            return [fe, ae - fe];
          }
        }, ee.prototype.getBBoxLR = function() {
          if (this.hasLabels) {
            var ae = this.node.attributes, oe = ae.get("side"), se = _(this.getPadAlignShift(oe), 2), ce = se[0], le = se[1], ue = this.hasLabels && !!ae.get("data-width-includes-label");
            return ue && this.frame && this.fSpace[0] && (ce -= this.fSpace[0]), le === "center" && !ue ? [ce, ce] : oe === "left" ? [ce, 0] : [0, ce];
          }
          return [0, 0];
        }, ee.prototype.getPadAlignShift = function(ae) {
          var oe = this.getTableData().L, se = this.length2em(this.node.attributes.get("minlabelspacing")), ce = oe + se, le = _(this.styles == null ? ["", ""] : [this.styles.get("padding-left"), this.styles.get("padding-right")], 2), ue = le[0], de = le[1];
          (ue || de) && (ce = Math.max(ce, this.length2em(ue || "0"), this.length2em(de || "0")));
          var fe = _(this.getAlignShift(), 2), he = fe[0], me = fe[1];
          return he === ae && (me = ae === "left" ? Math.max(ce, me) - ce : Math.min(-ce, me) + ce), [ce, he, me];
        }, ee.prototype.getAlignShift = function() {
          return this.isTop ? ne.prototype.getAlignShift.call(this) : [this.container.getChildAlign(this.containerI), 0];
        }, ee.prototype.getWidth = function() {
          return this.pWidth || this.getBBox().w;
        }, ee.prototype.getEqualRowHeight = function() {
          var ae = this.getTableData(), oe = ae.H, se = ae.D, ce = Array.from(oe.keys()).map(function(le) {
            return oe[le] + se[le];
          });
          return Math.max.apply(Math, ce);
        }, ee.prototype.getComputedWidths = function() {
          var ae = this, oe = this.getTableData().W, se = Array.from(oe.keys()).map(function(ce) {
            return typeof ae.cWidths[ce] == "number" ? ae.cWidths[ce] : oe[ce];
          });
          return this.node.attributes.get("equalcolumns") && (se = Array(se.length).fill((0, J.max)(se))), se;
        }, ee.prototype.getColumnWidths = function() {
          var ae = this.node.attributes.get("width");
          if (this.node.attributes.get("equalcolumns"))
            return this.getEqualColumns(ae);
          var oe = this.getColumnAttributes("columnwidth", 0);
          return ae === "auto" ? this.getColumnWidthsAuto(oe) : (0, G.isPercent)(ae) ? this.getColumnWidthsPercent(oe) : this.getColumnWidthsFixed(oe, this.length2em(ae));
        }, ee.prototype.getEqualColumns = function(ae) {
          var oe = Math.max(1, this.numCols), se;
          if (ae === "auto") {
            var ce = this.getTableData().W;
            se = (0, J.max)(ce);
          } else if ((0, G.isPercent)(ae))
            se = this.percent(1 / oe);
          else {
            var le = (0, J.sum)([].concat(this.cLines, this.cSpace)) + 2 * this.fSpace[0];
            se = Math.max(0, this.length2em(ae) - le) / oe;
          }
          return Array(this.numCols).fill(se);
        }, ee.prototype.getColumnWidthsAuto = function(ae) {
          var oe = this;
          return ae.map(function(se) {
            return se === "auto" || se === "fit" ? null : (0, G.isPercent)(se) ? se : oe.length2em(se);
          });
        }, ee.prototype.getColumnWidthsPercent = function(ae) {
          var oe = this, se = ae.indexOf("fit") >= 0, ce = (se ? this.getTableData() : { W: null }).W;
          return Array.from(ae.keys()).map(function(le) {
            var ue = ae[le];
            return ue === "fit" ? null : ue === "auto" ? se ? ce[le] : null : (0, G.isPercent)(ue) ? ue : oe.length2em(ue);
          });
        }, ee.prototype.getColumnWidthsFixed = function(ae, oe) {
          var se = this, ce = Array.from(ae.keys()), le = ce.filter(function(ye) {
            return ae[ye] === "fit";
          }), ue = ce.filter(function(ye) {
            return ae[ye] === "auto";
          }), de = le.length || ue.length, fe = (de ? this.getTableData() : { W: null }).W, he = oe - (0, J.sum)([].concat(this.cLines, this.cSpace)) - 2 * this.fSpace[0], me = he;
          ce.forEach(function(ye) {
            var ge = ae[ye];
            me -= ge === "fit" || ge === "auto" ? fe[ye] : se.length2em(ge, he);
          });
          var pe = de && me > 0 ? me / de : 0;
          return ce.map(function(ye) {
            var ge = ae[ye];
            return ge === "fit" ? fe[ye] + pe : ge === "auto" ? fe[ye] + (le.length === 0 ? pe : 0) : se.length2em(ge, he);
          });
        }, ee.prototype.getVerticalPosition = function(ae, oe) {
          for (var se = this.node.attributes.get("equalrows"), ce = this.getTableData(), le = ce.H, ue = ce.D, de = se ? this.getEqualRowHeight() : 0, fe = this.getRowHalfSpacing(), he = this.fLine, me = 0; me < ae; me++)
            he += fe[me] + (se ? de : le[me] + ue[me]) + fe[me + 1] + this.rLines[me];
          var pe = _(se ? [(de + le[ae] - ue[ae]) / 2, (de - le[ae] + ue[ae]) / 2] : [le[ae], ue[ae]], 2), ye = pe[0], ge = pe[1], ve = {
            top: 0,
            center: fe[ae] + (ye + ge) / 2,
            bottom: fe[ae] + ye + ge + fe[ae + 1],
            baseline: fe[ae] + ye,
            axis: fe[ae] + ye - 0.25
          };
          return he += ve[oe] || 0, he;
        }, ee.prototype.getEmHalfSpacing = function(ae, oe, se) {
          se === void 0 && (se = 1);
          var ce = this.em(ae * se), le = this.addEm(oe, 2 / se);
          return le.unshift(ce), le.push(ce), le;
        }, ee.prototype.getRowHalfSpacing = function() {
          var ae = this.rSpace.map(function(oe) {
            return oe / 2;
          });
          return ae.unshift(this.fSpace[1]), ae.push(this.fSpace[1]), ae;
        }, ee.prototype.getColumnHalfSpacing = function() {
          var ae = this.cSpace.map(function(oe) {
            return oe / 2;
          });
          return ae.unshift(this.fSpace[0]), ae.push(this.fSpace[0]), ae;
        }, ee.prototype.getAlignmentRow = function() {
          var ae = _((0, G.split)(this.node.attributes.get("align")), 2), oe = ae[0], se = ae[1];
          if (se == null)
            return [oe, null];
          var ce = parseInt(se);
          return ce < 0 && (ce += this.numRows + 1), [oe, ce < 1 || ce > this.numRows ? null : ce - 1];
        }, ee.prototype.getColumnAttributes = function(ae, oe) {
          oe === void 0 && (oe = 1);
          var se = this.numCols - oe, ce = this.getAttributeArray(ae);
          if (ce.length === 0)
            return null;
          for (; ce.length < se; )
            ce.push(ce[ce.length - 1]);
          return ce.length > se && ce.splice(se), ce;
        }, ee.prototype.getRowAttributes = function(ae, oe) {
          oe === void 0 && (oe = 1);
          var se = this.numRows - oe, ce = this.getAttributeArray(ae);
          if (ce.length === 0)
            return null;
          for (; ce.length < se; )
            ce.push(ce[ce.length - 1]);
          return ce.length > se && ce.splice(se), ce;
        }, ee.prototype.getAttributeArray = function(ae) {
          var oe = this.node.attributes.get(ae);
          return oe ? (0, G.split)(oe) : [this.node.attributes.getDefault(ae)];
        }, ee.prototype.addEm = function(ae, oe) {
          var se = this;
          return oe === void 0 && (oe = 1), ae ? ae.map(function(ce) {
            return se.em(ce / oe);
          }) : null;
        }, ee.prototype.convertLengths = function(ae) {
          var oe = this;
          return ae ? ae.map(function(se) {
            return oe.length2em(se);
          }) : null;
        }, ee;
      }(te);
    }
    return mtable$1.CommonMtableMixin = Z, mtable$1;
  }
  var mtable = {}, hasRequiredMtable$1;
  function requireMtable$1() {
    if (hasRequiredMtable$1) return mtable;
    hasRequiredMtable$1 = 1;
    var O = mtable.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = mtable.__assign || function() {
      return _ = Object.assign || function(J) {
        for (var Z, te = 1, ne = arguments.length; te < ne; te++) {
          Z = arguments[te];
          for (var ee in Z) Object.prototype.hasOwnProperty.call(Z, ee) && (J[ee] = Z[ee]);
        }
        return J;
      }, _.apply(this, arguments);
    }, H = mtable.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mtable, "__esModule", { value: !0 }), mtable.MmlMtable = void 0;
    var V = requireMmlNode(), q = requireString(), G = function(J) {
      O(Z, J);
      function Z() {
        var te = J !== null && J.apply(this, arguments) || this;
        return te.properties = {
          useHeight: !0
        }, te.texclass = V.TEXCLASS.ORD, te;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "mtable";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.setInheritedAttributes = function(te, ne, ee, ae) {
        var oe, se;
        try {
          for (var ce = H(V.indentAttributes), le = ce.next(); !le.done; le = ce.next()) {
            var ue = le.value;
            te[ue] && this.attributes.setInherited(ue, te[ue][1]), this.attributes.getExplicit(ue) !== void 0 && delete this.attributes.getAllAttributes()[ue];
          }
        } catch (de) {
          oe = { error: de };
        } finally {
          try {
            le && !le.done && (se = ce.return) && se.call(ce);
          } finally {
            if (oe) throw oe.error;
          }
        }
        J.prototype.setInheritedAttributes.call(this, te, ne, ee, ae);
      }, Z.prototype.setChildInheritedAttributes = function(te, ne, ee, ae) {
        var oe, se, ce, le;
        try {
          for (var ue = H(this.childNodes), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value;
            fe.isKind("mtr") || this.replaceChild(this.factory.create("mtr"), fe).appendChild(fe);
          }
        } catch (ge) {
          oe = { error: ge };
        } finally {
          try {
            de && !de.done && (se = ue.return) && se.call(ue);
          } finally {
            if (oe) throw oe.error;
          }
        }
        ee = this.getProperty("scriptlevel") || ee, ne = !!(this.attributes.getExplicit("displaystyle") || this.attributes.getDefault("displaystyle")), te = this.addInheritedAttributes(te, {
          columnalign: this.attributes.get("columnalign"),
          rowalign: "center"
        });
        var he = this.attributes.getExplicit("data-cramped"), me = (0, q.split)(this.attributes.get("rowalign"));
        try {
          for (var pe = H(this.childNodes), ye = pe.next(); !ye.done; ye = pe.next()) {
            var fe = ye.value;
            te.rowalign[1] = me.shift() || te.rowalign[1], fe.setInheritedAttributes(te, ne, ee, !!he);
          }
        } catch (ge) {
          ce = { error: ge };
        } finally {
          try {
            ye && !ye.done && (le = pe.return) && le.call(pe);
          } finally {
            if (ce) throw ce.error;
          }
        }
      }, Z.prototype.verifyChildren = function(te) {
        for (var ne = null, ee = this.factory, ae = 0; ae < this.childNodes.length; ae++) {
          var oe = this.childNodes[ae];
          if (oe.isKind("mtr"))
            ne = null;
          else {
            var se = oe.isKind("mtd");
            if (ne ? (this.removeChild(oe), ae--) : ne = this.replaceChild(ee.create("mtr"), oe), ne.appendChild(se ? oe : ee.create("mtd", {}, [oe])), !te.fixMtables) {
              oe.parent.removeChild(oe), oe.parent = this, se && ne.appendChild(ee.create("mtd"));
              var ce = oe.mError("Children of " + this.kind + " must be mtr or mlabeledtr", te, se);
              ne.childNodes[ne.childNodes.length - 1].appendChild(ce);
            }
          }
        }
        J.prototype.verifyChildren.call(this, te);
      }, Z.prototype.setTeXclass = function(te) {
        var ne, ee;
        this.getPrevClass(te);
        try {
          for (var ae = H(this.childNodes), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            se.setTeXclass(null);
          }
        } catch (ce) {
          ne = { error: ce };
        } finally {
          try {
            oe && !oe.done && (ee = ae.return) && ee.call(ae);
          } finally {
            if (ne) throw ne.error;
          }
        }
        return this;
      }, Z.defaults = _(_({}, V.AbstractMmlNode.defaults), { align: "axis", rowalign: "baseline", columnalign: "center", groupalign: "{left}", alignmentscope: !0, columnwidth: "auto", width: "auto", rowspacing: "1ex", columnspacing: ".8em", rowlines: "none", columnlines: "none", frame: "none", framespacing: "0.4em 0.5ex", equalrows: !1, equalcolumns: !1, displaystyle: !1, side: "right", minlabelspacing: "0.8em" }), Z;
    }(V.AbstractMmlNode);
    return mtable.MmlMtable = G, mtable;
  }
  var hasRequiredMtable;
  function requireMtable() {
    if (hasRequiredMtable) return mtable$2;
    hasRequiredMtable = 1;
    var O = mtable$2.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = mtable$2.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    }, H = mtable$2.__spreadArray || function(te, ne, ee) {
      if (ee || arguments.length === 2) for (var ae = 0, oe = ne.length, se; ae < oe; ae++)
        (se || !(ae in ne)) && (se || (se = Array.prototype.slice.call(ne, 0, ae)), se[ae] = ne[ae]);
      return te.concat(se || Array.prototype.slice.call(ne));
    };
    Object.defineProperty(mtable$2, "__esModule", { value: !0 }), mtable$2.SVGmtable = void 0;
    var V = requireWrapper(), q = requireMtable$2(), G = requireMtable$1(), J = "mjx-", Z = function(te) {
      O(ne, te);
      function ne(ee, ae, oe) {
        oe === void 0 && (oe = null);
        var se = te.call(this, ee, ae, oe) || this, ce = { "data-labels": !0 };
        return se.isTop && (ce.transform = "matrix(1 0 0 -1 0 0)"), se.labels = se.svg("g", ce), se;
      }
      return ne.prototype.toSVG = function(ee) {
        var ae = this.standardSVGnode(ee);
        this.placeRows(ae), this.handleColumnLines(ae), this.handleRowLines(ae), this.handleFrame(ae);
        var oe = this.handlePWidth(ae);
        this.handleLabels(ae, ee, oe);
      }, ne.prototype.placeRows = function(ee) {
        for (var ae, oe, se, ce = this.node.attributes.get("equalrows"), le = this.getTableData(), ue = le.H, de = le.D, fe = this.getEqualRowHeight(), he = this.getRowHalfSpacing(), me = H(H([this.fLine], _(this.rLines), !1), [this.fLine], !1), pe = this.getBBox().h - me[0], ye = 0; ye < this.numRows; ye++) {
          var ge = this.childNodes[ye];
          ae = _(this.getRowHD(ce, fe, ue[ye], de[ye]), 2), ge.H = ae[0], ge.D = ae[1], oe = _([he[ye], he[ye + 1]], 2), ge.tSpace = oe[0], ge.bSpace = oe[1], se = _([me[ye], me[ye + 1]], 2), ge.tLine = se[0], ge.bLine = se[1], ge.toSVG(ee), ge.place(0, pe - he[ye] - ge.H), pe -= he[ye] + ge.H + ge.D + he[ye + 1] + me[ye + 1];
        }
      }, ne.prototype.getRowHD = function(ee, ae, oe, se) {
        return ee ? [(ae + oe - se) / 2, (ae - oe + se) / 2] : [oe, se];
      }, ne.prototype.handleColor = function() {
        te.prototype.handleColor.call(this);
        var ee = this.firstChild();
        ee && this.adaptor.setAttribute(ee, "width", this.fixed(this.getWidth()));
      }, ne.prototype.handleColumnLines = function(ee) {
        if (this.node.attributes.get("columnlines") !== "none") {
          var ae = this.getColumnAttributes("columnlines");
          if (ae)
            for (var oe = this.getColumnHalfSpacing(), se = this.cLines, ce = this.getComputedWidths(), le = this.fLine, ue = 0; ue < ae.length; ue++)
              le += oe[ue] + ce[ue] + oe[ue + 1], ae[ue] !== "none" && this.adaptor.append(ee, this.makeVLine(le, ae[ue], se[ue])), le += se[ue];
        }
      }, ne.prototype.handleRowLines = function(ee) {
        if (this.node.attributes.get("rowlines") !== "none") {
          var ae = this.getRowAttributes("rowlines");
          if (ae)
            for (var oe = this.node.attributes.get("equalrows"), se = this.getTableData(), ce = se.H, le = se.D, ue = this.getEqualRowHeight(), de = this.getRowHalfSpacing(), fe = this.rLines, he = this.getBBox().h - this.fLine, me = 0; me < ae.length; me++) {
              var pe = _(this.getRowHD(oe, ue, ce[me], le[me]), 2), ye = pe[0], ge = pe[1];
              he -= de[me] + ye + ge + de[me + 1], ae[me] !== "none" && this.adaptor.append(ee, this.makeHLine(he, ae[me], fe[me])), he -= fe[me];
            }
        }
      }, ne.prototype.handleFrame = function(ee) {
        if (this.frame && this.fLine) {
          var ae = this.getBBox(), oe = ae.h, se = ae.d, ce = ae.w, le = this.node.attributes.get("frame");
          this.adaptor.append(ee, this.makeFrame(ce, oe, se, le));
        }
      }, ne.prototype.handlePWidth = function(ee) {
        if (!this.pWidth)
          return 0;
        var ae = this.getBBox(), oe = ae.w, se = ae.L, ce = ae.R, le = se + this.pWidth + ce, ue = this.getAlignShift()[0], de = Math.max(this.isTop ? le : 0, this.container.getWrapWidth(this.containerI)) - se - ce, fe = oe - (this.pWidth > de ? de : this.pWidth), he = ue === "left" ? 0 : ue === "right" ? fe : fe / 2;
        if (he) {
          var me = this.svg("g", {}, this.adaptor.childNodes(ee));
          this.place(he, 0, me), this.adaptor.append(ee, me);
        }
        return he;
      }, ne.prototype.lineClass = function(ee) {
        return J + ee;
      }, ne.prototype.makeFrame = function(ee, ae, oe, se) {
        var ce = this.fLine;
        return this.svg("rect", this.setLineThickness(ce, se, {
          "data-frame": !0,
          class: this.lineClass(se),
          width: this.fixed(ee - ce),
          height: this.fixed(ae + oe - ce),
          x: this.fixed(ce / 2),
          y: this.fixed(ce / 2 - oe)
        }));
      }, ne.prototype.makeVLine = function(ee, ae, oe) {
        var se = this.getBBox(), ce = se.h, le = se.d, ue = ae === "dotted" ? oe / 2 : 0, de = this.fixed(ee + oe / 2);
        return this.svg("line", this.setLineThickness(oe, ae, {
          "data-line": "v",
          class: this.lineClass(ae),
          x1: de,
          y1: this.fixed(ue - le),
          x2: de,
          y2: this.fixed(ce - ue)
        }));
      }, ne.prototype.makeHLine = function(ee, ae, oe) {
        var se = this.getBBox().w, ce = ae === "dotted" ? oe / 2 : 0, le = this.fixed(ee - oe / 2);
        return this.svg("line", this.setLineThickness(oe, ae, {
          "data-line": "h",
          class: this.lineClass(ae),
          x1: this.fixed(ce),
          y1: le,
          x2: this.fixed(se - ce),
          y2: le
        }));
      }, ne.prototype.setLineThickness = function(ee, ae, oe) {
        return ee !== 0.07 && (oe["stroke-thickness"] = this.fixed(ee), ae !== "solid" && (oe["stroke-dasharray"] = (ae === "dotted" ? "0," : "") + this.fixed(2 * ee))), oe;
      }, ne.prototype.handleLabels = function(ee, ae, oe) {
        if (this.hasLabels) {
          var se = this.labels, ce = this.node.attributes, le = ce.get("side");
          this.spaceLabels(), this.isTop ? this.topTable(ee, se, le) : this.subTable(ee, se, le, oe);
        }
      }, ne.prototype.spaceLabels = function() {
        for (var ee = this.adaptor, ae = this.getBBox().h, oe = this.getTableData().L, se = this.getRowHalfSpacing(), ce = ae - this.fLine, le = ee.firstChild(this.labels), ue = 0; ue < this.numRows; ue++) {
          var de = this.childNodes[ue];
          if (de.node.isKind("mlabeledtr")) {
            var fe = de.childNodes[0];
            ce -= se[ue] + de.H, de.placeCell(fe, { x: 0, y: ce, w: oe, lSpace: 0, rSpace: 0, lLine: 0, rLine: 0 }), ce -= de.D + se[ue + 1] + this.rLines[ue], le = ee.next(le);
          } else
            ce -= se[ue] + de.H + de.D + se[ue + 1] + this.rLines[ue];
        }
      }, ne.prototype.topTable = function(ee, ae, oe) {
        var se = this.adaptor, ce = this.getBBox(), le = ce.h, ue = ce.d, de = ce.w, fe = ce.L, he = ce.R, me = fe + (this.pWidth || de) + he, pe = this.getTableData().L, ye = _(this.getPadAlignShift(oe), 3), ge = ye[1], ve = ye[2], be = ve + (ge === "right" ? -me : ge === "center" ? -me / 2 : 0) + fe, Ce = "matrix(1 0 0 -1 0 0)", Se = "scale(".concat(this.jax.fixed(this.font.params.x_height * 1e3 / this.metrics.ex, 2), ")"), we = "translate(0 ".concat(this.fixed(le), ") ").concat(Ce, " ").concat(Se), Ee = this.svg("svg", {
          "data-table": !0,
          preserveAspectRatio: ge === "left" ? "xMinYMid" : ge === "right" ? "xMaxYMid" : "xMidYMid",
          viewBox: [this.fixed(-be), this.fixed(-le), 1, this.fixed(le + ue)].join(" ")
        }, [
          this.svg("g", { transform: Ce }, se.childNodes(ee))
        ]);
        ae = this.svg("svg", {
          "data-labels": !0,
          preserveAspectRatio: oe === "left" ? "xMinYMid" : "xMaxYMid",
          viewBox: [oe === "left" ? 0 : this.fixed(pe), this.fixed(-le), 1, this.fixed(le + ue)].join(" ")
        }, [ae]), se.append(ee, this.svg("g", { transform: we }, [Ee, ae])), this.place(-fe, 0, ee);
      }, ne.prototype.subTable = function(ee, ae, oe, se) {
        var ce = this.adaptor, le = this.getBBox(), ue = le.w, de = le.L, fe = le.R, he = de + (this.pWidth || ue) + fe, me = this.getTableData().L, pe = this.getAlignShift()[0], ye = Math.max(he, this.container.getWrapWidth(this.containerI));
        this.place(oe === "left" ? (pe === "left" ? 0 : pe === "right" ? he - ye + se : (he - ye) / 2 + se) - de : (pe === "left" ? ye : pe === "right" ? he + se : (ye + he) / 2 + se) - de - me, 0, ae), ce.append(ee, ae);
      }, ne.kind = G.MmlMtable.prototype.kind, ne.styles = {
        'g[data-mml-node="mtable"] > line[data-line], svg[data-table] > g > line[data-line]': {
          "stroke-width": "70px",
          fill: "none"
        },
        'g[data-mml-node="mtable"] > rect[data-frame], svg[data-table] > g > rect[data-frame]': {
          "stroke-width": "70px",
          fill: "none"
        },
        'g[data-mml-node="mtable"] > .mjx-dashed, svg[data-table] > g > .mjx-dashed': {
          "stroke-dasharray": "140"
        },
        'g[data-mml-node="mtable"] > .mjx-dotted, svg[data-table] > g > .mjx-dotted': {
          "stroke-linecap": "round",
          "stroke-dasharray": "0,140"
        },
        'g[data-mml-node="mtable"] > g > svg': {
          overflow: "visible"
        }
      }, ne;
    }((0, q.CommonMtableMixin)(V.SVGWrapper));
    return mtable$2.SVGmtable = Z, mtable$2;
  }
  var mtr$2 = {}, mtr$1 = {}, hasRequiredMtr$2;
  function requireMtr$2() {
    if (hasRequiredMtr$2) return mtr$1;
    hasRequiredMtr$2 = 1;
    var O = mtr$1.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mtr$1.__values || function(q) {
      var G = typeof Symbol == "function" && Symbol.iterator, J = G && q[G], Z = 0;
      if (J) return J.call(q);
      if (q && typeof q.length == "number") return {
        next: function() {
          return q && Z >= q.length && (q = void 0), { value: q && q[Z++], done: !q };
        }
      };
      throw new TypeError(G ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mtr$1, "__esModule", { value: !0 }), mtr$1.CommonMlabeledtrMixin = mtr$1.CommonMtrMixin = void 0;
    function H(q) {
      return function(G) {
        O(J, G);
        function J() {
          return G !== null && G.apply(this, arguments) || this;
        }
        return Object.defineProperty(J.prototype, "fixesPWidth", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(J.prototype, "numCells", {
          get: function() {
            return this.childNodes.length;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(J.prototype, "labeled", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(J.prototype, "tableCells", {
          get: function() {
            return this.childNodes;
          },
          enumerable: !1,
          configurable: !0
        }), J.prototype.getChild = function(Z) {
          return this.childNodes[Z];
        }, J.prototype.getChildBBoxes = function() {
          return this.childNodes.map(function(Z) {
            return Z.getBBox();
          });
        }, J.prototype.stretchChildren = function(Z) {
          var te, ne, ee, ae, oe, se;
          Z === void 0 && (Z = null);
          var ce = [], le = this.labeled ? this.childNodes.slice(1) : this.childNodes;
          try {
            for (var ue = _(le), de = ue.next(); !de.done; de = ue.next()) {
              var fe = de.value, he = fe.childNodes[0];
              he.canStretch(1) && ce.push(he);
            }
          } catch (ke) {
            te = { error: ke };
          } finally {
            try {
              de && !de.done && (ne = ue.return) && ne.call(ue);
            } finally {
              if (te) throw te.error;
            }
          }
          var me = ce.length, pe = this.childNodes.length;
          if (me && pe > 1) {
            if (Z === null) {
              var ye = 0, ge = 0, ve = me > 1 && me === pe;
              try {
                for (var be = _(le), Ce = be.next(); !Ce.done; Ce = be.next()) {
                  var fe = Ce.value, he = fe.childNodes[0], Se = he.stretch.dir === 0;
                  if (ve || Se) {
                    var we = he.getBBox(Se), Ee = we.h, xe = we.d;
                    Ee > ye && (ye = Ee), xe > ge && (ge = xe);
                  }
                }
              } catch (ke) {
                ee = { error: ke };
              } finally {
                try {
                  Ce && !Ce.done && (ae = be.return) && ae.call(be);
                } finally {
                  if (ee) throw ee.error;
                }
              }
              Z = [ye, ge];
            }
            try {
              for (var Le = _(ce), Qe = Le.next(); !Qe.done; Qe = Le.next()) {
                var he = Qe.value;
                he.coreMO().getStretchedVariant(Z);
              }
            } catch (ke) {
              oe = { error: ke };
            } finally {
              try {
                Qe && !Qe.done && (se = Le.return) && se.call(Le);
              } finally {
                if (oe) throw oe.error;
              }
            }
          }
        }, J;
      }(q);
    }
    mtr$1.CommonMtrMixin = H;
    function V(q) {
      return function(G) {
        O(J, G);
        function J() {
          return G !== null && G.apply(this, arguments) || this;
        }
        return Object.defineProperty(J.prototype, "numCells", {
          get: function() {
            return Math.max(0, this.childNodes.length - 1);
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(J.prototype, "labeled", {
          get: function() {
            return !0;
          },
          enumerable: !1,
          configurable: !0
        }), Object.defineProperty(J.prototype, "tableCells", {
          get: function() {
            return this.childNodes.slice(1);
          },
          enumerable: !1,
          configurable: !0
        }), J.prototype.getChild = function(Z) {
          return this.childNodes[Z + 1];
        }, J.prototype.getChildBBoxes = function() {
          return this.childNodes.slice(1).map(function(Z) {
            return Z.getBBox();
          });
        }, J;
      }(q);
    }
    return mtr$1.CommonMlabeledtrMixin = V, mtr$1;
  }
  var mtr = {}, hasRequiredMtr$1;
  function requireMtr$1() {
    if (hasRequiredMtr$1) return mtr;
    hasRequiredMtr$1 = 1;
    var O = mtr.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = mtr.__assign || function() {
      return _ = Object.assign || function(te) {
        for (var ne, ee = 1, ae = arguments.length; ee < ae; ee++) {
          ne = arguments[ee];
          for (var oe in ne) Object.prototype.hasOwnProperty.call(ne, oe) && (te[oe] = ne[oe]);
        }
        return te;
      }, _.apply(this, arguments);
    }, H = mtr.__values || function(te) {
      var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
      if (ee) return ee.call(te);
      if (te && typeof te.length == "number") return {
        next: function() {
          return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
        }
      };
      throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(mtr, "__esModule", { value: !0 }), mtr.MmlMlabeledtr = mtr.MmlMtr = void 0;
    var V = requireMmlNode(), q = requireAttributes(), G = requireString(), J = function(te) {
      O(ne, te);
      function ne() {
        return te !== null && te.apply(this, arguments) || this;
      }
      return Object.defineProperty(ne.prototype, "kind", {
        get: function() {
          return "mtr";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), ne.prototype.setChildInheritedAttributes = function(ee, ae, oe, se) {
        var ce, le, ue, de;
        try {
          for (var fe = H(this.childNodes), he = fe.next(); !he.done; he = fe.next()) {
            var me = he.value;
            me.isKind("mtd") || this.replaceChild(this.factory.create("mtd"), me).appendChild(me);
          }
        } catch (ve) {
          ce = { error: ve };
        } finally {
          try {
            he && !he.done && (le = fe.return) && le.call(fe);
          } finally {
            if (ce) throw ce.error;
          }
        }
        var pe = (0, G.split)(this.attributes.get("columnalign"));
        this.arity === 1 && pe.unshift(this.parent.attributes.get("side")), ee = this.addInheritedAttributes(ee, {
          rowalign: this.attributes.get("rowalign"),
          columnalign: "center"
        });
        try {
          for (var ye = H(this.childNodes), ge = ye.next(); !ge.done; ge = ye.next()) {
            var me = ge.value;
            ee.columnalign[1] = pe.shift() || ee.columnalign[1], me.setInheritedAttributes(ee, ae, oe, se);
          }
        } catch (ve) {
          ue = { error: ve };
        } finally {
          try {
            ge && !ge.done && (de = ye.return) && de.call(ye);
          } finally {
            if (ue) throw ue.error;
          }
        }
      }, ne.prototype.verifyChildren = function(ee) {
        var ae, oe;
        if (this.parent && !this.parent.isKind("mtable")) {
          this.mError(this.kind + " can only be a child of an mtable", ee, !0);
          return;
        }
        try {
          for (var se = H(this.childNodes), ce = se.next(); !ce.done; ce = se.next()) {
            var le = ce.value;
            if (!le.isKind("mtd")) {
              var ue = this.replaceChild(this.factory.create("mtd"), le);
              ue.appendChild(le), ee.fixMtables || le.mError("Children of " + this.kind + " must be mtd", ee);
            }
          }
        } catch (de) {
          ae = { error: de };
        } finally {
          try {
            ce && !ce.done && (oe = se.return) && oe.call(se);
          } finally {
            if (ae) throw ae.error;
          }
        }
        te.prototype.verifyChildren.call(this, ee);
      }, ne.prototype.setTeXclass = function(ee) {
        var ae, oe;
        this.getPrevClass(ee);
        try {
          for (var se = H(this.childNodes), ce = se.next(); !ce.done; ce = se.next()) {
            var le = ce.value;
            le.setTeXclass(null);
          }
        } catch (ue) {
          ae = { error: ue };
        } finally {
          try {
            ce && !ce.done && (oe = se.return) && oe.call(se);
          } finally {
            if (ae) throw ae.error;
          }
        }
        return this;
      }, ne.defaults = _(_({}, V.AbstractMmlNode.defaults), { rowalign: q.INHERIT, columnalign: q.INHERIT, groupalign: q.INHERIT }), ne;
    }(V.AbstractMmlNode);
    mtr.MmlMtr = J;
    var Z = function(te) {
      O(ne, te);
      function ne() {
        return te !== null && te.apply(this, arguments) || this;
      }
      return Object.defineProperty(ne.prototype, "kind", {
        get: function() {
          return "mlabeledtr";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(ne.prototype, "arity", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), ne;
    }(J);
    return mtr.MmlMlabeledtr = Z, mtr;
  }
  var hasRequiredMtr;
  function requireMtr() {
    if (hasRequiredMtr) return mtr$2;
    hasRequiredMtr = 1;
    var O = mtr$2.__extends || /* @__PURE__ */ function() {
      var ne = function(ee, ae) {
        return ne = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(oe, se) {
          oe.__proto__ = se;
        } || function(oe, se) {
          for (var ce in se) Object.prototype.hasOwnProperty.call(se, ce) && (oe[ce] = se[ce]);
        }, ne(ee, ae);
      };
      return function(ee, ae) {
        if (typeof ae != "function" && ae !== null)
          throw new TypeError("Class extends value " + String(ae) + " is not a constructor or null");
        ne(ee, ae);
        function oe() {
          this.constructor = ee;
        }
        ee.prototype = ae === null ? Object.create(ae) : (oe.prototype = ae.prototype, new oe());
      };
    }(), _ = mtr$2.__read || function(ne, ee) {
      var ae = typeof Symbol == "function" && ne[Symbol.iterator];
      if (!ae) return ne;
      var oe = ae.call(ne), se, ce = [], le;
      try {
        for (; (ee === void 0 || ee-- > 0) && !(se = oe.next()).done; ) ce.push(se.value);
      } catch (ue) {
        le = { error: ue };
      } finally {
        try {
          se && !se.done && (ae = oe.return) && ae.call(oe);
        } finally {
          if (le) throw le.error;
        }
      }
      return ce;
    }, H = mtr$2.__spreadArray || function(ne, ee, ae) {
      if (ae || arguments.length === 2) for (var oe = 0, se = ee.length, ce; oe < se; oe++)
        (ce || !(oe in ee)) && (ce || (ce = Array.prototype.slice.call(ee, 0, oe)), ce[oe] = ee[oe]);
      return ne.concat(ce || Array.prototype.slice.call(ee));
    };
    Object.defineProperty(mtr$2, "__esModule", { value: !0 }), mtr$2.SVGmlabeledtr = mtr$2.SVGmtr = void 0;
    var V = requireWrapper(), q = requireMtr$2(), G = requireMtr$2(), J = requireMtr$1(), Z = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee.prototype.toSVG = function(ae) {
        var oe = this.standardSVGnode(ae);
        this.placeCells(oe), this.placeColor();
      }, ee.prototype.placeCells = function(ae) {
        for (var oe = this.parent.getColumnHalfSpacing(), se = H(H([this.parent.fLine], _(this.parent.cLines), !1), [this.parent.fLine], !1), ce = this.parent.getComputedWidths(), le = 1 / this.getBBox().rscale, ue = se[0], de = 0; de < this.numCells; de++) {
          var fe = this.getChild(de);
          fe.toSVG(ae), ue += this.placeCell(fe, {
            x: ue,
            y: 0,
            lSpace: oe[de] * le,
            rSpace: oe[de + 1] * le,
            w: ce[de] * le,
            lLine: se[de] * le,
            rLine: se[de + 1] * le
          });
        }
      }, ee.prototype.placeCell = function(ae, oe) {
        var se = oe.x, ce = oe.y, le = oe.lSpace, ue = oe.w, de = oe.rSpace, fe = oe.lLine, he = oe.rLine, me = 1 / this.getBBox().rscale, pe = _([this.H * me, this.D * me], 2), ye = pe[0], ge = pe[1], ve = _([this.tSpace * me, this.bSpace * me], 2), be = ve[0], Ce = ve[1], Se = _(ae.placeCell(se + le, ce, ue, ye, ge), 2), we = Se[0], Ee = Se[1], xe = le + ue + de;
        return ae.placeColor(-(we + le + fe / 2), -(ge + Ce + Ee), xe + (fe + he) / 2, ye + ge + be + Ce), xe + he;
      }, ee.prototype.placeColor = function() {
        var ae = 1 / this.getBBox().rscale, oe = this.adaptor, se = this.firstChild();
        if (se && oe.kind(se) === "rect" && oe.getAttribute(se, "data-bgcolor")) {
          var ce = _([this.tLine / 2 * ae, this.bLine / 2 * ae], 2), le = ce[0], ue = ce[1], de = _([this.tSpace * ae, this.bSpace * ae], 2), fe = de[0], he = de[1], me = _([this.H * ae, this.D * ae], 2), pe = me[0], ye = me[1];
          oe.setAttribute(se, "y", this.fixed(-(ye + he + ue))), oe.setAttribute(se, "width", this.fixed(this.parent.getWidth() * ae)), oe.setAttribute(se, "height", this.fixed(le + fe + pe + ye + he + ue));
        }
      }, ee.kind = J.MmlMtr.prototype.kind, ee;
    }((0, q.CommonMtrMixin)(V.SVGWrapper));
    mtr$2.SVGmtr = Z;
    var te = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee.prototype.toSVG = function(ae) {
        ne.prototype.toSVG.call(this, ae);
        var oe = this.childNodes[0];
        oe && oe.toSVG(this.parent.labels);
      }, ee.kind = J.MmlMlabeledtr.prototype.kind, ee;
    }((0, G.CommonMlabeledtrMixin)(Z));
    return mtr$2.SVGmlabeledtr = te, mtr$2;
  }
  var mtd$2 = {}, mtd$1 = {}, hasRequiredMtd$2;
  function requireMtd$2() {
    if (hasRequiredMtd$2) return mtd$1;
    hasRequiredMtd$2 = 1;
    var O = mtd$1.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(mtd$1, "__esModule", { value: !0 }), mtd$1.CommonMtdMixin = void 0;
    function _(H) {
      return function(V) {
        O(q, V);
        function q() {
          return V !== null && V.apply(this, arguments) || this;
        }
        return Object.defineProperty(q.prototype, "fixesPWidth", {
          get: function() {
            return !1;
          },
          enumerable: !1,
          configurable: !0
        }), q.prototype.invalidateBBox = function() {
          this.bboxComputed = !1;
        }, q.prototype.getWrapWidth = function(G) {
          var J = this.parent.parent, Z = this.parent, te = this.node.childPosition() - (Z.labeled ? 1 : 0);
          return typeof J.cWidths[te] == "number" ? J.cWidths[te] : J.getTableData().W[te];
        }, q.prototype.getChildAlign = function(G) {
          return this.node.attributes.get("columnalign");
        }, q;
      }(H);
    }
    return mtd$1.CommonMtdMixin = _, mtd$1;
  }
  var mtd = {}, hasRequiredMtd$1;
  function requireMtd$1() {
    if (hasRequiredMtd$1) return mtd;
    hasRequiredMtd$1 = 1;
    var O = mtd.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = mtd.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mtd, "__esModule", { value: !0 }), mtd.MmlMtd = void 0;
    var H = requireMmlNode(), V = requireAttributes(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return Object.defineProperty(J.prototype, "kind", {
        get: function() {
          return "mtd";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "arity", {
        get: function() {
          return -1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "linebreakContainer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.verifyChildren = function(Z) {
        if (this.parent && !this.parent.isKind("mtr")) {
          this.mError(this.kind + " can only be a child of an mtr or mlabeledtr", Z, !0);
          return;
        }
        G.prototype.verifyChildren.call(this, Z);
      }, J.prototype.setTeXclass = function(Z) {
        return this.getPrevClass(Z), this.childNodes[0].setTeXclass(null), this;
      }, J.defaults = _(_({}, H.AbstractMmlBaseNode.defaults), { rowspan: 1, columnspan: 1, rowalign: V.INHERIT, columnalign: V.INHERIT, groupalign: V.INHERIT }), J;
    }(H.AbstractMmlBaseNode);
    return mtd.MmlMtd = q, mtd;
  }
  var hasRequiredMtd;
  function requireMtd() {
    if (hasRequiredMtd) return mtd$2;
    hasRequiredMtd = 1;
    var O = mtd$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(mtd$2, "__esModule", { value: !0 }), mtd$2.SVGmtd = void 0;
    var _ = requireWrapper(), H = requireMtd$2(), V = requireMtd$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.prototype.placeCell = function(Z, te, ne, ee, ae) {
        var oe = this.getBBox(), se = Math.max(oe.h, 0.75), ce = Math.max(oe.d, 0.25), le = this.node.attributes.get("columnalign"), ue = this.node.attributes.get("rowalign"), de = this.getAlignX(ne, oe, le), fe = this.getAlignY(ee, ae, se, ce, ue);
        return this.place(Z + de, te + fe), [de, fe];
      }, J.prototype.placeColor = function(Z, te, ne, ee) {
        var ae = this.adaptor, oe = this.firstChild();
        oe && ae.kind(oe) === "rect" && ae.getAttribute(oe, "data-bgcolor") && (ae.setAttribute(oe, "x", this.fixed(Z)), ae.setAttribute(oe, "y", this.fixed(te)), ae.setAttribute(oe, "width", this.fixed(ne)), ae.setAttribute(oe, "height", this.fixed(ee)));
      }, J.kind = V.MmlMtd.prototype.kind, J;
    }((0, H.CommonMtdMixin)(_.SVGWrapper));
    return mtd$2.SVGmtd = q, mtd$2;
  }
  var maction$2 = {}, maction$1 = {}, hasRequiredMaction$2;
  function requireMaction$2() {
    return hasRequiredMaction$2 || (hasRequiredMaction$2 = 1, function(O) {
      var _ = maction$1.__extends || /* @__PURE__ */ function() {
        var J = function(Z, te) {
          return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
            ne.__proto__ = ee;
          } || function(ne, ee) {
            for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
          }, J(Z, te);
        };
        return function(Z, te) {
          if (typeof te != "function" && te !== null)
            throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
          J(Z, te);
          function ne() {
            this.constructor = Z;
          }
          Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
        };
      }(), H = maction$1.__read || function(J, Z) {
        var te = typeof Symbol == "function" && J[Symbol.iterator];
        if (!te) return J;
        var ne = te.call(J), ee, ae = [], oe;
        try {
          for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
        } catch (se) {
          oe = { error: se };
        } finally {
          try {
            ee && !ee.done && (te = ne.return) && te.call(ne);
          } finally {
            if (oe) throw oe.error;
          }
        }
        return ae;
      }, V = maction$1.__spreadArray || function(J, Z, te) {
        if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
          (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
        return J.concat(ae || Array.prototype.slice.call(Z));
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.CommonMactionMixin = O.TooltipData = void 0;
      var q = requireString();
      O.TooltipData = {
        dx: ".2em",
        dy: ".1em",
        postDelay: 600,
        clearDelay: 100,
        hoverTimer: /* @__PURE__ */ new Map(),
        clearTimer: /* @__PURE__ */ new Map(),
        stopTimers: function(J, Z) {
          Z.clearTimer.has(J) && (clearTimeout(Z.clearTimer.get(J)), Z.clearTimer.delete(J)), Z.hoverTimer.has(J) && (clearTimeout(Z.hoverTimer.get(J)), Z.hoverTimer.delete(J));
        }
      };
      function G(J) {
        return function(Z) {
          _(te, Z);
          function te() {
            for (var ne = [], ee = 0; ee < arguments.length; ee++)
              ne[ee] = arguments[ee];
            var ae = Z.apply(this, V([], H(ne), !1)) || this, oe = ae.constructor.actions, se = ae.node.attributes.get("actiontype"), ce = H(oe.get(se) || [function(de, fe) {
            }, {}], 2), le = ce[0], ue = ce[1];
            return ae.action = le, ae.data = ue, ae.getParameters(), ae;
          }
          return Object.defineProperty(te.prototype, "selected", {
            get: function() {
              var ne = this.node.attributes.get("selection"), ee = Math.max(1, Math.min(this.childNodes.length, ne)) - 1;
              return this.childNodes[ee] || this.wrap(this.node.selected);
            },
            enumerable: !1,
            configurable: !0
          }), te.prototype.getParameters = function() {
            var ne = this.node.attributes.get("data-offsets"), ee = H((0, q.split)(ne || ""), 2), ae = ee[0], oe = ee[1];
            this.dx = this.length2em(ae || O.TooltipData.dx), this.dy = this.length2em(oe || O.TooltipData.dy);
          }, te.prototype.computeBBox = function(ne, ee) {
            ee === void 0 && (ee = !1), ne.updateFrom(this.selected.getOuterBBox()), this.selected.setChildPWidths(ee);
          }, te;
        }(J);
      }
      O.CommonMactionMixin = G;
    }(maction$1)), maction$1;
  }
  var maction = {}, hasRequiredMaction$1;
  function requireMaction$1() {
    if (hasRequiredMaction$1) return maction;
    hasRequiredMaction$1 = 1;
    var O = maction.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = maction.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(maction, "__esModule", { value: !0 }), maction.MmlMaction = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "maction";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "selected", {
        get: function() {
          var J = this.attributes.get("selection"), Z = Math.max(1, Math.min(this.childNodes.length, J)) - 1;
          return this.childNodes[Z] || this.factory.create("mrow");
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "isEmbellished", {
        get: function() {
          return this.selected.isEmbellished;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "isSpacelike", {
        get: function() {
          return this.selected.isSpacelike;
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.core = function() {
        return this.selected.core();
      }, G.prototype.coreMO = function() {
        return this.selected.coreMO();
      }, G.prototype.verifyAttributes = function(J) {
        if (q.prototype.verifyAttributes.call(this, J), this.attributes.get("actiontype") !== "toggle" && this.attributes.getExplicit("selection") !== void 0) {
          var Z = this.attributes.getAllAttributes();
          delete Z.selection;
        }
      }, G.prototype.setTeXclass = function(J) {
        this.attributes.get("actiontype") === "tooltip" && this.childNodes[1] && this.childNodes[1].setTeXclass(null);
        var Z = this.selected;
        return J = Z.setTeXclass(J), this.updateTeXclass(Z), J;
      }, G.prototype.nextToggleSelection = function() {
        var J = Math.max(1, this.attributes.get("selection") + 1);
        J > this.childNodes.length && (J = 1), this.attributes.set("selection", J);
      }, G.defaults = _(_({}, H.AbstractMmlNode.defaults), { actiontype: "toggle", selection: 1 }), G;
    }(H.AbstractMmlNode);
    return maction.MmlMaction = V, maction;
  }
  var hasRequiredMaction;
  function requireMaction() {
    if (hasRequiredMaction) return maction$2;
    hasRequiredMaction = 1;
    var O = maction$2.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }();
    Object.defineProperty(maction$2, "__esModule", { value: !0 }), maction$2.SVGmaction = void 0;
    var _ = requireWrapper(), H = requireMaction$2(), V = requireMaction$2(), q = requireMaction$1(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.toSVG = function(te) {
        var ne = this.standardSVGnode(te), ee = this.selected, ae = ee.getOuterBBox(), oe = ae.h, se = ae.d, ce = ae.w;
        this.adaptor.append(this.element, this.svg("rect", {
          width: this.fixed(ce),
          height: this.fixed(oe + se),
          y: this.fixed(-se),
          fill: "none",
          "pointer-events": "all"
        })), ee.toSVG(ne);
        var le = ee.getOuterBBox();
        ee.element && ee.place(le.L * le.rscale, 0), this.action(this, this.data);
      }, Z.prototype.setEventHandler = function(te, ne) {
        this.element.addEventListener(te, ne);
      }, Z.kind = q.MmlMaction.prototype.kind, Z.styles = {
        '[jax="SVG"] mjx-tool': {
          display: "inline-block",
          position: "relative",
          width: 0,
          height: 0
        },
        '[jax="SVG"] mjx-tool > mjx-tip': {
          position: "absolute",
          top: 0,
          left: 0
        },
        "mjx-tool > mjx-tip": {
          display: "inline-block",
          padding: ".2em",
          border: "1px solid #888",
          "font-size": "70%",
          "background-color": "#F8F8F8",
          color: "black",
          "box-shadow": "2px 2px 5px #AAAAAA"
        },
        'g[data-mml-node="maction"][data-toggle]': {
          cursor: "pointer"
        },
        "mjx-status": {
          display: "block",
          position: "fixed",
          left: "1em",
          bottom: "1em",
          "min-width": "25%",
          padding: ".2em .4em",
          border: "1px solid #888",
          "font-size": "90%",
          "background-color": "#F8F8F8",
          color: "black"
        }
      }, Z.actions = /* @__PURE__ */ new Map([
        ["toggle", [function(te, ne) {
          te.adaptor.setAttribute(te.element, "data-toggle", te.node.attributes.get("selection"));
          var ee = te.factory.jax.math, ae = te.factory.jax.document, oe = te.node;
          te.setEventHandler("click", function(se) {
            ee.end.node || (ee.start.node = ee.end.node = ee.typesetRoot, ee.start.n = ee.end.n = 0), oe.nextToggleSelection(), ee.rerender(ae), se.stopPropagation();
          });
        }, {}]],
        ["tooltip", [function(te, ne) {
          var ee = te.childNodes[1];
          if (ee) {
            var ae = te.firstChild();
            if (ee.node.isKind("mtext")) {
              var oe = ee.node.getText();
              te.adaptor.insert(te.svg("title", {}, [te.text(oe)]), ae);
            } else {
              var se = te.adaptor, ce = te.jax.container, le = te.node.factory.create("math", {}, [te.childNodes[1].node]), ue = te.html("mjx-tool", {}, [te.html("mjx-tip")]), de = se.append(ae, te.svg("foreignObject", { style: { display: "none" } }, [ue]));
              te.jax.processMath(le, se.firstChild(ue)), te.childNodes[1].node.parent = te.node, te.setEventHandler("mouseover", function(fe) {
                ne.stopTimers(te, ne), ne.hoverTimer.set(te, setTimeout(function() {
                  se.setStyle(ue, "left", "0"), se.setStyle(ue, "top", "0"), se.append(ce, ue);
                  var he = se.nodeBBox(ue), me = se.nodeBBox(te.element), pe = (me.right - he.left) / te.metrics.em + te.dx, ye = (me.bottom - he.bottom) / te.metrics.em + te.dy;
                  se.setStyle(ue, "left", te.px(pe)), se.setStyle(ue, "top", te.px(ye));
                }, ne.postDelay)), fe.stopPropagation();
              }), te.setEventHandler("mouseout", function(fe) {
                ne.stopTimers(te, ne);
                var he = setTimeout(function() {
                  return se.append(de, ue);
                }, ne.clearDelay);
                ne.clearTimer.set(te, he), fe.stopPropagation();
              });
            }
          }
        }, V.TooltipData]],
        ["statusline", [function(te, ne) {
          var ee = te.childNodes[1];
          if (ee && ee.node.isKind("mtext")) {
            var ae = te.adaptor, oe = ee.node.getText();
            ae.setAttribute(te.element, "data-statusline", oe), te.setEventHandler("mouseover", function(se) {
              if (ne.status === null) {
                var ce = ae.body(ae.document);
                ne.status = ae.append(ce, te.html("mjx-status", {}, [te.text(oe)]));
              }
              se.stopPropagation();
            }), te.setEventHandler("mouseout", function(se) {
              ne.status && (ae.remove(ne.status), ne.status = null), se.stopPropagation();
            });
          }
        }, {
          status: null
        }]]
      ]), Z;
    }((0, H.CommonMactionMixin)(_.SVGWrapper));
    return maction$2.SVGmaction = G, maction$2;
  }
  var menclose$2 = {}, menclose$1 = {}, Notation$1 = {}, hasRequiredNotation$1;
  function requireNotation$1() {
    return hasRequiredNotation$1 || (hasRequiredNotation$1 = 1, function(O) {
      var _ = Notation$1.__read || function(ee, ae) {
        var oe = typeof Symbol == "function" && ee[Symbol.iterator];
        if (!oe) return ee;
        var se = oe.call(ee), ce, le = [], ue;
        try {
          for (; (ae === void 0 || ae-- > 0) && !(ce = se.next()).done; ) le.push(ce.value);
        } catch (de) {
          ue = { error: de };
        } finally {
          try {
            ce && !ce.done && (oe = se.return) && oe.call(se);
          } finally {
            if (ue) throw ue.error;
          }
        }
        return le;
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.CommonArrow = O.CommonDiagonalArrow = O.CommonDiagonalStrike = O.CommonBorder2 = O.CommonBorder = O.arrowBBox = O.diagonalArrowDef = O.arrowDef = O.arrowBBoxW = O.arrowBBoxHD = O.arrowHead = O.fullBorder = O.fullPadding = O.fullBBox = O.sideNames = O.sideIndex = O.SOLID = O.PADDING = O.THICKNESS = O.ARROWY = O.ARROWDX = O.ARROWX = void 0, O.ARROWX = 4, O.ARROWDX = 1, O.ARROWY = 2, O.THICKNESS = 0.067, O.PADDING = 0.2, O.SOLID = O.THICKNESS + "em solid", O.sideIndex = { top: 0, right: 1, bottom: 2, left: 3 }, O.sideNames = Object.keys(O.sideIndex), O.fullBBox = function(ee) {
        return new Array(4).fill(ee.thickness + ee.padding);
      }, O.fullPadding = function(ee) {
        return new Array(4).fill(ee.padding);
      }, O.fullBorder = function(ee) {
        return new Array(4).fill(ee.thickness);
      };
      var H = function(ee) {
        return Math.max(ee.padding, ee.thickness * (ee.arrowhead.x + ee.arrowhead.dx + 1));
      };
      O.arrowHead = H;
      var V = function(ee, ae) {
        if (ee.childNodes[0]) {
          var oe = ee.childNodes[0].getBBox(), se = oe.h, ce = oe.d;
          ae[0] = ae[2] = Math.max(0, ee.thickness * ee.arrowhead.y - (se + ce) / 2);
        }
        return ae;
      };
      O.arrowBBoxHD = V;
      var q = function(ee, ae) {
        if (ee.childNodes[0]) {
          var oe = ee.childNodes[0].getBBox().w;
          ae[1] = ae[3] = Math.max(0, ee.thickness * ee.arrowhead.y - oe / 2);
        }
        return ae;
      };
      O.arrowBBoxW = q, O.arrowDef = {
        up: [-Math.PI / 2, !1, !0, "verticalstrike"],
        down: [Math.PI / 2, !1, !0, "verticakstrike"],
        right: [0, !1, !1, "horizontalstrike"],
        left: [Math.PI, !1, !1, "horizontalstrike"],
        updown: [Math.PI / 2, !0, !0, "verticalstrike uparrow downarrow"],
        leftright: [0, !0, !1, "horizontalstrike leftarrow rightarrow"]
      }, O.diagonalArrowDef = {
        updiagonal: [-1, 0, !1, "updiagonalstrike northeastarrow"],
        northeast: [-1, 0, !1, "updiagonalstrike updiagonalarrow"],
        southeast: [1, 0, !1, "downdiagonalstrike"],
        northwest: [1, Math.PI, !1, "downdiagonalstrike"],
        southwest: [-1, Math.PI, !1, "updiagonalstrike"],
        northeastsouthwest: [-1, 0, !0, "updiagonalstrike northeastarrow updiagonalarrow southwestarrow"],
        northwestsoutheast: [1, 0, !0, "downdiagonalstrike northwestarrow southeastarrow"]
      }, O.arrowBBox = {
        up: function(ee) {
          return (0, O.arrowBBoxW)(ee, [(0, O.arrowHead)(ee), 0, ee.padding, 0]);
        },
        down: function(ee) {
          return (0, O.arrowBBoxW)(ee, [ee.padding, 0, (0, O.arrowHead)(ee), 0]);
        },
        right: function(ee) {
          return (0, O.arrowBBoxHD)(ee, [0, (0, O.arrowHead)(ee), 0, ee.padding]);
        },
        left: function(ee) {
          return (0, O.arrowBBoxHD)(ee, [0, ee.padding, 0, (0, O.arrowHead)(ee)]);
        },
        updown: function(ee) {
          return (0, O.arrowBBoxW)(ee, [(0, O.arrowHead)(ee), 0, (0, O.arrowHead)(ee), 0]);
        },
        leftright: function(ee) {
          return (0, O.arrowBBoxHD)(ee, [0, (0, O.arrowHead)(ee), 0, (0, O.arrowHead)(ee)]);
        }
      };
      var G = function(ee) {
        return function(ae) {
          var oe = O.sideIndex[ae];
          return [ae, {
            renderer: ee,
            bbox: function(se) {
              var ce = [0, 0, 0, 0];
              return ce[oe] = se.thickness + se.padding, ce;
            },
            border: function(se) {
              var ce = [0, 0, 0, 0];
              return ce[oe] = se.thickness, ce;
            }
          }];
        };
      };
      O.CommonBorder = G;
      var J = function(ee) {
        return function(ae, oe, se) {
          var ce = O.sideIndex[oe], le = O.sideIndex[se];
          return [ae, {
            renderer: ee,
            bbox: function(ue) {
              var de = ue.thickness + ue.padding, fe = [0, 0, 0, 0];
              return fe[ce] = fe[le] = de, fe;
            },
            border: function(ue) {
              var de = [0, 0, 0, 0];
              return de[ce] = de[le] = ue.thickness, de;
            },
            remove: oe + " " + se
          }];
        };
      };
      O.CommonBorder2 = J;
      var Z = function(ee) {
        return function(ae) {
          var oe = "mjx-" + ae.charAt(0) + "strike";
          return [ae + "diagonalstrike", {
            renderer: ee(oe),
            bbox: O.fullBBox
          }];
        };
      };
      O.CommonDiagonalStrike = Z;
      var te = function(ee) {
        return function(ae) {
          var oe = _(O.diagonalArrowDef[ae], 4), se = oe[0], ce = oe[1], le = oe[2], ue = oe[3];
          return [ae + "arrow", {
            renderer: function(de, fe) {
              var he = _(de.arrowAW(), 2), me = he[0], pe = he[1], ye = de.arrow(pe, se * (me - ce), le);
              ee(de, ye);
            },
            bbox: function(de) {
              var fe = de.arrowData(), he = fe.a, me = fe.x, pe = fe.y, ye = _([de.arrowhead.x, de.arrowhead.y, de.arrowhead.dx], 3), ge = ye[0], ve = ye[1], be = ye[2], Ce = _(de.getArgMod(ge + be, ve), 2), Se = Ce[0], we = Ce[1], Ee = pe + (Se > he ? de.thickness * we * Math.sin(Se - he) : 0), xe = me + (Se > Math.PI / 2 - he ? de.thickness * we * Math.sin(Se + he - Math.PI / 2) : 0);
              return [Ee, xe, Ee, xe];
            },
            remove: ue
          }];
        };
      };
      O.CommonDiagonalArrow = te;
      var ne = function(ee) {
        return function(ae) {
          var oe = _(O.arrowDef[ae], 4), se = oe[0], ce = oe[1], le = oe[2], ue = oe[3];
          return [ae + "arrow", {
            renderer: function(de, fe) {
              var he = de.getBBox(), me = he.w, pe = he.h, ye = he.d, ge = _(le ? [pe + ye, "X"] : [me, "Y"], 2), ve = ge[0], be = ge[1], Ce = de.getOffset(be), Se = de.arrow(ve, se, ce, be, Ce);
              ee(de, Se);
            },
            bbox: O.arrowBBox[ae],
            remove: ue
          }];
        };
      };
      O.CommonArrow = ne;
    }(Notation$1)), Notation$1;
  }
  var hasRequiredMenclose$2;
  function requireMenclose$2() {
    if (hasRequiredMenclose$2) return menclose$1;
    hasRequiredMenclose$2 = 1;
    var O = menclose$1.__extends || /* @__PURE__ */ function() {
      var ee = function(ae, oe) {
        return ee = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(se, ce) {
          se.__proto__ = ce;
        } || function(se, ce) {
          for (var le in ce) Object.prototype.hasOwnProperty.call(ce, le) && (se[le] = ce[le]);
        }, ee(ae, oe);
      };
      return function(ae, oe) {
        if (typeof oe != "function" && oe !== null)
          throw new TypeError("Class extends value " + String(oe) + " is not a constructor or null");
        ee(ae, oe);
        function se() {
          this.constructor = ae;
        }
        ae.prototype = oe === null ? Object.create(oe) : (se.prototype = oe.prototype, new se());
      };
    }(), _ = menclose$1.__createBinding || (Object.create ? function(ee, ae, oe, se) {
      se === void 0 && (se = oe);
      var ce = Object.getOwnPropertyDescriptor(ae, oe);
      (!ce || ("get" in ce ? !ae.__esModule : ce.writable || ce.configurable)) && (ce = { enumerable: !0, get: function() {
        return ae[oe];
      } }), Object.defineProperty(ee, se, ce);
    } : function(ee, ae, oe, se) {
      se === void 0 && (se = oe), ee[se] = ae[oe];
    }), H = menclose$1.__setModuleDefault || (Object.create ? function(ee, ae) {
      Object.defineProperty(ee, "default", { enumerable: !0, value: ae });
    } : function(ee, ae) {
      ee.default = ae;
    }), V = menclose$1.__importStar || function(ee) {
      if (ee && ee.__esModule) return ee;
      var ae = {};
      if (ee != null) for (var oe in ee) oe !== "default" && Object.prototype.hasOwnProperty.call(ee, oe) && _(ae, ee, oe);
      return H(ae, ee), ae;
    }, q = menclose$1.__read || function(ee, ae) {
      var oe = typeof Symbol == "function" && ee[Symbol.iterator];
      if (!oe) return ee;
      var se = oe.call(ee), ce, le = [], ue;
      try {
        for (; (ae === void 0 || ae-- > 0) && !(ce = se.next()).done; ) le.push(ce.value);
      } catch (de) {
        ue = { error: de };
      } finally {
        try {
          ce && !ce.done && (oe = se.return) && oe.call(se);
        } finally {
          if (ue) throw ue.error;
        }
      }
      return le;
    }, G = menclose$1.__spreadArray || function(ee, ae, oe) {
      if (oe || arguments.length === 2) for (var se = 0, ce = ae.length, le; se < ce; se++)
        (le || !(se in ae)) && (le || (le = Array.prototype.slice.call(ae, 0, se)), le[se] = ae[se]);
      return ee.concat(le || Array.prototype.slice.call(ae));
    }, J = menclose$1.__values || function(ee) {
      var ae = typeof Symbol == "function" && Symbol.iterator, oe = ae && ee[ae], se = 0;
      if (oe) return oe.call(ee);
      if (ee && typeof ee.length == "number") return {
        next: function() {
          return ee && se >= ee.length && (ee = void 0), { value: ee && ee[se++], done: !ee };
        }
      };
      throw new TypeError(ae ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(menclose$1, "__esModule", { value: !0 }), menclose$1.CommonMencloseMixin = void 0;
    var Z = V(requireNotation$1()), te = requireString();
    function ne(ee) {
      return function(ae) {
        O(oe, ae);
        function oe() {
          for (var se = [], ce = 0; ce < arguments.length; ce++)
            se[ce] = arguments[ce];
          var le = ae.apply(this, G([], q(se), !1)) || this;
          return le.notations = {}, le.renderChild = null, le.msqrt = null, le.padding = Z.PADDING, le.thickness = Z.THICKNESS, le.arrowhead = { x: Z.ARROWX, y: Z.ARROWY, dx: Z.ARROWDX }, le.TRBL = [0, 0, 0, 0], le.getParameters(), le.getNotations(), le.removeRedundantNotations(), le.initializeNotations(), le.TRBL = le.getBBoxExtenders(), le;
        }
        return oe.prototype.getParameters = function() {
          var se = this.node.attributes, ce = se.get("data-padding");
          ce !== void 0 && (this.padding = this.length2em(ce, Z.PADDING));
          var le = se.get("data-thickness");
          le !== void 0 && (this.thickness = this.length2em(le, Z.THICKNESS));
          var ue = se.get("data-arrowhead");
          if (ue !== void 0) {
            var de = q((0, te.split)(ue), 3), fe = de[0], he = de[1], me = de[2];
            this.arrowhead = {
              x: fe ? parseFloat(fe) : Z.ARROWX,
              y: he ? parseFloat(he) : Z.ARROWY,
              dx: me ? parseFloat(me) : Z.ARROWDX
            };
          }
        }, oe.prototype.getNotations = function() {
          var se, ce, le = this.constructor.notations;
          try {
            for (var ue = J((0, te.split)(this.node.attributes.get("notation"))), de = ue.next(); !de.done; de = ue.next()) {
              var fe = de.value, he = le.get(fe);
              he && (this.notations[fe] = he, he.renderChild && (this.renderChild = he.renderer));
            }
          } catch (me) {
            se = { error: me };
          } finally {
            try {
              de && !de.done && (ce = ue.return) && ce.call(ue);
            } finally {
              if (se) throw se.error;
            }
          }
        }, oe.prototype.removeRedundantNotations = function() {
          var se, ce, le, ue;
          try {
            for (var de = J(Object.keys(this.notations)), fe = de.next(); !fe.done; fe = de.next()) {
              var he = fe.value;
              if (this.notations[he]) {
                var me = this.notations[he].remove || "";
                try {
                  for (var pe = (le = void 0, J(me.split(/ /))), ye = pe.next(); !ye.done; ye = pe.next()) {
                    var ge = ye.value;
                    delete this.notations[ge];
                  }
                } catch (ve) {
                  le = { error: ve };
                } finally {
                  try {
                    ye && !ye.done && (ue = pe.return) && ue.call(pe);
                  } finally {
                    if (le) throw le.error;
                  }
                }
              }
            }
          } catch (ve) {
            se = { error: ve };
          } finally {
            try {
              fe && !fe.done && (ce = de.return) && ce.call(de);
            } finally {
              if (se) throw se.error;
            }
          }
        }, oe.prototype.initializeNotations = function() {
          var se, ce;
          try {
            for (var le = J(Object.keys(this.notations)), ue = le.next(); !ue.done; ue = le.next()) {
              var de = ue.value, fe = this.notations[de].init;
              fe && fe(this);
            }
          } catch (he) {
            se = { error: he };
          } finally {
            try {
              ue && !ue.done && (ce = le.return) && ce.call(le);
            } finally {
              if (se) throw se.error;
            }
          }
        }, oe.prototype.computeBBox = function(se, ce) {
          ce === void 0 && (ce = !1);
          var le = q(this.TRBL, 4), ue = le[0], de = le[1], fe = le[2], he = le[3], me = this.childNodes[0].getBBox();
          se.combine(me, he, 0), se.h += ue, se.d += fe, se.w += de, this.setChildPWidths(ce);
        }, oe.prototype.getBBoxExtenders = function() {
          var se, ce, le = [0, 0, 0, 0];
          try {
            for (var ue = J(Object.keys(this.notations)), de = ue.next(); !de.done; de = ue.next()) {
              var fe = de.value;
              this.maximizeEntries(le, this.notations[fe].bbox(this));
            }
          } catch (he) {
            se = { error: he };
          } finally {
            try {
              de && !de.done && (ce = ue.return) && ce.call(ue);
            } finally {
              if (se) throw se.error;
            }
          }
          return le;
        }, oe.prototype.getPadding = function() {
          var se, ce, le = this, ue = [0, 0, 0, 0];
          try {
            for (var de = J(Object.keys(this.notations)), fe = de.next(); !fe.done; fe = de.next()) {
              var he = fe.value, me = this.notations[he].border;
              me && this.maximizeEntries(ue, me(this));
            }
          } catch (pe) {
            se = { error: pe };
          } finally {
            try {
              fe && !fe.done && (ce = de.return) && ce.call(de);
            } finally {
              if (se) throw se.error;
            }
          }
          return [0, 1, 2, 3].map(function(pe) {
            return le.TRBL[pe] - ue[pe];
          });
        }, oe.prototype.maximizeEntries = function(se, ce) {
          for (var le = 0; le < se.length; le++)
            se[le] < ce[le] && (se[le] = ce[le]);
        }, oe.prototype.getOffset = function(se) {
          var ce = q(this.TRBL, 4), le = ce[0], ue = ce[1], de = ce[2], fe = ce[3], he = (se === "X" ? ue - fe : de - le) / 2;
          return Math.abs(he) > 1e-3 ? he : 0;
        }, oe.prototype.getArgMod = function(se, ce) {
          return [Math.atan2(ce, se), Math.sqrt(se * se + ce * ce)];
        }, oe.prototype.arrow = function(se, ce, le, ue, de) {
          return null;
        }, oe.prototype.arrowData = function() {
          var se = q([this.padding, this.thickness], 2), ce = se[0], le = se[1], ue = le * (this.arrowhead.x + Math.max(1, this.arrowhead.dx)), de = this.childNodes[0].getBBox(), fe = de.h, he = de.d, me = de.w, pe = fe + he, ye = Math.sqrt(pe * pe + me * me), ge = Math.max(ce, ue * me / ye), ve = Math.max(ce, ue * pe / ye), be = q(this.getArgMod(me + 2 * ge, pe + 2 * ve), 2), Ce = be[0], Se = be[1];
          return { a: Ce, W: Se, x: ge, y: ve };
        }, oe.prototype.arrowAW = function() {
          var se = this.childNodes[0].getBBox(), ce = se.h, le = se.d, ue = se.w, de = q(this.TRBL, 4), fe = de[0], he = de[1], me = de[2], pe = de[3];
          return this.getArgMod(pe + ue + he, fe + ce + le + me);
        }, oe.prototype.createMsqrt = function(se) {
          var ce = this.node.factory, le = ce.create("msqrt");
          le.inheritAttributesFrom(this.node), le.childNodes[0] = se.node;
          var ue = this.wrap(le);
          return ue.parent = this, ue;
        }, oe.prototype.sqrtTRBL = function() {
          var se = this.msqrt.getBBox(), ce = this.msqrt.childNodes[0].getBBox();
          return [se.h - ce.h, 0, se.d - ce.d, se.w - ce.w];
        }, oe;
      }(ee);
    }
    return menclose$1.CommonMencloseMixin = ne, menclose$1;
  }
  var Notation = {}, hasRequiredNotation;
  function requireNotation() {
    return hasRequiredNotation || (hasRequiredNotation = 1, function(O) {
      var _ = Notation.__createBinding || (Object.create ? function(ce, le, ue, de) {
        de === void 0 && (de = ue);
        var fe = Object.getOwnPropertyDescriptor(le, ue);
        (!fe || ("get" in fe ? !le.__esModule : fe.writable || fe.configurable)) && (fe = { enumerable: !0, get: function() {
          return le[ue];
        } }), Object.defineProperty(ce, de, fe);
      } : function(ce, le, ue, de) {
        de === void 0 && (de = ue), ce[de] = le[ue];
      }), H = Notation.__setModuleDefault || (Object.create ? function(ce, le) {
        Object.defineProperty(ce, "default", { enumerable: !0, value: le });
      } : function(ce, le) {
        ce.default = le;
      }), V = Notation.__importStar || function(ce) {
        if (ce && ce.__esModule) return ce;
        var le = {};
        if (ce != null) for (var ue in ce) ue !== "default" && Object.prototype.hasOwnProperty.call(ce, ue) && _(le, ce, ue);
        return H(le, ce), le;
      }, q = Notation.__exportStar || function(ce, le) {
        for (var ue in ce) ue !== "default" && !Object.prototype.hasOwnProperty.call(le, ue) && _(le, ce, ue);
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.Arrow = O.DiagonalArrow = O.DiagonalStrike = O.Border2 = O.Border = O.RenderLine = O.lineOffset = O.lineData = O.computeLineData = void 0;
      var G = V(requireNotation$1());
      q(requireNotation$1(), O), O.computeLineData = {
        top: function(ce, le, ue, de) {
          return [0, ce - de, ue, ce - de];
        },
        right: function(ce, le, ue, de) {
          return [ue - de, -le, ue - de, ce];
        },
        bottom: function(ce, le, ue, de) {
          return [0, de - le, ue, de - le];
        },
        left: function(ce, le, ue, de) {
          return [de, -le, de, ce];
        },
        vertical: function(ce, le, ue, de) {
          return [ue / 2, ce, ue / 2, -le];
        },
        horizontal: function(ce, le, ue, de) {
          return [0, (ce - le) / 2, ue, (ce - le) / 2];
        },
        up: function(ce, le, ue, de) {
          return [de, de - le, ue - de, ce - de];
        },
        down: function(ce, le, ue, de) {
          return [de, ce - de, ue - de, de - le];
        }
      };
      var J = function(ce, le, ue) {
        ue === void 0 && (ue = "");
        var de = ce.getBBox(), fe = de.h, he = de.d, me = de.w, pe = ce.thickness / 2;
        return (0, O.lineOffset)(O.computeLineData[le](fe, he, me, pe), ce, ue);
      };
      O.lineData = J;
      var Z = function(ce, le, ue) {
        if (ue) {
          var de = le.getOffset(ue);
          de && (ue === "X" ? (ce[0] -= de, ce[2] -= de) : (ce[1] -= de, ce[3] -= de));
        }
        return ce;
      };
      O.lineOffset = Z;
      var te = function(ce, le) {
        return le === void 0 && (le = ""), function(ue, de) {
          var fe = ue.line((0, O.lineData)(ue, ce, le));
          ue.adaptor.append(ue.element, fe);
        };
      };
      O.RenderLine = te;
      var ne = function(ce) {
        return G.CommonBorder(function(le, ue) {
          le.adaptor.append(le.element, le.line((0, O.lineData)(le, ce)));
        })(ce);
      };
      O.Border = ne;
      var ee = function(ce, le, ue) {
        return G.CommonBorder2(function(de, fe) {
          de.adaptor.append(de.element, de.line((0, O.lineData)(de, le))), de.adaptor.append(de.element, de.line((0, O.lineData)(de, ue)));
        })(ce, le, ue);
      };
      O.Border2 = ee;
      var ae = function(ce) {
        return G.CommonDiagonalStrike(function(le) {
          return function(ue, de) {
            ue.adaptor.append(ue.element, ue.line((0, O.lineData)(ue, ce)));
          };
        })(ce);
      };
      O.DiagonalStrike = ae;
      var oe = function(ce) {
        return G.CommonDiagonalArrow(function(le, ue) {
          le.adaptor.append(le.element, ue);
        })(ce);
      };
      O.DiagonalArrow = oe;
      var se = function(ce) {
        return G.CommonArrow(function(le, ue) {
          le.adaptor.append(le.element, ue);
        })(ce);
      };
      O.Arrow = se;
    }(Notation)), Notation;
  }
  var menclose = {}, hasRequiredMenclose$1;
  function requireMenclose$1() {
    if (hasRequiredMenclose$1) return menclose;
    hasRequiredMenclose$1 = 1;
    var O = menclose.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = menclose.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(menclose, "__esModule", { value: !0 }), menclose.MmlMenclose = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "menclose";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return -1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "linebreakContininer", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.setTeXclass = function(J) {
        return J = this.childNodes[0].setTeXclass(J), this.updateTeXclass(this.childNodes[0]), J;
      }, G.defaults = _(_({}, H.AbstractMmlNode.defaults), { notation: "longdiv" }), G;
    }(H.AbstractMmlNode);
    return menclose.MmlMenclose = V, menclose;
  }
  var hasRequiredMenclose;
  function requireMenclose() {
    if (hasRequiredMenclose) return menclose$2;
    hasRequiredMenclose = 1;
    var O = menclose$2.__extends || /* @__PURE__ */ function() {
      var ae = function(oe, se) {
        return ae = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ce, le) {
          ce.__proto__ = le;
        } || function(ce, le) {
          for (var ue in le) Object.prototype.hasOwnProperty.call(le, ue) && (ce[ue] = le[ue]);
        }, ae(oe, se);
      };
      return function(oe, se) {
        if (typeof se != "function" && se !== null)
          throw new TypeError("Class extends value " + String(se) + " is not a constructor or null");
        ae(oe, se);
        function ce() {
          this.constructor = oe;
        }
        oe.prototype = se === null ? Object.create(se) : (ce.prototype = se.prototype, new ce());
      };
    }(), _ = menclose$2.__createBinding || (Object.create ? function(ae, oe, se, ce) {
      ce === void 0 && (ce = se);
      var le = Object.getOwnPropertyDescriptor(oe, se);
      (!le || ("get" in le ? !oe.__esModule : le.writable || le.configurable)) && (le = { enumerable: !0, get: function() {
        return oe[se];
      } }), Object.defineProperty(ae, ce, le);
    } : function(ae, oe, se, ce) {
      ce === void 0 && (ce = se), ae[ce] = oe[se];
    }), H = menclose$2.__setModuleDefault || (Object.create ? function(ae, oe) {
      Object.defineProperty(ae, "default", { enumerable: !0, value: oe });
    } : function(ae, oe) {
      ae.default = oe;
    }), V = menclose$2.__importStar || function(ae) {
      if (ae && ae.__esModule) return ae;
      var oe = {};
      if (ae != null) for (var se in ae) se !== "default" && Object.prototype.hasOwnProperty.call(ae, se) && _(oe, ae, se);
      return H(oe, ae), oe;
    }, q = menclose$2.__values || function(ae) {
      var oe = typeof Symbol == "function" && Symbol.iterator, se = oe && ae[oe], ce = 0;
      if (se) return se.call(ae);
      if (ae && typeof ae.length == "number") return {
        next: function() {
          return ae && ce >= ae.length && (ae = void 0), { value: ae && ae[ce++], done: !ae };
        }
      };
      throw new TypeError(oe ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, G = menclose$2.__read || function(ae, oe) {
      var se = typeof Symbol == "function" && ae[Symbol.iterator];
      if (!se) return ae;
      var ce = se.call(ae), le, ue = [], de;
      try {
        for (; (oe === void 0 || oe-- > 0) && !(le = ce.next()).done; ) ue.push(le.value);
      } catch (fe) {
        de = { error: fe };
      } finally {
        try {
          le && !le.done && (se = ce.return) && se.call(ce);
        } finally {
          if (de) throw de.error;
        }
      }
      return ue;
    };
    Object.defineProperty(menclose$2, "__esModule", { value: !0 }), menclose$2.SVGmenclose = void 0;
    var J = requireWrapper(), Z = requireMenclose$2(), te = V(requireNotation()), ne = requireMenclose$1(), ee = function(ae) {
      O(oe, ae);
      function oe() {
        return ae !== null && ae.apply(this, arguments) || this;
      }
      return oe.prototype.toSVG = function(se) {
        var ce, le, ue = this.standardSVGnode(se), de = this.getBBoxExtenders()[3], fe = {};
        de > 0 && (fe.transform = "translate(" + this.fixed(de) + ", 0)");
        var he = this.adaptor.append(ue, this.svg("g", fe));
        this.renderChild ? this.renderChild(this, he) : this.childNodes[0].toSVG(he);
        try {
          for (var me = q(Object.keys(this.notations)), pe = me.next(); !pe.done; pe = me.next()) {
            var ye = pe.value, ge = this.notations[ye];
            !ge.renderChild && ge.renderer(this, ue);
          }
        } catch (ve) {
          ce = { error: ve };
        } finally {
          try {
            pe && !pe.done && (le = me.return) && le.call(me);
          } finally {
            if (ce) throw ce.error;
          }
        }
      }, oe.prototype.arrow = function(se, ce, le, ue, de) {
        ue === void 0 && (ue = ""), de === void 0 && (de = 0);
        var fe = this.getBBox(), he = fe.w, me = fe.h, pe = fe.d, ye = (se - he) / 2, ge = (me - pe) / 2, ve = this.thickness, be = ve / 2, Ce = G([ve * this.arrowhead.x, ve * this.arrowhead.y, ve * this.arrowhead.dx], 3), Se = Ce[0], we = Ce[1], Ee = Ce[2], xe = le ? this.fill("M", he + ye, ge, "l", -(Se + Ee), we, "l", Ee, be - we, "L", Se - ye, ge + be, "l", Ee, we - be, "l", -(Se + Ee), -we, "l", Se + Ee, -we, "l", -Ee, we - be, "L", he + ye - Se, ge - be, "l", -Ee, be - we, "Z") : this.fill("M", he + ye, ge, "l", -(Se + Ee), we, "l", Ee, be - we, "L", -ye, ge + be, "l", 0, -ve, "L", he + ye - Se, ge - be, "l", -Ee, be - we, "Z"), Le = [];
        if (de && Le.push(ue === "X" ? "translate(".concat(this.fixed(-de), " 0)") : "translate(0 ".concat(this.fixed(de), ")")), ce) {
          var Qe = this.jax.fixed(-ce * 180 / Math.PI);
          Le.push("rotate(".concat(Qe, " ").concat(this.fixed(he / 2), " ").concat(this.fixed(ge), ")"));
        }
        return Le.length && this.adaptor.setAttribute(xe, "transform", Le.join(" ")), xe;
      }, oe.prototype.line = function(se) {
        var ce = G(se, 4), le = ce[0], ue = ce[1], de = ce[2], fe = ce[3];
        return this.svg("line", {
          x1: this.fixed(le),
          y1: this.fixed(ue),
          x2: this.fixed(de),
          y2: this.fixed(fe),
          "stroke-width": this.fixed(this.thickness)
        });
      }, oe.prototype.box = function(se, ce, le, ue) {
        ue === void 0 && (ue = 0);
        var de = this.thickness, fe = {
          x: this.fixed(de / 2),
          y: this.fixed(de / 2 - le),
          width: this.fixed(se - de),
          height: this.fixed(ce + le - de),
          fill: "none",
          "stroke-width": this.fixed(de)
        };
        return ue && (fe.rx = this.fixed(ue)), this.svg("rect", fe);
      }, oe.prototype.ellipse = function(se, ce, le) {
        var ue = this.thickness;
        return this.svg("ellipse", {
          rx: this.fixed((se - ue) / 2),
          ry: this.fixed((ce + le - ue) / 2),
          cx: this.fixed(se / 2),
          cy: this.fixed((ce - le) / 2),
          fill: "none",
          "stroke-width": this.fixed(ue)
        });
      }, oe.prototype.path = function(se) {
        for (var ce = this, le = [], ue = 1; ue < arguments.length; ue++)
          le[ue - 1] = arguments[ue];
        return this.svg("path", {
          d: le.map(function(de) {
            return typeof de == "string" ? de : ce.fixed(de);
          }).join(" "),
          style: { "stroke-width": this.fixed(this.thickness) },
          "stroke-linecap": "round",
          "stroke-linejoin": se,
          fill: "none"
        });
      }, oe.prototype.fill = function() {
        for (var se = this, ce = [], le = 0; le < arguments.length; le++)
          ce[le] = arguments[le];
        return this.svg("path", {
          d: ce.map(function(ue) {
            return typeof ue == "string" ? ue : se.fixed(ue);
          }).join(" ")
        });
      }, oe.kind = ne.MmlMenclose.prototype.kind, oe.notations = new Map([
        te.Border("top"),
        te.Border("right"),
        te.Border("bottom"),
        te.Border("left"),
        te.Border2("actuarial", "top", "right"),
        te.Border2("madruwb", "bottom", "right"),
        te.DiagonalStrike("up"),
        te.DiagonalStrike("down"),
        ["horizontalstrike", {
          renderer: te.RenderLine("horizontal", "Y"),
          bbox: function(se) {
            return [0, se.padding, 0, se.padding];
          }
        }],
        ["verticalstrike", {
          renderer: te.RenderLine("vertical", "X"),
          bbox: function(se) {
            return [se.padding, 0, se.padding, 0];
          }
        }],
        ["box", {
          renderer: function(se, ce) {
            var le = se.getBBox(), ue = le.w, de = le.h, fe = le.d;
            se.adaptor.append(se.element, se.box(ue, de, fe));
          },
          bbox: te.fullBBox,
          border: te.fullBorder,
          remove: "left right top bottom"
        }],
        ["roundedbox", {
          renderer: function(se, ce) {
            var le = se.getBBox(), ue = le.w, de = le.h, fe = le.d, he = se.thickness + se.padding;
            se.adaptor.append(se.element, se.box(ue, de, fe, he));
          },
          bbox: te.fullBBox
        }],
        ["circle", {
          renderer: function(se, ce) {
            var le = se.getBBox(), ue = le.w, de = le.h, fe = le.d;
            se.adaptor.append(se.element, se.ellipse(ue, de, fe));
          },
          bbox: te.fullBBox
        }],
        ["phasorangle", {
          renderer: function(se, ce) {
            var le = se.getBBox(), ue = le.w, de = le.h, fe = le.d, he = se.getArgMod(1.75 * se.padding, de + fe)[0], me = se.thickness / 2, pe = de + fe, ye = Math.cos(he);
            se.adaptor.append(se.element, se.path("mitre", "M", ue, me - fe, "L", me + ye * me, me - fe, "L", ye * pe + me, pe - fe - me));
          },
          bbox: function(se) {
            var ce = se.padding / 2, le = se.thickness;
            return [2 * ce, ce, ce + le, 3 * ce + le];
          },
          border: function(se) {
            return [0, 0, se.thickness, 0];
          },
          remove: "bottom"
        }],
        te.Arrow("up"),
        te.Arrow("down"),
        te.Arrow("left"),
        te.Arrow("right"),
        te.Arrow("updown"),
        te.Arrow("leftright"),
        te.DiagonalArrow("updiagonal"),
        te.DiagonalArrow("northeast"),
        te.DiagonalArrow("southeast"),
        te.DiagonalArrow("northwest"),
        te.DiagonalArrow("southwest"),
        te.DiagonalArrow("northeastsouthwest"),
        te.DiagonalArrow("northwestsoutheast"),
        ["longdiv", {
          renderer: function(se, ce) {
            var le = se.getBBox(), ue = le.w, de = le.h, fe = le.d, he = se.thickness / 2, me = se.padding;
            se.adaptor.append(se.element, se.path("round", "M", he, he - fe, "a", me - he / 2, (de + fe) / 2 - 4 * he, 0, "0,1", 0, de + fe - 2 * he, "L", ue - he, de - he));
          },
          bbox: function(se) {
            var ce = se.padding, le = se.thickness;
            return [ce + le, ce, ce, 2 * ce + le / 2];
          }
        }],
        ["radical", {
          renderer: function(se, ce) {
            se.msqrt.toSVG(ce);
            var le = se.sqrtTRBL()[3];
            se.place(-le, 0, ce);
          },
          init: function(se) {
            se.msqrt = se.createMsqrt(se.childNodes[0]);
          },
          bbox: function(se) {
            return se.sqrtTRBL();
          },
          renderChild: !0
        }]
      ]), oe;
    }((0, Z.CommonMencloseMixin)(J.SVGWrapper));
    return menclose$2.SVGmenclose = ee, menclose$2;
  }
  var semantics$2 = {}, semantics$1 = {}, hasRequiredSemantics$2;
  function requireSemantics$2() {
    if (hasRequiredSemantics$2) return semantics$1;
    hasRequiredSemantics$2 = 1;
    var O = semantics$1.__extends || /* @__PURE__ */ function() {
      var H = function(V, q) {
        return H = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(G, J) {
          G.__proto__ = J;
        } || function(G, J) {
          for (var Z in J) Object.prototype.hasOwnProperty.call(J, Z) && (G[Z] = J[Z]);
        }, H(V, q);
      };
      return function(V, q) {
        if (typeof q != "function" && q !== null)
          throw new TypeError("Class extends value " + String(q) + " is not a constructor or null");
        H(V, q);
        function G() {
          this.constructor = V;
        }
        V.prototype = q === null ? Object.create(q) : (G.prototype = q.prototype, new G());
      };
    }();
    Object.defineProperty(semantics$1, "__esModule", { value: !0 }), semantics$1.CommonSemanticsMixin = void 0;
    function _(H) {
      return function(V) {
        O(q, V);
        function q() {
          return V !== null && V.apply(this, arguments) || this;
        }
        return q.prototype.computeBBox = function(G, J) {
          if (this.childNodes.length) {
            var Z = this.childNodes[0].getBBox(), te = Z.w, ne = Z.h, ee = Z.d;
            G.w = te, G.h = ne, G.d = ee;
          }
        }, q;
      }(H);
    }
    return semantics$1.CommonSemanticsMixin = _, semantics$1;
  }
  var semantics = {}, hasRequiredSemantics$1;
  function requireSemantics$1() {
    if (hasRequiredSemantics$1) return semantics;
    hasRequiredSemantics$1 = 1;
    var O = semantics.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = semantics.__assign || function() {
      return _ = Object.assign || function(J) {
        for (var Z, te = 1, ne = arguments.length; te < ne; te++) {
          Z = arguments[te];
          for (var ee in Z) Object.prototype.hasOwnProperty.call(Z, ee) && (J[ee] = Z[ee]);
        }
        return J;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(semantics, "__esModule", { value: !0 }), semantics.MmlAnnotation = semantics.MmlAnnotationXML = semantics.MmlSemantics = void 0;
    var H = requireMmlNode(), V = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "semantics";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "arity", {
        get: function() {
          return 1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "notParent", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Z.defaults = _(_({}, H.AbstractMmlBaseNode.defaults), { definitionUrl: null, encoding: null }), Z;
    }(H.AbstractMmlBaseNode);
    semantics.MmlSemantics = V;
    var q = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "annotation-xml";
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.setChildInheritedAttributes = function() {
      }, Z.defaults = _(_({}, H.AbstractMmlNode.defaults), { definitionUrl: null, encoding: null, cd: "mathmlkeys", name: "", src: null }), Z;
    }(H.AbstractMmlNode);
    semantics.MmlAnnotationXML = q;
    var G = function(J) {
      O(Z, J);
      function Z() {
        var te = J !== null && J.apply(this, arguments) || this;
        return te.properties = {
          isChars: !0
        }, te;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "annotation";
        },
        enumerable: !1,
        configurable: !0
      }), Z.defaults = _({}, q.defaults), Z;
    }(q);
    return semantics.MmlAnnotation = G, semantics;
  }
  var hasRequiredSemantics;
  function requireSemantics() {
    if (hasRequiredSemantics) return semantics$2;
    hasRequiredSemantics = 1;
    var O = semantics$2.__extends || /* @__PURE__ */ function() {
      var ne = function(ee, ae) {
        return ne = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(oe, se) {
          oe.__proto__ = se;
        } || function(oe, se) {
          for (var ce in se) Object.prototype.hasOwnProperty.call(se, ce) && (oe[ce] = se[ce]);
        }, ne(ee, ae);
      };
      return function(ee, ae) {
        if (typeof ae != "function" && ae !== null)
          throw new TypeError("Class extends value " + String(ae) + " is not a constructor or null");
        ne(ee, ae);
        function oe() {
          this.constructor = ee;
        }
        ee.prototype = ae === null ? Object.create(ae) : (oe.prototype = ae.prototype, new oe());
      };
    }();
    Object.defineProperty(semantics$2, "__esModule", { value: !0 }), semantics$2.SVGxml = semantics$2.SVGannotationXML = semantics$2.SVGannotation = semantics$2.SVGsemantics = void 0;
    var _ = requireWrapper(), H = requireSemantics$2(), V = requireSemantics$1(), q = requireMmlNode(), G = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee.prototype.toSVG = function(ae) {
        var oe = this.standardSVGnode(ae);
        this.childNodes.length && this.childNodes[0].toSVG(oe);
      }, ee.kind = V.MmlSemantics.prototype.kind, ee;
    }((0, H.CommonSemanticsMixin)(_.SVGWrapper));
    semantics$2.SVGsemantics = G;
    var J = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee.prototype.toSVG = function(ae) {
        ne.prototype.toSVG.call(this, ae);
      }, ee.prototype.computeBBox = function() {
        return this.bbox;
      }, ee.kind = V.MmlAnnotation.prototype.kind, ee;
    }(_.SVGWrapper);
    semantics$2.SVGannotation = J;
    var Z = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee.kind = V.MmlAnnotationXML.prototype.kind, ee.styles = {
        "foreignObject[data-mjx-xml]": {
          "font-family": "initial",
          "line-height": "normal",
          overflow: "visible"
        }
      }, ee;
    }(_.SVGWrapper);
    semantics$2.SVGannotationXML = Z;
    var te = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee.prototype.toSVG = function(ae) {
        var oe = this.adaptor.clone(this.node.getXML()), se = this.jax.math.metrics.em * this.jax.math.metrics.scale, ce = this.fixed(1 / se), le = this.getBBox(), ue = le.w, de = le.h, fe = le.d;
        this.element = this.adaptor.append(ae, this.svg("foreignObject", {
          "data-mjx-xml": !0,
          y: this.jax.fixed(-de * se) + "px",
          width: this.jax.fixed(ue * se) + "px",
          height: this.jax.fixed((de + fe) * se) + "px",
          transform: "scale(".concat(ce, ") matrix(1 0 0 -1 0 0)")
        }, [oe]));
      }, ee.prototype.computeBBox = function(ae, oe) {
        var se = this.jax.measureXMLnode(this.node.getXML()), ce = se.w, le = se.h, ue = se.d;
        ae.w = ce, ae.h = le, ae.d = ue;
      }, ee.prototype.getStyles = function() {
      }, ee.prototype.getScale = function() {
      }, ee.prototype.getVariant = function() {
      }, ee.kind = q.XMLNode.prototype.kind, ee.autoStyle = !1, ee;
    }(_.SVGWrapper);
    return semantics$2.SVGxml = te, semantics$2;
  }
  var mglyph$2 = {}, mglyph$1 = {}, hasRequiredMglyph$2;
  function requireMglyph$2() {
    if (hasRequiredMglyph$2) return mglyph$1;
    hasRequiredMglyph$2 = 1;
    var O = mglyph$1.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mglyph$1.__read || function(q, G) {
      var J = typeof Symbol == "function" && q[Symbol.iterator];
      if (!J) return q;
      var Z = J.call(q), te, ne = [], ee;
      try {
        for (; (G === void 0 || G-- > 0) && !(te = Z.next()).done; ) ne.push(te.value);
      } catch (ae) {
        ee = { error: ae };
      } finally {
        try {
          te && !te.done && (J = Z.return) && J.call(Z);
        } finally {
          if (ee) throw ee.error;
        }
      }
      return ne;
    }, H = mglyph$1.__spreadArray || function(q, G, J) {
      if (J || arguments.length === 2) for (var Z = 0, te = G.length, ne; Z < te; Z++)
        (ne || !(Z in G)) && (ne || (ne = Array.prototype.slice.call(G, 0, Z)), ne[Z] = G[Z]);
      return q.concat(ne || Array.prototype.slice.call(G));
    };
    Object.defineProperty(mglyph$1, "__esModule", { value: !0 }), mglyph$1.CommonMglyphMixin = void 0;
    function V(q) {
      return function(G) {
        O(J, G);
        function J() {
          for (var Z = [], te = 0; te < arguments.length; te++)
            Z[te] = arguments[te];
          var ne = G.apply(this, H([], _(Z), !1)) || this;
          return ne.getParameters(), ne;
        }
        return J.prototype.getParameters = function() {
          var Z = this.node.attributes.getList("width", "height", "valign", "src", "index"), te = Z.width, ne = Z.height, ee = Z.valign, ae = Z.src, oe = Z.index;
          if (ae)
            this.width = te === "auto" ? 1 : this.length2em(te), this.height = ne === "auto" ? 1 : this.length2em(ne), this.valign = this.length2em(ee || "0");
          else {
            var se = String.fromCodePoint(parseInt(oe)), ce = this.node.factory;
            this.charWrapper = this.wrap(ce.create("text").setText(se)), this.charWrapper.parent = this;
          }
        }, J.prototype.computeBBox = function(Z, te) {
          this.charWrapper ? Z.updateFrom(this.charWrapper.getBBox()) : (Z.w = this.width, Z.h = this.height + this.valign, Z.d = -this.valign);
        }, J;
      }(q);
    }
    return mglyph$1.CommonMglyphMixin = V, mglyph$1;
  }
  var mglyph = {}, hasRequiredMglyph$1;
  function requireMglyph$1() {
    if (hasRequiredMglyph$1) return mglyph;
    hasRequiredMglyph$1 = 1;
    var O = mglyph.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mglyph.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mglyph, "__esModule", { value: !0 }), mglyph.MmlMglyph = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.texclass = H.TEXCLASS.ORD, J;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "mglyph";
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.verifyAttributes = function(J) {
        var Z = this.attributes.getList("src", "fontfamily", "index"), te = Z.src, ne = Z.fontfamily, ee = Z.index;
        te === "" && (ne === "" || ee === "") ? this.mError("mglyph must have either src or fontfamily and index attributes", J, !0) : q.prototype.verifyAttributes.call(this, J);
      }, G.defaults = _(_({}, H.AbstractMmlTokenNode.defaults), { alt: "", src: "", index: "", width: "auto", height: "auto", valign: "0em" }), G;
    }(H.AbstractMmlTokenNode);
    return mglyph.MmlMglyph = V, mglyph;
  }
  var hasRequiredMglyph;
  function requireMglyph() {
    if (hasRequiredMglyph) return mglyph$2;
    hasRequiredMglyph = 1;
    var O = mglyph$2.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(mglyph$2, "__esModule", { value: !0 }), mglyph$2.SVGmglyph = void 0;
    var _ = requireWrapper(), H = requireMglyph$2(), V = requireMglyph$1(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return J.prototype.toSVG = function(Z) {
        var te = this.standardSVGnode(Z);
        if (this.charWrapper) {
          this.charWrapper.toSVG(te);
          return;
        }
        var ne = this.node.attributes.getList("src", "alt"), ee = ne.src, ae = ne.alt, oe = this.fixed(this.height), se = this.fixed(this.width), ce = this.fixed(this.height + (this.valign || 0)), le = {
          width: se,
          height: oe,
          transform: "translate(0 " + ce + ") matrix(1 0 0 -1 0 0)",
          preserveAspectRatio: "none",
          "aria-label": ae,
          href: ee
        }, ue = this.svg("image", le);
        this.adaptor.append(te, ue);
      }, J.kind = V.MmlMglyph.prototype.kind, J;
    }((0, H.CommonMglyphMixin)(_.SVGWrapper));
    return mglyph$2.SVGmglyph = q, mglyph$2;
  }
  var TeXAtom$2 = {}, TeXAtom$1 = {}, hasRequiredTeXAtom$2;
  function requireTeXAtom$2() {
    if (hasRequiredTeXAtom$2) return TeXAtom$1;
    hasRequiredTeXAtom$2 = 1;
    var O = TeXAtom$1.__extends || /* @__PURE__ */ function() {
      var V = function(q, G) {
        return V = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(J, Z) {
          J.__proto__ = Z;
        } || function(J, Z) {
          for (var te in Z) Object.prototype.hasOwnProperty.call(Z, te) && (J[te] = Z[te]);
        }, V(q, G);
      };
      return function(q, G) {
        if (typeof G != "function" && G !== null)
          throw new TypeError("Class extends value " + String(G) + " is not a constructor or null");
        V(q, G);
        function J() {
          this.constructor = q;
        }
        q.prototype = G === null ? Object.create(G) : (J.prototype = G.prototype, new J());
      };
    }();
    Object.defineProperty(TeXAtom$1, "__esModule", { value: !0 }), TeXAtom$1.CommonTeXAtomMixin = void 0;
    var _ = requireMmlNode();
    function H(V) {
      return function(q) {
        O(G, q);
        function G() {
          return q !== null && q.apply(this, arguments) || this;
        }
        return G.prototype.computeBBox = function(J, Z) {
          if (Z === void 0 && (Z = !1), q.prototype.computeBBox.call(this, J, Z), this.childNodes[0] && this.childNodes[0].bbox.ic && (J.ic = this.childNodes[0].bbox.ic), this.node.texClass === _.TEXCLASS.VCENTER) {
            var te = J.h, ne = J.d, ee = this.font.params.axis_height, ae = (te + ne) / 2 + ee - te;
            J.h += ae, J.d -= ae;
          }
        }, G;
      }(V);
    }
    return TeXAtom$1.CommonTeXAtomMixin = H, TeXAtom$1;
  }
  var TeXAtom = {}, hasRequiredTeXAtom$1;
  function requireTeXAtom$1() {
    if (hasRequiredTeXAtom$1) return TeXAtom;
    hasRequiredTeXAtom$1 = 1;
    var O = TeXAtom.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = TeXAtom.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(TeXAtom, "__esModule", { value: !0 }), TeXAtom.TeXAtom = void 0;
    var H = requireMmlNode(), V = requireMo$2(), q = function(G) {
      O(J, G);
      function J(Z, te, ne) {
        var ee = G.call(this, Z, te, ne) || this;
        return ee.texclass = H.TEXCLASS.ORD, ee.setProperty("texClass", ee.texClass), ee;
      }
      return Object.defineProperty(J.prototype, "kind", {
        get: function() {
          return "TeXAtom";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "arity", {
        get: function() {
          return -1;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "notParent", {
        get: function() {
          return this.childNodes[0] && this.childNodes[0].childNodes.length === 1;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.setTeXclass = function(Z) {
        return this.childNodes[0].setTeXclass(null), this.adjustTeXclass(Z);
      }, J.prototype.adjustTeXclass = function(Z) {
        return Z;
      }, J.defaults = _({}, H.AbstractMmlBaseNode.defaults), J;
    }(H.AbstractMmlBaseNode);
    return TeXAtom.TeXAtom = q, q.prototype.adjustTeXclass = V.MmlMo.prototype.adjustTeXclass, TeXAtom;
  }
  var hasRequiredTeXAtom;
  function requireTeXAtom() {
    if (hasRequiredTeXAtom) return TeXAtom$2;
    hasRequiredTeXAtom = 1;
    var O = TeXAtom$2.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }();
    Object.defineProperty(TeXAtom$2, "__esModule", { value: !0 }), TeXAtom$2.SVGTeXAtom = void 0;
    var _ = requireWrapper(), H = requireTeXAtom$2(), V = requireTeXAtom$1(), q = requireMmlNode(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.toSVG = function(te) {
        if (J.prototype.toSVG.call(this, te), this.adaptor.setAttribute(this.element, "data-mjx-texclass", q.TEXCLASSNAMES[this.node.texClass]), this.node.texClass === q.TEXCLASS.VCENTER) {
          var ne = this.childNodes[0].getBBox(), ee = ne.h, ae = ne.d, oe = this.font.params.axis_height, se = (ee + ae) / 2 + oe - ee, ce = "translate(0 " + this.fixed(se) + ")";
          this.adaptor.setAttribute(this.element, "transform", ce);
        }
      }, Z.kind = V.TeXAtom.prototype.kind, Z;
    }((0, H.CommonTeXAtomMixin)(_.SVGWrapper));
    return TeXAtom$2.SVGTeXAtom = G, TeXAtom$2;
  }
  var TextNode$3 = {}, TextNode$2 = {}, hasRequiredTextNode$1;
  function requireTextNode$1() {
    if (hasRequiredTextNode$1) return TextNode$2;
    hasRequiredTextNode$1 = 1;
    var O = TextNode$2.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = TextNode$2.__values || function(q) {
      var G = typeof Symbol == "function" && Symbol.iterator, J = G && q[G], Z = 0;
      if (J) return J.call(q);
      if (q && typeof q.length == "number") return {
        next: function() {
          return q && Z >= q.length && (q = void 0), { value: q && q[Z++], done: !q };
        }
      };
      throw new TypeError(G ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = TextNode$2.__read || function(q, G) {
      var J = typeof Symbol == "function" && q[Symbol.iterator];
      if (!J) return q;
      var Z = J.call(q), te, ne = [], ee;
      try {
        for (; (G === void 0 || G-- > 0) && !(te = Z.next()).done; ) ne.push(te.value);
      } catch (ae) {
        ee = { error: ae };
      } finally {
        try {
          te && !te.done && (J = Z.return) && J.call(Z);
        } finally {
          if (ee) throw ee.error;
        }
      }
      return ne;
    };
    Object.defineProperty(TextNode$2, "__esModule", { value: !0 }), TextNode$2.CommonTextNodeMixin = void 0;
    function V(q) {
      return function(G) {
        O(J, G);
        function J() {
          return G !== null && G.apply(this, arguments) || this;
        }
        return J.prototype.computeBBox = function(Z, te) {
          var ne, ee, ae = this.parent.variant, oe = this.node.getText();
          if (ae === "-explicitFont") {
            var se = this.jax.getFontData(this.parent.styles), ce = this.jax.measureText(oe, ae, se), le = ce.w, ue = ce.h, de = ce.d;
            Z.h = ue, Z.d = de, Z.w = le;
          } else {
            var fe = this.remappedText(oe, ae);
            Z.empty();
            try {
              for (var he = _(fe), me = he.next(); !me.done; me = he.next()) {
                var pe = me.value, ye = H(this.getVariantChar(ae, pe), 4), ue = ye[0], de = ye[1], le = ye[2], ge = ye[3];
                if (ge.unknown) {
                  var ve = this.jax.measureText(String.fromCodePoint(pe), ae);
                  le = ve.w, ue = ve.h, de = ve.d;
                }
                Z.w += le, ue > Z.h && (Z.h = ue), de > Z.d && (Z.d = de), Z.ic = ge.ic || 0, Z.sk = ge.sk || 0, Z.dx = ge.dx || 0;
              }
            } catch (be) {
              ne = { error: be };
            } finally {
              try {
                me && !me.done && (ee = he.return) && ee.call(he);
              } finally {
                if (ne) throw ne.error;
              }
            }
            fe.length > 1 && (Z.sk = 0), Z.clean();
          }
        }, J.prototype.remappedText = function(Z, te) {
          var ne = this.parent.stretch.c;
          return ne ? [ne] : this.parent.remapChars(this.unicodeChars(Z, te));
        }, J.prototype.getStyles = function() {
        }, J.prototype.getVariant = function() {
        }, J.prototype.getScale = function() {
        }, J.prototype.getSpace = function() {
        }, J;
      }(q);
    }
    return TextNode$2.CommonTextNodeMixin = V, TextNode$2;
  }
  var hasRequiredTextNode;
  function requireTextNode() {
    if (hasRequiredTextNode) return TextNode$3;
    hasRequiredTextNode = 1;
    var O = TextNode$3.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = TextNode$3.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(TextNode$3, "__esModule", { value: !0 }), TextNode$3.SVGTextNode = void 0;
    var H = requireMmlNode(), V = requireWrapper(), q = requireTextNode$1(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Z.prototype.toSVG = function(te) {
        var ne, ee, ae = this.node.getText(), oe = this.parent.variant;
        if (ae.length !== 0)
          if (oe === "-explicitFont")
            this.element = this.adaptor.append(te, this.jax.unknownText(ae, oe));
          else {
            var se = this.remappedText(ae, oe);
            this.parent.childNodes.length > 1 && (te = this.element = this.adaptor.append(te, this.svg("g", { "data-mml-node": "text" })));
            var ce = 0;
            try {
              for (var le = _(se), ue = le.next(); !ue.done; ue = le.next()) {
                var de = ue.value;
                ce += this.placeChar(de, ce, 0, te, oe);
              }
            } catch (fe) {
              ne = { error: fe };
            } finally {
              try {
                ue && !ue.done && (ee = le.return) && ee.call(le);
              } finally {
                if (ne) throw ne.error;
              }
            }
          }
      }, Z.kind = H.TextNode.prototype.kind, Z.styles = {
        'mjx-container[jax="SVG"] path[data-c], mjx-container[jax="SVG"] use[data-c]': {
          "stroke-width": 3
        }
      }, Z;
    }((0, q.CommonTextNodeMixin)(V.SVGWrapper));
    return TextNode$3.SVGTextNode = G, TextNode$3;
  }
  var hasRequiredWrappers;
  function requireWrappers() {
    if (hasRequiredWrappers) return Wrappers;
    hasRequiredWrappers = 1;
    var O;
    Object.defineProperty(Wrappers, "__esModule", { value: !0 }), Wrappers.SVGWrappers = void 0;
    var _ = requireWrapper(), H = requireMath(), V = requireMrow(), q = requireMi(), G = requireMo(), J = requireMn(), Z = requireMs(), te = requireMtext(), ne = requireMerror(), ee = requireMspace(), ae = requireMpadded(), oe = requireMphantom(), se = requireMfrac(), ce = requireMsqrt(), le = requireMroot(), ue = requireMfenced(), de = requireMsubsup(), fe = requireMunderover(), he = requireMmultiscripts(), me = requireMtable(), pe = requireMtr(), ye = requireMtd(), ge = requireMaction(), ve = requireMenclose(), be = requireSemantics(), Ce = requireMglyph(), Se = requireTeXAtom(), we = requireTextNode();
    return Wrappers.SVGWrappers = (O = {}, O[H.SVGmath.kind] = H.SVGmath, O[V.SVGmrow.kind] = V.SVGmrow, O[V.SVGinferredMrow.kind] = V.SVGinferredMrow, O[q.SVGmi.kind] = q.SVGmi, O[G.SVGmo.kind] = G.SVGmo, O[J.SVGmn.kind] = J.SVGmn, O[Z.SVGms.kind] = Z.SVGms, O[te.SVGmtext.kind] = te.SVGmtext, O[ne.SVGmerror.kind] = ne.SVGmerror, O[ee.SVGmspace.kind] = ee.SVGmspace, O[ae.SVGmpadded.kind] = ae.SVGmpadded, O[oe.SVGmphantom.kind] = oe.SVGmphantom, O[se.SVGmfrac.kind] = se.SVGmfrac, O[ce.SVGmsqrt.kind] = ce.SVGmsqrt, O[le.SVGmroot.kind] = le.SVGmroot, O[ue.SVGmfenced.kind] = ue.SVGmfenced, O[de.SVGmsub.kind] = de.SVGmsub, O[de.SVGmsup.kind] = de.SVGmsup, O[de.SVGmsubsup.kind] = de.SVGmsubsup, O[fe.SVGmunder.kind] = fe.SVGmunder, O[fe.SVGmover.kind] = fe.SVGmover, O[fe.SVGmunderover.kind] = fe.SVGmunderover, O[he.SVGmmultiscripts.kind] = he.SVGmmultiscripts, O[me.SVGmtable.kind] = me.SVGmtable, O[pe.SVGmtr.kind] = pe.SVGmtr, O[pe.SVGmlabeledtr.kind] = pe.SVGmlabeledtr, O[ye.SVGmtd.kind] = ye.SVGmtd, O[ge.SVGmaction.kind] = ge.SVGmaction, O[ve.SVGmenclose.kind] = ve.SVGmenclose, O[be.SVGsemantics.kind] = be.SVGsemantics, O[be.SVGannotation.kind] = be.SVGannotation, O[be.SVGannotationXML.kind] = be.SVGannotationXML, O[be.SVGxml.kind] = be.SVGxml, O[Ce.SVGmglyph.kind] = Ce.SVGmglyph, O[Se.SVGTeXAtom.kind] = Se.SVGTeXAtom, O[we.SVGTextNode.kind] = we.SVGTextNode, O[_.SVGWrapper.kind] = _.SVGWrapper, O), Wrappers;
  }
  var hasRequiredWrapperFactory;
  function requireWrapperFactory() {
    if (hasRequiredWrapperFactory) return WrapperFactory$2;
    hasRequiredWrapperFactory = 1;
    var O = WrapperFactory$2.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }();
    Object.defineProperty(WrapperFactory$2, "__esModule", { value: !0 }), WrapperFactory$2.SVGWrapperFactory = void 0;
    var _ = requireWrapperFactory$1(), H = requireWrappers(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.jax = null, J;
      }
      return G.defaultNodes = H.SVGWrappers, G;
    }(_.CommonWrapperFactory);
    return WrapperFactory$2.SVGWrapperFactory = V, WrapperFactory$2;
  }
  var tex$1 = {}, FontData = {}, hasRequiredFontData;
  function requireFontData() {
    return hasRequiredFontData || (hasRequiredFontData = 1, function(O) {
      var _ = FontData.__extends || /* @__PURE__ */ function() {
        var ne = function(ee, ae) {
          return ne = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(oe, se) {
            oe.__proto__ = se;
          } || function(oe, se) {
            for (var ce in se) Object.prototype.hasOwnProperty.call(se, ce) && (oe[ce] = se[ce]);
          }, ne(ee, ae);
        };
        return function(ee, ae) {
          if (typeof ae != "function" && ae !== null)
            throw new TypeError("Class extends value " + String(ae) + " is not a constructor or null");
          ne(ee, ae);
          function oe() {
            this.constructor = ee;
          }
          ee.prototype = ae === null ? Object.create(ae) : (oe.prototype = ae.prototype, new oe());
        };
      }(), H = FontData.__assign || function() {
        return H = Object.assign || function(ne) {
          for (var ee, ae = 1, oe = arguments.length; ae < oe; ae++) {
            ee = arguments[ae];
            for (var se in ee) Object.prototype.hasOwnProperty.call(ee, se) && (ne[se] = ee[se]);
          }
          return ne;
        }, H.apply(this, arguments);
      }, V = FontData.__createBinding || (Object.create ? function(ne, ee, ae, oe) {
        oe === void 0 && (oe = ae);
        var se = Object.getOwnPropertyDescriptor(ee, ae);
        (!se || ("get" in se ? !ee.__esModule : se.writable || se.configurable)) && (se = { enumerable: !0, get: function() {
          return ee[ae];
        } }), Object.defineProperty(ne, oe, se);
      } : function(ne, ee, ae, oe) {
        oe === void 0 && (oe = ae), ne[oe] = ee[ae];
      }), q = FontData.__exportStar || function(ne, ee) {
        for (var ae in ne) ae !== "default" && !Object.prototype.hasOwnProperty.call(ee, ae) && V(ee, ne, ae);
      }, G = FontData.__values || function(ne) {
        var ee = typeof Symbol == "function" && Symbol.iterator, ae = ee && ne[ee], oe = 0;
        if (ae) return ae.call(ne);
        if (ne && typeof ne.length == "number") return {
          next: function() {
            return ne && oe >= ne.length && (ne = void 0), { value: ne && ne[oe++], done: !ne };
          }
        };
        throw new TypeError(ee ? "Object is not iterable." : "Symbol.iterator is not defined.");
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.AddPaths = O.SVGFontData = void 0;
      var J = requireFontData$1();
      q(requireFontData$1(), O);
      var Z = function(ne) {
        _(ee, ne);
        function ee() {
          return ne !== null && ne.apply(this, arguments) || this;
        }
        return ee.charOptions = function(ae, oe) {
          return ne.charOptions.call(this, ae, oe);
        }, ee.OPTIONS = H(H({}, J.FontData.OPTIONS), { dynamicPrefix: "./output/svg/fonts" }), ee.JAX = "SVG", ee;
      }(J.FontData);
      O.SVGFontData = Z;
      function te(ne, ee, ae) {
        var oe, se, ce, le;
        try {
          for (var ue = G(Object.keys(ee)), de = ue.next(); !de.done; de = ue.next()) {
            var fe = de.value, he = parseInt(fe);
            Z.charOptions(ne, he).p = ee[he];
          }
        } catch (ye) {
          oe = { error: ye };
        } finally {
          try {
            de && !de.done && (se = ue.return) && se.call(ue);
          } finally {
            if (oe) throw oe.error;
          }
        }
        try {
          for (var me = G(Object.keys(ae)), pe = me.next(); !pe.done; pe = me.next()) {
            var fe = pe.value, he = parseInt(fe);
            Z.charOptions(ne, he).c = ae[he];
          }
        } catch (ye) {
          ce = { error: ye };
        } finally {
          try {
            pe && !pe.done && (le = me.return) && le.call(me);
          } finally {
            if (ce) throw ce.error;
          }
        }
        return ne;
      }
      O.AddPaths = te;
    }(FontData)), FontData;
  }
  var tex = {}, hasRequiredTex$1;
  function requireTex$1() {
    if (hasRequiredTex$1) return tex;
    hasRequiredTex$1 = 1;
    var O = tex.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = tex.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    }, H = tex.__read || function(G, J) {
      var Z = typeof Symbol == "function" && G[Symbol.iterator];
      if (!Z) return G;
      var te = Z.call(G), ne, ee = [], ae;
      try {
        for (; (J === void 0 || J-- > 0) && !(ne = te.next()).done; ) ee.push(ne.value);
      } catch (oe) {
        ae = { error: oe };
      } finally {
        try {
          ne && !ne.done && (Z = te.return) && Z.call(te);
        } finally {
          if (ae) throw ae.error;
        }
      }
      return ee;
    }, V = tex.__spreadArray || function(G, J, Z) {
      if (Z || arguments.length === 2) for (var te = 0, ne = J.length, ee; te < ne; te++)
        (ee || !(te in J)) && (ee || (ee = Array.prototype.slice.call(J, 0, te)), ee[te] = J[te]);
      return G.concat(ee || Array.prototype.slice.call(J));
    };
    Object.defineProperty(tex, "__esModule", { value: !0 }), tex.CommonTeXFontMixin = void 0;
    function q(G) {
      var J;
      return J = function(Z) {
        O(te, Z);
        function te() {
          return Z !== null && Z.apply(this, arguments) || this;
        }
        return te.prototype.getDelimiterData = function(ne) {
          return this.getChar("-smallop", ne) || this.getChar("-size4", ne);
        }, te;
      }(G), J.NAME = "TeX", J.defaultVariants = V(V([], H(G.defaultVariants), !1), [
        ["-smallop", "normal"],
        ["-largeop", "normal"],
        ["-size3", "normal"],
        ["-size4", "normal"],
        ["-tex-calligraphic", "italic"],
        ["-tex-bold-calligraphic", "bold-italic"],
        ["-tex-oldstyle", "normal"],
        ["-tex-bold-oldstyle", "bold"],
        ["-tex-mathit", "italic"],
        ["-tex-variant", "normal"]
      ], !1), J.defaultCssFonts = _(_({}, G.defaultCssFonts), { "-smallop": ["serif", !1, !1], "-largeop": ["serif", !1, !1], "-size3": ["serif", !1, !1], "-size4": ["serif", !1, !1], "-tex-calligraphic": ["cursive", !0, !1], "-tex-bold-calligraphic": ["cursive", !0, !0], "-tex-oldstyle": ["serif", !1, !1], "-tex-bold-oldstyle": ["serif", !1, !0], "-tex-mathit": ["serif", !0, !1] }), J.defaultSizeVariants = ["normal", "-smallop", "-largeop", "-size3", "-size4", "-tex-variant"], J.defaultStretchVariants = ["-size4"], J;
    }
    return tex.CommonTeXFontMixin = q, tex;
  }
  var boldItalic$1 = {}, boldItalic = {}, hasRequiredBoldItalic$1;
  function requireBoldItalic$1() {
    return hasRequiredBoldItalic$1 || (hasRequiredBoldItalic$1 = 1, Object.defineProperty(boldItalic, "__esModule", { value: !0 }), boldItalic.boldItalic = void 0, boldItalic.boldItalic = {
      47: [0.711, 0.21, 0.894],
      305: [0.452, 8e-3, 0.394, { sk: 0.0319 }],
      567: [0.451, 0.201, 0.439, { sk: 0.0958 }],
      8260: [0.711, 0.21, 0.894],
      8710: [0.711, 0, 0.958, { sk: 0.192 }],
      10744: [0.711, 0.21, 0.894]
    }), boldItalic;
  }
  var hasRequiredBoldItalic;
  function requireBoldItalic() {
    if (hasRequiredBoldItalic) return boldItalic$1;
    hasRequiredBoldItalic = 1, Object.defineProperty(boldItalic$1, "__esModule", { value: !0 }), boldItalic$1.boldItalic = void 0;
    var O = requireFontData(), _ = requireBoldItalic$1();
    return boldItalic$1.boldItalic = (0, O.AddPaths)(_.boldItalic, {
      47: "189 -210Q179 -210 170 -203T160 -179Q160 -171 162 -166Q164 -163 420 266T679 698Q686 711 704 711Q714 711 723 704T733 681Q733 672 730 667Q723 654 469 228T211 -201Q202 -210 189 -210",
      305: "24 296Q24 305 34 328T63 380T115 430T187 452Q205 452 223 448T262 435T295 406T308 360Q308 345 287 290T240 170T207 87Q202 67 202 57Q202 42 215 42Q235 42 257 64Q288 92 302 140Q307 156 310 159T330 162H336H347Q367 162 367 148Q367 140 357 117T329 65T276 14T201 -8Q158 -8 121 15T83 84Q83 104 133 229T184 358Q189 376 189 388Q189 402 177 402Q156 402 134 380Q103 352 89 304Q84 288 81 285T61 282H55H44Q24 282 24 296",
      567: "297 360T297 373T294 392T288 400T278 401H276Q237 398 200 363Q181 343 170 325T156 299T149 287T129 282H123H116Q102 282 97 284T92 298Q93 303 98 315T118 349T151 390T201 427T267 451H279Q357 451 388 422T420 354V339L370 138Q321 -60 317 -69Q287 -157 163 -194Q133 -201 99 -201Q39 -201 14 -178T-12 -125Q-12 -94 11 -69T68 -43Q93 -43 108 -57T123 -95Q123 -121 100 -151H104Q131 -151 155 -125T193 -60Q195 -54 244 141T294 345Q297 360 297 373",
      8260: "189 -210Q179 -210 170 -203T160 -179Q160 -171 162 -166Q164 -163 420 266T679 698Q686 711 704 711Q714 711 723 704T733 681Q733 672 730 667Q723 654 469 228T211 -201Q202 -210 189 -210",
      8710: "65 0Q59 6 59 9T61 16Q64 20 334 357T608 698Q616 706 629 710Q630 710 634 710T644 710T656 711Q686 711 694 703Q698 699 700 693Q706 674 805 345T904 14Q904 7 894 1L479 0H65ZM630 342L567 551L232 134L462 133H693Q693 137 630 342",
      10744: "189 -210Q179 -210 170 -203T160 -179Q160 -171 162 -166Q164 -163 420 266T679 698Q686 711 704 711Q714 711 723 704T733 681Q733 672 730 667Q723 654 469 228T211 -201Q202 -210 189 -210"
    }, {}), boldItalic$1;
  }
  var bold$1 = {}, bold = {}, hasRequiredBold$1;
  function requireBold$1() {
    return hasRequiredBold$1 || (hasRequiredBold$1 = 1, Object.defineProperty(bold, "__esModule", { value: !0 }), bold.bold = void 0, bold.bold = {
      33: [0.705, 0, 0.35],
      34: [0.694, -0.329, 0.603],
      35: [0.694, 0.193, 0.958],
      36: [0.75, 0.056, 0.575],
      37: [0.75, 0.056, 0.958],
      38: [0.705, 0.011, 0.894],
      39: [0.694, -0.329, 0.319],
      40: [0.75, 0.249, 0.447],
      41: [0.75, 0.249, 0.447],
      42: [0.75, -0.306, 0.575],
      43: [0.633, 0.131, 0.894],
      44: [0.171, 0.194, 0.319],
      45: [0.278, -0.166, 0.383],
      46: [0.171, 0, 0.319],
      47: [0.75, 0.25, 0.575],
      58: [0.444, 0, 0.319],
      59: [0.444, 0.194, 0.319],
      60: [0.587, 0.085, 0.894],
      61: [0.393, -0.109, 0.894],
      62: [0.587, 0.085, 0.894],
      63: [0.7, 0, 0.543],
      64: [0.699, 6e-3, 0.894],
      91: [0.75, 0.25, 0.319],
      92: [0.75, 0.25, 0.575],
      93: [0.75, 0.25, 0.319],
      94: [0.694, -0.52, 0.575],
      95: [-0.01, 0.061, 0.575],
      96: [0.706, -0.503, 0.575],
      123: [0.75, 0.25, 0.575],
      124: [0.75, 0.249, 0.319],
      125: [0.75, 0.25, 0.575],
      126: [0.344, -0.202, 0.575],
      168: [0.695, -0.535, 0.575],
      172: [0.371, -0.061, 0.767],
      175: [0.607, -0.54, 0.575],
      176: [0.702, -0.536, 0.575],
      177: [0.728, 0.035, 0.894],
      180: [0.706, -0.503, 0.575],
      183: [0.336, -0.166, 0.319],
      215: [0.53, 0.028, 0.894],
      247: [0.597, 0.096, 0.894],
      305: [0.442, 0, 0.278, { sk: 0.0278 }],
      567: [0.442, 0.205, 0.306, { sk: 0.0833 }],
      697: [0.563, -0.033, 0.344],
      710: [0.694, -0.52, 0.575],
      711: [0.66, -0.515, 0.575],
      713: [0.607, -0.54, 0.575],
      714: [0.706, -0.503, 0.575],
      715: [0.706, -0.503, 0.575],
      728: [0.694, -0.5, 0.575],
      729: [0.695, -0.525, 0.575],
      730: [0.702, -0.536, 0.575],
      732: [0.694, -0.552, 0.575],
      768: [0.706, -0.503, 0],
      769: [0.706, -0.503, 0],
      770: [0.694, -0.52, 0],
      771: [0.694, -0.552, 0],
      772: [0.607, -0.54, 0],
      774: [0.694, -0.5, 0],
      775: [0.695, -0.525, 0],
      776: [0.695, -0.535, 0],
      778: [0.702, -0.536, 0],
      779: [0.714, -0.511, 0],
      780: [0.66, -0.515, 0],
      824: [0.711, 0.21, 0],
      8194: [0, 0, 0.5],
      8195: [0, 0, 0.999],
      8196: [0, 0, 0.333],
      8197: [0, 0, 0.25],
      8198: [0, 0, 0.167],
      8201: [0, 0, 0.167],
      8202: [0, 0, 0.083],
      8211: [0.3, -0.249, 0.575],
      8212: [0.3, -0.249, 1.15],
      8213: [0.3, -0.249, 1.15],
      8214: [0.75, 0.248, 0.575],
      8215: [-0.01, 0.061, 0.575],
      8216: [0.694, -0.329, 0.319],
      8217: [0.694, -0.329, 0.319],
      8220: [0.694, -0.329, 0.603],
      8221: [0.694, -0.329, 0.603],
      8224: [0.702, 0.211, 0.511],
      8225: [0.702, 0.202, 0.511],
      8226: [0.474, -0.028, 0.575],
      8230: [0.171, 0, 1.295],
      8242: [0.563, -0.033, 0.344],
      8243: [0.563, 0, 0.688],
      8244: [0.563, 0, 1.032],
      8254: [0.607, -0.54, 0.575],
      8260: [0.75, 0.25, 0.575],
      8279: [0.563, 0, 1.376],
      8407: [0.723, -0.513, 0.575],
      8463: [0.694, 8e-3, 0.668, { sk: -0.0319 }],
      8467: [0.702, 0.019, 0.474, { sk: 0.128 }],
      8472: [0.461, 0.21, 0.74],
      8501: [0.694, 0, 0.703],
      8592: [0.518, 0.017, 1.15],
      8593: [0.694, 0.193, 0.575],
      8594: [0.518, 0.017, 1.15],
      8595: [0.694, 0.194, 0.575],
      8596: [0.518, 0.017, 1.15],
      8597: [0.767, 0.267, 0.575],
      8598: [0.724, 0.194, 1.15],
      8599: [0.724, 0.193, 1.15],
      8600: [0.694, 0.224, 1.15],
      8601: [0.694, 0.224, 1.15],
      8602: [0.711, 0.21, 1.15],
      8603: [0.711, 0.21, 1.15],
      8614: [0.518, 0.017, 1.15],
      8617: [0.518, 0.017, 1.282],
      8618: [0.518, 0.017, 1.282],
      8622: [0.711, 0.21, 1.15],
      8636: [0.518, -0.22, 1.15],
      8637: [0.281, 0.017, 1.15],
      8640: [0.518, -0.22, 1.15],
      8641: [0.281, 0.017, 1.15],
      8652: [0.718, 0.017, 1.15],
      8653: [0.711, 0.21, 1.15],
      8654: [0.711, 0.21, 1.15],
      8655: [0.711, 0.21, 1.15],
      8656: [0.547, 0.046, 1.15],
      8657: [0.694, 0.193, 0.703],
      8658: [0.547, 0.046, 1.15],
      8659: [0.694, 0.194, 0.703],
      8660: [0.547, 0.046, 1.15],
      8661: [0.767, 0.267, 0.703],
      8704: [0.694, 0.016, 0.639],
      8707: [0.694, 0, 0.639],
      8708: [0.711, 0.21, 0.639],
      8709: [0.767, 0.073, 0.575],
      8710: [0.698, 0, 0.958],
      8712: [0.587, 0.086, 0.767],
      8713: [0.711, 0.21, 0.767],
      8715: [0.587, 0.086, 0.767],
      8716: [0.711, 0.21, 0.767],
      8722: [0.281, -0.221, 0.894],
      8723: [0.537, 0.227, 0.894],
      8725: [0.75, 0.25, 0.575],
      8726: [0.75, 0.25, 0.575],
      8727: [0.472, -0.028, 0.575],
      8728: [0.474, -0.028, 0.575],
      8729: [0.474, -0.028, 0.575],
      8730: [0.82, 0.18, 0.958, { ic: 0.03 }],
      8733: [0.451, 8e-3, 0.894],
      8734: [0.452, 8e-3, 1.15],
      8736: [0.714, 0, 0.722],
      8739: [0.75, 0.249, 0.319],
      8740: [0.75, 0.249, 0.319],
      8741: [0.75, 0.248, 0.575],
      8742: [0.75, 0.248, 0.575],
      8743: [0.604, 0.017, 0.767],
      8744: [0.604, 0.016, 0.767],
      8745: [0.603, 0.016, 0.767],
      8746: [0.604, 0.016, 0.767],
      8747: [0.711, 0.211, 0.569, { ic: 0.063 }],
      8764: [0.391, -0.109, 0.894],
      8768: [0.583, 0.082, 0.319],
      8769: [0.711, 0.21, 0.894],
      8771: [0.502, 0, 0.894],
      8772: [0.711, 0.21, 0.894],
      8773: [0.638, 0.027, 0.894],
      8775: [0.711, 0.21, 0.894],
      8776: [0.524, -0.032, 0.894],
      8777: [0.711, 0.21, 0.894],
      8781: [0.533, 0.032, 0.894],
      8784: [0.721, -0.109, 0.894],
      8800: [0.711, 0.21, 0.894],
      8801: [0.505, 0, 0.894],
      8802: [0.711, 0.21, 0.894],
      8804: [0.697, 0.199, 0.894],
      8805: [0.697, 0.199, 0.894],
      8810: [0.617, 0.116, 1.15],
      8811: [0.618, 0.116, 1.15],
      8813: [0.711, 0.21, 0.894],
      8814: [0.711, 0.21, 0.894],
      8815: [0.711, 0.21, 0.894],
      8816: [0.711, 0.21, 0.894],
      8817: [0.711, 0.21, 0.894],
      8826: [0.585, 0.086, 0.894],
      8827: [0.586, 0.086, 0.894],
      8832: [0.711, 0.21, 0.894],
      8833: [0.711, 0.21, 0.894],
      8834: [0.587, 0.085, 0.894],
      8835: [0.587, 0.086, 0.894],
      8836: [0.711, 0.21, 0.894],
      8837: [0.711, 0.21, 0.894],
      8838: [0.697, 0.199, 0.894],
      8839: [0.697, 0.199, 0.894],
      8840: [0.711, 0.21, 0.894],
      8841: [0.711, 0.21, 0.894],
      8846: [0.604, 0.016, 0.767],
      8849: [0.697, 0.199, 0.894],
      8850: [0.697, 0.199, 0.894],
      8851: [0.604, 0, 0.767],
      8852: [0.604, 0, 0.767],
      8853: [0.632, 0.132, 0.894],
      8854: [0.632, 0.132, 0.894],
      8855: [0.632, 0.132, 0.894],
      8856: [0.632, 0.132, 0.894],
      8857: [0.632, 0.132, 0.894],
      8866: [0.693, 0, 0.703],
      8867: [0.693, 0, 0.703],
      8868: [0.694, 0, 0.894],
      8869: [0.693, 0, 0.894],
      8872: [0.75, 0.249, 0.974],
      8876: [0.711, 0.21, 0.703],
      8877: [0.75, 0.249, 0.974],
      8900: [0.523, 0.021, 0.575],
      8901: [0.336, -0.166, 0.319],
      8902: [0.502, 0, 0.575],
      8904: [0.54, 0.039, 1],
      8930: [0.711, 0.21, 0.894],
      8931: [0.711, 0.21, 0.894],
      8942: [0.951, 0.029, 0.319],
      8943: [0.336, -0.166, 1.295],
      8945: [0.871, -0.101, 1.323],
      8968: [0.75, 0.248, 0.511],
      8969: [0.75, 0.248, 0.511],
      8970: [0.749, 0.248, 0.511],
      8971: [0.749, 0.248, 0.511],
      8994: [0.405, -0.108, 1.15],
      8995: [0.392, -0.126, 1.15],
      9001: [0.75, 0.249, 0.447],
      9002: [0.75, 0.249, 0.447],
      9651: [0.711, 0, 1.022],
      9653: [0.711, 0, 1.022],
      9657: [0.54, 0.039, 0.575],
      9661: [0.5, 0.21, 1.022],
      9663: [0.5, 0.21, 1.022],
      9667: [0.539, 0.038, 0.575],
      9711: [0.711, 0.211, 1.15],
      9824: [0.719, 0.129, 0.894],
      9825: [0.711, 0.024, 0.894],
      9826: [0.719, 0.154, 0.894],
      9827: [0.719, 0.129, 0.894],
      9837: [0.75, 0.017, 0.447],
      9838: [0.741, 0.223, 0.447],
      9839: [0.724, 0.224, 0.447],
      10072: [0.75, 0.249, 0.319],
      10216: [0.75, 0.249, 0.447],
      10217: [0.75, 0.249, 0.447],
      10229: [0.518, 0.017, 1.805],
      10230: [0.518, 0.017, 1.833],
      10231: [0.518, 0.017, 2.126],
      10232: [0.547, 0.046, 1.868],
      10233: [0.547, 0.046, 1.87],
      10234: [0.547, 0.046, 2.126],
      10236: [0.518, 0.017, 1.833],
      10744: [0.711, 0.21, 0.894],
      10799: [0.53, 0.028, 0.894],
      10815: [0.686, 0, 0.9],
      10927: [0.696, 0.199, 0.894],
      10928: [0.697, 0.199, 0.894],
      12296: [0.75, 0.249, 0.447],
      12297: [0.75, 0.249, 0.447]
    }), bold;
  }
  var hasRequiredBold;
  function requireBold() {
    if (hasRequiredBold) return bold$1;
    hasRequiredBold = 1, Object.defineProperty(bold$1, "__esModule", { value: !0 }), bold$1.bold = void 0;
    var O = requireFontData(), _ = requireBold$1();
    return bold$1.bold = (0, O.AddPaths)(_.bold, {
      33: "89 629Q89 663 116 684T171 705Q215 705 237 681T260 634Q260 619 233 434T204 244Q201 237 175 237Q150 237 146 244Q144 248 117 433T89 629ZM90 86Q90 125 116 148T177 171Q211 169 235 146T259 86Q259 48 235 25T175 1Q138 1 114 24T90 86",
      34: "38 572T38 608T61 669T121 694Q167 694 196 657T225 559Q225 520 214 482T186 418T151 370T119 339T99 329T82 340T70 360Q70 365 74 369T92 385T122 414Q142 441 154 471T170 518L172 535L166 532Q160 530 148 527T122 523Q85 523 62 547ZM305 572T305 608T328 669T388 694Q434 694 463 657T492 559Q492 520 481 482T453 418T418 370T386 339T366 329T349 340T337 360Q337 365 341 369T359 385T389 414Q409 441 421 471T436 518L439 535L433 532Q427 530 415 527T389 523Q352 523 329 547",
      35: "64 362Q64 380 87 393H363L366 404Q379 443 390 480T409 542T424 590T435 628T443 655T451 674T458 686T467 692T478 694Q490 694 499 686T509 662Q505 643 427 395Q427 393 523 393H620L623 404Q630 426 652 498T691 624T711 681Q718 694 735 694Q748 694 757 685T766 662Q762 643 684 395Q684 393 777 393H871Q872 392 875 390T881 386T887 381T891 374T893 363Q893 345 871 333L767 332H664L660 319Q660 318 638 245T614 171Q614 169 742 169H871L877 165Q883 161 885 159T890 151T893 138Q893 120 871 109L732 108H594L590 95Q587 84 546 -46Q508 -175 505 -178Q498 -193 478 -193Q462 -193 455 -183T448 -164Q448 -156 530 106Q530 108 434 108H337L333 95Q330 84 289 -46Q251 -175 248 -178Q240 -193 222 -193Q206 -193 199 -183T191 -164Q191 -154 273 106Q273 108 180 108L87 109Q64 117 64 139Q64 156 87 169H293L321 262Q326 277 331 294T340 321L343 330Q343 332 215 332L87 333Q64 343 64 362ZM600 330Q600 332 504 332H407L403 319Q403 318 381 245T357 171Q357 169 453 169H550L578 262Q583 277 588 294T597 321L600 330",
      36: "64 494Q64 541 80 579T120 638T171 674T219 693T253 698H256V750H318V699H323Q355 694 380 686T433 663T480 620T506 556Q510 539 510 520Q510 480 488 463T440 445L422 447Q407 451 398 459Q370 478 370 515Q370 542 384 559T412 580L427 584Q424 589 418 596T386 617T324 636H318V434Q411 419 460 355T510 217Q510 196 507 175T492 122T461 67T404 23T318 -4V-56H256V-5H254Q252 -3 240 -3Q194 4 160 23T108 64T80 112T67 156T64 190Q64 218 81 240T134 262Q171 262 187 240T204 193T190 150T153 125Q146 125 144 123Q142 123 149 113T174 89T218 66Q247 58 255 58Q256 58 256 173V287L239 291Q160 308 112 365T64 494ZM255 636Q246 635 236 632T206 620T173 591T160 543Q160 472 256 448V542Q256 636 255 636ZM320 59Q324 59 333 61T356 70T384 89T406 120T415 167Q415 200 395 225T356 260T318 274V59H320",
      37: "65 549Q65 609 84 652T132 714T187 742T236 750Q265 750 296 734T355 697T431 661T541 644Q662 644 736 730Q751 749 767 749T790 739T797 719Q797 710 768 672T504 329Q212 -47 211 -48Q203 -55 191 -55Q161 -55 161 -25Q161 -17 163 -12L642 609Q608 595 542 595Q515 595 488 599T441 608T405 619T381 628L373 632Q373 630 375 619T380 589T383 548Q383 455 343 401T235 347Q217 347 198 351T154 368T110 403T78 462T65 549ZM320 549Q320 633 295 665T235 698H234Q214 698 196 674Q182 650 182 549Q182 509 183 486T190 441T207 409T238 399Q269 399 294 431T320 549ZM745 -56Q727 -56 708 -52T664 -35T620 0T588 59T575 146T588 232T620 291T663 325T708 343T747 347Q810 347 851 294T893 146Q893 89 879 48T841 -15T794 -46T745 -56ZM830 146Q830 230 805 262T745 295H744Q724 295 706 271Q692 247 692 146Q692 106 693 83T700 38T717 6T748 -4Q779 -4 804 28T830 146",
      38: "255 -11Q209 -11 164 4T84 56T48 146Q48 159 50 171Q57 197 72 218T99 249T152 292Q204 333 204 334L194 356Q185 379 176 421T166 511Q166 538 168 551Q182 613 226 654T332 704Q334 704 343 704T358 705Q412 702 444 661T476 565V559Q476 489 334 371L330 368L335 357Q382 272 485 165L496 154L506 163Q543 200 597 273L671 382H601V444H610L732 441Q821 441 830 444H836V382H741L709 335Q702 324 687 302T665 270T646 244T625 216T605 191T581 162T553 132L537 116Q544 109 557 98T605 69T673 51Q711 51 739 70T767 115V118H829V114Q829 70 786 30T668 -11Q570 -11 474 37L451 49L441 43Q352 -11 255 -11ZM415 564Q415 596 400 625T356 654Q329 654 310 634T285 588Q283 580 283 554Q283 475 309 417L325 431Q415 512 415 564ZM192 182Q192 126 213 89T279 51Q348 51 400 83L389 91Q362 112 338 137T295 186T264 229T240 265T227 286Q226 285 222 280T217 272T211 263T205 251T200 238T196 222T193 204T192 182",
      39: "74 572T74 608T97 669T157 694Q203 694 232 657T261 559Q261 520 250 482T222 418T187 370T155 339T135 329Q128 329 117 340T106 359Q106 365 117 375T144 399T176 440T203 505Q204 511 205 518T208 530V535L202 532Q196 530 184 527T158 523Q121 523 98 547",
      40: "103 166T103 251T121 412T165 541T225 639T287 708T341 750H356H361Q382 750 382 736Q382 732 365 714T323 661T274 576T232 439T214 250Q214 -62 381 -229Q382 -231 382 -234Q382 -249 360 -249H356H341Q314 -231 287 -207T226 -138T165 -41T121 89",
      41: "231 251Q231 354 214 439T173 575T123 661T81 714T64 735Q64 744 73 749H75Q77 749 79 749T84 750T90 750H105Q132 732 159 708T220 639T281 542T325 413T343 251T325 89T281 -40T221 -138T159 -207T105 -249H90Q80 -249 76 -249T68 -245T64 -234Q64 -230 81 -212T123 -160T172 -75T214 61T231 251",
      42: "235 706Q235 724 251 737T287 750Q306 750 322 738T339 706Q339 685 318 580V579Q429 663 436 666Q441 668 449 668Q471 668 486 650T501 612Q501 582 478 572Q476 570 414 549L354 528L414 507Q420 505 430 502T445 497T458 492T470 488T479 483T487 478T493 471T497 463T500 454T501 443Q501 423 486 406T449 388H446Q435 388 370 437Q339 461 318 477V476Q339 371 339 350Q339 332 323 319T287 306T251 319T235 350Q235 371 256 476V477Q145 393 138 390Q133 388 125 388Q103 388 88 406T73 444Q73 474 96 484Q98 486 160 507L220 528L160 549Q154 551 144 554T129 559T116 564T104 568T95 573T87 578T81 585T77 593T74 602T73 613Q73 633 88 650T125 668H128Q139 668 204 619Q235 595 256 579V580Q235 685 235 706",
      43: "64 232T64 250T87 281H416V444Q416 608 418 612Q426 633 446 633T475 613Q477 608 477 444V281H807Q808 280 811 278T817 274T823 269T827 262T829 251Q829 230 807 221L642 220H477V57Q477 -107 475 -112Q468 -131 446 -131Q425 -131 418 -112Q416 -107 416 57V220H251L87 221Q64 232 64 250",
      44: "74 85Q74 120 97 145T159 171Q200 171 226 138Q258 101 258 37Q258 -5 246 -44T218 -109T183 -155T152 -184T135 -194Q129 -194 118 -183T106 -164Q106 -157 115 -149Q121 -145 130 -137T161 -100T195 -35Q197 -28 200 -17T204 3T205 11T199 9T183 3T159 0Q120 0 97 26T74 85",
      45: "13 166V278H318V166H13",
      46: "74 85Q74 121 99 146T156 171Q200 171 222 143T245 85Q245 56 224 29T160 1Q118 1 96 27T74 85",
      47: "451 730Q460 750 479 750Q492 750 501 740T510 718Q508 708 318 244L122 -232Q112 -250 95 -250Q82 -250 73 -241T64 -218Q66 -205 258 261T451 730",
      58: "74 359Q74 394 98 419T158 444Q200 444 222 417T245 358Q245 329 224 302T160 274Q116 274 95 301T74 359ZM74 85Q74 121 99 146T156 171Q200 171 222 143T245 85Q245 56 224 29T160 1Q118 1 96 27T74 85",
      59: "74 359Q74 394 98 419T158 444Q200 444 222 417T245 358Q245 329 224 302T160 274Q116 274 95 301T74 359ZM74 50T74 86T97 146T158 171Q204 171 226 132T248 38Q248 -23 223 -80T171 -165T135 -194Q129 -194 118 -183T106 -164Q106 -163 106 -160L107 -158Q108 -155 121 -142T150 -107T177 -58Q189 -32 194 3Q195 6 193 6Q172 0 158 0Q121 0 98 25",
      60: "797 -56Q797 -68 790 -76T767 -85H759L434 70Q108 226 105 229Q96 238 96 250Q96 263 105 272Q109 276 271 354T595 508T757 585Q763 587 766 587Q780 587 788 578T797 556Q797 544 788 535Q784 531 490 391L197 251Q213 242 359 173T644 37T788 -34Q797 -43 797 -56",
      61: "87 333Q64 343 64 362Q64 383 84 391Q89 393 448 393H807Q808 392 811 390T817 386T823 381T827 374T829 363Q829 345 807 333H87ZM87 109Q64 118 64 139Q64 159 86 168Q89 169 448 169H807L812 166Q816 163 818 162T823 157T827 149T829 139Q829 118 807 109H87",
      62: "127 -85Q110 -85 103 -75T96 -55Q96 -41 106 -34Q119 -24 308 65Q361 90 411 114L696 250L427 379Q106 533 103 537Q96 545 96 557Q96 568 104 577T128 587Q137 586 460 431T788 272Q797 263 797 250Q797 238 788 229Q785 226 459 70L135 -85H127",
      63: "65 570Q65 628 119 664T259 700Q326 700 372 688T440 654T469 613T478 569Q478 505 412 465Q287 391 287 294V283Q287 250 284 244T263 237H256H249Q232 237 229 242T225 272V287Q227 364 253 418Q274 463 311 504Q335 530 335 575Q335 622 323 635T259 648Q231 648 209 644T179 636T170 630L172 628Q174 627 177 625T183 620T190 611T197 601T202 587T204 570Q204 539 185 519T134 499Q105 499 85 517T65 570ZM171 86Q171 125 197 148T258 171Q292 169 316 146T340 86Q340 48 316 25T256 1Q218 1 195 24T171 86",
      64: "64 347Q64 511 171 605T434 699Q487 699 500 698Q624 684 703 621T811 464Q828 414 828 344Q828 232 788 179T691 125Q673 125 657 127T628 132T606 140T588 148T576 156T568 162L566 164Q565 164 549 154T504 135T444 125Q349 125 284 183T218 347Q218 455 284 512T448 569Q554 569 610 479H638Q670 479 674 471Q676 468 676 340V258Q676 213 679 199T694 178Q701 174 713 177Q767 187 767 340Q767 489 678 569T446 649Q299 649 213 566T126 346Q126 307 134 269T166 189T225 116T320 65T455 45H463Q606 51 721 91L746 99H782H801Q829 99 829 85Q829 78 825 75T804 65Q800 63 797 62Q625 -6 451 -6Q271 -6 168 91T64 347ZM547 468Q526 493 504 505T444 517T377 476T346 347Q346 306 354 271T386 206T448 177Q505 177 547 226V468",
      91: "128 -250V750H293V689H189V-189H293V-250H128",
      92: "64 718Q63 731 72 740T94 750Q106 750 113 743Q118 741 122 732L318 256Q508 -208 510 -218Q511 -231 502 -240T480 -250Q460 -250 451 -230Q451 -229 259 238T64 718",
      93: "25 689V750H190V-250H25V-189H129V689H25",
      94: "207 632L287 694Q289 693 368 632T448 570T431 545T413 520Q410 520 350 559L287 597L224 559Q164 520 161 520Q160 520 143 544T126 570T207 632",
      95: "0 -61V-10H574V-61H0",
      96: "114 634Q114 663 136 684T183 706Q191 706 196 705T208 700T219 693T232 681T245 666T262 645T282 620Q332 558 337 553Q338 552 318 527L299 503L223 543Q215 547 202 553T183 563T167 571T153 580T141 587T131 595T124 603T118 612T115 622T114 634",
      123: "504 -207T504 -225T500 -246T476 -250H469Q257 -250 227 -145L225 -135L224 0Q224 15 224 30T224 59T224 84T224 106T223 122T223 133V137Q222 138 221 144T213 162T195 185Q171 206 141 215Q123 222 107 223T84 225T74 229T70 250T73 270T83 276T106 276T141 285Q171 294 195 315Q201 321 206 328T214 341T219 352T222 360L223 363V367Q223 371 223 378T223 394T224 415T224 441T224 470T224 501L225 636Q249 739 426 749Q428 749 443 749T466 750H473Q495 750 499 747T504 725T501 704T480 699Q381 693 357 645Q352 634 351 617T350 497V412Q350 350 338 329Q325 303 298 284T251 258T227 251Q226 251 226 250L227 249Q231 248 238 246T265 236T299 217T329 184T349 137Q350 131 350 3T352 -130Q358 -160 392 -178T480 -199Q497 -200 500 -203",
      124: "160 -249Q138 -249 129 -225V250Q129 725 131 729Q139 750 159 750T190 725V-225Q181 -249 160 -249",
      125: "70 726Q71 744 74 747T99 750H106Q323 750 349 636L350 501Q350 486 350 470T350 441T350 416T350 394T351 378T351 367V363Q352 362 353 356T361 338T379 315Q403 294 433 285Q451 278 467 277T490 275T500 271T504 250T501 230T491 224T468 224T433 215Q403 206 379 185Q373 179 368 172T360 159T355 148T352 140L351 137V133Q351 129 351 122T351 106T350 85T350 59T350 31T350 0L349 -135L347 -145Q317 -250 106 -250H99Q79 -250 75 -247T70 -226Q70 -208 73 -204T95 -199Q193 -193 217 -145Q222 -134 223 -117T224 3Q224 20 224 48T223 86Q223 145 237 175T301 232Q335 249 347 249Q348 249 348 250L347 251Q343 252 336 254T309 264T275 284T245 316T225 363Q224 369 224 497T222 631Q216 660 182 678T95 699Q77 700 74 704T70 726",
      126: "343 202Q320 202 278 225T215 249Q181 249 146 214L134 202L115 219Q111 222 106 226T98 234L96 236Q158 306 165 313Q199 344 230 344Q239 344 244 343Q262 339 300 318T359 297Q393 297 428 332L440 344L459 327Q463 324 468 320T476 312L478 310Q416 240 409 233Q375 202 343 202",
      168: "96 615Q96 650 120 672T178 695Q214 693 234 669T255 615Q255 583 232 559T176 535Q147 535 122 556T96 615ZM319 615Q319 651 343 673T399 695Q426 695 452 675T478 615Q478 578 454 557T395 535Q364 537 342 559T319 615",
      172: "680 371Q683 369 688 366T695 361T698 356T701 346T701 332T702 308V216Q702 196 702 168T703 130Q703 90 697 76T671 61Q650 61 643 81Q641 86 641 198V310H364L87 311Q64 319 64 341Q64 362 84 369Q89 371 385 371H680",
      175: "80 540V607H494V540H80",
      176: "160 618Q160 653 193 677T279 702H284Q381 702 407 647Q414 634 414 618Q414 607 410 596T395 570T355 546T287 536T220 545T181 568T165 594T160 618ZM352 618Q352 645 341 652T301 659H292Q286 659 278 659T268 660Q247 660 236 653T224 638T222 619Q222 591 234 585T287 578Q315 578 326 580T345 590T352 618",
      177: "64 328T64 346T87 377H416V542L417 707Q431 728 443 728Q467 728 475 709Q477 704 477 540V377H807Q808 376 811 374T817 370T823 365T827 358T829 347Q829 326 807 317L642 316H477V25H807Q808 24 811 22T817 18T823 13T827 6T829 -5Q829 -26 807 -35H87Q64 -24 64 -6T87 25H416V316H251L87 317Q64 328 64 346",
      180: "391 706Q419 706 439 683T460 634Q460 608 441 593T366 550Q356 545 351 543L275 503L256 527Q236 552 237 553Q242 558 292 620Q299 629 309 641T324 659T336 673T346 685T354 693T363 699T371 703T380 705T391 706",
      183: "74 251Q74 286 99 311T156 336Q200 336 222 308T245 250Q245 221 224 194T160 166T96 193T74 251",
      215: "168 500Q168 515 178 522T195 530H198Q207 530 218 521T282 458Q312 428 331 409L447 294L563 409Q674 520 682 525Q687 529 695 529Q711 529 718 520T726 499V498Q726 489 720 481T666 427Q631 392 606 367L490 251L606 135Q717 23 721 17T726 2Q726 -9 719 -18T695 -28H692Q685 -28 674 -18T608 47Q581 74 563 92L447 207L331 91Q217 -22 208 -27Q206 -28 203 -28H197Q168 -28 168 2Q168 13 178 24T288 135L404 250L288 366Q177 479 173 485T168 500",
      247: "344 495Q344 535 372 566T447 597Q490 597 519 566T548 495Q548 452 518 423T446 393Q404 393 374 423T344 495ZM87 221Q64 230 64 251T84 279Q89 281 448 281H806Q807 280 810 278T816 274T822 269T826 262T828 251Q828 230 806 221H87ZM344 -36T344 6T373 78T446 108Q487 108 517 79T548 6Q548 -35 519 -65T446 -96Q406 -96 375 -66",
      305: "247 0Q232 3 143 3Q132 3 106 3T56 1L34 0H26V46H42Q70 46 91 49Q100 53 102 60T104 102V205V293Q104 345 102 359T88 378Q74 385 41 385H30V408Q30 431 32 431L42 432Q52 433 70 434T106 436Q123 437 142 438T171 441T182 442H185V62Q190 52 197 50T232 46H255V0H247",
      567: "28 -163Q58 -168 64 -168Q124 -168 135 -77Q137 -65 137 141T136 353Q132 371 120 377T72 385H52V408Q52 431 54 431L58 432Q62 432 70 432T87 433T108 434T133 436Q151 437 171 438T202 441T214 442H218V184Q217 -36 217 -59T211 -98Q195 -145 153 -175T58 -205Q9 -205 -23 -179T-55 -117Q-55 -94 -40 -79T-2 -64T36 -79T52 -118Q52 -143 28 -163",
      697: "240 563Q278 563 304 539T331 480V473Q331 462 316 431T217 236Q199 200 174 151T136 78T123 50Q113 33 105 33Q101 33 72 45T38 60Q35 63 35 65Q35 77 101 293T171 517Q182 542 202 552T240 563",
      710: "207 632L287 694Q289 693 368 632T448 570T431 545T413 520Q410 520 350 559L287 597L224 559Q164 520 161 520Q160 520 143 544T126 570T207 632",
      711: "131 603Q130 604 136 618T150 646T158 659L223 635L287 611L351 635L416 659Q417 660 424 647T437 619T443 603Q440 601 364 558T287 515T210 558T131 603",
      713: "80 540V607H494V540H80",
      714: "391 706Q419 706 439 683T460 634Q460 608 441 593T366 550Q356 545 351 543L275 503L256 527Q236 552 237 553Q242 558 292 620Q299 629 309 641T324 659T336 673T346 685T354 693T363 699T371 703T380 705T391 706",
      715: "114 634Q114 663 136 684T183 706Q191 706 196 705T208 700T219 693T232 681T245 666T262 645T282 620Q332 558 337 553Q338 552 318 527L299 503L223 543Q215 547 202 553T183 563T167 571T153 580T141 587T131 595T124 603T118 612T115 622T114 634",
      728: "287 500Q208 500 155 558T102 689V694H153V685Q153 681 154 674T164 648T186 615T226 590T287 578Q347 578 382 611T421 685V694H472V689Q472 623 422 562T287 500",
      729: "202 610Q202 647 227 671T283 695Q324 695 348 669T372 610T350 551T287 525Q248 525 225 551T202 610",
      730: "160 618Q160 653 193 677T279 702H284Q381 702 407 647Q414 634 414 618Q414 607 410 596T395 570T355 546T287 536T220 545T181 568T165 594T160 618ZM352 618Q352 645 341 652T301 659H292Q286 659 278 659T268 660Q247 660 236 653T224 638T222 619Q222 591 234 585T287 578Q315 578 326 580T345 590T352 618",
      732: "343 552Q320 552 278 575T215 599Q181 599 146 564L134 552L115 569Q111 572 106 576T98 584L96 586Q158 656 165 663Q199 694 230 694Q239 694 244 693Q262 689 300 668T359 647Q393 647 428 682L440 694L459 677Q463 674 468 670T476 662L478 660Q416 590 409 583Q375 552 343 552",
      768: "-461 634Q-461 663 -439 684T-392 706Q-384 706 -379 705T-367 700T-356 693T-343 681T-330 666T-313 645T-293 620Q-243 558 -238 553Q-237 552 -257 527L-276 503L-352 543Q-360 547 -373 553T-392 563T-408 571T-422 580T-434 587T-444 595T-451 603T-457 612T-460 622T-461 634",
      769: "-184 706Q-156 706 -136 683T-115 634Q-115 608 -134 593T-209 550Q-219 545 -224 543L-300 503L-319 527Q-339 552 -338 553Q-333 558 -283 620Q-276 629 -266 641T-251 659T-239 673T-229 685T-221 693T-212 699T-204 703T-195 705T-184 706",
      770: "-368 632L-288 694Q-286 693 -207 632T-127 570T-144 545T-162 520Q-165 520 -225 559L-288 597L-351 559Q-411 520 -414 520Q-415 520 -432 544T-449 570T-368 632",
      771: "-232 552Q-255 552 -297 575T-360 599Q-394 599 -429 564L-441 552L-460 569Q-464 572 -469 576T-476 584L-479 586Q-417 656 -410 663Q-376 694 -345 694Q-336 694 -331 693Q-313 689 -275 668T-216 647Q-182 647 -147 682L-135 694L-116 677Q-112 674 -107 670T-100 662L-97 660Q-159 590 -166 583Q-200 552 -232 552",
      772: "-495 540V607H-81V540H-495",
      774: "-288 500Q-367 500 -420 558T-473 689V694H-422V685Q-422 681 -421 674T-411 648T-389 615T-349 590T-288 578Q-228 578 -193 611T-154 685V694H-103V689Q-103 623 -153 562T-288 500",
      775: "-373 610Q-373 647 -348 671T-292 695Q-251 695 -227 669T-203 610T-225 551T-288 525Q-327 525 -350 551T-373 610",
      776: "-479 615Q-479 650 -456 672T-397 695Q-361 693 -341 669T-320 615Q-320 583 -343 559T-399 535Q-428 535 -453 556T-479 615ZM-256 615Q-256 651 -232 673T-176 695Q-149 695 -123 675T-97 615Q-97 578 -121 557T-180 535Q-211 537 -233 559T-256 615",
      778: "-415 618Q-415 653 -382 677T-296 702H-291Q-194 702 -168 647Q-161 634 -161 618Q-161 607 -165 596T-180 570T-220 546T-288 536T-355 545T-394 568T-410 594T-415 618ZM-223 618Q-223 645 -234 652T-274 659H-283Q-289 659 -297 659T-307 660Q-328 660 -339 653T-351 638T-353 619Q-353 591 -341 585T-288 578Q-260 578 -249 580T-230 590T-223 618",
      779: "-389 511L-442 543Q-442 544 -424 606T-404 674Q-390 705 -361 713Q-360 713 -356 713T-349 714Q-340 714 -330 712Q-273 690 -273 644Q-273 621 -290 604L-342 554L-389 511ZM-198 511L-251 543Q-251 544 -233 606T-213 674Q-199 705 -170 713Q-169 713 -165 713T-158 714Q-127 714 -105 693T-82 647Q-82 638 -84 631T-89 618T-99 604T-112 590T-130 574T-151 554L-198 511",
      780: "-444 603Q-445 604 -439 618T-425 646T-417 659L-352 635L-288 611L-224 635L-159 659Q-158 660 -151 647T-138 619T-132 603Q-135 601 -211 558T-288 515T-365 558T-444 603",
      824: "-705 -210Q-715 -210 -724 -203T-734 -179Q-734 -171 -732 -166Q-730 -163 -474 266T-215 698Q-208 711 -190 711Q-180 711 -171 704T-161 681Q-161 672 -164 667Q-171 654 -425 228T-683 -201Q-692 -210 -705 -210",
      8194: "",
      8195: "",
      8196: "",
      8197: "",
      8198: "",
      8201: "",
      8202: "",
      8211: "0 249V300H574V249H0",
      8212: "0 249V300H1149V249H0",
      8213: "0 249V300H1149V249H0",
      8214: "205 -225Q201 -234 199 -237T191 -244T175 -248T161 -246Q151 -240 146 -229Q145 -224 145 251Q145 725 146 730Q156 750 176 750Q193 748 205 727V-225ZM369 727L372 732Q375 737 377 740T385 747T398 750Q406 750 413 747Q423 740 428 730Q430 720 430 251Q430 -219 428 -229Q423 -240 413 -246Q408 -248 400 -248Q393 -248 388 -247T379 -242T375 -236T371 -230L369 -225V727",
      8215: "0 -61V-10H574V-61H0",
      8216: "58 461Q58 503 70 542T99 607T134 654T165 684T184 694T201 683T213 664Q213 658 202 648T175 624T143 583T116 518Q115 512 114 505T112 493L111 488Q132 500 161 500Q198 500 221 475T245 414T222 354T161 329Q112 329 85 369T58 461",
      8217: "74 572T74 608T97 669T157 694Q203 694 232 657T261 559Q261 520 250 482T222 418T187 370T155 339T135 329Q128 329 117 340T106 359Q106 365 117 375T144 399T176 440T203 505Q204 511 205 518T208 530V535L202 532Q196 530 184 527T158 523Q121 523 98 547",
      8220: "110 461Q110 502 121 541T150 606T185 653T217 684T235 694Q242 694 254 682T266 664Q266 659 254 648T226 623T193 578T167 511Q164 500 164 494T164 487Q188 500 212 500Q251 500 274 475T297 414Q297 378 274 354T212 329Q167 329 139 367T110 461ZM377 461Q377 502 388 541T417 606T452 653T484 684T502 694Q509 694 521 682T533 664Q533 659 521 648T493 623T460 578T434 511Q431 500 431 494T431 487Q455 500 479 500Q518 500 541 475T564 414Q564 378 541 354T479 329Q434 329 406 367T377 461",
      8221: "38 572T38 608T61 669T121 694Q167 694 196 657T225 559Q225 520 214 482T186 418T151 370T119 339T99 329T82 340T70 360Q70 365 74 369T92 385T122 414Q142 441 154 471T170 518L172 535L166 532Q160 530 148 527T122 523Q85 523 62 547ZM305 572T305 608T328 669T388 694Q434 694 463 657T492 559Q492 520 481 482T453 418T418 370T386 339T366 329T349 340T337 360Q337 365 341 369T359 385T389 414Q409 441 421 471T436 518L439 535L433 532Q427 530 415 527T389 523Q352 523 329 547",
      8224: "231 470Q232 471 232 473Q232 477 213 540T193 636Q192 642 192 651T204 677T239 700Q249 702 255 702Q300 702 315 660Q317 653 317 636Q317 603 298 539T279 472V470Q280 470 318 488T383 506Q408 506 423 493T442 467T446 444T443 421T424 396T383 382Q355 382 318 400T279 418Q278 416 285 392T303 334T316 284Q318 268 318 234Q318 149 311 45T296 -127T284 -203Q279 -211 255 -211Q237 -211 233 -210T226 -203Q222 -195 214 -129T199 41T192 234V245Q192 286 212 349Q233 413 231 418Q229 418 192 400T128 382Q102 382 86 396T67 421T64 444T67 466T86 492T128 506Q155 506 192 488T231 470",
      8225: "193 637Q193 663 206 679T231 698T255 702T279 699T304 679T317 637Q317 605 299 557T280 504Q280 503 281 503T320 521T382 539Q410 539 428 521T446 476Q446 454 432 434T383 414H377Q358 414 320 431T281 449L280 448Q280 444 298 396T317 316Q318 310 318 301T306 275T271 252Q261 250 255 250Q210 250 195 292Q193 299 193 316Q193 347 211 395T230 448Q230 449 229 449Q227 449 196 434Q151 414 133 414H127Q102 414 87 427T68 452T64 477Q64 503 81 521T127 539Q143 539 164 532T204 515T226 504Q230 502 230 504Q230 508 212 556T193 637ZM193 184Q193 210 206 226T231 245T255 249T279 246T304 226T317 184Q317 153 299 106T280 53Q280 51 282 51T322 68T383 86Q411 86 428 69T445 24T428 -21T382 -39Q358 -39 322 -22T282 -4Q280 -3 280 -3T280 -6Q281 -13 299 -59T317 -136Q318 -142 318 -151T306 -177T271 -200Q261 -202 255 -202Q210 -202 195 -160Q193 -153 193 -136Q193 -106 211 -60T230 -6Q230 -4 228 -4T188 -21T128 -39Q100 -39 83 -22T65 24Q65 53 82 69T127 86Q150 86 187 69T228 51Q230 50 230 50T230 53Q229 58 211 105T193 184",
      8226: "64 251Q64 303 80 344T121 409T175 448T230 469T275 474Q277 474 283 474T292 473Q385 473 447 415T510 251Q510 149 449 89T287 28T126 88T64 251",
      8230: "74 85Q74 121 99 146T156 171Q200 171 222 143T245 85Q245 56 224 29T160 1Q118 1 96 27T74 85ZM562 85Q562 121 587 146T644 171Q688 171 710 143T733 85Q733 56 712 29T648 1Q606 1 584 27T562 85ZM1050 85Q1050 121 1075 146T1132 171Q1176 171 1198 143T1221 85Q1221 56 1200 29T1136 1Q1094 1 1072 27T1050 85",
      8242: "240 563Q278 563 304 539T331 480V473Q331 462 316 431T217 236Q199 200 174 151T136 78T123 50Q113 33 105 33Q101 33 72 45T38 60Q35 63 35 65Q35 77 101 293T171 517Q182 542 202 552T240 563",
      8254: "80 540V607H494V540H80",
      8260: "451 730Q460 750 479 750Q492 750 501 740T510 718Q508 708 318 244L122 -232Q112 -250 95 -250Q82 -250 73 -241T64 -218Q66 -205 258 261T451 730",
      8407: "406 694Q406 704 413 713T433 723Q448 723 454 719T467 701Q483 665 522 648Q542 637 542 619Q542 605 536 599T514 586Q468 569 431 532Q411 513 399 513Q386 513 378 522T369 543Q369 557 381 568Q385 572 387 574L400 588H228L56 589Q33 598 33 618Q33 636 56 649H426Q406 676 406 694",
      8463: "477 56Q477 48 479 46T490 43Q522 45 544 75T577 140Q582 156 585 159T605 162H611H622Q642 162 642 148Q642 138 632 114T602 62T550 13T478 -8Q429 -8 394 17T358 83Q358 95 395 199T433 350Q433 400 394 400H388H383Q335 400 291 363Q256 332 236 298Q233 293 202 170T169 40Q160 18 141 5T99 -8Q70 -8 58 9T45 39Q45 51 116 336L167 540H80V607H184L188 622H184Q183 622 179 622T169 623T157 624T146 624T136 624T131 625Q119 628 119 642Q119 647 123 661T129 679Q133 684 142 685T220 690Q293 694 307 694Q324 694 328 679Q328 673 311 607H494V540H294Q286 507 278 473T264 420L260 403Q260 400 269 408Q327 451 393 451H401H410Q425 451 439 450T476 442T515 424T544 391T556 337Q556 286 517 179T477 56",
      8467: "245 -19Q228 -19 212 -16T184 -6T162 9T143 27T129 46T118 66T111 84T106 99T102 111L101 116L69 89L36 62Q31 60 24 62Q-1 88 -1 98Q-1 101 1 105Q1 106 73 170L95 189V197Q95 242 112 317T159 476T241 624T353 701Q357 702 367 702Q428 702 444 641Q446 630 446 606Q446 454 241 246L215 220L212 203Q203 150 203 114Q203 113 203 106T204 95T205 82T209 67T214 54T223 43T236 35T253 32Q277 32 305 44T352 70T389 98T407 112Q409 113 412 113Q420 113 432 95Q445 77 443 70Q440 64 416 44T342 3T245 -19ZM387 615Q387 651 366 651Q342 651 321 604T276 470L241 331Q246 331 280 373T350 486T387 615",
      8472: "399 159Q410 159 421 151T433 126Q433 104 410 85Q408 84 410 78Q411 72 414 66T428 51T455 43Q483 43 506 55T543 83T568 125T584 166T594 206Q595 211 596 214Q610 273 610 301Q610 365 542 365H538Q483 365 429 344T337 292T269 229T225 175T210 150L255 99Q261 92 274 78T292 58T305 41T316 22T321 3T324 -23Q324 -87 283 -148T174 -210H171Q161 -210 152 -209T128 -201T101 -180T81 -141T72 -78Q72 -72 72 -60T73 -45Q79 4 102 65L108 81Q84 117 84 167Q84 273 140 367T269 461Q285 461 285 447Q285 440 282 431Q278 418 276 415T264 410Q228 404 201 336T174 219Q174 218 176 202L184 214Q252 303 348 360T549 417Q614 417 658 391T719 317Q726 292 726 260Q726 148 646 70T451 -8Q407 -8 377 17T346 92Q346 159 396 159H399ZM178 -160Q200 -160 216 -132T232 -75Q232 -63 228 -56T203 -26Q196 -18 192 -14Q185 -5 176 5T161 20T156 27L153 28Q151 28 146 8T137 -42T132 -89Q132 -160 178 -160",
      8501: "590 427Q581 427 579 433T575 450T568 470V468L532 288L541 281Q620 220 634 165L637 154V124Q637 74 628 46Q623 32 612 16T592 0Q580 0 578 19T569 69T538 121Q532 126 385 240T236 355Q234 355 231 338T225 291T222 237Q222 222 223 213T225 201T228 195T231 190Q238 179 261 160T300 119T316 73Q316 41 291 23T231 1Q226 0 149 0H98Q73 0 69 3T64 24Q64 43 67 47T85 51H89Q119 51 134 55T152 64T154 76Q154 95 125 141T96 220Q96 243 104 270T123 319T145 360T164 391T172 404T150 421T102 468T68 529L65 541V570Q65 620 74 648Q79 664 91 679T111 694Q122 694 123 675T132 625T164 573Q168 569 319 452T471 335Q471 337 486 409T502 488Q502 489 491 493T467 511T448 546V573Q448 602 452 624T462 659T474 680T486 691T493 694Q499 694 502 691T507 682T513 673Q517 667 534 651T557 630Q558 629 590 616T631 587Q638 577 638 543Q637 489 622 458T590 427",
      8592: "1063 281Q1084 268 1084 251Q1084 231 1063 221L649 220H235Q340 133 364 17Q368 1 368 -2Q368 -16 343 -17Q340 -17 338 -17H332Q317 -17 314 -14T305 6Q298 34 285 62T247 126T179 189T78 233Q64 237 64 251Q64 261 74 265T108 277T154 297Q212 328 251 379T305 495Q309 511 313 514T333 518H338Q363 517 367 510Q368 507 368 503Q368 500 364 484Q345 401 287 331Q254 295 235 282L649 281H1063",
      8593: "33 396Q14 396 14 423Q14 445 18 449T41 459Q72 466 99 478T147 505T185 537T215 571T238 604T254 635T264 661T270 678L272 685Q276 694 288 694Q298 692 300 689T307 672Q331 592 392 535T535 459Q553 454 557 450T561 423Q561 396 542 396Q531 396 501 405T418 443T329 517L319 529L318 179V-171Q307 -193 288 -193Q265 -193 258 -171L257 179V529L247 517Q209 473 158 444T76 405T33 396",
      8594: "65 251Q65 270 87 281H500L914 282Q881 304 842 357T785 482Q781 500 781 501Q781 512 792 517Q794 518 812 518H817Q832 518 835 515T844 495Q864 412 923 351T1065 270Q1085 263 1085 251Q1085 240 1077 236T1044 225T995 204Q937 173 898 122T844 6Q840 -10 836 -13T816 -17H811Q786 -16 782 -9Q781 -6 781 -2Q781 1 785 17Q813 138 914 220H500L87 221Q65 228 65 251",
      8595: "14 77Q14 104 33 104Q44 104 74 96T156 57T247 -17L257 -29V321Q257 669 259 675Q268 694 289 694Q295 693 300 691T307 686T313 679T318 672V321L319 -29L329 -17Q366 26 417 55T499 94T542 104Q561 104 561 77Q561 56 557 51T535 41Q481 28 438 0T370 -58T330 -119T308 -167T302 -187Q297 -194 288 -194Q278 -194 273 -186T265 -165T251 -127T220 -77Q151 15 41 41Q22 46 18 50T14 77",
      8596: "305 495Q309 511 313 514T333 518H338Q363 517 367 510Q368 507 368 503Q368 500 364 484Q345 401 287 331Q254 295 235 282L404 281H744L914 282Q880 305 839 362T785 484Q781 500 781 503Q781 517 806 518Q809 518 811 518H817Q832 518 835 515T844 495Q864 412 923 351T1065 270Q1085 263 1085 251Q1085 240 1077 236T1044 225T995 204Q937 173 898 122T844 6Q840 -10 836 -13T816 -17H811Q786 -16 782 -9Q781 -6 781 -2Q781 1 785 17Q813 138 914 220H235Q340 133 364 17Q368 1 368 -2Q368 -16 343 -17Q340 -17 338 -17H332Q317 -17 314 -14T305 6Q298 34 285 62T247 126T179 189T78 233Q64 237 64 251Q64 261 74 265T108 277T154 297Q212 328 251 379T305 495",
      8597: "33 469Q14 469 14 496Q14 518 18 522T41 532Q121 551 182 608T268 745Q275 767 288 767Q299 767 303 755T320 713T355 650Q424 558 535 532Q553 527 557 523T561 496Q561 469 542 469Q531 469 501 478T418 516T329 590L319 602L318 426V74L319 -102L329 -90Q366 -47 417 -18T499 21T542 31Q561 31 561 4Q561 -17 557 -22T535 -32Q454 -51 393 -108T307 -245Q300 -267 288 -267Q279 -267 274 -259T266 -238T250 -200T220 -150Q151 -58 41 -32Q22 -27 18 -23T14 4Q14 31 33 31Q44 31 74 23T156 -16T247 -90L257 -102V602L247 590Q209 546 158 517T76 478T33 469",
      8598: "429 724Q438 724 452 711T466 690Q466 681 452 671Q361 602 242 599H218L756 135Q832 69 913 -1T1036 -108L1077 -143Q1084 -151 1084 -163Q1084 -180 1074 -187T1054 -194H1052Q1043 -194 939 -105Q866 -42 812 5Q180 549 178 549V546Q179 542 179 539Q183 520 183 483Q183 435 172 390T149 323T130 296Q121 292 115 295Q85 312 85 328Q85 331 95 350T115 406T125 486Q125 540 110 583T79 648T64 675Q64 681 68 687T81 693Q87 693 94 690Q162 657 232 657Q296 657 349 681T420 722Q422 724 429 724",
      8599: "1069 693Q1076 693 1080 687T1085 675Q1085 671 1076 656T1055 621T1034 565T1024 486Q1024 442 1034 406T1054 351T1064 328Q1064 321 1057 313T1042 300L1034 295Q1030 293 1027 293Q1023 293 1020 295T1014 301T1009 308T1005 316T1001 324Q980 368 971 419Q966 442 966 484V492Q966 528 972 553Q971 553 757 368T328 -3T107 -191Q103 -193 94 -193Q78 -193 71 -184T64 -164Q64 -153 72 -143Q79 -136 382 124L934 599H909Q837 599 760 634T683 690Q683 697 696 710T722 724Q726 724 742 714T779 691T838 668T920 657Q959 657 991 666T1043 684T1069 693",
      8600: "65 663Q65 680 74 687T93 694H96Q104 694 118 683T204 610Q280 545 338 495Q969 -49 971 -49L970 -46Q970 -42 970 -39Q966 -20 966 18Q966 65 977 110T1001 177T1019 204Q1028 208 1034 205Q1064 188 1064 172Q1064 169 1054 150T1034 94T1024 14Q1024 -28 1033 -64T1054 -120T1075 -155T1085 -175Q1085 -181 1081 -187T1068 -193Q1062 -193 1055 -190Q987 -157 919 -157Q817 -157 739 -215Q727 -224 720 -224Q712 -224 697 -210Q683 -199 683 -190T697 -171Q788 -102 907 -99H931L393 365Q317 431 236 501T114 608L72 643Q65 651 65 663",
      8601: "80 -193Q73 -193 69 -187T64 -175Q64 -172 79 -150T109 -84T125 14Q125 58 115 94T95 149T85 172Q85 179 92 187T108 200L115 205Q119 207 122 207Q126 207 129 205T135 199T140 192T144 184T148 176Q169 132 178 81Q183 58 183 17Q183 -7 182 -24T178 -48L177 -53Q178 -53 389 129T816 498T1043 692Q1049 694 1054 694Q1070 694 1077 684T1085 664Q1085 653 1077 643Q1070 636 767 376L215 -98L240 -99Q312 -99 389 -134T466 -190Q466 -197 452 -210T429 -224Q422 -224 411 -215Q330 -157 229 -157Q190 -157 158 -166T106 -184T80 -193",
      8614: "65 426Q74 448 95 448Q112 448 125 426V281H500L914 282Q881 304 842 357T785 482Q781 500 781 501Q781 512 792 517Q794 518 812 518H817Q832 518 835 515T844 495Q864 412 923 351T1065 270Q1085 263 1085 251Q1085 240 1077 236T1044 225T995 204Q937 173 898 122T844 6Q840 -10 836 -13T816 -17H811Q786 -16 782 -9Q781 -6 781 -2Q781 1 785 17Q813 138 914 220H500L135 221Q134 221 133 221T130 220H125V76Q115 54 95 54Q73 54 65 76V426",
      8617: "1029 475Q1029 505 1068 505Q1129 501 1173 463T1218 363Q1218 296 1170 259T1066 221H1063L649 220H235Q340 133 364 17Q368 1 368 -2Q368 -16 343 -17Q340 -17 338 -17H332Q317 -17 314 -14T305 6Q298 34 285 62T247 126T179 189T78 233Q64 237 64 251Q64 261 74 265T108 277T154 297Q212 328 251 379T305 495Q309 511 313 514T333 518H338Q363 517 367 510Q368 507 368 503Q368 500 364 484Q345 401 287 331Q254 295 235 282L649 281H1063Q1105 284 1131 305T1158 361Q1158 385 1146 401Q1122 441 1063 444Q1029 446 1029 475",
      8618: "225 221H218Q159 221 112 260T65 363Q65 431 116 468T221 505Q254 503 254 474Q254 456 245 450T216 443T188 438Q152 427 137 401Q125 385 125 362Q125 334 147 310Q171 288 221 281H632L1046 282Q1013 304 974 357T917 482Q913 500 913 501Q913 512 924 517Q926 518 944 518H949Q964 518 967 515T976 495Q996 412 1055 351T1197 270Q1217 263 1217 251Q1217 240 1209 236T1176 225T1127 204Q1069 173 1030 122T976 6Q972 -10 968 -13T948 -17H943Q918 -16 914 -9Q913 -6 913 -2Q913 1 917 17Q945 138 1046 220H632L225 221",
      8636: "1063 281Q1084 268 1084 251Q1084 231 1063 221L572 220Q79 220 77 221Q64 225 64 244Q64 250 64 254T67 261T71 265T78 268T85 272Q142 302 189 345T258 421T296 484T315 516Q319 518 337 518Q358 518 363 512Q370 504 367 496Q360 469 319 404T219 290L209 282L636 281H1063",
      8637: "1063 281Q1084 268 1084 251Q1084 231 1063 221L636 220H209L219 212Q278 162 319 97T367 5Q370 -3 363 -11Q358 -17 337 -17H332Q318 -17 314 -14T302 7Q278 55 246 95T185 160T130 202T88 228L70 237Q64 243 64 257Q64 274 75 279Q78 281 571 281H1063",
      8640: "65 251Q65 270 87 281H513L940 282L930 290Q871 338 830 403T782 496Q779 510 791 517Q794 518 812 518H817Q831 518 835 515T847 494Q871 445 903 404T966 338T1022 298T1064 272T1083 259Q1085 255 1085 245Q1085 225 1072 221Q1070 220 578 220L87 221Q65 228 65 251",
      8641: "84 279Q89 281 580 281Q1070 281 1074 279Q1085 275 1085 256Q1085 245 1083 241T1066 230Q919 153 847 7Q839 -11 835 -14T817 -17H812Q791 -17 786 -11Q779 -3 782 5Q789 31 830 96T930 212L940 220H513L87 221Q64 229 64 250Q64 272 84 279",
      8652: "65 451Q65 470 87 481H513L940 482L930 490Q871 538 830 603T782 696Q779 710 791 717Q794 718 812 718H817Q831 718 835 715T847 694Q871 645 903 604T966 538T1022 498T1064 472T1083 459Q1085 455 1085 445Q1085 425 1072 421Q1070 420 578 420L87 421Q65 428 65 451ZM1063 281Q1084 268 1084 251Q1084 231 1063 221L636 220H209L219 212Q278 162 319 97T367 5Q370 -3 363 -11Q358 -17 337 -17H332Q318 -17 314 -14T302 7Q278 55 246 95T185 160T130 202T88 228L70 237Q64 243 64 257Q64 274 75 279Q78 281 571 281H1063",
      8656: "1063 169L1068 166Q1072 163 1074 162T1079 157T1083 149T1085 139Q1085 118 1063 109L733 108H404L412 99Q455 50 488 -10Q498 -27 493 -37Q487 -46 465 -46H460Q446 -46 439 -39T426 -18T399 25T344 89Q239 194 99 229Q96 230 92 231T85 232T79 234T73 235T69 237T66 240T65 244T64 250Q64 267 90 271Q197 295 286 361T430 525Q439 542 442 544T460 547H465Q487 547 492 539Q496 531 496 530Q496 521 471 482T414 405L404 394L733 393H1063Q1064 392 1069 389T1076 384T1082 375T1085 362Q1085 344 1063 333L700 332H338L324 321Q283 290 230 264L205 250Q266 224 323 180L338 170L700 169H1063",
      8657: "672 343Q672 326 670 320T657 313Q644 313 602 335Q577 349 557 361T527 381T509 395T499 403T495 406T494 118Q494 -166 492 -174Q484 -193 465 -193H463Q456 -193 453 -192T444 -186T433 -170V465L423 477Q407 495 394 514T367 554T351 579Q349 576 339 560T313 520T279 477L269 465V-22V-102Q269 -132 269 -145T268 -169T266 -180T260 -185T253 -191Q248 -193 239 -193H237Q218 -193 210 -174Q208 -166 208 118Q208 406 207 406L199 399Q191 392 165 374T100 335Q58 313 45 313Q35 313 33 319T30 343V349Q30 359 30 362T35 369T45 374T66 383T100 401Q267 499 333 680Q339 694 351 694Q361 694 365 687T380 652T407 597Q442 536 489 489T573 420T638 383T670 365Q672 361 672 343",
      8658: "64 362Q64 380 87 393H416L745 394L735 405Q708 436 681 477T654 531Q654 547 679 547H684H689Q703 547 710 540T723 519T750 475T806 411Q914 303 1059 271Q1060 271 1063 270T1068 269T1072 268T1076 266T1079 264T1082 260T1083 256T1084 250Q1084 242 1080 238T1063 231T1035 225T992 211T934 185Q797 112 719 -24Q710 -40 706 -43T689 -46H684Q653 -46 653 -31Q653 -24 661 -10Q694 50 737 99L745 108H416L87 109Q64 117 64 139Q64 156 87 169H449L812 170L826 180Q842 193 860 204T892 223T918 237T937 246L944 250L919 264Q866 290 825 321L811 332H449L87 333Q64 343 64 362",
      8659: "30 157Q30 174 32 180T46 187Q59 187 104 163Q154 136 198 101L207 94Q208 94 208 382Q208 666 210 674Q219 694 241 694Q254 692 262 683Q266 679 267 674Q269 658 269 522V35L279 23Q295 5 308 -14T335 -54T351 -79Q353 -76 363 -60T389 -20T423 23L433 35V671Q439 682 444 686T452 692T463 693H465Q484 693 492 674Q494 666 494 382Q494 94 495 94L504 101Q547 135 593 160T652 187Q665 191 671 177Q672 175 672 157Q672 137 669 134T636 116Q606 101 578 83T528 47T486 9T452 -30T424 -68T403 -103T387 -134T377 -159T370 -176L367 -184Q360 -194 351 -194Q345 -194 342 -192T334 -182T327 -166T315 -137T295 -97Q260 -36 213 11T129 80T63 117T32 136Q30 139 30 157",
      8660: "336 497Q358 541 363 544Q367 547 379 547H384Q401 547 405 545Q418 538 414 525T389 474T346 408L335 393H814L803 408Q781 436 760 474T735 525T744 545Q748 547 765 547Q771 547 774 547T780 546T786 544T790 541T794 535T799 527T805 514T813 497Q841 446 877 406T950 340T1014 301T1068 276L1096 265Q1102 259 1102 251Q1102 240 1085 232Q981 195 902 121Q835 56 798 -25Q791 -40 787 -43T765 -46T744 -44Q735 -40 735 -30Q735 -15 760 28T806 98L814 108H335L343 98Q361 75 378 46T404 -1T414 -24Q418 -37 405 -44Q401 -46 384 -46T363 -43T351 -25Q314 56 247 121Q216 150 182 173T125 206T79 226T53 237Q47 243 47 251Q47 254 47 256T49 261T52 264T57 267T61 268T66 270T71 272Q246 335 336 497ZM985 251Q932 280 882 323L871 332H278Q264 321 253 311T237 297T214 282T164 251L176 244Q221 218 278 169H871Q928 218 973 244L985 251",
      8661: "49 441Q30 441 30 464V471V480Q30 498 44 502Q237 573 331 750Q337 767 351 767Q360 767 368 753T400 702T460 629Q504 584 552 554T632 511T666 497Q672 493 672 471Q672 454 670 449Q664 441 653 441Q639 443 591 465T508 513L495 522L494 386V114L495 -22L508 -13Q543 12 591 34T653 59Q672 59 672 36V29V20Q672 2 658 -2Q465 -71 367 -257Q360 -267 351 -267Q343 -267 336 -257T320 -231T292 -187T242 -129Q198 -84 150 -54T70 -11T36 3Q30 7 30 29Q30 46 32 51Q38 59 49 59Q63 57 111 35T194 -13L208 -22V522L194 513Q159 488 111 466T49 441ZM422 584Q411 594 400 606T383 626T366 648T351 667Q349 665 339 652T314 620T280 584L269 573V-73L280 -84Q305 -108 351 -166Q353 -164 363 -151T389 -119T422 -84L433 -73V573L422 584",
      8704: "1 664Q-2 685 23 693H27Q46 693 54 680T102 578L148 475H492L533 570Q541 586 548 603T560 630T569 650T576 667T582 678T588 686T594 691T600 693T609 694Q622 694 631 684T639 662Q637 653 492 325T341 -8Q333 -16 320 -16Q306 -16 298 -8Q294 -4 147 326L1 656V664ZM464 414H319Q175 414 175 413L319 88L464 414",
      8707: "81 347Q81 359 84 363T104 378H513V633H300L87 634Q64 642 64 664Q64 685 84 692Q89 694 321 694H552Q571 681 574 669V25Q567 7 552 1H87Q64 12 64 30T87 61H513V317H308Q103 317 99 319Q81 328 81 347",
      8709: "285 711Q307 711 326 708T357 701T370 698Q371 698 375 710T383 735T389 750Q395 767 415 767Q431 767 438 757T446 738T436 701T426 670Q426 668 433 664Q468 633 489 588Q511 542 519 488T528 344Q528 286 524 243T508 150T466 63T394 6Q345 -17 287 -17Q265 -17 246 -14T216 -7T203 -4Q191 -47 183 -60T159 -73Q146 -73 137 -63T128 -44Q128 -38 138 -7L148 24L141 30Q134 35 120 49Q94 77 78 113T56 194T48 268T46 344Q46 388 47 416T56 494T78 577T122 644T194 694Q239 711 285 711ZM351 639Q350 639 346 642T337 648T325 654T306 658T283 660Q254 660 221 638T181 567Q171 513 171 375Q171 164 182 129L351 639ZM402 356Q402 516 395 555Q395 557 395 559T394 563T394 566L393 568L223 57Q252 34 286 34H288Q318 34 346 53T387 109Q402 152 402 329V356",
      8710: "901 12Q901 7 892 0H479Q65 0 62 2Q56 6 56 11Q56 14 242 347T433 685Q438 694 450 696Q454 698 480 698H506L523 687Q526 683 711 354T899 17Q901 13 901 12ZM653 137L427 538L202 137L315 136H540L653 137",
      8712: "97 251Q97 393 194 484T417 586Q418 586 436 586T482 586T538 587H648Q649 586 652 584T658 580T664 575T668 568T670 557Q670 536 648 527L534 526Q515 526 491 526T457 526T435 526T417 525T404 523T390 521T374 517Q298 498 243 447T167 324Q159 295 159 283Q159 281 403 281H648Q649 280 652 278T658 274T664 269T668 262T670 251Q670 230 648 221L403 220Q159 220 159 218Q159 206 166 182T190 122T247 50T341 -6Q380 -20 405 -22T534 -25H648Q649 -26 654 -29T661 -34T667 -43T670 -56Q670 -74 648 -85L541 -86Q419 -86 396 -82Q276 -65 187 24T97 251",
      8713: "126 -210Q116 -210 107 -203T97 -179Q97 -171 99 -166Q99 -165 111 -145T150 -80T203 8Q97 104 97 251Q97 393 194 484T417 586Q418 586 436 586T482 586T538 587H549Q565 614 582 643T608 685L616 698Q623 711 641 711Q651 711 660 704T670 681Q670 672 667 667Q666 666 661 657T644 627T620 587H648Q649 586 652 584T658 580T664 575T668 568T670 557Q670 536 648 527L584 526L437 281H648Q649 280 652 278T658 274T664 269T668 262T670 251Q670 230 648 221L403 220H401L283 23Q311 5 341 -6Q380 -20 405 -22T534 -25H648Q649 -26 654 -29T661 -34T667 -43T670 -56Q670 -74 648 -85L541 -86Q419 -86 396 -82Q320 -71 252 -29Q152 -197 148 -201Q139 -210 126 -210ZM235 62L330 220Q159 219 159 218Q159 196 176 150T235 62ZM366 281L513 526Q503 526 487 526T465 526T448 525T433 525T422 525T412 524T403 523T394 521T385 519T374 517Q298 498 243 447T167 324Q159 295 159 283Q159 281 366 281",
      8715: "96 251Q96 268 119 281H363Q607 281 607 283Q607 295 600 319T576 379T519 451T425 507Q386 521 361 523T233 526L119 527Q96 535 96 557Q96 578 116 585Q121 587 229 587Q238 587 257 587T288 588Q366 588 435 568T568 488Q670 388 670 251Q670 155 621 78T499 -39T345 -85Q336 -86 225 -86L119 -85Q96 -77 96 -55Q96 -38 119 -25H233Q356 -24 371 -21Q373 -21 393 -16Q468 3 523 55T599 177Q607 206 607 218Q607 220 363 220L119 221Q96 229 96 251",
      8722: "119 221Q96 230 96 251T116 279Q121 281 448 281H775Q776 280 779 278T785 274T791 269T795 262T797 251Q797 230 775 221H119",
      8723: "64 155Q64 172 87 185H416V476H251L87 477Q64 485 64 507Q64 528 84 535Q89 537 448 537H807Q808 536 811 534T817 530T823 525T827 518T829 507Q829 486 807 477L642 476H477V185H807Q808 184 811 182T817 178T823 173T827 166T829 155Q829 134 807 125L642 124H477V-39Q477 -203 475 -208Q466 -227 446 -227Q427 -227 417 -205L416 -41V124H251L87 125Q64 133 64 155",
      8725: "451 730Q460 750 479 750Q492 750 501 740T510 718Q508 708 318 244L122 -232Q112 -250 95 -250Q82 -250 73 -241T64 -218Q66 -205 258 261T451 730",
      8726: "64 718Q63 731 72 740T94 750Q106 750 113 743Q118 741 122 732L318 256Q508 -208 510 -218Q511 -231 502 -240T480 -250Q460 -250 451 -230Q451 -229 259 238T64 718",
      8727: "236 431Q237 447 251 459T287 472T323 459T338 431Q338 423 328 363L317 300Q318 300 340 317T392 356T435 387Q442 390 450 390Q470 390 485 374T501 335Q501 326 500 320T494 309T486 300T473 293T458 287T438 280T414 272L353 250L414 228Q422 225 436 221T457 214T472 208T485 201T493 192T499 181T501 166Q501 141 484 126T450 111Q447 111 445 111T441 111T437 112T433 114T428 117T422 121T414 127T404 135T391 145T374 158L317 200L328 137Q338 77 338 69Q336 52 321 40T287 28T253 40T236 69Q236 77 246 137L257 200Q256 200 234 183T182 144T139 113Q132 110 124 110Q104 110 89 126T73 165Q73 174 74 180T80 191T88 200T101 207T116 213T136 220T160 228L221 250L160 272Q152 275 138 279T117 286T102 292T89 299T81 308T75 319T73 334Q73 359 90 374T124 389Q127 389 129 389T133 389T137 388T141 386T146 383T152 379T160 373T170 365T183 355T200 342L257 300L246 363Q236 423 236 431",
      8728: "64 251Q64 303 80 344T121 409T175 448T230 469T275 474Q277 474 283 474T292 473Q385 473 447 415T510 251Q510 149 449 89T287 28T126 88T64 251ZM448 251Q448 325 405 369T286 413Q215 413 171 371T126 251Q126 177 168 133T287 89Q361 89 404 132T448 251",
      8729: "64 251Q64 303 80 344T121 409T175 448T230 469T275 474Q277 474 283 474T292 473Q385 473 447 415T510 251Q510 149 449 89T287 28T126 88T64 251",
      8730: "107 178Q100 178 89 188T78 207Q78 216 84 220Q85 221 124 248T207 304T260 338Q269 340 275 335Q276 334 370 156L463 -20L698 393Q928 800 935 811Q944 820 954 820Q972 820 980 811T988 789Q988 781 858 553Q776 409 718 306Q452 -166 447 -171Q439 -179 422 -180Q405 -180 400 -175Q399 -174 346 -73T241 128T187 229L151 205Q111 178 107 178",
      8733: "65 222Q65 282 88 329T144 401T208 438T261 451H273Q312 451 320 450Q456 431 526 330L537 316Q638 451 778 451Q813 451 830 445V388Q821 391 799 391Q758 391 721 377T660 342T618 301T592 266L584 251Q648 152 697 114Q748 74 804 74H806Q823 74 829 77Q830 77 830 38V-1L820 -3Q801 -7 786 -7H771Q699 -7 632 25T527 114L516 128Q414 -8 276 -8Q192 -8 129 56T65 222ZM256 53Q296 53 332 67T392 102T434 143T461 178L469 193Q405 292 356 330Q308 369 251 369H243Q196 369 156 328T116 221Q116 191 124 161T158 99T225 55Q234 53 256 53",
      8734: "65 219Q65 318 132 385T302 452Q473 452 573 331L589 312L596 320Q710 452 857 452Q948 452 1016 386T1084 225Q1084 125 1017 59T848 -8Q679 -8 576 113L560 132L553 124Q439 -8 292 -8Q200 -8 133 58T65 219ZM1033 224Q1033 291 987 340T875 389Q748 389 648 261Q641 253 642 251Q717 163 748 137Q813 81 880 81Q941 81 987 120T1033 224ZM275 56Q315 56 353 70T418 104T466 144T497 178L507 192Q507 193 474 230T441 269Q355 362 267 362Q210 362 163 324T116 221Q116 150 162 103T275 56",
      8736: "71 0L68 2Q65 3 63 5T58 11T55 20Q55 21 56 23V25Q55 27 55 30Q55 31 56 33V35Q55 37 55 40Q55 42 57 48Q67 63 346 381Q421 467 518 578Q607 680 623 697T647 714Q656 714 661 708T666 694V692Q676 687 676 674Q676 668 673 663Q672 662 637 622T534 503T400 350L147 61L386 60H653Q666 50 666 40V38Q676 31 676 20Q676 8 661 0H71",
      8739: "160 -249Q138 -249 129 -225V250Q129 725 131 729Q139 750 159 750T190 725V-225Q181 -249 160 -249",
      8741: "205 -225Q201 -234 199 -237T191 -244T175 -248T161 -246Q151 -240 146 -229Q145 -224 145 251Q145 725 146 730Q156 750 176 750Q193 748 205 727V-225ZM369 727L372 732Q375 737 377 740T385 747T398 750Q406 750 413 747Q423 740 428 730Q430 720 430 251Q430 -219 428 -229Q423 -240 413 -246Q408 -248 400 -248Q393 -248 388 -247T379 -242T375 -236T371 -230L369 -225V727",
      8743: "95 -16Q78 -16 71 -6T64 14Q64 20 65 22L212 308Q359 593 361 595Q370 604 385 604Q398 602 405 595Q407 593 554 308L701 22Q702 20 702 15Q702 1 693 -8T671 -17Q661 -17 651 -9Q647 -5 515 251L383 506L251 251Q119 -5 116 -8Q108 -16 95 -16",
      8744: "64 572Q64 585 72 594T94 604T116 595Q119 592 251 336L383 81L515 336Q647 592 651 596Q661 604 671 604Q684 604 693 595T702 572Q702 567 701 565L554 279Q407 -6 405 -8Q404 -9 401 -11T397 -14Q392 -16 383 -16H380Q369 -16 361 -8Q359 -6 212 279L65 565Q65 566 65 568T64 572",
      8745: "94 -16Q73 -16 64 8V209Q64 239 64 287Q65 418 69 432Q70 434 70 435Q84 487 125 523T216 575T299 597T354 603H372Q444 603 501 590T591 558T648 515T681 471T696 435Q696 434 697 432Q701 417 702 309Q702 303 702 287Q702 239 702 209V8Q693 -16 672 -16Q650 -16 643 3Q641 8 641 201Q641 397 640 403Q631 472 558 507T383 542Q339 542 298 535T219 511T156 468T126 403Q125 397 125 201Q125 8 123 3Q116 -16 94 -16",
      8746: "672 603Q693 603 702 579V378Q702 348 702 300Q701 169 697 155Q696 153 696 152Q676 78 593 31T383 -16Q265 -16 179 28T70 152Q70 153 69 155Q65 170 64 278Q64 285 64 300Q64 348 64 378Q64 579 65 583Q74 604 94 604T123 584Q125 579 125 386Q125 190 126 184Q135 115 210 80T383 44Q426 44 467 51T546 75T609 119T640 184Q641 190 641 386Q641 579 643 584Q650 603 672 603",
      8747: "204 -71Q204 -108 181 -124T137 -141Q132 -141 132 -142Q142 -161 154 -161Q164 -161 186 -152Q200 -145 210 -135T228 -107T241 -77T249 -38T254 -2T258 38T262 74Q282 265 334 489Q334 490 337 503T341 523T347 544T355 569T365 594T379 620T397 643T420 666T447 685T481 700Q511 711 539 711T587 696T616 656T628 612T632 573Q632 536 610 519T562 501Q534 501 513 519T492 571Q492 608 515 624T559 641Q564 641 564 642Q554 661 542 661Q532 661 510 652Q496 645 486 635T468 607T455 577T447 538T442 502T438 462T434 426Q414 235 362 11Q352 -35 347 -54T328 -101T291 -152Q235 -208 162 -211Q147 -211 136 -208T109 -196T83 -165T67 -108Q64 -94 64 -73Q64 -37 86 -19T134 -1Q162 -1 183 -19T204 -71",
      8764: "64 155Q64 210 84 262T150 353T257 391Q300 391 341 371T417 321T484 264T557 215T637 194Q702 194 745 244T788 367Q796 391 808 391Q815 391 821 381T828 353V342Q828 252 776 181T637 109Q594 109 552 129T476 179T409 236T336 285T256 306Q193 306 149 258T105 132Q98 109 86 109Q76 109 70 122T64 155",
      8768: "64 561Q64 570 76 576T108 583Q174 583 214 535T254 407Q254 368 238 324T202 248T166 173T149 92Q149 43 169 2T217 -39Q231 -40 242 -46T254 -60Q254 -69 241 -75T210 -82Q145 -82 105 -34T64 93Q64 133 80 177T116 253T152 328T169 408Q169 461 148 500T105 540Q92 540 78 545T64 561",
      8771: "64 295Q64 378 117 440T257 502Q298 502 339 485T416 443T486 394T560 352T637 335Q693 335 740 373T788 478Q796 502 808 502Q815 502 821 492T828 465V455Q828 365 771 308T640 250Q603 250 562 265T501 294T439 336L370 382Q308 417 256 417Q205 417 164 388T110 317Q110 316 109 304T107 286T103 270T97 255T86 250Q76 250 70 263T64 295ZM64 6T64 27T87 56H93Q99 56 110 56T137 56T173 56T217 56T267 57T323 57T383 57T448 57H807Q808 56 811 54T815 52T819 49T823 45T826 40T828 34T829 27Q829 7 807 -3H87Q64 6 64 27",
      8773: "64 402Q64 457 84 509T150 600T257 638Q300 638 341 618T417 569T484 511T557 462T637 441Q702 441 745 491T788 614Q796 638 808 638Q815 638 821 628T828 600V589Q828 499 776 428T637 356Q594 356 552 376T476 425T409 483T336 532T256 553Q193 553 149 505T105 379Q98 356 86 356Q76 356 70 369T64 402ZM87 197Q64 207 64 226Q64 247 84 255Q89 257 448 257H807Q808 256 811 254T817 250T823 245T827 238T829 227Q829 209 807 197H87ZM87 -27Q64 -18 64 3Q64 23 86 32Q89 33 448 33H807L812 30Q816 27 818 26T823 21T827 13T829 3Q829 -18 807 -27H87",
      8776: "64 345Q64 423 119 473T250 524Q301 524 356 503T451 455T542 407T636 385Q700 385 743 417T786 481Q786 493 791 508T807 524Q817 524 823 512T829 479Q829 404 776 352T638 300Q590 300 537 321T443 369T352 417T256 439Q207 439 166 417T110 359Q109 357 107 341T100 312T85 300Q77 300 71 313T64 345ZM64 77Q64 155 119 205T250 256Q302 256 357 235T451 187T541 139T636 117Q699 117 742 148T786 213Q786 231 792 243T808 256T823 242T829 208Q829 134 776 83T640 32Q591 32 537 53T443 101T352 149T256 171Q206 171 165 148T110 91Q109 89 107 73T100 44T85 32Q77 32 71 45T64 77",
      8781: "798 533Q812 533 820 524T829 502T819 480T769 440Q655 355 537 330Q492 322 447 322Q401 322 356 330Q289 344 219 381T118 443T73 481Q64 490 64 503Q64 517 72 525T94 533Q99 533 102 532Q107 531 138 507T209 456T314 405T446 382Q604 382 765 515Q788 533 798 533ZM95 -32Q81 -32 73 -23T64 -1Q64 10 74 21T124 61Q213 127 293 153T421 179L422 180Q424 180 426 180T432 180T441 180T452 179Q612 179 769 61Q811 29 820 19T829 -1Q829 -14 821 -23T798 -32Q788 -32 765 -14Q608 118 446 118Q287 118 128 -14Q105 -32 95 -32",
      8784: "87 333Q64 343 64 362Q64 383 84 391Q89 393 448 393H807Q808 392 811 390T817 386T823 381T827 374T829 363Q829 345 807 333H87ZM87 109Q64 118 64 139Q64 159 86 168Q89 169 448 169H807L812 166Q816 163 818 162T823 157T827 149T829 139Q829 118 807 109H87ZM362 635Q362 671 387 696T444 721Q488 721 510 693T533 635Q533 606 512 579T448 551Q406 551 384 577T362 635",
      8800: "189 -210Q179 -210 170 -203T160 -179Q160 -171 162 -166Q165 -163 327 109H87Q64 118 64 139Q64 159 86 168Q89 169 363 169L461 333H87Q64 343 64 362Q64 383 84 391Q89 393 448 393H496Q533 455 583 539T656 660T679 698Q686 711 704 711Q714 711 723 704T733 681Q733 672 730 667Q729 664 709 631T645 523T567 393H807Q808 392 811 390T817 386T823 381T827 374T829 363Q829 345 807 333H532L433 169H807L812 166Q816 163 818 162T823 157T827 149T829 139Q829 118 807 109H398Q217 -195 211 -201Q202 -210 189 -210",
      8801: "87 445Q64 454 64 475Q64 497 84 503Q89 505 448 505H807Q808 504 812 502T818 497T823 492T827 484T829 474Q829 456 807 445H87ZM87 221Q64 230 64 251T84 279Q89 281 448 281H807Q808 280 811 278T817 274T823 269T827 262T829 251Q829 230 807 221H87ZM64 6T64 27T87 56H93Q99 56 110 56T137 56T173 56T217 56T267 57T323 57T383 57T448 57H807Q808 56 811 54T815 52T819 49T823 45T826 40T828 34T829 27Q829 7 807 -3H87Q64 6 64 27",
      8804: "797 55Q797 45 790 35T767 25H759L434 180Q108 336 105 339Q96 348 96 360Q96 378 114 388Q126 394 439 544T757 695Q763 697 766 697Q780 697 788 688T797 666Q797 654 788 645Q784 641 507 509T197 361L466 232Q785 80 790 74Q797 66 797 55ZM119 -199Q96 -191 96 -169Q96 -160 102 -152T119 -140H124Q130 -140 140 -140T164 -140T197 -140T237 -140T283 -139T334 -139T389 -139T448 -139H775Q797 -153 797 -169Q797 -187 775 -199H119",
      8805: "127 25Q110 25 103 34T96 54Q96 66 105 75Q109 80 439 238L696 361Q113 637 105 645Q96 654 96 667Q96 679 104 688T128 697Q137 696 460 541T788 382Q797 373 797 360Q797 348 788 339Q785 336 459 180L135 25H127ZM119 -199Q96 -191 96 -169Q96 -160 102 -152T119 -140H124Q130 -140 140 -140T164 -140T197 -140T237 -140T283 -139T334 -139T389 -139T448 -139H775Q797 -153 797 -169Q797 -187 775 -199H119",
      8810: "734 -74T734 -86T727 -107T704 -116H702Q694 -116 584 -55Q473 7 380 58Q87 219 73 229Q64 238 64 250Q64 263 73 272Q87 282 380 443Q695 616 699 617H700Q718 617 726 607T734 588Q734 568 717 560Q705 554 435 404L157 250L439 94Q721 -61 726 -66Q734 -74 734 -86ZM1085 -74T1085 -86T1078 -107T1055 -116H1053Q1045 -116 935 -55Q824 7 731 58Q438 219 424 229Q415 238 415 250Q415 263 424 272Q438 282 731 443Q1046 616 1050 617H1051Q1069 617 1077 607T1085 588Q1085 568 1068 560Q1056 554 786 404L508 250L790 94Q1072 -61 1077 -66Q1085 -74 1085 -86",
      8811: "64 588Q64 600 72 609T94 618H95Q103 618 209 559Q322 496 419 443Q712 282 725 272Q734 263 734 250Q734 238 725 229Q714 220 415 55T110 -113Q103 -116 95 -116Q78 -116 71 -106T64 -86Q64 -74 72 -66Q77 -61 359 94L641 250L363 404Q277 452 173 509Q95 552 82 560T66 576V577Q64 585 64 588ZM415 588Q415 600 423 609T445 618H446Q454 618 560 559Q673 496 770 443Q1063 282 1076 272Q1085 263 1085 250Q1085 238 1076 229Q1065 220 766 55T461 -113Q454 -116 446 -116Q429 -116 422 -106T415 -86Q415 -74 423 -66Q428 -61 710 94L992 250L714 404Q628 452 524 509Q446 552 433 560T417 576V577Q415 585 415 588",
      8826: "797 -57Q797 -65 790 -75T766 -86Q748 -86 741 -74T733 -43T719 8T681 72Q647 112 588 141T475 185T343 207T230 216T136 219Q96 219 96 250Q96 280 132 280H136Q193 281 239 283T347 292T457 310T556 342T643 391T703 460T735 553Q741 585 763 585Q781 585 789 575T797 556Q797 540 792 513T758 434T682 345Q605 285 481 254L462 249Q483 246 526 233T633 185T733 104Q767 63 782 15T797 -57",
      8827: "96 556Q96 568 104 577T126 586Q152 586 158 553Q164 503 188 462T247 394T331 345T429 313T539 294T649 284T758 280H760Q797 280 797 250Q797 219 760 219H758Q627 217 529 204T347 160T216 77T158 -54Q152 -86 126 -86Q110 -86 103 -76T96 -57Q96 -41 101 -14T135 65T211 154Q288 214 412 245L431 250Q410 252 367 265T259 314T160 395Q127 435 112 483T96 556",
      8834: "96 251Q96 389 191 482T417 586Q418 586 428 586T456 586T496 586T546 587T601 587H775Q776 586 779 584T785 580T791 575T795 568T797 557Q797 536 775 527L597 526Q411 525 395 522Q390 521 370 516Q285 494 222 424T158 251Q158 131 246 53Q313 -9 408 -23Q417 -24 597 -25H775Q776 -26 781 -29T788 -34T794 -43T797 -56Q797 -74 775 -85H493Q407 -85 376 -79Q257 -55 177 35T96 251",
      8835: "96 -55Q96 -38 119 -25H296Q482 -24 498 -21Q503 -20 523 -15Q609 7 672 77T735 251T665 431T485 524Q476 525 296 526L119 527Q96 535 96 557Q96 578 116 585Q121 587 300 587Q451 586 476 585T522 579Q632 556 714 468T796 251Q796 112 695 13Q612 -65 497 -82Q473 -86 289 -86L119 -85Q96 -77 96 -55",
      8838: "96 361Q96 499 191 592T417 696Q418 696 428 696T456 696T496 696T546 697T601 697H775Q776 696 779 694T785 690T791 685T795 678T797 667Q797 646 775 637L597 636Q411 635 395 632Q390 631 370 626Q285 604 222 534T158 361Q158 241 246 163Q313 101 408 87Q417 86 597 85H775Q776 84 781 81T788 76T794 67T797 54Q797 36 775 25H493Q407 25 376 31Q257 55 177 145T96 361ZM149 -199Q127 -191 127 -169T149 -140H154Q160 -140 169 -140T192 -140T224 -140T262 -140T306 -139T354 -139T407 -139T463 -139H775Q776 -140 779 -142T785 -146T791 -151T795 -158T797 -169Q797 -190 775 -199H149",
      8839: "96 55Q96 72 119 85H296Q482 86 498 89Q503 90 523 95Q609 117 672 187T735 361T665 541T485 634Q476 635 296 636L119 637Q96 645 96 667Q96 688 116 695Q121 697 300 697Q451 696 476 695T522 689Q632 666 714 578T796 361Q796 222 695 123Q612 45 497 28Q473 24 289 24L119 25Q96 33 96 55ZM119 -199Q96 -190 96 -169T116 -141Q121 -139 433 -139H745Q766 -152 766 -170Q766 -190 745 -199H119",
      8846: "672 603Q693 603 702 579V378Q702 348 702 300Q701 169 697 155Q696 153 696 152Q676 78 593 31T383 -16Q265 -16 179 28T70 152Q70 153 69 155Q65 170 64 278Q64 285 64 300Q64 348 64 378Q64 579 65 583Q74 604 94 604T123 584Q125 579 125 386Q125 190 126 184Q135 115 210 80T383 44Q426 44 467 51T546 75T609 119T640 184Q641 190 641 386Q641 579 643 584Q650 603 672 603ZM353 412Q353 420 353 435T352 456Q352 483 358 495T385 507Q403 506 409 494T415 457Q415 451 415 436T414 411V341H558Q579 329 579 311Q579 289 558 281L486 280H414V136Q400 114 384 114Q363 114 354 136L353 208V280H281L209 281Q187 289 187 310Q187 328 209 341H353V412",
      8849: "127 25Q111 29 104 49V362L105 675Q114 693 127 696H132Q138 696 149 696T174 696T208 696T249 696T297 697T350 697T407 697T468 697H806Q828 683 828 666Q828 646 806 637L485 636H165V85H805Q806 84 809 82T813 80T817 77T821 73T824 68T826 62T827 55Q827 34 806 25H127ZM96 -190T96 -169T119 -140H125Q131 -140 141 -140T167 -140T201 -140T242 -140T290 -139T344 -139T402 -139T463 -139H805Q806 -140 809 -142T813 -144T817 -147T821 -151T824 -156T826 -162T827 -169Q827 -190 806 -199H119Q96 -190 96 -169",
      8850: "66 55Q66 74 89 85H728V636H408L88 637Q66 645 66 667T88 696H94Q99 696 110 696T135 696T169 696T210 696T258 697T311 697T368 697T429 697H767Q786 684 789 672V49Q782 31 767 25H88Q66 32 66 55ZM88 -199Q66 -191 66 -169Q66 -148 87 -140Q91 -139 433 -139H775Q776 -140 779 -142T783 -144T787 -147T791 -151T794 -156T796 -162T797 -169Q797 -189 775 -199H88",
      8851: "131 25Q121 1 100 1Q81 1 71 23L70 301Q70 579 72 583Q77 598 90 602Q95 604 385 604H674Q693 591 696 579V25Q686 1 665 1Q646 1 636 23L635 283V543H131V25",
      8852: "696 25Q689 7 674 1H93Q77 7 71 23L70 301Q70 579 72 583Q80 604 100 604T131 579V61H635V579Q644 603 666 603Q687 603 696 579V25",
      8853: "64 250Q64 350 98 426T189 546T307 610T434 632Q485 632 496 631Q572 621 635 592Q669 575 699 550T760 484T809 384T828 250Q828 77 725 -27T446 -132Q272 -132 168 -27T64 250ZM416 282V570H414Q341 564 285 535T202 475T156 397T134 332T128 287Q127 283 127 282H416ZM765 288Q760 344 743 389T700 462T647 512T589 543T538 560T499 568L483 570H478V282H766L765 288ZM416 -69V220H127Q130 195 131 189T138 155T150 115T168 76T196 35T234 0T286 -35Q337 -61 410 -69H416ZM483 -69Q554 -60 607 -33T687 21T733 93T756 156T764 209Q766 217 766 220H478V-69H483",
      8854: "64 250Q64 350 98 426T189 546T307 610T434 632Q485 632 496 631Q572 621 635 592Q669 575 699 550T760 484T809 384T828 250Q828 77 725 -27T446 -132Q272 -132 168 -27T64 250ZM765 288Q753 424 666 497T446 571T227 498T128 288L127 282H766L765 288ZM446 -70Q578 -70 666 4T765 213L766 220H127Q130 195 131 189T138 155T150 115T168 76T196 35T234 0T286 -35Q353 -70 446 -70",
      8855: "64 250Q64 350 98 426T189 546T307 610T434 632Q485 632 496 631Q572 621 635 592Q669 575 699 550T760 484T809 384T828 250Q828 77 725 -27T446 -132Q272 -132 168 -27T64 250ZM647 512Q567 571 447 571Q340 571 262 523Q237 507 237 505L342 399L447 295L657 505L647 512ZM298 356L192 461Q180 445 161 411Q126 341 126 251Q126 128 192 40L403 250L298 356ZM701 41Q704 41 719 63T750 138T767 250Q767 310 750 362T719 437T701 460L491 250L701 41ZM238 -5Q238 -8 261 -22T336 -53T447 -70Q567 -70 647 -11L657 -4L447 206L342 101Q238 -1 238 -5",
      8856: "64 250Q64 350 98 426T189 546T307 610T434 632Q485 632 496 631Q572 621 635 592Q669 575 699 550T760 484T809 384T828 250Q828 77 725 -27T446 -132Q272 -132 168 -27T64 250ZM657 505Q656 506 650 510T638 518T623 527T604 537T581 547T553 556T522 563T486 569T446 571Q305 571 216 487T126 251Q126 128 192 40L657 505ZM447 -70Q591 -70 679 16T767 250Q767 308 751 360T719 436T701 460L469 228Q238 -1 238 -5Q238 -8 261 -22T336 -53T447 -70",
      8857: "64 250Q64 350 98 426T189 546T307 610T434 632Q485 632 496 631Q572 621 635 592Q669 575 699 550T760 484T809 384T828 250Q828 77 725 -27T446 -132Q272 -132 168 -27T64 250ZM767 252Q767 395 681 483T446 571Q303 571 215 486T126 249Q126 107 212 19T446 -70Q596 -70 681 18T767 252ZM335 251Q335 297 368 329T441 361Q498 361 527 327T557 250Q557 202 525 171T446 140Q397 140 366 173T335 251",
      8866: "65 672Q76 693 91 693Q115 693 123 674Q125 669 125 523V378H615Q618 376 622 373T628 369T632 366T635 362T636 356T637 347Q637 328 619 319Q615 317 370 317H125V171Q125 25 123 20Q114 1 94 1Q73 1 65 23V672",
      8867: "64 327T64 347T89 378H577V525L578 672Q592 693 604 693Q629 693 638 669V25Q628 1 607 1Q588 1 578 23L577 170V317H88Q64 327 64 347",
      8868: "64 664Q64 675 71 683T87 693H93Q99 693 110 693T137 693T173 693T217 694T267 694T323 694T383 694T448 694H807Q808 693 811 691T817 687T823 682T827 675T829 664Q829 643 807 634L642 633H477V25Q467 1 446 1Q427 1 417 23L416 328V633H251L87 634Q64 643 64 664",
      8869: "65 31Q65 38 66 41T71 50T87 61H416V366L417 672Q431 693 443 693Q468 693 477 669V61H807Q808 60 811 58T817 54T823 49T827 42T829 31Q829 10 807 1H87Q65 10 65 31",
      8872: "160 -249Q138 -249 129 -225V250Q129 725 131 729Q139 750 159 750T190 725V392Q219 393 537 393H896Q897 392 900 390T906 386T912 381T916 374T918 363Q918 345 896 333H190V169H896L900 166Q905 163 907 162T912 157T916 149T918 139Q918 118 896 109H190V-225Q181 -249 160 -249",
      8900: "280 522Q281 523 285 523H289Q301 523 366 457Q404 420 431 393Q533 291 546 277T560 250Q560 239 548 226T431 108Q313 -10 304 -16Q297 -21 287 -21Q278 -21 275 -19Q270 -17 146 107T18 238Q15 242 15 251Q15 258 18 263Q20 268 145 392T274 519L280 522ZM388 350L288 449L188 350L89 250L288 52L487 250L388 350",
      8901: "74 251Q74 286 99 311T156 336Q200 336 222 308T245 250Q245 221 224 194T160 166T96 193T74 251",
      8902: "270 491Q274 502 287 502Q298 502 304 491Q304 486 323 396T342 303L438 314Q520 324 534 324Q540 324 545 320T550 307Q550 298 539 290T456 243Q377 198 377 197L416 111Q456 26 456 22Q457 21 457 18Q457 11 451 6T438 0H437Q432 0 415 16Q387 42 358 68L287 133L216 68Q193 47 167 23Q142 0 136 0Q129 0 123 5T117 18Q117 21 118 22Q118 26 158 111L197 197Q197 198 156 221T72 269T26 298Q24 304 24 307Q24 315 29 319T40 324Q53 324 136 314L232 303Q232 306 251 396T270 491",
      8904: "906 251Q906 456 905 456Q550 252 549 251Q549 250 726 148T905 45T906 251ZM967 -14Q958 -38 939 -38H937Q928 -38 923 -35Q919 -34 748 64T500 209L71 -38Q69 -39 63 -39Q42 -39 33 -16V518Q45 540 63 540H65Q72 540 174 481Q247 439 302 407L500 292Q578 339 750 438T929 539H933Q958 539 967 515V-14ZM449 251L94 456Q93 456 93 251Q93 45 94 45L106 52Q119 59 139 71T186 98T242 131T301 165T357 197T404 225T437 244L449 251",
      8942: "74 55Q74 91 99 116T156 141Q200 141 222 113T245 55Q245 26 224 -1T160 -29Q118 -29 96 -3T74 55ZM74 465Q74 501 99 526T156 551Q200 551 222 523T245 465Q245 436 224 409T160 381Q118 381 96 407T74 465ZM74 865Q74 901 99 926T156 951Q200 951 222 923T245 865Q245 836 224 809T160 781Q118 781 96 807T74 865",
      8943: "74 251Q74 286 99 311T156 336Q200 336 222 308T245 250Q245 221 224 194T160 166T96 193T74 251ZM562 251Q562 286 587 311T644 336Q688 336 710 308T733 250Q733 221 712 194T648 166T584 193T562 251ZM1050 251Q1050 286 1075 311T1132 336Q1176 336 1198 308T1221 250Q1221 221 1200 194T1136 166T1072 193T1050 251",
      8945: "129 785Q129 821 154 846T211 871Q255 871 277 843T300 785Q300 756 279 729T215 701Q173 701 151 727T129 785ZM576 485Q576 521 601 546T658 571Q702 571 724 543T747 485Q747 456 726 429T662 401Q620 401 598 427T576 485ZM1023 185Q1023 221 1048 246T1105 271Q1149 271 1171 243T1194 185Q1194 156 1173 129T1109 101Q1067 101 1045 127T1023 185",
      8968: "194 728Q199 743 216 749H220Q223 749 229 749T245 749T265 750T289 750T316 750T345 750H471Q472 749 477 746T484 741T490 732T493 719Q493 701 471 690L362 689H254V-224Q244 -248 223 -248T194 -226V728",
      8969: "317 -224Q307 -248 286 -248Q267 -248 257 -226L256 231V689H148L40 690Q17 698 17 720Q17 741 37 748Q42 750 169 750H295Q314 737 317 725V-224",
      8970: "194 728Q204 749 220 749Q245 749 254 725V-188H471Q472 -189 477 -192T484 -197T490 -206T493 -219Q493 -237 471 -248H216Q200 -242 194 -226V728",
      8971: "17 -219Q17 -201 40 -188H256V270L257 728Q271 749 283 749Q308 749 317 725V-224Q310 -242 295 -248H40L38 -247Q35 -246 34 -245T30 -243T25 -239T21 -234T18 -227T17 -219",
      8994: "95 108Q85 108 75 114T65 139Q65 159 129 227Q316 405 573 405Q654 405 729 387T854 344T950 286T1015 232T1053 191Q1078 160 1083 152Q1084 148 1084 139Q1084 121 1074 115T1054 108Q1040 108 1029 122T990 167T922 223Q819 291 680 309Q641 315 575 315Q508 315 469 309Q303 288 197 201Q168 179 148 155T118 119T95 108",
      8995: "1054 392Q1067 392 1076 384T1085 362Q1085 351 1079 342T1050 310Q983 243 901 200Q753 126 575 126Q494 126 420 141T298 176T205 225T140 272T100 310Q64 346 64 362Q64 370 67 374Q75 393 93 393Q107 393 124 375Q272 214 575 214Q877 214 1025 375Q1039 392 1054 392",
      9001: "127 243V259L223 491Q251 557 286 642Q318 719 324 732T340 748H341Q347 750 351 750Q365 750 373 740T382 723Q382 713 286 482L190 251Q190 249 286 20T382 -219Q382 -232 373 -240T352 -249Q332 -249 323 -229Q320 -220 223 10L127 243",
      9002: "64 720Q64 732 72 741T94 750Q106 750 113 743Q118 741 122 732L319 259V243L122 -231Q112 -249 95 -249Q83 -249 74 -240T64 -218Q64 -210 160 20L256 251L160 482Q64 715 64 720",
      9651: "91 1Q69 10 69 31Q69 39 81 59T168 197Q327 447 485 697Q493 711 510 711Q523 711 532 702Q536 697 743 371T951 41Q953 35 953 31Q953 12 931 1H91ZM690 340Q651 401 604 476T534 586L512 621Q511 622 507 616Q498 604 332 342L154 62L333 61H689L867 62L690 340",
      9653: "91 1Q69 10 69 31Q69 39 81 59T168 197Q327 447 485 697Q493 711 510 711Q523 711 532 702Q536 697 743 371T951 41Q953 35 953 31Q953 12 931 1H91ZM690 340Q651 401 604 476T534 586L512 621Q511 622 507 616Q498 604 332 342L154 62L333 61H689L867 62L690 340",
      9657: "33 518Q45 540 63 540H65Q72 540 174 481Q247 439 302 407Q529 276 533 272Q542 263 542 250Q542 238 533 229Q528 224 304 95T71 -38Q69 -39 63 -39Q42 -39 33 -16V518ZM449 251L94 456Q93 456 93 251Q93 45 94 45L106 52Q119 59 139 71T186 98T242 131T301 165T357 197T404 225T437 244L449 251",
      9661: "68 470Q68 481 75 489T91 499H93Q296 500 512 500H931Q932 499 937 496T945 490T950 482T953 469Q953 465 951 459Q950 455 743 129T532 -202Q524 -210 511 -210Q497 -210 489 -202Q486 -199 281 124T71 456Q68 462 68 470ZM154 439Q155 437 332 158T510 -122Q510 -123 533 -87T600 18T688 157Q866 437 866 438Q867 439 805 439T511 439H154",
      9663: "68 470Q68 481 75 489T91 499H93Q296 500 512 500H931Q932 499 937 496T945 490T950 482T953 469Q953 465 951 459Q950 455 743 129T532 -202Q524 -210 511 -210Q497 -210 489 -202Q486 -199 281 124T71 456Q68 462 68 470ZM154 439Q155 437 332 158T510 -122Q510 -123 533 -87T600 18T688 157Q866 437 866 438Q867 439 805 439T511 439H154",
      9667: "542 -14Q533 -38 514 -38H512Q503 -38 498 -35Q494 -34 270 95T42 229Q33 238 33 251Q33 259 35 264Q36 265 38 268T42 272Q48 278 271 407T504 539H508Q533 539 542 515V-14ZM481 251Q481 456 480 456Q125 252 124 251Q124 250 301 148T480 45T481 251",
      9711: "65 42T65 250T204 584T574 711Q795 711 935 594Q955 577 974 555T1022 490T1067 385T1084 250Q1084 42 945 -84T574 -211T204 -85ZM1024 250Q1024 431 903 540T578 650Q482 650 404 627T274 565T189 474T140 366T125 250Q125 123 186 31T347 -106T573 -150Q772 -150 898 -45T1024 250",
      9824: "675 -18Q536 -18 527 62V70H477V55Q479 14 487 -21T502 -75T509 -101Q509 -120 491 -127Q487 -129 447 -129Q446 -129 439 -129T427 -130Q384 -130 384 -101Q384 -95 391 -76T406 -21T416 55V70H366Q364 52 360 40T342 14T300 -8T230 -17H218Q110 -17 75 117Q64 163 64 209Q64 290 116 357T261 495Q363 574 414 690Q425 719 445 719Q467 719 478 693Q507 627 547 578T623 503T702 438T777 357Q829 285 829 202V197Q826 128 808 81T762 15T714 -11T675 -18",
      9825: "65 491Q65 602 121 656T246 710Q375 710 440 624L447 615Q519 711 638 711Q723 711 775 652T828 491Q828 390 770 313T581 129Q539 95 514 63T483 14T469 -13T446 -24Q434 -24 427 -17T416 0T400 32T371 74Q352 97 310 131T229 199T151 276T89 374T65 491ZM249 649Q188 649 157 603T125 489Q125 409 181 338T352 176Q408 131 437 87L446 73L456 87Q479 121 507 147T579 207T659 278Q768 387 768 489Q768 506 766 524T756 566T731 611T687 642Q668 649 638 649Q609 649 593 644Q547 633 516 604T478 534Q473 505 447 505H445Q420 505 416 534Q407 577 372 608T285 648Q277 649 249 649",
      9826: "409 686Q410 688 412 691T415 696T418 701T421 706T424 709T427 713T431 715T435 717T440 718T446 719Q455 719 460 717T472 704T488 679T516 633T563 567Q624 485 687 422T787 330T826 296T828 282Q828 270 825 265T801 245Q696 161 612 59T477 -133Q465 -154 447 -154Q439 -154 434 -152T425 -146T414 -130T399 -104T372 -62T330 -3Q270 78 207 142T107 234T70 265Q64 274 64 282Q64 296 90 317Q284 472 409 686ZM749 282Q745 286 721 307T681 343T635 388T581 446T525 516T465 601Q462 606 457 613T450 624L447 627V628Q446 628 436 611T402 561T348 489T266 396T155 292L145 282Q147 280 185 245T257 177T343 79T442 -57Q446 -64 447 -64V-63Q450 -59 475 -22T530 56T619 160T749 282",
      9827: "240 527Q240 611 301 665T446 719T590 665T652 527Q652 431 571 373Q578 363 584 352T593 335T597 329L604 335Q611 341 617 345T637 356T667 366Q672 366 680 367T694 368Q767 368 814 310T861 177Q861 109 819 57T713 -12Q690 -17 656 -17Q535 -13 527 62V70H477V55Q479 14 487 -21T502 -75T509 -101Q509 -120 491 -127Q487 -129 447 -129Q446 -129 439 -129T427 -130Q384 -130 384 -101Q384 -95 391 -76T406 -21T416 55V70H366V62Q356 -12 237 -17Q130 -17 71 60Q32 111 32 178Q32 251 78 309T198 368Q217 368 233 364T260 354T279 343T291 333T296 329L300 336Q304 343 310 354T322 373Q240 432 240 527",
      9837: "230 480Q293 480 337 440T381 330V322Q381 240 323 161Q258 71 123 -11L114 -16L97 -17Q70 -17 66 -7Q64 -3 64 366V641Q64 717 65 731T75 748Q78 750 95 750Q117 750 122 742T127 694Q127 685 127 653T126 595V454Q183 480 230 480ZM242 333Q242 405 212 405H207Q147 405 130 370L127 364L126 219Q126 77 128 77Q133 82 140 90T167 127T202 183T229 253T242 333",
      9838: "345 -223Q333 -223 330 -214T327 -178V-116Q327 -23 326 -23L203 -82Q90 -134 77 -140Q65 -142 59 -130Q57 -126 57 295V595Q57 643 57 667T58 704T60 719T63 724Q93 741 101 741Q113 741 116 732T119 680V597Q119 467 120 467Q121 468 180 495T301 552T369 584Q381 586 387 574Q389 570 389 187V-88Q389 -132 389 -154T388 -188T386 -202T383 -206Q353 -223 345 -223ZM327 271Q327 421 326 421L120 323L119 173V23Q120 23 223 72L327 121V271",
      9839: "140 628Q151 628 154 620T158 591V549V484L166 488Q175 492 192 500T223 516L288 548V622V674Q288 681 288 685T289 693T289 699T291 703T295 707T298 709T304 712T311 716Q326 724 332 724Q343 724 346 715T350 685V644V579Q358 583 364 583Q376 583 380 574Q382 570 382 514V481Q382 459 380 454T363 441L350 435V135Q358 139 364 139Q376 139 380 130Q382 126 382 70V37Q382 15 380 10T363 -3L350 -9V-76Q350 -102 348 -106T328 -119Q312 -128 306 -128Q288 -128 288 -99V-77V-40L280 -44Q271 -48 254 -56T223 -72L158 -104V-150V-180Q158 -198 155 -202T135 -216Q119 -224 114 -224Q96 -224 96 -192V-172V-135Q86 -140 81 -140Q70 -140 66 -129Q64 -126 64 -70V-54Q64 -18 66 -12T83 3L96 9V309Q86 304 81 304Q70 304 66 315Q64 318 64 374V407Q64 429 66 434T83 447L96 453V602Q99 609 100 610T118 619Q134 628 140 628ZM288 254Q288 404 287 404L158 340V40L166 44Q175 48 192 56T223 72L288 104V254",
      10072: "160 -249Q138 -249 129 -225V250Q129 725 131 729Q139 750 159 750T190 725V-225Q181 -249 160 -249",
      10216: "127 243V259L223 491Q251 557 286 642Q318 719 324 732T340 748H341Q347 750 351 750Q365 750 373 740T382 723Q382 713 286 482L190 251Q190 249 286 20T382 -219Q382 -232 373 -240T352 -249Q332 -249 323 -229Q320 -220 223 10L127 243",
      10217: "64 720Q64 732 72 741T94 750Q106 750 113 743Q118 741 122 732L319 259V243L122 -231Q112 -249 95 -249Q83 -249 74 -240T64 -218Q64 -210 160 20L256 251L160 482Q64 715 64 720",
      10229: "1063 221L649 220H235Q340 133 364 17Q368 1 368 -2Q368 -16 343 -17Q340 -17 338 -17H332Q317 -17 314 -14T305 6Q298 34 285 62T247 126T179 189T78 233Q64 237 64 251Q64 261 74 265T108 277T154 297Q212 328 251 379T305 495Q309 511 313 514T333 518H338Q363 517 367 510Q368 507 368 503Q368 500 364 484Q345 401 287 331Q254 295 235 282L649 281H1063L1065 280Q1079 281 1392 281H1719Q1720 280 1723 278T1729 274T1735 269T1739 262T1741 251Q1741 230 1719 221H1063",
      10230: "119 221Q96 230 96 251T116 279Q121 281 448 281H1188L1602 282Q1569 304 1530 357T1473 482Q1469 500 1469 501Q1469 512 1480 517Q1482 518 1500 518H1505Q1520 518 1523 515T1532 495Q1552 412 1611 351T1753 270Q1773 263 1773 251Q1773 240 1765 236T1732 225T1683 204Q1625 173 1586 122T1532 6Q1528 -10 1524 -13T1504 -17H1499Q1474 -16 1470 -9Q1469 -6 1469 -2Q1469 1 1473 17Q1501 138 1602 220H1188L775 221H119",
      10231: "1063 221L649 220H235Q340 133 364 17Q368 1 368 -2Q368 -16 343 -17Q340 -17 338 -17H332Q317 -17 314 -14T305 6Q298 34 285 62T247 126T179 189T78 233Q64 237 64 251Q64 261 74 265T108 277T154 297Q212 328 251 379T305 495Q309 511 313 514T333 518H338Q363 517 367 510Q368 507 368 503Q368 500 364 484Q345 401 287 331Q254 295 235 282L649 281H1476L1890 282Q1857 304 1818 357T1761 482Q1757 500 1757 501Q1757 512 1768 517Q1770 518 1788 518H1793Q1808 518 1811 515T1820 495Q1840 412 1899 351T2041 270Q2061 263 2061 251Q2061 240 2053 236T2020 225T1971 204Q1913 173 1874 122T1820 6Q1816 -10 1812 -13T1792 -17H1787Q1762 -16 1758 -9Q1757 -6 1757 -2Q1757 1 1761 17Q1789 138 1890 220H1476L1063 221",
      10232: "1063 333L700 332H338L324 321Q283 290 230 264L205 250Q266 224 323 180L338 170L700 169H1063L1064 168Q1080 169 1423 169H1782L1786 166Q1791 163 1793 162T1798 157T1802 149T1804 139Q1804 118 1782 109H1063L733 108H404L412 99Q455 50 488 -10Q498 -27 493 -37Q487 -46 465 -46H460Q446 -46 439 -39T426 -18T399 25T344 89Q239 194 99 229Q96 230 92 231T85 232T79 234T73 235T69 237T66 240T65 244T64 250Q64 267 90 271Q197 295 286 361T430 525Q439 542 442 544T460 547H465Q487 547 492 539Q496 531 496 530Q496 521 471 482T414 405L404 394L733 393H1063Q1064 392 1065 392Q1081 393 1423 393H1782Q1783 392 1786 390T1792 386T1798 381T1802 374T1804 363Q1804 345 1782 333H1063",
      10233: "87 109Q64 118 64 139Q64 159 86 168Q89 169 448 169H1169L1532 170L1546 180Q1562 193 1580 204T1612 223T1638 237T1657 246L1664 250L1639 264Q1586 290 1545 321L1531 332H1169L807 333H87Q64 343 64 362Q64 383 84 391Q89 393 448 393H1136L1465 394L1455 405Q1428 436 1401 477T1374 531Q1374 547 1399 547H1404H1409Q1423 547 1430 540T1443 519T1470 475T1526 411Q1634 303 1779 271Q1780 271 1783 270T1788 269T1792 268T1796 266T1799 264T1802 260T1803 256T1804 250Q1804 242 1800 238T1783 231T1755 225T1712 211T1654 185Q1517 112 1439 -24Q1430 -40 1426 -43T1409 -46H1404Q1373 -46 1373 -31Q1373 -24 1381 -10Q1414 50 1457 99L1465 108H1136L807 109H87",
      10234: "1063 333L700 332H338L324 321Q283 290 230 264L205 250Q266 224 323 180L338 170L700 169H1425L1788 170L1802 180Q1818 193 1836 204T1868 223T1894 237T1913 246L1920 250L1895 264Q1842 290 1801 321L1787 332H1425L1063 333ZM733 393H1392L1721 394L1711 405Q1684 436 1657 477T1630 531Q1630 547 1655 547H1660H1665Q1679 547 1686 540T1699 519T1726 475T1782 411Q1890 303 2035 271Q2036 271 2039 270T2044 269T2048 268T2052 266T2055 264T2058 260T2059 256T2060 250Q2060 242 2056 238T2039 231T2011 225T1968 211T1910 185Q1773 112 1695 -24Q1686 -40 1682 -43T1665 -46H1660Q1629 -46 1629 -31Q1629 -24 1637 -10Q1670 50 1713 99L1721 108H1392L1063 109L733 108H404L412 99Q455 50 488 -10Q498 -27 493 -37Q487 -46 465 -46H460Q446 -46 439 -39T426 -18T399 25T344 89Q239 194 99 229Q96 230 92 231T85 232T79 234T73 235T69 237T66 240T65 244T64 250Q64 267 90 271Q197 295 286 361T430 525Q439 542 442 544T460 547H465Q487 547 492 539Q496 531 496 530Q496 521 471 482T414 405L404 394L733 393",
      10236: "65 426Q74 448 95 448Q112 448 125 426V281H130L132 280H134Q162 281 448 281H1188L1602 282Q1569 304 1530 357T1473 482Q1469 500 1469 501Q1469 512 1480 517Q1482 518 1500 518H1505Q1520 518 1523 515T1532 495Q1552 412 1611 351T1753 270Q1773 263 1773 251Q1773 240 1765 236T1732 225T1683 204Q1625 173 1586 122T1532 6Q1528 -10 1524 -13T1504 -17H1499Q1474 -16 1470 -9Q1469 -6 1469 -2Q1469 1 1473 17Q1501 138 1602 220H1188L775 221H135Q133 220 130 220H125V76Q115 54 95 54Q73 54 65 76V426",
      10744: "189 -210Q179 -210 170 -203T160 -179Q160 -171 162 -166Q164 -163 420 266T679 698Q686 711 704 711Q714 711 723 704T733 681Q733 672 730 667Q723 654 469 228T211 -201Q202 -210 189 -210",
      10799: "168 500Q168 515 178 522T195 530H198Q207 530 218 521T282 458Q312 428 331 409L447 294L563 409Q674 520 682 525Q687 529 695 529Q711 529 718 520T726 499V498Q726 489 720 481T666 427Q631 392 606 367L490 251L606 135Q717 23 721 17T726 2Q726 -9 719 -18T695 -28H692Q685 -28 674 -18T608 47Q581 74 563 92L447 207L331 91Q217 -22 208 -27Q206 -28 203 -28H197Q168 -28 168 2Q168 13 178 24T288 135L404 250L288 366Q177 479 173 485T168 500",
      10815: "39 655Q39 675 43 680T69 686Q110 684 225 684Q267 684 303 684T360 685T385 686Q401 686 405 680T409 651Q409 632 403 628T367 624H348H301V62H598V624H551H532Q502 624 496 628T490 651Q490 673 494 679T514 686Q518 686 558 685T675 684T792 685T836 686Q852 686 856 680T860 651Q860 632 854 628T818 624H799H752V62H799H809Q846 62 853 59T860 36V31V21Q860 6 850 2Q846 0 450 0H156Q75 0 60 1T40 11V18Q39 26 39 31Q39 54 44 58T82 63Q84 63 90 63T100 62H147V624H100H90Q53 624 46 627T39 650V655",
      10927: "796 54Q796 40 788 32T767 24Q741 24 735 57Q729 107 705 148T646 216T563 264T465 297T356 316T245 326T136 330H134Q96 330 96 360Q96 391 134 391H136Q193 392 239 394T347 403T457 421T556 453T643 502T703 571T735 664Q741 696 763 696Q781 696 789 686T797 667Q797 651 792 624T758 545T682 456Q605 396 481 365L462 360Q483 357 526 344T633 296T733 215Q767 173 781 128T796 54ZM119 -199Q96 -190 96 -169T116 -141Q121 -139 448 -139H775Q776 -140 779 -142T785 -146T791 -151T795 -158T797 -169Q797 -190 775 -199H119",
      10928: "127 24Q115 24 106 32T97 55Q97 95 124 156T211 265Q288 325 412 356L431 361Q410 363 367 376T259 425T160 506Q127 546 112 594T96 667Q96 679 104 688T126 697Q152 697 158 664Q164 614 188 573T247 505T331 456T429 424T539 405T649 395T758 391Q797 391 797 360Q797 330 761 330H758Q701 329 655 327T547 318T437 300T337 268T251 219T190 150T158 57Q151 24 127 24ZM119 -199Q96 -190 96 -169T116 -141Q121 -139 448 -139H775Q776 -140 779 -142T785 -146T791 -151T795 -158T797 -169Q797 -190 775 -199H119",
      12296: "127 243V259L223 491Q251 557 286 642Q318 719 324 732T340 748H341Q347 750 351 750Q365 750 373 740T382 723Q382 713 286 482L190 251Q190 249 286 20T382 -219Q382 -232 373 -240T352 -249Q332 -249 323 -229Q320 -220 223 10L127 243",
      12297: "64 720Q64 732 72 741T94 750Q106 750 113 743Q118 741 122 732L319 259V243L122 -231Q112 -249 95 -249Q83 -249 74 -240T64 -218Q64 -210 160 20L256 251L160 482Q64 715 64 720"
    }, {
      8243: "′′",
      8244: "′′′",
      8279: "′′′′",
      8602: "↚",
      8603: "↛",
      8622: "↮",
      8653: "⇍",
      8654: "⇎",
      8655: "⇏",
      8708: "∄",
      8716: "∌",
      8740: "∤",
      8742: "∦",
      8769: "≁",
      8772: "≄",
      8775: "≇",
      8777: "≉",
      8802: "≢",
      8813: "≭",
      8814: "≮",
      8815: "≯",
      8816: "≰",
      8817: "≱",
      8832: "⊀",
      8833: "⊁",
      8836: "⊄",
      8837: "⊅",
      8840: "⊈",
      8841: "⊉",
      8876: "⊬",
      8877: "⊭",
      8930: "⋢",
      8931: "⋣"
    }), bold$1;
  }
  var doubleStruck$1 = {}, doubleStruck = {}, hasRequiredDoubleStruck$1;
  function requireDoubleStruck$1() {
    return hasRequiredDoubleStruck$1 || (hasRequiredDoubleStruck$1 = 1, Object.defineProperty(doubleStruck, "__esModule", { value: !0 }), doubleStruck.doubleStruck = void 0, doubleStruck.doubleStruck = {}), doubleStruck;
  }
  var hasRequiredDoubleStruck;
  function requireDoubleStruck() {
    return hasRequiredDoubleStruck || (hasRequiredDoubleStruck = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.doubleStruck = void 0;
      var _ = requireDoubleStruck$1();
      Object.defineProperty(O, "doubleStruck", { enumerable: !0, get: function() {
        return _.doubleStruck;
      } });
    }(doubleStruck$1)), doubleStruck$1;
  }
  var frakturBold$1 = {}, frakturBold = {}, hasRequiredFrakturBold$1;
  function requireFrakturBold$1() {
    return hasRequiredFrakturBold$1 || (hasRequiredFrakturBold$1 = 1, Object.defineProperty(frakturBold, "__esModule", { value: !0 }), frakturBold.frakturBold = void 0, frakturBold.frakturBold = {
      33: [0.689, 0.012, 0.349],
      34: [0.695, -0.432, 0.254],
      38: [0.696, 0.016, 0.871],
      39: [0.695, -0.436, 0.25],
      40: [0.737, 0.186, 0.459],
      41: [0.735, 0.187, 0.459],
      42: [0.692, -0.449, 0.328],
      43: [0.598, 0.082, 0.893],
      44: [0.107, 0.191, 0.328],
      45: [0.275, -0.236, 0.893],
      46: [0.102, 0.015, 0.328],
      47: [0.721, 0.182, 0.593],
      48: [0.501, 0.012, 0.593],
      49: [0.489, 0, 0.593],
      50: [0.491, 0, 0.593],
      51: [0.487, 0.193, 0.593],
      52: [0.495, 0.196, 0.593],
      53: [0.481, 0.19, 0.593],
      54: [0.704, 0.012, 0.593],
      55: [0.479, 0.197, 0.593],
      56: [0.714, 5e-3, 0.593],
      57: [0.487, 0.195, 0.593],
      58: [0.457, 0.012, 0.255],
      59: [0.458, 0.19, 0.255],
      61: [0.343, -0.168, 0.582],
      63: [0.697, 0.014, 0.428],
      91: [0.74, 0.13, 0.257],
      93: [0.738, 0.132, 0.257],
      94: [0.734, -0.452, 0.59],
      8216: [0.708, -0.411, 0.254],
      8217: [0.692, -0.394, 0.254],
      8260: [0.721, 0.182, 0.593],
      58113: [0.63, 0.027, 0.587],
      58114: [0.693, 0.212, 0.394, { ic: 0.014 }],
      58115: [0.681, 0.219, 0.387],
      58116: [0.473, 0.212, 0.593],
      58117: [0.684, 0.027, 0.393],
      58120: [0.679, 0.22, 0.981],
      58121: [0.717, 0.137, 0.727]
    }), frakturBold;
  }
  var hasRequiredFrakturBold;
  function requireFrakturBold() {
    if (hasRequiredFrakturBold) return frakturBold$1;
    hasRequiredFrakturBold = 1, Object.defineProperty(frakturBold$1, "__esModule", { value: !0 }), frakturBold$1.frakturBold = void 0;
    var O = requireFontData(), _ = requireFrakturBold$1();
    return frakturBold$1.frakturBold = (0, O.AddPaths)(_.frakturBold, {
      33: "121 621Q121 657 132 673T177 689Q223 689 223 644V635Q223 604 222 595Q221 590 210 490T187 292T175 190V186L166 185L156 184Q156 185 139 393T121 621ZM107 47Q107 70 127 87T174 104Q201 104 221 89T241 48Q241 24 222 6T174 -12Q147 -12 127 6T107 47",
      34: "53 695Q74 695 90 679V622L65 433L52 432H39L27 516Q10 626 10 655Q10 680 26 688Q33 693 39 693Q49 695 53 695ZM151 668Q151 691 191 691Q217 691 224 685T231 661V652Q230 634 219 531L207 433L195 432Q183 432 183 433L168 541Q151 664 151 668",
      38: "290 -14Q186 -14 115 41T44 185Q44 222 54 249T88 300T131 336T189 371Q216 387 216 388Q185 459 185 510Q185 563 206 601T263 659T334 687T405 696Q476 696 503 668T531 603Q531 565 513 536T450 476Q423 459 370 432L334 413L354 384Q474 212 560 139L563 137Q611 185 611 250Q611 295 577 329Q549 356 496 357Q439 357 433 354Q432 354 432 379V403L437 402Q443 402 479 401T557 400Q653 400 735 403T831 407H836Q833 352 833 351L821 352Q809 352 792 352T756 352T720 353T696 354Q680 354 680 353L681 346Q682 339 683 327T685 306Q685 283 681 260T669 218T653 182T635 152T619 129T606 114L602 109Q604 107 618 99T659 81T707 71Q742 71 767 99T804 155L815 183Q815 184 821 183T833 180T839 177Q837 169 834 156T816 110T782 52T727 5T648 -16Q569 -16 499 35H498Q496 35 466 23T387 -1T290 -14ZM418 592Q418 617 398 639T352 661T302 642T278 574Q278 545 288 514T306 465T319 444Q342 456 353 463T382 488T409 529T418 584V592ZM159 239Q159 158 222 98T364 38Q386 38 447 57L469 63L434 98Q349 185 286 275Q258 316 238 345Q233 353 232 353Q159 316 159 239",
      39: "80 645T80 662T93 687T123 695Q158 695 158 659Q158 649 157 643L123 437Q123 436 114 436H104Q104 442 92 538Q80 645 80 662",
      40: "347 719Q325 708 311 698T272 656T233 580T207 455T195 267Q195 30 247 -79Q261 -110 291 -136Q320 -163 347 -172V-179Q347 -186 344 -186Q338 -186 328 -184T287 -165T230 -123Q134 -25 134 271Q134 417 158 514T226 662T335 734L346 737Q347 737 347 728V719",
      41: "264 262Q264 366 253 446T226 572T186 649T145 692T105 714V725Q105 735 107 735Q108 734 121 731T154 719T196 692T242 641T284 560T314 437T326 268Q326 112 299 7Q279 -78 239 -124T116 -185L105 -187V-179L106 -171L109 -169Q130 -161 138 -158T165 -146T190 -127T210 -101T229 -64T243 -12T255 58T261 148T264 262",
      42: "40 516L62 529Q85 542 110 556T140 574L126 582Q112 591 104 595T80 607T40 629Q53 642 57 645L65 652L78 642Q106 620 132 603L152 589V595Q152 630 149 681V692H179V689Q178 681 174 638T171 593Q173 593 240 639L258 652Q260 652 267 643L276 633L260 625Q190 587 175 576Q173 575 180 570Q183 569 186 567Q213 549 256 527L277 515L256 495Q246 501 228 515T194 539T170 554V543Q170 486 178 449H148V456Q152 492 152 550L151 562Q150 562 102 528L53 495Q40 514 40 516",
      43: "422 584L471 598Q472 598 472 440V282H837Q833 273 829 263L821 244L647 243H472V-63L448 -73L423 -82Q422 -82 422 81V243H239Q56 243 56 244Q60 253 65 263L73 282H422V584",
      44: "118 61Q118 80 135 93T169 107Q190 107 221 65T253 -23Q253 -39 251 -49T237 -80T198 -133Q148 -191 144 -191Q142 -191 137 -182T132 -172Q143 -161 160 -131T183 -83Q185 -77 185 -62Q185 -54 184 -48T182 -38T177 -28T171 -19T162 -8T150 6Q130 28 124 38T118 61",
      45: "54 236L73 275H453Q833 275 833 274Q830 265 825 255L818 236H54",
      46: "103 23T103 44T120 83T170 102Q200 102 218 84T237 44Q237 20 216 3T168 -15Q138 -15 121 4",
      47: "272 270Q503 721 506 721L509 720Q512 720 518 719T529 717L550 713L91 -181L66 -182Q41 -182 41 -181L272 270",
      48: "238 -12Q162 -12 102 42T42 185Q42 303 130 393Q163 425 208 452T284 490L313 501Q323 499 339 495T395 472T464 426Q533 357 533 273Q533 201 483 133T364 27T238 -12ZM428 208Q428 255 402 297T342 365T280 404T241 419Q214 419 178 374T142 259Q142 206 168 164T225 99Q259 74 310 74Q326 74 337 75T366 82T396 103T417 141Q428 171 428 208",
      49: "95 481Q102 481 217 485T383 489Q384 489 384 485Q367 397 367 165Q367 58 369 54Q374 46 380 44T410 42H466H546V40Q547 38 547 19L548 0H54V23Q54 29 54 34T54 44L55 47Q79 47 134 46T202 45Q226 45 234 52Q240 57 241 64T245 105Q254 236 254 320V347Q254 369 252 382T240 409T211 431L97 450L96 465Q95 480 95 481",
      50: "307 335Q307 374 283 397T224 421Q187 421 112 387Q105 384 100 382T95 381Q90 387 86 394L77 407L86 413Q219 491 298 491Q370 491 399 460T428 388Q428 373 424 358T409 326T391 297T363 264T335 235T301 202T269 171L199 104Q194 99 205 97Q209 96 214 96Q527 105 544 105Q553 107 563 102Q563 100 557 79T545 34T537 2H377Q338 2 247 2T130 4H44V26L104 77Q185 145 212 172T267 235Q307 291 307 335",
      51: "102 402L108 408Q115 413 122 418T141 431T165 447T194 461T227 474T263 483T302 487H307Q413 487 452 420Q465 400 465 371Q465 334 445 303T396 253T347 225T317 213Q314 213 314 211Q316 209 316 205Q317 201 320 201Q337 201 359 198T411 184T465 156T506 109T523 39Q523 -62 436 -127T229 -193Q179 -193 130 -178T56 -150T31 -133Q31 -132 41 -122L52 -112L63 -117Q128 -148 201 -148Q282 -148 331 -104T381 20Q381 71 363 100T304 145Q243 166 149 166H137V204H146Q179 204 211 210T275 229T326 268T346 329Q346 372 314 401Q292 423 245 423Q188 423 125 383L102 402",
      52: "346 -196Q344 -196 335 -187L336 -148Q337 -127 337 -55V0H13V29L187 253Q362 477 362 479L368 480Q375 481 387 483T411 487T434 491T452 494L459 495Q460 495 470 482V453Q470 389 466 230T461 62Q461 61 513 61T565 60L555 29L546 -1H461V-15Q461 -48 463 -100T465 -154L457 -157Q449 -160 434 -165T405 -175Q347 -196 346 -196ZM339 265V341Q339 362 335 362Q327 362 219 217T110 65V61H337V117Q338 133 338 187T339 265",
      53: "232 192Q176 192 122 152L95 162V481H306Q516 481 516 479Q514 477 501 433L486 389L319 388H152V386V382Q152 379 152 374T151 365Q147 329 146 260V218H149Q211 242 284 242Q353 242 402 224T474 176T508 117T518 55Q518 -62 432 -126T220 -190Q184 -190 151 -185T96 -172T57 -157T31 -145T20 -139T19 -138Q19 -136 27 -125L35 -112L51 -120Q114 -152 174 -152Q257 -152 314 -100T371 46Q371 107 340 149T232 192",
      54: "48 251Q48 330 76 403T150 529T253 623T370 683T485 704Q494 704 520 701T547 695Q547 692 542 659T536 625Q531 624 524 624L512 623L502 628Q489 635 468 640Q452 645 423 645Q403 645 379 640T320 617T255 568T201 481T171 348Q170 341 170 330V325L183 333Q275 385 357 385H361Q464 385 514 312Q546 267 546 217Q546 127 457 58T262 -12Q225 -12 189 3T120 49T68 132T48 251ZM448 165Q448 228 406 274T289 320Q264 320 236 312T190 295T173 284Q173 266 176 241T189 178T214 112T259 61T326 39Q372 39 410 75T448 165",
      55: "57 376L87 479H591V455L584 446Q544 399 491 328T349 117T185 -169L171 -196H159Q152 -197 102 -197Q58 -197 58 -196T56 -185L54 -175L299 158L443 359Q446 367 444 370H254L71 365L57 376",
      56: "88 533Q88 573 120 610T194 668T268 701T307 714Q324 714 352 711T422 695T486 659Q518 625 518 585Q518 536 479 489T384 406L371 398L385 390Q387 389 400 382T420 370T442 356T466 339T489 319T510 295T526 269T538 238T542 204Q542 125 463 60T256 -5Q145 -5 92 52Q45 97 45 165Q45 204 64 237T109 290T163 324T209 345T228 353L214 364Q199 375 179 392T138 431T103 480T88 533ZM405 557Q405 568 402 581T387 612T350 644T286 663Q283 663 280 663T274 664H272Q256 664 228 636T199 572Q199 547 238 507Q268 475 320 437L334 427Q345 433 358 443T388 483T405 549V557ZM304 42Q366 42 398 76T431 155Q431 178 420 200T396 238T359 270T321 296T283 318L263 328Q262 328 230 312Q190 290 175 266T160 198Q160 132 202 87T304 42",
      57: "549 220Q549 23 429 -82T105 -195H84V-189Q84 -179 85 -174V-164H93Q184 -156 238 -132T334 -56Q361 -23 376 16T394 78L397 100L363 88Q329 75 291 61T244 45Q237 44 218 44Q154 44 94 97Q29 152 29 240Q29 350 108 404Q145 429 257 480Q270 487 279 487Q403 487 470 421Q549 347 549 220ZM408 217Q408 276 390 320T346 385T297 415T259 424Q218 424 185 393T151 286Q151 216 213 154Q252 115 321 115Q368 115 388 134T408 217",
      58: "57 398Q57 419 72 438T117 457Q154 457 174 439T194 398Q194 379 176 361T119 343Q85 343 71 362T57 398ZM62 19T62 43T77 85T115 104Q153 104 175 86T197 42Q197 14 171 1T119 -12Q96 -12 79 3",
      59: "56 399Q56 424 73 440T104 456Q114 458 120 458Q149 458 170 440T192 399Q192 380 174 362T120 344Q85 344 71 362T56 399ZM78 53Q78 67 84 76T90 86Q90 88 98 92T116 98Q117 98 121 98T128 99Q152 97 181 58T211 -24Q211 -77 128 -165Q124 -170 121 -173T116 -178T113 -181T110 -185T106 -190L97 -184L88 -177L95 -168Q143 -104 143 -65Q143 -51 137 -40T113 -7T81 35Q78 41 78 53",
      61: "559 342L549 304H22L27 319Q29 328 30 333T33 343H296Q559 343 559 342ZM559 206L549 168H22L27 183Q29 192 30 197T33 207H296Q559 207 559 206",
      63: "121 590Q121 575 128 562T144 542T152 533T115 512L78 491Q55 499 47 516Q40 530 40 553Q40 601 77 632Q155 697 257 697H268Q316 697 355 679Q422 646 422 576Q422 518 388 476Q383 468 376 461T358 444T340 428T316 410T290 390L230 344Q180 307 180 275Q180 261 187 248T202 227L209 219Q209 215 176 193L142 170Q114 177 100 194T84 226V239Q84 259 93 276T113 302T150 331T192 362Q203 370 219 382T247 403T267 422Q312 471 312 546Q312 593 282 623T207 653Q170 653 146 636T121 590ZM95 23T95 49T117 94T173 113Q204 113 223 96T242 54Q242 27 221 7T167 -14Q136 -14 116 4",
      91: "226 711T225 711T86 699V-93H89Q94 -93 157 -96T223 -100H226V-119H223Q134 -119 42 -130H36V740H42Q61 738 156 736H226V723Q226 711 225 711",
      93: "69 732Q116 733 146 734T184 736T197 737T206 738H208V-132Q190 -129 160 -127T99 -125T66 -124H14V-103H19Q20 -103 84 -98T152 -92H158V699H151Q148 700 85 703T18 708H14V732H69",
      94: "1 463T1 464T148 599T296 734Q584 486 584 485L561 472Q538 459 537 461Q296 672 293 672L161 563Q133 539 97 509T44 466L28 452Q27 452 14 457",
      8216: "187 456Q187 437 169 424T138 411Q114 411 84 454T53 538Q53 565 75 597Q109 648 155 697L166 708L181 694L173 681Q124 610 124 577Q124 549 155 511T187 456",
      8217: "125 524Q125 545 92 588T58 651Q58 661 61 667Q65 674 80 683T107 692Q131 692 162 645T193 564Q193 540 176 509T144 460T87 394L78 400L68 406L79 421Q125 489 125 524",
      8260: "272 270Q503 721 506 721L509 720Q512 720 518 719T529 717L550 713L91 -181L66 -182Q41 -182 41 -181L272 270",
      58113: "388 427Q320 485 242 524T128 563H116Q95 563 87 561L77 559Q72 563 69 566T65 570T65 572L75 576Q106 592 154 611T212 630Q230 630 262 622T358 581T492 498L508 486Q512 463 512 396Q512 246 469 112L465 102Q453 94 341 25Q252 -27 247 -27Q243 -27 174 24T97 84Q90 100 90 214Q90 285 98 345Q100 360 102 363T118 377Q175 422 262 465Q264 463 270 460L277 456Q277 455 267 447T244 428T228 414Q206 382 206 269Q206 187 214 164T259 110Q286 89 342 58Q391 131 391 313Q391 355 388 412V427",
      58114: "39 362L37 366L38 368L82 405H133V474Q135 563 143 589T198 658Q210 669 224 676T247 687L255 690H253Q241 690 253 692Q254 692 256 692T260 693Q263 693 262 691L261 690Q300 690 361 662L373 656L388 666Q404 675 405 675L406 674Q406 672 406 670T406 664L408 655L301 555Q300 555 287 564T254 584T221 597Q190 597 176 583T161 550Q161 525 184 495T232 440T261 405H387V399Q377 389 364 379L340 359H258V315Q258 52 228 -18L172 -120L121 -211H109Q102 -212 96 -212L109 -174Q131 -108 135 -80T139 53V76V157V362H39",
      58115: "41 352Q40 354 39 355T37 358L36 360H37Q48 370 61 380L84 400H108Q131 400 131 402Q121 424 104 501L100 519Q109 560 134 602T196 664Q230 681 271 681Q291 681 316 669T358 644L373 631Q373 630 304 553Q299 548 294 547Q292 547 290 546H287Q286 546 274 562T243 593T205 609Q180 609 165 596T150 562Q150 526 191 488L217 462Q248 431 253 405V400H381L384 394L349 352H251V332Q249 271 231 17L227 -37L120 -217L109 -218Q103 -219 97 -219Q97 -218 101 -206T110 -177T118 -151Q126 -129 128 -120T136 -46T141 127Q141 250 136 340V352H41",
      58116: "107 370Q127 384 172 409T255 454T294 473L306 468Q356 446 425 431L435 429L524 468Q528 465 531 461Q499 395 499 271V263Q499 146 509 71T519 -8Q519 -28 512 -45Q510 -50 435 -123T355 -197Q296 -212 257 -212Q209 -212 164 -196T98 -167T67 -143L133 -44H144Q167 -88 216 -111T320 -134Q371 -134 390 -118T410 -69Q410 -52 404 -12T392 60T385 92L193 -29L158 5Q124 39 110 51L96 63V71Q94 79 94 121Q94 130 94 148T93 174Q93 230 96 275T103 344T107 370ZM221 397Q200 334 200 254Q200 170 210 140Q216 126 234 109T268 81L283 71L383 119V127Q384 132 384 241L385 347L368 349Q325 357 290 369T240 389T221 397",
      58117: "103 453Q103 631 95 661Q95 663 102 667T110 672L114 664Q117 655 123 641T131 621L140 597L154 606Q208 641 275 673L297 684Q300 683 302 682T307 679T310 678L314 676Q283 658 256 625Q238 601 231 579T223 515L224 512L282 548Q339 583 341 583T365 548T386 509Q326 443 318 443L316 446Q314 448 311 452T304 460T294 470T283 480T272 488T260 494T248 497Q231 497 223 474Q220 468 218 440T215 407V401H345L309 360H218V314Q218 181 221 139V129L253 108Q306 73 310 73Q315 73 343 83L373 92L374 87Q375 82 375 79T375 74T360 65T308 36T229 -13L208 -27L192 -13Q149 24 90 61Q89 61 89 62L90 68Q91 73 93 87T97 125T100 191T103 291V360H33V366L34 371L85 405H94L103 404V453",
      58120: "602 575Q505 508 505 489Q505 488 505 482T506 463T507 432Q507 314 456 237L449 226L434 216Q420 208 325 143L316 137Q453 82 488 82Q527 82 585 127L596 136Q597 136 599 126L602 115Q578 85 511 27T428 -31Q400 -31 308 10T170 51Q143 51 123 43T92 24T54 -15L34 6L41 14Q65 41 170 129L188 144L204 145Q254 147 293 164T350 208Q378 249 378 344Q378 422 362 478T320 563T268 605T213 618Q177 618 156 600T134 561Q134 539 162 508T217 446T245 394Q245 368 213 337T85 250L62 262Q73 269 86 279T116 308T133 338T108 378T57 439T32 499Q32 556 117 617T291 679Q350 679 393 658Q415 647 433 631T462 600T480 572T490 550T494 541T499 544T516 556T547 578T603 613T689 662L720 679L730 670Q742 659 756 649T785 629T810 615T836 601T855 590Q855 587 860 536T870 419T875 312Q875 114 800 -25Q794 -35 781 -47Q584 -220 398 -220Q322 -220 278 -190Q253 -173 239 -155L244 -150Q248 -145 255 -138T271 -120T290 -100T310 -80T328 -63T341 -51T349 -46Q350 -46 351 -46T354 -47Q357 -47 357 -52Q359 -68 364 -83T383 -118T424 -151T491 -166Q559 -166 613 -129Q629 -118 641 -108T674 -68T710 1T735 107T746 260Q746 433 727 507Q727 512 685 535T615 570L602 575",
      58121: "351 571Q317 571 247 563T171 555Q153 555 133 563T107 584Q94 605 98 609Q101 615 138 658T190 717H207Q204 710 204 699Q204 673 231 666Q235 665 264 665Q296 665 345 667T426 669Q474 669 501 660T545 626Q553 612 553 594Q553 531 498 474T379 384Q371 379 371 378Q371 376 390 376H411H434Q520 376 602 318Q621 303 627 288T633 234Q633 59 540 -34Q465 -109 348 -130Q308 -137 235 -137Q159 -136 143 -129Q132 -125 132 -118V-53Q118 -24 90 -24Q69 -24 37 -39L27 -44L25 -42Q23 -39 21 -35T17 -30Q17 -28 40 -14T103 19T177 44Q183 45 205 45Q219 45 227 44T245 37T259 20T264 -12Q264 -33 262 -48T259 -80Q259 -93 260 -95Q271 -110 305 -110Q343 -110 383 -86T443 -33Q491 34 491 154Q491 223 467 249Q428 288 334 288H322Q288 288 237 276L222 273L206 286L262 367Q279 369 303 377T358 403T410 452T431 524Q431 531 431 533T427 545T416 558T392 566T351 571"
    }, {}), frakturBold$1;
  }
  var fraktur$1 = {}, fraktur = {}, hasRequiredFraktur$1;
  function requireFraktur$1() {
    return hasRequiredFraktur$1 || (hasRequiredFraktur$1 = 1, Object.defineProperty(fraktur, "__esModule", { value: !0 }), fraktur.fraktur = void 0, fraktur.fraktur = {
      33: [0.689, 0.012, 0.296],
      34: [0.695, -0.432, 0.215],
      38: [0.698, 0.011, 0.738],
      39: [0.695, -0.436, 0.212],
      40: [0.737, 0.186, 0.389],
      41: [0.735, 0.187, 0.389],
      42: [0.692, -0.449, 0.278],
      43: [0.598, 0.082, 0.756],
      44: [0.107, 0.191, 0.278],
      45: [0.275, -0.236, 0.756],
      46: [0.102, 0.015, 0.278],
      47: [0.721, 0.182, 0.502],
      48: [0.492, 0.013, 0.502],
      49: [0.468, 0, 0.502],
      50: [0.474, 0, 0.502],
      51: [0.473, 0.182, 0.502],
      52: [0.476, 0.191, 0.502],
      53: [0.458, 0.184, 0.502],
      54: [0.7, 0.013, 0.502],
      55: [0.468, 0.181, 0.502],
      56: [0.705, 0.01, 0.502],
      57: [0.469, 0.182, 0.502],
      58: [0.457, 0.012, 0.216],
      59: [0.458, 0.189, 0.216],
      61: [0.368, -0.132, 0.756],
      63: [0.693, 0.011, 0.362],
      91: [0.74, 0.13, 0.278],
      93: [0.738, 0.131, 0.278],
      94: [0.734, -0.452, 0.5],
      8216: [0.708, -0.41, 0.215],
      8217: [0.692, -0.395, 0.215],
      8260: [0.721, 0.182, 0.502],
      58112: [0.683, 0.032, 0.497],
      58113: [0.616, 0.03, 0.498],
      58114: [0.68, 0.215, 0.333],
      58115: [0.679, 0.224, 0.329],
      58116: [0.471, 0.214, 0.503],
      58117: [0.686, 0.02, 0.333],
      58118: [0.577, 0.021, 0.334, { ic: 0.013 }],
      58119: [0.475, 0.022, 0.501, { ic: 0.013 }]
    }), fraktur;
  }
  var hasRequiredFraktur;
  function requireFraktur() {
    if (hasRequiredFraktur) return fraktur$1;
    hasRequiredFraktur = 1, Object.defineProperty(fraktur$1, "__esModule", { value: !0 }), fraktur$1.fraktur = void 0;
    var O = requireFontData(), _ = requireFraktur$1();
    return fraktur$1.fraktur = (0, O.AddPaths)(_.fraktur, {
      33: "102 582T102 620T112 673T152 689Q190 689 190 638Q190 605 167 373L148 187L133 184Q102 582 102 620ZM91 24T91 48T107 88T148 104Q171 104 187 87T204 48Q204 22 188 5T149 -12Q124 -12 108 6",
      34: "33 436Q8 603 8 648Q8 663 9 671T19 687T43 695Q63 695 74 681Q76 678 76 650V623L66 532Q57 443 55 436V432H33V436ZM128 666Q128 691 162 691T196 668Q196 634 186 531Q176 441 176 432H166Q155 432 155 434L142 545Q135 603 130 647Q128 664 128 666",
      38: "181 520Q181 604 231 650T328 697L330 698Q333 698 335 698Q336 698 340 698T346 697Q390 697 418 670T446 604Q446 554 414 511Q384 467 300 417L283 406Q281 405 296 374T347 286T425 182Q466 135 469 135Q470 135 473 140T480 152T486 165Q509 210 509 263Q509 282 507 292Q498 317 488 332T465 352T443 359T418 361Q388 361 357 358L346 356L347 374Q347 394 348 396V399H355Q366 396 535 396Q663 396 689 398L703 399Q703 398 702 375T700 351L688 353Q655 356 601 357Q553 357 553 355Q562 332 562 294Q562 280 561 267T555 241T548 218T539 195T529 175T518 156T508 141T497 126L489 115L496 107Q549 54 596 54Q623 54 644 67T677 101T697 140T708 174L710 187Q710 188 710 188L711 190Q714 190 723 187T733 183Q733 179 732 172T726 145T714 107T694 66T665 27T623 0T569 -11Q500 -11 443 41L434 49L425 42Q348 -9 275 -9Q251 -9 233 -6Q187 -1 152 18T98 60T67 111T52 159T49 195Q49 231 60 261T96 315T141 354T197 390L208 397Q181 459 181 520ZM374 577Q374 607 356 629T311 651Q289 651 271 636Q242 614 242 559Q242 533 249 502T262 453T270 435T279 440T298 453T314 464Q374 503 374 577ZM127 217Q127 145 173 89T291 32Q318 32 347 42T393 63T409 75Q384 100 325 184T227 351Q223 366 220 366Q215 366 178 338Q127 298 127 217",
      39: "69 666Q69 680 79 687T104 695Q112 695 117 694T128 684T134 659L104 438Q104 436 96 436T88 438Q88 447 79 540T69 655V666",
      40: "293 737V719Q291 718 285 714T276 708T268 702T258 695T250 686T241 674Q164 570 164 282Q164 -37 230 -119Q273 -171 292 -171Q293 -171 293 -179V-186H289Q270 -180 259 -175Q175 -132 145 -23T114 275Q114 491 157 598T293 737",
      41: "223 259Q223 386 212 474T178 609T136 677T89 714V735Q121 726 139 713Q276 622 276 273Q276 98 246 -23Q225 -106 189 -139T89 -187V-170L101 -165Q148 -146 172 -110T210 13T223 259",
      42: "118 573Q118 575 76 602L34 629L50 647L55 652L66 642Q76 633 91 620T117 598L128 590Q128 663 125 687V692H151V689V687Q151 685 151 683T150 678T150 672Q145 628 144 605V592L219 652L232 634Q232 632 220 624Q195 610 155 582L146 575L158 567Q178 552 197 540T225 523T233 516Q233 515 226 506T217 495L152 549L143 555V530Q144 509 145 492T148 466T149 454Q149 449 142 449H125V462Q128 522 128 549V562L115 552Q88 531 59 506L45 495Q44 495 39 505T34 516L118 573",
      43: "357 584L399 598V282H709L702 263L696 243H399V-64L379 -73L358 -82Q357 -82 357 81V243H47L51 253Q52 256 54 263T58 272L62 282H357V584",
      44: "99 62Q99 82 114 94T144 107Q159 107 178 77T205 26Q213 5 213 -23Q213 -49 207 -65T181 -113Q128 -189 122 -191Q121 -191 116 -184T111 -174Q111 -173 122 -155T145 -111T156 -62Q156 -44 152 -34T127 4L104 37Q99 49 99 62",
      45: "46 236L62 274Q62 275 384 275H706L699 255L693 236H46",
      46: "87 43Q87 69 104 85T142 102Q164 102 182 86T200 44Q200 20 183 3T141 -15Q118 -15 103 2T87 43",
      47: "230 270Q426 721 428 721Q437 719 447 717L466 713L448 672Q78 -180 77 -181Q77 -182 55 -182L34 -181L230 270",
      48: "212 -13Q184 -13 156 -2T101 32T59 97T42 195Q42 311 132 396Q170 433 211 462T262 492Q272 492 301 477T366 434T429 360T456 258Q456 161 378 74T212 -13ZM381 209Q381 257 365 295T328 355T282 390T243 408T223 413Q220 413 204 403T167 376T137 341Q119 305 119 250Q119 168 159 114T263 59Q308 59 344 93T381 209",
      49: "123 459Q145 459 170 460T217 462T256 464T284 466L295 467Q296 467 296 467T297 468Q299 468 302 466T307 462L309 459Q307 454 304 424T299 341T297 235Q297 139 298 101T302 55T313 44Q316 43 367 43L460 46Q460 35 459 22V-1H450Q402 2 281 6Q222 6 171 4T91 1T56 -1L47 -2V43H121H170Q195 43 201 45T209 56Q212 69 212 214Q212 333 209 365T194 409Q183 417 161 423T121 430L104 432Q103 432 103 446V459H123",
      50: "104 384Q115 394 133 409T199 449T281 474Q321 474 351 447T385 378Q385 328 333 255T228 127T176 72Q176 67 183 65Q184 65 203 65T260 67T331 69L475 73L484 67Q484 64 472 33L460 1H60V17L107 61Q210 159 249 208Q309 283 309 331Q309 363 285 389T228 415Q212 415 184 403T134 379L114 367L104 384",
      51: "305 328Q305 372 279 396T214 421H211Q172 421 128 384L107 398L116 405Q151 437 191 455T251 473H260Q314 473 341 455T382 394Q384 386 384 367T382 338Q362 263 271 217L256 210L257 206L259 202Q260 202 272 201T296 198T324 192T355 179T384 157T410 123T427 75Q429 64 429 41Q429 -59 353 -120T183 -182L88 -164Q81 -162 69 -157T48 -147T39 -141Q39 -139 46 -127L53 -114L69 -122Q129 -149 171 -149Q218 -149 253 -131T305 -83T330 -26T338 29Q338 41 336 55T328 89T308 127T273 153Q228 171 162 171Q158 171 152 171T142 170H127V204H134Q232 214 275 257Q305 292 305 328",
      52: "299 -179Q306 -156 306 -48V0H11V7Q10 10 10 18Q10 23 154 236L298 449Q298 450 339 463L379 476Q385 473 384 470V466Q384 463 384 457T384 444T383 427T383 408Q381 328 381 248Q381 46 384 40H387Q422 40 460 44Q465 44 470 44T478 44L481 45Q481 43 478 24T473 1Q473 -1 464 -1Q462 -1 451 -1T430 0H387V-76L389 -156V-161L311 -191Q299 -181 299 -179ZM299 364H287L277 352Q234 297 186 224T112 104T79 43Q79 42 192 42H306V115Q306 300 299 359V364",
      53: "334 25Q334 99 296 134T207 169Q154 169 107 123L98 114L89 120L80 125V458H420Q420 456 409 418L397 379Q397 378 264 378H131Q130 377 128 376T125 374T124 371T122 368T122 363T121 356T121 345V279V190L130 186L140 190Q196 214 260 214Q311 214 348 197T404 153T431 99T440 42T433 -16T406 -76T356 -130T276 -169T163 -184H156Q110 -184 57 -163L47 -159L53 -147L58 -134Q61 -134 74 -139T110 -148T156 -153Q206 -153 243 -135T299 -87T326 -30T334 25",
      54: "45 240Q45 328 73 406T143 536T235 626T327 681T399 699Q400 699 404 699T411 700Q424 700 441 696T459 689Q459 671 451 637Q451 633 447 632L444 629L434 633Q413 640 384 640H377Q299 640 222 565Q182 531 156 463T129 315V306H136L149 315Q229 376 316 376H318Q393 376 432 326T471 213Q471 129 402 58T237 -13T93 59T45 240ZM391 172Q391 231 354 272T258 314Q230 314 200 302T154 279T133 262L134 249Q154 32 266 32Q315 32 353 64T391 172",
      55: "395 377L391 382H225Q59 382 59 383L74 423Q89 464 89 465Q90 468 94 468Q146 460 350 458H498V442L473 406Q241 75 125 -156L113 -181H40L37 -168L57 -140Q115 -58 199 70T339 287T395 377",
      56: "220 -10Q168 -10 131 6T75 50T48 103T40 157Q40 223 77 266Q103 295 156 328T225 375Q247 393 247 394L206 361Q205 361 193 368T164 391T131 426T102 474T90 531Q90 580 114 615Q146 660 238 698L254 705L262 704Q288 704 332 693T402 656Q434 620 434 568Q434 518 401 475T321 402L305 391L336 368Q339 366 353 356T372 343T389 330T406 316T420 301T434 283T445 265T454 244T458 222T461 195Q461 106 389 48T220 -10ZM350 545Q350 578 337 601T304 634T266 649T234 653L224 654L204 639Q196 634 191 629T182 621T176 614T173 609T170 603T168 597Q165 585 165 567Q165 497 261 424L273 415Q350 467 350 545ZM261 405L263 407Q262 407 261 405ZM258 403Q257 403 255 401L254 399L256 400Q258 402 258 403ZM252 398Q251 398 249 396L248 394L250 395Q252 397 252 398ZM245 36Q276 36 300 45T338 69T360 102T371 136T374 168Q374 211 341 255Q324 275 305 289T235 332Q231 330 215 321T193 307T173 292T153 271T138 247T127 216T123 177Q123 146 132 117T170 62T245 36",
      57: "353 93T352 93T320 79T251 49T201 34Q127 37 87 79Q28 138 28 234Q28 273 37 304T60 355T101 396T152 429T218 462L234 469H243Q348 461 395 417Q466 348 466 201Q466 72 397 -29T211 -163Q155 -179 91 -182H72V-154H80Q144 -154 202 -131T297 -60Q318 -31 333 7T352 68L357 92Q353 93 352 93ZM369 208Q369 240 362 272T339 339T290 394T214 415Q171 415 144 372T116 266Q116 193 154 144T238 95H249Q369 95 369 208",
      58: "50 377T50 400T64 440T99 457Q128 457 146 440T165 399Q165 375 146 359T102 342T64 359ZM53 19T53 43T66 86T103 105Q129 105 148 87T168 41Q168 17 147 3T102 -12Q80 -12 67 3",
      59: "47 399Q47 424 62 441T101 458T143 442T162 400T144 359T101 343Q78 343 63 360T47 399ZM76 86Q76 88 80 91T91 96T106 99Q119 99 131 86Q179 35 179 -25Q179 -64 146 -115T89 -189Q86 -187 83 -185T79 -182T76 -180T75 -177T77 -173T80 -168Q121 -108 121 -64Q121 -44 94 -5T66 52Q66 66 71 75T76 86",
      61: "725 366Q724 365 721 349T716 331V329H385Q54 329 54 331Q55 332 59 349T63 368H394Q725 368 725 366ZM725 169Q724 168 721 152T716 134V132H385Q54 132 54 134Q55 135 59 152T63 171H394Q725 171 725 169",
      63: "46 557Q46 613 103 653T227 693Q287 693 322 659T357 564Q357 517 326 469T259 390T191 326T160 272Q160 240 187 221Q193 217 193 216Q182 209 170 200L147 184Q127 192 113 209T98 250Q98 290 193 376Q287 454 287 542Q287 581 262 616T188 652Q143 652 126 631T108 588Q108 559 140 527L79 490Q46 515 46 557ZM108 47Q108 68 123 85T160 103Q179 103 198 90T217 46Q215 24 201 7T164 -11Q142 -11 125 6T108 47",
      91: "262 -119Q224 -120 191 -123T141 -128T118 -130Q117 -130 117 305V740H122Q141 737 219 736H278V723Q278 711 277 711L159 699V-93H162Q167 -93 220 -96T276 -100Q278 -100 278 -109V-119H262",
      93: "64 733Q89 733 110 734T143 737T158 738H160V-131H154Q101 -125 40 -124H-4V-103H1Q3 -102 57 -98T113 -92H118V700L64 703Q7 707 3 708H-4V732H21Q34 733 64 733",
      94: "0 464L250 734L262 722Q274 712 384 598L495 486Q483 478 467 467L456 459L248 672L154 580L23 452Q17 454 10 458T0 464",
      8216: "117 410Q97 410 71 455T45 539Q45 588 129 694L140 708Q142 708 153 694L147 682Q106 609 106 582V577V571Q106 548 132 511T158 455Q158 434 143 422T117 410",
      8217: "105 529Q105 546 77 588T49 651Q49 658 51 666Q53 672 67 682T92 692Q111 692 137 644T163 563Q163 534 143 497T99 428T74 395Q72 395 65 400T58 407Q105 476 105 523V529",
      8260: "230 270Q426 721 428 721Q437 719 447 717L466 713L448 672Q78 -180 77 -181Q77 -182 55 -182L34 -181L230 270",
      58112: "427 436Q427 434 427 425T429 398T430 362Q430 222 396 109L393 99L305 33Q218 -32 216 -32Q208 -29 142 22L91 68L78 81L77 94Q75 130 75 173Q75 245 87 347L135 385Q178 418 184 424L177 428Q174 429 170 431Q116 454 96 473T75 534Q79 608 154 683Q164 677 164 673Q164 670 157 662T144 637T137 598Q137 552 182 518T280 470T380 447T427 436ZM342 371L275 394L208 417Q203 417 192 399T168 334T156 229Q153 187 153 157Q153 141 156 135Q158 125 208 88T280 51Q306 51 326 120T346 297Q346 339 344 354T342 371",
      58113: "39 551L35 569L52 577Q72 586 98 595T140 610T158 616Q174 612 200 604T293 560T412 477Q414 475 417 472Q428 462 430 450T432 376Q432 223 401 124Q395 106 393 103T382 92Q351 68 281 20T206 -29Q201 -31 137 26L100 60L78 83L77 112Q76 132 76 170Q76 259 86 342L88 360L101 371Q116 386 163 422T215 459Q216 459 224 455T233 450L229 446Q225 442 218 434T203 419Q179 394 175 389T168 372Q156 334 156 262Q156 167 164 137Q168 125 196 102T252 62L278 45Q279 45 285 52T302 78T322 126T339 205T346 316Q346 367 344 389L343 406L326 423Q228 520 113 559L100 564L70 557L39 551",
      58114: "123 386L120 431Q116 476 116 511V520Q116 593 174 649Q207 680 236 680Q258 680 284 664T312 648Q318 648 327 656Q328 657 330 659Q337 664 337 661Q337 660 338 657Q338 652 339 648L268 566L260 574Q234 600 206 600Q182 600 164 585T145 541Q145 492 211 386L267 385H324L299 354H214V312Q214 86 193 -58L192 -69L116 -215H108Q92 -215 92 -212Q93 -211 100 -189T116 -135T128 -80Q134 -41 134 22Q134 54 130 185T125 349V354H29L59 385H91Q123 385 123 386",
      58115: "91 530Q91 564 116 600T164 656T194 678Q195 678 200 678T209 679Q268 679 316 639L293 593Q267 547 263 546H262Q260 546 256 553Q222 613 180 613Q160 613 146 599T132 564T170 474T210 388H318L296 356H206V322Q204 284 204 255Q202 221 202 161V99Q202 28 194 -22T160 -124Q148 -146 116 -199L101 -224L91 -220Q85 -218 84 -217T83 -215L101 -161Q116 -114 119 -73T122 108Q119 334 117 352V356H72L28 357L66 388H92Q118 388 118 389L109 433Q91 514 91 530",
      58116: "254 -150Q293 -150 328 -126T363 -54Q363 -38 352 29T339 98L250 34Q160 -30 159 -30L77 64V71Q74 95 74 174Q74 212 75 243T79 294T83 328T87 352T90 366L117 384Q206 446 238 464L250 471Q277 455 306 443T350 427L365 423Q367 423 405 443T443 465L449 455Q431 414 426 362T418 201Q418 135 420 121Q438 -4 438 -19Q438 -26 438 -31T434 -42T429 -51T420 -63T408 -77T391 -95T370 -119T346 -147T325 -170T309 -187T291 -200T274 -207T252 -213T225 -214Q175 -214 132 -196T70 -160L52 -143Q52 -138 90 -48Q90 -47 95 -47H101Q108 -81 146 -115T254 -150ZM341 136Q341 157 344 242T347 348V355L334 356Q299 359 262 367T203 383T179 391Q177 391 173 377T163 323T158 227Q158 164 161 128V121L174 106Q203 75 223 59L341 127V136",
      58117: "92 446Q92 603 82 664Q94 670 95 670L96 666Q98 661 101 651T108 633Q121 598 121 597L141 612Q247 686 250 686Q251 686 266 679Q261 674 243 659T213 632T190 597T173 546Q172 541 171 530T170 511T170 502Q171 502 222 542L273 582Q308 522 315 504L279 449L269 462Q231 506 215 506Q202 506 190 490Q164 458 164 395V390H279L266 373L254 355H167V306Q169 252 169 217Q170 195 170 147V117L200 92Q234 64 237 64Q243 64 277 81L311 99V75Q310 75 242 27L174 -20L156 -3Q88 60 81 60L79 62Q80 60 82 62Q87 67 87 290V355H57L26 356L73 390H92V446",
      58118: "117 531Q117 533 137 544T178 566L198 577Q200 577 204 575T208 572V570Q208 568 208 566T207 560Q197 496 197 397V392H321L295 353H199V260Q199 157 200 145V122L269 68Q271 67 274 67Q282 67 310 83T342 100Q343 100 345 92T346 83L211 -21L172 12Q117 59 117 63Q117 65 117 87T119 150T120 238V353H75L29 354L65 391H118V460Q117 498 117 531",
      58119: "337 91V78L324 71Q288 53 256 29T206 -8T180 -22Q174 -22 158 -9Q82 46 60 46H59L63 51Q67 56 73 68T85 96Q101 158 101 254Q101 300 95 330T83 370T66 394L53 388Q48 385 41 382T24 374Q22 376 20 378T16 381T13 383T10 385V386L119 475Q150 439 160 430L171 422V409Q173 377 173 300Q173 228 166 183T152 122T145 102Q207 81 242 58L337 104V111Q340 146 340 227Q340 320 339 351T338 408V423L422 469Q425 465 429 462L426 438Q413 354 413 251Q413 152 423 119Q426 110 435 96T452 82Q454 82 509 103Q514 98 514 89Q514 87 507 81T472 51T409 -7L395 -20Q393 -18 390 -17Q386 -14 382 -6Q380 -2 379 1Q369 24 361 40T348 62T341 73T338 84L337 91"
    }, {}), fraktur$1;
  }
  var italic$1 = {}, italic = {}, hasRequiredItalic$1;
  function requireItalic$1() {
    return hasRequiredItalic$1 || (hasRequiredItalic$1 = 1, Object.defineProperty(italic, "__esModule", { value: !0 }), italic.italic = void 0, italic.italic = {
      33: [0.716, 0, 0.307, { ic: 0.073 }],
      34: [0.694, -0.379, 0.514, { ic: 0.024 }],
      35: [0.694, 0.194, 0.818, { ic: 0.01 }],
      37: [0.75, 0.056, 0.818, { ic: 0.029 }],
      38: [0.716, 0.022, 0.767, { ic: 0.035 }],
      39: [0.694, -0.379, 0.307, { ic: 0.07 }],
      40: [0.75, 0.25, 0.409, { ic: 0.108 }],
      41: [0.75, 0.25, 0.409],
      42: [0.75, -0.32, 0.511, { ic: 0.073 }],
      43: [0.557, 0.057, 0.767],
      44: [0.121, 0.194, 0.307],
      45: [0.251, -0.18, 0.358],
      46: [0.121, 0, 0.307],
      47: [0.716, 0.215, 0.778],
      48: [0.665, 0.021, 0.511, { ic: 0.051 }],
      49: [0.666, 0, 0.511],
      50: [0.666, 0.022, 0.511, { ic: 0.04 }],
      51: [0.666, 0.022, 0.511, { ic: 0.051 }],
      52: [0.666, 0.194, 0.511],
      53: [0.666, 0.022, 0.511, { ic: 0.056 }],
      54: [0.665, 0.022, 0.511, { ic: 0.054 }],
      55: [0.666, 0.022, 0.511, { ic: 0.123 }],
      56: [0.666, 0.021, 0.511, { ic: 0.042 }],
      57: [0.666, 0.022, 0.511, { ic: 0.042 }],
      58: [0.431, 0, 0.307],
      59: [0.431, 0.194, 0.307],
      61: [0.367, -0.133, 0.767],
      63: [0.716, 0, 0.511, { ic: 0.04 }],
      64: [0.705, 0.011, 0.767, { ic: 0.022 }],
      91: [0.75, 0.25, 0.307, { ic: 0.139 }],
      93: [0.75, 0.25, 0.307, { ic: 0.052 }],
      94: [0.694, -0.527, 0.511, { ic: 0.017 }],
      95: [-0.025, 0.062, 0.511, { ic: 0.043 }],
      126: [0.318, -0.208, 0.511, { ic: 0.06 }],
      305: [0.441, 0.01, 0.307, { ic: 0.033 }],
      567: [0.442, 0.204, 0.332],
      768: [0.697, -0.5, 0],
      769: [0.697, -0.5, 0, { ic: 0.039 }],
      770: [0.694, -0.527, 0, { ic: 0.017 }],
      771: [0.668, -0.558, 0, { ic: 0.06 }],
      772: [0.589, -0.544, 0, { ic: 0.054 }],
      774: [0.694, -0.515, 0, { ic: 0.062 }],
      775: [0.669, -0.548, 0],
      776: [0.669, -0.554, 0, { ic: 0.045 }],
      778: [0.716, -0.542, 0],
      779: [0.697, -0.503, 0, { ic: 0.065 }],
      780: [0.638, -0.502, 0, { ic: 0.029 }],
      989: [0.605, 0.085, 0.778],
      8211: [0.285, -0.248, 0.511, { ic: 0.043 }],
      8212: [0.285, -0.248, 1.022, { ic: 0.016 }],
      8213: [0.285, -0.248, 1.022, { ic: 0.016 }],
      8215: [-0.025, 0.062, 0.511, { ic: 0.043 }],
      8216: [0.694, -0.379, 0.307, { ic: 0.055 }],
      8217: [0.694, -0.379, 0.307, { ic: 0.07 }],
      8220: [0.694, -0.379, 0.514, { ic: 0.092 }],
      8221: [0.694, -0.379, 0.514, { ic: 0.024 }],
      8260: [0.716, 0.215, 0.778],
      8463: [0.695, 0.013, 0.54, { ic: 0.022 }],
      8710: [0.716, 0, 0.833, { sk: 0.167 }],
      10744: [0.716, 0.215, 0.778]
    }), italic;
  }
  var hasRequiredItalic;
  function requireItalic() {
    if (hasRequiredItalic) return italic$1;
    hasRequiredItalic = 1, Object.defineProperty(italic$1, "__esModule", { value: !0 }), italic$1.italic = void 0;
    var O = requireFontData(), _ = requireItalic$1();
    return italic$1.italic = (0, O.AddPaths)(_.italic, {
      33: "330 716Q351 716 365 703T380 670V659L304 433Q230 207 227 204Q225 198 205 198Q184 198 184 207T220 439T260 669Q270 694 297 708Q300 709 304 710T311 713T316 714Q324 716 330 716ZM107 51Q110 83 133 102T179 121Q207 121 219 105T231 72Q231 45 209 23T156 0Q138 0 123 11T107 51",
      34: "214 620Q214 647 236 670T289 694Q312 694 326 677T341 633V624Q341 588 327 550T294 482T253 428T216 392T196 379Q191 379 184 388T176 401Q176 404 195 421T243 472T287 547Q299 576 299 582L295 580Q291 578 282 576T264 573Q241 573 228 585T214 620ZM411 620Q411 647 433 670T486 694Q509 694 523 677T538 633V624Q538 588 524 550T491 482T450 428T413 392T393 379Q388 379 381 388T373 401Q373 404 392 421T440 472T484 547Q496 576 496 582L492 580Q488 578 479 576T461 573Q438 573 425 585T411 620",
      35: "281 327H228Q186 327 175 330T164 347Q164 359 176 365Q179 367 292 367H404L563 688Q569 694 578 694T593 686T597 670Q597 667 531 535L448 367H614L773 688Q779 694 787 694Q796 694 802 688T808 674V672L807 670Q807 667 732 517L658 368L736 367H814Q828 357 828 347Q828 336 812 327H637L599 250Q561 174 561 173H662H724Q760 173 769 170T779 153T770 136T729 132Q721 132 696 132T651 133H540L380 -188Q374 -194 366 -194Q357 -194 351 -188T345 -174L346 -172V-170Q346 -167 412 -35L495 133H330L170 -188Q164 -194 156 -194Q147 -194 141 -188T135 -174L136 -172V-170Q136 -167 211 -17L285 133H207L130 134Q115 140 115 153Q115 164 131 173H306L344 250Q382 326 382 327H281ZM592 327H427L389 250Q351 174 351 173H516L554 250Q592 326 592 327",
      37: "301 348Q257 348 231 383T205 480Q205 553 244 629T346 736Q370 750 394 750Q416 750 429 742T458 714Q512 644 606 644Q722 644 802 732Q817 750 827 750Q835 750 841 744T847 730Q847 725 827 700T689 540Q586 421 512 335Q180 -50 176 -52Q172 -56 165 -56Q157 -56 151 -50T145 -35Q145 -29 162 -9T330 186Q392 258 430 302Q711 627 711 628L701 624Q652 607 604 607Q551 607 494 634L479 642V624Q479 544 439 467Q414 416 377 382T301 348ZM270 443Q270 385 303 385H306H308Q326 385 348 407Q384 441 409 504T434 627Q434 713 390 713Q358 713 328 663Q307 631 289 556T270 451V443ZM608 -56Q573 -56 543 -23T513 76Q513 129 536 190T604 296L627 318Q670 347 704 347Q747 347 767 310T788 222Q788 126 732 35T608 -56ZM742 222Q742 310 699 310Q677 310 655 285T620 227Q608 197 593 138T578 42V36Q578 -18 613 -18Q657 -18 699 64T742 222",
      38: "209 444Q209 546 278 631T424 716Q473 716 501 683T530 601Q530 554 511 535T467 515Q446 515 437 527T427 553Q427 578 446 594T481 610H483V617Q480 641 464 660T419 679Q367 679 328 603Q316 578 305 538T288 470L282 443L295 449Q308 455 330 462T371 469Q397 469 413 450T430 404Q430 363 400 329T331 295Q291 295 256 322Q255 322 246 293T227 223T217 158Q217 117 232 88T273 43T327 22T387 15Q457 15 512 37T599 93T652 169T680 248T688 317T678 380T659 423T647 437Q643 435 637 431T613 416T581 392T554 364T538 332Q540 310 541 310Q554 335 587 335Q601 335 613 327T626 300Q626 290 622 279T603 255T563 242Q532 245 512 263T491 320Q494 351 511 377T547 418T600 456T652 494Q676 512 697 539T728 582T738 601Q738 602 736 602Q732 602 726 604T714 616T707 638Q707 661 724 677T764 694Q799 694 802 660Q802 625 767 562T688 467L694 458Q700 449 702 444T711 428T720 408T727 385T733 358T735 327Q735 281 724 235T685 141T617 59T515 1T375 -22Q270 -22 199 34T127 181Q127 216 147 270T207 374L216 386Q209 421 209 444ZM386 412Q386 432 366 432Q345 432 325 418T294 390T284 375Q284 371 289 362T306 343T335 332Q355 332 367 350T383 384T386 412",
      39: "250 620Q250 647 272 670T325 694Q348 694 362 677T377 633V624Q377 566 343 506T275 412T231 379Q226 379 220 388T213 401T232 421T279 472T323 547Q335 573 335 582L331 580Q327 578 318 576T300 573Q277 573 264 585T250 620",
      40: "241 -250Q203 -212 174 -140T144 39Q144 158 180 288T296 544T481 746L487 750H499Q517 750 517 740Q517 736 495 716Q399 630 331 491T236 228T208 3Q208 -73 224 -130T255 -214T271 -244Q271 -250 252 -250H241",
      41: "326 497Q326 546 320 588T304 655T285 699T269 728T262 740Q262 746 267 749L272 750Q276 750 281 750H293Q331 712 360 640T390 461Q390 332 339 171T188 -116Q161 -150 121 -188T47 -250H35Q17 -250 17 -240Q17 -236 39 -216Q135 -130 203 9T298 272T326 497",
      42: "560 658Q569 658 576 649T584 631Q584 625 583 620T577 611T569 603T556 595T540 587T519 578T494 566L428 536Q427 535 433 531T479 502Q525 475 532 469T539 450Q538 435 525 424T497 412Q489 412 482 418T442 456Q400 497 400 494L387 420Q376 353 373 343T352 323Q345 320 336 320H331Q322 320 316 327T309 343Q309 347 334 420L359 496Q358 496 297 456T234 414Q228 411 221 411Q212 411 204 417T195 439Q198 458 209 465T283 502L353 534L300 566Q255 593 247 599T239 616Q239 631 252 644T282 658Q290 658 295 654T335 615L378 573L391 647Q393 657 395 671T398 691T400 706T404 720T408 730T414 739T423 744T434 749Q435 749 439 749T445 750Q467 748 469 728Q469 723 457 685T432 610L420 573L481 613Q548 658 560 658",
      43: "139 237T139 250T151 266T198 270H293H431L465 407Q469 424 476 452Q494 528 500 542T519 557Q526 557 532 552T538 538Q538 536 507 409T472 272Q472 270 604 270Q737 270 741 268Q753 261 753 250Q753 237 742 233T696 229Q687 229 655 229T599 230H462L461 226Q461 224 427 91T392 -47Q387 -57 374 -57Q367 -57 361 -51T355 -37Q355 -31 388 99L421 230H288Q267 230 238 230T199 229Q163 229 151 233",
      44: "106 46Q106 68 121 90T167 120Q168 120 173 120T180 121Q232 121 232 59V54Q232 18 219 -20T186 -88T145 -143T109 -181T88 -194Q84 -194 77 -185T69 -171Q69 -168 70 -166T76 -161T85 -154T101 -139T124 -114Q146 -88 162 -58T183 -12T188 7Q187 7 183 5T172 2T156 0Q129 0 118 14T106 46",
      45: "205 180H131Q102 180 93 181T84 190Q90 238 103 251H334Q341 244 341 241Q341 236 336 214T327 186Q325 181 312 181T205 180",
      46: "107 50Q107 76 129 98T181 121Q203 121 217 108T231 72Q231 47 210 24T156 0Q135 0 121 13T107 50",
      47: "166 -215T159 -215T147 -212T141 -204T139 -197Q139 -190 144 -183Q157 -157 378 274T602 707Q605 716 618 716Q625 716 630 712T636 703T638 696Q638 691 406 241T170 -212Q166 -215 159 -215",
      48: "414 665Q562 665 562 490Q562 426 534 318Q451 -21 251 -21Q222 -21 202 -15Q155 2 134 40T110 144Q110 201 127 286T187 470T287 614Q348 665 414 665ZM187 98Q187 59 208 37T260 15Q320 15 365 83Q394 128 440 312T487 547Q487 580 471 600T433 627Q428 628 408 628Q381 628 353 609T311 569Q279 526 239 364T190 143Q187 120 187 98",
      49: "248 491Q228 491 228 502Q228 516 236 532Q237 536 246 537T275 541T314 552Q350 567 382 595T430 644L446 664Q450 666 454 666Q468 666 468 658Q468 647 395 359Q321 63 321 59Q321 52 334 50T388 46H422Q428 37 428 35Q428 19 421 5Q416 0 405 0Q400 0 361 1T263 2Q215 2 185 2T142 1T127 0Q110 0 110 11Q110 13 113 25T118 40Q120 46 146 46Q196 46 212 49T235 61Q238 66 295 295L353 526L340 519Q328 512 302 503T248 491",
      50: "159 404Q159 433 176 476T222 562T297 635T395 666Q466 666 508 617T551 497Q551 473 545 446Q534 388 482 333Q441 292 355 240T264 184Q216 151 179 101L171 91Q171 90 177 90Q206 90 269 77T366 64Q385 64 390 65Q418 73 441 98T475 156Q479 168 481 170T495 173H518Q524 167 524 166T521 152Q502 86 459 32T353 -22Q315 -22 259 15T172 53Q156 53 143 36T126 1L121 -16Q119 -22 98 -22H82Q76 -16 76 -13T80 5T98 50T132 111T189 178T274 242Q327 273 364 305T420 370T447 427T460 483Q466 514 466 538Q466 586 443 607T389 629Q338 629 293 584T226 487T204 399Q204 390 204 386T209 378T222 373Q258 376 282 422T307 493Q307 506 302 517T297 531Q297 537 308 546T327 551Q329 550 333 543T340 523T344 497Q344 450 306 393T216 336Q186 336 173 355T159 396V404",
      51: "296 531Q296 536 307 544T322 553Q330 553 338 534T346 501Q346 468 319 440T258 412Q232 412 216 430T200 478Q200 552 281 618Q345 666 416 666Q489 666 525 625T562 530Q562 473 525 419T430 335L416 329Q479 288 479 206Q479 142 440 89T344 7T229 -22Q173 -22 135 12T96 106Q96 192 157 192Q192 192 197 157Q197 134 184 117T142 96Q153 47 180 29Q201 15 232 15Q249 15 275 22Q307 34 331 57Q363 90 379 153T396 246Q396 261 393 272T384 290T371 301T355 308T341 311T326 312H316H307Q287 312 282 313T276 320Q276 323 279 337T283 352Q284 356 290 357T325 358Q364 359 368 360Q386 365 400 372T433 397T464 448T485 527Q487 535 487 556Q487 629 414 629Q350 629 298 580T245 476Q245 450 263 450H264Q280 450 294 463T308 496Q308 508 302 518T296 531",
      52: "448 34Q453 34 463 22T473 5Q473 -2 457 -7Q417 -22 383 -23H366L350 -91Q348 -98 345 -111T340 -130T335 -146T330 -161T325 -172T318 -182T310 -188T299 -193T286 -194Q256 -194 253 -165Q253 -159 271 -83T292 -5Q231 29 169 29Q114 29 91 14Q72 -2 65 1Q46 20 46 28Q46 35 55 43T77 60T96 74Q306 257 396 623Q410 666 444 666Q459 666 468 657T478 634Q478 627 470 595T440 504T387 381T303 239T187 99L164 75H178Q217 75 260 59L304 43Q304 48 325 127Q342 195 346 207T358 228Q372 242 391 242Q403 242 413 235T423 214Q423 205 402 116T378 25Q378 23 387 23Q405 23 418 25T439 31T448 34",
      53: "196 304Q189 309 189 314Q189 317 231 487T275 660Q278 666 283 666Q287 666 302 658T346 643T413 635Q447 635 481 642T537 658T559 666Q561 666 564 663T567 658Q565 637 557 629Q528 600 474 573T359 545Q342 545 327 546T304 550T294 552L291 540Q288 529 283 507T273 465L251 379Q307 420 364 420Q415 420 456 382T497 261Q497 165 429 82T262 -20Q256 -20 247 -21T233 -22Q176 -22 141 15T106 112Q106 208 173 208Q192 208 203 197T214 169Q214 143 195 125T156 107H153V100Q155 73 174 47T239 21Q245 21 259 23Q355 46 392 200Q393 205 394 207Q412 276 412 312Q412 352 396 367T358 383Q288 383 233 314Q226 306 224 305T209 304H196",
      54: "377 434Q425 434 457 404T499 341T509 278Q509 243 496 194T456 105T383 27Q322 -22 256 -22Q142 -22 122 114Q120 130 120 159Q120 221 135 292T195 452T310 599Q390 665 465 665Q565 665 565 583V574Q565 543 546 524Q528 506 504 506Q491 506 478 514T465 543Q465 585 515 602Q505 626 466 626Q419 626 372 587Q334 557 305 503T266 409L255 370Q287 410 339 429Q361 434 377 434ZM424 333Q424 359 411 378T365 397Q318 397 282 356T230 257T205 157T197 94Q197 67 211 45T260 22Q313 22 341 57T386 151Q424 283 424 333",
      55: "466 519Q448 519 435 528T416 550T400 571T376 581Q324 581 271 540T186 437Q185 435 183 432T181 428T179 426T177 424T174 423T171 422T165 422H159Q141 422 141 423Q136 423 136 431Q136 433 190 548T247 665Q249 666 266 666H282Q288 660 288 657Q288 655 284 646T276 628L273 620Q337 666 390 666Q413 666 425 652T438 620T444 584T457 559Q460 557 470 557Q497 557 524 582T571 635T594 665Q595 666 612 666H628Q634 660 634 657Q634 653 618 629T572 556T510 441T437 269T367 43Q356 -22 304 -22Q291 -22 278 -14T263 14Q263 36 281 95T354 269T486 507Q497 524 495 524Q482 519 466 519",
      56: "209 449Q209 545 278 605T416 666Q482 666 517 631T553 546Q553 513 539 482T504 430T463 394T426 370L410 360L430 343Q471 309 483 278T495 211Q495 141 441 75Q363 -21 253 -21Q182 -21 141 18T99 117Q99 161 119 201T170 268T222 308T259 331L272 338L259 349Q212 389 209 449ZM492 542Q492 586 469 605T415 625Q360 625 320 587T279 505Q279 495 281 487T286 474T295 460T306 449T321 436T337 422Q379 386 380 386Q389 386 420 412T472 471Q492 513 492 542ZM163 118Q163 76 189 49T258 21Q316 21 368 64T420 170Q420 193 412 208T395 233T350 271L302 312Q298 312 284 303T249 276T209 235T177 181T163 118",
      57: "297 211Q258 211 230 228T189 273T169 323T163 367Q163 411 183 472T254 585Q327 656 401 665Q403 665 412 665T427 666Q458 664 481 652T518 622T539 580T550 535T553 491Q553 448 544 395T515 277T454 148T358 37Q282 -22 213 -22Q166 -22 137 -1T107 55V64Q107 88 114 104T134 127T154 136T169 138Q185 138 196 128T207 101Q207 82 196 68T172 48L161 43Q161 40 167 36T187 26T219 21Q286 21 344 99Q364 126 382 169T408 241T417 275L412 269Q406 263 395 253T370 234T337 218T297 211ZM476 552Q476 626 417 626Q368 626 330 584Q312 563 300 533T270 433Q248 341 248 312Q248 286 262 267T310 248Q353 248 387 287T440 380T467 480T476 552",
      58: "184 358Q184 385 206 408T258 431Q279 431 293 418T308 383Q308 354 284 332T233 310Q212 310 198 324T184 358ZM107 50Q107 76 129 98T181 121Q203 121 217 108T231 72Q231 47 210 24T156 0Q135 0 121 13T107 50",
      59: "184 358Q184 385 206 408T258 431Q279 431 293 418T308 383Q308 354 284 332T233 310Q212 310 198 324T184 358ZM107 47Q107 77 130 99T180 121Q226 121 226 61Q226 25 214 -14T182 -84T144 -140T109 -180T88 -194T77 -185T70 -172Q70 -169 84 -155T121 -112T161 -48Q180 -10 180 3Q180 4 174 2Q172 2 166 1T156 0Q135 0 121 13T107 47",
      61: "776 357T776 347T761 327H470Q180 327 176 329Q164 334 164 347Q164 359 176 365Q179 367 470 367H761Q776 357 776 347ZM116 143T116 153T131 173H422Q713 173 717 171Q728 166 728 153T717 135Q713 133 422 133H131Q116 143 116 153",
      63: "235 431Q217 431 206 442T195 468Q195 490 215 537T280 638T380 707Q403 716 423 716Q425 716 429 716T436 715Q485 715 518 681T551 590Q551 543 530 503T482 439Q471 428 400 375T318 310Q300 287 300 259Q300 236 315 236Q333 236 352 251T384 300Q386 306 407 306H423Q429 300 429 297Q429 272 393 235T308 198Q287 198 269 215T251 270Q251 330 293 374L374 436Q377 438 401 456T432 480T457 503T481 531T494 561T501 598Q501 614 499 626Q482 678 430 678H426Q392 678 362 660T311 615T280 571T264 540L259 528Q259 527 266 526T283 516T294 492Q294 466 276 449T235 431ZM209 51Q212 83 235 102T281 121Q309 121 321 105T333 72Q333 45 311 23T258 0Q240 0 225 11T209 51",
      64: "198 250Q198 155 248 91T394 26Q514 26 640 80L650 84H675H683Q709 84 709 76Q709 73 708 71Q706 64 660 45T534 8T383 -11T260 24T181 115Q152 168 152 248Q152 410 268 552Q303 590 324 608Q439 705 551 705Q611 705 658 683T733 623T775 543T789 454Q789 380 766 304T720 192Q677 125 617 125Q591 125 573 137T548 160T541 176Q541 178 540 178L534 173Q527 168 515 160T488 144T454 131T417 125Q361 125 320 166T279 284Q279 393 356 481T523 569Q570 569 603 537Q623 515 632 490L637 480L657 479Q684 479 684 470Q684 465 650 333L617 199V185Q616 162 628 162Q677 162 712 278Q743 381 743 442Q743 555 687 611T553 668Q467 668 385 608T250 450T198 250ZM598 445Q598 453 594 470T569 510T518 532Q463 532 410 448T356 271Q356 220 374 191T423 162Q482 162 552 255L575 348Q598 440 598 445",
      91: "205 -221Q205 -239 194 -250H137H106Q73 -250 73 -242Q73 -232 194 255T321 747L324 750H381H417Q435 750 440 748T446 739Q446 730 443 723T437 712L434 710H350L349 706Q349 704 235 249T120 -208Q120 -210 159 -210Q166 -210 175 -210T187 -209Q205 -209 205 -221",
      93: "227 721Q227 739 238 750H295H326Q359 750 359 742Q359 732 238 245T111 -247L108 -250H51H15Q-3 -250 -8 -248T-14 -239Q-14 -230 -11 -223T-5 -212L-2 -210H82L83 -206Q83 -204 197 251T312 708Q312 710 273 710Q266 710 257 710T245 709Q227 709 227 721",
      94: "528 555Q528 549 514 538T496 527Q491 527 470 554Q458 569 449 580L414 625L353 578Q339 567 323 555T298 536L290 529Q286 527 285 527Q279 527 273 533T264 546L260 553Q260 559 263 562Q265 564 342 628T421 693T425 694Q430 694 433 691Q528 563 528 555",
      95: "98 -62Q91 -58 91 -51Q91 -31 100 -26Q102 -25 324 -25H442H500Q536 -25 545 -27T554 -36Q554 -50 548 -56Q546 -60 538 -61Q520 -62 319 -62H98",
      126: "266 208Q262 208 255 215T247 228Q247 233 250 236T274 259Q335 318 369 318Q394 318 420 292T464 265Q485 265 516 291T550 318Q554 318 562 311T571 297Q570 293 551 273T502 231T451 209H447Q421 209 396 235T355 261Q334 261 301 235T266 208",
      305: "75 287Q75 292 82 313T103 362T142 413T196 441H214Q248 441 270 419T293 357Q292 338 289 330T245 208Q193 72 193 46Q193 26 209 26Q228 26 247 43Q273 71 292 136Q295 148 297 150T311 153H317Q327 153 330 153T337 150T340 143Q340 133 330 105T292 41T228 -8Q220 -10 204 -10Q160 -10 141 15T122 71Q122 98 171 227T221 384Q221 396 218 400T203 405Q175 403 156 374T128 312T116 279Q115 278 97 278H81Q75 284 75 287",
      567: "75 284T75 287T81 305T101 343T133 389T180 426T240 442Q273 440 300 420T327 350V332L278 134Q267 92 253 37T233 -45T225 -73Q208 -123 162 -163T54 -204Q8 -204 -15 -181Q-32 -164 -32 -140Q-32 -112 -14 -96T27 -79Q48 -79 57 -91T67 -114Q67 -146 39 -166L44 -167H59H60Q112 -167 145 -74Q148 -65 198 134T251 347Q252 353 252 370Q252 382 251 388T245 399T230 405Q204 405 175 378Q157 360 145 337T126 298T117 280T98 278H81Q75 284 75 287",
      768: "-222 651Q-222 668 -206 682T-174 697Q-155 697 -145 680Q-140 671 -107 599T-74 526Q-74 522 -88 511T-107 500Q-109 500 -113 502T-167 568T-219 637Q-222 643 -222 651",
      769: "-148 500Q-154 500 -163 511T-173 528Q-173 529 -172 530V532Q-170 534 -97 610T-21 688Q-8 697 4 697Q19 697 29 688T39 663T30 638Q26 631 -50 573L-135 507Q-144 500 -148 500",
      770: "17 555Q17 549 3 538T-15 527Q-20 527 -41 554Q-53 569 -62 580L-97 625L-158 578Q-172 567 -188 555T-212 536L-221 529Q-225 527 -226 527Q-232 527 -238 533T-248 546L-251 553Q-251 559 -248 562Q-246 564 -169 628T-90 693T-86 694Q-81 694 -78 691Q17 563 17 555",
      771: "-245 558Q-249 558 -256 565T-264 578Q-264 583 -261 586T-237 609Q-176 668 -142 668Q-117 668 -91 642T-47 615Q-26 615 5 641T39 668Q43 668 51 661T60 647Q59 643 40 623T-9 581T-60 559H-64Q-90 559 -115 585T-156 611Q-177 611 -210 585T-245 558",
      772: "-275 544Q-282 548 -282 554Q-282 561 -279 573T-271 588Q-269 589 -111 589H-27H12Q38 589 46 587T54 578Q54 574 51 563T47 550Q45 546 32 545Q15 544 -118 544H-275",
      774: "-237 641Q-237 694 -218 694H-213Q-195 694 -195 684Q-195 683 -195 679T-197 667T-198 650Q-198 611 -176 589T-117 566Q-74 566 -34 597T23 678Q27 689 30 691T43 694Q62 694 62 684Q62 671 49 645T14 589T-46 537T-123 515Q-175 515 -206 550T-237 641",
      775: "-165 599Q-162 631 -139 650T-93 669Q-65 669 -53 653T-41 620Q-41 593 -63 571T-116 548Q-134 548 -149 559T-165 599",
      776: "-251 601Q-251 626 -230 647T-180 669Q-139 669 -133 625Q-133 595 -155 575T-203 554Q-223 554 -237 567T-251 601ZM-72 599Q-72 632 -48 650T-2 669Q18 669 31 657T45 623Q45 592 22 573T-25 554Q-68 554 -72 599",
      778: "-199 610Q-199 654 -161 685T-79 716Q-39 716 -16 693Q3 674 3 647Q3 607 -34 575T-118 542Q-199 542 -199 610ZM-41 631T-41 655T-83 679H-89Q-129 679 -142 656Q-146 650 -151 632T-156 604Q-156 578 -113 578H-108Q-94 578 -86 579T-69 586T-52 605Q-41 631 -41 655",
      779: "-217 503Q-221 503 -234 510T-248 523Q-248 528 -205 602Q-200 610 -192 623T-180 644T-170 661T-159 676T-151 686T-142 694T-134 696Q-132 697 -121 697Q-88 694 -88 664Q-88 652 -97 640T-152 574Q-214 504 -217 503ZM-64 503Q-68 503 -81 510T-95 523Q-95 528 -52 602Q-47 610 -39 623T-27 644T-17 661T-6 676T2 686T11 694T19 696Q21 697 32 697Q65 694 65 664Q65 652 56 640T1 574Q-61 504 -64 503",
      780: "11 637Q16 637 22 624T29 607Q29 606 27 602Q26 600 -47 552T-125 502H-127Q-133 502 -184 553Q-236 602 -236 608Q-236 612 -224 625T-206 638L-202 637L-196 632Q-190 628 -179 620T-158 603L-116 570Q-109 572 -52 604T11 637",
      989: "477 261Q477 257 473 256T455 253T417 251T348 250H235L155 -77L146 -82Q137 -85 109 -85Q55 -85 55 -77L139 261Q224 596 226 598Q229 603 239 603Q240 603 254 603T290 603T341 604T405 605T477 605Q656 603 687 602T719 596Q719 589 692 588T513 585H319L282 427L242 272Q242 270 351 270Q388 270 410 270T444 269T460 267T469 265T477 261",
      8211: "98 248Q91 252 91 259Q91 279 100 284Q102 285 324 285H442H500Q536 285 545 283T554 274Q554 260 548 254Q546 250 538 249Q520 248 319 248H98",
      8212: "124 248Q117 252 117 259Q117 279 126 284Q128 285 579 285T1033 284Q1037 280 1037 278Q1038 276 1038 274Q1038 253 1029 250Q1026 248 575 248H124",
      8213: "124 248Q117 252 117 259Q117 279 126 284Q128 285 579 285T1033 284Q1037 280 1037 278Q1038 276 1038 274Q1038 253 1029 250Q1026 248 575 248H124",
      8215: "98 -62Q91 -58 91 -51Q91 -31 100 -26Q102 -25 324 -25H442H500Q536 -25 545 -27T554 -36Q554 -50 548 -56Q546 -60 538 -61Q520 -62 319 -62H98",
      8216: "249 379Q228 379 213 396T197 448Q197 533 271 627L278 635Q286 643 295 652T314 671T332 687T344 694Q349 694 355 685T362 671Q362 668 345 654T301 608T256 537Q238 493 240 491Q241 491 245 493T258 498T275 500Q296 500 311 488T326 454Q326 426 304 403T249 379",
      8217: "250 620Q250 647 272 670T325 694Q348 694 362 677T377 633V624Q377 566 343 506T275 412T231 379Q226 379 220 388T213 401T232 421T279 472T323 547Q335 573 335 582L331 580Q327 578 318 576T300 573Q277 573 264 585T250 620",
      8220: "295 379Q274 379 259 396T243 448Q243 533 317 627Q326 638 354 666T391 694Q395 694 402 686T409 673Q409 668 392 654T348 608T302 537Q284 493 286 491Q287 491 291 493T304 498T321 500Q342 500 357 488T372 454Q372 426 350 403T295 379ZM492 379Q471 379 456 396T440 448Q440 533 514 627Q523 638 551 666T588 694Q592 694 599 685T606 672T589 654T544 608T499 537Q481 493 483 491Q484 491 488 493T501 498T518 500Q539 500 554 488T569 454Q569 426 547 403T492 379",
      8221: "214 620Q214 647 236 670T289 694Q312 694 326 677T341 633V624Q341 588 327 550T294 482T253 428T216 392T196 379Q191 379 184 388T176 401Q176 404 195 421T243 472T287 547Q299 576 299 582L295 580Q291 578 282 576T264 573Q241 573 228 585T214 620ZM411 620Q411 647 433 670T486 694Q509 694 523 677T538 633V624Q538 588 524 550T491 482T450 428T413 392T393 379Q388 379 381 388T373 401Q373 404 392 421T440 472T484 547Q496 576 496 582L492 580Q488 578 479 576T461 573Q438 573 425 585T411 620",
      8260: "166 -215T159 -215T147 -212T141 -204T139 -197Q139 -190 144 -183Q157 -157 378 274T602 707Q605 716 618 716Q625 716 630 712T636 703T638 696Q638 691 406 241T170 -212Q166 -215 159 -215",
      8463: "150 475Q147 475 118 466T82 457Q73 457 64 467T54 487Q54 490 55 492Q63 506 64 506Q67 512 118 526Q162 541 169 546Q173 559 175 575Q181 596 181 604Q181 613 166 617Q164 617 153 618T135 619Q119 619 114 621T109 630Q109 636 114 656T122 681Q125 685 202 688Q272 695 286 695Q304 695 304 684Q304 682 291 628L278 577L386 612Q466 635 476 635T492 627T499 607Q499 593 489 586Q485 583 373 546L262 512Q262 511 248 455T233 397T236 397T244 404Q295 441 357 441Q405 441 445 417T485 333Q485 284 449 178T412 58T426 44Q447 44 466 68Q485 87 500 130L509 152H531H543Q562 152 562 144Q562 128 546 93T494 23T415 -13Q385 -13 359 3T322 44Q318 52 318 77Q318 99 352 196T386 337Q386 386 346 386Q318 386 286 370Q267 361 245 338T211 292Q207 287 193 235T162 113T138 21Q128 7 122 4Q105 -12 83 -12Q66 -12 54 -2T42 26Q42 45 98 257L151 475H150",
      8710: "574 715L582 716Q589 716 595 716Q612 716 616 714Q621 712 621 709Q622 707 705 359T788 8Q786 5 785 3L781 0H416Q52 0 50 2T48 6Q48 9 305 358T567 711Q572 712 574 715ZM599 346L538 602L442 474Q347 345 252 217T157 87T409 86T661 88L654 120Q646 151 629 220T599 346",
      10744: "166 -215T159 -215T147 -212T141 -204T139 -197Q139 -190 144 -183Q157 -157 378 274T602 707Q605 716 618 716Q625 716 630 712T636 703T638 696Q638 691 406 241T170 -212Q166 -215 159 -215"
    }, {}), italic$1;
  }
  var largeop$1 = {}, largeop = {}, hasRequiredLargeop$1;
  function requireLargeop$1() {
    return hasRequiredLargeop$1 || (hasRequiredLargeop$1 = 1, Object.defineProperty(largeop, "__esModule", { value: !0 }), largeop.largeop = void 0, largeop.largeop = {
      40: [1.15, 0.649, 0.597],
      41: [1.15, 0.649, 0.597],
      47: [1.15, 0.649, 0.811],
      91: [1.15, 0.649, 0.472],
      92: [1.15, 0.649, 0.811],
      93: [1.15, 0.649, 0.472],
      123: [1.15, 0.649, 0.667],
      125: [1.15, 0.649, 0.667],
      710: [0.772, -0.565, 1],
      732: [0.75, -0.611, 1],
      770: [0.772, -0.565, 0],
      771: [0.75, -0.611, 0],
      8214: [0.602, 0, 0.778],
      8260: [1.15, 0.649, 0.811],
      8593: [0.6, 0, 0.667],
      8595: [0.6, 0, 0.667],
      8657: [0.599, 0, 0.778],
      8659: [0.6, 0, 0.778],
      8719: [0.95, 0.45, 1.278],
      8720: [0.95, 0.45, 1.278],
      8721: [0.95, 0.45, 1.444],
      8730: [1.15, 0.65, 1, { ic: 0.02 }],
      8739: [0.627, 0.015, 0.333],
      8741: [0.627, 0.015, 0.556],
      8747: [1.36, 0.862, 0.556, { ic: 0.388 }],
      8748: [1.36, 0.862, 1.084, { ic: 0.388 }],
      8749: [1.36, 0.862, 1.592, { ic: 0.388 }],
      8750: [1.36, 0.862, 0.556, { ic: 0.388 }],
      8896: [0.95, 0.45, 1.111],
      8897: [0.95, 0.45, 1.111],
      8898: [0.949, 0.45, 1.111],
      8899: [0.95, 0.449, 1.111],
      8968: [1.15, 0.649, 0.528],
      8969: [1.15, 0.649, 0.528],
      8970: [1.15, 0.649, 0.528],
      8971: [1.15, 0.649, 0.528],
      9001: [1.15, 0.649, 0.611],
      9002: [1.15, 0.649, 0.611],
      9168: [0.602, 0, 0.667],
      10072: [0.627, 0.015, 0.333],
      10216: [1.15, 0.649, 0.611],
      10217: [1.15, 0.649, 0.611],
      10752: [0.949, 0.449, 1.511],
      10753: [0.949, 0.449, 1.511],
      10754: [0.949, 0.449, 1.511],
      10756: [0.95, 0.449, 1.111],
      10758: [0.95, 0.45, 1.111],
      10764: [1.36, 0.862, 2.168, { ic: 0.388 }],
      12296: [1.15, 0.649, 0.611],
      12297: [1.15, 0.649, 0.611]
    }), largeop;
  }
  var hasRequiredLargeop;
  function requireLargeop() {
    if (hasRequiredLargeop) return largeop$1;
    hasRequiredLargeop = 1, Object.defineProperty(largeop$1, "__esModule", { value: !0 }), largeop$1.largeop = void 0;
    var O = requireFontData(), _ = requireLargeop$1();
    return largeop$1.largeop = (0, O.AddPaths)(_.largeop, {
      40: "180 96T180 250T205 541T266 770T353 944T444 1069T527 1150H555Q561 1144 561 1141Q561 1137 545 1120T504 1072T447 995T386 878T330 721T288 513T272 251Q272 133 280 56Q293 -87 326 -209T399 -405T475 -531T536 -609T561 -640Q561 -643 555 -649H527Q483 -612 443 -568T353 -443T266 -270T205 -41",
      41: "35 1138Q35 1150 51 1150H56H69Q113 1113 153 1069T243 944T330 771T391 541T416 250T391 -40T330 -270T243 -443T152 -568T69 -649H56Q43 -649 39 -647T35 -637Q65 -607 110 -548Q283 -316 316 56Q324 133 324 251Q324 368 316 445Q278 877 48 1123Q36 1137 35 1138",
      47: "78 -649Q56 -646 56 -625Q56 -614 382 261T712 1140Q716 1150 732 1150Q754 1147 754 1126Q754 1116 428 240T98 -639Q94 -649 78 -649",
      91: "224 -649V1150H455V1099H275V-598H455V-649H224",
      92: "754 -625Q754 -649 731 -649Q715 -649 712 -639Q709 -635 383 242T55 1124Q54 1135 61 1142T80 1150Q92 1150 98 1140Q101 1137 427 262T754 -625",
      93: "16 1099V1150H247V-649H16V-598H196V1099H16",
      123: "547 -643L541 -649H528Q515 -649 503 -645Q324 -582 293 -466Q289 -449 289 -428T287 -200L286 42L284 53Q274 98 248 135T196 190T146 222L121 235Q119 239 119 250Q119 262 121 266T133 273Q262 336 284 449L286 460L287 701Q287 737 287 794Q288 949 292 963Q293 966 293 967Q325 1080 508 1148Q516 1150 527 1150H541L547 1144V1130Q547 1117 546 1115T536 1109Q480 1086 437 1046T381 950L379 940L378 699Q378 657 378 594Q377 452 374 438Q373 437 373 436Q350 348 243 282Q192 257 186 254L176 251L188 245Q211 236 234 223T287 189T340 135T373 65Q373 64 374 63Q377 49 378 -93Q378 -156 378 -198L379 -438L381 -449Q393 -504 436 -544T536 -608Q544 -611 545 -613T547 -629V-643",
      125: "119 1130Q119 1144 121 1147T135 1150H139Q151 1150 182 1138T252 1105T326 1046T373 964Q378 942 378 702Q378 469 379 462Q386 394 439 339Q482 296 535 272Q544 268 545 266T547 251Q547 241 547 238T542 231T531 227T510 217T477 194Q390 129 379 39Q378 32 378 -201Q378 -441 373 -463Q342 -580 165 -644Q152 -649 139 -649Q125 -649 122 -646T119 -629Q119 -622 119 -619T121 -614T124 -610T132 -607T143 -602Q195 -579 235 -539T285 -447Q286 -435 287 -199T289 51Q294 74 300 91T329 138T390 197Q412 213 436 226T475 244L489 250L472 258Q455 265 430 279T377 313T327 366T293 434Q289 451 289 472T287 699Q286 941 285 948Q279 978 262 1005T227 1048T184 1080T151 1100T129 1109L127 1110Q119 1113 119 1130",
      710: "1004 603Q1004 600 999 583T991 565L960 574Q929 582 866 599T745 631L500 698Q497 698 254 631Q197 616 134 599T39 574L8 565Q5 565 0 582T-5 603L26 614Q58 624 124 646T248 687L499 772Q999 604 1004 603",
      732: "296 691Q258 691 216 683T140 663T79 639T34 619T16 611Q13 619 8 628L0 644L36 662Q206 749 321 749Q410 749 517 710T703 670Q741 670 783 678T859 698T920 722T965 742T983 750Q986 742 991 733L999 717L963 699Q787 611 664 611Q594 611 484 651T296 691",
      770: "4 603Q4 600 -1 583T-9 565L-40 574Q-71 582 -134 599T-255 631L-500 698Q-503 698 -746 631Q-803 616 -866 599T-961 574L-992 565Q-995 565 -1000 582T-1005 603L-974 614Q-942 624 -876 646T-752 687L-501 772Q-1 604 4 603",
      771: "-704 691Q-742 691 -784 683T-860 663T-921 639T-966 619T-984 611Q-987 619 -992 628L-1000 644L-964 662Q-794 749 -679 749Q-590 749 -483 710T-297 670Q-259 670 -217 678T-141 698T-80 722T-35 742T-17 750Q-14 742 -9 733L-1 717L-37 699Q-213 611 -336 611Q-405 611 -515 651T-704 691",
      8214: "257 0V602H300V0H257ZM478 0V602H521V0H478",
      8260: "78 -649Q56 -646 56 -625Q56 -614 382 261T712 1140Q716 1150 732 1150Q754 1147 754 1126Q754 1116 428 240T98 -639Q94 -649 78 -649",
      8593: "112 421L120 424Q127 427 136 430T161 441T191 458T224 481T260 510T295 546T328 591L333 600L340 589Q380 527 431 489T555 421V377L543 381Q445 418 368 492L355 504V0H312V504L299 492Q222 418 124 381L112 377V421",
      8595: "312 96V600H355V96L368 108Q445 182 543 219L555 223V179L546 176Q538 173 529 169T505 158T475 141T442 119T407 90T372 53T339 9L334 0L327 11Q287 73 236 111T112 179V223L124 219Q222 182 299 108L312 96",
      8657: "142 329Q300 419 389 599Q389 598 399 579T420 541T452 494T497 438T558 383T636 329T708 294L721 289V246Q718 246 694 256T623 293T532 356L522 364L521 182V0H478V405L466 417Q436 450 389 516Q388 515 378 500T352 463T312 417L300 405V0H257V364L247 356Q202 320 155 293T82 256L57 246V289L70 294Q101 305 142 329",
      8659: "257 236V600H300V195L312 183Q342 150 389 84Q390 85 400 100T426 137T466 183L478 195V600H521V418L522 236L532 244Q576 280 623 307T696 344L721 354V311L708 306Q677 295 636 271Q478 181 389 1Q389 2 379 21T358 59T326 106T281 162T220 217T142 271T70 306L57 311V354Q60 354 83 345T154 308T247 244L257 236",
      8719: "220 812Q220 813 218 819T214 829T208 840T199 853T185 866T166 878T140 887T107 893T66 896H56V950H1221V896H1211Q1080 896 1058 812V-311Q1076 -396 1211 -396H1221V-450H725V-396H735Q864 -396 888 -314Q889 -312 889 -311V896H388V292L389 -311Q405 -396 542 -396H552V-450H56V-396H66Q195 -396 219 -314Q220 -312 220 -311V812",
      8720: "220 812Q220 813 218 819T214 829T208 840T199 853T185 866T166 878T140 887T107 893T66 896H56V950H552V896H542Q411 896 389 812L388 208V-396H889V812Q889 813 887 819T883 829T877 840T868 853T854 866T835 878T809 887T776 893T735 896H725V950H1221V896H1211Q1080 896 1058 812V-311Q1076 -396 1211 -396H1221V-450H56V-396H66Q195 -396 219 -314Q220 -312 220 -311V812",
      8721: "60 948Q63 950 665 950H1267L1325 815Q1384 677 1388 669H1348L1341 683Q1320 724 1285 761Q1235 809 1174 838T1033 881T882 898T699 902H574H543H251L259 891Q722 258 724 252Q725 250 724 246Q721 243 460 -56L196 -356Q196 -357 407 -357Q459 -357 548 -357T676 -358Q812 -358 896 -353T1063 -332T1204 -283T1307 -196Q1328 -170 1348 -124H1388Q1388 -125 1381 -145T1356 -210T1325 -294L1267 -449L666 -450Q64 -450 61 -448Q55 -446 55 -439Q55 -437 57 -433L590 177Q590 178 557 222T452 366T322 544L56 909L55 924Q55 945 60 948",
      8730: "1001 1150Q1017 1150 1020 1132Q1020 1127 741 244L460 -643Q453 -650 436 -650H424Q423 -647 423 -645T421 -640T419 -631T415 -617T408 -594T399 -560T385 -512T367 -448T343 -364T312 -259L203 119L138 41L111 67L212 188L264 248L472 -474L983 1140Q988 1150 1001 1150",
      8739: "146 612Q151 627 166 627Q182 627 187 612Q188 610 188 306T187 0Q184 -15 166 -15Q149 -15 146 0V10Q146 19 146 35T146 73T146 122T145 179T145 241T145 306T145 370T145 433T145 489T146 538T146 576T146 602V612",
      8741: "146 612Q151 627 166 627Q182 627 187 612Q188 610 188 306T187 0Q184 -15 166 -15Q149 -15 146 0V10Q146 19 146 35T146 73T146 122T145 179T145 241T145 306T145 370T145 433T145 489T146 538T146 576T146 602V612ZM368 612Q373 627 388 627Q404 627 409 612Q410 610 410 306T409 0Q406 -15 389 -15Q371 -15 368 0V10Q368 19 368 35T368 73T368 122T367 179T367 241T367 306T367 370T367 433T367 489T368 538T368 576T368 602V612",
      8747: "114 -798Q132 -824 165 -824H167Q195 -824 223 -764T275 -600T320 -391T362 -164Q365 -143 367 -133Q439 292 523 655T645 1127Q651 1145 655 1157T672 1201T699 1257T733 1306T777 1346T828 1360Q884 1360 912 1325T944 1245Q944 1220 932 1205T909 1186T887 1183Q866 1183 849 1198T832 1239Q832 1287 885 1296L882 1300Q879 1303 874 1307T866 1313Q851 1323 833 1323Q819 1323 807 1311T775 1255T736 1139T689 936T633 628Q574 293 510 -5T410 -437T355 -629Q278 -862 165 -862Q125 -862 92 -831T55 -746Q55 -711 74 -698T112 -685Q133 -685 150 -700T167 -741Q167 -789 114 -798",
      8748: "114 -798Q132 -824 165 -824H167Q195 -824 223 -764T275 -600T320 -391T362 -164Q365 -143 367 -133Q439 292 523 655T645 1127Q651 1145 655 1157T672 1201T699 1257T733 1306T777 1346T828 1360Q884 1360 912 1325T944 1245Q944 1220 932 1205T909 1186T887 1183Q866 1183 849 1198T832 1239Q832 1287 885 1296L882 1300Q879 1303 874 1307T866 1313Q851 1323 833 1323Q819 1323 807 1311T775 1255T736 1139T689 936T633 628Q574 293 510 -5T410 -437T355 -629Q278 -862 165 -862Q125 -862 92 -831T55 -746Q55 -711 74 -698T112 -685Q133 -685 150 -700T167 -741Q167 -789 114 -798ZM642 -798Q660 -824 693 -824H695Q723 -824 751 -764T803 -600T848 -391T890 -164Q893 -143 895 -133Q967 292 1051 655T1173 1127Q1179 1145 1183 1157T1200 1201T1227 1257T1261 1306T1305 1346T1356 1360Q1412 1360 1440 1325T1472 1245Q1472 1220 1460 1205T1437 1186T1415 1183Q1394 1183 1377 1198T1360 1239Q1360 1287 1413 1296L1410 1300Q1407 1303 1402 1307T1394 1313Q1379 1323 1361 1323Q1347 1323 1335 1311T1303 1255T1264 1139T1217 936T1161 628Q1102 293 1038 -5T938 -437T883 -629Q806 -862 693 -862Q653 -862 620 -831T583 -746Q583 -711 602 -698T640 -685Q661 -685 678 -700T695 -741Q695 -789 642 -798",
      8749: "114 -798Q132 -824 165 -824H167Q195 -824 223 -764T275 -600T320 -391T362 -164Q365 -143 367 -133Q439 292 523 655T645 1127Q651 1145 655 1157T672 1201T699 1257T733 1306T777 1346T828 1360Q884 1360 912 1325T944 1245Q944 1220 932 1205T909 1186T887 1183Q866 1183 849 1198T832 1239Q832 1287 885 1296L882 1300Q879 1303 874 1307T866 1313Q851 1323 833 1323Q819 1323 807 1311T775 1255T736 1139T689 936T633 628Q574 293 510 -5T410 -437T355 -629Q278 -862 165 -862Q125 -862 92 -831T55 -746Q55 -711 74 -698T112 -685Q133 -685 150 -700T167 -741Q167 -789 114 -798ZM642 -798Q660 -824 693 -824H695Q723 -824 751 -764T803 -600T848 -391T890 -164Q893 -143 895 -133Q967 292 1051 655T1173 1127Q1179 1145 1183 1157T1200 1201T1227 1257T1261 1306T1305 1346T1356 1360Q1412 1360 1440 1325T1472 1245Q1472 1220 1460 1205T1437 1186T1415 1183Q1394 1183 1377 1198T1360 1239Q1360 1287 1413 1296L1410 1300Q1407 1303 1402 1307T1394 1313Q1379 1323 1361 1323Q1347 1323 1335 1311T1303 1255T1264 1139T1217 936T1161 628Q1102 293 1038 -5T938 -437T883 -629Q806 -862 693 -862Q653 -862 620 -831T583 -746Q583 -711 602 -698T640 -685Q661 -685 678 -700T695 -741Q695 -789 642 -798ZM1150 -798Q1168 -824 1201 -824H1203Q1231 -824 1259 -764T1311 -600T1356 -391T1398 -164Q1401 -143 1403 -133Q1475 292 1559 655T1681 1127Q1687 1145 1691 1157T1708 1201T1735 1257T1769 1306T1813 1346T1864 1360Q1920 1360 1948 1325T1980 1245Q1980 1220 1968 1205T1945 1186T1923 1183Q1902 1183 1885 1198T1868 1239Q1868 1287 1921 1296L1918 1300Q1915 1303 1910 1307T1902 1313Q1887 1323 1869 1323Q1855 1323 1843 1311T1811 1255T1772 1139T1725 936T1669 628Q1610 293 1546 -5T1446 -437T1391 -629Q1314 -862 1201 -862Q1161 -862 1128 -831T1091 -746Q1091 -711 1110 -698T1148 -685Q1169 -685 1186 -700T1203 -741Q1203 -789 1150 -798",
      8750: "114 -798Q132 -824 165 -824H167Q195 -824 223 -764T275 -600T320 -391T362 -164Q365 -143 367 -133Q382 -52 390 2Q314 40 276 99Q230 167 230 249Q230 363 305 436T484 519H494L503 563Q587 939 632 1087T727 1298Q774 1360 828 1360Q884 1360 912 1325T944 1245Q944 1220 932 1205T909 1186T887 1183Q866 1183 849 1198T832 1239Q832 1287 885 1296L882 1300Q879 1303 874 1307T866 1313Q851 1323 833 1323Q766 1323 688 929Q662 811 610 496Q770 416 770 249Q770 147 701 68T516 -21H506L497 -65Q407 -464 357 -623T237 -837Q203 -862 165 -862Q125 -862 92 -831T55 -746Q55 -711 74 -698T112 -685Q133 -685 150 -700T167 -741Q167 -789 114 -798ZM480 478Q460 478 435 470T380 444T327 401T287 335T271 249Q271 124 375 56L397 43L431 223L485 478H480ZM519 20Q545 20 578 33T647 72T706 144T730 249Q730 383 603 455Q603 454 597 421T582 343T569 276Q516 22 515 20H519",
      8896: "1055 -401Q1055 -419 1042 -434T1007 -450Q977 -450 963 -423Q959 -417 757 167L555 750L353 167Q151 -417 147 -423Q134 -450 104 -450Q84 -450 70 -436T55 -401Q55 -394 56 -390Q59 -381 284 270T512 925Q525 950 555 950Q583 950 597 926Q599 923 825 270T1054 -391Q1055 -394 1055 -401",
      8897: "55 900Q55 919 69 934T103 950Q134 950 147 924Q152 913 353 333L555 -250L757 333Q958 913 963 924Q978 950 1007 950Q1028 950 1041 935T1055 901Q1055 894 1054 891Q1052 884 826 231T597 -426Q583 -450 556 -450Q527 -450 512 -424Q510 -421 285 229T56 890Q55 893 55 900",
      8898: "57 516Q68 602 104 675T190 797T301 882T423 933T542 949Q594 949 606 948Q780 928 901 815T1048 545Q1053 516 1053 475T1055 49Q1055 -406 1054 -410Q1051 -427 1037 -438T1006 -450T976 -439T958 -411Q957 -407 957 37Q957 484 956 494Q945 643 831 747T554 852Q481 852 411 826Q301 786 232 696T154 494Q153 484 153 37Q153 -407 152 -411Q148 -428 135 -439T104 -450T73 -439T56 -410Q55 -406 55 49Q56 505 57 516",
      8899: "56 911Q58 926 71 938T103 950Q120 950 134 939T152 911Q153 907 153 463Q153 16 154 6Q165 -143 279 -247T556 -352Q716 -352 830 -248T956 6Q957 16 957 463Q957 907 958 911Q962 928 975 939T1006 950T1037 939T1054 911Q1055 906 1055 451Q1054 -5 1053 -16Q1029 -207 889 -328T555 -449Q363 -449 226 -331T62 -45Q57 -16 57 25T55 451Q55 906 56 911",
      8968: "224 -649V1150H511V1099H275V-649H224",
      8969: "16 1099V1150H303V-649H252V1099H16",
      8970: "224 -649V1150H275V-598H511V-649H224",
      8971: "252 -598V1150H303V-649H16V-598H252",
      9001: "112 244V258L473 1130Q482 1150 498 1150Q511 1150 517 1142T523 1125V1118L344 685Q304 587 257 473T187 305L165 251L344 -184L523 -616V-623Q524 -634 517 -641T499 -649Q484 -649 473 -629L112 244",
      9002: "112 -649Q103 -649 95 -642T87 -623V-616L266 -184L445 251Q445 252 356 466T178 898T86 1123Q85 1134 93 1142T110 1150Q126 1150 133 1137Q134 1136 317 695L498 258V244L317 -194Q134 -635 133 -636Q126 -649 112 -649",
      9168: "312 0V602H355V0H312",
      10072: "146 612Q151 627 166 627Q182 627 187 612Q188 610 188 306T187 0Q184 -15 166 -15Q149 -15 146 0V10Q146 19 146 35T146 73T146 122T145 179T145 241T145 306T145 370T145 433T145 489T146 538T146 576T146 602V612",
      10216: "112 244V258L473 1130Q482 1150 498 1150Q511 1150 517 1142T523 1125V1118L344 685Q304 587 257 473T187 305L165 251L344 -184L523 -616V-623Q524 -634 517 -641T499 -649Q484 -649 473 -629L112 244",
      10217: "112 -649Q103 -649 95 -642T87 -623V-616L266 -184L445 251Q445 252 356 466T178 898T86 1123Q85 1134 93 1142T110 1150Q126 1150 133 1137Q134 1136 317 695L498 258V244L317 -194Q134 -635 133 -636Q126 -649 112 -649",
      10752: "668 944Q697 949 744 949Q803 949 814 948Q916 937 1006 902T1154 826T1262 730T1336 638T1380 563Q1454 415 1454 250Q1454 113 1402 -14T1258 -238T1036 -391T755 -449Q608 -449 477 -392T255 -240T110 -16T56 250Q56 387 105 510T239 723T434 871T668 944ZM755 -352Q922 -352 1061 -269T1278 -48T1356 250Q1356 479 1202 652T809 850Q798 851 747 851Q634 851 527 806T337 682T204 491T154 251Q154 128 201 17T329 -176T521 -304T755 -352ZM665 250Q665 290 692 315T758 341Q792 339 818 315T845 250Q845 211 819 186T755 160Q716 160 691 186T665 250",
      10753: "668 944Q697 949 744 949Q803 949 814 948Q916 937 1006 902T1154 826T1262 730T1336 638T1380 563Q1454 415 1454 250Q1454 113 1402 -14T1258 -238T1036 -391T755 -449Q608 -449 477 -392T255 -240T110 -16T56 250Q56 387 105 510T239 723T434 871T668 944ZM706 299V850H704Q519 832 386 725T198 476Q181 433 169 379T156 300Q156 299 431 299H706ZM1116 732Q1054 778 982 807T871 842T810 849L804 850V299H1079Q1354 299 1354 300Q1354 311 1352 329T1336 402T1299 506T1228 620T1116 732ZM706 -350V201H431Q156 201 156 200Q156 189 158 171T174 98T211 -6T282 -120T395 -232Q428 -257 464 -277T527 -308T587 -328T636 -339T678 -346T706 -350ZM1354 200Q1354 201 1079 201H804V-350Q808 -349 838 -345T887 -338T940 -323T1010 -295Q1038 -282 1067 -265T1144 -208T1229 -121T1301 0T1349 158Q1354 188 1354 200",
      10754: "668 944Q697 949 744 949Q803 949 814 948Q916 937 1006 902T1154 826T1262 730T1336 638T1380 563Q1454 415 1454 250Q1454 113 1402 -14T1258 -238T1036 -391T755 -449Q608 -449 477 -392T255 -240T110 -16T56 250Q56 387 105 510T239 723T434 871T668 944ZM1143 709Q1138 714 1129 722T1086 752T1017 791T925 826T809 850Q798 851 747 851H728Q659 851 571 823T408 741Q367 713 367 709L755 320L1143 709ZM297 639Q296 639 282 622T247 570T205 491T169 382T154 250T168 118T204 9T247 -70T282 -122L297 -139L685 250L297 639ZM1213 -139Q1214 -139 1228 -122T1263 -70T1305 9T1341 118T1356 250T1342 382T1306 491T1263 570T1228 622L1213 639L825 250L1213 -139ZM367 -209Q373 -215 384 -224T434 -258T514 -302T622 -336T755 -352T887 -338T996 -302T1075 -259T1126 -224L1143 -209L755 180Q754 180 561 -14T367 -209",
      10756: "56 911Q58 926 71 938T103 950Q120 950 134 939T152 911Q153 907 153 463Q153 16 154 6Q165 -143 279 -247T556 -352Q716 -352 830 -248T956 6Q957 16 957 463Q957 907 958 911Q962 928 975 939T1006 950T1037 939T1054 911Q1055 906 1055 451Q1054 -5 1053 -16Q1029 -207 889 -328T555 -449Q363 -449 226 -331T62 -45Q57 -16 57 25T55 451Q55 906 56 911ZM507 554Q511 570 523 581T554 593Q571 593 585 582T603 554Q604 551 604 443V338H709Q817 338 820 337Q835 334 847 321T859 290Q859 254 819 241Q816 240 709 240H604V134Q604 48 604 34T598 11Q583 -15 555 -15Q526 -15 512 11Q507 20 507 34T506 134V240H401H344Q292 240 278 246Q251 259 251 290Q251 309 264 321T290 337Q293 338 401 338H506V443Q506 551 507 554",
      10758: "56 911Q60 927 72 938T103 950Q120 950 134 939T152 911Q153 907 153 277V-352H957V277Q957 907 958 911Q962 928 975 939T1006 950T1036 939T1054 911V891Q1054 871 1054 836T1054 754T1054 647T1055 525T1055 390T1055 250T1055 111T1055 -24T1055 -147T1054 -253T1054 -335T1054 -391V-411Q1047 -442 1016 -449Q1011 -450 552 -450L94 -449Q63 -439 56 -411V-391Q56 -371 56 -336T56 -254T56 -147T55 -25T55 110T55 250T55 389T55 524T55 647T56 753T56 835T56 891V911",
      12296: "112 244V258L473 1130Q482 1150 498 1150Q511 1150 517 1142T523 1125V1118L344 685Q304 587 257 473T187 305L165 251L344 -184L523 -616V-623Q524 -634 517 -641T499 -649Q484 -649 473 -629L112 244",
      12297: "112 -649Q103 -649 95 -642T87 -623V-616L266 -184L445 251Q445 252 356 466T178 898T86 1123Q85 1134 93 1142T110 1150Q126 1150 133 1137Q134 1136 317 695L498 258V244L317 -194Q134 -635 133 -636Q126 -649 112 -649"
    }, {
      10764: "∬∬"
    }), largeop$1;
  }
  var monospace$1 = {}, monospace = {}, hasRequiredMonospace$1;
  function requireMonospace$1() {
    return hasRequiredMonospace$1 || (hasRequiredMonospace$1 = 1, Object.defineProperty(monospace, "__esModule", { value: !0 }), monospace.monospace = void 0, monospace.monospace = {
      32: [0, 0, 0.525],
      33: [0.622, 0, 0.525],
      34: [0.623, -0.333, 0.525],
      35: [0.611, 0, 0.525],
      36: [0.694, 0.082, 0.525],
      37: [0.694, 0.083, 0.525],
      38: [0.622, 0.011, 0.525],
      39: [0.611, -0.287, 0.525],
      40: [0.694, 0.082, 0.525],
      41: [0.694, 0.082, 0.525],
      42: [0.52, -0.09, 0.525],
      43: [0.531, -0.081, 0.525],
      44: [0.14, 0.139, 0.525],
      45: [0.341, -0.271, 0.525],
      46: [0.14, 0, 0.525],
      47: [0.694, 0.083, 0.525],
      58: [0.431, 0, 0.525],
      59: [0.431, 0.139, 0.525],
      60: [0.557, -0.055, 0.525],
      61: [0.417, -0.195, 0.525],
      62: [0.557, -0.055, 0.525],
      63: [0.617, 0, 0.525],
      64: [0.617, 6e-3, 0.525],
      91: [0.694, 0.082, 0.525],
      92: [0.694, 0.083, 0.525],
      93: [0.694, 0.082, 0.525],
      94: [0.611, -0.46, 0.525],
      95: [-0.025, 0.095, 0.525],
      96: [0.681, -0.357, 0.525],
      123: [0.694, 0.083, 0.525],
      124: [0.694, 0.082, 0.525],
      125: [0.694, 0.083, 0.525],
      126: [0.611, -0.466, 0.525],
      127: [0.612, -0.519, 0.525],
      160: [0, 0, 0.525],
      305: [0.431, 0, 0.525],
      567: [0.431, 0.228, 0.525],
      697: [0.623, -0.334, 0.525],
      768: [0.611, -0.485, 0],
      769: [0.611, -0.485, 0],
      770: [0.611, -0.46, 0],
      771: [0.611, -0.466, 0],
      772: [0.577, -0.5, 0],
      774: [0.611, -0.504, 0],
      776: [0.612, -0.519, 0],
      778: [0.619, -0.499, 0],
      780: [0.577, -0.449, 0],
      913: [0.623, 0, 0.525],
      914: [0.611, 0, 0.525],
      915: [0.611, 0, 0.525],
      916: [0.623, 0, 0.525],
      917: [0.611, 0, 0.525],
      918: [0.611, 0, 0.525],
      919: [0.611, 0, 0.525],
      920: [0.621, 0.01, 0.525],
      921: [0.611, 0, 0.525],
      922: [0.611, 0, 0.525],
      923: [0.623, 0, 0.525],
      924: [0.611, 0, 0.525],
      925: [0.611, 0, 0.525],
      926: [0.611, 0, 0.525],
      927: [0.621, 0.01, 0.525],
      928: [0.611, 0, 0.525],
      929: [0.611, 0, 0.525],
      931: [0.611, 0, 0.525],
      932: [0.611, 0, 0.525],
      933: [0.622, 0, 0.525],
      934: [0.611, 0, 0.525],
      935: [0.611, 0, 0.525],
      936: [0.611, 0, 0.525],
      937: [0.622, 0, 0.525],
      8215: [-0.025, 0.095, 0.525],
      8242: [0.623, -0.334, 0.525],
      8243: [0.623, 0, 1.05],
      8244: [0.623, 0, 1.575],
      8260: [0.694, 0.083, 0.525],
      8279: [0.623, 0, 2.1],
      8710: [0.623, 0, 0.525]
    }), monospace;
  }
  var hasRequiredMonospace;
  function requireMonospace() {
    if (hasRequiredMonospace) return monospace$1;
    hasRequiredMonospace = 1, Object.defineProperty(monospace$1, "__esModule", { value: !0 }), monospace$1.monospace = void 0;
    var O = requireFontData(), _ = requireMonospace$1();
    return monospace$1.monospace = (0, O.AddPaths)(_.monospace, {
      32: "",
      33: "206 565Q206 590 222 606T265 622Q287 621 303 606T319 565T314 392L308 216Q299 194 273 194H262Q247 194 241 195T228 200T217 216L211 392Q206 539 206 565ZM206 56Q206 83 223 99T265 115Q288 113 304 99T320 58Q320 33 303 17T262 0Q237 0 222 17T206 56",
      34: "122 575Q122 593 137 608T173 623Q196 623 210 608T225 575Q225 562 218 464Q212 373 211 361T201 341Q193 333 173 333Q154 333 146 341Q138 348 137 360T129 464Q122 561 122 575ZM299 575Q299 593 314 608T350 623Q373 623 387 608T402 575Q402 562 395 464Q389 373 388 361T378 341Q370 333 350 333Q331 333 323 341Q315 348 314 360T306 464Q299 561 299 575",
      35: "93 163Q74 163 65 164T46 173T36 198Q36 210 40 215T61 233H131V236Q132 239 140 307T149 377Q149 379 105 379L61 380Q36 392 36 414Q36 450 86 450Q91 450 99 450T112 449H159Q163 480 167 517Q168 524 170 545T174 573T180 591T191 607T210 611Q223 611 232 604T243 588L245 580Q245 565 238 511T230 451Q230 449 282 449H333Q337 480 341 517Q342 524 343 537T345 556T348 573T352 589T359 600T370 608T384 611Q395 611 406 602T419 580Q419 565 412 511T404 451Q404 449 431 449H442Q477 449 485 429Q489 421 489 414Q489 392 463 380L428 379H394V376Q393 373 385 305T376 235Q376 233 419 233H463L468 230Q472 227 473 227T477 223T482 218T486 213T488 206T489 198Q489 162 436 162Q430 162 422 162T412 163H366V161Q364 159 357 92Q356 85 355 73T353 54T350 37T346 22T339 11T328 3T314 0Q303 0 292 9T279 31Q279 37 287 96T295 162Q295 163 244 163H192V161Q190 159 183 92Q182 85 181 73T179 54T176 37T172 22T165 11T154 3T140 0Q129 0 118 9T105 31Q105 37 113 96T121 162Q121 163 93 163ZM323 377Q323 379 272 379H220V376Q219 373 211 305T202 235Q202 233 253 233H305V236Q306 239 314 307T323 377",
      36: "415 397Q392 397 377 411T362 448Q362 464 376 485Q369 498 362 506T346 520T332 528T315 533T300 538V445L301 353L311 350Q382 334 424 284T466 174Q466 115 425 65T303 -2L300 -3V-30Q300 -64 291 -74Q283 -82 262 -82H255Q234 -82 225 -60L224 -32V-4L213 -2Q152 6 106 51T59 170V180Q59 197 74 213Q89 227 110 227T146 213T162 174Q162 156 147 137Q153 123 161 112T176 95T191 85T205 79T216 76T224 74V283L213 285Q147 298 103 343T58 449Q58 516 108 560T224 614V643V654Q224 666 226 673T237 687T264 694Q289 693 294 683T300 642V615H303Q355 607 390 587T440 540T460 493T466 453Q466 425 451 411T415 397ZM137 452Q137 425 158 404T198 376T223 369Q224 369 224 453T223 537Q198 532 168 509T137 452ZM301 75Q307 75 325 83T365 116T387 171Q387 238 300 267V171Q300 75 301 75",
      37: "35 560Q35 607 54 645T110 693Q111 693 116 693T125 694Q165 692 187 651T210 560Q210 506 186 467T123 428Q84 428 60 466T35 560ZM139 560Q139 574 136 587T130 608T124 615Q122 617 120 614Q106 595 106 561Q106 516 121 506Q123 504 125 507Q139 526 139 560ZM123 -83Q107 -83 98 -73T88 -48Q88 -43 89 -41Q90 -37 229 316T370 675Q381 694 400 694Q416 694 426 684T436 659Q436 654 435 652Q434 647 295 294T153 -65Q144 -83 123 -83ZM314 50Q314 104 338 143T400 183Q439 183 464 144T489 50T465 -43T402 -82Q358 -82 336 -41T314 50ZM417 50Q417 71 413 85T405 102L401 106Q386 95 386 50Q386 29 390 15T398 -2L402 -6Q417 5 417 50",
      38: "96 462Q96 546 132 584T211 622Q255 622 284 583T314 474Q314 395 224 305L208 288Q213 275 226 251L265 185L269 179Q273 184 299 246L332 333L342 363Q342 364 341 365Q334 365 334 393Q334 406 334 410T340 420T356 431H412H440Q467 431 478 424T490 393Q490 376 484 367T470 357T448 355H441H415L399 312Q349 176 322 127L315 115L323 106Q360 65 393 65Q405 65 410 80T416 109Q416 140 452 140Q487 140 487 105Q487 56 460 23T391 -11L286 41L273 53L262 42Q212 -11 151 -11Q97 -11 63 33T28 143Q28 161 30 176T38 205T47 227T60 247T72 261T84 274T94 283L122 311L119 323Q96 392 96 462ZM243 474Q243 533 218 545L215 546Q212 546 210 546Q182 546 169 501Q167 492 167 466Q167 419 179 368L188 377Q234 425 242 461Q243 465 243 474ZM217 129Q185 174 154 235Q121 214 115 176Q113 168 113 143Q113 83 139 67Q141 66 152 66Q191 66 228 112L217 129",
      39: "205 554Q205 577 221 594T263 611Q302 611 325 577T349 490Q349 409 298 347Q285 330 258 309T214 287Q203 289 189 302T175 327Q175 341 185 349T213 369T245 402Q269 437 273 483V497Q264 496 263 496Q240 496 223 513T205 554",
      40: "437 -53Q437 -82 399 -82H394Q377 -82 342 -55Q259 7 213 102T166 306Q166 412 211 507T342 667Q377 694 393 694H399Q437 694 437 665Q437 654 426 643T397 620T356 584T311 525Q301 511 290 488T264 412T250 306Q250 191 300 105T422 -27Q437 -37 437 -53",
      41: "87 664Q87 694 126 694Q138 694 147 690T183 667Q266 605 312 510T358 306Q358 193 307 93T161 -70Q142 -82 126 -82Q105 -82 96 -73T87 -53Q87 -47 88 -44Q92 -36 116 -19T173 34T230 119Q273 206 273 306Q273 408 231 494T109 635Q87 649 87 664",
      42: "222 487Q224 501 235 510T262 520Q279 520 289 510T302 487Q302 458 301 429Q301 421 301 413T301 398T300 386T300 377V374Q300 373 301 373Q304 373 353 403T416 434Q432 434 444 423T456 393Q456 389 456 386T454 379T451 373T448 368T442 363T436 358T427 353T417 348T405 342T391 334Q345 309 339 305L388 279Q400 273 412 266T432 255T441 250Q456 238 456 218Q456 200 445 189T417 177Q403 177 354 207T301 238Q300 238 300 237V234Q300 231 300 226T300 214T301 199T301 182Q302 153 302 124Q300 109 289 100T262 90T235 100T222 124Q222 153 223 182Q223 190 223 198T223 213T224 225T224 234V237Q224 238 223 238Q220 238 171 208T108 177Q92 177 80 188T68 218Q68 237 79 246T134 277Q180 303 185 306L136 332Q124 338 112 345T92 356T83 361Q68 373 68 393Q68 411 79 422T107 434Q121 434 170 404T223 373Q224 373 224 374V377Q224 380 224 385T224 397T223 412T223 429Q222 458 222 487",
      43: "147 271Q138 271 122 271T98 270Q68 270 53 277T38 306T53 335T98 342Q105 342 121 342T147 341H227V423L228 505Q241 531 262 531Q268 531 273 530T282 525T287 519T293 511L297 505V341H377H430Q457 341 467 338T483 321Q487 313 487 306Q487 295 480 286T463 273Q457 271 377 271H297V107Q281 81 262 81Q250 81 242 87T230 100L228 107L227 189V271H147",
      44: "193 37T193 70T213 121T260 140Q302 140 327 108T353 36Q353 -7 336 -43T294 -98T249 -128T215 -139Q204 -139 189 -125Q177 -111 174 -101Q172 -84 183 -77T217 -61T253 -33Q261 -24 272 1L265 0Q234 0 214 18",
      45: "57 306Q57 333 86 341H438Q468 332 468 306T438 271H86Q57 280 57 306",
      46: "193 70Q193 105 214 122T258 140Q291 140 311 120T332 70Q332 44 314 23T262 1Q234 1 214 18T193 70",
      47: "94 -83Q78 -83 68 -73T58 -48Q58 -44 60 -36Q62 -31 227 314T399 673Q410 694 431 694Q445 694 455 684T466 659Q466 656 464 648Q463 643 298 298T125 -62Q114 -83 94 -83",
      58: "193 361Q193 396 214 413T258 431Q291 431 311 411T332 361Q332 335 314 314T262 292Q234 292 214 309T193 361ZM193 70Q193 105 214 122T258 140Q291 140 311 120T332 70Q332 44 314 23T262 1Q234 1 214 18T193 70",
      59: "193 361Q193 396 214 413T258 431Q291 431 311 411T332 361Q332 335 314 314T262 292Q234 292 214 309T193 361ZM193 70Q193 105 214 122T259 140Q301 140 319 108T337 33Q337 -38 291 -88T214 -139Q203 -139 189 -126T175 -97Q175 -85 182 -78T200 -66T225 -50T249 -17Q256 -3 256 0Q252 1 248 1Q242 2 235 5T218 15T200 36T193 70",
      60: "468 90Q468 76 458 66T433 55Q426 55 419 58Q413 61 243 168T68 280Q57 291 57 306T68 332Q72 335 241 442T416 553Q424 557 432 557Q447 557 457 547T468 522T456 496Q454 494 305 399L158 306L305 213Q341 190 390 159Q443 125 452 119T464 106V105Q468 97 468 90",
      61: "38 382Q38 409 67 417H457Q487 408 487 382Q487 358 461 348H64Q51 352 45 360T38 376V382ZM67 195Q38 204 38 230Q38 255 62 264Q66 265 264 265H461L464 264Q467 262 469 261T475 256T481 249T485 240T487 230Q487 204 457 195H67",
      62: "57 522Q57 539 67 548T90 557Q98 557 105 554Q111 551 281 444T456 332Q468 320 468 306T456 280Q452 276 282 169T105 58Q98 55 91 55Q79 55 68 63T57 90Q57 105 68 116Q70 118 219 213L366 306L219 399Q75 491 71 494Q57 507 57 522",
      63: "62 493Q62 540 107 578T253 617Q366 617 414 578T462 490Q462 459 445 434T411 400L394 390Q315 347 296 287Q294 278 293 247V217Q285 201 278 198T246 194T216 197T201 215V245V253Q201 379 351 456Q366 464 375 477Q377 482 377 490Q377 517 339 528T251 540Q182 540 159 517Q166 503 166 490Q166 468 151 453T114 438Q96 438 79 451T62 493ZM190 58Q190 85 208 100T249 115Q272 113 288 99T304 58Q304 33 287 17T246 0T206 16T190 58",
      64: "44 306Q44 445 125 531T302 617Q332 617 358 607T411 574T456 502T479 387Q481 361 481 321Q481 203 421 143Q381 103 332 103Q266 103 225 165T183 307Q183 390 227 449T332 508Q358 508 378 498Q350 541 304 541Q229 541 172 473T115 305Q115 208 171 140T306 71H310Q358 71 397 105Q409 115 436 115Q458 115 462 113Q481 106 481 86Q481 73 468 61Q401 -6 305 -6Q262 -6 217 14T133 71T69 170T44 306ZM410 306Q410 361 386 396T333 431Q300 431 277 394T254 305Q254 256 276 218T332 180Q364 180 387 217T410 306",
      91: "237 -82Q221 -78 214 -58V305Q214 669 216 673Q220 687 231 690T278 694H350H461Q462 693 467 690T474 685T478 679T482 670T483 656Q483 632 471 625T428 617Q422 617 406 617T379 618H298V-7H379H420Q459 -7 471 -13T483 -45Q483 -55 483 -59T477 -70T461 -82H237",
      92: "58 659Q58 673 68 683T93 694Q114 694 125 673Q132 659 297 314T464 -36Q466 -44 466 -48Q466 -66 454 -74T431 -83Q410 -83 399 -62Q391 -47 226 298T60 648Q58 656 58 659",
      93: "41 656Q41 681 53 688T99 695Q107 695 133 695T177 694H288Q307 681 310 669V-58Q303 -76 288 -82H64Q41 -73 41 -45Q41 -21 53 -14T96 -6Q102 -6 118 -6T145 -7H226V618H145H100Q67 618 54 625T41 656",
      94: "138 460Q121 460 109 479T96 512Q96 527 106 534Q109 536 178 571T253 609Q256 611 264 611Q272 610 343 574Q357 567 369 561T389 550T402 543T411 538T416 535T420 532T422 529T425 525Q428 518 428 512Q428 498 416 479T386 460H384Q377 460 316 496L262 526L208 496Q147 460 138 460",
      95: "57 -60Q57 -33 86 -25H438Q468 -34 468 -60T438 -95H86Q57 -86 57 -60",
      96: "176 479Q176 563 227 622T310 681Q324 680 337 667T350 641Q350 627 340 619T312 599T280 566Q256 531 252 485V471Q261 472 262 472Q285 472 302 455T320 414Q320 389 303 373T261 357Q223 357 200 391T176 479",
      123: "430 -7H436Q449 -7 456 -8T469 -19T475 -45Q475 -69 466 -76T434 -83H419Q386 -82 363 -80T308 -69T253 -41T223 7L221 17L220 118V220L218 224Q215 229 214 230T210 235T204 241T195 246T184 252T170 257T151 262T127 265Q118 267 100 267T69 270T52 283Q50 288 50 306V314Q50 335 67 341Q68 342 102 343T172 355T217 386L220 392V493L221 595Q225 611 230 621T251 650T304 679T395 693L406 694Q418 694 426 694Q458 694 466 685Q475 676 475 656T466 627Q458 618 430 618Q319 618 305 587L304 486Q304 476 304 458T305 431Q305 385 295 358T251 311L243 306Q243 305 254 298T281 274T302 231Q304 223 304 125L305 25Q309 16 316 10T352 -1T430 -7",
      124: "228 668Q241 694 262 694Q268 694 273 693T282 688T287 682T293 674L297 668V-57Q282 -82 262 -82Q239 -82 228 -57V668",
      125: "49 655Q49 674 56 682T73 692T106 694Q141 693 167 690T224 677T275 647T303 595L305 392Q313 367 347 356T417 344T457 341Q475 335 475 306Q475 292 473 285T464 273T451 269T430 267Q352 262 327 246Q311 236 305 220L303 17L301 7Q294 -16 277 -33T242 -60T196 -74T150 -80T106 -83Q78 -83 72 -82T58 -74Q49 -65 49 -44Q49 -24 58 -16Q66 -7 94 -7Q143 -7 171 -1T207 10T220 25V125Q220 223 222 231Q228 257 243 274T270 299L281 306Q234 329 222 381Q220 387 220 486V587Q212 597 207 601T173 612T94 618Q66 618 58 627Q49 635 49 655",
      126: "125 467Q113 467 100 480T87 509Q88 520 111 543Q172 602 209 609Q219 611 224 611Q246 611 263 596T290 566T304 551Q319 551 367 594Q383 610 396 610H400Q411 610 424 597T437 568Q436 557 413 534Q348 469 305 466Q278 466 260 481T234 511T220 526Q205 526 157 483Q141 467 129 467H125",
      127: "104 565Q104 590 120 600T155 611Q175 611 180 610Q217 599 217 565Q217 545 202 532T166 519H159H155Q120 519 107 547Q104 553 104 565ZM307 565Q307 580 317 593T346 610Q348 610 350 610T354 611Q355 612 367 612Q395 611 408 597T421 565T409 534T365 519H358Q336 519 322 532T307 565",
      160: "",
      305: "411 76Q441 76 451 69T462 38Q462 29 462 26T460 18T453 9T440 1H94Q72 8 72 33V38Q72 46 72 49T74 58T81 68T94 76H233V355H167L102 356Q80 363 80 393Q80 418 91 425T138 432Q145 432 165 432T200 431H295Q297 429 303 425T310 420T314 415T317 404T317 389T318 363Q318 354 318 314T317 241V76H378H411",
      567: "75 -91T100 -91T138 -107T152 -144V-150L160 -151H193H203Q241 -151 267 -121Q284 -97 288 -73T292 23V151V355H218L145 356Q123 365 123 387V393Q123 422 145 430H148Q151 430 156 430T169 430T185 430T205 431T227 431T251 431H354Q356 430 360 427T365 424T369 420T372 416T373 410T375 402T376 391T377 376T377 356Q377 345 377 286T376 176Q376 -67 371 -88Q362 -123 342 -151T299 -194Q254 -228 180 -228Q84 -226 56 -177Q49 -162 48 -148Q48 -122 61 -107",
      697: "211 572Q211 593 226 608T262 623Q281 623 297 610T313 573Q313 561 307 465Q301 370 299 357T284 336Q279 334 262 334Q240 334 231 343Q226 350 225 362T217 465Q211 549 211 572",
      768: "-409 569Q-409 586 -399 596T-377 610Q-376 610 -372 610T-365 611Q-355 610 -284 588T-210 563Q-195 556 -195 537Q-195 533 -197 522T-208 498T-229 485Q-238 485 -312 508T-388 533Q-400 538 -405 552Q-409 559 -409 569",
      769: "-297 485Q-315 485 -323 505T-331 537Q-331 556 -316 563Q-307 569 -170 610Q-169 610 -165 610T-157 611Q-141 609 -131 600T-119 584T-117 569Q-117 555 -124 545T-138 533Q-140 531 -214 508T-297 485",
      770: "-387 460Q-404 460 -416 479T-429 512Q-429 527 -419 534Q-416 536 -347 571T-272 609Q-269 611 -261 611Q-254 610 -182 574Q-168 567 -156 561T-136 550T-123 543T-114 538T-109 535T-105 532T-103 529T-100 525Q-97 518 -97 512Q-97 498 -109 479T-139 460H-141Q-148 460 -209 496L-263 526L-317 496Q-378 460 -387 460",
      771: "-400 467Q-412 467 -425 480T-438 509Q-437 520 -414 543Q-353 602 -316 609Q-306 611 -301 611Q-279 611 -262 596T-235 566T-221 551Q-206 551 -158 594Q-142 610 -129 610H-125Q-114 610 -101 597T-88 568Q-89 557 -112 534Q-177 469 -220 466Q-247 466 -265 481T-291 511T-305 526Q-320 526 -368 483Q-384 467 -396 467H-400",
      772: "-429 500Q-440 504 -445 511T-450 522T-452 536Q-452 552 -451 556Q-445 571 -434 574T-379 578Q-369 578 -330 578T-261 577H-96Q-94 575 -90 573T-85 569T-81 564T-77 558T-75 550T-74 538Q-74 522 -78 515T-96 500H-429",
      774: "-446 579Q-446 611 -412 611H-407Q-383 609 -378 599T-358 587Q-340 583 -263 583H-235Q-159 583 -152 593Q-145 611 -120 611H-117H-115Q-79 611 -79 577Q-80 552 -95 536T-140 514T-191 506T-251 504H-263H-274Q-311 504 -334 505T-386 513T-431 536T-446 579",
      776: "-421 565Q-421 590 -405 600T-370 611Q-350 611 -345 610Q-308 599 -308 565Q-308 545 -323 532T-359 519H-366H-370Q-405 519 -418 547Q-421 553 -421 565ZM-218 565Q-218 580 -208 593T-179 610Q-177 610 -175 610T-171 611Q-170 612 -158 612Q-130 611 -117 597T-104 565T-116 534T-160 519H-167Q-189 519 -203 532T-218 565",
      778: "-344 558Q-344 583 -321 601T-262 619Q-225 619 -204 600T-182 560Q-182 536 -205 518T-264 499Q-301 499 -322 519T-344 558ZM-223 559Q-223 570 -234 579T-261 588T-289 580T-303 559Q-303 549 -293 540T-263 530T-234 539T-223 559",
      780: "-427 525Q-427 542 -417 559T-392 577Q-385 577 -323 553L-263 530L-203 553Q-143 576 -136 576Q-118 576 -109 559T-99 525Q-99 508 -107 502T-161 481Q-177 475 -186 472Q-256 449 -263 449Q-272 449 -339 472T-412 498Q-420 501 -423 508T-427 520V525",
      913: "191 76Q212 75 220 68T229 38Q229 10 208 1H129H80Q48 1 38 7T28 38Q28 51 29 57T40 69T70 76Q89 76 89 78Q90 79 117 205T173 461T205 599Q212 623 250 623H262H273Q312 623 319 599Q322 591 350 461T406 205T435 78Q435 76 454 76H458Q484 76 493 59Q496 53 496 38Q496 11 478 3Q474 1 395 1H317Q295 8 295 38Q295 65 311 73Q316 75 333 76L348 77V78Q348 80 341 112L334 143H190L183 112Q176 80 176 78Q175 76 178 76Q180 76 191 76ZM318 221Q313 238 288 366T263 519Q263 526 262 527Q261 527 261 520Q261 493 236 365T206 221Q206 219 262 219T318 221",
      914: "39 1Q17 10 17 32V38V46Q17 65 34 73Q40 76 61 76H84V535H61H54Q27 535 19 553Q17 557 17 573Q17 583 17 587T23 599T39 610Q40 611 179 611Q320 610 332 607Q332 607 339 605Q394 591 427 547T461 454Q461 413 436 378T369 325L358 320Q405 311 443 270T482 169Q482 112 445 64T345 3L334 1H39ZM309 533Q302 535 234 535H168V356H230Q284 357 296 358T323 368Q346 380 361 402T377 452Q377 482 358 505T309 533ZM398 176Q396 218 371 246T315 279Q310 280 237 280H168V76H239Q316 77 327 81Q329 82 334 84Q398 107 398 176",
      915: "466 611Q468 609 473 606T479 602T483 598T486 593T487 586T488 576T488 562V526V488Q488 452 470 444Q466 442 446 442Q421 442 413 450Q406 457 405 463T404 501V535H185V76H222H239Q260 76 270 69T281 38Q281 12 270 6T209 0H155H104Q48 0 37 5T25 38Q25 59 35 69Q44 76 76 76H101V535H76H64Q36 535 27 552Q25 557 25 573T27 594Q33 606 43 608T106 611H258H466",
      916: "232 622H237Q242 622 249 622T264 623H293Q295 622 300 619T308 613T314 608T319 601Q322 597 405 316T489 19Q489 9 473 1Q471 0 262 0T51 1Q35 9 35 19Q35 34 118 315T205 601Q214 616 232 622ZM267 501Q266 504 265 510T263 521T261 526V523Q261 508 211 332Q142 91 138 82H386Q385 84 345 224Q281 439 267 501",
      917: "374 271Q374 241 367 232T332 223Q307 223 299 231Q290 240 290 263V279H173V76H418V118V144Q418 167 426 176T460 186Q491 186 500 166Q502 161 502 93V52Q502 25 499 17T480 1H41Q19 9 19 32V38Q19 63 36 73Q42 76 65 76H89V535H65H55Q44 535 38 537T25 548T19 573Q19 602 41 610H47Q53 610 63 610T88 610T121 610T160 611T204 611T251 611H458Q460 609 465 606T471 602T475 598T478 593T479 586T480 576T480 562V526V488Q480 452 462 444Q458 442 438 442Q413 442 405 450Q398 457 397 463T396 501V535H173V355H290V371Q290 394 299 403T332 412Q363 412 372 392Q374 387 374 317V271",
      918: "71 1Q60 5 55 11T49 23T48 39V46Q48 56 58 73T131 183Q171 242 197 282L366 535H144V501Q144 470 143 464T135 450Q127 442 102 442H94Q71 442 62 461Q60 466 60 527L61 589Q70 607 83 610H88Q93 610 102 610T124 610T154 610T188 611T227 611T270 611H454Q456 609 461 606T467 601T471 597T474 591T475 584T476 572V565Q476 555 466 538T393 428Q353 369 327 329L158 76H397V120V146Q397 169 405 179T439 189Q470 189 479 169Q481 164 481 95V48Q481 24 478 16T459 1H71",
      919: "16 571Q16 597 27 604T74 611H125H208Q223 602 226 596T230 573Q230 559 227 551T217 540T204 536T186 535H165V356H359V535H338H333Q306 535 297 552Q295 556 295 573Q295 586 295 590T301 600T317 611H486Q501 602 504 596T508 573Q508 559 505 551T495 540T482 536T464 535H443V76H464H470Q482 76 489 75T502 64T508 38Q508 10 486 1H317Q306 5 301 11T296 21T295 38V44Q295 66 311 73Q318 76 338 76H359V280H165V76H186H192Q204 76 211 75T224 64T230 38Q230 10 208 1H39Q28 5 23 11T18 21T17 38V44Q17 66 33 73Q40 76 60 76H81V535H60Q45 535 38 536T24 545T16 571",
      920: "102 588Q140 621 240 621Q323 621 335 620Q393 613 422 588Q450 560 459 493T468 306Q468 185 460 118T422 23Q382 -10 289 -10H262H235Q142 -10 102 23Q74 50 65 118T56 306Q56 427 64 494T102 588ZM262 66Q285 66 300 67T329 74T351 86T366 108T376 138T381 181T383 235T384 306Q384 452 371 492T304 544Q296 545 251 545Q230 545 215 543T188 534T169 520T155 497T147 466T143 423T141 371T140 306Q140 247 141 215T146 151T158 107T179 82T212 69T262 66ZM179 356Q187 378 219 378H223Q240 377 249 372T260 360L261 355Q261 353 262 353T263 355Q263 362 272 369Q280 377 304 377H310Q325 377 331 374T346 356V256Q338 241 331 238T309 234H304Q280 234 272 242Q263 249 263 256Q263 258 262 258T261 256Q261 249 252 242Q244 234 220 234H216Q186 234 179 256V356",
      921: "400 76Q431 76 441 69T452 38Q452 29 452 26T450 18T443 9T430 1H95Q84 6 79 12T73 23T72 38Q72 65 90 73Q96 76 157 76H220V535H157H124Q93 535 83 542T72 573Q72 603 93 610Q97 611 264 611H430Q432 609 436 607T444 602T449 594Q452 588 452 573Q452 546 434 538Q428 535 367 535H304V76H367H400",
      922: "18 549T18 573T29 604T70 611H118H193Q207 603 210 596T214 573Q214 549 198 538Q191 535 172 535H152V421Q152 344 152 326T153 309L242 422L329 534Q327 535 322 536T314 538T308 542T303 548T300 558T298 573Q298 600 316 608Q322 611 392 611H463Q477 602 481 595T485 573Q485 535 446 535H441H420L281 357L436 77L454 76Q473 75 478 73Q495 62 495 38Q495 10 473 1H345Q334 5 329 11T324 21T323 38Q323 51 324 56T332 68T355 77L233 296L152 192V76H172Q191 76 198 73Q214 63 214 38Q214 9 193 1H41Q18 8 18 38Q18 61 35 73Q42 76 61 76H81V535H61Q42 535 35 538Q18 549 18 573",
      923: "30 38Q30 57 38 66T70 76Q88 76 88 78Q89 79 117 207T173 466T205 602Q213 617 231 622H236Q241 622 249 622T264 623H294Q315 609 319 602Q321 598 350 468T407 208T435 78Q436 76 454 76Q470 76 478 73Q495 62 495 38Q495 10 473 1H313Q290 10 290 38Q290 56 297 65T310 74T331 76Q350 76 350 78Q349 80 328 176T285 383T263 520Q263 526 262 527Q261 527 261 521Q261 497 240 388T198 181T174 78Q174 76 193 76Q220 75 227 65Q234 56 234 38Q234 28 234 24T228 13T212 1H52Q30 9 30 32V38",
      924: "50 535Q37 536 31 537T18 547T12 573Q12 598 22 604T62 611H91H121Q147 611 158 607T178 587Q183 579 222 446T261 293Q261 289 262 288Q263 288 263 292Q263 311 298 434T346 588Q353 603 365 607T402 611H435H450Q488 611 500 605T512 573Q512 556 506 547T493 537T474 535H459V76H474Q487 75 493 74T505 64T512 38Q512 11 494 3Q490 1 424 1H386Q355 1 345 7T335 38Q335 55 341 64T354 74T373 76H388V302Q388 512 387 519Q382 482 346 359T304 228Q292 204 262 204T220 228Q215 237 179 359T137 519Q136 512 136 302V76H151Q164 75 170 74T182 64T189 38Q189 11 171 3Q167 1 101 1H63Q32 1 22 7T12 38Q12 55 18 64T31 74T50 76H65V535H50",
      925: "20 571Q20 598 30 604T73 611H105H136Q152 611 160 611T177 607T189 601T198 587T206 568T217 537T231 497Q354 142 365 95L368 84V535H347H342Q314 535 306 552Q304 556 304 573Q304 586 304 590T310 600T326 611H482Q497 602 500 596T504 573Q504 559 501 551T491 540T478 536T460 535H439V25Q432 7 424 4T389 0H374Q334 0 322 31L293 115Q171 468 159 517L156 528V76H177H183Q195 76 202 75T215 64T221 38Q221 10 199 1H43Q32 5 27 11T22 21T21 38V44Q21 66 37 73Q44 76 64 76H85V535H64Q49 535 42 536T28 545T20 571",
      926: "37 555V569Q37 605 60 610H66Q71 610 81 610T105 610T137 610T175 611T217 611T264 611H465Q467 609 471 606T477 602T481 599T484 594T485 588T487 580T487 570T487 554Q487 526 486 520T478 506Q470 498 445 498T412 506Q403 515 403 531V539H121V531Q121 498 86 498H79H71Q48 498 39 517Q37 522 37 555ZM109 318V346Q109 366 113 374T132 389H170Q193 379 193 359V354H331V359Q331 379 354 389H392Q407 381 411 373T415 342V318V290Q415 270 411 262T392 247H354Q331 257 331 277V282H193V277Q193 257 170 247H132Q117 255 113 263T109 294V318ZM56 1Q41 7 37 15T33 42V58V80Q33 101 41 110T77 119Q87 118 91 118T103 114T114 103T117 83V72H407V83Q407 101 416 110T449 119T482 110Q489 103 490 97T491 59V41Q491 24 487 16T469 1H56",
      927: "102 588Q140 621 240 621Q323 621 335 620Q393 613 422 588Q450 560 459 493T468 306Q468 185 460 118T422 23Q382 -10 289 -10H262H235Q142 -10 102 23Q74 50 65 118T56 306Q56 427 64 494T102 588ZM363 513Q357 523 347 530T324 540T302 544T280 546H268Q192 546 167 521Q150 501 145 452T140 300Q140 235 142 197T151 130T172 89T207 71T262 65Q317 65 341 81T374 144T384 300Q384 474 363 513",
      928: "60 535Q45 535 38 536T24 545T16 571Q16 603 36 609Q41 611 264 611H486Q501 602 504 596T508 573Q508 559 505 551T495 540T482 536T464 535H443V76H464H470Q482 76 489 75T502 64T508 38Q508 10 486 1H317Q306 5 301 11T296 21T295 38V44Q295 66 311 73Q318 76 338 76H359V535H165V76H186H192Q204 76 211 75T224 64T230 38Q230 10 208 1H39Q28 5 23 11T18 21T17 38V44Q17 66 33 73Q40 76 60 76H81V535H60",
      929: "41 1Q19 9 19 32V38Q19 63 36 73Q42 76 65 76H89V535H65H55Q38 535 29 543T19 576Q19 603 41 610H49Q57 610 70 610T100 610T136 611T175 611Q190 611 216 611T255 612Q321 612 363 598T441 537Q480 486 480 427V421Q480 354 447 311T378 251Q339 230 275 230H239H173V76H197Q220 76 227 73Q244 62 244 38Q244 10 222 1H41ZM396 421Q396 461 369 491T300 533Q294 534 233 535H173V306H233Q294 307 300 308Q345 319 370 352T396 421",
      931: "40 575Q40 576 40 579T41 583T41 588T43 593T46 597T50 602T55 606T63 610H68Q74 610 84 610T108 610T139 610T176 611T219 611T264 611H462Q464 609 469 606T475 602T479 598T482 593T483 586T484 576T484 562V526V488Q484 452 466 444Q462 442 442 442Q417 442 409 450Q402 457 401 463T400 501V535H153Q153 533 218 430Q233 405 250 378T276 336T286 319Q290 311 290 307Q290 296 239 211Q229 194 223 184L161 78H400V112Q400 142 401 149T409 163Q418 172 442 172Q473 172 482 152Q484 147 484 86V49Q484 25 481 17T462 1H63Q41 10 41 31Q41 39 43 44Q43 45 81 109T157 238L195 303Q195 307 119 430T41 557T40 575",
      932: "129 38Q129 51 129 55T135 65T151 76H220V535H110V501Q110 470 109 464T101 450Q93 442 68 442H60Q37 442 28 461Q26 466 26 527L27 589Q36 607 49 610H55Q61 610 72 610T97 610T131 610T170 611T215 611T264 611H476Q478 609 483 606T489 602T493 598T496 593T497 586T498 576T498 562V526V488Q498 452 480 444Q476 442 456 442Q431 442 423 450Q416 457 415 463T414 501V535H304V76H374Q389 67 392 61T396 38Q396 10 374 1H151Q140 5 135 11T130 21T129 38",
      933: "38 494Q38 549 74 585T152 621Q168 621 179 619T209 606T241 566T262 492Q262 494 265 507T270 526T276 547T285 569T298 589T315 606T337 617T365 622Q416 622 451 584T486 494Q486 470 469 461Q464 459 445 459H437Q416 459 406 476Q404 479 403 502T393 541T365 558Q350 558 340 548T323 519T312 475T307 419T305 354T304 282Q304 254 304 239V76H358Q372 67 376 60T380 38Q380 10 358 1H167Q145 9 145 32V38Q145 54 148 60T167 76H220V239Q220 256 220 289T220 338T219 383T217 426T214 463T209 497T201 522T189 543T174 555Q168 558 159 558Q139 558 131 541T121 502T118 476Q108 459 84 459H79H71Q38 459 38 494",
      934: "139 573V578Q139 603 161 610H166Q172 610 182 610T204 610T232 611T264 611H364Q379 602 382 595T385 573Q385 544 364 536L334 535H304V441H306Q313 440 325 438T367 426T421 403T464 364T483 306Q483 251 430 216T317 172Q315 172 313 172T308 170H306H304V76H364Q379 67 382 60T385 38Q385 28 385 24T379 12T364 1H161Q139 8 139 33V38Q139 46 139 49T141 58T148 68T161 76H220V170H218Q211 171 199 173T157 185T103 208T60 248T41 306Q41 361 94 396T208 439Q210 439 212 439T216 440L218 441H220V535H190L161 536Q139 543 139 573ZM124 306Q124 286 147 271T194 252L218 247Q220 247 220 306V364H218Q212 364 192 359T148 340T124 306ZM400 305Q400 325 377 340T330 360L306 364Q304 364 304 306Q304 247 306 247Q312 247 332 252T376 271T400 305",
      935: "39 571Q39 597 49 604T93 611H141H218Q233 602 236 595T239 573Q239 538 210 535Q202 535 202 534T215 507T243 454L257 428L307 535H298Q266 538 266 573Q266 584 267 588T273 598T289 611H366H401Q442 611 454 605T466 573Q466 546 448 538Q442 535 421 535H398L299 327Q299 323 362 201L426 77L449 76Q467 76 475 75T489 65T495 38Q495 11 477 3Q473 1 395 1H317Q295 8 295 38Q295 73 325 76L334 77Q333 78 314 117T276 196L257 235L239 196Q221 157 204 118T186 77Q190 76 196 76Q211 74 218 67T227 55T228 38Q228 28 227 24T221 13T206 1H50Q28 9 28 32V38Q28 63 45 73Q51 76 73 76H96L214 324Q215 327 162 431L108 535H85H79Q67 535 60 536T46 546T39 571",
      936: "37 439Q38 451 40 457T52 469T77 475H79Q96 475 107 473T132 456T152 411Q152 409 153 396T154 372V365Q154 291 198 261Q215 251 219 251Q220 251 220 393V535H193L167 536Q145 545 145 567V573Q145 602 167 610Q168 611 264 611H358Q372 602 376 595T380 573Q380 545 358 536L331 535H304V393Q304 251 305 251Q307 251 310 252T323 259T339 272T355 295T367 331Q368 337 370 372Q370 382 371 395T372 411Q376 434 384 448T404 467T425 474T447 475Q461 474 467 473T480 463T487 437Q487 419 481 412Q476 403 459 398Q457 390 453 344T431 263Q415 228 383 205T332 177T306 172H304V76H358Q372 67 376 60T380 38Q380 10 358 1H167Q145 9 145 32V38Q145 54 148 60T167 76H220V172H218Q211 172 192 177T141 205T93 263Q74 298 71 343T67 391L66 398Q47 403 42 411T37 433V439",
      937: "40 404Q40 498 106 560T258 622Q357 622 420 558T484 406Q484 359 469 311T428 205T392 117Q382 84 382 78Q382 76 402 76H421V87Q421 110 431 116T457 123Q474 123 483 114Q490 107 491 100T492 61V42Q492 11 474 3Q470 1 397 1H324Q302 9 302 32V39Q302 104 351 225T400 405Q400 462 361 504T262 546Q200 546 162 504T124 405Q124 346 171 230T223 42V36Q223 11 205 3Q201 1 128 1H55Q39 7 33 23L32 60V80Q32 94 34 102T44 116T68 123Q103 123 103 87V76H123Q142 76 142 78Q142 100 117 156T66 282T40 404",
      8215: "57 -60Q57 -33 86 -25H438Q468 -34 468 -60T438 -95H86Q57 -86 57 -60",
      8242: "211 572Q211 593 226 608T262 623Q281 623 297 610T313 573Q313 561 307 465Q301 370 299 357T284 336Q279 334 262 334Q240 334 231 343Q226 350 225 362T217 465Q211 549 211 572",
      8260: "94 -83Q78 -83 68 -73T58 -48Q58 -44 60 -36Q62 -31 227 314T399 673Q410 694 431 694Q445 694 455 684T466 659Q466 656 464 648Q463 643 298 298T125 -62Q114 -83 94 -83",
      8710: "232 622H237Q242 622 249 622T264 623H293Q295 622 300 619T308 613T314 608T319 601Q322 597 405 316T489 19Q489 9 473 1Q471 0 262 0T51 1Q35 9 35 19Q35 34 118 315T205 601Q214 616 232 622ZM267 501Q266 504 265 510T263 521T261 526V523Q261 508 211 332Q142 91 138 82H386Q385 84 345 224Q281 439 267 501"
    }, {
      8243: "′′",
      8244: "′′′",
      8279: "′′′′"
    }), monospace$1;
  }
  var normal$1 = {}, normal = {}, hasRequiredNormal$1;
  function requireNormal$1() {
    return hasRequiredNormal$1 || (hasRequiredNormal$1 = 1, Object.defineProperty(normal, "__esModule", { value: !0 }), normal.normal = void 0, normal.normal = {
      32: [0, 0, 0.25],
      33: [0.716, 0, 0.278],
      34: [0.694, -0.379, 0.5],
      35: [0.694, 0.194, 0.833],
      36: [0.75, 0.056, 0.5],
      37: [0.75, 0.056, 0.833],
      38: [0.716, 0.022, 0.778],
      39: [0.694, -0.379, 0.278],
      40: [0.75, 0.25, 0.389],
      41: [0.75, 0.25, 0.389],
      42: [0.75, -0.32, 0.5],
      43: [0.583, 0.082, 0.778],
      44: [0.121, 0.194, 0.278],
      45: [0.252, -0.179, 0.333],
      46: [0.12, 0, 0.278],
      47: [0.75, 0.25, 0.5],
      48: [0.666, 0.022, 0.5],
      49: [0.666, 0, 0.5],
      50: [0.666, 0, 0.5],
      51: [0.665, 0.022, 0.5],
      52: [0.677, 0, 0.5],
      53: [0.666, 0.022, 0.5],
      54: [0.666, 0.022, 0.5],
      55: [0.676, 0.022, 0.5],
      56: [0.666, 0.022, 0.5],
      57: [0.666, 0.022, 0.5],
      58: [0.43, 0, 0.278],
      59: [0.43, 0.194, 0.278],
      60: [0.54, 0.04, 0.778],
      61: [0.583, 0.082, 0.778],
      62: [0.54, 0.04, 0.778],
      63: [0.705, 0, 0.472],
      64: [0.705, 0.011, 0.778],
      65: [0.716, 0, 0.75],
      66: [0.683, 0, 0.708],
      67: [0.705, 0.021, 0.722],
      68: [0.683, 0, 0.764],
      69: [0.68, 0, 0.681],
      70: [0.68, 0, 0.653],
      71: [0.705, 0.022, 0.785],
      72: [0.683, 0, 0.75],
      73: [0.683, 0, 0.361],
      74: [0.683, 0.022, 0.514],
      75: [0.683, 0, 0.778],
      76: [0.683, 0, 0.625],
      77: [0.683, 0, 0.917],
      78: [0.683, 0, 0.75],
      79: [0.705, 0.022, 0.778],
      80: [0.683, 0, 0.681],
      81: [0.705, 0.193, 0.778],
      82: [0.683, 0.022, 0.736],
      83: [0.705, 0.022, 0.556],
      84: [0.677, 0, 0.722],
      85: [0.683, 0.022, 0.75],
      86: [0.683, 0.022, 0.75],
      87: [0.683, 0.022, 1.028],
      88: [0.683, 0, 0.75],
      89: [0.683, 0, 0.75],
      90: [0.683, 0, 0.611],
      91: [0.75, 0.25, 0.278],
      92: [0.75, 0.25, 0.5],
      93: [0.75, 0.25, 0.278],
      94: [0.694, -0.531, 0.5],
      95: [-0.025, 0.062, 0.5],
      96: [0.699, -0.505, 0.5],
      97: [0.448, 0.011, 0.5],
      98: [0.694, 0.011, 0.556],
      99: [0.448, 0.011, 0.444],
      100: [0.694, 0.011, 0.556],
      101: [0.448, 0.011, 0.444],
      102: [0.705, 0, 0.306, { ic: 0.066 }],
      103: [0.453, 0.206, 0.5],
      104: [0.694, 0, 0.556],
      105: [0.669, 0, 0.278],
      106: [0.669, 0.205, 0.306],
      107: [0.694, 0, 0.528],
      108: [0.694, 0, 0.278],
      109: [0.442, 0, 0.833],
      110: [0.442, 0, 0.556],
      111: [0.448, 0.01, 0.5],
      112: [0.442, 0.194, 0.556],
      113: [0.442, 0.194, 0.528],
      114: [0.442, 0, 0.392],
      115: [0.448, 0.011, 0.394],
      116: [0.615, 0.01, 0.389],
      117: [0.442, 0.011, 0.556],
      118: [0.431, 0.011, 0.528],
      119: [0.431, 0.011, 0.722],
      120: [0.431, 0, 0.528],
      121: [0.431, 0.204, 0.528],
      122: [0.431, 0, 0.444],
      123: [0.75, 0.25, 0.5],
      124: [0.75, 0.249, 0.278],
      125: [0.75, 0.25, 0.5],
      126: [0.318, -0.215, 0.5],
      160: [0, 0, 0.25],
      163: [0.714, 0.011, 0.769],
      165: [0.683, 0, 0.75],
      168: [0.669, -0.554, 0.5],
      172: [0.356, -0.089, 0.667],
      174: [0.709, 0.175, 0.947],
      175: [0.59, -0.544, 0.5],
      176: [0.715, -0.542, 0.5],
      177: [0.666, 0, 0.778],
      180: [0.699, -0.505, 0.5],
      183: [0.31, -0.19, 0.278],
      215: [0.491, -9e-3, 0.778],
      240: [0.749, 0.021, 0.556],
      247: [0.537, 0.036, 0.778],
      305: [0.442, 0, 0.278, { sk: 0.0278 }],
      567: [0.442, 0.205, 0.306, { sk: 0.0833 }],
      697: [0.56, -0.043, 0.275],
      710: [0.694, -0.531, 0.5],
      711: [0.644, -0.513, 0.5],
      713: [0.59, -0.544, 0.5],
      714: [0.699, -0.505, 0.5],
      715: [0.699, -0.505, 0.5],
      728: [0.694, -0.515, 0.5],
      729: [0.669, -0.549, 0.5],
      730: [0.715, -0.542, 0.5],
      732: [0.668, -0.565, 0.5],
      768: [0.699, -0.505, 0],
      769: [0.699, -0.505, 0],
      770: [0.694, -0.531, 0],
      771: [0.668, -0.565, 0],
      772: [0.59, -0.544, 0],
      774: [0.694, -0.515, 0],
      775: [0.669, -0.549, 0],
      776: [0.669, -0.554, 0],
      778: [0.715, -0.542, 0],
      779: [0.701, -0.51, 0],
      780: [0.644, -0.513, 0],
      824: [0.716, 0.215, 0],
      913: [0.716, 0, 0.75],
      914: [0.683, 0, 0.708],
      915: [0.68, 0, 0.625],
      916: [0.716, 0, 0.833],
      917: [0.68, 0, 0.681],
      918: [0.683, 0, 0.611],
      919: [0.683, 0, 0.75],
      920: [0.705, 0.022, 0.778],
      921: [0.683, 0, 0.361],
      922: [0.683, 0, 0.778],
      923: [0.716, 0, 0.694],
      924: [0.683, 0, 0.917],
      925: [0.683, 0, 0.75],
      926: [0.677, 0, 0.667],
      927: [0.705, 0.022, 0.778],
      928: [0.68, 0, 0.75],
      929: [0.683, 0, 0.681],
      931: [0.683, 0, 0.722],
      932: [0.677, 0, 0.722],
      933: [0.705, 0, 0.778],
      934: [0.683, 0, 0.722],
      935: [0.683, 0, 0.75],
      936: [0.683, 0, 0.778],
      937: [0.704, 0, 0.722],
      8192: [0, 0, 0.5],
      8193: [0, 0, 1],
      8194: [0, 0, 0.5],
      8195: [0, 0, 1],
      8196: [0, 0, 0.333],
      8197: [0, 0, 0.25],
      8198: [0, 0, 0.167],
      8201: [0, 0, 0.167],
      8202: [0, 0, 0.1],
      8203: [0, 0, 0],
      8204: [0, 0, 0],
      8211: [0.285, -0.248, 0.5],
      8212: [0.285, -0.248, 1],
      8213: [0.285, -0.248, 1],
      8214: [0.75, 0.25, 0.5],
      8215: [-0.025, 0.062, 0.5],
      8216: [0.694, -0.379, 0.278],
      8217: [0.694, -0.379, 0.278],
      8220: [0.694, -0.379, 0.5],
      8221: [0.694, -0.379, 0.5],
      8224: [0.705, 0.216, 0.444],
      8225: [0.705, 0.205, 0.444],
      8226: [0.444, -0.055, 0.5],
      8230: [0.12, 0, 1.172],
      8242: [0.56, -0.043, 0.275],
      8243: [0.56, 0, 0.55],
      8244: [0.56, 0, 0.825],
      8245: [0.56, -0.043, 0.275],
      8246: [0.56, 0, 0.55],
      8247: [0.56, 0, 0.825],
      8254: [0.59, -0.544, 0.5],
      8260: [0.75, 0.25, 0.5],
      8279: [0.56, 0, 1.1],
      8288: [0, 0, 0],
      8289: [0, 0, 0],
      8290: [0, 0, 0],
      8291: [0, 0, 0],
      8292: [0, 0, 0],
      8407: [0.714, -0.516, 0.5],
      8450: [0.702, 0.019, 0.722],
      8459: [0.717, 0.036, 0.969, { ic: 0.272, sk: 0.333 }],
      8460: [0.666, 0.133, 0.72],
      8461: [0.683, 0, 0.778],
      8462: [0.694, 0.011, 0.576, { sk: -0.0278 }],
      8463: [0.695, 0.013, 0.54, { ic: 0.022 }],
      8464: [0.717, 0.017, 0.809, { ic: 0.137, sk: 0.333 }],
      8465: [0.686, 0.026, 0.554],
      8466: [0.717, 0.017, 0.874, { ic: 0.161, sk: 0.306 }],
      8467: [0.705, 0.02, 0.417, { sk: 0.111 }],
      8469: [0.683, 0.02, 0.722],
      8472: [0.453, 0.216, 0.636, { sk: 0.111 }],
      8473: [0.683, 0, 0.611],
      8474: [0.701, 0.181, 0.778],
      8475: [0.717, 0.017, 0.85, { ic: 0.037, sk: 0.194 }],
      8476: [0.686, 0.026, 0.828],
      8477: [0.683, 0, 0.722],
      8484: [0.683, 0, 0.667],
      8486: [0.704, 0, 0.722],
      8487: [0.684, 0.022, 0.722],
      8488: [0.729, 0.139, 0.602],
      8492: [0.708, 0.028, 0.908, { ic: 0.02, sk: 0.194 }],
      8493: [0.685, 0.024, 0.613],
      8496: [0.707, 8e-3, 0.562, { ic: 0.156, sk: 0.139 }],
      8497: [0.735, 0.036, 0.895, { ic: 0.095, sk: 0.222 }],
      8498: [0.695, 0, 0.556],
      8499: [0.721, 0.05, 1.08, { ic: 0.136, sk: 0.444 }],
      8501: [0.694, 0, 0.611],
      8502: [0.763, 0.021, 0.667, { ic: 0.02 }],
      8503: [0.764, 0.043, 0.444],
      8504: [0.764, 0.043, 0.667],
      8513: [0.705, 0.023, 0.639],
      8592: [0.511, 0.011, 1],
      8593: [0.694, 0.193, 0.5],
      8594: [0.511, 0.011, 1],
      8595: [0.694, 0.194, 0.5],
      8596: [0.511, 0.011, 1],
      8597: [0.772, 0.272, 0.5],
      8598: [0.72, 0.195, 1],
      8599: [0.72, 0.195, 1],
      8600: [0.695, 0.22, 1],
      8601: [0.695, 0.22, 1],
      8602: [0.437, -0.06, 1],
      8603: [0.437, -0.06, 1],
      8606: [0.417, -0.083, 1],
      8608: [0.417, -0.083, 1],
      8610: [0.417, -0.083, 1.111],
      8611: [0.417, -0.083, 1.111],
      8614: [0.511, 0.011, 1],
      8617: [0.511, 0.011, 1.126],
      8618: [0.511, 0.011, 1.126],
      8619: [0.575, 0.041, 1],
      8620: [0.575, 0.041, 1],
      8621: [0.417, -0.083, 1.389],
      8622: [0.437, -0.06, 1],
      8624: [0.722, 0, 0.5],
      8625: [0.722, 0, 0.5],
      8630: [0.461, 0, 1],
      8631: [0.46, 0, 1],
      8634: [0.65, 0.083, 0.778],
      8635: [0.65, 0.083, 0.778],
      8636: [0.511, -0.23, 1],
      8637: [0.27, 0.011, 1],
      8638: [0.694, 0.194, 0.417],
      8639: [0.694, 0.194, 0.417],
      8640: [0.511, -0.23, 1],
      8641: [0.27, 0.011, 1],
      8642: [0.694, 0.194, 0.417],
      8643: [0.694, 0.194, 0.417],
      8644: [0.667, 0, 1],
      8646: [0.667, 0, 1],
      8647: [0.583, 0.083, 1],
      8648: [0.694, 0.193, 0.833],
      8649: [0.583, 0.083, 1],
      8650: [0.694, 0.194, 0.833],
      8651: [0.514, 0.014, 1],
      8652: [0.671, 0.011, 1],
      8653: [0.534, 0.035, 1],
      8654: [0.534, 0.037, 1],
      8655: [0.534, 0.035, 1],
      8656: [0.525, 0.024, 1],
      8657: [0.694, 0.194, 0.611],
      8658: [0.525, 0.024, 1],
      8659: [0.694, 0.194, 0.611],
      8660: [0.526, 0.025, 1],
      8661: [0.772, 0.272, 0.611],
      8666: [0.611, 0.111, 1],
      8667: [0.611, 0.111, 1],
      8669: [0.417, -0.083, 1],
      8672: [0.437, -0.064, 1.334],
      8674: [0.437, -0.064, 1.334],
      8704: [0.694, 0.022, 0.556],
      8705: [0.846, 0.021, 0.5],
      8706: [0.715, 0.022, 0.531, { ic: 0.035, sk: 0.0833 }],
      8707: [0.694, 0, 0.556],
      8708: [0.716, 0.215, 0.556],
      8709: [0.772, 0.078, 0.5],
      8710: [0.716, 0, 0.833],
      8711: [0.683, 0.033, 0.833],
      8712: [0.54, 0.04, 0.667],
      8713: [0.716, 0.215, 0.667],
      8715: [0.54, 0.04, 0.667],
      8716: [0.716, 0.215, 0.667],
      8717: [0.44, 0, 0.429, { ic: 0.027 }],
      8719: [0.75, 0.25, 0.944],
      8720: [0.75, 0.25, 0.944],
      8721: [0.75, 0.25, 1.056],
      8722: [0.583, 0.082, 0.778],
      8723: [0.5, 0.166, 0.778],
      8724: [0.766, 0.093, 0.778],
      8725: [0.75, 0.25, 0.5],
      8726: [0.75, 0.25, 0.5],
      8727: [0.465, -0.035, 0.5],
      8728: [0.444, -0.055, 0.5],
      8729: [0.444, -0.055, 0.5],
      8730: [0.8, 0.2, 0.833, { ic: 0.02 }],
      8733: [0.442, 0.011, 0.778],
      8734: [0.442, 0.011, 1],
      8736: [0.694, 0, 0.722],
      8737: [0.714, 0.02, 0.722],
      8738: [0.551, 0.051, 0.722],
      8739: [0.75, 0.249, 0.278],
      8740: [0.75, 0.252, 0.278, { ic: 0.019 }],
      8741: [0.75, 0.25, 0.5],
      8742: [0.75, 0.25, 0.5, { ic: 0.018 }],
      8743: [0.598, 0.022, 0.667],
      8744: [0.598, 0.022, 0.667],
      8745: [0.598, 0.022, 0.667],
      8746: [0.598, 0.022, 0.667],
      8747: [0.716, 0.216, 0.417, { ic: 0.055 }],
      8748: [0.805, 0.306, 0.819, { ic: 0.138 }],
      8749: [0.805, 0.306, 1.166, { ic: 0.138 }],
      8750: [0.805, 0.306, 0.472, { ic: 0.138 }],
      8756: [0.471, 0.082, 0.667],
      8757: [0.471, 0.082, 0.667],
      8764: [0.367, -0.133, 0.778],
      8765: [0.367, -0.133, 0.778],
      8768: [0.583, 0.083, 0.278],
      8769: [0.467, -0.032, 0.778],
      8770: [0.463, -0.034, 0.778],
      8771: [0.464, -0.036, 0.778],
      8772: [0.716, 0.215, 0.778],
      8773: [0.589, -0.022, 0.778],
      8775: [0.652, 0.155, 0.778],
      8776: [0.483, -0.055, 0.778],
      8777: [0.716, 0.215, 0.778],
      8778: [0.579, 0.039, 0.778],
      8781: [0.484, -0.016, 0.778],
      8782: [0.492, -8e-3, 0.778],
      8783: [0.492, -0.133, 0.778],
      8784: [0.67, -0.133, 0.778],
      8785: [0.609, 0.108, 0.778],
      8786: [0.601, 0.101, 0.778],
      8787: [0.601, 0.102, 0.778],
      8790: [0.367, -0.133, 0.778],
      8791: [0.721, -0.133, 0.778],
      8796: [0.859, -0.133, 0.778],
      8800: [0.716, 0.215, 0.778],
      8801: [0.464, -0.036, 0.778],
      8802: [0.716, 0.215, 0.778],
      8804: [0.636, 0.138, 0.778],
      8805: [0.636, 0.138, 0.778],
      8806: [0.753, 0.175, 0.778],
      8807: [0.753, 0.175, 0.778],
      8808: [0.752, 0.286, 0.778],
      8809: [0.752, 0.286, 0.778],
      8810: [0.568, 0.067, 1],
      8811: [0.567, 0.067, 1],
      8812: [0.75, 0.25, 0.5],
      8813: [0.716, 0.215, 0.778],
      8814: [0.708, 0.209, 0.778],
      8815: [0.708, 0.209, 0.778],
      8816: [0.801, 0.303, 0.778],
      8817: [0.801, 0.303, 0.778],
      8818: [0.732, 0.228, 0.778],
      8819: [0.732, 0.228, 0.778],
      8820: [0.732, 0.228, 0.778],
      8821: [0.732, 0.228, 0.778],
      8822: [0.681, 0.253, 0.778],
      8823: [0.681, 0.253, 0.778],
      8824: [0.716, 0.253, 0.778],
      8825: [0.716, 0.253, 0.778],
      8826: [0.539, 0.041, 0.778],
      8827: [0.539, 0.041, 0.778],
      8828: [0.58, 0.153, 0.778],
      8829: [0.58, 0.154, 0.778],
      8830: [0.732, 0.228, 0.778],
      8831: [0.732, 0.228, 0.778],
      8832: [0.705, 0.208, 0.778],
      8833: [0.705, 0.208, 0.778],
      8834: [0.54, 0.04, 0.778],
      8835: [0.54, 0.04, 0.778],
      8836: [0.716, 0.215, 0.778],
      8837: [0.716, 0.215, 0.778],
      8838: [0.636, 0.138, 0.778],
      8839: [0.636, 0.138, 0.778],
      8840: [0.801, 0.303, 0.778],
      8841: [0.801, 0.303, 0.778],
      8842: [0.635, 0.241, 0.778],
      8843: [0.635, 0.241, 0.778],
      8846: [0.598, 0.022, 0.667],
      8847: [0.539, 0.041, 0.778],
      8848: [0.539, 0.041, 0.778],
      8849: [0.636, 0.138, 0.778],
      8850: [0.636, 0.138, 0.778],
      8851: [0.598, 0, 0.667],
      8852: [0.598, 0, 0.667],
      8853: [0.583, 0.083, 0.778],
      8854: [0.583, 0.083, 0.778],
      8855: [0.583, 0.083, 0.778],
      8856: [0.583, 0.083, 0.778],
      8857: [0.583, 0.083, 0.778],
      8858: [0.582, 0.082, 0.778],
      8859: [0.582, 0.082, 0.778],
      8861: [0.582, 0.082, 0.778],
      8862: [0.689, 0, 0.778],
      8863: [0.689, 0, 0.778],
      8864: [0.689, 0, 0.778],
      8865: [0.689, 0, 0.778],
      8866: [0.694, 0, 0.611],
      8867: [0.694, 0, 0.611],
      8868: [0.668, 0, 0.778],
      8869: [0.668, 0, 0.778],
      8872: [0.75, 0.249, 0.867],
      8873: [0.694, 0, 0.722],
      8874: [0.694, 0, 0.889],
      8876: [0.695, 0, 0.611],
      8877: [0.695, 0, 0.611],
      8878: [0.695, 0, 0.722],
      8879: [0.695, 0, 0.722],
      8882: [0.539, 0.041, 0.778],
      8883: [0.539, 0.041, 0.778],
      8884: [0.636, 0.138, 0.778],
      8885: [0.636, 0.138, 0.778],
      8888: [0.408, -0.092, 1.111],
      8890: [0.431, 0.212, 0.556],
      8891: [0.716, 0, 0.611],
      8892: [0.716, 0, 0.611],
      8896: [0.75, 0.249, 0.833],
      8897: [0.75, 0.249, 0.833],
      8898: [0.75, 0.249, 0.833],
      8899: [0.75, 0.249, 0.833],
      8900: [0.488, -0.012, 0.5],
      8901: [0.31, -0.19, 0.278],
      8902: [0.486, -0.016, 0.5],
      8903: [0.545, 0.044, 0.778],
      8904: [0.505, 5e-3, 0.9],
      8905: [0.492, -8e-3, 0.778],
      8906: [0.492, -8e-3, 0.778],
      8907: [0.694, 0.022, 0.778],
      8908: [0.694, 0.022, 0.778],
      8909: [0.464, -0.036, 0.778],
      8910: [0.578, 0.021, 0.76],
      8911: [0.578, 0.022, 0.76],
      8912: [0.54, 0.04, 0.778],
      8913: [0.54, 0.04, 0.778],
      8914: [0.598, 0.022, 0.667],
      8915: [0.598, 0.022, 0.667],
      8916: [0.736, 0.022, 0.667],
      8918: [0.541, 0.041, 0.778],
      8919: [0.541, 0.041, 0.778],
      8920: [0.568, 0.067, 1.333],
      8921: [0.568, 0.067, 1.333],
      8922: [0.886, 0.386, 0.778],
      8923: [0.886, 0.386, 0.778],
      8926: [0.734, 0, 0.778],
      8927: [0.734, 0, 0.778],
      8928: [0.801, 0.303, 0.778],
      8929: [0.801, 0.303, 0.778],
      8930: [0.716, 0.215, 0.778],
      8931: [0.716, 0.215, 0.778],
      8934: [0.73, 0.359, 0.778],
      8935: [0.73, 0.359, 0.778],
      8936: [0.73, 0.359, 0.778],
      8937: [0.73, 0.359, 0.778],
      8938: [0.706, 0.208, 0.778],
      8939: [0.706, 0.208, 0.778],
      8940: [0.802, 0.303, 0.778],
      8941: [0.801, 0.303, 0.778],
      8942: [1.3, 0.03, 0.278],
      8943: [0.31, -0.19, 1.172],
      8945: [1.52, -0.1, 1.282],
      8965: [0.716, 0, 0.611],
      8966: [0.813, 0.097, 0.611],
      8968: [0.75, 0.25, 0.444],
      8969: [0.75, 0.25, 0.444],
      8970: [0.75, 0.25, 0.444],
      8971: [0.75, 0.25, 0.444],
      8988: [0.694, -0.306, 0.5],
      8989: [0.694, -0.306, 0.5],
      8990: [0.366, 0.022, 0.5],
      8991: [0.366, 0.022, 0.5],
      8994: [0.388, -0.122, 1],
      8995: [0.378, -0.134, 1],
      9001: [0.75, 0.25, 0.389],
      9002: [0.75, 0.25, 0.389],
      9136: [0.744, 0.244, 0.412],
      9137: [0.744, 0.244, 0.412],
      9168: [0.602, 0, 0.667],
      9416: [0.709, 0.175, 0.902],
      9484: [0.694, -0.306, 0.5],
      9488: [0.694, -0.306, 0.5],
      9492: [0.366, 0.022, 0.5],
      9496: [0.366, 0.022, 0.5],
      9585: [0.694, 0.195, 0.889],
      9586: [0.694, 0.195, 0.889],
      9632: [0.689, 0, 0.778],
      9633: [0.689, 0, 0.778],
      9642: [0.689, 0, 0.778],
      9650: [0.575, 0.02, 0.722],
      9651: [0.716, 0, 0.889],
      9652: [0.575, 0.02, 0.722],
      9653: [0.716, 0, 0.889],
      9654: [0.539, 0.041, 0.778],
      9656: [0.539, 0.041, 0.778],
      9657: [0.505, 5e-3, 0.5],
      9660: [0.576, 0.019, 0.722],
      9661: [0.5, 0.215, 0.889],
      9662: [0.576, 0.019, 0.722],
      9663: [0.5, 0.215, 0.889],
      9664: [0.539, 0.041, 0.778],
      9666: [0.539, 0.041, 0.778],
      9667: [0.505, 5e-3, 0.5],
      9674: [0.716, 0.132, 0.667],
      9711: [0.715, 0.215, 1],
      9723: [0.689, 0, 0.778],
      9724: [0.689, 0, 0.778],
      9733: [0.694, 0.111, 0.944],
      9824: [0.727, 0.13, 0.778],
      9825: [0.716, 0.033, 0.778],
      9826: [0.727, 0.162, 0.778],
      9827: [0.726, 0.13, 0.778],
      9837: [0.75, 0.022, 0.389],
      9838: [0.734, 0.223, 0.389],
      9839: [0.723, 0.223, 0.389],
      10003: [0.706, 0.034, 0.833],
      10016: [0.716, 0.022, 0.833],
      10072: [0.75, 0.249, 0.278],
      10216: [0.75, 0.25, 0.389],
      10217: [0.75, 0.25, 0.389],
      10222: [0.744, 0.244, 0.412],
      10223: [0.744, 0.244, 0.412],
      10229: [0.511, 0.011, 1.609],
      10230: [0.511, 0.011, 1.638],
      10231: [0.511, 0.011, 1.859],
      10232: [0.525, 0.024, 1.609],
      10233: [0.525, 0.024, 1.638],
      10234: [0.525, 0.024, 1.858],
      10236: [0.511, 0.011, 1.638],
      10731: [0.716, 0.132, 0.667],
      10744: [0.716, 0.215, 0.778],
      10752: [0.75, 0.25, 1.111],
      10753: [0.75, 0.25, 1.111],
      10754: [0.75, 0.25, 1.111],
      10756: [0.75, 0.249, 0.833],
      10758: [0.75, 0.249, 0.833],
      10764: [0.805, 0.306, 1.638, { ic: 0.138 }],
      10799: [0.491, -9e-3, 0.778],
      10815: [0.683, 0, 0.75],
      10846: [0.813, 0.097, 0.611],
      10877: [0.636, 0.138, 0.778],
      10878: [0.636, 0.138, 0.778],
      10885: [0.762, 0.29, 0.778],
      10886: [0.762, 0.29, 0.778],
      10887: [0.635, 0.241, 0.778],
      10888: [0.635, 0.241, 0.778],
      10889: [0.761, 0.387, 0.778],
      10890: [0.761, 0.387, 0.778],
      10891: [1.003, 0.463, 0.778],
      10892: [1.003, 0.463, 0.778],
      10901: [0.636, 0.138, 0.778],
      10902: [0.636, 0.138, 0.778],
      10927: [0.636, 0.138, 0.778],
      10928: [0.636, 0.138, 0.778],
      10933: [0.752, 0.286, 0.778],
      10934: [0.752, 0.286, 0.778],
      10935: [0.761, 0.294, 0.778],
      10936: [0.761, 0.294, 0.778],
      10937: [0.761, 0.337, 0.778],
      10938: [0.761, 0.337, 0.778],
      10949: [0.753, 0.215, 0.778],
      10950: [0.753, 0.215, 0.778],
      10955: [0.783, 0.385, 0.778],
      10956: [0.783, 0.385, 0.778],
      12296: [0.75, 0.25, 0.389],
      12297: [0.75, 0.25, 0.389],
      57350: [0.43, 0.023, 0.222, { ic: 0.018 }],
      57351: [0.431, 0.024, 0.389, { ic: 0.018 }],
      57352: [0.605, 0.085, 0.778],
      57353: [0.434, 6e-3, 0.667, { ic: 0.067 }],
      57356: [0.752, 0.284, 0.778],
      57357: [0.752, 0.284, 0.778],
      57358: [0.919, 0.421, 0.778],
      57359: [0.801, 0.303, 0.778],
      57360: [0.801, 0.303, 0.778],
      57361: [0.919, 0.421, 0.778],
      57366: [0.828, 0.33, 0.778],
      57367: [0.752, 0.332, 0.778],
      57368: [0.828, 0.33, 0.778],
      57369: [0.752, 0.333, 0.778],
      57370: [0.634, 0.255, 0.778],
      57371: [0.634, 0.254, 0.778],
      119808: [0.698, 0, 0.869],
      119809: [0.686, 0, 0.818],
      119810: [0.697, 0.011, 0.831],
      119811: [0.686, 0, 0.882],
      119812: [0.68, 0, 0.756],
      119813: [0.68, 0, 0.724],
      119814: [0.697, 0.01, 0.904],
      119815: [0.686, 0, 0.9],
      119816: [0.686, 0, 0.436],
      119817: [0.686, 0.011, 0.594],
      119818: [0.686, 0, 0.901],
      119819: [0.686, 0, 0.692],
      119820: [0.686, 0, 1.092],
      119821: [0.686, 0, 0.9],
      119822: [0.696, 0.01, 0.864],
      119823: [0.686, 0, 0.786],
      119824: [0.696, 0.193, 0.864],
      119825: [0.686, 0.011, 0.862],
      119826: [0.697, 0.011, 0.639],
      119827: [0.675, 0, 0.8],
      119828: [0.686, 0.011, 0.885],
      119829: [0.686, 7e-3, 0.869],
      119830: [0.686, 7e-3, 1.189],
      119831: [0.686, 0, 0.869],
      119832: [0.686, 0, 0.869],
      119833: [0.686, 0, 0.703],
      119834: [0.453, 6e-3, 0.559],
      119835: [0.694, 6e-3, 0.639],
      119836: [0.453, 6e-3, 0.511],
      119837: [0.694, 6e-3, 0.639],
      119838: [0.452, 6e-3, 0.527],
      119839: [0.7, 0, 0.351, { ic: 0.101 }],
      119840: [0.455, 0.201, 0.575],
      119841: [0.694, 0, 0.639],
      119842: [0.695, 0, 0.319],
      119843: [0.695, 0.2, 0.351],
      119844: [0.694, 0, 0.607],
      119845: [0.694, 0, 0.319],
      119846: [0.45, 0, 0.958],
      119847: [0.45, 0, 0.639],
      119848: [0.452, 5e-3, 0.575],
      119849: [0.45, 0.194, 0.639],
      119850: [0.45, 0.194, 0.607],
      119851: [0.45, 0, 0.474],
      119852: [0.453, 6e-3, 0.454],
      119853: [0.635, 5e-3, 0.447],
      119854: [0.45, 6e-3, 0.639],
      119855: [0.444, 0, 0.607],
      119856: [0.444, 0, 0.831],
      119857: [0.444, 0, 0.607],
      119858: [0.444, 0.2, 0.607],
      119859: [0.444, 0, 0.511],
      119860: [0.716, 0, 0.75, { sk: 0.139 }],
      119861: [0.683, 0, 0.759, { sk: 0.0833 }],
      119862: [0.705, 0.022, 0.715, { ic: 0.045, sk: 0.0833 }],
      119863: [0.683, 0, 0.828, { sk: 0.0556 }],
      119864: [0.68, 0, 0.738, { ic: 0.026, sk: 0.0833 }],
      119865: [0.68, 0, 0.643, { ic: 0.106, sk: 0.0833 }],
      119866: [0.705, 0.022, 0.786, { sk: 0.0833 }],
      119867: [0.683, 0, 0.831, { ic: 0.057, sk: 0.0556 }],
      119868: [0.683, 0, 0.44, { ic: 0.064, sk: 0.111 }],
      119869: [0.683, 0.022, 0.555, { ic: 0.078, sk: 0.167 }],
      119870: [0.683, 0, 0.849, { ic: 0.04, sk: 0.0556 }],
      119871: [0.683, 0, 0.681, { sk: 0.0278 }],
      119872: [0.683, 0, 0.97, { ic: 0.081, sk: 0.0833 }],
      119873: [0.683, 0, 0.803, { ic: 0.085, sk: 0.0833 }],
      119874: [0.704, 0.022, 0.763, { sk: 0.0833 }],
      119875: [0.683, 0, 0.642, { ic: 0.109, sk: 0.0833 }],
      119876: [0.704, 0.194, 0.791, { sk: 0.0833 }],
      119877: [0.683, 0.021, 0.759, { sk: 0.0833 }],
      119878: [0.705, 0.022, 0.613, { ic: 0.032, sk: 0.0833 }],
      119879: [0.677, 0, 0.584, { ic: 0.12, sk: 0.0833 }],
      119880: [0.683, 0.022, 0.683, { ic: 0.084, sk: 0.0278 }],
      119881: [0.683, 0.022, 0.583, { ic: 0.186 }],
      119882: [0.683, 0.022, 0.944, { ic: 0.104 }],
      119883: [0.683, 0, 0.828, { ic: 0.024, sk: 0.0833 }],
      119884: [0.683, 0, 0.581, { ic: 0.182 }],
      119885: [0.683, 0, 0.683, { ic: 0.04, sk: 0.0833 }],
      119886: [0.441, 0.01, 0.529],
      119887: [0.694, 0.011, 0.429],
      119888: [0.442, 0.011, 0.433, { sk: 0.0556 }],
      119889: [0.694, 0.01, 0.52, { sk: 0.167 }],
      119890: [0.442, 0.011, 0.466, { sk: 0.0556 }],
      119891: [0.705, 0.205, 0.49, { ic: 0.06, sk: 0.167 }],
      119892: [0.442, 0.205, 0.477, { sk: 0.0278 }],
      119894: [0.661, 0.011, 0.345],
      119895: [0.661, 0.204, 0.412],
      119896: [0.694, 0.011, 0.521],
      119897: [0.694, 0.011, 0.298, { sk: 0.0833 }],
      119898: [0.442, 0.011, 0.878],
      119899: [0.442, 0.011, 0.6],
      119900: [0.441, 0.011, 0.485, { sk: 0.0556 }],
      119901: [0.442, 0.194, 0.503, { sk: 0.0833 }],
      119902: [0.442, 0.194, 0.446, { ic: 0.014, sk: 0.0833 }],
      119903: [0.442, 0.011, 0.451, { sk: 0.0556 }],
      119904: [0.442, 0.01, 0.469, { sk: 0.0556 }],
      119905: [0.626, 0.011, 0.361, { sk: 0.0833 }],
      119906: [0.442, 0.011, 0.572, { sk: 0.0278 }],
      119907: [0.443, 0.011, 0.485, { sk: 0.0278 }],
      119908: [0.443, 0.011, 0.716, { sk: 0.0833 }],
      119909: [0.442, 0.011, 0.572, { sk: 0.0278 }],
      119910: [0.442, 0.205, 0.49, { sk: 0.0556 }],
      119911: [0.442, 0.011, 0.465, { sk: 0.0556 }],
      119912: [0.711, 0, 0.869, { sk: 0.16 }],
      119913: [0.686, 0, 0.866, { sk: 0.0958 }],
      119914: [0.703, 0.017, 0.817, { ic: 0.038, sk: 0.0958 }],
      119915: [0.686, 0, 0.938, { sk: 0.0639 }],
      119916: [0.68, 0, 0.81, { ic: 0.015, sk: 0.0958 }],
      119917: [0.68, 0, 0.689, { ic: 0.12, sk: 0.0958 }],
      119918: [0.703, 0.016, 0.887, { sk: 0.0958 }],
      119919: [0.686, 0, 0.982, { ic: 0.045, sk: 0.0639 }],
      119920: [0.686, 0, 0.511, { ic: 0.062, sk: 0.128 }],
      119921: [0.686, 0.017, 0.631, { ic: 0.063, sk: 0.192 }],
      119922: [0.686, 0, 0.971, { ic: 0.032, sk: 0.0639 }],
      119923: [0.686, 0, 0.756, { sk: 0.0319 }],
      119924: [0.686, 0, 1.142, { ic: 0.077, sk: 0.0958 }],
      119925: [0.686, 0, 0.95, { ic: 0.077, sk: 0.0958 }],
      119926: [0.703, 0.017, 0.837, { sk: 0.0958 }],
      119927: [0.686, 0, 0.723, { ic: 0.124, sk: 0.0958 }],
      119928: [0.703, 0.194, 0.869, { sk: 0.0958 }],
      119929: [0.686, 0.017, 0.872, { sk: 0.0958 }],
      119930: [0.703, 0.017, 0.693, { ic: 0.021, sk: 0.0958 }],
      119931: [0.675, 0, 0.637, { ic: 0.135, sk: 0.0958 }],
      119932: [0.686, 0.016, 0.8, { ic: 0.077, sk: 0.0319 }],
      119933: [0.686, 0.016, 0.678, { ic: 0.208 }],
      119934: [0.686, 0.017, 1.093, { ic: 0.114 }],
      119935: [0.686, 0, 0.947, { sk: 0.0958 }],
      119936: [0.686, 0, 0.675, { ic: 0.201 }],
      119937: [0.686, 0, 0.773, { ic: 0.032, sk: 0.0958 }],
      119938: [0.452, 8e-3, 0.633],
      119939: [0.694, 8e-3, 0.521],
      119940: [0.451, 8e-3, 0.513, { sk: 0.0639 }],
      119941: [0.694, 8e-3, 0.61, { sk: 0.192 }],
      119942: [0.452, 8e-3, 0.554, { sk: 0.0639 }],
      119943: [0.701, 0.201, 0.568, { ic: 0.056, sk: 0.192 }],
      119944: [0.452, 0.202, 0.545, { sk: 0.0319 }],
      119945: [0.694, 8e-3, 0.668, { sk: -0.0319 }],
      119946: [0.694, 8e-3, 0.405],
      119947: [0.694, 0.202, 0.471],
      119948: [0.694, 8e-3, 0.604],
      119949: [0.694, 8e-3, 0.348, { sk: 0.0958 }],
      119950: [0.452, 8e-3, 1.032],
      119951: [0.452, 8e-3, 0.713],
      119952: [0.452, 8e-3, 0.585, { sk: 0.0639 }],
      119953: [0.452, 0.194, 0.601, { sk: 0.0958 }],
      119954: [0.452, 0.194, 0.542, { sk: 0.0958 }],
      119955: [0.452, 8e-3, 0.529, { sk: 0.0639 }],
      119956: [0.451, 8e-3, 0.531, { sk: 0.0639 }],
      119957: [0.643, 7e-3, 0.415, { sk: 0.0958 }],
      119958: [0.452, 8e-3, 0.681, { sk: 0.0319 }],
      119959: [0.453, 8e-3, 0.567, { sk: 0.0319 }],
      119960: [0.453, 8e-3, 0.831, { sk: 0.0958 }],
      119961: [0.452, 8e-3, 0.659, { sk: 0.0319 }],
      119962: [0.452, 0.202, 0.59, { sk: 0.0639 }],
      119963: [0.452, 8e-3, 0.555, { sk: 0.0639 }],
      119964: [0.717, 8e-3, 0.803, { ic: 0.213, sk: 0.389 }],
      119966: [0.728, 0.026, 0.666, { ic: 0.153, sk: 0.278 }],
      119967: [0.708, 0.031, 0.774, { ic: 0.081, sk: 0.111 }],
      119970: [0.717, 0.037, 0.61, { ic: 0.128, sk: 0.25 }],
      119973: [0.717, 0.314, 1.052, { ic: 0.081, sk: 0.417 }],
      119974: [0.717, 0.037, 0.914, { ic: 0.29, sk: 0.361 }],
      119977: [0.726, 0.036, 0.902, { ic: 0.306, sk: 0.389 }],
      119978: [0.707, 8e-3, 0.738, { ic: 0.067, sk: 0.167 }],
      119979: [0.716, 0.037, 1.013, { ic: 0.018, sk: 0.222 }],
      119980: [0.717, 0.017, 0.883, { sk: 0.278 }],
      119982: [0.708, 0.036, 0.868, { ic: 0.148, sk: 0.333 }],
      119983: [0.735, 0.037, 0.747, { ic: 0.249, sk: 0.222 }],
      119984: [0.717, 0.017, 0.8, { ic: 0.16, sk: 0.25 }],
      119985: [0.717, 0.017, 0.622, { ic: 0.228, sk: 0.222 }],
      119986: [0.717, 0.017, 0.805, { ic: 0.221, sk: 0.25 }],
      119987: [0.717, 0.017, 0.944, { ic: 0.187, sk: 0.278 }],
      119988: [0.716, 0.017, 0.71, { ic: 0.249, sk: 0.194 }],
      119989: [0.717, 0.016, 0.821, { ic: 0.211, sk: 0.306 }],
      120068: [0.696, 0.026, 0.718],
      120069: [0.691, 0.027, 0.884],
      120071: [0.685, 0.027, 0.832],
      120072: [0.685, 0.024, 0.663],
      120073: [0.686, 0.153, 0.611],
      120074: [0.69, 0.026, 0.785],
      120077: [0.686, 0.139, 0.552],
      120078: [0.68, 0.027, 0.668, { ic: 0.014 }],
      120079: [0.686, 0.026, 0.666],
      120080: [0.692, 0.027, 1.05],
      120081: [0.686, 0.025, 0.832],
      120082: [0.729, 0.027, 0.827],
      120083: [0.692, 0.218, 0.828],
      120084: [0.729, 0.069, 0.827],
      120086: [0.692, 0.027, 0.829],
      120087: [0.701, 0.027, 0.669],
      120088: [0.697, 0.027, 0.646, { ic: 0.019 }],
      120089: [0.686, 0.026, 0.831],
      120090: [0.686, 0.027, 1.046],
      120091: [0.688, 0.027, 0.719],
      120092: [0.686, 0.218, 0.833],
      120094: [0.47, 0.035, 0.5],
      120095: [0.685, 0.031, 0.513],
      120096: [0.466, 0.029, 0.389],
      120097: [0.609, 0.033, 0.499],
      120098: [0.467, 0.03, 0.401],
      120099: [0.681, 0.221, 0.326],
      120100: [0.47, 0.209, 0.504],
      120101: [0.688, 0.205, 0.521],
      120102: [0.673, 0.02, 0.279],
      120103: [0.672, 0.208, 0.281],
      120104: [0.689, 0.025, 0.389],
      120105: [0.685, 0.02, 0.28],
      120106: [0.475, 0.026, 0.767],
      120107: [0.475, 0.022, 0.527],
      120108: [0.48, 0.028, 0.489],
      120109: [0.541, 0.212, 0.5],
      120110: [0.479, 0.219, 0.489],
      120111: [0.474, 0.021, 0.389],
      120112: [0.478, 0.029, 0.443],
      120113: [0.64, 0.02, 0.333, { ic: 0.015 }],
      120114: [0.474, 0.023, 0.517],
      120115: [0.53, 0.028, 0.512],
      120116: [0.532, 0.028, 0.774],
      120117: [0.472, 0.188, 0.389],
      120118: [0.528, 0.218, 0.499],
      120119: [0.471, 0.214, 0.391],
      120120: [0.701, 0, 0.722],
      120121: [0.683, 0, 0.667],
      120123: [0.683, 0, 0.722],
      120124: [0.683, 0, 0.667],
      120125: [0.683, 0, 0.611],
      120126: [0.702, 0.019, 0.778],
      120128: [0.683, 0, 0.389],
      120129: [0.683, 0.077, 0.5],
      120130: [0.683, 0, 0.778],
      120131: [0.683, 0, 0.667],
      120132: [0.683, 0, 0.944],
      120134: [0.701, 0.019, 0.778],
      120138: [0.702, 0.012, 0.556],
      120139: [0.683, 0, 0.667],
      120140: [0.683, 0.019, 0.722],
      120141: [0.683, 0.02, 0.722],
      120142: [0.683, 0.019, 1],
      120143: [0.683, 0, 0.722],
      120144: [0.683, 0, 0.722],
      120172: [0.686, 0.031, 0.847],
      120173: [0.684, 0.031, 1.044],
      120174: [0.676, 0.032, 0.723],
      120175: [0.683, 0.029, 0.982],
      120176: [0.686, 0.029, 0.783],
      120177: [0.684, 0.146, 0.722],
      120178: [0.687, 0.029, 0.927],
      120179: [0.683, 0.126, 0.851],
      120180: [0.681, 0.025, 0.655],
      120181: [0.68, 0.141, 0.652],
      120182: [0.681, 0.026, 0.789, { ic: 0.017 }],
      120183: [0.683, 0.028, 0.786],
      120184: [0.683, 0.032, 1.239],
      120185: [0.679, 0.03, 0.983],
      120186: [0.726, 0.03, 0.976],
      120187: [0.688, 0.223, 0.977],
      120188: [0.726, 0.083, 0.976],
      120189: [0.688, 0.028, 0.978],
      120190: [0.685, 0.031, 0.978],
      120191: [0.686, 0.03, 0.79, { ic: 0.012 }],
      120192: [0.688, 0.039, 0.851, { ic: 0.02 }],
      120193: [0.685, 0.029, 0.982],
      120194: [0.683, 0.03, 1.235],
      120195: [0.681, 0.035, 0.849],
      120196: [0.688, 0.214, 0.984],
      120197: [0.677, 0.148, 0.711],
      120198: [0.472, 0.032, 0.603],
      120199: [0.69, 0.032, 0.59],
      120200: [0.473, 0.026, 0.464],
      120201: [0.632, 0.028, 0.589],
      120202: [0.471, 0.027, 0.472],
      120203: [0.687, 0.222, 0.388],
      120204: [0.472, 0.208, 0.595],
      120205: [0.687, 0.207, 0.615],
      120206: [0.686, 0.025, 0.331],
      120207: [0.682, 0.203, 0.332],
      120208: [0.682, 0.025, 0.464],
      120209: [0.681, 0.024, 0.337],
      120210: [0.476, 0.031, 0.921],
      120211: [0.473, 0.028, 0.654],
      120212: [0.482, 0.034, 0.609],
      120213: [0.557, 0.207, 0.604],
      120214: [0.485, 0.211, 0.596],
      120215: [0.472, 0.026, 0.46],
      120216: [0.479, 0.034, 0.523],
      120217: [0.648, 0.027, 0.393, { ic: 0.014 }],
      120218: [0.472, 0.032, 0.589, { ic: 0.014 }],
      120219: [0.546, 0.027, 0.604],
      120220: [0.549, 0.032, 0.918],
      120221: [0.471, 0.188, 0.459],
      120222: [0.557, 0.221, 0.589],
      120223: [0.471, 0.214, 0.461],
      120224: [0.694, 0, 0.667],
      120225: [0.694, 0, 0.667],
      120226: [0.705, 0.011, 0.639],
      120227: [0.694, 0, 0.722],
      120228: [0.691, 0, 0.597],
      120229: [0.691, 0, 0.569],
      120230: [0.704, 0.011, 0.667],
      120231: [0.694, 0, 0.708],
      120232: [0.694, 0, 0.278],
      120233: [0.694, 0.022, 0.472],
      120234: [0.694, 0, 0.694],
      120235: [0.694, 0, 0.542],
      120236: [0.694, 0, 0.875],
      120237: [0.694, 0, 0.708],
      120238: [0.715, 0.022, 0.736],
      120239: [0.694, 0, 0.639],
      120240: [0.715, 0.125, 0.736],
      120241: [0.694, 0, 0.646],
      120242: [0.716, 0.022, 0.556],
      120243: [0.688, 0, 0.681],
      120244: [0.694, 0.022, 0.688],
      120245: [0.694, 0, 0.667],
      120246: [0.694, 0, 0.944],
      120247: [0.694, 0, 0.667],
      120248: [0.694, 0, 0.667],
      120249: [0.694, 0, 0.611],
      120250: [0.46, 0.01, 0.481],
      120251: [0.694, 0.011, 0.517],
      120252: [0.46, 0.01, 0.444],
      120253: [0.694, 0.01, 0.517],
      120254: [0.461, 0.01, 0.444],
      120255: [0.705, 0, 0.306, { ic: 0.041 }],
      120256: [0.455, 0.206, 0.5],
      120257: [0.694, 0, 0.517],
      120258: [0.68, 0, 0.239],
      120259: [0.68, 0.205, 0.267],
      120260: [0.694, 0, 0.489],
      120261: [0.694, 0, 0.239],
      120262: [0.455, 0, 0.794],
      120263: [0.455, 0, 0.517],
      120264: [0.46, 0.01, 0.5],
      120265: [0.455, 0.194, 0.517],
      120266: [0.455, 0.194, 0.517],
      120267: [0.455, 0, 0.342],
      120268: [0.46, 0.01, 0.383],
      120269: [0.571, 0.01, 0.361],
      120270: [0.444, 0.01, 0.517],
      120271: [0.444, 0, 0.461],
      120272: [0.444, 0, 0.683],
      120273: [0.444, 0, 0.461],
      120274: [0.444, 0.204, 0.461],
      120275: [0.444, 0, 0.435],
      120276: [0.694, 0, 0.733],
      120277: [0.694, 0, 0.733],
      120278: [0.704, 0.011, 0.703],
      120279: [0.694, 0, 0.794],
      120280: [0.691, 0, 0.642],
      120281: [0.691, 0, 0.611],
      120282: [0.705, 0.011, 0.733],
      120283: [0.694, 0, 0.794],
      120284: [0.694, 0, 0.331],
      120285: [0.694, 0.022, 0.519],
      120286: [0.694, 0, 0.764],
      120287: [0.694, 0, 0.581],
      120288: [0.694, 0, 0.978],
      120289: [0.694, 0, 0.794],
      120290: [0.716, 0.022, 0.794],
      120291: [0.694, 0, 0.703],
      120292: [0.716, 0.106, 0.794],
      120293: [0.694, 0, 0.703],
      120294: [0.716, 0.022, 0.611],
      120295: [0.688, 0, 0.733],
      120296: [0.694, 0.022, 0.764],
      120297: [0.694, 0, 0.733],
      120298: [0.694, 0, 1.039],
      120299: [0.694, 0, 0.733],
      120300: [0.694, 0, 0.733],
      120301: [0.694, 0, 0.672],
      120302: [0.475, 0.011, 0.525],
      120303: [0.694, 0.01, 0.561],
      120304: [0.475, 0.011, 0.489],
      120305: [0.694, 0.011, 0.561],
      120306: [0.474, 0.01, 0.511],
      120307: [0.705, 0, 0.336, { ic: 0.045 }],
      120308: [0.469, 0.206, 0.55],
      120309: [0.694, 0, 0.561],
      120310: [0.695, 0, 0.256],
      120311: [0.695, 0.205, 0.286],
      120312: [0.694, 0, 0.531],
      120313: [0.694, 0, 0.256],
      120314: [0.469, 0, 0.867],
      120315: [0.468, 0, 0.561],
      120316: [0.474, 0.011, 0.55],
      120317: [0.469, 0.194, 0.561],
      120318: [0.469, 0.194, 0.561],
      120319: [0.469, 0, 0.372],
      120320: [0.474, 0.01, 0.422],
      120321: [0.589, 0.01, 0.404],
      120322: [0.458, 0.011, 0.561],
      120323: [0.458, 0, 0.5],
      120324: [0.458, 0, 0.744],
      120325: [0.458, 0, 0.5],
      120326: [0.458, 0.205, 0.5],
      120327: [0.458, 0, 0.476],
      120328: [0.694, 0, 0.667],
      120329: [0.694, 0, 0.667, { ic: 0.029 }],
      120330: [0.705, 0.01, 0.639, { ic: 0.08 }],
      120331: [0.694, 0, 0.722, { ic: 0.025 }],
      120332: [0.691, 0, 0.597, { ic: 0.091 }],
      120333: [0.691, 0, 0.569, { ic: 0.104 }],
      120334: [0.705, 0.011, 0.667, { ic: 0.063 }],
      120335: [0.694, 0, 0.708, { ic: 0.06 }],
      120336: [0.694, 0, 0.278, { ic: 0.06 }],
      120337: [0.694, 0.022, 0.472, { ic: 0.063 }],
      120338: [0.694, 0, 0.694, { ic: 0.091 }],
      120339: [0.694, 0, 0.542],
      120340: [0.694, 0, 0.875, { ic: 0.054 }],
      120341: [0.694, 0, 0.708, { ic: 0.058 }],
      120342: [0.716, 0.022, 0.736, { ic: 0.027 }],
      120343: [0.694, 0, 0.639, { ic: 0.051 }],
      120344: [0.716, 0.125, 0.736, { ic: 0.027 }],
      120345: [0.694, 0, 0.646, { ic: 0.052 }],
      120346: [0.716, 0.022, 0.556, { ic: 0.053 }],
      120347: [0.688, 0, 0.681, { ic: 0.109 }],
      120348: [0.694, 0.022, 0.688, { ic: 0.059 }],
      120349: [0.694, 0, 0.667, { ic: 0.132 }],
      120350: [0.694, 0, 0.944, { ic: 0.132 }],
      120351: [0.694, 0, 0.667, { ic: 0.091 }],
      120352: [0.694, 0, 0.667, { ic: 0.143 }],
      120353: [0.694, 0, 0.611, { ic: 0.091 }],
      120354: [0.461, 0.01, 0.481],
      120355: [0.694, 0.011, 0.517, { ic: 0.022 }],
      120356: [0.46, 0.011, 0.444, { ic: 0.055 }],
      120357: [0.694, 0.01, 0.517, { ic: 0.071 }],
      120358: [0.46, 0.011, 0.444, { ic: 0.028 }],
      120359: [0.705, 0, 0.306, { ic: 0.188 }],
      120360: [0.455, 0.206, 0.5, { ic: 0.068 }],
      120361: [0.694, 0, 0.517],
      120362: [0.68, 0, 0.239, { ic: 0.076 }],
      120363: [0.68, 0.204, 0.267, { ic: 0.069 }],
      120364: [0.694, 0, 0.489, { ic: 0.054 }],
      120365: [0.694, 0, 0.239, { ic: 0.072 }],
      120366: [0.455, 0, 0.794],
      120367: [0.454, 0, 0.517],
      120368: [0.461, 0.011, 0.5, { ic: 0.023 }],
      120369: [0.455, 0.194, 0.517, { ic: 0.021 }],
      120370: [0.455, 0.194, 0.517, { ic: 0.021 }],
      120371: [0.455, 0, 0.342, { ic: 0.082 }],
      120372: [0.461, 0.011, 0.383, { ic: 0.053 }],
      120373: [0.571, 0.011, 0.361, { ic: 0.049 }],
      120374: [0.444, 0.01, 0.517, { ic: 0.02 }],
      120375: [0.444, 0, 0.461, { ic: 0.079 }],
      120376: [0.444, 0, 0.683, { ic: 0.079 }],
      120377: [0.444, 0, 0.461, { ic: 0.076 }],
      120378: [0.444, 0.205, 0.461, { ic: 0.079 }],
      120379: [0.444, 0, 0.435, { ic: 0.059 }],
      120432: [0.623, 0, 0.525],
      120433: [0.611, 0, 0.525],
      120434: [0.622, 0.011, 0.525],
      120435: [0.611, 0, 0.525],
      120436: [0.611, 0, 0.525],
      120437: [0.611, 0, 0.525],
      120438: [0.622, 0.011, 0.525],
      120439: [0.611, 0, 0.525],
      120440: [0.611, 0, 0.525],
      120441: [0.611, 0.011, 0.525],
      120442: [0.611, 0, 0.525],
      120443: [0.611, 0, 0.525],
      120444: [0.611, 0, 0.525],
      120445: [0.611, 0, 0.525],
      120446: [0.621, 0.01, 0.525],
      120447: [0.611, 0, 0.525],
      120448: [0.621, 0.138, 0.525],
      120449: [0.611, 0.011, 0.525],
      120450: [0.622, 0.011, 0.525],
      120451: [0.611, 0, 0.525],
      120452: [0.611, 0.011, 0.525],
      120453: [0.611, 7e-3, 0.525],
      120454: [0.611, 7e-3, 0.525],
      120455: [0.611, 0, 0.525],
      120456: [0.611, 0, 0.525],
      120457: [0.611, 0, 0.525],
      120458: [0.439, 6e-3, 0.525],
      120459: [0.611, 6e-3, 0.525],
      120460: [0.44, 6e-3, 0.525],
      120461: [0.611, 6e-3, 0.525],
      120462: [0.44, 6e-3, 0.525],
      120463: [0.617, 0, 0.525],
      120464: [0.442, 0.229, 0.525],
      120465: [0.611, 0, 0.525],
      120466: [0.612, 0, 0.525],
      120467: [0.612, 0.228, 0.525],
      120468: [0.611, 0, 0.525],
      120469: [0.611, 0, 0.525],
      120470: [0.436, 0, 0.525, { ic: 0.011 }],
      120471: [0.436, 0, 0.525],
      120472: [0.44, 6e-3, 0.525],
      120473: [0.437, 0.221, 0.525],
      120474: [0.437, 0.221, 0.525, { ic: 0.02 }],
      120475: [0.437, 0, 0.525],
      120476: [0.44, 6e-3, 0.525],
      120477: [0.554, 6e-3, 0.525],
      120478: [0.431, 5e-3, 0.525],
      120479: [0.431, 0, 0.525],
      120480: [0.431, 0, 0.525],
      120481: [0.431, 0, 0.525],
      120482: [0.431, 0.228, 0.525],
      120483: [0.431, 0, 0.525],
      120488: [0.698, 0, 0.869],
      120489: [0.686, 0, 0.818],
      120490: [0.68, 0, 0.692],
      120491: [0.698, 0, 0.958],
      120492: [0.68, 0, 0.756],
      120493: [0.686, 0, 0.703],
      120494: [0.686, 0, 0.9],
      120495: [0.696, 0.01, 0.894],
      120496: [0.686, 0, 0.436],
      120497: [0.686, 0, 0.901],
      120498: [0.698, 0, 0.806],
      120499: [0.686, 0, 1.092],
      120500: [0.686, 0, 0.9],
      120501: [0.675, 0, 0.767],
      120502: [0.696, 0.01, 0.864],
      120503: [0.68, 0, 0.9],
      120504: [0.686, 0, 0.786],
      120506: [0.686, 0, 0.831],
      120507: [0.675, 0, 0.8],
      120508: [0.697, 0, 0.894],
      120509: [0.686, 0, 0.831],
      120510: [0.686, 0, 0.869],
      120511: [0.686, 0, 0.894],
      120512: [0.696, 0, 0.831],
      120513: [0.686, 0.024, 0.958],
      120546: [0.716, 0, 0.75, { sk: 0.139 }],
      120547: [0.683, 0, 0.759, { sk: 0.0833 }],
      120548: [0.68, 0, 0.615, { ic: 0.106, sk: 0.0833 }],
      120549: [0.716, 0, 0.833, { sk: 0.167 }],
      120550: [0.68, 0, 0.738, { ic: 0.026, sk: 0.0833 }],
      120551: [0.683, 0, 0.683, { ic: 0.04, sk: 0.0833 }],
      120552: [0.683, 0, 0.831, { ic: 0.057, sk: 0.0556 }],
      120553: [0.704, 0.022, 0.763, { sk: 0.0833 }],
      120554: [0.683, 0, 0.44, { ic: 0.064, sk: 0.111 }],
      120555: [0.683, 0, 0.849, { ic: 0.04, sk: 0.0556 }],
      120556: [0.716, 0, 0.694, { sk: 0.167 }],
      120557: [0.683, 0, 0.97, { ic: 0.081, sk: 0.0833 }],
      120558: [0.683, 0, 0.803, { ic: 0.085, sk: 0.0833 }],
      120559: [0.677, 0, 0.742, { ic: 0.035, sk: 0.0833 }],
      120560: [0.704, 0.022, 0.763, { sk: 0.0833 }],
      120561: [0.68, 0, 0.831, { ic: 0.056, sk: 0.0556 }],
      120562: [0.683, 0, 0.642, { ic: 0.109, sk: 0.0833 }],
      120564: [0.683, 0, 0.78, { ic: 0.026, sk: 0.0833 }],
      120565: [0.677, 0, 0.584, { ic: 0.12, sk: 0.0833 }],
      120566: [0.705, 0, 0.583, { ic: 0.117, sk: 0.0556 }],
      120567: [0.683, 0, 0.667, { sk: 0.0833 }],
      120568: [0.683, 0, 0.828, { ic: 0.024, sk: 0.0833 }],
      120569: [0.683, 0, 0.612, { ic: 0.08, sk: 0.0556 }],
      120570: [0.704, 0, 0.772, { ic: 0.014, sk: 0.0833 }],
      120572: [0.442, 0.011, 0.64, { sk: 0.0278 }],
      120573: [0.705, 0.194, 0.566, { sk: 0.0833 }],
      120574: [0.441, 0.216, 0.518, { ic: 0.025 }],
      120575: [0.717, 0.01, 0.444, { sk: 0.0556 }],
      120576: [0.452, 0.022, 0.466, { sk: 0.0833 }],
      120577: [0.704, 0.204, 0.438, { ic: 0.033, sk: 0.0833 }],
      120578: [0.442, 0.216, 0.497, { sk: 0.0556 }],
      120579: [0.705, 0.01, 0.469, { sk: 0.0833 }],
      120580: [0.442, 0.01, 0.354, { sk: 0.0556 }],
      120581: [0.442, 0.011, 0.576],
      120582: [0.694, 0.012, 0.583],
      120583: [0.442, 0.216, 0.603, { sk: 0.0278 }],
      120584: [0.442, 0, 0.494, { ic: 0.036, sk: 0.0278 }],
      120585: [0.704, 0.205, 0.438, { sk: 0.111 }],
      120586: [0.441, 0.011, 0.485, { sk: 0.0556 }],
      120587: [0.431, 0.011, 0.57],
      120588: [0.442, 0.216, 0.517, { sk: 0.0833 }],
      120589: [0.442, 0.107, 0.363, { ic: 0.042, sk: 0.0833 }],
      120590: [0.431, 0.011, 0.571],
      120591: [0.431, 0.013, 0.437, { ic: 0.08, sk: 0.0278 }],
      120592: [0.443, 0.01, 0.54, { sk: 0.0278 }],
      120593: [0.442, 0.218, 0.654, { sk: 0.0833 }],
      120594: [0.442, 0.204, 0.626, { sk: 0.0556 }],
      120595: [0.694, 0.205, 0.651, { sk: 0.111 }],
      120596: [0.443, 0.011, 0.622],
      120597: [0.715, 0.022, 0.531, { ic: 0.035, sk: 0.0833 }],
      120598: [0.431, 0.011, 0.406, { sk: 0.0556 }],
      120599: [0.705, 0.011, 0.591, { sk: 0.0833 }],
      120600: [0.434, 6e-3, 0.667, { ic: 0.067 }],
      120601: [0.694, 0.205, 0.596, { sk: 0.0833 }],
      120602: [0.442, 0.194, 0.517, { sk: 0.0833 }],
      120603: [0.431, 0.01, 0.828],
      120604: [0.711, 0, 0.869, { sk: 0.16 }],
      120605: [0.686, 0, 0.866, { sk: 0.0958 }],
      120606: [0.68, 0, 0.657, { ic: 0.12, sk: 0.0958 }],
      120607: [0.711, 0, 0.958, { sk: 0.192 }],
      120608: [0.68, 0, 0.81, { ic: 0.015, sk: 0.0958 }],
      120609: [0.686, 0, 0.773, { ic: 0.032, sk: 0.0958 }],
      120610: [0.686, 0, 0.982, { ic: 0.045, sk: 0.0639 }],
      120611: [0.702, 0.017, 0.867, { sk: 0.0958 }],
      120612: [0.686, 0, 0.511, { ic: 0.062, sk: 0.128 }],
      120613: [0.686, 0, 0.971, { ic: 0.032, sk: 0.0639 }],
      120614: [0.711, 0, 0.806, { sk: 0.192 }],
      120615: [0.686, 0, 1.142, { ic: 0.077, sk: 0.0958 }],
      120616: [0.686, 0, 0.95, { ic: 0.077, sk: 0.0958 }],
      120617: [0.675, 0, 0.841, { ic: 0.026, sk: 0.0958 }],
      120618: [0.703, 0.017, 0.837, { sk: 0.0958 }],
      120619: [0.68, 0, 0.982, { ic: 0.044, sk: 0.0639 }],
      120620: [0.686, 0, 0.723, { ic: 0.124, sk: 0.0958 }],
      120622: [0.686, 0, 0.885, { ic: 0.017, sk: 0.0958 }],
      120623: [0.675, 0, 0.637, { ic: 0.135, sk: 0.0958 }],
      120624: [0.703, 0, 0.671, { ic: 0.131, sk: 0.0639 }],
      120625: [0.686, 0, 0.767, { sk: 0.0958 }],
      120626: [0.686, 0, 0.947, { sk: 0.0958 }],
      120627: [0.686, 0, 0.714, { ic: 0.076, sk: 0.0639 }],
      120628: [0.703, 0, 0.879, { sk: 0.0958 }],
      120630: [0.452, 8e-3, 0.761, { sk: 0.0319 }],
      120631: [0.701, 0.194, 0.66, { sk: 0.0958 }],
      120632: [0.451, 0.211, 0.59, { ic: 0.027 }],
      120633: [0.725, 8e-3, 0.522, { sk: 0.0639 }],
      120634: [0.461, 0.017, 0.529, { sk: 0.0958 }],
      120635: [0.711, 0.202, 0.508, { ic: 0.013, sk: 0.0958 }],
      120636: [0.452, 0.211, 0.6, { sk: 0.0639 }],
      120637: [0.702, 8e-3, 0.562, { sk: 0.0958 }],
      120638: [0.452, 8e-3, 0.412, { sk: 0.0639 }],
      120639: [0.452, 8e-3, 0.668],
      120640: [0.694, 0.013, 0.671],
      120641: [0.452, 0.211, 0.708, { sk: 0.0319 }],
      120642: [0.452, 0, 0.577, { ic: 0.031, sk: 0.0319 }],
      120643: [0.711, 0.201, 0.508, { sk: 0.128 }],
      120644: [0.452, 8e-3, 0.585, { sk: 0.0639 }],
      120645: [0.444, 8e-3, 0.682],
      120646: [0.451, 0.211, 0.612, { sk: 0.0958 }],
      120647: [0.451, 0.105, 0.424, { ic: 0.033, sk: 0.0958 }],
      120648: [0.444, 8e-3, 0.686],
      120649: [0.444, 0.013, 0.521, { ic: 0.089, sk: 0.0319 }],
      120650: [0.453, 8e-3, 0.631, { sk: 0.0319 }],
      120651: [0.452, 0.216, 0.747, { sk: 0.0958 }],
      120652: [0.452, 0.201, 0.718, { sk: 0.0639 }],
      120653: [0.694, 0.202, 0.758, { sk: 0.128 }],
      120654: [0.453, 8e-3, 0.718],
      120655: [0.71, 0.017, 0.628, { ic: 0.029, sk: 0.0958 }],
      120656: [0.444, 7e-3, 0.483, { sk: 0.0639 }],
      120657: [0.701, 8e-3, 0.692, { sk: 0.0958 }],
      120658: [0.434, 6e-3, 0.667, { ic: 0.067 }],
      120659: [0.694, 0.202, 0.712, { sk: 0.0958 }],
      120660: [0.451, 0.194, 0.612, { sk: 0.0958 }],
      120661: [0.444, 8e-3, 0.975],
      120662: [0.694, 0, 0.733],
      120663: [0.694, 0, 0.733],
      120664: [0.691, 0, 0.581],
      120665: [0.694, 0, 0.917],
      120666: [0.691, 0, 0.642],
      120667: [0.694, 0, 0.672],
      120668: [0.694, 0, 0.794],
      120669: [0.716, 0.022, 0.856],
      120670: [0.694, 0, 0.331],
      120671: [0.694, 0, 0.764],
      120672: [0.694, 0, 0.672],
      120673: [0.694, 0, 0.978],
      120674: [0.694, 0, 0.794],
      120675: [0.688, 0, 0.733],
      120676: [0.716, 0.022, 0.794],
      120677: [0.691, 0, 0.794],
      120678: [0.694, 0, 0.703],
      120680: [0.694, 0, 0.794],
      120681: [0.688, 0, 0.733],
      120682: [0.715, 0, 0.856],
      120683: [0.694, 0, 0.794],
      120684: [0.694, 0, 0.733],
      120685: [0.694, 0, 0.856],
      120686: [0.716, 0, 0.794],
      120782: [0.654, 0.01, 0.575],
      120783: [0.655, 0, 0.575],
      120784: [0.654, 0, 0.575],
      120785: [0.655, 0.011, 0.575],
      120786: [0.656, 0, 0.575],
      120787: [0.655, 0.011, 0.575],
      120788: [0.655, 0.011, 0.575],
      120789: [0.676, 0.011, 0.575],
      120790: [0.654, 0.011, 0.575],
      120791: [0.654, 0.011, 0.575],
      120802: [0.678, 0.022, 0.5],
      120803: [0.678, 0, 0.5],
      120804: [0.677, 0, 0.5],
      120805: [0.678, 0.022, 0.5],
      120806: [0.656, 0, 0.5],
      120807: [0.656, 0.021, 0.5],
      120808: [0.677, 0.022, 0.5],
      120809: [0.656, 0.011, 0.5],
      120810: [0.678, 0.022, 0.5],
      120811: [0.677, 0.022, 0.5],
      120812: [0.715, 0.022, 0.55],
      120813: [0.716, 0, 0.55],
      120814: [0.716, 0, 0.55],
      120815: [0.716, 0.022, 0.55],
      120816: [0.694, 0, 0.55],
      120817: [0.694, 0.022, 0.55],
      120818: [0.716, 0.022, 0.55],
      120819: [0.695, 0.011, 0.55],
      120820: [0.715, 0.022, 0.55],
      120821: [0.716, 0.022, 0.55],
      120822: [0.621, 0.01, 0.525],
      120823: [0.622, 0, 0.525],
      120824: [0.622, 0, 0.525],
      120825: [0.622, 0.011, 0.525],
      120826: [0.624, 0, 0.525],
      120827: [0.611, 0.01, 0.525],
      120828: [0.622, 0.011, 0.525],
      120829: [0.627, 0.01, 0.525],
      120830: [0.621, 0.01, 0.525],
      120831: [0.622, 0.011, 0.525]
    }), normal;
  }
  var hasRequiredNormal;
  function requireNormal() {
    if (hasRequiredNormal) return normal$1;
    hasRequiredNormal = 1, Object.defineProperty(normal$1, "__esModule", { value: !0 }), normal$1.normal = void 0;
    var O = requireFontData(), _ = requireNormal$1();
    return normal$1.normal = (0, O.AddPaths)(_.normal, {
      32: "",
      33: "78 661Q78 682 96 699T138 716T180 700T199 661Q199 654 179 432T158 206Q156 198 139 198Q121 198 119 206Q118 209 98 431T78 661ZM79 61Q79 89 97 105T141 121Q164 119 181 104T198 61Q198 31 181 16T139 1Q114 1 97 16T79 61",
      34: "34 634Q34 659 50 676T93 694Q121 694 144 668T168 579Q168 525 146 476T101 403T73 379Q69 379 60 388T50 401Q50 404 62 417T88 448T116 500T131 572Q131 584 130 584T125 581T112 576T94 573Q69 573 52 590T34 634ZM238 634Q238 659 254 676T297 694Q325 694 348 668T372 579Q372 525 350 476T305 403T277 379Q273 379 264 388T254 401Q254 404 266 417T292 448T320 500T335 572Q335 584 334 584T329 581T316 576T298 573Q273 573 256 590T238 634",
      35: "56 347Q56 360 70 367H313L355 524Q394 676 401 686Q406 694 416 694Q434 694 436 676Q436 672 396 522Q355 374 355 369L354 367H543L585 524Q626 679 630 685Q636 694 646 694Q653 694 659 689T665 678Q665 668 626 522Q585 374 585 369L584 367H762Q777 359 777 347Q777 334 767 331T722 327H667H572L552 251L531 174Q531 173 647 173H720Q756 173 766 170T777 153T762 133H519L477 -24Q436 -179 432 -185Q426 -194 416 -194Q409 -194 403 -189T397 -177Q397 -167 436 -21Q477 125 477 131L478 133H289L247 -24Q206 -179 202 -185Q196 -194 186 -194Q179 -194 173 -189T167 -177Q167 -167 206 -21Q247 125 247 131L248 133H70Q56 140 56 153Q56 168 72 173H260L280 249L301 326Q301 327 186 327H72Q56 332 56 347ZM531 326Q531 327 437 327H342L322 251L301 174Q301 173 395 173H490L510 249L531 326",
      36: "162 187Q162 164 146 149T109 133H103V130Q108 115 115 105Q122 92 131 82T150 64T170 52T190 44T206 40T220 37L227 36V313Q190 320 162 335Q116 358 86 404T55 508Q55 567 85 614T165 685Q186 696 225 704H227V750H273V704L286 703Q369 690 413 631Q441 588 444 531Q444 514 443 509Q439 490 425 479T391 468Q368 468 353 483T337 522Q337 546 353 560T390 575L394 576V578Q386 599 372 614T342 637T314 649T288 656L273 658V408L288 405Q329 394 355 376Q396 348 420 300T444 199Q444 130 408 76T313 1Q286 -9 276 -9H273V-56H227V-10H221Q202 -6 193 -4T155 11T108 41T74 94T55 176V182Q55 227 95 238Q103 240 108 240Q129 240 145 226T162 187ZM225 657Q219 657 204 651T169 632T135 594T121 538Q121 512 131 491T156 457T187 435T213 423T227 420V539Q227 657 225 657ZM378 169Q378 230 339 265T274 301Q273 301 273 169V37Q324 50 351 87T378 169",
      37: "465 605Q428 605 394 614T340 632T319 641Q332 608 332 548Q332 458 293 403T202 347Q145 347 101 402T56 548Q56 637 101 693T202 750Q241 750 272 719Q359 642 464 642Q580 642 650 732Q662 748 668 749Q670 750 673 750Q682 750 688 743T693 726Q178 -47 170 -52Q166 -56 160 -56Q147 -56 142 -45Q137 -36 142 -27Q143 -24 363 304Q469 462 525 546T581 630Q528 605 465 605ZM207 385Q235 385 263 427T292 548Q292 617 267 664T200 712Q193 712 186 709T167 698T147 668T134 615Q132 595 132 548V527Q132 436 165 403Q183 385 203 385H207ZM500 146Q500 234 544 290T647 347Q699 347 737 292T776 146T737 0T646 -56Q590 -56 545 0T500 146ZM651 -18Q679 -18 707 24T736 146Q736 215 711 262T644 309Q637 309 630 306T611 295T591 265T578 212Q577 200 577 146V124Q577 -18 647 -18H651",
      38: "156 540Q156 620 201 668T302 716Q354 716 377 671T401 578Q401 505 287 386L274 373Q309 285 416 148L429 132L437 142Q474 191 543 309L562 341V349Q562 368 541 376T498 385H493V431H502L626 428Q709 428 721 431H727V385H712Q688 384 669 379T639 369T618 354T603 337T591 316T578 295Q537 223 506 176T464 117T454 104Q454 102 471 85T497 62Q543 24 585 24Q618 24 648 48T682 113V121H722V112Q721 94 714 75T692 32T646 -7T574 -22Q491 -19 414 42L402 51L391 42Q312 -22 224 -22Q144 -22 93 25T42 135Q42 153 46 169T55 197T74 225T96 249T125 278T156 308L195 347L190 360Q185 372 182 382T174 411T165 448T159 491T156 540ZM361 576Q361 613 348 646T305 679Q272 679 252 649T232 572Q232 497 255 426L259 411L267 420Q361 519 361 576ZM140 164Q140 103 167 64T240 24Q271 24 304 36T356 61T374 77Q295 156 235 262L220 292L210 310L193 293Q177 277 169 268T151 229T140 164",
      39: "78 634Q78 659 95 676T138 694Q166 694 189 668T212 579Q212 525 190 476T146 403T118 379Q114 379 105 388T95 401Q95 404 107 417T133 448T161 500T176 572Q176 584 175 584T170 581T157 576T139 573Q114 573 96 590T78 634",
      40: "94 250Q94 319 104 381T127 488T164 576T202 643T244 695T277 729T302 750H315H319Q333 750 333 741Q333 738 316 720T275 667T226 581T184 443T167 250T184 58T225 -81T274 -167T316 -220T333 -241Q333 -250 318 -250H315H302L274 -226Q180 -141 137 -14T94 250",
      41: "60 749L64 750Q69 750 74 750H86L114 726Q208 641 251 514T294 250Q294 182 284 119T261 12T224 -76T186 -143T145 -194T113 -227T90 -246Q87 -249 86 -250H74Q66 -250 63 -250T58 -247T55 -238Q56 -237 66 -225Q221 -64 221 250T66 725Q56 737 55 738Q55 746 60 749",
      42: "215 721Q216 732 225 741T248 750Q263 750 273 742T284 721L270 571L327 613Q383 654 388 657T399 660Q412 660 423 650T435 624T424 600T376 575Q363 569 355 566L289 534L355 504L424 470Q435 462 435 447Q435 431 424 420T399 409Q393 409 388 412T327 456L270 498L277 423L284 348Q280 320 250 320T215 348L229 498L172 456Q116 415 111 412T100 409Q87 409 76 420T64 447Q64 461 75 470L144 504L210 534L144 566Q136 570 122 576Q83 593 74 600T64 624Q64 639 75 649T100 660Q106 660 111 657T172 613L229 571Q229 578 222 643T215 721",
      43: "56 237T56 250T70 270H369V420L370 570Q380 583 389 583Q402 583 409 568V270H707Q722 262 722 250T707 230H409V-68Q401 -82 391 -82H389H387Q375 -82 369 -68V230H70Q56 237 56 250",
      44: "78 35T78 60T94 103T137 121Q165 121 187 96T210 8Q210 -27 201 -60T180 -117T154 -158T130 -185T117 -194Q113 -194 104 -185T95 -172Q95 -168 106 -156T131 -126T157 -76T173 -3V9L172 8Q170 7 167 6T161 3T152 1T140 0Q113 0 96 17",
      45: "11 179V252H277V179H11",
      46: "78 60Q78 84 95 102T138 120Q162 120 180 104T199 61Q199 36 182 18T139 0T96 17T78 60",
      47: "423 750Q432 750 438 744T444 730Q444 725 271 248T92 -240Q85 -250 75 -250Q68 -250 62 -245T56 -231Q56 -221 230 257T407 740Q411 750 423 750",
      48: "96 585Q152 666 249 666Q297 666 345 640T423 548Q460 465 460 320Q460 165 417 83Q397 41 362 16T301 -15T250 -22Q224 -22 198 -16T137 16T82 83Q39 165 39 320Q39 494 96 585ZM321 597Q291 629 250 629Q208 629 178 597Q153 571 145 525T137 333Q137 175 145 125T181 46Q209 16 250 16Q290 16 318 46Q347 76 354 130T362 333Q362 478 354 524T321 597",
      49: "213 578L200 573Q186 568 160 563T102 556H83V602H102Q149 604 189 617T245 641T273 663Q275 666 285 666Q294 666 302 660V361L303 61Q310 54 315 52T339 48T401 46H427V0H416Q395 3 257 3Q121 3 100 0H88V46H114Q136 46 152 46T177 47T193 50T201 52T207 57T213 61V578",
      50: "109 429Q82 429 66 447T50 491Q50 562 103 614T235 666Q326 666 387 610T449 465Q449 422 429 383T381 315T301 241Q265 210 201 149L142 93L218 92Q375 92 385 97Q392 99 409 186V189H449V186Q448 183 436 95T421 3V0H50V19V31Q50 38 56 46T86 81Q115 113 136 137Q145 147 170 174T204 211T233 244T261 278T284 308T305 340T320 369T333 401T340 431T343 464Q343 527 309 573T212 619Q179 619 154 602T119 569T109 550Q109 549 114 549Q132 549 151 535T170 489Q170 464 154 447T109 429",
      51: "127 463Q100 463 85 480T69 524Q69 579 117 622T233 665Q268 665 277 664Q351 652 390 611T430 522Q430 470 396 421T302 350L299 348Q299 347 308 345T337 336T375 315Q457 262 457 175Q457 96 395 37T238 -22Q158 -22 100 21T42 130Q42 158 60 175T105 193Q133 193 151 175T169 130Q169 119 166 110T159 94T148 82T136 74T126 70T118 67L114 66Q165 21 238 21Q293 21 321 74Q338 107 338 175V195Q338 290 274 322Q259 328 213 329L171 330L168 332Q166 335 166 348Q166 366 174 366Q202 366 232 371Q266 376 294 413T322 525V533Q322 590 287 612Q265 626 240 626Q208 626 181 615T143 592T132 580H135Q138 579 143 578T153 573T165 566T175 555T183 540T186 520Q186 498 172 481T127 463",
      52: "462 0Q444 3 333 3Q217 3 199 0H190V46H221Q241 46 248 46T265 48T279 53T286 61Q287 63 287 115V165H28V211L179 442Q332 674 334 675Q336 677 355 677H373L379 671V211H471V165H379V114Q379 73 379 66T385 54Q393 47 442 46H471V0H462ZM293 211V545L74 212L183 211H293",
      53: "164 157Q164 133 148 117T109 101H102Q148 22 224 22Q294 22 326 82Q345 115 345 210Q345 313 318 349Q292 382 260 382H254Q176 382 136 314Q132 307 129 306T114 304Q97 304 95 310Q93 314 93 485V614Q93 664 98 664Q100 666 102 666Q103 666 123 658T178 642T253 634Q324 634 389 662Q397 666 402 666Q410 666 410 648V635Q328 538 205 538Q174 538 149 544L139 546V374Q158 388 169 396T205 412T256 420Q337 420 393 355T449 201Q449 109 385 44T229 -22Q148 -22 99 32T50 154Q50 178 61 192T84 210T107 214Q132 214 148 197T164 157",
      54: "42 313Q42 476 123 571T303 666Q372 666 402 630T432 550Q432 525 418 510T379 495Q356 495 341 509T326 548Q326 592 373 601Q351 623 311 626Q240 626 194 566Q147 500 147 364L148 360Q153 366 156 373Q197 433 263 433H267Q313 433 348 414Q372 400 396 374T435 317Q456 268 456 210V192Q456 169 451 149Q440 90 387 34T253 -22Q225 -22 199 -14T143 16T92 75T56 172T42 313ZM257 397Q227 397 205 380T171 335T154 278T148 216Q148 133 160 97T198 39Q222 21 251 21Q302 21 329 59Q342 77 347 104T352 209Q352 289 347 316T329 361Q302 397 257 397",
      55: "55 458Q56 460 72 567L88 674Q88 676 108 676H128V672Q128 662 143 655T195 646T364 644H485V605L417 512Q408 500 387 472T360 435T339 403T319 367T305 330T292 284T284 230T278 162T275 80Q275 66 275 52T274 28V19Q270 2 255 -10T221 -22Q210 -22 200 -19T179 0T168 40Q168 198 265 368Q285 400 349 489L395 552H302Q128 552 119 546Q113 543 108 522T98 479L95 458V455H55V458",
      56: "70 417T70 494T124 618T248 666Q319 666 374 624T429 515Q429 485 418 459T392 417T361 389T335 371T324 363L338 354Q352 344 366 334T382 323Q457 264 457 174Q457 95 399 37T249 -22Q159 -22 101 29T43 155Q43 263 172 335L154 348Q133 361 127 368Q70 417 70 494ZM286 386L292 390Q298 394 301 396T311 403T323 413T334 425T345 438T355 454T364 471T369 491T371 513Q371 556 342 586T275 624Q268 625 242 625Q201 625 165 599T128 534Q128 511 141 492T167 463T217 431Q224 426 228 424L286 386ZM250 21Q308 21 350 55T392 137Q392 154 387 169T375 194T353 216T330 234T301 253T274 270Q260 279 244 289T218 306L210 311Q204 311 181 294T133 239T107 157Q107 98 150 60T250 21",
      57: "352 287Q304 211 232 211Q154 211 104 270T44 396Q42 412 42 436V444Q42 537 111 606Q171 666 243 666Q245 666 249 666T257 665H261Q273 665 286 663T323 651T370 619T413 560Q456 472 456 334Q456 194 396 97Q361 41 312 10T208 -22Q147 -22 108 7T68 93T121 149Q143 149 158 135T173 96Q173 78 164 65T148 49T135 44L131 43Q131 41 138 37T164 27T206 22H212Q272 22 313 86Q352 142 352 280V287ZM244 248Q292 248 321 297T351 430Q351 508 343 542Q341 552 337 562T323 588T293 615T246 625Q208 625 181 598Q160 576 154 546T147 441Q147 358 152 329T172 282Q197 248 244 248",
      58: "78 370Q78 394 95 412T138 430Q162 430 180 414T199 371Q199 346 182 328T139 310T96 327T78 370ZM78 60Q78 84 95 102T138 120Q162 120 180 104T199 61Q199 36 182 18T139 0T96 17T78 60",
      59: "78 370Q78 394 95 412T138 430Q162 430 180 414T199 371Q199 346 182 328T139 310T96 327T78 370ZM78 60Q78 85 94 103T137 121Q202 121 202 8Q202 -44 183 -94T144 -169T118 -194Q115 -194 106 -186T95 -174Q94 -171 107 -155T137 -107T160 -38Q161 -32 162 -22T165 -4T165 4Q165 5 161 4T142 0Q110 0 94 18T78 60",
      60: "694 -11T694 -19T688 -33T678 -40Q671 -40 524 29T234 166L90 235Q83 240 83 250Q83 261 91 266Q664 540 678 540Q681 540 687 534T694 519T687 505Q686 504 417 376L151 250L417 124Q686 -4 687 -5Q694 -11 694 -19",
      61: "56 347Q56 360 70 367H707Q722 359 722 347Q722 336 708 328L390 327H72Q56 332 56 347ZM56 153Q56 168 72 173H708Q722 163 722 153Q722 140 707 133H70Q56 140 56 153",
      62: "84 520Q84 528 88 533T96 539L99 540Q106 540 253 471T544 334L687 265Q694 260 694 250T687 235Q685 233 395 96L107 -40H101Q83 -38 83 -20Q83 -19 83 -17Q82 -10 98 -1Q117 9 248 71Q326 108 378 132L626 250L378 368Q90 504 86 509Q84 513 84 520",
      63: "226 668Q190 668 162 656T124 632L114 621Q116 621 119 620T130 616T145 607T157 591T162 567Q162 544 147 529T109 514T71 528T55 566Q55 625 100 661T199 704Q201 704 210 704T224 705H228Q281 705 320 692T378 656T407 612T416 567Q416 503 361 462Q267 395 247 303Q242 279 242 241V224Q242 205 239 202T222 198T205 201T202 218V249Q204 320 220 371T255 445T292 491T315 537Q317 546 317 574V587Q317 604 315 615T304 640T277 661T226 668ZM162 61Q162 89 180 105T224 121Q247 119 264 104T281 61Q281 31 264 16T222 1Q197 1 180 16T162 61",
      64: "56 347Q56 429 86 498T164 612T270 680T386 705Q522 705 622 603T722 349Q722 126 608 126Q541 126 513 176Q512 177 512 179T510 182L509 183Q508 183 503 177T487 163T464 146T429 132T385 126Q311 126 251 186T190 347Q190 448 251 508T385 568Q426 568 460 548T509 511T531 479H555Q580 479 582 478Q586 477 587 468Q588 454 588 338V260Q588 200 593 182T619 163Q641 163 655 178T674 223T680 273T682 325V330Q682 426 647 500Q611 569 544 618T388 668Q271 668 184 577T96 347Q96 216 180 121T396 26Q421 26 446 28T493 34T535 43T573 52T605 63T629 72T647 80T657 84H716Q722 78 722 74Q722 65 675 45T547 7T392 -11Q255 -11 156 90T56 347ZM274 347Q274 266 308 214T390 162Q420 162 449 182T498 235L504 245V449L498 459Q453 532 387 532Q347 532 311 483T274 347",
      65: "255 0Q240 3 140 3Q48 3 39 0H32V46H47Q119 49 139 88Q140 91 192 245T295 553T348 708Q351 716 366 716H376Q396 715 400 709Q402 707 508 390L617 67Q624 54 636 51T687 46H717V0H708Q699 3 581 3Q458 3 437 0H427V46H440Q510 46 510 64Q510 66 486 138L462 209H229L209 150Q189 91 189 85Q189 72 209 59T259 46H264V0H255ZM447 255L345 557L244 256Q244 255 345 255H447",
      66: "131 622Q124 629 120 631T104 634T61 637H28V683H229H267H346Q423 683 459 678T531 651Q574 627 599 590T624 512Q624 461 583 419T476 360L466 357Q539 348 595 302T651 187Q651 119 600 67T469 3Q456 1 242 0H28V46H61Q103 47 112 49T131 61V622ZM511 513Q511 560 485 594T416 636Q415 636 403 636T371 636T333 637Q266 637 251 636T232 628Q229 624 229 499V374H312L396 375L406 377Q410 378 417 380T442 393T474 417T499 456T511 513ZM537 188Q537 239 509 282T430 336L329 337H229V200V116Q229 57 234 52Q240 47 334 47H383Q425 47 443 53Q486 67 511 104T537 188",
      67: "56 342Q56 428 89 500T174 615T283 681T391 705Q394 705 400 705T408 704Q499 704 569 636L582 624L612 663Q639 700 643 704Q644 704 647 704T653 705H657Q660 705 666 699V419L660 413H626Q620 419 619 430Q610 512 571 572T476 651Q457 658 426 658Q322 658 252 588Q173 509 173 342Q173 221 211 151Q232 111 263 84T328 45T384 29T428 24Q517 24 571 93T626 244Q626 251 632 257H660L666 251V236Q661 133 590 56T403 -21Q262 -21 159 83T56 342",
      68: "130 622Q123 629 119 631T103 634T60 637H27V683H228Q399 682 419 682T461 676Q504 667 546 641T626 573T685 470T708 336Q708 210 634 116T442 3Q429 1 228 0H27V46H60Q102 47 111 49T130 61V622ZM593 338Q593 439 571 501T493 602Q439 637 355 637H322H294Q238 637 234 628Q231 624 231 344Q231 62 232 59Q233 49 248 48T339 46H350Q456 46 515 95Q561 133 577 191T593 338",
      69: "128 619Q121 626 117 628T101 631T58 634H25V680H597V676Q599 670 611 560T625 444V440H585V444Q584 447 582 465Q578 500 570 526T553 571T528 601T498 619T457 629T411 633T353 634Q266 634 251 633T233 622Q233 622 233 621Q232 619 232 497V376H286Q359 378 377 385Q413 401 416 469Q416 471 416 473V493H456V213H416V233Q415 268 408 288T383 317T349 328T297 330Q290 330 286 330H232V196V114Q232 57 237 52Q243 47 289 47H340H391Q428 47 452 50T505 62T552 92T584 146Q594 172 599 200T607 247T612 270V273H652V270Q651 267 632 137T610 3V0H25V46H58Q100 47 109 49T128 61V619",
      70: "128 619Q121 626 117 628T101 631T58 634H25V680H582V676Q584 670 596 560T610 444V440H570V444Q563 493 561 501Q555 538 543 563T516 601T477 622T431 631T374 633H334H286Q252 633 244 631T233 621Q232 619 232 490V363H284Q287 363 303 363T327 364T349 367T372 373T389 385Q407 403 410 459V480H450V200H410V221Q407 276 389 296Q381 303 371 307T348 313T327 316T303 317T284 317H232V189L233 61Q240 54 245 52T270 48T333 46H360V0H348Q324 3 182 3Q51 3 36 0H25V46H58Q100 47 109 49T128 61V619",
      71: "56 342Q56 428 89 500T174 615T283 681T391 705Q394 705 400 705T408 704Q499 704 569 636L582 624L612 663Q639 700 643 704Q644 704 647 704T653 705H657Q660 705 666 699V419L660 413H626Q620 419 619 430Q610 512 571 572T476 651Q457 658 426 658Q401 658 376 654T316 633T254 592T205 519T177 411Q173 369 173 335Q173 259 192 201T238 111T302 58T370 31T431 24Q478 24 513 45T559 100Q562 110 562 160V212Q561 213 557 216T551 220T542 223T526 225T502 226T463 227H437V273H449L609 270Q715 270 727 273H735V227H721Q674 227 668 215Q666 211 666 108V6Q660 0 657 0Q653 0 639 10Q617 25 600 42L587 54Q571 27 524 3T406 -22Q317 -22 238 22T108 151T56 342",
      72: "128 622Q121 629 117 631T101 634T58 637H25V683H36Q57 680 180 680Q315 680 324 683H335V637H302Q262 636 251 634T233 622L232 500V378H517V622Q510 629 506 631T490 634T447 637H414V683H425Q446 680 569 680Q704 680 713 683H724V637H691Q651 636 640 634T622 622V61Q628 51 639 49T691 46H724V0H713Q692 3 569 3Q434 3 425 0H414V46H447Q489 47 498 49T517 61V332H232V197L233 61Q239 51 250 49T302 46H335V0H324Q303 3 180 3Q45 3 36 0H25V46H58Q100 47 109 49T128 61V622",
      73: "328 0Q307 3 180 3T32 0H21V46H43Q92 46 106 49T126 60Q128 63 128 342Q128 620 126 623Q122 628 118 630T96 635T43 637H21V683H32Q53 680 180 680T328 683H339V637H317Q268 637 254 634T234 623Q232 620 232 342Q232 63 234 60Q238 55 242 53T264 48T317 46H339V0H328",
      74: "89 177Q115 177 133 160T152 112Q152 88 137 72T102 52Q99 51 101 49Q106 43 129 29Q159 15 190 15Q232 15 256 48T286 126Q286 127 286 142T286 183T286 238T287 306T287 378Q287 403 287 429T287 479T287 524T286 563T286 593T286 614V621Q281 630 263 633T182 637H154V683H166Q187 680 332 680Q439 680 457 683H465V637H449Q422 637 401 634Q393 631 389 623Q388 621 388 376T387 123Q377 61 322 20T194 -22Q188 -22 177 -21T160 -20Q96 -9 61 29T25 110Q25 144 44 160T89 177",
      75: "128 622Q121 629 117 631T101 634T58 637H25V683H36Q57 680 180 680Q315 680 324 683H335V637H313Q235 637 233 620Q232 618 232 462L233 307L379 449Q425 494 479 546Q518 584 524 591T531 607V608Q531 630 503 636Q501 636 498 636T493 637H489V683H499Q517 680 630 680Q704 680 716 683H722V637H708Q633 633 589 597Q584 592 495 506T406 419T515 254T631 80Q644 60 662 54T715 46H736V0H728Q719 3 615 3Q493 3 472 0H461V46H469Q515 46 515 72Q515 78 512 84L336 351Q332 348 278 296L232 251V156Q232 62 235 58Q243 47 302 46H335V0H324Q303 3 180 3Q45 3 36 0H25V46H58Q100 47 109 49T128 61V622",
      76: "128 622Q121 629 117 631T101 634T58 637H25V683H36Q48 680 182 680Q324 680 348 683H360V637H333Q273 637 258 635T233 622L232 342V129Q232 57 237 52Q243 47 313 47Q384 47 410 53Q470 70 498 110T536 221Q536 226 537 238T540 261T542 272T562 273H582V268Q580 265 568 137T554 5V0H25V46H58Q100 47 109 49T128 61V622",
      77: "132 622Q125 629 121 631T105 634T62 637H29V683H135Q221 683 232 682T249 675Q250 674 354 398L458 124L562 398Q666 674 668 675Q671 681 683 682T781 683H887V637H854Q814 636 803 634T785 622V61Q791 51 802 49T854 46H887V0H876Q855 3 736 3Q605 3 596 0H585V46H618Q660 47 669 49T688 61V347Q688 424 688 461T688 546T688 613L687 632Q454 14 450 7Q446 1 430 1T410 7Q409 9 292 316L176 624V606Q175 588 175 543T175 463T175 356L176 86Q187 50 261 46H278V0H269Q254 3 154 3Q52 3 37 0H29V46H46Q78 48 98 56T122 69T132 86V622",
      78: "42 46Q74 48 94 56T118 69T128 86V634H124Q114 637 52 637H25V683H232L235 680Q237 679 322 554T493 303L578 178V598Q572 608 568 613T544 627T492 637H475V683H483Q498 680 600 680Q706 680 715 683H724V637H707Q634 633 622 598L621 302V6L614 0H600Q585 0 582 3T481 150T282 443T171 605V345L172 86Q183 50 257 46H274V0H265Q250 3 150 3Q48 3 33 0H25V46H42",
      79: "56 340Q56 423 86 494T164 610T270 680T388 705Q521 705 621 601T722 341Q722 260 693 191T617 75T510 4T388 -22T267 3T160 74T85 189T56 340ZM467 647Q426 665 388 665Q360 665 331 654T269 620T213 549T179 439Q174 411 174 354Q174 144 277 61Q327 20 385 20H389H391Q474 20 537 99Q603 188 603 354Q603 411 598 439Q577 592 467 647",
      80: "130 622Q123 629 119 631T103 634T60 637H27V683H214Q237 683 276 683T331 684Q419 684 471 671T567 616Q624 563 624 489Q624 421 573 372T451 307Q429 302 328 301H234V181Q234 62 237 58Q245 47 304 46H337V0H326Q305 3 182 3Q47 3 38 0H27V46H60Q102 47 111 49T130 61V622ZM507 488Q507 514 506 528T500 564T483 597T450 620T397 635Q385 637 307 637H286Q237 637 234 628Q231 624 231 483V342H302H339Q390 342 423 349T481 382Q507 411 507 488",
      81: "56 341Q56 499 157 602T388 705Q521 705 621 601T722 341Q722 275 703 218T660 127T603 63T555 25T525 9Q524 8 524 8H523Q524 5 526 -1T537 -21T555 -47T581 -67T615 -76Q653 -76 678 -56T706 -3Q707 10 716 10Q721 10 728 5L727 -13Q727 -88 697 -140T606 -193Q563 -193 538 -166T498 -83Q483 -23 483 -8L471 -11Q459 -14 435 -18T388 -22Q254 -22 155 81T56 341ZM607 339Q607 429 586 496T531 598T461 649T390 665T318 649T248 598T192 496T170 339Q170 143 277 57Q301 39 305 39L304 42Q304 44 304 46Q301 53 301 68Q301 101 325 128T391 155Q454 155 495 70L501 58Q549 91 578 164Q607 234 607 339ZM385 18Q404 18 425 23T459 33T472 40Q471 47 468 57T449 88T412 115Q398 117 386 117Q367 117 353 102T338 67Q338 48 351 33T385 18",
      82: "130 622Q123 629 119 631T103 634T60 637H27V683H202H236H300Q376 683 417 677T500 648Q595 600 609 517Q610 512 610 501Q610 468 594 439T556 392T511 361T472 343L456 338Q459 335 467 332Q497 316 516 298T545 254T559 211T568 155T578 94Q588 46 602 31T640 16H645Q660 16 674 32T692 87Q692 98 696 101T712 105T728 103T732 90Q732 59 716 27T672 -16Q656 -22 630 -22Q481 -16 458 90Q456 101 456 163T449 246Q430 304 373 320L363 322L297 323H231V192L232 61Q238 51 249 49T301 46H334V0H323Q302 3 181 3Q59 3 38 0H27V46H60Q102 47 111 49T130 61V622ZM491 499V509Q491 527 490 539T481 570T462 601T424 623T362 636Q360 636 340 636T304 637H283Q238 637 234 628Q231 624 231 492V360H289Q390 360 434 378T489 456Q491 467 491 499",
      83: "55 507Q55 590 112 647T243 704H257Q342 704 405 641L426 672Q431 679 436 687T446 700L449 704Q450 704 453 704T459 705H463Q466 705 472 699V462L466 456H448Q437 456 435 459T430 479Q413 605 329 646Q292 662 254 662Q201 662 168 626T135 542Q135 508 152 480T200 435Q210 431 286 412T370 389Q427 367 463 314T500 191Q500 110 448 45T301 -21Q245 -21 201 -4T140 27L122 41Q118 36 107 21T87 -7T78 -21Q76 -22 68 -22H64Q61 -22 55 -16V101Q55 220 56 222Q58 227 76 227H89Q95 221 95 214Q95 182 105 151T139 90T205 42T305 24Q352 24 386 62T420 155Q420 198 398 233T340 281Q284 295 266 300Q261 301 239 306T206 314T174 325T141 343T112 367T85 402Q55 451 55 507",
      84: "36 443Q37 448 46 558T55 671V677H666V671Q667 666 676 556T685 443V437H645V443Q645 445 642 478T631 544T610 593Q593 614 555 625Q534 630 478 630H451H443Q417 630 414 618Q413 616 413 339V63Q420 53 439 50T528 46H558V0H545L361 3Q186 1 177 0H164V46H194Q264 46 283 49T309 63V339V550Q309 620 304 625T271 630H244H224Q154 630 119 601Q101 585 93 554T81 486T76 443V437H36V443",
      85: "128 622Q121 629 117 631T101 634T58 637H25V683H36Q57 680 180 680Q315 680 324 683H335V637H302Q262 636 251 634T233 622L232 418V291Q232 189 240 145T280 67Q325 24 389 24Q454 24 506 64T571 183Q575 206 575 410V598Q569 608 565 613T541 627T489 637H472V683H481Q496 680 598 680T715 683H724V637H707Q634 633 622 598L621 399Q620 194 617 180Q617 179 615 171Q595 83 531 31T389 -22Q304 -22 226 33T130 192Q129 201 128 412V622",
      86: "114 620Q113 621 110 624T107 627T103 630T98 632T91 634T80 635T67 636T48 637H19V683H28Q46 680 152 680Q273 680 294 683H305V637H284Q223 634 223 620Q223 618 313 372T404 126L490 358Q575 588 575 597Q575 616 554 626T508 637H503V683H512Q527 680 627 680Q718 680 724 683H730V637H723Q648 637 627 596Q627 595 515 291T401 -14Q396 -22 382 -22H374H367Q353 -22 348 -14Q346 -12 231 303Q114 617 114 620",
      87: "792 683Q810 680 914 680Q991 680 1003 683H1009V637H996Q931 633 915 598Q912 591 863 438T766 135T716 -17Q711 -22 694 -22Q676 -22 673 -15Q671 -13 593 231L514 477L435 234Q416 174 391 92T358 -6T341 -22H331Q314 -21 310 -15Q309 -14 208 302T104 622Q98 632 87 633Q73 637 35 637H18V683H27Q69 681 154 681Q164 681 181 681T216 681T249 682T276 683H287H298V637H285Q213 637 213 620Q213 616 289 381L364 144L427 339Q490 535 492 546Q487 560 482 578T475 602T468 618T461 628T449 633T433 636T408 637H380V683H388Q397 680 508 680Q629 680 650 683H660V637H647Q576 637 576 619L727 146Q869 580 869 600Q869 605 863 612T839 627T794 637H783V683H792",
      88: "270 0Q252 3 141 3Q46 3 31 0H23V46H40Q129 50 161 88Q165 94 244 216T324 339Q324 341 235 480T143 622Q133 631 119 634T57 637H37V683H46Q64 680 172 680Q297 680 318 683H329V637H324Q307 637 286 632T263 621Q263 618 322 525T384 431Q385 431 437 511T489 593Q490 595 490 599Q490 611 477 622T436 637H428V683H437Q455 680 566 680Q661 680 676 683H684V637H667Q585 634 551 599Q548 596 478 491Q412 388 412 387Q412 385 514 225T620 62Q628 53 642 50T695 46H726V0H717Q699 3 591 3Q466 3 445 0H434V46H440Q454 46 476 51T499 64Q499 67 463 124T390 238L353 295L350 292Q348 290 343 283T331 265T312 236T286 195Q219 88 218 84Q218 70 234 59T272 46H280V0H270",
      89: "518 0Q497 3 374 3Q253 3 232 0H221V46H254Q313 47 321 58Q324 62 324 167V273L221 446Q117 620 114 623Q106 631 91 634T31 637H11V683H20Q29 680 148 680Q273 680 294 683H305V637H287Q239 636 236 621Q236 619 321 475L407 332L483 460Q502 492 527 534Q563 594 563 604Q563 632 517 637H508V683H517H525Q533 683 545 683T571 682T600 681T626 681Q695 681 731 683H738V637H723Q640 633 613 588Q612 587 517 427L425 273V169V95Q425 66 428 59T444 49Q459 46 506 46H528V0H518",
      90: "69 443Q69 452 74 554T80 683H549Q555 677 555 664Q555 649 554 648Q552 645 366 348T179 50T192 49T263 49H275H302Q333 49 353 50T401 59T447 78T482 115T507 173Q513 200 520 273V282H560V274Q560 272 552 143T543 8V0H302L61 1L58 3Q55 8 55 21V35Q59 43 153 193T340 489T432 637H343Q259 637 214 625T141 573Q109 523 109 445Q109 443 89 443H69",
      91: "118 -250V750H255V710H158V-210H255V-250H118",
      92: "56 731Q56 740 62 745T75 750Q85 750 92 740Q96 733 270 255T444 -231Q444 -239 438 -244T424 -250Q414 -250 407 -240Q404 -236 230 242T56 731",
      93: "22 710V750H159V-250H22V-210H119V710H22",
      94: "112 560L249 694L257 686Q387 562 387 560L361 531Q359 532 303 581L250 627L195 580Q182 569 169 557T148 538L140 532Q138 530 125 546L112 560",
      95: "0 -62V-25H499V-62H0",
      96: "106 655Q106 671 119 685T150 699Q166 699 177 688Q190 671 222 629T275 561T295 533T282 519L267 505L196 563Q119 626 113 634Q106 643 106 655",
      97: "137 305T115 305T78 320T63 359Q63 394 97 421T218 448Q291 448 336 416T396 340Q401 326 401 309T402 194V124Q402 76 407 58T428 40Q443 40 448 56T453 109V145H493V106Q492 66 490 59Q481 29 455 12T400 -6T353 12T329 54V58L327 55Q325 52 322 49T314 40T302 29T287 17T269 6T247 -2T221 -8T190 -11Q130 -11 82 20T34 107Q34 128 41 147T68 188T116 225T194 253T304 268H318V290Q318 324 312 340Q290 411 215 411Q197 411 181 410T156 406T148 403Q170 388 170 359Q170 334 154 320ZM126 106Q126 75 150 51T209 26Q247 26 276 49T315 109Q317 116 318 175Q318 233 317 233Q309 233 296 232T251 223T193 203T147 166T126 106",
      98: "307 -11Q234 -11 168 55L158 37Q156 34 153 28T147 17T143 10L138 1L118 0H98V298Q98 599 97 603Q94 622 83 628T38 637H20V660Q20 683 22 683L32 684Q42 685 61 686T98 688Q115 689 135 690T165 693T176 694H179V543Q179 391 180 391L183 394Q186 397 192 401T207 411T228 421T254 431T286 439T323 442Q401 442 461 379T522 216Q522 115 458 52T307 -11ZM182 98Q182 97 187 90T196 79T206 67T218 55T233 44T250 35T271 29T295 26Q330 26 363 46T412 113Q424 148 424 212Q424 287 412 323Q385 405 300 405Q270 405 239 390T188 347L182 339V98",
      99: "370 305T349 305T313 320T297 358Q297 381 312 396Q317 401 317 402T307 404Q281 408 258 408Q209 408 178 376Q131 329 131 219Q131 137 162 90Q203 29 272 29Q313 29 338 55T374 117Q376 125 379 127T395 129H409Q415 123 415 120Q415 116 411 104T395 71T366 33T318 2T249 -11Q163 -11 99 53T34 214Q34 318 99 383T250 448T370 421T404 357Q404 334 387 320",
      100: "376 495Q376 511 376 535T377 568Q377 613 367 624T316 637H298V660Q298 683 300 683L310 684Q320 685 339 686T376 688Q393 689 413 690T443 693T454 694H457V390Q457 84 458 81Q461 61 472 55T517 46H535V0Q533 0 459 -5T380 -11H373V44L365 37Q307 -11 235 -11Q158 -11 96 50T34 215Q34 315 97 378T244 442Q319 442 376 393V495ZM373 342Q328 405 260 405Q211 405 173 369Q146 341 139 305T131 211Q131 155 138 120T173 59Q203 26 251 26Q322 26 373 103V342",
      101: "28 218Q28 273 48 318T98 391T163 433T229 448Q282 448 320 430T378 380T406 316T415 245Q415 238 408 231H126V216Q126 68 226 36Q246 30 270 30Q312 30 342 62Q359 79 369 104L379 128Q382 131 395 131H398Q415 131 415 121Q415 117 412 108Q393 53 349 21T250 -11Q155 -11 92 58T28 218ZM333 275Q322 403 238 411H236Q228 411 220 410T195 402T166 381T143 340T127 274V267H333V275",
      102: "273 0Q255 3 146 3Q43 3 34 0H26V46H42Q70 46 91 49Q99 52 103 60Q104 62 104 224V385H33V431H104V497L105 564L107 574Q126 639 171 668T266 704Q267 704 275 704T289 705Q330 702 351 679T372 627Q372 604 358 590T321 576T284 590T270 627Q270 647 288 667H284Q280 668 273 668Q245 668 223 647T189 592Q183 572 182 497V431H293V385H185V225Q185 63 186 61T189 57T194 54T199 51T206 49T213 48T222 47T231 47T241 46T251 46H282V0H273",
      103: "329 409Q373 453 429 453Q459 453 472 434T485 396Q485 382 476 371T449 360Q416 360 412 390Q410 404 415 411Q415 412 416 414V415Q388 412 363 393Q355 388 355 386Q355 385 359 381T368 369T379 351T388 325T392 292Q392 230 343 187T222 143Q172 143 123 171Q112 153 112 133Q112 98 138 81Q147 75 155 75T227 73Q311 72 335 67Q396 58 431 26Q470 -13 470 -72Q470 -139 392 -175Q332 -206 250 -206Q167 -206 107 -175Q29 -140 29 -75Q29 -39 50 -15T92 18L103 24Q67 55 67 108Q67 155 96 193Q52 237 52 292Q52 355 102 398T223 442Q274 442 318 416L329 409ZM299 343Q294 371 273 387T221 404Q192 404 171 388T145 343Q142 326 142 292Q142 248 149 227T179 192Q196 182 222 182Q244 182 260 189T283 207T294 227T299 242Q302 258 302 292T299 343ZM403 -75Q403 -50 389 -34T348 -11T299 -2T245 0H218Q151 0 138 -6Q118 -15 107 -34T95 -74Q95 -84 101 -97T122 -127T170 -155T250 -167Q319 -167 361 -139T403 -75",
      104: "41 46H55Q94 46 102 60V68Q102 77 102 91T102 124T102 167T103 217T103 272T103 329Q103 366 103 407T103 482T102 542T102 586T102 603Q99 622 88 628T43 637H25V660Q25 683 27 683L37 684Q47 685 66 686T103 688Q120 689 140 690T170 693T181 694H184V367Q244 442 328 442Q451 442 463 329Q464 322 464 190V104Q464 66 466 59T477 49Q498 46 526 46H542V0H534L510 1Q487 2 460 2T422 3Q319 3 310 0H302V46H318Q379 46 379 62Q380 64 380 200Q379 335 378 343Q372 371 358 385T334 402T308 404Q263 404 229 370Q202 343 195 315T187 232V168V108Q187 78 188 68T191 55T200 49Q221 46 249 46H265V0H257L234 1Q210 2 183 2T145 3Q42 3 33 0H25V46H41",
      105: "69 609Q69 637 87 653T131 669Q154 667 171 652T188 609Q188 579 171 564T129 549Q104 549 87 564T69 609ZM247 0Q232 3 143 3Q132 3 106 3T56 1L34 0H26V46H42Q70 46 91 49Q100 53 102 60T104 102V205V293Q104 345 102 359T88 378Q74 385 41 385H30V408Q30 431 32 431L42 432Q52 433 70 434T106 436Q123 437 142 438T171 441T182 442H185V62Q190 52 197 50T232 46H255V0H247",
      106: "98 609Q98 637 116 653T160 669Q183 667 200 652T217 609Q217 579 200 564T158 549Q133 549 116 564T98 609ZM28 -163Q58 -168 64 -168Q124 -168 135 -77Q137 -65 137 141T136 353Q132 371 120 377T72 385H52V408Q52 431 54 431L58 432Q62 432 70 432T87 433T108 434T133 436Q151 437 171 438T202 441T214 442H218V184Q217 -36 217 -59T211 -98Q195 -145 153 -175T58 -205Q9 -205 -23 -179T-55 -117Q-55 -94 -40 -79T-2 -64T36 -79T52 -118Q52 -143 28 -163",
      107: "36 46H50Q89 46 97 60V68Q97 77 97 91T97 124T98 167T98 217T98 272T98 329Q98 366 98 407T98 482T98 542T97 586T97 603Q94 622 83 628T38 637H20V660Q20 683 22 683L32 684Q42 685 61 686T98 688Q115 689 135 690T165 693T176 694H179V463L180 233L240 287Q300 341 304 347Q310 356 310 364Q310 383 289 385H284V431H293Q308 428 412 428Q475 428 484 431H489V385H476Q407 380 360 341Q286 278 286 274Q286 273 349 181T420 79Q434 60 451 53T500 46H511V0H505Q496 3 418 3Q322 3 307 0H299V46H306Q330 48 330 65Q330 72 326 79Q323 84 276 153T228 222L176 176V120V84Q176 65 178 59T189 49Q210 46 238 46H254V0H246Q231 3 137 3T28 0H20V46H36",
      108: "42 46H56Q95 46 103 60V68Q103 77 103 91T103 124T104 167T104 217T104 272T104 329Q104 366 104 407T104 482T104 542T103 586T103 603Q100 622 89 628T44 637H26V660Q26 683 28 683L38 684Q48 685 67 686T104 688Q121 689 141 690T171 693T182 694H185V379Q185 62 186 60Q190 52 198 49Q219 46 247 46H263V0H255L232 1Q209 2 183 2T145 3T107 3T57 1L34 0H26V46H42",
      109: "41 46H55Q94 46 102 60V68Q102 77 102 91T102 122T103 161T103 203Q103 234 103 269T102 328V351Q99 370 88 376T43 385H25V408Q25 431 27 431L37 432Q47 433 65 434T102 436Q119 437 138 438T167 441T178 442H181V402Q181 364 182 364T187 369T199 384T218 402T247 421T285 437Q305 442 336 442Q351 442 364 440T387 434T406 426T421 417T432 406T441 395T448 384T452 374T455 366L457 361L460 365Q463 369 466 373T475 384T488 397T503 410T523 422T546 432T572 439T603 442Q729 442 740 329Q741 322 741 190V104Q741 66 743 59T754 49Q775 46 803 46H819V0H811L788 1Q764 2 737 2T699 3Q596 3 587 0H579V46H595Q656 46 656 62Q657 64 657 200Q656 335 655 343Q649 371 635 385T611 402T585 404Q540 404 506 370Q479 343 472 315T464 232V168V108Q464 78 465 68T468 55T477 49Q498 46 526 46H542V0H534L510 1Q487 2 460 2T422 3Q319 3 310 0H302V46H318Q379 46 379 62Q380 64 380 200Q379 335 378 343Q372 371 358 385T334 402T308 404Q263 404 229 370Q202 343 195 315T187 232V168V108Q187 78 188 68T191 55T200 49Q221 46 249 46H265V0H257L234 1Q210 2 183 2T145 3Q42 3 33 0H25V46H41",
      110: "41 46H55Q94 46 102 60V68Q102 77 102 91T102 122T103 161T103 203Q103 234 103 269T102 328V351Q99 370 88 376T43 385H25V408Q25 431 27 431L37 432Q47 433 65 434T102 436Q119 437 138 438T167 441T178 442H181V402Q181 364 182 364T187 369T199 384T218 402T247 421T285 437Q305 442 336 442Q450 438 463 329Q464 322 464 190V104Q464 66 466 59T477 49Q498 46 526 46H542V0H534L510 1Q487 2 460 2T422 3Q319 3 310 0H302V46H318Q379 46 379 62Q380 64 380 200Q379 335 378 343Q372 371 358 385T334 402T308 404Q263 404 229 370Q202 343 195 315T187 232V168V108Q187 78 188 68T191 55T200 49Q221 46 249 46H265V0H257L234 1Q210 2 183 2T145 3Q42 3 33 0H25V46H41",
      111: "28 214Q28 309 93 378T250 448Q340 448 405 380T471 215Q471 120 407 55T250 -10Q153 -10 91 57T28 214ZM250 30Q372 30 372 193V225V250Q372 272 371 288T364 326T348 362T317 390T268 410Q263 411 252 411Q222 411 195 399Q152 377 139 338T126 246V226Q126 130 145 91Q177 30 250 30",
      112: "36 -148H50Q89 -148 97 -134V-126Q97 -119 97 -107T97 -77T98 -38T98 6T98 55T98 106Q98 140 98 177T98 243T98 296T97 335T97 351Q94 370 83 376T38 385H20V408Q20 431 22 431L32 432Q42 433 61 434T98 436Q115 437 135 438T165 441T176 442H179V416L180 390L188 397Q247 441 326 441Q407 441 464 377T522 216Q522 115 457 52T310 -11Q242 -11 190 33L182 40V-45V-101Q182 -128 184 -134T195 -145Q216 -148 244 -148H260V-194H252L228 -193Q205 -192 178 -192T140 -191Q37 -191 28 -194H20V-148H36ZM424 218Q424 292 390 347T305 402Q234 402 182 337V98Q222 26 294 26Q345 26 384 80T424 218",
      113: "33 218Q33 308 95 374T236 441H246Q330 441 381 372L387 364Q388 364 404 403L420 442H457V156Q457 -132 458 -134Q462 -142 470 -145Q491 -148 519 -148H535V-194H527L504 -193Q480 -192 453 -192T415 -191Q312 -191 303 -194H295V-148H311Q339 -148 360 -145Q369 -141 371 -135T373 -106V-41V49Q313 -11 236 -11Q154 -11 94 53T33 218ZM376 300Q346 389 278 401Q275 401 269 401T261 402Q211 400 171 350T131 214Q131 137 165 82T253 27Q296 27 328 54T376 118V300",
      114: "36 46H50Q89 46 97 60V68Q97 77 97 91T98 122T98 161T98 203Q98 234 98 269T98 328L97 351Q94 370 83 376T38 385H20V408Q20 431 22 431L32 432Q42 433 60 434T96 436Q112 437 131 438T160 441T171 442H174V373Q213 441 271 441H277Q322 441 343 419T364 373Q364 352 351 337T313 322Q288 322 276 338T263 372Q263 381 265 388T270 400T273 405Q271 407 250 401Q234 393 226 386Q179 341 179 207V154Q179 141 179 127T179 101T180 81T180 66V61Q181 59 183 57T188 54T193 51T200 49T207 48T216 47T225 47T235 46T245 46H276V0H267Q249 3 140 3Q37 3 28 0H20V46H36",
      115: "295 316Q295 356 268 385T190 414Q154 414 128 401Q98 382 98 349Q97 344 98 336T114 312T157 287Q175 282 201 278T245 269T277 256Q294 248 310 236T342 195T359 133Q359 71 321 31T198 -10H190Q138 -10 94 26L86 19L77 10Q71 4 65 -1L54 -11H46H42Q39 -11 33 -5V74V132Q33 153 35 157T45 162H54Q66 162 70 158T75 146T82 119T101 77Q136 26 198 26Q295 26 295 104Q295 133 277 151Q257 175 194 187T111 210Q75 227 54 256T33 318Q33 357 50 384T93 424T143 442T187 447H198Q238 447 268 432L283 424L292 431Q302 440 314 448H322H326Q329 448 335 442V310L329 304H301Q295 310 295 316",
      116: "27 422Q80 426 109 478T141 600V615H181V431H316V385H181V241Q182 116 182 100T189 68Q203 29 238 29Q282 29 292 100Q293 108 293 146V181H333V146V134Q333 57 291 17Q264 -10 221 -10Q187 -10 162 2T124 33T105 68T98 100Q97 107 97 248V385H18V422H27",
      117: "383 58Q327 -10 256 -10H249Q124 -10 105 89Q104 96 103 226Q102 335 102 348T96 369Q86 385 36 385H25V408Q25 431 27 431L38 432Q48 433 67 434T105 436Q122 437 142 438T172 441T184 442H187V261Q188 77 190 64Q193 49 204 40Q224 26 264 26Q290 26 311 35T343 58T363 90T375 120T379 144Q379 145 379 161T380 201T380 248V315Q380 361 370 372T320 385H302V431Q304 431 378 436T457 442H464V264Q464 84 465 81Q468 61 479 55T524 46H542V0Q540 0 467 -5T390 -11H383V58",
      118: "338 431Q344 429 422 429Q479 429 503 431H508V385H497Q439 381 423 345Q421 341 356 172T288 -2Q283 -11 263 -11Q244 -11 239 -2Q99 359 98 364Q93 378 82 381T43 385H19V431H25L33 430Q41 430 53 430T79 430T104 429T122 428Q217 428 232 431H240V385H226Q187 384 184 370Q184 366 235 234L286 102L377 341V349Q377 363 367 372T349 383T335 385H331V431H338",
      119: "90 368Q84 378 76 380T40 385H18V431H24L43 430Q62 430 84 429T116 428Q206 428 221 431H229V385H215Q177 383 177 368Q177 367 221 239L265 113L339 328L333 345Q323 374 316 379Q308 384 278 385H258V431H264Q270 428 348 428Q439 428 454 431H461V385H452Q404 385 404 369Q404 366 418 324T449 234T481 143L496 100L537 219Q579 341 579 347Q579 363 564 373T530 385H522V431H529Q541 428 624 428Q692 428 698 431H703V385H697Q696 385 691 385T682 384Q635 377 619 334L559 161Q546 124 528 71Q508 12 503 1T487 -11H479Q460 -11 456 -4Q455 -3 407 133L361 267Q359 263 266 -4Q261 -11 243 -11H238Q225 -11 220 -3L90 368",
      120: "201 0Q189 3 102 3Q26 3 17 0H11V46H25Q48 47 67 52T96 61T121 78T139 96T160 122T180 150L226 210L168 288Q159 301 149 315T133 336T122 351T113 363T107 370T100 376T94 379T88 381T80 383Q74 383 44 385H16V431H23Q59 429 126 429Q219 429 229 431H237V385Q201 381 201 369Q201 367 211 353T239 315T268 274L272 270L297 304Q329 345 329 358Q329 364 327 369T322 376T317 380T310 384L307 385H302V431H309Q324 428 408 428Q487 428 493 431H499V385H492Q443 385 411 368Q394 360 377 341T312 257L296 236L358 151Q424 61 429 57T446 50Q464 46 499 46H516V0H510H502Q494 1 482 1T457 2T432 2T414 3Q403 3 377 3T327 1L304 0H295V46H298Q309 46 320 51T331 63Q331 65 291 120L250 175Q249 174 219 133T185 88Q181 83 181 74Q181 63 188 55T206 46Q208 46 208 23V0H201",
      121: "69 -66Q91 -66 104 -80T118 -116Q118 -134 109 -145T91 -160Q84 -163 97 -166Q104 -168 111 -168Q131 -168 148 -159T175 -138T197 -106T213 -75T225 -43L242 0L170 183Q150 233 125 297Q101 358 96 368T80 381Q79 382 78 382Q66 385 34 385H19V431H26L46 430Q65 430 88 429T122 428Q129 428 142 428T171 429T200 430T224 430L233 431H241V385H232Q183 385 185 366L286 112Q286 113 332 227L376 341V350Q376 365 366 373T348 383T334 385H331V431H337H344Q351 431 361 431T382 430T405 429T422 429Q477 429 503 431H508V385H497Q441 380 422 345Q420 343 378 235T289 9T227 -131Q180 -204 113 -204Q69 -204 44 -177T19 -116Q19 -89 35 -78T69 -66",
      122: "42 263Q44 270 48 345T53 423V431H393Q399 425 399 415Q399 403 398 402L381 378Q364 355 331 309T265 220L134 41L182 40H206Q254 40 283 46T331 77Q352 105 359 185L361 201Q361 202 381 202H401V196Q401 195 393 103T384 6V0H209L34 1L31 3Q28 8 28 17Q28 30 29 31T160 210T294 394H236Q169 393 152 388Q127 382 113 367Q89 344 82 264V255H42V263",
      123: "434 -231Q434 -244 428 -250H410Q281 -250 230 -184Q225 -177 222 -172T217 -161T213 -148T211 -133T210 -111T209 -84T209 -47T209 0Q209 21 209 53Q208 142 204 153Q203 154 203 155Q189 191 153 211T82 231Q71 231 68 234T65 250T68 266T82 269Q116 269 152 289T203 345Q208 356 208 377T209 529V579Q209 634 215 656T244 698Q270 724 324 740Q361 748 377 749Q379 749 390 749T408 750H428Q434 744 434 732Q434 719 431 716Q429 713 415 713Q362 710 332 689T296 647Q291 634 291 499V417Q291 370 288 353T271 314Q240 271 184 255L170 250L184 245Q202 239 220 230T262 196T290 137Q291 131 291 1Q291 -134 296 -147Q306 -174 339 -192T415 -213Q429 -213 431 -216Q434 -219 434 -231",
      124: "139 -249H137Q125 -249 119 -235V251L120 737Q130 750 139 750Q152 750 159 735V-235Q151 -249 141 -249H139",
      125: "65 731Q65 745 68 747T88 750Q171 750 216 725T279 670Q288 649 289 635T291 501Q292 362 293 357Q306 312 345 291T417 269Q428 269 431 266T434 250T431 234T417 231Q380 231 345 210T298 157Q293 143 292 121T291 -28V-79Q291 -134 285 -156T256 -198Q202 -250 89 -250Q71 -250 68 -247T65 -230Q65 -224 65 -223T66 -218T69 -214T77 -213Q91 -213 108 -210T146 -200T183 -177T207 -139Q208 -134 209 3L210 139Q223 196 280 230Q315 247 330 250Q305 257 280 270Q225 304 212 352L210 362L209 498Q208 635 207 640Q195 680 154 696T77 713Q68 713 67 716T65 731",
      126: "179 251Q164 251 151 245T131 234T111 215L97 227L83 238Q83 239 95 253T121 283T142 304Q165 318 187 318T253 300T320 282Q335 282 348 288T368 299T388 318L402 306L416 295Q375 236 344 222Q330 215 313 215Q292 215 248 233T179 251",
      160: "",
      163: "699 578Q699 473 635 473Q597 473 595 508Q595 559 654 569V576Q654 619 637 648T581 677Q545 677 513 647T463 561Q460 554 437 464T414 371Q414 370 458 370H502Q508 364 508 362Q505 334 495 324H402L382 241Q377 224 373 206T366 180T361 163T358 151T354 142T350 133T344 120Q340 112 338 107T336 101L354 90Q398 63 422 54T476 44Q515 44 539 73T574 133Q578 144 580 146T598 148Q622 148 622 139Q622 138 620 130Q602 74 555 32T447 -11Q395 -11 317 38L294 51Q271 28 233 9T155 -10Q117 -10 103 5T88 39Q88 73 126 106T224 139Q236 139 247 138T266 134L273 132Q275 132 302 239L323 324H259Q253 330 253 332Q253 350 265 370H300L334 371L355 453Q356 457 360 477T366 501T372 522T379 545T387 565T397 587T409 606T425 627Q453 664 497 689T583 714Q640 714 669 676T699 578ZM245 76Q211 85 195 85Q173 85 158 71T142 42Q142 26 160 26H163Q211 30 245 76",
      165: "515 0Q494 3 374 3Q256 3 235 0H224V46H257Q316 47 324 58Q327 62 327 137V213H133Q121 213 113 213T97 213T86 213T78 213T73 214T70 215T69 216T68 218T67 220Q64 225 66 231T73 240Q76 242 202 242H327V273L247 407H115Q81 407 75 408T67 414Q64 419 66 425T73 434Q76 436 153 436Q228 436 228 437Q227 440 173 530T115 623Q101 637 31 637H11V683H20Q66 681 153 681Q169 681 202 681T262 682L288 683H298V637H280Q230 636 230 621Q230 619 250 584Q255 576 264 561T286 526T305 494L340 437L403 436H467L513 514Q564 596 564 605Q564 608 560 616Q550 634 517 637H508V683H516Q531 680 633 680Q722 680 731 683H738V637H723Q644 632 617 595Q614 591 568 515T521 437T597 436T676 434Q681 432 683 426T682 414T671 409T589 407H503L422 273V242H547Q673 242 676 240Q681 238 683 232T682 220Q682 219 682 218T681 217T679 216T677 215T672 214T664 213T652 213T637 213T616 213H422V139V87Q422 64 425 58T441 49Q456 46 503 46H525V0H515ZM449 406Q449 407 403 407Q358 407 358 406L370 387Q381 368 392 350L404 331Q447 404 449 406",
      168: "95 612Q95 633 112 651T153 669T193 652T210 612Q210 588 194 571T152 554L127 560Q95 577 95 612ZM289 611Q289 634 304 649T335 668Q336 668 340 668T346 669Q369 669 386 652T404 612T387 572T346 554Q323 554 306 570T289 611",
      172: "56 323T56 336T70 356H596Q603 353 611 343V102Q598 89 591 89Q587 89 584 90T579 94T575 98T572 102L571 209V316H70Q56 323 56 336",
      174: "915 266Q915 140 852 38T689 -120T474 -175Q312 -175 188 -71T38 190Q32 220 32 266V287Q32 345 57 416T129 545Q192 624 282 666T464 709Q513 709 522 708Q599 698 665 666T776 590T853 493T900 387T915 287V266ZM875 285Q875 339 853 399T789 517T676 616T519 668Q510 669 465 669Q380 669 299 630T155 514T77 336Q72 312 72 285V266V256Q72 123 163 11Q290 -135 474 -135Q614 -135 727 -46Q875 81 875 266V285ZM276 457Q275 458 274 460T272 463T270 465T267 467T264 469T258 471T252 472T243 473T232 474T218 474H204V514H335Q477 514 499 510Q560 502 610 467T661 375Q661 362 658 350T648 327T635 308T618 292T601 280T583 269T568 262T554 256L547 253Q548 252 556 247T570 237T586 223T602 202T614 174Q616 169 626 123T638 72Q652 23 683 23Q715 23 720 68Q721 78 724 81T740 84T756 82T760 70Q760 47 747 25T715 -7Q700 -14 673 -14Q672 -14 662 -14T643 -12T619 -7T593 2T568 16T547 37T534 67Q531 80 531 97Q531 103 531 116T532 136Q532 218 472 236Q466 238 413 239H360V148L361 58Q366 47 375 44T418 40H432V0H424Q409 3 318 3T212 0H204V40H218Q242 40 253 42T268 47T276 58V457ZM376 473Q365 471 363 464T360 430V366V276H416Q421 276 434 276T453 276T469 277T486 279T501 282T517 287T529 294T542 305Q561 324 561 375Q561 424 545 444T482 472Q478 473 427 474Q415 474 403 474T384 474L376 473",
      175: "69 544V590H430V544H69",
      176: "147 628Q147 669 179 692T244 715Q298 715 325 689T352 629Q352 592 323 567T249 542Q202 542 175 567T147 628ZM313 628Q313 660 300 669T259 678H253Q248 678 242 678T234 679Q217 679 207 674T192 659T188 644T187 629Q187 600 198 590Q210 579 250 579H265Q279 579 288 581T305 595T313 628",
      177: "56 320T56 333T70 353H369V502Q369 651 371 655Q376 666 388 666Q402 666 405 654T409 596V500V353H707Q722 345 722 333Q722 320 707 313H409V40H707Q722 32 722 20T707 0H70Q56 7 56 20T70 40H369V313H70Q56 320 56 333",
      180: "349 699Q367 699 380 686T393 656Q393 651 392 647T387 637T380 627T367 616T351 602T330 585T303 563L232 505L217 519Q203 533 204 533Q204 534 229 567T282 636T313 678L316 681Q318 684 321 686T328 692T337 697T349 699",
      183: "78 250Q78 274 95 292T138 310Q162 310 180 294T199 251Q199 226 182 208T139 190T96 207T78 250",
      215: "630 29Q630 9 609 9Q604 9 587 25T493 118L389 222L284 117Q178 13 175 11Q171 9 168 9Q160 9 154 15T147 29Q147 36 161 51T255 146L359 250L255 354Q174 435 161 449T147 471Q147 480 153 485T168 490Q173 490 175 489Q178 487 284 383L389 278L493 382Q570 459 587 475T609 491Q630 491 630 471Q630 464 620 453T522 355L418 250L522 145Q606 61 618 48T630 29",
      240: "75 566V604Q75 624 79 629T102 635Q124 635 127 629T131 588L133 550L191 588L249 628L231 635Q176 654 124 657Q116 657 106 658L95 659Q94 661 94 687T95 715Q99 717 113 717Q195 717 282 679L309 668L331 681Q351 697 391 721Q428 748 435 748Q437 749 446 749Q470 749 473 746Q478 744 478 681V621Q466 615 456 615Q435 615 424 624L422 661V699L382 675L344 648Q353 639 366 630Q480 538 504 413Q509 393 509 333V313Q509 284 507 257T495 184T466 102T413 33T329 -16Q311 -21 275 -21Q226 -21 195 -10Q150 7 110 50T53 141Q42 179 42 227Q42 332 101 403T245 474Q282 474 314 461T359 436T380 415Q386 405 389 408Q389 426 378 475Q368 505 355 529T329 567T306 590T288 603L282 606L120 501Q116 500 102 500Q84 500 75 506V566ZM388 225Q388 376 309 410Q299 416 273 419Q216 419 191 390Q174 371 168 342T162 218Q162 112 184 79Q212 39 273 39Q312 39 342 62T380 121Q388 159 388 225",
      247: "318 466Q318 500 339 518T386 537Q418 537 438 517T458 466Q458 438 440 417T388 396Q355 396 337 417T318 466ZM56 237T56 250T70 270H706Q721 262 721 250T706 230H70Q56 237 56 250ZM318 34Q318 68 339 86T386 105Q418 105 438 85T458 34Q458 6 440 -15T388 -36Q355 -36 337 -15T318 34",
      305: "247 0Q232 3 143 3Q132 3 106 3T56 1L34 0H26V46H42Q70 46 91 49Q100 53 102 60T104 102V205V293Q104 345 102 359T88 378Q74 385 41 385H30V408Q30 431 32 431L42 432Q52 433 70 434T106 436Q123 437 142 438T171 441T182 442H185V62Q190 52 197 50T232 46H255V0H247",
      567: "28 -163Q58 -168 64 -168Q124 -168 135 -77Q137 -65 137 141T136 353Q132 371 120 377T72 385H52V408Q52 431 54 431L58 432Q62 432 70 432T87 433T108 434T133 436Q151 437 171 438T202 441T214 442H218V184Q217 -36 217 -59T211 -98Q195 -145 153 -175T58 -205Q9 -205 -23 -179T-55 -117Q-55 -94 -40 -79T-2 -64T36 -79T52 -118Q52 -143 28 -163",
      697: "79 43Q73 43 52 49T30 61Q30 68 85 293T146 528Q161 560 198 560Q218 560 240 545T262 501Q262 496 260 486Q259 479 173 263T84 45T79 43",
      710: "112 560L249 694L257 686Q387 562 387 560L361 531Q359 532 303 581L250 627L195 580Q182 569 169 557T148 538L140 532Q138 530 125 546L112 560",
      711: "114 611L127 630L136 644Q138 644 193 612Q248 581 250 581L306 612Q361 644 363 644L385 611L318 562L249 513L114 611",
      713: "69 544V590H430V544H69",
      714: "349 699Q367 699 380 686T393 656Q393 651 392 647T387 637T380 627T367 616T351 602T330 585T303 563L232 505L217 519Q203 533 204 533Q204 534 229 567T282 636T313 678L316 681Q318 684 321 686T328 692T337 697T349 699",
      715: "106 655Q106 671 119 685T150 699Q166 699 177 688Q190 671 222 629T275 561T295 533T282 519L267 505L196 563Q119 626 113 634Q106 643 106 655",
      728: "250 515Q179 515 138 565T92 683V694H129V689Q129 688 129 683T130 675Q137 631 169 599T248 567Q304 567 337 608T370 689V694H407V683Q403 617 361 566T250 515",
      729: "190 609Q190 637 208 653T252 669Q275 667 292 652T309 609Q309 579 292 564T250 549Q225 549 208 564T190 609",
      730: "147 628Q147 669 179 692T244 715Q298 715 325 689T352 629Q352 592 323 567T249 542Q202 542 175 567T147 628ZM313 628Q313 660 300 669T259 678H253Q248 678 242 678T234 679Q217 679 207 674T192 659T188 644T187 629Q187 600 198 590Q210 579 250 579H265Q279 579 288 581T305 595T313 628",
      732: "179 601Q164 601 151 595T131 584T111 565L97 577L83 588Q83 589 95 603T121 633T142 654Q165 668 187 668T253 650T320 632Q335 632 348 638T368 649T388 668L402 656L416 645Q375 586 344 572Q330 565 313 565Q292 565 248 583T179 601",
      768: "-394 655Q-394 671 -381 685T-350 699Q-334 699 -323 688Q-310 671 -278 629T-225 561T-205 533T-218 519L-233 505L-304 563Q-381 626 -387 634Q-394 643 -394 655",
      769: "-151 699Q-133 699 -120 686T-107 656Q-107 651 -108 647T-113 637T-120 627T-133 616T-149 602T-170 585T-197 563L-268 505L-283 519Q-297 533 -296 533Q-296 534 -271 567T-218 636T-187 678L-184 681Q-182 684 -179 686T-172 692T-163 697T-151 699",
      770: "-388 560L-251 694L-243 686Q-113 562 -113 560L-139 531Q-141 532 -197 581L-250 627L-305 580Q-318 569 -331 557T-352 538L-360 532Q-362 530 -375 546L-388 560",
      771: "-321 601Q-336 601 -349 595T-369 584T-389 565L-403 577L-417 588Q-417 589 -405 603T-379 633T-358 654Q-335 668 -313 668T-247 650T-180 632Q-165 632 -152 638T-132 649T-112 668L-98 656L-84 645Q-125 586 -156 572Q-170 565 -187 565Q-208 565 -252 583T-321 601",
      772: "-431 544V590H-70V544H-431",
      774: "-250 515Q-321 515 -362 565T-408 683V694H-371V689Q-371 688 -371 683T-370 675Q-363 631 -331 599T-252 567Q-196 567 -163 608T-130 689V694H-93V683Q-97 617 -139 566T-250 515",
      775: "-310 609Q-310 637 -292 653T-248 669Q-225 667 -208 652T-191 609Q-191 579 -208 564T-250 549Q-275 549 -292 564T-310 609",
      776: "-405 612Q-405 633 -388 651T-347 669T-307 652T-290 612Q-290 588 -306 571T-348 554L-373 560Q-405 577 -405 612ZM-211 611Q-211 634 -196 649T-165 668Q-164 668 -160 668T-154 669Q-131 669 -114 652T-96 612T-113 572T-154 554Q-177 554 -194 570T-211 611",
      778: "-353 628Q-353 669 -321 692T-256 715Q-202 715 -175 689T-148 629Q-148 592 -177 567T-251 542Q-298 542 -325 567T-353 628ZM-187 628Q-187 660 -200 669T-241 678H-247Q-252 678 -258 678T-266 679Q-283 679 -293 674T-308 659T-312 644T-313 629Q-313 600 -302 590Q-290 579 -250 579H-235Q-221 579 -212 581T-195 595T-187 628",
      779: "-292 701Q-278 701 -262 690T-246 658Q-246 649 -250 641Q-252 637 -297 574T-344 510L-378 528Q-378 530 -355 598T-327 676Q-316 701 -292 701ZM-126 701Q-112 701 -96 690T-80 658Q-80 649 -84 641Q-86 637 -131 574T-178 510L-212 528Q-212 530 -189 598T-161 676Q-150 701 -126 701",
      780: "-386 611L-373 630L-364 644Q-362 644 -307 612Q-252 581 -250 581L-194 612Q-139 644 -137 644L-115 611L-182 562L-251 513L-386 611",
      824: "-612 -215T-619 -215T-631 -212T-637 -204T-639 -197Q-639 -190 -634 -183Q-621 -157 -400 274T-176 707Q-173 716 -160 716Q-153 716 -148 712T-142 703T-140 696Q-140 691 -372 241T-608 -212Q-612 -215 -619 -215",
      913: "255 0Q240 3 140 3Q48 3 39 0H32V46H47Q119 49 139 88Q140 91 192 245T295 553T348 708Q351 716 366 716H376Q396 715 400 709Q402 707 508 390L617 67Q624 54 636 51T687 46H717V0H708Q699 3 581 3Q458 3 437 0H427V46H440Q510 46 510 64Q510 66 486 138L462 209H229L209 150Q189 91 189 85Q189 72 209 59T259 46H264V0H255ZM447 255L345 557L244 256Q244 255 345 255H447",
      914: "131 622Q124 629 120 631T104 634T61 637H28V683H229H267H346Q423 683 459 678T531 651Q574 627 599 590T624 512Q624 461 583 419T476 360L466 357Q539 348 595 302T651 187Q651 119 600 67T469 3Q456 1 242 0H28V46H61Q103 47 112 49T131 61V622ZM511 513Q511 560 485 594T416 636Q415 636 403 636T371 636T333 637Q266 637 251 636T232 628Q229 624 229 499V374H312L396 375L406 377Q410 378 417 380T442 393T474 417T499 456T511 513ZM537 188Q537 239 509 282T430 336L329 337H229V200V116Q229 57 234 52Q240 47 334 47H383Q425 47 443 53Q486 67 511 104T537 188",
      915: "128 619Q121 626 117 628T101 631T58 634H25V680H554V676Q556 670 568 560T582 444V440H542V444Q542 445 538 478T523 545T492 598Q454 634 349 634H334Q264 634 249 633T233 621Q232 618 232 339L233 61Q240 54 245 52T270 48T333 46H360V0H348Q324 3 182 3Q51 3 36 0H25V46H58Q100 47 109 49T128 61V619",
      916: "51 0Q46 4 46 7Q46 9 215 357T388 709Q391 716 416 716Q439 716 444 709Q447 705 616 357T786 7Q786 4 781 0H51ZM507 344L384 596L137 92L383 91H630Q630 93 507 344",
      917: "128 619Q121 626 117 628T101 631T58 634H25V680H597V676Q599 670 611 560T625 444V440H585V444Q584 447 582 465Q578 500 570 526T553 571T528 601T498 619T457 629T411 633T353 634Q266 634 251 633T233 622Q233 622 233 621Q232 619 232 497V376H286Q359 378 377 385Q413 401 416 469Q416 471 416 473V493H456V213H416V233Q415 268 408 288T383 317T349 328T297 330Q290 330 286 330H232V196V114Q232 57 237 52Q243 47 289 47H340H391Q428 47 452 50T505 62T552 92T584 146Q594 172 599 200T607 247T612 270V273H652V270Q651 267 632 137T610 3V0H25V46H58Q100 47 109 49T128 61V619",
      918: "69 443Q69 452 74 554T80 683H549Q555 677 555 664Q555 649 554 648Q552 645 366 348T179 50T192 49T263 49H275H302Q333 49 353 50T401 59T447 78T482 115T507 173Q513 200 520 273V282H560V274Q560 272 552 143T543 8V0H302L61 1L58 3Q55 8 55 21V35Q59 43 153 193T340 489T432 637H343Q259 637 214 625T141 573Q109 523 109 445Q109 443 89 443H69",
      919: "128 622Q121 629 117 631T101 634T58 637H25V683H36Q57 680 180 680Q315 680 324 683H335V637H302Q262 636 251 634T233 622L232 500V378H517V622Q510 629 506 631T490 634T447 637H414V683H425Q446 680 569 680Q704 680 713 683H724V637H691Q651 636 640 634T622 622V61Q628 51 639 49T691 46H724V0H713Q692 3 569 3Q434 3 425 0H414V46H447Q489 47 498 49T517 61V332H232V197L233 61Q239 51 250 49T302 46H335V0H324Q303 3 180 3Q45 3 36 0H25V46H58Q100 47 109 49T128 61V622",
      920: "56 340Q56 423 86 494T164 610T270 680T388 705Q521 705 621 601T722 341Q722 260 693 191T617 75T510 4T388 -22T267 3T160 74T85 189T56 340ZM610 339Q610 428 590 495T535 598T463 651T384 668Q332 668 289 638T221 566Q168 485 168 339Q168 274 176 235Q189 158 228 105T324 28Q356 16 388 16Q415 16 442 24T501 54T555 111T594 205T610 339ZM223 263V422H263V388H514V422H554V263H514V297H263V263H223",
      921: "328 0Q307 3 180 3T32 0H21V46H43Q92 46 106 49T126 60Q128 63 128 342Q128 620 126 623Q122 628 118 630T96 635T43 637H21V683H32Q53 680 180 680T328 683H339V637H317Q268 637 254 634T234 623Q232 620 232 342Q232 63 234 60Q238 55 242 53T264 48T317 46H339V0H328",
      922: "128 622Q121 629 117 631T101 634T58 637H25V683H36Q57 680 180 680Q315 680 324 683H335V637H313Q235 637 233 620Q232 618 232 462L233 307L379 449Q425 494 479 546Q518 584 524 591T531 607V608Q531 630 503 636Q501 636 498 636T493 637H489V683H499Q517 680 630 680Q704 680 716 683H722V637H708Q633 633 589 597Q584 592 495 506T406 419T515 254T631 80Q644 60 662 54T715 46H736V0H728Q719 3 615 3Q493 3 472 0H461V46H469Q515 46 515 72Q515 78 512 84L336 351Q332 348 278 296L232 251V156Q232 62 235 58Q243 47 302 46H335V0H324Q303 3 180 3Q45 3 36 0H25V46H58Q100 47 109 49T128 61V622",
      923: "320 708Q326 716 340 716H348H355Q367 716 372 708Q374 706 423 547T523 226T575 62Q581 52 591 50T634 46H661V0H653Q644 3 532 3Q411 3 390 0H379V46H392Q464 46 464 65Q463 70 390 305T316 539L246 316Q177 95 177 84Q177 72 198 59T248 46H253V0H245Q230 3 130 3Q47 3 38 0H32V46H45Q112 51 127 91Q128 92 224 399T320 708",
      924: "132 622Q125 629 121 631T105 634T62 637H29V683H135Q221 683 232 682T249 675Q250 674 354 398L458 124L562 398Q666 674 668 675Q671 681 683 682T781 683H887V637H854Q814 636 803 634T785 622V61Q791 51 802 49T854 46H887V0H876Q855 3 736 3Q605 3 596 0H585V46H618Q660 47 669 49T688 61V347Q688 424 688 461T688 546T688 613L687 632Q454 14 450 7Q446 1 430 1T410 7Q409 9 292 316L176 624V606Q175 588 175 543T175 463T175 356L176 86Q187 50 261 46H278V0H269Q254 3 154 3Q52 3 37 0H29V46H46Q78 48 98 56T122 69T132 86V622",
      925: "42 46Q74 48 94 56T118 69T128 86V634H124Q114 637 52 637H25V683H232L235 680Q237 679 322 554T493 303L578 178V598Q572 608 568 613T544 627T492 637H475V683H483Q498 680 600 680Q706 680 715 683H724V637H707Q634 633 622 598L621 302V6L614 0H600Q585 0 582 3T481 150T282 443T171 605V345L172 86Q183 50 257 46H274V0H265Q250 3 150 3Q48 3 33 0H25V46H42",
      926: "47 509L55 676Q55 677 333 677T611 676L619 509Q619 508 599 508T579 510Q579 529 575 557T564 589Q550 594 333 594T102 589Q95 586 91 558T87 510Q87 508 67 508T47 509ZM139 260V445H179V394H487V445H527V260H487V311H179V260H139ZM50 0L42 180H62Q82 180 82 178Q82 133 89 105Q92 93 95 90T108 86Q137 83 333 83Q530 83 558 86Q568 87 571 90T577 105Q584 133 584 178Q584 180 604 180H624L616 0H50",
      927: "56 340Q56 423 86 494T164 610T270 680T388 705Q521 705 621 601T722 341Q722 260 693 191T617 75T510 4T388 -22T267 3T160 74T85 189T56 340ZM467 647Q426 665 388 665Q360 665 331 654T269 620T213 549T179 439Q174 411 174 354Q174 144 277 61Q327 20 385 20H389H391Q474 20 537 99Q603 188 603 354Q603 411 598 439Q577 592 467 647",
      928: "128 619Q121 626 117 628T101 631T58 634H25V680H724V634H691Q651 633 640 631T622 619V61Q628 51 639 49T691 46H724V0H713Q692 3 569 3Q434 3 425 0H414V46H447Q489 47 498 49T517 61V634H232V348L233 61Q239 51 250 49T302 46H335V0H324Q303 3 180 3Q45 3 36 0H25V46H58Q100 47 109 49T128 61V619",
      929: "130 622Q123 629 119 631T103 634T60 637H27V683H214Q237 683 276 683T331 684Q419 684 471 671T567 616Q624 563 624 489Q624 421 573 372T451 307Q429 302 328 301H234V181Q234 62 237 58Q245 47 304 46H337V0H326Q305 3 182 3Q47 3 38 0H27V46H60Q102 47 111 49T130 61V622ZM507 488Q507 514 506 528T500 564T483 597T450 620T397 635Q385 637 307 637H286Q237 637 234 628Q231 624 231 483V342H302H339Q390 342 423 349T481 382Q507 411 507 488",
      931: "666 247Q664 244 652 126T638 4V0H351Q131 0 95 0T57 5V6Q54 12 57 17L73 36Q89 54 121 90T182 159L305 299L56 644L55 658Q55 677 60 681Q63 683 351 683H638V679Q640 674 652 564T666 447V443H626V447Q618 505 604 543T559 605Q529 626 478 631T333 637H294H189L293 494Q314 465 345 422Q400 346 400 340Q400 338 399 337L154 57Q407 57 428 58Q476 60 508 68T551 83T575 103Q595 125 608 162T624 225L626 251H666V247",
      932: "36 443Q37 448 46 558T55 671V677H666V671Q667 666 676 556T685 443V437H645V443Q645 445 642 478T631 544T610 593Q593 614 555 625Q534 630 478 630H451H443Q417 630 414 618Q413 616 413 339V63Q420 53 439 50T528 46H558V0H545L361 3Q186 1 177 0H164V46H194Q264 46 283 49T309 63V339V550Q309 620 304 625T271 630H244H224Q154 630 119 601Q101 585 93 554T81 486T76 443V437H36V443",
      933: "55 551Q55 604 91 654T194 705Q240 705 277 681T334 624T367 556T385 498L389 474L392 488Q394 501 400 521T414 566T438 615T473 659T521 692T584 705Q620 705 648 689T691 647T714 597T722 551Q722 540 719 538T699 536Q680 536 677 541Q677 542 677 544T676 548Q676 576 650 596T588 616H582Q538 616 505 582Q466 543 454 477T441 318Q441 301 441 269T442 222V61Q448 55 452 53T478 48T542 46H569V0H557Q533 3 389 3T221 0H209V46H236Q256 46 270 46T295 47T311 48T322 51T328 54T332 57T337 61V209Q337 383 333 415Q313 616 189 616Q154 616 128 597T101 548Q101 540 97 538T78 536Q63 536 59 538T55 551",
      934: "312 622Q310 623 307 625T303 629T297 631T286 634T270 635T246 636T211 637H184V683H196Q220 680 361 680T526 683H538V637H511Q468 637 447 635T422 631T411 622V533L425 531Q525 519 595 466T665 342Q665 301 642 267T583 209T506 172T425 152L411 150V61Q417 55 421 53T447 48T511 46H538V0H526Q502 3 361 3T196 0H184V46H211Q231 46 245 46T270 47T286 48T297 51T303 54T307 57T312 61V150H310Q309 151 289 153T232 166T160 195Q149 201 136 210T103 238T69 284T56 342Q56 414 128 467T294 530Q309 532 310 533H312V622ZM170 342Q170 207 307 188H312V495H309Q301 495 282 491T231 469T186 423Q170 389 170 342ZM415 188Q487 199 519 236T551 342Q551 384 539 414T507 459T470 481T434 491T415 495H410V188H415",
      935: "270 0Q252 3 141 3Q46 3 31 0H23V46H40Q129 50 161 88Q165 94 244 216T324 339Q324 341 235 480T143 622Q133 631 119 634T57 637H37V683H46Q64 680 172 680Q297 680 318 683H329V637H324Q307 637 286 632T263 621Q263 618 322 525T384 431Q385 431 437 511T489 593Q490 595 490 599Q490 611 477 622T436 637H428V683H437Q455 680 566 680Q661 680 676 683H684V637H667Q585 634 551 599Q548 596 478 491Q412 388 412 387Q412 385 514 225T620 62Q628 53 642 50T695 46H726V0H717Q699 3 591 3Q466 3 445 0H434V46H440Q454 46 476 51T499 64Q499 67 463 124T390 238L353 295L350 292Q348 290 343 283T331 265T312 236T286 195Q219 88 218 84Q218 70 234 59T272 46H280V0H270",
      936: "340 622Q338 623 335 625T331 629T325 631T314 634T298 635T274 636T239 637H212V683H224Q248 680 389 680T554 683H566V637H539Q479 637 464 635T439 622L438 407Q438 192 439 192Q443 193 449 195T474 207T507 232T536 276T557 344Q560 365 562 417T573 493Q587 536 620 544Q627 546 671 546H715L722 540V515Q714 509 708 509Q680 505 671 476T658 392T644 307Q599 177 451 153L438 151V106L439 61Q446 54 451 52T476 48T539 46H566V0H554Q530 3 389 3T224 0H212V46H239Q259 46 273 46T298 47T314 48T325 51T331 54T335 57T340 61V151Q126 178 117 406Q115 503 69 509Q55 509 55 526Q55 541 59 543T86 546H107H120Q150 546 161 543T184 528Q198 514 204 493Q212 472 213 420T226 316T272 230Q287 216 303 207T330 194L339 192Q340 192 340 407V622",
      937: "55 454Q55 503 75 546T127 617T197 665T272 695T337 704H352Q396 704 404 703Q527 687 596 615T666 454Q666 392 635 330T559 200T499 83V80H543Q589 81 600 83T617 93Q622 102 629 135T636 172L637 177H677V175L660 89Q645 3 644 2V0H552H488Q461 0 456 3T451 20Q451 89 499 235T548 455Q548 512 530 555T483 622T424 656T361 668Q332 668 303 658T243 626T193 560T174 456Q174 380 222 233T270 20Q270 7 263 0H77V2Q76 3 61 89L44 175V177H84L85 172Q85 171 88 155T96 119T104 93Q109 86 120 84T178 80H222V83Q206 132 162 199T87 329T55 454",
      8192: "",
      8193: "",
      8194: "",
      8195: "",
      8196: "",
      8197: "",
      8198: "",
      8201: "",
      8202: "",
      8203: "",
      8204: "",
      8211: "0 248V285H499V248H0",
      8212: "0 248V285H999V248H0",
      8213: "0 248V285H999V248H0",
      8214: "133 736Q138 750 153 750Q164 750 170 739Q172 735 172 250T170 -239Q164 -250 152 -250Q144 -250 138 -244L137 -243Q133 -241 133 -179T132 250Q132 731 133 736ZM329 739Q334 750 346 750Q353 750 361 744L362 743Q366 741 366 679T367 250T367 -178T362 -243L361 -244Q355 -250 347 -250Q335 -250 329 -239Q327 -235 327 250T329 739",
      8215: "0 -62V-25H499V-62H0",
      8216: "64 494Q64 548 86 597T131 670T160 694Q163 694 172 685T182 672Q182 669 170 656T144 625T116 573T101 501Q101 489 102 489T107 491T120 497T138 500Q163 500 180 483T198 440T181 397T139 379Q110 379 87 405T64 494",
      8217: "78 634Q78 659 95 676T138 694Q166 694 189 668T212 579Q212 525 190 476T146 403T118 379Q114 379 105 388T95 401Q95 404 107 417T133 448T161 500T176 572Q176 584 175 584T170 581T157 576T139 573Q114 573 96 590T78 634",
      8220: "128 494Q128 528 137 560T158 616T185 658T209 685T223 694T236 685T245 670Q244 668 231 654T204 622T178 571T164 501Q164 489 165 489T170 491T183 497T201 500Q226 500 244 483T262 440T245 397T202 379Q173 379 151 405T128 494ZM332 494Q332 528 341 560T362 616T389 658T413 685T427 694T439 685T449 672Q449 669 437 656T411 625T383 573T368 501Q368 489 369 489T374 491T387 497T405 500Q430 500 448 483T466 440T449 397T406 379Q377 379 355 405T332 494",
      8221: "34 634Q34 659 50 676T93 694Q121 694 144 668T168 579Q168 525 146 476T101 403T73 379Q69 379 60 388T50 401Q50 404 62 417T88 448T116 500T131 572Q131 584 130 584T125 581T112 576T94 573Q69 573 52 590T34 634ZM238 634Q238 659 254 676T297 694Q325 694 348 668T372 579Q372 525 350 476T305 403T277 379Q273 379 264 388T254 401Q254 404 266 417T292 448T320 500T335 572Q335 584 334 584T329 581T316 576T298 573Q273 573 256 590T238 634",
      8224: "182 675Q195 705 222 705Q234 705 243 700T253 691T263 675L262 655Q262 620 252 549T240 454V449Q250 451 288 461T346 472T377 461T389 431Q389 417 379 404T346 390Q327 390 288 401T243 412H240V405Q245 367 250 339T258 301T261 274T263 225Q263 124 255 -41T239 -213Q236 -216 222 -216H217Q206 -216 204 -212T200 -186Q199 -175 199 -168Q181 38 181 225Q181 265 182 280T191 327T204 405V412H201Q196 412 157 401T98 390Q76 390 66 403T55 431T65 458T98 472Q116 472 155 462T205 449Q204 452 204 460T201 490T193 547Q182 619 182 655V675",
      8225: "181 658Q181 705 222 705T263 658Q263 633 252 572T240 497Q240 496 241 496Q243 496 285 507T345 519Q365 519 376 508T388 478Q388 466 384 458T375 447T361 438H344Q318 438 282 448T241 459Q240 458 240 456Q240 449 251 384T263 297Q263 278 255 267T238 253T222 250T206 252T190 266T181 297Q181 323 192 383T204 458Q204 459 203 459Q198 459 162 449T101 438H84Q74 443 70 446T61 457T56 478Q56 497 67 508T99 519Q117 519 159 508T203 496Q204 496 204 499Q204 507 193 572T181 658ZM181 202Q181 249 222 249T263 202Q263 185 259 161T249 103T240 48V41H243Q248 41 287 52T346 63T377 52T389 22Q389 8 379 -5T346 -19Q327 -19 288 -8T243 3H240V-4Q243 -24 249 -58T259 -117T263 -158Q263 -177 255 -188T238 -202T222 -205T206 -203T190 -189T181 -158Q181 -141 185 -117T195 -59T204 -4V3H201Q196 3 157 -8T98 -19Q76 -19 66 -6T55 22T65 49T98 63Q117 63 156 52T201 41H204V48Q201 68 195 102T185 161T181 202",
      8226: "55 251Q55 328 112 386T249 444T386 388T444 249Q444 171 388 113T250 55Q170 55 113 112T55 251",
      8230: "78 60Q78 84 95 102T138 120Q162 120 180 104T199 61Q199 36 182 18T139 0T96 17T78 60ZM525 60Q525 84 542 102T585 120Q609 120 627 104T646 61Q646 36 629 18T586 0T543 17T525 60ZM972 60Q972 84 989 102T1032 120Q1056 120 1074 104T1093 61Q1093 36 1076 18T1033 0T990 17T972 60",
      8242: "79 43Q73 43 52 49T30 61Q30 68 85 293T146 528Q161 560 198 560Q218 560 240 545T262 501Q262 496 260 486Q259 479 173 263T84 45T79 43",
      8245: "12 501Q12 527 31 542T63 558Q73 560 77 560Q114 560 128 528Q133 518 188 293T244 61Q244 56 223 50T195 43Q192 43 190 45T102 263T14 486Q12 496 12 501",
      8254: "69 544V590H430V544H69",
      8260: "423 750Q432 750 438 744T444 730Q444 725 271 248T92 -240Q85 -250 75 -250Q68 -250 62 -245T56 -231Q56 -221 230 257T407 740Q411 750 423 750",
      8288: "",
      8289: "",
      8290: "",
      8291: "",
      8292: "",
      8407: "377 694Q377 702 382 708T397 714Q404 714 409 709Q414 705 419 690Q429 653 460 633Q471 626 471 615Q471 606 468 603T454 594Q411 572 379 531Q377 529 374 525T369 519T364 517T357 516Q350 516 344 521T337 536Q337 555 384 595H213L42 596Q29 605 29 615Q29 622 42 635H401Q377 673 377 694",
      8450: "684 131Q684 125 672 109T633 71T573 29T489 -5T386 -19Q330 -19 276 -3T174 46T91 134T44 261Q39 283 39 341T44 421Q66 538 143 611T341 699Q344 699 364 700T395 701Q449 698 503 677T585 655Q603 655 611 662T620 678T625 694T639 702Q650 702 657 690V481L653 474Q640 467 628 472Q624 476 618 496T595 541Q562 587 507 625T390 663H381Q337 663 299 625Q212 547 212 336Q212 249 233 179Q274 30 405 30Q533 30 641 130Q658 147 666 147Q671 147 677 143T684 131ZM250 625Q264 643 261 643Q238 635 214 620T161 579T110 510T79 414Q74 384 74 341T79 268Q89 213 113 169T164 101T217 61T260 39L277 34Q270 41 264 48Q199 111 181 254Q178 281 178 344T181 434Q200 559 250 625ZM621 565V625Q617 623 613 623Q603 619 590 619H575L588 605Q608 583 610 579L621 565",
      8459: "331 505Q331 519 382 574T472 629H480Q512 629 529 614T547 576Q547 555 534 532T520 504Q520 493 549 493Q590 493 623 506T668 533L681 546Q731 600 772 641T832 700T852 717Q857 717 860 711T865 697L866 690Q866 688 813 617T708 475T656 403Q682 403 714 404H771L780 416Q1004 707 1167 707Q1209 707 1225 689T1241 646Q1241 604 1209 547T1118 447Q1081 422 1034 405T952 382T888 374T857 370H852L826 334Q733 204 708 144Q691 104 691 76Q691 29 748 29Q768 31 791 48T831 83T862 122T881 146Q883 148 900 148H917Q921 143 921 140T914 127Q810 -8 723 -8Q611 -4 611 100Q611 142 631 191T676 275T721 337T742 367Q716 367 685 366H628L620 355Q618 352 558 268Q486 168 461 141Q405 79 339 34T215 -28Q188 -36 153 -36Q86 -36 58 -11T29 46Q29 82 55 120T123 158Q144 158 154 146T164 119Q164 102 143 89T100 75Q92 75 86 76T77 80T72 82Q67 82 67 60Q67 28 99 14T170 0Q214 0 272 47T419 224L505 340L518 357Q513 357 504 356T467 347T415 330T360 300T308 253Q296 238 295 237H278H274Q256 237 256 243Q256 248 263 256Q291 294 330 321T407 362T476 382T530 393T552 398Q556 402 573 423T600 454Q602 457 604 460T608 465L610 467Q565 455 532 455Q465 455 449 483Q447 487 447 498Q447 513 463 538T479 579Q479 593 463 593Q436 593 385 519Q374 504 371 502T360 499H353H349Q331 499 331 505ZM1195 634Q1195 643 1195 648T1185 662T1157 671Q1130 671 1092 644T1019 579T952 502T901 436L882 409L891 410Q900 411 913 412T934 415Q1081 439 1144 520Q1195 590 1195 634",
      8460: "11 -16L1 0Q7 6 25 27T57 62T91 88T128 101Q159 99 195 66L203 59L211 67Q239 95 239 133Q239 158 210 213T152 330T123 430Q123 477 173 536T269 630T320 666Q376 610 440 606H443Q457 606 466 611T519 647L542 664Q543 664 543 654V643L522 622Q434 537 403 537Q388 537 366 543T329 555T293 570T270 580L261 585L253 574Q206 517 206 475Q206 452 218 416T242 356L255 331Q256 331 270 345T324 391T421 459L437 468H453Q545 463 608 421L618 415L623 392Q644 307 644 233Q644 97 612 9Q604 -10 601 -15T581 -35Q505 -104 467 -124Q446 -133 431 -133Q414 -132 399 -126T376 -115T368 -107Q368 -106 392 -75L415 -43Q432 -67 444 -73T472 -79H474Q479 -79 484 -78T501 -69T521 -50T538 -13T551 46Q558 97 558 180Q558 232 557 245Q553 277 547 300T528 349T488 389T424 404Q344 404 276 295Q272 288 273 285Q300 216 300 168Q300 161 300 156T298 145T297 137T293 129T289 123T283 116T277 107Q212 23 178 -13L166 -26L149 -9Q108 32 81 32Q63 32 21 -7L11 -16",
      8461: "14 666Q14 675 26 683H344L351 679Q361 665 351 655Q344 648 317 648Q287 645 282 641Q270 637 269 623T266 497V370H511V497Q511 519 510 553Q509 615 507 626T496 641H495Q489 645 459 648Q420 648 420 665Q420 672 426 679L433 683H751Q762 676 762 666Q762 648 724 648Q684 645 677 632Q675 626 675 341Q675 57 677 52Q684 38 724 35Q762 35 762 16Q762 6 751 -1H433L426 3Q420 10 420 17Q420 35 459 35Q501 38 506 52Q511 64 511 190V323H266V190Q266 60 271 52Q276 38 317 35Q342 35 351 28Q360 17 351 3L344 -1H26Q14 5 14 16Q14 35 53 35Q94 38 99 52Q104 60 104 341T99 632Q93 645 53 648Q14 648 14 666ZM233 341V553Q233 635 239 648H131Q134 641 135 638T137 603T139 517T139 341Q139 131 138 89T132 37Q131 36 131 35H239Q233 47 233 129V341ZM639 341V489Q639 548 639 576T640 620T642 639T646 648H537L542 639Q546 625 546 341Q546 130 545 88T538 37Q537 36 537 35H646Q643 41 643 42T641 55T639 84T639 140V341",
      8462: "137 683Q138 683 209 688T282 694Q294 694 294 685Q294 674 258 534Q220 386 220 383Q220 381 227 388Q288 442 357 442Q411 442 444 415T478 336Q478 285 440 178T402 50Q403 36 407 31T422 26Q450 26 474 56T513 138Q516 149 519 151T535 153Q555 153 555 145Q555 144 551 130Q535 71 500 33Q466 -10 419 -10H414Q367 -10 346 17T325 74Q325 90 361 192T398 345Q398 404 354 404H349Q266 404 205 306L198 293L164 158Q132 28 127 16Q114 -11 83 -11Q69 -11 59 -2T48 16Q48 30 121 320L195 616Q195 629 188 632T149 637H128Q122 643 122 645T124 664Q129 683 137 683",
      8463: "150 475Q147 475 118 466T82 457Q73 457 64 467T54 487Q54 490 55 492Q63 506 64 506Q67 512 118 526Q162 541 169 546Q173 559 175 575Q181 596 181 604Q181 613 166 617Q164 617 153 618T135 619Q119 619 114 621T109 630Q109 636 114 656T122 681Q125 685 202 688Q272 695 286 695Q304 695 304 684Q304 682 291 628L278 577L386 612Q466 635 476 635T492 627T499 607Q499 593 489 586Q485 583 373 546L262 512Q262 511 248 455T233 397T236 397T244 404Q295 441 357 441Q405 441 445 417T485 333Q485 284 449 178T412 58T426 44Q447 44 466 68Q485 87 500 130L509 152H531H543Q562 152 562 144Q562 128 546 93T494 23T415 -13Q385 -13 359 3T322 44Q318 52 318 77Q318 99 352 196T386 337Q386 386 346 386Q318 386 286 370Q267 361 245 338T211 292Q207 287 193 235T162 113T138 21Q128 7 122 4Q105 -12 83 -12Q66 -12 54 -2T42 26Q42 45 98 257L151 475H150",
      8464: "487 225Q398 255 398 342Q398 410 455 492Q491 545 552 582T669 636T800 673T918 712Q930 717 933 717Q939 717 942 706T946 689Q946 686 915 664T830 591T729 480Q691 429 657 351T615 260Q628 260 663 279T733 339T769 426Q769 442 767 459T764 479Q764 484 766 486Q769 488 781 493T797 498Q802 498 803 494T808 472Q813 442 813 425Q813 369 761 315Q692 246 605 224L592 220L584 209Q547 155 487 106T358 25Q270 -17 191 -17Q143 -17 101 1T59 59Q59 96 85 127T148 158Q169 158 179 146T189 119Q189 102 167 89T125 75Q116 75 109 77T101 81T97 80Q96 77 96 72Q96 50 123 36T204 21H216Q249 21 302 49T411 134Q439 161 459 187Q487 220 487 225ZM460 334Q460 308 472 290T498 268L510 263Q515 263 545 313T626 438T723 561Q751 589 775 609T808 636T817 644H816Q813 644 732 618Q681 601 645 584T585 548T549 514T518 476Q460 390 460 334",
      8465: "190 601Q161 601 137 587T97 553T71 512T55 477T48 463Q44 465 39 468L30 473L35 488Q73 594 106 636T199 685Q200 686 211 686Q250 686 326 652T417 617Q435 617 455 626T497 652T522 670Q532 660 532 654Q469 591 390 550L378 543L343 556Q223 601 190 601ZM378 208Q378 249 369 318T360 424Q360 430 360 439T361 451L362 462Q416 526 482 571L495 580L503 577L511 575L499 562Q442 502 442 465Q442 436 452 368T462 246Q462 169 442 128T385 56Q292 -26 195 -26Q150 -26 104 14L96 21L43 -16Q43 -15 43 -14T41 -10T38 0L48 13Q76 50 123 97L150 125Q154 131 159 131Q166 131 171 116T182 81T193 53Q199 43 216 33T261 22Q307 22 344 68Q378 113 378 208",
      8466: "572 704Q607 704 607 693Q607 681 590 664H588Q586 664 584 664T578 663Q504 658 434 592T363 457Q363 426 386 401Q417 371 481 361Q490 360 527 360H562Q565 363 595 404T666 494T755 596T854 682T945 717Q986 717 1010 696T1035 637Q1035 593 996 531T873 414Q809 378 753 360T674 338T651 333Q650 333 633 308T588 245T544 185Q498 126 426 78L413 68H414Q498 47 575 47Q626 47 676 74T755 139L762 148H779H783Q802 148 802 142Q802 137 795 129Q760 81 691 33T544 -16Q470 -16 366 20L341 29L331 24Q239 -17 155 -17H141Q90 -17 61 -12T23 1T14 22Q14 44 39 65T103 95Q126 101 180 101Q224 101 258 98T309 90T330 86Q332 86 353 103T389 135Q401 146 412 158T431 179T450 203T466 225T485 252T505 280L535 322H509Q391 322 340 362T289 452Q289 495 321 547T396 630Q438 665 486 684T572 704ZM978 635Q978 644 977 650T973 661T968 668T961 673T954 676T946 678T938 680Q929 680 925 677Q893 659 795 531T682 377Q683 377 711 385T755 401T801 421T856 453T906 495Q927 516 952 557T978 635ZM274 50Q274 51 258 54T216 61T166 65Q160 65 151 65T140 64Q115 58 102 48T88 31Q88 20 159 20Q191 20 219 27T261 42L274 50",
      8467: "345 104T349 104T361 95T369 80T352 59Q268 -20 206 -20Q170 -20 146 3T113 53T99 104L94 129Q94 130 79 116T48 86T28 70Q22 70 15 79T7 94Q7 98 12 103T58 147L91 179V185Q91 186 91 191T92 200Q92 282 128 400T223 612T336 705Q397 705 397 636V627Q397 453 194 233Q185 223 180 218T174 211T171 208T165 201L163 186Q159 142 159 123Q159 17 208 17Q228 17 253 30T293 56T335 94Q345 104 349 104ZM360 634Q360 655 354 661T336 668Q328 668 322 666T302 645T272 592Q252 547 229 467T192 330L179 273Q179 272 186 280T204 300T221 322Q327 453 355 590Q360 612 360 634",
      8469: "20 664Q20 666 31 683H142Q256 683 258 681Q259 680 279 653T342 572T422 468L582 259V425Q582 451 582 490T583 541Q583 611 573 628T522 648Q500 648 493 654Q484 665 493 679L500 683H691Q702 676 702 666Q702 657 698 652Q688 648 680 648Q633 648 627 612Q624 601 624 294V-8Q616 -20 607 -20Q601 -20 596 -15Q593 -13 371 270L156 548L153 319Q153 284 153 234T152 167Q152 103 156 78T172 44T213 34Q236 34 242 28Q253 17 242 3L236 -1H36Q24 6 24 16Q24 34 56 34Q58 35 69 36T86 40T100 50T109 72Q111 83 111 345V603L96 619Q72 643 44 648Q20 648 20 664ZM413 419L240 648H120L136 628Q137 626 361 341T587 54L589 68Q589 78 589 121V192L413 419",
      8472: "300 74Q300 133 338 133Q350 133 356 126T363 109Q363 88 340 76Q340 71 342 62T358 39T393 26Q435 26 474 67T532 182T551 290Q551 325 535 349T484 373Q430 373 378 348T291 289T228 218T187 157T174 130Q254 30 265 10Q276 -15 276 -41Q276 -101 235 -158T142 -216Q112 -216 90 -195T67 -118Q67 -40 104 64L110 81Q81 118 81 174Q81 268 134 360T247 453Q252 453 255 451T258 447L259 445Q259 432 253 420Q251 416 242 416Q209 411 176 341T142 203Q142 193 143 184T146 170T149 165L158 180Q215 280 303 345T485 410Q548 410 586 368T625 255Q625 157 553 74T389 -10H383Q349 -10 325 14Q302 37 300 74ZM105 -123Q105 -134 106 -141T110 -158T122 -173T145 -178Q155 -178 160 -176Q184 -163 199 -132T214 -73Q214 -69 214 -66T213 -59T212 -53T209 -47T205 -41T199 -33T193 -25T184 -14T174 -1L165 10Q156 22 148 32L139 43Q138 43 130 15T113 -54T105 -123",
      8473: "16 666Q16 675 28 683H195Q334 683 370 682T437 672Q511 657 554 611T597 495Q597 343 404 309Q402 308 401 308Q381 303 319 303H261V181Q261 157 262 120Q262 60 267 50T304 36Q310 35 313 35Q352 35 352 17Q352 10 346 3L339 -1H28Q16 5 16 16Q16 35 53 35Q68 36 75 37T87 42T95 52Q98 61 98 341T95 630Q91 640 83 643T53 648Q16 648 16 666ZM235 35Q228 46 227 84Q226 129 226 337V621L230 635L237 648H128Q128 647 133 632Q136 620 136 341Q136 64 133 50L128 35H235ZM301 341H313Q339 341 354 344T389 362T417 410T426 498Q426 586 401 616T322 647Q301 647 293 643Q271 637 264 621Q261 617 261 479V341H301ZM429 350Q431 350 443 353T476 367T515 391T548 432T562 490Q562 550 524 592Q507 607 484 619Q481 621 448 635L433 639L439 621Q462 578 462 506Q462 448 454 413T437 366T428 350H429",
      8474: "480 -10Q480 -13 486 -24T507 -50T541 -80T588 -104T648 -114Q666 -114 688 -110T714 -106Q724 -106 728 -114T729 -130Q723 -145 663 -163T548 -181Q503 -181 463 -169T395 -139T343 -97T307 -56T284 -19L280 -3L262 1Q188 24 131 81Q57 155 37 275Q34 292 34 342T37 410Q58 528 131 601Q179 652 248 676T388 701Q485 701 562 661Q698 595 731 448Q742 410 742 341T731 235Q707 141 646 81Q616 50 575 27T493 -5L480 -10ZM568 342Q568 613 437 659L395 666Q329 666 286 626Q232 570 213 439Q210 408 210 342T213 246Q231 113 286 57Q309 37 342 23Q357 19 389 19Q420 19 437 23Q469 38 491 57Q568 134 568 342ZM174 341V354Q174 393 175 419T183 484T205 561T246 635L249 639Q246 639 224 627T193 608Q189 606 183 601T169 589T155 577Q69 491 69 344Q69 133 231 52Q247 42 247 46Q247 46 246 48Q231 69 222 85T200 141T177 239Q174 269 174 341ZM708 341Q708 410 689 467T640 556T588 606T546 630Q532 638 531 638Q530 638 531 635Q563 590 577 543Q602 472 602 341V316Q602 264 599 230T580 144T531 48Q529 44 532 45T546 52Q575 68 596 84T642 128T683 200T706 299Q708 327 708 341ZM391 -17H333Q329 -15 326 -15Q324 -15 324 -17Q324 -21 362 -68Q424 -130 506 -143Q518 -144 544 -144Q569 -144 577 -143L589 -141L575 -139Q544 -127 509 -101T453 -37L442 -19L391 -17",
      8475: "224 266Q185 266 156 286T127 354Q127 419 176 487T282 594Q346 642 433 679T615 717Q732 717 802 680L815 673Q824 680 840 690T860 700Q864 700 867 693T872 680L873 673Q873 668 858 659L845 651L853 642Q887 605 887 561Q887 500 840 439Q790 379 681 336Q693 312 693 292Q692 276 689 263T672 229T653 198T620 152L575 87Q557 57 557 33Q557 24 560 17T566 8L569 5Q546 5 508 25T470 76Q470 83 473 92T545 198T616 310Q616 317 615 318T612 319Q603 319 575 315H560L545 291Q492 201 429 135T277 23Q202 -17 142 -17H130Q50 -17 16 17Q-2 35 -2 57Q-2 95 24 126T88 158Q106 158 116 147T127 121Q127 110 122 102Q116 93 99 84T63 75Q58 75 53 76T47 77T45 75T44 67Q45 52 57 42T88 27T120 21T144 19Q174 19 208 36T267 76T324 134T369 189T406 239Q462 319 504 374T616 503T755 631L770 644Q767 647 753 654T697 670T602 680Q493 680 399 631T247 516Q218 485 193 440T168 359Q168 328 188 316T234 303Q255 303 273 315T304 340T343 389T390 448Q428 490 441 510T456 548Q456 557 458 559Q459 560 476 567T496 575Q505 575 505 558Q505 511 434 412Q429 406 427 403Q397 360 378 343Q342 308 300 287T224 266ZM819 564Q819 595 800 619L784 606Q729 557 692 512T605 387L591 365L610 364Q622 364 631 363T641 361Q643 361 651 363Q725 388 772 449T819 564ZM794 141Q794 123 725 63T612 3Q609 3 612 5Q612 5 615 7Q639 19 678 57T742 131L755 148H772H780Q794 148 794 141ZM588 -3Q590 0 593 0H594L593 -1Q592 -1 590 -2L588 -3",
      8476: "27 496Q31 569 102 627T234 685Q236 685 241 685T251 686Q287 686 318 672T367 638T399 598T418 564L423 550Q424 554 434 567T463 601T505 639T561 671T626 685Q672 685 688 659T710 572Q713 533 721 523T766 513Q781 513 787 514T794 516Q796 512 798 509T801 504T802 501T787 493Q702 461 624 401L607 389Q655 383 688 358L697 352V342Q699 330 699 297Q704 209 710 173T734 103Q751 69 765 69Q769 69 806 83L824 90V74Q823 73 759 24T693 -26Q692 -26 660 32L628 90L629 111Q631 159 631 177Q631 278 614 300Q584 340 523 340Q500 340 467 333T431 325Q429 325 429 322Q428 321 426 308T420 275T410 230T392 178T366 125L358 112L342 99Q306 70 269 38T213 -10T193 -26Q192 -26 163 0T116 26Q82 26 50 -8L42 -16L35 -8L27 0L35 10Q43 21 58 38T104 80T158 106Q179 106 218 65L235 48Q238 48 255 60T295 99T329 158Q352 231 352 359Q352 555 242 614Q210 628 187 628Q140 628 116 600T91 548Q91 522 138 464T185 382V376Q185 345 158 313T103 263L76 246Q74 244 64 253L54 260L65 267Q91 285 100 302Q111 318 111 337Q111 355 69 410T27 496ZM562 628Q504 628 443 507L435 491L436 479Q437 471 437 446Q437 396 432 351L529 389L602 426Q673 462 673 463H672Q644 470 637 483T622 553Q608 628 562 628",
      8477: "17 665Q17 672 28 683H221Q415 681 439 677Q461 673 481 667T516 654T544 639T566 623T584 607T597 592T607 578T614 565T618 554L621 548Q626 530 626 497Q626 447 613 419Q578 348 473 326L455 321Q462 310 473 292T517 226T578 141T637 72T686 35Q705 30 705 16Q705 7 693 -1H510Q503 6 404 159L306 310H268V183Q270 67 271 59Q274 42 291 38Q295 37 319 35Q344 35 353 28Q362 17 353 3L346 -1H28Q16 5 16 16Q16 35 55 35Q96 38 101 52Q106 60 106 341T101 632Q95 645 55 648Q17 648 17 665ZM241 35Q238 42 237 45T235 78T233 163T233 337V621L237 635L244 648H133Q136 641 137 638T139 603T141 517T141 341Q141 131 140 89T134 37Q133 36 133 35H241ZM457 496Q457 540 449 570T425 615T400 634T377 643Q374 643 339 648Q300 648 281 635Q271 628 270 610T268 481V346H284Q327 346 375 352Q421 364 439 392T457 496ZM492 537T492 496T488 427T478 389T469 371T464 361Q464 360 465 360Q469 360 497 370Q593 400 593 495Q593 592 477 630L457 637L461 626Q474 611 488 561Q492 537 492 496ZM464 243Q411 317 410 317Q404 317 401 315Q384 315 370 312H346L526 35H619L606 50Q553 109 464 243",
      8484: "39 -1Q29 9 29 12Q29 23 60 77T219 337L410 648H364Q261 648 210 628Q168 612 142 588T109 545T97 509T88 490Q85 489 80 489Q72 489 61 503L70 588Q72 607 75 628T79 662T81 675Q84 677 88 681Q90 683 341 683H592Q604 673 604 666Q604 662 412 348L221 37Q221 35 301 35Q406 35 446 48Q504 68 543 111T597 212Q602 239 617 239Q624 239 629 234T635 223Q635 215 621 113T604 8L597 1Q595 -1 317 -1H39ZM148 637L166 648H112V632Q111 629 110 622T108 612Q108 608 110 608T116 612T129 623T148 637ZM552 646Q552 648 504 648Q452 648 450 643Q448 639 266 343T77 37Q77 35 128 35H179L366 339L552 646ZM572 35Q581 89 581 97L561 77Q542 59 526 48L508 37L539 35H572",
      8486: "55 454Q55 503 75 546T127 617T197 665T272 695T337 704H352Q396 704 404 703Q527 687 596 615T666 454Q666 392 635 330T559 200T499 83V80H543Q589 81 600 83T617 93Q622 102 629 135T636 172L637 177H677V175L660 89Q645 3 644 2V0H552H488Q461 0 456 3T451 20Q451 89 499 235T548 455Q548 512 530 555T483 622T424 656T361 668Q332 668 303 658T243 626T193 560T174 456Q174 380 222 233T270 20Q270 7 263 0H77V2Q76 3 61 89L44 175V177H84L85 172Q85 171 88 155T96 119T104 93Q109 86 120 84T178 80H222V83Q206 132 162 199T87 329T55 454",
      8487: "126 584Q119 584 110 539T97 493Q95 490 73 490Q44 490 44 501Q44 515 62 590Q75 672 82 679Q84 684 177 684Q193 684 214 684T241 685Q265 685 271 682T277 664V648Q271 572 229 434T186 231Q186 173 203 132T247 70T302 42T360 33Q391 33 419 42T474 72T517 133T533 231Q533 297 491 437T442 648Q442 675 446 679Q448 684 542 684Q635 684 637 681Q640 678 657 594T675 501Q675 490 646 490Q624 490 622 493Q620 493 609 538T593 584Q591 585 585 585T569 586T551 588H513Q514 586 518 573T538 531T582 453Q647 340 660 277Q663 259 663 232Q663 194 657 177Q652 151 629 112T560 39Q495 -5 424 -19Q403 -22 360 -22Q318 -22 297 -19Q239 -8 193 18T120 74T80 131T62 177Q56 194 56 229Q56 281 74 328T137 453Q160 491 174 518T193 555T201 575T206 588H168Q160 587 150 587T134 586T126 584",
      8488: "148 590Q95 592 91 627V633L160 729H176Q169 713 169 705Q169 670 244 670Q269 670 305 672T357 675Q405 675 432 661T468 609Q469 605 469 596Q469 572 460 540Q433 463 301 372Q325 378 359 378Q431 378 472 350T519 297Q532 249 532 198Q532 115 500 40T442 -57Q335 -139 202 -139Q165 -139 125 -131L112 -129V-100Q112 -49 106 -33T75 -17Q55 -17 31 -35L22 -42L11 -26L22 -18Q94 36 151 36H160Q171 36 178 33T188 27T194 13T196 -5T197 -32Q198 -79 206 -90Q217 -107 251 -107Q336 -107 389 -33T442 155Q442 240 407 274Q362 319 285 319Q236 319 192 298Q188 298 181 309L224 372Q227 373 234 374T246 376T257 379T271 384T285 391T302 402T321 417Q384 471 384 540Q384 562 366 581T306 600Q292 600 233 595T148 590",
      8492: "256 262Q161 262 161 351Q161 408 203 471T289 570Q380 645 475 676T617 707L627 708Q637 708 644 708Q759 708 831 675L844 669L857 677Q892 700 896 700Q902 700 907 685Q907 683 907 681T908 678T909 676T909 673Q909 671 909 670T906 667T903 664T897 660T889 655L878 647L889 636Q928 598 928 548Q928 529 923 510T907 474T886 442T861 412T837 388T815 368T800 355Q847 323 847 270V263Q847 205 806 145Q766 82 695 37T564 -8Q527 -8 506 10T484 58Q484 85 501 117T543 172Q607 226 685 228Q695 228 698 226Q703 220 692 206Q684 194 682 193T665 191Q625 189 595 172T550 133T529 93T522 66Q522 29 576 29Q642 29 705 109Q785 211 785 270Q785 287 779 300T769 316T755 327L740 319Q682 290 634 290Q611 290 592 294H588L565 261Q559 252 544 231T522 201T504 178T481 151T455 123Q394 63 314 18T159 -28Q103 -28 67 -6T31 54Q31 88 57 123T123 158Q144 158 154 146T164 119Q164 102 142 89T100 75Q94 75 87 77T76 80L72 81Q69 78 69 65Q69 35 102 22T175 9Q184 9 198 11Q248 23 300 70T403 187T508 331T636 489T789 629L801 639Q796 642 786 647T732 661T633 670Q592 670 558 665Q481 651 409 613T286 520Q274 507 258 485T222 424T202 354Q202 299 269 299Q282 299 295 301T318 307T339 317T358 329T376 345T391 362T406 380T420 398T433 417T445 435Q496 512 496 547Q496 559 497 560T516 569Q526 574 530 574Q538 574 538 540Q538 414 427 325Q342 262 256 262ZM689 382Q708 382 753 375L765 387Q860 482 860 555Q860 594 839 610L822 592Q794 563 752 511T680 420T651 380Q655 381 660 381Q664 382 689 382ZM697 344Q692 345 681 345H675Q671 345 665 345T655 344T650 344L648 342Q646 339 645 338Q643 333 639 327H653Q670 329 676 330Q706 342 706 343Q702 344 697 344",
      8493: "299 585Q333 609 384 634T470 672L505 685Q506 685 513 662T531 613T548 580Q553 576 563 576Q575 576 605 585Q607 585 607 575V564Q537 532 496 527Q475 542 456 567T427 610T415 627Q410 627 398 618T382 603Q373 588 373 558T386 475T400 399Q400 337 366 303Q343 281 309 266T254 247T226 242L214 257Q214 258 223 260T251 272T287 299Q304 316 304 360Q304 396 289 451T274 532Q274 553 277 561V564H269Q205 558 172 501T139 358Q139 207 226 127T443 46Q448 46 457 46T470 47L485 48L601 106Q602 106 602 93V80Q551 48 517 25T474 -4T460 -13T443 -19Q409 -24 367 -24Q360 -24 351 -24T335 -23T326 -22Q190 -2 125 87T59 319V328Q62 412 96 487L101 500L118 512Q189 563 245 591L266 601L299 585",
      8496: "280 398L279 400Q278 402 277 405T275 413T272 426T271 443Q271 494 302 544T379 629T472 685T553 707H565H573Q630 707 664 689Q718 661 718 604Q718 548 662 492T553 436Q525 436 508 451T490 492Q490 534 531 579T619 630Q632 630 632 623Q632 619 624 606Q614 593 602 592T578 580Q566 568 549 541T532 497Q532 474 565 474Q577 474 587 476Q600 481 611 489Q630 503 651 535T672 596Q672 660 553 660H548Q494 660 450 616Q421 587 384 531T343 439Q341 420 344 415H345Q346 415 352 415T369 417T391 418Q421 418 440 412T466 398T473 382Q473 367 452 353T398 339Q370 339 348 345T315 359L304 366Q297 365 284 360T234 321T163 234Q120 160 120 117Q120 83 149 57T252 30Q311 30 357 60Q386 79 414 114T452 179Q454 186 454 200Q454 230 415 242Q401 246 373 246Q353 246 347 244Q328 236 313 219T288 184T274 149T265 121T261 109Q260 107 247 102T230 97Q223 97 223 105Q223 148 271 216T386 284Q446 284 483 260T520 195Q520 121 427 57T239 -8Q192 -8 152 2T79 46T46 133Q46 212 107 285T269 394L280 398ZM427 376Q427 377 402 380Q386 380 386 379L425 375L427 376",
      8497: "258 428Q258 489 322 562T482 685T661 735Q726 735 824 693T977 651Q990 651 990 644Q990 639 971 612T948 581Q947 580 938 580Q878 580 784 621T617 663Q544 663 480 635T379 568T320 492T299 431Q299 387 362 387Q404 387 438 402T493 438T527 486T546 531T551 563Q551 569 550 575T549 584T549 590Q551 593 563 602T579 611Q584 611 592 605T608 584T616 548Q616 513 595 477T554 423Q518 392 464 372T349 351Q258 351 258 428ZM324 187T305 187T286 196Q286 202 301 217Q327 242 383 262T484 290L527 297L567 356Q624 441 643 467T688 521Q715 550 752 581T795 613T804 603T808 587T778 547T702 444T626 300H637Q663 302 685 306L697 308L703 317Q745 376 792 400Q806 406 818 406Q849 406 849 375Q847 355 831 338T797 312T763 296L747 290Q744 289 735 266T724 241Q722 240 702 232T664 217T645 210Q638 210 638 218Q638 224 653 246T669 270Q669 271 668 271Q663 270 624 264L607 263Q570 199 529 152Q513 133 484 106T409 45T305 -13T193 -36Q109 -36 74 -10T39 50Q39 86 64 121T128 158Q171 158 171 121Q171 97 141 83Q125 75 107 75Q93 75 80 83Q76 71 76 62Q76 29 117 15T207 0Q324 0 494 248L501 258H495Q368 239 330 195Q324 187 305 187ZM775 335Q806 358 806 368Q805 369 804 369Q800 369 791 360Q774 336 775 335",
      8498: "457 681Q471 695 477 695Q485 695 497 681V12L484 -1H68Q55 14 55 19T68 39H457V328H215L211 335Q198 346 211 359L217 368H457V681",
      8499: "112 -7Q86 -7 58 6T30 48T54 103T113 130Q129 130 141 121T153 94Q153 71 132 59T90 47H80Q95 30 133 30Q180 30 228 63T311 137T402 249T500 361Q566 425 703 529T910 693Q942 721 945 721T958 716T970 709Q974 704 964 691Q961 688 905 622T847 554L595 181Q553 121 527 77T496 19L492 5Q497 5 531 46Q579 98 685 224T850 409L972 524Q994 543 1004 556Q1012 567 1097 643T1186 720Q1194 720 1206 715T1215 703Q1215 701 1191 671T1133 599T1080 530Q1036 461 983 357T862 152Q802 64 799 17Q799 7 800 5T811 2Q836 2 882 37T969 126Q972 130 974 134T978 138T983 139T996 140H1012Q1018 134 1018 132Q1018 122 981 83T889 4T795 -35Q761 -35 745 -12T728 48Q728 122 781 190Q833 269 890 370L927 434L914 422Q848 360 752 245Q643 117 582 51T498 -33T461 -50Q424 -48 424 -4Q424 84 481 172L714 495Q591 406 523 333Q507 316 430 226T313 95Q263 48 221 24T162 -4T120 -7H112",
      8501: "55 613Q55 643 61 663T74 688T85 694Q94 694 94 681Q98 632 134 588L412 285Q416 311 430 397T447 509V519L438 526Q407 554 398 571T388 617T394 664T407 688T418 694Q425 694 427 684Q429 675 454 635T488 586Q490 584 496 579T513 563T537 540Q555 516 555 487Q555 460 549 441T537 416T528 409Q519 409 517 415T513 435T503 463Q492 481 490 481Q454 264 454 246Q454 237 479 212T529 152T555 79Q555 32 538 9Q531 1 524 1Q516 1 516 13Q512 62 476 106Q468 115 337 258T195 412L193 406Q191 401 189 394T183 377T176 352T171 322T167 284T165 240Q165 224 166 220Q171 199 211 152T252 70Q252 45 235 29T203 8T175 1Q170 0 115 0H79Q60 0 58 3T55 20Q55 31 58 34Q60 37 76 37Q112 39 126 46T140 70Q140 96 112 148T83 236Q83 281 102 334T140 419T159 452Q55 556 55 613",
      8502: "56 706V726Q56 763 76 763Q83 763 87 759T98 741Q108 726 116 721L127 717L340 715Q547 712 564 709Q575 705 587 692Q599 680 605 663L609 650V137H676Q687 124 687 115Q687 110 678 100T622 43L558 -21H-9Q-22 -6 -22 -1T-13 14T42 72L107 137H569V339Q569 541 567 546Q558 555 554 557L545 563H329Q118 566 101 569Q90 573 78 586Q54 610 54 661Q54 670 56 706",
      8503: "56 750Q68 764 76 764Q88 764 97 743T125 717Q131 715 240 715T358 713Q421 691 421 640Q421 608 399 588T358 566Q353 566 352 565T351 557L356 526Q356 488 379 346T402 97Q400 21 385 -12Q366 -43 351 -43Q335 -43 329 -10Q316 40 316 64Q316 67 315 67Q313 67 269 26L222 -21H-9Q-22 -7 -22 -1Q-22 4 -14 14T42 73L107 137H311V564H211H164Q115 564 93 573T60 615Q56 630 56 690V750",
      8504: "62 757Q69 764 75 764Q87 764 97 741Q102 731 105 728T117 721L129 715H349Q569 715 580 710Q618 701 635 670Q640 661 640 639Q640 609 622 590Q617 583 604 575T580 566H573V553Q575 547 576 531T582 469T600 353Q624 205 624 104Q624 46 617 17T591 -32Q581 -43 573 -43Q550 -43 540 44Q535 73 533 319V564H322Q117 566 100 570Q90 573 77 586Q54 609 54 663Q54 689 55 706Q55 738 56 745T62 757",
      8513: "239 665Q194 665 154 653T90 629T66 617Q59 617 53 623T46 637Q46 652 66 659Q129 695 197 701Q218 705 248 705Q293 705 335 693Q371 684 435 644Q543 562 573 417Q577 393 577 341Q577 290 573 266Q531 83 384 10Q346 -9 315 -16T234 -23H206Q202 -23 183 -23T152 -21T120 -18T88 -10T63 3T44 24L37 35V297L50 310H235Q248 297 248 290Q248 285 235 270H77V103Q77 88 77 80T77 63T78 50T80 43T82 38T85 35T89 32T95 30Q126 20 206 17Q289 17 330 30Q407 55 460 120T533 275Q538 305 538 342Q538 486 452 575T239 665",
      8592: "944 261T944 250T929 230H165Q167 228 182 216T211 189T244 152T277 96T303 25Q308 7 308 0Q308 -11 288 -11Q281 -11 278 -11T272 -7T267 2T263 21Q245 94 195 151T73 236Q58 242 55 247Q55 254 59 257T73 264Q121 283 158 314T215 375T247 434T264 480L267 497Q269 503 270 505T275 509T288 511Q308 511 308 500Q308 493 303 475Q293 438 278 406T246 352T215 315T185 287T165 270H929Q944 261 944 250",
      8593: "27 414Q17 414 17 433Q17 437 17 439T17 444T19 447T20 450T22 452T26 453T30 454T36 456Q80 467 120 494T180 549Q227 607 238 678Q240 694 251 694Q259 694 261 684Q261 677 265 659T284 608T320 549Q340 525 363 507T405 479T440 463T467 455T479 451Q483 447 483 433Q483 413 472 413Q467 413 458 416Q342 448 277 545L270 555V-179Q262 -193 252 -193H250H248Q236 -193 230 -179V555L223 545Q192 499 146 467T70 424T27 414",
      8594: "56 237T56 250T70 270H835Q719 357 692 493Q692 494 692 496T691 499Q691 511 708 511H711Q720 511 723 510T729 506T732 497T735 481T743 456Q765 389 816 336T935 261Q944 258 944 250Q944 244 939 241T915 231T877 212Q836 186 806 152T761 85T740 35T732 4Q730 -6 727 -8T711 -11Q691 -11 691 0Q691 7 696 25Q728 151 835 230H70Q56 237 56 250",
      8595: "473 86Q483 86 483 67Q483 63 483 61T483 56T481 53T480 50T478 48T474 47T470 46T464 44Q428 35 391 14T316 -55T264 -168Q264 -170 263 -173T262 -180T261 -184Q259 -194 251 -194Q242 -194 238 -176T221 -121T180 -49Q169 -34 155 -21T125 2T95 20T67 33T44 42T27 47L21 49Q17 53 17 67Q17 87 28 87Q33 87 42 84Q158 52 223 -45L230 -55V312Q230 391 230 482T229 591Q229 662 231 676T243 693Q244 694 251 694Q264 692 270 679V-55L277 -45Q307 1 353 33T430 76T473 86",
      8596: "263 479Q267 501 271 506T288 511Q308 511 308 500Q308 493 303 475Q293 438 278 406T246 352T215 315T185 287T165 270H835Q729 349 696 475Q691 493 691 500Q691 511 711 511Q720 511 723 510T729 506T732 497T735 481T743 456Q765 389 816 336T935 261Q944 258 944 250Q944 244 939 241T915 231T877 212Q836 186 806 152T761 85T740 35T732 4Q730 -6 727 -8T711 -11Q691 -11 691 0Q691 7 696 25Q728 151 835 230H165Q167 228 182 216T211 189T244 152T277 96T303 25Q308 7 308 0Q308 -11 288 -11Q281 -11 278 -11T272 -7T267 2T263 21Q245 94 195 151T73 236Q58 242 55 247Q55 254 59 257T73 264Q144 292 194 349T263 479",
      8597: "27 492Q17 492 17 511Q17 515 17 517T17 522T19 525T20 528T22 530T26 531T30 532T36 534Q80 545 120 572T180 627Q210 664 223 701T238 755T250 772T261 762Q261 757 264 741T282 691T319 628Q352 589 390 566T454 536L479 529Q483 525 483 511Q483 491 472 491Q467 491 458 494Q342 526 277 623L270 633V-133L277 -123Q307 -77 353 -45T430 -2T473 8Q483 8 483 -11Q483 -15 483 -17T483 -22T481 -25T480 -28T478 -30T474 -31T470 -32T464 -34Q407 -49 364 -84T300 -157T270 -223T261 -262Q259 -272 250 -272Q242 -272 239 -255T223 -201T180 -127Q169 -112 155 -99T125 -76T95 -58T67 -45T44 -36T27 -31L21 -29Q17 -25 17 -11Q17 9 28 9Q33 9 42 6Q158 -26 223 -123L230 -133V633L223 623Q192 577 146 545T70 502T27 492",
      8598: "204 662Q257 662 301 676T369 705T394 720Q398 720 407 711T417 697Q417 688 389 671T310 639T212 623Q176 623 153 628Q151 628 221 557T546 232Q942 -164 943 -168Q944 -170 944 -174Q944 -182 938 -188T924 -195Q922 -195 916 -193Q912 -191 517 204Q440 281 326 394T166 553L121 598Q126 589 126 541Q126 438 70 349Q59 332 52 332Q48 332 39 341T29 355Q29 358 38 372T57 407T77 464T86 545Q86 583 78 614T63 663T55 683Q55 693 65 693Q73 693 82 688Q136 662 204 662",
      8599: "582 697Q582 701 591 710T605 720Q607 720 630 706T697 677T795 662Q830 662 863 670T914 686T934 694Q942 694 944 685Q944 680 936 663T921 615T913 545Q913 490 927 446T956 379T970 355Q970 351 961 342T947 332Q940 332 929 349Q874 436 874 541Q874 590 878 598L832 553Q787 508 673 395T482 204Q87 -191 83 -193Q77 -195 75 -195Q67 -195 61 -189T55 -174Q55 -170 56 -168Q58 -164 453 232Q707 487 777 557T847 628Q824 623 787 623Q689 623 599 679Q582 690 582 697",
      8600: "55 675Q55 683 60 689T75 695Q77 695 83 693Q87 691 482 296Q532 246 605 174T717 62T799 -20T859 -80T878 -97Q874 -93 874 -41Q874 64 929 151Q940 168 947 168Q951 168 960 159T970 145Q970 143 956 121T928 54T913 -45Q913 -83 920 -114T936 -163T944 -185Q942 -194 934 -194Q932 -194 914 -186T864 -170T795 -162Q743 -162 698 -176T630 -205T605 -220Q601 -220 592 -211T582 -197Q582 -187 611 -170T691 -138T787 -123Q824 -123 847 -128Q848 -128 778 -57T453 268Q58 664 56 668Q55 670 55 675",
      8601: "126 -41Q126 -92 121 -97Q121 -98 139 -80T200 -20T281 61T394 173T517 296Q909 690 916 693Q922 695 924 695Q932 695 938 689T944 674Q944 670 943 668Q942 664 546 268Q292 13 222 -57T153 -128Q176 -123 212 -123Q310 -123 400 -179Q417 -190 417 -197Q417 -201 408 -210T394 -220Q392 -220 369 -206T302 -177T204 -162Q131 -162 67 -194Q63 -195 59 -192T55 -183Q55 -180 62 -163T78 -115T86 -45Q86 10 72 54T44 120T29 145Q29 149 38 158T52 168Q59 168 70 151Q126 62 126 -41",
      8602: "942 250Q942 244 928 230H511L457 148Q440 124 420 93Q404 68 400 64T389 60Q381 60 375 66T368 81Q368 88 415 159L462 230H175L188 214Q210 188 235 145T264 85Q264 75 260 74T231 72L206 74L191 103Q169 142 164 150Q130 195 64 239Q56 244 56 250T64 261Q115 294 142 323T191 397L206 428H231Q255 428 259 426T264 414Q260 397 235 355T188 288L175 272L331 270Q488 270 491 272Q491 275 542 352T597 432Q602 437 609 437Q617 437 622 432T628 417T582 341L537 272L735 270H931Q942 257 942 250",
      8603: "54 250Q54 258 66 270H277L488 272L542 350Q596 431 602 435Q604 437 609 437Q617 437 622 432T628 417T582 341L537 272L608 270H751L822 272L808 288Q786 313 761 355T733 414Q733 424 737 426T766 428H793L806 397Q829 354 864 314Q896 284 928 263Q942 257 942 250T928 237Q887 208 864 185Q829 147 806 103L793 74L766 72Q742 72 738 73T733 85Q735 102 756 137T797 198L817 225L822 230H511L457 148Q440 124 420 93Q404 68 400 64T389 60Q381 60 375 66T368 81Q368 88 415 159L462 230H264L66 232Q54 239 54 250",
      8606: "56 250Q103 277 142 322T199 417H221Q244 417 244 416Q244 414 237 397T208 344T158 278L151 270H276L285 277Q322 306 349 345T388 417H434Q434 413 424 392T393 338T349 279L340 270H634Q933 270 937 266L938 265Q944 259 944 250T938 235L937 234Q933 230 634 230H340L349 221Q372 196 393 163T424 108T434 83H388Q377 116 350 155T285 223L276 230H151L158 222Q186 191 207 156T236 104T244 84Q244 83 221 83H199Q181 133 142 178T56 250",
      8608: "943 250Q895 221 856 177T801 83H778Q755 83 755 84Q755 86 762 103T791 156T841 222L848 230H723L714 223Q677 194 650 155T611 83H565Q565 87 575 108T606 162T650 221L659 230H365Q66 230 62 234L61 235Q55 241 55 250T61 265L62 266Q66 270 365 270H659L650 279Q627 304 606 337T575 392T565 417H611Q622 384 649 345T714 277L723 270H848L841 278Q813 309 792 344T763 396T755 416Q755 417 778 417H801Q817 367 856 323T943 250",
      8610: "56 250Q103 277 142 322T199 417H221Q244 417 244 416Q244 414 237 397T208 344T158 278L151 270H873L882 277Q919 306 946 345T985 417H1031Q1031 413 1021 392T990 338T946 279L937 270V230L946 221Q969 196 990 163T1021 108T1031 83H985Q974 116 947 155T882 223L873 230H151L158 222Q186 191 207 156T236 104T244 84Q244 83 221 83H199Q181 133 142 178T56 250",
      8611: "1054 250Q1006 221 967 177T912 83H889Q866 83 866 84Q866 86 873 103T902 156T952 222L959 230H237L228 223Q191 194 164 155T125 83H79Q79 87 89 108T120 162T164 221L173 230V270L164 279Q141 304 120 337T89 392T79 417H125Q136 384 163 345T228 277L237 270H959L952 278Q924 309 903 344T874 396T866 416Q866 417 889 417H912Q928 367 967 323T1054 250",
      8614: "95 155V109Q95 83 92 73T75 63Q61 63 58 74T54 130Q54 140 54 180T55 250Q55 421 57 425Q61 437 75 437Q88 437 91 428T95 393V345V270H835Q719 357 692 493Q692 494 692 496T691 499Q691 511 708 511H711Q720 511 723 510T729 506T732 497T735 481T743 456Q765 389 816 336T935 261Q944 258 944 250Q944 244 939 241T915 231T877 212Q836 186 806 152T761 85T740 35T732 4Q730 -6 727 -8T711 -11Q691 -11 691 0Q691 7 696 25Q728 151 835 230H95V155",
      8617: "903 424T903 444T929 464Q976 464 1023 434T1070 347Q1070 316 1055 292T1016 256T971 237T929 230H165Q167 228 182 216T211 189T244 152T277 96T303 25Q308 7 308 0Q308 -11 288 -11Q281 -11 278 -11T272 -7T267 2T263 21Q245 94 195 151T73 236Q58 242 55 247Q55 254 59 257T73 264Q121 283 158 314T215 375T247 434T264 480L267 497Q269 503 270 505T275 509T288 511Q308 511 308 500Q308 493 303 475Q293 438 278 406T246 352T215 315T185 287T165 270H926Q929 270 941 271T960 275T978 280T998 290T1015 307Q1030 325 1030 347Q1030 355 1027 364T1014 387T983 411T929 424H928Q903 424 903 444",
      8618: "55 347Q55 380 72 404T113 441T159 458T197 464Q222 464 222 444Q222 429 204 426T157 417T110 387Q95 369 95 347Q95 339 98 330T111 307T142 283T196 270H961Q845 357 818 493Q818 494 818 496T817 499Q817 511 834 511H837Q846 511 849 510T855 506T858 497T861 481T869 456Q891 389 942 336T1061 261Q1070 258 1070 250Q1070 244 1065 241T1041 231T1003 212Q962 186 932 152T887 85T866 35T858 4Q856 -6 853 -8T837 -11Q817 -11 817 0Q817 7 822 25Q854 151 961 230H196Q149 230 102 260T55 347",
      8619: "56 250Q103 277 142 322T199 417H221Q244 417 244 416Q244 414 237 397T208 344T158 278L151 270H622V305Q622 356 624 388T635 460T661 521T709 559T785 575Q813 575 833 573T880 561T923 534T952 483T964 405Q964 374 959 350T942 307T918 276T884 255T847 242T804 235T760 231T713 230H662V-27Q654 -41 644 -41H642H640Q628 -41 622 -27V230H151L158 222Q186 191 207 156T236 104T244 84Q244 83 221 83H199Q181 133 142 178T56 250ZM924 403Q924 474 894 505T794 536Q758 536 734 526T696 500T675 453T665 395T662 319V270H699Q826 270 875 295T924 403",
      8620: "35 405Q35 454 48 489T86 542T137 567T195 575Q229 575 251 571T301 554T345 510T370 429Q377 384 377 305V270H848L841 278Q813 309 792 344T763 396T755 416Q755 417 778 417H801Q817 367 856 323T943 250Q896 221 857 177T801 83H778Q755 83 755 84Q755 86 762 103T791 156T841 222L848 230H377V-27Q369 -41 359 -41H357Q342 -41 337 -25V230H286Q247 231 225 232T169 238T115 255T75 284T45 333T35 405ZM75 406Q75 322 123 296T300 270H337V319Q335 432 317 477T240 534Q232 535 197 535Q140 535 108 507T75 406",
      8621: "57 250Q159 311 200 417H246L242 407Q215 340 159 278L152 270H276L315 310Q354 349 358 351Q366 356 376 351Q378 350 455 273L530 196L606 273Q683 350 686 351Q694 354 703 351Q705 350 782 273L858 196L933 273Q1010 350 1012 351Q1022 356 1030 351Q1034 349 1073 310L1112 270H1236L1229 278Q1173 340 1146 407L1142 417H1188Q1233 306 1331 250Q1231 192 1188 83H1142L1146 93Q1173 160 1229 222L1236 230H1168Q1155 230 1139 230T1119 229Q1112 229 1108 229T1099 231T1092 233T1085 238T1078 245T1068 256T1056 269L1021 304L984 267Q948 230 910 191T867 149Q857 144 848 150Q844 151 770 227T694 304T618 228T540 150Q531 144 521 149Q517 152 479 191T404 267L367 304L332 269Q328 264 320 256T310 246T303 239T296 234T289 231T280 229T269 229Q265 229 249 229T220 230H152L159 222Q215 160 242 93L246 83H223L200 84L195 96Q152 190 57 250",
      8622: "491 272Q491 275 542 352T597 432Q602 437 609 437Q617 437 622 432T628 417T582 341L537 272L608 270H751L822 272L808 288Q786 313 761 355T733 414Q733 424 737 426T766 428H793L806 397Q829 354 864 314Q896 284 928 263Q942 257 942 250T928 237Q887 208 864 185Q829 147 806 103L793 74L766 72Q742 72 738 73T733 85Q735 102 756 137T797 198L817 225L822 230H511L457 148Q440 124 420 93Q404 68 400 64T389 60Q381 60 375 66T368 81Q368 88 415 159L462 230H175L188 214Q210 188 235 145T264 85Q264 75 260 74T231 72L206 74L191 103Q169 142 164 150Q130 195 64 239Q56 244 56 250T64 261Q115 294 142 323T191 397L206 428H231Q255 428 259 426T264 414Q260 397 235 355T188 288L175 272L331 270Q488 270 491 272",
      8624: "56 555Q74 567 79 570T107 592T141 625T170 667T198 722H221Q244 722 244 721Q244 718 236 699T207 647T161 587L151 576L291 575H292H293H294H296H297H298H299H300H301H302H304H305H306H307H308H309H310H311H312H314H315H316H317H318H319H320H321H322H323H324H325H327H328H329H330H331H332H333H334H335H336H337H338H339H340H341H342H343H345Q435 574 438 570L439 569L440 568Q444 564 444 287Q444 15 442 12Q436 0 424 0T406 12Q404 15 404 275V535H151L162 523Q187 495 207 462T236 410T244 389H198L193 402Q171 457 131 497T56 555",
      8625: "301 722Q339 618 443 555L437 551Q431 547 422 541T401 526T377 504T352 477T327 443T306 402L301 389H255Q255 392 263 410T291 461T337 523L348 535H95V275Q95 15 93 12Q87 0 75 0T57 12Q55 15 55 287Q55 564 59 568L60 569Q64 573 76 573T208 575L348 576L338 587Q314 613 294 646T264 698T255 721Q255 722 278 722H301",
      8630: "361 210Q373 210 373 182V177Q373 155 370 151T348 139Q303 118 267 84T216 28T201 1Q197 -1 196 -1Q189 -1 184 8Q166 39 143 64T99 104T61 129T32 144T19 150Q17 152 17 179Q17 203 21 208Q28 210 39 206Q106 178 157 135L175 119V126Q179 130 179 155Q182 173 193 201Q228 305 312 374T510 459Q532 461 551 461H567Q678 461 784 386Q835 344 861 301Q902 245 926 173T950 32Q950 15 944 8Q930 -6 917 8Q910 12 910 43Q901 208 801 314T561 421Q453 421 359 359Q300 319 263 258T217 126L216 125Q216 124 216 123T217 122Q219 122 229 131T260 156T301 181Q314 189 336 199T361 210",
      8631: "972 209Q980 209 981 204T982 179Q982 155 979 151T957 139Q915 121 878 86T815 8Q808 -1 803 -1Q801 -1 797 1Q797 6 783 28T732 84T650 139L628 150Q626 152 626 177Q626 201 630 206Q636 210 637 210Q650 210 697 181Q727 166 764 137L784 119L782 132Q767 239 689 318T499 417Q474 421 442 421Q343 421 261 369T130 219Q86 121 86 28Q86 15 79 8Q73 1 66 1T53 8Q46 15 46 30Q46 102 77 192T186 361Q274 443 386 459Q396 460 426 460Q515 460 588 431T703 361T773 271T812 187T822 132Q822 123 825 123Q936 209 972 209",
      8634: "369 543T369 563T397 583Q408 583 440 579L454 577L464 581Q492 592 516 609T552 638T565 650Q604 638 607 637Q606 636 598 628T585 614T570 601T548 584T523 568L510 560L516 558Q522 555 527 553T541 546T559 536T580 523T603 506T626 485Q722 384 722 250Q722 106 622 12T387 -83Q253 -83 155 12T56 250Q56 357 110 433T235 545Q244 550 252 550Q270 550 270 531Q270 522 261 515T238 501T202 477T159 433Q95 352 95 250Q95 131 178 45T388 -42Q511 -42 596 43T682 250Q682 340 636 408T522 511Q495 526 488 526Q488 525 488 525T487 522T485 515L490 506Q505 481 516 451T531 404T535 384L532 385Q529 386 524 387T513 390L491 397L488 408Q472 483 413 542L399 543Q369 543 369 563",
      8635: "170 637L213 650Q270 597 313 581L323 577L337 579Q369 583 380 583Q408 583 408 563T380 543H378L364 542Q305 483 289 408L286 397L264 390Q259 389 254 388T245 385L242 384Q242 387 246 403T261 450T287 506L292 515Q291 519 291 521T290 524T289 526Q284 526 265 517T216 486T160 434T114 354T95 249Q95 132 178 45T388 -42Q513 -42 597 44T682 250Q682 337 638 404T532 506Q529 508 525 510T519 514T515 516T511 519T509 522T508 526T507 531Q507 550 525 550Q533 550 542 545Q569 532 596 511T653 454T702 366T721 250Q721 151 672 74T547 -43T388 -83Q254 -83 155 12T56 250Q56 385 151 485Q164 498 179 509T205 528T228 542T247 551T260 558L267 560L254 568Q215 590 170 637",
      8636: "62 230Q56 236 55 244Q55 252 57 255T69 265Q114 292 151 326T208 391T243 448T265 491T273 509Q276 511 288 511Q304 511 306 505Q309 501 303 484Q293 456 279 430T251 383T223 344T196 313T173 291T156 276L148 270H929Q944 261 944 250T929 230H62",
      8637: "55 256Q56 264 62 270H929Q944 261 944 250T929 230H148Q149 229 165 215T196 185T231 145T270 87T303 16Q309 -1 306 -5Q304 -11 288 -11Q279 -11 276 -10T269 -4T264 10T253 36T231 75Q172 173 69 235Q59 242 57 245T55 256",
      8638: "188 258V694H208L215 682Q246 628 293 594T375 551V528Q375 505 374 505Q369 505 351 510T299 534T237 578L228 587V205Q228 -178 226 -182Q221 -194 208 -194T190 -182Q188 -178 188 258",
      8639: "41 551Q76 559 123 592T201 682L208 694H228V258Q228 -178 226 -182Q221 -194 208 -194T190 -182Q188 -178 188 205V587L179 578Q151 552 117 534T65 511T42 505Q41 505 41 528V551",
      8640: "691 500Q691 511 711 511Q720 511 723 510T730 504T735 490T746 464T768 425Q796 378 835 339T897 285T933 263Q941 258 942 256T944 245T937 230H70Q56 237 56 250T70 270H852Q802 308 762 364T707 455T691 500",
      8641: "56 237T56 250T70 270H937Q944 263 944 256Q944 251 944 250T943 246T940 242T933 238Q794 153 734 7Q729 -7 726 -9T711 -11Q695 -11 693 -5Q690 -1 696 16Q721 84 763 139T852 230H70Q56 237 56 250",
      8642: "190 682Q195 694 208 694T226 683Q228 679 228 296V-87L237 -78Q265 -52 299 -34T351 -11T374 -5Q375 -5 375 -28V-51Q340 -60 293 -92T215 -182L208 -194H188V242Q188 678 190 682",
      8643: "188 295V573Q188 657 189 672T200 692Q206 694 208 694Q221 694 226 683Q228 679 228 242V-194H208L201 -182Q170 -128 123 -94T41 -51V-28Q41 -5 42 -5Q47 -5 65 -10T117 -34T179 -78L188 -87V295",
      8644: "943 500Q895 471 856 427T801 333H778Q755 333 755 334Q755 336 762 353T791 406T841 472L848 480H459Q70 480 67 482Q55 488 55 500T67 518Q70 520 459 520H848L841 528Q813 559 792 594T763 646T755 666Q755 667 778 667H801Q817 617 856 573T943 500ZM56 167Q102 194 141 238T198 333H221Q244 333 244 332Q221 265 161 198L151 187H539Q928 187 930 186Q944 182 944 167Q944 155 934 149Q930 147 541 147H151L160 137Q185 110 205 77T235 24T244 1Q244 0 221 0H199Q158 106 56 167",
      8646: "56 500Q103 527 142 572T199 667H221Q244 667 244 666Q244 664 237 647T208 594T158 528L151 520H539Q928 520 932 518Q944 513 944 500T932 482Q928 480 539 480H151L158 472Q186 441 207 406T236 354T244 334Q244 333 221 333H199Q181 383 142 428T56 500ZM943 167Q835 101 801 0H778Q755 0 755 1T758 9T765 25T771 39Q800 94 839 137L848 147H458Q68 147 66 149Q55 154 55 167Q55 182 69 186Q71 187 460 187H848L838 198Q811 228 791 261T762 314L755 332Q755 333 778 333H801Q841 227 943 167",
      8647: "930 437Q944 426 944 416T934 399Q930 397 540 397H150L159 387Q185 360 205 328T234 277T243 252Q243 237 217 191T159 113L150 103H540Q930 103 934 101Q944 94 944 84Q944 71 930 64L540 63H151Q180 34 203 -2T236 -61L244 -83H198Q178 -31 142 11T66 77L55 83L65 89Q157 145 197 246Q199 250 190 269Q150 359 65 411L55 417L66 423Q106 447 142 489T198 583H244Q202 488 151 437H930",
      8648: "83 551Q190 590 250 694Q251 689 263 671T307 621T380 567Q409 551 416 551Q422 551 447 563T511 608T577 684L582 694Q642 591 749 551V528Q749 505 748 505Q745 505 724 515T669 546T612 590L602 599V-181Q595 -193 585 -193H582H581Q568 -193 565 -183L563 -179L562 209V598L552 589Q517 556 473 531T414 506H412Q411 506 393 514T361 530T324 553T280 589L270 598V-179Q255 -192 250 -193H247Q237 -193 230 -181V599L220 590Q197 567 164 546T110 515T84 505Q83 505 83 528V551",
      8649: "55 416Q55 427 70 437H848Q819 466 796 502T764 561L755 583H801Q821 531 857 489T933 423L944 417L934 411Q843 355 802 254Q800 250 809 231Q849 141 934 89L944 83L933 77Q893 53 857 11T801 -83H755Q797 12 848 63H459L70 64Q55 70 55 84Q55 94 65 101Q69 103 459 103H849L840 113Q806 148 779 196T756 254Q756 255 760 264T770 286T786 315T809 351T840 387L849 397H459Q69 397 65 399Q55 406 55 416",
      8650: "230 681Q240 694 251 694Q260 693 270 680V-98L280 -89Q297 -73 314 -60T348 -38T374 -24T397 -13T412 -6H414Q428 -6 473 -32T552 -89L562 -98V291L563 680Q570 693 582 693Q593 694 602 681V-99L612 -90Q635 -68 668 -47T723 -15T748 -5Q749 -5 749 -28V-51Q642 -91 582 -194L577 -184Q551 -141 512 -108T447 -63T416 -51T385 -63T321 -108T255 -184L250 -194Q189 -89 83 -51V-28Q83 -5 84 -5Q88 -5 109 -15T164 -46T220 -90L230 -99V681",
      8651: "195 504L198 514H221Q244 514 244 512Q244 508 239 490T215 437T171 376L162 367H545Q928 367 932 365Q944 360 944 347T932 329Q928 327 492 327H55V347L67 354Q113 379 146 420T195 504ZM67 171Q70 173 507 173H944V153L932 146Q839 95 804 -4L801 -14H778Q755 -14 755 -12Q768 59 828 124L837 133H454Q71 133 67 135Q55 140 55 153Q55 165 67 171",
      8652: "691 660Q691 671 711 671Q720 671 723 670T730 664T735 650T746 624T768 585Q797 538 836 499T897 445T933 423Q941 418 942 416T944 405T937 390H70Q56 397 56 410T70 430H852Q802 468 762 524T707 615T691 660ZM55 256Q56 264 62 270H929Q944 261 944 250T929 230H148Q149 229 165 215T196 185T231 145T270 87T303 16Q309 -1 306 -5Q304 -11 288 -11Q279 -11 276 -10T269 -4T264 10T253 36T231 75Q172 173 69 235Q59 242 57 245T55 256",
      8653: "397 525Q410 525 414 524T418 516Q418 506 394 467T331 381L319 367H473L624 369L657 445Q674 487 684 507T699 531T709 534Q717 534 722 528T728 516Q728 510 695 434Q689 418 683 402T672 377T668 367H928Q942 355 942 347Q942 341 928 327H791Q651 327 651 325Q649 324 620 251T586 174Q586 172 757 172H928Q942 158 942 152Q942 143 928 132H568L537 54Q510 -9 503 -22T486 -35Q479 -35 473 -29T466 -17T495 61L526 132H319L331 118Q364 81 391 37T418 -17Q418 -23 415 -24T401 -26Q398 -26 397 -26L384 -24L377 -13Q344 49 301 97T218 170T143 210T84 233T55 245Q54 253 59 256T86 267Q281 327 377 512L384 525H397ZM606 325Q606 327 439 327H275Q258 312 179 265L148 249Q228 206 262 181L275 172H544L575 247L606 325",
      8654: "395 -24T395 -19T417 57T440 132H255L266 116Q308 64 340 -6Q342 -17 337 -21Q335 -26 320 -26T302 -19Q302 -15 294 4T265 54T217 117T145 182T49 236Q30 243 33 254Q40 261 49 263Q98 283 142 315T214 379T263 442T293 493T302 519Q305 525 320 525T337 521Q342 516 340 505Q308 435 266 383L255 370L384 367H515Q561 522 569 530Q574 534 580 534Q587 534 594 528T602 516Q602 512 580 441T557 367H651L742 370L731 383Q689 435 657 505Q655 516 660 521Q662 525 677 525T695 519Q695 515 703 496T732 446T780 383T853 317T949 263Q967 258 964 245Q959 240 949 236Q897 215 852 182T779 116T731 52T703 3T695 -19Q692 -26 677 -26T660 -21Q655 -17 657 -6Q670 21 682 42T702 77T717 99T728 114T735 122T739 126T740 130T613 132H482L460 54Q440 -9 433 -23T415 -37Q408 -37 402 -31ZM502 325Q502 327 360 327H217L195 310Q173 291 120 256L111 250Q114 248 143 229T195 190L217 172H335L453 174L502 325ZM886 250Q885 251 865 263T831 286T802 310L780 327H544L535 299Q531 283 511 223L495 174L637 172H780L802 190Q843 225 877 243L886 250",
      8655: "346 174Q348 176 378 249T411 325Q411 327 239 327H68Q55 342 55 347Q55 354 68 367H428L459 445Q487 509 494 521T510 534Q517 534 524 527T531 516Q531 515 502 438L471 367H677L666 381Q631 421 605 463T578 516Q578 522 582 523T599 525H615L619 512Q659 437 714 383T812 309T896 272T942 254Q943 246 938 243T911 232Q718 172 619 -13L615 -24L599 -26Q578 -26 578 -17Q578 -11 587 6T617 53T666 118L677 132H373L339 54Q323 12 313 -8T298 -32T288 -35Q280 -35 275 -29T269 -17Q269 -14 298 57T328 132H68Q55 145 55 152Q55 156 56 158T62 165T68 172H206Q346 172 346 174ZM848 249Q763 297 735 318L722 327H455L422 252L391 174Q391 172 557 172H722L735 181Q773 210 819 234L848 249",
      8656: "944 153Q944 140 929 133H318L328 123Q379 69 414 0Q419 -13 419 -17Q419 -24 399 -24Q388 -24 385 -23T377 -12Q332 77 253 144T72 237Q62 240 59 242T56 250T59 257T70 262T89 268T119 278T160 296Q303 366 377 512Q382 522 385 523T401 525Q419 524 419 515Q419 510 414 500Q379 431 328 377L318 367H929Q944 359 944 347Q944 336 930 328L602 327H274L264 319Q225 289 147 250Q148 249 165 241T210 217T264 181L274 173H930Q931 172 933 171T936 169T938 167T941 164T942 162T943 158T944 153",
      8657: "228 -179Q227 -180 226 -182T223 -186T221 -189T218 -192T214 -193T208 -194Q196 -194 189 -181L188 125V430L176 419Q122 369 59 338Q46 330 40 330Q38 330 31 337V350Q31 362 33 365T46 374Q60 381 77 390T128 426T190 484T247 567T292 677Q295 688 298 692Q302 694 305 694Q313 694 318 677Q334 619 363 568T420 485T481 427T532 391T564 374Q575 368 577 365T579 350V337Q572 330 570 330Q564 330 551 338Q487 370 435 419L423 430L422 125V-181Q409 -194 401 -194Q397 -194 394 -193T388 -189T385 -184T382 -180V-177V475L373 487Q331 541 305 602Q304 601 300 591T290 571T278 548T260 519T238 488L229 476L228 148V-179",
      8658: "580 514Q580 525 596 525Q601 525 604 525T609 525T613 524T615 523T617 520T619 517T622 512Q659 438 720 381T831 300T927 263Q944 258 944 250T935 239T898 228T840 204Q696 134 622 -12Q618 -21 615 -22T600 -24Q580 -24 580 -17Q580 -13 585 0Q620 69 671 123L681 133H70Q56 140 56 153Q56 168 72 173H725L735 181Q774 211 852 250Q851 251 834 259T789 283T735 319L725 327H72Q56 332 56 347Q56 360 70 367H681L671 377Q638 412 609 458T580 514",
      8659: "401 694Q412 694 422 681V375L423 70L435 81Q487 130 551 162Q564 170 570 170Q572 170 579 163V150Q579 138 577 135T564 126Q541 114 518 99T453 48T374 -46T318 -177Q313 -194 305 -194T293 -178T272 -119T225 -31Q158 70 46 126Q35 132 33 135T31 150V163Q38 170 40 170Q46 170 59 162Q122 131 176 81L188 70V375L189 681Q199 694 208 694Q219 694 228 680V352L229 25L238 12Q279 -42 305 -102Q344 -23 373 13L382 25V678Q387 692 401 694",
      8660: "308 524Q318 526 323 526Q340 526 340 514Q340 507 336 499Q326 476 314 454T292 417T274 391T260 374L255 368Q255 367 500 367Q744 367 744 368L739 374Q734 379 726 390T707 416T685 453T663 499Q658 511 658 515Q658 525 680 525Q687 524 690 523T695 519T701 507Q766 359 902 287Q921 276 939 269T961 259T966 250Q966 246 965 244T960 240T949 236T930 228T902 213Q763 137 701 -7Q697 -16 695 -19T690 -23T680 -25Q658 -25 658 -15Q658 -11 663 1Q673 24 685 46T707 83T725 109T739 126L744 132Q744 133 500 133Q255 133 255 132L260 126Q265 121 273 110T292 84T314 47T336 1Q341 -11 341 -15Q341 -25 319 -25Q312 -24 309 -23T304 -19T298 -7Q233 141 97 213Q83 221 70 227T51 235T41 239T35 243T34 250T35 256T40 261T51 265T70 273T97 287Q235 363 299 509Q305 522 308 524ZM792 319L783 327H216Q183 294 120 256L110 250L120 244Q173 212 207 181L216 173H783L792 181Q826 212 879 244L889 250L879 256Q826 288 792 319",
      8661: "290 755Q298 772 305 772T318 757T343 706T393 633Q431 588 473 558T545 515T579 497V484Q579 464 570 464Q564 464 550 470Q485 497 423 550L422 400V100L423 -50Q485 3 550 30Q565 36 570 36Q579 36 579 16V3Q575 -1 549 -12T480 -53T393 -132Q361 -172 342 -208T318 -258T305 -272T293 -258T268 -208T217 -132Q170 -80 128 -51T61 -12T31 3V16Q31 36 40 36Q46 36 61 30Q86 19 109 6T146 -18T173 -38T188 -50V550Q186 549 173 539T147 519T110 495T61 470Q46 464 40 464Q31 464 31 484V497Q34 500 63 513T135 557T217 633Q267 692 290 755ZM374 598Q363 610 351 625T332 651T316 676T305 695L294 676Q282 657 267 636T236 598L228 589V-89L236 -98Q247 -110 259 -125T278 -151T294 -176T305 -195L316 -176Q328 -157 343 -136T374 -98L382 -89V589L374 598",
      8666: "944 54Q942 44 929 36H372Q372 34 377 26T395 -4T422 -58Q442 -109 442 -110T408 -111H374L370 -100Q282 124 87 243L76 250L87 257Q284 377 370 600L374 611H408Q442 611 442 610Q423 550 381 480Q380 478 379 475T376 471T374 468T372 465V464H929Q942 456 944 446Q944 442 943 439T941 434T938 430T935 428T931 426T928 424H344L336 414Q277 336 200 277L191 270H560Q929 270 933 268Q944 262 944 250Q944 237 933 232Q929 230 560 230H191L200 223Q279 162 336 86L344 76H928Q929 76 931 75T934 73T938 70T941 66T943 61T944 54",
      8667: "56 250Q56 260 68 270H808L799 277Q720 338 663 414L655 424H363Q71 424 68 426Q55 432 55 444T68 462Q71 464 349 464H627Q627 466 622 474T604 504T577 558Q557 609 557 610T591 611H626L629 600Q717 376 912 257L923 250L912 243Q715 123 629 -100L626 -111H591Q557 -111 557 -110Q576 -50 618 20Q619 22 620 25T623 29T625 32T626 35L627 36H349Q71 36 68 38Q55 44 55 56T68 74Q71 76 363 76H655L663 86Q722 164 799 223L808 230H438L68 231Q56 236 56 250",
      8669: "76 230Q68 230 62 237T56 250Q56 257 63 264T91 291Q102 300 108 306L159 351Q168 356 177 351L218 316L303 239L353 195Q376 214 403 239L488 316L529 351Q538 356 546 351Q548 350 594 310L638 270H848L841 278Q813 309 792 344T763 396T755 416Q755 417 778 417H801Q817 367 856 323T943 250Q895 221 856 177T801 83H778Q755 83 755 84Q755 86 762 103T791 156T841 222L848 230H737Q625 230 622 232Q620 233 599 251T558 288L537 306Q537 305 451 228T362 149Q353 146 345 149Q341 150 255 227T169 306Q167 306 129 270Q123 265 115 257T102 245T93 237T84 232T76 230",
      8672: "292 419Q292 400 261 347T211 275H306H364Q400 275 411 271T422 250T411 230T366 225H306H211Q214 222 232 197T271 136T292 82Q292 71 285 68T262 64H250H241Q221 64 216 67T205 83Q186 127 153 167T78 230Q64 238 64 250Q64 258 69 263T82 272T106 288T139 318Q162 342 177 365T198 402T209 425T223 436Q224 437 252 437H258Q292 437 292 419ZM501 237T501 250T515 270H819Q834 262 834 250T819 230H515Q501 237 501 250ZM918 237T918 250T932 270H1236Q1251 262 1251 250T1236 230H932Q918 237 918 250",
      8674: "84 237T84 250T98 270H402Q417 262 417 250T402 230H98Q84 237 84 250ZM501 237T501 250T515 270H819Q834 262 834 250T819 230H515Q501 237 501 250ZM1022 417Q1022 437 1055 437H1067Q1090 437 1097 434T1109 417Q1128 373 1161 333T1236 270Q1251 261 1251 250Q1251 241 1244 236T1216 217T1175 182Q1149 155 1133 128T1109 85T1097 66Q1093 64 1065 64H1053Q1031 64 1025 72T1027 100Q1036 124 1049 147T1073 185T1091 210T1101 223L1103 225H1008H950Q914 225 903 229T892 250T903 270T948 275H1008H1103L1101 277Q1100 280 1091 291T1067 325T1039 374Q1022 408 1022 417",
      8704: "0 673Q0 684 7 689T20 694Q32 694 38 680T82 567L126 451H430L473 566Q483 593 494 622T512 668T519 685Q524 694 538 694Q556 692 556 674Q556 670 426 329T293 -15Q288 -22 278 -22T263 -15Q260 -11 131 328T0 673ZM414 410Q414 411 278 411T142 410L278 55L414 410",
      8705: "404 269Q412 269 418 267T428 261T435 253T441 245L444 240V172Q444 103 443 96Q440 81 431 65T403 27T344 -7T250 -21T156 -8T97 27T69 65T58 96Q56 103 56 413Q56 722 58 729Q74 822 215 845Q221 846 229 846H243Q282 846 290 845Q422 826 443 729Q444 722 444 653V586L442 583Q441 580 440 578T436 573T430 567T423 562T415 558T404 556Q377 556 367 583Q364 590 364 654V719Q363 721 360 726T355 733Q326 766 250 766H249Q235 766 219 765T174 752T137 719V107Q145 83 178 71T251 58H254Q340 58 364 107V172Q364 176 364 187T363 204Q363 269 404 269",
      8706: "202 508Q179 508 169 520T158 547Q158 557 164 577T185 624T230 675T301 710L333 715H345Q378 715 384 714Q447 703 489 661T549 568T566 457Q566 362 519 240T402 53Q321 -22 223 -22Q123 -22 73 56Q42 102 42 148V159Q42 276 129 370T322 465Q383 465 414 434T455 367L458 378Q478 461 478 515Q478 603 437 639T344 676Q266 676 223 612Q264 606 264 572Q264 547 246 528T202 508ZM430 306Q430 372 401 400T333 428Q270 428 222 382Q197 354 183 323T150 221Q132 149 132 116Q132 21 232 21Q244 21 250 22Q327 35 374 112Q389 137 409 196T430 306",
      8707: "56 661T56 674T70 694H487Q497 686 500 679V15Q497 10 487 1L279 0H70Q56 7 56 20T70 40H460V327H84Q70 334 70 347T84 367H460V654H70Q56 661 56 674",
      8709: "331 696Q335 708 339 722T345 744T350 759T357 769T367 772Q374 772 381 767T388 754Q388 746 377 712L366 673L378 661Q460 575 460 344Q460 281 456 234T432 126T373 27Q319 -22 250 -22Q214 -22 180 -7Q168 -3 168 -4L159 -33Q148 -71 142 -75Q138 -78 132 -78Q124 -78 118 -72T111 -60Q111 -52 122 -18L133 21L125 29Q39 111 39 344Q39 596 137 675Q187 716 251 716Q265 716 278 714T296 710T315 703T331 696ZM276 676Q264 679 246 679Q196 679 159 631Q134 597 128 536T121 356Q121 234 127 174T151 80L234 366Q253 430 275 506T308 618L318 654Q318 656 294 669L276 676ZM181 42Q207 16 250 16Q291 16 324 47Q354 78 366 136T378 356Q378 470 372 528T349 616L348 613Q348 611 264 326L181 42",
      8710: "51 0Q46 4 46 7Q46 9 215 357T388 709Q391 716 416 716Q439 716 444 709Q447 705 616 357T786 7Q786 4 781 0H51ZM507 344L384 596L137 92L383 91H630Q630 93 507 344",
      8711: "46 676Q46 679 51 683H781Q786 679 786 676Q786 674 617 326T444 -26Q439 -33 416 -33T388 -26Q385 -22 216 326T46 676ZM697 596Q697 597 445 597T193 596Q195 591 319 336T445 80L697 596",
      8712: "84 250Q84 372 166 450T360 539Q361 539 377 539T419 540T469 540H568Q583 532 583 520Q583 511 570 501L466 500Q355 499 329 494Q280 482 242 458T183 409T147 354T129 306T124 272V270H568Q583 262 583 250T568 230H124V228Q124 207 134 177T167 112T231 48T328 7Q355 1 466 0H570Q583 -10 583 -20Q583 -32 568 -40H471Q464 -40 446 -40T417 -41Q262 -41 172 45Q84 127 84 250",
      8713: "196 25Q84 109 84 250Q84 372 166 450T360 539Q361 539 375 539T413 540T460 540L547 707Q550 716 563 716Q570 716 575 712T581 703T583 696T505 540H568Q583 532 583 520Q583 511 570 501L484 500L366 270H568Q583 262 583 250T568 230H346L247 38Q284 16 328 7Q355 1 466 0H570Q583 -10 583 -20Q583 -32 568 -40H471Q464 -40 447 -40T419 -41Q304 -41 228 3Q117 -211 115 -212Q111 -215 104 -215T92 -212T86 -204T84 -197Q84 -190 89 -183L196 25ZM214 61L301 230H124V228Q124 196 147 147T214 61ZM321 270L440 500Q353 499 329 494Q280 482 242 458T183 409T147 354T129 306T124 272V270H321",
      8715: "83 520Q83 532 98 540H195Q202 540 220 540T249 541Q404 541 494 455Q582 374 582 250Q582 165 539 99T434 0T304 -39Q297 -40 195 -40H98Q83 -32 83 -20Q83 -10 96 0H200Q311 1 337 6Q369 14 401 28Q422 39 445 55Q484 85 508 127T537 191T542 228V230H98Q84 237 84 250T98 270H542V272Q542 280 539 295T527 336T497 391T445 445Q422 461 401 472Q386 479 374 483T347 491T325 495T298 498T273 499T239 500T200 500L96 501Q83 511 83 520",
      8717: "154 -1Q122 -1 112 3T102 26Q102 63 158 63H178Q192 64 206 65T228 66T240 68Q301 85 324 146L329 157H244Q158 157 153 161Q149 162 145 169T140 183Q140 201 158 215L167 221H256L344 223L349 237Q352 262 352 287Q352 308 351 315Q341 352 315 368T256 385Q231 385 206 376T166 356T149 346Q143 346 138 364T132 388Q132 396 147 406Q198 440 252 440Q291 440 318 435Q421 404 451 301Q456 288 456 248V234Q456 151 391 86Q330 25 240 3Q212 -1 154 -1",
      8719: "158 656Q147 684 131 694Q110 707 69 710H55V750H888V710H874Q840 708 820 698T795 678T786 656V-155Q798 -206 874 -210H888V-250H570V-210H584Q618 -208 638 -197T663 -178T673 -155V710H270V277L271 -155Q283 -206 359 -210H373V-250H55V-210H69Q103 -208 123 -197T148 -178T158 -155V656",
      8720: "158 656Q147 684 131 694Q110 707 69 710H55V750H373V710H359Q325 708 305 698T280 678T271 656L270 223V-210H673V656Q666 672 663 679T639 697T584 710H570V750H888V710H874Q840 708 820 698T795 678T786 656V-155Q798 -206 874 -210H888V-250H55V-210H69Q103 -208 123 -197T148 -178T158 -155V656",
      8721: "61 748Q64 750 489 750H913L954 640Q965 609 976 579T993 533T999 516H979L959 517Q936 579 886 621T777 682Q724 700 655 705T436 710H319Q183 710 183 709Q186 706 348 484T511 259Q517 250 513 244L490 216Q466 188 420 134T330 27L149 -187Q149 -188 362 -188Q388 -188 436 -188T506 -189Q679 -189 778 -162T936 -43Q946 -27 959 6H999L913 -249L489 -250Q65 -250 62 -248Q56 -246 56 -239Q56 -234 118 -161Q186 -81 245 -11L428 206Q428 207 242 462L57 717L56 728Q56 744 61 748",
      8722: "84 237T84 250T98 270H679Q694 262 694 250T679 230H98Q84 237 84 250",
      8723: "56 467T56 480T70 500H707Q722 492 722 480T707 460H409V187H707Q722 179 722 167Q722 154 707 147H409V0V-93Q409 -144 406 -155T389 -166Q376 -166 372 -155T368 -105Q368 -96 368 -62T369 -2V147H70Q56 154 56 167T70 187H369V460H70Q56 467 56 480",
      8724: "339 717Q339 739 354 752T388 766Q410 766 424 751T439 716T424 681T390 666Q369 666 354 681T339 717ZM57 237T57 250T71 270H369V425L370 581Q380 594 389 594Q402 594 409 579V270H707Q722 262 722 250T707 230H409V-79Q401 -93 391 -93H389H387Q375 -93 369 -79V230H71Q57 237 57 250",
      8725: "423 750Q432 750 438 744T444 730Q444 725 271 248T92 -240Q85 -250 75 -250Q68 -250 62 -245T56 -231Q56 -221 230 257T407 740Q411 750 423 750",
      8726: "56 731Q56 740 62 745T75 750Q85 750 92 740Q96 733 270 255T444 -231Q444 -239 438 -244T424 -250Q414 -250 407 -240Q404 -236 230 242T56 731",
      8727: "229 286Q216 420 216 436Q216 454 240 464Q241 464 245 464T251 465Q263 464 273 456T283 436Q283 419 277 356T270 286L328 328Q384 369 389 372T399 375Q412 375 423 365T435 338Q435 325 425 315Q420 312 357 282T289 250L355 219L425 184Q434 175 434 161Q434 146 425 136T401 125Q393 125 383 131T328 171L270 213Q283 79 283 63Q283 53 276 44T250 35Q231 35 224 44T216 63Q216 80 222 143T229 213L171 171Q115 130 110 127Q106 124 100 124Q87 124 76 134T64 161Q64 166 64 169T67 175T72 181T81 188T94 195T113 204T138 215T170 230T210 250L74 315Q65 324 65 338Q65 353 74 363T98 374Q106 374 116 368T171 328L229 286",
      8728: "55 251Q55 328 112 386T249 444T386 388T444 249Q444 171 388 113T250 55Q170 55 113 112T55 251ZM245 403Q188 403 142 361T96 250Q96 183 141 140T250 96Q284 96 313 109T354 135T375 160Q403 197 403 250Q403 313 360 358T245 403",
      8729: "55 251Q55 328 112 386T249 444T386 388T444 249Q444 171 388 113T250 55Q170 55 113 112T55 251",
      8730: "95 178Q89 178 81 186T72 200T103 230T169 280T207 309Q209 311 212 311H213Q219 311 227 294T281 177Q300 134 312 108L397 -77Q398 -77 501 136T707 565T814 786Q820 800 834 800Q841 800 846 794T853 782V776L620 293L385 -193Q381 -200 366 -200Q357 -200 354 -197Q352 -195 256 15L160 225L144 214Q129 202 113 190T95 178",
      8733: "56 124T56 216T107 375T238 442Q260 442 280 438T319 425T352 407T382 385T406 361T427 336T442 315T455 297T462 285L469 297Q555 442 679 442Q687 442 722 437V398H718Q710 400 694 400Q657 400 623 383T567 343T527 294T503 253T495 235Q495 231 520 192T554 143Q625 44 696 44Q717 44 719 46H722V-5Q695 -11 678 -11Q552 -11 457 141Q455 145 454 146L447 134Q362 -11 235 -11Q157 -11 107 56ZM93 213Q93 143 126 87T220 31Q258 31 292 48T349 88T389 137T413 178T421 196Q421 200 396 239T362 288Q322 345 288 366T213 387Q163 387 128 337T93 213",
      8734: "55 217Q55 305 111 373T254 442Q342 442 419 381Q457 350 493 303L507 284L514 294Q618 442 747 442Q833 442 888 374T944 214Q944 128 889 59T743 -11Q657 -11 580 50Q542 81 506 128L492 147L485 137Q381 -11 252 -11Q166 -11 111 57T55 217ZM907 217Q907 285 869 341T761 397Q740 397 720 392T682 378T648 359T619 335T594 310T574 285T559 263T548 246L543 238L574 198Q605 158 622 138T664 94T714 61T765 51Q827 51 867 100T907 217ZM92 214Q92 145 131 89T239 33Q357 33 456 193L425 233Q364 312 334 337Q285 380 233 380Q171 380 132 331T92 214",
      8736: "71 0L68 2Q65 3 63 5T58 11T55 20Q55 22 57 28Q67 43 346 361Q397 420 474 508Q595 648 616 671T647 694T661 688T666 674Q666 668 663 663Q662 662 627 622T524 503T390 350L120 41L386 40H653Q666 30 666 20Q666 8 651 0H71",
      8737: "71 0L68 2Q65 3 63 5T58 11T55 20Q55 22 57 28Q64 38 348 373T638 712Q644 714 646 714Q653 714 659 709T666 694V693Q666 687 633 647Q619 631 576 580Q528 524 495 485Q336 296 329 289Q328 288 348 264T395 182T433 54L434 40H651Q666 32 666 20T651 0H436Q431 -20 416 -20Q400 -20 396 -4V0H71ZM394 40Q394 51 389 76T366 149T319 234L302 256L119 41L256 40H394",
      8738: "666 -32Q666 -51 646 -51Q639 -51 365 85L75 228Q55 238 55 250Q55 257 59 262T68 268L72 270L611 536Q642 551 647 551T659 547T666 532Q666 521 657 515L525 449Q525 448 535 424T556 352T566 250T556 148T536 77T525 51L657 -15Q666 -21 666 -32ZM526 250Q526 297 517 342T499 409T488 431Q487 431 304 341T121 250T304 159T488 69Q526 143 526 250",
      8739: "139 -249H137Q125 -249 119 -235V251L120 737Q130 750 139 750Q152 750 159 735V-235Q151 -249 141 -249H139",
      8740: "118 737Q131 750 138 750L151 746L158 739V579L160 421L213 470Q269 519 276 519Q284 519 290 513T296 499V498Q296 493 291 488T244 445Q225 428 213 417L158 368V-239Q143 -252 136 -252L124 -248L120 -241L118 44V328L62 279Q4 231 0 230Q-8 230 -14 236T-20 250Q-20 257 -11 265T62 332L118 384V737",
      8741: "133 736Q138 750 153 750Q164 750 170 739Q172 735 172 250T170 -239Q164 -250 152 -250Q144 -250 138 -244L137 -243Q133 -241 133 -179T132 250Q132 731 133 736ZM329 739Q334 750 346 750Q353 750 361 744L362 743Q366 741 366 679T367 250T367 -178T362 -243L361 -244Q355 -250 347 -250Q335 -250 329 -239Q327 -235 327 250T329 739",
      8742: "131 737Q134 739 138 743T144 748T151 750T171 737V199L327 357V737Q340 750 347 750Q351 750 353 749T360 743T367 737V397L429 457Q493 518 498 519Q506 519 512 512T518 500Q518 489 442 417L367 339V-237Q352 -250 346 -250L333 -243L327 -237V301L171 143V-237Q156 -250 151 -250T131 -237V101L69 41Q24 -3 15 -12T0 -21Q-8 -21 -14 -14T-20 -2Q-20 5 -7 19T56 81L131 159V737",
      8743: "318 591Q325 598 333 598Q344 598 348 591Q349 590 414 445T545 151T611 -4Q609 -22 591 -22Q588 -22 586 -21T581 -20T577 -17T575 -13T572 -9T570 -4L333 528L96 -4Q87 -20 80 -21Q78 -22 75 -22Q57 -22 55 -4Q55 2 120 150T251 444T318 591",
      8744: "55 580Q56 587 61 592T75 598Q86 598 96 580L333 48L570 580Q579 596 586 597Q588 598 591 598Q609 598 611 580Q611 574 546 426T415 132T348 -15Q343 -22 333 -22T318 -15Q317 -14 252 131T121 425T55 580",
      8745: "88 -21T75 -21T55 -7V200Q55 231 55 280Q56 414 60 428Q61 430 61 431Q77 500 152 549T332 598Q443 598 522 544T610 405Q611 399 611 194V-7Q604 -22 591 -22Q582 -22 572 -9L570 405Q563 433 556 449T529 485Q498 519 445 538T334 558Q251 558 179 518T96 401Q95 396 95 193V-7Q88 -21 75 -21",
      8746: "591 598H592Q604 598 611 583V376Q611 345 611 296Q610 162 606 148Q605 146 605 145Q586 68 507 23T333 -22Q268 -22 209 -1T106 66T56 173Q55 180 55 384L56 585Q66 598 75 598Q85 598 95 585V378L96 172L98 162Q112 95 181 57T332 18Q415 18 487 58T570 175Q571 180 571 383V583Q579 598 591 598",
      8747: "151 -112Q151 -150 106 -161Q106 -165 114 -172T134 -179Q155 -179 170 -146Q181 -120 188 -64T206 101T232 310Q256 472 277 567Q308 716 392 716Q434 716 453 681T472 613Q472 590 458 577T424 564Q404 564 390 578T376 612Q376 650 421 661Q421 663 418 667T407 675T393 679Q387 679 380 675Q360 665 350 619T326 438Q302 190 253 -57Q235 -147 201 -186Q174 -213 138 -216Q93 -216 74 -181T55 -113Q55 -91 69 -78T103 -64Q123 -64 137 -78T151 -112",
      8748: "113 -244Q113 -246 119 -251T139 -263T167 -269Q186 -269 199 -260Q220 -247 232 -218T251 -133T262 -15T276 155T297 367Q300 390 305 438T314 512T325 580T340 647T361 703T390 751T428 784T479 804Q481 804 488 804T501 805Q552 802 581 769T610 695Q610 669 594 657T561 645Q542 645 527 658T512 694Q512 705 516 714T526 729T538 737T548 742L552 743Q552 745 545 751T525 762T498 768Q475 768 460 756T434 716T418 652T407 559T398 444T387 300T369 133Q349 -38 337 -102T303 -207Q256 -306 169 -306Q119 -306 87 -272T55 -196Q55 -170 71 -158T104 -146Q123 -146 138 -159T153 -195Q153 -206 149 -215T139 -230T127 -238T117 -242L113 -244ZM460 -244Q460 -246 466 -251T486 -263T514 -269Q532 -269 546 -260Q567 -247 579 -218T598 -133T609 -15T623 155T644 367Q647 390 652 438T661 512T672 580T687 647T708 703T737 751T775 784T826 804Q828 804 835 804T848 805Q899 802 928 769T957 695Q957 669 941 657T908 645Q889 645 874 658T859 694Q859 705 863 714T873 729T885 737T895 742L899 743Q899 745 892 751T872 762T845 768Q822 768 807 756T781 716T765 652T754 559T745 444T734 300T716 133Q696 -38 684 -102T650 -207Q603 -306 516 -306Q466 -306 434 -272T402 -196Q402 -170 418 -158T451 -146Q470 -146 485 -159T500 -195Q500 -206 496 -215T486 -230T474 -238T464 -242L460 -244",
      8749: "113 -244Q113 -246 119 -251T139 -263T167 -269Q186 -269 199 -260Q220 -247 232 -218T251 -133T262 -15T276 155T297 367Q300 390 305 438T314 512T325 580T340 647T361 703T390 751T428 784T479 804Q481 804 488 804T501 805Q552 802 581 769T610 695Q610 669 594 657T561 645Q542 645 527 658T512 694Q512 705 516 714T526 729T538 737T548 742L552 743Q552 745 545 751T525 762T498 768Q475 768 460 756T434 716T418 652T407 559T398 444T387 300T369 133Q349 -38 337 -102T303 -207Q256 -306 169 -306Q119 -306 87 -272T55 -196Q55 -170 71 -158T104 -146Q123 -146 138 -159T153 -195Q153 -206 149 -215T139 -230T127 -238T117 -242L113 -244ZM460 -244Q460 -246 466 -251T486 -263T514 -269Q532 -269 546 -260Q567 -247 579 -218T598 -133T609 -15T623 155T644 367Q647 390 652 438T661 512T672 580T687 647T708 703T737 751T775 784T826 804Q828 804 835 804T848 805Q899 802 928 769T957 695Q957 669 941 657T908 645Q889 645 874 658T859 694Q859 705 863 714T873 729T885 737T895 742L899 743Q899 745 892 751T872 762T845 768Q822 768 807 756T781 716T765 652T754 559T745 444T734 300T716 133Q696 -38 684 -102T650 -207Q603 -306 516 -306Q466 -306 434 -272T402 -196Q402 -170 418 -158T451 -146Q470 -146 485 -159T500 -195Q500 -206 496 -215T486 -230T474 -238T464 -242L460 -244ZM807 -244Q807 -246 813 -251T833 -263T861 -269Q880 -269 893 -260Q914 -247 926 -218T945 -133T956 -15T970 155T991 367Q994 390 999 438T1008 512T1019 580T1034 647T1055 703T1084 751T1122 784T1173 804Q1175 804 1182 804T1195 805Q1246 802 1275 769T1304 695Q1304 669 1288 657T1255 645Q1236 645 1221 658T1206 694Q1206 705 1210 714T1220 729T1232 737T1242 742L1246 743Q1246 745 1239 751T1219 762T1192 768Q1169 768 1154 756T1128 716T1112 652T1101 559T1092 444T1081 300T1063 133Q1043 -38 1031 -102T997 -207Q950 -306 863 -306Q813 -306 781 -272T749 -196Q749 -170 765 -158T798 -146Q817 -146 832 -159T847 -195Q847 -206 843 -215T833 -230T821 -238T811 -242L807 -244",
      8750: "269 74L256 80Q244 85 227 97T191 128T161 179T148 250Q148 332 199 379T302 433L306 434L307 444Q309 456 313 495T321 553T331 607T345 664T365 712T393 756T431 785T479 804Q481 804 488 804T501 805Q552 802 581 769T610 695Q610 669 594 657T561 645Q542 645 527 658T512 694Q512 705 516 714T526 729T538 737T548 742L552 743Q552 745 545 751T525 762T498 768Q471 768 454 752T427 693T414 626T406 536Q405 530 405 527L397 425L404 422Q410 419 421 413T445 399T470 376T494 345T511 303T518 250Q518 205 502 169T460 112T410 80T364 66L360 65L359 55Q357 38 353 4T346 -43T340 -81T333 -118T326 -148T316 -179T303 -207Q256 -306 169 -306Q119 -306 87 -272T55 -196Q55 -170 71 -158T104 -146Q123 -146 138 -159T153 -195Q153 -206 149 -215T139 -230T127 -238T117 -242L113 -244Q113 -246 119 -251T139 -263T167 -269Q186 -269 199 -260Q231 -241 242 -183T266 33L269 74ZM272 122Q272 156 300 391Q300 392 299 392Q287 392 263 379T213 331T187 249Q187 211 205 180T239 137T272 116V122ZM366 107Q378 107 402 119T453 167T479 249Q479 340 394 383V377Q394 375 394 374T393 371T393 366T392 357T391 342T389 321T386 291T382 251T377 199T369 133Q366 112 366 107",
      8756: "273 411Q273 437 291 454T334 471Q358 471 375 454T393 411T376 368T333 351Q307 351 290 368T273 411ZM84 38Q110 38 126 21T143 -22Q143 -46 127 -64T83 -82Q57 -82 41 -65T24 -22Q24 4 41 21T84 38ZM524 -22Q524 4 541 21T584 38Q608 38 625 21T643 -22Q643 -45 627 -63T583 -82Q557 -82 541 -65T524 -22",
      8757: "23 411Q23 437 41 454T84 471Q108 471 125 454T143 411T126 368T83 351Q57 351 40 368T23 411ZM523 411Q523 437 541 454T584 471Q608 471 625 454T643 411T626 368T583 351Q557 351 540 368T523 411ZM274 -22Q274 4 291 21T334 38Q356 38 374 22T392 -22T375 -65T333 -82Q307 -82 291 -65T274 -22",
      8764: "55 166Q55 241 101 304T222 367Q260 367 296 349T362 304T421 252T484 208T554 189Q616 189 655 236T694 338Q694 350 698 358T708 367Q722 367 722 334Q722 260 677 197T562 134H554Q517 134 481 152T414 196T355 248T292 293T223 311Q179 311 145 286Q109 257 96 218T80 156T69 133Q55 133 55 166",
      8765: "222 133Q147 133 102 197T56 335Q56 362 66 365Q71 369 77 364Q83 356 84 335T90 298Q102 254 137 222T223 189Q258 189 292 206T355 250T413 301T477 346T550 367Q628 367 673 309T722 171Q722 133 708 133Q703 133 699 141T694 162Q694 220 655 265T555 311Q519 311 485 293T421 248T363 196T298 152T222 133",
      8768: "55 569Q55 583 83 583Q122 583 151 565T194 519T215 464T222 411Q222 360 194 304T139 193T111 89Q111 38 134 -7T195 -55Q222 -57 222 -69Q222 -83 189 -83Q130 -83 93 -33T55 90Q55 130 72 174T110 252T148 328T166 411Q166 462 144 507T83 555Q55 556 55 569",
      8769: "220 366Q258 366 297 347T361 308T391 288Q394 288 464 370Q494 407 510 425T535 454T546 465T552 467H553Q560 467 566 461T573 448Q573 439 499 350Q424 266 424 261Q424 259 442 247T492 222T554 209Q607 209 646 243Q671 268 680 295T690 341T702 366Q719 366 719 314Q716 265 695 226Q682 199 664 179Q614 132 555 132Q517 132 477 151T412 190T383 210T347 172T278 89T233 37Q228 32 220 32Q210 32 206 38T201 48Q201 57 266 137Q272 144 275 148Q351 231 351 237Q351 239 333 251T283 276T221 289Q159 289 123 248T86 166Q86 156 82 145T73 132Q55 132 55 172Q55 220 79 272Q95 301 111 319Q161 366 220 366",
      8770: "55 439T55 443T56 449T62 456T68 463H706Q720 449 720 443T706 423H68Q55 439 55 443ZM56 72Q56 112 73 152T130 225T224 257Q259 257 294 240T360 199T419 149T484 107T553 90Q603 90 643 125T691 223Q693 257 704 257Q717 257 717 221Q717 147 671 91T554 34Q517 34 481 51T414 93T355 142T291 184T222 201Q172 201 131 167T84 67Q81 34 71 34Q56 37 56 72",
      8771: "55 283Q55 356 103 409T217 463Q262 463 297 447T395 382Q431 355 446 344T493 320T554 307H558Q613 307 652 344T694 433Q694 464 708 464T722 432Q722 356 673 304T564 251H554Q510 251 465 275T387 329T310 382T223 407H219Q164 407 122 367Q91 333 85 295T76 253T69 250Q55 250 55 283ZM56 56Q56 71 72 76H706Q722 70 722 56Q722 44 707 36H70Q56 43 56 56",
      8773: "55 388Q55 463 101 526T222 589Q260 589 296 571T362 526T421 474T484 430T554 411Q616 411 655 458T694 560Q694 572 698 580T708 589Q722 589 722 556Q722 482 677 419T562 356H554Q517 356 481 374T414 418T355 471T292 515T223 533Q179 533 145 508Q109 479 96 440T80 378T69 355Q55 355 55 388ZM56 236Q56 249 70 256H707Q722 248 722 236Q722 225 708 217L390 216H72Q56 221 56 236ZM56 42Q56 57 72 62H708Q722 52 722 42Q722 30 707 22H70Q56 29 56 42",
      8775: "55 417Q55 479 101 528T222 578Q259 578 294 564T393 507Q413 493 434 480T469 460T484 454L537 549Q587 639 595 647Q600 652 607 652Q615 652 621 647T628 634Q628 625 575 536Q524 446 524 443Q527 440 555 440Q603 440 644 469T691 547Q694 578 706 578T718 556Q718 555 718 551T717 545Q717 488 684 445T595 387Q582 384 558 384Q530 384 508 389L493 394L404 238L557 236H708Q720 224 720 217T706 196H379L291 43L499 41H708Q720 29 720 21T706 1H268L226 -71Q186 -143 179 -148Q173 -155 165 -155T152 -150T146 -137Q146 -133 184 -64L222 1H144L66 3L59 7Q54 14 54 20Q54 29 66 41H246L333 194Q333 196 202 196H68Q55 211 55 218T66 236H213L357 238L457 409L437 421Q432 423 393 450T307 500T222 523Q171 523 129 491T84 414Q82 383 70 383Q55 383 55 417",
      8776: "55 319Q55 360 72 393T114 444T163 472T205 482Q207 482 213 482T223 483Q262 483 296 468T393 413L443 381Q502 346 553 346Q609 346 649 375T694 454Q694 465 698 474T708 483Q722 483 722 452Q722 386 675 338T555 289Q514 289 468 310T388 357T308 404T224 426Q164 426 125 393T83 318Q81 289 69 289Q55 289 55 319ZM55 85Q55 126 72 159T114 210T163 238T205 248Q207 248 213 248T223 249Q262 249 296 234T393 179L443 147Q502 112 553 112Q609 112 649 141T694 220Q694 249 708 249T722 217Q722 153 675 104T555 55Q514 55 468 76T388 123T308 170T224 192Q164 192 125 159T83 84Q80 55 69 55Q55 55 55 85",
      8778: "220 523Q163 523 124 486T84 412Q81 383 69 383Q56 383 56 413Q56 441 67 470Q78 508 111 537T187 575Q203 579 219 579Q248 579 271 572Q304 565 393 508Q498 439 551 439Q620 439 662 486Q688 512 693 557Q693 565 697 572T707 579Q719 579 719 548Q719 483 673 434T550 384Q512 384 467 405T386 453T305 501T220 523ZM222 288Q164 288 124 251T84 177Q81 148 69 148Q56 148 56 178Q56 206 67 235Q78 274 111 302T187 339Q198 343 220 343Q244 343 259 341T308 322T393 272Q496 203 553 203Q612 203 651 241T691 312Q693 343 705 343Q719 343 719 313Q719 245 673 199Q626 148 552 148Q513 148 467 170T385 218T304 266T222 288ZM51 -19Q51 -6 62 -1H387Q713 -1 715 -3Q725 -10 725 -20Q725 -27 718 -34Q714 -38 672 -38T387 -39H62Q51 -25 51 -19",
      8781: "55 464Q55 471 60 477T74 484Q80 484 108 464T172 420T268 376T389 356Q436 356 483 368T566 399T630 436T675 467T695 482Q701 484 703 484Q711 484 716 478T722 464Q722 454 707 442Q550 316 389 316Q338 316 286 329T195 362T124 402T76 437T57 456Q55 462 55 464ZM57 45Q66 58 109 88T230 151T381 183Q438 183 494 168T587 135T658 94T703 61T720 45Q722 39 722 36Q722 28 717 22T703 16Q697 16 669 36T606 80T510 124T389 144Q341 144 294 132T211 101T147 64T102 33T82 18Q76 16 74 16Q66 16 61 22T55 36Q55 39 57 45",
      8782: "245 367Q251 415 288 453T392 492Q445 492 485 456T532 367H707Q722 359 722 347Q722 334 711 331T665 327H608H509Q500 332 498 336Q496 338 493 363T472 411Q443 451 389 451H387Q335 451 305 411Q290 392 287 374T282 344T268 327H72Q56 332 56 347Q56 360 70 367H245ZM56 153Q56 168 72 173H268Q277 168 279 164Q281 162 284 137T305 89Q334 49 389 49H391Q442 49 472 89Q487 108 490 126T495 156T509 173H608H666Q701 173 711 170T722 153T707 133H532Q526 81 486 45T389 8Q331 8 291 45T245 133H70Q56 140 56 153",
      8783: "245 367Q251 415 288 453T392 492Q445 492 485 456T532 367H707Q722 359 722 347Q722 334 711 331T665 327H608H509Q500 332 498 336Q496 338 493 363T472 411Q443 451 389 451H387Q335 451 305 411Q290 392 287 374T282 344T268 327H72Q56 332 56 347Q56 360 70 367H245ZM56 153Q56 168 72 173H708Q722 163 722 153Q722 140 707 133H70Q56 140 56 153",
      8784: "56 347Q56 360 70 367H707Q722 359 722 347Q722 336 708 328L390 327H72Q56 332 56 347ZM56 153Q56 168 72 173H708Q722 163 722 153Q722 140 707 133H70Q56 140 56 153ZM329 610Q329 634 346 652T389 670Q413 670 431 654T450 611Q450 586 433 568T390 550T347 567T329 610",
      8785: "421 474T389 474T339 493T321 541Q321 566 337 587T391 609Q456 602 456 541Q456 512 439 493ZM56 347Q56 360 70 367H707Q722 359 722 347Q722 336 708 328L390 327H72Q56 332 56 347ZM56 153Q56 168 72 173H708Q722 163 722 153Q722 140 707 133H70Q56 140 56 153ZM421 -108T389 -108T339 -89T321 -41Q321 -16 337 5T391 27Q456 20 456 -41Q456 -70 439 -89",
      8786: "15 541Q15 569 33 585T75 601T117 585T135 541Q135 514 118 498T75 481T32 498T15 541ZM56 347Q56 360 70 367H707Q722 359 722 347Q722 336 708 328L390 327H72Q56 332 56 347ZM56 153Q56 168 72 173H708Q722 163 722 153Q722 140 707 133H70Q56 140 56 153ZM642 -41Q642 -17 658 0T702 18Q726 18 744 3T762 -41Q762 -67 745 -84T702 -101Q676 -101 659 -85T642 -41",
      8787: "642 541Q642 569 660 585T702 601T744 585T762 541Q762 515 745 498T702 481Q676 481 659 497T642 541ZM56 347Q56 360 70 367H707Q722 359 722 347Q722 336 708 328L390 327H72Q56 332 56 347ZM56 153Q56 168 72 173H708Q722 163 722 153Q722 140 707 133H70Q56 140 56 153ZM14 -41Q14 -15 31 1T74 18Q101 18 118 0T135 -41Q135 -64 118 -83T75 -102Q51 -102 33 -85T14 -41",
      8790: "56 347Q56 360 70 367H707Q722 359 722 347Q722 334 711 331T658 327H586H465L472 318Q496 288 496 250T472 182L465 173H586H663Q700 173 711 170T722 153T707 133H70Q56 140 56 153Q56 168 72 173H312L305 182Q281 212 281 250T305 318L312 327H72Q56 332 56 347ZM473 250Q473 265 472 273T460 297T428 327H349Q328 313 318 298T306 273T304 250Q304 235 305 227T317 203T349 173H428Q449 187 459 202T471 227T473 250",
      8791: "279 612Q279 656 310 688T388 721Q433 721 465 689T498 612Q498 573 470 538T389 503Q336 503 308 538T279 612ZM458 614Q458 637 452 651T433 672T411 679T383 680T352 675T333 664T324 647T321 629T320 611Q320 593 321 584T332 562T359 545Q366 543 389 543H391Q406 543 414 544T435 552T452 573T458 614ZM56 347Q56 360 70 367H707Q722 359 722 347Q722 336 708 328L390 327H72Q56 332 56 347ZM56 153Q56 168 72 173H708Q722 163 722 153Q722 140 707 133H70Q56 140 56 153",
      8796: "192 482H190Q187 483 185 484T181 488T177 493T175 501Q175 506 178 512Q184 523 278 687T375 853Q379 857 383 857Q385 857 387 858T390 859Q397 859 403 853Q405 851 499 687T600 512Q603 506 603 501Q603 488 587 482H192ZM548 523L389 798Q388 798 309 661T230 523T389 522T548 523ZM56 347Q56 360 70 367H708Q723 359 723 347Q723 336 709 328L390 327H72Q56 332 56 347ZM56 153Q56 168 72 173H709Q723 163 723 153Q723 140 708 133H70Q56 140 56 153",
      8800: "166 -215T159 -215T147 -212T141 -204T139 -197Q139 -190 144 -183L306 133H70Q56 140 56 153Q56 168 72 173H327L406 327H72Q56 332 56 347Q56 360 70 367H426Q597 702 602 707Q605 716 618 716Q625 716 630 712T636 703T638 696Q638 692 471 367H707Q722 359 722 347Q722 336 708 328L451 327L371 173H708Q722 163 722 153Q722 140 707 133H351Q175 -210 170 -212Q166 -215 159 -215",
      8801: "56 444Q56 457 70 464H707Q722 456 722 444Q722 430 706 424H72Q56 429 56 444ZM56 237T56 250T70 270H707Q722 262 722 250T707 230H70Q56 237 56 250ZM56 56Q56 71 72 76H706Q722 70 722 56Q722 44 707 36H70Q56 43 56 56",
      8804: "674 636Q682 636 688 630T694 615T687 601Q686 600 417 472L151 346L399 228Q687 92 691 87Q694 81 694 76Q694 58 676 56H670L382 192Q92 329 90 331Q83 336 83 348Q84 359 96 365Q104 369 382 500T665 634Q669 636 674 636ZM84 -118Q84 -108 99 -98H678Q694 -104 694 -118Q694 -130 679 -138H98Q84 -131 84 -118",
      8805: "83 616Q83 624 89 630T99 636Q107 636 253 568T543 431T687 361Q694 356 694 346T687 331Q685 329 395 192L107 56H101Q83 58 83 76Q83 77 83 79Q82 86 98 95Q117 105 248 167Q326 204 378 228L626 346L360 472Q291 505 200 548Q112 589 98 597T83 616ZM84 -118Q84 -108 99 -98H678Q694 -104 694 -118Q694 -130 679 -138H98Q84 -131 84 -118",
      8806: "674 753Q682 753 688 747T694 732T687 718Q686 717 417 589L151 463L399 345Q687 209 691 204Q694 198 694 193Q694 175 676 173H670L382 309Q92 446 90 448Q83 453 83 465Q84 476 96 482Q104 486 382 617T665 751Q669 753 674 753ZM84 39Q84 49 99 59H678Q694 53 694 39Q694 26 679 19H98Q84 26 84 39ZM83 -157Q83 -153 84 -150T86 -145T89 -141T92 -139T96 -137T99 -135H678Q694 -146 694 -155Q694 -168 679 -175H98Q84 -168 83 -157",
      8807: "83 733Q83 741 89 747T99 753Q107 753 253 685T543 548T687 478Q694 473 694 463T687 448Q685 446 395 309L107 173H101Q83 175 83 193Q83 194 83 196Q82 203 98 212Q117 222 248 284Q326 321 378 345L626 463L360 589Q291 622 200 665Q112 706 98 714T83 733ZM84 39Q84 49 99 59H678Q694 53 694 39Q694 26 679 19H98Q84 26 84 39ZM83 -157Q83 -153 84 -150T86 -145T89 -141T92 -139T96 -137T99 -135H678Q694 -146 694 -155Q694 -168 679 -175H98Q84 -168 83 -157",
      8808: "86 472Q93 477 381 614T673 752Q680 752 686 746T693 732T689 721Q686 715 418 590L151 461L418 332Q684 207 689 201Q693 195 693 190Q693 183 687 177T675 170Q668 170 380 307T86 450Q82 454 82 461Q82 467 86 472ZM82 33Q82 37 83 40T89 47T95 54H473L520 105Q569 156 571 156Q573 157 578 157Q586 157 592 151T598 136Q598 130 562 92L526 56L604 54H682Q693 43 693 35Q693 31 692 28T686 21T680 14H489L342 -139L513 -142H682Q693 -148 693 -160Q693 -167 680 -182H304L258 -230Q248 -240 237 -251T221 -268T211 -278T203 -284T197 -286Q189 -286 184 -280T178 -264Q178 -257 213 -219L249 -182H171L93 -179L86 -175Q82 -170 82 -163Q82 -155 95 -142H289L360 -64L433 14H262L93 16Q82 23 82 33",
      8809: "89 745Q95 752 100 752Q106 752 394 615T689 472Q693 468 693 461T689 450Q684 445 396 308T100 170Q95 170 89 176T82 190Q82 195 86 201Q91 208 358 332L624 461L358 590Q90 715 86 721Q82 725 82 731Q82 739 89 745ZM82 33Q82 37 83 40T89 47T95 54H473L520 105Q569 156 571 156Q573 157 578 157Q586 157 592 151T598 136Q598 130 562 92L526 56L604 54H682Q693 43 693 35Q693 31 692 28T686 21T680 14H489L342 -139L513 -142H682Q693 -148 693 -160Q693 -167 680 -182H304L258 -230Q248 -240 237 -251T221 -268T211 -278T203 -284T197 -286Q189 -286 184 -280T178 -264Q178 -257 213 -219L249 -182H171L93 -179L86 -175Q82 -170 82 -163Q82 -155 95 -142H289L360 -64L433 14H262L93 16Q82 23 82 33",
      8810: "639 -48Q639 -54 634 -60T619 -67H618Q612 -67 536 -26Q430 33 329 88Q61 235 59 239Q56 243 56 250T59 261Q62 266 336 415T615 567L619 568Q622 567 625 567Q639 562 639 548Q639 540 633 534Q632 532 374 391L117 250L374 109Q632 -32 633 -34Q639 -40 639 -48ZM944 -48Q944 -54 939 -60T924 -67H923Q917 -67 841 -26Q735 33 634 88Q366 235 364 239Q361 243 361 250T364 261Q367 266 641 415T920 567L924 568Q927 567 930 567Q944 562 944 548Q944 540 938 534Q937 532 679 391L422 250L679 109Q937 -32 938 -34Q944 -40 944 -48",
      8811: "55 539T55 547T60 561T74 567Q81 567 207 498Q297 449 365 412Q633 265 636 261Q639 255 639 250Q639 241 626 232Q614 224 365 88Q83 -65 79 -66Q76 -67 73 -67Q65 -67 60 -61T55 -47Q55 -39 61 -33Q62 -33 95 -15T193 39T320 109L321 110H322L323 111H324L325 112L326 113H327L329 114H330L331 115H332L333 116L334 117H335L336 118H337L338 119H339L340 120L341 121H342L343 122H344L345 123H346L347 124L348 125H349L351 126H352L353 127H354L355 128L356 129H357L358 130H359L360 131H361L362 132L363 133H364L365 134H366L367 135H368L369 136H370L371 137L372 138H373L374 139H375L376 140L378 141L576 251Q63 530 62 533Q55 539 55 547ZM360 539T360 547T365 561T379 567Q386 567 512 498Q602 449 670 412Q938 265 941 261Q944 255 944 250Q944 241 931 232Q919 224 670 88Q388 -65 384 -66Q381 -67 378 -67Q370 -67 365 -61T360 -47Q360 -39 366 -33Q367 -33 400 -15T498 39T625 109L626 110H627L628 111H629L630 112L631 113H632L634 114H635L636 115H637L638 116L639 117H640L641 118H642L643 119H644L645 120L646 121H647L648 122H649L650 123H651L652 124L653 125H654L656 126H657L658 127H659L660 128L661 129H662L663 130H664L665 131H666L667 132L668 133H669L670 134H671L672 135H673L674 136H675L676 137L677 138H678L679 139H680L681 140L683 141L881 251Q368 530 367 533Q360 539 360 547",
      8812: "104 730Q104 749 123 749Q130 749 138 745Q186 717 237 671L250 659L261 670Q297 703 332 726T375 750T389 744T395 730Q395 721 390 717T364 699T322 668Q290 641 283 632Q280 628 281 627T293 612Q425 454 425 250Q425 144 388 51T293 -112Q282 -125 281 -126T283 -132Q306 -162 379 -209Q395 -219 395 -230Q395 -238 389 -244T375 -250T335 -228T262 -171L250 -159L238 -170Q202 -203 167 -226T124 -250T110 -244T104 -230Q104 -219 121 -209Q199 -156 216 -132Q219 -128 218 -127T206 -112Q74 46 74 250T206 612Q217 625 218 626T216 632Q199 656 121 709Q104 719 104 730ZM249 -94Q364 61 364 250Q364 430 265 574Q253 590 249 594L242 583Q134 439 134 250Q134 114 192 -1Q212 -44 242 -83L249 -94",
      8814: "693 -14T693 -20T687 -33T675 -41Q667 -41 506 37L344 112Q342 112 262 -46Q184 -196 176 -205Q172 -209 168 -209T162 -208Q155 -208 151 -203T146 -190Q146 -178 171 -137Q193 -91 251 21L306 132L198 183Q142 208 118 220T88 238T82 249Q82 253 86 261Q92 267 278 357L464 443L529 572Q561 637 577 667T597 703T607 708Q615 708 622 702T629 688Q629 680 575 579L524 474Q524 473 545 482T598 508Q666 541 673 541T686 535T693 521Q693 512 679 504T589 459L493 414L360 150Q366 148 378 142T431 116T529 70Q686 -8 689 -10Q693 -14 693 -20ZM380 277L433 383Q432 385 292 319T151 250T237 209T324 170L380 277",
      8815: "82 514T82 520T89 533T100 541Q106 541 271 463Q434 386 435 386L515 543Q593 699 600 706Q604 708 607 708Q615 708 622 702T629 688T549 526Q509 445 491 407T473 368Q522 343 580 317Q636 291 660 278T688 261T693 250V249Q693 241 681 234T580 184Q533 161 502 146Q317 59 315 59Q312 56 246 -74Q197 -170 186 -189T168 -209Q164 -209 162 -208Q155 -208 151 -203T146 -190Q146 -187 200 -79L253 28L218 11Q182 -6 144 -23T100 -41Q95 -41 89 -35T82 -21Q82 -12 96 -4T186 41L284 88L349 217Q377 273 395 311T413 350Q413 351 253 428Q101 498 86 510Q82 514 82 520ZM624 250Q461 330 455 330Q454 331 453 329T448 321T441 308T430 287T416 259T398 223L342 114L624 250",
      8816: "82 -124Q82 -120 83 -117T89 -110T95 -103H220L284 50Q346 204 344 206L218 268Q153 297 123 313T87 333T82 344T86 355Q104 369 291 455Q491 552 491 553L542 673Q581 767 590 784T609 801Q616 801 622 795T629 781Q629 773 586 677Q546 581 546 577L609 606Q669 635 673 635Q680 635 686 629T693 615Q693 610 692 608T670 593T604 561L524 521L400 226L542 157Q617 123 649 107T687 85T694 72Q694 66 690 60T679 54Q664 54 526 121Q513 127 495 136T464 150T438 162T416 173T399 180T388 185L384 186Q383 186 322 41L262 -103H680Q682 -105 684 -108T688 -113T691 -118T693 -124Q693 -134 682 -141L464 -143H246L213 -219Q182 -292 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -282 174 -213T202 -143H146L93 -141Q82 -134 82 -124ZM418 370L466 495Q464 495 308 420T151 344T204 317T311 267T364 244Q364 247 418 370",
      8817: "97 54Q82 54 82 72Q82 79 86 84Q95 91 222 153L351 215L398 324L442 433L258 519Q95 597 87 604Q82 608 82 615T88 628T102 635Q107 635 424 484L458 468L524 630Q593 789 597 795Q601 801 609 801Q616 801 622 795T629 781L562 615L493 450L589 406Q665 371 679 362T694 344Q694 339 693 337T677 326T631 302T538 257Q504 241 465 223T406 195T386 186Q384 185 322 39L262 -103H680Q682 -105 684 -108T688 -113T691 -118T693 -124Q693 -134 682 -141L464 -143H246L213 -219Q182 -292 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -282 174 -213T202 -143H146L93 -141Q82 -134 82 -124Q82 -120 83 -117T89 -110T95 -103H220L273 26Q326 156 326 157L218 106Q109 54 97 54ZM553 379Q480 412 480 415Q479 415 460 372T423 285T406 241Q408 240 516 291T624 344L553 379",
      8818: "674 732Q682 732 688 726T694 711T687 697Q686 696 417 568L151 442L399 324Q687 188 691 183Q694 177 694 172Q694 154 676 152H670L382 288Q92 425 90 427Q83 432 83 444Q84 455 96 461Q104 465 382 596T665 730Q669 732 674 732ZM56 -194Q56 -107 106 -51T222 6Q260 6 296 -12T362 -56T420 -108T483 -153T554 -171Q616 -171 654 -128T694 -29Q696 6 708 6Q722 6 722 -26Q722 -102 676 -164T557 -227Q518 -227 481 -209T415 -165T358 -113T294 -69T223 -51Q163 -51 125 -93T83 -196Q81 -228 69 -228Q56 -228 56 -202V-194",
      8819: "90 697Q83 704 83 712T88 726T99 732Q107 732 253 664T543 527T687 457Q694 452 694 442T687 427Q685 425 395 288L107 152H101Q83 154 83 172Q83 173 83 175Q82 182 98 191Q117 201 248 263Q326 300 378 324L626 442L360 568Q91 696 90 697ZM56 -194Q56 -107 106 -51T222 6Q260 6 296 -12T362 -56T420 -108T483 -153T554 -171Q616 -171 654 -128T694 -29Q696 6 708 6Q722 6 722 -26Q722 -102 676 -164T557 -227Q518 -227 481 -209T415 -165T358 -113T294 -69T223 -51Q163 -51 125 -93T83 -196Q81 -228 69 -228Q56 -228 56 -202V-194",
      8822: "734 181Q734 173 728 167T714 161Q711 161 386 280T54 404Q44 408 44 421Q44 432 52 437Q66 443 388 562T714 681Q721 681 727 675T734 661Q734 651 722 645Q711 639 462 546Q441 539 420 531L122 421L420 311L723 198Q734 192 734 181ZM44 247Q44 255 50 261T63 267Q66 267 391 148T723 24Q734 18 734 7T723 -10Q716 -14 391 -133T63 -253Q56 -253 50 -247T44 -233Q44 -223 55 -217Q67 -210 317 -118Q337 -110 357 -103L655 7L357 117L54 230Q44 236 44 247",
      8823: "83 661Q83 668 88 674T104 681Q111 679 396 560Q686 437 687 436Q694 431 694 421T687 406Q686 405 543 344T253 222T101 161Q83 163 83 180Q83 194 95 199Q96 199 130 213T232 257T361 311L621 421L357 532Q307 553 233 584Q121 631 102 640T83 661ZM673 267Q694 267 694 248Q694 237 687 232Q684 229 420 118L156 7L416 -103L683 -215Q694 -222 694 -233Q694 -251 676 -253Q670 -253 524 -192T235 -70T90 -8Q83 -1 83 7Q83 19 94 24Q97 25 378 144T667 266Q669 267 673 267",
      8826: "84 249Q84 262 91 266T117 270Q120 270 126 270T137 269Q388 273 512 333T653 512Q657 539 676 539Q685 538 689 532T694 520V515Q689 469 672 431T626 366T569 320T500 286T435 265T373 249Q379 248 404 242T440 233T477 221T533 199Q681 124 694 -17Q694 -41 674 -41Q658 -41 653 -17Q646 41 613 84T533 154T418 197T284 220T137 229H114Q104 229 98 230T88 235T84 249",
      8827: "84 517Q84 539 102 539Q115 539 119 529T125 503T137 459T171 404Q277 275 640 269H661Q694 269 694 249T661 229H640Q526 227 439 214T283 173T173 98T124 -17Q118 -41 103 -41Q83 -41 83 -17Q88 29 105 67T151 132T208 178T277 212T342 233T404 249Q401 250 380 254T345 263T302 276T245 299Q125 358 92 468Q84 502 84 517",
      8828: "112 270Q83 270 83 290Q83 301 94 307Q98 310 118 310Q516 310 620 464Q635 486 642 510T651 548T657 571T675 580Q693 577 693 559V552Q684 472 628 410T465 314Q436 303 372 290Q373 290 388 287T425 278T465 266Q674 199 693 28L694 17Q688 5 683 3Q677 0 673 0Q656 0 653 24Q623 270 118 270H112ZM110 116Q83 116 83 136T110 156H113Q134 156 160 155T231 146T318 128T407 95T489 44T550 -30T583 -131Q583 -153 563 -153Q556 -153 553 -152T547 -145T542 -127Q531 -54 478 0Q425 53 333 83T123 116H110",
      8829: "668 310Q694 310 694 290Q694 285 691 279Q684 271 664 270Q550 268 464 257T301 220T179 146T124 27Q119 0 103 0T83 16Q83 21 83 31T92 68T113 121T157 177T229 231Q295 268 405 290Q404 290 389 293T352 302T312 314Q138 371 96 500Q83 541 83 562Q83 568 89 574T103 580Q115 580 120 570T126 542T138 497T173 442Q289 310 659 310H668ZM194 -131Q201 -60 241 -6T343 82T477 133T628 155Q632 155 644 155T661 156Q685 155 690 147Q694 143 694 136Q694 132 693 129T689 124T685 120T681 117L656 116Q596 114 543 106T436 79T342 35T272 -33T235 -127Q231 -154 212 -154Q203 -153 199 -147T194 -136V-131",
      8830: "84 442Q84 455 91 459T117 463Q120 463 126 463T137 462Q388 466 512 526T653 705Q657 732 676 732Q685 731 689 725T694 714V708Q689 662 672 624T626 559T569 513T500 479T435 458T373 442Q379 441 404 435T440 426T477 414T533 392Q592 362 630 319T681 241T694 174Q694 153 674 153Q662 153 657 163T652 188T640 231T606 287Q500 416 137 422H114Q104 422 98 423T88 428T84 442ZM56 -194Q56 -107 106 -51T222 6Q260 6 296 -12T362 -56T420 -108T483 -153T554 -171Q616 -171 654 -128T694 -29Q696 6 708 6Q722 6 722 -26Q722 -102 676 -164T557 -227Q518 -227 481 -209T415 -165T358 -113T294 -69T223 -51Q163 -51 125 -93T83 -196Q81 -228 69 -228Q56 -228 56 -202V-194",
      8831: "84 710Q84 732 102 732Q115 732 119 722T125 696T137 652T171 597Q277 468 640 462H661Q694 462 694 442T661 422H640Q578 421 526 417T415 403T309 376T222 333T156 268T124 179Q122 162 118 158T103 153Q100 153 98 153T95 154T93 155T90 158T85 163Q83 167 83 176Q88 222 105 260T151 325T208 371T277 405T342 426T404 442Q401 443 380 447T345 456T302 469T245 492Q125 551 92 661Q84 695 84 710ZM56 -194Q56 -107 106 -51T222 6Q260 6 296 -12T362 -56T420 -108T483 -153T554 -171Q616 -171 654 -128T694 -29Q696 6 708 6Q722 6 722 -26Q722 -102 676 -164T557 -227Q518 -227 481 -209T415 -165T358 -113T294 -69T223 -51Q163 -51 125 -93T83 -196Q81 -228 69 -228Q56 -228 56 -202V-194",
      8832: "386 292Q388 292 439 393T543 598T598 703Q599 703 603 704T609 705Q616 705 622 699T629 685T533 494Q440 308 440 305Q451 310 462 312Q547 342 592 388T651 505Q654 525 658 532T673 539Q680 539 686 533T693 519Q693 495 678 450Q638 341 500 283Q433 259 418 259Q416 259 411 251T406 241T415 239Q482 224 544 190Q674 121 691 -10Q693 -28 691 -32Q684 -43 672 -43Q664 -43 658 -37Q656 -33 650 -6T634 47T589 109T500 168Q473 179 436 190T388 201H386L284 -1Q261 -45 232 -101T191 -181T178 -206Q176 -206 172 -207T166 -208Q160 -208 153 -202T146 -188Q146 -185 246 12Q344 206 344 210Q344 213 305 217T213 225T124 228H95Q82 241 82 248Q82 253 95 268H124Q172 268 236 273T343 283T386 292",
      8833: "103 -43Q96 -43 89 -39T82 -26L84 -10Q105 141 275 212Q342 236 355 236Q360 236 364 245L369 256H360Q284 280 275 283Q115 351 86 490Q82 507 82 517Q82 526 88 532T103 538Q110 538 115 534Q119 531 122 517T128 486T143 444T174 397T231 351T320 310Q371 292 389 292L491 496Q595 701 598 703Q599 703 603 704T609 705Q616 705 622 699T629 685Q629 684 531 485Q431 296 431 288Q431 278 520 273T651 268H680Q693 253 693 248Q693 241 680 228H651Q591 228 491 218T386 201L284 -1Q261 -45 232 -101T191 -181T178 -206Q176 -206 172 -207T166 -208Q160 -208 153 -202T146 -188Q146 -182 302 125L335 190L324 185Q313 185 289 172Q241 153 208 128T159 78T135 31T124 -11T118 -37Q112 -43 103 -43",
      8834: "84 250Q84 372 166 450T360 539Q361 539 370 539T395 539T430 540T475 540T524 540H679Q694 532 694 520Q694 511 681 501L522 500H470H441Q366 500 338 496T266 472Q244 461 224 446T179 404T139 337T124 250V245Q124 157 185 89Q244 25 328 7Q348 2 366 2T522 0H681Q694 -10 694 -20Q694 -32 679 -40H526Q510 -40 480 -40T434 -41Q350 -41 289 -25T172 45Q84 127 84 250",
      8835: "83 520Q83 532 98 540H251Q267 540 297 540T343 541Q427 541 488 525T605 455Q693 374 693 250Q693 165 650 99T545 0T415 -39Q407 -40 251 -40H98Q83 -32 83 -20Q83 -10 96 0H255H308H337Q412 0 439 4T512 28Q533 39 553 54T599 96T639 163T654 250Q654 341 592 411Q557 449 512 472Q468 491 439 495T335 500H306H255L96 501Q83 511 83 520",
      8838: "84 346Q84 468 166 546T360 635Q361 635 370 635T395 635T430 636T475 636T524 636H679Q694 628 694 616Q694 607 681 597L522 596H470H441Q366 596 338 592T266 568Q244 557 224 542T179 500T139 433T124 346V341Q124 253 185 185Q244 121 328 103Q348 98 366 98T522 96H681Q694 86 694 76Q694 64 679 56H526Q510 56 480 56T434 55Q350 55 289 71T172 141Q84 223 84 346ZM104 -131T104 -118T118 -98H679Q694 -106 694 -118T679 -138H118Q104 -131 104 -118",
      8839: "83 616Q83 628 98 636H251Q267 636 297 636T343 637Q427 637 488 621T605 551Q693 470 693 346Q693 261 650 195T545 96T415 57Q407 56 251 56H98Q83 64 83 76Q83 86 96 96H255H308H337Q412 96 439 100T512 124Q533 135 553 150T599 192T639 259T654 346Q654 437 592 507Q557 545 512 568Q468 587 439 591T335 596H306H255L96 597Q83 607 83 616ZM84 -131T84 -118T98 -98H659Q674 -106 674 -118T659 -138H98Q84 -131 84 -118",
      8840: "146 -283Q146 -282 174 -213T202 -143H115Q102 -127 102 -123T115 -103H220L291 68L278 73Q203 101 153 157T86 288Q83 309 83 344Q83 380 86 399Q107 480 160 539Q222 601 298 621Q328 630 345 631T435 635L526 637L560 715Q587 778 593 789T609 801Q616 801 622 795T629 781Q629 780 625 771T614 742T600 706L571 637Q571 635 626 635H680Q693 620 693 613T689 601L682 597L618 595H553L449 346Q425 288 399 223T359 127T346 95H356Q365 95 381 95T417 94T463 93T515 93H682Q693 82 693 74T680 53H511Q420 55 335 55L329 57L262 -103H680Q682 -105 684 -108T688 -113T691 -118T693 -124Q693 -134 682 -141L464 -143H246L213 -219Q182 -292 178 -299Q172 -303 166 -303T153 -297T146 -283ZM509 590Q509 595 438 595Q354 595 318 586Q246 567 195 516T126 395Q123 378 123 344T126 293Q141 229 184 181T291 110L306 104L406 346L509 590",
      8841: "82 606T82 613T95 635H251H348Q408 635 435 632T502 615L515 608L520 617Q520 619 558 708Q584 774 591 787T609 801Q616 801 622 795T629 781Q629 775 562 615L551 590L569 577Q646 527 678 437Q691 398 691 344T678 250Q653 182 597 132T469 64Q427 53 366 53H326L295 -25L262 -103H660Q673 -118 673 -124Q673 -129 669 -136L662 -141L453 -143H246L213 -219Q182 -292 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -282 174 -213T202 -143H95Q82 -128 82 -123T95 -103H220L251 -25L284 53H189L93 55L86 59Q82 64 82 71T95 93H302L400 333Q498 569 498 573L444 590Q431 593 260 595L93 597L86 601Q82 606 82 613ZM652 344V354Q652 451 575 521Q571 526 557 538T537 551Q534 551 533 548Q533 543 438 319L344 95L371 93H386Q487 93 557 150T649 293Q652 309 652 344",
      8842: "693 72Q693 68 692 66T686 59T680 52H524Q398 52 367 53T309 63Q236 82 180 132T98 250Q84 288 84 343Q84 397 98 437Q126 515 193 568T346 632Q347 632 373 633T440 634T520 635H680Q693 620 693 615Q693 608 680 595H526Q364 595 353 592Q279 582 221 539T138 430Q124 392 124 343Q124 296 138 257Q163 192 221 149T353 95Q364 92 526 92H680Q693 79 693 72ZM102 -132T102 -125T115 -103H382L420 -68Q429 -60 438 -52T452 -39T463 -28T472 -20T478 -14T483 -10T487 -7T490 -6T493 -5T496 -5Q502 -5 508 -12T515 -28Q515 -34 513 -37Q512 -38 507 -42T492 -55T475 -70L440 -101L562 -103H682Q693 -114 693 -122T680 -143H395L355 -179Q289 -241 280 -241Q273 -241 267 -235T260 -221T265 -208T300 -174L335 -143H224L113 -141L106 -137Q102 -132 102 -125",
      8843: "82 615Q82 620 95 635H251Q378 635 409 634T469 623Q540 605 596 555T678 437Q691 397 691 343T678 250Q649 172 581 119T426 55Q415 52 251 52H95Q93 55 89 59T84 65T82 72Q82 79 95 92H249Q411 92 422 95Q496 105 554 148T638 257Q651 296 651 343Q651 391 638 430Q613 495 555 538T422 592Q411 595 249 595H95Q82 608 82 615ZM82 -132T82 -125T95 -103H380L420 -57Q452 -21 460 -14T474 -6Q482 -6 488 -12T495 -25T451 -81L433 -101L549 -103H662Q673 -114 673 -122T660 -143H395L355 -190Q311 -239 309 -239Q305 -241 302 -241Q294 -241 287 -235T280 -221T324 -163L342 -143H218L93 -141L86 -137Q82 -132 82 -125",
      8846: "591 598H592Q604 598 611 583V376Q611 345 611 296Q610 162 606 148Q605 146 605 145Q586 68 507 23T333 -22Q268 -22 209 -1T106 66T56 173Q55 180 55 384L56 585Q66 598 75 598Q85 598 95 585V378L96 172L98 162Q112 95 181 57T332 18Q415 18 487 58T570 175Q571 180 571 383V583Q579 598 591 598ZM313 406Q313 417 313 435T312 459Q312 483 316 493T333 503T349 494T353 461V406V325H515Q516 325 519 323T527 316T531 305T527 294T520 287T515 285H353V204V152Q353 127 350 117T333 107T316 117T312 152Q312 158 312 175T313 204V285H151Q150 285 147 287T139 294T135 305T139 316T146 323T151 325H313V406",
      8847: "83 523Q87 535 99 539H679Q694 531 694 519Q694 506 679 499H123V-1H678Q694 -7 694 -21Q694 -34 679 -41H98Q93 -38 84 -28L83 247V523",
      8848: "64 506T64 519T78 539H699Q706 536 714 526V-28Q706 -38 699 -41H78Q64 -34 64 -21Q64 -6 80 -1H674V499H78Q64 506 64 519",
      8849: "94 620Q98 632 110 636H699Q714 628 714 616T699 596H134V96H698Q714 90 714 76Q714 64 699 56H109Q104 59 95 69L94 344V620ZM84 -118Q84 -103 100 -98H698Q714 -104 714 -118Q714 -130 699 -138H98Q84 -131 84 -118",
      8850: "64 603T64 616T78 636H668Q675 633 683 623V69Q675 59 668 56H78Q64 63 64 76Q64 91 80 96H643V596H78Q64 603 64 616ZM64 -118Q64 -108 79 -98H678Q694 -104 694 -118Q694 -130 679 -138H78Q64 -131 64 -118",
      8851: "83 0Q79 0 76 1T71 3T67 6T65 9T63 13T61 16V301L62 585Q70 595 76 598H592Q602 590 605 583V15Q598 2 587 0Q583 0 580 1T575 3T571 6T569 9T567 13T565 16V558H101V15Q94 2 83 0",
      8852: "77 0Q65 4 61 16V301L62 585Q72 598 81 598Q94 598 101 583V40H565V583Q573 598 585 598Q598 598 605 583V15Q602 10 592 1L335 0H77",
      8853: "56 250Q56 394 156 488T384 583Q530 583 626 485T722 250Q722 110 625 14T390 -83Q249 -83 153 14T56 250ZM364 542Q308 539 251 509T148 418T96 278V270H369V542H364ZM681 278Q675 338 650 386T592 462T522 509T458 535T412 542H409V270H681V278ZM96 222Q104 150 139 95T219 12T302 -29T366 -42H369V230H96V222ZM681 222V230H409V-42H412Q429 -42 456 -36T521 -10T590 37T649 113T681 222",
      8854: "56 250Q56 394 156 488T384 583Q530 583 626 485T722 250Q722 110 625 14T390 -83Q249 -83 153 14T56 250ZM681 278Q669 385 591 463T381 542Q283 542 196 471T96 278V270H681V278ZM275 -42T388 -42T585 32T681 222V230H96V222Q108 107 191 33",
      8855: "56 250Q56 394 156 488T384 583Q530 583 626 485T722 250Q722 110 625 14T390 -83Q249 -83 153 14T56 250ZM582 471Q531 510 496 523Q446 542 381 542Q324 542 272 519T196 471L389 278L485 375L582 471ZM167 442Q95 362 95 250Q95 137 167 58L359 250L167 442ZM610 58Q682 138 682 250Q682 363 610 442L418 250L610 58ZM196 29Q209 16 230 2T295 -27T388 -42Q409 -42 429 -40T465 -33T496 -23T522 -11T544 1T561 13T574 22T582 29L388 222L196 29",
      8856: "56 250Q56 394 156 488T384 583Q530 583 626 485T722 250Q722 110 625 14T390 -83Q249 -83 153 14T56 250ZM582 471Q581 472 571 480T556 491T539 502T517 514T491 525T460 534T424 539T381 542Q272 542 184 460T95 251Q95 198 113 150T149 80L167 58L582 471ZM388 -42Q513 -42 597 44T682 250Q682 363 610 442L196 29Q209 16 229 2T295 -27T388 -42",
      8857: "56 250Q56 394 156 488T384 583Q530 583 626 485T722 250Q722 110 625 14T390 -83Q249 -83 153 14T56 250ZM682 250Q682 322 649 387T546 497T381 542Q272 542 184 459T95 250Q95 132 178 45T389 -42Q515 -42 598 45T682 250ZM311 250Q311 285 332 304T375 328Q376 328 382 328T392 329Q424 326 445 305T466 250Q466 217 445 195T389 172Q354 172 333 195T311 250",
      8858: "57 250Q57 327 87 392T166 497T270 560T382 582H394Q512 582 610 500Q721 401 721 250Q721 112 626 15T389 -82Q251 -82 154 13T57 250ZM682 129T682 250T596 457T390 543Q269 543 183 457T96 250Q96 132 180 45T389 -43Q511 -43 596 43ZM250 250Q250 316 295 352T384 388Q451 388 489 347T528 250Q528 192 487 152T389 112Q331 112 291 152T250 250ZM488 250Q488 290 460 319T389 349Q348 349 319 320T290 250Q290 208 320 180T389 151Q431 151 459 181T488 250",
      8859: "57 250Q57 327 87 392T166 497T270 560T382 582H394Q512 582 610 500Q721 401 721 250Q721 112 626 15T389 -82Q251 -82 154 13T57 250ZM682 129T682 250T596 457T390 543Q269 543 183 457T96 250Q96 132 180 45T389 -43Q511 -43 596 43ZM204 339Q204 357 215 366T238 375Q247 375 283 348Q300 336 311 328L368 286Q369 286 366 323T359 398T355 437Q357 456 379 465Q380 465 384 465T391 466Q403 465 412 457T423 437Q423 436 420 398T413 323T410 286L467 328Q476 334 486 341T501 353T513 361T523 368T529 372T535 374T541 375Q554 375 564 365T575 339Q575 325 566 318T519 292Q504 285 496 281L430 250L496 219Q552 192 559 188T572 175Q575 168 575 161Q575 148 566 137T541 126H538Q530 126 499 149Q480 163 467 172L410 214Q409 214 412 177T419 102T423 63Q423 59 421 54T411 43T389 36T368 42T357 54T355 63Q355 64 358 102T365 177T368 214L311 172Q302 165 293 159T279 148T268 140T260 134T254 131T250 128T246 127T242 126T238 126Q223 126 214 135T204 161T213 183T282 219L348 250L282 281Q226 308 219 312T206 325Q204 330 204 339",
      8861: "57 250Q57 327 87 392T166 497T270 560T382 582H394Q512 582 610 500Q721 401 721 250Q721 112 626 15T389 -82Q251 -82 154 13T57 250ZM682 129T682 250T596 457T390 543Q269 543 183 457T96 250Q96 132 180 45T389 -43Q511 -43 596 43ZM223 250Q223 263 233 267T280 271Q289 271 325 271T389 270H490Q535 270 545 267T555 250Q555 241 549 235Q544 231 527 231T389 230Q239 230 235 232Q223 236 223 250",
      8862: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71ZM369 365V649H95V365H369ZM682 365V649H409V365H682ZM369 40V325H95V40H369ZM682 40V325H409V40H682",
      8863: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71ZM682 365V649H95V365H682ZM682 40V325H95V40H682",
      8864: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H707Q714 686 722 676V13Q714 3 707 0H71ZM123 649Q147 625 214 555T335 430T389 374L654 649H123ZM95 70Q99 74 229 209T360 345L95 619V70ZM682 70V619L418 346Q417 344 549 207L682 70ZM654 41L400 304L388 315L123 41L256 40H522L654 41",
      8865: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71ZM682 40V649H95V40H682ZM330 345Q330 371 347 388T390 405Q412 405 430 389T448 345Q448 317 430 301T389 285T348 301T330 345",
      8866: "55 678Q55 679 56 681T58 684T61 688T65 691T70 693T77 694Q88 692 95 679V367H540Q555 359 555 347Q555 334 540 327H95V15Q88 2 77 0Q73 0 70 1T65 3T61 6T59 9T57 13T55 16V678",
      8867: "515 678Q515 679 516 681T518 684T521 688T525 691T530 693T537 694Q548 692 555 679V15Q548 2 537 0Q533 0 530 1T525 3T521 6T519 9T517 13T515 16V327H71Q70 327 67 329T59 336T55 347T59 358T66 365T71 367H515V678",
      8868: "55 642T55 648T59 659T66 666T71 668H708Q723 660 723 648T708 628H409V15Q402 2 391 0Q387 0 384 1T379 3T375 6T373 9T371 13T369 16V628H71Q70 628 67 630T59 637",
      8869: "369 652Q369 653 370 655T372 658T375 662T379 665T384 667T391 668Q402 666 409 653V40H708Q723 32 723 20T708 0H71Q70 0 67 2T59 9T55 20T59 31T66 38T71 40H369V652",
      8872: "139 -249H137Q125 -249 119 -235V251L120 737Q130 750 139 750Q152 750 159 735V367H796Q811 359 811 347Q811 336 797 328L479 327H161L159 328V172L161 173H797Q798 172 800 171T803 169T805 167T808 164T809 162T810 158T811 153Q811 140 796 133H159V-235Q151 -249 141 -249H139",
      8873: "55 678Q55 679 56 681T58 684T61 688T65 691T70 693T77 694Q88 692 95 679V15Q88 2 77 0Q73 0 70 1T65 3T61 6T59 9T57 13T55 16V678ZM249 678Q249 679 250 681T252 684T255 688T259 691T264 693T271 694Q282 692 289 679V367H651Q666 359 666 347Q666 334 651 327H289V15Q282 2 271 0Q267 0 264 1T259 3T255 6T253 9T251 13T249 16V678",
      8874: "55 678Q55 679 56 681T58 684T61 688T65 691T70 693T77 694Q88 692 95 679V15Q88 2 77 0Q73 0 70 1T65 3T61 6T59 9T57 13T55 16V678ZM237 678Q237 679 238 681T240 684T243 688T247 691T252 693T259 694Q270 692 277 679V15Q270 2 259 0Q255 0 252 1T247 3T243 6T241 9T239 13T237 16V678ZM419 678Q419 679 420 681T422 684T425 688T429 691T434 693T441 694Q452 692 459 679V367H818Q833 359 833 347Q833 334 818 327H459V15Q452 2 441 0Q437 0 434 1T429 3T425 6T423 9T421 13T419 16V678",
      8876: "56 681Q70 695 76 695T96 681V368H243L381 530Q521 692 525 692Q537 700 547 688Q554 682 554 674Q554 671 553 669T548 661T539 649T522 631T499 604T465 565T421 512Q296 373 296 368H416H476Q525 368 539 365T554 348Q554 334 543 328H261L96 141V12Q81 -1 75 -1Q65 -1 58 10L56 50V92L18 48Q7 37 -1 28T-13 14T-19 6T-23 1T-27 0T-33 -1Q-42 -1 -48 4T-55 19Q-55 24 -47 34T12 103L56 155V681ZM205 326Q205 328 152 328H96V263Q96 203 98 203Q99 203 123 231T174 290T205 326",
      8877: "56 681Q70 695 76 695T96 681V466H327L425 579Q522 692 527 692Q529 693 534 693Q542 693 547 688T553 674Q553 668 549 663Q549 662 538 650T504 611T463 563L381 468L461 466H543Q554 453 554 446T541 426H345L209 272L376 270H543Q554 257 554 251T541 230H174L96 141V12Q81 -1 75 -1Q65 -1 58 10L56 50V92L18 48Q7 37 -1 28T-13 14T-19 6T-23 1T-27 0T-33 -1Q-42 -1 -48 4T-55 19Q-55 24 -47 34T12 103L56 155V681ZM267 399L292 426H96V270H158L201 321Q256 382 267 399ZM118 228L119 229Q119 230 109 230H96V201L107 212Q118 227 118 228",
      8878: "56 681Q70 695 77 695T96 683V428L98 175L252 323V681Q264 695 272 695Q278 695 292 681V526Q292 368 296 368Q298 368 447 510Q638 695 642 695H645Q651 695 658 688T665 673Q665 666 661 661Q659 660 639 641T578 582T505 512L356 370L505 368H654Q665 357 665 349Q665 343 652 328H314L303 317L292 308V12Q289 10 285 6T279 1T272 -1Q265 -1 252 12V139Q252 266 249 266L96 119V12Q80 -1 76 -1T70 0T63 6T56 12V79L29 55Q-26 -1 -35 -1Q-42 -1 -48 5T-55 19Q-55 25 -51 30T-15 66Q5 86 18 99L56 135V681",
      8879: "56 681Q70 695 77 695T96 683V428L98 175L252 323V681Q264 695 272 695Q278 695 292 681V466H401L503 563L621 679Q637 695 645 695Q652 695 658 688T665 673Q665 670 663 666Q663 665 651 652T611 612T561 563L458 468L556 466H654Q665 455 665 447T652 426H416L294 308L292 288V270H652Q665 255 665 250T652 230H292V12Q289 10 285 6T279 1T272 -1Q265 -1 252 12V139Q252 266 249 266L96 119V12Q80 -1 76 -1T70 0T63 6T56 12V79L29 55Q-26 -1 -35 -1Q-42 -1 -48 5T-55 19Q-55 25 -51 30T-15 66Q5 86 18 99L56 135V681ZM358 426H292V361L325 392L358 426",
      8882: "694 -26Q686 -40 676 -41H670L382 95Q92 232 90 234Q83 239 83 249Q83 262 96 267Q101 270 379 401T665 537Q671 539 674 539Q686 539 694 524V-26ZM654 11T654 249T653 487T402 369T151 249L275 190Q399 131 524 72T652 11Q654 11 654 249",
      8883: "83 523Q83 524 85 527T92 535T103 539Q107 539 389 406T680 268Q694 260 694 249Q694 239 687 234Q685 232 395 95L107 -41H101Q90 -40 83 -26V523ZM376 368Q323 393 254 425T155 472L125 487Q123 487 123 249T125 11Q127 12 252 71T502 190L626 249L376 368",
      8884: "694 71Q686 58 676 56H670L382 192Q92 329 90 331Q83 336 83 346Q83 359 96 364Q101 367 379 498T665 634Q671 636 674 636Q686 636 694 621V71ZM654 108T654 346T653 584T402 466T151 346L275 287Q399 228 524 169T652 108Q654 108 654 346ZM83 -120Q83 -116 84 -113T86 -108T89 -104T92 -102T96 -100T99 -98H678Q679 -98 681 -99T684 -101T688 -104T691 -108T693 -113T694 -120Q692 -130 679 -138H98Q84 -130 83 -120",
      8885: "83 620Q83 621 85 624T92 632T103 636Q107 636 389 503T680 365Q694 357 694 346Q694 336 687 331Q685 329 395 192L107 56H101Q90 58 83 71V620ZM376 465Q323 490 254 522T155 570L125 584Q123 584 123 346T125 108Q127 109 252 168T502 287L626 346L376 465ZM83 -120Q83 -116 84 -113T86 -108T89 -104T92 -102T96 -100T99 -98H678Q679 -98 681 -99T684 -101T688 -104T691 -108T693 -113T694 -120Q692 -130 679 -138H98Q84 -130 83 -120",
      8888: "1055 250Q1055 190 1012 141T896 92Q858 92 828 106T781 140T755 180T741 214L738 228V230H405Q71 230 68 232Q55 238 55 250T68 268Q71 270 405 270H738V272L740 280Q742 287 745 297T754 321T771 348T796 374T832 396T881 408H891Q969 408 1012 360T1055 250ZM896 132Q948 132 981 166T1014 250Q1014 301 985 330T920 367Q914 368 891 368Q853 368 816 338T778 250Q778 198 812 165T896 132",
      8890: "318 -182Q302 -212 280 -212H278H275Q249 -212 239 -182L238 84V351H162L87 352Q57 362 57 391T84 429Q89 431 280 431H470L474 429Q477 427 479 426T484 423T490 417T495 410T499 402T500 391Q500 365 470 352L394 351H318V-182",
      8891: "56 697Q56 706 62 711T75 716Q86 716 90 709Q91 708 104 680T147 592T199 483L305 261L411 483Q443 548 481 629Q512 694 518 705T535 716Q543 716 549 710T555 700Q555 693 501 577T388 340T325 210Q316 194 305 194Q292 194 285 210Q282 219 224 339T111 574T56 697ZM55 14T55 20T59 31T66 38T71 40H540Q555 32 555 20T540 0H71Q70 0 67 2T59 9",
      8892: "55 698Q56 708 70 716H540Q554 708 555 698Q555 694 554 691T552 686T549 682T546 680T542 678T539 676H71Q70 676 68 677T65 679T61 682T58 686T56 691T55 698ZM555 18Q554 12 549 6T536 0H535Q525 0 515 17T459 132Q430 194 410 235L305 455L199 233Q176 185 147 125T105 36T90 7Q85 0 75 0Q63 0 58 11Q55 15 55 21Q58 31 170 266T285 507Q295 522 305 522T320 515Q322 513 439 268L555 24V18",
      8896: "119 -249T97 -249T65 -235T55 -207Q55 -201 56 -198Q58 -190 218 268T380 729Q392 750 416 750Q438 750 451 732Q453 728 534 498T695 36L775 -194Q777 -204 777 -208Q777 -222 767 -235T735 -249Q713 -249 700 -231Q696 -225 557 177L416 579L276 177Q136 -226 132 -231Q119 -249 97 -249",
      8897: "55 708Q55 729 68 739T96 750Q119 750 132 731Q136 726 276 323L416 -79L557 323Q696 725 700 731Q713 749 735 749Q756 749 766 736T777 708Q777 700 696 466T533 1T451 -232Q436 -249 416 -249Q402 -249 391 -241Q384 -236 380 -226Q368 -198 219 230Q55 697 55 708",
      8898: "139 -217Q127 -241 114 -246Q106 -249 97 -249Q67 -249 57 -220Q55 -214 55 102Q55 152 55 221T54 312Q54 422 60 464T91 554Q120 612 165 654T257 714T337 741T392 749Q393 750 402 750Q414 750 422 749Q557 749 660 659T776 430Q777 422 777 102Q777 -214 775 -220Q765 -249 735 -249Q716 -249 708 -241T694 -217L692 428L690 441Q674 540 597 603T416 666H409Q388 666 364 662T294 638T212 581Q156 523 142 441L140 428L139 105V-217",
      8899: "96 750Q103 750 109 748T120 744T127 737T133 730T137 723T139 718V395L140 73L142 60Q159 -43 237 -104T416 -166Q521 -166 597 -103T690 60L692 73L694 718Q708 749 735 749Q765 749 775 720Q777 714 777 398Q777 78 776 71Q766 -51 680 -140Q571 -249 416 -249H411Q261 -249 152 -140Q66 -51 56 71Q55 78 55 398Q55 714 57 720Q60 734 70 740Q80 750 96 750",
      8900: "242 486Q245 488 250 488Q256 488 258 486Q262 484 373 373T486 258T488 250T486 242T373 127T258 14Q256 12 250 12Q245 12 242 14Q237 16 127 126T14 242Q12 245 12 250T14 258Q16 263 126 373T242 486ZM439 250L250 439L61 250L250 61L439 250",
      8901: "78 250Q78 274 95 292T138 310Q162 310 180 294T199 251Q199 226 182 208T139 190T96 207T78 250",
      8902: "210 282Q210 284 225 381T241 480Q241 484 245 484Q249 486 251 486Q258 486 260 477T272 406Q275 390 276 380Q290 286 290 282L388 299Q484 314 487 314H488Q497 314 497 302Q497 297 434 266Q416 257 404 251L315 206L361 118Q372 98 383 75T401 40L407 28Q407 16 395 16Q394 16 392 16L390 17L250 159L110 17L108 16Q106 16 105 16Q93 16 93 28L99 40Q105 52 116 75T139 118L185 206L96 251Q6 296 4 300Q3 301 3 302Q3 314 12 314H13Q16 314 112 299L210 282",
      8903: "366 543Q374 545 382 545Q405 545 419 538Q429 534 443 521T462 496Q466 478 466 467Q466 438 444 412Q422 390 388 390Q352 390 331 412Q311 434 311 467Q311 499 331 518Q345 533 366 543ZM146 472Q146 479 153 485T166 492Q171 492 187 476T279 385L386 278L495 385Q600 492 608 492Q615 492 621 486T628 472Q628 467 614 452T531 367L435 270H706Q720 256 720 250Q720 241 706 230H435L531 132Q600 63 614 48T628 27Q628 20 622 14T608 7Q600 7 495 114L386 221L279 114Q204 39 188 23T166 7Q159 7 153 13T146 27Q146 32 160 47T244 132L339 230H68Q55 243 55 250Q55 255 68 270H339L244 367Q175 436 161 451T146 472ZM466 34Q466 4 447 -20T388 -44Q353 -44 331 -22Q311 1 311 34Q311 66 331 85Q347 101 366 110Q374 112 382 112Q405 112 419 105Q429 100 443 87T462 63Q466 45 466 34",
      8904: "833 50T833 250T832 450T659 351T487 250T658 150T832 50Q833 50 833 250ZM873 10Q866 -5 854 -5Q851 -5 845 -3L449 226L260 115Q51 -5 43 -5Q39 -5 35 -1T28 7L26 11V489Q33 505 43 505Q51 505 260 385L449 274L845 503Q851 505 853 505Q866 505 873 490V10ZM412 250L67 450Q66 450 66 250T67 50Q69 51 240 150T412 250",
      8905: "146 479Q159 492 166 492Q171 492 189 475T279 386L386 279L495 386Q598 492 608 492Q615 492 621 486T628 472Q628 464 522 357L415 250L522 144Q628 37 628 28Q628 21 622 15T608 8Q599 8 495 115L386 221L279 115Q204 40 188 24T166 8Q162 8 160 9T153 15T146 21V479ZM186 77L359 250L186 424V77",
      8906: "146 472Q146 479 152 485T166 492Q171 492 189 475T279 386L386 279L495 386Q598 492 608 492Q615 492 628 479V21Q615 8 608 8Q599 8 495 115L386 221L279 115Q204 40 188 24T166 8Q159 8 153 14T146 28Q146 37 253 144L359 250L253 357Q146 464 146 472ZM588 77V424L499 337L415 250L588 77",
      8907: "55 674Q55 682 62 688T76 694H77Q83 694 100 677T208 561Q320 440 410 342Q462 286 541 201Q677 55 699 30T722 -2Q722 -9 716 -15T701 -22T688 -17Q687 -15 542 141T394 301L388 306L240 146Q119 15 101 -3T75 -22T61 -16T55 -2Q55 4 67 19T158 117Q190 151 209 172L361 336L209 500Q62 657 57 667Q55 671 55 674",
      8908: "84 -22T76 -22T62 -16T55 -2Q55 4 78 30T249 215Q321 293 367 342Q672 672 683 682Q695 694 702 694Q710 694 716 688T722 674Q722 668 710 653T619 555Q587 521 568 500L416 336L568 172Q715 15 720 5Q722 1 722 -2Q722 -9 716 -15T702 -22H700Q693 -22 671 1T537 146L389 306Q387 304 340 253T237 143T135 33L89 -17Q84 -22 76 -22",
      8909: "56 433Q56 464 71 464Q74 464 77 461Q82 454 82 438T91 397T123 347Q166 307 222 307Q264 307 308 331T386 385T465 438T556 463Q631 463 676 408T722 283Q722 250 708 250Q704 250 699 257Q695 265 693 286T682 330Q670 350 655 367Q612 407 556 407Q514 407 470 383T393 329T314 276T222 251Q148 251 102 306T56 433ZM57 56Q57 71 73 76H706Q722 70 722 56Q722 44 707 36H71Q57 43 57 56",
      8910: "83 558Q83 566 89 572T104 578Q108 578 116 577T146 570T190 555T239 526T286 480Q308 453 325 420T351 358T367 304T376 265T380 251T381 253Q381 262 395 312Q428 434 492 499T642 576Q654 578 655 578Q664 578 670 572T676 558Q676 543 657 540T599 524T525 476Q406 362 400 29V8Q400 -21 380 -21Q369 -21 362 -11Q360 -7 360 12Q360 115 348 200T308 360T231 480T111 537Q83 540 83 558",
      8911: "104 -22Q95 -22 89 -16T83 -2Q83 11 98 16T135 23T192 46T256 103Q360 233 360 549Q360 554 360 557T361 563T362 567T364 569T367 572T371 576Q377 578 380 578Q401 578 401 547Q401 543 401 537T400 527Q409 53 648 19Q676 16 676 -2Q676 -10 670 -16T655 -22Q654 -22 642 -20Q556 -9 492 56T395 244Q381 294 381 303Q381 305 380 305T374 275T352 201T310 110T234 27T117 -20Q105 -22 104 -22",
      8912: "84 250Q84 372 166 450T360 539Q361 539 370 539T395 539T430 540T475 540T524 540H679Q694 532 694 520Q694 511 681 501L522 500H470H441Q366 500 338 496T266 472Q244 461 224 446T179 404T139 337T124 250V245Q124 157 185 89Q244 25 328 7Q348 2 366 2T522 0H681Q694 -10 694 -20Q694 -32 679 -40H526Q510 -40 480 -40T434 -41Q350 -41 289 -25T172 45Q84 127 84 250ZM694 134Q694 123 679 114H425H384Q350 114 326 121T277 154Q238 193 238 251Q238 322 295 361Q318 378 339 382T412 387Q423 387 459 387T520 386H679Q694 377 694 366Q694 354 679 346H519Q493 346 458 346T411 347Q360 347 341 342T303 315Q278 287 278 250Q278 210 301 187T351 156Q358 154 519 154H679Q694 146 694 134",
      8913: "83 520Q83 532 98 540H251Q267 540 297 540T343 541Q427 541 488 525T605 455Q693 374 693 250Q693 165 650 99T545 0T415 -39Q407 -40 251 -40H98Q83 -32 83 -20Q83 -10 96 0H255H308H337Q412 0 439 4T512 28Q533 39 553 54T599 96T639 163T654 250Q654 341 592 411Q557 449 512 472Q468 491 439 495T335 500H306H255L96 501Q83 511 83 520ZM83 366Q83 376 96 386H244Q280 386 317 386T378 386L402 387Q456 387 498 348T540 250Q540 203 512 168T446 120Q427 114 353 114H99Q84 120 84 134Q84 147 98 154H258Q284 154 319 154T366 153Q416 153 436 158T474 185Q500 214 500 250Q500 290 477 313T426 344Q419 346 258 346H98Q83 354 83 366",
      8914: "88 -21T75 -21T55 -7V200Q55 231 55 280Q56 414 60 428Q61 430 61 431Q77 500 152 549T332 598Q443 598 522 544T610 405Q611 399 611 194V-7Q604 -22 591 -22Q582 -22 572 -9L570 405Q563 433 556 449T529 485Q498 519 445 538T334 558Q251 558 179 518T96 401Q95 396 95 193V-7Q88 -21 75 -21ZM229 -21H227Q215 -21 209 -7V166Q209 304 209 327T215 363Q226 398 259 421T333 444Q380 444 414 416T455 347Q457 339 457 166V-7Q449 -21 439 -21H437H435Q423 -21 417 -7V164Q417 303 417 325T411 358Q387 403 333 403T255 358Q250 347 250 325T249 164V-7Q241 -21 231 -21H229",
      8915: "591 598H592Q604 598 611 583V376Q611 345 611 296Q610 162 606 148Q605 146 605 145Q586 68 507 23T333 -22Q268 -22 209 -1T106 66T56 173Q55 180 55 384L56 585Q66 598 75 598Q85 598 95 585V378L96 172L98 162Q112 95 181 57T332 18Q415 18 487 58T570 175Q571 180 571 383V583Q579 598 591 598ZM437 598Q450 598 457 583V410Q457 237 455 229Q448 189 414 161T333 132Q291 132 255 157T211 230Q209 237 209 412L210 585Q220 598 229 598Q242 598 249 583V412Q249 273 249 251T255 218Q279 173 333 173T411 218Q416 229 416 251T417 412V583Q425 598 437 598",
      8916: "76 -22Q64 -22 56 -7V176L57 360L59 370Q66 401 83 426T123 468T171 495T221 513T265 522T298 527L311 528H314V625L315 723Q325 736 334 736Q346 736 354 721V528H356L368 527Q380 526 399 523T441 515T490 498T537 472T578 433T606 379Q611 359 611 171V-7Q604 -21 591 -21T571 -7V170Q571 313 571 337T565 375Q555 408 526 432T461 467T402 482T365 487H354V-7Q347 -21 334 -21T314 -7V487H303Q251 484 207 467Q121 438 99 367L97 357L96 174V-9Q86 -22 76 -22",
      8918: "86 261Q92 267 381 404T673 541Q680 541 686 535T693 521T689 510Q684 504 418 379L151 250L418 121Q686 -4 689 -10Q693 -14 693 -21T687 -34T675 -41Q668 -41 380 96T86 239Q82 244 82 250Q82 257 86 261ZM610 250Q610 224 592 198T531 172Q498 172 475 195Q453 214 453 250Q453 308 513 328Q515 330 535 330Q569 328 589 304T610 250",
      8919: "82 521Q82 529 89 535T100 541Q107 541 395 404T689 261Q693 257 693 250T689 239Q684 234 396 97T100 -41Q95 -41 89 -35T82 -21Q82 -12 96 -4Q118 9 358 121L624 250L358 379Q91 503 86 510Q82 514 82 521ZM165 250Q165 282 188 306T239 330Q262 330 275 323Q303 312 318 283Q322 272 322 250Q322 213 300 195Q277 172 246 172Q224 172 213 177Q165 200 165 250",
      8920: "639 -48Q639 -54 634 -60T619 -67H618Q612 -67 536 -26Q430 33 329 88Q61 235 59 239Q56 243 56 250T59 261Q62 266 336 415T615 567L619 568Q622 567 625 567Q639 562 639 548Q639 540 633 534Q632 532 374 391L117 250L374 109Q632 -32 633 -34Q639 -40 639 -48ZM958 -48Q958 -54 953 -60T938 -67H937Q931 -67 855 -26Q749 33 648 88Q380 235 378 239Q375 243 375 250T378 261Q381 266 655 415T934 567L938 568Q941 567 944 567Q958 562 958 548Q958 540 952 534Q951 532 693 391L436 250L693 109Q951 -32 952 -34Q958 -40 958 -48ZM1277 -48Q1277 -54 1272 -60T1257 -67H1256Q1250 -67 1174 -26Q1068 33 967 88Q699 235 697 239Q694 243 694 250T697 261Q700 266 974 415T1253 567L1257 568Q1260 567 1263 567Q1277 562 1277 548Q1277 540 1271 534Q1270 532 1012 391L755 250L1012 109Q1270 -32 1271 -34Q1277 -40 1277 -48",
      8921: "75 -67Q65 -67 60 -61T55 -48Q55 -40 61 -34Q62 -32 329 109L595 250L329 391Q62 532 61 534Q55 540 55 548Q55 562 69 567H77Q81 567 222 493T506 342T653 264Q667 250 653 236Q649 234 504 157T220 7T77 -67H75ZM364 547Q364 563 381 567L384 568Q387 568 518 499T795 353T955 269Q967 261 967 250T955 231Q925 216 780 139T513 -3T383 -67Q373 -67 369 -60T364 -47Q364 -40 370 -34Q373 -31 639 109L904 250L639 391Q373 531 370 534Q364 540 364 547ZM674 538T674 548T681 562T693 567Q699 567 816 505Q915 453 993 412Q1050 382 1132 339Q1241 282 1259 271T1277 250Q1277 241 1263 232Q1246 221 985 84Q698 -67 692 -67Q674 -67 674 -47Q674 -38 680 -33Q683 -30 947 109L1213 250L947 391Q683 530 680 533Q674 538 674 548",
      8922: "674 445Q674 438 669 432T655 425T369 531T90 640Q83 645 83 655Q83 668 95 673Q644 886 654 886Q662 886 668 880T674 866Q674 856 663 850Q649 843 411 751L160 655L407 560Q474 534 561 501Q646 469 660 462T674 445ZM84 250Q84 260 99 270H658Q674 264 674 250Q674 238 659 230H98Q84 237 84 250ZM83 55Q83 68 94 73Q98 76 104 76Q108 75 383 -30T664 -138Q674 -144 674 -155Q674 -165 667 -170Q664 -173 385 -279T104 -386Q85 -386 83 -368Q83 -354 92 -349Q93 -349 347 -251L597 -155L346 -59Q296 -40 223 -12Q118 28 101 36T83 55",
      8923: "111 425T102 425T88 431T83 445V446Q83 455 96 461Q111 469 203 504Q287 536 350 560L597 655L346 751Q94 848 92 850Q83 856 83 866Q83 873 88 879T104 886Q109 885 386 779T667 670Q674 665 674 655T667 640Q665 638 388 532ZM84 250Q84 260 99 270H658Q674 264 674 250Q674 238 659 230H98Q84 237 84 250ZM653 76Q656 76 660 75T669 68T674 56Q674 46 665 40Q663 38 411 -59L160 -155L410 -251Q664 -349 665 -349Q674 -354 674 -368Q672 -386 654 -386Q650 -386 371 -279T90 -170Q83 -165 83 -155Q83 -144 93 -138Q645 76 653 76",
      8926: "113 424Q83 424 83 444Q83 453 96 464H121Q181 466 234 474T341 501T435 545T505 613T542 707Q545 734 564 734Q583 731 583 714Q583 658 560 613T500 538T414 486T321 453T229 434T156 426T113 424ZM112 270Q83 270 83 290Q83 301 94 307Q98 310 118 310Q624 310 653 556Q657 580 675 580Q693 577 693 559V552Q684 472 628 410T465 314Q436 303 372 290Q373 290 388 287T425 278T465 266Q674 199 693 28L694 17L692 14Q691 11 689 8T683 3T673 0Q657 0 653 24Q623 270 118 270H112",
      8927: "195 713Q195 725 201 729T214 734Q227 734 231 722T238 691T255 641T299 580Q405 474 656 464H681Q694 451 694 443Q694 424 670 424H664Q535 424 415 465T235 595Q195 657 195 713ZM668 310Q694 310 694 290Q694 285 691 279Q684 271 664 270Q550 268 464 257T301 220T179 146T124 27Q119 0 103 0T83 16Q83 21 83 31T92 68T113 121T157 177T229 231Q295 268 405 290Q404 290 389 293T352 302T312 314Q138 371 96 500Q83 541 83 562Q83 568 89 574T103 580Q115 580 120 570T126 542T138 497T173 442Q289 310 659 310H668",
      8928: "82 344Q82 349 95 364H124Q266 364 398 390L429 397L509 595Q519 619 536 659Q581 766 590 783T609 801Q616 801 622 795T629 781Q629 776 553 595Q533 548 516 506T489 439T480 415Q482 415 505 426T538 444Q632 498 651 601Q654 621 658 628T673 635Q680 635 686 629T693 615Q693 591 678 546Q636 433 484 375L458 364L451 348Q443 332 443 329T455 324Q480 316 503 307T560 277T619 233T664 170T691 86Q693 68 691 64Q684 53 672 53Q664 53 658 59Q657 60 650 97T617 174T538 244Q515 257 476 273T428 289Q425 289 412 256Q381 179 344 90L262 -103H680Q682 -105 684 -108T688 -113T691 -118T693 -124Q693 -134 682 -141L464 -143H246L213 -219Q182 -292 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -282 174 -213T202 -143H95Q82 -128 82 -123T95 -103H220L302 97Q384 288 384 299Q384 302 341 308T235 319T124 324H95Q82 337 82 344ZM399 338Q403 338 406 346L409 353L375 344Q375 343 384 341T399 338",
      8929: "146 -283Q146 -282 174 -213T202 -143H95Q82 -127 82 -123T95 -103H220L300 93Q343 196 374 270Q385 294 386 299L373 295Q331 287 289 268Q241 249 208 224T159 174T135 127T124 85T118 59Q112 53 103 53Q91 53 84 64Q82 68 84 86Q96 185 174 248T375 337L400 344Q399 344 381 348T351 355T316 364T276 379T235 398T193 424T155 456T122 497T98 546Q82 587 82 615Q82 622 88 628T102 635Q112 635 116 628T124 601Q128 579 134 562T159 515T207 463T290 418T415 384L422 381L506 586Q571 744 584 772T609 801Q616 801 622 795T629 781T544 577Q525 529 504 478T473 402T462 375Q480 373 500 373Q579 364 651 364H680Q682 361 686 357T691 351T693 344Q693 337 680 324H651Q553 324 451 310L433 308L349 104L262 -101L473 -103H682Q694 -115 694 -123Q694 -133 682 -141L464 -143H246L213 -219Q182 -292 178 -299Q172 -303 166 -303T153 -297T146 -283",
      8934: "86 450Q93 455 380 592T673 730Q680 730 686 724T693 710Q693 702 688 699Q686 693 417 568L151 439L417 310Q685 185 688 179Q693 176 693 168Q693 161 687 155T675 148Q668 148 380 285T86 428Q74 438 86 450ZM55 -205Q55 -175 64 -142T92 -76T145 -22T222 -1Q288 -1 362 -66Q369 -72 372 -75T378 -79T382 -81T384 -79Q389 -74 439 21Q483 100 490 111T504 122Q510 122 518 118T526 103Q526 101 510 69T467 -12T419 -99L413 -112L433 -128Q498 -180 553 -180Q605 -180 646 -139Q672 -112 681 -77T693 -21T706 -1Q719 -1 719 -33Q719 -39 717 -57Q708 -141 655 -190Q625 -224 586 -232Q568 -237 551 -237Q487 -237 413 -172L391 -155Q391 -157 335 -255Q297 -325 286 -342T268 -359Q260 -359 254 -353T248 -339T304 -230L359 -126Q359 -124 337 -107T302 -81Q262 -57 221 -57Q170 -57 130 -93T84 -201Q82 -236 70 -236Q55 -236 55 -205",
      8935: "88 723Q95 730 99 730Q106 730 394 593T688 450Q693 447 693 439T688 428Q683 423 395 286T99 148Q94 148 88 155T82 168Q82 175 86 179Q89 184 357 310L624 439L357 568Q88 694 86 699Q81 703 81 711T88 723ZM55 -205Q55 -175 64 -142T92 -76T145 -22T222 -1Q288 -1 362 -66Q369 -72 372 -75T378 -79T382 -81T384 -79Q389 -74 439 21Q483 100 490 111T504 122Q510 122 518 118T526 103Q526 101 510 69T467 -12T419 -99L413 -112L433 -128Q498 -180 553 -180Q605 -180 646 -139Q672 -112 681 -77T693 -21T706 -1Q719 -1 719 -33Q719 -39 717 -57Q708 -141 655 -190Q625 -224 586 -232Q568 -237 551 -237Q487 -237 413 -172L391 -155Q391 -157 335 -255Q297 -325 286 -342T268 -359Q260 -359 254 -353T248 -339T304 -230L359 -126Q359 -124 337 -107T302 -81Q262 -57 221 -57Q170 -57 130 -93T84 -201Q82 -236 70 -236Q55 -236 55 -205",
      8936: "95 419Q81 433 81 439T95 459H124Q318 459 455 501Q515 521 556 550T615 607T641 659T652 702T659 725Q667 730 673 730Q680 730 686 724T693 710Q693 682 677 641Q668 616 654 594T622 554T586 522T545 497T504 477T464 462T428 452T397 444T375 439Q379 437 410 430T476 411T551 379T625 321T677 237Q693 196 693 168Q693 161 687 155T673 148Q662 148 658 154T651 181Q638 253 591 300T455 377Q318 419 124 419H95ZM55 -205Q55 -175 64 -142T92 -76T145 -22T222 -1Q288 -1 362 -66Q369 -72 372 -75T378 -79T382 -81T384 -79Q389 -74 439 21Q483 100 490 111T504 122Q510 122 518 118T526 103Q526 101 510 69T467 -12T419 -99L413 -112L433 -128Q498 -180 553 -180Q605 -180 646 -139Q672 -112 681 -77T693 -21T706 -1Q719 -1 719 -33Q719 -39 717 -57Q708 -141 655 -190Q625 -224 586 -232Q568 -237 551 -237Q487 -237 413 -172L391 -155Q391 -157 335 -255Q297 -325 286 -342T268 -359Q260 -359 254 -353T248 -339T304 -230L359 -126Q359 -124 337 -107T302 -81Q262 -57 221 -57Q170 -57 130 -93T84 -201Q82 -236 70 -236Q55 -236 55 -205",
      8937: "679 459Q693 445 693 439Q693 430 679 419H651Q455 419 319 377Q231 347 184 300T124 181Q120 161 116 155T102 148Q95 148 89 154T82 168Q82 192 97 237Q111 275 137 306T188 355T249 391T307 414T361 429T399 439Q397 440 364 447T298 467T224 499T149 557T97 641Q82 686 82 710Q82 717 88 723T102 730L115 725Q118 722 124 697Q137 625 184 578T319 501Q456 459 651 459H679ZM55 -205Q55 -175 64 -142T92 -76T145 -22T222 -1Q288 -1 362 -66Q369 -72 372 -75T378 -79T382 -81T384 -79Q389 -74 439 21Q483 100 490 111T504 122Q510 122 518 118T526 103Q526 101 510 69T467 -12T419 -99L413 -112L433 -128Q498 -180 553 -180Q605 -180 646 -139Q672 -112 681 -77T693 -21T706 -1Q719 -1 719 -33Q719 -39 717 -57Q708 -141 655 -190Q625 -224 586 -232Q568 -237 551 -237Q487 -237 413 -172L391 -155Q391 -157 335 -255Q297 -325 286 -342T268 -359Q260 -359 254 -353T248 -339T304 -230L359 -126Q359 -124 337 -107T302 -81Q262 -57 221 -57Q170 -57 130 -93T84 -201Q82 -236 70 -236Q55 -236 55 -205",
      8938: "693 -30Q686 -41 673 -41Q661 -41 506 34L346 110L280 -44Q228 -162 216 -185T193 -208Q177 -208 173 -192Q173 -186 242 -30T311 128Q271 145 184 186T86 236Q82 240 82 246Q82 251 86 259Q96 267 271 350L449 434L506 565Q537 635 551 664T571 700T582 706Q587 706 593 701T600 690Q600 679 553 572Q504 463 504 461L586 501Q672 539 673 539Q679 539 693 525V-30ZM653 10V488L566 445L480 405L422 276Q415 260 405 236T388 199T376 171T368 151T366 145Q368 143 510 77T653 10ZM422 374Q422 376 420 376T285 313T151 248Q315 168 326 163Q415 356 422 374",
      8939: "82 525Q96 539 102 539Q103 539 122 530T186 501T266 463L426 388Q428 388 495 541Q564 694 569 699Q573 706 581 706Q587 706 593 702T600 691Q600 676 533 528Q515 486 506 465T485 418T470 381T466 370Q466 369 575 316Q676 269 689 259Q693 253 693 248Q693 242 689 236Q688 235 506 145Q328 63 324 59Q324 50 266 -70Q224 -169 214 -188T193 -208Q177 -208 173 -192Q173 -183 222 -77Q244 -29 257 2T269 34L186 -6Q108 -43 99 -43Q93 -43 82 -30V525ZM271 416Q129 485 126 485H125Q122 485 122 250Q122 10 124 10L211 50L295 92L411 350Q411 351 271 416ZM624 248L449 332L440 319Q434 297 393 214Q353 121 353 119Q355 119 489 182T624 248",
      8940: "82 -123Q82 -114 93 -103H166L238 -101L293 50Q349 200 349 204L220 266Q166 291 140 304T100 325T84 336T82 344Q82 353 94 360Q112 372 282 453L473 541L482 568Q487 578 529 693Q559 785 569 795Q573 802 581 802Q587 802 593 797T599 786Q599 775 564 675L526 570Q526 568 561 584T633 617T673 635Q679 635 693 621V66Q686 54 679 54Q665 54 526 119Q491 137 458 153T405 177T386 184Q385 182 334 42T282 -101T482 -103H680Q682 -105 684 -108T688 -113T691 -118T693 -124Q693 -134 682 -141L473 -143H266L238 -219Q217 -278 210 -290T193 -303Q178 -303 173 -287Q173 -279 198 -214L222 -145Q222 -143 158 -143L93 -141L86 -136Q82 -131 82 -123ZM653 106V584L506 513L453 370Q442 339 428 300T407 243T400 224Q403 222 527 164T653 106ZM453 486Q453 488 451 488T300 417T151 344L224 308Q247 298 285 279T331 257L364 241L453 486",
      8941: "82 621Q96 635 102 635T249 568L420 486L449 473L469 533Q563 789 569 797Q573 801 581 801Q598 801 600 786Q602 781 544 617L484 455Q531 435 584 408Q677 364 689 355Q693 351 693 344Q693 339 692 337T676 325T631 302T538 257Q504 241 465 223T406 195T386 186Q384 184 333 44T282 -101Q282 -103 482 -103H680Q682 -105 684 -108T688 -113T691 -118T693 -124Q693 -134 682 -141L473 -143H266L238 -219Q217 -278 210 -290T193 -303Q178 -303 173 -287Q173 -279 198 -214L222 -145Q222 -143 158 -143L93 -141L86 -136Q82 -131 82 -123Q82 -114 93 -103H166L238 -101L333 159Q326 159 220 106Q110 54 97 54Q89 54 82 66V621ZM298 501Q155 567 142 575L122 584V344Q122 106 124 106Q125 106 180 132T291 185T351 213Q355 217 393 326L433 435Q433 436 298 501ZM549 381Q472 417 471 417L406 241Q408 240 516 291T624 344L549 381",
      8942: "78 30Q78 54 95 72T138 90Q162 90 180 74T199 31Q199 6 182 -12T139 -30T96 -13T78 30ZM78 440Q78 464 95 482T138 500Q162 500 180 484T199 441Q199 416 182 398T139 380T96 397T78 440ZM78 840Q78 864 95 882T138 900Q162 900 180 884T199 841Q199 816 182 798T139 780T96 797T78 840",
      8943: "78 250Q78 274 95 292T138 310Q162 310 180 294T199 251Q199 226 182 208T139 190T96 207T78 250ZM525 250Q525 274 542 292T585 310Q609 310 627 294T646 251Q646 226 629 208T586 190T543 207T525 250ZM972 250Q972 274 989 292T1032 310Q1056 310 1074 294T1093 251Q1093 226 1076 208T1033 190T990 207T972 250",
      8945: "133 760Q133 784 150 802T193 820Q217 820 235 804T254 761Q254 736 237 718T194 700T151 717T133 760ZM580 460Q580 484 597 502T640 520Q664 520 682 504T701 461Q701 436 684 418T641 400T598 417T580 460ZM1027 160Q1027 184 1044 202T1087 220Q1111 220 1129 204T1148 161Q1148 136 1131 118T1088 100T1045 117T1027 160",
      8965: "55 698Q56 708 70 716H540Q554 708 555 698Q555 694 554 691T552 686T549 682T546 680T542 678T539 676H71Q70 676 68 677T65 679T61 682T58 686T56 691T55 698ZM555 18Q554 12 549 6T536 0H535Q525 0 515 17T459 132Q430 194 410 235L305 455L199 233Q176 185 147 125T105 36T90 7Q85 0 75 0Q63 0 58 11Q55 15 55 21Q58 31 170 266T285 507Q295 522 305 522T320 515Q322 513 439 268L555 24V18",
      8966: "55 795Q56 806 70 813H540Q554 806 555 795Q555 791 554 788T552 783T549 779T546 777T542 775T539 773H71Q70 773 68 774T65 776T61 779T58 783T56 788T55 795ZM55 601Q56 612 70 619H540Q554 612 555 601Q555 597 554 594T552 589T549 585T546 583T542 581T539 579H71Q70 579 68 580T65 582T61 585T58 589T56 594T55 601ZM75 -97Q67 -97 61 -91T55 -81Q55 -71 169 166T285 410Q295 425 305 425Q311 425 316 422T323 414L325 410Q327 404 441 167T555 -81Q555 -85 549 -91T535 -97T520 -90Q519 -89 506 -61T463 27T411 136L305 358L199 136Q167 71 129 -10Q98 -75 92 -86T75 -97",
      8968: "174 734Q178 746 190 750H298H369Q400 750 411 747T422 730T411 713T372 709Q365 709 345 709T310 710H214V-235Q206 -248 196 -250Q192 -250 189 -249T184 -247T180 -244T178 -241T176 -237T174 -234V734",
      8969: "21 717T21 730T32 746T75 750H147H256Q266 742 269 735V-235Q262 -248 251 -250Q247 -250 244 -249T239 -247T235 -244T233 -241T231 -237T229 -234V710H133Q119 710 99 710T71 709Q43 709 32 713",
      8970: "174 734Q174 735 175 737T177 740T180 744T184 747T189 749T196 750Q206 748 214 735V-210H310H373Q401 -210 411 -213T422 -230T411 -247T369 -251Q362 -251 338 -251T298 -250H190Q178 -246 174 -234V734",
      8971: "229 734Q229 735 230 737T232 740T235 744T239 747T244 749T251 750Q262 748 269 735V-235Q266 -240 256 -249L147 -250H77Q43 -250 32 -247T21 -230T32 -213T72 -209Q79 -209 99 -209T133 -210H229V734",
      8988: "76 306Q62 306 59 319T55 386V500V596Q55 664 57 676T68 692Q71 694 250 694Q428 694 432 692Q444 685 444 674Q444 665 432 656Q428 654 261 654H95V487Q95 355 95 336T90 312Q84 306 76 306",
      8989: "424 306Q418 306 413 310T406 318L404 321V654H238Q71 654 68 656Q55 662 55 674T68 692Q71 694 250 694H379Q432 694 438 688Q443 683 443 662T444 500T444 338T438 312Q432 306 424 306",
      8990: "55 172V287Q55 341 58 353T76 366Q88 366 95 351V18H261Q428 18 432 16Q444 9 444 -2Q444 -11 432 -20Q428 -22 250 -22H120Q67 -22 61 -16Q56 -11 56 10T55 172",
      8991: "404 351Q410 366 424 366Q437 366 440 353T444 288V172V72Q444 8 443 -4T432 -20Q428 -22 250 -22Q71 -22 68 -20Q55 -14 55 -2T68 16Q71 18 238 18H404V351",
      8994: "55 141Q55 149 72 174T125 234T209 303T329 360T478 388H526Q649 383 765 319Q814 291 858 250T923 179T944 141Q944 133 938 128T924 122Q914 124 912 125T902 139Q766 328 500 328Q415 328 342 308T225 258T150 199T102 148T84 124Q81 122 75 122Q55 127 55 141",
      8995: "923 378Q944 378 944 358Q944 345 912 311T859 259Q710 134 500 134Q288 134 140 259Q55 336 55 358Q55 366 61 372T75 378Q78 378 84 376Q86 376 101 356T147 310T221 257T339 212T500 193Q628 193 734 236Q841 282 903 363Q914 378 923 378",
      9001: "333 -232Q332 -239 327 -244T313 -250Q303 -250 296 -240Q293 -233 202 6T110 250T201 494T296 740Q299 745 306 749L309 750Q312 750 313 750Q331 750 333 732Q333 727 243 489Q152 252 152 250T243 11Q333 -227 333 -232",
      9002: "55 732Q56 739 61 744T75 750Q85 750 92 740Q95 733 186 494T278 250T187 6T92 -240Q85 -250 75 -250Q67 -250 62 -245T55 -232Q55 -227 145 11Q236 248 236 250T145 489Q55 727 55 732",
      9136: "357 741V726Q357 720 349 715Q261 655 242 539Q240 526 240 454T239 315T239 247Q240 235 240 124V40Q240 -17 233 -53T201 -130Q155 -206 78 -244H69H64Q58 -244 57 -243T56 -234Q56 -232 56 -231V-225Q56 -218 63 -215Q153 -153 170 -39Q172 -25 173 119V219Q173 245 174 249Q173 258 173 376V460Q173 515 178 545T201 611Q244 695 327 741L334 744H354L357 741",
      9137: "78 744Q153 706 196 640T239 492V376Q239 341 239 314T238 271T238 253Q239 251 239 223V119V49Q239 -39 254 -85Q263 -111 275 -134T301 -172T326 -197T346 -213T356 -221T357 -232V-241L354 -244H334Q264 -209 222 -146T174 -12Q173 -6 173 95Q173 134 173 191T174 250Q173 258 173 382V451Q173 542 159 585Q145 626 120 658T75 706T56 723V731Q56 741 57 742T66 744H78",
      9168: "312 0V602H355V0H312",
      9416: "451 -175Q328 -175 226 -115T66 47T8 267Q8 303 15 342T39 431T94 531T186 622Q239 663 307 686T424 709H440Q604 709 716 622Q757 592 788 555T838 482T869 414T886 350T892 301T894 267Q894 147 835 45T674 -116T451 -175ZM854 268Q854 375 802 467T657 614T450 670Q283 670 166 552T49 267Q49 99 167 -18T453 -136Q617 -136 735 -18T854 268ZM273 378Q273 430 309 474T409 527Q411 527 417 527T428 528Q498 528 549 484L567 505Q583 528 590 528H594Q600 528 606 522V350L600 344H586Q577 344 574 344T569 347T566 357Q542 491 432 491Q389 491 365 465T340 407Q340 391 344 378T358 356T377 340T400 328T421 321T443 316T459 313Q499 305 517 300T559 279T601 238Q629 195 629 148Q629 80 583 33T471 -14Q392 -14 330 30Q312 6 293 -13Q292 -14 285 -14Q279 -14 273 -8V77V138Q273 160 275 165T286 170H294H307Q313 164 313 158Q313 108 350 67T471 26Q512 26 537 54T562 119Q562 137 558 151T544 176T527 193T504 205T483 212T459 218T441 222Q391 232 368 241T318 273Q273 316 273 378",
      9484: "76 306Q62 306 59 319T55 386V500V596Q55 664 57 676T68 692Q71 694 250 694Q428 694 432 692Q444 685 444 674Q444 665 432 656Q428 654 261 654H95V487Q95 355 95 336T90 312Q84 306 76 306",
      9488: "424 306Q418 306 413 310T406 318L404 321V654H238Q71 654 68 656Q55 662 55 674T68 692Q71 694 250 694H379Q432 694 438 688Q443 683 443 662T444 500T444 338T438 312Q432 306 424 306",
      9492: "55 172V287Q55 341 58 353T76 366Q88 366 95 351V18H261Q428 18 432 16Q444 9 444 -2Q444 -11 432 -20Q428 -22 250 -22H120Q67 -22 61 -16Q56 -11 56 10T55 172",
      9496: "404 351Q410 366 424 366Q437 366 440 353T444 288V172V72Q444 8 443 -4T432 -20Q428 -22 250 -22Q71 -22 68 -20Q55 -14 55 -2T68 16Q71 18 238 18H404V351",
      9585: "19 -195Q13 -195 7 -188T0 -176Q0 -169 18 -151L822 683Q835 694 840 694T852 688T860 674Q860 667 810 614T460 252Q57 -167 44 -179Q27 -195 19 -195",
      9586: "0 675Q0 681 6 687T19 694Q27 694 44 678L460 247Q759 -62 809 -115T860 -175Q860 -183 852 -189T840 -195Q835 -195 822 -184L18 649Q0 667 0 675",
      9632: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71",
      9633: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71ZM682 40V649H95V40H682",
      9642: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71",
      9650: "99 -20Q84 -11 84 0Q84 5 148 145T278 424L342 563Q347 575 360 575Q368 575 375 570Q376 569 441 430T571 148T637 0Q637 -11 622 -20H99",
      9651: "75 0L72 2Q69 3 67 5T62 11T59 20Q59 24 62 30Q65 37 245 370T428 707Q428 708 430 710T436 714T444 716Q451 716 455 712Q459 710 644 368L828 27V20Q828 7 814 0H75ZM610 347L444 653Q443 653 278 347T113 40H775Q775 42 610 347",
      9652: "99 -20Q84 -11 84 0Q84 5 148 145T278 424L342 563Q347 575 360 575Q368 575 375 570Q376 569 441 430T571 148T637 0Q637 -11 622 -20H99",
      9653: "75 0L72 2Q69 3 67 5T62 11T59 20Q59 24 62 30Q65 37 245 370T428 707Q428 708 430 710T436 714T444 716Q451 716 455 712Q459 710 644 368L828 27V20Q828 7 814 0H75ZM610 347L444 653Q443 653 278 347T113 40H775Q775 42 610 347",
      9654: "83 523Q83 524 85 527T92 535T103 539Q107 539 389 406T680 268Q694 260 694 249Q694 239 687 234Q685 232 395 95L107 -41H101Q90 -40 83 -26V523",
      9656: "83 523Q83 524 85 527T92 535T103 539Q107 539 389 406T680 268Q694 260 694 249Q694 239 687 234Q685 232 395 95L107 -41H101Q90 -40 83 -26V523",
      9657: "26 489Q33 505 43 505Q51 505 260 385Q464 266 471 259Q473 257 473 250Q473 242 469 239Q459 231 260 115Q51 -5 43 -5Q39 -5 35 -1T28 7L26 11V489ZM412 250L67 450Q66 450 66 250T67 50Q69 51 240 150T412 250",
      9660: "84 556Q84 567 99 576H622Q637 567 637 556Q637 551 572 409T441 127T375 -14Q368 -19 360 -19H358Q349 -19 342 -7T296 92Q249 193 211 275Q84 550 84 556",
      9661: "59 480Q59 485 61 489T66 495T72 498L75 500H814Q828 493 828 480V474L644 132Q458 -210 455 -212Q451 -215 444 -215T433 -212Q429 -210 342 -49T164 282T64 466Q59 478 59 480ZM775 460H113Q113 459 278 153T444 -153T610 153T775 460",
      9662: "84 556Q84 567 99 576H622Q637 567 637 556Q637 551 572 409T441 127T375 -14Q368 -19 360 -19H358Q349 -19 342 -7T296 92Q249 193 211 275Q84 550 84 556",
      9663: "59 480Q59 485 61 489T66 495T72 498L75 500H814Q828 493 828 480V474L644 132Q458 -210 455 -212Q451 -215 444 -215T433 -212Q429 -210 342 -49T164 282T64 466Q59 478 59 480ZM775 460H113Q113 459 278 153T444 -153T610 153T775 460",
      9664: "694 -26Q686 -40 676 -41H670L382 95Q92 232 90 234Q83 239 83 249Q83 262 96 267Q101 270 379 401T665 537Q671 539 674 539Q686 539 694 524V-26",
      9666: "694 -26Q686 -40 676 -41H670L382 95Q92 232 90 234Q83 239 83 249Q83 262 96 267Q101 270 379 401T665 537Q671 539 674 539Q686 539 694 524V-26",
      9667: "473 10Q466 -5 454 -5Q451 -5 445 -3Q444 -3 343 56T140 173T35 234Q26 239 26 250T35 266Q40 269 240 384T445 503Q451 505 453 505Q466 505 473 490V10ZM433 50T433 250T432 450T259 351T87 250T258 150T432 50Q433 50 433 250",
      9674: "318 709Q325 716 332 716Q340 716 344 713T474 511Q611 298 611 292Q611 285 526 152Q494 103 474 72Q347 -128 344 -130Q340 -132 333 -132T322 -130Q319 -128 257 -31T131 169T60 278Q56 285 56 292Q56 298 60 305Q73 326 194 516T318 709ZM567 290T567 291T451 475T333 658L100 293Q100 288 215 108L333 -74Q334 -74 450 108",
      9711: "56 250Q56 353 95 442T196 589T335 681T491 715Q573 715 635 693Q694 673 747 635T846 543T917 412T944 250Q944 58 815 -78T500 -215Q457 -215 429 -210Q274 -183 165 -56T56 250ZM500 -176Q664 -176 784 -54T904 250Q904 418 799 536T543 674Q534 675 493 675Q425 675 357 647T229 567T133 432T96 250Q96 160 129 80T217 -56T346 -144T500 -176",
      9723: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71ZM682 40V649H95V40H682",
      9724: "71 0Q59 4 55 16V346L56 676Q64 686 70 689H709Q719 681 722 674V15Q719 10 709 1L390 0H71",
      9733: "367 395Q374 416 398 492T442 627T463 688Q463 692 467 692Q471 694 472 694Q478 694 484 680T523 562Q553 469 576 400L577 395H731H819Q872 395 883 394T895 384Q895 380 891 376T832 333Q794 305 767 285Q643 195 643 194L690 47Q737 -96 737 -103Q737 -111 727 -111Q721 -111 594 -18L472 71L350 -18Q223 -111 217 -111Q207 -111 207 -103Q207 -96 254 47L301 194Q301 195 241 239T118 328T51 378Q49 382 49 384Q49 392 58 393T110 395H213H367",
      9824: "181 -21Q134 -21 96 27T55 193Q55 224 58 247T82 317T143 410Q172 443 234 498Q282 543 314 598T360 687T380 725Q386 727 389 727Q395 727 398 725T406 716T413 702T423 677T439 641Q481 556 544 498Q633 420 678 353T723 204Q723 142 711 94T669 12T590 -21Q520 -21 490 8T459 66V70H409V62Q409 22 416 -17T430 -82T437 -112Q437 -131 407 -131Q403 -131 397 -131T389 -130T382 -130T372 -131Q341 -131 341 -111Q341 -107 348 -82T362 -18T369 62V70H319V66Q319 57 314 44T297 16T257 -10T191 -21H181",
      9825: "55 490Q55 557 71 604T114 674T167 706T222 716Q279 716 322 684T389 605Q391 610 395 617T414 643T447 677T494 704T555 716Q642 716 682 652T723 490Q723 455 718 426T684 342T602 227Q573 196 537 161T485 110T449 63T412 -8Q408 -22 404 -27T389 -33Q382 -33 379 -31T372 -23T366 -8T355 18T335 54Q319 81 298 104T239 163T176 227Q102 310 79 371T55 490ZM198 674Q143 664 119 613T95 491Q95 415 137 346Q174 282 265 194T384 48L389 39Q391 42 397 54T406 71T415 86T427 104T442 122T464 146T491 172Q571 249 613 303Q683 396 683 487Q683 581 649 631Q613 676 556 676Q495 676 457 634T410 538Q407 514 390 514Q386 514 380 517Q372 520 369 536T355 581T319 635Q277 675 223 675H217H208L204 674Q200 674 198 674",
      9826: "370 714Q370 717 375 722T388 727Q398 727 403 721T417 697Q420 692 421 689Q536 465 709 304Q723 291 723 282T709 260Q529 93 406 -153Q402 -162 390 -162H389Q379 -162 376 -158T357 -125Q247 89 89 241L64 265Q55 272 55 282Q55 287 57 290T64 300T77 312T98 331T127 361Q197 435 258 523T344 663L370 714ZM655 299Q568 384 508 470T389 662L376 638Q362 613 341 577T289 497T215 399T123 299L105 282L123 265Q210 180 270 94T389 -98L402 -74Q416 -49 437 -13T489 67T563 165T655 265L673 282L655 299",
      9827: "213 532Q213 615 265 670T389 726Q461 726 513 671T565 532Q565 511 562 492T553 458T541 432T526 409T512 393T498 379L490 371L511 326Q512 326 516 330T528 341T546 353T572 363T606 368Q664 368 707 315T750 174Q750 87 699 33T579 -22Q567 -22 553 -20T517 -10T479 16T459 63V70H409V62Q409 22 416 -17T430 -82T437 -112Q437 -131 407 -131Q403 -131 397 -131T389 -130T382 -130T372 -131Q341 -131 341 -111Q341 -107 348 -82T362 -18T369 62V70H319V63Q315 25 281 2T197 -22Q132 -22 80 32T28 174Q28 255 69 311T175 368Q192 368 207 364T232 353T250 341T262 331T267 326L288 371L280 378Q272 385 267 391T253 407T238 430T226 457T217 492T213 532",
      9837: "200 467Q254 467 293 428T332 321Q332 147 104 -11L88 -22H75Q62 -22 56 -16L55 362V647Q55 743 60 748Q63 750 76 750H83Q87 750 95 744V434L104 440Q144 467 200 467ZM237 322Q237 360 225 388T183 417Q158 417 134 407T101 378Q96 370 96 349T95 197V34Q152 91 194 167T237 322",
      9838: "65 721Q78 734 94 734Q100 734 104 727V444L116 449Q129 454 157 465T208 486Q313 527 314 527Q318 527 324 521V-210Q306 -223 294 -223Q289 -223 284 -216V-13L270 -18Q257 -24 231 -34T180 -54Q77 -96 74 -96T65 -90V721ZM104 13Q282 84 283 85Q284 85 284 252Q284 418 283 418L230 396L140 360L104 346V13",
      9839: "101 -223Q94 -223 93 -217T91 -188V-151Q91 -88 90 -88Q87 -88 80 -92T68 -96Q62 -96 56 -90L55 -50V-22Q55 -8 58 -4T78 5L91 10V177Q91 343 90 343Q87 343 80 339T68 335Q62 335 56 341L55 381V409Q55 423 58 427T78 436L91 441V543V616Q91 643 93 648T106 656Q119 662 126 659Q130 657 130 645T131 554V456L257 503V607L258 710L260 712Q261 715 272 719T286 723Q293 723 295 715T297 671V617Q297 519 298 519Q301 519 307 522T319 526Q327 526 333 521V437L330 435Q328 432 312 427L297 421V254Q297 88 298 88Q301 88 307 91T319 95Q327 95 333 90V6L330 4Q328 1 312 -4L297 -10V-78V-122Q297 -145 295 -149T282 -156Q274 -160 268 -160Q257 -160 257 -130V-89V-25L131 -72V-210Q123 -215 116 -218T104 -222L101 -223ZM257 72V406L131 359V25L257 72",
      10003: "84 231Q84 244 114 264T170 285Q176 285 183 274T224 205Q267 129 268 129Q271 141 279 163T318 250T389 378T502 523T662 673Q702 706 732 706H734Q749 706 749 695Q749 682 730 666T660 607T559 505Q387 299 328 29Q324 0 295 -17T245 -34H241Q234 -34 225 -21T185 46Q166 79 154 101Q84 223 84 231",
      10016: "195 702T195 706T201 716H632Q638 710 638 706T636 700T621 690Q436 581 427 374V357H430Q554 357 645 421Q682 447 711 483T755 542T770 567Q775 572 786 563V131Q777 125 774 125T762 139Q709 228 642 274T482 333Q452 337 430 337H427V320Q430 279 437 247T462 170T521 82T621 4Q630 -2 633 -4T637 -7T638 -12Q638 -16 632 -22H201Q195 -16 195 -12T197 -6T212 4Q397 113 406 320V337H403Q279 337 188 273Q151 247 122 211T78 152T63 127Q58 122 48 131V563Q54 569 59 569Q62 569 71 555Q124 466 191 420T351 361Q381 357 403 357H406V374Q403 415 396 447T371 525T312 613T212 690Q199 697 197 699",
      10072: "139 -249H137Q125 -249 119 -235V251L120 737Q130 750 139 750Q152 750 159 735V-235Q151 -249 141 -249H139",
      10216: "333 -232Q332 -239 327 -244T313 -250Q303 -250 296 -240Q293 -233 202 6T110 250T201 494T296 740Q299 745 306 749L309 750Q312 750 313 750Q331 750 333 732Q333 727 243 489Q152 252 152 250T243 11Q333 -227 333 -232",
      10217: "55 732Q56 739 61 744T75 750Q85 750 92 740Q95 733 186 494T278 250T187 6T92 -240Q85 -250 75 -250Q67 -250 62 -245T55 -232Q55 -227 145 11Q236 248 236 250T145 489Q55 727 55 732",
      10222: "357 741V726Q357 720 349 715Q261 655 242 539Q240 526 240 394V331Q240 259 239 250Q240 242 240 119V49Q240 -42 254 -85Q263 -111 275 -134T301 -172T326 -197T346 -213T356 -221T357 -232V-241L354 -244H334Q264 -209 222 -146T174 -12Q173 -6 173 95Q173 134 173 191T174 250Q173 260 173 376V460Q173 515 178 545T201 611Q244 695 327 741L334 744H354L357 741",
      10223: "78 744Q153 706 196 640T239 492V376Q239 339 239 311T238 269T238 252Q240 236 240 124V40Q240 -18 233 -53T202 -130Q156 -206 79 -244H70H65Q58 -244 57 -242T56 -231T57 -220T64 -215Q153 -154 170 -39Q173 -18 174 119V247Q173 249 173 382V451Q173 542 159 585Q145 626 120 658T75 706T56 723V731Q56 741 57 742T66 744H78",
      10229: "165 270H1510Q1525 262 1525 250T1510 230H165Q167 228 182 216T211 189T244 152T277 96T303 25Q308 7 308 0Q308 -11 288 -11Q281 -11 278 -11T272 -7T267 2T263 21Q245 94 195 151T73 236Q58 242 55 247Q55 254 59 257T73 264Q121 283 158 314T215 375T247 434T264 480L267 497Q269 503 270 505T275 509T288 511Q308 511 308 500Q308 493 303 475Q293 438 278 406T246 352T215 315T185 287T165 270",
      10230: "84 237T84 250T98 270H1444Q1328 357 1301 493Q1301 494 1301 496T1300 499Q1300 511 1317 511H1320Q1329 511 1332 510T1338 506T1341 497T1344 481T1352 456Q1374 389 1425 336T1544 261Q1553 258 1553 250Q1553 244 1548 241T1524 231T1486 212Q1445 186 1415 152T1370 85T1349 35T1341 4Q1339 -6 1336 -8T1320 -11Q1300 -11 1300 0Q1300 7 1305 25Q1337 151 1444 230H98Q84 237 84 250",
      10231: "165 270H1694Q1578 357 1551 493Q1551 494 1551 496T1550 499Q1550 511 1567 511H1570Q1579 511 1582 510T1588 506T1591 497T1594 481T1602 456Q1624 389 1675 336T1794 261Q1803 258 1803 250Q1803 244 1798 241T1774 231T1736 212Q1695 186 1665 152T1620 85T1599 35T1591 4Q1589 -6 1586 -8T1570 -11Q1550 -11 1550 0Q1550 7 1555 25Q1587 151 1694 230H165Q167 228 182 216T211 189T244 152T277 96T303 25Q308 7 308 0Q308 -11 288 -11Q281 -11 278 -11T272 -7T267 2T263 21Q245 94 195 151T73 236Q58 242 55 247Q55 254 59 257T73 264Q121 283 158 314T215 375T247 434T264 480L267 497Q269 503 270 505T275 509T288 511Q308 511 308 500Q308 493 303 475Q293 438 278 406T246 352T215 315T185 287T165 270",
      10232: "274 173H1539Q1540 172 1542 171T1545 169T1547 167T1550 164T1551 162T1552 158T1553 153Q1553 140 1538 133H318L328 123Q379 69 414 0Q419 -13 419 -17Q419 -24 399 -24Q388 -24 385 -23T377 -12Q332 77 253 144T72 237Q62 240 59 242T56 250T59 257T70 262T89 268T119 278T160 296Q303 366 377 512Q382 522 385 523T401 525Q419 524 419 515Q419 510 414 500Q379 431 328 377L318 367H1538Q1553 359 1553 347Q1553 336 1539 328L1221 327H903L900 328L602 327H274L264 319Q225 289 147 250Q148 249 165 241T210 217T264 181L274 173",
      10233: "1218 514Q1218 525 1234 525Q1239 525 1242 525T1247 525T1251 524T1253 523T1255 520T1257 517T1260 512Q1297 438 1358 381T1469 300T1565 263Q1582 258 1582 250T1573 239T1536 228T1478 204Q1334 134 1260 -12Q1256 -21 1253 -22T1238 -24Q1218 -24 1218 -17Q1218 -13 1223 0Q1258 69 1309 123L1319 133H70Q56 140 56 153Q56 168 72 173H1363L1373 181Q1412 211 1490 250Q1489 251 1472 259T1427 283T1373 319L1363 327H710L707 328L390 327H72Q56 332 56 347Q56 360 70 367H1319L1309 377Q1276 412 1247 458T1218 514",
      10234: "1438 514Q1438 525 1454 525Q1459 525 1462 525T1467 525T1471 524T1473 523T1475 520T1477 517T1480 512Q1517 438 1578 381T1689 300T1785 263Q1802 258 1802 250T1793 239T1756 228T1698 204Q1554 134 1480 -12Q1476 -21 1473 -22T1458 -24Q1438 -24 1438 -17Q1438 -13 1443 0Q1478 69 1529 123L1539 133H318L328 123Q379 69 414 0Q419 -13 419 -17Q419 -24 399 -24Q388 -24 385 -23T377 -12Q332 77 253 144T72 237Q62 240 59 242T56 250T59 257T70 262T89 268T119 278T160 296Q303 366 377 512Q382 522 385 523T401 525Q419 524 419 515Q419 510 414 500Q379 431 328 377L318 367H1539L1529 377Q1496 412 1467 458T1438 514ZM274 173H1583L1593 181Q1632 211 1710 250Q1709 251 1692 259T1647 283T1593 319L1583 327H930L927 328L602 327H274L264 319Q225 289 147 250Q148 249 165 241T210 217T264 181L274 173",
      10236: "95 155V109Q95 83 92 73T75 63Q61 63 58 74T54 130Q54 140 54 180T55 250Q55 421 57 425Q61 437 75 437Q88 437 91 428T95 393V345V270H1444Q1328 357 1301 493Q1301 494 1301 496T1300 499Q1300 511 1317 511H1320Q1329 511 1332 510T1338 506T1341 497T1344 481T1352 456Q1374 389 1425 336T1544 261Q1553 258 1553 250Q1553 244 1548 241T1524 231T1486 212Q1445 186 1415 152T1370 85T1349 35T1341 4Q1339 -6 1336 -8T1320 -11Q1300 -11 1300 0Q1300 7 1305 25Q1337 151 1444 230H95V155",
      10731: "318 709Q325 716 332 716Q340 716 344 713T474 511Q611 298 611 292Q611 285 526 152Q494 103 474 72Q347 -128 344 -130Q340 -132 333 -132T322 -130Q319 -128 257 -31T131 169T60 278Q56 285 56 292Q56 298 60 305Q73 326 194 516T318 709",
      10744: "166 -215T159 -215T147 -212T141 -204T139 -197Q139 -190 144 -183Q157 -157 378 274T602 707Q605 716 618 716Q625 716 630 712T636 703T638 696Q638 691 406 241T170 -212Q166 -215 159 -215",
      10752: "555 -250Q420 -250 306 -185T124 -4T56 250Q56 453 193 595T526 749Q528 750 539 750Q554 750 562 749Q688 749 800 687T983 508T1054 250Q1054 112 987 -3T806 -184T555 -250ZM555 -165Q672 -165 767 -108T916 44T970 250Q970 418 861 532T600 664Q591 665 548 665Q446 665 353 614T200 466T140 250V243Q140 88 248 -30Q262 -46 280 -62T338 -105T434 -148T555 -165ZM478 250Q478 288 503 307T551 326Q586 326 609 305T632 250Q632 217 610 196T555 174T500 196T478 250",
      10753: "555 -250Q420 -250 306 -185T124 -4T56 250Q56 453 193 595T526 749Q528 750 539 750Q554 750 562 749Q688 749 800 687T983 508T1054 250Q1054 112 987 -3T806 -184T555 -250ZM513 478Q513 664 512 664Q504 664 481 660T406 637T313 588Q281 564 255 537T211 483T181 431T161 382T150 342T144 310T141 292H513V478ZM798 588Q758 616 711 634T639 658T602 663L597 664V292H969Q969 293 967 309T960 341T949 381T930 430T900 482T856 537T798 588ZM513 -164V208H141Q142 205 144 189T149 160T158 125T173 83T196 39T229 -9Q249 -34 273 -55T318 -92T363 -119T405 -138T444 -150T475 -158T499 -162T513 -164ZM775 -103Q801 -87 823 -68T863 -30T894 10T919 49T937 88T950 123T959 154T964 180T968 198L969 208H597V-164Q599 -163 616 -161T647 -155T683 -145T728 -128T775 -103",
      10754: "555 -250Q420 -250 306 -185T124 -4T56 250Q56 453 193 595T526 749Q528 750 539 750Q554 750 562 749Q688 749 800 687T983 508T1054 250Q1054 112 987 -3T806 -184T555 -250ZM600 664Q591 665 548 665Q414 665 306 583L292 573L423 441L555 310L687 441L818 573L804 583Q714 650 600 664ZM364 118L495 250L364 382L232 513L223 500Q140 391 140 250Q140 107 223 0L232 -13L364 118ZM970 250Q970 389 887 501L878 512Q878 513 861 496T812 447T746 381L615 250L746 118L878 -13L887 0Q970 109 970 250ZM687 59L555 190L423 59L292 -73L306 -83Q416 -166 555 -166T804 -83L818 -73L687 59",
      10756: "96 750Q103 750 109 748T120 744T127 737T133 730T137 723T139 718V395L140 73L142 60Q159 -43 237 -104T416 -166Q521 -166 597 -103T690 60L692 73L694 718Q708 749 735 749Q765 749 775 720Q777 714 777 398Q777 78 776 71Q766 -51 680 -140Q571 -249 416 -249H411Q261 -249 152 -140Q66 -51 56 71Q55 78 55 398Q55 714 57 720Q60 734 70 740Q80 750 96 750ZM223 276Q223 282 224 287T227 296T232 302T238 308T243 313T250 316L254 319H374V376V406Q374 438 382 454T418 470Q443 467 450 453T458 410V376V319H579Q580 319 583 317T589 313T594 308T600 302T604 295T608 287T609 276Q609 253 587 241Q577 235 513 235H458V178Q458 176 458 166T459 148Q459 84 415 84Q401 84 390 93T375 117Q374 120 374 178V235H319Q317 235 307 235T290 234Q223 234 223 276",
      10758: "777 -217Q766 -244 745 -249H88Q64 -242 57 -220Q55 -214 55 250T57 720Q60 734 70 740Q80 750 96 750Q127 750 137 720Q139 714 139 274V-166H693V274Q693 714 695 720Q705 749 735 749Q766 749 775 719Q777 713 777 248V-217",
      10799: "630 29Q630 9 609 9Q604 9 587 25T493 118L389 222L284 117Q178 13 175 11Q171 9 168 9Q160 9 154 15T147 29Q147 36 161 51T255 146L359 250L255 354Q174 435 161 449T147 471Q147 480 153 485T168 490Q173 490 175 489Q178 487 284 383L389 278L493 382Q570 459 587 475T609 491Q630 491 630 471Q630 464 620 453T522 355L418 250L522 145Q606 61 618 48T630 29",
      10815: "28 660Q28 676 31 679T46 683H50Q87 681 182 681Q217 681 247 681T294 682T315 682Q321 682 323 682T328 679T331 673T332 660Q332 643 328 640T304 637Q239 637 231 626Q229 620 229 334V46H520V334Q520 620 518 626Q510 637 445 637Q426 637 422 640T417 660Q417 675 420 678T432 682H435Q437 682 467 682T569 681T671 681T703 682Q714 682 717 679T721 660Q721 643 717 640T693 637Q628 637 620 626Q619 623 619 342Q619 60 620 57Q628 46 693 46Q714 46 717 43T721 23Q721 5 715 1Q713 0 374 0Q36 0 34 1Q28 5 28 23Q28 40 31 43T56 46Q121 46 129 57Q131 63 131 342Q131 620 129 626Q121 637 56 637Q35 637 32 640T28 660",
      10846: "55 795Q56 806 70 813H540Q554 806 555 795Q555 791 554 788T552 783T549 779T546 777T542 775T539 773H71Q70 773 68 774T65 776T61 779T58 783T56 788T55 795ZM55 601Q56 612 70 619H540Q554 612 555 601Q555 597 554 594T552 589T549 585T546 583T542 581T539 579H71Q70 579 68 580T65 582T61 585T58 589T56 594T55 601ZM75 -97Q67 -97 61 -91T55 -81Q55 -71 169 166T285 410Q295 425 305 425Q311 425 316 422T323 414L325 410Q327 404 441 167T555 -81Q555 -85 549 -91T535 -97T520 -90Q519 -89 506 -61T463 27T411 136L305 358L199 136Q167 71 129 -10Q98 -75 92 -86T75 -97",
      10877: "674 636Q682 636 688 630T694 615T687 601Q686 600 417 472L151 346L399 228Q687 92 691 87Q694 81 694 76Q694 58 676 56H670L382 192Q92 329 90 331Q83 336 83 348Q84 359 96 365Q104 369 382 500T665 634Q669 636 674 636ZM94 170Q102 172 104 172Q110 171 254 103T535 -30T678 -98Q694 -106 694 -118Q694 -136 676 -138H670L382 -2Q92 135 90 137Q83 142 83 154Q84 164 94 170",
      10878: "83 616Q83 624 89 630T99 636Q107 636 253 568T543 431T687 361Q694 356 694 346T687 331Q685 329 395 192L107 56H101Q83 58 83 76Q83 77 83 79Q82 86 98 95Q117 105 248 167Q326 204 378 228L626 346L360 472Q291 505 200 548Q112 589 98 597T83 616ZM674 172Q692 172 694 154Q694 142 687 137Q685 135 395 -2L107 -138H101Q83 -136 83 -118Q83 -106 96 -100Q100 -98 380 35T665 170T674 172",
      10885: "667 761Q669 762 673 762Q682 762 688 756T694 741Q694 731 687 727Q684 724 420 613L156 502L416 392Q476 367 544 338T647 295T682 280Q694 274 694 262Q694 244 676 242Q670 242 524 303T235 425T90 487Q83 493 83 501Q83 514 94 519Q97 520 378 639T667 761ZM55 -23Q55 43 103 90T223 138Q265 138 316 114Q342 100 393 68L443 36Q502 0 554 0Q609 0 650 32T694 109Q694 138 708 138Q710 138 713 136T719 127T722 108Q722 37 673 -9T557 -56Q514 -56 468 -35T387 13T308 60T223 82Q167 82 127 50T83 -27Q81 -56 69 -56Q55 -56 55 -23ZM55 -257Q55 -191 103 -144T223 -96Q265 -96 316 -120Q342 -134 393 -166L443 -198Q502 -234 554 -234Q609 -234 650 -202T694 -125Q694 -96 708 -96Q710 -96 713 -98T719 -107T722 -126Q722 -197 673 -243T557 -290Q514 -290 468 -269T387 -221T308 -174T223 -152Q167 -152 127 -184T83 -261Q80 -290 69 -290Q55 -290 55 -257",
      10886: "90 727Q83 734 83 743Q83 751 89 756T104 762Q111 760 396 641Q686 518 687 517Q694 512 694 502T687 487Q686 486 543 425T253 303T101 242Q83 244 83 262Q83 274 95 280Q96 280 130 294T232 338T361 392L621 502L357 613Q93 724 90 727ZM55 -23Q55 43 103 90T223 138Q265 138 316 114Q342 100 393 68L443 36Q502 0 554 0Q609 0 650 32T694 109Q694 138 708 138Q710 138 713 136T719 127T722 108Q722 37 673 -9T557 -56Q514 -56 468 -35T387 13T308 60T223 82Q167 82 127 50T83 -27Q81 -56 69 -56Q55 -56 55 -23ZM55 -257Q55 -191 103 -144T223 -96Q265 -96 316 -120Q342 -134 393 -166L443 -198Q502 -234 554 -234Q609 -234 650 -202T694 -125Q694 -96 708 -96Q710 -96 713 -98T719 -107T722 -126Q722 -197 673 -243T557 -290Q514 -290 468 -269T387 -221T308 -174T223 -152Q167 -152 127 -184T83 -261Q80 -290 69 -290Q55 -290 55 -257",
      10887: "380 497Q445 528 522 564T636 618T673 635Q680 635 686 628T693 615T689 603Q686 599 418 472L151 343L418 215Q686 88 689 83Q693 79 693 72T687 59T675 52Q669 52 381 189T86 332Q82 337 82 344Q82 350 86 355Q91 359 380 497ZM82 -130T82 -124T95 -103H380L431 -54Q476 -6 486 -6Q491 -6 498 -12T505 -27Q505 -28 505 -29T504 -32Q503 -33 498 -38T485 -53T469 -70L438 -103H680Q682 -106 686 -110T691 -116T693 -123Q693 -130 680 -143H398L346 -192Q300 -241 291 -241Q271 -241 271 -221Q271 -209 306 -179L340 -143H95Q82 -130 82 -124",
      10888: "82 614Q82 620 86 625T94 632T100 635Q106 635 394 498T689 355Q693 349 693 343Q693 338 689 332Q683 327 395 190T100 52Q95 52 89 58T82 72Q82 80 95 88Q114 99 358 215L624 343L358 472Q89 599 86 603Q82 607 82 614ZM82 -130T82 -124T95 -103H380L431 -54Q476 -6 486 -6Q491 -6 498 -12T505 -27Q505 -28 505 -29T504 -32Q503 -33 498 -38T485 -53T469 -70L438 -103H680Q682 -106 686 -110T691 -116T693 -123Q693 -130 680 -143H398L346 -192Q300 -241 291 -241Q271 -241 271 -221Q271 -209 306 -179L340 -143H95Q82 -130 82 -124",
      10889: "86 512Q93 518 381 639T673 761Q680 761 686 755T693 741Q693 733 688 730Q685 723 419 612L155 501L419 390Q685 277 688 272Q693 269 693 261Q693 254 687 248T675 241Q669 241 381 362T86 490Q74 500 86 512ZM70 -59Q57 -59 57 -24Q57 40 104 87Q116 102 146 118Q186 136 231 136Q232 136 242 135T258 133T276 128T302 118T334 101T377 74Q386 69 396 63T411 53T417 50Q435 87 453 134Q491 223 495 227Q498 230 505 230Q513 230 519 225T526 212Q526 203 491 118T453 30Q453 22 489 10T553 -3Q589 -3 622 14Q653 28 669 50T688 90T694 122T706 136Q718 136 718 114Q718 113 718 109T717 103Q717 31 668 -14T554 -60Q529 -60 499 -50T451 -32T433 -24Q431 -24 404 -90T375 -157Q375 -159 402 -178T473 -218T553 -239Q599 -239 641 -211T691 -130Q694 -99 706 -99T718 -122Q718 -123 718 -127T717 -133Q717 -204 668 -249T559 -295Q512 -295 470 -275T355 -206L322 -290Q313 -310 304 -332T289 -367T282 -382Q277 -387 270 -387Q262 -387 255 -382T248 -368Q248 -361 322 -186Q311 -177 280 -166T222 -155Q189 -155 153 -173Q122 -186 106 -208T87 -248T82 -280T71 -294Q57 -294 57 -259Q57 -195 104 -148Q122 -126 155 -113T220 -99Q245 -99 276 -109T324 -127T342 -135Q397 -2 397 1Q386 10 367 23T302 58T222 80Q175 80 132 52T84 -28Q82 -59 70 -59",
      10890: "86 730Q81 734 81 740Q81 747 88 754T99 761Q103 761 392 640T688 512Q693 509 693 501T688 490Q682 484 394 363T99 241Q94 241 88 248T82 261Q82 268 86 272Q89 277 355 390L619 501L355 612Q89 723 86 730ZM70 -59Q57 -59 57 -24Q57 40 104 87Q116 102 146 118Q186 136 231 136Q232 136 242 135T258 133T276 128T302 118T334 101T377 74Q386 69 396 63T411 53T417 50Q435 87 453 134Q491 223 495 227Q498 230 505 230Q513 230 519 225T526 212Q526 203 491 118T453 30Q453 22 489 10T553 -3Q589 -3 622 14Q653 28 669 50T688 90T694 122T706 136Q718 136 718 114Q718 113 718 109T717 103Q717 31 668 -14T554 -60Q529 -60 499 -50T451 -32T433 -24Q431 -24 404 -90T375 -157Q375 -159 402 -178T473 -218T553 -239Q599 -239 641 -211T691 -130Q694 -99 706 -99T718 -122Q718 -123 718 -127T717 -133Q717 -204 668 -249T559 -295Q512 -295 470 -275T355 -206L322 -290Q313 -310 304 -332T289 -367T282 -382Q277 -387 270 -387Q262 -387 255 -382T248 -368Q248 -361 322 -186Q311 -177 280 -166T222 -155Q189 -155 153 -173Q122 -186 106 -208T87 -248T82 -280T71 -294Q57 -294 57 -259Q57 -195 104 -148Q122 -126 155 -113T220 -99Q245 -99 276 -109T324 -127T342 -135Q397 -2 397 1Q386 10 367 23T302 58T222 80Q175 80 132 52T84 -28Q82 -59 70 -59",
      10891: "674 1003Q681 1003 687 999T694 983Q694 973 683 967Q669 959 420 868L162 772L422 676Q683 579 685 577Q694 571 694 560Q694 550 687 546T673 541Q669 542 384 647T93 755Q83 760 83 772Q83 783 91 788Q98 791 383 897T674 1003ZM84 354T84 367T98 387H679Q694 379 694 367Q694 354 679 347H98Q84 354 84 367ZM84 160T84 173T98 193H679Q694 185 694 173Q694 160 679 153H98Q84 160 84 173ZM94 -3Q102 -1 104 -1Q107 -2 392 -107T684 -215Q694 -219 694 -232Q694 -241 687 -247Q686 -248 395 -357Q106 -463 101 -463Q83 -461 83 -443Q83 -431 94 -426Q97 -423 357 -328L615 -232L355 -136Q94 -39 92 -37Q83 -31 83 -21Q83 -9 94 -3",
      10892: "104 541Q98 541 91 545T83 560Q83 571 92 577Q94 579 355 676L615 772L357 868Q108 959 94 967Q83 973 83 983Q83 989 87 996T104 1003Q109 1002 396 896T687 787Q694 781 694 772Q694 759 684 755Q678 752 393 647T104 541ZM84 367Q84 380 98 387H679Q694 379 694 367Q694 356 680 348L390 347H100Q84 352 84 367ZM84 173Q84 188 100 193H680Q694 183 694 173Q694 160 679 153H98Q84 160 84 173ZM674 -1Q682 -1 688 -6T694 -20Q694 -31 685 -37Q683 -39 422 -136L162 -232L420 -328Q680 -423 683 -426Q694 -431 694 -443Q694 -461 676 -463Q671 -463 382 -357Q91 -248 90 -247Q83 -242 83 -232Q83 -220 93 -215Q667 -1 674 -1",
      10901: "674 636Q682 636 688 631T694 616Q694 605 687 601Q685 599 395 462L107 326H101Q83 328 83 345Q83 358 96 365Q102 367 382 500T665 634Q671 636 674 636ZM674 442Q692 442 694 424Q694 412 687 407Q686 406 417 278L151 152L399 34Q687 -102 691 -107Q694 -113 694 -118Q694 -136 676 -138H670L382 -2Q92 135 90 137Q83 142 83 154Q84 165 96 171Q104 175 382 306T665 440Q669 442 674 442",
      10902: "83 616Q83 624 89 630T99 636Q107 636 253 568T543 431T687 361Q694 354 694 346Q694 328 676 326H670L382 462Q317 493 226 535Q119 585 101 595T83 616ZM94 440Q102 442 104 442Q110 441 254 373T535 240T678 172Q679 172 680 171Q694 164 694 153T687 137Q685 135 395 -2L107 -138H101Q83 -136 83 -118Q83 -106 93 -101L128 -84Q163 -68 230 -36T361 26L626 152L360 278Q91 406 90 407Q83 412 83 424Q84 434 94 440",
      10927: "84 346Q84 359 91 363T117 367Q120 367 126 367T137 366Q388 370 512 430T653 609Q657 636 676 636Q685 635 689 629T694 618V612Q689 566 672 528T626 463T569 417T500 383T435 362T373 346Q379 345 404 339T440 330T477 318T533 296Q592 266 630 223T681 145T694 78Q694 57 674 57Q662 57 657 67T652 92T640 135T606 191Q500 320 137 326H114Q104 326 98 327T88 332T84 346ZM84 -131T84 -118T98 -98H679Q694 -106 694 -118T679 -138H98Q84 -131 84 -118",
      10928: "84 614Q84 636 102 636Q115 636 119 626T125 600T137 556T171 501Q277 372 640 366H661Q694 366 694 346T661 326H640Q578 325 526 321T415 307T309 280T222 237T156 172T124 83Q122 66 118 62T103 57Q100 57 98 57T95 58T93 59T90 62T85 67Q83 71 83 80Q88 126 105 164T151 229T208 275T277 309T342 330T404 346Q401 347 380 351T345 360T302 373T245 396Q125 455 92 565Q84 599 84 614ZM84 -131T84 -118T98 -98H679Q694 -106 694 -118T679 -138H98Q84 -131 84 -118",
      10933: "653 734Q653 738 660 745T673 752T686 745T693 723Q672 555 466 485Q390 463 378 463Q373 463 373 461Q373 458 378 458Q390 458 466 436Q562 404 620 350Q682 283 693 198Q693 183 686 176Q681 170 674 170T660 176T653 187Q653 192 652 200T646 228T631 265T602 307T555 350Q435 431 151 441H95Q82 454 82 460T95 481H151Q165 482 197 483T238 485Q427 500 528 554T649 707Q653 729 653 734ZM82 33Q82 37 83 40T89 47T95 54H473L520 105Q569 156 571 156Q573 157 578 157Q586 157 592 151T598 136Q598 130 562 92L526 56L604 54H682Q693 43 693 35Q693 31 692 28T686 21T680 14H489L342 -139L513 -142H682Q693 -148 693 -160Q693 -167 680 -182H304L258 -230Q248 -240 237 -251T221 -268T211 -278T203 -284T197 -286Q189 -286 184 -280T178 -264Q178 -257 213 -219L249 -182H171L93 -179L86 -175Q82 -170 82 -163Q82 -155 95 -142H289L360 -64L433 14H262L93 16Q82 23 82 33",
      10934: "693 466T693 460T680 441H624Q608 439 577 438T538 436Q349 421 248 367T126 214Q122 192 122 187Q122 183 116 177T102 170Q95 170 89 176Q82 183 82 198Q93 283 155 350Q213 404 309 436Q385 458 398 458Q402 458 402 461Q402 463 398 463Q385 463 309 485Q103 555 82 723Q82 738 89 745T102 752T115 745T122 734Q122 721 126 701T155 640T220 572Q340 490 624 481H680Q693 466 693 460ZM82 33Q82 37 83 40T89 47T95 54H473L520 105Q569 156 571 156Q573 157 578 157Q586 157 592 151T598 136Q598 130 562 92L526 56L604 54H682Q693 43 693 35Q693 31 692 28T686 21T680 14H489L342 -139L513 -142H682Q693 -148 693 -160Q693 -167 680 -182H304L258 -230Q248 -240 237 -251T221 -268T211 -278T203 -284T197 -286Q189 -286 184 -280T178 -264Q178 -257 213 -219L249 -182H171L93 -179L86 -175Q82 -170 82 -163Q82 -155 95 -142H289L360 -64L433 14H262L93 16Q82 23 82 33",
      10935: "82 494T82 501T95 521H171Q405 527 511 569Q630 618 651 732Q652 734 653 740T655 748T658 754T663 759T672 761L686 754Q693 747 693 734Q684 668 648 623Q627 591 573 557T442 507L417 501Q428 496 442 494Q520 478 573 444T648 378Q684 333 693 267Q693 254 686 247Q673 234 659 245Q657 247 651 269Q630 383 511 432Q406 474 171 481H95Q82 494 82 501ZM70 -59Q57 -59 57 -26Q57 30 90 73T177 132Q191 136 226 136Q228 136 239 136T253 135T267 132T287 125T311 113T346 95T391 67Q462 20 502 5Q519 1 553 1Q586 1 602 5Q641 18 664 45T691 107Q694 136 704 136Q717 136 717 115V105Q717 39 671 -9T554 -58Q518 -58 481 -43T382 14Q302 63 273 74Q255 78 222 78Q188 78 173 74Q90 46 84 -28Q82 -59 70 -59ZM71 -294Q57 -294 57 -262Q57 -205 90 -162T177 -104Q191 -99 226 -99Q266 -103 277 -106Q310 -119 391 -168Q455 -212 502 -231Q519 -235 553 -235Q586 -235 602 -231Q640 -218 661 -195T686 -151T693 -115T704 -99Q717 -99 717 -121V-131Q717 -198 671 -246T556 -294Q519 -294 482 -279T382 -222Q307 -175 273 -162Q255 -157 222 -157Q188 -157 173 -162Q133 -175 110 -201T84 -264Q82 -294 71 -294",
      10936: "693 501Q693 493 679 481H604Q369 474 264 432Q143 382 124 269Q116 246 115 245Q101 234 88 247Q82 254 82 267Q89 329 126 378Q147 410 201 444T333 494L357 501Q354 502 340 505T318 510T295 516T269 525T243 535T215 548T188 565Q142 599 126 623Q89 672 82 734Q82 761 102 761L115 756Q116 755 124 732Q143 619 264 569Q371 527 604 521H679Q693 507 693 501ZM70 -59Q57 -59 57 -26Q57 30 90 73T177 132Q191 136 226 136Q228 136 239 136T253 135T267 132T287 125T311 113T346 95T391 67Q462 20 502 5Q519 1 553 1Q586 1 602 5Q641 18 664 45T691 107Q694 136 704 136Q717 136 717 115V105Q717 39 671 -9T554 -58Q518 -58 481 -43T382 14Q302 63 273 74Q255 78 222 78Q188 78 173 74Q90 46 84 -28Q82 -59 70 -59ZM71 -294Q57 -294 57 -262Q57 -205 90 -162T177 -104Q191 -99 226 -99Q266 -103 277 -106Q310 -119 391 -168Q455 -212 502 -231Q519 -235 553 -235Q586 -235 602 -231Q640 -218 661 -195T686 -151T693 -115T704 -99Q717 -99 717 -121V-131Q717 -198 671 -246T556 -294Q519 -294 482 -279T382 -222Q307 -175 273 -162Q255 -157 222 -157Q188 -157 173 -162Q133 -175 110 -201T84 -264Q82 -294 71 -294",
      10937: "82 494T82 501T95 521H171Q256 523 317 528T441 548T543 584T613 644T651 732Q652 734 653 740T655 748T658 754T663 759T672 761L686 754Q693 747 693 734Q686 686 664 647T615 586T548 545T482 518T417 501Q419 500 451 493T517 471T590 434T657 367T693 267Q693 241 673 241Q664 241 659 245Q656 249 650 273T635 323T593 380T511 432Q406 474 171 481H95Q82 494 82 501ZM57 -26Q57 39 101 87T219 136Q254 136 277 130Q320 114 382 72Q419 50 424 45Q426 45 459 110Q496 178 497 179Q500 180 504 180Q509 180 517 175T526 161Q526 158 495 90L462 25Q462 21 502 5Q519 1 553 1Q586 1 602 5Q641 18 664 45T691 107Q694 136 706 136T718 115Q718 114 718 111T717 105Q717 39 671 -9T554 -58L459 -33Q450 -29 444 -27T437 -26L371 -155L391 -168Q485 -235 538 -235H553Q586 -235 602 -230Q683 -204 691 -128Q694 -99 706 -99T718 -120Q718 -121 718 -124T717 -130Q717 -199 670 -246T557 -294T393 -228Q353 -205 351 -201Q348 -201 315 -266Q294 -310 285 -323T268 -337Q259 -337 254 -331T248 -317Q248 -305 282 -246L313 -181Q313 -177 273 -161Q255 -157 222 -157Q188 -157 173 -161Q134 -174 113 -198T88 -242T82 -278T71 -294Q57 -294 57 -261Q57 -204 91 -161T179 -104Q195 -99 228 -99Q274 -102 315 -124Q337 -132 337 -130L404 -1L384 12Q319 58 273 74Q255 79 222 79Q188 79 173 74Q133 61 112 37T88 -7T82 -43T70 -59Q57 -59 57 -26",
      10938: "693 501Q693 493 679 481H604Q548 479 509 477T418 469T331 454T257 429T194 392T150 340T124 270Q117 247 115 245Q101 236 88 247Q82 254 82 267Q89 330 126 379Q147 411 202 444T333 494L357 501Q239 531 188 565Q142 599 126 623Q89 672 82 734Q82 761 102 761L115 756Q116 755 124 732Q133 678 166 640T241 579T349 544T470 527T604 521H679Q693 507 693 501ZM57 -26Q57 39 101 87T219 136Q254 136 277 130Q320 114 382 72Q419 50 424 45Q426 45 459 110Q496 178 497 179Q500 180 504 180Q509 180 517 175T526 161Q526 158 495 90L462 25Q462 21 502 5Q519 1 553 1Q586 1 602 5Q641 18 664 45T691 107Q694 136 706 136T718 115Q718 114 718 111T717 105Q717 39 671 -9T554 -58L459 -33Q450 -29 444 -27T437 -26L371 -155L391 -168Q485 -235 538 -235H553Q586 -235 602 -230Q683 -204 691 -128Q694 -99 706 -99T718 -120Q718 -121 718 -124T717 -130Q717 -199 670 -246T557 -294T393 -228Q353 -205 351 -201Q348 -201 315 -266Q294 -310 285 -323T268 -337Q259 -337 254 -331T248 -317Q248 -305 282 -246L313 -181Q313 -177 273 -161Q255 -157 222 -157Q188 -157 173 -161Q134 -174 113 -198T88 -242T82 -278T71 -294Q57 -294 57 -261Q57 -204 91 -161T179 -104Q195 -99 228 -99Q274 -102 315 -124Q337 -132 337 -130L404 -1L384 12Q319 58 273 74Q255 79 222 79Q188 79 173 74Q133 61 112 37T88 -7T82 -43T70 -59Q57 -59 57 -26",
      10949: "84 463Q84 585 166 663T360 752Q361 752 370 752T395 752T430 752T475 753T524 753H679Q694 746 694 733Q694 724 681 714L522 713H470H441Q366 713 338 709T266 685Q244 674 224 659T179 617T139 550T124 463V458Q124 370 185 302Q244 238 328 220Q348 215 366 215T522 213H681Q694 203 694 193Q694 180 679 173H526Q510 173 480 173T434 172Q350 172 289 188T172 258Q84 340 84 463ZM84 -14T84 -1T98 19H679Q694 11 694 -1Q694 -14 679 -21H98Q84 -14 84 -1ZM84 -208T84 -195T98 -175H679Q694 -183 694 -195Q694 -208 679 -215H98Q84 -208 84 -195",
      10950: "83 733Q83 746 98 753H251Q267 753 297 753T343 754Q427 754 488 738T605 668Q693 587 693 463Q693 378 650 312T545 213T415 174Q407 173 251 173H98Q83 180 83 193Q83 203 96 213H255H308H337Q412 213 439 217T512 241Q533 252 553 267T599 309T639 376T654 463Q654 554 592 624Q557 662 512 685Q468 704 439 708T335 713H306H255L96 714Q83 724 83 733ZM84 -14T84 -1T98 19H679Q694 11 694 -1Q694 -14 679 -21H98Q84 -14 84 -1ZM84 -208T84 -195T98 -175H679Q694 -183 694 -195Q694 -208 679 -215H98Q84 -208 84 -195",
      10955: "693 221Q693 214 680 201H524Q398 201 367 202T309 212Q236 230 180 280T98 398Q84 438 84 492T98 585Q126 663 193 716T346 781Q347 781 373 781T440 782T520 783H680Q682 780 686 776T691 770T693 763T680 743H526Q364 743 353 741Q279 730 221 687T138 578Q124 540 124 492T138 405Q163 340 221 297T353 243Q364 241 526 241H680Q682 238 686 234T691 228T693 221ZM82 -48T82 -41T95 -19H462L513 41L569 105Q574 110 582 110T596 104T602 90Q602 87 600 83Q600 77 555 30L515 -17L600 -19H682Q693 -30 693 -38T680 -59H480L415 -137L349 -213L515 -215H682Q693 -226 693 -233T680 -255H313L260 -317Q224 -360 212 -372T192 -385Q184 -385 179 -377T173 -362Q174 -361 218 -306L260 -255H178L93 -253L86 -248Q82 -243 82 -235Q82 -226 93 -215H195L295 -213L362 -137L426 -59H260L93 -57L86 -53Q82 -48 82 -41",
      10956: "82 759T82 763T83 769T89 776T95 783H251Q378 783 409 782T469 772Q540 753 596 703T678 585Q691 546 691 492T678 398Q649 320 581 267T426 203Q415 201 251 201H95Q82 214 82 221Q82 225 83 227T89 234T95 241H249Q411 241 422 243Q496 253 554 296T638 405Q651 444 651 492Q651 539 638 578Q613 643 555 686T422 741Q411 743 249 743H95Q82 759 82 763ZM82 -48T82 -41T95 -19H462L513 41L569 105Q574 110 582 110T596 104T602 90Q602 87 600 83Q600 77 555 30L515 -17L600 -19H682Q693 -30 693 -38T680 -59H480L415 -137L349 -213L515 -215H682Q693 -226 693 -233T680 -255H313L260 -317Q224 -360 212 -372T192 -385Q184 -385 179 -377T173 -362Q174 -361 218 -306L260 -255H178L93 -253L86 -248Q82 -243 82 -235Q82 -226 93 -215H195L295 -213L362 -137L426 -59H260L93 -57L86 -53Q82 -48 82 -41",
      12296: "333 -232Q332 -239 327 -244T313 -250Q303 -250 296 -240Q293 -233 202 6T110 250T201 494T296 740Q299 745 306 749L309 750Q312 750 313 750Q331 750 333 732Q333 727 243 489Q152 252 152 250T243 11Q333 -227 333 -232",
      12297: "55 732Q56 739 61 744T75 750Q85 750 92 740Q95 733 186 494T278 250T187 6T92 -240Q85 -250 75 -250Q67 -250 62 -245T55 -232Q55 -227 145 11Q236 248 236 250T145 489Q55 727 55 732",
      57350: "91 417Q104 430 111 430T131 417V301L171 341Q201 373 207 378T220 384Q227 384 233 377T240 366Q240 357 187 299L131 244V-10Q116 -23 111 -23T91 -10V201L49 157Q20 127 14 121T0 115Q-8 115 -14 121T-20 132Q-20 139 17 178Q29 191 36 199L91 257V417",
      57351: "56 417Q68 431 76 431L89 426L96 419V317L98 215L193 273L291 330V375L293 419Q301 431 311 431Q331 431 331 388L333 355L356 370Q381 384 388 384Q394 384 400 377T407 363Q407 354 367 328L331 308V-10Q316 -23 310 -23Q300 -23 293 -12L291 135V284L98 168L96 77V-12Q84 -24 76 -24L62 -19L58 -12L56 66V144L31 128Q5 114 -2 114Q-8 114 -14 121T-20 136Q-20 142 -14 147T20 170L56 190V417",
      57352: "477 261Q477 257 473 256T455 253T417 251T348 250H235L155 -77L146 -82Q137 -85 109 -85Q55 -85 55 -77L139 261Q224 596 226 598Q229 603 239 603Q240 603 254 603T290 603T341 604T405 605T477 605Q656 603 687 602T719 596Q719 589 692 588T513 585H319L282 427L242 272Q242 270 351 270Q388 270 410 270T444 269T460 267T469 265T477 261",
      57353: "228 325Q170 322 156 316T127 309Q108 309 104 314Q99 319 99 322T108 341Q125 376 171 400T268 425H271Q302 425 319 396Q328 377 328 358Q328 332 324 314Q311 270 286 221Q274 194 274 192H275Q339 234 484 325T639 421Q669 434 691 434T723 425T734 406Q734 394 719 381Q715 376 644 330L575 287L566 267Q543 233 526 176Q520 160 515 143T508 115T506 105Q506 103 533 103Q585 103 607 110T641 118Q670 118 670 107Q670 100 661 85Q643 50 598 27T504 3Q465 3 450 36Q441 51 441 73Q441 84 444 96Q452 146 484 205L497 236L324 125Q143 12 135 10Q103 -6 77 -6Q61 -6 49 2T37 21Q37 36 49 46T124 96L195 141L204 156Q219 179 243 248T264 323Q264 325 228 325",
      57356: "86 472Q93 477 381 614T673 752Q680 752 686 746T693 732T689 721Q686 715 418 590L151 461L418 332Q684 207 689 201Q693 195 693 190Q693 183 687 177T675 170Q668 170 380 307T86 450Q82 454 82 461Q82 467 86 472ZM369 101V126Q369 156 382 156H384Q385 157 386 157Q409 157 409 115V98V54H680Q693 39 693 34T680 14H409V-142H680Q693 -155 693 -162Q693 -167 680 -182H409V-273Q396 -284 388 -284Q382 -284 369 -275V-182H95Q82 -167 82 -162Q82 -155 95 -142H369V14H95Q93 17 89 21T84 27T82 34T83 40T89 47T95 54H369V101",
      57357: "89 745Q95 752 100 752Q106 752 394 615T689 472Q693 468 693 461T689 450Q684 445 396 308T100 170Q95 170 89 176T82 190Q82 195 86 201Q91 208 358 332L624 461L358 590Q90 715 86 721Q82 725 82 731Q82 739 89 745ZM369 101V126Q369 156 382 156H384Q385 157 386 157Q409 157 409 115V98V54H680Q693 39 693 34T680 14H409V-142H680Q693 -155 693 -162Q693 -167 680 -182H409V-273Q396 -284 388 -284Q382 -284 369 -275V-182H95Q82 -167 82 -162Q82 -155 95 -142H369V14H95Q93 17 89 21T84 27T82 34T83 40T89 47T95 54H369V101",
      57358: "97 172Q82 172 82 190Q82 197 86 201Q94 209 173 246T327 319T402 357Q405 360 434 448T462 539L278 628Q96 713 86 721Q82 725 82 732T88 745T102 752Q103 752 125 742T198 709T293 666Q342 642 385 622T453 590T478 579Q479 579 506 659T562 824T598 915Q602 919 609 919T622 913T629 901Q629 898 571 728Q546 656 531 608T518 559Q555 539 602 519Q664 488 679 479T694 461Q694 457 689 450Q680 443 616 413T494 356T435 326L389 190L342 57L513 55H682Q694 43 694 34Q694 28 689 21L682 17L506 15H329L322 -8Q320 -13 310 -41T295 -85L275 -141H680Q682 -143 684 -146T688 -151T691 -156T693 -162Q693 -172 682 -179L473 -181H262L220 -303Q192 -388 185 -404T166 -421Q160 -421 153 -415T146 -403Q146 -400 179 -302T220 -185Q220 -181 158 -181L93 -179L86 -174Q82 -169 82 -161Q82 -152 93 -141H164L233 -139L260 -63L286 15H189L93 17L86 21Q82 26 82 34Q82 44 93 55H198L300 57L342 179Q350 204 361 238T378 286T382 301L246 237Q111 172 97 172ZM624 461Q621 464 560 492Q512 518 503 518Q500 518 500 517Q499 513 488 479T465 413T453 379L624 461",
      57359: "97 54Q82 54 82 72Q82 79 86 84Q95 91 222 153L351 215L398 324L442 433L258 519Q95 597 87 604Q82 608 82 615T88 628T102 635Q107 635 424 484L458 468L524 630Q593 789 597 795Q601 801 609 801Q616 801 622 795T629 781L562 615L493 450L589 406Q665 371 679 362T694 344Q694 339 693 337T677 326T631 302T538 257Q504 241 465 223T406 195T386 186Q383 185 344 92T306 -3L486 81Q662 168 673 168Q680 168 686 162T693 148T689 137Q688 136 482 35L280 -59L233 -176Q184 -291 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -279 185 -186T224 -90Q225 -88 223 -88Q219 -88 193 -101Q109 -143 98 -143Q82 -138 82 -122Q82 -116 85 -113T108 -98T171 -67L249 -30L289 61Q297 81 307 107T321 144T326 157L218 106Q109 54 97 54ZM553 379Q480 412 480 415Q479 415 460 372T423 285T406 241Q408 240 516 291T624 344L553 379",
      57360: "102 168Q103 168 151 146T247 102T295 81Q299 85 322 144T344 206L218 268Q153 297 123 313T87 333T82 344T86 355Q104 369 291 455Q491 552 491 553L542 673Q581 767 590 784T609 801Q616 801 622 795T629 781Q629 773 586 677Q546 581 546 577L609 606Q669 635 673 635Q680 635 686 629T693 615Q693 610 692 608T670 593T604 561L524 521L400 226L542 157Q617 123 649 107T687 85T694 72Q694 66 690 60T679 54Q665 54 526 119Q394 186 386 186Q385 186 342 88L331 61L509 -23Q680 -105 688 -111Q693 -115 693 -122T688 -135T675 -141H673Q664 -141 491 -59Q320 21 316 21H315L249 -136Q183 -293 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -282 154 -261T181 -197T213 -119L280 41Q280 46 186 86Q157 101 121 119Q92 133 87 136T82 148Q82 155 88 161T102 168ZM418 370L466 495Q464 495 308 420T151 344T204 317T311 267T364 244Q364 247 418 370",
      57361: "82 34Q82 44 93 55H198L300 57L342 179Q351 207 362 238T378 286T384 303T238 377Q109 435 86 450Q82 454 82 460T86 472Q90 476 302 579L511 679Q512 679 553 795Q569 842 577 866T592 903T600 917T608 919Q615 919 622 912T629 901Q629 899 595 799Q589 777 581 753T569 717T564 703L618 728Q666 752 673 752T686 746T693 732Q693 723 683 717T615 683L546 650L491 488Q464 410 450 368T438 326Q493 297 562 266Q660 219 677 209T694 190Q694 183 690 177T678 171Q664 171 546 228L424 286Q422 286 382 172L342 57L513 55H682Q694 43 694 34Q694 28 689 21L682 17L506 15H329L322 -8Q320 -13 310 -41T295 -85L275 -141H680Q682 -143 684 -146T688 -151T691 -156T693 -162Q693 -172 682 -179L473 -181H262L220 -303Q192 -388 185 -404T166 -421Q160 -421 153 -415T146 -403Q146 -400 179 -302T220 -185Q220 -181 158 -181L93 -179L86 -174Q82 -169 82 -161Q82 -152 93 -141H164L233 -139L260 -63L286 15H189L93 17L86 21Q82 26 82 34ZM495 623Q495 626 493 626T321 544T151 461L398 343Q399 343 405 360T423 415T446 483Q457 513 469 551T488 606T495 623",
      57366: "82 -6Q82 1 95 14H262L295 94Q331 171 331 174Q324 175 312 178T267 194T206 227T146 283T98 368Q84 406 84 461T98 554Q126 632 194 685T349 750Q360 752 480 752H591L604 783Q620 819 624 821Q631 828 640 828Q653 825 658 810Q658 808 646 781L635 754Q635 752 658 752Q680 752 686 746Q693 739 693 732Q693 728 692 726T686 719T680 712H615L506 466Q479 407 451 344T408 248T393 214Q393 210 535 210H680Q693 194 693 190T680 170H373L340 92L304 14H680Q693 1 693 -6Q693 -11 680 -26H286L253 -103L218 -179L451 -181H682Q694 -193 694 -201Q694 -212 682 -219L440 -221H200L178 -270Q160 -309 154 -319T139 -330Q122 -330 118 -312L155 -223Q155 -221 126 -221H95Q82 -206 82 -201T95 -181H175L206 -108Q237 -35 242 -30Q242 -26 169 -26H95Q82 -11 82 -6ZM571 710Q571 712 469 712Q443 712 416 712T371 711T351 710Q279 700 221 656T138 548Q124 508 124 461T138 374Q186 245 351 212L460 459Q571 709 571 710",
      57367: "82 -14T82 -7T95 15H431L529 170H435Q341 170 333 175Q149 218 98 368Q84 406 84 461Q84 515 98 555Q126 633 193 686T346 750Q347 750 373 750T440 751T520 752H680Q693 739 693 732Q693 727 680 712H526Q364 712 353 710Q268 700 207 646T126 512Q123 496 123 461T126 410Q141 350 180 304T280 232Q312 217 344 214T464 210H555L589 261Q613 301 620 311T635 321Q644 321 650 315T657 301Q657 296 651 286T630 252T604 212Q604 210 642 210H680Q693 197 693 190Q693 186 692 184T686 177T680 170H578L526 92L478 17L580 15H682Q693 4 693 -4T680 -25H451L353 -179L518 -181H682Q694 -193 694 -201Q694 -211 682 -219L504 -221H326L293 -272Q257 -332 246 -332Q238 -332 232 -326T225 -313Q225 -310 226 -308Q226 -305 251 -265T278 -223Q278 -221 186 -221H95Q93 -218 89 -214T84 -208T82 -201T95 -181H306L404 -25H249L93 -23L86 -19Q82 -14 82 -7",
      57368: "82 732Q82 739 95 752H251H348Q420 752 460 744T551 708Q566 697 566 701Q618 815 624 821Q631 828 640 828Q653 825 658 810L600 677Q600 671 615 656T653 605T689 517Q692 496 692 461T689 406Q668 325 615 266Q572 221 513 196T391 170H373L340 92L304 14H680Q693 1 693 -6Q693 -11 680 -26H286L253 -103L218 -179L451 -181H682Q694 -193 694 -201Q694 -212 682 -219L440 -221H200L178 -270Q160 -309 154 -319T139 -330Q122 -330 118 -312L155 -223Q155 -221 126 -221H95Q82 -206 82 -201T95 -181H175L206 -108Q237 -35 242 -30Q242 -26 169 -26H95Q82 -11 82 -6Q82 1 95 14H262L295 92L331 170H95Q93 172 91 175T87 180T84 185T82 191Q82 199 93 210H220L349 212L549 659Q507 692 462 702T338 712H249H95Q82 727 82 732ZM652 473Q652 513 636 552T603 611T582 632Q581 632 487 422T393 210Q424 210 460 220T535 253T605 316T649 410Q652 427 652 461V473",
      57369: "82 732Q82 739 95 752H251Q415 752 426 750Q539 736 615 657Q667 599 689 517Q692 496 692 461T689 406Q668 325 615 266Q522 170 382 170H355L326 95Q319 80 311 59T298 28T293 17Q293 15 486 15H680Q693 0 693 -6T680 -25H275L213 -179L449 -181H682Q693 -192 693 -199T680 -221H198L178 -270Q153 -333 139 -333Q132 -333 126 -327T119 -314T135 -266T153 -223Q153 -221 124 -221H95Q82 -207 82 -201T95 -181H171L233 -25H162L93 -23L86 -19Q82 -14 82 -7T95 15H251L313 170H202L93 172L86 177Q82 182 82 190Q82 199 93 210H211L329 212L349 261Q366 301 372 311T386 321Q392 321 399 315T407 302Q407 295 390 254T373 210Q374 209 377 209Q412 209 444 217Q512 231 564 273T638 377Q651 414 651 461Q651 509 638 548Q613 613 555 656T422 710Q411 712 249 712H95Q82 727 82 732",
      57370: "693 -115T693 -122T680 -144H315L269 -199Q221 -255 213 -255H212Q203 -255 197 -248T193 -231Q195 -225 229 -184L262 -144H186L113 -142L106 -137Q102 -130 102 -125Q102 -119 115 -104H298L426 52H386Q342 54 309 63Q236 79 180 129T98 249Q84 289 84 343Q84 398 98 436Q126 514 193 567T346 632Q347 632 373 632T440 633T520 634H680Q682 631 686 627T691 621T693 614T680 594H526Q364 594 353 592Q268 581 207 528T126 394Q123 378 123 343T126 292Q141 231 181 185T280 114Q329 92 415 92H462L506 147Q554 203 562 203H563Q572 203 578 196T582 178Q579 173 546 132L513 94L598 92H682Q693 81 693 73T680 52H480L349 -102L515 -104H682Q693 -115 693 -122",
      57371: "82 610T82 614T83 620T89 627T95 634H251Q378 634 409 633T469 623Q540 604 596 554T678 436Q691 397 691 343T678 249Q653 181 597 131T469 63Q427 52 362 52H315L213 -102L438 -104H662Q673 -115 673 -123Q673 -129 660 -144H186L151 -197Q114 -250 109 -253Q106 -254 104 -254Q100 -254 98 -253Q91 -253 87 -248T82 -235Q82 -230 109 -186L138 -144H115Q82 -144 82 -125Q82 -119 95 -104H166L266 49Q266 52 182 52H95Q82 65 82 72Q82 76 83 78T89 85T95 92H295L329 143Q365 195 369 198Q372 203 380 203Q385 203 391 197T398 185Q398 184 398 184L399 182Q399 175 369 129L344 94Q344 92 376 92Q402 92 422 94Q496 104 554 147T638 256Q651 295 651 343Q651 390 638 429Q613 494 555 537T422 592Q411 594 249 594H95Q82 610 82 614",
      119808: "296 0Q278 3 164 3Q58 3 49 0H40V62H92Q144 62 144 64Q388 682 397 689Q403 698 434 698Q463 698 471 689Q475 686 538 530T663 218L724 64Q724 62 776 62H828V0H817Q796 3 658 3Q509 3 485 0H472V62H517Q561 62 561 63L517 175H262L240 120Q218 65 217 64Q217 62 261 62H306V0H296ZM390 237L492 238L440 365Q390 491 388 491Q287 239 287 237H390",
      119809: "720 510Q720 476 704 448T665 404T619 377T580 362L564 359L583 356Q602 353 632 342T690 312Q712 292 725 276Q752 235 752 189V183Q752 160 741 125Q698 18 547 2Q543 1 288 0H39V62H147V624H39V686H264H409Q502 686 542 681T624 655Q720 607 720 510ZM563 513Q563 553 548 578T518 611T486 622Q479 624 385 624H293V382H375Q458 383 467 385Q563 405 563 513ZM590 192Q590 307 505 329Q504 330 503 330L398 331H293V62H391H400H444Q496 62 528 75T580 131Q590 155 590 192",
      119810: "64 343Q64 502 174 599T468 697Q502 697 533 691T586 674T623 655T647 639T657 632L694 663Q703 670 711 677T723 687T730 692T735 695T740 696T746 697Q759 697 762 692T766 668V627V489V449Q766 428 762 424T742 419H732H720Q699 419 697 436Q690 498 657 545Q611 618 532 632Q522 634 496 634Q356 634 286 553Q232 488 232 343T286 133Q355 52 497 52Q597 52 650 112T704 237Q704 248 709 251T729 254H735Q750 254 755 253T763 248T766 234Q766 136 680 63T469 -11Q285 -11 175 86T64 343",
      119811: "39 624V686H270H310H408Q500 686 545 680T638 649Q768 584 805 438Q817 388 817 338Q817 171 702 75Q628 17 515 2Q504 1 270 0H39V62H147V624H39ZM655 337Q655 370 655 390T650 442T639 494T616 540T580 580T526 607T451 623Q443 624 368 624H298V62H377H387H407Q445 62 472 65T540 83T606 129Q629 156 640 195T653 262T655 337",
      119812: "723 286Q721 284 700 145T677 3V0H39V62H147V618H39V680H660V676Q662 670 675 552T691 428V424H629V428Q629 429 627 448T618 494T601 541Q574 593 527 605T382 618H374H304V384H336Q338 384 347 384T361 384T376 386T392 390T407 397T421 407T432 423Q442 444 443 482V501H505V205H443V224Q442 258 435 278T411 307T380 318T336 322H304V62H375H394Q429 62 449 62T497 66T541 76T577 95T609 126T632 170T651 232Q661 287 661 289H723V286",
      119813: "425 0L228 3Q63 3 51 0H39V62H147V618H39V680H644V676Q647 670 659 552T675 428V424H613Q613 433 605 477Q599 511 589 535T562 574T530 599T488 612T441 617T387 618H368H304V371H333Q389 373 411 390T437 468V488H499V192H437V212Q436 244 430 263T408 292T378 305T333 309H304V62H439V0H425",
      119814: "465 -10Q281 -10 173 88T64 343Q64 413 85 471T143 568T217 631T298 670Q371 697 449 697Q452 697 459 697T470 696Q502 696 531 690T582 675T618 658T644 641T656 632L732 695Q734 697 745 697Q758 697 761 692T765 668V627V489V449Q765 428 761 424T741 419H731H724Q705 419 702 422T695 444Q683 520 631 577T495 635Q364 635 295 563Q261 528 247 477T232 343Q232 296 236 260T256 185T296 120T366 76T472 52Q481 51 498 51Q544 51 573 67T607 108Q608 111 608 164V214H464V276H479Q506 273 680 273Q816 273 834 276H845V214H765V113V51Q765 16 763 8T750 0Q742 2 709 16T658 40L648 46Q592 -10 465 -10",
      119815: "400 0Q376 3 226 3Q75 3 51 0H39V62H147V624H39V686H51Q75 683 226 683Q376 683 400 686H412V624H304V388H595V624H487V686H499Q523 683 673 683Q824 683 848 686H860V624H752V62H860V0H848Q824 3 674 3Q523 3 499 0H487V62H595V326H304V62H412V0H400",
      119816: "397 0Q370 3 218 3Q65 3 38 0H25V62H139V624H25V686H38Q65 683 218 683Q370 683 397 686H410V624H296V62H410V0H397",
      119817: "174 114Q174 96 169 82T159 63T144 47L155 45Q183 40 203 40Q271 40 290 104Q294 118 294 150T295 380V624H154V686H169Q196 683 365 683Q499 683 517 686H527V624H446V379Q446 183 446 153T441 108Q413 32 315 2Q266 -11 208 -11Q160 -11 118 -2T42 37T8 114V122Q8 150 30 174T91 198T152 174T174 122V114",
      119818: "400 0Q376 3 226 3Q75 3 51 0H39V62H147V624H39V686H51Q75 683 226 683Q376 683 400 686H412V624H304V338L472 483L634 624H565V686H576Q597 683 728 683Q814 683 829 686H836V624H730L614 524Q507 432 497 422Q496 422 498 418T514 395T553 342T627 241L759 63L805 62H852V0H842Q830 3 701 3Q550 3 526 0H513V62H549Q584 62 584 63Q583 65 486 196T388 328L304 256V62H412V0H400",
      119819: "643 285Q641 280 629 148T612 4V0H39V62H147V624H39V686H51Q75 683 228 683Q415 685 425 686H439V624H304V62H352H378Q492 62 539 138Q551 156 558 178T569 214T576 255T581 289H643V285",
      119820: "314 0Q296 3 181 3T48 0H39V62H147V624H39V686H305Q316 679 323 667Q330 653 434 414L546 157L658 414Q766 662 773 674Q778 681 788 686H1052V624H944V62H1052V0H1040Q1016 3 874 3T708 0H696V62H804V341L803 618L786 580Q770 543 735 462T671 315Q540 13 536 9Q528 1 507 1Q485 1 477 9Q472 14 408 162T281 457T217 603Q215 603 215 334V62H323V0H314",
      119821: "314 0Q296 3 181 3T48 0H39V62H147V624H39V686H171H265Q288 686 297 686T309 684T315 679Q317 676 500 455T684 233V624H576V686H585Q603 683 718 683T851 686H860V624H752V319Q752 15 750 11Q747 4 742 2T718 0H712Q708 0 706 0T700 0T696 1T693 2T690 4T687 7T684 11T679 16T674 23Q671 27 437 311L215 579V62H323V0H314",
      119822: "64 339Q64 431 96 502T182 614T295 675T420 696Q469 696 481 695Q620 680 709 589T798 339Q798 173 697 82T432 -10Q262 -10 163 85T64 339ZM625 454Q618 502 600 538T562 593T515 624T469 639T431 642Q331 642 276 563Q232 493 232 353Q232 315 234 285T244 216T267 148T308 94T372 56Q405 46 432 46Q517 46 567 106T627 267Q631 299 631 353Q631 418 625 454",
      119823: "400 0Q376 3 226 3Q75 3 51 0H39V62H147V624H39V686H253Q435 686 470 685T536 678Q585 668 621 648T675 605T705 557T718 514T721 483T718 451T704 409T673 362T616 322T530 293Q500 288 399 287H304V62H412V0H400ZM553 475Q553 554 537 582T459 622Q451 623 373 624H298V343H372Q457 344 480 350Q527 362 540 390T553 475",
      119824: "64 339Q64 431 96 502T182 614T295 675T420 696Q469 696 481 695Q620 680 709 589T798 339Q798 255 768 184Q720 77 611 26L600 21Q635 -26 682 -26H696Q769 -26 769 0Q769 7 774 12T787 18Q805 18 805 -7V-13Q803 -64 785 -106T737 -171Q720 -183 697 -191Q687 -193 668 -193Q636 -193 613 -182T575 -144T552 -94T532 -27Q531 -23 530 -16T528 -6T526 -3L512 -5Q499 -7 477 -8T431 -10Q393 -10 382 -9Q238 8 151 97T64 339ZM326 80Q326 113 356 138T430 163Q492 163 542 100L553 86Q554 85 561 91T578 108Q637 179 637 330Q637 430 619 498T548 604Q500 641 425 641Q408 641 390 637T347 623T299 590T259 535Q226 469 226 338Q226 244 246 180T318 79L325 74Q326 74 326 80ZM506 58Q480 112 433 112Q412 112 395 104T378 77Q378 44 431 44Q480 44 506 58",
      119825: "394 0Q370 3 222 3Q75 3 51 0H39V62H147V624H39V686H234Q256 686 299 686T362 687Q479 687 554 669T681 593Q716 550 716 497Q716 390 568 338Q569 337 572 336T577 332Q605 317 623 300T650 258T662 218T668 172Q678 98 689 76Q707 40 748 40Q770 40 780 54T795 88T801 111Q805 117 827 117H831Q846 117 852 113T858 92Q857 78 852 63T834 30T797 1T739 -11Q630 -11 580 12T511 87Q506 104 506 168Q506 170 506 178T507 194Q507 289 438 313Q424 318 356 318H298V62H406V0H394ZM366 369Q459 370 490 381Q548 402 548 476V498V517Q548 578 513 600Q479 624 392 624H358H298V369H366",
      119826: "64 493Q64 582 120 636T264 696H272Q280 697 285 697Q380 697 454 645L480 669Q484 672 488 676T495 683T500 688T504 691T508 693T511 695T514 696T517 697T522 697Q536 697 539 691T542 652V577Q542 557 542 532T543 500Q543 472 540 465T524 458H511H505Q489 458 485 461T479 478Q472 529 449 564T393 614T336 634T287 639Q228 639 203 610T177 544Q177 517 195 493T247 457Q253 454 343 436T475 391Q574 326 574 207V200Q574 163 559 120Q517 12 389 -9Q380 -10 346 -10Q308 -10 275 -5T221 7T184 22T160 35T151 40L126 17Q122 14 118 10T111 3T106 -2T102 -5T98 -7T95 -9T92 -10T89 -11T84 -11Q70 -11 67 -4T64 35V108Q64 128 64 153T63 185Q63 203 63 211T69 223T77 227T94 228H100Q118 228 122 225T126 205Q130 125 193 88T345 51Q408 51 434 82T460 157Q460 196 439 221T388 257Q384 259 305 276T221 295Q155 313 110 366T64 493",
      119827: "41 425Q41 426 51 545T62 669V675H737V669Q738 665 748 546T758 425V419H696V425Q687 517 669 555T595 607Q578 612 522 613H478V62H631V0H615Q585 3 399 3Q214 3 184 0H168V62H321V613H277H263Q164 613 134 561Q113 527 103 425V419H41V425",
      119828: "570 686Q588 683 703 683T836 686H845V624H737V420Q737 390 737 345T738 284Q738 205 729 164T689 83Q614 -11 465 -11Q321 -11 240 51T148 207Q147 214 147 421V624H39V686H51Q75 683 226 683Q376 683 400 686H412V624H304V405V370V268Q304 181 311 146T346 87Q387 52 466 52Q642 52 667 195Q668 204 669 415V624H561V686H570",
      119829: "592 686H604Q615 685 631 685T666 684T700 684T724 683Q829 683 835 686H843V624H744L611 315Q584 254 546 165Q492 40 482 19T461 -6L460 -7H409Q398 -4 391 9Q385 20 257 315L124 624H25V686H36Q57 683 190 683Q340 683 364 686H377V624H289L384 403L480 185L492 212Q504 240 529 298T575 405L670 624H582V686H592",
      119830: "915 686L1052 683Q1142 683 1157 686H1164V624H1073L957 320Q930 249 900 170T855 52T839 10Q834 0 826 -5Q821 -7 799 -7H792Q777 -7 772 -5T759 10Q759 11 748 39T716 122T676 228L594 442L512 228Q486 159 455 78Q433 19 428 9T416 -5Q411 -7 389 -7H379Q356 -7 349 10Q349 12 334 51T288 170T231 320L116 624H24V686H35Q44 683 183 683Q331 683 355 686H368V624H323Q278 624 278 623L437 207L499 369L561 531L526 624H434V686H445Q454 683 593 683Q741 683 765 686H778V624H733Q688 624 688 623L847 207Q848 207 927 415T1006 624H905V686H915",
      119831: "327 0Q306 3 174 3Q52 3 43 0H33V62H98L162 63L360 333L157 624H48V686H59Q80 683 217 683Q368 683 395 686H408V624H335L393 540L452 458L573 623Q573 624 528 624H483V686H494Q515 683 646 683Q769 683 778 686H787V624H658L575 511Q493 398 493 397L508 376Q522 356 553 312T611 229L727 62H835V0H824Q803 3 667 3Q516 3 489 0H476V62H513L549 63L401 274L247 63Q247 62 292 62H338V0H327",
      119832: "605 0Q581 3 434 3Q286 3 262 0H250V62H358V275L126 624H19V686H30Q54 683 189 683Q361 685 370 686H383V624H308L319 608Q330 591 353 556T396 491L484 359L660 623Q660 624 623 624H585V686H595Q613 683 728 683Q832 683 841 686H849V624H742L509 274V62H618V0H605",
      119833: "80 430L92 686H358Q624 686 628 684Q638 679 638 656Q638 640 637 639Q637 638 445 353Q401 288 351 214T277 103L253 67L256 66Q258 66 265 66T279 66T298 66H343Q380 66 406 68T464 81T518 110T557 164T579 250Q583 278 583 298Q583 299 614 299H645V291Q643 281 636 150T627 8V0H353Q79 0 75 2Q64 7 64 31Q64 48 66 52L259 340L451 623Q451 624 384 624Q294 623 259 612Q155 581 143 446Q142 440 142 432V430H80",
      119834: "64 349Q64 399 107 426T255 453Q346 453 402 423T473 341Q478 327 478 310T479 196V77Q493 63 529 62Q549 62 553 57T558 31Q558 9 552 5T514 0H497H481Q375 0 367 56L356 46Q300 -6 210 -6Q130 -6 81 30T32 121Q32 188 111 226T332 272H350V292Q350 313 348 327T337 361T306 391T248 402T194 399H189Q204 376 204 354Q204 327 187 306T134 284Q97 284 81 305T64 349ZM164 121Q164 89 186 67T238 45Q274 45 307 63T346 108L350 117V226H347Q248 218 206 189T164 121",
      119835: "32 686L123 690Q214 694 215 694H221V409Q289 450 378 450Q479 450 539 387T600 221Q600 122 535 58T358 -6H355Q272 -6 203 53L160 1L129 0H98V301Q98 362 98 435T99 525Q99 591 97 604T83 620Q69 624 42 624H29V686H32ZM227 105L232 99Q237 93 242 87T258 73T280 59T306 49T339 45Q380 45 411 66T451 131Q457 160 457 230Q457 264 456 284T448 329T430 367T396 389T343 398Q282 398 235 355L227 348V105",
      119836: "447 131H458Q478 131 478 117Q478 112 471 95T439 51T377 9Q330 -6 286 -6Q196 -6 135 35Q39 96 39 222Q39 324 101 384Q169 453 286 453Q359 453 411 431T464 353Q464 319 445 302T395 284Q360 284 343 305T325 353Q325 380 338 396H333Q317 398 295 398H292Q280 398 271 397T245 390T218 373T197 338T183 283Q182 275 182 231Q182 199 184 180T193 132T220 85T270 57Q289 50 317 50H326Q385 50 414 115Q419 127 423 129T447 131",
      119837: "351 686L442 690Q533 694 534 694H540V389Q540 327 540 253T539 163Q539 97 541 83T555 66Q569 62 596 62H609V31Q609 0 608 0Q588 0 510 -3T412 -6Q411 -6 411 16V38L401 31Q337 -6 265 -6Q159 -6 99 58T38 224Q38 265 51 303T92 375T165 429T272 449Q359 449 417 412V507V555Q417 597 415 607T402 620Q388 624 361 624H348V686H351ZM411 350Q362 399 291 399Q278 399 256 392T218 371Q195 351 189 320T182 238V221Q182 179 183 159T191 115T212 74Q241 46 288 46Q358 46 404 100L411 109V350",
      119838: "32 225Q32 332 102 392T272 452H283Q382 452 436 401Q494 343 494 243Q494 226 486 222T440 217Q431 217 394 217T327 218H175V209Q175 177 179 154T196 107T236 69T306 50Q312 49 323 49Q376 49 410 85Q421 99 427 111T434 127T442 133T463 135H468Q494 135 494 117Q494 110 489 97T468 66T431 32T373 5T292 -6Q181 -6 107 55T32 225ZM383 276Q377 346 348 374T280 402Q253 402 230 390T195 357Q179 331 176 279V266H383V276",
      119839: "308 0Q290 3 172 3Q58 3 49 0H40V62H109V382H42V444H109V503L110 562L112 572Q127 625 178 658T316 699Q318 699 330 699T348 700Q381 698 404 687T436 658T449 629T452 606Q452 576 432 557T383 537Q355 537 335 555T314 605Q314 635 328 649H325Q311 649 293 644T253 618T227 560Q226 555 226 498V444H340V382H232V62H318V0H308",
      119840: "50 300Q50 368 105 409T255 450Q328 450 376 426L388 420Q435 455 489 455Q517 455 533 441T554 414T558 389Q558 367 544 353T508 339Q484 339 471 354T458 387Q458 397 462 400Q464 401 461 400Q459 400 454 399Q429 392 427 390Q454 353 459 328Q461 315 461 300Q461 240 419 202Q364 149 248 149Q185 149 136 172Q129 158 129 148Q129 105 170 93Q176 91 263 91Q273 91 298 91T334 91T366 89T400 85T432 77T466 64Q544 22 544 -69Q544 -114 506 -145Q438 -201 287 -201Q149 -201 90 -161T30 -70Q30 -58 33 -47T42 -27T54 -13T69 -1T82 6T94 12T101 15Q66 57 66 106Q66 151 90 187L97 197L89 204Q50 243 50 300ZM485 403H492Q491 404 488 404L485 403V403ZM255 200Q279 200 295 206T319 219T331 242T335 268T336 300Q336 337 333 352T317 380Q298 399 255 399Q228 399 211 392T187 371T178 345T176 312V300V289Q176 235 194 219Q215 200 255 200ZM287 -150Q357 -150 400 -128T443 -71Q443 -65 442 -61T436 -50T420 -37T389 -27T339 -21L308 -20Q276 -20 253 -20Q190 -20 180 -20T156 -26Q130 -38 130 -69Q130 -105 173 -127T287 -150",
      119841: "40 686L131 690Q222 694 223 694H229V533L230 372L238 381Q248 394 264 407T317 435T398 450Q428 450 448 447T491 434T529 402T551 346Q553 335 554 198V62H623V0H614Q596 3 489 3Q374 3 365 0H356V62H425V194V275Q425 348 416 373T371 399Q326 399 288 370T238 290Q236 281 235 171V62H304V0H295Q277 3 171 3Q64 3 46 0H37V62H106V332Q106 387 106 453T107 534Q107 593 105 605T91 620Q77 624 50 624H37V686H40",
      119842: "72 610Q72 649 98 672T159 695Q193 693 217 670T241 610Q241 572 217 549T157 525Q120 525 96 548T72 610ZM46 442L136 446L226 450H232V62H294V0H286Q271 3 171 3Q67 3 49 0H40V62H109V209Q109 358 108 362Q103 380 55 380H43V442H46",
      119843: "104 610Q104 649 130 672T191 695Q225 693 249 670T273 610Q273 572 249 549T189 525Q152 525 128 548T104 610ZM78 442L173 446L268 450H274V196Q274 -5 274 -37T269 -83Q256 -132 201 -166T71 -200Q10 -200 -30 -173T-71 -102Q-71 -70 -51 -51T-1 -31Q27 -31 48 -49T69 -100Q69 -121 53 -147H56Q66 -149 77 -149H80Q90 -149 100 -146T127 -125T149 -73Q151 -55 151 149V362Q150 364 148 366T145 370T142 373T138 375T133 377T124 378T113 379T97 380H75V442H78",
      119844: "32 686L123 690Q214 694 215 694H221V255L377 382H346V444H355Q370 441 476 441Q544 441 556 444H562V382H476L347 277L515 62H587V0H579Q564 3 476 3Q370 3 352 0H343V62H358L373 63L260 206L237 189L216 172V62H285V0H277Q259 3 157 3Q46 3 37 0H29V62H98V332Q98 387 98 453T99 534Q99 593 97 605T83 620Q69 624 42 624H29V686H32",
      119845: "43 686L134 690Q225 694 226 694H232V62H301V0H292Q274 3 170 3Q67 3 49 0H40V62H109V332Q109 387 109 453T110 534Q110 593 108 605T94 620Q80 624 53 624H40V686H43",
      119846: "40 442Q217 450 218 450H224V365Q226 367 235 378T254 397T278 416T314 435T362 448Q376 450 400 450H406Q503 450 534 393Q545 376 545 370Q545 368 555 379Q611 450 716 450Q774 450 809 434Q850 414 861 379T873 276V213V198V62H942V0H933Q915 3 809 3Q702 3 684 0H675V62H744V194V275Q744 348 735 373T690 399Q645 399 607 370T557 290Q555 281 554 171V62H623V0H614Q596 3 489 3Q374 3 365 0H356V62H425V194V275Q425 348 416 373T371 399Q326 399 288 370T238 290Q236 281 235 171V62H304V0H295Q277 3 171 3Q64 3 46 0H37V62H106V210V303Q106 353 104 363T91 376Q77 380 50 380H37V442H40",
      119847: "40 442Q217 450 218 450H224V407L225 365Q233 378 245 391T289 422T362 448Q374 450 398 450Q428 450 448 447T491 434T529 402T551 346Q553 335 554 198V62H623V0H614Q596 3 489 3Q374 3 365 0H356V62H425V194V275Q425 348 416 373T371 399Q326 399 288 370T238 290Q236 281 235 171V62H304V0H295Q277 3 171 3Q64 3 46 0H37V62H106V210V303Q106 353 104 363T91 376Q77 380 50 380H37V442H40",
      119848: "287 -5Q228 -5 182 10T109 48T63 102T39 161T32 219Q32 272 50 314T94 382T154 423T214 446T265 452H279Q319 452 326 451Q428 439 485 376T542 221Q542 156 514 108T442 33Q384 -5 287 -5ZM399 230V250Q399 280 398 298T391 338T372 372T338 392T282 401Q241 401 212 380Q190 363 183 334T175 230Q175 202 175 189T177 153T183 118T195 91T215 68T245 56T287 50Q348 50 374 84Q388 101 393 132T399 230",
      119849: "32 442L123 446Q214 450 215 450H221V409Q222 409 229 413T251 423T284 436T328 446T382 450Q480 450 540 388T600 223Q600 128 539 61T361 -6H354Q292 -6 236 28L227 34V-132H296V-194H287Q269 -191 163 -191Q56 -191 38 -194H29V-132H98V113V284Q98 330 97 348T93 370T83 376Q69 380 42 380H29V442H32ZM457 224Q457 303 427 349T350 395Q282 395 235 352L227 345V104L233 97Q274 45 337 45Q383 45 420 86T457 224",
      119850: "38 220Q38 273 54 314T95 380T152 421T211 443T264 449Q368 449 429 386L438 377L484 450H540V-132H609V-194H600Q582 -191 475 -191Q360 -191 351 -194H342V-132H411V42Q409 41 399 34T383 25T367 16T347 7T324 1T296 -4T264 -6Q162 -6 100 56T38 220ZM287 46Q368 46 417 127V301L412 312Q398 347 369 371T302 395Q282 395 263 388T225 362T194 308T182 221Q182 126 214 86T287 46",
      119851: "405 293T374 293T324 312T305 361Q305 378 312 394Q315 397 315 399Q305 399 294 394T266 375T238 329T222 249Q221 241 221 149V62H308V0H298Q280 3 161 3Q47 3 38 0H29V62H98V210V303Q98 353 96 363T83 376Q69 380 42 380H29V442H32L118 446Q204 450 205 450H210V414L211 378Q247 449 315 449H321Q384 449 413 422T442 360Q442 332 424 313",
      119852: "38 315Q38 339 45 360T70 404T127 440T223 453Q273 453 320 436L338 445L357 453H366Q380 453 383 447T386 403V387V355Q386 331 383 326T365 321H355H349Q333 321 329 324T324 341Q317 406 224 406H216Q123 406 123 353Q123 334 143 321T188 304T244 294T285 286Q305 281 325 273T373 237T412 172Q414 162 414 142Q414 -6 230 -6Q154 -6 117 22L68 -6H58Q44 -6 41 0T38 42V73Q38 85 38 101T37 122Q37 144 42 148T68 153H75Q87 153 91 151T97 147T103 132Q131 46 220 46H230Q257 46 265 47Q330 58 330 108Q330 127 316 142Q300 156 284 162Q271 168 212 178T122 202Q38 243 38 315",
      119853: "272 49Q320 49 320 136V145V177H382V143Q382 106 380 99Q374 62 349 36T285 -2L272 -5H247Q173 -5 134 27Q109 46 102 74T94 160Q94 171 94 199T95 245V382H21V433H25Q58 433 90 456Q121 479 140 523T162 621V635H224V444H363V382H224V239V207V149Q224 98 228 81T249 55Q261 49 272 49",
      119854: "40 442L134 446Q228 450 229 450H235V273V165Q235 90 238 74T254 52Q268 46 304 46H319Q352 46 380 67T419 121L420 123Q424 135 425 199Q425 201 425 207Q425 233 425 249V316Q425 354 423 363T410 376Q396 380 369 380H356V442L554 450V267Q554 84 556 79Q561 62 610 62H623V31Q623 0 622 0Q603 0 527 -3T432 -6Q431 -6 431 25V56L420 45Q373 6 332 -1Q313 -6 281 -6Q208 -6 165 14T109 87L107 98L106 230Q106 358 104 366Q96 380 50 380H37V442H40",
      119855: "401 444Q413 441 495 441Q568 441 574 444H580V382H510L409 156Q348 18 339 6Q331 -4 320 -4Q318 -4 313 -4T303 -3H288Q273 -3 264 12T221 102Q206 135 197 156L96 382H26V444H34Q49 441 145 441Q252 441 270 444H279V382H231L284 264Q335 149 338 149Q338 150 389 264T442 381Q442 382 418 382H394V444H401",
      119856: "624 444Q636 441 722 441Q797 441 800 444H805V382H741L593 11Q592 10 590 8T586 4T584 2T581 0T579 -2T575 -3T571 -3T567 -4T561 -4T553 -4H542Q525 -4 518 6T490 70Q474 110 463 137L415 257L367 137Q357 111 341 72Q320 17 313 7T289 -4H277Q259 -4 253 -2T238 11L90 382H25V444H32Q47 441 140 441Q243 441 261 444H270V382H222L310 164L382 342L366 382H303V444H310Q322 441 407 441Q508 441 523 444H531V382H506Q481 382 481 380Q482 376 529 259T577 142L674 382H617V444H624",
      119857: "227 0Q212 3 121 3Q40 3 28 0H21V62H117L245 213L109 382H26V444H34Q49 441 143 441Q247 441 265 444H274V382H246L281 339Q315 297 316 297Q320 297 354 341L389 382H352V444H360Q375 441 466 441Q547 441 559 444H566V382H471L355 246L504 63L545 62H586V0H578Q563 3 469 3Q365 3 347 0H338V62H366Q366 63 326 112T285 163L198 63L217 62H235V0H227",
      119858: "84 -102Q84 -110 87 -119T102 -138T133 -149Q148 -148 162 -143T186 -131T206 -114T222 -95T234 -76T243 -59T249 -45T252 -37L269 0L96 382H26V444H34Q49 441 146 441Q252 441 270 444H279V382H255Q232 382 232 380L337 151L442 382H394V444H401Q413 441 495 441Q568 441 574 444H580V382H510L406 152Q298 -84 297 -87Q269 -139 225 -169T131 -200Q85 -200 54 -172T23 -100Q23 -64 44 -50T87 -35Q111 -35 130 -50T152 -92V-100H84V-102",
      119859: "48 262Q48 264 54 349T60 436V444H252Q289 444 336 444T394 445Q441 445 450 441T459 418Q459 406 458 404Q456 399 327 229T194 55H237Q260 56 268 56T297 58T325 65T348 77T370 98T384 128T395 170Q400 197 400 216Q400 217 431 217H462V211Q461 208 453 108T444 6V0H245Q46 0 43 2Q32 7 32 28V33Q32 41 40 52T84 112Q129 170 164 217L298 393H256Q189 392 165 380Q124 360 115 303Q110 280 110 256Q110 254 79 254H48V262",
      119860: "208 74Q208 50 254 46Q272 46 272 35Q272 34 270 22Q267 8 264 4T251 0Q249 0 239 0T205 1T141 2Q70 2 50 0H42Q35 7 35 11Q37 38 48 46H62Q132 49 164 96Q170 102 345 401T523 704Q530 716 547 716H555H572Q578 707 578 706L606 383Q634 60 636 57Q641 46 701 46Q726 46 726 36Q726 34 723 22Q720 7 718 4T704 0Q701 0 690 0T651 1T578 2Q484 2 455 0H443Q437 6 437 9T439 27Q443 40 445 43L449 46H469Q523 49 533 63L521 213H283L249 155Q208 86 208 74ZM516 260Q516 271 504 416T490 562L463 519Q447 492 400 412L310 260L413 259Q516 259 516 260",
      119861: "231 637Q204 637 199 638T194 649Q194 676 205 682Q206 683 335 683Q594 683 608 681Q671 671 713 636T756 544Q756 480 698 429T565 360L555 357Q619 348 660 311T702 219Q702 146 630 78T453 1Q446 0 242 0Q42 0 39 2Q35 5 35 10Q35 17 37 24Q42 43 47 45Q51 46 62 46H68Q95 46 128 49Q142 52 147 61Q150 65 219 339T288 628Q288 635 231 637ZM649 544Q649 574 634 600T585 634Q578 636 493 637Q473 637 451 637T416 636H403Q388 635 384 626Q382 622 352 506Q352 503 351 500L320 374H401Q482 374 494 376Q554 386 601 434T649 544ZM595 229Q595 273 572 302T512 336Q506 337 429 337Q311 337 310 336Q310 334 293 263T258 122L240 52Q240 48 252 48T333 46Q422 46 429 47Q491 54 543 105T595 229",
      119862: "50 252Q50 367 117 473T286 641T490 704Q580 704 633 653Q642 643 648 636T656 626L657 623Q660 623 684 649Q691 655 699 663T715 679T725 690L740 705H746Q760 705 760 698Q760 694 728 561Q692 422 692 421Q690 416 687 415T669 413H653Q647 419 647 422Q647 423 648 429T650 449T651 481Q651 552 619 605T510 659Q484 659 454 652T382 628T299 572T226 479Q194 422 175 346T156 222Q156 108 232 58Q280 24 350 24Q441 24 512 92T606 240Q610 253 612 255T628 257Q648 257 648 248Q648 243 647 239Q618 132 523 55T319 -22Q206 -22 128 53T50 252",
      119863: "287 628Q287 635 230 637Q207 637 200 638T193 647Q193 655 197 667T204 682Q206 683 403 683Q570 682 590 682T630 676Q702 659 752 597T803 431Q803 275 696 151T444 3L430 1L236 0H125H72Q48 0 41 2T33 11Q33 13 36 25Q40 41 44 43T67 46Q94 46 127 49Q141 52 146 61Q149 65 218 339T287 628ZM703 469Q703 507 692 537T666 584T629 613T590 629T555 636Q553 636 541 636T512 636T479 637H436Q392 637 386 627Q384 623 313 339T242 52Q242 48 253 48T330 47Q335 47 349 47T373 46Q499 46 581 128Q617 164 640 212T683 339T703 469",
      119864: "492 213Q472 213 472 226Q472 230 477 250T482 285Q482 316 461 323T364 330H312Q311 328 277 192T243 52Q243 48 254 48T334 46Q428 46 458 48T518 61Q567 77 599 117T670 248Q680 270 683 272Q690 274 698 274Q718 274 718 261Q613 7 608 2Q605 0 322 0H133Q31 0 31 11Q31 13 34 25Q38 41 42 43T65 46Q92 46 125 49Q139 52 144 61Q146 66 215 342T285 622Q285 629 281 629Q273 632 228 634H197Q191 640 191 642T193 659Q197 676 203 680H757Q764 676 764 669Q764 664 751 557T737 447Q735 440 717 440H705Q698 445 698 453L701 476Q704 500 704 528Q704 558 697 578T678 609T643 625T596 632T532 634H485Q397 633 392 631Q388 629 386 622Q385 619 355 499T324 377Q347 376 372 376H398Q464 376 489 391T534 472Q538 488 540 490T557 493Q562 493 565 493T570 492T572 491T574 487T577 483L544 351Q511 218 508 216Q505 213 492 213",
      119865: "48 1Q31 1 31 11Q31 13 34 25Q38 41 42 43T65 46Q92 46 125 49Q139 52 144 61Q146 66 215 342T285 622Q285 629 281 629Q273 632 228 634H197Q191 640 191 642T193 659Q197 676 203 680H742Q749 676 749 669Q749 664 736 557T722 447Q720 440 702 440H690Q683 445 683 453Q683 454 686 477T689 530Q689 560 682 579T663 610T626 626T575 633T503 634H480Q398 633 393 631Q388 629 386 623Q385 622 352 492L320 363H375Q378 363 398 363T426 364T448 367T472 374T489 386Q502 398 511 419T524 457T529 475Q532 480 548 480H560Q567 475 567 470Q567 467 536 339T502 207Q500 200 482 200H470Q463 206 463 212Q463 215 468 234T473 274Q473 303 453 310T364 317H309L277 190Q245 66 245 60Q245 46 334 46H359Q365 40 365 39T363 19Q359 6 353 0H336Q295 2 185 2Q120 2 86 2T48 1",
      119866: "50 252Q50 367 117 473T286 641T490 704Q580 704 633 653Q642 643 648 636T656 626L657 623Q660 623 684 649Q691 655 699 663T715 679T725 690L740 705H746Q760 705 760 698Q760 694 728 561Q692 422 692 421Q690 416 687 415T669 413H653Q647 419 647 422Q647 423 648 429T650 449T651 481Q651 552 619 605T510 659Q492 659 471 656T418 643T357 615T294 567T236 496T189 394T158 260Q156 242 156 221Q156 173 170 136T206 79T256 45T308 28T353 24Q407 24 452 47T514 106Q517 114 529 161T541 214Q541 222 528 224T468 227H431Q425 233 425 235T427 254Q431 267 437 273H454Q494 271 594 271Q634 271 659 271T695 272T707 272Q721 272 721 263Q721 261 719 249Q714 230 709 228Q706 227 694 227Q674 227 653 224Q646 221 643 215T629 164Q620 131 614 108Q589 6 586 3Q584 1 581 1Q571 1 553 21T530 52Q530 53 528 52T522 47Q448 -22 322 -22Q201 -22 126 55T50 252",
      119867: "228 637Q194 637 192 641Q191 643 191 649Q191 673 202 682Q204 683 219 683Q260 681 355 681Q389 681 418 681T463 682T483 682Q499 682 499 672Q499 670 497 658Q492 641 487 638H485Q483 638 480 638T473 638T464 637T455 637Q416 636 405 634T387 623Q384 619 355 500Q348 474 340 442T328 395L324 380Q324 378 469 378H614L615 381Q615 384 646 504Q674 619 674 627T617 637Q594 637 587 639T580 648Q580 650 582 660Q586 677 588 679T604 682Q609 682 646 681T740 680Q802 680 835 681T871 682Q888 682 888 672Q888 645 876 638H874Q872 638 869 638T862 638T853 637T844 637Q805 636 794 634T776 623Q773 618 704 340T634 58Q634 51 638 51Q646 48 692 46H723Q729 38 729 37T726 19Q722 6 716 0H701Q664 2 567 2Q533 2 504 2T458 2T437 1Q420 1 420 10Q420 15 423 24Q428 43 433 45Q437 46 448 46H454Q481 46 514 49Q520 50 522 50T528 55T534 64T540 82T547 110T558 153Q565 181 569 198Q602 330 602 331T457 332H312L279 197Q245 63 245 58Q245 51 253 49T303 46H334Q340 38 340 37T337 19Q333 6 327 0H312Q275 2 178 2Q144 2 115 2T69 2T48 1Q31 1 31 10Q31 12 34 24Q39 43 44 45Q48 46 59 46H65Q92 46 125 49Q139 52 144 61Q147 65 216 339T285 628Q285 635 228 637",
      119868: "43 1Q26 1 26 10Q26 12 29 24Q34 43 39 45Q42 46 54 46H60Q120 46 136 53Q137 53 138 54Q143 56 149 77T198 273Q210 318 216 344Q286 624 286 626Q284 630 284 631Q274 637 213 637H193Q184 643 189 662Q193 677 195 680T209 683H213Q285 681 359 681Q481 681 487 683H497Q504 676 504 672T501 655T494 639Q491 637 471 637Q440 637 407 634Q393 631 388 623Q381 609 337 432Q326 385 315 341Q245 65 245 59Q245 52 255 50T307 46H339Q345 38 345 37T342 19Q338 6 332 0H316Q279 2 179 2Q143 2 113 2T65 2T43 1",
      119869: "447 625Q447 637 354 637H329Q323 642 323 645T325 664Q329 677 335 683H352Q393 681 498 681Q541 681 568 681T605 682T619 682Q633 682 633 672Q633 670 630 658Q626 642 623 640T604 637Q552 637 545 623Q541 610 483 376Q420 128 419 127Q397 64 333 21T195 -22Q137 -22 97 8T57 88Q57 130 80 152T132 174Q177 174 182 130Q182 98 164 80T123 56Q115 54 115 53T122 44Q148 15 197 15Q235 15 271 47T324 130Q328 142 387 380T447 625",
      119870: "285 628Q285 635 228 637Q205 637 198 638T191 647Q191 649 193 661Q199 681 203 682Q205 683 214 683H219Q260 681 355 681Q389 681 418 681T463 682T483 682Q500 682 500 674Q500 669 497 660Q496 658 496 654T495 648T493 644T490 641T486 639T479 638T470 637T456 637Q416 636 405 634T387 623L306 305Q307 305 490 449T678 597Q692 611 692 620Q692 635 667 637Q651 637 651 648Q651 650 654 662T659 677Q662 682 676 682Q680 682 711 681T791 680Q814 680 839 681T869 682Q889 682 889 672Q889 650 881 642Q878 637 862 637Q787 632 726 586Q710 576 656 534T556 455L509 418L518 396Q527 374 546 329T581 244Q656 67 661 61Q663 59 666 57Q680 47 717 46H738Q744 38 744 37T741 19Q737 6 731 0H720Q680 3 625 3Q503 3 488 0H478Q472 6 472 9T474 27Q478 40 480 43T491 46H494Q544 46 544 71Q544 75 517 141T485 216L427 354L359 301L291 248L268 155Q245 63 245 58Q245 51 253 49T303 46H334Q340 37 340 35Q340 19 333 5Q328 0 317 0Q314 0 280 1T180 2Q118 2 85 2T49 1Q31 1 31 11Q31 13 34 25Q38 41 42 43T65 46Q92 46 125 49Q139 52 144 61Q147 65 216 339T285 628",
      119871: "228 637Q194 637 192 641Q191 643 191 649Q191 673 202 682Q204 683 217 683Q271 680 344 680Q485 680 506 683H518Q524 677 524 674T522 656Q517 641 513 637H475Q406 636 394 628Q387 624 380 600T313 336Q297 271 279 198T252 88L243 52Q243 48 252 48T311 46H328Q360 46 379 47T428 54T478 72T522 106T564 161Q580 191 594 228T611 270Q616 273 628 273H641Q647 264 647 262T627 203T583 83T557 9Q555 4 553 3T537 0T494 -1Q483 -1 418 -1T294 0H116Q32 0 32 10Q32 17 34 24Q39 43 44 45Q48 46 59 46H65Q92 46 125 49Q139 52 144 61Q147 65 216 339T285 628Q285 635 228 637",
      119872: "289 629Q289 635 232 637Q208 637 201 638T194 648Q194 649 196 659Q197 662 198 666T199 671T201 676T203 679T207 681T212 683T220 683T232 684Q238 684 262 684T307 683Q386 683 398 683T414 678Q415 674 451 396L487 117L510 154Q534 190 574 254T662 394Q837 673 839 675Q840 676 842 678T846 681L852 683H948Q965 683 988 683T1017 684Q1051 684 1051 673Q1051 668 1048 656T1045 643Q1041 637 1008 637Q968 636 957 634T939 623Q936 618 867 340T797 59Q797 55 798 54T805 50T822 48T855 46H886Q892 37 892 35Q892 19 885 5Q880 0 869 0Q864 0 828 1T736 2Q675 2 644 2T609 1Q592 1 592 11Q592 13 594 25Q598 41 602 43T625 46Q652 46 685 49Q699 52 704 61Q706 65 742 207T813 490T848 631L654 322Q458 10 453 5Q451 4 449 3Q444 0 433 0Q418 0 415 7Q413 11 374 317L335 624L267 354Q200 88 200 79Q206 46 272 46H282Q288 41 289 37T286 19Q282 3 278 1Q274 0 267 0Q265 0 255 0T221 1T157 2Q127 2 95 1T58 0Q43 0 39 2T35 11Q35 13 38 25T43 40Q45 46 65 46Q135 46 154 86Q158 92 223 354T289 629",
      119873: "234 637Q231 637 226 637Q201 637 196 638T191 649Q191 676 202 682Q204 683 299 683Q376 683 387 683T401 677Q612 181 616 168L670 381Q723 592 723 606Q723 633 659 637Q635 637 635 648Q635 650 637 660Q641 676 643 679T653 683Q656 683 684 682T767 680Q817 680 843 681T873 682Q888 682 888 672Q888 650 880 642Q878 637 858 637Q787 633 769 597L620 7Q618 0 599 0Q585 0 582 2Q579 5 453 305L326 604L261 344Q196 88 196 79Q201 46 268 46H278Q284 41 284 38T282 19Q278 6 272 0H259Q228 2 151 2Q123 2 100 2T63 2T46 1Q31 1 31 10Q31 14 34 26T39 40Q41 46 62 46Q130 49 150 85Q154 91 221 362L289 634Q287 635 234 637",
      119874: "740 435Q740 320 676 213T511 42T304 -22Q207 -22 138 35T51 201Q50 209 50 244Q50 346 98 438T227 601Q351 704 476 704Q514 704 524 703Q621 689 680 617T740 435ZM637 476Q637 565 591 615T476 665Q396 665 322 605Q242 542 200 428T157 216Q157 126 200 73T314 19Q404 19 485 98T608 313Q637 408 637 476",
      119875: "287 628Q287 635 230 637Q206 637 199 638T192 648Q192 649 194 659Q200 679 203 681T397 683Q587 682 600 680Q664 669 707 631T751 530Q751 453 685 389Q616 321 507 303Q500 302 402 301H307L277 182Q247 66 247 59Q247 55 248 54T255 50T272 48T305 46H336Q342 37 342 35Q342 19 335 5Q330 0 319 0Q316 0 282 1T182 2Q120 2 87 2T51 1Q33 1 33 11Q33 13 36 25Q40 41 44 43T67 46Q94 46 127 49Q141 52 146 61Q149 65 218 339T287 628ZM645 554Q645 567 643 575T634 597T609 619T560 635Q553 636 480 637Q463 637 445 637T416 636T404 636Q391 635 386 627Q384 621 367 550T332 412T314 344Q314 342 395 342H407H430Q542 342 590 392Q617 419 631 471T645 554",
      119876: "399 -80Q399 -47 400 -30T402 -11V-7L387 -11Q341 -22 303 -22Q208 -22 138 35T51 201Q50 209 50 244Q50 346 98 438T227 601Q351 704 476 704Q514 704 524 703Q621 689 680 617T740 435Q740 255 592 107Q529 47 461 16L444 8V3Q444 2 449 -24T470 -66T516 -82Q551 -82 583 -60T625 -3Q631 11 638 11Q647 11 649 2Q649 -6 639 -34T611 -100T557 -165T481 -194Q399 -194 399 -87V-80ZM636 468Q636 523 621 564T580 625T530 655T477 665Q429 665 379 640Q277 591 215 464T153 216Q153 110 207 59Q231 38 236 38V46Q236 86 269 120T347 155Q372 155 390 144T417 114T429 82T435 55L448 64Q512 108 557 185T619 334T636 468ZM314 18Q362 18 404 39L403 49Q399 104 366 115Q354 117 347 117Q344 117 341 117T337 118Q317 118 296 98T274 52Q274 18 314 18",
      119877: "230 637Q203 637 198 638T193 649Q193 676 204 682Q206 683 378 683Q550 682 564 680Q620 672 658 652T712 606T733 563T739 529Q739 484 710 445T643 385T576 351T538 338L545 333Q612 295 612 223Q612 212 607 162T602 80V71Q602 53 603 43T614 25T640 16Q668 16 686 38T712 85Q717 99 720 102T735 105Q755 105 755 93Q755 75 731 36Q693 -21 641 -21H632Q571 -21 531 4T487 82Q487 109 502 166T517 239Q517 290 474 313Q459 320 449 321T378 323H309L277 193Q244 61 244 59Q244 55 245 54T252 50T269 48T302 46H333Q339 38 339 37T336 19Q332 6 326 0H311Q275 2 180 2Q146 2 117 2T71 2T50 1Q33 1 33 10Q33 12 36 24Q41 43 46 45Q50 46 61 46H67Q94 46 127 49Q141 52 146 61Q149 65 218 339T287 628Q287 635 230 637ZM630 554Q630 586 609 608T523 636Q521 636 500 636T462 637H440Q393 637 386 627Q385 624 352 494T319 361Q319 360 388 360Q466 361 492 367Q556 377 592 426Q608 449 619 486T630 554",
      119878: "308 24Q367 24 416 76T466 197Q466 260 414 284Q308 311 278 321T236 341Q176 383 176 462Q176 523 208 573T273 648Q302 673 343 688T407 704H418H425Q521 704 564 640Q565 640 577 653T603 682T623 704Q624 704 627 704T632 705Q645 705 645 698T617 577T585 459T569 456Q549 456 549 465Q549 471 550 475Q550 478 551 494T553 520Q553 554 544 579T526 616T501 641Q465 662 419 662Q362 662 313 616T263 510Q263 480 278 458T319 427Q323 425 389 408T456 390Q490 379 522 342T554 242Q554 216 546 186Q541 164 528 137T492 78T426 18T332 -20Q320 -22 298 -22Q199 -22 144 33L134 44L106 13Q83 -14 78 -18T65 -22Q52 -22 52 -14Q52 -11 110 221Q112 227 130 227H143Q149 221 149 216Q149 214 148 207T144 186T142 153Q144 114 160 87T203 47T255 29T308 24",
      119879: "40 437Q21 437 21 445Q21 450 37 501T71 602L88 651Q93 669 101 677H569H659Q691 677 697 676T704 667Q704 661 687 553T668 444Q668 437 649 437Q640 437 637 437T631 442L629 445Q629 451 635 490T641 551Q641 586 628 604T573 629Q568 630 515 631Q469 631 457 630T439 622Q438 621 368 343T298 60Q298 48 386 46Q418 46 427 45T436 36Q436 31 433 22Q429 4 424 1L422 0Q419 0 415 0Q410 0 363 1T228 2Q99 2 64 0H49Q43 6 43 9T45 27Q49 40 55 46H83H94Q174 46 189 55Q190 56 191 56Q196 59 201 76T241 233Q258 301 269 344Q339 619 339 625Q339 630 310 630H279Q212 630 191 624Q146 614 121 583T67 467Q60 445 57 441T43 437H40",
      119880: "107 637Q73 637 71 641Q70 643 70 649Q70 673 81 682Q83 683 98 683Q139 681 234 681Q268 681 297 681T342 682T362 682Q378 682 378 672Q378 670 376 658Q371 641 366 638H364Q362 638 359 638T352 638T343 637T334 637Q295 636 284 634T266 623Q265 621 238 518T184 302T154 169Q152 155 152 140Q152 86 183 55T269 24Q336 24 403 69T501 205L552 406Q599 598 599 606Q599 633 535 637Q511 637 511 648Q511 650 513 660Q517 676 519 679T529 683Q532 683 561 682T645 680Q696 680 723 681T752 682Q767 682 767 672Q767 650 759 642Q756 637 737 637Q666 633 648 597Q646 592 598 404Q557 235 548 205Q515 105 433 42T263 -22Q171 -22 116 34T60 167V183Q60 201 115 421Q164 622 164 628Q164 635 107 637",
      119881: "52 648Q52 670 65 683H76Q118 680 181 680Q299 680 320 683H330Q336 677 336 674T334 656Q329 641 325 637H304Q282 635 274 635Q245 630 242 620Q242 618 271 369T301 118L374 235Q447 352 520 471T595 594Q599 601 599 609Q599 633 555 637Q537 637 537 648Q537 649 539 661Q542 675 545 679T558 683Q560 683 570 683T604 682T668 681Q737 681 755 683H762Q769 676 769 672Q769 655 760 640Q757 637 743 637Q730 636 719 635T698 630T682 623T670 615T660 608T652 599T645 592L452 282Q272 -9 266 -16Q263 -18 259 -21L241 -22H234Q216 -22 216 -15Q213 -9 177 305Q139 623 138 626Q133 637 76 637H59Q52 642 52 648",
      119882: "436 683Q450 683 486 682T553 680Q604 680 638 681T677 682Q695 682 695 674Q695 670 692 659Q687 641 683 639T661 637Q636 636 621 632T600 624T597 615Q597 603 613 377T629 138L631 141Q633 144 637 151T649 170T666 200T690 241T720 295T759 362Q863 546 877 572T892 604Q892 619 873 628T831 637Q817 637 817 647Q817 650 819 660Q823 676 825 679T839 682Q842 682 856 682T895 682T949 681Q1015 681 1034 683Q1048 683 1048 672Q1048 666 1045 655T1038 640T1028 637Q1006 637 988 631T958 617T939 600T927 584L923 578L754 282Q586 -14 585 -15Q579 -22 561 -22Q546 -22 542 -17Q539 -14 523 229T506 480L494 462Q472 425 366 239Q222 -13 220 -15T215 -19Q210 -22 197 -22Q178 -22 176 -15Q176 -12 154 304T131 622Q129 631 121 633T82 637H58Q51 644 51 648Q52 671 64 683H76Q118 680 176 680Q301 680 313 683H323Q329 677 329 674T327 656Q322 641 318 637H297Q236 634 232 620Q262 160 266 136L501 550L499 587Q496 629 489 632Q483 636 447 637Q428 637 422 639T416 648Q416 650 418 660Q419 664 420 669T421 676T424 680T428 682T436 683",
      119883: "42 0H40Q26 0 26 11Q26 15 29 27Q33 41 36 43T55 46Q141 49 190 98Q200 108 306 224T411 342Q302 620 297 625Q288 636 234 637H206Q200 643 200 645T202 664Q206 677 212 683H226Q260 681 347 681Q380 681 408 681T453 682T473 682Q490 682 490 671Q490 670 488 658Q484 643 481 640T465 637Q434 634 411 620L488 426L541 485Q646 598 646 610Q646 628 622 635Q617 635 609 637Q594 637 594 648Q594 650 596 664Q600 677 606 683H618Q619 683 643 683T697 681T738 680Q828 680 837 683H845Q852 676 852 672Q850 647 840 637H824Q790 636 763 628T722 611T698 593L687 584Q687 585 592 480L505 384Q505 383 536 304T601 142T638 56Q648 47 699 46Q734 46 734 37Q734 35 732 23Q728 7 725 4T711 1Q708 1 678 1T589 2Q528 2 496 2T461 1Q444 1 444 10Q444 11 446 25Q448 35 450 39T455 44T464 46T480 47T506 54Q523 62 523 64Q522 64 476 181L429 299Q241 95 236 84Q232 76 232 72Q232 53 261 47Q262 47 267 47T273 46Q276 46 277 46T280 45T283 42T284 35Q284 26 282 19Q279 6 276 4T261 1Q258 1 243 1T201 2T142 2Q64 2 42 0",
      119884: "66 637Q54 637 49 637T39 638T32 641T30 647T33 664T42 682Q44 683 56 683Q104 680 165 680Q288 680 306 683H316Q322 677 322 674T320 656Q316 643 310 637H298Q242 637 242 624Q242 619 292 477T343 333L346 336Q350 340 358 349T379 373T411 410T454 461Q546 568 561 587T577 618Q577 634 545 637Q528 637 528 647Q528 649 530 661Q533 676 535 679T549 683Q551 683 578 682T657 680Q684 680 713 681T746 682Q763 682 763 673Q763 669 760 657T755 643Q753 637 734 637Q662 632 617 587Q608 578 477 424L348 273L322 169Q295 62 295 57Q295 46 363 46Q379 46 384 45T390 35Q390 33 388 23Q384 6 382 4T366 1Q361 1 324 1T232 2Q170 2 138 2T102 1Q84 1 84 9Q84 14 87 24Q88 27 89 30T90 35T91 39T93 42T96 44T101 45T107 45T116 46T129 46Q168 47 180 50T198 63Q201 68 227 171L252 274L129 623Q128 624 127 625T125 627T122 629T118 631T113 633T105 634T96 635T83 636T66 637",
      119885: "58 8Q58 23 64 35Q64 36 329 334T596 635L586 637Q575 637 512 637H500H476Q442 637 420 635T365 624T311 598T266 548T228 469Q227 466 226 463T224 458T223 453T222 450L221 448Q218 443 202 443Q185 443 182 453L214 561Q228 606 241 651Q249 679 253 681Q256 683 487 683H718Q723 678 723 675Q723 673 717 649Q189 54 188 52L185 49H274Q369 50 377 51Q452 60 500 100T579 247Q587 272 590 277T603 282H607Q628 282 628 271Q547 5 541 2Q538 0 300 0H124Q58 0 58 8",
      119886: "33 157Q33 258 109 349T280 441Q331 441 370 392Q386 422 416 422Q429 422 439 414T449 394Q449 381 412 234T374 68Q374 43 381 35T402 26Q411 27 422 35Q443 55 463 131Q469 151 473 152Q475 153 483 153H487Q506 153 506 144Q506 138 501 117T481 63T449 13Q436 0 417 -8Q409 -10 393 -10Q359 -10 336 5T306 36L300 51Q299 52 296 50Q294 48 292 46Q233 -10 172 -10Q117 -10 75 30T33 157ZM351 328Q351 334 346 350T323 385T277 405Q242 405 210 374T160 293Q131 214 119 129Q119 126 119 118T118 106Q118 61 136 44T179 26Q217 26 254 59T298 110Q300 114 325 217T351 328",
      119887: "73 647Q73 657 77 670T89 683Q90 683 161 688T234 694Q246 694 246 685T212 542Q204 508 195 472T180 418L176 399Q176 396 182 402Q231 442 283 442Q345 442 383 396T422 280Q422 169 343 79T173 -11Q123 -11 82 27T40 150V159Q40 180 48 217T97 414Q147 611 147 623T109 637Q104 637 101 637H96Q86 637 83 637T76 640T73 647ZM336 325V331Q336 405 275 405Q258 405 240 397T207 376T181 352T163 330L157 322L136 236Q114 150 114 114Q114 66 138 42Q154 26 178 26Q211 26 245 58Q270 81 285 114T318 219Q336 291 336 325",
      119888: "34 159Q34 268 120 355T306 442Q362 442 394 418T427 355Q427 326 408 306T360 285Q341 285 330 295T319 325T330 359T352 380T366 386H367Q367 388 361 392T340 400T306 404Q276 404 249 390Q228 381 206 359Q162 315 142 235T121 119Q121 73 147 50Q169 26 205 26H209Q321 26 394 111Q403 121 406 121Q410 121 419 112T429 98T420 83T391 55T346 25T282 0T202 -11Q127 -11 81 37T34 159",
      119889: "366 683Q367 683 438 688T511 694Q523 694 523 686Q523 679 450 384T375 83T374 68Q374 26 402 26Q411 27 422 35Q443 55 463 131Q469 151 473 152Q475 153 483 153H487H491Q506 153 506 145Q506 140 503 129Q490 79 473 48T445 8T417 -8Q409 -10 393 -10Q359 -10 336 5T306 36L300 51Q299 52 296 50Q294 48 292 46Q233 -10 172 -10Q117 -10 75 30T33 157Q33 205 53 255T101 341Q148 398 195 420T280 442Q336 442 364 400Q369 394 369 396Q370 400 396 505T424 616Q424 629 417 632T378 637H357Q351 643 351 645T353 664Q358 683 366 683ZM352 326Q329 405 277 405Q242 405 210 374T160 293Q131 214 119 129Q119 126 119 118T118 106Q118 61 136 44T179 26Q233 26 290 98L298 109L352 326",
      119890: "39 168Q39 225 58 272T107 350T174 402T244 433T307 442H310Q355 442 388 420T421 355Q421 265 310 237Q261 224 176 223Q139 223 138 221Q138 219 132 186T125 128Q125 81 146 54T209 26T302 45T394 111Q403 121 406 121Q410 121 419 112T429 98T420 82T390 55T344 24T281 -1T205 -11Q126 -11 83 42T39 168ZM373 353Q367 405 305 405Q272 405 244 391T199 357T170 316T154 280T149 261Q149 260 169 260Q282 260 327 284T373 353",
      119891: "118 -162Q120 -162 124 -164T135 -167T147 -168Q160 -168 171 -155T187 -126Q197 -99 221 27T267 267T289 382V385H242Q195 385 192 387Q188 390 188 397L195 425Q197 430 203 430T250 431Q298 431 298 432Q298 434 307 482T319 540Q356 705 465 705Q502 703 526 683T550 630Q550 594 529 578T487 561Q443 561 443 603Q443 622 454 636T478 657L487 662Q471 668 457 668Q445 668 434 658T419 630Q412 601 403 552T387 469T380 433Q380 431 435 431Q480 431 487 430T498 424Q499 420 496 407T491 391Q489 386 482 386T428 385H372L349 263Q301 15 282 -47Q255 -132 212 -173Q175 -205 139 -205Q107 -205 81 -186T55 -132Q55 -95 76 -78T118 -61Q162 -61 162 -103Q162 -122 151 -136T127 -157L118 -162",
      119892: "311 43Q296 30 267 15T206 0Q143 0 105 45T66 160Q66 265 143 353T314 442Q361 442 401 394L404 398Q406 401 409 404T418 412T431 419T447 422Q461 422 470 413T480 394Q480 379 423 152T363 -80Q345 -134 286 -169T151 -205Q10 -205 10 -137Q10 -111 28 -91T74 -71Q89 -71 102 -80T116 -111Q116 -121 114 -130T107 -144T99 -154T92 -162L90 -164H91Q101 -167 151 -167Q189 -167 211 -155Q234 -144 254 -122T282 -75Q288 -56 298 -13Q311 35 311 43ZM384 328L380 339Q377 350 375 354T369 368T359 382T346 393T328 402T306 405Q262 405 221 352Q191 313 171 233T151 117Q151 38 213 38Q269 38 323 108L331 118L384 328",
      119894: "184 600Q184 624 203 642T247 661Q265 661 277 649T290 619Q290 596 270 577T226 557Q211 557 198 567T184 600ZM21 287Q21 295 30 318T54 369T98 420T158 442Q197 442 223 419T250 357Q250 340 236 301T196 196T154 83Q149 61 149 51Q149 26 166 26Q175 26 185 29T208 43T235 78T260 137Q263 149 265 151T282 153Q302 153 302 143Q302 135 293 112T268 61T223 11T161 -11Q129 -11 102 10T74 74Q74 91 79 106T122 220Q160 321 166 341T173 380Q173 404 156 404H154Q124 404 99 371T61 287Q60 286 59 284T58 281T56 279T53 278T49 278T41 278H27Q21 284 21 287",
      119895: "297 596Q297 627 318 644T361 661Q378 661 389 651T403 623Q403 595 384 576T340 557Q322 557 310 567T297 596ZM288 376Q288 405 262 405Q240 405 220 393T185 362T161 325T144 293L137 279Q135 278 121 278H107Q101 284 101 286T105 299Q126 348 164 391T252 441Q253 441 260 441T272 442Q296 441 316 432Q341 418 354 401T367 348V332L318 133Q267 -67 264 -75Q246 -125 194 -164T75 -204Q25 -204 7 -183T-12 -137Q-12 -110 7 -91T53 -71Q70 -71 82 -81T95 -112Q95 -148 63 -167Q69 -168 77 -168Q111 -168 139 -140T182 -74L193 -32Q204 11 219 72T251 197T278 308T289 365Q289 372 288 376",
      119896: "121 647Q121 657 125 670T137 683Q138 683 209 688T282 694Q294 694 294 686Q294 679 244 477Q194 279 194 272Q213 282 223 291Q247 309 292 354T362 415Q402 442 438 442Q468 442 485 423T503 369Q503 344 496 327T477 302T456 291T438 288Q418 288 406 299T394 328Q394 353 410 369T442 390L458 393Q446 405 434 405H430Q398 402 367 380T294 316T228 255Q230 254 243 252T267 246T293 238T320 224T342 206T359 180T365 147Q365 130 360 106T354 66Q354 26 381 26Q429 26 459 145Q461 153 479 153H483Q499 153 499 144Q499 139 496 130Q455 -11 378 -11Q333 -11 305 15T277 90Q277 108 280 121T283 145Q283 167 269 183T234 206T200 217T182 220H180Q168 178 159 139T145 81T136 44T129 20T122 7T111 -2Q98 -11 83 -11Q66 -11 57 -1T48 16Q48 26 85 176T158 471L195 616Q196 629 188 632T149 637H144Q134 637 131 637T124 640T121 647",
      119897: "117 59Q117 26 142 26Q179 26 205 131Q211 151 215 152Q217 153 225 153H229Q238 153 241 153T246 151T248 144Q247 138 245 128T234 90T214 43T183 6T137 -11Q101 -11 70 11T38 85Q38 97 39 102L104 360Q167 615 167 623Q167 626 166 628T162 632T157 634T149 635T141 636T132 637T122 637Q112 637 109 637T101 638T95 641T94 647Q94 649 96 661Q101 680 107 682T179 688Q194 689 213 690T243 693T254 694Q266 694 266 686Q266 675 193 386T118 83Q118 81 118 75T117 65V59",
      119898: "21 287Q22 293 24 303T36 341T56 388T88 425T132 442T175 435T205 417T221 395T229 376L231 369Q231 367 232 367L243 378Q303 442 384 442Q401 442 415 440T441 433T460 423T475 411T485 398T493 385T497 373T500 364T502 357L510 367Q573 442 659 442Q713 442 746 415T780 336Q780 285 742 178T704 50Q705 36 709 31T724 26Q752 26 776 56T815 138Q818 149 821 151T837 153Q857 153 857 145Q857 144 853 130Q845 101 831 73T785 17T716 -10Q669 -10 648 17T627 73Q627 92 663 193T700 345Q700 404 656 404H651Q565 404 506 303L499 291L466 157Q433 26 428 16Q415 -11 385 -11Q372 -11 364 -4T353 8T350 18Q350 29 384 161L420 307Q423 322 423 345Q423 404 379 404H374Q288 404 229 303L222 291L189 157Q156 26 151 16Q138 -11 108 -11Q95 -11 87 -5T76 7T74 17Q74 30 112 181Q151 335 151 342Q154 357 154 369Q154 405 129 405Q107 405 92 377T69 316T57 280Q55 278 41 278H27Q21 284 21 287",
      119899: "21 287Q22 293 24 303T36 341T56 388T89 425T135 442Q171 442 195 424T225 390T231 369Q231 367 232 367L243 378Q304 442 382 442Q436 442 469 415T503 336T465 179T427 52Q427 26 444 26Q450 26 453 27Q482 32 505 65T540 145Q542 153 560 153Q580 153 580 145Q580 144 576 130Q568 101 554 73T508 17T439 -10Q392 -10 371 17T350 73Q350 92 386 193T423 345Q423 404 379 404H374Q288 404 229 303L222 291L189 157Q156 26 151 16Q138 -11 108 -11Q95 -11 87 -5T76 7T74 17Q74 30 112 180T152 343Q153 348 153 366Q153 405 129 405Q91 405 66 305Q60 285 60 284Q58 278 41 278H27Q21 284 21 287",
      119900: "201 -11Q126 -11 80 38T34 156Q34 221 64 279T146 380Q222 441 301 441Q333 441 341 440Q354 437 367 433T402 417T438 387T464 338T476 268Q476 161 390 75T201 -11ZM121 120Q121 70 147 48T206 26Q250 26 289 58T351 142Q360 163 374 216T388 308Q388 352 370 375Q346 405 306 405Q243 405 195 347Q158 303 140 230T121 120",
      119901: "23 287Q24 290 25 295T30 317T40 348T55 381T75 411T101 433T134 442Q209 442 230 378L240 387Q302 442 358 442Q423 442 460 395T497 281Q497 173 421 82T249 -10Q227 -10 210 -4Q199 1 187 11T168 28L161 36Q160 35 139 -51T118 -138Q118 -144 126 -145T163 -148H188Q194 -155 194 -157T191 -175Q188 -187 185 -190T172 -194Q170 -194 161 -194T127 -193T65 -192Q-5 -192 -24 -194H-32Q-39 -187 -39 -183Q-37 -156 -26 -148H-6Q28 -147 33 -136Q36 -130 94 103T155 350Q156 355 156 364Q156 405 131 405Q109 405 94 377T71 316T59 280Q57 278 43 278H29Q23 284 23 287ZM178 102Q200 26 252 26Q282 26 310 49T356 107Q374 141 392 215T411 325V331Q411 405 350 405Q339 405 328 402T306 393T286 380T269 365T254 350T243 336T235 326L232 322Q232 321 229 308T218 264T204 212Q178 106 178 102",
      119902: "33 157Q33 258 109 349T280 441Q340 441 372 389Q373 390 377 395T388 406T404 418Q438 442 450 442Q454 442 457 439T460 434Q460 425 391 149Q320 -135 320 -139Q320 -147 365 -148H390Q396 -156 396 -157T393 -175Q389 -188 383 -194H370Q339 -192 262 -192Q234 -192 211 -192T174 -192T157 -193Q143 -193 143 -185Q143 -182 145 -170Q149 -154 152 -151T172 -148Q220 -148 230 -141Q238 -136 258 -53T279 32Q279 33 272 29Q224 -10 172 -10Q117 -10 75 30T33 157ZM352 326Q329 405 277 405Q242 405 210 374T160 293Q131 214 119 129Q119 126 119 118T118 106Q118 61 136 44T179 26Q233 26 290 98L298 109L352 326",
      119903: "21 287Q22 290 23 295T28 317T38 348T53 381T73 411T99 433T132 442Q161 442 183 430T214 408T225 388Q227 382 228 382T236 389Q284 441 347 441H350Q398 441 422 400Q430 381 430 363Q430 333 417 315T391 292T366 288Q346 288 334 299T322 328Q322 376 378 392Q356 405 342 405Q286 405 239 331Q229 315 224 298T190 165Q156 25 151 16Q138 -11 108 -11Q95 -11 87 -5T76 7T74 17Q74 30 114 189T154 366Q154 405 128 405Q107 405 92 377T68 316T57 280Q55 278 41 278H27Q21 284 21 287",
      119904: "131 289Q131 321 147 354T203 415T300 442Q362 442 390 415T419 355Q419 323 402 308T364 292Q351 292 340 300T328 326Q328 342 337 354T354 372T367 378Q368 378 368 379Q368 382 361 388T336 399T297 405Q249 405 227 379T204 326Q204 301 223 291T278 274T330 259Q396 230 396 163Q396 135 385 107T352 51T289 7T195 -10Q118 -10 86 19T53 87Q53 126 74 143T118 160Q133 160 146 151T160 120Q160 94 142 76T111 58Q109 57 108 57T107 55Q108 52 115 47T146 34T201 27Q237 27 263 38T301 66T318 97T323 122Q323 150 302 164T254 181T195 196T148 231Q131 256 131 289",
      119905: "26 385Q19 392 19 395Q19 399 22 411T27 425Q29 430 36 430T87 431H140L159 511Q162 522 166 540T173 566T179 586T187 603T197 615T211 624T229 626Q247 625 254 615T261 596Q261 589 252 549T232 470L222 433Q222 431 272 431H323Q330 424 330 420Q330 398 317 385H210L174 240Q135 80 135 68Q135 26 162 26Q197 26 230 60T283 144Q285 150 288 151T303 153H307Q322 153 322 145Q322 142 319 133Q314 117 301 95T267 48T216 6T155 -11Q125 -11 98 4T59 56Q57 64 57 83V101L92 241Q127 382 128 383Q128 385 77 385H26",
      119906: "21 287Q21 295 30 318T55 370T99 420T158 442Q204 442 227 417T250 358Q250 340 216 246T182 105Q182 62 196 45T238 27T291 44T328 78L339 95Q341 99 377 247Q407 367 413 387T427 416Q444 431 463 431Q480 431 488 421T496 402L420 84Q419 79 419 68Q419 43 426 35T447 26Q469 29 482 57T512 145Q514 153 532 153Q551 153 551 144Q550 139 549 130T540 98T523 55T498 17T462 -8Q454 -10 438 -10Q372 -10 347 46Q345 45 336 36T318 21T296 6T267 -6T233 -11Q189 -11 155 7Q103 38 103 113Q103 170 138 262T173 379Q173 380 173 381Q173 390 173 393T169 400T158 404H154Q131 404 112 385T82 344T65 302T57 280Q55 278 41 278H27Q21 284 21 287",
      119907: "173 380Q173 405 154 405Q130 405 104 376T61 287Q60 286 59 284T58 281T56 279T53 278T49 278T41 278H27Q21 284 21 287Q21 294 29 316T53 368T97 419T160 441Q202 441 225 417T249 361Q249 344 246 335Q246 329 231 291T200 202T182 113Q182 86 187 69Q200 26 250 26Q287 26 319 60T369 139T398 222T409 277Q409 300 401 317T383 343T365 361T357 383Q357 405 376 424T417 443Q436 443 451 425T467 367Q467 340 455 284T418 159T347 40T241 -11Q177 -11 139 22Q102 54 102 117Q102 148 110 181T151 298Q173 362 173 380",
      119908: "580 385Q580 406 599 424T641 443Q659 443 674 425T690 368Q690 339 671 253Q656 197 644 161T609 80T554 12T482 -11Q438 -11 404 5T355 48Q354 47 352 44Q311 -11 252 -11Q226 -11 202 -5T155 14T118 53T104 116Q104 170 138 262T173 379Q173 380 173 381Q173 390 173 393T169 400T158 404H154Q131 404 112 385T82 344T65 302T57 280Q55 278 41 278H27Q21 284 21 287Q21 293 29 315T52 366T96 418T161 441Q204 441 227 416T250 358Q250 340 217 250T184 111Q184 65 205 46T258 26Q301 26 334 87L339 96V119Q339 122 339 128T340 136T341 143T342 152T345 165T348 182T354 206T362 238T373 281Q402 395 406 404Q419 431 449 431Q468 431 475 421T483 402Q483 389 454 274T422 142Q420 131 420 107V100Q420 85 423 71T442 42T487 26Q558 26 600 148Q609 171 620 213T632 273Q632 306 619 325T593 357T580 385",
      119909: "52 289Q59 331 106 386T222 442Q257 442 286 424T329 379Q371 442 430 442Q467 442 494 420T522 361Q522 332 508 314T481 292T458 288Q439 288 427 299T415 328Q415 374 465 391Q454 404 425 404Q412 404 406 402Q368 386 350 336Q290 115 290 78Q290 50 306 38T341 26Q378 26 414 59T463 140Q466 150 469 151T485 153H489Q504 153 504 145Q504 144 502 134Q486 77 440 33T333 -11Q263 -11 227 52Q186 -10 133 -10H127Q78 -10 57 16T35 71Q35 103 54 123T99 143Q142 143 142 101Q142 81 130 66T107 46T94 41L91 40Q91 39 97 36T113 29T132 26Q168 26 194 71Q203 87 217 139T245 247T261 313Q266 340 266 352Q266 380 251 392T217 404Q177 404 142 372T93 290Q91 281 88 280T72 278H58Q52 284 52 289",
      119910: "21 287Q21 301 36 335T84 406T158 442Q199 442 224 419T250 355Q248 336 247 334Q247 331 231 288T198 191T182 105Q182 62 196 45T238 27Q261 27 281 38T312 61T339 94Q339 95 344 114T358 173T377 247Q415 397 419 404Q432 431 462 431Q475 431 483 424T494 412T496 403Q496 390 447 193T391 -23Q363 -106 294 -155T156 -205Q111 -205 77 -183T43 -117Q43 -95 50 -80T69 -58T89 -48T106 -45Q150 -45 150 -87Q150 -107 138 -122T115 -142T102 -147L99 -148Q101 -153 118 -160T152 -167H160Q177 -167 186 -165Q219 -156 247 -127T290 -65T313 -9T321 21L315 17Q309 13 296 6T270 -6Q250 -11 231 -11Q185 -11 150 11T104 82Q103 89 103 113Q103 170 138 262T173 379Q173 380 173 381Q173 390 173 393T169 400T158 404H154Q131 404 112 385T82 344T65 302T57 280Q55 278 41 278H27Q21 284 21 287",
      119911: "347 338Q337 338 294 349T231 360Q211 360 197 356T174 346T162 335T155 324L153 320Q150 317 138 317Q117 317 117 325Q117 330 120 339Q133 378 163 406T229 440Q241 442 246 442Q271 442 291 425T329 392T367 375Q389 375 411 408T434 441Q435 442 449 442H462Q468 436 468 434Q468 430 463 420T449 399T432 377T418 358L411 349Q368 298 275 214T160 106L148 94L163 93Q185 93 227 82T290 71Q328 71 360 90T402 140Q406 149 409 151T424 153Q443 153 443 143Q443 138 442 134Q425 72 376 31T278 -11Q252 -11 232 6T193 40T155 57Q111 57 76 -3Q70 -11 59 -11H54H41Q35 -5 35 -2Q35 13 93 84Q132 129 225 214T340 322Q352 338 347 338",
      119912: "65 0Q45 0 45 18Q48 52 61 60Q65 62 81 62Q155 62 165 74Q166 74 265 228T465 539T569 699Q576 707 583 709T611 711T637 710T649 700Q650 697 695 380L741 63L784 62H827Q839 50 839 45L835 29Q831 9 827 5T806 0Q803 0 790 0T743 1T657 2Q585 2 547 1T504 0Q481 0 481 17Q484 54 497 60Q501 62 541 62Q580 62 580 63Q580 68 573 121T564 179V181H308L271 124Q236 69 236 67T283 62H287Q316 62 316 46Q316 26 307 8Q302 3 295 0L262 1Q242 2 168 2Q119 2 93 1T65 0ZM537 372Q533 402 528 435T521 486T518 504V505Q517 505 433 375L348 244L451 243Q555 243 555 244L537 372",
      119913: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 449 686H477H586Q684 686 733 677T817 634Q853 598 853 547Q853 499 826 460T761 401T695 371T654 360H653L662 358Q670 357 683 354T712 344T744 327T774 303T795 269T804 224Q804 148 732 79T533 1Q524 0 288 0H58Q47 5 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624ZM703 550Q703 571 695 586T675 609T656 619T643 623L545 624H447L417 504Q386 384 386 383T470 382Q554 383 565 385Q632 397 667 447T703 550ZM651 240Q651 265 645 282T626 309T608 322T592 329Q587 330 479 331H373L340 198Q307 65 306 64Q306 62 406 62L507 63L519 65Q565 76 596 107T639 171T651 240",
      119914: "380 -17Q335 -17 293 -10T207 16T130 65T76 144T55 256Q55 306 70 361T122 476T211 582T345 663T525 702H545Q673 702 731 634L777 668Q783 672 789 677T800 685T808 691T814 695T818 698T822 700T825 702T828 703T830 703T833 703Q855 703 855 690Q855 686 823 558T789 426Q786 421 782 420T756 419Q734 420 729 421T724 432Q724 434 725 447T726 472Q726 552 678 604Q640 640 586 640H574Q533 640 494 632T409 604T324 541T260 437Q243 397 227 333T210 219Q210 152 237 117Q255 90 299 68T420 46H429Q506 46 580 100T678 234Q683 249 687 251T712 254H723Q743 254 743 240Q743 232 736 213T710 162T663 100T586 40T477 -5Q433 -17 380 -17",
      119915: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 437 686Q659 686 668 685Q727 680 772 662T842 621T883 568T905 517T913 475Q914 466 914 434Q914 373 892 307T828 179T712 69T548 7Q517 2 494 2T279 0H58Q47 5 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624ZM768 475Q768 515 753 544T718 588T666 611T613 622T563 624H538H532H452L382 344Q311 64 311 63T363 62H405Q490 62 545 76T656 142Q696 185 724 265T760 399T768 475",
      119916: "257 618H231Q198 618 198 636Q202 672 214 678L219 680H811Q817 677 820 673T824 666L825 664Q825 659 814 549T799 433Q793 424 771 424Q752 424 746 427T740 441Q740 445 742 466T744 505Q744 561 722 585T646 616Q639 617 545 618H456Q456 617 427 502T398 385Q398 384 435 384Q461 385 471 385T499 391T526 405T545 433T562 478Q566 494 571 497T595 501H604Q622 501 626 486Q626 482 593 349T557 213Q552 205 530 205Q499 205 499 219Q499 222 503 242T508 281Q508 308 491 314T429 322Q425 322 423 322H382L317 64Q317 62 390 62Q460 62 493 64T569 80T640 124Q665 149 686 187T719 253T733 283Q739 289 760 289Q791 289 791 274Q791 267 763 201T706 71L678 8Q676 4 667 0H58Q47 5 43 15Q47 54 60 60Q64 62 113 62H162L163 66Q163 67 231 341T301 616Q301 618 257 618",
      119917: "257 618H231Q198 618 198 636Q202 672 214 678L219 680H795Q801 677 804 673T808 666L809 664Q809 659 798 549T783 433Q777 424 755 424Q736 424 730 427T724 444Q724 448 725 468T727 507V524Q727 541 724 554T713 577T698 594T676 605T653 612T625 616T597 617T566 618T538 618H456L455 614Q455 611 424 491L394 371H429Q454 372 463 372T491 378T517 392T536 419T552 464Q556 481 561 484T586 488Q603 488 607 486Q616 482 616 473Q616 467 584 337T549 201Q542 192 521 192Q503 192 497 195T490 209Q490 212 492 224Q499 251 499 269Q499 288 489 296T465 306T417 308L379 309L348 188Q341 161 334 129T322 80L318 65L317 62H375H409Q430 62 438 59T447 45Q444 8 431 2L426 0L377 1Q347 2 231 2Q152 2 111 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L163 66Q163 67 231 341T301 616Q301 618 257 618",
      119918: "379 -16Q233 -16 145 52T56 255Q56 310 73 368T127 483T216 586T347 663T518 702H540Q562 702 582 700T616 696T644 689T667 681T686 670T702 659T717 647T731 635L776 668Q782 672 788 677T799 685T807 691T813 695T817 698T821 700T824 702T827 703T829 703T832 703Q854 703 854 690Q854 686 822 558T788 426Q785 421 781 420T755 419Q734 420 729 422T723 432Q723 434 724 446T725 469Q725 531 702 571T642 628Q616 640 575 640Q468 640 390 593T272 464Q247 415 229 340T210 214Q210 166 228 132T277 79T343 54T419 46Q445 46 465 50T500 59T526 76T544 96T557 123T566 150T574 182T581 214H519Q511 214 498 214T479 213Q443 213 443 230Q443 250 452 268Q457 273 464 276L514 275Q546 274 657 274Q735 274 768 275T803 276Q826 276 826 258Q823 224 810 216Q806 214 771 214H736Q736 211 710 109T683 5Q678 0 671 0Q666 0 637 14T597 36Q593 38 590 40T585 44T582 44T576 40Q511 -16 379 -16",
      119919: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 239 686Q290 684 403 684Q475 684 512 685T553 686Q576 686 576 668Q572 632 560 626Q555 624 506 624H457L399 389Q399 388 547 388H695L753 623Q753 624 709 624H686Q665 624 660 626T650 639Q653 678 668 684Q672 686 681 686Q685 686 726 685T847 684Q902 684 937 684T986 685T1004 686Q1027 686 1027 668Q1023 632 1011 626Q1006 624 957 624H908L839 344Q768 64 768 63T812 62H839Q871 62 871 44Q867 6 854 2L850 0L808 1Q782 2 675 2Q600 2 560 1T516 0Q499 0 494 15Q498 54 511 60Q515 62 564 62H613L614 66L679 324Q679 326 531 326H383L382 322L317 64Q317 62 361 62H388Q420 62 420 44Q416 6 403 2L399 0L357 1Q331 2 224 2Q149 2 109 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624",
      119920: "247 624Q242 624 233 624T220 623Q186 623 186 640Q186 647 190 664T202 684Q206 686 226 686Q277 684 393 684Q435 684 471 684T528 685T553 686Q573 686 573 670Q573 650 564 632Q556 624 537 624H501H449L380 344Q309 64 309 63T356 62Q361 62 370 62T384 63Q417 63 417 46Q417 26 408 8Q403 3 396 0L352 1Q325 2 216 2T82 1L45 0Q30 7 30 16Q33 51 46 60Q51 62 102 62H154L294 623Q294 624 247 624",
      119921: "205 131Q205 105 192 84T165 54L152 45Q152 44 160 42T182 37T213 35H216Q255 35 289 65Q314 90 329 129Q331 136 392 378T453 623Q453 624 393 624H332Q318 631 318 640Q318 647 322 664T334 684Q338 686 359 686Q413 684 533 684Q566 684 605 685T652 686Q677 686 685 683T694 669Q694 664 691 652Q686 631 681 628T647 624H602L542 380Q531 336 518 285T500 212T487 161T475 122T463 97T448 74T429 55Q351 -17 213 -17Q142 -17 99 7T43 70Q42 75 42 93Q42 143 73 168T139 194Q168 194 186 177T205 131",
      119922: "536 0Q522 6 522 18Q522 35 533 57Q539 62 557 62Q595 62 601 65L472 330L365 255L342 160Q318 65 317 64Q317 62 361 62H388Q420 62 420 44Q416 6 403 2L399 0L357 1Q331 2 224 2Q149 2 109 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 239 686Q290 684 403 684Q475 684 512 685T553 686Q576 686 576 668Q572 632 560 626Q555 624 506 624H457L422 481Q386 339 386 337L785 621Q779 624 749 624Q726 624 726 641Q726 645 730 659Q734 675 736 679T747 686L786 685Q812 684 888 684Q908 684 934 685T968 686Q1003 686 1003 669Q1003 646 991 629Q985 624 967 624Q918 624 888 617Q884 617 874 613L865 609Q864 608 732 515T599 420Q599 418 686 242T775 65Q784 62 829 62Q847 62 850 61T860 54Q862 52 862 43Q862 10 845 1Q844 1 842 1T836 0T797 1T694 2Q599 2 573 1L536 0",
      119923: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 239 686Q290 684 409 684Q454 684 492 684T552 685T579 686Q603 686 603 668Q599 632 587 626Q583 624 520 624H457L388 344Q317 64 317 63T353 62H390Q418 62 440 64T493 78T548 110T598 169T643 261Q651 282 655 285T680 289Q696 289 700 288T709 279Q711 274 711 269Q710 265 663 138T613 8Q611 4 602 0H58Q47 5 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624",
      119924: "258 624H231Q214 624 208 626T199 639Q203 678 216 684Q220 686 347 686H473Q474 685 478 682T484 677Q487 673 535 413L584 153L608 187Q631 221 672 281T761 410Q935 663 943 671Q949 678 962 686H1082H1166Q1201 686 1210 683T1219 668Q1215 632 1203 626Q1199 624 1149 624H1100L1031 344Q960 64 960 63T1004 62H1031Q1063 62 1063 44Q1060 7 1046 2Q1042 0 1034 0Q1030 0 990 1T875 2Q804 2 767 1T725 0H723Q707 0 703 15Q707 54 720 60Q724 62 773 62H822Q961 618 961 619L754 318Q546 15 543 12Q531 0 510 0Q500 0 495 0T484 5T477 19Q477 20 421 315L367 604L299 335Q234 72 234 68Q234 62 302 62Q334 62 334 46Q332 8 317 2Q313 0 306 0Q301 0 267 1T181 2Q125 2 96 1T63 0Q48 0 43 15Q43 19 47 35Q52 55 57 58T94 62Q147 64 164 69L233 345Q302 619 302 622Q302 624 258 624",
      119925: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 344 686H434Q464 686 477 680Q480 677 607 454Q738 227 739 227Q742 227 789 418T836 618Q836 620 835 620L821 622Q811 622 779 624Q755 624 749 625T740 632Q737 635 737 644Q737 656 742 669T754 685Q755 685 757 685T763 686Q768 686 803 685T890 684Q925 684 951 684T990 685T1006 686Q1014 686 1016 684Q1027 679 1027 668Q1023 632 1011 626Q1007 624 978 624Q912 622 907 617Q907 616 831 314T753 8Q749 0 723 0H712Q699 0 692 7Q692 8 671 44T607 155T526 296L361 580L296 323Q234 74 234 68T302 62H307Q334 62 334 44Q330 6 317 2L313 0L280 1Q260 2 181 2Q125 2 96 1T63 0Q48 0 43 15Q43 19 47 35Q52 55 57 58T94 62Q147 64 164 69L233 345Q302 619 302 622Q302 624 258 624",
      119926: "53 245Q53 297 70 356T125 478T216 590T349 671T523 703Q656 703 735 637T815 445Q815 378 791 307Q727 104 527 17Q437 -17 344 -17Q289 -17 242 -5T150 35T79 116T53 245ZM664 489Q664 575 618 611T511 648Q463 648 416 627T334 570Q297 531 270 472T230 355T213 261T208 206Q208 177 215 151T237 98T284 56T358 40Q440 40 510 98T618 270Q664 400 664 489",
      119927: "162 62L302 623Q302 624 258 624H234Q214 624 209 626T200 638Q200 677 217 684Q220 686 439 686Q667 685 684 682Q686 681 693 680Q713 677 733 671T782 649T829 602T847 528Q847 450 784 382T604 293Q571 288 469 287H373L346 176Q340 151 333 122T321 78L317 64Q317 62 361 62H387Q420 62 420 44Q417 10 404 2L399 0L357 1Q331 2 224 2Q149 2 109 1T65 0Q43 0 43 17Q43 21 47 33Q52 54 57 58T89 62H113H162ZM692 558Q692 611 617 622Q610 623 529 624H452L381 343H458H492Q604 343 641 389Q662 414 677 471T692 558",
      119928: "53 245Q53 297 70 356T125 478T216 590T349 671T523 703Q656 703 735 637T815 445Q815 410 808 370T781 277T729 178T643 87T519 14L525 4Q540 -19 553 -25T592 -32Q632 -32 654 -24T680 -7T689 10T704 18Q713 18 717 12T722 0Q722 -8 711 -36T681 -101T624 -166T541 -194Q513 -194 494 -183T465 -157T450 -118T444 -79T443 -41V-7L433 -9Q391 -17 344 -17Q301 -17 263 -10T185 15T118 62T71 138T53 245ZM666 482Q666 529 652 563T614 615T565 640T512 648Q412 648 335 573Q268 506 235 389T201 202Q201 164 210 136T230 95T259 66L262 76Q269 109 302 135T382 162Q401 162 415 159T449 140T484 92L491 78L496 82Q502 86 505 88T515 97T528 107T541 120T555 137T570 156T585 179T599 205T612 235Q629 278 647 351T666 482ZM439 56Q439 58 439 62T435 75T426 92T410 106T383 112Q353 112 332 96T311 63Q311 38 355 38H366Q391 39 415 45T439 56",
      119929: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 422 686H446H525Q634 686 698 674T806 620Q843 583 843 535Q843 505 833 478T805 432T768 396T728 370T690 352T662 342L651 338L654 336Q658 334 667 327T688 310Q719 278 719 237Q719 222 710 165T701 94Q701 35 748 35Q775 35 793 57T819 101Q822 112 826 114T843 117H849Q881 117 881 99Q881 78 852 39T781 -11Q765 -17 728 -17Q537 -13 537 94Q537 110 552 169T567 243Q567 292 529 309Q517 316 508 316T441 318H375L374 314Q374 312 343 189T311 64Q311 62 355 62H382Q414 62 414 44Q410 6 397 2L393 0L351 1Q325 2 221 2Q147 2 108 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624ZM687 555Q687 617 589 623Q581 624 513 624H451L420 498Q413 468 405 436T392 388L388 371Q388 369 458 369Q464 369 485 369T515 369T541 372T570 377T596 386T624 400Q649 417 664 457T683 522T687 555",
      119930: "354 45Q429 45 467 91T506 184Q506 225 477 250Q461 262 384 279T294 300Q238 318 210 362T182 452Q182 493 202 540T257 623Q338 702 449 702Q491 702 501 701Q571 690 610 654Q614 649 617 650Q618 650 650 675T683 702Q685 703 692 703Q714 703 714 690Q714 686 687 578T658 466Q653 458 629 458Q606 458 602 463Q600 465 599 466Q599 467 599 469T598 473Q598 475 599 487T600 511Q600 584 557 614T454 645Q386 645 347 606T308 520Q308 506 311 496T323 477T338 464T360 454T384 446T413 439T441 433Q523 416 549 401Q581 384 602 352Q631 309 631 254Q631 214 615 170T567 83T478 12T347 -16Q322 -16 300 -14T261 -8T230 0T205 10T187 20T172 30L162 37L130 11Q124 7 119 3T110 -4T104 -9T100 -13T96 -15T93 -16T91 -17T88 -17H82Q76 -17 73 -16T69 -14T66 -10T63 -5L90 107Q97 133 106 170Q116 211 120 219T136 228H148Q167 228 173 227T179 218Q179 216 176 200T173 168Q173 102 227 74T354 45",
      119931: "498 62Q511 58 511 43Q511 10 494 1L490 0Q487 0 482 0T424 1T271 2Q201 2 157 2T94 1T72 0H70Q46 0 46 17Q49 54 62 60L66 62H137Q208 62 209 63L218 98Q227 134 244 203T278 339L347 613H300Q262 612 246 611T198 599T146 564Q128 545 114 512T91 454T79 425Q73 419 52 419Q22 419 22 434Q22 440 41 498T80 611L100 666Q105 673 111 675H434Q758 675 762 673Q772 668 772 657Q772 655 756 549T738 434Q735 419 711 419H707Q690 419 686 421Q677 425 677 434Q676 436 678 449T683 485T686 529Q686 553 679 569T662 594T631 607T593 612T544 613H502L433 340Q418 279 400 207T374 100L365 65L364 62H498",
      119932: "856 686Q877 686 877 668Q877 663 873 649T867 631Q861 624 828 624Q762 622 757 617Q757 613 705 409T651 200Q620 112 540 48T328 -16Q251 -16 196 2T113 51T75 112T63 176Q63 202 70 232T117 422Q129 469 141 520T160 598L167 623Q167 624 123 624H96Q79 624 73 626T64 639Q68 678 81 684Q85 686 104 686Q155 684 268 684Q340 684 377 685T418 686Q441 686 441 668Q437 632 425 626Q421 624 371 624H322L270 415Q224 232 217 198T209 141Q209 45 336 45Q372 45 406 52T475 77T540 128T585 211L590 229Q594 247 601 274T617 336T636 409T654 482T670 547T681 595T686 618Q686 620 685 620H683Q681 621 678 621T671 622Q660 622 630 624Q616 624 610 624T598 626T589 630T587 640Q587 647 590 659Q594 677 598 681T613 686Q618 686 653 685T740 684Q775 684 801 684T840 685T856 686",
      119933: "401 686Q415 680 415 668Q415 651 404 629Q398 624 356 624Q318 624 318 623Q318 620 337 508T377 284L397 174L472 285Q548 396 623 507T699 620Q698 621 652 624Q634 624 627 627T619 641Q619 648 622 658Q627 677 631 681T650 686Q654 686 686 685T766 684Q794 684 823 684T858 685Q874 685 878 683T886 671Q886 667 882 651Q877 632 873 628T850 624Q800 624 779 617Q774 617 770 613Q767 610 560 304T350 -5Q346 -9 332 -16H306H291Q270 -16 267 -2Q267 -1 260 37T238 161T210 313L156 624H116H94Q62 624 62 642Q66 678 78 684Q82 686 99 686Q144 684 246 684Q330 684 368 685L401 686",
      119934: "111 624Q109 624 102 624T91 623Q61 623 61 640Q61 660 70 678Q78 686 98 686Q140 684 239 684Q277 684 309 684T360 685T383 686H385Q407 686 407 668Q404 634 391 626Q387 624 348 624Q307 624 307 622Q307 618 332 409Q359 198 359 195L570 532L564 576L558 622V624H522H504Q472 624 472 641Q475 678 488 684L493 686L529 685Q551 684 645 684Q716 684 753 685T795 686Q818 686 818 669Q815 632 802 626Q798 624 759 624Q718 624 718 622Q718 615 743 410Q770 199 770 196Q770 195 806 253T903 406Q1035 618 1035 619Q1025 624 968 624Q943 624 943 641Q943 648 946 659Q950 675 952 679T963 686L998 685Q1020 684 1093 684Q1113 684 1139 685T1173 686Q1207 686 1207 669Q1207 664 1204 652Q1199 631 1194 628T1164 624Q1113 622 1101 615Q1098 612 905 305Q715 -1 709 -7Q699 -17 673 -17Q645 -17 639 -8L581 441Q581 444 442 221Q331 44 314 18T288 -14Q279 -17 263 -17H254Q229 -17 227 -5Q225 2 186 311L147 620V624H111",
      119935: "931 686Q953 686 953 670Q953 650 944 632Q936 624 924 624H914Q823 624 803 611Q800 609 696 503T591 396Q591 394 667 229L743 62H787H814Q846 62 846 44Q843 7 829 2Q825 0 817 0Q813 0 775 1T664 2Q590 2 551 1T508 0H507Q484 0 484 18Q484 19 488 37Q492 56 497 58T534 62L566 63Q567 64 520 169T471 274Q469 274 369 172T268 67L315 62Q320 62 328 62L335 61Q347 58 347 44Q344 10 331 2L326 0L287 1Q263 2 177 2Q95 2 78 1L53 0Q38 6 38 17Q38 40 50 57Q56 62 78 62Q169 62 188 75Q194 77 435 324L444 334L439 347Q437 351 373 492L313 624H268H246Q220 624 212 632Q210 636 210 642Q210 655 215 669T227 684Q230 686 247 686Q295 684 398 684Q438 684 472 684T527 685T551 686Q567 686 572 671Q572 667 568 651Q563 631 558 628T523 624T492 623H488L526 540Q563 457 564 457Q564 456 574 466T604 496T645 537L724 619Q716 622 677 624H673Q645 624 645 640Q645 660 654 678Q659 683 666 686L704 685Q728 684 813 684Q847 684 873 684T913 685T931 686",
      119936: "97 624H73Q40 624 40 640Q40 660 49 678Q57 686 77 686Q122 684 228 684Q269 684 304 684T360 685T385 686Q406 686 406 668Q406 662 403 653Q398 631 393 628T361 624H353Q321 624 321 623T376 491T432 360L448 377Q465 394 493 424T553 490L673 620Q662 624 630 624Q606 624 606 640Q608 678 623 684Q627 686 634 686Q638 686 671 685T755 684Q777 684 805 685T841 686Q861 686 868 683T876 669Q876 664 873 652Q868 631 863 628T829 624Q764 622 747 611Q727 590 590 441L437 275L411 170Q385 65 384 64Q384 62 429 62H453Q473 62 478 60T487 48Q488 44 484 29Q479 6 473 3Q468 0 454 0Q450 0 436 0T386 1T294 2Q220 2 181 1T138 0Q121 0 116 15Q120 54 133 60Q137 62 187 62H236L289 275L142 624H97",
      119937: "223 430Q192 430 192 448Q192 450 225 561T261 677Q265 683 270 684Q273 686 534 686Q796 686 797 685Q805 682 805 673Q805 668 804 661T800 648T798 641Q796 637 531 352L266 67L329 66H364Q412 66 446 70T523 96T596 157Q617 186 630 220T649 273T663 297Q667 299 684 299H688Q715 299 715 281Q715 278 673 145T628 8Q626 4 617 0H348Q289 0 221 0T139 -1Q112 -1 99 -1T78 1T69 5T68 12Q68 16 71 31T77 49L84 57Q91 65 104 79T133 110T170 151T213 196L610 624H540Q533 624 514 624T488 624T467 623T443 620T422 616T398 609T373 600Q292 560 255 449Q251 436 246 433T223 430",
      119938: "222 -8Q140 -8 89 34T38 158Q38 191 48 227Q72 329 151 390T327 452Q361 452 385 443T421 425T433 416H434L441 421Q448 426 460 430T486 435Q509 435 523 422T538 386Q538 380 522 315T488 179T467 93Q466 87 466 72Q466 42 483 42Q505 42 521 75Q531 94 541 134Q546 155 550 158T571 162H576H587Q607 162 607 148Q606 142 604 132T590 94T566 47T528 9T474 -8Q396 -8 358 40Q295 -8 222 -8ZM404 351Q383 401 324 401Q300 401 270 385T221 330Q206 296 186 220Q166 136 166 106Q166 72 184 58T228 43Q256 43 284 57T328 84T343 103Q343 106 374 228L404 351",
      119939: "220 -8Q142 -8 94 35T45 155V167Q45 187 52 218T104 426L153 622H149Q148 622 144 622T134 623T122 624T111 624T101 624T96 625Q84 628 84 642Q84 647 88 661T94 679Q98 684 109 685T185 690Q258 694 272 694Q289 694 293 679Q293 676 263 553L232 429L244 434Q256 440 281 446T331 452Q417 452 465 407T513 285Q513 235 494 184T439 90T346 20T220 -8ZM385 337Q385 400 318 400Q269 400 226 360Q214 349 211 341T191 268Q162 149 162 113Q162 44 226 44Q269 44 299 76T339 135T362 215Q364 222 365 226Q385 303 385 337",
      119940: "362 325Q362 344 371 361T390 386L399 394Q390 401 355 401Q276 401 231 338Q207 301 189 230T170 122Q170 43 264 43Q392 43 457 105Q472 120 480 117Q486 114 497 102T509 83Q509 79 502 70T477 47T432 21T360 1T259 -8Q194 -8 148 9T80 54T49 109T40 167Q40 280 129 365T352 451Q390 451 396 450Q448 442 473 416T499 358T477 302T421 274H417Q393 274 378 288T362 325",
      119941: "222 -8Q140 -8 89 34T38 158Q38 220 68 285T151 391Q230 452 329 452Q382 452 416 428L422 424Q423 424 447 523L472 622H468Q467 622 463 622T453 623T441 624T430 624T420 624T415 625Q403 628 403 642Q403 647 407 661T413 679Q417 684 428 685T504 690Q577 694 591 694Q608 694 612 679L467 91Q466 87 466 72Q466 43 483 43Q518 43 541 134Q546 155 550 158T571 162H576H587Q607 162 607 148Q606 142 604 132T590 94T566 47T528 9T474 -8Q396 -8 358 40Q295 -8 222 -8ZM404 351Q383 401 324 401Q300 401 270 385T221 330Q206 296 186 220Q166 136 166 106Q166 72 184 58T228 43Q256 43 284 57T328 84T343 103Q343 106 374 228L404 351",
      119942: "260 -8Q196 -8 151 9T83 54T52 111T42 169Q42 188 44 210Q50 240 58 266Q127 434 335 451L338 452Q342 452 345 452Q347 452 353 452T363 451Q426 451 464 424T502 352Q502 289 442 250Q381 211 222 211H184Q184 210 181 196T175 162T171 126Q171 43 264 43Q391 43 457 105Q472 120 480 117Q486 114 497 102T509 83Q509 79 502 70T477 47T432 21T360 1T260 -8ZM237 262Q427 266 427 349Q427 368 409 384T354 401Q316 401 287 388T242 354T216 314T202 278L197 263Q197 262 237 262",
      119943: "584 444Q597 439 597 426Q597 409 586 387Q580 382 505 382H434V380Q432 378 421 314T395 162T368 30Q324 -164 203 -199Q194 -201 175 -201Q123 -201 94 -177T64 -117T88 -58T145 -33Q169 -33 184 -47T200 -84Q200 -122 166 -150L174 -151H185Q202 -148 217 -112Q222 -94 240 9Q246 40 262 132T293 303T307 382H247H210Q190 382 182 385T173 400Q177 436 189 442Q193 444 256 444H318L319 446Q337 565 355 602Q373 640 404 664T458 694T503 701Q569 701 596 676T624 617Q624 581 599 557T544 533Q520 533 504 547T488 585Q488 596 491 606T499 624T508 637T516 646L520 650Q515 650 509 651Q459 651 459 561V554L458 518L452 484Q446 448 445 447V444H584",
      119944: "227 0Q142 0 93 43T43 166Q43 207 58 252T104 341T188 414T310 451L313 452Q316 452 319 452Q321 452 326 452T335 451Q367 451 390 443T425 425L436 416L443 421Q450 426 462 430T488 435Q511 435 525 422T540 386Q540 378 483 151T424 -82Q401 -139 335 -170T167 -202Q72 -202 36 -183T0 -125Q0 -90 24 -67T81 -43Q103 -43 119 -56T136 -99Q133 -124 114 -149L123 -150H183Q222 -150 254 -127T299 -74Q300 -71 306 -47T318 -1T324 23Q323 23 310 18T274 6T227 0ZM407 353Q406 354 404 358T400 365T395 371T388 379T381 385T371 391T360 396T346 400T329 401Q300 401 277 389T240 355T219 317T205 280Q171 148 171 109Q171 77 190 64T235 51T296 67Q322 82 337 98L345 106L407 353",
      119945: "477 56Q477 48 479 46T490 43Q522 45 544 75T577 140Q582 156 585 159T605 162H611H622Q642 162 642 148Q642 138 632 114T602 62T550 13T478 -8Q429 -8 394 17T358 83Q358 95 395 199T433 350Q433 400 394 400H388H383Q335 400 291 363Q256 332 236 298Q233 293 202 170T169 40Q160 18 141 5T99 -8Q70 -8 58 9T45 39Q45 51 116 336L188 622H184Q183 622 179 622T169 623T157 624T146 624T136 624T131 625Q119 628 119 642Q119 647 123 661T129 679Q133 684 144 685T220 690Q293 694 307 694Q324 694 328 679Q328 672 294 540Q286 507 278 473T264 420L260 403Q260 400 269 408Q327 451 393 451H401H410Q425 451 439 450T476 442T515 424T544 391T556 337Q556 286 517 179T477 56",
      119946: "205 615Q205 646 229 670T283 694Q310 694 324 679T339 641Q339 610 315 586T258 562Q235 562 220 577T205 615ZM24 296Q24 305 34 328T63 380T115 430T187 452Q205 452 223 448T262 435T295 406T308 360Q308 345 287 290T240 170T207 87Q202 67 202 57Q202 42 215 42Q235 42 257 64Q288 92 302 140Q307 156 310 159T330 162H336H347Q367 162 367 148Q367 140 357 117T329 65T276 14T201 -8Q158 -8 121 15T83 84Q83 104 133 229T184 358Q189 376 189 388Q189 402 177 402Q156 402 134 380Q103 352 89 304Q84 288 81 285T61 282H55H44Q24 282 24 296",
      119947: "321 616Q321 648 346 671T400 694Q425 694 440 680T456 640Q456 609 431 586T376 562Q353 562 337 577T321 616ZM297 360T297 373T294 392T288 400T278 401H276Q241 399 210 372T158 303Q151 287 148 285T129 282H123H116Q102 282 97 284T92 298Q96 331 153 391T279 451Q357 451 388 422T420 354V339L371 142Q363 111 353 71T339 13T329 -30T318 -64T308 -88T296 -109T283 -125T264 -142Q190 -202 88 -202Q44 -202 16 -181T-12 -125Q-12 -93 11 -68T68 -43Q92 -43 108 -58T124 -93Q124 -121 106 -144Q101 -150 103 -150Q120 -150 147 -128T191 -64Q194 -57 242 136T294 345Q297 360 297 373",
      119948: "99 -8Q71 -8 58 9T45 39Q45 51 116 336L188 622H184Q183 622 179 622T169 623T157 624T146 624T136 624T131 625Q119 628 119 642Q119 647 123 661T129 679Q133 684 144 685T220 690Q293 694 307 694Q324 694 328 679Q328 674 280 482Q231 290 231 287Q231 285 234 286Q259 302 294 334T356 390T420 433T493 452Q528 452 546 427T564 364Q564 308 538 282T480 256Q456 256 441 269T425 308Q425 339 444 359T483 384L502 389Q502 395 496 398Q493 400 483 400Q465 400 449 395T409 374T373 347T323 305T268 257Q274 256 282 256Q312 251 329 247T371 232T411 202Q431 181 431 146Q431 132 427 110T422 73Q422 44 440 44H442Q462 44 478 64T502 102T514 141Q518 157 522 159T547 162H558Q578 162 578 148Q578 118 537 56T440 -7H432Q374 -7 337 21T299 94Q299 103 301 116T304 139Q304 164 281 181T235 202L212 206H211Q176 47 160 24Q137 -8 99 -8",
      119949: "157 -8Q138 -8 118 -4T77 11T41 42T27 91V106L156 622H152Q151 622 147 622T137 623T125 624T114 624T104 624T99 625Q87 628 87 642Q87 647 91 661T97 679Q101 684 112 685T188 690Q261 694 275 694Q292 694 296 679L151 91Q150 85 150 71Q150 42 167 42Q186 42 205 75Q216 96 225 134Q230 155 234 158T255 162H260H271Q291 162 291 148Q290 145 289 140T283 118T271 87T254 54T229 23T197 1T157 -8",
      119950: "24 296Q25 302 27 312T41 350T65 397T104 435T159 452Q203 452 234 435Q268 419 285 384L295 392Q305 401 317 410T349 429T389 445Q411 451 446 451Q560 451 592 383Q593 380 594 379L595 375L604 384Q675 452 762 452Q893 452 916 367Q918 356 918 334Q918 285 881 183T841 66Q838 43 849 43Q876 43 901 69T940 138Q945 156 949 159T969 162H975H986Q1006 162 1006 148Q1006 138 996 115T966 63T914 13T841 -8Q794 -8 758 16T721 82Q721 96 758 199T796 351Q796 401 753 401Q702 401 662 369T599 298Q597 294 567 172T533 40Q525 22 506 7T462 -8Q435 -8 422 8T409 39Q409 48 425 114T458 248T476 320Q478 330 478 348T474 377T462 393T449 399T433 400H428Q380 400 336 363Q301 332 281 298Q278 293 247 170T214 40Q206 22 187 7T143 -8T104 7T90 39Q90 47 108 124T146 274L164 347Q166 355 166 372Q166 401 149 401Q129 401 115 379T89 306Q84 288 80 285T55 282H44Q24 282 24 296",
      119951: "24 296Q25 302 27 312T41 350T65 397T104 435T159 452Q203 452 234 435Q268 419 285 384L293 391Q363 452 454 452Q575 446 597 367Q599 356 599 334Q599 285 562 183T522 66Q519 43 530 43Q557 43 582 69T621 138Q626 156 630 159T650 162H656H667Q687 162 687 148Q687 138 677 115T647 63T595 13T522 -8Q475 -8 439 16T402 82Q402 96 439 199T477 351Q477 401 434 401Q421 401 409 398Q341 388 285 305L278 295L247 170Q216 46 214 40Q206 22 187 7T143 -8T104 7T90 39Q90 47 108 124T146 274L164 347Q166 355 166 372Q166 401 149 401Q129 401 115 379T89 306Q84 288 80 285T55 282H44Q24 282 24 296",
      119952: "254 -8Q191 -8 146 9T80 54T49 111T39 169Q39 206 53 247T96 329T176 402T292 446Q317 451 336 451L344 452Q353 452 359 452Q457 452 516 408T576 279Q576 169 488 81T254 -8ZM349 400Q321 400 287 385T231 338Q206 301 188 228T170 126Q170 99 178 83Q198 44 260 44Q367 44 409 157Q419 185 432 238T445 317Q445 336 443 348Q435 372 416 384T384 398T349 400",
      119953: "24 296Q25 302 27 312T41 350T65 397T103 435T157 452Q235 452 273 404Q336 452 409 452Q434 452 458 448T507 432T550 402T581 354T593 285Q593 221 564 159T480 53Q401 -8 302 -8Q290 -8 279 -7T259 -3T242 3T228 9T218 14T212 18L209 20Q208 19 190 -55T171 -131T198 -132H213Q240 -132 240 -150Q237 -187 223 -192Q219 -194 212 -194Q208 -194 176 -193T95 -192Q48 -192 24 -193T-3 -194Q-11 -194 -16 -190T-22 -182T-23 -176Q-20 -142 -7 -134Q-3 -132 20 -132H44L164 354Q165 357 165 372Q165 401 148 401Q113 401 90 310Q85 289 82 286T60 282H55H44Q24 282 24 296ZM465 339Q465 373 447 387T403 401Q375 401 347 387T303 360T288 341Q288 338 257 216L227 93Q248 43 306 43Q332 43 361 59T410 115Q425 147 445 224Q465 309 465 339",
      119954: "38 159Q38 209 58 260T113 355T205 425T327 452Q338 452 348 451T366 449T382 444T394 440T405 434T414 429T422 423T429 418Q440 429 481 440T533 452Q540 452 545 447T550 437Q550 432 481 152Q410 -130 410 -131T437 -132H452Q479 -132 479 -150Q476 -187 462 -192Q458 -194 451 -194Q447 -194 414 -193T330 -192Q277 -192 249 -193T217 -194Q202 -194 197 -179Q197 -175 201 -159Q206 -139 211 -136T243 -132H283L319 15L307 10Q295 4 270 -2T220 -8Q134 -8 86 37T38 159ZM402 353Q402 358 395 368T369 390T324 401Q301 401 282 394T249 369T226 338T208 297T196 258T186 218Q166 141 166 107Q166 44 229 44Q265 44 294 61T337 95Q341 100 371 222T402 353",
      119955: "24 296Q24 298 28 313T42 352T67 398T104 436T154 452Q198 452 230 437T273 404L282 411Q334 452 393 452Q441 452 470 423T500 350Q500 301 473 279T418 256Q395 256 379 270T363 308Q363 366 424 386Q424 388 420 391T405 398T385 401Q363 401 343 390Q321 380 289 341L252 192Q214 42 212 39Q190 -8 142 -8Q117 -8 103 7T89 39Q89 52 127 202T165 369Q165 402 148 402Q139 402 128 393T110 369Q100 348 90 310Q85 289 82 286T60 282H55H44Q24 282 24 296",
      119956: "140 290Q140 311 148 336T176 388T235 433T326 451H336Q355 451 373 449T418 439T460 412T476 363Q476 333 456 311T406 289Q384 289 371 302T357 335Q357 351 364 364T379 384L386 391Q386 392 381 394T362 398T330 400Q299 400 278 393T247 373T235 352T232 335Q232 322 242 312Q258 299 315 290T403 260Q457 224 457 167Q457 152 453 134T435 90T397 43T330 7T229 -8Q133 -8 95 22T57 91Q57 127 81 152T139 177Q161 177 177 164T194 121Q189 80 153 56Q179 43 236 43Q275 43 303 52T343 75T361 101T366 124Q366 148 338 161T272 180T232 186Q187 198 164 227T140 290",
      119957: "82 382H55Q21 382 21 399Q21 422 33 439Q39 444 93 444H144L162 517Q163 522 167 541T174 567T181 589T192 611T206 627T226 639T253 643Q276 643 291 630T306 594Q306 586 288 514Q284 499 280 481T273 454L271 445Q271 444 317 444Q322 444 331 444T345 445Q377 445 377 428Q377 408 368 390Q360 382 341 382H306H255Q182 86 182 75Q182 43 201 43H204Q242 46 279 81Q298 101 309 119T323 145T330 157T350 162H356H363Q377 162 382 160T387 146Q385 136 372 114T337 64T275 14T192 -7Q131 -7 95 19T59 90V105L128 381Q128 382 82 382",
      119958: "189 389Q189 397 187 399T176 401Q150 399 126 372T89 304Q84 288 81 285T61 282H55H44Q24 282 24 296Q24 307 35 331T65 383T117 431T187 452Q237 452 272 427T308 362Q308 347 273 254T238 111Q238 43 291 43Q319 43 344 58T380 86T391 103T426 247T464 396Q472 414 491 429T535 444T574 429T588 397Q588 390 570 315T534 168L516 97Q514 89 514 72Q514 42 531 42Q544 42 556 56Q574 76 589 134Q594 155 598 158T619 162H624H635Q655 162 655 148Q654 142 652 132T638 94T614 47T575 9T520 -8Q509 -8 498 -7T478 -3T461 2T446 8T434 16T424 23T416 29T410 35T406 39L405 41L397 34Q347 -7 288 -7H281Q148 -7 122 78Q116 95 116 125V136Q116 174 152 273T189 389",
      119959: "380 367Q380 397 406 425T465 453Q493 453 516 430T540 357Q540 314 524 250T467 115T373 13Q338 -8 292 -8Q218 -8 167 23T116 129Q116 178 152 275T189 388Q189 396 187 398T176 401Q148 398 125 372T89 304Q84 288 81 285T61 282H55H44Q24 282 24 296Q24 306 34 330T64 382T116 431T189 452Q231 452 269 429T308 362Q308 346 273 255T238 114Q238 43 306 43Q336 43 363 65T407 118T437 182T456 239T462 268Q462 290 417 315Q380 335 380 367",
      119960: "636 367Q636 400 664 426T719 453Q748 453 772 431T796 357Q796 321 782 256T727 112T633 6Q604 -8 567 -8Q466 -8 415 43Q414 42 410 38T403 31T396 25T388 18T378 11T367 5T355 0T340 -4T324 -7T306 -8Q249 -8 209 5T151 40T125 84T117 129Q117 176 153 274T190 388Q190 408 158 396Q112 376 90 306Q85 288 81 285T61 282H55H44Q24 282 24 296Q24 305 34 328T63 380T114 430T187 452Q240 452 274 427T309 362Q309 346 275 255T240 117Q240 43 317 43Q325 43 333 45T347 50T359 57T369 66T377 75T383 83T388 90L390 95Q390 99 389 110T387 129Q387 139 391 167Q393 177 419 282T448 396Q456 414 475 429T519 444Q546 444 559 428T572 397Q572 384 542 265T511 114Q511 43 579 43Q608 43 633 66T673 122T699 188T714 244L718 267Q718 291 673 315Q636 335 636 367",
      119961: "74 282H63Q43 282 43 296Q43 298 45 307T56 332T76 365T110 401T159 433Q200 451 233 451H236Q273 451 282 450Q358 437 382 400L392 410Q434 452 483 452Q538 452 568 421T599 346Q599 303 573 280T517 256Q494 256 478 270T462 308Q462 343 488 367Q501 377 520 385Q520 386 516 389T502 396T480 400T462 398Q429 383 415 341Q354 116 354 80T405 44Q449 44 485 74T535 142Q539 156 542 159T562 162H568H579Q599 162 599 148Q599 135 586 111T550 60T485 12T397 -8Q313 -8 266 35L258 44Q215 -7 161 -7H156Q99 -7 71 25T43 95Q43 143 70 165T125 188Q148 188 164 174T180 136Q180 101 154 77Q141 67 122 59Q124 54 136 49T161 43Q183 43 200 61T226 103Q287 328 287 364T236 400Q200 400 164 377T107 302Q103 288 100 285T80 282H74",
      119962: "206 -150Q240 -150 268 -134T314 -95T344 -48T362 -7T367 14Q339 -7 280 -7Q230 -7 195 5T144 39T122 79T115 122Q115 175 152 274T189 388Q189 396 187 398T176 401Q148 398 125 372T89 304Q84 288 81 285T61 282H55H44Q24 282 24 296Q24 306 34 329T64 381T116 431T188 452Q239 452 273 427T308 361Q308 347 273 253T237 109Q237 43 291 43T388 98Q388 99 425 246T463 396Q471 414 490 429T534 444T573 430T587 399Q587 386 537 186T483 -25Q461 -84 410 -126T296 -188Q248 -202 204 -202Q127 -202 96 -175T64 -114Q64 -82 86 -57T144 -31Q169 -31 184 -45T199 -83Q199 -89 198 -94T196 -104T193 -113T189 -120T184 -128T179 -134T173 -141T168 -147Q189 -150 206 -150",
      119963: "66 -8H60Q34 -8 34 5Q34 9 42 22T70 59T115 110Q162 156 255 229T381 332L389 339H381Q348 332 298 329T212 325T172 321Q168 318 151 318H146Q116 318 116 332Q116 334 118 342Q128 374 154 402Q205 452 265 452Q304 452 352 422T426 392Q441 392 462 421T485 451T508 452H518Q539 452 539 438Q539 431 516 401T458 334Q412 289 312 210Q229 146 191 111L183 103H195Q203 103 213 104T240 107T268 110Q301 114 337 116T391 119T428 123T455 134T469 157Q473 166 495 166Q521 166 525 161Q532 156 528 141Q510 81 456 37T337 -7Q297 -7 251 22T177 52Q154 52 134 38T100 8L88 -7Q86 -8 66 -8",
      119964: "76 60Q83 29 135 29Q190 29 264 81Q280 93 280 95T277 99T265 106T248 118Q189 166 189 237Q189 307 245 388Q267 421 299 436Q336 455 386 455Q488 455 550 403L559 395L571 405Q727 544 944 679L957 687L966 701Q968 704 970 707T973 712T975 714T978 716T982 717T989 717H995Q1004 717 1007 717T1013 714T1016 708Q1016 705 829 403L636 92L630 83Q659 93 685 110T728 143L745 158Q758 158 752 138L748 127L740 119Q676 58 605 42L593 22Q590 17 587 12T583 6T579 2T573 0T564 0H530H484Q480 3 480 8Q480 15 489 26T498 39T497 40Q477 40 423 49T327 74L316 78L302 68Q200 -8 121 -8Q85 -8 60 7T35 53T60 113T123 144Q144 144 153 132T162 106Q162 89 143 75T99 61Q90 61 76 65V60ZM904 614L905 615Q901 615 840 573T700 469T581 369L587 359Q600 340 608 315T618 273T622 238T624 216L764 414Q904 612 904 614ZM525 363Q493 405 379 418H375Q342 418 309 378Q251 300 251 234Q251 174 306 137Q318 128 322 131Q323 132 329 139Q351 161 362 180Q430 273 509 348L525 363ZM579 205Q579 245 571 278T556 323T546 337L521 311Q461 248 422 196T362 121L353 111Q427 85 499 79Q517 77 520 77L525 76L549 111Q551 114 556 121T563 131T568 138T573 147T575 157T577 169T578 185T579 205",
      119966: "367 89Q367 84 353 77T334 70Q325 70 312 83T298 120Q298 169 364 233T496 298Q538 298 563 275T588 220V213Q588 132 501 53T306 -26Q251 -26 211 6T170 114Q171 148 181 184T205 248T232 298T255 334T265 349T246 350Q127 350 77 390T26 480Q26 533 71 581T178 656T295 683Q312 683 312 676Q312 674 311 672L302 660Q294 648 292 647L286 646Q280 646 276 646Q197 641 145 583T93 476Q93 387 265 387Q271 387 277 387T287 388T292 388T313 414T373 483T451 562Q618 712 732 727Q733 727 740 727T753 728Q790 727 804 708T819 665Q819 643 810 617T773 553T699 481T572 414T385 361Q381 361 367 341Q247 172 247 86Q247 11 325 11Q404 11 465 95Q518 165 528 208Q529 212 529 220Q529 237 518 249T480 261Q431 261 387 209T343 126Q343 111 355 101T367 89ZM777 658Q777 691 738 691Q704 691 658 662T570 590T491 504T432 432T410 400H411Q416 400 440 405T505 423T589 455T675 506T743 576Q777 627 777 658",
      119967: "251 272Q199 272 168 298T136 374Q136 432 175 496T267 603Q321 645 395 676T552 708Q610 708 655 694T724 659T763 618T784 584L790 569Q792 569 800 572T819 576T840 578Q855 578 855 571Q855 566 846 554T829 541Q824 541 817 539T804 533T799 529Q802 517 802 483Q802 376 742 256T585 53T392 -31Q353 -31 300 -7L289 -2L277 -6Q242 -17 192 -17Q141 -17 113 -13T77 -3T68 14Q68 20 73 28T93 45T131 58Q152 62 197 62Q241 62 272 48L282 44Q308 65 334 93Q380 145 467 266T622 454Q644 476 664 493T694 517T720 534T740 547Q699 671 540 671Q461 671 385 625T276 534Q265 520 251 499T219 438T202 368Q202 309 267 309Q325 309 396 383T491 545Q492 548 493 552Q494 558 496 559T511 564Q513 565 514 565Q523 568 527 568Q534 568 534 560Q534 554 529 537Q507 442 420 357T251 272ZM332 20Q332 19 341 15T368 6T400 2Q425 2 457 13T531 49T614 125T690 248Q750 376 750 476V490L738 479Q698 436 646 366T554 239T455 121T332 20ZM226 20Q226 25 186 25Q181 25 174 24H166Q161 24 158 23H152Q170 21 197 21Q205 21 213 21T223 20H226",
      119970: "206 318L249 320Q249 327 259 352T282 399Q312 447 366 510T465 610Q588 717 661 717Q695 717 716 701T738 653T704 577Q663 522 610 474T512 397T424 346T359 315T333 306Q331 304 327 290T323 267Q323 229 368 229Q402 229 439 254T505 312T571 382T638 435Q642 437 644 437Q658 437 658 409Q655 403 647 399T624 379T595 326Q583 296 564 267T523 179Q504 126 483 91T423 27Q335 -37 231 -37Q191 -37 157 -30T95 -2T67 53Q67 89 94 123T159 158Q180 158 190 146T200 119Q200 102 178 89T136 75Q128 75 120 78T110 81Q105 81 105 62Q105 38 129 24T180 5T238 1H244Q282 1 319 32Q349 55 371 85T436 190L465 239Q413 192 354 192Q300 192 274 217T245 276Q245 284 242 284Q220 280 187 280Q106 280 59 315T12 409Q12 457 43 508T118 597T210 659T295 683Q308 683 308 675Q308 670 299 658T282 646Q266 646 240 633Q182 606 118 511Q76 448 76 400Q76 368 105 343T206 318ZM677 660Q677 680 646 680Q616 680 565 629Q537 601 514 571Q493 544 455 492T389 398T360 355Q366 357 386 367Q463 406 534 473T641 591T677 660",
      119973: "829 148Q845 148 845 143T841 130T823 109T788 83T730 54T644 22Q637 20 634 19T627 16T622 13T618 10T612 3T604 -6Q510 -112 396 -203T201 -312Q190 -314 171 -314H166Q156 -314 147 -312T123 -304T101 -283T92 -245Q92 -203 117 -160T175 -93Q214 -65 280 -41T390 -6T509 22L532 27L547 47Q673 219 673 225L665 228Q657 231 648 235T627 249T606 270T591 300T584 341Q584 389 614 447T686 544Q735 588 799 615T959 667T1108 713Q1118 717 1119 717Q1125 717 1129 705T1133 689Q1133 686 1115 673Q1051 627 1001 579T918 481T867 403T828 328T805 277Q802 271 801 267T798 261T798 259Q849 270 900 317Q956 371 956 421Q956 438 953 457T950 480Q950 481 950 482L951 484Q953 487 966 492T985 498Q989 498 994 473Q1000 441 1000 423Q1000 362 934 302T790 223L775 220L767 207Q757 191 731 158T685 98T662 63Q662 61 690 70T755 99T811 142L816 148H829ZM646 332Q646 308 659 291T685 268L698 263L735 320Q828 465 865 511Q923 582 1003 643L1005 645Q1004 645 924 620Q775 572 716 492Q646 401 646 332ZM184 -278Q233 -278 311 -212T444 -80L498 -15H496Q468 -20 424 -30T293 -70T174 -133Q167 -140 158 -153T138 -190T127 -232Q127 -278 184 -278",
      119974: "521 506Q521 493 549 493Q608 493 660 527Q667 531 690 555L736 604Q750 619 777 646T825 694T848 716T852 717Q857 717 860 711T865 697L866 690Q866 687 812 615Q654 404 654 401Q655 401 656 401T659 402T665 403T680 404Q718 404 734 374Q735 370 743 376Q745 377 752 382Q806 416 852 466T933 563T982 623Q1028 666 1075 686T1155 706Q1165 706 1173 705T1186 703T1194 699T1199 695T1201 692T1202 688V687L1204 677Q1204 667 1191 670Q1189 670 1183 670T1174 671Q1137 671 1086 643T1001 564Q970 517 899 449T749 339L739 333L736 322Q730 295 703 253T652 165T628 77Q628 21 681 21Q692 21 698 23Q751 46 817 134L827 148H844Q861 148 864 146Q869 140 859 127Q818 75 761 29T665 -17Q649 -17 633 -12T598 4T569 40T558 100Q558 128 563 152T585 205T609 245T643 294Q647 300 645 300Q633 297 615 297Q607 297 600 298T589 300T584 301Q581 301 569 284T536 236T488 171T418 97T331 28Q233 -37 155 -37Q104 -37 68 -17T29 44Q29 82 55 120T123 158Q144 158 154 146T164 119Q164 102 143 89T100 75Q92 75 86 76T77 80T72 82Q68 82 68 59Q68 37 85 23T123 5T167 0Q203 0 238 21T311 85T382 174T468 288T563 410Q576 426 588 440T607 462T615 472L605 468Q568 456 533 456H527Q490 456 463 483Q451 496 451 508T468 545T486 581Q486 593 465 593Q448 593 425 568T384 518T364 493Q364 492 347 492Q326 492 326 499T351 537T414 599T477 630Q509 630 528 615T547 576Q547 556 534 533T521 506ZM675 357Q675 368 665 368Q656 368 644 363Q631 355 616 333H628Q640 334 672 341Q675 354 675 357",
      119977: "764 513Q764 482 772 423T780 330Q780 304 778 285T775 256T773 245Q778 252 826 328T932 484T1042 617Q1077 652 1114 678T1173 715T1200 726Q1208 726 1208 717Q1208 711 1206 695L1203 679L1199 675Q1197 675 1187 670T1161 657T1133 639Q1050 583 959 456Q906 381 858 307T779 179T725 83T691 18T679 -6Q677 -8 660 -8H656Q639 -8 639 -1Q639 4 646 17Q685 93 685 173V196Q685 233 681 288T676 380Q676 438 687 487L664 454Q505 230 454 170Q366 64 290 14T163 -36H152Q87 -36 58 -11T29 46Q29 82 55 120T123 158Q144 158 154 146T164 119Q164 102 143 89T100 75Q92 75 86 76T77 80T72 82Q67 82 67 59Q67 37 89 19T167 1Q187 1 197 3Q221 9 246 22T292 52T336 91T375 132T411 174T440 212T463 245T478 266Q779 695 784 698Q786 700 802 700H818Q824 694 824 692T809 668T779 604T764 513",
      119978: "259 -8Q230 -7 205 0T153 24T112 74T96 153Q96 254 174 379T374 599T607 707H621Q732 707 778 661Q805 634 805 598Q805 558 775 517T696 452L684 447V441Q684 378 626 273T484 97Q379 7 288 -7Q279 -8 259 -8ZM760 594Q760 670 608 670Q562 670 493 622T347 472Q174 240 174 131Q174 76 205 53T279 29Q294 29 306 32Q405 60 507 205Q592 325 616 401Q625 426 625 435Q625 436 621 436T603 440T569 449Q524 466 515 475Q513 477 513 481T525 496T541 508L548 504Q555 501 565 497T587 488T609 480T625 476Q627 476 626 481Q626 486 623 494T613 513T589 533T548 541Q495 541 413 454T286 281Q265 241 254 201T240 141T235 120Q234 118 217 111T197 104Q195 104 192 107T189 112Q190 125 193 147T220 231T280 348Q335 428 407 493T539 576Q548 578 563 578Q594 578 617 568T653 546T672 518T681 494T683 482Q683 481 684 481Q690 481 707 495T742 538T760 594",
      119979: "571 345Q571 384 612 418T687 452Q698 452 698 445Q698 436 679 417Q677 415 670 415Q650 412 633 389T615 350Q615 340 621 331T634 319T643 315L663 342Q751 462 817 536Q873 595 896 614L907 625Q843 680 701 680Q594 680 499 632T344 516Q317 486 296 449T267 384Q262 366 262 354Q262 332 276 316T326 299H327Q374 299 426 338Q481 376 537 456T597 598Q597 616 599 617Q601 619 614 624T630 630Q639 630 639 604V587V581Q639 519 597 456Q544 377 462 320T318 262Q278 262 250 282T222 350Q222 418 285 504Q360 597 480 656T702 716Q773 716 825 707T898 688T951 660Q962 670 985 685T1012 700Q1018 700 1022 690T1026 673Q1026 670 1019 664Q988 633 988 631Q988 630 999 618T1020 580T1031 522Q1031 471 1003 419T928 330Q854 275 765 264Q757 262 733 262H714L701 245Q615 121 473 42T218 -37Q159 -37 125 -15T90 46Q90 82 116 120T185 158Q203 158 213 147T224 121Q224 110 219 102Q198 75 159 75Q154 75 149 76T143 77T140 77Q137 72 137 53Q138 37 149 26T177 9T205 2T228 0Q313 0 419 74T602 257L620 281L614 285Q607 289 601 294T587 306T576 323T571 345ZM950 529Q950 576 943 576Q940 576 840 439T741 299H751Q804 300 845 334T924 438Q949 490 950 529",
      119980: "330 387Q330 331 402 331Q463 331 514 371T589 459T613 542Q613 559 608 570T598 588T593 596Q593 601 617 610Q632 617 636 616Q675 585 675 527Q675 464 629 409T516 324T387 294Q271 294 271 394V402Q271 438 292 478Q344 582 457 649T672 717Q765 717 825 675T885 548Q885 433 771 298T498 76Q493 73 491 72T486 69T484 67T485 66Q539 41 607 41Q655 41 703 71T780 139L787 148H804Q806 148 809 148Q826 149 826 140Q826 128 786 91T687 19T589 -16H576Q503 -16 414 20L396 27Q279 -17 192 -17Q130 -17 92 2T54 53Q54 92 107 123T222 155Q303 155 401 106L431 91L441 97Q476 118 527 157Q622 236 711 361T801 573Q801 591 795 607T775 641T732 668T660 679Q592 679 528 644T422 560T355 464T330 387ZM201 20Q232 20 267 27T322 40T342 49Q342 52 315 60T243 77T160 86Q150 88 144 88Q130 88 122 79T112 62L111 53Q111 20 201 20",
      119982: "346 463Q346 419 406 386T576 352H588L613 384L681 476Q767 594 842 651T973 708Q1016 708 1016 661Q1016 621 987 562T894 449Q802 375 696 338L682 334L665 312Q638 279 605 233T547 158T482 97Q418 46 332 5T158 -36Q87 -36 58 -11T29 48Q29 82 55 120T123 158Q144 158 154 146T164 119Q164 102 143 89T100 75Q92 75 86 76T77 80T72 82Q67 82 67 59Q67 28 98 14T166 0Q232 0 320 55T491 226Q530 279 551 305L558 314Q558 315 543 315Q417 321 353 365T289 460Q289 566 488 632Q578 662 660 665H679Q685 660 685 657T676 642Q670 633 666 629L664 628Q663 628 661 628T655 628Q606 628 546 615T426 568T350 486Q346 475 346 463ZM976 653Q976 671 959 671Q938 671 919 661T883 629T858 593T835 554Q832 548 830 545Q802 495 775 455T734 400T721 382L736 388Q876 449 946 568Q948 572 949 573Q976 622 976 653",
      119983: "354 350Q264 350 264 426Q264 442 265 448Q279 514 347 582T503 692T662 735Q719 735 774 714T882 672T983 651Q996 651 996 644Q996 639 977 612T954 581Q953 580 938 580Q909 582 884 587L869 591L870 587Q870 583 849 557T796 491T748 422Q729 391 692 313T620 188Q555 105 454 34T253 -37Q214 -37 181 -30T120 -2T92 53Q92 89 119 123T184 158Q205 158 215 146T225 119Q225 102 203 89T161 75Q153 75 145 78T135 81Q130 81 130 62Q130 39 153 24T204 5T267 0Q311 0 358 29T454 117T539 226T629 358T710 476Q726 496 744 516T778 551T807 577T828 595L836 601L785 623Q743 642 713 651T668 661T626 663Q564 663 509 644T418 596T356 535T317 475T305 431Q305 416 312 408Q323 388 369 388Q429 388 465 411T530 480Q557 526 557 565Q557 573 556 579T555 587T555 590Q555 591 568 600T584 611Q588 612 600 603Q622 581 622 549Q622 516 600 475T536 405Q454 350 354 350",
      119984: "55 377Q55 443 122 523T290 660T478 717Q533 717 575 689T618 594Q618 565 611 538T585 477T552 422T506 355T458 288L357 146Q307 68 307 31Q307 20 318 20Q326 20 331 21Q367 27 411 57T490 128L767 500L861 617H908H939Q953 617 956 616T960 609Q960 605 928 566T816 423T648 198Q587 113 571 86Q540 34 540 21Q540 20 545 20Q580 25 623 55T696 124Q702 132 704 133T722 134H739Q744 130 744 127T735 113Q713 80 655 38T548 -14Q527 -17 524 -17Q475 -17 473 47V63L462 55Q364 -17 302 -17Q235 -17 235 69Q235 88 239 105T248 135T268 171T292 205T328 251T369 304Q376 313 395 338T423 374T450 408T476 445T499 479T519 514T534 546T545 579T548 608Q548 647 522 663T460 680Q355 680 243 591T99 406Q97 399 97 383V377Q97 339 153 339Q169 339 175 340Q215 350 241 373T298 444Q333 496 349 535T367 588T374 603Q402 616 408 616Q416 616 416 608Q416 563 393 492T320 378Q233 302 140 302H132Q75 302 57 353Q55 361 55 377",
      119985: "540 717Q542 717 545 717Q562 717 562 710Q562 708 557 702T541 678T517 632T485 544T448 407Q447 405 443 388T438 366T433 345T427 321T420 299T411 274T400 250T387 223T372 197Q363 181 364 181L388 203Q476 284 527 354T620 490T718 612Q754 647 790 673T835 700Q839 700 842 691T848 672L850 662Q850 657 842 653Q803 630 768 600T699 527T653 467T610 405Q554 322 518 282T368 138Q307 84 273 51T231 9T218 -5L207 -17H175Q169 -11 169 -9Q169 -4 183 10Q227 56 258 120T302 234T330 350T356 445Q357 450 360 458L362 464Q317 434 276 434Q221 437 221 479Q221 498 240 521T259 552Q259 565 235 565Q209 565 174 546T105 482Q103 480 102 478T99 474T97 472T95 470T93 468T90 467T86 467T81 467H75Q56 467 56 475Q56 482 82 511T156 570T253 601Q289 601 311 590T334 557Q334 543 312 517T289 478Q289 471 297 471Q327 471 378 514Q384 519 390 531T412 571T451 632Q482 675 524 717H540",
      119986: "232 504Q232 492 263 492Q304 492 356 515L367 520L379 537Q443 632 515 705L527 717H543H552Q564 717 564 709Q564 705 562 703T554 694T540 677T518 643T488 589T448 504T398 385Q368 309 353 281L349 270L523 472L669 634Q726 695 737 706T757 717H765Q786 717 786 710Q786 704 776 691T738 627T675 497Q630 394 609 354T541 244Q456 120 449 111L447 107L448 108Q630 240 700 327Q734 368 788 463Q821 520 841 550T898 619T980 683Q1010 700 1018 700Q1020 700 1023 697T1026 692Q1026 688 1024 671T1020 652Q1018 650 1007 645T988 635Q940 609 902 565T842 477T781 374T699 272Q653 226 535 133Q423 47 373 -2L358 -17H342Q320 -17 320 -11Q320 -7 328 5T354 46T390 112Q416 161 439 217T488 326T564 453L589 490Q587 490 523 414T352 217T166 14Q138 -16 137 -16Q136 -17 120 -17Q106 -17 103 -16T99 -9Q99 -5 100 -3T106 3T116 14T132 35T154 72T184 129T222 212T270 327Q310 426 337 471L326 467Q278 455 243 455Q209 455 188 462T161 478T155 496Q155 508 176 533T198 576Q198 579 198 581T192 588T178 593Q151 593 100 519Q89 504 86 502T75 499H68Q46 499 46 506Q46 510 63 534T112 587T170 627Q178 629 195 629Q235 629 252 613T270 577Q270 556 251 532T232 504",
      119987: "351 351Q308 351 290 373T272 426Q272 487 329 566T478 688Q538 717 584 717Q635 717 681 696T745 620Q752 598 752 564T745 498L741 485Q742 486 769 516T825 573T889 634T962 689T1027 716Q1035 717 1060 717Q1083 716 1096 714T1120 705T1131 686Q1131 668 1109 647T1055 626Q1035 626 1026 638T1016 663Q1016 667 1020 679H1015Q971 671 886 589T728 413L688 360Q688 359 725 359H750Q762 359 766 357T770 348Q769 345 761 335T750 322Q748 321 704 321H660Q651 311 632 282T589 199T565 107Q565 25 653 20Q684 20 720 44T779 95T826 152T850 183L856 184Q861 184 865 184Q888 184 888 173Q883 163 845 117Q770 37 723 10T638 -17Q584 -14 554 17T523 101Q523 147 534 167L532 166Q530 164 526 160T518 153Q378 15 256 -15Q250 -16 226 -16Q161 -16 132 7T103 59Q103 93 129 125T194 158Q232 158 232 121Q233 118 233 113T221 96T188 77Q182 75 168 75T150 78V70Q150 43 178 32T241 20Q340 20 527 286L552 321H524Q489 321 489 330Q489 333 497 344T509 358Q511 359 545 359H579Q580 362 597 389T631 445T661 514T675 586Q675 637 645 658T572 680Q506 680 444 632T348 527T314 434Q314 388 361 388H364H366Q380 388 389 390T416 408T457 454Q487 497 505 536T526 594L529 613Q531 615 545 622T560 630Q568 630 573 613T578 577Q578 518 532 455Q504 413 453 382T351 351",
      119988: "155 280Q116 280 87 300T57 368Q57 409 87 466T192 589Q269 653 345 684T472 716Q572 716 613 675Q644 644 644 599Q644 585 643 574T637 550T629 529T616 505T600 481T578 450T554 416Q494 330 493 328L480 306Q466 278 466 256Q466 227 492 227H496Q514 227 534 234Q541 237 544 241T571 279L762 559Q777 579 792 595Q818 620 856 646T919 686T946 700Q951 700 955 692T959 677Q959 673 947 665T911 639T866 595Q816 538 749 408T640 225Q574 138 464 61T248 -17Q190 -17 144 1T98 62Q98 81 109 102T131 135Q156 156 183 158Q226 158 226 121Q226 111 224 107Q215 93 196 84T162 74Q154 74 147 77H144V70Q146 41 185 31T263 20Q363 20 493 175L507 192H504Q500 191 498 191Q484 189 476 189Q430 189 405 219T379 287Q379 294 379 299T382 311T385 322T391 335T398 347T408 363T420 379T434 399T450 422Q455 429 469 449T488 475T504 499T520 523T533 543T544 565T552 583T557 603T559 620Q559 680 467 680Q402 680 333 646T213 563T131 462T98 373Q98 343 119 330T169 317Q187 317 212 333Q242 354 291 423T352 555Q354 562 355 588Q355 612 356 612Q357 614 371 622T387 630Q391 630 397 623T409 597T415 556Q415 507 380 448T294 344Q216 280 155 280",
      119989: "221 428Q221 487 280 555T425 670T583 717H587Q641 717 665 695T689 646Q689 625 674 600T658 564Q658 550 671 541T695 530T710 528L718 539Q779 613 821 646Q894 707 964 707H970Q1010 707 1025 675Q1032 661 1032 645Q1032 626 1022 607Q1008 579 980 560T897 522Q867 512 836 505T788 496L771 493Q768 493 760 477T736 429T702 370Q700 367 698 363Q696 360 696 359H805Q809 355 809 350Q809 340 791 322Q789 321 728 321H668Q562 179 433 88L419 78L434 73Q505 54 554 54Q609 54 654 82T720 140H752Q758 134 758 132Q758 128 747 113Q711 67 657 32T552 -14Q540 -16 517 -16T480 -15T439 -3T375 27L354 38L338 30Q257 -8 191 -8H184Q154 -8 133 -5T103 1T88 10T83 19T83 29Q83 35 86 44T100 65T127 88T173 105T241 112Q286 112 342 99L360 95L372 105Q434 157 523 270L560 320Q560 321 533 321L507 322Q502 325 502 330Q502 339 521 358Q523 359 556 359H588L669 474L682 491Q676 492 665 494T647 498T632 503T614 510T596 521Q556 547 556 570Q556 585 579 618T603 663Q603 679 568 679Q510 679 452 650T354 581T288 500T262 431Q262 407 280 397T321 387Q331 387 341 390T360 398T376 409T390 423T400 435T409 447L414 454Q457 514 460 562Q460 575 461 576Q461 577 475 586T492 595Q496 595 503 588T514 572Q520 559 520 539Q520 473 452 412T308 351Q269 351 245 370T221 428ZM989 642Q989 667 953 671Q905 671 871 644Q853 632 832 604T799 554T787 531H788Q801 531 842 539T916 561Q989 592 989 642ZM198 29Q230 29 257 36T295 52L306 59Q306 63 259 73Q251 74 209 74Q177 74 158 66T134 48L130 40Q130 29 198 29",
      120068: "22 505Q22 563 94 624T271 685H280Q416 685 443 560Q447 535 447 504Q444 414 405 330L399 319L229 155Q233 154 241 153T253 150T265 145T281 135T301 119T328 93L357 64L402 92Q438 116 473 137L500 154V339Q500 528 495 593V601L559 649Q621 696 624 696L638 686L629 677Q599 650 593 638Q582 614 581 504Q580 490 580 443Q580 314 584 238Q584 235 584 224T584 210T585 199T586 187T588 176T591 164T595 152T601 137T609 121Q630 77 640 77Q661 77 703 101Q704 95 706 90L707 86V84L636 29Q618 15 601 2T574 -19T564 -25L500 121Q499 121 399 48L299 -26Q298 -26 291 -15T272 11T245 42T209 69T165 80Q120 80 58 43L48 37L40 42L32 48L122 117Q196 173 241 211Q319 280 343 327T368 447Q368 535 317 582Q264 633 199 633Q155 633 122 605T86 542Q86 518 133 467T181 387Q181 348 140 309Q113 281 73 260L64 255L50 265L59 273Q112 307 112 345Q112 363 90 387T45 441T22 505",
      120069: "48 506Q48 568 120 629T268 691Q362 691 425 594L431 585L441 594Q478 628 528 657T629 686Q665 686 687 670Q703 658 718 584T753 506Q756 505 763 505Q778 505 804 512L815 516L820 496Q820 494 808 490T774 476T732 454Q720 445 708 437L675 415L640 394L625 383Q626 382 635 382Q652 382 670 379T712 364T754 336T784 289T797 220Q797 172 776 122Q769 106 766 102T745 84Q654 11 619 -8T538 -27Q483 -27 387 10T249 47Q218 47 186 34T133 8T112 -5T104 7T97 21L196 82Q259 120 284 140Q333 181 351 214Q368 251 368 353Q368 588 228 620Q222 621 205 621Q160 621 139 596Q117 569 117 548Q117 526 162 470T208 387Q208 352 179 320T104 264Q88 256 86 256Q83 256 70 266L82 274Q134 309 134 343Q134 352 130 359Q118 377 100 401T72 439T56 470T48 506ZM453 528Q457 496 457 419L458 357L488 367Q554 390 622 425Q673 449 673 453L671 454Q669 456 665 460T657 473T648 498T639 541Q629 597 616 613Q599 633 567 633Q534 633 493 599Q471 577 457 540L453 528ZM713 176Q713 252 661 295T528 339Q512 339 494 336T466 330T455 325Q454 325 452 311T444 270T425 217L420 207L304 118L319 116Q381 111 475 74T602 37Q655 37 684 79T713 176",
      120071: "346 611Q300 611 261 604T194 584T144 555T107 519T82 481T65 443T55 410T50 383T48 367L37 372L27 378V393Q33 524 115 603Q160 648 230 666T365 685Q423 685 434 684Q745 653 745 405Q745 281 694 151L687 133L657 103Q524 -27 470 -27Q456 -27 331 18T157 64Q122 64 84 28Q67 9 57 -6L50 -16L43 -8L35 0Q35 3 46 22T83 74T141 133H154Q208 136 253 161T299 230Q299 259 277 290T232 348T209 392Q209 434 254 479T358 556L368 561L382 550Q382 549 381 548T377 545T371 541Q353 527 326 496T298 444Q298 430 343 365T389 277Q389 240 353 202T267 136L252 127Q304 118 400 83T513 47Q548 47 582 79T635 145Q671 208 671 323Q671 390 651 441T599 524T523 576T436 603T346 611",
      120072: "301 564Q240 564 203 507T165 361Q165 280 190 218T260 119T358 64T474 46Q491 46 499 47L511 48L628 106V80L499 -5Q498 -6 497 -7T494 -10T490 -12T485 -15T478 -17T468 -19T456 -21T439 -22T418 -23T392 -24Q252 -20 169 67T86 315Q86 409 124 491Q128 500 139 508Q211 560 272 591L293 601L325 585L338 594Q374 617 422 639T502 674L532 685Q533 685 540 663T557 614T574 581Q580 575 591 575Q600 575 616 580T634 585V564L623 559Q572 535 532 529L522 527Q491 547 442 627Q442 628 440 627T435 624T428 620Q410 610 405 597T399 559V553Q399 534 409 487Q419 431 427 369L476 400Q526 431 526 432Q531 426 537 420Q565 382 602 359Q611 353 610 352Q609 351 575 332T537 312Q536 312 505 335T463 358Q451 358 439 352T425 337Q394 243 263 223L251 221L243 229L235 236L245 239Q274 247 303 266Q320 277 325 290T330 336Q330 394 315 451T300 531Q300 549 303 561Q303 564 301 564",
      120073: "198 617Q177 617 159 609T128 587T106 557T91 524T82 494T76 472L75 463Q74 463 67 468T59 474Q59 488 63 510T80 564T114 622T170 667T253 686Q272 686 295 680Q330 672 384 645T468 598Q498 580 524 580Q549 580 595 626L612 643V616L599 599Q522 500 482 500Q466 500 435 514L420 521Q388 516 349 486T309 415Q309 396 323 366T352 315T367 294Q367 293 375 301T398 322T429 352L490 410Q537 355 542 332Q542 330 531 315T510 286L499 273Q479 313 449 313Q437 313 419 303T390 282L378 271L394 250Q444 183 470 129Q484 96 484 71Q484 -19 402 -86T223 -153H220Q175 -153 137 -120Q103 -89 103 -8Q103 12 106 40T109 96Q109 132 103 146T72 161Q46 161 16 143L14 154L11 165Q88 219 143 219Q163 219 177 197T192 127Q192 102 186 65T179 4Q179 -60 210 -88T272 -117Q319 -117 362 -75T405 33Q405 72 363 139T278 271T235 368Q235 385 243 404T264 439T292 472T324 500T353 522T374 538L382 543Q382 544 351 562T274 598T198 617",
      120074: "373 560Q373 543 378 509T388 448T394 421Q396 421 416 433T474 474T548 537L538 543Q481 570 452 628L487 659Q521 690 522 690L526 684Q529 677 535 667T551 646T575 625T607 608Q617 606 621 606Q644 606 660 622L664 600L497 455Q495 453 527 453H546Q628 453 665 407L666 406Q710 357 710 297Q710 196 631 109L621 98L577 69Q471 -7 452 -15Q413 -26 370 -26Q239 -26 155 59Q66 148 66 319Q66 442 113 505L191 552Q269 600 270 600H271Q273 600 280 596T293 589T299 586Q300 586 357 623Q413 659 415 659L426 649Q373 604 373 560ZM219 228Q305 262 305 329V343Q305 384 294 434T282 519Q282 562 287 567Q287 569 286 569Q222 556 191 520Q144 464 144 362Q144 213 228 128T420 42Q514 42 568 103T622 255V260Q622 320 578 357Q526 400 453 400Q434 400 410 396L399 394L400 378V372Q400 330 388 301T348 254T305 232T252 217Q245 215 242 214L234 213L226 220L219 228",
      120077: "194 601Q175 601 155 590T121 564T95 533T76 506L69 495Q55 503 55 507Q55 509 60 521Q87 586 132 636T222 686Q251 686 289 660T363 607T422 580Q442 580 466 588T506 604L522 612V591L487 574Q371 517 359 517H357Q332 522 280 561T194 601ZM383 39Q383 96 356 203T329 361Q329 364 329 370T330 378L331 388L339 399Q362 429 394 462T446 513T469 531Q470 531 477 527T484 521L474 509Q411 434 411 379Q411 341 434 223T458 82Q458 1 390 -69T239 -139Q191 -139 162 -123T123 -83T108 -30T101 23T88 64T52 80Q27 80 -4 64L-10 80Q-5 85 13 98T63 128T118 144Q149 144 163 129T179 93T182 38T187 -19Q199 -96 278 -96Q294 -96 309 -91T343 -73T372 -31T383 39",
      120078: "154 37Q219 41 219 142Q219 203 190 302T160 434Q160 445 172 472T209 534T269 602T354 657T460 680H472Q576 680 618 617V599Q618 578 615 555T608 517T602 501Q596 495 587 495Q586 495 586 499Q586 500 586 505T585 513Q567 628 415 628Q384 628 356 621T302 596T260 546T244 466Q244 416 258 377L261 367L276 382Q294 401 317 422T386 471T468 500Q500 500 518 477T544 422T554 386Q555 382 560 376T576 364T601 357Q612 357 623 361Q623 360 621 351T617 340L541 305L465 272Q469 271 476 270T488 266T502 260L512 255L519 242Q533 211 546 157T564 93Q584 53 617 53Q649 53 682 74V54L675 46Q646 17 612 -5T554 -27Q546 -27 538 -23T517 -9T494 28T478 91Q465 172 448 200T378 228Q337 228 289 207L288 196Q288 160 284 138Q283 125 262 98Q185 1 112 -24L102 -27L91 -25Q47 -13 17 31Q17 33 49 69L80 106Q111 37 154 37ZM421 434Q411 436 405 436Q370 436 336 407T275 337L267 325L271 313Q288 257 288 234Q289 234 395 276T502 319Q501 320 499 323T493 330T485 345T475 371Q473 378 468 388T449 416T421 434",
      120079: "251 231Q251 254 206 341T160 468Q160 529 238 605Q313 677 425 685L432 686Q440 686 445 686Q520 686 545 657T571 576Q571 548 568 517T564 480Q564 466 572 460T608 452H623V431L619 430Q618 430 611 428T591 423T566 417L516 404Q497 415 491 419T480 433T475 461Q475 470 477 498T480 546Q480 561 480 569T474 592T459 615T429 631T379 638Q322 638 283 606T239 526Q237 480 280 394Q319 310 320 268V250Q308 235 244 169L203 128H210Q215 128 256 112T357 80T466 64Q526 64 564 88T625 186L629 196Q643 187 644 186Q618 70 604 41Q577 -15 513 -24Q496 -26 476 -26Q418 -26 344 -3T211 45T133 69Q115 69 97 50T65 7T49 -20L45 -16Q41 -11 37 -6T33 0Q33 8 66 69T111 134Q251 156 251 231",
      120080: "687 578Q690 586 719 612T794 665T877 692Q909 692 926 670T953 614T973 570Q983 561 1000 560Q1023 560 1048 569V553L1034 546Q1003 528 972 504T933 466Q905 403 905 264Q905 205 909 172T920 126T940 91Q951 74 959 74T997 87L1027 100V79L956 27Q940 15 922 2T894 -19L885 -26Q883 -26 854 27L825 80V192Q826 314 830 341Q831 345 833 356Q840 398 844 404Q871 444 920 489L917 491Q914 493 911 495T903 501T898 508Q893 516 877 562T852 616Q840 628 818 628Q751 628 702 541L697 532L700 515Q708 467 708 419Q706 348 690 276T662 169T643 126L634 116Q626 105 611 87T581 51L522 -22L514 -15Q470 21 452 21Q431 21 394 -16L384 5L386 9L426 60Q449 87 460 95T486 104Q487 104 491 104T497 103Q514 99 541 81L559 69Q595 96 606 169T618 350Q618 486 598 543T517 601Q484 601 458 570T432 523Q432 516 434 492T436 443Q436 250 369 133L363 122L280 50Q207 -16 192 -27L182 -18Q136 26 106 26Q83 26 62 9T37 -16L27 0L33 10Q41 22 54 39T94 81T143 106Q178 106 225 62L235 53Q283 82 307 117Q352 190 352 359Q352 408 345 451T320 534T267 597T183 621Q159 621 147 617T120 598Q97 574 96 556Q96 528 143 469T191 377T161 311T102 262T70 245Q69 245 59 252T49 260L56 264Q63 268 73 275T94 291T111 312T118 338Q118 355 95 384T50 446T27 506Q31 567 101 626T255 686Q297 686 330 671T382 632T409 595T421 572V571L430 580Q454 610 496 634T588 659H590Q655 659 683 585L687 578",
      120081: "112 334Q112 356 70 410T27 497Q27 553 94 619T229 685Q230 685 236 685T246 686Q303 686 349 654Q373 636 392 607T419 558L426 538L454 576Q474 604 486 618T520 651T569 678T633 686Q668 684 687 673T713 651T730 609Q730 608 732 600T736 588T741 578T747 568T754 561T765 555T779 553Q789 553 817 562Q819 557 819 555V547L790 526Q743 492 730 479T712 447Q697 369 697 281Q697 166 726 108Q741 76 755 68Q759 66 767 66Q789 66 825 93V82Q825 71 822 70Q821 69 763 27T701 -18L692 -25L668 15Q662 25 650 45T635 70L627 85V107Q627 122 626 162T624 285Q624 381 632 398Q638 409 651 425T675 454T696 477T707 489H696Q683 490 679 492T669 507T653 551Q642 588 627 608T584 628Q572 628 560 625T538 616T519 602T502 586T486 568T473 549T463 532T454 517T448 504L445 497Q437 480 437 474Q437 472 439 461T444 421T446 348Q446 205 405 124Q396 105 392 100T368 78Q312 32 278 9T235 -18T214 -22Q191 -22 170 -10T139 12T129 25T160 66T192 105Q193 102 194 98T200 83T213 64T233 49T261 42Q303 42 339 90Q373 134 373 268Q373 397 339 493T235 618Q215 628 191 628Q155 628 126 604T97 548Q97 524 120 493T168 431T192 381Q192 346 164 318T86 260L70 250L54 266L63 272Q112 300 112 334",
      120082: "428 596Q412 596 386 595T350 593Q289 593 270 625Q267 632 267 640Q267 656 280 672T312 705T336 729H343Q351 729 351 728Q342 710 342 703Q342 683 382 676T493 662T604 643Q648 627 677 599T720 535T739 466T744 392Q744 317 732 260T696 166T659 116T621 83Q617 80 615 78Q485 -27 377 -27Q320 -25 272 -9T187 21T122 36H116Q65 36 29 -13L27 -15L12 0Q12 1 32 26T76 78T109 109Q145 123 166 150T187 207Q187 244 134 318T80 412Q80 454 112 498T176 566T213 590Q216 590 224 585L234 580L225 573Q216 566 207 557T188 536T172 511T165 484Q165 448 213 368T261 259Q261 241 252 219T228 179T200 146T176 122L167 112Q170 111 174 111Q188 110 233 91T339 55T453 37Q508 37 556 68T626 152Q655 219 655 328Q655 543 532 582Q484 596 428 596",
      120083: "112 339Q112 354 91 380T49 438T28 497Q28 565 95 628T242 692Q261 692 277 689T307 682T331 670T351 655T367 637T379 619T388 600T395 582T401 565T405 550Q409 554 422 570T453 603T500 641Q573 692 637 692Q656 692 670 686T692 672T705 647T713 618T718 584Q720 568 721 562T728 546T742 534T768 530Q776 531 782 532T791 535T796 536Q799 536 804 521Q801 519 789 513T764 499T738 480Q697 447 680 414Q677 407 677 396Q677 370 713 312T750 210Q750 125 686 57T560 -11Q540 -11 475 13L410 37V31Q410 -9 412 -50T417 -118T420 -150Q419 -150 373 -184T326 -218L305 -208Q305 -207 307 -196T314 -165T322 -116T328 -46T331 43V63L318 66Q270 80 250 80Q233 80 213 70Q183 57 138 -3L128 -16L118 5L125 20Q193 154 282 154Q309 154 331 146V287Q331 444 327 469Q321 522 301 560Q284 590 251 611T184 633Q146 633 119 607T92 550Q92 539 94 534Q100 516 143 460T186 386Q186 366 170 336T119 281Q102 264 70 250L49 260L56 266Q64 271 72 278T90 296T106 317T112 339ZM602 345Q602 357 608 371T622 397T642 421T661 441T678 456L686 462Q663 473 652 486T639 512T634 553Q631 594 624 608T593 631Q587 632 567 632Q539 632 497 600T416 497L410 484V122L467 103Q481 99 502 92T533 82T557 75T578 69T594 66T610 64Q647 64 672 87T697 144Q697 180 650 250T602 345",
      120084: "428 596Q412 596 386 595T350 593Q313 593 291 605T268 638Q268 644 269 648T274 658T284 669T301 689T326 718L336 729H343Q351 729 351 728Q342 710 342 703Q342 683 382 676T493 662T604 643Q744 592 744 398Q744 299 708 213T646 104L603 68L614 55Q670 -5 710 -5Q726 -5 744 1T772 14L781 20Q782 20 782 7V-6L771 -13Q673 -69 665 -69L647 -63Q552 -30 514 8H512Q509 8 500 3T471 -9T428 -23Q405 -27 377 -27Q305 -24 228 6T124 36Q69 36 27 -16Q23 -13 19 -8L11 0L27 20Q93 102 109 109Q145 123 166 150T187 207Q187 244 134 318T80 412Q80 454 112 498T176 566T213 590Q216 590 224 585L234 580L225 573Q216 566 207 557T188 536T172 511T165 484Q165 448 213 368T261 259Q261 241 252 219T228 179T200 146T176 122L167 112Q170 111 174 111Q188 110 233 91T339 55T453 37Q508 37 556 68T626 152Q655 219 655 328Q655 543 532 582Q484 596 428 596",
      120086: "750 276Q750 141 645 57T378 -27Q224 -27 146 67Q66 163 66 307Q66 361 80 412T127 515T217 609T356 676L388 684L420 692L442 681Q545 625 586 608T654 591Q688 591 710 609T737 646L742 665Q742 665 756 654L752 642Q736 594 706 566T621 515L607 510Q580 513 528 542T421 599T335 627Q272 627 214 569Q139 500 139 373Q139 308 159 247T217 136T316 56T455 26Q520 26 566 43T634 90T666 148T676 207Q676 264 649 297T580 338Q574 339 552 339Q516 339 442 325T327 310H324Q303 310 290 312T259 328T232 369Q230 375 230 390Q230 426 259 461Q286 488 338 507Q351 498 351 495L346 492Q342 490 339 488T330 482T320 473T312 462T305 447T303 428Q303 368 366 368Q393 368 487 388T613 409Q656 409 696 385T748 306Q750 294 750 276",
      120087: "289 686Q338 686 439 668T565 649Q606 649 641 672Q656 683 664 690T674 700L675 701Q676 698 676 692V681L641 636Q608 591 602 588Q577 575 528 575Q490 575 438 583L426 585L416 578Q335 520 335 465Q335 437 393 366T452 259Q452 226 406 171Q387 146 351 115L341 106L358 97Q438 58 475 58Q525 58 598 124L608 133Q617 119 617 118Q617 114 606 103Q513 0 448 -27L438 -25Q401 -19 337 14T234 48Q186 48 122 -3L112 -11L106 -3L101 5L110 15Q160 63 209 105L224 117H310Q384 163 384 210Q384 238 329 303T263 398Q262 400 262 404Q262 427 290 476T368 580Q383 596 381 596Q315 622 228 622Q159 622 118 593T76 516Q76 482 96 461T139 431L107 399L75 367Q34 403 34 472Q34 525 64 572T148 652Q207 686 289 686",
      120088: "160 247Q160 291 110 382T59 505Q59 542 106 597T207 690L218 697L229 682Q229 680 216 669T187 644T158 604T144 552Q144 526 165 482T207 387T228 291Q228 277 228 267T224 247T219 232T210 217T199 202T184 184T166 163L118 107Q124 105 132 105T160 98T220 77Q298 47 317 47Q354 47 409 105Q444 142 451 161T458 244V277Q458 339 455 457T447 596L575 686L591 676L583 668Q550 632 541 587T532 449V372V298Q532 198 550 147Q572 79 605 79Q617 79 631 84T656 95L665 100V79L595 27Q578 15 560 2T532 -19L523 -26Q522 -26 490 42L458 111L397 50Q389 42 374 27T355 8T340 -5T325 -16T311 -22T294 -26T274 -27Q232 -25 152 6T35 37Q28 36 19 30T4 18T-8 4T-16 -5Q-25 10 -25 11T-20 18Q-5 34 15 57L54 101Q70 103 86 116Q94 121 113 140T137 166Q160 201 160 247",
      120089: "54 266Q118 307 118 339Q118 360 74 413T27 493Q26 530 59 578T133 651Q187 686 256 686Q299 686 332 673T382 640T409 600T424 567T429 553Q431 553 463 586T542 653T624 686Q663 686 715 652T798 617Q812 617 825 622V606L815 599Q785 576 762 546Q724 495 724 432Q724 397 737 317T750 217Q750 148 711 96Q697 75 639 34T578 -10Q563 -21 555 -23T522 -26Q458 -26 363 29Q253 89 211 89Q192 89 173 83T139 67T114 49T96 34L90 27L80 43L95 59Q121 87 148 110T191 143T213 154Q261 154 300 195Q328 222 340 267T352 406Q352 562 255 611Q222 627 187 627H182Q143 627 115 598Q96 579 96 559Q96 528 144 470T192 381Q192 348 156 314T80 256L70 250L54 266ZM727 548Q701 550 650 583T567 617Q560 617 554 615Q532 610 501 580T445 501L437 485V465Q432 326 378 223L294 150Q321 143 421 87T560 31Q608 31 634 64Q660 102 660 198Q660 226 655 292T650 387Q650 418 658 452L660 462L735 548H727ZM264 150Q265 151 263 151H261Q261 150 264 150ZM280 150H276V149Q280 149 280 150",
      120090: "32 501Q35 564 103 625T264 686Q326 686 365 652Q402 618 420 564L441 585Q486 630 539 670L559 686L572 672Q637 606 665 559L682 576Q787 686 845 686Q877 686 929 656T992 623Q999 621 1016 621Q1041 621 1054 628L1051 617L1047 606Q1042 602 1037 597Q983 557 969 522T954 434Q954 394 961 336T968 221Q968 201 966 184T963 155T956 131T948 113T936 96T923 82T906 66T887 50L816 -2Q815 -3 810 -6T801 -11T791 -16T777 -20T759 -23T735 -25Q691 -25 619 4T505 37H496L463 21Q420 0 389 -20L378 -27H362Q316 -27 275 -9T202 28T150 46H146Q114 46 59 -5L48 -16L41 -6Q40 -4 38 -2T34 2L33 4Q33 8 77 54Q111 87 142 112L154 122H166Q239 127 288 182Q346 244 346 406Q346 489 326 537T259 610Q227 628 195 628Q155 628 128 606T101 549Q101 517 146 465T192 382Q192 348 156 310T85 254L75 250L64 258L55 266Q56 267 74 276T105 301T118 339Q118 362 75 413T32 501ZM952 547Q927 555 877 586T796 617Q744 617 682 525L676 516L677 498Q678 486 678 425Q678 301 652 206Q649 195 613 151T577 102Q577 100 582 100Q618 100 720 51Q761 32 790 32H794Q843 32 869 83Q884 110 884 189Q884 233 879 294T873 400Q874 433 879 451T889 478T915 507T952 547ZM241 105Q249 98 263 88T317 60T393 42Q478 42 547 109Q607 171 607 358Q607 371 607 380T604 417T595 467T577 517T546 566T500 601L479 580Q449 550 433 526L426 516V503Q426 311 357 200L352 191L296 149Q241 107 241 105",
      120091: "96 511L78 527L149 603Q207 666 227 686L237 685Q278 680 306 667T351 631T376 588T395 533L400 516L406 527Q439 600 523 653Q538 664 587 688Q589 688 596 672T613 635T629 606Q643 588 665 588Q677 588 693 596L709 603V585L607 526Q583 536 554 564T512 614Q461 571 435 507T405 381V367H581L563 347Q561 344 558 341T553 335T549 330T546 326L545 325H541Q537 325 488 329T411 334H405V325Q405 198 454 123Q497 54 568 54Q594 54 619 64T660 84L676 95V74L565 -16L553 -20Q528 -27 512 -27Q492 -27 475 -21T444 -5T418 19T398 47T382 77T371 105T363 128T358 145L357 151Q354 151 339 136T294 91T232 37Q152 -22 113 -22Q90 -22 69 -9T38 17T28 32Q28 33 58 68L86 101Q94 81 115 64T165 47Q206 47 249 84Q302 128 325 222Q334 263 336 312V334H312Q198 334 163 324H161Q162 326 182 348L198 367H336V398Q333 508 308 550Q295 570 274 585T227 601Q204 601 181 589T145 565T115 533T96 511",
      120092: "123 345Q123 358 75 415T27 496Q27 533 63 578T121 639Q181 685 256 685Q305 685 332 676T384 635Q419 588 425 542Q468 585 526 628T618 686Q632 667 658 645T704 609T724 594Q740 512 740 385V375V361Q740 128 682 -3Q656 -64 554 -141T361 -218Q319 -218 287 -203T243 -173T230 -150Q230 -148 267 -109L304 -69Q311 -115 341 -142T411 -170Q474 -170 534 -119T624 11Q660 111 660 312Q660 447 633 528L612 549Q578 583 552 596L543 601L528 592Q501 574 483 558Q454 528 436 494L431 484V473Q431 330 376 230Q366 213 359 206T306 159L250 112Q278 105 327 82T399 58Q421 58 440 69T471 92T484 104H485L493 90L480 73Q453 39 429 13Q393 -22 372 -22Q342 -22 260 15T153 53Q97 53 48 -16L38 5Q46 22 74 54T128 108L143 122H155Q226 127 275 168Q352 236 352 399Q352 501 309 564T197 628Q156 628 129 607T102 553Q102 517 147 463T192 383Q192 323 75 250L59 266Q123 314 123 345",
      120094: "86 363Q86 365 128 391T212 444L255 470Q256 470 263 465T283 453T308 442Q338 431 346 431Q350 431 358 436L414 465L421 459L418 434Q404 339 404 262Q404 147 408 119L425 97Q443 74 444 74L467 86L491 97Q492 97 497 83L436 28Q377 -26 374 -26L331 46L329 91L155 -35Q152 -35 114 10T68 65L67 86Q66 100 66 156Q66 177 66 198T68 236T71 268T74 296T77 320T80 338T83 351T86 360V363ZM314 365Q285 367 255 374T206 386L187 392Q181 392 172 376T154 311T145 197Q145 124 151 118Q156 110 187 78L208 56H214Q221 57 253 76L308 109L329 123V136Q328 143 328 257V365H314",
      120095: "177 427Q177 364 181 364Q184 364 192 370T223 395T271 433Q317 469 323 469Q325 469 338 462T377 440T432 413L440 409L441 396Q441 394 441 372T442 334Q442 203 425 108L423 97L400 83Q347 49 296 21T222 -19T196 -31Q192 -29 149 12T87 71L89 89Q100 155 100 319Q100 500 94 627Q94 632 94 638T94 648T94 656T93 662V664Q93 668 97 669T106 670H110Q114 653 118 633L127 596Q127 595 132 597Q136 599 195 642L255 685L272 673Q269 670 256 659T233 637T211 609T190 565T179 508Q177 494 177 427ZM295 377L274 385Q273 385 264 381T242 370T223 361Q213 355 188 340L178 333V123L198 103Q259 42 285 42Q299 42 314 55T339 85Q363 132 363 232Q363 310 358 343V349L345 355Q330 363 295 377",
      120096: "72 208Q72 323 84 361Q84 363 167 414Q248 466 250 466Q255 465 260 465Q283 460 307 450T344 430L357 422L306 343L301 342L296 340Q267 362 247 372T216 384T177 390Q154 349 154 238Q154 220 156 158V145L168 132Q179 121 208 93T244 59Q245 58 246 58Q255 58 302 83T350 108L352 104Q355 100 356 96L358 92Q358 91 274 32T187 -29L177 -20Q160 -6 120 36T77 84Q77 87 75 118T72 208",
      120097: "13 542Q13 544 77 576T147 609Q154 609 185 598T283 551T414 464L427 454V440Q427 436 427 412T428 372Q428 315 426 270T418 197T409 152T400 121T394 103L304 35Q285 21 261 2T225 -25L214 -33Q149 7 97 59L76 80L75 91V171Q75 221 76 254T79 299T80 313Q80 315 78 321Q78 323 78 326L77 330Q79 336 132 384T211 447L219 445Q221 445 224 444L228 443Q229 443 228 441T221 432T206 415L191 395Q175 378 162 339Q152 306 152 250Q152 217 159 140V134L171 121Q194 99 235 74T284 48Q296 48 310 75T337 156T349 267Q349 346 336 400L315 420Q246 489 176 525Q127 545 94 545H73L27 527L13 542",
      120098: "353 103Q353 97 358 87L337 71Q293 38 247 6Q191 -30 189 -30Q188 -30 173 -16T130 26T76 85L74 99Q70 139 70 190Q70 228 73 271T79 338T84 365L94 372Q105 378 126 392T166 417L246 467Q283 417 349 369L364 358L355 349Q249 249 169 180L151 166L152 152V139L165 126Q186 105 215 84T260 62Q267 62 309 82L353 103ZM267 323Q246 337 230 350T204 371T189 385T179 394T174 397Q149 381 149 219V211Q151 211 171 223T220 258T268 299L282 313L267 323",
      120099: "128 387Q128 391 112 456T92 546V555L101 568Q133 617 183 670L193 681L204 673Q238 648 262 648Q276 648 292 656L302 661L308 653Q308 652 278 618L249 585H234Q194 587 159 595Q158 595 156 596H153L150 597Q151 593 152 581T154 564T157 547T164 524T176 494Q199 436 203 400V392H207L323 386Q323 384 309 368L296 351H203V176Q201 -9 198 -32Q194 -61 166 -114Q158 -129 153 -138Q114 -214 110 -221Q105 -221 98 -220L87 -219V-216Q88 -215 106 -150T124 -82Q128 -73 129 155V351H30L64 386H96Q128 386 128 387",
      120100: "27 -144L17 -131L82 -75L146 -20L126 6Q99 37 92 48L78 65L77 79Q75 103 75 158Q75 217 78 268T86 343T91 368Q92 370 109 382T157 413T222 452Q240 465 249 469L251 470Q296 443 350 428L365 424L437 465L442 459Q442 456 441 453T437 444T432 424T427 385T423 317T421 215V133L423 119Q423 111 455 -17Q455 -21 380 -108T290 -201Q290 -202 284 -204T269 -207T252 -209Q172 -209 75 -135L62 -126Q60 -126 44 -135L27 -144ZM349 351Q267 359 178 389Q177 387 173 376T166 358T161 337T157 303T155 259Q155 195 159 135L160 124L170 112Q180 101 204 77T230 53Q231 53 286 88L340 123V141Q340 235 348 329Q348 334 348 339T348 348L349 351ZM364 -59Q364 -37 358 -1T347 64T341 93Q336 93 252 28T149 -57Q226 -134 293 -134Q325 -134 344 -118T364 -69V-59",
      120101: "88 427Q88 486 87 545T84 634T81 667Q81 673 82 673T89 676H94L117 600L200 661Q208 667 218 674T233 684L238 688L254 678Q222 646 216 637Q164 572 164 483V442V371Q187 390 244 431T312 478Q344 446 411 423L428 417L429 396Q434 297 434 218Q430 8 406 -34Q346 -111 262 -195L251 -205L249 -203Q248 -203 244 -199T236 -193Q226 -183 227 -182Q228 -182 244 -169T278 -140T301 -121Q330 -89 340 -22Q355 60 355 171Q355 200 353 244T348 321T344 354Q333 363 307 376T260 390Q241 390 179 350L164 341V121Q180 96 201 66L213 50Q210 46 172 11T131 -24Q130 -24 123 -11T102 25T77 65V78Q80 104 84 227T88 427",
      120102: "72 617Q72 618 102 645T133 673H134Q134 668 175 627L187 615L130 555L116 568Q72 615 72 617ZM27 369Q21 376 14 382L26 392Q83 440 119 474Q158 435 169 427L179 420L176 395Q166 302 166 227Q166 171 174 139Q178 119 186 103T202 81L208 75Q255 100 261 100Q262 100 264 92T267 83Q267 80 208 30T145 -20Q143 -20 118 17L91 55Q98 117 98 193Q98 339 88 369Q71 390 68 390Q63 390 39 376L27 369",
      120103: "101 644L132 672L145 657Q155 642 169 628L182 614L169 600Q148 579 140 570L125 555L112 568Q109 570 91 592T72 615Q72 617 101 644ZM19 -208Q-9 -185 -9 -183Q57 -134 80 -106Q103 -81 110 -19T118 179Q118 294 113 329T86 383Q78 389 76 389Q73 389 57 379L39 367Q26 380 26 381L33 387Q40 393 53 405T79 428L130 474Q147 457 186 428L195 422L196 376Q196 130 192 58T174 -35Q172 -39 104 -114T19 -208",
      120104: "93 348Q93 379 90 503T87 658V670Q87 671 94 671L100 672L112 636Q124 603 124 600L144 616Q196 659 234 682L246 689Q252 686 256 685T268 680L254 667Q225 641 211 626T184 579T168 505V497L178 504Q203 523 237 544L300 584L310 573Q359 522 359 481Q359 476 359 472T358 464T355 456T351 448T346 440T340 433T332 424T323 414T312 402T299 389L255 343L300 342H346L320 309H177V128L193 113Q214 92 244 68L254 60L303 84Q351 108 352 108L362 96Q355 85 277 30T194 -25L170 -4Q146 18 120 41T89 68Q94 85 94 168Q94 184 94 218T93 268V309H58L24 310L51 342H93V348ZM172 342Q217 342 252 374T287 445Q287 472 255 504Q242 517 241 517Q202 498 187 468T168 365V342H172",
      120105: "221 76L267 99Q268 98 269 96T271 88T270 82L262 76Q255 69 241 57T214 34L148 -20L98 56L99 89Q107 247 107 373Q107 494 101 621Q101 653 99 659V665L106 668L113 672L118 655Q122 642 131 617L138 597Q140 597 157 611L240 670L264 685Q265 685 270 680T276 674T269 667T247 647T219 617Q197 587 186 551T173 493T171 438Q171 376 175 282T179 147V117Q184 109 201 93T221 76",
      120106: "20 367L8 379Q9 380 63 425T118 471Q130 460 143 446L168 421V398L169 376L295 475Q362 433 415 418V399Q415 380 416 380T437 394T484 428T529 462L544 474L556 467Q590 449 614 438T646 424L653 421L665 417L664 412Q664 411 664 407T664 397T663 384Q660 342 660 335Q658 303 658 245Q658 186 660 152L661 126L669 115Q680 96 697 79L707 83Q716 87 723 90T735 96T741 100T746 102L747 103V102L750 95Q753 88 753 87L631 -18Q630 -17 622 -3T589 43L576 60L579 72Q592 146 592 218Q592 265 584 321Q581 345 578 350T560 363Q535 376 496 386L481 390Q475 387 425 358L415 351V238V157Q415 142 415 135T417 120T421 110T430 98T441 81L465 47Q462 44 458 41T443 28T420 8L380 -26L333 47L336 62Q339 77 342 109T345 184Q345 223 341 285T333 348Q322 364 258 382L240 388L169 347L168 240Q168 118 171 110L174 106Q178 101 183 93T195 78L217 48Q217 47 196 30T154 -5T133 -21L130 -16Q127 -10 122 0T111 19Q89 56 89 60Q95 76 95 153Q95 239 88 337V365L62 391L20 367",
      120107: "31 368Q20 379 20 380T72 427L121 470L129 463Q137 455 155 441T176 425V367L320 475L329 469Q351 454 385 437T434 417L432 402Q429 362 429 231V128L439 111Q442 107 445 102T449 95T453 90T456 86T459 84T463 82T467 82Q468 82 489 93T511 105T514 95V88L395 -18L340 65L341 77Q347 111 348 178Q348 343 343 353V358L332 363Q298 377 264 382Q260 382 254 382L249 383Q247 383 211 362L176 341V229V147Q176 134 176 127T177 115T179 106T183 99T189 91T197 81Q201 76 206 69T214 57L217 53Q217 52 175 15T132 -22Q132 -20 114 5T88 41Q84 45 84 50Q94 116 94 203Q94 227 93 263T92 324L90 364L67 388L50 378Q32 368 31 368",
      120108: "67 121Q70 265 84 359V364L97 371Q137 394 177 421T238 464T260 480L278 468Q295 457 327 442T394 418L406 414Q412 388 412 300Q412 263 410 228T405 170T399 130T393 103T391 92L381 86Q274 19 165 -28L161 -24Q157 -21 151 -15T136 -2T118 14T101 30T86 46T74 60L67 68V121ZM335 278Q335 286 335 303T334 335L333 349V356H326Q288 360 208 388L183 397Q178 393 172 383T154 333T142 239Q142 192 151 134Q152 124 155 120T175 99Q197 77 219 64T251 47T267 44T281 52T302 80T320 124Q335 182 335 278",
      120109: "23 25Q15 33 12 38L35 59Q82 102 87 105V351L68 378Q42 414 42 422Q42 437 67 472T114 530L125 541Q126 541 132 537T138 532Q121 507 121 488Q121 453 160 413L167 405L166 390L165 372Q169 372 233 421T305 478Q307 479 315 472Q395 421 423 415Q424 415 424 415T425 414L426 410Q426 407 426 401T426 388Q430 335 430 272Q430 139 408 80Q407 78 340 22L273 -33Q200 23 165 23Q164 23 164 -32Q165 -89 168 -155V-170L93 -212L78 -203Q85 -48 87 16V47H79Q53 44 23 25ZM319 41Q346 94 349 212Q349 278 343 343V351L330 357Q318 363 270 381L255 387L245 383Q212 370 178 345L166 336V287Q165 260 165 166V94H175Q219 90 299 51L319 41ZM132 57L129 60Q130 58 132 57ZM158 29Q159 29 155 34T145 45T138 50Q155 29 158 29",
      120110: "399 19Q399 -123 407 -174V-179L332 -219L322 -210L312 -202L314 -185Q320 -83 323 54V95L311 85Q255 40 162 -19Q146 -29 145 -27Q140 -22 103 20L63 65V73Q61 83 61 115Q61 122 61 135T60 154Q60 263 79 353L83 368L94 375Q123 391 147 405T186 429T212 446T229 458T238 466T243 470T247 471L260 464Q274 457 295 448T330 434L341 432L410 479L412 478Q414 476 416 475T419 473L417 461Q399 358 399 19ZM316 367Q287 370 252 377T196 391L176 397H175Q173 397 166 382T149 314T139 187V158Q139 138 141 132T155 114Q158 111 160 109Q210 58 211 58L218 62Q226 67 240 75T266 91L319 124V196Q319 334 325 361V367H316",
      120111: "357 398Q364 398 375 403L386 408Q386 404 387 400V393L369 377Q361 370 350 360T336 347T327 340T316 336T303 335Q270 335 234 371L224 379Q220 375 214 370T198 355L182 340V243Q182 164 183 152T189 129Q195 117 211 100T239 72T254 60T298 81Q342 101 343 101Q344 100 345 92T346 82L200 -21Q174 -5 117 55L97 75L99 92Q106 147 106 196Q106 216 104 282T97 355Q95 359 95 361Q95 364 77 378L65 387L48 380Q30 372 29 372Q28 371 23 375T17 380Q17 384 50 415T107 467L115 474L128 461Q171 421 182 414V382L235 425Q247 435 261 446T283 462L290 468L295 460Q335 398 357 398",
      120112: "272 33Q284 33 290 70T296 138Q296 181 290 198T262 230Q257 232 246 232Q231 232 218 227T156 194L108 167L101 176Q90 192 75 219L69 230V284Q69 346 70 357V377L146 427Q181 450 202 464T223 477L225 476Q227 475 229 474Q231 472 242 466T270 450T296 431Q309 418 326 418Q342 418 361 429T392 450T403 459Q406 445 406 442Q406 440 384 421T338 382L316 363Q315 362 310 362Q292 362 251 371T173 396Q156 403 155 403Q143 386 143 342Q143 305 150 278T168 244Q171 242 181 242L190 243L315 300Q330 295 357 274Q358 273 362 270T366 266T369 261T372 253T373 239T374 217T374 185Q374 157 372 133T368 98T363 71T358 50L336 40Q288 20 255 2T211 -22T198 -29L190 -22Q181 -14 168 -3T138 19T104 38T70 46Q50 46 32 37T3 20T-9 11L-18 23L-1 41Q46 89 62 99T112 110Q141 110 157 103T225 62Q268 33 272 33",
      120113: "228 640L242 627L227 613Q213 599 211 593T203 553Q197 500 196 435V389H301L272 353H195V274Q195 178 196 159L197 123Q249 71 273 68H280L312 83Q344 99 345 99Q348 99 348 78Q348 76 314 52T246 4L212 -20Q211 -20 164 19T112 62Q112 122 113 196Q113 228 113 262T114 318T114 341V353H27L62 389H116L117 431V558L228 640",
      120114: "171 315Q171 252 165 199T153 124L147 103Q147 102 196 80L244 57L335 107V116Q339 161 339 268Q339 315 338 351T337 412V424L420 468Q424 465 427 461L424 435Q412 331 412 226Q412 170 415 145T434 96Q442 84 452 83Q461 83 492 96L506 102Q507 102 510 96T513 88L493 71Q445 32 401 -9Q392 -18 388 -17L384 -16Q358 39 355 44Q336 77 333 77Q261 45 203 -9Q186 -23 181 -23Q177 -23 162 -11T121 18T69 44L57 48L61 54Q65 60 71 73T82 102Q100 156 100 262Q100 305 93 335T80 373T62 396L23 376Q18 377 9 386L119 474Q149 437 171 421V339V315",
      120115: "55 418Q55 443 100 503Q121 530 123 530Q125 528 127 528T131 528T134 528T135 524T131 516Q123 499 123 486Q123 457 159 416L169 405L170 388L171 372Q171 371 244 424L317 477L334 466Q394 428 432 416L433 402Q433 400 433 377T434 336V305Q434 195 420 131Q413 94 406 87Q403 84 298 29L192 -28L172 -8Q139 25 106 52L92 64V70Q95 97 95 198Q95 293 94 318L92 355L84 367Q55 406 55 418ZM174 306Q174 297 173 255T171 184Q171 153 175 136T198 100Q207 94 224 80T255 57T282 49Q304 49 328 80Q359 129 359 243Q359 284 352 345Q351 358 348 360Q333 373 304 384T265 396L251 390Q215 372 186 351L175 344V337Q175 319 174 306",
      120116: "45 425Q45 439 82 485T126 532Q127 532 128 532T131 531T137 529L132 519Q121 499 121 483Q121 473 131 454T156 419L167 408L169 376L185 388Q237 425 291 473L301 466Q337 443 394 416L419 405L418 387V369Q419 369 487 418T560 471L581 459Q597 450 621 437T664 415T684 406Q688 406 688 323Q688 254 679 193T662 103T648 71Q647 70 554 20L464 -28L443 -15Q416 1 386 17T343 40T331 49Q331 52 333 73T337 133T339 216Q339 309 334 339Q333 341 316 353T277 377T246 389Q235 389 201 369T166 346Q166 345 164 247Q164 232 164 210T163 179Q163 139 170 116T205 57L212 48L136 -27Q115 16 87 44L78 53L80 67Q88 124 88 211Q88 282 87 315T83 356T74 371Q45 410 45 425ZM591 352Q580 359 565 365T540 374T517 381T504 385L418 342L417 318V220Q417 212 417 194T417 170T418 152T420 134T424 120T431 106T440 95T454 83Q508 44 544 44Q566 44 583 77Q603 120 605 235Q605 289 600 322Q598 343 597 345T591 352",
      120117: "14 377Q63 428 117 472Q130 462 144 449L193 408V392V376L247 420Q300 464 301 464L310 451Q331 417 363 390L333 365Q303 340 301 340Q293 343 277 364T250 386Q241 386 215 361L195 341Q194 333 193 327T191 318T190 304T188 269Q188 220 192 148Q193 122 195 118T210 101Q255 60 271 60Q276 60 278 61L318 82Q327 86 336 91T351 98L356 101Q359 82 356 79Q354 77 283 27T210 -24L192 -7Q160 23 137 40L126 49L116 40Q66 -13 66 -54Q66 -117 167 -140L179 -142V-147L180 -152L123 -188L112 -185Q58 -170 31 -145Q10 -122 10 -96Q10 -80 22 -53Q44 -10 95 49Q97 51 100 54T104 59T107 64T109 71T111 81T112 97T113 119T114 149T114 188Q114 284 108 347V354L96 365Q76 385 67 385Q62 385 45 377L27 368L14 377",
      120118: "74 58Q89 124 89 241Q89 265 89 278T89 305T88 324T87 336T84 346T81 352T77 359T71 368Q57 388 52 398L45 409Q62 454 98 507L113 528Q128 528 128 524Q128 523 125 518T120 503T117 483V471Q123 456 130 445Q135 438 140 432T148 422T153 415T157 409T160 405T162 402T163 399T163 395V384V365Q165 365 237 419L311 473Q383 430 420 416Q426 414 426 411V404Q426 398 426 388T427 367Q431 233 431 201Q431 -3 391 -52Q387 -58 305 -138T221 -218L218 -217Q216 -216 211 -214T202 -210L184 -202L199 -194Q259 -162 288 -127T334 -19T350 184Q350 257 342 328L340 356Q282 390 262 390Q248 390 178 346L163 336V111L216 45L131 -26L124 -15Q114 3 85 44L74 58",
      120119: "47 335L36 347L52 362Q112 421 167 461L181 471L192 465Q236 444 260 417T284 369Q284 355 276 343T233 291Q195 246 195 245T205 242T241 228T300 198L302 187Q314 138 314 74Q314 -24 291 -95Q290 -100 222 -157L154 -214H145Q102 -214 53 -189T-7 -117Q41 -21 183 122L207 147Q205 149 186 157T140 172T100 180H97V192L140 231Q192 280 199 293Q205 303 205 315Q205 339 185 363T137 388Q119 388 63 347L47 335ZM179 -153Q212 -153 226 -113T240 14Q240 67 233 98T223 132T211 143L222 130Q216 125 206 116T170 82T124 33T88 -20T72 -70Q72 -99 105 -126T179 -153",
      120120: "130 -1H63Q34 -1 26 2T17 17Q17 24 22 29T35 35Q49 35 64 44T88 66Q101 93 210 383Q331 693 335 697T346 701T357 697Q358 696 493 399Q621 104 633 83Q656 35 686 35Q693 35 698 30T703 17Q703 5 693 2T643 -1H541Q388 -1 386 1Q378 6 378 16Q378 24 383 29T397 35Q412 35 434 45T456 65Q456 93 428 170L419 197H197L195 179Q184 134 184 97Q184 82 186 71T190 55T198 45T205 39T214 36L219 35Q241 31 241 17Q241 5 233 2T196 -1H130ZM493 68Q493 51 481 35H619Q604 56 515 256Q486 321 468 361L348 637Q347 637 330 592T313 543Q313 538 358 436T448 219T493 68ZM404 235Q404 239 355 355T295 488L275 430Q241 348 208 232H306Q404 232 404 235ZM155 48Q151 55 148 88V117L135 86Q118 47 117 46L110 37L135 35H159Q157 41 155 48",
      120121: "11 665Q11 672 22 683H213Q407 681 431 677Q582 649 582 515Q582 488 573 468Q554 413 484 372L474 366H475Q620 317 620 178Q620 115 568 69T420 6Q393 1 207 -1H22Q11 10 11 18Q11 35 51 35Q79 37 88 39T102 52Q107 70 107 341T102 630Q97 640 88 643T51 648H46Q11 648 11 665ZM142 341Q142 129 141 88T134 37Q133 36 133 35H240L233 48L229 61V623L233 635L240 648H133L138 639Q142 621 142 341ZM284 370Q365 378 391 411T417 508Q417 551 406 581T378 624T347 643T320 648Q298 648 278 635Q267 628 266 611T264 492V370H284ZM546 515Q546 551 531 577T494 617T454 635T422 641L411 643L420 630Q439 604 445 579T452 510V504Q452 481 451 467T441 430T415 383Q420 383 439 391T483 413T527 455T546 515ZM585 185Q585 221 570 249T534 294T490 320T453 334T436 337L435 336L440 330Q445 325 452 315T467 288T479 246T484 188Q484 145 474 110T454 62T442 48Q442 47 444 47Q450 47 470 54T517 75T564 119T585 185ZM449 184Q449 316 358 332Q355 332 335 333T302 335H264V199Q266 68 270 57Q275 50 289 43Q300 37 324 37Q449 37 449 184",
      120123: "16 666Q16 675 28 683H193Q329 683 364 682T430 672Q534 650 600 585T686 423Q688 406 688 352Q688 274 673 226Q641 130 565 72T381 1Q368 -1 195 -1H28Q16 5 16 16Q16 35 53 35Q68 36 75 37T87 42T95 52Q98 61 98 341T95 630Q91 640 83 643T53 648Q16 648 16 666ZM237 646Q237 648 184 648H128Q128 647 133 632Q136 620 136 341Q136 64 133 50L128 35H237L230 48L226 61V343Q228 620 231 633Q232 636 237 646ZM264 61Q278 40 310 35Q363 35 401 55T461 112T496 193T513 295Q515 333 515 349Q515 411 504 459Q481 598 373 641Q351 648 321 648Q304 648 292 643T277 635T264 621V61ZM461 628Q462 627 471 616T489 594T509 559T529 509T544 441T550 352Q550 165 479 75L468 59Q474 61 484 65T522 87T573 128T618 195T650 290Q654 322 654 354Q654 418 638 464T581 552Q559 576 529 595T480 621L461 628",
      120124: "12 666Q12 675 24 683H582Q590 680 593 672V588Q593 514 591 502T575 490Q567 490 563 495T555 517Q552 556 517 590Q486 623 445 634T340 648H282Q266 636 264 620T260 492V370H277Q329 375 358 391T404 439Q420 480 420 506Q420 529 436 529Q445 529 451 521Q455 517 455 361Q455 333 455 298T456 253Q456 217 453 207T437 197Q420 196 420 217Q420 240 406 270Q377 328 284 335H260V201Q261 174 261 134Q262 73 264 61T278 38Q281 36 282 35H331Q400 35 449 50Q571 93 602 179Q605 203 622 203Q629 203 634 197T640 183Q638 181 624 95T604 3L600 -1H24Q12 5 12 16Q12 35 51 35Q92 38 97 52Q102 60 102 341T97 632Q91 645 51 648Q12 648 12 666ZM137 341Q137 131 136 89T130 37Q129 36 129 35H235Q233 41 231 48L226 61V623L231 635L235 648H129Q132 641 133 638T135 603T137 517T137 341ZM557 603V648H504Q504 646 515 639Q527 634 542 619L557 603ZM420 317V397L406 383Q394 370 380 363L366 355Q373 350 382 346Q400 333 409 328L420 317ZM582 61L586 88Q585 88 582 83Q557 61 526 46L511 37L542 35H577Q577 36 578 39T580 49T582 61",
      120125: "584 499Q569 490 566 490Q558 490 552 497T546 515Q546 535 533 559Q526 574 506 593T469 621Q415 648 326 648Q293 648 287 647T275 641Q264 630 263 617Q262 609 260 492V370L275 372Q323 376 350 392T393 441Q409 473 409 506Q409 529 427 529Q437 529 442 519Q444 511 444 362Q444 212 442 206Q436 197 426 197Q409 197 409 217Q409 265 375 299Q346 328 280 335H260V206Q260 70 262 63Q265 46 276 41T326 35Q362 35 366 28Q377 17 366 3L360 -1H24Q12 5 12 16Q12 35 51 35Q92 38 97 52Q102 60 102 341T97 632Q91 645 51 648Q12 648 12 666Q12 675 24 683H573Q576 678 584 670V499ZM137 341Q137 131 136 89T130 37Q129 36 129 35H182Q233 35 233 39Q226 54 225 92T224 346L226 623L231 635L235 648H129Q132 641 133 638T135 603T137 517T137 341ZM549 603V648H495L506 641Q531 621 533 619L549 603ZM409 317V395L400 386Q390 376 375 366L357 355L373 346Q394 331 397 328L409 317",
      120126: "737 285Q749 277 749 268Q749 260 744 255T730 250Q695 250 677 217Q666 195 666 119Q666 52 664 50Q656 36 555 3Q483 -16 415 -19Q364 -19 348 -17Q226 -3 146 70T44 261Q39 283 39 341T44 421Q66 538 143 611T341 699Q344 699 364 700T395 701Q449 698 503 677T585 655Q603 655 611 662T620 678T625 694T639 702Q650 702 657 690V481L653 474Q640 467 628 472Q624 476 618 496T595 541Q562 587 507 625T390 663H381Q337 663 299 625Q213 547 213 337Q213 75 341 23Q357 19 397 19Q440 19 462 22T492 30T513 45V119Q513 184 506 203Q491 237 435 250Q421 250 415 257Q404 267 415 281L421 285H737ZM250 43Q250 45 243 55T225 87T203 139T185 224T177 343V361Q184 533 250 625Q264 643 261 643Q238 635 214 620T161 579T110 510T79 414Q74 384 74 341T79 268Q106 117 230 52L250 43ZM621 565V625Q617 623 613 623Q603 619 590 619H575L588 605Q608 583 610 579L621 565ZM655 250H517L524 241Q548 213 548 149V114V39Q549 39 562 44T592 55T615 63L630 70V134Q632 190 634 204T648 237Q655 245 655 250",
      120128: "20 666Q20 676 31 683H358Q369 676 369 666Q369 648 331 648Q288 645 282 632Q278 626 278 341Q278 57 282 50Q286 42 295 40T331 35Q369 35 369 16Q369 6 358 -1H31Q20 4 20 16Q20 35 58 35Q84 37 93 39T107 50Q113 60 113 341Q113 623 107 632Q101 645 58 648Q20 648 20 666ZM249 35Q246 40 246 41T244 54T242 83T242 139V341Q242 632 244 639L249 648H140Q146 634 147 596T149 341Q149 124 148 86T140 35H249",
      120129: "79 103Q108 103 129 83T151 38Q151 9 130 -15Q116 -34 130 -37Q133 -39 157 -39Q208 -39 219 -8L226 3V305Q226 612 224 621Q220 636 211 641T166 647Q137 647 128 654Q119 665 128 679L135 683H466Q478 677 478 666Q478 647 439 647Q399 644 393 632Q388 620 388 347Q386 69 384 59Q364 -6 316 -39T184 -77H172Q102 -77 56 -48T6 30Q6 62 26 82T79 103ZM353 354Q353 556 354 596T361 645Q362 646 362 647H253Q257 639 258 628T261 547T262 312V-4L255 -17Q248 -29 250 -29Q253 -29 258 -28T277 -20T302 -5T327 22T348 65Q350 74 353 354ZM115 36Q115 47 105 57T79 67Q73 67 67 66T52 56T44 34Q44 9 62 -8Q66 -11 71 -15T81 -22T86 -24L90 -13Q100 3 102 5Q115 22 115 36",
      120130: "22 666Q22 676 33 683H351L358 679Q368 665 358 655Q351 648 324 648Q288 645 280 637Q275 631 274 605T273 477L275 343L382 446Q473 530 492 553T512 599Q512 617 502 631T475 648Q455 651 455 666Q455 677 465 680T510 683H593H720Q732 676 732 666Q732 659 727 654T713 648Q670 648 589 581Q567 562 490 489T413 415Q413 413 554 245T711 61Q737 35 751 35Q758 35 763 29T768 15Q768 6 758 -1H624Q491 -1 486 3Q480 10 480 17Q480 25 487 30T506 35Q518 36 520 38T520 48L400 195L302 310L286 297L273 283V170Q275 65 277 57Q280 41 300 38Q302 37 324 35Q349 35 358 28Q367 17 358 3L351 -1H33Q22 4 22 16Q22 35 60 35Q101 38 106 52Q111 60 111 341T106 632Q100 645 60 648Q22 648 22 666ZM240 341V553Q240 635 246 648H138Q141 641 142 638T144 603T146 517T146 341Q146 131 145 89T139 37Q138 36 138 35H246Q240 47 240 129V341ZM595 632L615 648H535L542 637Q542 636 544 625T549 610V595L562 606Q565 608 577 618T595 632ZM524 226L386 388Q386 389 378 382T358 361Q330 338 330 333Q330 332 330 332L331 330L533 90Q558 55 558 41V35H684L671 50Q667 54 524 226",
      120131: "12 666Q12 675 24 683H333L340 679Q350 665 340 655Q333 648 309 648Q287 646 279 643T266 630Q264 623 264 346Q264 68 266 57Q274 40 284 35H340Q413 37 460 55Q514 78 553 117T602 197Q605 221 622 221Q629 221 634 215T640 201Q638 194 625 105T611 12Q611 6 600 -1H24Q12 5 12 16Q12 35 51 35Q92 38 97 52Q102 60 102 341T97 632Q91 645 51 648Q12 648 12 666ZM137 341Q137 131 136 89T130 37Q129 36 129 35H237Q235 41 233 48L229 61L226 339Q226 621 229 628Q230 630 231 636T233 643V648H129Q132 641 133 638T135 603T137 517T137 341ZM580 48Q580 59 583 74T586 97Q586 98 585 97T579 92T571 86Q549 64 513 43L500 35H577L580 48",
      120132: "18 666Q18 677 27 680T73 683H146Q261 683 266 679L465 215Q469 215 566 443Q663 676 668 681Q673 683 790 683H908L915 679Q924 664 915 655Q912 648 897 648Q851 639 835 606L833 346Q833 86 835 79Q838 69 849 58T873 41Q877 40 887 38T901 35Q926 35 926 16Q926 6 915 -1H604L597 3Q588 19 597 28Q600 35 615 35Q660 42 673 68L679 79V339Q679 409 679 443T679 520T679 580T677 597Q646 521 584 375T473 117T424 3Q416 -1 410 -1T401 1Q399 3 273 301L148 599L146 343Q146 86 148 79Q152 69 163 58T186 41Q190 40 200 38T215 35Q226 35 235 28Q244 17 235 3L228 -1H28Q17 4 17 17Q17 35 39 35Q84 42 97 68L104 79V639L88 641Q72 644 53 648Q34 648 26 651T18 666ZM457 166Q451 169 449 171T435 198T404 268T344 412L244 648H157L166 637Q169 633 293 346L413 66Q424 88 435 117L457 166ZM817 646Q817 648 766 648H715V72L708 57Q701 45 697 41L695 37Q695 35 757 35H819L813 46Q802 61 800 76Q797 105 797 346L799 612L804 626Q812 638 815 641L817 646ZM124 42Q119 42 119 38Q119 35 128 35Q132 35 132 36Q125 42 124 42",
      120134: "131 601Q180 652 249 676T387 701Q485 701 562 661Q628 629 671 575T731 448Q742 410 742 341T731 234Q707 140 646 81Q549 -19 389 -19Q228 -19 131 81Q57 155 37 274Q34 292 34 341Q34 392 37 410Q58 528 131 601ZM568 341Q568 613 437 659Q406 664 395 665Q329 665 286 625Q232 571 213 439Q210 408 210 341Q210 275 213 245Q232 111 286 57Q309 37 342 23Q357 19 389 19Q420 19 437 23Q469 38 491 57Q568 132 568 341ZM174 341Q174 403 177 441T197 535T249 639Q246 639 224 627T193 608Q189 606 183 601T169 589T155 577Q69 488 69 344Q69 133 231 52Q244 45 246 45Q248 45 246 48Q231 69 222 85T200 141T177 239Q174 269 174 341ZM708 341Q708 415 684 475T635 563T582 610Q578 612 565 619T546 630Q533 637 531 637Q530 637 530 636V635L531 634Q562 591 577 543Q602 471 602 341V316Q602 264 599 230T580 144T531 48L530 47V46Q530 45 531 45Q533 45 547 52T583 75T622 105Q708 195 708 341",
      120138: "54 238Q72 238 72 212Q72 174 106 121Q113 110 132 90T166 59Q221 23 264 23Q315 23 348 41Q368 50 384 79Q393 102 393 129Q393 181 356 219T221 299Q120 343 74 390T28 501Q28 561 55 610Q98 682 212 699Q214 699 231 700T261 701Q309 698 340 687T408 675Q431 678 445 690T465 702Q474 702 481 690V497L477 490Q464 481 450 490Q446 500 446 501Q446 546 386 606T260 666Q215 666 182 639T148 565Q148 528 186 496T319 428Q352 414 370 405T418 379T468 338T506 284Q528 239 528 191Q528 102 456 46T266 -10Q211 -10 176 2T110 15Q86 9 73 -1T53 -12Q44 -12 37 -1V112V182Q37 214 40 226T54 238ZM446 619Q446 648 444 648Q439 646 435 644Q425 644 415 639H404L417 624Q435 606 439 601L446 592V619ZM124 619L128 635Q126 635 108 617Q64 576 64 502Q64 489 65 479T76 449T102 414T150 376T228 335Q335 291 381 245T427 128Q427 94 419 75L415 61Q421 61 448 88Q490 127 490 190Q490 233 475 264Q456 299 430 321Q402 349 369 367T287 404T204 441Q138 481 119 526Q113 544 113 565Q113 596 124 619ZM75 43Q76 43 90 46T110 50H119L106 64L74 101Q72 101 72 72T75 43",
      120139: "33 672Q36 680 44 683H624Q632 680 635 672V490L631 483Q621 479 617 479Q611 479 606 485T600 499Q600 525 584 552Q577 567 558 588T524 617Q479 642 426 646L415 648V355Q415 62 422 52Q425 42 434 40T473 35Q500 35 509 28Q518 17 509 3L502 -1H166L160 3Q149 17 160 28Q167 35 195 35Q224 37 234 39T249 52Q253 66 253 355V648L242 646Q192 642 144 617Q129 609 110 588T84 552Q69 527 69 499Q69 490 64 484T50 478Q39 478 33 490V672ZM113 639L126 648H69V597L84 612Q93 623 113 639ZM389 35Q382 46 381 86Q380 134 380 350V648H289V350Q289 199 288 131T286 53T280 35H389ZM600 597V648H542L555 639Q575 623 584 612L600 597",
      120140: "16 666Q16 677 28 683H341L348 679Q359 665 348 654Q342 648 315 648Q270 644 266 632Q262 627 262 598T261 399Q261 372 261 325T260 260Q260 149 274 99T339 30Q355 25 393 25Q430 25 457 33T494 49T519 72Q562 115 575 205Q576 219 576 379Q576 538 575 550Q568 597 550 622T506 648Q498 648 493 654T487 667T499 683H697Q709 675 709 667T704 654T690 648Q653 648 633 597Q624 573 622 546T619 377Q617 193 613 174Q596 95 544 41Q477 -19 355 -19H344Q275 -16 226 5T153 57T120 110T106 154Q101 172 99 399Q99 618 95 632Q88 644 53 648Q16 648 16 666ZM228 639L233 648H128Q128 647 133 632Q135 621 135 412Q135 197 137 185Q148 115 181 79Q209 51 235 41Q242 36 258 31T277 25Q276 27 268 38T254 59T241 92T228 145Q226 161 226 399Q226 632 228 639ZM604 621Q606 626 619 648H577L586 634Q587 632 591 625T595 614L597 608L604 621",
      120141: "316 683Q327 676 327 666Q327 648 302 648Q272 642 258 628Q249 621 249 608Q252 589 263 556T289 485T322 406T357 325T388 256T411 205L420 185Q423 185 473 317Q547 497 547 590Q547 621 541 632T516 648Q501 648 498 654Q488 664 498 679L504 683H607H660Q695 683 707 680T719 667Q719 660 714 654T700 648Q678 648 658 628L642 614L513 301Q484 231 449 148T397 25T380 -15Q373 -20 368 -20Q361 -20 358 -15Q354 -13 287 135T149 438T67 610Q45 648 18 648Q11 648 6 653T0 666Q0 677 9 680T59 683H164H316ZM216 614Q216 620 216 622T216 628T216 633T217 635T218 638T219 640T221 644T224 648H84L96 632Q118 592 236 330L367 43L387 88L404 132L380 185Q250 468 222 568Q216 590 216 614ZM576 645Q584 628 584 597L587 568L598 597Q609 624 618 637L624 648H600Q576 648 576 645",
      120142: "785 664Q785 670 795 683H982Q994 675 994 665Q994 650 975 648Q953 643 939 619Q931 593 823 292T710 -15Q706 -19 699 -19T688 -15Q682 -6 639 107T555 328T513 437Q513 438 500 409T462 325T413 212Q315 -14 310 -17Q308 -19 302 -19T288 -15L57 619Q45 643 24 648Q5 650 5 665Q5 677 17 683H146H200Q256 683 270 681T285 666Q285 659 280 654T268 648Q253 648 239 634Q230 630 230 619Q230 598 264 481L362 192Q363 193 428 341T493 492Q493 496 473 546T446 608Q426 648 399 648Q392 648 387 653T382 667Q382 678 393 683H679Q690 670 690 665Q690 662 685 655T673 648Q653 648 633 632L622 625V610Q626 576 657 479T719 300T751 218Q754 218 779 294Q847 492 847 581Q847 648 802 648Q796 648 791 652T785 664ZM194 623Q194 630 199 648H82L90 632Q99 616 199 332L302 50Q303 50 322 94T342 141Q342 142 305 245T231 467T194 623ZM585 620Q585 634 593 648H530Q466 648 466 645Q479 632 595 323L699 54Q701 56 718 103T735 154L702 245Q585 562 585 620ZM884 572L890 587Q896 602 903 620T915 645Q915 648 893 648H868L875 634Q883 598 883 576Q883 572 884 572",
      120143: "22 666Q22 677 31 680T80 683H184H335Q346 675 346 667Q346 660 341 655Q335 648 315 648Q280 644 273 637Q273 630 300 583T356 492T386 448Q430 504 450 535T474 577T478 601Q478 620 469 634T444 648Q428 648 428 666Q428 678 436 680T488 683H559H630Q673 683 681 681T690 666Q690 648 673 648Q652 648 619 637Q571 615 517 550Q490 517 450 464T410 408Q415 399 501 273T617 106Q648 61 661 48T688 35Q705 35 705 16Q705 5 695 -1H539Q384 -1 379 3Q373 10 373 17Q373 27 380 31T408 35Q459 40 459 49Q459 59 418 129T335 259Q334 260 332 260Q328 260 273 197Q210 127 208 117Q199 104 199 82Q199 57 213 46T239 35Q247 35 252 29T257 15Q257 10 256 7T253 3T248 0L246 -1H28Q16 7 16 15T21 29T35 35Q61 35 117 88Q289 279 304 297Q307 303 255 377Q117 586 79 626Q60 648 39 648Q32 648 27 653T22 666ZM237 639V648H173Q113 647 113 646Q113 642 137 612Q186 546 302 373T453 139Q497 63 497 43Q497 39 495 35H559Q622 35 622 37Q622 38 583 94T486 233T373 399T277 552T237 639ZM553 637L566 648H504L508 637Q510 630 515 615V603L528 615Q529 616 539 625T553 637ZM170 46Q169 49 167 58T164 70V83L137 59L113 35H175Q175 38 170 46",
      120144: "16 659T16 667T28 683H295Q306 676 306 666Q306 648 284 648Q258 648 255 641Q255 634 265 615T339 479Q418 339 421 339L455 394Q489 448 523 502L557 557Q560 566 560 582Q560 637 504 648Q489 648 486 655Q475 664 486 679L493 683H693Q704 675 704 667Q704 650 684 648Q672 645 653 623Q633 604 614 576T517 426L439 301V183Q442 62 444 59Q449 35 504 35Q521 35 528 30Q538 16 528 3L521 -1H195L188 3Q178 16 188 30Q195 35 213 35Q266 35 273 59Q274 61 277 163V261L75 621Q64 638 58 643T37 648Q28 648 22 653ZM219 637V648H101Q110 634 215 446L313 270V166Q310 59 306 48L301 35H415L410 48Q404 65 404 175V290L317 443Q230 601 226 612Q219 625 219 637ZM608 630L624 648H575Q584 632 588 623L595 610L608 630",
      120172: "821 97Q822 97 824 88T827 77L793 53Q676 -25 670 -28Q669 -29 656 -27L583 123Q583 124 467 46L352 -31L341 -20Q305 18 264 47T192 77Q161 77 60 32L49 40Q37 47 38 49Q39 49 93 83T212 160T297 219Q411 312 411 452Q411 519 360 571T233 624Q180 624 157 601T133 548Q133 524 160 496T214 441T241 393Q241 356 199 321T100 256L86 249L77 256Q68 263 67 263L84 274Q101 286 118 304T135 339T109 384T56 446T29 504Q29 566 118 624Q207 686 309 686Q349 686 360 685Q405 678 439 661T491 625T520 583T534 543T537 511Q537 436 491 344L478 318L455 299Q420 272 308 179L284 160L294 158Q348 154 426 89L437 79Q513 110 579 153V175Q579 183 579 227T580 330T581 446T582 542L583 582L664 630Q681 640 703 653T734 673L744 679Q750 678 756 676L767 674L716 623V585Q716 568 712 463T708 289V250Q708 237 709 218T710 195L711 180L739 130Q768 79 771 79Q775 79 796 88T821 97",
      120173: "160 345Q160 357 144 376T109 413T73 458T57 509Q57 544 95 584Q142 631 205 657T331 684Q382 684 427 658T500 585L505 577L521 588Q537 599 562 614T616 646T679 673T738 684Q790 684 807 666T840 587Q850 552 863 532T888 508Q894 505 906 505Q917 505 930 507T953 512T963 514L964 504Q965 495 965 494T914 467T808 413T745 384H751Q782 380 802 377T854 362T904 334T937 287T951 217Q951 178 937 143T908 91Q903 86 820 34L734 -21L718 -24Q679 -31 639 -31Q561 -31 451 4T271 40Q190 40 119 -2L99 -13L91 1L84 15L86 16Q88 18 132 42T233 100T315 152Q377 199 386 233Q388 240 393 297T399 363Q399 487 353 551Q337 573 306 597T238 622Q201 622 179 602T157 557T214 476T272 396Q272 371 229 334T143 272T96 246Q95 246 85 252T74 259T95 273T138 306T160 345ZM529 443Q529 409 528 385T526 353L525 346Q526 346 649 390T773 435Q749 451 742 464T727 518Q727 519 725 532T721 548T717 562T712 577T706 589T698 601T688 608T675 614T658 616Q626 616 576 582T525 528Q525 527 526 518T528 489T529 443ZM772 57Q774 57 778 58T792 64T808 77T821 103T827 144Q827 222 784 266T660 322Q652 323 611 323H596Q577 323 535 316L523 314Q520 291 505 255L500 241L356 138L366 137Q443 131 518 110T650 72T748 54Q763 54 772 57",
      120174: "460 -32Q373 -32 305 -11T193 45T122 124T83 214T72 303Q72 395 114 476L119 486L313 592L338 568L359 580Q418 615 479 638T568 668T606 675Q607 675 608 676H610Q612 676 615 661T630 621T660 578Q673 568 694 568Q717 568 721 570H726Q724 565 722 559L717 549L706 545Q608 513 583 513Q568 517 559 522T533 546T493 603L490 609Q452 599 452 558Q452 537 469 481T486 393Q486 353 474 331T422 285T296 231L272 223L262 230L253 237Q279 246 314 274T351 338Q351 376 334 442T316 532Q316 546 319 552Q319 554 316 554Q304 554 288 547T250 523T214 466T199 371Q199 218 299 133T541 47Q571 47 585 51T652 81L712 108Q716 104 716 81L706 74Q695 68 673 54T633 29L550 -22L540 -24Q492 -32 460 -32",
      120175: "380 596Q307 596 250 582T158 546T100 493T67 433T56 373V361Q55 361 43 366L31 372V384Q31 455 69 523T173 627Q213 650 284 666T444 683H452Q629 683 735 629Q896 548 896 369Q896 263 839 163Q835 155 818 140Q746 82 662 27T563 -29Q525 -29 386 16T183 62Q147 62 127 52T63 1L48 -14L40 -4L31 5Q83 73 172 149L186 161H199Q291 161 329 181Q357 199 357 231Q357 258 301 316T245 396Q245 423 282 458T349 512T403 543L413 548L425 545L438 541Q373 491 373 462Q373 446 399 415T453 349T480 288Q480 251 433 212Q394 180 348 156L334 148L353 145Q408 134 513 105T654 76Q711 76 745 132T780 277Q780 434 676 517Q637 549 562 572T380 596",
      120176: "527 55Q574 55 619 69T691 97L717 111V85L562 -18Q520 -29 443 -29Q379 -29 325 -15T235 21T180 61T146 98Q74 186 74 307Q74 395 109 472Q113 482 123 489T190 533Q251 568 295 591L308 598L350 580L361 586Q403 612 464 636T564 673T609 686Q610 686 610 685Q612 683 616 670T627 636T646 601Q666 572 686 572H692Q713 572 726 576H728L725 565L723 554L692 544Q660 535 629 526T595 516Q585 514 574 519Q563 527 543 552T507 597T490 617Q467 604 456 579V564Q456 535 473 471T492 393L494 381L613 460L622 446Q630 433 650 411T696 371L703 365L614 312H596L580 322Q568 329 553 340T528 355T510 360Q496 358 491 354T484 345T471 326T435 297Q408 278 370 261T307 235T277 227Q273 227 266 234L256 240L267 245Q280 251 294 258T330 288T353 336Q353 373 335 444T316 530V537Q316 549 322 567Q270 554 233 499T196 370Q196 253 287 157Q392 55 527 55",
      120177: "424 522Q265 596 208 596Q193 596 180 593T150 579T116 542T89 474Q86 465 86 463L59 481L63 494Q87 578 137 627Q191 684 285 684Q334 684 406 658T538 607T621 581Q644 581 706 629L721 640Q722 640 725 630L727 620Q701 592 654 548T582 486L569 487Q533 490 485 504L468 508Q449 503 429 495T387 466T365 422Q365 373 439 299L453 310Q473 325 528 370L588 418Q614 398 642 368T668 331Q667 331 628 296L590 262L582 274Q557 311 526 311Q511 311 487 297T462 278Q462 277 492 244T551 166T581 88Q581 54 570 25T536 -27T505 -56T478 -76Q376 -146 274 -146H270Q199 -146 162 -118T124 -15Q124 12 128 30T132 96V107Q132 144 117 157Q102 169 85 169Q74 169 59 165T32 156T20 151Q20 152 19 158T17 167Q17 168 17 168T17 169T19 170T22 172T27 175T35 179Q131 230 195 230Q231 230 259 202Q270 190 270 171Q269 150 253 87T236 -16Q236 -67 261 -87T322 -107Q380 -107 428 -68Q467 -35 467 30Q467 60 447 91T383 171T316 251Q290 286 278 308T263 339T261 359Q261 384 284 418Q322 469 424 522",
      120178: "742 611Q784 611 812 631V611Q807 607 783 591T718 544T629 476L606 458Q608 458 628 457T667 453T713 443T762 423T804 388T836 335Q844 313 844 289Q844 231 814 182T746 103Q720 82 655 48T546 -18L520 -21Q456 -29 432 -29Q313 -29 223 33Q204 45 183 65T135 119T91 207T74 320Q74 428 109 480Q116 491 127 497T215 546L308 595L343 583L355 591Q387 613 433 636T488 660H489L491 659Q493 658 495 657T500 655L509 650L500 645Q479 635 460 612T441 552Q441 535 447 498T459 433T466 405L625 513L643 526Q620 530 585 546T535 586Q535 587 532 592T527 602T525 610Q525 613 577 649L630 687Q632 687 638 675T653 649T686 623T742 611ZM349 313Q349 328 327 413T305 510V516Q305 531 308 542T314 559T317 566T315 567Q297 567 270 548Q233 524 212 490T191 392Q191 337 206 288T244 207T284 156T316 128Q410 51 535 51Q632 51 675 102T718 217Q718 269 690 314T599 375Q574 381 535 381Q501 381 477 377L466 376Q469 364 469 349Q469 314 457 295T408 258Q366 236 308 219L288 213L279 220L270 227Q284 232 294 236T309 243T320 252T326 260T331 270T336 281Q349 310 349 313",
      120179: "288 139Q288 172 255 224T189 335T156 442Q156 495 242 579Q289 625 361 668Q364 671 368 673T376 678T380 681L384 683L392 676Q401 670 414 661T443 642T477 626T509 619Q543 619 618 668Q625 672 628 674T631 675Q632 673 633 663T633 651L564 595Q556 589 545 580T528 566T516 556T505 548T497 543T488 539T481 537T472 535T463 534T451 534H442Q385 534 304 581L291 589Q290 588 285 583T277 575T269 566T262 555T257 543T255 529V522Q255 507 260 487T276 446T293 409T311 376L321 359Q321 358 322 358T324 359T327 361T333 366Q386 409 481 460L503 472L543 471Q586 471 599 470Q692 459 714 430Q725 416 738 360T752 245Q752 184 742 127T725 51T703 -8Q700 -13 619 -64T518 -123Q508 -126 493 -126Q438 -126 398 -86L427 -52Q456 -17 457 -17Q460 -17 465 -16H473Q474 -21 481 -32T504 -56T539 -69Q572 -69 599 -34Q625 4 625 158Q625 264 609 311T532 378Q508 386 484 386Q455 386 419 372T360 345T337 330L346 313Q375 263 386 227Q389 215 389 202Q389 192 388 184T384 168T376 152T365 138T350 121T331 103T307 81T278 54L194 -24Q130 30 99 30Q85 30 64 20T31 1T16 -10Q15 -11 13 -7Q12 -6 11 -3Q8 4 6 8L32 35Q88 88 117 107T169 126Q177 126 182 125Q218 118 252 84L263 73Q288 113 288 139",
      120180: "500 615Q523 615 550 628T595 655T614 668L623 654L607 642Q512 569 440 534L427 527L413 529Q384 535 340 547T265 565T209 572Q173 572 145 556T101 522T60 465Q58 460 54 460T41 468L32 477L37 487Q96 599 139 640Q187 681 247 681Q275 681 283 680Q313 674 398 645T500 615ZM418 170Q418 186 410 260T401 382Q403 418 403 424L405 433L415 444Q482 515 571 571L582 578Q591 573 607 568L597 560Q522 504 522 450Q522 427 533 357T545 241V228Q545 190 536 159T508 106T478 73T446 48Q343 -25 238 -25Q179 -25 118 15L107 22L79 5Q51 -12 51 -12L38 2L55 18Q106 67 175 122L192 136Q202 130 206 123Q223 91 252 61Q263 50 266 48T278 39T297 32T320 30Q357 30 389 68Q415 102 418 170",
      120181: "65 510Q68 517 74 528T101 569T144 620T202 661T274 680Q308 680 389 628T503 576Q530 576 596 600Q615 607 616 607Q616 602 615 596V585Q605 581 576 568T531 548T485 531T418 509L400 503L358 522Q347 527 327 537T299 550T277 560T257 568T239 573T220 577T201 578H196Q181 578 169 575T135 554T88 502L83 496Q82 496 74 502T65 510ZM424 4Q424 50 395 151T365 313V320Q365 352 369 361T405 403Q431 432 465 462T521 508T547 525L549 524Q551 524 554 523T560 521L571 517L552 498Q515 461 499 430Q485 399 485 366Q485 326 512 231T539 84Q539 -14 460 -77T273 -141Q248 -141 234 -140T198 -131T160 -106T134 -59Q128 -40 124 -16T117 22T108 49T91 69T59 75T15 65L1 59Q-8 76 -7 77Q4 85 22 97T88 129T170 149Q218 149 234 125Q242 112 242 43V21Q242 -17 248 -41T274 -85T322 -105H325H330Q363 -105 396 -75Q424 -47 424 4",
      120182: "234 109Q234 144 194 245T153 404Q153 445 180 490Q232 572 325 626T517 681H524Q612 681 661 658Q683 647 699 632T717 604Q717 600 708 545L699 490L690 489Q681 488 679 488Q675 488 669 504T640 546T577 592Q520 620 446 620Q415 620 386 614T327 594T280 553T262 487Q262 468 265 447T271 413T279 384T285 362L295 371Q320 396 352 421T439 474T538 502Q577 502 596 484T627 428Q642 386 651 373T677 360H682Q698 360 727 369L724 357Q724 354 724 351T722 346V344Q559 289 539 283Q582 272 589 271L615 265L637 189Q662 109 663 108Q668 97 682 84Q698 68 722 68H730H738Q762 68 799 91L803 80L806 70Q795 59 770 40T703 -3T631 -26Q598 -26 578 -8Q548 24 536 92Q524 154 509 183T477 218T428 224Q409 224 385 220T346 212L331 207Q330 205 330 201T331 189T332 178Q332 158 325 116L305 96Q269 60 240 38Q171 -21 123 -21Q72 -21 33 18L20 32L62 74Q96 107 102 112T116 118Q120 118 122 113T131 95T150 69Q171 48 190 48Q198 48 206 51T224 69T234 109ZM519 367Q497 432 450 432Q379 432 313 333L300 314L304 299Q306 294 309 280T315 260L321 235L542 313Q530 325 519 367",
      120183: "277 226Q277 248 253 286T203 369T178 449Q178 490 212 533T284 607Q380 683 532 683Q610 683 639 660T668 583Q668 568 666 546T663 509Q663 478 683 460Q691 452 719 452L738 450Q732 437 729 437Q728 437 652 416T573 394Q554 394 541 409T527 444Q527 449 532 487T538 542Q536 584 501 606T418 628Q389 628 364 620T317 587T295 523Q295 478 333 401T372 276Q372 269 371 267Q371 264 318 206L264 149Q284 141 317 130T433 101T577 82Q619 82 652 95T701 127T728 164T742 196L744 209Q744 210 749 208T759 203T764 199T760 185T751 154T744 129Q714 42 680 13Q628 -28 566 -28Q490 -28 403 -5T249 42T153 66T106 53T70 15T47 -16Q46 -17 30 -5L39 13Q85 100 138 148L147 156L161 157Q218 165 246 179T277 226",
      120184: "134 338Q134 357 81 417T27 504Q27 516 34 530Q55 568 110 615Q190 683 305 683H314Q445 683 495 580L501 569L512 577Q608 646 681 646Q759 646 801 585L808 576L816 583Q860 619 921 650T1041 682Q1063 682 1077 675T1096 660T1112 631T1132 596Q1160 555 1188 555Q1204 555 1228 564Q1230 565 1231 562Q1231 560 1232 554V547L1215 538Q1179 521 1114 475Q1112 474 1106 470T1099 464T1093 459T1088 452T1085 441T1082 425T1081 404T1079 376T1079 339Q1079 282 1084 236T1098 160T1117 112T1138 85T1159 77Q1166 77 1180 81T1207 90L1219 94Q1220 94 1221 86T1222 76L1045 -32Q1044 -32 1004 15L964 64V167Q965 334 970 372V378L994 402Q1032 440 1057 460Q1061 463 1066 467Q1070 469 1070 470T1068 471T1060 474T1050 481Q1040 488 1021 531T996 583Q979 609 947 609Q922 609 887 592T820 537L821 524Q825 484 825 448Q825 268 768 155L759 137L589 -28L579 -20Q533 17 507 17Q475 17 449 -7L436 -18L424 2L441 20Q446 25 456 36T471 52T484 65T497 79T509 90T522 99T534 106T548 112T561 115T576 117Q602 117 639 86Q648 81 648 81Q650 82 657 94T668 112Q711 202 711 373Q711 484 677 533T600 583Q592 583 583 581T569 577T554 568T542 560T528 549T516 539L519 523Q527 485 527 461Q527 444 522 407Q506 266 447 150L437 130L217 -25L208 -15Q165 28 126 28Q89 28 62 1Q47 -14 43 -14Q42 -14 36 -8L28 0L44 17Q96 73 120 92T166 117Q182 123 204 123Q239 123 284 78L295 67Q307 72 337 102Q400 178 400 346Q400 508 325 571Q270 618 208 618Q180 618 168 614T140 594Q124 578 124 564Q124 540 182 480T240 396Q240 359 197 321Q154 285 94 252L80 245L76 248L67 257L61 262L71 268Q82 275 94 284T120 309T134 338",
      120185: "522 492Q521 492 517 502T512 513Q542 444 542 333Q542 226 503 137L498 125L396 53Q308 -8 292 -17T260 -27Q226 -27 191 -9T136 29L145 39Q162 56 192 89L230 129L235 128H241Q276 57 332 57Q358 57 391 80Q403 89 409 100T422 143T428 227Q428 329 406 408T347 530T272 594T196 615Q152 615 135 596T118 558Q118 535 146 502T203 438T232 385Q232 357 195 322T122 265T83 243Q82 242 72 249T61 258L66 262Q72 265 82 273T103 292Q125 314 125 333Q125 351 101 376T51 432T26 492Q26 549 108 614T290 679Q326 679 335 678Q353 675 370 670T400 658T425 642T445 625T463 606T477 588T487 571T495 556T500 543L504 535L523 553Q553 581 569 595T619 632T686 667T757 678Q778 678 793 675T819 664T833 651T844 633T852 617Q884 548 910 548H916Q938 548 962 556L967 542Q967 540 947 531Q909 509 883 492T847 467T838 458Q825 419 825 328Q825 234 833 191T858 121Q875 94 892 77Q898 71 907 71Q912 71 928 76T957 87T971 91L972 88Q972 84 972 81L973 73L957 63Q891 21 806 -23L794 -30L783 -14Q766 13 728 60L713 79V372L724 384Q743 406 765 427T800 460L813 471Q809 472 806 472Q783 479 766 503T741 551T715 594T672 614Q644 614 622 595Q597 576 572 550T534 508L522 492",
      120186: "254 595Q269 583 269 581L262 577Q256 573 247 566T228 549T212 527T205 502Q205 480 266 386T328 277Q328 234 239 150L221 134L231 133Q264 131 376 99T516 62Q567 50 604 50Q614 50 626 52Q643 57 662 71T703 115T739 198T753 323Q753 454 692 517Q652 555 584 565T382 577Q365 577 357 577H308L300 591L292 606Q292 608 342 665L392 724L403 725Q406 725 411 726H416L417 725L412 715Q408 705 408 698Q408 684 423 679Q431 677 516 672T663 655Q757 634 806 593T873 463Q881 421 881 380Q881 340 874 306Q859 223 809 147Q801 134 789 124Q595 -30 456 -30Q395 -30 289 3T147 36Q134 36 121 33T98 26T76 15T59 4T44 -8T32 -17L22 -7L12 4L56 59L100 114L116 118Q217 142 217 199Q217 230 185 276T120 365T87 430Q87 435 109 464T172 534T254 595",
      120187: "247 398Q247 372 206 334T126 272T83 247Q82 247 72 253T61 261Q60 261 61 262T66 265Q127 306 127 343Q127 364 63 430Q42 451 38 458T33 480V490V497Q33 526 63 567Q112 632 170 660T282 688Q341 688 384 667Q454 633 482 566Q483 565 484 566T496 574Q562 623 630 653Q699 681 751 681Q778 681 797 673Q818 662 830 609Q835 580 843 564Q863 524 895 524H901Q917 524 932 528Q936 522 938 518T942 513T942 511Q873 480 836 454Q789 423 789 395Q789 362 834 298T880 200Q880 170 867 145T820 81Q733 -20 647 -20Q581 -20 499 21V9Q499 -16 502 -53T509 -116L512 -141L370 -223L357 -216Q344 -209 344 -208L348 -196Q370 -113 370 33V52L355 58Q307 76 284 76Q258 76 228 60T183 29T141 -11Q137 -7 133 -2L126 7L134 18Q181 89 210 121T278 170Q304 179 328 179Q336 179 358 177L370 175Q368 268 367 359Q367 416 363 434Q362 438 362 441Q348 527 302 574T203 621Q169 621 148 599T127 557Q127 535 187 476T247 398ZM673 315Q673 357 786 442Q786 443 776 444T750 449T727 462Q719 471 716 484V496Q715 507 715 515Q715 571 698 588Q680 611 643 611Q592 611 547 571Q534 558 511 522L499 505V139L543 123Q702 64 744 64Q770 64 781 79T793 112Q793 143 733 217T673 315",
      120188: "254 595Q269 583 269 581L262 577Q256 573 247 566T228 549T212 527T205 502Q205 480 266 386T328 277Q328 234 239 150L221 134L231 133Q264 131 376 99T516 62Q567 50 604 50Q614 50 626 52Q643 57 662 71T703 115T739 198T753 323Q753 454 692 517Q652 555 584 565T382 577Q365 577 357 577H308L300 591L292 606Q292 608 342 665L392 724L403 725Q406 725 411 726H416L417 725L412 715Q408 705 408 698Q408 684 423 679Q431 677 516 672T663 655Q757 634 806 593T873 463Q881 421 881 380Q881 340 874 306Q864 250 838 196T791 126Q748 93 733 82L715 69Q714 68 723 60T748 40T774 23Q806 2 832 2Q849 2 870 6T904 14L917 17Q917 12 918 6V-3L882 -22Q806 -60 778 -73L755 -83Q640 -36 596 -7L586 0L576 -4Q513 -30 457 -30Q394 -30 289 2T149 35Q119 35 93 22T52 -4T36 -17T24 -7T12 4L56 59L100 114L116 118Q217 142 217 199Q217 230 185 276T120 365T87 430Q87 435 109 464T172 534T254 595",
      120189: "31 498Q34 541 76 586T176 659T279 688H290Q377 688 429 653T506 569L511 558L526 572Q620 663 707 682Q722 685 737 685Q781 685 804 665T830 619T838 565T854 525Q866 511 897 511Q917 511 925 513L937 515Q938 515 941 509T944 501T925 493T870 470T803 438Q735 406 735 401Q735 400 741 399T767 390T814 374L828 367L829 307Q829 233 833 202T852 144Q873 109 896 90Q906 82 928 82T976 95V92Q976 88 978 72L807 -28Q768 39 733 87L718 108V149Q718 230 714 257T693 298Q654 333 580 333Q524 333 520 329Q520 300 489 224T443 133Q441 131 333 53T223 -27Q221 -26 204 -11T169 16T136 28Q110 28 66 -8L56 -16Q52 -13 40 -1L48 7Q165 124 211 124Q232 124 287 77L298 67Q309 73 337 97Q397 150 397 347Q397 419 379 474T330 560T269 604T207 619Q177 619 152 601T126 563Q126 540 185 479T244 387Q240 336 160 289Q144 278 98 255L80 246L62 261L79 272Q96 283 113 301T130 337Q130 353 115 373T81 410T47 451T31 498ZM524 358Q537 358 657 405T777 457Q777 459 768 459Q749 462 738 474T723 499T714 539Q706 585 697 599Q681 618 657 618Q632 618 597 595T532 515L525 502L524 441Q524 375 523 369Q523 358 524 358",
      120190: "457 -31Q356 -31 272 6T135 120T82 304Q82 372 106 430T170 527T241 588T305 626Q341 643 386 657T460 678T495 685T554 660T674 609T778 584Q800 584 818 591T848 610T866 633T878 651T883 659L893 649L901 639Q879 574 803 532T666 490Q661 490 657 490T650 491T641 492T633 495T622 500T610 505T595 513T577 522T554 533T527 547Q436 594 415 602Q393 608 374 608Q303 608 253 545T202 386Q202 229 307 135T568 41Q674 41 748 85T822 198Q822 244 779 283T639 322Q595 322 499 303T383 283Q358 283 335 290T291 318T270 374Q270 418 313 460T424 510H431L435 505L440 500Q425 496 403 475T380 427Q380 382 431 373Q437 372 475 372Q543 372 626 388T742 404Q831 404 868 362T905 260Q905 182 831 108Q692 -31 457 -31",
      120191: "666 641Q737 641 794 686L802 662Q790 648 734 596L677 541L664 538Q630 528 583 528Q540 528 482 537L461 541Q402 512 402 456Q402 427 439 387T512 311T549 253Q549 220 455 139L440 126Q541 75 586 75Q600 75 619 80T654 94T685 110T709 124T719 130Q722 125 725 119L730 108Q700 72 568 -18Q551 -30 542 -30Q495 -30 404 6T270 42H263Q213 42 142 -11L131 -19L129 -8Q126 1 126 4Q218 84 301 126L316 134H406L413 142Q436 165 436 189Q436 202 421 221T364 281Q336 307 318 328T296 356T283 381L290 394Q338 478 410 540Q419 549 417 549Q415 550 369 558T268 575T195 584Q153 584 127 567T100 523Q100 499 116 479T151 447T170 433Q170 429 171 428Q171 427 131 394T88 359Q82 363 73 370T47 403T31 457Q31 513 79 565T197 648T332 679Q369 679 490 660T666 641",
      120192: "273 244Q273 281 244 331T186 428T155 502Q155 524 165 536Q239 634 333 688Q338 684 345 680L356 672L344 664Q310 642 295 624T280 582Q280 550 303 505T348 407T371 300Q371 270 362 248L247 123L358 92Q452 64 484 64Q507 64 523 72Q553 87 573 109Q583 121 586 146T593 283Q594 303 594 344Q594 401 591 461T584 558L581 595Q598 600 623 611T672 634T719 659T754 678L768 686Q770 686 784 673L782 670Q781 668 777 664T768 655Q747 635 738 616T721 535T714 359Q714 205 723 176Q727 164 744 133T771 89Q780 75 804 75Q814 75 853 87L867 92L871 73L671 -39L654 -10Q636 20 619 50T600 83Q600 84 589 75T539 34Q478 -16 475 -19Q469 -22 449 -28T414 -34Q410 -34 394 -32Q356 -28 282 -2L237 15Q169 38 126 38Q106 38 85 27T51 4T37 -8T27 -1T18 8Q18 10 70 63T124 116Q154 123 176 131T223 154T260 191T273 244",
      120193: "133 343Q133 360 79 416T25 496Q25 523 58 563T118 624Q197 685 293 685Q331 685 339 684Q453 665 489 558L493 546Q521 570 553 596T640 653T725 684Q753 684 783 672T844 641T889 618Q895 616 912 616Q924 616 936 617T956 620T965 622T966 612V604L952 595Q924 576 895 549Q864 517 856 496T847 448V434Q847 395 848 388L859 323Q874 241 874 212Q874 142 830 96Q796 62 724 14Q661 -29 603 -29Q555 -29 421 28T242 86Q182 86 110 31Q105 28 102 26T99 25Q88 36 88 42Q95 54 222 142Q252 163 262 165Q319 183 344 218Q378 266 378 377Q378 444 362 494T319 571T266 610T212 623Q181 623 156 603T131 562Q131 539 154 512T206 458T243 416Q246 409 246 399Q246 387 242 377T225 351T178 311T94 259L79 251Q72 256 68 261T62 268L61 270L70 277Q131 318 133 343ZM822 526Q778 531 719 564T628 597Q611 597 579 574Q543 543 513 506L505 495L506 473Q506 469 506 461T507 449Q507 348 467 271L462 261L404 218L348 174Q349 173 356 173Q384 169 450 144L546 105Q665 56 708 56Q737 56 746 72T756 118Q756 129 755 135L741 219Q725 314 725 334V344Q725 416 736 431Q748 450 815 510L832 526H822",
      120194: "133 317T133 338T80 413T26 496Q26 532 83 591Q100 608 111 616T151 644T219 672T304 682Q381 682 434 646T506 564L510 557Q513 557 534 573L677 665L707 683L790 561L803 572Q933 682 1001 682Q1037 682 1098 650T1193 616Q1208 616 1222 619L1235 622Q1239 622 1239 616Q1239 611 1240 609Q1240 608 1206 577T1138 503T1104 430Q1104 409 1123 330T1142 208Q1142 183 1136 147Q1127 118 1117 106Q1114 103 1031 48T935 -14Q930 -18 908 -22T862 -27Q826 -27 759 -6T647 26Q597 38 578 38Q573 38 561 33T533 20T505 4T480 -10L469 -16L452 -26L439 -28Q423 -30 411 -30Q358 -30 279 7T169 45Q125 45 58 -5L47 -14L41 -4L35 8Q35 11 56 29T113 75T181 125L200 139H217Q279 143 320 180T377 270T394 393Q394 453 378 498T334 568T277 605T213 617Q177 617 155 607Q140 600 130 587T119 560Q119 545 137 522T177 479T217 434T236 393Q236 324 98 251L89 246L76 253L63 261Q91 275 112 296ZM1088 526Q1066 526 1004 556T909 586Q863 586 816 539L802 526L804 514Q814 461 814 411Q814 319 781 238Q772 214 760 198T730 165T702 136L715 133Q759 122 848 90T973 57Q1003 57 1017 80Q1022 93 1022 116Q1022 152 1003 241T983 377V391Q983 405 985 409T1002 429Q1019 450 1045 475T1090 514L1107 528Q1104 527 1102 527T1096 527T1088 526ZM699 358Q699 391 696 419T688 467T675 503T660 530T642 550T626 563T608 574T593 582Q581 575 559 554T524 512Q523 510 523 477Q523 315 444 218L435 207L368 169Q301 132 301 131Q307 128 315 125L377 99Q476 57 515 57Q534 57 608 94L627 102L636 111Q699 187 699 358",
      120195: "273 679Q354 674 408 633T477 525L484 533Q496 548 524 574T571 615Q594 633 625 649T675 673T699 681Q724 632 747 607Q754 601 756 599T765 594T777 591T794 590Q818 590 834 594V585L835 577L704 513L693 518Q657 534 631 560T597 599Q596 601 581 584Q495 490 489 379V366H562L681 369Q682 369 679 366T668 355T651 341L620 314H485V295Q490 190 543 125T686 60Q720 60 789 88L801 93V89Q798 83 798 66Q781 59 685 -10L665 -25L634 -30Q596 -35 594 -35Q570 -35 536 -23T477 19Q461 37 445 67T418 118L409 138Q401 131 388 120T340 79T273 28T206 -12T151 -31Q129 -31 90 -12T32 22L113 101Q114 101 120 96T136 84T160 69T189 56T221 51Q256 51 305 90Q376 149 376 301V315H293Q276 315 251 315T210 314T190 313L168 312Q168 313 200 340L231 368L238 367Q275 367 311 366H378V387Q376 470 355 512T291 572Q274 579 252 579Q223 579 197 568T156 544T131 519T117 508Q112 512 108 518L99 527L117 545Q177 604 255 665L273 679",
      120196: "34 496Q34 518 53 549T107 610T195 661T310 682Q357 682 398 663T460 611Q467 600 475 583T489 554T495 542Q495 544 531 570T617 629T700 676L724 688Q742 670 756 657T784 635T806 621T830 606T856 592Q878 416 878 340Q878 154 805 -3L798 -20L779 -40Q706 -113 613 -163T421 -214Q359 -214 317 -196T256 -160L306 -63L313 -64L320 -66L326 -79Q337 -104 349 -120T392 -151T470 -166Q576 -166 644 -101Q750 7 750 292Q750 426 721 495T617 565H611Q563 565 513 509L506 501L508 493Q508 490 509 475T510 445Q510 319 458 236L451 225L436 216Q406 198 365 169T318 134L332 127Q336 126 397 103T489 80H493Q527 80 593 129L604 137L607 127Q610 119 610 116Q610 114 592 95T543 46T484 -4Q450 -27 446 -27Q441 -27 402 -18Q365 -9 290 20T188 50Q135 50 64 -7L52 -17L43 -7L34 2L51 19Q118 87 177 132L192 143H215Q259 145 289 155T335 184T355 214T366 245Q382 306 382 388Q382 426 381 436Q368 520 318 570T214 621Q184 621 165 608T142 583T137 562Q137 541 163 508L201 469Q245 425 251 408Q253 403 253 398Q253 383 240 366T212 335T161 295Q128 271 99 253L89 247L77 256L65 266L76 273Q125 301 134 329Q136 334 136 342Q136 357 124 372T88 410T49 455Q34 479 34 496",
      120197: "278 601Q242 601 212 591T167 570T121 533Q114 528 111 525L93 550Q223 661 244 667Q299 677 356 677Q415 677 456 666T515 634T541 596T549 555Q549 513 529 478T480 421T424 388T377 372Q365 370 365 367Q365 365 389 365T450 358T523 337T588 282T623 183Q624 177 624 161Q624 20 524 -60Q415 -148 285 -148Q242 -148 213 -139Q181 -131 159 -109Q136 -87 127 -56T114 6T104 49Q94 69 57 69Q38 69 13 58L1 53Q1 55 0 59T-3 68T-4 76Q78 130 138 142Q150 144 162 144Q213 144 227 120T242 31Q242 -30 263 -66T345 -102Q397 -102 444 -52T491 107Q491 172 471 211T428 265Q392 288 306 288Q269 288 233 284L218 282Q208 289 208 291L229 324L251 359Q250 360 248 360Q239 360 248 371L256 381H273Q344 385 378 409T413 495Q413 537 384 569T278 601",
      120198: "80 129V151Q80 241 99 363Q99 367 111 372T172 401T285 465L297 472Q340 455 405 443L423 440L455 453Q486 467 489 467L497 461L494 451Q480 390 480 292V283Q480 207 483 155L484 143L535 80L558 90L582 99Q586 95 586 83Q586 81 513 25L443 -29Q410 16 386 40L371 55V61Q371 63 371 67T370 74V80L278 25Q186 -29 184 -31Q182 -32 160 -12T112 35T80 75V129ZM359 366Q334 366 300 371T243 382L221 388Q218 388 212 375T200 323T194 228Q194 191 197 152L198 139L217 120Q245 92 269 74L279 66L304 78Q338 95 349 100L369 110V152Q368 164 368 210T367 275Q367 358 366 361V366H359",
      120199: "99 398Q99 610 86 662Q86 665 95 669T106 674L108 669Q109 664 112 654T119 635Q122 626 125 616T130 601L131 596Q214 649 273 678Q295 690 298 690Q299 690 304 688T313 682L317 679Q275 653 240 612Q210 569 210 469V459Q210 450 210 432T211 406L212 378L285 425Q301 435 321 447T350 466L360 472Q360 473 361 473T368 471T401 456T465 429L501 414V408Q504 386 504 309Q504 255 500 203T491 125T485 97Q485 95 445 74T343 23T237 -24L214 -32Q197 -22 165 3T109 49T87 73Q99 169 99 398ZM386 251Q386 320 380 347V350L305 374L282 382L214 348L213 274Q213 184 214 165V131L230 119Q288 76 349 54Q386 137 386 251",
      120200: "227 393Q215 393 210 351T205 269Q205 161 213 153Q220 145 244 125T290 88L312 72L365 92Q414 113 418 113V93L365 60Q255 -9 221 -26L211 -18Q158 21 91 88L90 107Q87 167 87 225Q87 267 90 302T96 351T100 366L295 473L311 470Q340 464 368 454T410 437T424 429L347 334L342 333H337L325 342Q299 363 271 378T228 393H227",
      120201: "88 117Q88 177 91 231T97 310T102 341Q102 343 118 357T168 397T239 447L257 459L268 454L278 449Q242 416 238 412L219 394Q219 391 216 378T211 349T206 307T203 249Q203 211 206 166L208 148Q224 132 261 108T333 70Q341 66 342 67T350 79Q393 157 393 302Q393 368 388 406V411L371 424Q199 558 101 558Q69 558 28 545L18 542L8 549L-1 557L24 569Q61 587 147 621L177 632Q179 631 194 627T216 621T240 613T269 602T302 589T340 571T382 549T431 522T484 488Q504 475 504 472Q511 449 511 365Q511 248 474 129L468 108L451 96Q427 77 347 28T254 -28Q235 -20 174 21T89 86L88 117",
      120202: "309 69Q391 98 416 108Q418 106 422 100T425 92Q419 86 326 30T229 -27Q228 -27 207 -13T154 27T97 76L85 87L84 106Q81 152 81 194Q81 295 93 359L95 369L286 471L313 449Q376 397 414 372L428 362Q428 360 375 318L188 181V170Q188 156 189 153V148L203 138Q228 119 266 94T309 69ZM209 389Q208 388 204 366T194 307T187 244Q187 225 188 225T201 233L245 261Q283 284 291 291Q324 313 324 316L296 334Q280 343 259 357T224 380L210 390Q209 390 209 389",
      120203: "128 400Q127 401 121 422T108 478T99 540V555L111 569Q135 597 165 626T214 671T235 687L249 678Q263 668 282 659T315 650Q335 650 362 666L372 654L286 569H271Q205 576 173 586V583Q173 558 208 492T252 401Q253 399 310 399T367 398L332 355H254V311Q251 160 235 16Q230 -28 226 -36Q225 -38 221 -45Q171 -140 121 -211L113 -222H104Q94 -222 94 -220Q94 -215 105 -187L121 -145Q139 -80 139 35V93Q139 222 135 314L134 354Q134 355 84 355H35L84 399H106Q128 399 128 400",
      120204: "92 71Q92 74 91 88T88 128T86 183Q86 230 91 275T102 342T109 366Q115 372 207 422T305 472Q407 426 431 426Q435 426 476 445L519 465L525 463L532 461Q497 392 497 268Q496 255 496 233Q496 179 516 92T539 -10L541 -22L526 -38Q441 -126 355 -194L339 -206L327 -207Q324 -207 319 -207T310 -208Q242 -208 171 -179T73 -131L56 -141Q40 -150 38 -150Q17 -140 17 -137Q17 -136 18 -136T98 -79L176 -23Q174 -21 134 24T92 71ZM226 393Q224 393 221 372T214 312T210 235Q210 182 214 144L215 132L230 118Q281 70 301 66Q304 66 331 80T373 105L384 112L383 165Q383 224 387 309Q387 314 387 319T387 329T388 336T388 341V343Q388 344 381 344T339 354T249 384Q246 385 243 386T236 389T231 391T228 392L226 393ZM414 -80Q414 -64 411 -43T403 -1T394 37T386 66T382 79Q381 79 286 15T189 -52Q312 -125 365 -125Q397 -125 405 -115T414 -80",
      120205: "95 661Q95 662 103 667T113 672L126 634L137 596L147 602Q235 656 275 677L292 687L303 680Q305 679 307 677T312 674L313 672L310 670Q307 669 301 667T289 660T274 649T259 634Q250 622 244 611T233 585T226 560T222 528T221 497T220 456T219 413V377L232 384Q244 391 271 409T339 455L362 471L383 461Q425 440 491 415L504 410V406Q507 399 507 269Q507 76 486 -21Q485 -30 483 -33T461 -57Q382 -139 299 -207L281 -197L263 -186L266 -185Q268 -184 280 -177T312 -155Q344 -130 353 -116Q394 -59 394 117Q394 162 391 216T386 301T382 335Q382 338 365 346T323 364T281 376L250 362Q220 347 219 347Q213 336 213 232Q213 177 217 144L218 128L224 119Q244 92 263 71L272 60Q206 21 157 -24Q156 -24 151 -16T132 11T98 52L89 62L91 103Q104 289 104 436Q104 471 103 506T101 568T99 616T96 649L95 661",
      120206: "73 613L164 686L184 666Q200 650 214 637T235 620T242 614T203 577T162 540Q158 540 122 570T73 613ZM92 58Q92 63 94 83T98 142T101 234Q101 318 97 358V366L59 387L40 379L21 371Q20 371 12 376T3 382L38 406Q78 431 125 466L138 477Q149 468 186 444L219 422V389Q215 324 215 247Q215 136 222 123Q226 113 238 98T258 83Q263 83 292 94L322 104Q322 103 324 97T327 89Q327 88 317 82T272 52T190 -7Q166 -25 164 -25L112 35Q92 55 92 58",
      120207: "74 611L155 682Q172 666 186 655T208 636L235 614Q227 606 191 574L154 540L135 556Q101 582 84 601L74 611ZM10 377L144 477Q145 476 184 453T229 428L233 425V416Q238 346 238 252Q238 93 215 -16L213 -30L185 -57Q29 -203 19 -203Q17 -203 -19 -189L-9 -183Q52 -146 78 -116T114 -37Q120 31 120 192V237Q120 327 113 351T72 380L53 372Q34 362 32 364L10 377",
      120208: "106 72Q110 105 111 193T114 294V308H74L34 309L83 346H115V430Q114 591 106 652Q105 662 107 665T114 668T123 672Q125 672 139 635L152 597L154 598Q156 600 160 602T167 607Q193 625 226 644T279 672T302 682L312 676L321 670L312 665Q281 649 263 626T241 587T233 547Q232 541 231 530T230 510T230 501Q231 501 265 522T334 564T369 583L380 570Q428 509 428 481Q428 475 427 470T423 459T416 448T404 434T389 418T369 397T344 371L321 347L365 346H409L372 308H227V294Q227 272 230 208T234 138Q234 136 256 119T302 84L324 68L372 88Q421 108 422 108T432 90L421 83Q373 53 270 -5L234 -25L204 -1Q172 25 124 60L106 72ZM336 434Q336 452 327 472T308 503T297 514Q296 514 290 510T275 499T264 490Q230 458 230 358V346H247Q268 346 276 350T302 372Q328 398 335 423Q335 424 335 428T336 434",
      120209: "111 275Q111 406 108 518T104 650V657Q105 657 109 660T117 665T122 666L133 629L144 594L161 606Q218 642 272 670L294 681Q295 681 300 677T306 672L302 669Q298 666 292 662T278 651T263 637T251 621Q232 587 227 530T222 343Q222 226 230 125L231 112L244 98L258 83Q271 87 285 92L312 102V84Q297 72 231 24T163 -23L100 55Q110 141 111 275",
      120210: "115 203Q115 257 114 291T112 338T111 355Q111 357 93 370L75 384L54 375Q32 366 31 365Q27 365 16 378Q25 383 89 430L152 476Q175 453 228 420Q229 420 229 418T229 410T227 394L225 369Q279 400 315 425T363 461T376 471Q480 424 514 416V412Q514 411 514 404T513 392L511 376L520 382Q529 387 548 399T584 422Q599 432 618 444T648 463L657 469H658Q661 469 681 461T735 440T796 420Q803 418 803 416Q801 414 798 390T791 325T788 247Q788 220 790 172T794 123Q799 115 814 97T835 78H838Q841 78 867 89L895 101Q896 101 896 100T897 92T900 78L873 62Q810 23 761 -12L736 -30Q735 -30 729 -22T707 7T671 48L661 59Q674 93 674 207V219Q674 341 670 344Q655 353 591 372L576 376L544 364Q511 351 510 351Q507 349 507 224V132L535 95Q541 87 548 78T560 63L563 58Q563 57 504 15T444 -28L385 53L387 67Q396 114 396 206Q396 289 393 334Q393 346 390 348Q369 358 306 373Q301 373 265 361L228 349V335Q227 322 227 284Q227 206 231 157Q231 151 231 144T232 133V129Q232 125 259 90Q286 56 286 53Q287 53 284 51T273 43T258 31L173 -31L166 -20Q160 -11 145 7T119 38T108 59Q108 62 110 81T113 133T115 203",
      120211: "608 88Q572 65 535 37T477 -8T455 -25Q432 7 389 53L375 68L378 82Q386 160 386 195V221Q386 284 385 307L384 344Q352 359 306 373L286 379L213 353V273Q214 229 214 161V129L275 62L163 -28L150 -14Q136 0 121 16T91 44Q86 48 86 50Q95 83 96 148Q96 224 89 340L88 366L79 374Q69 384 67 385L64 388L55 383Q52 382 44 378T33 373L21 367L13 374Q5 379 5 381Q5 384 69 428L133 473Q135 473 147 464T179 443T215 424L214 400V376Q271 404 342 457L363 472Q363 473 364 473Q366 473 375 469T418 449T502 414L512 411V407Q502 330 502 217V197V132L523 109Q527 104 533 97T543 87T547 83L550 80L578 92Q603 103 604 103Q606 103 608 88",
      120212: "107 102Q107 178 112 242T123 334T129 362Q129 363 140 368T199 400T315 469L336 482L346 476Q409 439 498 414L514 410L515 389Q515 208 502 141Q494 101 491 94Q490 89 478 81Q430 51 375 23T288 -20T254 -34Q250 -34 200 -1T119 56L108 65L107 76V102ZM389 355Q367 358 346 363T309 372T282 381T264 388L257 390H256Q254 390 249 381T238 348T227 293Q226 280 226 237Q226 183 231 146L232 131L244 122Q285 91 323 74T374 57H377L380 68Q405 154 405 267Q405 315 401 349V354L389 355",
      120213: "66 435Q66 445 117 501T173 557Q174 557 183 555T193 551Q174 526 174 509Q174 496 190 472T233 428V386L377 482L399 471Q450 445 509 425Q519 421 519 420L518 419Q518 418 518 416T517 410Q517 405 518 381T519 335Q519 222 501 137Q492 84 489 84L473 75Q457 66 423 44T354 -6L338 -19L329 -13Q320 -8 313 -4T297 4T284 10T270 14T258 17T245 20T233 22V12L241 -161L214 -172Q187 -184 160 -195T131 -207Q127 -207 112 -202L113 -188Q113 -182 115 -77T118 31Q118 32 109 32Q63 27 23 0L10 -9Q5 -4 -1 8Q1 13 52 57T114 101H115L117 123Q117 141 117 230V359L110 367Q85 394 71 421Q66 433 66 435ZM384 83Q386 83 389 110T396 180T400 254Q400 294 395 339L394 349L379 355Q308 383 294 383Q290 383 263 372L234 360L233 245V130Q270 125 305 113T361 92T384 83",
      120214: "362 -196Q375 -92 375 47V78L282 24Q189 -29 188 -30Q187 -30 139 21T90 75Q87 84 87 158Q88 206 94 259T107 342L113 372L308 478L322 473Q374 452 421 444L433 442L503 485Q515 479 515 477Q485 378 485 56Q485 -100 494 -164V-171L381 -211L371 -207L362 -202V-196ZM280 72Q301 77 323 86T358 101T372 110Q372 268 377 346L378 358H374Q368 360 358 360T323 365T257 380L234 386Q231 386 229 379Q215 353 211 310T207 180Q207 152 208 150Q210 142 235 114T280 72",
      120215: "23 367Q21 370 18 374T14 380L13 382L151 472L236 411L238 381L290 426Q298 432 307 439T322 452T333 461T342 467L344 469Q382 410 404 399Q410 397 416 397Q423 397 432 399T446 403L451 405Q453 405 453 399V393Q430 374 404 356T364 328T350 318L349 317Q321 320 276 356Q257 371 256 371Q253 374 249 366T242 351Q232 321 232 236Q232 214 232 205T232 182T233 162T235 148T238 137T242 129T249 120T257 114T268 105T281 95Q313 70 314 70L358 85Q377 92 389 96T402 100V90L403 80L229 -26L221 -18Q195 6 166 29T121 63T105 76T106 82T110 97T114 121T117 158T119 208Q119 269 114 329L113 341L103 350Q90 362 67 380L45 374L23 367",
      120216: "189 331Q190 304 196 282T207 252T214 244Q239 244 348 292L371 302L382 297Q398 290 415 279T433 265Q442 238 442 166Q442 103 423 45Q416 42 380 29T310 3T244 -26L227 -34Q139 40 73 40Q61 40 48 37T24 30T6 22T-8 14L-13 11Q-14 11 -18 18T-23 26T38 75T102 125Q107 128 146 131H153Q192 131 296 56Q318 40 318 43Q323 48 323 114Q323 157 321 177L319 194Q308 208 291 216T261 225Q239 225 160 185L123 167Q85 205 79 227Q78 230 78 304V377L171 428Q264 479 265 478Q268 478 287 465T334 440T384 427Q423 427 475 463L478 453Q481 446 481 442Q481 439 410 391L339 342H331Q309 345 277 361T222 391T198 406T195 399T191 372T189 331",
      120217: "328 69Q401 102 403 102Q404 102 405 94T406 84Q406 83 318 28L230 -27Q223 -21 206 -5T171 25T132 54L124 60V71Q129 154 129 297V359H43L44 363Q44 365 44 367L45 369L48 372Q51 374 57 378T68 387L90 405H129V553L285 648Q304 641 306 640L260 598V592Q259 589 255 505T249 413V405H353V402Q353 399 328 379L303 360H245V319Q245 150 253 125Q257 115 276 101T311 78T328 69",
      120218: "444 -31Q444 -29 384 66Q382 66 364 58T309 30T231 -17Q214 -29 212 -29L197 -20Q172 -4 140 11T88 34L68 42Q68 43 73 49T85 67T100 98T113 149T118 221Q118 272 105 332L100 356L58 383L23 365L9 379L76 425Q141 472 144 472Q144 471 183 443L221 414V404Q224 365 224 275V253Q224 159 196 113Q191 104 193 104Q203 104 285 72L308 62L374 89L375 106Q375 266 373 340Q373 364 371 396V424L430 445L491 467Q493 467 499 463T505 457Q505 456 503 442Q488 335 488 187V158L529 81L534 80Q541 80 568 90L598 101Q605 94 602 87L524 27Q445 -32 444 -31",
      120219: "95 67Q104 80 104 193Q104 261 100 321L98 355L91 363Q56 402 56 421Q56 441 82 472T132 524T159 546Q174 542 175 542Q159 520 159 501Q159 481 205 432L221 415L220 401Q219 394 219 387L288 429Q309 441 325 451T347 465T358 472T365 476L504 415V409Q504 408 505 374T507 318Q507 155 474 91L469 80L343 26Q314 14 281 0T232 -20L216 -27L202 -15Q192 -5 152 28Q141 35 126 45T103 60T95 67ZM386 349Q302 389 287 389Q271 383 253 375L220 361V136Q226 120 256 100T312 68T342 56Q355 56 360 68Q389 134 389 258Q389 310 386 341V349",
      120220: "90 58T90 59T92 64T97 78T102 105T107 150T109 218Q109 290 103 350V356L83 377Q55 407 55 425Q55 445 138 528Q158 549 162 549L164 548Q165 548 167 548T170 547L175 546L172 540Q168 533 165 523T161 502Q161 479 216 430L229 419V382Q232 382 366 471Q407 445 500 408L511 404V387L512 370L595 420Q678 469 679 469L693 462Q756 431 795 417L815 409L814 380Q812 187 782 96Q774 71 766 62T744 48T684 25T577 -23L557 -32L546 -26Q536 -19 519 -10T481 10T436 31T393 47Q384 50 380 50Q380 52 381 58T384 77T387 104Q391 174 391 256V292L390 333L377 340Q350 357 304 373L294 376L227 355V348Q224 322 224 243Q228 117 232 112L235 108Q238 103 245 95T257 80L281 50Q281 49 227 10T172 -29L159 -13Q133 19 116 36T94 56ZM652 64Q658 64 667 84T685 162T697 303V336L686 341Q653 356 619 367L591 376Q590 376 553 361T514 344T512 324T510 275T508 221Q508 167 510 152T521 126Q537 112 590 88T652 64",
      120221: "8 -90Q8 -68 13 -63Q13 -56 53 -8T120 63L128 71L129 85Q133 120 134 182Q134 308 131 331T106 365Q100 367 97 369L75 381L35 365L20 377Q20 378 47 397T110 440T161 471L253 413V396Q253 378 254 378L309 422Q364 466 365 466Q365 467 366 466T370 461T376 454Q403 419 426 396L441 380L438 377Q438 376 433 372T420 359T404 344L372 314Q351 320 338 327T310 344T277 364Q261 364 252 316Q251 306 251 235Q251 136 255 129Q257 127 258 124T268 113T298 92Q334 68 335 68Q340 70 349 73T377 84T408 95T421 99Q422 99 422 90L423 82L334 26Q246 -28 243 -28L200 8Q156 43 148 43Q144 43 130 36T99 9T83 -36Q83 -67 121 -89T198 -118L237 -124V-129L238 -133L193 -160Q183 -166 171 -173T152 -184L146 -188Q140 -187 131 -185T98 -173T56 -154T23 -127T8 -90",
      120222: "280 53Q272 47 246 27T199 -10T176 -27L167 -18Q137 17 107 44L90 60L93 71Q108 130 109 290V331Q109 339 109 344T108 353T107 359T105 364T102 369T97 374T91 381Q60 412 60 432Q60 448 86 479T138 534L164 557Q168 553 180 553Q163 532 163 511Q165 491 186 468Q206 443 231 423V404L232 385L371 477L389 468Q439 441 498 418L512 412V386Q512 360 507 190T500 14Q488 -26 445 -67Q401 -111 355 -148T282 -203T249 -221Q247 -220 230 -210T213 -199T229 -191T269 -172T306 -151Q361 -120 379 14Q391 92 391 182Q391 218 386 305Q384 339 380 341Q363 353 330 366T288 379Q282 379 258 368L230 356V181V141Q230 127 232 120T236 108T251 89T275 59L280 53",
      120223: "153 371Q141 371 126 365T100 354T78 340L65 331L57 338L50 346L62 356Q133 419 222 471Q274 453 306 422T338 366Q338 356 329 346T283 301L243 264L262 257Q298 246 361 214Q378 154 378 73Q378 33 371 -9T356 -74T345 -104Q340 -106 267 -160L191 -214H177Q60 -214 13 -150Q-7 -122 -7 -115Q-7 -112 19 -77T106 25T241 149Q241 152 227 158T181 173T109 185V190L108 194L158 229Q212 267 223 278T234 306Q234 329 208 350T153 371ZM258 1Q258 42 257 68T254 105T252 118Q235 105 210 85T144 22T102 -45Q102 -79 146 -106T234 -133H238Q248 -128 254 -80Q258 -58 258 1",
      120224: "183 181Q183 179 152 91T118 0H28L154 346L280 693Q281 694 333 694H385L511 349Q636 4 638 2Q638 0 584 0H530L464 183H184L183 181ZM324 606Q319 578 292 492T238 332T210 256Q210 254 324 254T438 255L429 281L419 308Q409 336 395 378T365 465T339 551T324 611V606",
      120225: "425 363Q438 363 465 353T526 324T585 270T610 192Q610 132 561 78T426 7Q404 2 387 2T240 0H90V694H227Q373 693 396 689Q484 673 533 623T583 517Q583 494 574 473T551 437T520 409T487 388T456 374T433 366L425 363ZM490 516Q490 527 485 539T467 568T423 599T347 621Q340 622 262 623H188V399H261H286Q432 399 478 475Q490 496 490 516ZM514 190Q514 245 462 280T343 322Q336 323 259 323H188V71H274Q365 72 388 77Q445 88 479 121T514 190",
      120226: "59 347Q59 440 100 521T218 654T392 705Q473 705 550 680Q577 670 577 667Q576 666 572 642T564 595T559 571Q515 601 479 613T392 626Q300 626 232 549T164 347Q164 231 229 150T397 68Q453 68 489 80T568 120L581 129L582 110Q584 91 585 71T587 46Q580 40 566 31T502 5T396 -11Q296 -11 218 41T99 174T59 347",
      120227: "88 0V694H237H258H316Q383 694 425 686T511 648Q578 604 622 525T666 343Q666 190 564 86Q494 18 400 3Q387 1 237 0H88ZM565 341Q565 409 546 463T495 550T429 600T359 621Q348 623 267 623H189V71H267Q272 71 286 71T310 70Q461 70 527 184Q565 251 565 341",
      120228: "86 0V691H541V611H366L190 612V397H513V321H190V85H372L554 86V0H86",
      120229: "86 0V691H526V611H358L190 612V384H485V308H190V0H86",
      120230: "59 346Q59 499 157 601T384 704Q436 704 466 700T541 679Q551 674 560 670T575 664T583 660T588 658T590 656Q590 652 582 605T573 557L564 564Q489 626 392 626Q301 626 233 549T164 347T233 145T392 68Q441 68 506 84V223H388V299H599V38L588 33Q494 -11 393 -11Q296 -11 219 40T100 172T59 346",
      120231: "86 0V694H190V399H517V694H621V0H517V323H190V0H86",
      120232: "87 0V694H191V0H87",
      120233: "181 53Q200 53 215 56T241 66T259 79T272 95T280 109T285 122L287 129V694H388V415V229Q388 135 385 112T369 63Q364 51 355 39T328 12T280 -12T212 -22Q172 -22 130 -12T66 8T43 20L46 42Q50 65 54 88L58 110Q58 111 65 104Q107 53 181 53",
      120234: "88 0V694H188V519L189 343L525 694H638L375 419L651 0H541L309 351L188 225V0H88",
      120235: "87 0V694H191V79L297 80H451L499 81V0H87",
      120236: "92 0V694H228L233 680Q236 675 284 547T382 275T436 106Q446 149 497 292T594 558L640 680L645 694H782V0H689V305L688 606Q688 577 500 78L479 23H392L364 96Q364 97 342 156T296 280T246 418T203 544T186 609V588Q185 568 185 517T185 427T185 305V0H92",
      120237: "88 0V694H235L252 659Q261 639 364 428T526 84V694H619V0H472L455 35Q453 39 330 294T185 601L181 611V0H88",
      120238: "55 345Q55 504 149 609T361 715Q386 715 406 713Q521 696 600 592T680 344Q680 193 590 86T368 -22Q239 -22 147 84T55 345ZM276 59T368 59T518 146T576 360Q576 473 525 545T401 634Q371 637 362 637Q284 637 222 562T159 360T217 147",
      120239: "88 0V694H230Q347 693 370 692T410 686Q487 667 535 611T583 485Q583 409 527 348T379 276Q369 274 279 274H192V0H88ZM486 485Q486 523 471 551T432 593T391 612T357 621Q350 622 268 623H189V347H268Q350 348 357 349Q370 351 383 354T416 368T450 391T475 429T486 485",
      120240: "55 345Q55 504 149 609T361 715Q386 715 406 713Q521 696 600 592T680 344Q680 284 665 231T629 143T587 85T551 48L536 35L648 -120L652 -125H531L452 -8L440 -12Q407 -22 369 -22Q239 -22 147 85T55 345ZM579 345Q579 473 517 555T369 637Q279 637 218 554T156 345Q156 223 215 141T368 58Q376 58 382 58T392 58T397 59T401 60T403 61H404Q404 63 360 128T315 194H421L453 150Q485 105 486 105Q490 108 496 113T517 138T545 182T567 247T579 334V345",
      120241: "88 0V694H227H259H302Q365 694 399 689T474 663Q528 637 558 595T589 504Q589 482 584 462T569 426T547 396T522 372T495 353T470 338T449 328T434 322L429 320L440 300Q452 280 477 238T523 160L617 1L565 0Q513 0 512 1Q512 2 424 156L337 309H189V0H88ZM492 504Q492 600 367 620Q354 622 271 623H189V385H271Q363 386 388 392Q432 402 462 430T492 504",
      120242: "55 514Q55 589 115 652T283 716Q315 716 345 711T396 699T432 685T457 672T467 667Q467 666 459 618T449 568Q383 634 282 634Q214 634 182 600T150 525Q150 507 155 492T172 465T194 446T222 432T247 423T272 416T289 412Q353 396 378 384Q432 358 466 307T500 194Q500 110 438 44T272 -22Q215 -22 159 -5T73 28T44 50Q45 51 49 75T57 122T62 146L65 143Q68 140 74 136T88 125T107 111T131 98T160 85T194 74T232 66T274 63H286Q327 63 366 96T406 182Q406 245 352 280Q329 296 265 310T173 339Q124 363 90 409T55 514",
      120243: "36 608V688H644V608H518L392 609V0H288V609L162 608H36",
      120244: "87 450V694H191V449Q192 203 193 194Q200 148 220 117T266 72T311 54T347 49Q404 49 446 84T501 178Q505 195 505 218T507 449V694H600V450Q600 414 600 356Q599 198 595 181Q594 178 594 177Q575 89 505 34T345 -22Q258 -22 184 34T89 196Q88 205 87 450",
      120245: "14 692Q14 694 68 694H122L146 633Q325 165 339 90Q340 87 341 87Q341 124 530 619L558 694H605Q652 694 652 692Q650 690 523 354T390 10L387 0H279L276 10Q271 18 144 354T14 692",
      120246: "115 694Q115 693 156 550T233 266T270 90L271 85Q272 86 272 92Q272 153 405 616L427 694H524L553 590Q672 174 681 95L682 84L684 95Q689 138 728 287T803 563T841 692Q841 694 885 694T929 693Q929 691 829 346L730 0H679L628 1L606 75Q478 524 470 600L469 611L467 600Q458 518 338 101L310 0H213L114 346Q14 691 14 693Q14 694 64 694H115",
      120247: "14 0Q16 5 144 184T275 367L153 528Q121 571 88 615T42 674T28 694H150L228 584Q315 463 316 461L326 448L497 694H610L609 692Q606 689 492 528Q440 454 409 410T378 366Q378 365 515 182L652 0H531L326 292Q326 293 299 254T226 146L128 0H14",
      120248: "4 693L64 694H125L174 621Q335 378 340 364L341 362Q361 398 395 450L558 694H663L383 277V0H282V278L143 485Q112 531 75 586T21 668L4 693",
      120249: "69 617V694H555V643L373 362Q190 81 190 79H234Q244 79 272 79T344 80T419 81H560V0H55V53L237 334Q420 615 420 617Q413 618 387 618Q380 618 334 618T245 617H69",
      120250: "236 387Q209 387 184 382T141 370T111 355T91 342T83 337L82 355Q80 373 79 393T77 417Q77 419 81 421Q86 423 91 426Q155 460 227 460H238Q319 460 368 400Q393 371 400 341T408 252Q408 240 408 207T407 152V0H317V39L306 32Q244 -10 159 -10H152Q109 -10 77 22Q38 61 38 126Q38 142 39 146Q55 199 130 223T295 252H314V277Q314 305 313 310Q308 342 287 364T236 387ZM303 186Q124 180 124 126Q124 105 144 86T208 66Q284 66 309 124Q314 137 314 166V186H303",
      120251: "303 -11Q280 -11 259 -6T222 6T194 21T176 33T168 38V0H75V694H165V550L166 405Q247 455 336 455Q397 455 439 389T482 226Q482 115 428 52T303 -11ZM390 221Q390 283 361 331T265 379Q214 379 177 342L168 334V118Q203 66 258 66Q316 66 353 106T390 221",
      120252: "34 223Q34 327 99 393T245 460Q290 460 301 459Q328 455 354 445T395 427T410 415L396 338L386 344Q377 349 362 357T335 370Q305 381 258 381H252Q182 381 146 315Q126 275 126 224Q126 158 159 113T255 68Q329 68 394 106L408 114L410 93Q411 72 412 52L414 32Q407 27 394 20T338 2T252 -10Q156 -10 95 58T34 223",
      120253: "33 224Q33 321 81 388T197 455Q277 455 342 414L351 408V694H441V0H348V44L338 37Q278 -10 198 -10Q177 -10 168 -8Q99 11 62 90Q33 148 33 224ZM348 337Q307 378 263 378Q260 378 256 378T251 379Q239 379 223 374T182 355T142 305T126 220Q126 90 225 67Q231 66 250 66H255Q306 66 342 115L348 124V337",
      120254: "28 226Q28 329 91 395T235 461Q258 461 279 456T325 436T368 397T399 332T415 238V219H113V215Q113 163 151 114T248 65Q273 65 298 70T341 82T373 96T396 108L403 113Q403 106 406 76T409 38Q409 34 408 33T393 24Q325 -10 252 -10Q155 -10 92 59T28 226ZM340 289L338 297Q335 305 333 310T327 326T317 343T304 358T286 372T263 381T233 385Q212 385 193 376T162 353T140 325T127 301T123 289H340",
      120255: "262 705H267Q300 705 347 694V612L336 616Q303 628 274 628H266Q224 628 199 605Q187 590 184 579T181 541V507V444H287V371H184V0H94V371H27V444H94V492Q94 544 95 550Q102 617 151 661T262 705",
      120256: "55 286Q55 357 105 406T224 455Q280 455 323 421L322 423L318 427Q318 428 339 434T396 448T465 455H471L478 416L485 377Q484 377 474 379T445 383T401 385Q397 385 391 385T381 384L362 383L357 387Q358 386 364 375T375 354T384 325T389 287Q389 217 340 168T221 119Q178 119 138 142Q133 145 131 143Q125 131 125 117Q125 82 155 72L227 71Q230 71 251 71T280 71T310 69T343 65T373 57T403 46T428 30T449 7Q471 -26 471 -62V-71Q471 -136 384 -178Q326 -206 250 -206Q159 -206 102 -172T30 -92Q28 -84 28 -68T31 -37T40 -12T52 7T64 21T75 31T82 38Q60 68 60 106Q60 145 80 180L86 189L80 199Q55 240 55 286ZM304 233T304 287T279 362T220 383Q189 383 165 361T140 287Q140 243 161 217T220 191Q253 191 278 212ZM250 -134Q298 -134 331 -122T375 -96T387 -69Q387 -21 306 -7Q288 -5 216 -5Q161 -5 153 -7Q146 -9 139 -13T122 -31T113 -66Q113 -75 113 -80T127 -97T166 -121Q203 -134 250 -134",
      120257: "163 395Q223 455 307 455Q417 455 438 354Q442 331 443 164V0H350V157Q349 315 348 320Q334 378 259 378H253Q224 378 204 358Q180 334 173 301T165 209Q165 198 165 172T166 129V0H73V694H163V395",
      120258: "67 576V680H171V576H67ZM74 0V444H164V0H74",
      120259: "88 576V680H192V576H88ZM31 -126Q40 -126 48 -125T62 -122T73 -117T82 -111T89 -105T94 -99T98 -92L102 -86V444H192V180Q191 -45 191 -70T184 -113Q171 -152 140 -178T63 -205Q34 -205 4 -197T-43 -181T-59 -171T-51 -133T-41 -96L-38 -99Q-34 -102 -28 -106T-13 -115T7 -123T31 -126",
      120260: "76 0V694H163V257L340 444H449L286 272L292 263Q296 259 378 138T463 12L471 0H372L309 92Q294 114 277 139T250 179T237 198L228 211L160 139V0H76",
      120261: "74 0V694H164V0H74",
      120262: "160 392Q223 455 304 455Q359 455 386 436T430 383L437 391Q495 455 584 455Q694 455 715 354Q719 331 720 164V0H627V157Q626 315 625 320Q611 378 536 378H530Q501 378 481 358Q457 334 450 301T442 209Q442 198 442 172T443 129V0H350V157Q349 315 348 320Q334 378 259 378H253Q224 378 204 358Q180 334 173 301T165 209Q165 198 165 172T166 129V0H73V450H160V392",
      120263: "160 392Q214 446 283 454Q285 454 292 454T303 455H306Q417 455 438 354Q442 331 443 164V0H350V157Q349 315 348 320Q334 378 259 378H253Q224 378 204 358Q180 334 173 301T165 209Q165 198 165 172T166 129V0H73V450H160V392",
      120264: "28 222Q28 323 95 391T244 460Q275 460 281 459Q364 445 417 377T471 219Q471 124 408 57T250 -10Q158 -10 93 57T28 222ZM377 230Q377 277 364 310T328 358T287 379T248 385Q233 385 219 382T186 369T155 342T132 297T122 230Q122 146 159 108T250 69H253Q263 69 274 70T305 81T339 106T365 154T377 230",
      120265: "166 404Q194 424 241 439T337 455H341Q410 455 451 370Q483 307 483 222Q483 128 433 59T306 -10Q282 -10 260 -5T222 7T194 21T176 33T168 38V-194H75V444H165V424L166 404ZM390 222Q390 287 354 331T266 376T177 340L168 332V118Q200 66 257 66Q313 66 351 112T390 222",
      120266: "33 220Q33 325 87 389T206 454Q286 454 341 406L351 398V455H441V-194H348V41L338 35Q276 -8 198 -11Q171 -11 154 -5Q102 12 68 74T33 220ZM126 220Q126 160 161 113T251 65Q305 65 351 123V298L350 301Q349 304 347 308T342 319T336 331T327 343T315 355T300 365T283 373Q273 375 256 375Q208 375 167 332T126 220",
      120267: "171 389Q237 455 320 455H327V373H317Q262 369 220 336T167 248Q165 239 164 119V0H74V450H159V377L171 389",
      120268: "33 326Q33 376 60 408T117 450T175 460H190Q245 460 272 454T345 429Q345 428 338 388L331 349Q278 386 188 386H183Q119 386 119 336Q119 307 142 295T210 276T278 256Q360 213 360 130Q360 108 354 88T332 43T281 5T195 -10Q152 -10 111 1T49 22T28 35Q29 36 32 56T39 95T43 115T51 110T72 99T102 85T143 73T193 68Q274 68 274 123Q274 152 248 167Q234 178 187 186T115 207Q66 229 46 273Q33 298 33 326",
      120269: "333 27Q333 24 314 16T257 -1T184 -10H172Q146 -10 128 14T105 58T99 91Q95 113 95 251V371H18V444H98V571H182V444H316V371H182V253Q183 128 189 104Q199 68 234 68Q277 72 314 98Q315 93 323 61T333 27",
      120270: "353 39Q352 38 344 34T331 27T315 19T295 11T270 3T241 -3T207 -8T168 -10H162Q148 -10 137 -8T111 2T87 30T74 81Q73 89 73 268V444H166V268L167 92Q171 79 174 74T192 64T238 59Q317 59 344 116Q349 128 349 148T350 291V444H443V0H353V39",
      120271: "178 0Q15 441 14 442Q14 444 60 444Q107 444 107 442Q108 441 136 364T196 194T232 67Q233 98 280 234T356 442Q356 444 401 444T446 442L282 0H178",
      120272: "14 444H105L108 434Q192 160 200 74V65L201 75Q206 144 282 399L296 444H381L399 381Q480 112 480 69L481 70Q481 113 562 386L580 444H668L534 1L483 0H432L429 10Q343 294 338 367L337 377Q336 375 336 370Q336 340 313 250T269 88T245 11L242 0H195L148 1L14 444",
      120273: "187 229L6 444H107L227 294L344 444H393L442 443Q439 437 299 268L267 229L460 0H359L294 88Q280 107 262 131T236 166L227 177L100 0H0Q1 1 47 58T140 171T187 229",
      120274: "113 -204Q83 -204 63 -200L43 -197Q43 -196 40 -157T36 -117L48 -121Q79 -133 114 -133Q124 -133 130 -132T145 -121T163 -94Q169 -82 184 -42T200 1L188 29Q176 57 152 115T107 223T62 330T26 416L14 443Q14 444 61 444H109L122 411Q230 155 236 75L237 65V74Q245 161 348 424L356 444H401Q446 444 446 443L396 313Q345 183 293 49T236 -93Q191 -204 123 -204H113",
      120275: "42 370V444H400V395L156 76L279 77H402V0H28V51L273 371L157 370H42",
      120276: "110 0H86Q42 0 42 27Q42 37 148 350T258 667Q269 687 291 692Q295 694 366 694H399Q432 694 448 689T474 667Q477 663 583 350T690 27Q690 0 642 0H617H592Q582 0 575 1T561 2T549 6T541 11T533 18T527 26T522 37T517 49T512 64T506 81L490 130H225Q225 128 208 79T189 27Q185 19 180 14T170 7T156 3T143 1T127 0T110 0ZM439 279Q359 524 359 547L357 555L355 543Q347 503 270 263L259 231H357Q455 231 455 232L439 279",
      120277: "119 1Q98 5 92 28V667Q98 686 118 693Q121 694 272 694H289H346Q439 694 500 681T600 625Q640 580 640 513Q640 451 601 414T504 364L518 361Q568 351 602 329T649 280T666 235T671 197Q671 172 665 147T642 91T586 37T488 5Q456 1 282 1H119ZM489 509Q489 532 479 548T450 573T421 585T394 591Q387 592 315 593H247V404H298H325Q432 404 466 444Q489 470 489 509ZM517 194Q517 235 502 261T458 299T407 313T353 317H329H322H247V101H319H357Q387 101 407 103T452 111T492 133T514 171Q516 176 517 194",
      120278: "423 -11Q339 -11 275 9T171 62T106 143T71 240T61 347Q61 450 93 527Q157 664 313 694Q357 704 416 704Q479 704 517 699T608 676Q634 667 635 660Q635 653 624 592L612 528L609 524Q604 521 601 521Q595 521 583 531T555 555T505 578T428 589H424Q298 589 250 494Q224 438 224 347Q224 292 233 251T265 175T329 122T432 104Q488 104 524 115T604 158Q607 160 610 162T615 165T619 168L621 170Q625 172 630 170T637 163Q638 160 642 109T647 54Q646 49 625 37T568 11T499 -7Q463 -11 423 -11",
      120279: "119 1Q98 5 92 28V667Q98 686 118 693H124Q131 693 142 693T168 694T200 694T237 694H296Q416 694 450 692T525 677Q732 617 732 342Q732 169 644 81Q593 32 528 16T372 0Q356 0 324 0T276 1H119ZM573 349Q573 387 571 413T559 473T532 527T482 567T403 591Q395 592 320 593H250V101H321Q418 102 456 114Q553 144 569 263Q573 303 573 349",
      120280: "277 122Q280 122 380 123T544 125Q552 125 557 125T565 124T569 124Q595 115 595 75V62V47Q595 9 569 2Q564 0 341 0L119 1Q99 7 92 28V664Q98 683 118 690Q121 691 335 691T554 689Q580 682 580 644V632V618Q580 582 554 573Q553 573 551 573T542 572T527 572Q464 572 364 573T260 575H253V412H385H459Q524 412 536 404T549 357Q549 341 549 334T542 318T523 305Q518 303 385 303H253V122H277",
      120281: "512 572Q451 572 356 573T258 575H253V400H370H431Q494 400 506 392T518 345Q518 307 507 299T437 291H370H253V161Q253 141 253 113T254 75Q254 23 245 12T195 0H170L119 1Q99 7 92 28V664Q98 683 118 690Q121 691 327 691T538 689Q564 682 564 644V632V618Q564 582 538 573Q537 573 535 573T526 572T512 572",
      120282: "61 347Q61 405 70 454T105 550T171 631T276 685T426 705Q483 705 537 693T620 668T650 646Q650 645 649 637T645 612T639 578L627 514L624 510Q620 507 615 507T597 520T566 548T512 577T430 590Q223 590 223 347T431 104Q478 104 506 112Q508 112 508 164V215H471L434 216L428 222L427 268Q427 315 429 318Q432 323 444 323T544 324H652Q655 320 659 317V45L656 43Q654 39 624 27T536 2T424 -11Q366 -11 317 -2T219 33T137 97T82 200T61 347",
      120283: "92 667Q101 694 143 694H172H198Q244 694 251 669Q253 663 253 539V415H540V539Q540 558 540 585T539 621Q539 673 550 683T611 694H621H646Q671 694 683 690T700 669Q702 663 702 347T700 25Q696 9 684 5T646 0H621H606Q560 0 550 11T539 76Q539 85 539 116T540 169V306H253V169Q253 147 253 116T254 75Q254 23 245 12T194 0H170L119 1Q99 7 92 28V667",
      120284: "85 667Q94 694 136 694H165H191Q237 694 244 669Q246 663 246 347T244 25Q235 0 192 0H163L112 1Q92 7 85 28V667",
      120285: "236 -22Q190 -22 144 -11T72 12T46 29Q63 147 69 153Q80 164 92 146Q124 91 191 91Q222 91 242 102T267 134Q268 139 268 402Q268 663 270 669Q275 687 294 692Q298 694 347 694H367Q393 694 406 690T425 669Q427 663 427 399Q427 132 426 125Q421 87 404 58T366 15T318 -9T273 -20T236 -22",
      120286: "92 667Q101 694 139 694H163H186Q225 694 234 671Q236 663 236 529L237 392L533 682Q550 694 590 694H623H681Q695 680 695 672Q695 670 693 664Q688 657 561 533L431 405L698 33Q701 28 701 23Q701 7 683 0H626H604Q571 0 564 2T545 13Q544 14 530 33T489 90T437 162L332 307Q331 307 284 260L236 214V122V65Q236 32 231 19T210 2Q205 0 161 0L119 1Q99 7 92 28V667",
      120287: "92 667Q98 684 109 689T142 694H172H198Q244 694 251 669Q253 663 253 389V116L278 117Q410 119 490 119H495Q511 119 517 115T534 93V63V48Q534 9 508 2Q503 0 310 0L119 1Q99 7 92 28V667",
      120288: "92 667Q98 684 109 689T146 695Q152 695 167 695T192 694Q200 694 214 694T234 695Q291 695 305 664Q313 651 400 419T487 165Q487 162 488 162T489 165Q489 187 574 413T671 664Q679 680 695 688Q708 694 785 694H828Q855 694 867 689T884 669Q886 663 886 347T884 25Q876 0 832 0H817H802Q758 0 750 25Q748 31 748 293V555L746 544Q737 509 692 386T606 160T564 52Q548 22 502 22H487H472Q423 22 410 52Q407 59 367 160T283 385T231 546L230 548Q229 548 229 293Q229 31 227 25Q222 9 211 5T176 0H158L119 1Q99 7 92 28V667",
      120289: "92 667Q98 684 109 689T146 694H185Q273 694 279 692Q301 689 315 669Q322 660 419 453L554 163L562 143Q564 143 564 401Q564 663 566 669Q574 694 618 694H633H648Q692 694 700 669Q702 663 702 347T700 25Q696 10 683 5T642 0H596H551Q520 0 505 4T478 25Q471 34 374 241L239 532Q231 550 231 552L229 479Q229 440 229 293Q229 31 227 25Q222 9 211 5T176 0H158L119 1Q99 7 92 28V667",
      120290: "362 715Q364 715 376 715T394 716H400Q542 716 626 643T727 426Q731 395 731 342Q731 271 722 225Q674 -22 396 -22Q320 -22 259 -3T148 68T77 201Q62 257 62 342Q62 447 86 522T173 649Q245 707 362 715ZM568 433Q551 623 396 623Q383 623 370 622T333 612T292 591T257 550T233 485Q223 442 223 350Q223 276 232 227T267 137Q309 74 397 74Q433 74 461 85T507 113T537 156T556 205T566 260T569 310T570 357Q570 409 568 433",
      120291: "641 470Q641 426 630 391T603 334T561 295T513 271T459 259T408 254T361 253H350H337H253V142Q253 125 253 100T254 67Q254 32 249 19T227 2Q222 0 170 0L119 1Q99 7 92 28V667Q98 686 118 693Q121 694 271 694Q428 693 462 688Q641 656 641 470ZM487 467Q487 495 485 510T474 546T442 578T382 592Q375 593 310 593H250V347H309H339Q364 347 380 348T418 354T451 368T474 395T486 438Q487 444 487 467",
      120292: "450 -20Q444 -20 429 -21T396 -22Q320 -22 259 -3T148 68T77 201Q62 257 62 342Q62 447 86 522T173 649Q245 707 362 715Q364 715 376 715T394 716Q732 716 732 340Q732 268 719 210T686 120T647 68T615 39T601 29T638 -22T676 -73Q679 -78 679 -83Q679 -98 661 -106H593Q526 -106 521 -104Q514 -103 507 -97T496 -84T477 -55L454 -19L450 -20ZM554 509Q516 622 391 622Q294 622 250 535Q220 475 220 345Q220 299 222 266T234 198T258 140T299 99T363 74Q378 71 393 71H395L381 92Q367 114 353 136T338 161Q336 165 336 170Q336 186 352 193L361 194Q370 194 384 194T412 194H452Q457 194 460 194T466 194T471 192T476 191T480 188T483 185T487 180T492 174T497 167T504 158L526 129Q532 127 552 175Q573 231 573 348Q573 455 554 509",
      120293: "654 24Q654 9 644 5T612 0H577L521 1Q509 5 503 13Q498 20 421 160L343 304H250V168Q250 147 250 118T251 78Q251 24 242 12T192 0H168L119 1Q99 7 92 28V667Q98 686 118 693H124Q131 693 141 693T165 694T195 694T229 694T280 694T332 695Q389 695 428 691T510 675T582 637T627 569Q641 532 641 493Q641 377 537 331L497 317L493 316L571 177Q653 28 654 24ZM487 472T487 492T485 525T476 553T450 577T404 591Q398 592 322 593H250V391H321Q327 391 353 391T385 392T412 395T438 401T457 412T474 430T483 456",
      120294: "61 503Q61 547 72 583T110 650T186 698T305 716Q405 716 496 671Q513 664 514 657Q514 656 513 648T509 623T503 589L491 525L488 521Q484 518 479 518H475L461 532Q430 565 395 581T305 598Q201 598 201 523Q201 480 240 462T345 431T443 394Q549 324 549 204Q549 160 538 123T502 51T427 -2T308 -22Q180 -22 69 41Q50 52 49 57Q49 58 50 66T54 91T60 125L72 189L75 193Q80 196 84 196Q87 196 104 182T145 149T212 117T304 102Q408 102 408 188Q408 215 396 234T362 263T319 278T267 290T219 302Q149 324 105 380T61 503",
      120295: "67 687Q70 688 366 688Q661 688 666 686Q692 680 692 641V629V615Q692 579 666 570H660Q655 569 648 569Q645 569 624 569T581 570Q505 570 475 572H447V302Q447 31 445 25Q436 0 393 0H364L313 1Q293 7 286 28L285 300V572H257Q227 570 151 570Q130 570 109 570T84 569Q77 569 72 570H66Q48 577 44 588T40 631L41 661Q47 680 67 687",
      120296: "92 667Q101 694 143 694H172H200Q242 694 251 671Q253 663 253 430Q254 189 255 185Q262 134 288 107T384 79Q498 79 516 168Q520 191 521 431Q521 663 523 671Q532 694 572 694H596H618Q639 694 648 692T665 679Q671 672 671 653Q672 632 672 555V432Q671 200 670 190Q652 79 581 29T383 -22Q137 -22 98 166Q92 195 92 303V667",
      120297: "27 667Q27 683 39 688T75 694H101Q155 694 159 692Q182 687 194 665Q202 652 283 419T374 142Q376 165 473 445Q552 664 553 666Q568 694 618 694H639H658Q681 694 693 689T705 667Q705 660 592 347Q481 32 477 28Q466 7 441 1H292Q266 7 255 28Q251 32 140 347Q27 660 27 667",
      120298: "994 694Q1012 683 1014 668Q1014 661 977 519T896 217T845 26Q831 0 783 0H747H711Q685 0 672 5T649 26Q644 36 583 272T517 548Q516 552 516 551Q503 479 437 227Q389 37 383 26Q367 0 323 0H288H254Q207 0 193 26Q191 32 108 346T24 665Q24 685 44 693Q47 694 98 694H115Q152 694 168 668Q174 657 235 417T297 144Q297 134 300 153Q307 204 362 421T427 668Q441 694 488 694H523Q586 694 597 688Q612 683 620 661T651 549Q664 496 673 462Q744 194 750 146V140Q767 223 800 354T857 576T883 668Q897 694 938 694H958H994",
      120299: "52 1Q37 11 37 23Q37 26 39 32Q39 34 158 202L275 369Q275 370 221 441T112 586T55 663Q53 669 53 672Q53 687 68 693H72Q77 693 84 693T99 694T118 694T139 694H176Q203 694 212 692T230 682Q231 681 239 669T265 634T296 591L358 504L418 591Q481 682 486 686Q491 691 499 692Q505 694 569 694H632Q650 685 650 672Q650 667 646 660Q643 654 592 582T491 440T441 369T566 201T693 29Q694 27 694 23Q694 11 677 0H607L537 1Q523 6 519 10T437 131Q422 153 411 170T390 200T375 222T365 237T359 245L357 247L348 232Q339 218 319 188T283 131Q222 37 211 22T186 1H52",
      120300: "635 694H668Q688 694 698 690T708 670Q708 664 704 658L446 278L445 152V27Q442 20 440 17T433 9T419 1L368 0H339Q316 0 305 5T288 26Q286 31 286 154V278L157 468Q135 500 101 550Q43 635 34 650T24 671Q24 686 39 693Q42 694 105 694H122H132Q163 694 180 689T214 666Q225 654 336 485Q373 425 373 420L374 418Q375 419 375 421Q378 432 418 493T496 609T536 667Q543 676 551 681T572 689T591 693T615 694T635 694",
      120301: "411 584Q243 581 131 581Q122 581 116 581T106 582T102 582Q84 589 80 600T76 640L77 667Q83 686 103 693Q106 694 343 694Q579 694 584 692Q592 691 599 684T609 668Q610 665 610 646Q610 614 608 608Q605 603 434 361L261 116Q340 117 402 118T490 119T533 120T560 120H572Q605 120 614 95Q616 89 616 60V46Q616 9 590 2Q585 0 339 0Q92 0 87 2Q79 3 72 10T62 26Q61 29 61 49Q61 84 63 90Q65 94 152 217T325 461T411 584",
      120302: "255 394Q218 394 186 383T138 358T109 333T94 321H91Q88 321 86 322T83 325T80 331T79 339T78 349T77 362T75 377Q72 410 72 420Q72 423 72 425T73 429T74 431T77 433T80 435T85 437Q166 475 262 475Q360 475 413 440Q462 406 471 341Q472 332 472 181Q472 155 472 119T473 73Q473 20 462 10T398 0H380Q349 0 337 8T324 48V65Q298 30 257 10T172 -11Q109 -11 70 37T31 145Q31 276 307 289H321V309Q321 337 318 352T300 381T255 394ZM176 146Q176 116 190 97T231 77Q251 77 266 85Q322 110 322 185Q322 189 322 192T322 198L321 201V234L308 232Q176 220 176 146",
      120303: "54 667Q63 694 102 694H127H151Q190 694 199 671Q201 663 201 544L202 422L211 428Q270 468 355 468Q523 468 523 231Q523 -10 321 -10Q286 -10 261 -2T204 33Q197 11 187 6T142 0H126L81 1Q61 7 54 28V667ZM372 230Q372 317 355 349T280 382Q251 382 204 356V107Q235 76 274 76Q301 76 320 84T349 105T364 139T371 180T372 230",
      120304: "188 233Q188 199 190 177T200 131T225 95T271 83H281Q356 83 421 130Q433 138 434 139Q446 141 448 131Q449 128 453 84T457 36Q455 30 432 20T364 -1T273 -11Q37 -11 37 232Q37 456 244 474Q246 474 257 474T276 475Q349 475 400 455Q443 436 448 431L451 425Q451 419 443 377Q442 372 441 366T439 356T438 348T436 340T435 334T433 330T431 327T429 325T426 324Q420 324 406 336Q376 362 350 372T281 382Q254 382 236 373T208 352T194 317T189 278T188 233",
      120305: "225 -11Q192 -11 164 -3T104 29T55 102T37 227Q37 321 63 376Q111 469 223 469Q292 469 349 433L359 426V546Q359 564 359 589T358 623Q358 675 369 684T433 694H451Q497 694 505 669Q507 663 507 347T505 25Q500 9 487 5T450 0H432H413Q356 0 356 36V40Q298 -11 225 -11ZM356 359Q326 382 286 382Q239 382 215 358Q189 330 189 256V229V203V195Q189 102 231 86Q251 76 274 76Q318 76 356 114V359",
      120306: "30 231Q30 301 51 351T107 426T181 463T262 474H268Q293 474 312 472T366 459T422 427T462 363T480 260Q480 230 466 222T405 213Q395 213 364 213T311 214H173V211Q173 183 183 148T216 96Q244 76 287 76Q319 76 350 85T399 104T433 125T451 136Q463 138 465 128Q466 125 469 84T473 39Q471 29 423 13T330 -9Q321 -10 286 -10Q213 -10 161 11T81 68T42 144T30 231ZM353 292Q350 324 342 344T320 373T294 384T264 387Q191 387 175 286V282H353V292",
      120307: "67 458H78V502V509Q78 538 79 556T90 604T117 651T169 685T253 704Q254 704 258 704T265 705T275 705T287 704Q364 704 381 687V638V607Q381 591 378 585T366 579Q362 581 351 591T323 610T282 618Q238 618 226 587Q224 581 223 519V458H253Q288 458 298 453Q315 441 315 415Q315 399 312 390T299 377T282 372T259 371H255H226V201Q226 31 224 25Q215 0 174 0H150L106 1Q86 7 79 28L78 199V371H69Q29 371 29 412Q29 425 30 431T40 447T67 458",
      120308: "63 108Q63 142 83 176L76 184Q35 227 35 300Q35 469 243 469Q330 469 385 437L397 443Q451 469 508 469Q515 468 517 466T522 456T528 425Q534 392 534 386Q532 379 523 377Q520 377 509 381T477 390T431 398L425 399Q453 362 453 297Q453 268 445 242T417 189T351 146T242 130Q169 130 119 153Q117 141 117 136Q117 95 155 83Q161 81 252 81Q354 80 362 79Q437 71 475 48T526 -24Q532 -42 532 -65Q532 -116 489 -156T341 -204Q323 -206 274 -206H256Q39 -206 18 -84Q17 -79 17 -68Q17 15 82 42L76 53Q63 79 63 108ZM310 300Q310 359 298 377T238 396Q217 396 204 390T186 368T179 339T178 300Q178 245 189 224T244 203T299 224T310 300ZM414 -66Q414 -55 411 -47T401 -32T387 -21T368 -14T346 -10T322 -7T297 -6T271 -6T246 -6H193Q163 -6 154 -10T140 -30Q135 -45 135 -62Q135 -134 274 -134Q414 -134 414 -66",
      120309: "53 667Q62 694 101 694H126H148Q191 694 198 669Q200 663 200 526V390Q263 469 361 469Q390 469 412 465T456 449T491 413T507 351Q508 342 508 185Q508 31 506 25Q498 0 450 0H432H413Q368 0 359 23Q357 31 357 186Q356 345 355 350Q349 369 336 376Q324 381 301 381H298Q269 381 242 362Q217 342 210 316T202 239Q202 229 202 202T203 157V82Q203 24 195 12T146 0H125L80 1Q60 7 53 28V667",
      120310: "72 574Q55 583 51 591T46 619V636L47 670L48 673Q50 676 52 678T56 684T63 690T73 694H81Q89 694 102 694T129 695H181Q193 687 196 685T203 676T207 661T208 634Q208 603 204 593T181 574H72ZM54 431Q63 458 102 458H127H149Q192 458 199 433Q201 427 201 229T199 25Q190 0 149 0H125L81 1Q61 7 54 28V431",
      120311: "70 634V648Q70 686 96 693Q100 695 151 695H176Q201 695 213 691T230 670Q232 665 232 634V620Q232 582 206 575Q202 573 151 573H126Q101 573 89 577T72 598Q70 603 70 634ZM-41 -84Q-1 -105 28 -105Q67 -105 78 -85Q83 -77 83 -48T84 180Q84 427 86 433Q93 458 136 458H158H180Q201 458 209 456T225 443Q230 436 231 418Q232 397 232 313V183V124V40Q232 -55 228 -87T203 -147Q166 -205 78 -205Q31 -205 -20 -189T-71 -159Q-71 -156 -62 -124T-52 -89Q-49 -84 -41 -84",
      120312: "496 23Q496 9 487 5T457 0H427H398Q367 0 354 11Q352 12 288 99L226 183L191 150V90V54Q191 30 186 18T165 2Q160 0 124 0L90 1Q70 7 63 28V667Q72 694 108 694H128H146Q183 694 192 671Q194 663 194 496L195 325L254 383Q266 394 281 409T301 429T316 441T329 450T341 455T357 458T376 458H409H436Q461 458 470 454T480 437Q480 430 477 427T445 395Q417 368 396 347L319 271Q319 270 358 217T442 103T494 32Q496 30 496 23",
      120313: "54 667Q63 694 102 694H127H149Q192 694 199 669Q201 663 201 347T199 25Q190 0 149 0H125L81 1Q61 7 54 28V667",
      120314: "197 386Q256 468 366 468Q404 468 430 461T471 438T491 413T503 385Q563 469 666 469Q731 469 769 446T814 350Q815 343 815 185Q815 31 813 25Q808 9 796 5T758 0H737L692 1Q672 7 665 28L664 186V206V290Q664 349 655 365T610 381Q581 381 560 370T529 341T515 311T510 291Q509 286 509 157V82Q509 24 501 12T452 0H431L386 1Q366 7 359 28L358 186V206V290Q358 349 349 365T304 381Q275 381 254 370T223 341T209 311T204 291Q203 286 203 157V82Q203 24 195 12T146 0H125L80 1Q60 7 53 28V437Q58 453 80 464H122H142Q167 464 178 460T195 439Q197 434 197 409V386",
      120315: "197 386Q264 468 350 468Q375 468 390 467T429 460T466 443T492 408T507 351Q508 342 508 185Q508 31 506 25Q498 0 450 0H432H413Q368 0 359 23Q357 31 357 186Q356 345 355 350Q349 369 336 376Q324 381 301 381H298Q269 381 242 362Q217 342 210 316T202 239Q202 229 202 202T203 157V82Q203 24 195 12T146 0H125L80 1Q60 7 53 28V437Q58 453 80 464H122H142Q167 464 178 460T195 439Q197 434 197 409V386",
      120316: "274 -11Q32 -11 32 225Q32 346 85 406T249 474H266H271Q302 474 325 471T385 458T451 419T498 346Q518 300 518 225Q518 -11 274 -11ZM367 233Q367 322 350 354T270 387Q240 387 222 377T195 344T184 298T182 233Q182 151 198 117T275 83H282Q318 83 339 104Q355 119 361 146T367 233",
      120317: "125 458H139Q174 458 185 452T202 420L211 426Q245 448 288 458T354 469Q356 469 361 469T369 468Q443 468 481 412Q523 355 523 223Q523 164 509 120T473 51T423 12T371 -7T323 -11Q260 -11 204 33V-65Q204 -80 204 -102T205 -131Q205 -162 200 -175T178 -192Q173 -194 126 -194L81 -193Q61 -187 54 -166V431Q58 447 81 458H125ZM372 230Q372 376 282 376Q247 376 204 352V107L208 103Q213 99 218 95T232 87T251 79T274 76Q323 76 349 116Q372 153 372 230",
      120318: "226 -11Q37 -11 37 236Q37 294 51 338T86 407T135 445T186 464T233 469H235Q300 469 349 422L359 413V425Q359 452 376 464Q384 469 433 469H455Q498 469 505 444Q507 438 507 137Q507 -163 505 -169Q500 -185 487 -189T450 -194H432H413Q367 -194 358 -171Q356 -163 356 -63V40L348 33Q296 -11 231 -11H226ZM281 375Q188 375 188 228Q188 77 275 77Q322 77 359 120V328Q338 357 324 366T281 375",
      120319: "54 437Q58 453 81 464H122H147Q186 464 194 439Q196 434 196 405V377L203 387Q245 456 324 468Q325 468 331 468T340 469Q347 469 356 462V360Q350 355 346 354T339 353T326 353T300 347Q260 337 234 311T202 252Q201 247 201 138Q201 122 201 98T202 66Q202 33 197 20T175 2Q170 0 125 0L81 1Q61 7 54 28V437",
      120320: "37 328Q37 392 75 433T203 474Q254 474 265 473Q319 465 370 442Q378 439 380 432Q380 426 372 384Q364 336 359 333Q358 331 355 331Q348 331 337 341Q282 388 216 388H208Q190 388 180 387T161 377T151 351Q151 333 164 323T224 306L267 297Q314 285 355 246T396 144Q396 17 282 -5Q260 -10 218 -10Q170 -10 124 2T55 26T30 44Q30 48 39 99T49 153Q52 159 60 159Q66 159 70 153Q100 120 133 101T218 82Q231 82 238 83T258 87T277 101T283 126Q283 149 260 160T200 176T153 186Q109 201 73 236T37 328",
      120321: "225 267Q225 202 226 169T232 115T244 88T265 82Q295 84 318 100T345 116Q352 116 354 110T364 77Q373 46 373 43Q373 28 312 9T190 -10Q160 -10 139 1T107 29T89 77T82 136T80 210V258V371H66H59Q39 371 27 386Q20 394 20 417Q21 432 23 437Q35 458 60 458H65H83V510L84 562Q93 589 131 589H154H174Q216 589 223 564Q225 558 225 508V458H274Q330 458 338 453Q355 441 355 415Q355 388 338 376Q330 371 274 371H225V267",
      120322: "53 431Q62 459 100 459Q105 459 114 459T127 458H152Q192 458 201 435Q203 427 203 262Q204 86 208 77Q209 74 216 71Q227 66 258 66H264Q334 66 354 140L356 150L357 290Q357 427 359 435Q365 449 377 453T412 458H432H450Q498 458 506 433Q508 427 508 229T506 25Q498 0 451 0H434H418Q386 0 374 7T360 43V58L352 49Q298 -11 199 -11Q135 -9 101 11T56 80Q52 100 52 273L53 431",
      120323: "26 429T26 435T32 448T44 456Q48 458 85 458H99Q145 458 161 431Q162 429 207 285L251 145L294 284Q333 410 341 430Q351 451 374 456Q379 458 420 458H430Q450 458 457 456T471 443Q473 437 473 435Q473 426 443 325T381 126L350 28Q339 7 316 2Q312 0 250 0Q187 0 183 2Q160 7 149 28L136 68Q124 109 106 166T70 283T39 385",
      120324: "699 458Q717 447 719 432Q719 426 666 230T610 27Q602 10 588 5T548 0H512H482Q431 0 420 17T384 135Q356 241 352 298V308L351 295Q348 251 322 145T290 28Q279 0 233 0H212H191Q146 0 133 27Q130 33 77 229T24 430Q24 449 44 457Q47 458 79 458Q122 458 126 456Q154 450 163 419L233 153Q241 187 272 304T307 431Q318 458 368 458Q394 458 398 456Q421 451 430 431Q434 423 509 147L547 286Q582 416 588 429Q600 454 624 457Q632 458 647 458H663H699",
      120325: "92 0Q87 0 77 0T62 -1Q24 -1 24 22Q24 29 33 41T106 136Q185 237 184 238Q184 239 147 284T73 376T33 427Q31 430 31 436Q31 451 45 457Q48 458 96 458H122Q152 458 163 450T208 394L247 345L282 394Q288 403 297 416T309 434T319 444T328 452T338 455T352 458T372 458H393H440Q457 449 457 435Q457 428 450 419T379 328Q308 239 308 237L389 137Q409 112 436 79Q475 31 475 23Q475 -1 436 -1Q432 -1 422 -1T407 0Q360 0 352 3Q343 6 336 16T291 83L247 151L245 148Q243 145 239 139T229 124T218 106T204 84Q167 24 160 15T141 1L92 0",
      120326: "454 458Q473 446 473 430Q473 426 394 184L311 -68Q291 -119 245 -162T123 -205Q51 -205 46 -190Q44 -187 40 -142T36 -92Q36 -90 36 -88L37 -87Q41 -80 46 -80Q48 -80 73 -92T126 -105Q146 -105 161 -98T185 -76T197 -53T206 -28L215 0L122 212Q29 427 29 435Q29 448 46 457Q49 458 91 458Q93 458 106 458T125 457T140 454T157 446T170 431Q183 410 224 305T266 158Q266 152 266 151Q267 151 268 163Q271 206 302 310T342 432Q354 458 398 458H418H454",
      120327: "268 376Q250 376 180 375T92 374Q69 374 63 380Q46 390 46 419Q46 428 49 437Q57 451 73 457Q76 458 242 458T413 456Q420 455 427 448Q439 438 439 413Q439 392 433 385Q432 383 318 236T204 88Q235 88 306 89T395 90H399Q408 90 414 89T427 84T438 70T442 45Q442 9 416 2Q411 0 236 0H136Q73 0 62 1T41 12Q31 23 31 47Q31 68 36 77Q37 78 51 97T96 155T153 228L268 376",
      120328: "28 0L429 694H533L585 350Q596 275 610 182T632 46L638 3V0H530L528 18Q527 25 515 103T503 183H223L135 29L118 1L73 0H28ZM492 254Q492 256 473 398T454 589V610Q433 552 290 301L264 255L378 254H492",
      120329: "501 363Q557 355 605 316T653 222Q653 148 586 85T403 2Q394 1 240 0Q90 0 90 1L100 46Q109 90 128 177T164 348L238 694H375Q518 693 546 688Q614 674 655 635T696 544Q696 490 648 441T516 368L501 363ZM601 530Q601 568 566 590T479 621Q472 622 394 623H320L297 513Q292 489 286 459T276 415L273 401V399H339H372Q504 399 571 466Q601 498 601 530ZM257 322Q256 320 230 197T203 73Q203 71 289 71Q379 72 387 73Q459 84 507 122T556 210Q556 255 519 283T428 320Q415 322 336 323Q257 323 257 322",
      120330: "124 266Q124 372 179 473T333 639T544 705Q592 705 635 697T698 679L718 670Q719 669 701 621T681 572L676 576Q670 580 661 586T641 598T614 611T583 620Q558 625 526 625Q406 625 318 516T230 276Q230 238 236 212Q251 148 294 108T412 68Q469 68 508 80T598 123Q608 129 608 128Q606 109 603 87L598 45L573 33Q521 7 486 -1T394 -10Q358 -10 346 -8Q260 5 202 62Q124 145 124 266",
      120331: "162 348L236 694H385Q535 693 543 692Q600 682 641 654T705 586T737 506T747 425Q747 296 672 187Q625 114 548 62T384 1Q376 0 262 0Q88 0 88 1L98 46Q107 90 126 177T162 348ZM622 533Q575 624 443 624Q434 624 419 624T399 623H321L263 348Q249 283 234 213T212 107L204 72Q204 71 289 71Q374 72 386 74Q501 94 573 193T646 422Q646 487 622 533",
      120332: "86 2Q88 4 160 346T233 689Q233 691 461 691Q688 691 688 689Q685 686 671 611H495L320 612L319 609Q319 607 297 501L274 397H436Q597 397 597 396L596 391Q595 386 593 376T589 358L581 322L420 321Q258 321 258 320Q209 89 208 87Q208 85 390 85Q417 85 460 85T518 86L572 85Q556 8 554 2V0H86V2",
      120333: "86 2Q88 4 160 346T233 689Q233 691 453 691T673 689Q670 686 656 611H488L320 612Q314 579 302 523T281 427T272 385Q272 384 419 384H567L551 308H255L223 156Q216 124 207 82T194 20L190 2Q190 0 138 0H86V2",
      120334: "125 267Q125 375 182 476T337 641T544 705Q598 705 644 693T710 669T730 655L712 609L693 560L692 557L681 567Q618 626 526 626Q447 626 378 573T269 440T229 277Q229 185 276 127T406 68Q422 68 451 71T502 78T524 84L526 93Q528 102 532 119T539 153L553 222Q553 223 495 223Q436 223 436 224Q436 230 444 262L452 299H662V296Q661 290 635 166T607 40Q606 37 576 25T492 1T391 -11Q272 -11 199 66T125 267",
      120335: "517 2Q518 3 551 161T585 322Q586 323 557 323T422 323H259L190 0H138Q86 0 86 1L96 46Q105 90 124 177T160 348L234 694H337V691Q336 690 306 545T275 399H602L603 403Q603 407 634 551L665 694H768V691Q768 690 695 348T621 2V0H517V2",
      120336: "161 348L235 694H338V691Q338 690 265 348T191 2V0H139Q87 0 87 1L96 46Q106 90 125 177T161 348",
      120337: "377 424L435 694H535V691Q534 685 476 412T416 135Q401 74 350 26T210 -22Q165 -22 124 -11T65 9T46 21L54 41Q62 61 70 83T81 109Q82 111 85 106Q86 105 87 103Q93 94 103 84T135 64T185 53Q238 53 272 76T317 142Q317 145 325 182T348 289T377 424",
      120338: "236 223Q235 222 213 113T188 2V0H138Q88 0 88 1L98 46Q107 90 126 177T162 348L236 694H285Q335 694 335 693L330 671Q326 649 316 603T298 518Q289 477 280 433T266 366L261 343L672 694H729L784 693L465 420L651 0H596L541 1L384 350Q383 351 310 288T236 223",
      120339: "161 348L235 694H338V691Q338 690 273 385T208 79Q278 80 362 80H516Q502 11 499 2V0H293Q87 0 87 1L96 46Q106 90 125 177T161 348",
      120340: "375 691Q456 215 459 124V106Q488 177 762 641L793 694H929V691Q929 690 856 348T782 2V0H689V2Q691 4 753 304Q817 604 818 606Q819 611 817 608Q817 607 815 603Q798 559 540 117L484 22H440L397 23L393 42Q393 47 373 169T334 422T315 594V609L250 306Q186 3 185 2Q185 0 138 0Q92 0 92 1L102 46Q111 90 130 177T166 348L240 694H375V691",
      120341: "311 609Q310 608 246 306T181 2V0H134Q88 0 88 1L98 46Q107 90 126 177T162 348L236 694H382L383 691Q383 688 418 561T493 286T541 97L544 84L545 89Q545 90 553 128T578 246T610 394L674 694H766V691Q766 690 693 348T619 2V0H472L469 13Q468 17 393 293T312 605L311 609",
      120342: "118 254Q118 366 174 473T324 648T517 716Q627 716 695 638T763 435Q763 321 706 215T555 43T362 -22Q256 -22 187 56T118 254ZM380 58Q452 58 518 116T622 263T661 442Q661 496 646 535T608 594T567 622T534 634Q516 636 496 636Q400 636 313 528T225 264Q225 172 267 115T380 58",
      120343: "162 348L236 694H378Q522 693 530 692Q604 680 647 635T690 524Q690 474 665 430T612 359Q550 299 465 280Q443 275 343 274H250V271Q250 269 235 201T206 68T192 2V0H140Q88 0 88 1L98 46Q107 90 126 177T162 348ZM594 513Q594 560 562 588T477 622Q470 623 394 623H321L293 487L263 349V347H342H347H375Q530 347 578 449Q594 483 594 513",
      120344: "118 254Q118 366 174 473T324 648T517 716Q627 716 695 638T763 435Q763 305 693 194T543 36Q547 29 586 -47T625 -125H504L450 -8Q406 -22 363 -22Q256 -22 187 56T118 254ZM661 437Q661 532 616 584T506 636Q428 636 361 578T257 433T220 258Q220 167 264 113T380 58Q390 58 397 58T408 59T413 60T417 61Q417 63 387 127T356 193Q356 194 409 194H462L485 150L508 105Q509 103 532 125T567 161Q661 278 661 437",
      120345: "162 348L236 694H375H414H445Q507 694 538 690T606 668Q698 623 698 534V528Q698 447 608 377Q582 358 555 345T512 326L497 321L617 0H565L513 1L402 309H255L189 0H138Q88 0 88 1L98 46Q107 90 126 177T162 348ZM603 525Q603 603 499 620Q486 622 403 623H321L297 506Q292 482 285 449T274 402L271 387V385H346Q350 385 363 385T386 384Q548 384 592 479Q603 503 603 525",
      120346: "161 478Q161 568 242 642T435 716Q527 716 599 673L609 667Q595 633 589 615L571 568Q570 568 564 575T546 592T518 611T475 628T417 635Q351 635 305 596T259 507Q259 465 290 444T372 411T432 396Q473 385 509 343T545 236Q545 140 464 59T270 -22Q155 -22 54 48L92 146Q93 146 101 138T124 117T161 92T216 72T288 63Q360 63 403 109T447 204Q447 220 444 233T435 256T421 273T404 285T385 295T366 301T347 306T331 310T315 314T292 321T265 331T235 346T207 367T183 395T168 431T161 478",
      120347: "165 608L182 687Q182 688 486 688H790L789 685L781 645L773 609H521L457 306Q393 3 392 2Q392 0 340 0H288V2Q289 5 353 304T417 605V609L291 608H165",
      120348: "340 -22Q251 -22 191 33T131 177V187Q131 192 131 195T132 205T133 215T136 231T141 253T147 285T156 328T168 384T184 457L235 694H338V691Q338 690 288 451T236 210Q234 194 234 177Q234 138 247 111T280 72T319 54T357 49Q408 49 449 74T510 128Q516 136 521 143T530 158T538 175T545 194T553 220T560 250T569 289T579 336T591 395T606 464L655 694H747V691Q651 243 645 213Q623 149 587 102Q482 -22 340 -22",
      120349: "220 348L161 694H216Q270 694 270 693L283 613Q334 313 346 215Q359 102 359 96Q359 87 358 84Q388 162 684 657L706 694H753Q799 694 799 693L387 0H333Q279 0 279 1L272 45Q264 89 249 177T220 348",
      120350: "596 540Q596 562 597 585T599 609Q599 588 436 255Q402 185 362 104L310 0H213V3Q213 6 188 347T161 694H263L265 664Q290 327 293 184Q293 112 289 85Q290 85 290 87Q290 95 301 123T332 194T373 282T419 380T463 469T498 541T517 579L574 694H671V689L674 646Q678 603 682 538T691 401T699 263T703 160Q703 102 700 87Q719 154 930 576L989 694H1076Q1076 693 903 347L730 0H628V4L626 26Q624 48 622 85T616 168T609 267T603 369T598 464T596 540",
      120351: "14 0Q17 3 184 184T352 367L265 529Q244 567 222 609T188 672L176 692Q176 694 236 694H297L338 612Q387 515 400 489L421 448L645 694H758L708 640Q481 393 456 368Q455 366 500 281T596 104T652 0H531L388 293L128 0H14",
      120352: "151 692Q151 694 212 694H272L418 362L696 683L705 694H758L809 693Q809 692 630 490T444 280Q442 275 413 139L383 1L333 0Q282 0 282 2Q283 3 312 141L341 278L246 484L151 692",
      120353: "67 54Q551 615 551 617Q543 618 517 618Q510 618 463 618T376 617Q200 617 200 618T209 657L216 694H459Q702 694 702 692Q702 689 697 667L692 643L207 80H392Q493 81 577 81Q577 70 560 2V0H55V2L67 54",
      120354: "313 386Q286 386 260 381T217 369T186 355T164 342T155 337Q154 338 159 377T165 418Q251 461 320 461Q322 461 328 461T337 460Q397 460 435 424T473 329Q473 325 473 318T472 308Q432 110 407 2V0H317V2L325 38Q295 21 269 10Q215 -10 156 -10H149Q76 -10 62 69Q61 75 61 90Q61 127 73 150T116 194Q146 215 207 231T348 252H368L373 277Q378 302 378 318Q378 367 339 384Q332 386 313 386ZM150 116Q150 93 171 79T223 65Q259 65 293 85T341 135Q343 140 348 160T353 184Q353 186 342 186Q298 186 231 174T153 134Q150 127 150 116",
      120355: "302 -11Q266 -11 235 1T190 26L176 38Q170 8 168 2V0H121Q75 0 75 1L84 46Q94 90 113 177T149 348L223 694H267Q312 694 312 693T282 551T251 407Q251 406 256 408T271 415Q347 454 430 454H438Q501 454 528 374Q539 339 539 299Q539 179 466 84T302 -11ZM443 275Q443 317 421 348T346 379Q318 379 296 369Q269 359 238 332L193 118L198 109Q220 65 269 65Q350 65 396 130T443 275",
      120356: "75 164Q75 226 100 282T165 377T252 437T342 460H347Q447 460 499 417L483 378Q468 339 468 338Q466 338 455 347T424 366T385 378Q355 382 334 382Q262 382 215 318T168 177Q168 120 196 95T259 69H269Q345 69 420 108Q432 114 432 113T427 72L422 32L402 22Q382 12 344 2T259 -11Q214 -11 180 2T126 36T95 81T79 126T75 164",
      120357: "73 156Q73 224 102 293T184 408T294 455Q375 455 432 413Q438 407 438 410T469 553L499 694H588V691Q588 690 515 348T441 2V0H348V2Q357 29 357 43L352 41Q332 24 288 7T196 -10H190Q178 -10 166 -7T134 8T98 46T75 113Q73 129 73 156ZM419 335Q419 339 412 348T386 368T342 379Q284 379 243 343T184 261T167 168Q167 122 191 94T263 66Q321 66 367 116L374 124L397 229Q419 333 419 335",
      120358: "248 -11Q170 -11 121 41T71 173Q71 265 133 349T285 454Q305 460 318 460H328Q368 460 399 448Q472 414 472 309Q472 274 464 234L462 219H159Q156 198 156 185Q156 137 179 107T237 68Q246 66 268 66Q345 66 427 113V109Q426 108 422 73T417 37Q417 34 409 29Q329 -11 248 -11ZM401 299Q399 337 376 361T316 385Q291 385 266 371Q220 350 184 289H401V299",
      120359: "381 443Q381 440 374 407T366 371H315Q263 371 263 369Q262 368 224 186Q215 145 205 97T189 25L184 2V0H94V2L99 25Q104 48 114 96T134 186Q172 368 173 369Q173 371 139 371H106V373L114 410L121 444H155L188 445L191 455L212 551Q232 612 288 658T415 705Q438 705 464 701T494 694Q478 614 477 614L467 618Q457 621 440 624T406 629H400Q333 629 306 579Q301 568 289 507L275 444H328Q381 444 381 443",
      120360: "113 252Q113 334 177 394T311 454Q332 454 350 451T379 442T398 432T410 424L413 421Q412 423 411 424L409 426Q409 429 434 436T496 449T560 455H568V451Q568 447 567 429T566 394L565 377L553 379Q522 385 479 385Q463 385 456 384L443 383L436 392Q454 357 454 324Q454 243 390 182T249 120Q233 120 219 122T195 128T178 136T167 142L163 145Q149 131 149 105Q149 78 171 72L242 71Q246 71 269 71T303 71T336 68T372 62T403 51T432 32Q461 8 461 -40Q461 -112 383 -159T211 -206Q123 -206 68 -172T12 -86Q12 -55 31 -23T82 32Q90 38 89 39Q89 40 87 44T82 59T80 82Q80 134 126 189Q113 228 113 252ZM369 319Q369 354 350 368T304 383Q274 383 252 369T218 333T202 291T197 255Q197 221 217 206T263 191Q317 191 343 233T369 319ZM373 -59Q373 -41 362 -30T330 -13T291 -7T247 -5H216Q167 -5 158 -6T139 -12Q123 -20 110 -38T97 -76Q97 -102 133 -118T221 -134Q242 -134 267 -130T316 -118T357 -94T373 -59",
      120361: "416 321Q416 379 336 379Q276 379 237 302Q226 276 209 202T180 66T166 2V0H119Q73 0 73 1L82 46Q92 90 111 177T147 348L221 694H265Q310 694 310 693T279 544L247 395Q325 455 403 455Q513 455 513 358Q513 334 508 309Q507 304 476 156T443 2V0H350V2L357 34Q364 65 373 110T392 200T409 281T416 321",
      120362: "189 578Q190 579 199 627T211 678V680H315V678Q313 675 304 627T293 578V576H189V578ZM168 442T168 443T213 444T258 443T212 225T164 2V0H74V2Q75 7 121 224",
      120363: "211 577L233 680H284Q336 680 336 679L315 576H263Q211 576 211 577ZM19 -204Q-12 -204 -40 -196T-82 -179T-96 -170Q-96 -168 -78 -132L-61 -95L-54 -103Q-32 -126 3 -126Q26 -126 50 -116Q76 -101 83 -85Q84 -79 140 180T196 443Q196 444 241 444T286 443Q286 441 232 186T175 -75Q163 -120 122 -162T19 -204",
      120364: "150 348L224 694H310V691Q218 259 218 258L232 270Q245 281 274 306T327 351L435 444H489L542 443Q542 442 443 357L344 272L471 1L422 0H372L366 14Q359 27 347 54T323 105L273 210Q271 210 231 174L190 139L160 0H118Q76 0 76 1L86 46Q95 90 114 177T150 348",
      120365: "148 348L222 694H311V691Q311 690 238 348T164 2V0H119Q74 0 74 1L84 46Q93 90 112 177T148 348",
      120366: "416 321Q416 379 336 379Q276 379 237 302Q226 276 209 202T180 66T166 2V0H119Q73 0 73 2L121 226L169 449Q169 450 213 450H256L249 421Q248 417 247 412T246 404T245 398T244 394T244 392Q250 398 261 407T307 433T379 454H392H400Q451 454 472 439Q482 434 489 427T500 412T506 399T510 388L511 384Q511 384 517 388Q563 431 620 446Q648 455 680 455Q790 455 790 358Q790 334 785 309Q784 304 753 156T720 2V0H627V2L634 34Q641 65 650 110T669 200T686 281T693 321Q693 379 613 379Q553 379 514 302Q503 276 486 202T457 66T443 2V0H350V2L357 34Q364 65 373 110T392 200T409 281T416 321",
      120367: "416 321Q416 379 336 379Q276 379 237 302Q226 276 209 202T180 66T166 2V0H119Q73 0 73 2L121 226L169 449Q169 450 213 450H256L249 421Q248 417 247 412T246 404T245 398T244 394T244 392Q250 398 261 407T307 433T379 454H392Q416 454 433 452T470 440T502 411T513 358Q513 334 508 309Q507 304 476 156T443 2V0H350V2L357 34Q364 65 373 110T392 200T409 281T416 321",
      120368: "69 169Q69 238 107 306T211 417T348 461Q419 461 471 412T523 271Q523 161 438 75T247 -11Q170 -11 120 39T69 169ZM432 279Q432 338 401 361T333 385Q280 385 240 352T182 273T164 178Q164 119 195 94T265 68Q306 68 344 94Q380 115 406 169T432 279",
      120369: "259 443Q251 405 251 404L260 409Q269 414 286 421T324 436T375 449T434 455Q482 455 510 417T538 303Q538 169 463 79T302 -11Q226 -11 176 39V36Q175 35 151 -80L127 -193Q127 -194 80 -194H34V-191L102 127L169 443Q169 444 214 444T259 443ZM269 65Q332 65 386 124T441 262Q441 304 422 334T370 373Q356 375 339 375Q293 375 238 331L193 118Q200 103 206 94T229 75T269 65",
      120370: "72 149Q72 272 146 363T304 455Q340 455 371 442T409 423T436 398Q438 411 442 427L448 455H538L400 -193Q400 -194 354 -194Q307 -194 307 -193L356 37V41Q355 41 350 38T332 27T302 13Q247 -10 191 -10H179Q138 -10 105 32T72 149ZM414 298Q402 376 341 376Q277 376 223 317T169 182Q169 121 198 93T265 65Q319 65 365 111L377 123L414 298",
      120371: "240 377L244 380Q248 384 255 390T272 404T296 419T325 434T361 446T401 454Q403 454 408 454T416 455H424L421 442Q419 435 413 405T406 373Q351 373 294 336T216 237Q213 231 201 173T178 60T164 2V0H119Q74 0 74 2L122 226L170 449Q170 450 213 450H255L247 414Q246 409 245 403T243 393T241 385T240 379T240 377",
      120372: "99 299Q99 318 106 341T133 393T195 441T298 461Q336 461 370 453T420 437L436 429Q436 428 421 389T405 350Q356 386 273 386H265Q248 386 237 384T211 371T191 337Q189 329 189 326Q189 320 190 315T194 306T200 299T209 293T218 289T228 285T239 283T251 281T263 278L270 276Q278 275 283 274T298 270T316 264T333 255T351 243T367 228T380 209T388 186T391 157Q391 96 341 43T193 -11Q171 -11 150 -8T114 -1T84 9T61 19T45 28T35 33Q35 36 67 116L76 109Q132 67 211 67Q258 67 279 88T301 135Q301 159 280 170T224 187T180 197Q141 212 120 239T99 299",
      120373: "245 68Q267 68 289 75T322 90L334 98Q338 94 338 28V24L324 19Q268 -4 218 -8Q198 -11 177 -11Q118 -11 118 75Q118 98 123 127Q125 137 149 251T174 369Q174 371 135 371H97V373L105 410L112 444H152L192 445L200 478Q208 512 213 541L219 571H261Q303 571 303 570T290 506L276 444H343Q410 444 410 443Q410 440 403 407T395 371H328Q261 371 261 369Q211 152 211 118Q211 68 245 68",
      120374: "166 -10H160Q146 -10 137 -8T115 0T97 22T90 63Q90 79 130 268L167 443Q167 444 214 444Q260 444 260 443L224 273Q187 97 187 86Q187 70 202 65T250 59Q303 59 336 83T379 139Q380 143 412 292T444 443Q444 444 491 444Q537 444 537 443T491 225T443 2V0H353V2L361 38L352 34Q344 29 326 22T286 7T232 -5T166 -10",
      120375: "177 6L108 442V444H201V442Q202 441 213 371T235 213T246 90V65Q259 117 429 406L450 444H495Q540 444 540 443Q539 442 411 221L282 1L230 0H178L177 6",
      120376: "148 5Q147 8 128 222T109 440L108 444H199V442Q200 441 204 385T214 253T219 140Q219 108 215 76Q215 72 214 67V65L215 66Q219 95 278 221L390 444H475V437Q497 203 497 121Q497 90 494 70Q494 67 494 67L496 73Q520 143 654 405L674 444H718Q762 444 762 443L534 1L483 0H432V5Q429 28 422 126T413 283Q413 343 416 370L417 378Q416 377 416 376Q401 303 248 12L242 0H148V5",
      120377: "317 229Q453 9 460 0H409L359 1L312 88Q266 176 265 176Q265 177 254 165T223 132T182 88L100 0H1L15 14Q29 28 61 59T118 115L236 229L226 244Q108 433 100 444H201L290 294L438 444H537L528 435Q526 432 512 418T468 376T418 327L317 229",
      120378: "11 -117L20 -120Q28 -124 46 -128T84 -132H100Q124 -122 149 -85Q200 -6 200 1Q200 17 155 204T109 442Q109 444 156 444H203Q203 443 208 419T221 357T235 277T248 190T254 114Q254 81 250 67V65Q251 65 251 67Q256 94 297 177Q339 259 422 397L450 444H540Q540 443 386 186T219 -90Q179 -153 145 -179T73 -205Q52 -205 34 -202Q29 -202 21 -201T7 -198L1 -197Q1 -196 6 -157T11 -117",
      120379: "129 408L136 444H315Q494 444 494 443Q494 441 489 419L484 396L164 76L291 77Q418 77 418 76T411 41T402 2V0H215Q28 0 28 2L34 27L38 50L360 371L240 370Q121 370 121 371Q124 388 129 408",
      120432: "191 76Q212 75 220 68T229 38Q229 10 208 1H129H80Q48 1 38 7T28 38Q28 51 29 57T40 69T70 76Q89 76 89 78Q90 79 117 205T173 461T205 599Q212 623 250 623H262H273Q312 623 319 599Q322 591 350 461T406 205T435 78Q435 76 454 76H458Q484 76 493 59Q496 53 496 38Q496 11 478 3Q474 1 395 1H317Q295 8 295 38Q295 65 311 73Q316 75 333 76L348 77V78Q348 80 341 112L334 143H190L183 112Q176 80 176 78Q175 76 178 76Q180 76 191 76ZM318 221Q313 238 288 366T263 519Q263 526 262 527Q261 527 261 520Q261 493 236 365T206 221Q206 219 262 219T318 221",
      120433: "39 1Q17 10 17 32V38V46Q17 65 34 73Q40 76 61 76H84V535H61H54Q27 535 19 553Q17 557 17 573Q17 583 17 587T23 599T39 610Q40 611 179 611Q320 610 332 607Q332 607 339 605Q394 591 427 547T461 454Q461 413 436 378T369 325L358 320Q405 311 443 270T482 169Q482 112 445 64T345 3L334 1H39ZM309 533Q302 535 234 535H168V356H230Q284 357 296 358T323 368Q346 380 361 402T377 452Q377 482 358 505T309 533ZM398 176Q396 218 371 246T315 279Q310 280 237 280H168V76H239Q316 77 327 81Q329 82 334 84Q398 107 398 176",
      120434: "40 305Q40 437 110 529T281 622Q315 622 343 611T387 589T404 578Q409 585 415 596T425 611T435 618T452 622Q472 622 478 609T485 566Q485 559 485 540T484 508V460Q484 413 478 403T442 393Q417 393 409 402Q400 409 400 420Q400 428 395 445T380 487T347 528T295 546Q235 546 180 483T124 306Q124 245 141 197T186 121T241 80T296 66Q346 66 373 103T400 178Q400 209 435 209H442H450Q484 209 484 172Q480 96 421 43T281 -11Q177 -11 109 84T40 305",
      120435: "38 1Q16 8 16 38Q16 62 32 73Q39 76 58 76H78V535H58Q40 535 32 538Q16 548 16 573Q16 587 17 591Q23 604 34 607T83 611H166H176Q188 611 209 611T239 612Q299 612 337 597T415 530Q485 438 485 300Q485 180 431 100T301 3L291 1H38ZM400 301Q400 363 385 410T346 482T303 519T267 534Q261 535 210 535H162V76H214L267 77Q323 89 361 148T400 301",
      120436: "374 271Q374 241 367 232T332 223Q307 223 299 231Q290 240 290 263V279H173V76H418V118V144Q418 167 426 176T460 186Q491 186 500 166Q502 161 502 93V52Q502 25 499 17T480 1H41Q19 9 19 32V38Q19 63 36 73Q42 76 65 76H89V535H65H55Q44 535 38 537T25 548T19 573Q19 602 41 610H47Q53 610 63 610T88 610T121 610T160 611T204 611T251 611H458Q460 609 465 606T471 602T475 598T478 593T479 586T480 576T480 562V526V488Q480 452 462 444Q458 442 438 442Q413 442 405 450Q398 457 397 463T396 501V535H173V355H290V371Q290 394 299 403T332 412Q363 412 372 392Q374 387 374 317V271",
      120437: "384 260Q384 230 377 221T342 212Q317 212 309 220Q300 229 300 252V268H179V76H249Q264 67 267 61T271 38Q271 10 249 1H44Q22 9 22 32V38Q22 63 39 73Q45 76 69 76H95V535H69H59Q42 535 32 542T22 573Q22 602 44 610H50Q56 610 66 610T91 610T125 610T164 611T208 611T257 611H468Q470 609 475 606T481 602T485 598T488 593T489 586T490 576T490 562V526V488Q490 452 472 444Q468 442 448 442Q423 442 415 450Q408 457 407 463T406 501V535H179V344H300V360Q300 383 309 392T342 401Q373 401 382 381Q384 376 384 306V260",
      120438: "38 306Q38 447 105 534T261 622Q280 622 298 618T329 608T350 596T366 585L371 581Q373 581 377 591T390 612T417 622Q437 622 443 609T450 566Q450 559 450 540T449 508V460Q449 413 443 403T407 393Q392 393 386 394T373 402T364 426Q360 472 335 509T271 546Q214 546 168 477T121 308Q121 210 164 138T271 65Q293 65 310 78T337 109T352 147T360 180T362 195Q362 196 333 196L304 197Q282 204 282 227V234Q282 247 282 251T288 261T304 272H474Q488 263 492 256T496 234Q496 211 479 199Q475 197 461 196H449V21Q441 6 434 3T412 -1H407H402Q385 -1 379 3T364 28Q350 14 322 2T260 -11Q173 -11 106 76T38 306",
      120439: "16 571Q16 597 27 604T74 611H125H208Q223 602 226 596T230 573Q230 559 227 551T217 540T204 536T186 535H165V356H359V535H338H333Q306 535 297 552Q295 556 295 573Q295 586 295 590T301 600T317 611H486Q501 602 504 596T508 573Q508 559 505 551T495 540T482 536T464 535H443V76H464H470Q482 76 489 75T502 64T508 38Q508 10 486 1H317Q306 5 301 11T296 21T295 38V44Q295 66 311 73Q318 76 338 76H359V280H165V76H186H192Q204 76 211 75T224 64T230 38Q230 10 208 1H39Q28 5 23 11T18 21T17 38V44Q17 66 33 73Q40 76 60 76H81V535H60Q45 535 38 536T24 545T16 571",
      120440: "400 76Q431 76 441 69T452 38Q452 29 452 26T450 18T443 9T430 1H95Q84 6 79 12T73 23T72 38Q72 65 90 73Q96 76 157 76H220V535H157H124Q93 535 83 542T72 573Q72 603 93 610Q97 611 264 611H430Q432 609 436 607T444 602T449 594Q452 588 452 573Q452 546 434 538Q428 535 367 535H304V76H367H400",
      120441: "202 543T202 573T224 610H228Q231 610 237 610T251 610T269 610T291 611T315 611T342 611H457Q471 602 475 595T479 573Q479 549 462 538Q454 535 432 535H408V328Q408 159 408 133T402 93Q386 48 340 19T229 -11Q158 -11 108 16T57 100Q57 129 73 141T108 154Q128 154 143 140T159 102Q159 93 155 79Q188 65 228 65H230Q290 65 318 106Q323 115 323 139T324 329V535H274L224 536Q202 543 202 573",
      120442: "18 549T18 573T29 604T70 611H118H193Q207 603 210 596T214 573Q214 549 198 538Q191 535 172 535H152V421Q152 344 152 326T153 309L242 422L329 534Q327 535 322 536T314 538T308 542T303 548T300 558T298 573Q298 600 316 608Q322 611 392 611H463Q477 602 481 595T485 573Q485 535 446 535H441H420L281 357L436 77L454 76Q473 75 478 73Q495 62 495 38Q495 10 473 1H345Q334 5 329 11T324 21T323 38Q323 51 324 56T332 68T355 77L233 296L152 192V76H172Q191 76 198 73Q214 63 214 38Q214 9 193 1H41Q18 8 18 38Q18 61 35 73Q42 76 61 76H81V535H61Q42 535 35 538Q18 549 18 573",
      120443: "27 594Q34 605 43 608T84 611H154H213Q258 611 269 605T281 573Q281 546 263 538Q257 535 222 535H185V76H404V118V145Q404 168 411 177T446 186H453Q478 186 486 167Q488 161 488 93V50Q488 24 485 17T466 1L258 0H147H99Q47 0 36 6T25 38Q25 59 35 69Q44 76 76 76H101V535H76H64Q36 535 27 552Q25 557 25 573T27 594",
      120444: "50 535Q37 536 31 537T18 547T12 573Q12 598 22 604T62 611H91H121Q147 611 158 607T178 587Q183 579 222 446T261 293Q261 289 262 288Q263 288 263 292Q263 311 298 434T346 588Q353 603 365 607T402 611H435H450Q488 611 500 605T512 573Q512 556 506 547T493 537T474 535H459V76H474Q487 75 493 74T505 64T512 38Q512 11 494 3Q490 1 424 1H386Q355 1 345 7T335 38Q335 55 341 64T354 74T373 76H388V302Q388 512 387 519Q382 482 346 359T304 228Q292 204 262 204T220 228Q215 237 179 359T137 519Q136 512 136 302V76H151Q164 75 170 74T182 64T189 38Q189 11 171 3Q167 1 101 1H63Q32 1 22 7T12 38Q12 55 18 64T31 74T50 76H65V535H50",
      120445: "20 571Q20 598 30 604T73 611H105H136Q152 611 160 611T177 607T189 601T198 587T206 568T217 537T231 497Q354 142 365 95L368 84V535H347H342Q314 535 306 552Q304 556 304 573Q304 586 304 590T310 600T326 611H482Q497 602 500 596T504 573Q504 559 501 551T491 540T478 536T460 535H439V25Q432 7 424 4T389 0H374Q334 0 322 31L293 115Q171 468 159 517L156 528V76H177H183Q195 76 202 75T215 64T221 38Q221 10 199 1H43Q32 5 27 11T22 21T21 38V44Q21 66 37 73Q44 76 64 76H85V535H64Q49 535 42 536T28 545T20 571",
      120446: "102 588Q140 621 240 621Q323 621 335 620Q393 613 422 588Q450 560 459 493T468 306Q468 185 460 118T422 23Q382 -10 289 -10H262H235Q142 -10 102 23Q74 50 65 118T56 306Q56 427 64 494T102 588ZM363 513Q357 523 347 530T324 540T302 544T280 546H268Q192 546 167 521Q150 501 145 452T140 300Q140 235 142 197T151 130T172 89T207 71T262 65Q317 65 341 81T374 144T384 300Q384 474 363 513",
      120447: "41 1Q19 9 19 32V38Q19 63 36 73Q42 76 65 76H89V535H65H55Q38 535 29 543T19 576Q19 603 41 610H49Q57 610 70 610T100 610T136 611T175 611Q190 611 216 611T255 612Q321 612 363 598T441 537Q480 486 480 427V421Q480 354 447 311T378 251Q339 230 275 230H239H173V76H197Q220 76 227 73Q244 62 244 38Q244 10 222 1H41ZM396 421Q396 461 369 491T300 533Q294 534 233 535H173V306H233Q294 307 300 308Q345 319 370 352T396 421",
      120448: "56 306Q56 380 58 426T68 510T87 568T120 600T170 617T240 621Q323 621 335 620Q393 613 422 588Q450 560 459 493T468 306Q468 124 447 66Q433 23 394 6L424 -53Q454 -112 454 -118Q454 -128 441 -138H377Q367 -135 363 -129T333 -69L304 -11H254Q205 -10 180 -8T128 6T91 36T70 92T58 178T56 306ZM227 151Q227 171 262 171H276H281Q292 171 296 171T305 170T313 165T317 158T323 145T332 127L353 88Q356 88 361 95T372 131T382 202Q384 228 384 306Q384 452 371 492T304 544Q296 545 251 545Q230 545 215 543T188 534T169 520T155 497T147 466T143 423T141 371T140 306Q140 248 141 217T146 154T157 109T178 83T212 68T262 65H266L264 70Q261 75 256 85T247 105Q227 145 227 151",
      120449: "16 571Q16 598 27 605T76 612Q84 612 108 612T148 611Q268 611 294 605Q346 592 389 550T432 440Q432 394 410 359Q393 329 366 310L358 303Q387 273 399 239Q405 219 405 178T408 106T421 68Q426 65 428 65Q433 65 435 74T438 96T441 112Q450 130 480 130H485Q519 130 522 100Q522 79 516 56T488 11T434 -11Q421 -11 408 -8T377 5T344 37T324 93Q322 101 322 154L321 209Q304 257 257 267Q252 268 207 268H165V76H186H192Q204 76 211 75T224 64T230 38Q230 10 208 1H39Q28 5 23 11T18 21T17 38V44Q17 66 33 73Q40 76 60 76H81V535H60Q45 535 38 536T24 545T16 571ZM348 440Q348 478 321 502T260 532Q252 534 208 535H165V344H208Q212 344 223 344T239 345T252 346T266 348T278 351T293 358Q348 387 348 440",
      120450: "52 454Q52 524 107 572T229 621Q266 621 274 620Q326 610 360 588L371 581Q377 594 379 598T386 610T397 619T412 622Q433 622 439 610T446 570Q446 563 446 545T445 515V479Q445 441 444 432T436 417Q428 408 403 408T370 417Q361 424 361 434Q361 439 360 448T351 476T331 509T295 535T238 546Q194 546 163 522T132 458Q132 435 148 412Q155 401 166 393T192 380T218 371T247 364T270 359Q341 342 349 339Q389 325 418 296T461 229Q472 201 472 164Q469 92 417 41T287 -11Q240 -11 200 -1T143 19L126 29Q117 6 109 -2Q100 -11 84 -11Q64 -11 58 1T51 42Q51 49 51 66T52 95V135Q52 173 53 180T61 194Q70 203 95 203Q119 203 127 194Q136 186 136 168Q143 66 284 66H290Q325 66 350 85Q391 115 391 165Q391 204 369 228T322 260Q320 260 255 275T185 293Q123 309 88 355T52 454",
      120451: "129 38Q129 51 129 55T135 65T151 76H220V535H110V501Q110 470 109 464T101 450Q93 442 68 442H60Q37 442 28 461Q26 466 26 527L27 589Q36 607 49 610H55Q61 610 72 610T97 610T131 610T170 611T215 611T264 611H476Q478 609 483 606T489 602T493 598T496 593T497 586T498 576T498 562V526V488Q498 452 480 444Q476 442 456 442Q431 442 423 450Q416 457 415 463T414 501V535H304V76H374Q389 67 392 61T396 38Q396 10 374 1H151Q140 5 135 11T130 21T129 38",
      120452: "-3 573Q-3 597 8 604T50 612Q57 612 77 612T111 611H200Q214 602 218 595T222 573Q222 549 205 538Q198 535 175 535H151V359Q151 333 151 291Q152 177 156 162Q157 160 157 159Q165 123 193 95T262 66Q303 66 330 94T367 159Q371 175 371 191T373 359V535H349H339Q328 535 322 537T309 548T303 573T306 595T325 611H506Q520 602 524 595T528 573Q528 549 511 538Q504 535 481 535H457V364Q457 189 456 182Q448 101 394 45T262 -11Q189 -11 132 43T68 182Q67 189 67 364V535H43H33Q22 535 16 537T3 548T-3 573",
      120453: "19 578Q19 585 20 590T23 598T29 604T38 608T48 610T62 611T78 612T97 611T119 611H195Q210 602 213 596T217 573Q217 561 216 555T206 542T179 535H164Q166 529 188 435T235 231T261 94L262 84V88Q263 91 263 94Q265 121 289 231T336 438L360 535H345Q308 535 308 566V573Q308 586 308 590T314 600T330 611H484Q499 602 502 595T505 573Q505 560 504 554T493 541T465 535H447L384 278Q321 19 319 14Q309 -7 278 -7H262H246Q215 -7 205 14Q203 19 140 278L78 535H59Q45 535 38 536T25 547T19 573V578",
      120454: "459 611Q491 611 501 605T512 573Q512 538 482 535H474L439 276Q406 26 402 11Q398 2 389 -3Q387 -3 386 -4L380 -7H359H349Q324 -7 313 13Q307 29 285 139T263 275Q263 283 262 283Q261 282 261 274Q261 248 239 137T211 13Q200 -7 175 -7H165H144Q136 -3 127 3Q121 10 117 36T85 276L50 535H42Q26 536 19 545T12 564V573Q12 603 33 610Q37 611 101 611H134Q165 611 175 604T186 573Q186 563 186 559T182 547T169 538T143 535H122V531Q124 517 133 446T155 266T172 96V84L173 102Q176 157 192 243T215 346Q227 367 259 367H262H265Q297 367 309 346Q316 329 332 243T351 102L352 84V96Q356 161 368 266T390 444T402 531V535H381Q366 535 359 536T345 547T338 573Q338 600 356 608Q362 611 425 611H459",
      120455: "39 571Q39 597 49 604T93 611H141H218Q233 602 236 595T239 573Q239 538 210 535Q202 535 202 534T215 507T243 454L257 428L307 535H298Q266 538 266 573Q266 584 267 588T273 598T289 611H366H401Q442 611 454 605T466 573Q466 546 448 538Q442 535 421 535H398L299 327Q299 323 362 201L426 77L449 76Q467 76 475 75T489 65T495 38Q495 11 477 3Q473 1 395 1H317Q295 8 295 38Q295 73 325 76L334 77Q333 78 314 117T276 196L257 235L239 196Q221 157 204 118T186 77Q190 76 196 76Q211 74 218 67T227 55T228 38Q228 28 227 24T221 13T206 1H50Q28 9 28 32V38Q28 63 45 73Q51 76 73 76H96L214 324Q215 327 162 431L108 535H85H79Q67 535 60 536T46 546T39 571",
      120456: "20 573Q20 597 30 604T72 611H121H198Q212 602 216 595T220 573Q220 568 219 563T217 555T214 549T211 544T207 541T203 538T198 537T194 536T190 536L188 535Q179 535 179 534L188 516Q196 497 208 470T232 415T252 363T261 332Q261 329 262 329T263 332Q263 354 333 508L345 534Q345 535 336 535Q305 538 305 567V573Q305 589 308 595T327 611H483Q505 598 505 573Q505 549 488 538Q481 535 460 535H438L304 245V76H325H331Q343 76 350 75T363 64T369 38Q369 10 347 1H178Q167 5 162 11T157 21T156 38V44Q156 66 172 73Q180 76 199 76H220V245L86 535H64Q44 535 36 538Q20 548 20 573",
      120457: "71 1Q60 5 55 11T49 23T48 39V46Q48 56 58 73T131 183Q171 242 197 282L366 535H144V501Q144 470 143 464T135 450Q127 442 102 442H94Q71 442 62 461Q60 466 60 527L61 589Q70 607 83 610H88Q93 610 102 610T124 610T154 610T188 611T227 611T270 611H454Q456 609 461 606T467 601T471 597T474 591T475 584T476 572V565Q476 555 466 538T393 428Q353 369 327 329L158 76H397V120V146Q397 169 405 179T439 189Q470 189 479 169Q481 164 481 95V48Q481 24 478 16T459 1H71",
      120458: "126 306Q105 306 90 321T74 359Q74 439 211 439Q268 439 276 438Q343 426 383 390T430 306Q431 301 431 190V81Q446 79 465 78T492 76T509 72T521 60T524 38Q524 11 506 3Q502 1 466 1Q426 1 406 5T379 14T355 36L345 30Q284 -6 205 -6Q135 -6 92 39T48 141Q48 182 79 212T158 256T252 278T342 285H347V290Q347 315 325 335T267 362Q258 363 224 363Q189 363 185 362H179L178 358Q178 353 178 352T176 345T174 337T170 330T165 322T158 316T150 311T139 308T126 306ZM132 140Q132 115 157 93T224 70Q269 70 302 87T344 133Q346 139 347 175V211H339Q256 209 194 190T132 140",
      120459: "4 573Q4 596 15 603T52 611H90H124Q146 611 155 608T171 591Q173 586 173 491V396L182 402Q217 424 256 431Q280 437 309 437Q376 437 434 379T492 217Q492 162 473 118T422 47T358 8T293 -6Q229 -6 174 38Q171 13 163 7T135 1H131H122Q99 1 90 23L89 279V535H58L27 536Q4 543 4 573ZM409 215Q409 269 377 315T283 361Q255 361 224 344T177 297L173 290V167Q189 124 213 97T278 70Q330 70 369 111T409 215",
      120460: "291 -6Q196 -6 131 60T66 216Q66 296 119 361Q154 403 200 421T273 439Q275 440 293 440H313Q400 440 433 409Q454 388 454 359Q454 335 439 321T402 306Q380 306 365 321T350 357V362L340 363Q339 363 326 363T303 364Q280 364 266 362Q217 352 184 313T151 215Q151 153 199 112T313 70Q341 70 357 85T381 118T394 140Q402 146 424 146Q443 146 447 144Q466 137 466 117Q466 106 457 88T429 47T374 10T291 -6",
      120461: "266 573Q266 596 277 603T314 611H352H385Q411 611 419 607T435 586V76H498Q512 67 516 60T520 38Q520 9 498 1H436Q429 1 417 1T398 0Q375 0 363 7T351 34V43L342 36Q288 -6 223 -6Q143 -6 87 58T31 216Q31 307 88 372T230 437Q292 437 342 405L351 399V535H320L289 536Q266 543 266 573ZM351 290Q347 302 337 316T302 346T244 361Q193 361 154 319T115 215Q115 152 152 111T235 70Q314 70 351 170V290",
      120462: "48 217Q48 295 100 361T248 439L258 440Q268 440 274 440Q329 438 369 416T428 359T456 292T464 228Q464 215 461 208T454 198T442 190L288 189H135L138 179Q153 132 199 102T303 71Q336 71 353 86T380 120T398 143Q404 146 422 146Q453 146 462 126Q464 120 464 116Q464 84 416 39T285 -6Q187 -6 118 59T48 217ZM377 264Q371 291 365 306T341 338T294 362Q288 363 264 363Q225 363 190 336T139 264H377",
      120463: "43 395Q44 405 44 408T47 416T53 423T66 431H176V461Q176 500 182 518Q201 570 252 593T353 617Q399 614 418 593T437 548Q437 528 424 514T387 499Q365 499 353 511T338 537V541H328Q275 536 261 494Q260 490 260 460V431H327Q334 431 346 431T364 432Q392 432 404 425T416 393T405 362T365 355H327H260V76H319Q375 76 388 71T401 38Q401 27 400 23T395 12T379 1H58Q47 6 42 12T36 23T35 38Q35 65 53 73Q59 76 117 76H176V355H121H93Q64 355 54 362T43 395",
      120464: "60 274Q60 337 107 386T233 436Q278 436 316 417L329 410L338 416Q384 442 427 442T489 423T509 381T494 345T460 332Q449 332 440 338Q432 341 427 348T419 360T415 365Q414 364 410 364L383 355Q406 320 406 274Q406 211 358 162T233 112Q189 112 155 128L146 133Q142 125 142 115Q142 99 150 85T175 71Q182 72 187 70Q188 70 195 70T218 70T254 69Q259 69 275 69T297 69T318 68T340 66T361 62T384 57T405 49T428 38Q495 -1 495 -76Q495 -143 427 -186T262 -229Q161 -229 94 -185T29 -73Q30 -60 33 -48T39 -26T47 -8T57 8T67 20T77 30T86 38L91 43Q91 44 86 53T75 80T70 117Q70 142 89 183L83 194Q60 232 60 274ZM321 274Q321 312 296 337T230 362Q197 362 171 338T145 274Q145 235 170 211T233 187Q273 187 297 212T321 274ZM422 -78Q422 -54 408 -38T366 -15T315 -6T255 -4H200Q198 -4 193 -4T183 -3Q148 -3 125 -26T102 -78Q102 -110 151 -132T261 -154Q321 -154 371 -132T422 -78",
      120465: "4 573Q4 596 15 603T52 611H90H124Q146 611 155 608T171 591Q173 586 173 489Q173 394 175 394L186 402Q197 410 219 420T269 434Q278 436 306 436Q343 436 371 423Q411 402 423 365T436 265Q436 257 436 239T435 211V198V76H498Q512 67 516 60T520 38Q520 9 498 1H308Q286 10 286 32V38V46Q286 65 303 73Q309 76 329 76H351V188Q351 204 351 230T352 266Q352 321 341 341T288 361Q253 361 222 341T176 274L174 264L173 170V76H236Q250 67 254 60T258 38Q258 9 236 1H27Q4 8 4 38Q4 53 8 60T27 76H89V535H58L27 536Q4 543 4 573",
      120466: "202 538T202 559T218 596T260 612Q283 612 300 597T317 560Q317 538 300 523T260 507Q235 507 219 522ZM411 76Q441 76 451 69T462 38Q462 29 462 26T460 18T453 9T440 1H94Q72 8 72 33V38Q72 46 72 49T74 58T81 68T94 76H233V355H167L102 356Q80 363 80 393Q80 418 91 425T138 432Q145 432 165 432T200 431H295Q297 429 303 425T310 420T314 415T317 404T317 389T318 363Q318 354 318 314T317 241V76H378H411",
      120467: "261 559Q261 580 277 596T319 612Q342 612 359 597T376 560T360 523T320 507Q296 507 279 523T261 559ZM75 -91T100 -91T138 -107T152 -144V-150L160 -151H193H203Q241 -151 267 -121Q284 -97 288 -73T292 23V151V355H218L145 356Q123 365 123 387V393Q123 422 145 430H148Q151 430 156 430T169 430T185 430T205 431T227 431T251 431H354Q356 430 360 427T365 424T369 420T372 416T373 410T375 402T376 391T377 376T377 356Q377 345 377 286T376 176Q376 -67 371 -88Q362 -123 342 -151T299 -194Q254 -228 180 -228Q84 -226 56 -177Q49 -162 48 -148Q48 -122 61 -107",
      120468: "13 42Q13 63 23 69T69 76H102V535H69H54Q34 535 24 542T13 573Q13 588 15 593Q22 605 29 608T56 611H95Q113 611 122 611T140 610T152 609T159 607T163 603T167 597T173 589V413L174 237L295 355H275Q260 355 253 356T239 367T232 393Q232 419 243 425T304 431H359H464Q479 422 482 415T485 393Q485 364 464 356L431 355H398L293 254L427 76H486Q501 67 504 60T507 38Q507 28 507 24T501 12T486 1H314Q292 8 292 38Q292 62 308 73Q312 75 326 76L338 77L290 140Q279 154 267 171T248 196L242 204L207 171L173 139V76H206H221Q241 76 251 69T262 38Q262 11 244 3Q240 1 138 1Q123 1 100 1T70 0Q32 0 23 7T13 42",
      120469: "51 573Q51 602 73 610H76Q79 610 84 610T97 610T113 610T133 611T155 611T179 611H282Q301 598 304 586V76H452Q466 67 470 60T474 38Q474 10 452 1H73Q51 9 51 32V38Q51 54 54 60T73 76H220V535H146L73 536Q51 545 51 567V573",
      120470: "133 76Q156 74 164 67T172 38Q172 9 151 1H11Q-12 8 -12 38Q-12 61 5 73Q10 75 28 76H45V355H28Q10 356 5 358Q-12 370 -12 393Q-12 419 11 431H52H70Q91 431 100 427T116 405Q163 436 200 436Q255 436 281 390L285 394Q289 398 292 400T301 407T314 415T329 423T346 429T366 434T389 436H392Q425 436 448 411Q469 390 474 360T480 268V232V203V76H497Q520 74 528 67T536 38Q536 9 515 1H396Q374 9 374 32V38Q374 73 402 76H409V191V242Q409 317 404 339T375 361Q343 361 323 332T299 264Q298 258 298 165V76H315Q338 74 346 67T354 38Q354 9 333 1H214Q192 9 192 32V38Q192 73 220 76H227V191V242Q227 317 222 339T193 361Q161 361 141 332T117 264Q116 258 116 165V76H133",
      120471: "89 431Q94 431 105 431T122 432Q173 432 173 399Q173 394 175 394Q176 394 190 404T233 425T298 436Q343 436 371 423Q411 402 423 365T436 265Q436 257 436 239T435 211V198V76H498Q512 67 516 60T520 38Q520 9 498 1H308Q286 9 286 32V38V45Q286 65 303 73Q309 76 329 76H351V188Q351 204 351 230T352 266Q352 321 341 341T288 361Q253 361 222 341T176 274L174 264L173 170V76H236Q250 67 254 60T258 38Q258 9 236 1H27Q4 8 4 38Q4 53 8 60T27 76H89V355H58L27 356Q4 363 4 393Q4 408 8 415T27 431H89",
      120472: "52 216Q52 318 118 379T261 440Q343 440 407 378T472 216Q472 121 410 58T262 -6Q176 -6 114 58T52 216ZM388 225Q388 281 351 322T261 364Q213 364 175 325T136 225Q136 158 174 114T262 70T350 114T388 225",
      120473: "89 431Q93 431 104 431T121 432Q173 432 173 401V396L182 402Q237 437 305 437Q376 437 434 378T492 217Q492 146 459 93T382 17T291 -6Q261 -6 232 5T188 26L174 37Q173 37 173 -54V-146H236Q250 -155 254 -162T258 -184Q258 -213 236 -221H27Q4 -214 4 -184Q4 -169 8 -162T27 -146H89V355H58L27 356Q4 363 4 393Q4 408 8 415T27 431H89ZM409 215Q409 269 377 315T283 361Q255 361 224 344T177 297L173 290V167Q189 124 213 97T278 70Q330 70 369 111T409 215",
      120474: "34 215Q34 309 91 368T222 436Q224 436 231 436T242 437Q309 437 372 390V401Q372 419 381 428T414 437Q426 437 432 436T444 430T456 412V-146H489H504Q524 -146 534 -153T545 -184Q545 -211 527 -219Q523 -221 414 -221Q398 -221 374 -221T342 -222Q304 -222 294 -216T283 -184Q283 -157 301 -149Q307 -146 339 -146H372V-51Q372 43 371 43L364 38Q357 33 345 26T318 12T280 -1T236 -6Q155 -6 95 55T34 215ZM117 215Q117 152 157 111T250 70Q289 70 318 92T363 146Q372 163 372 192V215L371 263Q339 360 254 360Q206 360 162 321T117 215",
      120475: "327 76Q359 76 369 70T380 38Q380 10 359 1H47Q24 8 24 38Q24 54 28 61T47 76H145V355H96L47 356Q24 363 24 393Q24 409 28 416T47 431H207Q223 419 226 414T229 393V387V369Q297 437 394 437Q436 437 461 417T487 368Q487 347 473 332T438 317Q428 317 420 320T407 327T398 337T393 347T390 356L388 361Q348 356 324 345Q228 299 228 170Q228 161 228 151T229 138V76H293H327",
      120476: "72 317Q72 361 108 396T229 439Q231 439 245 439T268 440Q303 439 324 435T353 427T363 423L372 432Q380 440 397 440Q430 440 430 395Q430 390 430 380T429 366V335Q429 311 422 302T387 293Q364 293 355 300T346 316T343 336T325 353Q306 364 257 364Q209 364 178 351T147 317Q147 284 231 272Q327 256 357 247Q458 210 458 129V121Q458 74 413 34T271 -6Q246 -6 224 -3T189 5T165 14T150 22T144 26Q142 23 139 18T135 11T132 6T128 1T124 -2T119 -4T113 -5T104 -6Q84 -6 78 6T71 43Q71 48 71 60T72 79Q72 132 73 141T81 157Q90 166 115 166Q135 166 142 162T157 140Q168 108 191 90T260 70Q297 70 323 76T361 91T379 110T384 129Q384 157 346 171T247 195T165 212Q119 228 96 256T72 317",
      120477: "25 395Q26 405 26 408T29 416T35 423T48 431H145V481L146 532Q154 547 161 550T184 554H189Q218 554 227 534Q229 529 229 480V431H405Q406 430 411 427T418 422T422 416T426 407T427 393Q427 387 427 382T424 374T421 368T417 363T413 360T408 358L405 356L317 355H229V249Q229 237 229 214T228 179Q228 126 241 98T295 70Q354 70 365 149Q366 167 375 174Q383 182 407 182H415Q438 182 446 166Q448 161 448 148Q448 84 398 39T282 -6Q226 -6 189 29T146 128Q145 134 145 247V355H96H72Q45 355 35 362T25 395",
      120478: "4 393Q4 416 15 423T52 431H90Q141 431 151 429T168 417Q171 412 173 409V254L174 100Q182 70 244 70Q320 70 344 119Q349 130 350 144T351 248V355H320L289 356Q266 363 266 393Q266 408 270 415T289 431H351H386Q409 431 418 428T433 411Q435 406 435 241V76H498Q512 67 516 60T520 38Q520 9 498 1H436H394Q372 1 364 5T351 26L342 21Q293 -5 227 -5Q118 -5 96 67Q91 82 90 101T89 227V355H58L27 356Q4 363 4 393",
      120479: "24 392Q24 417 36 424T79 432Q85 432 103 432T132 431H215Q229 422 233 415T237 393Q237 355 198 355H193H172L262 77L352 355H331H323Q288 355 288 393Q288 409 291 415T310 431H478Q491 423 495 416T500 393Q500 364 478 356L452 355H426L374 190Q320 24 318 20Q307 -4 273 -4H262H251Q217 -4 206 20Q204 24 150 190L98 355H72L47 356Q24 363 24 392",
      120480: "54 355Q16 355 16 388V393Q16 423 37 430Q41 431 125 431H162Q206 431 218 425T230 393Q230 366 212 358Q206 355 174 355Q141 355 141 354L150 296Q181 110 181 89V84Q182 85 183 96Q185 118 199 173T218 237Q223 247 245 259H264H268Q294 259 309 240Q315 229 329 174T343 92Q343 84 344 84V86Q344 88 344 91T345 97Q347 125 356 187T374 301T383 354Q383 355 350 355H333Q314 355 304 362T294 393Q294 420 312 428Q318 431 401 431H440Q485 431 496 425T508 393Q508 382 508 377T498 363T470 355L455 354Q455 353 441 271T413 104T396 16Q384 -4 355 -4H351Q315 -4 305 9T280 79Q278 90 276 96Q265 149 265 169Q265 176 264 169Q263 166 263 162Q261 130 248 79T230 18Q220 -4 183 -4H175L151 -3Q134 5 127 17L112 102Q97 188 83 270T69 354Q62 355 54 355",
      120481: "35 393Q35 417 46 424T89 432Q95 432 112 432T141 431H223Q238 422 241 415T244 393Q244 389 244 383T237 367T216 355Q209 355 209 354L234 319Q259 286 260 286L308 354Q308 355 301 355Q285 356 278 365T270 384L271 393Q271 420 289 428Q295 431 376 431H459Q460 430 465 427T472 422T476 416T480 407T481 393Q481 368 470 362T434 355H425H392L344 290Q295 225 295 223Q294 223 309 203T350 149L405 77L439 76H453Q474 76 484 69T495 38Q495 10 473 1H303Q281 9 281 32V38Q281 49 282 54T290 67T313 76Q324 76 324 77L259 173L197 77Q202 76 209 76Q225 75 233 68T241 55T242 38Q242 28 242 24T236 12T221 1H51Q29 9 29 32V38Q29 48 29 51T31 59T38 67T51 76H117L171 149Q224 222 224 223L124 355H90H78Q54 355 45 361T35 393",
      120482: "26 393Q26 417 37 424T80 431H134H217Q232 422 235 416T239 393Q239 379 236 371T226 360T214 356T197 355L179 354V353L188 330Q197 306 209 272T235 201T259 133T271 89V84L274 95Q279 122 298 185T335 300T352 354Q352 355 331 355Q312 355 304 358Q288 368 288 393Q288 408 291 415T310 431H478Q479 430 484 427T491 422T495 416T499 407T500 393Q500 376 493 367T479 357T458 355H452Q426 355 425 353Q420 337 351 124T280 -94Q240 -195 168 -220Q147 -228 125 -228Q89 -228 66 -201T42 -139Q42 -116 56 -102T93 -87Q117 -87 130 -102T144 -135V-138H126Q121 -148 121 -150T130 -152Q182 -147 207 -87Q211 -78 223 -40T236 1Q230 10 102 355H75L49 356Q26 363 26 393",
      120483: "56 1Q40 7 37 14T34 41Q34 59 36 64Q39 67 43 73Q65 95 191 213T341 355H133V334Q133 306 124 297Q116 289 91 289H83Q60 289 51 308Q49 313 49 361L50 409Q59 427 72 430H78Q83 430 92 430T115 430T144 430T179 431T219 431T262 431H450Q452 430 455 428T459 424T463 422T466 419T468 416T469 413T470 409T471 404T472 398T472 391Q472 374 469 368L462 358Q453 349 315 218Q210 122 164 76H391V103Q391 136 400 146Q409 155 433 155Q464 155 473 135Q475 130 475 78V46Q475 24 472 16T453 1H56",
      120488: "296 0Q278 3 164 3Q58 3 49 0H40V62H92Q144 62 144 64Q388 682 397 689Q403 698 434 698Q463 698 471 689Q475 686 538 530T663 218L724 64Q724 62 776 62H828V0H817Q796 3 658 3Q509 3 485 0H472V62H517Q561 62 561 63L517 175H262L240 120Q218 65 217 64Q217 62 261 62H306V0H296ZM390 237L492 238L440 365Q390 491 388 491Q287 239 287 237H390",
      120489: "720 510Q720 476 704 448T665 404T619 377T580 362L564 359L583 356Q602 353 632 342T690 312Q712 292 725 276Q752 235 752 189V183Q752 160 741 125Q698 18 547 2Q543 1 288 0H39V62H147V624H39V686H264H409Q502 686 542 681T624 655Q720 607 720 510ZM563 513Q563 553 548 578T518 611T486 622Q479 624 385 624H293V382H375Q458 383 467 385Q563 405 563 513ZM590 192Q590 307 505 329Q504 330 503 330L398 331H293V62H391H400H444Q496 62 528 75T580 131Q590 155 590 192",
      120490: "425 0L228 3Q63 3 51 0H39V62H147V618H39V680H612V676Q614 670 627 552T643 428V424H581V428Q580 430 576 461T562 524T532 576Q512 596 481 605T426 616T357 618H304V62H439V0H425",
      120491: "901 12Q901 7 892 0H479Q65 0 62 2Q56 6 56 11Q56 14 242 347T433 685Q438 694 450 696Q454 698 480 698H506L523 687Q526 683 711 354T899 17Q901 13 901 12ZM653 137L427 538L202 137L315 136H540L653 137",
      120492: "723 286Q721 284 700 145T677 3V0H39V62H147V618H39V680H660V676Q662 670 675 552T691 428V424H629V428Q629 429 627 448T618 494T601 541Q574 593 527 605T382 618H374H304V384H336Q338 384 347 384T361 384T376 386T392 390T407 397T421 407T432 423Q442 444 443 482V501H505V205H443V224Q442 258 435 278T411 307T380 318T336 322H304V62H375H394Q429 62 449 62T497 66T541 76T577 95T609 126T632 170T651 232Q661 287 661 289H723V286",
      120493: "80 430L92 686H358Q624 686 628 684Q638 679 638 656Q638 640 637 639Q637 638 445 353Q401 288 351 214T277 103L253 67L256 66Q258 66 265 66T279 66T298 66H343Q380 66 406 68T464 81T518 110T557 164T579 250Q583 278 583 298Q583 299 614 299H645V291Q643 281 636 150T627 8V0H353Q79 0 75 2Q64 7 64 31Q64 48 66 52L259 340L451 623Q451 624 384 624Q294 623 259 612Q155 581 143 446Q142 440 142 432V430H80",
      120494: "400 0Q376 3 226 3Q75 3 51 0H39V62H147V624H39V686H51Q75 683 226 683Q376 683 400 686H412V624H304V388H595V624H487V686H499Q523 683 673 683Q824 683 848 686H860V624H752V62H860V0H848Q824 3 674 3Q523 3 499 0H487V62H595V326H304V62H412V0H400",
      120495: "629 -10T446 -10T164 89T64 340Q64 380 71 420T102 510T163 596T266 662T418 696H438Q488 696 499 695Q582 686 644 655T741 584T796 495T823 409T829 338Q829 188 729 89ZM439 645Q416 645 390 638T333 615T275 564T236 480Q221 423 221 341Q221 272 230 228Q247 144 301 94T447 43T592 93T663 228Q672 272 672 341Q672 645 439 645ZM286 242V446H348V412H545V446H607V242H545V276H348V242H286",
      120496: "397 0Q370 3 218 3Q65 3 38 0H25V62H139V624H25V686H38Q65 683 218 683Q370 683 397 686H410V624H296V62H410V0H397",
      120497: "400 0Q376 3 226 3Q75 3 51 0H39V62H147V624H39V686H51Q75 683 226 683Q376 683 400 686H412V624H304V338L472 483L634 624H565V686H576Q597 683 728 683Q814 683 829 686H836V624H730L614 524Q507 432 497 422Q496 422 498 418T514 395T553 342T627 241L759 63L805 62H852V0H842Q830 3 701 3Q550 3 526 0H513V62H549Q584 62 584 63Q583 65 486 196T388 328L304 256V62H412V0H400",
      120498: "285 0Q267 3 154 3Q56 3 47 0H40V62H131Q131 63 167 160T244 369T321 578T359 678Q366 698 393 698H404H413Q437 698 446 678Q448 672 560 369T674 62H765V0H754Q733 3 604 3Q453 3 429 0H416V62H461L507 63L355 470Q353 468 279 265L203 63L249 62H294V0H285",
      120499: "314 0Q296 3 181 3T48 0H39V62H147V624H39V686H305Q316 679 323 667Q330 653 434 414L546 157L658 414Q766 662 773 674Q778 681 788 686H1052V624H944V62H1052V0H1040Q1016 3 874 3T708 0H696V62H804V341L803 618L786 580Q770 543 735 462T671 315Q540 13 536 9Q528 1 507 1Q485 1 477 9Q472 14 408 162T281 457T217 603Q215 603 215 334V62H323V0H314",
      120500: "314 0Q296 3 181 3T48 0H39V62H147V624H39V686H171H265Q288 686 297 686T309 684T315 679Q317 676 500 455T684 233V624H576V686H585Q603 683 718 683T851 686H860V624H752V319Q752 15 750 11Q747 4 742 2T718 0H712Q708 0 706 0T700 0T696 1T693 2T690 4T687 7T684 11T679 16T674 23Q671 27 437 311L215 579V62H323V0H314",
      120501: "54 465L63 674Q63 675 383 675T703 674L712 465Q712 464 681 464H650V467Q650 490 646 516T632 545Q612 550 383 550H283Q169 550 149 548T124 531Q123 530 123 529Q116 506 116 467V464H85Q54 464 54 465ZM160 256V447H222V414H544V447H606V256H544V289H222V256H160ZM57 0L48 222H110V219Q110 147 125 133Q127 130 130 129T160 127T235 126T383 126Q482 126 530 126T604 127T635 129T641 133Q656 146 656 219V222H718L709 0H57",
      120502: "64 339Q64 431 96 502T182 614T295 675T420 696Q469 696 481 695Q620 680 709 589T798 339Q798 173 697 82T432 -10Q262 -10 163 85T64 339ZM625 454Q618 502 600 538T562 593T515 624T469 639T431 642Q331 642 276 563Q232 493 232 353Q232 315 234 285T244 216T267 148T308 94T372 56Q405 46 432 46Q517 46 567 106T627 267Q631 299 631 353Q631 418 625 454",
      120503: "400 0Q376 3 226 3Q75 3 51 0H39V62H147V618H39V680H860V618H752V62H860V0H848Q824 3 674 3Q523 3 499 0H487V62H595V618H304V62H412V0H400",
      120504: "400 0Q376 3 226 3Q75 3 51 0H39V62H147V624H39V686H253Q435 686 470 685T536 678Q585 668 621 648T675 605T705 557T718 514T721 483T718 451T704 409T673 362T616 322T530 293Q500 288 399 287H304V62H412V0H400ZM553 475Q553 554 537 582T459 622Q451 623 373 624H298V343H372Q457 344 480 350Q527 362 540 390T553 475",
      120506: "766 271Q764 266 750 137T735 4V0H407Q74 0 71 4L70 5Q64 9 64 18Q64 24 82 41T213 158L359 288Q360 288 320 336T214 460Q67 633 66 635Q64 638 64 655Q64 679 75 684Q78 686 407 686H735V682Q738 676 751 558T766 434V430H735Q704 430 704 431Q704 434 703 444T696 477T681 520T654 563T613 598Q578 615 527 619T371 624H281L396 489Q506 358 513 351Q517 342 512 334Q503 325 371 208Q338 179 303 147T249 99L231 83L243 81Q258 81 364 81Q382 81 418 81T470 82T513 83T554 88T587 96T619 109T645 129Q689 173 702 260L704 274Q704 275 735 275H766V271",
      120507: "41 425Q41 426 51 545T62 669V675H737V669Q738 665 748 546T758 425V419H696V425Q687 517 669 555T595 607Q578 612 522 613H478V62H631V0H615Q585 3 399 3Q214 3 184 0H168V62H321V613H277H263Q164 613 134 561Q113 527 103 425V419H41V425",
      120508: "64 556Q69 619 114 658T224 697Q271 697 310 677T375 622T417 554T444 484L447 477V479Q456 516 473 551T516 620T582 676T670 697Q735 697 780 656T829 556Q829 539 818 532H772Q761 539 761 548Q761 571 681 571Q664 571 653 570T623 562T587 537T555 490Q536 448 531 410T525 300V210V62H660V0H646L447 3Q257 1 247 0H233V62H368V210V301Q368 373 363 410T338 490Q324 518 307 536T270 561T240 569T212 571Q132 571 132 548Q132 539 121 532H75Q64 538 64 556",
      120509: "609 0Q582 3 415 3T221 0H207V62H342V168L328 169Q193 180 117 241Q64 286 64 343T117 445Q193 506 328 517L342 518V624H207V686H221Q248 683 415 683T609 686H623V624H488V518L502 517Q637 506 713 445Q766 400 766 343T713 241Q637 180 502 169L488 168V62H623V0H609ZM342 219T342 343T340 467Q328 467 304 459Q277 451 261 439T237 409T228 378T226 343Q226 314 229 296T250 259T301 228Q331 219 341 219Q342 219 342 343ZM604 343Q604 365 602 379T591 413T560 446T503 464L489 467Q488 467 488 343T489 219Q499 219 529 228Q554 236 570 248T593 277T602 308T604 343",
      120510: "327 0Q306 3 174 3Q52 3 43 0H33V62H98L162 63L360 333L157 624H48V686H59Q80 683 217 683Q368 683 395 686H408V624H335L393 540L452 458L573 623Q573 624 528 624H483V686H494Q515 683 646 683Q769 683 778 686H787V624H658L575 511Q493 398 493 397L508 376Q522 356 553 312T611 229L727 62H835V0H824Q803 3 667 3Q516 3 489 0H476V62H513L549 63L401 274L247 63Q247 62 292 62H338V0H327",
      120511: "64 515Q64 532 71 536T104 540H139Q200 540 207 538Q225 533 236 521T253 489T260 454T264 414Q264 340 287 296T347 237Q369 226 373 226Q374 226 374 425V624H239V686H253Q280 683 447 683T641 686H655V624H520V226L522 227Q525 228 531 229T552 240T580 261T606 298T624 354Q627 368 628 394T631 440T637 482T654 518T686 538Q693 540 754 540H794Q817 540 823 536T829 515Q829 500 824 495T811 489T796 483T782 461T775 408Q767 212 568 175Q526 168 521 168Q520 168 520 115V62H655V0H641Q614 3 447 3T253 0H239V62H374V168L364 169Q290 178 243 203Q126 261 118 409Q117 443 111 461T98 484T83 489T70 495T64 515",
      120512: "598 645T415 645T232 458Q232 385 275 239T318 26Q318 8 311 4T272 -1Q265 -1 240 -1T196 0H88V3Q86 5 70 108Q52 211 51 212V215H113V212Q123 149 132 133Q136 124 149 122T202 118Q241 118 241 119Q241 132 132 277Q64 378 64 457Q64 564 158 630T403 696Q487 696 543 685T661 638Q722 599 744 549T766 458Q766 434 761 410T749 368T729 327T709 293T684 258T663 229Q632 187 614 160T592 126L589 119Q589 118 628 118Q667 119 680 121T698 133Q702 140 706 160T714 196L717 212V215H779V212Q778 211 760 108Q744 5 742 3V0H634H562Q528 0 520 4T512 26Q512 92 555 238T598 458Q598 645 415 645",
      120513: "56 673Q56 679 65 686H892Q901 679 901 673Q901 668 714 331T521 -15Q518 -18 506 -24H452Q440 -19 436 -15Q431 -8 337 162T150 501L57 669Q57 670 56 672V673ZM528 136L758 553H297Q298 551 414 341L528 136",
      120546: "208 74Q208 50 254 46Q272 46 272 35Q272 34 270 22Q267 8 264 4T251 0Q249 0 239 0T205 1T141 2Q70 2 50 0H42Q35 7 35 11Q37 38 48 46H62Q132 49 164 96Q170 102 345 401T523 704Q530 716 547 716H555H572Q578 707 578 706L606 383Q634 60 636 57Q641 46 701 46Q726 46 726 36Q726 34 723 22Q720 7 718 4T704 0Q701 0 690 0T651 1T578 2Q484 2 455 0H443Q437 6 437 9T439 27Q443 40 445 43L449 46H469Q523 49 533 63L521 213H283L249 155Q208 86 208 74ZM516 260Q516 271 504 416T490 562L463 519Q447 492 400 412L310 260L413 259Q516 259 516 260",
      120547: "231 637Q204 637 199 638T194 649Q194 676 205 682Q206 683 335 683Q594 683 608 681Q671 671 713 636T756 544Q756 480 698 429T565 360L555 357Q619 348 660 311T702 219Q702 146 630 78T453 1Q446 0 242 0Q42 0 39 2Q35 5 35 10Q35 17 37 24Q42 43 47 45Q51 46 62 46H68Q95 46 128 49Q142 52 147 61Q150 65 219 339T288 628Q288 635 231 637ZM649 544Q649 574 634 600T585 634Q578 636 493 637Q473 637 451 637T416 636H403Q388 635 384 626Q382 622 352 506Q352 503 351 500L320 374H401Q482 374 494 376Q554 386 601 434T649 544ZM595 229Q595 273 572 302T512 336Q506 337 429 337Q311 337 310 336Q310 334 293 263T258 122L240 52Q240 48 252 48T333 46Q422 46 429 47Q491 54 543 105T595 229",
      120548: "49 1Q31 1 31 10Q31 12 34 24Q39 43 44 45Q48 46 59 46H65Q92 46 125 49Q139 52 144 61Q146 66 215 342T285 622Q285 629 281 629Q273 632 228 634H197Q191 640 191 642T193 661Q197 674 203 680H714Q721 676 721 669Q721 664 708 557T694 447Q692 440 674 440H662Q655 445 655 454Q655 455 658 480T661 534Q661 572 652 592Q638 619 603 626T501 634H471Q398 633 393 630Q389 628 386 622Q385 619 315 341T245 60Q245 46 333 46H345Q366 46 366 35Q366 33 363 21T358 6Q356 1 339 1Q334 1 292 1T187 2Q122 2 88 2T49 1",
      120549: "574 715L582 716Q589 716 595 716Q612 716 616 714Q621 712 621 709Q622 707 705 359T788 8Q786 5 785 3L781 0H416Q52 0 50 2T48 6Q48 9 305 358T567 711Q572 712 574 715ZM599 346L538 602L442 474Q347 345 252 217T157 87T409 86T661 88L654 120Q646 151 629 220T599 346",
      120550: "492 213Q472 213 472 226Q472 230 477 250T482 285Q482 316 461 323T364 330H312Q311 328 277 192T243 52Q243 48 254 48T334 46Q428 46 458 48T518 61Q567 77 599 117T670 248Q680 270 683 272Q690 274 698 274Q718 274 718 261Q613 7 608 2Q605 0 322 0H133Q31 0 31 11Q31 13 34 25Q38 41 42 43T65 46Q92 46 125 49Q139 52 144 61Q146 66 215 342T285 622Q285 629 281 629Q273 632 228 634H197Q191 640 191 642T193 659Q197 676 203 680H757Q764 676 764 669Q764 664 751 557T737 447Q735 440 717 440H705Q698 445 698 453L701 476Q704 500 704 528Q704 558 697 578T678 609T643 625T596 632T532 634H485Q397 633 392 631Q388 629 386 622Q385 619 355 499T324 377Q347 376 372 376H398Q464 376 489 391T534 472Q538 488 540 490T557 493Q562 493 565 493T570 492T572 491T574 487T577 483L544 351Q511 218 508 216Q505 213 492 213",
      120551: "58 8Q58 23 64 35Q64 36 329 334T596 635L586 637Q575 637 512 637H500H476Q442 637 420 635T365 624T311 598T266 548T228 469Q227 466 226 463T224 458T223 453T222 450L221 448Q218 443 202 443Q185 443 182 453L214 561Q228 606 241 651Q249 679 253 681Q256 683 487 683H718Q723 678 723 675Q723 673 717 649Q189 54 188 52L185 49H274Q369 50 377 51Q452 60 500 100T579 247Q587 272 590 277T603 282H607Q628 282 628 271Q547 5 541 2Q538 0 300 0H124Q58 0 58 8",
      120552: "228 637Q194 637 192 641Q191 643 191 649Q191 673 202 682Q204 683 219 683Q260 681 355 681Q389 681 418 681T463 682T483 682Q499 682 499 672Q499 670 497 658Q492 641 487 638H485Q483 638 480 638T473 638T464 637T455 637Q416 636 405 634T387 623Q384 619 355 500Q348 474 340 442T328 395L324 380Q324 378 469 378H614L615 381Q615 384 646 504Q674 619 674 627T617 637Q594 637 587 639T580 648Q580 650 582 660Q586 677 588 679T604 682Q609 682 646 681T740 680Q802 680 835 681T871 682Q888 682 888 672Q888 645 876 638H874Q872 638 869 638T862 638T853 637T844 637Q805 636 794 634T776 623Q773 618 704 340T634 58Q634 51 638 51Q646 48 692 46H723Q729 38 729 37T726 19Q722 6 716 0H701Q664 2 567 2Q533 2 504 2T458 2T437 1Q420 1 420 10Q420 15 423 24Q428 43 433 45Q437 46 448 46H454Q481 46 514 49Q520 50 522 50T528 55T534 64T540 82T547 110T558 153Q565 181 569 198Q602 330 602 331T457 332H312L279 197Q245 63 245 58Q245 51 253 49T303 46H334Q340 38 340 37T337 19Q333 6 327 0H312Q275 2 178 2Q144 2 115 2T69 2T48 1Q31 1 31 10Q31 12 34 24Q39 43 44 45Q48 46 59 46H65Q92 46 125 49Q139 52 144 61Q147 65 216 339T285 628Q285 635 228 637",
      120553: "740 435Q740 320 676 213T511 42T304 -22Q207 -22 138 35T51 201Q50 209 50 244Q50 346 98 438T227 601Q351 704 476 704Q514 704 524 703Q621 689 680 617T740 435ZM640 466Q640 523 625 565T583 628T532 658T479 668Q370 668 273 559T151 255Q150 245 150 213Q150 156 165 116T207 55T259 26T313 17Q385 17 451 63T561 184Q590 234 615 312T640 466ZM510 276Q510 278 512 288L515 298Q515 299 384 299H253L250 285Q246 271 244 268T231 265H227Q216 265 214 266T207 274Q207 278 223 345T244 416Q247 419 260 419H263Q280 419 280 408Q280 406 278 396L275 386Q275 385 406 385H537L540 399Q544 413 546 416T559 419H563Q574 419 576 418T583 410Q583 403 566 339Q549 271 544 267Q542 265 538 265H530H527Q510 265 510 276",
      120554: "43 1Q26 1 26 10Q26 12 29 24Q34 43 39 45Q42 46 54 46H60Q120 46 136 53Q137 53 138 54Q143 56 149 77T198 273Q210 318 216 344Q286 624 286 626Q284 630 284 631Q274 637 213 637H193Q184 643 189 662Q193 677 195 680T209 683H213Q285 681 359 681Q481 681 487 683H497Q504 676 504 672T501 655T494 639Q491 637 471 637Q440 637 407 634Q393 631 388 623Q381 609 337 432Q326 385 315 341Q245 65 245 59Q245 52 255 50T307 46H339Q345 38 345 37T342 19Q338 6 332 0H316Q279 2 179 2Q143 2 113 2T65 2T43 1",
      120555: "285 628Q285 635 228 637Q205 637 198 638T191 647Q191 649 193 661Q199 681 203 682Q205 683 214 683H219Q260 681 355 681Q389 681 418 681T463 682T483 682Q500 682 500 674Q500 669 497 660Q496 658 496 654T495 648T493 644T490 641T486 639T479 638T470 637T456 637Q416 636 405 634T387 623L306 305Q307 305 490 449T678 597Q692 611 692 620Q692 635 667 637Q651 637 651 648Q651 650 654 662T659 677Q662 682 676 682Q680 682 711 681T791 680Q814 680 839 681T869 682Q889 682 889 672Q889 650 881 642Q878 637 862 637Q787 632 726 586Q710 576 656 534T556 455L509 418L518 396Q527 374 546 329T581 244Q656 67 661 61Q663 59 666 57Q680 47 717 46H738Q744 38 744 37T741 19Q737 6 731 0H720Q680 3 625 3Q503 3 488 0H478Q472 6 472 9T474 27Q478 40 480 43T491 46H494Q544 46 544 71Q544 75 517 141T485 216L427 354L359 301L291 248L268 155Q245 63 245 58Q245 51 253 49T303 46H334Q340 37 340 35Q340 19 333 5Q328 0 317 0Q314 0 280 1T180 2Q118 2 85 2T49 1Q31 1 31 11Q31 13 34 25Q38 41 42 43T65 46Q92 46 125 49Q139 52 144 61Q147 65 216 339T285 628",
      120556: "135 2Q114 2 90 2T60 1Q35 1 35 11Q35 28 42 40Q45 46 55 46Q119 46 151 94Q153 97 325 402T498 709Q505 716 526 716Q543 716 549 710Q550 709 560 548T580 224T591 57Q594 52 595 52Q603 47 638 46H663Q670 39 670 35Q669 12 657 0H644Q613 2 530 2Q497 2 469 2T424 2T405 1Q388 1 388 10Q388 15 391 24Q392 27 393 32T395 38T397 41T401 44T406 45T415 46Q473 46 487 64L472 306Q468 365 465 426T459 518L457 550Q456 550 328 322T198 88Q196 80 196 77Q196 49 243 46Q261 46 261 35Q261 34 259 22Q256 7 254 4T240 0Q237 0 211 1T135 2",
      120557: "289 629Q289 635 232 637Q208 637 201 638T194 648Q194 649 196 659Q197 662 198 666T199 671T201 676T203 679T207 681T212 683T220 683T232 684Q238 684 262 684T307 683Q386 683 398 683T414 678Q415 674 451 396L487 117L510 154Q534 190 574 254T662 394Q837 673 839 675Q840 676 842 678T846 681L852 683H948Q965 683 988 683T1017 684Q1051 684 1051 673Q1051 668 1048 656T1045 643Q1041 637 1008 637Q968 636 957 634T939 623Q936 618 867 340T797 59Q797 55 798 54T805 50T822 48T855 46H886Q892 37 892 35Q892 19 885 5Q880 0 869 0Q864 0 828 1T736 2Q675 2 644 2T609 1Q592 1 592 11Q592 13 594 25Q598 41 602 43T625 46Q652 46 685 49Q699 52 704 61Q706 65 742 207T813 490T848 631L654 322Q458 10 453 5Q451 4 449 3Q444 0 433 0Q418 0 415 7Q413 11 374 317L335 624L267 354Q200 88 200 79Q206 46 272 46H282Q288 41 289 37T286 19Q282 3 278 1Q274 0 267 0Q265 0 255 0T221 1T157 2Q127 2 95 1T58 0Q43 0 39 2T35 11Q35 13 38 25T43 40Q45 46 65 46Q135 46 154 86Q158 92 223 354T289 629",
      120558: "234 637Q231 637 226 637Q201 637 196 638T191 649Q191 676 202 682Q204 683 299 683Q376 683 387 683T401 677Q612 181 616 168L670 381Q723 592 723 606Q723 633 659 637Q635 637 635 648Q635 650 637 660Q641 676 643 679T653 683Q656 683 684 682T767 680Q817 680 843 681T873 682Q888 682 888 672Q888 650 880 642Q878 637 858 637Q787 633 769 597L620 7Q618 0 599 0Q585 0 582 2Q579 5 453 305L326 604L261 344Q196 88 196 79Q201 46 268 46H278Q284 41 284 38T282 19Q278 6 272 0H259Q228 2 151 2Q123 2 100 2T63 2T46 1Q31 1 31 10Q31 14 34 26T39 40Q41 46 62 46Q130 49 150 85Q154 91 221 362L289 634Q287 635 234 637",
      120559: "222 668Q222 670 229 677H654Q677 677 705 677T740 678Q764 678 770 676T777 667Q777 662 764 594Q761 579 757 559T751 528L749 519Q747 512 729 512H717Q710 519 710 525Q712 532 715 559T719 591Q718 595 711 595Q682 598 486 598Q252 598 246 592Q239 587 228 552L216 517Q214 512 197 512H185Q178 517 178 522Q178 524 198 591T222 668ZM227 262Q218 262 215 262T209 266L207 270L227 356Q247 435 250 439Q253 443 260 443H267H280Q287 438 287 433Q287 430 285 420T280 402L278 393Q278 392 431 392H585L590 415Q595 436 598 439T612 443H628Q635 438 635 433Q635 431 615 351T594 268Q592 262 575 262H572Q556 262 556 272Q556 280 560 293L565 313H258L252 292Q248 271 245 267T230 262H227ZM60 0Q53 4 53 11Q53 14 68 89T84 169Q88 176 98 176H104H116Q123 169 123 163Q122 160 117 127T112 88Q112 80 243 80H351H454Q554 80 574 81T597 88V89Q603 100 610 121T622 157T630 174Q633 176 646 176H658Q665 171 665 166Q665 164 643 89T618 7Q616 2 607 1T548 0H335H60",
      120560: "740 435Q740 320 676 213T511 42T304 -22Q207 -22 138 35T51 201Q50 209 50 244Q50 346 98 438T227 601Q351 704 476 704Q514 704 524 703Q621 689 680 617T740 435ZM637 476Q637 565 591 615T476 665Q396 665 322 605Q242 542 200 428T157 216Q157 126 200 73T314 19Q404 19 485 98T608 313Q637 408 637 476",
      120561: "48 1Q31 1 31 10Q31 12 34 24Q39 43 44 45Q48 46 59 46H65Q92 46 125 49Q139 52 144 61Q146 66 215 342T285 622Q285 629 281 629Q273 632 228 634H197Q191 640 191 642T193 661Q197 674 203 680H541Q621 680 709 680T812 681Q841 681 855 681T877 679T886 676T887 670Q887 663 885 656Q880 637 875 635Q871 634 860 634H854Q827 634 794 631Q780 628 775 619Q773 614 704 338T634 58Q634 51 638 51Q646 48 692 46H723Q729 38 729 37T726 19Q722 6 716 0H701Q664 2 567 2Q533 2 504 2T458 2T437 1Q420 1 420 10Q420 15 423 24Q428 43 433 45Q437 46 448 46H454Q481 46 514 49Q528 52 533 61Q536 67 572 209T642 491T678 632Q678 634 533 634H388Q387 631 316 347T245 59Q245 55 246 54T253 50T270 48T303 46H334Q340 38 340 37T337 19Q333 6 327 0H312Q275 2 178 2Q144 2 115 2T69 2T48 1",
      120562: "287 628Q287 635 230 637Q206 637 199 638T192 648Q192 649 194 659Q200 679 203 681T397 683Q587 682 600 680Q664 669 707 631T751 530Q751 453 685 389Q616 321 507 303Q500 302 402 301H307L277 182Q247 66 247 59Q247 55 248 54T255 50T272 48T305 46H336Q342 37 342 35Q342 19 335 5Q330 0 319 0Q316 0 282 1T182 2Q120 2 87 2T51 1Q33 1 33 11Q33 13 36 25Q40 41 44 43T67 46Q94 46 127 49Q141 52 146 61Q149 65 218 339T287 628ZM645 554Q645 567 643 575T634 597T609 619T560 635Q553 636 480 637Q463 637 445 637T416 636T404 636Q391 635 386 627Q384 621 367 550T332 412T314 344Q314 342 395 342H407H430Q542 342 590 392Q617 419 631 471T645 554",
      120564: "65 0Q58 4 58 11Q58 16 114 67Q173 119 222 164L377 304Q378 305 340 386T261 552T218 644Q217 648 219 660Q224 678 228 681Q231 683 515 683H799Q804 678 806 674Q806 667 793 559T778 448Q774 443 759 443Q747 443 743 445T739 456Q739 458 741 477T743 516Q743 552 734 574T710 609T663 627T596 635T502 637Q480 637 469 637H339Q344 627 411 486T478 341V339Q477 337 477 336L457 318Q437 300 398 265T322 196L168 57Q167 56 188 56T258 56H359Q426 56 463 58T537 69T596 97T639 146T680 225Q686 243 689 246T702 250H705Q726 250 726 239Q726 238 683 123T639 5Q637 1 610 1Q577 0 348 0H65",
      120565: "40 437Q21 437 21 445Q21 450 37 501T71 602L88 651Q93 669 101 677H569H659Q691 677 697 676T704 667Q704 661 687 553T668 444Q668 437 649 437Q640 437 637 437T631 442L629 445Q629 451 635 490T641 551Q641 586 628 604T573 629Q568 630 515 631Q469 631 457 630T439 622Q438 621 368 343T298 60Q298 48 386 46Q418 46 427 45T436 36Q436 31 433 22Q429 4 424 1L422 0Q419 0 415 0Q410 0 363 1T228 2Q99 2 64 0H49Q43 6 43 9T45 27Q49 40 55 46H83H94Q174 46 189 55Q190 56 191 56Q196 59 201 76T241 233Q258 301 269 344Q339 619 339 625Q339 630 310 630H279Q212 630 191 624Q146 614 121 583T67 467Q60 445 57 441T43 437H40",
      120566: "45 535Q34 535 31 536T28 544Q28 554 39 578T70 631T126 683T206 705Q230 705 251 698T295 671T330 612T344 514Q344 477 342 473V472Q343 472 347 480T361 509T380 547Q471 704 596 704Q615 704 625 702Q659 692 679 663T700 595Q700 565 696 552T687 537T670 535Q656 535 653 536T649 543Q649 544 649 550T650 562Q650 589 629 605T575 621Q502 621 448 547T365 361Q290 70 290 60Q290 46 379 46H404Q410 40 410 39T408 19Q404 6 398 0H381Q340 2 225 2Q184 2 149 2T94 2T69 1Q61 1 58 1T53 4T51 10Q51 11 53 23Q54 25 55 30T56 36T58 40T60 43T62 44T67 46T73 46T82 46H89Q144 46 163 49T190 62L198 93Q206 124 217 169T241 262T262 350T274 404Q281 445 281 486V494Q281 621 185 621Q147 621 116 601T74 550Q71 539 66 537T45 535",
      120567: "356 624Q356 637 267 637H243Q237 642 237 645T239 664Q243 677 249 683H264Q342 681 429 681Q565 681 571 683H583Q589 677 589 674T587 656Q582 641 578 637H540Q516 637 504 637T479 633T463 630T454 623T448 613T443 597T438 576Q436 566 434 556T430 539L428 533Q442 533 472 526T543 502T613 451T642 373Q642 301 567 241T386 158L336 150Q332 150 331 146Q310 66 310 60Q310 46 399 46H424Q430 40 430 39T428 19Q424 6 418 0H401Q360 2 247 2Q207 2 173 2T119 2T95 1Q87 1 84 1T79 4T77 10Q77 11 79 23Q80 25 81 30T82 36T84 40T86 43T88 44T93 46T99 46T108 46H115Q170 46 189 49T216 62Q220 74 228 107L239 150L223 152Q139 164 82 205T24 311Q24 396 125 462Q207 517 335 533L346 578Q356 619 356 624ZM130 291Q130 203 241 188H249Q249 190 287 342L325 495H324Q313 495 291 491T229 466T168 414Q130 357 130 291ZM536 393Q536 440 507 463T418 496L341 187L351 189Q443 201 487 255Q536 314 536 393",
      120568: "42 0H40Q26 0 26 11Q26 15 29 27Q33 41 36 43T55 46Q141 49 190 98Q200 108 306 224T411 342Q302 620 297 625Q288 636 234 637H206Q200 643 200 645T202 664Q206 677 212 683H226Q260 681 347 681Q380 681 408 681T453 682T473 682Q490 682 490 671Q490 670 488 658Q484 643 481 640T465 637Q434 634 411 620L488 426L541 485Q646 598 646 610Q646 628 622 635Q617 635 609 637Q594 637 594 648Q594 650 596 664Q600 677 606 683H618Q619 683 643 683T697 681T738 680Q828 680 837 683H845Q852 676 852 672Q850 647 840 637H824Q790 636 763 628T722 611T698 593L687 584Q687 585 592 480L505 384Q505 383 536 304T601 142T638 56Q648 47 699 46Q734 46 734 37Q734 35 732 23Q728 7 725 4T711 1Q708 1 678 1T589 2Q528 2 496 2T461 1Q444 1 444 10Q444 11 446 25Q448 35 450 39T455 44T464 46T480 47T506 54Q523 62 523 64Q522 64 476 181L429 299Q241 95 236 84Q232 76 232 72Q232 53 261 47Q262 47 267 47T273 46Q276 46 277 46T280 45T283 42T284 35Q284 26 282 19Q279 6 276 4T261 1Q258 1 243 1T201 2T142 2Q64 2 42 0",
      120569: "216 151Q48 174 48 329Q48 361 56 403T65 458Q65 482 58 494T43 507T28 510T21 520Q21 528 23 534T29 544L32 546H72H94Q110 546 119 544T139 536T154 514T159 476V465Q159 445 149 399T138 314Q142 229 197 201Q223 187 226 190L233 218Q240 246 253 300T280 407Q333 619 333 625Q333 637 244 637H220Q214 642 214 645T216 664Q220 677 226 683H241Q321 681 405 681Q543 681 549 683H560Q566 677 566 674T564 656Q559 641 555 637H517Q448 636 436 628Q429 623 423 600T373 404L320 192Q370 201 419 248Q451 281 469 317T500 400T518 457Q529 486 542 505T569 532T594 543T621 546H644H669Q692 546 692 536Q691 509 676 509Q623 509 593 399Q587 377 579 355T552 301T509 244T446 195T359 159Q324 151 314 151Q311 151 310 150T298 106T287 60Q287 46 376 46H401Q407 40 407 39T405 19Q401 6 395 0H378Q337 2 224 2Q184 2 150 2T96 2T72 1Q64 1 61 1T56 4T54 10Q54 11 56 23Q57 25 58 30T59 36T61 40T63 43T65 44T70 46T76 46T85 46H92Q147 46 166 49T193 62L204 106Q216 149 216 151",
      120570: "125 84Q127 78 194 76H243V78Q243 122 208 215T165 350Q164 359 162 389Q162 522 272 610Q328 656 396 680T525 704Q628 704 698 661Q734 637 755 601T781 544T786 504Q786 439 747 374T635 226T537 109Q518 81 518 77Q537 76 557 76Q608 76 620 78T640 92Q646 100 656 119T673 155T683 172Q690 173 698 173Q718 173 718 162Q718 161 681 82T642 2Q639 0 550 0H461Q455 5 455 9T458 28Q472 78 510 149T584 276T648 402T677 525Q677 594 636 631T530 668Q476 668 423 641T335 568Q284 499 271 400Q270 388 270 348Q270 298 277 228T285 115Q285 82 280 49T271 6Q269 1 258 1T175 0H87Q83 3 80 7V18Q80 22 82 98Q84 156 85 163T91 172Q94 173 104 173T119 172Q124 169 124 126Q125 104 125 84",
      120572: "34 156Q34 270 120 356T309 442Q379 442 421 402T478 304Q484 275 485 237V208Q534 282 560 374Q564 388 566 390T582 393Q603 393 603 385Q603 376 594 346T558 261T497 161L486 147L487 123Q489 67 495 47T514 26Q528 28 540 37T557 60Q559 67 562 68T577 70Q597 70 597 62Q597 56 591 43Q579 19 556 5T512 -10H505Q438 -10 414 62L411 69L400 61Q390 53 370 41T325 18T267 -2T203 -11Q124 -11 79 39T34 156ZM208 26Q257 26 306 47T379 90L403 112Q401 255 396 290Q382 405 304 405Q235 405 183 332Q156 292 139 224T121 120Q121 71 146 49T208 26",
      120573: "29 -194Q23 -188 23 -186Q23 -183 102 134T186 465Q208 533 243 584T309 658Q365 705 429 705H431Q493 705 533 667T573 570Q573 465 469 396L482 383Q533 332 533 252Q533 139 448 65T257 -10Q227 -10 203 -2T165 17T143 40T131 59T126 65L62 -188Q60 -194 42 -194H29ZM353 431Q392 431 427 419L432 422Q436 426 439 429T449 439T461 453T472 471T484 495T493 524T501 560Q503 569 503 593Q503 611 502 616Q487 667 426 667Q384 667 347 643T286 582T247 514T224 455Q219 439 186 308T152 168Q151 163 151 147Q151 99 173 68Q204 26 260 26Q302 26 349 51T425 137Q441 171 449 214T457 279Q457 337 422 372Q380 358 347 358H337Q258 358 258 389Q258 396 261 403Q275 431 353 431",
      120574: "31 249Q11 249 11 258Q11 275 26 304T66 365T129 418T206 441Q233 441 239 440Q287 429 318 386T371 255Q385 195 385 170Q385 166 386 166L398 193Q418 244 443 300T486 391T508 430Q510 431 524 431H537Q543 425 543 422Q543 418 522 378T463 251T391 71Q385 55 378 6T357 -100Q341 -165 330 -190T303 -216Q286 -216 286 -188Q286 -138 340 32L346 51L347 69Q348 79 348 100Q348 257 291 317Q251 355 196 355Q148 355 108 329T51 260Q49 251 47 251Q45 249 31 249",
      120575: "195 609Q195 656 227 686T302 717Q319 716 351 709T407 697T433 690Q451 682 451 662Q451 644 438 628T403 612Q382 612 348 641T288 671T249 657T235 628Q235 584 334 463Q401 379 401 292Q401 169 340 80T205 -10H198Q127 -10 83 36T36 153Q36 286 151 382Q191 413 252 434Q252 435 245 449T230 481T214 521T201 566T195 609ZM112 130Q112 83 136 55T204 27Q233 27 256 51T291 111T309 178T316 232Q316 267 309 298T295 344T269 400L259 396Q215 381 183 342T137 256T118 179T112 130",
      120576: "190 -22Q124 -22 76 11T27 107Q27 174 97 232L107 239L99 248Q76 273 76 304Q76 364 144 408T290 452H302Q360 452 405 421Q428 405 428 392Q428 381 417 369T391 356Q382 356 371 365T338 383T283 392Q217 392 167 368T116 308Q116 289 133 272Q142 263 145 262T157 264Q188 278 238 278H243Q308 278 308 247Q308 206 223 206Q177 206 142 219L132 212Q68 169 68 112Q68 39 201 39Q253 39 286 49T328 72T345 94T362 105Q376 103 376 88Q376 79 365 62T334 26T275 -8T190 -22",
      120577: "296 643Q298 704 324 704Q342 704 342 687Q342 682 339 664T336 633Q336 623 337 618T338 611Q339 612 341 612Q343 614 354 616T374 618L384 619H394Q471 619 471 586Q467 548 386 546H372Q338 546 320 564L311 558Q235 506 175 398T114 190Q114 171 116 155T125 127T137 104T153 86T171 72T192 61T213 53T235 46T256 39L322 16Q389 -10 389 -80Q389 -119 364 -154T300 -202Q292 -204 274 -204Q247 -204 225 -196Q210 -192 193 -182T172 -167Q167 -159 173 -148Q180 -139 191 -139Q195 -139 221 -153T283 -168Q298 -166 310 -152T322 -117Q322 -91 302 -75T250 -51T183 -29T116 4T65 62T44 160Q44 287 121 410T293 590L302 595Q296 613 296 643",
      120578: "21 287Q22 290 23 295T28 317T38 348T53 381T73 411T99 433T132 442Q156 442 175 435T205 417T221 395T229 376L231 369Q231 367 232 367L243 378Q304 442 382 442Q436 442 469 415T503 336V326Q503 302 439 53Q381 -182 377 -189Q364 -216 332 -216Q319 -216 310 -208T299 -186Q299 -177 358 57L420 307Q423 322 423 345Q423 404 379 404H374Q288 404 229 303L222 291L189 157Q156 26 151 16Q138 -11 108 -11Q95 -11 87 -5T76 7T74 17Q74 30 114 189T154 366Q154 405 128 405Q107 405 92 377T68 316T57 280Q55 278 41 278H27Q21 284 21 287",
      120579: "35 200Q35 302 74 415T180 610T319 704Q320 704 327 704T339 705Q393 701 423 656Q462 596 462 495Q462 380 417 261T302 66T168 -10H161Q125 -10 99 10T60 63T41 130T35 200ZM383 566Q383 668 330 668Q294 668 260 623T204 521T170 421T157 371Q206 370 254 370L351 371Q352 372 359 404T375 484T383 566ZM113 132Q113 26 166 26Q181 26 198 36T239 74T287 161T335 307L340 324H145Q145 321 136 286T120 208T113 132",
      120580: "139 -10Q111 -10 92 0T64 25T52 52T48 74Q48 89 55 109T85 199T135 375L137 384Q139 394 140 397T145 409T151 422T160 431T173 439T190 442Q202 442 213 435T225 410Q225 404 214 358T181 238T137 107Q126 74 126 54Q126 43 126 39T130 31T142 27H147Q206 27 255 78Q272 98 281 114T290 138T295 149T313 153Q321 153 324 153T329 152T332 149T332 143Q332 106 276 48T145 -10H139",
      120581: "83 -11Q70 -11 62 -4T51 8T49 17Q49 30 96 217T147 414Q160 442 193 442Q205 441 213 435T223 422T225 412Q225 401 208 337L192 270Q193 269 208 277T235 292Q252 304 306 349T396 412T467 431Q489 431 500 420T512 391Q512 366 494 347T449 327Q430 327 418 338T405 368Q405 370 407 380L397 375Q368 360 315 315L253 266L240 257H245Q262 257 300 251T366 230Q422 203 422 150Q422 140 417 114T411 67Q411 26 437 26Q484 26 513 137Q516 149 519 151T535 153Q554 153 554 144Q554 121 527 64T457 -7Q447 -10 431 -10Q386 -10 360 17T333 90Q333 108 336 122T339 146Q339 170 320 186T271 209T222 218T185 221H180L155 122Q129 22 126 16Q113 -11 83 -11",
      120582: "166 673Q166 685 183 694H202Q292 691 316 644Q322 629 373 486T474 207T524 67Q531 47 537 34T546 15T551 6T555 2T556 -2T550 -11H482Q457 3 450 18T399 152L354 277L340 262Q327 246 293 207T236 141Q211 112 174 69Q123 9 111 -1T83 -12Q47 -12 47 20Q47 37 61 52T199 187Q229 216 266 252T321 306L338 322Q338 323 288 462T234 612Q214 657 183 657Q166 657 166 673",
      120583: "58 -216Q44 -216 34 -208T23 -186Q23 -176 96 116T173 414Q186 442 219 442Q231 441 239 435T249 423T251 413Q251 401 220 279T187 142Q185 131 185 107V99Q185 26 252 26Q261 26 270 27T287 31T302 38T315 45T327 55T338 65T348 77T356 88T365 100L372 110L408 253Q444 395 448 404Q461 431 491 431Q504 431 512 424T523 412T525 402L449 84Q448 79 448 68Q448 43 455 35T476 26Q485 27 496 35Q517 55 537 131Q543 151 547 152Q549 153 557 153H561Q580 153 580 144Q580 138 575 117T555 63T523 13Q510 0 491 -8Q483 -10 467 -10Q446 -10 429 -4T402 11T385 29T376 44T374 51L368 45Q362 39 350 30T324 12T288 -4T246 -11Q199 -11 153 12L129 -85Q108 -167 104 -180T92 -202Q76 -216 58 -216",
      120584: "74 431Q75 431 146 436T219 442Q231 442 231 434Q231 428 185 241L137 51H140L150 55Q161 59 177 67T214 86T261 119T312 165Q410 264 445 394Q458 442 496 442Q509 442 519 434T530 411Q530 390 516 352T469 262T388 162T267 70T106 5Q81 -2 71 -2Q66 -2 59 -1T51 1Q45 5 45 11Q45 13 88 188L132 364Q133 377 125 380T86 385H65Q59 391 59 393T61 412Q65 431 74 431",
      120585: "268 632Q268 704 296 704Q314 704 314 687Q314 682 311 664T308 635T309 620V616H315Q342 619 360 619Q443 619 443 586Q439 548 358 546H344Q326 546 317 549T290 566Q257 550 226 505T195 405Q195 381 201 364T211 342T218 337Q266 347 298 347Q375 347 375 314Q374 297 359 288T327 277T280 275Q234 275 208 283L195 286Q149 260 119 214T88 130Q88 116 90 108Q101 79 129 63T229 20Q238 17 243 15Q337 -21 354 -33Q383 -53 383 -94Q383 -137 351 -171T273 -205Q240 -205 202 -190T158 -167Q156 -163 156 -159Q156 -151 161 -146T176 -140Q182 -140 189 -143Q232 -168 274 -168Q286 -168 292 -165Q313 -151 313 -129Q313 -112 301 -104T232 -75Q214 -68 204 -64Q198 -62 171 -52T136 -38T107 -24T78 -8T56 12T36 37T26 66T21 103Q21 149 55 206T145 301L154 307L148 313Q141 319 136 323T124 338T111 358T103 382T99 413Q99 471 143 524T259 602L271 607Q268 618 268 632",
      120586: "201 -11Q126 -11 80 38T34 156Q34 221 64 279T146 380Q222 441 301 441Q333 441 341 440Q354 437 367 433T402 417T438 387T464 338T476 268Q476 161 390 75T201 -11ZM121 120Q121 70 147 48T206 26Q250 26 289 58T351 142Q360 163 374 216T388 308Q388 352 370 375Q346 405 306 405Q243 405 195 347Q158 303 140 230T121 120",
      120587: "132 -11Q98 -11 98 22V33L111 61Q186 219 220 334L228 358H196Q158 358 142 355T103 336Q92 329 81 318T62 297T53 285Q51 284 38 284Q19 284 19 294Q19 300 38 329T93 391T164 429Q171 431 389 431Q549 431 553 430Q573 423 573 402Q573 371 541 360Q535 358 472 358H408L405 341Q393 269 393 222Q393 170 402 129T421 65T431 37Q431 20 417 5T381 -10Q370 -10 363 -7T347 17T331 77Q330 86 330 121Q330 170 339 226T357 318T367 358H269L268 354Q268 351 249 275T206 114T175 17Q164 -11 132 -11",
      120588: "58 -216Q25 -216 23 -186Q23 -176 73 26T127 234Q143 289 182 341Q252 427 341 441Q343 441 349 441T359 442Q432 442 471 394T510 276Q510 219 486 165T425 74T345 13T266 -10H255H248Q197 -10 165 35L160 41L133 -71Q108 -168 104 -181T92 -202Q76 -216 58 -216ZM424 322Q424 359 407 382T357 405Q322 405 287 376T231 300Q217 269 193 170L176 102Q193 26 260 26Q298 26 334 62Q367 92 389 158T418 266T424 322",
      120589: "31 207Q31 306 115 374T302 442Q341 442 373 430T405 400Q405 392 399 383T379 374Q373 375 348 390T296 405Q222 405 160 357T98 249Q98 232 103 218T112 195T132 175T154 159T186 141T219 122Q234 114 255 102T286 85T299 78L302 74Q306 71 308 69T315 61T322 51T328 40T332 25T334 8Q334 -31 305 -69T224 -107Q194 -107 163 -92Q156 -88 156 -80Q156 -73 162 -67T178 -61Q186 -61 190 -63Q209 -71 224 -71Q244 -71 253 -59T263 -30Q263 -25 263 -21T260 -12T255 -4T248 3T239 9T227 17T213 25T195 34T174 46Q170 48 150 58T122 74T97 90T70 112T51 137T36 169T31 207",
      120590: "184 -11Q116 -11 74 34T31 147Q31 247 104 333T274 430Q275 431 414 431H552Q553 430 555 429T559 427T562 425T565 422T567 420T569 416T570 412T571 407T572 401Q572 357 507 357Q500 357 490 357T476 358H416L421 348Q439 310 439 263Q439 153 359 71T184 -11ZM361 278Q361 358 276 358Q152 358 115 184Q114 180 114 178Q106 141 106 117Q106 67 131 47T188 26Q242 26 287 73Q316 103 334 153T356 233T361 278",
      120591: "39 284Q18 284 18 294Q18 301 45 338T99 398Q134 425 164 429Q170 431 332 431Q492 431 497 429Q517 424 517 402Q517 388 508 376T485 360Q479 358 389 358T299 356Q298 355 283 274T251 109T233 20Q228 5 215 -4T186 -13Q153 -13 153 20V30L203 192Q214 228 227 272T248 336L254 357Q254 358 208 358Q206 358 197 358T183 359Q105 359 61 295Q56 287 53 286T39 284",
      120592: "413 384Q413 406 432 424T473 443Q492 443 507 425T523 367Q523 334 508 270T468 153Q424 63 373 27T282 -10H268Q220 -10 186 2T135 36T111 78T104 121Q104 170 138 262T173 379Q173 380 173 381Q173 390 173 393T169 400T158 404H154Q131 404 112 385T82 344T65 302T57 280Q55 278 41 278H27Q21 284 21 287Q21 299 34 333T82 404T161 441Q200 441 225 419T250 355Q248 336 247 334Q247 331 232 291T201 199T185 118Q185 68 211 47T275 26Q317 26 355 57T416 132T452 216T465 277Q465 301 457 318T439 343T421 361T413 384",
      120593: "92 210Q92 176 106 149T142 108T185 85T220 72L235 70L237 71L250 112Q268 170 283 211T322 299T370 375T429 423T502 442Q547 442 582 410T618 302Q618 224 575 152T457 35T299 -10Q273 -10 273 -12L266 -48Q260 -83 252 -125T241 -179Q236 -203 215 -212Q204 -218 190 -218Q159 -215 159 -185Q159 -175 214 -2L209 0Q204 2 195 5T173 14T147 28T120 46T94 71T71 103T56 142T50 190Q50 238 76 311T149 431H162Q183 431 183 423Q183 417 175 409Q134 361 114 300T92 210ZM574 278Q574 320 550 344T486 369Q437 369 394 329T323 218Q309 184 295 109L286 64Q304 62 306 62Q423 62 498 131T574 278",
      120594: "576 -125Q576 -147 547 -175T487 -204H476Q394 -204 363 -157Q334 -114 293 26L284 59Q283 58 248 19T170 -66T92 -151T53 -191Q49 -194 43 -194Q36 -194 31 -189T25 -177T38 -154T151 -30L272 102L265 131Q189 405 135 405Q104 405 87 358Q86 351 68 351Q48 351 48 361Q48 369 56 386T89 423T148 442Q224 442 258 400Q276 375 297 320T330 222L341 180Q344 180 455 303T573 429Q579 431 582 431Q600 431 600 414Q600 407 587 392T477 270Q356 138 353 134L362 102Q392 -10 428 -89T490 -168Q504 -168 517 -156T536 -126Q539 -116 543 -115T557 -114T571 -115Q576 -118 576 -125",
      120595: "161 441Q202 441 226 417T250 358Q250 338 218 252T187 127Q190 85 214 61Q235 43 257 37Q275 29 288 29H289L371 360Q455 691 456 692Q459 694 472 694Q492 694 492 687Q492 678 411 356Q329 28 329 27T335 26Q421 26 498 114T576 278Q576 302 568 319T550 343T532 361T524 384Q524 405 541 424T583 443Q602 443 618 425T634 366Q634 337 623 288T605 220Q573 125 492 57T329 -11H319L296 -104Q272 -198 272 -199Q270 -205 252 -205H239Q233 -199 233 -197Q233 -192 256 -102T279 -9Q272 -8 265 -8Q106 14 106 139Q106 174 139 264T173 379Q173 380 173 381Q173 390 173 393T169 400T158 404H154Q131 404 112 385T82 344T65 302T57 280Q55 278 41 278H27Q21 284 21 287Q21 299 34 333T82 404T161 441",
      120596: "495 384Q495 406 514 424T555 443Q574 443 589 425T604 364Q604 334 592 278T555 155T483 38T377 -11Q297 -11 267 66Q266 68 260 61Q201 -11 125 -11Q15 -11 15 139Q15 230 56 325T123 434Q135 441 147 436Q160 429 160 418Q160 406 140 379T94 306T62 208Q61 202 61 187Q61 124 85 100T143 76Q201 76 245 129L253 137V156Q258 297 317 297Q348 297 348 261Q348 243 338 213T318 158L308 135Q309 133 310 129T318 115T334 97T358 83T393 76Q456 76 501 148T546 274Q546 305 533 325T508 357T495 384",
      120597: "202 508Q179 508 169 520T158 547Q158 557 164 577T185 624T230 675T301 710L333 715H345Q378 715 384 714Q447 703 489 661T549 568T566 457Q566 362 519 240T402 53Q321 -22 223 -22Q123 -22 73 56Q42 102 42 148V159Q42 276 129 370T322 465Q383 465 414 434T455 367L458 378Q478 461 478 515Q478 603 437 639T344 676Q266 676 223 612Q264 606 264 572Q264 547 246 528T202 508ZM430 306Q430 372 401 400T333 428Q270 428 222 382Q197 354 183 323T150 221Q132 149 132 116Q132 21 232 21Q244 21 250 22Q327 35 374 112Q389 137 409 196T430 306",
      120598: "227 -11Q149 -11 95 41T40 174Q40 262 87 322Q121 367 173 396T287 430Q289 431 329 431H367Q382 426 382 411Q382 385 341 385H325H312Q191 385 154 277L150 265H327Q340 256 340 246Q340 228 320 219H138V217Q128 187 128 143Q128 77 160 52T231 26Q258 26 284 36T326 57T343 68Q350 68 354 58T358 39Q358 36 357 35Q354 31 337 21T289 0T227 -11",
      120599: "537 500Q537 474 533 439T524 383L521 362Q558 355 561 351Q563 349 563 345Q563 321 552 318Q542 318 521 323L510 326Q496 261 459 187T362 51T241 -11Q100 -11 100 105Q100 139 127 242T154 366Q154 405 128 405Q107 405 92 377T68 316T57 280Q55 278 41 278H27Q21 284 21 287Q21 291 27 313T47 368T79 418Q103 442 134 442Q169 442 201 419T233 344Q232 330 206 228T180 98Q180 26 247 26Q292 26 332 90T404 260L427 349Q422 349 398 359T339 392T289 440Q265 476 265 520Q265 590 312 647T417 705Q463 705 491 670T528 592T537 500ZM464 564Q464 668 413 668Q373 668 339 622T304 522Q304 494 317 470T349 431T388 406T421 391T435 387H436L443 415Q450 443 457 485T464 564",
      120600: "228 325Q170 322 156 316T127 309Q108 309 104 314Q99 319 99 322T108 341Q125 376 171 400T268 425H271Q302 425 319 396Q328 377 328 358Q328 332 324 314Q311 270 286 221Q274 194 274 192H275Q339 234 484 325T639 421Q669 434 691 434T723 425T734 406Q734 394 719 381Q715 376 644 330L575 287L566 267Q543 233 526 176Q520 160 515 143T508 115T506 105Q506 103 533 103Q585 103 607 110T641 118Q670 118 670 107Q670 100 661 85Q643 50 598 27T504 3Q465 3 450 36Q441 51 441 73Q441 84 444 96Q452 146 484 205L497 236L324 125Q143 12 135 10Q103 -6 77 -6Q61 -6 49 2T37 21Q37 36 49 46T124 96L195 141L204 156Q219 179 243 248T264 323Q264 325 228 325",
      120601: "409 688Q413 694 421 694H429H442Q448 688 448 686Q448 679 418 563Q411 535 404 504T392 458L388 442Q388 441 397 441T429 435T477 418Q521 397 550 357T579 260T548 151T471 65T374 11T279 -10H275L251 -105Q245 -128 238 -160Q230 -192 227 -198T215 -205H209Q189 -205 189 -198Q189 -193 211 -103L234 -11Q234 -10 226 -10Q221 -10 206 -8T161 6T107 36T62 89T43 171Q43 231 76 284T157 370T254 422T342 441Q347 441 348 445L378 567Q409 686 409 688ZM122 150Q122 116 134 91T167 53T203 35T237 27H244L337 404Q333 404 326 403T297 395T255 379T211 350T170 304Q152 276 137 237Q122 191 122 150ZM500 282Q500 320 484 347T444 385T405 400T381 404H378L332 217L284 29Q284 27 285 27Q293 27 317 33T357 47Q400 66 431 100T475 170T494 234T500 282",
      120602: "205 -174Q136 -174 102 -153T67 -76Q67 -25 91 85T127 234Q143 289 182 341Q252 427 341 441Q343 441 349 441T359 442Q432 442 471 394T510 276Q510 169 431 80T253 -10Q226 -10 204 -2T169 19T146 44T132 64L128 73Q128 72 124 53T116 5T112 -44Q112 -68 117 -78T150 -95T236 -102Q327 -102 356 -111T386 -154Q386 -166 384 -178Q381 -190 378 -192T361 -194H348Q342 -188 342 -179Q342 -169 315 -169Q294 -169 264 -171T205 -174ZM424 322Q424 359 407 382T357 405Q322 405 287 376T231 300Q221 276 204 217Q188 152 188 116Q188 68 210 47T259 26Q297 26 334 62Q367 92 389 158T418 266T424 322",
      120603: "206 -10Q158 -10 136 24T114 110Q114 233 199 349L205 358H184Q144 358 121 347Q108 340 95 330T75 312T61 295T53 285Q51 284 38 284Q19 284 19 294Q19 300 38 329T93 391T164 429Q171 431 532 431Q799 431 803 430Q823 423 823 402Q823 377 801 364Q790 358 766 358Q748 358 748 357Q748 355 749 348T752 327T754 297Q754 258 738 207T693 107T618 24T520 -10Q488 -10 466 2T432 36T416 77T411 120Q411 128 410 128T404 122Q373 71 323 31T206 -10ZM714 296Q714 316 707 358H251Q250 357 244 348T230 328T212 301T193 267T176 229T164 187T159 144Q159 62 222 62Q290 62 349 127T432 285Q433 286 434 288T435 291T437 293T440 294T444 294T452 294H466Q472 288 472 286Q472 285 464 244T456 170Q456 62 534 62Q604 62 659 139T714 296",
      120604: "65 0Q45 0 45 18Q48 52 61 60Q65 62 81 62Q155 62 165 74Q166 74 265 228T465 539T569 699Q576 707 583 709T611 711T637 710T649 700Q650 697 695 380L741 63L784 62H827Q839 50 839 45L835 29Q831 9 827 5T806 0Q803 0 790 0T743 1T657 2Q585 2 547 1T504 0Q481 0 481 17Q484 54 497 60Q501 62 541 62Q580 62 580 63Q580 68 573 121T564 179V181H308L271 124Q236 69 236 67T283 62H287Q316 62 316 46Q316 26 307 8Q302 3 295 0L262 1Q242 2 168 2Q119 2 93 1T65 0ZM537 372Q533 402 528 435T521 486T518 504V505Q517 505 433 375L348 244L451 243Q555 243 555 244L537 372",
      120605: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 449 686H477H586Q684 686 733 677T817 634Q853 598 853 547Q853 499 826 460T761 401T695 371T654 360H653L662 358Q670 357 683 354T712 344T744 327T774 303T795 269T804 224Q804 148 732 79T533 1Q524 0 288 0H58Q47 5 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624ZM703 550Q703 571 695 586T675 609T656 619T643 623L545 624H447L417 504Q386 384 386 383T470 382Q554 383 565 385Q632 397 667 447T703 550ZM651 240Q651 265 645 282T626 309T608 322T592 329Q587 330 479 331H373L340 198Q307 65 306 64Q306 62 406 62L507 63L519 65Q565 76 596 107T639 171T651 240",
      120606: "257 618H231Q198 618 198 636Q202 672 214 678L219 680H763Q769 677 772 673T776 666L777 664Q777 659 766 549T751 433Q745 424 723 424Q704 424 699 427T693 441Q693 444 695 467T697 513Q697 543 689 563T670 594T636 610T592 617T534 618H516H456L455 614Q455 613 387 339T317 64Q317 62 375 62H411Q430 62 438 59T447 44Q444 7 430 2Q426 0 416 0Q409 0 359 1T231 2Q152 2 111 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L163 66Q163 67 231 341T301 616Q301 618 257 618",
      120607: "65 0Q59 6 59 9T61 16Q64 20 334 357T608 698Q616 706 629 710Q630 710 634 710T644 710T656 711Q686 711 694 703Q698 699 700 693Q706 674 805 345T904 14Q904 7 894 1L479 0H65ZM630 342L567 551L232 134L462 133H693Q693 137 630 342",
      120608: "257 618H231Q198 618 198 636Q202 672 214 678L219 680H811Q817 677 820 673T824 666L825 664Q825 659 814 549T799 433Q793 424 771 424Q752 424 746 427T740 441Q740 445 742 466T744 505Q744 561 722 585T646 616Q639 617 545 618H456Q456 617 427 502T398 385Q398 384 435 384Q461 385 471 385T499 391T526 405T545 433T562 478Q566 494 571 497T595 501H604Q622 501 626 486Q626 482 593 349T557 213Q552 205 530 205Q499 205 499 219Q499 222 503 242T508 281Q508 308 491 314T429 322Q425 322 423 322H382L317 64Q317 62 390 62Q460 62 493 64T569 80T640 124Q665 149 686 187T719 253T733 283Q739 289 760 289Q791 289 791 274Q791 267 763 201T706 71L678 8Q676 4 667 0H58Q47 5 43 15Q47 54 60 60Q64 62 113 62H162L163 66Q163 67 231 341T301 616Q301 618 257 618",
      120609: "223 430Q192 430 192 448Q192 450 225 561T261 677Q265 683 270 684Q273 686 534 686Q796 686 797 685Q805 682 805 673Q805 668 804 661T800 648T798 641Q796 637 531 352L266 67L329 66H364Q412 66 446 70T523 96T596 157Q617 186 630 220T649 273T663 297Q667 299 684 299H688Q715 299 715 281Q715 278 673 145T628 8Q626 4 617 0H348Q289 0 221 0T139 -1Q112 -1 99 -1T78 1T69 5T68 12Q68 16 71 31T77 49L84 57Q91 65 104 79T133 110T170 151T213 196L610 624H540Q533 624 514 624T488 624T467 623T443 620T422 616T398 609T373 600Q292 560 255 449Q251 436 246 433T223 430",
      120610: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 239 686Q290 684 403 684Q475 684 512 685T553 686Q576 686 576 668Q572 632 560 626Q555 624 506 624H457L399 389Q399 388 547 388H695L753 623Q753 624 709 624H686Q665 624 660 626T650 639Q653 678 668 684Q672 686 681 686Q685 686 726 685T847 684Q902 684 937 684T986 685T1004 686Q1027 686 1027 668Q1023 632 1011 626Q1006 624 957 624H908L839 344Q768 64 768 63T812 62H839Q871 62 871 44Q867 6 854 2L850 0L808 1Q782 2 675 2Q600 2 560 1T516 0Q499 0 494 15Q498 54 511 60Q515 62 564 62H613L614 66L679 324Q679 326 531 326H383L382 322L317 64Q317 62 361 62H388Q420 62 420 44Q416 6 403 2L399 0L357 1Q331 2 224 2Q149 2 109 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624",
      120611: "358 -17Q218 -17 136 49T54 243Q54 298 70 356T123 474T211 582T338 663T504 702H527Q578 702 590 701Q709 688 776 622T844 441Q844 379 825 315T765 192T668 86T532 11T358 -17ZM700 474Q700 525 685 561T642 616T587 643T528 652Q390 652 301 534Q252 472 225 373T198 210Q198 160 214 125T256 71T311 44T372 36Q484 36 571 119Q639 189 669 299T700 474ZM366 428Q366 425 364 419T362 411H466L570 412L573 422Q576 437 581 441T604 445Q620 445 623 444Q636 440 636 429Q636 423 616 340T593 253Q586 243 572 243H566Q545 243 539 249Q536 251 535 253T534 258T534 263T535 270T537 277H329L326 266Q323 251 318 247T295 243Q279 243 276 244Q263 248 263 259Q263 265 283 346Q288 366 295 394Q304 431 308 438T326 445H334H338Q366 445 366 428",
      120612: "247 624Q242 624 233 624T220 623Q186 623 186 640Q186 647 190 664T202 684Q206 686 226 686Q277 684 393 684Q435 684 471 684T528 685T553 686Q573 686 573 670Q573 650 564 632Q556 624 537 624H501H449L380 344Q309 64 309 63T356 62Q361 62 370 62T384 63Q417 63 417 46Q417 26 408 8Q403 3 396 0L352 1Q325 2 216 2T82 1L45 0Q30 7 30 16Q33 51 46 60Q51 62 102 62H154L294 623Q294 624 247 624",
      120613: "536 0Q522 6 522 18Q522 35 533 57Q539 62 557 62Q595 62 601 65L472 330L365 255L342 160Q318 65 317 64Q317 62 361 62H388Q420 62 420 44Q416 6 403 2L399 0L357 1Q331 2 224 2Q149 2 109 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L302 623Q302 624 258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 239 686Q290 684 403 684Q475 684 512 685T553 686Q576 686 576 668Q572 632 560 626Q555 624 506 624H457L422 481Q386 339 386 337L785 621Q779 624 749 624Q726 624 726 641Q726 645 730 659Q734 675 736 679T747 686L786 685Q812 684 888 684Q908 684 934 685T968 686Q1003 686 1003 669Q1003 646 991 629Q985 624 967 624Q918 624 888 617Q884 617 874 613L865 609Q864 608 732 515T599 420Q599 418 686 242T775 65Q784 62 829 62Q847 62 850 61T860 54Q862 52 862 43Q862 10 845 1Q844 1 842 1T836 0T797 1T694 2Q599 2 573 1L536 0",
      120614: "439 0Q425 6 425 18Q425 35 436 57Q442 62 485 62Q525 62 525 64L478 483Q478 484 465 463T422 394T350 277Q222 69 222 68Q223 67 224 67Q229 64 271 62Q290 62 297 59T305 45Q305 38 302 28Q297 9 293 5T274 0Q270 0 238 1T159 2Q133 2 105 2T72 1Q56 1 52 3T44 15Q44 19 48 35Q53 55 58 58T89 62Q142 64 151 73Q154 76 345 387T538 699Q550 711 570 711H580H592Q613 711 618 695Q619 692 654 379T690 63Q690 62 726 62H746Q776 62 776 44Q773 7 759 2Q755 0 747 0Q743 0 707 1T600 2Q502 2 476 1L439 0",
      120615: "258 624H231Q214 624 208 626T199 639Q203 678 216 684Q220 686 347 686H473Q474 685 478 682T484 677Q487 673 535 413L584 153L608 187Q631 221 672 281T761 410Q935 663 943 671Q949 678 962 686H1082H1166Q1201 686 1210 683T1219 668Q1215 632 1203 626Q1199 624 1149 624H1100L1031 344Q960 64 960 63T1004 62H1031Q1063 62 1063 44Q1060 7 1046 2Q1042 0 1034 0Q1030 0 990 1T875 2Q804 2 767 1T725 0H723Q707 0 703 15Q707 54 720 60Q724 62 773 62H822Q961 618 961 619L754 318Q546 15 543 12Q531 0 510 0Q500 0 495 0T484 5T477 19Q477 20 421 315L367 604L299 335Q234 72 234 68Q234 62 302 62Q334 62 334 46Q332 8 317 2Q313 0 306 0Q301 0 267 1T181 2Q125 2 96 1T63 0Q48 0 43 15Q43 19 47 35Q52 55 57 58T94 62Q147 64 164 69L233 345Q302 619 302 622Q302 624 258 624",
      120616: "258 624H235Q214 624 209 626T199 639Q203 678 216 684Q220 686 344 686H434Q464 686 477 680Q480 677 607 454Q738 227 739 227Q742 227 789 418T836 618Q836 620 835 620L821 622Q811 622 779 624Q755 624 749 625T740 632Q737 635 737 644Q737 656 742 669T754 685Q755 685 757 685T763 686Q768 686 803 685T890 684Q925 684 951 684T990 685T1006 686Q1014 686 1016 684Q1027 679 1027 668Q1023 632 1011 626Q1007 624 978 624Q912 622 907 617Q907 616 831 314T753 8Q749 0 723 0H712Q699 0 692 7Q692 8 671 44T607 155T526 296L361 580L296 323Q234 74 234 68T302 62H307Q334 62 334 44Q330 6 317 2L313 0L280 1Q260 2 181 2Q125 2 96 1T63 0Q48 0 43 15Q43 19 47 35Q52 55 57 58T94 62Q147 64 164 69L233 345Q302 619 302 622Q302 624 258 624",
      120617: "206 466Q175 466 175 484Q175 487 201 574T230 666Q235 673 241 675H547Q853 675 857 673Q867 668 867 657Q867 655 850 569T832 478Q827 466 808 466H802H795Q773 466 771 481Q771 486 775 508T779 541V549H774Q755 552 505 552Q281 551 267 548Q262 548 255 533T242 496T233 472Q228 466 206 466ZM259 255H252Q231 255 228 270Q228 275 248 355T270 439Q277 448 288 448H298Q321 448 326 440Q331 434 326 414Q326 413 486 413H647L650 424Q654 441 658 444T678 448H683H693Q702 448 705 446T714 432L694 348Q674 267 669 261Q664 255 652 255H643Q622 255 617 261Q613 265 613 272T613 283T616 289Q616 290 456 290H295L294 285Q293 280 292 275T288 265T281 257Q278 255 259 255ZM150 131Q150 124 281 123Q346 123 390 123Q640 123 664 126Q668 127 675 127Q686 131 704 198Q708 213 713 216T733 220H738Q755 220 759 218Q768 213 768 203Q768 198 741 105T710 8Q708 4 699 0H388Q77 0 73 2Q62 7 62 18Q62 27 81 115Q99 206 102 212Q106 220 120 220H129Q140 220 145 220T155 215T160 202Q160 196 159 192Q150 145 150 131",
      120618: "53 245Q53 297 70 356T125 478T216 590T349 671T523 703Q656 703 735 637T815 445Q815 378 791 307Q727 104 527 17Q437 -17 344 -17Q289 -17 242 -5T150 35T79 116T53 245ZM664 489Q664 575 618 611T511 648Q463 648 416 627T334 570Q297 531 270 472T230 355T213 261T208 206Q208 177 215 151T237 98T284 56T358 40Q440 40 510 98T618 270Q664 400 664 489",
      120619: "257 618H231Q198 618 198 636Q202 672 214 678L219 680H1011Q1022 675 1026 665Q1022 626 1009 620Q1005 618 956 618H907L906 614Q906 613 838 339T768 64Q768 62 812 62H839Q871 62 871 44Q867 6 854 2L850 0L808 1Q782 2 675 2Q600 2 560 1T516 0Q499 0 494 15Q498 54 511 60Q515 62 564 62H613L614 66Q614 67 682 341T752 616Q752 618 604 618H456L455 614Q455 613 387 339T317 64Q317 62 361 62H388Q420 62 420 44Q416 6 403 2L399 0L357 1Q331 2 224 2Q149 2 109 1T65 0Q48 0 43 15Q47 54 60 60Q64 62 113 62H162L163 66Q163 67 231 341T301 616Q301 618 257 618",
      120620: "162 62L302 623Q302 624 258 624H234Q214 624 209 626T200 638Q200 677 217 684Q220 686 439 686Q667 685 684 682Q686 681 693 680Q713 677 733 671T782 649T829 602T847 528Q847 450 784 382T604 293Q571 288 469 287H373L346 176Q340 151 333 122T321 78L317 64Q317 62 361 62H387Q420 62 420 44Q417 10 404 2L399 0L357 1Q331 2 224 2Q149 2 109 1T65 0Q43 0 43 17Q43 21 47 33Q52 54 57 58T89 62H113H162ZM692 558Q692 611 617 622Q610 623 529 624H452L381 343H458H492Q604 343 641 389Q662 414 677 471T692 558",
      120622: "847 430Q828 430 823 434T817 450Q817 454 817 466T818 487Q818 526 809 551T784 591T737 613T675 622T590 624H528H430L513 487Q594 351 596 345Q596 335 590 330Q583 323 418 204L250 81L363 80Q533 80 591 89T694 142Q739 185 765 252Q772 268 776 271T799 274Q816 274 820 272Q832 266 830 254Q829 250 784 130T736 7Q732 3 725 0H405Q84 0 80 2Q69 7 69 18Q69 26 75 32Q76 32 98 48T168 100T255 164L432 293Q429 300 329 465T225 637Q223 675 245 686H888Q900 680 902 671Q902 667 890 556T876 441Q871 430 847 430",
      120623: "498 62Q511 58 511 43Q511 10 494 1L490 0Q487 0 482 0T424 1T271 2Q201 2 157 2T94 1T72 0H70Q46 0 46 17Q49 54 62 60L66 62H137Q208 62 209 63L218 98Q227 134 244 203T278 339L347 613H300Q262 612 246 611T198 599T146 564Q128 545 114 512T91 454T79 425Q73 419 52 419Q22 419 22 434Q22 440 41 498T80 611L100 666Q105 673 111 675H434Q758 675 762 673Q772 668 772 657Q772 655 756 549T738 434Q735 419 711 419H707Q690 419 686 421Q677 425 677 434Q676 436 678 449T683 485T686 529Q686 553 679 569T662 594T631 607T593 612T544 613H502L433 340Q418 279 400 207T374 100L365 65L364 62H498",
      120624: "32 544Q32 586 91 644T229 703Q277 703 311 683T363 628T389 560T397 491V478L404 491Q455 589 526 646T677 703Q730 703 766 671T802 584Q802 551 793 541T766 531H757L736 532L732 535L729 539V549Q731 569 715 575T658 581H650Q545 581 477 443Q453 392 443 351Q441 345 424 273T389 133T371 64Q371 62 428 62H461Q483 62 492 59T501 44Q498 10 485 2L480 0L431 1Q401 2 278 2T127 1L85 0Q71 5 71 17Q71 24 74 33Q77 46 78 49T84 57T95 61T118 62H154H216Q232 126 249 193T273 287T287 345T296 388T299 416T300 452Q294 581 198 581Q154 581 132 575T106 562T99 546T86 533Q82 531 60 531L39 532Q32 537 32 544",
      120625: "323 624H293Q267 624 261 626T251 639Q255 678 268 684Q272 686 293 686Q348 684 475 684Q557 684 600 685T647 686H648Q671 686 671 668Q667 632 655 626Q650 624 588 624H525L500 520Q500 519 520 518T579 507T656 480Q737 440 737 372Q737 294 648 237Q562 180 426 169L412 168L399 118Q386 66 386 65L385 62H443H479Q498 62 506 59T515 44Q511 8 499 2L494 0L447 1Q417 2 298 2Q212 2 167 1T118 0Q100 0 95 15Q99 54 112 60Q116 62 179 62H241Q242 64 254 114T266 167Q266 168 262 168Q256 168 237 170T180 181T110 205T54 249T29 316Q29 391 112 446T327 516Q345 518 349 518Q351 518 353 518L355 519Q356 520 368 570T381 623Q381 624 323 624ZM342 466Q341 467 339 467Q320 467 283 455T225 420Q181 361 181 296Q181 273 193 257T222 233T254 222T277 219L280 220Q281 220 311 342T342 466ZM583 389Q583 409 576 423T557 444T533 456T509 463T492 467H486L455 343Q444 300 437 271T428 231T426 219Q430 219 445 222T483 232T521 250Q551 269 567 310T583 389",
      120626: "931 686Q953 686 953 670Q953 650 944 632Q936 624 924 624H914Q823 624 803 611Q800 609 696 503T591 396Q591 394 667 229L743 62H787H814Q846 62 846 44Q843 7 829 2Q825 0 817 0Q813 0 775 1T664 2Q590 2 551 1T508 0H507Q484 0 484 18Q484 19 488 37Q492 56 497 58T534 62L566 63Q567 64 520 169T471 274Q469 274 369 172T268 67L315 62Q320 62 328 62L335 61Q347 58 347 44Q344 10 331 2L326 0L287 1Q263 2 177 2Q95 2 78 1L53 0Q38 6 38 17Q38 40 50 57Q56 62 78 62Q169 62 188 75Q194 77 435 324L444 334L439 347Q437 351 373 492L313 624H268H246Q220 624 212 632Q210 636 210 642Q210 655 215 669T227 684Q230 686 247 686Q295 684 398 684Q438 684 472 684T527 685T551 686Q567 686 572 671Q572 667 568 651Q563 631 558 628T523 624T492 623H488L526 540Q563 457 564 457Q564 456 574 466T604 496T645 537L724 619Q716 622 677 624H673Q645 624 645 640Q645 660 654 678Q659 683 666 686L704 685Q728 684 813 684Q847 684 873 684T913 685T931 686",
      120627: "205 471Q205 445 196 403T186 333Q186 303 194 281T218 248T240 233T262 224L361 623Q361 624 303 624Q296 624 284 624T266 623Q232 623 232 641Q232 648 235 657Q240 678 244 682T268 686H273Q329 684 457 684Q502 684 540 684T599 685T626 686H628Q651 686 651 668Q649 633 631 624H505L407 227Q410 228 416 229T439 239T472 259T507 294T539 345Q549 365 563 416T597 498T649 538Q657 540 717 540Q725 540 737 540T755 541Q790 541 790 524Q790 512 784 497Q780 491 767 490T742 477Q736 471 731 463T722 449T715 433T710 419T705 403T701 389Q686 340 658 302T599 240T530 201T463 179T404 169L391 168L379 116Q365 67 365 63Q365 62 422 62H455Q477 62 486 59T495 44Q492 10 479 2L474 0L427 1Q397 2 278 2T131 1L90 0Q76 5 76 17Q76 24 79 33Q82 46 83 49T89 57T100 61T123 62H159H221Q247 162 247 168H244Q241 169 239 169Q202 176 178 182T123 207T74 252Q46 291 46 351Q46 375 52 404T59 454Q59 489 40 489Q32 489 27 494T22 507Q22 535 37 538Q40 540 99 540H128Q168 540 186 528T205 471",
      120628: "162 119Q181 115 235 115L273 116Q273 133 231 222T180 345Q173 368 173 391V406V414Q173 477 214 540Q255 600 315 635Q353 661 423 682T585 703Q656 703 711 690T799 656T851 608T879 555T886 503Q886 449 860 401Q840 359 810 322T725 230T643 146Q619 117 619 116T650 115Q707 115 722 120Q730 123 750 165T775 210Q779 212 796 212Q828 212 828 196Q828 191 807 144T764 52L743 7Q740 4 740 4T733 2T717 0T686 0H632H573Q535 0 526 3T517 17Q517 44 544 103T617 243T671 341Q729 454 729 535Q729 599 686 625T583 652Q549 652 517 645T450 616T388 561T344 470T327 340Q327 304 331 237T336 135Q336 93 330 50T313 2Q308 0 208 0H142Q107 0 100 4T93 25Q93 28 93 41T95 77T96 118L100 199Q105 208 109 210T131 212Q147 212 151 210T161 199V160Q161 131 162 125V119",
      120630: "39 166Q39 213 59 261T117 353T219 424T362 452Q420 452 466 433T536 384T573 325T586 269V265Q593 272 609 308T636 381Q640 397 644 399T669 402H680Q700 402 700 388Q700 379 691 351T659 276T604 188L593 173L595 153Q600 79 612 43H618Q634 45 642 51T653 64T658 71Q661 73 684 73Q712 73 712 59Q712 39 685 16T603 -7Q588 -7 575 -5T551 2T532 12T516 24T503 37T494 49T487 60T481 69L469 61Q362 -8 251 -8Q159 -8 99 36T39 166ZM260 43Q310 43 361 63T438 101T465 124Q458 240 453 277Q435 401 354 401Q291 401 245 355Q230 337 217 313Q201 279 186 216T170 126Q170 72 208 54Q230 43 260 43",
      120631: "59 -194H49Q31 -194 28 -182Q28 -178 107 139T192 473Q212 533 248 580T324 652T395 689T450 701H461Q514 701 551 688T605 652T630 607T637 561Q637 546 634 526T611 465T556 393Q572 382 590 347T608 262Q608 146 522 69T299 -8Q279 -8 261 -6T228 2T204 13T183 26T169 37T157 48L150 56L120 -64Q113 -90 104 -128Q93 -175 89 -184T73 -194H59ZM531 592Q531 651 463 651Q399 651 341 600T253 466Q250 458 217 327T182 185Q180 176 180 159Q180 108 212 76T301 44Q330 44 354 51T393 65T423 91T444 118T459 151T468 179T475 206Q490 264 491 296Q491 313 489 326T484 345L482 350Q481 350 477 348T464 344T444 340T413 335T372 333T334 334T301 340T274 355T265 380Q265 444 397 444Q425 444 445 441T476 436L485 433Q489 433 499 458Q509 482 520 527T531 592ZM424 390Q424 393 389 393Q383 393 374 393T362 392Q348 392 333 388Q345 384 379 384Q424 384 424 390",
      120632: "5 269Q5 285 19 312T57 368T124 421T215 451H241Q274 451 303 439T353 406T389 361T416 311T432 266T442 232L444 220L446 216L450 226Q473 278 513 357T561 441Q566 444 584 444H594Q617 444 617 430Q617 426 596 389T536 273T462 110L452 84L451 70Q447 12 427 -76T388 -192Q375 -211 355 -211Q339 -211 332 -198T325 -171Q325 -114 386 64L393 84V98Q393 181 371 241Q360 280 319 303T210 327Q158 327 126 317T84 296T68 272T59 258Q55 256 36 256Q23 256 18 256T9 260T5 269",
      120633: "216 610Q216 640 229 664T262 700T299 719T327 725Q330 725 406 709T487 690Q513 681 513 651Q513 627 494 607T450 587Q417 587 378 631Q346 663 314 663Q286 663 272 639Q271 637 271 634Q271 609 344 536L397 484Q438 448 458 410T478 313Q478 234 443 147T338 18Q298 -8 249 -8Q214 -8 180 0T113 26T60 81T39 168Q39 200 50 237T87 316T160 391T272 442L260 465Q216 553 216 610ZM348 235Q348 274 336 313T310 372L298 392Q295 391 290 390T269 380T241 359T212 323T185 267Q157 168 157 130Q157 83 186 63T255 43Q280 43 300 67Q317 89 332 138T348 235",
      120634: "224 -17Q126 -17 81 22T36 112Q36 178 84 226L93 236L88 246Q79 264 79 289Q79 341 124 388Q201 461 333 461Q402 461 455 425Q480 409 481 390Q481 365 464 350T428 334Q415 334 387 352T313 370Q141 370 141 293Q141 275 146 270Q147 270 148 270T155 272Q202 291 263 291H270Q349 291 349 244Q349 195 281 183Q274 182 239 182Q201 182 184 185T137 200Q123 188 112 168T100 129T112 98T148 81T189 75T237 74H243H251Q288 74 310 75T359 86T398 112Q405 124 417 124Q426 124 432 117T439 100Q439 88 427 70T390 32T322 -3T224 -17ZM286 238Q286 240 253 240Q245 240 234 239T216 237T209 235Q209 232 250 232Q286 232 286 238",
      120635: "361 711Q373 711 381 703T389 683Q389 681 388 676T383 656T376 618V612H379Q385 618 429 618Q521 618 521 572Q521 551 506 534Q483 510 415 507Q385 507 371 511T336 536L326 528Q254 472 204 381T154 209Q154 190 157 177Q159 165 162 154T170 135T182 119T195 106T212 95T229 86T249 78T269 72T290 66T311 60Q385 37 388 36Q437 14 454 -36Q456 -46 456 -64Q456 -83 455 -90Q445 -132 413 -167T333 -202Q300 -202 257 -191T206 -169Q203 -164 203 -158Q203 -148 210 -140T231 -130Q239 -130 263 -139T326 -151H329Q337 -151 342 -150T352 -143T357 -123Q356 -117 355 -113T350 -104T344 -96T335 -90T324 -85T310 -80T294 -74T275 -68T254 -62Q253 -62 231 -56T205 -48T179 -39T150 -26T125 -10T100 11T80 37T62 70T53 109T48 157Q48 281 123 396T317 586V612Q319 638 320 649T325 678T338 703T361 711ZM454 564Q445 567 424 567Q407 567 398 565T387 563Q387 558 411 558Q434 558 450 562L454 564",
      120636: "24 296Q25 302 27 312T41 350T65 397T103 435T157 452Q202 452 233 435Q267 419 284 384L294 392Q304 401 316 410T348 429T388 445Q410 451 445 451H453Q468 451 482 450T519 443T558 425T587 391T600 337V327Q600 311 538 64Q484 -158 478 -168Q457 -211 409 -211Q386 -211 372 -197T357 -161Q357 -158 415 80Q476 330 477 348Q477 366 473 377T461 393T448 399T432 400H427Q379 400 335 363Q300 332 280 298Q277 293 246 170T213 40Q205 22 186 7T142 -8T103 7T89 39Q89 49 106 117T142 260T164 351Q166 363 166 372Q166 402 148 402Q126 402 110 369Q100 350 90 310Q85 289 82 286T60 282H55H44Q24 282 24 296",
      120637: "213 -8Q130 -8 85 50T40 200V207Q40 303 83 428Q122 535 189 608Q279 702 381 702Q410 702 437 693T492 661T537 593T554 486Q554 428 539 362T495 230T425 111T330 25T213 -8ZM433 562Q433 600 419 625T377 651Q363 651 348 644T311 619T268 557T229 453Q225 441 217 411T208 378H401Q433 500 433 562ZM161 140Q161 43 217 43Q249 43 280 74Q310 103 332 150T378 287Q385 313 385 315Q385 316 289 316H192Q191 308 183 275T169 205T161 140",
      120638: "161 -8Q111 -8 75 16T38 85Q38 95 48 121T80 214T123 368L124 374Q126 381 127 386T132 399T139 414T149 428T162 440T180 448T203 452Q226 452 241 439T257 404Q257 386 230 290T171 111Q157 73 157 57Q157 43 166 43Q209 43 258 69T322 144Q326 157 330 159T348 162H355H366Q386 162 386 148Q386 143 383 132T367 100T335 61Q301 27 253 10T161 -8",
      120639: "258 405Q258 394 251 364T237 308T230 281T238 284T270 306T330 349Q365 377 389 394T450 427T519 444Q545 444 559 430T574 391Q574 360 551 336T491 312Q469 312 454 326T439 364Q439 372 438 372Q433 372 395 344T320 289T283 260H285Q287 260 290 260T297 259Q495 248 495 146Q495 132 491 110T486 74Q486 43 505 43Q520 43 531 53Q559 72 578 141Q582 157 586 159T611 162H622Q642 162 642 148T627 100T578 29T504 -7H495Q435 -7 399 22T363 96Q363 111 366 122T369 142Q369 155 364 165T347 182T326 193T298 200T269 204T238 207T212 210L211 206L190 123L169 40Q161 22 142 7T98 -8T59 7T45 39Q45 48 67 139T112 317L134 404Q142 424 161 438T204 452Q226 452 242 440T258 405",
      120640: "95 -13Q70 -13 55 4T40 41Q40 65 61 88Q65 92 210 207T357 322L235 602Q217 640 185 643Q182 643 178 644T173 645Q161 651 161 666Q161 677 167 684T181 692Q189 694 212 694Q335 694 358 660Q362 653 500 340T647 18Q652 10 652 6Q652 -8 622 -8H589H538L526 -5Q506 1 500 8Q494 16 444 128Q442 133 440 138L387 259L265 134Q156 20 137 4T95 -13",
      120641: "294 -8Q265 -8 244 -5T213 1T201 4Q200 4 192 -32T172 -111T155 -168Q134 -211 86 -211Q62 -211 48 -196T34 -158Q37 -144 103 123T174 404Q182 424 201 438T244 452Q271 452 284 436T298 404Q298 392 267 269T235 114Q235 43 305 43Q342 43 375 68T418 110Q420 112 455 253T492 397Q514 444 562 444Q587 444 601 429T615 397Q615 387 599 320T563 178T542 93Q540 81 540 72Q540 42 558 42Q580 42 596 75Q606 94 616 134Q621 155 624 158T646 162H651H662Q682 162 682 148Q681 142 679 132T665 94T641 47T602 9T548 -8Q523 -8 502 -3T468 11T446 27T432 40L429 46Q367 -8 294 -8",
      120642: "88 382Q70 382 65 385T59 402T64 427T78 443Q79 444 157 448T247 452Q256 452 261 448T266 440L267 437Q267 432 223 252L177 71L192 77Q293 117 371 199T480 388Q489 424 511 438T556 452Q579 452 593 438T608 402Q608 378 593 340T540 251T446 152T299 62T96 -1Q91 -2 78 -2Q38 -2 38 15Q38 22 82 198L127 379Q124 382 88 382",
      120643: "287 648Q291 671 293 680T305 700T329 711Q339 711 347 705T356 687Q356 680 351 653T345 619Q345 615 346 615Q358 618 398 618Q490 618 490 572Q490 553 476 536T434 512Q411 508 378 508H366Q332 508 306 534L298 525Q271 496 254 456T237 375Q237 336 244 336Q272 346 319 346H325Q354 346 372 344T406 331T422 300Q422 252 350 238Q332 236 303 236Q286 236 269 238T240 242T218 247T202 252L196 254Q191 254 174 237T141 191T124 139Q124 108 151 92T267 46Q285 40 295 37Q308 33 332 25T366 13T392 3T415 -8T432 -20T445 -36T451 -55T454 -80Q454 -118 427 -153T358 -199Q349 -201 327 -201Q313 -201 298 -200T271 -196T246 -191T226 -185T210 -180T200 -176L196 -174Q187 -170 187 -158T196 -138T216 -130Q217 -130 254 -140T329 -151Q349 -151 349 -135Q349 -127 340 -122T293 -104Q260 -93 238 -85Q130 -48 115 -41Q71 -19 47 15T23 88Q23 126 48 179T130 277L144 288L136 297Q99 336 99 390Q99 456 148 514T285 602V619Q285 624 286 635T287 648ZM355 563Q362 560 376 558Q424 558 423 564Q405 567 390 567Q369 567 355 563ZM279 292Q297 287 315 287Q355 287 355 293Q355 296 321 296Q316 296 308 296L301 295Q295 295 289 294L279 292",
      120644: "254 -8Q191 -8 146 9T80 54T49 111T39 169Q39 206 53 247T96 329T176 402T292 446Q317 451 336 451L344 452Q353 452 359 452Q457 452 516 408T576 279Q576 169 488 81T254 -8ZM349 400Q321 400 287 385T231 338Q206 301 188 228T170 126Q170 99 178 83Q198 44 260 44Q367 44 409 157Q419 185 432 238T445 317Q445 336 443 348Q435 372 416 384T384 398T349 400",
      120645: "55 289H43Q23 289 23 303Q23 308 33 322Q116 434 199 443Q200 444 418 444Q591 444 617 444T652 439Q674 426 674 400Q674 378 661 360T625 335Q621 334 549 333H479L477 321Q476 312 476 279Q476 219 491 174T521 104T536 65Q536 38 511 15T457 -8Q403 -8 386 94Q384 110 384 139Q384 181 391 229T406 304L413 331Q413 333 365 333H316L315 329Q315 328 312 314T301 272T288 220Q274 167 258 103Q244 49 240 38T228 18Q225 16 224 14Q200 -8 172 -8Q146 -8 132 7T118 44Q118 54 121 61Q122 65 142 102T190 195T235 293Q250 329 250 333Q177 333 166 332Q115 328 88 301L77 290L55 289",
      120646: "307 -8Q277 -8 251 0T215 14L205 20Q203 18 193 -25T171 -114T155 -168Q134 -211 87 -211Q64 -211 49 -198T34 -162Q34 -158 137 254Q153 299 179 334T232 390T277 419T311 434Q357 451 403 451Q435 451 455 449T506 435T560 400Q603 357 603 285Q603 172 520 82T307 -8ZM474 343Q474 364 458 382T409 400H406Q339 400 299 341Q281 313 264 257Q261 248 242 170T222 89Q222 84 230 74T260 54T308 43Q334 43 365 57T417 110Q437 145 456 228Q474 298 474 332V343",
      120647: "33 209Q33 277 80 334T195 421T330 451H344Q410 451 439 429Q457 417 457 402Q457 386 445 375T420 363Q415 363 406 368T383 383T354 398Q347 400 330 400Q256 400 196 361T135 265V257Q135 242 147 225T174 199L358 107Q400 77 400 28T362 -63T271 -105Q254 -105 229 -99T195 -86Q188 -82 188 -71Q188 -56 197 -50T216 -44Q225 -44 237 -48T270 -53H282Q293 -44 293 -31Q293 -19 283 -10Q278 -4 200 33T115 76Q77 98 55 133T33 209",
      120648: "35 151Q35 190 51 236T99 327T184 404T306 443Q307 443 316 443T342 443T378 444T425 444T476 444Q606 444 626 444T655 439Q677 426 677 400Q677 358 639 340Q625 333 563 333Q510 333 510 331Q518 319 518 272Q518 155 437 74T226 -8Q123 -8 79 41T35 151ZM396 278Q396 314 375 323T305 332Q249 332 222 310T180 243Q171 219 162 178T153 116V110Q153 43 234 43Q347 43 382 199Q383 203 383 204Q396 255 396 278",
      120649: "55 289H43Q23 289 23 303Q23 308 33 322Q116 434 199 443Q200 444 386 444Q571 444 577 442Q588 441 599 432T610 402Q610 359 572 340Q561 335 547 334T452 333H353V331Q352 330 342 261T320 119T306 40Q300 18 281 3Q257 -13 233 -13Q210 -13 196 0T181 35Q181 44 182 48Q183 53 229 187T279 331Q279 333 228 333H209Q163 333 136 328T88 301L77 290L55 289",
      120650: "189 388Q189 396 187 398T176 401Q144 399 122 369T89 304Q84 288 81 285T61 282H55H44Q24 282 24 296Q24 306 34 330T64 382T116 431T188 452Q232 452 270 430T308 361Q308 345 275 258T241 123Q241 44 336 44H344Q380 44 415 73T474 140T511 214T526 267Q526 280 513 292Q505 301 486 311T456 333T444 367Q444 400 471 426T529 453Q555 453 579 431T604 358Q604 327 592 271T557 172Q550 157 541 142T510 97T464 47T404 9T328 -8Q264 -8 219 5T154 41T125 85T117 131Q117 182 153 277T189 388",
      120651: "238 4Q230 5 218 8T174 26T118 58T73 112T53 190Q53 219 60 246Q77 313 103 362T143 426T163 443Q165 444 186 444Q217 444 217 432Q217 425 188 392Q157 351 137 291T116 206Q116 127 263 109L276 107L288 139Q347 304 414 378T566 452Q621 449 662 412T703 286Q698 167 598 82T357 -8H332V-11Q332 -12 327 -46T317 -116T310 -157Q306 -180 286 -198T234 -216Q211 -216 197 -203T183 -168Q183 -160 184 -155Q184 -152 198 -112T225 -34T238 4ZM637 260Q637 303 607 322T541 341Q502 341 466 319T405 264Q368 218 356 159Q348 111 348 104Q348 103 361 103Q502 103 569 152T637 260",
      120652: "58 -194Q32 -194 32 -164Q32 -153 46 -139L175 -13Q296 104 296 106Q271 232 226 308Q165 401 141 401Q128 401 117 390T100 365Q94 350 91 349T69 346H45Q35 348 35 359Q35 380 62 411T133 450Q143 452 165 452Q286 452 329 402Q347 379 366 333T394 254T404 215Q404 213 405 213Q405 212 420 227T463 268T520 324Q637 437 640 438Q647 444 661 444Q667 444 676 438T685 419Q685 405 670 389T549 271L420 145Q433 85 444 53Q466 -25 509 -88T575 -151Q590 -151 600 -140T617 -114T626 -98Q629 -96 650 -96H655Q681 -96 681 -108Q681 -114 679 -119Q670 -148 646 -169T591 -199Q581 -201 550 -201Q422 -201 381 -143Q345 -88 316 20L311 39L230 -40Q144 -126 114 -153Q85 -182 77 -188T58 -194",
      120653: "244 141Q244 117 254 98T277 70T305 55T329 48T342 47L344 48L424 366Q501 678 505 686Q508 691 512 692T533 694T555 693T562 688T565 683Q565 678 486 362T406 45Q406 43 415 43Q467 49 514 78T590 143T636 213T653 266Q653 282 641 293T613 311T585 332T572 367Q572 397 598 425T657 453Q684 453 708 430T732 358Q732 330 723 287T706 225Q671 124 578 58T401 -8H393L370 -101Q346 -196 345 -197Q340 -202 316 -202H306Q286 -202 286 -188Q286 -187 296 -144T318 -57T331 -8Q331 -6 328 -6Q290 -6 233 11T148 62Q119 97 119 146Q119 181 154 275T190 388Q190 401 181 401Q154 401 129 375T90 306Q85 288 81 285T61 282H55H44Q24 282 24 296Q24 306 34 329T64 381T116 431T189 452Q237 452 273 428T309 362Q309 343 277 260T244 141",
      120654: "532 367Q532 399 559 426T618 453Q645 453 668 430T691 357Q691 319 677 261T649 171Q640 149 626 125T586 68T521 14T438 -7Q401 -7 372 7T333 30T311 57Q246 -8 165 -8Q119 -8 82 19T30 102Q24 126 24 163V178Q24 210 37 255Q61 346 118 424Q141 451 161 451Q174 451 184 443T194 419Q194 402 179 387Q91 273 91 206Q91 159 122 138T189 117T281 145V173Q283 223 294 253Q304 276 323 289T364 303Q386 303 400 287T415 250Q415 219 385 157L378 144Q378 142 388 136T419 124T462 117Q522 117 566 165T610 255Q610 288 561 320Q532 337 532 367",
      120655: "230 475Q202 475 189 492T175 526T186 570T221 631T288 687T389 710Q430 710 438 709Q495 701 537 679T601 629T637 568T653 509T657 459Q657 409 640 341Q617 248 581 180T507 75T424 16T348 -11T282 -17Q171 -17 113 37Q60 88 60 159Q60 192 71 231Q96 336 184 402Q264 462 366 462Q407 462 439 448T497 389L502 380Q503 381 508 403T519 463T525 531Q523 580 499 610T447 648T387 657Q324 657 283 616Q271 604 276 604Q279 604 286 600T302 583T311 555Q311 523 287 499T230 475ZM196 110Q196 41 287 41Q351 41 398 88Q422 111 437 151Q473 243 473 298Q473 386 409 409Q408 409 383 411Q316 411 278 373Q265 360 259 351T241 311T217 226Q196 143 196 110",
      120656: "415 89Q423 89 429 74T436 46Q436 43 434 39Q432 36 420 29T380 11T322 -5Q311 -7 281 -7Q216 -7 168 10T94 54T56 110T44 167V181Q44 262 94 329Q104 343 119 357T162 391T234 425T327 443Q328 443 348 443T383 444Q434 444 442 438Q450 430 450 416Q446 392 424 383L376 382Q306 381 278 369Q230 349 208 294Q199 274 199 268Q199 267 291 267Q305 267 325 267T353 268Q383 268 394 263T406 241Q406 214 380 206Q375 205 279 205T183 203Q174 176 174 140Q174 87 208 65T292 43Q295 43 300 43T307 44Q337 49 372 69T415 89",
      120657: "114 132Q114 153 140 253T166 372Q166 402 148 402Q126 402 110 369Q100 350 90 310Q85 289 82 286T60 282H55H44Q24 282 24 296Q25 302 27 312T41 350T65 397T104 435T158 452Q184 452 211 445T263 414T288 354V339L265 245Q237 134 237 118V107V102Q237 87 239 77T257 56T300 43Q395 43 455 254Q479 346 479 347L460 354Q294 408 294 528Q294 606 350 653T464 701Q536 701 579 659Q634 601 634 491Q634 468 630 438T623 388L620 370Q624 370 631 369T647 364T656 352Q656 347 653 335Q647 317 642 316Q640 315 637 315Q635 315 619 317Q606 319 605 316Q605 315 603 308Q587 248 550 177T457 57Q379 -8 293 -8Q192 -8 153 23T114 116V132ZM519 566Q519 600 507 625T464 651Q425 651 391 617T356 529Q356 501 370 478T404 441T443 417T477 404L491 400Q493 400 499 428T512 497T519 566",
      120658: "228 325Q170 322 156 316T127 309Q108 309 104 314Q99 319 99 322T108 341Q125 376 171 400T268 425H271Q302 425 319 396Q328 377 328 358Q328 332 324 314Q311 270 286 221Q274 194 274 192H275Q339 234 484 325T639 421Q669 434 691 434T723 425T734 406Q734 394 719 381Q715 376 644 330L575 287L566 267Q543 233 526 176Q520 160 515 143T508 115T506 105Q506 103 533 103Q585 103 607 110T641 118Q670 118 670 107Q670 100 661 85Q643 50 598 27T504 3Q465 3 450 36Q441 51 441 73Q441 84 444 96Q452 146 484 205L497 236L324 125Q143 12 135 10Q103 -6 77 -6Q61 -6 49 2T37 21Q37 36 49 46T124 96L195 141L204 156Q219 179 243 248T264 323Q264 325 228 325",
      120659: "274 -7Q232 -4 195 7T125 38T71 94T51 176V190Q51 213 60 242T95 307T156 373T255 425T393 451L397 452L427 568Q434 597 443 636Q452 677 456 685T472 694H486H495Q517 694 517 680L514 665Q510 650 503 621T489 564L460 451H469Q527 447 574 430T657 370T693 266Q693 163 599 82T350 -7H346L322 -100Q301 -190 295 -197Q291 -202 283 -202H269H258Q238 -202 238 -188Q238 -186 260 -96L283 -7H274ZM449 400Q448 400 404 225T359 47T366 45Q464 55 516 119Q542 149 558 199T575 295Q575 387 462 398L449 400ZM384 398Q384 399 381 399Q350 399 298 378T214 308Q168 236 168 149Q168 68 259 49Q282 44 294 44H295L384 398",
      120660: "371 -168Q357 -168 323 -171T245 -175Q143 -175 109 -150T75 -66Q75 -5 100 108T137 254Q153 299 179 334T232 390T277 419T311 434Q357 451 403 451Q435 451 455 449T506 435T560 400Q603 357 603 282Q603 213 569 148T465 38T304 -8Q273 -8 247 -2T204 14T176 31T159 46T152 53Q152 52 148 27T144 -16Q144 -36 150 -44T189 -58T293 -64Q405 -65 432 -75Q466 -88 466 -127Q466 -140 459 -172Q455 -188 451 -191T426 -194H420Q405 -194 400 -191T395 -176Q396 -170 394 -169T378 -168Q373 -168 371 -168ZM236 116Q236 77 258 60T311 43Q369 43 407 94Q429 123 451 206T474 331Q474 400 409 400H406Q339 400 299 341Q276 305 256 227T236 116",
      120661: "55 289H43Q23 289 23 303Q23 308 33 322Q116 434 199 443Q200 444 562 444Q922 444 928 442Q961 434 961 400Q961 376 944 355T886 333H870Q872 322 872 295V279Q872 230 842 165T751 46T618 -8Q581 -8 554 6T513 45T494 84T484 119Q484 121 478 114Q477 113 476 111Q384 -7 268 -7H265Q251 -7 237 -4T199 11T162 54T147 132Q147 149 149 166T155 198T165 229T176 256T189 281T200 301T211 319T220 333H199Q120 333 88 301L77 290L55 289ZM639 103Q674 103 712 122T780 188T811 295Q811 318 808 330V333H289Q274 318 244 263T214 169Q214 133 236 118T288 103Q351 103 412 153T494 278Q497 290 502 292T529 295Q546 295 551 293T556 283Q556 281 553 260T550 218Q550 153 576 128T639 103",
      120662: "110 0H86Q42 0 42 27Q42 37 148 350T258 667Q269 687 291 692Q295 694 366 694H399Q432 694 448 689T474 667Q477 663 583 350T690 27Q690 0 642 0H617H592Q582 0 575 1T561 2T549 6T541 11T533 18T527 26T522 37T517 49T512 64T506 81L490 130H225Q225 128 208 79T189 27Q185 19 180 14T170 7T156 3T143 1T127 0T110 0ZM439 279Q359 524 359 547L357 555L355 543Q347 503 270 263L259 231H357Q455 231 455 232L439 279",
      120663: "119 1Q98 5 92 28V667Q98 686 118 693Q121 694 272 694H289H346Q439 694 500 681T600 625Q640 580 640 513Q640 451 601 414T504 364L518 361Q568 351 602 329T649 280T666 235T671 197Q671 172 665 147T642 91T586 37T488 5Q456 1 282 1H119ZM489 509Q489 532 479 548T450 573T421 585T394 591Q387 592 315 593H247V404H298H325Q432 404 466 444Q489 470 489 509ZM517 194Q517 235 502 261T458 299T407 313T353 317H329H322H247V101H319H357Q387 101 407 103T452 111T492 133T514 171Q516 176 517 194",
      120664: "92 664Q98 683 118 690Q121 691 312 691T508 689Q534 682 534 644V632V618Q534 582 508 573L502 572Q496 572 489 572Q486 572 463 572T416 573Q333 573 291 575H253V303Q253 31 251 25Q242 0 199 0H170L119 1Q99 7 92 28V664",
      120665: "381 692Q386 694 458 694Q516 694 527 693T549 687Q564 680 575 663Q576 658 715 349T856 27Q856 6 838 1H826Q815 1 795 1T747 1T686 1T616 0T539 0T458 0T378 0T300 0T230 0T169 1T122 1T90 1H78Q60 6 60 27Q62 38 201 349T341 663Q356 687 381 692ZM627 148Q626 149 581 250T492 453L447 554Q447 553 446 552Q444 546 326 278L268 148Q268 147 448 147Q627 147 627 148",
      120666: "277 122Q280 122 380 123T544 125Q552 125 557 125T565 124T569 124Q595 115 595 75V62V47Q595 9 569 2Q564 0 341 0L119 1Q99 7 92 28V664Q98 683 118 690Q121 691 335 691T554 689Q580 682 580 644V632V618Q580 582 554 573Q553 573 551 573T542 572T527 572Q464 572 364 573T260 575H253V412H385H459Q524 412 536 404T549 357Q549 341 549 334T542 318T523 305Q518 303 385 303H253V122H277",
      120667: "411 584Q243 581 131 581Q122 581 116 581T106 582T102 582Q84 589 80 600T76 640L77 667Q83 686 103 693Q106 694 343 694Q579 694 584 692Q592 691 599 684T609 668Q610 665 610 646Q610 614 608 608Q605 603 434 361L261 116Q340 117 402 118T490 119T533 120T560 120H572Q605 120 614 95Q616 89 616 60V46Q616 9 590 2Q585 0 339 0Q92 0 87 2Q79 3 72 10T62 26Q61 29 61 49Q61 84 63 90Q65 94 152 217T325 461T411 584",
      120668: "92 667Q101 694 143 694H172H198Q244 694 251 669Q253 663 253 539V415H540V539Q540 558 540 585T539 621Q539 673 550 683T611 694H621H646Q671 694 683 690T700 669Q702 663 702 347T700 25Q696 9 684 5T646 0H621H606Q560 0 550 11T539 76Q539 85 539 116T540 169V306H253V169Q253 147 253 116T254 75Q254 23 245 12T194 0H170L119 1Q99 7 92 28V667",
      120669: "62 340Q62 716 425 716Q511 716 576 696T681 642T747 559T783 458T793 341Q793 264 777 203T721 89T608 7T428 -22Q62 -22 62 340ZM638 333Q638 365 637 387T632 441T621 495T600 542T567 583T518 611T451 628Q443 629 427 629Q402 629 378 624T327 608T276 571T240 511Q217 453 217 345Q217 254 231 204T279 120Q333 69 428 69Q522 69 576 120Q638 183 638 333ZM279 349V373Q279 413 305 420Q309 422 427 422H487Q550 422 563 414T576 369V349Q576 345 576 337T577 324Q577 284 550 277Q545 275 428 275H369Q306 275 293 283T279 329V349",
      120670: "85 667Q94 694 136 694H165H191Q237 694 244 669Q246 663 246 347T244 25Q235 0 192 0H163L112 1Q92 7 85 28V667",
      120671: "92 667Q101 694 139 694H163H186Q225 694 234 671Q236 663 236 529L237 392L533 682Q550 694 590 694H623H681Q695 680 695 672Q695 670 693 664Q688 657 561 533L431 405L698 33Q701 28 701 23Q701 7 683 0H626H604Q571 0 564 2T545 13Q544 14 530 33T489 90T437 162L332 307Q331 307 284 260L236 214V122V65Q236 32 231 19T210 2Q205 0 161 0L119 1Q99 7 92 28V667",
      120672: "106 0H83Q41 0 41 28Q41 39 133 349T229 667Q242 694 296 694H335H375Q403 694 418 689T442 667Q445 660 537 350T630 28Q630 11 619 6T584 0H555H526Q478 0 465 27Q462 32 431 136T366 372T325 555V546Q320 503 287 376T222 141T186 27Q184 22 177 15T165 6Q154 0 106 0",
      120673: "92 667Q98 684 109 689T146 695Q152 695 167 695T192 694Q200 694 214 694T234 695Q291 695 305 664Q313 651 400 419T487 165Q487 162 488 162T489 165Q489 187 574 413T671 664Q679 680 695 688Q708 694 785 694H828Q855 694 867 689T884 669Q886 663 886 347T884 25Q876 0 832 0H817H802Q758 0 750 25Q748 31 748 293V555L746 544Q737 509 692 386T606 160T564 52Q548 22 502 22H487H472Q423 22 410 52Q407 59 367 160T283 385T231 546L230 548Q229 548 229 293Q229 31 227 25Q222 9 211 5T176 0H158L119 1Q99 7 92 28V667",
      120674: "92 667Q98 684 109 689T146 694H185Q273 694 279 692Q301 689 315 669Q322 660 419 453L554 163L562 143Q564 143 564 401Q564 663 566 669Q574 694 618 694H633H648Q692 694 700 669Q702 663 702 347T700 25Q696 10 683 5T642 0H596H551Q520 0 505 4T478 25Q471 34 374 241L239 532Q231 550 231 552L229 479Q229 440 229 293Q229 31 227 25Q222 9 211 5T176 0H158L119 1Q99 7 92 28V667",
      120675: "627 553Q609 553 512 554T366 555Q316 555 220 554T105 553Q96 553 90 553T82 554T78 554Q61 560 57 571T52 605V623L53 661Q59 680 79 687Q82 688 366 688Q649 688 654 686Q680 679 680 639V621V603Q680 563 654 554Q653 554 651 554T642 554T627 553ZM149 423Q152 424 366 424Q579 424 584 422Q610 415 610 376V358V340Q610 300 584 293Q579 291 366 291H232Q162 291 150 293T129 306Q122 315 122 360L123 397Q129 416 149 423ZM108 135Q143 135 226 134T363 133Q407 133 507 134T632 135H645Q675 135 684 110Q686 104 686 68V49Q686 9 660 2Q655 0 364 0L74 1Q57 7 49 21L47 28L46 65V83Q46 126 72 133Q80 135 108 135",
      120676: "362 715Q364 715 376 715T394 716H400Q542 716 626 643T727 426Q731 395 731 342Q731 271 722 225Q674 -22 396 -22Q320 -22 259 -3T148 68T77 201Q62 257 62 342Q62 447 86 522T173 649Q245 707 362 715ZM568 433Q551 623 396 623Q383 623 370 622T333 612T292 591T257 550T233 485Q223 442 223 350Q223 276 232 227T267 137Q309 74 397 74Q433 74 461 85T507 113T537 156T556 205T566 260T569 310T570 357Q570 409 568 433",
      120677: "92 664Q98 683 118 690Q121 691 396 691T676 689Q695 684 700 666Q702 660 702 345Q702 31 700 25Q696 9 684 5T646 0H621H596Q571 0 559 4T542 25Q540 31 540 307V582H253V307Q253 31 251 25Q242 0 199 0H170L119 1Q99 7 92 28V664",
      120678: "641 470Q641 426 630 391T603 334T561 295T513 271T459 259T408 254T361 253H350H337H253V142Q253 125 253 100T254 67Q254 32 249 19T227 2Q222 0 170 0L119 1Q99 7 92 28V667Q98 686 118 693Q121 694 271 694Q428 693 462 688Q641 656 641 470ZM487 467Q487 495 485 510T474 546T442 578T382 592Q375 593 310 593H250V347H309H339Q364 347 380 348T418 354T451 368T474 395T486 438Q487 444 487 467",
      120680: "322 124Q326 124 457 125T672 127H689Q721 127 730 102Q732 96 732 64V48Q732 9 706 2Q701 0 394 0L89 1Q76 5 69 13T62 29V36Q62 37 62 38Q62 47 70 58T126 126Q161 167 185 196Q302 335 302 336L187 463Q74 584 68 594Q61 603 61 639L62 667Q68 686 88 693Q91 694 396 694T706 692Q732 686 732 647V635V621Q732 585 706 576Q705 576 702 576T691 576T670 575L302 578Q302 577 394 475T490 371Q498 362 498 347Q498 336 488 323T408 226L322 124",
      120681: "67 687Q70 688 366 688Q661 688 666 686Q692 680 692 641V629V615Q692 579 666 570H660Q655 569 648 569Q645 569 624 569T581 570Q505 570 475 572H447V302Q447 31 445 25Q436 0 393 0H364L313 1Q293 7 286 28L285 300V572H257Q227 570 151 570Q130 570 109 570T84 569Q77 569 72 570H66Q48 577 44 588T40 631L41 661Q47 680 67 687",
      120682: "62 560Q62 607 94 644T169 698T253 715Q273 715 286 713T322 704T363 677T398 625Q413 597 423 556L428 540Q429 541 436 566T454 620T494 677T561 713Q570 715 593 715Q682 715 737 668T793 560Q793 549 793 545T786 533T767 520H670Q646 532 644 551T632 579Q618 594 591 594Q539 594 524 530T509 321V216Q509 31 507 25Q498 0 455 0H426L375 1Q355 7 348 28L347 232Q346 344 346 441Q346 442 343 468T335 521T312 571T266 594Q252 594 247 593Q228 586 220 576T212 557T209 539T191 523L185 520H88Q75 527 69 534T63 545T62 560",
      120683: "62 292T62 347T80 445T124 511T183 552T243 574T292 584L315 587H319V627L320 667Q329 694 370 694H397H422Q466 694 473 669Q475 663 475 625V587H478Q479 587 500 584T548 575T608 553T668 513T713 446T732 347Q732 253 674 187Q655 167 628 152T576 128T530 116T493 109L478 107H475V69V50Q475 9 449 2Q444 0 395 0L347 1Q327 7 320 28L319 67V107H315L292 110Q269 114 243 119T184 142T124 182T80 249ZM319 197T319 347T318 497Q316 497 307 494T284 485T262 471Q220 438 220 347Q220 285 239 249Q248 234 261 223T286 208T308 200L317 197Q319 197 319 347ZM572 347V357Q572 387 569 407T548 452T496 491Q495 491 494 491T487 493T475 497V197Q518 210 541 232T571 303Q572 312 572 347",
      120684: "52 1Q37 11 37 23Q37 26 39 32Q39 34 158 202L275 369Q275 370 221 441T112 586T55 663Q53 669 53 672Q53 687 68 693H72Q77 693 84 693T99 694T118 694T139 694H176Q203 694 212 692T230 682Q231 681 239 669T265 634T296 591L358 504L418 591Q481 682 486 686Q491 691 499 692Q505 694 569 694H632Q650 685 650 672Q650 667 646 660Q643 654 592 582T491 440T441 369T566 201T693 29Q694 27 694 23Q694 11 677 0H607L537 1Q523 6 519 10T437 131Q422 153 411 170T390 200T375 222T365 237T359 245L357 247L348 232Q339 218 319 188T283 131Q222 37 211 22T186 1H52",
      120685: "61 585Q62 594 62 597T64 606T73 616T89 626H138Q196 626 208 620Q243 602 253 546T261 431T271 309T325 219Q342 205 349 205Q350 205 350 436L351 667Q360 694 401 694H428H454Q495 694 504 671Q506 663 506 436L507 205Q542 222 561 251T586 318T593 392T595 472T602 546Q614 614 661 625Q665 626 708 626H730Q766 626 780 618T794 582Q794 548 768 540Q755 538 754 501T750 410T736 298T680 191T560 120Q550 116 512 109H506V70V50Q506 9 480 2Q475 0 426 0L378 1Q358 7 351 28L350 68V109L335 111Q298 117 267 129T214 156T175 191T146 229T127 272T115 314T109 357T106 395T105 429Q104 537 87 540Q66 548 63 565Q61 570 61 585",
      120686: "241 122Q225 154 191 199T131 278T83 363T61 464Q61 497 68 527T94 591T145 650T228 693T349 715Q354 715 370 715T396 716Q539 716 622 668Q658 647 682 617T715 556T728 505T732 465Q732 415 711 365T663 280T602 200T552 122H632Q649 122 669 122T693 123H697Q736 123 742 98Q744 92 744 62V47Q744 9 718 2Q713 0 591 0L471 1Q454 7 446 21Q444 27 444 45Q444 96 463 154T506 257T549 360T569 469Q569 504 563 530T538 580T485 616T396 629Q313 629 268 594T223 468Q223 419 243 361T286 258T330 152T350 41Q350 14 335 7T276 -1Q267 -1 241 -1T197 0L77 1Q57 7 50 28L49 59V74Q49 114 75 121Q81 123 100 123Q104 123 124 123T161 122H241",
      120782: "266 654H280H282Q500 654 524 418Q529 370 529 320Q529 125 456 52Q397 -10 287 -10Q110 -10 63 154Q45 212 45 316Q45 504 113 585Q140 618 185 636T266 654ZM374 548Q347 604 286 604Q247 604 218 575Q197 552 193 511T188 311Q188 159 196 116Q202 87 225 64T287 41Q339 41 367 87Q379 107 382 152T386 329Q386 518 374 548",
      120783: "481 0L294 3Q136 3 109 0H96V62H227V304Q227 546 225 546Q169 529 97 529H80V591H97Q231 591 308 647L319 655H333Q355 655 359 644Q361 640 361 351V62H494V0H481",
      120784: "175 580Q175 578 185 572T205 551T215 510Q215 467 191 449T137 430Q107 430 83 448T58 511Q58 558 91 592T168 640T259 654Q328 654 383 637Q451 610 484 563T517 459Q517 401 482 360T368 262Q340 243 265 184L210 140H274Q416 140 429 145Q439 148 447 186T455 237H517V233Q516 230 501 119Q489 9 486 4V0H57V25Q57 51 58 54Q60 57 109 106T215 214T288 291Q364 377 364 458Q364 515 328 553T231 592Q214 592 201 589T181 584T175 580",
      120785: "80 503Q80 565 133 610T274 655Q366 655 421 623T491 538Q493 528 493 510Q493 446 453 407T361 348L376 344Q452 324 489 281T526 184Q526 152 514 121T474 58T392 8T265 -11Q175 -11 111 34T48 152Q50 187 72 209T132 232Q171 232 193 208T216 147Q216 136 214 126T207 108T197 94T187 84T178 77T170 72L168 71Q168 70 179 65T215 54T266 48H270Q331 48 350 105Q358 128 358 185Q358 239 348 268T309 313Q292 321 242 322Q205 322 198 324T191 341V348Q191 366 196 369T232 375Q239 375 247 376T260 377T268 378Q284 383 297 393T326 436T341 517Q341 536 339 547T331 573T308 593T266 600Q248 600 241 599Q214 593 183 576Q234 556 234 503Q234 462 210 444T157 426Q126 426 103 446T80 503",
      120786: "531 0Q510 3 381 3Q238 3 214 0H201V62H313V155H32V217L205 434Q342 606 362 630T387 655L391 656Q395 656 401 656T414 656H427Q447 656 451 645Q453 641 453 429V217H542V155H453V62H542V0H531ZM324 217V494L103 218L213 217H324",
      120787: "100 565V605Q100 637 102 646T113 655Q116 655 139 647T202 631T286 623Q332 623 372 631T434 647T459 655Q466 655 469 651T472 643T472 629Q472 613 463 601Q370 487 219 487Q195 487 183 488T169 490T168 433V376Q169 376 174 379T188 387T211 397T244 405T288 409Q390 409 453 352T517 201Q517 106 445 48T253 -11Q169 -11 113 37T57 154Q57 187 79 208T131 229T183 209T206 154Q206 99 155 83Q152 82 157 78Q196 47 253 47Q347 47 358 135Q358 137 358 138Q360 158 360 209Q360 277 355 301T337 338Q315 358 282 358Q202 358 160 303Q153 294 149 292T130 290Q107 290 102 301Q100 304 100 474V565",
      120788: "48 318Q48 395 68 456T120 553T193 613T273 646T350 655Q425 655 461 616T497 524Q497 485 475 468T428 451Q399 451 378 470T357 521Q357 565 403 588Q375 601 351 601Q313 601 282 584Q242 565 222 526Q199 473 199 367Q201 369 210 380T227 396T246 410T275 422T312 426Q438 426 494 332Q526 285 526 208V199Q526 112 465 53Q428 17 388 3T285 -11Q236 -11 195 7T135 43T104 80Q48 165 48 318ZM375 231V244V268Q375 295 373 310T364 342T341 366T299 374H297Q231 374 208 287Q200 257 200 196Q201 120 209 100Q231 47 288 47Q351 47 368 90Q375 112 375 231",
      120789: "256 -11Q231 -11 208 5T185 65Q185 105 193 146T212 220T241 289T275 349T312 402T346 445T377 479T397 502L400 504H301Q156 503 150 497Q142 491 134 456T126 407H64V411Q65 414 82 544T99 675T130 676H161V673Q161 669 162 666T167 661T173 657T181 654T190 652T200 651T210 650T220 649T229 648Q237 648 254 647T276 646Q277 646 426 644H558V620V607Q558 596 551 586T509 537Q489 515 476 500Q390 401 384 393Q349 339 337 259T324 113T322 38Q307 -11 256 -11",
      120790: "80 474Q80 561 139 607T278 654Q357 654 411 632Q490 593 494 509Q494 424 416 376L407 371L418 364Q432 356 447 345T481 312T513 260T526 192Q526 100 461 45T285 -11Q184 -11 116 32T48 164Q48 181 50 196T58 225T69 249T84 270T100 286T117 300T134 311T149 321T162 329L152 336Q120 360 100 397T80 474ZM347 404Q404 446 404 503Q404 579 317 599Q309 600 276 600Q178 600 170 538Q170 532 171 527T173 518T178 509T184 501T194 492T205 484T219 476T235 467T254 456T275 445L347 404ZM289 47Q323 47 351 54T402 82T425 137Q425 147 421 161Q411 183 391 197T303 249Q224 293 223 293Q220 291 215 288T197 273T175 248T157 213T149 167Q149 109 188 78T289 47",
      120791: "178 59Q206 48 238 48Q311 48 345 102Q370 138 375 259V278Q374 278 369 271T350 252T322 232Q297 220 258 220Q172 220 110 275T48 438V446Q54 561 146 618Q199 654 278 654Q321 654 329 653Q526 621 526 330Q526 252 507 190T457 92T388 31T312 -2T240 -11Q165 -11 121 25T77 120Q77 159 99 176T147 193T194 177T217 122Q217 113 216 106T211 92T205 82T198 73T191 67T184 62T178 59ZM374 446V465Q374 523 364 552T315 598Q309 600 293 601Q227 601 210 562Q199 539 199 433Q199 343 204 319T235 279Q250 272 274 271H282Q293 271 303 274T327 288T353 323T371 385Q374 403 374 446",
      120802: "117 621Q174 678 247 678Q305 678 351 647Q396 617 424 557Q460 472 460 328Q460 271 455 224Q448 154 427 104T376 27T314 -10T249 -22Q201 -22 160 1T91 67Q39 154 39 316Q39 541 117 621ZM250 55Q274 55 293 66T324 93T344 136T357 185T364 240T366 291T367 340Q367 373 367 393T363 449T352 507T332 553T299 589T250 601Q217 601 194 584T159 542T141 479T133 411T132 340V331Q132 299 133 277T137 219T147 157T167 107T201 68T250 55",
      120803: "94 612Q172 616 211 632T284 678H307V73H430V0H88V73H213V317Q213 560 212 560Q210 558 197 554T155 546T96 540L83 539V612H94",
      120804: "222 599Q190 599 166 585T128 550T108 509T97 474T93 459L67 492L42 526L47 539Q72 608 120 642T225 677Q304 677 355 644Q449 579 449 454Q449 373 361 290Q351 280 315 250T199 144Q156 103 137 85L293 86H449V0H50V79L216 242Q284 302 317 349T351 456Q351 517 315 558T222 599",
      120805: "333 521Q333 554 313 579T243 604Q154 604 99 514L78 546Q56 577 56 579Q56 580 62 589T82 611T114 637T162 662T222 677Q224 677 231 677T242 678H245Q318 678 374 634T430 520Q430 483 417 452T382 398T351 369T329 354L328 353Q369 333 373 330Q408 306 432 268T457 184Q457 103 397 41T242 -22Q131 -22 51 58L42 68L49 105L55 142L58 138Q62 134 66 130T77 120T91 108T108 96T129 83T152 72T179 63T209 57T242 54Q285 54 319 86T353 184Q353 231 331 267T260 315L213 316H166V354Q166 392 167 392Q233 395 257 405Q290 418 311 450T333 521",
      120806: "271 654L272 656H380V235H471V159H380V0H286V159H28V235L149 443Q269 652 271 654ZM292 235V607Q292 604 290 591T286 571T280 548T269 517T252 476T226 422T189 354T140 267Q136 260 132 253T126 240L123 236Q123 235 207 235H292",
      120807: "257 350Q236 350 218 342T189 323T171 301T160 281L157 273Q157 272 116 272H75V656H416V577H162V486Q162 396 163 396T174 403T207 418T258 426Q339 426 394 360T449 203Q449 113 386 46T226 -21H223Q188 -21 156 -11T102 13T64 42T41 66T33 77Q34 78 44 95T64 128L73 144Q93 112 117 93Q165 54 223 54Q270 54 306 86T345 197Q345 350 257 350",
      120808: "42 318Q42 396 61 460T109 564T173 629T237 666T289 677H301H317Q359 677 408 658V621Q408 585 407 585H406Q359 605 308 605Q246 605 203 556T146 421Q143 403 144 403Q145 402 152 409Q216 469 299 469Q333 469 357 457T407 405Q457 330 457 226Q457 126 402 57Q340 -22 251 -22Q216 -22 183 -7T116 43T63 149T42 318ZM260 393Q216 393 188 365T150 306T141 243Q141 153 172 104Q192 68 230 56Q238 54 251 54Q311 54 342 116Q360 152 360 226Q360 297 344 332Q320 382 277 392Q275 392 270 392T260 393",
      120809: "42 570V656H457V577L447 564Q345 439 295 289T244 0V-11H145V12Q160 330 356 550Q360 556 365 561T374 571L208 570H42",
      120810: "55 500Q55 568 109 623T250 678Q327 678 385 627T444 501Q444 445 410 410T336 357L329 353H330Q378 335 417 293T456 184Q456 93 393 36T249 -22T106 35T43 184Q43 251 82 293T169 353Q171 354 166 356Q165 356 163 357Q113 378 84 416T55 500ZM358 496Q358 557 325 581T250 605Q206 605 174 580T141 496Q141 446 171 420T250 393Q298 393 328 419T358 496ZM245 316Q205 316 172 285T139 185V179Q139 79 222 57Q231 55 250 55H254Q295 55 327 84Q360 120 360 188Q360 254 326 285Q295 316 245 316",
      120811: "44 476Q57 561 116 619T245 677H255Q308 677 349 647Q392 619 424 545T457 334Q457 178 382 78T205 -22Q168 -22 135 -10T86 14L72 25Q73 25 91 58L110 91Q127 78 136 72T163 60T203 54Q238 54 265 71T308 110T335 164T350 214T357 253V257L347 248Q284 187 200 187Q165 187 140 201T87 258Q42 332 42 430Q42 458 44 476ZM269 604Q265 605 254 605Q222 605 199 591T164 554T148 517T141 487Q140 478 140 430T141 373Q146 335 164 307T207 269Q223 263 247 263Q299 266 328 308T358 417Q358 435 356 456T346 511T318 570T269 604",
      120812: "274 -22Q227 -22 190 -9T128 24T87 76T62 135T49 204T44 271T43 336V343V362Q43 407 45 440T56 524T86 613T141 677Q197 715 264 715Q314 715 353 702T418 669T460 616T487 555T500 483T505 413T506 343Q506 310 506 293T503 241T498 185T486 133T467 83T437 42T397 8T343 -13T274 -22ZM355 355V401Q355 448 354 476T349 537T336 587T311 617T272 629Q270 629 266 629T261 628Q219 618 207 568T194 419V355Q194 203 200 156T231 85Q250 66 275 66Q299 66 318 85Q342 109 348 156T355 355",
      120813: "118 560H116Q107 560 100 561T85 572T76 600Q76 612 77 618T83 632T99 644Q102 645 135 646T207 660T275 700Q292 716 310 716Q353 716 360 691Q362 685 362 386V87H446Q464 76 468 69T473 44Q473 12 446 1H118Q105 7 100 13T94 25T93 44V62Q100 79 119 87H210V329Q210 571 208 571Q182 560 118 560",
      120814: "339 477Q339 505 332 531T301 579T242 601Q165 601 136 503Q133 493 130 490T121 486Q116 486 94 514Q86 523 82 528Q46 572 46 577Q46 591 77 632T147 691Q192 716 257 716Q305 716 351 700Q416 674 455 615T494 481Q494 421 463 376T356 266Q326 240 287 205T224 146T199 122H331Q359 122 392 122T431 123H445Q485 123 492 98Q494 92 494 62V47Q494 9 468 2Q463 0 272 0L83 1Q63 7 56 28L55 57V89Q59 97 215 261Q255 303 275 327T317 394T339 477",
      120815: "61 624Q62 630 83 650T153 693T262 716Q328 716 373 698T438 650T465 593T473 536Q473 438 375 376L387 371Q450 350 476 305T503 208Q503 164 492 126T456 53T380 -2T261 -22Q224 -22 189 -15T130 2T86 24T57 43T46 53Q46 56 53 99T62 145Q65 152 71 152Q76 152 90 143T123 121T175 99T248 87Q302 87 321 113T341 202Q341 264 329 292T279 329L232 331L190 332L184 338V411Q190 417 192 417Q193 418 205 419T232 421T252 424Q280 430 299 461T318 539V551Q318 599 283 609Q276 611 257 611Q225 611 199 601T159 577T136 554T120 543T102 560T76 597T62 618T61 624",
      120816: "278 693H282Q285 693 291 693T305 694T322 694T342 694H377Q402 694 411 691T430 677Q434 670 434 646T435 456V249H461H472Q509 249 516 224Q518 219 518 194Q518 178 518 171T511 155T492 142Q488 140 461 140H435V86V53Q435 9 409 2Q405 0 366 0H351Q306 0 298 25Q296 31 296 86V140H179H123Q99 140 80 129T50 106T37 95Q31 95 31 163V208Q31 246 33 251Q251 673 262 684Q268 691 278 693ZM302 249V416L301 576Q301 536 165 276L151 250L226 249H302",
      120817: "109 282Q87 290 83 310V666Q83 667 84 670T87 676T91 682T98 688T108 693Q112 694 269 694T431 692Q457 686 457 648V637V624Q457 588 431 581Q426 579 326 579H227V510Q227 442 229 442Q243 450 288 450Q377 450 435 399T494 222Q494 -22 241 -22Q202 -22 167 -11T109 16T69 49T45 79T37 94T69 151Q91 185 97 185Q105 185 112 170Q127 135 160 111T240 87Q266 87 284 94T311 111T325 142T331 179T332 226Q332 307 324 335T281 363Q228 363 197 306Q189 289 172 282H109",
      120818: "414 589Q410 589 389 600T334 612Q275 612 243 575Q209 538 202 433V422L209 431Q243 487 317 487Q392 487 440 442Q478 402 490 357T503 236Q503 113 454 54Q421 13 381 -4T279 -22Q263 -22 250 -21T214 -15T173 1T133 30T96 77T68 146T50 242Q46 278 46 336Q46 406 52 447Q84 698 312 715L315 716Q318 716 321 716Q323 716 328 716T337 715Q398 715 425 688V596Q419 591 414 589ZM282 87Q324 89 336 117T348 231Q348 310 343 343T324 388T277 399Q249 399 231 373T208 317T202 253Q202 201 207 168T224 117T249 93T282 87",
      120819: "135 38Q135 190 198 335T353 572H215Q185 572 151 572T110 571H96Q55 571 48 596Q46 602 46 633V648Q46 686 72 693Q76 695 124 695Q134 695 183 695T274 694Q472 694 477 692Q503 686 503 648V637Q503 612 502 605T491 588Q300 349 292 46V36Q292 -4 266 -9Q262 -11 214 -11H192Q160 -11 148 -3T135 38",
      120820: "61 518Q61 574 79 614T128 676T192 706T263 715H270Q298 715 318 714T373 701T430 671T470 612T488 517Q488 459 458 423T390 376Q388 375 393 373Q395 372 398 371Q503 330 503 204Q503 -22 275 -22Q209 -22 163 -3T92 49T57 120T46 204Q46 230 50 252T61 289T77 318T96 339T116 353T134 363T148 369T158 373T160 376Q118 389 90 424T61 518ZM344 538Q344 563 340 578T326 600T307 609T279 612Q232 612 218 594T204 518Q204 459 216 439T275 418Q328 418 338 450Q344 464 344 515V538ZM248 88T274 88T315 94T338 117T346 149T349 197Q349 269 342 290Q338 309 320 320T274 331Q246 331 229 320T207 290Q200 269 200 197Q201 163 202 149T210 118T232 94",
      120821: "347 272Q346 272 342 266T330 250T309 230T276 214T230 207Q185 207 150 223Q116 240 90 276T54 357Q46 393 46 468Q46 469 46 484T47 502T48 520T51 540T55 559T61 579T69 599T81 620T96 640T115 661Q174 716 276 716Q299 716 317 714T369 698T426 658T471 580T499 456Q503 402 503 342Q503 115 392 29Q322 -22 231 -22Q163 -22 115 7L82 31Q76 38 81 46Q116 112 127 123Q130 126 134 126T148 116T179 97T226 87Q287 87 318 132Q323 139 326 146T332 165T337 182T340 204T342 225T345 249T347 272ZM201 547T201 454T211 329T262 294Q276 294 285 296T310 312T335 355Q347 391 347 447Q347 520 340 550T317 595Q300 612 277 612Q226 612 214 580",
      120822: "42 305Q42 450 111 535T257 621Q335 621 390 562Q482 468 482 306Q482 174 418 82T262 -10T106 82T42 305ZM257 545Q209 545 168 481T126 320Q126 220 162 147Q204 65 262 65Q318 65 358 139T398 320V328Q395 411 364 470T284 543Q270 545 257 545",
      120823: "99 461Q99 470 99 474T104 487T120 498T151 502Q213 517 251 596Q264 622 283 622Q308 622 319 597V76H373H401Q428 76 439 69T450 38Q450 11 428 1H127Q104 10 104 38Q104 62 115 69T153 76H181H235V269Q235 461 234 461Q184 426 137 424H133Q124 424 119 425T109 431T99 447V461",
      120824: "52 462Q52 528 110 575T247 622H250Q343 622 407 565T472 421Q472 371 446 324T390 248T308 178Q307 177 275 151T214 101L185 77Q185 76 286 76H388V87Q388 105 397 114T430 123T463 114Q470 107 471 100T472 61V42Q472 24 468 16T450 1H75Q53 10 53 32V38V48Q53 57 63 67T127 122Q153 144 169 157L289 256Q388 345 388 419Q388 473 346 509T231 545H224Q176 545 146 499L144 494Q155 476 155 459Q154 459 155 455T154 444T148 430T136 417T114 408Q113 408 110 408T104 407Q80 407 66 422T52 462",
      120825: "260 546Q233 546 211 541T180 531T171 524L174 514Q177 505 177 497Q177 476 162 461T125 446Q106 446 90 459T73 504Q76 540 98 565T150 601T203 616T239 621Q241 622 265 622Q322 620 362 602T420 558T444 513T451 478Q451 386 369 329L375 326Q381 323 386 320T401 311T419 298T436 283T452 263T466 240T475 212T479 180Q479 99 416 44T259 -11T105 28T44 130Q44 154 59 168T95 183Q117 183 132 169T148 131Q148 119 139 101Q175 65 260 65Q316 65 355 97T395 179Q395 211 375 240Q336 292 253 292H234H215Q194 292 185 299T175 330Q175 350 184 359Q192 368 238 370T309 384Q336 398 351 423T367 474Q367 496 350 513Q321 546 260 546",
      120826: "235 1Q213 10 213 32V38V46Q213 65 230 73Q236 76 274 76H314V168H183L52 169Q37 175 33 182T29 205V218L30 244Q53 283 155 443T264 613Q276 623 298 623H323H363Q378 616 385 601V244H429H450Q474 244 484 237T495 206Q495 179 477 171Q471 168 429 168H385V76H425H442Q466 76 476 69T487 38Q487 10 465 1H235ZM314 244V554L117 245L215 244H314",
      120827: "387 189Q387 244 354 278T273 313Q230 313 205 301T163 271T138 249H120Q102 249 97 251Q85 258 83 266T80 311Q80 320 80 359T81 430Q81 587 82 591Q88 605 103 610H108Q112 610 120 610T138 610T163 610T192 611T225 611T260 611H415Q416 610 421 607T428 602T432 596T436 587T437 573Q437 567 437 562T434 554T431 548T427 543T423 540T418 538L415 536L289 535H164V363L170 366Q175 368 184 372T207 380T238 386T276 389Q357 389 414 331T472 187Q472 116 412 53T245 -10Q218 -10 209 -9Q126 5 89 48T52 137Q52 164 68 177T104 191Q130 191 143 175T156 141Q156 132 154 125T149 113T146 107Q146 104 155 95T188 76T245 65Q298 65 342 98T387 189",
      120828: "357 536Q357 546 318 546Q258 546 205 497T133 357V353L144 361Q210 402 285 402Q362 402 414 350Q479 285 479 193Q479 111 418 50T263 -11Q234 -11 207 -3T149 26T97 81T60 171T45 301Q45 444 129 533T319 622Q388 622 421 589T454 510Q454 491 442 475T402 458Q373 458 362 475T350 510Q350 520 354 528L357 536ZM319 326T269 326T179 298T136 223Q136 202 143 174T176 112T237 68Q246 66 265 66Q319 66 360 107Q395 146 395 197Q395 250 356 289Q319 326 269 326",
      120829: "204 -10Q162 -10 162 40Q162 146 198 261T310 477Q311 478 321 491T342 517T358 535H128V524Q128 506 119 497Q111 489 86 489H78Q55 489 46 508Q44 513 44 557V580Q44 605 52 616T88 627H93Q114 627 125 611H458Q474 598 477 593T480 573Q480 559 478 553T469 543T446 521T408 477Q252 290 246 49Q246 43 246 37T246 27T245 22Q243 11 233 1T204 -10",
      120830: "58 460Q58 523 117 572T254 621Q290 621 298 620Q376 607 421 560T466 460Q466 441 460 424T443 393T421 370T397 352T374 340T357 332L350 330L356 328Q363 325 371 321T392 310T415 295T439 274T459 249T473 217T479 179Q479 102 418 46T262 -10T106 46T45 179Q45 202 52 222T70 257T96 284T123 305T148 319T167 328L174 330L170 332Q166 333 159 336T145 343Q104 362 81 393T58 460ZM382 458Q382 491 349 518T263 546Q215 546 179 521T142 458Q142 421 178 395T262 368Q315 368 348 396T382 458ZM396 178Q396 223 358 257T263 292Q206 292 167 258T128 178Q128 137 163 102T262 66Q324 66 360 101T396 178",
      120831: "392 259Q333 210 236 210H233Q163 210 109 262Q46 325 46 411T99 550Q164 622 264 622Q293 622 319 615T376 587T428 532T464 440T479 304Q479 167 400 78T217 -11Q140 -11 105 22T70 101Q70 124 84 138T122 153Q150 153 162 137T174 101Q174 91 168 76Q179 65 216 65Q267 65 300 93Q322 109 339 130T366 173T380 210T388 242T392 259ZM388 389Q388 438 357 492T268 546T185 520Q129 479 129 415Q129 384 138 363Q145 349 156 334T195 302T255 285Q305 285 345 313T388 389"
    }, {
      8243: "′′",
      8244: "′′′",
      8246: "‵‵",
      8247: "‵‵‵",
      8279: "′′′′",
      8708: "∄",
      8716: "∌",
      8772: "≄",
      8777: "≉",
      8802: "≢",
      8813: "≭",
      8820: "≴",
      8821: "≵",
      8824: "≸",
      8825: "≹",
      8836: "⊄",
      8837: "⊅",
      8930: "⋢",
      8931: "⋣",
      10764: "∬∬"
    }), normal$1;
  }
  var sansSerifBoldItalic$1 = {}, sansSerifBoldItalic = {}, hasRequiredSansSerifBoldItalic$1;
  function requireSansSerifBoldItalic$1() {
    return hasRequiredSansSerifBoldItalic$1 || (hasRequiredSansSerifBoldItalic$1 = 1, Object.defineProperty(sansSerifBoldItalic, "__esModule", { value: !0 }), sansSerifBoldItalic.sansSerifBoldItalic = void 0, sansSerifBoldItalic.sansSerifBoldItalic = {
      305: [0.458, 0, 0.256],
      567: [0.458, 0.205, 0.286]
    }), sansSerifBoldItalic;
  }
  var hasRequiredSansSerifBoldItalic;
  function requireSansSerifBoldItalic() {
    if (hasRequiredSansSerifBoldItalic) return sansSerifBoldItalic$1;
    hasRequiredSansSerifBoldItalic = 1, Object.defineProperty(sansSerifBoldItalic$1, "__esModule", { value: !0 }), sansSerifBoldItalic$1.sansSerifBoldItalic = void 0;
    var O = requireFontData(), _ = requireSansSerifBoldItalic$1();
    return sansSerifBoldItalic$1.sansSerifBoldItalic = (0, O.AddPaths)(_.sansSerifBoldItalic, {
      305: "54 431Q63 458 102 458H127H149Q192 458 199 433Q201 427 201 229T199 25Q190 0 149 0H125L81 1Q61 7 54 28V431",
      567: "-38 -84Q-36 -84 -14 -95T33 -106H38Q70 -103 78 -86Q83 -78 83 -49T84 180Q84 427 86 433Q93 458 136 458H158H180Q201 458 209 456T225 443Q230 436 231 418Q232 397 232 313V183V124V40Q232 -55 228 -87T203 -147Q166 -205 78 -205Q31 -205 -20 -189T-71 -159Q-71 -156 -59 -123Q-50 -96 -47 -91T-38 -84"
    }, {}), sansSerifBoldItalic$1;
  }
  var sansSerifBold$1 = {}, sansSerifBold = {}, hasRequiredSansSerifBold$1;
  function requireSansSerifBold$1() {
    return hasRequiredSansSerifBold$1 || (hasRequiredSansSerifBold$1 = 1, Object.defineProperty(sansSerifBold, "__esModule", { value: !0 }), sansSerifBold.sansSerifBold = void 0, sansSerifBold.sansSerifBold = {
      33: [0.694, 0, 0.367],
      34: [0.694, -0.442, 0.558],
      35: [0.694, 0.193, 0.917],
      36: [0.75, 0.056, 0.55],
      37: [0.75, 0.056, 1.029],
      38: [0.716, 0.022, 0.831],
      39: [0.694, -0.442, 0.306],
      40: [0.75, 0.249, 0.428],
      41: [0.75, 0.25, 0.428],
      42: [0.75, -0.293, 0.55],
      43: [0.617, 0.116, 0.856],
      44: [0.146, 0.106, 0.306],
      45: [0.273, -0.186, 0.367],
      46: [0.146, 0, 0.306],
      47: [0.75, 0.249, 0.55],
      58: [0.458, 0, 0.306],
      59: [0.458, 0.106, 0.306],
      61: [0.407, -0.094, 0.856],
      63: [0.705, 0, 0.519],
      64: [0.704, 0.011, 0.733],
      91: [0.75, 0.25, 0.343],
      93: [0.75, 0.25, 0.343],
      94: [0.694, -0.537, 0.55],
      95: [-0.023, 0.11, 0.55],
      126: [0.344, -0.198, 0.55],
      305: [0.458, 0, 0.256],
      567: [0.458, 0.205, 0.286],
      768: [0.694, -0.537, 0],
      769: [0.694, -0.537, 0],
      770: [0.694, -0.537, 0],
      771: [0.694, -0.548, 0],
      772: [0.66, -0.56, 0],
      774: [0.694, -0.552, 0],
      775: [0.695, -0.596, 0],
      776: [0.695, -0.595, 0],
      778: [0.694, -0.538, 0],
      779: [0.694, -0.537, 0],
      780: [0.657, -0.5, 0],
      8211: [0.327, -0.24, 0.55],
      8212: [0.327, -0.24, 1.1],
      8213: [0.327, -0.24, 1.1],
      8215: [-0.023, 0.11, 0.55],
      8216: [0.694, -0.443, 0.306],
      8217: [0.694, -0.442, 0.306],
      8220: [0.694, -0.443, 0.558],
      8221: [0.694, -0.442, 0.558],
      8260: [0.75, 0.249, 0.55],
      8710: [0.694, 0, 0.917]
    }), sansSerifBold;
  }
  var hasRequiredSansSerifBold;
  function requireSansSerifBold() {
    if (hasRequiredSansSerifBold) return sansSerifBold$1;
    hasRequiredSansSerifBold = 1, Object.defineProperty(sansSerifBold$1, "__esModule", { value: !0 }), sansSerifBold$1.sansSerifBold = void 0;
    var O = requireFontData(), _ = requireSansSerifBold$1();
    return sansSerifBold$1.sansSerifBold = (0, O.AddPaths)(_.sansSerifBold, {
      33: "111 669Q111 680 111 682T113 689T121 693T137 694H184H249Q255 686 256 681Q244 220 239 213Q236 208 229 208T183 207T137 207T127 213T118 431T111 669ZM110 71V95Q110 137 136 144Q140 146 183 146H200Q246 146 254 121Q256 115 256 73V51Q256 9 230 2Q225 0 181 0L138 1Q121 7 113 21L111 28L110 71",
      34: "38 687Q42 693 45 693Q46 694 111 694H176Q179 690 183 687V556L144 501Q139 494 134 486T126 474T120 465T114 457T110 451T106 447T102 445T98 443T94 443T89 442H73H62Q37 442 37 453Q37 458 55 501T74 546Q74 548 59 548L44 549L38 555V687ZM275 687Q279 693 282 693Q283 694 348 694H413Q416 690 420 687V556L381 501Q376 494 371 486T363 474T357 465T351 457T347 451T343 447T339 445T335 443T331 443T326 442H310H299Q274 442 274 453Q274 458 292 501T311 546Q311 548 296 548L281 549L275 555V687",
      35: "61 365Q61 370 62 375T65 383T69 390T74 395T80 399T85 403T90 406L94 407H344L381 536Q418 668 426 680Q431 687 445 692Q451 694 457 694Q477 694 488 682T500 651Q500 645 466 528T431 409Q431 407 504 407H577L609 521Q651 668 656 675Q669 693 691 693Q710 693 721 680T733 651Q733 645 699 528T664 409Q664 407 743 407H823L827 405Q831 403 834 402T841 397T848 389T853 379T855 365Q855 337 823 324L731 323H639L619 253Q599 181 598 180V177H823L827 175Q831 173 834 172T841 167T848 159T853 149T855 135Q855 107 823 94L698 93H573L540 -21Q498 -168 493 -175Q480 -193 458 -193Q439 -193 428 -180T416 -151Q416 -144 450 -27T485 91Q485 93 412 93H340L307 -21Q265 -168 260 -175Q247 -193 225 -193Q206 -193 195 -180T183 -151Q183 -144 217 -27T252 91Q252 93 173 93L94 94Q61 105 61 135Q61 140 62 145T65 153T69 160T74 165T80 169T85 173T90 176L94 177H185L277 178L297 248L318 320V323H206L94 324Q61 335 61 365ZM551 320V323H479Q406 323 405 322Q404 319 385 249T365 178T438 177L510 178L530 248L551 320",
      36: "231 712L232 716Q232 719 232 722T234 729T239 736T246 743T256 748T271 750Q318 750 318 711V703Q393 692 451 656Q469 645 470 640Q470 635 461 587L453 537Q445 529 442 529Q438 529 424 540T384 565T330 585Q328 585 325 585T320 586L318 587V434Q322 433 333 429T350 424T365 418T382 409T399 397T419 380Q488 313 488 213Q488 24 334 -5L318 -8V-17Q318 -56 279 -56H272Q253 -56 243 -46T232 -30T231 -18V-8H224Q170 2 127 20T68 50T49 67Q49 71 58 122T68 176Q71 182 79 182Q83 182 98 169T145 138T216 110Q228 108 229 108H231V288Q167 299 114 356T61 496Q61 537 70 570T94 624T126 661T162 684T195 696T219 701L228 702H231V712ZM231 521Q231 583 230 583Q226 582 220 579T204 561T193 521Q193 491 212 472Q226 458 230 458Q231 458 231 521ZM318 112Q356 131 356 187Q356 237 318 263V112",
      37: "61 549Q61 733 218 749Q220 749 228 749T241 750Q286 750 321 735T369 708T389 683Q422 634 422 548V538Q422 519 420 501T408 453T381 401T328 364T243 347Q61 347 61 549ZM292 549Q292 663 242 663Q221 663 211 648T198 615T196 566V548Q196 471 206 454Q218 434 242 434Q292 434 292 549ZM243 -55Q223 -55 212 -42T201 -13Q201 -2 207 7Q209 11 480 371T758 738Q770 750 788 750Q805 750 817 738T830 709Q830 694 820 681L544 315Q273 -43 269 -47Q258 -55 243 -55ZM606 146Q606 330 763 346Q764 346 773 346T786 347Q831 347 866 332T914 305T934 280Q966 233 966 146V135Q966 115 964 97T952 49T925 -2T872 -40T788 -56Q606 -56 606 146ZM742 146V132Q742 107 743 93T748 62T762 39T787 31Q819 31 832 79Q837 97 837 146Q837 260 787 260Q767 260 757 246T744 214T742 169V146",
      38: "757 117Q762 117 769 110V3Q759 -7 718 -14T641 -22T571 -15T518 1T477 19T453 31L437 23Q350 -19 257 -22Q158 -22 103 30T47 155Q47 188 57 216T87 263T119 292T151 313L182 333L187 336L181 349Q150 431 150 506Q150 605 211 660T347 716Q417 716 471 668T526 543Q526 517 518 495T471 432T360 343L342 330Q342 327 358 306T402 250T458 189L467 181Q518 225 556 289T607 395L620 437Q622 443 630 443Q631 443 679 430Q718 420 725 418T733 409Q733 402 722 365T670 255T573 123Q562 111 563 111Q589 94 644 94Q678 94 703 100T740 111T757 117ZM397 544Q397 573 384 601T346 629Q320 629 299 607T277 538Q277 484 295 429Q301 413 301 412Q302 409 309 415Q397 476 397 544ZM227 258Q197 228 197 177Q197 150 207 126T234 95Q242 93 251 93Q288 93 337 107L349 110L328 131Q266 196 234 248L227 258",
      39: "81 687Q85 693 88 693Q89 694 154 694H219Q222 690 226 687V556L187 501Q182 494 177 486T169 474T163 465T157 457T153 451T149 447T145 445T141 443T137 443T132 442H116H105Q80 442 80 453Q80 458 98 501T117 546Q117 548 102 548L87 549L81 555V687",
      40: "79 250Q79 352 100 441T152 585T213 678T266 733L287 749Q288 750 324 750H359Q366 741 366 738Q366 734 356 721T329 682T296 623T262 531T238 407Q230 346 230 250Q230 142 244 55T278 -82T318 -165T352 -215T366 -238Q366 -242 359 -249H286L277 -242Q79 -74 79 250",
      41: "61 737Q61 750 85 750H106H141L150 742Q348 574 348 250T150 -242L141 -249L106 -250H87Q61 -250 61 -238Q61 -233 74 -216Q157 -113 183 51Q197 130 197 250T183 449Q174 505 158 554T126 634T95 687T71 722T61 737",
      42: "241 579Q241 582 228 639T215 702Q215 722 233 736T271 750Q296 750 315 736T334 702V697Q334 693 328 664T314 607L308 579L352 620Q389 654 397 660T417 668Q447 668 464 647T482 602Q482 591 479 583T472 569T459 559T443 552T421 546T397 538L342 521L397 504Q405 501 420 497T442 490T458 483T472 473T479 460T482 440Q482 416 465 395T417 374Q406 375 398 381T352 422L308 463L314 435Q321 407 327 378T334 345Q336 333 327 319T296 295Q288 293 275 293Q241 293 227 311T215 345Q215 349 221 378T234 435L241 463L197 422Q160 388 152 382T132 374Q102 374 85 395T67 440Q67 451 70 459T77 473T90 483T106 490T128 496T152 504L207 521L152 538Q144 541 129 545T107 552T91 559T77 569T70 582T67 602Q67 626 84 647T132 668Q143 667 151 661T197 620L241 579",
      43: "61 250Q61 276 94 292H386V436V535Q386 577 388 589T401 607Q411 617 427 617Q458 617 468 587Q470 581 470 436V292H762L766 290Q770 288 773 287T780 282T787 274T792 264T794 250Q794 222 762 209L616 208H470V64Q470 -81 468 -87Q458 -116 428 -116T388 -87Q386 -81 386 64V208H240L94 209Q61 220 61 250",
      44: "81 139Q85 145 88 145Q89 146 154 146H219Q222 142 226 139V8L187 -47Q182 -54 177 -62T169 -74T163 -83T157 -91T153 -97T149 -101T145 -103T141 -105T137 -105T132 -106H116H105Q80 -106 80 -95Q80 -90 98 -47T117 -2Q117 0 102 0L87 1L81 7V139",
      45: "12 230Q12 257 26 265T80 274Q88 274 114 274T158 273T201 273T235 274Q276 274 290 266T305 230T291 194T235 185Q226 185 201 185T159 186Q143 186 119 186T85 185Q43 185 28 193T12 230",
      46: "219 146Q222 142 226 139V7L222 4L219 1L154 0Q102 0 94 0T82 6Q80 9 80 74L81 139Q85 145 88 145Q89 146 154 146H219",
      47: "103 -249Q81 -249 71 -235T61 -207Q61 -201 62 -198Q64 -192 235 265T409 727Q418 750 445 750Q464 750 476 737T488 707Q488 701 313 234Q143 -225 137 -232Q126 -249 103 -249",
      58: "226 319L219 313H87L81 319L80 384Q80 437 80 445T86 456Q89 458 154 458H219Q222 454 226 451V319ZM219 146Q222 142 226 139V7L222 4L219 1L154 0Q102 0 94 0T82 6Q80 9 80 74L81 139Q85 145 88 145Q89 146 154 146H219",
      59: "226 319L219 313H87L81 319L80 384Q80 437 80 445T86 456Q89 458 154 458H219Q222 454 226 451V319ZM81 139Q85 145 88 145Q89 146 154 146H219Q222 142 226 139V8L187 -47Q182 -54 177 -62T169 -74T163 -83T157 -91T153 -97T149 -101T145 -103T141 -105T137 -105T132 -106H116H105Q80 -106 80 -95Q80 -90 98 -47T117 -2Q117 0 102 0L87 1L81 7V139",
      61: "94 324Q61 335 61 366Q61 396 91 405Q96 407 429 407H762Q763 406 767 404T774 400T781 395T787 387T792 378T794 365Q794 338 762 324H94ZM94 94Q61 105 61 135Q61 149 69 160T92 175Q97 177 430 177H762L766 175Q770 173 773 172T780 167T787 159T792 149T794 135Q794 107 762 94H94",
      63: "61 644Q61 652 87 666T157 693T244 705Q344 705 400 671T457 551Q457 516 446 490T422 451T387 421T356 391Q330 361 318 332T305 292T303 252Q303 218 300 213T290 208T244 207H220Q194 207 188 213Q187 214 186 215V255Q187 282 188 296T198 345T229 417T288 496Q306 515 306 559Q306 596 296 607T253 618Q214 618 185 607T143 583T120 558T103 547Q99 547 95 551Q93 553 77 597T61 644ZM171 71V95Q171 137 197 144Q201 146 244 146H261Q307 146 315 121Q317 115 317 73V51Q317 9 291 2Q286 0 242 0L199 1Q182 7 174 21L172 28L171 71",
      64: "61 264T61 347T82 494T136 596T217 660T311 694T410 704Q460 704 471 703Q534 694 577 666Q633 623 651 552T670 370V342Q670 249 633 195Q583 116 454 116Q238 116 238 347Q238 443 276 499Q328 578 456 578Q488 578 494 577L504 575Q475 617 430 617H421Q196 617 196 347Q196 215 253 143Q310 76 427 76Q499 76 561 102L575 107H664Q671 97 671 94V89L663 81Q566 -11 422 -11Q365 -11 316 -2T219 33T137 97T82 200ZM469 490Q459 492 453 492Q429 492 405 472Q374 439 374 347Q374 233 423 210Q436 202 454 202L486 210Q536 228 536 347Q536 461 486 484Q476 490 469 490",
      91: "318 -206Q318 -235 305 -243T255 -251Q248 -251 229 -251T198 -250H143Q112 -250 99 -246T81 -225Q79 -219 79 250T81 725Q85 741 98 745T143 750H198Q210 750 229 750T255 751Q291 751 304 743T318 707Q318 680 301 668Q293 663 255 663H224V-163H255Q293 -163 301 -168Q318 -180 318 -206",
      93: "24 706Q24 734 39 742T90 751Q97 751 114 751T143 750H198Q230 750 243 746T261 725Q263 719 263 250T261 -225Q257 -241 244 -245T198 -250H143Q131 -250 112 -250T86 -251Q50 -251 37 -243T24 -207Q24 -180 41 -168Q49 -163 87 -163H118V663H87H71Q24 663 24 706",
      94: "108 550Q108 554 135 589T190 658T219 692Q221 694 275 694Q328 694 330 693Q331 692 381 629T438 557Q441 553 441 549T434 538L399 537Q363 537 362 538Q361 538 318 575L275 611Q274 611 231 575Q188 538 187 538Q186 537 150 537L115 538Q108 545 108 550",
      95: "0 -66Q0 -32 26 -25Q30 -23 274 -23Q469 -23 497 -23T532 -28Q549 -40 549 -67Q549 -93 532 -105Q525 -109 498 -109T275 -110Q31 -110 26 -108Q0 -101 0 -66",
      126: "92 215Q92 259 122 301T204 344Q238 344 264 329T310 300T343 285Q356 285 361 295T369 322T377 344H450Q457 334 457 330Q457 281 427 240T344 198Q312 198 285 213T239 242T206 257Q188 257 182 230T172 199L137 198H120Q102 198 97 200T92 215",
      305: "54 431Q63 458 102 458H127H149Q192 458 199 433Q201 427 201 229T199 25Q190 0 149 0H125L81 1Q61 7 54 28V431",
      567: "-38 -84Q-36 -84 -14 -95T33 -106H38Q70 -103 78 -86Q83 -78 83 -49T84 180Q84 427 86 433Q93 458 136 458H158H180Q201 458 209 456T225 443Q230 436 231 418Q232 397 232 313V183V124V40Q232 -55 228 -87T203 -147Q166 -205 78 -205Q31 -205 -20 -189T-71 -159Q-71 -156 -59 -123Q-50 -96 -47 -91T-38 -84",
      768: "-458 682Q-458 690 -452 692T-426 694H-381H-314L-312 691Q-311 691 -305 682T-287 655T-263 622L-218 555V549Q-218 544 -224 538L-259 537Q-295 537 -296 538Q-298 539 -376 606T-456 676Q-458 680 -458 682",
      769: "-290 537H-310Q-334 537 -334 549Q-334 553 -311 588T-264 656L-241 690Q-240 690 -239 691T-236 693Q-235 694 -167 694H-100Q-93 684 -93 681T-94 677Q-95 675 -173 607T-255 538Q-256 537 -290 537",
      770: "-442 550Q-442 554 -415 589T-360 658T-331 692Q-329 694 -275 694Q-222 694 -220 693Q-219 692 -169 629T-112 557Q-109 552 -109 549Q-109 545 -116 538L-151 537Q-187 537 -188 538Q-189 538 -232 575L-275 611Q-276 611 -319 575Q-362 538 -363 538Q-364 537 -400 537L-435 538Q-442 545 -442 550",
      771: "-458 565Q-458 609 -428 651T-346 694Q-312 694 -286 679T-240 650T-207 635Q-194 635 -189 645T-181 672T-173 694H-100Q-93 684 -93 680Q-93 631 -123 590T-206 548Q-238 548 -265 563T-311 592T-344 607Q-362 607 -368 580T-378 549L-413 548H-430Q-448 548 -453 550T-458 565",
      772: "-84 660Q-81 656 -77 653V567L-81 564L-84 561L-274 560H-383Q-469 560 -471 565L-472 566Q-474 569 -474 611L-473 653Q-469 659 -466 659Q-465 660 -274 660H-84",
      774: "-123 694Q-80 694 -80 657Q-80 626 -99 601T-161 563Q-199 552 -275 552Q-352 552 -389 563Q-470 586 -470 655Q-470 667 -468 673Q-457 694 -435 694H-431Q-408 694 -396 685Q-387 676 -387 671Q-384 661 -275 661Q-167 661 -164 671Q-164 674 -163 677T-151 687T-123 694",
      775: "-329 596Q-346 602 -351 611T-356 638V646Q-356 653 -356 654T-356 661T-355 668T-353 673T-351 679T-347 684T-341 689T-332 693T-274 695H-221Q-202 683 -198 676T-194 645Q-194 632 -195 625T-202 610T-221 596H-329",
      776: "-331 695Q-312 683 -308 676T-304 645Q-304 632 -304 626T-311 610T-331 596L-380 595H-408Q-448 595 -457 617Q-459 621 -459 645T-457 673Q-448 696 -409 696Q-405 696 -396 696T-380 695H-331ZM-247 644Q-247 658 -246 665T-239 680T-221 694Q-217 695 -169 695H-143Q-102 695 -93 672Q-91 664 -91 645V635Q-91 613 -106 602Q-113 597 -121 596T-171 595L-219 596Q-232 600 -238 608T-246 622T-247 644",
      778: "-365 616Q-365 658 -331 676T-256 694Q-253 694 -247 694T-236 693Q-166 693 -139 666Q-119 644 -119 616T-139 565Q-166 538 -237 538H-242Q-365 538 -365 616ZM-181 616Q-181 641 -195 647T-242 654Q-258 654 -266 654T-284 650T-298 638T-303 616Q-303 592 -289 585T-242 577Q-209 577 -195 584T-181 616",
      779: "-426 686Q-424 694 -394 694H-350H-283Q-277 686 -277 682Q-277 673 -317 608T-361 538L-396 537H-420Q-432 537 -436 539T-440 548Q-440 560 -434 616Q-432 633 -430 650T-427 677L-426 686ZM-243 686Q-241 694 -211 694H-167H-100Q-94 686 -94 682Q-94 673 -134 608T-178 538L-213 537H-237Q-249 537 -253 539T-257 548Q-257 560 -251 616Q-249 633 -247 650T-244 677L-243 686",
      780: "-442 645Q-442 657 -418 657H-398Q-393 657 -388 657T-379 657T-371 656T-365 656H-363L-319 620L-276 583Q-275 583 -232 619Q-189 656 -188 656Q-187 657 -151 657H-116Q-109 649 -109 645Q-109 642 -112 637Q-118 629 -168 566T-220 501Q-222 500 -275 500Q-329 500 -331 501Q-442 634 -442 645",
      8211: "0 284Q0 318 26 325Q30 327 274 327Q469 327 497 327T532 322Q549 310 549 283Q549 257 532 245Q525 241 498 241T275 240Q31 240 26 242Q0 249 0 284",
      8212: "0 284Q0 318 26 325Q30 327 549 327T1073 325Q1099 318 1099 284Q1099 249 1073 242Q1068 240 549 240Q31 240 26 242Q0 249 0 284",
      8213: "0 284Q0 318 26 325Q30 327 549 327T1073 325Q1099 318 1099 284Q1099 249 1073 242Q1068 240 549 240Q31 240 26 242Q0 249 0 284",
      8215: "0 -66Q0 -32 26 -25Q30 -23 274 -23Q469 -23 497 -23T532 -28Q549 -40 549 -67Q549 -93 532 -105Q525 -109 498 -109T275 -110Q31 -110 26 -108Q0 -101 0 -66",
      8216: "87 443L81 449V581L119 636Q125 644 131 653T141 667T148 677T154 685T158 689T163 692T167 693T173 694H190H201Q226 694 226 683Q226 678 208 635T189 590Q189 588 204 588H219Q222 584 226 581V449L219 443H87",
      8217: "81 687Q85 693 88 693Q89 694 154 694H219Q222 690 226 687V556L187 501Q182 494 177 486T169 474T163 465T157 457T153 451T149 447T145 445T141 443T137 443T132 442H116H105Q80 442 80 453Q80 458 98 501T117 546Q117 548 102 548L87 549L81 555V687",
      8220: "144 443L138 449V581L176 636Q182 644 188 653T198 667T205 677T211 685T215 689T220 692T224 693T230 694H247H258Q283 694 283 683Q283 678 265 635T246 590Q246 588 261 588H276Q279 584 283 581V449L276 443H144ZM381 443L375 449V581L413 636Q419 644 425 653T435 667T442 677T448 685T452 689T457 692T461 693T467 694H484H495Q520 694 520 683Q520 678 502 635T483 590Q483 588 498 588H513Q516 584 520 581V449L513 443H381",
      8221: "38 687Q42 693 45 693Q46 694 111 694H176Q179 690 183 687V556L144 501Q139 494 134 486T126 474T120 465T114 457T110 451T106 447T102 445T98 443T94 443T89 442H73H62Q37 442 37 453Q37 458 55 501T74 546Q74 548 59 548L44 549L38 555V687ZM275 687Q279 693 282 693Q283 694 348 694H413Q416 690 420 687V556L381 501Q376 494 371 486T363 474T357 465T351 457T347 451T343 447T339 445T335 443T331 443T326 442H310H299Q274 442 274 453Q274 458 292 501T311 546Q311 548 296 548L281 549L275 555V687",
      8260: "103 -249Q81 -249 71 -235T61 -207Q61 -201 62 -198Q64 -192 235 265T409 727Q418 750 445 750Q464 750 476 737T488 707Q488 701 313 234Q143 -225 137 -232Q126 -249 103 -249",
      8710: "381 692Q386 694 458 694Q516 694 527 693T549 687Q564 680 575 663Q576 658 715 349T856 27Q856 6 838 1H826Q815 1 795 1T747 1T686 1T616 0T539 0T458 0T378 0T300 0T230 0T169 1T122 1T90 1H78Q60 6 60 27Q62 38 201 349T341 663Q356 687 381 692ZM627 148Q626 149 581 250T492 453L447 554Q447 553 446 552Q444 546 326 278L268 148Q268 147 448 147Q627 147 627 148"
    }, {}), sansSerifBold$1;
  }
  var sansSerifItalic$1 = {}, sansSerifItalic = {}, hasRequiredSansSerifItalic$1;
  function requireSansSerifItalic$1() {
    return hasRequiredSansSerifItalic$1 || (hasRequiredSansSerifItalic$1 = 1, Object.defineProperty(sansSerifItalic, "__esModule", { value: !0 }), sansSerifItalic.sansSerifItalic = void 0, sansSerifItalic.sansSerifItalic = {
      33: [0.694, 0, 0.319, { ic: 0.036 }],
      34: [0.694, -0.471, 0.5],
      35: [0.694, 0.194, 0.833, { ic: 0.018 }],
      36: [0.75, 0.056, 0.5, { ic: 0.065 }],
      37: [0.75, 0.056, 0.833],
      38: [0.716, 0.022, 0.758],
      39: [0.694, -0.471, 0.278, { ic: 0.057 }],
      40: [0.75, 0.25, 0.389, { ic: 0.102 }],
      41: [0.75, 0.25, 0.389],
      42: [0.75, -0.306, 0.5, { ic: 0.068 }],
      43: [0.583, 0.083, 0.778],
      44: [0.098, 0.125, 0.278],
      45: [0.259, -0.186, 0.333],
      46: [0.098, 0, 0.278],
      47: [0.75, 0.25, 0.5, { ic: 0.1 }],
      48: [0.678, 0.022, 0.5, { ic: 0.049 }],
      49: [0.678, 0, 0.5],
      50: [0.678, 0, 0.5, { ic: 0.051 }],
      51: [0.678, 0.022, 0.5, { ic: 0.044 }],
      52: [0.656, 0, 0.5, { ic: 0.021 }],
      53: [0.656, 0.022, 0.5, { ic: 0.055 }],
      54: [0.678, 0.022, 0.5, { ic: 0.048 }],
      55: [0.656, 0.011, 0.5, { ic: 0.096 }],
      56: [0.678, 0.022, 0.5, { ic: 0.054 }],
      57: [0.677, 0.022, 0.5, { ic: 0.045 }],
      58: [0.444, 0, 0.278],
      59: [0.444, 0.125, 0.278],
      61: [0.37, -0.13, 0.778, { ic: 0.018 }],
      63: [0.704, 0, 0.472, { ic: 0.064 }],
      64: [0.705, 0.01, 0.667, { ic: 0.04 }],
      91: [0.75, 0.25, 0.289, { ic: 0.136 }],
      93: [0.75, 0.25, 0.289, { ic: 0.064 }],
      94: [0.694, -0.527, 0.5, { ic: 0.033 }],
      95: [-0.038, 0.114, 0.5, { ic: 0.065 }],
      126: [0.327, -0.193, 0.5, { ic: 0.06 }],
      305: [0.444, 0, 0.239, { ic: 0.019 }],
      567: [0.444, 0.204, 0.267, { ic: 0.019 }],
      768: [0.694, -0.527, 0],
      769: [0.694, -0.527, 0, { ic: 0.063 }],
      770: [0.694, -0.527, 0, { ic: 0.033 }],
      771: [0.677, -0.543, 0, { ic: 0.06 }],
      772: [0.631, -0.552, 0, { ic: 0.064 }],
      774: [0.694, -0.508, 0, { ic: 0.073 }],
      775: [0.68, -0.576, 0],
      776: [0.68, -0.582, 0, { ic: 0.04 }],
      778: [0.693, -0.527, 0],
      779: [0.694, -0.527, 0, { ic: 0.063 }],
      780: [0.654, -0.487, 0, { ic: 0.06 }],
      913: [0.694, 0, 0.667],
      914: [0.694, 0, 0.667, { ic: 0.029 }],
      915: [0.691, 0, 0.542, { ic: 0.104 }],
      916: [0.694, 0, 0.833],
      917: [0.691, 0, 0.597, { ic: 0.091 }],
      918: [0.694, 0, 0.611, { ic: 0.091 }],
      919: [0.694, 0, 0.708, { ic: 0.06 }],
      920: [0.715, 0.022, 0.778, { ic: 0.026 }],
      921: [0.694, 0, 0.278, { ic: 0.06 }],
      922: [0.694, 0, 0.694, { ic: 0.091 }],
      923: [0.694, 0, 0.611],
      924: [0.694, 0, 0.875, { ic: 0.054 }],
      925: [0.694, 0, 0.708, { ic: 0.058 }],
      926: [0.688, 0, 0.667, { ic: 0.098 }],
      927: [0.716, 0.022, 0.736, { ic: 0.027 }],
      928: [0.691, 0, 0.708, { ic: 0.06 }],
      929: [0.694, 0, 0.639, { ic: 0.051 }],
      931: [0.694, 0, 0.722, { ic: 0.091 }],
      932: [0.688, 0, 0.681, { ic: 0.109 }],
      933: [0.716, 0, 0.778, { ic: 0.065 }],
      934: [0.694, 0, 0.722, { ic: 0.021 }],
      935: [0.694, 0, 0.667, { ic: 0.091 }],
      936: [0.694, 0, 0.778, { ic: 0.076 }],
      937: [0.716, 0, 0.722, { ic: 0.047 }],
      8211: [0.312, -0.236, 0.5, { ic: 0.065 }],
      8212: [0.312, -0.236, 1, { ic: 0.065 }],
      8213: [0.312, -0.236, 1, { ic: 0.065 }],
      8215: [-0.038, 0.114, 0.5, { ic: 0.065 }],
      8216: [0.694, -0.471, 0.278, { ic: 0.058 }],
      8217: [0.694, -0.471, 0.278, { ic: 0.057 }],
      8220: [0.694, -0.471, 0.5, { ic: 0.114 }],
      8221: [0.694, -0.471, 0.5],
      8260: [0.75, 0.25, 0.5, { ic: 0.1 }],
      8710: [0.694, 0, 0.833]
    }), sansSerifItalic;
  }
  var hasRequiredSansSerifItalic;
  function requireSansSerifItalic() {
    if (hasRequiredSansSerifItalic) return sansSerifItalic$1;
    hasRequiredSansSerifItalic = 1, Object.defineProperty(sansSerifItalic$1, "__esModule", { value: !0 }), sansSerifItalic$1.sansSerifItalic = void 0;
    var O = requireFontData(), _ = requireSansSerifItalic$1();
    return sansSerifItalic$1.sansSerifItalic = (0, O.AddPaths)(_.sansSerifItalic, {
      33: "160 187L257 694H306Q355 694 355 693L238 186H199Q160 186 160 187ZM110 2Q111 3 120 49T131 96Q131 98 180 98T229 96L219 50Q209 3 208 2V0H110V2",
      34: "171 647L180 694H229Q278 694 278 693L276 686Q275 680 273 668T268 644L258 597L182 471H157Q133 471 133 472L189 595Q189 596 174 596H160V598Q160 601 171 647ZM365 647L374 694H423Q472 694 472 693L470 686Q469 680 467 668T462 644L452 597L376 471H351Q327 471 327 472L383 595Q383 596 368 596H354V598Q354 601 365 647",
      35: "793 170Q809 162 809 149Q809 145 807 141T802 135T796 132L793 130H547L472 -27Q397 -184 394 -187Q389 -194 379 -194Q367 -194 362 -183Q359 -179 359 -173Q360 -168 431 -20L503 129Q503 130 410 130H317L242 -27Q167 -184 164 -187Q159 -194 149 -194Q137 -194 132 -183Q129 -179 129 -173Q130 -168 201 -19L273 130H187L100 131Q87 141 87 150Q87 162 102 170H294L331 248Q339 265 349 286T365 318L370 328Q370 330 258 330Q145 330 142 332Q129 338 129 351Q129 362 140 368Q146 370 267 370L391 371L467 527Q542 684 544 686Q544 688 549 691T560 694H562Q565 693 567 692T571 690T575 686T578 681T579 672Q577 665 507 520T436 373L435 370H528L621 371L692 518Q767 675 769 677Q775 694 789 694Q798 694 804 688T809 672Q806 664 737 519L665 371L751 370Q835 370 841 368Q851 362 851 350Q851 337 841 334T799 330H765H741H645L606 250L568 170H793ZM600 328Q600 330 508 330H415Q412 326 338 171Q338 170 431 170H524L561 248Q569 265 579 286T595 318L600 328",
      36: "228 70Q233 92 246 155T270 266T280 316Q271 318 265 320T237 333T200 360T172 403T159 468Q159 537 205 600T325 691Q352 701 360 701Q361 701 361 701T362 703T364 711T368 727L372 750H409Q445 750 445 749L436 705Q436 703 450 702T494 691T554 657L565 649Q562 642 548 604L534 568Q511 591 484 605T440 621L424 623L419 624L372 405Q399 400 424 384Q490 338 490 247V240Q490 156 430 85Q374 13 294 -5L284 -7L280 -30Q279 -35 278 -41T275 -52L274 -55Q274 -56 237 -56Q201 -56 201 -54Q202 -53 205 -34T211 -11Q211 -9 206 -9Q154 -2 115 19Q80 35 56 59L88 141L99 131Q109 121 119 113T141 99T160 89T180 82T197 77T214 73T228 70ZM303 426Q304 427 313 471T332 564T345 620L335 616Q287 596 263 549Q252 525 252 499Q252 470 267 451T298 426Q303 424 303 426ZM302 75Q305 75 315 80T340 98T367 125T390 164T399 214Q399 247 384 268T349 297Q338 247 326 186L302 75",
      37: "268 347Q224 347 195 386T165 488Q165 517 173 552Q191 637 246 693T349 749Q389 749 414 725T448 673T456 614Q456 506 396 427T268 347ZM372 604Q372 674 339 674Q311 674 290 633T261 549T253 482V474Q253 438 272 426Q277 424 286 424Q319 424 345 485T372 604ZM189 -56Q179 -56 173 -49T167 -37Q167 -30 347 198Q425 296 475 360Q780 745 785 747Q790 750 796 750Q814 748 814 730Q814 725 811 719L204 -49Q198 -56 189 -56ZM523 87Q523 184 583 265T713 347Q758 347 786 308T815 207Q815 110 757 28T629 -55Q576 -55 550 -12T523 87ZM729 200Q729 271 696 271Q675 271 658 247T631 189T616 125T611 76Q611 21 644 21H647Q672 21 700 77T729 200",
      38: "219 -22Q158 -22 117 13T71 111Q71 131 74 150T84 185T98 215T118 241T137 262T159 281T179 295T199 308L214 318L258 348L256 362Q254 373 254 413V435Q254 483 271 537T325 641T411 708Q427 715 441 715Q446 716 455 716Q504 716 534 681T565 590Q565 522 519 468T377 347L358 334Q359 333 363 320T374 290T387 262Q404 227 428 187T460 139Q521 183 574 251T651 362T674 409L710 398Q746 388 747 388Q747 381 720 333T635 213T517 94L510 87Q542 57 598 57Q649 57 708 72Q716 75 718 75L709 34L701 -7Q636 -22 578 -22Q531 -22 498 -8T428 34L408 25Q314 -22 219 -22ZM480 579Q480 640 436 640Q410 640 385 615T351 554Q340 513 340 457Q340 413 343 410Q343 406 360 419Q431 471 455 505T480 579ZM245 57Q279 59 311 67T359 81T375 89T358 113T318 178T281 260L274 277L245 257Q167 205 167 135Q167 110 174 93T194 69T217 60T237 57H245",
      39: "228 647L237 694H286Q335 694 335 693L334 686Q332 680 330 668T325 644L315 597L239 471H214Q190 471 190 472L246 595Q246 596 231 596H217V598Q217 601 228 647",
      40: "195 37Q195 -7 200 -47T213 -113T231 -166T250 -204T268 -232T280 -250H204L194 -238Q104 -124 104 55Q104 238 181 432T405 740L417 750H454Q491 750 491 749L468 729Q446 709 411 667T337 565T262 405T208 188Q195 110 195 37",
      41: "300 463Q300 634 222 740L214 750H290L299 740Q300 738 309 726T323 707T337 682T353 651T367 613T379 566T387 510T390 444Q390 314 344 156T203 -125Q179 -155 145 -191Q111 -224 89 -241L78 -250H2Q4 -248 27 -227T65 -189T107 -140T155 -71T200 16T244 129T278 266Q300 372 300 463",
      42: "193 608Q193 628 210 644T246 660Q250 660 252 660T257 658T264 654T272 648T284 638T302 623Q340 590 340 593Q341 594 345 623T354 682T360 715Q365 729 378 739T407 750Q424 750 433 740T443 720Q443 712 427 652L410 591L462 623Q505 650 514 655T534 660Q549 660 558 650T568 625Q568 617 567 611T560 599T551 590T536 580T519 571T496 561T470 548L429 528L474 500Q482 495 492 489T506 481T516 475T523 469T527 464T529 458T530 450Q530 430 514 414T479 397H475Q468 397 460 402T423 433Q414 440 404 448T388 461L383 465L365 344Q348 306 314 306Q302 306 292 313T281 338Q281 347 297 404L313 464L260 433Q201 397 195 397H189Q173 397 165 407T156 432Q156 438 157 443T161 452T166 460T175 468T185 475T198 482T212 489T230 497T250 506L295 528L250 556Q203 582 202 585Q193 591 193 608",
      43: "108 244T108 250T112 261T119 268T124 270H426V272Q428 274 457 419Q489 565 492 573Q497 583 508 583Q516 583 522 577T528 565Q528 553 498 417Q491 384 483 346T471 288L467 270H760Q775 262 775 250T760 230H458Q456 221 426 77T394 -71Q389 -83 375 -83Q367 -83 362 -78T356 -64Q356 -58 387 84Q394 118 401 155T413 210L417 229Q417 230 271 230H124Q123 230 120 232T112 239",
      44: "90 2Q91 3 100 49T111 96Q111 98 160 98T209 96L199 50Q189 3 188 2Q188 0 149 -63L112 -125H63L120 0H105Q90 0 90 2",
      45: "66 257V259H332V257L324 220L317 186H184Q51 186 51 187T58 220T66 257",
      46: "90 2Q91 3 100 49T111 96Q111 98 160 98T209 96L199 50Q189 3 188 2V0H90V2",
      47: "564 744L568 747Q573 750 579 750Q588 750 594 744T599 729Q597 721 321 241T41 -243Q37 -250 27 -250Q6 -250 6 -230Q6 -228 8 -222Q9 -219 285 261T564 744",
      48: "245 -22Q209 -22 181 -11T135 20T107 65T92 116T88 171Q88 235 114 354T194 557Q226 606 269 635T340 671T392 678H395Q422 678 446 670T495 643T534 582T549 481Q549 430 534 350T499 213Q459 89 379 25Q315 -22 247 -22H245ZM430 582Q408 601 378 601Q313 601 269 534Q234 475 205 341Q181 232 181 174Q181 104 209 76Q231 54 260 54T318 73T368 125Q410 194 447 375Q460 445 460 487Q460 555 430 582",
      49: "234 613Q277 613 331 628T428 678H439Q451 678 451 676Q450 671 387 373T323 74T384 73H445L430 0H259L88 1L104 73H229L332 560Q278 541 198 539Q198 540 198 541T199 546T200 554T202 564T205 576L213 612H219Q221 612 226 612T234 613",
      50: "190 460Q189 460 181 475T164 507T155 527Q155 535 182 571Q259 678 380 678Q462 678 506 630T551 513V507Q551 418 487 349Q469 329 441 305T391 265T344 232T316 212Q158 87 158 86T188 85Q194 85 234 85T311 86Q467 86 467 85Q451 9 449 2V0H50Q54 18 58 40L67 79L133 133Q246 226 269 243Q369 318 410 373T452 492Q452 535 433 560T393 592T350 599Q311 599 279 578T231 532T203 484T190 460",
      51: "446 542Q446 576 424 590T372 605Q330 605 288 583T216 524Q209 515 208 516Q207 517 192 549L178 580L187 589Q224 627 276 652T386 678Q456 678 500 642T544 550Q544 515 530 482T495 427T453 387T418 362L403 353L413 348Q440 335 462 313Q500 271 500 217Q500 135 423 57T236 -22T63 59L56 68L85 141Q106 112 125 98Q177 54 254 54Q315 54 355 105T396 218Q396 242 393 254Q380 301 335 313Q327 315 280 316Q233 316 233 318L249 392Q298 392 322 399Q373 408 409 453T446 542",
      52: "78 235L411 656H465Q519 656 519 655T475 447T430 237V235H521V233L505 160Q505 159 459 159H414L380 0H286L320 159H62L63 164Q64 169 66 179T70 198L78 235ZM342 235L421 607Q420 607 419 604Q409 535 197 267Q173 236 173 235H342",
      53: "330 350Q263 350 214 272H133V275Q134 276 174 467L214 655Q214 656 385 656H555V653Q555 652 554 647T550 631T546 613L539 577H284L265 486Q261 464 256 441T248 406L246 395L250 398Q255 401 264 406T286 415T315 423T350 427Q412 427 455 381T498 256Q498 150 415 64T222 -22Q186 -22 155 -12T105 12T74 41T55 65T50 77L51 79Q61 89 78 112L104 145L107 138Q110 130 114 123T125 106T142 88T165 72T196 60T236 55Q282 55 316 79T366 140T389 208T396 267Q396 310 378 330T337 350H330",
      54: "437 605Q397 605 361 585T301 536T261 477T236 426T228 401L236 408Q244 414 260 424T296 445T345 462T402 469H404Q422 469 434 467T465 446T498 394Q515 351 515 307Q515 254 497 193T439 85Q352 -22 246 -22Q220 -22 196 -14T148 15T109 78T94 179Q94 272 123 373Q163 505 257 591T450 678Q474 678 498 674T535 664T548 656L540 621L532 586L520 590Q509 594 485 599T437 605ZM339 392Q281 392 233 334T185 163V158Q185 87 230 61Q244 54 262 54Q325 54 371 122Q395 158 407 217T419 298Q419 337 401 364T339 392",
      55: "173 614L181 656H389Q596 656 596 655L595 650Q594 645 592 635T588 616L580 578L554 551Q313 307 245 4L242 -11H192Q143 -11 143 -10Q144 0 148 17T169 89T212 198T285 327T393 470Q423 504 472 550Q479 555 485 561T496 571L329 570Q163 570 163 571L164 577Q166 583 168 593T173 614",
      56: "159 470Q159 547 229 612T394 678Q467 678 510 636T554 533Q554 512 549 493T535 458T515 429T492 405T467 386T443 372T423 362T409 356L404 354Q404 353 405 353Q411 353 432 341T476 295T500 218Q500 134 424 56T246 -22Q175 -22 126 22T77 143Q77 204 110 251T188 327L202 334Q216 340 229 346T243 353T235 358T214 372T189 393T168 426T159 470ZM467 527Q467 605 375 605Q317 605 281 566T244 472Q244 429 271 411T334 392Q392 392 429 430T467 527ZM405 228Q405 262 384 289T315 316Q257 316 216 266T174 144Q174 95 199 75T262 54Q329 54 367 109T405 228",
      57: "220 594Q303 677 389 677Q545 677 545 479Q545 413 526 327Q493 175 398 77T202 -22Q124 -22 77 25L130 91L137 83Q169 54 218 54Q255 54 290 76T347 129Q364 151 380 182T403 232T411 256Q410 255 390 241T353 217T303 197T236 187Q195 187 173 209Q155 226 140 263T124 352Q124 392 135 435Q154 527 220 594ZM455 497Q455 605 383 605Q340 605 305 577T246 492Q220 411 220 360Q220 278 279 264Q280 264 287 264T299 263Q347 263 387 302Q455 375 455 497",
      58: "174 396L184 444H233Q282 444 282 443Q277 421 272 394L262 346H213Q164 346 164 347Q169 369 174 396ZM90 2Q91 3 100 49T111 96Q111 98 160 98T209 96L199 50Q189 3 188 2V0H90V2",
      59: "174 396L184 444H233Q282 444 282 443Q277 421 272 394L262 346H213Q164 346 164 347Q169 369 174 396ZM90 2Q91 3 100 49T111 96Q111 98 160 98T209 96L199 50Q189 3 188 2Q188 0 149 -63L112 -125H63L120 0H105Q90 0 90 2",
      61: "142 368Q145 370 463 370Q780 370 784 368Q796 364 796 350T784 332Q780 330 463 330Q145 330 142 332Q129 338 129 351Q129 362 142 368ZM88 137T88 150T102 170H738Q739 170 742 168T750 161T754 150T750 139T743 132T738 130H102Q88 137 88 150",
      63: "194 652Q194 654 218 666T284 691T362 704Q444 704 490 678T536 583Q536 541 516 500T459 433Q415 400 387 371T343 313T321 266T307 216L301 186H262Q223 186 223 187Q224 199 228 218T250 288T294 377Q317 413 344 440T391 481T414 499Q442 527 442 574Q442 584 441 590T433 607T409 623T362 629Q335 629 310 624T267 610T235 595T214 582T205 576L200 614Q194 651 194 652ZM173 2Q174 3 183 49T194 96Q194 98 243 98T292 96L282 50Q272 3 271 2V0H173V2",
      64: "120 267Q120 377 179 478T336 642T538 705Q610 705 658 662T707 513Q707 425 681 331Q658 241 590 179T447 117Q386 117 343 163T300 288Q300 397 374 486T544 576Q575 576 608 562Q590 628 517 628Q406 628 309 522T212 278Q212 179 267 122T404 65T550 91H631Q513 -10 390 -10Q265 -10 193 70T120 267ZM600 397Q600 441 581 471T530 501Q476 501 433 436T390 298Q390 254 409 224T462 193Q512 193 556 257T600 397",
      91: "148 252L253 750H339Q425 750 425 749L424 744Q423 739 421 729T417 711L409 675L367 674H325L235 252Q145 -167 145 -172Q145 -174 187 -174H229V-176Q213 -240 213 -250H127Q41 -250 41 -248Q41 -245 148 252",
      93: "353 749Q353 746 303 512T200 27T141 -250H-31Q-31 -240 -15 -176V-174H70L250 674H208L165 675L181 750H267Q353 750 353 749",
      94: "190 527L360 694H434L484 611Q533 528 533 527H457L390 632L385 639L266 527H190",
      95: "59 -75L66 -38H316Q565 -38 565 -39T558 -75T549 -112Q549 -114 299 -114Q50 -114 50 -113L52 -108Q53 -103 55 -93T59 -75",
      126: "330 327Q356 326 388 298T446 269Q470 269 484 327H522Q560 327 560 325L557 316Q554 306 549 292T535 263T512 232T480 208Q453 193 429 193T370 222T315 251Q285 251 275 193H199V197Q214 257 251 292T330 327",
      305: "168 442T168 443T213 444T258 443T212 225T164 2V0H74V2Q75 7 121 224",
      567: "-54 -96L-48 -104Q-41 -111 -27 -118T7 -126Q60 -126 82 -87Q85 -81 140 181L196 443Q196 444 241 444T286 443Q286 441 232 186T175 -75Q163 -120 122 -162T19 -204Q-13 -204 -41 -196T-83 -180T-96 -170Q-55 -96 -54 -96",
      768: "-262 681L-270 694H-177L-132 612Q-89 530 -87 528Q-87 527 -125 527H-163L-208 598Q-254 670 -262 681",
      769: "-96 625L-29 694H63Q42 673 -31 605L-114 527H-190L-176 541Q-160 559 -96 625",
      770: "-310 527L-140 694H-66L-16 611Q33 528 33 527H-43L-110 632L-115 639L-234 527H-310",
      771: "-170 677Q-144 676 -112 648T-54 619Q-30 619 -16 677H22Q60 677 60 675L57 666Q54 656 49 642T35 613T12 582T-20 558Q-47 543 -71 543T-130 572T-185 601Q-215 601 -225 543H-301V547Q-286 607 -249 642T-170 677",
      772: "-314 553L-297 631H-116Q64 631 64 630Q60 612 56 591L47 553L-133 552Q-314 552 -314 553",
      774: "-142 508Q-205 508 -244 548T-284 652Q-284 666 -281 683L-280 694H-204Q-205 689 -205 677Q-205 650 -196 631T-173 604T-147 593T-125 590Q-85 590 -50 618T-5 686L-2 694H73V690Q53 610 -10 559T-142 508",
      775: "-180 578Q-179 579 -170 627T-158 678V680H-54V678Q-56 675 -65 627T-76 578V576H-180V578",
      776: "-273 584Q-272 585 -262 632L-252 678V680H-154V678L-164 632Q-174 585 -175 584Q-175 582 -224 582T-273 584ZM-78 586Q-78 587 -69 632T-58 678V680H40L39 677Q39 676 38 670T34 651T29 628L19 583L-30 582H-79L-78 586",
      778: "-227 597Q-227 639 -186 666T-102 693H-97Q-29 693 -8 649Q-2 637 -2 623Q-2 582 -43 555T-132 527Q-171 527 -199 546T-227 597ZM-59 619Q-59 635 -68 643T-104 652Q-142 652 -156 636T-171 602Q-171 569 -123 569Q-119 569 -111 570T-99 571Q-59 582 -59 619",
      779: "-236 619L-195 694H-149Q-103 694 -103 693L-211 527H-287L-282 536Q-281 539 -236 619ZM-70 619L-29 694H17Q63 694 63 693L-45 527H-121L-116 536Q-115 539 -70 619",
      780: "-283 654H-207L-140 549L-135 542L-16 654H60L-109 487H-147L-184 488L-234 570Q-283 653 -283 654",
      913: "28 0L429 694H533L585 350Q596 275 610 182T632 46L638 3V0H530L528 18Q527 25 515 103T503 183H223L135 29L118 1L73 0H28ZM492 254Q492 256 473 398T454 589V610Q433 552 290 301L264 255L378 254H492",
      914: "501 363Q557 355 605 316T653 222Q653 148 586 85T403 2Q394 1 240 0Q90 0 90 1L100 46Q109 90 128 177T164 348L238 694H375Q518 693 546 688Q614 674 655 635T696 544Q696 490 648 441T516 368L501 363ZM601 530Q601 568 566 590T479 621Q472 622 394 623H320L297 513Q292 489 286 459T276 415L273 401V399H339H372Q504 399 571 466Q601 498 601 530ZM257 322Q256 320 230 197T203 73Q203 71 289 71Q379 72 387 73Q459 84 507 122T556 210Q556 255 519 283T428 320Q415 322 336 323Q257 323 257 322",
      915: "87 2Q88 4 160 346T234 689Q234 691 440 691T646 689Q643 686 629 611H475L321 612Q193 4 191 2V0H87V2",
      916: "273 343L510 694H617Q790 2 790 0H416L42 1L273 343ZM539 576Q536 597 536 600Q536 602 535 605Q534 607 534 607Q527 580 222 130L201 98H651L648 110Q645 123 639 149T627 198Q554 489 539 576",
      917: "86 2Q88 4 160 346T233 689Q233 691 461 691Q688 691 688 689Q685 686 671 611H495L320 612L319 609Q319 607 297 501L274 397H436Q597 397 597 396L596 391Q595 386 593 376T589 358L581 322L420 321Q258 321 258 320Q209 89 208 87Q208 85 390 85Q417 85 460 85T518 86L572 85Q556 8 554 2V0H86V2",
      918: "67 54Q551 615 551 617Q543 618 517 618Q510 618 463 618T376 617Q200 617 200 618T209 657L216 694H459Q702 694 702 692Q702 689 697 667L692 643L207 80H392Q493 81 577 81Q577 70 560 2V0H55V2L67 54",
      919: "517 2Q518 3 551 161T585 322Q586 323 557 323T422 323H259L190 0H138Q86 0 86 1L96 46Q105 90 124 177T160 348L234 694H337V691Q336 690 306 545T275 399H602L603 403Q603 407 634 551L665 694H768V691Q768 690 695 348T621 2V0H517V2",
      920: "119 260Q119 348 157 433T254 579T387 677T533 715Q701 715 772 574Q804 511 804 431Q804 315 744 209T586 41T384 -22Q262 -22 191 59T119 260ZM706 426Q706 524 655 582T525 640Q454 640 395 600T293 502Q256 447 237 383T218 266Q218 168 269 112T401 55Q518 55 612 166T706 426ZM283 349L293 397H473Q652 397 652 396Q647 374 642 347L632 299H452Q273 299 273 300Q278 322 283 349",
      921: "161 348L235 694H338V691Q338 690 265 348T191 2V0H139Q87 0 87 1L96 46Q106 90 125 177T161 348",
      922: "236 223Q235 222 213 113T188 2V0H138Q88 0 88 1L98 46Q107 90 126 177T162 348L236 694H285Q335 694 335 693L330 671Q326 649 316 603T298 518Q289 477 280 433T266 366L261 343L672 694H729L784 693L465 420L651 0H596L541 1L384 350Q383 351 310 288T236 223",
      923: "28 0L401 694H504V690Q505 686 543 345T582 1Q582 0 528 0H473V3Q472 6 460 113T435 359T422 558Q422 593 424 603L425 610L424 608Q414 572 343 431Q287 316 143 49L117 1L73 0H28",
      924: "375 691Q456 215 459 124V106Q488 177 762 641L793 694H929V691Q929 690 856 348T782 2V0H689V2Q691 4 753 304Q817 604 818 606Q819 611 817 608Q817 607 815 603Q798 559 540 117L484 22H440L397 23L393 42Q393 47 373 169T334 422T315 594V609L250 306Q186 3 185 2Q185 0 138 0Q92 0 92 1L102 46Q111 90 130 177T166 348L240 694H375V691",
      925: "311 609Q310 608 246 306T181 2V0H134Q88 0 88 1L98 46Q107 90 126 177T162 348L236 694H382L383 691Q383 688 418 561T493 286T541 97L544 84L545 89Q545 90 553 128T578 246T610 394L674 694H766V691Q766 690 693 348T619 2V0H472L469 13Q468 17 393 293T312 605L311 609",
      926: "193 687Q193 688 479 688H765V686Q764 685 755 642L747 600H461L175 601Q175 602 184 645L193 687ZM196 400Q196 401 418 401T640 400L622 315Q622 314 400 314T178 315L196 400ZM42 2Q43 3 51 44T60 87H64Q68 87 75 87T93 87T119 87T151 88T190 88T237 88T291 88T352 88H643Q638 66 634 44T627 13T624 2V0H42V2",
      927: "118 254Q118 366 174 473T324 648T517 716Q627 716 695 638T763 435Q763 321 706 215T555 43T362 -22Q256 -22 187 56T118 254ZM380 58Q452 58 518 116T622 263T661 442Q661 496 646 535T608 594T567 622T534 634Q516 636 496 636Q400 636 313 528T225 264Q225 172 267 115T380 58",
      928: "86 2Q88 4 160 346T233 689Q233 691 501 691Q768 691 768 689Q766 688 694 346T621 2V0H517V2Q518 3 582 304T646 609L648 615H321L190 0H86V2",
      929: "162 348L236 694H378Q522 693 530 692Q604 680 647 635T690 524Q690 474 665 430T612 359Q550 299 465 280Q443 275 343 274H250V271Q250 269 235 201T206 68T192 2V0H140Q88 0 88 1L98 46Q107 90 126 177T162 348ZM594 513Q594 560 562 588T477 622Q470 623 394 623H321L293 487L263 349V347H342H347H375Q530 347 578 449Q594 483 594 513",
      931: "194 655L202 694H508Q813 694 813 693Q809 675 805 653L797 614H559L321 615Q327 606 405 478L485 347Q449 311 348 203T247 86Q247 84 294 84Q303 84 359 84T465 85H684Q684 84 675 42L666 0H360L55 1L195 154Q346 319 347 320L359 333L273 473Q187 614 186 614L187 620Q188 625 190 635T194 655",
      932: "165 608L182 687Q182 688 486 688H790L789 685L781 645L773 609H521L457 306Q393 3 392 2Q392 0 340 0H288V2Q289 5 353 304T417 605V609L291 608H165",
      933: "357 637Q320 637 297 612T266 555H173Q178 576 188 598Q214 651 265 683T373 716Q497 716 497 542V509L504 526Q579 715 711 715Q773 715 808 677T843 589Q843 576 840 555H747L748 557Q748 559 748 563T749 574V580Q749 604 731 622Q715 638 693 638Q591 638 543 465Q531 425 506 309T462 98T441 2V0H337V2Q425 401 436 486Q438 504 438 526Q438 637 364 637H357",
      934: "124 308Q124 399 208 481T433 587Q437 587 437 589Q438 590 449 643L459 694H508Q557 694 557 693Q557 691 546 641T535 587Q543 587 562 583T614 565T674 531T722 472T743 387Q743 288 656 209T449 110L433 106Q411 3 410 2Q410 0 361 0H312L313 3Q313 5 324 56L335 107H331L321 108Q311 110 297 114T266 124T228 141T190 168Q124 225 124 308ZM227 315Q227 282 239 257T270 218T306 197T338 186L350 184H351L386 346Q420 507 420 509H419Q411 509 393 505T342 485T284 444Q227 387 227 315ZM642 381Q642 413 629 437T599 475T563 496T533 507T519 510Q518 510 484 348T450 184Q544 201 593 258T642 381",
      935: "14 0Q17 3 184 184T352 367L265 529Q244 567 222 609T188 672L176 692Q176 694 236 694H297L338 612Q387 515 400 489L421 448L645 694H758L708 640Q481 393 456 368Q455 366 500 281T596 104T652 0H531L388 293L128 0H14",
      936: "325 556Q325 524 310 447T294 330Q294 289 304 260Q314 234 333 216T364 192T380 187L488 694H585V691Q584 689 531 438L478 188H479Q485 188 503 195T555 231T613 305Q637 352 654 435Q662 470 669 496T681 538T690 562T698 578T704 587Q719 609 733 615T772 621H802H854V619L838 546Q838 545 832 545Q775 539 749 418Q716 274 638 196Q616 173 590 156T543 131T503 117T473 110T460 106Q460 105 450 54T438 2V0H340V2Q341 3 351 54T362 106Q363 107 358 108T344 111T322 117T295 128T267 145T239 171T216 207T200 256T194 319Q194 356 203 408T213 483Q213 517 203 530T182 544T171 546Q184 609 187 619V621H239Q286 621 294 620T309 612Q325 596 325 556",
      937: "148 407Q148 475 182 534T269 633T386 694T511 716Q622 716 695 658T769 507Q769 461 747 409T699 321T628 225T562 136Q533 90 533 86Q542 85 557 85Q564 85 583 85T614 86Q695 86 695 85Q679 9 677 2V0H425Q426 3 433 30T447 72T480 131T549 241L554 248Q558 254 563 261T576 281T592 306T609 335T625 366T640 400T653 434T661 466T664 498Q664 562 618 601T497 640Q416 640 351 596T262 482Q250 441 250 392T276 237T302 70V56Q302 28 296 2V0H44V2L62 85Q62 86 143 86L225 85V88Q224 89 224 89T224 91T224 95T223 101T222 110T220 123T216 140T209 163T200 191T188 227Q148 344 148 407",
      8211: "59 275L66 312H316Q565 312 565 311T558 275T549 238Q549 236 299 236Q50 236 50 237L52 242Q53 247 55 257T59 275",
      8212: "59 275L66 312H566Q1065 312 1065 311T1058 275T1049 238Q1049 236 549 236Q50 236 50 237L52 242Q53 247 55 257T59 275",
      8213: "59 275L66 312H566Q1065 312 1065 311T1058 275T1049 238Q1049 236 549 236Q50 236 50 237L52 242Q53 247 55 257T59 275",
      8215: "59 -75L66 -38H316Q565 -38 565 -39T558 -75T549 -112Q549 -114 299 -114Q50 -114 50 -113L52 -108Q53 -103 55 -93T59 -75",
      8216: "309 567L299 520Q289 474 288 473Q288 471 239 471T190 473L192 480Q193 486 196 499T201 522L211 569L287 694H312L335 693L332 685Q328 677 321 661T307 630L279 570L294 569Q309 569 309 567",
      8217: "228 647L237 694H286Q335 694 335 693L334 686Q332 680 330 668T325 644L315 597L239 471H214Q190 471 190 472L246 595Q246 596 231 596H217V598Q217 601 228 647",
      8220: "393 567L383 520Q373 474 372 473Q372 471 323 471T274 473L276 480Q277 486 280 499T285 522L295 569L371 694H396L419 693L416 685Q412 677 405 661T391 630L363 570L378 569Q393 569 393 567ZM587 567L577 520Q567 474 566 473Q566 471 517 471T468 473L470 480Q471 486 474 499T479 522L489 569L565 694H590L613 693L610 685Q606 677 599 661T585 630L557 570L572 569Q587 569 587 567",
      8221: "171 647L180 694H229Q278 694 278 693L276 686Q275 680 273 668T268 644L258 597L182 471H157Q133 471 133 472L189 595Q189 596 174 596H160V598Q160 601 171 647ZM365 647L374 694H423Q472 694 472 693L470 686Q469 680 467 668T462 644L452 597L376 471H351Q327 471 327 472L383 595Q383 596 368 596H354V598Q354 601 365 647",
      8260: "564 744L568 747Q573 750 579 750Q588 750 594 744T599 729Q597 721 321 241T41 -243Q37 -250 27 -250Q6 -250 6 -230Q6 -228 8 -222Q9 -219 285 261T564 744",
      8710: "273 343L510 694H617Q790 2 790 0H416L42 1L273 343ZM539 576Q536 597 536 600Q536 602 535 605Q534 607 534 607Q527 580 222 130L201 98H651L648 110Q645 123 639 149T627 198Q554 489 539 576"
    }, {}), sansSerifItalic$1;
  }
  var sansSerif$1 = {}, sansSerif = {}, hasRequiredSansSerif$1;
  function requireSansSerif$1() {
    return hasRequiredSansSerif$1 || (hasRequiredSansSerif$1 = 1, Object.defineProperty(sansSerif, "__esModule", { value: !0 }), sansSerif.sansSerif = void 0, sansSerif.sansSerif = {
      33: [0.694, 0, 0.319],
      34: [0.694, -0.471, 0.5],
      35: [0.694, 0.194, 0.833],
      36: [0.75, 0.056, 0.5],
      37: [0.75, 0.056, 0.833],
      38: [0.716, 0.022, 0.758],
      39: [0.694, -0.471, 0.278],
      40: [0.75, 0.25, 0.389],
      41: [0.75, 0.25, 0.389],
      42: [0.75, -0.306, 0.5],
      43: [0.583, 0.082, 0.778],
      44: [0.098, 0.125, 0.278],
      45: [0.259, -0.186, 0.333],
      46: [0.098, 0, 0.278],
      47: [0.75, 0.25, 0.5],
      58: [0.444, 0, 0.278],
      59: [0.444, 0.125, 0.278],
      61: [0.37, -0.13, 0.778],
      63: [0.704, 0, 0.472],
      64: [0.704, 0.011, 0.667],
      91: [0.75, 0.25, 0.289],
      93: [0.75, 0.25, 0.289],
      94: [0.694, -0.527, 0.5],
      95: [-0.038, 0.114, 0.5],
      126: [0.327, -0.193, 0.5],
      305: [0.444, 0, 0.239],
      567: [0.444, 0.205, 0.267],
      768: [0.694, -0.527, 0],
      769: [0.694, -0.527, 0],
      770: [0.694, -0.527, 0],
      771: [0.677, -0.543, 0],
      772: [0.631, -0.552, 0],
      774: [0.694, -0.508, 0],
      775: [0.68, -0.576, 0],
      776: [0.68, -0.582, 0],
      778: [0.694, -0.527, 0],
      779: [0.694, -0.527, 0],
      780: [0.654, -0.487, 0],
      913: [0.694, 0, 0.667],
      914: [0.694, 0, 0.667],
      915: [0.691, 0, 0.542],
      916: [0.694, 0, 0.833],
      917: [0.691, 0, 0.597],
      918: [0.694, 0, 0.611],
      919: [0.694, 0, 0.708],
      920: [0.716, 0.021, 0.778],
      921: [0.694, 0, 0.278],
      922: [0.694, 0, 0.694],
      923: [0.694, 0, 0.611],
      924: [0.694, 0, 0.875],
      925: [0.694, 0, 0.708],
      926: [0.688, 0, 0.667],
      927: [0.715, 0.022, 0.736],
      928: [0.691, 0, 0.708],
      929: [0.694, 0, 0.639],
      931: [0.694, 0, 0.722],
      932: [0.688, 0, 0.681],
      933: [0.716, 0, 0.778],
      934: [0.694, 0, 0.722],
      935: [0.694, 0, 0.667],
      936: [0.694, 0, 0.778],
      937: [0.716, 0, 0.722],
      8211: [0.312, -0.236, 0.5],
      8212: [0.312, -0.236, 1],
      8213: [0.312, -0.236, 1],
      8215: [-0.038, 0.114, 0.5],
      8216: [0.694, -0.471, 0.278],
      8217: [0.694, -0.471, 0.278],
      8220: [0.694, -0.471, 0.5],
      8221: [0.694, -0.471, 0.5],
      8260: [0.75, 0.25, 0.5],
      8710: [0.694, 0, 0.833]
    }), sansSerif;
  }
  var hasRequiredSansSerif;
  function requireSansSerif() {
    if (hasRequiredSansSerif) return sansSerif$1;
    hasRequiredSansSerif = 1, Object.defineProperty(sansSerif$1, "__esModule", { value: !0 }), sansSerif$1.sansSerif = void 0;
    var O = requireFontData(), _ = requireSansSerif$1();
    return sansSerif$1.sansSerif = (0, O.AddPaths)(_.sansSerif, {
      33: "120 187Q120 225 115 440T110 693Q110 694 159 694T208 693Q208 655 203 440T198 187Q198 186 159 186T120 187ZM110 0V98H208V0H110",
      34: "33 596V694H131V597L82 471H32L47 532Q62 593 63 594Q63 596 48 596H33ZM227 596V694H325V597L276 471H226L241 532Q256 593 257 594Q257 596 242 596H227",
      35: "56 350Q56 363 70 370H192Q314 370 314 371L356 526Q396 676 401 685Q406 694 416 694Q423 694 429 689T436 677Q436 673 396 523T355 371Q355 370 449 370Q544 370 544 371L586 526Q628 682 630 685Q636 694 646 694Q653 694 659 689T665 678Q665 670 645 593T605 444L585 371Q585 370 673 370H762Q777 362 777 350Q777 337 767 334T723 330H668H573L567 305Q560 282 545 226L530 171L646 170H721Q756 170 766 167T777 150Q777 138 762 130H640Q518 130 518 129L476 -26Q434 -182 432 -185Q426 -194 416 -194Q409 -194 403 -189T397 -178Q397 -170 417 -93T457 56L477 129Q477 130 383 130Q288 130 288 129L246 -26Q204 -182 202 -185Q196 -194 186 -194Q179 -194 173 -189T167 -178Q167 -170 187 -94T227 56L247 129Q247 130 159 130H70Q56 137 56 150Q56 165 72 170H259L265 195Q273 222 287 275L302 330H186L70 331Q63 334 58 339Q56 343 56 350ZM489 170L532 330H343L337 305Q330 282 315 226L300 171L394 170H489",
      36: "55 509Q55 585 103 638T213 701V750H286V703H289Q312 703 354 689Q372 682 399 666T427 646L413 569Q413 568 403 575Q352 615 291 624H286V405Q357 389 400 331T444 199Q444 128 402 69T286 -7V-56H213V-9Q167 -3 125 14T63 44T44 60Q44 61 52 101L59 140L69 132Q78 125 87 119T107 107T124 97T141 90T157 84T173 80T187 76T201 73T213 70V317L202 319Q141 335 98 386T55 509ZM213 424V620L203 615Q143 587 143 522Q143 455 213 424ZM356 187Q356 208 350 227T334 258T315 280T298 293T287 297Q286 297 286 186V75Q356 110 356 187",
      37: "56 549Q56 639 98 694T195 750Q248 750 290 694T332 548Q332 455 290 401T195 347Q141 347 99 403T56 549ZM248 549Q248 602 234 638T195 674Q145 674 145 549Q145 423 195 423Q219 423 233 459T248 549ZM197 -56Q187 -56 182 -49T176 -35Q176 -33 178 -27Q179 -25 399 356T623 741Q626 750 639 750Q648 750 654 744T659 729Q657 721 435 336T207 -52Q203 -56 197 -56ZM500 146Q500 235 542 291T639 347Q692 347 734 293T776 146Q776 53 733 -1T639 -56Q587 -56 544 -2T500 146ZM692 146Q692 199 678 235T639 271Q589 271 589 146Q589 20 639 20Q663 20 677 56T692 146",
      38: "156 502Q156 600 198 658T302 716Q367 716 405 665T444 549Q444 531 442 523Q426 446 304 348L287 334Q305 297 340 249T402 170T430 139T443 149T472 181T509 231T549 303T583 394Q584 398 586 404Q587 408 587 409Q588 409 626 399T665 388Q663 381 660 369T644 322T614 253T567 176T502 98L491 87Q534 57 584 57Q653 57 700 75Q702 75 702 34T700 -7Q652 -22 586 -22H580Q505 -22 434 26L421 34Q419 33 405 25T374 11T336 -4T285 -17T226 -22Q143 -22 93 31T42 152Q42 184 51 211T81 260T111 291T144 317L184 348L178 365Q156 430 156 502ZM359 552Q359 588 345 614T302 640Q292 640 282 636T260 622T241 590T233 535Q236 474 253 417L257 407L271 419Q312 454 330 479Q359 514 359 552ZM345 102Q262 190 216 277Q215 277 204 267T180 247T165 236Q135 208 135 159Q135 123 152 97T198 61Q207 58 227 58Q286 58 357 89L345 102",
      39: "90 596V694H188V597L139 471H89L104 532Q119 593 120 594Q120 596 105 596H90",
      40: "74 250Q74 564 240 733L257 750H333L323 739Q167 573 167 250T323 -239L333 -250H257L240 -233Q74 -63 74 250",
      41: "221 -73T221 250T65 739L55 750H131L148 733Q314 567 314 250T148 -233L131 -250H55L65 -239Q221 -73 221 250",
      42: "208 717Q211 731 222 740T250 750Q265 750 277 741T291 717Q291 706 290 675T286 617L285 591L329 622Q369 651 376 655T393 659Q411 659 423 647T436 616Q436 609 434 603T429 594T419 585T407 577T389 567T368 556L316 528L368 500Q421 472 429 464Q436 455 436 440Q436 422 423 409T393 396Q390 396 388 396T384 397T380 398T375 401T367 406T358 413T346 422T329 434L285 465Q284 465 285 438T289 381T291 347Q291 327 278 317T250 306Q234 306 222 315T208 339Q208 350 209 381T212 439L214 465L170 434Q130 405 123 401T106 397Q88 397 76 409T63 440Q63 447 65 453T70 462T80 471T92 479T110 489T131 500L183 528L131 556Q78 584 70 592Q63 601 63 616Q63 634 76 647T106 660Q109 660 111 660T115 659T119 658T124 655T132 650T141 643T153 634T170 622L214 591L212 617Q211 643 210 674T208 717",
      43: "56 237T56 250T70 270H369V420L370 570Q380 583 389 583Q402 583 409 568V270H707Q722 262 722 250T707 230H409V-68Q401 -82 391 -82H389H387Q375 -82 369 -68V230H70Q56 237 56 250",
      44: "90 0V98H188V1L139 -125H89L104 -64Q119 -3 120 -2Q120 0 105 0H90",
      45: "11 186V259H277V186H11",
      46: "90 0V98H188V0H90",
      47: "423 750Q432 750 438 744T444 730Q444 725 271 248T92 -240Q85 -250 75 -250Q68 -250 62 -245T56 -231Q56 -221 230 257T407 740Q411 750 423 750",
      58: "90 346V444H188V346H90ZM90 0V98H188V0H90",
      59: "90 346V444H188V346H90ZM90 0V98H188V1L139 -125H89L104 -64Q119 -3 120 -2Q120 0 105 0H90",
      61: "56 350Q56 363 70 370H707Q722 362 722 350Q722 339 708 331L390 330H72Q56 335 56 350ZM56 150Q56 165 72 170H708Q722 160 722 150Q722 138 707 130H70Q56 137 56 150",
      63: "55 652Q63 658 77 666T132 689T214 704T265 703Q303 698 332 684T376 653T401 615T413 578T416 546Q416 475 360 426Q325 396 303 365T273 303T263 256T261 216V186H183V225Q184 281 194 322Q206 368 232 409T276 469T306 495Q323 517 323 550Q323 596 297 612T223 629Q187 629 157 618Q122 607 91 583L83 576L55 652ZM173 0V98H271V0H173",
      64: "422 576Q440 576 456 573T480 566L488 563Q488 565 484 571T472 588T452 607T424 622T387 629Q294 629 220 550T146 347Q146 233 210 155T365 66Q376 65 400 65Q465 68 517 86L532 91H612L598 76Q511 -11 388 -11Q250 -11 153 93T56 347Q56 454 107 538T231 663T378 704Q446 704 492 680T564 608T599 506T611 376Q611 320 607 299Q595 227 546 173T422 118Q343 118 288 185T232 347Q232 444 287 510T422 576ZM520 347Q520 429 487 465T421 501Q385 501 354 461T323 347Q323 270 355 232T422 193Q454 193 487 229T520 347",
      91: "94 -250V750H266V674H181V-174H266V-250H94",
      93: "22 674V750H194V-250H22V-174H107V674H22",
      94: "146 612L212 694H287L353 612Q417 532 420 529Q421 528 418 527Q414 527 383 527H345L250 639Q244 633 199 580L154 527H116Q79 528 79 529Q82 532 146 612",
      95: "0 -114V-38H499V-114H0",
      126: "83 204Q86 254 113 290T186 327Q211 327 251 299T312 270Q337 270 340 315V327H416V316Q413 258 382 226T315 193Q289 193 249 221T187 250Q162 250 159 205V193H83V204",
      305: "74 0V444H164V0H74",
      567: "-35 -95Q-4 -126 34 -126Q58 -126 76 -116T100 -88Q102 -82 102 181V444H192V180Q191 -45 191 -70T184 -113Q171 -152 140 -178T63 -205Q34 -205 4 -197T-43 -181T-59 -171L-47 -133L-35 -95",
      768: "-415 692L-417 694H-324L-262 612Q-249 594 -233 572T-208 539L-200 528L-237 527H-275L-344 608Q-359 625 -378 647T-406 680T-415 692",
      769: "-239 612L-177 694H-84L-86 692Q-86 691 -95 681T-123 648T-157 608L-226 527H-264L-301 528L-293 539Q-285 550 -269 572T-239 612",
      770: "-354 612L-288 694H-213L-147 612Q-83 532 -80 529Q-79 528 -82 527Q-86 527 -117 527H-155L-250 639Q-256 633 -301 580L-346 527H-384Q-421 528 -421 529Q-418 532 -354 612",
      771: "-417 554Q-414 604 -387 640T-314 677Q-289 677 -249 649T-188 620Q-163 620 -160 665V677H-84V666Q-87 608 -118 576T-185 543Q-211 543 -251 571T-313 600Q-338 600 -341 555V543H-417V554",
      772: "-431 552V631H-70V552H-431",
      774: "-250 508Q-331 508 -379 567T-427 689V694H-351V685Q-348 649 -321 620T-250 591Q-206 591 -180 619T-150 685V694H-74V689Q-74 624 -122 566T-250 508",
      775: "-302 576V680H-198V576H-302",
      776: "-397 582V680H-299V582H-397ZM-202 582V680H-104V582H-202",
      778: "-319 611Q-319 649 -285 671T-211 694Q-164 694 -132 671T-99 611Q-99 572 -133 550T-209 527T-285 549T-319 611ZM-155 610Q-155 635 -171 643T-215 651Q-263 651 -263 610Q-263 570 -211 570H-209H-207Q-155 570 -155 610",
      779: "-250 693Q-317 544 -323 527H-399L-343 694H-296Q-250 694 -250 693ZM-84 693Q-151 544 -157 527H-233L-177 694H-130Q-84 694 -84 693",
      780: "-421 652Q-422 653 -419 654Q-415 654 -384 654H-346L-301 601Q-287 585 -275 571T-258 551T-250 542L-155 654H-117Q-80 653 -80 652Q-83 649 -147 569L-213 487H-288L-354 569Q-418 649 -421 652",
      913: "183 181Q183 179 152 91T118 0H28L154 346L280 693Q281 694 333 694H385L511 349Q636 4 638 2Q638 0 584 0H530L464 183H184L183 181ZM324 606Q319 578 292 492T238 332T210 256Q210 254 324 254T438 255L429 281L419 308Q409 336 395 378T365 465T339 551T324 611V606",
      914: "425 363Q438 363 465 353T526 324T585 270T610 192Q610 132 561 78T426 7Q404 2 387 2T240 0H90V694H227Q373 693 396 689Q484 673 533 623T583 517Q583 494 574 473T551 437T520 409T487 388T456 374T433 366L425 363ZM490 516Q490 527 485 539T467 568T423 599T347 621Q340 622 262 623H188V399H261H286Q432 399 478 475Q490 496 490 516ZM514 190Q514 245 462 280T343 322Q336 323 259 323H188V71H274Q365 72 388 77Q445 88 479 121T514 190",
      915: "87 0V691H499V611H345L191 612V0H87",
      916: "203 348L362 694H470L629 348Q789 2 790 1Q790 0 416 0T42 1Q43 2 203 348ZM630 98Q630 100 584 198T481 422T407 603L405 610L403 600Q388 544 191 122L180 99L405 98H630",
      917: "86 0V691H541V611H366L190 612V397H513V321H190V85H372L554 86V0H86",
      918: "69 617V694H555V643L373 362Q190 81 190 79H234Q244 79 272 79T344 80T419 81H560V0H55V53L237 334Q420 615 420 617Q413 618 387 618Q380 618 334 618T245 617H69",
      919: "86 0V694H190V399H517V694H621V0H517V323H190V0H86",
      920: "56 344Q56 430 86 502T164 619T271 690T388 716Q448 716 506 691T613 619T692 501T722 344Q722 188 624 84T389 -21Q252 -21 154 83T56 344ZM624 345Q624 423 597 488T513 596T380 639Q343 639 305 621T232 568T175 475T153 344Q153 216 222 136T388 56Q487 56 555 138T624 345ZM209 299V397H568V299H209",
      921: "87 0V694H191V0H87",
      922: "88 0V694H188V519L189 343L525 694H638L375 419L651 0H541L309 351L188 225V0H88",
      923: "294 606Q294 574 252 430T163 144T117 0H72Q28 0 28 1T141 348L254 694H357L469 348Q582 2 582 1T527 0L473 1L469 11Q469 13 427 141T343 411T296 599L294 610V606",
      924: "92 0V694H228L233 680Q236 675 284 547T382 275T436 106Q446 149 497 292T594 558L640 680L645 694H782V0H689V305L688 606Q688 577 500 78L479 23H392L364 96Q364 97 342 156T296 280T246 418T203 544T186 609V588Q185 568 185 517T185 427T185 305V0H92",
      925: "88 0V694H235L252 659Q261 639 364 428T526 84V694H619V0H472L455 35Q453 39 330 294T185 601L181 611V0H88",
      926: "47 600V688H619V600H47ZM111 314V401H555V314H111ZM42 0V88H624V0H42",
      927: "55 345Q55 504 149 609T361 715Q386 715 406 713Q521 696 600 592T680 344Q680 193 590 86T368 -22Q239 -22 147 84T55 345ZM276 59T368 59T518 146T576 360Q576 473 525 545T401 634Q371 637 362 637Q284 637 222 562T159 360T217 147",
      928: "86 0V691H621V0H517V615H190V0H86",
      929: "88 0V694H230Q347 693 370 692T410 686Q487 667 535 611T583 485Q583 409 527 348T379 276Q369 274 279 274H192V0H88ZM486 485Q486 523 471 551T432 593T391 612T357 621Q350 622 268 623H189V347H268Q350 348 357 349Q370 351 383 354T416 368T450 391T475 429T486 485",
      931: "55 0Q56 3 171 167T288 332Q288 334 172 474L55 615V694H666V614H428L190 615L412 347L322 218Q236 97 228 84L447 85H666V0H55",
      932: "36 608V688H644V608H518L392 609V0H288V609L162 608H36",
      933: "55 565Q59 625 105 670T219 716H222Q310 716 353 627Q376 583 386 524L389 510L393 532Q397 555 407 584T433 644T482 695T557 716Q621 716 669 673T722 565V555H629V563Q627 592 607 615T557 638Q530 638 511 629T479 598T459 553T447 488T442 411T441 319V202V0H337V202Q337 453 331 497Q313 634 226 638Q185 638 167 612T148 563V555H55V565",
      934: "666 347Q666 326 661 302T638 247T594 190T520 140T413 107H410V0H312V54Q312 107 311 107Q286 107 229 128T125 192Q55 260 55 347Q55 396 77 438T131 507T200 552T265 579T311 587Q312 587 312 641V694H410V587H413Q476 576 524 552T598 502T640 444T661 390T666 347ZM310 510Q305 510 291 507T252 492T208 464T172 416T157 347T171 279T204 233T247 205T286 190T310 184H312V347Q312 510 310 510ZM564 347Q564 385 549 416T514 463T470 491T433 505T414 509L410 510V184Q413 184 426 187T464 200T510 227T548 275T564 347",
      935: "14 0Q16 5 144 184T275 367L153 528Q121 571 88 615T42 674T28 694H150L228 584Q315 463 316 461L326 448L497 694H610L609 692Q606 689 492 528Q440 454 409 410T378 366Q378 365 515 182L652 0H531L326 292Q326 293 299 254T226 146L128 0H14",
      936: "340 187V694H438V187Q481 206 495 219Q518 239 533 267T553 332T560 386T562 435Q562 576 593 608Q608 621 637 621H670H722V545H719Q718 545 715 545T710 544Q679 536 666 487Q664 474 662 429T654 344T633 259T580 175T486 119Q450 109 438 108V0H340V108L326 110Q122 149 117 415Q116 460 111 487Q98 536 67 544Q65 544 62 544T58 545H55V621H107Q160 621 163 620Q191 613 202 573Q213 536 213 473T220 351T256 249Q262 239 270 230T285 216T301 205T316 197T329 192T340 187",
      937: "55 462Q55 561 141 638T359 716Q492 716 579 640T666 462Q666 407 642 347T579 222T529 126Q515 91 515 86Q517 85 528 85Q530 85 552 85T596 86H677V0H425V14Q429 79 465 168L492 228Q494 232 504 254T516 283T527 310T539 340T548 368T556 399T560 428T562 460Q562 531 510 585T361 639Q263 639 211 585T159 460Q159 422 170 378T192 309T229 228L256 168Q292 79 296 14V0H44V86H125Q146 86 168 86T193 85L206 86Q206 103 183 148T131 241T79 352T55 462",
      8211: "0 236V312H499V236H0",
      8212: "0 236V312H999V236H0",
      8213: "0 236V312H999V236H0",
      8215: "0 -114V-38H499V-114H0",
      8216: "90 568L140 694H189L174 633Q159 572 158 571Q158 569 173 569H188V471H90V568",
      8217: "90 596V694H188V597L139 471H89L104 532Q119 593 120 594Q120 596 105 596H90",
      8220: "174 568L224 694H273L258 633Q243 572 242 571Q242 569 257 569H272V471H174V568ZM368 568L418 694H467L452 633Q437 572 436 571Q436 569 451 569H466V471H368V568",
      8221: "33 596V694H131V597L82 471H32L47 532Q62 593 63 594Q63 596 48 596H33ZM227 596V694H325V597L276 471H226L241 532Q256 593 257 594Q257 596 242 596H227",
      8260: "423 750Q432 750 438 744T444 730Q444 725 271 248T92 -240Q85 -250 75 -250Q68 -250 62 -245T56 -231Q56 -221 230 257T407 740Q411 750 423 750",
      8710: "203 348L362 694H470L629 348Q789 2 790 1Q790 0 416 0T42 1Q43 2 203 348ZM630 98Q630 100 584 198T481 422T407 603L405 610L403 600Q388 544 191 122L180 99L405 98H630"
    }, {}), sansSerif$1;
  }
  var scriptBold$1 = {}, scriptBold = {}, hasRequiredScriptBold$1;
  function requireScriptBold$1() {
    return hasRequiredScriptBold$1 || (hasRequiredScriptBold$1 = 1, Object.defineProperty(scriptBold, "__esModule", { value: !0 }), scriptBold.scriptBold = void 0, scriptBold.scriptBold = {}), scriptBold;
  }
  var hasRequiredScriptBold;
  function requireScriptBold() {
    return hasRequiredScriptBold || (hasRequiredScriptBold = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.scriptBold = void 0;
      var _ = requireScriptBold$1();
      Object.defineProperty(O, "scriptBold", { enumerable: !0, get: function() {
        return _.scriptBold;
      } });
    }(scriptBold$1)), scriptBold$1;
  }
  var script$1 = {}, script = {}, hasRequiredScript$1;
  function requireScript$1() {
    return hasRequiredScript$1 || (hasRequiredScript$1 = 1, Object.defineProperty(script, "__esModule", { value: !0 }), script.script = void 0, script.script = {}), script;
  }
  var hasRequiredScript;
  function requireScript() {
    return hasRequiredScript || (hasRequiredScript = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.script = void 0;
      var _ = requireScript$1();
      Object.defineProperty(O, "script", { enumerable: !0, get: function() {
        return _.script;
      } });
    }(script$1)), script$1;
  }
  var smallop$1 = {}, smallop = {}, hasRequiredSmallop$1;
  function requireSmallop$1() {
    return hasRequiredSmallop$1 || (hasRequiredSmallop$1 = 1, Object.defineProperty(smallop, "__esModule", { value: !0 }), smallop.smallop = void 0, smallop.smallop = {
      40: [0.85, 0.349, 0.458],
      41: [0.85, 0.349, 0.458],
      47: [0.85, 0.349, 0.578],
      91: [0.85, 0.349, 0.417],
      92: [0.85, 0.349, 0.578],
      93: [0.85, 0.349, 0.417],
      123: [0.85, 0.349, 0.583],
      125: [0.85, 0.349, 0.583],
      710: [0.744, -0.551, 0.556],
      732: [0.722, -0.597, 0.556],
      770: [0.744, -0.551, 0],
      771: [0.722, -0.597, 0],
      8214: [0.602, 0, 0.778],
      8260: [0.85, 0.349, 0.578],
      8593: [0.6, 0, 0.667],
      8595: [0.6, 0, 0.667],
      8657: [0.599, 0, 0.778],
      8659: [0.6, 0, 0.778],
      8719: [0.75, 0.25, 0.944],
      8720: [0.75, 0.25, 0.944],
      8721: [0.75, 0.25, 1.056],
      8730: [0.85, 0.35, 1, { ic: 0.02 }],
      8739: [0.627, 0.015, 0.333],
      8741: [0.627, 0.015, 0.556],
      8747: [0.805, 0.306, 0.472, { ic: 0.138 }],
      8748: [0.805, 0.306, 0.819, { ic: 0.138 }],
      8749: [0.805, 0.306, 1.166, { ic: 0.138 }],
      8750: [0.805, 0.306, 0.472, { ic: 0.138 }],
      8896: [0.75, 0.249, 0.833],
      8897: [0.75, 0.249, 0.833],
      8898: [0.75, 0.249, 0.833],
      8899: [0.75, 0.249, 0.833],
      8968: [0.85, 0.349, 0.472],
      8969: [0.85, 0.349, 0.472],
      8970: [0.85, 0.349, 0.472],
      8971: [0.85, 0.349, 0.472],
      9001: [0.85, 0.35, 0.472],
      9002: [0.85, 0.35, 0.472],
      9168: [0.602, 0, 0.667],
      10072: [0.627, 0.015, 0.333],
      10216: [0.85, 0.35, 0.472],
      10217: [0.85, 0.35, 0.472],
      10752: [0.75, 0.25, 1.111],
      10753: [0.75, 0.25, 1.111],
      10754: [0.75, 0.25, 1.111],
      10756: [0.75, 0.249, 0.833],
      10758: [0.75, 0.249, 0.833],
      10764: [0.805, 0.306, 1.638, { ic: 0.138 }],
      12296: [0.85, 0.35, 0.472],
      12297: [0.85, 0.35, 0.472]
    }), smallop;
  }
  var hasRequiredSmallop;
  function requireSmallop() {
    if (hasRequiredSmallop) return smallop$1;
    hasRequiredSmallop = 1, Object.defineProperty(smallop$1, "__esModule", { value: !0 }), smallop$1.smallop = void 0;
    var O = requireFontData(), _ = requireSmallop$1();
    return smallop$1.smallop = (0, O.AddPaths)(_.smallop, {
      40: "152 251Q152 646 388 850H416Q422 844 422 841Q422 837 403 816T357 753T302 649T255 482T236 250Q236 124 255 19T301 -147T356 -251T403 -315T422 -340Q422 -343 416 -349H388Q359 -325 332 -296T271 -213T212 -97T170 56T152 251",
      41: "305 251Q305 -145 69 -349H56Q43 -349 39 -347T35 -338Q37 -333 60 -307T108 -239T160 -136T204 27T221 250T204 473T160 636T108 740T60 807T35 839Q35 850 50 850H56H69Q197 743 256 566Q305 425 305 251",
      47: "481 838Q489 850 500 850Q508 850 515 844T522 827Q521 824 311 248T96 -337Q90 -349 77 -349Q68 -349 62 -343T55 -326Q56 -323 266 253T481 838",
      91: "202 -349V850H394V810H242V-309H394V-349H202",
      92: "522 -326Q522 -337 515 -343T500 -349Q487 -349 481 -337Q477 -328 267 248T55 827Q54 835 60 842T76 850Q89 850 96 838Q100 829 310 253T522 -326",
      93: "22 810V850H214V-349H22V-309H174V810H22",
      123: "477 -343L471 -349H458Q432 -349 367 -325T273 -263Q258 -245 250 -212L249 -51Q249 -27 249 12Q248 118 244 128Q243 129 243 130Q220 189 121 228Q109 232 107 235T105 250Q105 256 105 257T105 261T107 265T111 268T118 272T128 276T142 283T162 291Q224 324 243 371Q243 372 244 373Q248 384 249 469Q249 475 249 489Q249 528 249 552L250 714Q253 728 256 736T271 761T299 789T347 816T422 843Q440 849 441 849H443Q445 849 447 849T452 850T457 850H471L477 844V830Q477 820 476 817T470 811T459 807T437 801T404 785Q353 760 338 724Q333 710 333 550Q333 526 333 492T334 447Q334 393 327 368T295 318Q257 280 181 255L169 251L184 245Q318 198 332 112Q333 106 333 -49Q333 -209 338 -223Q351 -255 391 -277T469 -309Q477 -311 477 -329V-343",
      125: "110 849L115 850Q120 850 125 850Q151 850 215 826T309 764Q324 747 332 714L333 552Q333 528 333 489Q334 383 338 373Q339 372 339 371Q353 336 391 310T469 271Q477 268 477 251Q477 241 476 237T472 232T456 225T428 214Q357 179 339 130Q339 129 338 128Q334 117 333 32Q333 26 333 12Q333 -27 333 -51L332 -212Q328 -228 323 -240T302 -271T255 -307T175 -338Q139 -349 125 -349T108 -346T105 -329Q105 -314 107 -312T130 -304Q233 -271 248 -209Q249 -203 249 -49V57Q249 106 253 125T273 167Q307 213 398 245L413 251L401 255Q265 300 250 389Q249 395 249 550Q249 710 244 724Q224 774 112 811Q105 813 105 830Q105 845 110 849",
      710: "279 669Q273 669 142 610T9 551L0 569Q-8 585 -8 587Q-8 588 -7 588L12 598Q30 608 66 628T136 666L277 744L564 587L555 569Q549 556 547 554T544 552Q539 555 410 612T279 669",
      732: "374 597Q337 597 269 627T160 658Q101 658 34 606L24 597L12 611Q1 624 1 626Q1 627 27 648T55 671Q120 722 182 722Q219 722 286 692T395 661Q454 661 521 713L531 722L543 708Q554 695 554 693Q554 692 528 671T500 648Q434 597 374 597",
      770: "-277 669Q-283 669 -414 610T-547 551L-556 569Q-564 585 -564 587Q-564 588 -563 588L-544 598Q-526 608 -490 628T-420 666L-279 744L8 587L-1 569Q-7 556 -9 554T-12 552Q-17 555 -146 612T-277 669",
      771: "-182 597Q-219 597 -287 627T-396 658Q-455 658 -522 606L-532 597L-544 611Q-555 624 -555 626Q-555 627 -529 648T-501 671Q-436 722 -374 722Q-337 722 -270 692T-161 661Q-102 661 -35 713L-25 722L-13 708Q-2 695 -2 693Q-2 692 -28 671T-56 648Q-122 597 -182 597",
      8214: "257 0V602H300V0H257ZM478 0V602H521V0H478",
      8260: "481 838Q489 850 500 850Q508 850 515 844T522 827Q521 824 311 248T96 -337Q90 -349 77 -349Q68 -349 62 -343T55 -326Q56 -323 266 253T481 838",
      8593: "112 421L120 424Q127 427 136 430T161 441T191 458T224 481T260 510T295 546T328 591L333 600L340 589Q380 527 431 489T555 421V377L543 381Q445 418 368 492L355 504V0H312V504L299 492Q222 418 124 381L112 377V421",
      8595: "312 96V600H355V96L368 108Q445 182 543 219L555 223V179L546 176Q538 173 529 169T505 158T475 141T442 119T407 90T372 53T339 9L334 0L327 11Q287 73 236 111T112 179V223L124 219Q222 182 299 108L312 96",
      8657: "142 329Q300 419 389 599Q389 598 399 579T420 541T452 494T497 438T558 383T636 329T708 294L721 289V246Q718 246 694 256T623 293T532 356L522 364L521 182V0H478V405L466 417Q436 450 389 516Q388 515 378 500T352 463T312 417L300 405V0H257V364L247 356Q202 320 155 293T82 256L57 246V289L70 294Q101 305 142 329",
      8659: "257 236V600H300V195L312 183Q342 150 389 84Q390 85 400 100T426 137T466 183L478 195V600H521V418L522 236L532 244Q576 280 623 307T696 344L721 354V311L708 306Q677 295 636 271Q478 181 389 1Q389 2 379 21T358 59T326 106T281 162T220 217T142 271T70 306L57 311V354Q60 354 83 345T154 308T247 244L257 236",
      8719: "158 656Q147 684 131 694Q110 707 69 710H55V750H888V710H874Q840 708 820 698T795 678T786 656V-155Q798 -206 874 -210H888V-250H570V-210H584Q618 -208 638 -197T663 -178T673 -155V710H270V277L271 -155Q283 -206 359 -210H373V-250H55V-210H69Q103 -208 123 -197T148 -178T158 -155V656",
      8720: "158 656Q147 684 131 694Q110 707 69 710H55V750H373V710H359Q325 708 305 698T280 678T271 656L270 223V-210H673V656Q666 672 663 679T639 697T584 710H570V750H888V710H874Q840 708 820 698T795 678T786 656V-155Q798 -206 874 -210H888V-250H55V-210H69Q103 -208 123 -197T148 -178T158 -155V656",
      8721: "61 748Q64 750 489 750H913L954 640Q965 609 976 579T993 533T999 516H979L959 517Q936 579 886 621T777 682Q724 700 655 705T436 710H319Q183 710 183 709Q186 706 348 484T511 259Q517 250 513 244L490 216Q466 188 420 134T330 27L149 -187Q149 -188 362 -188Q388 -188 436 -188T506 -189Q679 -189 778 -162T936 -43Q946 -27 959 6H999L913 -249L489 -250Q65 -250 62 -248Q56 -246 56 -239Q56 -234 118 -161Q186 -81 245 -11L428 206Q428 207 242 462L57 717L56 728Q56 744 61 748",
      8730: "263 249Q264 249 315 130T417 -108T470 -228L725 302Q981 837 982 839Q989 850 1001 850Q1008 850 1013 844T1020 832V826L741 243Q645 43 540 -176Q479 -303 469 -324T453 -348Q449 -350 436 -350L424 -349L315 -96Q206 156 205 156L171 130Q138 104 137 104L111 130L263 249",
      8739: "146 612Q151 627 166 627Q182 627 187 612Q188 610 188 306T187 0Q184 -15 166 -15Q149 -15 146 0V10Q146 19 146 35T146 73T146 122T145 179T145 241T145 306T145 370T145 433T145 489T146 538T146 576T146 602V612",
      8741: "146 612Q151 627 166 627Q182 627 187 612Q188 610 188 306T187 0Q184 -15 166 -15Q149 -15 146 0V10Q146 19 146 35T146 73T146 122T145 179T145 241T145 306T145 370T145 433T145 489T146 538T146 576T146 602V612ZM368 612Q373 627 388 627Q404 627 409 612Q410 610 410 306T409 0Q406 -15 389 -15Q371 -15 368 0V10Q368 19 368 35T368 73T368 122T367 179T367 241T367 306T367 370T367 433T367 489T368 538T368 576T368 602V612",
      8747: "113 -244Q113 -246 119 -251T139 -263T167 -269Q186 -269 199 -260Q220 -247 232 -218T251 -133T262 -15T276 155T297 367Q300 390 305 438T314 512T325 580T340 647T361 703T390 751T428 784T479 804Q481 804 488 804T501 805Q552 802 581 769T610 695Q610 669 594 657T561 645Q542 645 527 658T512 694Q512 705 516 714T526 729T538 737T548 742L552 743Q552 745 545 751T525 762T498 768Q475 768 460 756T434 716T418 652T407 559T398 444T387 300T369 133Q349 -38 337 -102T303 -207Q256 -306 169 -306Q119 -306 87 -272T55 -196Q55 -170 71 -158T104 -146Q123 -146 138 -159T153 -195Q153 -206 149 -215T139 -230T127 -238T117 -242L113 -244",
      8748: "113 -244Q113 -246 119 -251T139 -263T167 -269Q186 -269 199 -260Q220 -247 232 -218T251 -133T262 -15T276 155T297 367Q300 390 305 438T314 512T325 580T340 647T361 703T390 751T428 784T479 804Q481 804 488 804T501 805Q552 802 581 769T610 695Q610 669 594 657T561 645Q542 645 527 658T512 694Q512 705 516 714T526 729T538 737T548 742L552 743Q552 745 545 751T525 762T498 768Q475 768 460 756T434 716T418 652T407 559T398 444T387 300T369 133Q349 -38 337 -102T303 -207Q256 -306 169 -306Q119 -306 87 -272T55 -196Q55 -170 71 -158T104 -146Q123 -146 138 -159T153 -195Q153 -206 149 -215T139 -230T127 -238T117 -242L113 -244ZM460 -244Q460 -246 466 -251T486 -263T514 -269Q532 -269 546 -260Q567 -247 579 -218T598 -133T609 -15T623 155T644 367Q647 390 652 438T661 512T672 580T687 647T708 703T737 751T775 784T826 804Q828 804 835 804T848 805Q899 802 928 769T957 695Q957 669 941 657T908 645Q889 645 874 658T859 694Q859 705 863 714T873 729T885 737T895 742L899 743Q899 745 892 751T872 762T845 768Q822 768 807 756T781 716T765 652T754 559T745 444T734 300T716 133Q696 -38 684 -102T650 -207Q603 -306 516 -306Q466 -306 434 -272T402 -196Q402 -170 418 -158T451 -146Q470 -146 485 -159T500 -195Q500 -206 496 -215T486 -230T474 -238T464 -242L460 -244",
      8749: "113 -244Q113 -246 119 -251T139 -263T167 -269Q186 -269 199 -260Q220 -247 232 -218T251 -133T262 -15T276 155T297 367Q300 390 305 438T314 512T325 580T340 647T361 703T390 751T428 784T479 804Q481 804 488 804T501 805Q552 802 581 769T610 695Q610 669 594 657T561 645Q542 645 527 658T512 694Q512 705 516 714T526 729T538 737T548 742L552 743Q552 745 545 751T525 762T498 768Q475 768 460 756T434 716T418 652T407 559T398 444T387 300T369 133Q349 -38 337 -102T303 -207Q256 -306 169 -306Q119 -306 87 -272T55 -196Q55 -170 71 -158T104 -146Q123 -146 138 -159T153 -195Q153 -206 149 -215T139 -230T127 -238T117 -242L113 -244ZM460 -244Q460 -246 466 -251T486 -263T514 -269Q532 -269 546 -260Q567 -247 579 -218T598 -133T609 -15T623 155T644 367Q647 390 652 438T661 512T672 580T687 647T708 703T737 751T775 784T826 804Q828 804 835 804T848 805Q899 802 928 769T957 695Q957 669 941 657T908 645Q889 645 874 658T859 694Q859 705 863 714T873 729T885 737T895 742L899 743Q899 745 892 751T872 762T845 768Q822 768 807 756T781 716T765 652T754 559T745 444T734 300T716 133Q696 -38 684 -102T650 -207Q603 -306 516 -306Q466 -306 434 -272T402 -196Q402 -170 418 -158T451 -146Q470 -146 485 -159T500 -195Q500 -206 496 -215T486 -230T474 -238T464 -242L460 -244ZM807 -244Q807 -246 813 -251T833 -263T861 -269Q880 -269 893 -260Q914 -247 926 -218T945 -133T956 -15T970 155T991 367Q994 390 999 438T1008 512T1019 580T1034 647T1055 703T1084 751T1122 784T1173 804Q1175 804 1182 804T1195 805Q1246 802 1275 769T1304 695Q1304 669 1288 657T1255 645Q1236 645 1221 658T1206 694Q1206 705 1210 714T1220 729T1232 737T1242 742L1246 743Q1246 745 1239 751T1219 762T1192 768Q1169 768 1154 756T1128 716T1112 652T1101 559T1092 444T1081 300T1063 133Q1043 -38 1031 -102T997 -207Q950 -306 863 -306Q813 -306 781 -272T749 -196Q749 -170 765 -158T798 -146Q817 -146 832 -159T847 -195Q847 -206 843 -215T833 -230T821 -238T811 -242L807 -244",
      8750: "269 74L256 80Q244 85 227 97T191 128T161 179T148 250Q148 332 199 379T302 433L306 434L307 444Q309 456 313 495T321 553T331 607T345 664T365 712T393 756T431 785T479 804Q481 804 488 804T501 805Q552 802 581 769T610 695Q610 669 594 657T561 645Q542 645 527 658T512 694Q512 705 516 714T526 729T538 737T548 742L552 743Q552 745 545 751T525 762T498 768Q471 768 454 752T427 693T414 626T406 536Q405 530 405 527L397 425L404 422Q410 419 421 413T445 399T470 376T494 345T511 303T518 250Q518 205 502 169T460 112T410 80T364 66L360 65L359 55Q357 38 353 4T346 -43T340 -81T333 -118T326 -148T316 -179T303 -207Q256 -306 169 -306Q119 -306 87 -272T55 -196Q55 -170 71 -158T104 -146Q123 -146 138 -159T153 -195Q153 -206 149 -215T139 -230T127 -238T117 -242L113 -244Q113 -246 119 -251T139 -263T167 -269Q186 -269 199 -260Q231 -241 242 -183T266 33L269 74ZM272 122Q272 156 300 391Q300 392 299 392Q287 392 263 379T213 331T187 249Q187 211 205 180T239 137T272 116V122ZM366 107Q378 107 402 119T453 167T479 249Q479 340 394 383V377Q394 375 394 374T393 371T393 366T392 357T391 342T389 321T386 291T382 251T377 199T369 133Q366 112 366 107",
      8896: "119 -249T97 -249T65 -235T55 -207Q55 -201 56 -198Q58 -190 218 268T380 729Q392 750 416 750Q438 750 451 732Q453 728 534 498T695 36L775 -194Q777 -204 777 -208Q777 -222 767 -235T735 -249Q713 -249 700 -231Q696 -225 557 177L416 579L276 177Q136 -226 132 -231Q119 -249 97 -249",
      8897: "55 708Q55 729 68 739T96 750Q119 750 132 731Q136 726 276 323L416 -79L557 323Q696 725 700 731Q713 749 735 749Q756 749 766 736T777 708Q777 700 696 466T533 1T451 -232Q436 -249 416 -249Q402 -249 391 -241Q384 -236 380 -226Q368 -198 219 230Q55 697 55 708",
      8898: "139 -217Q127 -241 114 -246Q106 -249 97 -249Q67 -249 57 -220Q55 -214 55 102Q55 152 55 221T54 312Q54 422 60 464T91 554Q120 612 165 654T257 714T337 741T392 749Q393 750 402 750Q414 750 422 749Q557 749 660 659T776 430Q777 422 777 102Q777 -214 775 -220Q765 -249 735 -249Q716 -249 708 -241T694 -217L692 428L690 441Q674 540 597 603T416 666H409Q388 666 364 662T294 638T212 581Q156 523 142 441L140 428L139 105V-217",
      8899: "96 750Q103 750 109 748T120 744T127 737T133 730T137 723T139 718V395L140 73L142 60Q159 -43 237 -104T416 -166Q521 -166 597 -103T690 60L692 73L694 718Q708 749 735 749Q765 749 775 720Q777 714 777 398Q777 78 776 71Q766 -51 680 -140Q571 -249 416 -249H411Q261 -249 152 -140Q66 -51 56 71Q55 78 55 398Q55 714 57 720Q60 734 70 740Q80 750 96 750",
      8968: "202 -349V850H449V810H242V-349H202",
      8969: "22 810V850H269V-349H229V810H22",
      8970: "202 -349V850H242V-309H449V-349H202",
      8971: "229 -309V850H269V-349H22V-309H229",
      9001: "373 850Q392 850 394 832Q394 825 267 538L139 250L267 -38Q394 -325 394 -332Q392 -350 375 -350Q361 -350 356 -338Q354 -331 289 -186T161 103T97 250T160 397T289 685T356 838Q362 850 373 850",
      9002: "77 832Q77 837 82 843T98 850Q110 849 115 838Q117 831 182 686T310 397T374 250T311 103T182 -185T115 -338Q110 -350 96 -350Q79 -350 77 -332Q77 -325 204 -38L332 250L204 538Q77 825 77 832",
      9168: "312 0V602H355V0H312",
      10072: "146 612Q151 627 166 627Q182 627 187 612Q188 610 188 306T187 0Q184 -15 166 -15Q149 -15 146 0V10Q146 19 146 35T146 73T146 122T145 179T145 241T145 306T145 370T145 433T145 489T146 538T146 576T146 602V612",
      10216: "373 850Q392 850 394 832Q394 825 267 538L139 250L267 -38Q394 -325 394 -332Q392 -350 375 -350Q361 -350 356 -338Q354 -331 289 -186T161 103T97 250T160 397T289 685T356 838Q362 850 373 850",
      10217: "77 832Q77 837 82 843T98 850Q110 849 115 838Q117 831 182 686T310 397T374 250T311 103T182 -185T115 -338Q110 -350 96 -350Q79 -350 77 -332Q77 -325 204 -38L332 250L204 538Q77 825 77 832",
      10752: "555 -250Q420 -250 306 -185T124 -4T56 250Q56 453 193 595T526 749Q528 750 539 750Q554 750 562 749Q688 749 800 687T983 508T1054 250Q1054 112 987 -3T806 -184T555 -250ZM555 -165Q672 -165 767 -108T916 44T970 250Q970 418 861 532T600 664Q591 665 548 665Q446 665 353 614T200 466T140 250V243Q140 88 248 -30Q262 -46 280 -62T338 -105T434 -148T555 -165ZM478 250Q478 288 503 307T551 326Q586 326 609 305T632 250Q632 217 610 196T555 174T500 196T478 250",
      10753: "555 -250Q420 -250 306 -185T124 -4T56 250Q56 453 193 595T526 749Q528 750 539 750Q554 750 562 749Q688 749 800 687T983 508T1054 250Q1054 112 987 -3T806 -184T555 -250ZM513 478Q513 664 512 664Q504 664 481 660T406 637T313 588Q281 564 255 537T211 483T181 431T161 382T150 342T144 310T141 292H513V478ZM798 588Q758 616 711 634T639 658T602 663L597 664V292H969Q969 293 967 309T960 341T949 381T930 430T900 482T856 537T798 588ZM513 -164V208H141Q142 205 144 189T149 160T158 125T173 83T196 39T229 -9Q249 -34 273 -55T318 -92T363 -119T405 -138T444 -150T475 -158T499 -162T513 -164ZM775 -103Q801 -87 823 -68T863 -30T894 10T919 49T937 88T950 123T959 154T964 180T968 198L969 208H597V-164Q599 -163 616 -161T647 -155T683 -145T728 -128T775 -103",
      10754: "555 -250Q420 -250 306 -185T124 -4T56 250Q56 453 193 595T526 749Q528 750 539 750Q554 750 562 749Q688 749 800 687T983 508T1054 250Q1054 112 987 -3T806 -184T555 -250ZM600 664Q591 665 548 665Q414 665 306 583L292 573L423 441L555 310L687 441L818 573L804 583Q714 650 600 664ZM364 118L495 250L364 382L232 513L223 500Q140 391 140 250Q140 107 223 0L232 -13L364 118ZM970 250Q970 389 887 501L878 512Q878 513 861 496T812 447T746 381L615 250L746 118L878 -13L887 0Q970 109 970 250ZM687 59L555 190L423 59L292 -73L306 -83Q416 -166 555 -166T804 -83L818 -73L687 59",
      10756: "96 750Q103 750 109 748T120 744T127 737T133 730T137 723T139 718V395L140 73L142 60Q159 -43 237 -104T416 -166Q521 -166 597 -103T690 60L692 73L694 718Q708 749 735 749Q765 749 775 720Q777 714 777 398Q777 78 776 71Q766 -51 680 -140Q571 -249 416 -249H411Q261 -249 152 -140Q66 -51 56 71Q55 78 55 398Q55 714 57 720Q60 734 70 740Q80 750 96 750ZM223 276Q223 282 224 287T227 296T232 302T238 308T243 313T250 316L254 319H374V376V406Q374 438 382 454T418 470Q443 467 450 453T458 410V376V319H579Q580 319 583 317T589 313T594 308T600 302T604 295T608 287T609 276Q609 253 587 241Q577 235 513 235H458V178Q458 176 458 166T459 148Q459 84 415 84Q401 84 390 93T375 117Q374 120 374 178V235H319Q317 235 307 235T290 234Q223 234 223 276",
      10758: "777 -217Q766 -244 745 -249H88Q64 -242 57 -220Q55 -214 55 250T57 720Q60 734 70 740Q80 750 96 750Q127 750 137 720Q139 714 139 274V-166H693V274Q693 714 695 720Q705 749 735 749Q766 749 775 719Q777 713 777 248V-217",
      12296: "373 850Q392 850 394 832Q394 825 267 538L139 250L267 -38Q394 -325 394 -332Q392 -350 375 -350Q361 -350 356 -338Q354 -331 289 -186T161 103T97 250T160 397T289 685T356 838Q362 850 373 850",
      12297: "77 832Q77 837 82 843T98 850Q110 849 115 838Q117 831 182 686T310 397T374 250T311 103T182 -185T115 -338Q110 -350 96 -350Q79 -350 77 -332Q77 -325 204 -38L332 250L204 538Q77 825 77 832"
    }, {
      10764: "∬∬"
    }), smallop$1;
  }
  var texCalligraphicBold$1 = {}, texCalligraphicBold = {}, hasRequiredTexCalligraphicBold$1;
  function requireTexCalligraphicBold$1() {
    return hasRequiredTexCalligraphicBold$1 || (hasRequiredTexCalligraphicBold$1 = 1, Object.defineProperty(texCalligraphicBold, "__esModule", { value: !0 }), texCalligraphicBold.texCalligraphicBold = void 0, texCalligraphicBold.texCalligraphicBold = {
      65: [0.751, 0.049, 0.921, { ic: 0.068, sk: 0.224 }],
      66: [0.705, 0.017, 0.748, { sk: 0.16 }],
      67: [0.703, 0.02, 0.613, { sk: 0.16 }],
      68: [0.686, 0, 0.892, { sk: 0.0958 }],
      69: [0.703, 0.016, 0.607, { ic: 0.02, sk: 0.128 }],
      70: [0.686, 0.03, 0.814, { ic: 0.116, sk: 0.128 }],
      71: [0.703, 0.113, 0.682, { sk: 0.128 }],
      72: [0.686, 0.048, 0.987, { sk: 0.128 }],
      73: [0.686, 0, 0.642, { ic: 0.104, sk: 0.0319 }],
      74: [0.686, 0.114, 0.779, { ic: 0.158, sk: 0.192 }],
      75: [0.703, 0.017, 0.871, { sk: 0.0639 }],
      76: [0.703, 0.017, 0.788, { sk: 0.16 }],
      77: [0.703, 0.049, 1.378, { sk: 0.16 }],
      78: [0.84, 0.049, 0.937, { ic: 0.168, sk: 0.0958 }],
      79: [0.703, 0.017, 0.906, { sk: 0.128 }],
      80: [0.686, 0.067, 0.81, { ic: 0.036, sk: 0.0958 }],
      81: [0.703, 0.146, 0.939, { sk: 0.128 }],
      82: [0.686, 0.017, 0.99, { sk: 0.0958 }],
      83: [0.703, 0.016, 0.696, { ic: 0.025, sk: 0.16 }],
      84: [0.72, 0.069, 0.644, { ic: 0.303, sk: 0.0319 }],
      85: [0.686, 0.024, 0.715, { ic: 0.056, sk: 0.0958 }],
      86: [0.686, 0.077, 0.737, { ic: 0.037, sk: 0.0319 }],
      87: [0.686, 0.077, 1.169, { ic: 0.037, sk: 0.0958 }],
      88: [0.686, 0, 0.817, { ic: 0.089, sk: 0.16 }],
      89: [0.686, 0.164, 0.759, { ic: 0.038, sk: 0.0958 }],
      90: [0.686, 0, 0.818, { ic: 0.035, sk: 0.16 }],
      305: [0.452, 8e-3, 0.394, { sk: 0.0319 }],
      567: [0.451, 0.201, 0.439, { sk: 0.0958 }]
    }), texCalligraphicBold;
  }
  var hasRequiredTexCalligraphicBold;
  function requireTexCalligraphicBold() {
    if (hasRequiredTexCalligraphicBold) return texCalligraphicBold$1;
    hasRequiredTexCalligraphicBold = 1, Object.defineProperty(texCalligraphicBold$1, "__esModule", { value: !0 }), texCalligraphicBold$1.texCalligraphicBold = void 0;
    var O = requireFontData(), _ = requireTexCalligraphicBold$1();
    return texCalligraphicBold$1.texCalligraphicBold = (0, O.AddPaths)(_.texCalligraphicBold, {
      65: "761 751Q784 751 784 728V711Q784 570 795 417T820 191Q844 76 865 76Q868 76 902 93T962 112H973Q989 104 989 94Q989 92 987 86Q978 47 846 -11Q812 -25 779 -26Q722 -26 708 9Q688 47 669 161H524L379 162L359 136Q286 43 234 -3T142 -49T71 -19T39 55Q39 92 60 131T103 174Q113 174 117 167T124 149T136 128T166 110Q183 105 190 105Q230 105 341 246Q401 322 453 397T531 514T582 601T611 651H640V663Q640 692 676 718T745 750Q747 750 752 750T761 751ZM658 266Q653 312 649 377T644 489T641 541V556L557 415Q500 325 448 253Q467 261 524 261H568H658V266",
      66: "441 83Q571 83 571 195Q571 246 538 279T466 322T386 333Q378 333 357 330T329 327Q307 327 307 345Q307 354 313 365T347 396T419 430Q546 472 581 498Q594 508 594 535Q594 574 567 589T508 604Q469 604 442 583Q390 546 342 457T265 256Q237 148 186 60Q167 29 144 13Q105 -15 68 -17H65Q40 -17 40 1Q40 7 53 29T87 100T124 210Q162 373 190 575Q188 575 174 567T138 550T98 542Q75 542 75 560Q75 588 123 618Q135 625 203 659T281 696Q299 703 316 703Q339 703 339 685Q339 656 328 583L333 588Q338 592 346 599T367 615T394 634T428 654T467 674T511 690T559 701T611 705Q651 704 681 689Q739 659 739 598Q739 507 595 427L584 421Q585 420 595 416T610 410T626 402T644 392T660 380T677 365T691 347T703 325T710 299T715 268Q717 193 665 130Q622 73 531 28T348 -17Q275 -17 225 10Q206 19 200 24T193 36Q193 57 231 86T311 119H322Q386 83 441 83",
      67: "243 -20Q135 -20 78 48T20 218Q20 280 39 344T98 470T193 580T324 661T488 702H490Q491 702 493 702T498 703T507 703T518 702H526Q599 702 599 631Q599 597 577 550T541 486Q523 463 490 447T431 430Q423 430 419 433Q409 437 409 450Q410 456 432 499T454 567Q454 586 446 594T415 603Q316 603 254 532Q216 487 190 421T164 284Q164 228 181 186T226 122T282 90T340 80Q377 80 405 96T449 131T492 167T553 187H555Q580 187 580 168Q580 153 551 123T476 60T367 4T243 -20",
      68: "107 0Q92 5 92 18Q92 33 113 53T160 86Q170 91 182 94T197 100L206 120Q248 226 273 337T304 501T309 585Q278 585 234 577T179 557Q172 550 166 532T156 509Q140 484 105 466T44 447Q20 447 20 465Q20 482 34 510T76 565Q122 608 173 632Q281 686 447 686H480H517Q692 686 784 631Q885 571 885 450Q885 339 805 239T586 75T286 1Q276 0 187 0H107ZM741 391Q741 424 731 452T694 510T613 558T481 584Q476 584 468 584T457 585L449 586V579Q441 501 425 424T391 292T357 193T330 125T319 100H324Q511 100 628 175Q688 215 714 275T741 391",
      69: "495 516Q485 516 478 520T470 532Q470 537 476 550T482 570Q482 589 465 596T401 603Q344 603 319 582Q295 558 295 519Q295 493 312 474T355 445T407 432T455 427Q477 427 477 409Q477 395 453 371T389 333Q380 330 345 327T279 312T223 272Q181 223 181 176Q181 131 225 107T324 83Q366 83 395 98T448 136T487 167Q517 185 547 187H551Q574 187 574 170Q574 151 535 113T421 36T271 -15Q260 -16 226 -16Q181 -16 152 -9Q104 4 71 33T37 111Q37 140 50 176T106 263T216 356Q215 357 207 362T190 374T172 392T156 419T150 456Q150 521 208 580T341 670T474 702Q475 703 499 703Q528 703 547 701T586 693T615 673T627 637Q627 584 581 550T495 516",
      70: "812 567Q804 567 795 570T786 579Q786 586 647 586H559L558 582Q558 575 539 510T506 407L498 384H775Q788 378 790 368Q790 328 746 298T665 268Q646 268 642 284H457L447 261Q396 150 360 94Q329 46 270 8T149 -30Q123 -30 100 -24T63 -10T37 9T22 26T17 36Q17 59 56 88T135 119Q145 119 149 117T165 104Q187 78 227 72Q232 72 234 76Q245 93 273 145T350 323T424 570L428 586H276Q265 575 239 563T190 551Q180 551 174 556T167 569Q167 580 179 597T213 634T278 668T371 685Q374 686 624 686Q863 685 888 682Q917 678 927 663Q930 658 930 650Q930 624 888 596T812 567",
      71: "50 279Q50 361 88 438T190 570T335 661T503 702H514Q524 703 532 703Q671 703 671 626Q671 580 614 514T495 447Q472 447 472 465Q472 477 499 511T527 562Q527 582 507 592T433 602Q351 602 302 569Q252 535 223 469T194 344Q194 266 237 217T352 168Q401 168 442 205T505 316Q515 345 551 368T622 393H625Q649 393 649 376Q649 371 635 318T612 237Q580 129 540 62T442 -49Q353 -114 264 -114Q259 -114 252 -114L246 -113Q190 -113 142 -107T81 -96Q71 -90 71 -81Q71 -57 110 -30T187 2Q196 2 236 -4T338 -14Q371 -14 377 -9Q410 24 446 113L451 127Q353 68 253 68Q157 68 104 129T50 279",
      72: "42 447Q20 447 20 465Q20 481 47 515T119 589T239 657T392 686Q443 686 463 664T484 616Q484 570 473 506T452 401L441 360Q441 359 550 359H660L663 370Q684 435 716 522T758 624Q771 646 806 666T870 686Q894 686 894 668Q894 667 867 597T804 416T752 218Q737 135 737 93Q737 77 746 65T778 53Q799 53 803 54T814 63Q831 86 864 103T924 120Q946 120 946 100Q945 85 931 63T888 16T806 -27T684 -48H681Q625 -48 603 -10Q593 4 593 29Q593 71 603 131T624 230L634 269Q632 269 624 266Q610 261 600 261T507 259H411L399 222Q344 62 322 21Q301 -7 268 -24T209 -41H207Q187 -41 185 -25Q185 -17 192 2T220 71T261 184Q284 256 284 258Q284 259 227 259H170Q169 259 166 261T162 264T158 266T156 271T155 277Q155 296 184 320T250 356Q259 358 286 359Q312 359 312 360Q314 372 318 385Q332 450 339 526Q339 530 339 535T340 543Q340 586 296 586Q255 586 227 576T188 553T165 523T146 497Q127 476 97 462T42 447",
      73: "56 499Q32 499 32 516Q32 540 64 580T165 649Q241 682 365 685Q366 685 376 685T405 685T445 686T496 686T553 686H732Q746 677 746 668Q746 646 711 620T642 587L572 586H503Q479 546 458 479T424 352T383 224T318 111L309 101L412 100H514L523 109Q567 150 618 153Q644 153 644 135Q644 132 642 124Q629 86 581 52T476 6Q454 2 433 2T216 0Q-11 0 -15 2Q-27 6 -27 18Q-27 37 2 61T59 93Q77 100 142 100H198Q255 177 299 369Q337 513 382 574L391 586H348Q261 586 176 576Q163 543 124 521T56 499",
      74: "286 390Q263 390 263 407Q263 432 293 481T367 566Q511 687 724 687Q738 687 761 687T793 686H923Q937 677 937 668Q937 648 905 623T842 589Q829 587 817 586T802 585T795 583T788 578Q709 506 632 189Q622 153 615 134T588 81T537 17Q482 -39 404 -76T247 -114Q192 -114 158 -100Q53 -61 53 32Q53 59 58 73T79 102Q126 147 177 147Q200 147 200 128Q200 123 198 112T196 96Q196 47 238 17T345 -13Q362 -13 377 -9T404 0T426 16T444 34T459 55T470 76T478 97T483 116T488 132L490 141Q511 222 520 257T554 364T608 486T675 576L685 586H634H612Q532 586 484 564Q453 549 436 526T409 478T395 447Q378 424 345 407T286 390",
      75: "98 542Q75 542 75 560Q75 588 123 618Q132 624 199 657T275 694Q291 703 315 703Q327 703 332 699T338 690T339 670Q339 596 323 505T283 337T237 194T198 90L181 53Q170 31 136 8T68 -17H65Q40 -17 40 0L76 92Q112 185 150 322T194 564V578L168 565Q125 542 98 542ZM834 142Q834 125 819 100T774 48T692 3T576 -16H560Q540 -16 508 6Q469 33 422 108T342 267T309 398Q309 411 310 417T320 442T347 482Q401 542 517 615T710 702Q712 702 721 702T735 703Q772 703 791 690Q819 674 819 646T792 597T733 574H722Q704 584 704 599Q706 607 700 610T672 617L660 613Q609 595 524 538T423 450V440Q423 376 488 247T604 83Q621 70 640 70Q677 70 701 82Q713 87 718 101T737 132T783 160Q792 163 807 163Q834 163 834 142",
      76: "63 -17Q41 -17 41 0Q41 22 85 54Q101 68 113 92T133 141T154 219T182 315Q230 462 306 553Q345 599 391 632T478 678T543 697T582 703Q584 703 589 703T598 702Q643 702 666 676T689 613Q689 588 683 575Q674 551 632 524T552 496Q530 496 530 512Q530 517 531 525T533 538Q533 559 522 577T480 596H476Q462 596 451 588T415 544Q350 447 310 281Q284 181 261 136L255 124H285Q342 123 441 107T583 90L596 89Q603 116 647 144T729 173Q751 173 751 157Q751 118 685 60T523 -15Q514 -16 479 -16Q421 -16 320 0T171 18H155L142 10Q98 -17 63 -17",
      77: "38 20Q38 59 60 99T104 139Q106 139 126 125T176 106H181Q200 106 221 139T286 281Q322 370 342 451T368 581T376 634Q384 657 420 680T487 703Q502 703 507 696T522 649Q538 589 554 537Q579 453 609 372T660 248T686 202Q687 201 739 244T830 322L1166 642Q1225 700 1230 701Q1230 701 1237 703Q1258 703 1258 667L1253 637Q1248 607 1241 558T1227 451T1214 326T1209 202Q1209 77 1232 77Q1237 77 1269 94T1326 112H1329Q1353 112 1353 94Q1353 81 1334 60Q1311 37 1248 7T1150 -24H1141H1135Q1085 -24 1074 26Q1064 75 1064 134Q1064 239 1086 426Q1087 430 1087 434L1061 410Q871 227 783 149L694 76Q653 44 647 40T631 34Q620 34 616 37T594 63Q546 125 514 198Q467 307 423 449L418 466L412 444Q376 310 306 153Q278 88 251 45T201 -18T163 -43T131 -49Q102 -48 70 -31T38 20",
      78: "47 139Q81 105 122 105Q137 105 147 117Q159 134 182 199T234 381T274 610Q275 634 284 647Q297 666 327 684T389 703Q403 703 408 695T428 645Q480 490 567 298Q628 163 673 103Q674 102 674 102T675 106Q732 331 803 551Q842 674 875 725Q908 775 966 807T1081 840H1084Q1105 840 1105 803Q1105 768 1088 733T1051 689Q1045 686 1032 686Q986 683 948 663T901 624Q881 579 837 430T760 154L726 28Q725 28 725 28T723 25Q716 0 682 -24T611 -48Q600 -48 595 -45T576 -23Q522 44 480 124Q417 243 332 463L328 473L325 457Q291 293 227 124Q159 -49 72 -49Q38 -49 5 -28Q-24 -8 -24 21Q-24 58 -3 98T41 139H47",
      79: "433 703Q456 703 456 685Q456 672 441 655T407 627Q402 623 378 611T328 579T276 524Q207 434 207 324Q207 222 270 153T441 84Q566 84 651 177T737 400V405Q737 496 693 549T576 603Q542 603 510 560Q490 537 472 502T442 454Q397 412 346 409Q320 409 320 427Q320 430 322 436Q331 465 360 507T433 594T542 671T677 703Q776 703 829 636T882 468Q882 369 831 277T702 122T528 21T343 -17Q214 -17 139 61T63 257Q63 336 94 409T173 534T272 625T367 684T432 703H433",
      80: "170 -67Q147 -67 147 -49Q147 -42 162 -8T204 99T253 254Q274 332 288 415T305 542L308 585Q277 585 234 577T179 557Q172 550 166 532T156 509Q140 484 105 466T44 447Q20 447 20 465Q20 482 34 510T76 565Q122 608 173 632Q279 686 448 686H495H537Q622 686 678 677T784 637Q846 598 846 533Q846 452 776 375T597 252T378 206H366L358 181Q341 130 316 68T282 -7Q262 -33 230 -50T170 -67ZM701 468Q701 512 661 540T570 577T461 586H448V582Q446 576 443 545T428 447T395 301L389 280Q390 280 398 284T419 295T441 303Q443 304 484 306T572 321T651 359Q701 402 701 468",
      81: "874 453Q874 372 836 298T750 177T638 89T543 33T486 8L483 7Q485 5 523 -7T622 -32T726 -46Q741 -46 746 -45T755 -41T762 -27Q770 -1 806 23T878 50H890Q905 42 905 33Q905 -8 838 -68T670 -145Q662 -146 628 -146Q538 -146 389 -100T164 -50Q132 -50 132 -32T162 11T227 47Q231 48 286 51T394 62T518 100T641 180Q730 271 730 387Q730 478 673 540T520 602Q410 602 337 525T264 355Q264 284 310 244T420 203Q476 203 568 222Q594 222 594 204Q594 184 565 161T508 128Q433 103 316 103Q227 103 174 157T120 290Q120 382 182 471T343 620T548 697Q578 703 601 703Q604 703 611 703T623 702Q663 702 687 696Q760 679 817 618T874 453",
      82: "159 0Q159 5 172 34T205 114T245 229T284 386T309 575V585H304Q303 585 295 585T282 584Q233 579 207 570T175 553T165 531T156 509Q140 484 105 466T44 447Q20 447 20 465Q20 482 34 510T76 565Q122 608 173 632Q279 686 448 686H505H582Q683 686 745 672T834 611Q842 594 842 565Q842 523 824 484T780 419T722 370T669 336T632 318L619 312L626 302Q640 279 667 227T696 172Q717 133 735 112T762 88T784 84Q824 84 872 118T957 153Q981 153 981 136Q981 114 937 78T820 13T684 -17Q646 -17 616 8T569 66T526 151T477 234Q461 256 446 265Q437 272 421 274Q400 274 400 291Q400 311 430 336T495 371Q496 371 543 374T627 392T681 436Q699 467 699 503Q699 550 644 568T471 586H449V582Q449 581 447 559T438 499T422 413T393 298T348 165Q313 73 296 45Q282 24 249 4T185 -17Q159 -17 159 0",
      83: "204 476Q204 525 248 577T372 666T539 703T674 683T721 612Q721 588 714 569Q704 547 669 524T601 499Q573 499 573 516Q573 521 575 527T577 543Q577 563 568 574T548 588L539 590Q490 603 444 603Q418 603 394 597T364 583Q348 567 348 533Q348 493 382 466T459 425T555 387T633 330Q662 292 662 249Q662 153 544 69T257 -16Q218 -16 208 -15Q118 1 64 46Q25 76 25 126Q25 185 82 235T203 290H207Q229 290 231 274Q231 243 180 213Q173 209 172 206T170 189T171 170T183 150T216 121Q273 83 356 83Q412 83 459 100Q493 111 507 141Q518 165 518 185Q518 208 506 228T478 262T437 288T398 306T360 320Q316 335 285 352T239 384T215 416T205 443T204 467V476",
      84: "61 462H59Q38 462 38 479Q38 528 109 594T289 683L304 685L837 687L846 693Q889 720 923 720Q947 720 947 702Q945 671 892 631T776 583Q774 583 772 583T769 582T766 582L764 581H758Q753 581 744 581T722 580T693 580T662 580H563L514 385Q507 355 493 299T475 225T460 172T443 119T426 76T402 24Q386 -11 355 -33T304 -61T266 -69Q242 -69 242 -50Q243 -45 253 -25T278 32T307 115L364 340Q405 511 413 538T436 580H207Q202 572 200 568T197 561T195 552T190 537Q176 511 135 487T61 462",
      85: "124 586Q107 586 74 569T15 552H13Q-10 552 -10 570Q-10 605 70 645T222 686Q283 686 283 631Q283 590 246 504T172 326T135 181Q135 130 157 107T205 83Q221 83 259 106Q347 165 453 301T604 548Q607 557 612 569T619 587T624 600T628 612T632 621T637 628T641 634T647 640T654 645T662 652Q706 686 748 686Q771 686 771 669Q771 656 754 614T700 467T630 229Q615 168 610 105Q610 88 617 78L641 90Q681 111 706 112Q733 112 733 95Q733 82 714 60Q694 40 633 10Q567 -23 532 -24Q507 -24 495 -17Q466 -4 466 32Q466 96 500 225Q277 -17 102 -17Q56 -17 23 17T-10 118Q-10 164 13 234T64 363T115 481T139 567Q139 586 124 586",
      86: "25 608Q25 628 60 657T148 686Q184 683 213 671T273 625T327 538T363 394T380 184L381 134L399 148Q503 226 574 302T667 415T689 467Q688 474 684 482T672 502T645 521T600 532Q576 532 576 567Q576 604 597 644T641 685H649Q701 685 737 648T774 545Q774 457 703 333T461 66Q397 13 332 -32T255 -77Q237 -77 237 -30V-23Q241 20 241 109Q241 483 115 569Q91 586 50 589Q25 589 25 608",
      87: "25 607Q25 629 62 657T142 686Q205 686 248 647T312 541T339 411T347 275Q347 249 345 203V189Q375 219 449 316T587 516Q629 584 629 587Q629 589 626 597T622 607Q622 629 658 656T732 686H744Q755 680 757 678Q757 677 769 649T799 577T835 475T874 339T904 183Q908 157 910 151L925 169Q997 252 1059 343T1121 474Q1120 498 1103 513T1059 532Q1036 532 1036 568Q1036 600 1053 636T1090 683L1097 686H1109Q1147 684 1176 652T1206 551Q1206 460 1131 320T897 7Q859 -33 840 -52T816 -74T804 -77Q788 -77 784 -32Q783 -28 783 -26Q774 108 744 239T691 436T665 501Q664 501 649 475T602 400T528 289T420 146T280 -15Q243 -56 231 -66T210 -77Q191 -77 191 -40Q191 -38 195 -4T204 91T209 217Q209 290 202 351T177 469T126 557T45 589Q25 589 25 607",
      88: "762 562Q762 579 737 584T711 604Q711 630 753 658T834 686Q864 686 885 669T906 627Q906 580 834 522T614 379L584 362V357Q585 354 589 315T597 233T603 183Q610 132 627 116T671 100Q678 100 704 113T754 126T778 107Q776 79 733 45T626 2Q615 1 578 1Q542 1 535 3Q521 7 510 15T491 31T477 54T467 78T460 108T456 137T452 170T449 201Q447 220 445 240T442 270L441 281Q435 281 357 233Q240 165 206 135Q200 128 200 124Q200 113 208 108T226 101T244 96T252 82Q252 61 214 31T129 1H120Q97 1 77 16T56 60Q56 105 133 168T414 345Q428 352 431 354T433 359Q422 493 414 522Q407 551 395 566T373 583T350 586H341L332 580Q290 560 265 560Q243 560 243 577Q243 585 248 596T269 624T306 653T365 676T447 686H456Q472 686 484 683T514 671T543 637T562 576Q565 557 570 501L577 437Q577 436 613 457T694 506T756 551Q762 558 762 562",
      89: "73 555Q49 555 49 573Q49 602 110 644T239 686Q319 686 376 624Q416 584 444 511T483 361T499 240T503 173Q503 165 504 165Q506 165 524 184T556 218Q631 297 674 377T718 485Q718 505 699 526Q673 552 628 552Q619 552 613 562T607 590Q607 617 621 645T658 685Q661 686 671 686Q718 686 757 652T797 545Q797 476 749 369T602 146Q500 29 371 -67T176 -164Q112 -164 74 -120T36 -29Q36 5 55 36T95 67Q104 67 108 59T115 39T128 12T154 -12Q183 -30 216 -30Q239 -30 305 7L361 44L367 49V54Q367 95 364 143T351 273T312 429T243 546Q206 581 156 588L146 581Q108 555 73 555",
      90: "622 574Q522 579 420 579H396Q373 579 364 574T351 550Q339 516 297 490T218 462Q195 462 195 479Q195 487 197 492Q218 565 313 625T509 685Q564 685 650 683T755 680Q787 680 807 683T831 686Q853 686 853 669Q853 657 826 626Q742 532 641 437L619 415L622 414Q626 414 631 414T642 414Q697 411 697 388Q697 367 670 345T607 323Q605 323 592 325T546 329H522L490 302Q457 274 400 226T289 136L260 113L318 112Q345 111 452 109T587 106H627Q650 143 656 170Q666 197 710 225T788 253Q811 253 811 237Q811 211 781 160T710 77Q619 0 515 0Q507 0 497 0T484 1Q434 1 319 3T177 6Q123 6 95 2Q83 2 71 0H68Q46 0 46 17Q46 28 58 44Q68 56 100 80T210 165T383 307L408 329H361L314 330Q297 338 297 350Q297 368 320 388T368 413Q375 415 441 415H506L647 555L664 574H622",
      305: "24 296Q24 305 34 328T63 380T115 430T187 452Q205 452 223 448T262 435T295 406T308 360Q308 345 287 290T240 170T207 87Q202 67 202 57Q202 42 215 42Q235 42 257 64Q288 92 302 140Q307 156 310 159T330 162H336H347Q367 162 367 148Q367 140 357 117T329 65T276 14T201 -8Q158 -8 121 15T83 84Q83 104 133 229T184 358Q189 376 189 388Q189 402 177 402Q156 402 134 380Q103 352 89 304Q84 288 81 285T61 282H55H44Q24 282 24 296",
      567: "297 360T297 373T294 392T288 400T278 401H276Q237 398 200 363Q181 343 170 325T156 299T149 287T129 282H123H116Q102 282 97 284T92 298Q93 303 98 315T118 349T151 390T201 427T267 451H279Q357 451 388 422T420 354V339L370 138Q321 -60 317 -69Q287 -157 163 -194Q133 -201 99 -201Q39 -201 14 -178T-12 -125Q-12 -94 11 -69T68 -43Q93 -43 108 -57T123 -95Q123 -121 100 -151H104Q131 -151 155 -125T193 -60Q195 -54 244 141T294 345Q297 360 297 373"
    }, {}), texCalligraphicBold$1;
  }
  var texCalligraphic$1 = {}, texCalligraphic = {}, hasRequiredTexCalligraphic$1;
  function requireTexCalligraphic$1() {
    return hasRequiredTexCalligraphic$1 || (hasRequiredTexCalligraphic$1 = 1, Object.defineProperty(texCalligraphic, "__esModule", { value: !0 }), texCalligraphic.texCalligraphic = void 0, texCalligraphic.texCalligraphic = {
      65: [0.728, 0.05, 0.798, { ic: 0.021, sk: 0.194 }],
      66: [0.705, 0.022, 0.657, { sk: 0.139 }],
      67: [0.705, 0.025, 0.527, { sk: 0.139 }],
      68: [0.683, 0, 0.771, { sk: 0.0833 }],
      69: [0.705, 0.022, 0.528, { ic: 0.036, sk: 0.111 }],
      70: [0.683, 0.032, 0.719, { ic: 0.11, sk: 0.111 }],
      71: [0.704, 0.119, 0.595, { sk: 0.111 }],
      72: [0.683, 0.048, 0.845, { sk: 0.111 }],
      73: [0.683, 0, 0.545, { ic: 0.097, sk: 0.0278 }],
      74: [0.683, 0.119, 0.678, { ic: 0.161, sk: 0.167 }],
      75: [0.705, 0.022, 0.762, { sk: 0.0556 }],
      76: [0.705, 0.022, 0.69, { sk: 0.139 }],
      77: [0.705, 0.05, 1.201, { sk: 0.139 }],
      78: [0.789, 0.05, 0.82, { ic: 0.159, sk: 0.0833 }],
      79: [0.705, 0.022, 0.796, { sk: 0.111 }],
      80: [0.683, 0.057, 0.696, { ic: 0.037, sk: 0.0833 }],
      81: [0.705, 0.131, 0.817, { sk: 0.111 }],
      82: [0.682, 0.022, 0.848, { sk: 0.0833 }],
      83: [0.705, 0.022, 0.606, { ic: 0.036, sk: 0.139 }],
      84: [0.717, 0.068, 0.545, { ic: 0.288, sk: 0.0278 }],
      85: [0.683, 0.028, 0.626, { ic: 0.061, sk: 0.0833 }],
      86: [0.683, 0.052, 0.613, { ic: 0.045, sk: 0.0278 }],
      87: [0.683, 0.053, 0.988, { ic: 0.046, sk: 0.0833 }],
      88: [0.683, 0, 0.713, { ic: 0.094, sk: 0.139 }],
      89: [0.683, 0.143, 0.668, { ic: 0.046, sk: 0.0833 }],
      90: [0.683, 0, 0.725, { ic: 0.042, sk: 0.139 }]
    }), texCalligraphic;
  }
  var hasRequiredTexCalligraphic;
  function requireTexCalligraphic() {
    if (hasRequiredTexCalligraphic) return texCalligraphic$1;
    hasRequiredTexCalligraphic = 1, Object.defineProperty(texCalligraphic$1, "__esModule", { value: !0 }), texCalligraphic$1.texCalligraphic = void 0;
    var O = requireFontData(), _ = requireTexCalligraphic$1();
    return texCalligraphic$1.texCalligraphic = (0, O.AddPaths)(_.texCalligraphic, {
      65: "576 668Q576 688 606 708T660 728Q676 728 675 712V571Q675 409 688 252Q696 122 720 57Q722 53 723 50T728 46T732 43T737 41T743 39L754 45Q788 61 803 61Q819 61 819 47Q818 43 814 35Q799 15 755 -7T675 -30Q659 -30 648 -25T630 -8T621 11T614 34Q603 77 599 106T594 146T591 160V163H460L329 164L316 145Q241 35 196 -7T119 -50T59 -24T30 43Q30 75 46 100T74 125Q81 125 83 120T88 104T96 84Q118 57 151 57Q189 57 277 182Q432 400 542 625L559 659H567Q574 659 575 660T576 668ZM584 249Q579 333 577 386T575 473T574 520V581L563 560Q497 426 412 290L372 228L370 224H371L383 228L393 232H586L584 249",
      66: "304 342Q292 342 292 353Q292 372 323 391Q331 396 417 428T533 487Q563 512 563 555V562Q563 575 557 589T530 618T475 636Q429 636 396 613T330 539Q263 446 210 238Q196 183 173 120Q135 31 121 16Q108 1 85 -10T47 -22T32 -10Q32 -5 44 18T77 93T112 206Q135 296 154 395T182 550T191 615Q191 616 190 616Q188 616 179 611T157 601T131 594Q113 594 113 605Q113 623 144 644Q154 650 205 676T267 703Q277 705 279 705Q295 705 295 693Q295 686 288 635T278 575Q278 572 287 582Q336 635 402 669T540 704Q603 704 633 673T664 599Q664 559 638 523T580 462Q553 440 504 413L491 407L504 402Q566 381 596 338T627 244Q627 172 575 110T444 13T284 -22Q208 -22 158 28Q144 42 146 50Q150 67 178 85T230 103Q236 103 246 95T267 75T302 56T357 47Q436 47 486 93Q526 136 526 198V210Q526 228 518 249T491 292T436 330T350 345Q335 345 321 344T304 342",
      67: "201 -25Q167 -25 136 -14T75 23T29 94T12 202Q12 290 50 394T161 574Q227 642 303 673T433 704Q435 705 457 705Q533 701 533 640Q533 606 507 548T464 474Q431 444 396 444Q381 444 381 453Q381 459 388 473T407 513T428 563Q433 580 433 594Q433 636 381 636Q314 636 260 594T175 489T128 363T112 247Q112 157 153 101T273 44Q347 44 398 121Q413 144 437 157T481 171Q496 171 496 160Q496 150 476 123Q426 56 350 16T201 -25",
      68: "37 475Q19 475 19 487Q19 536 103 604T327 682H356Q386 683 408 683H419Q475 683 506 681T582 668T667 633Q766 571 766 450Q766 365 723 287T611 152T455 57T279 6Q248 1 160 0Q148 0 131 0T108 -1Q72 -1 72 11Q72 24 90 40T133 64L144 68L152 88Q247 328 272 587Q275 613 272 613Q272 613 269 613Q225 610 195 602T149 579T129 556T119 532Q118 530 116 525T113 518Q102 502 80 490T37 475ZM665 407Q665 596 412 613Q403 614 383 614Q370 614 370 612Q370 598 363 542T323 357T242 103L228 69H265Q391 73 481 119Q536 148 575 188T633 268T658 338T665 392V407",
      69: "144 470Q144 556 240 630T451 705Q564 705 564 637Q564 611 540 573Q529 559 505 547T464 534Q448 534 448 545Q448 552 455 562Q463 577 463 591Q463 600 462 604T456 616T436 627T400 635Q396 635 390 635T380 636Q291 636 258 568Q245 544 245 516Q245 463 290 438T391 410Q415 410 415 398Q415 392 407 380T376 356T326 341Q288 340 260 327Q218 311 187 276T143 208T130 151Q130 113 156 88T211 55T268 47Q349 47 403 125Q415 144 439 157T483 171Q499 171 499 160Q499 148 475 120T413 59T315 3T197 -22Q124 -22 77 14T30 105Q30 126 39 154T66 216T122 288T209 354L223 362Q144 400 144 470",
      70: "199 579Q181 579 181 590Q181 598 188 611T212 639T260 666T335 682Q336 682 349 682T383 682T431 682T493 683T561 683Q776 682 784 681Q826 673 829 647Q829 620 797 600T744 580Q728 580 728 595Q729 607 713 610Q698 613 598 614H500L499 610Q499 598 467 486T428 367Q428 365 551 365H674Q683 360 684 355Q687 346 677 329Q666 312 642 299T598 285Q586 285 582 296H402L394 277Q386 258 373 229T346 167T315 102T286 51Q265 22 225 -5T133 -32Q108 -32 87 -25T54 -7T33 15T21 35T18 47Q18 60 44 80T98 103Q108 103 111 101T119 88Q130 66 150 54T179 39T195 37Q199 37 203 43Q217 67 245 125T318 300T391 532Q393 543 398 564T406 598T409 613T339 614H269Q229 579 199 579",
      71: "216 68Q155 68 115 100T59 177T44 273Q44 299 50 333T73 421T133 533T239 632Q346 704 466 704Q508 704 515 703Q555 696 577 681T599 635Q599 605 570 560T523 496Q490 466 455 466Q440 466 440 475T469 526T499 589Q499 605 489 617Q460 636 403 636Q343 636 295 611T220 548T174 464T150 382T144 318Q144 241 180 189T287 137Q325 137 359 160Q428 205 466 322Q472 342 501 359T551 376Q557 376 560 373T564 368L565 365Q560 341 551 302T512 173T451 31Q359 -119 204 -119Q163 -118 127 -109T74 -91T53 -77Q52 -75 52 -71Q52 -54 79 -35T132 -14H140L151 -19Q210 -49 281 -49H289Q312 -49 329 -31Q351 -7 372 36T405 109T416 142L408 136Q401 131 392 125T369 111T338 96T303 82T261 72T216 68",
      72: "18 487Q18 496 29 517T67 566T127 621T216 665T330 683Q359 683 376 669T397 643T400 622Q400 584 382 488T348 343Q348 342 467 342H587L594 366Q615 440 648 534T690 641Q701 656 723 669T764 683Q783 683 783 672L750 578Q716 485 677 346T625 101Q624 92 623 82T622 65T621 56Q621 20 658 20Q666 20 701 25Q709 52 736 69T785 87Q803 87 803 75T791 44T754 3T685 -33T588 -48Q568 -48 562 -46Q522 -31 522 13V23Q531 129 562 250L569 281L565 280Q561 278 556 277T549 274L438 273H328L321 249Q307 202 275 107T232 0Q219 -16 196 -28T155 -41Q149 -41 145 -39T140 -34T139 -29Q139 -24 148 -3T181 86T233 247Q240 270 240 272Q240 273 194 273H169Q139 273 139 285Q139 295 153 308T187 332Q206 341 236 342L260 343L264 359Q278 414 289 482T300 578Q300 613 260 613H254Q198 613 169 592Q148 578 127 544T104 508Q72 478 37 475Q18 475 18 487",
      73: "174 0H31Q-13 0 -21 2T-30 12Q-30 23 -17 36Q9 60 42 68L155 70Q187 102 214 179T257 333T302 491T366 610L369 614H305Q221 611 188 607T145 596T128 569Q119 543 94 529T47 512Q28 512 28 524Q28 527 32 539Q56 614 159 654Q218 678 312 682Q314 682 339 682T404 682T481 683H632Q642 678 642 671Q642 657 621 641T577 617Q570 615 507 614H444Q427 592 406 542Q382 478 355 366T310 209Q280 123 238 78L230 69H330Q442 70 442 74Q443 74 443 77T447 87T460 105Q490 134 527 137Q545 137 545 125Q545 120 542 112Q531 78 491 49T399 7Q379 2 360 2T174 0",
      74: "148 78Q148 16 189 -17T286 -50Q319 -50 348 -33T396 10T426 59T444 101L471 204Q498 306 521 372Q575 532 649 605L659 614H591Q517 613 494 607Q433 591 400 550T360 477Q353 454 325 437T275 419Q256 419 260 435Q280 523 376 597T583 681Q603 683 713 683H830Q839 674 839 671Q839 654 810 634T754 614Q735 614 721 601Q688 571 654 495T600 351T561 209T541 132Q507 29 412 -45T213 -119Q141 -119 94 -77T47 33Q47 55 50 69T58 90T71 103Q105 131 135 131Q152 131 152 120Q152 119 151 114T149 99T148 78",
      75: "194 618Q193 618 182 613T156 601T131 594Q113 594 113 605Q113 623 144 644Q154 650 205 676T267 703Q277 705 279 705Q295 705 295 691Q295 569 250 397Q225 306 197 217T151 81T128 25Q120 8 94 -7T47 -22Q32 -22 32 -10L64 76Q95 163 133 295T185 530Q198 611 194 618ZM331 429Q331 383 364 290T449 117T542 36Q574 36 607 51T652 103Q660 124 677 133T709 143Q727 143 727 128Q727 119 723 111Q704 56 639 17T497 -22H493Q463 -22 425 16Q401 40 382 71Q335 138 296 243T256 399Q256 434 288 473Q342 540 471 622T670 705Q691 704 703 696Q732 678 732 644Q732 613 714 600T677 586Q671 586 667 587T660 592T657 604V619Q657 647 629 647Q623 647 620 646Q576 635 495 583T365 482Q331 448 331 429",
      76: "62 -22T47 -22T32 -11Q32 -1 56 24T83 55Q113 96 138 172T180 320T234 473T323 609Q364 649 419 677T531 705Q559 705 578 696T604 671T615 645T618 623V611Q618 582 615 571T598 548Q581 531 558 520T518 509Q503 509 503 520Q503 523 505 536T507 560Q507 590 494 610T452 630Q423 630 410 617Q367 578 333 492T271 301T233 170Q211 123 204 112L198 103L224 102Q281 102 369 79T509 52H523Q535 64 544 87T579 128Q616 152 641 152Q656 152 656 142Q656 101 588 40T433 -22Q381 -22 289 1T156 28L141 29L131 20Q111 0 87 -11",
      77: "28 9Q28 37 43 63T73 90Q77 90 83 84T103 70T141 57H146Q162 57 178 79T222 167Q266 279 295 371T334 513T349 598T358 651T371 677Q397 705 432 705Q442 705 445 699T452 666Q453 661 453 659Q475 538 509 405T568 207L574 192Q581 178 587 164T594 150Q596 150 635 189T693 248Q765 324 863 438T1024 626T1089 701Q1093 705 1100 705Q1111 705 1111 682Q1111 675 1108 660T1099 611T1086 540Q1041 277 1041 144Q1041 98 1044 75T1050 48T1059 42Q1064 41 1075 46Q1102 61 1121 61Q1137 61 1137 50Q1137 28 1087 0T1000 -29Q983 -29 972 -23T955 -9T945 16T942 45T941 83V96Q941 158 952 256T974 422L985 489Q984 489 939 436T821 300T698 164Q665 128 620 85T568 37Q564 34 558 34Q550 34 546 37T535 54Q512 91 496 127T450 259T389 498L384 518Q349 367 294 223T198 15Q155 -50 117 -50Q87 -50 61 -35T30 -6Q28 2 28 9",
      78: "343 705Q358 705 358 698Q360 696 370 658T411 524T484 319Q536 174 590 82L595 73L615 152Q646 274 683 407Q729 571 752 637T799 727Q852 780 937 788Q939 788 947 788T958 789H962Q979 789 979 765Q979 722 951 692Q942 683 924 683Q888 681 859 672T818 654T803 639Q784 608 708 322T631 15Q631 14 630 15Q630 17 629 15Q628 14 628 12Q621 -4 601 -17T560 -31Q550 -31 546 -28T530 -7Q484 67 458 123T398 272Q352 392 314 514L306 535V534Q306 533 296 488T272 379T234 239T185 100T127 -7T61 -50Q34 -50 4 -34T-27 8Q-27 33 -12 61T18 90Q21 90 36 77T87 57H92Q109 57 123 78T162 173Q206 299 232 417T265 599T276 667Q284 681 304 693T343 705",
      79: "308 428Q289 428 289 438Q289 457 318 508T378 593Q417 638 475 671T599 705Q688 705 732 643T777 483Q777 380 733 285T620 123T464 18T293 -22Q188 -22 123 51T58 245Q58 327 87 403T159 533T249 626T333 685T388 705Q404 705 404 693Q404 674 363 649Q333 632 304 606T239 537T181 429T158 290Q158 179 214 114T364 48Q489 48 583 165T677 438Q677 473 670 505T648 568T601 617T528 636Q518 636 513 635Q486 629 460 600T419 544T392 490Q383 470 372 459Q341 430 308 428",
      80: "37 475Q19 475 19 487Q19 536 103 604T327 682Q329 682 344 682T380 682T421 683H463Q625 683 695 615Q718 591 726 564Q733 547 733 525Q733 412 607 312T321 205H312Q293 205 293 217Q293 224 302 236T333 260T385 274Q558 287 614 407Q633 445 633 477Q633 515 612 543T556 585T481 607T399 614H370L368 603Q352 463 312 312T242 82T202 -13Q190 -33 164 -45T121 -57Q108 -57 108 -45Q108 -40 120 -10T151 73T192 190T233 349T266 539Q267 546 269 565T272 598T274 613H270Q209 613 163 588Q131 572 113 518Q102 502 80 490T37 475",
      81: "114 286Q114 358 151 433T249 569T392 667T558 705Q653 705 713 641T774 460Q774 389 750 322T687 206T600 114T504 46T412 4L399 -2Q542 -62 636 -62Q660 -62 670 -54T686 -27T700 0Q734 34 770 34Q787 34 787 23Q787 -18 720 -74T563 -131Q485 -131 350 -83T145 -34Q127 -34 127 -22Q127 -12 144 5T190 31L200 34L237 35Q386 38 467 79Q550 120 612 210T675 416Q675 510 625 573T484 636Q410 636 346 587T248 469T214 333Q214 306 221 281T243 229T288 188T360 172Q403 172 441 188T490 205Q510 205 510 192Q505 162 432 132T287 102Q206 102 160 155T114 286",
      82: "37 475Q19 475 19 487Q19 503 35 530T83 589T180 647T327 682H374Q387 682 417 682T464 683Q519 683 559 679T642 663T708 625T731 557Q731 481 668 411T504 300Q506 296 512 286T528 257T553 202Q594 105 611 82Q635 47 665 47Q708 47 742 93Q758 113 786 128Q804 136 819 137Q837 137 837 125Q837 115 818 92T767 43T687 -2T589 -22Q549 -22 517 22T467 120T422 221T362 273Q346 273 346 287Q348 301 373 320T436 342Q437 342 446 343T462 345T481 348T504 353T527 362T553 375T577 393Q598 412 614 443T630 511Q630 545 613 566T541 600T393 614Q370 614 370 613L366 584Q349 446 311 307T243 96L213 25Q205 8 179 -7T132 -22Q125 -22 120 -18T117 -8Q117 -5 130 26T163 113T205 239T246 408T274 606V614Q273 614 259 613T231 609T198 602T163 588Q131 572 113 518Q102 502 80 490T37 475",
      83: "554 512Q536 512 536 522Q536 525 539 539T542 564Q542 588 528 604Q515 616 482 625T410 635Q374 635 349 624T312 594T295 561T290 532Q290 505 303 482T342 442T378 419T409 404Q435 391 451 383T494 357T535 323T562 282T574 231Q574 133 464 56T220 -22Q138 -22 78 21T18 123Q18 184 61 227T156 274Q178 274 178 263Q178 260 177 258Q172 247 164 239T151 227T136 218L127 213L124 202Q118 186 118 163Q120 124 165 86T292 48Q374 48 423 86T473 186V193Q473 267 347 327Q268 364 239 389Q191 431 191 486Q191 547 242 600T356 679T470 705Q472 705 478 705T489 704Q551 704 596 682T642 610Q642 566 621 545Q592 516 554 512",
      84: "49 475Q34 475 34 490Q34 552 106 611T261 681Q272 683 507 683H742Q790 717 816 717Q833 717 833 708Q833 682 795 653T714 615Q691 610 588 609Q490 609 490 607L483 580Q476 554 462 496T435 392Q410 289 395 231T363 116T335 34T309 -15T279 -47T242 -64Q231 -68 218 -68Q203 -68 203 -57Q203 -52 211 -38Q224 -7 234 20T251 66T268 123T283 179T304 261T328 360Q342 415 360 488Q380 567 384 582T397 605Q400 607 401 609H302H244Q200 609 188 607T167 596Q145 572 145 541Q145 520 109 498T49 475",
      85: "8 592Q8 616 70 649T193 683Q246 683 246 631Q246 587 205 492T124 297T83 143Q83 101 100 75T154 48Q202 48 287 135T450 342T560 553Q589 635 593 640Q603 656 626 668T669 683H670Q687 683 687 672T670 616T617 463T547 220Q525 137 521 68Q521 54 522 50T533 42L543 47Q573 61 588 61Q604 61 604 47Q599 16 506 -22Q486 -28 468 -28T436 -18T421 18Q421 92 468 258Q468 259 467 257T459 248Q426 206 391 167T303 81T194 6T83 -22Q66 -22 58 -20Q25 -11 4 19T-17 99Q-17 146 8 220T64 358T120 488T146 586Q146 604 141 608T123 613H120Q99 613 72 597T25 580Q8 580 8 592",
      86: "25 633Q25 647 47 665T100 683Q291 683 291 306Q291 264 288 213T282 132L279 102Q281 102 308 126T378 191T464 279T545 381T596 479Q600 490 600 502Q600 527 581 550T523 577Q505 577 505 601Q505 622 516 647T542 681Q546 683 558 683Q605 679 631 645T658 559Q658 423 487 215Q409 126 308 37T190 -52Q177 -52 177 -28Q177 -26 183 15T196 127T203 270Q203 356 192 421T165 523T126 583T83 613T41 620Q25 620 25 633",
      87: "25 633Q25 647 46 665T103 683Q168 683 207 632Q228 608 243 568Q269 485 269 374Q269 324 265 271T256 184L251 150L252 152Q254 153 257 157T264 167T274 180T286 197Q359 293 424 398T519 558T549 616Q549 618 547 624T545 638Q550 654 572 668T615 683Q626 683 632 672T657 595Q726 370 741 128L742 110Q752 122 767 142T823 217T894 321T950 424T976 511Q976 544 958 560T918 577Q906 577 906 602Q906 629 918 651T942 681Q948 683 954 683Q983 683 1008 658T1034 569T999 421T915 257T813 109T724 -3T681 -49Q666 -59 660 -45Q659 -41 657 35T639 233T591 477Q573 551 570 551Q569 551 554 523T507 439T433 315T323 155T182 -25Q160 -52 151 -53Q137 -53 137 -30Q137 -29 148 25T170 168T181 338Q181 424 168 483T131 571T87 609T40 620Q25 620 25 633",
      88: "324 614Q291 576 250 573Q231 573 231 584Q231 589 232 592Q235 601 244 614T271 643T324 671T400 683H403Q462 683 481 610Q485 594 490 545T498 454L501 413Q504 413 551 442T648 509T705 561Q707 565 707 578Q707 610 682 614Q667 614 667 626Q667 641 695 662T755 683Q765 683 775 680T796 662T807 623Q807 596 792 572T713 499T530 376L505 361V356Q508 346 511 278T524 148T557 75Q569 69 580 69Q585 69 593 77Q624 108 660 110Q667 110 670 110T676 106T678 94Q668 59 624 30T510 0Q487 0 471 9T445 32T430 71T422 117T417 173Q416 183 416 188Q413 214 411 244T407 286T405 299Q403 299 344 263T223 182T154 122Q152 118 152 105Q152 69 180 69Q183 69 187 66T191 60L192 58V56Q192 41 163 21T105 0Q94 0 84 3T63 21T52 60Q52 77 56 90T85 131T155 191Q197 223 259 263T362 327T402 352L391 489Q391 492 390 505T387 526T384 547T379 568T372 586T361 602T348 611Q346 612 341 613T333 614H324",
      89: "65 599Q65 618 107 650T204 683Q267 683 312 643T380 533T414 385T424 217Q424 186 423 160T422 123Q426 123 468 170T567 304T650 469Q661 503 661 519Q661 546 639 570Q615 591 583 591Q569 591 569 616Q569 640 582 661T613 683Q624 683 638 679T671 664T702 625T714 558Q714 472 639 329T426 45Q361 -21 282 -82T154 -143Q97 -143 64 -104T31 -20Q31 4 44 25T70 46Q78 46 81 39T87 16T97 -9Q127 -51 182 -51Q184 -51 187 -50H190Q233 -41 314 25Q330 36 330 40Q336 79 336 178Q336 508 223 594Q199 614 158 619L148 620L139 611Q111 586 83 586Q65 586 65 599",
      90: "694 220Q708 220 708 210Q708 195 695 167T658 105T593 42T502 3Q492 1 458 1Q400 1 293 11T150 22Q116 22 92 11T51 0Q37 0 37 10Q37 21 63 44T179 146T367 319L391 343H343L296 344Q285 350 285 358Q285 365 289 372T300 383T313 392T324 398L329 400H450L561 518Q597 558 607 571L621 587H596Q553 589 484 599T383 609Q342 609 326 596T301 555Q294 533 263 514T208 492Q189 492 189 503Q189 510 197 528T215 559Q249 607 318 645T466 683Q504 683 573 673T669 662L690 661Q734 682 748 683Q767 683 767 673Q767 666 746 640Q655 531 555 428L529 400Q529 399 543 399Q604 397 604 366Q604 350 587 337T551 322Q541 322 539 323Q529 328 529 334Q529 339 487 342L470 343L446 320Q272 153 200 96L235 95Q297 95 392 86T533 74H554Q586 116 597 159Q604 179 635 199T694 220"
    }, {}), texCalligraphic$1;
  }
  var texMathit$1 = {}, texMathit = {}, hasRequiredTexMathit$1;
  function requireTexMathit$1() {
    return hasRequiredTexMathit$1 || (hasRequiredTexMathit$1 = 1, Object.defineProperty(texMathit, "__esModule", { value: !0 }), texMathit.texMathit = void 0, texMathit.texMathit = {
      65: [0.716, 0, 0.743],
      66: [0.683, 0, 0.704],
      67: [0.705, 0.021, 0.716],
      68: [0.683, 0, 0.755],
      69: [0.68, 0, 0.678],
      70: [0.68, 0, 0.653],
      71: [0.705, 0.022, 0.774],
      72: [0.683, 0, 0.743],
      73: [0.683, 0, 0.386],
      74: [0.683, 0.021, 0.525],
      75: [0.683, 0, 0.769],
      76: [0.683, 0, 0.627],
      77: [0.683, 0, 0.897],
      78: [0.683, 0, 0.743],
      79: [0.704, 0.022, 0.767],
      80: [0.683, 0, 0.678],
      81: [0.704, 0.194, 0.767],
      82: [0.683, 0.022, 0.729],
      83: [0.705, 0.022, 0.562],
      84: [0.677, 0, 0.716],
      85: [0.683, 0.022, 0.743],
      86: [0.683, 0.022, 0.743],
      87: [0.683, 0.022, 0.999],
      88: [0.683, 0, 0.743],
      89: [0.683, 0, 0.743],
      90: [0.683, 0, 0.613],
      97: [0.442, 0.011, 0.511],
      98: [0.694, 0.011, 0.46],
      99: [0.441, 0.01, 0.46],
      100: [0.694, 0.011, 0.511],
      101: [0.442, 0.01, 0.46],
      102: [0.705, 0.204, 0.307],
      103: [0.442, 0.205, 0.46],
      104: [0.694, 0.011, 0.511],
      105: [0.656, 0.01, 0.307],
      106: [0.656, 0.204, 0.307],
      107: [0.694, 0.011, 0.46],
      108: [0.694, 0.011, 0.256],
      109: [0.442, 0.011, 0.818],
      110: [0.442, 0.011, 0.562],
      111: [0.442, 0.011, 0.511],
      112: [0.442, 0.194, 0.511],
      113: [0.442, 0.194, 0.46],
      114: [0.442, 0.011, 0.422],
      115: [0.442, 0.011, 0.409],
      116: [0.626, 0.011, 0.332],
      117: [0.441, 0.011, 0.537],
      118: [0.443, 0.01, 0.46],
      119: [0.443, 0.011, 0.664],
      120: [0.442, 0.011, 0.464],
      121: [0.441, 0.205, 0.486],
      122: [0.442, 0.011, 0.409]
    }), texMathit;
  }
  var hasRequiredTexMathit;
  function requireTexMathit() {
    if (hasRequiredTexMathit) return texMathit$1;
    hasRequiredTexMathit = 1, Object.defineProperty(texMathit$1, "__esModule", { value: !0 }), texMathit$1.texMathit = void 0;
    var O = requireFontData(), _ = requireTexMathit$1();
    return texMathit$1.texMathit = (0, O.AddPaths)(_.texMathit, {
      65: "85 46Q112 48 132 56T161 73T176 92T185 104Q185 106 353 407T524 709Q527 716 551 716Q568 716 572 712Q573 711 574 710Q576 708 594 384Q613 54 617 52H618Q626 46 672 46H689Q696 41 696 36Q696 13 683 0H670Q639 2 557 2Q526 2 500 2T459 2T441 1Q425 1 425 10Q425 12 427 24Q428 27 429 31T430 36T432 40T434 43T437 45T443 46T450 46Q514 46 514 69Q514 74 511 136L506 209H292L260 152Q222 84 222 74Q222 48 264 46Q280 46 280 35Q280 33 278 21Q275 7 272 4T259 0Q256 0 232 1T159 2Q135 2 109 1T78 0Q58 0 58 10Q58 14 61 26T66 40Q68 46 85 46ZM504 260Q503 263 496 407T486 553L466 520Q446 486 402 406L318 256Q318 255 411 255H504V260",
      66: "57 11Q57 38 69 45L74 46Q78 46 85 46T99 46Q134 47 145 50T162 62Q164 66 233 344T303 626Q303 627 302 629V631Q296 637 241 637H223Q217 642 217 645T219 664Q223 677 229 683H411L593 682L605 680Q616 678 628 675T660 662T694 639T720 601T732 547Q732 519 726 503Q710 452 662 414T556 360L545 357L556 355Q604 346 641 312T678 221Q678 155 622 92T482 8Q459 2 439 2T256 0H154H105Q74 0 66 2T57 11ZM629 549Q628 550 629 557T627 576T619 600T601 622T570 636Q564 637 490 637Q472 637 454 637T424 636T411 636Q399 635 395 622T364 500Q333 377 332 376Q332 374 408 374L485 375L495 377Q547 390 588 437T629 549ZM504 336Q500 337 410 337Q323 337 322 336Q322 334 305 263T270 122T252 51Q252 47 337 46Q346 46 361 46T384 45Q425 45 455 55T515 95Q574 156 574 235Q574 276 555 304T504 336",
      67: "395 -21Q279 -21 215 56T150 244Q150 402 265 543Q339 630 421 667T562 704Q596 704 604 703Q627 698 647 689T679 669T699 649T711 633T716 627L753 665Q790 704 792 704Q793 705 798 705Q812 705 812 698Q812 694 780 561Q744 422 744 421Q742 416 739 415T721 413H705Q699 419 699 426Q701 432 701 444Q705 464 705 493Q705 524 700 551T681 604T643 644T583 659Q480 659 387 570Q321 502 287 397T252 213Q252 123 297 74Q347 24 421 24Q500 24 564 89T653 240Q656 253 659 255T677 257Q700 257 700 248Q700 242 694 222Q681 183 656 143T593 65T504 3T395 -21",
      68: "56 11Q56 38 68 45L72 46Q77 46 84 46T98 46Q133 47 144 50T161 62Q163 66 232 344T302 626Q302 627 302 629L301 631Q295 637 240 637H222Q216 642 216 645T218 664Q222 677 228 683H403Q582 683 589 682Q672 674 723 608T775 440Q775 312 709 209T562 54Q502 14 432 2Q423 1 243 0H148H102Q72 0 64 2T56 11ZM254 51Q254 46 348 46Q395 46 422 50T484 71Q585 121 633 255Q679 396 679 477Q679 522 665 554T629 603T587 626T548 636Q547 636 536 636T510 636T480 637Q420 637 411 636T398 627Q396 623 325 339T254 51",
      69: "248 634Q216 634 214 638Q213 641 213 646Q213 674 224 678Q226 680 481 680H736Q743 676 743 669Q743 665 729 557T713 447Q711 440 690 440H675Q667 445 667 454Q667 455 671 481T675 536Q675 583 658 604T592 632Q574 634 475 634Q439 634 424 633T405 631T399 625Q397 622 367 501T336 377Q336 376 367 376H388Q451 376 477 389Q493 399 503 419T520 462T528 489Q531 493 549 493Q557 493 561 492T566 491T569 488T572 483L539 351Q507 221 503 216Q503 216 500 213H484Q468 213 465 216Q461 219 461 225Q461 228 466 250T472 290Q472 317 452 323T368 330H324Q323 326 289 191T255 51T331 46H360Q413 46 444 49T511 67T570 113Q589 137 608 175T638 242T652 272Q656 274 670 274Q693 274 693 262L648 148Q590 4 588 2Q585 0 323 0H61Q54 4 54 11Q54 27 61 41Q65 46 95 46Q131 47 142 50T159 62L194 200Q229 337 264 477T299 623Q299 630 292 631T248 634",
      70: "299 623Q299 630 292 631T247 634H219Q213 640 213 642T215 661Q218 673 225 680H724Q731 676 731 669Q731 665 717 557T701 447Q699 440 678 440H663Q656 444 656 452Q656 457 659 485T663 538Q663 586 644 607T566 633Q564 633 535 633T486 634H458Q404 634 399 625Q396 620 364 492L332 363H380Q446 365 464 373Q496 389 514 458Q518 477 523 479Q527 480 541 480H556Q563 475 563 470Q563 467 532 339T498 207Q496 200 475 200H460Q453 207 453 212Q454 213 456 225T461 254T464 278Q464 304 445 310T369 317H321L289 190Q257 66 257 59Q257 54 261 52T283 48T337 46Q348 46 352 46T360 45T366 42T368 37Q368 32 365 23Q360 4 355 2Q352 1 342 1Q336 1 297 1T199 2Q138 2 106 2T71 1H68Q54 1 54 11Q54 38 66 45L70 46Q75 46 82 46T96 46Q131 47 142 50T159 62L194 200Q229 337 264 477T299 623",
      71: "632 -1Q629 -1 622 5T604 25T583 53Q508 -22 394 -22Q287 -22 219 52T150 244Q150 402 265 543Q339 630 421 667T562 704Q596 704 604 703Q627 698 647 689T679 669T699 649T711 633T716 627L753 665Q790 704 792 704Q793 705 798 705Q812 705 812 698Q812 694 780 561Q744 422 744 421Q742 416 739 415T721 413H705Q699 419 699 426Q701 432 701 444Q705 464 705 493Q705 524 700 551T681 604T643 644T583 659Q480 659 387 570Q319 501 286 394T252 208Q252 156 269 118T314 61T369 33T425 24Q470 24 509 46T566 104Q571 116 583 162T595 214Q595 222 583 223Q561 227 517 227H495Q488 230 488 238Q488 254 495 268Q500 273 511 273Q515 273 532 273T581 272T649 271Q731 271 752 273H761Q767 267 767 264T765 246Q761 233 755 227H742Q698 227 693 213L639 4Q636 -1 632 -1",
      72: "61 0Q54 7 54 11Q54 27 61 41Q65 46 95 46Q131 47 142 50T159 62Q161 66 230 344T300 626Q300 627 300 629L299 631Q293 637 238 637H220Q214 642 214 645T216 664Q220 677 226 683H239Q307 681 372 681Q480 681 486 683H496Q502 677 502 674T500 656Q496 643 490 637H472Q418 637 406 630Q400 627 396 612T367 500Q360 474 352 442T340 395L336 380Q336 378 466 378H596Q657 622 657 626Q657 627 656 629V631Q650 637 595 637H577Q571 642 571 645T573 664Q577 677 583 683H596Q664 681 729 681Q837 681 843 683H853Q860 676 860 672Q858 647 848 637H819Q783 636 772 634T756 623Q753 618 684 340T614 57Q614 50 621 49T666 46Q697 46 699 40Q701 37 698 21Q693 3 689 1Q686 0 677 0Q673 0 657 0T611 1T546 2Q453 2 428 0H418Q411 7 411 11Q411 27 418 41Q422 46 452 46Q488 47 499 50T516 62Q517 64 550 196T584 331Q584 332 454 332H324L291 197Q257 64 257 56Q257 50 265 49T309 46Q340 46 342 40Q344 37 341 21Q336 3 332 1Q329 0 320 0Q316 0 300 0T254 1T189 2Q96 2 71 0H61",
      73: "235 637Q217 637 213 638T209 649Q209 673 220 682Q222 683 237 683Q278 681 369 681Q404 681 441 682T483 683Q499 683 503 681T508 672Q508 670 505 658T500 643Q498 637 464 637Q425 635 415 633T398 621Q396 618 327 340T257 58T260 52T278 48T322 46Q349 46 349 36Q349 31 346 22Q342 4 337 1Q336 1 334 1T329 0Q325 0 307 0T258 1T190 2Q95 2 67 0H56Q49 7 49 11Q51 38 62 46H91Q129 47 141 50T159 62Q161 66 230 344T300 625Q300 637 235 637",
      74: "361 637Q333 637 331 641Q330 644 330 649Q330 673 341 682Q343 683 360 683Q405 681 500 681Q551 681 577 681T607 682Q622 682 622 673Q622 665 616 647Q614 640 610 639T587 637Q547 636 541 624Q538 618 477 374T413 124Q391 64 333 22T207 -21T109 12T78 86Q78 130 101 150T149 170Q193 170 196 129Q196 100 178 80T137 58L132 57Q134 52 138 46T160 28T205 16Q242 16 273 48T319 122Q322 129 383 371T444 627Q444 637 361 637",
      75: "668 621Q668 635 645 637Q629 637 629 648Q629 650 632 662T637 677Q640 682 653 682Q657 682 688 681T764 680Q786 680 810 681T839 682Q859 682 859 672Q859 655 852 643Q849 637 839 637Q804 637 768 621T717 595T697 578Q696 578 600 497L505 417L508 408Q543 311 574 227T618 106T632 69Q645 47 688 46H707Q713 38 713 37T710 19Q706 6 700 0H688Q659 2 587 2Q557 2 532 2T492 2T474 1Q458 1 458 10Q458 13 460 23Q464 39 466 42T480 46Q526 46 526 72Q526 75 476 213L427 350Q426 350 396 325T334 272T302 242Q302 241 299 230T290 194T279 150Q257 61 257 55Q257 50 265 49T309 46H337Q343 40 343 38T341 19Q337 6 331 0H316Q280 2 190 2Q158 2 131 2T89 2T70 1Q54 1 54 11Q54 38 66 45L70 46Q75 46 82 46T96 46Q131 47 142 50T159 62Q161 66 230 344T300 626Q300 627 300 629L299 631Q293 637 238 637H220Q214 642 214 645T216 664Q220 677 226 683H239Q307 681 372 681Q386 681 414 681T464 682L487 683H496Q502 677 502 674T500 656Q495 641 491 637H462Q426 636 415 634T399 623Q396 618 358 467L320 314Q321 314 484 452Q510 474 552 509Q625 570 646 590T668 621",
      76: "61 0Q54 4 54 11Q54 27 61 41Q65 46 95 46Q131 47 142 50T159 62Q161 66 230 344T300 626Q300 627 300 629L299 631Q293 637 238 637H220Q214 642 214 645T216 664Q220 677 226 683H240Q275 681 371 681Q407 681 438 681T487 682T509 682Q527 682 527 674Q527 670 524 659Q523 657 523 654T522 649T520 645T519 642T517 640T515 639T512 638T507 637T502 637T494 637H478Q433 636 417 633T396 617Q395 614 325 334T255 51Q255 47 319 47Q387 47 410 52Q459 65 494 98T543 163T570 229T589 273H622Q628 264 628 262Q628 259 584 133T539 5Q537 1 511 1Q482 0 296 0H61",
      77: "72 1Q58 1 58 11Q58 33 66 41Q68 46 87 46Q150 50 168 84Q172 91 238 356T304 626Q304 627 304 629L303 631Q297 637 242 637H224Q218 642 218 645T220 664Q224 677 230 683H326H394Q417 683 422 682T429 676L480 127L502 163Q523 199 560 262T642 400Q801 668 805 676L817 683H1003Q1010 678 1010 672Q1010 650 997 637H979Q915 637 906 623Q903 618 834 340T764 57Q764 50 771 49T817 46H845Q851 38 851 37T848 19Q845 7 838 0H824Q789 2 701 2Q670 2 644 2T603 2T584 1Q569 1 569 11Q569 13 571 25Q576 42 581 45L586 46Q590 46 597 46T611 46Q646 47 657 50T674 62L813 616L634 313Q453 7 452 7Q446 0 428 0Q409 0 407 6Q405 10 379 304T351 604L285 348Q220 83 220 82Q220 65 233 57T279 46H295Q301 38 301 37T298 19Q295 7 288 0H275Q244 2 171 2Q145 2 123 2T88 2T72 1",
      78: "633 637Q624 637 621 639T617 650Q617 670 630 683H641Q682 680 736 680Q836 680 845 683H853Q860 676 860 672Q858 647 848 637H832Q764 633 750 598Q746 590 673 300Q656 230 638 156T610 44L600 7Q598 0 576 0H559Q553 6 448 297L342 588Q341 588 279 336T216 81Q216 49 274 46Q293 46 295 40Q297 37 294 21Q293 19 292 16T291 11T290 7T289 4T287 2T284 1T280 1T275 0T263 0T229 1T167 2Q91 2 70 0H61Q54 7 54 11T57 27Q61 41 64 43T83 46Q146 50 164 84Q167 90 235 362L303 634Q300 635 253 637H220Q214 643 214 645T216 664Q220 677 226 683H314Q386 683 397 683T410 677Q412 675 501 428L591 179Q591 178 592 180T595 189T600 209T610 246T624 303T645 385Q698 595 698 606Q698 618 683 627T633 637",
      79: "149 237Q149 326 186 413T282 563T412 665T552 704Q619 704 667 678T741 611T777 528T788 444Q788 328 728 219T572 44T377 -22Q275 -22 212 50T149 237ZM688 487Q688 570 650 617T548 665Q467 665 398 592T291 413T253 203Q253 119 290 70T387 20Q440 20 489 52T573 135T635 249T675 373T688 487",
      80: "62 0Q55 7 55 11Q55 27 62 41Q66 46 96 46Q132 47 143 50T160 62Q162 66 231 344T301 626Q301 627 300 629V631Q294 637 239 637H221Q215 642 215 645T217 664Q221 677 227 683H404H431H502Q578 683 615 675T684 636Q729 595 729 531Q729 462 671 396T524 308Q499 302 404 301H318L288 182Q258 63 258 55T310 46Q341 46 343 40Q345 37 342 21Q337 3 333 1Q330 0 321 0Q317 0 301 0T255 1T190 2Q97 2 72 0H62ZM626 555V562Q626 620 552 635Q546 636 481 637Q466 637 450 637T423 636T412 636Q401 635 398 627Q396 622 361 484Q353 452 344 416T330 362L325 344Q325 342 390 342H427Q523 342 567 386Q596 415 611 473T626 555",
      81: "460 -107Q460 -72 464 -42T468 -7L457 -10Q446 -14 424 -18T379 -22Q276 -22 213 50T149 237Q149 326 186 413T282 563T412 665T552 704Q664 704 726 631T788 442Q788 305 703 180Q627 64 517 13L506 8Q506 7 508 -12T513 -38T522 -59T538 -79T565 -85Q604 -85 634 -59T672 0Q676 11 684 11Q693 11 695 2Q695 -1 690 -20T673 -69T644 -126T599 -174T538 -194Q464 -194 460 -110Q460 -108 460 -107ZM689 481Q689 578 646 621T551 665Q468 665 391 586Q321 512 285 399T249 202Q249 106 295 58Q310 41 314 41Q315 41 315 46Q315 83 344 118T420 154Q450 154 473 135Q493 114 500 69L502 58L512 65Q571 110 613 192T672 348T689 481ZM472 49Q472 118 415 118Q393 118 373 98T353 51Q353 18 386 18H387Q424 18 472 40V49",
      82: "62 0Q55 7 55 11Q55 27 62 41Q66 46 96 46Q132 47 143 50T160 62Q162 66 231 344T301 626Q301 627 300 629V631Q294 637 239 637H221Q215 642 215 645T217 664Q221 677 227 683H386Q554 682 569 679Q571 678 580 676Q643 662 680 623T717 533Q717 473 667 420T528 337L538 330Q563 314 578 286T594 228Q594 212 588 147T581 65Q581 36 589 26T616 16H618Q637 16 652 37Q668 57 677 94Q679 105 701 105T723 95Q723 89 717 72T698 33T662 -5T610 -22Q555 -22 513 3T471 88Q471 107 486 168T502 244Q502 303 452 320Q445 322 382 323H320L288 192Q255 63 255 55T307 46Q338 46 340 40Q342 37 339 21Q335 3 330 1Q326 0 320 0Q317 0 306 0T265 1T190 2Q99 2 73 0H62ZM612 558Q612 566 612 568T610 581T603 597T590 611T567 625T532 635Q526 636 470 637Q458 637 445 637T422 636T412 636Q402 635 397 627L390 598Q383 570 373 532T354 455T337 389T330 361Q356 360 384 360H415Q483 360 527 382Q557 399 574 424T604 498Q612 533 612 558",
      83: "198 460Q198 551 269 628T432 705Q516 705 557 644L583 673Q589 679 593 684T600 693T605 698T609 702T611 704T614 705T618 705H620Q633 705 633 698T605 577T573 459L570 456H554Q546 456 543 456T536 457T532 460T531 466Q531 469 533 489T536 532Q536 573 525 600T496 640T462 657T427 662Q369 662 325 612T281 503Q281 475 290 458T318 430T356 415T407 401T463 383Q506 360 522 323T538 258V244Q538 141 465 60T300 -22Q198 -22 152 41L143 31Q137 25 126 12T106 -10T95 -21L92 -22Q88 -22 86 -22Q81 -22 78 -20T74 -16V-14Q74 -11 132 221Q134 227 155 227H171Q177 221 177 215Q177 212 175 205T171 182T169 147Q171 99 195 70T246 33T306 25Q358 25 400 70T453 169Q455 180 455 203V210Q455 263 414 285Q409 288 347 305Q271 328 254 339Q239 350 224 371Q198 409 198 460",
      84: "178 437Q173 442 171 446Q171 451 238 654Q243 670 250 677H681H762Q792 677 799 676T806 667Q806 661 788 553T768 444Q768 437 746 437Q727 437 723 445Q723 450 729 492T736 562Q736 589 728 602T693 624Q675 630 622 630H595Q575 630 571 629T564 623Q562 621 492 342T422 59Q422 48 502 46H542Q548 38 548 37T545 19Q541 6 535 0H517Q475 2 357 2Q315 2 279 2T223 2T198 1Q179 1 179 9Q179 14 182 24Q187 42 190 44Q194 46 206 46H232Q289 47 301 49T326 65L395 344Q465 619 465 626Q465 629 462 629Q456 631 411 631Q364 631 336 625T288 597T255 549T224 467Q215 442 210 437H178",
      85: "636 637Q627 637 624 639T620 650Q620 670 633 683H644Q702 681 753 681Q760 681 772 681T796 681T820 682T838 683H845H853Q860 676 860 672Q858 647 848 637H832Q764 633 750 598Q745 588 698 400T648 204Q627 140 584 86Q484 -22 378 -22Q300 -22 247 31T194 167Q194 176 194 182T196 198T200 218T207 248T217 288T231 346T250 422Q300 618 300 626Q300 627 300 629L299 631Q293 637 238 637H220Q214 642 214 645T216 664Q220 677 226 683H239Q307 681 372 681Q480 681 486 683H496Q502 677 502 674T500 656Q496 643 490 637H472Q418 637 406 630Q400 627 394 603T344 410Q299 232 292 198T284 135Q284 102 294 78T322 44T355 29T387 24Q455 24 515 74T604 211Q605 215 653 404T701 607Q701 618 686 627T636 637",
      86: "667 637Q657 637 654 639T650 650Q650 670 663 683H675Q704 681 772 681Q793 681 818 682T847 683Q868 683 868 672Q868 670 865 658T860 643Q857 637 848 637Q785 637 749 587L394 -15Q387 -22 366 -22Q346 -22 342 -16Q341 -13 313 303Q285 622 285 623Q283 631 273 634T229 637Q205 637 205 648Q205 654 208 666T217 682Q219 683 230 683Q276 680 329 680Q444 680 456 683H466Q472 677 472 674T470 656Q466 643 460 637H448Q384 637 384 615Q385 612 406 371T427 126Q427 125 495 240T632 473T704 596Q707 604 707 609Q707 633 667 637",
      87: "234 637H226Q205 637 205 648Q205 673 216 682Q218 683 231 683Q265 681 340 681Q371 681 404 682T443 683Q458 683 462 681T467 672Q467 670 464 658T459 643Q457 637 434 637Q407 636 394 632T378 623T376 613Q376 589 385 377T394 149L511 361Q542 419 596 519L613 551L612 585Q610 621 610 624Q608 637 559 637H555Q537 637 537 647Q537 654 540 664Q544 677 550 683H561Q600 680 656 680Q771 680 783 683H792Q798 677 798 675T796 658Q792 643 790 640T778 637H774Q721 637 708 620L717 385Q726 150 727 149Q727 148 752 193T812 303T882 433T942 546T969 596Q970 600 970 606Q970 610 969 613T966 620T961 625T955 628T949 631T941 633T934 634T927 636T920 637Q903 637 903 648Q903 650 905 664Q909 677 915 683H928Q960 681 1031 681Q1050 681 1073 681T1101 682Q1124 682 1124 672Q1124 655 1117 643Q1114 637 1104 637Q1085 637 1069 632T1043 618T1026 603T1014 588L1009 580L687 -16Q681 -22 660 -22Q643 -22 637 -16Q635 -14 627 223Q617 441 617 464L602 441Q578 397 487 228Q456 171 423 110T372 17T355 -15Q348 -22 328 -22Q312 -22 308 -20T303 -9Q303 -5 291 310T277 627Q273 636 234 637",
      88: "684 0Q670 0 634 1T569 2Q512 2 482 2T449 1Q433 1 433 10Q433 11 435 25Q437 34 438 37T442 43T448 45T459 46T476 49Q506 58 506 64Q506 65 467 179T426 295L382 244Q339 194 295 142T249 86Q245 79 245 72Q245 48 279 46Q293 46 293 32Q293 13 280 0H268Q206 2 151 2Q70 2 64 0Q50 0 50 11Q50 15 53 27Q57 41 60 43T78 46Q154 49 205 100Q207 103 312 225L411 341L407 353Q404 360 381 428T336 560T310 627Q301 636 255 637H229Q223 643 223 645T225 664Q229 677 235 683H246Q288 680 346 680Q462 680 477 683H487Q493 677 493 674T491 656Q488 644 485 641T471 637Q461 635 454 635Q419 626 421 619Q421 617 453 524T486 430T554 509T624 593Q631 604 631 611Q631 622 621 629T598 637Q583 637 583 648Q583 650 585 660Q589 676 591 679T602 683Q606 683 637 682T715 680Q742 680 771 681T804 682Q825 682 825 672Q825 650 817 642Q814 637 797 637Q739 634 700 608Q684 597 659 569T505 389L501 384L557 222Q612 61 616 57Q625 47 671 46Q691 46 697 45T704 36Q704 35 702 23Q701 19 700 14T699 7T696 3T692 1T684 0",
      89: "232 637Q198 637 198 647Q198 651 201 664T210 682Q212 683 223 683Q269 680 325 680Q443 680 455 683H465Q472 676 472 672Q472 656 465 642Q460 637 441 637Q395 634 395 623Q395 621 438 478T482 334T583 460T688 591Q688 593 694 601T700 617Q700 637 668 637H666Q655 637 655 648Q655 654 658 664Q660 672 660 673T663 678T668 682T677 683Q680 683 704 682T776 680Q801 680 828 681T858 682Q875 682 875 673Q875 669 872 657T867 643Q865 637 848 637Q788 634 749 597Q733 581 608 424L487 273L461 170Q454 145 448 118T438 76T434 60Q434 54 436 52T452 48T496 46H514Q520 41 520 38T518 19Q514 6 508 0H495Q427 2 364 2Q350 2 323 2T272 0H250H241Q234 7 234 11Q234 27 241 41Q245 46 275 46Q312 47 323 50T340 64Q340 65 344 79T355 120T368 171L393 274L341 448Q288 622 286 626Q278 636 232 637",
      90: "203 452Q203 455 236 565T270 677Q270 681 274 681Q276 683 488 683H699Q704 678 704 675Q704 663 697 649Q697 647 449 348L201 50L266 49H301Q442 49 495 116Q525 155 556 251Q563 274 565 278T579 282H585Q609 285 609 271Q609 270 570 142T528 8T518 1T466 0H303Q253 0 197 0T131 -1Q112 -1 102 -1T87 1T81 3T80 8Q80 30 89 39Q90 41 204 178T446 470T575 626L584 637H512H504H475Q446 637 426 635T378 624T330 597T289 546T254 467Q247 446 243 444Q239 442 226 442Q203 442 203 452",
      97: "418 53Q418 26 438 26Q466 26 494 131Q500 151 504 152Q507 153 516 153H521Q531 153 534 153T540 150T543 144Q543 141 540 126T529 88T509 43T477 5T434 -11Q404 -11 383 3T354 30T347 48H346Q345 47 342 45T337 40Q282 -11 228 -11Q172 -11 137 34T101 146Q101 260 177 351T333 442Q343 442 352 441T369 437T382 431T393 425T402 417T409 410T414 402T419 396Q423 406 436 414T461 422Q475 422 484 413T494 395Q494 384 459 244T420 88Q418 80 418 58V53ZM397 323Q397 344 382 374T333 405Q302 405 271 372Q249 349 235 316T203 215Q184 135 184 108V100V94Q184 54 207 35Q218 26 235 26Q279 26 330 91Q343 109 346 118T372 217Q397 317 397 323",
      98: "158 683Q163 683 223 688T300 694Q312 694 312 685T279 544Q243 405 243 403L256 412Q268 422 292 432T338 442Q395 442 431 398T467 284Q467 175 393 82T229 -11Q175 -11 142 32T108 142Q108 176 115 207T166 412Q177 458 190 510T209 588T216 616Q216 629 209 632T170 637H149Q143 643 143 645T145 664Q150 683 158 683ZM178 109Q178 27 234 27Q247 27 254 29Q295 44 323 94Q343 129 363 208T384 332Q384 354 382 361Q369 405 332 405Q282 405 228 326L222 317L205 250Q178 142 178 109",
      99: "257 -10Q183 -10 143 37T103 155Q103 257 173 341T337 440Q341 441 348 441H358Q421 441 445 415T469 356Q469 320 450 305T410 289Q392 289 381 299T370 325Q370 362 404 378L414 383Q392 402 365 405Q322 405 285 375T227 294Q217 271 202 213T187 119Q187 27 263 27Q299 27 330 40Q361 51 386 71T424 106T440 121Q444 121 455 110T466 96Q466 92 458 81T432 54T390 24T331 0T257 -10",
      100: "418 54Q418 26 438 26Q466 26 494 131Q500 151 504 152Q507 153 516 153H521H526Q543 153 543 144Q543 143 541 129Q531 91 521 65T487 14T434 -11T383 3T354 30T347 48H346Q345 47 342 45T337 40Q282 -11 228 -11Q172 -11 137 34T101 146Q101 255 174 348T337 441Q354 441 368 437T390 427T404 414T413 404T417 400L471 616Q471 629 464 632T425 637H404Q398 643 398 645T400 664Q405 683 413 683Q418 683 478 688T555 694Q567 694 567 686Q567 676 495 389L419 86Q418 80 418 61V54ZM397 323Q397 329 394 340T385 367T365 394T333 405Q302 405 271 372Q249 349 235 316T203 215Q184 135 184 108V100Q184 71 191 56Q204 26 235 26Q267 26 300 57T344 112Q347 117 372 217T397 323",
      101: "107 166Q107 230 131 283T193 369T270 420T345 441Q346 441 352 441T361 442H364Q409 442 439 418T470 355Q470 270 366 239Q308 223 218 223H205Q189 164 189 125Q189 83 206 55T261 27Q309 27 353 50T426 109Q436 121 440 121T453 111T466 97Q469 92 455 77Q424 41 372 16T258 -10Q184 -10 146 41T107 166ZM416 333T416 354T401 390T360 405Q322 405 292 384T246 336T223 288T215 261Q215 260 240 260Q262 261 276 262T314 266T353 275T384 291T408 317",
      102: "351 608Q351 642 389 661Q368 668 365 668Q358 668 352 664Q349 663 347 661T342 654T337 647T333 637T330 627T327 614T324 601T321 587T318 571L291 432Q291 431 343 431H394Q400 426 400 423T398 404Q394 390 389 386L335 385H282L255 246Q212 20 189 -51Q136 -199 58 -204Q22 -204 0 -185T-23 -134Q-23 -100 -3 -85T37 -69Q54 -69 65 -80T76 -108Q76 -115 75 -119Q70 -134 61 -144T46 -158L39 -161Q39 -162 42 -163T50 -166T61 -168Q77 -168 91 -145Q98 -128 105 -95L125 -1Q145 90 146 98Q149 109 163 180T189 317T202 384T158 385H114Q108 390 108 393T110 412Q113 424 120 431H165Q211 431 211 433Q213 435 219 473T237 561T266 639Q282 667 310 686T367 705Q402 705 426 686T450 635Q450 600 430 585T390 569Q373 569 362 580T351 608",
      103: "103 163Q106 263 173 347T320 441Q322 441 329 441T341 442Q387 439 419 398Q420 399 420 400Q421 402 425 406T440 416T464 422Q476 421 485 413T494 396Q494 386 465 268T407 38T377 -77Q365 -123 310 -164T179 -205Q46 -205 46 -139Q46 -114 64 -97T106 -79Q127 -79 136 -91T146 -115Q146 -127 141 -138T130 -155T124 -162T125 -163Q133 -166 170 -168Q200 -168 217 -162Q242 -153 264 -130T297 -78Q298 -74 305 -46T320 10T327 38Q326 38 317 31T291 15T256 2Q249 1 231 1Q182 1 143 38T103 163ZM398 324Q398 330 395 346T375 383T332 405Q330 405 326 405T320 404Q291 396 263 365Q230 324 208 239T185 115Q185 38 238 38Q258 38 279 50T312 77T336 106L348 122Q349 125 373 223T398 324",
      104: "398 44Q398 26 414 26Q431 26 451 43Q477 71 496 136Q499 148 501 150T515 153H521Q531 153 534 153T541 150T544 143Q544 133 534 105T496 41T432 -8Q424 -10 408 -10Q370 -10 348 12T326 72Q326 93 342 135Q397 288 397 349Q397 367 396 372Q386 405 357 405Q283 405 228 310Q217 290 212 274T180 152Q153 42 148 26T135 3Q121 -11 102 -11Q89 -11 80 -3T69 19L216 616Q216 629 209 632T170 637H149Q143 643 143 645T145 664Q150 683 158 683Q163 683 223 688T300 694Q312 694 312 685Q312 674 277 539Q241 395 241 393Q242 394 249 399T259 407T271 415T285 424T300 431T318 437T338 440T362 442Q423 442 449 410T475 338Q475 290 437 178T398 44",
      105: "234 599Q234 620 251 638T292 656Q306 656 319 647T332 617Q332 594 313 577T273 560Q260 560 247 569T234 599ZM75 287Q75 292 82 313T103 362T142 413T196 441H214Q248 441 270 419T293 357Q292 338 289 330T245 208Q193 72 193 46Q193 26 209 26Q228 26 247 43Q273 71 292 136Q295 148 297 150T311 153H317Q327 153 330 153T337 150T340 143Q340 133 330 105T292 41T228 -8Q220 -10 204 -10Q160 -10 141 15T122 71Q122 98 171 227T221 384Q221 396 218 400T203 405Q175 403 156 374T128 312T116 279Q115 278 97 278H81Q75 284 75 287",
      106: "266 600Q266 622 283 639T322 656Q340 656 352 645T364 616Q364 596 347 578T305 560Q291 560 279 569T266 600ZM75 287Q75 293 86 316T117 369T168 420T236 442Q282 442 304 415T327 358V332L278 134Q269 98 260 60T246 3T236 -36T227 -66T220 -85T213 -101T204 -114Q181 -150 139 -177T46 -204Q8 -204 -12 -186T-32 -140Q-32 -112 -14 -96T27 -79Q48 -79 57 -91T67 -114Q67 -146 38 -166Q42 -168 49 -168Q75 -168 98 -147T130 -108T146 -71Q150 -59 199 138T250 346Q253 359 253 373Q253 405 230 405Q206 405 184 386T149 345T126 301L117 280Q115 278 98 278H81Q75 284 75 287",
      107: "158 683Q163 683 223 688T300 694Q312 694 312 686Q312 679 262 478L211 273Q212 272 219 276T248 299T296 349Q317 371 328 382T360 410T399 434T439 442Q462 442 480 427T498 373Q498 329 479 313T437 296Q420 296 409 305T398 331Q398 357 413 372T445 391Q454 392 448 399Q445 405 431 405Q408 405 377 385Q351 368 314 327T250 261Q243 257 243 254Q249 254 279 243T328 220Q366 192 366 146Q366 131 361 109T355 62V54Q355 26 376 26Q379 26 387 28Q420 36 443 130Q449 151 454 152Q457 153 465 153H470Q484 153 488 152T492 144Q492 141 489 126T476 88T454 42T420 5T372 -11Q331 -11 306 17T280 88Q280 100 283 119T287 146Q287 172 265 190T221 215T198 220Q197 220 173 121Q152 37 148 24T135 3Q121 -11 102 -11Q89 -11 80 -3T69 19L216 616Q216 629 209 632T170 637H149Q143 643 143 645T145 664Q150 683 158 683",
      108: "162 61Q162 26 183 26Q211 26 239 131Q245 151 249 152Q252 153 261 153H266H271Q288 153 288 144Q288 143 286 129Q276 91 266 65T232 14T179 -11Q144 -11 116 12T87 81Q87 96 88 102L216 616Q216 629 209 632T170 637H149Q143 643 143 645T145 664Q150 683 158 683Q163 683 223 688T300 694Q312 694 312 686Q312 676 240 389L164 86Q162 74 162 61",
      109: "81 278Q75 284 75 289Q77 299 89 338Q101 373 114 396T142 428T166 439T186 442H189Q225 440 251 417Q266 401 271 384L275 374L286 386Q342 442 414 442Q428 442 440 440T461 435T479 427T493 418T503 407T511 397T516 387T520 378T523 370L524 366Q546 395 583 418T667 442Q729 442 755 411T782 338Q782 290 743 178T704 45Q704 26 720 26Q773 26 802 136Q805 148 807 150T822 153H828Q838 153 841 153T848 150T851 143Q851 137 843 115T821 63T778 12T715 -10Q671 -10 652 16T632 71Q632 88 668 191T704 349Q704 367 703 372Q693 405 664 405Q637 405 613 393T571 360T547 329T534 309Q523 290 518 274T487 151Q455 24 452 16Q438 -11 408 -11T376 18Q376 26 411 167T447 314Q449 325 449 346Q449 372 444 384Q431 405 408 405Q334 405 276 305Q266 289 262 273T231 151Q199 24 196 16Q182 -11 152 -11T120 18Q120 26 159 182T200 347Q202 361 202 372Q202 405 181 405Q168 405 159 391Q145 374 132 328T117 280T98 278H81",
      110: "449 44Q449 26 465 26Q482 26 502 43Q528 71 547 136Q550 148 552 150T566 153H572Q582 153 585 153T592 150T595 143Q595 133 585 105T547 41T483 -8Q475 -10 459 -10Q421 -10 399 12T377 72Q377 93 393 135Q448 288 448 349Q448 367 447 372Q437 405 408 405Q381 405 357 393T315 360T291 329T278 309Q267 290 262 274T231 151Q199 24 196 16Q182 -11 152 -11T120 18Q120 23 159 181Q199 343 199 346Q202 360 202 372Q202 406 183 406Q163 406 148 374Q142 360 135 338T124 299T117 280T98 278H81Q75 284 75 287Q76 293 78 303T90 341T110 388T141 425T184 442Q195 442 204 441T221 436T235 429T247 421T256 412T262 403T267 394T271 387T273 381L274 378V374L287 387Q342 442 414 442Q474 442 500 410T526 338Q526 290 488 178T449 44",
      111: "103 155Q103 266 185 354T366 442Q435 442 476 394T517 275Q517 169 436 79T255 -11Q194 -11 149 32T103 155ZM187 119Q187 67 209 47T260 26Q290 26 321 47Q354 68 380 113T426 260Q432 291 432 315Q432 361 408 385Q388 405 358 405Q319 405 283 374T227 294Q217 271 202 213T187 119",
      112: "81 278Q75 284 75 287Q93 379 131 417Q154 442 189 442Q222 440 243 423T272 382L280 390Q335 442 389 442Q446 442 482 398T518 284Q518 212 480 137T375 19Q321 -10 291 -10H282H278Q237 -10 204 28L202 32L181 -51Q160 -135 160 -139Q160 -147 205 -148H230Q236 -155 236 -157T233 -175Q230 -187 227 -190T214 -194Q211 -194 202 -194T169 -193T108 -192Q40 -192 21 -194H13Q6 -187 6 -183T9 -167Q13 -153 16 -151T39 -148Q73 -147 78 -136Q82 -128 139 104Q199 337 199 347Q202 362 202 372Q202 406 182 406Q169 406 159 391Q145 374 132 328T117 280T98 278H81ZM221 111Q234 26 286 26Q307 26 336 47T385 116Q398 147 416 217T435 332Q435 354 433 361Q420 405 383 405Q333 405 279 326L273 317L221 111",
      113: "228 -11Q172 -11 137 33T101 147Q101 205 125 266T201 377T318 441Q322 442 333 442Q388 442 420 394L429 403Q439 413 455 423T481 437T494 442Q498 442 501 439T504 434Q504 425 435 149Q364 -135 364 -139Q364 -147 409 -148H434Q440 -155 440 -157T437 -175Q433 -191 429 -193Q425 -194 418 -194Q416 -194 406 -194T372 -193T309 -192Q259 -192 233 -192T204 -193Q190 -193 190 -184Q190 -181 192 -169Q196 -153 199 -151T219 -148Q266 -148 277 -141Q283 -137 305 -51L325 29L316 22Q270 -11 228 -11ZM397 323Q397 329 394 340T385 367T365 394T333 405Q302 405 271 372Q249 349 235 316T203 215Q184 135 184 108V100Q184 71 191 56Q204 26 235 26Q267 26 300 57T344 112Q347 117 372 217T397 323",
      114: "81 278Q75 284 75 289Q77 301 89 339Q122 442 183 442Q219 442 241 425T271 384L283 396Q327 442 384 442Q424 442 454 421T484 362Q484 327 464 312T424 296Q407 296 396 305T385 331Q385 352 394 365T414 384T424 390Q409 405 378 405Q322 405 276 315L268 300L234 161Q200 25 196 16Q182 -11 152 -11T120 18Q120 23 159 181Q199 343 199 346Q202 360 202 372Q202 405 182 405Q164 405 150 377T128 316T117 280Q115 278 98 278H81",
      115: "153 285Q153 349 197 395T311 442Q355 442 386 420T418 356Q418 321 401 308T365 294Q336 294 331 326Q331 336 334 345T343 359T353 368T362 374L366 376Q365 379 362 383T344 396T308 404Q265 404 246 377T226 325T244 289T287 275T339 258T383 212Q395 188 395 163Q395 132 379 95T333 32Q279 -11 207 -11Q154 -11 115 13T76 86Q76 108 83 123T102 145T121 153T135 156Q154 156 164 145T175 117Q175 82 142 66L132 62Q131 62 131 61Q131 57 139 49T166 34T210 26Q250 26 277 44T312 83T321 123Q321 153 301 166T248 185T204 198Q176 211 162 241Q153 258 153 285",
      116: "94 385Q87 392 87 395Q87 399 90 411T95 425Q97 430 103 430T149 431H196L215 511Q218 521 222 539T228 565T234 585T242 603T251 615T264 623T281 626Q311 626 315 597Q315 591 296 513T275 433Q275 431 320 431H366Q373 424 373 420Q373 398 360 385H263L189 86Q188 80 188 61V54Q188 29 201 27Q213 23 229 30Q253 37 276 66T316 138Q321 149 324 151T342 153H347Q364 153 364 146T360 130Q331 63 290 26T202 -11Q158 -11 135 18T111 81Q111 93 129 168T166 314L184 383Q184 385 139 385H94",
      117: "75 287Q75 299 89 333T135 404T205 441Q246 441 269 420T293 357Q292 338 259 245T225 95Q225 26 274 26Q301 26 324 43T358 77T369 99Q369 102 406 249T446 404Q460 431 490 431T522 402Q522 394 485 245T446 89Q443 74 443 56Q443 28 461 26Q487 26 507 86Q524 130 524 146Q524 147 530 153H547Q570 153 570 144Q570 138 561 109T544 62Q530 29 512 12Q492 -11 454 -11Q429 -9 410 2T385 23T376 41L363 28Q350 16 325 3T269 -10Q204 -10 176 25T148 108Q148 161 184 262T221 383Q221 405 206 405Q178 405 158 375T128 313T116 279Q115 278 97 278H81Q75 284 75 287",
      118: "387 386Q387 407 406 425T445 443Q466 443 479 423T492 371Q492 330 473 253Q411 18 307 -8Q298 -10 279 -10Q208 -10 179 26T149 114Q149 162 185 261T221 384Q221 405 206 405Q177 405 157 375T128 313T116 279Q115 278 97 278H81Q75 284 75 287T79 304T93 342T119 388T158 425T210 441H218Q243 441 268 421T293 357Q292 337 259 245T226 102Q226 26 285 26Q328 26 360 72T422 237Q429 265 429 290Q428 324 408 349T387 386",
      119: "591 386Q591 407 610 425T649 443Q670 443 683 423T696 371Q696 327 672 232T621 85Q575 -11 493 -11Q469 -11 449 -6T418 8T398 24T386 38L382 43Q347 -10 293 -10H286Q221 -10 186 21T150 115Q150 164 185 262T221 384Q221 405 206 405Q177 405 157 375T128 313T116 279Q115 278 97 278H81Q75 284 75 287T79 304T93 342T119 388T158 425T210 441H218Q243 441 268 421T293 357Q292 336 260 246T227 108Q227 26 292 26H295Q332 26 361 93L366 103V119Q366 122 367 133T369 150Q372 167 401 282T433 404Q446 431 477 431Q507 431 509 402Q509 396 500 358T474 254T446 140Q444 126 444 104V92Q444 66 459 46T502 26H505Q527 28 545 43T577 88T602 149T623 226Q633 265 633 290Q632 324 612 349T591 386",
      120: "275 356Q275 383 262 394T233 405Q196 405 166 371T121 289Q119 280 116 279T98 278H81Q77 282 76 283T75 288T78 300Q88 332 109 363T153 411Q195 442 235 442Q306 442 333 386Q373 442 427 442Q461 442 487 421T513 364T494 312T453 296Q436 296 425 305T414 331Q414 352 424 366T446 386L456 390Q448 404 421 404H418Q382 404 358 341Q355 332 328 227T298 105Q295 90 295 75Q295 26 339 26Q365 26 388 43T424 82T444 123T451 146L457 153H474Q490 153 493 152T496 144Q496 133 486 110T456 58T404 10T333 -11Q276 -11 237 45Q197 -11 146 -11Q108 -11 83 10T58 67Q58 99 76 117T119 135Q136 135 147 124T158 96Q158 89 157 85Q149 57 125 45L115 41Q125 26 151 26Q171 26 187 45T214 93Q217 102 244 210T273 330Q275 339 275 356",
      121: "75 287Q75 299 89 333T135 404T205 441Q246 441 269 420T293 357Q292 338 259 245T225 95Q225 26 274 26Q325 26 367 93L405 245Q442 393 446 404Q460 431 490 431T522 402Q522 400 416 -24Q389 -102 327 -153T196 -205Q152 -205 122 -181T91 -119Q91 -84 110 -67T152 -49Q170 -49 180 -60T191 -87Q191 -108 174 -128Q167 -134 157 -138T146 -144Q155 -153 159 -156T173 -163T199 -167Q229 -167 255 -149T297 -105T325 -52T342 -6T347 15Q315 -10 269 -10Q204 -10 176 25T148 108Q148 161 184 262T221 383Q221 405 206 405Q178 405 158 375T128 313T116 279Q115 278 97 278H81Q75 284 75 287",
      122: "160 317Q158 317 155 317Q136 317 136 324Q136 351 171 396T260 442Q292 442 321 410T365 375H369Q384 375 404 408L425 441Q427 442 444 442H460Q466 436 466 434Q466 419 426 367Q387 314 294 222T178 105L170 95L181 94Q198 93 236 81T295 68Q328 68 356 89T395 141Q398 150 401 151T419 153Q441 153 441 144Q441 110 394 50T282 -11Q251 -11 221 23T171 57Q157 57 143 47T121 26T104 3T95 -10Q93 -11 76 -11H60Q54 -5 54 -2Q54 3 61 14Q103 88 233 215Q349 329 349 338L302 351Q269 362 247 362Q227 362 212 356T192 342T183 327T178 320Q175 317 160 317"
    }, {}), texMathit$1;
  }
  var texOldstyleBold$1 = {}, texOldstyleBold = {}, hasRequiredTexOldstyleBold$1;
  function requireTexOldstyleBold$1() {
    return hasRequiredTexOldstyleBold$1 || (hasRequiredTexOldstyleBold$1 = 1, Object.defineProperty(texOldstyleBold, "__esModule", { value: !0 }), texOldstyleBold.texOldstyleBold = void 0, texOldstyleBold.texOldstyleBold = {
      48: [0.46, 0.017, 0.575],
      49: [0.461, 0, 0.575],
      50: [0.46, 0, 0.575],
      51: [0.461, 0.211, 0.575],
      52: [0.469, 0.194, 0.575],
      53: [0.461, 0.211, 0.575],
      54: [0.66, 0.017, 0.575],
      55: [0.476, 0.211, 0.575],
      56: [0.661, 0.017, 0.575],
      57: [0.461, 0.21, 0.575],
      65: [0.751, 0.049, 0.921, { ic: 0.068, sk: 0.224 }],
      66: [0.705, 0.017, 0.748, { sk: 0.16 }],
      67: [0.703, 0.02, 0.613, { sk: 0.16 }],
      68: [0.686, 0, 0.892, { sk: 0.0958 }],
      69: [0.703, 0.016, 0.607, { ic: 0.02, sk: 0.128 }],
      70: [0.686, 0.03, 0.814, { ic: 0.116, sk: 0.128 }],
      71: [0.703, 0.113, 0.682, { sk: 0.128 }],
      72: [0.686, 0.048, 0.987, { sk: 0.128 }],
      73: [0.686, 0, 0.642, { ic: 0.104, sk: 0.0319 }],
      74: [0.686, 0.114, 0.779, { ic: 0.158, sk: 0.192 }],
      75: [0.703, 0.017, 0.871, { sk: 0.0639 }],
      76: [0.703, 0.017, 0.788, { sk: 0.16 }],
      77: [0.703, 0.049, 1.378, { sk: 0.16 }],
      78: [0.84, 0.049, 0.937, { ic: 0.168, sk: 0.0958 }],
      79: [0.703, 0.017, 0.906, { sk: 0.128 }],
      80: [0.686, 0.067, 0.81, { ic: 0.036, sk: 0.0958 }],
      81: [0.703, 0.146, 0.939, { sk: 0.128 }],
      82: [0.686, 0.017, 0.99, { sk: 0.0958 }],
      83: [0.703, 0.016, 0.696, { ic: 0.025, sk: 0.16 }],
      84: [0.72, 0.069, 0.644, { ic: 0.303, sk: 0.0319 }],
      85: [0.686, 0.024, 0.715, { ic: 0.056, sk: 0.0958 }],
      86: [0.686, 0.077, 0.737, { ic: 0.037, sk: 0.0319 }],
      87: [0.686, 0.077, 1.169, { ic: 0.037, sk: 0.0958 }],
      88: [0.686, 0, 0.817, { ic: 0.089, sk: 0.16 }],
      89: [0.686, 0.164, 0.759, { ic: 0.038, sk: 0.0958 }],
      90: [0.686, 0, 0.818, { ic: 0.035, sk: 0.16 }]
    }), texOldstyleBold;
  }
  var hasRequiredTexOldstyleBold;
  function requireTexOldstyleBold() {
    if (hasRequiredTexOldstyleBold) return texOldstyleBold$1;
    hasRequiredTexOldstyleBold = 1, Object.defineProperty(texOldstyleBold$1, "__esModule", { value: !0 }), texOldstyleBold$1.texOldstyleBold = void 0;
    var O = requireFontData(), _ = requireTexOldstyleBold$1();
    return texOldstyleBold$1.texOldstyleBold = (0, O.AddPaths)(_.texOldstyleBold, {
      48: "259 460H279Q352 460 403 444T491 378Q528 320 528 219Q528 100 475 45Q417 -17 287 -17Q152 -17 99 44T46 219Q46 246 47 265T57 318T82 376T131 422T210 454Q234 460 259 460ZM389 229V271Q389 351 371 380T280 409Q232 409 208 385Q194 371 190 345T185 229V210Q185 108 194 83Q208 35 287 35Q317 35 336 42Q372 55 380 85T389 204V229",
      49: "119 417Q191 417 240 428T310 450T338 461Q353 461 357 452T361 400Q361 389 361 342T360 254V62H420H456Q483 62 488 58T494 35V29Q494 13 491 7T473 0Q468 0 424 1T295 2T167 1T117 0Q114 0 111 0T107 1Q96 8 96 27V35Q96 54 102 58T140 62H170H229V213Q229 363 228 363Q220 360 196 358T150 356L130 355H93Q84 360 82 365T80 386Q80 404 81 407T95 417H119",
      50: "214 399Q202 399 191 398T175 395T170 392L171 390Q172 389 174 386T178 379T181 369T182 355Q182 328 165 312T123 296Q99 296 82 312T64 356Q64 397 105 428T253 460Q301 460 337 456T411 441T476 403T514 338Q516 328 516 310V304Q516 257 470 227Q456 217 427 205T376 184L354 176Q350 174 325 162T277 139L254 128Q414 129 428 130H439Q450 148 453 166T460 188T486 193H493Q515 193 517 178Q517 171 502 94T484 11Q481 3 472 2T410 0H269Q65 0 62 2Q55 5 53 10T51 32Q51 54 56 58Q60 62 173 131Q296 207 318 224Q368 264 368 308Q368 342 342 363T283 392T214 399",
      51: "80 309Q80 367 131 414T276 461Q388 461 441 417T494 313Q494 265 463 223T373 155L361 151L376 147Q436 132 480 92T525 -15T487 -127T393 -192T274 -211Q179 -211 114 -167T48 -53Q48 -18 69 5T128 29Q168 29 188 5T208 -50Q208 -59 207 -67T203 -81T197 -92T190 -101T183 -109T177 -114T170 -118L167 -121Q160 -125 164 -127Q167 -129 176 -133Q215 -152 268 -152H273Q337 -152 356 -92Q364 -69 364 -14Q364 45 352 74T303 120Q287 125 240 125H230Q201 125 196 127T191 146V152Q192 170 196 173T233 178Q234 178 241 178T252 179T262 181T274 183T285 188T297 195T308 205T319 218T328 235T337 257Q345 280 345 320V330Q345 345 343 355T335 379T312 399T270 407Q226 407 177 377Q226 356 226 310Q226 270 204 253T153 235Q123 235 102 253T80 309",
      52: "180 0Q155 0 124 0T84 -1Q48 -1 40 3T32 27V37Q32 59 34 63T204 265T377 465Q380 468 383 468H387Q391 468 398 468T411 469Q432 469 438 468T449 459Q451 455 451 258V62H489H498Q531 62 536 58T542 31Q542 9 537 5T504 0H489H451V-132H489H498Q530 -132 536 -136T542 -163Q542 -181 538 -187T522 -194Q518 -194 483 -193T378 -192Q281 -192 256 -193L217 -194Q203 -189 203 -169V-163V-158Q203 -140 209 -136T246 -132H265H314V0H180ZM325 62V320L105 63L215 62H325",
      53: "131 29Q159 26 175 12T194 -13T197 -35V-41Q197 -58 195 -66Q191 -76 185 -84T173 -97T161 -105T152 -109L148 -110Q148 -112 158 -121T195 -141T252 -152Q274 -152 280 -151Q335 -137 349 -104T363 9Q363 85 356 114T322 157Q305 166 286 166Q251 166 223 154T182 131T162 109Q154 98 150 96T130 93Q107 93 102 104Q100 107 100 279V371V424Q100 461 110 461Q114 461 123 458T149 450T183 441Q234 429 286 429Q313 429 340 432T387 440T422 449T447 457T458 461Q472 461 472 435Q472 420 470 418Q464 405 438 379T352 325T226 297Q189 297 171 301H168V182Q169 182 174 185T190 194T213 205T248 213T292 217Q391 217 454 159T517 5Q517 -92 444 -151T255 -211Q167 -211 112 -160T57 -44Q57 -12 76 8T131 29",
      54: "48 316Q48 398 70 462T124 562T198 621T274 652T339 660Q377 660 384 659Q497 632 497 532Q497 507 482 487T431 466Q397 466 381 486T365 530Q365 569 405 593Q382 603 381 603Q361 607 350 607Q316 607 279 589T221 533Q199 489 196 385L195 370Q196 371 203 379T217 395T237 410T263 424Q283 431 313 431Q409 431 468 368Q526 310 526 208Q526 131 492 81T405 5Q359 -17 289 -17Q256 -17 227 -11T163 17T105 73T65 170T48 316ZM293 380Q244 380 220 331T196 212Q196 104 216 80Q240 41 292 41Q295 41 301 41T309 42Q338 46 355 62Q372 81 375 108T379 230Q378 314 372 333Q358 375 299 380H293",
      55: "95 210H88Q66 210 64 225Q64 229 82 345T102 465Q106 476 125 476H131Q162 476 162 458Q162 451 213 448Q221 448 238 447T260 446Q261 446 410 444Q436 444 468 444T509 445Q544 445 551 440T558 413V404Q558 395 547 380T478 301L392 202Q354 150 339 81T321 -104V-132Q321 -187 286 -203Q273 -211 255 -211Q236 -211 217 -199T190 -160Q189 -153 189 -130Q194 17 282 156Q309 200 370 270Q404 307 400 307H293Q151 306 146 303Q140 300 132 259T120 215Q115 210 95 210",
      56: "48 164Q48 199 59 228T92 277T128 307T163 329Q159 332 155 336Q135 346 110 382Q80 427 80 477Q80 564 147 620Q202 660 264 660Q266 660 275 660T290 661Q384 661 439 619T494 506Q494 486 489 468T477 437T459 413T440 395T421 380T406 370L410 368Q414 365 421 361T437 351T456 335T476 316T495 291T511 262T522 228T526 189Q526 93 460 38T282 -17Q242 -17 205 -9T130 19T71 75T48 164ZM406 506Q406 545 379 576T283 607Q218 606 193 585T168 536Q168 522 172 513T195 490T224 471T275 443L346 403Q406 446 406 506ZM291 42Q365 42 395 70T425 134V141Q425 170 401 190T319 242Q308 248 302 251Q286 260 266 271T235 288L225 294Q222 292 217 289T198 274T175 249T157 212T148 163Q148 116 179 79T291 42",
      57: "175 -140Q198 -152 236 -152Q294 -152 332 -116Q356 -91 366 -54T379 62V78L376 74Q372 70 366 64T352 50T333 35T308 23Q289 17 262 17Q168 17 108 77T48 235Q48 273 59 317Q81 381 141 421T276 461Q279 461 285 461T295 460Q326 460 354 454T415 426T471 371T510 277T526 136Q526 42 501 -28T432 -136T341 -192T240 -210Q199 -210 169 -201T121 -178T94 -146T80 -112T77 -82Q77 -51 95 -33T143 -15Q170 -15 189 -33T209 -81Q209 -116 175 -140ZM377 244V274Q377 300 376 316T368 352T348 384T312 405Q307 406 283 406Q257 406 238 396T213 376T205 361Q196 341 196 259V233V181Q196 122 211 96T278 69H298Q330 82 345 104Q367 134 376 190Q377 200 377 244",
      65: "761 751Q784 751 784 728V711Q784 570 795 417T820 191Q844 76 865 76Q868 76 902 93T962 112H973Q989 104 989 94Q989 92 987 86Q978 47 846 -11Q812 -25 779 -26Q722 -26 708 9Q688 47 669 161H524L379 162L359 136Q286 43 234 -3T142 -49T71 -19T39 55Q39 92 60 131T103 174Q113 174 117 167T124 149T136 128T166 110Q183 105 190 105Q230 105 341 246Q401 322 453 397T531 514T582 601T611 651H640V663Q640 692 676 718T745 750Q747 750 752 750T761 751ZM658 266Q653 312 649 377T644 489T641 541V556L557 415Q500 325 448 253Q467 261 524 261H568H658V266",
      66: "441 83Q571 83 571 195Q571 246 538 279T466 322T386 333Q378 333 357 330T329 327Q307 327 307 345Q307 354 313 365T347 396T419 430Q546 472 581 498Q594 508 594 535Q594 574 567 589T508 604Q469 604 442 583Q390 546 342 457T265 256Q237 148 186 60Q167 29 144 13Q105 -15 68 -17H65Q40 -17 40 1Q40 7 53 29T87 100T124 210Q162 373 190 575Q188 575 174 567T138 550T98 542Q75 542 75 560Q75 588 123 618Q135 625 203 659T281 696Q299 703 316 703Q339 703 339 685Q339 656 328 583L333 588Q338 592 346 599T367 615T394 634T428 654T467 674T511 690T559 701T611 705Q651 704 681 689Q739 659 739 598Q739 507 595 427L584 421Q585 420 595 416T610 410T626 402T644 392T660 380T677 365T691 347T703 325T710 299T715 268Q717 193 665 130Q622 73 531 28T348 -17Q275 -17 225 10Q206 19 200 24T193 36Q193 57 231 86T311 119H322Q386 83 441 83",
      67: "243 -20Q135 -20 78 48T20 218Q20 280 39 344T98 470T193 580T324 661T488 702H490Q491 702 493 702T498 703T507 703T518 702H526Q599 702 599 631Q599 597 577 550T541 486Q523 463 490 447T431 430Q423 430 419 433Q409 437 409 450Q410 456 432 499T454 567Q454 586 446 594T415 603Q316 603 254 532Q216 487 190 421T164 284Q164 228 181 186T226 122T282 90T340 80Q377 80 405 96T449 131T492 167T553 187H555Q580 187 580 168Q580 153 551 123T476 60T367 4T243 -20",
      68: "107 0Q92 5 92 18Q92 33 113 53T160 86Q170 91 182 94T197 100L206 120Q248 226 273 337T304 501T309 585Q278 585 234 577T179 557Q172 550 166 532T156 509Q140 484 105 466T44 447Q20 447 20 465Q20 482 34 510T76 565Q122 608 173 632Q281 686 447 686H480H517Q692 686 784 631Q885 571 885 450Q885 339 805 239T586 75T286 1Q276 0 187 0H107ZM741 391Q741 424 731 452T694 510T613 558T481 584Q476 584 468 584T457 585L449 586V579Q441 501 425 424T391 292T357 193T330 125T319 100H324Q511 100 628 175Q688 215 714 275T741 391",
      69: "495 516Q485 516 478 520T470 532Q470 537 476 550T482 570Q482 589 465 596T401 603Q344 603 319 582Q295 558 295 519Q295 493 312 474T355 445T407 432T455 427Q477 427 477 409Q477 395 453 371T389 333Q380 330 345 327T279 312T223 272Q181 223 181 176Q181 131 225 107T324 83Q366 83 395 98T448 136T487 167Q517 185 547 187H551Q574 187 574 170Q574 151 535 113T421 36T271 -15Q260 -16 226 -16Q181 -16 152 -9Q104 4 71 33T37 111Q37 140 50 176T106 263T216 356Q215 357 207 362T190 374T172 392T156 419T150 456Q150 521 208 580T341 670T474 702Q475 703 499 703Q528 703 547 701T586 693T615 673T627 637Q627 584 581 550T495 516",
      70: "812 567Q804 567 795 570T786 579Q786 586 647 586H559L558 582Q558 575 539 510T506 407L498 384H775Q788 378 790 368Q790 328 746 298T665 268Q646 268 642 284H457L447 261Q396 150 360 94Q329 46 270 8T149 -30Q123 -30 100 -24T63 -10T37 9T22 26T17 36Q17 59 56 88T135 119Q145 119 149 117T165 104Q187 78 227 72Q232 72 234 76Q245 93 273 145T350 323T424 570L428 586H276Q265 575 239 563T190 551Q180 551 174 556T167 569Q167 580 179 597T213 634T278 668T371 685Q374 686 624 686Q863 685 888 682Q917 678 927 663Q930 658 930 650Q930 624 888 596T812 567",
      71: "50 279Q50 361 88 438T190 570T335 661T503 702H514Q524 703 532 703Q671 703 671 626Q671 580 614 514T495 447Q472 447 472 465Q472 477 499 511T527 562Q527 582 507 592T433 602Q351 602 302 569Q252 535 223 469T194 344Q194 266 237 217T352 168Q401 168 442 205T505 316Q515 345 551 368T622 393H625Q649 393 649 376Q649 371 635 318T612 237Q580 129 540 62T442 -49Q353 -114 264 -114Q259 -114 252 -114L246 -113Q190 -113 142 -107T81 -96Q71 -90 71 -81Q71 -57 110 -30T187 2Q196 2 236 -4T338 -14Q371 -14 377 -9Q410 24 446 113L451 127Q353 68 253 68Q157 68 104 129T50 279",
      72: "42 447Q20 447 20 465Q20 481 47 515T119 589T239 657T392 686Q443 686 463 664T484 616Q484 570 473 506T452 401L441 360Q441 359 550 359H660L663 370Q684 435 716 522T758 624Q771 646 806 666T870 686Q894 686 894 668Q894 667 867 597T804 416T752 218Q737 135 737 93Q737 77 746 65T778 53Q799 53 803 54T814 63Q831 86 864 103T924 120Q946 120 946 100Q945 85 931 63T888 16T806 -27T684 -48H681Q625 -48 603 -10Q593 4 593 29Q593 71 603 131T624 230L634 269Q632 269 624 266Q610 261 600 261T507 259H411L399 222Q344 62 322 21Q301 -7 268 -24T209 -41H207Q187 -41 185 -25Q185 -17 192 2T220 71T261 184Q284 256 284 258Q284 259 227 259H170Q169 259 166 261T162 264T158 266T156 271T155 277Q155 296 184 320T250 356Q259 358 286 359Q312 359 312 360Q314 372 318 385Q332 450 339 526Q339 530 339 535T340 543Q340 586 296 586Q255 586 227 576T188 553T165 523T146 497Q127 476 97 462T42 447",
      73: "56 499Q32 499 32 516Q32 540 64 580T165 649Q241 682 365 685Q366 685 376 685T405 685T445 686T496 686T553 686H732Q746 677 746 668Q746 646 711 620T642 587L572 586H503Q479 546 458 479T424 352T383 224T318 111L309 101L412 100H514L523 109Q567 150 618 153Q644 153 644 135Q644 132 642 124Q629 86 581 52T476 6Q454 2 433 2T216 0Q-11 0 -15 2Q-27 6 -27 18Q-27 37 2 61T59 93Q77 100 142 100H198Q255 177 299 369Q337 513 382 574L391 586H348Q261 586 176 576Q163 543 124 521T56 499",
      74: "286 390Q263 390 263 407Q263 432 293 481T367 566Q511 687 724 687Q738 687 761 687T793 686H923Q937 677 937 668Q937 648 905 623T842 589Q829 587 817 586T802 585T795 583T788 578Q709 506 632 189Q622 153 615 134T588 81T537 17Q482 -39 404 -76T247 -114Q192 -114 158 -100Q53 -61 53 32Q53 59 58 73T79 102Q126 147 177 147Q200 147 200 128Q200 123 198 112T196 96Q196 47 238 17T345 -13Q362 -13 377 -9T404 0T426 16T444 34T459 55T470 76T478 97T483 116T488 132L490 141Q511 222 520 257T554 364T608 486T675 576L685 586H634H612Q532 586 484 564Q453 549 436 526T409 478T395 447Q378 424 345 407T286 390",
      75: "98 542Q75 542 75 560Q75 588 123 618Q132 624 199 657T275 694Q291 703 315 703Q327 703 332 699T338 690T339 670Q339 596 323 505T283 337T237 194T198 90L181 53Q170 31 136 8T68 -17H65Q40 -17 40 0L76 92Q112 185 150 322T194 564V578L168 565Q125 542 98 542ZM834 142Q834 125 819 100T774 48T692 3T576 -16H560Q540 -16 508 6Q469 33 422 108T342 267T309 398Q309 411 310 417T320 442T347 482Q401 542 517 615T710 702Q712 702 721 702T735 703Q772 703 791 690Q819 674 819 646T792 597T733 574H722Q704 584 704 599Q706 607 700 610T672 617L660 613Q609 595 524 538T423 450V440Q423 376 488 247T604 83Q621 70 640 70Q677 70 701 82Q713 87 718 101T737 132T783 160Q792 163 807 163Q834 163 834 142",
      76: "63 -17Q41 -17 41 0Q41 22 85 54Q101 68 113 92T133 141T154 219T182 315Q230 462 306 553Q345 599 391 632T478 678T543 697T582 703Q584 703 589 703T598 702Q643 702 666 676T689 613Q689 588 683 575Q674 551 632 524T552 496Q530 496 530 512Q530 517 531 525T533 538Q533 559 522 577T480 596H476Q462 596 451 588T415 544Q350 447 310 281Q284 181 261 136L255 124H285Q342 123 441 107T583 90L596 89Q603 116 647 144T729 173Q751 173 751 157Q751 118 685 60T523 -15Q514 -16 479 -16Q421 -16 320 0T171 18H155L142 10Q98 -17 63 -17",
      77: "38 20Q38 59 60 99T104 139Q106 139 126 125T176 106H181Q200 106 221 139T286 281Q322 370 342 451T368 581T376 634Q384 657 420 680T487 703Q502 703 507 696T522 649Q538 589 554 537Q579 453 609 372T660 248T686 202Q687 201 739 244T830 322L1166 642Q1225 700 1230 701Q1230 701 1237 703Q1258 703 1258 667L1253 637Q1248 607 1241 558T1227 451T1214 326T1209 202Q1209 77 1232 77Q1237 77 1269 94T1326 112H1329Q1353 112 1353 94Q1353 81 1334 60Q1311 37 1248 7T1150 -24H1141H1135Q1085 -24 1074 26Q1064 75 1064 134Q1064 239 1086 426Q1087 430 1087 434L1061 410Q871 227 783 149L694 76Q653 44 647 40T631 34Q620 34 616 37T594 63Q546 125 514 198Q467 307 423 449L418 466L412 444Q376 310 306 153Q278 88 251 45T201 -18T163 -43T131 -49Q102 -48 70 -31T38 20",
      78: "47 139Q81 105 122 105Q137 105 147 117Q159 134 182 199T234 381T274 610Q275 634 284 647Q297 666 327 684T389 703Q403 703 408 695T428 645Q480 490 567 298Q628 163 673 103Q674 102 674 102T675 106Q732 331 803 551Q842 674 875 725Q908 775 966 807T1081 840H1084Q1105 840 1105 803Q1105 768 1088 733T1051 689Q1045 686 1032 686Q986 683 948 663T901 624Q881 579 837 430T760 154L726 28Q725 28 725 28T723 25Q716 0 682 -24T611 -48Q600 -48 595 -45T576 -23Q522 44 480 124Q417 243 332 463L328 473L325 457Q291 293 227 124Q159 -49 72 -49Q38 -49 5 -28Q-24 -8 -24 21Q-24 58 -3 98T41 139H47",
      79: "433 703Q456 703 456 685Q456 672 441 655T407 627Q402 623 378 611T328 579T276 524Q207 434 207 324Q207 222 270 153T441 84Q566 84 651 177T737 400V405Q737 496 693 549T576 603Q542 603 510 560Q490 537 472 502T442 454Q397 412 346 409Q320 409 320 427Q320 430 322 436Q331 465 360 507T433 594T542 671T677 703Q776 703 829 636T882 468Q882 369 831 277T702 122T528 21T343 -17Q214 -17 139 61T63 257Q63 336 94 409T173 534T272 625T367 684T432 703H433",
      80: "170 -67Q147 -67 147 -49Q147 -42 162 -8T204 99T253 254Q274 332 288 415T305 542L308 585Q277 585 234 577T179 557Q172 550 166 532T156 509Q140 484 105 466T44 447Q20 447 20 465Q20 482 34 510T76 565Q122 608 173 632Q279 686 448 686H495H537Q622 686 678 677T784 637Q846 598 846 533Q846 452 776 375T597 252T378 206H366L358 181Q341 130 316 68T282 -7Q262 -33 230 -50T170 -67ZM701 468Q701 512 661 540T570 577T461 586H448V582Q446 576 443 545T428 447T395 301L389 280Q390 280 398 284T419 295T441 303Q443 304 484 306T572 321T651 359Q701 402 701 468",
      81: "874 453Q874 372 836 298T750 177T638 89T543 33T486 8L483 7Q485 5 523 -7T622 -32T726 -46Q741 -46 746 -45T755 -41T762 -27Q770 -1 806 23T878 50H890Q905 42 905 33Q905 -8 838 -68T670 -145Q662 -146 628 -146Q538 -146 389 -100T164 -50Q132 -50 132 -32T162 11T227 47Q231 48 286 51T394 62T518 100T641 180Q730 271 730 387Q730 478 673 540T520 602Q410 602 337 525T264 355Q264 284 310 244T420 203Q476 203 568 222Q594 222 594 204Q594 184 565 161T508 128Q433 103 316 103Q227 103 174 157T120 290Q120 382 182 471T343 620T548 697Q578 703 601 703Q604 703 611 703T623 702Q663 702 687 696Q760 679 817 618T874 453",
      82: "159 0Q159 5 172 34T205 114T245 229T284 386T309 575V585H304Q303 585 295 585T282 584Q233 579 207 570T175 553T165 531T156 509Q140 484 105 466T44 447Q20 447 20 465Q20 482 34 510T76 565Q122 608 173 632Q279 686 448 686H505H582Q683 686 745 672T834 611Q842 594 842 565Q842 523 824 484T780 419T722 370T669 336T632 318L619 312L626 302Q640 279 667 227T696 172Q717 133 735 112T762 88T784 84Q824 84 872 118T957 153Q981 153 981 136Q981 114 937 78T820 13T684 -17Q646 -17 616 8T569 66T526 151T477 234Q461 256 446 265Q437 272 421 274Q400 274 400 291Q400 311 430 336T495 371Q496 371 543 374T627 392T681 436Q699 467 699 503Q699 550 644 568T471 586H449V582Q449 581 447 559T438 499T422 413T393 298T348 165Q313 73 296 45Q282 24 249 4T185 -17Q159 -17 159 0",
      83: "204 476Q204 525 248 577T372 666T539 703T674 683T721 612Q721 588 714 569Q704 547 669 524T601 499Q573 499 573 516Q573 521 575 527T577 543Q577 563 568 574T548 588L539 590Q490 603 444 603Q418 603 394 597T364 583Q348 567 348 533Q348 493 382 466T459 425T555 387T633 330Q662 292 662 249Q662 153 544 69T257 -16Q218 -16 208 -15Q118 1 64 46Q25 76 25 126Q25 185 82 235T203 290H207Q229 290 231 274Q231 243 180 213Q173 209 172 206T170 189T171 170T183 150T216 121Q273 83 356 83Q412 83 459 100Q493 111 507 141Q518 165 518 185Q518 208 506 228T478 262T437 288T398 306T360 320Q316 335 285 352T239 384T215 416T205 443T204 467V476",
      84: "61 462H59Q38 462 38 479Q38 528 109 594T289 683L304 685L837 687L846 693Q889 720 923 720Q947 720 947 702Q945 671 892 631T776 583Q774 583 772 583T769 582T766 582L764 581H758Q753 581 744 581T722 580T693 580T662 580H563L514 385Q507 355 493 299T475 225T460 172T443 119T426 76T402 24Q386 -11 355 -33T304 -61T266 -69Q242 -69 242 -50Q243 -45 253 -25T278 32T307 115L364 340Q405 511 413 538T436 580H207Q202 572 200 568T197 561T195 552T190 537Q176 511 135 487T61 462",
      85: "124 586Q107 586 74 569T15 552H13Q-10 552 -10 570Q-10 605 70 645T222 686Q283 686 283 631Q283 590 246 504T172 326T135 181Q135 130 157 107T205 83Q221 83 259 106Q347 165 453 301T604 548Q607 557 612 569T619 587T624 600T628 612T632 621T637 628T641 634T647 640T654 645T662 652Q706 686 748 686Q771 686 771 669Q771 656 754 614T700 467T630 229Q615 168 610 105Q610 88 617 78L641 90Q681 111 706 112Q733 112 733 95Q733 82 714 60Q694 40 633 10Q567 -23 532 -24Q507 -24 495 -17Q466 -4 466 32Q466 96 500 225Q277 -17 102 -17Q56 -17 23 17T-10 118Q-10 164 13 234T64 363T115 481T139 567Q139 586 124 586",
      86: "25 608Q25 628 60 657T148 686Q184 683 213 671T273 625T327 538T363 394T380 184L381 134L399 148Q503 226 574 302T667 415T689 467Q688 474 684 482T672 502T645 521T600 532Q576 532 576 567Q576 604 597 644T641 685H649Q701 685 737 648T774 545Q774 457 703 333T461 66Q397 13 332 -32T255 -77Q237 -77 237 -30V-23Q241 20 241 109Q241 483 115 569Q91 586 50 589Q25 589 25 608",
      87: "25 607Q25 629 62 657T142 686Q205 686 248 647T312 541T339 411T347 275Q347 249 345 203V189Q375 219 449 316T587 516Q629 584 629 587Q629 589 626 597T622 607Q622 629 658 656T732 686H744Q755 680 757 678Q757 677 769 649T799 577T835 475T874 339T904 183Q908 157 910 151L925 169Q997 252 1059 343T1121 474Q1120 498 1103 513T1059 532Q1036 532 1036 568Q1036 600 1053 636T1090 683L1097 686H1109Q1147 684 1176 652T1206 551Q1206 460 1131 320T897 7Q859 -33 840 -52T816 -74T804 -77Q788 -77 784 -32Q783 -28 783 -26Q774 108 744 239T691 436T665 501Q664 501 649 475T602 400T528 289T420 146T280 -15Q243 -56 231 -66T210 -77Q191 -77 191 -40Q191 -38 195 -4T204 91T209 217Q209 290 202 351T177 469T126 557T45 589Q25 589 25 607",
      88: "762 562Q762 579 737 584T711 604Q711 630 753 658T834 686Q864 686 885 669T906 627Q906 580 834 522T614 379L584 362V357Q585 354 589 315T597 233T603 183Q610 132 627 116T671 100Q678 100 704 113T754 126T778 107Q776 79 733 45T626 2Q615 1 578 1Q542 1 535 3Q521 7 510 15T491 31T477 54T467 78T460 108T456 137T452 170T449 201Q447 220 445 240T442 270L441 281Q435 281 357 233Q240 165 206 135Q200 128 200 124Q200 113 208 108T226 101T244 96T252 82Q252 61 214 31T129 1H120Q97 1 77 16T56 60Q56 105 133 168T414 345Q428 352 431 354T433 359Q422 493 414 522Q407 551 395 566T373 583T350 586H341L332 580Q290 560 265 560Q243 560 243 577Q243 585 248 596T269 624T306 653T365 676T447 686H456Q472 686 484 683T514 671T543 637T562 576Q565 557 570 501L577 437Q577 436 613 457T694 506T756 551Q762 558 762 562",
      89: "73 555Q49 555 49 573Q49 602 110 644T239 686Q319 686 376 624Q416 584 444 511T483 361T499 240T503 173Q503 165 504 165Q506 165 524 184T556 218Q631 297 674 377T718 485Q718 505 699 526Q673 552 628 552Q619 552 613 562T607 590Q607 617 621 645T658 685Q661 686 671 686Q718 686 757 652T797 545Q797 476 749 369T602 146Q500 29 371 -67T176 -164Q112 -164 74 -120T36 -29Q36 5 55 36T95 67Q104 67 108 59T115 39T128 12T154 -12Q183 -30 216 -30Q239 -30 305 7L361 44L367 49V54Q367 95 364 143T351 273T312 429T243 546Q206 581 156 588L146 581Q108 555 73 555",
      90: "622 574Q522 579 420 579H396Q373 579 364 574T351 550Q339 516 297 490T218 462Q195 462 195 479Q195 487 197 492Q218 565 313 625T509 685Q564 685 650 683T755 680Q787 680 807 683T831 686Q853 686 853 669Q853 657 826 626Q742 532 641 437L619 415L622 414Q626 414 631 414T642 414Q697 411 697 388Q697 367 670 345T607 323Q605 323 592 325T546 329H522L490 302Q457 274 400 226T289 136L260 113L318 112Q345 111 452 109T587 106H627Q650 143 656 170Q666 197 710 225T788 253Q811 253 811 237Q811 211 781 160T710 77Q619 0 515 0Q507 0 497 0T484 1Q434 1 319 3T177 6Q123 6 95 2Q83 2 71 0H68Q46 0 46 17Q46 28 58 44Q68 56 100 80T210 165T383 307L408 329H361L314 330Q297 338 297 350Q297 368 320 388T368 413Q375 415 441 415H506L647 555L664 574H622"
    }, {}), texOldstyleBold$1;
  }
  var texOldstyle$1 = {}, texOldstyle = {}, hasRequiredTexOldstyle$1;
  function requireTexOldstyle$1() {
    return hasRequiredTexOldstyle$1 || (hasRequiredTexOldstyle$1 = 1, Object.defineProperty(texOldstyle, "__esModule", { value: !0 }), texOldstyle.texOldstyle = void 0, texOldstyle.texOldstyle = {
      48: [0.452, 0.022, 0.5],
      49: [0.453, 0, 0.5],
      50: [0.453, 0, 0.5],
      51: [0.452, 0.216, 0.5],
      52: [0.464, 0.194, 0.5],
      53: [0.453, 0.216, 0.5],
      54: [0.665, 0.022, 0.5],
      55: [0.463, 0.216, 0.5],
      56: [0.666, 0.021, 0.5],
      57: [0.453, 0.216, 0.5],
      65: [0.728, 0.05, 0.798, { ic: 0.021, sk: 0.194 }],
      66: [0.705, 0.022, 0.657, { sk: 0.139 }],
      67: [0.705, 0.025, 0.527, { sk: 0.139 }],
      68: [0.683, 0, 0.771, { sk: 0.0833 }],
      69: [0.705, 0.022, 0.528, { ic: 0.036, sk: 0.111 }],
      70: [0.683, 0.032, 0.719, { ic: 0.11, sk: 0.111 }],
      71: [0.704, 0.119, 0.595, { sk: 0.111 }],
      72: [0.683, 0.048, 0.845, { sk: 0.111 }],
      73: [0.683, 0, 0.545, { ic: 0.097, sk: 0.0278 }],
      74: [0.683, 0.119, 0.678, { ic: 0.161, sk: 0.167 }],
      75: [0.705, 0.022, 0.762, { sk: 0.0556 }],
      76: [0.705, 0.022, 0.69, { sk: 0.139 }],
      77: [0.705, 0.05, 1.201, { sk: 0.139 }],
      78: [0.789, 0.05, 0.82, { ic: 0.159, sk: 0.0833 }],
      79: [0.705, 0.022, 0.796, { sk: 0.111 }],
      80: [0.683, 0.057, 0.696, { ic: 0.037, sk: 0.0833 }],
      81: [0.705, 0.131, 0.817, { sk: 0.111 }],
      82: [0.682, 0.022, 0.848, { sk: 0.0833 }],
      83: [0.705, 0.022, 0.606, { ic: 0.036, sk: 0.139 }],
      84: [0.717, 0.068, 0.545, { ic: 0.288, sk: 0.0278 }],
      85: [0.683, 0.028, 0.626, { ic: 0.061, sk: 0.0833 }],
      86: [0.683, 0.052, 0.613, { ic: 0.045, sk: 0.0278 }],
      87: [0.683, 0.053, 0.988, { ic: 0.046, sk: 0.0833 }],
      88: [0.683, 0, 0.713, { ic: 0.094, sk: 0.139 }],
      89: [0.683, 0.143, 0.668, { ic: 0.046, sk: 0.0833 }],
      90: [0.683, 0, 0.725, { ic: 0.042, sk: 0.139 }]
    }), texOldstyle;
  }
  var hasRequiredTexOldstyle;
  function requireTexOldstyle() {
    if (hasRequiredTexOldstyle) return texOldstyle$1;
    hasRequiredTexOldstyle = 1, Object.defineProperty(texOldstyle$1, "__esModule", { value: !0 }), texOldstyle$1.texOldstyle = void 0;
    var O = requireFontData(), _ = requireTexOldstyle$1();
    return texOldstyle$1.texOldstyle = (0, O.AddPaths)(_.texOldstyle, {
      48: "39 213Q39 274 53 319T89 389T139 429T192 448T242 452Q347 452 400 400Q460 335 460 213Q460 -22 250 -22Q39 -22 39 213ZM364 242Q364 279 363 301T355 348T338 385T306 406T254 415Q250 415 245 415T238 416Q217 416 190 404T150 368Q136 339 136 221Q136 114 146 78T200 23Q219 16 250 16Q280 16 299 23Q341 41 352 77T364 221V242",
      49: "116 410Q176 412 215 423T269 443T288 453H291Q293 453 301 447V254Q301 62 303 60Q307 52 322 49T394 46Q418 46 422 43T426 23Q426 8 424 4T411 0Q407 0 369 1T258 2T148 1T106 0Q96 0 94 4T91 23Q91 40 95 43T123 46Q180 46 195 49T215 61Q216 63 216 220V376Q192 367 119 364H93L86 371V403L92 410H116",
      50: "55 334Q55 386 105 419T236 453Q333 453 390 413T448 307Q448 278 437 256T406 218T365 193T318 172T277 151L248 134Q219 118 191 102T163 84T267 83L382 85H391Q399 99 406 126Q410 143 413 145T429 148Q440 148 442 147T449 139Q449 137 435 73T420 7Q420 6 414 0H233Q94 0 71 0T46 5Q46 5 46 6Q44 8 44 24Q44 39 46 41Q47 44 98 78T212 155T294 212Q347 257 347 304Q347 354 306 380T203 407Q150 407 120 377Q118 375 123 373Q146 362 146 332Q146 315 133 302T101 288Q85 288 70 298T55 334",
      51: "69 312Q69 377 122 414T233 452Q333 452 381 409T430 313Q430 268 402 223T311 149L301 144Q307 143 317 140T355 123T402 92T439 44T456 -25Q456 -101 396 -158T241 -216Q154 -216 98 -173T42 -68Q42 -58 44 -50T50 -35T57 -25T66 -17T75 -13T82 -10L87 -8Q92 -7 102 -7Q132 -7 147 -25T162 -66Q162 -112 118 -125L113 -126Q113 -129 127 -140T173 -162T239 -173Q268 -173 292 -158Q345 -124 345 -24Q345 33 329 67Q298 125 229 125H210H172Q166 131 166 142Q166 159 172 161Q178 161 208 164T244 169Q251 169 263 176T291 199T316 245T327 314Q327 413 238 413Q182 413 130 369Q177 350 181 312Q181 289 166 273T125 257Q102 257 86 272T69 312",
      52: "163 0Q139 0 109 0T71 -1Q43 -1 36 2T28 15V27V46L190 270Q325 457 330 462Q333 464 352 464H371L377 458V46H464L471 40V6L464 0H377V-65V-108Q377 -131 379 -137T391 -144Q409 -148 446 -148H464Q468 -151 471 -155V-187L464 -194H453Q395 -192 325 -192Q222 -192 210 -194H199L193 -188V-154L199 -148H228Q241 -148 250 -148T265 -146T275 -145T281 -143T284 -141T286 -138T289 -134V0H163ZM295 46V350L75 46H295",
      53: "159 -44Q159 -66 145 -80T109 -96H102L107 -105Q148 -173 228 -173Q255 -173 280 -162Q351 -128 351 -6V8Q351 67 344 98T316 151Q288 176 255 176Q175 176 136 109Q129 100 114 100Q97 100 95 106Q93 110 93 277V403Q93 451 98 451Q100 452 103 452Q105 452 124 445T177 431T251 423Q294 423 328 430T380 445T401 453Q410 453 410 435V422Q332 331 203 331Q152 331 140 339Q139 339 139 254V168Q194 214 256 214Q332 214 390 154T448 0Q448 -95 381 -155T229 -216Q153 -216 104 -166T50 -49Q50 -15 66 -1T105 13Q128 13 143 -3T159 -44",
      54: "42 313Q42 401 68 472T133 583T215 644T296 665H304Q317 665 329 664T360 657T393 640T418 608T432 557Q432 533 422 519T401 502T380 498Q358 498 343 512T328 550Q328 591 367 601L372 602Q372 604 365 609T341 620T307 626Q260 626 226 600T174 537Q147 483 143 376V356Q192 434 266 434Q317 434 357 409Q406 375 435 317Q456 268 456 210V192Q456 169 451 149Q440 90 387 34T253 -22Q225 -22 199 -14T143 16T92 75T56 172T42 313ZM251 396Q211 396 178 354T145 217Q145 159 152 122T166 73T187 47Q216 21 252 21Q294 21 321 47Q342 68 349 99T356 203V232Q356 264 354 285T345 331T322 373T280 395Q274 396 251 396",
      55: "75 246Q64 246 62 247T55 255Q55 259 72 357T90 458L94 462H99Q104 463 109 463H121Q128 460 128 452Q128 445 134 441T165 435T233 432T356 431H478L485 425V392L414 298Q408 290 385 260T358 223T337 191T317 154T302 116T289 68T282 14T275 -55T274 -137Q274 -173 272 -177Q262 -216 223 -216Q170 -216 170 -157V-148Q172 -78 189 -12T237 109T288 196T338 266Q345 275 348 279Q395 340 395 342Q396 343 376 343T274 343Q122 342 117 339T103 294T90 248Q88 246 75 246",
      56: "69 496Q69 570 124 618T247 666Q321 666 375 624T429 515Q429 468 405 433T320 361L346 344Q385 320 403 301T439 250Q456 212 456 181V172V160Q456 141 449 119T426 71T386 26T326 -8T246 -21Q160 -21 102 30T43 155Q43 265 167 332L176 337L161 347Q105 384 87 423Q69 458 69 496ZM371 513Q371 567 334 596T249 626Q198 626 163 598T127 532Q127 521 129 511T138 492T150 476T167 460T185 447T207 433T228 420L284 384L294 391Q346 424 363 469Q371 486 371 513ZM190 21T250 21T351 56T393 140Q393 180 362 213Q354 221 317 246T246 292L212 313Q210 313 200 307T173 287T142 256T117 212T106 157Q106 100 148 61",
      57: "171 -101Q171 -118 163 -130T146 -146T134 -151Q132 -151 132 -152Q132 -154 140 -159T167 -168T206 -173Q274 -173 317 -108Q356 -50 356 79V86L350 77Q308 9 231 9Q150 9 92 81Q42 141 42 228Q42 289 64 333Q93 390 142 421T235 452Q237 452 244 452T255 453Q289 453 321 439T386 391T437 290T457 128Q457 -29 381 -122T206 -216Q148 -216 108 -187T67 -104Q67 -75 84 -62T119 -49Q141 -49 156 -63T171 -101ZM242 46Q293 46 324 94T355 223Q355 323 337 356Q316 401 275 410Q267 412 248 412Q206 412 179 386Q155 360 149 328T143 224Q143 120 163 88Q192 46 242 46",
      65: "576 668Q576 688 606 708T660 728Q676 728 675 712V571Q675 409 688 252Q696 122 720 57Q722 53 723 50T728 46T732 43T737 41T743 39L754 45Q788 61 803 61Q819 61 819 47Q818 43 814 35Q799 15 755 -7T675 -30Q659 -30 648 -25T630 -8T621 11T614 34Q603 77 599 106T594 146T591 160V163H460L329 164L316 145Q241 35 196 -7T119 -50T59 -24T30 43Q30 75 46 100T74 125Q81 125 83 120T88 104T96 84Q118 57 151 57Q189 57 277 182Q432 400 542 625L559 659H567Q574 659 575 660T576 668ZM584 249Q579 333 577 386T575 473T574 520V581L563 560Q497 426 412 290L372 228L370 224H371L383 228L393 232H586L584 249",
      66: "304 342Q292 342 292 353Q292 372 323 391Q331 396 417 428T533 487Q563 512 563 555V562Q563 575 557 589T530 618T475 636Q429 636 396 613T330 539Q263 446 210 238Q196 183 173 120Q135 31 121 16Q108 1 85 -10T47 -22T32 -10Q32 -5 44 18T77 93T112 206Q135 296 154 395T182 550T191 615Q191 616 190 616Q188 616 179 611T157 601T131 594Q113 594 113 605Q113 623 144 644Q154 650 205 676T267 703Q277 705 279 705Q295 705 295 693Q295 686 288 635T278 575Q278 572 287 582Q336 635 402 669T540 704Q603 704 633 673T664 599Q664 559 638 523T580 462Q553 440 504 413L491 407L504 402Q566 381 596 338T627 244Q627 172 575 110T444 13T284 -22Q208 -22 158 28Q144 42 146 50Q150 67 178 85T230 103Q236 103 246 95T267 75T302 56T357 47Q436 47 486 93Q526 136 526 198V210Q526 228 518 249T491 292T436 330T350 345Q335 345 321 344T304 342",
      67: "201 -25Q167 -25 136 -14T75 23T29 94T12 202Q12 290 50 394T161 574Q227 642 303 673T433 704Q435 705 457 705Q533 701 533 640Q533 606 507 548T464 474Q431 444 396 444Q381 444 381 453Q381 459 388 473T407 513T428 563Q433 580 433 594Q433 636 381 636Q314 636 260 594T175 489T128 363T112 247Q112 157 153 101T273 44Q347 44 398 121Q413 144 437 157T481 171Q496 171 496 160Q496 150 476 123Q426 56 350 16T201 -25",
      68: "37 475Q19 475 19 487Q19 536 103 604T327 682H356Q386 683 408 683H419Q475 683 506 681T582 668T667 633Q766 571 766 450Q766 365 723 287T611 152T455 57T279 6Q248 1 160 0Q148 0 131 0T108 -1Q72 -1 72 11Q72 24 90 40T133 64L144 68L152 88Q247 328 272 587Q275 613 272 613Q272 613 269 613Q225 610 195 602T149 579T129 556T119 532Q118 530 116 525T113 518Q102 502 80 490T37 475ZM665 407Q665 596 412 613Q403 614 383 614Q370 614 370 612Q370 598 363 542T323 357T242 103L228 69H265Q391 73 481 119Q536 148 575 188T633 268T658 338T665 392V407",
      69: "144 470Q144 556 240 630T451 705Q564 705 564 637Q564 611 540 573Q529 559 505 547T464 534Q448 534 448 545Q448 552 455 562Q463 577 463 591Q463 600 462 604T456 616T436 627T400 635Q396 635 390 635T380 636Q291 636 258 568Q245 544 245 516Q245 463 290 438T391 410Q415 410 415 398Q415 392 407 380T376 356T326 341Q288 340 260 327Q218 311 187 276T143 208T130 151Q130 113 156 88T211 55T268 47Q349 47 403 125Q415 144 439 157T483 171Q499 171 499 160Q499 148 475 120T413 59T315 3T197 -22Q124 -22 77 14T30 105Q30 126 39 154T66 216T122 288T209 354L223 362Q144 400 144 470",
      70: "199 579Q181 579 181 590Q181 598 188 611T212 639T260 666T335 682Q336 682 349 682T383 682T431 682T493 683T561 683Q776 682 784 681Q826 673 829 647Q829 620 797 600T744 580Q728 580 728 595Q729 607 713 610Q698 613 598 614H500L499 610Q499 598 467 486T428 367Q428 365 551 365H674Q683 360 684 355Q687 346 677 329Q666 312 642 299T598 285Q586 285 582 296H402L394 277Q386 258 373 229T346 167T315 102T286 51Q265 22 225 -5T133 -32Q108 -32 87 -25T54 -7T33 15T21 35T18 47Q18 60 44 80T98 103Q108 103 111 101T119 88Q130 66 150 54T179 39T195 37Q199 37 203 43Q217 67 245 125T318 300T391 532Q393 543 398 564T406 598T409 613T339 614H269Q229 579 199 579",
      71: "216 68Q155 68 115 100T59 177T44 273Q44 299 50 333T73 421T133 533T239 632Q346 704 466 704Q508 704 515 703Q555 696 577 681T599 635Q599 605 570 560T523 496Q490 466 455 466Q440 466 440 475T469 526T499 589Q499 605 489 617Q460 636 403 636Q343 636 295 611T220 548T174 464T150 382T144 318Q144 241 180 189T287 137Q325 137 359 160Q428 205 466 322Q472 342 501 359T551 376Q557 376 560 373T564 368L565 365Q560 341 551 302T512 173T451 31Q359 -119 204 -119Q163 -118 127 -109T74 -91T53 -77Q52 -75 52 -71Q52 -54 79 -35T132 -14H140L151 -19Q210 -49 281 -49H289Q312 -49 329 -31Q351 -7 372 36T405 109T416 142L408 136Q401 131 392 125T369 111T338 96T303 82T261 72T216 68",
      72: "18 487Q18 496 29 517T67 566T127 621T216 665T330 683Q359 683 376 669T397 643T400 622Q400 584 382 488T348 343Q348 342 467 342H587L594 366Q615 440 648 534T690 641Q701 656 723 669T764 683Q783 683 783 672L750 578Q716 485 677 346T625 101Q624 92 623 82T622 65T621 56Q621 20 658 20Q666 20 701 25Q709 52 736 69T785 87Q803 87 803 75T791 44T754 3T685 -33T588 -48Q568 -48 562 -46Q522 -31 522 13V23Q531 129 562 250L569 281L565 280Q561 278 556 277T549 274L438 273H328L321 249Q307 202 275 107T232 0Q219 -16 196 -28T155 -41Q149 -41 145 -39T140 -34T139 -29Q139 -24 148 -3T181 86T233 247Q240 270 240 272Q240 273 194 273H169Q139 273 139 285Q139 295 153 308T187 332Q206 341 236 342L260 343L264 359Q278 414 289 482T300 578Q300 613 260 613H254Q198 613 169 592Q148 578 127 544T104 508Q72 478 37 475Q18 475 18 487",
      73: "174 0H31Q-13 0 -21 2T-30 12Q-30 23 -17 36Q9 60 42 68L155 70Q187 102 214 179T257 333T302 491T366 610L369 614H305Q221 611 188 607T145 596T128 569Q119 543 94 529T47 512Q28 512 28 524Q28 527 32 539Q56 614 159 654Q218 678 312 682Q314 682 339 682T404 682T481 683H632Q642 678 642 671Q642 657 621 641T577 617Q570 615 507 614H444Q427 592 406 542Q382 478 355 366T310 209Q280 123 238 78L230 69H330Q442 70 442 74Q443 74 443 77T447 87T460 105Q490 134 527 137Q545 137 545 125Q545 120 542 112Q531 78 491 49T399 7Q379 2 360 2T174 0",
      74: "148 78Q148 16 189 -17T286 -50Q319 -50 348 -33T396 10T426 59T444 101L471 204Q498 306 521 372Q575 532 649 605L659 614H591Q517 613 494 607Q433 591 400 550T360 477Q353 454 325 437T275 419Q256 419 260 435Q280 523 376 597T583 681Q603 683 713 683H830Q839 674 839 671Q839 654 810 634T754 614Q735 614 721 601Q688 571 654 495T600 351T561 209T541 132Q507 29 412 -45T213 -119Q141 -119 94 -77T47 33Q47 55 50 69T58 90T71 103Q105 131 135 131Q152 131 152 120Q152 119 151 114T149 99T148 78",
      75: "194 618Q193 618 182 613T156 601T131 594Q113 594 113 605Q113 623 144 644Q154 650 205 676T267 703Q277 705 279 705Q295 705 295 691Q295 569 250 397Q225 306 197 217T151 81T128 25Q120 8 94 -7T47 -22Q32 -22 32 -10L64 76Q95 163 133 295T185 530Q198 611 194 618ZM331 429Q331 383 364 290T449 117T542 36Q574 36 607 51T652 103Q660 124 677 133T709 143Q727 143 727 128Q727 119 723 111Q704 56 639 17T497 -22H493Q463 -22 425 16Q401 40 382 71Q335 138 296 243T256 399Q256 434 288 473Q342 540 471 622T670 705Q691 704 703 696Q732 678 732 644Q732 613 714 600T677 586Q671 586 667 587T660 592T657 604V619Q657 647 629 647Q623 647 620 646Q576 635 495 583T365 482Q331 448 331 429",
      76: "62 -22T47 -22T32 -11Q32 -1 56 24T83 55Q113 96 138 172T180 320T234 473T323 609Q364 649 419 677T531 705Q559 705 578 696T604 671T615 645T618 623V611Q618 582 615 571T598 548Q581 531 558 520T518 509Q503 509 503 520Q503 523 505 536T507 560Q507 590 494 610T452 630Q423 630 410 617Q367 578 333 492T271 301T233 170Q211 123 204 112L198 103L224 102Q281 102 369 79T509 52H523Q535 64 544 87T579 128Q616 152 641 152Q656 152 656 142Q656 101 588 40T433 -22Q381 -22 289 1T156 28L141 29L131 20Q111 0 87 -11",
      77: "28 9Q28 37 43 63T73 90Q77 90 83 84T103 70T141 57H146Q162 57 178 79T222 167Q266 279 295 371T334 513T349 598T358 651T371 677Q397 705 432 705Q442 705 445 699T452 666Q453 661 453 659Q475 538 509 405T568 207L574 192Q581 178 587 164T594 150Q596 150 635 189T693 248Q765 324 863 438T1024 626T1089 701Q1093 705 1100 705Q1111 705 1111 682Q1111 675 1108 660T1099 611T1086 540Q1041 277 1041 144Q1041 98 1044 75T1050 48T1059 42Q1064 41 1075 46Q1102 61 1121 61Q1137 61 1137 50Q1137 28 1087 0T1000 -29Q983 -29 972 -23T955 -9T945 16T942 45T941 83V96Q941 158 952 256T974 422L985 489Q984 489 939 436T821 300T698 164Q665 128 620 85T568 37Q564 34 558 34Q550 34 546 37T535 54Q512 91 496 127T450 259T389 498L384 518Q349 367 294 223T198 15Q155 -50 117 -50Q87 -50 61 -35T30 -6Q28 2 28 9",
      78: "343 705Q358 705 358 698Q360 696 370 658T411 524T484 319Q536 174 590 82L595 73L615 152Q646 274 683 407Q729 571 752 637T799 727Q852 780 937 788Q939 788 947 788T958 789H962Q979 789 979 765Q979 722 951 692Q942 683 924 683Q888 681 859 672T818 654T803 639Q784 608 708 322T631 15Q631 14 630 15Q630 17 629 15Q628 14 628 12Q621 -4 601 -17T560 -31Q550 -31 546 -28T530 -7Q484 67 458 123T398 272Q352 392 314 514L306 535V534Q306 533 296 488T272 379T234 239T185 100T127 -7T61 -50Q34 -50 4 -34T-27 8Q-27 33 -12 61T18 90Q21 90 36 77T87 57H92Q109 57 123 78T162 173Q206 299 232 417T265 599T276 667Q284 681 304 693T343 705",
      79: "308 428Q289 428 289 438Q289 457 318 508T378 593Q417 638 475 671T599 705Q688 705 732 643T777 483Q777 380 733 285T620 123T464 18T293 -22Q188 -22 123 51T58 245Q58 327 87 403T159 533T249 626T333 685T388 705Q404 705 404 693Q404 674 363 649Q333 632 304 606T239 537T181 429T158 290Q158 179 214 114T364 48Q489 48 583 165T677 438Q677 473 670 505T648 568T601 617T528 636Q518 636 513 635Q486 629 460 600T419 544T392 490Q383 470 372 459Q341 430 308 428",
      80: "37 475Q19 475 19 487Q19 536 103 604T327 682Q329 682 344 682T380 682T421 683H463Q625 683 695 615Q718 591 726 564Q733 547 733 525Q733 412 607 312T321 205H312Q293 205 293 217Q293 224 302 236T333 260T385 274Q558 287 614 407Q633 445 633 477Q633 515 612 543T556 585T481 607T399 614H370L368 603Q352 463 312 312T242 82T202 -13Q190 -33 164 -45T121 -57Q108 -57 108 -45Q108 -40 120 -10T151 73T192 190T233 349T266 539Q267 546 269 565T272 598T274 613H270Q209 613 163 588Q131 572 113 518Q102 502 80 490T37 475",
      81: "114 286Q114 358 151 433T249 569T392 667T558 705Q653 705 713 641T774 460Q774 389 750 322T687 206T600 114T504 46T412 4L399 -2Q542 -62 636 -62Q660 -62 670 -54T686 -27T700 0Q734 34 770 34Q787 34 787 23Q787 -18 720 -74T563 -131Q485 -131 350 -83T145 -34Q127 -34 127 -22Q127 -12 144 5T190 31L200 34L237 35Q386 38 467 79Q550 120 612 210T675 416Q675 510 625 573T484 636Q410 636 346 587T248 469T214 333Q214 306 221 281T243 229T288 188T360 172Q403 172 441 188T490 205Q510 205 510 192Q505 162 432 132T287 102Q206 102 160 155T114 286",
      82: "37 475Q19 475 19 487Q19 503 35 530T83 589T180 647T327 682H374Q387 682 417 682T464 683Q519 683 559 679T642 663T708 625T731 557Q731 481 668 411T504 300Q506 296 512 286T528 257T553 202Q594 105 611 82Q635 47 665 47Q708 47 742 93Q758 113 786 128Q804 136 819 137Q837 137 837 125Q837 115 818 92T767 43T687 -2T589 -22Q549 -22 517 22T467 120T422 221T362 273Q346 273 346 287Q348 301 373 320T436 342Q437 342 446 343T462 345T481 348T504 353T527 362T553 375T577 393Q598 412 614 443T630 511Q630 545 613 566T541 600T393 614Q370 614 370 613L366 584Q349 446 311 307T243 96L213 25Q205 8 179 -7T132 -22Q125 -22 120 -18T117 -8Q117 -5 130 26T163 113T205 239T246 408T274 606V614Q273 614 259 613T231 609T198 602T163 588Q131 572 113 518Q102 502 80 490T37 475",
      83: "554 512Q536 512 536 522Q536 525 539 539T542 564Q542 588 528 604Q515 616 482 625T410 635Q374 635 349 624T312 594T295 561T290 532Q290 505 303 482T342 442T378 419T409 404Q435 391 451 383T494 357T535 323T562 282T574 231Q574 133 464 56T220 -22Q138 -22 78 21T18 123Q18 184 61 227T156 274Q178 274 178 263Q178 260 177 258Q172 247 164 239T151 227T136 218L127 213L124 202Q118 186 118 163Q120 124 165 86T292 48Q374 48 423 86T473 186V193Q473 267 347 327Q268 364 239 389Q191 431 191 486Q191 547 242 600T356 679T470 705Q472 705 478 705T489 704Q551 704 596 682T642 610Q642 566 621 545Q592 516 554 512",
      84: "49 475Q34 475 34 490Q34 552 106 611T261 681Q272 683 507 683H742Q790 717 816 717Q833 717 833 708Q833 682 795 653T714 615Q691 610 588 609Q490 609 490 607L483 580Q476 554 462 496T435 392Q410 289 395 231T363 116T335 34T309 -15T279 -47T242 -64Q231 -68 218 -68Q203 -68 203 -57Q203 -52 211 -38Q224 -7 234 20T251 66T268 123T283 179T304 261T328 360Q342 415 360 488Q380 567 384 582T397 605Q400 607 401 609H302H244Q200 609 188 607T167 596Q145 572 145 541Q145 520 109 498T49 475",
      85: "8 592Q8 616 70 649T193 683Q246 683 246 631Q246 587 205 492T124 297T83 143Q83 101 100 75T154 48Q202 48 287 135T450 342T560 553Q589 635 593 640Q603 656 626 668T669 683H670Q687 683 687 672T670 616T617 463T547 220Q525 137 521 68Q521 54 522 50T533 42L543 47Q573 61 588 61Q604 61 604 47Q599 16 506 -22Q486 -28 468 -28T436 -18T421 18Q421 92 468 258Q468 259 467 257T459 248Q426 206 391 167T303 81T194 6T83 -22Q66 -22 58 -20Q25 -11 4 19T-17 99Q-17 146 8 220T64 358T120 488T146 586Q146 604 141 608T123 613H120Q99 613 72 597T25 580Q8 580 8 592",
      86: "25 633Q25 647 47 665T100 683Q291 683 291 306Q291 264 288 213T282 132L279 102Q281 102 308 126T378 191T464 279T545 381T596 479Q600 490 600 502Q600 527 581 550T523 577Q505 577 505 601Q505 622 516 647T542 681Q546 683 558 683Q605 679 631 645T658 559Q658 423 487 215Q409 126 308 37T190 -52Q177 -52 177 -28Q177 -26 183 15T196 127T203 270Q203 356 192 421T165 523T126 583T83 613T41 620Q25 620 25 633",
      87: "25 633Q25 647 46 665T103 683Q168 683 207 632Q228 608 243 568Q269 485 269 374Q269 324 265 271T256 184L251 150L252 152Q254 153 257 157T264 167T274 180T286 197Q359 293 424 398T519 558T549 616Q549 618 547 624T545 638Q550 654 572 668T615 683Q626 683 632 672T657 595Q726 370 741 128L742 110Q752 122 767 142T823 217T894 321T950 424T976 511Q976 544 958 560T918 577Q906 577 906 602Q906 629 918 651T942 681Q948 683 954 683Q983 683 1008 658T1034 569T999 421T915 257T813 109T724 -3T681 -49Q666 -59 660 -45Q659 -41 657 35T639 233T591 477Q573 551 570 551Q569 551 554 523T507 439T433 315T323 155T182 -25Q160 -52 151 -53Q137 -53 137 -30Q137 -29 148 25T170 168T181 338Q181 424 168 483T131 571T87 609T40 620Q25 620 25 633",
      88: "324 614Q291 576 250 573Q231 573 231 584Q231 589 232 592Q235 601 244 614T271 643T324 671T400 683H403Q462 683 481 610Q485 594 490 545T498 454L501 413Q504 413 551 442T648 509T705 561Q707 565 707 578Q707 610 682 614Q667 614 667 626Q667 641 695 662T755 683Q765 683 775 680T796 662T807 623Q807 596 792 572T713 499T530 376L505 361V356Q508 346 511 278T524 148T557 75Q569 69 580 69Q585 69 593 77Q624 108 660 110Q667 110 670 110T676 106T678 94Q668 59 624 30T510 0Q487 0 471 9T445 32T430 71T422 117T417 173Q416 183 416 188Q413 214 411 244T407 286T405 299Q403 299 344 263T223 182T154 122Q152 118 152 105Q152 69 180 69Q183 69 187 66T191 60L192 58V56Q192 41 163 21T105 0Q94 0 84 3T63 21T52 60Q52 77 56 90T85 131T155 191Q197 223 259 263T362 327T402 352L391 489Q391 492 390 505T387 526T384 547T379 568T372 586T361 602T348 611Q346 612 341 613T333 614H324",
      89: "65 599Q65 618 107 650T204 683Q267 683 312 643T380 533T414 385T424 217Q424 186 423 160T422 123Q426 123 468 170T567 304T650 469Q661 503 661 519Q661 546 639 570Q615 591 583 591Q569 591 569 616Q569 640 582 661T613 683Q624 683 638 679T671 664T702 625T714 558Q714 472 639 329T426 45Q361 -21 282 -82T154 -143Q97 -143 64 -104T31 -20Q31 4 44 25T70 46Q78 46 81 39T87 16T97 -9Q127 -51 182 -51Q184 -51 187 -50H190Q233 -41 314 25Q330 36 330 40Q336 79 336 178Q336 508 223 594Q199 614 158 619L148 620L139 611Q111 586 83 586Q65 586 65 599",
      90: "694 220Q708 220 708 210Q708 195 695 167T658 105T593 42T502 3Q492 1 458 1Q400 1 293 11T150 22Q116 22 92 11T51 0Q37 0 37 10Q37 21 63 44T179 146T367 319L391 343H343L296 344Q285 350 285 358Q285 365 289 372T300 383T313 392T324 398L329 400H450L561 518Q597 558 607 571L621 587H596Q553 589 484 599T383 609Q342 609 326 596T301 555Q294 533 263 514T208 492Q189 492 189 503Q189 510 197 528T215 559Q249 607 318 645T466 683Q504 683 573 673T669 662L690 661Q734 682 748 683Q767 683 767 673Q767 666 746 640Q655 531 555 428L529 400Q529 399 543 399Q604 397 604 366Q604 350 587 337T551 322Q541 322 539 323Q529 328 529 334Q529 339 487 342L470 343L446 320Q272 153 200 96L235 95Q297 95 392 86T533 74H554Q586 116 597 159Q604 179 635 199T694 220"
    }, {}), texOldstyle$1;
  }
  var texSize3$1 = {}, texSize3 = {}, hasRequiredTexSize3$1;
  function requireTexSize3$1() {
    return hasRequiredTexSize3$1 || (hasRequiredTexSize3$1 = 1, Object.defineProperty(texSize3, "__esModule", { value: !0 }), texSize3.texSize3 = void 0, texSize3.texSize3 = {
      40: [1.45, 0.949, 0.736],
      41: [1.45, 0.949, 0.736],
      47: [1.45, 0.949, 1.044],
      91: [1.45, 0.949, 0.528],
      92: [1.45, 0.949, 1.044],
      93: [1.45, 0.949, 0.528],
      123: [1.45, 0.949, 0.75],
      125: [1.45, 0.949, 0.75],
      710: [0.772, -0.564, 1.444],
      732: [0.749, -0.61, 1.444],
      770: [0.772, -0.564, 0],
      771: [0.749, -0.61, 0],
      8260: [1.45, 0.949, 1.044],
      8730: [1.45, 0.95, 1, { ic: 0.02 }],
      8968: [1.45, 0.949, 0.583],
      8969: [1.45, 0.949, 0.583],
      8970: [1.45, 0.949, 0.583],
      8971: [1.45, 0.949, 0.583],
      9001: [1.45, 0.95, 0.75],
      9002: [1.45, 0.949, 0.75],
      10216: [1.45, 0.95, 0.75],
      10217: [1.45, 0.949, 0.75],
      12296: [1.45, 0.95, 0.75],
      12297: [1.45, 0.949, 0.75]
    }), texSize3;
  }
  var hasRequiredTexSize3;
  function requireTexSize3() {
    if (hasRequiredTexSize3) return texSize3$1;
    hasRequiredTexSize3 = 1, Object.defineProperty(texSize3$1, "__esModule", { value: !0 }), texSize3$1.texSize3 = void 0;
    var O = requireFontData(), _ = requireTexSize3$1();
    return texSize3$1.texSize3 = (0, O.AddPaths)(_.texSize3, {
      40: "701 -940Q701 -943 695 -949H664Q662 -947 636 -922T591 -879T537 -818T475 -737T412 -636T350 -511T295 -362T250 -186T221 17T209 251Q209 962 573 1361Q596 1386 616 1405T649 1437T664 1450H695Q701 1444 701 1441Q701 1436 681 1415T629 1356T557 1261T476 1118T400 927T340 675T308 359Q306 321 306 250Q306 -139 400 -430T690 -924Q701 -936 701 -940",
      41: "34 1438Q34 1446 37 1448T50 1450H56H71Q73 1448 99 1423T144 1380T198 1319T260 1238T323 1137T385 1013T440 864T485 688T514 485T526 251Q526 134 519 53Q472 -519 162 -860Q139 -885 119 -904T86 -936T71 -949H56Q43 -949 39 -947T34 -937Q88 -883 140 -813Q428 -430 428 251Q428 453 402 628T338 922T245 1146T145 1309T46 1425Q44 1427 42 1429T39 1433T36 1436L34 1438",
      47: "81 -949Q71 -949 63 -941T55 -921Q55 -917 56 -915Q59 -906 498 264T939 1438Q945 1450 960 1450Q972 1450 980 1441T988 1421Q982 1403 839 1020L398 -155Q107 -934 103 -938Q96 -949 81 -949",
      91: "247 -949V1450H516V1388H309V-887H516V-949H247",
      92: "988 -922Q988 -933 980 -941T962 -949Q947 -949 940 -938Q936 -934 645 -155L204 1020Q56 1416 56 1424Q56 1433 62 1441T84 1450Q97 1448 103 1439Q107 1435 398 656L839 -519Q988 -918 988 -922",
      93: "11 1388V1450H280V-949H11V-887H218V1388H11",
      123: "618 -943L612 -949H582L568 -943Q472 -903 411 -841T332 -703Q327 -682 327 -653T325 -350Q324 -28 323 -18Q317 24 301 61T264 124T221 171T179 205T147 225T132 234Q130 238 130 250Q130 255 130 258T131 264T132 267T134 269T139 272T144 275Q207 308 256 367Q310 436 323 519Q324 529 325 851Q326 1124 326 1154T332 1205Q369 1358 566 1443L582 1450H612L618 1444V1429Q618 1413 616 1411L608 1406Q599 1402 585 1393T552 1372T515 1343T479 1305T449 1257T429 1200Q425 1180 425 1152T423 851Q422 579 422 549T416 498Q407 459 388 424T346 364T297 318T250 284T214 264T197 254L188 251L205 242Q290 200 345 138T416 3Q421 -18 421 -48T423 -349Q423 -397 423 -472Q424 -677 428 -694Q429 -697 429 -699Q434 -722 443 -743T465 -782T491 -816T519 -845T548 -868T574 -886T595 -899T610 -908L616 -910Q618 -912 618 -928V-943",
      125: "131 1414T131 1429T133 1447T148 1450H153H167L182 1444Q276 1404 336 1343T415 1207Q421 1184 421 1154T423 851L424 531L426 517Q434 462 460 415T518 339T571 296T608 274Q615 270 616 267T618 251Q618 241 618 238T615 232T608 227Q542 194 491 132T426 -15L424 -29L423 -350Q422 -622 422 -652T415 -706Q397 -780 337 -841T182 -943L167 -949H153Q137 -949 134 -946T131 -928Q131 -914 132 -911T144 -904Q146 -903 148 -902Q299 -820 323 -680Q324 -663 325 -349T327 -19Q355 145 541 241L561 250L541 260Q356 355 327 520Q326 537 325 850T323 1181Q315 1227 293 1267T244 1332T193 1374T151 1401T132 1413Q131 1414 131 1429",
      710: "1439 564Q1434 564 1080 631T722 698Q719 698 362 631Q7 564 4 564L0 583Q-4 602 -4 603L720 772L1083 688Q1446 603 1447 603Q1447 602 1443 583L1439 564",
      732: "1 643Q1 646 76 671T271 722T476 749Q555 749 626 736T742 706T856 676T999 662Q1088 662 1192 684T1363 727T1432 749Q1432 745 1437 731T1442 716Q1442 714 1381 693T1212 645T1012 611Q1000 610 955 610Q851 610 701 653T444 697Q355 697 251 676T80 632T11 610Q11 614 6 628T1 643",
      770: "-5 564Q-9 564 -363 631T-722 698Q-725 698 -1082 631Q-1437 564 -1440 564L-1444 583Q-1448 602 -1448 603L-724 772L-361 688Q2 603 3 603Q3 602 -1 583L-5 564",
      771: "-1443 643Q-1443 646 -1368 671T-1173 722T-968 749Q-889 749 -818 736T-702 706T-588 676T-445 662Q-356 662 -252 684T-81 727T-12 749Q-12 745 -7 731T-2 716Q-2 714 -63 693T-232 645T-432 611Q-444 610 -489 610Q-593 610 -743 653T-1000 697Q-1089 697 -1193 676T-1364 632T-1433 610Q-1433 614 -1438 628T-1443 643",
      8260: "81 -949Q71 -949 63 -941T55 -921Q55 -917 56 -915Q59 -906 498 264T939 1438Q945 1450 960 1450Q972 1450 980 1441T988 1421Q982 1403 839 1020L398 -155Q107 -934 103 -938Q96 -949 81 -949",
      8730: "424 -948Q422 -947 313 -434T202 80L170 31Q165 24 157 10Q137 -21 137 -21Q131 -16 124 -8L111 5L264 248L473 -720Q473 -717 727 359T983 1440Q989 1450 1001 1450Q1007 1450 1013 1445T1020 1433Q1020 1425 742 244T460 -941Q458 -950 439 -950H436Q424 -950 424 -948",
      8968: "246 -949V1450H571V1388H308V-949H246",
      8969: "11 1388V1450H336V-949H274V1388H11",
      8970: "246 -949V1450H308V-887H571V-949H246",
      8971: "274 -887V1450H336V-949H11V-887H274",
      9001: "126 242V259L361 845Q595 1431 597 1435Q610 1450 624 1450Q634 1450 644 1443T654 1419V1411L422 831Q190 253 190 250T422 -331L654 -910V-919Q654 -936 644 -943T624 -950Q612 -950 597 -935Q595 -931 361 -345L126 242",
      9002: "94 1424Q94 1426 97 1432T107 1444T124 1450Q141 1450 152 1435Q154 1431 388 845L623 259V242L388 -345Q153 -933 152 -934Q142 -949 127 -949H125Q95 -949 95 -919V-910L327 -331Q559 247 559 250T327 831Q94 1411 94 1424",
      10216: "126 242V259L361 845Q595 1431 597 1435Q610 1450 624 1450Q634 1450 644 1443T654 1419V1411L422 831Q190 253 190 250T422 -331L654 -910V-919Q654 -936 644 -943T624 -950Q612 -950 597 -935Q595 -931 361 -345L126 242",
      10217: "94 1424Q94 1426 97 1432T107 1444T124 1450Q141 1450 152 1435Q154 1431 388 845L623 259V242L388 -345Q153 -933 152 -934Q142 -949 127 -949H125Q95 -949 95 -919V-910L327 -331Q559 247 559 250T327 831Q94 1411 94 1424",
      12296: "126 242V259L361 845Q595 1431 597 1435Q610 1450 624 1450Q634 1450 644 1443T654 1419V1411L422 831Q190 253 190 250T422 -331L654 -910V-919Q654 -936 644 -943T624 -950Q612 -950 597 -935Q595 -931 361 -345L126 242",
      12297: "94 1424Q94 1426 97 1432T107 1444T124 1450Q141 1450 152 1435Q154 1431 388 845L623 259V242L388 -345Q153 -933 152 -934Q142 -949 127 -949H125Q95 -949 95 -919V-910L327 -331Q559 247 559 250T327 831Q94 1411 94 1424"
    }, {}), texSize3$1;
  }
  var texSize4$1 = {}, texSize4 = {}, hasRequiredTexSize4$1;
  function requireTexSize4$1() {
    return hasRequiredTexSize4$1 || (hasRequiredTexSize4$1 = 1, Object.defineProperty(texSize4, "__esModule", { value: !0 }), texSize4.texSize4 = void 0, texSize4.texSize4 = {
      40: [1.75, 1.249, 0.792],
      41: [1.75, 1.249, 0.792],
      47: [1.75, 1.249, 1.278],
      91: [1.75, 1.249, 0.583],
      92: [1.75, 1.249, 1.278],
      93: [1.75, 1.249, 0.583],
      123: [1.75, 1.249, 0.806],
      125: [1.75, 1.249, 0.806],
      710: [0.845, -0.561, 1.889, { ic: 0.013 }],
      732: [0.823, -0.583, 1.889],
      770: [0.845, -0.561, 0, { ic: 0.013 }],
      771: [0.823, -0.583, 0],
      8260: [1.75, 1.249, 1.278],
      8730: [1.75, 1.25, 1, { ic: 0.02 }],
      8968: [1.75, 1.249, 0.639],
      8969: [1.75, 1.249, 0.639],
      8970: [1.75, 1.249, 0.639],
      8971: [1.75, 1.249, 0.639],
      9001: [1.75, 1.248, 0.806],
      9002: [1.75, 1.248, 0.806],
      9115: [1.154, 0.655, 0.875],
      9116: [0.61, 0.01, 0.875],
      9117: [1.165, 0.644, 0.875],
      9118: [1.154, 0.655, 0.875],
      9119: [0.61, 0.01, 0.875],
      9120: [1.165, 0.644, 0.875],
      9121: [1.154, 0.645, 0.667],
      9122: [0.602, 0, 0.667],
      9123: [1.155, 0.644, 0.667],
      9124: [1.154, 0.645, 0.667],
      9125: [0.602, 0, 0.667],
      9126: [1.155, 0.644, 0.667],
      9127: [0.899, 0.01, 0.889],
      9128: [1.16, 0.66, 0.889],
      9129: [0.01, 0.899, 0.889],
      9130: [0.29, 0.015, 0.889],
      9131: [0.899, 0.01, 0.889],
      9132: [1.16, 0.66, 0.889],
      9133: [0.01, 0.899, 0.889],
      9143: [0.935, 0.885, 1.056],
      10216: [1.75, 1.248, 0.806],
      10217: [1.75, 1.248, 0.806],
      12296: [1.75, 1.248, 0.806],
      12297: [1.75, 1.248, 0.806],
      57344: [0.625, 0.014, 1.056],
      57345: [0.605, 0.014, 1.056, { ic: 0.02 }],
      57680: [0.12, 0.213, 0.45, { ic: 0.01 }],
      57681: [0.12, 0.213, 0.45, { ic: 0.024 }],
      57682: [0.333, 0, 0.45, { ic: 0.01 }],
      57683: [0.333, 0, 0.45, { ic: 0.024 }],
      57684: [0.32, 0.2, 0.4, { ic: 0.01 }],
      57685: [0.333, 0, 0.9, { ic: 0.01 }],
      57686: [0.12, 0.213, 0.9, { ic: 0.01 }]
    }), texSize4;
  }
  var hasRequiredTexSize4;
  function requireTexSize4() {
    if (hasRequiredTexSize4) return texSize4$1;
    hasRequiredTexSize4 = 1, Object.defineProperty(texSize4$1, "__esModule", { value: !0 }), texSize4$1.texSize4 = void 0;
    var O = requireFontData(), _ = requireTexSize4$1();
    return texSize4$1.texSize4 = (0, O.AddPaths)(_.texSize4, {
      40: "758 -1237T758 -1240T752 -1249H736Q718 -1249 717 -1248Q711 -1245 672 -1199Q237 -706 237 251T672 1700Q697 1730 716 1749Q718 1750 735 1750H752Q758 1744 758 1741Q758 1737 740 1713T689 1644T619 1537T540 1380T463 1176Q348 802 348 251Q348 -242 441 -599T744 -1218Q758 -1237 758 -1240",
      41: "33 1741Q33 1750 51 1750H60H65Q73 1750 81 1743T119 1700Q554 1207 554 251Q554 -707 119 -1199Q76 -1250 66 -1250Q65 -1250 62 -1250T56 -1249Q55 -1249 53 -1249T49 -1250Q33 -1250 33 -1239Q33 -1236 50 -1214T98 -1150T163 -1052T238 -910T311 -727Q443 -335 443 251Q443 402 436 532T405 831T339 1142T224 1438T50 1716Q33 1737 33 1741",
      47: "1166 1738Q1176 1750 1189 1750T1211 1742T1221 1721Q1221 1720 1221 1718T1220 1715Q1219 1708 666 238T111 -1237Q102 -1249 86 -1249Q74 -1249 65 -1240T56 -1220Q56 -1219 56 -1217T57 -1214Q58 -1207 611 263T1166 1738",
      91: "269 -1249V1750H577V1677H342V-1176H577V-1249H269",
      92: "56 1720Q56 1732 64 1741T85 1750Q104 1750 111 1738Q113 1734 666 264T1220 -1214Q1220 -1215 1220 -1217T1221 -1220Q1221 -1231 1212 -1240T1191 -1249Q1175 -1249 1166 -1237Q1164 -1233 611 237T57 1715Q57 1716 56 1718V1720",
      93: "5 1677V1750H313V-1249H5V-1176H240V1677H5",
      123: "661 -1243L655 -1249H622L604 -1240Q503 -1190 434 -1107T348 -909Q346 -897 346 -499L345 -98L343 -82Q335 3 287 87T157 223Q146 232 145 236Q144 240 144 250Q144 265 145 268T157 278Q242 333 288 417T343 583L345 600L346 1001Q346 1398 348 1410Q379 1622 600 1739L622 1750H655L661 1744V1727V1721Q661 1712 661 1710T657 1705T648 1700T630 1690T602 1668Q589 1659 574 1643T531 1593T484 1508T459 1398Q458 1389 458 1001Q458 614 457 605Q441 435 301 316Q254 277 202 251L250 222Q260 216 301 185Q443 66 457 -104Q458 -113 458 -501Q458 -888 459 -897Q463 -944 478 -988T509 -1060T548 -1114T580 -1149T602 -1167Q620 -1183 634 -1192T653 -1202T659 -1207T661 -1220V-1226V-1243",
      125: "144 1727Q144 1743 146 1746T162 1750H167H183L203 1740Q274 1705 325 1658T403 1562T440 1478T456 1410Q458 1398 458 1001Q459 661 459 624T465 558Q470 526 480 496T502 441T529 395T559 356T588 325T615 301T637 284T654 273L660 269V266Q660 263 660 259T661 250V239Q661 236 661 234T660 232T656 229T649 224Q577 179 528 105T465 -57Q460 -86 460 -123T458 -499V-661Q458 -857 457 -893T447 -955Q425 -1048 359 -1120T203 -1239L183 -1249H168Q150 -1249 147 -1246T144 -1226Q144 -1213 145 -1210T153 -1202Q169 -1193 186 -1181T232 -1140T282 -1081T322 -1000T345 -897Q346 -888 346 -501Q346 -113 347 -104Q359 58 503 184Q554 226 603 250Q504 299 430 393T347 605Q346 614 346 1002Q346 1389 345 1398Q338 1493 288 1573T153 1703Q146 1707 145 1710T144 1727",
      710: "5 561Q-4 561 -9 582T-14 618Q-14 623 -13 625Q-11 628 461 736T943 845Q945 845 1417 738T1896 628Q1902 628 1902 618Q1902 607 1897 584T1883 561Q1881 561 1412 654L945 750L476 654Q6 561 5 561",
      732: "1212 583Q1124 583 1048 603T923 647T799 691T635 711Q524 711 375 679T120 615L16 583Q14 584 12 587T9 592Q-2 650 2 659Q2 669 38 687Q54 696 146 723T309 767Q527 823 666 823Q759 823 837 803T964 759T1088 715T1252 695Q1363 695 1512 727T1764 791T1871 823Q1872 822 1874 819T1878 814Q1885 783 1885 753Q1885 748 1884 747Q1884 738 1849 719Q1836 712 1740 682T1484 617T1212 583",
      770: "-1884 561Q-1893 561 -1898 582T-1903 618Q-1903 623 -1902 625Q-1900 628 -1428 736T-946 845Q-944 845 -472 738T7 628Q13 628 13 618Q13 607 8 584T-6 561Q-8 561 -477 654L-944 750L-1413 654Q-1883 561 -1884 561",
      771: "-677 583Q-765 583 -841 603T-966 647T-1090 691T-1254 711Q-1365 711 -1514 679T-1768 615L-1873 583Q-1875 584 -1877 587T-1880 592Q-1891 650 -1887 659Q-1887 669 -1851 687Q-1835 696 -1743 723T-1580 767Q-1362 823 -1223 823Q-1130 823 -1052 803T-925 759T-801 715T-637 695Q-526 695 -377 727T-125 791T-18 823Q-17 822 -15 819T-11 814Q-4 782 -4 753Q-4 748 -5 747Q-5 738 -40 719Q-53 712 -149 682T-405 617T-677 583",
      8260: "1166 1738Q1176 1750 1189 1750T1211 1742T1221 1721Q1221 1720 1221 1718T1220 1715Q1219 1708 666 238T111 -1237Q102 -1249 86 -1249Q74 -1249 65 -1240T56 -1220Q56 -1219 56 -1217T57 -1214Q58 -1207 611 263T1166 1738",
      8730: "983 1739Q988 1750 1001 1750Q1008 1750 1013 1745T1020 1733Q1020 1726 742 244T460 -1241Q458 -1250 439 -1250H436Q424 -1250 424 -1248L410 -1166Q395 -1083 367 -920T312 -601L201 44L137 -83L111 -57L187 96L264 247Q265 246 369 -357Q470 -958 473 -963L727 384Q979 1729 983 1739",
      8968: "269 -1249V1750H633V1677H342V-1249H269",
      8969: "5 1677V1750H369V-1249H296V1677H5",
      8970: "269 -1249V1750H342V-1176H633V-1249H269",
      8971: "296 -1176V1750H369V-1249H5V-1176H296",
      9001: "140 242V260L386 994Q633 1729 635 1732Q643 1745 657 1749Q658 1749 662 1749T668 1750Q682 1749 692 1740T702 1714V1705L214 251L703 -1204L702 -1213Q702 -1230 692 -1239T667 -1248H664Q647 -1248 635 -1231Q633 -1228 386 -493L140 242",
      9002: "103 1714Q103 1732 114 1741T137 1750Q157 1750 170 1732Q172 1729 419 994L665 260V242L419 -493Q172 -1228 170 -1231Q158 -1248 141 -1248H138Q123 -1248 113 -1239T103 -1213V-1204L591 251L103 1705V1714",
      9115: "837 1154Q843 1148 843 1145Q843 1141 818 1106T753 1002T667 841T574 604T494 299Q417 -84 417 -609Q417 -641 416 -647T411 -654Q409 -655 366 -655Q299 -655 297 -654Q292 -652 292 -643T291 -583Q293 -400 304 -242T347 110T432 470T574 813T785 1136Q787 1139 790 1142T794 1147T796 1150T799 1152T802 1153T807 1154T813 1154H819H837",
      9116: "413 -9Q412 -9 407 -9T388 -10T354 -10Q300 -10 297 -9Q294 -8 293 -5Q291 5 291 127V300Q291 602 292 605L296 609Q298 610 366 610Q382 610 392 610T407 610T412 609Q416 609 416 592T417 473V127Q417 -9 413 -9",
      9117: "843 -635Q843 -638 837 -644H820Q801 -644 800 -643Q792 -635 785 -626Q684 -503 605 -363T473 -75T385 216T330 518T302 809T291 1093Q291 1144 291 1153T296 1164Q298 1165 366 1165Q409 1165 411 1164Q415 1163 416 1157T417 1119Q417 529 517 109T833 -617Q843 -631 843 -635",
      9118: "31 1143Q31 1154 49 1154H59Q72 1154 75 1152T89 1136Q190 1013 269 873T401 585T489 294T544 -8T572 -299T583 -583Q583 -634 583 -643T577 -654Q575 -655 508 -655Q465 -655 463 -654Q459 -653 458 -647T457 -609Q457 -58 371 340T100 1037Q87 1059 61 1098T31 1143",
      9119: "579 -9Q578 -9 573 -9T554 -10T520 -10Q466 -10 463 -9Q460 -8 459 -5Q457 5 457 127V300Q457 602 458 605L462 609Q464 610 532 610Q548 610 558 610T573 610T578 609Q582 609 582 592T583 473V127Q583 -9 579 -9",
      9120: "56 -644H50Q31 -644 31 -635Q31 -632 37 -622Q69 -579 100 -527Q286 -228 371 170T457 1119Q457 1161 462 1164Q464 1165 520 1165Q575 1165 577 1164Q582 1162 582 1153T583 1093Q581 910 570 752T527 400T442 40T300 -303T89 -626Q78 -640 75 -642T61 -644H56",
      9121: "319 -645V1154H666V1070H403V-645H319",
      9122: "319 0V602H403V0H319",
      9123: "319 -644V1155H403V-560H666V-644H319",
      9124: "0 1070V1154H347V-645H263V1070H0",
      9125: "263 0V602H347V0H263",
      9126: "263 -560V1155H347V-644H0V-560H263",
      9127: "712 899L718 893V876V865Q718 854 704 846Q627 793 577 710T510 525Q510 524 509 521Q505 493 504 349Q504 345 504 334Q504 277 504 240Q504 -2 503 -4Q502 -8 494 -9T444 -10Q392 -10 390 -9Q387 -8 386 -5Q384 5 384 230Q384 262 384 312T383 382Q383 481 392 535T434 656Q510 806 664 892L677 899H712",
      9128: "389 1159Q391 1160 455 1160Q496 1160 498 1159Q501 1158 502 1155Q504 1145 504 924Q504 691 503 682Q494 549 425 439T243 259L229 250L243 241Q349 175 421 66T503 -182Q504 -191 504 -424Q504 -600 504 -629T499 -659H498Q496 -660 444 -660T390 -659Q387 -658 386 -655Q384 -645 384 -425V-282Q384 -176 377 -116T342 10Q325 54 301 92T255 155T214 196T183 222T171 232Q170 233 170 250T171 268Q171 269 191 284T240 331T300 407T354 524T383 679Q384 691 384 925Q384 1152 385 1155L389 1159",
      9129: "718 -893L712 -899H677L666 -893Q542 -825 468 -714T385 -476Q384 -466 384 -282Q384 3 385 5L389 9Q392 10 444 10Q486 10 494 9T503 4Q504 2 504 -239V-310V-366Q504 -470 508 -513T530 -609Q546 -657 569 -698T617 -767T661 -812T699 -843T717 -856T718 -876V-893",
      9130: "384 150V266Q384 304 389 309Q391 310 455 310Q496 310 498 309Q502 308 503 298Q504 283 504 150Q504 32 504 12T499 -9H498Q496 -10 444 -10T390 -9Q386 -8 385 2Q384 17 384 150",
      9131: "170 875Q170 892 172 895T189 899H194H211L222 893Q345 826 420 715T503 476Q504 467 504 230Q504 51 504 21T499 -9H498Q496 -10 444 -10Q402 -10 394 -9T385 -4Q384 -2 384 240V311V366Q384 469 380 513T358 609Q342 657 319 698T271 767T227 812T189 843T171 856T170 875",
      9132: "389 1159Q391 1160 455 1160Q496 1160 498 1159Q501 1158 502 1155Q504 1145 504 925V782Q504 676 511 616T546 490Q563 446 587 408T633 345T674 304T705 278T717 268Q718 267 718 250T717 232Q717 231 697 216T648 169T588 93T534 -24T505 -179Q504 -191 504 -425Q504 -600 504 -629T499 -659H498Q496 -660 444 -660T390 -659Q387 -658 386 -655Q384 -645 384 -424Q384 -191 385 -182Q394 -49 463 61T645 241L659 250L645 259Q539 325 467 434T385 682Q384 692 384 873Q384 1153 385 1155L389 1159",
      9133: "384 -239V-57Q384 4 389 9Q391 10 455 10Q496 10 498 9Q501 8 502 5Q504 -5 504 -230Q504 -261 504 -311T505 -381Q505 -486 492 -551T435 -691Q357 -820 222 -893L211 -899H195Q176 -899 173 -896T170 -874Q170 -858 171 -855T184 -846Q262 -793 312 -709T378 -525Q378 -524 379 -522Q383 -493 384 -351Q384 -345 384 -334Q384 -276 384 -239",
      9143: "742 -871Q740 -873 737 -876T733 -880T730 -882T724 -884T714 -885H702L222 569L180 484Q138 399 137 399Q131 404 124 412L111 425L265 736L702 -586V168L703 922Q713 935 722 935Q734 935 742 920V-871",
      10216: "140 242V260L386 994Q633 1729 635 1732Q643 1745 657 1749Q658 1749 662 1749T668 1750Q682 1749 692 1740T702 1714V1705L214 251L703 -1204L702 -1213Q702 -1230 692 -1239T667 -1248H664Q647 -1248 635 -1231Q633 -1228 386 -493L140 242",
      10217: "103 1714Q103 1732 114 1741T137 1750Q157 1750 170 1732Q172 1729 419 994L665 260V242L419 -493Q172 -1228 170 -1231Q158 -1248 141 -1248H138Q123 -1248 113 -1239T103 -1213V-1204L591 251L103 1705V1714",
      12296: "140 242V260L386 994Q633 1729 635 1732Q643 1745 657 1749Q658 1749 662 1749T668 1750Q682 1749 692 1740T702 1714V1705L214 251L703 -1204L702 -1213Q702 -1230 692 -1239T667 -1248H664Q647 -1248 635 -1231Q633 -1228 386 -493L140 242",
      12297: "103 1714Q103 1732 114 1741T137 1750Q157 1750 170 1732Q172 1729 419 994L665 260V242L419 -493Q172 -1228 170 -1231Q158 -1248 141 -1248H138Q123 -1248 113 -1239T103 -1213V-1204L591 251L103 1705V1714",
      57344: "722 -14H720Q708 -14 702 0V306L703 612Q713 625 722 625Q734 625 742 610V0Q734 -14 724 -14H722",
      57345: "702 589Q706 601 718 605H1061Q1076 597 1076 585Q1076 572 1061 565H742V0Q734 -14 724 -14H722H720Q708 -14 702 0V589",
      57680: "-18 -213L-24 -207V-172L-16 -158Q75 2 260 84Q334 113 415 119Q418 119 427 119T440 120Q454 120 457 117T460 98V60V25Q460 7 457 4T441 0Q308 0 193 -55T25 -205Q21 -211 18 -212T-1 -213H-18",
      57681: "-10 60Q-10 104 -10 111T-5 118Q-1 120 10 120Q96 120 190 84Q375 2 466 -158L474 -172V-207L468 -213H451H447Q437 -213 434 -213T428 -209T423 -202T414 -187T396 -163Q331 -82 224 -41T9 0Q-4 0 -7 3T-10 25V60",
      57682: "-24 327L-18 333H-1Q11 333 15 333T22 329T27 322T35 308T54 284Q115 203 225 162T441 120Q454 120 457 117T460 95V60V28Q460 8 457 4T442 0Q355 0 260 36Q75 118 -16 278L-24 292V327",
      57683: "-10 60V95Q-10 113 -7 116T9 120Q151 120 250 171T396 284Q404 293 412 305T424 324T431 331Q433 333 451 333H468L474 327V292L466 278Q375 118 190 36Q95 0 8 0Q-5 0 -7 3T-10 24V60",
      57684: "-10 0V120H410V0H-10"
    }, {
      57685: "",
      57686: ""
    }), texSize4$1;
  }
  var texVariant$1 = {}, texVariant = {}, hasRequiredTexVariant$1;
  function requireTexVariant$1() {
    return hasRequiredTexVariant$1 || (hasRequiredTexVariant$1 = 1, Object.defineProperty(texVariant, "__esModule", { value: !0 }), texVariant.texVariant = void 0, texVariant.texVariant = {
      710: [0.845, -0.561, 2.333, { ic: 0.013 }],
      732: [0.899, -0.628, 2.333],
      770: [0.845, -0.561, 0, { ic: 0.013 }],
      771: [0.899, -0.628, 0],
      1008: [0.434, 6e-3, 0.667, { ic: 0.067 }],
      8463: [0.695, 0.013, 0.54, { ic: 0.022 }],
      8592: [0.437, -0.064, 0.5],
      8594: [0.437, -0.064, 0.5],
      8652: [0.514, 0.014, 1],
      8708: [0.86, 0.166, 0.556],
      8709: [0.587, 0, 0.778],
      8722: [0.27, -0.23, 0.5],
      8726: [0.43, 0.023, 0.778],
      8733: [0.472, -0.028, 0.778],
      8739: [0.43, 0.023, 0.222],
      8740: [0.43, 0.023, 0.222, { ic: 0.018 }],
      8741: [0.431, 0.023, 0.389],
      8742: [0.431, 0.024, 0.389, { ic: 0.018 }],
      8764: [0.365, -0.132, 0.778],
      8776: [0.481, -0.05, 0.778],
      8808: [0.752, 0.284, 0.778],
      8809: [0.752, 0.284, 0.778],
      8816: [0.919, 0.421, 0.778],
      8817: [0.919, 0.421, 0.778],
      8840: [0.828, 0.33, 0.778],
      8841: [0.828, 0.33, 0.778],
      8842: [0.634, 0.255, 0.778],
      8843: [0.634, 0.254, 0.778],
      8872: [0.694, 0, 0.611],
      8901: [0.189, 0, 0.278],
      8994: [0.378, -0.122, 0.778],
      8995: [0.378, -0.143, 0.778],
      9651: [0.575, 0.02, 0.722],
      9661: [0.576, 0.019, 0.722],
      10887: [0.801, 0.303, 0.778],
      10888: [0.801, 0.303, 0.778],
      10955: [0.752, 0.332, 0.778],
      10956: [0.752, 0.333, 0.778]
    }), texVariant;
  }
  var hasRequiredTexVariant;
  function requireTexVariant() {
    if (hasRequiredTexVariant) return texVariant$1;
    hasRequiredTexVariant = 1, Object.defineProperty(texVariant$1, "__esModule", { value: !0 }), texVariant$1.texVariant = void 0;
    var O = requireFontData(), _ = requireTexVariant$1();
    return texVariant$1.texVariant = (0, O.AddPaths)(_.texVariant, {
      710: "1 561Q-3 563 -6 577T-12 604L-14 618Q-14 625 -7 628T23 635Q31 636 36 637Q63 641 621 745Q1148 845 1165 845Q1167 845 1752 739L2338 630Q2346 630 2346 618Q2340 565 2332 561Q2329 561 1749 654Q1617 675 1466 699T1241 736T1167 748Q1165 748 1093 737T867 700T583 654Q3 561 1 561",
      732: "804 788Q717 788 606 772T397 732T213 685T75 645T18 628Q11 628 11 632Q8 637 4 668T2 704Q2 713 36 732Q55 739 242 795Q622 898 826 898Q893 898 947 895Q1009 887 1056 872T1187 819Q1286 776 1356 758T1527 739Q1614 739 1725 755T1934 795T2118 842T2256 882T2313 899Q2320 899 2320 895Q2323 890 2327 860T2329 824Q2329 818 2296 795Q2273 787 2089 732Q1810 657 1598 632Q1562 629 1492 629Q1389 629 1320 644T1144 708Q1048 751 977 769T804 788",
      770: "-2332 561Q-2336 563 -2340 577T-2346 604L-2347 618Q-2347 625 -2340 628T-2310 635Q-2302 636 -2297 637Q-2270 641 -1712 745Q-1185 845 -1168 845Q-1166 845 -581 739L5 630Q13 630 13 618Q7 565 -1 561Q-4 561 -584 654Q-716 675 -867 699T-1092 736T-1166 748Q-1168 748 -1240 737T-1466 700T-1750 654Q-2330 561 -2332 561",
      771: "-1529 788Q-1616 788 -1727 772T-1936 732T-2120 685T-2258 645T-2315 628Q-2322 628 -2322 632Q-2325 637 -2329 668T-2331 704Q-2331 713 -2297 732Q-2278 739 -2091 795Q-1711 898 -1507 898Q-1440 898 -1386 895Q-1324 887 -1277 872T-1146 819Q-1047 776 -977 758T-806 739Q-719 739 -608 755T-399 795T-215 842T-77 882T-20 899Q-13 899 -13 895Q-10 890 -6 860T-4 824Q-4 818 -37 795Q-60 787 -244 732Q-523 657 -735 632Q-771 629 -841 629Q-944 629 -1013 644T-1189 708Q-1285 751 -1356 769T-1529 788",
      1008: "228 325Q170 322 156 316T127 309Q108 309 104 314Q99 319 99 322T108 341Q125 376 171 400T268 425H271Q302 425 319 396Q328 377 328 358Q328 332 324 314Q311 270 286 221Q274 194 274 192H275Q339 234 484 325T639 421Q669 434 691 434T723 425T734 406Q734 394 719 381Q715 376 644 330L575 287L566 267Q543 233 526 176Q520 160 515 143T508 115T506 105Q506 103 533 103Q585 103 607 110T641 118Q670 118 670 107Q670 100 661 85Q643 50 598 27T504 3Q465 3 450 36Q441 51 441 73Q441 84 444 96Q452 146 484 205L497 236L324 125Q143 12 135 10Q103 -6 77 -6Q61 -6 49 2T37 21Q37 36 49 46T124 96L195 141L204 156Q219 179 243 248T264 323Q264 325 228 325",
      8463: "182 599Q182 611 174 615T133 619Q118 619 114 621T109 630Q109 636 114 656T122 681Q125 685 202 688Q272 695 286 695Q304 695 304 684Q304 682 295 644T282 597Q282 592 360 592H399Q430 592 445 587T460 563Q460 552 451 541L442 535H266L251 468Q247 453 243 436T236 409T233 399Q233 395 244 404Q295 441 357 441Q405 441 445 417T485 333Q485 284 449 178T412 58T426 44Q447 44 466 68Q485 87 500 130L509 152H531H543Q562 152 562 144Q562 128 546 93T494 23T415 -13Q385 -13 359 3T322 44Q318 52 318 77Q318 99 352 196T386 337Q386 386 346 386Q318 386 286 370Q267 361 245 338T211 292Q207 287 193 235T162 113T138 21Q128 7 122 4Q105 -12 83 -12Q66 -12 54 -2T42 26L166 530Q166 534 161 534T129 535Q127 535 122 535T112 534Q74 534 74 562Q74 570 77 576T84 585T96 589T109 591T124 592T138 592L182 595V599",
      8592: "292 419Q292 400 261 347T211 275H306Q319 275 338 275T364 276Q399 276 410 271T422 250T411 230T366 225H306H211Q214 222 232 197T271 136T292 82Q292 71 285 68T262 64H250H241Q221 64 216 67T205 83Q186 127 153 167T78 230Q64 238 64 250Q64 258 69 263T82 272T106 288T139 318Q162 342 177 365T198 402T209 425T223 436Q224 437 252 437H258Q292 437 292 419",
      8594: "188 417Q188 437 221 437H233Q256 437 263 434T275 417Q294 373 327 333T402 270Q417 261 417 250Q417 241 410 236T382 217T341 182Q315 155 299 128T275 85T263 66Q259 64 231 64H219Q197 64 191 72T193 100Q202 124 215 147T239 185T257 210T267 223L269 225H174H116Q80 225 69 229T58 250T70 271T114 276Q121 276 140 276T174 275H269L267 277Q266 280 257 291T233 325T205 374Q188 408 188 417",
      8652: "755 512Q755 514 778 514H801L804 503Q805 501 812 486T824 462T839 437T862 408T892 381T932 354L944 347V327H507Q70 327 67 329Q55 335 55 347T67 365Q70 367 454 367H837L828 376Q803 403 785 437T761 489T755 512ZM55 153V173H492Q928 173 932 171Q944 166 944 153T932 135Q928 133 545 133H162L171 124Q198 95 216 61T239 8L244 -12Q244 -14 221 -14H198L195 -4Q160 95 67 146L55 153",
      8708: "55 676Q55 688 66 694H199L333 696L351 772Q364 827 370 843T386 860Q393 860 399 854T406 841Q406 836 391 765L375 696Q375 694 431 694H484Q491 688 497 681V12L493 5L486 1L353 -1H219L202 -79Q184 -153 180 -159Q175 -166 165 -166Q146 -166 146 -148Q146 -141 161 -76T177 -4Q177 -1 122 -1H68Q55 12 55 20T66 39H126L186 41L219 181Q226 215 234 251T246 305T251 325Q251 328 166 328H79Q68 345 68 347Q68 352 75 359L82 368H262L291 505Q298 539 306 575T319 630T324 650V654H68Q55 669 55 676ZM457 368V654H411Q366 654 366 652Q365 651 361 634T349 580T333 514Q303 373 302 372V368H457ZM457 39V328H375Q293 328 293 325Q292 322 260 183T228 41T344 39H457",
      8709: "624 470Q624 468 639 446T668 382T683 291Q683 181 612 99T437 -1Q425 -2 387 -2T337 -1Q245 18 193 70L179 81L131 39Q96 8 89 3T75 -3Q55 -3 55 17Q55 24 61 30T111 73Q154 113 151 113Q151 114 140 130T115 177T95 241Q94 253 94 291T95 341Q112 431 173 495Q265 587 385 587Q410 587 437 581Q522 571 582 513L595 501L642 541Q689 586 695 586Q696 586 697 586T699 587Q706 587 713 583T720 568Q720 560 711 551T664 510Q651 499 642 490T628 475T624 470ZM564 477Q517 522 448 539Q428 546 375 546Q290 546 229 492T144 370Q133 332 133 279Q136 228 151 195Q157 179 168 160T184 141Q186 141 375 307T564 477ZM642 290Q642 318 637 343T625 386T611 416T598 436T593 444Q590 444 402 277T213 108Q213 104 231 89T293 55T392 37Q495 37 568 111T642 290",
      8722: "84 237T84 250T98 270H402Q417 262 417 250T402 230H98Q84 237 84 250",
      8726: "91 404T91 410T97 423T111 430Q117 430 395 224Q676 13 678 10Q685 3 685 -3T678 -16T664 -23Q658 -23 380 184T98 397Q91 404 91 410",
      8733: "56 250Q56 346 122 409T276 472Q349 472 407 430T486 326L489 316Q490 317 493 326T501 345T514 367T531 393Q557 425 602 448T698 472Q722 472 722 452Q722 437 702 435T642 421T571 377Q520 323 520 250Q520 179 568 126T693 68Q722 66 722 48Q722 28 698 28Q636 28 576 67T493 174L490 184Q489 181 483 167T475 150T468 136T458 120T447 107T432 90T412 73Q350 28 277 28Q188 28 122 91T56 250ZM199 68T278 68T408 122T459 250Q459 322 414 370T308 430Q302 431 273 431Q204 431 150 380T96 250Q96 176 147 122",
      8739: "91 417Q104 430 111 430T131 417V-10Q116 -23 111 -23T91 -10V417",
      8740: "91 417Q104 430 111 430T131 417V301L171 341Q201 373 207 378T220 384Q227 384 233 377T240 366Q240 357 187 299L131 244V-10Q116 -23 111 -23T91 -10V201L49 157Q20 127 14 121T0 115Q-8 115 -14 121T-20 132Q-20 139 17 178Q29 191 36 199L91 257V417",
      8741: "55 417Q69 431 76 431T95 419V-12Q84 -23 76 -23Q72 -23 69 -22T62 -16T55 -10V417ZM293 419Q300 431 310 431L324 424L331 417V-10Q316 -23 309 -23L297 -19L293 -12V419",
      8742: "56 417Q68 431 76 431L89 426L96 419V317L98 215L193 273L291 330V375L293 419Q301 431 311 431Q331 431 331 388L333 355L356 370Q381 384 388 384Q394 384 400 377T407 363Q407 354 367 328L331 308V-10Q316 -23 310 -23Q300 -23 293 -12L291 135V284L98 168L96 77V-12Q84 -24 76 -24L62 -19L58 -12L56 66V144L31 128Q5 114 -2 114Q-8 114 -14 121T-20 136Q-20 142 -14 147T20 170L56 190V417",
      8764: "73 132Q55 132 55 172Q55 220 79 272Q95 301 111 319Q148 353 195 363Q199 364 212 364Q262 364 294 350T408 272Q472 222 522 212Q537 208 555 208Q606 208 646 243Q671 268 680 296T691 342T702 365Q713 365 716 354T719 314Q714 236 664 179L660 176Q657 173 654 170T644 163T631 154T615 146T596 139T574 134T549 132Q510 132 465 156T386 211T307 265T223 290Q162 290 124 249T86 165Q86 155 82 144T73 132",
      8776: "55 326Q55 394 101 437T226 481Q268 479 313 460T392 419T469 379T555 361Q622 361 662 401Q686 423 688 450Q693 479 702 479H705Q719 479 719 442Q719 367 670 327T554 286Q512 286 466 304T386 345T307 385T220 404Q184 404 157 394T120 374L111 363Q86 339 86 317Q86 288 71 288Q55 288 55 326ZM55 90Q55 164 105 205T226 246Q269 243 314 224T392 183T470 144T558 126Q622 126 662 166Q686 187 688 214Q693 244 704 244Q716 244 719 210Q719 165 702 132T658 82T605 58T552 50T498 58T447 77T384 110Q322 146 302 152Q263 168 220 168Q179 168 144 152Q128 147 107 125T86 81Q86 52 71 52Q55 52 55 90",
      8808: "86 472Q93 477 381 614T673 752Q680 752 686 746T693 732T689 721Q686 715 418 590L151 461L418 332Q684 207 689 201Q693 195 693 190Q693 183 687 177T675 170Q668 170 380 307T86 450Q82 454 82 461Q82 467 86 472ZM369 101V126Q369 156 382 156H384Q385 157 386 157Q409 157 409 115V98V54H680Q693 39 693 34T680 14H409V-142H680Q693 -155 693 -162Q693 -167 680 -182H409V-273Q396 -284 388 -284Q382 -284 369 -275V-182H95Q82 -167 82 -162Q82 -155 95 -142H369V14H95Q93 17 89 21T84 27T82 34T83 40T89 47T95 54H369V101",
      8809: "89 745Q95 752 100 752Q106 752 394 615T689 472Q693 468 693 461T689 450Q684 445 396 308T100 170Q95 170 89 176T82 190Q82 195 86 201Q91 208 358 332L624 461L358 590Q90 715 86 721Q82 725 82 731Q82 739 89 745ZM369 101V126Q369 156 382 156H384Q385 157 386 157Q409 157 409 115V98V54H680Q693 39 693 34T680 14H409V-142H680Q693 -155 693 -162Q693 -167 680 -182H409V-273Q396 -284 388 -284Q382 -284 369 -275V-182H95Q82 -167 82 -162Q82 -155 95 -142H369V14H95Q93 17 89 21T84 27T82 34T83 40T89 47T95 54H369V101",
      8816: "82 34Q82 44 93 55H198L300 57L342 179Q351 207 362 238T378 286T384 303T238 377Q109 435 86 450Q82 454 82 460T86 472Q90 476 302 579L511 679Q512 679 553 795Q569 842 577 866T592 903T600 917T608 919Q615 919 622 912T629 901Q629 899 595 799Q589 777 581 753T569 717T564 703L618 728Q666 752 673 752T686 746T693 732Q693 723 683 717T615 683L546 650L491 488Q464 410 450 368T438 326Q493 297 562 266Q660 219 677 209T694 190Q694 183 690 177T678 171Q664 171 546 228L424 286Q422 286 382 172L342 57L513 55H682Q694 43 694 34Q694 28 689 21L682 17L506 15H329L322 -8Q320 -13 310 -41T295 -85L275 -141H680Q682 -143 684 -146T688 -151T691 -156T693 -162Q693 -172 682 -179L473 -181H262L220 -303Q192 -388 185 -404T166 -421Q160 -421 153 -415T146 -403Q146 -400 179 -302T220 -185Q220 -181 158 -181L93 -179L86 -174Q82 -169 82 -161Q82 -152 93 -141H164L233 -139L260 -63L286 15H189L93 17L86 21Q82 26 82 34ZM495 623Q495 626 493 626T321 544T151 461L398 343Q399 343 405 360T423 415T446 483Q457 513 469 551T488 606T495 623",
      8817: "97 172Q82 172 82 190Q82 197 86 201Q94 209 173 246T327 319T402 357Q405 360 434 448T462 539L278 628Q96 713 86 721Q82 725 82 732T88 745T102 752Q103 752 125 742T198 709T293 666Q342 642 385 622T453 590T478 579Q479 579 506 659T562 824T598 915Q602 919 609 919T622 913T629 901Q629 898 571 728Q546 656 531 608T518 559Q555 539 602 519Q664 488 679 479T694 461Q694 457 689 450Q680 443 616 413T494 356T435 326L389 190L342 57L513 55H682Q694 43 694 34Q694 28 689 21L682 17L506 15H329L322 -8Q320 -13 310 -41T295 -85L275 -141H680Q682 -143 684 -146T688 -151T691 -156T693 -162Q693 -172 682 -179L473 -181H262L220 -303Q192 -388 185 -404T166 -421Q160 -421 153 -415T146 -403Q146 -400 179 -302T220 -185Q220 -181 158 -181L93 -179L86 -174Q82 -169 82 -161Q82 -152 93 -141H164L233 -139L260 -63L286 15H189L93 17L86 21Q82 26 82 34Q82 44 93 55H198L300 57L342 179Q350 204 361 238T378 286T382 301L246 237Q111 172 97 172ZM624 461Q621 464 560 492Q512 518 503 518Q500 518 500 517Q499 513 488 479T465 413T453 379L624 461",
      8840: "82 -6Q82 1 95 14H262L295 94Q331 171 331 174Q324 175 312 178T267 194T206 227T146 283T98 368Q84 406 84 461T98 554Q126 632 194 685T349 750Q360 752 480 752H591L604 783Q620 819 624 821Q631 828 640 828Q653 825 658 810Q658 808 646 781L635 754Q635 752 658 752Q680 752 686 746Q693 739 693 732Q693 728 692 726T686 719T680 712H615L506 466Q479 407 451 344T408 248T393 214Q393 210 535 210H680Q693 194 693 190T680 170H373L340 92L304 14H680Q693 1 693 -6Q693 -11 680 -26H286L253 -103L218 -179L451 -181H682Q694 -193 694 -201Q694 -212 682 -219L440 -221H200L178 -270Q160 -309 154 -319T139 -330Q122 -330 118 -312L155 -223Q155 -221 126 -221H95Q82 -206 82 -201T95 -181H175L206 -108Q237 -35 242 -30Q242 -26 169 -26H95Q82 -11 82 -6ZM571 710Q571 712 469 712Q443 712 416 712T371 711T351 710Q279 700 221 656T138 548Q124 508 124 461T138 374Q186 245 351 212L460 459Q571 709 571 710",
      8841: "82 732Q82 739 95 752H251H348Q420 752 460 744T551 708Q566 697 566 701Q618 815 624 821Q631 828 640 828Q653 825 658 810L600 677Q600 671 615 656T653 605T689 517Q692 496 692 461T689 406Q668 325 615 266Q572 221 513 196T391 170H373L340 92L304 14H680Q693 1 693 -6Q693 -11 680 -26H286L253 -103L218 -179L451 -181H682Q694 -193 694 -201Q694 -212 682 -219L440 -221H200L178 -270Q160 -309 154 -319T139 -330Q122 -330 118 -312L155 -223Q155 -221 126 -221H95Q82 -206 82 -201T95 -181H175L206 -108Q237 -35 242 -30Q242 -26 169 -26H95Q82 -11 82 -6Q82 1 95 14H262L295 92L331 170H95Q93 172 91 175T87 180T84 185T82 191Q82 199 93 210H220L349 212L549 659Q507 692 462 702T338 712H249H95Q82 727 82 732ZM652 473Q652 513 636 552T603 611T582 632Q581 632 487 422T393 210Q424 210 460 220T535 253T605 316T649 410Q652 427 652 461V473",
      8842: "693 -115T693 -122T680 -144H315L269 -199Q221 -255 213 -255H212Q203 -255 197 -248T193 -231Q195 -225 229 -184L262 -144H186L113 -142L106 -137Q102 -130 102 -125Q102 -119 115 -104H298L426 52H386Q342 54 309 63Q236 79 180 129T98 249Q84 289 84 343Q84 398 98 436Q126 514 193 567T346 632Q347 632 373 632T440 633T520 634H680Q682 631 686 627T691 621T693 614T680 594H526Q364 594 353 592Q268 581 207 528T126 394Q123 378 123 343T126 292Q141 231 181 185T280 114Q329 92 415 92H462L506 147Q554 203 562 203H563Q572 203 578 196T582 178Q579 173 546 132L513 94L598 92H682Q693 81 693 73T680 52H480L349 -102L515 -104H682Q693 -115 693 -122",
      8843: "82 610T82 614T83 620T89 627T95 634H251Q378 634 409 633T469 623Q540 604 596 554T678 436Q691 397 691 343T678 249Q653 181 597 131T469 63Q427 52 362 52H315L213 -102L438 -104H662Q673 -115 673 -123Q673 -129 660 -144H186L151 -197Q114 -250 109 -253Q106 -254 104 -254Q100 -254 98 -253Q91 -253 87 -248T82 -235Q82 -230 109 -186L138 -144H115Q82 -144 82 -125Q82 -119 95 -104H166L266 49Q266 52 182 52H95Q82 65 82 72Q82 76 83 78T89 85T95 92H295L329 143Q365 195 369 198Q372 203 380 203Q385 203 391 197T398 185Q398 184 398 184L399 182Q399 175 369 129L344 94Q344 92 376 92Q402 92 422 94Q496 104 554 147T638 256Q651 295 651 343Q651 390 638 429Q613 494 555 537T422 592Q411 594 249 594H95Q82 610 82 614",
      8872: "55 678Q55 679 56 681T58 684T61 688T65 691T70 693T77 694Q88 692 95 679V464H540Q554 456 555 446Q555 442 554 439T552 434T549 430T546 428T542 426T539 424H95V270H539Q540 270 542 269T545 267T549 264T552 260T554 255T555 248Q554 238 540 230H95V15Q88 2 77 0Q73 0 70 1T65 3T61 6T59 9T57 13T55 16V678",
      8901: "71 0Q59 4 55 16V96L56 176Q59 180 66 187L70 189H209Q219 181 222 174V15Q219 10 209 1L140 0H71",
      8994: "77 122Q68 122 63 126T57 135T55 142Q55 151 68 176T111 235T177 302T271 356T388 378Q451 378 508 355T602 300T668 233T709 174T722 142Q722 124 704 122Q692 122 685 134T658 179T606 243Q511 338 390 338Q354 338 320 329Q251 312 193 263T97 141Q87 123 77 122",
      8995: "389 143Q324 143 266 164T171 215T107 277T67 330T55 358T60 371T77 378Q85 377 92 367T116 331T158 280Q256 182 389 182Q475 182 552 227T675 351Q688 378 704 378Q722 376 722 358Q722 352 710 330T670 276T605 215T511 164T389 143",
      9651: "99 -20Q84 -11 84 0Q84 5 148 145T278 424L342 563Q347 575 360 575Q368 575 375 570Q376 569 441 430T571 148T637 0Q637 -11 622 -20H99ZM476 260L360 509L248 266Q137 24 135 22Q135 20 360 20Q586 20 586 21L476 260",
      9661: "84 556Q84 567 99 576H622Q637 567 637 556Q637 551 572 409T441 127T375 -14Q368 -19 360 -19H358Q349 -19 342 -7T296 92Q249 193 211 275Q84 550 84 556ZM586 534Q586 536 361 536Q135 536 135 535L358 52L361 47L473 290Q584 532 586 534",
      10887: "102 168Q103 168 151 146T247 102T295 81Q299 85 322 144T344 206L218 268Q153 297 123 313T87 333T82 344T86 355Q104 369 291 455Q491 552 491 553L542 673Q581 767 590 784T609 801Q616 801 622 795T629 781Q629 773 586 677Q546 581 546 577L609 606Q669 635 673 635Q680 635 686 629T693 615Q693 610 692 608T670 593T604 561L524 521L400 226L542 157Q617 123 649 107T687 85T694 72Q694 66 690 60T679 54Q665 54 526 119Q394 186 386 186Q385 186 342 88L331 61L509 -23Q680 -105 688 -111Q693 -115 693 -122T688 -135T675 -141H673Q664 -141 491 -59Q320 21 316 21H315L249 -136Q183 -293 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -282 154 -261T181 -197T213 -119L280 41Q280 46 186 86Q157 101 121 119Q92 133 87 136T82 148Q82 155 88 161T102 168ZM418 370L466 495Q464 495 308 420T151 344T204 317T311 267T364 244Q364 247 418 370",
      10888: "97 54Q82 54 82 72Q82 79 86 84Q95 91 222 153L351 215L398 324L442 433L258 519Q95 597 87 604Q82 608 82 615T88 628T102 635Q107 635 424 484L458 468L524 630Q593 789 597 795Q601 801 609 801Q616 801 622 795T629 781L562 615L493 450L589 406Q665 371 679 362T694 344Q694 339 693 337T677 326T631 302T538 257Q504 241 465 223T406 195T386 186Q383 185 344 92T306 -3L486 81Q662 168 673 168Q680 168 686 162T693 148T689 137Q688 136 482 35L280 -59L233 -176Q184 -291 178 -299Q172 -303 166 -303T153 -297T146 -283Q146 -279 185 -186T224 -90Q225 -88 223 -88Q219 -88 193 -101Q109 -143 98 -143Q82 -138 82 -122Q82 -116 85 -113T108 -98T171 -67L249 -30L289 61Q297 81 307 107T321 144T326 157L218 106Q109 54 97 54ZM553 379Q480 412 480 415Q479 415 460 372T423 285T406 241Q408 240 516 291T624 344L553 379",
      10955: "82 -14T82 -7T95 15H431L529 170H435Q341 170 333 175Q149 218 98 368Q84 406 84 461Q84 515 98 555Q126 633 193 686T346 750Q347 750 373 750T440 751T520 752H680Q693 739 693 732Q693 727 680 712H526Q364 712 353 710Q268 700 207 646T126 512Q123 496 123 461T126 410Q141 350 180 304T280 232Q312 217 344 214T464 210H555L589 261Q613 301 620 311T635 321Q644 321 650 315T657 301Q657 296 651 286T630 252T604 212Q604 210 642 210H680Q693 197 693 190Q693 186 692 184T686 177T680 170H578L526 92L478 17L580 15H682Q693 4 693 -4T680 -25H451L353 -179L518 -181H682Q694 -193 694 -201Q694 -211 682 -219L504 -221H326L293 -272Q257 -332 246 -332Q238 -332 232 -326T225 -313Q225 -310 226 -308Q226 -305 251 -265T278 -223Q278 -221 186 -221H95Q93 -218 89 -214T84 -208T82 -201T95 -181H306L404 -25H249L93 -23L86 -19Q82 -14 82 -7",
      10956: "82 732Q82 739 95 752H251Q415 752 426 750Q539 736 615 657Q667 599 689 517Q692 496 692 461T689 406Q668 325 615 266Q522 170 382 170H355L326 95Q319 80 311 59T298 28T293 17Q293 15 486 15H680Q693 0 693 -6T680 -25H275L213 -179L449 -181H682Q693 -192 693 -199T680 -221H198L178 -270Q153 -333 139 -333Q132 -333 126 -327T119 -314T135 -266T153 -223Q153 -221 124 -221H95Q82 -207 82 -201T95 -181H171L233 -25H162L93 -23L86 -19Q82 -14 82 -7T95 15H251L313 170H202L93 172L86 177Q82 182 82 190Q82 199 93 210H211L329 212L349 261Q366 301 372 311T386 321Q392 321 399 315T407 302Q407 295 390 254T373 210Q374 209 377 209Q412 209 444 217Q512 231 564 273T638 377Q651 414 651 461Q651 509 638 548Q613 613 555 656T422 710Q411 712 249 712H95Q82 727 82 732"
    }, {}), texVariant$1;
  }
  var delimiters$1 = {}, hasRequiredDelimiters;
  function requireDelimiters() {
    return hasRequiredDelimiters || (hasRequiredDelimiters = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.delimiters = O.VSIZES = O.HDW3 = O.HDW2 = O.HDW1 = void 0;
      var _ = requireFontData$1();
      O.HDW1 = [0.75, 0.25, 0.875], O.HDW2 = [0.85, 0.349, 0.667], O.HDW3 = [0.583, 0.082, 0.5], O.VSIZES = [1, 1.2, 1.8, 2.4, 3];
      var H = { c: 47, dir: _.V, sizes: O.VSIZES }, V = { c: 175, dir: _.H, sizes: [0.5], stretch: [0, 175], HDW: [0.59, -0.544, 0.5] }, q = { c: 710, dir: _.H, sizes: [0.5, 0.556, 1, 1.444, 1.889] }, G = { c: 732, dir: _.H, sizes: [0.5, 0.556, 1, 1.444, 1.889] }, J = { c: 8211, dir: _.H, sizes: [0.5], stretch: [0, 8211], HDW: [0.285, -0.248, 0.5] }, Z = { c: 8592, dir: _.H, sizes: [1], stretch: [8592, 8722], HDW: O.HDW3 }, te = { c: 8594, dir: _.H, sizes: [1], stretch: [0, 8722, 8594], HDW: O.HDW3 }, ne = { c: 8596, dir: _.H, sizes: [1], stretch: [8592, 8722, 8594], HDW: O.HDW3 }, ee = { c: 8612, dir: _.H, stretch: [8592, 8722, 8739], HDW: O.HDW3, min: 1.278 }, ae = { c: 8614, dir: _.H, sizes: [1], stretch: [8739, 8722, 8594], HDW: O.HDW3 }, oe = { c: 8656, dir: _.H, sizes: [1], stretch: [8656, 61], HDW: O.HDW3 }, se = { c: 8658, dir: _.H, sizes: [1], stretch: [0, 61, 8658], HDW: O.HDW3 }, ce = { c: 8660, dir: _.H, sizes: [1], stretch: [8656, 61, 8658], HDW: O.HDW3 }, le = { c: 8722, dir: _.H, sizes: [0.778], stretch: [0, 8722], HDW: O.HDW3 }, ue = { c: 8739, dir: _.V, sizes: [1], stretch: [0, 8739], HDW: [0.627, 0.015, 0.333] }, de = {
        c: 9180,
        dir: _.H,
        sizes: [0.778, 1],
        schar: [8994, 8994],
        variants: [5, 0],
        stretch: [57680, 57684, 57681],
        HDW: [0.32, 0.2, 0.5]
      }, fe = {
        c: 9181,
        dir: _.H,
        sizes: [0.778, 1],
        schar: [8995, 8995],
        variants: [5, 0],
        stretch: [57682, 57684, 57683],
        HDW: [0.32, 0.2, 0.5]
      }, he = { c: 9182, dir: _.H, stretch: [57680, 57684, 57681, 57685], HDW: [0.32, 0.2, 0.5], min: 1.8 }, me = { c: 9183, dir: _.H, stretch: [57682, 57684, 57683, 57686], HDW: [0.32, 0.2, 0.5], min: 1.8 }, pe = { c: 10216, dir: _.V, sizes: O.VSIZES }, ye = { c: 10217, dir: _.V, sizes: O.VSIZES }, ge = { c: 10502, dir: _.H, stretch: [8656, 61, 8739], HDW: O.HDW3, min: 1.278 }, ve = { c: 10503, dir: _.H, stretch: [8872, 61, 8658], HDW: O.HDW3, min: 1.278 };
      O.delimiters = {
        40: { dir: _.V, sizes: O.VSIZES, stretch: [9115, 9116, 9117], HDW: [0.85, 0.349, 0.875] },
        41: { dir: _.V, sizes: O.VSIZES, stretch: [9118, 9119, 9120], HDW: [0.85, 0.349, 0.875] },
        45: le,
        47: H,
        61: { dir: _.H, sizes: [0.778], stretch: [0, 61], HDW: O.HDW3 },
        91: { dir: _.V, sizes: O.VSIZES, stretch: [9121, 9122, 9123], HDW: O.HDW2 },
        92: { dir: _.V, sizes: O.VSIZES },
        93: { dir: _.V, sizes: O.VSIZES, stretch: [9124, 9125, 9126], HDW: O.HDW2 },
        94: q,
        95: J,
        123: { dir: _.V, sizes: O.VSIZES, stretch: [9127, 9130, 9129, 9128], HDW: [0.85, 0.349, 0.889] },
        124: { dir: _.V, sizes: [1], stretch: [0, 8739], HDW: [0.75, 0.25, 0.333] },
        125: { dir: _.V, sizes: O.VSIZES, stretch: [9131, 9130, 9133, 9132], HDW: [0.85, 0.349, 0.889] },
        126: G,
        175: V,
        710: q,
        713: V,
        732: G,
        770: q,
        771: G,
        818: J,
        8211: J,
        8212: J,
        8213: J,
        8214: { dir: _.V, sizes: [0.602, 1], schar: [0, 8741], variants: [1, 0], stretch: [0, 8741], HDW: [0.602, 0, 0.556] },
        8215: J,
        8254: V,
        8407: te,
        8592: Z,
        8593: { dir: _.V, sizes: [0.888], stretch: [8593, 9168], HDW: [0.6, 0, 0.667] },
        8594: te,
        8595: { dir: _.V, sizes: [0.888], stretch: [0, 9168, 8595], HDW: [0.6, 0, 0.667] },
        8596: ne,
        8597: { dir: _.V, sizes: [1.044], stretch: [8593, 9168, 8595], HDW: O.HDW1 },
        8606: { dir: _.H, sizes: [1], stretch: [8606, 8722], HDW: O.HDW3 },
        8608: { dir: _.H, sizes: [1], stretch: [0, 8722, 8608], HDW: O.HDW3 },
        8612: ee,
        8613: { dir: _.V, stretch: [8593, 9168, 8869], HDW: O.HDW1, min: 1.555 },
        8614: ae,
        8615: { dir: _.V, stretch: [8868, 9168, 8595], HDW: O.HDW1, min: 1.555 },
        8624: { dir: _.V, sizes: [0.722], stretch: [8624, 9168], HDW: O.HDW1 },
        8625: { dir: _.V, sizes: [0.722], stretch: [8625, 9168], HDW: O.HDW1 },
        8636: { dir: _.H, sizes: [1], stretch: [8636, 8722], HDW: O.HDW3 },
        8637: { dir: _.H, sizes: [1], stretch: [8637, 8722], HDW: O.HDW3 },
        8638: { dir: _.V, sizes: [0.888], stretch: [8638, 9168], HDW: O.HDW1 },
        8639: { dir: _.V, sizes: [0.888], stretch: [8639, 9168], HDW: O.HDW1 },
        8640: { dir: _.H, sizes: [1], stretch: [0, 8722, 8640], HDW: O.HDW3 },
        8641: { dir: _.H, sizes: [1], stretch: [0, 8722, 8641], HDW: O.HDW3 },
        8642: { dir: _.V, sizes: [0.888], stretch: [0, 9168, 8642], HDW: O.HDW1 },
        8643: { dir: _.V, sizes: [0.888], stretch: [0, 9168, 8643], HDW: O.HDW1 },
        8656: oe,
        8657: { dir: _.V, sizes: [0.888], stretch: [8657, 8214], HDW: [0.599, 0, 0.778] },
        8658: se,
        8659: { dir: _.V, sizes: [0.888], stretch: [0, 8214, 8659], HDW: [0.6, 0, 0.778] },
        8660: ce,
        8661: { dir: _.V, sizes: [1.044], stretch: [8657, 8214, 8659], HDW: [0.75, 0.25, 0.778] },
        8666: { dir: _.H, sizes: [1], stretch: [8666, 8801], HDW: [0.464, -0.036, 0.5] },
        8667: { dir: _.H, sizes: [1], stretch: [0, 8801, 8667], HDW: [0.464, -0.036, 0.5] },
        8722: le,
        8725: H,
        8730: { dir: _.V, sizes: O.VSIZES, stretch: [57345, 57344, 9143], fullExt: [0.65, 2.3], HDW: [0.85, 0.35, 1.056] },
        8739: ue,
        8741: { dir: _.V, sizes: [1], stretch: [0, 8741], HDW: [0.627, 0.015, 0.556] },
        8968: { dir: _.V, sizes: O.VSIZES, stretch: [9121, 9122], HDW: O.HDW2 },
        8969: { dir: _.V, sizes: O.VSIZES, stretch: [9124, 9125], HDW: O.HDW2 },
        8970: { dir: _.V, sizes: O.VSIZES, stretch: [0, 9122, 9123], HDW: O.HDW2 },
        8971: { dir: _.V, sizes: O.VSIZES, stretch: [0, 9125, 9126], HDW: O.HDW2 },
        8978: de,
        8994: de,
        8995: fe,
        9001: pe,
        9002: ye,
        9130: { dir: _.V, sizes: [0.32], stretch: [9130, 9130, 9130], HDW: [0.29, 0.015, 0.889] },
        9135: J,
        9136: { dir: _.V, sizes: [0.989], stretch: [9127, 9130, 9133], HDW: [0.75, 0.25, 0.889] },
        9137: { dir: _.V, sizes: [0.989], stretch: [9131, 9130, 9129], HDW: [0.75, 0.25, 0.889] },
        9140: { dir: _.H, stretch: [9484, 8722, 9488], HDW: O.HDW3, min: 1 },
        9141: { dir: _.H, stretch: [9492, 8722, 9496], HDW: O.HDW3, min: 1 },
        9168: { dir: _.V, sizes: [0.602, 1], schar: [0, 8739], variants: [1, 0], stretch: [0, 8739], HDW: [0.602, 0, 0.333] },
        9180: de,
        9181: fe,
        9182: he,
        9183: me,
        9184: { dir: _.H, stretch: [714, 713, 715], HDW: [0.59, -0.544, 0.5], min: 1 },
        9185: { dir: _.H, stretch: [715, 713, 714], HDW: [0.59, -0.544, 0.5], min: 1 },
        9472: J,
        10072: ue,
        10216: pe,
        10217: ye,
        10222: { dir: _.V, sizes: [0.989], stretch: [9127, 9130, 9129], HDW: [0.75, 0.25, 0.889] },
        10223: { dir: _.V, sizes: [0.989], stretch: [9131, 9130, 9133], HDW: [0.75, 0.25, 0.889] },
        10229: Z,
        10230: te,
        10231: ne,
        10232: oe,
        10233: se,
        10234: ce,
        10235: ee,
        10236: ae,
        10237: ge,
        10238: ve,
        10502: ge,
        10503: ve,
        10574: { dir: _.H, stretch: [8636, 8722, 8640], HDW: O.HDW3, min: 2 },
        10575: { dir: _.V, stretch: [8638, 9168, 8642], HDW: O.HDW1, min: 1.776 },
        10576: { dir: _.H, stretch: [8637, 8722, 8641], HDW: O.HDW3, min: 2 },
        10577: { dir: _.V, stretch: [8639, 9168, 8643], HDW: O.HDW1, min: 0.5 },
        10586: { dir: _.H, stretch: [8636, 8722, 8739], HDW: O.HDW3, min: 1.278 },
        10587: { dir: _.H, stretch: [8739, 8722, 8640], HDW: O.HDW3, min: 1.278 },
        10588: { dir: _.V, stretch: [8638, 9168, 8869], HDW: O.HDW1, min: 1.556 },
        10589: { dir: _.V, stretch: [8868, 9168, 8642], HDW: O.HDW1, min: 1.556 },
        10590: { dir: _.H, stretch: [8637, 8722, 8739], HDW: O.HDW3, min: 1.278 },
        10591: { dir: _.H, stretch: [8739, 8722, 8641], HDW: O.HDW3, min: 1.278 },
        10592: { dir: _.V, stretch: [8639, 9168, 8869], HDW: O.HDW1, min: 1.776 },
        10593: { dir: _.V, stretch: [8868, 9168, 8643], HDW: O.HDW1, min: 1.776 },
        12296: pe,
        12297: ye,
        65079: he,
        65080: me
      };
    }(delimiters$1)), delimiters$1;
  }
  var hasRequiredTex;
  function requireTex() {
    if (hasRequiredTex) return tex$1;
    hasRequiredTex = 1;
    var O = tex$1.__extends || /* @__PURE__ */ function() {
      var xe = function(Le, Qe) {
        return xe = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ke, Me) {
          ke.__proto__ = Me;
        } || function(ke, Me) {
          for (var Re in Me) Object.prototype.hasOwnProperty.call(Me, Re) && (ke[Re] = Me[Re]);
        }, xe(Le, Qe);
      };
      return function(Le, Qe) {
        if (typeof Qe != "function" && Qe !== null)
          throw new TypeError("Class extends value " + String(Qe) + " is not a constructor or null");
        xe(Le, Qe);
        function ke() {
          this.constructor = Le;
        }
        Le.prototype = Qe === null ? Object.create(Qe) : (ke.prototype = Qe.prototype, new ke());
      };
    }(), _ = tex$1.__values || function(xe) {
      var Le = typeof Symbol == "function" && Symbol.iterator, Qe = Le && xe[Le], ke = 0;
      if (Qe) return Qe.call(xe);
      if (xe && typeof xe.length == "number") return {
        next: function() {
          return xe && ke >= xe.length && (xe = void 0), { value: xe && xe[ke++], done: !xe };
        }
      };
      throw new TypeError(Le ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(tex$1, "__esModule", { value: !0 }), tex$1.TeXFont = void 0;
    var H = requireFontData(), V = requireTex$1(), q = requireBoldItalic(), G = requireBold(), J = requireDoubleStruck(), Z = requireFrakturBold(), te = requireFraktur(), ne = requireItalic(), ee = requireLargeop(), ae = requireMonospace(), oe = requireNormal(), se = requireSansSerifBoldItalic(), ce = requireSansSerifBold(), le = requireSansSerifItalic(), ue = requireSansSerif(), de = requireScriptBold(), fe = requireScript(), he = requireSmallop(), me = requireTexCalligraphicBold(), pe = requireTexCalligraphic(), ye = requireTexMathit(), ge = requireTexOldstyleBold(), ve = requireTexOldstyle(), be = requireTexSize3(), Ce = requireTexSize4(), Se = requireTexVariant(), we = requireDelimiters(), Ee = function(xe) {
      O(Le, xe);
      function Le(Qe) {
        var ke, Me;
        Qe === void 0 && (Qe = null);
        var Re = xe.call(this, Qe) || this, De = Re.constructor;
        try {
          for (var Pe = _(Object.keys(De.variantCacheIds)), $e = Pe.next(); !$e.done; $e = Pe.next()) {
            var qe = $e.value;
            Re.variant[qe].cacheID = "TEX-" + De.variantCacheIds[qe];
          }
        } catch (Xe) {
          ke = { error: Xe };
        } finally {
          try {
            $e && !$e.done && (Me = Pe.return) && Me.call(Pe);
          } finally {
            if (ke) throw ke.error;
          }
        }
        return Re;
      }
      return Le.defaultDelimiters = we.delimiters, Le.defaultChars = {
        normal: oe.normal,
        bold: G.bold,
        italic: ne.italic,
        "bold-italic": q.boldItalic,
        "double-struck": J.doubleStruck,
        fraktur: te.fraktur,
        "bold-fraktur": Z.frakturBold,
        script: fe.script,
        "bold-script": de.scriptBold,
        "sans-serif": ue.sansSerif,
        "bold-sans-serif": ce.sansSerifBold,
        "sans-serif-italic": le.sansSerifItalic,
        "sans-serif-bold-italic": se.sansSerifBoldItalic,
        monospace: ae.monospace,
        "-smallop": he.smallop,
        "-largeop": ee.largeop,
        "-size3": be.texSize3,
        "-size4": Ce.texSize4,
        "-tex-calligraphic": pe.texCalligraphic,
        "-tex-bold-calligraphic": me.texCalligraphicBold,
        "-tex-mathit": ye.texMathit,
        "-tex-oldstyle": ve.texOldstyle,
        "-tex-bold-oldstyle": ge.texOldstyleBold,
        "-tex-variant": Se.texVariant
      }, Le.variantCacheIds = {
        normal: "N",
        bold: "B",
        italic: "I",
        "bold-italic": "BI",
        "double-struck": "D",
        fraktur: "F",
        "bold-fraktur": "BF",
        script: "S",
        "bold-script": "BS",
        "sans-serif": "SS",
        "bold-sans-serif": "BSS",
        "sans-serif-italic": "SSI",
        "sans-serif-bold-italic": "SSBI",
        monospace: "M",
        "-smallop": "SO",
        "-largeop": "LO",
        "-size3": "S3",
        "-size4": "S4",
        "-tex-calligraphic": "C",
        "-tex-bold-calligraphic": "BC",
        "-tex-mathit": "MI",
        "-tex-oldstyle": "OS",
        "-tex-bold-oldstyle": "BOS",
        "-tex-variant": "V"
      }, Le;
    }((0, V.CommonTeXFontMixin)(H.SVGFontData));
    return tex$1.TeXFont = Ee, tex$1;
  }
  var FontCache = {}, hasRequiredFontCache;
  function requireFontCache() {
    if (hasRequiredFontCache) return FontCache;
    hasRequiredFontCache = 1, Object.defineProperty(FontCache, "__esModule", { value: !0 }), FontCache.FontCache = void 0;
    var O = function() {
      function _(H) {
        this.cache = /* @__PURE__ */ new Map(), this.defs = null, this.localID = "", this.nextID = 0, this.jax = H;
      }
      return _.prototype.cachePath = function(H, V, q) {
        var G = "MJX-" + this.localID + (this.jax.font.getVariant(H).cacheID || "") + "-" + V;
        return this.cache.has(G) || (this.cache.set(G, q), this.jax.adaptor.append(this.defs, this.jax.svg("path", { id: G, d: q }))), G;
      }, _.prototype.clearLocalID = function() {
        this.localID = "";
      }, _.prototype.useLocalID = function(H) {
        H === void 0 && (H = null), this.localID = (H ?? ++this.nextID) + (H === "" ? "" : "-");
      }, _.prototype.clearCache = function() {
        this.cache = /* @__PURE__ */ new Map(), this.defs = this.jax.svg("defs");
      }, _.prototype.getCache = function() {
        return this.defs;
      }, _;
    }();
    return FontCache.FontCache = O, FontCache;
  }
  var hasRequiredSvg;
  function requireSvg() {
    return hasRequiredSvg || (hasRequiredSvg = 1, function(O) {
      var _ = svg$2.__extends || /* @__PURE__ */ function() {
        var ae = function(oe, se) {
          return ae = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ce, le) {
            ce.__proto__ = le;
          } || function(ce, le) {
            for (var ue in le) Object.prototype.hasOwnProperty.call(le, ue) && (ce[ue] = le[ue]);
          }, ae(oe, se);
        };
        return function(oe, se) {
          if (typeof se != "function" && se !== null)
            throw new TypeError("Class extends value " + String(se) + " is not a constructor or null");
          ae(oe, se);
          function ce() {
            this.constructor = oe;
          }
          oe.prototype = se === null ? Object.create(se) : (ce.prototype = se.prototype, new ce());
        };
      }(), H = svg$2.__assign || function() {
        return H = Object.assign || function(ae) {
          for (var oe, se = 1, ce = arguments.length; se < ce; se++) {
            oe = arguments[se];
            for (var le in oe) Object.prototype.hasOwnProperty.call(oe, le) && (ae[le] = oe[le]);
          }
          return ae;
        }, H.apply(this, arguments);
      }, V = svg$2.__read || function(ae, oe) {
        var se = typeof Symbol == "function" && ae[Symbol.iterator];
        if (!se) return ae;
        var ce = se.call(ae), le, ue = [], de;
        try {
          for (; (oe === void 0 || oe-- > 0) && !(le = ce.next()).done; ) ue.push(le.value);
        } catch (fe) {
          de = { error: fe };
        } finally {
          try {
            le && !le.done && (se = ce.return) && se.call(ce);
          } finally {
            if (de) throw de.error;
          }
        }
        return ue;
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.SVG = O.XLINKNS = O.SVGNS = void 0;
      var q = requireOutputJax(), G = requireWrapperFactory(), J = requireTex(), Z = requireFontCache(), te = requireString(), ne = requireLengths();
      O.SVGNS = "http://www.w3.org/2000/svg", O.XLINKNS = "http://www.w3.org/1999/xlink";
      var ee = function(ae) {
        _(oe, ae);
        function oe(se) {
          se === void 0 && (se = null);
          var ce = ae.call(this, se, G.SVGWrapperFactory, J.TeXFont) || this;
          return ce.minwidth = 0, ce.shift = 0, ce.container = null, ce.svgStyles = null, ce.fontCache = new Z.FontCache(ce), ce;
        }
        return oe.prototype.initialize = function() {
          this.options.fontCache === "global" && this.fontCache.clearCache();
        }, oe.prototype.clearFontCache = function() {
          this.fontCache.clearCache();
        }, oe.prototype.reset = function() {
          this.clearFontCache();
        }, oe.prototype.setScale = function(se) {
          this.options.scale !== 1 && this.adaptor.setStyle(se, "fontSize", (0, ne.percent)(this.options.scale));
        }, oe.prototype.escaped = function(se, ce) {
          return this.setDocument(ce), this.html("span", {}, [this.text(se.math)]);
        }, oe.prototype.styleSheet = function(se) {
          if (this.svgStyles)
            return this.svgStyles;
          var ce = this.svgStyles = ae.prototype.styleSheet.call(this, se);
          return this.adaptor.setAttribute(ce, "id", oe.STYLESHEETID), ce;
        }, oe.prototype.pageElements = function(se) {
          return this.options.fontCache === "global" && !this.findCache(se) ? this.svg("svg", { id: oe.FONTCACHEID, style: { display: "none" } }, [this.fontCache.getCache()]) : null;
        }, oe.prototype.findCache = function(se) {
          for (var ce = this.adaptor, le = ce.tags(ce.body(se.document), "svg"), ue = le.length - 1; ue >= 0; ue--)
            if (this.adaptor.getAttribute(le[ue], "id") === oe.FONTCACHEID)
              return !0;
          return !1;
        }, oe.prototype.processMath = function(se, ce) {
          var le = this.container;
          this.container = ce;
          var ue = this.factory.wrap(se), de = V(this.createRoot(ue), 2), fe = de[0], he = de[1];
          this.typesetSVG(ue, fe, he), this.container = le;
        }, oe.prototype.createRoot = function(se) {
          var ce = se.getOuterBBox(), le = ce.w, ue = ce.h, de = ce.d, fe = ce.pwidth, he = se.metrics.em / 1e3, me = Math.max(le, he), pe = Math.max(ue + de, he), ye = this.svg("g", {
            stroke: "currentColor",
            fill: "currentColor",
            "stroke-width": 0,
            transform: "scale(1,-1)"
          }), ge = this.adaptor, ve = ge.append(this.container, this.svg("svg", {
            xmlns: O.SVGNS,
            width: this.ex(me),
            height: this.ex(pe),
            role: "img",
            focusable: !1,
            style: { "vertical-align": this.ex(-de) },
            viewBox: [0, this.fixed(-ue * 1e3, 1), this.fixed(me * 1e3, 1), this.fixed(pe * 1e3, 1)].join(" ")
          }, [ye]));
          if (me === 1e-3 && (ge.setAttribute(ve, "preserveAspectRatio", "xMidYMid slice"), le < 0 && ge.setStyle(this.container, "margin-right", this.ex(le))), fe) {
            ge.setStyle(ve, "min-width", this.ex(me)), ge.setAttribute(ve, "width", fe), ge.removeAttribute(ve, "viewBox");
            var be = this.fixed(se.metrics.ex / (this.font.params.x_height * 1e3), 6);
            ge.setAttribute(ye, "transform", "scale(".concat(be, ",-").concat(be, ") translate(0, ").concat(this.fixed(-ue * 1e3, 1), ")"));
          }
          return this.options.fontCache !== "none" && ge.setAttribute(ve, "xmlns:xlink", O.XLINKNS), [ve, ye];
        }, oe.prototype.typesetSVG = function(se, ce, le) {
          var ue = this.adaptor;
          if (this.minwidth = this.shift = 0, this.options.fontCache === "local" && (this.fontCache.clearCache(), this.fontCache.useLocalID(this.options.localID), ue.insert(this.fontCache.getCache(), le)), se.toSVG(le), this.fontCache.clearLocalID(), this.minwidth)
            ue.setStyle(ce, "minWidth", this.ex(this.minwidth)), ue.setStyle(this.container, "minWidth", this.ex(this.minwidth));
          else if (this.shift) {
            var de = ue.getAttribute(this.container, "justify") || "center";
            this.setIndent(ce, de, this.shift);
          }
        }, oe.prototype.setIndent = function(se, ce, le) {
          (ce === "center" || ce === "left") && this.adaptor.setStyle(se, "margin-left", this.ex(le)), (ce === "center" || ce === "right") && this.adaptor.setStyle(se, "margin-right", this.ex(-le));
        }, oe.prototype.ex = function(se) {
          return se /= this.font.params.x_height, Math.abs(se) < 1e-3 ? "0" : se.toFixed(3).replace(/\.?0+$/, "") + "ex";
        }, oe.prototype.svg = function(se, ce, le) {
          return ce === void 0 && (ce = {}), le === void 0 && (le = []), this.html(se, ce, le, O.SVGNS);
        }, oe.prototype.unknownText = function(se, ce) {
          var le = this.math.metrics, ue = this.font.params.x_height / le.ex * le.em * 1e3, de = this.svg("text", {
            "data-variant": ce,
            transform: "scale(1,-1)",
            "font-size": this.fixed(ue, 1) + "px"
          }, [this.text(se)]), fe = this.adaptor;
          if (ce !== "-explicitFont") {
            var he = (0, te.unicodeChars)(se);
            if (he.length !== 1 || he[0] < 119808 || he[0] > 120831) {
              var me = V(this.font.getCssFont(ce), 3), pe = me[0], ye = me[1], ge = me[2];
              fe.setAttribute(de, "font-family", pe), ye && fe.setAttribute(de, "font-style", "italic"), ge && fe.setAttribute(de, "font-weight", "bold");
            }
          }
          return de;
        }, oe.prototype.measureTextNode = function(se) {
          var ce = this.adaptor;
          se = ce.clone(se), ce.removeAttribute(se, "transform");
          var le = this.fixed(this.font.params.x_height * 1e3, 1), ue = this.svg("svg", {
            position: "absolute",
            visibility: "hidden",
            width: "1ex",
            height: "1ex",
            viewBox: [0, 0, le, le].join(" ")
          }, [se]);
          ce.append(ce.body(ce.document), ue);
          var de = ce.nodeSize(se, 1e3, !0)[0];
          return ce.remove(ue), { w: de, h: 0.75, d: 0.2 };
        }, oe.NAME = "SVG", oe.OPTIONS = H(H({}, q.CommonOutputJax.OPTIONS), { internalSpeechTitles: !0, titleID: 0, fontCache: "local", localID: null }), oe.commonStyles = {
          'mjx-container[jax="SVG"]': {
            direction: "ltr"
          },
          'mjx-container[jax="SVG"] > svg': {
            overflow: "visible",
            "min-height": "1px",
            "min-width": "1px"
          },
          'mjx-container[jax="SVG"] > svg a': {
            fill: "blue",
            stroke: "blue"
          }
        }, oe.FONTCACHEID = "MJX-SVG-global-cache", oe.STYLESHEETID = "MJX-SVG-styles", oe;
      }(q.CommonOutputJax);
      O.SVG = ee;
    }(svg$2)), svg$2;
  }
  var svgExports = requireSvg(), browserAdaptor = {}, HTMLAdaptor = {}, DOMAdaptor = {}, hasRequiredDOMAdaptor;
  function requireDOMAdaptor() {
    if (hasRequiredDOMAdaptor) return DOMAdaptor;
    hasRequiredDOMAdaptor = 1;
    var O = DOMAdaptor.__values || function(H) {
      var V = typeof Symbol == "function" && Symbol.iterator, q = V && H[V], G = 0;
      if (q) return q.call(H);
      if (H && typeof H.length == "number") return {
        next: function() {
          return H && G >= H.length && (H = void 0), { value: H && H[G++], done: !H };
        }
      };
      throw new TypeError(V ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(DOMAdaptor, "__esModule", { value: !0 }), DOMAdaptor.AbstractDOMAdaptor = void 0;
    var _ = function() {
      function H(V) {
        V === void 0 && (V = null), this.document = V;
      }
      return H.prototype.node = function(V, q, G, J) {
        var Z, te;
        q === void 0 && (q = {}), G === void 0 && (G = []);
        var ne = this.create(V, J);
        this.setAttributes(ne, q);
        try {
          for (var ee = O(G), ae = ee.next(); !ae.done; ae = ee.next()) {
            var oe = ae.value;
            this.append(ne, oe);
          }
        } catch (se) {
          Z = { error: se };
        } finally {
          try {
            ae && !ae.done && (te = ee.return) && te.call(ee);
          } finally {
            if (Z) throw Z.error;
          }
        }
        return ne;
      }, H.prototype.setAttributes = function(V, q) {
        var G, J, Z, te, ne, ee;
        if (q.style && typeof q.style != "string")
          try {
            for (var ae = O(Object.keys(q.style)), oe = ae.next(); !oe.done; oe = ae.next()) {
              var se = oe.value;
              this.setStyle(V, se.replace(/-([a-z])/g, function(fe, he) {
                return he.toUpperCase();
              }), q.style[se]);
            }
          } catch (fe) {
            G = { error: fe };
          } finally {
            try {
              oe && !oe.done && (J = ae.return) && J.call(ae);
            } finally {
              if (G) throw G.error;
            }
          }
        if (q.properties)
          try {
            for (var ce = O(Object.keys(q.properties)), le = ce.next(); !le.done; le = ce.next()) {
              var se = le.value;
              V[se] = q.properties[se];
            }
          } catch (fe) {
            Z = { error: fe };
          } finally {
            try {
              le && !le.done && (te = ce.return) && te.call(ce);
            } finally {
              if (Z) throw Z.error;
            }
          }
        try {
          for (var ue = O(Object.keys(q)), de = ue.next(); !de.done; de = ue.next()) {
            var se = de.value;
            (se !== "style" || typeof q.style == "string") && se !== "properties" && this.setAttribute(V, se, q[se]);
          }
        } catch (fe) {
          ne = { error: fe };
        } finally {
          try {
            de && !de.done && (ee = ue.return) && ee.call(ue);
          } finally {
            if (ne) throw ne.error;
          }
        }
      }, H.prototype.replace = function(V, q) {
        return this.insert(V, q), this.remove(q), q;
      }, H.prototype.childNode = function(V, q) {
        return this.childNodes(V)[q];
      }, H.prototype.allClasses = function(V) {
        var q = this.getAttribute(V, "class");
        return q ? q.replace(/  +/g, " ").replace(/^ /, "").replace(/ $/, "").split(/ /) : [];
      }, H;
    }();
    return DOMAdaptor.AbstractDOMAdaptor = _, DOMAdaptor;
  }
  var hasRequiredHTMLAdaptor;
  function requireHTMLAdaptor() {
    if (hasRequiredHTMLAdaptor) return HTMLAdaptor;
    hasRequiredHTMLAdaptor = 1;
    var O = HTMLAdaptor.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = HTMLAdaptor.__values || function(q) {
      var G = typeof Symbol == "function" && Symbol.iterator, J = G && q[G], Z = 0;
      if (J) return J.call(q);
      if (q && typeof q.length == "number") return {
        next: function() {
          return q && Z >= q.length && (q = void 0), { value: q && q[Z++], done: !q };
        }
      };
      throw new TypeError(G ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(HTMLAdaptor, "__esModule", { value: !0 }), HTMLAdaptor.HTMLAdaptor = void 0;
    var H = requireDOMAdaptor(), V = function(q) {
      O(G, q);
      function G(J) {
        var Z = q.call(this, J.document) || this;
        return Z.window = J, Z.parser = new J.DOMParser(), Z;
      }
      return G.prototype.parse = function(J, Z) {
        return Z === void 0 && (Z = "text/html"), this.parser.parseFromString(J, Z);
      }, G.prototype.create = function(J, Z) {
        return Z ? this.document.createElementNS(Z, J) : this.document.createElement(J);
      }, G.prototype.text = function(J) {
        return this.document.createTextNode(J);
      }, G.prototype.head = function(J) {
        return J.head || J;
      }, G.prototype.body = function(J) {
        return J.body || J;
      }, G.prototype.root = function(J) {
        return J.documentElement || J;
      }, G.prototype.doctype = function(J) {
        return J.doctype ? "<!DOCTYPE ".concat(J.doctype.name, ">") : "";
      }, G.prototype.tags = function(J, Z, te) {
        te === void 0 && (te = null);
        var ne = te ? J.getElementsByTagNameNS(te, Z) : J.getElementsByTagName(Z);
        return Array.from(ne);
      }, G.prototype.getElements = function(J, Z) {
        var te, ne, ee = [];
        try {
          for (var ae = _(J), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            typeof se == "string" ? ee = ee.concat(Array.from(this.document.querySelectorAll(se))) : Array.isArray(se) || se instanceof this.window.NodeList || se instanceof this.window.HTMLCollection ? ee = ee.concat(Array.from(se)) : ee.push(se);
          }
        } catch (ce) {
          te = { error: ce };
        } finally {
          try {
            oe && !oe.done && (ne = ae.return) && ne.call(ae);
          } finally {
            if (te) throw te.error;
          }
        }
        return ee;
      }, G.prototype.contains = function(J, Z) {
        return J.contains(Z);
      }, G.prototype.parent = function(J) {
        return J.parentNode;
      }, G.prototype.append = function(J, Z) {
        return J.appendChild(Z);
      }, G.prototype.insert = function(J, Z) {
        return this.parent(Z).insertBefore(J, Z);
      }, G.prototype.remove = function(J) {
        return this.parent(J).removeChild(J);
      }, G.prototype.replace = function(J, Z) {
        return this.parent(Z).replaceChild(J, Z);
      }, G.prototype.clone = function(J) {
        return J.cloneNode(!0);
      }, G.prototype.split = function(J, Z) {
        return J.splitText(Z);
      }, G.prototype.next = function(J) {
        return J.nextSibling;
      }, G.prototype.previous = function(J) {
        return J.previousSibling;
      }, G.prototype.firstChild = function(J) {
        return J.firstChild;
      }, G.prototype.lastChild = function(J) {
        return J.lastChild;
      }, G.prototype.childNodes = function(J) {
        return Array.from(J.childNodes);
      }, G.prototype.childNode = function(J, Z) {
        return J.childNodes[Z];
      }, G.prototype.kind = function(J) {
        var Z = J.nodeType;
        return Z === 1 || Z === 3 || Z === 8 ? J.nodeName.toLowerCase() : "";
      }, G.prototype.value = function(J) {
        return J.nodeValue || "";
      }, G.prototype.textContent = function(J) {
        return J.textContent;
      }, G.prototype.innerHTML = function(J) {
        return J.innerHTML;
      }, G.prototype.outerHTML = function(J) {
        return J.outerHTML;
      }, G.prototype.serializeXML = function(J) {
        var Z = new this.window.XMLSerializer();
        return Z.serializeToString(J);
      }, G.prototype.setAttribute = function(J, Z, te, ne) {
        return ne === void 0 && (ne = null), ne ? (Z = ne.replace(/.*\//, "") + ":" + Z.replace(/^.*:/, ""), J.setAttributeNS(ne, Z, te)) : J.setAttribute(Z, te);
      }, G.prototype.getAttribute = function(J, Z) {
        return J.getAttribute(Z);
      }, G.prototype.removeAttribute = function(J, Z) {
        return J.removeAttribute(Z);
      }, G.prototype.hasAttribute = function(J, Z) {
        return J.hasAttribute(Z);
      }, G.prototype.allAttributes = function(J) {
        return Array.from(J.attributes).map(function(Z) {
          return { name: Z.name, value: Z.value };
        });
      }, G.prototype.addClass = function(J, Z) {
        J.classList ? J.classList.add(Z) : J.className = (J.className + " " + Z).trim();
      }, G.prototype.removeClass = function(J, Z) {
        J.classList ? J.classList.remove(Z) : J.className = J.className.split(/ /).filter(function(te) {
          return te !== Z;
        }).join(" ");
      }, G.prototype.hasClass = function(J, Z) {
        return J.classList ? J.classList.contains(Z) : J.className.split(/ /).indexOf(Z) >= 0;
      }, G.prototype.setStyle = function(J, Z, te) {
        J.style[Z] = te;
      }, G.prototype.getStyle = function(J, Z) {
        return J.style[Z];
      }, G.prototype.allStyles = function(J) {
        return J.style.cssText;
      }, G.prototype.insertRules = function(J, Z) {
        var te, ne;
        try {
          for (var ee = _(Z.reverse()), ae = ee.next(); !ae.done; ae = ee.next()) {
            var oe = ae.value;
            try {
              J.sheet.insertRule(oe, 0);
            } catch (se) {
              console.warn("MathJax: can't insert css rule '".concat(oe, "': ").concat(se.message));
            }
          }
        } catch (se) {
          te = { error: se };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (te) throw te.error;
          }
        }
      }, G.prototype.fontSize = function(J) {
        var Z = this.window.getComputedStyle(J);
        return parseFloat(Z.fontSize);
      }, G.prototype.fontFamily = function(J) {
        var Z = this.window.getComputedStyle(J);
        return Z.fontFamily || "";
      }, G.prototype.nodeSize = function(J, Z, te) {
        if (Z === void 0 && (Z = 1), te === void 0 && (te = !1), te && J.getBBox) {
          var ne = J.getBBox(), ee = ne.width, ae = ne.height;
          return [ee / Z, ae / Z];
        }
        return [J.offsetWidth / Z, J.offsetHeight / Z];
      }, G.prototype.nodeBBox = function(J) {
        var Z = J.getBoundingClientRect(), te = Z.left, ne = Z.right, ee = Z.top, ae = Z.bottom;
        return { left: te, right: ne, top: ee, bottom: ae };
      }, G;
    }(H.AbstractDOMAdaptor);
    return HTMLAdaptor.HTMLAdaptor = V, HTMLAdaptor;
  }
  var hasRequiredBrowserAdaptor;
  function requireBrowserAdaptor() {
    if (hasRequiredBrowserAdaptor) return browserAdaptor;
    hasRequiredBrowserAdaptor = 1, Object.defineProperty(browserAdaptor, "__esModule", { value: !0 }), browserAdaptor.browserAdaptor = void 0;
    var O = requireHTMLAdaptor();
    function _() {
      return new O.HTMLAdaptor(window);
    }
    return browserAdaptor.browserAdaptor = _, browserAdaptor;
  }
  var browserAdaptorExports = requireBrowserAdaptor(), html = {}, HTMLHandler = {}, Handler$2 = {}, MathDocument = {}, MathList = {}, LinkedList = {}, hasRequiredLinkedList;
  function requireLinkedList() {
    return hasRequiredLinkedList || (hasRequiredLinkedList = 1, function(O) {
      var _ = LinkedList.__generator || function(Z, te) {
        var ne = { label: 0, sent: function() {
          if (oe[0] & 1) throw oe[1];
          return oe[1];
        }, trys: [], ops: [] }, ee, ae, oe, se;
        return se = { next: ce(0), throw: ce(1), return: ce(2) }, typeof Symbol == "function" && (se[Symbol.iterator] = function() {
          return this;
        }), se;
        function ce(ue) {
          return function(de) {
            return le([ue, de]);
          };
        }
        function le(ue) {
          if (ee) throw new TypeError("Generator is already executing.");
          for (; ne; ) try {
            if (ee = 1, ae && (oe = ue[0] & 2 ? ae.return : ue[0] ? ae.throw || ((oe = ae.return) && oe.call(ae), 0) : ae.next) && !(oe = oe.call(ae, ue[1])).done) return oe;
            switch (ae = 0, oe && (ue = [ue[0] & 2, oe.value]), ue[0]) {
              case 0:
              case 1:
                oe = ue;
                break;
              case 4:
                return ne.label++, { value: ue[1], done: !1 };
              case 5:
                ne.label++, ae = ue[1], ue = [0];
                continue;
              case 7:
                ue = ne.ops.pop(), ne.trys.pop();
                continue;
              default:
                if (oe = ne.trys, !(oe = oe.length > 0 && oe[oe.length - 1]) && (ue[0] === 6 || ue[0] === 2)) {
                  ne = 0;
                  continue;
                }
                if (ue[0] === 3 && (!oe || ue[1] > oe[0] && ue[1] < oe[3])) {
                  ne.label = ue[1];
                  break;
                }
                if (ue[0] === 6 && ne.label < oe[1]) {
                  ne.label = oe[1], oe = ue;
                  break;
                }
                if (oe && ne.label < oe[2]) {
                  ne.label = oe[2], ne.ops.push(ue);
                  break;
                }
                oe[2] && ne.ops.pop(), ne.trys.pop();
                continue;
            }
            ue = te.call(Z, ne);
          } catch (de) {
            ue = [6, de], ae = 0;
          } finally {
            ee = oe = 0;
          }
          if (ue[0] & 5) throw ue[1];
          return { value: ue[0] ? ue[1] : void 0, done: !0 };
        }
      }, H = LinkedList.__read || function(Z, te) {
        var ne = typeof Symbol == "function" && Z[Symbol.iterator];
        if (!ne) return Z;
        var ee = ne.call(Z), ae, oe = [], se;
        try {
          for (; (te === void 0 || te-- > 0) && !(ae = ee.next()).done; ) oe.push(ae.value);
        } catch (ce) {
          se = { error: ce };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (se) throw se.error;
          }
        }
        return oe;
      }, V = LinkedList.__spreadArray || function(Z, te, ne) {
        if (ne || arguments.length === 2) for (var ee = 0, ae = te.length, oe; ee < ae; ee++)
          (oe || !(ee in te)) && (oe || (oe = Array.prototype.slice.call(te, 0, ee)), oe[ee] = te[ee]);
        return Z.concat(oe || Array.prototype.slice.call(te));
      }, q = LinkedList.__values || function(Z) {
        var te = typeof Symbol == "function" && Symbol.iterator, ne = te && Z[te], ee = 0;
        if (ne) return ne.call(Z);
        if (Z && typeof Z.length == "number") return {
          next: function() {
            return Z && ee >= Z.length && (Z = void 0), { value: Z && Z[ee++], done: !Z };
          }
        };
        throw new TypeError(te ? "Object is not iterable." : "Symbol.iterator is not defined.");
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.LinkedList = O.ListItem = O.END = void 0, O.END = Symbol();
      var G = /* @__PURE__ */ function() {
        function Z(te) {
          te === void 0 && (te = null), this.next = null, this.prev = null, this.data = te;
        }
        return Z;
      }();
      O.ListItem = G;
      var J = function() {
        function Z() {
          for (var te = [], ne = 0; ne < arguments.length; ne++)
            te[ne] = arguments[ne];
          this.list = new G(O.END), this.list.next = this.list.prev = this.list, this.push.apply(this, V([], H(te), !1));
        }
        return Z.prototype.isBefore = function(te, ne) {
          return te < ne;
        }, Z.prototype.push = function() {
          for (var te, ne, ee = [], ae = 0; ae < arguments.length; ae++)
            ee[ae] = arguments[ae];
          try {
            for (var oe = q(ee), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value, le = new G(ce);
              le.next = this.list, le.prev = this.list.prev, this.list.prev = le, le.prev.next = le;
            }
          } catch (ue) {
            te = { error: ue };
          } finally {
            try {
              se && !se.done && (ne = oe.return) && ne.call(oe);
            } finally {
              if (te) throw te.error;
            }
          }
          return this;
        }, Z.prototype.pop = function() {
          var te = this.list.prev;
          return te.data === O.END ? null : (this.list.prev = te.prev, te.prev.next = this.list, te.next = te.prev = null, te.data);
        }, Z.prototype.unshift = function() {
          for (var te, ne, ee = [], ae = 0; ae < arguments.length; ae++)
            ee[ae] = arguments[ae];
          try {
            for (var oe = q(ee.slice(0).reverse()), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value, le = new G(ce);
              le.next = this.list.next, le.prev = this.list, this.list.next = le, le.next.prev = le;
            }
          } catch (ue) {
            te = { error: ue };
          } finally {
            try {
              se && !se.done && (ne = oe.return) && ne.call(oe);
            } finally {
              if (te) throw te.error;
            }
          }
          return this;
        }, Z.prototype.shift = function() {
          var te = this.list.next;
          return te.data === O.END ? null : (this.list.next = te.next, te.next.prev = this.list, te.next = te.prev = null, te.data);
        }, Z.prototype.remove = function() {
          for (var te, ne, ee = [], ae = 0; ae < arguments.length; ae++)
            ee[ae] = arguments[ae];
          var oe = /* @__PURE__ */ new Map();
          try {
            for (var se = q(ee), ce = se.next(); !ce.done; ce = se.next()) {
              var le = ce.value;
              oe.set(le, !0);
            }
          } catch (fe) {
            te = { error: fe };
          } finally {
            try {
              ce && !ce.done && (ne = se.return) && ne.call(se);
            } finally {
              if (te) throw te.error;
            }
          }
          for (var ue = this.list.next; ue.data !== O.END; ) {
            var de = ue.next;
            oe.has(ue.data) && (ue.prev.next = ue.next, ue.next.prev = ue.prev, ue.next = ue.prev = null), ue = de;
          }
        }, Z.prototype.clear = function() {
          return this.list.next.prev = this.list.prev.next = null, this.list.next = this.list.prev = this.list, this;
        }, Z.prototype[Symbol.iterator] = function() {
          var te;
          return _(this, function(ne) {
            switch (ne.label) {
              case 0:
                te = this.list.next, ne.label = 1;
              case 1:
                return te.data === O.END ? [3, 3] : [4, te.data];
              case 2:
                return ne.sent(), te = te.next, [3, 1];
              case 3:
                return [2];
            }
          });
        }, Z.prototype.reversed = function() {
          var te;
          return _(this, function(ne) {
            switch (ne.label) {
              case 0:
                te = this.list.prev, ne.label = 1;
              case 1:
                return te.data === O.END ? [3, 3] : [4, te.data];
              case 2:
                return ne.sent(), te = te.prev, [3, 1];
              case 3:
                return [2];
            }
          });
        }, Z.prototype.insert = function(te, ne) {
          ne === void 0 && (ne = null), ne === null && (ne = this.isBefore.bind(this));
          for (var ee = new G(te), ae = this.list.next; ae.data !== O.END && ne(ae.data, ee.data); )
            ae = ae.next;
          return ee.prev = ae.prev, ee.next = ae, ae.prev.next = ae.prev = ee, this;
        }, Z.prototype.sort = function(te) {
          var ne, ee;
          te === void 0 && (te = null), te === null && (te = this.isBefore.bind(this));
          var ae = [];
          try {
            for (var oe = q(this), se = oe.next(); !se.done; se = oe.next()) {
              var ce = se.value;
              ae.push(new Z(ce));
            }
          } catch (de) {
            ne = { error: de };
          } finally {
            try {
              se && !se.done && (ee = oe.return) && ee.call(oe);
            } finally {
              if (ne) throw ne.error;
            }
          }
          for (this.list.next = this.list.prev = this.list; ae.length > 1; ) {
            var le = ae.shift(), ue = ae.shift();
            le.merge(ue, te), ae.push(le);
          }
          return ae.length && (this.list = ae[0].list), this;
        }, Z.prototype.merge = function(te, ne) {
          var ee, ae, oe, se, ce;
          ne === void 0 && (ne = null), ne === null && (ne = this.isBefore.bind(this));
          for (var le = this.list.next, ue = te.list.next; le.data !== O.END && ue.data !== O.END; )
            ne(ue.data, le.data) ? (ee = H([le, ue], 2), ue.prev.next = ee[0], le.prev.next = ee[1], ae = H([le.prev, ue.prev], 2), ue.prev = ae[0], le.prev = ae[1], oe = H([te.list, this.list], 2), this.list.prev.next = oe[0], te.list.prev.next = oe[1], se = H([te.list.prev, this.list.prev], 2), this.list.prev = se[0], te.list.prev = se[1], ce = H([ue.next, le], 2), le = ce[0], ue = ce[1]) : le = le.next;
          return ue.data !== O.END && (this.list.prev.next = te.list.next, te.list.next.prev = this.list.prev, te.list.prev.next = this.list, this.list.prev = te.list.prev, te.list.next = te.list.prev = te.list), this;
        }, Z;
      }();
      O.LinkedList = J;
    }(LinkedList)), LinkedList;
  }
  var hasRequiredMathList;
  function requireMathList() {
    if (hasRequiredMathList) return MathList;
    hasRequiredMathList = 1;
    var O = MathList.__extends || /* @__PURE__ */ function() {
      var V = function(q, G) {
        return V = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(J, Z) {
          J.__proto__ = Z;
        } || function(J, Z) {
          for (var te in Z) Object.prototype.hasOwnProperty.call(Z, te) && (J[te] = Z[te]);
        }, V(q, G);
      };
      return function(q, G) {
        if (typeof G != "function" && G !== null)
          throw new TypeError("Class extends value " + String(G) + " is not a constructor or null");
        V(q, G);
        function J() {
          this.constructor = q;
        }
        q.prototype = G === null ? Object.create(G) : (J.prototype = G.prototype, new J());
      };
    }();
    Object.defineProperty(MathList, "__esModule", { value: !0 }), MathList.AbstractMathList = void 0;
    var _ = requireLinkedList(), H = function(V) {
      O(q, V);
      function q() {
        return V !== null && V.apply(this, arguments) || this;
      }
      return q.prototype.isBefore = function(G, J) {
        return G.start.i < J.start.i || G.start.i === J.start.i && G.start.n < J.start.n;
      }, q;
    }(_.LinkedList);
    return MathList.AbstractMathList = H, MathList;
  }
  var MmlFactory = {}, NodeFactory = {}, hasRequiredNodeFactory;
  function requireNodeFactory() {
    if (hasRequiredNodeFactory) return NodeFactory;
    hasRequiredNodeFactory = 1;
    var O = NodeFactory.__extends || /* @__PURE__ */ function() {
      var V = function(q, G) {
        return V = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(J, Z) {
          J.__proto__ = Z;
        } || function(J, Z) {
          for (var te in Z) Object.prototype.hasOwnProperty.call(Z, te) && (J[te] = Z[te]);
        }, V(q, G);
      };
      return function(q, G) {
        if (typeof G != "function" && G !== null)
          throw new TypeError("Class extends value " + String(G) + " is not a constructor or null");
        V(q, G);
        function J() {
          this.constructor = q;
        }
        q.prototype = G === null ? Object.create(G) : (J.prototype = G.prototype, new J());
      };
    }();
    Object.defineProperty(NodeFactory, "__esModule", { value: !0 }), NodeFactory.AbstractNodeFactory = void 0;
    var _ = requireFactory(), H = function(V) {
      O(q, V);
      function q() {
        return V !== null && V.apply(this, arguments) || this;
      }
      return q.prototype.create = function(G, J, Z) {
        return J === void 0 && (J = {}), Z === void 0 && (Z = []), this.node[G](J, Z);
      }, q;
    }(_.AbstractFactory);
    return NodeFactory.AbstractNodeFactory = H, NodeFactory;
  }
  var MML = {}, mstyle = {}, hasRequiredMstyle;
  function requireMstyle() {
    if (hasRequiredMstyle) return mstyle;
    hasRequiredMstyle = 1;
    var O = mstyle.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = mstyle.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mstyle, "__esModule", { value: !0 }), mstyle.MmlMstyle = void 0;
    var H = requireMmlNode(), V = requireAttributes(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return Object.defineProperty(J.prototype, "kind", {
        get: function() {
          return "mstyle";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "notParent", {
        get: function() {
          return this.childNodes[0] && this.childNodes[0].childNodes.length === 1;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.setChildInheritedAttributes = function(Z, te, ne, ee) {
        var ae = this.attributes.getExplicit("scriptlevel");
        ae != null && (ae = ae.toString(), ae.match(/^\s*[-+]/) ? ne += parseInt(ae) : ne = parseInt(ae), ee = !1);
        var oe = this.attributes.getExplicit("displaystyle");
        oe != null && (te = oe === !0, ee = !1);
        var se = this.attributes.getExplicit("data-cramped");
        se != null && (ee = se), Z = this.addInheritedAttributes(Z, this.attributes.getAllAttributes()), this.childNodes[0].setInheritedAttributes(Z, te, ne, ee);
      }, J.defaults = _(_({}, H.AbstractMmlLayoutNode.defaults), { scriptlevel: V.INHERIT, displaystyle: V.INHERIT, scriptsizemultiplier: 1 / Math.sqrt(2), scriptminsize: "8px", mathbackground: V.INHERIT, mathcolor: V.INHERIT, dir: V.INHERIT, infixlinebreakstyle: "before" }), J;
    }(H.AbstractMmlLayoutNode);
    return mstyle.MmlMstyle = q, mstyle;
  }
  var maligngroup = {}, hasRequiredMaligngroup;
  function requireMaligngroup() {
    if (hasRequiredMaligngroup) return maligngroup;
    hasRequiredMaligngroup = 1;
    var O = maligngroup.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = maligngroup.__assign || function() {
      return _ = Object.assign || function(G) {
        for (var J, Z = 1, te = arguments.length; Z < te; Z++) {
          J = arguments[Z];
          for (var ne in J) Object.prototype.hasOwnProperty.call(J, ne) && (G[ne] = J[ne]);
        }
        return G;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(maligngroup, "__esModule", { value: !0 }), maligngroup.MmlMaligngroup = void 0;
    var H = requireMmlNode(), V = requireAttributes(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return Object.defineProperty(J.prototype, "kind", {
        get: function() {
          return "maligngroup";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "isSpacelike", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.setChildInheritedAttributes = function(Z, te, ne, ee) {
        Z = this.addInheritedAttributes(Z, this.attributes.getAllAttributes()), G.prototype.setChildInheritedAttributes.call(this, Z, te, ne, ee);
      }, J.defaults = _(_({}, H.AbstractMmlLayoutNode.defaults), { groupalign: V.INHERIT }), J;
    }(H.AbstractMmlLayoutNode);
    return maligngroup.MmlMaligngroup = q, maligngroup;
  }
  var malignmark = {}, hasRequiredMalignmark;
  function requireMalignmark() {
    if (hasRequiredMalignmark) return malignmark;
    hasRequiredMalignmark = 1;
    var O = malignmark.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = malignmark.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(malignmark, "__esModule", { value: !0 }), malignmark.MmlMalignmark = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "malignmark";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return 0;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "isSpacelike", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), G.defaults = _(_({}, H.AbstractMmlNode.defaults), { edge: "left" }), G;
    }(H.AbstractMmlNode);
    return malignmark.MmlMalignmark = V, malignmark;
  }
  var mathchoice = {}, hasRequiredMathchoice;
  function requireMathchoice() {
    if (hasRequiredMathchoice) return mathchoice;
    hasRequiredMathchoice = 1;
    var O = mathchoice.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }(), _ = mathchoice.__assign || function() {
      return _ = Object.assign || function(q) {
        for (var G, J = 1, Z = arguments.length; J < Z; J++) {
          G = arguments[J];
          for (var te in G) Object.prototype.hasOwnProperty.call(G, te) && (q[te] = G[te]);
        }
        return q;
      }, _.apply(this, arguments);
    };
    Object.defineProperty(mathchoice, "__esModule", { value: !0 }), mathchoice.MathChoice = void 0;
    var H = requireMmlNode(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return Object.defineProperty(G.prototype, "kind", {
        get: function() {
          return "MathChoice";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "arity", {
        get: function() {
          return 4;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(G.prototype, "notParent", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), G.prototype.setInheritedAttributes = function(J, Z, te, ne) {
        var ee = Z ? 0 : Math.max(0, Math.min(te, 2)) + 1, ae = this.childNodes[ee] || this.factory.create("mrow");
        this.parent.replaceChild(ae, this), ae.setInheritedAttributes(J, Z, te, ne);
      }, G.defaults = _({}, H.AbstractMmlBaseNode.defaults), G;
    }(H.AbstractMmlBaseNode);
    return mathchoice.MathChoice = V, mathchoice;
  }
  var hasRequiredMML;
  function requireMML() {
    if (hasRequiredMML) return MML;
    hasRequiredMML = 1;
    var O;
    Object.defineProperty(MML, "__esModule", { value: !0 }), MML.MML = void 0;
    var _ = requireMmlNode(), H = requireMath$1(), V = requireMi$1(), q = requireMn$1(), G = requireMo$2(), J = requireMtext$1(), Z = requireMspace$1(), te = requireMs$1(), ne = requireMrow$1(), ee = requireMfrac$1(), ae = requireMsqrt$1(), oe = requireMroot$1(), se = requireMstyle(), ce = requireMerror$1(), le = requireMpadded$1(), ue = requireMphantom$1(), de = requireMfenced$1(), fe = requireMenclose$1(), he = requireMaction$1(), me = requireMsubsup$1(), pe = requireMunderover$1(), ye = requireMmultiscripts$1(), ge = requireMtable$1(), ve = requireMtr$1(), be = requireMtd$1(), Ce = requireMaligngroup(), Se = requireMalignmark(), we = requireMglyph$1(), Ee = requireSemantics$1(), xe = requireTeXAtom$1(), Le = requireMathchoice();
    return MML.MML = (O = {}, O[H.MmlMath.prototype.kind] = H.MmlMath, O[V.MmlMi.prototype.kind] = V.MmlMi, O[q.MmlMn.prototype.kind] = q.MmlMn, O[G.MmlMo.prototype.kind] = G.MmlMo, O[J.MmlMtext.prototype.kind] = J.MmlMtext, O[Z.MmlMspace.prototype.kind] = Z.MmlMspace, O[te.MmlMs.prototype.kind] = te.MmlMs, O[ne.MmlMrow.prototype.kind] = ne.MmlMrow, O[ne.MmlInferredMrow.prototype.kind] = ne.MmlInferredMrow, O[ee.MmlMfrac.prototype.kind] = ee.MmlMfrac, O[ae.MmlMsqrt.prototype.kind] = ae.MmlMsqrt, O[oe.MmlMroot.prototype.kind] = oe.MmlMroot, O[se.MmlMstyle.prototype.kind] = se.MmlMstyle, O[ce.MmlMerror.prototype.kind] = ce.MmlMerror, O[le.MmlMpadded.prototype.kind] = le.MmlMpadded, O[ue.MmlMphantom.prototype.kind] = ue.MmlMphantom, O[de.MmlMfenced.prototype.kind] = de.MmlMfenced, O[fe.MmlMenclose.prototype.kind] = fe.MmlMenclose, O[he.MmlMaction.prototype.kind] = he.MmlMaction, O[me.MmlMsub.prototype.kind] = me.MmlMsub, O[me.MmlMsup.prototype.kind] = me.MmlMsup, O[me.MmlMsubsup.prototype.kind] = me.MmlMsubsup, O[pe.MmlMunder.prototype.kind] = pe.MmlMunder, O[pe.MmlMover.prototype.kind] = pe.MmlMover, O[pe.MmlMunderover.prototype.kind] = pe.MmlMunderover, O[ye.MmlMmultiscripts.prototype.kind] = ye.MmlMmultiscripts, O[ye.MmlMprescripts.prototype.kind] = ye.MmlMprescripts, O[ye.MmlNone.prototype.kind] = ye.MmlNone, O[ge.MmlMtable.prototype.kind] = ge.MmlMtable, O[ve.MmlMlabeledtr.prototype.kind] = ve.MmlMlabeledtr, O[ve.MmlMtr.prototype.kind] = ve.MmlMtr, O[be.MmlMtd.prototype.kind] = be.MmlMtd, O[Ce.MmlMaligngroup.prototype.kind] = Ce.MmlMaligngroup, O[Se.MmlMalignmark.prototype.kind] = Se.MmlMalignmark, O[we.MmlMglyph.prototype.kind] = we.MmlMglyph, O[Ee.MmlSemantics.prototype.kind] = Ee.MmlSemantics, O[Ee.MmlAnnotation.prototype.kind] = Ee.MmlAnnotation, O[Ee.MmlAnnotationXML.prototype.kind] = Ee.MmlAnnotationXML, O[xe.TeXAtom.prototype.kind] = xe.TeXAtom, O[Le.MathChoice.prototype.kind] = Le.MathChoice, O[_.TextNode.prototype.kind] = _.TextNode, O[_.XMLNode.prototype.kind] = _.XMLNode, O), MML;
  }
  var hasRequiredMmlFactory;
  function requireMmlFactory() {
    if (hasRequiredMmlFactory) return MmlFactory;
    hasRequiredMmlFactory = 1;
    var O = MmlFactory.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }();
    Object.defineProperty(MmlFactory, "__esModule", { value: !0 }), MmlFactory.MmlFactory = void 0;
    var _ = requireNodeFactory(), H = requireMML(), V = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return Object.defineProperty(G.prototype, "MML", {
        get: function() {
          return this.node;
        },
        enumerable: !1,
        configurable: !0
      }), G.defaultNodes = H.MML, G;
    }(_.AbstractNodeFactory);
    return MmlFactory.MmlFactory = V, MmlFactory;
  }
  var BitField = {}, hasRequiredBitField;
  function requireBitField() {
    if (hasRequiredBitField) return BitField;
    hasRequiredBitField = 1;
    var O = BitField.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = BitField.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = BitField.__read || function(J, Z) {
      var te = typeof Symbol == "function" && J[Symbol.iterator];
      if (!te) return J;
      var ne = te.call(J), ee, ae = [], oe;
      try {
        for (; (Z === void 0 || Z-- > 0) && !(ee = ne.next()).done; ) ae.push(ee.value);
      } catch (se) {
        oe = { error: se };
      } finally {
        try {
          ee && !ee.done && (te = ne.return) && te.call(ne);
        } finally {
          if (oe) throw oe.error;
        }
      }
      return ae;
    }, V = BitField.__spreadArray || function(J, Z, te) {
      if (te || arguments.length === 2) for (var ne = 0, ee = Z.length, ae; ne < ee; ne++)
        (ae || !(ne in Z)) && (ae || (ae = Array.prototype.slice.call(Z, 0, ne)), ae[ne] = Z[ne]);
      return J.concat(ae || Array.prototype.slice.call(Z));
    };
    Object.defineProperty(BitField, "__esModule", { value: !0 }), BitField.BitFieldClass = BitField.BitField = void 0;
    var q = function() {
      function J() {
        this.bits = 0;
      }
      return J.allocate = function() {
        for (var Z, te, ne = [], ee = 0; ee < arguments.length; ee++)
          ne[ee] = arguments[ee];
        try {
          for (var ae = _(ne), oe = ae.next(); !oe.done; oe = ae.next()) {
            var se = oe.value;
            if (this.has(se))
              throw new Error("Bit already allocated for " + se);
            if (this.next === J.MAXBIT)
              throw new Error("Maximum number of bits already allocated");
            this.names.set(se, this.next), this.next <<= 1;
          }
        } catch (ce) {
          Z = { error: ce };
        } finally {
          try {
            oe && !oe.done && (te = ae.return) && te.call(ae);
          } finally {
            if (Z) throw Z.error;
          }
        }
      }, J.has = function(Z) {
        return this.names.has(Z);
      }, J.prototype.set = function(Z) {
        this.bits |= this.getBit(Z);
      }, J.prototype.clear = function(Z) {
        this.bits &= ~this.getBit(Z);
      }, J.prototype.isSet = function(Z) {
        return !!(this.bits & this.getBit(Z));
      }, J.prototype.reset = function() {
        this.bits = 0;
      }, J.prototype.getBit = function(Z) {
        var te = this.constructor.names.get(Z);
        if (!te)
          throw new Error("Unknown bit-field name: " + Z);
        return te;
      }, J.MAXBIT = 1 << 31, J.next = 1, J.names = /* @__PURE__ */ new Map(), J;
    }();
    BitField.BitField = q;
    function G() {
      for (var J = [], Z = 0; Z < arguments.length; Z++)
        J[Z] = arguments[Z];
      var te = function(ne) {
        O(ee, ne);
        function ee() {
          return ne !== null && ne.apply(this, arguments) || this;
        }
        return ee;
      }(q);
      return te.allocate.apply(te, V([], H(J), !1)), te;
    }
    return BitField.BitFieldClass = G, BitField;
  }
  var hasRequiredMathDocument;
  function requireMathDocument() {
    return hasRequiredMathDocument || (hasRequiredMathDocument = 1, function(O) {
      var _ = MathDocument.__extends || /* @__PURE__ */ function() {
        var he = function(me, pe) {
          return he = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ye, ge) {
            ye.__proto__ = ge;
          } || function(ye, ge) {
            for (var ve in ge) Object.prototype.hasOwnProperty.call(ge, ve) && (ye[ve] = ge[ve]);
          }, he(me, pe);
        };
        return function(me, pe) {
          if (typeof pe != "function" && pe !== null)
            throw new TypeError("Class extends value " + String(pe) + " is not a constructor or null");
          he(me, pe);
          function ye() {
            this.constructor = me;
          }
          me.prototype = pe === null ? Object.create(pe) : (ye.prototype = pe.prototype, new ye());
        };
      }(), H = MathDocument.__values || function(he) {
        var me = typeof Symbol == "function" && Symbol.iterator, pe = me && he[me], ye = 0;
        if (pe) return pe.call(he);
        if (he && typeof he.length == "number") return {
          next: function() {
            return he && ye >= he.length && (he = void 0), { value: he && he[ye++], done: !he };
          }
        };
        throw new TypeError(me ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, V = MathDocument.__read || function(he, me) {
        var pe = typeof Symbol == "function" && he[Symbol.iterator];
        if (!pe) return he;
        var ye = pe.call(he), ge, ve = [], be;
        try {
          for (; (me === void 0 || me-- > 0) && !(ge = ye.next()).done; ) ve.push(ge.value);
        } catch (Ce) {
          be = { error: Ce };
        } finally {
          try {
            ge && !ge.done && (pe = ye.return) && pe.call(ye);
          } finally {
            if (be) throw be.error;
          }
        }
        return ve;
      }, q = MathDocument.__spreadArray || function(he, me, pe) {
        if (pe || arguments.length === 2) for (var ye = 0, ge = me.length, ve; ye < ge; ye++)
          (ve || !(ye in me)) && (ve || (ve = Array.prototype.slice.call(me, 0, ye)), ve[ye] = me[ye]);
        return he.concat(ve || Array.prototype.slice.call(me));
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.AbstractMathDocument = O.resetAllOptions = O.resetOptions = O.RenderList = void 0;
      var G = requireOptions(), J = requireInputJax(), Z = requireOutputJax$1(), te = requireMathList(), ne = requireMathItem(), ee = requireMmlFactory(), ae = requireBitField(), oe = requirePrioritizedList(), se = function(he) {
        _(me, he);
        function me() {
          return he !== null && he.apply(this, arguments) || this;
        }
        return me.create = function(pe) {
          var ye, ge, ve = new this();
          try {
            for (var be = H(Object.keys(pe)), Ce = be.next(); !Ce.done; Ce = be.next()) {
              var Se = Ce.value, we = V(this.action(Se, pe[Se]), 2), Ee = we[0], xe = we[1];
              xe && ve.add(Ee, xe);
            }
          } catch (Le) {
            ye = { error: Le };
          } finally {
            try {
              Ce && !Ce.done && (ge = be.return) && ge.call(be);
            } finally {
              if (ye) throw ye.error;
            }
          }
          return ve;
        }, me.action = function(pe, ye) {
          var ge, ve, be, Ce, Se, we, Ee = !0, xe = ye[0];
          if (ye.length === 1 || typeof ye[1] == "boolean")
            ye.length === 2 && (Ee = ye[1]), ge = V(this.methodActions(pe), 2), Se = ge[0], we = ge[1];
          else if (typeof ye[1] == "string")
            if (typeof ye[2] == "string") {
              ye.length === 4 && (Ee = ye[3]);
              var Le = V(ye.slice(1), 2), Qe = Le[0], ke = Le[1];
              ve = V(this.methodActions(Qe, ke), 2), Se = ve[0], we = ve[1];
            } else
              ye.length === 3 && (Ee = ye[2]), be = V(this.methodActions(ye[1]), 2), Se = be[0], we = be[1];
          else
            ye.length === 4 && (Ee = ye[3]), Ce = V(ye.slice(1), 2), Se = Ce[0], we = Ce[1];
          return [{ id: pe, renderDoc: Se, renderMath: we, convert: Ee }, xe];
        }, me.methodActions = function(pe, ye) {
          return ye === void 0 && (ye = pe), [
            function(ge) {
              return pe && ge[pe](), !1;
            },
            function(ge, ve) {
              return ye && ge[ye](ve), !1;
            }
          ];
        }, me.prototype.renderDoc = function(pe, ye) {
          var ge, ve;
          ye === void 0 && (ye = ne.STATE.UNPROCESSED);
          try {
            for (var be = H(this.items), Ce = be.next(); !Ce.done; Ce = be.next()) {
              var Se = Ce.value;
              if (Se.priority >= ye && Se.item.renderDoc(pe))
                return;
            }
          } catch (we) {
            ge = { error: we };
          } finally {
            try {
              Ce && !Ce.done && (ve = be.return) && ve.call(be);
            } finally {
              if (ge) throw ge.error;
            }
          }
        }, me.prototype.renderMath = function(pe, ye, ge) {
          var ve, be;
          ge === void 0 && (ge = ne.STATE.UNPROCESSED);
          try {
            for (var Ce = H(this.items), Se = Ce.next(); !Se.done; Se = Ce.next()) {
              var we = Se.value;
              if (we.priority >= ge && we.item.renderMath(pe, ye))
                return;
            }
          } catch (Ee) {
            ve = { error: Ee };
          } finally {
            try {
              Se && !Se.done && (be = Ce.return) && be.call(Ce);
            } finally {
              if (ve) throw ve.error;
            }
          }
        }, me.prototype.renderConvert = function(pe, ye, ge) {
          var ve, be;
          ge === void 0 && (ge = ne.STATE.LAST);
          try {
            for (var Ce = H(this.items), Se = Ce.next(); !Se.done; Se = Ce.next()) {
              var we = Se.value;
              if (we.priority > ge || we.item.convert && we.item.renderMath(pe, ye))
                return;
            }
          } catch (Ee) {
            ve = { error: Ee };
          } finally {
            try {
              Se && !Se.done && (be = Ce.return) && be.call(Ce);
            } finally {
              if (ve) throw ve.error;
            }
          }
        }, me.prototype.findID = function(pe) {
          var ye, ge;
          try {
            for (var ve = H(this.items), be = ve.next(); !be.done; be = ve.next()) {
              var Ce = be.value;
              if (Ce.item.id === pe)
                return Ce.item;
            }
          } catch (Se) {
            ye = { error: Se };
          } finally {
            try {
              be && !be.done && (ge = ve.return) && ge.call(ve);
            } finally {
              if (ye) throw ye.error;
            }
          }
          return null;
        }, me;
      }(oe.PrioritizedList);
      O.RenderList = se, O.resetOptions = {
        all: !1,
        processed: !1,
        inputJax: null,
        outputJax: null
      }, O.resetAllOptions = {
        all: !0,
        processed: !0,
        inputJax: [],
        outputJax: []
      };
      var ce = function(he) {
        _(me, he);
        function me() {
          return he !== null && he.apply(this, arguments) || this;
        }
        return me.prototype.compile = function(pe) {
          return null;
        }, me;
      }(J.AbstractInputJax), le = function(he) {
        _(me, he);
        function me() {
          return he !== null && he.apply(this, arguments) || this;
        }
        return me.prototype.typeset = function(pe, ye) {
          return null;
        }, me.prototype.escaped = function(pe, ye) {
          return null;
        }, me;
      }(Z.AbstractOutputJax), ue = function(he) {
        _(me, he);
        function me() {
          return he !== null && he.apply(this, arguments) || this;
        }
        return me;
      }(te.AbstractMathList), de = function(he) {
        _(me, he);
        function me() {
          return he !== null && he.apply(this, arguments) || this;
        }
        return me;
      }(ne.AbstractMathItem), fe = function() {
        function he(me, pe, ye) {
          var ge = this, ve = this.constructor;
          this.document = me, this.options = (0, G.userOptions)((0, G.defaultOptions)({}, ve.OPTIONS), ye), this.math = new (this.options.MathList || ue)(), this.renderActions = se.create(this.options.renderActions), this.processed = new he.ProcessBits(), this.outputJax = this.options.OutputJax || new le();
          var be = this.options.InputJax || [new ce()];
          Array.isArray(be) || (be = [be]), this.inputJax = be, this.adaptor = pe, this.outputJax.setAdaptor(pe), this.inputJax.map(function(Ce) {
            return Ce.setAdaptor(pe);
          }), this.mmlFactory = this.options.MmlFactory || new ee.MmlFactory(), this.inputJax.map(function(Ce) {
            return Ce.setMmlFactory(ge.mmlFactory);
          }), this.outputJax.initialize(), this.inputJax.map(function(Ce) {
            return Ce.initialize();
          });
        }
        return Object.defineProperty(he.prototype, "kind", {
          get: function() {
            return this.constructor.KIND;
          },
          enumerable: !1,
          configurable: !0
        }), he.prototype.addRenderAction = function(me) {
          for (var pe = [], ye = 1; ye < arguments.length; ye++)
            pe[ye - 1] = arguments[ye];
          var ge = V(se.action(me, pe), 2), ve = ge[0], be = ge[1];
          this.renderActions.add(ve, be);
        }, he.prototype.removeRenderAction = function(me) {
          var pe = this.renderActions.findID(me);
          pe && this.renderActions.remove(pe);
        }, he.prototype.render = function() {
          return this.renderActions.renderDoc(this), this;
        }, he.prototype.rerender = function(me) {
          return me === void 0 && (me = ne.STATE.RERENDER), this.state(me - 1), this.render(), this;
        }, he.prototype.convert = function(me, pe) {
          pe === void 0 && (pe = {});
          var ye = (0, G.userOptions)({
            format: this.inputJax[0].name,
            display: !0,
            end: ne.STATE.LAST,
            em: 16,
            ex: 8,
            containerWidth: null,
            lineWidth: 1e6,
            scale: 1,
            family: ""
          }, pe), ge = ye.format, ve = ye.display, be = ye.end, Ce = ye.ex, Se = ye.em, we = ye.containerWidth, Ee = ye.lineWidth, xe = ye.scale, Le = ye.family;
          we === null && (we = 80 * Ce);
          var Qe = this.inputJax.reduce(function(Me, Re) {
            return Re.name === ge ? Re : Me;
          }, null), ke = new this.options.MathItem(me, Qe, ve);
          return ke.start.node = this.adaptor.body(this.document), ke.setMetrics(Se, Ce, we, Ee, xe), this.outputJax.options.mtextInheritFont && (ke.outputData.mtextFamily = Le), this.outputJax.options.merrorInheritFont && (ke.outputData.merrorFamily = Le), ke.convert(this, be), ke.typesetRoot || ke.root;
        }, he.prototype.findMath = function(me) {
          return this.processed.set("findMath"), this;
        }, he.prototype.compile = function() {
          var me, pe, ye, ge;
          if (!this.processed.isSet("compile")) {
            var ve = [];
            try {
              for (var be = H(this.math), Ce = be.next(); !Ce.done; Ce = be.next()) {
                var Se = Ce.value;
                this.compileMath(Se), Se.inputData.recompile !== void 0 && ve.push(Se);
              }
            } catch (Le) {
              me = { error: Le };
            } finally {
              try {
                Ce && !Ce.done && (pe = be.return) && pe.call(be);
              } finally {
                if (me) throw me.error;
              }
            }
            try {
              for (var we = H(ve), Ee = we.next(); !Ee.done; Ee = we.next()) {
                var Se = Ee.value, xe = Se.inputData.recompile;
                Se.state(xe.state), Se.inputData.recompile = xe, this.compileMath(Se);
              }
            } catch (Le) {
              ye = { error: Le };
            } finally {
              try {
                Ee && !Ee.done && (ge = we.return) && ge.call(we);
              } finally {
                if (ye) throw ye.error;
              }
            }
            this.processed.set("compile");
          }
          return this;
        }, he.prototype.compileMath = function(me) {
          try {
            me.compile(this);
          } catch (pe) {
            if (pe.retry || pe.restart)
              throw pe;
            this.options.compileError(this, me, pe), me.inputData.error = pe;
          }
        }, he.prototype.compileError = function(me, pe) {
          me.root = this.mmlFactory.create("math", null, [
            this.mmlFactory.create("merror", { "data-mjx-error": pe.message, title: pe.message }, [
              this.mmlFactory.create("mtext", null, [
                this.mmlFactory.create("text").setText("Math input error")
              ])
            ])
          ]), me.display && me.root.attributes.set("display", "block"), me.inputData.error = pe.message;
        }, he.prototype.typeset = function() {
          var me, pe;
          if (!this.processed.isSet("typeset")) {
            try {
              for (var ye = H(this.math), ge = ye.next(); !ge.done; ge = ye.next()) {
                var ve = ge.value;
                try {
                  ve.typeset(this);
                } catch (be) {
                  if (be.retry || be.restart)
                    throw be;
                  this.options.typesetError(this, ve, be), ve.outputData.error = be;
                }
              }
            } catch (be) {
              me = { error: be };
            } finally {
              try {
                ge && !ge.done && (pe = ye.return) && pe.call(ye);
              } finally {
                if (me) throw me.error;
              }
            }
            this.processed.set("typeset");
          }
          return this;
        }, he.prototype.typesetError = function(me, pe) {
          me.typesetRoot = this.adaptor.node("mjx-container", {
            class: "MathJax mjx-output-error",
            jax: this.outputJax.name
          }, [
            this.adaptor.node("span", {
              "data-mjx-error": pe.message,
              title: pe.message,
              style: {
                color: "red",
                "background-color": "yellow",
                "line-height": "normal"
              }
            }, [
              this.adaptor.text("Math output error")
            ])
          ]), me.display && this.adaptor.setAttributes(me.typesetRoot, {
            style: {
              display: "block",
              margin: "1em 0",
              "text-align": "center"
            }
          }), me.outputData.error = pe.message;
        }, he.prototype.getMetrics = function() {
          return this.processed.isSet("getMetrics") || (this.outputJax.getMetrics(this), this.processed.set("getMetrics")), this;
        }, he.prototype.updateDocument = function() {
          var me, pe;
          if (!this.processed.isSet("updateDocument")) {
            try {
              for (var ye = H(this.math.reversed()), ge = ye.next(); !ge.done; ge = ye.next()) {
                var ve = ge.value;
                ve.updateDocument(this);
              }
            } catch (be) {
              me = { error: be };
            } finally {
              try {
                ge && !ge.done && (pe = ye.return) && pe.call(ye);
              } finally {
                if (me) throw me.error;
              }
            }
            this.processed.set("updateDocument");
          }
          return this;
        }, he.prototype.removeFromDocument = function(me) {
          return this;
        }, he.prototype.state = function(me, pe) {
          var ye, ge;
          pe === void 0 && (pe = !1);
          try {
            for (var ve = H(this.math), be = ve.next(); !be.done; be = ve.next()) {
              var Ce = be.value;
              Ce.state(me, pe);
            }
          } catch (Se) {
            ye = { error: Se };
          } finally {
            try {
              be && !be.done && (ge = ve.return) && ge.call(ve);
            } finally {
              if (ye) throw ye.error;
            }
          }
          return me < ne.STATE.INSERTED && this.processed.clear("updateDocument"), me < ne.STATE.TYPESET && (this.processed.clear("typeset"), this.processed.clear("getMetrics")), me < ne.STATE.COMPILED && this.processed.clear("compile"), this;
        }, he.prototype.reset = function(me) {
          var pe;
          return me === void 0 && (me = { processed: !0 }), me = (0, G.userOptions)(Object.assign({}, O.resetOptions), me), me.all && Object.assign(me, O.resetAllOptions), me.processed && this.processed.reset(), me.inputJax && this.inputJax.forEach(function(ye) {
            return ye.reset.apply(ye, q([], V(me.inputJax), !1));
          }), me.outputJax && (pe = this.outputJax).reset.apply(pe, q([], V(me.outputJax), !1)), this;
        }, he.prototype.clear = function() {
          return this.reset(), this.math.clear(), this;
        }, he.prototype.concat = function(me) {
          return this.math.merge(me), this;
        }, he.prototype.clearMathItemsWithin = function(me) {
          var pe, ye = this.getMathItemsWithin(me);
          return (pe = this.math).remove.apply(pe, q([], V(ye), !1)), ye;
        }, he.prototype.getMathItemsWithin = function(me) {
          var pe, ye, ge, ve;
          Array.isArray(me) || (me = [me]);
          var be = this.adaptor, Ce = [], Se = be.getElements(me, this.document);
          try {
            e: for (var we = H(this.math), Ee = we.next(); !Ee.done; Ee = we.next()) {
              var xe = Ee.value;
              try {
                for (var Le = (ge = void 0, H(Se)), Qe = Le.next(); !Qe.done; Qe = Le.next()) {
                  var ke = Qe.value;
                  if (xe.start.node && be.contains(ke, xe.start.node)) {
                    Ce.push(xe);
                    continue e;
                  }
                }
              } catch (Me) {
                ge = { error: Me };
              } finally {
                try {
                  Qe && !Qe.done && (ve = Le.return) && ve.call(Le);
                } finally {
                  if (ge) throw ge.error;
                }
              }
            }
          } catch (Me) {
            pe = { error: Me };
          } finally {
            try {
              Ee && !Ee.done && (ye = we.return) && ye.call(we);
            } finally {
              if (pe) throw pe.error;
            }
          }
          return Ce;
        }, he.KIND = "MathDocument", he.OPTIONS = {
          OutputJax: null,
          InputJax: null,
          MmlFactory: null,
          MathList: ue,
          MathItem: de,
          compileError: function(me, pe, ye) {
            me.compileError(pe, ye);
          },
          typesetError: function(me, pe, ye) {
            me.typesetError(pe, ye);
          },
          renderActions: (0, G.expandable)({
            find: [ne.STATE.FINDMATH, "findMath", "", !1],
            compile: [ne.STATE.COMPILED],
            metrics: [ne.STATE.METRICS, "getMetrics", "", !1],
            typeset: [ne.STATE.TYPESET],
            update: [ne.STATE.INSERTED, "updateDocument", !1]
          })
        }, he.ProcessBits = (0, ae.BitFieldClass)("findMath", "compile", "getMetrics", "typeset", "updateDocument"), he;
      }();
      O.AbstractMathDocument = fe;
    }(MathDocument)), MathDocument;
  }
  var hasRequiredHandler;
  function requireHandler() {
    if (hasRequiredHandler) return Handler$2;
    hasRequiredHandler = 1;
    var O = Handler$2.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }();
    Object.defineProperty(Handler$2, "__esModule", { value: !0 }), Handler$2.AbstractHandler = void 0;
    var _ = requireMathDocument(), H = function(q) {
      O(G, q);
      function G() {
        return q !== null && q.apply(this, arguments) || this;
      }
      return G;
    }(_.AbstractMathDocument), V = function() {
      function q(G, J) {
        J === void 0 && (J = 5), this.documentClass = H, this.adaptor = G, this.priority = J;
      }
      return Object.defineProperty(q.prototype, "name", {
        get: function() {
          return this.constructor.NAME;
        },
        enumerable: !1,
        configurable: !0
      }), q.prototype.handlesDocument = function(G) {
        return !1;
      }, q.prototype.create = function(G, J) {
        return new this.documentClass(G, this.adaptor, J);
      }, q.NAME = "generic", q;
    }();
    return Handler$2.AbstractHandler = V, Handler$2;
  }
  var HTMLDocument = {}, HTMLMathItem = {}, hasRequiredHTMLMathItem;
  function requireHTMLMathItem() {
    if (hasRequiredHTMLMathItem) return HTMLMathItem;
    hasRequiredHTMLMathItem = 1;
    var O = HTMLMathItem.__extends || /* @__PURE__ */ function() {
      var V = function(q, G) {
        return V = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(J, Z) {
          J.__proto__ = Z;
        } || function(J, Z) {
          for (var te in Z) Object.prototype.hasOwnProperty.call(Z, te) && (J[te] = Z[te]);
        }, V(q, G);
      };
      return function(q, G) {
        if (typeof G != "function" && G !== null)
          throw new TypeError("Class extends value " + String(G) + " is not a constructor or null");
        V(q, G);
        function J() {
          this.constructor = q;
        }
        q.prototype = G === null ? Object.create(G) : (J.prototype = G.prototype, new J());
      };
    }();
    Object.defineProperty(HTMLMathItem, "__esModule", { value: !0 }), HTMLMathItem.HTMLMathItem = void 0;
    var _ = requireMathItem(), H = function(V) {
      O(q, V);
      function q(G, J, Z, te, ne) {
        return Z === void 0 && (Z = !0), te === void 0 && (te = { node: null, n: 0, delim: "" }), ne === void 0 && (ne = { node: null, n: 0, delim: "" }), V.call(this, G, J, Z, te, ne) || this;
      }
      return Object.defineProperty(q.prototype, "adaptor", {
        get: function() {
          return this.inputJax.adaptor;
        },
        enumerable: !1,
        configurable: !0
      }), q.prototype.updateDocument = function(G) {
        if (this.state() < _.STATE.INSERTED) {
          if (this.inputJax.processStrings) {
            var J = this.start.node;
            if (J === this.end.node)
              this.end.n && this.end.n < this.adaptor.value(this.end.node).length && this.adaptor.split(this.end.node, this.end.n), this.start.n && (J = this.adaptor.split(this.start.node, this.start.n)), this.adaptor.replace(this.typesetRoot, J);
            else {
              for (this.start.n && (J = this.adaptor.split(J, this.start.n)); J !== this.end.node; ) {
                var Z = this.adaptor.next(J);
                this.adaptor.remove(J), J = Z;
              }
              this.adaptor.insert(this.typesetRoot, J), this.end.n < this.adaptor.value(J).length && this.adaptor.split(J, this.end.n), this.adaptor.remove(J);
            }
          } else
            this.adaptor.replace(this.typesetRoot, this.start.node);
          this.start.node = this.end.node = this.typesetRoot, this.start.n = this.end.n = 0, this.state(_.STATE.INSERTED);
        }
      }, q.prototype.updateStyleSheet = function(G) {
        G.addStyleSheet();
      }, q.prototype.removeFromDocument = function(G) {
        if (G === void 0 && (G = !1), this.state() >= _.STATE.TYPESET) {
          var J = this.adaptor, Z = this.start.node, te = J.text("");
          if (G) {
            var ne = this.start.delim + this.math + this.end.delim;
            if (this.inputJax.processStrings)
              te = J.text(ne);
            else {
              var ee = J.parse(ne, "text/html");
              te = J.firstChild(J.body(ee));
            }
          }
          J.parent(Z) && J.replace(te, Z), this.start.node = this.end.node = te, this.start.n = this.end.n = 0;
        }
      }, q;
    }(_.AbstractMathItem);
    return HTMLMathItem.HTMLMathItem = H, HTMLMathItem;
  }
  var HTMLMathList = {}, hasRequiredHTMLMathList;
  function requireHTMLMathList() {
    if (hasRequiredHTMLMathList) return HTMLMathList;
    hasRequiredHTMLMathList = 1;
    var O = HTMLMathList.__extends || /* @__PURE__ */ function() {
      var V = function(q, G) {
        return V = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(J, Z) {
          J.__proto__ = Z;
        } || function(J, Z) {
          for (var te in Z) Object.prototype.hasOwnProperty.call(Z, te) && (J[te] = Z[te]);
        }, V(q, G);
      };
      return function(q, G) {
        if (typeof G != "function" && G !== null)
          throw new TypeError("Class extends value " + String(G) + " is not a constructor or null");
        V(q, G);
        function J() {
          this.constructor = q;
        }
        q.prototype = G === null ? Object.create(G) : (J.prototype = G.prototype, new J());
      };
    }();
    Object.defineProperty(HTMLMathList, "__esModule", { value: !0 }), HTMLMathList.HTMLMathList = void 0;
    var _ = requireMathList(), H = function(V) {
      O(q, V);
      function q() {
        return V !== null && V.apply(this, arguments) || this;
      }
      return q;
    }(_.AbstractMathList);
    return HTMLMathList.HTMLMathList = H, HTMLMathList;
  }
  var HTMLDomStrings = {}, hasRequiredHTMLDomStrings;
  function requireHTMLDomStrings() {
    if (hasRequiredHTMLDomStrings) return HTMLDomStrings;
    hasRequiredHTMLDomStrings = 1;
    var O = HTMLDomStrings.__read || function(V, q) {
      var G = typeof Symbol == "function" && V[Symbol.iterator];
      if (!G) return V;
      var J = G.call(V), Z, te = [], ne;
      try {
        for (; (q === void 0 || q-- > 0) && !(Z = J.next()).done; ) te.push(Z.value);
      } catch (ee) {
        ne = { error: ee };
      } finally {
        try {
          Z && !Z.done && (G = J.return) && G.call(J);
        } finally {
          if (ne) throw ne.error;
        }
      }
      return te;
    };
    Object.defineProperty(HTMLDomStrings, "__esModule", { value: !0 }), HTMLDomStrings.HTMLDomStrings = void 0;
    var _ = requireOptions(), H = function() {
      function V(q) {
        q === void 0 && (q = null);
        var G = this.constructor;
        this.options = (0, _.userOptions)((0, _.defaultOptions)({}, G.OPTIONS), q), this.init(), this.getPatterns();
      }
      return V.prototype.init = function() {
        this.strings = [], this.string = "", this.snodes = [], this.nodes = [], this.stack = [];
      }, V.prototype.getPatterns = function() {
        var q = (0, _.makeArray)(this.options.skipHtmlTags), G = (0, _.makeArray)(this.options.ignoreHtmlClass), J = (0, _.makeArray)(this.options.processHtmlClass);
        this.skipHtmlTags = new RegExp("^(?:" + q.join("|") + ")$", "i"), this.ignoreHtmlClass = new RegExp("(?:^| )(?:" + G.join("|") + ")(?: |$)"), this.processHtmlClass = new RegExp("(?:^| )(?:" + J + ")(?: |$)");
      }, V.prototype.pushString = function() {
        this.string.match(/\S/) && (this.strings.push(this.string), this.nodes.push(this.snodes)), this.string = "", this.snodes = [];
      }, V.prototype.extendString = function(q, G) {
        this.snodes.push([q, G.length]), this.string += G;
      }, V.prototype.handleText = function(q, G) {
        return G || this.extendString(q, this.adaptor.value(q)), this.adaptor.next(q);
      }, V.prototype.handleTag = function(q, G) {
        if (!G) {
          var J = this.options.includeHtmlTags[this.adaptor.kind(q)];
          this.extendString(q, J);
        }
        return this.adaptor.next(q);
      }, V.prototype.handleContainer = function(q, G) {
        this.pushString();
        var J = this.adaptor.getAttribute(q, "class") || "", Z = this.adaptor.kind(q) || "", te = this.processHtmlClass.exec(J), ne = q;
        return this.adaptor.firstChild(q) && !this.adaptor.getAttribute(q, "data-MJX") && (te || !this.skipHtmlTags.exec(Z)) ? (this.adaptor.next(q) && this.stack.push([this.adaptor.next(q), G]), ne = this.adaptor.firstChild(q), G = (G || this.ignoreHtmlClass.exec(J)) && !te) : ne = this.adaptor.next(q), [ne, G];
      }, V.prototype.handleOther = function(q, G) {
        return this.pushString(), this.adaptor.next(q);
      }, V.prototype.find = function(q) {
        var G, J;
        this.init();
        for (var Z = this.adaptor.next(q), te = !1, ne = this.options.includeHtmlTags; q && q !== Z; ) {
          var ee = this.adaptor.kind(q);
          ee === "#text" ? q = this.handleText(q, te) : ne.hasOwnProperty(ee) ? q = this.handleTag(q, te) : ee ? (G = O(this.handleContainer(q, te), 2), q = G[0], te = G[1]) : q = this.handleOther(q, te), !q && this.stack.length && (this.pushString(), J = O(this.stack.pop(), 2), q = J[0], te = J[1]);
        }
        this.pushString();
        var ae = [this.strings, this.nodes];
        return this.init(), ae;
      }, V.OPTIONS = {
        skipHtmlTags: ["script", "noscript", "style", "textarea", "pre", "code", "annotation", "annotation-xml"],
        includeHtmlTags: { br: `
`, wbr: "", "#comment": "" },
        ignoreHtmlClass: "mathjax_ignore",
        processHtmlClass: "mathjax_process"
      }, V;
    }();
    return HTMLDomStrings.HTMLDomStrings = H, HTMLDomStrings;
  }
  var hasRequiredHTMLDocument;
  function requireHTMLDocument() {
    if (hasRequiredHTMLDocument) return HTMLDocument;
    hasRequiredHTMLDocument = 1;
    var O = HTMLDocument.__extends || /* @__PURE__ */ function() {
      var ae = function(oe, se) {
        return ae = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ce, le) {
          ce.__proto__ = le;
        } || function(ce, le) {
          for (var ue in le) Object.prototype.hasOwnProperty.call(le, ue) && (ce[ue] = le[ue]);
        }, ae(oe, se);
      };
      return function(oe, se) {
        if (typeof se != "function" && se !== null)
          throw new TypeError("Class extends value " + String(se) + " is not a constructor or null");
        ae(oe, se);
        function ce() {
          this.constructor = oe;
        }
        oe.prototype = se === null ? Object.create(se) : (ce.prototype = se.prototype, new ce());
      };
    }(), _ = HTMLDocument.__assign || function() {
      return _ = Object.assign || function(ae) {
        for (var oe, se = 1, ce = arguments.length; se < ce; se++) {
          oe = arguments[se];
          for (var le in oe) Object.prototype.hasOwnProperty.call(oe, le) && (ae[le] = oe[le]);
        }
        return ae;
      }, _.apply(this, arguments);
    }, H = HTMLDocument.__read || function(ae, oe) {
      var se = typeof Symbol == "function" && ae[Symbol.iterator];
      if (!se) return ae;
      var ce = se.call(ae), le, ue = [], de;
      try {
        for (; (oe === void 0 || oe-- > 0) && !(le = ce.next()).done; ) ue.push(le.value);
      } catch (fe) {
        de = { error: fe };
      } finally {
        try {
          le && !le.done && (se = ce.return) && se.call(ce);
        } finally {
          if (de) throw de.error;
        }
      }
      return ue;
    }, V = HTMLDocument.__values || function(ae) {
      var oe = typeof Symbol == "function" && Symbol.iterator, se = oe && ae[oe], ce = 0;
      if (se) return se.call(ae);
      if (ae && typeof ae.length == "number") return {
        next: function() {
          return ae && ce >= ae.length && (ae = void 0), { value: ae && ae[ce++], done: !ae };
        }
      };
      throw new TypeError(oe ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(HTMLDocument, "__esModule", { value: !0 }), HTMLDocument.HTMLDocument = void 0;
    var q = requireMathDocument(), G = requireOptions(), J = requireHTMLMathItem(), Z = requireHTMLMathList(), te = requireHTMLDomStrings(), ne = requireMathItem(), ee = function(ae) {
      O(oe, ae);
      function oe(se, ce, le) {
        var ue = this, de = H((0, G.separateOptions)(le, te.HTMLDomStrings.OPTIONS), 2), fe = de[0], he = de[1];
        return ue = ae.call(this, se, ce, fe) || this, ue.domStrings = ue.options.DomStrings || new te.HTMLDomStrings(he), ue.domStrings.adaptor = ce, ue.styles = [], ue;
      }
      return oe.prototype.findPosition = function(se, ce, le, ue) {
        var de, fe, he = this.adaptor;
        try {
          for (var me = V(ue[se]), pe = me.next(); !pe.done; pe = me.next()) {
            var ye = pe.value, ge = H(ye, 2), ve = ge[0], be = ge[1];
            if (ce <= be && he.kind(ve) === "#text")
              return { node: ve, n: Math.max(ce, 0), delim: le };
            ce -= be;
          }
        } catch (Ce) {
          de = { error: Ce };
        } finally {
          try {
            pe && !pe.done && (fe = me.return) && fe.call(me);
          } finally {
            if (de) throw de.error;
          }
        }
        return { node: null, n: 0, delim: le };
      }, oe.prototype.mathItem = function(se, ce, le) {
        var ue = se.math, de = this.findPosition(se.n, se.start.n, se.open, le), fe = this.findPosition(se.n, se.end.n, se.close, le);
        return new this.options.MathItem(ue, ce, se.display, de, fe);
      }, oe.prototype.findMath = function(se) {
        var ce, le, ue, de, fe, he, me, pe, ye;
        if (!this.processed.isSet("findMath")) {
          this.adaptor.document = this.document, se = (0, G.userOptions)({ elements: this.options.elements || [this.adaptor.body(this.document)] }, se);
          try {
            for (var ge = V(this.adaptor.getElements(se.elements, this.document)), ve = ge.next(); !ve.done; ve = ge.next()) {
              var be = ve.value, Ce = H([null, null], 2), Se = Ce[0], we = Ce[1];
              try {
                for (var Ee = (ue = void 0, V(this.inputJax)), xe = Ee.next(); !xe.done; xe = Ee.next()) {
                  var Le = xe.value, Qe = new this.options.MathList();
                  if (Le.processStrings) {
                    Se === null && (fe = H(this.domStrings.find(be), 2), Se = fe[0], we = fe[1]);
                    try {
                      for (var ke = (he = void 0, V(Le.findMath(Se))), Me = ke.next(); !Me.done; Me = ke.next()) {
                        var Re = Me.value;
                        Qe.push(this.mathItem(Re, Le, we));
                      }
                    } catch (qe) {
                      he = { error: qe };
                    } finally {
                      try {
                        Me && !Me.done && (me = ke.return) && me.call(ke);
                      } finally {
                        if (he) throw he.error;
                      }
                    }
                  } else
                    try {
                      for (var De = (pe = void 0, V(Le.findMath(be))), Pe = De.next(); !Pe.done; Pe = De.next()) {
                        var Re = Pe.value, $e = new this.options.MathItem(Re.math, Le, Re.display, Re.start, Re.end);
                        Qe.push($e);
                      }
                    } catch (qe) {
                      pe = { error: qe };
                    } finally {
                      try {
                        Pe && !Pe.done && (ye = De.return) && ye.call(De);
                      } finally {
                        if (pe) throw pe.error;
                      }
                    }
                  this.math.merge(Qe);
                }
              } catch (qe) {
                ue = { error: qe };
              } finally {
                try {
                  xe && !xe.done && (de = Ee.return) && de.call(Ee);
                } finally {
                  if (ue) throw ue.error;
                }
              }
            }
          } catch (qe) {
            ce = { error: qe };
          } finally {
            try {
              ve && !ve.done && (le = ge.return) && le.call(ge);
            } finally {
              if (ce) throw ce.error;
            }
          }
          this.processed.set("findMath");
        }
        return this;
      }, oe.prototype.updateDocument = function() {
        return this.processed.isSet("updateDocument") || (this.addPageElements(), this.addStyleSheet(), ae.prototype.updateDocument.call(this), this.processed.set("updateDocument")), this;
      }, oe.prototype.addPageElements = function() {
        var se = this.adaptor.body(this.document), ce = this.documentPageElements();
        ce && this.adaptor.append(se, ce);
      }, oe.prototype.addStyleSheet = function() {
        var se = this.documentStyleSheet(), ce = this.adaptor;
        if (se && !ce.parent(se)) {
          var le = ce.head(this.document), ue = this.findSheet(le, ce.getAttribute(se, "id"));
          ue ? ce.replace(se, ue) : ce.append(le, se);
        }
      }, oe.prototype.findSheet = function(se, ce) {
        var le, ue;
        if (ce)
          try {
            for (var de = V(this.adaptor.tags(se, "style")), fe = de.next(); !fe.done; fe = de.next()) {
              var he = fe.value;
              if (this.adaptor.getAttribute(he, "id") === ce)
                return he;
            }
          } catch (me) {
            le = { error: me };
          } finally {
            try {
              fe && !fe.done && (ue = de.return) && ue.call(de);
            } finally {
              if (le) throw le.error;
            }
          }
        return null;
      }, oe.prototype.removeFromDocument = function(se) {
        var ce, le;
        if (se === void 0 && (se = !1), this.processed.isSet("updateDocument"))
          try {
            for (var ue = V(this.math), de = ue.next(); !de.done; de = ue.next()) {
              var fe = de.value;
              fe.state() >= ne.STATE.INSERTED && fe.state(ne.STATE.TYPESET, se);
            }
          } catch (he) {
            ce = { error: he };
          } finally {
            try {
              de && !de.done && (le = ue.return) && le.call(ue);
            } finally {
              if (ce) throw ce.error;
            }
          }
        return this.processed.clear("updateDocument"), this;
      }, oe.prototype.documentStyleSheet = function() {
        return this.outputJax.styleSheet(this);
      }, oe.prototype.documentPageElements = function() {
        return this.outputJax.pageElements(this);
      }, oe.prototype.addStyles = function(se) {
        this.styles.push(se);
      }, oe.prototype.getStyles = function() {
        return this.styles;
      }, oe.KIND = "HTML", oe.OPTIONS = _(_({}, q.AbstractMathDocument.OPTIONS), { renderActions: (0, G.expandable)(_(_({}, q.AbstractMathDocument.OPTIONS.renderActions), { styles: [ne.STATE.INSERTED + 1, "", "updateStyleSheet", !1] })), MathList: Z.HTMLMathList, MathItem: J.HTMLMathItem, DomStrings: null }), oe;
    }(q.AbstractMathDocument);
    return HTMLDocument.HTMLDocument = ee, HTMLDocument;
  }
  var hasRequiredHTMLHandler;
  function requireHTMLHandler() {
    if (hasRequiredHTMLHandler) return HTMLHandler;
    hasRequiredHTMLHandler = 1;
    var O = HTMLHandler.__extends || /* @__PURE__ */ function() {
      var q = function(G, J) {
        return q = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(Z, te) {
          Z.__proto__ = te;
        } || function(Z, te) {
          for (var ne in te) Object.prototype.hasOwnProperty.call(te, ne) && (Z[ne] = te[ne]);
        }, q(G, J);
      };
      return function(G, J) {
        if (typeof J != "function" && J !== null)
          throw new TypeError("Class extends value " + String(J) + " is not a constructor or null");
        q(G, J);
        function Z() {
          this.constructor = G;
        }
        G.prototype = J === null ? Object.create(J) : (Z.prototype = J.prototype, new Z());
      };
    }();
    Object.defineProperty(HTMLHandler, "__esModule", { value: !0 }), HTMLHandler.HTMLHandler = void 0;
    var _ = requireHandler(), H = requireHTMLDocument(), V = function(q) {
      O(G, q);
      function G() {
        var J = q !== null && q.apply(this, arguments) || this;
        return J.documentClass = H.HTMLDocument, J;
      }
      return G.prototype.handlesDocument = function(J) {
        var Z = this.adaptor;
        if (typeof J == "string")
          try {
            J = Z.parse(J, "text/html");
          } catch {
          }
        return J instanceof Z.window.Document || J instanceof Z.window.HTMLElement || J instanceof Z.window.DocumentFragment;
      }, G.prototype.create = function(J, Z) {
        var te = this.adaptor;
        if (typeof J == "string")
          J = te.parse(J, "text/html");
        else if (J instanceof te.window.HTMLElement || J instanceof te.window.DocumentFragment) {
          var ne = J;
          J = te.parse("", "text/html"), te.append(te.body(J), ne);
        }
        return q.prototype.create.call(this, J, Z);
      }, G;
    }(_.AbstractHandler);
    return HTMLHandler.HTMLHandler = V, HTMLHandler;
  }
  var hasRequiredHtml;
  function requireHtml() {
    if (hasRequiredHtml) return html;
    hasRequiredHtml = 1, Object.defineProperty(html, "__esModule", { value: !0 }), html.RegisterHTMLHandler = void 0;
    var O = requireMathjax(), _ = requireHTMLHandler();
    function H(V) {
      var q = new _.HTMLHandler(V);
      return O.mathjax.handlers.register(q), q;
    }
    return html.RegisterHTMLHandler = H, html;
  }
  var htmlExports = requireHtml(), AllPackages = {}, ActionConfiguration = {}, hasRequiredActionConfiguration;
  function requireActionConfiguration() {
    return hasRequiredActionConfiguration || (hasRequiredActionConfiguration = 1, function(O) {
      var _ = ActionConfiguration.__importDefault || function(J) {
        return J && J.__esModule ? J : { default: J };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.ActionConfiguration = O.ActionMethods = void 0;
      var H = requireConfiguration(), V = _(requireTexParser()), q = requireSymbolMap(), G = _(requireBaseMethods());
      O.ActionMethods = {}, O.ActionMethods.Macro = G.default.Macro, O.ActionMethods.Toggle = function(J, Z) {
        for (var te = [], ne; (ne = J.GetArgument(Z)) !== "\\endtoggle"; )
          te.push(new V.default(ne, J.stack.env, J.configuration).mml());
        J.Push(J.create("node", "maction", te, { actiontype: "toggle" }));
      }, O.ActionMethods.Mathtip = function(J, Z) {
        var te = J.ParseArg(Z), ne = J.ParseArg(Z);
        J.Push(J.create("node", "maction", [te, ne], { actiontype: "tooltip" }));
      }, new q.CommandMap("action-macros", {
        toggle: "Toggle",
        mathtip: "Mathtip",
        texttip: ["Macro", "\\mathtip{#1}{\\text{#2}}", 2]
      }, O.ActionMethods), O.ActionConfiguration = H.Configuration.create("action", { handler: { macro: ["action-macros"] } });
    }(ActionConfiguration)), ActionConfiguration;
  }
  var AmsConfiguration = {}, AmsItems = {}, hasRequiredAmsItems;
  function requireAmsItems() {
    if (hasRequiredAmsItems) return AmsItems;
    hasRequiredAmsItems = 1;
    var O = AmsItems.__extends || /* @__PURE__ */ function() {
      var ee = function(ae, oe) {
        return ee = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(se, ce) {
          se.__proto__ = ce;
        } || function(se, ce) {
          for (var le in ce) Object.prototype.hasOwnProperty.call(ce, le) && (se[le] = ce[le]);
        }, ee(ae, oe);
      };
      return function(ae, oe) {
        if (typeof oe != "function" && oe !== null)
          throw new TypeError("Class extends value " + String(oe) + " is not a constructor or null");
        ee(ae, oe);
        function se() {
          this.constructor = ae;
        }
        ae.prototype = oe === null ? Object.create(oe) : (se.prototype = oe.prototype, new se());
      };
    }(), _ = AmsItems.__assign || function() {
      return _ = Object.assign || function(ee) {
        for (var ae, oe = 1, se = arguments.length; oe < se; oe++) {
          ae = arguments[oe];
          for (var ce in ae) Object.prototype.hasOwnProperty.call(ae, ce) && (ee[ce] = ae[ce]);
        }
        return ee;
      }, _.apply(this, arguments);
    }, H = AmsItems.__importDefault || function(ee) {
      return ee && ee.__esModule ? ee : { default: ee };
    };
    Object.defineProperty(AmsItems, "__esModule", { value: !0 }), AmsItems.FlalignItem = AmsItems.MultlineItem = void 0;
    var V = requireBaseItems(), q = H(requireParseUtil()), G = H(requireNodeUtil()), J = H(requireTexError()), Z = requireTexConstants(), te = function(ee) {
      O(ae, ee);
      function ae(oe) {
        for (var se = [], ce = 1; ce < arguments.length; ce++)
          se[ce - 1] = arguments[ce];
        var le = ee.call(this, oe) || this;
        return le.factory.configuration.tags.start("multline", !0, se[0]), le;
      }
      return Object.defineProperty(ae.prototype, "kind", {
        get: function() {
          return "multline";
        },
        enumerable: !1,
        configurable: !0
      }), ae.prototype.EndEntry = function() {
        this.table.length && q.default.fixInitialMO(this.factory.configuration, this.nodes);
        var oe = this.getProperty("shove"), se = this.create("node", "mtd", this.nodes, oe ? { columnalign: oe } : {});
        this.setProperty("shove", null), this.row.push(se), this.Clear();
      }, ae.prototype.EndRow = function() {
        if (this.row.length !== 1)
          throw new J.default("MultlineRowsOneCol", "The rows within the %1 environment must have exactly one column", "multline");
        var oe = this.create("node", "mtr", this.row);
        this.table.push(oe), this.row = [];
      }, ae.prototype.EndTable = function() {
        if (ee.prototype.EndTable.call(this), this.table.length) {
          var oe = this.table.length - 1, se = -1;
          G.default.getAttribute(G.default.getChildren(this.table[0])[0], "columnalign") || G.default.setAttribute(G.default.getChildren(this.table[0])[0], "columnalign", Z.TexConstant.Align.LEFT), G.default.getAttribute(G.default.getChildren(this.table[oe])[0], "columnalign") || G.default.setAttribute(G.default.getChildren(this.table[oe])[0], "columnalign", Z.TexConstant.Align.RIGHT);
          var ce = this.factory.configuration.tags.getTag();
          if (ce) {
            se = this.arraydef.side === Z.TexConstant.Align.LEFT ? 0 : this.table.length - 1;
            var le = this.table[se], ue = this.create("node", "mlabeledtr", [ce].concat(G.default.getChildren(le)));
            G.default.copyAttributes(le, ue), this.table[se] = ue;
          }
        }
        this.factory.configuration.tags.end();
      }, ae;
    }(V.ArrayItem);
    AmsItems.MultlineItem = te;
    var ne = function(ee) {
      O(ae, ee);
      function ae(oe, se, ce, le, ue) {
        var de = ee.call(this, oe) || this;
        return de.name = se, de.numbered = ce, de.padded = le, de.center = ue, de.factory.configuration.tags.start(se, ce, ce), de;
      }
      return Object.defineProperty(ae.prototype, "kind", {
        get: function() {
          return "flalign";
        },
        enumerable: !1,
        configurable: !0
      }), ae.prototype.EndEntry = function() {
        ee.prototype.EndEntry.call(this);
        var oe = this.getProperty("xalignat");
        if (oe && this.row.length > oe)
          throw new J.default("XalignOverflow", "Extra %1 in row of %2", "&", this.name);
      }, ae.prototype.EndRow = function() {
        for (var oe, se = this.row, ce = this.getProperty("xalignat"); se.length < ce; )
          se.push(this.create("node", "mtd"));
        for (this.row = [], this.padded && this.row.push(this.create("node", "mtd")); oe = se.shift(); )
          this.row.push(oe), oe = se.shift(), oe && this.row.push(oe), (se.length || this.padded) && this.row.push(this.create("node", "mtd"));
        this.row.length > this.maxrow && (this.maxrow = this.row.length), ee.prototype.EndRow.call(this);
        var le = this.table[this.table.length - 1];
        if (this.getProperty("zeroWidthLabel") && le.isKind("mlabeledtr")) {
          var ue = G.default.getChildren(le)[0], de = this.factory.configuration.options.tagSide, fe = _({ width: 0 }, de === "right" ? { lspace: "-1width" } : {}), he = this.create("node", "mpadded", G.default.getChildren(ue), fe);
          ue.setChildren([he]);
        }
      }, ae.prototype.EndTable = function() {
        if (ee.prototype.EndTable.call(this), this.center && this.maxrow <= 2) {
          var oe = this.arraydef;
          delete oe.width, delete this.global.indentalign;
        }
      }, ae;
    }(V.EqnArrayItem);
    return AmsItems.FlalignItem = ne, AmsItems;
  }
  var AmsMethods = {}, hasRequiredAmsMethods;
  function requireAmsMethods() {
    return hasRequiredAmsMethods || (hasRequiredAmsMethods = 1, function(O) {
      var _ = AmsMethods.__assign || function() {
        return _ = Object.assign || function(le) {
          for (var ue, de = 1, fe = arguments.length; de < fe; de++) {
            ue = arguments[de];
            for (var he in ue) Object.prototype.hasOwnProperty.call(ue, he) && (le[he] = ue[he]);
          }
          return le;
        }, _.apply(this, arguments);
      }, H = AmsMethods.__read || function(le, ue) {
        var de = typeof Symbol == "function" && le[Symbol.iterator];
        if (!de) return le;
        var fe = de.call(le), he, me = [], pe;
        try {
          for (; (ue === void 0 || ue-- > 0) && !(he = fe.next()).done; ) me.push(he.value);
        } catch (ye) {
          pe = { error: ye };
        } finally {
          try {
            he && !he.done && (de = fe.return) && de.call(fe);
          } finally {
            if (pe) throw pe.error;
          }
        }
        return me;
      }, V = AmsMethods.__importDefault || function(le) {
        return le && le.__esModule ? le : { default: le };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.NEW_OPS = O.AmsMethods = void 0;
      var q = V(requireParseUtil()), G = V(requireParseMethods()), J = V(requireNodeUtil()), Z = requireTexConstants(), te = V(requireTexParser()), ne = V(requireTexError()), ee = require_Symbol(), ae = V(requireBaseMethods()), oe = requireMmlNode();
      O.AmsMethods = {}, O.AmsMethods.AmsEqnArray = function(le, ue, de, fe, he, me, pe) {
        var ye = le.GetBrackets("\\begin{" + ue.getName() + "}"), ge = ae.default.EqnArray(le, ue, de, fe, he, me, pe);
        return q.default.setArrayAlign(ge, ye);
      }, O.AmsMethods.AlignAt = function(le, ue, de, fe) {
        var he = ue.getName(), me, pe, ye = "", ge = [];
        if (fe || (pe = le.GetBrackets("\\begin{" + he + "}")), me = le.GetArgument("\\begin{" + he + "}"), me.match(/[^0-9]/))
          throw new ne.default("PositiveIntegerArg", "Argument to %1 must me a positive integer", "\\begin{" + he + "}");
        for (var ve = parseInt(me, 10); ve > 0; )
          ye += "rl", ge.push("0em 0em"), ve--;
        var be = ge.join(" ");
        if (fe)
          return O.AmsMethods.EqnArray(le, ue, de, fe, ye, be);
        var Ce = O.AmsMethods.EqnArray(le, ue, de, fe, ye, be);
        return q.default.setArrayAlign(Ce, pe);
      }, O.AmsMethods.Multline = function(le, ue, de) {
        le.Push(ue), q.default.checkEqnEnv(le);
        var fe = le.itemFactory.create("multline", de, le.stack);
        return fe.arraydef = {
          displaystyle: !0,
          rowspacing: ".5em",
          columnspacing: "100%",
          width: le.options.ams.multlineWidth,
          side: le.options.tagSide,
          minlabelspacing: le.options.tagIndent,
          framespacing: le.options.ams.multlineIndent + " 0",
          frame: "",
          "data-width-includes-label": !0
        }, fe;
      }, O.AmsMethods.XalignAt = function(le, ue, de, fe) {
        var he = le.GetArgument("\\begin{" + ue.getName() + "}");
        if (he.match(/[^0-9]/))
          throw new ne.default("PositiveIntegerArg", "Argument to %1 must me a positive integer", "\\begin{" + ue.getName() + "}");
        var me = fe ? "crl" : "rlc", pe = fe ? "fit auto auto" : "auto auto fit", ye = O.AmsMethods.FlalignArray(le, ue, de, fe, !1, me, pe, !0);
        return ye.setProperty("xalignat", 2 * parseInt(he)), ye;
      }, O.AmsMethods.FlalignArray = function(le, ue, de, fe, he, me, pe, ye) {
        ye === void 0 && (ye = !1), le.Push(ue), q.default.checkEqnEnv(le), me = me.split("").join(" ").replace(/r/g, "right").replace(/l/g, "left").replace(/c/g, "center");
        var ge = le.itemFactory.create("flalign", ue.getName(), de, fe, he, le.stack);
        return ge.arraydef = {
          width: "100%",
          displaystyle: !0,
          columnalign: me,
          columnspacing: "0em",
          columnwidth: pe,
          rowspacing: "3pt",
          side: le.options.tagSide,
          minlabelspacing: ye ? "0" : le.options.tagIndent,
          "data-width-includes-label": !0
        }, ge.setProperty("zeroWidthLabel", ye), ge;
      }, O.NEW_OPS = "ams-declare-ops", O.AmsMethods.HandleDeclareOp = function(le, ue) {
        var de = le.GetStar() ? "*" : "", fe = q.default.trimSpaces(le.GetArgument(ue));
        fe.charAt(0) === "\\" && (fe = fe.substr(1));
        var he = le.GetArgument(ue);
        le.configuration.handlers.retrieve(O.NEW_OPS).add(fe, new ee.Macro(fe, O.AmsMethods.Macro, ["\\operatorname".concat(de, "{").concat(he, "}")]));
      }, O.AmsMethods.HandleOperatorName = function(le, ue) {
        var de = le.GetStar(), fe = q.default.trimSpaces(le.GetArgument(ue)), he = new te.default(fe, _(_({}, le.stack.env), { font: Z.TexConstant.Variant.NORMAL, multiLetterIdentifiers: /^[-*a-z]+/i, operatorLetters: !0 }), le.configuration).mml();
        if (he.isKind("mi") || (he = le.create("node", "TeXAtom", [he])), J.default.setProperties(he, { movesupsub: de, movablelimits: !0, texClass: oe.TEXCLASS.OP }), !de) {
          var me = le.GetNext(), pe = le.i;
          me === "\\" && ++le.i && le.GetCS() !== "limits" && (le.i = pe);
        }
        le.Push(he);
      }, O.AmsMethods.SideSet = function(le, ue) {
        var de = H(se(le.ParseArg(ue)), 2), fe = de[0], he = de[1], me = H(se(le.ParseArg(ue)), 2), pe = me[0], ye = me[1], ge = le.ParseArg(ue), ve = ge;
        fe && (he ? fe.replaceChild(le.create("node", "mphantom", [
          le.create("node", "mpadded", [q.default.copyNode(ge, le)], { width: 0 })
        ]), J.default.getChildAt(fe, 0)) : (ve = le.create("node", "mmultiscripts", [ge]), pe && J.default.appendChildren(ve, [
          J.default.getChildAt(pe, 1) || le.create("node", "none"),
          J.default.getChildAt(pe, 2) || le.create("node", "none")
        ]), J.default.setProperty(ve, "scriptalign", "left"), J.default.appendChildren(ve, [
          le.create("node", "mprescripts"),
          J.default.getChildAt(fe, 1) || le.create("node", "none"),
          J.default.getChildAt(fe, 2) || le.create("node", "none")
        ]))), pe && ve === ge && (pe.replaceChild(ge, J.default.getChildAt(pe, 0)), ve = pe);
        var be = le.create("node", "TeXAtom", [], { texClass: oe.TEXCLASS.OP, movesupsub: !0, movablelimits: !0 });
        he && (fe && be.appendChild(fe), be.appendChild(he)), be.appendChild(ve), ye && be.appendChild(ye), le.Push(be);
      };
      function se(le) {
        if (!le || le.isInferred && le.childNodes.length === 0)
          return [null, null];
        if (le.isKind("msubsup") && ce(le))
          return [le, null];
        var ue = J.default.getChildAt(le, 0);
        return le.isInferred && ue && ce(ue) ? (le.childNodes.splice(0, 1), [ue, le]) : [null, le];
      }
      function ce(le) {
        var ue = le.childNodes[0];
        return ue && ue.isKind("mi") && ue.getText() === "";
      }
      O.AmsMethods.operatorLetter = function(le, ue) {
        return le.stack.env.operatorLetters ? G.default.variable(le, ue) : !1;
      }, O.AmsMethods.MultiIntegral = function(le, ue, de) {
        var fe = le.GetNext();
        if (fe === "\\") {
          var he = le.i;
          fe = le.GetArgument(ue), le.i = he, fe === "\\limits" && (ue === "\\idotsint" ? de = "\\!\\!\\mathop{\\,\\," + de + "}" : de = "\\!\\!\\!\\mathop{\\,\\,\\," + de + "}");
        }
        le.string = de + " " + le.string.slice(le.i), le.i = 0;
      }, O.AmsMethods.xArrow = function(le, ue, de, fe, he) {
        var me = { width: "+" + q.default.Em((fe + he) / 18), lspace: q.default.Em(fe / 18) }, pe = le.GetBrackets(ue), ye = le.ParseArg(ue), ge = le.create("node", "mspace", [], { depth: ".25em" }), ve = le.create("token", "mo", { stretchy: !0, texClass: oe.TEXCLASS.REL }, String.fromCodePoint(de));
        ve = le.create("node", "mstyle", [ve], { scriptlevel: 0 });
        var be = le.create("node", "munderover", [ve]), Ce = le.create("node", "mpadded", [ye, ge], me);
        if (J.default.setAttribute(Ce, "voffset", "-.2em"), J.default.setAttribute(Ce, "height", "-.2em"), J.default.setChild(be, be.over, Ce), pe) {
          var Se = new te.default(pe, le.stack.env, le.configuration).mml(), we = le.create("node", "mspace", [], { height: ".75em" });
          Ce = le.create("node", "mpadded", [Se, we], me), J.default.setAttribute(Ce, "voffset", ".15em"), J.default.setAttribute(Ce, "depth", "-.15em"), J.default.setChild(be, be.under, Ce);
        }
        J.default.setProperty(be, "subsupOK", !0), le.Push(be);
      }, O.AmsMethods.HandleShove = function(le, ue, de) {
        var fe = le.stack.Top();
        if (fe.kind !== "multline")
          throw new ne.default("CommandOnlyAllowedInEnv", "%1 only allowed in %2 environment", le.currentCS, "multline");
        if (fe.Size())
          throw new ne.default("CommandAtTheBeginingOfLine", "%1 must come at the beginning of the line", le.currentCS);
        fe.setProperty("shove", de);
      }, O.AmsMethods.CFrac = function(le, ue) {
        var de = q.default.trimSpaces(le.GetBrackets(ue, "")), fe = le.GetArgument(ue), he = le.GetArgument(ue), me = {
          l: Z.TexConstant.Align.LEFT,
          r: Z.TexConstant.Align.RIGHT,
          "": ""
        }, pe = new te.default("\\strut\\textstyle{" + fe + "}", le.stack.env, le.configuration).mml(), ye = new te.default("\\strut\\textstyle{" + he + "}", le.stack.env, le.configuration).mml(), ge = le.create("node", "mfrac", [pe, ye]);
        if (de = me[de], de == null)
          throw new ne.default("IllegalAlign", "Illegal alignment specified in %1", le.currentCS);
        de && J.default.setProperties(ge, { numalign: de, denomalign: de }), le.Push(ge);
      }, O.AmsMethods.Genfrac = function(le, ue, de, fe, he, me) {
        de == null && (de = le.GetDelimiterArg(ue)), fe == null && (fe = le.GetDelimiterArg(ue)), he == null && (he = le.GetArgument(ue)), me == null && (me = q.default.trimSpaces(le.GetArgument(ue)));
        var pe = le.ParseArg(ue), ye = le.ParseArg(ue), ge = le.create("node", "mfrac", [pe, ye]);
        if (he !== "" && J.default.setAttribute(ge, "linethickness", he), (de || fe) && (J.default.setProperty(ge, "withDelims", !0), ge = q.default.fixedFence(le.configuration, de, ge, fe)), me !== "") {
          var ve = parseInt(me, 10), be = ["D", "T", "S", "SS"][ve];
          if (be == null)
            throw new ne.default("BadMathStyleFor", "Bad math style for %1", le.currentCS);
          ge = le.create("node", "mstyle", [ge]), be === "D" ? J.default.setProperties(ge, { displaystyle: !0, scriptlevel: 0 }) : J.default.setProperties(ge, {
            displaystyle: !1,
            scriptlevel: ve - 1
          });
        }
        le.Push(ge);
      }, O.AmsMethods.HandleTag = function(le, ue) {
        if (!le.tags.currentTag.taggable && le.tags.env)
          throw new ne.default("CommandNotAllowedInEnv", "%1 not allowed in %2 environment", le.currentCS, le.tags.env);
        if (le.tags.currentTag.tag)
          throw new ne.default("MultipleCommand", "Multiple %1", le.currentCS);
        var de = le.GetStar(), fe = q.default.trimSpaces(le.GetArgument(ue));
        le.tags.tag(fe, de);
      }, O.AmsMethods.HandleNoTag = ae.default.HandleNoTag, O.AmsMethods.HandleRef = ae.default.HandleRef, O.AmsMethods.Macro = ae.default.Macro, O.AmsMethods.Accent = ae.default.Accent, O.AmsMethods.Tilde = ae.default.Tilde, O.AmsMethods.Array = ae.default.Array, O.AmsMethods.Spacer = ae.default.Spacer, O.AmsMethods.NamedOp = ae.default.NamedOp, O.AmsMethods.EqnArray = ae.default.EqnArray, O.AmsMethods.Equation = ae.default.Equation;
    }(AmsMethods)), AmsMethods;
  }
  var AmsMappings = {}, hasRequiredAmsMappings;
  function requireAmsMappings() {
    if (hasRequiredAmsMappings) return AmsMappings;
    hasRequiredAmsMappings = 1;
    var O = AmsMappings.__createBinding || (Object.create ? function(ae, oe, se, ce) {
      ce === void 0 && (ce = se);
      var le = Object.getOwnPropertyDescriptor(oe, se);
      (!le || ("get" in le ? !oe.__esModule : le.writable || le.configurable)) && (le = { enumerable: !0, get: function() {
        return oe[se];
      } }), Object.defineProperty(ae, ce, le);
    } : function(ae, oe, se, ce) {
      ce === void 0 && (ce = se), ae[ce] = oe[se];
    }), _ = AmsMappings.__setModuleDefault || (Object.create ? function(ae, oe) {
      Object.defineProperty(ae, "default", { enumerable: !0, value: oe });
    } : function(ae, oe) {
      ae.default = oe;
    }), H = AmsMappings.__importStar || function(ae) {
      if (ae && ae.__esModule) return ae;
      var oe = {};
      if (ae != null) for (var se in ae) se !== "default" && Object.prototype.hasOwnProperty.call(ae, se) && O(oe, ae, se);
      return _(oe, ae), oe;
    }, V = AmsMappings.__importDefault || function(ae) {
      return ae && ae.__esModule ? ae : { default: ae };
    };
    Object.defineProperty(AmsMappings, "__esModule", { value: !0 });
    var q = requireAmsMethods(), G = H(requireSymbolMap()), J = requireTexConstants(), Z = V(requireParseMethods()), te = V(requireParseUtil()), ne = requireMmlNode(), ee = requireLengths();
    return new G.CharacterMap("AMSmath-mathchar0mo", Z.default.mathchar0mo, {
      iiiint: ["⨌", { texClass: ne.TEXCLASS.OP }]
    }), new G.RegExpMap("AMSmath-operatorLetter", q.AmsMethods.operatorLetter, /[-*]/i), new G.CommandMap("AMSmath-macros", {
      mathring: ["Accent", "02DA"],
      nobreakspace: "Tilde",
      negmedspace: ["Spacer", ee.MATHSPACE.negativemediummathspace],
      negthickspace: ["Spacer", ee.MATHSPACE.negativethickmathspace],
      idotsint: ["MultiIntegral", "\\int\\cdots\\int"],
      dddot: ["Accent", "20DB"],
      ddddot: ["Accent", "20DC"],
      sideset: "SideSet",
      boxed: ["Macro", "\\fbox{$\\displaystyle{#1}$}", 1],
      tag: "HandleTag",
      notag: "HandleNoTag",
      eqref: ["HandleRef", !0],
      substack: ["Macro", "\\begin{subarray}{c}#1\\end{subarray}", 1],
      injlim: ["NamedOp", "inj&thinsp;lim"],
      projlim: ["NamedOp", "proj&thinsp;lim"],
      varliminf: ["Macro", "\\mathop{\\underline{\\mmlToken{mi}{lim}}}"],
      varlimsup: ["Macro", "\\mathop{\\overline{\\mmlToken{mi}{lim}}}"],
      varinjlim: ["Macro", "\\mathop{\\underrightarrow{\\mmlToken{mi}{lim}}}"],
      varprojlim: ["Macro", "\\mathop{\\underleftarrow{\\mmlToken{mi}{lim}}}"],
      DeclareMathOperator: "HandleDeclareOp",
      operatorname: "HandleOperatorName",
      genfrac: "Genfrac",
      frac: ["Genfrac", "", "", "", ""],
      tfrac: ["Genfrac", "", "", "", "1"],
      dfrac: ["Genfrac", "", "", "", "0"],
      binom: ["Genfrac", "(", ")", "0", ""],
      tbinom: ["Genfrac", "(", ")", "0", "1"],
      dbinom: ["Genfrac", "(", ")", "0", "0"],
      cfrac: "CFrac",
      shoveleft: ["HandleShove", J.TexConstant.Align.LEFT],
      shoveright: ["HandleShove", J.TexConstant.Align.RIGHT],
      xrightarrow: ["xArrow", 8594, 5, 10],
      xleftarrow: ["xArrow", 8592, 10, 5]
    }, q.AmsMethods), new G.EnvironmentMap("AMSmath-environment", Z.default.environment, {
      "equation*": ["Equation", null, !1],
      "eqnarray*": [
        "EqnArray",
        null,
        !1,
        !0,
        "rcl",
        te.default.cols(0, ee.MATHSPACE.thickmathspace),
        ".5em"
      ],
      align: ["EqnArray", null, !0, !0, "rl", te.default.cols(0, 2)],
      "align*": ["EqnArray", null, !1, !0, "rl", te.default.cols(0, 2)],
      multline: ["Multline", null, !0],
      "multline*": ["Multline", null, !1],
      split: ["EqnArray", null, !1, !1, "rl", te.default.cols(0)],
      gather: ["EqnArray", null, !0, !0, "c"],
      "gather*": ["EqnArray", null, !1, !0, "c"],
      alignat: ["AlignAt", null, !0, !0],
      "alignat*": ["AlignAt", null, !1, !0],
      alignedat: ["AlignAt", null, !1, !1],
      aligned: ["AmsEqnArray", null, null, null, "rl", te.default.cols(0, 2), ".5em", "D"],
      gathered: ["AmsEqnArray", null, null, null, "c", null, ".5em", "D"],
      xalignat: ["XalignAt", null, !0, !0],
      "xalignat*": ["XalignAt", null, !1, !0],
      xxalignat: ["XalignAt", null, !1, !1],
      flalign: ["FlalignArray", null, !0, !1, !0, "rlc", "auto auto fit"],
      "flalign*": ["FlalignArray", null, !1, !1, !0, "rlc", "auto auto fit"],
      subarray: ["Array", null, null, null, null, te.default.cols(0), "0.1em", "S", 1],
      smallmatrix: [
        "Array",
        null,
        null,
        null,
        "c",
        te.default.cols(1 / 3),
        ".2em",
        "S",
        1
      ],
      matrix: ["Array", null, null, null, "c"],
      pmatrix: ["Array", null, "(", ")", "c"],
      bmatrix: ["Array", null, "[", "]", "c"],
      Bmatrix: ["Array", null, "\\{", "\\}", "c"],
      vmatrix: ["Array", null, "\\vert", "\\vert", "c"],
      Vmatrix: ["Array", null, "\\Vert", "\\Vert", "c"],
      cases: ["Array", null, "\\{", ".", "ll", null, ".2em", "T"]
    }, q.AmsMethods), new G.DelimiterMap("AMSmath-delimiter", Z.default.delimiter, {
      "\\lvert": ["|", { texClass: ne.TEXCLASS.OPEN }],
      "\\rvert": ["|", { texClass: ne.TEXCLASS.CLOSE }],
      "\\lVert": ["‖", { texClass: ne.TEXCLASS.OPEN }],
      "\\rVert": ["‖", { texClass: ne.TEXCLASS.CLOSE }]
    }), new G.CharacterMap("AMSsymbols-mathchar0mi", Z.default.mathchar0mi, {
      digamma: "ϝ",
      varkappa: "ϰ",
      varGamma: ["Γ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varDelta: ["Δ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varTheta: ["Θ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varLambda: ["Λ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varXi: ["Ξ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varPi: ["Π", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varSigma: ["Σ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varUpsilon: ["Υ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varPhi: ["Φ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varPsi: ["Ψ", { mathvariant: J.TexConstant.Variant.ITALIC }],
      varOmega: ["Ω", { mathvariant: J.TexConstant.Variant.ITALIC }],
      beth: "ℶ",
      gimel: "ℷ",
      daleth: "ℸ",
      backprime: ["‵", { variantForm: !0 }],
      hslash: "ℏ",
      varnothing: ["∅", { variantForm: !0 }],
      blacktriangle: "▴",
      triangledown: ["▽", { variantForm: !0 }],
      blacktriangledown: "▾",
      square: "◻",
      Box: "◻",
      blacksquare: "◼",
      lozenge: "◊",
      Diamond: "◊",
      blacklozenge: "⧫",
      circledS: ["Ⓢ", { mathvariant: J.TexConstant.Variant.NORMAL }],
      bigstar: "★",
      sphericalangle: "∢",
      measuredangle: "∡",
      nexists: "∄",
      complement: "∁",
      mho: "℧",
      eth: ["ð", { mathvariant: J.TexConstant.Variant.NORMAL }],
      Finv: "Ⅎ",
      diagup: "╱",
      Game: "⅁",
      diagdown: "╲",
      Bbbk: [
        "k",
        { mathvariant: J.TexConstant.Variant.DOUBLESTRUCK }
      ],
      yen: "¥",
      circledR: "®",
      checkmark: "✓",
      maltese: "✠"
    }), new G.CharacterMap("AMSsymbols-mathchar0mo", Z.default.mathchar0mo, {
      dotplus: "∔",
      ltimes: "⋉",
      smallsetminus: ["∖", { variantForm: !0 }],
      rtimes: "⋊",
      Cap: "⋒",
      doublecap: "⋒",
      leftthreetimes: "⋋",
      Cup: "⋓",
      doublecup: "⋓",
      rightthreetimes: "⋌",
      barwedge: "⊼",
      curlywedge: "⋏",
      veebar: "⊻",
      curlyvee: "⋎",
      doublebarwedge: "⩞",
      boxminus: "⊟",
      circleddash: "⊝",
      boxtimes: "⊠",
      circledast: "⊛",
      boxdot: "⊡",
      circledcirc: "⊚",
      boxplus: "⊞",
      centerdot: ["⋅", { variantForm: !0 }],
      divideontimes: "⋇",
      intercal: "⊺",
      leqq: "≦",
      geqq: "≧",
      leqslant: "⩽",
      geqslant: "⩾",
      eqslantless: "⪕",
      eqslantgtr: "⪖",
      lesssim: "≲",
      gtrsim: "≳",
      lessapprox: "⪅",
      gtrapprox: "⪆",
      approxeq: "≊",
      lessdot: "⋖",
      gtrdot: "⋗",
      lll: "⋘",
      llless: "⋘",
      ggg: "⋙",
      gggtr: "⋙",
      lessgtr: "≶",
      gtrless: "≷",
      lesseqgtr: "⋚",
      gtreqless: "⋛",
      lesseqqgtr: "⪋",
      gtreqqless: "⪌",
      doteqdot: "≑",
      Doteq: "≑",
      eqcirc: "≖",
      risingdotseq: "≓",
      circeq: "≗",
      fallingdotseq: "≒",
      triangleq: "≜",
      backsim: "∽",
      thicksim: ["∼", { variantForm: !0 }],
      backsimeq: "⋍",
      thickapprox: ["≈", { variantForm: !0 }],
      subseteqq: "⫅",
      supseteqq: "⫆",
      Subset: "⋐",
      Supset: "⋑",
      sqsubset: "⊏",
      sqsupset: "⊐",
      preccurlyeq: "≼",
      succcurlyeq: "≽",
      curlyeqprec: "⋞",
      curlyeqsucc: "⋟",
      precsim: "≾",
      succsim: "≿",
      precapprox: "⪷",
      succapprox: "⪸",
      vartriangleleft: "⊲",
      lhd: "⊲",
      vartriangleright: "⊳",
      rhd: "⊳",
      trianglelefteq: "⊴",
      unlhd: "⊴",
      trianglerighteq: "⊵",
      unrhd: "⊵",
      vDash: ["⊨", { variantForm: !0 }],
      Vdash: "⊩",
      Vvdash: "⊪",
      smallsmile: ["⌣", { variantForm: !0 }],
      shortmid: ["∣", { variantForm: !0 }],
      smallfrown: ["⌢", { variantForm: !0 }],
      shortparallel: ["∥", { variantForm: !0 }],
      bumpeq: "≏",
      between: "≬",
      Bumpeq: "≎",
      pitchfork: "⋔",
      varpropto: ["∝", { variantForm: !0 }],
      backepsilon: "∍",
      blacktriangleleft: "◂",
      blacktriangleright: "▸",
      therefore: "∴",
      because: "∵",
      eqsim: "≂",
      vartriangle: ["△", { variantForm: !0 }],
      Join: "⋈",
      nless: "≮",
      ngtr: "≯",
      nleq: "≰",
      ngeq: "≱",
      nleqslant: ["⪇", { variantForm: !0 }],
      ngeqslant: ["⪈", { variantForm: !0 }],
      nleqq: ["≰", { variantForm: !0 }],
      ngeqq: ["≱", { variantForm: !0 }],
      lneq: "⪇",
      gneq: "⪈",
      lneqq: "≨",
      gneqq: "≩",
      lvertneqq: ["≨", { variantForm: !0 }],
      gvertneqq: ["≩", { variantForm: !0 }],
      lnsim: "⋦",
      gnsim: "⋧",
      lnapprox: "⪉",
      gnapprox: "⪊",
      nprec: "⊀",
      nsucc: "⊁",
      npreceq: ["⋠", { variantForm: !0 }],
      nsucceq: ["⋡", { variantForm: !0 }],
      precneqq: "⪵",
      succneqq: "⪶",
      precnsim: "⋨",
      succnsim: "⋩",
      precnapprox: "⪹",
      succnapprox: "⪺",
      nsim: "≁",
      ncong: "≇",
      nshortmid: ["∤", { variantForm: !0 }],
      nshortparallel: ["∦", { variantForm: !0 }],
      nmid: "∤",
      nparallel: "∦",
      nvdash: "⊬",
      nvDash: "⊭",
      nVdash: "⊮",
      nVDash: "⊯",
      ntriangleleft: "⋪",
      ntriangleright: "⋫",
      ntrianglelefteq: "⋬",
      ntrianglerighteq: "⋭",
      nsubseteq: "⊈",
      nsupseteq: "⊉",
      nsubseteqq: ["⊈", { variantForm: !0 }],
      nsupseteqq: ["⊉", { variantForm: !0 }],
      subsetneq: "⊊",
      supsetneq: "⊋",
      varsubsetneq: ["⊊", { variantForm: !0 }],
      varsupsetneq: ["⊋", { variantForm: !0 }],
      subsetneqq: "⫋",
      supsetneqq: "⫌",
      varsubsetneqq: ["⫋", { variantForm: !0 }],
      varsupsetneqq: ["⫌", { variantForm: !0 }],
      leftleftarrows: "⇇",
      rightrightarrows: "⇉",
      leftrightarrows: "⇆",
      rightleftarrows: "⇄",
      Lleftarrow: "⇚",
      Rrightarrow: "⇛",
      twoheadleftarrow: "↞",
      twoheadrightarrow: "↠",
      leftarrowtail: "↢",
      rightarrowtail: "↣",
      looparrowleft: "↫",
      looparrowright: "↬",
      leftrightharpoons: "⇋",
      rightleftharpoons: ["⇌", { variantForm: !0 }],
      curvearrowleft: "↶",
      curvearrowright: "↷",
      circlearrowleft: "↺",
      circlearrowright: "↻",
      Lsh: "↰",
      Rsh: "↱",
      upuparrows: "⇈",
      downdownarrows: "⇊",
      upharpoonleft: "↿",
      upharpoonright: "↾",
      downharpoonleft: "⇃",
      restriction: "↾",
      multimap: "⊸",
      downharpoonright: "⇂",
      leftrightsquigarrow: "↭",
      rightsquigarrow: "⇝",
      leadsto: "⇝",
      dashrightarrow: "⇢",
      dashleftarrow: "⇠",
      nleftarrow: "↚",
      nrightarrow: "↛",
      nLeftarrow: "⇍",
      nRightarrow: "⇏",
      nleftrightarrow: "↮",
      nLeftrightarrow: "⇎"
    }), new G.DelimiterMap("AMSsymbols-delimiter", Z.default.delimiter, {
      "\\ulcorner": "⌜",
      "\\urcorner": "⌝",
      "\\llcorner": "⌞",
      "\\lrcorner": "⌟"
    }), new G.CommandMap("AMSsymbols-macros", {
      implies: ["Macro", "\\;\\Longrightarrow\\;"],
      impliedby: ["Macro", "\\;\\Longleftarrow\\;"]
    }, q.AmsMethods), AmsMappings;
  }
  var hasRequiredAmsConfiguration;
  function requireAmsConfiguration() {
    if (hasRequiredAmsConfiguration) return AmsConfiguration;
    hasRequiredAmsConfiguration = 1;
    var O = AmsConfiguration.__extends || /* @__PURE__ */ function() {
      var ne = function(ee, ae) {
        return ne = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(oe, se) {
          oe.__proto__ = se;
        } || function(oe, se) {
          for (var ce in se) Object.prototype.hasOwnProperty.call(se, ce) && (oe[ce] = se[ce]);
        }, ne(ee, ae);
      };
      return function(ee, ae) {
        if (typeof ae != "function" && ae !== null)
          throw new TypeError("Class extends value " + String(ae) + " is not a constructor or null");
        ne(ee, ae);
        function oe() {
          this.constructor = ee;
        }
        ee.prototype = ae === null ? Object.create(ae) : (oe.prototype = ae.prototype, new oe());
      };
    }(), _;
    Object.defineProperty(AmsConfiguration, "__esModule", { value: !0 }), AmsConfiguration.AmsConfiguration = AmsConfiguration.AmsTags = void 0;
    var H = requireConfiguration(), V = requireAmsItems(), q = requireTags(), G = requireAmsMethods();
    requireAmsMappings();
    var J = requireSymbolMap(), Z = function(ne) {
      O(ee, ne);
      function ee() {
        return ne !== null && ne.apply(this, arguments) || this;
      }
      return ee;
    }(q.AbstractTags);
    AmsConfiguration.AmsTags = Z;
    var te = function(ne) {
      new J.CommandMap(G.NEW_OPS, {}, {}), ne.append(H.Configuration.local({
        handler: { macro: [G.NEW_OPS] },
        priority: -1
      }));
    };
    return AmsConfiguration.AmsConfiguration = H.Configuration.create("ams", {
      handler: {
        character: ["AMSmath-operatorLetter"],
        delimiter: ["AMSsymbols-delimiter", "AMSmath-delimiter"],
        macro: [
          "AMSsymbols-mathchar0mi",
          "AMSsymbols-mathchar0mo",
          "AMSsymbols-delimiter",
          "AMSsymbols-macros",
          "AMSmath-mathchar0mo",
          "AMSmath-macros",
          "AMSmath-delimiter"
        ],
        environment: ["AMSmath-environment"]
      },
      items: (_ = {}, _[V.MultlineItem.prototype.kind] = V.MultlineItem, _[V.FlalignItem.prototype.kind] = V.FlalignItem, _),
      tags: { ams: Z },
      init: te,
      config: function(ne, ee) {
        ee.parseOptions.options.multlineWidth && (ee.parseOptions.options.ams.multlineWidth = ee.parseOptions.options.multlineWidth), delete ee.parseOptions.options.multlineWidth;
      },
      options: {
        multlineWidth: "",
        ams: {
          multlineWidth: "100%",
          multlineIndent: "1em"
        }
      }
    }), AmsConfiguration;
  }
  var AmsCdConfiguration = {}, AmsCdMappings = {}, AmsCdMethods = {}, hasRequiredAmsCdMethods;
  function requireAmsCdMethods() {
    if (hasRequiredAmsCdMethods) return AmsCdMethods;
    hasRequiredAmsCdMethods = 1;
    var O = AmsCdMethods.__importDefault || function(J) {
      return J && J.__esModule ? J : { default: J };
    };
    Object.defineProperty(AmsCdMethods, "__esModule", { value: !0 });
    var _ = O(requireTexParser()), H = requireBaseConfiguration(), V = requireMmlNode(), q = O(requireNodeUtil()), G = {};
    return G.CD = function(J, Z) {
      J.Push(Z);
      var te = J.itemFactory.create("array"), ne = J.configuration.options.amscd;
      return te.setProperties({
        minw: J.stack.env.CD_minw || ne.harrowsize,
        minh: J.stack.env.CD_minh || ne.varrowsize
      }), te.arraydef = {
        columnalign: "center",
        columnspacing: ne.colspace,
        rowspacing: ne.rowspace,
        displaystyle: !0
      }, te;
    }, G.arrow = function(J, Z) {
      var te = J.string.charAt(J.i);
      if (te.match(/[><VA.|=]/))
        J.i++;
      else
        return (0, H.Other)(J, Z);
      var ne = J.stack.Top();
      (!ne.isKind("array") || ne.Size()) && (G.cell(J, Z), ne = J.stack.Top());
      for (var ee = ne, ae = ee.table.length % 2 === 1, oe = (ee.row.length + (ae ? 0 : 1)) % 2; oe; )
        G.cell(J, Z), oe--;
      var se, ce = { minsize: ee.getProperty("minw"), stretchy: !0 }, le = {
        minsize: ee.getProperty("minh"),
        stretchy: !0,
        symmetric: !0,
        lspace: 0,
        rspace: 0
      };
      if (te !== ".") if (te === "|")
        se = J.create("token", "mo", le, "∥");
      else if (te === "=")
        se = J.create("token", "mo", ce, "=");
      else {
        var ue = {
          ">": "→",
          "<": "←",
          V: "↓",
          A: "↑"
        }[te], de = J.GetUpTo(Z + te, te), fe = J.GetUpTo(Z + te, te);
        if (te === ">" || te === "<") {
          if (se = J.create("token", "mo", ce, ue), de || (de = "\\kern " + ee.getProperty("minw")), de || fe) {
            var he = { width: "+.67em", lspace: ".33em" };
            if (se = J.create("node", "munderover", [se]), de) {
              var me = new _.default(de, J.stack.env, J.configuration).mml(), pe = J.create("node", "mpadded", [me], he);
              q.default.setAttribute(pe, "voffset", ".1em"), q.default.setChild(se, se.over, pe);
            }
            if (fe) {
              var ye = new _.default(fe, J.stack.env, J.configuration).mml();
              q.default.setChild(se, se.under, J.create("node", "mpadded", [ye], he));
            }
            J.configuration.options.amscd.hideHorizontalLabels && (se = J.create("node", "mpadded", se, { depth: 0, height: ".67em" }));
          }
        } else {
          var ge = J.create("token", "mo", le, ue);
          se = ge, (de || fe) && (se = J.create("node", "mrow"), de && q.default.appendChildren(se, [new _.default("\\scriptstyle\\llap{" + de + "}", J.stack.env, J.configuration).mml()]), ge.texClass = V.TEXCLASS.ORD, q.default.appendChildren(se, [ge]), fe && q.default.appendChildren(se, [new _.default("\\scriptstyle\\rlap{" + fe + "}", J.stack.env, J.configuration).mml()]));
        }
      }
      se && J.Push(se), G.cell(J, Z);
    }, G.cell = function(J, Z) {
      var te = J.stack.Top();
      (te.table || []).length % 2 === 0 && (te.row || []).length === 0 && J.Push(J.create("node", "mpadded", [], { height: "8.5pt", depth: "2pt" })), J.Push(J.itemFactory.create("cell").setProperties({ isEntry: !0, name: Z }));
    }, G.minCDarrowwidth = function(J, Z) {
      J.stack.env.CD_minw = J.GetDimen(Z);
    }, G.minCDarrowheight = function(J, Z) {
      J.stack.env.CD_minh = J.GetDimen(Z);
    }, AmsCdMethods.default = G, AmsCdMethods;
  }
  var hasRequiredAmsCdMappings;
  function requireAmsCdMappings() {
    if (hasRequiredAmsCdMappings) return AmsCdMappings;
    hasRequiredAmsCdMappings = 1;
    var O = AmsCdMappings.__createBinding || (Object.create ? function(Z, te, ne, ee) {
      ee === void 0 && (ee = ne);
      var ae = Object.getOwnPropertyDescriptor(te, ne);
      (!ae || ("get" in ae ? !te.__esModule : ae.writable || ae.configurable)) && (ae = { enumerable: !0, get: function() {
        return te[ne];
      } }), Object.defineProperty(Z, ee, ae);
    } : function(Z, te, ne, ee) {
      ee === void 0 && (ee = ne), Z[ee] = te[ne];
    }), _ = AmsCdMappings.__setModuleDefault || (Object.create ? function(Z, te) {
      Object.defineProperty(Z, "default", { enumerable: !0, value: te });
    } : function(Z, te) {
      Z.default = te;
    }), H = AmsCdMappings.__importStar || function(Z) {
      if (Z && Z.__esModule) return Z;
      var te = {};
      if (Z != null) for (var ne in Z) ne !== "default" && Object.prototype.hasOwnProperty.call(Z, ne) && O(te, Z, ne);
      return _(te, Z), te;
    }, V = AmsCdMappings.__importDefault || function(Z) {
      return Z && Z.__esModule ? Z : { default: Z };
    };
    Object.defineProperty(AmsCdMappings, "__esModule", { value: !0 });
    var q = H(requireSymbolMap()), G = V(requireParseMethods()), J = V(requireAmsCdMethods());
    return new q.EnvironmentMap("amscd_environment", G.default.environment, { CD: "CD" }, J.default), new q.CommandMap("amscd_macros", {
      minCDarrowwidth: "minCDarrowwidth",
      minCDarrowheight: "minCDarrowheight"
    }, J.default), new q.MacroMap("amscd_special", { "@": "arrow" }, J.default), AmsCdMappings;
  }
  var hasRequiredAmsCdConfiguration;
  function requireAmsCdConfiguration() {
    if (hasRequiredAmsCdConfiguration) return AmsCdConfiguration;
    hasRequiredAmsCdConfiguration = 1, Object.defineProperty(AmsCdConfiguration, "__esModule", { value: !0 }), AmsCdConfiguration.AmsCdConfiguration = void 0;
    var O = requireConfiguration();
    return requireAmsCdMappings(), AmsCdConfiguration.AmsCdConfiguration = O.Configuration.create("amscd", {
      handler: {
        character: ["amscd_special"],
        macro: ["amscd_macros"],
        environment: ["amscd_environment"]
      },
      options: {
        amscd: {
          colspace: "5pt",
          rowspace: "5pt",
          harrowsize: "2.75em",
          varrowsize: "1.75em",
          hideHorizontalLabels: !1
        }
      }
    }), AmsCdConfiguration;
  }
  var BboxConfiguration = {}, hasRequiredBboxConfiguration;
  function requireBboxConfiguration() {
    return hasRequiredBboxConfiguration || (hasRequiredBboxConfiguration = 1, function(O) {
      var _ = BboxConfiguration.__importDefault || function(Z) {
        return Z && Z.__esModule ? Z : { default: Z };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.BboxConfiguration = O.BboxMethods = void 0;
      var H = requireConfiguration(), V = requireSymbolMap(), q = _(requireTexError());
      O.BboxMethods = {}, O.BboxMethods.BBox = function(Z, te) {
        for (var ne = Z.GetBrackets(te, ""), ee = Z.ParseArg(te), ae = ne.split(/,/), oe, se, ce, le = 0, ue = ae.length; le < ue; le++) {
          var de = ae[le].trim(), fe = de.match(/^(\.\d+|\d+(\.\d*)?)(pt|em|ex|mu|px|in|cm|mm)$/);
          if (fe) {
            if (oe)
              throw new q.default("MultipleBBoxProperty", "%1 specified twice in %2", "Padding", te);
            var he = J(fe[1] + fe[3]);
            he && (oe = {
              height: "+" + he,
              depth: "+" + he,
              lspace: he,
              width: "+" + 2 * parseInt(fe[1], 10) + fe[3]
            });
          } else if (de.match(/^([a-z0-9]+|\#[0-9a-f]{6}|\#[0-9a-f]{3})$/i)) {
            if (se)
              throw new q.default("MultipleBBoxProperty", "%1 specified twice in %2", "Background", te);
            se = de;
          } else if (de.match(/^[-a-z]+:/i)) {
            if (ce)
              throw new q.default("MultipleBBoxProperty", "%1 specified twice in %2", "Style", te);
            ce = G(de);
          } else if (de !== "")
            throw new q.default("InvalidBBoxProperty", `"%1" doesn't look like a color, a padding dimension, or a style`, de);
        }
        oe && (ee = Z.create("node", "mpadded", [ee], oe)), (se || ce) && (oe = {}, se && Object.assign(oe, { mathbackground: se }), ce && Object.assign(oe, { style: ce }), ee = Z.create("node", "mstyle", [ee], oe)), Z.Push(ee);
      };
      var G = function(Z) {
        return Z;
      }, J = function(Z) {
        return Z;
      };
      new V.CommandMap("bbox", { bbox: "BBox" }, O.BboxMethods), O.BboxConfiguration = H.Configuration.create("bbox", { handler: { macro: ["bbox"] } });
    }(BboxConfiguration)), BboxConfiguration;
  }
  var BoldsymbolConfiguration = {}, hasRequiredBoldsymbolConfiguration;
  function requireBoldsymbolConfiguration() {
    return hasRequiredBoldsymbolConfiguration || (hasRequiredBoldsymbolConfiguration = 1, function(O) {
      var _ = BoldsymbolConfiguration.__values || function(ae) {
        var oe = typeof Symbol == "function" && Symbol.iterator, se = oe && ae[oe], ce = 0;
        if (se) return se.call(ae);
        if (ae && typeof ae.length == "number") return {
          next: function() {
            return ae && ce >= ae.length && (ae = void 0), { value: ae && ae[ce++], done: !ae };
          }
        };
        throw new TypeError(oe ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, H = BoldsymbolConfiguration.__importDefault || function(ae) {
        return ae && ae.__esModule ? ae : { default: ae };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.BoldsymbolConfiguration = O.rewriteBoldTokens = O.createBoldToken = O.BoldsymbolMethods = void 0;
      var V = requireConfiguration(), q = H(requireNodeUtil()), G = requireTexConstants(), J = requireSymbolMap(), Z = requireNodeFactory$1(), te = {};
      te[G.TexConstant.Variant.NORMAL] = G.TexConstant.Variant.BOLD, te[G.TexConstant.Variant.ITALIC] = G.TexConstant.Variant.BOLDITALIC, te[G.TexConstant.Variant.FRAKTUR] = G.TexConstant.Variant.BOLDFRAKTUR, te[G.TexConstant.Variant.SCRIPT] = G.TexConstant.Variant.BOLDSCRIPT, te[G.TexConstant.Variant.SANSSERIF] = G.TexConstant.Variant.BOLDSANSSERIF, te["-tex-calligraphic"] = "-tex-bold-calligraphic", te["-tex-oldstyle"] = "-tex-bold-oldstyle", te["-tex-mathit"] = G.TexConstant.Variant.BOLDITALIC, O.BoldsymbolMethods = {}, O.BoldsymbolMethods.Boldsymbol = function(ae, oe) {
        var se = ae.stack.env.boldsymbol;
        ae.stack.env.boldsymbol = !0;
        var ce = ae.ParseArg(oe);
        ae.stack.env.boldsymbol = se, ae.Push(ce);
      }, new J.CommandMap("boldsymbol", { boldsymbol: "Boldsymbol" }, O.BoldsymbolMethods);
      function ne(ae, oe, se, ce) {
        var le = Z.NodeFactory.createToken(ae, oe, se, ce);
        return oe !== "mtext" && ae.configuration.parser.stack.env.boldsymbol && (q.default.setProperty(le, "fixBold", !0), ae.configuration.addNode("fixBold", le)), le;
      }
      O.createBoldToken = ne;
      function ee(ae) {
        var oe, se;
        try {
          for (var ce = _(ae.data.getList("fixBold")), le = ce.next(); !le.done; le = ce.next()) {
            var ue = le.value;
            if (q.default.getProperty(ue, "fixBold")) {
              var de = q.default.getAttribute(ue, "mathvariant");
              de == null ? q.default.setAttribute(ue, "mathvariant", G.TexConstant.Variant.BOLD) : q.default.setAttribute(ue, "mathvariant", te[de] || de), q.default.removeProperties(ue, "fixBold");
            }
          }
        } catch (fe) {
          oe = { error: fe };
        } finally {
          try {
            le && !le.done && (se = ce.return) && se.call(ce);
          } finally {
            if (oe) throw oe.error;
          }
        }
      }
      O.rewriteBoldTokens = ee, O.BoldsymbolConfiguration = V.Configuration.create("boldsymbol", {
        handler: { macro: ["boldsymbol"] },
        nodes: { token: ne },
        postprocessors: [ee]
      });
    }(BoldsymbolConfiguration)), BoldsymbolConfiguration;
  }
  var BraketConfiguration = {}, BraketItems = {}, hasRequiredBraketItems;
  function requireBraketItems() {
    if (hasRequiredBraketItems) return BraketItems;
    hasRequiredBraketItems = 1;
    var O = BraketItems.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = BraketItems.__importDefault || function(J) {
      return J && J.__esModule ? J : { default: J };
    };
    Object.defineProperty(BraketItems, "__esModule", { value: !0 }), BraketItems.BraketItem = void 0;
    var H = requireStackItem(), V = requireMmlNode(), q = _(requireParseUtil()), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "braket";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(Z.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.checkItem = function(te) {
        return te.isKind("close") ? [[this.factory.create("mml", this.toMml())], !0] : te.isKind("mml") ? (this.Push(te.toMml()), this.getProperty("single") ? [[this.toMml()], !0] : H.BaseItem.fail) : J.prototype.checkItem.call(this, te);
      }, Z.prototype.toMml = function() {
        var te = J.prototype.toMml.call(this), ne = this.getProperty("open"), ee = this.getProperty("close");
        if (this.getProperty("stretchy"))
          return q.default.fenced(this.factory.configuration, ne, te, ee);
        var ae = { fence: !0, stretchy: !1, symmetric: !0, texClass: V.TEXCLASS.OPEN }, oe = this.create("token", "mo", ae, ne);
        ae.texClass = V.TEXCLASS.CLOSE;
        var se = this.create("token", "mo", ae, ee), ce = this.create("node", "mrow", [oe, te, se], { open: ne, close: ee, texClass: V.TEXCLASS.INNER });
        return ce;
      }, Z;
    }(H.BaseItem);
    return BraketItems.BraketItem = G, BraketItems;
  }
  var BraketMappings = {}, BraketMethods = {}, hasRequiredBraketMethods;
  function requireBraketMethods() {
    if (hasRequiredBraketMethods) return BraketMethods;
    hasRequiredBraketMethods = 1;
    var O = BraketMethods.__importDefault || function(G) {
      return G && G.__esModule ? G : { default: G };
    };
    Object.defineProperty(BraketMethods, "__esModule", { value: !0 });
    var _ = O(requireBaseMethods()), H = requireMmlNode(), V = O(requireTexError()), q = {};
    return q.Macro = _.default.Macro, q.Braket = function(G, J, Z, te, ne, ee) {
      var ae = G.GetNext();
      if (ae === "")
        throw new V.default("MissingArgFor", "Missing argument for %1", G.currentCS);
      var oe = !0;
      ae === "{" && (G.i++, oe = !1), G.Push(G.itemFactory.create("braket").setProperties({
        barmax: ee,
        barcount: 0,
        open: Z,
        close: te,
        stretchy: ne,
        single: oe
      }));
    }, q.Bar = function(G, J) {
      var Z = J === "|" ? "|" : "∥", te = G.stack.Top();
      if (te.kind !== "braket" || te.getProperty("barcount") >= te.getProperty("barmax")) {
        var ne = G.create("token", "mo", { texClass: H.TEXCLASS.ORD, stretchy: !1 }, Z);
        G.Push(ne);
        return;
      }
      Z === "|" && G.GetNext() === "|" && (G.i++, Z = "∥");
      var ee = te.getProperty("stretchy");
      if (!ee) {
        var ae = G.create("token", "mo", { stretchy: !1, braketbar: !0 }, Z);
        G.Push(ae);
        return;
      }
      var oe = G.create("node", "TeXAtom", [], { texClass: H.TEXCLASS.CLOSE });
      G.Push(oe), te.setProperty("barcount", te.getProperty("barcount") + 1), oe = G.create("token", "mo", { stretchy: !0, braketbar: !0 }, Z), G.Push(oe), oe = G.create("node", "TeXAtom", [], { texClass: H.TEXCLASS.OPEN }), G.Push(oe);
    }, BraketMethods.default = q, BraketMethods;
  }
  var hasRequiredBraketMappings;
  function requireBraketMappings() {
    if (hasRequiredBraketMappings) return BraketMappings;
    hasRequiredBraketMappings = 1;
    var O = BraketMappings.__importDefault || function(V) {
      return V && V.__esModule ? V : { default: V };
    };
    Object.defineProperty(BraketMappings, "__esModule", { value: !0 });
    var _ = requireSymbolMap(), H = O(requireBraketMethods());
    return new _.CommandMap("Braket-macros", {
      bra: ["Macro", "{\\langle {#1} \\vert}", 1],
      ket: ["Macro", "{\\vert {#1} \\rangle}", 1],
      braket: ["Braket", "⟨", "⟩", !1, 1 / 0],
      set: ["Braket", "{", "}", !1, 1],
      Bra: ["Macro", "{\\left\\langle {#1} \\right\\vert}", 1],
      Ket: ["Macro", "{\\left\\vert {#1} \\right\\rangle}", 1],
      Braket: ["Braket", "⟨", "⟩", !0, 1 / 0],
      Set: ["Braket", "{", "}", !0, 1],
      ketbra: ["Macro", "{\\vert {#1} \\rangle\\langle {#2} \\vert}", 2],
      Ketbra: ["Macro", "{\\left\\vert {#1} \\right\\rangle\\left\\langle {#2} \\right\\vert}", 2],
      "|": "Bar"
    }, H.default), new _.MacroMap("Braket-characters", {
      "|": "Bar"
    }, H.default), BraketMappings;
  }
  var hasRequiredBraketConfiguration;
  function requireBraketConfiguration() {
    if (hasRequiredBraketConfiguration) return BraketConfiguration;
    hasRequiredBraketConfiguration = 1;
    var O;
    Object.defineProperty(BraketConfiguration, "__esModule", { value: !0 }), BraketConfiguration.BraketConfiguration = void 0;
    var _ = requireConfiguration(), H = requireBraketItems();
    return requireBraketMappings(), BraketConfiguration.BraketConfiguration = _.Configuration.create("braket", {
      handler: {
        character: ["Braket-characters"],
        macro: ["Braket-macros"]
      },
      items: (O = {}, O[H.BraketItem.prototype.kind] = H.BraketItem, O)
    }), BraketConfiguration;
  }
  var BussproofsConfiguration = {}, BussproofsItems = {}, BussproofsUtil = {}, hasRequiredBussproofsUtil;
  function requireBussproofsUtil() {
    return hasRequiredBussproofsUtil || (hasRequiredBussproofsUtil = 1, function(O) {
      var _ = BussproofsUtil.__read || function(De, Pe) {
        var $e = typeof Symbol == "function" && De[Symbol.iterator];
        if (!$e) return De;
        var qe = $e.call(De), Xe, Ve = [], Ze;
        try {
          for (; (Pe === void 0 || Pe-- > 0) && !(Xe = qe.next()).done; ) Ve.push(Xe.value);
        } catch (Ge) {
          Ze = { error: Ge };
        } finally {
          try {
            Xe && !Xe.done && ($e = qe.return) && $e.call(qe);
          } finally {
            if (Ze) throw Ze.error;
          }
        }
        return Ve;
      }, H = BussproofsUtil.__values || function(De) {
        var Pe = typeof Symbol == "function" && Symbol.iterator, $e = Pe && De[Pe], qe = 0;
        if ($e) return $e.call(De);
        if (De && typeof De.length == "number") return {
          next: function() {
            return De && qe >= De.length && (De = void 0), { value: De && De[qe++], done: !De };
          }
        };
        throw new TypeError(Pe ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, V = BussproofsUtil.__importDefault || function(De) {
        return De && De.__esModule ? De : { default: De };
      }, q;
      Object.defineProperty(O, "__esModule", { value: !0 }), O.clearDocument = O.saveDocument = O.makeBsprAttributes = O.removeProperty = O.getProperty = O.setProperty = O.balanceRules = void 0;
      var G = V(requireNodeUtil()), J = V(requireParseUtil()), Z = null, te = null, ne = function(De) {
        te.root = De;
        var Pe = Z.outputJax.getBBox(te, Z).w;
        return Pe;
      }, ee = function(De) {
        for (var Pe = 0; De && !G.default.isType(De, "mtable"); ) {
          if (G.default.isType(De, "text"))
            return null;
          if (G.default.isType(De, "mrow")) {
            De = De.childNodes[0], Pe = 0;
            continue;
          }
          De = De.parent.childNodes[Pe], Pe++;
        }
        return De;
      }, ae = function(De, Pe) {
        return De.childNodes[Pe === "up" ? 1 : 0].childNodes[0].childNodes[0].childNodes[0].childNodes[0];
      }, oe = function(De, Pe) {
        return De.childNodes[Pe].childNodes[0].childNodes[0];
      }, se = function(De) {
        return oe(De, 0);
      }, ce = function(De) {
        return oe(De, De.childNodes.length - 1);
      }, le = function(De, Pe) {
        return De.childNodes[Pe === "up" ? 0 : 1].childNodes[0].childNodes[0].childNodes[0];
      }, ue = function(De) {
        for (; De && !G.default.isType(De, "mtd"); )
          De = De.parent;
        return De;
      }, de = function(De) {
        return De.parent.childNodes[De.parent.childNodes.indexOf(De) + 1];
      }, fe = function(De) {
        for (; De && (0, O.getProperty)(De, "inference") == null; )
          De = De.parent;
        return De;
      }, he = function(De, Pe, $e) {
        $e === void 0 && ($e = !1);
        var qe = 0;
        if (De === Pe)
          return qe;
        if (De !== Pe.parent) {
          var Xe = De.childNodes, Ve = $e ? Xe.length - 1 : 0;
          G.default.isType(Xe[Ve], "mspace") && (qe += ne(Xe[Ve])), De = Pe.parent;
        }
        if (De === Pe)
          return qe;
        var Ze = De.childNodes, Ge = $e ? Ze.length - 1 : 0;
        return Ze[Ge] !== Pe && (qe += ne(Ze[Ge])), qe;
      }, me = function(De, Pe) {
        Pe === void 0 && (Pe = !1);
        var $e = ee(De), qe = le($e, (0, O.getProperty)($e, "inferenceRule")), Xe = he(De, $e, Pe), Ve = ne($e), Ze = ne(qe);
        return Xe + (Ve - Ze) / 2;
      }, pe = function(De, Pe, $e, qe) {
        if (qe === void 0 && (qe = !1), (0, O.getProperty)(Pe, "inferenceRule") || (0, O.getProperty)(Pe, "labelledRule")) {
          var Xe = De.nodeFactory.create("node", "mrow");
          Pe.parent.replaceChild(Xe, Pe), Xe.setChildren([Pe]), ye(Pe, Xe), Pe = Xe;
        }
        var Ve = qe ? Pe.childNodes.length - 1 : 0, Ze = Pe.childNodes[Ve];
        if (G.default.isType(Ze, "mspace")) {
          G.default.setAttribute(Ze, "width", J.default.Em(J.default.dimen2em(G.default.getAttribute(Ze, "width")) + $e));
          return;
        }
        if (Ze = De.nodeFactory.create("node", "mspace", [], { width: J.default.Em($e) }), qe) {
          Pe.appendChild(Ze);
          return;
        }
        Ze.parent = Pe, Pe.childNodes.unshift(Ze);
      }, ye = function(De, Pe) {
        var $e = ["inference", "proof", "maxAdjust", "labelledRule"];
        $e.forEach(function(qe) {
          var Xe = (0, O.getProperty)(De, qe);
          Xe != null && ((0, O.setProperty)(Pe, qe, Xe), (0, O.removeProperty)(De, qe));
        });
      }, ge = function(De) {
        var Pe = De.nodeLists.sequent;
        if (Pe)
          for (var $e = Pe.length - 1, qe = void 0; qe = Pe[$e]; $e--) {
            if ((0, O.getProperty)(qe, "sequentProcessed")) {
              (0, O.removeProperty)(qe, "sequentProcessed");
              continue;
            }
            var Xe = [], Ve = fe(qe);
            if ((0, O.getProperty)(Ve, "inference") === 1) {
              for (Xe.push(qe); (0, O.getProperty)(Ve, "inference") === 1; ) {
                Ve = ee(Ve);
                var Ze = se(ae(Ve, (0, O.getProperty)(Ve, "inferenceRule"))), Ge = (0, O.getProperty)(Ze, "inferenceRule") ? le(Ze, (0, O.getProperty)(Ze, "inferenceRule")) : Ze;
                (0, O.getProperty)(Ge, "sequent") && (qe = Ge.childNodes[0], Xe.push(qe), (0, O.setProperty)(qe, "sequentProcessed", !0)), Ve = Ze;
              }
              be(De, Xe);
            }
          }
      }, ve = function(De, Pe, $e, qe, Xe) {
        var Ve = De.nodeFactory.create("node", "mspace", [], { width: J.default.Em(Xe) });
        if (qe === "left") {
          var Ze = Pe.childNodes[$e].childNodes[0];
          Ve.parent = Ze, Ze.childNodes.unshift(Ve);
        } else
          Pe.childNodes[$e].appendChild(Ve);
        (0, O.setProperty)(Pe.parent, "sequentAdjust_" + qe, Xe);
      }, be = function(De, Pe) {
        for (var $e = Pe.pop(); Pe.length; ) {
          var qe = Pe.pop(), Xe = _(Ce($e, qe), 2), Ve = Xe[0], Ze = Xe[1];
          (0, O.getProperty)($e.parent, "axiom") && (ve(De, Ve < 0 ? $e : qe, 0, "left", Math.abs(Ve)), ve(De, Ze < 0 ? $e : qe, 2, "right", Math.abs(Ze))), $e = qe;
        }
      }, Ce = function(De, Pe) {
        var $e = ne(De.childNodes[2]), qe = ne(Pe.childNodes[2]), Xe = ne(De.childNodes[0]), Ve = ne(Pe.childNodes[0]), Ze = Xe - Ve, Ge = $e - qe;
        return [Ze, Ge];
      }, Se = function(De) {
        var Pe, $e;
        te = new De.document.options.MathItem("", null, De.math.display);
        var qe = De.data;
        ge(qe);
        var Xe = qe.nodeLists.inference || [];
        try {
          for (var Ve = H(Xe), Ze = Ve.next(); !Ze.done; Ze = Ve.next()) {
            var Ge = Ze.value, ze = (0, O.getProperty)(Ge, "proof"), We = ee(Ge), at = ae(We, (0, O.getProperty)(We, "inferenceRule")), ct = se(at);
            if ((0, O.getProperty)(ct, "inference")) {
              var et = me(ct);
              if (et) {
                pe(qe, ct, -et);
                var dt = he(Ge, We, !1);
                pe(qe, Ge, et - dt);
              }
            }
            var Je = ce(at);
            if ((0, O.getProperty)(Je, "inference") != null) {
              var it = me(Je, !0);
              pe(qe, Je, -it, !0);
              var st = he(Ge, We, !0), gt = (0, O.getProperty)(Ge, "maxAdjust");
              gt != null && (it = Math.max(it, gt));
              var bt = void 0;
              if (ze || !(bt = ue(Ge))) {
                pe(qe, (0, O.getProperty)(Ge, "proof") ? Ge : Ge.parent, it - st, !0);
                continue;
              }
              var yt = de(bt);
              if (yt) {
                var St = qe.nodeFactory.create("node", "mspace", [], { width: it - st + "em" });
                yt.appendChild(St), Ge.removeProperty("maxAdjust");
                continue;
              }
              var Lt = fe(bt);
              Lt && (it = (0, O.getProperty)(Lt, "maxAdjust") ? Math.max((0, O.getProperty)(Lt, "maxAdjust"), it) : it, (0, O.setProperty)(Lt, "maxAdjust", it));
            }
          }
        } catch (Rt) {
          Pe = { error: Rt };
        } finally {
          try {
            Ze && !Ze.done && ($e = Ve.return) && $e.call(Ve);
          } finally {
            if (Pe) throw Pe.error;
          }
        }
      };
      O.balanceRules = Se;
      var we = "bspr_", Ee = (q = {}, q[we + "maxAdjust"] = !0, q), xe = function(De, Pe, $e) {
        G.default.setProperty(De, we + Pe, $e);
      };
      O.setProperty = xe;
      var Le = function(De, Pe) {
        return G.default.getProperty(De, we + Pe);
      };
      O.getProperty = Le;
      var Qe = function(De, Pe) {
        De.removeProperty(we + Pe);
      };
      O.removeProperty = Qe;
      var ke = function(De) {
        De.data.root.walkTree(function(Pe, $e) {
          var qe = [];
          Pe.getPropertyNames().forEach(function(Xe) {
            !Ee[Xe] && Xe.match(RegExp("^" + we)) && qe.push(Xe + ":" + Pe.getProperty(Xe));
          }), qe.length && G.default.setAttribute(Pe, "semantics", qe.join(";"));
        });
      };
      O.makeBsprAttributes = ke;
      var Me = function(De) {
        if (Z = De.document, !("getBBox" in Z.outputJax))
          throw Error("The bussproofs extension requires an output jax with a getBBox() method");
      };
      O.saveDocument = Me;
      var Re = function(De) {
        Z = null;
      };
      O.clearDocument = Re;
    }(BussproofsUtil)), BussproofsUtil;
  }
  var hasRequiredBussproofsItems;
  function requireBussproofsItems() {
    if (hasRequiredBussproofsItems) return BussproofsItems;
    hasRequiredBussproofsItems = 1;
    var O = BussproofsItems.__extends || /* @__PURE__ */ function() {
      var ee = function(ae, oe) {
        return ee = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(se, ce) {
          se.__proto__ = ce;
        } || function(se, ce) {
          for (var le in ce) Object.prototype.hasOwnProperty.call(ce, le) && (se[le] = ce[le]);
        }, ee(ae, oe);
      };
      return function(ae, oe) {
        if (typeof oe != "function" && oe !== null)
          throw new TypeError("Class extends value " + String(oe) + " is not a constructor or null");
        ee(ae, oe);
        function se() {
          this.constructor = ae;
        }
        ae.prototype = oe === null ? Object.create(oe) : (se.prototype = oe.prototype, new se());
      };
    }(), _ = BussproofsItems.__createBinding || (Object.create ? function(ee, ae, oe, se) {
      se === void 0 && (se = oe);
      var ce = Object.getOwnPropertyDescriptor(ae, oe);
      (!ce || ("get" in ce ? !ae.__esModule : ce.writable || ce.configurable)) && (ce = { enumerable: !0, get: function() {
        return ae[oe];
      } }), Object.defineProperty(ee, se, ce);
    } : function(ee, ae, oe, se) {
      se === void 0 && (se = oe), ee[se] = ae[oe];
    }), H = BussproofsItems.__setModuleDefault || (Object.create ? function(ee, ae) {
      Object.defineProperty(ee, "default", { enumerable: !0, value: ae });
    } : function(ee, ae) {
      ee.default = ae;
    }), V = BussproofsItems.__importStar || function(ee) {
      if (ee && ee.__esModule) return ee;
      var ae = {};
      if (ee != null) for (var oe in ee) oe !== "default" && Object.prototype.hasOwnProperty.call(ee, oe) && _(ae, ee, oe);
      return H(ae, ee), ae;
    }, q = BussproofsItems.__importDefault || function(ee) {
      return ee && ee.__esModule ? ee : { default: ee };
    };
    Object.defineProperty(BussproofsItems, "__esModule", { value: !0 }), BussproofsItems.ProofTreeItem = void 0;
    var G = q(requireTexError()), J = requireStackItem(), Z = q(requireStack()), te = V(requireBussproofsUtil()), ne = function(ee) {
      O(ae, ee);
      function ae() {
        var oe = ee !== null && ee.apply(this, arguments) || this;
        return oe.leftLabel = null, oe.rigthLabel = null, oe.innerStack = new Z.default(oe.factory, {}, !0), oe;
      }
      return Object.defineProperty(ae.prototype, "kind", {
        get: function() {
          return "proofTree";
        },
        enumerable: !1,
        configurable: !0
      }), ae.prototype.checkItem = function(oe) {
        if (oe.isKind("end") && oe.getName() === "prooftree") {
          var se = this.toMml();
          return te.setProperty(se, "proof", !0), [[this.factory.create("mml", se), oe], !0];
        }
        if (oe.isKind("stop"))
          throw new G.default("EnvMissingEnd", "Missing \\end{%1}", this.getName());
        return this.innerStack.Push(oe), J.BaseItem.fail;
      }, ae.prototype.toMml = function() {
        var oe = ee.prototype.toMml.call(this), se = this.innerStack.Top();
        if (se.isKind("start") && !se.Size())
          return oe;
        this.innerStack.Push(this.factory.create("stop"));
        var ce = this.innerStack.Top().toMml();
        return this.create("node", "mrow", [ce, oe], {});
      }, ae;
    }(J.BaseItem);
    return BussproofsItems.ProofTreeItem = ne, BussproofsItems;
  }
  var BussproofsMappings = {}, BussproofsMethods = {}, hasRequiredBussproofsMethods;
  function requireBussproofsMethods() {
    if (hasRequiredBussproofsMethods) return BussproofsMethods;
    hasRequiredBussproofsMethods = 1;
    var O = BussproofsMethods.__createBinding || (Object.create ? function(ce, le, ue, de) {
      de === void 0 && (de = ue);
      var fe = Object.getOwnPropertyDescriptor(le, ue);
      (!fe || ("get" in fe ? !le.__esModule : fe.writable || fe.configurable)) && (fe = { enumerable: !0, get: function() {
        return le[ue];
      } }), Object.defineProperty(ce, de, fe);
    } : function(ce, le, ue, de) {
      de === void 0 && (de = ue), ce[de] = le[ue];
    }), _ = BussproofsMethods.__setModuleDefault || (Object.create ? function(ce, le) {
      Object.defineProperty(ce, "default", { enumerable: !0, value: le });
    } : function(ce, le) {
      ce.default = le;
    }), H = BussproofsMethods.__importStar || function(ce) {
      if (ce && ce.__esModule) return ce;
      var le = {};
      if (ce != null) for (var ue in ce) ue !== "default" && Object.prototype.hasOwnProperty.call(ce, ue) && O(le, ce, ue);
      return _(le, ce), le;
    }, V = BussproofsMethods.__read || function(ce, le) {
      var ue = typeof Symbol == "function" && ce[Symbol.iterator];
      if (!ue) return ce;
      var de = ue.call(ce), fe, he = [], me;
      try {
        for (; (le === void 0 || le-- > 0) && !(fe = de.next()).done; ) he.push(fe.value);
      } catch (pe) {
        me = { error: pe };
      } finally {
        try {
          fe && !fe.done && (ue = de.return) && ue.call(de);
        } finally {
          if (me) throw me.error;
        }
      }
      return he;
    }, q = BussproofsMethods.__spreadArray || function(ce, le, ue) {
      if (ue || arguments.length === 2) for (var de = 0, fe = le.length, he; de < fe; de++)
        (he || !(de in le)) && (he || (he = Array.prototype.slice.call(le, 0, de)), he[de] = le[de]);
      return ce.concat(he || Array.prototype.slice.call(le));
    }, G = BussproofsMethods.__importDefault || function(ce) {
      return ce && ce.__esModule ? ce : { default: ce };
    };
    Object.defineProperty(BussproofsMethods, "__esModule", { value: !0 });
    var J = G(requireTexError()), Z = G(requireTexParser()), te = G(requireParseUtil()), ne = H(requireBussproofsUtil()), ee = {};
    ee.Prooftree = function(ce, le) {
      ce.Push(le);
      var ue = ce.itemFactory.create("proofTree").setProperties({
        name: le.getName(),
        line: "solid",
        currentLine: "solid",
        rootAtTop: !1
      });
      return ue;
    }, ee.Axiom = function(ce, le) {
      var ue = ce.stack.Top();
      if (ue.kind !== "proofTree")
        throw new J.default("IllegalProofCommand", "Proof commands only allowed in prooftree environment.");
      var de = ae(ce, ce.GetArgument(le));
      ne.setProperty(de, "axiom", !0), ue.Push(de);
    };
    var ae = function(ce, le) {
      var ue = te.default.internalMath(ce, te.default.trimSpaces(le), 0);
      if (!ue[0].childNodes[0].childNodes.length)
        return ce.create("node", "mrow", []);
      var de = ce.create("node", "mspace", [], { width: ".5ex" }), fe = ce.create("node", "mspace", [], { width: ".5ex" });
      return ce.create("node", "mrow", q(q([de], V(ue), !1), [fe], !1));
    };
    ee.Inference = function(ce, le, ue) {
      var de = ce.stack.Top();
      if (de.kind !== "proofTree")
        throw new J.default("IllegalProofCommand", "Proof commands only allowed in prooftree environment.");
      if (de.Size() < ue)
        throw new J.default("BadProofTree", "Proof tree badly specified.");
      var fe = de.getProperty("rootAtTop"), he = ue === 1 && !de.Peek()[0].childNodes.length ? 0 : ue, me = [];
      do
        me.length && me.unshift(ce.create("node", "mtd", [], {})), me.unshift(ce.create("node", "mtd", [de.Pop()], { rowalign: fe ? "top" : "bottom" })), ue--;
      while (ue > 0);
      var pe = ce.create("node", "mtr", me, {}), ye = ce.create("node", "mtable", [pe], { framespacing: "0 0" }), ge = ae(ce, ce.GetArgument(le)), ve = de.getProperty("currentLine");
      ve !== de.getProperty("line") && de.setProperty("currentLine", de.getProperty("line"));
      var be = oe(ce, ye, [ge], de.getProperty("left"), de.getProperty("right"), ve, fe);
      de.setProperty("left", null), de.setProperty("right", null), ne.setProperty(be, "inference", he), ce.configuration.addNode("inference", be), de.Push(be);
    };
    function oe(ce, le, ue, de, fe, he, me) {
      var pe = ce.create("node", "mtr", [ce.create("node", "mtd", [le], {})], {}), ye = ce.create("node", "mtr", [ce.create("node", "mtd", ue, {})], {}), ge = ce.create("node", "mtable", me ? [ye, pe] : [pe, ye], { align: "top 2", rowlines: he, framespacing: "0 0" });
      ne.setProperty(ge, "inferenceRule", me ? "up" : "down");
      var ve, be;
      de && (ve = ce.create("node", "mpadded", [de], { height: "+.5em", width: "+.5em", voffset: "-.15em" }), ne.setProperty(ve, "prooflabel", "left")), fe && (be = ce.create("node", "mpadded", [fe], { height: "+.5em", width: "+.5em", voffset: "-.15em" }), ne.setProperty(be, "prooflabel", "right"));
      var Ce, Se;
      if (de && fe)
        Ce = [ve, ge, be], Se = "both";
      else if (de)
        Ce = [ve, ge], Se = "left";
      else if (fe)
        Ce = [ge, be], Se = "right";
      else
        return ge;
      return ge = ce.create("node", "mrow", Ce), ne.setProperty(ge, "labelledRule", Se), ge;
    }
    ee.Label = function(ce, le, ue) {
      var de = ce.stack.Top();
      if (de.kind !== "proofTree")
        throw new J.default("IllegalProofCommand", "Proof commands only allowed in prooftree environment.");
      var fe = te.default.internalMath(ce, ce.GetArgument(le), 0), he = fe.length > 1 ? ce.create("node", "mrow", fe, {}) : fe[0];
      de.setProperty(ue, he);
    }, ee.SetLine = function(ce, le, ue, de) {
      var fe = ce.stack.Top();
      if (fe.kind !== "proofTree")
        throw new J.default("IllegalProofCommand", "Proof commands only allowed in prooftree environment.");
      fe.setProperty("currentLine", ue), de && fe.setProperty("line", ue);
    }, ee.RootAtTop = function(ce, le, ue) {
      var de = ce.stack.Top();
      if (de.kind !== "proofTree")
        throw new J.default("IllegalProofCommand", "Proof commands only allowed in prooftree environment.");
      de.setProperty("rootAtTop", ue);
    }, ee.AxiomF = function(ce, le) {
      var ue = ce.stack.Top();
      if (ue.kind !== "proofTree")
        throw new J.default("IllegalProofCommand", "Proof commands only allowed in prooftree environment.");
      var de = se(ce, le);
      ne.setProperty(de, "axiom", !0), ue.Push(de);
    };
    function se(ce, le) {
      var ue = ce.GetNext();
      if (ue !== "$")
        throw new J.default("IllegalUseOfCommand", "Use of %1 does not match it's definition.", le);
      ce.i++;
      var de = ce.GetUpTo(le, "$");
      if (de.indexOf("\\fCenter") === -1)
        throw new J.default("IllegalUseOfCommand", "Missing \\fCenter in %1.", le);
      var fe = V(de.split("\\fCenter"), 2), he = fe[0], me = fe[1], pe = new Z.default(he, ce.stack.env, ce.configuration).mml(), ye = new Z.default(me, ce.stack.env, ce.configuration).mml(), ge = new Z.default("\\fCenter", ce.stack.env, ce.configuration).mml(), ve = ce.create("node", "mtd", [pe], {}), be = ce.create("node", "mtd", [ge], {}), Ce = ce.create("node", "mtd", [ye], {}), Se = ce.create("node", "mtr", [ve, be, Ce], {}), we = ce.create("node", "mtable", [Se], { columnspacing: ".5ex", columnalign: "center 2" });
      return ne.setProperty(we, "sequent", !0), ce.configuration.addNode("sequent", Se), we;
    }
    return ee.FCenter = function(ce, le) {
    }, ee.InferenceF = function(ce, le, ue) {
      var de = ce.stack.Top();
      if (de.kind !== "proofTree")
        throw new J.default("IllegalProofCommand", "Proof commands only allowed in prooftree environment.");
      if (de.Size() < ue)
        throw new J.default("BadProofTree", "Proof tree badly specified.");
      var fe = de.getProperty("rootAtTop"), he = ue === 1 && !de.Peek()[0].childNodes.length ? 0 : ue, me = [];
      do
        me.length && me.unshift(ce.create("node", "mtd", [], {})), me.unshift(ce.create("node", "mtd", [de.Pop()], { rowalign: fe ? "top" : "bottom" })), ue--;
      while (ue > 0);
      var pe = ce.create("node", "mtr", me, {}), ye = ce.create("node", "mtable", [pe], { framespacing: "0 0" }), ge = se(ce, le), ve = de.getProperty("currentLine");
      ve !== de.getProperty("line") && de.setProperty("currentLine", de.getProperty("line"));
      var be = oe(ce, ye, [ge], de.getProperty("left"), de.getProperty("right"), ve, fe);
      de.setProperty("left", null), de.setProperty("right", null), ne.setProperty(be, "inference", he), ce.configuration.addNode("inference", be), de.Push(be);
    }, BussproofsMethods.default = ee, BussproofsMethods;
  }
  var hasRequiredBussproofsMappings;
  function requireBussproofsMappings() {
    if (hasRequiredBussproofsMappings) return BussproofsMappings;
    hasRequiredBussproofsMappings = 1;
    var O = BussproofsMappings.__importDefault || function(q) {
      return q && q.__esModule ? q : { default: q };
    };
    Object.defineProperty(BussproofsMappings, "__esModule", { value: !0 });
    var _ = O(requireBussproofsMethods()), H = O(requireParseMethods()), V = requireSymbolMap();
    return new V.CommandMap("Bussproofs-macros", {
      AxiomC: "Axiom",
      UnaryInfC: ["Inference", 1],
      BinaryInfC: ["Inference", 2],
      TrinaryInfC: ["Inference", 3],
      QuaternaryInfC: ["Inference", 4],
      QuinaryInfC: ["Inference", 5],
      RightLabel: ["Label", "right"],
      LeftLabel: ["Label", "left"],
      AXC: "Axiom",
      UIC: ["Inference", 1],
      BIC: ["Inference", 2],
      TIC: ["Inference", 3],
      RL: ["Label", "right"],
      LL: ["Label", "left"],
      noLine: ["SetLine", "none", !1],
      singleLine: ["SetLine", "solid", !1],
      solidLine: ["SetLine", "solid", !1],
      dashedLine: ["SetLine", "dashed", !1],
      alwaysNoLine: ["SetLine", "none", !0],
      alwaysSingleLine: ["SetLine", "solid", !0],
      alwaysSolidLine: ["SetLine", "solid", !0],
      alwaysDashedLine: ["SetLine", "dashed", !0],
      rootAtTop: ["RootAtTop", !0],
      alwaysRootAtTop: ["RootAtTop", !0],
      rootAtBottom: ["RootAtTop", !1],
      alwaysRootAtBottom: ["RootAtTop", !1],
      fCenter: "FCenter",
      Axiom: "AxiomF",
      UnaryInf: ["InferenceF", 1],
      BinaryInf: ["InferenceF", 2],
      TrinaryInf: ["InferenceF", 3],
      QuaternaryInf: ["InferenceF", 4],
      QuinaryInf: ["InferenceF", 5]
    }, _.default), new V.EnvironmentMap("Bussproofs-environments", H.default.environment, {
      prooftree: ["Prooftree", null, !1]
    }, _.default), BussproofsMappings;
  }
  var hasRequiredBussproofsConfiguration;
  function requireBussproofsConfiguration() {
    if (hasRequiredBussproofsConfiguration) return BussproofsConfiguration;
    hasRequiredBussproofsConfiguration = 1;
    var O;
    Object.defineProperty(BussproofsConfiguration, "__esModule", { value: !0 }), BussproofsConfiguration.BussproofsConfiguration = void 0;
    var _ = requireConfiguration(), H = requireBussproofsItems(), V = requireBussproofsUtil();
    return requireBussproofsMappings(), BussproofsConfiguration.BussproofsConfiguration = _.Configuration.create("bussproofs", {
      handler: {
        macro: ["Bussproofs-macros"],
        environment: ["Bussproofs-environments"]
      },
      items: (O = {}, O[H.ProofTreeItem.prototype.kind] = H.ProofTreeItem, O),
      preprocessors: [
        [V.saveDocument, 1]
      ],
      postprocessors: [
        [V.clearDocument, 3],
        [V.makeBsprAttributes, 2],
        [V.balanceRules, 1]
      ]
    }), BussproofsConfiguration;
  }
  var CancelConfiguration = {}, EncloseConfiguration = {}, hasRequiredEncloseConfiguration;
  function requireEncloseConfiguration() {
    return hasRequiredEncloseConfiguration || (hasRequiredEncloseConfiguration = 1, function(O) {
      var _ = EncloseConfiguration.__importDefault || function(G) {
        return G && G.__esModule ? G : { default: G };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.EncloseConfiguration = O.EncloseMethods = O.ENCLOSE_OPTIONS = void 0;
      var H = requireConfiguration(), V = requireSymbolMap(), q = _(requireParseUtil());
      O.ENCLOSE_OPTIONS = {
        "data-arrowhead": 1,
        color: 1,
        mathcolor: 1,
        background: 1,
        mathbackground: 1,
        "data-padding": 1,
        "data-thickness": 1
      }, O.EncloseMethods = {}, O.EncloseMethods.Enclose = function(G, J) {
        var Z = G.GetArgument(J).replace(/,/g, " "), te = G.GetBrackets(J, ""), ne = G.ParseArg(J), ee = q.default.keyvalOptions(te, O.ENCLOSE_OPTIONS);
        ee.notation = Z, G.Push(G.create("node", "menclose", [ne], ee));
      }, new V.CommandMap("enclose", { enclose: "Enclose" }, O.EncloseMethods), O.EncloseConfiguration = H.Configuration.create("enclose", { handler: { macro: ["enclose"] } });
    }(EncloseConfiguration)), EncloseConfiguration;
  }
  var hasRequiredCancelConfiguration;
  function requireCancelConfiguration() {
    return hasRequiredCancelConfiguration || (hasRequiredCancelConfiguration = 1, function(O) {
      var _ = CancelConfiguration.__importDefault || function(Z) {
        return Z && Z.__esModule ? Z : { default: Z };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.CancelConfiguration = O.CancelMethods = void 0;
      var H = requireConfiguration(), V = requireTexConstants(), q = requireSymbolMap(), G = _(requireParseUtil()), J = requireEncloseConfiguration();
      O.CancelMethods = {}, O.CancelMethods.Cancel = function(Z, te, ne) {
        var ee = Z.GetBrackets(te, ""), ae = Z.ParseArg(te), oe = G.default.keyvalOptions(ee, J.ENCLOSE_OPTIONS);
        oe.notation = ne, Z.Push(Z.create("node", "menclose", [ae], oe));
      }, O.CancelMethods.CancelTo = function(Z, te) {
        var ne = Z.GetBrackets(te, ""), ee = Z.ParseArg(te), ae = Z.ParseArg(te), oe = G.default.keyvalOptions(ne, J.ENCLOSE_OPTIONS);
        oe.notation = [
          V.TexConstant.Notation.UPDIAGONALSTRIKE,
          V.TexConstant.Notation.UPDIAGONALARROW,
          V.TexConstant.Notation.NORTHEASTARROW
        ].join(" "), ee = Z.create("node", "mpadded", [ee], { depth: "-.1em", height: "+.1em", voffset: ".1em" }), Z.Push(Z.create("node", "msup", [Z.create("node", "menclose", [ae], oe), ee]));
      }, new q.CommandMap("cancel", {
        cancel: ["Cancel", V.TexConstant.Notation.UPDIAGONALSTRIKE],
        bcancel: ["Cancel", V.TexConstant.Notation.DOWNDIAGONALSTRIKE],
        xcancel: ["Cancel", V.TexConstant.Notation.UPDIAGONALSTRIKE + " " + V.TexConstant.Notation.DOWNDIAGONALSTRIKE],
        cancelto: "CancelTo"
      }, O.CancelMethods), O.CancelConfiguration = H.Configuration.create("cancel", { handler: { macro: ["cancel"] } });
    }(CancelConfiguration)), CancelConfiguration;
  }
  var CasesConfiguration = {}, EmpheqUtil = {}, hasRequiredEmpheqUtil;
  function requireEmpheqUtil() {
    return hasRequiredEmpheqUtil || (hasRequiredEmpheqUtil = 1, function(O) {
      var _ = EmpheqUtil.__read || function(Z, te) {
        var ne = typeof Symbol == "function" && Z[Symbol.iterator];
        if (!ne) return Z;
        var ee = ne.call(Z), ae, oe = [], se;
        try {
          for (; (te === void 0 || te-- > 0) && !(ae = ee.next()).done; ) oe.push(ae.value);
        } catch (ce) {
          se = { error: ce };
        } finally {
          try {
            ae && !ae.done && (ne = ee.return) && ne.call(ee);
          } finally {
            if (se) throw se.error;
          }
        }
        return oe;
      }, H = EmpheqUtil.__spreadArray || function(Z, te, ne) {
        if (ne || arguments.length === 2) for (var ee = 0, ae = te.length, oe; ee < ae; ee++)
          (oe || !(ee in te)) && (oe || (oe = Array.prototype.slice.call(te, 0, ee)), oe[ee] = te[ee]);
        return Z.concat(oe || Array.prototype.slice.call(te));
      }, V = EmpheqUtil.__values || function(Z) {
        var te = typeof Symbol == "function" && Symbol.iterator, ne = te && Z[te], ee = 0;
        if (ne) return ne.call(Z);
        if (Z && typeof Z.length == "number") return {
          next: function() {
            return Z && ee >= Z.length && (Z = void 0), { value: Z && Z[ee++], done: !Z };
          }
        };
        throw new TypeError(te ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, q = EmpheqUtil.__importDefault || function(Z) {
        return Z && Z.__esModule ? Z : { default: Z };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.EmpheqUtil = void 0;
      var G = q(requireParseUtil()), J = q(requireTexParser());
      O.EmpheqUtil = {
        environment: function(Z, te, ne, ee) {
          var ae = ee[0], oe = Z.itemFactory.create(ae + "-begin").setProperties({ name: te, end: ae });
          Z.Push(ne.apply(void 0, H([Z, oe], _(ee.slice(1)), !1)));
        },
        splitOptions: function(Z, te) {
          return te === void 0 && (te = null), G.default.keyvalOptions(Z, te, !0);
        },
        columnCount: function(Z) {
          var te, ne, ee = 0;
          try {
            for (var ae = V(Z.childNodes), oe = ae.next(); !oe.done; oe = ae.next()) {
              var se = oe.value, ce = se.childNodes.length - (se.isKind("mlabeledtr") ? 1 : 0);
              ce > ee && (ee = ce);
            }
          } catch (le) {
            te = { error: le };
          } finally {
            try {
              oe && !oe.done && (ne = ae.return) && ne.call(ae);
            } finally {
              if (te) throw te.error;
            }
          }
          return ee;
        },
        cellBlock: function(Z, te, ne, ee) {
          var ae, oe, se = ne.create("node", "mpadded", [], { height: 0, depth: 0, voffset: "-1height" }), ce = new J.default(Z, ne.stack.env, ne.configuration), le = ce.mml();
          ee && ce.configuration.tags.label && (ce.configuration.tags.currentTag.env = ee, ce.configuration.tags.getTag(!0));
          try {
            for (var ue = V(le.isInferred ? le.childNodes : [le]), de = ue.next(); !de.done; de = ue.next()) {
              var fe = de.value;
              se.appendChild(fe);
            }
          } catch (he) {
            ae = { error: he };
          } finally {
            try {
              de && !de.done && (oe = ue.return) && oe.call(ue);
            } finally {
              if (ae) throw ae.error;
            }
          }
          return se.appendChild(ne.create("node", "mphantom", [
            ne.create("node", "mpadded", [te], { width: 0 })
          ])), se;
        },
        topRowTable: function(Z, te) {
          var ne = G.default.copyNode(Z, te);
          return ne.setChildren(ne.childNodes.slice(0, 1)), ne.attributes.set("align", "baseline 1"), Z.factory.create("mphantom", {}, [te.create("node", "mpadded", [ne], { width: 0 })]);
        },
        rowspanCell: function(Z, te, ne, ee, ae) {
          Z.appendChild(ee.create("node", "mpadded", [
            this.cellBlock(te, G.default.copyNode(ne, ee), ee, ae),
            this.topRowTable(ne, ee)
          ], { height: 0, depth: 0, voffset: "height" }));
        },
        left: function(Z, te, ne, ee, ae) {
          var oe, se;
          ae === void 0 && (ae = ""), Z.attributes.set("columnalign", "right " + (Z.attributes.get("columnalign") || "")), Z.attributes.set("columnspacing", "0em " + (Z.attributes.get("columnspacing") || ""));
          var ce;
          try {
            for (var le = V(Z.childNodes.slice(0).reverse()), ue = le.next(); !ue.done; ue = le.next()) {
              var de = ue.value;
              ce = ee.create("node", "mtd"), de.childNodes.unshift(ce), ce.parent = de, de.isKind("mlabeledtr") && (de.childNodes[0] = de.childNodes[1], de.childNodes[1] = ce);
            }
          } catch (fe) {
            oe = { error: fe };
          } finally {
            try {
              ue && !ue.done && (se = le.return) && se.call(le);
            } finally {
              if (oe) throw oe.error;
            }
          }
          this.rowspanCell(ce, ne, te, ee, ae);
        },
        right: function(Z, te, ne, ee, ae) {
          ae === void 0 && (ae = ""), Z.childNodes.length === 0 && Z.appendChild(ee.create("node", "mtr"));
          for (var oe = O.EmpheqUtil.columnCount(Z), se = Z.childNodes[0]; se.childNodes.length < oe; )
            se.appendChild(ee.create("node", "mtd"));
          var ce = se.appendChild(ee.create("node", "mtd"));
          O.EmpheqUtil.rowspanCell(ce, ne, te, ee, ae), Z.attributes.set("columnalign", (Z.attributes.get("columnalign") || "").split(/ /).slice(0, oe).join(" ") + " left"), Z.attributes.set("columnspacing", (Z.attributes.get("columnspacing") || "").split(/ /).slice(0, oe - 1).join(" ") + " 0em");
        },
        adjustTable: function(Z, te) {
          var ne = Z.getProperty("left"), ee = Z.getProperty("right");
          if (ne || ee) {
            var ae = Z.Last, oe = G.default.copyNode(ae, te);
            ne && this.left(ae, oe, ne, te), ee && this.right(ae, oe, ee, te);
          }
        },
        allowEnv: {
          equation: !0,
          align: !0,
          gather: !0,
          flalign: !0,
          alignat: !0,
          multline: !0
        },
        checkEnv: function(Z) {
          return this.allowEnv.hasOwnProperty(Z.replace(/\*$/, "")) || !1;
        }
      };
    }(EmpheqUtil)), EmpheqUtil;
  }
  var hasRequiredCasesConfiguration;
  function requireCasesConfiguration() {
    return hasRequiredCasesConfiguration || (hasRequiredCasesConfiguration = 1, function(O) {
      var _ = CasesConfiguration.__extends || /* @__PURE__ */ function() {
        var ce = function(le, ue) {
          return ce = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(de, fe) {
            de.__proto__ = fe;
          } || function(de, fe) {
            for (var he in fe) Object.prototype.hasOwnProperty.call(fe, he) && (de[he] = fe[he]);
          }, ce(le, ue);
        };
        return function(le, ue) {
          if (typeof ue != "function" && ue !== null)
            throw new TypeError("Class extends value " + String(ue) + " is not a constructor or null");
          ce(le, ue);
          function de() {
            this.constructor = le;
          }
          le.prototype = ue === null ? Object.create(ue) : (de.prototype = ue.prototype, new de());
        };
      }(), H = CasesConfiguration.__importDefault || function(ce) {
        return ce && ce.__esModule ? ce : { default: ce };
      }, V;
      Object.defineProperty(O, "__esModule", { value: !0 }), O.CasesConfiguration = O.CasesMethods = O.CasesTags = O.CasesBeginItem = void 0;
      var q = requireConfiguration(), G = requireSymbolMap(), J = H(requireParseUtil()), Z = H(requireBaseMethods()), te = H(requireTexError()), ne = requireBaseItems(), ee = requireAmsConfiguration(), ae = requireEmpheqUtil(), oe = function(ce) {
        _(le, ce);
        function le() {
          return ce !== null && ce.apply(this, arguments) || this;
        }
        return Object.defineProperty(le.prototype, "kind", {
          get: function() {
            return "cases-begin";
          },
          enumerable: !1,
          configurable: !0
        }), le.prototype.checkItem = function(ue) {
          return ue.isKind("end") && ue.getName() === this.getName() && this.getProperty("end") ? (this.setProperty("end", !1), [[], !0]) : ce.prototype.checkItem.call(this, ue);
        }, le;
      }(ne.BeginItem);
      O.CasesBeginItem = oe;
      var se = function(ce) {
        _(le, ce);
        function le() {
          var ue = ce !== null && ce.apply(this, arguments) || this;
          return ue.subcounter = 0, ue;
        }
        return le.prototype.start = function(ue, de, fe) {
          this.subcounter = 0, ce.prototype.start.call(this, ue, de, fe);
        }, le.prototype.autoTag = function() {
          this.currentTag.tag == null && (this.currentTag.env === "subnumcases" ? (this.subcounter === 0 && this.counter++, this.subcounter++, this.tag(this.formatNumber(this.counter, this.subcounter), !1)) : ((this.subcounter === 0 || this.currentTag.env !== "numcases-left") && this.counter++, this.tag(this.formatNumber(this.counter), !1)));
        }, le.prototype.formatNumber = function(ue, de) {
          return de === void 0 && (de = null), ue.toString() + (de === null ? "" : String.fromCharCode(96 + de));
        }, le;
      }(ee.AmsTags);
      O.CasesTags = se, O.CasesMethods = {
        NumCases: function(ce, le) {
          if (ce.stack.env.closing === le.getName()) {
            delete ce.stack.env.closing, ce.Push(ce.itemFactory.create("end").setProperty("name", le.getName()));
            var ue = ce.stack.Top(), de = ue.Last, fe = J.default.copyNode(de, ce), he = ue.getProperty("left");
            return ae.EmpheqUtil.left(de, fe, he + "\\empheqlbrace\\,", ce, "numcases-left"), ce.Push(ce.itemFactory.create("end").setProperty("name", le.getName())), null;
          } else {
            var he = ce.GetArgument("\\begin{" + le.getName() + "}");
            le.setProperty("left", he);
            var me = Z.default.EqnArray(ce, le, !0, !0, "ll");
            return me.arraydef.displaystyle = !1, me.arraydef.rowspacing = ".2em", me.setProperty("numCases", !0), ce.Push(le), me;
          }
        },
        Entry: function(ce, le) {
          if (!ce.stack.Top().getProperty("numCases"))
            return Z.default.Entry(ce, le);
          ce.Push(ce.itemFactory.create("cell").setProperties({ isEntry: !0, name: le }));
          for (var ue = ce.string, de = 0, fe = ce.i, he = ue.length; fe < he; ) {
            var me = ue.charAt(fe);
            if (me === "{")
              de++, fe++;
            else if (me === "}") {
              if (de === 0)
                break;
              de--, fe++;
            } else {
              if (me === "&" && de === 0)
                throw new te.default("ExtraCasesAlignTab", "Extra alignment tab in text for numcase environment");
              if (me === "\\" && de === 0) {
                var pe = (ue.slice(fe + 1).match(/^[a-z]+|./i) || [])[0];
                if (pe === "\\" || pe === "cr" || pe === "end" || pe === "label")
                  break;
                fe += pe.length;
              } else
                fe++;
            }
          }
          var ye = ue.substr(ce.i, fe - ce.i).replace(/^\s*/, "");
          ce.PushAll(J.default.internalMath(ce, ye, 0)), ce.i = fe;
        }
      }, new G.EnvironmentMap("cases-env", ae.EmpheqUtil.environment, {
        numcases: ["NumCases", "cases"],
        subnumcases: ["NumCases", "cases"]
      }, O.CasesMethods), new G.MacroMap("cases-macros", {
        "&": "Entry"
      }, O.CasesMethods), O.CasesConfiguration = q.Configuration.create("cases", {
        handler: {
          environment: ["cases-env"],
          character: ["cases-macros"]
        },
        items: (V = {}, V[oe.prototype.kind] = oe, V),
        tags: { cases: se }
      });
    }(CasesConfiguration)), CasesConfiguration;
  }
  var CenternotConfiguration = {}, hasRequiredCenternotConfiguration;
  function requireCenternotConfiguration() {
    if (hasRequiredCenternotConfiguration) return CenternotConfiguration;
    hasRequiredCenternotConfiguration = 1;
    var O = CenternotConfiguration.__values || function(te) {
      var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
      if (ee) return ee.call(te);
      if (te && typeof te.length == "number") return {
        next: function() {
          return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
        }
      };
      throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = CenternotConfiguration.__importDefault || function(te) {
      return te && te.__esModule ? te : { default: te };
    };
    Object.defineProperty(CenternotConfiguration, "__esModule", { value: !0 }), CenternotConfiguration.CenternotConfiguration = CenternotConfiguration.filterCenterOver = void 0;
    var H = requireConfiguration(), V = _(requireTexParser()), q = _(requireNodeUtil()), G = requireSymbolMap(), J = _(requireBaseMethods());
    new G.CommandMap("centernot", {
      centerOver: "CenterOver",
      centernot: ["Macro", "\\centerOver{#1}{{⧸}}", 1]
    }, {
      CenterOver: function(te, ne) {
        var ee = "{" + te.GetArgument(ne) + "}", ae = te.ParseArg(ne), oe = new V.default(ee, te.stack.env, te.configuration).mml(), se = te.create("node", "TeXAtom", [
          new V.default(ee, te.stack.env, te.configuration).mml(),
          te.create("node", "mpadded", [
            te.create("node", "mpadded", [ae], { width: 0, lspace: "-.5width" }),
            te.create("node", "mphantom", [oe])
          ], { width: 0, lspace: "-.5width" })
        ]);
        te.configuration.addNode("centerOver", oe), te.Push(se);
      },
      Macro: J.default.Macro
    });
    function Z(te) {
      var ne, ee, ae = te.data;
      try {
        for (var oe = O(ae.getList("centerOver")), se = oe.next(); !se.done; se = oe.next()) {
          var ce = se.value, le = q.default.getTexClass(ce.childNodes[0].childNodes[0]);
          le !== null && q.default.setProperties(ce.parent.parent.parent.parent.parent.parent, { texClass: le });
        }
      } catch (ue) {
        ne = { error: ue };
      } finally {
        try {
          se && !se.done && (ee = oe.return) && ee.call(oe);
        } finally {
          if (ne) throw ne.error;
        }
      }
    }
    return CenternotConfiguration.filterCenterOver = Z, CenternotConfiguration.CenternotConfiguration = H.Configuration.create("centernot", {
      handler: { macro: ["centernot"] },
      postprocessors: [Z]
    }), CenternotConfiguration;
  }
  var ColorConfiguration = {}, ColorMethods = {}, hasRequiredColorMethods;
  function requireColorMethods() {
    return hasRequiredColorMethods || (hasRequiredColorMethods = 1, function(O) {
      var _ = ColorMethods.__importDefault || function(G) {
        return G && G.__esModule ? G : { default: G };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.ColorMethods = void 0;
      var H = _(requireNodeUtil()), V = _(requireParseUtil());
      function q(G) {
        var J = "+".concat(G), Z = G.replace(/^.*?([a-z]*)$/, "$1"), te = 2 * parseFloat(J);
        return {
          width: "+".concat(te).concat(Z),
          height: J,
          depth: J,
          lspace: G
        };
      }
      O.ColorMethods = {}, O.ColorMethods.Color = function(G, J) {
        var Z = G.GetBrackets(J, ""), te = G.GetArgument(J), ne = G.configuration.packageData.get("color").model, ee = ne.getColor(Z, te), ae = G.itemFactory.create("style").setProperties({ styles: { mathcolor: ee } });
        G.stack.env.color = ee, G.Push(ae);
      }, O.ColorMethods.TextColor = function(G, J) {
        var Z = G.GetBrackets(J, ""), te = G.GetArgument(J), ne = G.configuration.packageData.get("color").model, ee = ne.getColor(Z, te), ae = G.stack.env.color;
        G.stack.env.color = ee;
        var oe = G.ParseArg(J);
        ae ? G.stack.env.color = ae : delete G.stack.env.color;
        var se = G.create("node", "mstyle", [oe], { mathcolor: ee });
        G.Push(se);
      }, O.ColorMethods.DefineColor = function(G, J) {
        var Z = G.GetArgument(J), te = G.GetArgument(J), ne = G.GetArgument(J), ee = G.configuration.packageData.get("color").model;
        ee.defineColor(te, Z, ne);
      }, O.ColorMethods.ColorBox = function(G, J) {
        var Z = G.GetArgument(J), te = V.default.internalMath(G, G.GetArgument(J)), ne = G.configuration.packageData.get("color").model, ee = G.create("node", "mpadded", te, {
          mathbackground: ne.getColor("named", Z)
        });
        H.default.setProperties(ee, q(G.options.color.padding)), G.Push(ee);
      }, O.ColorMethods.FColorBox = function(G, J) {
        var Z = G.GetArgument(J), te = G.GetArgument(J), ne = V.default.internalMath(G, G.GetArgument(J)), ee = G.options.color, ae = G.configuration.packageData.get("color").model, oe = G.create("node", "mpadded", ne, {
          mathbackground: ae.getColor("named", te),
          style: "border: ".concat(ee.borderWidth, " solid ").concat(ae.getColor("named", Z))
        });
        H.default.setProperties(oe, q(ee.padding)), G.Push(oe);
      };
    }(ColorMethods)), ColorMethods;
  }
  var ColorUtil = {}, ColorConstants = {}, hasRequiredColorConstants;
  function requireColorConstants() {
    return hasRequiredColorConstants || (hasRequiredColorConstants = 1, Object.defineProperty(ColorConstants, "__esModule", { value: !0 }), ColorConstants.COLORS = void 0, ColorConstants.COLORS = /* @__PURE__ */ new Map([
      ["Apricot", "#FBB982"],
      ["Aquamarine", "#00B5BE"],
      ["Bittersweet", "#C04F17"],
      ["Black", "#221E1F"],
      ["Blue", "#2D2F92"],
      ["BlueGreen", "#00B3B8"],
      ["BlueViolet", "#473992"],
      ["BrickRed", "#B6321C"],
      ["Brown", "#792500"],
      ["BurntOrange", "#F7921D"],
      ["CadetBlue", "#74729A"],
      ["CarnationPink", "#F282B4"],
      ["Cerulean", "#00A2E3"],
      ["CornflowerBlue", "#41B0E4"],
      ["Cyan", "#00AEEF"],
      ["Dandelion", "#FDBC42"],
      ["DarkOrchid", "#A4538A"],
      ["Emerald", "#00A99D"],
      ["ForestGreen", "#009B55"],
      ["Fuchsia", "#8C368C"],
      ["Goldenrod", "#FFDF42"],
      ["Gray", "#949698"],
      ["Green", "#00A64F"],
      ["GreenYellow", "#DFE674"],
      ["JungleGreen", "#00A99A"],
      ["Lavender", "#F49EC4"],
      ["LimeGreen", "#8DC73E"],
      ["Magenta", "#EC008C"],
      ["Mahogany", "#A9341F"],
      ["Maroon", "#AF3235"],
      ["Melon", "#F89E7B"],
      ["MidnightBlue", "#006795"],
      ["Mulberry", "#A93C93"],
      ["NavyBlue", "#006EB8"],
      ["OliveGreen", "#3C8031"],
      ["Orange", "#F58137"],
      ["OrangeRed", "#ED135A"],
      ["Orchid", "#AF72B0"],
      ["Peach", "#F7965A"],
      ["Periwinkle", "#7977B8"],
      ["PineGreen", "#008B72"],
      ["Plum", "#92268F"],
      ["ProcessBlue", "#00B0F0"],
      ["Purple", "#99479B"],
      ["RawSienna", "#974006"],
      ["Red", "#ED1B23"],
      ["RedOrange", "#F26035"],
      ["RedViolet", "#A1246B"],
      ["Rhodamine", "#EF559F"],
      ["RoyalBlue", "#0071BC"],
      ["RoyalPurple", "#613F99"],
      ["RubineRed", "#ED017D"],
      ["Salmon", "#F69289"],
      ["SeaGreen", "#3FBC9D"],
      ["Sepia", "#671800"],
      ["SkyBlue", "#46C5DD"],
      ["SpringGreen", "#C6DC67"],
      ["Tan", "#DA9D76"],
      ["TealBlue", "#00AEB3"],
      ["Thistle", "#D883B7"],
      ["Turquoise", "#00B4CE"],
      ["Violet", "#58429B"],
      ["VioletRed", "#EF58A0"],
      ["White", "#FFFFFF"],
      ["WildStrawberry", "#EE2967"],
      ["Yellow", "#FFF200"],
      ["YellowGreen", "#98CC70"],
      ["YellowOrange", "#FAA21A"]
    ])), ColorConstants;
  }
  var hasRequiredColorUtil;
  function requireColorUtil() {
    if (hasRequiredColorUtil) return ColorUtil;
    hasRequiredColorUtil = 1;
    var O = ColorUtil.__values || function(J) {
      var Z = typeof Symbol == "function" && Symbol.iterator, te = Z && J[Z], ne = 0;
      if (te) return te.call(J);
      if (J && typeof J.length == "number") return {
        next: function() {
          return J && ne >= J.length && (J = void 0), { value: J && J[ne++], done: !J };
        }
      };
      throw new TypeError(Z ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = ColorUtil.__importDefault || function(J) {
      return J && J.__esModule ? J : { default: J };
    };
    Object.defineProperty(ColorUtil, "__esModule", { value: !0 }), ColorUtil.ColorModel = void 0;
    var H = _(requireTexError()), V = requireColorConstants(), q = /* @__PURE__ */ new Map(), G = function() {
      function J() {
        this.userColors = /* @__PURE__ */ new Map();
      }
      return J.prototype.normalizeColor = function(Z, te) {
        if (!Z || Z === "named")
          return te;
        if (q.has(Z)) {
          var ne = q.get(Z);
          return ne(te);
        }
        throw new H.default("UndefinedColorModel", "Color model '%1' not defined", Z);
      }, J.prototype.getColor = function(Z, te) {
        return !Z || Z === "named" ? this.getColorByName(te) : this.normalizeColor(Z, te);
      }, J.prototype.getColorByName = function(Z) {
        return this.userColors.has(Z) ? this.userColors.get(Z) : V.COLORS.has(Z) ? V.COLORS.get(Z) : Z;
      }, J.prototype.defineColor = function(Z, te, ne) {
        var ee = this.normalizeColor(Z, ne);
        this.userColors.set(te, ee);
      }, J;
    }();
    return ColorUtil.ColorModel = G, q.set("rgb", function(J) {
      var Z, te, ne = J.trim().split(/\s*,\s*/), ee = "#";
      if (ne.length !== 3)
        throw new H.default("ModelArg1", "Color values for the %1 model require 3 numbers", "rgb");
      try {
        for (var ae = O(ne), oe = ae.next(); !oe.done; oe = ae.next()) {
          var se = oe.value;
          if (!se.match(/^(\d+(\.\d*)?|\.\d+)$/))
            throw new H.default("InvalidDecimalNumber", "Invalid decimal number");
          var ce = parseFloat(se);
          if (ce < 0 || ce > 1)
            throw new H.default("ModelArg2", "Color values for the %1 model must be between %2 and %3", "rgb", "0", "1");
          var le = Math.floor(ce * 255).toString(16);
          le.length < 2 && (le = "0" + le), ee += le;
        }
      } catch (ue) {
        Z = { error: ue };
      } finally {
        try {
          oe && !oe.done && (te = ae.return) && te.call(ae);
        } finally {
          if (Z) throw Z.error;
        }
      }
      return ee;
    }), q.set("RGB", function(J) {
      var Z, te, ne = J.trim().split(/\s*,\s*/), ee = "#";
      if (ne.length !== 3)
        throw new H.default("ModelArg1", "Color values for the %1 model require 3 numbers", "RGB");
      try {
        for (var ae = O(ne), oe = ae.next(); !oe.done; oe = ae.next()) {
          var se = oe.value;
          if (!se.match(/^\d+$/))
            throw new H.default("InvalidNumber", "Invalid number");
          var ce = parseInt(se);
          if (ce > 255)
            throw new H.default("ModelArg2", "Color values for the %1 model must be between %2 and %3", "RGB", "0", "255");
          var le = ce.toString(16);
          le.length < 2 && (le = "0" + le), ee += le;
        }
      } catch (ue) {
        Z = { error: ue };
      } finally {
        try {
          oe && !oe.done && (te = ae.return) && te.call(ae);
        } finally {
          if (Z) throw Z.error;
        }
      }
      return ee;
    }), q.set("gray", function(J) {
      if (!J.match(/^\s*(\d+(\.\d*)?|\.\d+)\s*$/))
        throw new H.default("InvalidDecimalNumber", "Invalid decimal number");
      var Z = parseFloat(J);
      if (Z < 0 || Z > 1)
        throw new H.default("ModelArg2", "Color values for the %1 model must be between %2 and %3", "gray", "0", "1");
      var te = Math.floor(Z * 255).toString(16);
      return te.length < 2 && (te = "0" + te), "#".concat(te).concat(te).concat(te);
    }), ColorUtil;
  }
  var hasRequiredColorConfiguration;
  function requireColorConfiguration() {
    if (hasRequiredColorConfiguration) return ColorConfiguration;
    hasRequiredColorConfiguration = 1, Object.defineProperty(ColorConfiguration, "__esModule", { value: !0 }), ColorConfiguration.ColorConfiguration = void 0;
    var O = requireSymbolMap(), _ = requireConfiguration(), H = requireColorMethods(), V = requireColorUtil();
    new O.CommandMap("color", {
      color: "Color",
      textcolor: "TextColor",
      definecolor: "DefineColor",
      colorbox: "ColorBox",
      fcolorbox: "FColorBox"
    }, H.ColorMethods);
    var q = function(G, J) {
      J.parseOptions.packageData.set("color", { model: new V.ColorModel() });
    };
    return ColorConfiguration.ColorConfiguration = _.Configuration.create("color", {
      handler: {
        macro: ["color"]
      },
      options: {
        color: {
          padding: "5px",
          borderWidth: "2px"
        }
      },
      config: q
    }), ColorConfiguration;
  }
  var ColorV2Configuration = {}, hasRequiredColorV2Configuration;
  function requireColorV2Configuration() {
    return hasRequiredColorV2Configuration || (hasRequiredColorV2Configuration = 1, function(O) {
      Object.defineProperty(O, "__esModule", { value: !0 }), O.ColorConfiguration = O.ColorV2Methods = void 0;
      var _ = requireSymbolMap(), H = requireConfiguration();
      O.ColorV2Methods = {
        Color: function(V, q) {
          var G = V.GetArgument(q), J = V.stack.env.color;
          V.stack.env.color = G;
          var Z = V.ParseArg(q);
          J ? V.stack.env.color = J : delete V.stack.env.color;
          var te = V.create("node", "mstyle", [Z], { mathcolor: G });
          V.Push(te);
        }
      }, new _.CommandMap("colorv2", { color: "Color" }, O.ColorV2Methods), O.ColorConfiguration = H.Configuration.create("colorv2", { handler: { macro: ["colorv2"] } });
    }(ColorV2Configuration)), ColorV2Configuration;
  }
  var ColortblConfiguration = {}, hasRequiredColortblConfiguration;
  function requireColortblConfiguration() {
    if (hasRequiredColortblConfiguration) return ColortblConfiguration;
    hasRequiredColortblConfiguration = 1;
    var O = ColortblConfiguration.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = ColortblConfiguration.__importDefault || function(te) {
      return te && te.__esModule ? te : { default: te };
    };
    Object.defineProperty(ColortblConfiguration, "__esModule", { value: !0 }), ColortblConfiguration.ColortblConfiguration = ColortblConfiguration.ColorArrayItem = void 0;
    var H = requireBaseItems(), V = requireConfiguration(), q = requireSymbolMap(), G = _(requireTexError()), J = function(te) {
      O(ne, te);
      function ne() {
        var ee = te !== null && te.apply(this, arguments) || this;
        return ee.color = {
          cell: "",
          row: "",
          col: []
        }, ee.hasColor = !1, ee;
      }
      return ne.prototype.EndEntry = function() {
        te.prototype.EndEntry.call(this);
        var ee = this.row[this.row.length - 1], ae = this.color.cell || this.color.row || this.color.col[this.row.length - 1];
        ae && (ee.attributes.set("mathbackground", ae), this.color.cell = "", this.hasColor = !0);
      }, ne.prototype.EndRow = function() {
        te.prototype.EndRow.call(this), this.color.row = "";
      }, ne.prototype.createMml = function() {
        var ee = te.prototype.createMml.call(this), ae = ee.isKind("mrow") ? ee.childNodes[1] : ee;
        return ae.isKind("menclose") && (ae = ae.childNodes[0].childNodes[0]), this.hasColor && ae.attributes.get("frame") === "none" && ae.attributes.set("frame", ""), ee;
      }, ne;
    }(H.ArrayItem);
    ColortblConfiguration.ColorArrayItem = J, new q.CommandMap("colortbl", {
      cellcolor: ["TableColor", "cell"],
      rowcolor: ["TableColor", "row"],
      columncolor: ["TableColor", "col"]
    }, {
      TableColor: function(te, ne, ee) {
        var ae = te.configuration.packageData.get("color").model, oe = te.GetBrackets(ne, ""), se = ae.getColor(oe, te.GetArgument(ne)), ce = te.stack.Top();
        if (!(ce instanceof J))
          throw new G.default("UnsupportedTableColor", "Unsupported use of %1", te.currentCS);
        if (ee === "col") {
          if (ce.table.length)
            throw new G.default("ColumnColorNotTop", "%1 must be in the top row", ne);
          ce.color.col[ce.row.length] = se, te.GetBrackets(ne, "") && te.GetBrackets(ne, "");
        } else if (ce.color[ee] = se, ee === "row" && (ce.Size() || ce.row.length))
          throw new G.default("RowColorNotFirst", "%1 must be at the beginning of a row", ne);
      }
    });
    var Z = function(te, ne) {
      ne.parseOptions.packageData.has("color") || V.ConfigurationHandler.get("color").config(te, ne);
    };
    return ColortblConfiguration.ColortblConfiguration = V.Configuration.create("colortbl", {
      handler: { macro: ["colortbl"] },
      items: { array: J },
      priority: 10,
      config: [Z, 10]
    }), ColortblConfiguration;
  }
  var ConfigMacrosConfiguration = {}, NewcommandMethods = {}, NewcommandUtil = {}, hasRequiredNewcommandUtil;
  function requireNewcommandUtil() {
    if (hasRequiredNewcommandUtil) return NewcommandUtil;
    hasRequiredNewcommandUtil = 1;
    var O = NewcommandUtil.__importDefault || function(G) {
      return G && G.__esModule ? G : { default: G };
    };
    Object.defineProperty(NewcommandUtil, "__esModule", { value: !0 });
    var _ = O(requireParseUtil()), H = O(requireTexError()), V = require_Symbol(), q;
    return function(G) {
      function J(de, fe) {
        var he = [de, fe.char];
        if (fe.attributes)
          for (var me in fe.attributes)
            he.push(me), he.push(fe.attributes[me]);
        return he;
      }
      G.disassembleSymbol = J;
      function Z(de) {
        for (var fe = de[0], he = de[1], me = {}, pe = 2; pe < de.length; pe = pe + 2)
          me[de[pe]] = de[pe + 1];
        return new V.Symbol(fe, he, me);
      }
      G.assembleSymbol = Z;
      function te(de, fe) {
        var he = de.GetNext();
        if (he !== "\\")
          throw new H.default("MissingCS", "%1 must be followed by a control sequence", fe);
        var me = _.default.trimSpaces(de.GetArgument(fe));
        return me.substr(1);
      }
      G.GetCSname = te;
      function ne(de, fe) {
        var he = _.default.trimSpaces(de.GetArgument(fe));
        if (he.charAt(0) === "\\" && (he = he.substr(1)), !he.match(/^(.|[a-z]+)$/i))
          throw new H.default("IllegalControlSequenceName", "Illegal control sequence name for %1", fe);
        return he;
      }
      G.GetCsNameArgument = ne;
      function ee(de, fe) {
        var he = de.GetBrackets(fe);
        if (he && (he = _.default.trimSpaces(he), !he.match(/^[0-9]+$/)))
          throw new H.default("IllegalParamNumber", "Illegal number of parameters specified in %1", fe);
        return he;
      }
      G.GetArgCount = ee;
      function ae(de, fe, he) {
        for (var me = de.GetNext(), pe = [], ye = 0, ge = de.i; de.i < de.string.length; ) {
          if (me = de.GetNext(), me === "#") {
            if (ge !== de.i && (pe[ye] = de.string.substr(ge, de.i - ge)), me = de.string.charAt(++de.i), !me.match(/^[1-9]$/))
              throw new H.default("CantUseHash2", "Illegal use of # in template for %1", he);
            if (parseInt(me) !== ++ye)
              throw new H.default("SequentialParam", "Parameters for %1 must be numbered sequentially", he);
            ge = de.i + 1;
          } else if (me === "{")
            return ge !== de.i && (pe[ye] = de.string.substr(ge, de.i - ge)), pe.length > 0 ? [ye.toString()].concat(pe) : ye;
          de.i++;
        }
        throw new H.default("MissingReplacementString", "Missing replacement string for definition of %1", fe);
      }
      G.GetTemplate = ae;
      function oe(de, fe, he) {
        if (he == null)
          return de.GetArgument(fe);
        for (var me = de.i, pe = 0, ye = 0; de.i < de.string.length; ) {
          var ge = de.string.charAt(de.i);
          if (ge === "{")
            de.i === me && (ye = 1), de.GetArgument(fe), pe = de.i - me;
          else {
            if (se(de, he))
              return ye && (me++, pe -= 2), de.string.substr(me, pe);
            if (ge === "\\") {
              de.i++, pe++, ye = 0;
              var ve = de.string.substr(de.i).match(/[a-z]+|./i);
              ve && (de.i += ve[0].length, pe = de.i - me);
            } else
              de.i++, pe++, ye = 0;
          }
        }
        throw new H.default("RunawayArgument", "Runaway argument for %1?", fe);
      }
      G.GetParameter = oe;
      function se(de, fe) {
        return de.string.substr(de.i, fe.length) !== fe || fe.match(/\\[a-z]+$/i) && de.string.charAt(de.i + fe.length).match(/[a-z]/i) ? 0 : (de.i += fe.length, 1);
      }
      G.MatchParam = se;
      function ce(de, fe, he, me) {
        var pe = de.configuration.handlers, ye = pe.retrieve(G.NEW_DELIMITER);
        ye.add(fe, new V.Symbol(fe, he, me));
      }
      G.addDelimiter = ce;
      function le(de, fe, he, me, pe) {
        pe === void 0 && (pe = "");
        var ye = de.configuration.handlers, ge = ye.retrieve(G.NEW_COMMAND);
        ge.add(fe, new V.Macro(pe || fe, he, me));
      }
      G.addMacro = le;
      function ue(de, fe, he, me) {
        var pe = de.configuration.handlers, ye = pe.retrieve(G.NEW_ENVIRONMENT);
        ye.add(fe, new V.Macro(fe, he, me));
      }
      G.addEnvironment = ue, G.NEW_DELIMITER = "new-Delimiter", G.NEW_COMMAND = "new-Command", G.NEW_ENVIRONMENT = "new-Environment";
    }(q || (q = {})), NewcommandUtil.default = q, NewcommandUtil;
  }
  var hasRequiredNewcommandMethods;
  function requireNewcommandMethods() {
    if (hasRequiredNewcommandMethods) return NewcommandMethods;
    hasRequiredNewcommandMethods = 1;
    var O = NewcommandMethods.__createBinding || (Object.create ? function(ee, ae, oe, se) {
      se === void 0 && (se = oe);
      var ce = Object.getOwnPropertyDescriptor(ae, oe);
      (!ce || ("get" in ce ? !ae.__esModule : ce.writable || ce.configurable)) && (ce = { enumerable: !0, get: function() {
        return ae[oe];
      } }), Object.defineProperty(ee, se, ce);
    } : function(ee, ae, oe, se) {
      se === void 0 && (se = oe), ee[se] = ae[oe];
    }), _ = NewcommandMethods.__setModuleDefault || (Object.create ? function(ee, ae) {
      Object.defineProperty(ee, "default", { enumerable: !0, value: ae });
    } : function(ee, ae) {
      ee.default = ae;
    }), H = NewcommandMethods.__importStar || function(ee) {
      if (ee && ee.__esModule) return ee;
      var ae = {};
      if (ee != null) for (var oe in ee) oe !== "default" && Object.prototype.hasOwnProperty.call(ee, oe) && O(ae, ee, oe);
      return _(ae, ee), ae;
    }, V = NewcommandMethods.__importDefault || function(ee) {
      return ee && ee.__esModule ? ee : { default: ee };
    };
    Object.defineProperty(NewcommandMethods, "__esModule", { value: !0 });
    var q = V(requireTexError()), G = H(requireSymbolMap()), J = V(requireBaseMethods()), Z = V(requireParseUtil()), te = V(requireNewcommandUtil()), ne = {};
    return ne.NewCommand = function(ee, ae) {
      var oe = te.default.GetCsNameArgument(ee, ae), se = te.default.GetArgCount(ee, ae), ce = ee.GetBrackets(ae), le = ee.GetArgument(ae);
      te.default.addMacro(ee, oe, ne.Macro, [le, se, ce]);
    }, ne.NewEnvironment = function(ee, ae) {
      var oe = Z.default.trimSpaces(ee.GetArgument(ae)), se = te.default.GetArgCount(ee, ae), ce = ee.GetBrackets(ae), le = ee.GetArgument(ae), ue = ee.GetArgument(ae);
      te.default.addEnvironment(ee, oe, ne.BeginEnv, [!0, le, ue, se, ce]);
    }, ne.MacroDef = function(ee, ae) {
      var oe = te.default.GetCSname(ee, ae), se = te.default.GetTemplate(ee, ae, "\\" + oe), ce = ee.GetArgument(ae);
      se instanceof Array ? te.default.addMacro(ee, oe, ne.MacroWithTemplate, [ce].concat(se)) : te.default.addMacro(ee, oe, ne.Macro, [ce, se]);
    }, ne.Let = function(ee, ae) {
      var oe = te.default.GetCSname(ee, ae), se = ee.GetNext();
      se === "=" && (ee.i++, se = ee.GetNext());
      var ce = ee.configuration.handlers;
      if (se === "\\") {
        ae = te.default.GetCSname(ee, ae);
        var le = ce.get("delimiter").lookup("\\" + ae);
        if (le) {
          te.default.addDelimiter(ee, "\\" + oe, le.char, le.attributes);
          return;
        }
        var ue = ce.get("macro").applicable(ae);
        if (!ue)
          return;
        if (ue instanceof G.MacroMap) {
          var de = ue.lookup(ae);
          te.default.addMacro(ee, oe, de.func, de.args, de.symbol);
          return;
        }
        le = ue.lookup(ae);
        var fe = te.default.disassembleSymbol(oe, le), he = function(pe, ye) {
          for (var ge = [], ve = 2; ve < arguments.length; ve++)
            ge[ve - 2] = arguments[ve];
          var be = te.default.assembleSymbol(ge);
          return ue.parser(pe, be);
        };
        te.default.addMacro(ee, oe, he, fe);
        return;
      }
      ee.i++;
      var me = ce.get("delimiter").lookup(se);
      if (me) {
        te.default.addDelimiter(ee, "\\" + oe, me.char, me.attributes);
        return;
      }
      te.default.addMacro(ee, oe, ne.Macro, [se]);
    }, ne.MacroWithTemplate = function(ee, ae, oe, se) {
      for (var ce = [], le = 4; le < arguments.length; le++)
        ce[le - 4] = arguments[le];
      var ue = parseInt(se, 10);
      if (ue) {
        var de = [];
        if (ee.GetNext(), ce[0] && !te.default.MatchParam(ee, ce[0]))
          throw new q.default("MismatchUseDef", "Use of %1 doesn't match its definition", ae);
        for (var fe = 0; fe < ue; fe++)
          de.push(te.default.GetParameter(ee, ae, ce[fe + 1]));
        oe = Z.default.substituteArgs(ee, de, oe);
      }
      ee.string = Z.default.addArgs(ee, oe, ee.string.slice(ee.i)), ee.i = 0, Z.default.checkMaxMacros(ee);
    }, ne.BeginEnv = function(ee, ae, oe, se, ce, le) {
      if (ae.getProperty("end") && ee.stack.env.closing === ae.getName()) {
        delete ee.stack.env.closing;
        var ue = ee.string.slice(ee.i);
        return ee.string = se, ee.i = 0, ee.Parse(), ee.string = ue, ee.i = 0, ee.itemFactory.create("end").setProperty("name", ae.getName());
      }
      if (ce) {
        var de = [];
        if (le != null) {
          var fe = ee.GetBrackets("\\begin{" + ae.getName() + "}");
          de.push(fe ?? le);
        }
        for (var he = de.length; he < ce; he++)
          de.push(ee.GetArgument("\\begin{" + ae.getName() + "}"));
        oe = Z.default.substituteArgs(ee, de, oe), se = Z.default.substituteArgs(ee, [], se);
      }
      return ee.string = Z.default.addArgs(ee, oe, ee.string.slice(ee.i)), ee.i = 0, ee.itemFactory.create("beginEnv").setProperty("name", ae.getName());
    }, ne.Macro = J.default.Macro, NewcommandMethods.default = ne, NewcommandMethods;
  }
  var NewcommandItems = {}, hasRequiredNewcommandItems;
  function requireNewcommandItems() {
    if (hasRequiredNewcommandItems) return NewcommandItems;
    hasRequiredNewcommandItems = 1;
    var O = NewcommandItems.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }(), _ = NewcommandItems.__importDefault || function(G) {
      return G && G.__esModule ? G : { default: G };
    };
    Object.defineProperty(NewcommandItems, "__esModule", { value: !0 }), NewcommandItems.BeginEnvItem = void 0;
    var H = _(requireTexError()), V = requireStackItem(), q = function(G) {
      O(J, G);
      function J() {
        return G !== null && G.apply(this, arguments) || this;
      }
      return Object.defineProperty(J.prototype, "kind", {
        get: function() {
          return "beginEnv";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(J.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), J.prototype.checkItem = function(Z) {
        if (Z.isKind("end")) {
          if (Z.getName() !== this.getName())
            throw new H.default("EnvBadEnd", "\\begin{%1} ended with \\end{%2}", this.getName(), Z.getName());
          return [[this.factory.create("mml", this.toMml())], !0];
        }
        if (Z.isKind("stop"))
          throw new H.default("EnvMissingEnd", "Missing \\end{%1}", this.getName());
        return G.prototype.checkItem.call(this, Z);
      }, J;
    }(V.BaseItem);
    return NewcommandItems.BeginEnvItem = q, NewcommandItems;
  }
  var hasRequiredConfigMacrosConfiguration;
  function requireConfigMacrosConfiguration() {
    if (hasRequiredConfigMacrosConfiguration) return ConfigMacrosConfiguration;
    hasRequiredConfigMacrosConfiguration = 1;
    var O = ConfigMacrosConfiguration.__values || function(ue) {
      var de = typeof Symbol == "function" && Symbol.iterator, fe = de && ue[de], he = 0;
      if (fe) return fe.call(ue);
      if (ue && typeof ue.length == "number") return {
        next: function() {
          return ue && he >= ue.length && (ue = void 0), { value: ue && ue[he++], done: !ue };
        }
      };
      throw new TypeError(de ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, _ = ConfigMacrosConfiguration.__importDefault || function(ue) {
      return ue && ue.__esModule ? ue : { default: ue };
    }, H;
    Object.defineProperty(ConfigMacrosConfiguration, "__esModule", { value: !0 }), ConfigMacrosConfiguration.ConfigMacrosConfiguration = void 0;
    var V = requireConfiguration(), q = requireOptions(), G = requireSymbolMap(), J = _(requireParseMethods()), Z = require_Symbol(), te = _(requireNewcommandMethods()), ne = requireNewcommandItems(), ee = "configmacros-map", ae = "configmacros-env-map";
    function oe(ue) {
      new G.CommandMap(ee, {}, {}), new G.EnvironmentMap(ae, J.default.environment, {}, {}), ue.append(V.Configuration.local({
        handler: {
          macro: [ee],
          environment: [ae]
        },
        priority: 3
      }));
    }
    function se(ue, de) {
      ce(de), le(de);
    }
    function ce(ue) {
      var de, fe, he = ue.parseOptions.handlers.retrieve(ee), me = ue.parseOptions.options.macros;
      try {
        for (var pe = O(Object.keys(me)), ye = pe.next(); !ye.done; ye = pe.next()) {
          var ge = ye.value, ve = typeof me[ge] == "string" ? [me[ge]] : me[ge], be = Array.isArray(ve[2]) ? new Z.Macro(ge, te.default.MacroWithTemplate, ve.slice(0, 2).concat(ve[2])) : new Z.Macro(ge, te.default.Macro, ve);
          he.add(ge, be);
        }
      } catch (Ce) {
        de = { error: Ce };
      } finally {
        try {
          ye && !ye.done && (fe = pe.return) && fe.call(pe);
        } finally {
          if (de) throw de.error;
        }
      }
    }
    function le(ue) {
      var de, fe, he = ue.parseOptions.handlers.retrieve(ae), me = ue.parseOptions.options.environments;
      try {
        for (var pe = O(Object.keys(me)), ye = pe.next(); !ye.done; ye = pe.next()) {
          var ge = ye.value;
          he.add(ge, new Z.Macro(ge, te.default.BeginEnv, [!0].concat(me[ge])));
        }
      } catch (ve) {
        de = { error: ve };
      } finally {
        try {
          ye && !ye.done && (fe = pe.return) && fe.call(pe);
        } finally {
          if (de) throw de.error;
        }
      }
    }
    return ConfigMacrosConfiguration.ConfigMacrosConfiguration = V.Configuration.create("configmacros", {
      init: oe,
      config: se,
      items: (H = {}, H[ne.BeginEnvItem.prototype.kind] = ne.BeginEnvItem, H),
      options: {
        macros: (0, q.expandable)({}),
        environments: (0, q.expandable)({})
      }
    }), ConfigMacrosConfiguration;
  }
  var EmpheqConfiguration = {}, hasRequiredEmpheqConfiguration;
  function requireEmpheqConfiguration() {
    return hasRequiredEmpheqConfiguration || (hasRequiredEmpheqConfiguration = 1, function(O) {
      var _ = EmpheqConfiguration.__extends || /* @__PURE__ */ function() {
        var oe = function(se, ce) {
          return oe = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(le, ue) {
            le.__proto__ = ue;
          } || function(le, ue) {
            for (var de in ue) Object.prototype.hasOwnProperty.call(ue, de) && (le[de] = ue[de]);
          }, oe(se, ce);
        };
        return function(se, ce) {
          if (typeof ce != "function" && ce !== null)
            throw new TypeError("Class extends value " + String(ce) + " is not a constructor or null");
          oe(se, ce);
          function le() {
            this.constructor = se;
          }
          se.prototype = ce === null ? Object.create(ce) : (le.prototype = ce.prototype, new le());
        };
      }(), H = EmpheqConfiguration.__read || function(oe, se) {
        var ce = typeof Symbol == "function" && oe[Symbol.iterator];
        if (!ce) return oe;
        var le = ce.call(oe), ue, de = [], fe;
        try {
          for (; (se === void 0 || se-- > 0) && !(ue = le.next()).done; ) de.push(ue.value);
        } catch (he) {
          fe = { error: he };
        } finally {
          try {
            ue && !ue.done && (ce = le.return) && ce.call(le);
          } finally {
            if (fe) throw fe.error;
          }
        }
        return de;
      }, V = EmpheqConfiguration.__importDefault || function(oe) {
        return oe && oe.__esModule ? oe : { default: oe };
      }, q;
      Object.defineProperty(O, "__esModule", { value: !0 }), O.EmpheqConfiguration = O.EmpheqMethods = O.EmpheqBeginItem = void 0;
      var G = requireConfiguration(), J = requireSymbolMap(), Z = V(requireParseUtil()), te = V(requireTexError()), ne = requireBaseItems(), ee = requireEmpheqUtil(), ae = function(oe) {
        _(se, oe);
        function se() {
          return oe !== null && oe.apply(this, arguments) || this;
        }
        return Object.defineProperty(se.prototype, "kind", {
          get: function() {
            return "empheq-begin";
          },
          enumerable: !1,
          configurable: !0
        }), se.prototype.checkItem = function(ce) {
          return ce.isKind("end") && ce.getName() === this.getName() && this.setProperty("end", !1), oe.prototype.checkItem.call(this, ce);
        }, se;
      }(ne.BeginItem);
      O.EmpheqBeginItem = ae, O.EmpheqMethods = {
        Empheq: function(oe, se) {
          if (oe.stack.env.closing === se.getName()) {
            delete oe.stack.env.closing, oe.Push(oe.itemFactory.create("end").setProperty("name", oe.stack.global.empheq)), oe.stack.global.empheq = "";
            var ce = oe.stack.Top();
            ee.EmpheqUtil.adjustTable(ce, oe), oe.Push(oe.itemFactory.create("end").setProperty("name", "empheq"));
          } else {
            Z.default.checkEqnEnv(oe), delete oe.stack.global.eqnenv;
            var le = oe.GetBrackets("\\begin{" + se.getName() + "}") || "", ue = H((oe.GetArgument("\\begin{" + se.getName() + "}") || "").split(/=/), 2), de = ue[0], fe = ue[1];
            if (!ee.EmpheqUtil.checkEnv(de))
              throw new te.default("UnknownEnv", 'Unknown environment "%1"', de);
            le && se.setProperties(ee.EmpheqUtil.splitOptions(le, { left: 1, right: 1 })), oe.stack.global.empheq = de, oe.string = "\\begin{" + de + "}" + (fe ? "{" + fe + "}" : "") + oe.string.slice(oe.i), oe.i = 0, oe.Push(se);
          }
        },
        EmpheqMO: function(oe, se, ce) {
          oe.Push(oe.create("token", "mo", {}, ce));
        },
        EmpheqDelim: function(oe, se) {
          var ce = oe.GetDelimiter(se);
          oe.Push(oe.create("token", "mo", { stretchy: !0, symmetric: !0 }, ce));
        }
      }, new J.EnvironmentMap("empheq-env", ee.EmpheqUtil.environment, {
        empheq: ["Empheq", "empheq"]
      }, O.EmpheqMethods), new J.CommandMap("empheq-macros", {
        empheqlbrace: ["EmpheqMO", "{"],
        empheqrbrace: ["EmpheqMO", "}"],
        empheqlbrack: ["EmpheqMO", "["],
        empheqrbrack: ["EmpheqMO", "]"],
        empheqlangle: ["EmpheqMO", "⟨"],
        empheqrangle: ["EmpheqMO", "⟩"],
        empheqlparen: ["EmpheqMO", "("],
        empheqrparen: ["EmpheqMO", ")"],
        empheqlvert: ["EmpheqMO", "|"],
        empheqrvert: ["EmpheqMO", "|"],
        empheqlVert: ["EmpheqMO", "‖"],
        empheqrVert: ["EmpheqMO", "‖"],
        empheqlfloor: ["EmpheqMO", "⌊"],
        empheqrfloor: ["EmpheqMO", "⌋"],
        empheqlceil: ["EmpheqMO", "⌈"],
        empheqrceil: ["EmpheqMO", "⌉"],
        empheqbiglbrace: ["EmpheqMO", "{"],
        empheqbigrbrace: ["EmpheqMO", "}"],
        empheqbiglbrack: ["EmpheqMO", "["],
        empheqbigrbrack: ["EmpheqMO", "]"],
        empheqbiglangle: ["EmpheqMO", "⟨"],
        empheqbigrangle: ["EmpheqMO", "⟩"],
        empheqbiglparen: ["EmpheqMO", "("],
        empheqbigrparen: ["EmpheqMO", ")"],
        empheqbiglvert: ["EmpheqMO", "|"],
        empheqbigrvert: ["EmpheqMO", "|"],
        empheqbiglVert: ["EmpheqMO", "‖"],
        empheqbigrVert: ["EmpheqMO", "‖"],
        empheqbiglfloor: ["EmpheqMO", "⌊"],
        empheqbigrfloor: ["EmpheqMO", "⌋"],
        empheqbiglceil: ["EmpheqMO", "⌈"],
        empheqbigrceil: ["EmpheqMO", "⌉"],
        empheql: "EmpheqDelim",
        empheqr: "EmpheqDelim",
        empheqbigl: "EmpheqDelim",
        empheqbigr: "EmpheqDelim"
      }, O.EmpheqMethods), O.EmpheqConfiguration = G.Configuration.create("empheq", {
        handler: {
          macro: ["empheq-macros"],
          environment: ["empheq-env"]
        },
        items: (q = {}, q[ae.prototype.kind] = ae, q)
      });
    }(EmpheqConfiguration)), EmpheqConfiguration;
  }
  var ExtpfeilConfiguration = {}, NewcommandConfiguration = {}, NewcommandMappings = {}, hasRequiredNewcommandMappings;
  function requireNewcommandMappings() {
    if (hasRequiredNewcommandMappings) return NewcommandMappings;
    hasRequiredNewcommandMappings = 1;
    var O = NewcommandMappings.__importDefault || function(V) {
      return V && V.__esModule ? V : { default: V };
    };
    Object.defineProperty(NewcommandMappings, "__esModule", { value: !0 });
    var _ = O(requireNewcommandMethods()), H = requireSymbolMap();
    return new H.CommandMap("Newcommand-macros", {
      newcommand: "NewCommand",
      renewcommand: "NewCommand",
      newenvironment: "NewEnvironment",
      renewenvironment: "NewEnvironment",
      def: "MacroDef",
      let: "Let"
    }, _.default), NewcommandMappings;
  }
  var hasRequiredNewcommandConfiguration;
  function requireNewcommandConfiguration() {
    if (hasRequiredNewcommandConfiguration) return NewcommandConfiguration;
    hasRequiredNewcommandConfiguration = 1;
    var O = NewcommandConfiguration.__createBinding || (Object.create ? function(ae, oe, se, ce) {
      ce === void 0 && (ce = se);
      var le = Object.getOwnPropertyDescriptor(oe, se);
      (!le || ("get" in le ? !oe.__esModule : le.writable || le.configurable)) && (le = { enumerable: !0, get: function() {
        return oe[se];
      } }), Object.defineProperty(ae, ce, le);
    } : function(ae, oe, se, ce) {
      ce === void 0 && (ce = se), ae[ce] = oe[se];
    }), _ = NewcommandConfiguration.__setModuleDefault || (Object.create ? function(ae, oe) {
      Object.defineProperty(ae, "default", { enumerable: !0, value: oe });
    } : function(ae, oe) {
      ae.default = oe;
    }), H = NewcommandConfiguration.__importStar || function(ae) {
      if (ae && ae.__esModule) return ae;
      var oe = {};
      if (ae != null) for (var se in ae) se !== "default" && Object.prototype.hasOwnProperty.call(ae, se) && O(oe, ae, se);
      return _(oe, ae), oe;
    }, V = NewcommandConfiguration.__importDefault || function(ae) {
      return ae && ae.__esModule ? ae : { default: ae };
    }, q;
    Object.defineProperty(NewcommandConfiguration, "__esModule", { value: !0 }), NewcommandConfiguration.NewcommandConfiguration = void 0;
    var G = requireConfiguration(), J = requireNewcommandItems(), Z = V(requireNewcommandUtil());
    requireNewcommandMappings();
    var te = V(requireParseMethods()), ne = H(requireSymbolMap()), ee = function(ae) {
      new ne.DelimiterMap(Z.default.NEW_DELIMITER, te.default.delimiter, {}), new ne.CommandMap(Z.default.NEW_COMMAND, {}, {}), new ne.EnvironmentMap(Z.default.NEW_ENVIRONMENT, te.default.environment, {}, {}), ae.append(G.Configuration.local({
        handler: {
          character: [],
          delimiter: [Z.default.NEW_DELIMITER],
          macro: [
            Z.default.NEW_DELIMITER,
            Z.default.NEW_COMMAND
          ],
          environment: [Z.default.NEW_ENVIRONMENT]
        },
        priority: -1
      }));
    };
    return NewcommandConfiguration.NewcommandConfiguration = G.Configuration.create("newcommand", {
      handler: {
        macro: ["Newcommand-macros"]
      },
      items: (q = {}, q[J.BeginEnvItem.prototype.kind] = J.BeginEnvItem, q),
      options: { maxMacros: 1e3 },
      init: ee
    }), NewcommandConfiguration;
  }
  var hasRequiredExtpfeilConfiguration;
  function requireExtpfeilConfiguration() {
    return hasRequiredExtpfeilConfiguration || (hasRequiredExtpfeilConfiguration = 1, function(O) {
      var _ = ExtpfeilConfiguration.__importDefault || function(ne) {
        return ne && ne.__esModule ? ne : { default: ne };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.ExtpfeilConfiguration = O.ExtpfeilMethods = void 0;
      var H = requireConfiguration(), V = requireSymbolMap(), q = requireAmsMethods(), G = _(requireNewcommandUtil()), J = requireNewcommandConfiguration(), Z = _(requireTexError());
      O.ExtpfeilMethods = {}, O.ExtpfeilMethods.xArrow = q.AmsMethods.xArrow, O.ExtpfeilMethods.NewExtArrow = function(ne, ee) {
        var ae = ne.GetArgument(ee), oe = ne.GetArgument(ee), se = ne.GetArgument(ee);
        if (!ae.match(/^\\([a-z]+|.)$/i))
          throw new Z.default("NewextarrowArg1", "First argument to %1 must be a control sequence name", ee);
        if (!oe.match(/^(\d+),(\d+)$/))
          throw new Z.default("NewextarrowArg2", "Second argument to %1 must be two integers separated by a comma", ee);
        if (!se.match(/^(\d+|0x[0-9A-F]+)$/i))
          throw new Z.default("NewextarrowArg3", "Third argument to %1 must be a unicode character number", ee);
        ae = ae.substr(1);
        var ce = oe.split(",");
        G.default.addMacro(ne, ae, O.ExtpfeilMethods.xArrow, [parseInt(se), parseInt(ce[0]), parseInt(ce[1])]);
      }, new V.CommandMap("extpfeil", {
        xtwoheadrightarrow: ["xArrow", 8608, 12, 16],
        xtwoheadleftarrow: ["xArrow", 8606, 17, 13],
        xmapsto: ["xArrow", 8614, 6, 7],
        xlongequal: ["xArrow", 61, 7, 7],
        xtofrom: ["xArrow", 8644, 12, 12],
        Newextarrow: "NewExtArrow"
      }, O.ExtpfeilMethods);
      var te = function(ne) {
        J.NewcommandConfiguration.init(ne);
      };
      O.ExtpfeilConfiguration = H.Configuration.create("extpfeil", {
        handler: { macro: ["extpfeil"] },
        init: te
      });
    }(ExtpfeilConfiguration)), ExtpfeilConfiguration;
  }
  var GensymbConfiguration = {}, hasRequiredGensymbConfiguration;
  function requireGensymbConfiguration() {
    if (hasRequiredGensymbConfiguration) return GensymbConfiguration;
    hasRequiredGensymbConfiguration = 1, Object.defineProperty(GensymbConfiguration, "__esModule", { value: !0 }), GensymbConfiguration.GensymbConfiguration = void 0;
    var O = requireConfiguration(), _ = requireTexConstants(), H = requireSymbolMap();
    function V(q, G) {
      var J = G.attributes || {};
      J.mathvariant = _.TexConstant.Variant.NORMAL, J.class = "MathML-Unit";
      var Z = q.create("token", "mi", J, G.char);
      q.Push(Z);
    }
    return new H.CharacterMap("gensymb-symbols", V, {
      ohm: "Ω",
      degree: "°",
      celsius: "℃",
      perthousand: "‰",
      micro: "µ"
    }), GensymbConfiguration.GensymbConfiguration = O.Configuration.create("gensymb", {
      handler: { macro: ["gensymb-symbols"] }
    }), GensymbConfiguration;
  }
  var HtmlConfiguration = {}, HtmlMethods = {}, hasRequiredHtmlMethods;
  function requireHtmlMethods() {
    if (hasRequiredHtmlMethods) return HtmlMethods;
    hasRequiredHtmlMethods = 1;
    var O = HtmlMethods.__importDefault || function(q) {
      return q && q.__esModule ? q : { default: q };
    };
    Object.defineProperty(HtmlMethods, "__esModule", { value: !0 });
    var _ = O(requireNodeUtil()), H = {};
    H.Href = function(q, G) {
      var J = q.GetArgument(G), Z = V(q, G);
      _.default.setAttribute(Z, "href", J), q.Push(Z);
    }, H.Class = function(q, G) {
      var J = q.GetArgument(G), Z = V(q, G), te = _.default.getAttribute(Z, "class");
      te && (J = te + " " + J), _.default.setAttribute(Z, "class", J), q.Push(Z);
    }, H.Style = function(q, G) {
      var J = q.GetArgument(G), Z = V(q, G), te = _.default.getAttribute(Z, "style");
      te && (J.charAt(J.length - 1) !== ";" && (J += ";"), J = te + " " + J), _.default.setAttribute(Z, "style", J), q.Push(Z);
    }, H.Id = function(q, G) {
      var J = q.GetArgument(G), Z = V(q, G);
      _.default.setAttribute(Z, "id", J), q.Push(Z);
    };
    var V = function(q, G) {
      var J = q.ParseArg(G);
      if (!_.default.isInferred(J))
        return J;
      var Z = _.default.getChildren(J);
      if (Z.length === 1)
        return Z[0];
      var te = q.create("node", "mrow");
      return _.default.copyChildren(J, te), _.default.copyAttributes(J, te), te;
    };
    return HtmlMethods.default = H, HtmlMethods;
  }
  var hasRequiredHtmlConfiguration;
  function requireHtmlConfiguration() {
    if (hasRequiredHtmlConfiguration) return HtmlConfiguration;
    hasRequiredHtmlConfiguration = 1;
    var O = HtmlConfiguration.__importDefault || function(q) {
      return q && q.__esModule ? q : { default: q };
    };
    Object.defineProperty(HtmlConfiguration, "__esModule", { value: !0 }), HtmlConfiguration.HtmlConfiguration = void 0;
    var _ = requireConfiguration(), H = requireSymbolMap(), V = O(requireHtmlMethods());
    return new H.CommandMap("html_macros", {
      href: "Href",
      class: "Class",
      style: "Style",
      cssId: "Id"
    }, V.default), HtmlConfiguration.HtmlConfiguration = _.Configuration.create("html", { handler: { macro: ["html_macros"] } }), HtmlConfiguration;
  }
  var MathtoolsConfiguration = {}, MathtoolsMappings = {}, MathtoolsMethods = {}, MathtoolsUtil = {}, hasRequiredMathtoolsUtil;
  function requireMathtoolsUtil() {
    if (hasRequiredMathtoolsUtil) return MathtoolsUtil;
    hasRequiredMathtoolsUtil = 1;
    var O = MathtoolsUtil.__read || function(ee, ae) {
      var oe = typeof Symbol == "function" && ee[Symbol.iterator];
      if (!oe) return ee;
      var se = oe.call(ee), ce, le = [], ue;
      try {
        for (; (ae === void 0 || ae-- > 0) && !(ce = se.next()).done; ) le.push(ce.value);
      } catch (de) {
        ue = { error: de };
      } finally {
        try {
          ce && !ce.done && (oe = se.return) && oe.call(se);
        } finally {
          if (ue) throw ue.error;
        }
      }
      return le;
    }, _ = MathtoolsUtil.__importDefault || function(ee) {
      return ee && ee.__esModule ? ee : { default: ee };
    };
    Object.defineProperty(MathtoolsUtil, "__esModule", { value: !0 }), MathtoolsUtil.MathtoolsUtil = void 0;
    var H = requireBaseItems(), V = _(requireParseUtil()), q = _(requireTexParser()), G = _(requireTexError()), J = require_Symbol(), Z = requireOptions(), te = requireMathtoolsMethods(), ne = requireMathtoolsConfiguration();
    return MathtoolsUtil.MathtoolsUtil = {
      setDisplayLevel: function(ee, ae) {
        if (ae) {
          var oe = O((0, Z.lookup)(ae, {
            "\\displaystyle": [!0, 0],
            "\\textstyle": [!1, 0],
            "\\scriptstyle": [!1, 1],
            "\\scriptscriptstyle": [!1, 2]
          }, [null, null]), 2), se = oe[0], ce = oe[1];
          se !== null && (ee.attributes.set("displaystyle", se), ee.attributes.set("scriptlevel", ce));
        }
      },
      checkAlignment: function(ee, ae) {
        var oe = ee.stack.Top();
        if (oe.kind !== H.EqnArrayItem.prototype.kind)
          throw new G.default("NotInAlignment", "%1 can only be used in aligment environments", ae);
        return oe;
      },
      addPairedDelims: function(ee, ae, oe) {
        var se = ee.handlers.retrieve(ne.PAIREDDELIMS);
        se.add(ae, new J.Macro(ae, te.MathtoolsMethods.PairedDelimiters, oe));
      },
      spreadLines: function(ee, ae) {
        if (ee.isKind("mtable")) {
          var oe = ee.attributes.get("rowspacing");
          if (oe) {
            var se = V.default.dimen2em(ae);
            oe = oe.split(/ /).map(function(ce) {
              return V.default.Em(Math.max(0, V.default.dimen2em(ce) + se));
            }).join(" ");
          } else
            oe = ae;
          ee.attributes.set("rowspacing", oe);
        }
      },
      plusOrMinus: function(ee, ae) {
        if (ae = ae.trim(), !ae.match(/^[-+]?(?:\d+(?:\.\d*)?|\.\d+)$/))
          throw new G.default("NotANumber", "Argument to %1 is not a number", ee);
        return ae.match(/^[-+]/) ? ae : "+" + ae;
      },
      getScript: function(ee, ae, oe) {
        var se = V.default.trimSpaces(ee.GetArgument(ae));
        if (se === "")
          return ee.create("node", "none");
        var ce = ee.options.mathtools["prescript-".concat(oe, "-format")];
        return ce && (se = "".concat(ce, "{").concat(se, "}")), new q.default(se, ee.stack.env, ee.configuration).mml();
      }
    }, MathtoolsUtil;
  }
  var hasRequiredMathtoolsMethods;
  function requireMathtoolsMethods() {
    return hasRequiredMathtoolsMethods || (hasRequiredMathtoolsMethods = 1, function(O) {
      var _ = MathtoolsMethods.__assign || function() {
        return _ = Object.assign || function(de) {
          for (var fe, he = 1, me = arguments.length; he < me; he++) {
            fe = arguments[he];
            for (var pe in fe) Object.prototype.hasOwnProperty.call(fe, pe) && (de[pe] = fe[pe]);
          }
          return de;
        }, _.apply(this, arguments);
      }, H = MathtoolsMethods.__read || function(de, fe) {
        var he = typeof Symbol == "function" && de[Symbol.iterator];
        if (!he) return de;
        var me = he.call(de), pe, ye = [], ge;
        try {
          for (; (fe === void 0 || fe-- > 0) && !(pe = me.next()).done; ) ye.push(pe.value);
        } catch (ve) {
          ge = { error: ve };
        } finally {
          try {
            pe && !pe.done && (he = me.return) && he.call(me);
          } finally {
            if (ge) throw ge.error;
          }
        }
        return ye;
      }, V = MathtoolsMethods.__values || function(de) {
        var fe = typeof Symbol == "function" && Symbol.iterator, he = fe && de[fe], me = 0;
        if (he) return he.call(de);
        if (de && typeof de.length == "number") return {
          next: function() {
            return de && me >= de.length && (de = void 0), { value: de && de[me++], done: !de };
          }
        };
        throw new TypeError(fe ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, q = MathtoolsMethods.__importDefault || function(de) {
        return de && de.__esModule ? de : { default: de };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.MathtoolsMethods = void 0;
      var G = q(requireParseUtil()), J = requireAmsMethods(), Z = q(requireBaseMethods()), te = q(requireTexParser()), ne = q(requireTexError()), ee = q(requireNodeUtil()), ae = requireMmlNode(), oe = requireLengths(), se = requireOptions(), ce = q(requireNewcommandUtil()), le = q(requireNewcommandMethods()), ue = requireMathtoolsUtil();
      O.MathtoolsMethods = {
        MtMatrix: function(de, fe, he, me) {
          var pe = de.GetBrackets("\\begin{".concat(fe.getName(), "}"), "c");
          return O.MathtoolsMethods.Array(de, fe, he, me, pe);
        },
        MtSmallMatrix: function(de, fe, he, me, pe) {
          return pe || (pe = de.GetBrackets("\\begin{".concat(fe.getName(), "}"), de.options.mathtools["smallmatrix-align"])), O.MathtoolsMethods.Array(de, fe, he, me, pe, G.default.Em(1 / 3), ".2em", "S", 1);
        },
        MtMultlined: function(de, fe) {
          var he, me = "\\begin{".concat(fe.getName(), "}"), pe = de.GetBrackets(me, de.options.mathtools["multlined-pos"] || "c"), ye = pe ? de.GetBrackets(me, "") : "";
          pe && !pe.match(/^[cbt]$/) && (he = H([pe, ye], 2), ye = he[0], pe = he[1]), de.Push(fe);
          var ge = de.itemFactory.create("multlined", de, fe);
          return ge.arraydef = {
            displaystyle: !0,
            rowspacing: ".5em",
            width: ye || "auto",
            columnwidth: "100%"
          }, G.default.setArrayAlign(ge, pe || "c");
        },
        HandleShove: function(de, fe, he) {
          var me = de.stack.Top();
          if (me.kind !== "multline" && me.kind !== "multlined")
            throw new ne.default("CommandInMultlined", "%1 can only appear within the multline or multlined environments", fe);
          if (me.Size())
            throw new ne.default("CommandAtTheBeginingOfLine", "%1 must come at the beginning of the line", fe);
          me.setProperty("shove", he);
          var pe = de.GetBrackets(fe), ye = de.ParseArg(fe);
          if (pe) {
            var ge = de.create("node", "mrow", []), ve = de.create("node", "mspace", [], { width: pe });
            he === "left" ? (ge.appendChild(ve), ge.appendChild(ye)) : (ge.appendChild(ye), ge.appendChild(ve)), ye = ge;
          }
          de.Push(ye);
        },
        SpreadLines: function(de, fe) {
          var he, me;
          if (de.stack.env.closing === fe.getName()) {
            delete de.stack.env.closing;
            var pe = de.stack.Pop(), ye = pe.toMml(), ge = pe.getProperty("spread");
            if (ye.isInferred)
              try {
                for (var ve = V(ee.default.getChildren(ye)), be = ve.next(); !be.done; be = ve.next()) {
                  var Ce = be.value;
                  ue.MathtoolsUtil.spreadLines(Ce, ge);
                }
              } catch (Se) {
                he = { error: Se };
              } finally {
                try {
                  be && !be.done && (me = ve.return) && me.call(ve);
                } finally {
                  if (he) throw he.error;
                }
              }
            else
              ue.MathtoolsUtil.spreadLines(ye, ge);
            de.Push(ye);
          } else {
            var ge = de.GetDimen("\\begin{".concat(fe.getName(), "}"));
            fe.setProperty("spread", ge), de.Push(fe);
          }
        },
        Cases: function(de, fe, he, me, pe) {
          var ye = de.itemFactory.create("array").setProperty("casesEnv", fe.getName());
          return ye.arraydef = {
            rowspacing: ".2em",
            columnspacing: "1em",
            columnalign: "left"
          }, pe === "D" && (ye.arraydef.displaystyle = !0), ye.setProperties({ open: he, close: me }), de.Push(fe), ye;
        },
        MathLap: function(de, fe, he, me) {
          var pe = de.GetBrackets(fe, "").trim(), ye = de.create("node", "mstyle", [
            de.create("node", "mpadded", [de.ParseArg(fe)], _({ width: 0 }, he === "r" ? {} : { lspace: he === "l" ? "-1width" : "-.5width" }))
          ], { "data-cramped": me });
          ue.MathtoolsUtil.setDisplayLevel(ye, pe), de.Push(de.create("node", "TeXAtom", [ye]));
        },
        Cramped: function(de, fe) {
          var he = de.GetBrackets(fe, "").trim(), me = de.ParseArg(fe), pe = de.create("node", "mstyle", [me], { "data-cramped": !0 });
          ue.MathtoolsUtil.setDisplayLevel(pe, he), de.Push(pe);
        },
        MtLap: function(de, fe, he) {
          var me = G.default.internalMath(de, de.GetArgument(fe), 0), pe = de.create("node", "mpadded", me, { width: 0 });
          he !== "r" && ee.default.setAttribute(pe, "lspace", he === "l" ? "-1width" : "-.5width"), de.Push(pe);
        },
        MathMakeBox: function(de, fe) {
          var he = de.GetBrackets(fe), me = de.GetBrackets(fe, "c"), pe = de.create("node", "mpadded", [de.ParseArg(fe)]);
          he && ee.default.setAttribute(pe, "width", he);
          var ye = (0, se.lookup)(me, { c: "center", r: "right" }, "");
          ye && ee.default.setAttribute(pe, "data-align", ye), de.Push(pe);
        },
        MathMBox: function(de, fe) {
          de.Push(de.create("node", "mrow", [de.ParseArg(fe)]));
        },
        UnderOverBracket: function(de, fe) {
          var he = (0, oe.length2em)(de.GetBrackets(fe, ".1em"), 0.1), me = de.GetBrackets(fe, ".2em"), pe = de.GetArgument(fe), ye = H(fe.charAt(1) === "o" ? ["over", "accent", "bottom"] : ["under", "accentunder", "top"], 3), ge = ye[0], ve = ye[1], be = ye[2], Ce = (0, oe.em)(he), Se = new te.default(pe, de.stack.env, de.configuration).mml(), we = new te.default(pe, de.stack.env, de.configuration).mml(), Ee = de.create("node", "mpadded", [
            de.create("node", "mphantom", [we])
          ], {
            style: "border: ".concat(Ce, " solid; border-").concat(be, ": none"),
            height: me,
            depth: 0
          }), xe = G.default.underOver(de, Se, Ee, ge, !0), Le = ee.default.getChildAt(ee.default.getChildAt(xe, 0), 0);
          ee.default.setAttribute(Le, ve, !0), de.Push(xe);
        },
        Aboxed: function(de, fe) {
          var he = ue.MathtoolsUtil.checkAlignment(de, fe);
          he.row.length % 2 === 1 && he.row.push(de.create("node", "mtd", []));
          var me = de.GetArgument(fe), pe = de.string.substr(de.i);
          de.string = me + "&&\\endAboxed", de.i = 0;
          var ye = de.GetUpTo(fe, "&"), ge = de.GetUpTo(fe, "&");
          de.GetUpTo(fe, "\\endAboxed");
          var ve = G.default.substituteArgs(de, [ye, ge], "\\rlap{\\boxed{#1{}#2}}\\kern.267em\\phantom{#1}&\\phantom{{}#2}\\kern.267em");
          de.string = ve + pe, de.i = 0;
        },
        ArrowBetweenLines: function(de, fe) {
          var he = ue.MathtoolsUtil.checkAlignment(de, fe);
          if (he.Size() || he.row.length)
            throw new ne.default("BetweenLines", "%1 must be on a row by itself", fe);
          var me = de.GetStar(), pe = de.GetBrackets(fe, "\\Updownarrow");
          me && (he.EndEntry(), he.EndEntry());
          var ye = me ? "\\quad" + pe : pe + "\\quad", ge = new te.default(ye, de.stack.env, de.configuration).mml();
          de.Push(ge), he.EndEntry(), he.EndRow();
        },
        VDotsWithin: function(de, fe) {
          var he = de.stack.Top(), me = he.getProperty("flushspaceabove") === he.table.length, pe = "\\mmlToken{mi}{}" + de.GetArgument(fe) + "\\mmlToken{mi}{}", ye = new te.default(pe, de.stack.env, de.configuration).mml(), ge = de.create("node", "mpadded", [
            de.create("node", "mpadded", [
              de.create("node", "mo", [
                de.create("text", "⋮")
              ])
            ], _({ width: 0, lspace: "-.5width" }, me ? { height: "-.6em", voffset: "-.18em" } : {})),
            de.create("node", "mphantom", [ye])
          ], {
            lspace: ".5width"
          });
          de.Push(ge);
        },
        ShortVDotsWithin: function(de, fe) {
          var he = de.stack.Top(), me = de.GetStar();
          O.MathtoolsMethods.FlushSpaceAbove(de, "\\MTFlushSpaceAbove"), !me && he.EndEntry(), O.MathtoolsMethods.VDotsWithin(de, "\\vdotswithin"), me && he.EndEntry(), O.MathtoolsMethods.FlushSpaceBelow(de, "\\MTFlushSpaceBelow");
        },
        FlushSpaceAbove: function(de, fe) {
          var he = ue.MathtoolsUtil.checkAlignment(de, fe);
          he.setProperty("flushspaceabove", he.table.length), he.addRowSpacing("-" + de.options.mathtools.shortvdotsadjustabove);
        },
        FlushSpaceBelow: function(de, fe) {
          var he = ue.MathtoolsUtil.checkAlignment(de, fe);
          he.Size() && he.EndEntry(), he.EndRow(), he.addRowSpacing("-" + de.options.mathtools.shortvdotsadjustbelow);
        },
        PairedDelimiters: function(de, fe, he, me, pe, ye, ge, ve) {
          pe === void 0 && (pe = "#1"), ye === void 0 && (ye = 1), ge === void 0 && (ge = ""), ve === void 0 && (ve = "");
          var be = de.GetStar(), Ce = be ? "" : de.GetBrackets(fe), Se = H(be ? ["\\left", "\\right"] : Ce ? [Ce + "l", Ce + "r"] : ["", ""], 2), we = Se[0], Ee = Se[1], xe = be ? "\\middle" : Ce || "";
          if (ye) {
            for (var Le = [], Qe = Le.length; Qe < ye; Qe++)
              Le.push(de.GetArgument(fe));
            ge = G.default.substituteArgs(de, Le, ge), pe = G.default.substituteArgs(de, Le, pe), ve = G.default.substituteArgs(de, Le, ve);
          }
          pe = pe.replace(/\\delimsize/g, xe), de.string = [ge, we, he, pe, Ee, me, ve, de.string.substr(de.i)].reduce(function(ke, Me) {
            return G.default.addArgs(de, ke, Me);
          }, ""), de.i = 0, G.default.checkMaxMacros(de);
        },
        DeclarePairedDelimiter: function(de, fe) {
          var he = ce.default.GetCsNameArgument(de, fe), me = de.GetArgument(fe), pe = de.GetArgument(fe);
          ue.MathtoolsUtil.addPairedDelims(de.configuration, he, [me, pe]);
        },
        DeclarePairedDelimiterX: function(de, fe) {
          var he = ce.default.GetCsNameArgument(de, fe), me = ce.default.GetArgCount(de, fe), pe = de.GetArgument(fe), ye = de.GetArgument(fe), ge = de.GetArgument(fe);
          ue.MathtoolsUtil.addPairedDelims(de.configuration, he, [pe, ye, ge, me]);
        },
        DeclarePairedDelimiterXPP: function(de, fe) {
          var he = ce.default.GetCsNameArgument(de, fe), me = ce.default.GetArgCount(de, fe), pe = de.GetArgument(fe), ye = de.GetArgument(fe), ge = de.GetArgument(fe), ve = de.GetArgument(fe), be = de.GetArgument(fe);
          ue.MathtoolsUtil.addPairedDelims(de.configuration, he, [ye, ge, be, me, pe, ve]);
        },
        CenterColon: function(de, fe, he, me, pe) {
          me === void 0 && (me = !1), pe === void 0 && (pe = !1);
          var ye = de.options.mathtools, ge = de.create("token", "mo", {}, ":");
          if (he && (ye.centercolon || me)) {
            var ve = ye["centercolon-offset"];
            ge = de.create("node", "mpadded", [ge], _({ voffset: ve, height: "+".concat(ve), depth: "-".concat(ve) }, pe ? { width: ye["thincolon-dw"], lspace: ye["thincolon-dx"] } : {}));
          }
          de.Push(ge);
        },
        Relation: function(de, fe, he, me) {
          var pe = de.options.mathtools;
          pe["use-unicode"] && me ? de.Push(de.create("token", "mo", { texClass: ae.TEXCLASS.REL }, me)) : (he = "\\mathrel{" + he.replace(/:/g, "\\MTThinColon").replace(/-/g, "\\mathrel{-}") + "}", de.string = G.default.addArgs(de, he, de.string.substr(de.i)), de.i = 0);
        },
        NArrow: function(de, fe, he, me) {
          de.Push(de.create("node", "TeXAtom", [
            de.create("token", "mtext", {}, he),
            de.create("node", "mpadded", [
              de.create("node", "mpadded", [
                de.create("node", "menclose", [
                  de.create("node", "mspace", [], { height: ".2em", depth: 0, width: ".4em" })
                ], { notation: "updiagonalstrike", "data-thickness": ".05em", "data-padding": 0 })
              ], { width: 0, lspace: "-.5width", voffset: me }),
              de.create("node", "mphantom", [
                de.create("token", "mtext", {}, he)
              ])
            ], { width: 0, lspace: "-.5width" })
          ], { texClass: ae.TEXCLASS.REL }));
        },
        SplitFrac: function(de, fe, he) {
          var me = de.ParseArg(fe), pe = de.ParseArg(fe);
          de.Push(de.create("node", "mstyle", [
            de.create("node", "mfrac", [
              de.create("node", "mstyle", [
                me,
                de.create("token", "mi"),
                de.create("token", "mspace", { width: "1em" })
              ], { scriptlevel: 0 }),
              de.create("node", "mstyle", [
                de.create("token", "mspace", { width: "1em" }),
                de.create("token", "mi"),
                pe
              ], { scriptlevel: 0 })
            ], { linethickness: 0, numalign: "left", denomalign: "right" })
          ], { displaystyle: he, scriptlevel: 0 }));
        },
        XMathStrut: function(de, fe) {
          var he = de.GetBrackets(fe), me = de.GetArgument(fe);
          me = ue.MathtoolsUtil.plusOrMinus(fe, me), he = ue.MathtoolsUtil.plusOrMinus(fe, he || me), de.Push(de.create("node", "TeXAtom", [
            de.create("node", "mpadded", [
              de.create("node", "mphantom", [
                de.create("token", "mo", { stretchy: !1 }, "(")
              ])
            ], { width: 0, height: me + "height", depth: he + "depth" })
          ], { texClass: ae.TEXCLASS.ORD }));
        },
        Prescript: function(de, fe) {
          var he = ue.MathtoolsUtil.getScript(de, fe, "sup"), me = ue.MathtoolsUtil.getScript(de, fe, "sub"), pe = ue.MathtoolsUtil.getScript(de, fe, "arg");
          if (ee.default.isType(he, "none") && ee.default.isType(me, "none")) {
            de.Push(pe);
            return;
          }
          var ye = de.create("node", "mmultiscripts", [pe]);
          ee.default.getChildren(ye).push(null, null), ee.default.appendChildren(ye, [de.create("node", "mprescripts"), me, he]), ye.setProperty("fixPrescript", !0), de.Push(ye);
        },
        NewTagForm: function(de, fe, he) {
          he === void 0 && (he = !1);
          var me = de.tags;
          if (!("mtFormats" in me))
            throw new ne.default("TagsNotMT", "%1 can only be used with ams or mathtools tags", fe);
          var pe = de.GetArgument(fe).trim();
          if (!pe)
            throw new ne.default("InvalidTagFormID", "Tag form name can't be empty");
          var ye = de.GetBrackets(fe, ""), ge = de.GetArgument(fe), ve = de.GetArgument(fe);
          if (!he && me.mtFormats.has(pe))
            throw new ne.default("DuplicateTagForm", "Duplicate tag form: %1", pe);
          me.mtFormats.set(pe, [ge, ve, ye]);
        },
        UseTagForm: function(de, fe) {
          var he = de.tags;
          if (!("mtFormats" in he))
            throw new ne.default("TagsNotMT", "%1 can only be used with ams or mathtools tags", fe);
          var me = de.GetArgument(fe).trim();
          if (!me) {
            he.mtCurrent = null;
            return;
          }
          if (!he.mtFormats.has(me))
            throw new ne.default("UndefinedTagForm", "Undefined tag form: %1", me);
          he.mtCurrent = he.mtFormats.get(me);
        },
        SetOptions: function(de, fe) {
          var he, me, pe = de.options.mathtools;
          if (!pe["allow-mathtoolsset"])
            throw new ne.default("ForbiddenMathtoolsSet", "%1 is disabled", fe);
          var ye = {};
          Object.keys(pe).forEach(function(we) {
            we !== "pariedDelimiters" && we !== "tagforms" && we !== "allow-mathtoolsset" && (ye[we] = 1);
          });
          var ge = de.GetArgument(fe), ve = G.default.keyvalOptions(ge, ye, !0);
          try {
            for (var be = V(Object.keys(ve)), Ce = be.next(); !Ce.done; Ce = be.next()) {
              var Se = Ce.value;
              pe[Se] = ve[Se];
            }
          } catch (we) {
            he = { error: we };
          } finally {
            try {
              Ce && !Ce.done && (me = be.return) && me.call(be);
            } finally {
              if (he) throw he.error;
            }
          }
        },
        Array: Z.default.Array,
        Macro: Z.default.Macro,
        xArrow: J.AmsMethods.xArrow,
        HandleRef: J.AmsMethods.HandleRef,
        AmsEqnArray: J.AmsMethods.AmsEqnArray,
        MacroWithTemplate: le.default.MacroWithTemplate
      };
    }(MathtoolsMethods)), MathtoolsMethods;
  }
  var hasRequiredMathtoolsMappings;
  function requireMathtoolsMappings() {
    if (hasRequiredMathtoolsMappings) return MathtoolsMappings;
    hasRequiredMathtoolsMappings = 1;
    var O = MathtoolsMappings.__importDefault || function(G) {
      return G && G.__esModule ? G : { default: G };
    };
    Object.defineProperty(MathtoolsMappings, "__esModule", { value: !0 });
    var _ = O(requireParseMethods()), H = requireSymbolMap(), V = requireTexConstants(), q = requireMathtoolsMethods();
    return new H.CommandMap("mathtools-macros", {
      shoveleft: ["HandleShove", V.TexConstant.Align.LEFT],
      shoveright: ["HandleShove", V.TexConstant.Align.RIGHT],
      xleftrightarrow: ["xArrow", 8596, 10, 10],
      xLeftarrow: ["xArrow", 8656, 12, 7],
      xRightarrow: ["xArrow", 8658, 7, 12],
      xLeftrightarrow: ["xArrow", 8660, 12, 12],
      xhookleftarrow: ["xArrow", 8617, 10, 5],
      xhookrightarrow: ["xArrow", 8618, 5, 10],
      xmapsto: ["xArrow", 8614, 10, 10],
      xrightharpoondown: ["xArrow", 8641, 5, 10],
      xleftharpoondown: ["xArrow", 8637, 10, 5],
      xrightleftharpoons: ["xArrow", 8652, 10, 10],
      xrightharpoonup: ["xArrow", 8640, 5, 10],
      xleftharpoonup: ["xArrow", 8636, 10, 5],
      xleftrightharpoons: ["xArrow", 8651, 10, 10],
      mathllap: ["MathLap", "l", !1],
      mathrlap: ["MathLap", "r", !1],
      mathclap: ["MathLap", "c", !1],
      clap: ["MtLap", "c"],
      textllap: ["MtLap", "l"],
      textrlap: ["MtLap", "r"],
      textclap: ["MtLap", "c"],
      cramped: "Cramped",
      crampedllap: ["MathLap", "l", !0],
      crampedrlap: ["MathLap", "r", !0],
      crampedclap: ["MathLap", "c", !0],
      crampedsubstack: ["Macro", "\\begin{crampedsubarray}{c}#1\\end{crampedsubarray}", 1],
      mathmbox: "MathMBox",
      mathmakebox: "MathMakeBox",
      overbracket: "UnderOverBracket",
      underbracket: "UnderOverBracket",
      refeq: "HandleRef",
      MoveEqLeft: ["Macro", "\\hspace{#1em}&\\hspace{-#1em}", 1, "2"],
      Aboxed: "Aboxed",
      ArrowBetweenLines: "ArrowBetweenLines",
      vdotswithin: "VDotsWithin",
      shortvdotswithin: "ShortVDotsWithin",
      MTFlushSpaceAbove: "FlushSpaceAbove",
      MTFlushSpaceBelow: "FlushSpaceBelow",
      DeclarePairedDelimiter: "DeclarePairedDelimiter",
      DeclarePairedDelimiterX: "DeclarePairedDelimiterX",
      DeclarePairedDelimiterXPP: "DeclarePairedDelimiterXPP",
      DeclarePairedDelimiters: "DeclarePairedDelimiter",
      DeclarePairedDelimitersX: "DeclarePairedDelimiterX",
      DeclarePairedDelimitersXPP: "DeclarePairedDelimiterXPP",
      centercolon: ["CenterColon", !0, !0],
      ordinarycolon: ["CenterColon", !1],
      MTThinColon: ["CenterColon", !0, !0, !0],
      coloneqq: ["Relation", ":=", "≔"],
      Coloneqq: ["Relation", "::=", "⩴"],
      coloneq: ["Relation", ":-"],
      Coloneq: ["Relation", "::-"],
      eqqcolon: ["Relation", "=:", "≕"],
      Eqqcolon: ["Relation", "=::"],
      eqcolon: ["Relation", "-:", "∹"],
      Eqcolon: ["Relation", "-::"],
      colonapprox: ["Relation", ":\\approx"],
      Colonapprox: ["Relation", "::\\approx"],
      colonsim: ["Relation", ":\\sim"],
      Colonsim: ["Relation", "::\\sim"],
      dblcolon: ["Relation", "::", "∷"],
      nuparrow: ["NArrow", "↑", ".06em"],
      ndownarrow: ["NArrow", "↓", ".25em"],
      bigtimes: ["Macro", "\\mathop{\\Large\\kern-.1em\\boldsymbol{\\times}\\kern-.1em}"],
      splitfrac: ["SplitFrac", !1],
      splitdfrac: ["SplitFrac", !0],
      xmathstrut: "XMathStrut",
      prescript: "Prescript",
      newtagform: ["NewTagForm", !1],
      renewtagform: ["NewTagForm", !0],
      usetagform: "UseTagForm",
      adjustlimits: [
        "MacroWithTemplate",
        "\\mathop{{#1}\\vphantom{{#3}}}_{{#2}\\vphantom{{#4}}}\\mathop{{#3}\\vphantom{{#1}}}_{{#4}\\vphantom{{#2}}}",
        4,
        ,
        "_",
        ,
        "_"
      ],
      mathtoolsset: "SetOptions"
    }, q.MathtoolsMethods), new H.EnvironmentMap("mathtools-environments", _.default.environment, {
      dcases: ["Array", null, "\\{", "", "ll", null, ".2em", "D"],
      rcases: ["Array", null, "", "\\}", "ll", null, ".2em"],
      drcases: ["Array", null, "", "\\}", "ll", null, ".2em", "D"],
      "dcases*": ["Cases", null, "{", "", "D"],
      "rcases*": ["Cases", null, "", "}"],
      "drcases*": ["Cases", null, "", "}", "D"],
      "cases*": ["Cases", null, "{", ""],
      "matrix*": ["MtMatrix", null, null, null],
      "pmatrix*": ["MtMatrix", null, "(", ")"],
      "bmatrix*": ["MtMatrix", null, "[", "]"],
      "Bmatrix*": ["MtMatrix", null, "\\{", "\\}"],
      "vmatrix*": ["MtMatrix", null, "\\vert", "\\vert"],
      "Vmatrix*": ["MtMatrix", null, "\\Vert", "\\Vert"],
      "smallmatrix*": ["MtSmallMatrix", null, null, null],
      psmallmatrix: ["MtSmallMatrix", null, "(", ")", "c"],
      "psmallmatrix*": ["MtSmallMatrix", null, "(", ")"],
      bsmallmatrix: ["MtSmallMatrix", null, "[", "]", "c"],
      "bsmallmatrix*": ["MtSmallMatrix", null, "[", "]"],
      Bsmallmatrix: ["MtSmallMatrix", null, "\\{", "\\}", "c"],
      "Bsmallmatrix*": ["MtSmallMatrix", null, "\\{", "\\}"],
      vsmallmatrix: ["MtSmallMatrix", null, "\\vert", "\\vert", "c"],
      "vsmallmatrix*": ["MtSmallMatrix", null, "\\vert", "\\vert"],
      Vsmallmatrix: ["MtSmallMatrix", null, "\\Vert", "\\Vert", "c"],
      "Vsmallmatrix*": ["MtSmallMatrix", null, "\\Vert", "\\Vert"],
      crampedsubarray: ["Array", null, null, null, null, "0em", "0.1em", "S'", 1],
      multlined: "MtMultlined",
      spreadlines: ["SpreadLines", !0],
      lgathered: ["AmsEqnArray", null, null, null, "l", null, ".5em", "D"],
      rgathered: ["AmsEqnArray", null, null, null, "r", null, ".5em", "D"]
    }, q.MathtoolsMethods), new H.DelimiterMap("mathtools-delimiters", _.default.delimiter, {
      "\\lparen": "(",
      "\\rparen": ")"
    }), new H.CommandMap("mathtools-characters", {
      ":": ["CenterColon", !0]
    }, q.MathtoolsMethods), MathtoolsMappings;
  }
  var MathtoolsTags = {}, hasRequiredMathtoolsTags;
  function requireMathtoolsTags() {
    if (hasRequiredMathtoolsTags) return MathtoolsTags;
    hasRequiredMathtoolsTags = 1;
    var O = MathtoolsTags.__extends || /* @__PURE__ */ function() {
      var te = function(ne, ee) {
        return te = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ae, oe) {
          ae.__proto__ = oe;
        } || function(ae, oe) {
          for (var se in oe) Object.prototype.hasOwnProperty.call(oe, se) && (ae[se] = oe[se]);
        }, te(ne, ee);
      };
      return function(ne, ee) {
        if (typeof ee != "function" && ee !== null)
          throw new TypeError("Class extends value " + String(ee) + " is not a constructor or null");
        te(ne, ee);
        function ae() {
          this.constructor = ne;
        }
        ne.prototype = ee === null ? Object.create(ee) : (ae.prototype = ee.prototype, new ae());
      };
    }(), _ = MathtoolsTags.__values || function(te) {
      var ne = typeof Symbol == "function" && Symbol.iterator, ee = ne && te[ne], ae = 0;
      if (ee) return ee.call(te);
      if (te && typeof te.length == "number") return {
        next: function() {
          return te && ae >= te.length && (te = void 0), { value: te && te[ae++], done: !te };
        }
      };
      throw new TypeError(ne ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = MathtoolsTags.__read || function(te, ne) {
      var ee = typeof Symbol == "function" && te[Symbol.iterator];
      if (!ee) return te;
      var ae = ee.call(te), oe, se = [], ce;
      try {
        for (; (ne === void 0 || ne-- > 0) && !(oe = ae.next()).done; ) se.push(oe.value);
      } catch (le) {
        ce = { error: le };
      } finally {
        try {
          oe && !oe.done && (ee = ae.return) && ee.call(ae);
        } finally {
          if (ce) throw ce.error;
        }
      }
      return se;
    }, V = MathtoolsTags.__importDefault || function(te) {
      return te && te.__esModule ? te : { default: te };
    };
    Object.defineProperty(MathtoolsTags, "__esModule", { value: !0 }), MathtoolsTags.MathtoolsTagFormat = void 0;
    var q = V(requireTexError()), G = requireTags(), J = 0;
    function Z(te, ne) {
      var ee = ne.parseOptions.options.tags;
      ee !== "base" && te.tags.hasOwnProperty(ee) && G.TagsFactory.add(ee, te.tags[ee]);
      var ae = G.TagsFactory.create(ne.parseOptions.options.tags).constructor, oe = function(ce) {
        O(le, ce);
        function le() {
          var ue, de, fe = ce.call(this) || this;
          fe.mtFormats = /* @__PURE__ */ new Map(), fe.mtCurrent = null;
          var he = ne.parseOptions.options.mathtools.tagforms;
          try {
            for (var me = _(Object.keys(he)), pe = me.next(); !pe.done; pe = me.next()) {
              var ye = pe.value;
              if (!Array.isArray(he[ye]) || he[ye].length !== 3)
                throw new q.default("InvalidTagFormDef", 'The tag form definition for "%1" should be an array fo three strings', ye);
              fe.mtFormats.set(ye, he[ye]);
            }
          } catch (ge) {
            ue = { error: ge };
          } finally {
            try {
              pe && !pe.done && (de = me.return) && de.call(me);
            } finally {
              if (ue) throw ue.error;
            }
          }
          return fe;
        }
        return le.prototype.formatTag = function(ue) {
          if (this.mtCurrent) {
            var de = H(this.mtCurrent, 3), fe = de[0], he = de[1], me = de[2];
            return me ? "".concat(fe).concat(me, "{").concat(ue, "}").concat(he) : "".concat(fe).concat(ue).concat(he);
          }
          return ce.prototype.formatTag.call(this, ue);
        }, le;
      }(ae);
      J++;
      var se = "MathtoolsTags-" + J;
      G.TagsFactory.add(se, oe), ne.parseOptions.options.tags = se;
    }
    return MathtoolsTags.MathtoolsTagFormat = Z, MathtoolsTags;
  }
  var MathtoolsItems = {}, hasRequiredMathtoolsItems;
  function requireMathtoolsItems() {
    if (hasRequiredMathtoolsItems) return MathtoolsItems;
    hasRequiredMathtoolsItems = 1;
    var O = MathtoolsItems.__extends || /* @__PURE__ */ function() {
      var J = function(Z, te) {
        return J = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ne, ee) {
          ne.__proto__ = ee;
        } || function(ne, ee) {
          for (var ae in ee) Object.prototype.hasOwnProperty.call(ee, ae) && (ne[ae] = ee[ae]);
        }, J(Z, te);
      };
      return function(Z, te) {
        if (typeof te != "function" && te !== null)
          throw new TypeError("Class extends value " + String(te) + " is not a constructor or null");
        J(Z, te);
        function ne() {
          this.constructor = Z;
        }
        Z.prototype = te === null ? Object.create(te) : (ne.prototype = te.prototype, new ne());
      };
    }(), _ = MathtoolsItems.__importDefault || function(J) {
      return J && J.__esModule ? J : { default: J };
    };
    Object.defineProperty(MathtoolsItems, "__esModule", { value: !0 }), MathtoolsItems.MultlinedItem = void 0;
    var H = requireAmsItems(), V = _(requireNodeUtil()), q = requireTexConstants(), G = function(J) {
      O(Z, J);
      function Z() {
        return J !== null && J.apply(this, arguments) || this;
      }
      return Object.defineProperty(Z.prototype, "kind", {
        get: function() {
          return "multlined";
        },
        enumerable: !1,
        configurable: !0
      }), Z.prototype.EndTable = function() {
        if ((this.Size() || this.row.length) && (this.EndEntry(), this.EndRow()), this.table.length > 1) {
          var te = this.factory.configuration.options.mathtools, ne = te.multlinegap, ee = te["firstline-afterskip"] || ne, ae = te["lastline-preskip"] || ne, oe = V.default.getChildren(this.table[0])[0];
          V.default.getAttribute(oe, "columnalign") !== q.TexConstant.Align.RIGHT && oe.appendChild(this.create("node", "mspace", [], { width: ee }));
          var se = V.default.getChildren(this.table[this.table.length - 1])[0];
          if (V.default.getAttribute(se, "columnalign") !== q.TexConstant.Align.LEFT) {
            var ce = V.default.getChildren(se)[0];
            ce.childNodes.unshift(null);
            var le = this.create("node", "mspace", [], { width: ae });
            V.default.setChild(ce, 0, le);
          }
        }
        J.prototype.EndTable.call(this);
      }, Z;
    }(H.MultlineItem);
    return MathtoolsItems.MultlinedItem = G, MathtoolsItems;
  }
  var hasRequiredMathtoolsConfiguration;
  function requireMathtoolsConfiguration() {
    return hasRequiredMathtoolsConfiguration || (hasRequiredMathtoolsConfiguration = 1, function(O) {
      var _ = MathtoolsConfiguration.__values || function(ce) {
        var le = typeof Symbol == "function" && Symbol.iterator, ue = le && ce[le], de = 0;
        if (ue) return ue.call(ce);
        if (ce && typeof ce.length == "number") return {
          next: function() {
            return ce && de >= ce.length && (ce = void 0), { value: ce && ce[de++], done: !ce };
          }
        };
        throw new TypeError(le ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, H = MathtoolsConfiguration.__importDefault || function(ce) {
        return ce && ce.__esModule ? ce : { default: ce };
      }, V;
      Object.defineProperty(O, "__esModule", { value: !0 }), O.MathtoolsConfiguration = O.fixPrescripts = O.PAIREDDELIMS = void 0;
      var q = requireConfiguration(), G = requireSymbolMap(), J = H(requireNodeUtil()), Z = requireOptions();
      requireMathtoolsMappings();
      var te = requireMathtoolsUtil(), ne = requireMathtoolsTags(), ee = requireMathtoolsItems();
      O.PAIREDDELIMS = "mathtools-paired-delims";
      function ae(ce) {
        new G.CommandMap(O.PAIREDDELIMS, {}, {}), ce.append(q.Configuration.local({ handler: { macro: [O.PAIREDDELIMS] }, priority: -5 }));
      }
      function oe(ce, le) {
        var ue, de, fe = le.parseOptions, he = fe.options.mathtools.pairedDelimiters;
        try {
          for (var me = _(Object.keys(he)), pe = me.next(); !pe.done; pe = me.next()) {
            var ye = pe.value;
            te.MathtoolsUtil.addPairedDelims(fe, ye, he[ye]);
          }
        } catch (ge) {
          ue = { error: ge };
        } finally {
          try {
            pe && !pe.done && (de = me.return) && de.call(me);
          } finally {
            if (ue) throw ue.error;
          }
        }
        (0, ne.MathtoolsTagFormat)(ce, le);
      }
      function se(ce) {
        var le, ue, de, fe, he, me, pe = ce.data;
        try {
          for (var ye = _(pe.getList("mmultiscripts")), ge = ye.next(); !ge.done; ge = ye.next()) {
            var ve = ge.value;
            if (ve.getProperty("fixPrescript")) {
              var be = J.default.getChildren(ve), Ce = 0;
              try {
                for (var Se = (de = void 0, _([1, 2])), we = Se.next(); !we.done; we = Se.next()) {
                  var Ee = we.value;
                  be[Ee] || (J.default.setChild(ve, Ee, pe.nodeFactory.create("node", "none")), Ce++);
                }
              } catch (Qe) {
                de = { error: Qe };
              } finally {
                try {
                  we && !we.done && (fe = Se.return) && fe.call(Se);
                } finally {
                  if (de) throw de.error;
                }
              }
              try {
                for (var xe = (he = void 0, _([4, 5])), Le = xe.next(); !Le.done; Le = xe.next()) {
                  var Ee = Le.value;
                  J.default.isType(be[Ee], "mrow") && J.default.getChildren(be[Ee]).length === 0 && J.default.setChild(ve, Ee, pe.nodeFactory.create("node", "none"));
                }
              } catch (Qe) {
                he = { error: Qe };
              } finally {
                try {
                  Le && !Le.done && (me = xe.return) && me.call(xe);
                } finally {
                  if (he) throw he.error;
                }
              }
              Ce === 2 && be.splice(1, 2);
            }
          }
        } catch (Qe) {
          le = { error: Qe };
        } finally {
          try {
            ge && !ge.done && (ue = ye.return) && ue.call(ye);
          } finally {
            if (le) throw le.error;
          }
        }
      }
      O.fixPrescripts = se, O.MathtoolsConfiguration = q.Configuration.create("mathtools", {
        handler: {
          macro: ["mathtools-macros", "mathtools-delimiters"],
          environment: ["mathtools-environments"],
          delimiter: ["mathtools-delimiters"],
          character: ["mathtools-characters"]
        },
        items: (V = {}, V[ee.MultlinedItem.prototype.kind] = ee.MultlinedItem, V),
        init: ae,
        config: oe,
        postprocessors: [[se, -6]],
        options: {
          mathtools: {
            multlinegap: "1em",
            "multlined-pos": "c",
            "firstline-afterskip": "",
            "lastline-preskip": "",
            "smallmatrix-align": "c",
            shortvdotsadjustabove: ".2em",
            shortvdotsadjustbelow: ".2em",
            centercolon: !1,
            "centercolon-offset": ".04em",
            "thincolon-dx": "-.04em",
            "thincolon-dw": "-.08em",
            "use-unicode": !1,
            "prescript-sub-format": "",
            "prescript-sup-format": "",
            "prescript-arg-format": "",
            "allow-mathtoolsset": !0,
            pairedDelimiters: (0, Z.expandable)({}),
            tagforms: (0, Z.expandable)({})
          }
        }
      });
    }(MathtoolsConfiguration)), MathtoolsConfiguration;
  }
  var MhchemConfiguration = {}, mhchemParser = {}, hasRequiredMhchemParser;
  function requireMhchemParser() {
    if (hasRequiredMhchemParser) return mhchemParser;
    hasRequiredMhchemParser = 1;
    /*!
     *************************************************************************
     *
     *  mhchemParser.ts
     *  4.2.1
     *
     *  Parser for the \ce command and \pu command for MathJax and Co.
     *
     *  mhchem's \ce is a tool for writing beautiful chemical equations easily.
     *  mhchem's \pu is a tool for writing physical units easily.
     *
     *  ----------------------------------------------------------------------
     *
     *  Copyright (c) 2015-2023 Martin Hensel
     *
     *  Licensed under the Apache License, Version 2.0 (the "License");
     *  you may not use this file except in compliance with the License.
     *  You may obtain a copy of the License at
     *
     *      http://www.apache.org/licenses/LICENSE-2.0
     *
     *  Unless required by applicable law or agreed to in writing, software
     *  distributed under the License is distributed on an "AS IS" BASIS,
     *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     *  See the License for the specific language governing permissions and
     *  limitations under the License.
     *
     *  ----------------------------------------------------------------------
     *
     *  https://github.com/mhchem/mhchemParser
     *
     */
    Object.defineProperty(mhchemParser, "__esModule", { value: !0 }), mhchemParser.mhchemParser = void 0;
    var O = function() {
      function q() {
      }
      return q.toTex = function(G, J) {
        return V.go(H.go(G, J), J !== "tex");
      }, q;
    }();
    mhchemParser.mhchemParser = O;
    function _(q) {
      var G, J, Z = {};
      for (G in q)
        for (J in q[G]) {
          var te = J.split("|");
          q[G][J].stateArray = te;
          for (var ne = 0; ne < te.length; ne++)
            Z[te[ne]] = [];
        }
      for (G in q)
        for (J in q[G])
          for (var te = q[G][J].stateArray || [], ne = 0; ne < te.length; ne++) {
            var ee = q[G][J];
            ee.action_ = [].concat(ee.action_);
            for (var ae = 0; ae < ee.action_.length; ae++)
              typeof ee.action_[ae] == "string" && (ee.action_[ae] = { type_: ee.action_[ae] });
            for (var oe = G.split("|"), se = 0; se < oe.length; se++)
              if (te[ne] === "*") {
                var ce = void 0;
                for (ce in Z)
                  Z[ce].push({ pattern: oe[se], task: ee });
              } else
                Z[te[ne]].push({ pattern: oe[se], task: ee });
          }
      return Z;
    }
    var H = {
      go: function(q, G) {
        if (!q)
          return [];
        G === void 0 && (G = "ce");
        var J = "0", Z = {};
        Z.parenthesisLevel = 0, q = q.replace(/\n/g, " "), q = q.replace(/[\u2212\u2013\u2014\u2010]/g, "-"), q = q.replace(/[\u2026]/g, "...");
        for (var te, ne = 10, ee = []; ; ) {
          te !== q ? (ne = 10, te = q) : ne--;
          var ae = H.stateMachines[G], oe = ae.transitions[J] || ae.transitions["*"];
          e: for (var se = 0; se < oe.length; se++) {
            var ce = H.patterns.match_(oe[se].pattern, q);
            if (ce) {
              for (var le = oe[se].task, ue = 0; ue < le.action_.length; ue++) {
                var de = void 0;
                if (ae.actions[le.action_[ue].type_])
                  de = ae.actions[le.action_[ue].type_](Z, ce.match_, le.action_[ue].option);
                else if (H.actions[le.action_[ue].type_])
                  de = H.actions[le.action_[ue].type_](Z, ce.match_, le.action_[ue].option);
                else
                  throw ["MhchemBugA", "mhchem bug A. Please report. (" + le.action_[ue].type_ + ")"];
                H.concatArray(ee, de);
              }
              if (J = le.nextState || J, q.length > 0) {
                if (le.revisit || (q = ce.remainder), !le.toContinue)
                  break e;
              } else
                return ee;
            }
          }
          if (ne <= 0)
            throw ["MhchemBugU", "mhchem bug U. Please report."];
        }
      },
      concatArray: function(q, G) {
        if (G)
          if (Array.isArray(G))
            for (var J = 0; J < G.length; J++)
              q.push(G[J]);
          else
            q.push(G);
      },
      patterns: {
        patterns: {
          empty: /^$/,
          else: /^./,
          else2: /^./,
          space: /^\s/,
          "space A": /^\s(?=[A-Z\\$])/,
          space$: /^\s$/,
          "a-z": /^[a-z]/,
          x: /^x/,
          x$: /^x$/,
          i$: /^i$/,
          letters: /^(?:[a-zA-Z\u03B1-\u03C9\u0391-\u03A9?@]|(?:\\(?:alpha|beta|gamma|delta|epsilon|zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|omicron|pi|rho|sigma|tau|upsilon|phi|chi|psi|omega|Gamma|Delta|Theta|Lambda|Xi|Pi|Sigma|Upsilon|Phi|Psi|Omega)(?:\s+|\{\}|(?![a-zA-Z]))))+/,
          "\\greek": /^\\(?:alpha|beta|gamma|delta|epsilon|zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|omicron|pi|rho|sigma|tau|upsilon|phi|chi|psi|omega|Gamma|Delta|Theta|Lambda|Xi|Pi|Sigma|Upsilon|Phi|Psi|Omega)(?:\s+|\{\}|(?![a-zA-Z]))/,
          "one lowercase latin letter $": /^(?:([a-z])(?:$|[^a-zA-Z]))$/,
          "$one lowercase latin letter$ $": /^\$(?:([a-z])(?:$|[^a-zA-Z]))\$$/,
          "one lowercase greek letter $": /^(?:\$?[\u03B1-\u03C9]\$?|\$?\\(?:alpha|beta|gamma|delta|epsilon|zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|omicron|pi|rho|sigma|tau|upsilon|phi|chi|psi|omega)\s*\$?)(?:\s+|\{\}|(?![a-zA-Z]))$/,
          digits: /^[0-9]+/,
          "-9.,9": /^[+\-]?(?:[0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+))/,
          "-9.,9 no missing 0": /^[+\-]?[0-9]+(?:[.,][0-9]+)?/,
          "(-)(9.,9)(e)(99)": function(q) {
            var G = q.match(/^(\+\-|\+\/\-|\+|\-|\\pm\s?)?([0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+))?(\((?:[0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+))\))?(?:(?:([eE])|\s*(\*|x|\\times|\u00D7)\s*10\^)([+\-]?[0-9]+|\{[+\-]?[0-9]+\}))?/);
            return G && G[0] ? { match_: G.slice(1), remainder: q.substr(G[0].length) } : null;
          },
          "(-)(9)^(-9)": /^(\+\-|\+\/\-|\+|\-|\\pm\s?)?([0-9]+(?:[,.][0-9]+)?|[0-9]*(?:\.[0-9]+)?)\^([+\-]?[0-9]+|\{[+\-]?[0-9]+\})/,
          "state of aggregation $": function(q) {
            var G = H.patterns.findObserveGroups(q, "", /^\([a-z]{1,3}(?=[\),])/, ")", "");
            if (G && G.remainder.match(/^($|[\s,;\)\]\}])/))
              return G;
            var J = q.match(/^(?:\((?:\\ca\s?)?\$[amothc]\$\))/);
            return J ? { match_: J[0], remainder: q.substr(J[0].length) } : null;
          },
          "_{(state of aggregation)}$": /^_\{(\([a-z]{1,3}\))\}/,
          "{[(": /^(?:\\\{|\[|\()/,
          ")]}": /^(?:\)|\]|\\\})/,
          ", ": /^[,;]\s*/,
          ",": /^[,;]/,
          ".": /^[.]/,
          ". __* ": /^([.\u22C5\u00B7\u2022]|[*])\s*/,
          "...": /^\.\.\.(?=$|[^.])/,
          "^{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "^{", "", "", "}");
          },
          "^($...$)": function(q) {
            return H.patterns.findObserveGroups(q, "^", "$", "$", "");
          },
          "^a": /^\^([0-9]+|[^\\_])/,
          "^\\x{}{}": function(q) {
            return H.patterns.findObserveGroups(q, "^", /^\\[a-zA-Z]+\{/, "}", "", "", "{", "}", "", !0);
          },
          "^\\x{}": function(q) {
            return H.patterns.findObserveGroups(q, "^", /^\\[a-zA-Z]+\{/, "}", "");
          },
          "^\\x": /^\^(\\[a-zA-Z]+)\s*/,
          "^(-1)": /^\^(-?\d+)/,
          "'": /^'/,
          "_{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "_{", "", "", "}");
          },
          "_($...$)": function(q) {
            return H.patterns.findObserveGroups(q, "_", "$", "$", "");
          },
          _9: /^_([+\-]?[0-9]+|[^\\])/,
          "_\\x{}{}": function(q) {
            return H.patterns.findObserveGroups(q, "_", /^\\[a-zA-Z]+\{/, "}", "", "", "{", "}", "", !0);
          },
          "_\\x{}": function(q) {
            return H.patterns.findObserveGroups(q, "_", /^\\[a-zA-Z]+\{/, "}", "");
          },
          "_\\x": /^_(\\[a-zA-Z]+)\s*/,
          "^_": /^(?:\^(?=_)|\_(?=\^)|[\^_]$)/,
          "{}^": /^\{\}(?=\^)/,
          "{}": /^\{\}/,
          "{...}": function(q) {
            return H.patterns.findObserveGroups(q, "", "{", "}", "");
          },
          "{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "{", "", "", "}");
          },
          "$...$": function(q) {
            return H.patterns.findObserveGroups(q, "", "$", "$", "");
          },
          "${(...)}$__$(...)$": function(q) {
            return H.patterns.findObserveGroups(q, "${", "", "", "}$") || H.patterns.findObserveGroups(q, "$", "", "", "$");
          },
          "=<>": /^[=<>]/,
          "#": /^[#\u2261]/,
          "+": /^\+/,
          "-$": /^-(?=[\s_},;\]/]|$|\([a-z]+\))/,
          "-9": /^-(?=[0-9])/,
          "- orbital overlap": /^-(?=(?:[spd]|sp)(?:$|[\s,;\)\]\}]))/,
          "-": /^-/,
          "pm-operator": /^(?:\\pm|\$\\pm\$|\+-|\+\/-)/,
          operator: /^(?:\+|(?:[\-=<>]|<<|>>|\\approx|\$\\approx\$)(?=\s|$|-?[0-9]))/,
          arrowUpDown: /^(?:v|\(v\)|\^|\(\^\))(?=$|[\s,;\)\]\}])/,
          "\\bond{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\bond{", "", "", "}");
          },
          "->": /^(?:<->|<-->|->|<-|<=>>|<<=>|<=>|[\u2192\u27F6\u21CC])/,
          CMT: /^[CMT](?=\[)/,
          "[(...)]": function(q) {
            return H.patterns.findObserveGroups(q, "[", "", "", "]");
          },
          "1st-level escape": /^(&|\\\\|\\hline)\s*/,
          "\\,": /^(?:\\[,\ ;:])/,
          "\\x{}{}": function(q) {
            return H.patterns.findObserveGroups(q, "", /^\\[a-zA-Z]+\{/, "}", "", "", "{", "}", "", !0);
          },
          "\\x{}": function(q) {
            return H.patterns.findObserveGroups(q, "", /^\\[a-zA-Z]+\{/, "}", "");
          },
          "\\ca": /^\\ca(?:\s+|(?![a-zA-Z]))/,
          "\\x": /^(?:\\[a-zA-Z]+\s*|\\[_&{}%])/,
          orbital: /^(?:[0-9]{1,2}[spdfgh]|[0-9]{0,2}sp)(?=$|[^a-zA-Z])/,
          others: /^[\/~|]/,
          "\\frac{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\frac{", "", "", "}", "{", "", "", "}");
          },
          "\\overset{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\overset{", "", "", "}", "{", "", "", "}");
          },
          "\\underset{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\underset{", "", "", "}", "{", "", "", "}");
          },
          "\\underbrace{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\underbrace{", "", "", "}_", "{", "", "", "}");
          },
          "\\color{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\color{", "", "", "}");
          },
          "\\color{(...)}{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\color{", "", "", "}", "{", "", "", "}") || H.patterns.findObserveGroups(q, "\\color", "\\", "", /^(?=\{)/, "{", "", "", "}");
          },
          "\\ce{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\ce{", "", "", "}");
          },
          "\\pu{(...)}": function(q) {
            return H.patterns.findObserveGroups(q, "\\pu{", "", "", "}");
          },
          oxidation$: /^(?:[+-][IVX]+|(?:\\pm|\$\\pm\$|\+-|\+\/-)\s*0)$/,
          "d-oxidation$": /^(?:[+-]?[IVX]+|(?:\\pm|\$\\pm\$|\+-|\+\/-)\s*0)$/,
          "1/2$": /^[+\-]?(?:[0-9]+|\$[a-z]\$|[a-z])\/[0-9]+(?:\$[a-z]\$|[a-z])?$/,
          amount: function(q) {
            var G;
            if (G = q.match(/^(?:(?:(?:\([+\-]?[0-9]+\/[0-9]+\)|[+\-]?(?:[0-9]+|\$[a-z]\$|[a-z])\/[0-9]+|[+\-]?[0-9]+[.,][0-9]+|[+\-]?\.[0-9]+|[+\-]?[0-9]+)(?:[a-z](?=\s*[A-Z]))?)|[+\-]?[a-z](?=\s*[A-Z])|\+(?!\s))/), G)
              return { match_: G[0], remainder: q.substr(G[0].length) };
            var J = H.patterns.findObserveGroups(q, "", "$", "$", "");
            return J && (G = J.match_.match(/^\$(?:\(?[+\-]?(?:[0-9]*[a-z]?[+\-])?[0-9]*[a-z](?:[+\-][0-9]*[a-z]?)?\)?|\+|-)\$$/), G) ? { match_: G[0], remainder: q.substr(G[0].length) } : null;
          },
          amount2: function(q) {
            return this.amount(q);
          },
          "(KV letters),": /^(?:[A-Z][a-z]{0,2}|i)(?=,)/,
          formula$: function(q) {
            if (q.match(/^\([a-z]+\)$/))
              return null;
            var G = q.match(/^(?:[a-z]|(?:[0-9\ \+\-\,\.\(\)]+[a-z])+[0-9\ \+\-\,\.\(\)]*|(?:[a-z][0-9\ \+\-\,\.\(\)]+)+[a-z]?)$/);
            return G ? { match_: G[0], remainder: q.substr(G[0].length) } : null;
          },
          uprightEntities: /^(?:pH|pOH|pC|pK|iPr|iBu)(?=$|[^a-zA-Z])/,
          "/": /^\s*(\/)\s*/,
          "//": /^\s*(\/\/)\s*/,
          "*": /^\s*[*.]\s*/
        },
        findObserveGroups: function(q, G, J, Z, te, ne, ee, ae, oe, se) {
          var ce = function(pe, ye) {
            if (typeof ye == "string")
              return pe.indexOf(ye) !== 0 ? null : ye;
            var ge = pe.match(ye);
            return ge ? ge[0] : null;
          }, le = function(pe, ye, ge) {
            for (var ve = 0; ye < pe.length; ) {
              var be = pe.charAt(ye), Ce = ce(pe.substr(ye), ge);
              if (Ce !== null && ve === 0)
                return { endMatchBegin: ye, endMatchEnd: ye + Ce.length };
              if (be === "{")
                ve++;
              else if (be === "}") {
                if (ve === 0)
                  throw ["ExtraCloseMissingOpen", "Extra close brace or missing open brace"];
                ve--;
              }
              ye++;
            }
            return ve > 0, null;
          }, ue = ce(q, G);
          if (ue === null || (q = q.substr(ue.length), ue = ce(q, J), ue === null))
            return null;
          var de = le(q, ue.length, Z || te);
          if (de === null)
            return null;
          var fe = q.substring(0, Z ? de.endMatchEnd : de.endMatchBegin);
          if (ne || ee) {
            var he = this.findObserveGroups(q.substr(de.endMatchEnd), ne, ee, ae, oe);
            if (he === null)
              return null;
            var me = [fe, he.match_];
            return {
              match_: se ? me.join("") : me,
              remainder: he.remainder
            };
          } else
            return {
              match_: fe,
              remainder: q.substr(de.endMatchEnd)
            };
        },
        match_: function(q, G) {
          var J = H.patterns.patterns[q];
          if (J === void 0)
            throw ["MhchemBugP", "mhchem bug P. Please report. (" + q + ")"];
          if (typeof J == "function")
            return H.patterns.patterns[q](G);
          var Z = G.match(J);
          return Z ? Z.length > 2 ? { match_: Z.slice(1), remainder: G.substr(Z[0].length) } : { match_: Z[1] || Z[0], remainder: G.substr(Z[0].length) } : null;
        }
      },
      actions: {
        "a=": function(q, G) {
          q.a = (q.a || "") + G;
        },
        "b=": function(q, G) {
          q.b = (q.b || "") + G;
        },
        "p=": function(q, G) {
          q.p = (q.p || "") + G;
        },
        "o=": function(q, G) {
          q.o = (q.o || "") + G;
        },
        "o=+p1": function(q, G, J) {
          q.o = (q.o || "") + J;
        },
        "q=": function(q, G) {
          q.q = (q.q || "") + G;
        },
        "d=": function(q, G) {
          q.d = (q.d || "") + G;
        },
        "rm=": function(q, G) {
          q.rm = (q.rm || "") + G;
        },
        "text=": function(q, G) {
          q.text_ = (q.text_ || "") + G;
        },
        insert: function(q, G, J) {
          return { type_: J };
        },
        "insert+p1": function(q, G, J) {
          return { type_: J, p1: G };
        },
        "insert+p1+p2": function(q, G, J) {
          return { type_: J, p1: G[0], p2: G[1] };
        },
        copy: function(q, G) {
          return G;
        },
        write: function(q, G, J) {
          return J;
        },
        rm: function(q, G) {
          return { type_: "rm", p1: G };
        },
        text: function(q, G) {
          return H.go(G, "text");
        },
        "tex-math": function(q, G) {
          return H.go(G, "tex-math");
        },
        "tex-math tight": function(q, G) {
          return H.go(G, "tex-math tight");
        },
        bond: function(q, G, J) {
          return { type_: "bond", kind_: J || G };
        },
        "color0-output": function(q, G) {
          return { type_: "color0", color: G };
        },
        ce: function(q, G) {
          return H.go(G, "ce");
        },
        pu: function(q, G) {
          return H.go(G, "pu");
        },
        "1/2": function(q, G) {
          var J = [];
          G.match(/^[+\-]/) && (J.push(G.substr(0, 1)), G = G.substr(1));
          var Z = G.match(/^([0-9]+|\$[a-z]\$|[a-z])\/([0-9]+)(\$[a-z]\$|[a-z])?$/);
          return Z[1] = Z[1].replace(/\$/g, ""), J.push({ type_: "frac", p1: Z[1], p2: Z[2] }), Z[3] && (Z[3] = Z[3].replace(/\$/g, ""), J.push({ type_: "tex-math", p1: Z[3] })), J;
        },
        "9,9": function(q, G) {
          return H.go(G, "9,9");
        }
      },
      stateMachines: {
        tex: {
          transitions: _({
            empty: {
              0: { action_: "copy" }
            },
            "\\ce{(...)}": {
              0: { action_: [{ type_: "write", option: "{" }, "ce", { type_: "write", option: "}" }] }
            },
            "\\pu{(...)}": {
              0: { action_: [{ type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }] }
            },
            else: {
              0: { action_: "copy" }
            }
          }),
          actions: {}
        },
        ce: {
          transitions: _({
            empty: {
              "*": { action_: "output" }
            },
            else: {
              "0|1|2": { action_: "beginsWithBond=false", revisit: !0, toContinue: !0 }
            },
            oxidation$: {
              0: { action_: "oxidation-output" }
            },
            CMT: {
              r: { action_: "rdt=", nextState: "rt" },
              rd: { action_: "rqt=", nextState: "rdt" }
            },
            arrowUpDown: {
              "0|1|2|as": { action_: ["sb=false", "output", "operator"], nextState: "1" }
            },
            uprightEntities: {
              "0|1|2": { action_: ["o=", "output"], nextState: "1" }
            },
            orbital: {
              "0|1|2|3": { action_: "o=", nextState: "o" }
            },
            "->": {
              "0|1|2|3": { action_: "r=", nextState: "r" },
              "a|as": { action_: ["output", "r="], nextState: "r" },
              "*": { action_: ["output", "r="], nextState: "r" }
            },
            "+": {
              o: { action_: "d= kv", nextState: "d" },
              "d|D": { action_: "d=", nextState: "d" },
              q: { action_: "d=", nextState: "qd" },
              "qd|qD": { action_: "d=", nextState: "qd" },
              dq: { action_: ["output", "d="], nextState: "d" },
              3: { action_: ["sb=false", "output", "operator"], nextState: "0" }
            },
            amount: {
              "0|2": { action_: "a=", nextState: "a" }
            },
            "pm-operator": {
              "0|1|2|a|as": { action_: ["sb=false", "output", { type_: "operator", option: "\\pm" }], nextState: "0" }
            },
            operator: {
              "0|1|2|a|as": { action_: ["sb=false", "output", "operator"], nextState: "0" }
            },
            "-$": {
              "o|q": { action_: ["charge or bond", "output"], nextState: "qd" },
              d: { action_: "d=", nextState: "d" },
              D: { action_: ["output", { type_: "bond", option: "-" }], nextState: "3" },
              q: { action_: "d=", nextState: "qd" },
              qd: { action_: "d=", nextState: "qd" },
              "qD|dq": { action_: ["output", { type_: "bond", option: "-" }], nextState: "3" }
            },
            "-9": {
              "3|o": { action_: ["output", { type_: "insert", option: "hyphen" }], nextState: "3" }
            },
            "- orbital overlap": {
              o: { action_: ["output", { type_: "insert", option: "hyphen" }], nextState: "2" },
              d: { action_: ["output", { type_: "insert", option: "hyphen" }], nextState: "2" }
            },
            "-": {
              "0|1|2": { action_: [{ type_: "output", option: 1 }, "beginsWithBond=true", { type_: "bond", option: "-" }], nextState: "3" },
              3: { action_: { type_: "bond", option: "-" } },
              a: { action_: ["output", { type_: "insert", option: "hyphen" }], nextState: "2" },
              as: { action_: [{ type_: "output", option: 2 }, { type_: "bond", option: "-" }], nextState: "3" },
              b: { action_: "b=" },
              o: { action_: { type_: "- after o/d", option: !1 }, nextState: "2" },
              q: { action_: { type_: "- after o/d", option: !1 }, nextState: "2" },
              "d|qd|dq": { action_: { type_: "- after o/d", option: !0 }, nextState: "2" },
              "D|qD|p": { action_: ["output", { type_: "bond", option: "-" }], nextState: "3" }
            },
            amount2: {
              "1|3": { action_: "a=", nextState: "a" }
            },
            letters: {
              "0|1|2|3|a|as|b|p|bp|o": { action_: "o=", nextState: "o" },
              "q|dq": { action_: ["output", "o="], nextState: "o" },
              "d|D|qd|qD": { action_: "o after d", nextState: "o" }
            },
            digits: {
              o: { action_: "q=", nextState: "q" },
              "d|D": { action_: "q=", nextState: "dq" },
              q: { action_: ["output", "o="], nextState: "o" },
              a: { action_: "o=", nextState: "o" }
            },
            "space A": {
              "b|p|bp": { action_: [] }
            },
            space: {
              a: { action_: [], nextState: "as" },
              0: { action_: "sb=false" },
              "1|2": { action_: "sb=true" },
              "r|rt|rd|rdt|rdq": { action_: "output", nextState: "0" },
              "*": { action_: ["output", "sb=true"], nextState: "1" }
            },
            "1st-level escape": {
              "1|2": { action_: ["output", { type_: "insert+p1", option: "1st-level escape" }] },
              "*": { action_: ["output", { type_: "insert+p1", option: "1st-level escape" }], nextState: "0" }
            },
            "[(...)]": {
              "r|rt": { action_: "rd=", nextState: "rd" },
              "rd|rdt": { action_: "rq=", nextState: "rdq" }
            },
            "...": {
              "o|d|D|dq|qd|qD": { action_: ["output", { type_: "bond", option: "..." }], nextState: "3" },
              "*": { action_: [{ type_: "output", option: 1 }, { type_: "insert", option: "ellipsis" }], nextState: "1" }
            },
            ". __* ": {
              "*": { action_: ["output", { type_: "insert", option: "addition compound" }], nextState: "1" }
            },
            "state of aggregation $": {
              "*": { action_: ["output", "state of aggregation"], nextState: "1" }
            },
            "{[(": {
              "a|as|o": { action_: ["o=", "output", "parenthesisLevel++"], nextState: "2" },
              "0|1|2|3": { action_: ["o=", "output", "parenthesisLevel++"], nextState: "2" },
              "*": { action_: ["output", "o=", "output", "parenthesisLevel++"], nextState: "2" }
            },
            ")]}": {
              "0|1|2|3|b|p|bp|o": { action_: ["o=", "parenthesisLevel--"], nextState: "o" },
              "a|as|d|D|q|qd|qD|dq": { action_: ["output", "o=", "parenthesisLevel--"], nextState: "o" }
            },
            ", ": {
              "*": { action_: ["output", "comma"], nextState: "0" }
            },
            "^_": {
              "*": { action_: [] }
            },
            "^{(...)}|^($...$)": {
              "0|1|2|as": { action_: "b=", nextState: "b" },
              p: { action_: "b=", nextState: "bp" },
              "3|o": { action_: "d= kv", nextState: "D" },
              q: { action_: "d=", nextState: "qD" },
              "d|D|qd|qD|dq": { action_: ["output", "d="], nextState: "D" }
            },
            "^a|^\\x{}{}|^\\x{}|^\\x|'": {
              "0|1|2|as": { action_: "b=", nextState: "b" },
              p: { action_: "b=", nextState: "bp" },
              "3|o": { action_: "d= kv", nextState: "d" },
              q: { action_: "d=", nextState: "qd" },
              "d|qd|D|qD": { action_: "d=" },
              dq: { action_: ["output", "d="], nextState: "d" }
            },
            "_{(state of aggregation)}$": {
              "d|D|q|qd|qD|dq": { action_: ["output", "q="], nextState: "q" }
            },
            "_{(...)}|_($...$)|_9|_\\x{}{}|_\\x{}|_\\x": {
              "0|1|2|as": { action_: "p=", nextState: "p" },
              b: { action_: "p=", nextState: "bp" },
              "3|o": { action_: "q=", nextState: "q" },
              "d|D": { action_: "q=", nextState: "dq" },
              "q|qd|qD|dq": { action_: ["output", "q="], nextState: "q" }
            },
            "=<>": {
              "0|1|2|3|a|as|o|q|d|D|qd|qD|dq": { action_: [{ type_: "output", option: 2 }, "bond"], nextState: "3" }
            },
            "#": {
              "0|1|2|3|a|as|o": { action_: [{ type_: "output", option: 2 }, { type_: "bond", option: "#" }], nextState: "3" }
            },
            "{}^": {
              "*": { action_: [{ type_: "output", option: 1 }, { type_: "insert", option: "tinySkip" }], nextState: "1" }
            },
            "{}": {
              "*": { action_: { type_: "output", option: 1 }, nextState: "1" }
            },
            "{...}": {
              "0|1|2|3|a|as|b|p|bp": { action_: "o=", nextState: "o" },
              "o|d|D|q|qd|qD|dq": { action_: ["output", "o="], nextState: "o" }
            },
            "$...$": {
              a: { action_: "a=" },
              "0|1|2|3|as|b|p|bp|o": { action_: "o=", nextState: "o" },
              "as|o": { action_: "o=" },
              "q|d|D|qd|qD|dq": { action_: ["output", "o="], nextState: "o" }
            },
            "\\bond{(...)}": {
              "*": { action_: [{ type_: "output", option: 2 }, "bond"], nextState: "3" }
            },
            "\\frac{(...)}": {
              "*": { action_: [{ type_: "output", option: 1 }, "frac-output"], nextState: "3" }
            },
            "\\overset{(...)}": {
              "*": { action_: [{ type_: "output", option: 2 }, "overset-output"], nextState: "3" }
            },
            "\\underset{(...)}": {
              "*": { action_: [{ type_: "output", option: 2 }, "underset-output"], nextState: "3" }
            },
            "\\underbrace{(...)}": {
              "*": { action_: [{ type_: "output", option: 2 }, "underbrace-output"], nextState: "3" }
            },
            "\\color{(...)}{(...)}": {
              "*": { action_: [{ type_: "output", option: 2 }, "color-output"], nextState: "3" }
            },
            "\\color{(...)}": {
              "*": { action_: [{ type_: "output", option: 2 }, "color0-output"] }
            },
            "\\ce{(...)}": {
              "*": { action_: [{ type_: "output", option: 2 }, "ce"], nextState: "3" }
            },
            "\\,": {
              "*": { action_: [{ type_: "output", option: 1 }, "copy"], nextState: "1" }
            },
            "\\pu{(...)}": {
              "*": { action_: ["output", { type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }], nextState: "3" }
            },
            "\\x{}{}|\\x{}|\\x": {
              "0|1|2|3|a|as|b|p|bp|o|c0": { action_: ["o=", "output"], nextState: "3" },
              "*": { action_: ["output", "o=", "output"], nextState: "3" }
            },
            others: {
              "*": { action_: [{ type_: "output", option: 1 }, "copy"], nextState: "3" }
            },
            else2: {
              a: { action_: "a to o", nextState: "o", revisit: !0 },
              as: { action_: ["output", "sb=true"], nextState: "1", revisit: !0 },
              "r|rt|rd|rdt|rdq": { action_: ["output"], nextState: "0", revisit: !0 },
              "*": { action_: ["output", "copy"], nextState: "3" }
            }
          }),
          actions: {
            "o after d": function(q, G) {
              var J;
              if ((q.d || "").match(/^[1-9][0-9]*$/)) {
                var Z = q.d;
                q.d = void 0, J = this.output(q), J.push({ type_: "tinySkip" }), q.b = Z;
              } else
                J = this.output(q);
              return H.actions["o="](q, G), J;
            },
            "d= kv": function(q, G) {
              q.d = G, q.dType = "kv";
            },
            "charge or bond": function(q, G) {
              if (q.beginsWithBond) {
                var J = [];
                return H.concatArray(J, this.output(q)), H.concatArray(J, H.actions.bond(q, G, "-")), J;
              } else {
                q.d = G;
                return;
              }
            },
            "- after o/d": function(q, G, J) {
              var Z = H.patterns.match_("orbital", q.o || ""), te = H.patterns.match_("one lowercase greek letter $", q.o || ""), ne = H.patterns.match_("one lowercase latin letter $", q.o || ""), ee = H.patterns.match_("$one lowercase latin letter$ $", q.o || ""), ae = G === "-" && (Z && Z.remainder === "" || te || ne || ee);
              ae && !q.a && !q.b && !q.p && !q.d && !q.q && !Z && ne && (q.o = "$" + q.o + "$");
              var oe = [];
              return ae ? (H.concatArray(oe, this.output(q)), oe.push({ type_: "hyphen" })) : (Z = H.patterns.match_("digits", q.d || ""), J && Z && Z.remainder === "" ? (H.concatArray(oe, H.actions["d="](q, G)), H.concatArray(oe, this.output(q))) : (H.concatArray(oe, this.output(q)), H.concatArray(oe, H.actions.bond(q, G, "-")))), oe;
            },
            "a to o": function(q) {
              q.o = q.a, q.a = void 0;
            },
            "sb=true": function(q) {
              q.sb = !0;
            },
            "sb=false": function(q) {
              q.sb = !1;
            },
            "beginsWithBond=true": function(q) {
              q.beginsWithBond = !0;
            },
            "beginsWithBond=false": function(q) {
              q.beginsWithBond = !1;
            },
            "parenthesisLevel++": function(q) {
              q.parenthesisLevel++;
            },
            "parenthesisLevel--": function(q) {
              q.parenthesisLevel--;
            },
            "state of aggregation": function(q, G) {
              return { type_: "state of aggregation", p1: H.go(G, "o") };
            },
            comma: function(q, G) {
              var J = G.replace(/\s*$/, ""), Z = J !== G;
              return Z && q.parenthesisLevel === 0 ? { type_: "comma enumeration L", p1: J } : { type_: "comma enumeration M", p1: J };
            },
            output: function(q, G, J) {
              var Z;
              if (!q.r)
                Z = [], !q.a && !q.b && !q.p && !q.o && !q.q && !q.d && !J || (q.sb && Z.push({ type_: "entitySkip" }), !q.o && !q.q && !q.d && !q.b && !q.p && J !== 2 ? (q.o = q.a, q.a = void 0) : !q.o && !q.q && !q.d && (q.b || q.p) ? (q.o = q.a, q.d = q.b, q.q = q.p, q.a = q.b = q.p = void 0) : q.o && q.dType === "kv" && H.patterns.match_("d-oxidation$", q.d || "") ? q.dType = "oxidation" : q.o && q.dType === "kv" && !q.q && (q.dType = void 0), Z.push({
                  type_: "chemfive",
                  a: H.go(q.a, "a"),
                  b: H.go(q.b, "bd"),
                  p: H.go(q.p, "pq"),
                  o: H.go(q.o, "o"),
                  q: H.go(q.q, "pq"),
                  d: H.go(q.d, q.dType === "oxidation" ? "oxidation" : "bd"),
                  dType: q.dType
                }));
              else {
                var te = void 0;
                q.rdt === "M" ? te = H.go(q.rd, "tex-math") : q.rdt === "T" ? te = [{ type_: "text", p1: q.rd || "" }] : te = H.go(q.rd, "ce");
                var ne = void 0;
                q.rqt === "M" ? ne = H.go(q.rq, "tex-math") : q.rqt === "T" ? ne = [{ type_: "text", p1: q.rq || "" }] : ne = H.go(q.rq, "ce"), Z = {
                  type_: "arrow",
                  r: q.r,
                  rd: te,
                  rq: ne
                };
              }
              for (var ee in q)
                ee !== "parenthesisLevel" && ee !== "beginsWithBond" && delete q[ee];
              return Z;
            },
            "oxidation-output": function(q, G) {
              var J = ["{"];
              return H.concatArray(J, H.go(G, "oxidation")), J.push("}"), J;
            },
            "frac-output": function(q, G) {
              return { type_: "frac-ce", p1: H.go(G[0], "ce"), p2: H.go(G[1], "ce") };
            },
            "overset-output": function(q, G) {
              return { type_: "overset", p1: H.go(G[0], "ce"), p2: H.go(G[1], "ce") };
            },
            "underset-output": function(q, G) {
              return { type_: "underset", p1: H.go(G[0], "ce"), p2: H.go(G[1], "ce") };
            },
            "underbrace-output": function(q, G) {
              return { type_: "underbrace", p1: H.go(G[0], "ce"), p2: H.go(G[1], "ce") };
            },
            "color-output": function(q, G) {
              return { type_: "color", color1: G[0], color2: H.go(G[1], "ce") };
            },
            "r=": function(q, G) {
              q.r = G;
            },
            "rdt=": function(q, G) {
              q.rdt = G;
            },
            "rd=": function(q, G) {
              q.rd = G;
            },
            "rqt=": function(q, G) {
              q.rqt = G;
            },
            "rq=": function(q, G) {
              q.rq = G;
            },
            operator: function(q, G, J) {
              return { type_: "operator", kind_: J || G };
            }
          }
        },
        a: {
          transitions: _({
            empty: {
              "*": { action_: [] }
            },
            "1/2$": {
              0: { action_: "1/2" }
            },
            else: {
              0: { action_: [], nextState: "1", revisit: !0 }
            },
            "${(...)}$__$(...)$": {
              "*": { action_: "tex-math tight", nextState: "1" }
            },
            ",": {
              "*": { action_: { type_: "insert", option: "commaDecimal" } }
            },
            else2: {
              "*": { action_: "copy" }
            }
          }),
          actions: {}
        },
        o: {
          transitions: _({
            empty: {
              "*": { action_: [] }
            },
            "1/2$": {
              0: { action_: "1/2" }
            },
            else: {
              0: { action_: [], nextState: "1", revisit: !0 }
            },
            letters: {
              "*": { action_: "rm" }
            },
            "\\ca": {
              "*": { action_: { type_: "insert", option: "circa" } }
            },
            "\\pu{(...)}": {
              "*": { action_: [{ type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }] }
            },
            "\\x{}{}|\\x{}|\\x": {
              "*": { action_: "copy" }
            },
            "${(...)}$__$(...)$": {
              "*": { action_: "tex-math" }
            },
            "{(...)}": {
              "*": { action_: [{ type_: "write", option: "{" }, "text", { type_: "write", option: "}" }] }
            },
            else2: {
              "*": { action_: "copy" }
            }
          }),
          actions: {}
        },
        text: {
          transitions: _({
            empty: {
              "*": { action_: "output" }
            },
            "{...}": {
              "*": { action_: "text=" }
            },
            "${(...)}$__$(...)$": {
              "*": { action_: "tex-math" }
            },
            "\\greek": {
              "*": { action_: ["output", "rm"] }
            },
            "\\pu{(...)}": {
              "*": { action_: ["output", { type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }] }
            },
            "\\,|\\x{}{}|\\x{}|\\x": {
              "*": { action_: ["output", "copy"] }
            },
            else: {
              "*": { action_: "text=" }
            }
          }),
          actions: {
            output: function(q) {
              if (q.text_) {
                var G = { type_: "text", p1: q.text_ };
                for (var J in q)
                  delete q[J];
                return G;
              }
            }
          }
        },
        pq: {
          transitions: _({
            empty: {
              "*": { action_: [] }
            },
            "state of aggregation $": {
              "*": { action_: "state of aggregation" }
            },
            i$: {
              0: { action_: [], nextState: "!f", revisit: !0 }
            },
            "(KV letters),": {
              0: { action_: "rm", nextState: "0" }
            },
            formula$: {
              0: { action_: [], nextState: "f", revisit: !0 }
            },
            "1/2$": {
              0: { action_: "1/2" }
            },
            else: {
              0: { action_: [], nextState: "!f", revisit: !0 }
            },
            "${(...)}$__$(...)$": {
              "*": { action_: "tex-math" }
            },
            "{(...)}": {
              "*": { action_: "text" }
            },
            "a-z": {
              f: { action_: "tex-math" }
            },
            letters: {
              "*": { action_: "rm" }
            },
            "-9.,9": {
              "*": { action_: "9,9" }
            },
            ",": {
              "*": { action_: { type_: "insert+p1", option: "comma enumeration S" } }
            },
            "\\color{(...)}{(...)}": {
              "*": { action_: "color-output" }
            },
            "\\color{(...)}": {
              "*": { action_: "color0-output" }
            },
            "\\ce{(...)}": {
              "*": { action_: "ce" }
            },
            "\\pu{(...)}": {
              "*": { action_: [{ type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }] }
            },
            "\\,|\\x{}{}|\\x{}|\\x": {
              "*": { action_: "copy" }
            },
            else2: {
              "*": { action_: "copy" }
            }
          }),
          actions: {
            "state of aggregation": function(q, G) {
              return { type_: "state of aggregation subscript", p1: H.go(G, "o") };
            },
            "color-output": function(q, G) {
              return { type_: "color", color1: G[0], color2: H.go(G[1], "pq") };
            }
          }
        },
        bd: {
          transitions: _({
            empty: {
              "*": { action_: [] }
            },
            x$: {
              0: { action_: [], nextState: "!f", revisit: !0 }
            },
            formula$: {
              0: { action_: [], nextState: "f", revisit: !0 }
            },
            else: {
              0: { action_: [], nextState: "!f", revisit: !0 }
            },
            "-9.,9 no missing 0": {
              "*": { action_: "9,9" }
            },
            ".": {
              "*": { action_: { type_: "insert", option: "electron dot" } }
            },
            "a-z": {
              f: { action_: "tex-math" }
            },
            x: {
              "*": { action_: { type_: "insert", option: "KV x" } }
            },
            letters: {
              "*": { action_: "rm" }
            },
            "'": {
              "*": { action_: { type_: "insert", option: "prime" } }
            },
            "${(...)}$__$(...)$": {
              "*": { action_: "tex-math" }
            },
            "{(...)}": {
              "*": { action_: "text" }
            },
            "\\color{(...)}{(...)}": {
              "*": { action_: "color-output" }
            },
            "\\color{(...)}": {
              "*": { action_: "color0-output" }
            },
            "\\ce{(...)}": {
              "*": { action_: "ce" }
            },
            "\\pu{(...)}": {
              "*": { action_: [{ type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }] }
            },
            "\\,|\\x{}{}|\\x{}|\\x": {
              "*": { action_: "copy" }
            },
            else2: {
              "*": { action_: "copy" }
            }
          }),
          actions: {
            "color-output": function(q, G) {
              return { type_: "color", color1: G[0], color2: H.go(G[1], "bd") };
            }
          }
        },
        oxidation: {
          transitions: _({
            empty: {
              "*": { action_: "roman-numeral" }
            },
            "pm-operator": {
              "*": { action_: { type_: "o=+p1", option: "\\pm" } }
            },
            else: {
              "*": { action_: "o=" }
            }
          }),
          actions: {
            "roman-numeral": function(q) {
              return { type_: "roman numeral", p1: q.o || "" };
            }
          }
        },
        "tex-math": {
          transitions: _({
            empty: {
              "*": { action_: "output" }
            },
            "\\ce{(...)}": {
              "*": { action_: ["output", "ce"] }
            },
            "\\pu{(...)}": {
              "*": { action_: ["output", { type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }] }
            },
            "{...}|\\,|\\x{}{}|\\x{}|\\x": {
              "*": { action_: "o=" }
            },
            else: {
              "*": { action_: "o=" }
            }
          }),
          actions: {
            output: function(q) {
              if (q.o) {
                var G = { type_: "tex-math", p1: q.o };
                for (var J in q)
                  delete q[J];
                return G;
              }
            }
          }
        },
        "tex-math tight": {
          transitions: _({
            empty: {
              "*": { action_: "output" }
            },
            "\\ce{(...)}": {
              "*": { action_: ["output", "ce"] }
            },
            "\\pu{(...)}": {
              "*": { action_: ["output", { type_: "write", option: "{" }, "pu", { type_: "write", option: "}" }] }
            },
            "{...}|\\,|\\x{}{}|\\x{}|\\x": {
              "*": { action_: "o=" }
            },
            "-|+": {
              "*": { action_: "tight operator" }
            },
            else: {
              "*": { action_: "o=" }
            }
          }),
          actions: {
            "tight operator": function(q, G) {
              q.o = (q.o || "") + "{" + G + "}";
            },
            output: function(q) {
              if (q.o) {
                var G = { type_: "tex-math", p1: q.o };
                for (var J in q)
                  delete q[J];
                return G;
              }
            }
          }
        },
        "9,9": {
          transitions: _({
            empty: {
              "*": { action_: [] }
            },
            ",": {
              "*": { action_: "comma" }
            },
            else: {
              "*": { action_: "copy" }
            }
          }),
          actions: {
            comma: function() {
              return { type_: "commaDecimal" };
            }
          }
        },
        pu: {
          transitions: _({
            empty: {
              "*": { action_: "output" }
            },
            space$: {
              "*": { action_: ["output", "space"] }
            },
            "{[(|)]}": {
              "0|a": { action_: "copy" }
            },
            "(-)(9)^(-9)": {
              0: { action_: "number^", nextState: "a" }
            },
            "(-)(9.,9)(e)(99)": {
              0: { action_: "enumber", nextState: "a" }
            },
            space: {
              "0|a": { action_: [] }
            },
            "pm-operator": {
              "0|a": { action_: { type_: "operator", option: "\\pm" }, nextState: "0" }
            },
            operator: {
              "0|a": { action_: "copy", nextState: "0" }
            },
            "//": {
              d: { action_: "o=", nextState: "/" }
            },
            "/": {
              d: { action_: "o=", nextState: "/" }
            },
            "{...}|else": {
              "0|d": { action_: "d=", nextState: "d" },
              a: { action_: ["space", "d="], nextState: "d" },
              "/|q": { action_: "q=", nextState: "q" }
            }
          }),
          actions: {
            enumber: function(q, G) {
              var J = [];
              return G[0] === "+-" || G[0] === "+/-" ? J.push("\\pm ") : G[0] && J.push(G[0]), G[1] && (H.concatArray(J, H.go(G[1], "pu-9,9")), G[2] && (G[2].match(/[,.]/) ? H.concatArray(J, H.go(G[2], "pu-9,9")) : J.push(G[2])), (G[3] || G[4]) && (G[3] === "e" || G[4] === "*" ? J.push({ type_: "cdot" }) : J.push({ type_: "times" }))), G[5] && J.push("10^{" + G[5] + "}"), J;
            },
            "number^": function(q, G) {
              var J = [];
              return G[0] === "+-" || G[0] === "+/-" ? J.push("\\pm ") : G[0] && J.push(G[0]), H.concatArray(J, H.go(G[1], "pu-9,9")), J.push("^{" + G[2] + "}"), J;
            },
            operator: function(q, G, J) {
              return { type_: "operator", kind_: J || G };
            },
            space: function() {
              return { type_: "pu-space-1" };
            },
            output: function(q) {
              var G, J = H.patterns.match_("{(...)}", q.d || "");
              J && J.remainder === "" && (q.d = J.match_);
              var Z = H.patterns.match_("{(...)}", q.q || "");
              if (Z && Z.remainder === "" && (q.q = Z.match_), q.d && (q.d = q.d.replace(/\u00B0C|\^oC|\^{o}C/g, "{}^{\\circ}C"), q.d = q.d.replace(/\u00B0F|\^oF|\^{o}F/g, "{}^{\\circ}F")), q.q) {
                q.q = q.q.replace(/\u00B0C|\^oC|\^{o}C/g, "{}^{\\circ}C"), q.q = q.q.replace(/\u00B0F|\^oF|\^{o}F/g, "{}^{\\circ}F");
                var te = {
                  d: H.go(q.d, "pu"),
                  q: H.go(q.q, "pu")
                };
                q.o === "//" ? G = { type_: "pu-frac", p1: te.d, p2: te.q } : (G = te.d, te.d.length > 1 || te.q.length > 1 ? G.push({ type_: " / " }) : G.push({ type_: "/" }), H.concatArray(G, te.q));
              } else
                G = H.go(q.d, "pu-2");
              for (var ne in q)
                delete q[ne];
              return G;
            }
          }
        },
        "pu-2": {
          transitions: _({
            empty: {
              "*": { action_: "output" }
            },
            "*": {
              "*": { action_: ["output", "cdot"], nextState: "0" }
            },
            "\\x": {
              "*": { action_: "rm=" }
            },
            space: {
              "*": { action_: ["output", "space"], nextState: "0" }
            },
            "^{(...)}|^(-1)": {
              1: { action_: "^(-1)" }
            },
            "-9.,9": {
              0: { action_: "rm=", nextState: "0" },
              1: { action_: "^(-1)", nextState: "0" }
            },
            "{...}|else": {
              "*": { action_: "rm=", nextState: "1" }
            }
          }),
          actions: {
            cdot: function() {
              return { type_: "tight cdot" };
            },
            "^(-1)": function(q, G) {
              q.rm += "^{" + G + "}";
            },
            space: function() {
              return { type_: "pu-space-2" };
            },
            output: function(q) {
              var G = [];
              if (q.rm) {
                var J = H.patterns.match_("{(...)}", q.rm || "");
                J && J.remainder === "" ? G = H.go(J.match_, "pu") : G = { type_: "rm", p1: q.rm };
              }
              for (var Z in q)
                delete q[Z];
              return G;
            }
          }
        },
        "pu-9,9": {
          transitions: _({
            empty: {
              0: { action_: "output-0" },
              o: { action_: "output-o" }
            },
            ",": {
              0: { action_: ["output-0", "comma"], nextState: "o" }
            },
            ".": {
              0: { action_: ["output-0", "copy"], nextState: "o" }
            },
            else: {
              "*": { action_: "text=" }
            }
          }),
          actions: {
            comma: function() {
              return { type_: "commaDecimal" };
            },
            "output-0": function(q) {
              var G = [];
              if (q.text_ = q.text_ || "", q.text_.length > 4) {
                var J = q.text_.length % 3;
                J === 0 && (J = 3);
                for (var Z = q.text_.length - 3; Z > 0; Z -= 3)
                  G.push(q.text_.substr(Z, 3)), G.push({ type_: "1000 separator" });
                G.push(q.text_.substr(0, J)), G.reverse();
              } else
                G.push(q.text_);
              for (var te in q)
                delete q[te];
              return G;
            },
            "output-o": function(q) {
              var G = [];
              if (q.text_ = q.text_ || "", q.text_.length > 4) {
                var J = q.text_.length - 3, Z = void 0;
                for (Z = 0; Z < J; Z += 3)
                  G.push(q.text_.substr(Z, 3)), G.push({ type_: "1000 separator" });
                G.push(q.text_.substr(Z));
              } else
                G.push(q.text_);
              for (var te in q)
                delete q[te];
              return G;
            }
          }
        }
      }
    }, V = {
      go: function(q, G) {
        if (!q)
          return "";
        for (var J = "", Z = !1, te = 0; te < q.length; te++) {
          var ne = q[te];
          typeof ne == "string" ? J += ne : (J += V._go2(ne), ne.type_ === "1st-level escape" && (Z = !0));
        }
        return G && !Z && J && (J = "{" + J + "}"), J;
      },
      _goInner: function(q) {
        return V.go(q, !1);
      },
      _go2: function(q) {
        var G;
        switch (q.type_) {
          case "chemfive":
            G = "";
            var J = {
              a: V._goInner(q.a),
              b: V._goInner(q.b),
              p: V._goInner(q.p),
              o: V._goInner(q.o),
              q: V._goInner(q.q),
              d: V._goInner(q.d)
            };
            J.a && (J.a.match(/^[+\-]/) && (J.a = "{" + J.a + "}"), G += J.a + "\\,"), (J.b || J.p) && (G += "{\\vphantom{A}}", G += "^{\\hphantom{" + (J.b || "") + "}}_{\\hphantom{" + (J.p || "") + "}}", G += "\\mkern-1.5mu", G += "{\\vphantom{A}}", G += "^{\\smash[t]{\\vphantom{2}}\\llap{" + (J.b || "") + "}}", G += "_{\\vphantom{2}\\llap{\\smash[t]{" + (J.p || "") + "}}}"), J.o && (J.o.match(/^[+\-]/) && (J.o = "{" + J.o + "}"), G += J.o), q.dType === "kv" ? ((J.d || J.q) && (G += "{\\vphantom{A}}"), J.d && (G += "^{" + J.d + "}"), J.q && (G += "_{\\smash[t]{" + J.q + "}}")) : q.dType === "oxidation" ? (J.d && (G += "{\\vphantom{A}}", G += "^{" + J.d + "}"), J.q && (G += "{\\vphantom{A}}", G += "_{\\smash[t]{" + J.q + "}}")) : (J.q && (G += "{\\vphantom{A}}", G += "_{\\smash[t]{" + J.q + "}}"), J.d && (G += "{\\vphantom{A}}", G += "^{" + J.d + "}"));
            break;
          case "rm":
            G = "\\mathrm{" + q.p1 + "}";
            break;
          case "text":
            q.p1.match(/[\^_]/) ? (q.p1 = q.p1.replace(" ", "~").replace("-", "\\text{-}"), G = "\\mathrm{" + q.p1 + "}") : G = "\\text{" + q.p1 + "}";
            break;
          case "roman numeral":
            G = "\\mathrm{" + q.p1 + "}";
            break;
          case "state of aggregation":
            G = "\\mskip2mu " + V._goInner(q.p1);
            break;
          case "state of aggregation subscript":
            G = "\\mskip1mu " + V._goInner(q.p1);
            break;
          case "bond":
            if (G = V._getBond(q.kind_), !G)
              throw ["MhchemErrorBond", "mhchem Error. Unknown bond type (" + q.kind_ + ")"];
            break;
          case "frac":
            var Z = "\\frac{" + q.p1 + "}{" + q.p2 + "}";
            G = "\\mathchoice{\\textstyle" + Z + "}{" + Z + "}{" + Z + "}{" + Z + "}";
            break;
          case "pu-frac":
            var te = "\\frac{" + V._goInner(q.p1) + "}{" + V._goInner(q.p2) + "}";
            G = "\\mathchoice{\\textstyle" + te + "}{" + te + "}{" + te + "}{" + te + "}";
            break;
          case "tex-math":
            G = q.p1 + " ";
            break;
          case "frac-ce":
            G = "\\frac{" + V._goInner(q.p1) + "}{" + V._goInner(q.p2) + "}";
            break;
          case "overset":
            G = "\\overset{" + V._goInner(q.p1) + "}{" + V._goInner(q.p2) + "}";
            break;
          case "underset":
            G = "\\underset{" + V._goInner(q.p1) + "}{" + V._goInner(q.p2) + "}";
            break;
          case "underbrace":
            G = "\\underbrace{" + V._goInner(q.p1) + "}_{" + V._goInner(q.p2) + "}";
            break;
          case "color":
            G = "{\\color{" + q.color1 + "}{" + V._goInner(q.color2) + "}}";
            break;
          case "color0":
            G = "\\color{" + q.color + "}";
            break;
          case "arrow":
            var ne = {
              rd: V._goInner(q.rd),
              rq: V._goInner(q.rq)
            }, ee = V._getArrow(q.r);
            ne.rd || ne.rq ? q.r === "<=>" || q.r === "<=>>" || q.r === "<<=>" || q.r === "<-->" ? (ee = "\\long" + ee, ne.rd && (ee = "\\overset{" + ne.rd + "}{" + ee + "}"), ne.rq && (q.r === "<-->" ? ee = "\\underset{\\lower2mu{" + ne.rq + "}}{" + ee + "}" : ee = "\\underset{\\lower6mu{" + ne.rq + "}}{" + ee + "}"), ee = " {}\\mathrel{" + ee + "}{} ") : (ne.rq && (ee += "[{" + ne.rq + "}]"), ee += "{" + ne.rd + "}", ee = " {}\\mathrel{\\x" + ee + "}{} ") : ee = " {}\\mathrel{\\long" + ee + "}{} ", G = ee;
            break;
          case "operator":
            G = V._getOperator(q.kind_);
            break;
          case "1st-level escape":
            G = q.p1 + " ";
            break;
          case "space":
            G = " ";
            break;
          case "tinySkip":
            G = "\\mkern2mu";
            break;
          case "entitySkip":
            G = "~";
            break;
          case "pu-space-1":
            G = "~";
            break;
          case "pu-space-2":
            G = "\\mkern3mu ";
            break;
          case "1000 separator":
            G = "\\mkern2mu ";
            break;
          case "commaDecimal":
            G = "{,}";
            break;
          case "comma enumeration L":
            G = "{" + q.p1 + "}\\mkern6mu ";
            break;
          case "comma enumeration M":
            G = "{" + q.p1 + "}\\mkern3mu ";
            break;
          case "comma enumeration S":
            G = "{" + q.p1 + "}\\mkern1mu ";
            break;
          case "hyphen":
            G = "\\text{-}";
            break;
          case "addition compound":
            G = "\\,{\\cdot}\\,";
            break;
          case "electron dot":
            G = "\\mkern1mu \\bullet\\mkern1mu ";
            break;
          case "KV x":
            G = "{\\times}";
            break;
          case "prime":
            G = "\\prime ";
            break;
          case "cdot":
            G = "\\cdot ";
            break;
          case "tight cdot":
            G = "\\mkern1mu{\\cdot}\\mkern1mu ";
            break;
          case "times":
            G = "\\times ";
            break;
          case "circa":
            G = "{\\sim}";
            break;
          case "^":
            G = "uparrow";
            break;
          case "v":
            G = "downarrow";
            break;
          case "ellipsis":
            G = "\\ldots ";
            break;
          case "/":
            G = "/";
            break;
          case " / ":
            G = "\\,/\\,";
            break;
          default:
            throw ["MhchemBugT", "mhchem bug T. Please report."];
        }
        return G;
      },
      _getArrow: function(q) {
        switch (q) {
          case "->":
            return "rightarrow";
          case "→":
            return "rightarrow";
          case "⟶":
            return "rightarrow";
          case "<-":
            return "leftarrow";
          case "<->":
            return "leftrightarrow";
          case "<-->":
            return "leftrightarrows";
          case "<=>":
            return "rightleftharpoons";
          case "⇌":
            return "rightleftharpoons";
          case "<=>>":
            return "Rightleftharpoons";
          case "<<=>":
            return "Leftrightharpoons";
          default:
            throw ["MhchemBugT", "mhchem bug T. Please report."];
        }
      },
      _getBond: function(q) {
        switch (q) {
          case "-":
            return "{-}";
          case "1":
            return "{-}";
          case "=":
            return "{=}";
          case "2":
            return "{=}";
          case "#":
            return "{\\equiv}";
          case "3":
            return "{\\equiv}";
          case "~":
            return "{\\tripledash}";
          case "~-":
            return "{\\rlap{\\lower.1em{-}}\\raise.1em{\\tripledash}}";
          case "~=":
            return "{\\rlap{\\lower.2em{-}}\\rlap{\\raise.2em{\\tripledash}}-}";
          case "~--":
            return "{\\rlap{\\lower.2em{-}}\\rlap{\\raise.2em{\\tripledash}}-}";
          case "-~-":
            return "{\\rlap{\\lower.2em{-}}\\rlap{\\raise.2em{-}}\\tripledash}";
          case "...":
            return "{{\\cdot}{\\cdot}{\\cdot}}";
          case "....":
            return "{{\\cdot}{\\cdot}{\\cdot}{\\cdot}}";
          case "->":
            return "{\\rightarrow}";
          case "<-":
            return "{\\leftarrow}";
          case "<":
            return "{<}";
          case ">":
            return "{>}";
          default:
            throw ["MhchemBugT", "mhchem bug T. Please report."];
        }
      },
      _getOperator: function(q) {
        switch (q) {
          case "+":
            return " {}+{} ";
          case "-":
            return " {}-{} ";
          case "=":
            return " {}={} ";
          case "<":
            return " {}<{} ";
          case ">":
            return " {}>{} ";
          case "<<":
            return " {}\\ll{} ";
          case ">>":
            return " {}\\gg{} ";
          case "\\pm":
            return " {}\\pm{} ";
          case "\\approx":
            return " {}\\approx{} ";
          case "$\\approx$":
            return " {}\\approx{} ";
          case "v":
            return " \\downarrow{} ";
          case "(v)":
            return " \\downarrow{} ";
          case "^":
            return " \\uparrow{} ";
          case "(^)":
            return " \\uparrow{} ";
          default:
            throw ["MhchemBugT", "mhchem bug T. Please report."];
        }
      }
    };
    return mhchemParser;
  }
  var hasRequiredMhchemConfiguration;
  function requireMhchemConfiguration() {
    if (hasRequiredMhchemConfiguration) return MhchemConfiguration;
    hasRequiredMhchemConfiguration = 1;
    var O = MhchemConfiguration.__importDefault || function(te) {
      return te && te.__esModule ? te : { default: te };
    };
    Object.defineProperty(MhchemConfiguration, "__esModule", { value: !0 }), MhchemConfiguration.MhchemConfiguration = void 0;
    var _ = requireConfiguration(), H = requireSymbolMap(), V = O(requireTexError()), q = O(requireBaseMethods()), G = requireAmsMethods(), J = requireMhchemParser(), Z = {};
    return Z.Macro = q.default.Macro, Z.xArrow = G.AmsMethods.xArrow, Z.Machine = function(te, ne, ee) {
      var ae = te.GetArgument(ne), oe;
      try {
        oe = J.mhchemParser.toTex(ae, ee);
      } catch (se) {
        throw new V.default(se[0], se[1]);
      }
      te.string = oe + te.string.substr(te.i), te.i = 0;
    }, new H.CommandMap("mhchem", {
      ce: ["Machine", "ce"],
      pu: ["Machine", "pu"],
      longrightleftharpoons: [
        "Macro",
        "\\stackrel{\\textstyle{-}\\!\\!{\\rightharpoonup}}{\\smash{{\\leftharpoondown}\\!\\!{-}}}"
      ],
      longRightleftharpoons: [
        "Macro",
        "\\stackrel{\\textstyle{-}\\!\\!{\\rightharpoonup}}{\\smash{\\leftharpoondown}}"
      ],
      longLeftrightharpoons: [
        "Macro",
        "\\stackrel{\\textstyle\\vphantom{{-}}{\\rightharpoonup}}{\\smash{{\\leftharpoondown}\\!\\!{-}}}"
      ],
      longleftrightarrows: [
        "Macro",
        "\\stackrel{\\longrightarrow}{\\smash{\\longleftarrow}\\Rule{0px}{.25em}{0px}}"
      ],
      tripledash: [
        "Macro",
        "\\vphantom{-}\\raise2mu{\\kern2mu\\tiny\\text{-}\\kern1mu\\text{-}\\kern1mu\\text{-}\\kern2mu}"
      ],
      xleftrightarrow: ["xArrow", 8596, 6, 6],
      xrightleftharpoons: ["xArrow", 8652, 5, 7],
      xRightleftharpoons: ["xArrow", 8652, 5, 7],
      xLeftrightharpoons: ["xArrow", 8652, 5, 7]
    }, Z), MhchemConfiguration.MhchemConfiguration = _.Configuration.create("mhchem", { handler: { macro: ["mhchem"] } }), MhchemConfiguration;
  }
  var NoErrorsConfiguration = {}, hasRequiredNoErrorsConfiguration;
  function requireNoErrorsConfiguration() {
    if (hasRequiredNoErrorsConfiguration) return NoErrorsConfiguration;
    hasRequiredNoErrorsConfiguration = 1, Object.defineProperty(NoErrorsConfiguration, "__esModule", { value: !0 }), NoErrorsConfiguration.NoErrorsConfiguration = void 0;
    var O = requireConfiguration();
    function _(H, V, q, G) {
      var J = H.create("token", "mtext", {}, G.replace(/\n/g, " ")), Z = H.create("node", "merror", [J], { "data-mjx-error": V, title: V });
      return Z;
    }
    return NoErrorsConfiguration.NoErrorsConfiguration = O.Configuration.create("noerrors", { nodes: { error: _ } }), NoErrorsConfiguration;
  }
  var NoUndefinedConfiguration = {}, hasRequiredNoUndefinedConfiguration;
  function requireNoUndefinedConfiguration() {
    if (hasRequiredNoUndefinedConfiguration) return NoUndefinedConfiguration;
    hasRequiredNoUndefinedConfiguration = 1;
    var O = NoUndefinedConfiguration.__values || function(V) {
      var q = typeof Symbol == "function" && Symbol.iterator, G = q && V[q], J = 0;
      if (G) return G.call(V);
      if (V && typeof V.length == "number") return {
        next: function() {
          return V && J >= V.length && (V = void 0), { value: V && V[J++], done: !V };
        }
      };
      throw new TypeError(q ? "Object is not iterable." : "Symbol.iterator is not defined.");
    };
    Object.defineProperty(NoUndefinedConfiguration, "__esModule", { value: !0 }), NoUndefinedConfiguration.NoUndefinedConfiguration = void 0;
    var _ = requireConfiguration();
    function H(V, q) {
      var G, J, Z = V.create("text", "\\" + q), te = V.options.noundefined || {}, ne = {};
      try {
        for (var ee = O(["color", "background", "size"]), ae = ee.next(); !ae.done; ae = ee.next()) {
          var oe = ae.value;
          te[oe] && (ne["math" + oe] = te[oe]);
        }
      } catch (se) {
        G = { error: se };
      } finally {
        try {
          ae && !ae.done && (J = ee.return) && J.call(ee);
        } finally {
          if (G) throw G.error;
        }
      }
      V.Push(V.create("node", "mtext", [], ne, Z));
    }
    return NoUndefinedConfiguration.NoUndefinedConfiguration = _.Configuration.create("noundefined", {
      fallback: { macro: H },
      options: {
        noundefined: {
          color: "red",
          background: "",
          size: ""
        }
      },
      priority: 3
    }), NoUndefinedConfiguration;
  }
  var PhysicsConfiguration = {}, PhysicsItems = {}, hasRequiredPhysicsItems;
  function requirePhysicsItems() {
    if (hasRequiredPhysicsItems) return PhysicsItems;
    hasRequiredPhysicsItems = 1;
    var O = PhysicsItems.__extends || /* @__PURE__ */ function() {
      var Z = function(te, ne) {
        return Z = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(ee, ae) {
          ee.__proto__ = ae;
        } || function(ee, ae) {
          for (var oe in ae) Object.prototype.hasOwnProperty.call(ae, oe) && (ee[oe] = ae[oe]);
        }, Z(te, ne);
      };
      return function(te, ne) {
        if (typeof ne != "function" && ne !== null)
          throw new TypeError("Class extends value " + String(ne) + " is not a constructor or null");
        Z(te, ne);
        function ee() {
          this.constructor = te;
        }
        te.prototype = ne === null ? Object.create(ne) : (ee.prototype = ne.prototype, new ee());
      };
    }(), _ = PhysicsItems.__importDefault || function(Z) {
      return Z && Z.__esModule ? Z : { default: Z };
    };
    Object.defineProperty(PhysicsItems, "__esModule", { value: !0 }), PhysicsItems.AutoOpen = void 0;
    var H = requireStackItem(), V = _(requireParseUtil()), q = _(requireNodeUtil()), G = _(requireTexParser()), J = function(Z) {
      O(te, Z);
      function te() {
        var ne = Z !== null && Z.apply(this, arguments) || this;
        return ne.openCount = 0, ne;
      }
      return Object.defineProperty(te.prototype, "kind", {
        get: function() {
          return "auto open";
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(te.prototype, "isOpen", {
        get: function() {
          return !0;
        },
        enumerable: !1,
        configurable: !0
      }), te.prototype.toMml = function() {
        var ne = this.factory.configuration.parser, ee = this.getProperty("right");
        if (this.getProperty("smash")) {
          var ae = Z.prototype.toMml.call(this), oe = ne.create("node", "mpadded", [ae], { height: 0, depth: 0 });
          this.Clear(), this.Push(ne.create("node", "TeXAtom", [oe]));
        }
        ee && this.Push(new G.default(ee, ne.stack.env, ne.configuration).mml());
        var se = V.default.fenced(this.factory.configuration, this.getProperty("open"), Z.prototype.toMml.call(this), this.getProperty("close"), this.getProperty("big"));
        return q.default.removeProperties(se, "open", "close", "texClass"), se;
      }, te.prototype.checkItem = function(ne) {
        if (ne.isKind("mml") && ne.Size() === 1) {
          var ee = ne.toMml();
          ee.isKind("mo") && ee.getText() === this.getProperty("open") && this.openCount++;
        }
        var ae = ne.getProperty("autoclose");
        return ae && ae === this.getProperty("close") && !this.openCount-- ? this.getProperty("ignore") ? (this.Clear(), [[], !0]) : [[this.toMml()], !0] : Z.prototype.checkItem.call(this, ne);
      }, te.errors = Object.assign(Object.create(H.BaseItem.errors), {
        stop: ["ExtraOrMissingDelims", "Extra open or missing close delimiter"]
      }), te;
    }(H.BaseItem);
    return PhysicsItems.AutoOpen = J, PhysicsItems;
  }
  var PhysicsMappings = {}, PhysicsMethods = {}, hasRequiredPhysicsMethods;
  function requirePhysicsMethods() {
    if (hasRequiredPhysicsMethods) return PhysicsMethods;
    hasRequiredPhysicsMethods = 1;
    var O = PhysicsMethods.__read || function(ye, ge) {
      var ve = typeof Symbol == "function" && ye[Symbol.iterator];
      if (!ve) return ye;
      var be = ve.call(ye), Ce, Se = [], we;
      try {
        for (; (ge === void 0 || ge-- > 0) && !(Ce = be.next()).done; ) Se.push(Ce.value);
      } catch (Ee) {
        we = { error: Ee };
      } finally {
        try {
          Ce && !Ce.done && (ve = be.return) && ve.call(be);
        } finally {
          if (we) throw we.error;
        }
      }
      return Se;
    }, _ = PhysicsMethods.__importDefault || function(ye) {
      return ye && ye.__esModule ? ye : { default: ye };
    };
    Object.defineProperty(PhysicsMethods, "__esModule", { value: !0 });
    var H = _(requireBaseMethods()), V = _(requireTexParser()), q = _(requireTexError()), G = requireMmlNode(), J = _(requireParseUtil()), Z = _(requireNodeUtil()), te = requireNodeFactory$1(), ne = {}, ee = {
      "(": ")",
      "[": "]",
      "{": "}",
      "|": "|"
    }, ae = /^(b|B)i(g{1,2})$/;
    ne.Quantity = function(ye, ge, ve, be, Ce, Se, we) {
      ve === void 0 && (ve = "("), be === void 0 && (be = ")"), Ce === void 0 && (Ce = !1), Se === void 0 && (Se = ""), we === void 0 && (we = "");
      var Ee = Ce ? ye.GetStar() : !1, xe = ye.GetNext(), Le = ye.i, Qe = null;
      if (xe === "\\") {
        if (ye.i++, Qe = ye.GetCS(), !Qe.match(ae)) {
          var ke = ye.create("node", "mrow");
          ye.Push(J.default.fenced(ye.configuration, ve, ke, be)), ye.i = Le;
          return;
        }
        xe = ye.GetNext();
      }
      var Me = ee[xe];
      if (Ce && xe !== "{")
        throw new q.default("MissingArgFor", "Missing argument for %1", ye.currentCS);
      if (!Me) {
        var ke = ye.create("node", "mrow");
        ye.Push(J.default.fenced(ye.configuration, ve, ke, be)), ye.i = Le;
        return;
      }
      if (Se) {
        var Re = ye.create("token", "mi", { texClass: G.TEXCLASS.OP }, Se);
        we && Z.default.setAttribute(Re, "mathvariant", we), ye.Push(ye.itemFactory.create("fn", Re));
      }
      if (xe === "{") {
        var De = ye.GetArgument(ge);
        xe = Ce ? ve : "\\{", Me = Ce ? be : "\\}", De = Ee ? xe + " " + De + " " + Me : Qe ? "\\" + Qe + "l" + xe + " " + De + " \\" + Qe + "r" + Me : "\\left" + xe + " " + De + " \\right" + Me, ye.Push(new V.default(De, ye.stack.env, ye.configuration).mml());
        return;
      }
      Ce && (xe = ve, Me = be), ye.i++, ye.Push(ye.itemFactory.create("auto open").setProperties({ open: xe, close: Me, big: Qe }));
    }, ne.Eval = function(ye, ge) {
      var ve = ye.GetStar(), be = ye.GetNext();
      if (be === "{") {
        var Ce = ye.GetArgument(ge), Se = "\\left. " + (ve ? "\\smash{" + Ce + "}" : Ce) + " \\vphantom{\\int}\\right|";
        ye.string = ye.string.slice(0, ye.i) + Se + ye.string.slice(ye.i);
        return;
      }
      if (be === "(" || be === "[") {
        ye.i++, ye.Push(ye.itemFactory.create("auto open").setProperties({
          open: be,
          close: "|",
          smash: ve,
          right: "\\vphantom{\\int}"
        }));
        return;
      }
      throw new q.default("MissingArgFor", "Missing argument for %1", ye.currentCS);
    }, ne.Commutator = function(ye, ge, ve, be) {
      ve === void 0 && (ve = "["), be === void 0 && (be = "]");
      var Ce = ye.GetStar(), Se = ye.GetNext(), we = null;
      if (Se === "\\") {
        if (ye.i++, we = ye.GetCS(), !we.match(ae))
          throw new q.default("MissingArgFor", "Missing argument for %1", ye.currentCS);
        Se = ye.GetNext();
      }
      if (Se !== "{")
        throw new q.default("MissingArgFor", "Missing argument for %1", ye.currentCS);
      var Ee = ye.GetArgument(ge), xe = ye.GetArgument(ge), Le = Ee + "," + xe;
      Le = Ce ? ve + " " + Le + " " + be : we ? "\\" + we + "l" + ve + " " + Le + " \\" + we + "r" + be : "\\left" + ve + " " + Le + " \\right" + be, ye.Push(new V.default(Le, ye.stack.env, ye.configuration).mml());
    };
    var oe = [65, 90], se = [97, 122], ce = [913, 937], le = [945, 969], ue = [48, 57];
    function de(ye, ge) {
      return ye >= ge[0] && ye <= ge[1];
    }
    function fe(ye, ge, ve, be) {
      var Ce = ye.configuration.parser, Se = te.NodeFactory.createToken(ye, ge, ve, be), we = be.codePointAt(0);
      return be.length === 1 && !Ce.stack.env.font && Ce.stack.env.vectorFont && (de(we, oe) || de(we, se) || de(we, ce) || de(we, ue) || de(we, le) && Ce.stack.env.vectorStar || Z.default.getAttribute(Se, "accent")) && Z.default.setAttribute(Se, "mathvariant", Ce.stack.env.vectorFont), Se;
    }
    ne.VectorBold = function(ye, ge) {
      var ve = ye.GetStar(), be = ye.GetArgument(ge), Ce = ye.configuration.nodeFactory.get("token"), Se = ye.stack.env.font;
      delete ye.stack.env.font, ye.configuration.nodeFactory.set("token", fe), ye.stack.env.vectorFont = ve ? "bold-italic" : "bold", ye.stack.env.vectorStar = ve;
      var we = new V.default(be, ye.stack.env, ye.configuration).mml();
      Se && (ye.stack.env.font = Se), delete ye.stack.env.vectorFont, delete ye.stack.env.vectorStar, ye.configuration.nodeFactory.set("token", Ce), ye.Push(we);
    }, ne.StarMacro = function(ye, ge, ve) {
      for (var be = [], Ce = 3; Ce < arguments.length; Ce++)
        be[Ce - 3] = arguments[Ce];
      var Se = ye.GetStar(), we = [];
      if (ve)
        for (var Ee = we.length; Ee < ve; Ee++)
          we.push(ye.GetArgument(ge));
      var xe = be.join(Se ? "*" : "");
      xe = J.default.substituteArgs(ye, we, xe), ye.string = J.default.addArgs(ye, xe, ye.string.slice(ye.i)), ye.i = 0, J.default.checkMaxMacros(ye);
    };
    var he = function(ye, ge, ve, be, Ce) {
      var Se = new V.default(be, ye.stack.env, ye.configuration).mml();
      ye.Push(ye.itemFactory.create(ge, Se));
      var we = ye.GetNext(), Ee = ee[we];
      if (Ee) {
        var xe = "", Le = "", Qe = "", ke = Ce.indexOf(we) !== -1;
        if (we === "{") {
          Qe = ye.GetArgument(ve), xe = ke ? "\\left\\{" : "", Le = ke ? "\\right\\}" : "";
          var Me = xe + " " + Qe + " " + Le;
          ye.string = Me + ye.string.slice(ye.i), ye.i = 0;
          return;
        }
        ke && (ye.i++, ye.Push(ye.itemFactory.create("auto open").setProperties({ open: we, close: Ee })));
      }
    };
    ne.OperatorApplication = function(ye, ge, ve) {
      for (var be = [], Ce = 3; Ce < arguments.length; Ce++)
        be[Ce - 3] = arguments[Ce];
      he(ye, "fn", ge, ve, be);
    }, ne.VectorOperator = function(ye, ge, ve) {
      for (var be = [], Ce = 3; Ce < arguments.length; Ce++)
        be[Ce - 3] = arguments[Ce];
      he(ye, "mml", ge, ve, be);
    }, ne.Expression = function(ye, ge, ve, be) {
      ve === void 0 && (ve = !0), be === void 0 && (be = ""), be = be || ge.slice(1);
      var Ce = ve ? ye.GetBrackets(ge) : null, Se = ye.create("token", "mi", { texClass: G.TEXCLASS.OP }, be);
      if (Ce) {
        var we = new V.default(Ce, ye.stack.env, ye.configuration).mml();
        Se = ye.create("node", "msup", [Se, we]);
      }
      ye.Push(ye.itemFactory.create("fn", Se)), ye.GetNext() === "(" && (ye.i++, ye.Push(ye.itemFactory.create("auto open").setProperties({ open: "(", close: ")" })));
    }, ne.Qqtext = function(ye, ge, ve) {
      var be = ye.GetStar(), Ce = ve || ye.GetArgument(ge), Se = (be ? "" : "\\quad") + "\\text{" + Ce + "}\\quad ";
      ye.string = ye.string.slice(0, ye.i) + Se + ye.string.slice(ye.i);
    }, ne.Differential = function(ye, ge, ve) {
      var be = ye.GetBrackets(ge), Ce = be != null ? "^{" + be + "}" : " ", Se = ye.GetNext() === "(", we = ye.GetNext() === "{", Ee = ve + Ce;
      if (!(Se || we)) {
        Ee += ye.GetArgument(ge, !0) || "";
        var xe = new V.default(Ee, ye.stack.env, ye.configuration).mml();
        ye.Push(xe);
        return;
      }
      if (we) {
        Ee += ye.GetArgument(ge);
        var xe = new V.default(Ee, ye.stack.env, ye.configuration).mml();
        ye.Push(ye.create("node", "TeXAtom", [xe], { texClass: G.TEXCLASS.OP }));
        return;
      }
      ye.Push(new V.default(Ee, ye.stack.env, ye.configuration).mml()), ye.i++, ye.Push(ye.itemFactory.create("auto open").setProperties({ open: "(", close: ")" }));
    }, ne.Derivative = function(ye, ge, ve, be) {
      var Ce = ye.GetStar(), Se = ye.GetBrackets(ge), we = 1, Ee = [];
      for (Ee.push(ye.GetArgument(ge)); ye.GetNext() === "{" && we < ve; )
        Ee.push(ye.GetArgument(ge)), we++;
      var xe = !1, Le = " ", Qe = " ";
      ve > 2 && Ee.length > 2 ? (Le = "^{" + (Ee.length - 1) + "}", xe = !0) : Se != null && (ve > 2 && Ee.length > 1 && (xe = !0), Le = "^{" + Se + "}", Qe = Le);
      for (var ke = Ce ? "\\flatfrac" : "\\frac", Me = Ee.length > 1 ? Ee[0] : "", Re = Ee.length > 1 ? Ee[1] : Ee[0], De = "", Pe = 2, $e = void 0; $e = Ee[Pe]; Pe++)
        De += be + " " + $e;
      var qe = ke + "{" + be + Le + Me + "}{" + be + " " + Re + Qe + " " + De + "}";
      ye.Push(new V.default(qe, ye.stack.env, ye.configuration).mml()), ye.GetNext() === "(" && (ye.i++, ye.Push(ye.itemFactory.create("auto open").setProperties({ open: "(", close: ")", ignore: xe })));
    }, ne.Bra = function(ye, ge) {
      var ve = ye.GetStar(), be = ye.GetArgument(ge), Ce = "", Se = !1, we = !1;
      if (ye.GetNext() === "\\") {
        var Ee = ye.i;
        ye.i++;
        var xe = ye.GetCS(), Le = ye.lookup("macro", xe);
        Le && Le.symbol === "ket" ? (Se = !0, Ee = ye.i, we = ye.GetStar(), ye.GetNext() === "{" ? Ce = ye.GetArgument(xe, !0) : (ye.i = Ee, we = !1)) : ye.i = Ee;
      }
      var Qe = "";
      Se ? Qe = ve || we ? "\\langle{".concat(be, "}\\vert{").concat(Ce, "}\\rangle") : "\\left\\langle{".concat(be, "}\\middle\\vert{").concat(Ce, "}\\right\\rangle") : Qe = ve || we ? "\\langle{".concat(be, "}\\vert") : "\\left\\langle{".concat(be, "}\\right\\vert{").concat(Ce, "}"), ye.Push(new V.default(Qe, ye.stack.env, ye.configuration).mml());
    }, ne.Ket = function(ye, ge) {
      var ve = ye.GetStar(), be = ye.GetArgument(ge), Ce = ve ? "\\vert{".concat(be, "}\\rangle") : "\\left\\vert{".concat(be, "}\\right\\rangle");
      ye.Push(new V.default(Ce, ye.stack.env, ye.configuration).mml());
    }, ne.BraKet = function(ye, ge) {
      var ve = ye.GetStar(), be = ye.GetArgument(ge), Ce = null;
      ye.GetNext() === "{" && (Ce = ye.GetArgument(ge, !0));
      var Se = "";
      Ce == null ? Se = ve ? "\\langle{".concat(be, "}\\vert{").concat(be, "}\\rangle") : "\\left\\langle{".concat(be, "}\\middle\\vert{").concat(be, "}\\right\\rangle") : Se = ve ? "\\langle{".concat(be, "}\\vert{").concat(Ce, "}\\rangle") : "\\left\\langle{".concat(be, "}\\middle\\vert{").concat(Ce, "}\\right\\rangle"), ye.Push(new V.default(Se, ye.stack.env, ye.configuration).mml());
    }, ne.KetBra = function(ye, ge) {
      var ve = ye.GetStar(), be = ye.GetArgument(ge), Ce = null;
      ye.GetNext() === "{" && (Ce = ye.GetArgument(ge, !0));
      var Se = "";
      Ce == null ? Se = ve ? "\\vert{".concat(be, "}\\rangle\\!\\langle{").concat(be, "}\\vert") : "\\left\\vert{".concat(be, "}\\middle\\rangle\\!\\middle\\langle{").concat(be, "}\\right\\vert") : Se = ve ? "\\vert{".concat(be, "}\\rangle\\!\\langle{").concat(Ce, "}\\vert") : "\\left\\vert{".concat(be, "}\\middle\\rangle\\!\\middle\\langle{").concat(Ce, "}\\right\\vert"), ye.Push(new V.default(Se, ye.stack.env, ye.configuration).mml());
    };
    function me(ye, ge, ve) {
      var be = O(ye, 3), Ce = be[0], Se = be[1], we = be[2];
      return ge && ve ? "\\left\\langle{".concat(Ce, "}\\middle\\vert{").concat(Se, "}\\middle\\vert{").concat(we, "}\\right\\rangle") : ge ? "\\langle{".concat(Ce, "}\\vert{").concat(Se, "}\\vert{").concat(we, "}\\rangle") : "\\left\\langle{".concat(Ce, "}\\right\\vert{").concat(Se, "}\\left\\vert{").concat(we, "}\\right\\rangle");
    }
    ne.Expectation = function(ye, ge) {
      var ve = ye.GetStar(), be = ve && ye.GetStar(), Ce = ye.GetArgument(ge), Se = null;
      ye.GetNext() === "{" && (Se = ye.GetArgument(ge, !0));
      var we = Ce && Se ? me([Se, Ce, Se], ve, be) : ve ? "\\langle {".concat(Ce, "} \\rangle") : "\\left\\langle {".concat(Ce, "} \\right\\rangle");
      ye.Push(new V.default(we, ye.stack.env, ye.configuration).mml());
    }, ne.MatrixElement = function(ye, ge) {
      var ve = ye.GetStar(), be = ve && ye.GetStar(), Ce = ye.GetArgument(ge), Se = ye.GetArgument(ge), we = ye.GetArgument(ge), Ee = me([Ce, Se, we], ve, be);
      ye.Push(new V.default(Ee, ye.stack.env, ye.configuration).mml());
    }, ne.MatrixQuantity = function(ye, ge, ve) {
      var be = ye.GetStar(), Ce = ye.GetNext(), Se = ve ? "smallmatrix" : "array", we = "", Ee = "", xe = "";
      switch (Ce) {
        case "{":
          we = ye.GetArgument(ge);
          break;
        case "(":
          ye.i++, Ee = be ? "\\lgroup" : "(", xe = be ? "\\rgroup" : ")", we = ye.GetUpTo(ge, ")");
          break;
        case "[":
          ye.i++, Ee = "[", xe = "]", we = ye.GetUpTo(ge, "]");
          break;
        case "|":
          ye.i++, Ee = "|", xe = "|", we = ye.GetUpTo(ge, "|");
          break;
        default:
          Ee = "(", xe = ")";
          break;
      }
      var Le = (Ee ? "\\left" : "") + Ee + "\\begin{" + Se + "}{} " + we + "\\end{" + Se + "}" + (Ee ? "\\right" : "") + xe;
      ye.Push(new V.default(Le, ye.stack.env, ye.configuration).mml());
    }, ne.IdentityMatrix = function(ye, ge) {
      var ve = ye.GetArgument(ge), be = parseInt(ve, 10);
      if (isNaN(be))
        throw new q.default("InvalidNumber", "Invalid number");
      if (be <= 1) {
        ye.string = "1" + ye.string.slice(ye.i), ye.i = 0;
        return;
      }
      for (var Ce = Array(be).fill("0"), Se = [], we = 0; we < be; we++) {
        var Ee = Ce.slice();
        Ee[we] = "1", Se.push(Ee.join(" & "));
      }
      ye.string = Se.join("\\\\ ") + ye.string.slice(ye.i), ye.i = 0;
    }, ne.XMatrix = function(ye, ge) {
      var ve = ye.GetStar(), be = ye.GetArgument(ge), Ce = ye.GetArgument(ge), Se = ye.GetArgument(ge), we = parseInt(Ce, 10), Ee = parseInt(Se, 10);
      if (isNaN(we) || isNaN(Ee) || Ee.toString() !== Se || we.toString() !== Ce)
        throw new q.default("InvalidNumber", "Invalid number");
      if (we = we < 1 ? 1 : we, Ee = Ee < 1 ? 1 : Ee, !ve) {
        var xe = Array(Ee).fill(be).join(" & "), Le = Array(we).fill(xe).join("\\\\ ");
        ye.string = Le + ye.string.slice(ye.i), ye.i = 0;
        return;
      }
      var Qe = "";
      if (we === 1 && Ee === 1)
        Qe = be;
      else if (we === 1) {
        for (var xe = [], ke = 1; ke <= Ee; ke++)
          xe.push("".concat(be, "_{").concat(ke, "}"));
        Qe = xe.join(" & ");
      } else if (Ee === 1) {
        for (var xe = [], ke = 1; ke <= we; ke++)
          xe.push("".concat(be, "_{").concat(ke, "}"));
        Qe = xe.join("\\\\ ");
      } else {
        for (var Me = [], ke = 1; ke <= we; ke++) {
          for (var xe = [], Re = 1; Re <= Ee; Re++)
            xe.push("".concat(be, "_{{").concat(ke, "}{").concat(Re, "}}"));
          Me.push(xe.join(" & "));
        }
        Qe = Me.join("\\\\ ");
      }
      ye.string = Qe + ye.string.slice(ye.i), ye.i = 0;
    }, ne.PauliMatrix = function(ye, ge) {
      var ve = ye.GetArgument(ge), be = ve.slice(1);
      switch (ve[0]) {
        case "0":
          be += " 1 & 0\\\\ 0 & 1";
          break;
        case "1":
        case "x":
          be += " 0 & 1\\\\ 1 & 0";
          break;
        case "2":
        case "y":
          be += " 0 & -i\\\\ i & 0";
          break;
        case "3":
        case "z":
          be += " 1 & 0\\\\ 0 & -1";
          break;
      }
      ye.string = be + ye.string.slice(ye.i), ye.i = 0;
    }, ne.DiagonalMatrix = function(ye, ge, ve) {
      if (ye.GetNext() === "{") {
        var be = ye.i;
        ye.GetArgument(ge);
        var Ce = ye.i;
        ye.i = be + 1;
        for (var Se = [], we = "", Ee = ye.i; Ee < Ce; ) {
          try {
            we = ye.GetUpTo(ge, ",");
          } catch {
            ye.i = Ce, Se.push(ye.string.slice(Ee, Ce - 1));
            break;
          }
          if (ye.i >= Ce) {
            Se.push(ye.string.slice(Ee, Ce));
            break;
          }
          Ee = ye.i, Se.push(we);
        }
        ye.string = pe(Se, ve) + ye.string.slice(Ce), ye.i = 0;
      }
    };
    function pe(ye, ge) {
      for (var ve = ye.length, be = [], Ce = 0; Ce < ve; Ce++)
        be.push(Array(ge ? ve - Ce : Ce + 1).join("&") + "\\mqty{" + ye[Ce] + "}");
      return be.join("\\\\ ");
    }
    return ne.AutoClose = function(ye, ge, ve) {
      var be = ye.create("token", "mo", { stretchy: !1 }, ge), Ce = ye.itemFactory.create("mml", be).setProperties({ autoclose: ge });
      ye.Push(Ce);
    }, ne.Vnabla = function(ye, ge) {
      var ve = ye.options.physics.arrowdel ? "\\vec{\\gradientnabla}" : "{\\gradientnabla}";
      return ye.Push(new V.default(ve, ye.stack.env, ye.configuration).mml());
    }, ne.DiffD = function(ye, ge) {
      var ve = ye.options.physics.italicdiff ? "d" : "{\\rm d}";
      return ye.Push(new V.default(ve, ye.stack.env, ye.configuration).mml());
    }, ne.Macro = H.default.Macro, ne.NamedFn = H.default.NamedFn, ne.Array = H.default.Array, PhysicsMethods.default = ne, PhysicsMethods;
  }
  var hasRequiredPhysicsMappings;
  function requirePhysicsMappings() {
    if (hasRequiredPhysicsMappings) return PhysicsMappings;
    hasRequiredPhysicsMappings = 1;
    var O = PhysicsMappings.__importDefault || function(J) {
      return J && J.__esModule ? J : { default: J };
    };
    Object.defineProperty(PhysicsMappings, "__esModule", { value: !0 });
    var _ = requireSymbolMap(), H = O(requirePhysicsMethods()), V = requireTexConstants(), q = O(requireParseMethods()), G = requireMmlNode();
    return new _.CommandMap("Physics-automatic-bracing-macros", {
      quantity: "Quantity",
      qty: "Quantity",
      pqty: ["Quantity", "(", ")", !0],
      bqty: ["Quantity", "[", "]", !0],
      vqty: ["Quantity", "|", "|", !0],
      Bqty: ["Quantity", "\\{", "\\}", !0],
      absolutevalue: ["Quantity", "|", "|", !0],
      abs: ["Quantity", "|", "|", !0],
      norm: ["Quantity", "\\|", "\\|", !0],
      evaluated: "Eval",
      eval: "Eval",
      order: [
        "Quantity",
        "(",
        ")",
        !0,
        "O",
        V.TexConstant.Variant.CALLIGRAPHIC
      ],
      commutator: "Commutator",
      comm: "Commutator",
      anticommutator: ["Commutator", "\\{", "\\}"],
      acomm: ["Commutator", "\\{", "\\}"],
      poissonbracket: ["Commutator", "\\{", "\\}"],
      pb: ["Commutator", "\\{", "\\}"]
    }, H.default), new _.CharacterMap("Physics-vector-mo", q.default.mathchar0mo, {
      dotproduct: ["⋅", { mathvariant: V.TexConstant.Variant.BOLD }],
      vdot: ["⋅", { mathvariant: V.TexConstant.Variant.BOLD }],
      crossproduct: "×",
      cross: "×",
      cp: "×",
      gradientnabla: ["∇", { mathvariant: V.TexConstant.Variant.BOLD }]
    }), new _.CharacterMap("Physics-vector-mi", q.default.mathchar0mi, {
      real: ["ℜ", { mathvariant: V.TexConstant.Variant.NORMAL }],
      imaginary: ["ℑ", { mathvariant: V.TexConstant.Variant.NORMAL }]
    }), new _.CommandMap("Physics-vector-macros", {
      vnabla: "Vnabla",
      vectorbold: "VectorBold",
      vb: "VectorBold",
      vectorarrow: ["StarMacro", 1, "\\vec{\\vb", "{#1}}"],
      va: ["StarMacro", 1, "\\vec{\\vb", "{#1}}"],
      vectorunit: ["StarMacro", 1, "\\hat{\\vb", "{#1}}"],
      vu: ["StarMacro", 1, "\\hat{\\vb", "{#1}}"],
      gradient: ["OperatorApplication", "\\vnabla", "(", "["],
      grad: ["OperatorApplication", "\\vnabla", "(", "["],
      divergence: ["VectorOperator", "\\vnabla\\vdot", "(", "["],
      div: ["VectorOperator", "\\vnabla\\vdot", "(", "["],
      curl: ["VectorOperator", "\\vnabla\\crossproduct", "(", "["],
      laplacian: ["OperatorApplication", "\\nabla^2", "(", "["]
    }, H.default), new _.CommandMap("Physics-expressions-macros", {
      sin: "Expression",
      sinh: "Expression",
      arcsin: "Expression",
      asin: "Expression",
      cos: "Expression",
      cosh: "Expression",
      arccos: "Expression",
      acos: "Expression",
      tan: "Expression",
      tanh: "Expression",
      arctan: "Expression",
      atan: "Expression",
      csc: "Expression",
      csch: "Expression",
      arccsc: "Expression",
      acsc: "Expression",
      sec: "Expression",
      sech: "Expression",
      arcsec: "Expression",
      asec: "Expression",
      cot: "Expression",
      coth: "Expression",
      arccot: "Expression",
      acot: "Expression",
      exp: ["Expression", !1],
      log: "Expression",
      ln: "Expression",
      det: ["Expression", !1],
      Pr: ["Expression", !1],
      tr: ["Expression", !1],
      trace: ["Expression", !1, "tr"],
      Tr: ["Expression", !1],
      Trace: ["Expression", !1, "Tr"],
      rank: "NamedFn",
      erf: ["Expression", !1],
      Residue: ["Macro", "\\mathrm{Res}"],
      Res: ["OperatorApplication", "\\Residue", "(", "[", "{"],
      principalvalue: ["OperatorApplication", "{\\cal P}"],
      pv: ["OperatorApplication", "{\\cal P}"],
      PV: ["OperatorApplication", "{\\rm P.V.}"],
      Re: ["OperatorApplication", "\\mathrm{Re}", "{"],
      Im: ["OperatorApplication", "\\mathrm{Im}", "{"],
      sine: ["NamedFn", "sin"],
      hypsine: ["NamedFn", "sinh"],
      arcsine: ["NamedFn", "arcsin"],
      asine: ["NamedFn", "asin"],
      cosine: ["NamedFn", "cos"],
      hypcosine: ["NamedFn", "cosh"],
      arccosine: ["NamedFn", "arccos"],
      acosine: ["NamedFn", "acos"],
      tangent: ["NamedFn", "tan"],
      hyptangent: ["NamedFn", "tanh"],
      arctangent: ["NamedFn", "arctan"],
      atangent: ["NamedFn", "atan"],
      cosecant: ["NamedFn", "csc"],
      hypcosecant: ["NamedFn", "csch"],
      arccosecant: ["NamedFn", "arccsc"],
      acosecant: ["NamedFn", "acsc"],
      secant: ["NamedFn", "sec"],
      hypsecant: ["NamedFn", "sech"],
      arcsecant: ["NamedFn", "arcsec"],
      asecant: ["NamedFn", "asec"],
      cotangent: ["NamedFn", "cot"],
      hypcotangent: ["NamedFn", "coth"],
      arccotangent: ["NamedFn", "arccot"],
      acotangent: ["NamedFn", "acot"],
      exponential: ["NamedFn", "exp"],
      logarithm: ["NamedFn", "log"],
      naturallogarithm: ["NamedFn", "ln"],
      determinant: ["NamedFn", "det"],
      Probability: ["NamedFn", "Pr"]
    }, H.default), new _.CommandMap("Physics-quick-quad-macros", {
      qqtext: "Qqtext",
      qq: "Qqtext",
      qcomma: ["Macro", "\\qqtext*{,}"],
      qc: ["Macro", "\\qqtext*{,}"],
      qcc: ["Qqtext", "c.c."],
      qif: ["Qqtext", "if"],
      qthen: ["Qqtext", "then"],
      qelse: ["Qqtext", "else"],
      qotherwise: ["Qqtext", "otherwise"],
      qunless: ["Qqtext", "unless"],
      qgiven: ["Qqtext", "given"],
      qusing: ["Qqtext", "using"],
      qassume: ["Qqtext", "assume"],
      qsince: ["Qqtext", "since"],
      qlet: ["Qqtext", "let"],
      qfor: ["Qqtext", "for"],
      qall: ["Qqtext", "all"],
      qeven: ["Qqtext", "even"],
      qodd: ["Qqtext", "odd"],
      qinteger: ["Qqtext", "integer"],
      qand: ["Qqtext", "and"],
      qor: ["Qqtext", "or"],
      qas: ["Qqtext", "as"],
      qin: ["Qqtext", "in"]
    }, H.default), new _.CommandMap("Physics-derivative-macros", {
      diffd: "DiffD",
      flatfrac: ["Macro", "\\left.#1\\middle/#2\\right.", 2],
      differential: ["Differential", "\\diffd"],
      dd: ["Differential", "\\diffd"],
      variation: ["Differential", "\\delta"],
      var: ["Differential", "\\delta"],
      derivative: ["Derivative", 2, "\\diffd"],
      dv: ["Derivative", 2, "\\diffd"],
      partialderivative: ["Derivative", 3, "\\partial"],
      pderivative: ["Derivative", 3, "\\partial"],
      pdv: ["Derivative", 3, "\\partial"],
      functionalderivative: ["Derivative", 2, "\\delta"],
      fderivative: ["Derivative", 2, "\\delta"],
      fdv: ["Derivative", 2, "\\delta"]
    }, H.default), new _.CommandMap("Physics-bra-ket-macros", {
      bra: "Bra",
      ket: "Ket",
      innerproduct: "BraKet",
      ip: "BraKet",
      braket: "BraKet",
      outerproduct: "KetBra",
      dyad: "KetBra",
      ketbra: "KetBra",
      op: "KetBra",
      expectationvalue: "Expectation",
      expval: "Expectation",
      ev: "Expectation",
      matrixelement: "MatrixElement",
      matrixel: "MatrixElement",
      mel: "MatrixElement"
    }, H.default), new _.CommandMap("Physics-matrix-macros", {
      matrixquantity: "MatrixQuantity",
      mqty: "MatrixQuantity",
      pmqty: ["Macro", "\\mqty(#1)", 1],
      Pmqty: ["Macro", "\\mqty*(#1)", 1],
      bmqty: ["Macro", "\\mqty[#1]", 1],
      vmqty: ["Macro", "\\mqty|#1|", 1],
      smallmatrixquantity: ["MatrixQuantity", !0],
      smqty: ["MatrixQuantity", !0],
      spmqty: ["Macro", "\\smqty(#1)", 1],
      sPmqty: ["Macro", "\\smqty*(#1)", 1],
      sbmqty: ["Macro", "\\smqty[#1]", 1],
      svmqty: ["Macro", "\\smqty|#1|", 1],
      matrixdeterminant: ["Macro", "\\vmqty{#1}", 1],
      mdet: ["Macro", "\\vmqty{#1}", 1],
      smdet: ["Macro", "\\svmqty{#1}", 1],
      identitymatrix: "IdentityMatrix",
      imat: "IdentityMatrix",
      xmatrix: "XMatrix",
      xmat: "XMatrix",
      zeromatrix: ["Macro", "\\xmat{0}{#1}{#2}", 2],
      zmat: ["Macro", "\\xmat{0}{#1}{#2}", 2],
      paulimatrix: "PauliMatrix",
      pmat: "PauliMatrix",
      diagonalmatrix: "DiagonalMatrix",
      dmat: "DiagonalMatrix",
      antidiagonalmatrix: ["DiagonalMatrix", !0],
      admat: ["DiagonalMatrix", !0]
    }, H.default), new _.EnvironmentMap("Physics-aux-envs", q.default.environment, {
      smallmatrix: ["Array", null, null, null, "c", "0.333em", ".2em", "S", 1]
    }, H.default), new _.MacroMap("Physics-characters", {
      "|": ["AutoClose", G.TEXCLASS.ORD],
      ")": "AutoClose",
      "]": "AutoClose"
    }, H.default), PhysicsMappings;
  }
  var hasRequiredPhysicsConfiguration;
  function requirePhysicsConfiguration() {
    if (hasRequiredPhysicsConfiguration) return PhysicsConfiguration;
    hasRequiredPhysicsConfiguration = 1;
    var O;
    Object.defineProperty(PhysicsConfiguration, "__esModule", { value: !0 }), PhysicsConfiguration.PhysicsConfiguration = void 0;
    var _ = requireConfiguration(), H = requirePhysicsItems();
    return requirePhysicsMappings(), PhysicsConfiguration.PhysicsConfiguration = _.Configuration.create("physics", {
      handler: {
        macro: [
          "Physics-automatic-bracing-macros",
          "Physics-vector-macros",
          "Physics-vector-mo",
          "Physics-vector-mi",
          "Physics-derivative-macros",
          "Physics-expressions-macros",
          "Physics-quick-quad-macros",
          "Physics-bra-ket-macros",
          "Physics-matrix-macros"
        ],
        character: ["Physics-characters"],
        environment: ["Physics-aux-envs"]
      },
      items: (O = {}, O[H.AutoOpen.prototype.kind] = H.AutoOpen, O),
      options: {
        physics: {
          italicdiff: !1,
          arrowdel: !1
        }
      }
    }), PhysicsConfiguration;
  }
  var SetOptionsConfiguration = {}, hasRequiredSetOptionsConfiguration;
  function requireSetOptionsConfiguration() {
    return hasRequiredSetOptionsConfiguration || (hasRequiredSetOptionsConfiguration = 1, function(O) {
      var _ = SetOptionsConfiguration.__values || function(oe) {
        var se = typeof Symbol == "function" && Symbol.iterator, ce = se && oe[se], le = 0;
        if (ce) return ce.call(oe);
        if (oe && typeof oe.length == "number") return {
          next: function() {
            return oe && le >= oe.length && (oe = void 0), { value: oe && oe[le++], done: !oe };
          }
        };
        throw new TypeError(se ? "Object is not iterable." : "Symbol.iterator is not defined.");
      }, H = SetOptionsConfiguration.__importDefault || function(oe) {
        return oe && oe.__esModule ? oe : { default: oe };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.SetOptionsConfiguration = O.SetOptionsUtil = void 0;
      var V = requireConfiguration(), q = requireSymbolMap(), G = H(requireTexError()), J = H(requireParseUtil()), Z = require_Symbol(), te = H(requireBaseMethods()), ne = requireOptions();
      O.SetOptionsUtil = {
        filterPackage: function(oe, se) {
          if (se !== "tex" && !V.ConfigurationHandler.get(se))
            throw new G.default("NotAPackage", "Not a defined package: %1", se);
          var ce = oe.options.setoptions, le = ce.allowOptions[se];
          if (le === void 0 && !ce.allowPackageDefault || le === !1)
            throw new G.default("PackageNotSettable", `Options can't be set for package "%1"`, se);
          return !0;
        },
        filterOption: function(oe, se, ce) {
          var le, ue = oe.options.setoptions, de = ue.allowOptions[se] || {}, fe = de.hasOwnProperty(ce) && !(0, ne.isObject)(de[ce]) ? de[ce] : null;
          if (fe === !1 || fe === null && !ue.allowOptionsDefault)
            throw new G.default("OptionNotSettable", 'Option "%1" is not allowed to be set', ce);
          if (!(!((le = se === "tex" ? oe.options : oe.options[se]) === null || le === void 0) && le.hasOwnProperty(ce)))
            throw se === "tex" ? new G.default("InvalidTexOption", 'Invalid TeX option "%1"', ce) : new G.default("InvalidOptionKey", 'Invalid option "%1" for package "%2"', ce, se);
          return !0;
        },
        filterValue: function(oe, se, ce, le) {
          return le;
        }
      };
      var ee = new q.CommandMap("setoptions", {
        setOptions: "SetOptions"
      }, {
        SetOptions: function(oe, se) {
          var ce, le, ue = oe.GetBrackets(se) || "tex", de = J.default.keyvalOptions(oe.GetArgument(se)), fe = oe.options.setoptions;
          if (fe.filterPackage(oe, ue))
            try {
              for (var he = _(Object.keys(de)), me = he.next(); !me.done; me = he.next()) {
                var pe = me.value;
                fe.filterOption(oe, ue, pe) && ((ue === "tex" ? oe.options : oe.options[ue])[pe] = fe.filterValue(oe, ue, pe, de[pe]));
              }
            } catch (ye) {
              ce = { error: ye };
            } finally {
              try {
                me && !me.done && (le = he.return) && le.call(he);
              } finally {
                if (ce) throw ce.error;
              }
            }
        }
      });
      function ae(oe, se) {
        var ce = se.parseOptions.handlers.get("macro").lookup("require");
        ce && (ee.add("Require", new Z.Macro("Require", ce._func)), ee.add("require", new Z.Macro("require", te.default.Macro, ["\\Require{#2}\\setOptions[#2]{#1}", 2, ""])));
      }
      O.SetOptionsConfiguration = V.Configuration.create("setoptions", {
        handler: { macro: ["setoptions"] },
        config: ae,
        priority: 3,
        options: {
          setoptions: {
            filterPackage: O.SetOptionsUtil.filterPackage,
            filterOption: O.SetOptionsUtil.filterOption,
            filterValue: O.SetOptionsUtil.filterValue,
            allowPackageDefault: !0,
            allowOptionsDefault: !0,
            allowOptions: (0, ne.expandable)({
              tex: {
                FindTeX: !1,
                formatError: !1,
                package: !1,
                baseURL: !1,
                tags: !1,
                maxBuffer: !1,
                maxMaxros: !1,
                macros: !1,
                environments: !1
              },
              setoptions: !1,
              autoload: !1,
              require: !1,
              configmacros: !1,
              tagformat: !1
            })
          }
        }
      });
    }(SetOptionsConfiguration)), SetOptionsConfiguration;
  }
  var TagFormatConfiguration = {}, hasRequiredTagFormatConfiguration;
  function requireTagFormatConfiguration() {
    if (hasRequiredTagFormatConfiguration) return TagFormatConfiguration;
    hasRequiredTagFormatConfiguration = 1;
    var O = TagFormatConfiguration.__extends || /* @__PURE__ */ function() {
      var G = function(J, Z) {
        return G = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(te, ne) {
          te.__proto__ = ne;
        } || function(te, ne) {
          for (var ee in ne) Object.prototype.hasOwnProperty.call(ne, ee) && (te[ee] = ne[ee]);
        }, G(J, Z);
      };
      return function(J, Z) {
        if (typeof Z != "function" && Z !== null)
          throw new TypeError("Class extends value " + String(Z) + " is not a constructor or null");
        G(J, Z);
        function te() {
          this.constructor = J;
        }
        J.prototype = Z === null ? Object.create(Z) : (te.prototype = Z.prototype, new te());
      };
    }();
    Object.defineProperty(TagFormatConfiguration, "__esModule", { value: !0 }), TagFormatConfiguration.TagFormatConfiguration = TagFormatConfiguration.tagformatConfig = void 0;
    var _ = requireConfiguration(), H = requireTags(), V = 0;
    function q(G, J) {
      var Z = J.parseOptions.options.tags;
      Z !== "base" && G.tags.hasOwnProperty(Z) && H.TagsFactory.add(Z, G.tags[Z]);
      var te = H.TagsFactory.create(J.parseOptions.options.tags).constructor, ne = function(ae) {
        O(oe, ae);
        function oe() {
          return ae !== null && ae.apply(this, arguments) || this;
        }
        return oe.prototype.formatNumber = function(se) {
          return J.parseOptions.options.tagformat.number(se);
        }, oe.prototype.formatTag = function(se) {
          return J.parseOptions.options.tagformat.tag(se);
        }, oe.prototype.formatId = function(se) {
          return J.parseOptions.options.tagformat.id(se);
        }, oe.prototype.formatUrl = function(se, ce) {
          return J.parseOptions.options.tagformat.url(se, ce);
        }, oe;
      }(te);
      V++;
      var ee = "configTags-" + V;
      H.TagsFactory.add(ee, ne), J.parseOptions.options.tags = ee;
    }
    return TagFormatConfiguration.tagformatConfig = q, TagFormatConfiguration.TagFormatConfiguration = _.Configuration.create("tagformat", {
      config: [q, 10],
      options: {
        tagformat: {
          number: function(G) {
            return G.toString();
          },
          tag: function(G) {
            return "(" + G + ")";
          },
          id: function(G) {
            return "mjx-eqn:" + G.replace(/\s/g, "_");
          },
          url: function(G, J) {
            return J + "#" + encodeURIComponent(G);
          }
        }
      }
    }), TagFormatConfiguration;
  }
  var TextcompConfiguration = {}, TextcompMappings = {}, TextMacrosMethods = {}, hasRequiredTextMacrosMethods;
  function requireTextMacrosMethods() {
    if (hasRequiredTextMacrosMethods) return TextMacrosMethods;
    hasRequiredTextMacrosMethods = 1;
    var O = TextMacrosMethods.__importDefault || function(q) {
      return q && q.__esModule ? q : { default: q };
    };
    Object.defineProperty(TextMacrosMethods, "__esModule", { value: !0 }), TextMacrosMethods.TextMacrosMethods = void 0;
    var _ = O(requireTexParser()), H = requireRetries(), V = O(requireBaseMethods());
    return TextMacrosMethods.TextMacrosMethods = {
      Comment: function(q, G) {
        for (; q.i < q.string.length && q.string.charAt(q.i) !== `
`; )
          q.i++;
        q.i++;
      },
      Math: function(q, G) {
        q.saveText();
        for (var J = q.i, Z, te, ne = 0; te = q.GetNext(); )
          switch (Z = q.i++, te) {
            case "\\":
              var ee = q.GetCS();
              ee === ")" && (te = "\\(");
            case "$":
              if (ne === 0 && G === te) {
                var ae = q.texParser.configuration, oe = new _.default(q.string.substr(J, Z - J), q.stack.env, ae).mml();
                q.PushMath(oe);
                return;
              }
              break;
            case "{":
              ne++;
              break;
            case "}":
              ne === 0 && q.Error("ExtraCloseMissingOpen", "Extra close brace or missing open brace"), ne--;
              break;
          }
        q.Error("MathNotTerminated", "Math-mode is not properly terminated");
      },
      MathModeOnly: function(q, G) {
        q.Error("MathModeOnly", "'%1' allowed only in math mode", G);
      },
      Misplaced: function(q, G) {
        q.Error("Misplaced", "'%1' can not be used here", G);
      },
      OpenBrace: function(q, G) {
        var J = q.stack.env;
        q.envStack.push(J), q.stack.env = Object.assign({}, J);
      },
      CloseBrace: function(q, G) {
        q.envStack.length ? (q.saveText(), q.stack.env = q.envStack.pop()) : q.Error("ExtraCloseMissingOpen", "Extra close brace or missing open brace");
      },
      OpenQuote: function(q, G) {
        q.string.charAt(q.i) === G ? (q.text += "“", q.i++) : q.text += "‘";
      },
      CloseQuote: function(q, G) {
        q.string.charAt(q.i) === G ? (q.text += "”", q.i++) : q.text += "’";
      },
      Tilde: function(q, G) {
        q.text += " ";
      },
      Space: function(q, G) {
        for (q.text += " "; q.GetNext().match(/\s/); )
          q.i++;
      },
      SelfQuote: function(q, G) {
        q.text += G.substr(1);
      },
      Insert: function(q, G, J) {
        q.text += J;
      },
      Accent: function(q, G, J) {
        var Z = q.ParseArg(G), te = q.create("token", "mo", {}, J);
        q.addAttributes(te), q.Push(q.create("node", "mover", [Z, te]));
      },
      Emph: function(q, G) {
        var J = q.stack.env.mathvariant === "-tex-mathit" ? "normal" : "-tex-mathit";
        q.Push(q.ParseTextArg(G, { mathvariant: J }));
      },
      SetFont: function(q, G, J) {
        q.saveText(), q.stack.env.mathvariant = J;
      },
      SetSize: function(q, G, J) {
        q.saveText(), q.stack.env.mathsize = J;
      },
      CheckAutoload: function(q, G) {
        var J = q.configuration.packageData.get("autoload"), Z = q.texParser;
        G = G.slice(1);
        var te = Z.lookup("macro", G);
        if (!te || J && te._func === J.Autoload) {
          if (Z.parse("macro", [Z, G]), !te)
            return;
          (0, H.retryAfter)(Promise.resolve());
        }
        Z.parse("macro", [q, G]);
      },
      Macro: V.default.Macro,
      Spacer: V.default.Spacer,
      Hskip: V.default.Hskip,
      rule: V.default.rule,
      Rule: V.default.Rule,
      HandleRef: V.default.HandleRef
    }, TextMacrosMethods;
  }
  var TextParser = {}, hasRequiredTextParser;
  function requireTextParser() {
    if (hasRequiredTextParser) return TextParser;
    hasRequiredTextParser = 1;
    var O = TextParser.__extends || /* @__PURE__ */ function() {
      var oe = function(se, ce) {
        return oe = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(le, ue) {
          le.__proto__ = ue;
        } || function(le, ue) {
          for (var de in ue) Object.prototype.hasOwnProperty.call(ue, de) && (le[de] = ue[de]);
        }, oe(se, ce);
      };
      return function(se, ce) {
        if (typeof ce != "function" && ce !== null)
          throw new TypeError("Class extends value " + String(ce) + " is not a constructor or null");
        oe(se, ce);
        function le() {
          this.constructor = se;
        }
        se.prototype = ce === null ? Object.create(ce) : (le.prototype = ce.prototype, new le());
      };
    }(), _ = TextParser.__values || function(oe) {
      var se = typeof Symbol == "function" && Symbol.iterator, ce = se && oe[se], le = 0;
      if (ce) return ce.call(oe);
      if (oe && typeof oe.length == "number") return {
        next: function() {
          return oe && le >= oe.length && (oe = void 0), { value: oe && oe[le++], done: !oe };
        }
      };
      throw new TypeError(se ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }, H = TextParser.__read || function(oe, se) {
      var ce = typeof Symbol == "function" && oe[Symbol.iterator];
      if (!ce) return oe;
      var le = ce.call(oe), ue, de = [], fe;
      try {
        for (; (se === void 0 || se-- > 0) && !(ue = le.next()).done; ) de.push(ue.value);
      } catch (he) {
        fe = { error: he };
      } finally {
        try {
          ue && !ue.done && (ce = le.return) && ce.call(le);
        } finally {
          if (fe) throw fe.error;
        }
      }
      return de;
    }, V = TextParser.__spreadArray || function(oe, se, ce) {
      if (ce || arguments.length === 2) for (var le = 0, ue = se.length, de; le < ue; le++)
        (de || !(le in se)) && (de || (de = Array.prototype.slice.call(se, 0, le)), de[le] = se[le]);
      return oe.concat(de || Array.prototype.slice.call(se));
    }, q = TextParser.__importDefault || function(oe) {
      return oe && oe.__esModule ? oe : { default: oe };
    };
    Object.defineProperty(TextParser, "__esModule", { value: !0 }), TextParser.TextParser = void 0;
    var G = q(requireTexParser()), J = q(requireTexError()), Z = q(requireParseUtil()), te = requireMmlNode(), ne = q(requireNodeUtil()), ee = requireBaseItems(), ae = function(oe) {
      O(se, oe);
      function se(ce, le, ue, de) {
        var fe = oe.call(this, ce, le, ue) || this;
        return fe.level = de, fe;
      }
      return Object.defineProperty(se.prototype, "texParser", {
        get: function() {
          return this.configuration.packageData.get("textmacros").texParser;
        },
        enumerable: !1,
        configurable: !0
      }), Object.defineProperty(se.prototype, "tags", {
        get: function() {
          return this.texParser.tags;
        },
        enumerable: !1,
        configurable: !0
      }), se.prototype.mml = function() {
        return this.level != null ? this.create("node", "mstyle", this.nodes, { displaystyle: !1, scriptlevel: this.level }) : this.nodes.length === 1 ? this.nodes[0] : this.create("node", "mrow", this.nodes);
      }, se.prototype.Parse = function() {
        this.text = "", this.nodes = [], this.envStack = [], oe.prototype.Parse.call(this);
      }, se.prototype.saveText = function() {
        if (this.text) {
          var ce = this.stack.env.mathvariant, le = Z.default.internalText(this, this.text, ce ? { mathvariant: ce } : {});
          this.text = "", this.Push(le);
        }
      }, se.prototype.Push = function(ce) {
        if (this.text && this.saveText(), ce instanceof ee.StopItem)
          return oe.prototype.Push.call(this, ce);
        if (ce instanceof ee.StyleItem) {
          this.stack.env.mathcolor = this.stack.env.color;
          return;
        }
        ce instanceof te.AbstractMmlNode && (this.addAttributes(ce), this.nodes.push(ce));
      }, se.prototype.PushMath = function(ce) {
        var le, ue, de = this.stack.env;
        ce.isKind("TeXAtom") || (ce = this.create("node", "TeXAtom", [ce]));
        try {
          for (var fe = _(["mathsize", "mathcolor"]), he = fe.next(); !he.done; he = fe.next()) {
            var me = he.value;
            de[me] && !ce.attributes.getExplicit(me) && (!ce.isToken && !ce.isKind("mstyle") && (ce = this.create("node", "mstyle", [ce])), ne.default.setAttribute(ce, me, de[me]));
          }
        } catch (pe) {
          le = { error: pe };
        } finally {
          try {
            he && !he.done && (ue = fe.return) && ue.call(fe);
          } finally {
            if (le) throw le.error;
          }
        }
        ce.isInferred && (ce = this.create("node", "mrow", ce.childNodes)), this.nodes.push(ce);
      }, se.prototype.addAttributes = function(ce) {
        var le, ue, de = this.stack.env;
        if (ce.isToken)
          try {
            for (var fe = _(["mathsize", "mathcolor", "mathvariant"]), he = fe.next(); !he.done; he = fe.next()) {
              var me = he.value;
              de[me] && !ce.attributes.getExplicit(me) && ne.default.setAttribute(ce, me, de[me]);
            }
          } catch (pe) {
            le = { error: pe };
          } finally {
            try {
              he && !he.done && (ue = fe.return) && ue.call(fe);
            } finally {
              if (le) throw le.error;
            }
          }
      }, se.prototype.ParseTextArg = function(ce, le) {
        var ue = this.GetArgument(ce);
        return le = Object.assign(Object.assign({}, this.stack.env), le), new se(ue, le, this.configuration).mml();
      }, se.prototype.ParseArg = function(ce) {
        return new se(this.GetArgument(ce), this.stack.env, this.configuration).mml();
      }, se.prototype.Error = function(ce, le) {
        for (var ue = [], de = 2; de < arguments.length; de++)
          ue[de - 2] = arguments[de];
        throw new (J.default.bind.apply(J.default, V([void 0, ce, le], H(ue), !1)))();
      }, se;
    }(G.default);
    return TextParser.TextParser = ae, TextParser;
  }
  var hasRequiredTextcompMappings;
  function requireTextcompMappings() {
    if (hasRequiredTextcompMappings) return TextcompMappings;
    hasRequiredTextcompMappings = 1;
    var O = TextcompMappings.__importDefault || function(J) {
      return J && J.__esModule ? J : { default: J };
    };
    Object.defineProperty(TextcompMappings, "__esModule", { value: !0 });
    var _ = requireSymbolMap(), H = requireTexConstants(), V = requireTextMacrosMethods(), q = O(requireParseUtil()), G = requireTextParser();
    return new _.CommandMap("textcomp-macros", {
      textasciicircum: ["Insert", "^"],
      textasciitilde: ["Insert", "~"],
      textasteriskcentered: ["Insert", "*"],
      textbackslash: ["Insert", "\\"],
      textbar: ["Insert", "|"],
      textbraceleft: ["Insert", "{"],
      textbraceright: ["Insert", "}"],
      textbullet: ["Insert", "•"],
      textdagger: ["Insert", "†"],
      textdaggerdbl: ["Insert", "‡"],
      textellipsis: ["Insert", "…"],
      textemdash: ["Insert", "—"],
      textendash: ["Insert", "–"],
      textexclamdown: ["Insert", "¡"],
      textgreater: ["Insert", ">"],
      textless: ["Insert", "<"],
      textordfeminine: ["Insert", "ª"],
      textordmasculine: ["Insert", "º"],
      textparagraph: ["Insert", "¶"],
      textperiodcentered: ["Insert", "·"],
      textquestiondown: ["Insert", "¿"],
      textquotedblleft: ["Insert", "“"],
      textquotedblright: ["Insert", "”"],
      textquoteleft: ["Insert", "‘"],
      textquoteright: ["Insert", "’"],
      textsection: ["Insert", "§"],
      textunderscore: ["Insert", "_"],
      textvisiblespace: ["Insert", "␣"],
      textacutedbl: ["Insert", "˝"],
      textasciiacute: ["Insert", "´"],
      textasciibreve: ["Insert", "˘"],
      textasciicaron: ["Insert", "ˇ"],
      textasciidieresis: ["Insert", "¨"],
      textasciimacron: ["Insert", "¯"],
      textgravedbl: ["Insert", "˵"],
      texttildelow: ["Insert", "˷"],
      textbaht: ["Insert", "฿"],
      textcent: ["Insert", "¢"],
      textcolonmonetary: ["Insert", "₡"],
      textcurrency: ["Insert", "¤"],
      textdollar: ["Insert", "$"],
      textdong: ["Insert", "₫"],
      texteuro: ["Insert", "€"],
      textflorin: ["Insert", "ƒ"],
      textguarani: ["Insert", "₲"],
      textlira: ["Insert", "₤"],
      textnaira: ["Insert", "₦"],
      textpeso: ["Insert", "₱"],
      textsterling: ["Insert", "£"],
      textwon: ["Insert", "₩"],
      textyen: ["Insert", "¥"],
      textcircledP: ["Insert", "℗"],
      textcompwordmark: ["Insert", "‌"],
      textcopyleft: ["Insert", "🄯"],
      textcopyright: ["Insert", "©"],
      textregistered: ["Insert", "®"],
      textservicemark: ["Insert", "℠"],
      texttrademark: ["Insert", "™"],
      textbardbl: ["Insert", "‖"],
      textbigcircle: ["Insert", "◯"],
      textblank: ["Insert", "␢"],
      textbrokenbar: ["Insert", "¦"],
      textdiscount: ["Insert", "⁒"],
      textestimated: ["Insert", "℮"],
      textinterrobang: ["Insert", "‽"],
      textinterrobangdown: ["Insert", "⸘"],
      textmusicalnote: ["Insert", "♪"],
      textnumero: ["Insert", "№"],
      textopenbullet: ["Insert", "◦"],
      textpertenthousand: ["Insert", "‱"],
      textperthousand: ["Insert", "‰"],
      textrecipe: ["Insert", "℞"],
      textreferencemark: ["Insert", "※"],
      textlangle: ["Insert", "〈"],
      textrangle: ["Insert", "〉"],
      textlbrackdbl: ["Insert", "⟦"],
      textrbrackdbl: ["Insert", "⟧"],
      textlquill: ["Insert", "⁅"],
      textrquill: ["Insert", "⁆"],
      textcelsius: ["Insert", "℃"],
      textdegree: ["Insert", "°"],
      textdiv: ["Insert", "÷"],
      textdownarrow: ["Insert", "↓"],
      textfractionsolidus: ["Insert", "⁄"],
      textleftarrow: ["Insert", "←"],
      textlnot: ["Insert", "¬"],
      textmho: ["Insert", "℧"],
      textminus: ["Insert", "−"],
      textmu: ["Insert", "µ"],
      textohm: ["Insert", "Ω"],
      textonehalf: ["Insert", "½"],
      textonequarter: ["Insert", "¼"],
      textonesuperior: ["Insert", "¹"],
      textpm: ["Insert", "±"],
      textrightarrow: ["Insert", "→"],
      textsurd: ["Insert", "√"],
      textthreequarters: ["Insert", "¾"],
      textthreesuperior: ["Insert", "³"],
      texttimes: ["Insert", "×"],
      texttwosuperior: ["Insert", "²"],
      textuparrow: ["Insert", "↑"],
      textborn: ["Insert", "*"],
      textdied: ["Insert", "†"],
      textdivorced: ["Insert", "⚮"],
      textmarried: ["Insert", "⚭"],
      textcentoldstyle: ["Insert", "¢", H.TexConstant.Variant.OLDSTYLE],
      textdollaroldstyle: ["Insert", "$", H.TexConstant.Variant.OLDSTYLE],
      textzerooldstyle: ["Insert", "0", H.TexConstant.Variant.OLDSTYLE],
      textoneoldstyle: ["Insert", "1", H.TexConstant.Variant.OLDSTYLE],
      texttwooldstyle: ["Insert", "2", H.TexConstant.Variant.OLDSTYLE],
      textthreeoldstyle: ["Insert", "3", H.TexConstant.Variant.OLDSTYLE],
      textfouroldstyle: ["Insert", "4", H.TexConstant.Variant.OLDSTYLE],
      textfiveoldstyle: ["Insert", "5", H.TexConstant.Variant.OLDSTYLE],
      textsixoldstyle: ["Insert", "6", H.TexConstant.Variant.OLDSTYLE],
      textsevenoldstyle: ["Insert", "7", H.TexConstant.Variant.OLDSTYLE],
      texteightoldstyle: ["Insert", "8", H.TexConstant.Variant.OLDSTYLE],
      textnineoldstyle: ["Insert", "9", H.TexConstant.Variant.OLDSTYLE]
    }, {
      Insert: function(J, Z, te, ne) {
        if (J instanceof G.TextParser) {
          if (!ne) {
            V.TextMacrosMethods.Insert(J, Z, te);
            return;
          }
          J.saveText();
        }
        J.Push(q.default.internalText(J, te, ne ? { mathvariant: ne } : {}));
      }
    }), TextcompMappings;
  }
  var hasRequiredTextcompConfiguration;
  function requireTextcompConfiguration() {
    if (hasRequiredTextcompConfiguration) return TextcompConfiguration;
    hasRequiredTextcompConfiguration = 1, Object.defineProperty(TextcompConfiguration, "__esModule", { value: !0 }), TextcompConfiguration.TextcompConfiguration = void 0;
    var O = requireConfiguration();
    return requireTextcompMappings(), TextcompConfiguration.TextcompConfiguration = O.Configuration.create("textcomp", {
      handler: { macro: ["textcomp-macros"] }
    }), TextcompConfiguration;
  }
  var TextMacrosConfiguration = {}, TextMacrosMappings = {}, hasRequiredTextMacrosMappings;
  function requireTextMacrosMappings() {
    if (hasRequiredTextMacrosMappings) return TextMacrosMappings;
    hasRequiredTextMacrosMappings = 1, Object.defineProperty(TextMacrosMappings, "__esModule", { value: !0 });
    var O = requireSymbolMap(), _ = requireTexConstants(), H = requireTextMacrosMethods(), V = requireLengths();
    return new O.MacroMap("text-special", {
      $: "Math",
      "%": "Comment",
      "^": "MathModeOnly",
      _: "MathModeOnly",
      "&": "Misplaced",
      "#": "Misplaced",
      "~": "Tilde",
      " ": "Space",
      "	": "Space",
      "\r": "Space",
      "\n": "Space",
      " ": "Tilde",
      "{": "OpenBrace",
      "}": "CloseBrace",
      "`": "OpenQuote",
      "'": "CloseQuote"
    }, H.TextMacrosMethods), new O.CommandMap("text-macros", {
      "(": "Math",
      $: "SelfQuote",
      _: "SelfQuote",
      "%": "SelfQuote",
      "{": "SelfQuote",
      "}": "SelfQuote",
      " ": "SelfQuote",
      "&": "SelfQuote",
      "#": "SelfQuote",
      "\\": "SelfQuote",
      "'": ["Accent", "´"],
      "’": ["Accent", "´"],
      "`": ["Accent", "`"],
      "‘": ["Accent", "`"],
      "^": ["Accent", "^"],
      '"': ["Accent", "¨"],
      "~": ["Accent", "~"],
      "=": ["Accent", "¯"],
      ".": ["Accent", "˙"],
      u: ["Accent", "˘"],
      v: ["Accent", "ˇ"],
      emph: "Emph",
      rm: ["SetFont", _.TexConstant.Variant.NORMAL],
      mit: ["SetFont", _.TexConstant.Variant.ITALIC],
      oldstyle: ["SetFont", _.TexConstant.Variant.OLDSTYLE],
      cal: ["SetFont", _.TexConstant.Variant.CALLIGRAPHIC],
      it: ["SetFont", "-tex-mathit"],
      bf: ["SetFont", _.TexConstant.Variant.BOLD],
      bbFont: ["SetFont", _.TexConstant.Variant.DOUBLESTRUCK],
      scr: ["SetFont", _.TexConstant.Variant.SCRIPT],
      frak: ["SetFont", _.TexConstant.Variant.FRAKTUR],
      sf: ["SetFont", _.TexConstant.Variant.SANSSERIF],
      tt: ["SetFont", _.TexConstant.Variant.MONOSPACE],
      tiny: ["SetSize", 0.5],
      Tiny: ["SetSize", 0.6],
      scriptsize: ["SetSize", 0.7],
      small: ["SetSize", 0.85],
      normalsize: ["SetSize", 1],
      large: ["SetSize", 1.2],
      Large: ["SetSize", 1.44],
      LARGE: ["SetSize", 1.73],
      huge: ["SetSize", 2.07],
      Huge: ["SetSize", 2.49],
      Bbb: ["Macro", "{\\bbFont #1}", 1],
      textnormal: ["Macro", "{\\rm #1}", 1],
      textup: ["Macro", "{\\rm #1}", 1],
      textrm: ["Macro", "{\\rm #1}", 1],
      textit: ["Macro", "{\\it #1}", 1],
      textbf: ["Macro", "{\\bf #1}", 1],
      textsf: ["Macro", "{\\sf #1}", 1],
      texttt: ["Macro", "{\\tt #1}", 1],
      dagger: ["Insert", "†"],
      ddagger: ["Insert", "‡"],
      S: ["Insert", "§"],
      ",": ["Spacer", V.MATHSPACE.thinmathspace],
      ":": ["Spacer", V.MATHSPACE.mediummathspace],
      ">": ["Spacer", V.MATHSPACE.mediummathspace],
      ";": ["Spacer", V.MATHSPACE.thickmathspace],
      "!": ["Spacer", V.MATHSPACE.negativethinmathspace],
      enspace: ["Spacer", 0.5],
      quad: ["Spacer", 1],
      qquad: ["Spacer", 2],
      thinspace: ["Spacer", V.MATHSPACE.thinmathspace],
      negthinspace: ["Spacer", V.MATHSPACE.negativethinmathspace],
      hskip: "Hskip",
      hspace: "Hskip",
      kern: "Hskip",
      mskip: "Hskip",
      mspace: "Hskip",
      mkern: "Hskip",
      rule: "rule",
      Rule: ["Rule"],
      Space: ["Rule", "blank"],
      color: "CheckAutoload",
      textcolor: "CheckAutoload",
      colorbox: "CheckAutoload",
      fcolorbox: "CheckAutoload",
      href: "CheckAutoload",
      style: "CheckAutoload",
      class: "CheckAutoload",
      cssId: "CheckAutoload",
      unicode: "CheckAutoload",
      ref: ["HandleRef", !1],
      eqref: ["HandleRef", !0]
    }, H.TextMacrosMethods), TextMacrosMappings;
  }
  var hasRequiredTextMacrosConfiguration;
  function requireTextMacrosConfiguration() {
    if (hasRequiredTextMacrosConfiguration) return TextMacrosConfiguration;
    hasRequiredTextMacrosConfiguration = 1;
    var O = TextMacrosConfiguration.__importDefault || function(ne) {
      return ne && ne.__esModule ? ne : { default: ne };
    }, _;
    Object.defineProperty(TextMacrosConfiguration, "__esModule", { value: !0 }), TextMacrosConfiguration.TextMacrosConfiguration = TextMacrosConfiguration.TextBaseConfiguration = void 0;
    var H = requireConfiguration(), V = O(requireParseOptions$1()), q = requireTags(), G = requireBaseItems(), J = requireTextParser(), Z = requireTextMacrosMethods();
    requireTextMacrosMappings(), TextMacrosConfiguration.TextBaseConfiguration = H.Configuration.create("text-base", {
      parser: "text",
      handler: {
        character: ["command", "text-special"],
        macro: ["text-macros"]
      },
      fallback: {
        character: function(ne, ee) {
          ne.text += ee;
        },
        macro: function(ne, ee) {
          var ae = ne.texParser, oe = ae.lookup("macro", ee);
          oe && oe._func !== Z.TextMacrosMethods.Macro && ne.Error("MathMacro", "%1 is only supported in math mode", "\\" + ee), ae.parse("macro", [ne, ee]);
        }
      },
      items: (_ = {}, _[G.StartItem.prototype.kind] = G.StartItem, _[G.StopItem.prototype.kind] = G.StopItem, _[G.MmlItem.prototype.kind] = G.MmlItem, _[G.StyleItem.prototype.kind] = G.StyleItem, _)
    });
    function te(ne, ee, ae, oe) {
      var se = ne.configuration.packageData.get("textmacros");
      return ne instanceof J.TextParser || (se.texParser = ne), [new J.TextParser(ee, oe ? { mathvariant: oe } : {}, se.parseOptions, ae).mml()];
    }
    return TextMacrosConfiguration.TextMacrosConfiguration = H.Configuration.create("textmacros", {
      config: function(ne, ee) {
        var ae = new H.ParserConfiguration(ee.parseOptions.options.textmacros.packages, ["tex", "text"]);
        ae.init();
        var oe = new V.default(ae, []);
        oe.options = ee.parseOptions.options, ae.config(ee), q.TagsFactory.addTags(ae.tags), oe.tags = q.TagsFactory.getDefault(), oe.tags.configuration = oe, oe.packageData = ee.parseOptions.packageData, oe.packageData.set("textmacros", { parseOptions: oe, jax: ee, texParser: null }), oe.options.internalMath = te;
      },
      preprocessors: [function(ne) {
        var ee = ne.data.packageData.get("textmacros");
        ee.parseOptions.nodeFactory.setMmlFactory(ee.jax.mmlFactory);
      }],
      options: {
        textmacros: {
          packages: ["text-base"]
        }
      }
    }), TextMacrosConfiguration;
  }
  var UpgreekConfiguration = {}, hasRequiredUpgreekConfiguration;
  function requireUpgreekConfiguration() {
    if (hasRequiredUpgreekConfiguration) return UpgreekConfiguration;
    hasRequiredUpgreekConfiguration = 1, Object.defineProperty(UpgreekConfiguration, "__esModule", { value: !0 }), UpgreekConfiguration.UpgreekConfiguration = void 0;
    var O = requireConfiguration(), _ = requireSymbolMap(), H = requireTexConstants();
    function V(q, G) {
      var J = G.attributes || {};
      J.mathvariant = H.TexConstant.Variant.NORMAL;
      var Z = q.create("token", "mi", J, G.char);
      q.Push(Z);
    }
    return new _.CharacterMap("upgreek", V, {
      upalpha: "α",
      upbeta: "β",
      upgamma: "γ",
      updelta: "δ",
      upepsilon: "ϵ",
      upzeta: "ζ",
      upeta: "η",
      uptheta: "θ",
      upiota: "ι",
      upkappa: "κ",
      uplambda: "λ",
      upmu: "μ",
      upnu: "ν",
      upxi: "ξ",
      upomicron: "ο",
      uppi: "π",
      uprho: "ρ",
      upsigma: "σ",
      uptau: "τ",
      upupsilon: "υ",
      upphi: "ϕ",
      upchi: "χ",
      uppsi: "ψ",
      upomega: "ω",
      upvarepsilon: "ε",
      upvartheta: "ϑ",
      upvarpi: "ϖ",
      upvarrho: "ϱ",
      upvarsigma: "ς",
      upvarphi: "φ",
      Upgamma: "Γ",
      Updelta: "Δ",
      Uptheta: "Θ",
      Uplambda: "Λ",
      Upxi: "Ξ",
      Uppi: "Π",
      Upsigma: "Σ",
      Upupsilon: "Υ",
      Upphi: "Φ",
      Uppsi: "Ψ",
      Upomega: "Ω"
    }), UpgreekConfiguration.UpgreekConfiguration = O.Configuration.create("upgreek", {
      handler: { macro: ["upgreek"] }
    }), UpgreekConfiguration;
  }
  var UnicodeConfiguration = {}, hasRequiredUnicodeConfiguration;
  function requireUnicodeConfiguration() {
    return hasRequiredUnicodeConfiguration || (hasRequiredUnicodeConfiguration = 1, function(O) {
      var _ = UnicodeConfiguration.__importDefault || function(ne) {
        return ne && ne.__esModule ? ne : { default: ne };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.UnicodeConfiguration = O.UnicodeMethods = void 0;
      var H = requireConfiguration(), V = _(requireTexError()), q = requireSymbolMap(), G = _(requireParseUtil()), J = _(requireNodeUtil()), Z = requireEntities();
      O.UnicodeMethods = {};
      var te = {};
      O.UnicodeMethods.Unicode = function(ne, ee) {
        var ae = ne.GetBrackets(ee), oe = null, se = null;
        ae && (ae.replace(/ /g, "").match(/^(\d+(\.\d*)?|\.\d+),(\d+(\.\d*)?|\.\d+)$/) ? (oe = ae.replace(/ /g, "").split(/,/), se = ne.GetBrackets(ee)) : se = ae);
        var ce = G.default.trimSpaces(ne.GetArgument(ee)).replace(/^0x/, "x");
        if (!ce.match(/^(x[0-9A-Fa-f]+|[0-9]+)$/))
          throw new V.default("BadUnicode", "Argument to \\unicode must be a number");
        var le = parseInt(ce.match(/^x/) ? "0" + ce : ce);
        te[le] ? se || (se = te[le][2]) : te[le] = [800, 200, se, le], oe && (te[le][0] = Math.floor(parseFloat(oe[0]) * 1e3), te[le][1] = Math.floor(parseFloat(oe[1]) * 1e3));
        var ue = ne.stack.env.font, de = {};
        se ? (te[le][2] = de.fontfamily = se.replace(/'/g, "'"), ue && (ue.match(/bold/) && (de.fontweight = "bold"), ue.match(/italic|-mathit/) && (de.fontstyle = "italic"))) : ue && (de.mathvariant = ue);
        var fe = ne.create("token", "mtext", de, (0, Z.numeric)(ce));
        J.default.setProperty(fe, "unicode", !0), ne.Push(fe);
      }, new q.CommandMap("unicode", { unicode: "Unicode" }, O.UnicodeMethods), O.UnicodeConfiguration = H.Configuration.create("unicode", { handler: { macro: ["unicode"] } });
    }(UnicodeConfiguration)), UnicodeConfiguration;
  }
  var VerbConfiguration = {}, hasRequiredVerbConfiguration;
  function requireVerbConfiguration() {
    return hasRequiredVerbConfiguration || (hasRequiredVerbConfiguration = 1, function(O) {
      var _ = VerbConfiguration.__importDefault || function(J) {
        return J && J.__esModule ? J : { default: J };
      };
      Object.defineProperty(O, "__esModule", { value: !0 }), O.VerbConfiguration = O.VerbMethods = void 0;
      var H = requireConfiguration(), V = requireTexConstants(), q = requireSymbolMap(), G = _(requireTexError());
      O.VerbMethods = {}, O.VerbMethods.Verb = function(J, Z) {
        var te = J.GetNext(), ne = ++J.i;
        if (te === "")
          throw new G.default("MissingArgFor", "Missing argument for %1", Z);
        for (; J.i < J.string.length && J.string.charAt(J.i) !== te; )
          J.i++;
        if (J.i === J.string.length)
          throw new G.default("NoClosingDelim", "Can't find closing delimiter for %1", J.currentCS);
        var ee = J.string.slice(ne, J.i).replace(/ /g, " ");
        J.i++, J.Push(J.create("token", "mtext", { mathvariant: V.TexConstant.Variant.MONOSPACE }, ee));
      }, new q.CommandMap("verb", { verb: "Verb" }, O.VerbMethods), O.VerbConfiguration = H.Configuration.create("verb", { handler: { macro: ["verb"] } });
    }(VerbConfiguration)), VerbConfiguration;
  }
  var hasRequiredAllPackages;
  function requireAllPackages() {
    return hasRequiredAllPackages || (hasRequiredAllPackages = 1, Object.defineProperty(AllPackages, "__esModule", { value: !0 }), AllPackages.AllPackages = void 0, requireBaseConfiguration(), requireActionConfiguration(), requireAmsConfiguration(), requireAmsCdConfiguration(), requireBboxConfiguration(), requireBoldsymbolConfiguration(), requireBraketConfiguration(), requireBussproofsConfiguration(), requireCancelConfiguration(), requireCasesConfiguration(), requireCenternotConfiguration(), requireColorConfiguration(), requireColorV2Configuration(), requireColortblConfiguration(), requireConfigMacrosConfiguration(), requireEmpheqConfiguration(), requireEncloseConfiguration(), requireExtpfeilConfiguration(), requireGensymbConfiguration(), requireHtmlConfiguration(), requireMathtoolsConfiguration(), requireMhchemConfiguration(), requireNewcommandConfiguration(), requireNoErrorsConfiguration(), requireNoUndefinedConfiguration(), requirePhysicsConfiguration(), requireSetOptionsConfiguration(), requireTagFormatConfiguration(), requireTextcompConfiguration(), requireTextMacrosConfiguration(), requireUpgreekConfiguration(), requireUnicodeConfiguration(), requireVerbConfiguration(), typeof MathJax < "u" && MathJax.loader && MathJax.loader.preLoad("[tex]/action", "[tex]/ams", "[tex]/amscd", "[tex]/bbox", "[tex]/boldsymbol", "[tex]/braket", "[tex]/bussproofs", "[tex]/cancel", "[tex]/cases", "[tex]/centernot", "[tex]/color", "[tex]/colorv2", "[tex]/colortbl", "[tex]/empheq", "[tex]/enclose", "[tex]/extpfeil", "[tex]/gensymb", "[tex]/html", "[tex]/mathtools", "[tex]/mhchem", "[tex]/newcommand", "[tex]/noerrors", "[tex]/noundefined", "[tex]/physics", "[tex]/upgreek", "[tex]/unicode", "[tex]/verb", "[tex]/configmacros", "[tex]/tagformat", "[tex]/textcomp", "[tex]/textmacros", "[tex]/setoptions"), AllPackages.AllPackages = [
      "base",
      "action",
      "ams",
      "amscd",
      "bbox",
      "boldsymbol",
      "braket",
      "bussproofs",
      "cancel",
      "cases",
      "centernot",
      "color",
      "colortbl",
      "empheq",
      "enclose",
      "extpfeil",
      "gensymb",
      "html",
      "mathtools",
      "mhchem",
      "newcommand",
      "noerrors",
      "noundefined",
      "upgreek",
      "unicode",
      "verb",
      "configmacros",
      "tagformat",
      "textcomp",
      "textmacros"
    ]), AllPackages;
  }
  var AllPackagesExports = requireAllPackages(), FindTeXExports = requireFindTeX();
  const adaptor = browserAdaptorExports.browserAdaptor();
  htmlExports.RegisterHTMLHandler(adaptor);
  const svg$1 = new svgExports.SVG({ fontCache: "local" }), defaults$4 = {
    display: !0,
    em: 16,
    ex: 8,
    containerWidth: 80 * 16
  };
  function tex2svg(O, _, H = {}) {
    const V = new texExports.TeX({ packages: AllPackagesExports.AllPackages, macros: H });
    return mathjaxExports.mathjax.document("", { InputJax: V, OutputJax: svg$1 }).convert(O, { ...defaults$4, ..._ });
  }
  function ascii2svg(O) {
    throw new Error("not implemented");
  }
  function mathml2svg(O) {
    const _ = new mathmlExports.MathML({});
    return mathjaxExports.mathjax.document("", { InputJax: _, OutputJax: svg$1 }).convert(O, defaults$4);
  }
  function find_tex(O) {
    return new FindTeXExports.FindTeX({
      processEnvironments: !1,
      processEscapes: !1,
      processRefs: !1
    }).findMath([O]);
  }
  const index$2 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    ascii2svg,
    find_tex,
    mathml2svg,
    tex2svg
  }, Symbol.toStringTag, { value: "Module" })), _hoisted_1$5 = ["id"], _sfc_main$8 = /* @__PURE__ */ defineComponent({
    __name: "Bokeh",
    props: {
      plotJson: {},
      responsive: { type: Boolean },
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = [], H = v4(), V = O, q = (Z) => {
        Z.doc.roots.forEach((ne) => {
          ne.attributes.sizing_mode = V.singleBlockEmbed ? "stretch_both" : "stretch_width";
        });
      }, G = (Z, te) => {
        if (Z.uuid === te) {
          Z.clear();
          const ne = documents.indexOf(Z);
          ne > -1 && documents.splice(ne, 1);
        }
      }, J = async () => {
        try {
          V.responsive && q(V.plotJson), (await embed_item(V.plotJson, H)).roots.forEach((te) => {
            const ne = v4();
            te.model.document.uuid = ne, _.push(ne);
          });
        } catch (Z) {
          console.error("An error occurred while rendering a Bokeh chart: ", Z);
        }
      };
      return onMounted(() => {
        J();
      }), onUnmounted(() => {
        for (const Z of documents)
          for (const te of _)
            G(Z, te);
      }), (Z, te) => (openBlock(), createElementBlock("div", {
        "data-cy": "block-bokeh",
        id: unref(H),
        class: normalizeClass([
          "bk-root m-auto flex justify-center items-center w-full",
          { "w-full": Z.responsive, "h-iframe": Z.singleBlockEmbed }
        ])
      }, null, 10, _hoisted_1$5));
    }
  }), Bokeh = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    default: _sfc_main$8
  }, Symbol.toStringTag, { value: "Module" }));
  class SourceLocation {
    // The + prefix indicates that these fields aren't writeable
    // Lexer holding the input string.
    // Start offset, zero-based inclusive.
    // End offset, zero-based exclusive.
    constructor(_, H, V) {
      this.lexer = void 0, this.start = void 0, this.end = void 0, this.lexer = _, this.start = H, this.end = V;
    }
    /**
     * Merges two `SourceLocation`s from location providers, given they are
     * provided in order of appearance.
     * - Returns the first one's location if only the first is provided.
     * - Returns a merged range of the first and the last if both are provided
     *   and their lexers match.
     * - Otherwise, returns null.
     */
    static range(_, H) {
      return H ? !_ || !_.loc || !H.loc || _.loc.lexer !== H.loc.lexer ? null : new SourceLocation(_.loc.lexer, _.loc.start, H.loc.end) : _ && _.loc;
    }
  }
  class Token {
    // don't expand the token
    // used in \noexpand
    constructor(_, H) {
      this.text = void 0, this.loc = void 0, this.noexpand = void 0, this.treatAsRelax = void 0, this.text = _, this.loc = H;
    }
    /**
     * Given a pair of tokens (this and endToken), compute a `Token` encompassing
     * the whole input range enclosed by these two.
     */
    range(_, H) {
      return new Token(H, SourceLocation.range(this, _));
    }
  }
  class ParseError {
    // Error start position based on passed-in Token or ParseNode.
    // Length of affected text based on passed-in Token or ParseNode.
    // The underlying error message without any context added.
    constructor(_, H) {
      this.name = void 0, this.position = void 0, this.length = void 0, this.rawMessage = void 0;
      var V = "KaTeX parse error: " + _, q, G, J = H && H.loc;
      if (J && J.start <= J.end) {
        var Z = J.lexer.input;
        q = J.start, G = J.end, q === Z.length ? V += " at end of input: " : V += " at position " + (q + 1) + ": ";
        var te = Z.slice(q, G).replace(/[^]/g, "$&̲"), ne;
        q > 15 ? ne = "…" + Z.slice(q - 15, q) : ne = Z.slice(0, q);
        var ee;
        G + 15 < Z.length ? ee = Z.slice(G, G + 15) + "…" : ee = Z.slice(G), V += ne + te + ee;
      }
      var ae = new Error(V);
      return ae.name = "ParseError", ae.__proto__ = ParseError.prototype, ae.position = q, q != null && G != null && (ae.length = G - q), ae.rawMessage = _, ae;
    }
  }
  ParseError.prototype.__proto__ = Error.prototype;
  var contains$4 = function O(_, H) {
    return _.indexOf(H) !== -1;
  }, deflt = function O(_, H) {
    return _ === void 0 ? H : _;
  }, uppercase = /([A-Z])/g, hyphenate = function O(_) {
    return _.replace(uppercase, "-$1").toLowerCase();
  }, ESCAPE_LOOKUP = {
    "&": "&amp;",
    ">": "&gt;",
    "<": "&lt;",
    '"': "&quot;",
    "'": "&#x27;"
  }, ESCAPE_REGEX = /[&><"']/g;
  function escape$1(O) {
    return String(O).replace(ESCAPE_REGEX, (_) => ESCAPE_LOOKUP[_]);
  }
  var getBaseElem = function O(_) {
    return _.type === "ordgroup" || _.type === "color" ? _.body.length === 1 ? O(_.body[0]) : _ : _.type === "font" ? O(_.body) : _;
  }, isCharacterBox = function O(_) {
    var H = getBaseElem(_);
    return H.type === "mathord" || H.type === "textord" || H.type === "atom";
  }, assert$1 = function O(_) {
    if (!_)
      throw new Error("Expected non-null, but got " + String(_));
    return _;
  }, protocolFromUrl = function O(_) {
    var H = /^[\x00-\x20]*([^\\/#?]*?)(:|&#0*58|&#x0*3a|&colon)/i.exec(_);
    return H ? H[2] !== ":" || !/^[a-zA-Z][a-zA-Z0-9+\-.]*$/.test(H[1]) ? null : H[1].toLowerCase() : "_relative";
  }, utils = {
    contains: contains$4,
    deflt,
    escape: escape$1,
    hyphenate,
    getBaseElem,
    isCharacterBox,
    protocolFromUrl
  }, SETTINGS_SCHEMA = {
    displayMode: {
      type: "boolean",
      description: "Render math in display mode, which puts the math in display style (so \\int and \\sum are large, for example), and centers the math on the page on its own line.",
      cli: "-d, --display-mode"
    },
    output: {
      type: {
        enum: ["htmlAndMathml", "html", "mathml"]
      },
      description: "Determines the markup language of the output.",
      cli: "-F, --format <type>"
    },
    leqno: {
      type: "boolean",
      description: "Render display math in leqno style (left-justified tags)."
    },
    fleqn: {
      type: "boolean",
      description: "Render display math flush left."
    },
    throwOnError: {
      type: "boolean",
      default: !0,
      cli: "-t, --no-throw-on-error",
      cliDescription: "Render errors (in the color given by --error-color) instead of throwing a ParseError exception when encountering an error."
    },
    errorColor: {
      type: "string",
      default: "#cc0000",
      cli: "-c, --error-color <color>",
      cliDescription: "A color string given in the format 'rgb' or 'rrggbb' (no #). This option determines the color of errors rendered by the -t option.",
      cliProcessor: (O) => "#" + O
    },
    macros: {
      type: "object",
      cli: "-m, --macro <def>",
      cliDescription: "Define custom macro of the form '\\foo:expansion' (use multiple -m arguments for multiple macros).",
      cliDefault: [],
      cliProcessor: (O, _) => (_.push(O), _)
    },
    minRuleThickness: {
      type: "number",
      description: "Specifies a minimum thickness, in ems, for fraction lines, `\\sqrt` top lines, `{array}` vertical lines, `\\hline`, `\\hdashline`, `\\underline`, `\\overline`, and the borders of `\\fbox`, `\\boxed`, and `\\fcolorbox`.",
      processor: (O) => Math.max(0, O),
      cli: "--min-rule-thickness <size>",
      cliProcessor: parseFloat
    },
    colorIsTextColor: {
      type: "boolean",
      description: "Makes \\color behave like LaTeX's 2-argument \\textcolor, instead of LaTeX's one-argument \\color mode change.",
      cli: "-b, --color-is-text-color"
    },
    strict: {
      type: [{
        enum: ["warn", "ignore", "error"]
      }, "boolean", "function"],
      description: "Turn on strict / LaTeX faithfulness mode, which throws an error if the input uses features that are not supported by LaTeX.",
      cli: "-S, --strict",
      cliDefault: !1
    },
    trust: {
      type: ["boolean", "function"],
      description: "Trust the input, enabling all HTML features such as \\url.",
      cli: "-T, --trust"
    },
    maxSize: {
      type: "number",
      default: 1 / 0,
      description: "If non-zero, all user-specified sizes, e.g. in \\rule{500em}{500em}, will be capped to maxSize ems. Otherwise, elements and spaces can be arbitrarily large",
      processor: (O) => Math.max(0, O),
      cli: "-s, --max-size <n>",
      cliProcessor: parseInt
    },
    maxExpand: {
      type: "number",
      default: 1e3,
      description: "Limit the number of macro expansions to the specified number, to prevent e.g. infinite macro loops. If set to Infinity, the macro expander will try to fully expand as in LaTeX.",
      processor: (O) => Math.max(0, O),
      cli: "-e, --max-expand <n>",
      cliProcessor: (O) => O === "Infinity" ? 1 / 0 : parseInt(O)
    },
    globalGroup: {
      type: "boolean",
      cli: !1
    }
  };
  function getDefaultValue(O) {
    if (O.default)
      return O.default;
    var _ = O.type, H = Array.isArray(_) ? _[0] : _;
    if (typeof H != "string")
      return H.enum[0];
    switch (H) {
      case "boolean":
        return !1;
      case "string":
        return "";
      case "number":
        return 0;
      case "object":
        return {};
    }
  }
  class Settings {
    constructor(_) {
      this.displayMode = void 0, this.output = void 0, this.leqno = void 0, this.fleqn = void 0, this.throwOnError = void 0, this.errorColor = void 0, this.macros = void 0, this.minRuleThickness = void 0, this.colorIsTextColor = void 0, this.strict = void 0, this.trust = void 0, this.maxSize = void 0, this.maxExpand = void 0, this.globalGroup = void 0, _ = _ || {};
      for (var H in SETTINGS_SCHEMA)
        if (SETTINGS_SCHEMA.hasOwnProperty(H)) {
          var V = SETTINGS_SCHEMA[H];
          this[H] = _[H] !== void 0 ? V.processor ? V.processor(_[H]) : _[H] : getDefaultValue(V);
        }
    }
    /**
     * Report nonstrict (non-LaTeX-compatible) input.
     * Can safely not be called if `this.strict` is false in JavaScript.
     */
    reportNonstrict(_, H, V) {
      var q = this.strict;
      if (typeof q == "function" && (q = q(_, H, V)), !(!q || q === "ignore")) {
        if (q === !0 || q === "error")
          throw new ParseError("LaTeX-incompatible input and strict mode is set to 'error': " + (H + " [" + _ + "]"), V);
        q === "warn" ? typeof console < "u" && console.warn("LaTeX-incompatible input and strict mode is set to 'warn': " + (H + " [" + _ + "]")) : typeof console < "u" && console.warn("LaTeX-incompatible input and strict mode is set to " + ("unrecognized '" + q + "': " + H + " [" + _ + "]"));
      }
    }
    /**
     * Check whether to apply strict (LaTeX-adhering) behavior for unusual
     * input (like `\\`).  Unlike `nonstrict`, will not throw an error;
     * instead, "error" translates to a return value of `true`, while "ignore"
     * translates to a return value of `false`.  May still print a warning:
     * "warn" prints a warning and returns `false`.
     * This is for the second category of `errorCode`s listed in the README.
     */
    useStrictBehavior(_, H, V) {
      var q = this.strict;
      if (typeof q == "function")
        try {
          q = q(_, H, V);
        } catch {
          q = "error";
        }
      return !q || q === "ignore" ? !1 : q === !0 || q === "error" ? !0 : q === "warn" ? (typeof console < "u" && console.warn("LaTeX-incompatible input and strict mode is set to 'warn': " + (H + " [" + _ + "]")), !1) : (typeof console < "u" && console.warn("LaTeX-incompatible input and strict mode is set to " + ("unrecognized '" + q + "': " + H + " [" + _ + "]")), !1);
    }
    /**
     * Check whether to test potentially dangerous input, and return
     * `true` (trusted) or `false` (untrusted).  The sole argument `context`
     * should be an object with `command` field specifying the relevant LaTeX
     * command (as a string starting with `\`), and any other arguments, etc.
     * If `context` has a `url` field, a `protocol` field will automatically
     * get added by this function (changing the specified object).
     */
    isTrusted(_) {
      if (_.url && !_.protocol) {
        var H = utils.protocolFromUrl(_.url);
        if (H == null)
          return !1;
        _.protocol = H;
      }
      var V = typeof this.trust == "function" ? this.trust(_) : this.trust;
      return !!V;
    }
  }
  class Style {
    constructor(_, H, V) {
      this.id = void 0, this.size = void 0, this.cramped = void 0, this.id = _, this.size = H, this.cramped = V;
    }
    /**
     * Get the style of a superscript given a base in the current style.
     */
    sup() {
      return styles$1[sup[this.id]];
    }
    /**
     * Get the style of a subscript given a base in the current style.
     */
    sub() {
      return styles$1[sub[this.id]];
    }
    /**
     * Get the style of a fraction numerator given the fraction in the current
     * style.
     */
    fracNum() {
      return styles$1[fracNum[this.id]];
    }
    /**
     * Get the style of a fraction denominator given the fraction in the current
     * style.
     */
    fracDen() {
      return styles$1[fracDen[this.id]];
    }
    /**
     * Get the cramped version of a style (in particular, cramping a cramped style
     * doesn't change the style).
     */
    cramp() {
      return styles$1[cramp[this.id]];
    }
    /**
     * Get a text or display version of this style.
     */
    text() {
      return styles$1[text$1$1[this.id]];
    }
    /**
     * Return true if this style is tightly spaced (scriptstyle/scriptscriptstyle)
     */
    isTight() {
      return this.size >= 2;
    }
  }
  var D$2 = 0, Dc = 1, T = 2, Tc = 3, S = 4, Sc = 5, SS = 6, SSc = 7, styles$1 = [new Style(D$2, 0, !1), new Style(Dc, 0, !0), new Style(T, 1, !1), new Style(Tc, 1, !0), new Style(S, 2, !1), new Style(Sc, 2, !0), new Style(SS, 3, !1), new Style(SSc, 3, !0)], sup = [S, Sc, S, Sc, SS, SSc, SS, SSc], sub = [Sc, Sc, Sc, Sc, SSc, SSc, SSc, SSc], fracNum = [T, Tc, S, Sc, SS, SSc, SS, SSc], fracDen = [Tc, Tc, Sc, Sc, SSc, SSc, SSc, SSc], cramp = [Dc, Dc, Tc, Tc, Sc, Sc, SSc, SSc], text$1$1 = [D$2, Dc, T, Tc, T, Tc, T, Tc], Style$1 = {
    DISPLAY: styles$1[D$2],
    TEXT: styles$1[T],
    SCRIPT: styles$1[S],
    SCRIPTSCRIPT: styles$1[SS]
  }, scriptData = [{
    // Latin characters beyond the Latin-1 characters we have metrics for.
    // Needed for Czech, Hungarian and Turkish text, for example.
    name: "latin",
    blocks: [
      [256, 591],
      // Latin Extended-A and Latin Extended-B
      [768, 879]
      // Combining Diacritical marks
    ]
  }, {
    // The Cyrillic script used by Russian and related languages.
    // A Cyrillic subset used to be supported as explicitly defined
    // symbols in symbols.js
    name: "cyrillic",
    blocks: [[1024, 1279]]
  }, {
    // Armenian
    name: "armenian",
    blocks: [[1328, 1423]]
  }, {
    // The Brahmic scripts of South and Southeast Asia
    // Devanagari (0900–097F)
    // Bengali (0980–09FF)
    // Gurmukhi (0A00–0A7F)
    // Gujarati (0A80–0AFF)
    // Oriya (0B00–0B7F)
    // Tamil (0B80–0BFF)
    // Telugu (0C00–0C7F)
    // Kannada (0C80–0CFF)
    // Malayalam (0D00–0D7F)
    // Sinhala (0D80–0DFF)
    // Thai (0E00–0E7F)
    // Lao (0E80–0EFF)
    // Tibetan (0F00–0FFF)
    // Myanmar (1000–109F)
    name: "brahmic",
    blocks: [[2304, 4255]]
  }, {
    name: "georgian",
    blocks: [[4256, 4351]]
  }, {
    // Chinese and Japanese.
    // The "k" in cjk is for Korean, but we've separated Korean out
    name: "cjk",
    blocks: [
      [12288, 12543],
      // CJK symbols and punctuation, Hiragana, Katakana
      [19968, 40879],
      // CJK ideograms
      [65280, 65376]
      // Fullwidth punctuation
      // TODO: add halfwidth Katakana and Romanji glyphs
    ]
  }, {
    // Korean
    name: "hangul",
    blocks: [[44032, 55215]]
  }];
  function scriptFromCodepoint(O) {
    for (var _ = 0; _ < scriptData.length; _++)
      for (var H = scriptData[_], V = 0; V < H.blocks.length; V++) {
        var q = H.blocks[V];
        if (O >= q[0] && O <= q[1])
          return H.name;
      }
    return null;
  }
  var allBlocks = [];
  scriptData.forEach((O) => O.blocks.forEach((_) => allBlocks.push(..._)));
  function supportedCodepoint(O) {
    for (var _ = 0; _ < allBlocks.length; _ += 2)
      if (O >= allBlocks[_] && O <= allBlocks[_ + 1])
        return !0;
    return !1;
  }
  var hLinePad = 80, sqrtMain = function O(_, H) {
    return "M95," + (622 + _ + H) + `
c-2.7,0,-7.17,-2.7,-13.5,-8c-5.8,-5.3,-9.5,-10,-9.5,-14
c0,-2,0.3,-3.3,1,-4c1.3,-2.7,23.83,-20.7,67.5,-54
c44.2,-33.3,65.8,-50.3,66.5,-51c1.3,-1.3,3,-2,5,-2c4.7,0,8.7,3.3,12,10
s173,378,173,378c0.7,0,35.3,-71,104,-213c68.7,-142,137.5,-285,206.5,-429
c69,-144,104.5,-217.7,106.5,-221
l` + _ / 2.075 + " -" + _ + `
c5.3,-9.3,12,-14,20,-14
H400000v` + (40 + _) + `H845.2724
s-225.272,467,-225.272,467s-235,486,-235,486c-2.7,4.7,-9,7,-19,7
c-6,0,-10,-1,-12,-3s-194,-422,-194,-422s-65,47,-65,47z
M` + (834 + _) + " " + H + "h400000v" + (40 + _) + "h-400000z";
  }, sqrtSize1 = function O(_, H) {
    return "M263," + (601 + _ + H) + `c0.7,0,18,39.7,52,119
c34,79.3,68.167,158.7,102.5,238c34.3,79.3,51.8,119.3,52.5,120
c340,-704.7,510.7,-1060.3,512,-1067
l` + _ / 2.084 + " -" + _ + `
c4.7,-7.3,11,-11,19,-11
H40000v` + (40 + _) + `H1012.3
s-271.3,567,-271.3,567c-38.7,80.7,-84,175,-136,283c-52,108,-89.167,185.3,-111.5,232
c-22.3,46.7,-33.8,70.3,-34.5,71c-4.7,4.7,-12.3,7,-23,7s-12,-1,-12,-1
s-109,-253,-109,-253c-72.7,-168,-109.3,-252,-110,-252c-10.7,8,-22,16.7,-34,26
c-22,17.3,-33.3,26,-34,26s-26,-26,-26,-26s76,-59,76,-59s76,-60,76,-60z
M` + (1001 + _) + " " + H + "h400000v" + (40 + _) + "h-400000z";
  }, sqrtSize2 = function O(_, H) {
    return "M983 " + (10 + _ + H) + `
l` + _ / 3.13 + " -" + _ + `
c4,-6.7,10,-10,18,-10 H400000v` + (40 + _) + `
H1013.1s-83.4,268,-264.1,840c-180.7,572,-277,876.3,-289,913c-4.7,4.7,-12.7,7,-24,7
s-12,0,-12,0c-1.3,-3.3,-3.7,-11.7,-7,-25c-35.3,-125.3,-106.7,-373.3,-214,-744
c-10,12,-21,25,-33,39s-32,39,-32,39c-6,-5.3,-15,-14,-27,-26s25,-30,25,-30
c26.7,-32.7,52,-63,76,-91s52,-60,52,-60s208,722,208,722
c56,-175.3,126.3,-397.3,211,-666c84.7,-268.7,153.8,-488.2,207.5,-658.5
c53.7,-170.3,84.5,-266.8,92.5,-289.5z
M` + (1001 + _) + " " + H + "h400000v" + (40 + _) + "h-400000z";
  }, sqrtSize3 = function O(_, H) {
    return "M424," + (2398 + _ + H) + `
c-1.3,-0.7,-38.5,-172,-111.5,-514c-73,-342,-109.8,-513.3,-110.5,-514
c0,-2,-10.7,14.3,-32,49c-4.7,7.3,-9.8,15.7,-15.5,25c-5.7,9.3,-9.8,16,-12.5,20
s-5,7,-5,7c-4,-3.3,-8.3,-7.7,-13,-13s-13,-13,-13,-13s76,-122,76,-122s77,-121,77,-121
s209,968,209,968c0,-2,84.7,-361.7,254,-1079c169.3,-717.3,254.7,-1077.7,256,-1081
l` + _ / 4.223 + " -" + _ + `c4,-6.7,10,-10,18,-10 H400000
v` + (40 + _) + `H1014.6
s-87.3,378.7,-272.6,1166c-185.3,787.3,-279.3,1182.3,-282,1185
c-2,6,-10,9,-24,9
c-8,0,-12,-0.7,-12,-2z M` + (1001 + _) + " " + H + `
h400000v` + (40 + _) + "h-400000z";
  }, sqrtSize4 = function O(_, H) {
    return "M473," + (2713 + _ + H) + `
c339.3,-1799.3,509.3,-2700,510,-2702 l` + _ / 5.298 + " -" + _ + `
c3.3,-7.3,9.3,-11,18,-11 H400000v` + (40 + _) + `H1017.7
s-90.5,478,-276.2,1466c-185.7,988,-279.5,1483,-281.5,1485c-2,6,-10,9,-24,9
c-8,0,-12,-0.7,-12,-2c0,-1.3,-5.3,-32,-16,-92c-50.7,-293.3,-119.7,-693.3,-207,-1200
c0,-1.3,-5.3,8.7,-16,30c-10.7,21.3,-21.3,42.7,-32,64s-16,33,-16,33s-26,-26,-26,-26
s76,-153,76,-153s77,-151,77,-151c0.7,0.7,35.7,202,105,604c67.3,400.7,102,602.7,104,
606zM` + (1001 + _) + " " + H + "h400000v" + (40 + _) + "H1017.7z";
  }, phasePath = function O(_) {
    var H = _ / 2;
    return "M400000 " + _ + " H0 L" + H + " 0 l65 45 L145 " + (_ - 80) + " H400000z";
  }, sqrtTall = function O(_, H, V) {
    var q = V - 54 - H - _;
    return "M702 " + (_ + H) + "H400000" + (40 + _) + `
H742v` + q + `l-4 4-4 4c-.667.7 -2 1.5-4 2.5s-4.167 1.833-6.5 2.5-5.5 1-9.5 1
h-12l-28-84c-16.667-52-96.667 -294.333-240-727l-212 -643 -85 170
c-4-3.333-8.333-7.667-13 -13l-13-13l77-155 77-156c66 199.333 139 419.667
219 661 l218 661zM702 ` + H + "H400000v" + (40 + _) + "H742z";
  }, sqrtPath = function O(_, H, V) {
    H = 1e3 * H;
    var q = "";
    switch (_) {
      case "sqrtMain":
        q = sqrtMain(H, hLinePad);
        break;
      case "sqrtSize1":
        q = sqrtSize1(H, hLinePad);
        break;
      case "sqrtSize2":
        q = sqrtSize2(H, hLinePad);
        break;
      case "sqrtSize3":
        q = sqrtSize3(H, hLinePad);
        break;
      case "sqrtSize4":
        q = sqrtSize4(H, hLinePad);
        break;
      case "sqrtTall":
        q = sqrtTall(H, hLinePad, V);
    }
    return q;
  }, innerPath = function O(_, H) {
    switch (_) {
      case "⎜":
        return "M291 0 H417 V" + H + " H291z M291 0 H417 V" + H + " H291z";
      case "∣":
        return "M145 0 H188 V" + H + " H145z M145 0 H188 V" + H + " H145z";
      case "∥":
        return "M145 0 H188 V" + H + " H145z M145 0 H188 V" + H + " H145z" + ("M367 0 H410 V" + H + " H367z M367 0 H410 V" + H + " H367z");
      case "⎟":
        return "M457 0 H583 V" + H + " H457z M457 0 H583 V" + H + " H457z";
      case "⎢":
        return "M319 0 H403 V" + H + " H319z M319 0 H403 V" + H + " H319z";
      case "⎥":
        return "M263 0 H347 V" + H + " H263z M263 0 H347 V" + H + " H263z";
      case "⎪":
        return "M384 0 H504 V" + H + " H384z M384 0 H504 V" + H + " H384z";
      case "⏐":
        return "M312 0 H355 V" + H + " H312z M312 0 H355 V" + H + " H312z";
      case "‖":
        return "M257 0 H300 V" + H + " H257z M257 0 H300 V" + H + " H257z" + ("M478 0 H521 V" + H + " H478z M478 0 H521 V" + H + " H478z");
      default:
        return "";
    }
  }, path$4 = {
    // The doubleleftarrow geometry is from glyph U+21D0 in the font KaTeX Main
    doubleleftarrow: `M262 157
l10-10c34-36 62.7-77 86-123 3.3-8 5-13.3 5-16 0-5.3-6.7-8-20-8-7.3
 0-12.2.5-14.5 1.5-2.3 1-4.8 4.5-7.5 10.5-49.3 97.3-121.7 169.3-217 216-28
 14-57.3 25-88 33-6.7 2-11 3.8-13 5.5-2 1.7-3 4.2-3 7.5s1 5.8 3 7.5
c2 1.7 6.3 3.5 13 5.5 68 17.3 128.2 47.8 180.5 91.5 52.3 43.7 93.8 96.2 124.5
 157.5 9.3 8 15.3 12.3 18 13h6c12-.7 18-4 18-10 0-2-1.7-7-5-15-23.3-46-52-87
-86-123l-10-10h399738v-40H218c328 0 0 0 0 0l-10-8c-26.7-20-65.7-43-117-69 2.7
-2 6-3.7 10-5 36.7-16 72.3-37.3 107-64l10-8h399782v-40z
m8 0v40h399730v-40zm0 194v40h399730v-40z`,
    // doublerightarrow is from glyph U+21D2 in font KaTeX Main
    doublerightarrow: `M399738 392l
-10 10c-34 36-62.7 77-86 123-3.3 8-5 13.3-5 16 0 5.3 6.7 8 20 8 7.3 0 12.2-.5
 14.5-1.5 2.3-1 4.8-4.5 7.5-10.5 49.3-97.3 121.7-169.3 217-216 28-14 57.3-25 88
-33 6.7-2 11-3.8 13-5.5 2-1.7 3-4.2 3-7.5s-1-5.8-3-7.5c-2-1.7-6.3-3.5-13-5.5-68
-17.3-128.2-47.8-180.5-91.5-52.3-43.7-93.8-96.2-124.5-157.5-9.3-8-15.3-12.3-18
-13h-6c-12 .7-18 4-18 10 0 2 1.7 7 5 15 23.3 46 52 87 86 123l10 10H0v40h399782
c-328 0 0 0 0 0l10 8c26.7 20 65.7 43 117 69-2.7 2-6 3.7-10 5-36.7 16-72.3 37.3
-107 64l-10 8H0v40zM0 157v40h399730v-40zm0 194v40h399730v-40z`,
    // leftarrow is from glyph U+2190 in font KaTeX Main
    leftarrow: `M400000 241H110l3-3c68.7-52.7 113.7-120
 135-202 4-14.7 6-23 6-25 0-7.3-7-11-21-11-8 0-13.2.8-15.5 2.5-2.3 1.7-4.2 5.8
-5.5 12.5-1.3 4.7-2.7 10.3-4 17-12 48.7-34.8 92-68.5 130S65.3 228.3 18 247
c-10 4-16 7.7-18 11 0 8.7 6 14.3 18 17 47.3 18.7 87.8 47 121.5 85S196 441.3 208
 490c.7 2 1.3 5 2 9s1.2 6.7 1.5 8c.3 1.3 1 3.3 2 6s2.2 4.5 3.5 5.5c1.3 1 3.3
 1.8 6 2.5s6 1 10 1c14 0 21-3.7 21-11 0-2-2-10.3-6-25-20-79.3-65-146.7-135-202
 l-3-3h399890zM100 241v40h399900v-40z`,
    // overbrace is from glyphs U+23A9/23A8/23A7 in font KaTeX_Size4-Regular
    leftbrace: `M6 548l-6-6v-35l6-11c56-104 135.3-181.3 238-232 57.3-28.7 117
-45 179-50h399577v120H403c-43.3 7-81 15-113 26-100.7 33-179.7 91-237 174-2.7
 5-6 9-10 13-.7 1-7.3 1-20 1H6z`,
    leftbraceunder: `M0 6l6-6h17c12.688 0 19.313.3 20 1 4 4 7.313 8.3 10 13
 35.313 51.3 80.813 93.8 136.5 127.5 55.688 33.7 117.188 55.8 184.5 66.5.688
 0 2 .3 4 1 18.688 2.7 76 4.3 172 5h399450v120H429l-6-1c-124.688-8-235-61.7
-331-161C60.687 138.7 32.312 99.3 7 54L0 41V6z`,
    // overgroup is from the MnSymbol package (public domain)
    leftgroup: `M400000 80
H435C64 80 168.3 229.4 21 260c-5.9 1.2-18 0-18 0-2 0-3-1-3-3v-38C76 61 257 0
 435 0h399565z`,
    leftgroupunder: `M400000 262
H435C64 262 168.3 112.6 21 82c-5.9-1.2-18 0-18 0-2 0-3 1-3 3v38c76 158 257 219
 435 219h399565z`,
    // Harpoons are from glyph U+21BD in font KaTeX Main
    leftharpoon: `M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3
-3.3 10.2-9.5 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5
-18.3 3-21-1.3-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7
-196 228-6.7 4.7-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40z`,
    leftharpoonplus: `M0 267c.7 5.3 3 10 7 14h399993v-40H93c3.3-3.3 10.2-9.5
 20.5-18.5s17.8-15.8 22.5-20.5c50.7-52 88-110.3 112-175 4-11.3 5-18.3 3-21-1.3
-4-7.3-6-18-6-8 0-13 .7-15 2s-4.7 6.7-8 16c-42 98.7-107.3 174.7-196 228-6.7 4.7
-10.7 8-12 10-1.3 2-2 5.7-2 11zm100-26v40h399900v-40zM0 435v40h400000v-40z
m0 0v40h400000v-40z`,
    leftharpoondown: `M7 241c-4 4-6.333 8.667-7 14 0 5.333.667 9 2 11s5.333
 5.333 12 10c90.667 54 156 130 196 228 3.333 10.667 6.333 16.333 9 17 2 .667 5
 1 9 1h5c10.667 0 16.667-2 18-6 2-2.667 1-9.667-3-21-32-87.333-82.667-157.667
-152-211l-3-3h399907v-40zM93 281 H400000 v-40L7 241z`,
    leftharpoondownplus: `M7 435c-4 4-6.3 8.7-7 14 0 5.3.7 9 2 11s5.3 5.3 12
 10c90.7 54 156 130 196 228 3.3 10.7 6.3 16.3 9 17 2 .7 5 1 9 1h5c10.7 0 16.7
-2 18-6 2-2.7 1-9.7-3-21-32-87.3-82.7-157.7-152-211l-3-3h399907v-40H7zm93 0
v40h399900v-40zM0 241v40h399900v-40zm0 0v40h399900v-40z`,
    // hook is from glyph U+21A9 in font KaTeX Main
    lefthook: `M400000 281 H103s-33-11.2-61-33.5S0 197.3 0 164s14.2-61.2 42.5
-83.5C70.8 58.2 104 47 142 47 c16.7 0 25 6.7 25 20 0 12-8.7 18.7-26 20-40 3.3
-68.7 15.7-86 37-10 12-15 25.3-15 40 0 22.7 9.8 40.7 29.5 54 19.7 13.3 43.5 21
 71.5 23h399859zM103 281v-40h399897v40z`,
    leftlinesegment: `M40 281 V428 H0 V94 H40 V241 H400000 v40z
M40 281 V428 H0 V94 H40 V241 H400000 v40z`,
    leftmapsto: `M40 281 V448H0V74H40V241H400000v40z
M40 281 V448H0V74H40V241H400000v40z`,
    // tofrom is from glyph U+21C4 in font KaTeX AMS Regular
    leftToFrom: `M0 147h400000v40H0zm0 214c68 40 115.7 95.7 143 167h22c15.3 0 23
-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69-70-101l-7-8h399905v-40H95l7-8
c28.7-32 52-65.7 70-101 10.7-23.3 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 265.3
 68 321 0 361zm0-174v-40h399900v40zm100 154v40h399900v-40z`,
    longequal: `M0 50 h400000 v40H0z m0 194h40000v40H0z
M0 50 h400000 v40H0z m0 194h40000v40H0z`,
    midbrace: `M200428 334
c-100.7-8.3-195.3-44-280-108-55.3-42-101.7-93-139-153l-9-14c-2.7 4-5.7 8.7-9 14
-53.3 86.7-123.7 153-211 199-66.7 36-137.3 56.3-212 62H0V214h199568c178.3-11.7
 311.7-78.3 403-201 6-8 9.7-12 11-12 .7-.7 6.7-1 18-1s17.3.3 18 1c1.3 0 5 4 11
 12 44.7 59.3 101.3 106.3 170 141s145.3 54.3 229 60h199572v120z`,
    midbraceunder: `M199572 214
c100.7 8.3 195.3 44 280 108 55.3 42 101.7 93 139 153l9 14c2.7-4 5.7-8.7 9-14
 53.3-86.7 123.7-153 211-199 66.7-36 137.3-56.3 212-62h199568v120H200432c-178.3
 11.7-311.7 78.3-403 201-6 8-9.7 12-11 12-.7.7-6.7 1-18 1s-17.3-.3-18-1c-1.3 0
-5-4-11-12-44.7-59.3-101.3-106.3-170-141s-145.3-54.3-229-60H0V214z`,
    oiintSize1: `M512.6 71.6c272.6 0 320.3 106.8 320.3 178.2 0 70.8-47.7 177.6
-320.3 177.6S193.1 320.6 193.1 249.8c0-71.4 46.9-178.2 319.5-178.2z
m368.1 178.2c0-86.4-60.9-215.4-368.1-215.4-306.4 0-367.3 129-367.3 215.4 0 85.8
60.9 214.8 367.3 214.8 307.2 0 368.1-129 368.1-214.8z`,
    oiintSize2: `M757.8 100.1c384.7 0 451.1 137.6 451.1 230 0 91.3-66.4 228.8
-451.1 228.8-386.3 0-452.7-137.5-452.7-228.8 0-92.4 66.4-230 452.7-230z
m502.4 230c0-111.2-82.4-277.2-502.4-277.2s-504 166-504 277.2
c0 110 84 276 504 276s502.4-166 502.4-276z`,
    oiiintSize1: `M681.4 71.6c408.9 0 480.5 106.8 480.5 178.2 0 70.8-71.6 177.6
-480.5 177.6S202.1 320.6 202.1 249.8c0-71.4 70.5-178.2 479.3-178.2z
m525.8 178.2c0-86.4-86.8-215.4-525.7-215.4-437.9 0-524.7 129-524.7 215.4 0
85.8 86.8 214.8 524.7 214.8 438.9 0 525.7-129 525.7-214.8z`,
    oiiintSize2: `M1021.2 53c603.6 0 707.8 165.8 707.8 277.2 0 110-104.2 275.8
-707.8 275.8-606 0-710.2-165.8-710.2-275.8C311 218.8 415.2 53 1021.2 53z
m770.4 277.1c0-131.2-126.4-327.6-770.5-327.6S248.4 198.9 248.4 330.1
c0 130 128.8 326.4 772.7 326.4s770.5-196.4 770.5-326.4z`,
    rightarrow: `M0 241v40h399891c-47.3 35.3-84 78-110 128
-16.7 32-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20
 11 8 0 13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7
 39-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85
-40.5-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5
-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67
 151.7 139 205zm0 0v40h399900v-40z`,
    rightbrace: `M400000 542l
-6 6h-17c-12.7 0-19.3-.3-20-1-4-4-7.3-8.3-10-13-35.3-51.3-80.8-93.8-136.5-127.5
s-117.2-55.8-184.5-66.5c-.7 0-2-.3-4-1-18.7-2.7-76-4.3-172-5H0V214h399571l6 1
c124.7 8 235 61.7 331 161 31.3 33.3 59.7 72.7 85 118l7 13v35z`,
    rightbraceunder: `M399994 0l6 6v35l-6 11c-56 104-135.3 181.3-238 232-57.3
 28.7-117 45-179 50H-300V214h399897c43.3-7 81-15 113-26 100.7-33 179.7-91 237
-174 2.7-5 6-9 10-13 .7-1 7.3-1 20-1h17z`,
    rightgroup: `M0 80h399565c371 0 266.7 149.4 414 180 5.9 1.2 18 0 18 0 2 0
 3-1 3-3v-38c-76-158-257-219-435-219H0z`,
    rightgroupunder: `M0 262h399565c371 0 266.7-149.4 414-180 5.9-1.2 18 0 18
 0 2 0 3 1 3 3v38c-76 158-257 219-435 219H0z`,
    rightharpoon: `M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3
-3.7-15.3-11-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2
-10.7 0-16.7 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58
 69.2 92 94.5zm0 0v40h399900v-40z`,
    rightharpoonplus: `M0 241v40h399993c4.7-4.7 7-9.3 7-14 0-9.3-3.7-15.3-11
-18-92.7-56.7-159-133.7-199-231-3.3-9.3-6-14.7-8-16-2-1.3-7-2-15-2-10.7 0-16.7
 2-18 6-2 2.7-1 9.7 3 21 15.3 42 36.7 81.8 64 119.5 27.3 37.7 58 69.2 92 94.5z
m0 0v40h399900v-40z m100 194v40h399900v-40zm0 0v40h399900v-40z`,
    rightharpoondown: `M399747 511c0 7.3 6.7 11 20 11 8 0 13-.8 15-2.5s4.7-6.8
 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3 8.5-5.8 9.5
-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3-64.7 57-92 95
-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 241v40h399900v-40z`,
    rightharpoondownplus: `M399747 705c0 7.3 6.7 11 20 11 8 0 13-.8
 15-2.5s4.7-6.8 8-15.5c40-94 99.3-166.3 178-217 13.3-8 20.3-12.3 21-13 5.3-3.3
 8.5-5.8 9.5-7.5 1-1.7 1.5-5.2 1.5-10.5s-2.3-10.3-7-15H0v40h399908c-34 25.3
-64.7 57-92 95-27.3 38-48.7 77.7-64 119-3.3 8.7-5 14-5 16zM0 435v40h399900v-40z
m0-194v40h400000v-40zm0 0v40h400000v-40z`,
    righthook: `M399859 241c-764 0 0 0 0 0 40-3.3 68.7-15.7 86-37 10-12 15-25.3
 15-40 0-22.7-9.8-40.7-29.5-54-19.7-13.3-43.5-21-71.5-23-17.3-1.3-26-8-26-20 0
-13.3 8.7-20 26-20 38 0 71 11.2 99 33.5 0 0 7 5.6 21 16.7 14 11.2 21 33.5 21
 66.8s-14 61.2-42 83.5c-28 22.3-61 33.5-99 33.5L0 241z M0 281v-40h399859v40z`,
    rightlinesegment: `M399960 241 V94 h40 V428 h-40 V281 H0 v-40z
M399960 241 V94 h40 V428 h-40 V281 H0 v-40z`,
    rightToFrom: `M400000 167c-70.7-42-118-97.7-142-167h-23c-15.3 0-23 .3-23
 1 0 1.3 5.3 13.7 16 37 18 35.3 41.3 69 70 101l7 8H0v40h399905l-7 8c-28.7 32
-52 65.7-70 101-10.7 23.3-16 35.7-16 37 0 .7 7.7 1 23 1h23c24-69.3 71.3-125 142
-167z M100 147v40h399900v-40zM0 341v40h399900v-40z`,
    // twoheadleftarrow is from glyph U+219E in font KaTeX AMS Regular
    twoheadleftarrow: `M0 167c68 40
 115.7 95.7 143 167h22c15.3 0 23-.3 23-1 0-1.3-5.3-13.7-16-37-18-35.3-41.3-69
-70-101l-7-8h125l9 7c50.7 39.3 85 86 103 140h46c0-4.7-6.3-18.7-19-42-18-35.3
-40-67.3-66-96l-9-9h399716v-40H284l9-9c26-28.7 48-60.7 66-96 12.7-23.333 19
-37.333 19-42h-46c-18 54-52.3 100.7-103 140l-9 7H95l7-8c28.7-32 52-65.7 70-101
 10.7-23.333 16-35.7 16-37 0-.7-7.7-1-23-1h-22C115.7 71.3 68 127 0 167z`,
    twoheadrightarrow: `M400000 167
c-68-40-115.7-95.7-143-167h-22c-15.3 0-23 .3-23 1 0 1.3 5.3 13.7 16 37 18 35.3
 41.3 69 70 101l7 8h-125l-9-7c-50.7-39.3-85-86-103-140h-46c0 4.7 6.3 18.7 19 42
 18 35.3 40 67.3 66 96l9 9H0v40h399716l-9 9c-26 28.7-48 60.7-66 96-12.7 23.333
-19 37.333-19 42h46c18-54 52.3-100.7 103-140l9-7h125l-7 8c-28.7 32-52 65.7-70
 101-10.7 23.333-16 35.7-16 37 0 .7 7.7 1 23 1h22c27.3-71.3 75-127 143-167z`,
    // tilde1 is a modified version of a glyph from the MnSymbol package
    tilde1: `M200 55.538c-77 0-168 73.953-177 73.953-3 0-7
-2.175-9-5.437L2 97c-1-2-2-4-2-6 0-4 2-7 5-9l20-12C116 12 171 0 207 0c86 0
 114 68 191 68 78 0 168-68 177-68 4 0 7 2 9 5l12 19c1 2.175 2 4.35 2 6.525 0
 4.35-2 7.613-5 9.788l-19 13.05c-92 63.077-116.937 75.308-183 76.128
-68.267.847-113-73.952-191-73.952z`,
    // ditto tilde2, tilde3, & tilde4
    tilde2: `M344 55.266c-142 0-300.638 81.316-311.5 86.418
-8.01 3.762-22.5 10.91-23.5 5.562L1 120c-1-2-1-3-1-4 0-5 3-9 8-10l18.4-9C160.9
 31.9 283 0 358 0c148 0 188 122 331 122s314-97 326-97c4 0 8 2 10 7l7 21.114
c1 2.14 1 3.21 1 4.28 0 5.347-3 9.626-7 10.696l-22.3 12.622C852.6 158.372 751
 181.476 676 181.476c-149 0-189-126.21-332-126.21z`,
    tilde3: `M786 59C457 59 32 175.242 13 175.242c-6 0-10-3.457
-11-10.37L.15 138c-1-7 3-12 10-13l19.2-6.4C378.4 40.7 634.3 0 804.3 0c337 0
 411.8 157 746.8 157 328 0 754-112 773-112 5 0 10 3 11 9l1 14.075c1 8.066-.697
 16.595-6.697 17.492l-21.052 7.31c-367.9 98.146-609.15 122.696-778.15 122.696
 -338 0-409-156.573-744-156.573z`,
    tilde4: `M786 58C457 58 32 177.487 13 177.487c-6 0-10-3.345
-11-10.035L.15 143c-1-7 3-12 10-13l22-6.7C381.2 35 637.15 0 807.15 0c337 0 409
 177 744 177 328 0 754-127 773-127 5 0 10 3 11 9l1 14.794c1 7.805-3 13.38-9
 14.495l-20.7 5.574c-366.85 99.79-607.3 139.372-776.3 139.372-338 0-409
 -175.236-744-175.236z`,
    // vec is from glyph U+20D7 in font KaTeX Main
    vec: `M377 20c0-5.333 1.833-10 5.5-14S391 0 397 0c4.667 0 8.667 1.667 12 5
3.333 2.667 6.667 9 10 19 6.667 24.667 20.333 43.667 41 57 7.333 4.667 11
10.667 11 18 0 6-1 10-3 12s-6.667 5-14 9c-28.667 14.667-53.667 35.667-75 63
-1.333 1.333-3.167 3.5-5.5 6.5s-4 4.833-5 5.5c-1 .667-2.5 1.333-4.5 2s-4.333 1
-7 1c-4.667 0-9.167-1.833-13.5-5.5S337 184 337 178c0-12.667 15.667-32.333 47-59
H213l-171-1c-8.667-6-13-12.333-13-19 0-4.667 4.333-11.333 13-20h359
c-16-25.333-24-45-24-59z`,
    // widehat1 is a modified version of a glyph from the MnSymbol package
    widehat1: `M529 0h5l519 115c5 1 9 5 9 10 0 1-1 2-1 3l-4 22
c-1 5-5 9-11 9h-2L532 67 19 159h-2c-5 0-9-4-11-9l-5-22c-1-6 2-12 8-13z`,
    // ditto widehat2, widehat3, & widehat4
    widehat2: `M1181 0h2l1171 176c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 220h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,
    widehat3: `M1181 0h2l1171 236c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 280h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,
    widehat4: `M1181 0h2l1171 296c6 0 10 5 10 11l-2 23c-1 6-5 10
-11 10h-1L1182 67 15 340h-1c-6 0-10-4-11-10l-2-23c-1-6 4-11 10-11z`,
    // widecheck paths are all inverted versions of widehat
    widecheck1: `M529,159h5l519,-115c5,-1,9,-5,9,-10c0,-1,-1,-2,-1,-3l-4,-22c-1,
-5,-5,-9,-11,-9h-2l-512,92l-513,-92h-2c-5,0,-9,4,-11,9l-5,22c-1,6,2,12,8,13z`,
    widecheck2: `M1181,220h2l1171,-176c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,153l-1167,-153h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,
    widecheck3: `M1181,280h2l1171,-236c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,213l-1167,-213h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,
    widecheck4: `M1181,340h2l1171,-296c6,0,10,-5,10,-11l-2,-23c-1,-6,-5,-10,
-11,-10h-1l-1168,273l-1167,-273h-1c-6,0,-10,4,-11,10l-2,23c-1,6,4,11,10,11z`,
    // The next ten paths support reaction arrows from the mhchem package.
    // Arrows for \ce{<-->} are offset from xAxis by 0.22ex, per mhchem in LaTeX
    // baraboveleftarrow is mostly from glyph U+2190 in font KaTeX Main
    baraboveleftarrow: `M400000 620h-399890l3 -3c68.7 -52.7 113.7 -120 135 -202
c4 -14.7 6 -23 6 -25c0 -7.3 -7 -11 -21 -11c-8 0 -13.2 0.8 -15.5 2.5
c-2.3 1.7 -4.2 5.8 -5.5 12.5c-1.3 4.7 -2.7 10.3 -4 17c-12 48.7 -34.8 92 -68.5 130
s-74.2 66.3 -121.5 85c-10 4 -16 7.7 -18 11c0 8.7 6 14.3 18 17c47.3 18.7 87.8 47
121.5 85s56.5 81.3 68.5 130c0.7 2 1.3 5 2 9s1.2 6.7 1.5 8c0.3 1.3 1 3.3 2 6
s2.2 4.5 3.5 5.5c1.3 1 3.3 1.8 6 2.5s6 1 10 1c14 0 21 -3.7 21 -11
c0 -2 -2 -10.3 -6 -25c-20 -79.3 -65 -146.7 -135 -202l-3 -3h399890z
M100 620v40h399900v-40z M0 241v40h399900v-40zM0 241v40h399900v-40z`,
    // rightarrowabovebar is mostly from glyph U+2192, KaTeX Main
    rightarrowabovebar: `M0 241v40h399891c-47.3 35.3-84 78-110 128-16.7 32
-27.7 63.7-33 95 0 1.3-.2 2.7-.5 4-.3 1.3-.5 2.3-.5 3 0 7.3 6.7 11 20 11 8 0
13.2-.8 15.5-2.5 2.3-1.7 4.2-5.5 5.5-11.5 2-13.3 5.7-27 11-41 14.7-44.7 39
-84.5 73-119.5s73.7-60.2 119-75.5c6-2 9-5.7 9-11s-3-9-9-11c-45.3-15.3-85-40.5
-119-75.5s-58.3-74.8-73-119.5c-4.7-14-8.3-27.3-11-40-1.3-6.7-3.2-10.8-5.5
-12.5-2.3-1.7-7.5-2.5-15.5-2.5-14 0-21 3.7-21 11 0 2 2 10.3 6 25 20.7 83.3 67
151.7 139 205zm96 379h399894v40H0zm0 0h399904v40H0z`,
    // The short left harpoon has 0.5em (i.e. 500 units) kern on the left end.
    // Ref from mhchem.sty: \rlap{\raisebox{-.22ex}{$\kern0.5em
    baraboveshortleftharpoon: `M507,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11
c1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17
c2,0.7,5,1,9,1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21
c-32,-87.3,-82.7,-157.7,-152,-211c0,0,-3,-3,-3,-3l399351,0l0,-40
c-398570,0,-399437,0,-399437,0z M593 435 v40 H399500 v-40z
M0 281 v-40 H399908 v40z M0 281 v-40 H399908 v40z`,
    rightharpoonaboveshortbar: `M0,241 l0,40c399126,0,399993,0,399993,0
c4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,
-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6
c-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z
M0 241 v40 H399908 v-40z M0 475 v-40 H399500 v40z M0 475 v-40 H399500 v40z`,
    shortbaraboveleftharpoon: `M7,435c-4,4,-6.3,8.7,-7,14c0,5.3,0.7,9,2,11
c1.3,2,5.3,5.3,12,10c90.7,54,156,130,196,228c3.3,10.7,6.3,16.3,9,17c2,0.7,5,1,9,
1c0,0,5,0,5,0c10.7,0,16.7,-2,18,-6c2,-2.7,1,-9.7,-3,-21c-32,-87.3,-82.7,-157.7,
-152,-211c0,0,-3,-3,-3,-3l399907,0l0,-40c-399126,0,-399993,0,-399993,0z
M93 435 v40 H400000 v-40z M500 241 v40 H400000 v-40z M500 241 v40 H400000 v-40z`,
    shortrightharpoonabovebar: `M53,241l0,40c398570,0,399437,0,399437,0
c4.7,-4.7,7,-9.3,7,-14c0,-9.3,-3.7,-15.3,-11,-18c-92.7,-56.7,-159,-133.7,-199,
-231c-3.3,-9.3,-6,-14.7,-8,-16c-2,-1.3,-7,-2,-15,-2c-10.7,0,-16.7,2,-18,6
c-2,2.7,-1,9.7,3,21c15.3,42,36.7,81.8,64,119.5c27.3,37.7,58,69.2,92,94.5z
M500 241 v40 H399408 v-40z M500 435 v40 H400000 v-40z`
  }, tallDelim = function O(_, H) {
    switch (_) {
      case "lbrack":
        return "M403 1759 V84 H666 V0 H319 V1759 v" + H + ` v1759 h347 v-84
H403z M403 1759 V0 H319 V1759 v` + H + " v1759 h84z";
      case "rbrack":
        return "M347 1759 V0 H0 V84 H263 V1759 v" + H + ` v1759 H0 v84 H347z
M347 1759 V0 H263 V1759 v` + H + " v1759 h84z";
      case "vert":
        return "M145 15 v585 v" + H + ` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v` + -H + ` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M188 15 H145 v585 v` + H + " v585 h43z";
      case "doublevert":
        return "M145 15 v585 v" + H + ` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v` + -H + ` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M188 15 H145 v585 v` + H + ` v585 h43z
M367 15 v585 v` + H + ` v585 c2.667,10,9.667,15,21,15
c10,0,16.667,-5,20,-15 v-585 v` + -H + ` v-585 c-2.667,-10,-9.667,-15,-21,-15
c-10,0,-16.667,5,-20,15z M410 15 H367 v585 v` + H + " v585 h43z";
      case "lfloor":
        return "M319 602 V0 H403 V602 v" + H + ` v1715 h263 v84 H319z
MM319 602 V0 H403 V602 v` + H + " v1715 H319z";
      case "rfloor":
        return "M319 602 V0 H403 V602 v" + H + ` v1799 H0 v-84 H319z
MM319 602 V0 H403 V602 v` + H + " v1715 H319z";
      case "lceil":
        return "M403 1759 V84 H666 V0 H319 V1759 v" + H + ` v602 h84z
M403 1759 V0 H319 V1759 v` + H + " v602 h84z";
      case "rceil":
        return "M347 1759 V0 H0 V84 H263 V1759 v" + H + ` v602 h84z
M347 1759 V0 h-84 V1759 v` + H + " v602 h84z";
      case "lparen":
        return `M863,9c0,-2,-2,-5,-6,-9c0,0,-17,0,-17,0c-12.7,0,-19.3,0.3,-20,1
c-5.3,5.3,-10.3,11,-15,17c-242.7,294.7,-395.3,682,-458,1162c-21.3,163.3,-33.3,349,
-36,557 l0,` + (H + 84) + `c0.2,6,0,26,0,60c2,159.3,10,310.7,24,454c53.3,528,210,
949.7,470,1265c4.7,6,9.7,11.7,15,17c0.7,0.7,7,1,19,1c0,0,18,0,18,0c4,-4,6,-7,6,-9
c0,-2.7,-3.3,-8.7,-10,-18c-135.3,-192.7,-235.5,-414.3,-300.5,-665c-65,-250.7,-102.5,
-544.7,-112.5,-882c-2,-104,-3,-167,-3,-189
l0,-` + (H + 92) + `c0,-162.7,5.7,-314,17,-454c20.7,-272,63.7,-513,129,-723c65.3,
-210,155.3,-396.3,270,-559c6.7,-9.3,10,-15.3,10,-18z`;
      case "rparen":
        return `M76,0c-16.7,0,-25,3,-25,9c0,2,2,6.3,6,13c21.3,28.7,42.3,60.3,
63,95c96.7,156.7,172.8,332.5,228.5,527.5c55.7,195,92.8,416.5,111.5,664.5
c11.3,139.3,17,290.7,17,454c0,28,1.7,43,3.3,45l0,` + (H + 9) + `
c-3,4,-3.3,16.7,-3.3,38c0,162,-5.7,313.7,-17,455c-18.7,248,-55.8,469.3,-111.5,664
c-55.7,194.7,-131.8,370.3,-228.5,527c-20.7,34.7,-41.7,66.3,-63,95c-2,3.3,-4,7,-6,11
c0,7.3,5.7,11,17,11c0,0,11,0,11,0c9.3,0,14.3,-0.3,15,-1c5.3,-5.3,10.3,-11,15,-17
c242.7,-294.7,395.3,-681.7,458,-1161c21.3,-164.7,33.3,-350.7,36,-558
l0,-` + (H + 144) + `c-2,-159.3,-10,-310.7,-24,-454c-53.3,-528,-210,-949.7,
-470,-1265c-4.7,-6,-9.7,-11.7,-15,-17c-0.7,-0.7,-6.7,-1,-18,-1z`;
      default:
        throw new Error("Unknown stretchy delimiter.");
    }
  };
  class DocumentFragment {
    // HtmlDomNode
    // Never used; needed for satisfying interface.
    constructor(_) {
      this.children = void 0, this.classes = void 0, this.height = void 0, this.depth = void 0, this.maxFontSize = void 0, this.style = void 0, this.children = _, this.classes = [], this.height = 0, this.depth = 0, this.maxFontSize = 0, this.style = {};
    }
    hasClass(_) {
      return utils.contains(this.classes, _);
    }
    /** Convert the fragment into a node. */
    toNode() {
      for (var _ = document.createDocumentFragment(), H = 0; H < this.children.length; H++)
        _.appendChild(this.children[H].toNode());
      return _;
    }
    /** Convert the fragment into HTML markup. */
    toMarkup() {
      for (var _ = "", H = 0; H < this.children.length; H++)
        _ += this.children[H].toMarkup();
      return _;
    }
    /**
     * Converts the math node into a string, similar to innerText. Applies to
     * MathDomNode's only.
     */
    toText() {
      var _ = (H) => H.toText();
      return this.children.map(_).join("");
    }
  }
  var fontMetricsData = {
    "AMS-Regular": {
      32: [0, 0, 0, 0, 0.25],
      65: [0, 0.68889, 0, 0, 0.72222],
      66: [0, 0.68889, 0, 0, 0.66667],
      67: [0, 0.68889, 0, 0, 0.72222],
      68: [0, 0.68889, 0, 0, 0.72222],
      69: [0, 0.68889, 0, 0, 0.66667],
      70: [0, 0.68889, 0, 0, 0.61111],
      71: [0, 0.68889, 0, 0, 0.77778],
      72: [0, 0.68889, 0, 0, 0.77778],
      73: [0, 0.68889, 0, 0, 0.38889],
      74: [0.16667, 0.68889, 0, 0, 0.5],
      75: [0, 0.68889, 0, 0, 0.77778],
      76: [0, 0.68889, 0, 0, 0.66667],
      77: [0, 0.68889, 0, 0, 0.94445],
      78: [0, 0.68889, 0, 0, 0.72222],
      79: [0.16667, 0.68889, 0, 0, 0.77778],
      80: [0, 0.68889, 0, 0, 0.61111],
      81: [0.16667, 0.68889, 0, 0, 0.77778],
      82: [0, 0.68889, 0, 0, 0.72222],
      83: [0, 0.68889, 0, 0, 0.55556],
      84: [0, 0.68889, 0, 0, 0.66667],
      85: [0, 0.68889, 0, 0, 0.72222],
      86: [0, 0.68889, 0, 0, 0.72222],
      87: [0, 0.68889, 0, 0, 1],
      88: [0, 0.68889, 0, 0, 0.72222],
      89: [0, 0.68889, 0, 0, 0.72222],
      90: [0, 0.68889, 0, 0, 0.66667],
      107: [0, 0.68889, 0, 0, 0.55556],
      160: [0, 0, 0, 0, 0.25],
      165: [0, 0.675, 0.025, 0, 0.75],
      174: [0.15559, 0.69224, 0, 0, 0.94666],
      240: [0, 0.68889, 0, 0, 0.55556],
      295: [0, 0.68889, 0, 0, 0.54028],
      710: [0, 0.825, 0, 0, 2.33334],
      732: [0, 0.9, 0, 0, 2.33334],
      770: [0, 0.825, 0, 0, 2.33334],
      771: [0, 0.9, 0, 0, 2.33334],
      989: [0.08167, 0.58167, 0, 0, 0.77778],
      1008: [0, 0.43056, 0.04028, 0, 0.66667],
      8245: [0, 0.54986, 0, 0, 0.275],
      8463: [0, 0.68889, 0, 0, 0.54028],
      8487: [0, 0.68889, 0, 0, 0.72222],
      8498: [0, 0.68889, 0, 0, 0.55556],
      8502: [0, 0.68889, 0, 0, 0.66667],
      8503: [0, 0.68889, 0, 0, 0.44445],
      8504: [0, 0.68889, 0, 0, 0.66667],
      8513: [0, 0.68889, 0, 0, 0.63889],
      8592: [-0.03598, 0.46402, 0, 0, 0.5],
      8594: [-0.03598, 0.46402, 0, 0, 0.5],
      8602: [-0.13313, 0.36687, 0, 0, 1],
      8603: [-0.13313, 0.36687, 0, 0, 1],
      8606: [0.01354, 0.52239, 0, 0, 1],
      8608: [0.01354, 0.52239, 0, 0, 1],
      8610: [0.01354, 0.52239, 0, 0, 1.11111],
      8611: [0.01354, 0.52239, 0, 0, 1.11111],
      8619: [0, 0.54986, 0, 0, 1],
      8620: [0, 0.54986, 0, 0, 1],
      8621: [-0.13313, 0.37788, 0, 0, 1.38889],
      8622: [-0.13313, 0.36687, 0, 0, 1],
      8624: [0, 0.69224, 0, 0, 0.5],
      8625: [0, 0.69224, 0, 0, 0.5],
      8630: [0, 0.43056, 0, 0, 1],
      8631: [0, 0.43056, 0, 0, 1],
      8634: [0.08198, 0.58198, 0, 0, 0.77778],
      8635: [0.08198, 0.58198, 0, 0, 0.77778],
      8638: [0.19444, 0.69224, 0, 0, 0.41667],
      8639: [0.19444, 0.69224, 0, 0, 0.41667],
      8642: [0.19444, 0.69224, 0, 0, 0.41667],
      8643: [0.19444, 0.69224, 0, 0, 0.41667],
      8644: [0.1808, 0.675, 0, 0, 1],
      8646: [0.1808, 0.675, 0, 0, 1],
      8647: [0.1808, 0.675, 0, 0, 1],
      8648: [0.19444, 0.69224, 0, 0, 0.83334],
      8649: [0.1808, 0.675, 0, 0, 1],
      8650: [0.19444, 0.69224, 0, 0, 0.83334],
      8651: [0.01354, 0.52239, 0, 0, 1],
      8652: [0.01354, 0.52239, 0, 0, 1],
      8653: [-0.13313, 0.36687, 0, 0, 1],
      8654: [-0.13313, 0.36687, 0, 0, 1],
      8655: [-0.13313, 0.36687, 0, 0, 1],
      8666: [0.13667, 0.63667, 0, 0, 1],
      8667: [0.13667, 0.63667, 0, 0, 1],
      8669: [-0.13313, 0.37788, 0, 0, 1],
      8672: [-0.064, 0.437, 0, 0, 1.334],
      8674: [-0.064, 0.437, 0, 0, 1.334],
      8705: [0, 0.825, 0, 0, 0.5],
      8708: [0, 0.68889, 0, 0, 0.55556],
      8709: [0.08167, 0.58167, 0, 0, 0.77778],
      8717: [0, 0.43056, 0, 0, 0.42917],
      8722: [-0.03598, 0.46402, 0, 0, 0.5],
      8724: [0.08198, 0.69224, 0, 0, 0.77778],
      8726: [0.08167, 0.58167, 0, 0, 0.77778],
      8733: [0, 0.69224, 0, 0, 0.77778],
      8736: [0, 0.69224, 0, 0, 0.72222],
      8737: [0, 0.69224, 0, 0, 0.72222],
      8738: [0.03517, 0.52239, 0, 0, 0.72222],
      8739: [0.08167, 0.58167, 0, 0, 0.22222],
      8740: [0.25142, 0.74111, 0, 0, 0.27778],
      8741: [0.08167, 0.58167, 0, 0, 0.38889],
      8742: [0.25142, 0.74111, 0, 0, 0.5],
      8756: [0, 0.69224, 0, 0, 0.66667],
      8757: [0, 0.69224, 0, 0, 0.66667],
      8764: [-0.13313, 0.36687, 0, 0, 0.77778],
      8765: [-0.13313, 0.37788, 0, 0, 0.77778],
      8769: [-0.13313, 0.36687, 0, 0, 0.77778],
      8770: [-0.03625, 0.46375, 0, 0, 0.77778],
      8774: [0.30274, 0.79383, 0, 0, 0.77778],
      8776: [-0.01688, 0.48312, 0, 0, 0.77778],
      8778: [0.08167, 0.58167, 0, 0, 0.77778],
      8782: [0.06062, 0.54986, 0, 0, 0.77778],
      8783: [0.06062, 0.54986, 0, 0, 0.77778],
      8785: [0.08198, 0.58198, 0, 0, 0.77778],
      8786: [0.08198, 0.58198, 0, 0, 0.77778],
      8787: [0.08198, 0.58198, 0, 0, 0.77778],
      8790: [0, 0.69224, 0, 0, 0.77778],
      8791: [0.22958, 0.72958, 0, 0, 0.77778],
      8796: [0.08198, 0.91667, 0, 0, 0.77778],
      8806: [0.25583, 0.75583, 0, 0, 0.77778],
      8807: [0.25583, 0.75583, 0, 0, 0.77778],
      8808: [0.25142, 0.75726, 0, 0, 0.77778],
      8809: [0.25142, 0.75726, 0, 0, 0.77778],
      8812: [0.25583, 0.75583, 0, 0, 0.5],
      8814: [0.20576, 0.70576, 0, 0, 0.77778],
      8815: [0.20576, 0.70576, 0, 0, 0.77778],
      8816: [0.30274, 0.79383, 0, 0, 0.77778],
      8817: [0.30274, 0.79383, 0, 0, 0.77778],
      8818: [0.22958, 0.72958, 0, 0, 0.77778],
      8819: [0.22958, 0.72958, 0, 0, 0.77778],
      8822: [0.1808, 0.675, 0, 0, 0.77778],
      8823: [0.1808, 0.675, 0, 0, 0.77778],
      8828: [0.13667, 0.63667, 0, 0, 0.77778],
      8829: [0.13667, 0.63667, 0, 0, 0.77778],
      8830: [0.22958, 0.72958, 0, 0, 0.77778],
      8831: [0.22958, 0.72958, 0, 0, 0.77778],
      8832: [0.20576, 0.70576, 0, 0, 0.77778],
      8833: [0.20576, 0.70576, 0, 0, 0.77778],
      8840: [0.30274, 0.79383, 0, 0, 0.77778],
      8841: [0.30274, 0.79383, 0, 0, 0.77778],
      8842: [0.13597, 0.63597, 0, 0, 0.77778],
      8843: [0.13597, 0.63597, 0, 0, 0.77778],
      8847: [0.03517, 0.54986, 0, 0, 0.77778],
      8848: [0.03517, 0.54986, 0, 0, 0.77778],
      8858: [0.08198, 0.58198, 0, 0, 0.77778],
      8859: [0.08198, 0.58198, 0, 0, 0.77778],
      8861: [0.08198, 0.58198, 0, 0, 0.77778],
      8862: [0, 0.675, 0, 0, 0.77778],
      8863: [0, 0.675, 0, 0, 0.77778],
      8864: [0, 0.675, 0, 0, 0.77778],
      8865: [0, 0.675, 0, 0, 0.77778],
      8872: [0, 0.69224, 0, 0, 0.61111],
      8873: [0, 0.69224, 0, 0, 0.72222],
      8874: [0, 0.69224, 0, 0, 0.88889],
      8876: [0, 0.68889, 0, 0, 0.61111],
      8877: [0, 0.68889, 0, 0, 0.61111],
      8878: [0, 0.68889, 0, 0, 0.72222],
      8879: [0, 0.68889, 0, 0, 0.72222],
      8882: [0.03517, 0.54986, 0, 0, 0.77778],
      8883: [0.03517, 0.54986, 0, 0, 0.77778],
      8884: [0.13667, 0.63667, 0, 0, 0.77778],
      8885: [0.13667, 0.63667, 0, 0, 0.77778],
      8888: [0, 0.54986, 0, 0, 1.11111],
      8890: [0.19444, 0.43056, 0, 0, 0.55556],
      8891: [0.19444, 0.69224, 0, 0, 0.61111],
      8892: [0.19444, 0.69224, 0, 0, 0.61111],
      8901: [0, 0.54986, 0, 0, 0.27778],
      8903: [0.08167, 0.58167, 0, 0, 0.77778],
      8905: [0.08167, 0.58167, 0, 0, 0.77778],
      8906: [0.08167, 0.58167, 0, 0, 0.77778],
      8907: [0, 0.69224, 0, 0, 0.77778],
      8908: [0, 0.69224, 0, 0, 0.77778],
      8909: [-0.03598, 0.46402, 0, 0, 0.77778],
      8910: [0, 0.54986, 0, 0, 0.76042],
      8911: [0, 0.54986, 0, 0, 0.76042],
      8912: [0.03517, 0.54986, 0, 0, 0.77778],
      8913: [0.03517, 0.54986, 0, 0, 0.77778],
      8914: [0, 0.54986, 0, 0, 0.66667],
      8915: [0, 0.54986, 0, 0, 0.66667],
      8916: [0, 0.69224, 0, 0, 0.66667],
      8918: [0.0391, 0.5391, 0, 0, 0.77778],
      8919: [0.0391, 0.5391, 0, 0, 0.77778],
      8920: [0.03517, 0.54986, 0, 0, 1.33334],
      8921: [0.03517, 0.54986, 0, 0, 1.33334],
      8922: [0.38569, 0.88569, 0, 0, 0.77778],
      8923: [0.38569, 0.88569, 0, 0, 0.77778],
      8926: [0.13667, 0.63667, 0, 0, 0.77778],
      8927: [0.13667, 0.63667, 0, 0, 0.77778],
      8928: [0.30274, 0.79383, 0, 0, 0.77778],
      8929: [0.30274, 0.79383, 0, 0, 0.77778],
      8934: [0.23222, 0.74111, 0, 0, 0.77778],
      8935: [0.23222, 0.74111, 0, 0, 0.77778],
      8936: [0.23222, 0.74111, 0, 0, 0.77778],
      8937: [0.23222, 0.74111, 0, 0, 0.77778],
      8938: [0.20576, 0.70576, 0, 0, 0.77778],
      8939: [0.20576, 0.70576, 0, 0, 0.77778],
      8940: [0.30274, 0.79383, 0, 0, 0.77778],
      8941: [0.30274, 0.79383, 0, 0, 0.77778],
      8994: [0.19444, 0.69224, 0, 0, 0.77778],
      8995: [0.19444, 0.69224, 0, 0, 0.77778],
      9416: [0.15559, 0.69224, 0, 0, 0.90222],
      9484: [0, 0.69224, 0, 0, 0.5],
      9488: [0, 0.69224, 0, 0, 0.5],
      9492: [0, 0.37788, 0, 0, 0.5],
      9496: [0, 0.37788, 0, 0, 0.5],
      9585: [0.19444, 0.68889, 0, 0, 0.88889],
      9586: [0.19444, 0.74111, 0, 0, 0.88889],
      9632: [0, 0.675, 0, 0, 0.77778],
      9633: [0, 0.675, 0, 0, 0.77778],
      9650: [0, 0.54986, 0, 0, 0.72222],
      9651: [0, 0.54986, 0, 0, 0.72222],
      9654: [0.03517, 0.54986, 0, 0, 0.77778],
      9660: [0, 0.54986, 0, 0, 0.72222],
      9661: [0, 0.54986, 0, 0, 0.72222],
      9664: [0.03517, 0.54986, 0, 0, 0.77778],
      9674: [0.11111, 0.69224, 0, 0, 0.66667],
      9733: [0.19444, 0.69224, 0, 0, 0.94445],
      10003: [0, 0.69224, 0, 0, 0.83334],
      10016: [0, 0.69224, 0, 0, 0.83334],
      10731: [0.11111, 0.69224, 0, 0, 0.66667],
      10846: [0.19444, 0.75583, 0, 0, 0.61111],
      10877: [0.13667, 0.63667, 0, 0, 0.77778],
      10878: [0.13667, 0.63667, 0, 0, 0.77778],
      10885: [0.25583, 0.75583, 0, 0, 0.77778],
      10886: [0.25583, 0.75583, 0, 0, 0.77778],
      10887: [0.13597, 0.63597, 0, 0, 0.77778],
      10888: [0.13597, 0.63597, 0, 0, 0.77778],
      10889: [0.26167, 0.75726, 0, 0, 0.77778],
      10890: [0.26167, 0.75726, 0, 0, 0.77778],
      10891: [0.48256, 0.98256, 0, 0, 0.77778],
      10892: [0.48256, 0.98256, 0, 0, 0.77778],
      10901: [0.13667, 0.63667, 0, 0, 0.77778],
      10902: [0.13667, 0.63667, 0, 0, 0.77778],
      10933: [0.25142, 0.75726, 0, 0, 0.77778],
      10934: [0.25142, 0.75726, 0, 0, 0.77778],
      10935: [0.26167, 0.75726, 0, 0, 0.77778],
      10936: [0.26167, 0.75726, 0, 0, 0.77778],
      10937: [0.26167, 0.75726, 0, 0, 0.77778],
      10938: [0.26167, 0.75726, 0, 0, 0.77778],
      10949: [0.25583, 0.75583, 0, 0, 0.77778],
      10950: [0.25583, 0.75583, 0, 0, 0.77778],
      10955: [0.28481, 0.79383, 0, 0, 0.77778],
      10956: [0.28481, 0.79383, 0, 0, 0.77778],
      57350: [0.08167, 0.58167, 0, 0, 0.22222],
      57351: [0.08167, 0.58167, 0, 0, 0.38889],
      57352: [0.08167, 0.58167, 0, 0, 0.77778],
      57353: [0, 0.43056, 0.04028, 0, 0.66667],
      57356: [0.25142, 0.75726, 0, 0, 0.77778],
      57357: [0.25142, 0.75726, 0, 0, 0.77778],
      57358: [0.41951, 0.91951, 0, 0, 0.77778],
      57359: [0.30274, 0.79383, 0, 0, 0.77778],
      57360: [0.30274, 0.79383, 0, 0, 0.77778],
      57361: [0.41951, 0.91951, 0, 0, 0.77778],
      57366: [0.25142, 0.75726, 0, 0, 0.77778],
      57367: [0.25142, 0.75726, 0, 0, 0.77778],
      57368: [0.25142, 0.75726, 0, 0, 0.77778],
      57369: [0.25142, 0.75726, 0, 0, 0.77778],
      57370: [0.13597, 0.63597, 0, 0, 0.77778],
      57371: [0.13597, 0.63597, 0, 0, 0.77778]
    },
    "Caligraphic-Regular": {
      32: [0, 0, 0, 0, 0.25],
      65: [0, 0.68333, 0, 0.19445, 0.79847],
      66: [0, 0.68333, 0.03041, 0.13889, 0.65681],
      67: [0, 0.68333, 0.05834, 0.13889, 0.52653],
      68: [0, 0.68333, 0.02778, 0.08334, 0.77139],
      69: [0, 0.68333, 0.08944, 0.11111, 0.52778],
      70: [0, 0.68333, 0.09931, 0.11111, 0.71875],
      71: [0.09722, 0.68333, 0.0593, 0.11111, 0.59487],
      72: [0, 0.68333, 965e-5, 0.11111, 0.84452],
      73: [0, 0.68333, 0.07382, 0, 0.54452],
      74: [0.09722, 0.68333, 0.18472, 0.16667, 0.67778],
      75: [0, 0.68333, 0.01445, 0.05556, 0.76195],
      76: [0, 0.68333, 0, 0.13889, 0.68972],
      77: [0, 0.68333, 0, 0.13889, 1.2009],
      78: [0, 0.68333, 0.14736, 0.08334, 0.82049],
      79: [0, 0.68333, 0.02778, 0.11111, 0.79611],
      80: [0, 0.68333, 0.08222, 0.08334, 0.69556],
      81: [0.09722, 0.68333, 0, 0.11111, 0.81667],
      82: [0, 0.68333, 0, 0.08334, 0.8475],
      83: [0, 0.68333, 0.075, 0.13889, 0.60556],
      84: [0, 0.68333, 0.25417, 0, 0.54464],
      85: [0, 0.68333, 0.09931, 0.08334, 0.62583],
      86: [0, 0.68333, 0.08222, 0, 0.61278],
      87: [0, 0.68333, 0.08222, 0.08334, 0.98778],
      88: [0, 0.68333, 0.14643, 0.13889, 0.7133],
      89: [0.09722, 0.68333, 0.08222, 0.08334, 0.66834],
      90: [0, 0.68333, 0.07944, 0.13889, 0.72473],
      160: [0, 0, 0, 0, 0.25]
    },
    "Fraktur-Regular": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69141, 0, 0, 0.29574],
      34: [0, 0.69141, 0, 0, 0.21471],
      38: [0, 0.69141, 0, 0, 0.73786],
      39: [0, 0.69141, 0, 0, 0.21201],
      40: [0.24982, 0.74947, 0, 0, 0.38865],
      41: [0.24982, 0.74947, 0, 0, 0.38865],
      42: [0, 0.62119, 0, 0, 0.27764],
      43: [0.08319, 0.58283, 0, 0, 0.75623],
      44: [0, 0.10803, 0, 0, 0.27764],
      45: [0.08319, 0.58283, 0, 0, 0.75623],
      46: [0, 0.10803, 0, 0, 0.27764],
      47: [0.24982, 0.74947, 0, 0, 0.50181],
      48: [0, 0.47534, 0, 0, 0.50181],
      49: [0, 0.47534, 0, 0, 0.50181],
      50: [0, 0.47534, 0, 0, 0.50181],
      51: [0.18906, 0.47534, 0, 0, 0.50181],
      52: [0.18906, 0.47534, 0, 0, 0.50181],
      53: [0.18906, 0.47534, 0, 0, 0.50181],
      54: [0, 0.69141, 0, 0, 0.50181],
      55: [0.18906, 0.47534, 0, 0, 0.50181],
      56: [0, 0.69141, 0, 0, 0.50181],
      57: [0.18906, 0.47534, 0, 0, 0.50181],
      58: [0, 0.47534, 0, 0, 0.21606],
      59: [0.12604, 0.47534, 0, 0, 0.21606],
      61: [-0.13099, 0.36866, 0, 0, 0.75623],
      63: [0, 0.69141, 0, 0, 0.36245],
      65: [0, 0.69141, 0, 0, 0.7176],
      66: [0, 0.69141, 0, 0, 0.88397],
      67: [0, 0.69141, 0, 0, 0.61254],
      68: [0, 0.69141, 0, 0, 0.83158],
      69: [0, 0.69141, 0, 0, 0.66278],
      70: [0.12604, 0.69141, 0, 0, 0.61119],
      71: [0, 0.69141, 0, 0, 0.78539],
      72: [0.06302, 0.69141, 0, 0, 0.7203],
      73: [0, 0.69141, 0, 0, 0.55448],
      74: [0.12604, 0.69141, 0, 0, 0.55231],
      75: [0, 0.69141, 0, 0, 0.66845],
      76: [0, 0.69141, 0, 0, 0.66602],
      77: [0, 0.69141, 0, 0, 1.04953],
      78: [0, 0.69141, 0, 0, 0.83212],
      79: [0, 0.69141, 0, 0, 0.82699],
      80: [0.18906, 0.69141, 0, 0, 0.82753],
      81: [0.03781, 0.69141, 0, 0, 0.82699],
      82: [0, 0.69141, 0, 0, 0.82807],
      83: [0, 0.69141, 0, 0, 0.82861],
      84: [0, 0.69141, 0, 0, 0.66899],
      85: [0, 0.69141, 0, 0, 0.64576],
      86: [0, 0.69141, 0, 0, 0.83131],
      87: [0, 0.69141, 0, 0, 1.04602],
      88: [0, 0.69141, 0, 0, 0.71922],
      89: [0.18906, 0.69141, 0, 0, 0.83293],
      90: [0.12604, 0.69141, 0, 0, 0.60201],
      91: [0.24982, 0.74947, 0, 0, 0.27764],
      93: [0.24982, 0.74947, 0, 0, 0.27764],
      94: [0, 0.69141, 0, 0, 0.49965],
      97: [0, 0.47534, 0, 0, 0.50046],
      98: [0, 0.69141, 0, 0, 0.51315],
      99: [0, 0.47534, 0, 0, 0.38946],
      100: [0, 0.62119, 0, 0, 0.49857],
      101: [0, 0.47534, 0, 0, 0.40053],
      102: [0.18906, 0.69141, 0, 0, 0.32626],
      103: [0.18906, 0.47534, 0, 0, 0.5037],
      104: [0.18906, 0.69141, 0, 0, 0.52126],
      105: [0, 0.69141, 0, 0, 0.27899],
      106: [0, 0.69141, 0, 0, 0.28088],
      107: [0, 0.69141, 0, 0, 0.38946],
      108: [0, 0.69141, 0, 0, 0.27953],
      109: [0, 0.47534, 0, 0, 0.76676],
      110: [0, 0.47534, 0, 0, 0.52666],
      111: [0, 0.47534, 0, 0, 0.48885],
      112: [0.18906, 0.52396, 0, 0, 0.50046],
      113: [0.18906, 0.47534, 0, 0, 0.48912],
      114: [0, 0.47534, 0, 0, 0.38919],
      115: [0, 0.47534, 0, 0, 0.44266],
      116: [0, 0.62119, 0, 0, 0.33301],
      117: [0, 0.47534, 0, 0, 0.5172],
      118: [0, 0.52396, 0, 0, 0.5118],
      119: [0, 0.52396, 0, 0, 0.77351],
      120: [0.18906, 0.47534, 0, 0, 0.38865],
      121: [0.18906, 0.47534, 0, 0, 0.49884],
      122: [0.18906, 0.47534, 0, 0, 0.39054],
      160: [0, 0, 0, 0, 0.25],
      8216: [0, 0.69141, 0, 0, 0.21471],
      8217: [0, 0.69141, 0, 0, 0.21471],
      58112: [0, 0.62119, 0, 0, 0.49749],
      58113: [0, 0.62119, 0, 0, 0.4983],
      58114: [0.18906, 0.69141, 0, 0, 0.33328],
      58115: [0.18906, 0.69141, 0, 0, 0.32923],
      58116: [0.18906, 0.47534, 0, 0, 0.50343],
      58117: [0, 0.69141, 0, 0, 0.33301],
      58118: [0, 0.62119, 0, 0, 0.33409],
      58119: [0, 0.47534, 0, 0, 0.50073]
    },
    "Main-Bold": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69444, 0, 0, 0.35],
      34: [0, 0.69444, 0, 0, 0.60278],
      35: [0.19444, 0.69444, 0, 0, 0.95833],
      36: [0.05556, 0.75, 0, 0, 0.575],
      37: [0.05556, 0.75, 0, 0, 0.95833],
      38: [0, 0.69444, 0, 0, 0.89444],
      39: [0, 0.69444, 0, 0, 0.31944],
      40: [0.25, 0.75, 0, 0, 0.44722],
      41: [0.25, 0.75, 0, 0, 0.44722],
      42: [0, 0.75, 0, 0, 0.575],
      43: [0.13333, 0.63333, 0, 0, 0.89444],
      44: [0.19444, 0.15556, 0, 0, 0.31944],
      45: [0, 0.44444, 0, 0, 0.38333],
      46: [0, 0.15556, 0, 0, 0.31944],
      47: [0.25, 0.75, 0, 0, 0.575],
      48: [0, 0.64444, 0, 0, 0.575],
      49: [0, 0.64444, 0, 0, 0.575],
      50: [0, 0.64444, 0, 0, 0.575],
      51: [0, 0.64444, 0, 0, 0.575],
      52: [0, 0.64444, 0, 0, 0.575],
      53: [0, 0.64444, 0, 0, 0.575],
      54: [0, 0.64444, 0, 0, 0.575],
      55: [0, 0.64444, 0, 0, 0.575],
      56: [0, 0.64444, 0, 0, 0.575],
      57: [0, 0.64444, 0, 0, 0.575],
      58: [0, 0.44444, 0, 0, 0.31944],
      59: [0.19444, 0.44444, 0, 0, 0.31944],
      60: [0.08556, 0.58556, 0, 0, 0.89444],
      61: [-0.10889, 0.39111, 0, 0, 0.89444],
      62: [0.08556, 0.58556, 0, 0, 0.89444],
      63: [0, 0.69444, 0, 0, 0.54305],
      64: [0, 0.69444, 0, 0, 0.89444],
      65: [0, 0.68611, 0, 0, 0.86944],
      66: [0, 0.68611, 0, 0, 0.81805],
      67: [0, 0.68611, 0, 0, 0.83055],
      68: [0, 0.68611, 0, 0, 0.88194],
      69: [0, 0.68611, 0, 0, 0.75555],
      70: [0, 0.68611, 0, 0, 0.72361],
      71: [0, 0.68611, 0, 0, 0.90416],
      72: [0, 0.68611, 0, 0, 0.9],
      73: [0, 0.68611, 0, 0, 0.43611],
      74: [0, 0.68611, 0, 0, 0.59444],
      75: [0, 0.68611, 0, 0, 0.90138],
      76: [0, 0.68611, 0, 0, 0.69166],
      77: [0, 0.68611, 0, 0, 1.09166],
      78: [0, 0.68611, 0, 0, 0.9],
      79: [0, 0.68611, 0, 0, 0.86388],
      80: [0, 0.68611, 0, 0, 0.78611],
      81: [0.19444, 0.68611, 0, 0, 0.86388],
      82: [0, 0.68611, 0, 0, 0.8625],
      83: [0, 0.68611, 0, 0, 0.63889],
      84: [0, 0.68611, 0, 0, 0.8],
      85: [0, 0.68611, 0, 0, 0.88472],
      86: [0, 0.68611, 0.01597, 0, 0.86944],
      87: [0, 0.68611, 0.01597, 0, 1.18888],
      88: [0, 0.68611, 0, 0, 0.86944],
      89: [0, 0.68611, 0.02875, 0, 0.86944],
      90: [0, 0.68611, 0, 0, 0.70277],
      91: [0.25, 0.75, 0, 0, 0.31944],
      92: [0.25, 0.75, 0, 0, 0.575],
      93: [0.25, 0.75, 0, 0, 0.31944],
      94: [0, 0.69444, 0, 0, 0.575],
      95: [0.31, 0.13444, 0.03194, 0, 0.575],
      97: [0, 0.44444, 0, 0, 0.55902],
      98: [0, 0.69444, 0, 0, 0.63889],
      99: [0, 0.44444, 0, 0, 0.51111],
      100: [0, 0.69444, 0, 0, 0.63889],
      101: [0, 0.44444, 0, 0, 0.52708],
      102: [0, 0.69444, 0.10903, 0, 0.35139],
      103: [0.19444, 0.44444, 0.01597, 0, 0.575],
      104: [0, 0.69444, 0, 0, 0.63889],
      105: [0, 0.69444, 0, 0, 0.31944],
      106: [0.19444, 0.69444, 0, 0, 0.35139],
      107: [0, 0.69444, 0, 0, 0.60694],
      108: [0, 0.69444, 0, 0, 0.31944],
      109: [0, 0.44444, 0, 0, 0.95833],
      110: [0, 0.44444, 0, 0, 0.63889],
      111: [0, 0.44444, 0, 0, 0.575],
      112: [0.19444, 0.44444, 0, 0, 0.63889],
      113: [0.19444, 0.44444, 0, 0, 0.60694],
      114: [0, 0.44444, 0, 0, 0.47361],
      115: [0, 0.44444, 0, 0, 0.45361],
      116: [0, 0.63492, 0, 0, 0.44722],
      117: [0, 0.44444, 0, 0, 0.63889],
      118: [0, 0.44444, 0.01597, 0, 0.60694],
      119: [0, 0.44444, 0.01597, 0, 0.83055],
      120: [0, 0.44444, 0, 0, 0.60694],
      121: [0.19444, 0.44444, 0.01597, 0, 0.60694],
      122: [0, 0.44444, 0, 0, 0.51111],
      123: [0.25, 0.75, 0, 0, 0.575],
      124: [0.25, 0.75, 0, 0, 0.31944],
      125: [0.25, 0.75, 0, 0, 0.575],
      126: [0.35, 0.34444, 0, 0, 0.575],
      160: [0, 0, 0, 0, 0.25],
      163: [0, 0.69444, 0, 0, 0.86853],
      168: [0, 0.69444, 0, 0, 0.575],
      172: [0, 0.44444, 0, 0, 0.76666],
      176: [0, 0.69444, 0, 0, 0.86944],
      177: [0.13333, 0.63333, 0, 0, 0.89444],
      184: [0.17014, 0, 0, 0, 0.51111],
      198: [0, 0.68611, 0, 0, 1.04166],
      215: [0.13333, 0.63333, 0, 0, 0.89444],
      216: [0.04861, 0.73472, 0, 0, 0.89444],
      223: [0, 0.69444, 0, 0, 0.59722],
      230: [0, 0.44444, 0, 0, 0.83055],
      247: [0.13333, 0.63333, 0, 0, 0.89444],
      248: [0.09722, 0.54167, 0, 0, 0.575],
      305: [0, 0.44444, 0, 0, 0.31944],
      338: [0, 0.68611, 0, 0, 1.16944],
      339: [0, 0.44444, 0, 0, 0.89444],
      567: [0.19444, 0.44444, 0, 0, 0.35139],
      710: [0, 0.69444, 0, 0, 0.575],
      711: [0, 0.63194, 0, 0, 0.575],
      713: [0, 0.59611, 0, 0, 0.575],
      714: [0, 0.69444, 0, 0, 0.575],
      715: [0, 0.69444, 0, 0, 0.575],
      728: [0, 0.69444, 0, 0, 0.575],
      729: [0, 0.69444, 0, 0, 0.31944],
      730: [0, 0.69444, 0, 0, 0.86944],
      732: [0, 0.69444, 0, 0, 0.575],
      733: [0, 0.69444, 0, 0, 0.575],
      915: [0, 0.68611, 0, 0, 0.69166],
      916: [0, 0.68611, 0, 0, 0.95833],
      920: [0, 0.68611, 0, 0, 0.89444],
      923: [0, 0.68611, 0, 0, 0.80555],
      926: [0, 0.68611, 0, 0, 0.76666],
      928: [0, 0.68611, 0, 0, 0.9],
      931: [0, 0.68611, 0, 0, 0.83055],
      933: [0, 0.68611, 0, 0, 0.89444],
      934: [0, 0.68611, 0, 0, 0.83055],
      936: [0, 0.68611, 0, 0, 0.89444],
      937: [0, 0.68611, 0, 0, 0.83055],
      8211: [0, 0.44444, 0.03194, 0, 0.575],
      8212: [0, 0.44444, 0.03194, 0, 1.14999],
      8216: [0, 0.69444, 0, 0, 0.31944],
      8217: [0, 0.69444, 0, 0, 0.31944],
      8220: [0, 0.69444, 0, 0, 0.60278],
      8221: [0, 0.69444, 0, 0, 0.60278],
      8224: [0.19444, 0.69444, 0, 0, 0.51111],
      8225: [0.19444, 0.69444, 0, 0, 0.51111],
      8242: [0, 0.55556, 0, 0, 0.34444],
      8407: [0, 0.72444, 0.15486, 0, 0.575],
      8463: [0, 0.69444, 0, 0, 0.66759],
      8465: [0, 0.69444, 0, 0, 0.83055],
      8467: [0, 0.69444, 0, 0, 0.47361],
      8472: [0.19444, 0.44444, 0, 0, 0.74027],
      8476: [0, 0.69444, 0, 0, 0.83055],
      8501: [0, 0.69444, 0, 0, 0.70277],
      8592: [-0.10889, 0.39111, 0, 0, 1.14999],
      8593: [0.19444, 0.69444, 0, 0, 0.575],
      8594: [-0.10889, 0.39111, 0, 0, 1.14999],
      8595: [0.19444, 0.69444, 0, 0, 0.575],
      8596: [-0.10889, 0.39111, 0, 0, 1.14999],
      8597: [0.25, 0.75, 0, 0, 0.575],
      8598: [0.19444, 0.69444, 0, 0, 1.14999],
      8599: [0.19444, 0.69444, 0, 0, 1.14999],
      8600: [0.19444, 0.69444, 0, 0, 1.14999],
      8601: [0.19444, 0.69444, 0, 0, 1.14999],
      8636: [-0.10889, 0.39111, 0, 0, 1.14999],
      8637: [-0.10889, 0.39111, 0, 0, 1.14999],
      8640: [-0.10889, 0.39111, 0, 0, 1.14999],
      8641: [-0.10889, 0.39111, 0, 0, 1.14999],
      8656: [-0.10889, 0.39111, 0, 0, 1.14999],
      8657: [0.19444, 0.69444, 0, 0, 0.70277],
      8658: [-0.10889, 0.39111, 0, 0, 1.14999],
      8659: [0.19444, 0.69444, 0, 0, 0.70277],
      8660: [-0.10889, 0.39111, 0, 0, 1.14999],
      8661: [0.25, 0.75, 0, 0, 0.70277],
      8704: [0, 0.69444, 0, 0, 0.63889],
      8706: [0, 0.69444, 0.06389, 0, 0.62847],
      8707: [0, 0.69444, 0, 0, 0.63889],
      8709: [0.05556, 0.75, 0, 0, 0.575],
      8711: [0, 0.68611, 0, 0, 0.95833],
      8712: [0.08556, 0.58556, 0, 0, 0.76666],
      8715: [0.08556, 0.58556, 0, 0, 0.76666],
      8722: [0.13333, 0.63333, 0, 0, 0.89444],
      8723: [0.13333, 0.63333, 0, 0, 0.89444],
      8725: [0.25, 0.75, 0, 0, 0.575],
      8726: [0.25, 0.75, 0, 0, 0.575],
      8727: [-0.02778, 0.47222, 0, 0, 0.575],
      8728: [-0.02639, 0.47361, 0, 0, 0.575],
      8729: [-0.02639, 0.47361, 0, 0, 0.575],
      8730: [0.18, 0.82, 0, 0, 0.95833],
      8733: [0, 0.44444, 0, 0, 0.89444],
      8734: [0, 0.44444, 0, 0, 1.14999],
      8736: [0, 0.69224, 0, 0, 0.72222],
      8739: [0.25, 0.75, 0, 0, 0.31944],
      8741: [0.25, 0.75, 0, 0, 0.575],
      8743: [0, 0.55556, 0, 0, 0.76666],
      8744: [0, 0.55556, 0, 0, 0.76666],
      8745: [0, 0.55556, 0, 0, 0.76666],
      8746: [0, 0.55556, 0, 0, 0.76666],
      8747: [0.19444, 0.69444, 0.12778, 0, 0.56875],
      8764: [-0.10889, 0.39111, 0, 0, 0.89444],
      8768: [0.19444, 0.69444, 0, 0, 0.31944],
      8771: [222e-5, 0.50222, 0, 0, 0.89444],
      8773: [0.027, 0.638, 0, 0, 0.894],
      8776: [0.02444, 0.52444, 0, 0, 0.89444],
      8781: [222e-5, 0.50222, 0, 0, 0.89444],
      8801: [222e-5, 0.50222, 0, 0, 0.89444],
      8804: [0.19667, 0.69667, 0, 0, 0.89444],
      8805: [0.19667, 0.69667, 0, 0, 0.89444],
      8810: [0.08556, 0.58556, 0, 0, 1.14999],
      8811: [0.08556, 0.58556, 0, 0, 1.14999],
      8826: [0.08556, 0.58556, 0, 0, 0.89444],
      8827: [0.08556, 0.58556, 0, 0, 0.89444],
      8834: [0.08556, 0.58556, 0, 0, 0.89444],
      8835: [0.08556, 0.58556, 0, 0, 0.89444],
      8838: [0.19667, 0.69667, 0, 0, 0.89444],
      8839: [0.19667, 0.69667, 0, 0, 0.89444],
      8846: [0, 0.55556, 0, 0, 0.76666],
      8849: [0.19667, 0.69667, 0, 0, 0.89444],
      8850: [0.19667, 0.69667, 0, 0, 0.89444],
      8851: [0, 0.55556, 0, 0, 0.76666],
      8852: [0, 0.55556, 0, 0, 0.76666],
      8853: [0.13333, 0.63333, 0, 0, 0.89444],
      8854: [0.13333, 0.63333, 0, 0, 0.89444],
      8855: [0.13333, 0.63333, 0, 0, 0.89444],
      8856: [0.13333, 0.63333, 0, 0, 0.89444],
      8857: [0.13333, 0.63333, 0, 0, 0.89444],
      8866: [0, 0.69444, 0, 0, 0.70277],
      8867: [0, 0.69444, 0, 0, 0.70277],
      8868: [0, 0.69444, 0, 0, 0.89444],
      8869: [0, 0.69444, 0, 0, 0.89444],
      8900: [-0.02639, 0.47361, 0, 0, 0.575],
      8901: [-0.02639, 0.47361, 0, 0, 0.31944],
      8902: [-0.02778, 0.47222, 0, 0, 0.575],
      8968: [0.25, 0.75, 0, 0, 0.51111],
      8969: [0.25, 0.75, 0, 0, 0.51111],
      8970: [0.25, 0.75, 0, 0, 0.51111],
      8971: [0.25, 0.75, 0, 0, 0.51111],
      8994: [-0.13889, 0.36111, 0, 0, 1.14999],
      8995: [-0.13889, 0.36111, 0, 0, 1.14999],
      9651: [0.19444, 0.69444, 0, 0, 1.02222],
      9657: [-0.02778, 0.47222, 0, 0, 0.575],
      9661: [0.19444, 0.69444, 0, 0, 1.02222],
      9667: [-0.02778, 0.47222, 0, 0, 0.575],
      9711: [0.19444, 0.69444, 0, 0, 1.14999],
      9824: [0.12963, 0.69444, 0, 0, 0.89444],
      9825: [0.12963, 0.69444, 0, 0, 0.89444],
      9826: [0.12963, 0.69444, 0, 0, 0.89444],
      9827: [0.12963, 0.69444, 0, 0, 0.89444],
      9837: [0, 0.75, 0, 0, 0.44722],
      9838: [0.19444, 0.69444, 0, 0, 0.44722],
      9839: [0.19444, 0.69444, 0, 0, 0.44722],
      10216: [0.25, 0.75, 0, 0, 0.44722],
      10217: [0.25, 0.75, 0, 0, 0.44722],
      10815: [0, 0.68611, 0, 0, 0.9],
      10927: [0.19667, 0.69667, 0, 0, 0.89444],
      10928: [0.19667, 0.69667, 0, 0, 0.89444],
      57376: [0.19444, 0.69444, 0, 0, 0]
    },
    "Main-BoldItalic": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69444, 0.11417, 0, 0.38611],
      34: [0, 0.69444, 0.07939, 0, 0.62055],
      35: [0.19444, 0.69444, 0.06833, 0, 0.94444],
      37: [0.05556, 0.75, 0.12861, 0, 0.94444],
      38: [0, 0.69444, 0.08528, 0, 0.88555],
      39: [0, 0.69444, 0.12945, 0, 0.35555],
      40: [0.25, 0.75, 0.15806, 0, 0.47333],
      41: [0.25, 0.75, 0.03306, 0, 0.47333],
      42: [0, 0.75, 0.14333, 0, 0.59111],
      43: [0.10333, 0.60333, 0.03306, 0, 0.88555],
      44: [0.19444, 0.14722, 0, 0, 0.35555],
      45: [0, 0.44444, 0.02611, 0, 0.41444],
      46: [0, 0.14722, 0, 0, 0.35555],
      47: [0.25, 0.75, 0.15806, 0, 0.59111],
      48: [0, 0.64444, 0.13167, 0, 0.59111],
      49: [0, 0.64444, 0.13167, 0, 0.59111],
      50: [0, 0.64444, 0.13167, 0, 0.59111],
      51: [0, 0.64444, 0.13167, 0, 0.59111],
      52: [0.19444, 0.64444, 0.13167, 0, 0.59111],
      53: [0, 0.64444, 0.13167, 0, 0.59111],
      54: [0, 0.64444, 0.13167, 0, 0.59111],
      55: [0.19444, 0.64444, 0.13167, 0, 0.59111],
      56: [0, 0.64444, 0.13167, 0, 0.59111],
      57: [0, 0.64444, 0.13167, 0, 0.59111],
      58: [0, 0.44444, 0.06695, 0, 0.35555],
      59: [0.19444, 0.44444, 0.06695, 0, 0.35555],
      61: [-0.10889, 0.39111, 0.06833, 0, 0.88555],
      63: [0, 0.69444, 0.11472, 0, 0.59111],
      64: [0, 0.69444, 0.09208, 0, 0.88555],
      65: [0, 0.68611, 0, 0, 0.86555],
      66: [0, 0.68611, 0.0992, 0, 0.81666],
      67: [0, 0.68611, 0.14208, 0, 0.82666],
      68: [0, 0.68611, 0.09062, 0, 0.87555],
      69: [0, 0.68611, 0.11431, 0, 0.75666],
      70: [0, 0.68611, 0.12903, 0, 0.72722],
      71: [0, 0.68611, 0.07347, 0, 0.89527],
      72: [0, 0.68611, 0.17208, 0, 0.8961],
      73: [0, 0.68611, 0.15681, 0, 0.47166],
      74: [0, 0.68611, 0.145, 0, 0.61055],
      75: [0, 0.68611, 0.14208, 0, 0.89499],
      76: [0, 0.68611, 0, 0, 0.69777],
      77: [0, 0.68611, 0.17208, 0, 1.07277],
      78: [0, 0.68611, 0.17208, 0, 0.8961],
      79: [0, 0.68611, 0.09062, 0, 0.85499],
      80: [0, 0.68611, 0.0992, 0, 0.78721],
      81: [0.19444, 0.68611, 0.09062, 0, 0.85499],
      82: [0, 0.68611, 0.02559, 0, 0.85944],
      83: [0, 0.68611, 0.11264, 0, 0.64999],
      84: [0, 0.68611, 0.12903, 0, 0.7961],
      85: [0, 0.68611, 0.17208, 0, 0.88083],
      86: [0, 0.68611, 0.18625, 0, 0.86555],
      87: [0, 0.68611, 0.18625, 0, 1.15999],
      88: [0, 0.68611, 0.15681, 0, 0.86555],
      89: [0, 0.68611, 0.19803, 0, 0.86555],
      90: [0, 0.68611, 0.14208, 0, 0.70888],
      91: [0.25, 0.75, 0.1875, 0, 0.35611],
      93: [0.25, 0.75, 0.09972, 0, 0.35611],
      94: [0, 0.69444, 0.06709, 0, 0.59111],
      95: [0.31, 0.13444, 0.09811, 0, 0.59111],
      97: [0, 0.44444, 0.09426, 0, 0.59111],
      98: [0, 0.69444, 0.07861, 0, 0.53222],
      99: [0, 0.44444, 0.05222, 0, 0.53222],
      100: [0, 0.69444, 0.10861, 0, 0.59111],
      101: [0, 0.44444, 0.085, 0, 0.53222],
      102: [0.19444, 0.69444, 0.21778, 0, 0.4],
      103: [0.19444, 0.44444, 0.105, 0, 0.53222],
      104: [0, 0.69444, 0.09426, 0, 0.59111],
      105: [0, 0.69326, 0.11387, 0, 0.35555],
      106: [0.19444, 0.69326, 0.1672, 0, 0.35555],
      107: [0, 0.69444, 0.11111, 0, 0.53222],
      108: [0, 0.69444, 0.10861, 0, 0.29666],
      109: [0, 0.44444, 0.09426, 0, 0.94444],
      110: [0, 0.44444, 0.09426, 0, 0.64999],
      111: [0, 0.44444, 0.07861, 0, 0.59111],
      112: [0.19444, 0.44444, 0.07861, 0, 0.59111],
      113: [0.19444, 0.44444, 0.105, 0, 0.53222],
      114: [0, 0.44444, 0.11111, 0, 0.50167],
      115: [0, 0.44444, 0.08167, 0, 0.48694],
      116: [0, 0.63492, 0.09639, 0, 0.385],
      117: [0, 0.44444, 0.09426, 0, 0.62055],
      118: [0, 0.44444, 0.11111, 0, 0.53222],
      119: [0, 0.44444, 0.11111, 0, 0.76777],
      120: [0, 0.44444, 0.12583, 0, 0.56055],
      121: [0.19444, 0.44444, 0.105, 0, 0.56166],
      122: [0, 0.44444, 0.13889, 0, 0.49055],
      126: [0.35, 0.34444, 0.11472, 0, 0.59111],
      160: [0, 0, 0, 0, 0.25],
      168: [0, 0.69444, 0.11473, 0, 0.59111],
      176: [0, 0.69444, 0, 0, 0.94888],
      184: [0.17014, 0, 0, 0, 0.53222],
      198: [0, 0.68611, 0.11431, 0, 1.02277],
      216: [0.04861, 0.73472, 0.09062, 0, 0.88555],
      223: [0.19444, 0.69444, 0.09736, 0, 0.665],
      230: [0, 0.44444, 0.085, 0, 0.82666],
      248: [0.09722, 0.54167, 0.09458, 0, 0.59111],
      305: [0, 0.44444, 0.09426, 0, 0.35555],
      338: [0, 0.68611, 0.11431, 0, 1.14054],
      339: [0, 0.44444, 0.085, 0, 0.82666],
      567: [0.19444, 0.44444, 0.04611, 0, 0.385],
      710: [0, 0.69444, 0.06709, 0, 0.59111],
      711: [0, 0.63194, 0.08271, 0, 0.59111],
      713: [0, 0.59444, 0.10444, 0, 0.59111],
      714: [0, 0.69444, 0.08528, 0, 0.59111],
      715: [0, 0.69444, 0, 0, 0.59111],
      728: [0, 0.69444, 0.10333, 0, 0.59111],
      729: [0, 0.69444, 0.12945, 0, 0.35555],
      730: [0, 0.69444, 0, 0, 0.94888],
      732: [0, 0.69444, 0.11472, 0, 0.59111],
      733: [0, 0.69444, 0.11472, 0, 0.59111],
      915: [0, 0.68611, 0.12903, 0, 0.69777],
      916: [0, 0.68611, 0, 0, 0.94444],
      920: [0, 0.68611, 0.09062, 0, 0.88555],
      923: [0, 0.68611, 0, 0, 0.80666],
      926: [0, 0.68611, 0.15092, 0, 0.76777],
      928: [0, 0.68611, 0.17208, 0, 0.8961],
      931: [0, 0.68611, 0.11431, 0, 0.82666],
      933: [0, 0.68611, 0.10778, 0, 0.88555],
      934: [0, 0.68611, 0.05632, 0, 0.82666],
      936: [0, 0.68611, 0.10778, 0, 0.88555],
      937: [0, 0.68611, 0.0992, 0, 0.82666],
      8211: [0, 0.44444, 0.09811, 0, 0.59111],
      8212: [0, 0.44444, 0.09811, 0, 1.18221],
      8216: [0, 0.69444, 0.12945, 0, 0.35555],
      8217: [0, 0.69444, 0.12945, 0, 0.35555],
      8220: [0, 0.69444, 0.16772, 0, 0.62055],
      8221: [0, 0.69444, 0.07939, 0, 0.62055]
    },
    "Main-Italic": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69444, 0.12417, 0, 0.30667],
      34: [0, 0.69444, 0.06961, 0, 0.51444],
      35: [0.19444, 0.69444, 0.06616, 0, 0.81777],
      37: [0.05556, 0.75, 0.13639, 0, 0.81777],
      38: [0, 0.69444, 0.09694, 0, 0.76666],
      39: [0, 0.69444, 0.12417, 0, 0.30667],
      40: [0.25, 0.75, 0.16194, 0, 0.40889],
      41: [0.25, 0.75, 0.03694, 0, 0.40889],
      42: [0, 0.75, 0.14917, 0, 0.51111],
      43: [0.05667, 0.56167, 0.03694, 0, 0.76666],
      44: [0.19444, 0.10556, 0, 0, 0.30667],
      45: [0, 0.43056, 0.02826, 0, 0.35778],
      46: [0, 0.10556, 0, 0, 0.30667],
      47: [0.25, 0.75, 0.16194, 0, 0.51111],
      48: [0, 0.64444, 0.13556, 0, 0.51111],
      49: [0, 0.64444, 0.13556, 0, 0.51111],
      50: [0, 0.64444, 0.13556, 0, 0.51111],
      51: [0, 0.64444, 0.13556, 0, 0.51111],
      52: [0.19444, 0.64444, 0.13556, 0, 0.51111],
      53: [0, 0.64444, 0.13556, 0, 0.51111],
      54: [0, 0.64444, 0.13556, 0, 0.51111],
      55: [0.19444, 0.64444, 0.13556, 0, 0.51111],
      56: [0, 0.64444, 0.13556, 0, 0.51111],
      57: [0, 0.64444, 0.13556, 0, 0.51111],
      58: [0, 0.43056, 0.0582, 0, 0.30667],
      59: [0.19444, 0.43056, 0.0582, 0, 0.30667],
      61: [-0.13313, 0.36687, 0.06616, 0, 0.76666],
      63: [0, 0.69444, 0.1225, 0, 0.51111],
      64: [0, 0.69444, 0.09597, 0, 0.76666],
      65: [0, 0.68333, 0, 0, 0.74333],
      66: [0, 0.68333, 0.10257, 0, 0.70389],
      67: [0, 0.68333, 0.14528, 0, 0.71555],
      68: [0, 0.68333, 0.09403, 0, 0.755],
      69: [0, 0.68333, 0.12028, 0, 0.67833],
      70: [0, 0.68333, 0.13305, 0, 0.65277],
      71: [0, 0.68333, 0.08722, 0, 0.77361],
      72: [0, 0.68333, 0.16389, 0, 0.74333],
      73: [0, 0.68333, 0.15806, 0, 0.38555],
      74: [0, 0.68333, 0.14028, 0, 0.525],
      75: [0, 0.68333, 0.14528, 0, 0.76888],
      76: [0, 0.68333, 0, 0, 0.62722],
      77: [0, 0.68333, 0.16389, 0, 0.89666],
      78: [0, 0.68333, 0.16389, 0, 0.74333],
      79: [0, 0.68333, 0.09403, 0, 0.76666],
      80: [0, 0.68333, 0.10257, 0, 0.67833],
      81: [0.19444, 0.68333, 0.09403, 0, 0.76666],
      82: [0, 0.68333, 0.03868, 0, 0.72944],
      83: [0, 0.68333, 0.11972, 0, 0.56222],
      84: [0, 0.68333, 0.13305, 0, 0.71555],
      85: [0, 0.68333, 0.16389, 0, 0.74333],
      86: [0, 0.68333, 0.18361, 0, 0.74333],
      87: [0, 0.68333, 0.18361, 0, 0.99888],
      88: [0, 0.68333, 0.15806, 0, 0.74333],
      89: [0, 0.68333, 0.19383, 0, 0.74333],
      90: [0, 0.68333, 0.14528, 0, 0.61333],
      91: [0.25, 0.75, 0.1875, 0, 0.30667],
      93: [0.25, 0.75, 0.10528, 0, 0.30667],
      94: [0, 0.69444, 0.06646, 0, 0.51111],
      95: [0.31, 0.12056, 0.09208, 0, 0.51111],
      97: [0, 0.43056, 0.07671, 0, 0.51111],
      98: [0, 0.69444, 0.06312, 0, 0.46],
      99: [0, 0.43056, 0.05653, 0, 0.46],
      100: [0, 0.69444, 0.10333, 0, 0.51111],
      101: [0, 0.43056, 0.07514, 0, 0.46],
      102: [0.19444, 0.69444, 0.21194, 0, 0.30667],
      103: [0.19444, 0.43056, 0.08847, 0, 0.46],
      104: [0, 0.69444, 0.07671, 0, 0.51111],
      105: [0, 0.65536, 0.1019, 0, 0.30667],
      106: [0.19444, 0.65536, 0.14467, 0, 0.30667],
      107: [0, 0.69444, 0.10764, 0, 0.46],
      108: [0, 0.69444, 0.10333, 0, 0.25555],
      109: [0, 0.43056, 0.07671, 0, 0.81777],
      110: [0, 0.43056, 0.07671, 0, 0.56222],
      111: [0, 0.43056, 0.06312, 0, 0.51111],
      112: [0.19444, 0.43056, 0.06312, 0, 0.51111],
      113: [0.19444, 0.43056, 0.08847, 0, 0.46],
      114: [0, 0.43056, 0.10764, 0, 0.42166],
      115: [0, 0.43056, 0.08208, 0, 0.40889],
      116: [0, 0.61508, 0.09486, 0, 0.33222],
      117: [0, 0.43056, 0.07671, 0, 0.53666],
      118: [0, 0.43056, 0.10764, 0, 0.46],
      119: [0, 0.43056, 0.10764, 0, 0.66444],
      120: [0, 0.43056, 0.12042, 0, 0.46389],
      121: [0.19444, 0.43056, 0.08847, 0, 0.48555],
      122: [0, 0.43056, 0.12292, 0, 0.40889],
      126: [0.35, 0.31786, 0.11585, 0, 0.51111],
      160: [0, 0, 0, 0, 0.25],
      168: [0, 0.66786, 0.10474, 0, 0.51111],
      176: [0, 0.69444, 0, 0, 0.83129],
      184: [0.17014, 0, 0, 0, 0.46],
      198: [0, 0.68333, 0.12028, 0, 0.88277],
      216: [0.04861, 0.73194, 0.09403, 0, 0.76666],
      223: [0.19444, 0.69444, 0.10514, 0, 0.53666],
      230: [0, 0.43056, 0.07514, 0, 0.71555],
      248: [0.09722, 0.52778, 0.09194, 0, 0.51111],
      338: [0, 0.68333, 0.12028, 0, 0.98499],
      339: [0, 0.43056, 0.07514, 0, 0.71555],
      710: [0, 0.69444, 0.06646, 0, 0.51111],
      711: [0, 0.62847, 0.08295, 0, 0.51111],
      713: [0, 0.56167, 0.10333, 0, 0.51111],
      714: [0, 0.69444, 0.09694, 0, 0.51111],
      715: [0, 0.69444, 0, 0, 0.51111],
      728: [0, 0.69444, 0.10806, 0, 0.51111],
      729: [0, 0.66786, 0.11752, 0, 0.30667],
      730: [0, 0.69444, 0, 0, 0.83129],
      732: [0, 0.66786, 0.11585, 0, 0.51111],
      733: [0, 0.69444, 0.1225, 0, 0.51111],
      915: [0, 0.68333, 0.13305, 0, 0.62722],
      916: [0, 0.68333, 0, 0, 0.81777],
      920: [0, 0.68333, 0.09403, 0, 0.76666],
      923: [0, 0.68333, 0, 0, 0.69222],
      926: [0, 0.68333, 0.15294, 0, 0.66444],
      928: [0, 0.68333, 0.16389, 0, 0.74333],
      931: [0, 0.68333, 0.12028, 0, 0.71555],
      933: [0, 0.68333, 0.11111, 0, 0.76666],
      934: [0, 0.68333, 0.05986, 0, 0.71555],
      936: [0, 0.68333, 0.11111, 0, 0.76666],
      937: [0, 0.68333, 0.10257, 0, 0.71555],
      8211: [0, 0.43056, 0.09208, 0, 0.51111],
      8212: [0, 0.43056, 0.09208, 0, 1.02222],
      8216: [0, 0.69444, 0.12417, 0, 0.30667],
      8217: [0, 0.69444, 0.12417, 0, 0.30667],
      8220: [0, 0.69444, 0.1685, 0, 0.51444],
      8221: [0, 0.69444, 0.06961, 0, 0.51444],
      8463: [0, 0.68889, 0, 0, 0.54028]
    },
    "Main-Regular": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69444, 0, 0, 0.27778],
      34: [0, 0.69444, 0, 0, 0.5],
      35: [0.19444, 0.69444, 0, 0, 0.83334],
      36: [0.05556, 0.75, 0, 0, 0.5],
      37: [0.05556, 0.75, 0, 0, 0.83334],
      38: [0, 0.69444, 0, 0, 0.77778],
      39: [0, 0.69444, 0, 0, 0.27778],
      40: [0.25, 0.75, 0, 0, 0.38889],
      41: [0.25, 0.75, 0, 0, 0.38889],
      42: [0, 0.75, 0, 0, 0.5],
      43: [0.08333, 0.58333, 0, 0, 0.77778],
      44: [0.19444, 0.10556, 0, 0, 0.27778],
      45: [0, 0.43056, 0, 0, 0.33333],
      46: [0, 0.10556, 0, 0, 0.27778],
      47: [0.25, 0.75, 0, 0, 0.5],
      48: [0, 0.64444, 0, 0, 0.5],
      49: [0, 0.64444, 0, 0, 0.5],
      50: [0, 0.64444, 0, 0, 0.5],
      51: [0, 0.64444, 0, 0, 0.5],
      52: [0, 0.64444, 0, 0, 0.5],
      53: [0, 0.64444, 0, 0, 0.5],
      54: [0, 0.64444, 0, 0, 0.5],
      55: [0, 0.64444, 0, 0, 0.5],
      56: [0, 0.64444, 0, 0, 0.5],
      57: [0, 0.64444, 0, 0, 0.5],
      58: [0, 0.43056, 0, 0, 0.27778],
      59: [0.19444, 0.43056, 0, 0, 0.27778],
      60: [0.0391, 0.5391, 0, 0, 0.77778],
      61: [-0.13313, 0.36687, 0, 0, 0.77778],
      62: [0.0391, 0.5391, 0, 0, 0.77778],
      63: [0, 0.69444, 0, 0, 0.47222],
      64: [0, 0.69444, 0, 0, 0.77778],
      65: [0, 0.68333, 0, 0, 0.75],
      66: [0, 0.68333, 0, 0, 0.70834],
      67: [0, 0.68333, 0, 0, 0.72222],
      68: [0, 0.68333, 0, 0, 0.76389],
      69: [0, 0.68333, 0, 0, 0.68056],
      70: [0, 0.68333, 0, 0, 0.65278],
      71: [0, 0.68333, 0, 0, 0.78472],
      72: [0, 0.68333, 0, 0, 0.75],
      73: [0, 0.68333, 0, 0, 0.36111],
      74: [0, 0.68333, 0, 0, 0.51389],
      75: [0, 0.68333, 0, 0, 0.77778],
      76: [0, 0.68333, 0, 0, 0.625],
      77: [0, 0.68333, 0, 0, 0.91667],
      78: [0, 0.68333, 0, 0, 0.75],
      79: [0, 0.68333, 0, 0, 0.77778],
      80: [0, 0.68333, 0, 0, 0.68056],
      81: [0.19444, 0.68333, 0, 0, 0.77778],
      82: [0, 0.68333, 0, 0, 0.73611],
      83: [0, 0.68333, 0, 0, 0.55556],
      84: [0, 0.68333, 0, 0, 0.72222],
      85: [0, 0.68333, 0, 0, 0.75],
      86: [0, 0.68333, 0.01389, 0, 0.75],
      87: [0, 0.68333, 0.01389, 0, 1.02778],
      88: [0, 0.68333, 0, 0, 0.75],
      89: [0, 0.68333, 0.025, 0, 0.75],
      90: [0, 0.68333, 0, 0, 0.61111],
      91: [0.25, 0.75, 0, 0, 0.27778],
      92: [0.25, 0.75, 0, 0, 0.5],
      93: [0.25, 0.75, 0, 0, 0.27778],
      94: [0, 0.69444, 0, 0, 0.5],
      95: [0.31, 0.12056, 0.02778, 0, 0.5],
      97: [0, 0.43056, 0, 0, 0.5],
      98: [0, 0.69444, 0, 0, 0.55556],
      99: [0, 0.43056, 0, 0, 0.44445],
      100: [0, 0.69444, 0, 0, 0.55556],
      101: [0, 0.43056, 0, 0, 0.44445],
      102: [0, 0.69444, 0.07778, 0, 0.30556],
      103: [0.19444, 0.43056, 0.01389, 0, 0.5],
      104: [0, 0.69444, 0, 0, 0.55556],
      105: [0, 0.66786, 0, 0, 0.27778],
      106: [0.19444, 0.66786, 0, 0, 0.30556],
      107: [0, 0.69444, 0, 0, 0.52778],
      108: [0, 0.69444, 0, 0, 0.27778],
      109: [0, 0.43056, 0, 0, 0.83334],
      110: [0, 0.43056, 0, 0, 0.55556],
      111: [0, 0.43056, 0, 0, 0.5],
      112: [0.19444, 0.43056, 0, 0, 0.55556],
      113: [0.19444, 0.43056, 0, 0, 0.52778],
      114: [0, 0.43056, 0, 0, 0.39167],
      115: [0, 0.43056, 0, 0, 0.39445],
      116: [0, 0.61508, 0, 0, 0.38889],
      117: [0, 0.43056, 0, 0, 0.55556],
      118: [0, 0.43056, 0.01389, 0, 0.52778],
      119: [0, 0.43056, 0.01389, 0, 0.72222],
      120: [0, 0.43056, 0, 0, 0.52778],
      121: [0.19444, 0.43056, 0.01389, 0, 0.52778],
      122: [0, 0.43056, 0, 0, 0.44445],
      123: [0.25, 0.75, 0, 0, 0.5],
      124: [0.25, 0.75, 0, 0, 0.27778],
      125: [0.25, 0.75, 0, 0, 0.5],
      126: [0.35, 0.31786, 0, 0, 0.5],
      160: [0, 0, 0, 0, 0.25],
      163: [0, 0.69444, 0, 0, 0.76909],
      167: [0.19444, 0.69444, 0, 0, 0.44445],
      168: [0, 0.66786, 0, 0, 0.5],
      172: [0, 0.43056, 0, 0, 0.66667],
      176: [0, 0.69444, 0, 0, 0.75],
      177: [0.08333, 0.58333, 0, 0, 0.77778],
      182: [0.19444, 0.69444, 0, 0, 0.61111],
      184: [0.17014, 0, 0, 0, 0.44445],
      198: [0, 0.68333, 0, 0, 0.90278],
      215: [0.08333, 0.58333, 0, 0, 0.77778],
      216: [0.04861, 0.73194, 0, 0, 0.77778],
      223: [0, 0.69444, 0, 0, 0.5],
      230: [0, 0.43056, 0, 0, 0.72222],
      247: [0.08333, 0.58333, 0, 0, 0.77778],
      248: [0.09722, 0.52778, 0, 0, 0.5],
      305: [0, 0.43056, 0, 0, 0.27778],
      338: [0, 0.68333, 0, 0, 1.01389],
      339: [0, 0.43056, 0, 0, 0.77778],
      567: [0.19444, 0.43056, 0, 0, 0.30556],
      710: [0, 0.69444, 0, 0, 0.5],
      711: [0, 0.62847, 0, 0, 0.5],
      713: [0, 0.56778, 0, 0, 0.5],
      714: [0, 0.69444, 0, 0, 0.5],
      715: [0, 0.69444, 0, 0, 0.5],
      728: [0, 0.69444, 0, 0, 0.5],
      729: [0, 0.66786, 0, 0, 0.27778],
      730: [0, 0.69444, 0, 0, 0.75],
      732: [0, 0.66786, 0, 0, 0.5],
      733: [0, 0.69444, 0, 0, 0.5],
      915: [0, 0.68333, 0, 0, 0.625],
      916: [0, 0.68333, 0, 0, 0.83334],
      920: [0, 0.68333, 0, 0, 0.77778],
      923: [0, 0.68333, 0, 0, 0.69445],
      926: [0, 0.68333, 0, 0, 0.66667],
      928: [0, 0.68333, 0, 0, 0.75],
      931: [0, 0.68333, 0, 0, 0.72222],
      933: [0, 0.68333, 0, 0, 0.77778],
      934: [0, 0.68333, 0, 0, 0.72222],
      936: [0, 0.68333, 0, 0, 0.77778],
      937: [0, 0.68333, 0, 0, 0.72222],
      8211: [0, 0.43056, 0.02778, 0, 0.5],
      8212: [0, 0.43056, 0.02778, 0, 1],
      8216: [0, 0.69444, 0, 0, 0.27778],
      8217: [0, 0.69444, 0, 0, 0.27778],
      8220: [0, 0.69444, 0, 0, 0.5],
      8221: [0, 0.69444, 0, 0, 0.5],
      8224: [0.19444, 0.69444, 0, 0, 0.44445],
      8225: [0.19444, 0.69444, 0, 0, 0.44445],
      8230: [0, 0.123, 0, 0, 1.172],
      8242: [0, 0.55556, 0, 0, 0.275],
      8407: [0, 0.71444, 0.15382, 0, 0.5],
      8463: [0, 0.68889, 0, 0, 0.54028],
      8465: [0, 0.69444, 0, 0, 0.72222],
      8467: [0, 0.69444, 0, 0.11111, 0.41667],
      8472: [0.19444, 0.43056, 0, 0.11111, 0.63646],
      8476: [0, 0.69444, 0, 0, 0.72222],
      8501: [0, 0.69444, 0, 0, 0.61111],
      8592: [-0.13313, 0.36687, 0, 0, 1],
      8593: [0.19444, 0.69444, 0, 0, 0.5],
      8594: [-0.13313, 0.36687, 0, 0, 1],
      8595: [0.19444, 0.69444, 0, 0, 0.5],
      8596: [-0.13313, 0.36687, 0, 0, 1],
      8597: [0.25, 0.75, 0, 0, 0.5],
      8598: [0.19444, 0.69444, 0, 0, 1],
      8599: [0.19444, 0.69444, 0, 0, 1],
      8600: [0.19444, 0.69444, 0, 0, 1],
      8601: [0.19444, 0.69444, 0, 0, 1],
      8614: [0.011, 0.511, 0, 0, 1],
      8617: [0.011, 0.511, 0, 0, 1.126],
      8618: [0.011, 0.511, 0, 0, 1.126],
      8636: [-0.13313, 0.36687, 0, 0, 1],
      8637: [-0.13313, 0.36687, 0, 0, 1],
      8640: [-0.13313, 0.36687, 0, 0, 1],
      8641: [-0.13313, 0.36687, 0, 0, 1],
      8652: [0.011, 0.671, 0, 0, 1],
      8656: [-0.13313, 0.36687, 0, 0, 1],
      8657: [0.19444, 0.69444, 0, 0, 0.61111],
      8658: [-0.13313, 0.36687, 0, 0, 1],
      8659: [0.19444, 0.69444, 0, 0, 0.61111],
      8660: [-0.13313, 0.36687, 0, 0, 1],
      8661: [0.25, 0.75, 0, 0, 0.61111],
      8704: [0, 0.69444, 0, 0, 0.55556],
      8706: [0, 0.69444, 0.05556, 0.08334, 0.5309],
      8707: [0, 0.69444, 0, 0, 0.55556],
      8709: [0.05556, 0.75, 0, 0, 0.5],
      8711: [0, 0.68333, 0, 0, 0.83334],
      8712: [0.0391, 0.5391, 0, 0, 0.66667],
      8715: [0.0391, 0.5391, 0, 0, 0.66667],
      8722: [0.08333, 0.58333, 0, 0, 0.77778],
      8723: [0.08333, 0.58333, 0, 0, 0.77778],
      8725: [0.25, 0.75, 0, 0, 0.5],
      8726: [0.25, 0.75, 0, 0, 0.5],
      8727: [-0.03472, 0.46528, 0, 0, 0.5],
      8728: [-0.05555, 0.44445, 0, 0, 0.5],
      8729: [-0.05555, 0.44445, 0, 0, 0.5],
      8730: [0.2, 0.8, 0, 0, 0.83334],
      8733: [0, 0.43056, 0, 0, 0.77778],
      8734: [0, 0.43056, 0, 0, 1],
      8736: [0, 0.69224, 0, 0, 0.72222],
      8739: [0.25, 0.75, 0, 0, 0.27778],
      8741: [0.25, 0.75, 0, 0, 0.5],
      8743: [0, 0.55556, 0, 0, 0.66667],
      8744: [0, 0.55556, 0, 0, 0.66667],
      8745: [0, 0.55556, 0, 0, 0.66667],
      8746: [0, 0.55556, 0, 0, 0.66667],
      8747: [0.19444, 0.69444, 0.11111, 0, 0.41667],
      8764: [-0.13313, 0.36687, 0, 0, 0.77778],
      8768: [0.19444, 0.69444, 0, 0, 0.27778],
      8771: [-0.03625, 0.46375, 0, 0, 0.77778],
      8773: [-0.022, 0.589, 0, 0, 0.778],
      8776: [-0.01688, 0.48312, 0, 0, 0.77778],
      8781: [-0.03625, 0.46375, 0, 0, 0.77778],
      8784: [-0.133, 0.673, 0, 0, 0.778],
      8801: [-0.03625, 0.46375, 0, 0, 0.77778],
      8804: [0.13597, 0.63597, 0, 0, 0.77778],
      8805: [0.13597, 0.63597, 0, 0, 0.77778],
      8810: [0.0391, 0.5391, 0, 0, 1],
      8811: [0.0391, 0.5391, 0, 0, 1],
      8826: [0.0391, 0.5391, 0, 0, 0.77778],
      8827: [0.0391, 0.5391, 0, 0, 0.77778],
      8834: [0.0391, 0.5391, 0, 0, 0.77778],
      8835: [0.0391, 0.5391, 0, 0, 0.77778],
      8838: [0.13597, 0.63597, 0, 0, 0.77778],
      8839: [0.13597, 0.63597, 0, 0, 0.77778],
      8846: [0, 0.55556, 0, 0, 0.66667],
      8849: [0.13597, 0.63597, 0, 0, 0.77778],
      8850: [0.13597, 0.63597, 0, 0, 0.77778],
      8851: [0, 0.55556, 0, 0, 0.66667],
      8852: [0, 0.55556, 0, 0, 0.66667],
      8853: [0.08333, 0.58333, 0, 0, 0.77778],
      8854: [0.08333, 0.58333, 0, 0, 0.77778],
      8855: [0.08333, 0.58333, 0, 0, 0.77778],
      8856: [0.08333, 0.58333, 0, 0, 0.77778],
      8857: [0.08333, 0.58333, 0, 0, 0.77778],
      8866: [0, 0.69444, 0, 0, 0.61111],
      8867: [0, 0.69444, 0, 0, 0.61111],
      8868: [0, 0.69444, 0, 0, 0.77778],
      8869: [0, 0.69444, 0, 0, 0.77778],
      8872: [0.249, 0.75, 0, 0, 0.867],
      8900: [-0.05555, 0.44445, 0, 0, 0.5],
      8901: [-0.05555, 0.44445, 0, 0, 0.27778],
      8902: [-0.03472, 0.46528, 0, 0, 0.5],
      8904: [5e-3, 0.505, 0, 0, 0.9],
      8942: [0.03, 0.903, 0, 0, 0.278],
      8943: [-0.19, 0.313, 0, 0, 1.172],
      8945: [-0.1, 0.823, 0, 0, 1.282],
      8968: [0.25, 0.75, 0, 0, 0.44445],
      8969: [0.25, 0.75, 0, 0, 0.44445],
      8970: [0.25, 0.75, 0, 0, 0.44445],
      8971: [0.25, 0.75, 0, 0, 0.44445],
      8994: [-0.14236, 0.35764, 0, 0, 1],
      8995: [-0.14236, 0.35764, 0, 0, 1],
      9136: [0.244, 0.744, 0, 0, 0.412],
      9137: [0.244, 0.745, 0, 0, 0.412],
      9651: [0.19444, 0.69444, 0, 0, 0.88889],
      9657: [-0.03472, 0.46528, 0, 0, 0.5],
      9661: [0.19444, 0.69444, 0, 0, 0.88889],
      9667: [-0.03472, 0.46528, 0, 0, 0.5],
      9711: [0.19444, 0.69444, 0, 0, 1],
      9824: [0.12963, 0.69444, 0, 0, 0.77778],
      9825: [0.12963, 0.69444, 0, 0, 0.77778],
      9826: [0.12963, 0.69444, 0, 0, 0.77778],
      9827: [0.12963, 0.69444, 0, 0, 0.77778],
      9837: [0, 0.75, 0, 0, 0.38889],
      9838: [0.19444, 0.69444, 0, 0, 0.38889],
      9839: [0.19444, 0.69444, 0, 0, 0.38889],
      10216: [0.25, 0.75, 0, 0, 0.38889],
      10217: [0.25, 0.75, 0, 0, 0.38889],
      10222: [0.244, 0.744, 0, 0, 0.412],
      10223: [0.244, 0.745, 0, 0, 0.412],
      10229: [0.011, 0.511, 0, 0, 1.609],
      10230: [0.011, 0.511, 0, 0, 1.638],
      10231: [0.011, 0.511, 0, 0, 1.859],
      10232: [0.024, 0.525, 0, 0, 1.609],
      10233: [0.024, 0.525, 0, 0, 1.638],
      10234: [0.024, 0.525, 0, 0, 1.858],
      10236: [0.011, 0.511, 0, 0, 1.638],
      10815: [0, 0.68333, 0, 0, 0.75],
      10927: [0.13597, 0.63597, 0, 0, 0.77778],
      10928: [0.13597, 0.63597, 0, 0, 0.77778],
      57376: [0.19444, 0.69444, 0, 0, 0]
    },
    "Math-BoldItalic": {
      32: [0, 0, 0, 0, 0.25],
      48: [0, 0.44444, 0, 0, 0.575],
      49: [0, 0.44444, 0, 0, 0.575],
      50: [0, 0.44444, 0, 0, 0.575],
      51: [0.19444, 0.44444, 0, 0, 0.575],
      52: [0.19444, 0.44444, 0, 0, 0.575],
      53: [0.19444, 0.44444, 0, 0, 0.575],
      54: [0, 0.64444, 0, 0, 0.575],
      55: [0.19444, 0.44444, 0, 0, 0.575],
      56: [0, 0.64444, 0, 0, 0.575],
      57: [0.19444, 0.44444, 0, 0, 0.575],
      65: [0, 0.68611, 0, 0, 0.86944],
      66: [0, 0.68611, 0.04835, 0, 0.8664],
      67: [0, 0.68611, 0.06979, 0, 0.81694],
      68: [0, 0.68611, 0.03194, 0, 0.93812],
      69: [0, 0.68611, 0.05451, 0, 0.81007],
      70: [0, 0.68611, 0.15972, 0, 0.68889],
      71: [0, 0.68611, 0, 0, 0.88673],
      72: [0, 0.68611, 0.08229, 0, 0.98229],
      73: [0, 0.68611, 0.07778, 0, 0.51111],
      74: [0, 0.68611, 0.10069, 0, 0.63125],
      75: [0, 0.68611, 0.06979, 0, 0.97118],
      76: [0, 0.68611, 0, 0, 0.75555],
      77: [0, 0.68611, 0.11424, 0, 1.14201],
      78: [0, 0.68611, 0.11424, 0, 0.95034],
      79: [0, 0.68611, 0.03194, 0, 0.83666],
      80: [0, 0.68611, 0.15972, 0, 0.72309],
      81: [0.19444, 0.68611, 0, 0, 0.86861],
      82: [0, 0.68611, 421e-5, 0, 0.87235],
      83: [0, 0.68611, 0.05382, 0, 0.69271],
      84: [0, 0.68611, 0.15972, 0, 0.63663],
      85: [0, 0.68611, 0.11424, 0, 0.80027],
      86: [0, 0.68611, 0.25555, 0, 0.67778],
      87: [0, 0.68611, 0.15972, 0, 1.09305],
      88: [0, 0.68611, 0.07778, 0, 0.94722],
      89: [0, 0.68611, 0.25555, 0, 0.67458],
      90: [0, 0.68611, 0.06979, 0, 0.77257],
      97: [0, 0.44444, 0, 0, 0.63287],
      98: [0, 0.69444, 0, 0, 0.52083],
      99: [0, 0.44444, 0, 0, 0.51342],
      100: [0, 0.69444, 0, 0, 0.60972],
      101: [0, 0.44444, 0, 0, 0.55361],
      102: [0.19444, 0.69444, 0.11042, 0, 0.56806],
      103: [0.19444, 0.44444, 0.03704, 0, 0.5449],
      104: [0, 0.69444, 0, 0, 0.66759],
      105: [0, 0.69326, 0, 0, 0.4048],
      106: [0.19444, 0.69326, 0.0622, 0, 0.47083],
      107: [0, 0.69444, 0.01852, 0, 0.6037],
      108: [0, 0.69444, 88e-4, 0, 0.34815],
      109: [0, 0.44444, 0, 0, 1.0324],
      110: [0, 0.44444, 0, 0, 0.71296],
      111: [0, 0.44444, 0, 0, 0.58472],
      112: [0.19444, 0.44444, 0, 0, 0.60092],
      113: [0.19444, 0.44444, 0.03704, 0, 0.54213],
      114: [0, 0.44444, 0.03194, 0, 0.5287],
      115: [0, 0.44444, 0, 0, 0.53125],
      116: [0, 0.63492, 0, 0, 0.41528],
      117: [0, 0.44444, 0, 0, 0.68102],
      118: [0, 0.44444, 0.03704, 0, 0.56666],
      119: [0, 0.44444, 0.02778, 0, 0.83148],
      120: [0, 0.44444, 0, 0, 0.65903],
      121: [0.19444, 0.44444, 0.03704, 0, 0.59028],
      122: [0, 0.44444, 0.04213, 0, 0.55509],
      160: [0, 0, 0, 0, 0.25],
      915: [0, 0.68611, 0.15972, 0, 0.65694],
      916: [0, 0.68611, 0, 0, 0.95833],
      920: [0, 0.68611, 0.03194, 0, 0.86722],
      923: [0, 0.68611, 0, 0, 0.80555],
      926: [0, 0.68611, 0.07458, 0, 0.84125],
      928: [0, 0.68611, 0.08229, 0, 0.98229],
      931: [0, 0.68611, 0.05451, 0, 0.88507],
      933: [0, 0.68611, 0.15972, 0, 0.67083],
      934: [0, 0.68611, 0, 0, 0.76666],
      936: [0, 0.68611, 0.11653, 0, 0.71402],
      937: [0, 0.68611, 0.04835, 0, 0.8789],
      945: [0, 0.44444, 0, 0, 0.76064],
      946: [0.19444, 0.69444, 0.03403, 0, 0.65972],
      947: [0.19444, 0.44444, 0.06389, 0, 0.59003],
      948: [0, 0.69444, 0.03819, 0, 0.52222],
      949: [0, 0.44444, 0, 0, 0.52882],
      950: [0.19444, 0.69444, 0.06215, 0, 0.50833],
      951: [0.19444, 0.44444, 0.03704, 0, 0.6],
      952: [0, 0.69444, 0.03194, 0, 0.5618],
      953: [0, 0.44444, 0, 0, 0.41204],
      954: [0, 0.44444, 0, 0, 0.66759],
      955: [0, 0.69444, 0, 0, 0.67083],
      956: [0.19444, 0.44444, 0, 0, 0.70787],
      957: [0, 0.44444, 0.06898, 0, 0.57685],
      958: [0.19444, 0.69444, 0.03021, 0, 0.50833],
      959: [0, 0.44444, 0, 0, 0.58472],
      960: [0, 0.44444, 0.03704, 0, 0.68241],
      961: [0.19444, 0.44444, 0, 0, 0.6118],
      962: [0.09722, 0.44444, 0.07917, 0, 0.42361],
      963: [0, 0.44444, 0.03704, 0, 0.68588],
      964: [0, 0.44444, 0.13472, 0, 0.52083],
      965: [0, 0.44444, 0.03704, 0, 0.63055],
      966: [0.19444, 0.44444, 0, 0, 0.74722],
      967: [0.19444, 0.44444, 0, 0, 0.71805],
      968: [0.19444, 0.69444, 0.03704, 0, 0.75833],
      969: [0, 0.44444, 0.03704, 0, 0.71782],
      977: [0, 0.69444, 0, 0, 0.69155],
      981: [0.19444, 0.69444, 0, 0, 0.7125],
      982: [0, 0.44444, 0.03194, 0, 0.975],
      1009: [0.19444, 0.44444, 0, 0, 0.6118],
      1013: [0, 0.44444, 0, 0, 0.48333],
      57649: [0, 0.44444, 0, 0, 0.39352],
      57911: [0.19444, 0.44444, 0, 0, 0.43889]
    },
    "Math-Italic": {
      32: [0, 0, 0, 0, 0.25],
      48: [0, 0.43056, 0, 0, 0.5],
      49: [0, 0.43056, 0, 0, 0.5],
      50: [0, 0.43056, 0, 0, 0.5],
      51: [0.19444, 0.43056, 0, 0, 0.5],
      52: [0.19444, 0.43056, 0, 0, 0.5],
      53: [0.19444, 0.43056, 0, 0, 0.5],
      54: [0, 0.64444, 0, 0, 0.5],
      55: [0.19444, 0.43056, 0, 0, 0.5],
      56: [0, 0.64444, 0, 0, 0.5],
      57: [0.19444, 0.43056, 0, 0, 0.5],
      65: [0, 0.68333, 0, 0.13889, 0.75],
      66: [0, 0.68333, 0.05017, 0.08334, 0.75851],
      67: [0, 0.68333, 0.07153, 0.08334, 0.71472],
      68: [0, 0.68333, 0.02778, 0.05556, 0.82792],
      69: [0, 0.68333, 0.05764, 0.08334, 0.7382],
      70: [0, 0.68333, 0.13889, 0.08334, 0.64306],
      71: [0, 0.68333, 0, 0.08334, 0.78625],
      72: [0, 0.68333, 0.08125, 0.05556, 0.83125],
      73: [0, 0.68333, 0.07847, 0.11111, 0.43958],
      74: [0, 0.68333, 0.09618, 0.16667, 0.55451],
      75: [0, 0.68333, 0.07153, 0.05556, 0.84931],
      76: [0, 0.68333, 0, 0.02778, 0.68056],
      77: [0, 0.68333, 0.10903, 0.08334, 0.97014],
      78: [0, 0.68333, 0.10903, 0.08334, 0.80347],
      79: [0, 0.68333, 0.02778, 0.08334, 0.76278],
      80: [0, 0.68333, 0.13889, 0.08334, 0.64201],
      81: [0.19444, 0.68333, 0, 0.08334, 0.79056],
      82: [0, 0.68333, 773e-5, 0.08334, 0.75929],
      83: [0, 0.68333, 0.05764, 0.08334, 0.6132],
      84: [0, 0.68333, 0.13889, 0.08334, 0.58438],
      85: [0, 0.68333, 0.10903, 0.02778, 0.68278],
      86: [0, 0.68333, 0.22222, 0, 0.58333],
      87: [0, 0.68333, 0.13889, 0, 0.94445],
      88: [0, 0.68333, 0.07847, 0.08334, 0.82847],
      89: [0, 0.68333, 0.22222, 0, 0.58056],
      90: [0, 0.68333, 0.07153, 0.08334, 0.68264],
      97: [0, 0.43056, 0, 0, 0.52859],
      98: [0, 0.69444, 0, 0, 0.42917],
      99: [0, 0.43056, 0, 0.05556, 0.43276],
      100: [0, 0.69444, 0, 0.16667, 0.52049],
      101: [0, 0.43056, 0, 0.05556, 0.46563],
      102: [0.19444, 0.69444, 0.10764, 0.16667, 0.48959],
      103: [0.19444, 0.43056, 0.03588, 0.02778, 0.47697],
      104: [0, 0.69444, 0, 0, 0.57616],
      105: [0, 0.65952, 0, 0, 0.34451],
      106: [0.19444, 0.65952, 0.05724, 0, 0.41181],
      107: [0, 0.69444, 0.03148, 0, 0.5206],
      108: [0, 0.69444, 0.01968, 0.08334, 0.29838],
      109: [0, 0.43056, 0, 0, 0.87801],
      110: [0, 0.43056, 0, 0, 0.60023],
      111: [0, 0.43056, 0, 0.05556, 0.48472],
      112: [0.19444, 0.43056, 0, 0.08334, 0.50313],
      113: [0.19444, 0.43056, 0.03588, 0.08334, 0.44641],
      114: [0, 0.43056, 0.02778, 0.05556, 0.45116],
      115: [0, 0.43056, 0, 0.05556, 0.46875],
      116: [0, 0.61508, 0, 0.08334, 0.36111],
      117: [0, 0.43056, 0, 0.02778, 0.57246],
      118: [0, 0.43056, 0.03588, 0.02778, 0.48472],
      119: [0, 0.43056, 0.02691, 0.08334, 0.71592],
      120: [0, 0.43056, 0, 0.02778, 0.57153],
      121: [0.19444, 0.43056, 0.03588, 0.05556, 0.49028],
      122: [0, 0.43056, 0.04398, 0.05556, 0.46505],
      160: [0, 0, 0, 0, 0.25],
      915: [0, 0.68333, 0.13889, 0.08334, 0.61528],
      916: [0, 0.68333, 0, 0.16667, 0.83334],
      920: [0, 0.68333, 0.02778, 0.08334, 0.76278],
      923: [0, 0.68333, 0, 0.16667, 0.69445],
      926: [0, 0.68333, 0.07569, 0.08334, 0.74236],
      928: [0, 0.68333, 0.08125, 0.05556, 0.83125],
      931: [0, 0.68333, 0.05764, 0.08334, 0.77986],
      933: [0, 0.68333, 0.13889, 0.05556, 0.58333],
      934: [0, 0.68333, 0, 0.08334, 0.66667],
      936: [0, 0.68333, 0.11, 0.05556, 0.61222],
      937: [0, 0.68333, 0.05017, 0.08334, 0.7724],
      945: [0, 0.43056, 37e-4, 0.02778, 0.6397],
      946: [0.19444, 0.69444, 0.05278, 0.08334, 0.56563],
      947: [0.19444, 0.43056, 0.05556, 0, 0.51773],
      948: [0, 0.69444, 0.03785, 0.05556, 0.44444],
      949: [0, 0.43056, 0, 0.08334, 0.46632],
      950: [0.19444, 0.69444, 0.07378, 0.08334, 0.4375],
      951: [0.19444, 0.43056, 0.03588, 0.05556, 0.49653],
      952: [0, 0.69444, 0.02778, 0.08334, 0.46944],
      953: [0, 0.43056, 0, 0.05556, 0.35394],
      954: [0, 0.43056, 0, 0, 0.57616],
      955: [0, 0.69444, 0, 0, 0.58334],
      956: [0.19444, 0.43056, 0, 0.02778, 0.60255],
      957: [0, 0.43056, 0.06366, 0.02778, 0.49398],
      958: [0.19444, 0.69444, 0.04601, 0.11111, 0.4375],
      959: [0, 0.43056, 0, 0.05556, 0.48472],
      960: [0, 0.43056, 0.03588, 0, 0.57003],
      961: [0.19444, 0.43056, 0, 0.08334, 0.51702],
      962: [0.09722, 0.43056, 0.07986, 0.08334, 0.36285],
      963: [0, 0.43056, 0.03588, 0, 0.57141],
      964: [0, 0.43056, 0.1132, 0.02778, 0.43715],
      965: [0, 0.43056, 0.03588, 0.02778, 0.54028],
      966: [0.19444, 0.43056, 0, 0.08334, 0.65417],
      967: [0.19444, 0.43056, 0, 0.05556, 0.62569],
      968: [0.19444, 0.69444, 0.03588, 0.11111, 0.65139],
      969: [0, 0.43056, 0.03588, 0, 0.62245],
      977: [0, 0.69444, 0, 0.08334, 0.59144],
      981: [0.19444, 0.69444, 0, 0.08334, 0.59583],
      982: [0, 0.43056, 0.02778, 0, 0.82813],
      1009: [0.19444, 0.43056, 0, 0.08334, 0.51702],
      1013: [0, 0.43056, 0, 0.05556, 0.4059],
      57649: [0, 0.43056, 0, 0.02778, 0.32246],
      57911: [0.19444, 0.43056, 0, 0.08334, 0.38403]
    },
    "SansSerif-Bold": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69444, 0, 0, 0.36667],
      34: [0, 0.69444, 0, 0, 0.55834],
      35: [0.19444, 0.69444, 0, 0, 0.91667],
      36: [0.05556, 0.75, 0, 0, 0.55],
      37: [0.05556, 0.75, 0, 0, 1.02912],
      38: [0, 0.69444, 0, 0, 0.83056],
      39: [0, 0.69444, 0, 0, 0.30556],
      40: [0.25, 0.75, 0, 0, 0.42778],
      41: [0.25, 0.75, 0, 0, 0.42778],
      42: [0, 0.75, 0, 0, 0.55],
      43: [0.11667, 0.61667, 0, 0, 0.85556],
      44: [0.10556, 0.13056, 0, 0, 0.30556],
      45: [0, 0.45833, 0, 0, 0.36667],
      46: [0, 0.13056, 0, 0, 0.30556],
      47: [0.25, 0.75, 0, 0, 0.55],
      48: [0, 0.69444, 0, 0, 0.55],
      49: [0, 0.69444, 0, 0, 0.55],
      50: [0, 0.69444, 0, 0, 0.55],
      51: [0, 0.69444, 0, 0, 0.55],
      52: [0, 0.69444, 0, 0, 0.55],
      53: [0, 0.69444, 0, 0, 0.55],
      54: [0, 0.69444, 0, 0, 0.55],
      55: [0, 0.69444, 0, 0, 0.55],
      56: [0, 0.69444, 0, 0, 0.55],
      57: [0, 0.69444, 0, 0, 0.55],
      58: [0, 0.45833, 0, 0, 0.30556],
      59: [0.10556, 0.45833, 0, 0, 0.30556],
      61: [-0.09375, 0.40625, 0, 0, 0.85556],
      63: [0, 0.69444, 0, 0, 0.51945],
      64: [0, 0.69444, 0, 0, 0.73334],
      65: [0, 0.69444, 0, 0, 0.73334],
      66: [0, 0.69444, 0, 0, 0.73334],
      67: [0, 0.69444, 0, 0, 0.70278],
      68: [0, 0.69444, 0, 0, 0.79445],
      69: [0, 0.69444, 0, 0, 0.64167],
      70: [0, 0.69444, 0, 0, 0.61111],
      71: [0, 0.69444, 0, 0, 0.73334],
      72: [0, 0.69444, 0, 0, 0.79445],
      73: [0, 0.69444, 0, 0, 0.33056],
      74: [0, 0.69444, 0, 0, 0.51945],
      75: [0, 0.69444, 0, 0, 0.76389],
      76: [0, 0.69444, 0, 0, 0.58056],
      77: [0, 0.69444, 0, 0, 0.97778],
      78: [0, 0.69444, 0, 0, 0.79445],
      79: [0, 0.69444, 0, 0, 0.79445],
      80: [0, 0.69444, 0, 0, 0.70278],
      81: [0.10556, 0.69444, 0, 0, 0.79445],
      82: [0, 0.69444, 0, 0, 0.70278],
      83: [0, 0.69444, 0, 0, 0.61111],
      84: [0, 0.69444, 0, 0, 0.73334],
      85: [0, 0.69444, 0, 0, 0.76389],
      86: [0, 0.69444, 0.01528, 0, 0.73334],
      87: [0, 0.69444, 0.01528, 0, 1.03889],
      88: [0, 0.69444, 0, 0, 0.73334],
      89: [0, 0.69444, 0.0275, 0, 0.73334],
      90: [0, 0.69444, 0, 0, 0.67223],
      91: [0.25, 0.75, 0, 0, 0.34306],
      93: [0.25, 0.75, 0, 0, 0.34306],
      94: [0, 0.69444, 0, 0, 0.55],
      95: [0.35, 0.10833, 0.03056, 0, 0.55],
      97: [0, 0.45833, 0, 0, 0.525],
      98: [0, 0.69444, 0, 0, 0.56111],
      99: [0, 0.45833, 0, 0, 0.48889],
      100: [0, 0.69444, 0, 0, 0.56111],
      101: [0, 0.45833, 0, 0, 0.51111],
      102: [0, 0.69444, 0.07639, 0, 0.33611],
      103: [0.19444, 0.45833, 0.01528, 0, 0.55],
      104: [0, 0.69444, 0, 0, 0.56111],
      105: [0, 0.69444, 0, 0, 0.25556],
      106: [0.19444, 0.69444, 0, 0, 0.28611],
      107: [0, 0.69444, 0, 0, 0.53056],
      108: [0, 0.69444, 0, 0, 0.25556],
      109: [0, 0.45833, 0, 0, 0.86667],
      110: [0, 0.45833, 0, 0, 0.56111],
      111: [0, 0.45833, 0, 0, 0.55],
      112: [0.19444, 0.45833, 0, 0, 0.56111],
      113: [0.19444, 0.45833, 0, 0, 0.56111],
      114: [0, 0.45833, 0.01528, 0, 0.37222],
      115: [0, 0.45833, 0, 0, 0.42167],
      116: [0, 0.58929, 0, 0, 0.40417],
      117: [0, 0.45833, 0, 0, 0.56111],
      118: [0, 0.45833, 0.01528, 0, 0.5],
      119: [0, 0.45833, 0.01528, 0, 0.74445],
      120: [0, 0.45833, 0, 0, 0.5],
      121: [0.19444, 0.45833, 0.01528, 0, 0.5],
      122: [0, 0.45833, 0, 0, 0.47639],
      126: [0.35, 0.34444, 0, 0, 0.55],
      160: [0, 0, 0, 0, 0.25],
      168: [0, 0.69444, 0, 0, 0.55],
      176: [0, 0.69444, 0, 0, 0.73334],
      180: [0, 0.69444, 0, 0, 0.55],
      184: [0.17014, 0, 0, 0, 0.48889],
      305: [0, 0.45833, 0, 0, 0.25556],
      567: [0.19444, 0.45833, 0, 0, 0.28611],
      710: [0, 0.69444, 0, 0, 0.55],
      711: [0, 0.63542, 0, 0, 0.55],
      713: [0, 0.63778, 0, 0, 0.55],
      728: [0, 0.69444, 0, 0, 0.55],
      729: [0, 0.69444, 0, 0, 0.30556],
      730: [0, 0.69444, 0, 0, 0.73334],
      732: [0, 0.69444, 0, 0, 0.55],
      733: [0, 0.69444, 0, 0, 0.55],
      915: [0, 0.69444, 0, 0, 0.58056],
      916: [0, 0.69444, 0, 0, 0.91667],
      920: [0, 0.69444, 0, 0, 0.85556],
      923: [0, 0.69444, 0, 0, 0.67223],
      926: [0, 0.69444, 0, 0, 0.73334],
      928: [0, 0.69444, 0, 0, 0.79445],
      931: [0, 0.69444, 0, 0, 0.79445],
      933: [0, 0.69444, 0, 0, 0.85556],
      934: [0, 0.69444, 0, 0, 0.79445],
      936: [0, 0.69444, 0, 0, 0.85556],
      937: [0, 0.69444, 0, 0, 0.79445],
      8211: [0, 0.45833, 0.03056, 0, 0.55],
      8212: [0, 0.45833, 0.03056, 0, 1.10001],
      8216: [0, 0.69444, 0, 0, 0.30556],
      8217: [0, 0.69444, 0, 0, 0.30556],
      8220: [0, 0.69444, 0, 0, 0.55834],
      8221: [0, 0.69444, 0, 0, 0.55834]
    },
    "SansSerif-Italic": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69444, 0.05733, 0, 0.31945],
      34: [0, 0.69444, 316e-5, 0, 0.5],
      35: [0.19444, 0.69444, 0.05087, 0, 0.83334],
      36: [0.05556, 0.75, 0.11156, 0, 0.5],
      37: [0.05556, 0.75, 0.03126, 0, 0.83334],
      38: [0, 0.69444, 0.03058, 0, 0.75834],
      39: [0, 0.69444, 0.07816, 0, 0.27778],
      40: [0.25, 0.75, 0.13164, 0, 0.38889],
      41: [0.25, 0.75, 0.02536, 0, 0.38889],
      42: [0, 0.75, 0.11775, 0, 0.5],
      43: [0.08333, 0.58333, 0.02536, 0, 0.77778],
      44: [0.125, 0.08333, 0, 0, 0.27778],
      45: [0, 0.44444, 0.01946, 0, 0.33333],
      46: [0, 0.08333, 0, 0, 0.27778],
      47: [0.25, 0.75, 0.13164, 0, 0.5],
      48: [0, 0.65556, 0.11156, 0, 0.5],
      49: [0, 0.65556, 0.11156, 0, 0.5],
      50: [0, 0.65556, 0.11156, 0, 0.5],
      51: [0, 0.65556, 0.11156, 0, 0.5],
      52: [0, 0.65556, 0.11156, 0, 0.5],
      53: [0, 0.65556, 0.11156, 0, 0.5],
      54: [0, 0.65556, 0.11156, 0, 0.5],
      55: [0, 0.65556, 0.11156, 0, 0.5],
      56: [0, 0.65556, 0.11156, 0, 0.5],
      57: [0, 0.65556, 0.11156, 0, 0.5],
      58: [0, 0.44444, 0.02502, 0, 0.27778],
      59: [0.125, 0.44444, 0.02502, 0, 0.27778],
      61: [-0.13, 0.37, 0.05087, 0, 0.77778],
      63: [0, 0.69444, 0.11809, 0, 0.47222],
      64: [0, 0.69444, 0.07555, 0, 0.66667],
      65: [0, 0.69444, 0, 0, 0.66667],
      66: [0, 0.69444, 0.08293, 0, 0.66667],
      67: [0, 0.69444, 0.11983, 0, 0.63889],
      68: [0, 0.69444, 0.07555, 0, 0.72223],
      69: [0, 0.69444, 0.11983, 0, 0.59722],
      70: [0, 0.69444, 0.13372, 0, 0.56945],
      71: [0, 0.69444, 0.11983, 0, 0.66667],
      72: [0, 0.69444, 0.08094, 0, 0.70834],
      73: [0, 0.69444, 0.13372, 0, 0.27778],
      74: [0, 0.69444, 0.08094, 0, 0.47222],
      75: [0, 0.69444, 0.11983, 0, 0.69445],
      76: [0, 0.69444, 0, 0, 0.54167],
      77: [0, 0.69444, 0.08094, 0, 0.875],
      78: [0, 0.69444, 0.08094, 0, 0.70834],
      79: [0, 0.69444, 0.07555, 0, 0.73611],
      80: [0, 0.69444, 0.08293, 0, 0.63889],
      81: [0.125, 0.69444, 0.07555, 0, 0.73611],
      82: [0, 0.69444, 0.08293, 0, 0.64584],
      83: [0, 0.69444, 0.09205, 0, 0.55556],
      84: [0, 0.69444, 0.13372, 0, 0.68056],
      85: [0, 0.69444, 0.08094, 0, 0.6875],
      86: [0, 0.69444, 0.1615, 0, 0.66667],
      87: [0, 0.69444, 0.1615, 0, 0.94445],
      88: [0, 0.69444, 0.13372, 0, 0.66667],
      89: [0, 0.69444, 0.17261, 0, 0.66667],
      90: [0, 0.69444, 0.11983, 0, 0.61111],
      91: [0.25, 0.75, 0.15942, 0, 0.28889],
      93: [0.25, 0.75, 0.08719, 0, 0.28889],
      94: [0, 0.69444, 0.0799, 0, 0.5],
      95: [0.35, 0.09444, 0.08616, 0, 0.5],
      97: [0, 0.44444, 981e-5, 0, 0.48056],
      98: [0, 0.69444, 0.03057, 0, 0.51667],
      99: [0, 0.44444, 0.08336, 0, 0.44445],
      100: [0, 0.69444, 0.09483, 0, 0.51667],
      101: [0, 0.44444, 0.06778, 0, 0.44445],
      102: [0, 0.69444, 0.21705, 0, 0.30556],
      103: [0.19444, 0.44444, 0.10836, 0, 0.5],
      104: [0, 0.69444, 0.01778, 0, 0.51667],
      105: [0, 0.67937, 0.09718, 0, 0.23889],
      106: [0.19444, 0.67937, 0.09162, 0, 0.26667],
      107: [0, 0.69444, 0.08336, 0, 0.48889],
      108: [0, 0.69444, 0.09483, 0, 0.23889],
      109: [0, 0.44444, 0.01778, 0, 0.79445],
      110: [0, 0.44444, 0.01778, 0, 0.51667],
      111: [0, 0.44444, 0.06613, 0, 0.5],
      112: [0.19444, 0.44444, 0.0389, 0, 0.51667],
      113: [0.19444, 0.44444, 0.04169, 0, 0.51667],
      114: [0, 0.44444, 0.10836, 0, 0.34167],
      115: [0, 0.44444, 0.0778, 0, 0.38333],
      116: [0, 0.57143, 0.07225, 0, 0.36111],
      117: [0, 0.44444, 0.04169, 0, 0.51667],
      118: [0, 0.44444, 0.10836, 0, 0.46111],
      119: [0, 0.44444, 0.10836, 0, 0.68334],
      120: [0, 0.44444, 0.09169, 0, 0.46111],
      121: [0.19444, 0.44444, 0.10836, 0, 0.46111],
      122: [0, 0.44444, 0.08752, 0, 0.43472],
      126: [0.35, 0.32659, 0.08826, 0, 0.5],
      160: [0, 0, 0, 0, 0.25],
      168: [0, 0.67937, 0.06385, 0, 0.5],
      176: [0, 0.69444, 0, 0, 0.73752],
      184: [0.17014, 0, 0, 0, 0.44445],
      305: [0, 0.44444, 0.04169, 0, 0.23889],
      567: [0.19444, 0.44444, 0.04169, 0, 0.26667],
      710: [0, 0.69444, 0.0799, 0, 0.5],
      711: [0, 0.63194, 0.08432, 0, 0.5],
      713: [0, 0.60889, 0.08776, 0, 0.5],
      714: [0, 0.69444, 0.09205, 0, 0.5],
      715: [0, 0.69444, 0, 0, 0.5],
      728: [0, 0.69444, 0.09483, 0, 0.5],
      729: [0, 0.67937, 0.07774, 0, 0.27778],
      730: [0, 0.69444, 0, 0, 0.73752],
      732: [0, 0.67659, 0.08826, 0, 0.5],
      733: [0, 0.69444, 0.09205, 0, 0.5],
      915: [0, 0.69444, 0.13372, 0, 0.54167],
      916: [0, 0.69444, 0, 0, 0.83334],
      920: [0, 0.69444, 0.07555, 0, 0.77778],
      923: [0, 0.69444, 0, 0, 0.61111],
      926: [0, 0.69444, 0.12816, 0, 0.66667],
      928: [0, 0.69444, 0.08094, 0, 0.70834],
      931: [0, 0.69444, 0.11983, 0, 0.72222],
      933: [0, 0.69444, 0.09031, 0, 0.77778],
      934: [0, 0.69444, 0.04603, 0, 0.72222],
      936: [0, 0.69444, 0.09031, 0, 0.77778],
      937: [0, 0.69444, 0.08293, 0, 0.72222],
      8211: [0, 0.44444, 0.08616, 0, 0.5],
      8212: [0, 0.44444, 0.08616, 0, 1],
      8216: [0, 0.69444, 0.07816, 0, 0.27778],
      8217: [0, 0.69444, 0.07816, 0, 0.27778],
      8220: [0, 0.69444, 0.14205, 0, 0.5],
      8221: [0, 0.69444, 316e-5, 0, 0.5]
    },
    "SansSerif-Regular": {
      32: [0, 0, 0, 0, 0.25],
      33: [0, 0.69444, 0, 0, 0.31945],
      34: [0, 0.69444, 0, 0, 0.5],
      35: [0.19444, 0.69444, 0, 0, 0.83334],
      36: [0.05556, 0.75, 0, 0, 0.5],
      37: [0.05556, 0.75, 0, 0, 0.83334],
      38: [0, 0.69444, 0, 0, 0.75834],
      39: [0, 0.69444, 0, 0, 0.27778],
      40: [0.25, 0.75, 0, 0, 0.38889],
      41: [0.25, 0.75, 0, 0, 0.38889],
      42: [0, 0.75, 0, 0, 0.5],
      43: [0.08333, 0.58333, 0, 0, 0.77778],
      44: [0.125, 0.08333, 0, 0, 0.27778],
      45: [0, 0.44444, 0, 0, 0.33333],
      46: [0, 0.08333, 0, 0, 0.27778],
      47: [0.25, 0.75, 0, 0, 0.5],
      48: [0, 0.65556, 0, 0, 0.5],
      49: [0, 0.65556, 0, 0, 0.5],
      50: [0, 0.65556, 0, 0, 0.5],
      51: [0, 0.65556, 0, 0, 0.5],
      52: [0, 0.65556, 0, 0, 0.5],
      53: [0, 0.65556, 0, 0, 0.5],
      54: [0, 0.65556, 0, 0, 0.5],
      55: [0, 0.65556, 0, 0, 0.5],
      56: [0, 0.65556, 0, 0, 0.5],
      57: [0, 0.65556, 0, 0, 0.5],
      58: [0, 0.44444, 0, 0, 0.27778],
      59: [0.125, 0.44444, 0, 0, 0.27778],
      61: [-0.13, 0.37, 0, 0, 0.77778],
      63: [0, 0.69444, 0, 0, 0.47222],
      64: [0, 0.69444, 0, 0, 0.66667],
      65: [0, 0.69444, 0, 0, 0.66667],
      66: [0, 0.69444, 0, 0, 0.66667],
      67: [0, 0.69444, 0, 0, 0.63889],
      68: [0, 0.69444, 0, 0, 0.72223],
      69: [0, 0.69444, 0, 0, 0.59722],
      70: [0, 0.69444, 0, 0, 0.56945],
      71: [0, 0.69444, 0, 0, 0.66667],
      72: [0, 0.69444, 0, 0, 0.70834],
      73: [0, 0.69444, 0, 0, 0.27778],
      74: [0, 0.69444, 0, 0, 0.47222],
      75: [0, 0.69444, 0, 0, 0.69445],
      76: [0, 0.69444, 0, 0, 0.54167],
      77: [0, 0.69444, 0, 0, 0.875],
      78: [0, 0.69444, 0, 0, 0.70834],
      79: [0, 0.69444, 0, 0, 0.73611],
      80: [0, 0.69444, 0, 0, 0.63889],
      81: [0.125, 0.69444, 0, 0, 0.73611],
      82: [0, 0.69444, 0, 0, 0.64584],
      83: [0, 0.69444, 0, 0, 0.55556],
      84: [0, 0.69444, 0, 0, 0.68056],
      85: [0, 0.69444, 0, 0, 0.6875],
      86: [0, 0.69444, 0.01389, 0, 0.66667],
      87: [0, 0.69444, 0.01389, 0, 0.94445],
      88: [0, 0.69444, 0, 0, 0.66667],
      89: [0, 0.69444, 0.025, 0, 0.66667],
      90: [0, 0.69444, 0, 0, 0.61111],
      91: [0.25, 0.75, 0, 0, 0.28889],
      93: [0.25, 0.75, 0, 0, 0.28889],
      94: [0, 0.69444, 0, 0, 0.5],
      95: [0.35, 0.09444, 0.02778, 0, 0.5],
      97: [0, 0.44444, 0, 0, 0.48056],
      98: [0, 0.69444, 0, 0, 0.51667],
      99: [0, 0.44444, 0, 0, 0.44445],
      100: [0, 0.69444, 0, 0, 0.51667],
      101: [0, 0.44444, 0, 0, 0.44445],
      102: [0, 0.69444, 0.06944, 0, 0.30556],
      103: [0.19444, 0.44444, 0.01389, 0, 0.5],
      104: [0, 0.69444, 0, 0, 0.51667],
      105: [0, 0.67937, 0, 0, 0.23889],
      106: [0.19444, 0.67937, 0, 0, 0.26667],
      107: [0, 0.69444, 0, 0, 0.48889],
      108: [0, 0.69444, 0, 0, 0.23889],
      109: [0, 0.44444, 0, 0, 0.79445],
      110: [0, 0.44444, 0, 0, 0.51667],
      111: [0, 0.44444, 0, 0, 0.5],
      112: [0.19444, 0.44444, 0, 0, 0.51667],
      113: [0.19444, 0.44444, 0, 0, 0.51667],
      114: [0, 0.44444, 0.01389, 0, 0.34167],
      115: [0, 0.44444, 0, 0, 0.38333],
      116: [0, 0.57143, 0, 0, 0.36111],
      117: [0, 0.44444, 0, 0, 0.51667],
      118: [0, 0.44444, 0.01389, 0, 0.46111],
      119: [0, 0.44444, 0.01389, 0, 0.68334],
      120: [0, 0.44444, 0, 0, 0.46111],
      121: [0.19444, 0.44444, 0.01389, 0, 0.46111],
      122: [0, 0.44444, 0, 0, 0.43472],
      126: [0.35, 0.32659, 0, 0, 0.5],
      160: [0, 0, 0, 0, 0.25],
      168: [0, 0.67937, 0, 0, 0.5],
      176: [0, 0.69444, 0, 0, 0.66667],
      184: [0.17014, 0, 0, 0, 0.44445],
      305: [0, 0.44444, 0, 0, 0.23889],
      567: [0.19444, 0.44444, 0, 0, 0.26667],
      710: [0, 0.69444, 0, 0, 0.5],
      711: [0, 0.63194, 0, 0, 0.5],
      713: [0, 0.60889, 0, 0, 0.5],
      714: [0, 0.69444, 0, 0, 0.5],
      715: [0, 0.69444, 0, 0, 0.5],
      728: [0, 0.69444, 0, 0, 0.5],
      729: [0, 0.67937, 0, 0, 0.27778],
      730: [0, 0.69444, 0, 0, 0.66667],
      732: [0, 0.67659, 0, 0, 0.5],
      733: [0, 0.69444, 0, 0, 0.5],
      915: [0, 0.69444, 0, 0, 0.54167],
      916: [0, 0.69444, 0, 0, 0.83334],
      920: [0, 0.69444, 0, 0, 0.77778],
      923: [0, 0.69444, 0, 0, 0.61111],
      926: [0, 0.69444, 0, 0, 0.66667],
      928: [0, 0.69444, 0, 0, 0.70834],
      931: [0, 0.69444, 0, 0, 0.72222],
      933: [0, 0.69444, 0, 0, 0.77778],
      934: [0, 0.69444, 0, 0, 0.72222],
      936: [0, 0.69444, 0, 0, 0.77778],
      937: [0, 0.69444, 0, 0, 0.72222],
      8211: [0, 0.44444, 0.02778, 0, 0.5],
      8212: [0, 0.44444, 0.02778, 0, 1],
      8216: [0, 0.69444, 0, 0, 0.27778],
      8217: [0, 0.69444, 0, 0, 0.27778],
      8220: [0, 0.69444, 0, 0, 0.5],
      8221: [0, 0.69444, 0, 0, 0.5]
    },
    "Script-Regular": {
      32: [0, 0, 0, 0, 0.25],
      65: [0, 0.7, 0.22925, 0, 0.80253],
      66: [0, 0.7, 0.04087, 0, 0.90757],
      67: [0, 0.7, 0.1689, 0, 0.66619],
      68: [0, 0.7, 0.09371, 0, 0.77443],
      69: [0, 0.7, 0.18583, 0, 0.56162],
      70: [0, 0.7, 0.13634, 0, 0.89544],
      71: [0, 0.7, 0.17322, 0, 0.60961],
      72: [0, 0.7, 0.29694, 0, 0.96919],
      73: [0, 0.7, 0.19189, 0, 0.80907],
      74: [0.27778, 0.7, 0.19189, 0, 1.05159],
      75: [0, 0.7, 0.31259, 0, 0.91364],
      76: [0, 0.7, 0.19189, 0, 0.87373],
      77: [0, 0.7, 0.15981, 0, 1.08031],
      78: [0, 0.7, 0.3525, 0, 0.9015],
      79: [0, 0.7, 0.08078, 0, 0.73787],
      80: [0, 0.7, 0.08078, 0, 1.01262],
      81: [0, 0.7, 0.03305, 0, 0.88282],
      82: [0, 0.7, 0.06259, 0, 0.85],
      83: [0, 0.7, 0.19189, 0, 0.86767],
      84: [0, 0.7, 0.29087, 0, 0.74697],
      85: [0, 0.7, 0.25815, 0, 0.79996],
      86: [0, 0.7, 0.27523, 0, 0.62204],
      87: [0, 0.7, 0.27523, 0, 0.80532],
      88: [0, 0.7, 0.26006, 0, 0.94445],
      89: [0, 0.7, 0.2939, 0, 0.70961],
      90: [0, 0.7, 0.24037, 0, 0.8212],
      160: [0, 0, 0, 0, 0.25]
    },
    "Size1-Regular": {
      32: [0, 0, 0, 0, 0.25],
      40: [0.35001, 0.85, 0, 0, 0.45834],
      41: [0.35001, 0.85, 0, 0, 0.45834],
      47: [0.35001, 0.85, 0, 0, 0.57778],
      91: [0.35001, 0.85, 0, 0, 0.41667],
      92: [0.35001, 0.85, 0, 0, 0.57778],
      93: [0.35001, 0.85, 0, 0, 0.41667],
      123: [0.35001, 0.85, 0, 0, 0.58334],
      125: [0.35001, 0.85, 0, 0, 0.58334],
      160: [0, 0, 0, 0, 0.25],
      710: [0, 0.72222, 0, 0, 0.55556],
      732: [0, 0.72222, 0, 0, 0.55556],
      770: [0, 0.72222, 0, 0, 0.55556],
      771: [0, 0.72222, 0, 0, 0.55556],
      8214: [-99e-5, 0.601, 0, 0, 0.77778],
      8593: [1e-5, 0.6, 0, 0, 0.66667],
      8595: [1e-5, 0.6, 0, 0, 0.66667],
      8657: [1e-5, 0.6, 0, 0, 0.77778],
      8659: [1e-5, 0.6, 0, 0, 0.77778],
      8719: [0.25001, 0.75, 0, 0, 0.94445],
      8720: [0.25001, 0.75, 0, 0, 0.94445],
      8721: [0.25001, 0.75, 0, 0, 1.05556],
      8730: [0.35001, 0.85, 0, 0, 1],
      8739: [-599e-5, 0.606, 0, 0, 0.33333],
      8741: [-599e-5, 0.606, 0, 0, 0.55556],
      8747: [0.30612, 0.805, 0.19445, 0, 0.47222],
      8748: [0.306, 0.805, 0.19445, 0, 0.47222],
      8749: [0.306, 0.805, 0.19445, 0, 0.47222],
      8750: [0.30612, 0.805, 0.19445, 0, 0.47222],
      8896: [0.25001, 0.75, 0, 0, 0.83334],
      8897: [0.25001, 0.75, 0, 0, 0.83334],
      8898: [0.25001, 0.75, 0, 0, 0.83334],
      8899: [0.25001, 0.75, 0, 0, 0.83334],
      8968: [0.35001, 0.85, 0, 0, 0.47222],
      8969: [0.35001, 0.85, 0, 0, 0.47222],
      8970: [0.35001, 0.85, 0, 0, 0.47222],
      8971: [0.35001, 0.85, 0, 0, 0.47222],
      9168: [-99e-5, 0.601, 0, 0, 0.66667],
      10216: [0.35001, 0.85, 0, 0, 0.47222],
      10217: [0.35001, 0.85, 0, 0, 0.47222],
      10752: [0.25001, 0.75, 0, 0, 1.11111],
      10753: [0.25001, 0.75, 0, 0, 1.11111],
      10754: [0.25001, 0.75, 0, 0, 1.11111],
      10756: [0.25001, 0.75, 0, 0, 0.83334],
      10758: [0.25001, 0.75, 0, 0, 0.83334]
    },
    "Size2-Regular": {
      32: [0, 0, 0, 0, 0.25],
      40: [0.65002, 1.15, 0, 0, 0.59722],
      41: [0.65002, 1.15, 0, 0, 0.59722],
      47: [0.65002, 1.15, 0, 0, 0.81111],
      91: [0.65002, 1.15, 0, 0, 0.47222],
      92: [0.65002, 1.15, 0, 0, 0.81111],
      93: [0.65002, 1.15, 0, 0, 0.47222],
      123: [0.65002, 1.15, 0, 0, 0.66667],
      125: [0.65002, 1.15, 0, 0, 0.66667],
      160: [0, 0, 0, 0, 0.25],
      710: [0, 0.75, 0, 0, 1],
      732: [0, 0.75, 0, 0, 1],
      770: [0, 0.75, 0, 0, 1],
      771: [0, 0.75, 0, 0, 1],
      8719: [0.55001, 1.05, 0, 0, 1.27778],
      8720: [0.55001, 1.05, 0, 0, 1.27778],
      8721: [0.55001, 1.05, 0, 0, 1.44445],
      8730: [0.65002, 1.15, 0, 0, 1],
      8747: [0.86225, 1.36, 0.44445, 0, 0.55556],
      8748: [0.862, 1.36, 0.44445, 0, 0.55556],
      8749: [0.862, 1.36, 0.44445, 0, 0.55556],
      8750: [0.86225, 1.36, 0.44445, 0, 0.55556],
      8896: [0.55001, 1.05, 0, 0, 1.11111],
      8897: [0.55001, 1.05, 0, 0, 1.11111],
      8898: [0.55001, 1.05, 0, 0, 1.11111],
      8899: [0.55001, 1.05, 0, 0, 1.11111],
      8968: [0.65002, 1.15, 0, 0, 0.52778],
      8969: [0.65002, 1.15, 0, 0, 0.52778],
      8970: [0.65002, 1.15, 0, 0, 0.52778],
      8971: [0.65002, 1.15, 0, 0, 0.52778],
      10216: [0.65002, 1.15, 0, 0, 0.61111],
      10217: [0.65002, 1.15, 0, 0, 0.61111],
      10752: [0.55001, 1.05, 0, 0, 1.51112],
      10753: [0.55001, 1.05, 0, 0, 1.51112],
      10754: [0.55001, 1.05, 0, 0, 1.51112],
      10756: [0.55001, 1.05, 0, 0, 1.11111],
      10758: [0.55001, 1.05, 0, 0, 1.11111]
    },
    "Size3-Regular": {
      32: [0, 0, 0, 0, 0.25],
      40: [0.95003, 1.45, 0, 0, 0.73611],
      41: [0.95003, 1.45, 0, 0, 0.73611],
      47: [0.95003, 1.45, 0, 0, 1.04445],
      91: [0.95003, 1.45, 0, 0, 0.52778],
      92: [0.95003, 1.45, 0, 0, 1.04445],
      93: [0.95003, 1.45, 0, 0, 0.52778],
      123: [0.95003, 1.45, 0, 0, 0.75],
      125: [0.95003, 1.45, 0, 0, 0.75],
      160: [0, 0, 0, 0, 0.25],
      710: [0, 0.75, 0, 0, 1.44445],
      732: [0, 0.75, 0, 0, 1.44445],
      770: [0, 0.75, 0, 0, 1.44445],
      771: [0, 0.75, 0, 0, 1.44445],
      8730: [0.95003, 1.45, 0, 0, 1],
      8968: [0.95003, 1.45, 0, 0, 0.58334],
      8969: [0.95003, 1.45, 0, 0, 0.58334],
      8970: [0.95003, 1.45, 0, 0, 0.58334],
      8971: [0.95003, 1.45, 0, 0, 0.58334],
      10216: [0.95003, 1.45, 0, 0, 0.75],
      10217: [0.95003, 1.45, 0, 0, 0.75]
    },
    "Size4-Regular": {
      32: [0, 0, 0, 0, 0.25],
      40: [1.25003, 1.75, 0, 0, 0.79167],
      41: [1.25003, 1.75, 0, 0, 0.79167],
      47: [1.25003, 1.75, 0, 0, 1.27778],
      91: [1.25003, 1.75, 0, 0, 0.58334],
      92: [1.25003, 1.75, 0, 0, 1.27778],
      93: [1.25003, 1.75, 0, 0, 0.58334],
      123: [1.25003, 1.75, 0, 0, 0.80556],
      125: [1.25003, 1.75, 0, 0, 0.80556],
      160: [0, 0, 0, 0, 0.25],
      710: [0, 0.825, 0, 0, 1.8889],
      732: [0, 0.825, 0, 0, 1.8889],
      770: [0, 0.825, 0, 0, 1.8889],
      771: [0, 0.825, 0, 0, 1.8889],
      8730: [1.25003, 1.75, 0, 0, 1],
      8968: [1.25003, 1.75, 0, 0, 0.63889],
      8969: [1.25003, 1.75, 0, 0, 0.63889],
      8970: [1.25003, 1.75, 0, 0, 0.63889],
      8971: [1.25003, 1.75, 0, 0, 0.63889],
      9115: [0.64502, 1.155, 0, 0, 0.875],
      9116: [1e-5, 0.6, 0, 0, 0.875],
      9117: [0.64502, 1.155, 0, 0, 0.875],
      9118: [0.64502, 1.155, 0, 0, 0.875],
      9119: [1e-5, 0.6, 0, 0, 0.875],
      9120: [0.64502, 1.155, 0, 0, 0.875],
      9121: [0.64502, 1.155, 0, 0, 0.66667],
      9122: [-99e-5, 0.601, 0, 0, 0.66667],
      9123: [0.64502, 1.155, 0, 0, 0.66667],
      9124: [0.64502, 1.155, 0, 0, 0.66667],
      9125: [-99e-5, 0.601, 0, 0, 0.66667],
      9126: [0.64502, 1.155, 0, 0, 0.66667],
      9127: [1e-5, 0.9, 0, 0, 0.88889],
      9128: [0.65002, 1.15, 0, 0, 0.88889],
      9129: [0.90001, 0, 0, 0, 0.88889],
      9130: [0, 0.3, 0, 0, 0.88889],
      9131: [1e-5, 0.9, 0, 0, 0.88889],
      9132: [0.65002, 1.15, 0, 0, 0.88889],
      9133: [0.90001, 0, 0, 0, 0.88889],
      9143: [0.88502, 0.915, 0, 0, 1.05556],
      10216: [1.25003, 1.75, 0, 0, 0.80556],
      10217: [1.25003, 1.75, 0, 0, 0.80556],
      57344: [-499e-5, 0.605, 0, 0, 1.05556],
      57345: [-499e-5, 0.605, 0, 0, 1.05556],
      57680: [0, 0.12, 0, 0, 0.45],
      57681: [0, 0.12, 0, 0, 0.45],
      57682: [0, 0.12, 0, 0, 0.45],
      57683: [0, 0.12, 0, 0, 0.45]
    },
    "Typewriter-Regular": {
      32: [0, 0, 0, 0, 0.525],
      33: [0, 0.61111, 0, 0, 0.525],
      34: [0, 0.61111, 0, 0, 0.525],
      35: [0, 0.61111, 0, 0, 0.525],
      36: [0.08333, 0.69444, 0, 0, 0.525],
      37: [0.08333, 0.69444, 0, 0, 0.525],
      38: [0, 0.61111, 0, 0, 0.525],
      39: [0, 0.61111, 0, 0, 0.525],
      40: [0.08333, 0.69444, 0, 0, 0.525],
      41: [0.08333, 0.69444, 0, 0, 0.525],
      42: [0, 0.52083, 0, 0, 0.525],
      43: [-0.08056, 0.53055, 0, 0, 0.525],
      44: [0.13889, 0.125, 0, 0, 0.525],
      45: [-0.08056, 0.53055, 0, 0, 0.525],
      46: [0, 0.125, 0, 0, 0.525],
      47: [0.08333, 0.69444, 0, 0, 0.525],
      48: [0, 0.61111, 0, 0, 0.525],
      49: [0, 0.61111, 0, 0, 0.525],
      50: [0, 0.61111, 0, 0, 0.525],
      51: [0, 0.61111, 0, 0, 0.525],
      52: [0, 0.61111, 0, 0, 0.525],
      53: [0, 0.61111, 0, 0, 0.525],
      54: [0, 0.61111, 0, 0, 0.525],
      55: [0, 0.61111, 0, 0, 0.525],
      56: [0, 0.61111, 0, 0, 0.525],
      57: [0, 0.61111, 0, 0, 0.525],
      58: [0, 0.43056, 0, 0, 0.525],
      59: [0.13889, 0.43056, 0, 0, 0.525],
      60: [-0.05556, 0.55556, 0, 0, 0.525],
      61: [-0.19549, 0.41562, 0, 0, 0.525],
      62: [-0.05556, 0.55556, 0, 0, 0.525],
      63: [0, 0.61111, 0, 0, 0.525],
      64: [0, 0.61111, 0, 0, 0.525],
      65: [0, 0.61111, 0, 0, 0.525],
      66: [0, 0.61111, 0, 0, 0.525],
      67: [0, 0.61111, 0, 0, 0.525],
      68: [0, 0.61111, 0, 0, 0.525],
      69: [0, 0.61111, 0, 0, 0.525],
      70: [0, 0.61111, 0, 0, 0.525],
      71: [0, 0.61111, 0, 0, 0.525],
      72: [0, 0.61111, 0, 0, 0.525],
      73: [0, 0.61111, 0, 0, 0.525],
      74: [0, 0.61111, 0, 0, 0.525],
      75: [0, 0.61111, 0, 0, 0.525],
      76: [0, 0.61111, 0, 0, 0.525],
      77: [0, 0.61111, 0, 0, 0.525],
      78: [0, 0.61111, 0, 0, 0.525],
      79: [0, 0.61111, 0, 0, 0.525],
      80: [0, 0.61111, 0, 0, 0.525],
      81: [0.13889, 0.61111, 0, 0, 0.525],
      82: [0, 0.61111, 0, 0, 0.525],
      83: [0, 0.61111, 0, 0, 0.525],
      84: [0, 0.61111, 0, 0, 0.525],
      85: [0, 0.61111, 0, 0, 0.525],
      86: [0, 0.61111, 0, 0, 0.525],
      87: [0, 0.61111, 0, 0, 0.525],
      88: [0, 0.61111, 0, 0, 0.525],
      89: [0, 0.61111, 0, 0, 0.525],
      90: [0, 0.61111, 0, 0, 0.525],
      91: [0.08333, 0.69444, 0, 0, 0.525],
      92: [0.08333, 0.69444, 0, 0, 0.525],
      93: [0.08333, 0.69444, 0, 0, 0.525],
      94: [0, 0.61111, 0, 0, 0.525],
      95: [0.09514, 0, 0, 0, 0.525],
      96: [0, 0.61111, 0, 0, 0.525],
      97: [0, 0.43056, 0, 0, 0.525],
      98: [0, 0.61111, 0, 0, 0.525],
      99: [0, 0.43056, 0, 0, 0.525],
      100: [0, 0.61111, 0, 0, 0.525],
      101: [0, 0.43056, 0, 0, 0.525],
      102: [0, 0.61111, 0, 0, 0.525],
      103: [0.22222, 0.43056, 0, 0, 0.525],
      104: [0, 0.61111, 0, 0, 0.525],
      105: [0, 0.61111, 0, 0, 0.525],
      106: [0.22222, 0.61111, 0, 0, 0.525],
      107: [0, 0.61111, 0, 0, 0.525],
      108: [0, 0.61111, 0, 0, 0.525],
      109: [0, 0.43056, 0, 0, 0.525],
      110: [0, 0.43056, 0, 0, 0.525],
      111: [0, 0.43056, 0, 0, 0.525],
      112: [0.22222, 0.43056, 0, 0, 0.525],
      113: [0.22222, 0.43056, 0, 0, 0.525],
      114: [0, 0.43056, 0, 0, 0.525],
      115: [0, 0.43056, 0, 0, 0.525],
      116: [0, 0.55358, 0, 0, 0.525],
      117: [0, 0.43056, 0, 0, 0.525],
      118: [0, 0.43056, 0, 0, 0.525],
      119: [0, 0.43056, 0, 0, 0.525],
      120: [0, 0.43056, 0, 0, 0.525],
      121: [0.22222, 0.43056, 0, 0, 0.525],
      122: [0, 0.43056, 0, 0, 0.525],
      123: [0.08333, 0.69444, 0, 0, 0.525],
      124: [0.08333, 0.69444, 0, 0, 0.525],
      125: [0.08333, 0.69444, 0, 0, 0.525],
      126: [0, 0.61111, 0, 0, 0.525],
      127: [0, 0.61111, 0, 0, 0.525],
      160: [0, 0, 0, 0, 0.525],
      176: [0, 0.61111, 0, 0, 0.525],
      184: [0.19445, 0, 0, 0, 0.525],
      305: [0, 0.43056, 0, 0, 0.525],
      567: [0.22222, 0.43056, 0, 0, 0.525],
      711: [0, 0.56597, 0, 0, 0.525],
      713: [0, 0.56555, 0, 0, 0.525],
      714: [0, 0.61111, 0, 0, 0.525],
      715: [0, 0.61111, 0, 0, 0.525],
      728: [0, 0.61111, 0, 0, 0.525],
      730: [0, 0.61111, 0, 0, 0.525],
      770: [0, 0.61111, 0, 0, 0.525],
      771: [0, 0.61111, 0, 0, 0.525],
      776: [0, 0.61111, 0, 0, 0.525],
      915: [0, 0.61111, 0, 0, 0.525],
      916: [0, 0.61111, 0, 0, 0.525],
      920: [0, 0.61111, 0, 0, 0.525],
      923: [0, 0.61111, 0, 0, 0.525],
      926: [0, 0.61111, 0, 0, 0.525],
      928: [0, 0.61111, 0, 0, 0.525],
      931: [0, 0.61111, 0, 0, 0.525],
      933: [0, 0.61111, 0, 0, 0.525],
      934: [0, 0.61111, 0, 0, 0.525],
      936: [0, 0.61111, 0, 0, 0.525],
      937: [0, 0.61111, 0, 0, 0.525],
      8216: [0, 0.61111, 0, 0, 0.525],
      8217: [0, 0.61111, 0, 0, 0.525],
      8242: [0, 0.61111, 0, 0, 0.525],
      9251: [0.11111, 0.21944, 0, 0, 0.525]
    }
  }, sigmasAndXis = {
    slant: [0.25, 0.25, 0.25],
    // sigma1
    space: [0, 0, 0],
    // sigma2
    stretch: [0, 0, 0],
    // sigma3
    shrink: [0, 0, 0],
    // sigma4
    xHeight: [0.431, 0.431, 0.431],
    // sigma5
    quad: [1, 1.171, 1.472],
    // sigma6
    extraSpace: [0, 0, 0],
    // sigma7
    num1: [0.677, 0.732, 0.925],
    // sigma8
    num2: [0.394, 0.384, 0.387],
    // sigma9
    num3: [0.444, 0.471, 0.504],
    // sigma10
    denom1: [0.686, 0.752, 1.025],
    // sigma11
    denom2: [0.345, 0.344, 0.532],
    // sigma12
    sup1: [0.413, 0.503, 0.504],
    // sigma13
    sup2: [0.363, 0.431, 0.404],
    // sigma14
    sup3: [0.289, 0.286, 0.294],
    // sigma15
    sub1: [0.15, 0.143, 0.2],
    // sigma16
    sub2: [0.247, 0.286, 0.4],
    // sigma17
    supDrop: [0.386, 0.353, 0.494],
    // sigma18
    subDrop: [0.05, 0.071, 0.1],
    // sigma19
    delim1: [2.39, 1.7, 1.98],
    // sigma20
    delim2: [1.01, 1.157, 1.42],
    // sigma21
    axisHeight: [0.25, 0.25, 0.25],
    // sigma22
    // These font metrics are extracted from TeX by using tftopl on cmex10.tfm;
    // they correspond to the font parameters of the extension fonts (family 3).
    // See the TeXbook, page 441. In AMSTeX, the extension fonts scale; to
    // match cmex7, we'd use cmex7.tfm values for script and scriptscript
    // values.
    defaultRuleThickness: [0.04, 0.049, 0.049],
    // xi8; cmex7: 0.049
    bigOpSpacing1: [0.111, 0.111, 0.111],
    // xi9
    bigOpSpacing2: [0.166, 0.166, 0.166],
    // xi10
    bigOpSpacing3: [0.2, 0.2, 0.2],
    // xi11
    bigOpSpacing4: [0.6, 0.611, 0.611],
    // xi12; cmex7: 0.611
    bigOpSpacing5: [0.1, 0.143, 0.143],
    // xi13; cmex7: 0.143
    // The \sqrt rule width is taken from the height of the surd character.
    // Since we use the same font at all sizes, this thickness doesn't scale.
    sqrtRuleThickness: [0.04, 0.04, 0.04],
    // This value determines how large a pt is, for metrics which are defined
    // in terms of pts.
    // This value is also used in katex.scss; if you change it make sure the
    // values match.
    ptPerEm: [10, 10, 10],
    // The space between adjacent `|` columns in an array definition. From
    // `\showthe\doublerulesep` in LaTeX. Equals 2.0 / ptPerEm.
    doubleRuleSep: [0.2, 0.2, 0.2],
    // The width of separator lines in {array} environments. From
    // `\showthe\arrayrulewidth` in LaTeX. Equals 0.4 / ptPerEm.
    arrayRuleWidth: [0.04, 0.04, 0.04],
    // Two values from LaTeX source2e:
    fboxsep: [0.3, 0.3, 0.3],
    //        3 pt / ptPerEm
    fboxrule: [0.04, 0.04, 0.04]
    // 0.4 pt / ptPerEm
  }, extraCharacterMap = {
    // Latin-1
    Å: "A",
    Ð: "D",
    Þ: "o",
    å: "a",
    ð: "d",
    þ: "o",
    // Cyrillic
    А: "A",
    Б: "B",
    В: "B",
    Г: "F",
    Д: "A",
    Е: "E",
    Ж: "K",
    З: "3",
    И: "N",
    Й: "N",
    К: "K",
    Л: "N",
    М: "M",
    Н: "H",
    О: "O",
    П: "N",
    Р: "P",
    С: "C",
    Т: "T",
    У: "y",
    Ф: "O",
    Х: "X",
    Ц: "U",
    Ч: "h",
    Ш: "W",
    Щ: "W",
    Ъ: "B",
    Ы: "X",
    Ь: "B",
    Э: "3",
    Ю: "X",
    Я: "R",
    а: "a",
    б: "b",
    в: "a",
    г: "r",
    д: "y",
    е: "e",
    ж: "m",
    з: "e",
    и: "n",
    й: "n",
    к: "n",
    л: "n",
    м: "m",
    н: "n",
    о: "o",
    п: "n",
    р: "p",
    с: "c",
    т: "o",
    у: "y",
    ф: "b",
    х: "x",
    ц: "n",
    ч: "n",
    ш: "w",
    щ: "w",
    ъ: "a",
    ы: "m",
    ь: "a",
    э: "e",
    ю: "m",
    я: "r"
  };
  function setFontMetrics(O, _) {
    fontMetricsData[O] = _;
  }
  function getCharacterMetrics(O, _, H) {
    if (!fontMetricsData[_])
      throw new Error("Font metrics not found for font: " + _ + ".");
    var V = O.charCodeAt(0), q = fontMetricsData[_][V];
    if (!q && O[0] in extraCharacterMap && (V = extraCharacterMap[O[0]].charCodeAt(0), q = fontMetricsData[_][V]), !q && H === "text" && supportedCodepoint(V) && (q = fontMetricsData[_][77]), q)
      return {
        depth: q[0],
        height: q[1],
        italic: q[2],
        skew: q[3],
        width: q[4]
      };
  }
  var fontMetricsBySizeIndex = {};
  function getGlobalMetrics(O) {
    var _;
    if (O >= 5 ? _ = 0 : O >= 3 ? _ = 1 : _ = 2, !fontMetricsBySizeIndex[_]) {
      var H = fontMetricsBySizeIndex[_] = {
        cssEmPerMu: sigmasAndXis.quad[_] / 18
      };
      for (var V in sigmasAndXis)
        sigmasAndXis.hasOwnProperty(V) && (H[V] = sigmasAndXis[V][_]);
    }
    return fontMetricsBySizeIndex[_];
  }
  var sizeStyleMap = [
    // Each element contains [textsize, scriptsize, scriptscriptsize].
    // The size mappings are taken from TeX with \normalsize=10pt.
    [1, 1, 1],
    // size1: [5, 5, 5]              \tiny
    [2, 1, 1],
    // size2: [6, 5, 5]
    [3, 1, 1],
    // size3: [7, 5, 5]              \scriptsize
    [4, 2, 1],
    // size4: [8, 6, 5]              \footnotesize
    [5, 2, 1],
    // size5: [9, 6, 5]              \small
    [6, 3, 1],
    // size6: [10, 7, 5]             \normalsize
    [7, 4, 2],
    // size7: [12, 8, 6]             \large
    [8, 6, 3],
    // size8: [14.4, 10, 7]          \Large
    [9, 7, 6],
    // size9: [17.28, 12, 10]        \LARGE
    [10, 8, 7],
    // size10: [20.74, 14.4, 12]     \huge
    [11, 10, 9]
    // size11: [24.88, 20.74, 17.28] \HUGE
  ], sizeMultipliers = [
    // fontMetrics.js:getGlobalMetrics also uses size indexes, so if
    // you change size indexes, change that function.
    0.5,
    0.6,
    0.7,
    0.8,
    0.9,
    1,
    1.2,
    1.44,
    1.728,
    2.074,
    2.488
  ], sizeAtStyle = function O(_, H) {
    return H.size < 2 ? _ : sizeStyleMap[_ - 1][H.size - 1];
  };
  class Options {
    // A font family applies to a group of fonts (i.e. SansSerif), while a font
    // represents a specific font (i.e. SansSerif Bold).
    // See: https://tex.stackexchange.com/questions/22350/difference-between-textrm-and-mathrm
    /**
     * The base size index.
     */
    constructor(_) {
      this.style = void 0, this.color = void 0, this.size = void 0, this.textSize = void 0, this.phantom = void 0, this.font = void 0, this.fontFamily = void 0, this.fontWeight = void 0, this.fontShape = void 0, this.sizeMultiplier = void 0, this.maxSize = void 0, this.minRuleThickness = void 0, this._fontMetrics = void 0, this.style = _.style, this.color = _.color, this.size = _.size || Options.BASESIZE, this.textSize = _.textSize || this.size, this.phantom = !!_.phantom, this.font = _.font || "", this.fontFamily = _.fontFamily || "", this.fontWeight = _.fontWeight || "", this.fontShape = _.fontShape || "", this.sizeMultiplier = sizeMultipliers[this.size - 1], this.maxSize = _.maxSize, this.minRuleThickness = _.minRuleThickness, this._fontMetrics = void 0;
    }
    /**
     * Returns a new options object with the same properties as "this".  Properties
     * from "extension" will be copied to the new options object.
     */
    extend(_) {
      var H = {
        style: this.style,
        size: this.size,
        textSize: this.textSize,
        color: this.color,
        phantom: this.phantom,
        font: this.font,
        fontFamily: this.fontFamily,
        fontWeight: this.fontWeight,
        fontShape: this.fontShape,
        maxSize: this.maxSize,
        minRuleThickness: this.minRuleThickness
      };
      for (var V in _)
        _.hasOwnProperty(V) && (H[V] = _[V]);
      return new Options(H);
    }
    /**
     * Return an options object with the given style. If `this.style === style`,
     * returns `this`.
     */
    havingStyle(_) {
      return this.style === _ ? this : this.extend({
        style: _,
        size: sizeAtStyle(this.textSize, _)
      });
    }
    /**
     * Return an options object with a cramped version of the current style. If
     * the current style is cramped, returns `this`.
     */
    havingCrampedStyle() {
      return this.havingStyle(this.style.cramp());
    }
    /**
     * Return an options object with the given size and in at least `\textstyle`.
     * Returns `this` if appropriate.
     */
    havingSize(_) {
      return this.size === _ && this.textSize === _ ? this : this.extend({
        style: this.style.text(),
        size: _,
        textSize: _,
        sizeMultiplier: sizeMultipliers[_ - 1]
      });
    }
    /**
     * Like `this.havingSize(BASESIZE).havingStyle(style)`. If `style` is omitted,
     * changes to at least `\textstyle`.
     */
    havingBaseStyle(_) {
      _ = _ || this.style.text();
      var H = sizeAtStyle(Options.BASESIZE, _);
      return this.size === H && this.textSize === Options.BASESIZE && this.style === _ ? this : this.extend({
        style: _,
        size: H
      });
    }
    /**
     * Remove the effect of sizing changes such as \Huge.
     * Keep the effect of the current style, such as \scriptstyle.
     */
    havingBaseSizing() {
      var _;
      switch (this.style.id) {
        case 4:
        case 5:
          _ = 3;
          break;
        case 6:
        case 7:
          _ = 1;
          break;
        default:
          _ = 6;
      }
      return this.extend({
        style: this.style.text(),
        size: _
      });
    }
    /**
     * Create a new options object with the given color.
     */
    withColor(_) {
      return this.extend({
        color: _
      });
    }
    /**
     * Create a new options object with "phantom" set to true.
     */
    withPhantom() {
      return this.extend({
        phantom: !0
      });
    }
    /**
     * Creates a new options object with the given math font or old text font.
     * @type {[type]}
     */
    withFont(_) {
      return this.extend({
        font: _
      });
    }
    /**
     * Create a new options objects with the given fontFamily.
     */
    withTextFontFamily(_) {
      return this.extend({
        fontFamily: _,
        font: ""
      });
    }
    /**
     * Creates a new options object with the given font weight
     */
    withTextFontWeight(_) {
      return this.extend({
        fontWeight: _,
        font: ""
      });
    }
    /**
     * Creates a new options object with the given font weight
     */
    withTextFontShape(_) {
      return this.extend({
        fontShape: _,
        font: ""
      });
    }
    /**
     * Return the CSS sizing classes required to switch from enclosing options
     * `oldOptions` to `this`. Returns an array of classes.
     */
    sizingClasses(_) {
      return _.size !== this.size ? ["sizing", "reset-size" + _.size, "size" + this.size] : [];
    }
    /**
     * Return the CSS sizing classes required to switch to the base size. Like
     * `this.havingSize(BASESIZE).sizingClasses(this)`.
     */
    baseSizingClasses() {
      return this.size !== Options.BASESIZE ? ["sizing", "reset-size" + this.size, "size" + Options.BASESIZE] : [];
    }
    /**
     * Return the font metrics for this size.
     */
    fontMetrics() {
      return this._fontMetrics || (this._fontMetrics = getGlobalMetrics(this.size)), this._fontMetrics;
    }
    /**
     * Gets the CSS color of the current options object
     */
    getColor() {
      return this.phantom ? "transparent" : this.color;
    }
  }
  Options.BASESIZE = 6;
  var ptPerUnit = {
    // https://en.wikibooks.org/wiki/LaTeX/Lengths and
    // https://tex.stackexchange.com/a/8263
    pt: 1,
    // TeX point
    mm: 7227 / 2540,
    // millimeter
    cm: 7227 / 254,
    // centimeter
    in: 72.27,
    // inch
    bp: 803 / 800,
    // big (PostScript) points
    pc: 12,
    // pica
    dd: 1238 / 1157,
    // didot
    cc: 14856 / 1157,
    // cicero (12 didot)
    nd: 685 / 642,
    // new didot
    nc: 1370 / 107,
    // new cicero (12 new didot)
    sp: 1 / 65536,
    // scaled point (TeX's internal smallest unit)
    // https://tex.stackexchange.com/a/41371
    px: 803 / 800
    // \pdfpxdimen defaults to 1 bp in pdfTeX and LuaTeX
  }, relativeUnit = {
    ex: !0,
    em: !0,
    mu: !0
  }, validUnit = function O(_) {
    return typeof _ != "string" && (_ = _.unit), _ in ptPerUnit || _ in relativeUnit || _ === "ex";
  }, calculateSize = function O(_, H) {
    var V;
    if (_.unit in ptPerUnit)
      V = ptPerUnit[_.unit] / H.fontMetrics().ptPerEm / H.sizeMultiplier;
    else if (_.unit === "mu")
      V = H.fontMetrics().cssEmPerMu;
    else {
      var q;
      if (H.style.isTight() ? q = H.havingStyle(H.style.text()) : q = H, _.unit === "ex")
        V = q.fontMetrics().xHeight;
      else if (_.unit === "em")
        V = q.fontMetrics().quad;
      else
        throw new ParseError("Invalid unit: '" + _.unit + "'");
      q !== H && (V *= q.sizeMultiplier / H.sizeMultiplier);
    }
    return Math.min(_.number * V, H.maxSize);
  }, makeEm = function O(_) {
    return +_.toFixed(4) + "em";
  }, createClass = function O(_) {
    return _.filter((H) => H).join(" ");
  }, initNode = function O(_, H, V) {
    if (this.classes = _ || [], this.attributes = {}, this.height = 0, this.depth = 0, this.maxFontSize = 0, this.style = V || {}, H) {
      H.style.isTight() && this.classes.push("mtight");
      var q = H.getColor();
      q && (this.style.color = q);
    }
  }, toNode = function O(_) {
    var H = document.createElement(_);
    H.className = createClass(this.classes);
    for (var V in this.style)
      this.style.hasOwnProperty(V) && (H.style[V] = this.style[V]);
    for (var q in this.attributes)
      this.attributes.hasOwnProperty(q) && H.setAttribute(q, this.attributes[q]);
    for (var G = 0; G < this.children.length; G++)
      H.appendChild(this.children[G].toNode());
    return H;
  }, toMarkup = function O(_) {
    var H = "<" + _;
    this.classes.length && (H += ' class="' + utils.escape(createClass(this.classes)) + '"');
    var V = "";
    for (var q in this.style)
      this.style.hasOwnProperty(q) && (V += utils.hyphenate(q) + ":" + this.style[q] + ";");
    V && (H += ' style="' + utils.escape(V) + '"');
    for (var G in this.attributes)
      this.attributes.hasOwnProperty(G) && (H += " " + G + '="' + utils.escape(this.attributes[G]) + '"');
    H += ">";
    for (var J = 0; J < this.children.length; J++)
      H += this.children[J].toMarkup();
    return H += "</" + _ + ">", H;
  };
  let Span$1 = class {
    constructor(_, H, V, q) {
      this.children = void 0, this.attributes = void 0, this.classes = void 0, this.height = void 0, this.depth = void 0, this.width = void 0, this.maxFontSize = void 0, this.style = void 0, initNode.call(this, _, V, q), this.children = H || [];
    }
    /**
     * Sets an arbitrary attribute on the span. Warning: use this wisely. Not
     * all browsers support attributes the same, and having too many custom
     * attributes is probably bad.
     */
    setAttribute(_, H) {
      this.attributes[_] = H;
    }
    hasClass(_) {
      return utils.contains(this.classes, _);
    }
    toNode() {
      return toNode.call(this, "span");
    }
    toMarkup() {
      return toMarkup.call(this, "span");
    }
  };
  class Anchor {
    constructor(_, H, V, q) {
      this.children = void 0, this.attributes = void 0, this.classes = void 0, this.height = void 0, this.depth = void 0, this.maxFontSize = void 0, this.style = void 0, initNode.call(this, H, q), this.children = V || [], this.setAttribute("href", _);
    }
    setAttribute(_, H) {
      this.attributes[_] = H;
    }
    hasClass(_) {
      return utils.contains(this.classes, _);
    }
    toNode() {
      return toNode.call(this, "a");
    }
    toMarkup() {
      return toMarkup.call(this, "a");
    }
  }
  class Img {
    constructor(_, H, V) {
      this.src = void 0, this.alt = void 0, this.classes = void 0, this.height = void 0, this.depth = void 0, this.maxFontSize = void 0, this.style = void 0, this.alt = H, this.src = _, this.classes = ["mord"], this.style = V;
    }
    hasClass(_) {
      return utils.contains(this.classes, _);
    }
    toNode() {
      var _ = document.createElement("img");
      _.src = this.src, _.alt = this.alt, _.className = "mord";
      for (var H in this.style)
        this.style.hasOwnProperty(H) && (_.style[H] = this.style[H]);
      return _;
    }
    toMarkup() {
      var _ = '<img src="' + utils.escape(this.src) + '"' + (' alt="' + utils.escape(this.alt) + '"'), H = "";
      for (var V in this.style)
        this.style.hasOwnProperty(V) && (H += utils.hyphenate(V) + ":" + this.style[V] + ";");
      return H && (_ += ' style="' + utils.escape(H) + '"'), _ += "'/>", _;
    }
  }
  var iCombinations = {
    î: "ı̂",
    ï: "ı̈",
    í: "ı́",
    // 'ī': '\u0131\u0304', // enable when we add Extended Latin
    ì: "ı̀"
  };
  class SymbolNode {
    constructor(_, H, V, q, G, J, Z, te) {
      this.text = void 0, this.height = void 0, this.depth = void 0, this.italic = void 0, this.skew = void 0, this.width = void 0, this.maxFontSize = void 0, this.classes = void 0, this.style = void 0, this.text = _, this.height = H || 0, this.depth = V || 0, this.italic = q || 0, this.skew = G || 0, this.width = J || 0, this.classes = Z || [], this.style = te || {}, this.maxFontSize = 0;
      var ne = scriptFromCodepoint(this.text.charCodeAt(0));
      ne && this.classes.push(ne + "_fallback"), /[îïíì]/.test(this.text) && (this.text = iCombinations[this.text]);
    }
    hasClass(_) {
      return utils.contains(this.classes, _);
    }
    /**
     * Creates a text node or span from a symbol node. Note that a span is only
     * created if it is needed.
     */
    toNode() {
      var _ = document.createTextNode(this.text), H = null;
      this.italic > 0 && (H = document.createElement("span"), H.style.marginRight = makeEm(this.italic)), this.classes.length > 0 && (H = H || document.createElement("span"), H.className = createClass(this.classes));
      for (var V in this.style)
        this.style.hasOwnProperty(V) && (H = H || document.createElement("span"), H.style[V] = this.style[V]);
      return H ? (H.appendChild(_), H) : _;
    }
    /**
     * Creates markup for a symbol node.
     */
    toMarkup() {
      var _ = !1, H = "<span";
      this.classes.length && (_ = !0, H += ' class="', H += utils.escape(createClass(this.classes)), H += '"');
      var V = "";
      this.italic > 0 && (V += "margin-right:" + this.italic + "em;");
      for (var q in this.style)
        this.style.hasOwnProperty(q) && (V += utils.hyphenate(q) + ":" + this.style[q] + ";");
      V && (_ = !0, H += ' style="' + utils.escape(V) + '"');
      var G = utils.escape(this.text);
      return _ ? (H += ">", H += G, H += "</span>", H) : G;
    }
  }
  class SvgNode {
    constructor(_, H) {
      this.children = void 0, this.attributes = void 0, this.children = _ || [], this.attributes = H || {};
    }
    toNode() {
      var _ = "http://www.w3.org/2000/svg", H = document.createElementNS(_, "svg");
      for (var V in this.attributes)
        Object.prototype.hasOwnProperty.call(this.attributes, V) && H.setAttribute(V, this.attributes[V]);
      for (var q = 0; q < this.children.length; q++)
        H.appendChild(this.children[q].toNode());
      return H;
    }
    toMarkup() {
      var _ = '<svg xmlns="http://www.w3.org/2000/svg"';
      for (var H in this.attributes)
        Object.prototype.hasOwnProperty.call(this.attributes, H) && (_ += " " + H + '="' + utils.escape(this.attributes[H]) + '"');
      _ += ">";
      for (var V = 0; V < this.children.length; V++)
        _ += this.children[V].toMarkup();
      return _ += "</svg>", _;
    }
  }
  class PathNode {
    constructor(_, H) {
      this.pathName = void 0, this.alternate = void 0, this.pathName = _, this.alternate = H;
    }
    toNode() {
      var _ = "http://www.w3.org/2000/svg", H = document.createElementNS(_, "path");
      return this.alternate ? H.setAttribute("d", this.alternate) : H.setAttribute("d", path$4[this.pathName]), H;
    }
    toMarkup() {
      return this.alternate ? '<path d="' + utils.escape(this.alternate) + '"/>' : '<path d="' + utils.escape(path$4[this.pathName]) + '"/>';
    }
  }
  class LineNode {
    constructor(_) {
      this.attributes = void 0, this.attributes = _ || {};
    }
    toNode() {
      var _ = "http://www.w3.org/2000/svg", H = document.createElementNS(_, "line");
      for (var V in this.attributes)
        Object.prototype.hasOwnProperty.call(this.attributes, V) && H.setAttribute(V, this.attributes[V]);
      return H;
    }
    toMarkup() {
      var _ = "<line";
      for (var H in this.attributes)
        Object.prototype.hasOwnProperty.call(this.attributes, H) && (_ += " " + H + '="' + utils.escape(this.attributes[H]) + '"');
      return _ += "/>", _;
    }
  }
  function assertSymbolDomNode(O) {
    if (O instanceof SymbolNode)
      return O;
    throw new Error("Expected symbolNode but got " + String(O) + ".");
  }
  function assertSpan(O) {
    if (O instanceof Span$1)
      return O;
    throw new Error("Expected span<HtmlDomNode> but got " + String(O) + ".");
  }
  var ATOMS = {
    bin: 1,
    close: 1,
    inner: 1,
    open: 1,
    punct: 1,
    rel: 1
  }, NON_ATOMS = {
    "accent-token": 1,
    mathord: 1,
    "op-token": 1,
    spacing: 1,
    textord: 1
  }, symbols$3 = {
    math: {},
    text: {}
  };
  function defineSymbol(O, _, H, V, q, G) {
    symbols$3[O][q] = {
      font: _,
      group: H,
      replace: V
    }, G && V && (symbols$3[O][V] = symbols$3[O][q]);
  }
  var math = "math", text$3 = "text", main$3 = "main", ams = "ams", accent = "accent-token", bin$3 = "bin", close = "close", inner = "inner", mathord = "mathord", op = "op-token", open$1 = "open", punct = "punct", rel = "rel", spacing = "spacing", textord = "textord";
  defineSymbol(math, main$3, rel, "≡", "\\equiv", !0);
  defineSymbol(math, main$3, rel, "≺", "\\prec", !0);
  defineSymbol(math, main$3, rel, "≻", "\\succ", !0);
  defineSymbol(math, main$3, rel, "∼", "\\sim", !0);
  defineSymbol(math, main$3, rel, "⊥", "\\perp");
  defineSymbol(math, main$3, rel, "⪯", "\\preceq", !0);
  defineSymbol(math, main$3, rel, "⪰", "\\succeq", !0);
  defineSymbol(math, main$3, rel, "≃", "\\simeq", !0);
  defineSymbol(math, main$3, rel, "∣", "\\mid", !0);
  defineSymbol(math, main$3, rel, "≪", "\\ll", !0);
  defineSymbol(math, main$3, rel, "≫", "\\gg", !0);
  defineSymbol(math, main$3, rel, "≍", "\\asymp", !0);
  defineSymbol(math, main$3, rel, "∥", "\\parallel");
  defineSymbol(math, main$3, rel, "⋈", "\\bowtie", !0);
  defineSymbol(math, main$3, rel, "⌣", "\\smile", !0);
  defineSymbol(math, main$3, rel, "⊑", "\\sqsubseteq", !0);
  defineSymbol(math, main$3, rel, "⊒", "\\sqsupseteq", !0);
  defineSymbol(math, main$3, rel, "≐", "\\doteq", !0);
  defineSymbol(math, main$3, rel, "⌢", "\\frown", !0);
  defineSymbol(math, main$3, rel, "∋", "\\ni", !0);
  defineSymbol(math, main$3, rel, "∝", "\\propto", !0);
  defineSymbol(math, main$3, rel, "⊢", "\\vdash", !0);
  defineSymbol(math, main$3, rel, "⊣", "\\dashv", !0);
  defineSymbol(math, main$3, rel, "∋", "\\owns");
  defineSymbol(math, main$3, punct, ".", "\\ldotp");
  defineSymbol(math, main$3, punct, "⋅", "\\cdotp");
  defineSymbol(math, main$3, textord, "#", "\\#");
  defineSymbol(text$3, main$3, textord, "#", "\\#");
  defineSymbol(math, main$3, textord, "&", "\\&");
  defineSymbol(text$3, main$3, textord, "&", "\\&");
  defineSymbol(math, main$3, textord, "ℵ", "\\aleph", !0);
  defineSymbol(math, main$3, textord, "∀", "\\forall", !0);
  defineSymbol(math, main$3, textord, "ℏ", "\\hbar", !0);
  defineSymbol(math, main$3, textord, "∃", "\\exists", !0);
  defineSymbol(math, main$3, textord, "∇", "\\nabla", !0);
  defineSymbol(math, main$3, textord, "♭", "\\flat", !0);
  defineSymbol(math, main$3, textord, "ℓ", "\\ell", !0);
  defineSymbol(math, main$3, textord, "♮", "\\natural", !0);
  defineSymbol(math, main$3, textord, "♣", "\\clubsuit", !0);
  defineSymbol(math, main$3, textord, "℘", "\\wp", !0);
  defineSymbol(math, main$3, textord, "♯", "\\sharp", !0);
  defineSymbol(math, main$3, textord, "♢", "\\diamondsuit", !0);
  defineSymbol(math, main$3, textord, "ℜ", "\\Re", !0);
  defineSymbol(math, main$3, textord, "♡", "\\heartsuit", !0);
  defineSymbol(math, main$3, textord, "ℑ", "\\Im", !0);
  defineSymbol(math, main$3, textord, "♠", "\\spadesuit", !0);
  defineSymbol(math, main$3, textord, "§", "\\S", !0);
  defineSymbol(text$3, main$3, textord, "§", "\\S");
  defineSymbol(math, main$3, textord, "¶", "\\P", !0);
  defineSymbol(text$3, main$3, textord, "¶", "\\P");
  defineSymbol(math, main$3, textord, "†", "\\dag");
  defineSymbol(text$3, main$3, textord, "†", "\\dag");
  defineSymbol(text$3, main$3, textord, "†", "\\textdagger");
  defineSymbol(math, main$3, textord, "‡", "\\ddag");
  defineSymbol(text$3, main$3, textord, "‡", "\\ddag");
  defineSymbol(text$3, main$3, textord, "‡", "\\textdaggerdbl");
  defineSymbol(math, main$3, close, "⎱", "\\rmoustache", !0);
  defineSymbol(math, main$3, open$1, "⎰", "\\lmoustache", !0);
  defineSymbol(math, main$3, close, "⟯", "\\rgroup", !0);
  defineSymbol(math, main$3, open$1, "⟮", "\\lgroup", !0);
  defineSymbol(math, main$3, bin$3, "∓", "\\mp", !0);
  defineSymbol(math, main$3, bin$3, "⊖", "\\ominus", !0);
  defineSymbol(math, main$3, bin$3, "⊎", "\\uplus", !0);
  defineSymbol(math, main$3, bin$3, "⊓", "\\sqcap", !0);
  defineSymbol(math, main$3, bin$3, "∗", "\\ast");
  defineSymbol(math, main$3, bin$3, "⊔", "\\sqcup", !0);
  defineSymbol(math, main$3, bin$3, "◯", "\\bigcirc", !0);
  defineSymbol(math, main$3, bin$3, "∙", "\\bullet", !0);
  defineSymbol(math, main$3, bin$3, "‡", "\\ddagger");
  defineSymbol(math, main$3, bin$3, "≀", "\\wr", !0);
  defineSymbol(math, main$3, bin$3, "⨿", "\\amalg");
  defineSymbol(math, main$3, bin$3, "&", "\\And");
  defineSymbol(math, main$3, rel, "⟵", "\\longleftarrow", !0);
  defineSymbol(math, main$3, rel, "⇐", "\\Leftarrow", !0);
  defineSymbol(math, main$3, rel, "⟸", "\\Longleftarrow", !0);
  defineSymbol(math, main$3, rel, "⟶", "\\longrightarrow", !0);
  defineSymbol(math, main$3, rel, "⇒", "\\Rightarrow", !0);
  defineSymbol(math, main$3, rel, "⟹", "\\Longrightarrow", !0);
  defineSymbol(math, main$3, rel, "↔", "\\leftrightarrow", !0);
  defineSymbol(math, main$3, rel, "⟷", "\\longleftrightarrow", !0);
  defineSymbol(math, main$3, rel, "⇔", "\\Leftrightarrow", !0);
  defineSymbol(math, main$3, rel, "⟺", "\\Longleftrightarrow", !0);
  defineSymbol(math, main$3, rel, "↦", "\\mapsto", !0);
  defineSymbol(math, main$3, rel, "⟼", "\\longmapsto", !0);
  defineSymbol(math, main$3, rel, "↗", "\\nearrow", !0);
  defineSymbol(math, main$3, rel, "↩", "\\hookleftarrow", !0);
  defineSymbol(math, main$3, rel, "↪", "\\hookrightarrow", !0);
  defineSymbol(math, main$3, rel, "↘", "\\searrow", !0);
  defineSymbol(math, main$3, rel, "↼", "\\leftharpoonup", !0);
  defineSymbol(math, main$3, rel, "⇀", "\\rightharpoonup", !0);
  defineSymbol(math, main$3, rel, "↙", "\\swarrow", !0);
  defineSymbol(math, main$3, rel, "↽", "\\leftharpoondown", !0);
  defineSymbol(math, main$3, rel, "⇁", "\\rightharpoondown", !0);
  defineSymbol(math, main$3, rel, "↖", "\\nwarrow", !0);
  defineSymbol(math, main$3, rel, "⇌", "\\rightleftharpoons", !0);
  defineSymbol(math, ams, rel, "≮", "\\nless", !0);
  defineSymbol(math, ams, rel, "", "\\@nleqslant");
  defineSymbol(math, ams, rel, "", "\\@nleqq");
  defineSymbol(math, ams, rel, "⪇", "\\lneq", !0);
  defineSymbol(math, ams, rel, "≨", "\\lneqq", !0);
  defineSymbol(math, ams, rel, "", "\\@lvertneqq");
  defineSymbol(math, ams, rel, "⋦", "\\lnsim", !0);
  defineSymbol(math, ams, rel, "⪉", "\\lnapprox", !0);
  defineSymbol(math, ams, rel, "⊀", "\\nprec", !0);
  defineSymbol(math, ams, rel, "⋠", "\\npreceq", !0);
  defineSymbol(math, ams, rel, "⋨", "\\precnsim", !0);
  defineSymbol(math, ams, rel, "⪹", "\\precnapprox", !0);
  defineSymbol(math, ams, rel, "≁", "\\nsim", !0);
  defineSymbol(math, ams, rel, "", "\\@nshortmid");
  defineSymbol(math, ams, rel, "∤", "\\nmid", !0);
  defineSymbol(math, ams, rel, "⊬", "\\nvdash", !0);
  defineSymbol(math, ams, rel, "⊭", "\\nvDash", !0);
  defineSymbol(math, ams, rel, "⋪", "\\ntriangleleft");
  defineSymbol(math, ams, rel, "⋬", "\\ntrianglelefteq", !0);
  defineSymbol(math, ams, rel, "⊊", "\\subsetneq", !0);
  defineSymbol(math, ams, rel, "", "\\@varsubsetneq");
  defineSymbol(math, ams, rel, "⫋", "\\subsetneqq", !0);
  defineSymbol(math, ams, rel, "", "\\@varsubsetneqq");
  defineSymbol(math, ams, rel, "≯", "\\ngtr", !0);
  defineSymbol(math, ams, rel, "", "\\@ngeqslant");
  defineSymbol(math, ams, rel, "", "\\@ngeqq");
  defineSymbol(math, ams, rel, "⪈", "\\gneq", !0);
  defineSymbol(math, ams, rel, "≩", "\\gneqq", !0);
  defineSymbol(math, ams, rel, "", "\\@gvertneqq");
  defineSymbol(math, ams, rel, "⋧", "\\gnsim", !0);
  defineSymbol(math, ams, rel, "⪊", "\\gnapprox", !0);
  defineSymbol(math, ams, rel, "⊁", "\\nsucc", !0);
  defineSymbol(math, ams, rel, "⋡", "\\nsucceq", !0);
  defineSymbol(math, ams, rel, "⋩", "\\succnsim", !0);
  defineSymbol(math, ams, rel, "⪺", "\\succnapprox", !0);
  defineSymbol(math, ams, rel, "≆", "\\ncong", !0);
  defineSymbol(math, ams, rel, "", "\\@nshortparallel");
  defineSymbol(math, ams, rel, "∦", "\\nparallel", !0);
  defineSymbol(math, ams, rel, "⊯", "\\nVDash", !0);
  defineSymbol(math, ams, rel, "⋫", "\\ntriangleright");
  defineSymbol(math, ams, rel, "⋭", "\\ntrianglerighteq", !0);
  defineSymbol(math, ams, rel, "", "\\@nsupseteqq");
  defineSymbol(math, ams, rel, "⊋", "\\supsetneq", !0);
  defineSymbol(math, ams, rel, "", "\\@varsupsetneq");
  defineSymbol(math, ams, rel, "⫌", "\\supsetneqq", !0);
  defineSymbol(math, ams, rel, "", "\\@varsupsetneqq");
  defineSymbol(math, ams, rel, "⊮", "\\nVdash", !0);
  defineSymbol(math, ams, rel, "⪵", "\\precneqq", !0);
  defineSymbol(math, ams, rel, "⪶", "\\succneqq", !0);
  defineSymbol(math, ams, rel, "", "\\@nsubseteqq");
  defineSymbol(math, ams, bin$3, "⊴", "\\unlhd");
  defineSymbol(math, ams, bin$3, "⊵", "\\unrhd");
  defineSymbol(math, ams, rel, "↚", "\\nleftarrow", !0);
  defineSymbol(math, ams, rel, "↛", "\\nrightarrow", !0);
  defineSymbol(math, ams, rel, "⇍", "\\nLeftarrow", !0);
  defineSymbol(math, ams, rel, "⇏", "\\nRightarrow", !0);
  defineSymbol(math, ams, rel, "↮", "\\nleftrightarrow", !0);
  defineSymbol(math, ams, rel, "⇎", "\\nLeftrightarrow", !0);
  defineSymbol(math, ams, rel, "△", "\\vartriangle");
  defineSymbol(math, ams, textord, "ℏ", "\\hslash");
  defineSymbol(math, ams, textord, "▽", "\\triangledown");
  defineSymbol(math, ams, textord, "◊", "\\lozenge");
  defineSymbol(math, ams, textord, "Ⓢ", "\\circledS");
  defineSymbol(math, ams, textord, "®", "\\circledR");
  defineSymbol(text$3, ams, textord, "®", "\\circledR");
  defineSymbol(math, ams, textord, "∡", "\\measuredangle", !0);
  defineSymbol(math, ams, textord, "∄", "\\nexists");
  defineSymbol(math, ams, textord, "℧", "\\mho");
  defineSymbol(math, ams, textord, "Ⅎ", "\\Finv", !0);
  defineSymbol(math, ams, textord, "⅁", "\\Game", !0);
  defineSymbol(math, ams, textord, "‵", "\\backprime");
  defineSymbol(math, ams, textord, "▲", "\\blacktriangle");
  defineSymbol(math, ams, textord, "▼", "\\blacktriangledown");
  defineSymbol(math, ams, textord, "■", "\\blacksquare");
  defineSymbol(math, ams, textord, "⧫", "\\blacklozenge");
  defineSymbol(math, ams, textord, "★", "\\bigstar");
  defineSymbol(math, ams, textord, "∢", "\\sphericalangle", !0);
  defineSymbol(math, ams, textord, "∁", "\\complement", !0);
  defineSymbol(math, ams, textord, "ð", "\\eth", !0);
  defineSymbol(text$3, main$3, textord, "ð", "ð");
  defineSymbol(math, ams, textord, "╱", "\\diagup");
  defineSymbol(math, ams, textord, "╲", "\\diagdown");
  defineSymbol(math, ams, textord, "□", "\\square");
  defineSymbol(math, ams, textord, "□", "\\Box");
  defineSymbol(math, ams, textord, "◊", "\\Diamond");
  defineSymbol(math, ams, textord, "¥", "\\yen", !0);
  defineSymbol(text$3, ams, textord, "¥", "\\yen", !0);
  defineSymbol(math, ams, textord, "✓", "\\checkmark", !0);
  defineSymbol(text$3, ams, textord, "✓", "\\checkmark");
  defineSymbol(math, ams, textord, "ℶ", "\\beth", !0);
  defineSymbol(math, ams, textord, "ℸ", "\\daleth", !0);
  defineSymbol(math, ams, textord, "ℷ", "\\gimel", !0);
  defineSymbol(math, ams, textord, "ϝ", "\\digamma", !0);
  defineSymbol(math, ams, textord, "ϰ", "\\varkappa");
  defineSymbol(math, ams, open$1, "┌", "\\@ulcorner", !0);
  defineSymbol(math, ams, close, "┐", "\\@urcorner", !0);
  defineSymbol(math, ams, open$1, "└", "\\@llcorner", !0);
  defineSymbol(math, ams, close, "┘", "\\@lrcorner", !0);
  defineSymbol(math, ams, rel, "≦", "\\leqq", !0);
  defineSymbol(math, ams, rel, "⩽", "\\leqslant", !0);
  defineSymbol(math, ams, rel, "⪕", "\\eqslantless", !0);
  defineSymbol(math, ams, rel, "≲", "\\lesssim", !0);
  defineSymbol(math, ams, rel, "⪅", "\\lessapprox", !0);
  defineSymbol(math, ams, rel, "≊", "\\approxeq", !0);
  defineSymbol(math, ams, bin$3, "⋖", "\\lessdot");
  defineSymbol(math, ams, rel, "⋘", "\\lll", !0);
  defineSymbol(math, ams, rel, "≶", "\\lessgtr", !0);
  defineSymbol(math, ams, rel, "⋚", "\\lesseqgtr", !0);
  defineSymbol(math, ams, rel, "⪋", "\\lesseqqgtr", !0);
  defineSymbol(math, ams, rel, "≑", "\\doteqdot");
  defineSymbol(math, ams, rel, "≓", "\\risingdotseq", !0);
  defineSymbol(math, ams, rel, "≒", "\\fallingdotseq", !0);
  defineSymbol(math, ams, rel, "∽", "\\backsim", !0);
  defineSymbol(math, ams, rel, "⋍", "\\backsimeq", !0);
  defineSymbol(math, ams, rel, "⫅", "\\subseteqq", !0);
  defineSymbol(math, ams, rel, "⋐", "\\Subset", !0);
  defineSymbol(math, ams, rel, "⊏", "\\sqsubset", !0);
  defineSymbol(math, ams, rel, "≼", "\\preccurlyeq", !0);
  defineSymbol(math, ams, rel, "⋞", "\\curlyeqprec", !0);
  defineSymbol(math, ams, rel, "≾", "\\precsim", !0);
  defineSymbol(math, ams, rel, "⪷", "\\precapprox", !0);
  defineSymbol(math, ams, rel, "⊲", "\\vartriangleleft");
  defineSymbol(math, ams, rel, "⊴", "\\trianglelefteq");
  defineSymbol(math, ams, rel, "⊨", "\\vDash", !0);
  defineSymbol(math, ams, rel, "⊪", "\\Vvdash", !0);
  defineSymbol(math, ams, rel, "⌣", "\\smallsmile");
  defineSymbol(math, ams, rel, "⌢", "\\smallfrown");
  defineSymbol(math, ams, rel, "≏", "\\bumpeq", !0);
  defineSymbol(math, ams, rel, "≎", "\\Bumpeq", !0);
  defineSymbol(math, ams, rel, "≧", "\\geqq", !0);
  defineSymbol(math, ams, rel, "⩾", "\\geqslant", !0);
  defineSymbol(math, ams, rel, "⪖", "\\eqslantgtr", !0);
  defineSymbol(math, ams, rel, "≳", "\\gtrsim", !0);
  defineSymbol(math, ams, rel, "⪆", "\\gtrapprox", !0);
  defineSymbol(math, ams, bin$3, "⋗", "\\gtrdot");
  defineSymbol(math, ams, rel, "⋙", "\\ggg", !0);
  defineSymbol(math, ams, rel, "≷", "\\gtrless", !0);
  defineSymbol(math, ams, rel, "⋛", "\\gtreqless", !0);
  defineSymbol(math, ams, rel, "⪌", "\\gtreqqless", !0);
  defineSymbol(math, ams, rel, "≖", "\\eqcirc", !0);
  defineSymbol(math, ams, rel, "≗", "\\circeq", !0);
  defineSymbol(math, ams, rel, "≜", "\\triangleq", !0);
  defineSymbol(math, ams, rel, "∼", "\\thicksim");
  defineSymbol(math, ams, rel, "≈", "\\thickapprox");
  defineSymbol(math, ams, rel, "⫆", "\\supseteqq", !0);
  defineSymbol(math, ams, rel, "⋑", "\\Supset", !0);
  defineSymbol(math, ams, rel, "⊐", "\\sqsupset", !0);
  defineSymbol(math, ams, rel, "≽", "\\succcurlyeq", !0);
  defineSymbol(math, ams, rel, "⋟", "\\curlyeqsucc", !0);
  defineSymbol(math, ams, rel, "≿", "\\succsim", !0);
  defineSymbol(math, ams, rel, "⪸", "\\succapprox", !0);
  defineSymbol(math, ams, rel, "⊳", "\\vartriangleright");
  defineSymbol(math, ams, rel, "⊵", "\\trianglerighteq");
  defineSymbol(math, ams, rel, "⊩", "\\Vdash", !0);
  defineSymbol(math, ams, rel, "∣", "\\shortmid");
  defineSymbol(math, ams, rel, "∥", "\\shortparallel");
  defineSymbol(math, ams, rel, "≬", "\\between", !0);
  defineSymbol(math, ams, rel, "⋔", "\\pitchfork", !0);
  defineSymbol(math, ams, rel, "∝", "\\varpropto");
  defineSymbol(math, ams, rel, "◀", "\\blacktriangleleft");
  defineSymbol(math, ams, rel, "∴", "\\therefore", !0);
  defineSymbol(math, ams, rel, "∍", "\\backepsilon");
  defineSymbol(math, ams, rel, "▶", "\\blacktriangleright");
  defineSymbol(math, ams, rel, "∵", "\\because", !0);
  defineSymbol(math, ams, rel, "⋘", "\\llless");
  defineSymbol(math, ams, rel, "⋙", "\\gggtr");
  defineSymbol(math, ams, bin$3, "⊲", "\\lhd");
  defineSymbol(math, ams, bin$3, "⊳", "\\rhd");
  defineSymbol(math, ams, rel, "≂", "\\eqsim", !0);
  defineSymbol(math, main$3, rel, "⋈", "\\Join");
  defineSymbol(math, ams, rel, "≑", "\\Doteq", !0);
  defineSymbol(math, ams, bin$3, "∔", "\\dotplus", !0);
  defineSymbol(math, ams, bin$3, "∖", "\\smallsetminus");
  defineSymbol(math, ams, bin$3, "⋒", "\\Cap", !0);
  defineSymbol(math, ams, bin$3, "⋓", "\\Cup", !0);
  defineSymbol(math, ams, bin$3, "⩞", "\\doublebarwedge", !0);
  defineSymbol(math, ams, bin$3, "⊟", "\\boxminus", !0);
  defineSymbol(math, ams, bin$3, "⊞", "\\boxplus", !0);
  defineSymbol(math, ams, bin$3, "⋇", "\\divideontimes", !0);
  defineSymbol(math, ams, bin$3, "⋉", "\\ltimes", !0);
  defineSymbol(math, ams, bin$3, "⋊", "\\rtimes", !0);
  defineSymbol(math, ams, bin$3, "⋋", "\\leftthreetimes", !0);
  defineSymbol(math, ams, bin$3, "⋌", "\\rightthreetimes", !0);
  defineSymbol(math, ams, bin$3, "⋏", "\\curlywedge", !0);
  defineSymbol(math, ams, bin$3, "⋎", "\\curlyvee", !0);
  defineSymbol(math, ams, bin$3, "⊝", "\\circleddash", !0);
  defineSymbol(math, ams, bin$3, "⊛", "\\circledast", !0);
  defineSymbol(math, ams, bin$3, "⋅", "\\centerdot");
  defineSymbol(math, ams, bin$3, "⊺", "\\intercal", !0);
  defineSymbol(math, ams, bin$3, "⋒", "\\doublecap");
  defineSymbol(math, ams, bin$3, "⋓", "\\doublecup");
  defineSymbol(math, ams, bin$3, "⊠", "\\boxtimes", !0);
  defineSymbol(math, ams, rel, "⇢", "\\dashrightarrow", !0);
  defineSymbol(math, ams, rel, "⇠", "\\dashleftarrow", !0);
  defineSymbol(math, ams, rel, "⇇", "\\leftleftarrows", !0);
  defineSymbol(math, ams, rel, "⇆", "\\leftrightarrows", !0);
  defineSymbol(math, ams, rel, "⇚", "\\Lleftarrow", !0);
  defineSymbol(math, ams, rel, "↞", "\\twoheadleftarrow", !0);
  defineSymbol(math, ams, rel, "↢", "\\leftarrowtail", !0);
  defineSymbol(math, ams, rel, "↫", "\\looparrowleft", !0);
  defineSymbol(math, ams, rel, "⇋", "\\leftrightharpoons", !0);
  defineSymbol(math, ams, rel, "↶", "\\curvearrowleft", !0);
  defineSymbol(math, ams, rel, "↺", "\\circlearrowleft", !0);
  defineSymbol(math, ams, rel, "↰", "\\Lsh", !0);
  defineSymbol(math, ams, rel, "⇈", "\\upuparrows", !0);
  defineSymbol(math, ams, rel, "↿", "\\upharpoonleft", !0);
  defineSymbol(math, ams, rel, "⇃", "\\downharpoonleft", !0);
  defineSymbol(math, main$3, rel, "⊶", "\\origof", !0);
  defineSymbol(math, main$3, rel, "⊷", "\\imageof", !0);
  defineSymbol(math, ams, rel, "⊸", "\\multimap", !0);
  defineSymbol(math, ams, rel, "↭", "\\leftrightsquigarrow", !0);
  defineSymbol(math, ams, rel, "⇉", "\\rightrightarrows", !0);
  defineSymbol(math, ams, rel, "⇄", "\\rightleftarrows", !0);
  defineSymbol(math, ams, rel, "↠", "\\twoheadrightarrow", !0);
  defineSymbol(math, ams, rel, "↣", "\\rightarrowtail", !0);
  defineSymbol(math, ams, rel, "↬", "\\looparrowright", !0);
  defineSymbol(math, ams, rel, "↷", "\\curvearrowright", !0);
  defineSymbol(math, ams, rel, "↻", "\\circlearrowright", !0);
  defineSymbol(math, ams, rel, "↱", "\\Rsh", !0);
  defineSymbol(math, ams, rel, "⇊", "\\downdownarrows", !0);
  defineSymbol(math, ams, rel, "↾", "\\upharpoonright", !0);
  defineSymbol(math, ams, rel, "⇂", "\\downharpoonright", !0);
  defineSymbol(math, ams, rel, "⇝", "\\rightsquigarrow", !0);
  defineSymbol(math, ams, rel, "⇝", "\\leadsto");
  defineSymbol(math, ams, rel, "⇛", "\\Rrightarrow", !0);
  defineSymbol(math, ams, rel, "↾", "\\restriction");
  defineSymbol(math, main$3, textord, "‘", "`");
  defineSymbol(math, main$3, textord, "$", "\\$");
  defineSymbol(text$3, main$3, textord, "$", "\\$");
  defineSymbol(text$3, main$3, textord, "$", "\\textdollar");
  defineSymbol(math, main$3, textord, "%", "\\%");
  defineSymbol(text$3, main$3, textord, "%", "\\%");
  defineSymbol(math, main$3, textord, "_", "\\_");
  defineSymbol(text$3, main$3, textord, "_", "\\_");
  defineSymbol(text$3, main$3, textord, "_", "\\textunderscore");
  defineSymbol(math, main$3, textord, "∠", "\\angle", !0);
  defineSymbol(math, main$3, textord, "∞", "\\infty", !0);
  defineSymbol(math, main$3, textord, "′", "\\prime");
  defineSymbol(math, main$3, textord, "△", "\\triangle");
  defineSymbol(math, main$3, textord, "Γ", "\\Gamma", !0);
  defineSymbol(math, main$3, textord, "Δ", "\\Delta", !0);
  defineSymbol(math, main$3, textord, "Θ", "\\Theta", !0);
  defineSymbol(math, main$3, textord, "Λ", "\\Lambda", !0);
  defineSymbol(math, main$3, textord, "Ξ", "\\Xi", !0);
  defineSymbol(math, main$3, textord, "Π", "\\Pi", !0);
  defineSymbol(math, main$3, textord, "Σ", "\\Sigma", !0);
  defineSymbol(math, main$3, textord, "Υ", "\\Upsilon", !0);
  defineSymbol(math, main$3, textord, "Φ", "\\Phi", !0);
  defineSymbol(math, main$3, textord, "Ψ", "\\Psi", !0);
  defineSymbol(math, main$3, textord, "Ω", "\\Omega", !0);
  defineSymbol(math, main$3, textord, "A", "Α");
  defineSymbol(math, main$3, textord, "B", "Β");
  defineSymbol(math, main$3, textord, "E", "Ε");
  defineSymbol(math, main$3, textord, "Z", "Ζ");
  defineSymbol(math, main$3, textord, "H", "Η");
  defineSymbol(math, main$3, textord, "I", "Ι");
  defineSymbol(math, main$3, textord, "K", "Κ");
  defineSymbol(math, main$3, textord, "M", "Μ");
  defineSymbol(math, main$3, textord, "N", "Ν");
  defineSymbol(math, main$3, textord, "O", "Ο");
  defineSymbol(math, main$3, textord, "P", "Ρ");
  defineSymbol(math, main$3, textord, "T", "Τ");
  defineSymbol(math, main$3, textord, "X", "Χ");
  defineSymbol(math, main$3, textord, "¬", "\\neg", !0);
  defineSymbol(math, main$3, textord, "¬", "\\lnot");
  defineSymbol(math, main$3, textord, "⊤", "\\top");
  defineSymbol(math, main$3, textord, "⊥", "\\bot");
  defineSymbol(math, main$3, textord, "∅", "\\emptyset");
  defineSymbol(math, ams, textord, "∅", "\\varnothing");
  defineSymbol(math, main$3, mathord, "α", "\\alpha", !0);
  defineSymbol(math, main$3, mathord, "β", "\\beta", !0);
  defineSymbol(math, main$3, mathord, "γ", "\\gamma", !0);
  defineSymbol(math, main$3, mathord, "δ", "\\delta", !0);
  defineSymbol(math, main$3, mathord, "ϵ", "\\epsilon", !0);
  defineSymbol(math, main$3, mathord, "ζ", "\\zeta", !0);
  defineSymbol(math, main$3, mathord, "η", "\\eta", !0);
  defineSymbol(math, main$3, mathord, "θ", "\\theta", !0);
  defineSymbol(math, main$3, mathord, "ι", "\\iota", !0);
  defineSymbol(math, main$3, mathord, "κ", "\\kappa", !0);
  defineSymbol(math, main$3, mathord, "λ", "\\lambda", !0);
  defineSymbol(math, main$3, mathord, "μ", "\\mu", !0);
  defineSymbol(math, main$3, mathord, "ν", "\\nu", !0);
  defineSymbol(math, main$3, mathord, "ξ", "\\xi", !0);
  defineSymbol(math, main$3, mathord, "ο", "\\omicron", !0);
  defineSymbol(math, main$3, mathord, "π", "\\pi", !0);
  defineSymbol(math, main$3, mathord, "ρ", "\\rho", !0);
  defineSymbol(math, main$3, mathord, "σ", "\\sigma", !0);
  defineSymbol(math, main$3, mathord, "τ", "\\tau", !0);
  defineSymbol(math, main$3, mathord, "υ", "\\upsilon", !0);
  defineSymbol(math, main$3, mathord, "ϕ", "\\phi", !0);
  defineSymbol(math, main$3, mathord, "χ", "\\chi", !0);
  defineSymbol(math, main$3, mathord, "ψ", "\\psi", !0);
  defineSymbol(math, main$3, mathord, "ω", "\\omega", !0);
  defineSymbol(math, main$3, mathord, "ε", "\\varepsilon", !0);
  defineSymbol(math, main$3, mathord, "ϑ", "\\vartheta", !0);
  defineSymbol(math, main$3, mathord, "ϖ", "\\varpi", !0);
  defineSymbol(math, main$3, mathord, "ϱ", "\\varrho", !0);
  defineSymbol(math, main$3, mathord, "ς", "\\varsigma", !0);
  defineSymbol(math, main$3, mathord, "φ", "\\varphi", !0);
  defineSymbol(math, main$3, bin$3, "∗", "*", !0);
  defineSymbol(math, main$3, bin$3, "+", "+");
  defineSymbol(math, main$3, bin$3, "−", "-", !0);
  defineSymbol(math, main$3, bin$3, "⋅", "\\cdot", !0);
  defineSymbol(math, main$3, bin$3, "∘", "\\circ", !0);
  defineSymbol(math, main$3, bin$3, "÷", "\\div", !0);
  defineSymbol(math, main$3, bin$3, "±", "\\pm", !0);
  defineSymbol(math, main$3, bin$3, "×", "\\times", !0);
  defineSymbol(math, main$3, bin$3, "∩", "\\cap", !0);
  defineSymbol(math, main$3, bin$3, "∪", "\\cup", !0);
  defineSymbol(math, main$3, bin$3, "∖", "\\setminus", !0);
  defineSymbol(math, main$3, bin$3, "∧", "\\land");
  defineSymbol(math, main$3, bin$3, "∨", "\\lor");
  defineSymbol(math, main$3, bin$3, "∧", "\\wedge", !0);
  defineSymbol(math, main$3, bin$3, "∨", "\\vee", !0);
  defineSymbol(math, main$3, textord, "√", "\\surd");
  defineSymbol(math, main$3, open$1, "⟨", "\\langle", !0);
  defineSymbol(math, main$3, open$1, "∣", "\\lvert");
  defineSymbol(math, main$3, open$1, "∥", "\\lVert");
  defineSymbol(math, main$3, close, "?", "?");
  defineSymbol(math, main$3, close, "!", "!");
  defineSymbol(math, main$3, close, "⟩", "\\rangle", !0);
  defineSymbol(math, main$3, close, "∣", "\\rvert");
  defineSymbol(math, main$3, close, "∥", "\\rVert");
  defineSymbol(math, main$3, rel, "=", "=");
  defineSymbol(math, main$3, rel, ":", ":");
  defineSymbol(math, main$3, rel, "≈", "\\approx", !0);
  defineSymbol(math, main$3, rel, "≅", "\\cong", !0);
  defineSymbol(math, main$3, rel, "≥", "\\ge");
  defineSymbol(math, main$3, rel, "≥", "\\geq", !0);
  defineSymbol(math, main$3, rel, "←", "\\gets");
  defineSymbol(math, main$3, rel, ">", "\\gt", !0);
  defineSymbol(math, main$3, rel, "∈", "\\in", !0);
  defineSymbol(math, main$3, rel, "", "\\@not");
  defineSymbol(math, main$3, rel, "⊂", "\\subset", !0);
  defineSymbol(math, main$3, rel, "⊃", "\\supset", !0);
  defineSymbol(math, main$3, rel, "⊆", "\\subseteq", !0);
  defineSymbol(math, main$3, rel, "⊇", "\\supseteq", !0);
  defineSymbol(math, ams, rel, "⊈", "\\nsubseteq", !0);
  defineSymbol(math, ams, rel, "⊉", "\\nsupseteq", !0);
  defineSymbol(math, main$3, rel, "⊨", "\\models");
  defineSymbol(math, main$3, rel, "←", "\\leftarrow", !0);
  defineSymbol(math, main$3, rel, "≤", "\\le");
  defineSymbol(math, main$3, rel, "≤", "\\leq", !0);
  defineSymbol(math, main$3, rel, "<", "\\lt", !0);
  defineSymbol(math, main$3, rel, "→", "\\rightarrow", !0);
  defineSymbol(math, main$3, rel, "→", "\\to");
  defineSymbol(math, ams, rel, "≱", "\\ngeq", !0);
  defineSymbol(math, ams, rel, "≰", "\\nleq", !0);
  defineSymbol(math, main$3, spacing, " ", "\\ ");
  defineSymbol(math, main$3, spacing, " ", "\\space");
  defineSymbol(math, main$3, spacing, " ", "\\nobreakspace");
  defineSymbol(text$3, main$3, spacing, " ", "\\ ");
  defineSymbol(text$3, main$3, spacing, " ", " ");
  defineSymbol(text$3, main$3, spacing, " ", "\\space");
  defineSymbol(text$3, main$3, spacing, " ", "\\nobreakspace");
  defineSymbol(math, main$3, spacing, null, "\\nobreak");
  defineSymbol(math, main$3, spacing, null, "\\allowbreak");
  defineSymbol(math, main$3, punct, ",", ",");
  defineSymbol(math, main$3, punct, ";", ";");
  defineSymbol(math, ams, bin$3, "⊼", "\\barwedge", !0);
  defineSymbol(math, ams, bin$3, "⊻", "\\veebar", !0);
  defineSymbol(math, main$3, bin$3, "⊙", "\\odot", !0);
  defineSymbol(math, main$3, bin$3, "⊕", "\\oplus", !0);
  defineSymbol(math, main$3, bin$3, "⊗", "\\otimes", !0);
  defineSymbol(math, main$3, textord, "∂", "\\partial", !0);
  defineSymbol(math, main$3, bin$3, "⊘", "\\oslash", !0);
  defineSymbol(math, ams, bin$3, "⊚", "\\circledcirc", !0);
  defineSymbol(math, ams, bin$3, "⊡", "\\boxdot", !0);
  defineSymbol(math, main$3, bin$3, "△", "\\bigtriangleup");
  defineSymbol(math, main$3, bin$3, "▽", "\\bigtriangledown");
  defineSymbol(math, main$3, bin$3, "†", "\\dagger");
  defineSymbol(math, main$3, bin$3, "⋄", "\\diamond");
  defineSymbol(math, main$3, bin$3, "⋆", "\\star");
  defineSymbol(math, main$3, bin$3, "◃", "\\triangleleft");
  defineSymbol(math, main$3, bin$3, "▹", "\\triangleright");
  defineSymbol(math, main$3, open$1, "{", "\\{");
  defineSymbol(text$3, main$3, textord, "{", "\\{");
  defineSymbol(text$3, main$3, textord, "{", "\\textbraceleft");
  defineSymbol(math, main$3, close, "}", "\\}");
  defineSymbol(text$3, main$3, textord, "}", "\\}");
  defineSymbol(text$3, main$3, textord, "}", "\\textbraceright");
  defineSymbol(math, main$3, open$1, "{", "\\lbrace");
  defineSymbol(math, main$3, close, "}", "\\rbrace");
  defineSymbol(math, main$3, open$1, "[", "\\lbrack", !0);
  defineSymbol(text$3, main$3, textord, "[", "\\lbrack", !0);
  defineSymbol(math, main$3, close, "]", "\\rbrack", !0);
  defineSymbol(text$3, main$3, textord, "]", "\\rbrack", !0);
  defineSymbol(math, main$3, open$1, "(", "\\lparen", !0);
  defineSymbol(math, main$3, close, ")", "\\rparen", !0);
  defineSymbol(text$3, main$3, textord, "<", "\\textless", !0);
  defineSymbol(text$3, main$3, textord, ">", "\\textgreater", !0);
  defineSymbol(math, main$3, open$1, "⌊", "\\lfloor", !0);
  defineSymbol(math, main$3, close, "⌋", "\\rfloor", !0);
  defineSymbol(math, main$3, open$1, "⌈", "\\lceil", !0);
  defineSymbol(math, main$3, close, "⌉", "\\rceil", !0);
  defineSymbol(math, main$3, textord, "\\", "\\backslash");
  defineSymbol(math, main$3, textord, "∣", "|");
  defineSymbol(math, main$3, textord, "∣", "\\vert");
  defineSymbol(text$3, main$3, textord, "|", "\\textbar", !0);
  defineSymbol(math, main$3, textord, "∥", "\\|");
  defineSymbol(math, main$3, textord, "∥", "\\Vert");
  defineSymbol(text$3, main$3, textord, "∥", "\\textbardbl");
  defineSymbol(text$3, main$3, textord, "~", "\\textasciitilde");
  defineSymbol(text$3, main$3, textord, "\\", "\\textbackslash");
  defineSymbol(text$3, main$3, textord, "^", "\\textasciicircum");
  defineSymbol(math, main$3, rel, "↑", "\\uparrow", !0);
  defineSymbol(math, main$3, rel, "⇑", "\\Uparrow", !0);
  defineSymbol(math, main$3, rel, "↓", "\\downarrow", !0);
  defineSymbol(math, main$3, rel, "⇓", "\\Downarrow", !0);
  defineSymbol(math, main$3, rel, "↕", "\\updownarrow", !0);
  defineSymbol(math, main$3, rel, "⇕", "\\Updownarrow", !0);
  defineSymbol(math, main$3, op, "∐", "\\coprod");
  defineSymbol(math, main$3, op, "⋁", "\\bigvee");
  defineSymbol(math, main$3, op, "⋀", "\\bigwedge");
  defineSymbol(math, main$3, op, "⨄", "\\biguplus");
  defineSymbol(math, main$3, op, "⋂", "\\bigcap");
  defineSymbol(math, main$3, op, "⋃", "\\bigcup");
  defineSymbol(math, main$3, op, "∫", "\\int");
  defineSymbol(math, main$3, op, "∫", "\\intop");
  defineSymbol(math, main$3, op, "∬", "\\iint");
  defineSymbol(math, main$3, op, "∭", "\\iiint");
  defineSymbol(math, main$3, op, "∏", "\\prod");
  defineSymbol(math, main$3, op, "∑", "\\sum");
  defineSymbol(math, main$3, op, "⨂", "\\bigotimes");
  defineSymbol(math, main$3, op, "⨁", "\\bigoplus");
  defineSymbol(math, main$3, op, "⨀", "\\bigodot");
  defineSymbol(math, main$3, op, "∮", "\\oint");
  defineSymbol(math, main$3, op, "∯", "\\oiint");
  defineSymbol(math, main$3, op, "∰", "\\oiiint");
  defineSymbol(math, main$3, op, "⨆", "\\bigsqcup");
  defineSymbol(math, main$3, op, "∫", "\\smallint");
  defineSymbol(text$3, main$3, inner, "…", "\\textellipsis");
  defineSymbol(math, main$3, inner, "…", "\\mathellipsis");
  defineSymbol(text$3, main$3, inner, "…", "\\ldots", !0);
  defineSymbol(math, main$3, inner, "…", "\\ldots", !0);
  defineSymbol(math, main$3, inner, "⋯", "\\@cdots", !0);
  defineSymbol(math, main$3, inner, "⋱", "\\ddots", !0);
  defineSymbol(math, main$3, textord, "⋮", "\\varvdots");
  defineSymbol(text$3, main$3, textord, "⋮", "\\varvdots");
  defineSymbol(math, main$3, accent, "ˊ", "\\acute");
  defineSymbol(math, main$3, accent, "ˋ", "\\grave");
  defineSymbol(math, main$3, accent, "¨", "\\ddot");
  defineSymbol(math, main$3, accent, "~", "\\tilde");
  defineSymbol(math, main$3, accent, "ˉ", "\\bar");
  defineSymbol(math, main$3, accent, "˘", "\\breve");
  defineSymbol(math, main$3, accent, "ˇ", "\\check");
  defineSymbol(math, main$3, accent, "^", "\\hat");
  defineSymbol(math, main$3, accent, "⃗", "\\vec");
  defineSymbol(math, main$3, accent, "˙", "\\dot");
  defineSymbol(math, main$3, accent, "˚", "\\mathring");
  defineSymbol(math, main$3, mathord, "", "\\@imath");
  defineSymbol(math, main$3, mathord, "", "\\@jmath");
  defineSymbol(math, main$3, textord, "ı", "ı");
  defineSymbol(math, main$3, textord, "ȷ", "ȷ");
  defineSymbol(text$3, main$3, textord, "ı", "\\i", !0);
  defineSymbol(text$3, main$3, textord, "ȷ", "\\j", !0);
  defineSymbol(text$3, main$3, textord, "ß", "\\ss", !0);
  defineSymbol(text$3, main$3, textord, "æ", "\\ae", !0);
  defineSymbol(text$3, main$3, textord, "œ", "\\oe", !0);
  defineSymbol(text$3, main$3, textord, "ø", "\\o", !0);
  defineSymbol(text$3, main$3, textord, "Æ", "\\AE", !0);
  defineSymbol(text$3, main$3, textord, "Œ", "\\OE", !0);
  defineSymbol(text$3, main$3, textord, "Ø", "\\O", !0);
  defineSymbol(text$3, main$3, accent, "ˊ", "\\'");
  defineSymbol(text$3, main$3, accent, "ˋ", "\\`");
  defineSymbol(text$3, main$3, accent, "ˆ", "\\^");
  defineSymbol(text$3, main$3, accent, "˜", "\\~");
  defineSymbol(text$3, main$3, accent, "ˉ", "\\=");
  defineSymbol(text$3, main$3, accent, "˘", "\\u");
  defineSymbol(text$3, main$3, accent, "˙", "\\.");
  defineSymbol(text$3, main$3, accent, "¸", "\\c");
  defineSymbol(text$3, main$3, accent, "˚", "\\r");
  defineSymbol(text$3, main$3, accent, "ˇ", "\\v");
  defineSymbol(text$3, main$3, accent, "¨", '\\"');
  defineSymbol(text$3, main$3, accent, "˝", "\\H");
  defineSymbol(text$3, main$3, accent, "◯", "\\textcircled");
  var ligatures = {
    "--": !0,
    "---": !0,
    "``": !0,
    "''": !0
  };
  defineSymbol(text$3, main$3, textord, "–", "--", !0);
  defineSymbol(text$3, main$3, textord, "–", "\\textendash");
  defineSymbol(text$3, main$3, textord, "—", "---", !0);
  defineSymbol(text$3, main$3, textord, "—", "\\textemdash");
  defineSymbol(text$3, main$3, textord, "‘", "`", !0);
  defineSymbol(text$3, main$3, textord, "‘", "\\textquoteleft");
  defineSymbol(text$3, main$3, textord, "’", "'", !0);
  defineSymbol(text$3, main$3, textord, "’", "\\textquoteright");
  defineSymbol(text$3, main$3, textord, "“", "``", !0);
  defineSymbol(text$3, main$3, textord, "“", "\\textquotedblleft");
  defineSymbol(text$3, main$3, textord, "”", "''", !0);
  defineSymbol(text$3, main$3, textord, "”", "\\textquotedblright");
  defineSymbol(math, main$3, textord, "°", "\\degree", !0);
  defineSymbol(text$3, main$3, textord, "°", "\\degree");
  defineSymbol(text$3, main$3, textord, "°", "\\textdegree", !0);
  defineSymbol(math, main$3, textord, "£", "\\pounds");
  defineSymbol(math, main$3, textord, "£", "\\mathsterling", !0);
  defineSymbol(text$3, main$3, textord, "£", "\\pounds");
  defineSymbol(text$3, main$3, textord, "£", "\\textsterling", !0);
  defineSymbol(math, ams, textord, "✠", "\\maltese");
  defineSymbol(text$3, ams, textord, "✠", "\\maltese");
  var mathTextSymbols = '0123456789/@."';
  for (var i$1 = 0; i$1 < mathTextSymbols.length; i$1++) {
    var ch$1 = mathTextSymbols.charAt(i$1);
    defineSymbol(math, main$3, textord, ch$1, ch$1);
  }
  var textSymbols = '0123456789!@*()-=+";:?/.,';
  for (var _i = 0; _i < textSymbols.length; _i++) {
    var _ch = textSymbols.charAt(_i);
    defineSymbol(text$3, main$3, textord, _ch, _ch);
  }
  var letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
  for (var _i2 = 0; _i2 < letters.length; _i2++) {
    var _ch2 = letters.charAt(_i2);
    defineSymbol(math, main$3, mathord, _ch2, _ch2), defineSymbol(text$3, main$3, textord, _ch2, _ch2);
  }
  defineSymbol(math, ams, textord, "C", "ℂ");
  defineSymbol(text$3, ams, textord, "C", "ℂ");
  defineSymbol(math, ams, textord, "H", "ℍ");
  defineSymbol(text$3, ams, textord, "H", "ℍ");
  defineSymbol(math, ams, textord, "N", "ℕ");
  defineSymbol(text$3, ams, textord, "N", "ℕ");
  defineSymbol(math, ams, textord, "P", "ℙ");
  defineSymbol(text$3, ams, textord, "P", "ℙ");
  defineSymbol(math, ams, textord, "Q", "ℚ");
  defineSymbol(text$3, ams, textord, "Q", "ℚ");
  defineSymbol(math, ams, textord, "R", "ℝ");
  defineSymbol(text$3, ams, textord, "R", "ℝ");
  defineSymbol(math, ams, textord, "Z", "ℤ");
  defineSymbol(text$3, ams, textord, "Z", "ℤ");
  defineSymbol(math, main$3, mathord, "h", "ℎ");
  defineSymbol(text$3, main$3, mathord, "h", "ℎ");
  var wideChar = "";
  for (var _i3 = 0; _i3 < letters.length; _i3++) {
    var _ch3 = letters.charAt(_i3);
    wideChar = String.fromCharCode(55349, 56320 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56372 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56424 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56580 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56684 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56736 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56788 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56840 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56944 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), _i3 < 26 && (wideChar = String.fromCharCode(55349, 56632 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar), wideChar = String.fromCharCode(55349, 56476 + _i3), defineSymbol(math, main$3, mathord, _ch3, wideChar), defineSymbol(text$3, main$3, textord, _ch3, wideChar));
  }
  wideChar = "𝕜";
  defineSymbol(math, main$3, mathord, "k", wideChar);
  defineSymbol(text$3, main$3, textord, "k", wideChar);
  for (var _i4 = 0; _i4 < 10; _i4++) {
    var _ch4 = _i4.toString();
    wideChar = String.fromCharCode(55349, 57294 + _i4), defineSymbol(math, main$3, mathord, _ch4, wideChar), defineSymbol(text$3, main$3, textord, _ch4, wideChar), wideChar = String.fromCharCode(55349, 57314 + _i4), defineSymbol(math, main$3, mathord, _ch4, wideChar), defineSymbol(text$3, main$3, textord, _ch4, wideChar), wideChar = String.fromCharCode(55349, 57324 + _i4), defineSymbol(math, main$3, mathord, _ch4, wideChar), defineSymbol(text$3, main$3, textord, _ch4, wideChar), wideChar = String.fromCharCode(55349, 57334 + _i4), defineSymbol(math, main$3, mathord, _ch4, wideChar), defineSymbol(text$3, main$3, textord, _ch4, wideChar);
  }
  var extraLatin = "ÐÞþ";
  for (var _i5 = 0; _i5 < extraLatin.length; _i5++) {
    var _ch5 = extraLatin.charAt(_i5);
    defineSymbol(math, main$3, mathord, _ch5, _ch5), defineSymbol(text$3, main$3, textord, _ch5, _ch5);
  }
  var wideLatinLetterData = [
    ["mathbf", "textbf", "Main-Bold"],
    // A-Z bold upright
    ["mathbf", "textbf", "Main-Bold"],
    // a-z bold upright
    ["mathnormal", "textit", "Math-Italic"],
    // A-Z italic
    ["mathnormal", "textit", "Math-Italic"],
    // a-z italic
    ["boldsymbol", "boldsymbol", "Main-BoldItalic"],
    // A-Z bold italic
    ["boldsymbol", "boldsymbol", "Main-BoldItalic"],
    // a-z bold italic
    // Map fancy A-Z letters to script, not calligraphic.
    // This aligns with unicode-math and math fonts (except Cambria Math).
    ["mathscr", "textscr", "Script-Regular"],
    // A-Z script
    ["", "", ""],
    // a-z script.  No font
    ["", "", ""],
    // A-Z bold script. No font
    ["", "", ""],
    // a-z bold script. No font
    ["mathfrak", "textfrak", "Fraktur-Regular"],
    // A-Z Fraktur
    ["mathfrak", "textfrak", "Fraktur-Regular"],
    // a-z Fraktur
    ["mathbb", "textbb", "AMS-Regular"],
    // A-Z double-struck
    ["mathbb", "textbb", "AMS-Regular"],
    // k double-struck
    // Note that we are using a bold font, but font metrics for regular Fraktur.
    ["mathboldfrak", "textboldfrak", "Fraktur-Regular"],
    // A-Z bold Fraktur
    ["mathboldfrak", "textboldfrak", "Fraktur-Regular"],
    // a-z bold Fraktur
    ["mathsf", "textsf", "SansSerif-Regular"],
    // A-Z sans-serif
    ["mathsf", "textsf", "SansSerif-Regular"],
    // a-z sans-serif
    ["mathboldsf", "textboldsf", "SansSerif-Bold"],
    // A-Z bold sans-serif
    ["mathboldsf", "textboldsf", "SansSerif-Bold"],
    // a-z bold sans-serif
    ["mathitsf", "textitsf", "SansSerif-Italic"],
    // A-Z italic sans-serif
    ["mathitsf", "textitsf", "SansSerif-Italic"],
    // a-z italic sans-serif
    ["", "", ""],
    // A-Z bold italic sans. No font
    ["", "", ""],
    // a-z bold italic sans. No font
    ["mathtt", "texttt", "Typewriter-Regular"],
    // A-Z monospace
    ["mathtt", "texttt", "Typewriter-Regular"]
    // a-z monospace
  ], wideNumeralData = [
    ["mathbf", "textbf", "Main-Bold"],
    // 0-9 bold
    ["", "", ""],
    // 0-9 double-struck. No KaTeX font.
    ["mathsf", "textsf", "SansSerif-Regular"],
    // 0-9 sans-serif
    ["mathboldsf", "textboldsf", "SansSerif-Bold"],
    // 0-9 bold sans-serif
    ["mathtt", "texttt", "Typewriter-Regular"]
    // 0-9 monospace
  ], wideCharacterFont = function O(_, H) {
    var V = _.charCodeAt(0), q = _.charCodeAt(1), G = (V - 55296) * 1024 + (q - 56320) + 65536, J = H === "math" ? 0 : 1;
    if (119808 <= G && G < 120484) {
      var Z = Math.floor((G - 119808) / 26);
      return [wideLatinLetterData[Z][2], wideLatinLetterData[Z][J]];
    } else if (120782 <= G && G <= 120831) {
      var te = Math.floor((G - 120782) / 10);
      return [wideNumeralData[te][2], wideNumeralData[te][J]];
    } else {
      if (G === 120485 || G === 120486)
        return [wideLatinLetterData[0][2], wideLatinLetterData[0][J]];
      if (120486 < G && G < 120782)
        return ["", ""];
      throw new ParseError("Unsupported character: " + _);
    }
  }, lookupSymbol = function O(_, H, V) {
    return symbols$3[V][_] && symbols$3[V][_].replace && (_ = symbols$3[V][_].replace), {
      value: _,
      metrics: getCharacterMetrics(_, H, V)
    };
  }, makeSymbol = function O(_, H, V, q, G) {
    var J = lookupSymbol(_, H, V), Z = J.metrics;
    _ = J.value;
    var te;
    if (Z) {
      var ne = Z.italic;
      (V === "text" || q && q.font === "mathit") && (ne = 0), te = new SymbolNode(_, Z.height, Z.depth, ne, Z.skew, Z.width, G);
    } else
      typeof console < "u" && console.warn("No character metrics " + ("for '" + _ + "' in style '" + H + "' and mode '" + V + "'")), te = new SymbolNode(_, 0, 0, 0, 0, 0, G);
    if (q) {
      te.maxFontSize = q.sizeMultiplier, q.style.isTight() && te.classes.push("mtight");
      var ee = q.getColor();
      ee && (te.style.color = ee);
    }
    return te;
  }, mathsym = function O(_, H, V, q) {
    return q === void 0 && (q = []), V.font === "boldsymbol" && lookupSymbol(_, "Main-Bold", H).metrics ? makeSymbol(_, "Main-Bold", H, V, q.concat(["mathbf"])) : _ === "\\" || symbols$3[H][_].font === "main" ? makeSymbol(_, "Main-Regular", H, V, q) : makeSymbol(_, "AMS-Regular", H, V, q.concat(["amsrm"]));
  }, boldsymbol = function O(_, H, V, q, G) {
    return G !== "textord" && lookupSymbol(_, "Math-BoldItalic", H).metrics ? {
      fontName: "Math-BoldItalic",
      fontClass: "boldsymbol"
    } : {
      fontName: "Main-Bold",
      fontClass: "mathbf"
    };
  }, makeOrd = function O(_, H, V) {
    var q = _.mode, G = _.text, J = ["mord"], Z = q === "math" || q === "text" && H.font, te = Z ? H.font : H.fontFamily, ne = "", ee = "";
    if (G.charCodeAt(0) === 55349 && ([ne, ee] = wideCharacterFont(G, q)), ne.length > 0)
      return makeSymbol(G, ne, q, H, J.concat(ee));
    if (te) {
      var ae, oe;
      if (te === "boldsymbol") {
        var se = boldsymbol(G, q, H, J, V);
        ae = se.fontName, oe = [se.fontClass];
      } else Z ? (ae = fontMap[te].fontName, oe = [te]) : (ae = retrieveTextFontName(te, H.fontWeight, H.fontShape), oe = [te, H.fontWeight, H.fontShape]);
      if (lookupSymbol(G, ae, q).metrics)
        return makeSymbol(G, ae, q, H, J.concat(oe));
      if (ligatures.hasOwnProperty(G) && ae.slice(0, 10) === "Typewriter") {
        for (var ce = [], le = 0; le < G.length; le++)
          ce.push(makeSymbol(G[le], ae, q, H, J.concat(oe)));
        return makeFragment(ce);
      }
    }
    if (V === "mathord")
      return makeSymbol(G, "Math-Italic", q, H, J.concat(["mathnormal"]));
    if (V === "textord") {
      var ue = symbols$3[q][G] && symbols$3[q][G].font;
      if (ue === "ams") {
        var de = retrieveTextFontName("amsrm", H.fontWeight, H.fontShape);
        return makeSymbol(G, de, q, H, J.concat("amsrm", H.fontWeight, H.fontShape));
      } else if (ue === "main" || !ue) {
        var fe = retrieveTextFontName("textrm", H.fontWeight, H.fontShape);
        return makeSymbol(G, fe, q, H, J.concat(H.fontWeight, H.fontShape));
      } else {
        var he = retrieveTextFontName(ue, H.fontWeight, H.fontShape);
        return makeSymbol(G, he, q, H, J.concat(he, H.fontWeight, H.fontShape));
      }
    } else
      throw new Error("unexpected type: " + V + " in makeOrd");
  }, canCombine = (O, _) => {
    if (createClass(O.classes) !== createClass(_.classes) || O.skew !== _.skew || O.maxFontSize !== _.maxFontSize)
      return !1;
    if (O.classes.length === 1) {
      var H = O.classes[0];
      if (H === "mbin" || H === "mord")
        return !1;
    }
    for (var V in O.style)
      if (O.style.hasOwnProperty(V) && O.style[V] !== _.style[V])
        return !1;
    for (var q in _.style)
      if (_.style.hasOwnProperty(q) && O.style[q] !== _.style[q])
        return !1;
    return !0;
  }, tryCombineChars = (O) => {
    for (var _ = 0; _ < O.length - 1; _++) {
      var H = O[_], V = O[_ + 1];
      H instanceof SymbolNode && V instanceof SymbolNode && canCombine(H, V) && (H.text += V.text, H.height = Math.max(H.height, V.height), H.depth = Math.max(H.depth, V.depth), H.italic = V.italic, O.splice(_ + 1, 1), _--);
    }
    return O;
  }, sizeElementFromChildren = function O(_) {
    for (var H = 0, V = 0, q = 0, G = 0; G < _.children.length; G++) {
      var J = _.children[G];
      J.height > H && (H = J.height), J.depth > V && (V = J.depth), J.maxFontSize > q && (q = J.maxFontSize);
    }
    _.height = H, _.depth = V, _.maxFontSize = q;
  }, makeSpan$2 = function O(_, H, V, q) {
    var G = new Span$1(_, H, V, q);
    return sizeElementFromChildren(G), G;
  }, makeSvgSpan = (O, _, H, V) => new Span$1(O, _, H, V), makeLineSpan = function O(_, H, V) {
    var q = makeSpan$2([_], [], H);
    return q.height = Math.max(V || H.fontMetrics().defaultRuleThickness, H.minRuleThickness), q.style.borderBottomWidth = makeEm(q.height), q.maxFontSize = 1, q;
  }, makeAnchor = function O(_, H, V, q) {
    var G = new Anchor(_, H, V, q);
    return sizeElementFromChildren(G), G;
  }, makeFragment = function O(_) {
    var H = new DocumentFragment(_);
    return sizeElementFromChildren(H), H;
  }, wrapFragment = function O(_, H) {
    return _ instanceof DocumentFragment ? makeSpan$2([], [_], H) : _;
  }, getVListChildrenAndDepth = function O(_) {
    if (_.positionType === "individualShift") {
      for (var H = _.children, V = [H[0]], q = -H[0].shift - H[0].elem.depth, G = q, J = 1; J < H.length; J++) {
        var Z = -H[J].shift - G - H[J].elem.depth, te = Z - (H[J - 1].elem.height + H[J - 1].elem.depth);
        G = G + Z, V.push({
          type: "kern",
          size: te
        }), V.push(H[J]);
      }
      return {
        children: V,
        depth: q
      };
    }
    var ne;
    if (_.positionType === "top") {
      for (var ee = _.positionData, ae = 0; ae < _.children.length; ae++) {
        var oe = _.children[ae];
        ee -= oe.type === "kern" ? oe.size : oe.elem.height + oe.elem.depth;
      }
      ne = ee;
    } else if (_.positionType === "bottom")
      ne = -_.positionData;
    else {
      var se = _.children[0];
      if (se.type !== "elem")
        throw new Error('First child must have type "elem".');
      if (_.positionType === "shift")
        ne = -se.elem.depth - _.positionData;
      else if (_.positionType === "firstBaseline")
        ne = -se.elem.depth;
      else
        throw new Error("Invalid positionType " + _.positionType + ".");
    }
    return {
      children: _.children,
      depth: ne
    };
  }, makeVList = function O(_, H) {
    for (var {
      children: V,
      depth: q
    } = getVListChildrenAndDepth(_), G = 0, J = 0; J < V.length; J++) {
      var Z = V[J];
      if (Z.type === "elem") {
        var te = Z.elem;
        G = Math.max(G, te.maxFontSize, te.height);
      }
    }
    G += 2;
    var ne = makeSpan$2(["pstrut"], []);
    ne.style.height = makeEm(G);
    for (var ee = [], ae = q, oe = q, se = q, ce = 0; ce < V.length; ce++) {
      var le = V[ce];
      if (le.type === "kern")
        se += le.size;
      else {
        var ue = le.elem, de = le.wrapperClasses || [], fe = le.wrapperStyle || {}, he = makeSpan$2(de, [ne, ue], void 0, fe);
        he.style.top = makeEm(-G - se - ue.depth), le.marginLeft && (he.style.marginLeft = le.marginLeft), le.marginRight && (he.style.marginRight = le.marginRight), ee.push(he), se += ue.height + ue.depth;
      }
      ae = Math.min(ae, se), oe = Math.max(oe, se);
    }
    var me = makeSpan$2(["vlist"], ee);
    me.style.height = makeEm(oe);
    var pe;
    if (ae < 0) {
      var ye = makeSpan$2([], []), ge = makeSpan$2(["vlist"], [ye]);
      ge.style.height = makeEm(-ae);
      var ve = makeSpan$2(["vlist-s"], [new SymbolNode("​")]);
      pe = [makeSpan$2(["vlist-r"], [me, ve]), makeSpan$2(["vlist-r"], [ge])];
    } else
      pe = [makeSpan$2(["vlist-r"], [me])];
    var be = makeSpan$2(["vlist-t"], pe);
    return pe.length === 2 && be.classes.push("vlist-t2"), be.height = oe, be.depth = -ae, be;
  }, makeGlue = (O, _) => {
    var H = makeSpan$2(["mspace"], [], _), V = calculateSize(O, _);
    return H.style.marginRight = makeEm(V), H;
  }, retrieveTextFontName = function O(_, H, V) {
    var q = "";
    switch (_) {
      case "amsrm":
        q = "AMS";
        break;
      case "textrm":
        q = "Main";
        break;
      case "textsf":
        q = "SansSerif";
        break;
      case "texttt":
        q = "Typewriter";
        break;
      default:
        q = _;
    }
    var G;
    return H === "textbf" && V === "textit" ? G = "BoldItalic" : H === "textbf" ? G = "Bold" : H === "textit" ? G = "Italic" : G = "Regular", q + "-" + G;
  }, fontMap = {
    // styles
    mathbf: {
      variant: "bold",
      fontName: "Main-Bold"
    },
    mathrm: {
      variant: "normal",
      fontName: "Main-Regular"
    },
    textit: {
      variant: "italic",
      fontName: "Main-Italic"
    },
    mathit: {
      variant: "italic",
      fontName: "Main-Italic"
    },
    mathnormal: {
      variant: "italic",
      fontName: "Math-Italic"
    },
    mathsfit: {
      variant: "sans-serif-italic",
      fontName: "SansSerif-Italic"
    },
    // "boldsymbol" is missing because they require the use of multiple fonts:
    // Math-BoldItalic and Main-Bold.  This is handled by a special case in
    // makeOrd which ends up calling boldsymbol.
    // families
    mathbb: {
      variant: "double-struck",
      fontName: "AMS-Regular"
    },
    mathcal: {
      variant: "script",
      fontName: "Caligraphic-Regular"
    },
    mathfrak: {
      variant: "fraktur",
      fontName: "Fraktur-Regular"
    },
    mathscr: {
      variant: "script",
      fontName: "Script-Regular"
    },
    mathsf: {
      variant: "sans-serif",
      fontName: "SansSerif-Regular"
    },
    mathtt: {
      variant: "monospace",
      fontName: "Typewriter-Regular"
    }
  }, svgData = {
    //   path, width, height
    vec: ["vec", 0.471, 0.714],
    // values from the font glyph
    oiintSize1: ["oiintSize1", 0.957, 0.499],
    // oval to overlay the integrand
    oiintSize2: ["oiintSize2", 1.472, 0.659],
    oiiintSize1: ["oiiintSize1", 1.304, 0.499],
    oiiintSize2: ["oiiintSize2", 1.98, 0.659]
  }, staticSvg = function O(_, H) {
    var [V, q, G] = svgData[_], J = new PathNode(V), Z = new SvgNode([J], {
      width: makeEm(q),
      height: makeEm(G),
      // Override CSS rule `.katex svg { width: 100% }`
      style: "width:" + makeEm(q),
      viewBox: "0 0 " + 1e3 * q + " " + 1e3 * G,
      preserveAspectRatio: "xMinYMin"
    }), te = makeSvgSpan(["overlay"], [Z], H);
    return te.height = G, te.style.height = makeEm(G), te.style.width = makeEm(q), te;
  }, buildCommon = {
    fontMap,
    makeSymbol,
    mathsym,
    makeSpan: makeSpan$2,
    makeSvgSpan,
    makeLineSpan,
    makeAnchor,
    makeFragment,
    wrapFragment,
    makeVList,
    makeOrd,
    makeGlue,
    staticSvg,
    svgData,
    tryCombineChars
  }, thinspace = {
    number: 3,
    unit: "mu"
  }, mediumspace = {
    number: 4,
    unit: "mu"
  }, thickspace = {
    number: 5,
    unit: "mu"
  }, spacings = {
    mord: {
      mop: thinspace,
      mbin: mediumspace,
      mrel: thickspace,
      minner: thinspace
    },
    mop: {
      mord: thinspace,
      mop: thinspace,
      mrel: thickspace,
      minner: thinspace
    },
    mbin: {
      mord: mediumspace,
      mop: mediumspace,
      mopen: mediumspace,
      minner: mediumspace
    },
    mrel: {
      mord: thickspace,
      mop: thickspace,
      mopen: thickspace,
      minner: thickspace
    },
    mopen: {},
    mclose: {
      mop: thinspace,
      mbin: mediumspace,
      mrel: thickspace,
      minner: thinspace
    },
    mpunct: {
      mord: thinspace,
      mop: thinspace,
      mrel: thickspace,
      mopen: thinspace,
      mclose: thinspace,
      mpunct: thinspace,
      minner: thinspace
    },
    minner: {
      mord: thinspace,
      mop: thinspace,
      mbin: mediumspace,
      mrel: thickspace,
      mopen: thinspace,
      mpunct: thinspace,
      minner: thinspace
    }
  }, tightSpacings = {
    mord: {
      mop: thinspace
    },
    mop: {
      mord: thinspace,
      mop: thinspace
    },
    mbin: {},
    mrel: {},
    mopen: {},
    mclose: {
      mop: thinspace
    },
    mpunct: {},
    minner: {
      mop: thinspace
    }
  }, _functions = {}, _htmlGroupBuilders = {}, _mathmlGroupBuilders = {};
  function defineFunction(O) {
    for (var {
      type: _,
      names: H,
      props: V,
      handler: q,
      htmlBuilder: G,
      mathmlBuilder: J
    } = O, Z = {
      type: _,
      numArgs: V.numArgs,
      argTypes: V.argTypes,
      allowedInArgument: !!V.allowedInArgument,
      allowedInText: !!V.allowedInText,
      allowedInMath: V.allowedInMath === void 0 ? !0 : V.allowedInMath,
      numOptionalArgs: V.numOptionalArgs || 0,
      infix: !!V.infix,
      primitive: !!V.primitive,
      handler: q
    }, te = 0; te < H.length; ++te)
      _functions[H[te]] = Z;
    _ && (G && (_htmlGroupBuilders[_] = G), J && (_mathmlGroupBuilders[_] = J));
  }
  function defineFunctionBuilders(O) {
    var {
      type: _,
      htmlBuilder: H,
      mathmlBuilder: V
    } = O;
    defineFunction({
      type: _,
      names: [],
      props: {
        numArgs: 0
      },
      handler() {
        throw new Error("Should never be called.");
      },
      htmlBuilder: H,
      mathmlBuilder: V
    });
  }
  var normalizeArgument = function O(_) {
    return _.type === "ordgroup" && _.body.length === 1 ? _.body[0] : _;
  }, ordargument = function O(_) {
    return _.type === "ordgroup" ? _.body : [_];
  }, makeSpan$1 = buildCommon.makeSpan, binLeftCanceller = ["leftmost", "mbin", "mopen", "mrel", "mop", "mpunct"], binRightCanceller = ["rightmost", "mrel", "mclose", "mpunct"], styleMap$1 = {
    display: Style$1.DISPLAY,
    text: Style$1.TEXT,
    script: Style$1.SCRIPT,
    scriptscript: Style$1.SCRIPTSCRIPT
  }, DomEnum = {
    mord: "mord",
    mop: "mop",
    mbin: "mbin",
    mrel: "mrel",
    mopen: "mopen",
    mclose: "mclose",
    mpunct: "mpunct",
    minner: "minner"
  }, buildExpression$1 = function O(_, H, V, q) {
    q === void 0 && (q = [null, null]);
    for (var G = [], J = 0; J < _.length; J++) {
      var Z = buildGroup$1(_[J], H);
      if (Z instanceof DocumentFragment) {
        var te = Z.children;
        G.push(...te);
      } else
        G.push(Z);
    }
    if (buildCommon.tryCombineChars(G), !V)
      return G;
    var ne = H;
    if (_.length === 1) {
      var ee = _[0];
      ee.type === "sizing" ? ne = H.havingSize(ee.size) : ee.type === "styling" && (ne = H.havingStyle(styleMap$1[ee.style]));
    }
    var ae = makeSpan$1([q[0] || "leftmost"], [], H), oe = makeSpan$1([q[1] || "rightmost"], [], H), se = V === "root";
    return traverseNonSpaceNodes(G, (ce, le) => {
      var ue = le.classes[0], de = ce.classes[0];
      ue === "mbin" && utils.contains(binRightCanceller, de) ? le.classes[0] = "mord" : de === "mbin" && utils.contains(binLeftCanceller, ue) && (ce.classes[0] = "mord");
    }, {
      node: ae
    }, oe, se), traverseNonSpaceNodes(G, (ce, le) => {
      var ue = getTypeOfDomTree(le), de = getTypeOfDomTree(ce), fe = ue && de ? ce.hasClass("mtight") ? tightSpacings[ue][de] : spacings[ue][de] : null;
      if (fe)
        return buildCommon.makeGlue(fe, ne);
    }, {
      node: ae
    }, oe, se), G;
  }, traverseNonSpaceNodes = function O(_, H, V, q, G) {
    q && _.push(q);
    for (var J = 0; J < _.length; J++) {
      var Z = _[J], te = checkPartialGroup(Z);
      if (te) {
        O(te.children, H, V, null, G);
        continue;
      }
      var ne = !Z.hasClass("mspace");
      if (ne) {
        var ee = H(Z, V.node);
        ee && (V.insertAfter ? V.insertAfter(ee) : (_.unshift(ee), J++));
      }
      ne ? V.node = Z : G && Z.hasClass("newline") && (V.node = makeSpan$1(["leftmost"])), V.insertAfter = /* @__PURE__ */ ((ae) => (oe) => {
        _.splice(ae + 1, 0, oe), J++;
      })(J);
    }
    q && _.pop();
  }, checkPartialGroup = function O(_) {
    return _ instanceof DocumentFragment || _ instanceof Anchor || _ instanceof Span$1 && _.hasClass("enclosing") ? _ : null;
  }, getOutermostNode = function O(_, H) {
    var V = checkPartialGroup(_);
    if (V) {
      var q = V.children;
      if (q.length) {
        if (H === "right")
          return O(q[q.length - 1], "right");
        if (H === "left")
          return O(q[0], "left");
      }
    }
    return _;
  }, getTypeOfDomTree = function O(_, H) {
    return _ ? (H && (_ = getOutermostNode(_, H)), DomEnum[_.classes[0]] || null) : null;
  }, makeNullDelimiter = function O(_, H) {
    var V = ["nulldelimiter"].concat(_.baseSizingClasses());
    return makeSpan$1(H.concat(V));
  }, buildGroup$1 = function O(_, H, V) {
    if (!_)
      return makeSpan$1();
    if (_htmlGroupBuilders[_.type]) {
      var q = _htmlGroupBuilders[_.type](_, H);
      if (V && H.size !== V.size) {
        q = makeSpan$1(H.sizingClasses(V), [q], H);
        var G = H.sizeMultiplier / V.sizeMultiplier;
        q.height *= G, q.depth *= G;
      }
      return q;
    } else
      throw new ParseError("Got group of unknown type: '" + _.type + "'");
  };
  function buildHTMLUnbreakable(O, _) {
    var H = makeSpan$1(["base"], O, _), V = makeSpan$1(["strut"]);
    return V.style.height = makeEm(H.height + H.depth), H.depth && (V.style.verticalAlign = makeEm(-H.depth)), H.children.unshift(V), H;
  }
  function buildHTML(O, _) {
    var H = null;
    O.length === 1 && O[0].type === "tag" && (H = O[0].tag, O = O[0].body);
    var V = buildExpression$1(O, _, "root"), q;
    V.length === 2 && V[1].hasClass("tag") && (q = V.pop());
    for (var G = [], J = [], Z = 0; Z < V.length; Z++)
      if (J.push(V[Z]), V[Z].hasClass("mbin") || V[Z].hasClass("mrel") || V[Z].hasClass("allowbreak")) {
        for (var te = !1; Z < V.length - 1 && V[Z + 1].hasClass("mspace") && !V[Z + 1].hasClass("newline"); )
          Z++, J.push(V[Z]), V[Z].hasClass("nobreak") && (te = !0);
        te || (G.push(buildHTMLUnbreakable(J, _)), J = []);
      } else V[Z].hasClass("newline") && (J.pop(), J.length > 0 && (G.push(buildHTMLUnbreakable(J, _)), J = []), G.push(V[Z]));
    J.length > 0 && G.push(buildHTMLUnbreakable(J, _));
    var ne;
    H ? (ne = buildHTMLUnbreakable(buildExpression$1(H, _, !0)), ne.classes = ["tag"], G.push(ne)) : q && G.push(q);
    var ee = makeSpan$1(["katex-html"], G);
    if (ee.setAttribute("aria-hidden", "true"), ne) {
      var ae = ne.children[0];
      ae.style.height = makeEm(ee.height + ee.depth), ee.depth && (ae.style.verticalAlign = makeEm(-ee.depth));
    }
    return ee;
  }
  function newDocumentFragment(O) {
    return new DocumentFragment(O);
  }
  class MathNode {
    constructor(_, H, V) {
      this.type = void 0, this.attributes = void 0, this.children = void 0, this.classes = void 0, this.type = _, this.attributes = {}, this.children = H || [], this.classes = V || [];
    }
    /**
     * Sets an attribute on a MathML node. MathML depends on attributes to convey a
     * semantic content, so this is used heavily.
     */
    setAttribute(_, H) {
      this.attributes[_] = H;
    }
    /**
     * Gets an attribute on a MathML node.
     */
    getAttribute(_) {
      return this.attributes[_];
    }
    /**
     * Converts the math node into a MathML-namespaced DOM element.
     */
    toNode() {
      var _ = document.createElementNS("http://www.w3.org/1998/Math/MathML", this.type);
      for (var H in this.attributes)
        Object.prototype.hasOwnProperty.call(this.attributes, H) && _.setAttribute(H, this.attributes[H]);
      this.classes.length > 0 && (_.className = createClass(this.classes));
      for (var V = 0; V < this.children.length; V++)
        _.appendChild(this.children[V].toNode());
      return _;
    }
    /**
     * Converts the math node into an HTML markup string.
     */
    toMarkup() {
      var _ = "<" + this.type;
      for (var H in this.attributes)
        Object.prototype.hasOwnProperty.call(this.attributes, H) && (_ += " " + H + '="', _ += utils.escape(this.attributes[H]), _ += '"');
      this.classes.length > 0 && (_ += ' class ="' + utils.escape(createClass(this.classes)) + '"'), _ += ">";
      for (var V = 0; V < this.children.length; V++)
        _ += this.children[V].toMarkup();
      return _ += "</" + this.type + ">", _;
    }
    /**
     * Converts the math node into a string, similar to innerText, but escaped.
     */
    toText() {
      return this.children.map((_) => _.toText()).join("");
    }
  }
  let TextNode$1 = class {
    constructor(_) {
      this.text = void 0, this.text = _;
    }
    /**
     * Converts the text node into a DOM text node.
     */
    toNode() {
      return document.createTextNode(this.text);
    }
    /**
     * Converts the text node into escaped HTML markup
     * (representing the text itself).
     */
    toMarkup() {
      return utils.escape(this.toText());
    }
    /**
     * Converts the text node into a string
     * (representing the text itself).
     */
    toText() {
      return this.text;
    }
  };
  class SpaceNode {
    /**
     * Create a Space node with width given in CSS ems.
     */
    constructor(_) {
      this.width = void 0, this.character = void 0, this.width = _, _ >= 0.05555 && _ <= 0.05556 ? this.character = " " : _ >= 0.1666 && _ <= 0.1667 ? this.character = " " : _ >= 0.2222 && _ <= 0.2223 ? this.character = " " : _ >= 0.2777 && _ <= 0.2778 ? this.character = "  " : _ >= -0.05556 && _ <= -0.05555 ? this.character = " ⁣" : _ >= -0.1667 && _ <= -0.1666 ? this.character = " ⁣" : _ >= -0.2223 && _ <= -0.2222 ? this.character = " ⁣" : _ >= -0.2778 && _ <= -0.2777 ? this.character = " ⁣" : this.character = null;
    }
    /**
     * Converts the math node into a MathML-namespaced DOM element.
     */
    toNode() {
      if (this.character)
        return document.createTextNode(this.character);
      var _ = document.createElementNS("http://www.w3.org/1998/Math/MathML", "mspace");
      return _.setAttribute("width", makeEm(this.width)), _;
    }
    /**
     * Converts the math node into an HTML markup string.
     */
    toMarkup() {
      return this.character ? "<mtext>" + this.character + "</mtext>" : '<mspace width="' + makeEm(this.width) + '"/>';
    }
    /**
     * Converts the math node into a string, similar to innerText.
     */
    toText() {
      return this.character ? this.character : " ";
    }
  }
  var mathMLTree = {
    MathNode,
    TextNode: TextNode$1,
    SpaceNode,
    newDocumentFragment
  }, makeText = function O(_, H, V) {
    return symbols$3[H][_] && symbols$3[H][_].replace && _.charCodeAt(0) !== 55349 && !(ligatures.hasOwnProperty(_) && V && (V.fontFamily && V.fontFamily.slice(4, 6) === "tt" || V.font && V.font.slice(4, 6) === "tt")) && (_ = symbols$3[H][_].replace), new mathMLTree.TextNode(_);
  }, makeRow = function O(_) {
    return _.length === 1 ? _[0] : new mathMLTree.MathNode("mrow", _);
  }, getVariant = function O(_, H) {
    if (H.fontFamily === "texttt")
      return "monospace";
    if (H.fontFamily === "textsf")
      return H.fontShape === "textit" && H.fontWeight === "textbf" ? "sans-serif-bold-italic" : H.fontShape === "textit" ? "sans-serif-italic" : H.fontWeight === "textbf" ? "bold-sans-serif" : "sans-serif";
    if (H.fontShape === "textit" && H.fontWeight === "textbf")
      return "bold-italic";
    if (H.fontShape === "textit")
      return "italic";
    if (H.fontWeight === "textbf")
      return "bold";
    var V = H.font;
    if (!V || V === "mathnormal")
      return null;
    var q = _.mode;
    if (V === "mathit")
      return "italic";
    if (V === "boldsymbol")
      return _.type === "textord" ? "bold" : "bold-italic";
    if (V === "mathbf")
      return "bold";
    if (V === "mathbb")
      return "double-struck";
    if (V === "mathsfit")
      return "sans-serif-italic";
    if (V === "mathfrak")
      return "fraktur";
    if (V === "mathscr" || V === "mathcal")
      return "script";
    if (V === "mathsf")
      return "sans-serif";
    if (V === "mathtt")
      return "monospace";
    var G = _.text;
    if (utils.contains(["\\imath", "\\jmath"], G))
      return null;
    symbols$3[q][G] && symbols$3[q][G].replace && (G = symbols$3[q][G].replace);
    var J = buildCommon.fontMap[V].fontName;
    return getCharacterMetrics(G, J, q) ? buildCommon.fontMap[V].variant : null;
  }, buildExpression = function O(_, H, V) {
    if (_.length === 1) {
      var q = buildGroup(_[0], H);
      return V && q instanceof MathNode && q.type === "mo" && (q.setAttribute("lspace", "0em"), q.setAttribute("rspace", "0em")), [q];
    }
    for (var G = [], J, Z = 0; Z < _.length; Z++) {
      var te = buildGroup(_[Z], H);
      if (te instanceof MathNode && J instanceof MathNode) {
        if (te.type === "mtext" && J.type === "mtext" && te.getAttribute("mathvariant") === J.getAttribute("mathvariant")) {
          J.children.push(...te.children);
          continue;
        } else if (te.type === "mn" && J.type === "mn") {
          J.children.push(...te.children);
          continue;
        } else if (te.type === "mi" && te.children.length === 1 && J.type === "mn") {
          var ne = te.children[0];
          if (ne instanceof TextNode$1 && ne.text === ".") {
            J.children.push(...te.children);
            continue;
          }
        } else if (J.type === "mi" && J.children.length === 1) {
          var ee = J.children[0];
          if (ee instanceof TextNode$1 && ee.text === "̸" && (te.type === "mo" || te.type === "mi" || te.type === "mn")) {
            var ae = te.children[0];
            ae instanceof TextNode$1 && ae.text.length > 0 && (ae.text = ae.text.slice(0, 1) + "̸" + ae.text.slice(1), G.pop());
          }
        }
      }
      G.push(te), J = te;
    }
    return G;
  }, buildExpressionRow = function O(_, H, V) {
    return makeRow(buildExpression(_, H, V));
  }, buildGroup = function O(_, H) {
    if (!_)
      return new mathMLTree.MathNode("mrow");
    if (_mathmlGroupBuilders[_.type]) {
      var V = _mathmlGroupBuilders[_.type](_, H);
      return V;
    } else
      throw new ParseError("Got group of unknown type: '" + _.type + "'");
  };
  function buildMathML(O, _, H, V, q) {
    var G = buildExpression(O, H), J;
    G.length === 1 && G[0] instanceof MathNode && utils.contains(["mrow", "mtable"], G[0].type) ? J = G[0] : J = new mathMLTree.MathNode("mrow", G);
    var Z = new mathMLTree.MathNode("annotation", [new mathMLTree.TextNode(_)]);
    Z.setAttribute("encoding", "application/x-tex");
    var te = new mathMLTree.MathNode("semantics", [J, Z]), ne = new mathMLTree.MathNode("math", [te]);
    ne.setAttribute("xmlns", "http://www.w3.org/1998/Math/MathML"), V && ne.setAttribute("display", "block");
    var ee = q ? "katex" : "katex-mathml";
    return buildCommon.makeSpan([ee], [ne]);
  }
  var optionsFromSettings = function O(_) {
    return new Options({
      style: _.displayMode ? Style$1.DISPLAY : Style$1.TEXT,
      maxSize: _.maxSize,
      minRuleThickness: _.minRuleThickness
    });
  }, displayWrap = function O(_, H) {
    if (H.displayMode) {
      var V = ["katex-display"];
      H.leqno && V.push("leqno"), H.fleqn && V.push("fleqn"), _ = buildCommon.makeSpan(V, [_]);
    }
    return _;
  }, buildTree$1 = function O(_, H, V) {
    var q = optionsFromSettings(V), G;
    if (V.output === "mathml")
      return buildMathML(_, H, q, V.displayMode, !0);
    if (V.output === "html") {
      var J = buildHTML(_, q);
      G = buildCommon.makeSpan(["katex"], [J]);
    } else {
      var Z = buildMathML(_, H, q, V.displayMode, !1), te = buildHTML(_, q);
      G = buildCommon.makeSpan(["katex"], [Z, te]);
    }
    return displayWrap(G, V);
  }, buildHTMLTree = function O(_, H, V) {
    var q = optionsFromSettings(V), G = buildHTML(_, q), J = buildCommon.makeSpan(["katex"], [G]);
    return displayWrap(J, V);
  }, stretchyCodePoint = {
    widehat: "^",
    widecheck: "ˇ",
    widetilde: "~",
    utilde: "~",
    overleftarrow: "←",
    underleftarrow: "←",
    xleftarrow: "←",
    overrightarrow: "→",
    underrightarrow: "→",
    xrightarrow: "→",
    underbrace: "⏟",
    overbrace: "⏞",
    overgroup: "⏠",
    undergroup: "⏡",
    overleftrightarrow: "↔",
    underleftrightarrow: "↔",
    xleftrightarrow: "↔",
    Overrightarrow: "⇒",
    xRightarrow: "⇒",
    overleftharpoon: "↼",
    xleftharpoonup: "↼",
    overrightharpoon: "⇀",
    xrightharpoonup: "⇀",
    xLeftarrow: "⇐",
    xLeftrightarrow: "⇔",
    xhookleftarrow: "↩",
    xhookrightarrow: "↪",
    xmapsto: "↦",
    xrightharpoondown: "⇁",
    xleftharpoondown: "↽",
    xrightleftharpoons: "⇌",
    xleftrightharpoons: "⇋",
    xtwoheadleftarrow: "↞",
    xtwoheadrightarrow: "↠",
    xlongequal: "=",
    xtofrom: "⇄",
    xrightleftarrows: "⇄",
    xrightequilibrium: "⇌",
    // Not a perfect match.
    xleftequilibrium: "⇋",
    // None better available.
    "\\cdrightarrow": "→",
    "\\cdleftarrow": "←",
    "\\cdlongequal": "="
  }, mathMLnode = function O(_) {
    var H = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode(stretchyCodePoint[_.replace(/^\\/, "")])]);
    return H.setAttribute("stretchy", "true"), H;
  }, katexImagesData = {
    //   path(s), minWidth, height, align
    overrightarrow: [["rightarrow"], 0.888, 522, "xMaxYMin"],
    overleftarrow: [["leftarrow"], 0.888, 522, "xMinYMin"],
    underrightarrow: [["rightarrow"], 0.888, 522, "xMaxYMin"],
    underleftarrow: [["leftarrow"], 0.888, 522, "xMinYMin"],
    xrightarrow: [["rightarrow"], 1.469, 522, "xMaxYMin"],
    "\\cdrightarrow": [["rightarrow"], 3, 522, "xMaxYMin"],
    // CD minwwidth2.5pc
    xleftarrow: [["leftarrow"], 1.469, 522, "xMinYMin"],
    "\\cdleftarrow": [["leftarrow"], 3, 522, "xMinYMin"],
    Overrightarrow: [["doublerightarrow"], 0.888, 560, "xMaxYMin"],
    xRightarrow: [["doublerightarrow"], 1.526, 560, "xMaxYMin"],
    xLeftarrow: [["doubleleftarrow"], 1.526, 560, "xMinYMin"],
    overleftharpoon: [["leftharpoon"], 0.888, 522, "xMinYMin"],
    xleftharpoonup: [["leftharpoon"], 0.888, 522, "xMinYMin"],
    xleftharpoondown: [["leftharpoondown"], 0.888, 522, "xMinYMin"],
    overrightharpoon: [["rightharpoon"], 0.888, 522, "xMaxYMin"],
    xrightharpoonup: [["rightharpoon"], 0.888, 522, "xMaxYMin"],
    xrightharpoondown: [["rightharpoondown"], 0.888, 522, "xMaxYMin"],
    xlongequal: [["longequal"], 0.888, 334, "xMinYMin"],
    "\\cdlongequal": [["longequal"], 3, 334, "xMinYMin"],
    xtwoheadleftarrow: [["twoheadleftarrow"], 0.888, 334, "xMinYMin"],
    xtwoheadrightarrow: [["twoheadrightarrow"], 0.888, 334, "xMaxYMin"],
    overleftrightarrow: [["leftarrow", "rightarrow"], 0.888, 522],
    overbrace: [["leftbrace", "midbrace", "rightbrace"], 1.6, 548],
    underbrace: [["leftbraceunder", "midbraceunder", "rightbraceunder"], 1.6, 548],
    underleftrightarrow: [["leftarrow", "rightarrow"], 0.888, 522],
    xleftrightarrow: [["leftarrow", "rightarrow"], 1.75, 522],
    xLeftrightarrow: [["doubleleftarrow", "doublerightarrow"], 1.75, 560],
    xrightleftharpoons: [["leftharpoondownplus", "rightharpoonplus"], 1.75, 716],
    xleftrightharpoons: [["leftharpoonplus", "rightharpoondownplus"], 1.75, 716],
    xhookleftarrow: [["leftarrow", "righthook"], 1.08, 522],
    xhookrightarrow: [["lefthook", "rightarrow"], 1.08, 522],
    overlinesegment: [["leftlinesegment", "rightlinesegment"], 0.888, 522],
    underlinesegment: [["leftlinesegment", "rightlinesegment"], 0.888, 522],
    overgroup: [["leftgroup", "rightgroup"], 0.888, 342],
    undergroup: [["leftgroupunder", "rightgroupunder"], 0.888, 342],
    xmapsto: [["leftmapsto", "rightarrow"], 1.5, 522],
    xtofrom: [["leftToFrom", "rightToFrom"], 1.75, 528],
    // The next three arrows are from the mhchem package.
    // In mhchem.sty, min-length is 2.0em. But these arrows might appear in the
    // document as \xrightarrow or \xrightleftharpoons. Those have
    // min-length = 1.75em, so we set min-length on these next three to match.
    xrightleftarrows: [["baraboveleftarrow", "rightarrowabovebar"], 1.75, 901],
    xrightequilibrium: [["baraboveshortleftharpoon", "rightharpoonaboveshortbar"], 1.75, 716],
    xleftequilibrium: [["shortbaraboveleftharpoon", "shortrightharpoonabovebar"], 1.75, 716]
  }, groupLength = function O(_) {
    return _.type === "ordgroup" ? _.body.length : 1;
  }, svgSpan = function O(_, H) {
    function V() {
      var Z = 4e5, te = _.label.slice(1);
      if (utils.contains(["widehat", "widecheck", "widetilde", "utilde"], te)) {
        var ne = _, ee = groupLength(ne.base), ae, oe, se;
        if (ee > 5)
          te === "widehat" || te === "widecheck" ? (ae = 420, Z = 2364, se = 0.42, oe = te + "4") : (ae = 312, Z = 2340, se = 0.34, oe = "tilde4");
        else {
          var ce = [1, 1, 2, 2, 3, 3][ee];
          te === "widehat" || te === "widecheck" ? (Z = [0, 1062, 2364, 2364, 2364][ce], ae = [0, 239, 300, 360, 420][ce], se = [0, 0.24, 0.3, 0.3, 0.36, 0.42][ce], oe = te + ce) : (Z = [0, 600, 1033, 2339, 2340][ce], ae = [0, 260, 286, 306, 312][ce], se = [0, 0.26, 0.286, 0.3, 0.306, 0.34][ce], oe = "tilde" + ce);
        }
        var le = new PathNode(oe), ue = new SvgNode([le], {
          width: "100%",
          height: makeEm(se),
          viewBox: "0 0 " + Z + " " + ae,
          preserveAspectRatio: "none"
        });
        return {
          span: buildCommon.makeSvgSpan([], [ue], H),
          minWidth: 0,
          height: se
        };
      } else {
        var de = [], fe = katexImagesData[te], [he, me, pe] = fe, ye = pe / 1e3, ge = he.length, ve, be;
        if (ge === 1) {
          var Ce = fe[3];
          ve = ["hide-tail"], be = [Ce];
        } else if (ge === 2)
          ve = ["halfarrow-left", "halfarrow-right"], be = ["xMinYMin", "xMaxYMin"];
        else if (ge === 3)
          ve = ["brace-left", "brace-center", "brace-right"], be = ["xMinYMin", "xMidYMin", "xMaxYMin"];
        else
          throw new Error(`Correct katexImagesData or update code here to support
                    ` + ge + " children.");
        for (var Se = 0; Se < ge; Se++) {
          var we = new PathNode(he[Se]), Ee = new SvgNode([we], {
            width: "400em",
            height: makeEm(ye),
            viewBox: "0 0 " + Z + " " + pe,
            preserveAspectRatio: be[Se] + " slice"
          }), xe = buildCommon.makeSvgSpan([ve[Se]], [Ee], H);
          if (ge === 1)
            return {
              span: xe,
              minWidth: me,
              height: ye
            };
          xe.style.height = makeEm(ye), de.push(xe);
        }
        return {
          span: buildCommon.makeSpan(["stretchy"], de, H),
          minWidth: me,
          height: ye
        };
      }
    }
    var {
      span: q,
      minWidth: G,
      height: J
    } = V();
    return q.height = J, q.style.height = makeEm(J), G > 0 && (q.style.minWidth = makeEm(G)), q;
  }, encloseSpan = function O(_, H, V, q, G) {
    var J, Z = _.height + _.depth + V + q;
    if (/fbox|color|angl/.test(H)) {
      if (J = buildCommon.makeSpan(["stretchy", H], [], G), H === "fbox") {
        var te = G.color && G.getColor();
        te && (J.style.borderColor = te);
      }
    } else {
      var ne = [];
      /^[bx]cancel$/.test(H) && ne.push(new LineNode({
        x1: "0",
        y1: "0",
        x2: "100%",
        y2: "100%",
        "stroke-width": "0.046em"
      })), /^x?cancel$/.test(H) && ne.push(new LineNode({
        x1: "0",
        y1: "100%",
        x2: "100%",
        y2: "0",
        "stroke-width": "0.046em"
      }));
      var ee = new SvgNode(ne, {
        width: "100%",
        height: makeEm(Z)
      });
      J = buildCommon.makeSvgSpan([], [ee], G);
    }
    return J.height = Z, J.style.height = makeEm(Z), J;
  }, stretchy = {
    encloseSpan,
    mathMLnode,
    svgSpan
  };
  function assertNodeType(O, _) {
    if (!O || O.type !== _)
      throw new Error("Expected node of type " + _ + ", but got " + (O ? "node of type " + O.type : String(O)));
    return O;
  }
  function assertSymbolNodeType(O) {
    var _ = checkSymbolNodeType(O);
    if (!_)
      throw new Error("Expected node of symbol group type, but got " + (O ? "node of type " + O.type : String(O)));
    return _;
  }
  function checkSymbolNodeType(O) {
    return O && (O.type === "atom" || NON_ATOMS.hasOwnProperty(O.type)) ? O : null;
  }
  var htmlBuilder$a = (O, _) => {
    var H, V, q;
    O && O.type === "supsub" ? (V = assertNodeType(O.base, "accent"), H = V.base, O.base = H, q = assertSpan(buildGroup$1(O, _)), O.base = V) : (V = assertNodeType(O, "accent"), H = V.base);
    var G = buildGroup$1(H, _.havingCrampedStyle()), J = V.isShifty && utils.isCharacterBox(H), Z = 0;
    if (J) {
      var te = utils.getBaseElem(H), ne = buildGroup$1(te, _.havingCrampedStyle());
      Z = assertSymbolDomNode(ne).skew;
    }
    var ee = V.label === "\\c", ae = ee ? G.height + G.depth : Math.min(G.height, _.fontMetrics().xHeight), oe;
    if (V.isStretchy)
      oe = stretchy.svgSpan(V, _), oe = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: G
        }, {
          type: "elem",
          elem: oe,
          wrapperClasses: ["svg-align"],
          wrapperStyle: Z > 0 ? {
            width: "calc(100% - " + makeEm(2 * Z) + ")",
            marginLeft: makeEm(2 * Z)
          } : void 0
        }]
      }, _);
    else {
      var se, ce;
      V.label === "\\vec" ? (se = buildCommon.staticSvg("vec", _), ce = buildCommon.svgData.vec[1]) : (se = buildCommon.makeOrd({
        mode: V.mode,
        text: V.label
      }, _, "textord"), se = assertSymbolDomNode(se), se.italic = 0, ce = se.width, ee && (ae += se.depth)), oe = buildCommon.makeSpan(["accent-body"], [se]);
      var le = V.label === "\\textcircled";
      le && (oe.classes.push("accent-full"), ae = G.height);
      var ue = Z;
      le || (ue -= ce / 2), oe.style.left = makeEm(ue), V.label === "\\textcircled" && (oe.style.top = ".2em"), oe = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: G
        }, {
          type: "kern",
          size: -ae
        }, {
          type: "elem",
          elem: oe
        }]
      }, _);
    }
    var de = buildCommon.makeSpan(["mord", "accent"], [oe], _);
    return q ? (q.children[0] = de, q.height = Math.max(de.height, q.height), q.classes[0] = "mord", q) : de;
  }, mathmlBuilder$9 = (O, _) => {
    var H = O.isStretchy ? stretchy.mathMLnode(O.label) : new mathMLTree.MathNode("mo", [makeText(O.label, O.mode)]), V = new mathMLTree.MathNode("mover", [buildGroup(O.base, _), H]);
    return V.setAttribute("accent", "true"), V;
  }, NON_STRETCHY_ACCENT_REGEX = new RegExp(["\\acute", "\\grave", "\\ddot", "\\tilde", "\\bar", "\\breve", "\\check", "\\hat", "\\vec", "\\dot", "\\mathring"].map((O) => "\\" + O).join("|"));
  defineFunction({
    type: "accent",
    names: ["\\acute", "\\grave", "\\ddot", "\\tilde", "\\bar", "\\breve", "\\check", "\\hat", "\\vec", "\\dot", "\\mathring", "\\widecheck", "\\widehat", "\\widetilde", "\\overrightarrow", "\\overleftarrow", "\\Overrightarrow", "\\overleftrightarrow", "\\overgroup", "\\overlinesegment", "\\overleftharpoon", "\\overrightharpoon"],
    props: {
      numArgs: 1
    },
    handler: (O, _) => {
      var H = normalizeArgument(_[0]), V = !NON_STRETCHY_ACCENT_REGEX.test(O.funcName), q = !V || O.funcName === "\\widehat" || O.funcName === "\\widetilde" || O.funcName === "\\widecheck";
      return {
        type: "accent",
        mode: O.parser.mode,
        label: O.funcName,
        isStretchy: V,
        isShifty: q,
        base: H
      };
    },
    htmlBuilder: htmlBuilder$a,
    mathmlBuilder: mathmlBuilder$9
  });
  defineFunction({
    type: "accent",
    names: ["\\'", "\\`", "\\^", "\\~", "\\=", "\\u", "\\.", '\\"', "\\c", "\\r", "\\H", "\\v", "\\textcircled"],
    props: {
      numArgs: 1,
      allowedInText: !0,
      allowedInMath: !0,
      // unless in strict mode
      argTypes: ["primitive"]
    },
    handler: (O, _) => {
      var H = _[0], V = O.parser.mode;
      return V === "math" && (O.parser.settings.reportNonstrict("mathVsTextAccents", "LaTeX's accent " + O.funcName + " works only in text mode"), V = "text"), {
        type: "accent",
        mode: V,
        label: O.funcName,
        isStretchy: !1,
        isShifty: !0,
        base: H
      };
    },
    htmlBuilder: htmlBuilder$a,
    mathmlBuilder: mathmlBuilder$9
  });
  defineFunction({
    type: "accentUnder",
    names: ["\\underleftarrow", "\\underrightarrow", "\\underleftrightarrow", "\\undergroup", "\\underlinesegment", "\\utilde"],
    props: {
      numArgs: 1
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0];
      return {
        type: "accentUnder",
        mode: H.mode,
        label: V,
        base: q
      };
    },
    htmlBuilder: (O, _) => {
      var H = buildGroup$1(O.base, _), V = stretchy.svgSpan(O, _), q = O.label === "\\utilde" ? 0.12 : 0, G = buildCommon.makeVList({
        positionType: "top",
        positionData: H.height,
        children: [{
          type: "elem",
          elem: V,
          wrapperClasses: ["svg-align"]
        }, {
          type: "kern",
          size: q
        }, {
          type: "elem",
          elem: H
        }]
      }, _);
      return buildCommon.makeSpan(["mord", "accentunder"], [G], _);
    },
    mathmlBuilder: (O, _) => {
      var H = stretchy.mathMLnode(O.label), V = new mathMLTree.MathNode("munder", [buildGroup(O.base, _), H]);
      return V.setAttribute("accentunder", "true"), V;
    }
  });
  var paddedNode = (O) => {
    var _ = new mathMLTree.MathNode("mpadded", O ? [O] : []);
    return _.setAttribute("width", "+0.6em"), _.setAttribute("lspace", "0.3em"), _;
  };
  defineFunction({
    type: "xArrow",
    names: [
      "\\xleftarrow",
      "\\xrightarrow",
      "\\xLeftarrow",
      "\\xRightarrow",
      "\\xleftrightarrow",
      "\\xLeftrightarrow",
      "\\xhookleftarrow",
      "\\xhookrightarrow",
      "\\xmapsto",
      "\\xrightharpoondown",
      "\\xrightharpoonup",
      "\\xleftharpoondown",
      "\\xleftharpoonup",
      "\\xrightleftharpoons",
      "\\xleftrightharpoons",
      "\\xlongequal",
      "\\xtwoheadrightarrow",
      "\\xtwoheadleftarrow",
      "\\xtofrom",
      // The next 3 functions are here to support the mhchem extension.
      // Direct use of these functions is discouraged and may break someday.
      "\\xrightleftarrows",
      "\\xrightequilibrium",
      "\\xleftequilibrium",
      // The next 3 functions are here only to support the {CD} environment.
      "\\\\cdrightarrow",
      "\\\\cdleftarrow",
      "\\\\cdlongequal"
    ],
    props: {
      numArgs: 1,
      numOptionalArgs: 1
    },
    handler(O, _, H) {
      var {
        parser: V,
        funcName: q
      } = O;
      return {
        type: "xArrow",
        mode: V.mode,
        label: q,
        body: _[0],
        below: H[0]
      };
    },
    // Flow is unable to correctly infer the type of `group`, even though it's
    // unambiguously determined from the passed-in `type` above.
    htmlBuilder(O, _) {
      var H = _.style, V = _.havingStyle(H.sup()), q = buildCommon.wrapFragment(buildGroup$1(O.body, V, _), _), G = O.label.slice(0, 2) === "\\x" ? "x" : "cd";
      q.classes.push(G + "-arrow-pad");
      var J;
      O.below && (V = _.havingStyle(H.sub()), J = buildCommon.wrapFragment(buildGroup$1(O.below, V, _), _), J.classes.push(G + "-arrow-pad"));
      var Z = stretchy.svgSpan(O, _), te = -_.fontMetrics().axisHeight + 0.5 * Z.height, ne = -_.fontMetrics().axisHeight - 0.5 * Z.height - 0.111;
      (q.depth > 0.25 || O.label === "\\xleftequilibrium") && (ne -= q.depth);
      var ee;
      if (J) {
        var ae = -_.fontMetrics().axisHeight + J.height + 0.5 * Z.height + 0.111;
        ee = buildCommon.makeVList({
          positionType: "individualShift",
          children: [{
            type: "elem",
            elem: q,
            shift: ne
          }, {
            type: "elem",
            elem: Z,
            shift: te
          }, {
            type: "elem",
            elem: J,
            shift: ae
          }]
        }, _);
      } else
        ee = buildCommon.makeVList({
          positionType: "individualShift",
          children: [{
            type: "elem",
            elem: q,
            shift: ne
          }, {
            type: "elem",
            elem: Z,
            shift: te
          }]
        }, _);
      return ee.children[0].children[0].children[1].classes.push("svg-align"), buildCommon.makeSpan(["mrel", "x-arrow"], [ee], _);
    },
    mathmlBuilder(O, _) {
      var H = stretchy.mathMLnode(O.label);
      H.setAttribute("minsize", O.label.charAt(0) === "x" ? "1.75em" : "3.0em");
      var V;
      if (O.body) {
        var q = paddedNode(buildGroup(O.body, _));
        if (O.below) {
          var G = paddedNode(buildGroup(O.below, _));
          V = new mathMLTree.MathNode("munderover", [H, G, q]);
        } else
          V = new mathMLTree.MathNode("mover", [H, q]);
      } else if (O.below) {
        var J = paddedNode(buildGroup(O.below, _));
        V = new mathMLTree.MathNode("munder", [H, J]);
      } else
        V = paddedNode(), V = new mathMLTree.MathNode("mover", [H, V]);
      return V;
    }
  });
  var makeSpan = buildCommon.makeSpan;
  function htmlBuilder$9(O, _) {
    var H = buildExpression$1(O.body, _, !0);
    return makeSpan([O.mclass], H, _);
  }
  function mathmlBuilder$8(O, _) {
    var H, V = buildExpression(O.body, _);
    return O.mclass === "minner" ? H = new mathMLTree.MathNode("mpadded", V) : O.mclass === "mord" ? O.isCharacterBox ? (H = V[0], H.type = "mi") : H = new mathMLTree.MathNode("mi", V) : (O.isCharacterBox ? (H = V[0], H.type = "mo") : H = new mathMLTree.MathNode("mo", V), O.mclass === "mbin" ? (H.attributes.lspace = "0.22em", H.attributes.rspace = "0.22em") : O.mclass === "mpunct" ? (H.attributes.lspace = "0em", H.attributes.rspace = "0.17em") : O.mclass === "mopen" || O.mclass === "mclose" ? (H.attributes.lspace = "0em", H.attributes.rspace = "0em") : O.mclass === "minner" && (H.attributes.lspace = "0.0556em", H.attributes.width = "+0.1111em")), H;
  }
  defineFunction({
    type: "mclass",
    names: ["\\mathord", "\\mathbin", "\\mathrel", "\\mathopen", "\\mathclose", "\\mathpunct", "\\mathinner"],
    props: {
      numArgs: 1,
      primitive: !0
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0];
      return {
        type: "mclass",
        mode: H.mode,
        mclass: "m" + V.slice(5),
        // TODO(kevinb): don't prefix with 'm'
        body: ordargument(q),
        isCharacterBox: utils.isCharacterBox(q)
      };
    },
    htmlBuilder: htmlBuilder$9,
    mathmlBuilder: mathmlBuilder$8
  });
  var binrelClass = (O) => {
    var _ = O.type === "ordgroup" && O.body.length ? O.body[0] : O;
    return _.type === "atom" && (_.family === "bin" || _.family === "rel") ? "m" + _.family : "mord";
  };
  defineFunction({
    type: "mclass",
    names: ["\\@binrel"],
    props: {
      numArgs: 2
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "mclass",
        mode: H.mode,
        mclass: binrelClass(_[0]),
        body: ordargument(_[1]),
        isCharacterBox: utils.isCharacterBox(_[1])
      };
    }
  });
  defineFunction({
    type: "mclass",
    names: ["\\stackrel", "\\overset", "\\underset"],
    props: {
      numArgs: 2
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O, q = _[1], G = _[0], J;
      V !== "\\stackrel" ? J = binrelClass(q) : J = "mrel";
      var Z = {
        type: "op",
        mode: q.mode,
        limits: !0,
        alwaysHandleSupSub: !0,
        parentIsSupSub: !1,
        symbol: !1,
        suppressBaseShift: V !== "\\stackrel",
        body: ordargument(q)
      }, te = {
        type: "supsub",
        mode: G.mode,
        base: Z,
        sup: V === "\\underset" ? null : G,
        sub: V === "\\underset" ? G : null
      };
      return {
        type: "mclass",
        mode: H.mode,
        mclass: J,
        body: [te],
        isCharacterBox: utils.isCharacterBox(te)
      };
    },
    htmlBuilder: htmlBuilder$9,
    mathmlBuilder: mathmlBuilder$8
  });
  defineFunction({
    type: "pmb",
    names: ["\\pmb"],
    props: {
      numArgs: 1,
      allowedInText: !0
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "pmb",
        mode: H.mode,
        mclass: binrelClass(_[0]),
        body: ordargument(_[0])
      };
    },
    htmlBuilder(O, _) {
      var H = buildExpression$1(O.body, _, !0), V = buildCommon.makeSpan([O.mclass], H, _);
      return V.style.textShadow = "0.02em 0.01em 0.04px", V;
    },
    mathmlBuilder(O, _) {
      var H = buildExpression(O.body, _), V = new mathMLTree.MathNode("mstyle", H);
      return V.setAttribute("style", "text-shadow: 0.02em 0.01em 0.04px"), V;
    }
  });
  var cdArrowFunctionName = {
    ">": "\\\\cdrightarrow",
    "<": "\\\\cdleftarrow",
    "=": "\\\\cdlongequal",
    A: "\\uparrow",
    V: "\\downarrow",
    "|": "\\Vert",
    ".": "no arrow"
  }, newCell = () => ({
    type: "styling",
    body: [],
    mode: "math",
    style: "display"
  }), isStartOfArrow = (O) => O.type === "textord" && O.text === "@", isLabelEnd = (O, _) => (O.type === "mathord" || O.type === "atom") && O.text === _;
  function cdArrow(O, _, H) {
    var V = cdArrowFunctionName[O];
    switch (V) {
      case "\\\\cdrightarrow":
      case "\\\\cdleftarrow":
        return H.callFunction(V, [_[0]], [_[1]]);
      case "\\uparrow":
      case "\\downarrow": {
        var q = H.callFunction("\\\\cdleft", [_[0]], []), G = {
          type: "atom",
          text: V,
          mode: "math",
          family: "rel"
        }, J = H.callFunction("\\Big", [G], []), Z = H.callFunction("\\\\cdright", [_[1]], []), te = {
          type: "ordgroup",
          mode: "math",
          body: [q, J, Z]
        };
        return H.callFunction("\\\\cdparent", [te], []);
      }
      case "\\\\cdlongequal":
        return H.callFunction("\\\\cdlongequal", [], []);
      case "\\Vert": {
        var ne = {
          type: "textord",
          text: "\\Vert",
          mode: "math"
        };
        return H.callFunction("\\Big", [ne], []);
      }
      default:
        return {
          type: "textord",
          text: " ",
          mode: "math"
        };
    }
  }
  function parseCD(O) {
    var _ = [];
    for (O.gullet.beginGroup(), O.gullet.macros.set("\\cr", "\\\\\\relax"), O.gullet.beginGroup(); ; ) {
      _.push(O.parseExpression(!1, "\\\\")), O.gullet.endGroup(), O.gullet.beginGroup();
      var H = O.fetch().text;
      if (H === "&" || H === "\\\\")
        O.consume();
      else if (H === "\\end") {
        _[_.length - 1].length === 0 && _.pop();
        break;
      } else
        throw new ParseError("Expected \\\\ or \\cr or \\end", O.nextToken);
    }
    for (var V = [], q = [V], G = 0; G < _.length; G++) {
      for (var J = _[G], Z = newCell(), te = 0; te < J.length; te++)
        if (!isStartOfArrow(J[te]))
          Z.body.push(J[te]);
        else {
          V.push(Z), te += 1;
          var ne = assertSymbolNodeType(J[te]).text, ee = new Array(2);
          if (ee[0] = {
            type: "ordgroup",
            mode: "math",
            body: []
          }, ee[1] = {
            type: "ordgroup",
            mode: "math",
            body: []
          }, !("=|.".indexOf(ne) > -1)) if ("<>AV".indexOf(ne) > -1)
            for (var ae = 0; ae < 2; ae++) {
              for (var oe = !0, se = te + 1; se < J.length; se++) {
                if (isLabelEnd(J[se], ne)) {
                  oe = !1, te = se;
                  break;
                }
                if (isStartOfArrow(J[se]))
                  throw new ParseError("Missing a " + ne + " character to complete a CD arrow.", J[se]);
                ee[ae].body.push(J[se]);
              }
              if (oe)
                throw new ParseError("Missing a " + ne + " character to complete a CD arrow.", J[te]);
            }
          else
            throw new ParseError('Expected one of "<>AV=|." after @', J[te]);
          var ce = cdArrow(ne, ee, O), le = {
            type: "styling",
            body: [ce],
            mode: "math",
            style: "display"
            // CD is always displaystyle.
          };
          V.push(le), Z = newCell();
        }
      G % 2 === 0 ? V.push(Z) : V.shift(), V = [], q.push(V);
    }
    O.gullet.endGroup(), O.gullet.endGroup();
    var ue = new Array(q[0].length).fill({
      type: "align",
      align: "c",
      pregap: 0.25,
      // CD package sets \enskip between columns.
      postgap: 0.25
      // So pre and post each get half an \enskip, i.e. 0.25em.
    });
    return {
      type: "array",
      mode: "math",
      body: q,
      arraystretch: 1,
      addJot: !0,
      rowGaps: [null],
      cols: ue,
      colSeparationType: "CD",
      hLinesBeforeRow: new Array(q.length + 1).fill([])
    };
  }
  defineFunction({
    type: "cdlabel",
    names: ["\\\\cdleft", "\\\\cdright"],
    props: {
      numArgs: 1
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O;
      return {
        type: "cdlabel",
        mode: H.mode,
        side: V.slice(4),
        label: _[0]
      };
    },
    htmlBuilder(O, _) {
      var H = _.havingStyle(_.style.sup()), V = buildCommon.wrapFragment(buildGroup$1(O.label, H, _), _);
      return V.classes.push("cd-label-" + O.side), V.style.bottom = makeEm(0.8 - V.depth), V.height = 0, V.depth = 0, V;
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mrow", [buildGroup(O.label, _)]);
      return H = new mathMLTree.MathNode("mpadded", [H]), H.setAttribute("width", "0"), O.side === "left" && H.setAttribute("lspace", "-1width"), H.setAttribute("voffset", "0.7em"), H = new mathMLTree.MathNode("mstyle", [H]), H.setAttribute("displaystyle", "false"), H.setAttribute("scriptlevel", "1"), H;
    }
  });
  defineFunction({
    type: "cdlabelparent",
    names: ["\\\\cdparent"],
    props: {
      numArgs: 1
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "cdlabelparent",
        mode: H.mode,
        fragment: _[0]
      };
    },
    htmlBuilder(O, _) {
      var H = buildCommon.wrapFragment(buildGroup$1(O.fragment, _), _);
      return H.classes.push("cd-vert-arrow"), H;
    },
    mathmlBuilder(O, _) {
      return new mathMLTree.MathNode("mrow", [buildGroup(O.fragment, _)]);
    }
  });
  defineFunction({
    type: "textord",
    names: ["\\@char"],
    props: {
      numArgs: 1,
      allowedInText: !0
    },
    handler(O, _) {
      for (var {
        parser: H
      } = O, V = assertNodeType(_[0], "ordgroup"), q = V.body, G = "", J = 0; J < q.length; J++) {
        var Z = assertNodeType(q[J], "textord");
        G += Z.text;
      }
      var te = parseInt(G), ne;
      if (isNaN(te))
        throw new ParseError("\\@char has non-numeric argument " + G);
      if (te < 0 || te >= 1114111)
        throw new ParseError("\\@char with invalid code point " + G);
      return te <= 65535 ? ne = String.fromCharCode(te) : (te -= 65536, ne = String.fromCharCode((te >> 10) + 55296, (te & 1023) + 56320)), {
        type: "textord",
        mode: H.mode,
        text: ne
      };
    }
  });
  var htmlBuilder$8 = (O, _) => {
    var H = buildExpression$1(O.body, _.withColor(O.color), !1);
    return buildCommon.makeFragment(H);
  }, mathmlBuilder$7 = (O, _) => {
    var H = buildExpression(O.body, _.withColor(O.color)), V = new mathMLTree.MathNode("mstyle", H);
    return V.setAttribute("mathcolor", O.color), V;
  };
  defineFunction({
    type: "color",
    names: ["\\textcolor"],
    props: {
      numArgs: 2,
      allowedInText: !0,
      argTypes: ["color", "original"]
    },
    handler(O, _) {
      var {
        parser: H
      } = O, V = assertNodeType(_[0], "color-token").color, q = _[1];
      return {
        type: "color",
        mode: H.mode,
        color: V,
        body: ordargument(q)
      };
    },
    htmlBuilder: htmlBuilder$8,
    mathmlBuilder: mathmlBuilder$7
  });
  defineFunction({
    type: "color",
    names: ["\\color"],
    props: {
      numArgs: 1,
      allowedInText: !0,
      argTypes: ["color"]
    },
    handler(O, _) {
      var {
        parser: H,
        breakOnTokenText: V
      } = O, q = assertNodeType(_[0], "color-token").color;
      H.gullet.macros.set("\\current@color", q);
      var G = H.parseExpression(!0, V);
      return {
        type: "color",
        mode: H.mode,
        color: q,
        body: G
      };
    },
    htmlBuilder: htmlBuilder$8,
    mathmlBuilder: mathmlBuilder$7
  });
  defineFunction({
    type: "cr",
    names: ["\\\\"],
    props: {
      numArgs: 0,
      numOptionalArgs: 0,
      allowedInText: !0
    },
    handler(O, _, H) {
      var {
        parser: V
      } = O, q = V.gullet.future().text === "[" ? V.parseSizeGroup(!0) : null, G = !V.settings.displayMode || !V.settings.useStrictBehavior("newLineInDisplayMode", "In LaTeX, \\\\ or \\newline does nothing in display mode");
      return {
        type: "cr",
        mode: V.mode,
        newLine: G,
        size: q && assertNodeType(q, "size").value
      };
    },
    // The following builders are called only at the top level,
    // not within tabular/array environments.
    htmlBuilder(O, _) {
      var H = buildCommon.makeSpan(["mspace"], [], _);
      return O.newLine && (H.classes.push("newline"), O.size && (H.style.marginTop = makeEm(calculateSize(O.size, _)))), H;
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mspace");
      return O.newLine && (H.setAttribute("linebreak", "newline"), O.size && H.setAttribute("height", makeEm(calculateSize(O.size, _)))), H;
    }
  });
  var globalMap = {
    "\\global": "\\global",
    "\\long": "\\\\globallong",
    "\\\\globallong": "\\\\globallong",
    "\\def": "\\gdef",
    "\\gdef": "\\gdef",
    "\\edef": "\\xdef",
    "\\xdef": "\\xdef",
    "\\let": "\\\\globallet",
    "\\futurelet": "\\\\globalfuture"
  }, checkControlSequence = (O) => {
    var _ = O.text;
    if (/^(?:[\\{}$&#^_]|EOF)$/.test(_))
      throw new ParseError("Expected a control sequence", O);
    return _;
  }, getRHS = (O) => {
    var _ = O.gullet.popToken();
    return _.text === "=" && (_ = O.gullet.popToken(), _.text === " " && (_ = O.gullet.popToken())), _;
  }, letCommand = (O, _, H, V) => {
    var q = O.gullet.macros.get(H.text);
    q == null && (H.noexpand = !0, q = {
      tokens: [H],
      numArgs: 0,
      // reproduce the same behavior in expansion
      unexpandable: !O.gullet.isExpandable(H.text)
    }), O.gullet.macros.set(_, q, V);
  };
  defineFunction({
    type: "internal",
    names: [
      "\\global",
      "\\long",
      "\\\\globallong"
      // can’t be entered directly
    ],
    props: {
      numArgs: 0,
      allowedInText: !0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H
      } = O;
      _.consumeSpaces();
      var V = _.fetch();
      if (globalMap[V.text])
        return (H === "\\global" || H === "\\\\globallong") && (V.text = globalMap[V.text]), assertNodeType(_.parseFunction(), "internal");
      throw new ParseError("Invalid token after macro prefix", V);
    }
  });
  defineFunction({
    type: "internal",
    names: ["\\def", "\\gdef", "\\edef", "\\xdef"],
    props: {
      numArgs: 0,
      allowedInText: !0,
      primitive: !0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H
      } = O, V = _.gullet.popToken(), q = V.text;
      if (/^(?:[\\{}$&#^_]|EOF)$/.test(q))
        throw new ParseError("Expected a control sequence", V);
      for (var G = 0, J, Z = [[]]; _.gullet.future().text !== "{"; )
        if (V = _.gullet.popToken(), V.text === "#") {
          if (_.gullet.future().text === "{") {
            J = _.gullet.future(), Z[G].push("{");
            break;
          }
          if (V = _.gullet.popToken(), !/^[1-9]$/.test(V.text))
            throw new ParseError('Invalid argument number "' + V.text + '"');
          if (parseInt(V.text) !== G + 1)
            throw new ParseError('Argument number "' + V.text + '" out of order');
          G++, Z.push([]);
        } else {
          if (V.text === "EOF")
            throw new ParseError("Expected a macro definition");
          Z[G].push(V.text);
        }
      var {
        tokens: te
      } = _.gullet.consumeArg();
      return J && te.unshift(J), (H === "\\edef" || H === "\\xdef") && (te = _.gullet.expandTokens(te), te.reverse()), _.gullet.macros.set(q, {
        tokens: te,
        numArgs: G,
        delimiters: Z
      }, H === globalMap[H]), {
        type: "internal",
        mode: _.mode
      };
    }
  });
  defineFunction({
    type: "internal",
    names: [
      "\\let",
      "\\\\globallet"
      // can’t be entered directly
    ],
    props: {
      numArgs: 0,
      allowedInText: !0,
      primitive: !0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H
      } = O, V = checkControlSequence(_.gullet.popToken());
      _.gullet.consumeSpaces();
      var q = getRHS(_);
      return letCommand(_, V, q, H === "\\\\globallet"), {
        type: "internal",
        mode: _.mode
      };
    }
  });
  defineFunction({
    type: "internal",
    names: [
      "\\futurelet",
      "\\\\globalfuture"
      // can’t be entered directly
    ],
    props: {
      numArgs: 0,
      allowedInText: !0,
      primitive: !0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H
      } = O, V = checkControlSequence(_.gullet.popToken()), q = _.gullet.popToken(), G = _.gullet.popToken();
      return letCommand(_, V, G, H === "\\\\globalfuture"), _.gullet.pushToken(G), _.gullet.pushToken(q), {
        type: "internal",
        mode: _.mode
      };
    }
  });
  var getMetrics = function O(_, H, V) {
    var q = symbols$3.math[_] && symbols$3.math[_].replace, G = getCharacterMetrics(q || _, H, V);
    if (!G)
      throw new Error("Unsupported symbol " + _ + " and font size " + H + ".");
    return G;
  }, styleWrap = function O(_, H, V, q) {
    var G = V.havingBaseStyle(H), J = buildCommon.makeSpan(q.concat(G.sizingClasses(V)), [_], V), Z = G.sizeMultiplier / V.sizeMultiplier;
    return J.height *= Z, J.depth *= Z, J.maxFontSize = G.sizeMultiplier, J;
  }, centerSpan = function O(_, H, V) {
    var q = H.havingBaseStyle(V), G = (1 - H.sizeMultiplier / q.sizeMultiplier) * H.fontMetrics().axisHeight;
    _.classes.push("delimcenter"), _.style.top = makeEm(G), _.height -= G, _.depth += G;
  }, makeSmallDelim = function O(_, H, V, q, G, J) {
    var Z = buildCommon.makeSymbol(_, "Main-Regular", G, q), te = styleWrap(Z, H, q, J);
    return V && centerSpan(te, q, H), te;
  }, mathrmSize = function O(_, H, V, q) {
    return buildCommon.makeSymbol(_, "Size" + H + "-Regular", V, q);
  }, makeLargeDelim = function O(_, H, V, q, G, J) {
    var Z = mathrmSize(_, H, G, q), te = styleWrap(buildCommon.makeSpan(["delimsizing", "size" + H], [Z], q), Style$1.TEXT, q, J);
    return V && centerSpan(te, q, Style$1.TEXT), te;
  }, makeGlyphSpan = function O(_, H, V) {
    var q;
    H === "Size1-Regular" ? q = "delim-size1" : q = "delim-size4";
    var G = buildCommon.makeSpan(["delimsizinginner", q], [buildCommon.makeSpan([], [buildCommon.makeSymbol(_, H, V)])]);
    return {
      type: "elem",
      elem: G
    };
  }, makeInner = function O(_, H, V) {
    var q = fontMetricsData["Size4-Regular"][_.charCodeAt(0)] ? fontMetricsData["Size4-Regular"][_.charCodeAt(0)][4] : fontMetricsData["Size1-Regular"][_.charCodeAt(0)][4], G = new PathNode("inner", innerPath(_, Math.round(1e3 * H))), J = new SvgNode([G], {
      width: makeEm(q),
      height: makeEm(H),
      // Override CSS rule `.katex svg { width: 100% }`
      style: "width:" + makeEm(q),
      viewBox: "0 0 " + 1e3 * q + " " + Math.round(1e3 * H),
      preserveAspectRatio: "xMinYMin"
    }), Z = buildCommon.makeSvgSpan([], [J], V);
    return Z.height = H, Z.style.height = makeEm(H), Z.style.width = makeEm(q), {
      type: "elem",
      elem: Z
    };
  }, lapInEms = 8e-3, lap = {
    type: "kern",
    size: -1 * lapInEms
  }, verts = ["|", "\\lvert", "\\rvert", "\\vert"], doubleVerts = ["\\|", "\\lVert", "\\rVert", "\\Vert"], makeStackedDelim = function O(_, H, V, q, G, J) {
    var Z, te, ne, ee, ae = "", oe = 0;
    Z = ne = ee = _, te = null;
    var se = "Size1-Regular";
    _ === "\\uparrow" ? ne = ee = "⏐" : _ === "\\Uparrow" ? ne = ee = "‖" : _ === "\\downarrow" ? Z = ne = "⏐" : _ === "\\Downarrow" ? Z = ne = "‖" : _ === "\\updownarrow" ? (Z = "\\uparrow", ne = "⏐", ee = "\\downarrow") : _ === "\\Updownarrow" ? (Z = "\\Uparrow", ne = "‖", ee = "\\Downarrow") : utils.contains(verts, _) ? (ne = "∣", ae = "vert", oe = 333) : utils.contains(doubleVerts, _) ? (ne = "∥", ae = "doublevert", oe = 556) : _ === "[" || _ === "\\lbrack" ? (Z = "⎡", ne = "⎢", ee = "⎣", se = "Size4-Regular", ae = "lbrack", oe = 667) : _ === "]" || _ === "\\rbrack" ? (Z = "⎤", ne = "⎥", ee = "⎦", se = "Size4-Regular", ae = "rbrack", oe = 667) : _ === "\\lfloor" || _ === "⌊" ? (ne = Z = "⎢", ee = "⎣", se = "Size4-Regular", ae = "lfloor", oe = 667) : _ === "\\lceil" || _ === "⌈" ? (Z = "⎡", ne = ee = "⎢", se = "Size4-Regular", ae = "lceil", oe = 667) : _ === "\\rfloor" || _ === "⌋" ? (ne = Z = "⎥", ee = "⎦", se = "Size4-Regular", ae = "rfloor", oe = 667) : _ === "\\rceil" || _ === "⌉" ? (Z = "⎤", ne = ee = "⎥", se = "Size4-Regular", ae = "rceil", oe = 667) : _ === "(" || _ === "\\lparen" ? (Z = "⎛", ne = "⎜", ee = "⎝", se = "Size4-Regular", ae = "lparen", oe = 875) : _ === ")" || _ === "\\rparen" ? (Z = "⎞", ne = "⎟", ee = "⎠", se = "Size4-Regular", ae = "rparen", oe = 875) : _ === "\\{" || _ === "\\lbrace" ? (Z = "⎧", te = "⎨", ee = "⎩", ne = "⎪", se = "Size4-Regular") : _ === "\\}" || _ === "\\rbrace" ? (Z = "⎫", te = "⎬", ee = "⎭", ne = "⎪", se = "Size4-Regular") : _ === "\\lgroup" || _ === "⟮" ? (Z = "⎧", ee = "⎩", ne = "⎪", se = "Size4-Regular") : _ === "\\rgroup" || _ === "⟯" ? (Z = "⎫", ee = "⎭", ne = "⎪", se = "Size4-Regular") : _ === "\\lmoustache" || _ === "⎰" ? (Z = "⎧", ee = "⎭", ne = "⎪", se = "Size4-Regular") : (_ === "\\rmoustache" || _ === "⎱") && (Z = "⎫", ee = "⎩", ne = "⎪", se = "Size4-Regular");
    var ce = getMetrics(Z, se, G), le = ce.height + ce.depth, ue = getMetrics(ne, se, G), de = ue.height + ue.depth, fe = getMetrics(ee, se, G), he = fe.height + fe.depth, me = 0, pe = 1;
    if (te !== null) {
      var ye = getMetrics(te, se, G);
      me = ye.height + ye.depth, pe = 2;
    }
    var ge = le + he + me, ve = Math.max(0, Math.ceil((H - ge) / (pe * de))), be = ge + ve * pe * de, Ce = q.fontMetrics().axisHeight;
    V && (Ce *= q.sizeMultiplier);
    var Se = be / 2 - Ce, we = [];
    if (ae.length > 0) {
      var Ee = be - le - he, xe = Math.round(be * 1e3), Le = tallDelim(ae, Math.round(Ee * 1e3)), Qe = new PathNode(ae, Le), ke = (oe / 1e3).toFixed(3) + "em", Me = (xe / 1e3).toFixed(3) + "em", Re = new SvgNode([Qe], {
        width: ke,
        height: Me,
        viewBox: "0 0 " + oe + " " + xe
      }), De = buildCommon.makeSvgSpan([], [Re], q);
      De.height = xe / 1e3, De.style.width = ke, De.style.height = Me, we.push({
        type: "elem",
        elem: De
      });
    } else {
      if (we.push(makeGlyphSpan(ee, se, G)), we.push(lap), te === null) {
        var Pe = be - le - he + 2 * lapInEms;
        we.push(makeInner(ne, Pe, q));
      } else {
        var $e = (be - le - he - me) / 2 + 2 * lapInEms;
        we.push(makeInner(ne, $e, q)), we.push(lap), we.push(makeGlyphSpan(te, se, G)), we.push(lap), we.push(makeInner(ne, $e, q));
      }
      we.push(lap), we.push(makeGlyphSpan(Z, se, G));
    }
    var qe = q.havingBaseStyle(Style$1.TEXT), Xe = buildCommon.makeVList({
      positionType: "bottom",
      positionData: Se,
      children: we
    }, qe);
    return styleWrap(buildCommon.makeSpan(["delimsizing", "mult"], [Xe], qe), Style$1.TEXT, q, J);
  }, vbPad = 80, emPad = 0.08, sqrtSvg = function O(_, H, V, q, G) {
    var J = sqrtPath(_, q, V), Z = new PathNode(_, J), te = new SvgNode([Z], {
      // Note: 1000:1 ratio of viewBox to document em width.
      width: "400em",
      height: makeEm(H),
      viewBox: "0 0 400000 " + V,
      preserveAspectRatio: "xMinYMin slice"
    });
    return buildCommon.makeSvgSpan(["hide-tail"], [te], G);
  }, makeSqrtImage = function O(_, H) {
    var V = H.havingBaseSizing(), q = traverseSequence("\\surd", _ * V.sizeMultiplier, stackLargeDelimiterSequence, V), G = V.sizeMultiplier, J = Math.max(0, H.minRuleThickness - H.fontMetrics().sqrtRuleThickness), Z, te = 0, ne = 0, ee = 0, ae;
    return q.type === "small" ? (ee = 1e3 + 1e3 * J + vbPad, _ < 1 ? G = 1 : _ < 1.4 && (G = 0.7), te = (1 + J + emPad) / G, ne = (1 + J) / G, Z = sqrtSvg("sqrtMain", te, ee, J, H), Z.style.minWidth = "0.853em", ae = 0.833 / G) : q.type === "large" ? (ee = (1e3 + vbPad) * sizeToMaxHeight[q.size], ne = (sizeToMaxHeight[q.size] + J) / G, te = (sizeToMaxHeight[q.size] + J + emPad) / G, Z = sqrtSvg("sqrtSize" + q.size, te, ee, J, H), Z.style.minWidth = "1.02em", ae = 1 / G) : (te = _ + J + emPad, ne = _ + J, ee = Math.floor(1e3 * _ + J) + vbPad, Z = sqrtSvg("sqrtTall", te, ee, J, H), Z.style.minWidth = "0.742em", ae = 1.056), Z.height = ne, Z.style.height = makeEm(te), {
      span: Z,
      advanceWidth: ae,
      // Calculate the actual line width.
      // This actually should depend on the chosen font -- e.g. \boldmath
      // should use the thicker surd symbols from e.g. KaTeX_Main-Bold, and
      // have thicker rules.
      ruleWidth: (H.fontMetrics().sqrtRuleThickness + J) * G
    };
  }, stackLargeDelimiters = ["(", "\\lparen", ")", "\\rparen", "[", "\\lbrack", "]", "\\rbrack", "\\{", "\\lbrace", "\\}", "\\rbrace", "\\lfloor", "\\rfloor", "⌊", "⌋", "\\lceil", "\\rceil", "⌈", "⌉", "\\surd"], stackAlwaysDelimiters = ["\\uparrow", "\\downarrow", "\\updownarrow", "\\Uparrow", "\\Downarrow", "\\Updownarrow", "|", "\\|", "\\vert", "\\Vert", "\\lvert", "\\rvert", "\\lVert", "\\rVert", "\\lgroup", "\\rgroup", "⟮", "⟯", "\\lmoustache", "\\rmoustache", "⎰", "⎱"], stackNeverDelimiters = ["<", ">", "\\langle", "\\rangle", "/", "\\backslash", "\\lt", "\\gt"], sizeToMaxHeight = [0, 1.2, 1.8, 2.4, 3], makeSizedDelim = function O(_, H, V, q, G) {
    if (_ === "<" || _ === "\\lt" || _ === "⟨" ? _ = "\\langle" : (_ === ">" || _ === "\\gt" || _ === "⟩") && (_ = "\\rangle"), utils.contains(stackLargeDelimiters, _) || utils.contains(stackNeverDelimiters, _))
      return makeLargeDelim(_, H, !1, V, q, G);
    if (utils.contains(stackAlwaysDelimiters, _))
      return makeStackedDelim(_, sizeToMaxHeight[H], !1, V, q, G);
    throw new ParseError("Illegal delimiter: '" + _ + "'");
  }, stackNeverDelimiterSequence = [{
    type: "small",
    style: Style$1.SCRIPTSCRIPT
  }, {
    type: "small",
    style: Style$1.SCRIPT
  }, {
    type: "small",
    style: Style$1.TEXT
  }, {
    type: "large",
    size: 1
  }, {
    type: "large",
    size: 2
  }, {
    type: "large",
    size: 3
  }, {
    type: "large",
    size: 4
  }], stackAlwaysDelimiterSequence = [{
    type: "small",
    style: Style$1.SCRIPTSCRIPT
  }, {
    type: "small",
    style: Style$1.SCRIPT
  }, {
    type: "small",
    style: Style$1.TEXT
  }, {
    type: "stack"
  }], stackLargeDelimiterSequence = [{
    type: "small",
    style: Style$1.SCRIPTSCRIPT
  }, {
    type: "small",
    style: Style$1.SCRIPT
  }, {
    type: "small",
    style: Style$1.TEXT
  }, {
    type: "large",
    size: 1
  }, {
    type: "large",
    size: 2
  }, {
    type: "large",
    size: 3
  }, {
    type: "large",
    size: 4
  }, {
    type: "stack"
  }], delimTypeToFont = function O(_) {
    if (_.type === "small")
      return "Main-Regular";
    if (_.type === "large")
      return "Size" + _.size + "-Regular";
    if (_.type === "stack")
      return "Size4-Regular";
    throw new Error("Add support for delim type '" + _.type + "' here.");
  }, traverseSequence = function O(_, H, V, q) {
    for (var G = Math.min(2, 3 - q.style.size), J = G; J < V.length && V[J].type !== "stack"; J++) {
      var Z = getMetrics(_, delimTypeToFont(V[J]), "math"), te = Z.height + Z.depth;
      if (V[J].type === "small") {
        var ne = q.havingBaseStyle(V[J].style);
        te *= ne.sizeMultiplier;
      }
      if (te > H)
        return V[J];
    }
    return V[V.length - 1];
  }, makeCustomSizedDelim = function O(_, H, V, q, G, J) {
    _ === "<" || _ === "\\lt" || _ === "⟨" ? _ = "\\langle" : (_ === ">" || _ === "\\gt" || _ === "⟩") && (_ = "\\rangle");
    var Z;
    utils.contains(stackNeverDelimiters, _) ? Z = stackNeverDelimiterSequence : utils.contains(stackLargeDelimiters, _) ? Z = stackLargeDelimiterSequence : Z = stackAlwaysDelimiterSequence;
    var te = traverseSequence(_, H, Z, q);
    return te.type === "small" ? makeSmallDelim(_, te.style, V, q, G, J) : te.type === "large" ? makeLargeDelim(_, te.size, V, q, G, J) : makeStackedDelim(_, H, V, q, G, J);
  }, makeLeftRightDelim = function O(_, H, V, q, G, J) {
    var Z = q.fontMetrics().axisHeight * q.sizeMultiplier, te = 901, ne = 5 / q.fontMetrics().ptPerEm, ee = Math.max(H - Z, V + Z), ae = Math.max(
      // In real TeX, calculations are done using integral values which are
      // 65536 per pt, or 655360 per em. So, the division here truncates in
      // TeX but doesn't here, producing different results. If we wanted to
      // exactly match TeX's calculation, we could do
      //   Math.floor(655360 * maxDistFromAxis / 500) *
      //    delimiterFactor / 655360
      // (To see the difference, compare
      //    x^{x^{\left(\rule{0.1em}{0.68em}\right)}}
      // in TeX and KaTeX)
      ee / 500 * te,
      2 * ee - ne
    );
    return makeCustomSizedDelim(_, ae, !0, q, G, J);
  }, delimiter = {
    sqrtImage: makeSqrtImage,
    sizedDelim: makeSizedDelim,
    sizeToMaxHeight,
    customSizedDelim: makeCustomSizedDelim,
    leftRightDelim: makeLeftRightDelim
  }, delimiterSizes = {
    "\\bigl": {
      mclass: "mopen",
      size: 1
    },
    "\\Bigl": {
      mclass: "mopen",
      size: 2
    },
    "\\biggl": {
      mclass: "mopen",
      size: 3
    },
    "\\Biggl": {
      mclass: "mopen",
      size: 4
    },
    "\\bigr": {
      mclass: "mclose",
      size: 1
    },
    "\\Bigr": {
      mclass: "mclose",
      size: 2
    },
    "\\biggr": {
      mclass: "mclose",
      size: 3
    },
    "\\Biggr": {
      mclass: "mclose",
      size: 4
    },
    "\\bigm": {
      mclass: "mrel",
      size: 1
    },
    "\\Bigm": {
      mclass: "mrel",
      size: 2
    },
    "\\biggm": {
      mclass: "mrel",
      size: 3
    },
    "\\Biggm": {
      mclass: "mrel",
      size: 4
    },
    "\\big": {
      mclass: "mord",
      size: 1
    },
    "\\Big": {
      mclass: "mord",
      size: 2
    },
    "\\bigg": {
      mclass: "mord",
      size: 3
    },
    "\\Bigg": {
      mclass: "mord",
      size: 4
    }
  }, delimiters = ["(", "\\lparen", ")", "\\rparen", "[", "\\lbrack", "]", "\\rbrack", "\\{", "\\lbrace", "\\}", "\\rbrace", "\\lfloor", "\\rfloor", "⌊", "⌋", "\\lceil", "\\rceil", "⌈", "⌉", "<", ">", "\\langle", "⟨", "\\rangle", "⟩", "\\lt", "\\gt", "\\lvert", "\\rvert", "\\lVert", "\\rVert", "\\lgroup", "\\rgroup", "⟮", "⟯", "\\lmoustache", "\\rmoustache", "⎰", "⎱", "/", "\\backslash", "|", "\\vert", "\\|", "\\Vert", "\\uparrow", "\\Uparrow", "\\downarrow", "\\Downarrow", "\\updownarrow", "\\Updownarrow", "."];
  function checkDelimiter(O, _) {
    var H = checkSymbolNodeType(O);
    if (H && utils.contains(delimiters, H.text))
      return H;
    throw H ? new ParseError("Invalid delimiter '" + H.text + "' after '" + _.funcName + "'", O) : new ParseError("Invalid delimiter type '" + O.type + "'", O);
  }
  defineFunction({
    type: "delimsizing",
    names: ["\\bigl", "\\Bigl", "\\biggl", "\\Biggl", "\\bigr", "\\Bigr", "\\biggr", "\\Biggr", "\\bigm", "\\Bigm", "\\biggm", "\\Biggm", "\\big", "\\Big", "\\bigg", "\\Bigg"],
    props: {
      numArgs: 1,
      argTypes: ["primitive"]
    },
    handler: (O, _) => {
      var H = checkDelimiter(_[0], O);
      return {
        type: "delimsizing",
        mode: O.parser.mode,
        size: delimiterSizes[O.funcName].size,
        mclass: delimiterSizes[O.funcName].mclass,
        delim: H.text
      };
    },
    htmlBuilder: (O, _) => O.delim === "." ? buildCommon.makeSpan([O.mclass]) : delimiter.sizedDelim(O.delim, O.size, _, O.mode, [O.mclass]),
    mathmlBuilder: (O) => {
      var _ = [];
      O.delim !== "." && _.push(makeText(O.delim, O.mode));
      var H = new mathMLTree.MathNode("mo", _);
      O.mclass === "mopen" || O.mclass === "mclose" ? H.setAttribute("fence", "true") : H.setAttribute("fence", "false"), H.setAttribute("stretchy", "true");
      var V = makeEm(delimiter.sizeToMaxHeight[O.size]);
      return H.setAttribute("minsize", V), H.setAttribute("maxsize", V), H;
    }
  });
  function assertParsed(O) {
    if (!O.body)
      throw new Error("Bug: The leftright ParseNode wasn't fully parsed.");
  }
  defineFunction({
    type: "leftright-right",
    names: ["\\right"],
    props: {
      numArgs: 1,
      primitive: !0
    },
    handler: (O, _) => {
      var H = O.parser.gullet.macros.get("\\current@color");
      if (H && typeof H != "string")
        throw new ParseError("\\current@color set to non-string in \\right");
      return {
        type: "leftright-right",
        mode: O.parser.mode,
        delim: checkDelimiter(_[0], O).text,
        color: H
        // undefined if not set via \color
      };
    }
  });
  defineFunction({
    type: "leftright",
    names: ["\\left"],
    props: {
      numArgs: 1,
      primitive: !0
    },
    handler: (O, _) => {
      var H = checkDelimiter(_[0], O), V = O.parser;
      ++V.leftrightDepth;
      var q = V.parseExpression(!1);
      --V.leftrightDepth, V.expect("\\right", !1);
      var G = assertNodeType(V.parseFunction(), "leftright-right");
      return {
        type: "leftright",
        mode: V.mode,
        body: q,
        left: H.text,
        right: G.delim,
        rightColor: G.color
      };
    },
    htmlBuilder: (O, _) => {
      assertParsed(O);
      for (var H = buildExpression$1(O.body, _, !0, ["mopen", "mclose"]), V = 0, q = 0, G = !1, J = 0; J < H.length; J++)
        H[J].isMiddle ? G = !0 : (V = Math.max(H[J].height, V), q = Math.max(H[J].depth, q));
      V *= _.sizeMultiplier, q *= _.sizeMultiplier;
      var Z;
      if (O.left === "." ? Z = makeNullDelimiter(_, ["mopen"]) : Z = delimiter.leftRightDelim(O.left, V, q, _, O.mode, ["mopen"]), H.unshift(Z), G)
        for (var te = 1; te < H.length; te++) {
          var ne = H[te], ee = ne.isMiddle;
          ee && (H[te] = delimiter.leftRightDelim(ee.delim, V, q, ee.options, O.mode, []));
        }
      var ae;
      if (O.right === ".")
        ae = makeNullDelimiter(_, ["mclose"]);
      else {
        var oe = O.rightColor ? _.withColor(O.rightColor) : _;
        ae = delimiter.leftRightDelim(O.right, V, q, oe, O.mode, ["mclose"]);
      }
      return H.push(ae), buildCommon.makeSpan(["minner"], H, _);
    },
    mathmlBuilder: (O, _) => {
      assertParsed(O);
      var H = buildExpression(O.body, _);
      if (O.left !== ".") {
        var V = new mathMLTree.MathNode("mo", [makeText(O.left, O.mode)]);
        V.setAttribute("fence", "true"), H.unshift(V);
      }
      if (O.right !== ".") {
        var q = new mathMLTree.MathNode("mo", [makeText(O.right, O.mode)]);
        q.setAttribute("fence", "true"), O.rightColor && q.setAttribute("mathcolor", O.rightColor), H.push(q);
      }
      return makeRow(H);
    }
  });
  defineFunction({
    type: "middle",
    names: ["\\middle"],
    props: {
      numArgs: 1,
      primitive: !0
    },
    handler: (O, _) => {
      var H = checkDelimiter(_[0], O);
      if (!O.parser.leftrightDepth)
        throw new ParseError("\\middle without preceding \\left", H);
      return {
        type: "middle",
        mode: O.parser.mode,
        delim: H.text
      };
    },
    htmlBuilder: (O, _) => {
      var H;
      if (O.delim === ".")
        H = makeNullDelimiter(_, []);
      else {
        H = delimiter.sizedDelim(O.delim, 1, _, O.mode, []);
        var V = {
          delim: O.delim,
          options: _
        };
        H.isMiddle = V;
      }
      return H;
    },
    mathmlBuilder: (O, _) => {
      var H = O.delim === "\\vert" || O.delim === "|" ? makeText("|", "text") : makeText(O.delim, O.mode), V = new mathMLTree.MathNode("mo", [H]);
      return V.setAttribute("fence", "true"), V.setAttribute("lspace", "0.05em"), V.setAttribute("rspace", "0.05em"), V;
    }
  });
  var htmlBuilder$7 = (O, _) => {
    var H = buildCommon.wrapFragment(buildGroup$1(O.body, _), _), V = O.label.slice(1), q = _.sizeMultiplier, G, J = 0, Z = utils.isCharacterBox(O.body);
    if (V === "sout")
      G = buildCommon.makeSpan(["stretchy", "sout"]), G.height = _.fontMetrics().defaultRuleThickness / q, J = -0.5 * _.fontMetrics().xHeight;
    else if (V === "phase") {
      var te = calculateSize({
        number: 0.6,
        unit: "pt"
      }, _), ne = calculateSize({
        number: 0.35,
        unit: "ex"
      }, _), ee = _.havingBaseSizing();
      q = q / ee.sizeMultiplier;
      var ae = H.height + H.depth + te + ne;
      H.style.paddingLeft = makeEm(ae / 2 + te);
      var oe = Math.floor(1e3 * ae * q), se = phasePath(oe), ce = new SvgNode([new PathNode("phase", se)], {
        width: "400em",
        height: makeEm(oe / 1e3),
        viewBox: "0 0 400000 " + oe,
        preserveAspectRatio: "xMinYMin slice"
      });
      G = buildCommon.makeSvgSpan(["hide-tail"], [ce], _), G.style.height = makeEm(ae), J = H.depth + te + ne;
    } else {
      /cancel/.test(V) ? Z || H.classes.push("cancel-pad") : V === "angl" ? H.classes.push("anglpad") : H.classes.push("boxpad");
      var le = 0, ue = 0, de = 0;
      /box/.test(V) ? (de = Math.max(
        _.fontMetrics().fboxrule,
        // default
        _.minRuleThickness
        // User override.
      ), le = _.fontMetrics().fboxsep + (V === "colorbox" ? 0 : de), ue = le) : V === "angl" ? (de = Math.max(_.fontMetrics().defaultRuleThickness, _.minRuleThickness), le = 4 * de, ue = Math.max(0, 0.25 - H.depth)) : (le = Z ? 0.2 : 0, ue = le), G = stretchy.encloseSpan(H, V, le, ue, _), /fbox|boxed|fcolorbox/.test(V) ? (G.style.borderStyle = "solid", G.style.borderWidth = makeEm(de)) : V === "angl" && de !== 0.049 && (G.style.borderTopWidth = makeEm(de), G.style.borderRightWidth = makeEm(de)), J = H.depth + ue, O.backgroundColor && (G.style.backgroundColor = O.backgroundColor, O.borderColor && (G.style.borderColor = O.borderColor));
    }
    var fe;
    if (O.backgroundColor)
      fe = buildCommon.makeVList({
        positionType: "individualShift",
        children: [
          // Put the color background behind inner;
          {
            type: "elem",
            elem: G,
            shift: J
          },
          {
            type: "elem",
            elem: H,
            shift: 0
          }
        ]
      }, _);
    else {
      var he = /cancel|phase/.test(V) ? ["svg-align"] : [];
      fe = buildCommon.makeVList({
        positionType: "individualShift",
        children: [
          // Write the \cancel stroke on top of inner.
          {
            type: "elem",
            elem: H,
            shift: 0
          },
          {
            type: "elem",
            elem: G,
            shift: J,
            wrapperClasses: he
          }
        ]
      }, _);
    }
    return /cancel/.test(V) && (fe.height = H.height, fe.depth = H.depth), /cancel/.test(V) && !Z ? buildCommon.makeSpan(["mord", "cancel-lap"], [fe], _) : buildCommon.makeSpan(["mord"], [fe], _);
  }, mathmlBuilder$6 = (O, _) => {
    var H = 0, V = new mathMLTree.MathNode(O.label.indexOf("colorbox") > -1 ? "mpadded" : "menclose", [buildGroup(O.body, _)]);
    switch (O.label) {
      case "\\cancel":
        V.setAttribute("notation", "updiagonalstrike");
        break;
      case "\\bcancel":
        V.setAttribute("notation", "downdiagonalstrike");
        break;
      case "\\phase":
        V.setAttribute("notation", "phasorangle");
        break;
      case "\\sout":
        V.setAttribute("notation", "horizontalstrike");
        break;
      case "\\fbox":
        V.setAttribute("notation", "box");
        break;
      case "\\angl":
        V.setAttribute("notation", "actuarial");
        break;
      case "\\fcolorbox":
      case "\\colorbox":
        if (H = _.fontMetrics().fboxsep * _.fontMetrics().ptPerEm, V.setAttribute("width", "+" + 2 * H + "pt"), V.setAttribute("height", "+" + 2 * H + "pt"), V.setAttribute("lspace", H + "pt"), V.setAttribute("voffset", H + "pt"), O.label === "\\fcolorbox") {
          var q = Math.max(
            _.fontMetrics().fboxrule,
            // default
            _.minRuleThickness
            // user override
          );
          V.setAttribute("style", "border: " + q + "em solid " + String(O.borderColor));
        }
        break;
      case "\\xcancel":
        V.setAttribute("notation", "updiagonalstrike downdiagonalstrike");
        break;
    }
    return O.backgroundColor && V.setAttribute("mathbackground", O.backgroundColor), V;
  };
  defineFunction({
    type: "enclose",
    names: ["\\colorbox"],
    props: {
      numArgs: 2,
      allowedInText: !0,
      argTypes: ["color", "text"]
    },
    handler(O, _, H) {
      var {
        parser: V,
        funcName: q
      } = O, G = assertNodeType(_[0], "color-token").color, J = _[1];
      return {
        type: "enclose",
        mode: V.mode,
        label: q,
        backgroundColor: G,
        body: J
      };
    },
    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineFunction({
    type: "enclose",
    names: ["\\fcolorbox"],
    props: {
      numArgs: 3,
      allowedInText: !0,
      argTypes: ["color", "color", "text"]
    },
    handler(O, _, H) {
      var {
        parser: V,
        funcName: q
      } = O, G = assertNodeType(_[0], "color-token").color, J = assertNodeType(_[1], "color-token").color, Z = _[2];
      return {
        type: "enclose",
        mode: V.mode,
        label: q,
        backgroundColor: J,
        borderColor: G,
        body: Z
      };
    },
    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineFunction({
    type: "enclose",
    names: ["\\fbox"],
    props: {
      numArgs: 1,
      argTypes: ["hbox"],
      allowedInText: !0
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "enclose",
        mode: H.mode,
        label: "\\fbox",
        body: _[0]
      };
    }
  });
  defineFunction({
    type: "enclose",
    names: ["\\cancel", "\\bcancel", "\\xcancel", "\\sout", "\\phase"],
    props: {
      numArgs: 1
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0];
      return {
        type: "enclose",
        mode: H.mode,
        label: V,
        body: q
      };
    },
    htmlBuilder: htmlBuilder$7,
    mathmlBuilder: mathmlBuilder$6
  });
  defineFunction({
    type: "enclose",
    names: ["\\angl"],
    props: {
      numArgs: 1,
      argTypes: ["hbox"],
      allowedInText: !1
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "enclose",
        mode: H.mode,
        label: "\\angl",
        body: _[0]
      };
    }
  });
  var _environments = {};
  function defineEnvironment(O) {
    for (var {
      type: _,
      names: H,
      props: V,
      handler: q,
      htmlBuilder: G,
      mathmlBuilder: J
    } = O, Z = {
      type: _,
      numArgs: V.numArgs || 0,
      allowedInText: !1,
      numOptionalArgs: 0,
      handler: q
    }, te = 0; te < H.length; ++te)
      _environments[H[te]] = Z;
    G && (_htmlGroupBuilders[_] = G), J && (_mathmlGroupBuilders[_] = J);
  }
  var _macros = {};
  function defineMacro(O, _) {
    _macros[O] = _;
  }
  function getHLines(O) {
    var _ = [];
    O.consumeSpaces();
    var H = O.fetch().text;
    for (H === "\\relax" && (O.consume(), O.consumeSpaces(), H = O.fetch().text); H === "\\hline" || H === "\\hdashline"; )
      O.consume(), _.push(H === "\\hdashline"), O.consumeSpaces(), H = O.fetch().text;
    return _;
  }
  var validateAmsEnvironmentContext = (O) => {
    var _ = O.parser.settings;
    if (!_.displayMode)
      throw new ParseError("{" + O.envName + "} can be used only in display mode.");
  };
  function getAutoTag(O) {
    if (O.indexOf("ed") === -1)
      return O.indexOf("*") === -1;
  }
  function parseArray$1(O, _, H) {
    var {
      hskipBeforeAndAfter: V,
      addJot: q,
      cols: G,
      arraystretch: J,
      colSeparationType: Z,
      autoTag: te,
      singleRow: ne,
      emptySingleRow: ee,
      maxNumCols: ae,
      leqno: oe
    } = _;
    if (O.gullet.beginGroup(), ne || O.gullet.macros.set("\\cr", "\\\\\\relax"), !J) {
      var se = O.gullet.expandMacroAsText("\\arraystretch");
      if (se == null)
        J = 1;
      else if (J = parseFloat(se), !J || J < 0)
        throw new ParseError("Invalid \\arraystretch: " + se);
    }
    O.gullet.beginGroup();
    var ce = [], le = [ce], ue = [], de = [], fe = te != null ? [] : void 0;
    function he() {
      te && O.gullet.macros.set("\\@eqnsw", "1", !0);
    }
    function me() {
      fe && (O.gullet.macros.get("\\df@tag") ? (fe.push(O.subparse([new Token("\\df@tag")])), O.gullet.macros.set("\\df@tag", void 0, !0)) : fe.push(!!te && O.gullet.macros.get("\\@eqnsw") === "1"));
    }
    for (he(), de.push(getHLines(O)); ; ) {
      var pe = O.parseExpression(!1, ne ? "\\end" : "\\\\");
      O.gullet.endGroup(), O.gullet.beginGroup(), pe = {
        type: "ordgroup",
        mode: O.mode,
        body: pe
      }, H && (pe = {
        type: "styling",
        mode: O.mode,
        style: H,
        body: [pe]
      }), ce.push(pe);
      var ye = O.fetch().text;
      if (ye === "&") {
        if (ae && ce.length === ae) {
          if (ne || Z)
            throw new ParseError("Too many tab characters: &", O.nextToken);
          O.settings.reportNonstrict("textEnv", "Too few columns specified in the {array} column argument.");
        }
        O.consume();
      } else if (ye === "\\end") {
        me(), ce.length === 1 && pe.type === "styling" && pe.body[0].body.length === 0 && (le.length > 1 || !ee) && le.pop(), de.length < le.length + 1 && de.push([]);
        break;
      } else if (ye === "\\\\") {
        O.consume();
        var ge = void 0;
        O.gullet.future().text !== " " && (ge = O.parseSizeGroup(!0)), ue.push(ge ? ge.value : null), me(), de.push(getHLines(O)), ce = [], le.push(ce), he();
      } else
        throw new ParseError("Expected & or \\\\ or \\cr or \\end", O.nextToken);
    }
    return O.gullet.endGroup(), O.gullet.endGroup(), {
      type: "array",
      mode: O.mode,
      addJot: q,
      arraystretch: J,
      body: le,
      cols: G,
      rowGaps: ue,
      hskipBeforeAndAfter: V,
      hLinesBeforeRow: de,
      colSeparationType: Z,
      tags: fe,
      leqno: oe
    };
  }
  function dCellStyle(O) {
    return O.slice(0, 1) === "d" ? "display" : "text";
  }
  var htmlBuilder$6 = function O(_, H) {
    var V, q, G = _.body.length, J = _.hLinesBeforeRow, Z = 0, te = new Array(G), ne = [], ee = Math.max(
      // From LaTeX \showthe\arrayrulewidth. Equals 0.04 em.
      H.fontMetrics().arrayRuleWidth,
      H.minRuleThickness
      // User override.
    ), ae = 1 / H.fontMetrics().ptPerEm, oe = 5 * ae;
    if (_.colSeparationType && _.colSeparationType === "small") {
      var se = H.havingStyle(Style$1.SCRIPT).sizeMultiplier;
      oe = 0.2778 * (se / H.sizeMultiplier);
    }
    var ce = _.colSeparationType === "CD" ? calculateSize({
      number: 3,
      unit: "ex"
    }, H) : 12 * ae, le = 3 * ae, ue = _.arraystretch * ce, de = 0.7 * ue, fe = 0.3 * ue, he = 0;
    function me(bt) {
      for (var yt = 0; yt < bt.length; ++yt)
        yt > 0 && (he += 0.25), ne.push({
          pos: he,
          isDashed: bt[yt]
        });
    }
    for (me(J[0]), V = 0; V < _.body.length; ++V) {
      var pe = _.body[V], ye = de, ge = fe;
      Z < pe.length && (Z = pe.length);
      var ve = new Array(pe.length);
      for (q = 0; q < pe.length; ++q) {
        var be = buildGroup$1(pe[q], H);
        ge < be.depth && (ge = be.depth), ye < be.height && (ye = be.height), ve[q] = be;
      }
      var Ce = _.rowGaps[V], Se = 0;
      Ce && (Se = calculateSize(Ce, H), Se > 0 && (Se += fe, ge < Se && (ge = Se), Se = 0)), _.addJot && (ge += le), ve.height = ye, ve.depth = ge, he += ye, ve.pos = he, he += ge + Se, te[V] = ve, me(J[V + 1]);
    }
    var we = he / 2 + H.fontMetrics().axisHeight, Ee = _.cols || [], xe = [], Le, Qe, ke = [];
    if (_.tags && _.tags.some((bt) => bt))
      for (V = 0; V < G; ++V) {
        var Me = te[V], Re = Me.pos - we, De = _.tags[V], Pe = void 0;
        De === !0 ? Pe = buildCommon.makeSpan(["eqn-num"], [], H) : De === !1 ? Pe = buildCommon.makeSpan([], [], H) : Pe = buildCommon.makeSpan([], buildExpression$1(De, H, !0), H), Pe.depth = Me.depth, Pe.height = Me.height, ke.push({
          type: "elem",
          elem: Pe,
          shift: Re
        });
      }
    for (
      q = 0, Qe = 0;
      // Continue while either there are more columns or more column
      // descriptions, so trailing separators don't get lost.
      q < Z || Qe < Ee.length;
      ++q, ++Qe
    ) {
      for (var $e = Ee[Qe] || {}, qe = !0; $e.type === "separator"; ) {
        if (qe || (Le = buildCommon.makeSpan(["arraycolsep"], []), Le.style.width = makeEm(H.fontMetrics().doubleRuleSep), xe.push(Le)), $e.separator === "|" || $e.separator === ":") {
          var Xe = $e.separator === "|" ? "solid" : "dashed", Ve = buildCommon.makeSpan(["vertical-separator"], [], H);
          Ve.style.height = makeEm(he), Ve.style.borderRightWidth = makeEm(ee), Ve.style.borderRightStyle = Xe, Ve.style.margin = "0 " + makeEm(-ee / 2);
          var Ze = he - we;
          Ze && (Ve.style.verticalAlign = makeEm(-Ze)), xe.push(Ve);
        } else
          throw new ParseError("Invalid separator type: " + $e.separator);
        Qe++, $e = Ee[Qe] || {}, qe = !1;
      }
      if (!(q >= Z)) {
        var Ge = void 0;
        (q > 0 || _.hskipBeforeAndAfter) && (Ge = utils.deflt($e.pregap, oe), Ge !== 0 && (Le = buildCommon.makeSpan(["arraycolsep"], []), Le.style.width = makeEm(Ge), xe.push(Le)));
        var ze = [];
        for (V = 0; V < G; ++V) {
          var We = te[V], at = We[q];
          if (at) {
            var ct = We.pos - we;
            at.depth = We.depth, at.height = We.height, ze.push({
              type: "elem",
              elem: at,
              shift: ct
            });
          }
        }
        ze = buildCommon.makeVList({
          positionType: "individualShift",
          children: ze
        }, H), ze = buildCommon.makeSpan(["col-align-" + ($e.align || "c")], [ze]), xe.push(ze), (q < Z - 1 || _.hskipBeforeAndAfter) && (Ge = utils.deflt($e.postgap, oe), Ge !== 0 && (Le = buildCommon.makeSpan(["arraycolsep"], []), Le.style.width = makeEm(Ge), xe.push(Le)));
      }
    }
    if (te = buildCommon.makeSpan(["mtable"], xe), ne.length > 0) {
      for (var et = buildCommon.makeLineSpan("hline", H, ee), dt = buildCommon.makeLineSpan("hdashline", H, ee), Je = [{
        type: "elem",
        elem: te,
        shift: 0
      }]; ne.length > 0; ) {
        var it = ne.pop(), st = it.pos - we;
        it.isDashed ? Je.push({
          type: "elem",
          elem: dt,
          shift: st
        }) : Je.push({
          type: "elem",
          elem: et,
          shift: st
        });
      }
      te = buildCommon.makeVList({
        positionType: "individualShift",
        children: Je
      }, H);
    }
    if (ke.length === 0)
      return buildCommon.makeSpan(["mord"], [te], H);
    var gt = buildCommon.makeVList({
      positionType: "individualShift",
      children: ke
    }, H);
    return gt = buildCommon.makeSpan(["tag"], [gt], H), buildCommon.makeFragment([te, gt]);
  }, alignMap = {
    c: "center ",
    l: "left ",
    r: "right "
  }, mathmlBuilder$5 = function O(_, H) {
    for (var V = [], q = new mathMLTree.MathNode("mtd", [], ["mtr-glue"]), G = new mathMLTree.MathNode("mtd", [], ["mml-eqn-num"]), J = 0; J < _.body.length; J++) {
      for (var Z = _.body[J], te = [], ne = 0; ne < Z.length; ne++)
        te.push(new mathMLTree.MathNode("mtd", [buildGroup(Z[ne], H)]));
      _.tags && _.tags[J] && (te.unshift(q), te.push(q), _.leqno ? te.unshift(G) : te.push(G)), V.push(new mathMLTree.MathNode("mtr", te));
    }
    var ee = new mathMLTree.MathNode("mtable", V), ae = _.arraystretch === 0.5 ? 0.1 : 0.16 + _.arraystretch - 1 + (_.addJot ? 0.09 : 0);
    ee.setAttribute("rowspacing", makeEm(ae));
    var oe = "", se = "";
    if (_.cols && _.cols.length > 0) {
      var ce = _.cols, le = "", ue = !1, de = 0, fe = ce.length;
      ce[0].type === "separator" && (oe += "top ", de = 1), ce[ce.length - 1].type === "separator" && (oe += "bottom ", fe -= 1);
      for (var he = de; he < fe; he++)
        ce[he].type === "align" ? (se += alignMap[ce[he].align], ue && (le += "none "), ue = !0) : ce[he].type === "separator" && ue && (le += ce[he].separator === "|" ? "solid " : "dashed ", ue = !1);
      ee.setAttribute("columnalign", se.trim()), /[sd]/.test(le) && ee.setAttribute("columnlines", le.trim());
    }
    if (_.colSeparationType === "align") {
      for (var me = _.cols || [], pe = "", ye = 1; ye < me.length; ye++)
        pe += ye % 2 ? "0em " : "1em ";
      ee.setAttribute("columnspacing", pe.trim());
    } else _.colSeparationType === "alignat" || _.colSeparationType === "gather" ? ee.setAttribute("columnspacing", "0em") : _.colSeparationType === "small" ? ee.setAttribute("columnspacing", "0.2778em") : _.colSeparationType === "CD" ? ee.setAttribute("columnspacing", "0.5em") : ee.setAttribute("columnspacing", "1em");
    var ge = "", ve = _.hLinesBeforeRow;
    oe += ve[0].length > 0 ? "left " : "", oe += ve[ve.length - 1].length > 0 ? "right " : "";
    for (var be = 1; be < ve.length - 1; be++)
      ge += ve[be].length === 0 ? "none " : ve[be][0] ? "dashed " : "solid ";
    return /[sd]/.test(ge) && ee.setAttribute("rowlines", ge.trim()), oe !== "" && (ee = new mathMLTree.MathNode("menclose", [ee]), ee.setAttribute("notation", oe.trim())), _.arraystretch && _.arraystretch < 1 && (ee = new mathMLTree.MathNode("mstyle", [ee]), ee.setAttribute("scriptlevel", "1")), ee;
  }, alignedHandler = function O(_, H) {
    _.envName.indexOf("ed") === -1 && validateAmsEnvironmentContext(_);
    var V = [], q = _.envName.indexOf("at") > -1 ? "alignat" : "align", G = _.envName === "split", J = parseArray$1(_.parser, {
      cols: V,
      addJot: !0,
      autoTag: G ? void 0 : getAutoTag(_.envName),
      emptySingleRow: !0,
      colSeparationType: q,
      maxNumCols: G ? 2 : void 0,
      leqno: _.parser.settings.leqno
    }, "display"), Z, te = 0, ne = {
      type: "ordgroup",
      mode: _.mode,
      body: []
    };
    if (H[0] && H[0].type === "ordgroup") {
      for (var ee = "", ae = 0; ae < H[0].body.length; ae++) {
        var oe = assertNodeType(H[0].body[ae], "textord");
        ee += oe.text;
      }
      Z = Number(ee), te = Z * 2;
    }
    var se = !te;
    J.body.forEach(function(de) {
      for (var fe = 1; fe < de.length; fe += 2) {
        var he = assertNodeType(de[fe], "styling"), me = assertNodeType(he.body[0], "ordgroup");
        me.body.unshift(ne);
      }
      if (se)
        te < de.length && (te = de.length);
      else {
        var pe = de.length / 2;
        if (Z < pe)
          throw new ParseError("Too many math in a row: " + ("expected " + Z + ", but got " + pe), de[0]);
      }
    });
    for (var ce = 0; ce < te; ++ce) {
      var le = "r", ue = 0;
      ce % 2 === 1 ? le = "l" : ce > 0 && se && (ue = 1), V[ce] = {
        type: "align",
        align: le,
        pregap: ue,
        postgap: 0
      };
    }
    return J.colSeparationType = se ? "align" : "alignat", J;
  };
  defineEnvironment({
    type: "array",
    names: ["array", "darray"],
    props: {
      numArgs: 1
    },
    handler(O, _) {
      var H = checkSymbolNodeType(_[0]), V = H ? [_[0]] : assertNodeType(_[0], "ordgroup").body, q = V.map(function(J) {
        var Z = assertSymbolNodeType(J), te = Z.text;
        if ("lcr".indexOf(te) !== -1)
          return {
            type: "align",
            align: te
          };
        if (te === "|")
          return {
            type: "separator",
            separator: "|"
          };
        if (te === ":")
          return {
            type: "separator",
            separator: ":"
          };
        throw new ParseError("Unknown column alignment: " + te, J);
      }), G = {
        cols: q,
        hskipBeforeAndAfter: !0,
        // \@preamble in lttab.dtx
        maxNumCols: q.length
      };
      return parseArray$1(O.parser, G, dCellStyle(O.envName));
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["matrix", "pmatrix", "bmatrix", "Bmatrix", "vmatrix", "Vmatrix", "matrix*", "pmatrix*", "bmatrix*", "Bmatrix*", "vmatrix*", "Vmatrix*"],
    props: {
      numArgs: 0
    },
    handler(O) {
      var _ = {
        matrix: null,
        pmatrix: ["(", ")"],
        bmatrix: ["[", "]"],
        Bmatrix: ["\\{", "\\}"],
        vmatrix: ["|", "|"],
        Vmatrix: ["\\Vert", "\\Vert"]
      }[O.envName.replace("*", "")], H = "c", V = {
        hskipBeforeAndAfter: !1,
        cols: [{
          type: "align",
          align: H
        }]
      };
      if (O.envName.charAt(O.envName.length - 1) === "*") {
        var q = O.parser;
        if (q.consumeSpaces(), q.fetch().text === "[") {
          if (q.consume(), q.consumeSpaces(), H = q.fetch().text, "lcr".indexOf(H) === -1)
            throw new ParseError("Expected l or c or r", q.nextToken);
          q.consume(), q.consumeSpaces(), q.expect("]"), q.consume(), V.cols = [{
            type: "align",
            align: H
          }];
        }
      }
      var G = parseArray$1(O.parser, V, dCellStyle(O.envName)), J = Math.max(0, ...G.body.map((Z) => Z.length));
      return G.cols = new Array(J).fill({
        type: "align",
        align: H
      }), _ ? {
        type: "leftright",
        mode: O.mode,
        body: [G],
        left: _[0],
        right: _[1],
        rightColor: void 0
        // \right uninfluenced by \color in array
      } : G;
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["smallmatrix"],
    props: {
      numArgs: 0
    },
    handler(O) {
      var _ = {
        arraystretch: 0.5
      }, H = parseArray$1(O.parser, _, "script");
      return H.colSeparationType = "small", H;
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["subarray"],
    props: {
      numArgs: 1
    },
    handler(O, _) {
      var H = checkSymbolNodeType(_[0]), V = H ? [_[0]] : assertNodeType(_[0], "ordgroup").body, q = V.map(function(J) {
        var Z = assertSymbolNodeType(J), te = Z.text;
        if ("lc".indexOf(te) !== -1)
          return {
            type: "align",
            align: te
          };
        throw new ParseError("Unknown column alignment: " + te, J);
      });
      if (q.length > 1)
        throw new ParseError("{subarray} can contain only one column");
      var G = {
        cols: q,
        hskipBeforeAndAfter: !1,
        arraystretch: 0.5
      };
      if (G = parseArray$1(O.parser, G, "script"), G.body.length > 0 && G.body[0].length > 1)
        throw new ParseError("{subarray} can contain only one column");
      return G;
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["cases", "dcases", "rcases", "drcases"],
    props: {
      numArgs: 0
    },
    handler(O) {
      var _ = {
        arraystretch: 1.2,
        cols: [{
          type: "align",
          align: "l",
          pregap: 0,
          // TODO(kevinb) get the current style.
          // For now we use the metrics for TEXT style which is what we were
          // doing before.  Before attempting to get the current style we
          // should look at TeX's behavior especially for \over and matrices.
          postgap: 1
          /* 1em quad */
        }, {
          type: "align",
          align: "l",
          pregap: 0,
          postgap: 0
        }]
      }, H = parseArray$1(O.parser, _, dCellStyle(O.envName));
      return {
        type: "leftright",
        mode: O.mode,
        body: [H],
        left: O.envName.indexOf("r") > -1 ? "." : "\\{",
        right: O.envName.indexOf("r") > -1 ? "\\}" : ".",
        rightColor: void 0
      };
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["align", "align*", "aligned", "split"],
    props: {
      numArgs: 0
    },
    handler: alignedHandler,
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["gathered", "gather", "gather*"],
    props: {
      numArgs: 0
    },
    handler(O) {
      utils.contains(["gather", "gather*"], O.envName) && validateAmsEnvironmentContext(O);
      var _ = {
        cols: [{
          type: "align",
          align: "c"
        }],
        addJot: !0,
        colSeparationType: "gather",
        autoTag: getAutoTag(O.envName),
        emptySingleRow: !0,
        leqno: O.parser.settings.leqno
      };
      return parseArray$1(O.parser, _, "display");
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["alignat", "alignat*", "alignedat"],
    props: {
      numArgs: 1
    },
    handler: alignedHandler,
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["equation", "equation*"],
    props: {
      numArgs: 0
    },
    handler(O) {
      validateAmsEnvironmentContext(O);
      var _ = {
        autoTag: getAutoTag(O.envName),
        emptySingleRow: !0,
        singleRow: !0,
        maxNumCols: 1,
        leqno: O.parser.settings.leqno
      };
      return parseArray$1(O.parser, _, "display");
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineEnvironment({
    type: "array",
    names: ["CD"],
    props: {
      numArgs: 0
    },
    handler(O) {
      return validateAmsEnvironmentContext(O), parseCD(O.parser);
    },
    htmlBuilder: htmlBuilder$6,
    mathmlBuilder: mathmlBuilder$5
  });
  defineMacro("\\nonumber", "\\gdef\\@eqnsw{0}");
  defineMacro("\\notag", "\\nonumber");
  defineFunction({
    type: "text",
    // Doesn't matter what this is.
    names: ["\\hline", "\\hdashline"],
    props: {
      numArgs: 0,
      allowedInText: !0,
      allowedInMath: !0
    },
    handler(O, _) {
      throw new ParseError(O.funcName + " valid only within array environment");
    }
  });
  var environments = _environments;
  defineFunction({
    type: "environment",
    names: ["\\begin", "\\end"],
    props: {
      numArgs: 1,
      argTypes: ["text"]
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0];
      if (q.type !== "ordgroup")
        throw new ParseError("Invalid environment name", q);
      for (var G = "", J = 0; J < q.body.length; ++J)
        G += assertNodeType(q.body[J], "textord").text;
      if (V === "\\begin") {
        if (!environments.hasOwnProperty(G))
          throw new ParseError("No such environment: " + G, q);
        var Z = environments[G], {
          args: te,
          optArgs: ne
        } = H.parseArguments("\\begin{" + G + "}", Z), ee = {
          mode: H.mode,
          envName: G,
          parser: H
        }, ae = Z.handler(ee, te, ne);
        H.expect("\\end", !1);
        var oe = H.nextToken, se = assertNodeType(H.parseFunction(), "environment");
        if (se.name !== G)
          throw new ParseError("Mismatch: \\begin{" + G + "} matched by \\end{" + se.name + "}", oe);
        return ae;
      }
      return {
        type: "environment",
        mode: H.mode,
        name: G,
        nameGroup: q
      };
    }
  });
  var htmlBuilder$5 = (O, _) => {
    var H = O.font, V = _.withFont(H);
    return buildGroup$1(O.body, V);
  }, mathmlBuilder$4 = (O, _) => {
    var H = O.font, V = _.withFont(H);
    return buildGroup(O.body, V);
  }, fontAliases = {
    "\\Bbb": "\\mathbb",
    "\\bold": "\\mathbf",
    "\\frak": "\\mathfrak",
    "\\bm": "\\boldsymbol"
  };
  defineFunction({
    type: "font",
    names: [
      // styles, except \boldsymbol defined below
      "\\mathrm",
      "\\mathit",
      "\\mathbf",
      "\\mathnormal",
      "\\mathsfit",
      // families
      "\\mathbb",
      "\\mathcal",
      "\\mathfrak",
      "\\mathscr",
      "\\mathsf",
      "\\mathtt",
      // aliases, except \bm defined below
      "\\Bbb",
      "\\bold",
      "\\frak"
    ],
    props: {
      numArgs: 1,
      allowedInArgument: !0
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = normalizeArgument(_[0]), G = V;
      return G in fontAliases && (G = fontAliases[G]), {
        type: "font",
        mode: H.mode,
        font: G.slice(1),
        body: q
      };
    },
    htmlBuilder: htmlBuilder$5,
    mathmlBuilder: mathmlBuilder$4
  });
  defineFunction({
    type: "mclass",
    names: ["\\boldsymbol", "\\bm"],
    props: {
      numArgs: 1
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O, V = _[0], q = utils.isCharacterBox(V);
      return {
        type: "mclass",
        mode: H.mode,
        mclass: binrelClass(V),
        body: [{
          type: "font",
          mode: H.mode,
          font: "boldsymbol",
          body: V
        }],
        isCharacterBox: q
      };
    }
  });
  defineFunction({
    type: "font",
    names: ["\\rm", "\\sf", "\\tt", "\\bf", "\\it", "\\cal"],
    props: {
      numArgs: 0,
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V,
        breakOnTokenText: q
      } = O, {
        mode: G
      } = H, J = H.parseExpression(!0, q), Z = "math" + V.slice(1);
      return {
        type: "font",
        mode: G,
        font: Z,
        body: {
          type: "ordgroup",
          mode: H.mode,
          body: J
        }
      };
    },
    htmlBuilder: htmlBuilder$5,
    mathmlBuilder: mathmlBuilder$4
  });
  var adjustStyle = (O, _) => {
    var H = _;
    return O === "display" ? H = H.id >= Style$1.SCRIPT.id ? H.text() : Style$1.DISPLAY : O === "text" && H.size === Style$1.DISPLAY.size ? H = Style$1.TEXT : O === "script" ? H = Style$1.SCRIPT : O === "scriptscript" && (H = Style$1.SCRIPTSCRIPT), H;
  }, htmlBuilder$4 = (O, _) => {
    var H = adjustStyle(O.size, _.style), V = H.fracNum(), q = H.fracDen(), G;
    G = _.havingStyle(V);
    var J = buildGroup$1(O.numer, G, _);
    if (O.continued) {
      var Z = 8.5 / _.fontMetrics().ptPerEm, te = 3.5 / _.fontMetrics().ptPerEm;
      J.height = J.height < Z ? Z : J.height, J.depth = J.depth < te ? te : J.depth;
    }
    G = _.havingStyle(q);
    var ne = buildGroup$1(O.denom, G, _), ee, ae, oe;
    O.hasBarLine ? (O.barSize ? (ae = calculateSize(O.barSize, _), ee = buildCommon.makeLineSpan("frac-line", _, ae)) : ee = buildCommon.makeLineSpan("frac-line", _), ae = ee.height, oe = ee.height) : (ee = null, ae = 0, oe = _.fontMetrics().defaultRuleThickness);
    var se, ce, le;
    H.size === Style$1.DISPLAY.size || O.size === "display" ? (se = _.fontMetrics().num1, ae > 0 ? ce = 3 * oe : ce = 7 * oe, le = _.fontMetrics().denom1) : (ae > 0 ? (se = _.fontMetrics().num2, ce = oe) : (se = _.fontMetrics().num3, ce = 3 * oe), le = _.fontMetrics().denom2);
    var ue;
    if (ee) {
      var fe = _.fontMetrics().axisHeight;
      se - J.depth - (fe + 0.5 * ae) < ce && (se += ce - (se - J.depth - (fe + 0.5 * ae))), fe - 0.5 * ae - (ne.height - le) < ce && (le += ce - (fe - 0.5 * ae - (ne.height - le)));
      var he = -(fe - 0.5 * ae);
      ue = buildCommon.makeVList({
        positionType: "individualShift",
        children: [{
          type: "elem",
          elem: ne,
          shift: le
        }, {
          type: "elem",
          elem: ee,
          shift: he
        }, {
          type: "elem",
          elem: J,
          shift: -se
        }]
      }, _);
    } else {
      var de = se - J.depth - (ne.height - le);
      de < ce && (se += 0.5 * (ce - de), le += 0.5 * (ce - de)), ue = buildCommon.makeVList({
        positionType: "individualShift",
        children: [{
          type: "elem",
          elem: ne,
          shift: le
        }, {
          type: "elem",
          elem: J,
          shift: -se
        }]
      }, _);
    }
    G = _.havingStyle(H), ue.height *= G.sizeMultiplier / _.sizeMultiplier, ue.depth *= G.sizeMultiplier / _.sizeMultiplier;
    var me;
    H.size === Style$1.DISPLAY.size ? me = _.fontMetrics().delim1 : H.size === Style$1.SCRIPTSCRIPT.size ? me = _.havingStyle(Style$1.SCRIPT).fontMetrics().delim2 : me = _.fontMetrics().delim2;
    var pe, ye;
    return O.leftDelim == null ? pe = makeNullDelimiter(_, ["mopen"]) : pe = delimiter.customSizedDelim(O.leftDelim, me, !0, _.havingStyle(H), O.mode, ["mopen"]), O.continued ? ye = buildCommon.makeSpan([]) : O.rightDelim == null ? ye = makeNullDelimiter(_, ["mclose"]) : ye = delimiter.customSizedDelim(O.rightDelim, me, !0, _.havingStyle(H), O.mode, ["mclose"]), buildCommon.makeSpan(["mord"].concat(G.sizingClasses(_)), [pe, buildCommon.makeSpan(["mfrac"], [ue]), ye], _);
  }, mathmlBuilder$3 = (O, _) => {
    var H = new mathMLTree.MathNode("mfrac", [buildGroup(O.numer, _), buildGroup(O.denom, _)]);
    if (!O.hasBarLine)
      H.setAttribute("linethickness", "0px");
    else if (O.barSize) {
      var V = calculateSize(O.barSize, _);
      H.setAttribute("linethickness", makeEm(V));
    }
    var q = adjustStyle(O.size, _.style);
    if (q.size !== _.style.size) {
      H = new mathMLTree.MathNode("mstyle", [H]);
      var G = q.size === Style$1.DISPLAY.size ? "true" : "false";
      H.setAttribute("displaystyle", G), H.setAttribute("scriptlevel", "0");
    }
    if (O.leftDelim != null || O.rightDelim != null) {
      var J = [];
      if (O.leftDelim != null) {
        var Z = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode(O.leftDelim.replace("\\", ""))]);
        Z.setAttribute("fence", "true"), J.push(Z);
      }
      if (J.push(H), O.rightDelim != null) {
        var te = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode(O.rightDelim.replace("\\", ""))]);
        te.setAttribute("fence", "true"), J.push(te);
      }
      return makeRow(J);
    }
    return H;
  };
  defineFunction({
    type: "genfrac",
    names: [
      "\\dfrac",
      "\\frac",
      "\\tfrac",
      "\\dbinom",
      "\\binom",
      "\\tbinom",
      "\\\\atopfrac",
      // can’t be entered directly
      "\\\\bracefrac",
      "\\\\brackfrac"
      // ditto
    ],
    props: {
      numArgs: 2,
      allowedInArgument: !0
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0], G = _[1], J, Z = null, te = null, ne = "auto";
      switch (V) {
        case "\\dfrac":
        case "\\frac":
        case "\\tfrac":
          J = !0;
          break;
        case "\\\\atopfrac":
          J = !1;
          break;
        case "\\dbinom":
        case "\\binom":
        case "\\tbinom":
          J = !1, Z = "(", te = ")";
          break;
        case "\\\\bracefrac":
          J = !1, Z = "\\{", te = "\\}";
          break;
        case "\\\\brackfrac":
          J = !1, Z = "[", te = "]";
          break;
        default:
          throw new Error("Unrecognized genfrac command");
      }
      switch (V) {
        case "\\dfrac":
        case "\\dbinom":
          ne = "display";
          break;
        case "\\tfrac":
        case "\\tbinom":
          ne = "text";
          break;
      }
      return {
        type: "genfrac",
        mode: H.mode,
        continued: !1,
        numer: q,
        denom: G,
        hasBarLine: J,
        leftDelim: Z,
        rightDelim: te,
        size: ne,
        barSize: null
      };
    },
    htmlBuilder: htmlBuilder$4,
    mathmlBuilder: mathmlBuilder$3
  });
  defineFunction({
    type: "genfrac",
    names: ["\\cfrac"],
    props: {
      numArgs: 2
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0], G = _[1];
      return {
        type: "genfrac",
        mode: H.mode,
        continued: !0,
        numer: q,
        denom: G,
        hasBarLine: !0,
        leftDelim: null,
        rightDelim: null,
        size: "display",
        barSize: null
      };
    }
  });
  defineFunction({
    type: "infix",
    names: ["\\over", "\\choose", "\\atop", "\\brace", "\\brack"],
    props: {
      numArgs: 0,
      infix: !0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H,
        token: V
      } = O, q;
      switch (H) {
        case "\\over":
          q = "\\frac";
          break;
        case "\\choose":
          q = "\\binom";
          break;
        case "\\atop":
          q = "\\\\atopfrac";
          break;
        case "\\brace":
          q = "\\\\bracefrac";
          break;
        case "\\brack":
          q = "\\\\brackfrac";
          break;
        default:
          throw new Error("Unrecognized infix genfrac command");
      }
      return {
        type: "infix",
        mode: _.mode,
        replaceWith: q,
        token: V
      };
    }
  });
  var stylArray = ["display", "text", "script", "scriptscript"], delimFromValue = function O(_) {
    var H = null;
    return _.length > 0 && (H = _, H = H === "." ? null : H), H;
  };
  defineFunction({
    type: "genfrac",
    names: ["\\genfrac"],
    props: {
      numArgs: 6,
      allowedInArgument: !0,
      argTypes: ["math", "math", "size", "text", "math", "math"]
    },
    handler(O, _) {
      var {
        parser: H
      } = O, V = _[4], q = _[5], G = normalizeArgument(_[0]), J = G.type === "atom" && G.family === "open" ? delimFromValue(G.text) : null, Z = normalizeArgument(_[1]), te = Z.type === "atom" && Z.family === "close" ? delimFromValue(Z.text) : null, ne = assertNodeType(_[2], "size"), ee, ae = null;
      ne.isBlank ? ee = !0 : (ae = ne.value, ee = ae.number > 0);
      var oe = "auto", se = _[3];
      if (se.type === "ordgroup") {
        if (se.body.length > 0) {
          var ce = assertNodeType(se.body[0], "textord");
          oe = stylArray[Number(ce.text)];
        }
      } else
        se = assertNodeType(se, "textord"), oe = stylArray[Number(se.text)];
      return {
        type: "genfrac",
        mode: H.mode,
        numer: V,
        denom: q,
        continued: !1,
        hasBarLine: ee,
        barSize: ae,
        leftDelim: J,
        rightDelim: te,
        size: oe
      };
    },
    htmlBuilder: htmlBuilder$4,
    mathmlBuilder: mathmlBuilder$3
  });
  defineFunction({
    type: "infix",
    names: ["\\above"],
    props: {
      numArgs: 1,
      argTypes: ["size"],
      infix: !0
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V,
        token: q
      } = O;
      return {
        type: "infix",
        mode: H.mode,
        replaceWith: "\\\\abovefrac",
        size: assertNodeType(_[0], "size").value,
        token: q
      };
    }
  });
  defineFunction({
    type: "genfrac",
    names: ["\\\\abovefrac"],
    props: {
      numArgs: 3,
      argTypes: ["math", "size", "math"]
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0], G = assert$1(assertNodeType(_[1], "infix").size), J = _[2], Z = G.number > 0;
      return {
        type: "genfrac",
        mode: H.mode,
        numer: q,
        denom: J,
        continued: !1,
        hasBarLine: Z,
        barSize: G,
        leftDelim: null,
        rightDelim: null,
        size: "auto"
      };
    },
    htmlBuilder: htmlBuilder$4,
    mathmlBuilder: mathmlBuilder$3
  });
  var htmlBuilder$3 = (O, _) => {
    var H = _.style, V, q;
    O.type === "supsub" ? (V = O.sup ? buildGroup$1(O.sup, _.havingStyle(H.sup()), _) : buildGroup$1(O.sub, _.havingStyle(H.sub()), _), q = assertNodeType(O.base, "horizBrace")) : q = assertNodeType(O, "horizBrace");
    var G = buildGroup$1(q.base, _.havingBaseStyle(Style$1.DISPLAY)), J = stretchy.svgSpan(q, _), Z;
    if (q.isOver ? (Z = buildCommon.makeVList({
      positionType: "firstBaseline",
      children: [{
        type: "elem",
        elem: G
      }, {
        type: "kern",
        size: 0.1
      }, {
        type: "elem",
        elem: J
      }]
    }, _), Z.children[0].children[0].children[1].classes.push("svg-align")) : (Z = buildCommon.makeVList({
      positionType: "bottom",
      positionData: G.depth + 0.1 + J.height,
      children: [{
        type: "elem",
        elem: J
      }, {
        type: "kern",
        size: 0.1
      }, {
        type: "elem",
        elem: G
      }]
    }, _), Z.children[0].children[0].children[0].classes.push("svg-align")), V) {
      var te = buildCommon.makeSpan(["mord", q.isOver ? "mover" : "munder"], [Z], _);
      q.isOver ? Z = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: te
        }, {
          type: "kern",
          size: 0.2
        }, {
          type: "elem",
          elem: V
        }]
      }, _) : Z = buildCommon.makeVList({
        positionType: "bottom",
        positionData: te.depth + 0.2 + V.height + V.depth,
        children: [{
          type: "elem",
          elem: V
        }, {
          type: "kern",
          size: 0.2
        }, {
          type: "elem",
          elem: te
        }]
      }, _);
    }
    return buildCommon.makeSpan(["mord", q.isOver ? "mover" : "munder"], [Z], _);
  }, mathmlBuilder$2 = (O, _) => {
    var H = stretchy.mathMLnode(O.label);
    return new mathMLTree.MathNode(O.isOver ? "mover" : "munder", [buildGroup(O.base, _), H]);
  };
  defineFunction({
    type: "horizBrace",
    names: ["\\overbrace", "\\underbrace"],
    props: {
      numArgs: 1
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O;
      return {
        type: "horizBrace",
        mode: H.mode,
        label: V,
        isOver: /^\\over/.test(V),
        base: _[0]
      };
    },
    htmlBuilder: htmlBuilder$3,
    mathmlBuilder: mathmlBuilder$2
  });
  defineFunction({
    type: "href",
    names: ["\\href"],
    props: {
      numArgs: 2,
      argTypes: ["url", "original"],
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O, V = _[1], q = assertNodeType(_[0], "url").url;
      return H.settings.isTrusted({
        command: "\\href",
        url: q
      }) ? {
        type: "href",
        mode: H.mode,
        href: q,
        body: ordargument(V)
      } : H.formatUnsupportedCmd("\\href");
    },
    htmlBuilder: (O, _) => {
      var H = buildExpression$1(O.body, _, !1);
      return buildCommon.makeAnchor(O.href, [], H, _);
    },
    mathmlBuilder: (O, _) => {
      var H = buildExpressionRow(O.body, _);
      return H instanceof MathNode || (H = new MathNode("mrow", [H])), H.setAttribute("href", O.href), H;
    }
  });
  defineFunction({
    type: "href",
    names: ["\\url"],
    props: {
      numArgs: 1,
      argTypes: ["url"],
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O, V = assertNodeType(_[0], "url").url;
      if (!H.settings.isTrusted({
        command: "\\url",
        url: V
      }))
        return H.formatUnsupportedCmd("\\url");
      for (var q = [], G = 0; G < V.length; G++) {
        var J = V[G];
        J === "~" && (J = "\\textasciitilde"), q.push({
          type: "textord",
          mode: "text",
          text: J
        });
      }
      var Z = {
        type: "text",
        mode: H.mode,
        font: "\\texttt",
        body: q
      };
      return {
        type: "href",
        mode: H.mode,
        href: V,
        body: ordargument(Z)
      };
    }
  });
  defineFunction({
    type: "hbox",
    names: ["\\hbox"],
    props: {
      numArgs: 1,
      argTypes: ["text"],
      allowedInText: !0,
      primitive: !0
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "hbox",
        mode: H.mode,
        body: ordargument(_[0])
      };
    },
    htmlBuilder(O, _) {
      var H = buildExpression$1(O.body, _, !1);
      return buildCommon.makeFragment(H);
    },
    mathmlBuilder(O, _) {
      return new mathMLTree.MathNode("mrow", buildExpression(O.body, _));
    }
  });
  defineFunction({
    type: "html",
    names: ["\\htmlClass", "\\htmlId", "\\htmlStyle", "\\htmlData"],
    props: {
      numArgs: 2,
      argTypes: ["raw", "original"],
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V,
        token: q
      } = O, G = assertNodeType(_[0], "raw").string, J = _[1];
      H.settings.strict && H.settings.reportNonstrict("htmlExtension", "HTML extension is disabled on strict mode");
      var Z, te = {};
      switch (V) {
        case "\\htmlClass":
          te.class = G, Z = {
            command: "\\htmlClass",
            class: G
          };
          break;
        case "\\htmlId":
          te.id = G, Z = {
            command: "\\htmlId",
            id: G
          };
          break;
        case "\\htmlStyle":
          te.style = G, Z = {
            command: "\\htmlStyle",
            style: G
          };
          break;
        case "\\htmlData": {
          for (var ne = G.split(","), ee = 0; ee < ne.length; ee++) {
            var ae = ne[ee].split("=");
            if (ae.length !== 2)
              throw new ParseError("Error parsing key-value for \\htmlData");
            te["data-" + ae[0].trim()] = ae[1].trim();
          }
          Z = {
            command: "\\htmlData",
            attributes: te
          };
          break;
        }
        default:
          throw new Error("Unrecognized html command");
      }
      return H.settings.isTrusted(Z) ? {
        type: "html",
        mode: H.mode,
        attributes: te,
        body: ordargument(J)
      } : H.formatUnsupportedCmd(V);
    },
    htmlBuilder: (O, _) => {
      var H = buildExpression$1(O.body, _, !1), V = ["enclosing"];
      O.attributes.class && V.push(...O.attributes.class.trim().split(/\s+/));
      var q = buildCommon.makeSpan(V, H, _);
      for (var G in O.attributes)
        G !== "class" && O.attributes.hasOwnProperty(G) && q.setAttribute(G, O.attributes[G]);
      return q;
    },
    mathmlBuilder: (O, _) => buildExpressionRow(O.body, _)
  });
  defineFunction({
    type: "htmlmathml",
    names: ["\\html@mathml"],
    props: {
      numArgs: 2,
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O;
      return {
        type: "htmlmathml",
        mode: H.mode,
        html: ordargument(_[0]),
        mathml: ordargument(_[1])
      };
    },
    htmlBuilder: (O, _) => {
      var H = buildExpression$1(O.html, _, !1);
      return buildCommon.makeFragment(H);
    },
    mathmlBuilder: (O, _) => buildExpressionRow(O.mathml, _)
  });
  var sizeData = function O(_) {
    if (/^[-+]? *(\d+(\.\d*)?|\.\d+)$/.test(_))
      return {
        number: +_,
        unit: "bp"
      };
    var H = /([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(_);
    if (!H)
      throw new ParseError("Invalid size: '" + _ + "' in \\includegraphics");
    var V = {
      number: +(H[1] + H[2]),
      // sign + magnitude, cast to number
      unit: H[3]
    };
    if (!validUnit(V))
      throw new ParseError("Invalid unit: '" + V.unit + "' in \\includegraphics.");
    return V;
  };
  defineFunction({
    type: "includegraphics",
    names: ["\\includegraphics"],
    props: {
      numArgs: 1,
      numOptionalArgs: 1,
      argTypes: ["raw", "url"],
      allowedInText: !1
    },
    handler: (O, _, H) => {
      var {
        parser: V
      } = O, q = {
        number: 0,
        unit: "em"
      }, G = {
        number: 0.9,
        unit: "em"
      }, J = {
        number: 0,
        unit: "em"
      }, Z = "";
      if (H[0])
        for (var te = assertNodeType(H[0], "raw").string, ne = te.split(","), ee = 0; ee < ne.length; ee++) {
          var ae = ne[ee].split("=");
          if (ae.length === 2) {
            var oe = ae[1].trim();
            switch (ae[0].trim()) {
              case "alt":
                Z = oe;
                break;
              case "width":
                q = sizeData(oe);
                break;
              case "height":
                G = sizeData(oe);
                break;
              case "totalheight":
                J = sizeData(oe);
                break;
              default:
                throw new ParseError("Invalid key: '" + ae[0] + "' in \\includegraphics.");
            }
          }
        }
      var se = assertNodeType(_[0], "url").url;
      return Z === "" && (Z = se, Z = Z.replace(/^.*[\\/]/, ""), Z = Z.substring(0, Z.lastIndexOf("."))), V.settings.isTrusted({
        command: "\\includegraphics",
        url: se
      }) ? {
        type: "includegraphics",
        mode: V.mode,
        alt: Z,
        width: q,
        height: G,
        totalheight: J,
        src: se
      } : V.formatUnsupportedCmd("\\includegraphics");
    },
    htmlBuilder: (O, _) => {
      var H = calculateSize(O.height, _), V = 0;
      O.totalheight.number > 0 && (V = calculateSize(O.totalheight, _) - H);
      var q = 0;
      O.width.number > 0 && (q = calculateSize(O.width, _));
      var G = {
        height: makeEm(H + V)
      };
      q > 0 && (G.width = makeEm(q)), V > 0 && (G.verticalAlign = makeEm(-V));
      var J = new Img(O.src, O.alt, G);
      return J.height = H, J.depth = V, J;
    },
    mathmlBuilder: (O, _) => {
      var H = new mathMLTree.MathNode("mglyph", []);
      H.setAttribute("alt", O.alt);
      var V = calculateSize(O.height, _), q = 0;
      if (O.totalheight.number > 0 && (q = calculateSize(O.totalheight, _) - V, H.setAttribute("valign", makeEm(-q))), H.setAttribute("height", makeEm(V + q)), O.width.number > 0) {
        var G = calculateSize(O.width, _);
        H.setAttribute("width", makeEm(G));
      }
      return H.setAttribute("src", O.src), H;
    }
  });
  defineFunction({
    type: "kern",
    names: ["\\kern", "\\mkern", "\\hskip", "\\mskip"],
    props: {
      numArgs: 1,
      argTypes: ["size"],
      primitive: !0,
      allowedInText: !0
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O, q = assertNodeType(_[0], "size");
      if (H.settings.strict) {
        var G = V[1] === "m", J = q.value.unit === "mu";
        G ? (J || H.settings.reportNonstrict("mathVsTextUnits", "LaTeX's " + V + " supports only mu units, " + ("not " + q.value.unit + " units")), H.mode !== "math" && H.settings.reportNonstrict("mathVsTextUnits", "LaTeX's " + V + " works only in math mode")) : J && H.settings.reportNonstrict("mathVsTextUnits", "LaTeX's " + V + " doesn't support mu units");
      }
      return {
        type: "kern",
        mode: H.mode,
        dimension: q.value
      };
    },
    htmlBuilder(O, _) {
      return buildCommon.makeGlue(O.dimension, _);
    },
    mathmlBuilder(O, _) {
      var H = calculateSize(O.dimension, _);
      return new mathMLTree.SpaceNode(H);
    }
  });
  defineFunction({
    type: "lap",
    names: ["\\mathllap", "\\mathrlap", "\\mathclap"],
    props: {
      numArgs: 1,
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0];
      return {
        type: "lap",
        mode: H.mode,
        alignment: V.slice(5),
        body: q
      };
    },
    htmlBuilder: (O, _) => {
      var H;
      O.alignment === "clap" ? (H = buildCommon.makeSpan([], [buildGroup$1(O.body, _)]), H = buildCommon.makeSpan(["inner"], [H], _)) : H = buildCommon.makeSpan(["inner"], [buildGroup$1(O.body, _)]);
      var V = buildCommon.makeSpan(["fix"], []), q = buildCommon.makeSpan([O.alignment], [H, V], _), G = buildCommon.makeSpan(["strut"]);
      return G.style.height = makeEm(q.height + q.depth), q.depth && (G.style.verticalAlign = makeEm(-q.depth)), q.children.unshift(G), q = buildCommon.makeSpan(["thinbox"], [q], _), buildCommon.makeSpan(["mord", "vbox"], [q], _);
    },
    mathmlBuilder: (O, _) => {
      var H = new mathMLTree.MathNode("mpadded", [buildGroup(O.body, _)]);
      if (O.alignment !== "rlap") {
        var V = O.alignment === "llap" ? "-1" : "-0.5";
        H.setAttribute("lspace", V + "width");
      }
      return H.setAttribute("width", "0px"), H;
    }
  });
  defineFunction({
    type: "styling",
    names: ["\\(", "$"],
    props: {
      numArgs: 0,
      allowedInText: !0,
      allowedInMath: !1
    },
    handler(O, _) {
      var {
        funcName: H,
        parser: V
      } = O, q = V.mode;
      V.switchMode("math");
      var G = H === "\\(" ? "\\)" : "$", J = V.parseExpression(!1, G);
      return V.expect(G), V.switchMode(q), {
        type: "styling",
        mode: V.mode,
        style: "text",
        body: J
      };
    }
  });
  defineFunction({
    type: "text",
    // Doesn't matter what this is.
    names: ["\\)", "\\]"],
    props: {
      numArgs: 0,
      allowedInText: !0,
      allowedInMath: !1
    },
    handler(O, _) {
      throw new ParseError("Mismatched " + O.funcName);
    }
  });
  var chooseMathStyle = (O, _) => {
    switch (_.style.size) {
      case Style$1.DISPLAY.size:
        return O.display;
      case Style$1.TEXT.size:
        return O.text;
      case Style$1.SCRIPT.size:
        return O.script;
      case Style$1.SCRIPTSCRIPT.size:
        return O.scriptscript;
      default:
        return O.text;
    }
  };
  defineFunction({
    type: "mathchoice",
    names: ["\\mathchoice"],
    props: {
      numArgs: 4,
      primitive: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O;
      return {
        type: "mathchoice",
        mode: H.mode,
        display: ordargument(_[0]),
        text: ordargument(_[1]),
        script: ordargument(_[2]),
        scriptscript: ordargument(_[3])
      };
    },
    htmlBuilder: (O, _) => {
      var H = chooseMathStyle(O, _), V = buildExpression$1(H, _, !1);
      return buildCommon.makeFragment(V);
    },
    mathmlBuilder: (O, _) => {
      var H = chooseMathStyle(O, _);
      return buildExpressionRow(H, _);
    }
  });
  var assembleSupSub = (O, _, H, V, q, G, J) => {
    O = buildCommon.makeSpan([], [O]);
    var Z = H && utils.isCharacterBox(H), te, ne;
    if (_) {
      var ee = buildGroup$1(_, V.havingStyle(q.sup()), V);
      ne = {
        elem: ee,
        kern: Math.max(V.fontMetrics().bigOpSpacing1, V.fontMetrics().bigOpSpacing3 - ee.depth)
      };
    }
    if (H) {
      var ae = buildGroup$1(H, V.havingStyle(q.sub()), V);
      te = {
        elem: ae,
        kern: Math.max(V.fontMetrics().bigOpSpacing2, V.fontMetrics().bigOpSpacing4 - ae.height)
      };
    }
    var oe;
    if (ne && te) {
      var se = V.fontMetrics().bigOpSpacing5 + te.elem.height + te.elem.depth + te.kern + O.depth + J;
      oe = buildCommon.makeVList({
        positionType: "bottom",
        positionData: se,
        children: [{
          type: "kern",
          size: V.fontMetrics().bigOpSpacing5
        }, {
          type: "elem",
          elem: te.elem,
          marginLeft: makeEm(-G)
        }, {
          type: "kern",
          size: te.kern
        }, {
          type: "elem",
          elem: O
        }, {
          type: "kern",
          size: ne.kern
        }, {
          type: "elem",
          elem: ne.elem,
          marginLeft: makeEm(G)
        }, {
          type: "kern",
          size: V.fontMetrics().bigOpSpacing5
        }]
      }, V);
    } else if (te) {
      var ce = O.height - J;
      oe = buildCommon.makeVList({
        positionType: "top",
        positionData: ce,
        children: [{
          type: "kern",
          size: V.fontMetrics().bigOpSpacing5
        }, {
          type: "elem",
          elem: te.elem,
          marginLeft: makeEm(-G)
        }, {
          type: "kern",
          size: te.kern
        }, {
          type: "elem",
          elem: O
        }]
      }, V);
    } else if (ne) {
      var le = O.depth + J;
      oe = buildCommon.makeVList({
        positionType: "bottom",
        positionData: le,
        children: [{
          type: "elem",
          elem: O
        }, {
          type: "kern",
          size: ne.kern
        }, {
          type: "elem",
          elem: ne.elem,
          marginLeft: makeEm(G)
        }, {
          type: "kern",
          size: V.fontMetrics().bigOpSpacing5
        }]
      }, V);
    } else
      return O;
    var ue = [oe];
    if (te && G !== 0 && !Z) {
      var de = buildCommon.makeSpan(["mspace"], [], V);
      de.style.marginRight = makeEm(G), ue.unshift(de);
    }
    return buildCommon.makeSpan(["mop", "op-limits"], ue, V);
  }, noSuccessor = ["\\smallint"], htmlBuilder$2 = (O, _) => {
    var H, V, q = !1, G;
    O.type === "supsub" ? (H = O.sup, V = O.sub, G = assertNodeType(O.base, "op"), q = !0) : G = assertNodeType(O, "op");
    var J = _.style, Z = !1;
    J.size === Style$1.DISPLAY.size && G.symbol && !utils.contains(noSuccessor, G.name) && (Z = !0);
    var te;
    if (G.symbol) {
      var ne = Z ? "Size2-Regular" : "Size1-Regular", ee = "";
      if ((G.name === "\\oiint" || G.name === "\\oiiint") && (ee = G.name.slice(1), G.name = ee === "oiint" ? "\\iint" : "\\iiint"), te = buildCommon.makeSymbol(G.name, ne, "math", _, ["mop", "op-symbol", Z ? "large-op" : "small-op"]), ee.length > 0) {
        var ae = te.italic, oe = buildCommon.staticSvg(ee + "Size" + (Z ? "2" : "1"), _);
        te = buildCommon.makeVList({
          positionType: "individualShift",
          children: [{
            type: "elem",
            elem: te,
            shift: 0
          }, {
            type: "elem",
            elem: oe,
            shift: Z ? 0.08 : 0
          }]
        }, _), G.name = "\\" + ee, te.classes.unshift("mop"), te.italic = ae;
      }
    } else if (G.body) {
      var se = buildExpression$1(G.body, _, !0);
      se.length === 1 && se[0] instanceof SymbolNode ? (te = se[0], te.classes[0] = "mop") : te = buildCommon.makeSpan(["mop"], se, _);
    } else {
      for (var ce = [], le = 1; le < G.name.length; le++)
        ce.push(buildCommon.mathsym(G.name[le], G.mode, _));
      te = buildCommon.makeSpan(["mop"], ce, _);
    }
    var ue = 0, de = 0;
    return (te instanceof SymbolNode || G.name === "\\oiint" || G.name === "\\oiiint") && !G.suppressBaseShift && (ue = (te.height - te.depth) / 2 - _.fontMetrics().axisHeight, de = te.italic), q ? assembleSupSub(te, H, V, _, J, de, ue) : (ue && (te.style.position = "relative", te.style.top = makeEm(ue)), te);
  }, mathmlBuilder$1 = (O, _) => {
    var H;
    if (O.symbol)
      H = new MathNode("mo", [makeText(O.name, O.mode)]), utils.contains(noSuccessor, O.name) && H.setAttribute("largeop", "false");
    else if (O.body)
      H = new MathNode("mo", buildExpression(O.body, _));
    else {
      H = new MathNode("mi", [new TextNode$1(O.name.slice(1))]);
      var V = new MathNode("mo", [makeText("⁡", "text")]);
      O.parentIsSupSub ? H = new MathNode("mrow", [H, V]) : H = newDocumentFragment([H, V]);
    }
    return H;
  }, singleCharBigOps = {
    "∏": "\\prod",
    "∐": "\\coprod",
    "∑": "\\sum",
    "⋀": "\\bigwedge",
    "⋁": "\\bigvee",
    "⋂": "\\bigcap",
    "⋃": "\\bigcup",
    "⨀": "\\bigodot",
    "⨁": "\\bigoplus",
    "⨂": "\\bigotimes",
    "⨄": "\\biguplus",
    "⨆": "\\bigsqcup"
  };
  defineFunction({
    type: "op",
    names: ["\\coprod", "\\bigvee", "\\bigwedge", "\\biguplus", "\\bigcap", "\\bigcup", "\\intop", "\\prod", "\\sum", "\\bigotimes", "\\bigoplus", "\\bigodot", "\\bigsqcup", "\\smallint", "∏", "∐", "∑", "⋀", "⋁", "⋂", "⋃", "⨀", "⨁", "⨂", "⨄", "⨆"],
    props: {
      numArgs: 0
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = V;
      return q.length === 1 && (q = singleCharBigOps[q]), {
        type: "op",
        mode: H.mode,
        limits: !0,
        parentIsSupSub: !1,
        symbol: !0,
        name: q
      };
    },
    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  });
  defineFunction({
    type: "op",
    names: ["\\mathop"],
    props: {
      numArgs: 1,
      primitive: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O, V = _[0];
      return {
        type: "op",
        mode: H.mode,
        limits: !1,
        parentIsSupSub: !1,
        symbol: !1,
        body: ordargument(V)
      };
    },
    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  });
  var singleCharIntegrals = {
    "∫": "\\int",
    "∬": "\\iint",
    "∭": "\\iiint",
    "∮": "\\oint",
    "∯": "\\oiint",
    "∰": "\\oiiint"
  };
  defineFunction({
    type: "op",
    names: ["\\arcsin", "\\arccos", "\\arctan", "\\arctg", "\\arcctg", "\\arg", "\\ch", "\\cos", "\\cosec", "\\cosh", "\\cot", "\\cotg", "\\coth", "\\csc", "\\ctg", "\\cth", "\\deg", "\\dim", "\\exp", "\\hom", "\\ker", "\\lg", "\\ln", "\\log", "\\sec", "\\sin", "\\sinh", "\\sh", "\\tan", "\\tanh", "\\tg", "\\th"],
    props: {
      numArgs: 0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H
      } = O;
      return {
        type: "op",
        mode: _.mode,
        limits: !1,
        parentIsSupSub: !1,
        symbol: !1,
        name: H
      };
    },
    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  });
  defineFunction({
    type: "op",
    names: ["\\det", "\\gcd", "\\inf", "\\lim", "\\max", "\\min", "\\Pr", "\\sup"],
    props: {
      numArgs: 0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H
      } = O;
      return {
        type: "op",
        mode: _.mode,
        limits: !0,
        parentIsSupSub: !1,
        symbol: !1,
        name: H
      };
    },
    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  });
  defineFunction({
    type: "op",
    names: ["\\int", "\\iint", "\\iiint", "\\oint", "\\oiint", "\\oiiint", "∫", "∬", "∭", "∮", "∯", "∰"],
    props: {
      numArgs: 0
    },
    handler(O) {
      var {
        parser: _,
        funcName: H
      } = O, V = H;
      return V.length === 1 && (V = singleCharIntegrals[V]), {
        type: "op",
        mode: _.mode,
        limits: !1,
        parentIsSupSub: !1,
        symbol: !0,
        name: V
      };
    },
    htmlBuilder: htmlBuilder$2,
    mathmlBuilder: mathmlBuilder$1
  });
  var htmlBuilder$1 = (O, _) => {
    var H, V, q = !1, G;
    O.type === "supsub" ? (H = O.sup, V = O.sub, G = assertNodeType(O.base, "operatorname"), q = !0) : G = assertNodeType(O, "operatorname");
    var J;
    if (G.body.length > 0) {
      for (var Z = G.body.map((ae) => {
        var oe = ae.text;
        return typeof oe == "string" ? {
          type: "textord",
          mode: ae.mode,
          text: oe
        } : ae;
      }), te = buildExpression$1(Z, _.withFont("mathrm"), !0), ne = 0; ne < te.length; ne++) {
        var ee = te[ne];
        ee instanceof SymbolNode && (ee.text = ee.text.replace(/\u2212/, "-").replace(/\u2217/, "*"));
      }
      J = buildCommon.makeSpan(["mop"], te, _);
    } else
      J = buildCommon.makeSpan(["mop"], [], _);
    return q ? assembleSupSub(J, H, V, _, _.style, 0, 0) : J;
  }, mathmlBuilder = (O, _) => {
    for (var H = buildExpression(O.body, _.withFont("mathrm")), V = !0, q = 0; q < H.length; q++) {
      var G = H[q];
      if (!(G instanceof mathMLTree.SpaceNode)) if (G instanceof mathMLTree.MathNode)
        switch (G.type) {
          case "mi":
          case "mn":
          case "ms":
          case "mspace":
          case "mtext":
            break;
          // Do nothing yet.
          case "mo": {
            var J = G.children[0];
            G.children.length === 1 && J instanceof mathMLTree.TextNode ? J.text = J.text.replace(/\u2212/, "-").replace(/\u2217/, "*") : V = !1;
            break;
          }
          default:
            V = !1;
        }
      else
        V = !1;
    }
    if (V) {
      var Z = H.map((ee) => ee.toText()).join("");
      H = [new mathMLTree.TextNode(Z)];
    }
    var te = new mathMLTree.MathNode("mi", H);
    te.setAttribute("mathvariant", "normal");
    var ne = new mathMLTree.MathNode("mo", [makeText("⁡", "text")]);
    return O.parentIsSupSub ? new mathMLTree.MathNode("mrow", [te, ne]) : mathMLTree.newDocumentFragment([te, ne]);
  };
  defineFunction({
    type: "operatorname",
    names: ["\\operatorname@", "\\operatornamewithlimits"],
    props: {
      numArgs: 1
    },
    handler: (O, _) => {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0];
      return {
        type: "operatorname",
        mode: H.mode,
        body: ordargument(q),
        alwaysHandleSupSub: V === "\\operatornamewithlimits",
        limits: !1,
        parentIsSupSub: !1
      };
    },
    htmlBuilder: htmlBuilder$1,
    mathmlBuilder
  });
  defineMacro("\\operatorname", "\\@ifstar\\operatornamewithlimits\\operatorname@");
  defineFunctionBuilders({
    type: "ordgroup",
    htmlBuilder(O, _) {
      return O.semisimple ? buildCommon.makeFragment(buildExpression$1(O.body, _, !1)) : buildCommon.makeSpan(["mord"], buildExpression$1(O.body, _, !0), _);
    },
    mathmlBuilder(O, _) {
      return buildExpressionRow(O.body, _, !0);
    }
  });
  defineFunction({
    type: "overline",
    names: ["\\overline"],
    props: {
      numArgs: 1
    },
    handler(O, _) {
      var {
        parser: H
      } = O, V = _[0];
      return {
        type: "overline",
        mode: H.mode,
        body: V
      };
    },
    htmlBuilder(O, _) {
      var H = buildGroup$1(O.body, _.havingCrampedStyle()), V = buildCommon.makeLineSpan("overline-line", _), q = _.fontMetrics().defaultRuleThickness, G = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: H
        }, {
          type: "kern",
          size: 3 * q
        }, {
          type: "elem",
          elem: V
        }, {
          type: "kern",
          size: q
        }]
      }, _);
      return buildCommon.makeSpan(["mord", "overline"], [G], _);
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode("‾")]);
      H.setAttribute("stretchy", "true");
      var V = new mathMLTree.MathNode("mover", [buildGroup(O.body, _), H]);
      return V.setAttribute("accent", "true"), V;
    }
  });
  defineFunction({
    type: "phantom",
    names: ["\\phantom"],
    props: {
      numArgs: 1,
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O, V = _[0];
      return {
        type: "phantom",
        mode: H.mode,
        body: ordargument(V)
      };
    },
    htmlBuilder: (O, _) => {
      var H = buildExpression$1(O.body, _.withPhantom(), !1);
      return buildCommon.makeFragment(H);
    },
    mathmlBuilder: (O, _) => {
      var H = buildExpression(O.body, _);
      return new mathMLTree.MathNode("mphantom", H);
    }
  });
  defineFunction({
    type: "hphantom",
    names: ["\\hphantom"],
    props: {
      numArgs: 1,
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O, V = _[0];
      return {
        type: "hphantom",
        mode: H.mode,
        body: V
      };
    },
    htmlBuilder: (O, _) => {
      var H = buildCommon.makeSpan([], [buildGroup$1(O.body, _.withPhantom())]);
      if (H.height = 0, H.depth = 0, H.children)
        for (var V = 0; V < H.children.length; V++)
          H.children[V].height = 0, H.children[V].depth = 0;
      return H = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: H
        }]
      }, _), buildCommon.makeSpan(["mord"], [H], _);
    },
    mathmlBuilder: (O, _) => {
      var H = buildExpression(ordargument(O.body), _), V = new mathMLTree.MathNode("mphantom", H), q = new mathMLTree.MathNode("mpadded", [V]);
      return q.setAttribute("height", "0px"), q.setAttribute("depth", "0px"), q;
    }
  });
  defineFunction({
    type: "vphantom",
    names: ["\\vphantom"],
    props: {
      numArgs: 1,
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        parser: H
      } = O, V = _[0];
      return {
        type: "vphantom",
        mode: H.mode,
        body: V
      };
    },
    htmlBuilder: (O, _) => {
      var H = buildCommon.makeSpan(["inner"], [buildGroup$1(O.body, _.withPhantom())]), V = buildCommon.makeSpan(["fix"], []);
      return buildCommon.makeSpan(["mord", "rlap"], [H, V], _);
    },
    mathmlBuilder: (O, _) => {
      var H = buildExpression(ordargument(O.body), _), V = new mathMLTree.MathNode("mphantom", H), q = new mathMLTree.MathNode("mpadded", [V]);
      return q.setAttribute("width", "0px"), q;
    }
  });
  defineFunction({
    type: "raisebox",
    names: ["\\raisebox"],
    props: {
      numArgs: 2,
      argTypes: ["size", "hbox"],
      allowedInText: !0
    },
    handler(O, _) {
      var {
        parser: H
      } = O, V = assertNodeType(_[0], "size").value, q = _[1];
      return {
        type: "raisebox",
        mode: H.mode,
        dy: V,
        body: q
      };
    },
    htmlBuilder(O, _) {
      var H = buildGroup$1(O.body, _), V = calculateSize(O.dy, _);
      return buildCommon.makeVList({
        positionType: "shift",
        positionData: -V,
        children: [{
          type: "elem",
          elem: H
        }]
      }, _);
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mpadded", [buildGroup(O.body, _)]), V = O.dy.number + O.dy.unit;
      return H.setAttribute("voffset", V), H;
    }
  });
  defineFunction({
    type: "internal",
    names: ["\\relax"],
    props: {
      numArgs: 0,
      allowedInText: !0
    },
    handler(O) {
      var {
        parser: _
      } = O;
      return {
        type: "internal",
        mode: _.mode
      };
    }
  });
  defineFunction({
    type: "rule",
    names: ["\\rule"],
    props: {
      numArgs: 2,
      numOptionalArgs: 1,
      allowedInText: !0,
      allowedInMath: !0,
      argTypes: ["size", "size", "size"]
    },
    handler(O, _, H) {
      var {
        parser: V
      } = O, q = H[0], G = assertNodeType(_[0], "size"), J = assertNodeType(_[1], "size");
      return {
        type: "rule",
        mode: V.mode,
        shift: q && assertNodeType(q, "size").value,
        width: G.value,
        height: J.value
      };
    },
    htmlBuilder(O, _) {
      var H = buildCommon.makeSpan(["mord", "rule"], [], _), V = calculateSize(O.width, _), q = calculateSize(O.height, _), G = O.shift ? calculateSize(O.shift, _) : 0;
      return H.style.borderRightWidth = makeEm(V), H.style.borderTopWidth = makeEm(q), H.style.bottom = makeEm(G), H.width = V, H.height = q + G, H.depth = -G, H.maxFontSize = q * 1.125 * _.sizeMultiplier, H;
    },
    mathmlBuilder(O, _) {
      var H = calculateSize(O.width, _), V = calculateSize(O.height, _), q = O.shift ? calculateSize(O.shift, _) : 0, G = _.color && _.getColor() || "black", J = new mathMLTree.MathNode("mspace");
      J.setAttribute("mathbackground", G), J.setAttribute("width", makeEm(H)), J.setAttribute("height", makeEm(V));
      var Z = new mathMLTree.MathNode("mpadded", [J]);
      return q >= 0 ? Z.setAttribute("height", makeEm(q)) : (Z.setAttribute("height", makeEm(q)), Z.setAttribute("depth", makeEm(-q))), Z.setAttribute("voffset", makeEm(q)), Z;
    }
  });
  function sizingGroup(O, _, H) {
    for (var V = buildExpression$1(O, _, !1), q = _.sizeMultiplier / H.sizeMultiplier, G = 0; G < V.length; G++) {
      var J = V[G].classes.indexOf("sizing");
      J < 0 ? Array.prototype.push.apply(V[G].classes, _.sizingClasses(H)) : V[G].classes[J + 1] === "reset-size" + _.size && (V[G].classes[J + 1] = "reset-size" + H.size), V[G].height *= q, V[G].depth *= q;
    }
    return buildCommon.makeFragment(V);
  }
  var sizeFuncs = ["\\tiny", "\\sixptsize", "\\scriptsize", "\\footnotesize", "\\small", "\\normalsize", "\\large", "\\Large", "\\LARGE", "\\huge", "\\Huge"], htmlBuilder = (O, _) => {
    var H = _.havingSize(O.size);
    return sizingGroup(O.body, H, _);
  };
  defineFunction({
    type: "sizing",
    names: sizeFuncs,
    props: {
      numArgs: 0,
      allowedInText: !0
    },
    handler: (O, _) => {
      var {
        breakOnTokenText: H,
        funcName: V,
        parser: q
      } = O, G = q.parseExpression(!1, H);
      return {
        type: "sizing",
        mode: q.mode,
        // Figure out what size to use based on the list of functions above
        size: sizeFuncs.indexOf(V) + 1,
        body: G
      };
    },
    htmlBuilder,
    mathmlBuilder: (O, _) => {
      var H = _.havingSize(O.size), V = buildExpression(O.body, H), q = new mathMLTree.MathNode("mstyle", V);
      return q.setAttribute("mathsize", makeEm(H.sizeMultiplier)), q;
    }
  });
  defineFunction({
    type: "smash",
    names: ["\\smash"],
    props: {
      numArgs: 1,
      numOptionalArgs: 1,
      allowedInText: !0
    },
    handler: (O, _, H) => {
      var {
        parser: V
      } = O, q = !1, G = !1, J = H[0] && assertNodeType(H[0], "ordgroup");
      if (J)
        for (var Z = "", te = 0; te < J.body.length; ++te) {
          var ne = J.body[te];
          if (Z = ne.text, Z === "t")
            q = !0;
          else if (Z === "b")
            G = !0;
          else {
            q = !1, G = !1;
            break;
          }
        }
      else
        q = !0, G = !0;
      var ee = _[0];
      return {
        type: "smash",
        mode: V.mode,
        body: ee,
        smashHeight: q,
        smashDepth: G
      };
    },
    htmlBuilder: (O, _) => {
      var H = buildCommon.makeSpan([], [buildGroup$1(O.body, _)]);
      if (!O.smashHeight && !O.smashDepth)
        return H;
      if (O.smashHeight && (H.height = 0, H.children))
        for (var V = 0; V < H.children.length; V++)
          H.children[V].height = 0;
      if (O.smashDepth && (H.depth = 0, H.children))
        for (var q = 0; q < H.children.length; q++)
          H.children[q].depth = 0;
      var G = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: H
        }]
      }, _);
      return buildCommon.makeSpan(["mord"], [G], _);
    },
    mathmlBuilder: (O, _) => {
      var H = new mathMLTree.MathNode("mpadded", [buildGroup(O.body, _)]);
      return O.smashHeight && H.setAttribute("height", "0px"), O.smashDepth && H.setAttribute("depth", "0px"), H;
    }
  });
  defineFunction({
    type: "sqrt",
    names: ["\\sqrt"],
    props: {
      numArgs: 1,
      numOptionalArgs: 1
    },
    handler(O, _, H) {
      var {
        parser: V
      } = O, q = H[0], G = _[0];
      return {
        type: "sqrt",
        mode: V.mode,
        body: G,
        index: q
      };
    },
    htmlBuilder(O, _) {
      var H = buildGroup$1(O.body, _.havingCrampedStyle());
      H.height === 0 && (H.height = _.fontMetrics().xHeight), H = buildCommon.wrapFragment(H, _);
      var V = _.fontMetrics(), q = V.defaultRuleThickness, G = q;
      _.style.id < Style$1.TEXT.id && (G = _.fontMetrics().xHeight);
      var J = q + G / 4, Z = H.height + H.depth + J + q, {
        span: te,
        ruleWidth: ne,
        advanceWidth: ee
      } = delimiter.sqrtImage(Z, _), ae = te.height - ne;
      ae > H.height + H.depth + J && (J = (J + ae - H.height - H.depth) / 2);
      var oe = te.height - H.height - J - ne;
      H.style.paddingLeft = makeEm(ee);
      var se = buildCommon.makeVList({
        positionType: "firstBaseline",
        children: [{
          type: "elem",
          elem: H,
          wrapperClasses: ["svg-align"]
        }, {
          type: "kern",
          size: -(H.height + oe)
        }, {
          type: "elem",
          elem: te
        }, {
          type: "kern",
          size: ne
        }]
      }, _);
      if (O.index) {
        var ce = _.havingStyle(Style$1.SCRIPTSCRIPT), le = buildGroup$1(O.index, ce, _), ue = 0.6 * (se.height - se.depth), de = buildCommon.makeVList({
          positionType: "shift",
          positionData: -ue,
          children: [{
            type: "elem",
            elem: le
          }]
        }, _), fe = buildCommon.makeSpan(["root"], [de]);
        return buildCommon.makeSpan(["mord", "sqrt"], [fe, se], _);
      } else
        return buildCommon.makeSpan(["mord", "sqrt"], [se], _);
    },
    mathmlBuilder(O, _) {
      var {
        body: H,
        index: V
      } = O;
      return V ? new mathMLTree.MathNode("mroot", [buildGroup(H, _), buildGroup(V, _)]) : new mathMLTree.MathNode("msqrt", [buildGroup(H, _)]);
    }
  });
  var styleMap = {
    display: Style$1.DISPLAY,
    text: Style$1.TEXT,
    script: Style$1.SCRIPT,
    scriptscript: Style$1.SCRIPTSCRIPT
  };
  defineFunction({
    type: "styling",
    names: ["\\displaystyle", "\\textstyle", "\\scriptstyle", "\\scriptscriptstyle"],
    props: {
      numArgs: 0,
      allowedInText: !0,
      primitive: !0
    },
    handler(O, _) {
      var {
        breakOnTokenText: H,
        funcName: V,
        parser: q
      } = O, G = q.parseExpression(!0, H), J = V.slice(1, V.length - 5);
      return {
        type: "styling",
        mode: q.mode,
        // Figure out what style to use by pulling out the style from
        // the function name
        style: J,
        body: G
      };
    },
    htmlBuilder(O, _) {
      var H = styleMap[O.style], V = _.havingStyle(H).withFont("");
      return sizingGroup(O.body, V, _);
    },
    mathmlBuilder(O, _) {
      var H = styleMap[O.style], V = _.havingStyle(H), q = buildExpression(O.body, V), G = new mathMLTree.MathNode("mstyle", q), J = {
        display: ["0", "true"],
        text: ["0", "false"],
        script: ["1", "false"],
        scriptscript: ["2", "false"]
      }, Z = J[O.style];
      return G.setAttribute("scriptlevel", Z[0]), G.setAttribute("displaystyle", Z[1]), G;
    }
  });
  var htmlBuilderDelegate = function O(_, H) {
    var V = _.base;
    if (V)
      if (V.type === "op") {
        var q = V.limits && (H.style.size === Style$1.DISPLAY.size || V.alwaysHandleSupSub);
        return q ? htmlBuilder$2 : null;
      } else if (V.type === "operatorname") {
        var G = V.alwaysHandleSupSub && (H.style.size === Style$1.DISPLAY.size || V.limits);
        return G ? htmlBuilder$1 : null;
      } else {
        if (V.type === "accent")
          return utils.isCharacterBox(V.base) ? htmlBuilder$a : null;
        if (V.type === "horizBrace") {
          var J = !_.sub;
          return J === V.isOver ? htmlBuilder$3 : null;
        } else
          return null;
      }
    else return null;
  };
  defineFunctionBuilders({
    type: "supsub",
    htmlBuilder(O, _) {
      var H = htmlBuilderDelegate(O, _);
      if (H)
        return H(O, _);
      var {
        base: V,
        sup: q,
        sub: G
      } = O, J = buildGroup$1(V, _), Z, te, ne = _.fontMetrics(), ee = 0, ae = 0, oe = V && utils.isCharacterBox(V);
      if (q) {
        var se = _.havingStyle(_.style.sup());
        Z = buildGroup$1(q, se, _), oe || (ee = J.height - se.fontMetrics().supDrop * se.sizeMultiplier / _.sizeMultiplier);
      }
      if (G) {
        var ce = _.havingStyle(_.style.sub());
        te = buildGroup$1(G, ce, _), oe || (ae = J.depth + ce.fontMetrics().subDrop * ce.sizeMultiplier / _.sizeMultiplier);
      }
      var le;
      _.style === Style$1.DISPLAY ? le = ne.sup1 : _.style.cramped ? le = ne.sup3 : le = ne.sup2;
      var ue = _.sizeMultiplier, de = makeEm(0.5 / ne.ptPerEm / ue), fe = null;
      if (te) {
        var he = O.base && O.base.type === "op" && O.base.name && (O.base.name === "\\oiint" || O.base.name === "\\oiiint");
        (J instanceof SymbolNode || he) && (fe = makeEm(-J.italic));
      }
      var me;
      if (Z && te) {
        ee = Math.max(ee, le, Z.depth + 0.25 * ne.xHeight), ae = Math.max(ae, ne.sub2);
        var pe = ne.defaultRuleThickness, ye = 4 * pe;
        if (ee - Z.depth - (te.height - ae) < ye) {
          ae = ye - (ee - Z.depth) + te.height;
          var ge = 0.8 * ne.xHeight - (ee - Z.depth);
          ge > 0 && (ee += ge, ae -= ge);
        }
        var ve = [{
          type: "elem",
          elem: te,
          shift: ae,
          marginRight: de,
          marginLeft: fe
        }, {
          type: "elem",
          elem: Z,
          shift: -ee,
          marginRight: de
        }];
        me = buildCommon.makeVList({
          positionType: "individualShift",
          children: ve
        }, _);
      } else if (te) {
        ae = Math.max(ae, ne.sub1, te.height - 0.8 * ne.xHeight);
        var be = [{
          type: "elem",
          elem: te,
          marginLeft: fe,
          marginRight: de
        }];
        me = buildCommon.makeVList({
          positionType: "shift",
          positionData: ae,
          children: be
        }, _);
      } else if (Z)
        ee = Math.max(ee, le, Z.depth + 0.25 * ne.xHeight), me = buildCommon.makeVList({
          positionType: "shift",
          positionData: -ee,
          children: [{
            type: "elem",
            elem: Z,
            marginRight: de
          }]
        }, _);
      else
        throw new Error("supsub must have either sup or sub.");
      var Ce = getTypeOfDomTree(J, "right") || "mord";
      return buildCommon.makeSpan([Ce], [J, buildCommon.makeSpan(["msupsub"], [me])], _);
    },
    mathmlBuilder(O, _) {
      var H = !1, V, q;
      O.base && O.base.type === "horizBrace" && (q = !!O.sup, q === O.base.isOver && (H = !0, V = O.base.isOver)), O.base && (O.base.type === "op" || O.base.type === "operatorname") && (O.base.parentIsSupSub = !0);
      var G = [buildGroup(O.base, _)];
      O.sub && G.push(buildGroup(O.sub, _)), O.sup && G.push(buildGroup(O.sup, _));
      var J;
      if (H)
        J = V ? "mover" : "munder";
      else if (O.sub)
        if (O.sup) {
          var ne = O.base;
          ne && ne.type === "op" && ne.limits && _.style === Style$1.DISPLAY || ne && ne.type === "operatorname" && ne.alwaysHandleSupSub && (_.style === Style$1.DISPLAY || ne.limits) ? J = "munderover" : J = "msubsup";
        } else {
          var te = O.base;
          te && te.type === "op" && te.limits && (_.style === Style$1.DISPLAY || te.alwaysHandleSupSub) || te && te.type === "operatorname" && te.alwaysHandleSupSub && (te.limits || _.style === Style$1.DISPLAY) ? J = "munder" : J = "msub";
        }
      else {
        var Z = O.base;
        Z && Z.type === "op" && Z.limits && (_.style === Style$1.DISPLAY || Z.alwaysHandleSupSub) || Z && Z.type === "operatorname" && Z.alwaysHandleSupSub && (Z.limits || _.style === Style$1.DISPLAY) ? J = "mover" : J = "msup";
      }
      return new mathMLTree.MathNode(J, G);
    }
  });
  defineFunctionBuilders({
    type: "atom",
    htmlBuilder(O, _) {
      return buildCommon.mathsym(O.text, O.mode, _, ["m" + O.family]);
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mo", [makeText(O.text, O.mode)]);
      if (O.family === "bin") {
        var V = getVariant(O, _);
        V === "bold-italic" && H.setAttribute("mathvariant", V);
      } else O.family === "punct" ? H.setAttribute("separator", "true") : (O.family === "open" || O.family === "close") && H.setAttribute("stretchy", "false");
      return H;
    }
  });
  var defaultVariant = {
    mi: "italic",
    mn: "normal",
    mtext: "normal"
  };
  defineFunctionBuilders({
    type: "mathord",
    htmlBuilder(O, _) {
      return buildCommon.makeOrd(O, _, "mathord");
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mi", [makeText(O.text, O.mode, _)]), V = getVariant(O, _) || "italic";
      return V !== defaultVariant[H.type] && H.setAttribute("mathvariant", V), H;
    }
  });
  defineFunctionBuilders({
    type: "textord",
    htmlBuilder(O, _) {
      return buildCommon.makeOrd(O, _, "textord");
    },
    mathmlBuilder(O, _) {
      var H = makeText(O.text, O.mode, _), V = getVariant(O, _) || "normal", q;
      return O.mode === "text" ? q = new mathMLTree.MathNode("mtext", [H]) : /[0-9]/.test(O.text) ? q = new mathMLTree.MathNode("mn", [H]) : O.text === "\\prime" ? q = new mathMLTree.MathNode("mo", [H]) : q = new mathMLTree.MathNode("mi", [H]), V !== defaultVariant[q.type] && q.setAttribute("mathvariant", V), q;
    }
  });
  var cssSpace = {
    "\\nobreak": "nobreak",
    "\\allowbreak": "allowbreak"
  }, regularSpace = {
    " ": {},
    "\\ ": {},
    "~": {
      className: "nobreak"
    },
    "\\space": {},
    "\\nobreakspace": {
      className: "nobreak"
    }
  };
  defineFunctionBuilders({
    type: "spacing",
    htmlBuilder(O, _) {
      if (regularSpace.hasOwnProperty(O.text)) {
        var H = regularSpace[O.text].className || "";
        if (O.mode === "text") {
          var V = buildCommon.makeOrd(O, _, "textord");
          return V.classes.push(H), V;
        } else
          return buildCommon.makeSpan(["mspace", H], [buildCommon.mathsym(O.text, O.mode, _)], _);
      } else {
        if (cssSpace.hasOwnProperty(O.text))
          return buildCommon.makeSpan(["mspace", cssSpace[O.text]], [], _);
        throw new ParseError('Unknown type of space "' + O.text + '"');
      }
    },
    mathmlBuilder(O, _) {
      var H;
      if (regularSpace.hasOwnProperty(O.text))
        H = new mathMLTree.MathNode("mtext", [new mathMLTree.TextNode(" ")]);
      else {
        if (cssSpace.hasOwnProperty(O.text))
          return new mathMLTree.MathNode("mspace");
        throw new ParseError('Unknown type of space "' + O.text + '"');
      }
      return H;
    }
  });
  var pad$3 = () => {
    var O = new mathMLTree.MathNode("mtd", []);
    return O.setAttribute("width", "50%"), O;
  };
  defineFunctionBuilders({
    type: "tag",
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mtable", [new mathMLTree.MathNode("mtr", [pad$3(), new mathMLTree.MathNode("mtd", [buildExpressionRow(O.body, _)]), pad$3(), new mathMLTree.MathNode("mtd", [buildExpressionRow(O.tag, _)])])]);
      return H.setAttribute("width", "100%"), H;
    }
  });
  var textFontFamilies = {
    "\\text": void 0,
    "\\textrm": "textrm",
    "\\textsf": "textsf",
    "\\texttt": "texttt",
    "\\textnormal": "textrm"
  }, textFontWeights = {
    "\\textbf": "textbf",
    "\\textmd": "textmd"
  }, textFontShapes = {
    "\\textit": "textit",
    "\\textup": "textup"
  }, optionsWithFont = (O, _) => {
    var H = O.font;
    if (H) {
      if (textFontFamilies[H])
        return _.withTextFontFamily(textFontFamilies[H]);
      if (textFontWeights[H])
        return _.withTextFontWeight(textFontWeights[H]);
      if (H === "\\emph")
        return _.fontShape === "textit" ? _.withTextFontShape("textup") : _.withTextFontShape("textit");
    } else return _;
    return _.withTextFontShape(textFontShapes[H]);
  };
  defineFunction({
    type: "text",
    names: [
      // Font families
      "\\text",
      "\\textrm",
      "\\textsf",
      "\\texttt",
      "\\textnormal",
      // Font weights
      "\\textbf",
      "\\textmd",
      // Font Shapes
      "\\textit",
      "\\textup",
      "\\emph"
    ],
    props: {
      numArgs: 1,
      argTypes: ["text"],
      allowedInArgument: !0,
      allowedInText: !0
    },
    handler(O, _) {
      var {
        parser: H,
        funcName: V
      } = O, q = _[0];
      return {
        type: "text",
        mode: H.mode,
        body: ordargument(q),
        font: V
      };
    },
    htmlBuilder(O, _) {
      var H = optionsWithFont(O, _), V = buildExpression$1(O.body, H, !0);
      return buildCommon.makeSpan(["mord", "text"], V, H);
    },
    mathmlBuilder(O, _) {
      var H = optionsWithFont(O, _);
      return buildExpressionRow(O.body, H);
    }
  });
  defineFunction({
    type: "underline",
    names: ["\\underline"],
    props: {
      numArgs: 1,
      allowedInText: !0
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "underline",
        mode: H.mode,
        body: _[0]
      };
    },
    htmlBuilder(O, _) {
      var H = buildGroup$1(O.body, _), V = buildCommon.makeLineSpan("underline-line", _), q = _.fontMetrics().defaultRuleThickness, G = buildCommon.makeVList({
        positionType: "top",
        positionData: H.height,
        children: [{
          type: "kern",
          size: q
        }, {
          type: "elem",
          elem: V
        }, {
          type: "kern",
          size: 3 * q
        }, {
          type: "elem",
          elem: H
        }]
      }, _);
      return buildCommon.makeSpan(["mord", "underline"], [G], _);
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.MathNode("mo", [new mathMLTree.TextNode("‾")]);
      H.setAttribute("stretchy", "true");
      var V = new mathMLTree.MathNode("munder", [buildGroup(O.body, _), H]);
      return V.setAttribute("accentunder", "true"), V;
    }
  });
  defineFunction({
    type: "vcenter",
    names: ["\\vcenter"],
    props: {
      numArgs: 1,
      argTypes: ["original"],
      // In LaTeX, \vcenter can act only on a box.
      allowedInText: !1
    },
    handler(O, _) {
      var {
        parser: H
      } = O;
      return {
        type: "vcenter",
        mode: H.mode,
        body: _[0]
      };
    },
    htmlBuilder(O, _) {
      var H = buildGroup$1(O.body, _), V = _.fontMetrics().axisHeight, q = 0.5 * (H.height - V - (H.depth + V));
      return buildCommon.makeVList({
        positionType: "shift",
        positionData: q,
        children: [{
          type: "elem",
          elem: H
        }]
      }, _);
    },
    mathmlBuilder(O, _) {
      return new mathMLTree.MathNode("mpadded", [buildGroup(O.body, _)], ["vcenter"]);
    }
  });
  defineFunction({
    type: "verb",
    names: ["\\verb"],
    props: {
      numArgs: 0,
      allowedInText: !0
    },
    handler(O, _, H) {
      throw new ParseError("\\verb ended by end of line instead of matching delimiter");
    },
    htmlBuilder(O, _) {
      for (var H = makeVerb(O), V = [], q = _.havingStyle(_.style.text()), G = 0; G < H.length; G++) {
        var J = H[G];
        J === "~" && (J = "\\textasciitilde"), V.push(buildCommon.makeSymbol(J, "Typewriter-Regular", O.mode, q, ["mord", "texttt"]));
      }
      return buildCommon.makeSpan(["mord", "text"].concat(q.sizingClasses(_)), buildCommon.tryCombineChars(V), q);
    },
    mathmlBuilder(O, _) {
      var H = new mathMLTree.TextNode(makeVerb(O)), V = new mathMLTree.MathNode("mtext", [H]);
      return V.setAttribute("mathvariant", "monospace"), V;
    }
  });
  var makeVerb = (O) => O.body.replace(/ /g, O.star ? "␣" : " "), functions = _functions, spaceRegexString = `[ \r
	]`, controlWordRegexString = "\\\\[a-zA-Z@]+", controlSymbolRegexString = "\\\\[^\uD800-\uDFFF]", controlWordWhitespaceRegexString = "(" + controlWordRegexString + ")" + spaceRegexString + "*", controlSpaceRegexString = `\\\\(
|[ \r	]+
?)[ \r	]*`, combiningDiacriticalMarkString = "[̀-ͯ]", combiningDiacriticalMarksEndRegex = new RegExp(combiningDiacriticalMarkString + "+$"), tokenRegexString = "(" + spaceRegexString + "+)|" + // whitespace
  (controlSpaceRegexString + "|") + // \whitespace
  "([!-\\[\\]-‧‪-퟿豈-]" + // single codepoint
  (combiningDiacriticalMarkString + "*") + // ...plus accents
  "|[\uD800-\uDBFF][\uDC00-\uDFFF]" + // surrogate pair
  (combiningDiacriticalMarkString + "*") + // ...plus accents
  "|\\\\verb\\*([^]).*?\\4|\\\\verb([^*a-zA-Z]).*?\\5" + // \verb unstarred
  ("|" + controlWordWhitespaceRegexString) + // \macroName + spaces
  ("|" + controlSymbolRegexString + ")");
  class Lexer {
    // Category codes. The lexer only supports comment characters (14) for now.
    // MacroExpander additionally distinguishes active (13).
    constructor(_, H) {
      this.input = void 0, this.settings = void 0, this.tokenRegex = void 0, this.catcodes = void 0, this.input = _, this.settings = H, this.tokenRegex = new RegExp(tokenRegexString, "g"), this.catcodes = {
        "%": 14,
        // comment character
        "~": 13
        // active character
      };
    }
    setCatcode(_, H) {
      this.catcodes[_] = H;
    }
    /**
     * This function lexes a single token.
     */
    lex() {
      var _ = this.input, H = this.tokenRegex.lastIndex;
      if (H === _.length)
        return new Token("EOF", new SourceLocation(this, H, H));
      var V = this.tokenRegex.exec(_);
      if (V === null || V.index !== H)
        throw new ParseError("Unexpected character: '" + _[H] + "'", new Token(_[H], new SourceLocation(this, H, H + 1)));
      var q = V[6] || V[3] || (V[2] ? "\\ " : " ");
      if (this.catcodes[q] === 14) {
        var G = _.indexOf(`
`, this.tokenRegex.lastIndex);
        return G === -1 ? (this.tokenRegex.lastIndex = _.length, this.settings.reportNonstrict("commentAtEnd", "% comment has no terminating newline; LaTeX would fail because of commenting the end of math mode (e.g. $)")) : this.tokenRegex.lastIndex = G + 1, this.lex();
      }
      return new Token(q, new SourceLocation(this, H, this.tokenRegex.lastIndex));
    }
  }
  class Namespace {
    /**
     * Both arguments are optional.  The first argument is an object of
     * built-in mappings which never change.  The second argument is an object
     * of initial (global-level) mappings, which will constantly change
     * according to any global/top-level `set`s done.
     */
    constructor(_, H) {
      _ === void 0 && (_ = {}), H === void 0 && (H = {}), this.current = void 0, this.builtins = void 0, this.undefStack = void 0, this.current = H, this.builtins = _, this.undefStack = [];
    }
    /**
     * Start a new nested group, affecting future local `set`s.
     */
    beginGroup() {
      this.undefStack.push({});
    }
    /**
     * End current nested group, restoring values before the group began.
     */
    endGroup() {
      if (this.undefStack.length === 0)
        throw new ParseError("Unbalanced namespace destruction: attempt to pop global namespace; please report this as a bug");
      var _ = this.undefStack.pop();
      for (var H in _)
        _.hasOwnProperty(H) && (_[H] == null ? delete this.current[H] : this.current[H] = _[H]);
    }
    /**
     * Ends all currently nested groups (if any), restoring values before the
     * groups began.  Useful in case of an error in the middle of parsing.
     */
    endGroups() {
      for (; this.undefStack.length > 0; )
        this.endGroup();
    }
    /**
     * Detect whether `name` has a definition.  Equivalent to
     * `get(name) != null`.
     */
    has(_) {
      return this.current.hasOwnProperty(_) || this.builtins.hasOwnProperty(_);
    }
    /**
     * Get the current value of a name, or `undefined` if there is no value.
     *
     * Note: Do not use `if (namespace.get(...))` to detect whether a macro
     * is defined, as the definition may be the empty string which evaluates
     * to `false` in JavaScript.  Use `if (namespace.get(...) != null)` or
     * `if (namespace.has(...))`.
     */
    get(_) {
      return this.current.hasOwnProperty(_) ? this.current[_] : this.builtins[_];
    }
    /**
     * Set the current value of a name, and optionally set it globally too.
     * Local set() sets the current value and (when appropriate) adds an undo
     * operation to the undo stack.  Global set() may change the undo
     * operation at every level, so takes time linear in their number.
     * A value of undefined means to delete existing definitions.
     */
    set(_, H, V) {
      if (V === void 0 && (V = !1), V) {
        for (var q = 0; q < this.undefStack.length; q++)
          delete this.undefStack[q][_];
        this.undefStack.length > 0 && (this.undefStack[this.undefStack.length - 1][_] = H);
      } else {
        var G = this.undefStack[this.undefStack.length - 1];
        G && !G.hasOwnProperty(_) && (G[_] = this.current[_]);
      }
      H == null ? delete this.current[_] : this.current[_] = H;
    }
  }
  var macros = _macros;
  defineMacro("\\noexpand", function(O) {
    var _ = O.popToken();
    return O.isExpandable(_.text) && (_.noexpand = !0, _.treatAsRelax = !0), {
      tokens: [_],
      numArgs: 0
    };
  });
  defineMacro("\\expandafter", function(O) {
    var _ = O.popToken();
    return O.expandOnce(!0), {
      tokens: [_],
      numArgs: 0
    };
  });
  defineMacro("\\@firstoftwo", function(O) {
    var _ = O.consumeArgs(2);
    return {
      tokens: _[0],
      numArgs: 0
    };
  });
  defineMacro("\\@secondoftwo", function(O) {
    var _ = O.consumeArgs(2);
    return {
      tokens: _[1],
      numArgs: 0
    };
  });
  defineMacro("\\@ifnextchar", function(O) {
    var _ = O.consumeArgs(3);
    O.consumeSpaces();
    var H = O.future();
    return _[0].length === 1 && _[0][0].text === H.text ? {
      tokens: _[1],
      numArgs: 0
    } : {
      tokens: _[2],
      numArgs: 0
    };
  });
  defineMacro("\\@ifstar", "\\@ifnextchar *{\\@firstoftwo{#1}}");
  defineMacro("\\TextOrMath", function(O) {
    var _ = O.consumeArgs(2);
    return O.mode === "text" ? {
      tokens: _[0],
      numArgs: 0
    } : {
      tokens: _[1],
      numArgs: 0
    };
  });
  var digitToNumber = {
    0: 0,
    1: 1,
    2: 2,
    3: 3,
    4: 4,
    5: 5,
    6: 6,
    7: 7,
    8: 8,
    9: 9,
    a: 10,
    A: 10,
    b: 11,
    B: 11,
    c: 12,
    C: 12,
    d: 13,
    D: 13,
    e: 14,
    E: 14,
    f: 15,
    F: 15
  };
  defineMacro("\\char", function(O) {
    var _ = O.popToken(), H, V = "";
    if (_.text === "'")
      H = 8, _ = O.popToken();
    else if (_.text === '"')
      H = 16, _ = O.popToken();
    else if (_.text === "`")
      if (_ = O.popToken(), _.text[0] === "\\")
        V = _.text.charCodeAt(1);
      else {
        if (_.text === "EOF")
          throw new ParseError("\\char` missing argument");
        V = _.text.charCodeAt(0);
      }
    else
      H = 10;
    if (H) {
      if (V = digitToNumber[_.text], V == null || V >= H)
        throw new ParseError("Invalid base-" + H + " digit " + _.text);
      for (var q; (q = digitToNumber[O.future().text]) != null && q < H; )
        V *= H, V += q, O.popToken();
    }
    return "\\@char{" + V + "}";
  });
  var newcommand = (O, _, H) => {
    var V = O.consumeArg().tokens;
    if (V.length !== 1)
      throw new ParseError("\\newcommand's first argument must be a macro name");
    var q = V[0].text, G = O.isDefined(q);
    if (G && !_)
      throw new ParseError("\\newcommand{" + q + "} attempting to redefine " + (q + "; use \\renewcommand"));
    if (!G && !H)
      throw new ParseError("\\renewcommand{" + q + "} when command " + q + " does not yet exist; use \\newcommand");
    var J = 0;
    if (V = O.consumeArg().tokens, V.length === 1 && V[0].text === "[") {
      for (var Z = "", te = O.expandNextToken(); te.text !== "]" && te.text !== "EOF"; )
        Z += te.text, te = O.expandNextToken();
      if (!Z.match(/^\s*[0-9]+\s*$/))
        throw new ParseError("Invalid number of arguments: " + Z);
      J = parseInt(Z), V = O.consumeArg().tokens;
    }
    return O.macros.set(q, {
      tokens: V,
      numArgs: J
    }), "";
  };
  defineMacro("\\newcommand", (O) => newcommand(O, !1, !0));
  defineMacro("\\renewcommand", (O) => newcommand(O, !0, !1));
  defineMacro("\\providecommand", (O) => newcommand(O, !0, !0));
  defineMacro("\\message", (O) => {
    var _ = O.consumeArgs(1)[0];
    return console.log(_.reverse().map((H) => H.text).join("")), "";
  });
  defineMacro("\\errmessage", (O) => {
    var _ = O.consumeArgs(1)[0];
    return console.error(_.reverse().map((H) => H.text).join("")), "";
  });
  defineMacro("\\show", (O) => {
    var _ = O.popToken(), H = _.text;
    return console.log(_, O.macros.get(H), functions[H], symbols$3.math[H], symbols$3.text[H]), "";
  });
  defineMacro("\\bgroup", "{");
  defineMacro("\\egroup", "}");
  defineMacro("~", "\\nobreakspace");
  defineMacro("\\lq", "`");
  defineMacro("\\rq", "'");
  defineMacro("\\aa", "\\r a");
  defineMacro("\\AA", "\\r A");
  defineMacro("\\textcopyright", "\\html@mathml{\\textcircled{c}}{\\char`©}");
  defineMacro("\\copyright", "\\TextOrMath{\\textcopyright}{\\text{\\textcopyright}}");
  defineMacro("\\textregistered", "\\html@mathml{\\textcircled{\\scriptsize R}}{\\char`®}");
  defineMacro("ℬ", "\\mathscr{B}");
  defineMacro("ℰ", "\\mathscr{E}");
  defineMacro("ℱ", "\\mathscr{F}");
  defineMacro("ℋ", "\\mathscr{H}");
  defineMacro("ℐ", "\\mathscr{I}");
  defineMacro("ℒ", "\\mathscr{L}");
  defineMacro("ℳ", "\\mathscr{M}");
  defineMacro("ℛ", "\\mathscr{R}");
  defineMacro("ℭ", "\\mathfrak{C}");
  defineMacro("ℌ", "\\mathfrak{H}");
  defineMacro("ℨ", "\\mathfrak{Z}");
  defineMacro("\\Bbbk", "\\Bbb{k}");
  defineMacro("·", "\\cdotp");
  defineMacro("\\llap", "\\mathllap{\\textrm{#1}}");
  defineMacro("\\rlap", "\\mathrlap{\\textrm{#1}}");
  defineMacro("\\clap", "\\mathclap{\\textrm{#1}}");
  defineMacro("\\mathstrut", "\\vphantom{(}");
  defineMacro("\\underbar", "\\underline{\\text{#1}}");
  defineMacro("\\not", '\\html@mathml{\\mathrel{\\mathrlap\\@not}}{\\char"338}');
  defineMacro("\\neq", "\\html@mathml{\\mathrel{\\not=}}{\\mathrel{\\char`≠}}");
  defineMacro("\\ne", "\\neq");
  defineMacro("≠", "\\neq");
  defineMacro("\\notin", "\\html@mathml{\\mathrel{{\\in}\\mathllap{/\\mskip1mu}}}{\\mathrel{\\char`∉}}");
  defineMacro("∉", "\\notin");
  defineMacro("≘", "\\html@mathml{\\mathrel{=\\kern{-1em}\\raisebox{0.4em}{$\\scriptsize\\frown$}}}{\\mathrel{\\char`≘}}");
  defineMacro("≙", "\\html@mathml{\\stackrel{\\tiny\\wedge}{=}}{\\mathrel{\\char`≘}}");
  defineMacro("≚", "\\html@mathml{\\stackrel{\\tiny\\vee}{=}}{\\mathrel{\\char`≚}}");
  defineMacro("≛", "\\html@mathml{\\stackrel{\\scriptsize\\star}{=}}{\\mathrel{\\char`≛}}");
  defineMacro("≝", "\\html@mathml{\\stackrel{\\tiny\\mathrm{def}}{=}}{\\mathrel{\\char`≝}}");
  defineMacro("≞", "\\html@mathml{\\stackrel{\\tiny\\mathrm{m}}{=}}{\\mathrel{\\char`≞}}");
  defineMacro("≟", "\\html@mathml{\\stackrel{\\tiny?}{=}}{\\mathrel{\\char`≟}}");
  defineMacro("⟂", "\\perp");
  defineMacro("‼", "\\mathclose{!\\mkern-0.8mu!}");
  defineMacro("∌", "\\notni");
  defineMacro("⌜", "\\ulcorner");
  defineMacro("⌝", "\\urcorner");
  defineMacro("⌞", "\\llcorner");
  defineMacro("⌟", "\\lrcorner");
  defineMacro("©", "\\copyright");
  defineMacro("®", "\\textregistered");
  defineMacro("️", "\\textregistered");
  defineMacro("\\ulcorner", '\\html@mathml{\\@ulcorner}{\\mathop{\\char"231c}}');
  defineMacro("\\urcorner", '\\html@mathml{\\@urcorner}{\\mathop{\\char"231d}}');
  defineMacro("\\llcorner", '\\html@mathml{\\@llcorner}{\\mathop{\\char"231e}}');
  defineMacro("\\lrcorner", '\\html@mathml{\\@lrcorner}{\\mathop{\\char"231f}}');
  defineMacro("\\vdots", "{\\varvdots\\rule{0pt}{15pt}}");
  defineMacro("⋮", "\\vdots");
  defineMacro("\\varGamma", "\\mathit{\\Gamma}");
  defineMacro("\\varDelta", "\\mathit{\\Delta}");
  defineMacro("\\varTheta", "\\mathit{\\Theta}");
  defineMacro("\\varLambda", "\\mathit{\\Lambda}");
  defineMacro("\\varXi", "\\mathit{\\Xi}");
  defineMacro("\\varPi", "\\mathit{\\Pi}");
  defineMacro("\\varSigma", "\\mathit{\\Sigma}");
  defineMacro("\\varUpsilon", "\\mathit{\\Upsilon}");
  defineMacro("\\varPhi", "\\mathit{\\Phi}");
  defineMacro("\\varPsi", "\\mathit{\\Psi}");
  defineMacro("\\varOmega", "\\mathit{\\Omega}");
  defineMacro("\\substack", "\\begin{subarray}{c}#1\\end{subarray}");
  defineMacro("\\colon", "\\nobreak\\mskip2mu\\mathpunct{}\\mathchoice{\\mkern-3mu}{\\mkern-3mu}{}{}{:}\\mskip6mu\\relax");
  defineMacro("\\boxed", "\\fbox{$\\displaystyle{#1}$}");
  defineMacro("\\iff", "\\DOTSB\\;\\Longleftrightarrow\\;");
  defineMacro("\\implies", "\\DOTSB\\;\\Longrightarrow\\;");
  defineMacro("\\impliedby", "\\DOTSB\\;\\Longleftarrow\\;");
  defineMacro("\\dddot", "{\\overset{\\raisebox{-0.1ex}{\\normalsize ...}}{#1}}");
  defineMacro("\\ddddot", "{\\overset{\\raisebox{-0.1ex}{\\normalsize ....}}{#1}}");
  var dotsByToken = {
    ",": "\\dotsc",
    "\\not": "\\dotsb",
    // \keybin@ checks for the following:
    "+": "\\dotsb",
    "=": "\\dotsb",
    "<": "\\dotsb",
    ">": "\\dotsb",
    "-": "\\dotsb",
    "*": "\\dotsb",
    ":": "\\dotsb",
    // Symbols whose definition starts with \DOTSB:
    "\\DOTSB": "\\dotsb",
    "\\coprod": "\\dotsb",
    "\\bigvee": "\\dotsb",
    "\\bigwedge": "\\dotsb",
    "\\biguplus": "\\dotsb",
    "\\bigcap": "\\dotsb",
    "\\bigcup": "\\dotsb",
    "\\prod": "\\dotsb",
    "\\sum": "\\dotsb",
    "\\bigotimes": "\\dotsb",
    "\\bigoplus": "\\dotsb",
    "\\bigodot": "\\dotsb",
    "\\bigsqcup": "\\dotsb",
    "\\And": "\\dotsb",
    "\\longrightarrow": "\\dotsb",
    "\\Longrightarrow": "\\dotsb",
    "\\longleftarrow": "\\dotsb",
    "\\Longleftarrow": "\\dotsb",
    "\\longleftrightarrow": "\\dotsb",
    "\\Longleftrightarrow": "\\dotsb",
    "\\mapsto": "\\dotsb",
    "\\longmapsto": "\\dotsb",
    "\\hookrightarrow": "\\dotsb",
    "\\doteq": "\\dotsb",
    // Symbols whose definition starts with \mathbin:
    "\\mathbin": "\\dotsb",
    // Symbols whose definition starts with \mathrel:
    "\\mathrel": "\\dotsb",
    "\\relbar": "\\dotsb",
    "\\Relbar": "\\dotsb",
    "\\xrightarrow": "\\dotsb",
    "\\xleftarrow": "\\dotsb",
    // Symbols whose definition starts with \DOTSI:
    "\\DOTSI": "\\dotsi",
    "\\int": "\\dotsi",
    "\\oint": "\\dotsi",
    "\\iint": "\\dotsi",
    "\\iiint": "\\dotsi",
    "\\iiiint": "\\dotsi",
    "\\idotsint": "\\dotsi",
    // Symbols whose definition starts with \DOTSX:
    "\\DOTSX": "\\dotsx"
  };
  defineMacro("\\dots", function(O) {
    var _ = "\\dotso", H = O.expandAfterFuture().text;
    return H in dotsByToken ? _ = dotsByToken[H] : (H.slice(0, 4) === "\\not" || H in symbols$3.math && utils.contains(["bin", "rel"], symbols$3.math[H].group)) && (_ = "\\dotsb"), _;
  });
  var spaceAfterDots = {
    // \rightdelim@ checks for the following:
    ")": !0,
    "]": !0,
    "\\rbrack": !0,
    "\\}": !0,
    "\\rbrace": !0,
    "\\rangle": !0,
    "\\rceil": !0,
    "\\rfloor": !0,
    "\\rgroup": !0,
    "\\rmoustache": !0,
    "\\right": !0,
    "\\bigr": !0,
    "\\biggr": !0,
    "\\Bigr": !0,
    "\\Biggr": !0,
    // \extra@ also tests for the following:
    $: !0,
    // \extrap@ checks for the following:
    ";": !0,
    ".": !0,
    ",": !0
  };
  defineMacro("\\dotso", function(O) {
    var _ = O.future().text;
    return _ in spaceAfterDots ? "\\ldots\\," : "\\ldots";
  });
  defineMacro("\\dotsc", function(O) {
    var _ = O.future().text;
    return _ in spaceAfterDots && _ !== "," ? "\\ldots\\," : "\\ldots";
  });
  defineMacro("\\cdots", function(O) {
    var _ = O.future().text;
    return _ in spaceAfterDots ? "\\@cdots\\," : "\\@cdots";
  });
  defineMacro("\\dotsb", "\\cdots");
  defineMacro("\\dotsm", "\\cdots");
  defineMacro("\\dotsi", "\\!\\cdots");
  defineMacro("\\dotsx", "\\ldots\\,");
  defineMacro("\\DOTSI", "\\relax");
  defineMacro("\\DOTSB", "\\relax");
  defineMacro("\\DOTSX", "\\relax");
  defineMacro("\\tmspace", "\\TextOrMath{\\kern#1#3}{\\mskip#1#2}\\relax");
  defineMacro("\\,", "\\tmspace+{3mu}{.1667em}");
  defineMacro("\\thinspace", "\\,");
  defineMacro("\\>", "\\mskip{4mu}");
  defineMacro("\\:", "\\tmspace+{4mu}{.2222em}");
  defineMacro("\\medspace", "\\:");
  defineMacro("\\;", "\\tmspace+{5mu}{.2777em}");
  defineMacro("\\thickspace", "\\;");
  defineMacro("\\!", "\\tmspace-{3mu}{.1667em}");
  defineMacro("\\negthinspace", "\\!");
  defineMacro("\\negmedspace", "\\tmspace-{4mu}{.2222em}");
  defineMacro("\\negthickspace", "\\tmspace-{5mu}{.277em}");
  defineMacro("\\enspace", "\\kern.5em ");
  defineMacro("\\enskip", "\\hskip.5em\\relax");
  defineMacro("\\quad", "\\hskip1em\\relax");
  defineMacro("\\qquad", "\\hskip2em\\relax");
  defineMacro("\\tag", "\\@ifstar\\tag@literal\\tag@paren");
  defineMacro("\\tag@paren", "\\tag@literal{({#1})}");
  defineMacro("\\tag@literal", (O) => {
    if (O.macros.get("\\df@tag"))
      throw new ParseError("Multiple \\tag");
    return "\\gdef\\df@tag{\\text{#1}}";
  });
  defineMacro("\\bmod", "\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}\\mathbin{\\rm mod}\\mathchoice{\\mskip1mu}{\\mskip1mu}{\\mskip5mu}{\\mskip5mu}");
  defineMacro("\\pod", "\\allowbreak\\mathchoice{\\mkern18mu}{\\mkern8mu}{\\mkern8mu}{\\mkern8mu}(#1)");
  defineMacro("\\pmod", "\\pod{{\\rm mod}\\mkern6mu#1}");
  defineMacro("\\mod", "\\allowbreak\\mathchoice{\\mkern18mu}{\\mkern12mu}{\\mkern12mu}{\\mkern12mu}{\\rm mod}\\,\\,#1");
  defineMacro("\\newline", "\\\\\\relax");
  defineMacro("\\TeX", "\\textrm{\\html@mathml{T\\kern-.1667em\\raisebox{-.5ex}{E}\\kern-.125emX}{TeX}}");
  var latexRaiseA = makeEm(fontMetricsData["Main-Regular"][84][1] - 0.7 * fontMetricsData["Main-Regular"][65][1]);
  defineMacro("\\LaTeX", "\\textrm{\\html@mathml{" + ("L\\kern-.36em\\raisebox{" + latexRaiseA + "}{\\scriptstyle A}") + "\\kern-.15em\\TeX}{LaTeX}}");
  defineMacro("\\KaTeX", "\\textrm{\\html@mathml{" + ("K\\kern-.17em\\raisebox{" + latexRaiseA + "}{\\scriptstyle A}") + "\\kern-.15em\\TeX}{KaTeX}}");
  defineMacro("\\hspace", "\\@ifstar\\@hspacer\\@hspace");
  defineMacro("\\@hspace", "\\hskip #1\\relax");
  defineMacro("\\@hspacer", "\\rule{0pt}{0pt}\\hskip #1\\relax");
  defineMacro("\\ordinarycolon", ":");
  defineMacro("\\vcentcolon", "\\mathrel{\\mathop\\ordinarycolon}");
  defineMacro("\\dblcolon", '\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-.9mu}\\vcentcolon}}{\\mathop{\\char"2237}}');
  defineMacro("\\coloneqq", '\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}=}}{\\mathop{\\char"2254}}');
  defineMacro("\\Coloneqq", '\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}=}}{\\mathop{\\char"2237\\char"3d}}');
  defineMacro("\\coloneq", '\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}{\\mathop{\\char"3a\\char"2212}}');
  defineMacro("\\Coloneq", '\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\mathrel{-}}}{\\mathop{\\char"2237\\char"2212}}');
  defineMacro("\\eqqcolon", '\\html@mathml{\\mathrel{=\\mathrel{\\mkern-1.2mu}\\vcentcolon}}{\\mathop{\\char"2255}}');
  defineMacro("\\Eqqcolon", '\\html@mathml{\\mathrel{=\\mathrel{\\mkern-1.2mu}\\dblcolon}}{\\mathop{\\char"3d\\char"2237}}');
  defineMacro("\\eqcolon", '\\html@mathml{\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\vcentcolon}}{\\mathop{\\char"2239}}');
  defineMacro("\\Eqcolon", '\\html@mathml{\\mathrel{\\mathrel{-}\\mathrel{\\mkern-1.2mu}\\dblcolon}}{\\mathop{\\char"2212\\char"2237}}');
  defineMacro("\\colonapprox", '\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\approx}}{\\mathop{\\char"3a\\char"2248}}');
  defineMacro("\\Colonapprox", '\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\approx}}{\\mathop{\\char"2237\\char"2248}}');
  defineMacro("\\colonsim", '\\html@mathml{\\mathrel{\\vcentcolon\\mathrel{\\mkern-1.2mu}\\sim}}{\\mathop{\\char"3a\\char"223c}}');
  defineMacro("\\Colonsim", '\\html@mathml{\\mathrel{\\dblcolon\\mathrel{\\mkern-1.2mu}\\sim}}{\\mathop{\\char"2237\\char"223c}}');
  defineMacro("∷", "\\dblcolon");
  defineMacro("∹", "\\eqcolon");
  defineMacro("≔", "\\coloneqq");
  defineMacro("≕", "\\eqqcolon");
  defineMacro("⩴", "\\Coloneqq");
  defineMacro("\\ratio", "\\vcentcolon");
  defineMacro("\\coloncolon", "\\dblcolon");
  defineMacro("\\colonequals", "\\coloneqq");
  defineMacro("\\coloncolonequals", "\\Coloneqq");
  defineMacro("\\equalscolon", "\\eqqcolon");
  defineMacro("\\equalscoloncolon", "\\Eqqcolon");
  defineMacro("\\colonminus", "\\coloneq");
  defineMacro("\\coloncolonminus", "\\Coloneq");
  defineMacro("\\minuscolon", "\\eqcolon");
  defineMacro("\\minuscoloncolon", "\\Eqcolon");
  defineMacro("\\coloncolonapprox", "\\Colonapprox");
  defineMacro("\\coloncolonsim", "\\Colonsim");
  defineMacro("\\simcolon", "\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\vcentcolon}");
  defineMacro("\\simcoloncolon", "\\mathrel{\\sim\\mathrel{\\mkern-1.2mu}\\dblcolon}");
  defineMacro("\\approxcolon", "\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\vcentcolon}");
  defineMacro("\\approxcoloncolon", "\\mathrel{\\approx\\mathrel{\\mkern-1.2mu}\\dblcolon}");
  defineMacro("\\notni", "\\html@mathml{\\not\\ni}{\\mathrel{\\char`∌}}");
  defineMacro("\\limsup", "\\DOTSB\\operatorname*{lim\\,sup}");
  defineMacro("\\liminf", "\\DOTSB\\operatorname*{lim\\,inf}");
  defineMacro("\\injlim", "\\DOTSB\\operatorname*{inj\\,lim}");
  defineMacro("\\projlim", "\\DOTSB\\operatorname*{proj\\,lim}");
  defineMacro("\\varlimsup", "\\DOTSB\\operatorname*{\\overline{lim}}");
  defineMacro("\\varliminf", "\\DOTSB\\operatorname*{\\underline{lim}}");
  defineMacro("\\varinjlim", "\\DOTSB\\operatorname*{\\underrightarrow{lim}}");
  defineMacro("\\varprojlim", "\\DOTSB\\operatorname*{\\underleftarrow{lim}}");
  defineMacro("\\gvertneqq", "\\html@mathml{\\@gvertneqq}{≩}");
  defineMacro("\\lvertneqq", "\\html@mathml{\\@lvertneqq}{≨}");
  defineMacro("\\ngeqq", "\\html@mathml{\\@ngeqq}{≱}");
  defineMacro("\\ngeqslant", "\\html@mathml{\\@ngeqslant}{≱}");
  defineMacro("\\nleqq", "\\html@mathml{\\@nleqq}{≰}");
  defineMacro("\\nleqslant", "\\html@mathml{\\@nleqslant}{≰}");
  defineMacro("\\nshortmid", "\\html@mathml{\\@nshortmid}{∤}");
  defineMacro("\\nshortparallel", "\\html@mathml{\\@nshortparallel}{∦}");
  defineMacro("\\nsubseteqq", "\\html@mathml{\\@nsubseteqq}{⊈}");
  defineMacro("\\nsupseteqq", "\\html@mathml{\\@nsupseteqq}{⊉}");
  defineMacro("\\varsubsetneq", "\\html@mathml{\\@varsubsetneq}{⊊}");
  defineMacro("\\varsubsetneqq", "\\html@mathml{\\@varsubsetneqq}{⫋}");
  defineMacro("\\varsupsetneq", "\\html@mathml{\\@varsupsetneq}{⊋}");
  defineMacro("\\varsupsetneqq", "\\html@mathml{\\@varsupsetneqq}{⫌}");
  defineMacro("\\imath", "\\html@mathml{\\@imath}{ı}");
  defineMacro("\\jmath", "\\html@mathml{\\@jmath}{ȷ}");
  defineMacro("\\llbracket", "\\html@mathml{\\mathopen{[\\mkern-3.2mu[}}{\\mathopen{\\char`⟦}}");
  defineMacro("\\rrbracket", "\\html@mathml{\\mathclose{]\\mkern-3.2mu]}}{\\mathclose{\\char`⟧}}");
  defineMacro("⟦", "\\llbracket");
  defineMacro("⟧", "\\rrbracket");
  defineMacro("\\lBrace", "\\html@mathml{\\mathopen{\\{\\mkern-3.2mu[}}{\\mathopen{\\char`⦃}}");
  defineMacro("\\rBrace", "\\html@mathml{\\mathclose{]\\mkern-3.2mu\\}}}{\\mathclose{\\char`⦄}}");
  defineMacro("⦃", "\\lBrace");
  defineMacro("⦄", "\\rBrace");
  defineMacro("\\minuso", "\\mathbin{\\html@mathml{{\\mathrlap{\\mathchoice{\\kern{0.145em}}{\\kern{0.145em}}{\\kern{0.1015em}}{\\kern{0.0725em}}\\circ}{-}}}{\\char`⦵}}");
  defineMacro("⦵", "\\minuso");
  defineMacro("\\darr", "\\downarrow");
  defineMacro("\\dArr", "\\Downarrow");
  defineMacro("\\Darr", "\\Downarrow");
  defineMacro("\\lang", "\\langle");
  defineMacro("\\rang", "\\rangle");
  defineMacro("\\uarr", "\\uparrow");
  defineMacro("\\uArr", "\\Uparrow");
  defineMacro("\\Uarr", "\\Uparrow");
  defineMacro("\\N", "\\mathbb{N}");
  defineMacro("\\R", "\\mathbb{R}");
  defineMacro("\\Z", "\\mathbb{Z}");
  defineMacro("\\alef", "\\aleph");
  defineMacro("\\alefsym", "\\aleph");
  defineMacro("\\Alpha", "\\mathrm{A}");
  defineMacro("\\Beta", "\\mathrm{B}");
  defineMacro("\\bull", "\\bullet");
  defineMacro("\\Chi", "\\mathrm{X}");
  defineMacro("\\clubs", "\\clubsuit");
  defineMacro("\\cnums", "\\mathbb{C}");
  defineMacro("\\Complex", "\\mathbb{C}");
  defineMacro("\\Dagger", "\\ddagger");
  defineMacro("\\diamonds", "\\diamondsuit");
  defineMacro("\\empty", "\\emptyset");
  defineMacro("\\Epsilon", "\\mathrm{E}");
  defineMacro("\\Eta", "\\mathrm{H}");
  defineMacro("\\exist", "\\exists");
  defineMacro("\\harr", "\\leftrightarrow");
  defineMacro("\\hArr", "\\Leftrightarrow");
  defineMacro("\\Harr", "\\Leftrightarrow");
  defineMacro("\\hearts", "\\heartsuit");
  defineMacro("\\image", "\\Im");
  defineMacro("\\infin", "\\infty");
  defineMacro("\\Iota", "\\mathrm{I}");
  defineMacro("\\isin", "\\in");
  defineMacro("\\Kappa", "\\mathrm{K}");
  defineMacro("\\larr", "\\leftarrow");
  defineMacro("\\lArr", "\\Leftarrow");
  defineMacro("\\Larr", "\\Leftarrow");
  defineMacro("\\lrarr", "\\leftrightarrow");
  defineMacro("\\lrArr", "\\Leftrightarrow");
  defineMacro("\\Lrarr", "\\Leftrightarrow");
  defineMacro("\\Mu", "\\mathrm{M}");
  defineMacro("\\natnums", "\\mathbb{N}");
  defineMacro("\\Nu", "\\mathrm{N}");
  defineMacro("\\Omicron", "\\mathrm{O}");
  defineMacro("\\plusmn", "\\pm");
  defineMacro("\\rarr", "\\rightarrow");
  defineMacro("\\rArr", "\\Rightarrow");
  defineMacro("\\Rarr", "\\Rightarrow");
  defineMacro("\\real", "\\Re");
  defineMacro("\\reals", "\\mathbb{R}");
  defineMacro("\\Reals", "\\mathbb{R}");
  defineMacro("\\Rho", "\\mathrm{P}");
  defineMacro("\\sdot", "\\cdot");
  defineMacro("\\sect", "\\S");
  defineMacro("\\spades", "\\spadesuit");
  defineMacro("\\sub", "\\subset");
  defineMacro("\\sube", "\\subseteq");
  defineMacro("\\supe", "\\supseteq");
  defineMacro("\\Tau", "\\mathrm{T}");
  defineMacro("\\thetasym", "\\vartheta");
  defineMacro("\\weierp", "\\wp");
  defineMacro("\\Zeta", "\\mathrm{Z}");
  defineMacro("\\argmin", "\\DOTSB\\operatorname*{arg\\,min}");
  defineMacro("\\argmax", "\\DOTSB\\operatorname*{arg\\,max}");
  defineMacro("\\plim", "\\DOTSB\\mathop{\\operatorname{plim}}\\limits");
  defineMacro("\\bra", "\\mathinner{\\langle{#1}|}");
  defineMacro("\\ket", "\\mathinner{|{#1}\\rangle}");
  defineMacro("\\braket", "\\mathinner{\\langle{#1}\\rangle}");
  defineMacro("\\Bra", "\\left\\langle#1\\right|");
  defineMacro("\\Ket", "\\left|#1\\right\\rangle");
  var braketHelper = (O) => (_) => {
    var H = _.consumeArg().tokens, V = _.consumeArg().tokens, q = _.consumeArg().tokens, G = _.consumeArg().tokens, J = _.macros.get("|"), Z = _.macros.get("\\|");
    _.macros.beginGroup();
    var te = (ae) => (oe) => {
      O && (oe.macros.set("|", J), q.length && oe.macros.set("\\|", Z));
      var se = ae;
      if (!ae && q.length) {
        var ce = oe.future();
        ce.text === "|" && (oe.popToken(), se = !0);
      }
      return {
        tokens: se ? q : V,
        numArgs: 0
      };
    };
    _.macros.set("|", te(!1)), q.length && _.macros.set("\\|", te(!0));
    var ne = _.consumeArg().tokens, ee = _.expandTokens([
      ...G,
      ...ne,
      ...H
      // reversed
    ]);
    return _.macros.endGroup(), {
      tokens: ee.reverse(),
      numArgs: 0
    };
  };
  defineMacro("\\bra@ket", braketHelper(!1));
  defineMacro("\\bra@set", braketHelper(!0));
  defineMacro("\\Braket", "\\bra@ket{\\left\\langle}{\\,\\middle\\vert\\,}{\\,\\middle\\vert\\,}{\\right\\rangle}");
  defineMacro("\\Set", "\\bra@set{\\left\\{\\:}{\\;\\middle\\vert\\;}{\\;\\middle\\Vert\\;}{\\:\\right\\}}");
  defineMacro("\\set", "\\bra@set{\\{\\,}{\\mid}{}{\\,\\}}");
  defineMacro("\\angln", "{\\angl n}");
  defineMacro("\\blue", "\\textcolor{##6495ed}{#1}");
  defineMacro("\\orange", "\\textcolor{##ffa500}{#1}");
  defineMacro("\\pink", "\\textcolor{##ff00af}{#1}");
  defineMacro("\\red", "\\textcolor{##df0030}{#1}");
  defineMacro("\\green", "\\textcolor{##28ae7b}{#1}");
  defineMacro("\\gray", "\\textcolor{gray}{#1}");
  defineMacro("\\purple", "\\textcolor{##9d38bd}{#1}");
  defineMacro("\\blueA", "\\textcolor{##ccfaff}{#1}");
  defineMacro("\\blueB", "\\textcolor{##80f6ff}{#1}");
  defineMacro("\\blueC", "\\textcolor{##63d9ea}{#1}");
  defineMacro("\\blueD", "\\textcolor{##11accd}{#1}");
  defineMacro("\\blueE", "\\textcolor{##0c7f99}{#1}");
  defineMacro("\\tealA", "\\textcolor{##94fff5}{#1}");
  defineMacro("\\tealB", "\\textcolor{##26edd5}{#1}");
  defineMacro("\\tealC", "\\textcolor{##01d1c1}{#1}");
  defineMacro("\\tealD", "\\textcolor{##01a995}{#1}");
  defineMacro("\\tealE", "\\textcolor{##208170}{#1}");
  defineMacro("\\greenA", "\\textcolor{##b6ffb0}{#1}");
  defineMacro("\\greenB", "\\textcolor{##8af281}{#1}");
  defineMacro("\\greenC", "\\textcolor{##74cf70}{#1}");
  defineMacro("\\greenD", "\\textcolor{##1fab54}{#1}");
  defineMacro("\\greenE", "\\textcolor{##0d923f}{#1}");
  defineMacro("\\goldA", "\\textcolor{##ffd0a9}{#1}");
  defineMacro("\\goldB", "\\textcolor{##ffbb71}{#1}");
  defineMacro("\\goldC", "\\textcolor{##ff9c39}{#1}");
  defineMacro("\\goldD", "\\textcolor{##e07d10}{#1}");
  defineMacro("\\goldE", "\\textcolor{##a75a05}{#1}");
  defineMacro("\\redA", "\\textcolor{##fca9a9}{#1}");
  defineMacro("\\redB", "\\textcolor{##ff8482}{#1}");
  defineMacro("\\redC", "\\textcolor{##f9685d}{#1}");
  defineMacro("\\redD", "\\textcolor{##e84d39}{#1}");
  defineMacro("\\redE", "\\textcolor{##bc2612}{#1}");
  defineMacro("\\maroonA", "\\textcolor{##ffbde0}{#1}");
  defineMacro("\\maroonB", "\\textcolor{##ff92c6}{#1}");
  defineMacro("\\maroonC", "\\textcolor{##ed5fa6}{#1}");
  defineMacro("\\maroonD", "\\textcolor{##ca337c}{#1}");
  defineMacro("\\maroonE", "\\textcolor{##9e034e}{#1}");
  defineMacro("\\purpleA", "\\textcolor{##ddd7ff}{#1}");
  defineMacro("\\purpleB", "\\textcolor{##c6b9fc}{#1}");
  defineMacro("\\purpleC", "\\textcolor{##aa87ff}{#1}");
  defineMacro("\\purpleD", "\\textcolor{##7854ab}{#1}");
  defineMacro("\\purpleE", "\\textcolor{##543b78}{#1}");
  defineMacro("\\mintA", "\\textcolor{##f5f9e8}{#1}");
  defineMacro("\\mintB", "\\textcolor{##edf2df}{#1}");
  defineMacro("\\mintC", "\\textcolor{##e0e5cc}{#1}");
  defineMacro("\\grayA", "\\textcolor{##f6f7f7}{#1}");
  defineMacro("\\grayB", "\\textcolor{##f0f1f2}{#1}");
  defineMacro("\\grayC", "\\textcolor{##e3e5e6}{#1}");
  defineMacro("\\grayD", "\\textcolor{##d6d8da}{#1}");
  defineMacro("\\grayE", "\\textcolor{##babec2}{#1}");
  defineMacro("\\grayF", "\\textcolor{##888d93}{#1}");
  defineMacro("\\grayG", "\\textcolor{##626569}{#1}");
  defineMacro("\\grayH", "\\textcolor{##3b3e40}{#1}");
  defineMacro("\\grayI", "\\textcolor{##21242c}{#1}");
  defineMacro("\\kaBlue", "\\textcolor{##314453}{#1}");
  defineMacro("\\kaGreen", "\\textcolor{##71B307}{#1}");
  var implicitCommands = {
    "^": !0,
    // Parser.js
    _: !0,
    // Parser.js
    "\\limits": !0,
    // Parser.js
    "\\nolimits": !0
    // Parser.js
  };
  class MacroExpander {
    constructor(_, H, V) {
      this.settings = void 0, this.expansionCount = void 0, this.lexer = void 0, this.macros = void 0, this.stack = void 0, this.mode = void 0, this.settings = H, this.expansionCount = 0, this.feed(_), this.macros = new Namespace(macros, H.macros), this.mode = V, this.stack = [];
    }
    /**
     * Feed a new input string to the same MacroExpander
     * (with existing macros etc.).
     */
    feed(_) {
      this.lexer = new Lexer(_, this.settings);
    }
    /**
     * Switches between "text" and "math" modes.
     */
    switchMode(_) {
      this.mode = _;
    }
    /**
     * Start a new group nesting within all namespaces.
     */
    beginGroup() {
      this.macros.beginGroup();
    }
    /**
     * End current group nesting within all namespaces.
     */
    endGroup() {
      this.macros.endGroup();
    }
    /**
     * Ends all currently nested groups (if any), restoring values before the
     * groups began.  Useful in case of an error in the middle of parsing.
     */
    endGroups() {
      this.macros.endGroups();
    }
    /**
     * Returns the topmost token on the stack, without expanding it.
     * Similar in behavior to TeX's `\futurelet`.
     */
    future() {
      return this.stack.length === 0 && this.pushToken(this.lexer.lex()), this.stack[this.stack.length - 1];
    }
    /**
     * Remove and return the next unexpanded token.
     */
    popToken() {
      return this.future(), this.stack.pop();
    }
    /**
     * Add a given token to the token stack.  In particular, this get be used
     * to put back a token returned from one of the other methods.
     */
    pushToken(_) {
      this.stack.push(_);
    }
    /**
     * Append an array of tokens to the token stack.
     */
    pushTokens(_) {
      this.stack.push(..._);
    }
    /**
     * Find an macro argument without expanding tokens and append the array of
     * tokens to the token stack. Uses Token as a container for the result.
     */
    scanArgument(_) {
      var H, V, q;
      if (_) {
        if (this.consumeSpaces(), this.future().text !== "[")
          return null;
        H = this.popToken(), {
          tokens: q,
          end: V
        } = this.consumeArg(["]"]);
      } else
        ({
          tokens: q,
          start: H,
          end: V
        } = this.consumeArg());
      return this.pushToken(new Token("EOF", V.loc)), this.pushTokens(q), H.range(V, "");
    }
    /**
     * Consume all following space tokens, without expansion.
     */
    consumeSpaces() {
      for (; ; ) {
        var _ = this.future();
        if (_.text === " ")
          this.stack.pop();
        else
          break;
      }
    }
    /**
     * Consume an argument from the token stream, and return the resulting array
     * of tokens and start/end token.
     */
    consumeArg(_) {
      var H = [], V = _ && _.length > 0;
      V || this.consumeSpaces();
      var q = this.future(), G, J = 0, Z = 0;
      do {
        if (G = this.popToken(), H.push(G), G.text === "{")
          ++J;
        else if (G.text === "}") {
          if (--J, J === -1)
            throw new ParseError("Extra }", G);
        } else if (G.text === "EOF")
          throw new ParseError("Unexpected end of input in a macro argument, expected '" + (_ && V ? _[Z] : "}") + "'", G);
        if (_ && V)
          if ((J === 0 || J === 1 && _[Z] === "{") && G.text === _[Z]) {
            if (++Z, Z === _.length) {
              H.splice(-Z, Z);
              break;
            }
          } else
            Z = 0;
      } while (J !== 0 || V);
      return q.text === "{" && H[H.length - 1].text === "}" && (H.pop(), H.shift()), H.reverse(), {
        tokens: H,
        start: q,
        end: G
      };
    }
    /**
     * Consume the specified number of (delimited) arguments from the token
     * stream and return the resulting array of arguments.
     */
    consumeArgs(_, H) {
      if (H) {
        if (H.length !== _ + 1)
          throw new ParseError("The length of delimiters doesn't match the number of args!");
        for (var V = H[0], q = 0; q < V.length; q++) {
          var G = this.popToken();
          if (V[q] !== G.text)
            throw new ParseError("Use of the macro doesn't match its definition", G);
        }
      }
      for (var J = [], Z = 0; Z < _; Z++)
        J.push(this.consumeArg(H && H[Z + 1]).tokens);
      return J;
    }
    /**
     * Increment `expansionCount` by the specified amount.
     * Throw an error if it exceeds `maxExpand`.
     */
    countExpansion(_) {
      if (this.expansionCount += _, this.expansionCount > this.settings.maxExpand)
        throw new ParseError("Too many expansions: infinite loop or need to increase maxExpand setting");
    }
    /**
     * Expand the next token only once if possible.
     *
     * If the token is expanded, the resulting tokens will be pushed onto
     * the stack in reverse order, and the number of such tokens will be
     * returned.  This number might be zero or positive.
     *
     * If not, the return value is `false`, and the next token remains at the
     * top of the stack.
     *
     * In either case, the next token will be on the top of the stack,
     * or the stack will be empty (in case of empty expansion
     * and no other tokens).
     *
     * Used to implement `expandAfterFuture` and `expandNextToken`.
     *
     * If expandableOnly, only expandable tokens are expanded and
     * an undefined control sequence results in an error.
     */
    expandOnce(_) {
      var H = this.popToken(), V = H.text, q = H.noexpand ? null : this._getExpansion(V);
      if (q == null || _ && q.unexpandable) {
        if (_ && q == null && V[0] === "\\" && !this.isDefined(V))
          throw new ParseError("Undefined control sequence: " + V);
        return this.pushToken(H), !1;
      }
      this.countExpansion(1);
      var G = q.tokens, J = this.consumeArgs(q.numArgs, q.delimiters);
      if (q.numArgs) {
        G = G.slice();
        for (var Z = G.length - 1; Z >= 0; --Z) {
          var te = G[Z];
          if (te.text === "#") {
            if (Z === 0)
              throw new ParseError("Incomplete placeholder at end of macro body", te);
            if (te = G[--Z], te.text === "#")
              G.splice(Z + 1, 1);
            else if (/^[1-9]$/.test(te.text))
              G.splice(Z, 2, ...J[+te.text - 1]);
            else
              throw new ParseError("Not a valid argument number", te);
          }
        }
      }
      return this.pushTokens(G), G.length;
    }
    /**
     * Expand the next token only once (if possible), and return the resulting
     * top token on the stack (without removing anything from the stack).
     * Similar in behavior to TeX's `\expandafter\futurelet`.
     * Equivalent to expandOnce() followed by future().
     */
    expandAfterFuture() {
      return this.expandOnce(), this.future();
    }
    /**
     * Recursively expand first token, then return first non-expandable token.
     */
    expandNextToken() {
      for (; ; )
        if (this.expandOnce() === !1) {
          var _ = this.stack.pop();
          return _.treatAsRelax && (_.text = "\\relax"), _;
        }
      throw new Error();
    }
    /**
     * Fully expand the given macro name and return the resulting list of
     * tokens, or return `undefined` if no such macro is defined.
     */
    expandMacro(_) {
      return this.macros.has(_) ? this.expandTokens([new Token(_)]) : void 0;
    }
    /**
     * Fully expand the given token stream and return the resulting list of
     * tokens.  Note that the input tokens are in reverse order, but the
     * output tokens are in forward order.
     */
    expandTokens(_) {
      var H = [], V = this.stack.length;
      for (this.pushTokens(_); this.stack.length > V; )
        if (this.expandOnce(!0) === !1) {
          var q = this.stack.pop();
          q.treatAsRelax && (q.noexpand = !1, q.treatAsRelax = !1), H.push(q);
        }
      return this.countExpansion(H.length), H;
    }
    /**
     * Fully expand the given macro name and return the result as a string,
     * or return `undefined` if no such macro is defined.
     */
    expandMacroAsText(_) {
      var H = this.expandMacro(_);
      return H && H.map((V) => V.text).join("");
    }
    /**
     * Returns the expanded macro as a reversed array of tokens and a macro
     * argument count.  Or returns `null` if no such macro.
     */
    _getExpansion(_) {
      var H = this.macros.get(_);
      if (H == null)
        return H;
      if (_.length === 1) {
        var V = this.lexer.catcodes[_];
        if (V != null && V !== 13)
          return;
      }
      var q = typeof H == "function" ? H(this) : H;
      if (typeof q == "string") {
        var G = 0;
        if (q.indexOf("#") !== -1)
          for (var J = q.replace(/##/g, ""); J.indexOf("#" + (G + 1)) !== -1; )
            ++G;
        for (var Z = new Lexer(q, this.settings), te = [], ne = Z.lex(); ne.text !== "EOF"; )
          te.push(ne), ne = Z.lex();
        te.reverse();
        var ee = {
          tokens: te,
          numArgs: G
        };
        return ee;
      }
      return q;
    }
    /**
     * Determine whether a command is currently "defined" (has some
     * functionality), meaning that it's a macro (in the current group),
     * a function, a symbol, or one of the special commands listed in
     * `implicitCommands`.
     */
    isDefined(_) {
      return this.macros.has(_) || functions.hasOwnProperty(_) || symbols$3.math.hasOwnProperty(_) || symbols$3.text.hasOwnProperty(_) || implicitCommands.hasOwnProperty(_);
    }
    /**
     * Determine whether a command is expandable.
     */
    isExpandable(_) {
      var H = this.macros.get(_);
      return H != null ? typeof H == "string" || typeof H == "function" || !H.unexpandable : functions.hasOwnProperty(_) && !functions[_].primitive;
    }
  }
  var unicodeSubRegEx = /^[₊₋₌₍₎₀₁₂₃₄₅₆₇₈₉ₐₑₕᵢⱼₖₗₘₙₒₚᵣₛₜᵤᵥₓᵦᵧᵨᵩᵪ]/, uSubsAndSups = Object.freeze({
    "₊": "+",
    "₋": "-",
    "₌": "=",
    "₍": "(",
    "₎": ")",
    "₀": "0",
    "₁": "1",
    "₂": "2",
    "₃": "3",
    "₄": "4",
    "₅": "5",
    "₆": "6",
    "₇": "7",
    "₈": "8",
    "₉": "9",
    "ₐ": "a",
    "ₑ": "e",
    "ₕ": "h",
    "ᵢ": "i",
    "ⱼ": "j",
    "ₖ": "k",
    "ₗ": "l",
    "ₘ": "m",
    "ₙ": "n",
    "ₒ": "o",
    "ₚ": "p",
    "ᵣ": "r",
    "ₛ": "s",
    "ₜ": "t",
    "ᵤ": "u",
    "ᵥ": "v",
    "ₓ": "x",
    "ᵦ": "β",
    "ᵧ": "γ",
    "ᵨ": "ρ",
    "ᵩ": "ϕ",
    "ᵪ": "χ",
    "⁺": "+",
    "⁻": "-",
    "⁼": "=",
    "⁽": "(",
    "⁾": ")",
    "⁰": "0",
    "¹": "1",
    "²": "2",
    "³": "3",
    "⁴": "4",
    "⁵": "5",
    "⁶": "6",
    "⁷": "7",
    "⁸": "8",
    "⁹": "9",
    "ᴬ": "A",
    "ᴮ": "B",
    "ᴰ": "D",
    "ᴱ": "E",
    "ᴳ": "G",
    "ᴴ": "H",
    "ᴵ": "I",
    "ᴶ": "J",
    "ᴷ": "K",
    "ᴸ": "L",
    "ᴹ": "M",
    "ᴺ": "N",
    "ᴼ": "O",
    "ᴾ": "P",
    "ᴿ": "R",
    "ᵀ": "T",
    "ᵁ": "U",
    "ⱽ": "V",
    "ᵂ": "W",
    "ᵃ": "a",
    "ᵇ": "b",
    "ᶜ": "c",
    "ᵈ": "d",
    "ᵉ": "e",
    "ᶠ": "f",
    "ᵍ": "g",
    ʰ: "h",
    "ⁱ": "i",
    ʲ: "j",
    "ᵏ": "k",
    ˡ: "l",
    "ᵐ": "m",
    ⁿ: "n",
    "ᵒ": "o",
    "ᵖ": "p",
    ʳ: "r",
    ˢ: "s",
    "ᵗ": "t",
    "ᵘ": "u",
    "ᵛ": "v",
    ʷ: "w",
    ˣ: "x",
    ʸ: "y",
    "ᶻ": "z",
    "ᵝ": "β",
    "ᵞ": "γ",
    "ᵟ": "δ",
    "ᵠ": "ϕ",
    "ᵡ": "χ",
    "ᶿ": "θ"
  }), unicodeAccents = {
    "́": {
      text: "\\'",
      math: "\\acute"
    },
    "̀": {
      text: "\\`",
      math: "\\grave"
    },
    "̈": {
      text: '\\"',
      math: "\\ddot"
    },
    "̃": {
      text: "\\~",
      math: "\\tilde"
    },
    "̄": {
      text: "\\=",
      math: "\\bar"
    },
    "̆": {
      text: "\\u",
      math: "\\breve"
    },
    "̌": {
      text: "\\v",
      math: "\\check"
    },
    "̂": {
      text: "\\^",
      math: "\\hat"
    },
    "̇": {
      text: "\\.",
      math: "\\dot"
    },
    "̊": {
      text: "\\r",
      math: "\\mathring"
    },
    "̋": {
      text: "\\H"
    },
    "̧": {
      text: "\\c"
    }
  }, unicodeSymbols = {
    á: "á",
    à: "à",
    ä: "ä",
    ǟ: "ǟ",
    ã: "ã",
    ā: "ā",
    ă: "ă",
    ắ: "ắ",
    ằ: "ằ",
    ẵ: "ẵ",
    ǎ: "ǎ",
    â: "â",
    ấ: "ấ",
    ầ: "ầ",
    ẫ: "ẫ",
    ȧ: "ȧ",
    ǡ: "ǡ",
    å: "å",
    ǻ: "ǻ",
    ḃ: "ḃ",
    ć: "ć",
    ḉ: "ḉ",
    č: "č",
    ĉ: "ĉ",
    ċ: "ċ",
    ç: "ç",
    ď: "ď",
    ḋ: "ḋ",
    ḑ: "ḑ",
    é: "é",
    è: "è",
    ë: "ë",
    ẽ: "ẽ",
    ē: "ē",
    ḗ: "ḗ",
    ḕ: "ḕ",
    ĕ: "ĕ",
    ḝ: "ḝ",
    ě: "ě",
    ê: "ê",
    ế: "ế",
    ề: "ề",
    ễ: "ễ",
    ė: "ė",
    ȩ: "ȩ",
    ḟ: "ḟ",
    ǵ: "ǵ",
    ḡ: "ḡ",
    ğ: "ğ",
    ǧ: "ǧ",
    ĝ: "ĝ",
    ġ: "ġ",
    ģ: "ģ",
    ḧ: "ḧ",
    ȟ: "ȟ",
    ĥ: "ĥ",
    ḣ: "ḣ",
    ḩ: "ḩ",
    í: "í",
    ì: "ì",
    ï: "ï",
    ḯ: "ḯ",
    ĩ: "ĩ",
    ī: "ī",
    ĭ: "ĭ",
    ǐ: "ǐ",
    î: "î",
    ǰ: "ǰ",
    ĵ: "ĵ",
    ḱ: "ḱ",
    ǩ: "ǩ",
    ķ: "ķ",
    ĺ: "ĺ",
    ľ: "ľ",
    ļ: "ļ",
    ḿ: "ḿ",
    ṁ: "ṁ",
    ń: "ń",
    ǹ: "ǹ",
    ñ: "ñ",
    ň: "ň",
    ṅ: "ṅ",
    ņ: "ņ",
    ó: "ó",
    ò: "ò",
    ö: "ö",
    ȫ: "ȫ",
    õ: "õ",
    ṍ: "ṍ",
    ṏ: "ṏ",
    ȭ: "ȭ",
    ō: "ō",
    ṓ: "ṓ",
    ṑ: "ṑ",
    ŏ: "ŏ",
    ǒ: "ǒ",
    ô: "ô",
    ố: "ố",
    ồ: "ồ",
    ỗ: "ỗ",
    ȯ: "ȯ",
    ȱ: "ȱ",
    ő: "ő",
    ṕ: "ṕ",
    ṗ: "ṗ",
    ŕ: "ŕ",
    ř: "ř",
    ṙ: "ṙ",
    ŗ: "ŗ",
    ś: "ś",
    ṥ: "ṥ",
    š: "š",
    ṧ: "ṧ",
    ŝ: "ŝ",
    ṡ: "ṡ",
    ş: "ş",
    ẗ: "ẗ",
    ť: "ť",
    ṫ: "ṫ",
    ţ: "ţ",
    ú: "ú",
    ù: "ù",
    ü: "ü",
    ǘ: "ǘ",
    ǜ: "ǜ",
    ǖ: "ǖ",
    ǚ: "ǚ",
    ũ: "ũ",
    ṹ: "ṹ",
    ū: "ū",
    ṻ: "ṻ",
    ŭ: "ŭ",
    ǔ: "ǔ",
    û: "û",
    ů: "ů",
    ű: "ű",
    ṽ: "ṽ",
    ẃ: "ẃ",
    ẁ: "ẁ",
    ẅ: "ẅ",
    ŵ: "ŵ",
    ẇ: "ẇ",
    ẘ: "ẘ",
    ẍ: "ẍ",
    ẋ: "ẋ",
    ý: "ý",
    ỳ: "ỳ",
    ÿ: "ÿ",
    ỹ: "ỹ",
    ȳ: "ȳ",
    ŷ: "ŷ",
    ẏ: "ẏ",
    ẙ: "ẙ",
    ź: "ź",
    ž: "ž",
    ẑ: "ẑ",
    ż: "ż",
    Á: "Á",
    À: "À",
    Ä: "Ä",
    Ǟ: "Ǟ",
    Ã: "Ã",
    Ā: "Ā",
    Ă: "Ă",
    Ắ: "Ắ",
    Ằ: "Ằ",
    Ẵ: "Ẵ",
    Ǎ: "Ǎ",
    Â: "Â",
    Ấ: "Ấ",
    Ầ: "Ầ",
    Ẫ: "Ẫ",
    Ȧ: "Ȧ",
    Ǡ: "Ǡ",
    Å: "Å",
    Ǻ: "Ǻ",
    Ḃ: "Ḃ",
    Ć: "Ć",
    Ḉ: "Ḉ",
    Č: "Č",
    Ĉ: "Ĉ",
    Ċ: "Ċ",
    Ç: "Ç",
    Ď: "Ď",
    Ḋ: "Ḋ",
    Ḑ: "Ḑ",
    É: "É",
    È: "È",
    Ë: "Ë",
    Ẽ: "Ẽ",
    Ē: "Ē",
    Ḗ: "Ḗ",
    Ḕ: "Ḕ",
    Ĕ: "Ĕ",
    Ḝ: "Ḝ",
    Ě: "Ě",
    Ê: "Ê",
    Ế: "Ế",
    Ề: "Ề",
    Ễ: "Ễ",
    Ė: "Ė",
    Ȩ: "Ȩ",
    Ḟ: "Ḟ",
    Ǵ: "Ǵ",
    Ḡ: "Ḡ",
    Ğ: "Ğ",
    Ǧ: "Ǧ",
    Ĝ: "Ĝ",
    Ġ: "Ġ",
    Ģ: "Ģ",
    Ḧ: "Ḧ",
    Ȟ: "Ȟ",
    Ĥ: "Ĥ",
    Ḣ: "Ḣ",
    Ḩ: "Ḩ",
    Í: "Í",
    Ì: "Ì",
    Ï: "Ï",
    Ḯ: "Ḯ",
    Ĩ: "Ĩ",
    Ī: "Ī",
    Ĭ: "Ĭ",
    Ǐ: "Ǐ",
    Î: "Î",
    İ: "İ",
    Ĵ: "Ĵ",
    Ḱ: "Ḱ",
    Ǩ: "Ǩ",
    Ķ: "Ķ",
    Ĺ: "Ĺ",
    Ľ: "Ľ",
    Ļ: "Ļ",
    Ḿ: "Ḿ",
    Ṁ: "Ṁ",
    Ń: "Ń",
    Ǹ: "Ǹ",
    Ñ: "Ñ",
    Ň: "Ň",
    Ṅ: "Ṅ",
    Ņ: "Ņ",
    Ó: "Ó",
    Ò: "Ò",
    Ö: "Ö",
    Ȫ: "Ȫ",
    Õ: "Õ",
    Ṍ: "Ṍ",
    Ṏ: "Ṏ",
    Ȭ: "Ȭ",
    Ō: "Ō",
    Ṓ: "Ṓ",
    Ṑ: "Ṑ",
    Ŏ: "Ŏ",
    Ǒ: "Ǒ",
    Ô: "Ô",
    Ố: "Ố",
    Ồ: "Ồ",
    Ỗ: "Ỗ",
    Ȯ: "Ȯ",
    Ȱ: "Ȱ",
    Ő: "Ő",
    Ṕ: "Ṕ",
    Ṗ: "Ṗ",
    Ŕ: "Ŕ",
    Ř: "Ř",
    Ṙ: "Ṙ",
    Ŗ: "Ŗ",
    Ś: "Ś",
    Ṥ: "Ṥ",
    Š: "Š",
    Ṧ: "Ṧ",
    Ŝ: "Ŝ",
    Ṡ: "Ṡ",
    Ş: "Ş",
    Ť: "Ť",
    Ṫ: "Ṫ",
    Ţ: "Ţ",
    Ú: "Ú",
    Ù: "Ù",
    Ü: "Ü",
    Ǘ: "Ǘ",
    Ǜ: "Ǜ",
    Ǖ: "Ǖ",
    Ǚ: "Ǚ",
    Ũ: "Ũ",
    Ṹ: "Ṹ",
    Ū: "Ū",
    Ṻ: "Ṻ",
    Ŭ: "Ŭ",
    Ǔ: "Ǔ",
    Û: "Û",
    Ů: "Ů",
    Ű: "Ű",
    Ṽ: "Ṽ",
    Ẃ: "Ẃ",
    Ẁ: "Ẁ",
    Ẅ: "Ẅ",
    Ŵ: "Ŵ",
    Ẇ: "Ẇ",
    Ẍ: "Ẍ",
    Ẋ: "Ẋ",
    Ý: "Ý",
    Ỳ: "Ỳ",
    Ÿ: "Ÿ",
    Ỹ: "Ỹ",
    Ȳ: "Ȳ",
    Ŷ: "Ŷ",
    Ẏ: "Ẏ",
    Ź: "Ź",
    Ž: "Ž",
    Ẑ: "Ẑ",
    Ż: "Ż",
    ά: "ά",
    ὰ: "ὰ",
    ᾱ: "ᾱ",
    ᾰ: "ᾰ",
    έ: "έ",
    ὲ: "ὲ",
    ή: "ή",
    ὴ: "ὴ",
    ί: "ί",
    ὶ: "ὶ",
    ϊ: "ϊ",
    ΐ: "ΐ",
    ῒ: "ῒ",
    ῑ: "ῑ",
    ῐ: "ῐ",
    ό: "ό",
    ὸ: "ὸ",
    ύ: "ύ",
    ὺ: "ὺ",
    ϋ: "ϋ",
    ΰ: "ΰ",
    ῢ: "ῢ",
    ῡ: "ῡ",
    ῠ: "ῠ",
    ώ: "ώ",
    ὼ: "ὼ",
    Ύ: "Ύ",
    Ὺ: "Ὺ",
    Ϋ: "Ϋ",
    Ῡ: "Ῡ",
    Ῠ: "Ῠ",
    Ώ: "Ώ",
    Ὼ: "Ὼ"
  };
  let Parser$1 = class Ky {
    constructor(_, H) {
      this.mode = void 0, this.gullet = void 0, this.settings = void 0, this.leftrightDepth = void 0, this.nextToken = void 0, this.mode = "math", this.gullet = new MacroExpander(_, H, this.mode), this.settings = H, this.leftrightDepth = 0;
    }
    /**
     * Checks a result to make sure it has the right type, and throws an
     * appropriate error otherwise.
     */
    expect(_, H) {
      if (H === void 0 && (H = !0), this.fetch().text !== _)
        throw new ParseError("Expected '" + _ + "', got '" + this.fetch().text + "'", this.fetch());
      H && this.consume();
    }
    /**
     * Discards the current lookahead token, considering it consumed.
     */
    consume() {
      this.nextToken = null;
    }
    /**
     * Return the current lookahead token, or if there isn't one (at the
     * beginning, or if the previous lookahead token was consume()d),
     * fetch the next token as the new lookahead token and return it.
     */
    fetch() {
      return this.nextToken == null && (this.nextToken = this.gullet.expandNextToken()), this.nextToken;
    }
    /**
     * Switches between "text" and "math" modes.
     */
    switchMode(_) {
      this.mode = _, this.gullet.switchMode(_);
    }
    /**
     * Main parsing function, which parses an entire input.
     */
    parse() {
      this.settings.globalGroup || this.gullet.beginGroup(), this.settings.colorIsTextColor && this.gullet.macros.set("\\color", "\\textcolor");
      try {
        var _ = this.parseExpression(!1);
        return this.expect("EOF"), this.settings.globalGroup || this.gullet.endGroup(), _;
      } finally {
        this.gullet.endGroups();
      }
    }
    /**
     * Fully parse a separate sequence of tokens as a separate job.
     * Tokens should be specified in reverse order, as in a MacroDefinition.
     */
    subparse(_) {
      var H = this.nextToken;
      this.consume(), this.gullet.pushToken(new Token("}")), this.gullet.pushTokens(_);
      var V = this.parseExpression(!1);
      return this.expect("}"), this.nextToken = H, V;
    }
    /**
     * Parses an "expression", which is a list of atoms.
     *
     * `breakOnInfix`: Should the parsing stop when we hit infix nodes? This
     *                 happens when functions have higher precedence han infix
     *                 nodes in implicit parses.
     *
     * `breakOnTokenText`: The text of the token that the expression should end
     *                     with, or `null` if something else should end the
     *                     expression.
     */
    parseExpression(_, H) {
      for (var V = []; ; ) {
        this.mode === "math" && this.consumeSpaces();
        var q = this.fetch();
        if (Ky.endOfExpression.indexOf(q.text) !== -1 || H && q.text === H || _ && functions[q.text] && functions[q.text].infix)
          break;
        var G = this.parseAtom(H);
        if (G) {
          if (G.type === "internal")
            continue;
        } else break;
        V.push(G);
      }
      return this.mode === "text" && this.formLigatures(V), this.handleInfixNodes(V);
    }
    /**
     * Rewrites infix operators such as \over with corresponding commands such
     * as \frac.
     *
     * There can only be one infix operator per group.  If there's more than one
     * then the expression is ambiguous.  This can be resolved by adding {}.
     */
    handleInfixNodes(_) {
      for (var H = -1, V, q = 0; q < _.length; q++)
        if (_[q].type === "infix") {
          if (H !== -1)
            throw new ParseError("only one infix operator per group", _[q].token);
          H = q, V = _[q].replaceWith;
        }
      if (H !== -1 && V) {
        var G, J, Z = _.slice(0, H), te = _.slice(H + 1);
        Z.length === 1 && Z[0].type === "ordgroup" ? G = Z[0] : G = {
          type: "ordgroup",
          mode: this.mode,
          body: Z
        }, te.length === 1 && te[0].type === "ordgroup" ? J = te[0] : J = {
          type: "ordgroup",
          mode: this.mode,
          body: te
        };
        var ne;
        return V === "\\\\abovefrac" ? ne = this.callFunction(V, [G, _[H], J], []) : ne = this.callFunction(V, [G, J], []), [ne];
      } else
        return _;
    }
    /**
     * Handle a subscript or superscript with nice errors.
     */
    handleSupSubscript(_) {
      var H = this.fetch(), V = H.text;
      this.consume(), this.consumeSpaces();
      var q = this.parseGroup(_);
      if (!q)
        throw new ParseError("Expected group after '" + V + "'", H);
      return q;
    }
    /**
     * Converts the textual input of an unsupported command into a text node
     * contained within a color node whose color is determined by errorColor
     */
    formatUnsupportedCmd(_) {
      for (var H = [], V = 0; V < _.length; V++)
        H.push({
          type: "textord",
          mode: "text",
          text: _[V]
        });
      var q = {
        type: "text",
        mode: this.mode,
        body: H
      }, G = {
        type: "color",
        mode: this.mode,
        color: this.settings.errorColor,
        body: [q]
      };
      return G;
    }
    /**
     * Parses a group with optional super/subscripts.
     */
    parseAtom(_) {
      var H = this.parseGroup("atom", _);
      if (this.mode === "text")
        return H;
      for (var V, q; ; ) {
        this.consumeSpaces();
        var G = this.fetch();
        if (G.text === "\\limits" || G.text === "\\nolimits") {
          if (H && H.type === "op") {
            var J = G.text === "\\limits";
            H.limits = J, H.alwaysHandleSupSub = !0;
          } else if (H && H.type === "operatorname")
            H.alwaysHandleSupSub && (H.limits = G.text === "\\limits");
          else
            throw new ParseError("Limit controls must follow a math operator", G);
          this.consume();
        } else if (G.text === "^") {
          if (V)
            throw new ParseError("Double superscript", G);
          V = this.handleSupSubscript("superscript");
        } else if (G.text === "_") {
          if (q)
            throw new ParseError("Double subscript", G);
          q = this.handleSupSubscript("subscript");
        } else if (G.text === "'") {
          if (V)
            throw new ParseError("Double superscript", G);
          var Z = {
            type: "textord",
            mode: this.mode,
            text: "\\prime"
          }, te = [Z];
          for (this.consume(); this.fetch().text === "'"; )
            te.push(Z), this.consume();
          this.fetch().text === "^" && te.push(this.handleSupSubscript("superscript")), V = {
            type: "ordgroup",
            mode: this.mode,
            body: te
          };
        } else if (uSubsAndSups[G.text]) {
          var ne = unicodeSubRegEx.test(G.text), ee = [];
          for (ee.push(new Token(uSubsAndSups[G.text])), this.consume(); ; ) {
            var ae = this.fetch().text;
            if (!uSubsAndSups[ae] || unicodeSubRegEx.test(ae) !== ne)
              break;
            ee.unshift(new Token(uSubsAndSups[ae])), this.consume();
          }
          var oe = this.subparse(ee);
          ne ? q = {
            type: "ordgroup",
            mode: "math",
            body: oe
          } : V = {
            type: "ordgroup",
            mode: "math",
            body: oe
          };
        } else
          break;
      }
      return V || q ? {
        type: "supsub",
        mode: this.mode,
        base: H,
        sup: V,
        sub: q
      } : H;
    }
    /**
     * Parses an entire function, including its base and all of its arguments.
     */
    parseFunction(_, H) {
      var V = this.fetch(), q = V.text, G = functions[q];
      if (!G)
        return null;
      if (this.consume(), H && H !== "atom" && !G.allowedInArgument)
        throw new ParseError("Got function '" + q + "' with no arguments" + (H ? " as " + H : ""), V);
      if (this.mode === "text" && !G.allowedInText)
        throw new ParseError("Can't use function '" + q + "' in text mode", V);
      if (this.mode === "math" && G.allowedInMath === !1)
        throw new ParseError("Can't use function '" + q + "' in math mode", V);
      var {
        args: J,
        optArgs: Z
      } = this.parseArguments(q, G);
      return this.callFunction(q, J, Z, V, _);
    }
    /**
     * Call a function handler with a suitable context and arguments.
     */
    callFunction(_, H, V, q, G) {
      var J = {
        funcName: _,
        parser: this,
        token: q,
        breakOnTokenText: G
      }, Z = functions[_];
      if (Z && Z.handler)
        return Z.handler(J, H, V);
      throw new ParseError("No function handler for " + _);
    }
    /**
     * Parses the arguments of a function or environment
     */
    parseArguments(_, H) {
      var V = H.numArgs + H.numOptionalArgs;
      if (V === 0)
        return {
          args: [],
          optArgs: []
        };
      for (var q = [], G = [], J = 0; J < V; J++) {
        var Z = H.argTypes && H.argTypes[J], te = J < H.numOptionalArgs;
        (H.primitive && Z == null || // \sqrt expands into primitive if optional argument doesn't exist
        H.type === "sqrt" && J === 1 && G[0] == null) && (Z = "primitive");
        var ne = this.parseGroupOfType("argument to '" + _ + "'", Z, te);
        if (te)
          G.push(ne);
        else if (ne != null)
          q.push(ne);
        else
          throw new ParseError("Null argument, please report this as a bug");
      }
      return {
        args: q,
        optArgs: G
      };
    }
    /**
     * Parses a group when the mode is changing.
     */
    parseGroupOfType(_, H, V) {
      switch (H) {
        case "color":
          return this.parseColorGroup(V);
        case "size":
          return this.parseSizeGroup(V);
        case "url":
          return this.parseUrlGroup(V);
        case "math":
        case "text":
          return this.parseArgumentGroup(V, H);
        case "hbox": {
          var q = this.parseArgumentGroup(V, "text");
          return q != null ? {
            type: "styling",
            mode: q.mode,
            body: [q],
            style: "text"
            // simulate \textstyle
          } : null;
        }
        case "raw": {
          var G = this.parseStringGroup("raw", V);
          return G != null ? {
            type: "raw",
            mode: "text",
            string: G.text
          } : null;
        }
        case "primitive": {
          if (V)
            throw new ParseError("A primitive argument cannot be optional");
          var J = this.parseGroup(_);
          if (J == null)
            throw new ParseError("Expected group as " + _, this.fetch());
          return J;
        }
        case "original":
        case null:
        case void 0:
          return this.parseArgumentGroup(V);
        default:
          throw new ParseError("Unknown group type as " + _, this.fetch());
      }
    }
    /**
     * Discard any space tokens, fetching the next non-space token.
     */
    consumeSpaces() {
      for (; this.fetch().text === " "; )
        this.consume();
    }
    /**
     * Parses a group, essentially returning the string formed by the
     * brace-enclosed tokens plus some position information.
     */
    parseStringGroup(_, H) {
      var V = this.gullet.scanArgument(H);
      if (V == null)
        return null;
      for (var q = "", G; (G = this.fetch()).text !== "EOF"; )
        q += G.text, this.consume();
      return this.consume(), V.text = q, V;
    }
    /**
     * Parses a regex-delimited group: the largest sequence of tokens
     * whose concatenated strings match `regex`. Returns the string
     * formed by the tokens plus some position information.
     */
    parseRegexGroup(_, H) {
      for (var V = this.fetch(), q = V, G = "", J; (J = this.fetch()).text !== "EOF" && _.test(G + J.text); )
        q = J, G += q.text, this.consume();
      if (G === "")
        throw new ParseError("Invalid " + H + ": '" + V.text + "'", V);
      return V.range(q, G);
    }
    /**
     * Parses a color description.
     */
    parseColorGroup(_) {
      var H = this.parseStringGroup("color", _);
      if (H == null)
        return null;
      var V = /^(#[a-f0-9]{3}|#?[a-f0-9]{6}|[a-z]+)$/i.exec(H.text);
      if (!V)
        throw new ParseError("Invalid color: '" + H.text + "'", H);
      var q = V[0];
      return /^[0-9a-f]{6}$/i.test(q) && (q = "#" + q), {
        type: "color-token",
        mode: this.mode,
        color: q
      };
    }
    /**
     * Parses a size specification, consisting of magnitude and unit.
     */
    parseSizeGroup(_) {
      var H, V = !1;
      if (this.gullet.consumeSpaces(), !_ && this.gullet.future().text !== "{" ? H = this.parseRegexGroup(/^[-+]? *(?:$|\d+|\d+\.\d*|\.\d*) *[a-z]{0,2} *$/, "size") : H = this.parseStringGroup("size", _), !H)
        return null;
      !_ && H.text.length === 0 && (H.text = "0pt", V = !0);
      var q = /([-+]?) *(\d+(?:\.\d*)?|\.\d+) *([a-z]{2})/.exec(H.text);
      if (!q)
        throw new ParseError("Invalid size: '" + H.text + "'", H);
      var G = {
        number: +(q[1] + q[2]),
        // sign + magnitude, cast to number
        unit: q[3]
      };
      if (!validUnit(G))
        throw new ParseError("Invalid unit: '" + G.unit + "'", H);
      return {
        type: "size",
        mode: this.mode,
        value: G,
        isBlank: V
      };
    }
    /**
     * Parses an URL, checking escaped letters and allowed protocols,
     * and setting the catcode of % as an active character (as in \hyperref).
     */
    parseUrlGroup(_) {
      this.gullet.lexer.setCatcode("%", 13), this.gullet.lexer.setCatcode("~", 12);
      var H = this.parseStringGroup("url", _);
      if (this.gullet.lexer.setCatcode("%", 14), this.gullet.lexer.setCatcode("~", 13), H == null)
        return null;
      var V = H.text.replace(/\\([#$%&~_^{}])/g, "$1");
      return {
        type: "url",
        mode: this.mode,
        url: V
      };
    }
    /**
     * Parses an argument with the mode specified.
     */
    parseArgumentGroup(_, H) {
      var V = this.gullet.scanArgument(_);
      if (V == null)
        return null;
      var q = this.mode;
      H && this.switchMode(H), this.gullet.beginGroup();
      var G = this.parseExpression(!1, "EOF");
      this.expect("EOF"), this.gullet.endGroup();
      var J = {
        type: "ordgroup",
        mode: this.mode,
        loc: V.loc,
        body: G
      };
      return H && this.switchMode(q), J;
    }
    /**
     * Parses an ordinary group, which is either a single nucleus (like "x")
     * or an expression in braces (like "{x+y}") or an implicit group, a group
     * that starts at the current position, and ends right before a higher explicit
     * group ends, or at EOF.
     */
    parseGroup(_, H) {
      var V = this.fetch(), q = V.text, G;
      if (q === "{" || q === "\\begingroup") {
        this.consume();
        var J = q === "{" ? "}" : "\\endgroup";
        this.gullet.beginGroup();
        var Z = this.parseExpression(!1, J), te = this.fetch();
        this.expect(J), this.gullet.endGroup(), G = {
          type: "ordgroup",
          mode: this.mode,
          loc: SourceLocation.range(V, te),
          body: Z,
          // A group formed by \begingroup...\endgroup is a semi-simple group
          // which doesn't affect spacing in math mode, i.e., is transparent.
          // https://tex.stackexchange.com/questions/1930/when-should-one-
          // use-begingroup-instead-of-bgroup
          semisimple: q === "\\begingroup" || void 0
        };
      } else if (G = this.parseFunction(H, _) || this.parseSymbol(), G == null && q[0] === "\\" && !implicitCommands.hasOwnProperty(q)) {
        if (this.settings.throwOnError)
          throw new ParseError("Undefined control sequence: " + q, V);
        G = this.formatUnsupportedCmd(q), this.consume();
      }
      return G;
    }
    /**
     * Form ligature-like combinations of characters for text mode.
     * This includes inputs like "--", "---", "``" and "''".
     * The result will simply replace multiple textord nodes with a single
     * character in each value by a single textord node having multiple
     * characters in its value.  The representation is still ASCII source.
     * The group will be modified in place.
     */
    formLigatures(_) {
      for (var H = _.length - 1, V = 0; V < H; ++V) {
        var q = _[V], G = q.text;
        G === "-" && _[V + 1].text === "-" && (V + 1 < H && _[V + 2].text === "-" ? (_.splice(V, 3, {
          type: "textord",
          mode: "text",
          loc: SourceLocation.range(q, _[V + 2]),
          text: "---"
        }), H -= 2) : (_.splice(V, 2, {
          type: "textord",
          mode: "text",
          loc: SourceLocation.range(q, _[V + 1]),
          text: "--"
        }), H -= 1)), (G === "'" || G === "`") && _[V + 1].text === G && (_.splice(V, 2, {
          type: "textord",
          mode: "text",
          loc: SourceLocation.range(q, _[V + 1]),
          text: G + G
        }), H -= 1);
      }
    }
    /**
     * Parse a single symbol out of the string. Here, we handle single character
     * symbols and special functions like \verb.
     */
    parseSymbol() {
      var _ = this.fetch(), H = _.text;
      if (/^\\verb[^a-zA-Z]/.test(H)) {
        this.consume();
        var V = H.slice(5), q = V.charAt(0) === "*";
        if (q && (V = V.slice(1)), V.length < 2 || V.charAt(0) !== V.slice(-1))
          throw new ParseError(`\\verb assertion failed --
                    please report what input caused this bug`);
        return V = V.slice(1, -1), {
          type: "verb",
          mode: "text",
          body: V,
          star: q
        };
      }
      unicodeSymbols.hasOwnProperty(H[0]) && !symbols$3[this.mode][H[0]] && (this.settings.strict && this.mode === "math" && this.settings.reportNonstrict("unicodeTextInMathMode", 'Accented Unicode text character "' + H[0] + '" used in math mode', _), H = unicodeSymbols[H[0]] + H.slice(1));
      var G = combiningDiacriticalMarksEndRegex.exec(H);
      G && (H = H.substring(0, G.index), H === "i" ? H = "ı" : H === "j" && (H = "ȷ"));
      var J;
      if (symbols$3[this.mode][H]) {
        this.settings.strict && this.mode === "math" && extraLatin.indexOf(H) >= 0 && this.settings.reportNonstrict("unicodeTextInMathMode", 'Latin-1/Unicode text character "' + H[0] + '" used in math mode', _);
        var Z = symbols$3[this.mode][H].group, te = SourceLocation.range(_), ne;
        if (ATOMS.hasOwnProperty(Z)) {
          var ee = Z;
          ne = {
            type: "atom",
            mode: this.mode,
            family: ee,
            loc: te,
            text: H
          };
        } else
          ne = {
            type: Z,
            mode: this.mode,
            loc: te,
            text: H
          };
        J = ne;
      } else if (H.charCodeAt(0) >= 128)
        this.settings.strict && (supportedCodepoint(H.charCodeAt(0)) ? this.mode === "math" && this.settings.reportNonstrict("unicodeTextInMathMode", 'Unicode text character "' + H[0] + '" used in math mode', _) : this.settings.reportNonstrict("unknownSymbol", 'Unrecognized Unicode character "' + H[0] + '"' + (" (" + H.charCodeAt(0) + ")"), _)), J = {
          type: "textord",
          mode: "text",
          loc: SourceLocation.range(_),
          text: H
        };
      else
        return null;
      if (this.consume(), G)
        for (var ae = 0; ae < G[0].length; ae++) {
          var oe = G[0][ae];
          if (!unicodeAccents[oe])
            throw new ParseError("Unknown accent ' " + oe + "'", _);
          var se = unicodeAccents[oe][this.mode] || unicodeAccents[oe].text;
          if (!se)
            throw new ParseError("Accent " + oe + " unsupported in " + this.mode + " mode", _);
          J = {
            type: "accent",
            mode: this.mode,
            loc: SourceLocation.range(_),
            label: se,
            isStretchy: !1,
            isShifty: !0,
            // $FlowFixMe
            base: J
          };
        }
      return J;
    }
  };
  Parser$1.endOfExpression = ["}", "\\endgroup", "\\end", "\\right", "&"];
  var parseTree = function O(_, H) {
    if (!(typeof _ == "string" || _ instanceof String))
      throw new TypeError("KaTeX can only parse string typed expression");
    var V = new Parser$1(_, H);
    delete V.gullet.macros.current["\\df@tag"];
    var q = V.parse();
    if (delete V.gullet.macros.current["\\current@color"], delete V.gullet.macros.current["\\color"], V.gullet.macros.get("\\df@tag")) {
      if (!H.displayMode)
        throw new ParseError("\\tag works only in display equations");
      q = [{
        type: "tag",
        mode: "text",
        body: q,
        tag: V.subparse([new Token("\\df@tag")])
      }];
    }
    return q;
  }, render = function O(_, H, V) {
    H.textContent = "";
    var q = renderToDomTree(_, V).toNode();
    H.appendChild(q);
  };
  typeof document < "u" && document.compatMode !== "CSS1Compat" && (typeof console < "u" && console.warn("Warning: KaTeX doesn't work in quirks mode. Make sure your website has a suitable doctype."), render = function() {
    throw new ParseError("KaTeX doesn't work in quirks mode.");
  });
  var renderToString = function O(_, H) {
    var V = renderToDomTree(_, H).toMarkup();
    return V;
  }, generateParseTree = function O(_, H) {
    var V = new Settings(H);
    return parseTree(_, V);
  }, renderError = function O(_, H, V) {
    if (V.throwOnError || !(_ instanceof ParseError))
      throw _;
    var q = buildCommon.makeSpan(["katex-error"], [new SymbolNode(H)]);
    return q.setAttribute("title", _.toString()), q.setAttribute("style", "color:" + V.errorColor), q;
  }, renderToDomTree = function O(_, H) {
    var V = new Settings(H);
    try {
      var q = parseTree(_, V);
      return buildTree$1(q, _, V);
    } catch (G) {
      return renderError(G, _, V);
    }
  }, renderToHTMLTree = function O(_, H) {
    var V = new Settings(H);
    try {
      var q = parseTree(_, V);
      return buildHTMLTree(q, _, V);
    } catch (G) {
      return renderError(G, _, V);
    }
  }, katex = {
    /**
     * Current KaTeX version
     */
    version: "0.16.15",
    /**
     * Renders the given LaTeX into an HTML+MathML combination, and adds
     * it as a child to the specified DOM node.
     */
    render,
    /**
     * Renders the given LaTeX into an HTML+MathML combination string,
     * for sending to the client.
     */
    renderToString,
    /**
     * KaTeX error, usually during parsing.
     */
    ParseError,
    /**
     * The shema of Settings
     */
    SETTINGS_SCHEMA,
    /**
     * Parses the given LaTeX into KaTeX's internal parse tree structure,
     * without rendering to HTML or MathML.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __parse: generateParseTree,
    /**
     * Renders the given LaTeX into an HTML+MathML internal DOM tree
     * representation, without flattening that representation to a string.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __renderToDomTree: renderToDomTree,
    /**
     * Renders the given LaTeX into an HTML internal DOM tree representation,
     * without MathML and without flattening that representation to a string.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __renderToHTMLTree: renderToHTMLTree,
    /**
     * extends internal font metrics object with a new object
     * each key in the new object represents a font name
    */
    __setFontMetrics: setFontMetrics,
    /**
     * adds a new symbol to builtin symbols table
     */
    __defineSymbol: defineSymbol,
    /**
     * adds a new function to builtin function list,
     * which directly produce parse tree elements
     * and have their own html/mathml builders
     */
    __defineFunction: defineFunction,
    /**
     * adds a new macro to builtin macro list
     */
    __defineMacro: defineMacro,
    /**
     * Expose the dom tree node types, which can be useful for type checking nodes.
     *
     * NOTE: This method is not currently recommended for public use.
     * The internal tree representation is unstable and is very likely
     * to change. Use at your own risk.
     */
    __domTree: {
      Span: Span$1,
      Anchor,
      SymbolNode,
      SvgNode,
      PathNode,
      LineNode
    }
  };
  const _sfc_main$7 = /* @__PURE__ */ defineComponent({
    __name: "Formula",
    props: {
      content: {}
    },
    setup(O) {
      const _ = O, H = (q) => {
        try {
          katex.render(_.content, q);
        } catch (G) {
          console.error(`Error rendering formula: ${G}`);
        }
      }, V = (q) => {
        q && H(q);
      };
      return (q, G) => (openBlock(), createElementBlock(Fragment, null, [
        G[0] || (G[0] = createBaseVNode("link", {
          rel: "stylesheet",
          href: "https://cdn.jsdelivr.net/npm/katex@0.16.15/dist/katex.css",
          integrity: "sha256-4aMlusw7NpNRNuQ50hS7anjdz2PU+g2PLD0FRjz8BsA=",
          crossorigin: "anonymous"
        }, null, -1)),
        createBaseVNode("div", {
          "data-cy": "block-formula",
          className: "w-full overflow-y-hidden bg-white flex justify-center",
          ref: V
        })
      ], 64));
    }
  }), Formula$1 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    default: _sfc_main$7
  }, Symbol.toStringTag, { value: "Module" }));
  var plotly_min = { exports: {} }, hasRequiredPlotly_min;
  function requirePlotly_min() {
    return hasRequiredPlotly_min || (hasRequiredPlotly_min = 1, function(O, _) {
      /*! For license information please see plotly.min.js.LICENSE.txt */
      (function(H, V) {
        O.exports = V();
      })(self, function() {
        return function() {
          var H = { 6713: function(G, J, Z) {
            var te = Z(34809), ne = { "X,X div": 'direction:ltr;font-family:"Open Sans",verdana,arial,sans-serif;margin:0;padding:0;', "X input,X button": 'font-family:"Open Sans",verdana,arial,sans-serif;', "X input:focus,X button:focus": "outline:none;", "X a": "text-decoration:none;", "X a:hover": "text-decoration:none;", "X .crisp": "shape-rendering:crispEdges;", "X .user-select-none": "-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;-o-user-select:none;user-select:none;", "X svg": "overflow:hidden;", "X svg a": "fill:#447adb;", "X svg a:hover": "fill:#3c6dc5;", "X .main-svg": "position:absolute;top:0;left:0;pointer-events:none;", "X .main-svg .draglayer": "pointer-events:all;", "X .cursor-default": "cursor:default;", "X .cursor-pointer": "cursor:pointer;", "X .cursor-crosshair": "cursor:crosshair;", "X .cursor-move": "cursor:move;", "X .cursor-col-resize": "cursor:col-resize;", "X .cursor-row-resize": "cursor:row-resize;", "X .cursor-ns-resize": "cursor:ns-resize;", "X .cursor-ew-resize": "cursor:ew-resize;", "X .cursor-sw-resize": "cursor:sw-resize;", "X .cursor-s-resize": "cursor:s-resize;", "X .cursor-se-resize": "cursor:se-resize;", "X .cursor-w-resize": "cursor:w-resize;", "X .cursor-e-resize": "cursor:e-resize;", "X .cursor-nw-resize": "cursor:nw-resize;", "X .cursor-n-resize": "cursor:n-resize;", "X .cursor-ne-resize": "cursor:ne-resize;", "X .cursor-grab": "cursor:-webkit-grab;cursor:grab;", "X .modebar": "position:absolute;top:2px;right:2px;", "X .ease-bg": "-webkit-transition:background-color .3s ease 0s;-moz-transition:background-color .3s ease 0s;-ms-transition:background-color .3s ease 0s;-o-transition:background-color .3s ease 0s;transition:background-color .3s ease 0s;", "X .modebar--hover>:not(.watermark)": "opacity:0;-webkit-transition:opacity .3s ease 0s;-moz-transition:opacity .3s ease 0s;-ms-transition:opacity .3s ease 0s;-o-transition:opacity .3s ease 0s;transition:opacity .3s ease 0s;", "X:hover .modebar--hover .modebar-group": "opacity:1;", "X .modebar-group": "float:left;display:inline-block;box-sizing:border-box;padding-left:8px;position:relative;vertical-align:middle;white-space:nowrap;", "X .modebar-btn": "position:relative;font-size:16px;padding:3px 4px;height:22px;cursor:pointer;line-height:normal;box-sizing:border-box;", "X .modebar-btn svg": "position:relative;top:2px;", "X .modebar.vertical": "display:flex;flex-direction:column;flex-wrap:wrap;align-content:flex-end;max-height:100%;", "X .modebar.vertical svg": "top:-1px;", "X .modebar.vertical .modebar-group": "display:block;float:none;padding-left:0px;padding-bottom:8px;", "X .modebar.vertical .modebar-group .modebar-btn": "display:block;text-align:center;", "X [data-title]:before,X [data-title]:after": "position:absolute;-webkit-transform:translate3d(0, 0, 0);-moz-transform:translate3d(0, 0, 0);-ms-transform:translate3d(0, 0, 0);-o-transform:translate3d(0, 0, 0);transform:translate3d(0, 0, 0);display:none;opacity:0;z-index:1001;pointer-events:none;top:110%;right:50%;", "X [data-title]:hover:before,X [data-title]:hover:after": "display:block;opacity:1;", "X [data-title]:before": 'content:"";position:absolute;background:rgba(0,0,0,0);border:6px solid rgba(0,0,0,0);z-index:1002;margin-top:-12px;border-bottom-color:#69738a;margin-right:-6px;', "X [data-title]:after": "content:attr(data-title);background:#69738a;color:#fff;padding:8px 10px;font-size:12px;line-height:12px;white-space:nowrap;margin-right:-18px;border-radius:2px;", "X .vertical [data-title]:before,X .vertical [data-title]:after": "top:0%;right:200%;", "X .vertical [data-title]:before": "border:6px solid rgba(0,0,0,0);border-left-color:#69738a;margin-top:8px;margin-right:-30px;", Y: 'font-family:"Open Sans",verdana,arial,sans-serif;position:fixed;top:50px;right:20px;z-index:10000;font-size:10pt;max-width:180px;', "Y p": "margin:0;", "Y .notifier-note": "min-width:180px;max-width:250px;border:1px solid #fff;z-index:3000;margin:0;background-color:#8c97af;background-color:rgba(140,151,175,.9);color:#fff;padding:10px;overflow-wrap:break-word;word-wrap:break-word;-ms-hyphens:auto;-webkit-hyphens:auto;hyphens:auto;", "Y .notifier-close": "color:#fff;opacity:.8;float:right;padding:0 5px;background:none;border:none;font-size:20px;font-weight:bold;line-height:20px;", "Y .notifier-close:hover": "color:#444;text-decoration:none;cursor:pointer;" };
            for (var ee in ne) {
              var ae = ee.replace(/^,/, " ,").replace(/X/g, ".js-plotly-plot .plotly").replace(/Y/g, ".plotly-notifier");
              te.addStyleRule(ae, ne[ee]);
            }
          }, 14187: function(G, J, Z) {
            G.exports = Z(47908);
          }, 20273: function(G, J, Z) {
            G.exports = Z(58218);
          }, 6457: function(G, J, Z) {
            G.exports = Z(89362);
          }, 15849: function(G, J, Z) {
            G.exports = Z(53794);
          }, 38847: function(G, J, Z) {
            G.exports = Z(29698);
          }, 7659: function(G, J, Z) {
            G.exports = Z(51252);
          }, 60089: function(G, J, Z) {
            G.exports = Z(48050);
          }, 22084: function(G, J, Z) {
            G.exports = Z(58075);
          }, 35892: function(G, J, Z) {
            G.exports = Z(9419);
          }, 81204: function(G, J, Z) {
            G.exports = Z(28128);
          }, 55857: function(G, J, Z) {
            G.exports = Z(47050);
          }, 12862: function(G, J, Z) {
            G.exports = Z(91405);
          }, 97629: function(G, J, Z) {
            G.exports = Z(34406);
          }, 67549: function(G, J, Z) {
            G.exports = Z(17430);
          }, 2660: function(G, J, Z) {
            G.exports = Z(91995);
          }, 86071: function(G, J, Z) {
            G.exports = Z(81264);
          }, 66200: function(G, J, Z) {
            G.exports = Z(42849);
          }, 53446: function(G, J, Z) {
            G.exports = Z(52213);
          }, 86899: function(G, J, Z) {
            G.exports = Z(91132);
          }, 13430: function(G, J, Z) {
            G.exports = Z(50453);
          }, 21548: function(G, J, Z) {
            G.exports = Z(29251);
          }, 53939: function(G, J, Z) {
            G.exports = Z(72892);
          }, 1902: function(G, J, Z) {
            G.exports = Z(74461);
          }, 29096: function(G, J, Z) {
            G.exports = Z(66143);
          }, 23820: function(G, J, Z) {
            G.exports = Z(81955);
          }, 82017: function(G, J, Z) {
            G.exports = Z(36858);
          }, 113: function(G, J, Z) {
            G.exports = Z(92106);
          }, 20260: function(G, J, Z) {
            var te = Z(67549);
            te.register([Z(20273), Z(15849), Z(21548), Z(1902), Z(29096), Z(23820), Z(12862), Z(1639), Z(10067), Z(53446), Z(31014), Z(113), Z(78170), Z(8202), Z(92382), Z(82017), Z(86899), Z(54357), Z(66903), Z(90594), Z(71680), Z(7412), Z(55857), Z(784), Z(74221), Z(22084), Z(44001), Z(97281), Z(12345), Z(53939), Z(29117), Z(5410), Z(5057), Z(81204), Z(86071), Z(14226), Z(35892), Z(2660), Z(96599), Z(28573), Z(76832), Z(60089), Z(51469), Z(97629), Z(27700), Z(7659), Z(11780), Z(27195), Z(6457), Z(84639), Z(14187), Z(66200), Z(13430), Z(90590), Z(38847)]), G.exports = te;
          }, 28573: function(G, J, Z) {
            G.exports = Z(25638);
          }, 90594: function(G, J, Z) {
            G.exports = Z(75297);
          }, 7412: function(G, J, Z) {
            G.exports = Z(58859);
          }, 27700: function(G, J, Z) {
            G.exports = Z(12683);
          }, 5410: function(G, J, Z) {
            G.exports = Z(6305);
          }, 29117: function(G, J, Z) {
            G.exports = Z(83910);
          }, 78170: function(G, J, Z) {
            G.exports = Z(49913);
          }, 12345: function(G, J, Z) {
            G.exports = Z(15186);
          }, 96599: function(G, J, Z) {
            G.exports = Z(71760);
          }, 54357: function(G, J, Z) {
            G.exports = Z(17822);
          }, 51469: function(G, J, Z) {
            G.exports = Z(56534);
          }, 74221: function(G, J, Z) {
            G.exports = Z(18070);
          }, 44001: function(G, J, Z) {
            G.exports = Z(52378);
          }, 14226: function(G, J, Z) {
            G.exports = Z(30929);
          }, 5057: function(G, J, Z) {
            G.exports = Z(83866);
          }, 11780: function(G, J, Z) {
            G.exports = Z(66939);
          }, 27195: function(G, J, Z) {
            G.exports = Z(23748);
          }, 84639: function(G, J, Z) {
            G.exports = Z(73304);
          }, 1639: function(G, J, Z) {
            G.exports = Z(12864);
          }, 90590: function(G, J, Z) {
            G.exports = Z(99855);
          }, 97281: function(G, J, Z) {
            G.exports = Z(91450);
          }, 784: function(G, J, Z) {
            G.exports = Z(51943);
          }, 8202: function(G, J, Z) {
            G.exports = Z(80809);
          }, 66903: function(G, J, Z) {
            G.exports = Z(95984);
          }, 76832: function(G, J, Z) {
            G.exports = Z(51671);
          }, 92382: function(G, J, Z) {
            G.exports = Z(47181);
          }, 10067: function(G, J, Z) {
            G.exports = Z(37276);
          }, 71680: function(G, J, Z) {
            G.exports = Z(75703);
          }, 31014: function(G, J, Z) {
            G.exports = Z(38261);
          }, 11645: function(G) {
            G.exports = [{ path: "", backoff: 0 }, { path: "M-2.4,-3V3L0.6,0Z", backoff: 0.6 }, { path: "M-3.7,-2.5V2.5L1.3,0Z", backoff: 1.3 }, { path: "M-4.45,-3L-1.65,-0.2V0.2L-4.45,3L1.55,0Z", backoff: 1.55 }, { path: "M-2.2,-2.2L-0.2,-0.2V0.2L-2.2,2.2L-1.4,3L1.6,0L-1.4,-3Z", backoff: 1.6 }, { path: "M-4.4,-2.1L-0.6,-0.2V0.2L-4.4,2.1L-4,3L2,0L-4,-3Z", backoff: 2 }, { path: "M2,0A2,2 0 1,1 0,-2A2,2 0 0,1 2,0Z", backoff: 0, noRotate: !0 }, { path: "M2,2V-2H-2V2Z", backoff: 0, noRotate: !0 }];
          }, 50222: function(G, J, Z) {
            var te = Z(11645), ne = Z(80337), ee = Z(54826), ae = Z(78032).templatedArray;
            Z(35081), G.exports = ae("annotation", { visible: { valType: "boolean", dflt: !0, editType: "calc+arraydraw" }, text: { valType: "string", editType: "calc+arraydraw" }, textangle: { valType: "angle", dflt: 0, editType: "calc+arraydraw" }, font: ne({ editType: "calc+arraydraw", colorEditType: "arraydraw" }), width: { valType: "number", min: 1, dflt: null, editType: "calc+arraydraw" }, height: { valType: "number", min: 1, dflt: null, editType: "calc+arraydraw" }, opacity: { valType: "number", min: 0, max: 1, dflt: 1, editType: "arraydraw" }, align: { valType: "enumerated", values: ["left", "center", "right"], dflt: "center", editType: "arraydraw" }, valign: { valType: "enumerated", values: ["top", "middle", "bottom"], dflt: "middle", editType: "arraydraw" }, bgcolor: { valType: "color", dflt: "rgba(0,0,0,0)", editType: "arraydraw" }, bordercolor: { valType: "color", dflt: "rgba(0,0,0,0)", editType: "arraydraw" }, borderpad: { valType: "number", min: 0, dflt: 1, editType: "calc+arraydraw" }, borderwidth: { valType: "number", min: 0, dflt: 1, editType: "calc+arraydraw" }, showarrow: { valType: "boolean", dflt: !0, editType: "calc+arraydraw" }, arrowcolor: { valType: "color", editType: "arraydraw" }, arrowhead: { valType: "integer", min: 0, max: te.length, dflt: 1, editType: "arraydraw" }, startarrowhead: { valType: "integer", min: 0, max: te.length, dflt: 1, editType: "arraydraw" }, arrowside: { valType: "flaglist", flags: ["end", "start"], extras: ["none"], dflt: "end", editType: "arraydraw" }, arrowsize: { valType: "number", min: 0.3, dflt: 1, editType: "calc+arraydraw" }, startarrowsize: { valType: "number", min: 0.3, dflt: 1, editType: "calc+arraydraw" }, arrowwidth: { valType: "number", min: 0.1, editType: "calc+arraydraw" }, standoff: { valType: "number", min: 0, dflt: 0, editType: "calc+arraydraw" }, startstandoff: { valType: "number", min: 0, dflt: 0, editType: "calc+arraydraw" }, ax: { valType: "any", editType: "calc+arraydraw" }, ay: { valType: "any", editType: "calc+arraydraw" }, axref: { valType: "enumerated", dflt: "pixel", values: ["pixel", ee.idRegex.x.toString()], editType: "calc" }, ayref: { valType: "enumerated", dflt: "pixel", values: ["pixel", ee.idRegex.y.toString()], editType: "calc" }, xref: { valType: "enumerated", values: ["paper", ee.idRegex.x.toString()], editType: "calc" }, x: { valType: "any", editType: "calc+arraydraw" }, xanchor: { valType: "enumerated", values: ["auto", "left", "center", "right"], dflt: "auto", editType: "calc+arraydraw" }, xshift: { valType: "number", dflt: 0, editType: "calc+arraydraw" }, yref: { valType: "enumerated", values: ["paper", ee.idRegex.y.toString()], editType: "calc" }, y: { valType: "any", editType: "calc+arraydraw" }, yanchor: { valType: "enumerated", values: ["auto", "top", "middle", "bottom"], dflt: "auto", editType: "calc+arraydraw" }, yshift: { valType: "number", dflt: 0, editType: "calc+arraydraw" }, clicktoshow: { valType: "enumerated", values: [!1, "onoff", "onout"], dflt: !1, editType: "arraydraw" }, xclick: { valType: "any", editType: "arraydraw" }, yclick: { valType: "any", editType: "arraydraw" }, hovertext: { valType: "string", editType: "arraydraw" }, hoverlabel: { bgcolor: { valType: "color", editType: "arraydraw" }, bordercolor: { valType: "color", editType: "arraydraw" }, font: ne({ editType: "arraydraw" }), editType: "arraydraw" }, captureevents: { valType: "boolean", editType: "arraydraw" }, editType: "calc", _deprecated: { ref: { valType: "string", editType: "calc" } } });
          }, 60317: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(3377).draw;
            function ae(se) {
              var ce = se._fullLayout;
              te.filterVisible(ce.annotations).forEach(function(le) {
                var ue = ne.getFromId(se, le.xref), de = ne.getFromId(se, le.yref), fe = ne.getRefType(le.xref), he = ne.getRefType(le.yref);
                le._extremes = {}, fe === "range" && oe(le, ue), he === "range" && oe(le, de);
              });
            }
            function oe(se, ce) {
              var le, ue = ce._id, de = ue.charAt(0), fe = se[de], he = se["a" + de], me = se[de + "ref"], pe = se["a" + de + "ref"], ye = se["_" + de + "padplus"], ge = se["_" + de + "padminus"], ve = { x: 1, y: -1 }[de] * se[de + "shift"], be = 3 * se.arrowsize * se.arrowwidth || 0, Ce = be + ve, Se = be - ve, we = 3 * se.startarrowsize * se.arrowwidth || 0, Ee = we + ve, xe = we - ve;
              if (pe === me) {
                var Le = ne.findExtremes(ce, [ce.r2c(fe)], { ppadplus: Ce, ppadminus: Se }), Qe = ne.findExtremes(ce, [ce.r2c(he)], { ppadplus: Math.max(ye, Ee), ppadminus: Math.max(ge, xe) });
                le = { min: [Le.min[0], Qe.min[0]], max: [Le.max[0], Qe.max[0]] };
              } else Ee = he ? Ee + he : Ee, xe = he ? xe - he : xe, le = ne.findExtremes(ce, [ce.r2c(fe)], { ppadplus: Math.max(ye, Ce, Ee), ppadminus: Math.max(ge, Se, xe) });
              se._extremes[ue] = le;
            }
            G.exports = function(se) {
              var ce = se._fullLayout;
              if (te.filterVisible(ce.annotations).length && se._fullData.length) return te.syncOrAsync([ee, ae], se);
            };
          }, 6035: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626), ee = Z(78032).arrayEditor;
            function ae(se, ce) {
              var le, ue, de, fe, he, me, pe, ye = se._fullLayout.annotations, ge = [], ve = [], be = [], Ce = (ce || []).length;
              for (le = 0; le < ye.length; le++) if (fe = (de = ye[le]).clicktoshow) {
                for (ue = 0; ue < Ce; ue++) if (me = (he = ce[ue]).xaxis, pe = he.yaxis, me._id === de.xref && pe._id === de.yref && me.d2r(he.x) === oe(de._xclick, me) && pe.d2r(he.y) === oe(de._yclick, pe)) {
                  (de.visible ? fe === "onout" ? ve : be : ge).push(le);
                  break;
                }
                ue === Ce && de.visible && fe === "onout" && ve.push(le);
              }
              return { on: ge, off: ve, explicitOff: be };
            }
            function oe(se, ce) {
              return ce.type === "log" ? ce.l2r(se) : ce.d2r(se);
            }
            G.exports = { hasClickToShow: function(se, ce) {
              var le = ae(se, ce);
              return le.on.length > 0 || le.explicitOff.length > 0;
            }, onClick: function(se, ce) {
              var le, ue, de = ae(se, ce), fe = de.on, he = de.off.concat(de.explicitOff), me = {}, pe = se._fullLayout.annotations;
              if (fe.length || he.length) {
                for (le = 0; le < fe.length; le++) (ue = ee(se.layout, "annotations", pe[fe[le]])).modifyItem("visible", !0), te.extendFlat(me, ue.getUpdateObj());
                for (le = 0; le < he.length; le++) (ue = ee(se.layout, "annotations", pe[he[le]])).modifyItem("visible", !1), te.extendFlat(me, ue.getUpdateObj());
                return ne.call("update", se, {}, me);
              }
            } };
          }, 53271: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766);
            G.exports = function(ee, ae, oe, se) {
              se("opacity");
              var ce = se("bgcolor"), le = se("bordercolor"), ue = ne.opacity(le);
              se("borderpad");
              var de = se("borderwidth"), fe = se("showarrow");
              if (se("text", fe ? " " : oe._dfltTitle.annotation), se("textangle"), te.coerceFont(se, "font", oe.font), se("width"), se("align"), se("height") && se("valign"), fe) {
                var he, me, pe = se("arrowside");
                pe.indexOf("end") !== -1 && (he = se("arrowhead"), me = se("arrowsize")), pe.indexOf("start") !== -1 && (se("startarrowhead", he), se("startarrowsize", me)), se("arrowcolor", ue ? ae.bordercolor : ne.defaultLine), se("arrowwidth", 2 * (ue && de || 1)), se("standoff"), se("startstandoff");
              }
              var ye = se("hovertext"), ge = oe.hoverlabel || {};
              if (ye) {
                var ve = se("hoverlabel.bgcolor", ge.bgcolor || (ne.opacity(ce) ? ne.rgb(ce) : ne.defaultLine)), be = se("hoverlabel.bordercolor", ge.bordercolor || ne.contrast(ve)), Ce = te.extendFlat({}, ge.font);
                Ce.color || (Ce.color = be), te.coerceFont(se, "hoverlabel.font", Ce);
              }
              se("captureevents", !!ye);
            };
          }, 59741: function(G, J, Z) {
            var te = Z(10721), ne = Z(8083);
            G.exports = function(ee, ae, oe, se) {
              ae = ae || {};
              var ce = oe === "log" && ae.type === "linear", le = oe === "linear" && ae.type === "log";
              if (ce || le) for (var ue, de, fe = ee._fullLayout.annotations, he = ae._id.charAt(0), me = 0; me < fe.length; me++) ue = fe[me], de = "annotations[" + me + "].", ue[he + "ref"] === ae._id && pe(he), ue["a" + he + "ref"] === ae._id && pe("a" + he);
              function pe(ye) {
                var ge = ue[ye], ve = null;
                ve = ce ? ne(ge, ae.range) : Math.pow(10, ge), te(ve) || (ve = null), se(de + ye, ve);
              }
            };
          }, 63737: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(59008), ae = Z(53271), oe = Z(50222);
            function se(ce, le, ue) {
              function de(Qe, ke) {
                return te.coerce(ce, le, oe, Qe, ke);
              }
              var fe = de("visible"), he = de("clicktoshow");
              if (fe || he) {
                ae(ce, le, ue, de);
                for (var me = le.showarrow, pe = ["x", "y"], ye = [-10, -30], ge = { _fullLayout: ue }, ve = 0; ve < 2; ve++) {
                  var be = pe[ve], Ce = ne.coerceRef(ce, le, ge, be, "", "paper");
                  if (Ce !== "paper" && ne.getFromId(ge, Ce)._annIndices.push(le._index), ne.coercePosition(le, ge, de, Ce, be, 0.5), me) {
                    var Se = "a" + be, we = ne.coerceRef(ce, le, ge, Se, "pixel", ["pixel", "paper"]);
                    we !== "pixel" && we !== Ce && (we = le[Se] = "pixel");
                    var Ee = we === "pixel" ? ye[ve] : 0.4;
                    ne.coercePosition(le, ge, de, we, Se, Ee);
                  }
                  de(be + "anchor"), de(be + "shift");
                }
                if (te.noneOrAll(ce, le, ["x", "y"]), me && te.noneOrAll(ce, le, ["ax", "ay"]), he) {
                  var xe = de("xclick"), Le = de("yclick");
                  le._xclick = xe === void 0 ? le.x : ne.cleanPosition(xe, ge, le.xref), le._yclick = Le === void 0 ? le.y : ne.cleanPosition(Le, ge, le.yref);
                }
              }
            }
            G.exports = function(ce, le) {
              ee(ce, le, { name: "annotations", handleItemDefaults: se });
            };
          }, 3377: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(44122), ae = Z(34809), oe = ae.strTranslate, se = Z(29714), ce = Z(78766), le = Z(62203), ue = Z(32141), de = Z(30635), fe = Z(27983), he = Z(14751), me = Z(78032).arrayEditor, pe = Z(23768);
            function ye(be, Ce) {
              var Se = be._fullLayout.annotations[Ce] || {}, we = se.getFromId(be, Se.xref), Ee = se.getFromId(be, Se.yref);
              we && we.setScale(), Ee && Ee.setScale(), ve(be, Se, Ce, !1, we, Ee);
            }
            function ge(be, Ce, Se, we, Ee) {
              var xe = Ee[Se], Le = Ee[Se + "ref"], Qe = Se.indexOf("y") !== -1, ke = se.getRefType(Le) === "domain", Me = Qe ? we.h : we.w;
              return be ? ke ? xe + (Qe ? -Ce : Ce) / be._length : be.p2r(be.r2p(xe) + Ce) : xe + (Qe ? -Ce : Ce) / Me;
            }
            function ve(be, Ce, Se, we, Ee, xe) {
              var Le, Qe, ke = be._fullLayout, Me = be._fullLayout._size, Re = be._context.edits;
              we ? (Le = "annotation-" + we, Qe = we + ".annotations") : (Le = "annotation", Qe = "annotations");
              var De = me(be.layout, Qe, Ce), Pe = De.modifyBase, $e = De.modifyItem, qe = De.getUpdateObj;
              ke._infolayer.selectAll("." + Le + '[data-index="' + Se + '"]').remove();
              var Xe = "clip" + ke._uid + "_ann" + Se;
              if (Ce._input && Ce.visible !== !1) {
                var Ve = { x: {}, y: {} }, Ze = +Ce.textangle || 0, Ge = ke._infolayer.append("g").classed(Le, !0).attr("data-index", String(Se)).style("opacity", Ce.opacity), ze = Ge.append("g").classed("annotation-text-g", !0), We = Re[Ce.showarrow ? "annotationTail" : "annotationPosition"], at = Ce.captureevents || Re.annotationText || We, ct = ze.append("g").style("pointer-events", at ? "all" : null).call(fe, "pointer").on("click", function() {
                  be._dragging = !1, be.emit("plotly_clickannotation", Lt(te.event));
                });
                Ce.hovertext && ct.on("mouseover", function() {
                  var $t = Ce.hoverlabel, Pt = $t.font, Xt = this.getBoundingClientRect(), Nt = be.getBoundingClientRect();
                  ue.loneHover({ x0: Xt.left - Nt.left, x1: Xt.right - Nt.left, y: (Xt.top + Xt.bottom) / 2 - Nt.top, text: Ce.hovertext, color: $t.bgcolor, borderColor: $t.bordercolor, fontFamily: Pt.family, fontSize: Pt.size, fontColor: Pt.color, fontWeight: Pt.weight, fontStyle: Pt.style, fontVariant: Pt.variant, fontShadow: Pt.fontShadow, fontLineposition: Pt.fontLineposition, fontTextcase: Pt.fontTextcase }, { container: ke._hoverlayer.node(), outerContainer: ke._paper.node(), gd: be });
                }).on("mouseout", function() {
                  ue.loneUnhover(ke._hoverlayer.node());
                });
                var et = Ce.borderwidth, dt = Ce.borderpad, Je = et + dt, it = ct.append("rect").attr("class", "bg").style("stroke-width", et + "px").call(ce.stroke, Ce.bordercolor).call(ce.fill, Ce.bgcolor), st = Ce.width || Ce.height, gt = ke._topclips.selectAll("#" + Xe).data(st ? [0] : []);
                gt.enter().append("clipPath").classed("annclip", !0).attr("id", Xe).append("rect"), gt.exit().remove();
                var bt = Ce.font, yt = ke._meta ? ae.templateString(Ce.text, ke._meta) : Ce.text, St = ct.append("text").classed("annotation-text", !0).text(yt);
                Re.annotationText ? St.call(de.makeEditable, { delegate: ct, gd: be }).call(Rt).on("edit", function($t) {
                  Ce.text = $t, this.call(Rt), $e("text", $t), Ee && Ee.autorange && Pe(Ee._name + ".autorange", !0), xe && xe.autorange && Pe(xe._name + ".autorange", !0), ne.call("_guiRelayout", be, qe());
                }) : St.call(Rt);
              } else te.selectAll("#" + Xe).remove();
              function Lt($t) {
                var Pt = { index: Se, annotation: Ce._input, fullAnnotation: Ce, event: $t };
                return we && (Pt.subplotId = we), Pt;
              }
              function Rt($t) {
                return $t.call(le.font, bt).attr({ "text-anchor": { left: "start", right: "end" }[Ce.align] || "middle" }), de.convertToTspans($t, be, vt), $t;
              }
              function vt() {
                var $t = St.selectAll("a");
                $t.size() === 1 && $t.text() === St.text() && ct.insert("a", ":first-child").attr({ "xlink:xlink:href": $t.attr("xlink:href"), "xlink:xlink:show": $t.attr("xlink:show") }).style({ cursor: "pointer" }).node().appendChild(it.node());
                var Pt = ct.select(".annotation-text-math-group"), Xt = !Pt.empty(), Nt = le.bBox((Xt ? Pt : St).node()), Gt = Nt.width, Ft = Nt.height, Tt = Ce.width || Gt, It = Ce.height || Ft, Ct = Math.round(Tt + 2 * Je), Kt = Math.round(It + 2 * Je);
                function Vt(In, kn) {
                  return kn === "auto" && (kn = In < 0.3333333333333333 ? "left" : In > 0.6666666666666666 ? "right" : "center"), { center: 0, middle: 0, left: 0.5, bottom: -0.5, right: -0.5, top: 0.5 }[kn];
                }
                for (var Yt = !1, sn = ["x", "y"], hn = 0; hn < sn.length; hn++) {
                  var Qt, Ut, Wt, rn, vn, dn = sn[hn], Ht = Ce[dn + "ref"] || dn, jt = Ce["a" + dn + "ref"], cn = { x: Ee, y: xe }[dn], wn = (Ze + (dn === "x" ? 0 : -90)) * Math.PI / 180, Rn = Ct * Math.cos(wn), Pn = Kt * Math.sin(wn), Ie = Math.abs(Rn) + Math.abs(Pn), rt = Ce[dn + "anchor"], ft = Ce[dn + "shift"] * (dn === "x" ? 1 : -1), pt = Ve[dn], wt = se.getRefType(Ht);
                  if (cn && wt !== "domain") {
                    var Bt = cn.r2fraction(Ce[dn]);
                    (Bt < 0 || Bt > 1) && (jt === Ht ? ((Bt = cn.r2fraction(Ce["a" + dn])) < 0 || Bt > 1) && (Yt = !0) : Yt = !0), Qt = cn._offset + cn.r2p(Ce[dn]), rn = 0.5;
                  } else {
                    var At = wt === "domain";
                    dn === "x" ? (Wt = Ce[dn], Qt = At ? cn._offset + cn._length * Wt : Qt = Me.l + Me.w * Wt) : (Wt = 1 - Ce[dn], Qt = At ? cn._offset + cn._length * Wt : Qt = Me.t + Me.h * Wt), rn = Ce.showarrow ? 0.5 : Wt;
                  }
                  if (Ce.showarrow) {
                    pt.head = Qt;
                    var Dt = Ce["a" + dn];
                    if (vn = Rn * Vt(0.5, Ce.xanchor) - Pn * Vt(0.5, Ce.yanchor), jt === Ht) {
                      var Ot = se.getRefType(jt);
                      Ot === "domain" ? (dn === "y" && (Dt = 1 - Dt), pt.tail = cn._offset + cn._length * Dt) : Ot === "paper" ? dn === "y" ? (Dt = 1 - Dt, pt.tail = Me.t + Me.h * Dt) : pt.tail = Me.l + Me.w * Dt : pt.tail = cn._offset + cn.r2p(Dt), Ut = vn;
                    } else pt.tail = Qt + Dt, Ut = vn + Dt;
                    pt.text = pt.tail + vn;
                    var un = ke[dn === "x" ? "width" : "height"];
                    if (Ht === "paper" && (pt.head = ae.constrain(pt.head, 1, un - 1)), jt === "pixel") {
                      var yn = -Math.max(pt.tail - 3, pt.text), _n = Math.min(pt.tail + 3, pt.text) - un;
                      yn > 0 ? (pt.tail += yn, pt.text += yn) : _n > 0 && (pt.tail -= _n, pt.text -= _n);
                    }
                    pt.tail += ft, pt.head += ft;
                  } else Ut = vn = Ie * Vt(rn, rt), pt.text = Qt + vn;
                  pt.text += ft, vn += ft, Ut += ft, Ce["_" + dn + "padplus"] = Ie / 2 + Ut, Ce["_" + dn + "padminus"] = Ie / 2 - Ut, Ce["_" + dn + "size"] = Ie, Ce["_" + dn + "shift"] = vn;
                }
                if (Yt) ct.remove();
                else {
                  var gn = 0, bn = 0;
                  if (Ce.align !== "left" && (gn = (Tt - Gt) * (Ce.align === "center" ? 0.5 : 1)), Ce.valign !== "top" && (bn = (It - Ft) * (Ce.valign === "middle" ? 0.5 : 1)), Xt) Pt.select("svg").attr({ x: Je + gn - 1, y: Je + bn }).call(le.setClipUrl, st ? Xe : null, be);
                  else {
                    var an = Je + bn - Nt.top, Zt = Je + gn - Nt.left;
                    St.call(de.positionText, Zt, an).call(le.setClipUrl, st ? Xe : null, be);
                  }
                  gt.select("rect").call(le.setRect, Je, Je, Tt, It), it.call(le.setRect, et / 2, et / 2, Ct - et, Kt - et), ct.call(le.setTranslate, Math.round(Ve.x.text - Ct / 2), Math.round(Ve.y.text - Kt / 2)), ze.attr({ transform: "rotate(" + Ze + "," + Ve.x.text + "," + Ve.y.text + ")" });
                  var Mn, Ln = function(In, kn) {
                    Ge.selectAll(".annotation-arrow-g").remove();
                    var er = Ve.x.head, jn = Ve.y.head, ur = Ve.x.tail + In, Kn = Ve.y.tail + kn, Jn = Ve.x.text + In, Lr = Ve.y.text + kn, mr = ae.rotationXYMatrix(Ze, Jn, Lr), Ar = ae.apply2DTransform(mr), Yr = ae.apply2DTransform2(mr), jr = +it.attr("width"), Mr = +it.attr("height"), Wr = Jn - 0.5 * jr, Bi = Wr + jr, gi = Lr - 0.5 * Mr, Zr = gi + Mr, fi = [[Wr, gi, Wr, Zr], [Wr, Zr, Bi, Zr], [Bi, Zr, Bi, gi], [Bi, gi, Wr, gi]].map(Yr);
                    if (!fi.reduce(function(hi, Gr) {
                      return hi ^ !!ae.segmentsIntersect(er, jn, er + 1e6, jn + 1e6, Gr[0], Gr[1], Gr[2], Gr[3]);
                    }, !1)) {
                      fi.forEach(function(hi) {
                        var Gr = ae.segmentsIntersect(ur, Kn, er, jn, hi[0], hi[1], hi[2], hi[3]);
                        Gr && (ur = Gr.x, Kn = Gr.y);
                      });
                      var ri = Ce.arrowwidth, bi = Ce.arrowcolor, Zi = Ce.arrowside, $i = Ge.append("g").style({ opacity: ce.opacity(bi) }).classed("annotation-arrow-g", !0), Br = $i.append("path").attr("d", "M" + ur + "," + Kn + "L" + er + "," + jn).style("stroke-width", ri + "px").call(ce.stroke, ce.rgb(bi));
                      if (pe(Br, Zi, Ce), Re.annotationPosition && Br.node().parentNode && !we) {
                        var Pr = er, ei = jn;
                        if (Ce.standoff) {
                          var Ci = Math.sqrt(Math.pow(er - ur, 2) + Math.pow(jn - Kn, 2));
                          Pr += Ce.standoff * (ur - er) / Ci, ei += Ce.standoff * (Kn - jn) / Ci;
                        }
                        var oa, ka, aa = $i.append("path").classed("annotation-arrow", !0).classed("anndrag", !0).classed("cursor-move", !0).attr({ d: "M3,3H-3V-3H3ZM0,0L" + (ur - Pr) + "," + (Kn - ei), transform: oe(Pr, ei) }).style("stroke-width", ri + 6 + "px").call(ce.stroke, "rgba(0,0,0,0)").call(ce.fill, "rgba(0,0,0,0)");
                        he.init({ element: aa.node(), gd: be, prepFn: function() {
                          var hi = le.getTranslate(ct);
                          oa = hi.x, ka = hi.y, Ee && Ee.autorange && Pe(Ee._name + ".autorange", !0), xe && xe.autorange && Pe(xe._name + ".autorange", !0);
                        }, moveFn: function(hi, Gr) {
                          var Ca = Ar(oa, ka), Sr = Ca[0] + hi, Dr = Ca[1] + Gr;
                          ct.call(le.setTranslate, Sr, Dr), $e("x", ge(Ee, hi, "x", Me, Ce)), $e("y", ge(xe, Gr, "y", Me, Ce)), Ce.axref === Ce.xref && $e("ax", ge(Ee, hi, "ax", Me, Ce)), Ce.ayref === Ce.yref && $e("ay", ge(xe, Gr, "ay", Me, Ce)), $i.attr("transform", oe(hi, Gr)), ze.attr({ transform: "rotate(" + Ze + "," + Sr + "," + Dr + ")" });
                        }, doneFn: function() {
                          ne.call("_guiRelayout", be, qe());
                          var hi = document.querySelector(".js-notes-box-panel");
                          hi && hi.redraw(hi.selectedObj);
                        } });
                      }
                    }
                  };
                  Ce.showarrow && Ln(0, 0), We && he.init({ element: ct.node(), gd: be, prepFn: function() {
                    Mn = ze.attr("transform");
                  }, moveFn: function(In, kn) {
                    var er = "pointer";
                    if (Ce.showarrow) Ce.axref === Ce.xref ? $e("ax", ge(Ee, In, "ax", Me, Ce)) : $e("ax", Ce.ax + In), Ce.ayref === Ce.yref ? $e("ay", ge(xe, kn, "ay", Me.w, Ce)) : $e("ay", Ce.ay + kn), Ln(In, kn);
                    else {
                      if (we) return;
                      var jn, ur;
                      if (Ee) jn = ge(Ee, In, "x", Me, Ce);
                      else {
                        var Kn = Ce._xsize / Me.w, Jn = Ce.x + (Ce._xshift - Ce.xshift) / Me.w - Kn / 2;
                        jn = he.align(Jn + In / Me.w, Kn, 0, 1, Ce.xanchor);
                      }
                      if (xe) ur = ge(xe, kn, "y", Me, Ce);
                      else {
                        var Lr = Ce._ysize / Me.h, mr = Ce.y - (Ce._yshift + Ce.yshift) / Me.h - Lr / 2;
                        ur = he.align(mr - kn / Me.h, Lr, 0, 1, Ce.yanchor);
                      }
                      $e("x", jn), $e("y", ur), Ee && xe || (er = he.getCursor(Ee ? 0.5 : jn, xe ? 0.5 : ur, Ce.xanchor, Ce.yanchor));
                    }
                    ze.attr({ transform: oe(In, kn) + Mn }), fe(ct, er);
                  }, clickFn: function(In, kn) {
                    Ce.captureevents && be.emit("plotly_clickannotation", Lt(kn));
                  }, doneFn: function() {
                    fe(ct), ne.call("_guiRelayout", be, qe());
                    var In = document.querySelector(".js-notes-box-panel");
                    In && In.redraw(In.selectedObj);
                  } });
                }
              }
            }
            G.exports = { draw: function(be) {
              var Ce = be._fullLayout;
              Ce._infolayer.selectAll(".annotation").remove();
              for (var Se = 0; Se < Ce.annotations.length; Se++) Ce.annotations[Se].visible && ye(be, Se);
              return ee.previousPromises(be);
            }, drawOne: ye, drawRaw: ve };
          }, 23768: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(11645), ae = Z(34809), oe = ae.strScale, se = ae.strRotate, ce = ae.strTranslate;
            G.exports = function(le, ue, de) {
              var fe, he, me, pe, ye = le.node(), ge = ee[de.arrowhead || 0], ve = ee[de.startarrowhead || 0], be = (de.arrowwidth || 1) * (de.arrowsize || 1), Ce = (de.arrowwidth || 1) * (de.startarrowsize || 1), Se = ue.indexOf("start") >= 0, we = ue.indexOf("end") >= 0, Ee = ge.backoff * be + de.standoff, xe = ve.backoff * Ce + de.startstandoff;
              if (ye.nodeName === "line") {
                fe = { x: +le.attr("x1"), y: +le.attr("y1") }, he = { x: +le.attr("x2"), y: +le.attr("y2") };
                var Le = fe.x - he.x, Qe = fe.y - he.y;
                if (pe = (me = Math.atan2(Qe, Le)) + Math.PI, Ee && xe && Ee + xe > Math.sqrt(Le * Le + Qe * Qe)) return void Ge();
                if (Ee) {
                  if (Ee * Ee > Le * Le + Qe * Qe) return void Ge();
                  var ke = Ee * Math.cos(me), Me = Ee * Math.sin(me);
                  he.x += ke, he.y += Me, le.attr({ x2: he.x, y2: he.y });
                }
                if (xe) {
                  if (xe * xe > Le * Le + Qe * Qe) return void Ge();
                  var Re = xe * Math.cos(me), De = xe * Math.sin(me);
                  fe.x -= Re, fe.y -= De, le.attr({ x1: fe.x, y1: fe.y });
                }
              } else if (ye.nodeName === "path") {
                var Pe = ye.getTotalLength(), $e = "";
                if (Pe < Ee + xe) return void Ge();
                var qe = ye.getPointAtLength(0), Xe = ye.getPointAtLength(0.1);
                me = Math.atan2(qe.y - Xe.y, qe.x - Xe.x), fe = ye.getPointAtLength(Math.min(xe, Pe)), $e = "0px," + xe + "px,";
                var Ve = ye.getPointAtLength(Pe), Ze = ye.getPointAtLength(Pe - 0.1);
                pe = Math.atan2(Ve.y - Ze.y, Ve.x - Ze.x), he = ye.getPointAtLength(Math.max(0, Pe - Ee)), $e += Pe - ($e ? xe + Ee : Ee) + "px," + Pe + "px", le.style("stroke-dasharray", $e);
              }
              function Ge() {
                le.style("stroke-dasharray", "0px,100px");
              }
              function ze(We, at, ct, et) {
                We.path && (We.noRotate && (ct = 0), te.select(ye.parentNode).append("path").attr({ class: le.attr("class"), d: We.path, transform: ce(at.x, at.y) + se(180 * ct / Math.PI) + oe(et) }).style({ fill: ne.rgb(de.arrowcolor), "stroke-width": 0 }));
              }
              Se && ze(ve, fe, me, Ce), we && ze(ge, he, pe, be);
            };
          }, 3599: function(G, J, Z) {
            var te = Z(3377), ne = Z(6035);
            G.exports = { moduleType: "component", name: "annotations", layoutAttributes: Z(50222), supplyLayoutDefaults: Z(63737), includeBasePlot: Z(20706)("annotations"), calcAutorange: Z(60317), draw: te.draw, drawOne: te.drawOne, drawRaw: te.drawRaw, hasClickToShow: ne.hasClickToShow, onClick: ne.onClick, convertCoords: Z(59741) };
          }, 38239: function(G, J, Z) {
            var te = Z(50222), ne = Z(13582).overrideAll, ee = Z(78032).templatedArray;
            G.exports = ne(ee("annotation", { visible: te.visible, x: { valType: "any" }, y: { valType: "any" }, z: { valType: "any" }, ax: { valType: "number" }, ay: { valType: "number" }, xanchor: te.xanchor, xshift: te.xshift, yanchor: te.yanchor, yshift: te.yshift, text: te.text, textangle: te.textangle, font: te.font, width: te.width, height: te.height, opacity: te.opacity, align: te.align, valign: te.valign, bgcolor: te.bgcolor, bordercolor: te.bordercolor, borderpad: te.borderpad, borderwidth: te.borderwidth, showarrow: te.showarrow, arrowcolor: te.arrowcolor, arrowhead: te.arrowhead, startarrowhead: te.startarrowhead, arrowside: te.arrowside, arrowsize: te.arrowsize, startarrowsize: te.startarrowsize, arrowwidth: te.arrowwidth, standoff: te.standoff, startstandoff: te.startstandoff, hovertext: te.hovertext, hoverlabel: te.hoverlabel, captureevents: te.captureevents }), "calc", "from-root");
          }, 47979: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714);
            function ee(ae, oe) {
              var se = oe.fullSceneLayout.domain, ce = oe.fullLayout._size, le = { pdata: null, type: "linear", autorange: !1, range: [-1 / 0, 1 / 0] };
              ae._xa = {}, te.extendFlat(ae._xa, le), ne.setConvert(ae._xa), ae._xa._offset = ce.l + se.x[0] * ce.w, ae._xa.l2p = function() {
                return 0.5 * (1 + ae._pdata[0] / ae._pdata[3]) * ce.w * (se.x[1] - se.x[0]);
              }, ae._ya = {}, te.extendFlat(ae._ya, le), ne.setConvert(ae._ya), ae._ya._offset = ce.t + (1 - se.y[1]) * ce.h, ae._ya.l2p = function() {
                return 0.5 * (1 - ae._pdata[1] / ae._pdata[3]) * ce.h * (se.y[1] - se.y[0]);
              };
            }
            G.exports = function(ae) {
              for (var oe = ae.fullSceneLayout.annotations, se = 0; se < oe.length; se++) ee(oe[se], ae);
              ae.fullLayout._infolayer.selectAll(".annotation-" + ae.id).remove();
            };
          }, 34232: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(59008), ae = Z(53271), oe = Z(38239);
            function se(ce, le, ue, de) {
              function fe(me, pe) {
                return te.coerce(ce, le, oe, me, pe);
              }
              function he(me) {
                var pe = me + "axis", ye = { _fullLayout: {} };
                return ye._fullLayout[pe] = ue[pe], ne.coercePosition(le, ye, fe, me, me, 0.5);
              }
              fe("visible") && (ae(ce, le, de.fullLayout, fe), he("x"), he("y"), he("z"), te.noneOrAll(ce, le, ["x", "y", "z"]), le.xref = "x", le.yref = "y", le.zref = "z", fe("xanchor"), fe("yanchor"), fe("xshift"), fe("yshift"), le.showarrow && (le.axref = "pixel", le.ayref = "pixel", fe("ax", -10), fe("ay", -30), te.noneOrAll(ce, le, ["ax", "ay"])));
            }
            G.exports = function(ce, le, ue) {
              ee(ce, le, { name: "annotations", handleItemDefaults: se, fullLayout: ue.fullLayout });
            };
          }, 9756: function(G, J, Z) {
            var te = Z(3377).drawRaw, ne = Z(25802), ee = ["x", "y", "z"];
            G.exports = function(ae) {
              for (var oe = ae.fullSceneLayout, se = ae.dataScale, ce = oe.annotations, le = 0; le < ce.length; le++) {
                for (var ue = ce[le], de = !1, fe = 0; fe < 3; fe++) {
                  var he = ee[fe], me = ue[he], pe = oe[he + "axis"].r2fraction(me);
                  if (pe < 0 || pe > 1) {
                    de = !0;
                    break;
                  }
                }
                de ? ae.fullLayout._infolayer.select(".annotation-" + ae.id + '[data-index="' + le + '"]').remove() : (ue._pdata = ne(ae.glplot.cameraParams, [oe.xaxis.r2l(ue.x) * se[0], oe.yaxis.r2l(ue.y) * se[1], oe.zaxis.r2l(ue.z) * se[2]]), te(ae.graphDiv, ue, le, ae.id, ue._xa, ue._ya));
              }
            };
          }, 83348: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809);
            G.exports = { moduleType: "component", name: "annotations3d", schema: { subplots: { scene: { annotations: Z(38239) } } }, layoutAttributes: Z(38239), handleDefaults: Z(34232), includeBasePlot: function(ee, ae) {
              var oe = te.subplotsRegistry.gl3d;
              if (oe) for (var se = oe.attrRegex, ce = Object.keys(ee), le = 0; le < ce.length; le++) {
                var ue = ce[le];
                se.test(ue) && (ee[ue].annotations || []).length && (ne.pushUnique(ae._basePlotModules, oe), ne.pushUnique(ae._subplots.gl3d, ue));
              }
            }, convert: Z(47979), draw: Z(9756) };
          }, 37177: function(G, J, Z) {
            G.exports = Z(24453), Z(23428), Z(1401), Z(72210), Z(28569), Z(81133), Z(78295), Z(25512), Z(42645), Z(62324), Z(91662), Z(66445), Z(50506), Z(84756), Z(41858), Z(57985);
          }, 29698: function(G, J, Z) {
            var te = Z(37177), ne = Z(34809), ee = Z(63821), ae = ee.EPOCHJD, oe = ee.ONEDAY, se = { valType: "enumerated", values: ne.sortObjectKeys(te.calendars), editType: "calc", dflt: "gregorian" }, ce = function(be, Ce, Se, we) {
              var Ee = {};
              return Ee[Se] = se, ne.coerce(be, Ce, Ee, Se, we);
            }, le = "##", ue = { d: { 0: "dd", "-": "d" }, e: { 0: "d", "-": "d" }, a: { 0: "D", "-": "D" }, A: { 0: "DD", "-": "DD" }, j: { 0: "oo", "-": "o" }, W: { 0: "ww", "-": "w" }, m: { 0: "mm", "-": "m" }, b: { 0: "M", "-": "M" }, B: { 0: "MM", "-": "MM" }, y: { 0: "yy", "-": "yy" }, Y: { 0: "yyyy", "-": "yyyy" }, U: le, w: le, c: { 0: "D M d %X yyyy", "-": "D M d %X yyyy" }, x: { 0: "mm/dd/yyyy", "-": "mm/dd/yyyy" } }, de = {};
            function fe(be) {
              var Ce = de[be];
              return Ce || (de[be] = te.instance(be));
            }
            function he(be) {
              return ne.extendFlat({}, se, { description: be });
            }
            function me(be) {
              return "Sets the calendar system to use with `" + be + "` date data.";
            }
            var pe = { xcalendar: he(me("x")) }, ye = ne.extendFlat({}, pe, { ycalendar: he(me("y")) }), ge = ne.extendFlat({}, ye, { zcalendar: he(me("z")) }), ve = he(["Sets the calendar system to use for `range` and `tick0`", "if this is a date axis. This does not set the calendar for", "interpreting data on this axis, that's specified in the trace", "or via the global `layout.calendar`"].join(" "));
            G.exports = { moduleType: "component", name: "calendars", schema: { traces: { scatter: ye, bar: ye, box: ye, heatmap: ye, contour: ye, histogram: ye, histogram2d: ye, histogram2dcontour: ye, scatter3d: ge, surface: ge, mesh3d: ge, scattergl: ye, ohlc: pe, candlestick: pe }, layout: { calendar: he(["Sets the default calendar system to use for interpreting and", "displaying dates throughout the plot."].join(" ")) }, subplots: { xaxis: { calendar: ve }, yaxis: { calendar: ve }, scene: { xaxis: { calendar: ve }, yaxis: { calendar: ve }, zaxis: { calendar: ve } }, polar: { radialaxis: { calendar: ve } } }, transforms: { filter: { valuecalendar: he(["WARNING: All transforms are deprecated and may be removed from the API in next major version.", "Sets the calendar system to use for `value`, if it is a date."].join(" ")), targetcalendar: he(["WARNING: All transforms are deprecated and may be removed from the API in next major version.", "Sets the calendar system to use for `target`, if it is an", "array of dates. If `target` is a string (eg *x*) we use the", "corresponding trace attribute (eg `xcalendar`) if it exists,", "even if `targetcalendar` is provided."].join(" ")) } } }, layoutAttributes: se, handleDefaults: ce, handleTraceDefaults: function(be, Ce, Se, we) {
              for (var Ee = 0; Ee < Se.length; Ee++) ce(be, Ce, Se[Ee] + "calendar", we.calendar);
            }, CANONICAL_SUNDAY: { chinese: "2000-01-02", coptic: "2000-01-03", discworld: "2000-01-03", ethiopian: "2000-01-05", hebrew: "5000-01-01", islamic: "1000-01-02", julian: "2000-01-03", mayan: "5000-01-01", nanakshahi: "1000-01-05", nepali: "2000-01-05", persian: "1000-01-01", jalali: "1000-01-01", taiwan: "1000-01-04", thai: "2000-01-04", ummalqura: "1400-01-06" }, CANONICAL_TICK: { chinese: "2000-01-01", coptic: "2000-01-01", discworld: "2000-01-01", ethiopian: "2000-01-01", hebrew: "5000-01-01", islamic: "1000-01-01", julian: "2000-01-01", mayan: "5000-01-01", nanakshahi: "1000-01-01", nepali: "2000-01-01", persian: "1000-01-01", jalali: "1000-01-01", taiwan: "1000-01-01", thai: "2000-01-01", ummalqura: "1400-01-01" }, DFLTRANGE: { chinese: ["2000-01-01", "2001-01-01"], coptic: ["1700-01-01", "1701-01-01"], discworld: ["1800-01-01", "1801-01-01"], ethiopian: ["2000-01-01", "2001-01-01"], hebrew: ["5700-01-01", "5701-01-01"], islamic: ["1400-01-01", "1401-01-01"], julian: ["2000-01-01", "2001-01-01"], mayan: ["5200-01-01", "5201-01-01"], nanakshahi: ["0500-01-01", "0501-01-01"], nepali: ["2000-01-01", "2001-01-01"], persian: ["1400-01-01", "1401-01-01"], jalali: ["1400-01-01", "1401-01-01"], taiwan: ["0100-01-01", "0101-01-01"], thai: ["2500-01-01", "2501-01-01"], ummalqura: ["1400-01-01", "1401-01-01"] }, getCal: fe, worldCalFmt: function(be, Ce, Se) {
              for (var we, Ee, xe, Le, Qe, ke = Math.floor((Ce + 0.05) / oe) + ae, Me = fe(Se).fromJD(ke), Re = 0; (Re = be.indexOf("%", Re)) !== -1; ) (we = be.charAt(Re + 1)) === "0" || we === "-" || we === "_" ? (xe = 3, Ee = be.charAt(Re + 2), we === "_" && (we = "-")) : (Ee = we, we = "0", xe = 2), (Le = ue[Ee]) ? (Qe = Le === le ? le : Me.formatDate(Le[we]), be = be.substr(0, Re) + Qe + be.substr(Re + xe), Re += Qe.length) : Re += xe;
              return be;
            } };
          }, 10229: function(G, J) {
            J.defaults = ["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"], J.defaultLine = "#444", J.lightLine = "#eee", J.background = "#fff", J.borderLine = "#BEC8D9", J.lightFraction = 90.9090909090909;
          }, 78766: function(G, J, Z) {
            var te = Z(65657), ne = Z(10721), ee = Z(87800).isTypedArray, ae = G.exports = {}, oe = Z(10229);
            ae.defaults = oe.defaults;
            var se = ae.defaultLine = oe.defaultLine;
            ae.lightLine = oe.lightLine;
            var ce = ae.background = oe.background;
            function le(ue) {
              if (ne(ue) || typeof ue != "string") return ue;
              var de = ue.trim();
              if (de.substr(0, 3) !== "rgb") return ue;
              var fe = de.match(/^rgba?\s*\(([^()]*)\)$/);
              if (!fe) return ue;
              var he = fe[1].trim().split(/\s*[\s,]\s*/), me = de.charAt(3) === "a" && he.length === 4;
              if (!me && he.length !== 3) return ue;
              for (var pe = 0; pe < he.length; pe++) {
                if (!he[pe].length || (he[pe] = Number(he[pe]), !(he[pe] >= 0))) return ue;
                if (pe === 3) he[pe] > 1 && (he[pe] = 1);
                else if (he[pe] >= 1) return ue;
              }
              var ye = Math.round(255 * he[0]) + ", " + Math.round(255 * he[1]) + ", " + Math.round(255 * he[2]);
              return me ? "rgba(" + ye + ", " + he[3] + ")" : "rgb(" + ye + ")";
            }
            ae.tinyRGB = function(ue) {
              var de = ue.toRgb();
              return "rgb(" + Math.round(de.r) + ", " + Math.round(de.g) + ", " + Math.round(de.b) + ")";
            }, ae.rgb = function(ue) {
              return ae.tinyRGB(te(ue));
            }, ae.opacity = function(ue) {
              return ue ? te(ue).getAlpha() : 0;
            }, ae.addOpacity = function(ue, de) {
              var fe = te(ue).toRgb();
              return "rgba(" + Math.round(fe.r) + ", " + Math.round(fe.g) + ", " + Math.round(fe.b) + ", " + de + ")";
            }, ae.combine = function(ue, de) {
              var fe = te(ue).toRgb();
              if (fe.a === 1) return te(ue).toRgbString();
              var he = te(de || ce).toRgb(), me = he.a === 1 ? he : { r: 255 * (1 - he.a) + he.r * he.a, g: 255 * (1 - he.a) + he.g * he.a, b: 255 * (1 - he.a) + he.b * he.a }, pe = { r: me.r * (1 - fe.a) + fe.r * fe.a, g: me.g * (1 - fe.a) + fe.g * fe.a, b: me.b * (1 - fe.a) + fe.b * fe.a };
              return te(pe).toRgbString();
            }, ae.interpolate = function(ue, de, fe) {
              var he = te(ue).toRgb(), me = te(de).toRgb(), pe = { r: fe * he.r + (1 - fe) * me.r, g: fe * he.g + (1 - fe) * me.g, b: fe * he.b + (1 - fe) * me.b };
              return te(pe).toRgbString();
            }, ae.contrast = function(ue, de, fe) {
              var he = te(ue);
              return he.getAlpha() !== 1 && (he = te(ae.combine(ue, ce))), (he.isDark() ? de ? he.lighten(de) : ce : fe ? he.darken(fe) : se).toString();
            }, ae.stroke = function(ue, de) {
              var fe = te(de);
              ue.style({ stroke: ae.tinyRGB(fe), "stroke-opacity": fe.getAlpha() });
            }, ae.fill = function(ue, de) {
              var fe = te(de);
              ue.style({ fill: ae.tinyRGB(fe), "fill-opacity": fe.getAlpha() });
            }, ae.clean = function(ue) {
              if (ue && typeof ue == "object") {
                var de, fe, he, me, pe = Object.keys(ue);
                for (de = 0; de < pe.length; de++) if (me = ue[he = pe[de]], he.substr(he.length - 5) === "color") if (Array.isArray(me)) for (fe = 0; fe < me.length; fe++) me[fe] = le(me[fe]);
                else ue[he] = le(me);
                else if (he.substr(he.length - 10) === "colorscale" && Array.isArray(me)) for (fe = 0; fe < me.length; fe++) Array.isArray(me[fe]) && (me[fe][1] = le(me[fe][1]));
                else if (Array.isArray(me)) {
                  var ye = me[0];
                  if (!Array.isArray(ye) && ye && typeof ye == "object") for (fe = 0; fe < me.length; fe++) ae.clean(me[fe]);
                } else me && typeof me == "object" && !ee(me) && ae.clean(me);
              }
            };
          }, 25158: function(G, J, Z) {
            var te = Z(25829), ne = Z(80337), ee = Z(93049).extendFlat, ae = Z(13582).overrideAll;
            G.exports = ae({ orientation: { valType: "enumerated", values: ["h", "v"], dflt: "v" }, thicknessmode: { valType: "enumerated", values: ["fraction", "pixels"], dflt: "pixels" }, thickness: { valType: "number", min: 0, dflt: 30 }, lenmode: { valType: "enumerated", values: ["fraction", "pixels"], dflt: "fraction" }, len: { valType: "number", min: 0, dflt: 1 }, x: { valType: "number" }, xref: { valType: "enumerated", dflt: "paper", values: ["container", "paper"], editType: "layoutstyle" }, xanchor: { valType: "enumerated", values: ["left", "center", "right"] }, xpad: { valType: "number", min: 0, dflt: 10 }, y: { valType: "number" }, yref: { valType: "enumerated", dflt: "paper", values: ["container", "paper"], editType: "layoutstyle" }, yanchor: { valType: "enumerated", values: ["top", "middle", "bottom"] }, ypad: { valType: "number", min: 0, dflt: 10 }, outlinecolor: te.linecolor, outlinewidth: te.linewidth, bordercolor: te.linecolor, borderwidth: { valType: "number", min: 0, dflt: 0 }, bgcolor: { valType: "color", dflt: "rgba(0,0,0,0)" }, tickmode: te.minor.tickmode, nticks: te.nticks, tick0: te.tick0, dtick: te.dtick, tickvals: te.tickvals, ticktext: te.ticktext, ticks: ee({}, te.ticks, { dflt: "" }), ticklabeloverflow: ee({}, te.ticklabeloverflow, {}), ticklabelposition: { valType: "enumerated", values: ["outside", "inside", "outside top", "inside top", "outside left", "inside left", "outside right", "inside right", "outside bottom", "inside bottom"], dflt: "outside" }, ticklen: te.ticklen, tickwidth: te.tickwidth, tickcolor: te.tickcolor, ticklabelstep: te.ticklabelstep, showticklabels: te.showticklabels, labelalias: te.labelalias, tickfont: ne({}), tickangle: te.tickangle, tickformat: te.tickformat, tickformatstops: te.tickformatstops, tickprefix: te.tickprefix, showtickprefix: te.showtickprefix, ticksuffix: te.ticksuffix, showticksuffix: te.showticksuffix, separatethousands: te.separatethousands, exponentformat: te.exponentformat, minexponent: te.minexponent, showexponent: te.showexponent, title: { text: { valType: "string" }, font: ne({}), side: { valType: "enumerated", values: ["right", "top", "bottom"] } }, _deprecated: { title: { valType: "string" }, titlefont: ne({}), titleside: { valType: "enumerated", values: ["right", "top", "bottom"], dflt: "top" } } }, "colorbars", "from-root");
          }, 34554: function(G) {
            G.exports = { cn: { colorbar: "colorbar", cbbg: "cbbg", cbfill: "cbfill", cbfills: "cbfills", cbline: "cbline", cblines: "cblines", cbaxis: "cbaxis", cbtitleunshift: "cbtitleunshift", cbtitle: "cbtitle", cboutline: "cboutline", crisp: "crisp", jsPlaceholder: "js-placeholder" } };
          }, 42097: function(G, J, Z) {
            var te = Z(34809), ne = Z(78032), ee = Z(22777), ae = Z(87433), oe = Z(12036), se = Z(54616), ce = Z(25158);
            G.exports = function(le, ue, de) {
              var fe = ne.newContainer(ue, "colorbar"), he = le.colorbar || {};
              function me($e, qe) {
                return te.coerce(he, fe, ce, $e, qe);
              }
              var pe = de.margin || { t: 0, b: 0, l: 0, r: 0 }, ye = de.width - pe.l - pe.r, ge = de.height - pe.t - pe.b, ve = me("orientation") === "v", be = me("thicknessmode");
              me("thickness", be === "fraction" ? 30 / (ve ? ye : ge) : 30);
              var Ce = me("lenmode");
              me("len", Ce === "fraction" ? 1 : ve ? ge : ye);
              var Se, we, Ee, xe = me("yref") === "paper", Le = me("xref") === "paper", Qe = "left";
              ve ? (Ee = "middle", Qe = Le ? "left" : "right", Se = Le ? 1.02 : 1, we = 0.5) : (Ee = xe ? "bottom" : "top", Qe = "center", Se = 0.5, we = xe ? 1.02 : 1), te.coerce(he, fe, { x: { valType: "number", min: Le ? -2 : 0, max: Le ? 3 : 1, dflt: Se } }, "x"), te.coerce(he, fe, { y: { valType: "number", min: xe ? -2 : 0, max: xe ? 3 : 1, dflt: we } }, "y"), me("xanchor", Qe), me("xpad"), me("yanchor", Ee), me("ypad"), te.noneOrAll(he, fe, ["x", "y"]), me("outlinecolor"), me("outlinewidth"), me("bordercolor"), me("borderwidth"), me("bgcolor");
              var ke = te.coerce(he, fe, { ticklabelposition: { valType: "enumerated", dflt: "outside", values: ve ? ["outside", "inside", "outside top", "inside top", "outside bottom", "inside bottom"] : ["outside", "inside", "outside left", "inside left", "outside right", "inside right"] } }, "ticklabelposition");
              me("ticklabeloverflow", ke.indexOf("inside") !== -1 ? "hide past domain" : "hide past div"), ee(he, fe, me, "linear");
              var Me = de.font, Re = { noAutotickangles: !0, noTicklabelshift: !0, noTicklabelstandoff: !0, outerTicks: !1, font: Me };
              ke.indexOf("inside") !== -1 && (Re.bgColor = "black"), se(he, fe, me, "linear", Re), oe(he, fe, me, "linear", Re), ae(he, fe, me, "linear", Re), me("title.text", de._dfltTitle.colorbar);
              var De = fe.showticklabels ? fe.tickfont : Me, Pe = te.extendFlat({}, Me, { family: De.family, size: te.bigFont(De.size) });
              te.coerceFont(me, "title.font", Pe), me("title.side", ve ? "top" : "right");
            };
          }, 5881: function(G, J, Z) {
            var te = Z(45568), ne = Z(65657), ee = Z(44122), ae = Z(33626), oe = Z(29714), se = Z(14751), ce = Z(34809), le = ce.strTranslate, ue = Z(93049).extendFlat, de = Z(27983), fe = Z(62203), he = Z(78766), me = Z(17240), pe = Z(30635), ye = Z(65477).flipScale, ge = Z(97655), ve = Z(40957), be = Z(25829), Ce = Z(4530), Se = Ce.LINE_SPACING, we = Ce.FROM_TL, Ee = Ce.FROM_BR, xe = Z(34554).cn;
            G.exports = { draw: function(Le) {
              var Qe = Le._fullLayout._infolayer.selectAll("g." + xe.colorbar).data(function(ke) {
                var Me, Re, De, Pe, $e = ke._fullLayout, qe = ke.calcdata, Xe = [];
                function Ve(st) {
                  return ue(st, { _fillcolor: null, _line: { color: null, width: null, dash: null }, _levels: { start: null, end: null, size: null }, _filllevels: null, _fillgradient: null, _zrange: null });
                }
                function Ze() {
                  typeof Pe.calc == "function" ? Pe.calc(ke, De, Me) : (Me._fillgradient = Re.reversescale ? ye(Re.colorscale) : Re.colorscale, Me._zrange = [Re[Pe.min], Re[Pe.max]]);
                }
                for (var Ge = 0; Ge < qe.length; Ge++) {
                  var ze = qe[Ge];
                  if ((De = ze[0].trace)._module) {
                    var We = De._module.colorbar;
                    if (De.visible === !0 && We) for (var at = Array.isArray(We), ct = at ? We : [We], et = 0; et < ct.length; et++) {
                      var dt = (Pe = ct[et]).container;
                      (Re = dt ? De[dt] : De) && Re.showscale && ((Me = Ve(Re.colorbar))._id = "cb" + De.uid + (at && dt ? "-" + dt : ""), Me._traceIndex = De.index, Me._propPrefix = (dt ? dt + "." : "") + "colorbar.", Me._meta = De._meta, Ze(), Xe.push(Me));
                    }
                  }
                }
                for (var Je in $e._colorAxes) if ((Re = $e[Je]).showscale) {
                  var it = $e._colorAxes[Je];
                  (Me = Ve(Re.colorbar))._id = "cb" + Je, Me._propPrefix = Je + ".colorbar.", Me._meta = $e._meta, Pe = { min: "cmin", max: "cmax" }, it[0] !== "heatmap" && (De = it[1], Pe.calc = De._module.colorbar.calc), Ze(), Xe.push(Me);
                }
                return Xe;
              }(Le), function(ke) {
                return ke._id;
              });
              Qe.enter().append("g").attr("class", function(ke) {
                return ke._id;
              }).classed(xe.colorbar, !0), Qe.each(function(ke) {
                var Me = te.select(this);
                ce.ensureSingle(Me, "rect", xe.cbbg), ce.ensureSingle(Me, "g", xe.cbfills), ce.ensureSingle(Me, "g", xe.cblines), ce.ensureSingle(Me, "g", xe.cbaxis, function(De) {
                  De.classed(xe.crisp, !0);
                }), ce.ensureSingle(Me, "g", xe.cbtitleunshift, function(De) {
                  De.append("g").classed(xe.cbtitle, !0);
                }), ce.ensureSingle(Me, "rect", xe.cboutline);
                var Re = function(De, Pe, $e) {
                  var qe = Pe.orientation === "v", Xe = Pe.len, Ve = Pe.lenmode, Ze = Pe.thickness, Ge = Pe.thicknessmode, ze = Pe.outlinewidth, We = Pe.borderwidth, at = Pe.bgcolor, ct = Pe.xanchor, et = Pe.yanchor, dt = Pe.xpad, Je = Pe.ypad, it = Pe.x, st = qe ? Pe.y : 1 - Pe.y, gt = Pe.yref === "paper", bt = Pe.xref === "paper", yt = $e._fullLayout, St = yt._size, Lt = Pe._fillcolor, Rt = Pe._line, vt = Pe.title, $t = vt.side, Pt = Pe._zrange || te.extent((typeof Lt == "function" ? Lt : Rt.color).domain()), Xt = typeof Rt.color == "function" ? Rt.color : function() {
                    return Rt.color;
                  }, Nt = typeof Lt == "function" ? Lt : function() {
                    return Lt;
                  }, Gt = Pe._levels, Ft = function(Ot, un, yn) {
                    var _n, gn, bn = un._levels, an = [], Zt = [], Mn = bn.end + bn.size / 100, Ln = bn.size, In = 1.001 * yn[0] - 1e-3 * yn[1], kn = 1.001 * yn[1] - 1e-3 * yn[0];
                    for (gn = 0; gn < 1e5 && (_n = bn.start + gn * Ln, !(Ln > 0 ? _n >= Mn : _n <= Mn)); gn++) _n > In && _n < kn && an.push(_n);
                    if (un._fillgradient) Zt = [0];
                    else if (typeof un._fillcolor == "function") {
                      var er = un._filllevels;
                      if (er) for (Mn = er.end + er.size / 100, Ln = er.size, gn = 0; gn < 1e5 && (_n = er.start + gn * Ln, !(Ln > 0 ? _n >= Mn : _n <= Mn)); gn++) _n > yn[0] && _n < yn[1] && Zt.push(_n);
                      else (Zt = an.map(function(jn) {
                        return jn - bn.size / 2;
                      })).push(Zt[Zt.length - 1] + bn.size);
                    } else un._fillcolor && typeof un._fillcolor == "string" && (Zt = [0]);
                    return bn.size < 0 && (an.reverse(), Zt.reverse()), { line: an, fill: Zt };
                  }(0, Pe, Pt), Tt = Ft.fill, It = Ft.line, Ct = Math.round(Ze * (Ge === "fraction" ? qe ? St.w : St.h : 1)), Kt = Ct / (qe ? St.w : St.h), Vt = Math.round(Xe * (Ve === "fraction" ? qe ? St.h : St.w : 1)), Yt = Vt / (qe ? St.h : St.w), sn = bt ? St.w : $e._fullLayout.width, hn = gt ? St.h : $e._fullLayout.height, Qt = Math.round(qe ? it * sn + dt : st * hn + Je), Ut = { center: 0.5, right: 1 }[ct] || 0, Wt = { top: 1, middle: 0.5 }[et] || 0, rn = qe ? it - Ut * Kt : st - Wt * Kt, vn = qe ? st - Wt * Yt : it - Ut * Yt, dn = Math.round(qe ? hn * (1 - vn) : sn * vn);
                  Pe._lenFrac = Yt, Pe._thickFrac = Kt, Pe._uFrac = rn, Pe._vFrac = vn;
                  var Ht = Pe._axis = function(Ot, un, yn) {
                    var _n = Ot._fullLayout, gn = un.orientation === "v", bn = { type: "linear", range: yn, tickmode: un.tickmode, nticks: un.nticks, tick0: un.tick0, dtick: un.dtick, tickvals: un.tickvals, ticktext: un.ticktext, ticks: un.ticks, ticklen: un.ticklen, tickwidth: un.tickwidth, tickcolor: un.tickcolor, showticklabels: un.showticklabels, labelalias: un.labelalias, ticklabelposition: un.ticklabelposition, ticklabeloverflow: un.ticklabeloverflow, ticklabelstep: un.ticklabelstep, tickfont: un.tickfont, tickangle: un.tickangle, tickformat: un.tickformat, exponentformat: un.exponentformat, minexponent: un.minexponent, separatethousands: un.separatethousands, showexponent: un.showexponent, showtickprefix: un.showtickprefix, tickprefix: un.tickprefix, showticksuffix: un.showticksuffix, ticksuffix: un.ticksuffix, title: un.title, showline: !0, anchor: "free", side: gn ? "right" : "bottom", position: 1 }, an = gn ? "y" : "x", Zt = { type: "linear", _id: an + un._id }, Mn = { letter: an, font: _n.font, noAutotickangles: an === "y", noHover: !0, noTickson: !0, noTicklabelmode: !0, noInsideRange: !0, calendar: _n.calendar };
                    function Ln(In, kn) {
                      return ce.coerce(bn, Zt, be, In, kn);
                    }
                    return ge(bn, Zt, Ln, Mn, _n), ve(bn, Zt, Ln, Mn), Zt;
                  }($e, Pe, Pt);
                  Ht.position = Kt + (qe ? it + dt / St.w : st + Je / St.h);
                  var jt = ["top", "bottom"].indexOf($t) !== -1;
                  if (qe && jt && (Ht.title.side = $t, Ht.titlex = it + dt / St.w, Ht.titley = vn + (vt.side === "top" ? Yt - Je / St.h : Je / St.h)), qe || jt || (Ht.title.side = $t, Ht.titley = st + Je / St.h, Ht.titlex = vn + dt / St.w), Rt.color && Pe.tickmode === "auto") {
                    Ht.tickmode = "linear", Ht.tick0 = Gt.start;
                    var cn = Gt.size, wn = ce.constrain(Vt / 50, 4, 15) + 1, Rn = (Pt[1] - Pt[0]) / ((Pe.nticks || wn) * cn);
                    if (Rn > 1) {
                      var Pn = Math.pow(10, Math.floor(Math.log(Rn) / Math.LN10));
                      cn *= Pn * ce.roundUp(Rn / Pn, [2, 5, 10]), (Math.abs(Gt.start) / Gt.size + 1e-6) % 1 < 2e-6 && (Ht.tick0 = 0);
                    }
                    Ht.dtick = cn;
                  }
                  Ht.domain = qe ? [vn + Je / St.h, vn + Yt - Je / St.h] : [vn + dt / St.w, vn + Yt - dt / St.w], Ht.setScale(), De.attr("transform", le(Math.round(St.l), Math.round(St.t)));
                  var Ie, rt = De.select("." + xe.cbtitleunshift).attr("transform", le(-Math.round(St.l), -Math.round(St.t))), ft = Ht.ticklabelposition, pt = Ht.title.font.size, wt = De.select("." + xe.cbaxis), Bt = 0, At = 0;
                  function Dt(Ot, un) {
                    var yn = { propContainer: Ht, propName: Pe._propPrefix + "title", traceIndex: Pe._traceIndex, _meta: Pe._meta, placeholder: yt._dfltTitle.colorbar, containerGroup: De.select("." + xe.cbtitle) }, _n = Ot.charAt(0) === "h" ? Ot.substr(1) : "h" + Ot;
                    De.selectAll("." + _n + ",." + _n + "-math-group").remove(), me.draw($e, Ot, ue(yn, un || {}));
                  }
                  return ce.syncOrAsync([ee.previousPromises, function() {
                    var Ot, un;
                    (qe && jt || !qe && !jt) && ($t === "top" && (Ot = dt + St.l + sn * it, un = Je + St.t + hn * (1 - vn - Yt) + 3 + 0.75 * pt), $t === "bottom" && (Ot = dt + St.l + sn * it, un = Je + St.t + hn * (1 - vn) - 3 - 0.25 * pt), $t === "right" && (un = Je + St.t + hn * st + 3 + 0.75 * pt, Ot = dt + St.l + sn * vn), Dt(Ht._id + "title", { attributes: { x: Ot, y: un, "text-anchor": qe ? "start" : "middle" } }));
                  }, function() {
                    if (!qe && !jt || qe && jt) {
                      var Ot, un = De.select("." + xe.cbtitle), yn = un.select("text"), _n = [-ze / 2, ze / 2], gn = un.select(".h" + Ht._id + "title-math-group").node(), bn = 15.6;
                      if (yn.node() && (bn = parseInt(yn.node().style.fontSize, 10) * Se), gn ? (Ot = fe.bBox(gn), At = Ot.width, (Bt = Ot.height) > bn && (_n[1] -= (Bt - bn) / 2)) : yn.node() && !yn.classed(xe.jsPlaceholder) && (Ot = fe.bBox(yn.node()), At = Ot.width, Bt = Ot.height), qe) {
                        if (Bt) {
                          if (Bt += 5, $t === "top") Ht.domain[1] -= Bt / St.h, _n[1] *= -1;
                          else {
                            Ht.domain[0] += Bt / St.h;
                            var an = pe.lineCount(yn);
                            _n[1] += (1 - an) * bn;
                          }
                          un.attr("transform", le(_n[0], _n[1])), Ht.setScale();
                        }
                      } else At && ($t === "right" && (Ht.domain[0] += (At + pt / 2) / St.w), un.attr("transform", le(_n[0], _n[1])), Ht.setScale());
                    }
                    De.selectAll("." + xe.cbfills + ",." + xe.cblines).attr("transform", qe ? le(0, Math.round(St.h * (1 - Ht.domain[1]))) : le(Math.round(St.w * Ht.domain[0]), 0)), wt.attr("transform", qe ? le(0, Math.round(-St.t)) : le(Math.round(-St.l), 0));
                    var Zt = De.select("." + xe.cbfills).selectAll("rect." + xe.cbfill).attr("style", "").data(Tt);
                    Zt.enter().append("rect").classed(xe.cbfill, !0).attr("style", ""), Zt.exit().remove();
                    var Mn = Pt.map(Ht.c2p).map(Math.round).sort(function(jn, ur) {
                      return jn - ur;
                    });
                    Zt.each(function(jn, ur) {
                      var Kn = [ur === 0 ? Pt[0] : (Tt[ur] + Tt[ur - 1]) / 2, ur === Tt.length - 1 ? Pt[1] : (Tt[ur] + Tt[ur + 1]) / 2].map(Ht.c2p).map(Math.round);
                      qe && (Kn[1] = ce.constrain(Kn[1] + (Kn[1] > Kn[0]) ? 1 : -1, Mn[0], Mn[1]));
                      var Jn = te.select(this).attr(qe ? "x" : "y", Qt).attr(qe ? "y" : "x", te.min(Kn)).attr(qe ? "width" : "height", Math.max(Ct, 2)).attr(qe ? "height" : "width", Math.max(te.max(Kn) - te.min(Kn), 2));
                      if (Pe._fillgradient) fe.gradient(Jn, $e, Pe._id, qe ? "vertical" : "horizontalreversed", Pe._fillgradient, "fill");
                      else {
                        var Lr = Nt(jn).replace("e-", "");
                        Jn.attr("fill", ne(Lr).toHexString());
                      }
                    });
                    var Ln = De.select("." + xe.cblines).selectAll("path." + xe.cbline).data(Rt.color && Rt.width ? It : []);
                    Ln.enter().append("path").classed(xe.cbline, !0), Ln.exit().remove(), Ln.each(function(jn) {
                      var ur = Qt, Kn = Math.round(Ht.c2p(jn)) + Rt.width / 2 % 1;
                      te.select(this).attr("d", "M" + (qe ? ur + "," + Kn : Kn + "," + ur) + (qe ? "h" : "v") + Ct).call(fe.lineGroupStyle, Rt.width, Xt(jn), Rt.dash);
                    }), wt.selectAll("g." + Ht._id + "tick,path").remove();
                    var In = Qt + Ct + (ze || 0) / 2 - (Pe.ticks === "outside" ? 1 : 0), kn = oe.calcTicks(Ht), er = oe.getTickSigns(Ht)[2];
                    return oe.drawTicks($e, Ht, { vals: Ht.ticks === "inside" ? oe.clipEnds(Ht, kn) : kn, layer: wt, path: oe.makeTickPath(Ht, In, er), transFn: oe.makeTransTickFn(Ht) }), oe.drawLabels($e, Ht, { vals: kn, layer: wt, transFn: oe.makeTransTickLabelFn(Ht), labelFns: oe.makeLabelFns(Ht, In) });
                  }, function() {
                    if (qe && !jt || !qe && jt) {
                      var Ot, un, yn = Ht.position || 0, _n = Ht._offset + Ht._length / 2;
                      if ($t === "right") un = _n, Ot = St.l + sn * yn + 10 + pt * (Ht.showticklabels ? 1 : 0.5);
                      else if (Ot = _n, $t === "bottom" && (un = St.t + hn * yn + 10 + (ft.indexOf("inside") === -1 ? Ht.tickfont.size : 0) + (Ht.ticks !== "intside" && Pe.ticklen || 0)), $t === "top") {
                        var gn = vt.text.split("<br>").length;
                        un = St.t + hn * yn + 10 - Ct - Se * pt * gn;
                      }
                      Dt((qe ? "h" : "v") + Ht._id + "title", { avoid: { selection: te.select($e).selectAll("g." + Ht._id + "tick"), side: $t, offsetTop: qe ? 0 : St.t, offsetLeft: qe ? St.l : 0, maxShift: qe ? yt.width : yt.height }, attributes: { x: Ot, y: un, "text-anchor": "middle" }, transform: { rotate: qe ? -90 : 0, offset: 0 } });
                    }
                  }, ee.previousPromises, function() {
                    var Ot, un = Ct + ze / 2;
                    ft.indexOf("inside") === -1 && (Ot = fe.bBox(wt.node()), un += qe ? Ot.width : Ot.height), Ie = rt.select("text");
                    var yn = 0, _n = qe && $t === "top", gn = !qe && $t === "right", bn = 0;
                    if (Ie.node() && !Ie.classed(xe.jsPlaceholder)) {
                      var an, Zt = rt.select(".h" + Ht._id + "title-math-group").node();
                      Zt && (qe && jt || !qe && !jt) ? (yn = (Ot = fe.bBox(Zt)).width, an = Ot.height) : (yn = (Ot = fe.bBox(rt.node())).right - St.l - (qe ? Qt : dn), an = Ot.bottom - St.t - (qe ? dn : Qt), qe || $t !== "top" || (un += Ot.height, bn = Ot.height)), gn && (Ie.attr("transform", le(yn / 2 + pt / 2, 0)), yn *= 2), un = Math.max(un, qe ? yn : an);
                    }
                    var Mn = 2 * (qe ? dt : Je) + un + We + ze / 2, Ln = 0;
                    !qe && vt.text && et === "bottom" && st <= 0 && (Mn += Ln = Mn / 2, bn += Ln), yt._hColorbarMoveTitle = Ln, yt._hColorbarMoveCBTitle = bn;
                    var In = We + ze, kn = (qe ? Qt : dn) - In / 2 - (qe ? dt : 0), er = (qe ? dn : Qt) - (qe ? Vt : Je + bn - Ln);
                    De.select("." + xe.cbbg).attr("x", kn).attr("y", er).attr(qe ? "width" : "height", Math.max(Mn - Ln, 2)).attr(qe ? "height" : "width", Math.max(Vt + In, 2)).call(he.fill, at).call(he.stroke, Pe.bordercolor).style("stroke-width", We);
                    var jn = gn ? Math.max(yn - 10, 0) : 0;
                    De.selectAll("." + xe.cboutline).attr("x", (qe ? Qt : dn + dt) + jn).attr("y", (qe ? dn + Je - Vt : Qt) + (_n ? Bt : 0)).attr(qe ? "width" : "height", Math.max(Ct, 2)).attr(qe ? "height" : "width", Math.max(Vt - (qe ? 2 * Je + Bt : 2 * dt + jn), 2)).call(he.stroke, Pe.outlinecolor).style({ fill: "none", "stroke-width": ze });
                    var ur = qe ? Ut * Mn : 0, Kn = qe ? 0 : (1 - Wt) * Mn - bn;
                    if (ur = bt ? St.l - ur : -ur, Kn = gt ? St.t - Kn : -Kn, De.attr("transform", le(ur, Kn)), !qe && (We || ne(at).getAlpha() && !ne.equals(yt.paper_bgcolor, at))) {
                      var Jn = wt.selectAll("text"), Lr = Jn[0].length, mr = De.select("." + xe.cbbg).node(), Ar = fe.bBox(mr), Yr = fe.getTranslate(De);
                      Jn.each(function(Zi, $i) {
                        var Br = Lr - 1;
                        if ($i === 0 || $i === Br) {
                          var Pr, ei = fe.bBox(this), Ci = fe.getTranslate(this);
                          if ($i === Br) {
                            var oa = ei.right + Ci.x;
                            (Pr = Ar.right + Yr.x + dn - We - 2 + it - oa) > 0 && (Pr = 0);
                          } else if ($i === 0) {
                            var ka = ei.left + Ci.x;
                            (Pr = Ar.left + Yr.x + dn + We + 2 - ka) < 0 && (Pr = 0);
                          }
                          Pr && (Lr < 3 ? this.setAttribute("transform", "translate(" + Pr + ",0) " + this.getAttribute("transform")) : this.setAttribute("visibility", "hidden"));
                        }
                      });
                    }
                    var jr = {}, Mr = we[ct], Wr = Ee[ct], Bi = we[et], gi = Ee[et], Zr = Mn - Ct;
                    qe ? (Ve === "pixels" ? (jr.y = st, jr.t = Vt * Bi, jr.b = Vt * gi) : (jr.t = jr.b = 0, jr.yt = st + Xe * Bi, jr.yb = st - Xe * gi), Ge === "pixels" ? (jr.x = it, jr.l = Mn * Mr, jr.r = Mn * Wr) : (jr.l = Zr * Mr, jr.r = Zr * Wr, jr.xl = it - Ze * Mr, jr.xr = it + Ze * Wr)) : (Ve === "pixels" ? (jr.x = it, jr.l = Vt * Mr, jr.r = Vt * Wr) : (jr.l = jr.r = 0, jr.xl = it + Xe * Mr, jr.xr = it - Xe * Wr), Ge === "pixels" ? (jr.y = 1 - st, jr.t = Mn * Bi, jr.b = Mn * gi) : (jr.t = Zr * Bi, jr.b = Zr * gi, jr.yt = st - Ze * Bi, jr.yb = st + Ze * gi));
                    var fi = Pe.y < 0.5 ? "b" : "t", ri = Pe.x < 0.5 ? "l" : "r";
                    $e._fullLayout._reservedMargin[Pe._id] = {};
                    var bi = { r: yt.width - kn - ur, l: kn + jr.r, b: yt.height - er - Kn, t: er + jr.b };
                    bt && gt ? ee.autoMargin($e, Pe._id, jr) : bt ? $e._fullLayout._reservedMargin[Pe._id][fi] = bi[fi] : gt || qe ? $e._fullLayout._reservedMargin[Pe._id][ri] = bi[ri] : $e._fullLayout._reservedMargin[Pe._id][fi] = bi[fi];
                  }], $e);
                }(Me, ke, Le);
                Re && Re.then && (Le._promises || []).push(Re), Le._context.edits.colorbarPosition && function(De, Pe, $e) {
                  var qe, Xe, Ve, Ze = Pe.orientation === "v", Ge = $e._fullLayout._size;
                  se.init({ element: De.node(), gd: $e, prepFn: function() {
                    qe = De.attr("transform"), de(De);
                  }, moveFn: function(ze, We) {
                    De.attr("transform", qe + le(ze, We)), Xe = se.align((Ze ? Pe._uFrac : Pe._vFrac) + ze / Ge.w, Ze ? Pe._thickFrac : Pe._lenFrac, 0, 1, Pe.xanchor), Ve = se.align((Ze ? Pe._vFrac : 1 - Pe._uFrac) - We / Ge.h, Ze ? Pe._lenFrac : Pe._thickFrac, 0, 1, Pe.yanchor);
                    var at = se.getCursor(Xe, Ve, Pe.xanchor, Pe.yanchor);
                    de(De, at);
                  }, doneFn: function() {
                    if (de(De), Xe !== void 0 && Ve !== void 0) {
                      var ze = {};
                      ze[Pe._propPrefix + "x"] = Xe, ze[Pe._propPrefix + "y"] = Ve, Pe._traceIndex !== void 0 ? ae.call("_guiRestyle", $e, ze, Pe._traceIndex) : ae.call("_guiRelayout", $e, ze);
                    }
                  } });
                }(Me, ke, Le);
              }), Qe.exit().each(function(ke) {
                ee.autoMargin(Le, ke._id);
              }).remove(), Qe.order();
            } };
          }, 91362: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne) {
              return te.isPlainObject(ne.colorbar);
            };
          }, 96919: function(G, J, Z) {
            G.exports = { moduleType: "component", name: "colorbar", attributes: Z(25158), supplyDefaults: Z(42097), draw: Z(5881).draw, hasColorbar: Z(91362) };
          }, 87163: function(G, J, Z) {
            var te = Z(25158), ne = Z(90694).counter, ee = Z(62994), ae = Z(19017).scales;
            function oe(se) {
              return "`" + se + "`";
            }
            ee(ae), G.exports = function(se, ce) {
              se = se || "";
              var le, ue = (ce = ce || {}).cLetter || "c", de = ("onlyIfNumerical" in ce && ce.onlyIfNumerical, "noScale" in ce ? ce.noScale : se === "marker.line"), fe = "showScaleDflt" in ce ? ce.showScaleDflt : ue === "z", he = typeof ce.colorscaleDflt == "string" ? ae[ce.colorscaleDflt] : null, me = ce.editTypeOverride || "", pe = se ? se + "." : "";
              "colorAttr" in ce ? (le = ce.colorAttr, ce.colorAttr) : oe(pe + (le = { z: "z", c: "color" }[ue]));
              var ye = ue + "auto", ge = ue + "min", ve = ue + "max", be = ue + "mid", Ce = {};
              Ce[ge] = Ce[ve] = void 0;
              var Se = {};
              Se[ye] = !1;
              var we = {};
              return le === "color" && (we.color = { valType: "color", arrayOk: !0, editType: me || "style" }, ce.anim && (we.color.anim = !0)), we[ye] = { valType: "boolean", dflt: !0, editType: "calc", impliedEdits: Ce }, we[ge] = { valType: "number", dflt: null, editType: me || "plot", impliedEdits: Se }, we[ve] = { valType: "number", dflt: null, editType: me || "plot", impliedEdits: Se }, we[be] = { valType: "number", dflt: null, editType: "calc", impliedEdits: Ce }, we.colorscale = { valType: "colorscale", editType: "calc", dflt: he, impliedEdits: { autocolorscale: !1 } }, we.autocolorscale = { valType: "boolean", dflt: ce.autoColorDflt !== !1, editType: "calc", impliedEdits: { colorscale: void 0 } }, we.reversescale = { valType: "boolean", dflt: !1, editType: "plot" }, de || (we.showscale = { valType: "boolean", dflt: fe, editType: "calc" }, we.colorbar = te), ce.noColorAxis || (we.coloraxis = { valType: "subplotid", regex: ne("coloraxis"), dflt: null, editType: "calc" }), we;
            };
          }, 28379: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(65477).extractOpts;
            G.exports = function(ae, oe, se) {
              var ce, le = ae._fullLayout, ue = se.vals, de = se.containerStr, fe = de ? ne.nestedProperty(oe, de).get() : oe, he = ee(fe), me = he.auto !== !1, pe = he.min, ye = he.max, ge = he.mid, ve = function() {
                return ne.aggNums(Math.min, null, ue);
              }, be = function() {
                return ne.aggNums(Math.max, null, ue);
              };
              pe === void 0 ? pe = ve() : me && (pe = fe._colorAx && te(pe) ? Math.min(pe, ve()) : ve()), ye === void 0 ? ye = be() : me && (ye = fe._colorAx && te(ye) ? Math.max(ye, be()) : be()), me && ge !== void 0 && (ye - ge > ge - pe ? pe = ge - (ye - ge) : ye - ge < ge - pe && (ye = ge + (ge - pe))), pe === ye && (pe -= 0.5, ye += 0.5), he._sync("min", pe), he._sync("max", ye), he.autocolorscale && (ce = pe * ye < 0 ? le.colorscale.diverging : pe >= 0 ? le.colorscale.sequential : le.colorscale.sequentialminus, he._sync("colorscale", ce));
            };
          }, 67623: function(G, J, Z) {
            var te = Z(34809), ne = Z(65477).hasColorscale, ee = Z(65477).extractOpts;
            G.exports = function(ae, oe) {
              function se(me, pe) {
                var ye = me["_" + pe];
                ye !== void 0 && (me[pe] = ye);
              }
              function ce(me, pe) {
                var ye = pe.container ? te.nestedProperty(me, pe.container).get() : me;
                if (ye) if (ye.coloraxis) ye._colorAx = oe[ye.coloraxis];
                else {
                  var ge = ee(ye), ve = ge.auto;
                  (ve || ge.min === void 0) && se(ye, pe.min), (ve || ge.max === void 0) && se(ye, pe.max), ge.autocolorscale && se(ye, "colorscale");
                }
              }
              for (var le = 0; le < ae.length; le++) {
                var ue = ae[le], de = ue._module.colorbar;
                if (de) if (Array.isArray(de)) for (var fe = 0; fe < de.length; fe++) ce(ue, de[fe]);
                else ce(ue, de);
                ne(ue, "marker.line") && ce(ue, { container: "marker.line", min: "cmin", max: "cmax" });
              }
              for (var he in oe._colorAxes) ce(oe[he], { min: "cmin", max: "cmax" });
            };
          }, 39356: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(91362), ae = Z(42097), oe = Z(19017).isValid, se = Z(33626).traceIs;
            function ce(le, ue) {
              var de = ue.slice(0, ue.length - 1);
              return ue ? ne.nestedProperty(le, de).get() || {} : le;
            }
            G.exports = function le(ue, de, fe, he, me) {
              var pe = me.prefix, ye = me.cLetter, ge = "_module" in de, ve = ce(ue, pe), be = ce(de, pe), Ce = ce(de._template || {}, pe) || {}, Se = function() {
                return delete ue.coloraxis, delete de.coloraxis, le(ue, de, fe, he, me);
              };
              if (ge) {
                var we = fe._colorAxes || {}, Ee = he(pe + "coloraxis");
                if (Ee) {
                  var xe = se(de, "contour") && ne.nestedProperty(de, "contours.coloring").get() || "heatmap", Le = we[Ee];
                  return void (Le ? (Le[2].push(Se), Le[0] !== xe && (Le[0] = !1, ne.warn(["Ignoring coloraxis:", Ee, "setting", "as it is linked to incompatible colorscales."].join(" ")))) : we[Ee] = [xe, de, [Se]]);
                }
              }
              var Qe = ve[ye + "min"], ke = ve[ye + "max"], Me = te(Qe) && te(ke) && Qe < ke;
              he(pe + ye + "auto", !Me) ? he(pe + ye + "mid") : (he(pe + ye + "min"), he(pe + ye + "max"));
              var Re, De, Pe = ve.colorscale, $e = Ce.colorscale;
              Pe !== void 0 && (Re = !oe(Pe)), $e !== void 0 && (Re = !oe($e)), he(pe + "autocolorscale", Re), he(pe + "colorscale"), he(pe + "reversescale"), pe !== "marker.line." && (pe && ge && (De = ee(ve)), he(pe + "showscale", De) && (pe && Ce && (be._template = Ce), ae(ve, be, fe)));
            };
          }, 65477: function(G, J, Z) {
            var te = Z(45568), ne = Z(65657), ee = Z(10721), ae = Z(34809), oe = Z(78766), se = Z(19017).isValid, ce = ["showscale", "autocolorscale", "colorscale", "reversescale", "colorbar"], le = ["min", "max", "mid", "auto"];
            function ue(pe) {
              var ye, ge, ve, be = pe._colorAx, Ce = be || pe, Se = {};
              for (ge = 0; ge < ce.length; ge++) Se[ve = ce[ge]] = Ce[ve];
              if (be) for (ye = "c", ge = 0; ge < le.length; ge++) Se[ve = le[ge]] = Ce["c" + ve];
              else {
                var we;
                for (ge = 0; ge < le.length; ge++) ((we = "c" + (ve = le[ge])) in Ce || (we = "z" + ve) in Ce) && (Se[ve] = Ce[we]);
                ye = we.charAt(0);
              }
              return Se._sync = function(Ee, xe) {
                var Le = le.indexOf(Ee) !== -1 ? ye + Ee : Ee;
                Ce[Le] = Ce["_" + Le] = xe;
              }, Se;
            }
            function de(pe) {
              for (var ye = ue(pe), ge = ye.min, ve = ye.max, be = ye.reversescale ? fe(ye.colorscale) : ye.colorscale, Ce = be.length, Se = new Array(Ce), we = new Array(Ce), Ee = 0; Ee < Ce; Ee++) {
                var xe = be[Ee];
                Se[Ee] = ge + xe[0] * (ve - ge), we[Ee] = xe[1];
              }
              return { domain: Se, range: we };
            }
            function fe(pe) {
              for (var ye = pe.length, ge = new Array(ye), ve = ye - 1, be = 0; ve >= 0; ve--, be++) {
                var Ce = pe[ve];
                ge[be] = [1 - Ce[0], Ce[1]];
              }
              return ge;
            }
            function he(pe, ye) {
              ye = ye || {};
              for (var ge = pe.domain, ve = pe.range, be = ve.length, Ce = new Array(be), Se = 0; Se < be; Se++) {
                var we = ne(ve[Se]).toRgb();
                Ce[Se] = [we.r, we.g, we.b, we.a];
              }
              var Ee, xe = te.scale.linear().domain(ge).range(Ce).clamp(!0), Le = ye.noNumericCheck, Qe = ye.returnArray;
              return (Ee = Le && Qe ? xe : Le ? function(ke) {
                return me(xe(ke));
              } : Qe ? function(ke) {
                return ee(ke) ? xe(ke) : ne(ke).isValid() ? ke : oe.defaultLine;
              } : function(ke) {
                return ee(ke) ? me(xe(ke)) : ne(ke).isValid() ? ke : oe.defaultLine;
              }).domain = xe.domain, Ee.range = function() {
                return ve;
              }, Ee;
            }
            function me(pe) {
              var ye = { r: pe[0], g: pe[1], b: pe[2], a: pe[3] };
              return ne(ye).toRgbString();
            }
            G.exports = { hasColorscale: function(pe, ye, ge) {
              var ve = ye ? ae.nestedProperty(pe, ye).get() || {} : pe, be = ve[ge || "color"];
              be && be._inputArray && (be = be._inputArray);
              var Ce = !1;
              if (ae.isArrayOrTypedArray(be)) {
                for (var Se = 0; Se < be.length; Se++) if (ee(be[Se])) {
                  Ce = !0;
                  break;
                }
              }
              return ae.isPlainObject(ve) && (Ce || ve.showscale === !0 || ee(ve.cmin) && ee(ve.cmax) || se(ve.colorscale) || ae.isPlainObject(ve.colorbar));
            }, extractOpts: ue, extractScale: de, flipScale: fe, makeColorScaleFunc: he, makeColorScaleFuncFromTrace: function(pe, ye) {
              return he(de(pe), ye);
            } };
          }, 88856: function(G, J, Z) {
            var te = Z(19017), ne = Z(65477);
            G.exports = { moduleType: "component", name: "colorscale", attributes: Z(87163), layoutAttributes: Z(56978), supplyLayoutDefaults: Z(64613), handleDefaults: Z(39356), crossTraceDefaults: Z(67623), calc: Z(28379), scales: te.scales, defaultScale: te.defaultScale, getScale: te.get, isValidScale: te.isValid, hasColorscale: ne.hasColorscale, extractOpts: ne.extractOpts, extractScale: ne.extractScale, flipScale: ne.flipScale, makeColorScaleFunc: ne.makeColorScaleFunc, makeColorScaleFuncFromTrace: ne.makeColorScaleFuncFromTrace };
          }, 56978: function(G, J, Z) {
            var te = Z(93049).extendFlat, ne = Z(87163), ee = Z(19017).scales;
            G.exports = { editType: "calc", colorscale: { editType: "calc", sequential: { valType: "colorscale", dflt: ee.Reds, editType: "calc" }, sequentialminus: { valType: "colorscale", dflt: ee.Blues, editType: "calc" }, diverging: { valType: "colorscale", dflt: ee.RdBu, editType: "calc" } }, coloraxis: te({ _isSubplotObj: !0, editType: "calc" }, ne("", { colorAttr: "corresponding trace color array(s)", noColorAxis: !0, showScaleDflt: !0 })) };
          }, 64613: function(G, J, Z) {
            var te = Z(34809), ne = Z(78032), ee = Z(56978), ae = Z(39356);
            G.exports = function(oe, se) {
              function ce(ye, ge) {
                return te.coerce(oe, se, ee, ye, ge);
              }
              ce("colorscale.sequential"), ce("colorscale.sequentialminus"), ce("colorscale.diverging");
              var le, ue, de = se._colorAxes;
              function fe(ye, ge) {
                return te.coerce(le, ue, ee.coloraxis, ye, ge);
              }
              for (var he in de) {
                var me = de[he];
                if (me[0]) le = oe[he] || {}, (ue = ne.newContainer(se, he, "coloraxis"))._name = he, ae(le, ue, se, fe, { prefix: "", cLetter: "c" });
                else {
                  for (var pe = 0; pe < me[2].length; pe++) me[2][pe]();
                  delete se._colorAxes[he];
                }
              }
            };
          }, 19017: function(G, J, Z) {
            var te = Z(65657), ne = { Greys: [[0, "rgb(0,0,0)"], [1, "rgb(255,255,255)"]], YlGnBu: [[0, "rgb(8,29,88)"], [0.125, "rgb(37,52,148)"], [0.25, "rgb(34,94,168)"], [0.375, "rgb(29,145,192)"], [0.5, "rgb(65,182,196)"], [0.625, "rgb(127,205,187)"], [0.75, "rgb(199,233,180)"], [0.875, "rgb(237,248,217)"], [1, "rgb(255,255,217)"]], Greens: [[0, "rgb(0,68,27)"], [0.125, "rgb(0,109,44)"], [0.25, "rgb(35,139,69)"], [0.375, "rgb(65,171,93)"], [0.5, "rgb(116,196,118)"], [0.625, "rgb(161,217,155)"], [0.75, "rgb(199,233,192)"], [0.875, "rgb(229,245,224)"], [1, "rgb(247,252,245)"]], YlOrRd: [[0, "rgb(128,0,38)"], [0.125, "rgb(189,0,38)"], [0.25, "rgb(227,26,28)"], [0.375, "rgb(252,78,42)"], [0.5, "rgb(253,141,60)"], [0.625, "rgb(254,178,76)"], [0.75, "rgb(254,217,118)"], [0.875, "rgb(255,237,160)"], [1, "rgb(255,255,204)"]], Bluered: [[0, "rgb(0,0,255)"], [1, "rgb(255,0,0)"]], RdBu: [[0, "rgb(5,10,172)"], [0.35, "rgb(106,137,247)"], [0.5, "rgb(190,190,190)"], [0.6, "rgb(220,170,132)"], [0.7, "rgb(230,145,90)"], [1, "rgb(178,10,28)"]], Reds: [[0, "rgb(220,220,220)"], [0.2, "rgb(245,195,157)"], [0.4, "rgb(245,160,105)"], [1, "rgb(178,10,28)"]], Blues: [[0, "rgb(5,10,172)"], [0.35, "rgb(40,60,190)"], [0.5, "rgb(70,100,245)"], [0.6, "rgb(90,120,245)"], [0.7, "rgb(106,137,247)"], [1, "rgb(220,220,220)"]], Picnic: [[0, "rgb(0,0,255)"], [0.1, "rgb(51,153,255)"], [0.2, "rgb(102,204,255)"], [0.3, "rgb(153,204,255)"], [0.4, "rgb(204,204,255)"], [0.5, "rgb(255,255,255)"], [0.6, "rgb(255,204,255)"], [0.7, "rgb(255,153,255)"], [0.8, "rgb(255,102,204)"], [0.9, "rgb(255,102,102)"], [1, "rgb(255,0,0)"]], Rainbow: [[0, "rgb(150,0,90)"], [0.125, "rgb(0,0,200)"], [0.25, "rgb(0,25,255)"], [0.375, "rgb(0,152,255)"], [0.5, "rgb(44,255,150)"], [0.625, "rgb(151,255,0)"], [0.75, "rgb(255,234,0)"], [0.875, "rgb(255,111,0)"], [1, "rgb(255,0,0)"]], Portland: [[0, "rgb(12,51,131)"], [0.25, "rgb(10,136,186)"], [0.5, "rgb(242,211,56)"], [0.75, "rgb(242,143,56)"], [1, "rgb(217,30,30)"]], Jet: [[0, "rgb(0,0,131)"], [0.125, "rgb(0,60,170)"], [0.375, "rgb(5,255,255)"], [0.625, "rgb(255,255,0)"], [0.875, "rgb(250,0,0)"], [1, "rgb(128,0,0)"]], Hot: [[0, "rgb(0,0,0)"], [0.3, "rgb(230,0,0)"], [0.6, "rgb(255,210,0)"], [1, "rgb(255,255,255)"]], Blackbody: [[0, "rgb(0,0,0)"], [0.2, "rgb(230,0,0)"], [0.4, "rgb(230,210,0)"], [0.7, "rgb(255,255,255)"], [1, "rgb(160,200,255)"]], Earth: [[0, "rgb(0,0,130)"], [0.1, "rgb(0,180,180)"], [0.2, "rgb(40,210,40)"], [0.4, "rgb(230,230,50)"], [0.6, "rgb(120,70,20)"], [1, "rgb(255,255,255)"]], Electric: [[0, "rgb(0,0,0)"], [0.15, "rgb(30,0,100)"], [0.4, "rgb(120,0,100)"], [0.6, "rgb(160,90,0)"], [0.8, "rgb(230,200,0)"], [1, "rgb(255,250,220)"]], Viridis: [[0, "#440154"], [0.06274509803921569, "#48186a"], [0.12549019607843137, "#472d7b"], [0.18823529411764706, "#424086"], [0.25098039215686274, "#3b528b"], [0.3137254901960784, "#33638d"], [0.3764705882352941, "#2c728e"], [0.4392156862745098, "#26828e"], [0.5019607843137255, "#21918c"], [0.5647058823529412, "#1fa088"], [0.6274509803921569, "#28ae80"], [0.6901960784313725, "#3fbc73"], [0.7529411764705882, "#5ec962"], [0.8156862745098039, "#84d44b"], [0.8784313725490196, "#addc30"], [0.9411764705882353, "#d8e219"], [1, "#fde725"]], Cividis: [[0, "rgb(0,32,76)"], [0.058824, "rgb(0,42,102)"], [0.117647, "rgb(0,52,110)"], [0.176471, "rgb(39,63,108)"], [0.235294, "rgb(60,74,107)"], [0.294118, "rgb(76,85,107)"], [0.352941, "rgb(91,95,109)"], [0.411765, "rgb(104,106,112)"], [0.470588, "rgb(117,117,117)"], [0.529412, "rgb(131,129,120)"], [0.588235, "rgb(146,140,120)"], [0.647059, "rgb(161,152,118)"], [0.705882, "rgb(176,165,114)"], [0.764706, "rgb(192,177,109)"], [0.823529, "rgb(209,191,102)"], [0.882353, "rgb(225,204,92)"], [0.941176, "rgb(243,219,79)"], [1, "rgb(255,233,69)"]] }, ee = ne.RdBu;
            function ae(oe) {
              var se = 0;
              if (!Array.isArray(oe) || oe.length < 2 || !oe[0] || !oe[oe.length - 1] || +oe[0][0] != 0 || +oe[oe.length - 1][0] != 1) return !1;
              for (var ce = 0; ce < oe.length; ce++) {
                var le = oe[ce];
                if (le.length !== 2 || +le[0] < se || !te(le[1]).isValid()) return !1;
                se = +le[0];
              }
              return !0;
            }
            G.exports = { scales: ne, defaultScale: ee, get: function(oe, se) {
              if (se || (se = ee), !oe) return se;
              function ce() {
                try {
                  oe = ne[oe] || JSON.parse(oe);
                } catch {
                  oe = se;
                }
              }
              return typeof oe == "string" && (ce(), typeof oe == "string" && ce()), ae(oe) ? oe : se;
            }, isValid: function(oe) {
              return ne[oe] !== void 0 || ae(oe);
            } };
          }, 53770: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              var ae = (J - te) / (ne - te), oe = ae + Z / (ne - te), se = (ae + oe) / 2;
              return ee === "left" || ee === "bottom" ? ae : ee === "center" || ee === "middle" ? se : ee === "right" || ee === "top" ? oe : ae < 0.6666666666666666 - se ? ae : oe > 1.3333333333333333 - se ? oe : se;
            };
          }, 4001: function(G, J, Z) {
            var te = Z(34809), ne = [["sw-resize", "s-resize", "se-resize"], ["w-resize", "move", "e-resize"], ["nw-resize", "n-resize", "ne-resize"]];
            G.exports = function(ee, ae, oe, se) {
              return ee = oe === "left" ? 0 : oe === "center" ? 1 : oe === "right" ? 2 : te.constrain(Math.floor(3 * ee), 0, 2), ae = se === "bottom" ? 0 : se === "middle" ? 1 : se === "top" ? 2 : te.constrain(Math.floor(3 * ae), 0, 2), ne[ae][ee];
            };
          }, 70414: function(G, J) {
            J.selectMode = function(Z) {
              return Z === "lasso" || Z === "select";
            }, J.drawMode = function(Z) {
              return Z === "drawclosedpath" || Z === "drawopenpath" || Z === "drawline" || Z === "drawrect" || Z === "drawcircle";
            }, J.openMode = function(Z) {
              return Z === "drawline" || Z === "drawopenpath";
            }, J.rectMode = function(Z) {
              return Z === "select" || Z === "drawline" || Z === "drawrect" || Z === "drawcircle";
            }, J.freeMode = function(Z) {
              return Z === "lasso" || Z === "drawclosedpath" || Z === "drawopenpath";
            }, J.selectingOrDrawing = function(Z) {
              return J.freeMode(Z) || J.rectMode(Z);
            };
          }, 14751: function(G, J, Z) {
            var te = Z(44039), ne = Z(39784), ee = Z(74043), ae = Z(34809).removeElement, oe = Z(54826), se = G.exports = {};
            se.align = Z(53770), se.getCursor = Z(4001);
            var ce = Z(60148);
            function le() {
              var de = document.createElement("div");
              de.className = "dragcover";
              var fe = de.style;
              return fe.position = "fixed", fe.left = 0, fe.right = 0, fe.top = 0, fe.bottom = 0, fe.zIndex = 999999999, fe.background = "none", document.body.appendChild(de), de;
            }
            function ue(de) {
              return te(de.changedTouches ? de.changedTouches[0] : de, document.body);
            }
            se.unhover = ce.wrapped, se.unhoverRaw = ce.raw, se.init = function(de) {
              var fe, he, me, pe, ye, ge, ve, be, Ce = de.gd, Se = 1, we = Ce._context.doubleClickDelay, Ee = de.element;
              Ce._mouseDownTime || (Ce._mouseDownTime = 0), Ee.style.pointerEvents = "all", Ee.onmousedown = Le, ee ? (Ee._ontouchstart && Ee.removeEventListener("touchstart", Ee._ontouchstart), Ee._ontouchstart = Le, Ee.addEventListener("touchstart", Le, { passive: !1 })) : Ee.ontouchstart = Le;
              var xe = de.clampFn || function(Me, Re, De) {
                return Math.abs(Me) < De && (Me = 0), Math.abs(Re) < De && (Re = 0), [Me, Re];
              };
              function Le(Me) {
                Ce._dragged = !1, Ce._dragging = !0;
                var Re = ue(Me);
                fe = Re[0], he = Re[1], ve = Me.target, ge = Me, be = Me.buttons === 2 || Me.ctrlKey, Me.clientX === void 0 && Me.clientY === void 0 && (Me.clientX = fe, Me.clientY = he), (me = (/* @__PURE__ */ new Date()).getTime()) - Ce._mouseDownTime < we ? Se += 1 : (Se = 1, Ce._mouseDownTime = me), de.prepFn && de.prepFn(Me, fe, he), ne && !be ? (ye = le()).style.cursor = window.getComputedStyle(Ee).cursor : ne || (ye = document, pe = window.getComputedStyle(document.documentElement).cursor, document.documentElement.style.cursor = window.getComputedStyle(Ee).cursor), document.addEventListener("mouseup", ke), document.addEventListener("touchend", ke), de.dragmode !== !1 && (Me.preventDefault(), document.addEventListener("mousemove", Qe), document.addEventListener("touchmove", Qe, { passive: !1 }));
              }
              function Qe(Me) {
                Me.preventDefault();
                var Re = ue(Me), De = de.minDrag || oe.MINDRAG, Pe = xe(Re[0] - fe, Re[1] - he, De), $e = Pe[0], qe = Pe[1];
                ($e || qe) && (Ce._dragged = !0, se.unhover(Ce, Me)), Ce._dragged && de.moveFn && !be && (Ce._dragdata = { element: Ee, dx: $e, dy: qe }, de.moveFn($e, qe));
              }
              function ke(Me) {
                if (delete Ce._dragdata, de.dragmode !== !1 && (Me.preventDefault(), document.removeEventListener("mousemove", Qe), document.removeEventListener("touchmove", Qe)), document.removeEventListener("mouseup", ke), document.removeEventListener("touchend", ke), ne ? ae(ye) : pe && (ye.documentElement.style.cursor = pe, pe = null), Ce._dragging) {
                  if (Ce._dragging = !1, (/* @__PURE__ */ new Date()).getTime() - Ce._mouseDownTime > we && (Se = Math.max(Se - 1, 1)), Ce._dragged) de.doneFn && de.doneFn();
                  else if (de.clickFn && de.clickFn(Se, ge), !be) {
                    var Re;
                    try {
                      Re = new MouseEvent("click", Me);
                    } catch {
                      var De = ue(Me);
                      (Re = document.createEvent("MouseEvents")).initMouseEvent("click", Me.bubbles, Me.cancelable, Me.view, Me.detail, Me.screenX, Me.screenY, De[0], De[1], Me.ctrlKey, Me.altKey, Me.shiftKey, Me.metaKey, Me.button, Me.relatedTarget);
                    }
                    ve.dispatchEvent(Re);
                  }
                  Ce._dragging = !1, Ce._dragged = !1;
                } else Ce._dragged = !1;
              }
            }, se.coverSlip = le;
          }, 60148: function(G, J, Z) {
            var te = Z(68596), ne = Z(64025), ee = Z(95425).getGraphDiv, ae = Z(85988), oe = G.exports = {};
            oe.wrapped = function(se, ce, le) {
              (se = ee(se))._fullLayout && ne.clear(se._fullLayout._uid + ae.HOVERID), oe.raw(se, ce, le);
            }, oe.raw = function(se, ce) {
              var le = se._fullLayout, ue = se._hoverdata;
              ce || (ce = {}), ce.target && !se._dragged && te.triggerHandler(se, "plotly_beforehover", ce) === !1 || (le._hoverlayer.selectAll("g").remove(), le._hoverlayer.selectAll("line").remove(), le._hoverlayer.selectAll("circle").remove(), se._hoverdata = void 0, ce.target && ue && se.emit("plotly_unhover", { event: ce, points: ue }));
            };
          }, 94850: function(G, J) {
            J.T = { valType: "string", values: ["solid", "dot", "dash", "longdash", "dashdot", "longdashdot"], dflt: "solid", editType: "style" }, J.k = { shape: { valType: "enumerated", values: ["", "/", "\\", "x", "-", "|", "+", "."], dflt: "", arrayOk: !0, editType: "style" }, fillmode: { valType: "enumerated", values: ["replace", "overlay"], dflt: "replace", editType: "style" }, bgcolor: { valType: "color", arrayOk: !0, editType: "style" }, fgcolor: { valType: "color", arrayOk: !0, editType: "style" }, fgopacity: { valType: "number", editType: "style", min: 0, max: 1 }, size: { valType: "number", min: 0, dflt: 8, arrayOk: !0, editType: "style" }, solidity: { valType: "number", min: 0, max: 1, dflt: 0.3, arrayOk: !0, editType: "style" }, editType: "style" };
          }, 62203: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = ne.numberFormat, ae = Z(10721), oe = Z(65657), se = Z(33626), ce = Z(78766), le = Z(88856), ue = ne.strTranslate, de = Z(30635), fe = Z(62972), he = Z(4530).LINE_SPACING, me = Z(20438).DESELECTDIM, pe = Z(64726), ye = Z(92527), ge = Z(36040).appendArrayPointValue, ve = G.exports = {};
            function be(Nt) {
              return Nt === "none" ? void 0 : Nt;
            }
            ve.font = function(Nt, Gt) {
              var Ft = Gt.variant, Tt = Gt.style, It = Gt.weight, Ct = Gt.color, Kt = Gt.size, Vt = Gt.family, Yt = Gt.shadow, sn = Gt.lineposition, hn = Gt.textcase;
              Vt && Nt.style("font-family", Vt), Kt + 1 && Nt.style("font-size", Kt + "px"), Ct && Nt.call(ce.fill, Ct), It && Nt.style("font-weight", It), Tt && Nt.style("font-style", Tt), Ft && Nt.style("font-variant", Ft), hn && Nt.style("text-transform", be(function(Qt) {
                return Ce[Qt];
              }(hn))), Yt && Nt.style("text-shadow", Yt === "auto" ? de.makeTextShadow(ce.contrast(Ct)) : be(Yt)), sn && Nt.style("text-decoration-line", be(function(Qt) {
                return Qt.replace("under", "underline").replace("over", "overline").replace("through", "line-through").split("+").join(" ");
              }(sn)));
            };
            var Ce = { normal: "none", lower: "lowercase", upper: "uppercase", "word caps": "capitalize" };
            function Se(Nt, Gt, Ft, Tt) {
              var It = Gt.fillpattern, Ct = Gt.fillgradient, Kt = It && ve.getPatternAttr(It.shape, 0, "");
              if (Kt) {
                var Vt = ve.getPatternAttr(It.bgcolor, 0, null), Yt = ve.getPatternAttr(It.fgcolor, 0, null), sn = It.fgopacity, hn = ve.getPatternAttr(It.size, 0, 8), Qt = ve.getPatternAttr(It.solidity, 0, 0.3), Ut = Gt.uid;
                ve.pattern(Nt, "point", Ft, Ut, Kt, hn, Qt, void 0, It.fillmode, Vt, Yt, sn);
              } else if (Ct && Ct.type !== "none") {
                var Wt, rn, vn = Ct.type, dn = "scatterfill-" + Gt.uid;
                Tt && (dn = "legendfill-" + Gt.uid), Tt || Ct.start === void 0 && Ct.stop === void 0 ? (vn === "horizontal" && (vn += "reversed"), Nt.call(ve.gradient, Ft, dn, vn, Ct.colorscale, "fill")) : (vn === "horizontal" ? (Wt = { x: Ct.start, y: 0 }, rn = { x: Ct.stop, y: 0 }) : vn === "vertical" && (Wt = { x: 0, y: Ct.start }, rn = { x: 0, y: Ct.stop }), Wt.x = Gt._xA.c2p(Wt.x === void 0 ? Gt._extremes.x.min[0].val : Wt.x, !0), Wt.y = Gt._yA.c2p(Wt.y === void 0 ? Gt._extremes.y.min[0].val : Wt.y, !0), rn.x = Gt._xA.c2p(rn.x === void 0 ? Gt._extremes.x.max[0].val : rn.x, !0), rn.y = Gt._yA.c2p(rn.y === void 0 ? Gt._extremes.y.max[0].val : rn.y, !0), Nt.call(ke, Ft, dn, "linear", Ct.colorscale, "fill", Wt, rn, !0, !1));
              } else Gt.fillcolor && Nt.call(ce.fill, Gt.fillcolor);
            }
            ve.setPosition = function(Nt, Gt, Ft) {
              Nt.attr("x", Gt).attr("y", Ft);
            }, ve.setSize = function(Nt, Gt, Ft) {
              Nt.attr("width", Gt).attr("height", Ft);
            }, ve.setRect = function(Nt, Gt, Ft, Tt, It) {
              Nt.call(ve.setPosition, Gt, Ft).call(ve.setSize, Tt, It);
            }, ve.translatePoint = function(Nt, Gt, Ft, Tt) {
              var It = Ft.c2p(Nt.x), Ct = Tt.c2p(Nt.y);
              return !!(ae(It) && ae(Ct) && Gt.node()) && (Gt.node().nodeName === "text" ? Gt.attr("x", It).attr("y", Ct) : Gt.attr("transform", ue(It, Ct)), !0);
            }, ve.translatePoints = function(Nt, Gt, Ft) {
              Nt.each(function(Tt) {
                var It = te.select(this);
                ve.translatePoint(Tt, It, Gt, Ft);
              });
            }, ve.hideOutsideRangePoint = function(Nt, Gt, Ft, Tt, It, Ct) {
              Gt.attr("display", Ft.isPtWithinRange(Nt, It) && Tt.isPtWithinRange(Nt, Ct) ? null : "none");
            }, ve.hideOutsideRangePoints = function(Nt, Gt) {
              if (Gt._hasClipOnAxisFalse) {
                var Ft = Gt.xaxis, Tt = Gt.yaxis;
                Nt.each(function(It) {
                  var Ct = It[0].trace, Kt = Ct.xcalendar, Vt = Ct.ycalendar, Yt = se.traceIs(Ct, "bar-like") ? ".bartext" : ".point,.textpoint";
                  Nt.selectAll(Yt).each(function(sn) {
                    ve.hideOutsideRangePoint(sn, te.select(this), Ft, Tt, Kt, Vt);
                  });
                });
              }
            }, ve.crispRound = function(Nt, Gt, Ft) {
              return Gt && ae(Gt) ? Nt._context.staticPlot ? Gt : Gt < 1 ? 1 : Math.round(Gt) : Ft || 0;
            }, ve.singleLineStyle = function(Nt, Gt, Ft, Tt, It) {
              Gt.style("fill", "none");
              var Ct = (((Nt || [])[0] || {}).trace || {}).line || {}, Kt = Ft || Ct.width || 0, Vt = It || Ct.dash || "";
              ce.stroke(Gt, Tt || Ct.color), ve.dashLine(Gt, Vt, Kt);
            }, ve.lineGroupStyle = function(Nt, Gt, Ft, Tt) {
              Nt.style("fill", "none").each(function(It) {
                var Ct = (((It || [])[0] || {}).trace || {}).line || {}, Kt = Gt || Ct.width || 0, Vt = Tt || Ct.dash || "";
                te.select(this).call(ce.stroke, Ft || Ct.color).call(ve.dashLine, Vt, Kt);
              });
            }, ve.dashLine = function(Nt, Gt, Ft) {
              Ft = +Ft || 0, Gt = ve.dashStyle(Gt, Ft), Nt.style({ "stroke-dasharray": Gt, "stroke-width": Ft + "px" });
            }, ve.dashStyle = function(Nt, Gt) {
              Gt = +Gt || 1;
              var Ft = Math.max(Gt, 3);
              return Nt === "solid" ? Nt = "" : Nt === "dot" ? Nt = Ft + "px," + Ft + "px" : Nt === "dash" ? Nt = 3 * Ft + "px," + 3 * Ft + "px" : Nt === "longdash" ? Nt = 5 * Ft + "px," + 5 * Ft + "px" : Nt === "dashdot" ? Nt = 3 * Ft + "px," + Ft + "px," + Ft + "px," + Ft + "px" : Nt === "longdashdot" && (Nt = 5 * Ft + "px," + 2 * Ft + "px," + Ft + "px," + 2 * Ft + "px"), Nt;
            }, ve.singleFillStyle = function(Nt, Gt) {
              var Ft = te.select(Nt.node());
              Se(Nt, ((Ft.data()[0] || [])[0] || {}).trace || {}, Gt, !1);
            }, ve.fillGroupStyle = function(Nt, Gt, Ft) {
              Nt.style("stroke-width", 0).each(function(Tt) {
                var It = te.select(this);
                Tt[0].trace && Se(It, Tt[0].trace, Gt, Ft);
              });
            };
            var we = Z(38882);
            ve.symbolNames = [], ve.symbolFuncs = [], ve.symbolBackOffs = [], ve.symbolNeedLines = {}, ve.symbolNoDot = {}, ve.symbolNoFill = {}, ve.symbolList = [], Object.keys(we).forEach(function(Nt) {
              var Gt = we[Nt], Ft = Gt.n;
              ve.symbolList.push(Ft, String(Ft), Nt, Ft + 100, String(Ft + 100), Nt + "-open"), ve.symbolNames[Ft] = Nt, ve.symbolFuncs[Ft] = Gt.f, ve.symbolBackOffs[Ft] = Gt.backoff || 0, Gt.needLine && (ve.symbolNeedLines[Ft] = !0), Gt.noDot ? ve.symbolNoDot[Ft] = !0 : ve.symbolList.push(Ft + 200, String(Ft + 200), Nt + "-dot", Ft + 300, String(Ft + 300), Nt + "-open-dot"), Gt.noFill && (ve.symbolNoFill[Ft] = !0);
            });
            var Ee = ve.symbolNames.length;
            function xe(Nt, Gt, Ft, Tt) {
              var It = Nt % 100;
              return ve.symbolFuncs[It](Gt, Ft, Tt) + (Nt >= 200 ? "M0,0.5L0.5,0L0,-0.5L-0.5,0Z" : "");
            }
            ve.symbolNumber = function(Nt) {
              if (ae(Nt)) Nt = +Nt;
              else if (typeof Nt == "string") {
                var Gt = 0;
                Nt.indexOf("-open") > 0 && (Gt = 100, Nt = Nt.replace("-open", "")), Nt.indexOf("-dot") > 0 && (Gt += 200, Nt = Nt.replace("-dot", "")), (Nt = ve.symbolNames.indexOf(Nt)) >= 0 && (Nt += Gt);
              }
              return Nt % 100 >= Ee || Nt >= 400 ? 0 : Math.floor(Math.max(Nt, 0));
            };
            var Le = ee("~f"), Qe = { radial: { type: "radial" }, radialreversed: { type: "radial", reversed: !0 }, horizontal: { type: "linear", start: { x: 1, y: 0 }, stop: { x: 0, y: 0 } }, horizontalreversed: { type: "linear", start: { x: 1, y: 0 }, stop: { x: 0, y: 0 }, reversed: !0 }, vertical: { type: "linear", start: { x: 0, y: 1 }, stop: { x: 0, y: 0 } }, verticalreversed: { type: "linear", start: { x: 0, y: 1 }, stop: { x: 0, y: 0 }, reversed: !0 } };
            function ke(Nt, Gt, Ft, Tt, It, Ct, Kt, Vt, Yt, sn) {
              var hn, Qt = It.length;
              Tt === "linear" ? hn = { node: "linearGradient", attrs: { x1: Kt.x, y1: Kt.y, x2: Vt.x, y2: Vt.y, gradientUnits: Yt ? "userSpaceOnUse" : "objectBoundingBox" }, reversed: sn } : Tt === "radial" && (hn = { node: "radialGradient", reversed: sn });
              for (var Ut = new Array(Qt), Wt = 0; Wt < Qt; Wt++) hn.reversed ? Ut[Qt - 1 - Wt] = [Le(100 * (1 - It[Wt][0])), It[Wt][1]] : Ut[Wt] = [Le(100 * It[Wt][0]), It[Wt][1]];
              var rn = Gt._fullLayout, vn = "g" + rn._uid + "-" + Ft, dn = rn._defs.select(".gradients").selectAll("#" + vn).data([Tt + Ut.join(";")], ne.identity);
              dn.exit().remove(), dn.enter().append(hn.node).each(function() {
                var Ht = te.select(this);
                hn.attrs && Ht.attr(hn.attrs), Ht.attr("id", vn);
                var jt = Ht.selectAll("stop").data(Ut);
                jt.exit().remove(), jt.enter().append("stop"), jt.each(function(cn) {
                  var wn = oe(cn[1]);
                  te.select(this).attr({ offset: cn[0] + "%", "stop-color": ce.tinyRGB(wn), "stop-opacity": wn.getAlpha() });
                });
              }), Nt.style(Ct, et(vn, Gt)).style(Ct + "-opacity", null), Nt.classed("gradient_filled", !0);
            }
            ve.gradient = function(Nt, Gt, Ft, Tt, It, Ct) {
              var Kt = Qe[Tt];
              return ke(Nt, Gt, Ft, Kt.type, It, Ct, Kt.start, Kt.stop, !1, Kt.reversed);
            }, ve.pattern = function(Nt, Gt, Ft, Tt, It, Ct, Kt, Vt, Yt, sn, hn, Qt) {
              var Ut = Gt === "legend";
              Vt && (Yt === "overlay" ? (sn = Vt, hn = ce.contrast(sn)) : (sn = void 0, hn = Vt));
              var Wt, rn, vn, dn, Ht, jt, cn, wn, Rn, Pn = Ft._fullLayout, Ie = "p" + Pn._uid + "-" + Tt, rt = {}, ft = oe(hn), pt = ce.tinyRGB(ft), wt = Qt * ft.getAlpha();
              switch (It) {
                case "/":
                  Wt = Ct * Math.sqrt(2), rn = Ct * Math.sqrt(2), jt = "path", rt = { d: vn = "M-" + Wt / 4 + "," + rn / 4 + "l" + Wt / 2 + ",-" + rn / 2 + "M0," + rn + "L" + Wt + ",0M" + Wt / 4 * 3 + "," + rn / 4 * 5 + "l" + Wt / 2 + ",-" + rn / 2, opacity: wt, stroke: pt, "stroke-width": (dn = Kt * Ct) + "px" };
                  break;
                case "\\":
                  Wt = Ct * Math.sqrt(2), rn = Ct * Math.sqrt(2), jt = "path", rt = { d: vn = "M" + Wt / 4 * 3 + ",-" + rn / 4 + "l" + Wt / 2 + "," + rn / 2 + "M0,0L" + Wt + "," + rn + "M-" + Wt / 4 + "," + rn / 4 * 3 + "l" + Wt / 2 + "," + rn / 2, opacity: wt, stroke: pt, "stroke-width": (dn = Kt * Ct) + "px" };
                  break;
                case "x":
                  Wt = Ct * Math.sqrt(2), rn = Ct * Math.sqrt(2), vn = "M-" + Wt / 4 + "," + rn / 4 + "l" + Wt / 2 + ",-" + rn / 2 + "M0," + rn + "L" + Wt + ",0M" + Wt / 4 * 3 + "," + rn / 4 * 5 + "l" + Wt / 2 + ",-" + rn / 2 + "M" + Wt / 4 * 3 + ",-" + rn / 4 + "l" + Wt / 2 + "," + rn / 2 + "M0,0L" + Wt + "," + rn + "M-" + Wt / 4 + "," + rn / 4 * 3 + "l" + Wt / 2 + "," + rn / 2, dn = Ct - Ct * Math.sqrt(1 - Kt), jt = "path", rt = { d: vn, opacity: wt, stroke: pt, "stroke-width": dn + "px" };
                  break;
                case "|":
                  jt = "path", jt = "path", rt = { d: vn = "M" + (Wt = Ct) / 2 + ",0L" + Wt / 2 + "," + (rn = Ct), opacity: wt, stroke: pt, "stroke-width": (dn = Kt * Ct) + "px" };
                  break;
                case "-":
                  jt = "path", jt = "path", rt = { d: vn = "M0," + (rn = Ct) / 2 + "L" + (Wt = Ct) + "," + rn / 2, opacity: wt, stroke: pt, "stroke-width": (dn = Kt * Ct) + "px" };
                  break;
                case "+":
                  jt = "path", vn = "M" + (Wt = Ct) / 2 + ",0L" + Wt / 2 + "," + (rn = Ct) + "M0," + rn / 2 + "L" + Wt + "," + rn / 2, dn = Ct - Ct * Math.sqrt(1 - Kt), jt = "path", rt = { d: vn, opacity: wt, stroke: pt, "stroke-width": dn + "px" };
                  break;
                case ".":
                  Wt = Ct, rn = Ct, Kt < Math.PI / 4 ? Ht = Math.sqrt(Kt * Ct * Ct / Math.PI) : (cn = Kt, wn = Math.PI / 4, Ht = (Rn = Ct / 2) + (Ct / Math.sqrt(2) - Rn) * (cn - wn) / (1 - wn)), jt = "circle", rt = { cx: Wt / 2, cy: rn / 2, r: Ht, opacity: wt, fill: pt };
              }
              var Bt = [It || "noSh", sn || "noBg", hn || "noFg", Ct, Kt].join(";"), At = Pn._defs.select(".patterns").selectAll("#" + Ie).data([Bt], ne.identity);
              At.exit().remove(), At.enter().append("pattern").each(function() {
                var Dt = te.select(this);
                if (Dt.attr({ id: Ie, width: Wt + "px", height: rn + "px", patternUnits: "userSpaceOnUse", patternTransform: Ut ? "scale(0.8)" : "" }), sn) {
                  var Ot = oe(sn), un = ce.tinyRGB(Ot), yn = Ot.getAlpha(), _n = Dt.selectAll("rect").data([0]);
                  _n.exit().remove(), _n.enter().append("rect").attr({ width: Wt + "px", height: rn + "px", fill: un, "fill-opacity": yn });
                }
                var gn = Dt.selectAll(jt).data([0]);
                gn.exit().remove(), gn.enter().append(jt).attr(rt);
              }), Nt.style("fill", et(Ie, Ft)).style("fill-opacity", null), Nt.classed("pattern_filled", !0);
            }, ve.initGradients = function(Nt) {
              var Gt = Nt._fullLayout;
              ne.ensureSingle(Gt._defs, "g", "gradients").selectAll("linearGradient,radialGradient").remove(), te.select(Nt).selectAll(".gradient_filled").classed("gradient_filled", !1);
            }, ve.initPatterns = function(Nt) {
              var Gt = Nt._fullLayout;
              ne.ensureSingle(Gt._defs, "g", "patterns").selectAll("pattern").remove(), te.select(Nt).selectAll(".pattern_filled").classed("pattern_filled", !1);
            }, ve.getPatternAttr = function(Nt, Gt, Ft) {
              return Nt && ne.isArrayOrTypedArray(Nt) ? Gt < Nt.length ? Nt[Gt] : Ft : Nt;
            }, ve.pointStyle = function(Nt, Gt, Ft, Tt) {
              if (Nt.size()) {
                var It = ve.makePointStyleFns(Gt);
                Nt.each(function(Ct) {
                  ve.singlePointStyle(Ct, te.select(this), Gt, It, Ft, Tt);
                });
              }
            }, ve.singlePointStyle = function(Nt, Gt, Ft, Tt, It, Ct) {
              var Kt = Ft.marker, Vt = Kt.line;
              if (Ct && Ct.i >= 0 && Nt.i === void 0 && (Nt.i = Ct.i), Gt.style("opacity", Tt.selectedOpacityFn ? Tt.selectedOpacityFn(Nt) : Nt.mo === void 0 ? Kt.opacity : Nt.mo), Tt.ms2mrc) {
                var Yt;
                Yt = Nt.ms === "various" || Kt.size === "various" ? 3 : Tt.ms2mrc(Nt.ms), Nt.mrc = Yt, Tt.selectedSizeFn && (Yt = Nt.mrc = Tt.selectedSizeFn(Nt));
                var sn = ve.symbolNumber(Nt.mx || Kt.symbol) || 0;
                Nt.om = sn % 200 >= 100;
                var hn = Xt(Nt, Ft), Qt = it(Nt, Ft);
                Gt.attr("d", xe(sn, Yt, hn, Qt));
              }
              var Ut, Wt, rn, vn = !1;
              if (Nt.so) rn = Vt.outlierwidth, Wt = Vt.outliercolor, Ut = Kt.outliercolor;
              else {
                var dn = (Vt || {}).width;
                rn = (Nt.mlw + 1 || dn + 1 || (Nt.trace ? (Nt.trace.marker.line || {}).width : 0) + 1) - 1 || 0, Wt = "mlc" in Nt ? Nt.mlcc = Tt.lineScale(Nt.mlc) : ne.isArrayOrTypedArray(Vt.color) ? ce.defaultLine : Vt.color, ne.isArrayOrTypedArray(Kt.color) && (Ut = ce.defaultLine, vn = !0), Ut = "mc" in Nt ? Nt.mcc = Tt.markerScale(Nt.mc) : Kt.color || Kt.colors || "rgba(0,0,0,0)", Tt.selectedColorFn && (Ut = Tt.selectedColorFn(Nt));
              }
              if (Nt.om) Gt.call(ce.stroke, Ut).style({ "stroke-width": (rn || 1) + "px", fill: "none" });
              else {
                Gt.style("stroke-width", (Nt.isBlank ? 0 : rn) + "px");
                var Ht = Kt.gradient, jt = Nt.mgt;
                jt ? vn = !0 : jt = Ht && Ht.type, ne.isArrayOrTypedArray(jt) && (jt = jt[0], Qe[jt] || (jt = 0));
                var cn = Kt.pattern, wn = cn && ve.getPatternAttr(cn.shape, Nt.i, "");
                if (jt && jt !== "none") {
                  var Rn = Nt.mgc;
                  Rn ? vn = !0 : Rn = Ht.color;
                  var Pn = Ft.uid;
                  vn && (Pn += "-" + Nt.i), ve.gradient(Gt, It, Pn, jt, [[0, Rn], [1, Ut]], "fill");
                } else if (wn) {
                  var Ie = !1, rt = cn.fgcolor;
                  !rt && Ct && Ct.color && (rt = Ct.color, Ie = !0);
                  var ft = ve.getPatternAttr(rt, Nt.i, Ct && Ct.color || null), pt = ve.getPatternAttr(cn.bgcolor, Nt.i, null), wt = cn.fgopacity, Bt = ve.getPatternAttr(cn.size, Nt.i, 8), At = ve.getPatternAttr(cn.solidity, Nt.i, 0.3);
                  Ie = Ie || Nt.mcc || ne.isArrayOrTypedArray(cn.shape) || ne.isArrayOrTypedArray(cn.bgcolor) || ne.isArrayOrTypedArray(cn.fgcolor) || ne.isArrayOrTypedArray(cn.size) || ne.isArrayOrTypedArray(cn.solidity);
                  var Dt = Ft.uid;
                  Ie && (Dt += "-" + Nt.i), ve.pattern(Gt, "point", It, Dt, wn, Bt, At, Nt.mcc, cn.fillmode, pt, ft, wt);
                } else ne.isArrayOrTypedArray(Ut) ? ce.fill(Gt, Ut[Nt.i]) : ce.fill(Gt, Ut);
                rn && ce.stroke(Gt, Wt);
              }
            }, ve.makePointStyleFns = function(Nt) {
              var Gt = {}, Ft = Nt.marker;
              return Gt.markerScale = ve.tryColorscale(Ft, ""), Gt.lineScale = ve.tryColorscale(Ft, "line"), se.traceIs(Nt, "symbols") && (Gt.ms2mrc = pe.isBubble(Nt) ? ye(Nt) : function() {
                return (Ft.size || 6) / 2;
              }), Nt.selectedpoints && ne.extendFlat(Gt, ve.makeSelectedPointStyleFns(Nt)), Gt;
            }, ve.makeSelectedPointStyleFns = function(Nt) {
              var Gt = {}, Ft = Nt.selected || {}, Tt = Nt.unselected || {}, It = Nt.marker || {}, Ct = Ft.marker || {}, Kt = Tt.marker || {}, Vt = It.opacity, Yt = Ct.opacity, sn = Kt.opacity, hn = Yt !== void 0, Qt = sn !== void 0;
              (ne.isArrayOrTypedArray(Vt) || hn || Qt) && (Gt.selectedOpacityFn = function(wn) {
                var Rn = wn.mo === void 0 ? It.opacity : wn.mo;
                return wn.selected ? hn ? Yt : Rn : Qt ? sn : me * Rn;
              });
              var Ut = It.color, Wt = Ct.color, rn = Kt.color;
              (Wt || rn) && (Gt.selectedColorFn = function(wn) {
                var Rn = wn.mcc || Ut;
                return wn.selected ? Wt || Rn : rn || Rn;
              });
              var vn = It.size, dn = Ct.size, Ht = Kt.size, jt = dn !== void 0, cn = Ht !== void 0;
              return se.traceIs(Nt, "symbols") && (jt || cn) && (Gt.selectedSizeFn = function(wn) {
                var Rn = wn.mrc || vn / 2;
                return wn.selected ? jt ? dn / 2 : Rn : cn ? Ht / 2 : Rn;
              }), Gt;
            }, ve.makeSelectedTextStyleFns = function(Nt) {
              var Gt = {}, Ft = Nt.selected || {}, Tt = Nt.unselected || {}, It = Nt.textfont || {}, Ct = Ft.textfont || {}, Kt = Tt.textfont || {}, Vt = It.color, Yt = Ct.color, sn = Kt.color;
              return Gt.selectedTextColorFn = function(hn) {
                var Qt = hn.tc || Vt;
                return hn.selected ? Yt || Qt : sn || (Yt ? Qt : ce.addOpacity(Qt, me));
              }, Gt;
            }, ve.selectedPointStyle = function(Nt, Gt) {
              if (Nt.size() && Gt.selectedpoints) {
                var Ft = ve.makeSelectedPointStyleFns(Gt), Tt = Gt.marker || {}, It = [];
                Ft.selectedOpacityFn && It.push(function(Ct, Kt) {
                  Ct.style("opacity", Ft.selectedOpacityFn(Kt));
                }), Ft.selectedColorFn && It.push(function(Ct, Kt) {
                  ce.fill(Ct, Ft.selectedColorFn(Kt));
                }), Ft.selectedSizeFn && It.push(function(Ct, Kt) {
                  var Vt = Kt.mx || Tt.symbol || 0, Yt = Ft.selectedSizeFn(Kt);
                  Ct.attr("d", xe(ve.symbolNumber(Vt), Yt, Xt(Kt, Gt), it(Kt, Gt))), Kt.mrc2 = Yt;
                }), It.length && Nt.each(function(Ct) {
                  for (var Kt = te.select(this), Vt = 0; Vt < It.length; Vt++) It[Vt](Kt, Ct);
                });
              }
            }, ve.tryColorscale = function(Nt, Gt) {
              var Ft = Gt ? ne.nestedProperty(Nt, Gt).get() : Nt;
              if (Ft) {
                var Tt = Ft.color;
                if ((Ft.colorscale || Ft._colorAx) && ne.isArrayOrTypedArray(Tt)) return le.makeColorScaleFuncFromTrace(Ft);
              }
              return ne.identity;
            };
            var Me, Re, De = { start: 1, end: -1, middle: 0, bottom: 1, top: -1 };
            function Pe(Nt, Gt, Ft, Tt, It) {
              var Ct = te.select(Nt.node().parentNode), Kt = Gt.indexOf("top") !== -1 ? "top" : Gt.indexOf("bottom") !== -1 ? "bottom" : "middle", Vt = Gt.indexOf("left") !== -1 ? "end" : Gt.indexOf("right") !== -1 ? "start" : "middle", Yt = Tt ? Tt / 0.8 + 1 : 0, sn = (de.lineCount(Nt) - 1) * he + 1, hn = De[Vt] * Yt, Qt = 0.75 * Ft + De[Kt] * Yt + (De[Kt] - 1) * sn * Ft / 2;
              Nt.attr("text-anchor", Vt), It || Ct.attr("transform", ue(hn, Qt));
            }
            function $e(Nt, Gt) {
              var Ft = Nt.ts || Gt.textfont.size;
              return ae(Ft) && Ft > 0 ? Ft : 0;
            }
            function qe(Nt, Gt, Ft) {
              return Ft && (Nt = We(Nt)), Gt ? Ve(Nt[1]) : Xe(Nt[0]);
            }
            function Xe(Nt) {
              var Gt = te.round(Nt, 2);
              return Me = Gt, Gt;
            }
            function Ve(Nt) {
              var Gt = te.round(Nt, 2);
              return Re = Gt, Gt;
            }
            function Ze(Nt, Gt, Ft, Tt) {
              var It = Nt[0] - Gt[0], Ct = Nt[1] - Gt[1], Kt = Ft[0] - Gt[0], Vt = Ft[1] - Gt[1], Yt = Math.pow(It * It + Ct * Ct, 0.25), sn = Math.pow(Kt * Kt + Vt * Vt, 0.25), hn = (sn * sn * It - Yt * Yt * Kt) * Tt, Qt = (sn * sn * Ct - Yt * Yt * Vt) * Tt, Ut = 3 * sn * (Yt + sn), Wt = 3 * Yt * (Yt + sn);
              return [[Xe(Gt[0] + (Ut && hn / Ut)), Ve(Gt[1] + (Ut && Qt / Ut))], [Xe(Gt[0] - (Wt && hn / Wt)), Ve(Gt[1] - (Wt && Qt / Wt))]];
            }
            ve.textPointStyle = function(Nt, Gt, Ft) {
              if (Nt.size()) {
                var Tt;
                if (Gt.selectedpoints) {
                  var It = ve.makeSelectedTextStyleFns(Gt);
                  Tt = It.selectedTextColorFn;
                }
                var Ct = Gt.texttemplate, Kt = Ft._fullLayout;
                Nt.each(function(Vt) {
                  var Yt = te.select(this), sn = Ct ? ne.extractOption(Vt, Gt, "txt", "texttemplate") : ne.extractOption(Vt, Gt, "tx", "text");
                  if (sn || sn === 0) {
                    if (Ct) {
                      var hn = Gt._module.formatLabels, Qt = hn ? hn(Vt, Gt, Kt) : {}, Ut = {};
                      ge(Ut, Gt, Vt.i);
                      var Wt = Gt._meta || {};
                      sn = ne.texttemplateString(sn, Qt, Kt._d3locale, Ut, Vt, Wt);
                    }
                    var rn = Vt.tp || Gt.textposition, vn = $e(Vt, Gt), dn = Tt ? Tt(Vt) : Vt.tc || Gt.textfont.color;
                    Yt.call(ve.font, { family: Vt.tf || Gt.textfont.family, weight: Vt.tw || Gt.textfont.weight, style: Vt.ty || Gt.textfont.style, variant: Vt.tv || Gt.textfont.variant, textcase: Vt.tC || Gt.textfont.textcase, lineposition: Vt.tE || Gt.textfont.lineposition, shadow: Vt.tS || Gt.textfont.shadow, size: vn, color: dn }).text(sn).call(de.convertToTspans, Ft).call(Pe, rn, vn, Vt.mrc);
                  } else Yt.remove();
                });
              }
            }, ve.selectedTextStyle = function(Nt, Gt) {
              if (Nt.size() && Gt.selectedpoints) {
                var Ft = ve.makeSelectedTextStyleFns(Gt);
                Nt.each(function(Tt) {
                  var It = te.select(this), Ct = Ft.selectedTextColorFn(Tt), Kt = Tt.tp || Gt.textposition, Vt = $e(Tt, Gt);
                  ce.fill(It, Ct);
                  var Yt = se.traceIs(Gt, "bar-like");
                  Pe(It, Kt, Vt, Tt.mrc2 || Tt.mrc, Yt);
                });
              }
            }, ve.smoothopen = function(Nt, Gt) {
              if (Nt.length < 3) return "M" + Nt.join("L");
              var Ft, Tt = "M" + Nt[0], It = [];
              for (Ft = 1; Ft < Nt.length - 1; Ft++) It.push(Ze(Nt[Ft - 1], Nt[Ft], Nt[Ft + 1], Gt));
              for (Tt += "Q" + It[0][0] + " " + Nt[1], Ft = 2; Ft < Nt.length - 1; Ft++) Tt += "C" + It[Ft - 2][1] + " " + It[Ft - 1][0] + " " + Nt[Ft];
              return Tt + "Q" + It[Nt.length - 3][1] + " " + Nt[Nt.length - 1];
            }, ve.smoothclosed = function(Nt, Gt) {
              if (Nt.length < 3) return "M" + Nt.join("L") + "Z";
              var Ft, Tt = "M" + Nt[0], It = Nt.length - 1, Ct = [Ze(Nt[It], Nt[0], Nt[1], Gt)];
              for (Ft = 1; Ft < It; Ft++) Ct.push(Ze(Nt[Ft - 1], Nt[Ft], Nt[Ft + 1], Gt));
              for (Ct.push(Ze(Nt[It - 1], Nt[It], Nt[0], Gt)), Ft = 1; Ft <= It; Ft++) Tt += "C" + Ct[Ft - 1][1] + " " + Ct[Ft][0] + " " + Nt[Ft];
              return Tt + "C" + Ct[It][1] + " " + Ct[0][0] + " " + Nt[0] + "Z";
            };
            var Ge = { hv: function(Nt, Gt, Ft) {
              return "H" + Xe(Gt[0]) + "V" + qe(Gt, 1, Ft);
            }, vh: function(Nt, Gt, Ft) {
              return "V" + Ve(Gt[1]) + "H" + qe(Gt, 0, Ft);
            }, hvh: function(Nt, Gt, Ft) {
              return "H" + Xe((Nt[0] + Gt[0]) / 2) + "V" + Ve(Gt[1]) + "H" + qe(Gt, 0, Ft);
            }, vhv: function(Nt, Gt, Ft) {
              return "V" + Ve((Nt[1] + Gt[1]) / 2) + "H" + Xe(Gt[0]) + "V" + qe(Gt, 1, Ft);
            } }, ze = function(Nt, Gt, Ft) {
              return "L" + qe(Gt, 0, Ft) + "," + qe(Gt, 1, Ft);
            };
            function We(Nt, Gt) {
              var Ft = Nt.backoff, Tt = Nt.trace, It = Nt.d, Ct = Nt.i;
              if (Ft && Tt && Tt.marker && Tt.marker.angle % 360 == 0 && Tt.line && Tt.line.shape !== "spline") {
                var Kt = ne.isArrayOrTypedArray(Ft), Vt = Nt, Yt = Gt ? Gt[0] : Me || 0, sn = Gt ? Gt[1] : Re || 0, hn = Vt[0], Qt = Vt[1], Ut = hn - Yt, Wt = Qt - sn, rn = Math.atan2(Wt, Ut), vn = Kt ? Ft[Ct] : Ft;
                if (vn === "auto") {
                  var dn = Vt.i;
                  Tt.type === "scatter" && dn--;
                  var Ht = Vt.marker, jt = Ht.symbol;
                  ne.isArrayOrTypedArray(jt) && (jt = jt[dn]);
                  var cn = Ht.size;
                  ne.isArrayOrTypedArray(cn) && (cn = cn[dn]), vn = Ht ? ve.symbolBackOffs[ve.symbolNumber(jt)] * cn : 0, vn += ve.getMarkerStandoff(It[dn], Tt) || 0;
                }
                var wn = hn - vn * Math.cos(rn), Rn = Qt - vn * Math.sin(rn);
                (wn <= hn && wn >= Yt || wn >= hn && wn <= Yt) && (Rn <= Qt && Rn >= sn || Rn >= Qt && Rn <= sn) && (Nt = [wn, Rn]);
              }
              return Nt;
            }
            ve.steps = function(Nt) {
              var Gt = Ge[Nt] || ze;
              return function(Ft) {
                for (var Tt = "M" + Xe(Ft[0][0]) + "," + Ve(Ft[0][1]), It = Ft.length, Ct = 1; Ct < It; Ct++) Tt += Gt(Ft[Ct - 1], Ft[Ct], Ct === It - 1);
                return Tt;
              };
            }, ve.applyBackoff = We, ve.makeTester = function() {
              var Nt = ne.ensureSingleById(te.select("body"), "svg", "js-plotly-tester", function(Ft) {
                Ft.attr(fe.svgAttrs).style({ position: "absolute", left: "-10000px", top: "-10000px", width: "9000px", height: "9000px", "z-index": "1" });
              }), Gt = ne.ensureSingle(Nt, "path", "js-reference-point", function(Ft) {
                Ft.attr("d", "M0,0H1V1H0Z").style({ "stroke-width": 0, fill: "black" });
              });
              ve.tester = Nt, ve.testref = Gt;
            }, ve.savedBBoxes = {};
            var at = 0;
            function ct(Nt) {
              var Gt = Nt.getAttribute("data-unformatted");
              if (Gt !== null) return Gt + Nt.getAttribute("data-math") + Nt.getAttribute("text-anchor") + Nt.getAttribute("style");
            }
            function et(Nt, Gt) {
              if (!Nt) return null;
              var Ft = Gt._context, Tt = Ft._exportedPlot ? "" : Ft._baseUrl || "";
              return Tt ? "url('" + Tt + "#" + Nt + "')" : "url(#" + Nt + ")";
            }
            ve.bBox = function(Nt, Gt, Ft) {
              var Tt, It, Ct;
              if (Ft || (Ft = ct(Nt)), Ft) {
                if (Tt = ve.savedBBoxes[Ft]) return ne.extendFlat({}, Tt);
              } else if (Nt.childNodes.length === 1) {
                var Kt = Nt.childNodes[0];
                if (Ft = ct(Kt)) {
                  var Vt = +Kt.getAttribute("x") || 0, Yt = +Kt.getAttribute("y") || 0, sn = Kt.getAttribute("transform");
                  if (!sn) {
                    var hn = ve.bBox(Kt, !1, Ft);
                    return Vt && (hn.left += Vt, hn.right += Vt), Yt && (hn.top += Yt, hn.bottom += Yt), hn;
                  }
                  if (Ft += "~" + Vt + "~" + Yt + "~" + sn, Tt = ve.savedBBoxes[Ft]) return ne.extendFlat({}, Tt);
                }
              }
              Gt ? It = Nt : (Ct = ve.tester.node(), It = Nt.cloneNode(!0), Ct.appendChild(It)), te.select(It).attr("transform", null).call(de.positionText, 0, 0);
              var Qt = It.getBoundingClientRect(), Ut = ve.testref.node().getBoundingClientRect();
              Gt || Ct.removeChild(It);
              var Wt = { height: Qt.height, width: Qt.width, left: Qt.left - Ut.left, top: Qt.top - Ut.top, right: Qt.right - Ut.left, bottom: Qt.bottom - Ut.top };
              return at >= 1e4 && (ve.savedBBoxes = {}, at = 0), Ft && (ve.savedBBoxes[Ft] = Wt), at++, ne.extendFlat({}, Wt);
            }, ve.setClipUrl = function(Nt, Gt, Ft) {
              Nt.attr("clip-path", et(Gt, Ft));
            }, ve.getTranslate = function(Nt) {
              var Gt = (Nt[Nt.attr ? "attr" : "getAttribute"]("transform") || "").replace(/.*\btranslate\((-?\d*\.?\d*)[^-\d]*(-?\d*\.?\d*)[^\d].*/, function(Ft, Tt, It) {
                return [Tt, It].join(" ");
              }).split(" ");
              return { x: +Gt[0] || 0, y: +Gt[1] || 0 };
            }, ve.setTranslate = function(Nt, Gt, Ft) {
              var Tt = Nt.attr ? "attr" : "getAttribute", It = Nt.attr ? "attr" : "setAttribute", Ct = Nt[Tt]("transform") || "";
              return Gt = Gt || 0, Ft = Ft || 0, Ct = Ct.replace(/(\btranslate\(.*?\);?)/, "").trim(), Ct = (Ct += ue(Gt, Ft)).trim(), Nt[It]("transform", Ct), Ct;
            }, ve.getScale = function(Nt) {
              var Gt = (Nt[Nt.attr ? "attr" : "getAttribute"]("transform") || "").replace(/.*\bscale\((\d*\.?\d*)[^\d]*(\d*\.?\d*)[^\d].*/, function(Ft, Tt, It) {
                return [Tt, It].join(" ");
              }).split(" ");
              return { x: +Gt[0] || 1, y: +Gt[1] || 1 };
            }, ve.setScale = function(Nt, Gt, Ft) {
              var Tt = Nt.attr ? "attr" : "getAttribute", It = Nt.attr ? "attr" : "setAttribute", Ct = Nt[Tt]("transform") || "";
              return Gt = Gt || 1, Ft = Ft || 1, Ct = Ct.replace(/(\bscale\(.*?\);?)/, "").trim(), Ct = (Ct += "scale(" + Gt + "," + Ft + ")").trim(), Nt[It]("transform", Ct), Ct;
            };
            var dt = /\s*sc.*/;
            ve.setPointGroupScale = function(Nt, Gt, Ft) {
              if (Gt = Gt || 1, Ft = Ft || 1, Nt) {
                var Tt = Gt === 1 && Ft === 1 ? "" : "scale(" + Gt + "," + Ft + ")";
                Nt.each(function() {
                  var It = (this.getAttribute("transform") || "").replace(dt, "");
                  It = (It += Tt).trim(), this.setAttribute("transform", It);
                });
              }
            };
            var Je = /translate\([^)]*\)\s*$/;
            function it(Nt, Gt) {
              var Ft;
              return Nt && (Ft = Nt.mf), Ft === void 0 && (Ft = Gt.marker && Gt.marker.standoff || 0), Gt._geo || Gt._xA ? Ft : -Ft;
            }
            ve.setTextPointsScale = function(Nt, Gt, Ft) {
              Nt && Nt.each(function() {
                var Tt, It = te.select(this), Ct = It.select("text");
                if (Ct.node()) {
                  var Kt = parseFloat(Ct.attr("x") || 0), Vt = parseFloat(Ct.attr("y") || 0), Yt = (It.attr("transform") || "").match(Je);
                  Tt = Gt === 1 && Ft === 1 ? [] : [ue(Kt, Vt), "scale(" + Gt + "," + Ft + ")", ue(-Kt, -Vt)], Yt && Tt.push(Yt), It.attr("transform", Tt.join(""));
                }
              });
            }, ve.getMarkerStandoff = it;
            var st, gt, bt, yt, St, Lt, Rt = Math.atan2, vt = Math.cos, $t = Math.sin;
            function Pt(Nt, Gt) {
              var Ft = Gt[0], Tt = Gt[1];
              return [Ft * vt(Nt) - Tt * $t(Nt), Ft * $t(Nt) + Tt * vt(Nt)];
            }
            function Xt(Nt, Gt) {
              var Ft, Tt, It = Nt.ma;
              It === void 0 && ((It = Gt.marker.angle) && !ne.isArrayOrTypedArray(It) || (It = 0));
              var Ct = Gt.marker.angleref;
              if (Ct === "previous" || Ct === "north") {
                if (Gt._geo) {
                  var Kt = Gt._geo.project(Nt.lonlat);
                  Ft = Kt[0], Tt = Kt[1];
                } else {
                  var Vt = Gt._xA, Yt = Gt._yA;
                  if (!Vt || !Yt) return 90;
                  Ft = Vt.c2p(Nt.x), Tt = Yt.c2p(Nt.y);
                }
                if (Gt._geo) {
                  var sn, hn = Nt.lonlat[0], Qt = Nt.lonlat[1], Ut = Gt._geo.project([hn, Qt + 1e-5]), Wt = Gt._geo.project([hn + 1e-5, Qt]), rn = Rt(Wt[1] - Tt, Wt[0] - Ft), vn = Rt(Ut[1] - Tt, Ut[0] - Ft);
                  if (Ct === "north") sn = It / 180 * Math.PI;
                  else if (Ct === "previous") {
                    var dn = hn / 180 * Math.PI, Ht = Qt / 180 * Math.PI, jt = st / 180 * Math.PI, cn = gt / 180 * Math.PI, wn = jt - dn, Rn = vt(cn) * $t(wn), Pn = $t(cn) * vt(Ht) - vt(cn) * $t(Ht) * vt(wn);
                    sn = -Rt(Rn, Pn) - Math.PI, st = hn, gt = Qt;
                  }
                  var Ie = Pt(rn, [vt(sn), 0]), rt = Pt(vn, [$t(sn), 0]);
                  It = Rt(Ie[1] + rt[1], Ie[0] + rt[0]) / Math.PI * 180, Ct !== "previous" || Lt === Gt.uid && Nt.i === St + 1 || (It = null);
                }
                if (Ct === "previous" && !Gt._geo) if (Lt === Gt.uid && Nt.i === St + 1 && ae(Ft) && ae(Tt)) {
                  var ft = Ft - bt, pt = Tt - yt, wt = Gt.line && Gt.line.shape || "", Bt = wt.slice(wt.length - 1);
                  Bt === "h" && (pt = 0), Bt === "v" && (ft = 0), It += Rt(pt, ft) / Math.PI * 180 + 90;
                } else It = null;
              }
              return bt = Ft, yt = Tt, St = Nt.i, Lt = Gt.uid, It;
            }
            ve.getMarkerAngle = Xt;
          }, 38882: function(G, J, Z) {
            var te, ne, ee, ae, oe = Z(26953), se = Z(45568).round, ce = "M0,0Z", le = Math.sqrt(2), ue = Math.sqrt(3), de = Math.PI, fe = Math.cos, he = Math.sin;
            function me(ye) {
              return ye === null;
            }
            function pe(ye, ge, ve) {
              if (!(ye && ye % 360 != 0 || ge)) return ve;
              if (ee === ye && ae === ge && te === ve) return ne;
              function be($e, qe) {
                var Xe = fe($e), Ve = he($e), Ze = qe[0], Ge = qe[1] + (ge || 0);
                return [Ze * Xe - Ge * Ve, Ze * Ve + Ge * Xe];
              }
              ee = ye, ae = ge, te = ve;
              for (var Ce = ye / 180 * de, Se = 0, we = 0, Ee = oe(ve), xe = "", Le = 0; Le < Ee.length; Le++) {
                var Qe = Ee[Le], ke = Qe[0], Me = Se, Re = we;
                if (ke === "M" || ke === "L") Se = +Qe[1], we = +Qe[2];
                else if (ke === "m" || ke === "l") Se += +Qe[1], we += +Qe[2];
                else if (ke === "H") Se = +Qe[1];
                else if (ke === "h") Se += +Qe[1];
                else if (ke === "V") we = +Qe[1];
                else if (ke === "v") we += +Qe[1];
                else if (ke === "A") {
                  Se = +Qe[1], we = +Qe[2];
                  var De = be(Ce, [+Qe[6], +Qe[7]]);
                  Qe[6] = De[0], Qe[7] = De[1], Qe[3] = +Qe[3] + ye;
                }
                ke !== "H" && ke !== "V" || (ke = "L"), ke !== "h" && ke !== "v" || (ke = "l"), ke !== "m" && ke !== "l" || (Se -= Me, we -= Re);
                var Pe = be(Ce, [Se, we]);
                ke !== "H" && ke !== "V" || (ke = "L"), ke !== "M" && ke !== "L" && ke !== "m" && ke !== "l" || (Qe[1] = Pe[0], Qe[2] = Pe[1]), Qe[0] = ke, xe += Qe[0] + Qe.slice(1).join(",");
              }
              return ne = xe, xe;
            }
            G.exports = { circle: { n: 0, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2), Ce = "M" + be + ",0A" + be + "," + be + " 0 1,1 0,-" + be + "A" + be + "," + be + " 0 0,1 " + be + ",0Z";
              return ve ? pe(ge, ve, Ce) : Ce;
            } }, square: { n: 1, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M" + be + "," + be + "H-" + be + "V-" + be + "H" + be + "Z");
            } }, diamond: { n: 2, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.3 * ye, 2);
              return pe(ge, ve, "M" + be + ",0L0," + be + "L-" + be + ",0L0,-" + be + "Z");
            } }, cross: { n: 3, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.4 * ye, 2), Ce = se(1.2 * ye, 2);
              return pe(ge, ve, "M" + Ce + "," + be + "H" + be + "V" + Ce + "H-" + be + "V" + be + "H-" + Ce + "V-" + be + "H-" + be + "V-" + Ce + "H" + be + "V-" + be + "H" + Ce + "Z");
            } }, x: { n: 4, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.8 * ye / le, 2), Ce = "l" + be + "," + be, Se = "l" + be + ",-" + be, we = "l-" + be + ",-" + be, Ee = "l-" + be + "," + be;
              return pe(ge, ve, "M0," + be + Ce + Se + we + Se + we + Ee + we + Ee + Ce + Ee + Ce + "Z");
            } }, "triangle-up": { n: 5, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye / ue, 2);
              return pe(ge, ve, "M-" + be + "," + se(ye / 2, 2) + "H" + be + "L0,-" + se(ye, 2) + "Z");
            } }, "triangle-down": { n: 6, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye / ue, 2);
              return pe(ge, ve, "M-" + be + ",-" + se(ye / 2, 2) + "H" + be + "L0," + se(ye, 2) + "Z");
            } }, "triangle-left": { n: 7, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye / ue, 2);
              return pe(ge, ve, "M" + se(ye / 2, 2) + ",-" + be + "V" + be + "L-" + se(ye, 2) + ",0Z");
            } }, "triangle-right": { n: 8, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye / ue, 2);
              return pe(ge, ve, "M-" + se(ye / 2, 2) + ",-" + be + "V" + be + "L" + se(ye, 2) + ",0Z");
            } }, "triangle-ne": { n: 9, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.6 * ye, 2), Ce = se(1.2 * ye, 2);
              return pe(ge, ve, "M-" + Ce + ",-" + be + "H" + be + "V" + Ce + "Z");
            } }, "triangle-se": { n: 10, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.6 * ye, 2), Ce = se(1.2 * ye, 2);
              return pe(ge, ve, "M" + be + ",-" + Ce + "V" + be + "H-" + Ce + "Z");
            } }, "triangle-sw": { n: 11, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.6 * ye, 2), Ce = se(1.2 * ye, 2);
              return pe(ge, ve, "M" + Ce + "," + be + "H-" + be + "V-" + Ce + "Z");
            } }, "triangle-nw": { n: 12, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.6 * ye, 2), Ce = se(1.2 * ye, 2);
              return pe(ge, ve, "M-" + be + "," + Ce + "V-" + be + "H" + Ce + "Z");
            } }, pentagon: { n: 13, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.951 * ye, 2), Ce = se(0.588 * ye, 2), Se = se(-ye, 2), we = se(-0.309 * ye, 2);
              return pe(ge, ve, "M" + be + "," + we + "L" + Ce + "," + se(0.809 * ye, 2) + "H-" + Ce + "L-" + be + "," + we + "L0," + Se + "Z");
            } }, hexagon: { n: 14, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2), Ce = se(ye / 2, 2), Se = se(ye * ue / 2, 2);
              return pe(ge, ve, "M" + Se + ",-" + Ce + "V" + Ce + "L0," + be + "L-" + Se + "," + Ce + "V-" + Ce + "L0,-" + be + "Z");
            } }, hexagon2: { n: 15, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2), Ce = se(ye / 2, 2), Se = se(ye * ue / 2, 2);
              return pe(ge, ve, "M-" + Ce + "," + Se + "H" + Ce + "L" + be + ",0L" + Ce + ",-" + Se + "H-" + Ce + "L-" + be + ",0Z");
            } }, octagon: { n: 16, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.924 * ye, 2), Ce = se(0.383 * ye, 2);
              return pe(ge, ve, "M-" + Ce + ",-" + be + "H" + Ce + "L" + be + ",-" + Ce + "V" + Ce + "L" + Ce + "," + be + "H-" + Ce + "L-" + be + "," + Ce + "V-" + Ce + "Z");
            } }, star: { n: 17, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = 1.4 * ye, Ce = se(0.225 * be, 2), Se = se(0.951 * be, 2), we = se(0.363 * be, 2), Ee = se(0.588 * be, 2), xe = se(-be, 2), Le = se(-0.309 * be, 2), Qe = se(0.118 * be, 2), ke = se(0.809 * be, 2);
              return pe(ge, ve, "M" + Ce + "," + Le + "H" + Se + "L" + we + "," + Qe + "L" + Ee + "," + ke + "L0," + se(0.382 * be, 2) + "L-" + Ee + "," + ke + "L-" + we + "," + Qe + "L-" + Se + "," + Le + "H-" + Ce + "L0," + xe + "Z");
            } }, hexagram: { n: 18, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.66 * ye, 2), Ce = se(0.38 * ye, 2), Se = se(0.76 * ye, 2);
              return pe(ge, ve, "M-" + Se + ",0l-" + Ce + ",-" + be + "h" + Se + "l" + Ce + ",-" + be + "l" + Ce + "," + be + "h" + Se + "l-" + Ce + "," + be + "l" + Ce + "," + be + "h-" + Se + "l-" + Ce + "," + be + "l-" + Ce + ",-" + be + "h-" + Se + "Z");
            } }, "star-triangle-up": { n: 19, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye * ue * 0.8, 2), Ce = se(0.8 * ye, 2), Se = se(1.6 * ye, 2), we = se(4 * ye, 2), Ee = "A " + we + "," + we + " 0 0 1 ";
              return pe(ge, ve, "M-" + be + "," + Ce + Ee + be + "," + Ce + Ee + "0,-" + Se + Ee + "-" + be + "," + Ce + "Z");
            } }, "star-triangle-down": { n: 20, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye * ue * 0.8, 2), Ce = se(0.8 * ye, 2), Se = se(1.6 * ye, 2), we = se(4 * ye, 2), Ee = "A " + we + "," + we + " 0 0 1 ";
              return pe(ge, ve, "M" + be + ",-" + Ce + Ee + "-" + be + ",-" + Ce + Ee + "0," + Se + Ee + be + ",-" + Ce + "Z");
            } }, "star-square": { n: 21, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.1 * ye, 2), Ce = se(2 * ye, 2), Se = "A " + Ce + "," + Ce + " 0 0 1 ";
              return pe(ge, ve, "M-" + be + ",-" + be + Se + "-" + be + "," + be + Se + be + "," + be + Se + be + ",-" + be + Se + "-" + be + ",-" + be + "Z");
            } }, "star-diamond": { n: 22, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.4 * ye, 2), Ce = se(1.9 * ye, 2), Se = "A " + Ce + "," + Ce + " 0 0 1 ";
              return pe(ge, ve, "M-" + be + ",0" + Se + "0," + be + Se + be + ",0" + Se + "0,-" + be + Se + "-" + be + ",0Z");
            } }, "diamond-tall": { n: 23, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(0.7 * ye, 2), Ce = se(1.4 * ye, 2);
              return pe(ge, ve, "M0," + Ce + "L" + be + ",0L0,-" + Ce + "L-" + be + ",0Z");
            } }, "diamond-wide": { n: 24, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.4 * ye, 2), Ce = se(0.7 * ye, 2);
              return pe(ge, ve, "M0," + Ce + "L" + be + ",0L0,-" + Ce + "L-" + be + ",0Z");
            } }, hourglass: { n: 25, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M" + be + "," + be + "H-" + be + "L" + be + ",-" + be + "H-" + be + "Z");
            }, noDot: !0 }, bowtie: { n: 26, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M" + be + "," + be + "V-" + be + "L-" + be + "," + be + "V-" + be + "Z");
            }, noDot: !0 }, "circle-cross": { n: 27, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M0," + be + "V-" + be + "M" + be + ",0H-" + be + "M" + be + ",0A" + be + "," + be + " 0 1,1 0,-" + be + "A" + be + "," + be + " 0 0,1 " + be + ",0Z");
            }, needLine: !0, noDot: !0 }, "circle-x": { n: 28, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2), Ce = se(ye / le, 2);
              return pe(ge, ve, "M" + Ce + "," + Ce + "L-" + Ce + ",-" + Ce + "M" + Ce + ",-" + Ce + "L-" + Ce + "," + Ce + "M" + be + ",0A" + be + "," + be + " 0 1,1 0,-" + be + "A" + be + "," + be + " 0 0,1 " + be + ",0Z");
            }, needLine: !0, noDot: !0 }, "square-cross": { n: 29, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M0," + be + "V-" + be + "M" + be + ",0H-" + be + "M" + be + "," + be + "H-" + be + "V-" + be + "H" + be + "Z");
            }, needLine: !0, noDot: !0 }, "square-x": { n: 30, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M" + be + "," + be + "L-" + be + ",-" + be + "M" + be + ",-" + be + "L-" + be + "," + be + "M" + be + "," + be + "H-" + be + "V-" + be + "H" + be + "Z");
            }, needLine: !0, noDot: !0 }, "diamond-cross": { n: 31, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.3 * ye, 2);
              return pe(ge, ve, "M" + be + ",0L0," + be + "L-" + be + ",0L0,-" + be + "ZM0,-" + be + "V" + be + "M-" + be + ",0H" + be);
            }, needLine: !0, noDot: !0 }, "diamond-x": { n: 32, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.3 * ye, 2), Ce = se(0.65 * ye, 2);
              return pe(ge, ve, "M" + be + ",0L0," + be + "L-" + be + ",0L0,-" + be + "ZM-" + Ce + ",-" + Ce + "L" + Ce + "," + Ce + "M-" + Ce + "," + Ce + "L" + Ce + ",-" + Ce);
            }, needLine: !0, noDot: !0 }, "cross-thin": { n: 33, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.4 * ye, 2);
              return pe(ge, ve, "M0," + be + "V-" + be + "M" + be + ",0H-" + be);
            }, needLine: !0, noDot: !0, noFill: !0 }, "x-thin": { n: 34, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M" + be + "," + be + "L-" + be + ",-" + be + "M" + be + ",-" + be + "L-" + be + "," + be);
            }, needLine: !0, noDot: !0, noFill: !0 }, asterisk: { n: 35, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.2 * ye, 2), Ce = se(0.85 * ye, 2);
              return pe(ge, ve, "M0," + be + "V-" + be + "M" + be + ",0H-" + be + "M" + Ce + "," + Ce + "L-" + Ce + ",-" + Ce + "M" + Ce + ",-" + Ce + "L-" + Ce + "," + Ce);
            }, needLine: !0, noDot: !0, noFill: !0 }, hash: { n: 36, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye / 2, 2), Ce = se(ye, 2);
              return pe(ge, ve, "M" + be + "," + Ce + "V-" + Ce + "M" + (be - Ce) + ",-" + Ce + "V" + Ce + "M" + Ce + "," + be + "H-" + Ce + "M-" + Ce + "," + (be - Ce) + "H" + Ce);
            }, needLine: !0, noFill: !0 }, "y-up": { n: 37, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.2 * ye, 2), Ce = se(1.6 * ye, 2), Se = se(0.8 * ye, 2);
              return pe(ge, ve, "M-" + be + "," + Se + "L0,0M" + be + "," + Se + "L0,0M0,-" + Ce + "L0,0");
            }, needLine: !0, noDot: !0, noFill: !0 }, "y-down": { n: 38, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.2 * ye, 2), Ce = se(1.6 * ye, 2), Se = se(0.8 * ye, 2);
              return pe(ge, ve, "M-" + be + ",-" + Se + "L0,0M" + be + ",-" + Se + "L0,0M0," + Ce + "L0,0");
            }, needLine: !0, noDot: !0, noFill: !0 }, "y-left": { n: 39, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.2 * ye, 2), Ce = se(1.6 * ye, 2), Se = se(0.8 * ye, 2);
              return pe(ge, ve, "M" + Se + "," + be + "L0,0M" + Se + ",-" + be + "L0,0M-" + Ce + ",0L0,0");
            }, needLine: !0, noDot: !0, noFill: !0 }, "y-right": { n: 40, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.2 * ye, 2), Ce = se(1.6 * ye, 2), Se = se(0.8 * ye, 2);
              return pe(ge, ve, "M-" + Se + "," + be + "L0,0M-" + Se + ",-" + be + "L0,0M" + Ce + ",0L0,0");
            }, needLine: !0, noDot: !0, noFill: !0 }, "line-ew": { n: 41, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.4 * ye, 2);
              return pe(ge, ve, "M" + be + ",0H-" + be);
            }, needLine: !0, noDot: !0, noFill: !0 }, "line-ns": { n: 42, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(1.4 * ye, 2);
              return pe(ge, ve, "M0," + be + "V-" + be);
            }, needLine: !0, noDot: !0, noFill: !0 }, "line-ne": { n: 43, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M" + be + ",-" + be + "L-" + be + "," + be);
            }, needLine: !0, noDot: !0, noFill: !0 }, "line-nw": { n: 44, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M" + be + "," + be + "L-" + be + ",-" + be);
            }, needLine: !0, noDot: !0, noFill: !0 }, "arrow-up": { n: 45, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M0,0L-" + be + "," + se(2 * ye, 2) + "H" + be + "Z");
            }, backoff: 1, noDot: !0 }, "arrow-down": { n: 46, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M0,0L-" + be + ",-" + se(2 * ye, 2) + "H" + be + "Z");
            }, noDot: !0 }, "arrow-left": { n: 47, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye, 2), Ce = se(ye, 2);
              return pe(ge, ve, "M0,0L" + be + ",-" + Ce + "V" + Ce + "Z");
            }, noDot: !0 }, "arrow-right": { n: 48, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye, 2), Ce = se(ye, 2);
              return pe(ge, ve, "M0,0L-" + be + ",-" + Ce + "V" + Ce + "Z");
            }, noDot: !0 }, "arrow-bar-up": { n: 49, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M-" + be + ",0H" + be + "M0,0L-" + be + "," + se(2 * ye, 2) + "H" + be + "Z");
            }, backoff: 1, needLine: !0, noDot: !0 }, "arrow-bar-down": { n: 50, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(ye, 2);
              return pe(ge, ve, "M-" + be + ",0H" + be + "M0,0L-" + be + ",-" + se(2 * ye, 2) + "H" + be + "Z");
            }, needLine: !0, noDot: !0 }, "arrow-bar-left": { n: 51, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye, 2), Ce = se(ye, 2);
              return pe(ge, ve, "M0,-" + Ce + "V" + Ce + "M0,0L" + be + ",-" + Ce + "V" + Ce + "Z");
            }, needLine: !0, noDot: !0 }, "arrow-bar-right": { n: 52, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = se(2 * ye, 2), Ce = se(ye, 2);
              return pe(ge, ve, "M0,-" + Ce + "V" + Ce + "M0,0L-" + be + ",-" + Ce + "V" + Ce + "Z");
            }, needLine: !0, noDot: !0 }, arrow: { n: 53, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = de / 2.5, Ce = 2 * ye * fe(be), Se = 2 * ye * he(be);
              return pe(ge, ve, "M0,0L" + -Ce + "," + Se + "L" + Ce + "," + Se + "Z");
            }, backoff: 0.9, noDot: !0 }, "arrow-wide": { n: 54, f: function(ye, ge, ve) {
              if (me(ge)) return ce;
              var be = de / 4, Ce = 2 * ye * fe(be), Se = 2 * ye * he(be);
              return pe(ge, ve, "M0,0L" + -Ce + "," + Se + "A " + 2 * ye + "," + 2 * ye + " 0 0 1 " + Ce + "," + Se + "Z");
            }, backoff: 0.4, noDot: !0 } };
          }, 75568: function(G) {
            G.exports = { visible: { valType: "boolean", editType: "calc" }, type: { valType: "enumerated", values: ["percent", "constant", "sqrt", "data"], editType: "calc" }, symmetric: { valType: "boolean", editType: "calc" }, array: { valType: "data_array", editType: "calc" }, arrayminus: { valType: "data_array", editType: "calc" }, value: { valType: "number", min: 0, dflt: 10, editType: "calc" }, valueminus: { valType: "number", min: 0, dflt: 10, editType: "calc" }, traceref: { valType: "integer", min: 0, dflt: 0, editType: "style" }, tracerefminus: { valType: "integer", min: 0, dflt: 0, editType: "style" }, copy_ystyle: { valType: "boolean", editType: "plot" }, copy_zstyle: { valType: "boolean", editType: "style" }, color: { valType: "color", editType: "style" }, thickness: { valType: "number", min: 0, dflt: 2, editType: "style" }, width: { valType: "number", min: 0, editType: "plot" }, editType: "calc", _deprecated: { opacity: { valType: "number", editType: "style" } } };
          }, 352: function(G, J, Z) {
            var te = Z(10721), ne = Z(33626), ee = Z(29714), ae = Z(34809), oe = Z(25589);
            function se(ce, le, ue, de) {
              var fe = le["error_" + de] || {}, he = [];
              if (fe.visible && ["linear", "log"].indexOf(ue.type) !== -1) {
                for (var me = oe(fe), pe = 0; pe < ce.length; pe++) {
                  var ye = ce[pe], ge = ye.i;
                  if (ge === void 0) ge = pe;
                  else if (ge === null) continue;
                  var ve = ye[de];
                  if (te(ue.c2l(ve))) {
                    var be = me(ve, ge);
                    if (te(be[0]) && te(be[1])) {
                      var Ce = ye[de + "s"] = ve - be[0], Se = ye[de + "h"] = ve + be[1];
                      he.push(Ce, Se);
                    }
                  }
                }
                var we = ue._id, Ee = le._extremes[we], xe = ee.findExtremes(ue, he, ae.extendFlat({ tozero: Ee.opts.tozero }, { padded: !0 }));
                Ee.min = Ee.min.concat(xe.min), Ee.max = Ee.max.concat(xe.max);
              }
            }
            G.exports = function(ce) {
              for (var le = ce.calcdata, ue = 0; ue < le.length; ue++) {
                var de = le[ue], fe = de[0].trace;
                if (fe.visible === !0 && ne.traceIs(fe, "errorBarsOK")) {
                  var he = ee.getFromId(ce, fe.xaxis), me = ee.getFromId(ce, fe.yaxis);
                  se(de, fe, he, "x"), se(de, fe, me, "y");
                }
              }
            };
          }, 25589: function(G) {
            function J(Z, te) {
              return Z === "percent" ? function(ne) {
                return Math.abs(ne * te / 100);
              } : Z === "constant" ? function() {
                return Math.abs(te);
              } : Z === "sqrt" ? function(ne) {
                return Math.sqrt(Math.abs(ne));
              } : void 0;
            }
            G.exports = function(Z) {
              var te = Z.type, ne = Z.symmetric;
              if (te === "data") {
                var ee = Z.array || [];
                if (ne) return function(ce, le) {
                  var ue = +ee[le];
                  return [ue, ue];
                };
                var ae = Z.arrayminus || [];
                return function(ce, le) {
                  var ue = +ee[le], de = +ae[le];
                  return isNaN(ue) && isNaN(de) ? [NaN, NaN] : [de || 0, ue || 0];
                };
              }
              var oe = J(te, Z.value), se = J(te, Z.valueminus);
              return ne || Z.valueminus === void 0 ? function(ce) {
                var le = oe(ce);
                return [le, le];
              } : function(ce) {
                return [se(ce), oe(ce)];
              };
            };
          }, 5543: function(G, J, Z) {
            var te = Z(10721), ne = Z(33626), ee = Z(34809), ae = Z(78032), oe = Z(75568);
            G.exports = function(se, ce, le, ue) {
              var de = "error_" + ue.axis, fe = ae.newContainer(ce, de), he = se[de] || {};
              function me(ve, be) {
                return ee.coerce(he, fe, oe, ve, be);
              }
              if (me("visible", he.array !== void 0 || he.value !== void 0 || he.type === "sqrt") !== !1) {
                var pe = me("type", "array" in he ? "data" : "percent"), ye = !0;
                pe !== "sqrt" && (ye = me("symmetric", !((pe === "data" ? "arrayminus" : "valueminus") in he))), pe === "data" ? (me("array"), me("traceref"), ye || (me("arrayminus"), me("tracerefminus"))) : pe !== "percent" && pe !== "constant" || (me("value"), ye || me("valueminus"));
                var ge = "copy_" + ue.inherit + "style";
                ue.inherit && (ce["error_" + ue.inherit] || {}).visible && me(ge, !(he.color || te(he.thickness) || te(he.width))), ue.inherit && fe[ge] || (me("color", le), me("thickness"), me("width", ne.traceIs(ce, "gl3d") ? 0 : 4));
              }
            };
          }, 77901: function(G, J, Z) {
            var te = Z(34809), ne = Z(13582).overrideAll, ee = Z(75568), ae = { error_x: te.extendFlat({}, ee), error_y: te.extendFlat({}, ee) };
            delete ae.error_x.copy_zstyle, delete ae.error_y.copy_zstyle, delete ae.error_y.copy_ystyle;
            var oe = { error_x: te.extendFlat({}, ee), error_y: te.extendFlat({}, ee), error_z: te.extendFlat({}, ee) };
            delete oe.error_x.copy_ystyle, delete oe.error_y.copy_ystyle, delete oe.error_z.copy_ystyle, delete oe.error_z.copy_zstyle, G.exports = { moduleType: "component", name: "errorbars", schema: { traces: { scatter: ae, bar: ae, histogram: ae, scatter3d: ne(oe, "calc", "nested"), scattergl: ne(ae, "calc", "nested") } }, supplyDefaults: Z(5543), calc: Z(352), makeComputeError: Z(25589), plot: Z(42130), style: Z(22800), hoverInfo: function(se, ce, le) {
              (ce.error_y || {}).visible && (le.yerr = se.yh - se.y, ce.error_y.symmetric || (le.yerrneg = se.y - se.ys)), (ce.error_x || {}).visible && (le.xerr = se.xh - se.x, ce.error_x.symmetric || (le.xerrneg = se.x - se.xs));
            } };
          }, 42130: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(62203), ae = Z(64726);
            G.exports = function(oe, se, ce, le) {
              var ue = ce.xaxis, de = ce.yaxis, fe = le && le.duration > 0, he = oe._context.staticPlot;
              se.each(function(me) {
                var pe, ye = me[0].trace, ge = ye.error_x || {}, ve = ye.error_y || {};
                ye.ids && (pe = function(we) {
                  return we.id;
                });
                var be = ae.hasMarkers(ye) && ye.marker.maxdisplayed > 0;
                ve.visible || ge.visible || (me = []);
                var Ce = te.select(this).selectAll("g.errorbar").data(me, pe);
                if (Ce.exit().remove(), me.length) {
                  ge.visible || Ce.selectAll("path.xerror").remove(), ve.visible || Ce.selectAll("path.yerror").remove(), Ce.style("opacity", 1);
                  var Se = Ce.enter().append("g").classed("errorbar", !0);
                  fe && Se.style("opacity", 0).transition().duration(le.duration).style("opacity", 1), ee.setClipUrl(Ce, ce.layerClipId, oe), Ce.each(function(we) {
                    var Ee = te.select(this), xe = function(De, Pe, $e) {
                      var qe = { x: Pe.c2p(De.x), y: $e.c2p(De.y) };
                      return De.yh !== void 0 && (qe.yh = $e.c2p(De.yh), qe.ys = $e.c2p(De.ys), ne(qe.ys) || (qe.noYS = !0, qe.ys = $e.c2p(De.ys, !0))), De.xh !== void 0 && (qe.xh = Pe.c2p(De.xh), qe.xs = Pe.c2p(De.xs), ne(qe.xs) || (qe.noXS = !0, qe.xs = Pe.c2p(De.xs, !0))), qe;
                    }(we, ue, de);
                    if (!be || we.vis) {
                      var Le, Qe = Ee.select("path.yerror");
                      if (ve.visible && ne(xe.x) && ne(xe.yh) && ne(xe.ys)) {
                        var ke = ve.width;
                        Le = "M" + (xe.x - ke) + "," + xe.yh + "h" + 2 * ke + "m-" + ke + ",0V" + xe.ys, xe.noYS || (Le += "m-" + ke + ",0h" + 2 * ke), Qe.size() ? fe && (Qe = Qe.transition().duration(le.duration).ease(le.easing)) : Qe = Ee.append("path").style("vector-effect", he ? "none" : "non-scaling-stroke").classed("yerror", !0), Qe.attr("d", Le);
                      } else Qe.remove();
                      var Me = Ee.select("path.xerror");
                      if (ge.visible && ne(xe.y) && ne(xe.xh) && ne(xe.xs)) {
                        var Re = (ge.copy_ystyle ? ve : ge).width;
                        Le = "M" + xe.xh + "," + (xe.y - Re) + "v" + 2 * Re + "m0,-" + Re + "H" + xe.xs, xe.noXS || (Le += "m0,-" + Re + "v" + 2 * Re), Me.size() ? fe && (Me = Me.transition().duration(le.duration).ease(le.easing)) : Me = Ee.append("path").style("vector-effect", he ? "none" : "non-scaling-stroke").classed("xerror", !0), Me.attr("d", Le);
                      } else Me.remove();
                    }
                  });
                }
              });
            };
          }, 22800: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766);
            G.exports = function(ee) {
              ee.each(function(ae) {
                var oe = ae[0].trace, se = oe.error_y || {}, ce = oe.error_x || {}, le = te.select(this);
                le.selectAll("path.yerror").style("stroke-width", se.thickness + "px").call(ne.stroke, se.color), ce.copy_ystyle && (ce = se), le.selectAll("path.xerror").style("stroke-width", ce.thickness + "px").call(ne.stroke, ce.color);
              });
            };
          }, 70192: function(G, J, Z) {
            var te = Z(80337), ne = Z(6811).hoverlabel, ee = Z(93049).extendFlat;
            G.exports = { hoverlabel: { bgcolor: ee({}, ne.bgcolor, { arrayOk: !0 }), bordercolor: ee({}, ne.bordercolor, { arrayOk: !0 }), font: te({ arrayOk: !0, editType: "none" }), align: ee({}, ne.align, { arrayOk: !0 }), namelength: ee({}, ne.namelength, { arrayOk: !0 }), editType: "none" } };
          }, 83552: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626);
            function ee(ae, oe, se, ce) {
              ce = ce || te.identity, Array.isArray(ae) && (oe[0][se] = ce(ae));
            }
            G.exports = function(ae) {
              var oe = ae.calcdata, se = ae._fullLayout;
              function ce(he) {
                return function(me) {
                  return te.coerceHoverinfo({ hoverinfo: me }, { _module: he._module }, se);
                };
              }
              for (var le = 0; le < oe.length; le++) {
                var ue = oe[le], de = ue[0].trace;
                if (!ne.traceIs(de, "pie-like")) {
                  var fe = ne.traceIs(de, "2dMap") ? ee : te.fillArray;
                  fe(de.hoverinfo, ue, "hi", ce(de)), de.hovertemplate && fe(de.hovertemplate, ue, "ht"), de.hoverlabel && (fe(de.hoverlabel.bgcolor, ue, "hbg"), fe(de.hoverlabel.bordercolor, ue, "hbc"), fe(de.hoverlabel.font.size, ue, "hts"), fe(de.hoverlabel.font.color, ue, "htc"), fe(de.hoverlabel.font.family, ue, "htf"), fe(de.hoverlabel.font.weight, ue, "htw"), fe(de.hoverlabel.font.style, ue, "hty"), fe(de.hoverlabel.font.variant, ue, "htv"), fe(de.hoverlabel.namelength, ue, "hnl"), fe(de.hoverlabel.align, ue, "hta"));
                }
              }
            };
          }, 94225: function(G, J, Z) {
            var te = Z(33626), ne = Z(38103).hover;
            G.exports = function(ee, ae, oe) {
              var se = te.getComponentMethod("annotations", "onClick")(ee, ee._hoverdata);
              function ce() {
                ee.emit("plotly_click", { points: ee._hoverdata, event: ae });
              }
              oe !== void 0 && ne(ee, ae, oe, !0), ee._hoverdata && ae && ae.target && (se && se.then ? se.then(ce) : ce(), ae.stopImmediatePropagation && ae.stopImmediatePropagation());
            };
          }, 85988: function(G) {
            G.exports = { YANGLE: 60, HOVERARROWSIZE: 6, HOVERTEXTPAD: 3, HOVERFONTSIZE: 13, HOVERFONT: "Arial, sans-serif", HOVERMINTIME: 50, HOVERID: "-hover" };
          }, 3239: function(G, J, Z) {
            var te = Z(34809), ne = Z(70192), ee = Z(26430);
            G.exports = function(ae, oe, se, ce) {
              var le = te.extendFlat({}, ce.hoverlabel);
              oe.hovertemplate && (le.namelength = -1), ee(ae, oe, function(ue, de) {
                return te.coerce(ae, oe, ne, ue, de);
              }, le);
            };
          }, 36040: function(G, J, Z) {
            var te = Z(34809);
            J.getSubplot = function(ce) {
              return ce.subplot || ce.xaxis + ce.yaxis || ce.geo;
            }, J.isTraceInSubplots = function(ce, le) {
              if (ce.type === "splom") {
                for (var ue = ce.xaxes || [], de = ce.yaxes || [], fe = 0; fe < ue.length; fe++) for (var he = 0; he < de.length; he++) if (le.indexOf(ue[fe] + de[he]) !== -1) return !0;
                return !1;
              }
              return le.indexOf(J.getSubplot(ce)) !== -1;
            }, J.flat = function(ce, le) {
              for (var ue = new Array(ce.length), de = 0; de < ce.length; de++) ue[de] = le;
              return ue;
            }, J.p2c = function(ce, le) {
              for (var ue = new Array(ce.length), de = 0; de < ce.length; de++) ue[de] = ce[de].p2c(le);
              return ue;
            }, J.getDistanceFunction = function(ce, le, ue, de) {
              return ce === "closest" ? de || J.quadrature(le, ue) : ce.charAt(0) === "x" ? le : ue;
            }, J.getClosest = function(ce, le, ue) {
              if (ue.index !== !1) ue.index >= 0 && ue.index < ce.length ? ue.distance = 0 : ue.index = !1;
              else for (var de = 0; de < ce.length; de++) {
                var fe = le(ce[de]);
                fe <= ue.distance && (ue.index = de, ue.distance = fe);
              }
              return ue;
            }, J.inbox = function(ce, le, ue) {
              return ce * le < 0 || ce === 0 ? ue : 1 / 0;
            }, J.quadrature = function(ce, le) {
              return function(ue) {
                var de = ce(ue), fe = le(ue);
                return Math.sqrt(de * de + fe * fe);
              };
            }, J.makeEventData = function(ce, le, ue) {
              var de = "index" in ce ? ce.index : ce.pointNumber, fe = { data: le._input, fullData: le, curveNumber: le.index, pointNumber: de };
              if (le._indexToPoints) {
                var he = le._indexToPoints[de];
                he.length === 1 ? fe.pointIndex = he[0] : fe.pointIndices = he;
              } else fe.pointIndex = de;
              return le._module.eventData ? fe = le._module.eventData(fe, ce, le, ue, de) : ("xVal" in ce ? fe.x = ce.xVal : "x" in ce && (fe.x = ce.x), "yVal" in ce ? fe.y = ce.yVal : "y" in ce && (fe.y = ce.y), ce.xa && (fe.xaxis = ce.xa), ce.ya && (fe.yaxis = ce.ya), ce.zLabelVal !== void 0 && (fe.z = ce.zLabelVal)), J.appendArrayPointValue(fe, le, de), fe;
            }, J.appendArrayPointValue = function(ce, le, ue) {
              var de = le._arrayAttrs;
              if (de) for (var fe = 0; fe < de.length; fe++) {
                var he = de[fe], me = ee(he);
                if (ce[me] === void 0) {
                  var pe = ae(te.nestedProperty(le, he).get(), ue);
                  pe !== void 0 && (ce[me] = pe);
                }
              }
            }, J.appendArrayMultiPointValues = function(ce, le, ue) {
              var de = le._arrayAttrs;
              if (de) for (var fe = 0; fe < de.length; fe++) {
                var he = de[fe], me = ee(he);
                if (ce[me] === void 0) {
                  for (var pe = te.nestedProperty(le, he).get(), ye = new Array(ue.length), ge = 0; ge < ue.length; ge++) ye[ge] = ae(pe, ue[ge]);
                  ce[me] = ye;
                }
              }
            };
            var ne = { ids: "id", locations: "location", labels: "label", values: "value", "marker.colors": "color", parents: "parent" };
            function ee(ce) {
              return ne[ce] || ce;
            }
            function ae(ce, le) {
              return Array.isArray(le) ? Array.isArray(ce) && Array.isArray(ce[le[0]]) ? ce[le[0]][le[1]] : void 0 : ce[le];
            }
            var oe = { x: !0, y: !0 }, se = { "x unified": !0, "y unified": !0 };
            J.isUnifiedHover = function(ce) {
              return typeof ce == "string" && !!se[ce];
            }, J.isXYhover = function(ce) {
              return typeof ce == "string" && !!oe[ce];
            };
          }, 38103: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(65657), ae = Z(34809), oe = ae.pushUnique, se = ae.strTranslate, ce = ae.strRotate, le = Z(68596), ue = Z(30635), de = Z(93134), fe = Z(62203), he = Z(78766), me = Z(14751), pe = Z(29714), ye = Z(54826).zindexSeparator, ge = Z(33626), ve = Z(36040), be = Z(85988), Ce = Z(73970), Se = Z(6134), we = be.YANGLE, Ee = Math.PI * we / 180, xe = 1 / Math.sin(Ee), Le = Math.cos(Ee), Qe = Math.sin(Ee), ke = be.HOVERARROWSIZE, Me = be.HOVERTEXTPAD, Re = { box: !0, ohlc: !0, violin: !0, candlestick: !0 }, De = { scatter: !0, scattergl: !0, splom: !0 };
            function Pe(st, gt) {
              return st.distance - gt.distance;
            }
            function $e(st) {
              return [st.trace.index, st.index, st.x0, st.y0, st.name, st.attr, st.xa ? st.xa._id : "", st.ya ? st.ya._id : ""].join(",");
            }
            J.hover = function(st, gt, bt, yt) {
              st = ae.getGraphDiv(st);
              var St = gt.target;
              ae.throttle(st._fullLayout._uid + be.HOVERID, be.HOVERMINTIME, function() {
                (function(Lt, Rt, vt, $t, Pt) {
                  vt || (vt = "xy"), typeof vt == "string" && (vt = vt.split(ye)[0]);
                  var Xt, Nt, Gt, Ft = Array.isArray(vt) ? vt : [vt], Tt = Lt._fullLayout, It = Tt.hoversubplots, Ct = Tt._plots || [], Kt = Ct[vt], Vt = Tt._has("cartesian"), Yt = Rt.hovermode || Tt.hovermode, sn = (Yt || "").charAt(0) === "x", hn = (Yt || "").charAt(0) === "y";
                  if (Vt && (sn || hn) && It === "axis") {
                    for (var Qt = Ft.length, Ut = 0; Ut < Qt; Ut++) if (Ct[Xt = Ft[Ut]]) {
                      Nt = pe.getFromId(Lt, Xt, "x"), Gt = pe.getFromId(Lt, Xt, "y");
                      var Wt = (sn ? Nt : Gt)._subplotsWith;
                      if (Wt && Wt.length) for (var rn = 0; rn < Wt.length; rn++) oe(Ft, Wt[rn]);
                    }
                  }
                  if (Kt && It !== "single") {
                    var vn = Kt.overlays.map(function(vr) {
                      return vr.id;
                    });
                    Ft = Ft.concat(vn);
                  }
                  for (var dn = Ft.length, Ht = new Array(dn), jt = new Array(dn), cn = !1, wn = 0; wn < dn; wn++) if (Ct[Xt = Ft[wn]]) cn = !0, Ht[wn] = Ct[Xt].xaxis, jt[wn] = Ct[Xt].yaxis;
                  else {
                    if (!Tt[Xt] || !Tt[Xt]._subplot) return void ae.warn("Unrecognized subplot: " + Xt);
                    var Rn = Tt[Xt]._subplot;
                    Ht[wn] = Rn.xaxis, jt[wn] = Rn.yaxis;
                  }
                  if (Yt && !cn && (Yt = "closest"), ["x", "y", "closest", "x unified", "y unified"].indexOf(Yt) === -1 || !Lt.calcdata || Lt.querySelector(".zoombox") || Lt._dragging) return me.unhoverRaw(Lt, Rt);
                  var Pn = Tt.hoverdistance;
                  Pn === -1 && (Pn = 1 / 0);
                  var Ie = Tt.spikedistance;
                  Ie === -1 && (Ie = 1 / 0);
                  var rt, ft, pt, wt, Bt, At, Dt, Ot, un, yn, _n, gn, bn, an = [], Zt = [], Mn = { hLinePoint: null, vLinePoint: null }, Ln = !1;
                  if (Array.isArray(Rt)) for (Yt = "array", pt = 0; pt < Rt.length; pt++) (Bt = Lt.calcdata[Rt[pt].curveNumber || 0]) && (At = Bt[0].trace, Bt[0].trace.hoverinfo !== "skip" && (Zt.push(Bt), At.orientation === "h" && (Ln = !0)));
                  else {
                    var In, kn, er = Lt.calcdata.slice();
                    for (er.sort(function(vr, Rr) {
                      return (vr[0].trace.zorder || 0) - (Rr[0].trace.zorder || 0);
                    }), wt = 0; wt < er.length; wt++) Bt = er[wt], (At = Bt[0].trace).hoverinfo !== "skip" && ve.isTraceInSubplots(At, Ft) && (Zt.push(Bt), At.orientation === "h" && (Ln = !0));
                    if (Pt) {
                      if (le.triggerHandler(Lt, "plotly_beforehover", Rt) === !1) return;
                      var jn = Pt.getBoundingClientRect();
                      In = Rt.clientX - jn.left, kn = Rt.clientY - jn.top, Tt._calcInverseTransform(Lt);
                      var ur = ae.apply3DTransform(Tt._invTransform)(In, kn);
                      if (In = ur[0], kn = ur[1], In < 0 || In > Ht[0]._length || kn < 0 || kn > jt[0]._length) return me.unhoverRaw(Lt, Rt);
                    } else In = "xpx" in Rt ? Rt.xpx : Ht[0]._length / 2, kn = "ypx" in Rt ? Rt.ypx : jt[0]._length / 2;
                    if (Rt.pointerX = In + Ht[0]._offset, Rt.pointerY = kn + jt[0]._offset, rt = "xval" in Rt ? ve.flat(Ft, Rt.xval) : ve.p2c(Ht, In), ft = "yval" in Rt ? ve.flat(Ft, Rt.yval) : ve.p2c(jt, kn), !ne(rt[0]) || !ne(ft[0])) return ae.warn("Fx.hover failed", Rt, Lt), me.unhoverRaw(Lt, Rt);
                  }
                  var Kn = 1 / 0;
                  function Jn(vr, Rr) {
                    for (wt = 0; wt < Zt.length; wt++) if ((Bt = Zt[wt]) && Bt[0] && Bt[0].trace && (At = Bt[0].trace).visible === !0 && At._length !== 0 && ["carpet", "contourcarpet"].indexOf(At._module.name) === -1) {
                      if (un = Yt, ve.isUnifiedHover(un) && (un = un.charAt(0)), At.type === "splom" ? Dt = Ft[Ot = 0] : (Dt = ve.getSubplot(At), Ot = Ft.indexOf(Dt)), gn = { cd: Bt, trace: At, xa: Ht[Ot], ya: jt[Ot], maxHoverDistance: Pn, maxSpikeDistance: Ie, index: !1, distance: Math.min(Kn, Pn), spikeDistance: 1 / 0, xSpike: void 0, ySpike: void 0, color: he.defaultLine, name: At.name, x0: void 0, x1: void 0, y0: void 0, y1: void 0, xLabelVal: void 0, yLabelVal: void 0, zLabelVal: void 0, text: void 0 }, Tt[Dt] && (gn.subplot = Tt[Dt]._subplot), Tt._splomScenes && Tt._splomScenes[At.uid] && (gn.scene = Tt._splomScenes[At.uid]), un === "array") {
                        var Vr = Rt[wt];
                        "pointNumber" in Vr ? (gn.index = Vr.pointNumber, un = "closest") : (un = "", "xval" in Vr && (yn = Vr.xval, un = "x"), "yval" in Vr && (_n = Vr.yval, un = un ? "closest" : "y"));
                      } else vr !== void 0 && Rr !== void 0 ? (yn = vr, _n = Rr) : (yn = rt[Ot], _n = ft[Ot]);
                      if (bn = an.length, Pn !== 0) if (At._module && At._module.hoverPoints) {
                        var Xr = At._module.hoverPoints(gn, yn, _n, un, { finiteRange: !0, hoverLayer: Tt._hoverlayer, hoversubplots: It, gd: Lt });
                        if (Xr) for (var Jr, ti = 0; ti < Xr.length; ti++) Jr = Xr[ti], ne(Jr.x0) && ne(Jr.y0) && an.push(ze(Jr, Yt));
                      } else ae.log("Unrecognized trace type in hover:", At);
                      if (Yt === "closest" && an.length > bn && (an.splice(0, bn), Kn = an[0].distance), Vt && Ie !== 0 && an.length === 0) {
                        gn.distance = Ie, gn.index = !1;
                        var Hr = At._module.hoverPoints(gn, yn, _n, "closest", { hoverLayer: Tt._hoverlayer });
                        if (Hr && (Hr = Hr.filter(function(Ai) {
                          return Ai.spikeDistance <= Ie;
                        })), Hr && Hr.length) {
                          var Er, gr = Hr.filter(function(Ai) {
                            return Ai.xa.showspikes && Ai.xa.spikesnap !== "hovered data";
                          });
                          if (gr.length) {
                            var $r = gr[0];
                            ne($r.x0) && ne($r.y0) && (Er = mr($r), (!Mn.vLinePoint || Mn.vLinePoint.spikeDistance > Er.spikeDistance) && (Mn.vLinePoint = Er));
                          }
                          var li = Hr.filter(function(Ai) {
                            return Ai.ya.showspikes && Ai.ya.spikesnap !== "hovered data";
                          });
                          if (li.length) {
                            var Qi = li[0];
                            ne(Qi.x0) && ne(Qi.y0) && (Er = mr(Qi), (!Mn.hLinePoint || Mn.hLinePoint.spikeDistance > Er.spikeDistance) && (Mn.hLinePoint = Er));
                          }
                        }
                      }
                    }
                  }
                  function Lr(vr, Rr, Vr) {
                    for (var Xr, Jr = null, ti = 1 / 0, Hr = 0; Hr < vr.length; Hr++) Nt && Nt._id !== vr[Hr].xa._id || Gt && Gt._id !== vr[Hr].ya._id || (Xr = vr[Hr].spikeDistance, Vr && Hr === 0 && (Xr = -1 / 0), Xr <= ti && Xr <= Rr && (Jr = vr[Hr], ti = Xr));
                    return Jr;
                  }
                  function mr(vr) {
                    return vr ? { xa: vr.xa, ya: vr.ya, x: vr.xSpike !== void 0 ? vr.xSpike : (vr.x0 + vr.x1) / 2, y: vr.ySpike !== void 0 ? vr.ySpike : (vr.y0 + vr.y1) / 2, distance: vr.distance, spikeDistance: vr.spikeDistance, curveNumber: vr.trace.index, color: vr.color, pointNumber: vr.index } : null;
                  }
                  Jn();
                  var Ar = { fullLayout: Tt, container: Tt._hoverlayer, event: Rt };
                  Lt._spikepoints;
                  var Yr = { vLinePoint: Mn.vLinePoint, hLinePoint: Mn.hLinePoint };
                  Lt._spikepoints = Yr;
                  var jr = function() {
                    var vr = an.filter(function(Vr) {
                      return Nt && Nt._id === Vr.xa._id && Gt && Gt._id === Vr.ya._id;
                    }), Rr = an.filter(function(Vr) {
                      return !(Nt && Nt._id === Vr.xa._id && Gt && Gt._id === Vr.ya._id);
                    });
                    vr.sort(Pe), Rr.sort(Pe), an = function(Vr, Xr) {
                      for (var Jr = Xr.charAt(0), ti = [], Hr = [], Er = [], gr = 0; gr < Vr.length; gr++) {
                        var $r = Vr[gr];
                        ge.traceIs($r.trace, "bar-like") || ge.traceIs($r.trace, "box-violin") ? Er.push($r) : $r.trace[Jr + "period"] ? Hr.push($r) : ti.push($r);
                      }
                      return ti.concat(Hr).concat(Er);
                    }(an = vr.concat(Rr), Yt);
                  };
                  jr();
                  var Mr = Yt.charAt(0), Wr = (Mr === "x" || Mr === "y") && an[0] && De[an[0].trace.type];
                  if (Vt && Ie !== 0 && an.length !== 0) {
                    var Bi = Lr(an.filter(function(vr) {
                      return vr.ya.showspikes;
                    }), Ie, Wr);
                    Mn.hLinePoint = mr(Bi);
                    var gi = Lr(an.filter(function(vr) {
                      return vr.xa.showspikes;
                    }), Ie, Wr);
                    Mn.vLinePoint = mr(gi);
                  }
                  if (an.length === 0) {
                    var Zr = me.unhoverRaw(Lt, Rt);
                    return !Vt || Mn.hLinePoint === null && Mn.vLinePoint === null || We(Lt, Mn, Ar), Zr;
                  }
                  if (Vt && at() && We(Lt, Mn, Ar), ve.isXYhover(un) && an[0].length !== 0 && an[0].trace.type !== "splom") {
                    var fi = an[0], ri = (an = Re[fi.trace.type] ? an.filter(function(vr) {
                      return vr.trace.index === fi.trace.index;
                    }) : [fi]).length;
                    Jn(et("x", fi, Tt), et("y", fi, Tt));
                    var bi, Zi = [], $i = {}, Br = 0, Pr = function(vr) {
                      var Rr = Re[vr.trace.type] ? $e(vr) : vr.trace.index;
                      if ($i[Rr]) {
                        var Vr = $i[Rr] - 1, Xr = Zi[Vr];
                        Vr > 0 && Math.abs(vr.distance) < Math.abs(Xr.distance) && (Zi[Vr] = vr);
                      } else Br++, $i[Rr] = Br, Zi.push(vr);
                    };
                    for (bi = 0; bi < ri; bi++) Pr(an[bi]);
                    for (bi = an.length - 1; bi > ri - 1; bi--) Pr(an[bi]);
                    an = Zi, jr();
                  }
                  var ei = Lt._hoverdata, Ci = [], oa = dt(Lt), ka = Je(Lt);
                  for (pt = 0; pt < an.length; pt++) {
                    var aa = an[pt], hi = ve.makeEventData(aa, aa.trace, aa.cd);
                    if (aa.hovertemplate !== !1) {
                      var Gr = !1;
                      aa.cd[aa.index] && aa.cd[aa.index].ht && (Gr = aa.cd[aa.index].ht), aa.hovertemplate = Gr || aa.trace.hovertemplate || !1;
                    }
                    if (aa.xa && aa.ya) {
                      var Ca = aa.x0 + aa.xa._offset, Sr = aa.x1 + aa.xa._offset, Dr = aa.y0 + aa.ya._offset, si = aa.y1 + aa.ya._offset, zr = Math.min(Ca, Sr), di = Math.max(Ca, Sr), Da = Math.min(Dr, si), la = Math.max(Dr, si);
                      hi.bbox = { x0: zr + ka, x1: di + ka, y0: Da + oa, y1: la + oa };
                    }
                    aa.eventData = [hi], Ci.push(hi);
                  }
                  Lt._hoverdata = Ci;
                  var wa = Yt === "y" && (Zt.length > 1 || an.length > 1) || Yt === "closest" && Ln && an.length > 1, Wa = he.combine(Tt.plot_bgcolor || he.background, Tt.paper_bgcolor), Qr = Xe(an, { gd: Lt, hovermode: Yt, rotateLabels: wa, bgColor: Wa, container: Tt._hoverlayer, outerContainer: Tt._paper.node(), commonLabelOpts: Tt.hoverlabel, hoverdistance: Tt.hoverdistance }), Ur = Qr.hoverLabels;
                  if (ve.isUnifiedHover(Yt) || (function(vr, Rr, Vr, Xr) {
                    var Jr, ti, Hr, Er, gr, $r, li, Qi = Rr ? "xa" : "ya", Ai = Rr ? "ya" : "xa", Li = 0, Ni = 1, Pi = vr.size(), Oi = new Array(Pi), ki = 0, Mi = Xr.minX, Vi = Xr.maxX, ua = Xr.minY, Ki = Xr.maxY, Fi = function(Fr) {
                      return Fr * Vr._invScaleX;
                    }, na = function(Fr) {
                      return Fr * Vr._invScaleY;
                    };
                    function nn(Fr) {
                      var Ti = Fr[0], Si = Fr[Fr.length - 1];
                      if (ti = Ti.pmin - Ti.pos - Ti.dp + Ti.size, Hr = Si.pos + Si.dp + Si.size - Ti.pmax, ti > 0.01) {
                        for (gr = Fr.length - 1; gr >= 0; gr--) Fr[gr].dp += ti;
                        Jr = !1;
                      }
                      if (!(Hr < 0.01)) {
                        if (ti < -0.01) {
                          for (gr = Fr.length - 1; gr >= 0; gr--) Fr[gr].dp -= Hr;
                          Jr = !1;
                        }
                        if (Jr) {
                          var Ei = 0;
                          for (Er = 0; Er < Fr.length; Er++) ($r = Fr[Er]).pos + $r.dp + $r.size > Ti.pmax && Ei++;
                          for (Er = Fr.length - 1; Er >= 0 && !(Ei <= 0); Er--) ($r = Fr[Er]).pos > Ti.pmax - 1 && ($r.del = !0, Ei--);
                          for (Er = 0; Er < Fr.length && !(Ei <= 0); Er++) if (($r = Fr[Er]).pos < Ti.pmin + 1) for ($r.del = !0, Ei--, Hr = 2 * $r.size, gr = Fr.length - 1; gr >= 0; gr--) Fr[gr].dp -= Hr;
                          for (Er = Fr.length - 1; Er >= 0 && !(Ei <= 0); Er--) ($r = Fr[Er]).pos + $r.dp + $r.size > Ti.pmax && ($r.del = !0, Ei--);
                        }
                      }
                    }
                    for (vr.each(function(Fr) {
                      var Ti = Fr[Qi], Si = Fr[Ai], Ei = Ti._id.charAt(0) === "x", Fa = Ti.range;
                      ki === 0 && Fa && Fa[0] > Fa[1] !== Ei && (Ni = -1);
                      var $o = 0, Gs = Ei ? Vr.width : Vr.height;
                      if (Vr.hovermode === "x" || Vr.hovermode === "y") {
                        var Al, ul, gc = Ze(Fr, Rr), Gl = Fr.anchor, Sl = Gl === "end" ? -1 : 1;
                        if (Gl === "middle") ul = (Al = Fr.crossPos + (Ei ? na(gc.y - Fr.by / 2) : Fi(Fr.bx / 2 + Fr.tx2width / 2))) + (Ei ? na(Fr.by) : Fi(Fr.bx));
                        else if (Ei) ul = (Al = Fr.crossPos + na(ke + gc.y) - na(Fr.by / 2 - ke)) + na(Fr.by);
                        else {
                          var ls = Fi(Sl * ke + gc.x), vs = ls + Fi(Sl * Fr.bx);
                          Al = Fr.crossPos + Math.min(ls, vs), ul = Fr.crossPos + Math.max(ls, vs);
                        }
                        Ei ? ua !== void 0 && Ki !== void 0 && Math.min(ul, Ki) - Math.max(Al, ua) > 1 && (Si.side === "left" ? ($o = Si._mainLinePosition, Gs = Vr.width) : Gs = Si._mainLinePosition) : Mi !== void 0 && Vi !== void 0 && Math.min(ul, Vi) - Math.max(Al, Mi) > 1 && (Si.side === "top" ? ($o = Si._mainLinePosition, Gs = Vr.height) : Gs = Si._mainLinePosition);
                      }
                      Oi[ki++] = [{ datum: Fr, traceIndex: Fr.trace.index, dp: 0, pos: Fr.pos, posref: Fr.posref, size: Fr.by * (Ei ? xe : 1) / 2, pmin: $o, pmax: Gs }];
                    }), Oi.sort(function(Fr, Ti) {
                      return Fr[0].posref - Ti[0].posref || Ni * (Ti[0].traceIndex - Fr[0].traceIndex);
                    }); !Jr && Li <= Pi; ) {
                      for (Li++, Jr = !0, Er = 0; Er < Oi.length - 1; ) {
                        var Tn = Oi[Er], Dn = Oi[Er + 1], Fn = Tn[Tn.length - 1], fr = Dn[0];
                        if ((ti = Fn.pos + Fn.dp + Fn.size - fr.pos - fr.dp + fr.size) > 0.01) {
                          for (gr = Dn.length - 1; gr >= 0; gr--) Dn[gr].dp += ti;
                          for (Tn.push.apply(Tn, Dn), Oi.splice(Er + 1, 1), li = 0, gr = Tn.length - 1; gr >= 0; gr--) li += Tn[gr].dp;
                          for (Hr = li / Tn.length, gr = Tn.length - 1; gr >= 0; gr--) Tn[gr].dp -= Hr;
                          Jr = !1;
                        } else Er++;
                      }
                      Oi.forEach(nn);
                    }
                    for (Er = Oi.length - 1; Er >= 0; Er--) {
                      var dr = Oi[Er];
                      for (gr = dr.length - 1; gr >= 0; gr--) {
                        var yr = dr[gr], ai = yr.datum;
                        ai.offset = yr.dp, ai.del = yr.del;
                      }
                    }
                  }(Ur, wa, Tt, Qr.commonLabelBoundingBox), Ge(Ur, wa, Tt._invScaleX, Tt._invScaleY)), Pt && Pt.tagName) {
                    var rr = ge.getComponentMethod("annotations", "hasClickToShow")(Lt, Ci);
                    de(te.select(Pt), rr ? "pointer" : "");
                  }
                  Pt && !$t && function(vr, Rr, Vr) {
                    if (!Vr || Vr.length !== vr._hoverdata.length) return !0;
                    for (var Xr = Vr.length - 1; Xr >= 0; Xr--) {
                      var Jr = Vr[Xr], ti = vr._hoverdata[Xr];
                      if (Jr.curveNumber !== ti.curveNumber || String(Jr.pointNumber) !== String(ti.pointNumber) || String(Jr.pointNumbers) !== String(ti.pointNumbers)) return !0;
                    }
                    return !1;
                  }(Lt, 0, ei) && (ei && Lt.emit("plotly_unhover", { event: Rt, points: ei }), Lt.emit("plotly_hover", { event: Rt, points: Lt._hoverdata, xaxes: Ht, yaxes: jt, xvals: rt, yvals: ft }));
                })(st, gt, bt, yt, St);
              });
            }, J.loneHover = function(st, gt) {
              var bt = !0;
              Array.isArray(st) || (bt = !1, st = [st]);
              var yt = gt.gd, St = dt(yt), Lt = Je(yt), Rt = Xe(st.map(function(Pt) {
                var Xt = Pt._x0 || Pt.x0 || Pt.x || 0, Nt = Pt._x1 || Pt.x1 || Pt.x || 0, Gt = Pt._y0 || Pt.y0 || Pt.y || 0, Ft = Pt._y1 || Pt.y1 || Pt.y || 0, Tt = Pt.eventData;
                if (Tt) {
                  var It = Math.min(Xt, Nt), Ct = Math.max(Xt, Nt), Kt = Math.min(Gt, Ft), Vt = Math.max(Gt, Ft), Yt = Pt.trace;
                  if (ge.traceIs(Yt, "gl3d")) {
                    var sn = yt._fullLayout[Yt.scene]._scene.container, hn = sn.offsetLeft, Qt = sn.offsetTop;
                    It += hn, Ct += hn, Kt += Qt, Vt += Qt;
                  }
                  Tt.bbox = { x0: It + Lt, x1: Ct + Lt, y0: Kt + St, y1: Vt + St }, gt.inOut_bbox && gt.inOut_bbox.push(Tt.bbox);
                } else Tt = !1;
                return { color: Pt.color || he.defaultLine, x0: Pt.x0 || Pt.x || 0, x1: Pt.x1 || Pt.x || 0, y0: Pt.y0 || Pt.y || 0, y1: Pt.y1 || Pt.y || 0, xLabel: Pt.xLabel, yLabel: Pt.yLabel, zLabel: Pt.zLabel, text: Pt.text, name: Pt.name, idealAlign: Pt.idealAlign, borderColor: Pt.borderColor, fontFamily: Pt.fontFamily, fontSize: Pt.fontSize, fontColor: Pt.fontColor, fontWeight: Pt.fontWeight, fontStyle: Pt.fontStyle, fontVariant: Pt.fontVariant, nameLength: Pt.nameLength, textAlign: Pt.textAlign, trace: Pt.trace || { index: 0, hoverinfo: "" }, xa: { _offset: 0 }, ya: { _offset: 0 }, index: 0, hovertemplate: Pt.hovertemplate || !1, hovertemplateLabels: Pt.hovertemplateLabels || !1, eventData: Tt };
              }), { gd: yt, hovermode: "closest", rotateLabels: !1, bgColor: gt.bgColor || he.background, container: te.select(gt.container), outerContainer: gt.outerContainer || gt.container }).hoverLabels, vt = 0, $t = 0;
              return Rt.sort(function(Pt, Xt) {
                return Pt.y0 - Xt.y0;
              }).each(function(Pt, Xt) {
                var Nt = Pt.y0 - Pt.by / 2;
                Pt.offset = Nt - 5 < vt ? vt - Nt + 5 : 0, vt = Nt + Pt.by + Pt.offset, Xt === gt.anchorIndex && ($t = Pt.offset);
              }).each(function(Pt) {
                Pt.offset -= $t;
              }), Ge(Rt, !1, yt._fullLayout._invScaleX, yt._fullLayout._invScaleY), bt ? Rt : Rt.node();
            };
            var qe = /<extra>([\s\S]*)<\/extra>/;
            function Xe(st, gt) {
              var bt = gt.gd, yt = bt._fullLayout, St = gt.hovermode, Lt = gt.rotateLabels, Rt = gt.bgColor, vt = gt.container, $t = gt.outerContainer, Pt = gt.commonLabelOpts || {};
              if (st.length === 0) return [[]];
              var Xt = gt.fontFamily || be.HOVERFONT, Nt = gt.fontSize || be.HOVERFONTSIZE, Gt = gt.fontWeight || yt.font.weight, Ft = gt.fontStyle || yt.font.style, Tt = gt.fontVariant || yt.font.variant, It = gt.fontTextcase || yt.font.textcase, Ct = gt.fontLineposition || yt.font.lineposition, Kt = gt.fontShadow || yt.font.shadow, Vt = st[0], Yt = Vt.xa, sn = Vt.ya, hn = St.charAt(0), Qt = hn + "Label", Ut = Vt[Qt];
              if (Ut === void 0 && Yt.type === "multicategory") for (var Wt = 0; Wt < st.length && (Ut = st[Wt][Qt]) === void 0; Wt++) ;
              var rn = it(bt, $t), vn = rn.top, dn = rn.width, Ht = rn.height, jt = Ut !== void 0 && Vt.distance <= gt.hoverdistance && (St === "x" || St === "y");
              if (jt) {
                var cn, wn, Rn = !0;
                for (cn = 0; cn < st.length; cn++) if (Rn && st[cn].zLabel === void 0 && (Rn = !1), wn = st[cn].hoverinfo || st[cn].trace.hoverinfo) {
                  var Pn = Array.isArray(wn) ? wn : wn.split("+");
                  if (Pn.indexOf("all") === -1 && Pn.indexOf(St) === -1) {
                    jt = !1;
                    break;
                  }
                }
                Rn && (jt = !1);
              }
              var Ie = vt.selectAll("g.axistext").data(jt ? [0] : []);
              Ie.enter().append("g").classed("axistext", !0), Ie.exit().remove();
              var rt = { minX: 0, maxX: 0, minY: 0, maxY: 0 };
              if (Ie.each(function() {
                var Wr = te.select(this), Bi = ae.ensureSingle(Wr, "path", "", function(di) {
                  di.style({ "stroke-width": "1px" });
                }), gi = ae.ensureSingle(Wr, "text", "", function(di) {
                  di.attr("data-notex", 1);
                }), Zr = Pt.bgcolor || he.defaultLine, fi = Pt.bordercolor || he.contrast(Zr), ri = he.contrast(Zr), bi = Pt.font, Zi = { weight: bi.weight || Gt, style: bi.style || Ft, variant: bi.variant || Tt, textcase: bi.textcase || It, lineposition: bi.lineposition || Ct, shadow: bi.shadow || Kt, family: bi.family || Xt, size: bi.size || Nt, color: bi.color || ri };
                Bi.style({ fill: Zr, stroke: fi }), gi.text(Ut).call(fe.font, Zi).call(ue.positionText, 0, 0).call(ue.convertToTspans, bt), Wr.attr("transform", "");
                var $i, Br, Pr = it(bt, gi.node());
                if (St === "x") {
                  var ei = Yt.side === "top" ? "-" : "";
                  gi.attr("text-anchor", "middle").call(ue.positionText, 0, Yt.side === "top" ? vn - Pr.bottom - ke - Me : vn - Pr.top + ke + Me), $i = Yt._offset + (Vt.x0 + Vt.x1) / 2, Br = sn._offset + (Yt.side === "top" ? 0 : sn._length);
                  var Ci = Pr.width / 2 + Me, oa = $i;
                  $i < Ci ? oa = Ci : $i > yt.width - Ci && (oa = yt.width - Ci), Bi.attr("d", "M" + ($i - oa) + ",0L" + ($i - oa + ke) + "," + ei + ke + "H" + Ci + "v" + ei + (2 * Me + Pr.height) + "H" + -Ci + "V" + ei + ke + "H" + ($i - oa - ke) + "Z"), $i = oa, rt.minX = $i - Ci, rt.maxX = $i + Ci, Yt.side === "top" ? (rt.minY = Br - (2 * Me + Pr.height), rt.maxY = Br - Me) : (rt.minY = Br + Me, rt.maxY = Br + (2 * Me + Pr.height));
                } else {
                  var ka, aa, hi;
                  sn.side === "right" ? (ka = "start", aa = 1, hi = "", $i = Yt._offset + Yt._length) : (ka = "end", aa = -1, hi = "-", $i = Yt._offset), Br = sn._offset + (Vt.y0 + Vt.y1) / 2, gi.attr("text-anchor", ka), Bi.attr("d", "M0,0L" + hi + ke + "," + ke + "V" + (Me + Pr.height / 2) + "h" + hi + (2 * Me + Pr.width) + "V-" + (Me + Pr.height / 2) + "H" + hi + ke + "V-" + ke + "Z"), rt.minY = Br - (Me + Pr.height / 2), rt.maxY = Br + (Me + Pr.height / 2), sn.side === "right" ? (rt.minX = $i + ke, rt.maxX = $i + ke + (2 * Me + Pr.width)) : (rt.minX = $i - ke - (2 * Me + Pr.width), rt.maxX = $i - ke);
                  var Gr, Ca = Pr.height / 2, Sr = vn - Pr.top - Ca, Dr = "clip" + yt._uid + "commonlabel" + sn._id;
                  if ($i < Pr.width + 2 * Me + ke) {
                    Gr = "M-" + (ke + Me) + "-" + Ca + "h-" + (Pr.width - Me) + "V" + Ca + "h" + (Pr.width - Me) + "Z";
                    var si = Pr.width - $i + Me;
                    ue.positionText(gi, si, Sr), ka === "end" && gi.selectAll("tspan").each(function() {
                      var di = te.select(this), Da = fe.tester.append("text").text(di.text()).call(fe.font, Zi), la = it(bt, Da.node());
                      Math.round(la.width) < Math.round(Pr.width) && di.attr("x", si - la.width), Da.remove();
                    });
                  } else ue.positionText(gi, aa * (Me + ke), Sr), Gr = null;
                  var zr = yt._topclips.selectAll("#" + Dr).data(Gr ? [0] : []);
                  zr.enter().append("clipPath").attr("id", Dr).append("path"), zr.exit().remove(), zr.select("path").attr("d", Gr), fe.setClipUrl(gi, Gr ? Dr : null, bt);
                }
                Wr.attr("transform", se($i, Br));
              }), ve.isUnifiedHover(St)) {
                vt.selectAll("g.hovertext").remove();
                var ft = st.filter(function(Wr) {
                  return Wr.hoverinfo !== "none";
                });
                if (ft.length === 0) return [];
                var pt = yt.hoverlabel, wt = pt.font, Bt = { showlegend: !0, legend: { title: { text: Ut, font: wt }, font: wt, bgcolor: pt.bgcolor, bordercolor: pt.bordercolor, borderwidth: 1, tracegroupgap: 7, traceorder: yt.legend ? yt.legend.traceorder : void 0, orientation: "v" } }, At = { font: wt };
                Ce(Bt, At, bt._fullData);
                var Dt = At.legend;
                Dt.entries = [];
                for (var Ot = 0; Ot < ft.length; Ot++) {
                  var un = ft[Ot];
                  if (un.hoverinfo !== "none") {
                    var yn = Ve(un, !0, St, yt, Ut), _n = yn[0], gn = yn[1];
                    un.name = gn, un.text = gn !== "" ? gn + " : " + _n : _n;
                    var bn = un.cd[un.index];
                    bn && (bn.mc && (un.mc = bn.mc), bn.mcc && (un.mc = bn.mcc), bn.mlc && (un.mlc = bn.mlc), bn.mlcc && (un.mlc = bn.mlcc), bn.mlw && (un.mlw = bn.mlw), bn.mrc && (un.mrc = bn.mrc), bn.dir && (un.dir = bn.dir)), un._distinct = !0, Dt.entries.push([un]);
                  }
                }
                Dt.entries.sort(function(Wr, Bi) {
                  return Wr[0].trace.index - Bi[0].trace.index;
                }), Dt.layer = vt, Dt._inHover = !0, Dt._groupTitleFont = pt.grouptitlefont, Se(bt, Dt);
                var an, Zt, Mn, Ln, In = vt.select("g.legend"), kn = it(bt, In.node()), er = kn.width + 2 * Me, jn = kn.height + 2 * Me, ur = ft[0], Kn = (ur.x0 + ur.x1) / 2, Jn = (ur.y0 + ur.y1) / 2, Lr = !(ge.traceIs(ur.trace, "bar-like") || ge.traceIs(ur.trace, "box-violin"));
                hn === "y" ? Lr ? (Zt = Jn - Me, an = Jn + Me) : (Zt = Math.min.apply(null, ft.map(function(Wr) {
                  return Math.min(Wr.y0, Wr.y1);
                })), an = Math.max.apply(null, ft.map(function(Wr) {
                  return Math.max(Wr.y0, Wr.y1);
                }))) : Zt = an = ae.mean(ft.map(function(Wr) {
                  return (Wr.y0 + Wr.y1) / 2;
                })) - jn / 2, hn === "x" ? Lr ? (Mn = Kn + Me, Ln = Kn - Me) : (Mn = Math.max.apply(null, ft.map(function(Wr) {
                  return Math.max(Wr.x0, Wr.x1);
                })), Ln = Math.min.apply(null, ft.map(function(Wr) {
                  return Math.min(Wr.x0, Wr.x1);
                }))) : Mn = Ln = ae.mean(ft.map(function(Wr) {
                  return (Wr.x0 + Wr.x1) / 2;
                })) - er / 2;
                var mr, Ar, Yr = Yt._offset, jr = sn._offset;
                return Ln += Yr - er, Zt += jr - jn, mr = (Mn += Yr) + er < dn && Mn >= 0 ? Mn : Ln + er < dn && Ln >= 0 ? Ln : Yr + er < dn ? Yr : Mn - Kn < Kn - Ln + er ? dn - er : 0, mr += Me, Ar = (an += jr) + jn < Ht && an >= 0 ? an : Zt + jn < Ht && Zt >= 0 ? Zt : jr + jn < Ht ? jr : an - Jn < Jn - Zt + jn ? Ht - jn : 0, Ar += Me, In.attr("transform", se(mr - 1, Ar - 1)), In;
              }
              var Mr = vt.selectAll("g.hovertext").data(st, function(Wr) {
                return $e(Wr);
              });
              return Mr.enter().append("g").classed("hovertext", !0).each(function() {
                var Wr = te.select(this);
                Wr.append("rect").call(he.fill, he.addOpacity(Rt, 0.8)), Wr.append("text").classed("name", !0), Wr.append("path").style("stroke-width", "1px"), Wr.append("text").classed("nums", !0).call(fe.font, { weight: Gt, style: Ft, variant: Tt, textcase: It, lineposition: Ct, shadow: Kt, family: Xt, size: Nt });
              }), Mr.exit().remove(), Mr.each(function(Wr) {
                var Bi = te.select(this).attr("transform", ""), gi = Wr.color;
                Array.isArray(gi) && (gi = gi[Wr.eventData[0].pointNumber]);
                var Zr = Wr.bgcolor || gi, fi = he.combine(he.opacity(Zr) ? Zr : he.defaultLine, Rt), ri = he.combine(he.opacity(gi) ? gi : he.defaultLine, Rt), bi = Wr.borderColor || he.contrast(fi), Zi = Ve(Wr, jt, St, yt, Ut, Bi), $i = Zi[0], Br = Zi[1], Pr = Bi.select("text.nums").call(fe.font, { family: Wr.fontFamily || Xt, size: Wr.fontSize || Nt, color: Wr.fontColor || bi, weight: Wr.fontWeight || Gt, style: Wr.fontStyle || Ft, variant: Wr.fontVariant || Tt, textcase: Wr.fontTextcase || It, lineposition: Wr.fontLineposition || Ct, shadow: Wr.fontShadow || Kt }).text($i).attr("data-notex", 1).call(ue.positionText, 0, 0).call(ue.convertToTspans, bt), ei = Bi.select("text.name"), Ci = 0, oa = 0;
                if (Br && Br !== $i) {
                  ei.call(fe.font, { family: Wr.fontFamily || Xt, size: Wr.fontSize || Nt, color: ri, weight: Wr.fontWeight || Gt, style: Wr.fontStyle || Ft, variant: Wr.fontVariant || Tt, textcase: Wr.fontTextcase || It, lineposition: Wr.fontLineposition || Ct, shadow: Wr.fontShadow || Kt }).text(Br).attr("data-notex", 1).call(ue.positionText, 0, 0).call(ue.convertToTspans, bt);
                  var ka = it(bt, ei.node());
                  Ci = ka.width + 2 * Me, oa = ka.height + 2 * Me;
                } else ei.remove(), Bi.select("rect").remove();
                Bi.select("path").style({ fill: fi, stroke: bi });
                var aa = Wr.xa._offset + (Wr.x0 + Wr.x1) / 2, hi = Wr.ya._offset + (Wr.y0 + Wr.y1) / 2, Gr = Math.abs(Wr.x1 - Wr.x0), Ca = Math.abs(Wr.y1 - Wr.y0), Sr = it(bt, Pr.node()), Dr = Sr.width / yt._invScaleX, si = Sr.height / yt._invScaleY;
                Wr.ty0 = (vn - Sr.top) / yt._invScaleY, Wr.bx = Dr + 2 * Me, Wr.by = Math.max(si + 2 * Me, oa), Wr.anchor = "start", Wr.txwidth = Dr, Wr.tx2width = Ci, Wr.offset = 0;
                var zr, di, Da = (Dr + ke + Me + Ci) * yt._invScaleX;
                if (Lt) Wr.pos = aa, zr = hi + Ca / 2 + Da <= Ht, di = hi - Ca / 2 - Da >= 0, Wr.idealAlign !== "top" && zr || !di ? zr ? (hi += Ca / 2, Wr.anchor = "start") : Wr.anchor = "middle" : (hi -= Ca / 2, Wr.anchor = "end"), Wr.crossPos = hi;
                else {
                  if (Wr.pos = hi, zr = aa + Gr / 2 + Da <= dn, di = aa - Gr / 2 - Da >= 0, Wr.idealAlign !== "left" && zr || !di) if (zr) aa += Gr / 2, Wr.anchor = "start";
                  else {
                    Wr.anchor = "middle";
                    var la = Da / 2, wa = aa + la - dn, Wa = aa - la;
                    wa > 0 && (aa -= wa), Wa < 0 && (aa += -Wa);
                  }
                  else aa -= Gr / 2, Wr.anchor = "end";
                  Wr.crossPos = aa;
                }
                Pr.attr("text-anchor", Wr.anchor), Ci && ei.attr("text-anchor", Wr.anchor), Bi.attr("transform", se(aa, hi) + (Lt ? ce(we) : ""));
              }), { hoverLabels: Mr, commonLabelBoundingBox: rt };
            }
            function Ve(st, gt, bt, yt, St, Lt) {
              var Rt = "", vt = "";
              st.nameOverride !== void 0 && (st.name = st.nameOverride), st.name && (st.trace._meta && (st.name = ae.templateString(st.name, st.trace._meta)), Rt = ct(st.name, st.nameLength));
              var $t = bt.charAt(0), Pt = $t === "x" ? "y" : "x";
              st.zLabel !== void 0 ? (st.xLabel !== void 0 && (vt += "x: " + st.xLabel + "<br>"), st.yLabel !== void 0 && (vt += "y: " + st.yLabel + "<br>"), st.trace.type !== "choropleth" && st.trace.type !== "choroplethmapbox" && st.trace.type !== "choroplethmap" && (vt += (vt ? "z: " : "") + st.zLabel)) : gt && st[$t + "Label"] === St ? vt = st[Pt + "Label"] || "" : st.xLabel === void 0 ? st.yLabel !== void 0 && st.trace.type !== "scattercarpet" && (vt = st.yLabel) : vt = st.yLabel === void 0 ? st.xLabel : "(" + st.xLabel + ", " + st.yLabel + ")", !st.text && st.text !== 0 || Array.isArray(st.text) || (vt += (vt ? "<br>" : "") + st.text), st.extraText !== void 0 && (vt += (vt ? "<br>" : "") + st.extraText), Lt && vt === "" && !st.hovertemplate && (Rt === "" && Lt.remove(), vt = Rt);
              var Xt = st.hovertemplate || !1;
              if (Xt) {
                var Nt = st.hovertemplateLabels || st;
                st[$t + "Label"] !== St && (Nt[$t + "other"] = Nt[$t + "Val"], Nt[$t + "otherLabel"] = Nt[$t + "Label"]), vt = (vt = ae.hovertemplateString(Xt, Nt, yt._d3locale, st.eventData[0] || {}, st.trace._meta)).replace(qe, function(Gt, Ft) {
                  return Rt = ct(Ft, st.nameLength), "";
                });
              }
              return [vt, Rt];
            }
            function Ze(st, gt) {
              var bt = 0, yt = st.offset;
              return gt && (yt *= -Qe, bt = st.offset * Le), { x: bt, y: yt };
            }
            function Ge(st, gt, bt, yt) {
              var St = function(Rt) {
                return Rt * bt;
              }, Lt = function(Rt) {
                return Rt * yt;
              };
              st.each(function(Rt) {
                var vt = te.select(this);
                if (Rt.del) return vt.remove();
                var $t, Pt, Xt, Nt, Gt = vt.select("text.nums"), Ft = Rt.anchor, Tt = Ft === "end" ? -1 : 1, It = (Nt = (Xt = (Pt = { start: 1, end: -1, middle: 0 }[($t = Rt).anchor]) * (ke + Me)) + Pt * ($t.txwidth + Me), $t.anchor === "middle" && (Xt -= $t.tx2width / 2, Nt += $t.txwidth / 2 + Me), { alignShift: Pt, textShiftX: Xt, text2ShiftX: Nt }), Ct = Ze(Rt, gt), Kt = Ct.x, Vt = Ct.y, Yt = Ft === "middle";
                vt.select("path").attr("d", Yt ? "M-" + St(Rt.bx / 2 + Rt.tx2width / 2) + "," + Lt(Vt - Rt.by / 2) + "h" + St(Rt.bx) + "v" + Lt(Rt.by) + "h-" + St(Rt.bx) + "Z" : "M0,0L" + St(Tt * ke + Kt) + "," + Lt(ke + Vt) + "v" + Lt(Rt.by / 2 - ke) + "h" + St(Tt * Rt.bx) + "v-" + Lt(Rt.by) + "H" + St(Tt * ke + Kt) + "V" + Lt(Vt - ke) + "Z");
                var sn = Kt + It.textShiftX, hn = Vt + Rt.ty0 - Rt.by / 2 + Me, Qt = Rt.textAlign || "auto";
                Qt !== "auto" && (Qt === "left" && Ft !== "start" ? (Gt.attr("text-anchor", "start"), sn = Yt ? -Rt.bx / 2 - Rt.tx2width / 2 + Me : -Rt.bx - Me) : Qt === "right" && Ft !== "end" && (Gt.attr("text-anchor", "end"), sn = Yt ? Rt.bx / 2 - Rt.tx2width / 2 - Me : Rt.bx + Me)), Gt.call(ue.positionText, St(sn), Lt(hn)), Rt.tx2width && (vt.select("text.name").call(ue.positionText, St(It.text2ShiftX + It.alignShift * Me + Kt), Lt(Vt + Rt.ty0 - Rt.by / 2 + Me)), vt.select("rect").call(fe.setRect, St(It.text2ShiftX + (It.alignShift - 1) * Rt.tx2width / 2 + Kt), Lt(Vt - Rt.by / 2 - 1), St(Rt.tx2width), Lt(Rt.by + 2)));
              });
            }
            function ze(st, gt) {
              var bt = st.index, yt = st.trace || {}, St = st.cd[0], Lt = st.cd[bt] || {};
              function Rt(Gt) {
                return Gt || ne(Gt) && Gt === 0;
              }
              var vt = Array.isArray(bt) ? function(Gt, Ft) {
                var Tt = ae.castOption(St, bt, Gt);
                return Rt(Tt) ? Tt : ae.extractOption({}, yt, "", Ft);
              } : function(Gt, Ft) {
                return ae.extractOption(Lt, yt, Gt, Ft);
              };
              function $t(Gt, Ft, Tt) {
                var It = vt(Ft, Tt);
                Rt(It) && (st[Gt] = It);
              }
              if ($t("hoverinfo", "hi", "hoverinfo"), $t("bgcolor", "hbg", "hoverlabel.bgcolor"), $t("borderColor", "hbc", "hoverlabel.bordercolor"), $t("fontFamily", "htf", "hoverlabel.font.family"), $t("fontSize", "hts", "hoverlabel.font.size"), $t("fontColor", "htc", "hoverlabel.font.color"), $t("fontWeight", "htw", "hoverlabel.font.weight"), $t("fontStyle", "hty", "hoverlabel.font.style"), $t("fontVariant", "htv", "hoverlabel.font.variant"), $t("nameLength", "hnl", "hoverlabel.namelength"), $t("textAlign", "hta", "hoverlabel.align"), st.posref = gt === "y" || gt === "closest" && yt.orientation === "h" ? st.xa._offset + (st.x0 + st.x1) / 2 : st.ya._offset + (st.y0 + st.y1) / 2, st.x0 = ae.constrain(st.x0, 0, st.xa._length), st.x1 = ae.constrain(st.x1, 0, st.xa._length), st.y0 = ae.constrain(st.y0, 0, st.ya._length), st.y1 = ae.constrain(st.y1, 0, st.ya._length), st.xLabelVal !== void 0 && (st.xLabel = "xLabel" in st ? st.xLabel : pe.hoverLabelText(st.xa, st.xLabelVal, yt.xhoverformat), st.xVal = st.xa.c2d(st.xLabelVal)), st.yLabelVal !== void 0 && (st.yLabel = "yLabel" in st ? st.yLabel : pe.hoverLabelText(st.ya, st.yLabelVal, yt.yhoverformat), st.yVal = st.ya.c2d(st.yLabelVal)), st.zLabelVal !== void 0 && st.zLabel === void 0 && (st.zLabel = String(st.zLabelVal)), !(isNaN(st.xerr) || st.xa.type === "log" && st.xerr <= 0)) {
                var Pt = pe.tickText(st.xa, st.xa.c2l(st.xerr), "hover").text;
                st.xerrneg !== void 0 ? st.xLabel += " +" + Pt + " / -" + pe.tickText(st.xa, st.xa.c2l(st.xerrneg), "hover").text : st.xLabel += " ± " + Pt, gt === "x" && (st.distance += 1);
              }
              if (!(isNaN(st.yerr) || st.ya.type === "log" && st.yerr <= 0)) {
                var Xt = pe.tickText(st.ya, st.ya.c2l(st.yerr), "hover").text;
                st.yerrneg !== void 0 ? st.yLabel += " +" + Xt + " / -" + pe.tickText(st.ya, st.ya.c2l(st.yerrneg), "hover").text : st.yLabel += " ± " + Xt, gt === "y" && (st.distance += 1);
              }
              var Nt = st.hoverinfo || st.trace.hoverinfo;
              return Nt && Nt !== "all" && ((Nt = Array.isArray(Nt) ? Nt : Nt.split("+")).indexOf("x") === -1 && (st.xLabel = void 0), Nt.indexOf("y") === -1 && (st.yLabel = void 0), Nt.indexOf("z") === -1 && (st.zLabel = void 0), Nt.indexOf("text") === -1 && (st.text = void 0), Nt.indexOf("name") === -1 && (st.name = void 0)), st;
            }
            function We(st, gt, bt) {
              var yt, St, Lt = bt.container, Rt = bt.fullLayout, vt = Rt._size, $t = bt.event, Pt = !!gt.hLinePoint, Xt = !!gt.vLinePoint;
              if (Lt.selectAll(".spikeline").remove(), Xt || Pt) {
                var Nt = he.combine(Rt.plot_bgcolor, Rt.paper_bgcolor);
                if (Pt) {
                  var Gt, Ft, Tt = gt.hLinePoint;
                  yt = Tt && Tt.xa, (St = Tt && Tt.ya).spikesnap === "cursor" ? (Gt = $t.pointerX, Ft = $t.pointerY) : (Gt = yt._offset + Tt.x, Ft = St._offset + Tt.y);
                  var It, Ct, Kt = ee.readability(Tt.color, Nt) < 1.5 ? he.contrast(Nt) : Tt.color, Vt = St.spikemode, Yt = St.spikethickness, sn = St.spikecolor || Kt, hn = pe.getPxPosition(st, St);
                  if (Vt.indexOf("toaxis") !== -1 || Vt.indexOf("across") !== -1) {
                    if (Vt.indexOf("toaxis") !== -1 && (It = hn, Ct = Gt), Vt.indexOf("across") !== -1) {
                      var Qt = St._counterDomainMin, Ut = St._counterDomainMax;
                      St.anchor === "free" && (Qt = Math.min(Qt, St.position), Ut = Math.max(Ut, St.position)), It = vt.l + Qt * vt.w, Ct = vt.l + Ut * vt.w;
                    }
                    Lt.insert("line", ":first-child").attr({ x1: It, x2: Ct, y1: Ft, y2: Ft, "stroke-width": Yt, stroke: sn, "stroke-dasharray": fe.dashStyle(St.spikedash, Yt) }).classed("spikeline", !0).classed("crisp", !0), Lt.insert("line", ":first-child").attr({ x1: It, x2: Ct, y1: Ft, y2: Ft, "stroke-width": Yt + 2, stroke: Nt }).classed("spikeline", !0).classed("crisp", !0);
                  }
                  Vt.indexOf("marker") !== -1 && Lt.insert("circle", ":first-child").attr({ cx: hn + (St.side !== "right" ? Yt : -Yt), cy: Ft, r: Yt, fill: sn }).classed("spikeline", !0);
                }
                if (Xt) {
                  var Wt, rn, vn = gt.vLinePoint;
                  yt = vn && vn.xa, St = vn && vn.ya, yt.spikesnap === "cursor" ? (Wt = $t.pointerX, rn = $t.pointerY) : (Wt = yt._offset + vn.x, rn = St._offset + vn.y);
                  var dn, Ht, jt = ee.readability(vn.color, Nt) < 1.5 ? he.contrast(Nt) : vn.color, cn = yt.spikemode, wn = yt.spikethickness, Rn = yt.spikecolor || jt, Pn = pe.getPxPosition(st, yt);
                  if (cn.indexOf("toaxis") !== -1 || cn.indexOf("across") !== -1) {
                    if (cn.indexOf("toaxis") !== -1 && (dn = Pn, Ht = rn), cn.indexOf("across") !== -1) {
                      var Ie = yt._counterDomainMin, rt = yt._counterDomainMax;
                      yt.anchor === "free" && (Ie = Math.min(Ie, yt.position), rt = Math.max(rt, yt.position)), dn = vt.t + (1 - rt) * vt.h, Ht = vt.t + (1 - Ie) * vt.h;
                    }
                    Lt.insert("line", ":first-child").attr({ x1: Wt, x2: Wt, y1: dn, y2: Ht, "stroke-width": wn, stroke: Rn, "stroke-dasharray": fe.dashStyle(yt.spikedash, wn) }).classed("spikeline", !0).classed("crisp", !0), Lt.insert("line", ":first-child").attr({ x1: Wt, x2: Wt, y1: dn, y2: Ht, "stroke-width": wn + 2, stroke: Nt }).classed("spikeline", !0).classed("crisp", !0);
                  }
                  cn.indexOf("marker") !== -1 && Lt.insert("circle", ":first-child").attr({ cx: Wt, cy: Pn - (yt.side !== "top" ? wn : -wn), r: wn, fill: Rn }).classed("spikeline", !0);
                }
              }
            }
            function at(st, gt) {
              return !0;
            }
            function ct(st, gt) {
              return ue.plainText(st || "", { len: gt, allowedTags: ["br", "sub", "sup", "b", "i", "em", "s", "u"] });
            }
            function et(st, gt, bt) {
              var yt = gt[st + "a"], St = gt[st + "Val"], Lt = gt.cd[0];
              if (yt.type === "category" || yt.type === "multicategory") St = yt._categoriesMap[St];
              else if (yt.type === "date") {
                var Rt = gt.trace[st + "periodalignment"];
                if (Rt) {
                  var vt = gt.cd[gt.index], $t = vt[st + "Start"];
                  $t === void 0 && ($t = vt[st]);
                  var Pt = vt[st + "End"];
                  Pt === void 0 && (Pt = vt[st]);
                  var Xt = Pt - $t;
                  Rt === "end" ? St += Xt : Rt === "middle" && (St += Xt / 2);
                }
                St = yt.d2c(St);
              }
              return Lt && Lt.t && Lt.t.posLetter === yt._id && (bt.boxmode !== "group" && bt.violinmode !== "group" || (St += Lt.t.dPos)), St;
            }
            function dt(st) {
              return st.offsetTop + st.clientTop;
            }
            function Je(st) {
              return st.offsetLeft + st.clientLeft;
            }
            function it(st, gt) {
              var bt = st._fullLayout, yt = gt.getBoundingClientRect(), St = yt.left, Lt = yt.top, Rt = St + yt.width, vt = Lt + yt.height, $t = ae.apply3DTransform(bt._invTransform)(St, Lt), Pt = ae.apply3DTransform(bt._invTransform)(Rt, vt), Xt = $t[0], Nt = $t[1], Gt = Pt[0], Ft = Pt[1];
              return { x: Xt, y: Nt, width: Gt - Xt, height: Ft - Nt, top: Math.min(Nt, Ft), left: Math.min(Xt, Gt), right: Math.max(Xt, Gt), bottom: Math.max(Nt, Ft) };
            }
          }, 26430: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(36040).isUnifiedHover;
            G.exports = function(ae, oe, se, ce) {
              ce = ce || {};
              var le = oe.legend;
              function ue(de) {
                ce.font[de] || (ce.font[de] = le ? oe.legend.font[de] : oe.font[de]);
              }
              oe && ee(oe.hovermode) && (ce.font || (ce.font = {}), ue("size"), ue("family"), ue("color"), ue("weight"), ue("style"), ue("variant"), le ? (ce.bgcolor || (ce.bgcolor = ne.combine(oe.legend.bgcolor, oe.paper_bgcolor)), ce.bordercolor || (ce.bordercolor = oe.legend.bordercolor)) : ce.bgcolor || (ce.bgcolor = oe.paper_bgcolor)), se("hoverlabel.bgcolor", ce.bgcolor), se("hoverlabel.bordercolor", ce.bordercolor), se("hoverlabel.namelength", ce.namelength), te.coerceFont(se, "hoverlabel.font", ce.font), se("hoverlabel.align", ce.align);
            };
          }, 45265: function(G, J, Z) {
            var te = Z(34809), ne = Z(6811);
            G.exports = function(ee, ae) {
              function oe(se, ce) {
                return ae[se] !== void 0 ? ae[se] : te.coerce(ee, ae, ne, se, ce);
              }
              return oe("clickmode"), oe("hoversubplots"), oe("hovermode");
            };
          }, 32141: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(14751), ae = Z(36040), oe = Z(6811), se = Z(38103);
            G.exports = { moduleType: "component", name: "fx", constants: Z(85988), schema: { layout: oe }, attributes: Z(70192), layoutAttributes: oe, supplyLayoutGlobalDefaults: Z(5358), supplyDefaults: Z(3239), supplyLayoutDefaults: Z(8412), calc: Z(83552), getDistanceFunction: ae.getDistanceFunction, getClosest: ae.getClosest, inbox: ae.inbox, quadrature: ae.quadrature, appendArrayPointValue: ae.appendArrayPointValue, castHoverOption: function(ce, le, ue) {
              return ne.castOption(ce, le, "hoverlabel." + ue);
            }, castHoverinfo: function(ce, le, ue) {
              return ne.castOption(ce, ue, "hoverinfo", function(de) {
                return ne.coerceHoverinfo({ hoverinfo: de }, { _module: ce._module }, le);
              });
            }, hover: se.hover, unhover: ee.unhover, loneHover: se.loneHover, loneUnhover: function(ce) {
              var le = ne.isD3Selection(ce) ? ce : te.select(ce);
              le.selectAll("g.hovertext").remove(), le.selectAll(".spikeline").remove();
            }, click: Z(94225) };
          }, 6811: function(G, J, Z) {
            var te = Z(85988), ne = Z(80337), ee = ne({ editType: "none" });
            ee.family.dflt = te.HOVERFONT, ee.size.dflt = te.HOVERFONTSIZE, G.exports = { clickmode: { valType: "flaglist", flags: ["event", "select"], dflt: "event", editType: "plot", extras: ["none"] }, dragmode: { valType: "enumerated", values: ["zoom", "pan", "select", "lasso", "drawclosedpath", "drawopenpath", "drawline", "drawrect", "drawcircle", "orbit", "turntable", !1], dflt: "zoom", editType: "modebar" }, hovermode: { valType: "enumerated", values: ["x", "y", "closest", !1, "x unified", "y unified"], dflt: "closest", editType: "modebar" }, hoversubplots: { valType: "enumerated", values: ["single", "overlaying", "axis"], dflt: "overlaying", editType: "none" }, hoverdistance: { valType: "integer", min: -1, dflt: 20, editType: "none" }, spikedistance: { valType: "integer", min: -1, dflt: -1, editType: "none" }, hoverlabel: { bgcolor: { valType: "color", editType: "none" }, bordercolor: { valType: "color", editType: "none" }, font: ee, grouptitlefont: ne({ editType: "none" }), align: { valType: "enumerated", values: ["left", "right", "auto"], dflt: "auto", editType: "none" }, namelength: { valType: "integer", min: -1, dflt: 15, editType: "none" }, editType: "none" }, selectdirection: { valType: "enumerated", values: ["h", "v", "d", "any"], dflt: "any", editType: "none" } };
          }, 8412: function(G, J, Z) {
            var te = Z(34809), ne = Z(6811), ee = Z(45265), ae = Z(26430);
            G.exports = function(oe, se) {
              function ce(he, me) {
                return te.coerce(oe, se, ne, he, me);
              }
              ee(oe, se) && (ce("hoverdistance"), ce("spikedistance")), ce("dragmode") === "select" && ce("selectdirection");
              var le = se._has("mapbox"), ue = se._has("map"), de = se._has("geo"), fe = se._basePlotModules.length;
              se.dragmode === "zoom" && ((le || ue || de) && fe === 1 || (le || ue) && de && fe === 2) && (se.dragmode = "pan"), ae(oe, se, ce), te.coerceFont(ce, "hoverlabel.grouptitlefont", se.hoverlabel.font);
            };
          }, 5358: function(G, J, Z) {
            var te = Z(34809), ne = Z(26430), ee = Z(6811);
            G.exports = function(ae, oe) {
              ne(ae, oe, function(se, ce) {
                return te.coerce(ae, oe, ee, se, ce);
              });
            };
          }, 83595: function(G, J, Z) {
            var te = Z(34809), ne = Z(90694).counter, ee = Z(13792).u, ae = Z(54826).idRegex, oe = Z(78032), se = { rows: { valType: "integer", min: 1, editType: "plot" }, roworder: { valType: "enumerated", values: ["top to bottom", "bottom to top"], dflt: "top to bottom", editType: "plot" }, columns: { valType: "integer", min: 1, editType: "plot" }, subplots: { valType: "info_array", freeLength: !0, dimensions: 2, items: { valType: "enumerated", values: [ne("xy").toString(), ""], editType: "plot" }, editType: "plot" }, xaxes: { valType: "info_array", freeLength: !0, items: { valType: "enumerated", values: [ae.x.toString(), ""], editType: "plot" }, editType: "plot" }, yaxes: { valType: "info_array", freeLength: !0, items: { valType: "enumerated", values: [ae.y.toString(), ""], editType: "plot" }, editType: "plot" }, pattern: { valType: "enumerated", values: ["independent", "coupled"], dflt: "coupled", editType: "plot" }, xgap: { valType: "number", min: 0, max: 1, editType: "plot" }, ygap: { valType: "number", min: 0, max: 1, editType: "plot" }, domain: ee({ name: "grid", editType: "plot", noGridCell: !0 }, {}), xside: { valType: "enumerated", values: ["bottom", "bottom plot", "top plot", "top"], dflt: "bottom plot", editType: "plot" }, yside: { valType: "enumerated", values: ["left", "left plot", "right plot", "right"], dflt: "left plot", editType: "plot" }, editType: "plot" };
            function ce(de, fe, he) {
              var me = fe[he + "axes"], pe = Object.keys((de._splomAxes || {})[he] || {});
              return Array.isArray(me) ? me : pe.length ? pe : void 0;
            }
            function le(de, fe, he, me, pe, ye) {
              var ge = fe(de + "gap", he), ve = fe("domain." + de);
              fe(de + "side", me);
              for (var be = new Array(pe), Ce = ve[0], Se = (ve[1] - Ce) / (pe - ge), we = Se * (1 - ge), Ee = 0; Ee < pe; Ee++) {
                var xe = Ce + Se * Ee;
                be[ye ? pe - 1 - Ee : Ee] = [xe, xe + we];
              }
              return be;
            }
            function ue(de, fe, he, me, pe) {
              var ye, ge = new Array(he);
              function ve(be, Ce) {
                fe.indexOf(Ce) !== -1 && me[Ce] === void 0 ? (ge[be] = Ce, me[Ce] = be) : ge[be] = "";
              }
              if (Array.isArray(de)) for (ye = 0; ye < he; ye++) ve(ye, de[ye]);
              else for (ve(0, pe), ye = 1; ye < he; ye++) ve(ye, pe + (ye + 1));
              return ge;
            }
            G.exports = { moduleType: "component", name: "grid", schema: { layout: { grid: se } }, layoutAttributes: se, sizeDefaults: function(de, fe) {
              var he = de.grid || {}, me = ce(fe, he, "x"), pe = ce(fe, he, "y");
              if (de.grid || me || pe) {
                var ye, ge, ve = Array.isArray(he.subplots) && Array.isArray(he.subplots[0]), be = Array.isArray(me), Ce = Array.isArray(pe), Se = be && me !== he.xaxes && Ce && pe !== he.yaxes;
                ve ? (ye = he.subplots.length, ge = he.subplots[0].length) : (Ce && (ye = pe.length), be && (ge = me.length));
                var we = oe.newContainer(fe, "grid"), Ee = De("rows", ye), xe = De("columns", ge);
                if (Ee * xe > 1) {
                  ve || be || Ce || De("pattern") === "independent" && (ve = !0), we._hasSubplotGrid = ve;
                  var Le, Qe, ke = De("roworder") === "top to bottom", Me = ve ? 0.2 : 0.1, Re = ve ? 0.3 : 0.1;
                  Se && fe._splomGridDflt && (Le = fe._splomGridDflt.xside, Qe = fe._splomGridDflt.yside), we._domains = { x: le("x", De, Me, Le, xe), y: le("y", De, Re, Qe, Ee, ke) };
                } else delete fe.grid;
              }
              function De(Pe, $e) {
                return te.coerce(he, we, se, Pe, $e);
              }
            }, contentDefaults: function(de, fe) {
              var he = fe.grid;
              if (he && he._domains) {
                var me, pe, ye, ge, ve, be, Ce, Se = de.grid || {}, we = fe._subplots, Ee = he._hasSubplotGrid, xe = he.rows, Le = he.columns, Qe = he.pattern === "independent", ke = he._axisMap = {};
                if (Ee) {
                  var Me = Se.subplots || [];
                  be = he.subplots = new Array(xe);
                  var Re = 1;
                  for (me = 0; me < xe; me++) {
                    var De = be[me] = new Array(Le), Pe = Me[me] || [];
                    for (pe = 0; pe < Le; pe++) if (Qe ? (ve = Re === 1 ? "xy" : "x" + Re + "y" + Re, Re++) : ve = Pe[pe], De[pe] = "", we.cartesian.indexOf(ve) !== -1) {
                      if (Ce = ve.indexOf("y"), ye = ve.slice(0, Ce), ge = ve.slice(Ce), ke[ye] !== void 0 && ke[ye] !== pe || ke[ge] !== void 0 && ke[ge] !== me) continue;
                      De[pe] = ve, ke[ye] = pe, ke[ge] = me;
                    }
                  }
                } else {
                  var $e = ce(fe, Se, "x"), qe = ce(fe, Se, "y");
                  he.xaxes = ue($e, we.xaxis, Le, ke, "x"), he.yaxes = ue(qe, we.yaxis, xe, ke, "y");
                }
                var Xe = he._anchors = {}, Ve = he.roworder === "top to bottom";
                for (var Ze in ke) {
                  var Ge, ze, We, at = Ze.charAt(0), ct = he[at + "side"];
                  if (ct.length < 8) Xe[Ze] = "free";
                  else if (at === "x") {
                    if (ct.charAt(0) === "t" === Ve ? (Ge = 0, ze = 1, We = xe) : (Ge = xe - 1, ze = -1, We = -1), Ee) {
                      var et = ke[Ze];
                      for (me = Ge; me !== We; me += ze) if ((ve = be[me][et]) && (Ce = ve.indexOf("y"), ve.slice(0, Ce) === Ze)) {
                        Xe[Ze] = ve.slice(Ce);
                        break;
                      }
                    } else for (me = Ge; me !== We; me += ze) if (ge = he.yaxes[me], we.cartesian.indexOf(Ze + ge) !== -1) {
                      Xe[Ze] = ge;
                      break;
                    }
                  } else if (ct.charAt(0) === "l" ? (Ge = 0, ze = 1, We = Le) : (Ge = Le - 1, ze = -1, We = -1), Ee) {
                    var dt = ke[Ze];
                    for (me = Ge; me !== We; me += ze) if ((ve = be[dt][me]) && (Ce = ve.indexOf("y"), ve.slice(Ce) === Ze)) {
                      Xe[Ze] = ve.slice(0, Ce);
                      break;
                    }
                  } else for (me = Ge; me !== We; me += ze) if (ye = he.xaxes[me], we.cartesian.indexOf(ye + Ze) !== -1) {
                    Xe[Ze] = ye;
                    break;
                  }
                }
              }
            } };
          }, 37260: function(G, J, Z) {
            var te = Z(54826), ne = Z(78032).templatedArray;
            Z(35081), G.exports = ne("image", { visible: { valType: "boolean", dflt: !0, editType: "arraydraw" }, source: { valType: "string", editType: "arraydraw" }, layer: { valType: "enumerated", values: ["below", "above"], dflt: "above", editType: "arraydraw" }, sizex: { valType: "number", dflt: 0, editType: "arraydraw" }, sizey: { valType: "number", dflt: 0, editType: "arraydraw" }, sizing: { valType: "enumerated", values: ["fill", "contain", "stretch"], dflt: "contain", editType: "arraydraw" }, opacity: { valType: "number", min: 0, max: 1, dflt: 1, editType: "arraydraw" }, x: { valType: "any", dflt: 0, editType: "arraydraw" }, y: { valType: "any", dflt: 0, editType: "arraydraw" }, xanchor: { valType: "enumerated", values: ["left", "center", "right"], dflt: "left", editType: "arraydraw" }, yanchor: { valType: "enumerated", values: ["top", "middle", "bottom"], dflt: "top", editType: "arraydraw" }, xref: { valType: "enumerated", values: ["paper", te.idRegex.x.toString()], dflt: "paper", editType: "arraydraw" }, yref: { valType: "enumerated", values: ["paper", te.idRegex.y.toString()], dflt: "paper", editType: "arraydraw" }, editType: "arraydraw" });
          }, 89443: function(G, J, Z) {
            var te = Z(10721), ne = Z(8083);
            G.exports = function(ee, ae, oe, se) {
              ae = ae || {};
              var ce = oe === "log" && ae.type === "linear", le = oe === "linear" && ae.type === "log";
              if (ce || le) {
                for (var ue, de, fe = ee._fullLayout.images, he = ae._id.charAt(0), me = 0; me < fe.length; me++) if (de = "images[" + me + "].", (ue = fe[me])[he + "ref"] === ae._id) {
                  var pe = ue[he], ye = ue["size" + he], ge = null, ve = null;
                  if (ce) {
                    ge = ne(pe, ae.range);
                    var be = ye / Math.pow(10, ge) / 2;
                    ve = 2 * Math.log(be + Math.sqrt(1 + be * be)) / Math.LN10;
                  } else ve = (ge = Math.pow(10, pe)) * (Math.pow(10, ye / 2) - Math.pow(10, -ye / 2));
                  te(ge) ? te(ve) || (ve = null) : (ge = null, ve = null), se(de + he, ge), se(de + "size" + he, ve);
                }
              }
            };
          }, 507: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(59008), ae = Z(37260);
            function oe(se, ce, le) {
              function ue(ge, ve) {
                return te.coerce(se, ce, ae, ge, ve);
              }
              var de = ue("source");
              if (!ue("visible", !!de)) return ce;
              ue("layer"), ue("xanchor"), ue("yanchor"), ue("sizex"), ue("sizey"), ue("sizing"), ue("opacity");
              for (var fe = { _fullLayout: le }, he = ["x", "y"], me = 0; me < 2; me++) {
                var pe = he[me], ye = ne.coerceRef(se, ce, fe, pe, "paper", void 0);
                ye !== "paper" && ne.getFromId(fe, ye)._imgIndices.push(ce._index), ne.coercePosition(ce, fe, ue, ye, pe, 0);
              }
              return ce;
            }
            G.exports = function(se, ce) {
              ee(se, ce, { name: "images", handleItemDefaults: oe });
            };
          }, 32211: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(29714), ae = Z(5975), oe = Z(62972);
            G.exports = function(se) {
              var ce, le, ue = se._fullLayout, de = [], fe = {}, he = [];
              for (le = 0; le < ue.images.length; le++) {
                var me = ue.images[le];
                if (me.visible) if (me.layer === "below" && me.xref !== "paper" && me.yref !== "paper") {
                  ce = ae.ref2id(me.xref) + ae.ref2id(me.yref);
                  var pe = ue._plots[ce];
                  if (!pe) {
                    he.push(me);
                    continue;
                  }
                  pe.mainplot && (ce = pe.mainplot.id), fe[ce] || (fe[ce] = []), fe[ce].push(me);
                } else me.layer === "above" ? de.push(me) : he.push(me);
              }
              var ye = { left: { sizing: "xMin", offset: 0 }, center: { sizing: "xMid", offset: -0.5 }, right: { sizing: "xMax", offset: -1 } }, ge = { top: { sizing: "YMin", offset: 0 }, middle: { sizing: "YMid", offset: -0.5 }, bottom: { sizing: "YMax", offset: -1 } };
              function ve(Le) {
                var Qe = te.select(this);
                if (this._imgSrc !== Le.source) if (Qe.attr("xmlns", oe.svg), Le.source && Le.source.slice(0, 5) === "data:") Qe.attr("xlink:href", Le.source), this._imgSrc = Le.source;
                else {
                  var ke = new Promise((function(Me) {
                    var Re = new Image();
                    function De() {
                      Qe.remove(), Me();
                    }
                    this.img = Re, Re.setAttribute("crossOrigin", "anonymous"), Re.onerror = De, Re.onload = function() {
                      var Pe = document.createElement("canvas");
                      Pe.width = this.width, Pe.height = this.height, Pe.getContext("2d", { willReadFrequently: !0 }).drawImage(this, 0, 0);
                      var $e = Pe.toDataURL("image/png");
                      Qe.attr("xlink:href", $e), Me();
                    }, Qe.on("error", De), Re.src = Le.source, this._imgSrc = Le.source;
                  }).bind(this));
                  se._promises.push(ke);
                }
              }
              function be(Le) {
                var Qe, ke, Me = te.select(this), Re = ee.getFromId(se, Le.xref), De = ee.getFromId(se, Le.yref), Pe = ee.getRefType(Le.xref) === "domain", $e = ee.getRefType(Le.yref) === "domain", qe = ue._size;
                Qe = Re !== void 0 ? typeof Le.xref == "string" && Pe ? Re._length * Le.sizex : Math.abs(Re.l2p(Le.sizex) - Re.l2p(0)) : Le.sizex * qe.w, ke = De !== void 0 ? typeof Le.yref == "string" && $e ? De._length * Le.sizey : Math.abs(De.l2p(Le.sizey) - De.l2p(0)) : Le.sizey * qe.h;
                var Xe, Ve, Ze = Qe * ye[Le.xanchor].offset, Ge = ke * ge[Le.yanchor].offset, ze = ye[Le.xanchor].sizing + ge[Le.yanchor].sizing;
                switch (Xe = Re !== void 0 ? typeof Le.xref == "string" && Pe ? Re._length * Le.x + Re._offset : Re.r2p(Le.x) + Re._offset : Le.x * qe.w + qe.l, Xe += Ze, Ve = De !== void 0 ? typeof Le.yref == "string" && $e ? De._length * (1 - Le.y) + De._offset : De.r2p(Le.y) + De._offset : qe.h - Le.y * qe.h + qe.t, Ve += Ge, Le.sizing) {
                  case "fill":
                    ze += " slice";
                    break;
                  case "stretch":
                    ze = "none";
                }
                Me.attr({ x: Xe, y: Ve, width: Qe, height: ke, preserveAspectRatio: ze, opacity: Le.opacity });
                var We = (Re && ee.getRefType(Le.xref) !== "domain" ? Re._id : "") + (De && ee.getRefType(Le.yref) !== "domain" ? De._id : "");
                ne.setClipUrl(Me, We ? "clip" + ue._uid + We : null, se);
              }
              var Ce = ue._imageLowerLayer.selectAll("image").data(he), Se = ue._imageUpperLayer.selectAll("image").data(de);
              Ce.enter().append("image"), Se.enter().append("image"), Ce.exit().remove(), Se.exit().remove(), Ce.each(function(Le) {
                ve.bind(this)(Le), be.bind(this)(Le);
              }), Se.each(function(Le) {
                ve.bind(this)(Le), be.bind(this)(Le);
              });
              var we = Object.keys(ue._plots);
              for (le = 0; le < we.length; le++) {
                ce = we[le];
                var Ee = ue._plots[ce];
                if (Ee.imagelayer) {
                  var xe = Ee.imagelayer.selectAll("image").data(fe[ce] || []);
                  xe.enter().append("image"), xe.exit().remove(), xe.each(function(Le) {
                    ve.bind(this)(Le), be.bind(this)(Le);
                  });
                }
              }
            };
          }, 15553: function(G, J, Z) {
            G.exports = { moduleType: "component", name: "images", layoutAttributes: Z(37260), supplyLayoutDefaults: Z(507), includeBasePlot: Z(20706)("images"), draw: Z(32211), convertCoords: Z(89443) };
          }, 86405: function(G, J, Z) {
            var te = Z(80337), ne = Z(10229);
            G.exports = { _isSubplotObj: !0, visible: { valType: "boolean", dflt: !0, editType: "legend" }, bgcolor: { valType: "color", editType: "legend" }, bordercolor: { valType: "color", dflt: ne.defaultLine, editType: "legend" }, borderwidth: { valType: "number", min: 0, dflt: 0, editType: "legend" }, font: te({ editType: "legend" }), grouptitlefont: te({ editType: "legend" }), orientation: { valType: "enumerated", values: ["v", "h"], dflt: "v", editType: "legend" }, traceorder: { valType: "flaglist", flags: ["reversed", "grouped"], extras: ["normal"], editType: "legend" }, tracegroupgap: { valType: "number", min: 0, dflt: 10, editType: "legend" }, entrywidth: { valType: "number", min: 0, editType: "legend" }, entrywidthmode: { valType: "enumerated", values: ["fraction", "pixels"], dflt: "pixels", editType: "legend" }, indentation: { valType: "number", min: -15, dflt: 0, editType: "legend" }, itemsizing: { valType: "enumerated", values: ["trace", "constant"], dflt: "trace", editType: "legend" }, itemwidth: { valType: "number", min: 30, dflt: 30, editType: "legend" }, itemclick: { valType: "enumerated", values: ["toggle", "toggleothers", !1], dflt: "toggle", editType: "legend" }, itemdoubleclick: { valType: "enumerated", values: ["toggle", "toggleothers", !1], dflt: "toggleothers", editType: "legend" }, groupclick: { valType: "enumerated", values: ["toggleitem", "togglegroup"], dflt: "togglegroup", editType: "legend" }, x: { valType: "number", editType: "legend" }, xref: { valType: "enumerated", dflt: "paper", values: ["container", "paper"], editType: "layoutstyle" }, xanchor: { valType: "enumerated", values: ["auto", "left", "center", "right"], dflt: "left", editType: "legend" }, y: { valType: "number", editType: "legend" }, yref: { valType: "enumerated", dflt: "paper", values: ["container", "paper"], editType: "layoutstyle" }, yanchor: { valType: "enumerated", values: ["auto", "top", "middle", "bottom"], editType: "legend" }, uirevision: { valType: "any", editType: "none" }, valign: { valType: "enumerated", values: ["top", "middle", "bottom"], dflt: "middle", editType: "legend" }, title: { text: { valType: "string", dflt: "", editType: "legend" }, font: te({ editType: "legend" }), side: { valType: "enumerated", values: ["top", "left", "top left", "top center", "top right"], editType: "legend" }, editType: "legend" }, editType: "legend" };
          }, 72783: function(G) {
            G.exports = { scrollBarWidth: 6, scrollBarMinHeight: 20, scrollBarColor: "#808BA4", scrollBarMargin: 4, scrollBarEnterAttrs: { rx: 20, ry: 3, width: 0, height: 0 }, titlePad: 2, itemGap: 5 };
          }, 73970: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(78032), ae = Z(9829), oe = Z(86405), se = Z(6704), ce = Z(57599);
            function le(ue, de, fe, he) {
              var me = de[ue] || {}, pe = ee.newContainer(fe, ue);
              function ye(ze, We) {
                return ne.coerce(me, pe, oe, ze, We);
              }
              var ge = ne.coerceFont(ye, "font", fe.font);
              if (ye("bgcolor", fe.paper_bgcolor), ye("bordercolor"), ye("visible")) {
                for (var ve, be = function(ze, We) {
                  var at = ve._input, ct = ve;
                  return ne.coerce(at, ct, ae, ze, We);
                }, Ce = fe.font || {}, Se = ne.coerceFont(ye, "grouptitlefont", Ce, { overrideDflt: { size: Math.round(1.1 * Ce.size) } }), we = 0, Ee = !1, xe = "normal", Le = (fe.shapes || []).filter(function(ze) {
                  return ze.showlegend;
                }), Qe = he.concat(Le).filter(function(ze) {
                  return ue === (ze.legend || "legend");
                }), ke = 0; ke < Qe.length; ke++) if ((ve = Qe[ke]).visible) {
                  var Me = ve._isShape;
                  (ve.showlegend || ve._dfltShowLegend && !(ve._module && ve._module.attributes && ve._module.attributes.showlegend && ve._module.attributes.showlegend.dflt === !1)) && (we++, ve.showlegend && (Ee = !0, (!Me && te.traceIs(ve, "pie-like") || ve._input.showlegend === !0) && we++), ne.coerceFont(be, "legendgrouptitle.font", Se)), (!Me && te.traceIs(ve, "bar") && fe.barmode === "stack" || ["tonextx", "tonexty"].indexOf(ve.fill) !== -1) && (xe = ce.isGrouped({ traceorder: xe }) ? "grouped+reversed" : "reversed"), ve.legendgroup !== void 0 && ve.legendgroup !== "" && (xe = ce.isReversed({ traceorder: xe }) ? "reversed+grouped" : "grouped");
                }
                var Re = ne.coerce(de, fe, se, "showlegend", Ee && we > (ue === "legend" ? 1 : 0));
                if (Re === !1 && (fe[ue] = void 0), (Re !== !1 || me.uirevision) && (ye("uirevision", fe.uirevision), Re !== !1)) {
                  ye("borderwidth");
                  var De, Pe, $e, qe = ye("orientation") === "h", Xe = ye("yref") === "paper", Ve = ye("xref") === "paper", Ze = "left";
                  if (qe ? (De = 0, te.getComponentMethod("rangeslider", "isVisible")(de.xaxis) ? Xe ? (Pe = 1.1, $e = "bottom") : (Pe = 1, $e = "top") : Xe ? (Pe = -0.1, $e = "top") : (Pe = 0, $e = "bottom")) : (Pe = 1, $e = "auto", Ve ? De = 1.02 : (De = 1, Ze = "right")), ne.coerce(me, pe, { x: { valType: "number", editType: "legend", min: Ve ? -2 : 0, max: Ve ? 3 : 1, dflt: De } }, "x"), ne.coerce(me, pe, { y: { valType: "number", editType: "legend", min: Xe ? -2 : 0, max: Xe ? 3 : 1, dflt: Pe } }, "y"), ye("traceorder", xe), ce.isGrouped(fe[ue]) && ye("tracegroupgap"), ye("entrywidth"), ye("entrywidthmode"), ye("indentation"), ye("itemsizing"), ye("itemwidth"), ye("itemclick"), ye("itemdoubleclick"), ye("groupclick"), ye("xanchor", Ze), ye("yanchor", $e), ye("valign"), ne.noneOrAll(me, pe, ["x", "y"]), ye("title.text")) {
                    ye("title.side", qe ? "left" : "top");
                    var Ge = ne.extendFlat({}, ge, { size: ne.bigFont(ge.size) });
                    ne.coerceFont(ye, "title.font", Ge);
                  }
                }
              }
            }
            G.exports = function(ue, de, fe) {
              var he, me = fe.slice(), pe = de.shapes;
              if (pe) for (he = 0; he < pe.length; he++) {
                var ye = pe[he];
                if (ye.showlegend) {
                  var ge = { _input: ye._input, visible: ye.visible, showlegend: ye.showlegend, legend: ye.legend };
                  me.push(ge);
                }
              }
              var ve = ["legend"];
              for (he = 0; he < me.length; he++) ne.pushUnique(ve, me[he].legend);
              for (de._legends = [], he = 0; he < ve.length; he++) {
                var be = ve[he];
                le(be, ue, de, me), de[be] && de[be].visible && (de[be]._id = be), de._legends.push(be);
              }
            };
          }, 6134: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(44122), ae = Z(33626), oe = Z(68596), se = Z(14751), ce = Z(62203), le = Z(78766), ue = Z(30635), de = Z(22165), fe = Z(72783), he = Z(4530), me = he.LINE_SPACING, pe = he.FROM_TL, ye = he.FROM_BR, ge = Z(851), ve = Z(14375), be = Z(57599), Ce = 1, Se = /^legend[0-9]*$/;
            function we($e, qe) {
              var Xe, Ve, Ze = qe || {}, Ge = $e._fullLayout, ze = Pe(Ze), We = Ze._inHover;
              if (We ? (Ve = Ze.layer, Xe = "hover") : (Ve = Ge._infolayer, Xe = ze), Ve) {
                var at;
                if (Xe += Ge._uid, $e._legendMouseDownTime || ($e._legendMouseDownTime = 0), We) {
                  if (!Ze.entries) return;
                  at = ge(Ze.entries, Ze);
                } else {
                  for (var ct = ($e.calcdata || []).slice(), et = Ge.shapes, dt = 0; dt < et.length; dt++) {
                    var Je = et[dt];
                    if (Je.showlegend) {
                      var it = { _isShape: !0, _fullInput: Je, index: Je._index, name: Je.name || Je.label.text || "shape " + Je._index, legend: Je.legend, legendgroup: Je.legendgroup, legendgrouptitle: Je.legendgrouptitle, legendrank: Je.legendrank, legendwidth: Je.legendwidth, showlegend: Je.showlegend, visible: Je.visible, opacity: Je.opacity, mode: Je.type === "line" ? "lines" : "markers", line: Je.line, marker: { line: Je.line, color: Je.fillcolor, size: 12, symbol: Je.type === "rect" ? "square" : Je.type === "circle" ? "circle" : "hexagon2" } };
                      ct.push([{ trace: it }]);
                    }
                  }
                  at = Ge.showlegend && ge(ct, Ze, Ge._legends.length > 1);
                }
                var st = Ge.hiddenlabels || [];
                if (!(We || Ge.showlegend && at.length)) return Ve.selectAll("." + ze).remove(), Ge._topdefs.select("#" + Xe).remove(), ee.autoMargin($e, ze);
                var gt = ne.ensureSingle(Ve, "g", ze, function(Xt) {
                  We || Xt.attr("pointer-events", "all");
                }), bt = ne.ensureSingleById(Ge._topdefs, "clipPath", Xe, function(Xt) {
                  Xt.append("rect");
                }), yt = ne.ensureSingle(gt, "rect", "bg", function(Xt) {
                  Xt.attr("shape-rendering", "crispEdges");
                });
                yt.call(le.stroke, Ze.bordercolor).call(le.fill, Ze.bgcolor).style("stroke-width", Ze.borderwidth + "px");
                var St, Lt = ne.ensureSingle(gt, "g", "scrollbox"), Rt = Ze.title;
                Ze._titleWidth = 0, Ze._titleHeight = 0, Rt.text ? ((St = ne.ensureSingle(Lt, "text", ze + "titletext")).attr("text-anchor", "start").call(ce.font, Rt.font).text(Rt.text), Me(St, Lt, $e, Ze, Ce)) : Lt.selectAll("." + ze + "titletext").remove();
                var vt = ne.ensureSingle(gt, "rect", "scrollbar", function(Xt) {
                  Xt.attr(fe.scrollBarEnterAttrs).call(le.fill, fe.scrollBarColor);
                }), $t = Lt.selectAll("g.groups").data(at);
                $t.enter().append("g").attr("class", "groups"), $t.exit().remove();
                var Pt = $t.selectAll("g.traces").data(ne.identity);
                Pt.enter().append("g").attr("class", "traces"), Pt.exit().remove(), Pt.style("opacity", function(Xt) {
                  var Nt = Xt[0].trace;
                  return ae.traceIs(Nt, "pie-like") ? st.indexOf(Xt[0].label) !== -1 ? 0.5 : 1 : Nt.visible === "legendonly" ? 0.5 : 1;
                }).each(function() {
                  te.select(this).call(Le, $e, Ze);
                }).call(ve, $e, Ze).each(function() {
                  We || te.select(this).call(ke, $e, ze);
                }), ne.syncOrAsync([ee.previousPromises, function() {
                  return function(Xt, Nt, Gt, Ft) {
                    var Tt = Xt._fullLayout, It = Pe(Ft);
                    Ft || (Ft = Tt[It]);
                    var Ct = Tt._size, Kt = be.isVertical(Ft), Vt = be.isGrouped(Ft), Yt = Ft.entrywidthmode === "fraction", sn = Ft.borderwidth, hn = 2 * sn, Qt = fe.itemGap, Ut = Ft.indentation + Ft.itemwidth + 2 * Qt, Wt = 2 * (sn + Qt), rn = De(Ft), vn = Ft.y < 0 || Ft.y === 0 && rn === "top", dn = Ft.y > 1 || Ft.y === 1 && rn === "bottom", Ht = Ft.tracegroupgap, jt = {};
                    Ft._maxHeight = Math.max(vn || dn ? Tt.height / 2 : Ct.h, 30);
                    var cn = 0;
                    Ft._width = 0, Ft._height = 0;
                    var wn = function(Ln) {
                      var In = 0, kn = 0, er = Ln.title.side;
                      return er && (er.indexOf("left") !== -1 && (In = Ln._titleWidth), er.indexOf("top") !== -1 && (kn = Ln._titleHeight)), [In, kn];
                    }(Ft);
                    if (Kt) Gt.each(function(Ln) {
                      var In = Ln[0].height;
                      ce.setTranslate(this, sn + wn[0], sn + wn[1] + Ft._height + In / 2 + Qt), Ft._height += In, Ft._width = Math.max(Ft._width, Ln[0].width);
                    }), cn = Ut + Ft._width, Ft._width += Qt + Ut + hn, Ft._height += Wt, Vt && (Nt.each(function(Ln, In) {
                      ce.setTranslate(this, 0, In * Ft.tracegroupgap);
                    }), Ft._height += (Ft._lgroupsLength - 1) * Ft.tracegroupgap);
                    else {
                      var Rn = Re(Ft), Pn = Ft.x < 0 || Ft.x === 0 && Rn === "right", Ie = Ft.x > 1 || Ft.x === 1 && Rn === "left", rt = dn || vn, ft = Tt.width / 2;
                      Ft._maxWidth = Math.max(Pn ? rt && Rn === "left" ? Ct.l + Ct.w : ft : Ie ? rt && Rn === "right" ? Ct.r + Ct.w : ft : Ct.w, 2 * Ut);
                      var pt = 0, wt = 0;
                      Gt.each(function(Ln) {
                        var In = Ee(Ln, Ft, Ut);
                        pt = Math.max(pt, In), wt += In;
                      }), cn = null;
                      var Bt = 0;
                      if (Vt) {
                        var At = 0, Dt = 0, Ot = 0;
                        Nt.each(function() {
                          var Ln = 0, In = 0;
                          te.select(this).selectAll("g.traces").each(function(er) {
                            var jn = Ee(er, Ft, Ut), ur = er[0].height;
                            ce.setTranslate(this, wn[0], wn[1] + sn + Qt + ur / 2 + In), In += ur, Ln = Math.max(Ln, jn), jt[er[0].trace.legendgroup] = Ln;
                          });
                          var kn = Ln + Qt;
                          Dt > 0 && kn + sn + Dt > Ft._maxWidth ? (Bt = Math.max(Bt, Dt), Dt = 0, Ot += At + Ht, At = In) : At = Math.max(At, In), ce.setTranslate(this, Dt, Ot), Dt += kn;
                        }), Ft._width = Math.max(Bt, Dt) + sn, Ft._height = Ot + At + Wt;
                      } else {
                        var un = Gt.size(), yn = wt + hn + (un - 1) * Qt < Ft._maxWidth, _n = 0, gn = 0, bn = 0, an = 0;
                        Gt.each(function(Ln) {
                          var In = Ln[0].height, kn = Ee(Ln, Ft, Ut), er = yn ? kn : pt;
                          Yt || (er += Qt), er + sn + gn - Qt >= Ft._maxWidth && (Bt = Math.max(Bt, an), gn = 0, bn += _n, Ft._height += _n, _n = 0), ce.setTranslate(this, wn[0] + sn + gn, wn[1] + sn + bn + In / 2 + Qt), an = gn + kn + Qt, gn += er, _n = Math.max(_n, In);
                        }), yn ? (Ft._width = gn + hn, Ft._height = _n + Wt) : (Ft._width = Math.max(Bt, an) + hn, Ft._height += _n + Wt);
                      }
                    }
                    Ft._width = Math.ceil(Math.max(Ft._width + wn[0], Ft._titleWidth + 2 * (sn + fe.titlePad))), Ft._height = Math.ceil(Math.max(Ft._height + wn[1], Ft._titleHeight + 2 * (sn + fe.itemGap))), Ft._effHeight = Math.min(Ft._height, Ft._maxHeight);
                    var Zt = Xt._context.edits, Mn = Zt.legendText || Zt.legendPosition;
                    Gt.each(function(Ln) {
                      var In = te.select(this).select("." + It + "toggle"), kn = Ln[0].height, er = Ln[0].trace.legendgroup, jn = Ee(Ln, Ft, Ut);
                      Vt && er !== "" && (jn = jt[er]);
                      var ur = Mn ? Ut : cn || jn;
                      Kt || Yt || (ur += Qt / 2), ce.setRect(In, 0, -kn / 2, ur, kn);
                    });
                  }($e, $t, Pt, Ze);
                }, function() {
                  var Xt, Nt, Gt, Ft, Tt = Ge._size, It = Ze.borderwidth, Ct = Ze.xref === "paper", Kt = Ze.yref === "paper";
                  if (Rt.text && function(rt, ft, pt) {
                    if (ft.title.side === "top center" || ft.title.side === "top right") {
                      var wt = ft.title.font.size * me, Bt = 0, At = rt.node(), Dt = ce.bBox(At).width;
                      ft.title.side === "top center" ? Bt = 0.5 * (ft._width - 2 * pt - 2 * fe.titlePad - Dt) : ft.title.side === "top right" && (Bt = ft._width - 2 * pt - 2 * fe.titlePad - Dt), ue.positionText(rt, pt + fe.titlePad + Bt, pt + wt);
                    }
                  }(St, Ze, It), !We) {
                    var Vt, Yt;
                    Vt = Ct ? Tt.l + Tt.w * Ze.x - pe[Re(Ze)] * Ze._width : Ge.width * Ze.x - pe[Re(Ze)] * Ze._width, Yt = Kt ? Tt.t + Tt.h * (1 - Ze.y) - pe[De(Ze)] * Ze._effHeight : Ge.height * (1 - Ze.y) - pe[De(Ze)] * Ze._effHeight;
                    var sn = function(rt, ft, pt, wt) {
                      var Bt = rt._fullLayout, At = Bt[ft], Dt = Re(At), Ot = De(At), un = At.xref === "paper", yn = At.yref === "paper";
                      rt._fullLayout._reservedMargin[ft] = {};
                      var _n = At.y < 0.5 ? "b" : "t", gn = At.x < 0.5 ? "l" : "r", bn = { r: Bt.width - pt, l: pt + At._width, b: Bt.height - wt, t: wt + At._effHeight };
                      if (un && yn) return ee.autoMargin(rt, ft, { x: At.x, y: At.y, l: At._width * pe[Dt], r: At._width * ye[Dt], b: At._effHeight * ye[Ot], t: At._effHeight * pe[Ot] });
                      un ? rt._fullLayout._reservedMargin[ft][_n] = bn[_n] : yn || At.orientation === "v" ? rt._fullLayout._reservedMargin[ft][gn] = bn[gn] : rt._fullLayout._reservedMargin[ft][_n] = bn[_n];
                    }($e, ze, Vt, Yt);
                    if (sn) return;
                    if (Ge.margin.autoexpand) {
                      var hn = Vt, Qt = Yt;
                      Vt = Ct ? ne.constrain(Vt, 0, Ge.width - Ze._width) : hn, Yt = Kt ? ne.constrain(Yt, 0, Ge.height - Ze._effHeight) : Qt, Vt !== hn && ne.log("Constrain " + ze + ".x to make legend fit inside graph"), Yt !== Qt && ne.log("Constrain " + ze + ".y to make legend fit inside graph");
                    }
                    ce.setTranslate(gt, Vt, Yt);
                  }
                  if (vt.on(".drag", null), gt.on("wheel", null), We || Ze._height <= Ze._maxHeight || $e._context.staticPlot) {
                    var Ut = Ze._effHeight;
                    We && (Ut = Ze._height), yt.attr({ width: Ze._width - It, height: Ut - It, x: It / 2, y: It / 2 }), ce.setTranslate(Lt, 0, 0), bt.select("rect").attr({ width: Ze._width - 2 * It, height: Ut - 2 * It, x: It, y: It }), ce.setClipUrl(Lt, Xe, $e), ce.setRect(vt, 0, 0, 0, 0), delete Ze._scrollY;
                  } else {
                    var Wt, rn, vn, dn = Math.max(fe.scrollBarMinHeight, Ze._effHeight * Ze._effHeight / Ze._height), Ht = Ze._effHeight - dn - 2 * fe.scrollBarMargin, jt = Ze._height - Ze._effHeight, cn = Ht / jt, wn = Math.min(Ze._scrollY || 0, jt);
                    yt.attr({ width: Ze._width - 2 * It + fe.scrollBarWidth + fe.scrollBarMargin, height: Ze._effHeight - It, x: It / 2, y: It / 2 }), bt.select("rect").attr({ width: Ze._width - 2 * It + fe.scrollBarWidth + fe.scrollBarMargin, height: Ze._effHeight - 2 * It, x: It, y: It + wn }), ce.setClipUrl(Lt, Xe, $e), Ie(wn, dn, cn), gt.on("wheel", function() {
                      Ie(wn = ne.constrain(Ze._scrollY + te.event.deltaY / Ht * jt, 0, jt), dn, cn), wn !== 0 && wn !== jt && te.event.preventDefault();
                    });
                    var Rn = te.behavior.drag().on("dragstart", function() {
                      var rt = te.event.sourceEvent;
                      Wt = rt.type === "touchstart" ? rt.changedTouches[0].clientY : rt.clientY, vn = wn;
                    }).on("drag", function() {
                      var rt = te.event.sourceEvent;
                      rt.buttons === 2 || rt.ctrlKey || (rn = rt.type === "touchmove" ? rt.changedTouches[0].clientY : rt.clientY, wn = function(ft, pt, wt) {
                        var Bt = (wt - pt) / cn + ft;
                        return ne.constrain(Bt, 0, jt);
                      }(vn, Wt, rn), Ie(wn, dn, cn));
                    });
                    vt.call(Rn);
                    var Pn = te.behavior.drag().on("dragstart", function() {
                      var rt = te.event.sourceEvent;
                      rt.type === "touchstart" && (Wt = rt.changedTouches[0].clientY, vn = wn);
                    }).on("drag", function() {
                      var rt = te.event.sourceEvent;
                      rt.type === "touchmove" && (rn = rt.changedTouches[0].clientY, wn = function(ft, pt, wt) {
                        var Bt = (pt - wt) / cn + ft;
                        return ne.constrain(Bt, 0, jt);
                      }(vn, Wt, rn), Ie(wn, dn, cn));
                    });
                    Lt.call(Pn);
                  }
                  function Ie(rt, ft, pt) {
                    Ze._scrollY = $e._fullLayout[ze]._scrollY = rt, ce.setTranslate(Lt, 0, -rt), ce.setRect(vt, Ze._width, fe.scrollBarMargin + rt * pt, fe.scrollBarWidth, ft), bt.select("rect").attr("y", It + rt);
                  }
                  $e._context.edits.legendPosition && (gt.classed("cursor-move", !0), se.init({ element: gt.node(), gd: $e, prepFn: function(rt) {
                    if (rt.target !== vt.node()) {
                      var ft = ce.getTranslate(gt);
                      Gt = ft.x, Ft = ft.y;
                    }
                  }, moveFn: function(rt, ft) {
                    if (Gt !== void 0 && Ft !== void 0) {
                      var pt = Gt + rt, wt = Ft + ft;
                      ce.setTranslate(gt, pt, wt), Xt = se.align(pt, Ze._width, Tt.l, Tt.l + Tt.w, Ze.xanchor), Nt = se.align(wt + Ze._height, -Ze._height, Tt.t + Tt.h, Tt.t, Ze.yanchor);
                    }
                  }, doneFn: function() {
                    if (Xt !== void 0 && Nt !== void 0) {
                      var rt = {};
                      rt[ze + ".x"] = Xt, rt[ze + ".y"] = Nt, ae.call("_guiRelayout", $e, rt);
                    }
                  }, clickFn: function(rt, ft) {
                    var pt = Ve.selectAll("g.traces").filter(function() {
                      var wt = this.getBoundingClientRect();
                      return ft.clientX >= wt.left && ft.clientX <= wt.right && ft.clientY >= wt.top && ft.clientY <= wt.bottom;
                    });
                    pt.size() > 0 && xe($e, gt, pt, rt, ft);
                  } }));
                }], $e);
              }
            }
            function Ee($e, qe, Xe) {
              var Ve = $e[0], Ze = Ve.width, Ge = qe.entrywidthmode, ze = Ve.trace.legendwidth || qe.entrywidth;
              return Ge === "fraction" ? qe._maxWidth * ze : Xe + (ze || Ze);
            }
            function xe($e, qe, Xe, Ve, Ze) {
              var Ge = Xe.data()[0][0].trace, ze = { event: Ze, node: Xe.node(), curveNumber: Ge.index, expandedIndex: Ge._expandedIndex, data: $e.data, layout: $e.layout, frames: $e._transitionData._frames, config: $e._context, fullData: $e._fullData, fullLayout: $e._fullLayout };
              Ge._group && (ze.group = Ge._group), ae.traceIs(Ge, "pie-like") && (ze.label = Xe.datum()[0].label);
              var We = oe.triggerHandler($e, "plotly_legendclick", ze);
              if (Ve === 1) {
                if (We === !1) return;
                qe._clickTimeout = setTimeout(function() {
                  $e._fullLayout && de(Xe, $e, Ve);
                }, $e._context.doubleClickDelay);
              } else Ve === 2 && (qe._clickTimeout && clearTimeout(qe._clickTimeout), $e._legendMouseDownTime = 0, oe.triggerHandler($e, "plotly_legenddoubleclick", ze) !== !1 && We !== !1 && de(Xe, $e, Ve));
            }
            function Le($e, qe, Xe) {
              var Ve, Ze, Ge = Pe(Xe), ze = $e.data()[0][0], We = ze.trace, at = ae.traceIs(We, "pie-like"), ct = !Xe._inHover && qe._context.edits.legendText && !at, et = Xe._maxNameLength;
              ze.groupTitle ? (Ve = ze.groupTitle.text, Ze = ze.groupTitle.font) : (Ze = Xe.font, Xe.entries ? Ve = ze.text : (Ve = at ? ze.label : We.name, We._meta && (Ve = ne.templateString(Ve, We._meta))));
              var dt = ne.ensureSingle($e, "text", Ge + "text");
              dt.attr("text-anchor", "start").call(ce.font, Ze).text(ct ? Qe(Ve, et) : Ve);
              var Je = Xe.indentation + Xe.itemwidth + 2 * fe.itemGap;
              ue.positionText(dt, Je, 0), ct ? dt.call(ue.makeEditable, { gd: qe, text: Ve }).call(Me, $e, qe, Xe).on("edit", function(it) {
                this.text(Qe(it, et)).call(Me, $e, qe, Xe);
                var st = ze.trace._fullInput || {}, gt = {};
                if (ae.hasTransform(st, "groupby")) {
                  var bt = ae.getTransformIndices(st, "groupby"), yt = bt[bt.length - 1], St = ne.keyedContainer(st, "transforms[" + yt + "].styles", "target", "value.name");
                  St.set(ze.trace._group, it), gt = St.constructUpdate();
                } else gt.name = it;
                return st._isShape ? ae.call("_guiRelayout", qe, "shapes[" + We.index + "].name", gt.name) : ae.call("_guiRestyle", qe, gt, We.index);
              }) : Me(dt, $e, qe, Xe);
            }
            function Qe($e, qe) {
              var Xe = Math.max(4, qe);
              if ($e && $e.trim().length >= Xe / 2) return $e;
              for (var Ve = Xe - ($e = $e || "").length; Ve > 0; Ve--) $e += " ";
              return $e;
            }
            function ke($e, qe, Xe) {
              var Ve, Ze = qe._context.doubleClickDelay, Ge = 1, ze = ne.ensureSingle($e, "rect", Xe + "toggle", function(We) {
                qe._context.staticPlot || We.style("cursor", "pointer").attr("pointer-events", "all"), We.call(le.fill, "rgba(0,0,0,0)");
              });
              qe._context.staticPlot || (ze.on("mousedown", function() {
                (Ve = (/* @__PURE__ */ new Date()).getTime()) - qe._legendMouseDownTime < Ze ? Ge += 1 : (Ge = 1, qe._legendMouseDownTime = Ve);
              }), ze.on("mouseup", function() {
                if (!qe._dragged && !qe._editing) {
                  var We = qe._fullLayout[Xe];
                  (/* @__PURE__ */ new Date()).getTime() - qe._legendMouseDownTime > Ze && (Ge = Math.max(Ge - 1, 1)), xe(qe, We, $e, Ge, te.event);
                }
              }));
            }
            function Me($e, qe, Xe, Ve, Ze) {
              Ve._inHover && $e.attr("data-notex", !0), ue.convertToTspans($e, Xe, function() {
                (function(Ge, ze, We, at) {
                  var ct = Ge.data()[0][0];
                  if (We._inHover || !ct || ct.trace.showlegend) {
                    var et = Ge.select("g[class*=math-group]"), dt = et.node(), Je = Pe(We);
                    We || (We = ze._fullLayout[Je]);
                    var it, st, gt = We.borderwidth, bt = (at === Ce ? We.title.font : ct.groupTitle ? ct.groupTitle.font : We.font).size * me;
                    if (dt) {
                      var yt = ce.bBox(dt);
                      it = yt.height, st = yt.width, at === Ce ? ce.setTranslate(et, gt, gt + 0.75 * it) : ce.setTranslate(et, 0, 0.25 * it);
                    } else {
                      var St = "." + Je + (at === Ce ? "title" : "") + "text", Lt = Ge.select(St), Rt = ue.lineCount(Lt), vt = Lt.node();
                      if (it = bt * Rt, st = vt ? ce.bBox(vt).width : 0, at === Ce) We.title.side === "left" && (st += 2 * fe.itemGap), ue.positionText(Lt, gt + fe.titlePad, gt + bt);
                      else {
                        var $t = 2 * fe.itemGap + We.indentation + We.itemwidth;
                        ct.groupTitle && ($t = fe.itemGap, st -= We.indentation + We.itemwidth), ue.positionText(Lt, $t, -bt * ((Rt - 1) / 2 - 0.3));
                      }
                    }
                    at === Ce ? (We._titleWidth = st, We._titleHeight = it) : (ct.lineHeight = bt, ct.height = Math.max(it, 16) + 3, ct.width = st);
                  } else Ge.remove();
                })(qe, Xe, Ve, Ze);
              });
            }
            function Re($e) {
              return ne.isRightAnchor($e) ? "right" : ne.isCenterAnchor($e) ? "center" : "left";
            }
            function De($e) {
              return ne.isBottomAnchor($e) ? "bottom" : ne.isMiddleAnchor($e) ? "middle" : "top";
            }
            function Pe($e) {
              return $e._id || "legend";
            }
            G.exports = function($e, qe) {
              if (qe) we($e, qe);
              else {
                var Xe = $e._fullLayout, Ve = Xe._legends;
                Xe._infolayer.selectAll('[class^="legend"]').each(function() {
                  var ze = te.select(this), We = ze.attr("class").split(" ")[0];
                  We.match(Se) && Ve.indexOf(We) === -1 && ze.remove();
                });
                for (var Ze = 0; Ze < Ve.length; Ze++) {
                  var Ge = Ve[Ze];
                  we($e, $e._fullLayout[Ge]);
                }
              }
            };
          }, 851: function(G, J, Z) {
            var te = Z(33626), ne = Z(57599);
            G.exports = function(ee, ae, oe) {
              var se, ce, le = ae._inHover, ue = ne.isGrouped(ae), de = ne.isReversed(ae), fe = {}, he = [], me = !1, pe = {}, ye = 0, ge = 0;
              function ve(Ve, Ze, Ge) {
                if (ae.visible !== !1 && (!oe || Ve === ae._id)) if (Ze !== "" && ne.isGrouped(ae)) he.indexOf(Ze) === -1 ? (he.push(Ze), me = !0, fe[Ze] = [Ge]) : fe[Ze].push(Ge);
                else {
                  var ze = "~~i" + ye;
                  he.push(ze), fe[ze] = [Ge], ye++;
                }
              }
              for (se = 0; se < ee.length; se++) {
                var be = ee[se], Ce = be[0], Se = Ce.trace, we = Se.legend, Ee = Se.legendgroup;
                if (le || Se.visible && Se.showlegend) if (te.traceIs(Se, "pie-like")) for (pe[Ee] || (pe[Ee] = {}), ce = 0; ce < be.length; ce++) {
                  var xe = be[ce].label;
                  pe[Ee][xe] || (ve(we, Ee, { label: xe, color: be[ce].color, i: be[ce].i, trace: Se, pts: be[ce].pts }), pe[Ee][xe] = !0, ge = Math.max(ge, (xe || "").length));
                }
                else ve(we, Ee, Ce), ge = Math.max(ge, (Se.name || "").length);
              }
              if (!he.length) return [];
              var Le = !me || !ue, Qe = [];
              for (se = 0; se < he.length; se++) {
                var ke = fe[he[se]];
                Le ? Qe.push(ke[0]) : Qe.push(ke);
              }
              for (Le && (Qe = [Qe]), se = 0; se < Qe.length; se++) {
                var Me = 1 / 0;
                for (ce = 0; ce < Qe[se].length; ce++) {
                  var Re = Qe[se][ce].trace.legendrank;
                  Me > Re && (Me = Re);
                }
                Qe[se][0]._groupMinRank = Me, Qe[se][0]._preGroupSort = se;
              }
              var De = function(Ve, Ze) {
                return Ve.trace.legendrank - Ze.trace.legendrank || Ve._preSort - Ze._preSort;
              };
              for (Qe.forEach(function(Ve, Ze) {
                Ve[0]._preGroupSort = Ze;
              }), Qe.sort(function(Ve, Ze) {
                return Ve[0]._groupMinRank - Ze[0]._groupMinRank || Ve[0]._preGroupSort - Ze[0]._preGroupSort;
              }), se = 0; se < Qe.length; se++) {
                Qe[se].forEach(function(Ve, Ze) {
                  Ve._preSort = Ze;
                }), Qe[se].sort(De);
                var Pe = Qe[se][0].trace, $e = null;
                for (ce = 0; ce < Qe[se].length; ce++) {
                  var qe = Qe[se][ce].trace.legendgrouptitle;
                  if (qe && qe.text) {
                    $e = qe, le && (qe.font = ae._groupTitleFont);
                    break;
                  }
                }
                if (de && Qe[se].reverse(), $e) {
                  var Xe = !1;
                  for (ce = 0; ce < Qe[se].length; ce++) if (te.traceIs(Qe[se][ce].trace, "pie-like")) {
                    Xe = !0;
                    break;
                  }
                  Qe[se].unshift({ i: -1, groupTitle: $e, noClick: Xe, trace: { showlegend: Pe.showlegend, legendgroup: Pe.legendgroup, visible: ae.groupclick === "toggleitem" || Pe.visible } });
                }
                for (ce = 0; ce < Qe[se].length; ce++) Qe[se][ce] = [Qe[se][ce]];
              }
              return ae._lgroupsLength = Qe.length, ae._maxNameLength = ge, Qe;
            };
          }, 22165: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = ne.pushUnique, ae = !0;
            G.exports = function(oe, se, ce) {
              var le = se._fullLayout;
              if (!se._dragged && !se._editing) {
                var ue, de = le.legend.itemclick, fe = le.legend.itemdoubleclick, he = le.legend.groupclick;
                if (ce === 1 && de === "toggle" && fe === "toggleothers" && ae && se.data && se._context.showTips && ne.notifier(ne._(se, "Double-click on legend to isolate one trace"), "long"), ae = !1, ce === 1 ? ue = de : ce === 2 && (ue = fe), ue) {
                  var me = he === "togglegroup", pe = le.hiddenlabels ? le.hiddenlabels.slice() : [], ye = oe.data()[0][0];
                  if (!ye.groupTitle || !ye.noClick) {
                    var ge = se._fullData, ve = (le.shapes || []).filter(function(Nt) {
                      return Nt.showlegend;
                    }), be = ge.concat(ve), Ce = ye.trace;
                    Ce._isShape && (Ce = Ce._fullInput);
                    var Se, we, Ee, xe, Le, Qe = Ce.legendgroup, ke = {}, Me = [], Re = [], De = [], Pe = (le.shapes || []).map(function(Nt) {
                      return Nt._input;
                    }), $e = !1, qe = Ce.legend, Xe = Ce._fullInput;
                    if (Xe && Xe._isShape || !te.traceIs(Ce, "pie-like")) {
                      var Ve, Ze = Qe && Qe.length, Ge = [];
                      if (Ze) for (Se = 0; Se < be.length; Se++) (Ve = be[Se]).visible && Ve.legendgroup === Qe && Ge.push(Se);
                      if (ue === "toggle") {
                        var ze;
                        switch (Ce.visible) {
                          case !0:
                            ze = "legendonly";
                            break;
                          case !1:
                            ze = !1;
                            break;
                          case "legendonly":
                            ze = !0;
                        }
                        if (Ze) if (me) for (Se = 0; Se < be.length; Se++) {
                          var We = be[Se];
                          We.visible !== !1 && We.legendgroup === Qe && Xt(We, ze);
                        }
                        else Xt(Ce, ze);
                        else Xt(Ce, ze);
                      } else if (ue === "toggleothers") {
                        var at, ct, et, dt, Je = !0;
                        for (Se = 0; Se < be.length; Se++) if (at = (dt = be[Se]) === Ce, ct = dt.showlegend !== !0, !(at || ct || Ze && dt.legendgroup === Qe || dt.legend !== qe || dt.visible !== !0 || te.traceIs(dt, "notLegendIsolatable"))) {
                          Je = !1;
                          break;
                        }
                        for (Se = 0; Se < be.length; Se++) if ((dt = be[Se]).visible !== !1 && dt.legend === qe && !te.traceIs(dt, "notLegendIsolatable")) switch (Ce.visible) {
                          case "legendonly":
                            Xt(dt, !0);
                            break;
                          case !0:
                            et = !!Je || "legendonly", at = dt === Ce, ct = dt.showlegend !== !0 && !dt.legendgroup, Xt(dt, !!(at || Ze && dt.legendgroup === Qe || ct) || et);
                        }
                      }
                      for (Se = 0; Se < Re.length; Se++) if (Ee = Re[Se]) {
                        var it = Ee.constructUpdate(), st = Object.keys(it);
                        for (we = 0; we < st.length; we++) xe = st[we], (ke[xe] = ke[xe] || [])[De[Se]] = it[xe];
                      }
                      for (Le = Object.keys(ke), Se = 0; Se < Le.length; Se++) for (xe = Le[Se], we = 0; we < Me.length; we++) ke[xe].hasOwnProperty(we) || (ke[xe][we] = void 0);
                      $e ? te.call("_guiUpdate", se, ke, { shapes: Pe }, Me) : te.call("_guiRestyle", se, ke, Me);
                    } else {
                      var gt = ye.label, bt = pe.indexOf(gt);
                      if (ue === "toggle") bt === -1 ? pe.push(gt) : pe.splice(bt, 1);
                      else if (ue === "toggleothers") {
                        var yt = bt !== -1, St = [];
                        for (Se = 0; Se < se.calcdata.length; Se++) {
                          var Lt = se.calcdata[Se];
                          for (we = 0; we < Lt.length; we++) {
                            var Rt = Lt[we].label;
                            qe === Lt[0].trace.legend && gt !== Rt && (pe.indexOf(Rt) === -1 && (yt = !0), ee(pe, Rt), St.push(Rt));
                          }
                        }
                        if (!yt) for (var vt = 0; vt < St.length; vt++) {
                          var $t = pe.indexOf(St[vt]);
                          $t !== -1 && pe.splice($t, 1);
                        }
                      }
                      te.call("_guiRelayout", se, "hiddenlabels", pe);
                    }
                  }
                }
              }
              function Pt(Nt, Gt) {
                var Ft = Me.indexOf(Nt), Tt = ke.visible;
                return Tt || (Tt = ke.visible = []), Me.indexOf(Nt) === -1 && (Me.push(Nt), Ft = Me.length - 1), Tt[Ft] = Gt, Ft;
              }
              function Xt(Nt, Gt) {
                if (!ye.groupTitle || me) {
                  var Ft, Tt = Nt._fullInput || Nt, It = Tt._isShape, Ct = Tt.index;
                  if (Ct === void 0 && (Ct = Tt._index), te.hasTransform(Tt, "groupby")) {
                    var Kt = Re[Ct];
                    if (!Kt) {
                      var Vt = te.getTransformIndices(Tt, "groupby"), Yt = Vt[Vt.length - 1];
                      Kt = ne.keyedContainer(Tt, "transforms[" + Yt + "].styles", "target", "value.visible"), Re[Ct] = Kt;
                    }
                    var sn = Kt.get(Nt._group);
                    sn === void 0 && (sn = !0), sn !== !1 && Kt.set(Nt._group, Gt), De[Ct] = Pt(Ct, Tt.visible !== !1);
                  } else {
                    var hn = Tt.visible !== !1 && Gt;
                    It ? (Ft = hn, Pe[Ct].visible = Ft, $e = !0) : Pt(Ct, hn);
                  }
                }
              }
            };
          }, 57599: function(G, J) {
            J.isGrouped = function(Z) {
              return (Z.traceorder || "").indexOf("grouped") !== -1;
            }, J.isVertical = function(Z) {
              return Z.orientation !== "h";
            }, J.isReversed = function(Z) {
              return (Z.traceorder || "").indexOf("reversed") !== -1;
            };
          }, 82494: function(G, J, Z) {
            G.exports = { moduleType: "component", name: "legend", layoutAttributes: Z(86405), supplyLayoutDefaults: Z(73970), draw: Z(6134), style: Z(14375) };
          }, 14375: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(34809), ae = ee.strTranslate, oe = Z(62203), se = Z(78766), ce = Z(65477).extractOpts, le = Z(64726), ue = Z(32891), de = Z(37252).castOption, fe = Z(72783);
            function he(ye, ge) {
              return (ge ? "radial" : "horizontal") + (ye ? "" : "reversed");
            }
            function me(ye) {
              var ge = ye[0].trace, ve = ge.contours, be = le.hasLines(ge), Ce = le.hasMarkers(ge), Se = ge.visible && ge.fill && ge.fill !== "none", we = !1, Ee = !1;
              if (ve) {
                var xe = ve.coloring;
                xe === "lines" ? we = !0 : be = xe === "none" || xe === "heatmap" || ve.showlines, ve.type === "constraint" ? Se = ve._operation !== "=" : xe !== "fill" && xe !== "heatmap" || (Ee = !0);
              }
              return { showMarker: Ce, showLine: be, showFill: Se, showGradientLine: we, showGradientFill: Ee, anyLine: be || we, anyFill: Se || Ee };
            }
            function pe(ye, ge, ve) {
              return ye && ee.isArrayOrTypedArray(ye) ? ge : ye > ve ? ve : ye;
            }
            G.exports = function(ye, ge, ve) {
              var be = ge._fullLayout;
              ve || (ve = be.legend);
              var Ce = ve.itemsizing === "constant", Se = ve.itemwidth, we = (Se + 2 * fe.itemGap) / 2, Ee = ae(we, 0), xe = function(ke, Me, Re, De) {
                var Pe;
                if (ke + 1) Pe = ke;
                else {
                  if (!(Me && Me.width > 0)) return 0;
                  Pe = Me.width;
                }
                return Ce ? De : Math.min(Pe, Re);
              };
              function Le(ke, Me, Re) {
                var De = ke[0].trace, Pe = De.marker || {}, $e = Pe.line || {}, qe = Pe.cornerradius ? "M6,3a3,3,0,0,1-3,3H-3a3,3,0,0,1-3-3V-3a3,3,0,0,1,3-3H3a3,3,0,0,1,3,3Z" : "M6,6H-6V-6H6Z", Xe = Re ? De.visible && De.type === Re : ne.traceIs(De, "bar"), Ve = te.select(Me).select("g.legendpoints").selectAll("path.legend" + Re).data(Xe ? [ke] : []);
                Ve.enter().append("path").classed("legend" + Re, !0).attr("d", qe).attr("transform", Ee), Ve.exit().remove(), Ve.each(function(Ze) {
                  var Ge = te.select(this), ze = Ze[0], We = xe(ze.mlw, Pe.line, 5, 2);
                  Ge.style("stroke-width", We + "px");
                  var at = ze.mcc;
                  if (!ve._inHover && "mc" in ze) {
                    var ct = ce(Pe), et = ct.mid;
                    et === void 0 && (et = (ct.max + ct.min) / 2), at = oe.tryColorscale(Pe, "")(et);
                  }
                  var dt = at || ze.mc || Pe.color, Je = Pe.pattern, it = Je && oe.getPatternAttr(Je.shape, 0, "");
                  if (it) {
                    var st = oe.getPatternAttr(Je.bgcolor, 0, null), gt = oe.getPatternAttr(Je.fgcolor, 0, null), bt = Je.fgopacity, yt = pe(Je.size, 8, 10), St = pe(Je.solidity, 0.5, 1), Lt = "legend-" + De.uid;
                    Ge.call(oe.pattern, "legend", ge, Lt, it, yt, St, at, Je.fillmode, st, gt, bt);
                  } else Ge.call(se.fill, dt);
                  We && se.stroke(Ge, ze.mlc || $e.color);
                });
              }
              function Qe(ke, Me, Re) {
                var De = ke[0], Pe = De.trace, $e = Re ? Pe.visible && Pe.type === Re : ne.traceIs(Pe, Re), qe = te.select(Me).select("g.legendpoints").selectAll("path.legend" + Re).data($e ? [ke] : []);
                if (qe.enter().append("path").classed("legend" + Re, !0).attr("d", "M6,6H-6V-6H6Z").attr("transform", Ee), qe.exit().remove(), qe.size()) {
                  var Xe = Pe.marker || {}, Ve = xe(de(Xe.line.width, De.pts), Xe.line, 5, 2), Ze = "pieLike", Ge = ee.minExtend(Pe, { marker: { line: { width: Ve } } }, Ze), ze = ee.minExtend(De, { trace: Ge }, Ze);
                  ue(qe, ze, Ge, ge);
                }
              }
              ye.each(function(ke) {
                var Me = te.select(this), Re = ee.ensureSingle(Me, "g", "layers");
                Re.style("opacity", ke[0].trace.opacity);
                var De = ve.indentation, Pe = ve.valign, $e = ke[0].lineHeight, qe = ke[0].height;
                if (Pe === "middle" && De === 0 || !$e || !qe) Re.attr("transform", null);
                else {
                  var Xe = { top: 1, bottom: -1 }[Pe] * (0.5 * ($e - qe + 3)) || 0, Ve = ve.indentation;
                  Re.attr("transform", ae(Ve, Xe));
                }
                Re.selectAll("g.legendfill").data([ke]).enter().append("g").classed("legendfill", !0), Re.selectAll("g.legendlines").data([ke]).enter().append("g").classed("legendlines", !0);
                var Ze = Re.selectAll("g.legendsymbols").data([ke]);
                Ze.enter().append("g").classed("legendsymbols", !0), Ze.selectAll("g.legendpoints").data([ke]).enter().append("g").classed("legendpoints", !0);
              }).each(function(ke) {
                var Me, Re = ke[0].trace, De = [];
                if (Re.visible) switch (Re.type) {
                  case "histogram2d":
                  case "heatmap":
                    De = [["M-15,-2V4H15V-2Z"]], Me = !0;
                    break;
                  case "choropleth":
                  case "choroplethmapbox":
                  case "choroplethmap":
                    De = [["M-6,-6V6H6V-6Z"]], Me = !0;
                    break;
                  case "densitymapbox":
                  case "densitymap":
                    De = [["M-6,0 a6,6 0 1,0 12,0 a 6,6 0 1,0 -12,0"]], Me = "radial";
                    break;
                  case "cone":
                    De = [["M-6,2 A2,2 0 0,0 -6,6 V6L6,4Z"], ["M-6,-6 A2,2 0 0,0 -6,-2 L6,-4Z"], ["M-6,-2 A2,2 0 0,0 -6,2 L6,0Z"]], Me = !1;
                    break;
                  case "streamtube":
                    De = [["M-6,2 A2,2 0 0,0 -6,6 H6 A2,2 0 0,1 6,2 Z"], ["M-6,-6 A2,2 0 0,0 -6,-2 H6 A2,2 0 0,1 6,-6 Z"], ["M-6,-2 A2,2 0 0,0 -6,2 H6 A2,2 0 0,1 6,-2 Z"]], Me = !1;
                    break;
                  case "surface":
                    De = [["M-6,-6 A2,3 0 0,0 -6,0 H6 A2,3 0 0,1 6,-6 Z"], ["M-6,1 A2,3 0 0,1 -6,6 H6 A2,3 0 0,0 6,0 Z"]], Me = !0;
                    break;
                  case "mesh3d":
                    De = [["M-6,6H0L-6,-6Z"], ["M6,6H0L6,-6Z"], ["M-6,-6H6L0,6Z"]], Me = !1;
                    break;
                  case "volume":
                    De = [["M-6,6H0L-6,-6Z"], ["M6,6H0L6,-6Z"], ["M-6,-6H6L0,6Z"]], Me = !0;
                    break;
                  case "isosurface":
                    De = [["M-6,6H0L-6,-6Z"], ["M6,6H0L6,-6Z"], ["M-6,-6 A12,24 0 0,0 6,-6 L0,6Z"]], Me = !1;
                }
                var Pe = te.select(this).select("g.legendpoints").selectAll("path.legend3dandfriends").data(De);
                Pe.enter().append("path").classed("legend3dandfriends", !0).attr("transform", Ee).style("stroke-miterlimit", 1), Pe.exit().remove(), Pe.each(function($e, qe) {
                  var Xe, Ve = te.select(this), Ze = ce(Re), Ge = Ze.colorscale, ze = Ze.reversescale;
                  if (Ge) {
                    if (!Me) {
                      var We = Ge.length;
                      Xe = qe === 0 ? Ge[ze ? We - 1 : 0][1] : qe === 1 ? Ge[ze ? 0 : We - 1][1] : Ge[Math.floor((We - 1) / 2)][1];
                    }
                  } else {
                    var at = Re.vertexcolor || Re.facecolor || Re.color;
                    Xe = ee.isArrayOrTypedArray(at) ? at[qe] || at[0] : at;
                  }
                  Ve.attr("d", $e[0]), Xe ? Ve.call(se.fill, Xe) : Ve.call(function(ct) {
                    if (ct.size()) {
                      var et = "legendfill-" + Re.uid;
                      oe.gradient(ct, ge, et, he(ze, Me === "radial"), Ge, "fill");
                    }
                  });
                });
              }).each(function(ke) {
                var Me = ke[0].trace, Re = Me.type === "waterfall";
                if (ke[0]._distinct && Re) {
                  var De = ke[0].trace[ke[0].dir].marker;
                  return ke[0].mc = De.color, ke[0].mlw = De.line.width, ke[0].mlc = De.line.color, Le(ke, this, "waterfall");
                }
                var Pe = [];
                Me.visible && Re && (Pe = ke[0].hasTotals ? [["increasing", "M-6,-6V6H0Z"], ["totals", "M6,6H0L-6,-6H-0Z"], ["decreasing", "M6,6V-6H0Z"]] : [["increasing", "M-6,-6V6H6Z"], ["decreasing", "M6,6V-6H-6Z"]]);
                var $e = te.select(this).select("g.legendpoints").selectAll("path.legendwaterfall").data(Pe);
                $e.enter().append("path").classed("legendwaterfall", !0).attr("transform", Ee).style("stroke-miterlimit", 1), $e.exit().remove(), $e.each(function(qe) {
                  var Xe = te.select(this), Ve = Me[qe[0]].marker, Ze = xe(void 0, Ve.line, 5, 2);
                  Xe.attr("d", qe[1]).style("stroke-width", Ze + "px").call(se.fill, Ve.color), Ze && Xe.call(se.stroke, Ve.line.color);
                });
              }).each(function(ke) {
                Le(ke, this, "funnel");
              }).each(function(ke) {
                Le(ke, this);
              }).each(function(ke) {
                var Me = ke[0].trace, Re = te.select(this).select("g.legendpoints").selectAll("path.legendbox").data(Me.visible && ne.traceIs(Me, "box-violin") ? [ke] : []);
                Re.enter().append("path").classed("legendbox", !0).attr("d", "M6,6H-6V-6H6Z").attr("transform", Ee), Re.exit().remove(), Re.each(function() {
                  var De = te.select(this);
                  if (Me.boxpoints !== "all" && Me.points !== "all" || se.opacity(Me.fillcolor) !== 0 || se.opacity((Me.line || {}).color) !== 0) {
                    var Pe = xe(void 0, Me.line, 5, 2);
                    De.style("stroke-width", Pe + "px").call(se.fill, Me.fillcolor), Pe && se.stroke(De, Me.line.color);
                  } else {
                    var $e = ee.minExtend(Me, { marker: { size: Ce ? 12 : ee.constrain(Me.marker.size, 2, 16), sizeref: 1, sizemin: 1, sizemode: "diameter" } });
                    Re.call(oe.pointStyle, $e, ge);
                  }
                });
              }).each(function(ke) {
                Qe(ke, this, "funnelarea");
              }).each(function(ke) {
                Qe(ke, this, "pie");
              }).each(function(ke) {
                var Me, Re, De = me(ke), Pe = De.showFill, $e = De.showLine, qe = De.showGradientLine, Xe = De.showGradientFill, Ve = De.anyFill, Ze = De.anyLine, Ge = ke[0], ze = Ge.trace, We = ce(ze), at = We.colorscale, ct = We.reversescale, et = le.hasMarkers(ze) || !Ve ? "M5,0" : Ze ? "M5,-2" : "M5,-3", dt = te.select(this), Je = dt.select(".legendfill").selectAll("path").data(Pe || Xe ? [ke] : []);
                if (Je.enter().append("path").classed("js-fill", !0), Je.exit().remove(), Je.attr("d", et + "h" + Se + "v6h-" + Se + "z").call(function(gt) {
                  if (gt.size()) if (Pe) oe.fillGroupStyle(gt, ge, !0);
                  else {
                    var bt = "legendfill-" + ze.uid;
                    oe.gradient(gt, ge, bt, he(ct), at, "fill");
                  }
                }), $e || qe) {
                  var it = xe(void 0, ze.line, 10, 5);
                  Re = ee.minExtend(ze, { line: { width: it } }), Me = [ee.minExtend(Ge, { trace: Re })];
                }
                var st = dt.select(".legendlines").selectAll("path").data($e || qe ? [Me] : []);
                st.enter().append("path").classed("js-line", !0), st.exit().remove(), st.attr("d", et + (qe ? "l" + Se + ",0.0001" : "h" + Se)).call($e ? oe.lineGroupStyle : function(gt) {
                  if (gt.size()) {
                    var bt = "legendline-" + ze.uid;
                    oe.lineGroupStyle(gt), oe.gradient(gt, ge, bt, he(ct), at, "stroke");
                  }
                });
              }).each(function(ke) {
                var Me, Re, De = me(ke), Pe = De.anyFill, $e = De.anyLine, qe = De.showLine, Xe = De.showMarker, Ve = ke[0], Ze = Ve.trace, Ge = !Xe && !$e && !Pe && le.hasText(Ze);
                function ze(st, gt, bt, yt) {
                  var St = ee.nestedProperty(Ze, st).get(), Lt = ee.isArrayOrTypedArray(St) && gt ? gt(St) : St;
                  if (Ce && Lt && yt !== void 0 && (Lt = yt), bt) {
                    if (Lt < bt[0]) return bt[0];
                    if (Lt > bt[1]) return bt[1];
                  }
                  return Lt;
                }
                function We(st) {
                  return Ve._distinct && Ve.index && st[Ve.index] ? st[Ve.index] : st[0];
                }
                if (Xe || Ge || qe) {
                  var at = {}, ct = {};
                  if (Xe) {
                    at.mc = ze("marker.color", We), at.mx = ze("marker.symbol", We), at.mo = ze("marker.opacity", ee.mean, [0.2, 1]), at.mlc = ze("marker.line.color", We), at.mlw = ze("marker.line.width", ee.mean, [0, 5], 2), ct.marker = { sizeref: 1, sizemin: 1, sizemode: "diameter" };
                    var et = ze("marker.size", ee.mean, [2, 16], 12);
                    at.ms = et, ct.marker.size = et;
                  }
                  qe && (ct.line = { width: ze("line.width", We, [0, 10], 5) }), Ge && (at.tx = "Aa", at.tp = ze("textposition", We), at.ts = 10, at.tc = ze("textfont.color", We), at.tf = ze("textfont.family", We), at.tw = ze("textfont.weight", We), at.ty = ze("textfont.style", We), at.tv = ze("textfont.variant", We), at.tC = ze("textfont.textcase", We), at.tE = ze("textfont.lineposition", We), at.tS = ze("textfont.shadow", We)), Me = [ee.minExtend(Ve, at)], (Re = ee.minExtend(Ze, ct)).selectedpoints = null, Re.texttemplate = null;
                }
                var dt = te.select(this).select("g.legendpoints"), Je = dt.selectAll("path.scatterpts").data(Xe ? Me : []);
                Je.enter().insert("path", ":first-child").classed("scatterpts", !0).attr("transform", Ee), Je.exit().remove(), Je.call(oe.pointStyle, Re, ge), Xe && (Me[0].mrc = 3);
                var it = dt.selectAll("g.pointtext").data(Ge ? Me : []);
                it.enter().append("g").classed("pointtext", !0).append("text").attr("transform", Ee), it.exit().remove(), it.selectAll("text").call(oe.textPointStyle, Re, ge);
              }).each(function(ke) {
                var Me = ke[0].trace, Re = te.select(this).select("g.legendpoints").selectAll("path.legendcandle").data(Me.visible && Me.type === "candlestick" ? [ke, ke] : []);
                Re.enter().append("path").classed("legendcandle", !0).attr("d", function(De, Pe) {
                  return Pe ? "M-15,0H-8M-8,6V-6H8Z" : "M15,0H8M8,-6V6H-8Z";
                }).attr("transform", Ee).style("stroke-miterlimit", 1), Re.exit().remove(), Re.each(function(De, Pe) {
                  var $e = te.select(this), qe = Me[Pe ? "increasing" : "decreasing"], Xe = xe(void 0, qe.line, 5, 2);
                  $e.style("stroke-width", Xe + "px").call(se.fill, qe.fillcolor), Xe && se.stroke($e, qe.line.color);
                });
              }).each(function(ke) {
                var Me = ke[0].trace, Re = te.select(this).select("g.legendpoints").selectAll("path.legendohlc").data(Me.visible && Me.type === "ohlc" ? [ke, ke] : []);
                Re.enter().append("path").classed("legendohlc", !0).attr("d", function(De, Pe) {
                  return Pe ? "M-15,0H0M-8,-6V0" : "M15,0H0M8,6V0";
                }).attr("transform", Ee).style("stroke-miterlimit", 1), Re.exit().remove(), Re.each(function(De, Pe) {
                  var $e = te.select(this), qe = Me[Pe ? "increasing" : "decreasing"], Xe = xe(void 0, qe.line, 5, 2);
                  $e.style("fill", "none").call(oe.dashLine, qe.line.dash, Xe), Xe && se.stroke($e, qe.line.color);
                });
              });
            };
          }, 50308: function(G, J, Z) {
            Z(87632), G.exports = { editType: "modebar", orientation: { valType: "enumerated", values: ["v", "h"], dflt: "h", editType: "modebar" }, bgcolor: { valType: "color", editType: "modebar" }, color: { valType: "color", editType: "modebar" }, activecolor: { valType: "color", editType: "modebar" }, uirevision: { valType: "any", editType: "none" }, add: { valType: "string", arrayOk: !0, dflt: "", editType: "modebar" }, remove: { valType: "string", arrayOk: !0, dflt: "", editType: "modebar" } };
          }, 5832: function(G, J, Z) {
            var te = Z(33626), ne = Z(44122), ee = Z(5975), ae = Z(35188), oe = Z(28231).eraseActiveShape, se = Z(34809), ce = se._, le = G.exports = {};
            function ue(Se, we) {
              var Ee, xe, Le = we.currentTarget, Qe = Le.getAttribute("data-attr"), ke = Le.getAttribute("data-val") || !0, Me = Se._fullLayout, Re = {}, De = ee.list(Se, null, !0), Pe = Me._cartesianSpikesEnabled;
              if (Qe === "zoom") {
                var $e, qe = ke === "in" ? 0.5 : 2, Xe = (1 + qe) / 2, Ve = (1 - qe) / 2;
                for (xe = 0; xe < De.length; xe++) if (!(Ee = De[xe]).fixedrange) if ($e = Ee._name, ke === "auto") Re[$e + ".autorange"] = !0;
                else if (ke === "reset") Ee._rangeInitial0 === void 0 && Ee._rangeInitial1 === void 0 ? Re[$e + ".autorange"] = !0 : Ee._rangeInitial0 === void 0 ? (Re[$e + ".autorange"] = Ee._autorangeInitial, Re[$e + ".range"] = [null, Ee._rangeInitial1]) : Ee._rangeInitial1 === void 0 ? (Re[$e + ".range"] = [Ee._rangeInitial0, null], Re[$e + ".autorange"] = Ee._autorangeInitial) : Re[$e + ".range"] = [Ee._rangeInitial0, Ee._rangeInitial1], Ee._showSpikeInitial !== void 0 && (Re[$e + ".showspikes"] = Ee._showSpikeInitial, Pe !== "on" || Ee._showSpikeInitial || (Pe = "off"));
                else {
                  var Ze = [Ee.r2l(Ee.range[0]), Ee.r2l(Ee.range[1])], Ge = [Xe * Ze[0] + Ve * Ze[1], Xe * Ze[1] + Ve * Ze[0]];
                  Re[$e + ".range[0]"] = Ee.l2r(Ge[0]), Re[$e + ".range[1]"] = Ee.l2r(Ge[1]);
                }
              } else Qe !== "hovermode" || ke !== "x" && ke !== "y" || (ke = Me._isHoriz ? "y" : "x", Le.setAttribute("data-val", ke)), Re[Qe] = ke;
              Me._cartesianSpikesEnabled = Pe, te.call("_guiRelayout", Se, Re);
            }
            function de(Se, we) {
              for (var Ee = we.currentTarget, xe = Ee.getAttribute("data-attr"), Le = Ee.getAttribute("data-val") || !0, Qe = Se._fullLayout._subplots.gl3d || [], ke = {}, Me = xe.split("."), Re = 0; Re < Qe.length; Re++) ke[Qe[Re] + "." + Me[1]] = Le;
              var De = Le === "pan" ? Le : "zoom";
              ke.dragmode = De, te.call("_guiRelayout", Se, ke);
            }
            function fe(Se, we) {
              for (var Ee = we.currentTarget.getAttribute("data-attr"), xe = Ee === "resetLastSave", Le = Ee === "resetDefault", Qe = Se._fullLayout, ke = Qe._subplots.gl3d || [], Me = {}, Re = 0; Re < ke.length; Re++) {
                var De, Pe = ke[Re], $e = Pe + ".camera", qe = Pe + ".aspectratio", Xe = Pe + ".aspectmode", Ve = Qe[Pe]._scene;
                xe ? (Me[$e + ".up"] = Ve.viewInitial.up, Me[$e + ".eye"] = Ve.viewInitial.eye, Me[$e + ".center"] = Ve.viewInitial.center, De = !0) : Le && (Me[$e + ".up"] = null, Me[$e + ".eye"] = null, Me[$e + ".center"] = null, De = !0), De && (Me[qe + ".x"] = Ve.viewInitial.aspectratio.x, Me[qe + ".y"] = Ve.viewInitial.aspectratio.y, Me[qe + ".z"] = Ve.viewInitial.aspectratio.z, Me[Xe] = Ve.viewInitial.aspectmode);
              }
              te.call("_guiRelayout", Se, Me);
            }
            function he(Se, we) {
              var Ee = we.currentTarget, xe = Ee._previousVal, Le = Se._fullLayout, Qe = Le._subplots.gl3d || [], ke = ["xaxis", "yaxis", "zaxis"], Me = {}, Re = {};
              if (xe) Re = xe, Ee._previousVal = null;
              else {
                for (var De = 0; De < Qe.length; De++) {
                  var Pe = Qe[De], $e = Le[Pe], qe = Pe + ".hovermode";
                  Me[qe] = $e.hovermode, Re[qe] = !1;
                  for (var Xe = 0; Xe < 3; Xe++) {
                    var Ve = ke[Xe], Ze = Pe + "." + Ve + ".showspikes";
                    Re[Ze] = !1, Me[Ze] = $e[Ve].showspikes;
                  }
                }
                Ee._previousVal = Me;
              }
              return Re;
            }
            function me(Se, we) {
              for (var Ee = we.currentTarget, xe = Ee.getAttribute("data-attr"), Le = Ee.getAttribute("data-val") || !0, Qe = Se._fullLayout, ke = Qe._subplots.geo || [], Me = 0; Me < ke.length; Me++) {
                var Re = ke[Me], De = Qe[Re];
                if (xe === "zoom") {
                  var Pe = De.projection.scale, $e = Le === "in" ? 2 * Pe : 0.5 * Pe;
                  te.call("_guiRelayout", Se, Re + ".projection.scale", $e);
                }
              }
              xe === "reset" && Ce(Se, "geo");
            }
            function pe(Se) {
              var we = Se._fullLayout;
              return !we.hovermode && (we._has("cartesian") ? we._isHoriz ? "y" : "x" : "closest");
            }
            function ye(Se) {
              var we = pe(Se);
              te.call("_guiRelayout", Se, "hovermode", we);
            }
            function ge(Se, we) {
              be(Se, we, "mapbox");
            }
            function ve(Se, we) {
              be(Se, we, "map");
            }
            function be(Se, we, Ee) {
              for (var xe = we.currentTarget.getAttribute("data-val"), Le = Se._fullLayout, Qe = Le._subplots[Ee] || [], ke = {}, Me = 0; Me < Qe.length; Me++) {
                var Re = Qe[Me], De = Le[Re].zoom, Pe = xe === "in" ? 1.05 * De : De / 1.05;
                ke[Re + ".zoom"] = Pe;
              }
              te.call("_guiRelayout", Se, ke);
            }
            function Ce(Se, we) {
              for (var Ee = Se._fullLayout, xe = Ee._subplots[we] || [], Le = {}, Qe = 0; Qe < xe.length; Qe++) for (var ke = xe[Qe], Me = Ee[ke]._subplot.viewInitial, Re = Object.keys(Me), De = 0; De < Re.length; De++) {
                var Pe = Re[De];
                Le[ke + "." + Pe] = Me[Pe];
              }
              te.call("_guiRelayout", Se, Le);
            }
            le.toImage = { name: "toImage", title: function(Se) {
              var we = (Se._context.toImageButtonOptions || {}).format || "png";
              return ce(Se, we === "png" ? "Download plot as a png" : "Download plot");
            }, icon: ae.camera, click: function(Se) {
              var we = Se._context.toImageButtonOptions, Ee = { format: we.format || "png" };
              se.notifier(ce(Se, "Taking snapshot - this may take a few seconds"), "long"), Ee.format !== "svg" && se.isIE() && (se.notifier(ce(Se, "IE only supports svg.  Changing format to svg."), "long"), Ee.format = "svg"), ["filename", "width", "height", "scale"].forEach(function(xe) {
                xe in we && (Ee[xe] = we[xe]);
              }), te.call("downloadImage", Se, Ee).then(function(xe) {
                se.notifier(ce(Se, "Snapshot succeeded") + " - " + xe, "long");
              }).catch(function() {
                se.notifier(ce(Se, "Sorry, there was a problem downloading your snapshot!"), "long");
              });
            } }, le.sendDataToCloud = { name: "sendDataToCloud", title: function(Se) {
              return ce(Se, "Edit in Chart Studio");
            }, icon: ae.disk, click: function(Se) {
              ne.sendDataToCloud(Se);
            } }, le.editInChartStudio = { name: "editInChartStudio", title: function(Se) {
              return ce(Se, "Edit in Chart Studio");
            }, icon: ae.pencil, click: function(Se) {
              ne.sendDataToCloud(Se);
            } }, le.zoom2d = { name: "zoom2d", _cat: "zoom", title: function(Se) {
              return ce(Se, "Zoom");
            }, attr: "dragmode", val: "zoom", icon: ae.zoombox, click: ue }, le.pan2d = { name: "pan2d", _cat: "pan", title: function(Se) {
              return ce(Se, "Pan");
            }, attr: "dragmode", val: "pan", icon: ae.pan, click: ue }, le.select2d = { name: "select2d", _cat: "select", title: function(Se) {
              return ce(Se, "Box Select");
            }, attr: "dragmode", val: "select", icon: ae.selectbox, click: ue }, le.lasso2d = { name: "lasso2d", _cat: "lasso", title: function(Se) {
              return ce(Se, "Lasso Select");
            }, attr: "dragmode", val: "lasso", icon: ae.lasso, click: ue }, le.drawclosedpath = { name: "drawclosedpath", title: function(Se) {
              return ce(Se, "Draw closed freeform");
            }, attr: "dragmode", val: "drawclosedpath", icon: ae.drawclosedpath, click: ue }, le.drawopenpath = { name: "drawopenpath", title: function(Se) {
              return ce(Se, "Draw open freeform");
            }, attr: "dragmode", val: "drawopenpath", icon: ae.drawopenpath, click: ue }, le.drawline = { name: "drawline", title: function(Se) {
              return ce(Se, "Draw line");
            }, attr: "dragmode", val: "drawline", icon: ae.drawline, click: ue }, le.drawrect = { name: "drawrect", title: function(Se) {
              return ce(Se, "Draw rectangle");
            }, attr: "dragmode", val: "drawrect", icon: ae.drawrect, click: ue }, le.drawcircle = { name: "drawcircle", title: function(Se) {
              return ce(Se, "Draw circle");
            }, attr: "dragmode", val: "drawcircle", icon: ae.drawcircle, click: ue }, le.eraseshape = { name: "eraseshape", title: function(Se) {
              return ce(Se, "Erase active shape");
            }, icon: ae.eraseshape, click: oe }, le.zoomIn2d = { name: "zoomIn2d", _cat: "zoomin", title: function(Se) {
              return ce(Se, "Zoom in");
            }, attr: "zoom", val: "in", icon: ae.zoom_plus, click: ue }, le.zoomOut2d = { name: "zoomOut2d", _cat: "zoomout", title: function(Se) {
              return ce(Se, "Zoom out");
            }, attr: "zoom", val: "out", icon: ae.zoom_minus, click: ue }, le.autoScale2d = { name: "autoScale2d", _cat: "autoscale", title: function(Se) {
              return ce(Se, "Autoscale");
            }, attr: "zoom", val: "auto", icon: ae.autoscale, click: ue }, le.resetScale2d = { name: "resetScale2d", _cat: "resetscale", title: function(Se) {
              return ce(Se, "Reset axes");
            }, attr: "zoom", val: "reset", icon: ae.home, click: ue }, le.hoverClosestCartesian = { name: "hoverClosestCartesian", _cat: "hoverclosest", title: function(Se) {
              return ce(Se, "Show closest data on hover");
            }, attr: "hovermode", val: "closest", icon: ae.tooltip_basic, gravity: "ne", click: ue }, le.hoverCompareCartesian = { name: "hoverCompareCartesian", _cat: "hoverCompare", title: function(Se) {
              return ce(Se, "Compare data on hover");
            }, attr: "hovermode", val: function(Se) {
              return Se._fullLayout._isHoriz ? "y" : "x";
            }, icon: ae.tooltip_compare, gravity: "ne", click: ue }, le.zoom3d = { name: "zoom3d", _cat: "zoom", title: function(Se) {
              return ce(Se, "Zoom");
            }, attr: "scene.dragmode", val: "zoom", icon: ae.zoombox, click: de }, le.pan3d = { name: "pan3d", _cat: "pan", title: function(Se) {
              return ce(Se, "Pan");
            }, attr: "scene.dragmode", val: "pan", icon: ae.pan, click: de }, le.orbitRotation = { name: "orbitRotation", title: function(Se) {
              return ce(Se, "Orbital rotation");
            }, attr: "scene.dragmode", val: "orbit", icon: ae["3d_rotate"], click: de }, le.tableRotation = { name: "tableRotation", title: function(Se) {
              return ce(Se, "Turntable rotation");
            }, attr: "scene.dragmode", val: "turntable", icon: ae["z-axis"], click: de }, le.resetCameraDefault3d = { name: "resetCameraDefault3d", _cat: "resetCameraDefault", title: function(Se) {
              return ce(Se, "Reset camera to default");
            }, attr: "resetDefault", icon: ae.home, click: fe }, le.resetCameraLastSave3d = { name: "resetCameraLastSave3d", _cat: "resetCameraLastSave", title: function(Se) {
              return ce(Se, "Reset camera to last save");
            }, attr: "resetLastSave", icon: ae.movie, click: fe }, le.hoverClosest3d = { name: "hoverClosest3d", _cat: "hoverclosest", title: function(Se) {
              return ce(Se, "Toggle show closest data on hover");
            }, attr: "hovermode", val: null, toggle: !0, icon: ae.tooltip_basic, gravity: "ne", click: function(Se, we) {
              var Ee = he(Se, we);
              te.call("_guiRelayout", Se, Ee);
            } }, le.zoomInGeo = { name: "zoomInGeo", _cat: "zoomin", title: function(Se) {
              return ce(Se, "Zoom in");
            }, attr: "zoom", val: "in", icon: ae.zoom_plus, click: me }, le.zoomOutGeo = { name: "zoomOutGeo", _cat: "zoomout", title: function(Se) {
              return ce(Se, "Zoom out");
            }, attr: "zoom", val: "out", icon: ae.zoom_minus, click: me }, le.resetGeo = { name: "resetGeo", _cat: "reset", title: function(Se) {
              return ce(Se, "Reset");
            }, attr: "reset", val: null, icon: ae.autoscale, click: me }, le.hoverClosestGeo = { name: "hoverClosestGeo", _cat: "hoverclosest", title: function(Se) {
              return ce(Se, "Toggle show closest data on hover");
            }, attr: "hovermode", val: null, toggle: !0, icon: ae.tooltip_basic, gravity: "ne", click: ye }, le.hoverClosestGl2d = { name: "hoverClosestGl2d", _cat: "hoverclosest", title: function(Se) {
              return ce(Se, "Toggle show closest data on hover");
            }, attr: "hovermode", val: null, toggle: !0, icon: ae.tooltip_basic, gravity: "ne", click: ye }, le.hoverClosestPie = { name: "hoverClosestPie", _cat: "hoverclosest", title: function(Se) {
              return ce(Se, "Toggle show closest data on hover");
            }, attr: "hovermode", val: "closest", icon: ae.tooltip_basic, gravity: "ne", click: ye }, le.resetViewSankey = { name: "resetSankeyGroup", title: function(Se) {
              return ce(Se, "Reset view");
            }, icon: ae.home, click: function(Se) {
              for (var we = { "node.groups": [], "node.x": [], "node.y": [] }, Ee = 0; Ee < Se._fullData.length; Ee++) {
                var xe = Se._fullData[Ee]._viewInitial;
                we["node.groups"].push(xe.node.groups.slice()), we["node.x"].push(xe.node.x.slice()), we["node.y"].push(xe.node.y.slice());
              }
              te.call("restyle", Se, we);
            } }, le.toggleHover = { name: "toggleHover", title: function(Se) {
              return ce(Se, "Toggle show closest data on hover");
            }, attr: "hovermode", val: null, toggle: !0, icon: ae.tooltip_basic, gravity: "ne", click: function(Se, we) {
              var Ee = he(Se, we);
              Ee.hovermode = pe(Se), te.call("_guiRelayout", Se, Ee);
            } }, le.resetViews = { name: "resetViews", title: function(Se) {
              return ce(Se, "Reset views");
            }, icon: ae.home, click: function(Se, we) {
              var Ee = we.currentTarget;
              Ee.setAttribute("data-attr", "zoom"), Ee.setAttribute("data-val", "reset"), ue(Se, we), Ee.setAttribute("data-attr", "resetLastSave"), fe(Se, we), Ce(Se, "geo"), Ce(Se, "mapbox"), Ce(Se, "map");
            } }, le.toggleSpikelines = { name: "toggleSpikelines", title: function(Se) {
              return ce(Se, "Toggle Spike Lines");
            }, icon: ae.spikeline, attr: "_cartesianSpikesEnabled", val: "on", click: function(Se) {
              var we = Se._fullLayout, Ee = we._cartesianSpikesEnabled;
              we._cartesianSpikesEnabled = Ee === "on" ? "off" : "on", te.call("_guiRelayout", Se, function(xe) {
                for (var Le = xe._fullLayout._cartesianSpikesEnabled === "on", Qe = ee.list(xe, null, !0), ke = {}, Me = 0; Me < Qe.length; Me++) {
                  var Re = Qe[Me];
                  ke[Re._name + ".showspikes"] = !!Le || Re._showSpikeInitial;
                }
                return ke;
              }(Se));
            } }, le.resetViewMapbox = { name: "resetViewMapbox", _cat: "resetView", title: function(Se) {
              return ce(Se, "Reset view");
            }, attr: "reset", icon: ae.home, click: function(Se) {
              Ce(Se, "mapbox");
            } }, le.resetViewMap = { name: "resetViewMap", _cat: "resetView", title: function(Se) {
              return ce(Se, "Reset view");
            }, attr: "reset", icon: ae.home, click: function(Se) {
              Ce(Se, "map");
            } }, le.zoomInMapbox = { name: "zoomInMapbox", _cat: "zoomin", title: function(Se) {
              return ce(Se, "Zoom in");
            }, attr: "zoom", val: "in", icon: ae.zoom_plus, click: ge }, le.zoomInMap = { name: "zoomInMap", _cat: "zoomin", title: function(Se) {
              return ce(Se, "Zoom in");
            }, attr: "zoom", val: "in", icon: ae.zoom_plus, click: ve }, le.zoomOutMapbox = { name: "zoomOutMapbox", _cat: "zoomout", title: function(Se) {
              return ce(Se, "Zoom out");
            }, attr: "zoom", val: "out", icon: ae.zoom_minus, click: ge }, le.zoomOutMap = { name: "zoomOutMap", _cat: "zoomout", title: function(Se) {
              return ce(Se, "Zoom out");
            }, attr: "zoom", val: "out", icon: ae.zoom_minus, click: ve };
          }, 87632: function(G, J, Z) {
            var te = Z(5832), ne = Object.keys(te), ee = ["drawline", "drawopenpath", "drawclosedpath", "drawcircle", "drawrect", "eraseshape"], ae = ["v1hovermode", "hoverclosest", "hovercompare", "togglehover", "togglespikelines"].concat(ee), oe = [];
            ne.forEach(function(se) {
              (function(ce) {
                if (ae.indexOf(ce._cat || ce.name) === -1) {
                  var le = ce.name, ue = (ce._cat || ce.name).toLowerCase();
                  oe.indexOf(le) === -1 && oe.push(le), oe.indexOf(ue) === -1 && oe.push(ue);
                }
              })(te[se]);
            }), oe.sort(), G.exports = { DRAW_MODES: ee, backButtons: ae, foreButtons: oe };
          }, 17683: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(78032), ae = Z(50308);
            G.exports = function(oe, se) {
              var ce = oe.modebar || {}, le = ee.newContainer(se, "modebar");
              function ue(fe, he) {
                return te.coerce(ce, le, ae, fe, he);
              }
              ue("orientation"), ue("bgcolor", ne.addOpacity(se.paper_bgcolor, 0.5));
              var de = ne.contrast(ne.rgb(se.modebar.bgcolor));
              ue("color", ne.addOpacity(de, 0.3)), ue("activecolor", ne.addOpacity(de, 0.7)), ue("uirevision", se.uirevision), ue("add"), ue("remove");
            };
          }, 95433: function(G, J, Z) {
            G.exports = { moduleType: "component", name: "modebar", layoutAttributes: Z(50308), supplyLayoutDefaults: Z(17683), manage: Z(75442) };
          }, 75442: function(G, J, Z) {
            var te = Z(5975), ne = Z(64726), ee = Z(33626), ae = Z(36040).isUnifiedHover, oe = Z(85393), se = Z(5832), ce = Z(87632).DRAW_MODES, le = Z(34809).extendDeep;
            G.exports = function(ue) {
              var de = ue._fullLayout, fe = ue._context, he = de._modeBar;
              if (fe.displayModeBar || fe.watermark) {
                if (!Array.isArray(fe.modeBarButtonsToRemove)) throw new Error(["*modeBarButtonsToRemove* configuration options", "must be an array."].join(" "));
                if (!Array.isArray(fe.modeBarButtonsToAdd)) throw new Error(["*modeBarButtonsToAdd* configuration options", "must be an array."].join(" "));
                var me, pe = fe.modeBarButtons;
                me = Array.isArray(pe) && pe.length ? function(ye) {
                  for (var ge = le([], ye), ve = 0; ve < ge.length; ve++) for (var be = ge[ve], Ce = 0; Ce < be.length; Ce++) {
                    var Se = be[Ce];
                    if (typeof Se == "string") {
                      if (se[Se] === void 0) throw new Error(["*modeBarButtons* configuration options", "invalid button name"].join(" "));
                      ge[ve][Ce] = se[Se];
                    }
                  }
                  return ge;
                }(pe) : !fe.displayModeBar && fe.watermark ? [] : function(ye) {
                  var ge = ye._fullLayout, ve = ye._fullData, be = ye._context;
                  function Ce(Lt, Rt) {
                    if (typeof Rt == "string") {
                      if (Rt.toLowerCase() === Lt.toLowerCase()) return !0;
                    } else {
                      var vt = Rt.name, $t = Rt._cat || Rt.name;
                      if (vt === Lt || $t === Lt.toLowerCase()) return !0;
                    }
                    return !1;
                  }
                  var Se = ge.modebar.add;
                  typeof Se == "string" && (Se = [Se]);
                  var we = ge.modebar.remove;
                  typeof we == "string" && (we = [we]);
                  var Ee = be.modeBarButtonsToAdd.concat(Se.filter(function(Lt) {
                    for (var Rt = 0; Rt < be.modeBarButtonsToRemove.length; Rt++) if (Ce(Lt, be.modeBarButtonsToRemove[Rt])) return !1;
                    return !0;
                  })), xe = be.modeBarButtonsToRemove.concat(we.filter(function(Lt) {
                    for (var Rt = 0; Rt < be.modeBarButtonsToAdd.length; Rt++) if (Ce(Lt, be.modeBarButtonsToAdd[Rt])) return !1;
                    return !0;
                  })), Le = ge._has("cartesian"), Qe = ge._has("gl3d"), ke = ge._has("geo"), Me = ge._has("pie"), Re = ge._has("funnelarea"), De = ge._has("gl2d"), Pe = ge._has("ternary"), $e = ge._has("mapbox"), qe = ge._has("map"), Xe = ge._has("polar"), Ve = ge._has("smith"), Ze = ge._has("sankey"), Ge = function(Lt) {
                    for (var Rt = te.list({ _fullLayout: Lt }, null, !0), vt = 0; vt < Rt.length; vt++) if (!Rt[vt].fixedrange) return !1;
                    return !0;
                  }(ge), ze = ae(ge.hovermode), We = [];
                  function at(Lt) {
                    if (Lt.length) {
                      for (var Rt = [], vt = 0; vt < Lt.length; vt++) {
                        for (var $t = Lt[vt], Pt = se[$t], Xt = Pt.name.toLowerCase(), Nt = (Pt._cat || Pt.name).toLowerCase(), Gt = !1, Ft = 0; Ft < xe.length; Ft++) {
                          var Tt = xe[Ft].toLowerCase();
                          if (Tt === Xt || Tt === Nt) {
                            Gt = !0;
                            break;
                          }
                        }
                        Gt || Rt.push(se[$t]);
                      }
                      We.push(Rt);
                    }
                  }
                  var ct = ["toImage"];
                  be.showEditInChartStudio ? ct.push("editInChartStudio") : be.showSendToCloud && ct.push("sendDataToCloud"), at(ct);
                  var et = [], dt = [], Je = [], it = [];
                  (Le || De || Me || Re || Pe) + ke + Qe + $e + qe + Xe + Ve > 1 ? (dt = ["toggleHover"], Je = ["resetViews"]) : ke ? (et = ["zoomInGeo", "zoomOutGeo"], dt = ["hoverClosestGeo"], Je = ["resetGeo"]) : Qe ? (dt = ["hoverClosest3d"], Je = ["resetCameraDefault3d", "resetCameraLastSave3d"]) : $e ? (et = ["zoomInMapbox", "zoomOutMapbox"], dt = ["toggleHover"], Je = ["resetViewMapbox"]) : qe ? (et = ["zoomInMap", "zoomOutMap"], dt = ["toggleHover"], Je = ["resetViewMap"]) : De ? dt = ["hoverClosestGl2d"] : Me ? dt = ["hoverClosestPie"] : Ze ? (dt = ["hoverClosestCartesian", "hoverCompareCartesian"], Je = ["resetViewSankey"]) : dt = ["toggleHover"], Le && dt.push("toggleSpikelines", "hoverClosestCartesian", "hoverCompareCartesian"), (function(Lt) {
                    for (var Rt = 0; Rt < Lt.length; Rt++) if (!ee.traceIs(Lt[Rt], "noHover")) return !1;
                    return !0;
                  }(ve) || ze) && (dt = []), !Le && !De || Ge || (et = ["zoomIn2d", "zoomOut2d", "autoScale2d"], Je[0] !== "resetViews" && (Je = ["resetScale2d"])), Qe ? it = ["zoom3d", "pan3d", "orbitRotation", "tableRotation"] : (Le || De) && !Ge || Pe ? it = ["zoom2d", "pan2d"] : $e || qe || ke ? it = ["pan2d"] : Xe && (it = ["zoom2d"]), function(Lt) {
                    for (var Rt = !1, vt = 0; vt < Lt.length && !Rt; vt++) {
                      var $t = Lt[vt];
                      $t._module && $t._module.selectPoints && (ee.traceIs($t, "scatter-like") ? (ne.hasMarkers($t) || ne.hasText($t)) && (Rt = !0) : ee.traceIs($t, "box-violin") && $t.boxpoints !== "all" && $t.points !== "all" || (Rt = !0));
                    }
                    return Rt;
                  }(ve) && it.push("select2d", "lasso2d");
                  var st = [], gt = function(Lt) {
                    st.indexOf(Lt) === -1 && dt.indexOf(Lt) !== -1 && st.push(Lt);
                  };
                  if (Array.isArray(Ee)) {
                    for (var bt = [], yt = 0; yt < Ee.length; yt++) {
                      var St = Ee[yt];
                      typeof St == "string" ? (St = St.toLowerCase(), ce.indexOf(St) !== -1 ? (ge._has("mapbox") || ge._has("map") || ge._has("cartesian")) && it.push(St) : St === "togglespikelines" ? gt("toggleSpikelines") : St === "togglehover" ? gt("toggleHover") : St === "hovercompare" ? gt("hoverCompareCartesian") : St === "hoverclosest" ? (gt("hoverClosestCartesian"), gt("hoverClosestGeo"), gt("hoverClosest3d"), gt("hoverClosestGl2d"), gt("hoverClosestPie")) : St === "v1hovermode" && (gt("hoverClosestCartesian"), gt("hoverCompareCartesian"), gt("hoverClosestGeo"), gt("hoverClosest3d"), gt("hoverClosestGl2d"), gt("hoverClosestPie"))) : bt.push(St);
                    }
                    Ee = bt;
                  }
                  return at(it), at(et.concat(Je)), at(st), function(Lt, Rt) {
                    if (Rt.length) if (Array.isArray(Rt[0])) for (var vt = 0; vt < Rt.length; vt++) Lt.push(Rt[vt]);
                    else Lt.push(Rt);
                    return Lt;
                  }(We, Ee);
                }(ue), he ? he.update(ue, me) : de._modeBar = oe(ue, me);
              } else he && (he.destroy(), delete de._modeBar);
            };
          }, 85393: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(34809), ae = Z(35188), oe = Z(29697).version, se = new DOMParser();
            function ce(ue) {
              this.container = ue.container, this.element = document.createElement("div"), this.update(ue.graphInfo, ue.buttons), this.container.appendChild(this.element);
            }
            var le = ce.prototype;
            le.update = function(ue, de) {
              this.graphInfo = ue;
              var fe = this.graphInfo._context, he = this.graphInfo._fullLayout, me = "modebar-" + he._uid;
              this.element.setAttribute("id", me), this._uid = me, this.element.className = "modebar", fe.displayModeBar === "hover" && (this.element.className += " modebar--hover ease-bg"), he.modebar.orientation === "v" && (this.element.className += " vertical", de = de.reverse());
              var pe = he.modebar, ye = fe.displayModeBar === "hover" ? ".js-plotly-plot .plotly:hover " : "";
              ee.deleteRelatedStyleRule(me), ee.addRelatedStyleRule(me, ye + "#" + me + " .modebar-group", "background-color: " + pe.bgcolor), ee.addRelatedStyleRule(me, "#" + me + " .modebar-btn .icon path", "fill: " + pe.color), ee.addRelatedStyleRule(me, "#" + me + " .modebar-btn:hover .icon path", "fill: " + pe.activecolor), ee.addRelatedStyleRule(me, "#" + me + " .modebar-btn.active .icon path", "fill: " + pe.activecolor);
              var ge = !this.hasButtons(de), ve = this.hasLogo !== fe.displaylogo, be = this.locale !== fe.locale;
              if (this.locale = fe.locale, (ge || ve || be) && (this.removeAllButtons(), this.updateButtons(de), fe.watermark || fe.displaylogo)) {
                var Ce = this.getLogo();
                fe.watermark && (Ce.className = Ce.className + " watermark"), he.modebar.orientation === "v" ? this.element.insertBefore(Ce, this.element.childNodes[0]) : this.element.appendChild(Ce), this.hasLogo = !0;
              }
              this.updateActiveButton();
            }, le.updateButtons = function(ue) {
              var de = this;
              this.buttons = ue, this.buttonElements = [], this.buttonsNames = [], this.buttons.forEach(function(fe) {
                var he = de.createGroup();
                fe.forEach(function(me) {
                  var pe = me.name;
                  if (!pe) throw new Error("must provide button 'name' in button config");
                  if (de.buttonsNames.indexOf(pe) !== -1) throw new Error("button name '" + pe + "' is taken");
                  de.buttonsNames.push(pe);
                  var ye = de.createButton(me);
                  de.buttonElements.push(ye), he.appendChild(ye);
                }), de.element.appendChild(he);
              });
            }, le.createGroup = function() {
              var ue = document.createElement("div");
              return ue.className = "modebar-group", ue;
            }, le.createButton = function(ue) {
              var de = this, fe = document.createElement("a");
              fe.setAttribute("rel", "tooltip"), fe.className = "modebar-btn";
              var he = ue.title;
              he === void 0 ? he = ue.name : typeof he == "function" && (he = he(this.graphInfo)), (he || he === 0) && fe.setAttribute("data-title", he), ue.attr !== void 0 && fe.setAttribute("data-attr", ue.attr);
              var me = ue.val;
              if (me !== void 0 && (typeof me == "function" && (me = me(this.graphInfo)), fe.setAttribute("data-val", me)), typeof ue.click != "function") throw new Error("must provide button 'click' function in button config");
              fe.addEventListener("click", function(ye) {
                ue.click(de.graphInfo, ye), de.updateActiveButton(ye.currentTarget);
              }), fe.setAttribute("data-toggle", ue.toggle || !1), ue.toggle && te.select(fe).classed("active", !0);
              var pe = ue.icon;
              return typeof pe == "function" ? fe.appendChild(pe()) : fe.appendChild(this.createIcon(pe || ae.question)), fe.setAttribute("data-gravity", ue.gravity || "n"), fe;
            }, le.createIcon = function(ue) {
              var de, fe = ne(ue.height) ? Number(ue.height) : ue.ascent - ue.descent, he = "http://www.w3.org/2000/svg";
              if (ue.path) {
                (de = document.createElementNS(he, "svg")).setAttribute("viewBox", [0, 0, ue.width, fe].join(" ")), de.setAttribute("class", "icon");
                var me = document.createElementNS(he, "path");
                me.setAttribute("d", ue.path), ue.transform ? me.setAttribute("transform", ue.transform) : ue.ascent !== void 0 && me.setAttribute("transform", "matrix(1 0 0 -1 0 " + ue.ascent + ")"), de.appendChild(me);
              }
              return ue.svg && (de = se.parseFromString(ue.svg, "application/xml").childNodes[0]), de.setAttribute("height", "1em"), de.setAttribute("width", "1em"), de;
            }, le.updateActiveButton = function(ue) {
              var de = this.graphInfo._fullLayout, fe = ue !== void 0 ? ue.getAttribute("data-attr") : null;
              this.buttonElements.forEach(function(he) {
                var me = he.getAttribute("data-val") || !0, pe = he.getAttribute("data-attr"), ye = he.getAttribute("data-toggle") === "true", ge = te.select(he);
                if (ye) pe === fe && ge.classed("active", !ge.classed("active"));
                else {
                  var ve = pe === null ? pe : ee.nestedProperty(de, pe).get();
                  ge.classed("active", ve === me);
                }
              });
            }, le.hasButtons = function(ue) {
              var de = this.buttons;
              if (!de || ue.length !== de.length) return !1;
              for (var fe = 0; fe < ue.length; ++fe) {
                if (ue[fe].length !== de[fe].length) return !1;
                for (var he = 0; he < ue[fe].length; he++) if (ue[fe][he].name !== de[fe][he].name) return !1;
              }
              return !0;
            }, le.getLogo = function() {
              var ue = this.createGroup(), de = document.createElement("a");
              return de.href = "https://plotly.com/", de.target = "_blank", de.setAttribute("data-title", ee._(this.graphInfo, "Produced with Plotly.js") + " (v" + oe + ")"), de.className = "modebar-btn plotlyjsicon modebar-btn--logo", de.appendChild(this.createIcon(ae.newplotlylogo)), ue.appendChild(de), ue;
            }, le.removeAllButtons = function() {
              for (; this.element.firstChild; ) this.element.removeChild(this.element.firstChild);
              this.hasLogo = !1;
            }, le.destroy = function() {
              ee.removeElement(this.container.querySelector(".modebar")), ee.deleteRelatedStyleRule(this._uid);
            }, G.exports = function(ue, de) {
              var fe = ue._fullLayout, he = new ce({ graphInfo: ue, container: fe._modebardiv.node(), buttons: de });
              return fe._privateplot && te.select(he.element).append("span").classed("badge-private float--left", !0).text("PRIVATE"), he;
            };
          }, 91032: function(G, J, Z) {
            var te = Z(80337), ne = Z(10229), ee = (0, Z(78032).templatedArray)("button", { visible: { valType: "boolean", dflt: !0, editType: "plot" }, step: { valType: "enumerated", values: ["month", "year", "day", "hour", "minute", "second", "all"], dflt: "month", editType: "plot" }, stepmode: { valType: "enumerated", values: ["backward", "todate"], dflt: "backward", editType: "plot" }, count: { valType: "number", min: 0, dflt: 1, editType: "plot" }, label: { valType: "string", editType: "plot" }, editType: "plot" });
            G.exports = { visible: { valType: "boolean", editType: "plot" }, buttons: ee, x: { valType: "number", min: -2, max: 3, editType: "plot" }, xanchor: { valType: "enumerated", values: ["auto", "left", "center", "right"], dflt: "left", editType: "plot" }, y: { valType: "number", min: -2, max: 3, editType: "plot" }, yanchor: { valType: "enumerated", values: ["auto", "top", "middle", "bottom"], dflt: "bottom", editType: "plot" }, font: te({ editType: "plot" }), bgcolor: { valType: "color", dflt: ne.lightLine, editType: "plot" }, activecolor: { valType: "color", editType: "plot" }, bordercolor: { valType: "color", dflt: ne.defaultLine, editType: "plot" }, borderwidth: { valType: "number", min: 0, dflt: 0, editType: "plot" }, editType: "plot" };
          }, 68508: function(G) {
            G.exports = { yPad: 0.02, minButtonWidth: 30, rx: 3, ry: 3, lightAmount: 25, darkAmount: 10 };
          }, 86255: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(78032), ae = Z(59008), oe = Z(91032), se = Z(68508);
            function ce(le, ue, de, fe) {
              var he = fe.calendar;
              function me(ye, ge) {
                return te.coerce(le, ue, oe.buttons, ye, ge);
              }
              if (me("visible")) {
                var pe = me("step");
                pe !== "all" && (!he || he === "gregorian" || pe !== "month" && pe !== "year" ? me("stepmode") : ue.stepmode = "backward", me("count")), me("label");
              }
            }
            G.exports = function(le, ue, de, fe, he) {
              var me = le.rangeselector || {}, pe = ee.newContainer(ue, "rangeselector");
              function ye(be, Ce) {
                return te.coerce(me, pe, oe, be, Ce);
              }
              if (ye("visible", ae(me, pe, { name: "buttons", handleItemDefaults: ce, calendar: he }).length > 0)) {
                var ge = function(be, Ce, Se) {
                  for (var we = Se.filter(function(Qe) {
                    return Ce[Qe].anchor === be._id;
                  }), Ee = 0, xe = 0; xe < we.length; xe++) {
                    var Le = Ce[we[xe]].domain;
                    Le && (Ee = Math.max(Le[1], Ee));
                  }
                  return [be.domain[0], Ee + se.yPad];
                }(ue, de, fe);
                ye("x", ge[0]), ye("y", ge[1]), te.noneOrAll(le, ue, ["x", "y"]), ye("xanchor"), ye("yanchor"), te.coerceFont(ye, "font", de.font);
                var ve = ye("bgcolor");
                ye("activecolor", ne.contrast(ve, se.lightAmount, se.darkAmount)), ye("bordercolor"), ye("borderwidth");
              }
            };
          }, 45431: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(44122), ae = Z(78766), oe = Z(62203), se = Z(34809), ce = se.strTranslate, le = Z(30635), ue = Z(5975), de = Z(4530), fe = de.LINE_SPACING, he = de.FROM_TL, me = de.FROM_BR, pe = Z(68508), ye = Z(16383);
            function ge(Ce) {
              return Ce._id;
            }
            function ve(Ce, Se, we) {
              var Ee = se.ensureSingle(Ce, "rect", "selector-rect", function(xe) {
                xe.attr("shape-rendering", "crispEdges");
              });
              Ee.attr({ rx: pe.rx, ry: pe.ry }), Ee.call(ae.stroke, Se.bordercolor).call(ae.fill, function(xe, Le) {
                return Le._isActive || Le._isHovered ? xe.activecolor : xe.bgcolor;
              }(Se, we)).style("stroke-width", Se.borderwidth + "px");
            }
            function be(Ce, Se, we, Ee) {
              var xe, Le;
              se.ensureSingle(Ce, "text", "selector-text", function(Qe) {
                Qe.attr("text-anchor", "middle");
              }).call(oe.font, Se.font).text((xe = we, Le = Ee._fullLayout._meta, xe.label ? Le ? se.templateString(xe.label, Le) : xe.label : xe.step === "all" ? "all" : xe.count + xe.step.charAt(0))).call(function(Qe) {
                le.convertToTspans(Qe, Ee);
              });
            }
            G.exports = function(Ce) {
              var Se = Ce._fullLayout._infolayer.selectAll(".rangeselector").data(function(we) {
                for (var Ee = ue.list(we, "x", !0), xe = [], Le = 0; Le < Ee.length; Le++) {
                  var Qe = Ee[Le];
                  Qe.rangeselector && Qe.rangeselector.visible && xe.push(Qe);
                }
                return xe;
              }(Ce), ge);
              Se.enter().append("g").classed("rangeselector", !0), Se.exit().remove(), Se.style({ cursor: "pointer", "pointer-events": "all" }), Se.each(function(we) {
                var Ee = te.select(this), xe = we, Le = xe.rangeselector, Qe = Ee.selectAll("g.button").data(se.filterVisible(Le.buttons));
                Qe.enter().append("g").classed("button", !0), Qe.exit().remove(), Qe.each(function(ke) {
                  var Me = te.select(this), Re = ye(xe, ke);
                  ke._isActive = function(De, Pe, $e) {
                    if (Pe.step === "all") return De.autorange === !0;
                    var qe = Object.keys($e);
                    return De.range[0] === $e[qe[0]] && De.range[1] === $e[qe[1]];
                  }(xe, ke, Re), Me.call(ve, Le, ke), Me.call(be, Le, ke, Ce), Me.on("click", function() {
                    Ce._dragged || ne.call("_guiRelayout", Ce, Re);
                  }), Me.on("mouseover", function() {
                    ke._isHovered = !0, Me.call(ve, Le, ke);
                  }), Me.on("mouseout", function() {
                    ke._isHovered = !1, Me.call(ve, Le, ke);
                  });
                }), function(ke, Me, Re, De, Pe) {
                  var $e = 0, qe = 0, Xe = Re.borderwidth;
                  Me.each(function() {
                    var at = te.select(this).select(".selector-text"), ct = Re.font.size * fe, et = Math.max(ct * le.lineCount(at), 16) + 3;
                    qe = Math.max(qe, et);
                  }), Me.each(function() {
                    var at = te.select(this), ct = at.select(".selector-rect"), et = at.select(".selector-text"), dt = et.node() && oe.bBox(et.node()).width, Je = Re.font.size * fe, it = le.lineCount(et), st = Math.max(dt + 10, pe.minButtonWidth);
                    at.attr("transform", ce(Xe + $e, Xe)), ct.attr({ x: 0, y: 0, width: st, height: qe }), le.positionText(et, st / 2, qe / 2 - (it - 1) * Je / 2 + 3), $e += st + 5;
                  });
                  var Ve = ke._fullLayout._size, Ze = Ve.l + Ve.w * Re.x, Ge = Ve.t + Ve.h * (1 - Re.y), ze = "left";
                  se.isRightAnchor(Re) && (Ze -= $e, ze = "right"), se.isCenterAnchor(Re) && (Ze -= $e / 2, ze = "center");
                  var We = "top";
                  se.isBottomAnchor(Re) && (Ge -= qe, We = "bottom"), se.isMiddleAnchor(Re) && (Ge -= qe / 2, We = "middle"), $e = Math.ceil($e), qe = Math.ceil(qe), Ze = Math.round(Ze), Ge = Math.round(Ge), ee.autoMargin(ke, De + "-range-selector", { x: Re.x, y: Re.y, l: $e * he[ze], r: $e * me[ze], b: qe * me[We], t: qe * he[We] }), Pe.attr("transform", ce(Ze, Ge));
                }(Ce, Qe, Le, xe._name, Ee);
              });
            };
          }, 16383: function(G, J, Z) {
            var te = Z(50936), ne = Z(34809).titleCase;
            G.exports = function(ee, ae) {
              var oe = ee._name, se = {};
              if (ae.step === "all") se[oe + ".autorange"] = !0;
              else {
                var ce = function(le, ue) {
                  var de, fe = le.range, he = new Date(le.r2l(fe[1])), me = ue.step, pe = te["utc" + ne(me)], ye = ue.count;
                  switch (ue.stepmode) {
                    case "backward":
                      de = le.l2r(+pe.offset(he, -ye));
                      break;
                    case "todate":
                      var ge = pe.offset(he, -ye);
                      de = le.l2r(+pe.ceil(ge));
                  }
                  return [de, fe[1]];
                }(ee, ae);
                se[oe + ".range[0]"] = ce[0], se[oe + ".range[1]"] = ce[1];
              }
              return se;
            };
          }, 44453: function(G, J, Z) {
            G.exports = { moduleType: "component", name: "rangeselector", schema: { subplots: { xaxis: { rangeselector: Z(91032) } } }, layoutAttributes: Z(91032), handleDefaults: Z(86255), draw: Z(45431) };
          }, 63608: function(G, J, Z) {
            var te = Z(10229);
            G.exports = { bgcolor: { valType: "color", dflt: te.background, editType: "plot" }, bordercolor: { valType: "color", dflt: te.defaultLine, editType: "plot" }, borderwidth: { valType: "integer", dflt: 0, min: 0, editType: "plot" }, autorange: { valType: "boolean", dflt: !0, editType: "calc", impliedEdits: { "range[0]": void 0, "range[1]": void 0 } }, range: { valType: "info_array", items: [{ valType: "any", editType: "calc", impliedEdits: { "^autorange": !1 } }, { valType: "any", editType: "calc", impliedEdits: { "^autorange": !1 } }], editType: "calc", impliedEdits: { autorange: !1 } }, thickness: { valType: "number", dflt: 0.15, min: 0, max: 1, editType: "plot" }, visible: { valType: "boolean", dflt: !0, editType: "calc" }, editType: "calc" };
          }, 46223: function(G, J, Z) {
            var te = Z(5975).list, ne = Z(32919).getAutoRange, ee = Z(20604);
            G.exports = function(ae) {
              for (var oe = te(ae, "x", !0), se = 0; se < oe.length; se++) {
                var ce = oe[se], le = ce[ee.name];
                le && le.visible && le.autorange && (le._input.autorange = !0, le._input.range = le.range = ne(ae, ce));
              }
            };
          }, 20604: function(G) {
            G.exports = { name: "rangeslider", containerClassName: "rangeslider-container", bgClassName: "rangeslider-bg", rangePlotClassName: "rangeslider-rangeplot", maskMinClassName: "rangeslider-mask-min", maskMaxClassName: "rangeslider-mask-max", slideBoxClassName: "rangeslider-slidebox", grabberMinClassName: "rangeslider-grabber-min", grabAreaMinClassName: "rangeslider-grabarea-min", handleMinClassName: "rangeslider-handle-min", grabberMaxClassName: "rangeslider-grabber-max", grabAreaMaxClassName: "rangeslider-grabarea-max", handleMaxClassName: "rangeslider-handle-max", maskMinOppAxisClassName: "rangeslider-mask-min-opp-axis", maskMaxOppAxisClassName: "rangeslider-mask-max-opp-axis", maskColor: "rgba(0,0,0,0.4)", maskOppAxisColor: "rgba(0,0,0,0.2)", slideBoxFill: "transparent", slideBoxCursor: "ew-resize", grabAreaFill: "transparent", grabAreaCursor: "col-resize", grabAreaWidth: 10, handleWidth: 4, handleRadius: 1, handleStrokeWidth: 1, extraPad: 15 };
          }, 41295: function(G, J, Z) {
            var te = Z(34809), ne = Z(78032), ee = Z(5975), ae = Z(63608), oe = Z(66249);
            G.exports = function(se, ce, le) {
              var ue = se[le], de = ce[le];
              if (ue.rangeslider || ce._requestRangeslider[de._id]) {
                te.isPlainObject(ue.rangeslider) || (ue.rangeslider = {});
                var fe, he, me = ue.rangeslider, pe = ne.newContainer(de, "rangeslider");
                if (Ee("visible")) {
                  Ee("bgcolor", ce.plot_bgcolor), Ee("bordercolor"), Ee("borderwidth"), Ee("thickness"), Ee("autorange", !de.isValidRange(me.range)), Ee("range");
                  var ye = ce._subplots;
                  if (ye) for (var ge = ye.cartesian.filter(function(Le) {
                    return Le.substr(0, Le.indexOf("y")) === ee.name2id(le);
                  }).map(function(Le) {
                    return Le.substr(Le.indexOf("y"), Le.length);
                  }), ve = te.simpleMap(ge, ee.id2name), be = 0; be < ve.length; be++) {
                    var Ce = ve[be];
                    fe = me[Ce] || {}, he = ne.newContainer(pe, Ce, "yaxis");
                    var Se, we = ce[Ce];
                    fe.range && we.isValidRange(fe.range) && (Se = "fixed"), xe("rangemode", Se) !== "match" && xe("range", we.range.slice());
                  }
                  pe._input = me;
                }
              }
              function Ee(Le, Qe) {
                return te.coerce(me, pe, ae, Le, Qe);
              }
              function xe(Le, Qe) {
                return te.coerce(fe, he, oe, Le, Qe);
              }
            };
          }, 88887: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(44122), ae = Z(34809), oe = ae.strTranslate, se = Z(62203), ce = Z(78766), le = Z(17240), ue = Z(37703), de = Z(5975), fe = Z(14751), he = Z(27983), me = Z(20604);
            function pe(we) {
              return typeof we.clientX == "number" ? we.clientX : we.touches && we.touches.length > 0 ? we.touches[0].clientX : 0;
            }
            function ye(we, Ee, xe, Le) {
              var Qe = ae.ensureSingle(we, "rect", me.bgClassName, function(De) {
                De.attr({ x: 0, y: 0, "shape-rendering": "crispEdges" });
              }), ke = Le.borderwidth % 2 == 0 ? Le.borderwidth : Le.borderwidth - 1, Me = -Le._offsetShift, Re = se.crispRound(Ee, Le.borderwidth);
              Qe.attr({ width: Le._width + ke, height: Le._height + ke, transform: oe(Me, Me), "stroke-width": Re }).call(ce.stroke, Le.bordercolor).call(ce.fill, Le.bgcolor);
            }
            function ge(we, Ee, xe, Le) {
              var Qe = Ee._fullLayout;
              ae.ensureSingleById(Qe._topdefs, "clipPath", Le._clipId, function(ke) {
                ke.append("rect").attr({ x: 0, y: 0 });
              }).select("rect").attr({ width: Le._width, height: Le._height });
            }
            function ve(we, Ee, xe, Le) {
              var Qe, ke = Ee.calcdata, Me = we.selectAll("g." + me.rangePlotClassName).data(xe._subplotsWith, ae.identity);
              Me.enter().append("g").attr("class", function(Re) {
                return me.rangePlotClassName + " " + Re;
              }).call(se.setClipUrl, Le._clipId, Ee), Me.order(), Me.exit().remove(), Me.each(function(Re, De) {
                var Pe = te.select(this), $e = De === 0, qe = de.getFromId(Ee, Re, "y"), Xe = qe._name, Ve = Le[Xe], Ze = { data: [], layout: { xaxis: { type: xe.type, domain: [0, 1], range: Le.range.slice(), calendar: xe.calendar }, width: Le._width, height: Le._height, margin: { t: 0, b: 0, l: 0, r: 0 } }, _context: Ee._context };
                xe.rangebreaks && (Ze.layout.xaxis.rangebreaks = xe.rangebreaks), Ze.layout[Xe] = { type: qe.type, domain: [0, 1], range: Ve.rangemode !== "match" ? Ve.range.slice() : qe.range.slice(), calendar: qe.calendar }, qe.rangebreaks && (Ze.layout[Xe].rangebreaks = qe.rangebreaks), ee.supplyDefaults(Ze);
                var Ge = Ze._fullLayout.xaxis, ze = Ze._fullLayout[Xe];
                Ge.clearCalc(), Ge.setScale(), ze.clearCalc(), ze.setScale();
                var We = { id: Re, plotgroup: Pe, xaxis: Ge, yaxis: ze, isRangePlot: !0 };
                $e ? Qe = We : (We.mainplot = "xy", We.mainplotinfo = Qe), ue.rangePlot(Ee, We, function(at, ct) {
                  for (var et = [], dt = 0; dt < at.length; dt++) {
                    var Je = at[dt], it = Je[0].trace;
                    it.xaxis + it.yaxis === ct && et.push(Je);
                  }
                  return et;
                }(ke, Re));
              });
            }
            function be(we, Ee, xe, Le, Qe) {
              ae.ensureSingle(we, "rect", me.maskMinClassName, function(ke) {
                ke.attr({ x: 0, y: 0, "shape-rendering": "crispEdges" });
              }).attr("height", Le._height).call(ce.fill, me.maskColor), ae.ensureSingle(we, "rect", me.maskMaxClassName, function(ke) {
                ke.attr({ y: 0, "shape-rendering": "crispEdges" });
              }).attr("height", Le._height).call(ce.fill, me.maskColor), Qe.rangemode !== "match" && (ae.ensureSingle(we, "rect", me.maskMinOppAxisClassName, function(ke) {
                ke.attr({ y: 0, "shape-rendering": "crispEdges" });
              }).attr("width", Le._width).call(ce.fill, me.maskOppAxisColor), ae.ensureSingle(we, "rect", me.maskMaxOppAxisClassName, function(ke) {
                ke.attr({ y: 0, "shape-rendering": "crispEdges" });
              }).attr("width", Le._width).style("border-top", me.maskOppBorder).call(ce.fill, me.maskOppAxisColor));
            }
            function Ce(we, Ee, xe, Le) {
              Ee._context.staticPlot || ae.ensureSingle(we, "rect", me.slideBoxClassName, function(Qe) {
                Qe.attr({ y: 0, cursor: me.slideBoxCursor, "shape-rendering": "crispEdges" });
              }).attr({ height: Le._height, fill: me.slideBoxFill });
            }
            function Se(we, Ee, xe, Le) {
              var Qe = ae.ensureSingle(we, "g", me.grabberMinClassName), ke = ae.ensureSingle(we, "g", me.grabberMaxClassName), Me = { x: 0, width: me.handleWidth, rx: me.handleRadius, fill: ce.background, stroke: ce.defaultLine, "stroke-width": me.handleStrokeWidth, "shape-rendering": "crispEdges" }, Re = { y: Math.round(Le._height / 4), height: Math.round(Le._height / 2) };
              ae.ensureSingle(Qe, "rect", me.handleMinClassName, function(Pe) {
                Pe.attr(Me);
              }).attr(Re), ae.ensureSingle(ke, "rect", me.handleMaxClassName, function(Pe) {
                Pe.attr(Me);
              }).attr(Re);
              var De = { width: me.grabAreaWidth, x: 0, y: 0, fill: me.grabAreaFill, cursor: Ee._context.staticPlot ? void 0 : me.grabAreaCursor };
              ae.ensureSingle(Qe, "rect", me.grabAreaMinClassName, function(Pe) {
                Pe.attr(De);
              }).attr("height", Le._height), ae.ensureSingle(ke, "rect", me.grabAreaMaxClassName, function(Pe) {
                Pe.attr(De);
              }).attr("height", Le._height);
            }
            G.exports = function(we) {
              for (var Ee = we._fullLayout, xe = Ee._rangeSliderData, Le = 0; Le < xe.length; Le++) {
                var Qe = xe[Le][me.name];
                Qe._clipId = Qe._id + "-" + Ee._uid;
              }
              var ke = Ee._infolayer.selectAll("g." + me.containerClassName).data(xe, function(Me) {
                return Me._name;
              });
              ke.exit().each(function(Me) {
                var Re = Me[me.name];
                Ee._topdefs.select("#" + Re._clipId).remove();
              }).remove(), xe.length !== 0 && (ke.enter().append("g").classed(me.containerClassName, !0).attr("pointer-events", "all"), ke.each(function(Me) {
                var Re = te.select(this), De = Me[me.name], Pe = Ee[de.id2name(Me.anchor)], $e = De[de.id2name(Me.anchor)];
                if (De.range) {
                  var qe, Xe = ae.simpleMap(De.range, Me.r2l), Ve = ae.simpleMap(Me.range, Me.r2l);
                  qe = Ve[0] < Ve[1] ? [Math.min(Xe[0], Ve[0]), Math.max(Xe[1], Ve[1])] : [Math.max(Xe[0], Ve[0]), Math.min(Xe[1], Ve[1])], De.range = De._input.range = ae.simpleMap(qe, Me.l2r);
                }
                Me.cleanRange("rangeslider.range");
                var Ze = Ee._size, Ge = Me.domain;
                De._width = Ze.w * (Ge[1] - Ge[0]);
                var ze = Math.round(Ze.l + Ze.w * Ge[0]), We = Math.round(Ze.t + Ze.h * (1 - Me._counterDomainMin) + (Me.side === "bottom" ? Me._depth : 0) + De._offsetShift + me.extraPad);
                Re.attr("transform", oe(ze, We)), De._rl = ae.simpleMap(De.range, Me.r2l);
                var at = De._rl[0], ct = De._rl[1], et = ct - at;
                if (De.p2d = function(Lt) {
                  return Lt / De._width * et + at;
                }, De.d2p = function(Lt) {
                  return (Lt - at) / et * De._width;
                }, Me.rangebreaks) {
                  var dt = Me.locateBreaks(at, ct);
                  if (dt.length) {
                    var Je, it, st = 0;
                    for (Je = 0; Je < dt.length; Je++) st += (it = dt[Je]).max - it.min;
                    var gt = De._width / (ct - at - st), bt = [-gt * at];
                    for (Je = 0; Je < dt.length; Je++) it = dt[Je], bt.push(bt[bt.length - 1] - gt * (it.max - it.min));
                    for (De.d2p = function(Lt) {
                      for (var Rt = bt[0], vt = 0; vt < dt.length; vt++) {
                        var $t = dt[vt];
                        if (Lt >= $t.max) Rt = bt[vt + 1];
                        else if (Lt < $t.min) break;
                      }
                      return Rt + gt * Lt;
                    }, Je = 0; Je < dt.length; Je++) (it = dt[Je]).pmin = De.d2p(it.min), it.pmax = De.d2p(it.max);
                    De.p2d = function(Lt) {
                      for (var Rt = bt[0], vt = 0; vt < dt.length; vt++) {
                        var $t = dt[vt];
                        if (Lt >= $t.pmax) Rt = bt[vt + 1];
                        else if (Lt < $t.pmin) break;
                      }
                      return (Lt - Rt) / gt;
                    };
                  }
                }
                if ($e.rangemode !== "match") {
                  var yt = Pe.r2l($e.range[0]), St = Pe.r2l($e.range[1]) - yt;
                  De.d2pOppAxis = function(Lt) {
                    return (Lt - yt) / St * De._height;
                  };
                }
                Re.call(ye, we, Me, De).call(ge, we, Me, De).call(ve, we, Me, De).call(be, we, Me, De, $e).call(Ce, we, Me, De).call(Se, we, Me, De), function(Lt, Rt, vt, $t) {
                  if (!Rt._context.staticPlot) {
                    var Pt = Lt.select("rect." + me.slideBoxClassName).node(), Xt = Lt.select("rect." + me.grabAreaMinClassName).node(), Nt = Lt.select("rect." + me.grabAreaMaxClassName).node();
                    Lt.on("mousedown", Gt), Lt.on("touchstart", Gt);
                  }
                  function Gt() {
                    var Ft = te.event, Tt = Ft.target, It = pe(Ft), Ct = It - Lt.node().getBoundingClientRect().left, Kt = $t.d2p(vt._rl[0]), Vt = $t.d2p(vt._rl[1]), Yt = fe.coverSlip();
                    function sn(Qt) {
                      var Ut, Wt, rn, vn = +pe(Qt) - It;
                      switch (Tt) {
                        case Pt:
                          if (rn = "ew-resize", Kt + vn > vt._length || Vt + vn < 0) return;
                          Ut = Kt + vn, Wt = Vt + vn;
                          break;
                        case Xt:
                          if (rn = "col-resize", Kt + vn > vt._length) return;
                          Ut = Kt + vn, Wt = Vt;
                          break;
                        case Nt:
                          if (rn = "col-resize", Vt + vn < 0) return;
                          Ut = Kt, Wt = Vt + vn;
                          break;
                        default:
                          rn = "ew-resize", Ut = Ct, Wt = Ct + vn;
                      }
                      if (Wt < Ut) {
                        var dn = Wt;
                        Wt = Ut, Ut = dn;
                      }
                      $t._pixelMin = Ut, $t._pixelMax = Wt, he(te.select(Yt), rn), function(Ht, jt, cn, wn) {
                        function Rn(rt) {
                          return cn.l2r(ae.constrain(rt, wn._rl[0], wn._rl[1]));
                        }
                        var Pn = Rn(wn.p2d(wn._pixelMin)), Ie = Rn(wn.p2d(wn._pixelMax));
                        window.requestAnimationFrame(function() {
                          ne.call("_guiRelayout", jt, cn._name + ".range", [Pn, Ie]);
                        });
                      }(0, Rt, vt, $t);
                    }
                    function hn() {
                      Yt.removeEventListener("mousemove", sn), Yt.removeEventListener("mouseup", hn), this.removeEventListener("touchmove", sn), this.removeEventListener("touchend", hn), ae.removeElement(Yt);
                    }
                    this.addEventListener("touchmove", sn), this.addEventListener("touchend", hn), Yt.addEventListener("mousemove", sn), Yt.addEventListener("mouseup", hn);
                  }
                }(Re, we, Me, De), function(Lt, Rt, vt, $t, Pt, Xt) {
                  var Nt = me.handleWidth / 2;
                  function Gt(Qt) {
                    return ae.constrain(Qt, 0, $t._width);
                  }
                  function Ft(Qt) {
                    return ae.constrain(Qt, 0, $t._height);
                  }
                  function Tt(Qt) {
                    return ae.constrain(Qt, -Nt, $t._width + Nt);
                  }
                  var It = Gt($t.d2p(vt._rl[0])), Ct = Gt($t.d2p(vt._rl[1]));
                  if (Lt.select("rect." + me.slideBoxClassName).attr("x", It).attr("width", Ct - It), Lt.select("rect." + me.maskMinClassName).attr("width", It), Lt.select("rect." + me.maskMaxClassName).attr("x", Ct).attr("width", $t._width - Ct), Xt.rangemode !== "match") {
                    var Kt = $t._height - Ft($t.d2pOppAxis(Pt._rl[1])), Vt = $t._height - Ft($t.d2pOppAxis(Pt._rl[0]));
                    Lt.select("rect." + me.maskMinOppAxisClassName).attr("x", It).attr("height", Kt).attr("width", Ct - It), Lt.select("rect." + me.maskMaxOppAxisClassName).attr("x", It).attr("y", Vt).attr("height", $t._height - Vt).attr("width", Ct - It), Lt.select("rect." + me.slideBoxClassName).attr("y", Kt).attr("height", Vt - Kt);
                  }
                  var Yt = 0.5, sn = Math.round(Tt(It - Nt)) - Yt, hn = Math.round(Tt(Ct - Nt)) + Yt;
                  Lt.select("g." + me.grabberMinClassName).attr("transform", oe(sn, Yt)), Lt.select("g." + me.grabberMaxClassName).attr("transform", oe(hn, Yt));
                }(Re, 0, Me, De, Pe, $e), Me.side === "bottom" && le.draw(we, Me._id + "title", { propContainer: Me, propName: Me._name + ".title", placeholder: Ee._dfltTitle.x, attributes: { x: Me._offset + Me._length / 2, y: We + De._height + De._offsetShift + 10 + 1.5 * Me.title.font.size, "text-anchor": "middle" } });
              }));
            };
          }, 80400: function(G, J, Z) {
            var te = Z(5975), ne = Z(30635), ee = Z(20604), ae = Z(4530).LINE_SPACING, oe = ee.name;
            function se(ce) {
              var le = ce && ce[oe];
              return le && le.visible;
            }
            J.isVisible = se, J.makeData = function(ce) {
              var le = te.list({ _fullLayout: ce }, "x", !0), ue = ce.margin, de = [];
              if (!ce._has("gl2d")) for (var fe = 0; fe < le.length; fe++) {
                var he = le[fe];
                if (se(he)) {
                  de.push(he);
                  var me = he[oe];
                  me._id = oe + he._id, me._height = (ce.height - ue.b - ue.t) * me.thickness, me._offsetShift = Math.floor(me.borderwidth / 2);
                }
              }
              ce._rangeSliderData = de;
            }, J.autoMarginOpts = function(ce, le) {
              var ue = ce._fullLayout, de = le[oe], fe = le._id.charAt(0), he = 0, me = 0;
              return le.side === "bottom" && (he = le._depth, le.title.text !== ue._dfltTitle[fe] && (me = 1.5 * le.title.font.size + 10 + de._offsetShift, me += (le.title.text.match(ne.BR_TAG_ALL) || []).length * le.title.font.size * ae)), { x: 0, y: le._counterDomainMin, l: 0, r: 0, t: 0, b: de._height + he + Math.max(ue.margin.b, me), pad: ee.extraPad + 2 * de._offsetShift };
            };
          }, 55429: function(G, J, Z) {
            var te = Z(34809), ne = Z(63608), ee = Z(66249), ae = Z(80400);
            G.exports = { moduleType: "component", name: "rangeslider", schema: { subplots: { xaxis: { rangeslider: te.extendFlat({}, ne, { yaxis: ee }) } } }, layoutAttributes: Z(63608), handleDefaults: Z(41295), calcAutorange: Z(46223), draw: Z(88887), isVisible: ae.isVisible, makeData: ae.makeData, autoMarginOpts: ae.autoMarginOpts };
          }, 66249: function(G) {
            G.exports = { _isSubplotObj: !0, rangemode: { valType: "enumerated", values: ["auto", "fixed", "match"], dflt: "match", editType: "calc" }, range: { valType: "info_array", items: [{ valType: "any", editType: "plot" }, { valType: "any", editType: "plot" }], editType: "plot" }, editType: "calc" };
          }, 4327: function(G, J, Z) {
            var te = Z(50222), ne = Z(36640).line, ee = Z(94850).T, ae = Z(93049).extendFlat, oe = Z(13582).overrideAll, se = Z(78032).templatedArray;
            Z(35081), G.exports = oe(se("selection", { type: { valType: "enumerated", values: ["rect", "path"] }, xref: ae({}, te.xref, {}), yref: ae({}, te.yref, {}), x0: { valType: "any" }, x1: { valType: "any" }, y0: { valType: "any" }, y1: { valType: "any" }, path: { valType: "string", editType: "arraydraw" }, opacity: { valType: "number", min: 0, max: 1, dflt: 0.7, editType: "arraydraw" }, line: { color: ne.color, width: ae({}, ne.width, { min: 1, dflt: 1 }), dash: ae({}, ee, { dflt: "dot" }) } }), "arraydraw", "from-root");
          }, 78865: function(G) {
            G.exports = { BENDPX: 1.5, MINSELECT: 12, SELECTDELAY: 100, SELECTID: "-select" };
          }, 2272: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(59008), ae = Z(4327), oe = Z(49728);
            function se(ce, le, ue) {
              function de(Me, Re) {
                return te.coerce(ce, le, ae, Me, Re);
              }
              var fe = de("path"), he = de("type", fe ? "path" : "rect") !== "path";
              he && delete le.path, de("opacity"), de("line.color"), de("line.width"), de("line.dash");
              for (var me = ["x", "y"], pe = 0; pe < 2; pe++) {
                var ye, ge, ve, be = me[pe], Ce = { _fullLayout: ue }, Se = ne.coerceRef(ce, le, Ce, be);
                if ((ye = ne.getFromId(Ce, Se))._selectionIndices.push(le._index), ve = oe.rangeToShapePosition(ye), ge = oe.shapePositionToRange(ye), he) {
                  var we = be + "0", Ee = be + "1", xe = ce[we], Le = ce[Ee];
                  ce[we] = ge(ce[we], !0), ce[Ee] = ge(ce[Ee], !0), ne.coercePosition(le, Ce, de, Se, we), ne.coercePosition(le, Ce, de, Se, Ee);
                  var Qe = le[we], ke = le[Ee];
                  Qe !== void 0 && ke !== void 0 && (le[we] = ve(Qe), le[Ee] = ve(ke), ce[we] = xe, ce[Ee] = Le);
                }
              }
              he && te.noneOrAll(ce, le, ["x0", "x1", "y0", "y1"]);
            }
            G.exports = function(ce, le) {
              ee(ce, le, { name: "selections", handleItemDefaults: se });
              for (var ue = le.selections, de = 0; de < ue.length; de++) {
                var fe = ue[de];
                fe && fe.path === void 0 && (fe.x0 !== void 0 && fe.x1 !== void 0 && fe.y0 !== void 0 && fe.y1 !== void 0 || (le.selections[de] = null));
              }
            };
          }, 7028: function(G, J, Z) {
            var te = Z(81055).readPaths, ne = Z(561), ee = Z(78534).clearOutlineControllers, ae = Z(78766), oe = Z(62203), se = Z(78032).arrayEditor, ce = Z(49728), le = ce.getPathString;
            function ue(pe) {
              var ye = pe._fullLayout;
              for (var ge in ee(pe), ye._selectionLayer.selectAll("path").remove(), ye._plots) {
                var ve = ye._plots[ge].selectionLayer;
                ve && ve.selectAll("path").remove();
              }
              for (var be = 0; be < ye.selections.length; be++) fe(pe, be);
            }
            function de(pe) {
              return pe._context.editSelection;
            }
            function fe(pe, ye) {
              pe._fullLayout._paperdiv.selectAll('.selectionlayer [data-index="' + ye + '"]').remove();
              var ge = ce.makeSelectionsOptionsAndPlotinfo(pe, ye), ve = ge.options, be = ge.plotinfo;
              ve._input && function(Ce) {
                var Se = le(pe, ve), we = { "data-index": ye, "fill-rule": "evenodd", d: Se }, Ee = ve.opacity, xe = "rgba(0,0,0,0)", Le = ve.line.color || ae.contrast(pe._fullLayout.plot_bgcolor), Qe = ve.line.width, ke = ve.line.dash;
                Qe || (Qe = 5, ke = "solid");
                var Me = de(pe) && pe._fullLayout._activeSelectionIndex === ye;
                Me && (xe = pe._fullLayout.activeselection.fillcolor, Ee = pe._fullLayout.activeselection.opacity);
                for (var Re = [], De = 1; De >= 0; De--) {
                  var Pe = Ce.append("path").attr(we).style("opacity", De ? 0.1 : Ee).call(ae.stroke, Le).call(ae.fill, xe).call(oe.dashLine, De ? "solid" : ke, De ? 4 + Qe : Qe);
                  if (he(Pe, pe, ve), Me) {
                    var $e = se(pe.layout, "selections", ve);
                    Pe.style({ cursor: "move" });
                    var qe = { element: Pe.node(), plotinfo: be, gd: pe, editHelpers: $e, isActiveSelection: !0 }, Xe = te(Se, pe);
                    ne(Xe, Pe, qe);
                  } else Pe.style("pointer-events", De ? "all" : "none");
                  Re[De] = Pe;
                }
                var Ve = Re[0];
                Re[1].node().addEventListener("click", function() {
                  return function(Ze, Ge) {
                    if (de(Ze)) {
                      var ze = +Ge.node().getAttribute("data-index");
                      if (ze >= 0) {
                        if (ze === Ze._fullLayout._activeSelectionIndex) return void me(Ze);
                        Ze._fullLayout._activeSelectionIndex = ze, Ze._fullLayout._deactivateSelection = me, ue(Ze);
                      }
                    }
                  }(pe, Ve);
                });
              }(pe._fullLayout._selectionLayer);
            }
            function he(pe, ye, ge) {
              var ve = ge.xref + ge.yref;
              oe.setClipUrl(pe, "clip" + ye._fullLayout._uid + ve, ye);
            }
            function me(pe) {
              de(pe) && pe._fullLayout._activeSelectionIndex >= 0 && (ee(pe), delete pe._fullLayout._activeSelectionIndex, ue(pe));
            }
            G.exports = { draw: ue, drawOne: fe, activateLastSelection: function(pe) {
              if (de(pe)) {
                var ye = pe._fullLayout.selections.length - 1;
                pe._fullLayout._activeSelectionIndex = ye, pe._fullLayout._deactivateSelection = me, ue(pe);
              }
            } };
          }, 52307: function(G, J, Z) {
            var te = Z(94850).T, ne = Z(93049).extendFlat;
            G.exports = { newselection: { mode: { valType: "enumerated", values: ["immediate", "gradual"], dflt: "immediate", editType: "none" }, line: { color: { valType: "color", editType: "none" }, width: { valType: "number", min: 1, dflt: 1, editType: "none" }, dash: ne({}, te, { dflt: "dot", editType: "none" }), editType: "none" }, editType: "none" }, activeselection: { fillcolor: { valType: "color", dflt: "rgba(0,0,0,0)", editType: "none" }, opacity: { valType: "number", min: 0, max: 1, dflt: 0.5, editType: "none" }, editType: "none" } };
          }, 43028: function(G) {
            G.exports = function(J, Z, te) {
              te("newselection.mode"), te("newselection.line.width") && (te("newselection.line.color"), te("newselection.line.dash")), te("activeselection.fillcolor"), te("activeselection.opacity");
            };
          }, 51817: function(G, J, Z) {
            var te = Z(70414).selectMode, ne = Z(78534).clearOutline, ee = Z(81055), ae = ee.readPaths, oe = ee.writePaths, se = ee.fixDatesForPaths;
            G.exports = function(ce, le) {
              if (ce.length) {
                var ue = ce[0][0];
                if (ue) {
                  var de = ue.getAttribute("d"), fe = le.gd, he = fe._fullLayout.newselection, me = le.plotinfo, pe = me.xaxis, ye = me.yaxis, ge = le.isActiveSelection, ve = le.dragmode, be = (fe.layout || {}).selections || [];
                  if (!te(ve) && ge !== void 0) {
                    var Ce = fe._fullLayout._activeSelectionIndex;
                    if (Ce < be.length) switch (fe._fullLayout.selections[Ce].type) {
                      case "rect":
                        ve = "select";
                        break;
                      case "path":
                        ve = "lasso";
                    }
                  }
                  var Se, we = ae(de, fe, me, ge), Ee = { xref: pe._id, yref: ye._id, opacity: he.opacity, line: { color: he.line.color, width: he.line.width, dash: he.line.dash } };
                  we.length === 1 && (Se = we[0]), Se && Se.length === 5 && ve === "select" ? (Ee.type = "rect", Ee.x0 = Se[0][1], Ee.y0 = Se[0][2], Ee.x1 = Se[2][1], Ee.y1 = Se[2][2]) : (Ee.type = "path", pe && ye && se(we, pe, ye), Ee.path = oe(we), Se = null), ne(fe);
                  for (var xe = le.editHelpers, Le = (xe || {}).modifyItem, Qe = [], ke = 0; ke < be.length; ke++) {
                    var Me = fe._fullLayout.selections[ke];
                    if (Me) {
                      if (Qe[ke] = Me._input, ge !== void 0 && ke === fe._fullLayout._activeSelectionIndex) {
                        var Re = Ee;
                        switch (Me.type) {
                          case "rect":
                            Le("x0", Re.x0), Le("x1", Re.x1), Le("y0", Re.y0), Le("y1", Re.y1);
                            break;
                          case "path":
                            Le("path", Re.path);
                        }
                      }
                    } else Qe[ke] = Me;
                  }
                  return ge === void 0 ? (Qe.push(Ee), Qe) : xe ? xe.getUpdateObj() : {};
                }
              }
            };
          }, 49801: function(G, J, Z) {
            var te = Z(34809).strTranslate;
            function ne(ee, ae) {
              switch (ee.type) {
                case "log":
                  return ee.p2d(ae);
                case "date":
                  return ee.p2r(ae, 0, ee.calendar);
                default:
                  return ee.p2r(ae);
              }
            }
            G.exports = { p2r: ne, r2p: function(ee, ae) {
              switch (ee.type) {
                case "log":
                  return ee.d2p(ae);
                case "date":
                  return ee.r2p(ae, 0, ee.calendar);
                default:
                  return ee.r2p(ae);
              }
            }, axValue: function(ee) {
              var ae = ee._id.charAt(0) === "y" ? 1 : 0;
              return function(oe) {
                return ne(ee, oe[ae]);
              };
            }, getTransform: function(ee) {
              return te(ee.xaxis._offset, ee.yaxis._offset);
            } };
          }, 44844: function(G, J, Z) {
            var te = Z(7028), ne = Z(88666);
            G.exports = { moduleType: "component", name: "selections", layoutAttributes: Z(4327), supplyLayoutDefaults: Z(2272), supplyDrawNewSelectionDefaults: Z(43028), includeBasePlot: Z(20706)("selections"), draw: te.draw, drawOne: te.drawOne, reselect: ne.reselect, prepSelect: ne.prepSelect, clearOutline: ne.clearOutline, clearSelectionsCache: ne.clearSelectionsCache, selectOnClick: ne.selectOnClick };
          }, 88666: function(G, J, Z) {
            var te = Z(11516), ne = Z(52773), ee = Z(33626), ae = Z(62203).dashStyle, oe = Z(78766), se = Z(32141), ce = Z(36040).makeEventData, le = Z(70414), ue = le.freeMode, de = le.rectMode, fe = le.drawMode, he = le.openMode, me = le.selectMode, pe = Z(49728), ye = Z(2956), ge = Z(561), ve = Z(78534).clearOutline, be = Z(81055), Ce = be.handleEllipse, Se = be.readPaths, we = Z(87562).newShapes, Ee = Z(51817), xe = Z(7028).activateLastSelection, Le = Z(34809), Qe = Le.sorterAsc, ke = Z(80899), Me = Z(64025), Re = Z(5975).getFromId, De = Z(34823), Pe = Z(71817).redrawReglTraces, $e = Z(78865), qe = $e.MINSELECT, Xe = ke.filter, Ve = ke.tester, Ze = Z(49801), Ge = Ze.p2r, ze = Ze.axValue, We = Ze.getTransform;
            function at(sn) {
              return sn.subplot !== void 0;
            }
            function ct(sn, hn, Qt, Ut, Wt, rn, vn) {
              var dn, Ht, jt, cn, wn, Rn, Pn, Ie, rt, ft = hn._hoverdata, pt = hn._fullLayout.clickmode.indexOf("event") > -1, wt = [];
              if (function(un) {
                return un && Array.isArray(un) && un[0].hoverOnBox !== !0;
              }(ft)) {
                it(sn, hn, rn);
                var Bt = function(un, yn) {
                  var _n, gn, bn = un[0], an = -1, Zt = [];
                  for (gn = 0; gn < yn.length; gn++) if (_n = yn[gn], bn.fullData._expandedIndex === _n.cd[0].trace._expandedIndex) {
                    if (bn.hoverOnBox === !0) break;
                    bn.pointNumber !== void 0 ? an = bn.pointNumber : bn.binNumber !== void 0 && (an = bn.binNumber, Zt = bn.pointNumbers);
                    break;
                  }
                  return { pointNumber: an, pointNumbers: Zt, searchInfo: _n };
                }(ft, dn = bt(hn, Qt, Ut, Wt));
                if (Bt.pointNumbers.length > 0 ? function(un, yn) {
                  var _n, gn, bn, an = [];
                  for (bn = 0; bn < un.length; bn++) (_n = un[bn]).cd[0].trace.selectedpoints && _n.cd[0].trace.selectedpoints.length > 0 && an.push(_n);
                  if (an.length === 1 && an[0] === yn.searchInfo && (gn = yn.searchInfo.cd[0].trace).selectedpoints.length === yn.pointNumbers.length) {
                    for (bn = 0; bn < yn.pointNumbers.length; bn++) if (gn.selectedpoints.indexOf(yn.pointNumbers[bn]) < 0) return !1;
                    return !0;
                  }
                  return !1;
                }(dn, Bt) : function(un) {
                  var yn, _n, gn = 0;
                  for (_n = 0; _n < un.length; _n++) if ((yn = un[_n].cd[0].trace).selectedpoints && (yn.selectedpoints.length > 1 || (gn += yn.selectedpoints.length) > 1))
                    return !1;
                  return gn === 1;
                }(dn) && (Rn = St(Bt))) {
                  for (vn && vn.remove(), rt = 0; rt < dn.length; rt++) (Ht = dn[rt])._module.selectPoints(Ht, !1);
                  Lt(hn, dn), st(rn), pt && Yt(hn);
                } else {
                  for (Pn = sn.shiftKey && (Rn !== void 0 ? Rn : St(Bt)), jt = /* @__PURE__ */ function(un, yn, _n) {
                    return { pointNumber: un, searchInfo: yn, subtract: !!_n };
                  }(Bt.pointNumber, Bt.searchInfo, Pn), cn = Je(rn.selectionDefs.concat([jt])), rt = 0; rt < dn.length; rt++) if (wn = vt(dn[rt]._module.selectPoints(dn[rt], cn), dn[rt]), wt.length) for (var At = 0; At < wn.length; At++) wt.push(wn[At]);
                  else wt = wn;
                  if (Lt(hn, dn, Ie = { points: wt }), jt && rn && rn.selectionDefs.push(jt), vn) {
                    var Dt = rn.mergedPolygons, Ot = he(rn.dragmode);
                    ge($t(Dt, Ot), vn, rn);
                  }
                  pt && Vt(hn, Ie);
                }
              }
            }
            function et(sn) {
              return "pointNumber" in sn && "searchInfo" in sn;
            }
            function dt(sn) {
              return { xmin: 0, xmax: 0, ymin: 0, ymax: 0, pts: [], contains: function(hn, Qt, Ut, Wt) {
                var rn = sn.searchInfo.cd[0].trace._expandedIndex;
                return Wt.cd[0].trace._expandedIndex === rn && Ut === sn.pointNumber;
              }, isRect: !1, degenerate: !1, subtract: !!sn.subtract };
            }
            function Je(sn) {
              if (sn.length) {
                for (var hn = [], Qt = et(sn[0]) ? 0 : sn[0][0][0], Ut = Qt, Wt = et(sn[0]) ? 0 : sn[0][0][1], rn = Wt, vn = 0; vn < sn.length; vn++) if (et(sn[vn])) hn.push(dt(sn[vn]));
                else {
                  var dn = Ve(sn[vn]);
                  dn.subtract = !!sn[vn].subtract, hn.push(dn), Qt = Math.min(Qt, dn.xmin), Ut = Math.max(Ut, dn.xmax), Wt = Math.min(Wt, dn.ymin), rn = Math.max(rn, dn.ymax);
                }
                return { xmin: Qt, xmax: Ut, ymin: Wt, ymax: rn, pts: [], contains: function(Ht, jt, cn, wn) {
                  for (var Rn = !1, Pn = 0; Pn < hn.length; Pn++) hn[Pn].contains(Ht, jt, cn, wn) && (Rn = !hn[Pn].subtract);
                  return Rn;
                }, isRect: !1, degenerate: !1 };
              }
            }
            function it(sn, hn, Qt) {
              var Ut = hn._fullLayout, Wt = Qt.plotinfo, rn = Qt.dragmode, vn = Ut._lastSelectedSubplot && Ut._lastSelectedSubplot === Wt.id, dn = (sn.shiftKey || sn.altKey) && !(fe(rn) && he(rn));
              vn && dn && Wt.selection && Wt.selection.selectionDefs && !Qt.selectionDefs ? (Qt.selectionDefs = Wt.selection.selectionDefs, Qt.mergedPolygons = Wt.selection.mergedPolygons) : dn && Wt.selection || st(Qt), vn || (ve(hn), Ut._lastSelectedSubplot = Wt.id);
            }
            function st(sn, hn) {
              var Qt = sn.dragmode, Ut = sn.plotinfo, Wt = sn.gd;
              (function(wn) {
                return wn._fullLayout._activeShapeIndex >= 0;
              })(Wt) && Wt._fullLayout._deactivateShape(Wt), function(wn) {
                return wn._fullLayout._activeSelectionIndex >= 0;
              }(Wt) && Wt._fullLayout._deactivateSelection(Wt);
              var rn = Wt._fullLayout._zoomlayer, vn = fe(Qt), dn = me(Qt);
              if (vn || dn) {
                var Ht, jt, cn = rn.selectAll(".select-outline-" + Ut.id);
                cn && Wt._fullLayout._outlining && (vn && (Ht = we(cn, sn)), Ht && ee.call("_guiRelayout", Wt, { shapes: Ht }), dn && !at(sn) && (jt = Ee(cn, sn)), jt && (Wt._fullLayout._noEmitSelectedAtStart = !0, ee.call("_guiRelayout", Wt, { selections: jt }).then(function() {
                  hn && xe(Wt);
                })), Wt._fullLayout._outlining = !1);
              }
              Ut.selection = {}, Ut.selection.selectionDefs = sn.selectionDefs = [], Ut.selection.mergedPolygons = sn.mergedPolygons = [];
            }
            function gt(sn) {
              return sn._id;
            }
            function bt(sn, hn, Qt, Ut) {
              if (!sn.calcdata) return [];
              var Wt, rn, vn, dn = [], Ht = hn.map(gt), jt = Qt.map(gt);
              for (vn = 0; vn < sn.calcdata.length; vn++) if ((rn = (Wt = sn.calcdata[vn])[0].trace).visible === !0 && rn._module && rn._module.selectPoints) if (!at({ subplot: Ut }) || rn.subplot !== Ut && rn.geo !== Ut) if (rn.type === "splom") {
                if (rn._xaxes[Ht[0]] && rn._yaxes[jt[0]]) {
                  var cn = yt(rn._module, Wt, hn[0], Qt[0]);
                  cn.scene = sn._fullLayout._splomScenes[rn.uid], dn.push(cn);
                }
              } else if (rn.type === "sankey") {
                var wn = yt(rn._module, Wt, hn[0], Qt[0]);
                dn.push(wn);
              } else {
                if (!(Ht.indexOf(rn.xaxis) !== -1 || rn._xA && rn._xA.overlaying) || !(jt.indexOf(rn.yaxis) !== -1 || rn._yA && rn._yA.overlaying)) continue;
                dn.push(yt(rn._module, Wt, Re(sn, rn.xaxis), Re(sn, rn.yaxis)));
              }
              else dn.push(yt(rn._module, Wt, hn[0], Qt[0]));
              return dn;
            }
            function yt(sn, hn, Qt, Ut) {
              return { _module: sn, cd: hn, xaxis: Qt, yaxis: Ut };
            }
            function St(sn) {
              var hn = sn.searchInfo.cd[0].trace, Qt = sn.pointNumber, Ut = sn.pointNumbers, Wt = Ut.length > 0 ? Ut[0] : Qt;
              return !!hn.selectedpoints && hn.selectedpoints.indexOf(Wt) > -1;
            }
            function Lt(sn, hn, Qt) {
              var Ut, Wt;
              for (Ut = 0; Ut < hn.length; Ut++) {
                var rn = hn[Ut].cd[0].trace._fullInput, vn = sn._fullLayout._tracePreGUI[rn.uid] || {};
                vn.selectedpoints === void 0 && (vn.selectedpoints = rn._input.selectedpoints || null);
              }
              if (Qt) {
                var dn = Qt.points || [];
                for (Ut = 0; Ut < hn.length; Ut++) (Wt = hn[Ut].cd[0].trace)._input.selectedpoints = Wt._fullInput.selectedpoints = [], Wt._fullInput !== Wt && (Wt.selectedpoints = []);
                for (var Ht = 0; Ht < dn.length; Ht++) {
                  var jt = dn[Ht], cn = jt.data, wn = jt.fullData, Rn = jt.pointIndex, Pn = jt.pointIndices;
                  Pn ? ([].push.apply(cn.selectedpoints, Pn), Wt._fullInput !== Wt && [].push.apply(wn.selectedpoints, Pn)) : (cn.selectedpoints.push(Rn), Wt._fullInput !== Wt && wn.selectedpoints.push(Rn));
                }
              } else for (Ut = 0; Ut < hn.length; Ut++) delete (Wt = hn[Ut].cd[0].trace).selectedpoints, delete Wt._input.selectedpoints, Wt._fullInput !== Wt && delete Wt._fullInput.selectedpoints;
              (function(Ie, rt) {
                for (var ft = !1, pt = 0; pt < rt.length; pt++) {
                  var wt = rt[pt], Bt = wt.cd;
                  ee.traceIs(Bt[0].trace, "regl") && (ft = !0);
                  var At = wt._module, Dt = At.styleOnSelect || At.style;
                  Dt && (Dt(Ie, Bt, Bt[0].node3), Bt[0].nodeRangePlot3 && Dt(Ie, Bt, Bt[0].nodeRangePlot3));
                }
                ft && (De(Ie), Pe(Ie));
              })(sn, hn);
            }
            function Rt(sn, hn, Qt) {
              for (var Ut = (Qt ? te.difference : te.union)({ regions: sn }, { regions: [hn] }).regions.reverse(), Wt = 0; Wt < Ut.length; Wt++) {
                var rn = Ut[Wt];
                rn.subtract = Tt(rn, Ut.slice(0, Wt));
              }
              return Ut;
            }
            function vt(sn, hn) {
              if (Array.isArray(sn)) for (var Qt = hn.cd, Ut = hn.cd[0].trace, Wt = 0; Wt < sn.length; Wt++) sn[Wt] = ce(sn[Wt], Ut, Qt);
              return sn;
            }
            function $t(sn, hn) {
              for (var Qt = [], Ut = 0; Ut < sn.length; Ut++) {
                Qt[Ut] = [];
                for (var Wt = 0; Wt < sn[Ut].length; Wt++) {
                  Qt[Ut][Wt] = [], Qt[Ut][Wt][0] = Wt ? "L" : "M";
                  for (var rn = 0; rn < sn[Ut][Wt].length; rn++) Qt[Ut][Wt].push(sn[Ut][Wt][rn]);
                }
                hn || Qt[Ut].push(["Z", Qt[Ut][0][1], Qt[Ut][0][2]]);
              }
              return Qt;
            }
            function Pt(sn, hn) {
              for (var Qt, Ut, Wt = [], rn = [], vn = 0; vn < hn.length; vn++) {
                var dn = hn[vn];
                Ut = dn._module.selectPoints(dn, sn), rn.push(Ut), Qt = vt(Ut, dn), Wt = Wt.concat(Qt);
              }
              return Wt;
            }
            function Xt(sn, hn, Qt, Ut, Wt) {
              var rn, vn, dn, Ht = !!Ut;
              Wt && (rn = Wt.plotinfo, vn = Wt.xaxes[0]._id, dn = Wt.yaxes[0]._id);
              var jt = [], cn = [], wn = Ft(sn), Rn = sn._fullLayout;
              if (rn) {
                var Pn = Rn._zoomlayer, Ie = Rn.dragmode, rt = fe(Ie), ft = me(Ie);
                if (rt || ft) {
                  var pt = Re(sn, vn, "x"), wt = Re(sn, dn, "y");
                  if (pt && wt) {
                    var Bt = Pn.selectAll(".select-outline-" + rn.id);
                    if (Bt && sn._fullLayout._outlining && Bt.length) {
                      for (var At = Bt[0][0].getAttribute("d"), Dt = Se(At, sn, rn), Ot = [], un = 0; un < Dt.length; un++) {
                        for (var yn = Dt[un], _n = [], gn = 0; gn < yn.length; gn++) _n.push([It(pt, yn[gn][1]), It(wt, yn[gn][2])]);
                        _n.xref = vn, _n.yref = dn, _n.subtract = Tt(_n, Ot), Ot.push(_n);
                      }
                      wn = wn.concat(Ot);
                    }
                  }
                }
              }
              var bn = vn && dn ? [vn + dn] : Rn._subplots.cartesian;
              (function(Br) {
                var Pr = Br.calcdata;
                if (Pr) for (var ei = 0; ei < Pr.length; ei++) {
                  var Ci = Pr[ei][0].trace, oa = Br._fullLayout._splomScenes;
                  if (oa) {
                    var ka = oa[Ci.uid];
                    ka && (ka.selectBatch = []);
                  }
                }
              })(sn);
              for (var an = {}, Zt = 0; Zt < bn.length; Zt++) {
                var Mn = bn[Zt], Ln = Mn.indexOf("y"), In = Mn.slice(0, Ln), kn = Mn.slice(Ln), er = vn && dn ? Qt : void 0;
                if (er = Gt(wn, In, kn, er)) {
                  var jn = Ut;
                  if (!Ht) {
                    var ur = Re(sn, In, "x"), Kn = Re(sn, kn, "y");
                    jn = bt(sn, [ur], [Kn], Mn);
                    for (var Jn = 0; Jn < jn.length; Jn++) {
                      var Lr = jn[Jn], mr = Lr.cd[0], Ar = mr.trace;
                      if (Lr._module.name === "scattergl" && !mr.t.xpx) {
                        var Yr = Ar.x, jr = Ar.y, Mr = Ar._length;
                        mr.t.xpx = [], mr.t.ypx = [];
                        for (var Wr = 0; Wr < Mr; Wr++) mr.t.xpx[Wr] = ur.c2p(Yr[Wr]), mr.t.ypx[Wr] = Kn.c2p(jr[Wr]);
                      }
                      Lr._module.name === "splom" && (an[Ar.uid] || (an[Ar.uid] = !0));
                    }
                  }
                  var Bi = Pt(er, jn);
                  jt = jt.concat(Bi), cn = cn.concat(jn);
                }
              }
              var gi = { points: jt };
              Lt(sn, cn, gi);
              var Zr = Rn.clickmode.indexOf("event") > -1 && hn;
              if (!rn && hn) {
                var fi = Ft(sn, !0);
                if (fi.length) {
                  var ri = fi[0].xref, bi = fi[0].yref;
                  if (ri && bi) {
                    var Zi = Ct(fi);
                    Kt([Re(sn, ri, "x"), Re(sn, bi, "y")])(gi, Zi);
                  }
                }
                sn._fullLayout._noEmitSelectedAtStart ? sn._fullLayout._noEmitSelectedAtStart = !1 : Zr && Vt(sn, gi), Rn._reselect = !1;
              }
              if (!rn && Rn._deselect) {
                var $i = Rn._deselect;
                (function(Br, Pr, ei) {
                  for (var Ci = 0; Ci < ei.length; Ci++) {
                    var oa = ei[Ci];
                    if (oa.xaxis && oa.xaxis._id === Br && oa.yaxis && oa.yaxis._id === Pr) return !0;
                  }
                  return !1;
                })(vn = $i.xref, dn = $i.yref, cn) || Nt(sn, vn, dn, Ut), Zr && (gi.points.length ? Vt(sn, gi) : Yt(sn)), Rn._deselect = !1;
              }
              return { eventData: gi, selectionTesters: Qt };
            }
            function Nt(sn, hn, Qt, Ut) {
              Ut = bt(sn, [Re(sn, hn, "x")], [Re(sn, Qt, "y")], hn + Qt);
              for (var Wt = 0; Wt < Ut.length; Wt++) {
                var rn = Ut[Wt];
                rn._module.selectPoints(rn, !1);
              }
              Lt(sn, Ut);
            }
            function Gt(sn, hn, Qt, Ut) {
              for (var Wt, rn = 0; rn < sn.length; rn++) {
                var vn = sn[rn];
                hn === vn.xref && Qt === vn.yref && (Wt ? Ut = Je(Wt = Rt(Wt, vn, !!vn.subtract)) : (Wt = [vn], Ut = Ve(vn)));
              }
              return Ut;
            }
            function Ft(sn, hn) {
              for (var Qt = [], Ut = sn._fullLayout, Wt = Ut.selections, rn = Wt.length, vn = 0; vn < rn; vn++) if (!hn || vn === Ut._activeSelectionIndex) {
                var dn = Wt[vn];
                if (dn) {
                  var Ht, jt, cn, wn, Rn, Pn = dn.xref, Ie = dn.yref, rt = Re(sn, Pn, "x"), ft = Re(sn, Ie, "y");
                  if (dn.type === "rect") {
                    Rn = [];
                    var pt = It(rt, dn.x0), wt = It(rt, dn.x1), Bt = It(ft, dn.y0), At = It(ft, dn.y1);
                    Rn = [[pt, Bt], [pt, At], [wt, At], [wt, Bt]], Ht = Math.min(pt, wt), jt = Math.max(pt, wt), cn = Math.min(Bt, At), wn = Math.max(Bt, At), Rn.xmin = Ht, Rn.xmax = jt, Rn.ymin = cn, Rn.ymax = wn, Rn.xref = Pn, Rn.yref = Ie, Rn.subtract = !1, Rn.isRect = !0, Qt.push(Rn);
                  } else if (dn.type === "path") for (var Dt = dn.path.split("Z"), Ot = [], un = 0; un < Dt.length; un++) {
                    var yn = Dt[un];
                    if (yn) {
                      yn += "Z";
                      var _n = pe.extractPathCoords(yn, ye.paramIsX, "raw"), gn = pe.extractPathCoords(yn, ye.paramIsY, "raw");
                      Ht = 1 / 0, jt = -1 / 0, cn = 1 / 0, wn = -1 / 0, Rn = [];
                      for (var bn = 0; bn < _n.length; bn++) {
                        var an = It(rt, _n[bn]), Zt = It(ft, gn[bn]);
                        Rn.push([an, Zt]), Ht = Math.min(an, Ht), jt = Math.max(an, jt), cn = Math.min(Zt, cn), wn = Math.max(Zt, wn);
                      }
                      Rn.xmin = Ht, Rn.xmax = jt, Rn.ymin = cn, Rn.ymax = wn, Rn.xref = Pn, Rn.yref = Ie, Rn.subtract = Tt(Rn, Ot), Ot.push(Rn), Qt.push(Rn);
                    }
                  }
                }
              }
              return Qt;
            }
            function Tt(sn, hn) {
              for (var Qt = !1, Ut = 0; Ut < hn.length; Ut++) for (var Wt = hn[Ut], rn = 0; rn < sn.length; rn++) if (ne(sn[rn], Wt)) {
                Qt = !Qt;
                break;
              }
              return Qt;
            }
            function It(sn, hn) {
              return sn.type === "date" && (hn = hn.replace("_", " ")), sn.type === "log" ? sn.c2p(hn) : sn.r2p(hn, null, sn.calendar);
            }
            function Ct(sn) {
              for (var hn = sn.length, Qt = [], Ut = 0; Ut < hn; Ut++) {
                var Wt = sn[Ut];
                Qt = (Qt = Qt.concat(Wt)).concat([Wt[0]]);
              }
              return (rn = Qt).isRect = rn.length === 5 && rn[0][0] === rn[4][0] && rn[0][1] === rn[4][1] && rn[0][0] === rn[1][0] && rn[2][0] === rn[3][0] && rn[0][1] === rn[3][1] && rn[1][1] === rn[2][1] || rn[0][1] === rn[1][1] && rn[2][1] === rn[3][1] && rn[0][0] === rn[3][0] && rn[1][0] === rn[2][0], rn.isRect && (rn.xmin = Math.min(rn[0][0], rn[2][0]), rn.xmax = Math.max(rn[0][0], rn[2][0]), rn.ymin = Math.min(rn[0][1], rn[2][1]), rn.ymax = Math.max(rn[0][1], rn[2][1])), rn;
              var rn;
            }
            function Kt(sn) {
              return function(hn, Qt) {
                for (var Ut, Wt, rn = 0; rn < sn.length; rn++) {
                  var vn = sn[rn], dn = vn._id, Ht = dn.charAt(0);
                  if (Qt.isRect) {
                    Ut || (Ut = {});
                    var jt = Qt[Ht + "min"], cn = Qt[Ht + "max"];
                    jt !== void 0 && cn !== void 0 && (Ut[dn] = [Ge(vn, jt), Ge(vn, cn)].sort(Qe));
                  } else Wt || (Wt = {}), Wt[dn] = Qt.map(ze(vn));
                }
                Ut && (hn.range = Ut), Wt && (hn.lassoPoints = Wt);
              };
            }
            function Vt(sn, hn) {
              hn && (hn.selections = (sn.layout || {}).selections || []), sn.emit("plotly_selected", hn);
            }
            function Yt(sn) {
              sn.emit("plotly_deselect", null);
            }
            G.exports = { reselect: Xt, prepSelect: function(sn, hn, Qt, Ut, Wt) {
              var rn = !at(Ut), vn = ue(Wt), dn = de(Wt), Ht = he(Wt), jt = fe(Wt), cn = me(Wt), wn = Wt === "drawcircle", Rn = Wt === "drawline" || wn, Pn = Ut.gd, Ie = Pn._fullLayout, rt = cn && Ie.newselection.mode === "immediate" && rn, ft = Ie._zoomlayer, pt = Ut.element.getBoundingClientRect(), wt = Ut.plotinfo, Bt = We(wt), At = hn - pt.left, Dt = Qt - pt.top;
              Ie._calcInverseTransform(Pn);
              var Ot = Le.apply3DTransform(Ie._invTransform)(At, Dt);
              At = Ot[0], Dt = Ot[1];
              var un, yn, _n, gn, bn, an, Zt, Mn = Ie._invScaleX, Ln = Ie._invScaleY, In = At, kn = Dt, er = "M" + At + "," + Dt, jn = Ut.xaxes[0], ur = Ut.yaxes[0], Kn = jn._length, Jn = ur._length, Lr = sn.altKey && !(fe(Wt) && Ht);
              it(sn, Pn, Ut), vn && (un = Xe([[At, Dt]], $e.BENDPX));
              var mr = ft.selectAll("path.select-outline-" + wt.id).data([1]), Ar = jt ? Ie.newshape : Ie.newselection;
              jt && (Ut.hasText = Ar.label.text || Ar.label.texttemplate);
              var Yr = jt && !Ht ? Ar.fillcolor : "rgba(0,0,0,0)", jr = Ar.line.color || (rn ? oe.contrast(Pn._fullLayout.plot_bgcolor) : "#7f7f7f");
              mr.enter().append("path").attr("class", "select-outline select-outline-" + wt.id).style({ opacity: jt ? Ar.opacity / 2 : 1, "stroke-dasharray": ae(Ar.line.dash, Ar.line.width), "stroke-width": Ar.line.width + "px", "shape-rendering": "crispEdges" }).call(oe.stroke, jr).call(oe.fill, Yr).attr("fill-rule", "evenodd").classed("cursor-move", !!jt).attr("transform", Bt).attr("d", er + "Z");
              var Mr = ft.append("path").attr("class", "zoombox-corners").style({ fill: oe.background, stroke: oe.defaultLine, "stroke-width": 1 }).attr("transform", Bt).attr("d", "M0,0Z");
              if (jt && Ut.hasText) {
                var Wr = ft.select(".label-temp");
                Wr.empty() && (Wr = ft.append("g").classed("label-temp", !0).classed("select-outline", !0).style({ opacity: 0.8 }));
              }
              var Bi = Ie._uid + $e.SELECTID, gi = [], Zr = bt(Pn, Ut.xaxes, Ut.yaxes, Ut.subplot);
              rt && !sn.shiftKey && (Ut._clearSubplotSelections = function() {
                if (rn) {
                  var ri = jn._id, bi = ur._id;
                  Nt(Pn, ri, bi, Zr);
                  for (var Zi = (Pn.layout || {}).selections || [], $i = [], Br = !1, Pr = 0; Pr < Zi.length; Pr++) {
                    var ei = Ie.selections[Pr];
                    ei.xref !== ri || ei.yref !== bi ? $i.push(Zi[Pr]) : Br = !0;
                  }
                  Br && (Pn._fullLayout._noEmitSelectedAtStart = !0, ee.call("_guiRelayout", Pn, { selections: $i }));
                }
              });
              var fi = function(ri) {
                return ri.plotinfo.fillRangeItems || Kt(ri.xaxes.concat(ri.yaxes));
              }(Ut);
              Ut.moveFn = function(ri, bi) {
                Ut._clearSubplotSelections && (Ut._clearSubplotSelections(), Ut._clearSubplotSelections = void 0), In = Math.max(0, Math.min(Kn, Mn * ri + At)), kn = Math.max(0, Math.min(Jn, Ln * bi + Dt));
                var Zi = Math.abs(In - At), $i = Math.abs(kn - Dt);
                if (dn) {
                  var Br, Pr, ei;
                  if (cn) {
                    var Ci = Ie.selectdirection;
                    switch (Br = Ci === "any" ? $i < Math.min(0.6 * Zi, qe) ? "h" : Zi < Math.min(0.6 * $i, qe) ? "v" : "d" : Ci) {
                      case "h":
                        Pr = wn ? Jn / 2 : 0, ei = Jn;
                        break;
                      case "v":
                        Pr = wn ? Kn / 2 : 0, ei = Kn;
                    }
                  }
                  if (jt) switch (Ie.newshape.drawdirection) {
                    case "vertical":
                      Br = "h", Pr = wn ? Jn / 2 : 0, ei = Jn;
                      break;
                    case "horizontal":
                      Br = "v", Pr = wn ? Kn / 2 : 0, ei = Kn;
                      break;
                    case "ortho":
                      Zi < $i ? (Br = "h", Pr = Dt, ei = kn) : (Br = "v", Pr = At, ei = In);
                      break;
                    default:
                      Br = "d";
                  }
                  Br === "h" ? ((gn = Rn ? Ce(wn, [In, Pr], [In, ei]) : [[At, Pr], [At, ei], [In, ei], [In, Pr]]).xmin = Rn ? In : Math.min(At, In), gn.xmax = Rn ? In : Math.max(At, In), gn.ymin = Math.min(Pr, ei), gn.ymax = Math.max(Pr, ei), Mr.attr("d", "M" + gn.xmin + "," + (Dt - qe) + "h-4v" + 2 * qe + "h4ZM" + (gn.xmax - 1) + "," + (Dt - qe) + "h4v" + 2 * qe + "h-4Z")) : Br === "v" ? ((gn = Rn ? Ce(wn, [Pr, kn], [ei, kn]) : [[Pr, Dt], [Pr, kn], [ei, kn], [ei, Dt]]).xmin = Math.min(Pr, ei), gn.xmax = Math.max(Pr, ei), gn.ymin = Rn ? kn : Math.min(Dt, kn), gn.ymax = Rn ? kn : Math.max(Dt, kn), Mr.attr("d", "M" + (At - qe) + "," + gn.ymin + "v-4h" + 2 * qe + "v4ZM" + (At - qe) + "," + (gn.ymax - 1) + "v4h" + 2 * qe + "v-4Z")) : Br === "d" && ((gn = Rn ? Ce(wn, [At, Dt], [In, kn]) : [[At, Dt], [At, kn], [In, kn], [In, Dt]]).xmin = Math.min(At, In), gn.xmax = Math.max(At, In), gn.ymin = Math.min(Dt, kn), gn.ymax = Math.max(Dt, kn), Mr.attr("d", "M0,0Z"));
                } else vn && (un.addPt([In, kn]), gn = un.filtered);
                if (Ut.selectionDefs && Ut.selectionDefs.length ? (_n = Rt(Ut.mergedPolygons, gn, Lr), gn.subtract = Lr, yn = Je(Ut.selectionDefs.concat([gn]))) : (_n = [gn], yn = Ve(gn)), ge($t(_n, Ht), mr, Ut), cn) {
                  var oa, ka = Xt(Pn, !1), aa = ka.eventData ? ka.eventData.points.slice() : [];
                  ka = Xt(Pn, !1, yn, Zr, Ut), yn = ka.selectionTesters, Zt = ka.eventData, oa = un ? un.filtered : Ct(_n), Me.throttle(Bi, $e.SELECTDELAY, function() {
                    for (var hi = (gi = Pt(yn, Zr)).slice(), Gr = 0; Gr < aa.length; Gr++) {
                      for (var Ca = aa[Gr], Sr = !1, Dr = 0; Dr < hi.length; Dr++) if (hi[Dr].curveNumber === Ca.curveNumber && hi[Dr].pointNumber === Ca.pointNumber) {
                        Sr = !0;
                        break;
                      }
                      Sr || hi.push(Ca);
                    }
                    hi.length && (Zt || (Zt = {}), Zt.points = hi), fi(Zt, oa), function(si, zr) {
                      si.emit("plotly_selecting", zr);
                    }(Pn, Zt);
                  });
                }
              }, Ut.clickFn = function(ri, bi) {
                if (Mr.remove(), Pn._fullLayout._activeShapeIndex >= 0) Pn._fullLayout._deactivateShape(Pn);
                else if (!jt) {
                  var Zi = Ie.clickmode;
                  Me.done(Bi).then(function() {
                    if (Me.clear(Bi), ri === 2) {
                      for (mr.remove(), bn = 0; bn < Zr.length; bn++) (an = Zr[bn])._module.selectPoints(an, !1);
                      if (Lt(Pn, Zr), st(Ut), Yt(Pn), Zr.length) {
                        var $i = Zr[0].xaxis, Br = Zr[0].yaxis;
                        if ($i && Br) {
                          for (var Pr = [], ei = Pn._fullLayout.selections, Ci = 0; Ci < ei.length; Ci++) {
                            var oa = ei[Ci];
                            oa && (oa.xref === $i._id && oa.yref === Br._id || Pr.push(oa));
                          }
                          Pr.length < ei.length && (Pn._fullLayout._noEmitSelectedAtStart = !0, ee.call("_guiRelayout", Pn, { selections: Pr }));
                        }
                      }
                    } else Zi.indexOf("select") > -1 && ct(bi, Pn, Ut.xaxes, Ut.yaxes, Ut.subplot, Ut, mr), Zi === "event" && Vt(Pn, void 0);
                    se.click(Pn, bi, wt.id);
                  }).catch(Le.error);
                }
              }, Ut.doneFn = function() {
                Mr.remove(), Me.done(Bi).then(function() {
                  Me.clear(Bi), !rt && gn && Ut.selectionDefs && (gn.subtract = Lr, Ut.selectionDefs.push(gn), Ut.mergedPolygons.length = 0, [].push.apply(Ut.mergedPolygons, _n)), (rt || jt) && st(Ut, rt), Ut.doneFnCompleted && Ut.doneFnCompleted(gi), cn && Vt(Pn, Zt);
                }).catch(Le.error);
              };
            }, clearOutline: ve, clearSelectionsCache: st, selectOnClick: ct };
          }, 43144: function(G, J, Z) {
            var te = Z(50222), ne = Z(80337), ee = Z(36640).line, ae = Z(94850).T, oe = Z(93049).extendFlat, se = Z(78032).templatedArray, ce = (Z(35081), Z(9829)), le = Z(3208).LF, ue = Z(41235);
            G.exports = se("shape", { visible: oe({}, ce.visible, { editType: "calc+arraydraw" }), showlegend: { valType: "boolean", dflt: !1, editType: "calc+arraydraw" }, legend: oe({}, ce.legend, { editType: "calc+arraydraw" }), legendgroup: oe({}, ce.legendgroup, { editType: "calc+arraydraw" }), legendgrouptitle: { text: oe({}, ce.legendgrouptitle.text, { editType: "calc+arraydraw" }), font: ne({ editType: "calc+arraydraw" }), editType: "calc+arraydraw" }, legendrank: oe({}, ce.legendrank, { editType: "calc+arraydraw" }), legendwidth: oe({}, ce.legendwidth, { editType: "calc+arraydraw" }), type: { valType: "enumerated", values: ["circle", "rect", "path", "line"], editType: "calc+arraydraw" }, layer: { valType: "enumerated", values: ["below", "above", "between"], dflt: "above", editType: "arraydraw" }, xref: oe({}, te.xref, {}), xsizemode: { valType: "enumerated", values: ["scaled", "pixel"], dflt: "scaled", editType: "calc+arraydraw" }, xanchor: { valType: "any", editType: "calc+arraydraw" }, x0: { valType: "any", editType: "calc+arraydraw" }, x1: { valType: "any", editType: "calc+arraydraw" }, x0shift: { valType: "number", dflt: 0, min: -1, max: 1, editType: "calc" }, x1shift: { valType: "number", dflt: 0, min: -1, max: 1, editType: "calc" }, yref: oe({}, te.yref, {}), ysizemode: { valType: "enumerated", values: ["scaled", "pixel"], dflt: "scaled", editType: "calc+arraydraw" }, yanchor: { valType: "any", editType: "calc+arraydraw" }, y0: { valType: "any", editType: "calc+arraydraw" }, y1: { valType: "any", editType: "calc+arraydraw" }, y0shift: { valType: "number", dflt: 0, min: -1, max: 1, editType: "calc" }, y1shift: { valType: "number", dflt: 0, min: -1, max: 1, editType: "calc" }, path: { valType: "string", editType: "calc+arraydraw" }, opacity: { valType: "number", min: 0, max: 1, dflt: 1, editType: "arraydraw" }, line: { color: oe({}, ee.color, { editType: "arraydraw" }), width: oe({}, ee.width, { editType: "calc+arraydraw" }), dash: oe({}, ae, { editType: "arraydraw" }), editType: "calc+arraydraw" }, fillcolor: { valType: "color", dflt: "rgba(0,0,0,0)", editType: "arraydraw" }, fillrule: { valType: "enumerated", values: ["evenodd", "nonzero"], dflt: "evenodd", editType: "arraydraw" }, editable: { valType: "boolean", dflt: !1, editType: "calc+arraydraw" }, label: { text: { valType: "string", dflt: "", editType: "arraydraw" }, texttemplate: le({}, { keys: Object.keys(ue) }), font: ne({ editType: "calc+arraydraw", colorEditType: "arraydraw" }), textposition: { valType: "enumerated", values: ["top left", "top center", "top right", "middle left", "middle center", "middle right", "bottom left", "bottom center", "bottom right", "start", "middle", "end"], editType: "arraydraw" }, textangle: { valType: "angle", dflt: "auto", editType: "calc+arraydraw" }, xanchor: { valType: "enumerated", values: ["auto", "left", "center", "right"], dflt: "auto", editType: "calc+arraydraw" }, yanchor: { valType: "enumerated", values: ["top", "middle", "bottom"], editType: "calc+arraydraw" }, padding: { valType: "number", dflt: 3, min: 0, editType: "arraydraw" }, editType: "arraydraw" }, editType: "arraydraw" });
          }, 44959: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(2956), ae = Z(49728);
            function oe(ue) {
              return ce(ue.line.width, ue.xsizemode, ue.x0, ue.x1, ue.path, !1);
            }
            function se(ue) {
              return ce(ue.line.width, ue.ysizemode, ue.y0, ue.y1, ue.path, !0);
            }
            function ce(ue, de, fe, he, me, pe) {
              var ye = ue / 2, ge = pe;
              if (de === "pixel") {
                var ve = me ? ae.extractPathCoords(me, pe ? ee.paramIsY : ee.paramIsX) : [fe, he], be = te.aggNums(Math.max, null, ve), Ce = te.aggNums(Math.min, null, ve), Se = Ce < 0 ? Math.abs(Ce) + ye : ye, we = be > 0 ? be + ye : ye;
                return { ppad: ye, ppadplus: ge ? Se : we, ppadminus: ge ? we : Se };
              }
              return { ppad: ye };
            }
            function le(ue, de, fe) {
              var he, me, pe = ue._id.charAt(0) === "x" ? "x" : "y", ye = ue.type === "category" || ue.type === "multicategory", ge = 0, ve = 0, be = ye ? ue.r2c : ue.d2c;
              if (de[pe + "sizemode"] === "scaled" ? (he = de[pe + "0"], me = de[pe + "1"], ye && (ge = de[pe + "0shift"], ve = de[pe + "1shift"])) : (he = de[pe + "anchor"], me = de[pe + "anchor"]), he !== void 0) return [be(he) + ge, be(me) + ve];
              if (de.path) {
                var Ce, Se, we, Ee, xe = 1 / 0, Le = -1 / 0, Qe = de.path.match(ee.segmentRE);
                for (ue.type === "date" && (be = ae.decodeDate(be)), Ce = 0; Ce < Qe.length; Ce++) (Se = fe[Qe[Ce].charAt(0)].drawn) !== void 0 && (!(we = Qe[Ce].substr(1).match(ee.paramRE)) || we.length < Se || ((Ee = be(we[Se])) < xe && (xe = Ee), Ee > Le && (Le = Ee)));
                return Le >= xe ? [xe, Le] : void 0;
              }
            }
            G.exports = function(ue) {
              var de = ue._fullLayout, fe = te.filterVisible(de.shapes);
              if (fe.length && ue._fullData.length) for (var he = 0; he < fe.length; he++) {
                var me, pe, ye = fe[he];
                ye._extremes = {};
                var ge = ne.getRefType(ye.xref), ve = ne.getRefType(ye.yref);
                ye.xref !== "paper" && ge !== "domain" && (pe = le(me = ne.getFromId(ue, ye.xref), ye, ee.paramIsX)) && (ye._extremes[me._id] = ne.findExtremes(me, pe, oe(ye))), ye.yref !== "paper" && ve !== "domain" && (pe = le(me = ne.getFromId(ue, ye.yref), ye, ee.paramIsY)) && (ye._extremes[me._id] = ne.findExtremes(me, pe, se(ye)));
              }
            };
          }, 2956: function(G) {
            G.exports = { segmentRE: /[MLHVQCTSZ][^MLHVQCTSZ]*/g, paramRE: /[^\s,]+/g, paramIsX: { M: { 0: !0, drawn: 0 }, L: { 0: !0, drawn: 0 }, H: { 0: !0, drawn: 0 }, V: {}, Q: { 0: !0, 2: !0, drawn: 2 }, C: { 0: !0, 2: !0, 4: !0, drawn: 4 }, T: { 0: !0, drawn: 0 }, S: { 0: !0, 2: !0, drawn: 2 }, Z: {} }, paramIsY: { M: { 1: !0, drawn: 1 }, L: { 1: !0, drawn: 1 }, H: {}, V: { 0: !0, drawn: 0 }, Q: { 1: !0, 3: !0, drawn: 3 }, C: { 1: !0, 3: !0, 5: !0, drawn: 5 }, T: { 1: !0, drawn: 1 }, S: { 1: !0, 3: !0, drawn: 5 }, Z: {} }, numParams: { M: 2, L: 2, H: 1, V: 1, Q: 4, C: 6, T: 2, S: 4, Z: 0 } };
          }, 74367: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(59008), ae = Z(43144), oe = Z(49728);
            function se(ce, le, ue) {
              function de(Ze, Ge) {
                return te.coerce(ce, le, ae, Ze, Ge);
              }
              if (le._isShape = !0, de("visible")) {
                de("showlegend") && (de("legend"), de("legendwidth"), de("legendgroup"), de("legendgrouptitle.text"), te.coerceFont(de, "legendgrouptitle.font"), de("legendrank"));
                var fe = de("path"), he = de("type", fe ? "path" : "rect"), me = he !== "path";
                me && delete le.path, de("editable"), de("layer"), de("opacity"), de("fillcolor"), de("fillrule"), de("line.width") && (de("line.color"), de("line.dash"));
                for (var pe = de("xsizemode"), ye = de("ysizemode"), ge = ["x", "y"], ve = 0; ve < 2; ve++) {
                  var be, Ce, Se, we = ge[ve], Ee = we + "anchor", xe = we === "x" ? pe : ye, Le = { _fullLayout: ue }, Qe = ne.coerceRef(ce, le, Le, we, void 0, "paper");
                  if (ne.getRefType(Qe) === "range" ? ((be = ne.getFromId(Le, Qe))._shapeIndices.push(le._index), Se = oe.rangeToShapePosition(be), Ce = oe.shapePositionToRange(be), be.type !== "category" && be.type !== "multicategory" || (de(we + "0shift"), de(we + "1shift"))) : Ce = Se = te.identity, me) {
                    var ke = we + "0", Me = we + "1", Re = ce[ke], De = ce[Me];
                    ce[ke] = Ce(ce[ke], !0), ce[Me] = Ce(ce[Me], !0), xe === "pixel" ? (de(ke, 0), de(Me, 10)) : (ne.coercePosition(le, Le, de, Qe, ke, 0.25), ne.coercePosition(le, Le, de, Qe, Me, 0.75)), le[ke] = Se(le[ke]), le[Me] = Se(le[Me]), ce[ke] = Re, ce[Me] = De;
                  }
                  if (xe === "pixel") {
                    var Pe = ce[Ee];
                    ce[Ee] = Ce(ce[Ee], !0), ne.coercePosition(le, Le, de, Qe, Ee, 0.25), le[Ee] = Se(le[Ee]), ce[Ee] = Pe;
                  }
                }
                me && te.noneOrAll(ce, le, ["x0", "x1", "y0", "y1"]);
                var $e, qe, Xe = he === "line";
                if (me && ($e = de("label.texttemplate")), $e || (qe = de("label.text")), qe || $e) {
                  de("label.textangle");
                  var Ve = de("label.textposition", Xe ? "middle" : "middle center");
                  de("label.xanchor"), de("label.yanchor", function(Ze, Ge) {
                    return Ze ? "bottom" : Ge.indexOf("top") !== -1 ? "top" : Ge.indexOf("bottom") !== -1 ? "bottom" : "middle";
                  }(Xe, Ve)), de("label.padding"), te.coerceFont(de, "label.font", ue.font);
                }
              }
            }
            G.exports = function(ce, le) {
              ee(ce, le, { name: "shapes", handleItemDefaults: se });
            };
          }, 44433: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(30635), ae = Z(62203), oe = Z(81055).readPaths, se = Z(49728), ce = se.getPathString, le = Z(41235), ue = Z(4530).FROM_TL;
            G.exports = function(de, fe, he, me) {
              if (me.selectAll(".shape-label").remove(), he.label.text || he.label.texttemplate) {
                var pe;
                if (he.label.texttemplate) {
                  var ye = {};
                  if (he.type !== "path") {
                    var ge = ne.getFromId(de, he.xref), ve = ne.getFromId(de, he.yref);
                    for (var be in le) {
                      var Ce = le[be](he, ge, ve);
                      Ce !== void 0 && (ye[be] = Ce);
                    }
                  }
                  pe = te.texttemplateStringForShapes(he.label.texttemplate, {}, de._fullLayout._d3locale, ye);
                } else pe = he.label.text;
                var Se, we, Ee, xe, Le = { "data-index": fe }, Qe = he.label.font, ke = me.append("g").attr(Le).classed("shape-label", !0).append("text").attr({ "data-notex": 1 }).classed("shape-label-text", !0).text(pe);
                if (he.path) {
                  var Me = ce(de, he), Re = oe(Me, de);
                  Se = 1 / 0, Ee = 1 / 0, we = -1 / 0, xe = -1 / 0;
                  for (var De = 0; De < Re.length; De++) for (var Pe = 0; Pe < Re[De].length; Pe++) for (var $e = Re[De][Pe], qe = 1; qe < $e.length; qe += 2) {
                    var Xe = $e[qe], Ve = $e[qe + 1];
                    Se = Math.min(Se, Xe), we = Math.max(we, Xe), Ee = Math.min(Ee, Ve), xe = Math.max(xe, Ve);
                  }
                } else {
                  var Ze = ne.getFromId(de, he.xref), Ge = he.x0shift, ze = he.x1shift, We = ne.getRefType(he.xref), at = ne.getFromId(de, he.yref), ct = he.y0shift, et = he.y1shift, dt = ne.getRefType(he.yref), Je = function(Lt, Rt) {
                    return se.getDataToPixel(de, Ze, Rt, !1, We)(Lt);
                  }, it = function(Lt, Rt) {
                    return se.getDataToPixel(de, at, Rt, !0, dt)(Lt);
                  };
                  Se = Je(he.x0, Ge), we = Je(he.x1, ze), Ee = it(he.y0, ct), xe = it(he.y1, et);
                }
                var st = he.label.textangle;
                st === "auto" && (st = he.type === "line" ? function(Lt, Rt, vt, $t) {
                  var Pt, Xt;
                  return Xt = Math.abs(vt - Lt), Pt = vt >= Lt ? Rt - $t : $t - Rt, -180 / Math.PI * Math.atan2(Pt, Xt);
                }(Se, Ee, we, xe) : 0), ke.call(function(Lt) {
                  return Lt.call(ae.font, Qe).attr({}), ee.convertToTspans(Lt, de), Lt;
                });
                var gt = function(Lt, Rt, vt, $t, Pt, Xt, Nt) {
                  var Gt, Ft, Tt, It, Ct = Pt.label.textposition, Kt = Pt.label.textangle, Vt = Pt.label.padding, Yt = Pt.type, sn = Math.PI / 180 * Xt, hn = Math.sin(sn), Qt = Math.cos(sn), Ut = Pt.label.xanchor, Wt = Pt.label.yanchor;
                  if (Yt === "line") {
                    Ct === "start" ? (Gt = Lt, Ft = Rt) : Ct === "end" ? (Gt = vt, Ft = $t) : (Gt = (Lt + vt) / 2, Ft = (Rt + $t) / 2), Ut === "auto" && (Ut = Ct === "start" ? Kt === "auto" ? vt > Lt ? "left" : vt < Lt ? "right" : "center" : vt > Lt ? "right" : vt < Lt ? "left" : "center" : Ct === "end" ? Kt === "auto" ? vt > Lt ? "right" : vt < Lt ? "left" : "center" : vt > Lt ? "left" : vt < Lt ? "right" : "center" : "center");
                    var rn = { bottom: -1, middle: 0, top: 1 };
                    if (Kt === "auto") {
                      var vn = rn[Wt];
                      Tt = -Vt * hn * vn, It = Vt * Qt * vn;
                    } else Tt = Vt * { left: 1, center: 0, right: -1 }[Ut], It = Vt * rn[Wt];
                    Gt += Tt, Ft += It;
                  } else Tt = Vt + 3, Ct.indexOf("right") !== -1 ? (Gt = Math.max(Lt, vt) - Tt, Ut === "auto" && (Ut = "right")) : Ct.indexOf("left") !== -1 ? (Gt = Math.min(Lt, vt) + Tt, Ut === "auto" && (Ut = "left")) : (Gt = (Lt + vt) / 2, Ut === "auto" && (Ut = "center")), Ft = Ct.indexOf("top") !== -1 ? Math.min(Rt, $t) : Ct.indexOf("bottom") !== -1 ? Math.max(Rt, $t) : (Rt + $t) / 2, It = Vt, Wt === "bottom" ? Ft -= It : Wt === "top" && (Ft += It);
                  var dn = ue[Wt], Ht = Pt.label.font.size, jt = Nt.height;
                  return { textx: Gt + (jt * dn - Ht) * hn, texty: Ft + -(jt * dn - Ht) * Qt, xanchor: Ut };
                }(Se, Ee, we, xe, he, st, ae.bBox(ke.node())), bt = gt.textx, yt = gt.texty, St = gt.xanchor;
                ke.attr({ "text-anchor": { left: "start", center: "middle", right: "end" }[St], y: yt, x: bt, transform: "rotate(" + st + "," + bt + "," + yt + ")" }).call(ee.positionText, bt, yt);
              }
            };
          }, 561: function(G, J, Z) {
            var te = Z(34809).strTranslate, ne = Z(14751), ee = Z(70414), ae = ee.drawMode, oe = ee.selectMode, se = Z(33626), ce = Z(78766), le = Z(93391), ue = le.i000, de = le.i090, fe = le.i180, he = le.i270, me = Z(78534).clearOutlineControllers, pe = Z(81055), ye = pe.pointsOnRectangle, ge = pe.pointsOnEllipse, ve = pe.writePaths, be = Z(87562).newShapes, Ce = Z(87562).createShapeObj, Se = Z(51817), we = Z(44433);
            function Ee(xe, Le) {
              var Qe, ke, Me, Re = xe[Le][1], De = xe[Le][2], Pe = xe.length;
              return ke = xe[Qe = (Le + 1) % Pe][1], Me = xe[Qe][2], ke === Re && Me === De && (ke = xe[Qe = (Le + 2) % Pe][1], Me = xe[Qe][2]), [Qe, ke, Me];
            }
            G.exports = function xe(Le, Qe, ke, Me) {
              Me || (Me = 0);
              var Re = ke.gd;
              function De() {
                xe(Le, Qe, ke, Me++), (ge(Le[0]) || ke.hasText) && Pe({ redrawing: !0 });
              }
              function Pe(Lt) {
                var Rt = {};
                ke.isActiveShape !== void 0 && (ke.isActiveShape = !1, Rt = be(Qe, ke)), ke.isActiveSelection !== void 0 && (ke.isActiveSelection = !1, Rt = Se(Qe, ke), Re._fullLayout._reselect = !0), Object.keys(Rt).length && se.call((Lt || {}).redrawing ? "relayout" : "_guiRelayout", Re, Rt);
              }
              var $e, qe, Xe, Ve, Ze, Ge = Re._fullLayout._zoomlayer, ze = ke.dragmode, We = ae(ze), at = oe(ze);
              if ((We || at) && (Re._fullLayout._outlining = !0), me(Re), Qe.attr("d", ve(Le)), Me || !ke.isActiveShape && !ke.isActiveSelection || (Ze = function(Lt, Rt) {
                for (var vt = 0; vt < Rt.length; vt++) {
                  var $t = Rt[vt];
                  Lt[vt] = [];
                  for (var Pt = 0; Pt < $t.length; Pt++) {
                    Lt[vt][Pt] = [];
                    for (var Xt = 0; Xt < $t[Pt].length; Xt++) Lt[vt][Pt][Xt] = $t[Pt][Xt];
                  }
                }
                return Lt;
              }([], Le), function(Lt) {
                $e = [];
                for (var Rt = 0; Rt < Le.length; Rt++) {
                  var vt = Le[Rt], $t = ye(vt), Pt = !$t && ge(vt);
                  $e[Rt] = [];
                  for (var Xt = vt.length, Nt = 0; Nt < Xt; Nt++) if (vt[Nt][0] !== "Z" && (!Pt || Nt === ue || Nt === de || Nt === fe || Nt === he)) {
                    var Gt, Ft = $t && ke.isActiveSelection;
                    Ft && (Gt = Ee(vt, Nt));
                    var Tt = vt[Nt][1], It = vt[Nt][2], Ct = Lt.append(Ft ? "rect" : "circle").attr("data-i", Rt).attr("data-j", Nt).style({ fill: ce.background, stroke: ce.defaultLine, "stroke-width": 1, "shape-rendering": "crispEdges" });
                    if (Ft) {
                      var Kt = Gt[1] - Tt, Vt = Gt[2] - It, Yt = Vt ? 5 : Math.max(Math.min(25, Math.abs(Kt) - 5), 5), sn = Kt ? 5 : Math.max(Math.min(25, Math.abs(Vt) - 5), 5);
                      Ct.classed(Vt ? "cursor-ew-resize" : "cursor-ns-resize", !0).attr("width", Yt).attr("height", sn).attr("x", Tt - Yt / 2).attr("y", It - sn / 2).attr("transform", te(Kt / 2, Vt / 2));
                    } else Ct.classed("cursor-grab", !0).attr("r", 5).attr("cx", Tt).attr("cy", It);
                    $e[Rt][Nt] = { element: Ct.node(), gd: Re, prepFn: dt, doneFn: it, clickFn: st }, ne.init($e[Rt][Nt]);
                  }
                }
              }(Ge.append("g").attr("class", "outline-controllers")), function() {
                qe = [], Le.length && (qe[0] = { element: Qe[0][0], gd: Re, prepFn: bt, doneFn: yt, clickFn: St }, ne.init(qe[0]));
              }()), We && ke.hasText) {
                var ct = Ge.select(".label-temp"), et = Ce(Qe, ke, ke.dragmode);
                we(Re, "label-temp", et, ct);
              }
              function dt(Lt) {
                Xe = +Lt.srcElement.getAttribute("data-i"), Ve = +Lt.srcElement.getAttribute("data-j"), $e[Xe][Ve].moveFn = Je;
              }
              function Je(Lt, Rt) {
                if (Le.length) {
                  var vt = Ze[Xe][Ve][1], $t = Ze[Xe][Ve][2], Pt = Le[Xe], Xt = Pt.length;
                  if (ye(Pt)) {
                    var Nt = Lt, Gt = Rt;
                    ke.isActiveSelection && (Ee(Pt, Ve)[1] === Pt[Ve][1] ? Gt = 0 : Nt = 0);
                    for (var Ft = 0; Ft < Xt; Ft++) if (Ft !== Ve) {
                      var Tt = Pt[Ft];
                      Tt[1] === Pt[Ve][1] && (Tt[1] = vt + Nt), Tt[2] === Pt[Ve][2] && (Tt[2] = $t + Gt);
                    }
                    if (Pt[Ve][1] = vt + Nt, Pt[Ve][2] = $t + Gt, !ye(Pt)) for (var It = 0; It < Xt; It++) for (var Ct = 0; Ct < Pt[It].length; Ct++) Pt[It][Ct] = Ze[Xe][It][Ct];
                  } else Pt[Ve][1] = vt + Lt, Pt[Ve][2] = $t + Rt;
                  De();
                }
              }
              function it() {
                Pe();
              }
              function st(Lt, Rt) {
                if (Lt === 2) {
                  Xe = +Rt.srcElement.getAttribute("data-i"), Ve = +Rt.srcElement.getAttribute("data-j");
                  var vt = Le[Xe];
                  ye(vt) || ge(vt) || function() {
                    if (Le.length && Le[Xe] && Le[Xe].length) {
                      for (var $t = [], Pt = 0; Pt < Le[Xe].length; Pt++) Pt !== Ve && $t.push(Le[Xe][Pt]);
                      $t.length > 1 && ($t.length !== 2 || $t[1][0] !== "Z") && (Ve === 0 && ($t[0][0] = "M"), Le[Xe] = $t, De(), Pe());
                    }
                  }();
                }
              }
              function gt(Lt, Rt) {
                (function(vt, $t) {
                  if (Le.length) for (var Pt = 0; Pt < Le.length; Pt++) for (var Xt = 0; Xt < Le[Pt].length; Xt++) for (var Nt = 0; Nt + 2 < Le[Pt][Xt].length; Nt += 2) Le[Pt][Xt][Nt + 1] = Ze[Pt][Xt][Nt + 1] + vt, Le[Pt][Xt][Nt + 2] = Ze[Pt][Xt][Nt + 2] + $t;
                })(Lt, Rt), De();
              }
              function bt(Lt) {
                (Xe = +Lt.srcElement.getAttribute("data-i")) || (Xe = 0), qe[Xe].moveFn = gt;
              }
              function yt() {
                Pe();
              }
              function St(Lt) {
                Lt === 2 && function(Rt) {
                  if (oe(Rt._fullLayout.dragmode)) {
                    me(Rt);
                    var vt = Rt._fullLayout._activeSelectionIndex, $t = (Rt.layout || {}).selections || [];
                    if (vt < $t.length) {
                      for (var Pt = [], Xt = 0; Xt < $t.length; Xt++) Xt !== vt && Pt.push($t[Xt]);
                      delete Rt._fullLayout._activeSelectionIndex;
                      var Nt = Rt._fullLayout.selections[vt];
                      Rt._fullLayout._deselect = { xref: Nt.xref, yref: Nt.yref }, se.call("_guiRelayout", Rt, { selections: Pt });
                    }
                  }
                }(Re);
              }
            };
          }, 28231: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(34809), ae = Z(29714), oe = Z(81055).readPaths, se = Z(561), ce = Z(44433), le = Z(78534).clearOutlineControllers, ue = Z(78766), de = Z(62203), fe = Z(78032).arrayEditor, he = Z(14751), me = Z(27983), pe = Z(2956), ye = Z(49728), ge = ye.getPathString;
            function ve(Le) {
              var Qe = Le._fullLayout;
              for (var ke in Qe._shapeUpperLayer.selectAll("path").remove(), Qe._shapeLowerLayer.selectAll("path").remove(), Qe._shapeUpperLayer.selectAll("text").remove(), Qe._shapeLowerLayer.selectAll("text").remove(), Qe._plots) {
                var Me = Qe._plots[ke].shapelayer;
                Me && (Me.selectAll("path").remove(), Me.selectAll("text").remove());
              }
              for (var Re = 0; Re < Qe.shapes.length; Re++) Qe.shapes[Re].visible === !0 && Se(Le, Re);
            }
            function be(Le) {
              return !!Le._fullLayout._outlining;
            }
            function Ce(Le) {
              return !Le._context.edits.shapePosition;
            }
            function Se(Le, Qe) {
              Le._fullLayout._paperdiv.selectAll('.shapelayer [data-index="' + Qe + '"]').remove();
              var ke = ye.makeShapesOptionsAndPlotinfo(Le, Qe), Me = ke.options, Re = ke.plotinfo;
              function De(Pe) {
                var $e = ge(Le, Me), qe = { "data-index": Qe, "fill-rule": Me.fillrule, d: $e }, Xe = Me.opacity, Ve = Me.fillcolor, Ze = Me.line.width ? Me.line.color : "rgba(0,0,0,0)", Ge = Me.line.width, ze = Me.line.dash;
                Ge || Me.editable !== !0 || (Ge = 5, ze = "solid");
                var We = $e[$e.length - 1] !== "Z", at = Ce(Le) && Me.editable && Le._fullLayout._activeShapeIndex === Qe;
                at && (Ve = We ? "rgba(0,0,0,0)" : Le._fullLayout.activeshape.fillcolor, Xe = Le._fullLayout.activeshape.opacity);
                var ct, et = Pe.append("g").classed("shape-group", !0).attr({ "data-index": Qe }), dt = et.append("path").attr(qe).style("opacity", Xe).call(ue.stroke, Ze).call(ue.fill, Ve).call(de.dashLine, ze, Ge);
                if (we(et, Le, Me), ce(Le, Qe, Me, et), (at || Le._context.edits.shapePosition) && (ct = fe(Le.layout, "shapes", Me)), at) {
                  dt.style({ cursor: "move" });
                  var Je = { element: dt.node(), plotinfo: Re, gd: Le, editHelpers: ct, hasText: Me.label.text || Me.label.texttemplate, isActiveShape: !0 }, it = oe($e, Le);
                  se(it, dt, Je);
                } else Le._context.edits.shapePosition ? function(st, gt, bt, yt, St, Lt) {
                  var Rt, vt, $t, Pt, Xt, Nt, Gt, Ft, Tt, It, Ct, Kt, Vt, Yt, sn, hn, Qt = 10, Ut = 10, Wt = bt.xsizemode === "pixel", rn = bt.ysizemode === "pixel", vn = bt.type === "line", dn = bt.type === "path", Ht = Lt.modifyItem, jt = te.select(gt.node().parentNode), cn = ae.getFromId(st, bt.xref), wn = ae.getRefType(bt.xref), Rn = ae.getFromId(st, bt.yref), Pn = ae.getRefType(bt.yref), Ie = bt.x0shift, rt = bt.x1shift, ft = bt.y0shift, pt = bt.y1shift, wt = function(Zt, Mn) {
                    return ye.getDataToPixel(st, cn, Mn, !1, wn)(Zt);
                  }, Bt = function(Zt, Mn) {
                    return ye.getDataToPixel(st, Rn, Mn, !0, Pn)(Zt);
                  }, At = ye.getPixelToData(st, cn, !1, wn), Dt = ye.getPixelToData(st, Rn, !0, Pn), Ot = vn ? function() {
                    var Zt = 10, Mn = Math.max(bt.line.width, Zt), Ln = St.append("g").attr("data-index", yt).attr("drag-helper", !0);
                    Ln.append("path").attr("d", gt.attr("d")).style({ cursor: "move", "stroke-width": Mn, "stroke-opacity": "0" });
                    var In = { "fill-opacity": "0" }, kn = Math.max(Mn / 2, Zt);
                    return Ln.append("circle").attr({ "data-line-point": "start-point", cx: Wt ? wt(bt.xanchor) + bt.x0 : wt(bt.x0, Ie), cy: rn ? Bt(bt.yanchor) - bt.y0 : Bt(bt.y0, ft), r: kn }).style(In).classed("cursor-grab", !0), Ln.append("circle").attr({ "data-line-point": "end-point", cx: Wt ? wt(bt.xanchor) + bt.x1 : wt(bt.x1, rt), cy: rn ? Bt(bt.yanchor) - bt.y1 : Bt(bt.y1, pt), r: kn }).style(In).classed("cursor-grab", !0), Ln;
                  }() : gt, un = { element: Ot.node(), gd: st, prepFn: function(Zt) {
                    be(st) || (Wt && (Xt = wt(bt.xanchor)), rn && (Nt = Bt(bt.yanchor)), bt.type === "path" ? sn = bt.path : (Rt = Wt ? bt.x0 : wt(bt.x0), vt = rn ? bt.y0 : Bt(bt.y0), $t = Wt ? bt.x1 : wt(bt.x1), Pt = rn ? bt.y1 : Bt(bt.y1)), Rt < $t ? (Tt = Rt, Vt = "x0", It = $t, Yt = "x1") : (Tt = $t, Vt = "x1", It = Rt, Yt = "x0"), !rn && vt < Pt || rn && vt > Pt ? (Gt = vt, Ct = "y0", Ft = Pt, Kt = "y1") : (Gt = Pt, Ct = "y1", Ft = vt, Kt = "y0"), yn(Zt), bn(St, bt), function(Mn, Ln, In) {
                      var kn = Ln.xref, er = Ln.yref, jn = ae.getFromId(In, kn), ur = ae.getFromId(In, er), Kn = "";
                      kn === "paper" || jn.autorange || (Kn += kn), er === "paper" || ur.autorange || (Kn += er), de.setClipUrl(Mn, Kn ? "clip" + In._fullLayout._uid + Kn : null, In);
                    }(gt, bt, st), un.moveFn = hn === "move" ? _n : gn, un.altKey = Zt.altKey);
                  }, doneFn: function() {
                    be(st) || (me(gt), an(St), we(gt, st, bt), ne.call("_guiRelayout", st, Lt.getUpdateObj()));
                  }, clickFn: function() {
                    be(st) || an(St);
                  } };
                  function yn(Zt) {
                    if (be(st)) hn = null;
                    else if (vn) hn = Zt.target.tagName === "path" ? "move" : Zt.target.attributes["data-line-point"].value === "start-point" ? "resize-over-start-point" : "resize-over-end-point";
                    else {
                      var Mn = un.element.getBoundingClientRect(), Ln = Mn.right - Mn.left, In = Mn.bottom - Mn.top, kn = Zt.clientX - Mn.left, er = Zt.clientY - Mn.top, jn = !dn && Ln > Qt && In > Ut && !Zt.shiftKey ? he.getCursor(kn / Ln, 1 - er / In) : "move";
                      me(gt, jn), hn = jn.split("-")[0];
                    }
                  }
                  function _n(Zt, Mn) {
                    if (bt.type === "path") {
                      var Ln = function(er) {
                        return er;
                      }, In = Ln, kn = Ln;
                      Wt ? Ht("xanchor", bt.xanchor = At(Xt + Zt)) : (In = function(er) {
                        return At(wt(er) + Zt);
                      }, cn && cn.type === "date" && (In = ye.encodeDate(In))), rn ? Ht("yanchor", bt.yanchor = Dt(Nt + Mn)) : (kn = function(er) {
                        return Dt(Bt(er) + Mn);
                      }, Rn && Rn.type === "date" && (kn = ye.encodeDate(kn))), Ht("path", bt.path = Ee(sn, In, kn));
                    } else Wt ? Ht("xanchor", bt.xanchor = At(Xt + Zt)) : (Ht("x0", bt.x0 = At(Rt + Zt)), Ht("x1", bt.x1 = At($t + Zt))), rn ? Ht("yanchor", bt.yanchor = Dt(Nt + Mn)) : (Ht("y0", bt.y0 = Dt(vt + Mn)), Ht("y1", bt.y1 = Dt(Pt + Mn)));
                    gt.attr("d", ge(st, bt)), bn(St, bt), ce(st, yt, bt, jt);
                  }
                  function gn(Zt, Mn) {
                    if (dn) {
                      var Ln = function(gi) {
                        return gi;
                      }, In = Ln, kn = Ln;
                      Wt ? Ht("xanchor", bt.xanchor = At(Xt + Zt)) : (In = function(gi) {
                        return At(wt(gi) + Zt);
                      }, cn && cn.type === "date" && (In = ye.encodeDate(In))), rn ? Ht("yanchor", bt.yanchor = Dt(Nt + Mn)) : (kn = function(gi) {
                        return Dt(Bt(gi) + Mn);
                      }, Rn && Rn.type === "date" && (kn = ye.encodeDate(kn))), Ht("path", bt.path = Ee(sn, In, kn));
                    } else if (vn) {
                      if (hn === "resize-over-start-point") {
                        var er = Rt + Zt, jn = rn ? vt - Mn : vt + Mn;
                        Ht("x0", bt.x0 = Wt ? er : At(er)), Ht("y0", bt.y0 = rn ? jn : Dt(jn));
                      } else if (hn === "resize-over-end-point") {
                        var ur = $t + Zt, Kn = rn ? Pt - Mn : Pt + Mn;
                        Ht("x1", bt.x1 = Wt ? ur : At(ur)), Ht("y1", bt.y1 = rn ? Kn : Dt(Kn));
                      }
                    } else {
                      var Jn = function(gi) {
                        return hn.indexOf(gi) !== -1;
                      }, Lr = Jn("n"), mr = Jn("s"), Ar = Jn("w"), Yr = Jn("e"), jr = Lr ? Gt + Mn : Gt, Mr = mr ? Ft + Mn : Ft, Wr = Ar ? Tt + Zt : Tt, Bi = Yr ? It + Zt : It;
                      rn && (Lr && (jr = Gt - Mn), mr && (Mr = Ft - Mn)), (!rn && Mr - jr > Ut || rn && jr - Mr > Ut) && (Ht(Ct, bt[Ct] = rn ? jr : Dt(jr)), Ht(Kt, bt[Kt] = rn ? Mr : Dt(Mr))), Bi - Wr > Qt && (Ht(Vt, bt[Vt] = Wt ? Wr : At(Wr)), Ht(Yt, bt[Yt] = Wt ? Bi : At(Bi)));
                    }
                    gt.attr("d", ge(st, bt)), bn(St, bt), ce(st, yt, bt, jt);
                  }
                  function bn(Zt, Mn) {
                    (Wt || rn) && function() {
                      var Ln = Mn.type !== "path", In = Zt.selectAll(".visual-cue").data([0]);
                      In.enter().append("path").attr({ fill: "#fff", "fill-rule": "evenodd", stroke: "#000", "stroke-width": 1 }).classed("visual-cue", !0);
                      var kn = wt(Wt ? Mn.xanchor : ee.midRange(Ln ? [Mn.x0, Mn.x1] : ye.extractPathCoords(Mn.path, pe.paramIsX))), er = Bt(rn ? Mn.yanchor : ee.midRange(Ln ? [Mn.y0, Mn.y1] : ye.extractPathCoords(Mn.path, pe.paramIsY)));
                      if (kn = ye.roundPositionForSharpStrokeRendering(kn, 1), er = ye.roundPositionForSharpStrokeRendering(er, 1), Wt && rn) {
                        var jn = "M" + (kn - 1 - 1) + "," + (er - 1 - 1) + "h-8v2h8 v8h2v-8 h8v-2h-8 v-8h-2 Z";
                        In.attr("d", jn);
                      } else if (Wt) {
                        var ur = "M" + (kn - 1 - 1) + "," + (er - 9 - 1) + "v18 h2 v-18 Z";
                        In.attr("d", ur);
                      } else {
                        var Kn = "M" + (kn - 9 - 1) + "," + (er - 1 - 1) + "h18 v2 h-18 Z";
                        In.attr("d", Kn);
                      }
                    }();
                  }
                  function an(Zt) {
                    Zt.selectAll(".visual-cue").remove();
                  }
                  he.init(un), Ot.node().onmousemove = yn;
                }(Le, dt, Me, Qe, Pe, ct) : Me.editable === !0 && dt.style("pointer-events", We || ue.opacity(Ve) * Xe <= 0.5 ? "stroke" : "all");
                dt.node().addEventListener("click", function() {
                  return function(st, gt) {
                    if (Ce(st)) {
                      var bt = +gt.node().getAttribute("data-index");
                      if (bt >= 0) {
                        if (bt === st._fullLayout._activeShapeIndex) return void xe(st);
                        st._fullLayout._activeShapeIndex = bt, st._fullLayout._deactivateShape = xe, ve(st);
                      }
                    }
                  }(Le, dt);
                });
              }
              Me._input && Me.visible === !0 && (Me.layer === "above" ? De(Le._fullLayout._shapeUpperLayer) : Me.xref === "paper" || Me.yref === "paper" ? De(Le._fullLayout._shapeLowerLayer) : Me.layer === "between" ? De(Re.shapelayerBetween) : Re._hadPlotinfo ? De((Re.mainplotinfo || Re).shapelayer) : De(Le._fullLayout._shapeLowerLayer));
            }
            function we(Le, Qe, ke) {
              var Me = (ke.xref + ke.yref).replace(/paper/g, "").replace(/[xyz][1-9]* *domain/g, "");
              de.setClipUrl(Le, Me ? "clip" + Qe._fullLayout._uid + Me : null, Qe);
            }
            function Ee(Le, Qe, ke) {
              return Le.replace(pe.segmentRE, function(Me) {
                var Re = 0, De = Me.charAt(0), Pe = pe.paramIsX[De], $e = pe.paramIsY[De], qe = pe.numParams[De];
                return De + Me.substr(1).replace(pe.paramRE, function(Xe) {
                  return Re >= qe || (Pe[Re] ? Xe = Qe(Xe) : $e[Re] && (Xe = ke(Xe)), Re++), Xe;
                });
              });
            }
            function xe(Le) {
              Ce(Le) && Le._fullLayout._activeShapeIndex >= 0 && (le(Le), delete Le._fullLayout._activeShapeIndex, ve(Le));
            }
            G.exports = { draw: ve, drawOne: Se, eraseActiveShape: function(Le) {
              if (Ce(Le)) {
                le(Le);
                var Qe = Le._fullLayout._activeShapeIndex, ke = (Le.layout || {}).shapes || [];
                if (Qe < ke.length) {
                  for (var Me = [], Re = 0; Re < ke.length; Re++) Re !== Qe && Me.push(ke[Re]);
                  return delete Le._fullLayout._activeShapeIndex, ne.call("_guiRelayout", Le, { shapes: Me });
                }
              }
            }, drawLabel: ce };
          }, 64101: function(G, J, Z) {
            var te = Z(13582).overrideAll, ne = Z(9829), ee = Z(80337), ae = Z(94850).T, oe = Z(93049).extendFlat, se = Z(3208).LF, ce = Z(41235);
            G.exports = te({ newshape: { visible: oe({}, ne.visible, {}), showlegend: { valType: "boolean", dflt: !1 }, legend: oe({}, ne.legend, {}), legendgroup: oe({}, ne.legendgroup, {}), legendgrouptitle: { text: oe({}, ne.legendgrouptitle.text, {}), font: ee({}) }, legendrank: oe({}, ne.legendrank, {}), legendwidth: oe({}, ne.legendwidth, {}), line: { color: { valType: "color" }, width: { valType: "number", min: 0, dflt: 4 }, dash: oe({}, ae, { dflt: "solid" }) }, fillcolor: { valType: "color", dflt: "rgba(0,0,0,0)" }, fillrule: { valType: "enumerated", values: ["evenodd", "nonzero"], dflt: "evenodd" }, opacity: { valType: "number", min: 0, max: 1, dflt: 1 }, layer: { valType: "enumerated", values: ["below", "above", "between"], dflt: "above" }, drawdirection: { valType: "enumerated", values: ["ortho", "horizontal", "vertical", "diagonal"], dflt: "diagonal" }, name: oe({}, ne.name, {}), label: { text: { valType: "string", dflt: "" }, texttemplate: se({ newshape: !0 }, { keys: Object.keys(ce) }), font: ee({}), textposition: { valType: "enumerated", values: ["top left", "top center", "top right", "middle left", "middle center", "middle right", "bottom left", "bottom center", "bottom right", "start", "middle", "end"] }, textangle: { valType: "angle", dflt: "auto" }, xanchor: { valType: "enumerated", values: ["auto", "left", "center", "right"], dflt: "auto" }, yanchor: { valType: "enumerated", values: ["top", "middle", "bottom"] }, padding: { valType: "number", dflt: 3, min: 0 } } }, activeshape: { fillcolor: { valType: "color", dflt: "rgb(255,0,255)" }, opacity: { valType: "number", min: 0, max: 1, dflt: 0.5 } } }, "none", "from-root");
          }, 93391: function(G) {
            G.exports = { CIRCLE_SIDES: 32, i000: 0, i090: 8, i180: 16, i270: 24, cos45: Math.cos(Math.PI / 4), sin45: Math.sin(Math.PI / 4), SQRT2: Math.sqrt(2) };
          }, 85522: function(G, J, Z) {
            var te = Z(78766), ne = Z(34809);
            G.exports = function(ee, ae, oe) {
              if (oe("newshape.visible"), oe("newshape.name"), oe("newshape.showlegend"), oe("newshape.legend"), oe("newshape.legendwidth"), oe("newshape.legendgroup"), oe("newshape.legendgrouptitle.text"), ne.coerceFont(oe, "newshape.legendgrouptitle.font"), oe("newshape.legendrank"), oe("newshape.drawdirection"), oe("newshape.layer"), oe("newshape.fillcolor"), oe("newshape.fillrule"), oe("newshape.opacity"), oe("newshape.line.width")) {
                var se = (ee || {}).plot_bgcolor || "#FFF";
                oe("newshape.line.color", te.contrast(se)), oe("newshape.line.dash");
              }
              var ce = ee.dragmode === "drawline", le = oe("newshape.label.text"), ue = oe("newshape.label.texttemplate");
              if (le || ue) {
                oe("newshape.label.textangle");
                var de = oe("newshape.label.textposition", ce ? "middle" : "middle center");
                oe("newshape.label.xanchor"), oe("newshape.label.yanchor", function(fe, he) {
                  return fe ? "bottom" : he.indexOf("top") !== -1 ? "top" : he.indexOf("bottom") !== -1 ? "bottom" : "middle";
                }(ce, de)), oe("newshape.label.padding"), ne.coerceFont(oe, "newshape.label.font", ae.font);
              }
              oe("activeshape.fillcolor"), oe("activeshape.opacity");
            };
          }, 81055: function(G, J, Z) {
            var te = Z(26953), ne = Z(93391), ee = ne.CIRCLE_SIDES, ae = ne.SQRT2, oe = Z(49801), se = oe.p2r, ce = oe.r2p, le = [0, 3, 4, 5, 6, 1, 2], ue = [0, 3, 4, 1, 2];
            function de(he, me) {
              return Math.abs(he - me) <= 1e-6;
            }
            function fe(he, me) {
              var pe = me[1] - he[1], ye = me[2] - he[2];
              return Math.sqrt(pe * pe + ye * ye);
            }
            J.writePaths = function(he) {
              var me = he.length;
              if (!me) return "M0,0Z";
              for (var pe = "", ye = 0; ye < me; ye++) for (var ge = he[ye].length, ve = 0; ve < ge; ve++) {
                var be = he[ye][ve][0];
                if (be === "Z") pe += "Z";
                else for (var Ce = he[ye][ve].length, Se = 0; Se < Ce; Se++) {
                  var we = Se;
                  be === "Q" || be === "S" ? we = ue[Se] : be === "C" && (we = le[Se]), pe += he[ye][ve][we], Se > 0 && Se < Ce - 1 && (pe += ",");
                }
              }
              return pe;
            }, J.readPaths = function(he, me, pe, ye) {
              var ge, ve, be, Ce = te(he), Se = [], we = -1, Ee = 0, xe = 0, Le = function() {
                ve = Ee, be = xe;
              };
              Le();
              for (var Qe = 0; Qe < Ce.length; Qe++) {
                var ke, Me, Re, De, Pe = [], $e = Ce[Qe][0], qe = $e;
                switch ($e) {
                  case "M":
                    Se[++we] = [], Ee = +Ce[Qe][1], xe = +Ce[Qe][2], Pe.push([qe, Ee, xe]), Le();
                    break;
                  case "Q":
                  case "S":
                    ke = +Ce[Qe][1], Re = +Ce[Qe][2], Ee = +Ce[Qe][3], xe = +Ce[Qe][4], Pe.push([qe, Ee, xe, ke, Re]);
                    break;
                  case "C":
                    ke = +Ce[Qe][1], Re = +Ce[Qe][2], Me = +Ce[Qe][3], De = +Ce[Qe][4], Ee = +Ce[Qe][5], xe = +Ce[Qe][6], Pe.push([qe, Ee, xe, ke, Re, Me, De]);
                    break;
                  case "T":
                  case "L":
                    Ee = +Ce[Qe][1], xe = +Ce[Qe][2], Pe.push([qe, Ee, xe]);
                    break;
                  case "H":
                    qe = "L", Ee = +Ce[Qe][1], Pe.push([qe, Ee, xe]);
                    break;
                  case "V":
                    qe = "L", xe = +Ce[Qe][1], Pe.push([qe, Ee, xe]);
                    break;
                  case "A":
                    qe = "L";
                    var Xe = +Ce[Qe][1], Ve = +Ce[Qe][2];
                    +Ce[Qe][4] || (Xe = -Xe, Ve = -Ve);
                    var Ze = Ee - Xe, Ge = xe;
                    for (ge = 1; ge <= ee / 2; ge++) {
                      var ze = 2 * Math.PI * ge / ee;
                      Pe.push([qe, Ze + Xe * Math.cos(ze), Ge + Ve * Math.sin(ze)]);
                    }
                    break;
                  case "Z":
                    Ee === ve && xe === be || (Ee = ve, xe = be, Pe.push([qe, Ee, xe]));
                }
                for (var We = (pe || {}).domain, at = me._fullLayout._size, ct = pe && pe.xsizemode === "pixel", et = pe && pe.ysizemode === "pixel", dt = ye === !1, Je = 0; Je < Pe.length; Je++) {
                  for (ge = 0; ge + 2 < 7; ge += 2) {
                    var it = Pe[Je][ge + 1], st = Pe[Je][ge + 2];
                    it !== void 0 && st !== void 0 && (Ee = it, xe = st, pe && (pe.xaxis && pe.xaxis.p2r ? (dt && (it -= pe.xaxis._offset), it = ct ? ce(pe.xaxis, pe.xanchor) + it : se(pe.xaxis, it)) : (dt && (it -= at.l), We ? it = We.x[0] + it / at.w : it /= at.w), pe.yaxis && pe.yaxis.p2r ? (dt && (st -= pe.yaxis._offset), st = et ? ce(pe.yaxis, pe.yanchor) - st : se(pe.yaxis, st)) : (dt && (st -= at.t), st = We ? We.y[1] - st / at.h : 1 - st / at.h)), Pe[Je][ge + 1] = it, Pe[Je][ge + 2] = st);
                  }
                  Se[we].push(Pe[Je].slice());
                }
              }
              return Se;
            }, J.pointsOnRectangle = function(he) {
              if (he.length !== 5) return !1;
              for (var me = 1; me < 3; me++)
                if (!de(he[0][me] - he[1][me], he[3][me] - he[2][me]) || !de(he[0][me] - he[3][me], he[1][me] - he[2][me])) return !1;
              return !(!de(he[0][1], he[1][1]) && !de(he[0][1], he[3][1]) || !(fe(he[0], he[1]) * fe(he[0], he[3])));
            }, J.pointsOnEllipse = function(he) {
              var me = he.length;
              if (me !== ee + 1) return !1;
              me = ee;
              for (var pe = 0; pe < me; pe++) {
                var ye = (2 * me - pe) % me, ge = (me / 2 + ye) % me, ve = (me / 2 + pe) % me;
                if (!de(fe(he[pe], he[ve]), fe(he[ye], he[ge]))) return !1;
              }
              return !0;
            }, J.handleEllipse = function(he, me, pe) {
              if (!he) return [me, pe];
              var ye = J.ellipseOver({ x0: me[0], y0: me[1], x1: pe[0], y1: pe[1] }), ge = (ye.x1 + ye.x0) / 2, ve = (ye.y1 + ye.y0) / 2, be = (ye.x1 - ye.x0) / 2, Ce = (ye.y1 - ye.y0) / 2;
              be || (be = Ce /= ae), Ce || (Ce = be /= ae);
              for (var Se = [], we = 0; we < ee; we++) {
                var Ee = 2 * we * Math.PI / ee;
                Se.push([ge + be * Math.cos(Ee), ve + Ce * Math.sin(Ee)]);
              }
              return Se;
            }, J.ellipseOver = function(he) {
              var me = he.x0, pe = he.y0, ye = he.x1, ge = he.y1, ve = ye - me, be = ge - pe, Ce = ((me -= ve) + ye) / 2, Se = ((pe -= be) + ge) / 2;
              return { x0: Ce - (ve *= ae), y0: Se - (be *= ae), x1: Ce + ve, y1: Se + be };
            }, J.fixDatesForPaths = function(he, me, pe) {
              var ye = me.type === "date", ge = pe.type === "date";
              if (!ye && !ge) return he;
              for (var ve = 0; ve < he.length; ve++) for (var be = 0; be < he[ve].length; be++) for (var Ce = 0; Ce + 2 < he[ve][be].length; Ce += 2) ye && (he[ve][be][Ce + 1] = he[ve][be][Ce + 1].replace(" ", "_")), ge && (he[ve][be][Ce + 2] = he[ve][be][Ce + 2].replace(" ", "_"));
              return he;
            };
          }, 87562: function(G, J, Z) {
            var te = Z(70414), ne = te.drawMode, ee = te.openMode, ae = Z(93391), oe = ae.i000, se = ae.i090, ce = ae.i180, le = ae.i270, ue = ae.cos45, de = ae.sin45, fe = Z(49801), he = fe.p2r, me = fe.r2p, pe = Z(78534).clearOutline, ye = Z(81055), ge = ye.readPaths, ve = ye.writePaths, be = ye.ellipseOver, Ce = ye.fixDatesForPaths;
            function Se(we, Ee, xe) {
              var Le, Qe = we[0][0], ke = Ee.gd, Me = Qe.getAttribute("d"), Re = ke._fullLayout.newshape, De = Ee.plotinfo, Pe = Ee.isActiveShape, $e = De.xaxis, qe = De.yaxis, Xe = !!De.domain || !De.xaxis, Ve = !!De.domain || !De.yaxis, Ze = ee(xe), Ge = ge(Me, ke, De, Pe), ze = { editable: !0, visible: Re.visible, name: Re.name, showlegend: Re.showlegend, legend: Re.legend, legendwidth: Re.legendwidth, legendgroup: Re.legendgroup, legendgrouptitle: { text: Re.legendgrouptitle.text, font: Re.legendgrouptitle.font }, legendrank: Re.legendrank, label: Re.label, xref: Xe ? "paper" : $e._id, yref: Ve ? "paper" : qe._id, layer: Re.layer, opacity: Re.opacity, line: { color: Re.line.color, width: Re.line.width, dash: Re.line.dash } };
              if (Ze || (ze.fillcolor = Re.fillcolor, ze.fillrule = Re.fillrule), Ge.length === 1 && (Le = Ge[0]), Le && Le.length === 5 && xe === "drawrect") ze.type = "rect", ze.x0 = Le[0][1], ze.y0 = Le[0][2], ze.x1 = Le[2][1], ze.y1 = Le[2][2];
              else if (Le && xe === "drawline") ze.type = "line", ze.x0 = Le[0][1], ze.y0 = Le[0][2], ze.x1 = Le[1][1], ze.y1 = Le[1][2];
              else if (Le && xe === "drawcircle") {
                ze.type = "circle";
                var We = Le[oe][1], at = Le[se][1], ct = Le[ce][1], et = Le[le][1], dt = Le[oe][2], Je = Le[se][2], it = Le[ce][2], st = Le[le][2], gt = De.xaxis && (De.xaxis.type === "date" || De.xaxis.type === "log"), bt = De.yaxis && (De.yaxis.type === "date" || De.yaxis.type === "log");
                gt && (We = me(De.xaxis, We), at = me(De.xaxis, at), ct = me(De.xaxis, ct), et = me(De.xaxis, et)), bt && (dt = me(De.yaxis, dt), Je = me(De.yaxis, Je), it = me(De.yaxis, it), st = me(De.yaxis, st));
                var yt = (at + et) / 2, St = (dt + it) / 2, Lt = be({ x0: yt, y0: St, x1: yt + (et - at + ct - We) / 2 * ue, y1: St + (st - Je + it - dt) / 2 * de });
                gt && (Lt.x0 = he(De.xaxis, Lt.x0), Lt.x1 = he(De.xaxis, Lt.x1)), bt && (Lt.y0 = he(De.yaxis, Lt.y0), Lt.y1 = he(De.yaxis, Lt.y1)), ze.x0 = Lt.x0, ze.y0 = Lt.y0, ze.x1 = Lt.x1, ze.y1 = Lt.y1;
              } else ze.type = "path", $e && qe && Ce(Ge, $e, qe), ze.path = ve(Ge), Le = null;
              return ze;
            }
            G.exports = { newShapes: function(we, Ee) {
              if (we.length && we[0][0]) {
                var xe = Ee.gd, Le = Ee.isActiveShape, Qe = Ee.dragmode, ke = (xe.layout || {}).shapes || [];
                if (!ne(Qe) && Le !== void 0) {
                  var Me = xe._fullLayout._activeShapeIndex;
                  if (Me < ke.length) switch (xe._fullLayout.shapes[Me].type) {
                    case "rect":
                      Qe = "drawrect";
                      break;
                    case "circle":
                      Qe = "drawcircle";
                      break;
                    case "line":
                      Qe = "drawline";
                      break;
                    case "path":
                      var Re = ke[Me].path || "";
                      Qe = Re[Re.length - 1] === "Z" ? "drawclosedpath" : "drawopenpath";
                  }
                }
                var De = Se(we, Ee, Qe);
                pe(xe);
                for (var Pe = Ee.editHelpers, $e = (Pe || {}).modifyItem, qe = [], Xe = 0; Xe < ke.length; Xe++) {
                  var Ve = xe._fullLayout.shapes[Xe];
                  if (qe[Xe] = Ve._input, Le !== void 0 && Xe === xe._fullLayout._activeShapeIndex) {
                    var Ze = De;
                    switch (Ve.type) {
                      case "line":
                      case "rect":
                      case "circle":
                        $e("x0", Ze.x0 - (Ve.x0shift || 0)), $e("x1", Ze.x1 - (Ve.x1shift || 0)), $e("y0", Ze.y0 - (Ve.y0shift || 0)), $e("y1", Ze.y1 - (Ve.y1shift || 0));
                        break;
                      case "path":
                        $e("path", Ze.path);
                    }
                  }
                }
                return Le === void 0 ? (qe.push(De), qe) : Pe ? Pe.getUpdateObj() : {};
              }
            }, createShapeObj: Se };
          }, 78534: function(G) {
            G.exports = { clearOutlineControllers: function(J) {
              var Z = J._fullLayout._zoomlayer;
              Z && Z.selectAll(".outline-controllers").remove();
            }, clearOutline: function(J) {
              var Z = J._fullLayout._zoomlayer;
              Z && Z.selectAll(".select-outline").remove(), J._fullLayout._outlining = !1;
            } };
          }, 49728: function(G, J, Z) {
            var te = Z(2956), ne = Z(34809), ee = Z(29714);
            function ae(oe, se) {
              var ce = 0;
              return (se = se || 0) && oe && (oe.type === "category" || oe.type === "multicategory") && (ce = (oe.r2p(1) - oe.r2p(0)) * se), ce;
            }
            J.rangeToShapePosition = function(oe) {
              return oe.type === "log" ? oe.r2d : function(se) {
                return se;
              };
            }, J.shapePositionToRange = function(oe) {
              return oe.type === "log" ? oe.d2r : function(se) {
                return se;
              };
            }, J.decodeDate = function(oe) {
              return function(se) {
                return se.replace && (se = se.replace("_", " ")), oe(se);
              };
            }, J.encodeDate = function(oe) {
              return function(se) {
                return oe(se).replace(" ", "_");
              };
            }, J.extractPathCoords = function(oe, se, ce) {
              var le = [];
              return oe.match(te.segmentRE).forEach(function(ue) {
                var de = se[ue.charAt(0)].drawn;
                if (de !== void 0) {
                  var fe = ue.substr(1).match(te.paramRE);
                  if (fe && !(fe.length < de)) {
                    var he = fe[de], me = ce ? he : ne.cleanNumber(he);
                    le.push(me);
                  }
                }
              }), le;
            }, J.getDataToPixel = function(oe, se, ce, le, ue) {
              var de, fe = oe._fullLayout._size;
              if (se) if (ue === "domain") de = function(me) {
                return se._length * (le ? 1 - me : me) + se._offset;
              };
              else {
                var he = J.shapePositionToRange(se);
                de = function(me) {
                  var pe = ae(se, ce);
                  return se._offset + se.r2p(he(me, !0)) + pe;
                }, se.type === "date" && (de = J.decodeDate(de));
              }
              else de = le ? function(me) {
                return fe.t + fe.h * (1 - me);
              } : function(me) {
                return fe.l + fe.w * me;
              };
              return de;
            }, J.getPixelToData = function(oe, se, ce, le) {
              var ue, de = oe._fullLayout._size;
              if (se) if (le === "domain") ue = function(he) {
                var me = (he - se._offset) / se._length;
                return ce ? 1 - me : me;
              };
              else {
                var fe = J.rangeToShapePosition(se);
                ue = function(he) {
                  return fe(se.p2r(he - se._offset));
                };
              }
              else ue = ce ? function(he) {
                return 1 - (he - de.t) / de.h;
              } : function(he) {
                return (he - de.l) / de.w;
              };
              return ue;
            }, J.roundPositionForSharpStrokeRendering = function(oe, se) {
              var ce = Math.round(se % 2) === 1, le = Math.round(oe);
              return ce ? le + 0.5 : le;
            }, J.makeShapesOptionsAndPlotinfo = function(oe, se) {
              var ce = oe._fullLayout.shapes[se] || {}, le = oe._fullLayout._plots[ce.xref + ce.yref];
              return le ? le._hadPlotinfo = !0 : (le = {}, ce.xref && ce.xref !== "paper" && (le.xaxis = oe._fullLayout[ce.xref + "axis"]), ce.yref && ce.yref !== "paper" && (le.yaxis = oe._fullLayout[ce.yref + "axis"])), le.xsizemode = ce.xsizemode, le.ysizemode = ce.ysizemode, le.xanchor = ce.xanchor, le.yanchor = ce.yanchor, { options: ce, plotinfo: le };
            }, J.makeSelectionsOptionsAndPlotinfo = function(oe, se) {
              var ce = oe._fullLayout.selections[se] || {}, le = oe._fullLayout._plots[ce.xref + ce.yref];
              return le ? le._hadPlotinfo = !0 : (le = {}, ce.xref && (le.xaxis = oe._fullLayout[ce.xref + "axis"]), ce.yref && (le.yaxis = oe._fullLayout[ce.yref + "axis"])), { options: ce, plotinfo: le };
            }, J.getPathString = function(oe, se) {
              var ce, le, ue, de, fe, he, me, pe, ye = se.type, ge = ee.getRefType(se.xref), ve = ee.getRefType(se.yref), be = ee.getFromId(oe, se.xref), Ce = ee.getFromId(oe, se.yref), Se = oe._fullLayout._size, we = ae(be, se.x0shift), Ee = ae(be, se.x1shift), xe = ae(Ce, se.y0shift), Le = ae(Ce, se.y1shift);
              if (be ? ge === "domain" ? le = function(Xe) {
                return be._offset + be._length * Xe;
              } : (ce = J.shapePositionToRange(be), le = function(Xe) {
                return be._offset + be.r2p(ce(Xe, !0));
              }) : le = function(Xe) {
                return Se.l + Se.w * Xe;
              }, Ce ? ve === "domain" ? de = function(Xe) {
                return Ce._offset + Ce._length * (1 - Xe);
              } : (ue = J.shapePositionToRange(Ce), de = function(Xe) {
                return Ce._offset + Ce.r2p(ue(Xe, !0));
              }) : de = function(Xe) {
                return Se.t + Se.h * (1 - Xe);
              }, ye === "path") return be && be.type === "date" && (le = J.decodeDate(le)), Ce && Ce.type === "date" && (de = J.decodeDate(de)), function(Xe, Ve, Ze) {
                var Ge = Xe.path, ze = Xe.xsizemode, We = Xe.ysizemode, at = Xe.xanchor, ct = Xe.yanchor;
                return Ge.replace(te.segmentRE, function(et) {
                  var dt = 0, Je = et.charAt(0), it = te.paramIsX[Je], st = te.paramIsY[Je], gt = te.numParams[Je], bt = et.substr(1).replace(te.paramRE, function(yt) {
                    return it[dt] ? yt = ze === "pixel" ? Ve(at) + Number(yt) : Ve(yt) : st[dt] && (yt = We === "pixel" ? Ze(ct) - Number(yt) : Ze(yt)), ++dt > gt && (yt = "X"), yt;
                  });
                  return dt > gt && (bt = bt.replace(/[\s,]*X.*/, ""), ne.log("Ignoring extra params in segment " + et)), Je + bt;
                });
              }(se, le, de);
              if (se.xsizemode === "pixel") {
                var Qe = le(se.xanchor);
                fe = Qe + se.x0 + we, he = Qe + se.x1 + Ee;
              } else fe = le(se.x0) + we, he = le(se.x1) + Ee;
              if (se.ysizemode === "pixel") {
                var ke = de(se.yanchor);
                me = ke - se.y0 + xe, pe = ke - se.y1 + Le;
              } else me = de(se.y0) + xe, pe = de(se.y1) + Le;
              if (ye === "line") return "M" + fe + "," + me + "L" + he + "," + pe;
              if (ye === "rect") return "M" + fe + "," + me + "H" + he + "V" + pe + "H" + fe + "Z";
              var Me = (fe + he) / 2, Re = (me + pe) / 2, De = Math.abs(Me - fe), Pe = Math.abs(Re - me), $e = "A" + De + "," + Pe, qe = Me + De + "," + Re;
              return "M" + qe + $e + " 0 1,1 " + Me + "," + (Re - Pe) + $e + " 0 0,1 " + qe + "Z";
            };
          }, 43701: function(G, J, Z) {
            var te = Z(28231);
            G.exports = { moduleType: "component", name: "shapes", layoutAttributes: Z(43144), supplyLayoutDefaults: Z(74367), supplyDrawNewShapeDefaults: Z(85522), includeBasePlot: Z(20706)("shapes"), calcAutorange: Z(44959), draw: te.draw, drawOne: te.drawOne };
          }, 41235: function(G) {
            function J(ce, le) {
              return le ? le.d2l(ce) : ce;
            }
            function Z(ce, le) {
              return le ? le.l2d(ce) : ce;
            }
            function te(ce) {
              return ce.x0shift || 0;
            }
            function ne(ce) {
              return ce.x1shift || 0;
            }
            function ee(ce) {
              return ce.y0shift || 0;
            }
            function ae(ce) {
              return ce.y1shift || 0;
            }
            function oe(ce, le) {
              return J(ce.x1, le) + ne(ce) - J(ce.x0, le) - te(ce);
            }
            function se(ce, le, ue) {
              return J(ce.y1, ue) + ae(ce) - J(ce.y0, ue) - ee(ce);
            }
            G.exports = { x0: function(ce) {
              return ce.x0;
            }, x1: function(ce) {
              return ce.x1;
            }, y0: function(ce) {
              return ce.y0;
            }, y1: function(ce) {
              return ce.y1;
            }, slope: function(ce, le, ue) {
              return ce.type !== "line" ? void 0 : se(ce, 0, ue) / oe(ce, le);
            }, dx: oe, dy: se, width: function(ce, le) {
              return Math.abs(oe(ce, le));
            }, height: function(ce, le, ue) {
              return Math.abs(se(ce, 0, ue));
            }, length: function(ce, le, ue) {
              return ce.type !== "line" ? void 0 : Math.sqrt(Math.pow(oe(ce, le), 2) + Math.pow(se(ce, 0, ue), 2));
            }, xcenter: function(ce, le) {
              return Z((J(ce.x1, le) + ne(ce) + J(ce.x0, le) + te(ce)) / 2, le);
            }, ycenter: function(ce, le, ue) {
              return Z((J(ce.y1, ue) + ae(ce) + J(ce.y0, ue) + ee(ce)) / 2, ue);
            } };
          }, 8606: function(G, J, Z) {
            var te = Z(80337), ne = Z(57891), ee = Z(93049).extendDeepAll, ae = Z(13582).overrideAll, oe = Z(49722), se = Z(78032).templatedArray, ce = Z(64194), le = se("step", { visible: { valType: "boolean", dflt: !0 }, method: { valType: "enumerated", values: ["restyle", "relayout", "animate", "update", "skip"], dflt: "restyle" }, args: { valType: "info_array", freeLength: !0, items: [{ valType: "any" }, { valType: "any" }, { valType: "any" }] }, label: { valType: "string" }, value: { valType: "string" }, execute: { valType: "boolean", dflt: !0 } });
            G.exports = ae(se("slider", { visible: { valType: "boolean", dflt: !0 }, active: { valType: "number", min: 0, dflt: 0 }, steps: le, lenmode: { valType: "enumerated", values: ["fraction", "pixels"], dflt: "fraction" }, len: { valType: "number", min: 0, dflt: 1 }, x: { valType: "number", min: -2, max: 3, dflt: 0 }, pad: ee(ne({ editType: "arraydraw" }), {}, { t: { dflt: 20 } }), xanchor: { valType: "enumerated", values: ["auto", "left", "center", "right"], dflt: "left" }, y: { valType: "number", min: -2, max: 3, dflt: 0 }, yanchor: { valType: "enumerated", values: ["auto", "top", "middle", "bottom"], dflt: "top" }, transition: { duration: { valType: "number", min: 0, dflt: 150 }, easing: { valType: "enumerated", values: oe.transition.easing.values, dflt: "cubic-in-out" } }, currentvalue: { visible: { valType: "boolean", dflt: !0 }, xanchor: { valType: "enumerated", values: ["left", "center", "right"], dflt: "left" }, offset: { valType: "number", dflt: 10 }, prefix: { valType: "string" }, suffix: { valType: "string" }, font: te({}) }, font: te({}), activebgcolor: { valType: "color", dflt: ce.gripBgActiveColor }, bgcolor: { valType: "color", dflt: ce.railBgColor }, bordercolor: { valType: "color", dflt: ce.railBorderColor }, borderwidth: { valType: "number", min: 0, dflt: ce.railBorderWidth }, ticklen: { valType: "number", min: 0, dflt: ce.tickLength }, tickcolor: { valType: "color", dflt: ce.tickColor }, tickwidth: { valType: "number", min: 0, dflt: 1 }, minorticklen: { valType: "number", min: 0, dflt: ce.minorTickLength } }), "arraydraw", "from-root");
          }, 64194: function(G) {
            G.exports = { name: "sliders", containerClassName: "slider-container", groupClassName: "slider-group", inputAreaClass: "slider-input-area", railRectClass: "slider-rail-rect", railTouchRectClass: "slider-rail-touch-rect", gripRectClass: "slider-grip-rect", tickRectClass: "slider-tick-rect", inputProxyClass: "slider-input-proxy", labelsClass: "slider-labels", labelGroupClass: "slider-label-group", labelClass: "slider-label", currentValueClass: "slider-current-value", railHeight: 5, menuIndexAttrName: "slider-active-index", autoMarginIdRoot: "slider-", minWidth: 30, minHeight: 30, textPadX: 40, arrowOffsetX: 4, railRadius: 2, railWidth: 5, railBorder: 4, railBorderWidth: 1, railBorderColor: "#bec8d9", railBgColor: "#f8fafc", railInset: 8, stepInset: 10, gripRadius: 10, gripWidth: 20, gripHeight: 20, gripBorder: 20, gripBorderWidth: 1, gripBorderColor: "#bec8d9", gripBgColor: "#f6f8fa", gripBgActiveColor: "#dbdde0", labelPadding: 8, labelOffset: 0, tickWidth: 1, tickColor: "#333", tickOffset: 25, tickLength: 7, minorTickOffset: 25, minorTickColor: "#333", minorTickLength: 4, currentValuePadding: 8, currentValueInset: 0 };
          }, 74537: function(G, J, Z) {
            var te = Z(34809), ne = Z(59008), ee = Z(8606), ae = Z(64194).name, oe = ee.steps;
            function se(le, ue, de) {
              function fe(ge, ve) {
                return te.coerce(le, ue, ee, ge, ve);
              }
              for (var he = ne(le, ue, { name: "steps", handleItemDefaults: ce }), me = 0, pe = 0; pe < he.length; pe++) he[pe].visible && me++;
              if (me < 2 ? ue.visible = !1 : fe("visible")) {
                ue._stepCount = me;
                var ye = ue._visibleSteps = te.filterVisible(he);
                (he[fe("active")] || {}).visible || (ue.active = ye[0]._index), fe("x"), fe("y"), te.noneOrAll(le, ue, ["x", "y"]), fe("xanchor"), fe("yanchor"), fe("len"), fe("lenmode"), fe("pad.t"), fe("pad.r"), fe("pad.b"), fe("pad.l"), te.coerceFont(fe, "font", de.font), fe("currentvalue.visible") && (fe("currentvalue.xanchor"), fe("currentvalue.prefix"), fe("currentvalue.suffix"), fe("currentvalue.offset"), te.coerceFont(fe, "currentvalue.font", ue.font)), fe("transition.duration"), fe("transition.easing"), fe("bgcolor"), fe("activebgcolor"), fe("bordercolor"), fe("borderwidth"), fe("ticklen"), fe("tickwidth"), fe("tickcolor"), fe("minorticklen");
              }
            }
            function ce(le, ue) {
              function de(he, me) {
                return te.coerce(le, ue, oe, he, me);
              }
              if (le.method === "skip" || Array.isArray(le.args) ? de("visible") : ue.visible = !1) {
                de("method"), de("args");
                var fe = de("label", "step-" + ue._index);
                de("value", fe), de("execute");
              }
            }
            G.exports = function(le, ue) {
              ne(le, ue, { name: ae, handleItemDefaults: se });
            };
          }, 44097: function(G, J, Z) {
            var te = Z(45568), ne = Z(44122), ee = Z(78766), ae = Z(62203), oe = Z(34809), se = oe.strTranslate, ce = Z(30635), le = Z(78032).arrayEditor, ue = Z(64194), de = Z(4530), fe = de.LINE_SPACING, he = de.FROM_TL, me = de.FROM_BR;
            function pe(Pe) {
              return ue.autoMarginIdRoot + Pe._index;
            }
            function ye(Pe) {
              return Pe._index;
            }
            function ge(Pe, $e) {
              var qe = ae.tester.selectAll("g." + ue.labelGroupClass).data($e._visibleSteps);
              qe.enter().append("g").classed(ue.labelGroupClass, !0);
              var Xe = 0, Ve = 0;
              qe.each(function(Je) {
                var it = Ce(te.select(this), { step: Je }, $e).node();
                if (it) {
                  var st = ae.bBox(it);
                  Ve = Math.max(Ve, st.height), Xe = Math.max(Xe, st.width);
                }
              }), qe.remove();
              var Ze = $e._dims = {};
              Ze.inputAreaWidth = Math.max(ue.railWidth, ue.gripHeight);
              var Ge = Pe._fullLayout._size;
              Ze.lx = Ge.l + Ge.w * $e.x, Ze.ly = Ge.t + Ge.h * (1 - $e.y), $e.lenmode === "fraction" ? Ze.outerLength = Math.round(Ge.w * $e.len) : Ze.outerLength = $e.len, Ze.inputAreaStart = 0, Ze.inputAreaLength = Math.round(Ze.outerLength - $e.pad.l - $e.pad.r);
              var ze = (Ze.inputAreaLength - 2 * ue.stepInset) / ($e._stepCount - 1), We = Xe + ue.labelPadding;
              if (Ze.labelStride = Math.max(1, Math.ceil(We / ze)), Ze.labelHeight = Ve, Ze.currentValueMaxWidth = 0, Ze.currentValueHeight = 0, Ze.currentValueTotalHeight = 0, Ze.currentValueMaxLines = 1, $e.currentvalue.visible) {
                var at = ae.tester.append("g");
                qe.each(function(Je) {
                  var it = ve(at, $e, Je.label), st = it.node() && ae.bBox(it.node()) || { width: 0, height: 0 }, gt = ce.lineCount(it);
                  Ze.currentValueMaxWidth = Math.max(Ze.currentValueMaxWidth, Math.ceil(st.width)), Ze.currentValueHeight = Math.max(Ze.currentValueHeight, Math.ceil(st.height)), Ze.currentValueMaxLines = Math.max(Ze.currentValueMaxLines, gt);
                }), Ze.currentValueTotalHeight = Ze.currentValueHeight + $e.currentvalue.offset, at.remove();
              }
              Ze.height = Ze.currentValueTotalHeight + ue.tickOffset + $e.ticklen + ue.labelOffset + Ze.labelHeight + $e.pad.t + $e.pad.b;
              var ct = "left";
              oe.isRightAnchor($e) && (Ze.lx -= Ze.outerLength, ct = "right"), oe.isCenterAnchor($e) && (Ze.lx -= Ze.outerLength / 2, ct = "center");
              var et = "top";
              oe.isBottomAnchor($e) && (Ze.ly -= Ze.height, et = "bottom"), oe.isMiddleAnchor($e) && (Ze.ly -= Ze.height / 2, et = "middle"), Ze.outerLength = Math.ceil(Ze.outerLength), Ze.height = Math.ceil(Ze.height), Ze.lx = Math.round(Ze.lx), Ze.ly = Math.round(Ze.ly);
              var dt = { y: $e.y, b: Ze.height * me[et], t: Ze.height * he[et] };
              $e.lenmode === "fraction" ? (dt.l = 0, dt.xl = $e.x - $e.len * he[ct], dt.r = 0, dt.xr = $e.x + $e.len * me[ct]) : (dt.x = $e.x, dt.l = Ze.outerLength * he[ct], dt.r = Ze.outerLength * me[ct]), ne.autoMargin(Pe, pe($e), dt);
            }
            function ve(Pe, $e, qe) {
              if ($e.currentvalue.visible) {
                var Xe, Ve, Ze = $e._dims;
                switch ($e.currentvalue.xanchor) {
                  case "right":
                    Xe = Ze.inputAreaLength - ue.currentValueInset - Ze.currentValueMaxWidth, Ve = "left";
                    break;
                  case "center":
                    Xe = 0.5 * Ze.inputAreaLength, Ve = "middle";
                    break;
                  default:
                    Xe = ue.currentValueInset, Ve = "left";
                }
                var Ge = oe.ensureSingle(Pe, "text", ue.labelClass, function(dt) {
                  dt.attr({ "text-anchor": Ve, "data-notex": 1 });
                }), ze = $e.currentvalue.prefix ? $e.currentvalue.prefix : "";
                if (typeof qe == "string") ze += qe;
                else {
                  var We = $e.steps[$e.active].label, at = $e._gd._fullLayout._meta;
                  at && (We = oe.templateString(We, at)), ze += We;
                }
                $e.currentvalue.suffix && (ze += $e.currentvalue.suffix), Ge.call(ae.font, $e.currentvalue.font).text(ze).call(ce.convertToTspans, $e._gd);
                var ct = ce.lineCount(Ge), et = (Ze.currentValueMaxLines + 1 - ct) * $e.currentvalue.font.size * fe;
                return ce.positionText(Ge, Xe, et), Ge;
              }
            }
            function be(Pe, $e, qe) {
              oe.ensureSingle(Pe, "rect", ue.gripRectClass, function(Xe) {
                Xe.call(xe, $e, Pe, qe).style("pointer-events", "all");
              }).attr({ width: ue.gripWidth, height: ue.gripHeight, rx: ue.gripRadius, ry: ue.gripRadius }).call(ee.stroke, qe.bordercolor).call(ee.fill, qe.bgcolor).style("stroke-width", qe.borderwidth + "px");
            }
            function Ce(Pe, $e, qe) {
              var Xe = oe.ensureSingle(Pe, "text", ue.labelClass, function(Ge) {
                Ge.attr({ "text-anchor": "middle", "data-notex": 1 });
              }), Ve = $e.step.label, Ze = qe._gd._fullLayout._meta;
              return Ze && (Ve = oe.templateString(Ve, Ze)), Xe.call(ae.font, qe.font).text(Ve).call(ce.convertToTspans, qe._gd), Xe;
            }
            function Se(Pe, $e) {
              var qe = oe.ensureSingle(Pe, "g", ue.labelsClass), Xe = $e._dims, Ve = qe.selectAll("g." + ue.labelGroupClass).data(Xe.labelSteps);
              Ve.enter().append("g").classed(ue.labelGroupClass, !0), Ve.exit().remove(), Ve.each(function(Ze) {
                var Ge = te.select(this);
                Ge.call(Ce, Ze, $e), ae.setTranslate(Ge, ke($e, Ze.fraction), ue.tickOffset + $e.ticklen + $e.font.size * fe + ue.labelOffset + Xe.currentValueTotalHeight);
              });
            }
            function we(Pe, $e, qe, Xe, Ve) {
              var Ze = Math.round(Xe * (qe._stepCount - 1)), Ge = qe._visibleSteps[Ze]._index;
              Ge !== qe.active && Ee(Pe, $e, qe, Ge, !0, Ve);
            }
            function Ee(Pe, $e, qe, Xe, Ve, Ze) {
              var Ge = qe.active;
              qe.active = Xe, le(Pe.layout, ue.name, qe).applyUpdate("active", Xe);
              var ze = qe.steps[qe.active];
              $e.call(Qe, qe, Ze), $e.call(ve, qe), Pe.emit("plotly_sliderchange", { slider: qe, step: qe.steps[qe.active], interaction: Ve, previousActive: Ge }), ze && ze.method && Ve && ($e._nextMethod ? ($e._nextMethod.step = ze, $e._nextMethod.doCallback = Ve, $e._nextMethod.doTransition = Ze) : ($e._nextMethod = { step: ze, doCallback: Ve, doTransition: Ze }, $e._nextMethodRaf = window.requestAnimationFrame(function() {
                var We = $e._nextMethod.step;
                We.method && (We.execute && ne.executeAPICommand(Pe, We.method, We.args), $e._nextMethod = null, $e._nextMethodRaf = null);
              })));
            }
            function xe(Pe, $e, qe) {
              if (!$e._context.staticPlot) {
                var Xe = qe.node(), Ve = te.select($e);
                Pe.on("mousedown", Ge), Pe.on("touchstart", Ge);
              }
              function Ze() {
                return qe.data()[0];
              }
              function Ge() {
                var ze = Ze();
                $e.emit("plotly_sliderstart", { slider: ze });
                var We = qe.select("." + ue.gripRectClass);
                te.event.stopPropagation(), te.event.preventDefault(), We.call(ee.fill, ze.activebgcolor);
                var at = Me(ze, te.mouse(Xe)[0]);
                function ct() {
                  var dt = Ze(), Je = Me(dt, te.mouse(Xe)[0]);
                  we($e, qe, dt, Je, !1);
                }
                function et() {
                  var dt = Ze();
                  dt._dragging = !1, We.call(ee.fill, dt.bgcolor), Ve.on("mouseup", null), Ve.on("mousemove", null), Ve.on("touchend", null), Ve.on("touchmove", null), $e.emit("plotly_sliderend", { slider: dt, step: dt.steps[dt.active] });
                }
                we($e, qe, ze, at, !0), ze._dragging = !0, Ve.on("mousemove", ct), Ve.on("touchmove", ct), Ve.on("mouseup", et), Ve.on("touchend", et);
              }
            }
            function Le(Pe, $e) {
              var qe = Pe.selectAll("rect." + ue.tickRectClass).data($e._visibleSteps), Xe = $e._dims;
              qe.enter().append("rect").classed(ue.tickRectClass, !0), qe.exit().remove(), qe.attr({ width: $e.tickwidth + "px", "shape-rendering": "crispEdges" }), qe.each(function(Ve, Ze) {
                var Ge = Ze % Xe.labelStride == 0, ze = te.select(this);
                ze.attr({ height: Ge ? $e.ticklen : $e.minorticklen }).call(ee.fill, $e.tickcolor), ae.setTranslate(ze, ke($e, Ze / ($e._stepCount - 1)) - 0.5 * $e.tickwidth, (Ge ? ue.tickOffset : ue.minorTickOffset) + Xe.currentValueTotalHeight);
              });
            }
            function Qe(Pe, $e, qe) {
              for (var Xe = Pe.select("rect." + ue.gripRectClass), Ve = 0, Ze = 0; Ze < $e._stepCount; Ze++) if ($e._visibleSteps[Ze]._index === $e.active) {
                Ve = Ze;
                break;
              }
              var Ge = ke($e, Ve / ($e._stepCount - 1));
              if (!$e._invokingCommand) {
                var ze = Xe;
                qe && $e.transition.duration > 0 && (ze = ze.transition().duration($e.transition.duration).ease($e.transition.easing)), ze.attr("transform", se(Ge - 0.5 * ue.gripWidth, $e._dims.currentValueTotalHeight));
              }
            }
            function ke(Pe, $e) {
              var qe = Pe._dims;
              return qe.inputAreaStart + ue.stepInset + (qe.inputAreaLength - 2 * ue.stepInset) * Math.min(1, Math.max(0, $e));
            }
            function Me(Pe, $e) {
              var qe = Pe._dims;
              return Math.min(1, Math.max(0, ($e - ue.stepInset - qe.inputAreaStart) / (qe.inputAreaLength - 2 * ue.stepInset - 2 * qe.inputAreaStart)));
            }
            function Re(Pe, $e, qe) {
              var Xe = qe._dims, Ve = oe.ensureSingle(Pe, "rect", ue.railTouchRectClass, function(Ze) {
                Ze.call(xe, $e, Pe, qe).style("pointer-events", "all");
              });
              Ve.attr({ width: Xe.inputAreaLength, height: Math.max(Xe.inputAreaWidth, ue.tickOffset + qe.ticklen + Xe.labelHeight) }).call(ee.fill, qe.bgcolor).attr("opacity", 0), ae.setTranslate(Ve, 0, Xe.currentValueTotalHeight);
            }
            function De(Pe, $e) {
              var qe = $e._dims, Xe = qe.inputAreaLength - 2 * ue.railInset, Ve = oe.ensureSingle(Pe, "rect", ue.railRectClass);
              Ve.attr({ width: Xe, height: ue.railWidth, rx: ue.railRadius, ry: ue.railRadius, "shape-rendering": "crispEdges" }).call(ee.stroke, $e.bordercolor).call(ee.fill, $e.bgcolor).style("stroke-width", $e.borderwidth + "px"), ae.setTranslate(Ve, ue.railInset, 0.5 * (qe.inputAreaWidth - ue.railWidth) + qe.currentValueTotalHeight);
            }
            G.exports = function(Pe) {
              var $e = Pe._context.staticPlot, qe = Pe._fullLayout, Xe = function(at, ct) {
                for (var et = at[ue.name], dt = [], Je = 0; Je < et.length; Je++) {
                  var it = et[Je];
                  it.visible && (it._gd = ct, dt.push(it));
                }
                return dt;
              }(qe, Pe), Ve = qe._infolayer.selectAll("g." + ue.containerClassName).data(Xe.length > 0 ? [0] : []);
              function Ze(at) {
                at._commandObserver && (at._commandObserver.remove(), delete at._commandObserver), ne.autoMargin(Pe, pe(at));
              }
              if (Ve.enter().append("g").classed(ue.containerClassName, !0).style("cursor", $e ? null : "ew-resize"), Ve.exit().each(function() {
                te.select(this).selectAll("g." + ue.groupClassName).each(Ze);
              }).remove(), Xe.length !== 0) {
                var Ge = Ve.selectAll("g." + ue.groupClassName).data(Xe, ye);
                Ge.enter().append("g").classed(ue.groupClassName, !0), Ge.exit().each(Ze).remove();
                for (var ze = 0; ze < Xe.length; ze++) {
                  var We = Xe[ze];
                  ge(Pe, We);
                }
                Ge.each(function(at) {
                  var ct = te.select(this);
                  (function(et) {
                    var dt = et._dims;
                    dt.labelSteps = [];
                    for (var Je = et._stepCount, it = 0; it < Je; it += dt.labelStride) dt.labelSteps.push({ fraction: it / (Je - 1), step: et._visibleSteps[it] });
                  })(at), ne.manageCommandObserver(Pe, at, at._visibleSteps, function(et) {
                    var dt = ct.data()[0];
                    dt.active !== et.index && (dt._dragging || Ee(Pe, ct, dt, et.index, !1, !0));
                  }), function(et, dt, Je) {
                    (Je.steps[Je.active] || {}).visible || (Je.active = Je._visibleSteps[0]._index), dt.call(ve, Je).call(De, Je).call(Se, Je).call(Le, Je).call(Re, et, Je).call(be, et, Je);
                    var it = Je._dims;
                    ae.setTranslate(dt, it.lx + Je.pad.l, it.ly + Je.pad.t), dt.call(Qe, Je, !1), dt.call(ve, Je);
                  }(Pe, te.select(this), at);
                });
              }
            };
          }, 15359: function(G, J, Z) {
            var te = Z(64194);
            G.exports = { moduleType: "component", name: te.name, layoutAttributes: Z(8606), supplyLayoutDefaults: Z(74537), draw: Z(44097) };
          }, 17240: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(44122), ae = Z(33626), oe = Z(34809), se = oe.strTranslate, ce = Z(62203), le = Z(78766), ue = Z(30635), de = Z(20438), fe = Z(4530).OPPOSITE_SIDE, he = / [XY][0-9]* /;
            G.exports = { draw: function(me, pe, ye) {
              var ge, ve = me._fullLayout, be = ye.propContainer, Ce = ye.propName, Se = ye.placeholder, we = ye.traceIndex, Ee = ye.avoid || {}, xe = ye.attributes, Le = ye.transform, Qe = ye.containerGroup, ke = 1, Me = be.title, Re = (Me && Me.text ? Me.text : "").trim(), De = !1, Pe = Me && Me.font ? Me.font : {}, $e = Pe.family, qe = Pe.size, Xe = Pe.color, Ve = Pe.weight, Ze = Pe.style, Ge = Pe.variant, ze = Pe.textcase, We = Pe.lineposition, at = Pe.shadow, ct = !!ye.subtitlePropName, et = ye.subtitlePlaceholder, dt = (be.title || {}).subtitle || { text: "", font: {} }, Je = dt.text.trim(), it = !1, st = 1, gt = dt.font, bt = gt.family, yt = gt.size, St = gt.color, Lt = gt.weight, Rt = gt.style, vt = gt.variant, $t = gt.textcase, Pt = gt.lineposition, Xt = gt.shadow;
              Ce === "title.text" ? ge = "titleText" : Ce.indexOf("axis") !== -1 ? ge = "axisTitleText" : Ce.indexOf(!0) && (ge = "colorbarTitleText");
              var Nt = me._context.edits[ge];
              function Gt(rn, vn) {
                return rn !== void 0 && vn !== void 0 && rn.replace(he, " % ") === vn.replace(he, " % ");
              }
              Re === "" ? ke = 0 : Gt(Re, Se) && (Nt || (Re = ""), ke = 0.2, De = !0), ct && (Je === "" ? st = 0 : Gt(Je, et) && (Nt || (Je = ""), st = 0.2, it = !0)), ye._meta ? Re = oe.templateString(Re, ye._meta) : ve._meta && (Re = oe.templateString(Re, ve._meta));
              var Ft, Tt = Re || Je || Nt;
              Qe || (Qe = oe.ensureSingle(ve._infolayer, "g", "g-" + pe), Ft = ve._hColorbarMoveTitle);
              var It = Qe.selectAll("text." + pe).data(Tt ? [0] : []);
              It.enter().append("text"), It.text(Re).attr("class", pe), It.exit().remove();
              var Ct = null, Kt = pe + "-subtitle", Vt = Je || Nt;
              if (ct && Vt && ((Ct = Qe.selectAll("text." + Kt).data(Vt ? [0] : [])).enter().append("text"), Ct.text(Je).attr("class", Kt), Ct.exit().remove()), !Tt) return Qe;
              function Yt(rn, vn) {
                oe.syncOrAsync([sn, hn], { title: rn, subtitle: vn });
              }
              function sn(rn) {
                var vn, dn = rn.title, Ht = rn.subtitle;
                if (!Le && Ft && (Le = {}), Le ? (vn = "", Le.rotate && (vn += "rotate(" + [Le.rotate, xe.x, xe.y] + ")"), (Le.offset || Ft) && (vn += se(0, (Le.offset || 0) - (Ft || 0)))) : vn = null, dn.attr("transform", vn), dn.style("opacity", ke * le.opacity(Xe)).call(ce.font, { color: le.rgb(Xe), size: te.round(qe, 2), family: $e, weight: Ve, style: Ze, variant: Ge, textcase: ze, shadow: at, lineposition: We }).attr(xe).call(ue.convertToTspans, me, function(Ie) {
                  if (Ie) {
                    var rt = te.select(Ie.node().parentNode).select("." + Kt);
                    if (!rt.empty()) {
                      var ft = Ie.node().getBBox();
                      if (ft.height) {
                        var pt = ft.y + ft.height + 1.6 * yt;
                        rt.attr("y", pt);
                      }
                    }
                  }
                }), Ht) {
                  var jt = Qe.select("." + pe + "-math-group"), cn = dn.node().getBBox(), wn = jt.node() ? jt.node().getBBox() : void 0, Rn = wn ? wn.y + wn.height + 1.6 * yt : cn.y + cn.height + 1.6 * yt, Pn = oe.extendFlat({}, xe, { y: Rn });
                  Ht.attr("transform", vn), Ht.style("opacity", st * le.opacity(St)).call(ce.font, { color: le.rgb(St), size: te.round(yt, 2), family: bt, weight: Lt, style: Rt, variant: vt, textcase: $t, shadow: Xt, lineposition: Pt }).attr(Pn).call(ue.convertToTspans, me);
                }
                return ee.previousPromises(me);
              }
              function hn(rn) {
                var vn = rn.title, dn = te.select(vn.node().parentNode);
                if (Ee.selection && Ee.side && Re) {
                  dn.attr("transform", null);
                  var Ht = fe[Ee.side], jt = Ee.side === "left" || Ee.side === "top" ? -1 : 1, cn = ne(Ee.pad) ? Ee.pad : 2, wn = ce.bBox(dn.node()), Rn = { t: 0, b: 0, l: 0, r: 0 }, Pn = me._fullLayout._reservedMargin;
                  for (var Ie in Pn) for (var rt in Pn[Ie]) {
                    var ft = Pn[Ie][rt];
                    Rn[rt] = Math.max(Rn[rt], ft);
                  }
                  var pt = { left: Rn.l, top: Rn.t, right: ve.width - Rn.r, bottom: ve.height - Rn.b }, wt = Ee.maxShift || jt * (pt[Ee.side] - wn[Ee.side]), Bt = 0;
                  if (wt < 0) Bt = wt;
                  else {
                    var At = Ee.offsetLeft || 0, Dt = Ee.offsetTop || 0;
                    wn.left -= At, wn.right -= At, wn.top -= Dt, wn.bottom -= Dt, Ee.selection.each(function() {
                      var un = ce.bBox(this);
                      oe.bBoxIntersect(wn, un, cn) && (Bt = Math.max(Bt, jt * (un[Ee.side] - wn[Ht]) + cn));
                    }), Bt = Math.min(wt, Bt), be._titleScoot = Math.abs(Bt);
                  }
                  if (Bt > 0 || wt < 0) {
                    var Ot = { left: [-Bt, 0], right: [Bt, 0], top: [0, -Bt], bottom: [0, Bt] }[Ee.side];
                    dn.attr("transform", se(Ot[0], Ot[1]));
                  }
                }
              }
              function Qt(rn, vn) {
                rn.text(vn).on("mouseover.opacity", function() {
                  te.select(this).transition().duration(de.SHOW_PLACEHOLDER).style("opacity", 1);
                }).on("mouseout.opacity", function() {
                  te.select(this).transition().duration(de.HIDE_PLACEHOLDER).style("opacity", 0);
                });
              }
              if (It.call(Yt, Ct), Nt && (Re ? It.on(".opacity", null) : (Qt(It, Se), De = !0), It.call(ue.makeEditable, { gd: me }).on("edit", function(rn) {
                we !== void 0 ? ae.call("_guiRestyle", me, Ce, rn, we) : ae.call("_guiRelayout", me, Ce, rn);
              }).on("cancel", function() {
                this.text(this.attr("data-unformatted")).call(Yt);
              }).on("input", function(rn) {
                this.text(rn || " ").call(ue.positionText, xe.x, xe.y);
              }), ct)) {
                if (ct && !Re) {
                  var Ut = It.node().getBBox(), Wt = Ut.y + Ut.height + 1.6 * yt;
                  Ct.attr("y", Wt);
                }
                Je ? Ct.on(".opacity", null) : (Qt(Ct, et), it = !0), Ct.call(ue.makeEditable, { gd: me }).on("edit", function(rn) {
                  ae.call("_guiRelayout", me, "title.subtitle.text", rn);
                }).on("cancel", function() {
                  this.text(this.attr("data-unformatted")).call(Yt);
                }).on("input", function(rn) {
                  this.text(rn || " ").call(ue.positionText, Ct.attr("x"), Ct.attr("y"));
                });
              }
              return It.classed("js-placeholder", De), Ct && Ct.classed("js-placeholder", it), Qe;
            }, SUBTITLE_PADDING_EM: 1.6, SUBTITLE_PADDING_MATHJAX_EM: 1.6 };
          }, 85389: function(G, J, Z) {
            var te = Z(80337), ne = Z(10229), ee = Z(93049).extendFlat, ae = Z(13582).overrideAll, oe = Z(57891), se = Z(78032).templatedArray, ce = se("button", { visible: { valType: "boolean" }, method: { valType: "enumerated", values: ["restyle", "relayout", "animate", "update", "skip"], dflt: "restyle" }, args: { valType: "info_array", freeLength: !0, items: [{ valType: "any" }, { valType: "any" }, { valType: "any" }] }, args2: { valType: "info_array", freeLength: !0, items: [{ valType: "any" }, { valType: "any" }, { valType: "any" }] }, label: { valType: "string", dflt: "" }, execute: { valType: "boolean", dflt: !0 } });
            G.exports = ae(se("updatemenu", { _arrayAttrRegexps: [/^updatemenus\[(0|[1-9][0-9]+)\]\.buttons/], visible: { valType: "boolean" }, type: { valType: "enumerated", values: ["dropdown", "buttons"], dflt: "dropdown" }, direction: { valType: "enumerated", values: ["left", "right", "up", "down"], dflt: "down" }, active: { valType: "integer", min: -1, dflt: 0 }, showactive: { valType: "boolean", dflt: !0 }, buttons: ce, x: { valType: "number", min: -2, max: 3, dflt: -0.05 }, xanchor: { valType: "enumerated", values: ["auto", "left", "center", "right"], dflt: "right" }, y: { valType: "number", min: -2, max: 3, dflt: 1 }, yanchor: { valType: "enumerated", values: ["auto", "top", "middle", "bottom"], dflt: "top" }, pad: ee(oe({ editType: "arraydraw" }), {}), font: te({}), bgcolor: { valType: "color" }, bordercolor: { valType: "color", dflt: ne.borderLine }, borderwidth: { valType: "number", min: 0, dflt: 1, editType: "arraydraw" } }), "arraydraw", "from-root");
          }, 71559: function(G) {
            G.exports = { name: "updatemenus", containerClassName: "updatemenu-container", headerGroupClassName: "updatemenu-header-group", headerClassName: "updatemenu-header", headerArrowClassName: "updatemenu-header-arrow", dropdownButtonGroupClassName: "updatemenu-dropdown-button-group", dropdownButtonClassName: "updatemenu-dropdown-button", buttonClassName: "updatemenu-button", itemRectClassName: "updatemenu-item-rect", itemTextClassName: "updatemenu-item-text", menuIndexAttrName: "updatemenu-active-index", autoMarginIdRoot: "updatemenu-", blankHeaderOpts: { label: "  " }, minWidth: 30, minHeight: 30, textPadX: 24, arrowPadX: 16, rx: 2, ry: 2, textOffsetX: 12, textOffsetY: 3, arrowOffsetX: 4, gapButtonHeader: 5, gapButton: 2, activeColor: "#F4FAFF", hoverColor: "#F4FAFF", arrowSymbol: { left: "◄", right: "►", up: "▲", down: "▼" } };
          }, 42746: function(G, J, Z) {
            var te = Z(34809), ne = Z(59008), ee = Z(85389), ae = Z(71559).name, oe = ee.buttons;
            function se(le, ue, de) {
              function fe(he, me) {
                return te.coerce(le, ue, ee, he, me);
              }
              fe("visible", ne(le, ue, { name: "buttons", handleItemDefaults: ce }).length > 0) && (fe("active"), fe("direction"), fe("type"), fe("showactive"), fe("x"), fe("y"), te.noneOrAll(le, ue, ["x", "y"]), fe("xanchor"), fe("yanchor"), fe("pad.t"), fe("pad.r"), fe("pad.b"), fe("pad.l"), te.coerceFont(fe, "font", de.font), fe("bgcolor", de.paper_bgcolor), fe("bordercolor"), fe("borderwidth"));
            }
            function ce(le, ue) {
              function de(fe, he) {
                return te.coerce(le, ue, oe, fe, he);
              }
              de("visible", le.method === "skip" || Array.isArray(le.args)) && (de("method"), de("args"), de("args2"), de("label"), de("execute"));
            }
            G.exports = function(le, ue) {
              ne(le, ue, { name: ae, handleItemDefaults: se });
            };
          }, 40974: function(G, J, Z) {
            var te = Z(45568), ne = Z(44122), ee = Z(78766), ae = Z(62203), oe = Z(34809), se = Z(30635), ce = Z(78032).arrayEditor, le = Z(4530).LINE_SPACING, ue = Z(71559), de = Z(21736);
            function fe(ke) {
              return ke._index;
            }
            function he(ke, Me) {
              return +ke.attr(ue.menuIndexAttrName) === Me._index;
            }
            function me(ke, Me, Re, De, Pe, $e, qe, Xe) {
              Me.active = qe, ce(ke.layout, ue.name, Me).applyUpdate("active", qe), Me.type === "buttons" ? ye(ke, De, null, null, Me) : Me.type === "dropdown" && (Pe.attr(ue.menuIndexAttrName, "-1"), pe(ke, De, Pe, $e, Me), Xe || ye(ke, De, Pe, $e, Me));
            }
            function pe(ke, Me, Re, De, Pe) {
              var $e = oe.ensureSingle(Me, "g", ue.headerClassName, function(ze) {
                ze.style("pointer-events", "all");
              }), qe = Pe._dims, Xe = Pe.active, Ve = Pe.buttons[Xe] || ue.blankHeaderOpts, Ze = { y: Pe.pad.t, yPad: 0, x: Pe.pad.l, xPad: 0, index: 0 }, Ge = { width: qe.headerWidth, height: qe.headerHeight };
              $e.call(ge, Pe, Ve, ke).call(Le, Pe, Ze, Ge), oe.ensureSingle(Me, "text", ue.headerArrowClassName, function(ze) {
                ze.attr("text-anchor", "end").call(ae.font, Pe.font).text(ue.arrowSymbol[Pe.direction]);
              }).attr({ x: qe.headerWidth - ue.arrowOffsetX + Pe.pad.l, y: qe.headerHeight / 2 + ue.textOffsetY + Pe.pad.t }), $e.on("click", function() {
                Re.call(Qe, String(he(Re, Pe) ? -1 : Pe._index)), ye(ke, Me, Re, De, Pe);
              }), $e.on("mouseover", function() {
                $e.call(Se);
              }), $e.on("mouseout", function() {
                $e.call(we, Pe);
              }), ae.setTranslate(Me, qe.lx, qe.ly);
            }
            function ye(ke, Me, Re, De, Pe) {
              Re || (Re = Me).attr("pointer-events", "all");
              var $e = function(dt) {
                return +dt.attr(ue.menuIndexAttrName) == -1;
              }(Re) && Pe.type !== "buttons" ? [] : Pe.buttons, qe = Pe.type === "dropdown" ? ue.dropdownButtonClassName : ue.buttonClassName, Xe = Re.selectAll("g." + qe).data(oe.filterVisible($e)), Ve = Xe.enter().append("g").classed(qe, !0), Ze = Xe.exit();
              Pe.type === "dropdown" ? (Ve.attr("opacity", "0").transition().attr("opacity", "1"), Ze.transition().attr("opacity", "0").remove()) : Ze.remove();
              var Ge = 0, ze = 0, We = Pe._dims, at = ["up", "down"].indexOf(Pe.direction) !== -1;
              Pe.type === "dropdown" && (at ? ze = We.headerHeight + ue.gapButtonHeader : Ge = We.headerWidth + ue.gapButtonHeader), Pe.type === "dropdown" && Pe.direction === "up" && (ze = -ue.gapButtonHeader + ue.gapButton - We.openHeight), Pe.type === "dropdown" && Pe.direction === "left" && (Ge = -ue.gapButtonHeader + ue.gapButton - We.openWidth);
              var ct = { x: We.lx + Ge + Pe.pad.l, y: We.ly + ze + Pe.pad.t, yPad: ue.gapButton, xPad: ue.gapButton, index: 0 }, et = { l: ct.x + Pe.borderwidth, t: ct.y + Pe.borderwidth };
              Xe.each(function(dt, Je) {
                var it = te.select(this);
                it.call(ge, Pe, dt, ke).call(Le, Pe, ct), it.on("click", function() {
                  te.event.defaultPrevented || (dt.execute && (dt.args2 && Pe.active === Je ? (me(ke, Pe, 0, Me, Re, De, -1), ne.executeAPICommand(ke, dt.method, dt.args2)) : (me(ke, Pe, 0, Me, Re, De, Je), ne.executeAPICommand(ke, dt.method, dt.args))), ke.emit("plotly_buttonclicked", { menu: Pe, button: dt, active: Pe.active }));
                }), it.on("mouseover", function() {
                  it.call(Se);
                }), it.on("mouseout", function() {
                  it.call(we, Pe), Xe.call(Ce, Pe);
                });
              }), Xe.call(Ce, Pe), at ? (et.w = Math.max(We.openWidth, We.headerWidth), et.h = ct.y - et.t) : (et.w = ct.x - et.l, et.h = Math.max(We.openHeight, We.headerHeight)), et.direction = Pe.direction, De && (Xe.size() ? function(dt, Je, it, st, gt, bt) {
                var yt, St, Lt, Rt = gt.direction, vt = Rt === "up" || Rt === "down", $t = gt._dims, Pt = gt.active;
                if (vt) for (St = 0, Lt = 0; Lt < Pt; Lt++) St += $t.heights[Lt] + ue.gapButton;
                else for (yt = 0, Lt = 0; Lt < Pt; Lt++) yt += $t.widths[Lt] + ue.gapButton;
                st.enable(bt, yt, St), st.hbar && st.hbar.attr("opacity", "0").transition().attr("opacity", "1"), st.vbar && st.vbar.attr("opacity", "0").transition().attr("opacity", "1");
              }(0, 0, 0, De, Pe, et) : function(dt) {
                var Je = !!dt.hbar, it = !!dt.vbar;
                Je && dt.hbar.transition().attr("opacity", "0").each("end", function() {
                  Je = !1, it || dt.disable();
                }), it && dt.vbar.transition().attr("opacity", "0").each("end", function() {
                  it = !1, Je || dt.disable();
                });
              }(De));
            }
            function ge(ke, Me, Re, De) {
              ke.call(ve, Me).call(be, Me, Re, De);
            }
            function ve(ke, Me) {
              oe.ensureSingle(ke, "rect", ue.itemRectClassName, function(Re) {
                Re.attr({ rx: ue.rx, ry: ue.ry, "shape-rendering": "crispEdges" });
              }).call(ee.stroke, Me.bordercolor).call(ee.fill, Me.bgcolor).style("stroke-width", Me.borderwidth + "px");
            }
            function be(ke, Me, Re, De) {
              var Pe = oe.ensureSingle(ke, "text", ue.itemTextClassName, function(Xe) {
                Xe.attr({ "text-anchor": "start", "data-notex": 1 });
              }), $e = Re.label, qe = De._fullLayout._meta;
              qe && ($e = oe.templateString($e, qe)), Pe.call(ae.font, Me.font).text($e).call(se.convertToTspans, De);
            }
            function Ce(ke, Me) {
              var Re = Me.active;
              ke.each(function(De, Pe) {
                var $e = te.select(this);
                Pe === Re && Me.showactive && $e.select("rect." + ue.itemRectClassName).call(ee.fill, ue.activeColor);
              });
            }
            function Se(ke) {
              ke.select("rect." + ue.itemRectClassName).call(ee.fill, ue.hoverColor);
            }
            function we(ke, Me) {
              ke.select("rect." + ue.itemRectClassName).call(ee.fill, Me.bgcolor);
            }
            function Ee(ke, Me) {
              var Re = Me._dims = { width1: 0, height1: 0, heights: [], widths: [], totalWidth: 0, totalHeight: 0, openWidth: 0, openHeight: 0, lx: 0, ly: 0 }, De = ae.tester.selectAll("g." + ue.dropdownButtonClassName).data(oe.filterVisible(Me.buttons));
              De.enter().append("g").classed(ue.dropdownButtonClassName, !0);
              var Pe = ["up", "down"].indexOf(Me.direction) !== -1;
              De.each(function(Ge, ze) {
                var We = te.select(this);
                We.call(ge, Me, Ge, ke);
                var at = We.select("." + ue.itemTextClassName), ct = at.node() && ae.bBox(at.node()).width, et = Math.max(ct + ue.textPadX, ue.minWidth), dt = Me.font.size * le, Je = se.lineCount(at), it = Math.max(dt * Je, ue.minHeight) + ue.textOffsetY;
                it = Math.ceil(it), et = Math.ceil(et), Re.widths[ze] = et, Re.heights[ze] = it, Re.height1 = Math.max(Re.height1, it), Re.width1 = Math.max(Re.width1, et), Pe ? (Re.totalWidth = Math.max(Re.totalWidth, et), Re.openWidth = Re.totalWidth, Re.totalHeight += it + ue.gapButton, Re.openHeight += it + ue.gapButton) : (Re.totalWidth += et + ue.gapButton, Re.openWidth += et + ue.gapButton, Re.totalHeight = Math.max(Re.totalHeight, it), Re.openHeight = Re.totalHeight);
              }), Pe ? Re.totalHeight -= ue.gapButton : Re.totalWidth -= ue.gapButton, Re.headerWidth = Re.width1 + ue.arrowPadX, Re.headerHeight = Re.height1, Me.type === "dropdown" && (Pe ? (Re.width1 += ue.arrowPadX, Re.totalHeight = Re.height1) : Re.totalWidth = Re.width1, Re.totalWidth += ue.arrowPadX), De.remove();
              var $e = Re.totalWidth + Me.pad.l + Me.pad.r, qe = Re.totalHeight + Me.pad.t + Me.pad.b, Xe = ke._fullLayout._size;
              Re.lx = Xe.l + Xe.w * Me.x, Re.ly = Xe.t + Xe.h * (1 - Me.y);
              var Ve = "left";
              oe.isRightAnchor(Me) && (Re.lx -= $e, Ve = "right"), oe.isCenterAnchor(Me) && (Re.lx -= $e / 2, Ve = "center");
              var Ze = "top";
              oe.isBottomAnchor(Me) && (Re.ly -= qe, Ze = "bottom"), oe.isMiddleAnchor(Me) && (Re.ly -= qe / 2, Ze = "middle"), Re.totalWidth = Math.ceil(Re.totalWidth), Re.totalHeight = Math.ceil(Re.totalHeight), Re.lx = Math.round(Re.lx), Re.ly = Math.round(Re.ly), ne.autoMargin(ke, xe(Me), { x: Me.x, y: Me.y, l: $e * ({ right: 1, center: 0.5 }[Ve] || 0), r: $e * ({ left: 1, center: 0.5 }[Ve] || 0), b: qe * ({ top: 1, middle: 0.5 }[Ze] || 0), t: qe * ({ bottom: 1, middle: 0.5 }[Ze] || 0) });
            }
            function xe(ke) {
              return ue.autoMarginIdRoot + ke._index;
            }
            function Le(ke, Me, Re, De) {
              De = De || {};
              var Pe = ke.select("." + ue.itemRectClassName), $e = ke.select("." + ue.itemTextClassName), qe = Me.borderwidth, Xe = Re.index, Ve = Me._dims;
              ae.setTranslate(ke, qe + Re.x, qe + Re.y);
              var Ze = ["up", "down"].indexOf(Me.direction) !== -1, Ge = De.height || (Ze ? Ve.heights[Xe] : Ve.height1);
              Pe.attr({ x: 0, y: 0, width: De.width || (Ze ? Ve.width1 : Ve.widths[Xe]), height: Ge });
              var ze = Me.font.size * le, We = (se.lineCount($e) - 1) * ze / 2;
              se.positionText($e, ue.textOffsetX, Ge / 2 - We + ue.textOffsetY), Ze ? Re.y += Ve.heights[Xe] + Re.yPad : Re.x += Ve.widths[Xe] + Re.xPad, Re.index++;
            }
            function Qe(ke, Me) {
              ke.attr(ue.menuIndexAttrName, Me || "-1").selectAll("g." + ue.dropdownButtonClassName).remove();
            }
            G.exports = function(ke) {
              var Me = ke._fullLayout, Re = oe.filterVisible(Me[ue.name]);
              function De(ze) {
                ne.autoMargin(ke, xe(ze));
              }
              var Pe = Me._menulayer.selectAll("g." + ue.containerClassName).data(Re.length > 0 ? [0] : []);
              if (Pe.enter().append("g").classed(ue.containerClassName, !0).style("cursor", "pointer"), Pe.exit().each(function() {
                te.select(this).selectAll("g." + ue.headerGroupClassName).each(De);
              }).remove(), Re.length !== 0) {
                var $e = Pe.selectAll("g." + ue.headerGroupClassName).data(Re, fe);
                $e.enter().append("g").classed(ue.headerGroupClassName, !0);
                for (var qe = oe.ensureSingle(Pe, "g", ue.dropdownButtonGroupClassName, function(ze) {
                  ze.style("pointer-events", "all");
                }), Xe = 0; Xe < Re.length; Xe++) {
                  var Ve = Re[Xe];
                  Ee(ke, Ve);
                }
                var Ze = "updatemenus" + Me._uid, Ge = new de(ke, qe, Ze);
                $e.enter().size() && (qe.node().parentNode.appendChild(qe.node()), qe.call(Qe)), $e.exit().each(function(ze) {
                  qe.call(Qe), De(ze);
                }).remove(), $e.each(function(ze) {
                  var We = te.select(this), at = ze.type === "dropdown" ? qe : null;
                  ne.manageCommandObserver(ke, ze, ze.buttons, function(ct) {
                    me(ke, ze, ze.buttons[ct.index], We, at, Ge, ct.index, !0);
                  }), ze.type === "dropdown" ? (pe(ke, We, qe, Ge, ze), he(qe, ze) && ye(ke, We, qe, Ge, ze)) : ye(ke, We, null, null, ze);
                });
              }
            };
          }, 46230: function(G, J, Z) {
            var te = Z(71559);
            G.exports = { moduleType: "component", name: te.name, layoutAttributes: Z(85389), supplyLayoutDefaults: Z(42746), draw: Z(40974) };
          }, 21736: function(G, J, Z) {
            G.exports = oe;
            var te = Z(45568), ne = Z(78766), ee = Z(62203), ae = Z(34809);
            function oe(se, ce, le) {
              this.gd = se, this.container = ce, this.id = le, this.position = null, this.translateX = null, this.translateY = null, this.hbar = null, this.vbar = null, this.bg = this.container.selectAll("rect.scrollbox-bg").data([0]), this.bg.exit().on(".drag", null).on("wheel", null).remove(), this.bg.enter().append("rect").classed("scrollbox-bg", !0).style("pointer-events", "all").attr({ opacity: 0, x: 0, y: 0, width: 0, height: 0 });
            }
            oe.barWidth = 2, oe.barLength = 20, oe.barRadius = 2, oe.barPad = 1, oe.barColor = "#808BA4", oe.prototype.enable = function(se, ce, le) {
              var ue = this.gd._fullLayout, de = ue.width, fe = ue.height;
              this.position = se;
              var he, me, pe, ye, ge = this.position.l, ve = this.position.w, be = this.position.t, Ce = this.position.h, Se = this.position.direction, we = Se === "down", Ee = Se === "left", xe = Se === "up", Le = ve, Qe = Ce;
              we || Ee || Se === "right" || xe || (this.position.direction = "down", we = !0), we || xe ? (me = (he = ge) + Le, we ? (pe = be, Qe = (ye = Math.min(pe + Qe, fe)) - pe) : Qe = (ye = be + Qe) - (pe = Math.max(ye - Qe, 0))) : (ye = (pe = be) + Qe, Ee ? Le = (me = ge + Le) - (he = Math.max(me - Le, 0)) : (he = ge, Le = (me = Math.min(he + Le, de)) - he)), this._box = { l: he, t: pe, w: Le, h: Qe };
              var ke = ve > Le, Me = oe.barLength + 2 * oe.barPad, Re = oe.barWidth + 2 * oe.barPad, De = ge, Pe = be + Ce;
              Pe + Re > fe && (Pe = fe - Re);
              var $e = this.container.selectAll("rect.scrollbar-horizontal").data(ke ? [0] : []);
              $e.exit().on(".drag", null).remove(), $e.enter().append("rect").classed("scrollbar-horizontal", !0).call(ne.fill, oe.barColor), ke ? (this.hbar = $e.attr({ rx: oe.barRadius, ry: oe.barRadius, x: De, y: Pe, width: Me, height: Re }), this._hbarXMin = De + Me / 2, this._hbarTranslateMax = Le - Me) : (delete this.hbar, delete this._hbarXMin, delete this._hbarTranslateMax);
              var qe = Ce > Qe, Xe = oe.barWidth + 2 * oe.barPad, Ve = oe.barLength + 2 * oe.barPad, Ze = ge + ve, Ge = be;
              Ze + Xe > de && (Ze = de - Xe);
              var ze = this.container.selectAll("rect.scrollbar-vertical").data(qe ? [0] : []);
              ze.exit().on(".drag", null).remove(), ze.enter().append("rect").classed("scrollbar-vertical", !0).call(ne.fill, oe.barColor), qe ? (this.vbar = ze.attr({ rx: oe.barRadius, ry: oe.barRadius, x: Ze, y: Ge, width: Xe, height: Ve }), this._vbarYMin = Ge + Ve / 2, this._vbarTranslateMax = Qe - Ve) : (delete this.vbar, delete this._vbarYMin, delete this._vbarTranslateMax);
              var We = this.id, at = he - 0.5, ct = qe ? me + Xe + 0.5 : me + 0.5, et = pe - 0.5, dt = ke ? ye + Re + 0.5 : ye + 0.5, Je = ue._topdefs.selectAll("#" + We).data(ke || qe ? [0] : []);
              if (Je.exit().remove(), Je.enter().append("clipPath").attr("id", We).append("rect"), ke || qe ? (this._clipRect = Je.select("rect").attr({ x: Math.floor(at), y: Math.floor(et), width: Math.ceil(ct) - Math.floor(at), height: Math.ceil(dt) - Math.floor(et) }), this.container.call(ee.setClipUrl, We, this.gd), this.bg.attr({ x: ge, y: be, width: ve, height: Ce })) : (this.bg.attr({ width: 0, height: 0 }), this.container.on("wheel", null).on(".drag", null).call(ee.setClipUrl, null), delete this._clipRect), ke || qe) {
                var it = te.behavior.drag().on("dragstart", function() {
                  te.event.sourceEvent.preventDefault();
                }).on("drag", this._onBoxDrag.bind(this));
                this.container.on("wheel", null).on("wheel", this._onBoxWheel.bind(this)).on(".drag", null).call(it);
                var st = te.behavior.drag().on("dragstart", function() {
                  te.event.sourceEvent.preventDefault(), te.event.sourceEvent.stopPropagation();
                }).on("drag", this._onBarDrag.bind(this));
                ke && this.hbar.on(".drag", null).call(st), qe && this.vbar.on(".drag", null).call(st);
              }
              this.setTranslate(ce, le);
            }, oe.prototype.disable = function() {
              (this.hbar || this.vbar) && (this.bg.attr({ width: 0, height: 0 }), this.container.on("wheel", null).on(".drag", null).call(ee.setClipUrl, null), delete this._clipRect), this.hbar && (this.hbar.on(".drag", null), this.hbar.remove(), delete this.hbar, delete this._hbarXMin, delete this._hbarTranslateMax), this.vbar && (this.vbar.on(".drag", null), this.vbar.remove(), delete this.vbar, delete this._vbarYMin, delete this._vbarTranslateMax);
            }, oe.prototype._onBoxDrag = function() {
              var se = this.translateX, ce = this.translateY;
              this.hbar && (se -= te.event.dx), this.vbar && (ce -= te.event.dy), this.setTranslate(se, ce);
            }, oe.prototype._onBoxWheel = function() {
              var se = this.translateX, ce = this.translateY;
              this.hbar && (se += te.event.deltaY), this.vbar && (ce += te.event.deltaY), this.setTranslate(se, ce);
            }, oe.prototype._onBarDrag = function() {
              var se = this.translateX, ce = this.translateY;
              if (this.hbar) {
                var le = se + this._hbarXMin, ue = le + this._hbarTranslateMax;
                se = (ae.constrain(te.event.x, le, ue) - le) / (ue - le) * (this.position.w - this._box.w);
              }
              if (this.vbar) {
                var de = ce + this._vbarYMin, fe = de + this._vbarTranslateMax;
                ce = (ae.constrain(te.event.y, de, fe) - de) / (fe - de) * (this.position.h - this._box.h);
              }
              this.setTranslate(se, ce);
            }, oe.prototype.setTranslate = function(se, ce) {
              var le = this.position.w - this._box.w, ue = this.position.h - this._box.h;
              if (se = ae.constrain(se || 0, 0, le), ce = ae.constrain(ce || 0, 0, ue), this.translateX = se, this.translateY = ce, this.container.call(ee.setTranslate, this._box.l - this.position.l - se, this._box.t - this.position.t - ce), this._clipRect && this._clipRect.attr({ x: Math.floor(this.position.l + se - 0.5), y: Math.floor(this.position.t + ce - 0.5) }), this.hbar) {
                var de = se / le;
                this.hbar.call(ee.setTranslate, se + de * this._hbarTranslateMax, ce);
              }
              if (this.vbar) {
                var fe = ce / ue;
                this.vbar.call(ee.setTranslate, se, ce + fe * this._vbarTranslateMax);
              }
            };
          }, 4530: function(G) {
            G.exports = { FROM_BL: { left: 0, center: 0.5, right: 1, bottom: 0, middle: 0.5, top: 1 }, FROM_TL: { left: 0, center: 0.5, right: 1, bottom: 1, middle: 0.5, top: 0 }, FROM_BR: { left: 1, center: 0.5, right: 0, bottom: 0, middle: 0.5, top: 1 }, LINE_SPACING: 1.3, CAP_SHIFT: 0.7, MID_SHIFT: 0.35, OPPOSITE_SIDE: { left: "right", right: "left", top: "bottom", bottom: "top" } };
          }, 35081: function(G) {
            G.exports = { axisRefDescription: function(J, Z, te) {
              return ["If set to a", J, "axis id (e.g. *" + J + "* or", "*" + J + "2*), the `" + J + "` position refers to a", J, "coordinate. If set to *paper*, the `" + J + "`", "position refers to the distance from the", Z, "of the plotting", "area in normalized coordinates where *0* (*1*) corresponds to the", Z, "(" + te + "). If set to a", J, "axis ID followed by", "*domain* (separated by a space), the position behaves like for", "*paper*, but refers to the distance in fractions of the domain", "length from the", Z, "of the domain of that axis: e.g.,", "*" + J + "2 domain* refers to the domain of the second", J, " axis and a", J, "position of 0.5 refers to the", "point between the", Z, "and the", te, "of the domain of the", "second", J, "axis."].join(" ");
            } };
          }, 20909: function(G) {
            G.exports = { INCREASING: { COLOR: "#3D9970", SYMBOL: "▲" }, DECREASING: { COLOR: "#FF4136", SYMBOL: "▼" } };
          }, 87296: function(G) {
            G.exports = { FORMAT_LINK: "https://github.com/d3/d3-format/tree/v1.4.5#d3-format", DATE_FORMAT_LINK: "https://github.com/d3/d3-time-format/tree/v2.2.3#locale_format" };
          }, 20726: function(G) {
            G.exports = { COMPARISON_OPS: ["=", "!=", "<", ">=", ">", "<="], COMPARISON_OPS2: ["=", "<", ">=", ">", "<="], INTERVAL_OPS: ["[]", "()", "[)", "(]", "][", ")(", "](", ")["], SET_OPS: ["{}", "}{"], CONSTRAINT_REDUCTION: { "=": "=", "<": "<", "<=": "<", ">": ">", ">=": ">", "[]": "[]", "()": "[]", "[)": "[]", "(]": "[]", "][": "][", ")(": "][", "](": "][", ")[": "][" } };
          }, 84770: function(G) {
            G.exports = { solid: [[], 0], dot: [[0.5, 1], 200], dash: [[0.5, 1], 50], longdash: [[0.5, 1], 10], dashdot: [[0.5, 0.625, 0.875, 1], 50], longdashdot: [[0.5, 0.7, 0.8, 1], 10] };
          }, 49467: function(G) {
            G.exports = { circle: "●", "circle-open": "○", square: "■", "square-open": "□", diamond: "◆", "diamond-open": "◇", cross: "+", x: "❌" };
          }, 20438: function(G) {
            G.exports = { SHOW_PLACEHOLDER: 100, HIDE_PLACEHOLDER: 1e3, DESELECTDIM: 0.2 };
          }, 63821: function(G) {
            G.exports = { BADNUM: void 0, FP_SAFE: 1e-4 * Number.MAX_VALUE, ONEMAXYEAR: 316224e5, ONEAVGYEAR: 315576e5, ONEMINYEAR: 31536e6, ONEMAXQUARTER: 79488e5, ONEAVGQUARTER: 78894e5, ONEMINQUARTER: 76896e5, ONEMAXMONTH: 26784e5, ONEAVGMONTH: 26298e5, ONEMINMONTH: 24192e5, ONEWEEK: 6048e5, ONEDAY: 864e5, ONEHOUR: 36e5, ONEMIN: 6e4, ONESEC: 1e3, ONEMILLI: 1, ONEMICROSEC: 1e-3, EPOCHJD: 24405875e-1, ALMOST_EQUAL: 0.999999, LOG_CLIP: 10, MINUS_SIGN: "−" };
          }, 1837: function(G, J) {
            J.CSS_DECLARATIONS = [["image-rendering", "optimizeSpeed"], ["image-rendering", "-moz-crisp-edges"], ["image-rendering", "-o-crisp-edges"], ["image-rendering", "-webkit-optimize-contrast"], ["image-rendering", "optimize-contrast"], ["image-rendering", "crisp-edges"], ["image-rendering", "pixelated"]], J.STYLE = J.CSS_DECLARATIONS.map(function(Z) {
              return Z.join(": ") + "; ";
            }).join("");
          }, 62972: function(G, J) {
            J.xmlns = "http://www.w3.org/2000/xmlns/", J.svg = "http://www.w3.org/2000/svg", J.xlink = "http://www.w3.org/1999/xlink", J.svgAttrs = { xmlns: J.svg, "xmlns:xlink": J.xlink };
          }, 17430: function(G, J, Z) {
            J.version = Z(29697).version, Z(71116), Z(6713);
            for (var te = Z(33626), ne = J.register = te.register, ee = Z(90742), ae = Object.keys(ee), oe = 0; oe < ae.length; oe++) {
              var se = ae[oe];
              se.charAt(0) !== "_" && (J[se] = ee[se]), ne({ moduleType: "apiMethod", name: se, fn: ee[se] });
            }
            ne(Z(69693)), ne([Z(3599), Z(83348), Z(44844), Z(43701), Z(15553), Z(46230), Z(15359), Z(55429), Z(44453), Z(83595), Z(77901), Z(88856), Z(96919), Z(82494), Z(32141), Z(95433)]), ne([Z(30227), Z(44611)]), window.PlotlyLocales && Array.isArray(window.PlotlyLocales) && (ne(window.PlotlyLocales), delete window.PlotlyLocales), J.Icons = Z(35188);
            var ce = Z(32141), le = Z(44122);
            J.Plots = { resize: le.resize, graphJson: le.graphJson, sendDataToCloud: le.sendDataToCloud }, J.Fx = { hover: ce.hover, unhover: ce.unhover, loneHover: ce.loneHover, loneUnhover: ce.loneUnhover }, J.Snapshot = Z(6170), J.PlotSchema = Z(57297);
          }, 35188: function(G) {
            G.exports = { undo: { width: 857.1, height: 1e3, path: "m857 350q0-87-34-166t-91-137-137-92-166-34q-96 0-183 41t-147 114q-4 6-4 13t5 11l76 77q6 5 14 5 9-1 13-7 41-53 100-82t126-29q58 0 110 23t92 61 61 91 22 111-22 111-61 91-92 61-110 23q-55 0-105-20t-90-57l77-77q17-16 8-38-10-23-33-23h-250q-15 0-25 11t-11 25v250q0 24 22 33 22 10 39-8l72-72q60 57 137 88t159 31q87 0 166-34t137-92 91-137 34-166z", transform: "matrix(1 0 0 -1 0 850)" }, home: { width: 928.6, height: 1e3, path: "m786 296v-267q0-15-11-26t-25-10h-214v214h-143v-214h-214q-15 0-25 10t-11 26v267q0 1 0 2t0 2l321 264 321-264q1-1 1-4z m124 39l-34-41q-5-5-12-6h-2q-7 0-12 3l-386 322-386-322q-7-4-13-4-7 2-12 7l-35 41q-4 5-3 13t6 12l401 334q18 15 42 15t43-15l136-114v109q0 8 5 13t13 5h107q8 0 13-5t5-13v-227l122-102q5-5 6-12t-4-13z", transform: "matrix(1 0 0 -1 0 850)" }, "camera-retro": { width: 1e3, height: 1e3, path: "m518 386q0 8-5 13t-13 5q-37 0-63-27t-26-63q0-8 5-13t13-5 12 5 5 13q0 23 16 38t38 16q8 0 13 5t5 13z m125-73q0-59-42-101t-101-42-101 42-42 101 42 101 101 42 101-42 42-101z m-572-320h858v71h-858v-71z m643 320q0 89-62 152t-152 62-151-62-63-152 63-151 151-63 152 63 62 151z m-571 358h214v72h-214v-72z m-72-107h858v143h-462l-36-71h-360v-72z m929 143v-714q0-30-21-51t-50-21h-858q-29 0-50 21t-21 51v714q0 30 21 51t50 21h858q29 0 50-21t21-51z", transform: "matrix(1 0 0 -1 0 850)" }, zoombox: { width: 1e3, height: 1e3, path: "m1000-25l-250 251c40 63 63 138 63 218 0 224-182 406-407 406-224 0-406-182-406-406s183-406 407-406c80 0 155 22 218 62l250-250 125 125z m-812 250l0 438 437 0 0-438-437 0z m62 375l313 0 0-312-313 0 0 312z", transform: "matrix(1 0 0 -1 0 850)" }, pan: { width: 1e3, height: 1e3, path: "m1000 350l-187 188 0-125-250 0 0 250 125 0-188 187-187-187 125 0 0-250-250 0 0 125-188-188 186-187 0 125 252 0 0-250-125 0 187-188 188 188-125 0 0 250 250 0 0-126 187 188z", transform: "matrix(1 0 0 -1 0 850)" }, zoom_plus: { width: 875, height: 1e3, path: "m1 787l0-875 875 0 0 875-875 0z m687-500l-187 0 0-187-125 0 0 187-188 0 0 125 188 0 0 187 125 0 0-187 187 0 0-125z", transform: "matrix(1 0 0 -1 0 850)" }, zoom_minus: { width: 875, height: 1e3, path: "m0 788l0-876 875 0 0 876-875 0z m688-500l-500 0 0 125 500 0 0-125z", transform: "matrix(1 0 0 -1 0 850)" }, autoscale: { width: 1e3, height: 1e3, path: "m250 850l-187 0-63 0 0-62 0-188 63 0 0 188 187 0 0 62z m688 0l-188 0 0-62 188 0 0-188 62 0 0 188 0 62-62 0z m-875-938l0 188-63 0 0-188 0-62 63 0 187 0 0 62-187 0z m875 188l0-188-188 0 0-62 188 0 62 0 0 62 0 188-62 0z m-125 188l-1 0-93-94-156 156 156 156 92-93 2 0 0 250-250 0 0-2 93-92-156-156-156 156 94 92 0 2-250 0 0-250 0 0 93 93 157-156-157-156-93 94 0 0 0-250 250 0 0 0-94 93 156 157 156-157-93-93 0 0 250 0 0 250z", transform: "matrix(1 0 0 -1 0 850)" }, tooltip_basic: { width: 1500, height: 1e3, path: "m375 725l0 0-375-375 375-374 0-1 1125 0 0 750-1125 0z", transform: "matrix(1 0 0 -1 0 850)" }, tooltip_compare: { width: 1125, height: 1e3, path: "m187 786l0 2-187-188 188-187 0 0 937 0 0 373-938 0z m0-499l0 1-187-188 188-188 0 0 937 0 0 376-938-1z", transform: "matrix(1 0 0 -1 0 850)" }, plotlylogo: { width: 1542, height: 1e3, path: "m0-10h182v-140h-182v140z m228 146h183v-286h-183v286z m225 714h182v-1000h-182v1000z m225-285h182v-715h-182v715z m225 142h183v-857h-183v857z m231-428h182v-429h-182v429z m225-291h183v-138h-183v138z", transform: "matrix(1 0 0 -1 0 850)" }, "z-axis": { width: 1e3, height: 1e3, path: "m833 5l-17 108v41l-130-65 130-66c0 0 0 38 0 39 0-1 36-14 39-25 4-15-6-22-16-30-15-12-39-16-56-20-90-22-187-23-279-23-261 0-341 34-353 59 3 60 228 110 228 110-140-8-351-35-351-116 0-120 293-142 474-142 155 0 477 22 477 142 0 50-74 79-163 96z m-374 94c-58-5-99-21-99-40 0-24 65-43 144-43 79 0 143 19 143 43 0 19-42 34-98 40v216h87l-132 135-133-135h88v-216z m167 515h-136v1c16 16 31 34 46 52l84 109v54h-230v-71h124v-1c-16-17-28-32-44-51l-89-114v-51h245v72z", transform: "matrix(1 0 0 -1 0 850)" }, "3d_rotate": { width: 1e3, height: 1e3, path: "m922 660c-5 4-9 7-14 11-359 263-580-31-580-31l-102 28 58-400c0 1 1 1 2 2 118 108 351 249 351 249s-62 27-100 42c88 83 222 183 347 122 16-8 30-17 44-27-2 1-4 2-6 4z m36-329c0 0 64 229-88 296-62 27-124 14-175-11 157-78 225-208 249-266 8-19 11-31 11-31 2 5 6 15 11 32-5-13-8-20-8-20z m-775-239c70-31 117-50 198-32-121 80-199 346-199 346l-96-15-58-12c0 0 55-226 155-287z m603 133l-317-139c0 0 4-4 19-14 7-5 24-15 24-15s-177-147-389 4c235-287 536-112 536-112l31-22 100 299-4-1z m-298-153c6-4 14-9 24-15 0 0-17 10-24 15z", transform: "matrix(1 0 0 -1 0 850)" }, camera: { width: 1e3, height: 1e3, path: "m500 450c-83 0-150-67-150-150 0-83 67-150 150-150 83 0 150 67 150 150 0 83-67 150-150 150z m400 150h-120c-16 0-34 13-39 29l-31 93c-6 15-23 28-40 28h-340c-16 0-34-13-39-28l-31-94c-6-15-23-28-40-28h-120c-55 0-100-45-100-100v-450c0-55 45-100 100-100h800c55 0 100 45 100 100v450c0 55-45 100-100 100z m-400-550c-138 0-250 112-250 250 0 138 112 250 250 250 138 0 250-112 250-250 0-138-112-250-250-250z m365 380c-19 0-35 16-35 35 0 19 16 35 35 35 19 0 35-16 35-35 0-19-16-35-35-35z", transform: "matrix(1 0 0 -1 0 850)" }, movie: { width: 1e3, height: 1e3, path: "m938 413l-188-125c0 37-17 71-44 94 64 38 107 107 107 187 0 121-98 219-219 219-121 0-219-98-219-219 0-61 25-117 66-156h-115c30 33 49 76 49 125 0 103-84 187-187 187s-188-84-188-187c0-57 26-107 65-141-38-22-65-62-65-109v-250c0-70 56-126 125-126h500c69 0 125 56 125 126l188-126c34 0 62 28 62 63v375c0 35-28 63-62 63z m-750 0c-69 0-125 56-125 125s56 125 125 125 125-56 125-125-56-125-125-125z m406-1c-87 0-157 70-157 157 0 86 70 156 157 156s156-70 156-156-70-157-156-157z", transform: "matrix(1 0 0 -1 0 850)" }, question: { width: 857.1, height: 1e3, path: "m500 82v107q0 8-5 13t-13 5h-107q-8 0-13-5t-5-13v-107q0-8 5-13t13-5h107q8 0 13 5t5 13z m143 375q0 49-31 91t-77 65-95 23q-136 0-207-119-9-14 4-24l74-55q4-4 10-4 9 0 14 7 30 38 48 51 19 14 48 14 27 0 48-15t21-33q0-21-11-34t-38-25q-35-16-65-48t-29-70v-20q0-8 5-13t13-5h107q8 0 13 5t5 13q0 10 12 27t30 28q18 10 28 16t25 19 25 27 16 34 7 45z m214-107q0-117-57-215t-156-156-215-58-216 58-155 156-58 215 58 215 155 156 216 58 215-58 156-156 57-215z", transform: "matrix(1 0 0 -1 0 850)" }, disk: { width: 857.1, height: 1e3, path: "m214-7h429v214h-429v-214z m500 0h72v500q0 8-6 21t-11 20l-157 156q-5 6-19 12t-22 5v-232q0-22-15-38t-38-16h-322q-22 0-37 16t-16 38v232h-72v-714h72v232q0 22 16 38t37 16h465q22 0 38-16t15-38v-232z m-214 518v178q0 8-5 13t-13 5h-107q-7 0-13-5t-5-13v-178q0-8 5-13t13-5h107q7 0 13 5t5 13z m357-18v-518q0-22-15-38t-38-16h-750q-23 0-38 16t-16 38v750q0 22 16 38t38 16h517q23 0 50-12t42-26l156-157q16-15 27-42t11-49z", transform: "matrix(1 0 0 -1 0 850)" }, drawopenpath: { width: 70, height: 70, path: "M33.21,85.65a7.31,7.31,0,0,1-2.59-.48c-8.16-3.11-9.27-19.8-9.88-41.3-.1-3.58-.19-6.68-.35-9-.15-2.1-.67-3.48-1.43-3.79-2.13-.88-7.91,2.32-12,5.86L3,32.38c1.87-1.64,11.55-9.66,18.27-6.9,2.13.87,4.75,3.14,5.17,9,.17,2.43.26,5.59.36,9.25a224.17,224.17,0,0,0,1.5,23.4c1.54,10.76,4,12.22,4.48,12.4.84.32,2.79-.46,5.76-3.59L43,80.07C41.53,81.57,37.68,85.64,33.21,85.65ZM74.81,69a11.34,11.34,0,0,0,6.09-6.72L87.26,44.5,74.72,32,56.9,38.35c-2.37.86-5.57,3.42-6.61,6L38.65,72.14l8.42,8.43ZM55,46.27a7.91,7.91,0,0,1,3.64-3.17l14.8-5.3,8,8L76.11,60.6l-.06.19a6.37,6.37,0,0,1-3,3.43L48.25,74.59,44.62,71Zm16.57,7.82A6.9,6.9,0,1,0,64.64,61,6.91,6.91,0,0,0,71.54,54.09Zm-4.05,0a2.85,2.85,0,1,1-2.85-2.85A2.86,2.86,0,0,1,67.49,54.09Zm-4.13,5.22L60.5,56.45,44.26,72.7l2.86,2.86ZM97.83,35.67,84.14,22l-8.57,8.57L89.26,44.24Zm-13.69-8,8,8-2.85,2.85-8-8Z", transform: "matrix(1 0 0 1 -15 -15)" }, drawclosedpath: { width: 90, height: 90, path: "M88.41,21.12a26.56,26.56,0,0,0-36.18,0l-2.07,2-2.07-2a26.57,26.57,0,0,0-36.18,0,23.74,23.74,0,0,0,0,34.8L48,90.12a3.22,3.22,0,0,0,4.42,0l36-34.21a23.73,23.73,0,0,0,0-34.79ZM84,51.24,50.16,83.35,16.35,51.25a17.28,17.28,0,0,1,0-25.47,20,20,0,0,1,27.3,0l4.29,4.07a3.23,3.23,0,0,0,4.44,0l4.29-4.07a20,20,0,0,1,27.3,0,17.27,17.27,0,0,1,0,25.46ZM66.76,47.68h-33v6.91h33ZM53.35,35H46.44V68h6.91Z", transform: "matrix(1 0 0 1 -5 -5)" }, lasso: { width: 1031, height: 1e3, path: "m1018 538c-36 207-290 336-568 286-277-48-473-256-436-463 10-57 36-108 76-151-13-66 11-137 68-183 34-28 75-41 114-42l-55-70 0 0c-2-1-3-2-4-3-10-14-8-34 5-45 14-11 34-8 45 4 1 1 2 3 2 5l0 0 113 140c16 11 31 24 45 40 4 3 6 7 8 11 48-3 100 0 151 9 278 48 473 255 436 462z m-624-379c-80 14-149 48-197 96 42 42 109 47 156 9 33-26 47-66 41-105z m-187-74c-19 16-33 37-39 60 50-32 109-55 174-68-42-25-95-24-135 8z m360 75c-34-7-69-9-102-8 8 62-16 128-68 170-73 59-175 54-244-5-9 20-16 40-20 61-28 159 121 317 333 354s407-60 434-217c28-159-121-318-333-355z", transform: "matrix(1 0 0 -1 0 850)" }, selectbox: { width: 1e3, height: 1e3, path: "m0 850l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m285 0l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m-857-286l0-143 143 0 0 143-143 0z m857 0l0-143 143 0 0 143-143 0z m-857-285l0-143 143 0 0 143-143 0z m857 0l0-143 143 0 0 143-143 0z m-857-286l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z m285 0l0-143 143 0 0 143-143 0z m286 0l0-143 143 0 0 143-143 0z", transform: "matrix(1 0 0 -1 0 850)" }, drawline: { width: 70, height: 70, path: "M60.64,62.3a11.29,11.29,0,0,0,6.09-6.72l6.35-17.72L60.54,25.31l-17.82,6.4c-2.36.86-5.57,3.41-6.6,6L24.48,65.5l8.42,8.42ZM40.79,39.63a7.89,7.89,0,0,1,3.65-3.17l14.79-5.31,8,8L61.94,54l-.06.19a6.44,6.44,0,0,1-3,3.43L34.07,68l-3.62-3.63Zm16.57,7.81a6.9,6.9,0,1,0-6.89,6.9A6.9,6.9,0,0,0,57.36,47.44Zm-4,0a2.86,2.86,0,1,1-2.85-2.85A2.86,2.86,0,0,1,53.32,47.44Zm-4.13,5.22L46.33,49.8,30.08,66.05l2.86,2.86ZM83.65,29,70,15.34,61.4,23.9,75.09,37.59ZM70,21.06l8,8-2.84,2.85-8-8ZM87,80.49H10.67V87H87Z", transform: "matrix(1 0 0 1 -15 -15)" }, drawrect: { width: 80, height: 80, path: "M78,22V79H21V22H78m9-9H12V88H87V13ZM68,46.22H31V54H68ZM53,32H45.22V69H53Z", transform: "matrix(1 0 0 1 -10 -10)" }, drawcircle: { width: 80, height: 80, path: "M50,84.72C26.84,84.72,8,69.28,8,50.3S26.84,15.87,50,15.87,92,31.31,92,50.3,73.16,84.72,50,84.72Zm0-60.59c-18.6,0-33.74,11.74-33.74,26.17S31.4,76.46,50,76.46,83.74,64.72,83.74,50.3,68.6,24.13,50,24.13Zm17.15,22h-34v7.11h34Zm-13.8-13H46.24v34h7.11Z", transform: "matrix(1 0 0 1 -10 -10)" }, eraseshape: { width: 80, height: 80, path: "M82.77,78H31.85L6,49.57,31.85,21.14H82.77a8.72,8.72,0,0,1,8.65,8.77V69.24A8.72,8.72,0,0,1,82.77,78ZM35.46,69.84H82.77a.57.57,0,0,0,.49-.6V29.91a.57.57,0,0,0-.49-.61H35.46L17,49.57Zm32.68-34.7-24,24,5,5,24-24Zm-19,.53-5,5,24,24,5-5Z", transform: "matrix(1 0 0 1 -10 -10)" }, spikeline: { width: 1e3, height: 1e3, path: "M512 409c0-57-46-104-103-104-57 0-104 47-104 104 0 57 47 103 104 103 57 0 103-46 103-103z m-327-39l92 0 0 92-92 0z m-185 0l92 0 0 92-92 0z m370-186l92 0 0 93-92 0z m0-184l92 0 0 92-92 0z", transform: "matrix(1.5 0 0 -1.5 0 850)" }, pencil: { width: 1792, height: 1792, path: "M491 1536l91-91-235-235-91 91v107h128v128h107zm523-928q0-22-22-22-10 0-17 7l-542 542q-7 7-7 17 0 22 22 22 10 0 17-7l542-542q7-7 7-17zm-54-192l416 416-832 832h-416v-416zm683 96q0 53-37 90l-166 166-416-416 166-165q36-38 90-38 53 0 91 38l235 234q37 39 37 91z", transform: "matrix(1 0 0 1 0 1)" }, newplotlylogo: { name: "newplotlylogo", svg: ["<svg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 132 132'>", "<defs>", " <style>", "  .cls-0{fill:#000;}", "  .cls-1{fill:#FFF;}", "  .cls-2{fill:#F26;}", "  .cls-3{fill:#D69;}", "  .cls-4{fill:#BAC;}", "  .cls-5{fill:#9EF;}", " </style>", "</defs>", " <title>plotly-logomark</title>", " <g id='symbol'>", "  <rect class='cls-0' x='0' y='0' width='132' height='132' rx='18' ry='18'/>", "  <circle class='cls-5' cx='102' cy='30' r='6'/>", "  <circle class='cls-4' cx='78' cy='30' r='6'/>", "  <circle class='cls-4' cx='78' cy='54' r='6'/>", "  <circle class='cls-3' cx='54' cy='30' r='6'/>", "  <circle class='cls-2' cx='30' cy='30' r='6'/>", "  <circle class='cls-2' cx='30' cy='54' r='6'/>", "  <path class='cls-1' d='M30,72a6,6,0,0,0-6,6v24a6,6,0,0,0,12,0V78A6,6,0,0,0,30,72Z'/>", "  <path class='cls-1' d='M78,72a6,6,0,0,0-6,6v24a6,6,0,0,0,12,0V78A6,6,0,0,0,78,72Z'/>", "  <path class='cls-1' d='M54,48a6,6,0,0,0-6,6v48a6,6,0,0,0,12,0V54A6,6,0,0,0,54,48Z'/>", "  <path class='cls-1' d='M102,48a6,6,0,0,0-6,6v48a6,6,0,0,0,12,0V54A6,6,0,0,0,102,48Z'/>", " </g>", "</svg>"].join("") } };
          }, 32546: function(G, J) {
            J.isLeftAnchor = function(Z) {
              return Z.xanchor === "left" || Z.xanchor === "auto" && Z.x <= 0.3333333333333333;
            }, J.isCenterAnchor = function(Z) {
              return Z.xanchor === "center" || Z.xanchor === "auto" && Z.x > 0.3333333333333333 && Z.x < 0.6666666666666666;
            }, J.isRightAnchor = function(Z) {
              return Z.xanchor === "right" || Z.xanchor === "auto" && Z.x >= 0.6666666666666666;
            }, J.isTopAnchor = function(Z) {
              return Z.yanchor === "top" || Z.yanchor === "auto" && Z.y >= 0.6666666666666666;
            }, J.isMiddleAnchor = function(Z) {
              return Z.yanchor === "middle" || Z.yanchor === "auto" && Z.y > 0.3333333333333333 && Z.y < 0.6666666666666666;
            }, J.isBottomAnchor = function(Z) {
              return Z.yanchor === "bottom" || Z.yanchor === "auto" && Z.y <= 0.3333333333333333;
            };
          }, 44313: function(G, J, Z) {
            var te = Z(98953), ne = te.mod, ee = te.modHalf, ae = Math.PI, oe = 2 * ae;
            function se(de) {
              return Math.abs(de[1] - de[0]) > oe - 1e-14;
            }
            function ce(de, fe) {
              return ee(fe - de, oe);
            }
            function le(de, fe) {
              if (se(fe)) return !0;
              var he, me;
              fe[0] < fe[1] ? (he = fe[0], me = fe[1]) : (he = fe[1], me = fe[0]), (he = ne(he, oe)) > (me = ne(me, oe)) && (me += oe);
              var pe = ne(de, oe), ye = pe + oe;
              return pe >= he && pe <= me || ye >= he && ye <= me;
            }
            function ue(de, fe, he, me, pe, ye, ge) {
              pe = pe || 0, ye = ye || 0;
              var ve, be, Ce, Se, we, Ee = se([he, me]);
              function xe(Me, Re) {
                return [Me * Math.cos(Re) + pe, ye - Me * Math.sin(Re)];
              }
              Ee ? (ve = 0, be = ae, Ce = oe) : he < me ? (ve = he, Ce = me) : (ve = me, Ce = he), de < fe ? (Se = de, we = fe) : (Se = fe, we = de);
              var Le, Qe = Math.abs(Ce - ve) <= ae ? 0 : 1;
              function ke(Me, Re, De) {
                return "A" + [Me, Me] + " " + [0, Qe, De] + " " + xe(Me, Re);
              }
              return Ee ? Le = Se === null ? "M" + xe(we, ve) + ke(we, be, 0) + ke(we, Ce, 0) + "Z" : "M" + xe(Se, ve) + ke(Se, be, 0) + ke(Se, Ce, 0) + "ZM" + xe(we, ve) + ke(we, be, 1) + ke(we, Ce, 1) + "Z" : Se === null ? (Le = "M" + xe(we, ve) + ke(we, Ce, 0), ge && (Le += "L0,0Z")) : Le = "M" + xe(Se, ve) + "L" + xe(we, ve) + ke(we, Ce, 0) + "L" + xe(Se, Ce) + ke(Se, ve, 1) + "Z", Le;
            }
            G.exports = { deg2rad: function(de) {
              return de / 180 * ae;
            }, rad2deg: function(de) {
              return de / ae * 180;
            }, angleDelta: ce, angleDist: function(de, fe) {
              return Math.abs(ce(de, fe));
            }, isFullCircle: se, isAngleInsideSector: le, isPtInsideSector: function(de, fe, he, me) {
              return !!le(fe, me) && (he[0] < he[1] ? (pe = he[0], ye = he[1]) : (pe = he[1], ye = he[0]), de >= pe && de <= ye);
              var pe, ye;
            }, pathArc: function(de, fe, he, me, pe) {
              return ue(null, de, fe, he, me, pe, 0);
            }, pathSector: function(de, fe, he, me, pe) {
              return ue(null, de, fe, he, me, pe, 1);
            }, pathAnnulus: function(de, fe, he, me, pe, ye) {
              return ue(de, fe, he, me, pe, ye, 1);
            } };
          }, 87800: function(G, J, Z) {
            var te = Z(93229).decode, ne = Z(56174), ee = Array.isArray, ae = ArrayBuffer, oe = DataView;
            function se(fe) {
              return ae.isView(fe) && !(fe instanceof oe);
            }
            function ce(fe) {
              return ee(fe) || se(fe);
            }
            J.isTypedArray = se, J.isArrayOrTypedArray = ce, J.isArray1D = function(fe) {
              return !ce(fe[0]);
            }, J.ensureArray = function(fe, he) {
              return ee(fe) || (fe = []), fe.length = he, fe;
            };
            var le = { u1c: typeof Uint8ClampedArray > "u" ? void 0 : Uint8ClampedArray, i1: typeof Int8Array > "u" ? void 0 : Int8Array, u1: typeof Uint8Array > "u" ? void 0 : Uint8Array, i2: typeof Int16Array > "u" ? void 0 : Int16Array, u2: typeof Uint16Array > "u" ? void 0 : Uint16Array, i4: typeof Int32Array > "u" ? void 0 : Int32Array, u4: typeof Uint32Array > "u" ? void 0 : Uint32Array, f4: typeof Float32Array > "u" ? void 0 : Float32Array, f8: typeof Float64Array > "u" ? void 0 : Float64Array };
            function ue(fe) {
              return fe.constructor === ArrayBuffer;
            }
            function de(fe, he, me) {
              if (ce(fe)) {
                if (ce(fe[0])) {
                  for (var pe = me, ye = 0; ye < fe.length; ye++) pe = he(pe, fe[ye].length);
                  return pe;
                }
                return fe.length;
              }
              return 0;
            }
            le.uint8c = le.u1c, le.uint8 = le.u1, le.int8 = le.i1, le.uint16 = le.u2, le.int16 = le.i2, le.uint32 = le.u4, le.int32 = le.i4, le.float32 = le.f4, le.float64 = le.f8, J.isArrayBuffer = ue, J.decodeTypedArraySpec = function(fe) {
              var he = [], me = function(Me) {
                return { bdata: Me.bdata, dtype: Me.dtype, shape: Me.shape };
              }(fe), pe = me.dtype, ye = le[pe];
              if (!ye) throw new Error('Error in dtype: "' + pe + '"');
              var ge = ye.BYTES_PER_ELEMENT, ve = me.bdata;
              ue(ve) || (ve = te(ve));
              var be = me.shape === void 0 ? [ve.byteLength / ge] : ("" + me.shape).split(",");
              be.reverse();
              var Ce, Se, we = be.length, Ee = +be[0], xe = ge * Ee, Le = 0;
              if (we === 1) he = new ye(ve);
              else if (we === 2) for (Ce = +be[1], Se = 0; Se < Ce; Se++) he[Se] = new ye(ve, Le, Ee), Le += xe;
              else {
                if (we !== 3) throw new Error("ndim: " + we + 'is not supported with the shape:"' + me.shape + '"');
                Ce = +be[1];
                for (var Qe = +be[2], ke = 0; ke < Qe; ke++) for (he[ke] = [], Se = 0; Se < Ce; Se++) he[ke][Se] = new ye(ve, Le, Ee), Le += xe;
              }
              return he.bdata = me.bdata, he.dtype = me.dtype, he.shape = be.reverse().join(","), fe._inputArray = he, he;
            }, J.isTypedArraySpec = function(fe) {
              return ne(fe) && fe.hasOwnProperty("dtype") && typeof fe.dtype == "string" && fe.hasOwnProperty("bdata") && (typeof fe.bdata == "string" || ue(fe.bdata)) && (fe.shape === void 0 || fe.hasOwnProperty("shape") && (typeof fe.shape == "string" || typeof fe.shape == "number"));
            }, J.concat = function() {
              var fe, he, me, pe, ye, ge, ve, be, Ce = [], Se = !0, we = 0;
              for (me = 0; me < arguments.length; me++) (ge = (pe = arguments[me]).length) && (he ? Ce.push(pe) : (he = pe, ye = ge), ee(pe) ? fe = !1 : (Se = !1, we ? fe !== pe.constructor && (fe = !1) : fe = pe.constructor), we += ge);
              if (!we) return [];
              if (!Ce.length) return he;
              if (Se) return he.concat.apply(he, Ce);
              if (fe) {
                for ((ve = new fe(we)).set(he), me = 0; me < Ce.length; me++) pe = Ce[me], ve.set(pe, ye), ye += pe.length;
                return ve;
              }
              for (ve = new Array(we), be = 0; be < he.length; be++) ve[be] = he[be];
              for (me = 0; me < Ce.length; me++) {
                for (pe = Ce[me], be = 0; be < pe.length; be++) ve[ye + be] = pe[be];
                ye += be;
              }
              return ve;
            }, J.maxRowLength = function(fe) {
              return de(fe, Math.max, 0);
            }, J.minRowLength = function(fe) {
              return de(fe, Math.min, 1 / 0);
            };
          }, 44498: function(G, J, Z) {
            var te = Z(10721), ne = Z(63821).BADNUM, ee = /^['"%,$#\s']+|[, ]|['"%,$#\s']+$/g;
            G.exports = function(ae) {
              return typeof ae == "string" && (ae = ae.replace(ee, "")), te(ae) ? Number(ae) : ne;
            };
          }, 34823: function(G) {
            G.exports = function(J) {
              var Z = J._fullLayout;
              Z._glcanvas && Z._glcanvas.size() && Z._glcanvas.each(function(te) {
                te.regl && te.regl.clear({ color: !0, depth: !0 });
              });
            };
          }, 23493: function(G) {
            G.exports = function(J) {
              J._responsiveChartHandler && (window.removeEventListener("resize", J._responsiveChartHandler), delete J._responsiveChartHandler);
            };
          }, 34220: function(G, J, Z) {
            var te = Z(10721), ne = Z(65657), ee = Z(93049).extendFlat, ae = Z(9829), oe = Z(19017), se = Z(78766), ce = Z(20438).DESELECTDIM, le = Z(35632), ue = Z(90694).counter, de = Z(98953).modHalf, fe = Z(87800).isArrayOrTypedArray, he = Z(87800).isTypedArraySpec, me = Z(87800).decodeTypedArraySpec;
            function pe(ye, ge) {
              var ve = J.valObjectMeta[ge.valType];
              if (ge.arrayOk && fe(ye)) return !0;
              if (ve.validateFunction) return ve.validateFunction(ye, ge);
              var be = {}, Ce = be, Se = { set: function(we) {
                Ce = we;
              } };
              return ve.coerceFunction(ye, Se, be, ge), Ce !== be;
            }
            J.valObjectMeta = { data_array: { coerceFunction: function(ye, ge, ve) {
              ge.set(fe(ye) ? ye : he(ye) ? me(ye) : ve);
            } }, enumerated: { coerceFunction: function(ye, ge, ve, be) {
              be.coerceNumber && (ye = +ye), be.values.indexOf(ye) === -1 ? ge.set(ve) : ge.set(ye);
            }, validateFunction: function(ye, ge) {
              ge.coerceNumber && (ye = +ye);
              for (var ve = ge.values, be = 0; be < ve.length; be++) {
                var Ce = String(ve[be]);
                if (Ce.charAt(0) === "/" && Ce.charAt(Ce.length - 1) === "/") {
                  if (new RegExp(Ce.substr(1, Ce.length - 2)).test(ye)) return !0;
                } else if (ye === ve[be]) return !0;
              }
              return !1;
            } }, boolean: { coerceFunction: function(ye, ge, ve) {
              ye === !0 || ye === !1 ? ge.set(ye) : ge.set(ve);
            } }, number: { coerceFunction: function(ye, ge, ve, be) {
              he(ye) && (ye = me(ye)), !te(ye) || be.min !== void 0 && ye < be.min || be.max !== void 0 && ye > be.max ? ge.set(ve) : ge.set(+ye);
            } }, integer: { coerceFunction: function(ye, ge, ve, be) {
              (be.extras || []).indexOf(ye) === -1 ? (he(ye) && (ye = me(ye)), ye % 1 || !te(ye) || be.min !== void 0 && ye < be.min || be.max !== void 0 && ye > be.max ? ge.set(ve) : ge.set(+ye)) : ge.set(ye);
            } }, string: { coerceFunction: function(ye, ge, ve, be) {
              if (typeof ye != "string") {
                var Ce = typeof ye == "number";
                be.strict !== !0 && Ce ? ge.set(String(ye)) : ge.set(ve);
              } else be.noBlank && !ye ? ge.set(ve) : ge.set(ye);
            } }, color: { coerceFunction: function(ye, ge, ve) {
              he(ye) && (ye = me(ye)), ne(ye).isValid() ? ge.set(ye) : ge.set(ve);
            } }, colorlist: { coerceFunction: function(ye, ge, ve) {
              Array.isArray(ye) && ye.length && ye.every(function(be) {
                return ne(be).isValid();
              }) ? ge.set(ye) : ge.set(ve);
            } }, colorscale: { coerceFunction: function(ye, ge, ve) {
              ge.set(oe.get(ye, ve));
            } }, angle: { coerceFunction: function(ye, ge, ve) {
              he(ye) && (ye = me(ye)), ye === "auto" ? ge.set("auto") : te(ye) ? ge.set(de(+ye, 360)) : ge.set(ve);
            } }, subplotid: { coerceFunction: function(ye, ge, ve, be) {
              var Ce = be.regex || ue(ve);
              typeof ye == "string" && Ce.test(ye) ? ge.set(ye) : ge.set(ve);
            }, validateFunction: function(ye, ge) {
              var ve = ge.dflt;
              return ye === ve || typeof ye == "string" && !!ue(ve).test(ye);
            } }, flaglist: { coerceFunction: function(ye, ge, ve, be) {
              if ((be.extras || []).indexOf(ye) === -1) if (typeof ye == "string") {
                for (var Ce = ye.split("+"), Se = 0; Se < Ce.length; ) {
                  var we = Ce[Se];
                  be.flags.indexOf(we) === -1 || Ce.indexOf(we) < Se ? Ce.splice(Se, 1) : Se++;
                }
                Ce.length ? ge.set(Ce.join("+")) : ge.set(ve);
              } else ge.set(ve);
              else ge.set(ye);
            } }, any: { coerceFunction: function(ye, ge, ve) {
              ye === void 0 ? ge.set(ve) : ge.set(he(ye) ? me(ye) : ye);
            } }, info_array: { coerceFunction: function(ye, ge, ve, be) {
              function Ce(Xe, Ve, Ze) {
                var Ge, ze = { set: function(We) {
                  Ge = We;
                } };
                return Ze === void 0 && (Ze = Ve.dflt), J.valObjectMeta[Ve.valType].coerceFunction(Xe, ze, Ze, Ve), Ge;
              }
              if (he(ye) && (ye = me(ye)), fe(ye)) {
                var Se, we, Ee, xe, Le, Qe, ke = be.dimensions === 2 || be.dimensions === "1-2" && Array.isArray(ye) && fe(ye[0]), Me = be.items, Re = [], De = Array.isArray(Me), Pe = De && ke && fe(Me[0]), $e = ke && De && !Pe, qe = De && !$e ? Me.length : ye.length;
                if (ve = Array.isArray(ve) ? ve : [], ke) for (Se = 0; Se < qe; Se++) for (Re[Se] = [], Ee = fe(ye[Se]) ? ye[Se] : [], Le = $e ? Me.length : De ? Me[Se].length : Ee.length, we = 0; we < Le; we++) xe = $e ? Me[we] : De ? Me[Se][we] : Me, (Qe = Ce(Ee[we], xe, (ve[Se] || [])[we])) !== void 0 && (Re[Se][we] = Qe);
                else for (Se = 0; Se < qe; Se++) (Qe = Ce(ye[Se], De ? Me[Se] : Me, ve[Se])) !== void 0 && (Re[Se] = Qe);
                ge.set(Re);
              } else ge.set(ve);
            }, validateFunction: function(ye, ge) {
              if (!fe(ye)) return !1;
              var ve = ge.items, be = Array.isArray(ve), Ce = ge.dimensions === 2;
              if (!ge.freeLength && ye.length !== ve.length) return !1;
              for (var Se = 0; Se < ye.length; Se++) if (Ce) {
                if (!fe(ye[Se]) || !ge.freeLength && ye[Se].length !== ve[Se].length) return !1;
                for (var we = 0; we < ye[Se].length; we++) if (!pe(ye[Se][we], be ? ve[Se][we] : ve)) return !1;
              } else if (!pe(ye[Se], be ? ve[Se] : ve)) return !1;
              return !0;
            } } }, J.coerce = function(ye, ge, ve, be, Ce) {
              var Se = le(ve, be).get(), we = le(ye, be), Ee = le(ge, be), xe = we.get(), Le = ge._template;
              if (xe === void 0 && Le && (xe = le(Le, be).get(), Le = 0), Ce === void 0 && (Ce = Se.dflt), Se.arrayOk) {
                if (fe(xe)) return Ee.set(xe), xe;
                if (he(xe)) return xe = me(xe), Ee.set(xe), xe;
              }
              var Qe = J.valObjectMeta[Se.valType].coerceFunction;
              Qe(xe, Ee, Ce, Se);
              var ke = Ee.get();
              return Le && ke === Ce && !pe(xe, Se) && (Qe(xe = le(Le, be).get(), Ee, Ce, Se), ke = Ee.get()), ke;
            }, J.coerce2 = function(ye, ge, ve, be, Ce) {
              var Se = le(ye, be), we = J.coerce(ye, ge, ve, be, Ce);
              return Se.get() != null && we;
            }, J.coerceFont = function(ye, ge, ve, be) {
              be || (be = {}), ve = ee({}, ve);
              var Ce = { family: ye(ge + ".family", (ve = ee(ve, be.overrideDflt || {})).family), size: ye(ge + ".size", ve.size), color: ye(ge + ".color", ve.color), weight: ye(ge + ".weight", ve.weight), style: ye(ge + ".style", ve.style) };
              if (be.noFontVariant || (Ce.variant = ye(ge + ".variant", ve.variant)), be.noFontLineposition || (Ce.lineposition = ye(ge + ".lineposition", ve.lineposition)), be.noFontTextcase || (Ce.textcase = ye(ge + ".textcase", ve.textcase)), !be.noFontShadow) {
                var Se = ve.shadow;
                Se === "none" && be.autoShadowDflt && (Se = "auto"), Ce.shadow = ye(ge + ".shadow", Se);
              }
              return Ce;
            }, J.coercePattern = function(ye, ge, ve, be) {
              if (ye(ge + ".shape")) {
                ye(ge + ".solidity"), ye(ge + ".size");
                var Ce = ye(ge + ".fillmode") === "overlay";
                if (!be) {
                  var Se = ye(ge + ".bgcolor", Ce ? ve : void 0);
                  ye(ge + ".fgcolor", Ce ? se.contrast(Se) : ve);
                }
                ye(ge + ".fgopacity", Ce ? 0.5 : 1);
              }
            }, J.coerceHoverinfo = function(ye, ge, ve) {
              var be, Ce = ge._module.attributes, Se = Ce.hoverinfo ? Ce : ae, we = Se.hoverinfo;
              if (ve._dataLength === 1) {
                var Ee = we.dflt === "all" ? we.flags.slice() : we.dflt.split("+");
                Ee.splice(Ee.indexOf("name"), 1), be = Ee.join("+");
              }
              return J.coerce(ye, ge, Se, "hoverinfo", be);
            }, J.coerceSelectionMarkerOpacity = function(ye, ge) {
              if (ye.marker) {
                var ve, be, Ce = ye.marker.opacity;
                Ce !== void 0 && (fe(Ce) || ye.selected || ye.unselected || (ve = Ce, be = ce * Ce), ge("selected.marker.opacity", ve), ge("unselected.marker.opacity", be));
              }
            }, J.validate = pe;
          }, 92596: function(G, J, Z) {
            var te, ne, ee = Z(42696).DC, ae = Z(10721), oe = Z(48636), se = Z(98953).mod, ce = Z(63821), le = ce.BADNUM, ue = ce.ONEDAY, de = ce.ONEHOUR, fe = ce.ONEMIN, he = ce.ONESEC, me = ce.EPOCHJD, pe = Z(33626), ye = Z(42696).aL, ge = /^\s*(-?\d\d\d\d|\d\d)(-(\d?\d)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d(:?\d\d)?)?)?)?)?)?\s*$/m, ve = /^\s*(-?\d\d\d\d|\d\d)(-(\d?\di?)(-(\d?\d)([ Tt]([01]?\d|2[0-3])(:([0-5]\d)(:([0-5]\d(\.\d+)?))?(Z|z|[+\-]\d\d(:?\d\d)?)?)?)?)?)?\s*$/m, be = (/* @__PURE__ */ new Date()).getFullYear() - 70;
            function Ce($e) {
              return $e && pe.componentsRegistry.calendars && typeof $e == "string" && $e !== "gregorian";
            }
            function Se($e, qe) {
              return String($e + Math.pow(10, qe)).substr(1);
            }
            J.dateTick0 = function($e, qe) {
              var Xe = function(Ze, Ge) {
                return Ce(Ze) ? Ge ? pe.getComponentMethod("calendars", "CANONICAL_SUNDAY")[Ze] : pe.getComponentMethod("calendars", "CANONICAL_TICK")[Ze] : Ge ? "2000-01-02" : "2000-01-01";
              }($e, !!qe);
              if (qe < 2) return Xe;
              var Ve = J.dateTime2ms(Xe, $e);
              return Ve += ue * (qe - 1), J.ms2DateTime(Ve, 0, $e);
            }, J.dfltRange = function($e) {
              return Ce($e) ? pe.getComponentMethod("calendars", "DFLTRANGE")[$e] : ["2000-01-01", "2001-01-01"];
            }, J.isJSDate = function($e) {
              return typeof $e == "object" && $e !== null && typeof $e.getTime == "function";
            }, J.dateTime2ms = function($e, qe) {
              if (J.isJSDate($e)) {
                var Xe = $e.getTimezoneOffset() * fe, Ve = ($e.getUTCMinutes() - $e.getMinutes()) * fe + ($e.getUTCSeconds() - $e.getSeconds()) * he + ($e.getUTCMilliseconds() - $e.getMilliseconds());
                if (Ve) {
                  var Ze = 3 * fe;
                  Xe = Xe - Ze / 2 + se(Ve - Xe + Ze / 2, Ze);
                }
                return ($e = Number($e) - Xe) >= te && $e <= ne ? $e : le;
              }
              if (typeof $e != "string" && typeof $e != "number") return le;
              $e = String($e);
              var Ge = Ce(qe), ze = $e.charAt(0);
              !Ge || ze !== "G" && ze !== "g" || ($e = $e.substr(1), qe = "");
              var We = Ge && qe.substr(0, 7) === "chinese", at = $e.match(We ? ve : ge);
              if (!at) return le;
              var ct = at[1], et = at[3] || "1", dt = Number(at[5] || 1), Je = Number(at[7] || 0), it = Number(at[9] || 0), st = Number(at[11] || 0);
              if (Ge) {
                if (ct.length === 2) return le;
                var gt;
                ct = Number(ct);
                try {
                  var bt = pe.getComponentMethod("calendars", "getCal")(qe);
                  if (We) {
                    var yt = et.charAt(et.length - 1) === "i";
                    et = parseInt(et, 10), gt = bt.newDate(ct, bt.toMonthIndex(ct, et, yt), dt);
                  } else gt = bt.newDate(ct, Number(et), dt);
                } catch {
                  return le;
                }
                return gt ? (gt.toJD() - me) * ue + Je * de + it * fe + st * he : le;
              }
              ct = ct.length === 2 ? (Number(ct) + 2e3 - be) % 100 + be : Number(ct), et -= 1;
              var St = new Date(Date.UTC(2e3, et, dt, Je, it));
              return St.setUTCFullYear(ct), St.getUTCMonth() !== et || St.getUTCDate() !== dt ? le : St.getTime() + st * he;
            }, te = J.MIN_MS = J.dateTime2ms("-9999"), ne = J.MAX_MS = J.dateTime2ms("9999-12-31 23:59:59.9999"), J.isDateTime = function($e, qe) {
              return J.dateTime2ms($e, qe) !== le;
            };
            var we = 90 * ue, Ee = 3 * de, xe = 5 * fe;
            function Le($e, qe, Xe, Ve, Ze) {
              if ((qe || Xe || Ve || Ze) && ($e += " " + Se(qe, 2) + ":" + Se(Xe, 2), (Ve || Ze) && ($e += ":" + Se(Ve, 2), Ze))) {
                for (var Ge = 4; Ze % 10 == 0; ) Ge -= 1, Ze /= 10;
                $e += "." + Se(Ze, Ge);
              }
              return $e;
            }
            J.ms2DateTime = function($e, qe, Xe) {
              if (typeof $e != "number" || !($e >= te && $e <= ne)) return le;
              qe || (qe = 0);
              var Ve, Ze, Ge, ze, We, at, ct = Math.floor(10 * se($e + 0.05, 1)), et = Math.round($e - ct / 10);
              if (Ce(Xe)) {
                var dt = Math.floor(et / ue) + me, Je = Math.floor(se($e, ue));
                try {
                  Ve = pe.getComponentMethod("calendars", "getCal")(Xe).fromJD(dt).formatDate("yyyy-mm-dd");
                } catch {
                  Ve = ye("G%Y-%m-%d")(new Date(et));
                }
                if (Ve.charAt(0) === "-") for (; Ve.length < 11; ) Ve = "-0" + Ve.substr(1);
                else for (; Ve.length < 10; ) Ve = "0" + Ve;
                Ze = qe < we ? Math.floor(Je / de) : 0, Ge = qe < we ? Math.floor(Je % de / fe) : 0, ze = qe < Ee ? Math.floor(Je % fe / he) : 0, We = qe < xe ? Je % he * 10 + ct : 0;
              } else at = new Date(et), Ve = ye("%Y-%m-%d")(at), Ze = qe < we ? at.getUTCHours() : 0, Ge = qe < we ? at.getUTCMinutes() : 0, ze = qe < Ee ? at.getUTCSeconds() : 0, We = qe < xe ? 10 * at.getUTCMilliseconds() + ct : 0;
              return Le(Ve, Ze, Ge, ze, We);
            }, J.ms2DateTimeLocal = function($e) {
              if (!($e >= te + ue && $e <= ne - ue)) return le;
              var qe = Math.floor(10 * se($e + 0.05, 1)), Xe = new Date(Math.round($e - qe / 10));
              return Le(ee("%Y-%m-%d")(Xe), Xe.getHours(), Xe.getMinutes(), Xe.getSeconds(), 10 * Xe.getUTCMilliseconds() + qe);
            }, J.cleanDate = function($e, qe, Xe) {
              if ($e === le) return qe;
              if (J.isJSDate($e) || typeof $e == "number" && isFinite($e)) {
                if (Ce(Xe)) return oe.error("JS Dates and milliseconds are incompatible with world calendars", $e), qe;
                if (!($e = J.ms2DateTimeLocal(+$e)) && qe !== void 0) return qe;
              } else if (!J.isDateTime($e, Xe)) return oe.error("unrecognized date", $e), qe;
              return $e;
            };
            var Qe = /%\d?f/g, ke = /%h/g, Me = { 1: "1", 2: "1", 3: "2", 4: "2" };
            function Re($e, qe, Xe, Ve) {
              $e = $e.replace(Qe, function(Ge) {
                var ze = Math.min(+Ge.charAt(1) || 6, 6);
                return (qe / 1e3 % 1 + 2).toFixed(ze).substr(2).replace(/0+$/, "") || "0";
              });
              var Ze = new Date(Math.floor(qe + 0.05));
              if ($e = $e.replace(ke, function() {
                return Me[Xe("%q")(Ze)];
              }), Ce(Ve)) try {
                $e = pe.getComponentMethod("calendars", "worldCalFmt")($e, qe, Ve);
              } catch {
                return "Invalid";
              }
              return Xe($e)(Ze);
            }
            var De = [59, 59.9, 59.99, 59.999, 59.9999];
            J.formatDate = function($e, qe, Xe, Ve, Ze, Ge) {
              if (Ze = Ce(Ze) && Ze, !qe) if (Xe === "y") qe = Ge.year;
              else if (Xe === "m") qe = Ge.month;
              else {
                if (Xe !== "d") return function(ze, We) {
                  var at = se(ze + 0.05, ue), ct = Se(Math.floor(at / de), 2) + ":" + Se(se(Math.floor(at / fe), 60), 2);
                  if (We !== "M") {
                    ae(We) || (We = 0);
                    var et = (100 + Math.min(se(ze / he, 60), De[We])).toFixed(We).substr(1);
                    We > 0 && (et = et.replace(/0+$/, "").replace(/[\.]$/, "")), ct += ":" + et;
                  }
                  return ct;
                }($e, Xe) + `
` + Re(Ge.dayMonthYear, $e, Ve, Ze);
                qe = Ge.dayMonth + `
` + Ge.year;
              }
              return Re(qe, $e, Ve, Ze);
            };
            var Pe = 3 * ue;
            J.incrementMonth = function($e, qe, Xe) {
              Xe = Ce(Xe) && Xe;
              var Ve = se($e, ue);
              if ($e = Math.round($e - Ve), Xe) try {
                var Ze = Math.round($e / ue) + me, Ge = pe.getComponentMethod("calendars", "getCal")(Xe), ze = Ge.fromJD(Ze);
                return qe % 12 ? Ge.add(ze, qe, "m") : Ge.add(ze, qe / 12, "y"), (ze.toJD() - me) * ue + Ve;
              } catch {
                oe.error("invalid ms " + $e + " in calendar " + Xe);
              }
              var We = new Date($e + Pe);
              return We.setUTCMonth(We.getUTCMonth() + qe) + Ve - Pe;
            }, J.findExactDates = function($e, qe) {
              for (var Xe, Ve, Ze = 0, Ge = 0, ze = 0, We = 0, at = Ce(qe) && pe.getComponentMethod("calendars", "getCal")(qe), ct = 0; ct < $e.length; ct++) if (Ve = $e[ct], ae(Ve)) {
                if (!(Ve % ue)) if (at) try {
                  (Xe = at.fromJD(Ve / ue + me)).day() === 1 ? Xe.month() === 1 ? Ze++ : Ge++ : ze++;
                } catch {
                }
                else (Xe = new Date(Ve)).getUTCDate() === 1 ? Xe.getUTCMonth() === 0 ? Ze++ : Ge++ : ze++;
              } else We++;
              ze += Ge += Ze;
              var et = $e.length - We;
              return { exactYears: Ze / et, exactMonths: Ge / et, exactDays: ze / et };
            };
          }, 95425: function(G, J, Z) {
            var te = Z(45568), ne = Z(48636), ee = Z(15236), ae = Z(11191);
            function oe(de) {
              var fe = de && de.parentNode;
              fe && fe.removeChild(de);
            }
            function se(de, fe, he) {
              var me = "plotly.js-style-" + de, pe = document.getElementById(me);
              pe || ((pe = document.createElement("style")).setAttribute("id", me), pe.appendChild(document.createTextNode("")), document.head.appendChild(pe));
              var ye = pe.sheet;
              ye.insertRule ? ye.insertRule(fe + "{" + he + "}", 0) : ye.addRule ? ye.addRule(fe, he, 0) : ne.warn("addStyleRule failed");
            }
            function ce(de) {
              var fe = window.getComputedStyle(de, null), he = fe.getPropertyValue("-webkit-transform") || fe.getPropertyValue("-moz-transform") || fe.getPropertyValue("-ms-transform") || fe.getPropertyValue("-o-transform") || fe.getPropertyValue("transform");
              return he === "none" ? null : he.replace("matrix", "").replace("3d", "").slice(1, -1).split(",").map(function(me) {
                return +me;
              });
            }
            function le(de) {
              for (var fe = []; ue(de); ) fe.push(de), de = de.parentNode, typeof ShadowRoot == "function" && de instanceof ShadowRoot && (de = de.host);
              return fe;
            }
            function ue(de) {
              return de && (de instanceof Element || de instanceof HTMLElement);
            }
            G.exports = { getGraphDiv: function(de) {
              var fe;
              if (typeof de == "string") {
                if ((fe = document.getElementById(de)) === null) throw new Error("No DOM element with id '" + de + "' exists on the page.");
                return fe;
              }
              if (de == null) throw new Error("DOM element provided is null or undefined");
              return de;
            }, isPlotDiv: function(de) {
              var fe = te.select(de);
              return fe.node() instanceof HTMLElement && fe.size() && fe.classed("js-plotly-plot");
            }, removeElement: oe, addStyleRule: function(de, fe) {
              se("global", de, fe);
            }, addRelatedStyleRule: se, deleteRelatedStyleRule: function(de) {
              var fe = "plotly.js-style-" + de, he = document.getElementById(fe);
              he && oe(he);
            }, getFullTransformMatrix: function(de) {
              var fe = le(de), he = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
              return fe.forEach(function(me) {
                var pe = ce(me);
                if (pe) {
                  var ye = ee.convertCssMatrix(pe);
                  he = ae.multiply(he, he, ye);
                }
              }), he;
            }, getElementTransformMatrix: ce, getElementAndAncestors: le, equalDomRects: function(de, fe) {
              return de && fe && de.top === fe.top && de.left === fe.left && de.right === fe.right && de.bottom === fe.bottom;
            } };
          }, 68596: function(G, J, Z) {
            var te = Z(7683).EventEmitter, ne = { init: function(ee) {
              if (ee._ev instanceof te) return ee;
              var ae = new te(), oe = new te();
              return ee._ev = ae, ee._internalEv = oe, ee.on = ae.on.bind(ae), ee.once = ae.once.bind(ae), ee.removeListener = ae.removeListener.bind(ae), ee.removeAllListeners = ae.removeAllListeners.bind(ae), ee._internalOn = oe.on.bind(oe), ee._internalOnce = oe.once.bind(oe), ee._removeInternalListener = oe.removeListener.bind(oe), ee._removeAllInternalListeners = oe.removeAllListeners.bind(oe), ee.emit = function(se, ce) {
                typeof jQuery < "u" && jQuery(ee).trigger(se, ce), ae.emit(se, ce), oe.emit(se, ce);
              }, ee;
            }, triggerHandler: function(ee, ae, oe) {
              var se, ce;
              typeof jQuery < "u" && (se = jQuery(ee).triggerHandler(ae, oe));
              var le = ee._ev;
              if (!le) return se;
              var ue, de = le._events[ae];
              if (!de) return se;
              function fe(he) {
                return he.listener ? (le.removeListener(ae, he.listener), he.fired ? void 0 : (he.fired = !0, he.listener.apply(le, [oe]))) : he.apply(le, [oe]);
              }
              for (de = Array.isArray(de) ? de : [de], ue = 0; ue < de.length - 1; ue++) fe(de[ue]);
              return ce = fe(de[ue]), se !== void 0 ? se : ce;
            }, purge: function(ee) {
              return delete ee._ev, delete ee.on, delete ee.once, delete ee.removeListener, delete ee.removeAllListeners, delete ee.emit, delete ee._ev, delete ee._internalEv, delete ee._internalOn, delete ee._internalOnce, delete ee._removeInternalListener, delete ee._removeAllInternalListeners, ee;
            } };
            G.exports = ne;
          }, 93049: function(G, J, Z) {
            var te = Z(56174), ne = Array.isArray;
            function ee(ae, oe, se, ce) {
              var le, ue, de, fe, he, me, pe, ye = ae[0], ge = ae.length;
              if (ge === 2 && ne(ye) && ne(ae[1]) && ye.length === 0) {
                if (pe = function(be, Ce) {
                  var Se, we;
                  for (Se = 0; Se < be.length; Se++) {
                    if ((we = be[Se]) !== null && typeof we == "object") return !1;
                    we !== void 0 && (Ce[Se] = we);
                  }
                  return !0;
                }(ae[1], ye), pe) return ye;
                ye.splice(0, ye.length);
              }
              for (var ve = 1; ve < ge; ve++) for (ue in le = ae[ve]) de = ye[ue], fe = le[ue], ce && ne(fe) ? ye[ue] = fe : oe && fe && (te(fe) || (he = ne(fe))) ? (he ? (he = !1, me = de && ne(de) ? de : []) : me = de && te(de) ? de : {}, ye[ue] = ee([me, fe], oe, se, ce)) : (fe !== void 0 || se) && (ye[ue] = fe);
              return ye;
            }
            J.extendFlat = function() {
              return ee(arguments, !1, !1, !1);
            }, J.extendDeep = function() {
              return ee(arguments, !0, !1, !1);
            }, J.extendDeepAll = function() {
              return ee(arguments, !0, !0, !1);
            }, J.extendDeepNoArrays = function() {
              return ee(arguments, !0, !1, !0);
            };
          }, 48965: function(G) {
            G.exports = function(J) {
              for (var Z = {}, te = [], ne = 0, ee = 0; ee < J.length; ee++) {
                var ae = J[ee];
                Z[ae] !== 1 && (Z[ae] = 1, te[ne++] = ae);
              }
              return te;
            };
          }, 78926: function(G) {
            function J(te) {
              return te.visible === !0;
            }
            function Z(te) {
              var ne = te[0].trace;
              return ne.visible === !0 && ne._length !== 0;
            }
            G.exports = function(te) {
              for (var ne, ee = (ne = te, Array.isArray(ne) && Array.isArray(ne[0]) && ne[0][0] && ne[0][0].trace ? Z : J), ae = [], oe = 0; oe < te.length; oe++) {
                var se = te[oe];
                ee(se) && ae.push(se);
              }
              return ae;
            };
          }, 3994: function(G, J, Z) {
            var te = Z(45568), ne = Z(78171), { area: ee } = Z(61990), { centroid: ae } = Z(30035), { bbox: oe } = Z(25368), se = Z(29527), ce = Z(48636), le = Z(56174), ue = Z(35632), de = Z(80899), fe = Object.keys(ne), he = { "ISO-3": se, "USA-states": se, "country names": function(pe) {
              for (var ye = 0; ye < fe.length; ye++) {
                var ge = fe[ye];
                if (new RegExp(ne[ge]).test(pe.trim().toLowerCase())) return ge;
              }
              return ce.log("Unrecognized country name: " + pe + "."), !1;
            } };
            function me(pe) {
              var ye = pe.geojson, ge = window.PlotlyGeoAssets || {}, ve = typeof ye == "string" ? ge[ye] : ye;
              return le(ve) ? ve : (ce.error("Oops ... something went wrong when fetching " + ye), !1);
            }
            G.exports = { locationToFeature: function(pe, ye, ge) {
              if (!ye || typeof ye != "string") return !1;
              var ve, be, Ce, Se = he[pe](ye);
              if (Se) {
                if (pe === "USA-states") for (ve = [], Ce = 0; Ce < ge.length; Ce++) (be = ge[Ce]).properties && be.properties.gu && be.properties.gu === "USA" && ve.push(be);
                else ve = ge;
                for (Ce = 0; Ce < ve.length; Ce++) if ((be = ve[Ce]).id === Se) return be;
                ce.log(["Location with id", Se, "does not have a matching topojson feature at this resolution."].join(" "));
              }
              return !1;
            }, feature2polygons: function(pe) {
              var ye, ge, ve, be, Ce = pe.geometry, Se = Ce.coordinates, we = pe.id, Ee = [];
              function xe(Le) {
                for (var Qe = 0; Qe < Le.length - 1; Qe++) if (Le[Qe][0] > 0 && Le[Qe + 1][0] < 0) return Qe;
                return null;
              }
              switch (ye = we === "RUS" || we === "FJI" ? function(Le) {
                var Qe;
                if (xe(Le) === null) Qe = Le;
                else for (Qe = new Array(Le.length), be = 0; be < Le.length; be++) Qe[be] = [Le[be][0] < 0 ? Le[be][0] + 360 : Le[be][0], Le[be][1]];
                Ee.push(de.tester(Qe));
              } : we === "ATA" ? function(Le) {
                var Qe = xe(Le);
                if (Qe === null) return Ee.push(de.tester(Le));
                var ke = new Array(Le.length + 1), Me = 0;
                for (be = 0; be < Le.length; be++) be > Qe ? ke[Me++] = [Le[be][0] + 360, Le[be][1]] : be === Qe ? (ke[Me++] = Le[be], ke[Me++] = [Le[be][0], -90]) : ke[Me++] = Le[be];
                var Re = de.tester(ke);
                Re.pts.pop(), Ee.push(Re);
              } : function(Le) {
                Ee.push(de.tester(Le));
              }, Ce.type) {
                case "MultiPolygon":
                  for (ge = 0; ge < Se.length; ge++) for (ve = 0; ve < Se[ge].length; ve++) ye(Se[ge][ve]);
                  break;
                case "Polygon":
                  for (ge = 0; ge < Se.length; ge++) ye(Se[ge]);
              }
              return Ee;
            }, getTraceGeojson: me, extractTraceFeature: function(pe) {
              var ye = pe[0].trace, ge = me(ye);
              if (!ge) return !1;
              var ve, be = {}, Ce = [];
              for (ve = 0; ve < ye._length; ve++) {
                var Se = pe[ve];
                (Se.loc || Se.loc === 0) && (be[Se.loc] = Se);
              }
              function we(Le) {
                var Qe = ue(Le, ye.featureidkey || "id").get(), ke = be[Qe];
                if (ke) {
                  var Me = Le.geometry;
                  if (Me.type === "Polygon" || Me.type === "MultiPolygon") {
                    var Re = { type: "Feature", id: Qe, geometry: Me, properties: {} };
                    Re.geometry.coordinates.length > 0 ? Re.properties.ct = function(De) {
                      var Pe, $e = De.geometry;
                      if ($e.type === "MultiPolygon") for (var qe = $e.coordinates, Xe = 0, Ve = 0; Ve < qe.length; Ve++) {
                        var Ze = { type: "Polygon", coordinates: qe[Ve] }, Ge = ee(Ze);
                        Ge > Xe && (Xe = Ge, Pe = Ze);
                      }
                      else Pe = $e;
                      return ae(Pe).geometry.coordinates;
                    }(Re) : Re.properties.ct = [NaN, NaN], ke.fIn = Le, ke.fOut = Re, Ce.push(Re);
                  } else ce.log(["Location", ke.loc, "does not have a valid GeoJSON geometry.", "Traces with locationmode *geojson-id* only support", "*Polygon* and *MultiPolygon* geometries."].join(" "));
                }
                delete be[Qe];
              }
              switch (ge.type) {
                case "FeatureCollection":
                  var Ee = ge.features;
                  for (ve = 0; ve < Ee.length; ve++) we(Ee[ve]);
                  break;
                case "Feature":
                  we(ge);
                  break;
                default:
                  return ce.warn(["Invalid GeoJSON type", (ge.type || "none") + ".", "Traces with locationmode *geojson-id* only support", "*FeatureCollection* and *Feature* types."].join(" ")), !1;
              }
              for (var xe in be) ce.log(["Location *" + xe + "*", "does not have a matching feature with id-key", "*" + ye.featureidkey + "*."].join(" "));
              return Ce;
            }, fetchTraceGeoData: function(pe) {
              var ye = window.PlotlyGeoAssets || {}, ge = [];
              function ve(we) {
                return new Promise(function(Ee, xe) {
                  te.json(we, function(Le, Qe) {
                    if (Le) {
                      delete ye[we];
                      var ke = Le.status === 404 ? 'GeoJSON at URL "' + we + '" does not exist.' : "Unexpected error while fetching from " + we;
                      return xe(new Error(ke));
                    }
                    return ye[we] = Qe, Ee(Qe);
                  });
                });
              }
              function be(we) {
                return new Promise(function(Ee, xe) {
                  var Le = 0, Qe = setInterval(function() {
                    return ye[we] && ye[we] !== "pending" ? (clearInterval(Qe), Ee(ye[we])) : Le > 100 ? (clearInterval(Qe), xe("Unexpected error while fetching from " + we)) : void Le++;
                  }, 50);
                });
              }
              for (var Ce = 0; Ce < pe.length; Ce++) {
                var Se = pe[Ce][0].trace.geojson;
                typeof Se == "string" && (ye[Se] ? ye[Se] === "pending" && ge.push(be(Se)) : (ye[Se] = "pending", ge.push(ve(Se))));
              }
              return ge;
            }, computeBbox: function(pe) {
              return oe(pe);
            } };
          }, 39532: function(G, J, Z) {
            var te = Z(63821).BADNUM;
            J.calcTraceToLineCoords = function(ne) {
              for (var ee = ne[0].trace.connectgaps, ae = [], oe = [], se = 0; se < ne.length; se++) {
                var ce = ne[se].lonlat;
                ce[0] !== te ? oe.push(ce) : !ee && oe.length > 0 && (ae.push(oe), oe = []);
              }
              return oe.length > 0 && ae.push(oe), ae;
            }, J.makeLine = function(ne) {
              return ne.length === 1 ? { type: "LineString", coordinates: ne[0] } : { type: "MultiLineString", coordinates: ne };
            }, J.makePolygon = function(ne) {
              if (ne.length === 1) return { type: "Polygon", coordinates: ne };
              for (var ee = new Array(ne.length), ae = 0; ae < ne.length; ae++) ee[ae] = [ne[ae]];
              return { type: "MultiPolygon", coordinates: ee };
            }, J.makeBlank = function() {
              return { type: "Point", coordinates: [] };
            };
          }, 3447: function(G, J, Z) {
            var te, ne, ee, ae = Z(98953).mod;
            function oe(ce, le, ue, de, fe, he, me, pe) {
              var ye = ue - ce, ge = fe - ce, ve = me - fe, be = de - le, Ce = he - le, Se = pe - he, we = ye * Se - ve * be;
              if (we === 0) return null;
              var Ee = (ge * Se - ve * Ce) / we, xe = (ge * be - ye * Ce) / we;
              return xe < 0 || xe > 1 || Ee < 0 || Ee > 1 ? null : { x: ce + ye * Ee, y: le + be * Ee };
            }
            function se(ce, le, ue, de, fe) {
              var he = de * ce + fe * le;
              if (he < 0) return de * de + fe * fe;
              if (he > ue) {
                var me = de - ce, pe = fe - le;
                return me * me + pe * pe;
              }
              var ye = de * le - fe * ce;
              return ye * ye / ue;
            }
            J.segmentsIntersect = oe, J.segmentDistance = function(ce, le, ue, de, fe, he, me, pe) {
              if (oe(ce, le, ue, de, fe, he, me, pe)) return 0;
              var ye = ue - ce, ge = de - le, ve = me - fe, be = pe - he, Ce = ye * ye + ge * ge, Se = ve * ve + be * be, we = Math.min(se(ye, ge, Ce, fe - ce, he - le), se(ye, ge, Ce, me - ce, pe - le), se(ve, be, Se, ce - fe, le - he), se(ve, be, Se, ue - fe, de - he));
              return Math.sqrt(we);
            }, J.getTextLocation = function(ce, le, ue, de) {
              if (ce === ne && de === ee || (te = {}, ne = ce, ee = de), te[ue]) return te[ue];
              var fe = ce.getPointAtLength(ae(ue - de / 2, le)), he = ce.getPointAtLength(ae(ue + de / 2, le)), me = Math.atan((he.y - fe.y) / (he.x - fe.x)), pe = ce.getPointAtLength(ae(ue, le)), ye = { x: (4 * pe.x + fe.x + he.x) / 6, y: (4 * pe.y + fe.y + he.y) / 6, theta: me };
              return te[ue] = ye, ye;
            }, J.clearLocationCache = function() {
              ne = null;
            }, J.getVisibleSegment = function(ce, le, ue) {
              var de, fe, he = le.left, me = le.right, pe = le.top, ye = le.bottom, ge = 0, ve = ce.getTotalLength(), be = ve;
              function Ce(we) {
                var Ee = ce.getPointAtLength(we);
                we === 0 ? de = Ee : we === ve && (fe = Ee);
                var xe = Ee.x < he ? he - Ee.x : Ee.x > me ? Ee.x - me : 0, Le = Ee.y < pe ? pe - Ee.y : Ee.y > ye ? Ee.y - ye : 0;
                return Math.sqrt(xe * xe + Le * Le);
              }
              for (var Se = Ce(ge); Se; ) {
                if ((ge += Se + ue) > be) return;
                Se = Ce(ge);
              }
              for (Se = Ce(be); Se; ) {
                if (ge > (be -= Se + ue)) return;
                Se = Ce(be);
              }
              return { min: ge, max: be, len: be - ge, total: ve, isClosed: ge === 0 && be === ve && Math.abs(de.x - fe.x) < 0.1 && Math.abs(de.y - fe.y) < 0.1 };
            }, J.findPointOnPath = function(ce, le, ue, de) {
              for (var fe, he, me, pe = (de = de || {}).pathLength || ce.getTotalLength(), ye = de.tolerance || 1e-3, ge = de.iterationLimit || 30, ve = ce.getPointAtLength(0)[ue] > ce.getPointAtLength(pe)[ue] ? -1 : 1, be = 0, Ce = 0, Se = pe; be < ge; ) {
                if (fe = (Ce + Se) / 2, me = (he = ce.getPointAtLength(fe))[ue] - le, Math.abs(me) < ye) return he;
                ve * me > 0 ? Se = fe : Ce = fe, be++;
              }
              return he;
            };
          }, 46998: function(G, J, Z) {
            var te = Z(10721), ne = Z(65657), ee = Z(162), ae = Z(88856), oe = Z(10229).defaultLine, se = Z(87800).isArrayOrTypedArray, ce = ee(oe);
            function le(fe, he) {
              var me = fe;
              return me[3] *= he, me;
            }
            function ue(fe) {
              if (te(fe)) return ce;
              var he = ee(fe);
              return he.length ? he : ce;
            }
            function de(fe) {
              return te(fe) ? fe : 1;
            }
            G.exports = { formatColor: function(fe, he, me) {
              var pe = fe.color;
              pe && pe._inputArray && (pe = pe._inputArray);
              var ye, ge, ve, be, Ce, Se = se(pe), we = se(he), Ee = ae.extractOpts(fe), xe = [];
              if (ye = Ee.colorscale !== void 0 ? ae.makeColorScaleFuncFromTrace(fe) : ue, ge = Se ? function(Qe, ke) {
                return Qe[ke] === void 0 ? ce : ee(ye(Qe[ke]));
              } : ue, ve = we ? function(Qe, ke) {
                return Qe[ke] === void 0 ? 1 : de(Qe[ke]);
              } : de, Se || we) for (var Le = 0; Le < me; Le++) be = ge(pe, Le), Ce = ve(he, Le), xe[Le] = le(be, Ce);
              else xe = le(ee(pe), he);
              return xe;
            }, parseColorScale: function(fe) {
              var he = ae.extractOpts(fe), me = he.colorscale;
              return he.reversescale && (me = ae.flipScale(he.colorscale)), me.map(function(pe) {
                var ye = pe[0], ge = ne(pe[1]).toRgb();
                return { index: ye, rgb: [ge.r, ge.g, ge.b, ge.a] };
              });
            } };
          }, 71293: function(G, J, Z) {
            var te = Z(29527);
            function ne(ee) {
              return [ee];
            }
            G.exports = { keyFun: function(ee) {
              return ee.key;
            }, repeat: ne, descend: te, wrap: ne, unwrap: function(ee) {
              return ee[0];
            } };
          }, 29527: function(G) {
            G.exports = function(J) {
              return J;
            };
          }, 10688: function(G) {
            G.exports = function(J, Z) {
              if (!Z) return J;
              var te = 1 / Math.abs(Z), ne = te > 1 ? (te * J + te * Z) / te : J + Z, ee = String(ne).length;
              if (ee > 16) {
                var ae = String(Z).length;
                if (ee >= String(J).length + ae) {
                  var oe = parseFloat(ne).toPrecision(12);
                  oe.indexOf("e+") === -1 && (ne = +oe);
                }
              }
              return ne;
            };
          }, 34809: function(G, J, Z) {
            var te = Z(45568), ne = Z(42696).aL, ee = Z(36464).GP, ae = Z(10721), oe = Z(63821), se = oe.FP_SAFE, ce = -se, le = oe.BADNUM, ue = G.exports = {};
            ue.adjustFormat = function(Je) {
              return !Je || /^\d[.]\df/.test(Je) || /[.]\d%/.test(Je) ? Je : Je === "0.f" ? "~f" : /^\d%/.test(Je) ? "~%" : /^\ds/.test(Je) ? "~s" : !/^[~,.0$]/.test(Je) && /[&fps]/.test(Je) ? "~" + Je : Je;
            };
            var de = {};
            ue.warnBadFormat = function(Je) {
              var it = String(Je);
              de[it] || (de[it] = 1, ue.warn('encountered bad format: "' + it + '"'));
            }, ue.noFormat = function(Je) {
              return String(Je);
            }, ue.numberFormat = function(Je) {
              var it;
              try {
                it = ee(ue.adjustFormat(Je));
              } catch {
                return ue.warnBadFormat(Je), ue.noFormat;
              }
              return it;
            }, ue.nestedProperty = Z(35632), ue.keyedContainer = Z(34967), ue.relativeAttr = Z(82047), ue.isPlainObject = Z(56174), ue.toLogRange = Z(8083), ue.relinkPrivateKeys = Z(80428);
            var fe = Z(87800);
            ue.isArrayBuffer = fe.isArrayBuffer, ue.isTypedArray = fe.isTypedArray, ue.isArrayOrTypedArray = fe.isArrayOrTypedArray, ue.isArray1D = fe.isArray1D, ue.ensureArray = fe.ensureArray, ue.concat = fe.concat, ue.maxRowLength = fe.maxRowLength, ue.minRowLength = fe.minRowLength;
            var he = Z(98953);
            ue.mod = he.mod, ue.modHalf = he.modHalf;
            var me = Z(34220);
            ue.valObjectMeta = me.valObjectMeta, ue.coerce = me.coerce, ue.coerce2 = me.coerce2, ue.coerceFont = me.coerceFont, ue.coercePattern = me.coercePattern, ue.coerceHoverinfo = me.coerceHoverinfo, ue.coerceSelectionMarkerOpacity = me.coerceSelectionMarkerOpacity, ue.validate = me.validate;
            var pe = Z(92596);
            ue.dateTime2ms = pe.dateTime2ms, ue.isDateTime = pe.isDateTime, ue.ms2DateTime = pe.ms2DateTime, ue.ms2DateTimeLocal = pe.ms2DateTimeLocal, ue.cleanDate = pe.cleanDate, ue.isJSDate = pe.isJSDate, ue.formatDate = pe.formatDate, ue.incrementMonth = pe.incrementMonth, ue.dateTick0 = pe.dateTick0, ue.dfltRange = pe.dfltRange, ue.findExactDates = pe.findExactDates, ue.MIN_MS = pe.MIN_MS, ue.MAX_MS = pe.MAX_MS;
            var ye = Z(98813);
            ue.findBin = ye.findBin, ue.sorterAsc = ye.sorterAsc, ue.sorterDes = ye.sorterDes, ue.distinctVals = ye.distinctVals, ue.roundUp = ye.roundUp, ue.sort = ye.sort, ue.findIndexOfMin = ye.findIndexOfMin, ue.sortObjectKeys = Z(62994);
            var ge = Z(89258);
            ue.aggNums = ge.aggNums, ue.len = ge.len, ue.mean = ge.mean, ue.geometricMean = ge.geometricMean, ue.median = ge.median, ue.midRange = ge.midRange, ue.variance = ge.variance, ue.stdev = ge.stdev, ue.interp = ge.interp;
            var ve = Z(15236);
            ue.init2dArray = ve.init2dArray, ue.transposeRagged = ve.transposeRagged, ue.dot = ve.dot, ue.translationMatrix = ve.translationMatrix, ue.rotationMatrix = ve.rotationMatrix, ue.rotationXYMatrix = ve.rotationXYMatrix, ue.apply3DTransform = ve.apply3DTransform, ue.apply2DTransform = ve.apply2DTransform, ue.apply2DTransform2 = ve.apply2DTransform2, ue.convertCssMatrix = ve.convertCssMatrix, ue.inverseTransformMatrix = ve.inverseTransformMatrix;
            var be = Z(44313);
            ue.deg2rad = be.deg2rad, ue.rad2deg = be.rad2deg, ue.angleDelta = be.angleDelta, ue.angleDist = be.angleDist, ue.isFullCircle = be.isFullCircle, ue.isAngleInsideSector = be.isAngleInsideSector, ue.isPtInsideSector = be.isPtInsideSector, ue.pathArc = be.pathArc, ue.pathSector = be.pathSector, ue.pathAnnulus = be.pathAnnulus;
            var Ce = Z(32546);
            ue.isLeftAnchor = Ce.isLeftAnchor, ue.isCenterAnchor = Ce.isCenterAnchor, ue.isRightAnchor = Ce.isRightAnchor, ue.isTopAnchor = Ce.isTopAnchor, ue.isMiddleAnchor = Ce.isMiddleAnchor, ue.isBottomAnchor = Ce.isBottomAnchor;
            var Se = Z(3447);
            ue.segmentsIntersect = Se.segmentsIntersect, ue.segmentDistance = Se.segmentDistance, ue.getTextLocation = Se.getTextLocation, ue.clearLocationCache = Se.clearLocationCache, ue.getVisibleSegment = Se.getVisibleSegment, ue.findPointOnPath = Se.findPointOnPath;
            var we = Z(93049);
            ue.extendFlat = we.extendFlat, ue.extendDeep = we.extendDeep, ue.extendDeepAll = we.extendDeepAll, ue.extendDeepNoArrays = we.extendDeepNoArrays;
            var Ee = Z(48636);
            ue.log = Ee.log, ue.warn = Ee.warn, ue.error = Ee.error;
            var xe = Z(90694);
            ue.counterRegex = xe.counter;
            var Le = Z(64025);
            ue.throttle = Le.throttle, ue.throttleDone = Le.done, ue.clearThrottle = Le.clear;
            var Qe = Z(95425);
            function ke(Je) {
              var it = {};
              for (var st in Je) for (var gt = Je[st], bt = 0; bt < gt.length; bt++) it[gt[bt]] = +st;
              return it;
            }
            ue.getGraphDiv = Qe.getGraphDiv, ue.isPlotDiv = Qe.isPlotDiv, ue.removeElement = Qe.removeElement, ue.addStyleRule = Qe.addStyleRule, ue.addRelatedStyleRule = Qe.addRelatedStyleRule, ue.deleteRelatedStyleRule = Qe.deleteRelatedStyleRule, ue.getFullTransformMatrix = Qe.getFullTransformMatrix, ue.getElementTransformMatrix = Qe.getElementTransformMatrix, ue.getElementAndAncestors = Qe.getElementAndAncestors, ue.equalDomRects = Qe.equalDomRects, ue.clearResponsive = Z(23493), ue.preserveDrawingBuffer = Z(32521), ue.makeTraceGroups = Z(75944), ue._ = Z(38514), ue.notifier = Z(87355), ue.filterUnique = Z(48965), ue.filterVisible = Z(78926), ue.pushUnique = Z(36539), ue.increment = Z(10688), ue.cleanNumber = Z(44498), ue.ensureNumber = function(Je) {
              return ae(Je) ? (Je = Number(Je)) > se || Je < ce ? le : Je : le;
            }, ue.isIndex = function(Je, it) {
              return !(it !== void 0 && Je >= it) && ae(Je) && Je >= 0 && Je % 1 == 0;
            }, ue.noop = Z(4969), ue.identity = Z(29527), ue.repeat = function(Je, it) {
              for (var st = new Array(it), gt = 0; gt < it; gt++) st[gt] = Je;
              return st;
            }, ue.swapAttrs = function(Je, it, st, gt) {
              st || (st = "x"), gt || (gt = "y");
              for (var bt = 0; bt < it.length; bt++) {
                var yt = it[bt], St = ue.nestedProperty(Je, yt.replace("?", st)), Lt = ue.nestedProperty(Je, yt.replace("?", gt)), Rt = St.get();
                St.set(Lt.get()), Lt.set(Rt);
              }
            }, ue.raiseToTop = function(Je) {
              Je.parentNode.appendChild(Je);
            }, ue.cancelTransition = function(Je) {
              return Je.transition().duration(0);
            }, ue.constrain = function(Je, it, st) {
              return it > st ? Math.max(st, Math.min(it, Je)) : Math.max(it, Math.min(st, Je));
            }, ue.bBoxIntersect = function(Je, it, st) {
              return st = st || 0, Je.left <= it.right + st && it.left <= Je.right + st && Je.top <= it.bottom + st && it.top <= Je.bottom + st;
            }, ue.simpleMap = function(Je, it, st, gt, bt) {
              for (var yt = Je.length, St = new Array(yt), Lt = 0; Lt < yt; Lt++) St[Lt] = it(Je[Lt], st, gt, bt);
              return St;
            }, ue.randstr = function Je(it, st, gt, bt) {
              if (gt || (gt = 16), st === void 0 && (st = 24), st <= 0) return "0";
              var yt, St, Lt = Math.log(Math.pow(2, st)) / Math.log(gt), Rt = "";
              for (yt = 2; Lt === 1 / 0; yt *= 2) Lt = Math.log(Math.pow(2, st / yt)) / Math.log(gt) * yt;
              var vt = Lt - Math.floor(Lt);
              for (yt = 0; yt < Math.floor(Lt); yt++) Rt = Math.floor(Math.random() * gt).toString(gt) + Rt;
              vt && (St = Math.pow(gt, vt), Rt = Math.floor(Math.random() * St).toString(gt) + Rt);
              var $t = parseInt(Rt, gt);
              return it && it[Rt] || $t !== 1 / 0 && $t >= Math.pow(2, st) ? bt > 10 ? (ue.warn("randstr failed uniqueness"), Rt) : Je(it, st, gt, (bt || 0) + 1) : Rt;
            }, ue.OptionControl = function(Je, it) {
              Je || (Je = {}), it || (it = "opt");
              var st = { optionList: [], _newoption: function(gt) {
                gt[it] = Je, st[gt.name] = gt, st.optionList.push(gt);
              } };
              return st["_" + it] = Je, st;
            }, ue.smooth = function(Je, it) {
              if ((it = Math.round(it) || 0) < 2) return Je;
              var st, gt, bt, yt, St = Je.length, Lt = 2 * St, Rt = 2 * it - 1, vt = new Array(Rt), $t = new Array(St);
              for (st = 0; st < Rt; st++) vt[st] = (1 - Math.cos(Math.PI * (st + 1) / it)) / (2 * it);
              for (st = 0; st < St; st++) {
                for (yt = 0, gt = 0; gt < Rt; gt++) (bt = st + gt + 1 - it) < -St ? bt -= Lt * Math.round(bt / Lt) : bt >= Lt && (bt -= Lt * Math.floor(bt / Lt)), bt < 0 ? bt = -1 - bt : bt >= St && (bt = Lt - 1 - bt), yt += Je[bt] * vt[gt];
                $t[st] = yt;
              }
              return $t;
            }, ue.syncOrAsync = function(Je, it, st) {
              var gt;
              function bt() {
                return ue.syncOrAsync(Je, it, st);
              }
              for (; Je.length; ) if ((gt = (0, Je.splice(0, 1)[0])(it)) && gt.then) return gt.then(bt);
              return st && st(it);
            }, ue.stripTrailingSlash = function(Je) {
              return Je.substr(-1) === "/" ? Je.substr(0, Je.length - 1) : Je;
            }, ue.noneOrAll = function(Je, it, st) {
              if (Je) {
                var gt, bt = !1, yt = !0;
                for (gt = 0; gt < st.length; gt++) Je[st[gt]] != null ? bt = !0 : yt = !1;
                if (bt && !yt) for (gt = 0; gt < st.length; gt++) Je[st[gt]] = it[st[gt]];
              }
            }, ue.mergeArray = function(Je, it, st, gt) {
              var bt = typeof gt == "function";
              if (ue.isArrayOrTypedArray(Je)) for (var yt = Math.min(Je.length, it.length), St = 0; St < yt; St++) {
                var Lt = Je[St];
                it[St][st] = bt ? gt(Lt) : Lt;
              }
            }, ue.mergeArrayCastPositive = function(Je, it, st) {
              return ue.mergeArray(Je, it, st, function(gt) {
                var bt = +gt;
                return isFinite(bt) && bt > 0 ? bt : 0;
              });
            }, ue.fillArray = function(Je, it, st, gt) {
              if (gt = gt || ue.identity, ue.isArrayOrTypedArray(Je)) for (var bt = 0; bt < it.length; bt++) it[bt][st] = gt(Je[bt]);
            }, ue.castOption = function(Je, it, st, gt) {
              gt = gt || ue.identity;
              var bt = ue.nestedProperty(Je, st).get();
              return ue.isArrayOrTypedArray(bt) ? Array.isArray(it) && ue.isArrayOrTypedArray(bt[it[0]]) ? gt(bt[it[0]][it[1]]) : gt(bt[it]) : bt;
            }, ue.extractOption = function(Je, it, st, gt) {
              if (st in Je) return Je[st];
              var bt = ue.nestedProperty(it, gt).get();
              return Array.isArray(bt) ? void 0 : bt;
            }, ue.tagSelected = function(Je, it, st) {
              var gt, bt, yt = it.selectedpoints, St = it._indexToPoints;
              St && (gt = ke(St));
              for (var Lt = 0; Lt < yt.length; Lt++) {
                var Rt = yt[Lt];
                if (ue.isIndex(Rt) || ue.isArrayOrTypedArray(Rt) && ue.isIndex(Rt[0]) && ue.isIndex(Rt[1])) {
                  var vt = gt ? gt[Rt] : Rt, $t = st ? st[vt] : vt;
                  (bt = $t) !== void 0 && bt < Je.length && (Je[$t].selected = 1);
                }
              }
            }, ue.selIndices2selPoints = function(Je) {
              var it = Je.selectedpoints, st = Je._indexToPoints;
              if (st) {
                for (var gt = ke(st), bt = [], yt = 0; yt < it.length; yt++) {
                  var St = it[yt];
                  if (ue.isIndex(St)) {
                    var Lt = gt[St];
                    ue.isIndex(Lt) && bt.push(Lt);
                  }
                }
                return bt;
              }
              return it;
            }, ue.getTargetArray = function(Je, it) {
              var st = it.target;
              if (typeof st == "string" && st) {
                var gt = ue.nestedProperty(Je, st).get();
                return !!ue.isArrayOrTypedArray(gt) && gt;
              }
              return !!ue.isArrayOrTypedArray(st) && st;
            }, ue.minExtend = function Je(it, st, gt) {
              var bt = {};
              typeof st != "object" && (st = {});
              var yt, St, Lt, Rt = gt === "pieLike" ? -1 : 3, vt = Object.keys(it);
              for (yt = 0; yt < vt.length; yt++) Lt = it[St = vt[yt]], St.charAt(0) !== "_" && typeof Lt != "function" && (St === "module" ? bt[St] = Lt : Array.isArray(Lt) ? bt[St] = St === "colorscale" || Rt === -1 ? Lt.slice() : Lt.slice(0, Rt) : ue.isTypedArray(Lt) ? bt[St] = Rt === -1 ? Lt.subarray() : Lt.subarray(0, Rt) : bt[St] = Lt && typeof Lt == "object" ? Je(it[St], st[St], gt) : Lt);
              for (vt = Object.keys(st), yt = 0; yt < vt.length; yt++) typeof (Lt = st[St = vt[yt]]) == "object" && St in bt && typeof bt[St] == "object" || (bt[St] = Lt);
              return bt;
            }, ue.titleCase = function(Je) {
              return Je.charAt(0).toUpperCase() + Je.substr(1);
            }, ue.containsAny = function(Je, it) {
              for (var st = 0; st < it.length; st++) if (Je.indexOf(it[st]) !== -1) return !0;
              return !1;
            }, ue.isIE = function() {
              return window.navigator.msSaveBlob !== void 0;
            };
            var Me = /Version\/[\d\.]+.*Safari/;
            ue.isSafari = function() {
              return Me.test(window.navigator.userAgent);
            };
            var Re = /iPad|iPhone|iPod/;
            ue.isIOS = function() {
              return Re.test(window.navigator.userAgent);
            };
            var De = /Firefox\/(\d+)\.\d+/;
            ue.getFirefoxVersion = function() {
              var Je = De.exec(window.navigator.userAgent);
              if (Je && Je.length === 2) {
                var it = parseInt(Je[1]);
                if (!isNaN(it)) return it;
              }
              return null;
            }, ue.isD3Selection = function(Je) {
              return Je instanceof te.selection;
            }, ue.ensureSingle = function(Je, it, st, gt) {
              var bt = Je.select(it + (st ? "." + st : ""));
              if (bt.size()) return bt;
              var yt = Je.append(it);
              return st && yt.classed(st, !0), gt && yt.call(gt), yt;
            }, ue.ensureSingleById = function(Je, it, st, gt) {
              var bt = Je.select(it + "#" + st);
              if (bt.size()) return bt;
              var yt = Je.append(it).attr("id", st);
              return gt && yt.call(gt), yt;
            }, ue.objectFromPath = function(Je, it) {
              for (var st, gt = Je.split("."), bt = st = {}, yt = 0; yt < gt.length; yt++) {
                var St = gt[yt], Lt = null, Rt = gt[yt].match(/(.*)\[([0-9]+)\]/);
                Rt ? (St = Rt[1], Lt = Rt[2], st = st[St] = [], yt === gt.length - 1 ? st[Lt] = it : st[Lt] = {}, st = st[Lt]) : (yt === gt.length - 1 ? st[St] = it : st[St] = {}, st = st[St]);
              }
              return bt;
            };
            var Pe = /^([^\[\.]+)\.(.+)?/, $e = /^([^\.]+)\[([0-9]+)\](\.)?(.+)?/;
            function qe(Je) {
              return Je.slice(0, 2) === "__";
            }
            ue.expandObjectPaths = function(Je) {
              var it, st, gt, bt, yt, St, Lt;
              if (typeof Je == "object" && !Array.isArray(Je)) {
                for (st in Je) if (Je.hasOwnProperty(st)) if (it = st.match(Pe)) {
                  if (bt = Je[st], qe(gt = it[1])) continue;
                  delete Je[st], Je[gt] = ue.extendDeepNoArrays(Je[gt] || {}, ue.objectFromPath(st, ue.expandObjectPaths(bt))[gt]);
                } else if (it = st.match($e)) {
                  if (bt = Je[st], qe(gt = it[1])) continue;
                  if (yt = parseInt(it[2]), delete Je[st], Je[gt] = Je[gt] || [], it[3] === ".") Lt = it[4], St = Je[gt][yt] = Je[gt][yt] || {}, ue.extendDeepNoArrays(St, ue.objectFromPath(Lt, ue.expandObjectPaths(bt)));
                  else {
                    if (qe(gt)) continue;
                    Je[gt][yt] = ue.expandObjectPaths(bt);
                  }
                } else {
                  if (qe(st)) continue;
                  Je[st] = ue.expandObjectPaths(Je[st]);
                }
              }
              return Je;
            }, ue.numSeparate = function(Je, it, st) {
              if (st || (st = !1), typeof it != "string" || it.length === 0) throw new Error("Separator string required for formatting!");
              typeof Je == "number" && (Je = String(Je));
              var gt = /(\d+)(\d{3})/, bt = it.charAt(0), yt = it.charAt(1), St = Je.split("."), Lt = St[0], Rt = St.length > 1 ? bt + St[1] : "";
              if (yt && (St.length > 1 || Lt.length > 4 || st)) for (; gt.test(Lt); ) Lt = Lt.replace(gt, "$1" + yt + "$2");
              return Lt + Rt;
            }, ue.TEMPLATE_STRING_REGEX = /%{([^\s%{}:]*)([:|\|][^}]*)?}/g;
            var Xe = /^\w*$/;
            ue.templateString = function(Je, it) {
              var st = {};
              return Je.replace(ue.TEMPLATE_STRING_REGEX, function(gt, bt) {
                var yt;
                return Xe.test(bt) ? yt = it[bt] : (st[bt] = st[bt] || ue.nestedProperty(it, bt).get, yt = st[bt]()), ue.isValidTextValue(yt) ? yt : "";
              });
            };
            var Ve = { max: 10, count: 0, name: "hovertemplate" };
            ue.hovertemplateString = function() {
              return at.apply(Ve, arguments);
            };
            var Ze = { max: 10, count: 0, name: "texttemplate" };
            ue.texttemplateString = function() {
              return at.apply(Ze, arguments);
            };
            var Ge = /^(\S+)([\*\/])(-?\d+(\.\d+)?)$/, ze = { max: 10, count: 0, name: "texttemplate", parseMultDiv: !0 };
            ue.texttemplateStringForShapes = function() {
              return at.apply(ze, arguments);
            };
            var We = /^[:|\|]/;
            function at(Je, it, st) {
              var gt = this, bt = arguments;
              it || (it = {});
              var yt = {};
              return Je.replace(ue.TEMPLATE_STRING_REGEX, function(St, Lt, Rt) {
                var vt = Lt === "_xother" || Lt === "_yother", $t = Lt === "_xother_" || Lt === "_yother_", Pt = Lt === "xother_" || Lt === "yother_", Xt = Lt === "xother" || Lt === "yother" || vt || Pt || $t, Nt = Lt;
                (vt || $t) && (Nt = Nt.substring(1)), (Pt || $t) && (Nt = Nt.substring(0, Nt.length - 1));
                var Gt, Ft, Tt, It = null, Ct = null;
                if (gt.parseMultDiv) {
                  var Kt = function(hn) {
                    var Qt = hn.match(Ge);
                    return Qt ? { key: Qt[1], op: Qt[2], number: Number(Qt[3]) } : { key: hn, op: null, number: null };
                  }(Nt);
                  Nt = Kt.key, It = Kt.op, Ct = Kt.number;
                }
                if (Xt) {
                  if ((Gt = it[Nt]) === void 0) return "";
                } else for (Tt = 3; Tt < bt.length; Tt++) if (Ft = bt[Tt]) {
                  if (Ft.hasOwnProperty(Nt)) {
                    Gt = Ft[Nt];
                    break;
                  }
                  if (Xe.test(Nt) || (Gt = ue.nestedProperty(Ft, Nt).get(), (Gt = yt[Nt] || ue.nestedProperty(Ft, Nt).get()) && (yt[Nt] = Gt)), Gt !== void 0) break;
                }
                if (Gt !== void 0 && (It === "*" && (Gt *= Ct), It === "/" && (Gt /= Ct)), Gt === void 0 && gt) return gt.count < gt.max && (ue.warn("Variable '" + Nt + "' in " + gt.name + " could not be found!"), Gt = St), gt.count === gt.max && ue.warn("Too many " + gt.name + " warnings - additional warnings will be suppressed"), gt.count++, St;
                if (Rt) {
                  var Vt;
                  if (Rt[0] === ":" && (Vt = st ? st.numberFormat : ue.numberFormat, Gt !== "" && (Gt = Vt(Rt.replace(We, ""))(Gt))), Rt[0] === "|") {
                    Vt = st ? st.timeFormat : ne;
                    var Yt = ue.dateTime2ms(Gt);
                    Gt = ue.formatDate(Yt, Rt.replace(We, ""), !1, Vt);
                  }
                } else {
                  var sn = Nt + "Label";
                  it.hasOwnProperty(sn) && (Gt = it[sn]);
                }
                return Xt && (Gt = "(" + Gt + ")", (vt || $t) && (Gt = " " + Gt), (Pt || $t) && (Gt += " ")), Gt;
              });
            }
            ue.subplotSort = function(Je, it) {
              for (var st = Math.min(Je.length, it.length) + 1, gt = 0, bt = 0, yt = 0; yt < st; yt++) {
                var St = Je.charCodeAt(yt) || 0, Lt = it.charCodeAt(yt) || 0, Rt = St >= 48 && St <= 57, vt = Lt >= 48 && Lt <= 57;
                if (Rt && (gt = 10 * gt + St - 48), vt && (bt = 10 * bt + Lt - 48), !Rt || !vt) {
                  if (gt !== bt) return gt - bt;
                  if (St !== Lt) return St - Lt;
                }
              }
              return bt - gt;
            };
            var ct = 2e9;
            ue.seedPseudoRandom = function() {
              ct = 2e9;
            }, ue.pseudoRandom = function() {
              var Je = ct;
              return ct = (69069 * ct + 1) % 4294967296, Math.abs(ct - Je) < 429496729 ? ue.pseudoRandom() : ct / 4294967296;
            }, ue.fillText = function(Je, it, st) {
              var gt = Array.isArray(st) ? function(St) {
                st.push(St);
              } : function(St) {
                st.text = St;
              }, bt = ue.extractOption(Je, it, "htx", "hovertext");
              if (ue.isValidTextValue(bt)) return gt(bt);
              var yt = ue.extractOption(Je, it, "tx", "text");
              return ue.isValidTextValue(yt) ? gt(yt) : void 0;
            }, ue.isValidTextValue = function(Je) {
              return Je || Je === 0;
            }, ue.formatPercent = function(Je, it) {
              it = it || 0;
              for (var st = (Math.round(100 * Je * Math.pow(10, it)) * Math.pow(0.1, it)).toFixed(it) + "%", gt = 0; gt < it; gt++) st.indexOf(".") !== -1 && (st = (st = st.replace("0%", "%")).replace(".%", "%"));
              return st;
            }, ue.isHidden = function(Je) {
              var it = window.getComputedStyle(Je).display;
              return !it || it === "none";
            }, ue.strTranslate = function(Je, it) {
              return Je || it ? "translate(" + Je + "," + it + ")" : "";
            }, ue.strRotate = function(Je) {
              return Je ? "rotate(" + Je + ")" : "";
            }, ue.strScale = function(Je) {
              return Je !== 1 ? "scale(" + Je + ")" : "";
            }, ue.getTextTransform = function(Je) {
              var it = Je.noCenter, st = Je.textX, gt = Je.textY, bt = Je.targetX, yt = Je.targetY, St = Je.anchorX || 0, Lt = Je.anchorY || 0, Rt = Je.rotate, vt = Je.scale;
              return vt ? vt > 1 && (vt = 1) : vt = 0, ue.strTranslate(bt - vt * (st + St), yt - vt * (gt + Lt)) + ue.strScale(vt) + (Rt ? "rotate(" + Rt + (it ? "" : " " + st + " " + gt) + ")" : "");
            }, ue.setTransormAndDisplay = function(Je, it) {
              Je.attr("transform", ue.getTextTransform(it)), Je.style("display", it.scale ? null : "none");
            }, ue.ensureUniformFontSize = function(Je, it) {
              var st = ue.extendFlat({}, it);
              return st.size = Math.max(it.size, Je._fullLayout.uniformtext.minsize || 0), st;
            }, ue.join2 = function(Je, it, st) {
              var gt = Je.length;
              return gt > 1 ? Je.slice(0, -1).join(it) + st + Je[gt - 1] : Je.join(it);
            }, ue.bigFont = function(Je) {
              return Math.round(1.2 * Je);
            };
            var et = ue.getFirefoxVersion(), dt = et !== null && et < 86;
            ue.getPositionFromD3Event = function() {
              return dt ? [te.event.layerX, te.event.layerY] : [te.event.offsetX, te.event.offsetY];
            };
          }, 56174: function(G) {
            G.exports = function(J) {
              return window && window.process && window.process.versions ? Object.prototype.toString.call(J) === "[object Object]" : Object.prototype.toString.call(J) === "[object Object]" && Object.getPrototypeOf(J).hasOwnProperty("hasOwnProperty");
            };
          }, 34967: function(G, J, Z) {
            var te = Z(35632), ne = /^\w*$/;
            G.exports = function(ee, ae, oe, se) {
              var ce, le, ue;
              oe = oe || "name", se = se || "value";
              var de = {};
              ae && ae.length ? (ue = te(ee, ae), le = ue.get()) : le = ee, ae = ae || "";
              var fe = {};
              if (le) for (ce = 0; ce < le.length; ce++) fe[le[ce][oe]] = ce;
              var he = ne.test(se), me = { set: function(pe, ye) {
                var ge = ye === null ? 4 : 0;
                if (!le) {
                  if (!ue || ge === 4) return;
                  le = [], ue.set(le);
                }
                var ve = fe[pe];
                if (ve === void 0) {
                  if (ge === 4) return;
                  ge |= 3, ve = le.length, fe[pe] = ve;
                } else ye !== (he ? le[ve][se] : te(le[ve], se).get()) && (ge |= 2);
                var be = le[ve] = le[ve] || {};
                return be[oe] = pe, he ? be[se] = ye : te(be, se).set(ye), ye !== null && (ge &= -5), de[ve] = de[ve] | ge, me;
              }, get: function(pe) {
                if (le) {
                  var ye = fe[pe];
                  return ye === void 0 ? void 0 : he ? le[ye][se] : te(le[ye], se).get();
                }
              }, rename: function(pe, ye) {
                var ge = fe[pe];
                return ge === void 0 || (de[ge] = 1 | de[ge], fe[ye] = ge, delete fe[pe], le[ge][oe] = ye), me;
              }, remove: function(pe) {
                var ye = fe[pe];
                if (ye === void 0) return me;
                var ge = le[ye];
                if (Object.keys(ge).length > 2) return de[ye] = 2 | de[ye], me.set(pe, null);
                if (he) {
                  for (ce = ye; ce < le.length; ce++) de[ce] = 3 | de[ce];
                  for (ce = ye; ce < le.length; ce++) fe[le[ce][oe]]--;
                  le.splice(ye, 1), delete fe[pe];
                } else te(ge, se).set(null), de[ye] = 6 | de[ye];
                return me;
              }, constructUpdate: function() {
                for (var pe, ye, ge = {}, ve = Object.keys(de), be = 0; be < ve.length; be++) ye = ve[be], pe = ae + "[" + ye + "]", le[ye] ? (1 & de[ye] && (ge[pe + "." + oe] = le[ye][oe]), 2 & de[ye] && (ge[pe + "." + se] = he ? 4 & de[ye] ? null : le[ye][se] : 4 & de[ye] ? null : te(le[ye], se).get())) : ge[pe] = null;
                return ge;
              } };
              return me;
            };
          }, 38514: function(G, J, Z) {
            var te = Z(33626);
            G.exports = function(ne, ee) {
              for (var ae = ne._context.locale, oe = 0; oe < 2; oe++) {
                for (var se = ne._context.locales, ce = 0; ce < 2; ce++) {
                  var le = (se[ae] || {}).dictionary;
                  if (le) {
                    var ue = le[ee];
                    if (ue) return ue;
                  }
                  se = te.localeRegistry;
                }
                var de = ae.split("-")[0];
                if (de === ae) break;
                ae = de;
              }
              return ee;
            };
          }, 48636: function(G, J, Z) {
            var te = Z(24452).dfltConfig, ne = Z(87355), ee = G.exports = {};
            ee.log = function() {
              var ae;
              if (te.logging > 1) {
                var oe = ["LOG:"];
                for (ae = 0; ae < arguments.length; ae++) oe.push(arguments[ae]);
                console.trace.apply(console, oe);
              }
              if (te.notifyOnLogging > 1) {
                var se = [];
                for (ae = 0; ae < arguments.length; ae++) se.push(arguments[ae]);
                ne(se.join("<br>"), "long");
              }
            }, ee.warn = function() {
              var ae;
              if (te.logging > 0) {
                var oe = ["WARN:"];
                for (ae = 0; ae < arguments.length; ae++) oe.push(arguments[ae]);
                console.trace.apply(console, oe);
              }
              if (te.notifyOnLogging > 0) {
                var se = [];
                for (ae = 0; ae < arguments.length; ae++) se.push(arguments[ae]);
                ne(se.join("<br>"), "stick");
              }
            }, ee.error = function() {
              var ae;
              if (te.logging > 0) {
                var oe = ["ERROR:"];
                for (ae = 0; ae < arguments.length; ae++) oe.push(arguments[ae]);
                console.error.apply(console, oe);
              }
              if (te.notifyOnLogging > 0) {
                var se = [];
                for (ae = 0; ae < arguments.length; ae++) se.push(arguments[ae]);
                ne(se.join("<br>"), "stick");
              }
            };
          }, 75944: function(G, J, Z) {
            var te = Z(45568);
            G.exports = function(ne, ee, ae) {
              var oe = ne.selectAll("g." + ae.replace(/\s/g, ".")).data(ee, function(ce) {
                return ce[0].trace.uid;
              });
              oe.exit().remove(), oe.enter().append("g").attr("class", ae), oe.order();
              var se = ne.classed("rangeplot") ? "nodeRangePlot3" : "node3";
              return oe.each(function(ce) {
                ce[0][se] = te.select(this);
              }), oe;
            };
          }, 15236: function(G, J, Z) {
            var te = Z(11191);
            J.init2dArray = function(ne, ee) {
              for (var ae = new Array(ne), oe = 0; oe < ne; oe++) ae[oe] = new Array(ee);
              return ae;
            }, J.transposeRagged = function(ne) {
              var ee, ae, oe = 0, se = ne.length;
              for (ee = 0; ee < se; ee++) oe = Math.max(oe, ne[ee].length);
              var ce = new Array(oe);
              for (ee = 0; ee < oe; ee++) for (ce[ee] = new Array(se), ae = 0; ae < se; ae++) ce[ee][ae] = ne[ae][ee];
              return ce;
            }, J.dot = function(ne, ee) {
              if (!ne.length || !ee.length || ne.length !== ee.length) return null;
              var ae, oe, se = ne.length;
              if (ne[0].length) for (ae = new Array(se), oe = 0; oe < se; oe++) ae[oe] = J.dot(ne[oe], ee);
              else if (ee[0].length) {
                var ce = J.transposeRagged(ee);
                for (ae = new Array(ce.length), oe = 0; oe < ce.length; oe++) ae[oe] = J.dot(ne, ce[oe]);
              } else for (ae = 0, oe = 0; oe < se; oe++) ae += ne[oe] * ee[oe];
              return ae;
            }, J.translationMatrix = function(ne, ee) {
              return [[1, 0, ne], [0, 1, ee], [0, 0, 1]];
            }, J.rotationMatrix = function(ne) {
              var ee = ne * Math.PI / 180;
              return [[Math.cos(ee), -Math.sin(ee), 0], [Math.sin(ee), Math.cos(ee), 0], [0, 0, 1]];
            }, J.rotationXYMatrix = function(ne, ee, ae) {
              return J.dot(J.dot(J.translationMatrix(ee, ae), J.rotationMatrix(ne)), J.translationMatrix(-ee, -ae));
            }, J.apply3DTransform = function(ne) {
              return function() {
                var ee = arguments, ae = arguments.length === 1 ? ee[0] : [ee[0], ee[1], ee[2] || 0];
                return J.dot(ne, [ae[0], ae[1], ae[2], 1]).slice(0, 3);
              };
            }, J.apply2DTransform = function(ne) {
              return function() {
                var ee = arguments;
                ee.length === 3 && (ee = ee[0]);
                var ae = arguments.length === 1 ? ee[0] : [ee[0], ee[1]];
                return J.dot(ne, [ae[0], ae[1], 1]).slice(0, 2);
              };
            }, J.apply2DTransform2 = function(ne) {
              var ee = J.apply2DTransform(ne);
              return function(ae) {
                return ee(ae.slice(0, 2)).concat(ee(ae.slice(2, 4)));
              };
            }, J.convertCssMatrix = function(ne) {
              if (ne) {
                var ee = ne.length;
                if (ee === 16) return ne;
                if (ee === 6) return [ne[0], ne[1], 0, 0, ne[2], ne[3], 0, 0, 0, 0, 1, 0, ne[4], ne[5], 0, 1];
              }
              return [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
            }, J.inverseTransformMatrix = function(ne) {
              var ee = [];
              return te.invert(ee, ne), [[ee[0], ee[1], ee[2], ee[3]], [ee[4], ee[5], ee[6], ee[7]], [ee[8], ee[9], ee[10], ee[11]], [ee[12], ee[13], ee[14], ee[15]]];
            };
          }, 98953: function(G) {
            G.exports = { mod: function(J, Z) {
              var te = J % Z;
              return te < 0 ? te + Z : te;
            }, modHalf: function(J, Z) {
              return Math.abs(J) > Z / 2 ? J - Math.round(J / Z) * Z : J;
            } };
          }, 35632: function(G, J, Z) {
            var te = Z(10721), ne = Z(87800).isArrayOrTypedArray;
            function ee(de, fe) {
              return function() {
                var he, me, pe, ye, ge, ve = de;
                for (ye = 0; ye < fe.length - 1; ye++) {
                  if ((he = fe[ye]) === -1) {
                    for (me = !0, pe = [], ge = 0; ge < ve.length; ge++) pe[ge] = ee(ve[ge], fe.slice(ye + 1))(), pe[ge] !== pe[0] && (me = !1);
                    return me ? pe[0] : pe;
                  }
                  if (typeof he == "number" && !ne(ve) || typeof (ve = ve[he]) != "object" || ve === null) return;
                }
                if (typeof ve == "object" && ve !== null && (pe = ve[fe[ye]]) !== null) return pe;
              };
            }
            G.exports = function(de, fe) {
              if (te(fe)) fe = String(fe);
              else if (typeof fe != "string" || fe.substr(fe.length - 4) === "[-1]") throw "bad property string";
              var he, me, pe, ye, ge = fe.split(".");
              for (ye = 0; ye < ge.length; ye++) if (String(ge[ye]).slice(0, 2) === "__") throw "bad property string";
              for (ye = 0; ye < ge.length; ) {
                if (he = String(ge[ye]).match(/^([^\[\]]*)((\[\-?[0-9]*\])+)$/)) {
                  if (he[1]) ge[ye] = he[1];
                  else {
                    if (ye !== 0) throw "bad property string";
                    ge.splice(0, 1);
                  }
                  for (me = he[2].substr(1, he[2].length - 2).split("]["), pe = 0; pe < me.length; pe++) ye++, ge.splice(ye, 0, Number(me[pe]));
                }
                ye++;
              }
              return typeof de != "object" ? /* @__PURE__ */ function(ve, be, Ce) {
                return { set: function() {
                  throw "bad container";
                }, get: function() {
                }, astr: be, parts: Ce, obj: ve };
              }(de, fe, ge) : { set: se(de, ge, fe), get: ee(de, ge), astr: fe, parts: ge, obj: de };
            };
            var ae = /(^|\.)args\[/;
            function oe(de, fe) {
              return de === void 0 || de === null && !fe.match(ae);
            }
            function se(de, fe, he) {
              return function(me) {
                var pe, ye, ge = de, ve = "", be = oe(me, he);
                for (ye = 0; ye < fe.length - 1; ye++) {
                  if (typeof (pe = fe[ye]) == "number" && !ne(ge)) throw "array index but container is not an array";
                  if (pe === -1) {
                    if (be = !le(ge, fe.slice(ye + 1), me, he)) break;
                    return;
                  }
                  if (!ue(ge, pe, fe[ye + 1], be)) break;
                  if (typeof (ge = ge[pe]) != "object" || ge === null) throw "container is not an object";
                  ve = ce(ve, pe);
                }
                if (be) {
                  if (ye === fe.length - 1 && (delete ge[fe[ye]], Array.isArray(ge) && +fe[ye] == ge.length - 1)) for (; ge.length && ge[ge.length - 1] === void 0; ) ge.pop();
                } else ge[fe[ye]] = me;
              };
            }
            function ce(de, fe) {
              var he = fe;
              return te(fe) ? he = "[" + fe + "]" : de && (he = "." + fe), de + he;
            }
            function le(de, fe, he, me) {
              var pe, ye = ne(he), ge = !0, ve = he, be = me.replace("-1", 0), Ce = !ye && oe(he, be), Se = fe[0];
              for (pe = 0; pe < de.length; pe++) be = me.replace("-1", pe), ye && (Ce = oe(ve = he[pe % he.length], be)), Ce && (ge = !1), ue(de, pe, Se, Ce) && se(de[pe], fe, me.replace("-1", pe))(ve);
              return ge;
            }
            function ue(de, fe, he, me) {
              if (de[fe] === void 0) {
                if (me) return !1;
                de[fe] = typeof he == "number" ? [] : {};
              }
              return !0;
            }
          }, 4969: function(G) {
            G.exports = function() {
            };
          }, 87355: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = [];
            G.exports = function(ae, oe) {
              if (ee.indexOf(ae) === -1) {
                ee.push(ae);
                var se = 1e3;
                ne(oe) ? se = oe : oe === "long" && (se = 3e3);
                var ce = te.select("body").selectAll(".plotly-notifier").data([0]);
                ce.enter().append("div").classed("plotly-notifier", !0), ce.selectAll(".notifier-note").data(ee).enter().append("div").classed("notifier-note", !0).style("opacity", 0).each(function(ue) {
                  var de = te.select(this);
                  de.append("button").classed("notifier-close", !0).html("&times;").on("click", function() {
                    de.transition().call(le);
                  });
                  for (var fe = de.append("p"), he = ue.split(/<br\s*\/?>/g), me = 0; me < he.length; me++) me && fe.append("br"), fe.append("span").text(he[me]);
                  oe === "stick" ? de.transition().duration(350).style("opacity", 1) : de.transition().duration(700).style("opacity", 1).transition().delay(se).call(le);
                });
              }
              function le(ue) {
                ue.duration(700).style("opacity", 0).each("end", function(de) {
                  var fe = ee.indexOf(de);
                  fe !== -1 && ee.splice(fe, 1), te.select(this).remove();
                });
              }
            };
          }, 93134: function(G, J, Z) {
            var te = Z(27983), ne = "data-savedcursor";
            G.exports = function(ee, ae) {
              var oe = ee.attr(ne);
              if (ae) {
                if (!oe) {
                  for (var se = (ee.attr("class") || "").split(" "), ce = 0; ce < se.length; ce++) {
                    var le = se[ce];
                    le.indexOf("cursor-") === 0 && ee.attr(ne, le.substr(7)).classed(le, !1);
                  }
                  ee.attr(ne) || ee.attr(ne, "!!");
                }
                te(ee, ae);
              } else oe && (ee.attr(ne, null), oe === "!!" ? te(ee) : te(ee, oe));
            };
          }, 80899: function(G, J, Z) {
            var te = Z(15236).dot, ne = Z(63821).BADNUM, ee = G.exports = {};
            ee.tester = function(ae) {
              var oe, se = ae.slice(), ce = se[0][0], le = ce, ue = se[0][1], de = ue;
              for (se[se.length - 1][0] === se[0][0] && se[se.length - 1][1] === se[0][1] || se.push(se[0]), oe = 1; oe < se.length; oe++) ce = Math.min(ce, se[oe][0]), le = Math.max(le, se[oe][0]), ue = Math.min(ue, se[oe][1]), de = Math.max(de, se[oe][1]);
              var fe, he = !1;
              se.length === 5 && (se[0][0] === se[1][0] ? se[2][0] === se[3][0] && se[0][1] === se[3][1] && se[1][1] === se[2][1] && (he = !0, fe = function(ye) {
                return ye[0] === se[0][0];
              }) : se[0][1] === se[1][1] && se[2][1] === se[3][1] && se[0][0] === se[3][0] && se[1][0] === se[2][0] && (he = !0, fe = function(ye) {
                return ye[1] === se[0][1];
              }));
              var me = !0, pe = se[0];
              for (oe = 1; oe < se.length; oe++) if (pe[0] !== se[oe][0] || pe[1] !== se[oe][1]) {
                me = !1;
                break;
              }
              return { xmin: ce, xmax: le, ymin: ue, ymax: de, pts: se, contains: he ? function(ye, ge) {
                var ve = ye[0], be = ye[1];
                return !(ve === ne || ve < ce || ve > le || be === ne || be < ue || be > de || ge && fe(ye));
              } : function(ye, ge) {
                var ve = ye[0], be = ye[1];
                if (ve === ne || ve < ce || ve > le || be === ne || be < ue || be > de) return !1;
                var Ce, Se, we, Ee, xe, Le = se.length, Qe = se[0][0], ke = se[0][1], Me = 0;
                for (Ce = 1; Ce < Le; Ce++) if (Se = Qe, we = ke, Qe = se[Ce][0], ke = se[Ce][1], !(ve < (Ee = Math.min(Se, Qe)) || ve > Math.max(Se, Qe) || be > Math.max(we, ke))) if (be < Math.min(we, ke)) ve !== Ee && Me++;
                else {
                  if (be === (xe = Qe === Se ? be : we + (ve - Se) * (ke - we) / (Qe - Se))) return Ce !== 1 || !ge;
                  be <= xe && ve !== Ee && Me++;
                }
                return Me % 2 == 1;
              }, isRect: he, degenerate: me };
            }, ee.isSegmentBent = function(ae, oe, se, ce) {
              var le, ue, de, fe = ae[oe], he = [ae[se][0] - fe[0], ae[se][1] - fe[1]], me = te(he, he), pe = Math.sqrt(me), ye = [-he[1] / pe, he[0] / pe];
              for (le = oe + 1; le < se; le++) if (ue = [ae[le][0] - fe[0], ae[le][1] - fe[1]], (de = te(ue, he)) < 0 || de > me || Math.abs(te(ue, ye)) > ce) return !0;
              return !1;
            }, ee.filter = function(ae, oe) {
              var se = [ae[0]], ce = 0, le = 0;
              function ue(de) {
                ae.push(de);
                var fe = se.length, he = ce;
                se.splice(le + 1);
                for (var me = he + 1; me < ae.length; me++) (me === ae.length - 1 || ee.isSegmentBent(ae, he, me + 1, oe)) && (se.push(ae[me]), se.length < fe - 2 && (ce = me, le = se.length - 1), he = me);
              }
              return ae.length > 1 && ue(ae.pop()), { addPt: ue, raw: ae, filtered: se };
            };
          }, 22459: function(G, J, Z) {
            var te = Z(97464), ne = Z(81330);
            G.exports = function(ee, ae, oe) {
              var se = ee._fullLayout, ce = !0;
              return se._glcanvas.each(function(le) {
                if (le.regl) le.regl.preloadCachedCode(oe);
                else if (!le.pick || se._has("parcoords")) {
                  try {
                    le.regl = ne({ canvas: this, attributes: { antialias: !le.pick, preserveDrawingBuffer: !0 }, pixelRatio: ee._context.plotGlPixelRatio || Z.g.devicePixelRatio, extensions: ae || [], cachedCode: oe || {} });
                  } catch {
                    ce = !1;
                  }
                  le.regl || (ce = !1), ce && this.addEventListener("webglcontextlost", function(ue) {
                    ee && ee.emit && ee.emit("plotly_webglcontextlost", { event: ue, layer: le.key });
                  }, !1);
                }
              }), ce || te({ container: se._glcontainer.node() }), ce;
            };
          }, 32521: function(G, J, Z) {
            var te = Z(10721), ne = Z(13087);
            G.exports = function(ee) {
              var ae;
              if (typeof (ae = ee && ee.hasOwnProperty("userAgent") ? ee.userAgent : function() {
                var fe;
                return typeof navigator < "u" && (fe = navigator.userAgent), fe && fe.headers && typeof fe.headers["user-agent"] == "string" && (fe = fe.headers["user-agent"]), fe;
              }()) != "string") return !0;
              var oe = ne({ ua: { headers: { "user-agent": ae } }, tablet: !0, featureDetect: !1 });
              if (!oe) {
                for (var se = ae.split(" "), ce = 1; ce < se.length; ce++) if (se[ce].indexOf("Safari") !== -1) for (var le = ce - 1; le > -1; le--) {
                  var ue = se[le];
                  if (ue.substr(0, 8) === "Version/") {
                    var de = ue.substr(8).split(".")[0];
                    if (te(de) && (de = +de), de >= 13) return !0;
                  }
                }
              }
              return oe;
            };
          }, 36539: function(G) {
            G.exports = function(J, Z) {
              if (Z instanceof RegExp) {
                for (var te = Z.toString(), ne = 0; ne < J.length; ne++) if (J[ne] instanceof RegExp && J[ne].toString() === te) return J;
                J.push(Z);
              } else !Z && Z !== 0 || J.indexOf(Z) !== -1 || J.push(Z);
              return J;
            };
          }, 40486: function(G, J, Z) {
            var te = Z(34809), ne = Z(24452).dfltConfig, ee = { add: function(ae, oe, se, ce, le) {
              var ue, de;
              ae.undoQueue = ae.undoQueue || { index: 0, queue: [], sequence: !1 }, de = ae.undoQueue.index, ae.autoplay ? ae.undoQueue.inSequence || (ae.autoplay = !1) : (!ae.undoQueue.sequence || ae.undoQueue.beginSequence ? (ue = { undo: { calls: [], args: [] }, redo: { calls: [], args: [] } }, ae.undoQueue.queue.splice(de, ae.undoQueue.queue.length - de, ue), ae.undoQueue.index += 1) : ue = ae.undoQueue.queue[de - 1], ae.undoQueue.beginSequence = !1, ue && (ue.undo.calls.unshift(oe), ue.undo.args.unshift(se), ue.redo.calls.push(ce), ue.redo.args.push(le)), ae.undoQueue.queue.length > ne.queueLength && (ae.undoQueue.queue.shift(), ae.undoQueue.index--));
            }, startSequence: function(ae) {
              ae.undoQueue = ae.undoQueue || { index: 0, queue: [], sequence: !1 }, ae.undoQueue.sequence = !0, ae.undoQueue.beginSequence = !0;
            }, stopSequence: function(ae) {
              ae.undoQueue = ae.undoQueue || { index: 0, queue: [], sequence: !1 }, ae.undoQueue.sequence = !1, ae.undoQueue.beginSequence = !1;
            }, undo: function(ae) {
              var oe, se;
              if (!(ae.undoQueue === void 0 || isNaN(ae.undoQueue.index) || ae.undoQueue.index <= 0)) {
                for (ae.undoQueue.index--, oe = ae.undoQueue.queue[ae.undoQueue.index], ae.undoQueue.inSequence = !0, se = 0; se < oe.undo.calls.length; se++) ee.plotDo(ae, oe.undo.calls[se], oe.undo.args[se]);
                ae.undoQueue.inSequence = !1, ae.autoplay = !1;
              }
            }, redo: function(ae) {
              var oe, se;
              if (!(ae.undoQueue === void 0 || isNaN(ae.undoQueue.index) || ae.undoQueue.index >= ae.undoQueue.queue.length)) {
                for (oe = ae.undoQueue.queue[ae.undoQueue.index], ae.undoQueue.inSequence = !0, se = 0; se < oe.redo.calls.length; se++) ee.plotDo(ae, oe.redo.calls[se], oe.redo.args[se]);
                ae.undoQueue.inSequence = !1, ae.autoplay = !1, ae.undoQueue.index++;
              }
            }, plotDo: function(ae, oe, se) {
              ae.autoplay = !0, se = function(ce, le) {
                for (var ue, de = [], fe = 0; fe < le.length; fe++) ue = le[fe], de[fe] = ue === ce ? ue : typeof ue == "object" ? Array.isArray(ue) ? te.extendDeep([], ue) : te.extendDeepAll({}, ue) : ue;
                return de;
              }(ae, se), oe.apply(null, se);
            } };
            G.exports = ee;
          }, 90694: function(G, J) {
            J.counter = function(Z, te, ne, ee) {
              var ae = (te || "") + (ne ? "" : "$"), oe = ee === !1 ? "" : "^";
              return Z === "xy" ? new RegExp(oe + "x([2-9]|[1-9][0-9]+)?y([2-9]|[1-9][0-9]+)?" + ae) : new RegExp(oe + Z + "([2-9]|[1-9][0-9]+)?" + ae);
            };
          }, 82047: function(G) {
            var J = /^(.*)(\.[^\.\[\]]+|\[\d\])$/, Z = /^[^\.\[\]]+$/;
            G.exports = function(te, ne) {
              for (; ne; ) {
                var ee = te.match(J);
                if (ee) te = ee[1];
                else {
                  if (!te.match(Z)) throw new Error("bad relativeAttr call:" + [te, ne]);
                  te = "";
                }
                if (ne.charAt(0) !== "^") break;
                ne = ne.slice(1);
              }
              return te && ne.charAt(0) !== "[" ? te + "." + ne : te + ne;
            };
          }, 80428: function(G, J, Z) {
            var te = Z(87800).isArrayOrTypedArray, ne = Z(56174);
            G.exports = function ee(ae, oe) {
              for (var se in oe) {
                var ce = oe[se], le = ae[se];
                if (le !== ce) if (se.charAt(0) === "_" || typeof ce == "function") {
                  if (se in ae) continue;
                  ae[se] = ce;
                } else if (te(ce) && te(le) && ne(ce[0])) {
                  if (se === "customdata" || se === "ids") continue;
                  for (var ue = Math.min(ce.length, le.length), de = 0; de < ue; de++) le[de] !== ce[de] && ne(ce[de]) && ne(le[de]) && ee(le[de], ce[de]);
                } else ne(ce) && ne(le) && (ee(le, ce), Object.keys(le).length || delete ae[se]);
              }
            };
          }, 98813: function(G, J, Z) {
            var te = Z(10721), ne = Z(48636), ee = Z(29527), ae = Z(63821).BADNUM, oe = 1e-9;
            function se(de, fe) {
              return de < fe;
            }
            function ce(de, fe) {
              return de <= fe;
            }
            function le(de, fe) {
              return de > fe;
            }
            function ue(de, fe) {
              return de >= fe;
            }
            J.findBin = function(de, fe, he) {
              if (te(fe.start)) return he ? Math.ceil((de - fe.start) / fe.size - oe) - 1 : Math.floor((de - fe.start) / fe.size + oe);
              var me, pe, ye = 0, ge = fe.length, ve = 0, be = ge > 1 ? (fe[ge - 1] - fe[0]) / (ge - 1) : 1;
              for (pe = be >= 0 ? he ? se : ce : he ? ue : le, de += be * oe * (he ? -1 : 1) * (be >= 0 ? 1 : -1); ye < ge && ve++ < 100; ) pe(fe[me = Math.floor((ye + ge) / 2)], de) ? ye = me + 1 : ge = me;
              return ve > 90 && ne.log("Long binary search..."), ye - 1;
            }, J.sorterAsc = function(de, fe) {
              return de - fe;
            }, J.sorterDes = function(de, fe) {
              return fe - de;
            }, J.distinctVals = function(de) {
              var fe, he = de.slice();
              for (he.sort(J.sorterAsc), fe = he.length - 1; fe > -1 && he[fe] === ae; fe--) ;
              for (var me, pe = he[fe] - he[0] || 1, ye = pe / (fe || 1) / 1e4, ge = [], ve = 0; ve <= fe; ve++) {
                var be = he[ve], Ce = be - me;
                me === void 0 ? (ge.push(be), me = be) : Ce > ye && (pe = Math.min(pe, Ce), ge.push(be), me = be);
              }
              return { vals: ge, minDiff: pe };
            }, J.roundUp = function(de, fe, he) {
              for (var me, pe = 0, ye = fe.length - 1, ge = 0, ve = he ? 0 : 1, be = he ? 1 : 0, Ce = he ? Math.ceil : Math.floor; pe < ye && ge++ < 100; ) fe[me = Ce((pe + ye) / 2)] <= de ? pe = me + ve : ye = me - be;
              return fe[pe];
            }, J.sort = function(de, fe) {
              for (var he = 0, me = 0, pe = 1; pe < de.length; pe++) {
                var ye = fe(de[pe], de[pe - 1]);
                if (ye < 0 ? he = 1 : ye > 0 && (me = 1), he && me) return de.sort(fe);
              }
              return me ? de : de.reverse();
            }, J.findIndexOfMin = function(de, fe) {
              fe = fe || ee;
              for (var he, me = 1 / 0, pe = 0; pe < de.length; pe++) {
                var ye = fe(de[pe]);
                ye < me && (me = ye, he = pe);
              }
              return he;
            };
          }, 27983: function(G) {
            G.exports = function(J, Z) {
              (J.attr("class") || "").split(" ").forEach(function(te) {
                te.indexOf("cursor-") === 0 && J.classed(te, !1);
              }), Z && J.classed("cursor-" + Z, !0);
            };
          }, 97464: function(G, J, Z) {
            var te = Z(78766), ne = function() {
            };
            G.exports = function(ee) {
              for (var ae in ee) typeof ee[ae] == "function" && (ee[ae] = ne);
              ee.destroy = function() {
                ee.container.parentNode.removeChild(ee.container);
              };
              var oe = document.createElement("div");
              oe.className = "no-webgl", oe.style.cursor = "pointer", oe.style.fontSize = "24px", oe.style.color = te.defaults[0], oe.style.position = "absolute", oe.style.left = oe.style.top = "0px", oe.style.width = oe.style.height = "100%", oe.style["background-color"] = te.lightLine, oe.style["z-index"] = 30;
              var se = document.createElement("p");
              return se.textContent = "WebGL is not supported by your browser - visit https://get.webgl.org for more info", se.style.position = "relative", se.style.top = "50%", se.style.left = "50%", se.style.height = "30%", se.style.width = "50%", se.style.margin = "-15% 0 0 -25%", oe.appendChild(se), ee.container.appendChild(oe), ee.container.style.background = "#FFFFFF", ee.container.onclick = function() {
                window.open("https://get.webgl.org");
              }, !1;
            };
          }, 62994: function(G) {
            G.exports = function(J) {
              return Object.keys(J).sort();
            };
          }, 89258: function(G, J, Z) {
            var te = Z(10721), ne = Z(87800).isArrayOrTypedArray;
            J.aggNums = function(ee, ae, oe, se) {
              var ce, le;
              if ((!se || se > oe.length) && (se = oe.length), te(ae) || (ae = !1), ne(oe[0])) {
                for (le = new Array(se), ce = 0; ce < se; ce++) le[ce] = J.aggNums(ee, ae, oe[ce]);
                oe = le;
              }
              for (ce = 0; ce < se; ce++) te(ae) ? te(oe[ce]) && (ae = ee(+ae, +oe[ce])) : ae = oe[ce];
              return ae;
            }, J.len = function(ee) {
              return J.aggNums(function(ae) {
                return ae + 1;
              }, 0, ee);
            }, J.mean = function(ee, ae) {
              return ae || (ae = J.len(ee)), J.aggNums(function(oe, se) {
                return oe + se;
              }, 0, ee) / ae;
            }, J.geometricMean = function(ee, ae) {
              return ae || (ae = J.len(ee)), Math.pow(J.aggNums(function(oe, se) {
                return oe * se;
              }, 1, ee), 1 / ae);
            }, J.midRange = function(ee) {
              if (ee !== void 0 && ee.length !== 0) return (J.aggNums(Math.max, null, ee) + J.aggNums(Math.min, null, ee)) / 2;
            }, J.variance = function(ee, ae, oe) {
              return ae || (ae = J.len(ee)), te(oe) || (oe = J.mean(ee, ae)), J.aggNums(function(se, ce) {
                return se + Math.pow(ce - oe, 2);
              }, 0, ee) / ae;
            }, J.stdev = function(ee, ae, oe) {
              return Math.sqrt(J.variance(ee, ae, oe));
            }, J.median = function(ee) {
              var ae = ee.slice().sort();
              return J.interp(ae, 0.5);
            }, J.interp = function(ee, ae) {
              if (!te(ae)) throw "n should be a finite number";
              if ((ae = ae * ee.length - 0.5) < 0) return ee[0];
              if (ae > ee.length - 1) return ee[ee.length - 1];
              var oe = ae % 1;
              return oe * ee[Math.ceil(ae)] + (1 - oe) * ee[Math.floor(ae)];
            };
          }, 55010: function(G, J, Z) {
            var te = Z(162);
            G.exports = function(ne) {
              return ne ? te(ne) : [0, 0, 0, 1];
            };
          }, 95544: function(G, J, Z) {
            var te = Z(1837), ne = Z(62203), ee = Z(34809), ae = null;
            G.exports = function() {
              if (ae !== null) return ae;
              ae = !1;
              var oe = ee.isIE() || ee.isSafari() || ee.isIOS();
              if (window.navigator.userAgent && !oe) {
                var se = Array.from(te.CSS_DECLARATIONS).reverse(), ce = window.CSS && window.CSS.supports || window.supportsCSS;
                if (typeof ce == "function") ae = se.some(function(de) {
                  return ce.apply(null, de);
                });
                else {
                  var le = ne.tester.append("image").attr("style", te.STYLE), ue = window.getComputedStyle(le.node()).imageRendering;
                  ae = se.some(function(de) {
                    var fe = de[1];
                    return ue === fe || ue === fe.toLowerCase();
                  }), le.remove();
                }
              }
              return ae;
            };
          }, 30635: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = ne.strTranslate, ae = Z(62972), oe = Z(4530).LINE_SPACING, se = /([^$]*)([$]+[^$]*[$]+)([^$]*)/;
            J.convertToTspans = function($e, qe, Xe) {
              var Ve = $e.text(), Ze = !$e.attr("data-notex") && qe && qe._context.typesetMath && typeof MathJax < "u" && Ve.match(se), Ge = te.select($e.node().parentNode);
              if (!Ge.empty()) {
                var ze = $e.attr("class") ? $e.attr("class").split(" ")[0] : "text";
                return ze += "-math", Ge.selectAll("svg." + ze).remove(), Ge.selectAll("g." + ze + "-group").remove(), $e.style("display", null).attr({ "data-unformatted": Ve, "data-math": "N" }), Ze ? (qe && qe._promises || []).push(new Promise(function(at) {
                  $e.style("display", "none");
                  var ct = parseInt($e.node().style.fontSize, 10), et = { fontSize: ct };
                  (function(dt, Je, it) {
                    var st, gt, bt, yt, St = parseInt((MathJax.version || "").split(".")[0]);
                    if (St === 2 || St === 3) {
                      var Lt = function() {
                        var vt = "math-output-" + ne.randstr({}, 64), $t = (yt = te.select("body").append("div").attr({ id: vt }).style({ visibility: "hidden", position: "absolute", "font-size": Je.fontSize + "px" }).text(dt.replace(ce, "\\lt ").replace(le, "\\gt "))).node();
                        return St === 2 ? MathJax.Hub.Typeset($t) : MathJax.typeset([$t]);
                      }, Rt = function() {
                        var vt = yt.select(St === 2 ? ".MathJax_SVG" : ".MathJax"), $t = !vt.empty() && yt.select("svg").node();
                        if ($t) {
                          var Pt, Xt = $t.getBoundingClientRect();
                          Pt = St === 2 ? te.select("body").select("#MathJax_SVG_glyphs") : vt.select("defs"), it(vt, Pt, Xt);
                        } else ne.log("There was an error in the tex syntax.", dt), it();
                        yt.remove();
                      };
                      St === 2 ? MathJax.Hub.Queue(function() {
                        return gt = ne.extendDeepAll({}, MathJax.Hub.config), bt = MathJax.Hub.processSectionDelay, MathJax.Hub.processSectionDelay !== void 0 && (MathJax.Hub.processSectionDelay = 0), MathJax.Hub.Config({ messageStyle: "none", tex2jax: { inlineMath: ue }, displayAlign: "left" });
                      }, function() {
                        if ((st = MathJax.Hub.config.menuSettings.renderer) !== "SVG") return MathJax.Hub.setRenderer("SVG");
                      }, Lt, Rt, function() {
                        if (st !== "SVG") return MathJax.Hub.setRenderer(st);
                      }, function() {
                        return bt !== void 0 && (MathJax.Hub.processSectionDelay = bt), MathJax.Hub.Config(gt);
                      }) : St === 3 && (gt = ne.extendDeepAll({}, MathJax.config), MathJax.config.tex || (MathJax.config.tex = {}), MathJax.config.tex.inlineMath = ue, (st = MathJax.config.startup.output) !== "svg" && (MathJax.config.startup.output = "svg"), MathJax.startup.defaultReady(), MathJax.startup.promise.then(function() {
                        Lt(), Rt(), st !== "svg" && (MathJax.config.startup.output = st), MathJax.config = gt;
                      }));
                    } else ne.warn("No MathJax version:", MathJax.version);
                  })(Ze[2], et, function(dt, Je, it) {
                    Ge.selectAll("svg." + ze).remove(), Ge.selectAll("g." + ze + "-group").remove();
                    var st = dt && dt.select("svg");
                    if (!st || !st.node()) return We(), void at();
                    var gt = Ge.append("g").classed(ze + "-group", !0).attr({ "pointer-events": "none", "data-unformatted": Ve, "data-math": "Y" });
                    gt.node().appendChild(st.node()), Je && Je.node() && st.node().insertBefore(Je.node().cloneNode(!0), st.node().firstChild);
                    var bt = it.width, yt = it.height;
                    st.attr({ class: ze, height: yt, preserveAspectRatio: "xMinYMin meet" }).style({ overflow: "visible", "pointer-events": "none" });
                    var St = $e.node().style.fill || "black", Lt = st.select("g");
                    Lt.attr({ fill: St, stroke: St });
                    var Rt = Lt.node().getBoundingClientRect(), vt = Rt.width, $t = Rt.height;
                    (vt > bt || $t > yt) && (st.style("overflow", "hidden"), vt = (Rt = st.node().getBoundingClientRect()).width, $t = Rt.height);
                    var Pt = +$e.attr("x"), Xt = +$e.attr("y"), Nt = -(ct || $e.node().getBoundingClientRect().height) / 4;
                    if (ze[0] === "y") gt.attr({ transform: "rotate(" + [-90, Pt, Xt] + ")" + ee(-vt / 2, Nt - $t / 2) });
                    else if (ze[0] === "l") Xt = Nt - $t / 2;
                    else if (ze[0] === "a" && ze.indexOf("atitle") !== 0) Pt = 0, Xt = Nt;
                    else {
                      var Gt = $e.attr("text-anchor");
                      Pt -= vt * (Gt === "middle" ? 0.5 : Gt === "end" ? 1 : 0), Xt = Xt + Nt - $t / 2;
                    }
                    st.attr({ x: Pt, y: Xt }), Xe && Xe.call($e, gt), at(gt);
                  });
                })) : We(), $e;
              }
              function We() {
                Ge.empty() || (ze = $e.attr("class") + "-math", Ge.select("svg." + ze).remove()), $e.text("").style("white-space", "pre");
                var at = function(ct, et) {
                  et = et.replace(ye, " ");
                  var dt, Je = !1, it = [], st = -1;
                  function gt() {
                    st++;
                    var Ct = document.createElementNS(ae.svg, "tspan");
                    te.select(Ct).attr({ class: "line", dy: st * oe + "em" }), ct.appendChild(Ct), dt = Ct;
                    var Kt = it;
                    if (it = [{ node: Ct }], Kt.length > 1) for (var Vt = 1; Vt < Kt.length; Vt++) bt(Kt[Vt]);
                  }
                  function bt(Ct) {
                    var Kt, Vt = Ct.type, Yt = {};
                    if (Vt === "a") {
                      Kt = "a";
                      var sn = Ct.target, hn = Ct.href, Qt = Ct.popup;
                      hn && (Yt = { "xlink:xlink:show": sn === "_blank" || sn.charAt(0) !== "_" ? "new" : "replace", target: sn, "xlink:xlink:href": hn }, Qt && (Yt.onclick = 'window.open(this.href.baseVal,this.target.baseVal,"' + Qt + '");return false;'));
                    } else Kt = "tspan";
                    Ct.style && (Yt.style = Ct.style);
                    var Ut = document.createElementNS(ae.svg, Kt);
                    if (Vt === "sup" || Vt === "sub") {
                      yt(dt, me), dt.appendChild(Ut);
                      var Wt = document.createElementNS(ae.svg, "tspan");
                      yt(Wt, me), te.select(Wt).attr("dy", he[Vt]), Yt.dy = fe[Vt], dt.appendChild(Ut), dt.appendChild(Wt);
                    } else dt.appendChild(Ut);
                    te.select(Ut).attr(Yt), dt = Ct.node = Ut, it.push(Ct);
                  }
                  function yt(Ct, Kt) {
                    Ct.appendChild(document.createTextNode(Kt));
                  }
                  function St(Ct) {
                    if (it.length !== 1) {
                      var Kt = it.pop();
                      Ct !== Kt.type && ne.log("Start tag <" + Kt.type + "> doesnt match end tag <" + Ct + ">. Pretending it did match.", et), dt = it[it.length - 1].node;
                    } else ne.log("Ignoring unexpected end tag </" + Ct + ">.", et);
                  }
                  be.test(et) ? gt() : (dt = ct, it = [{ node: ct }]);
                  for (var Lt = et.split(ge), Rt = 0; Rt < Lt.length; Rt++) {
                    var vt = Lt[Rt], $t = vt.match(ve), Pt = $t && $t[2].toLowerCase(), Xt = de[Pt];
                    if (Pt === "br") gt();
                    else if (Xt === void 0) yt(dt, Me(vt));
                    else if ($t[1]) St(Pt);
                    else {
                      var Nt = $t[4], Gt = { type: Pt }, Ft = xe(Nt, Ce);
                      if (Ft ? (Ft = Ft.replace(Le, "$1 fill:"), Xt && (Ft += ";" + Xt)) : Xt && (Ft = Xt), Ft && (Gt.style = Ft), Pt === "a") {
                        Je = !0;
                        var Tt = xe(Nt, Se);
                        if (Tt) {
                          var It = Re(Tt);
                          It && (Gt.href = It, Gt.target = xe(Nt, we) || "_blank", Gt.popup = xe(Nt, Ee));
                        }
                      }
                      bt(Gt);
                    }
                  }
                  return Je;
                }($e.node(), Ve);
                at && $e.style("pointer-events", "all"), J.positionText($e), Xe && Xe.call($e);
              }
            };
            var ce = /(<|&lt;|&#60;)/g, le = /(>|&gt;|&#62;)/g, ue = [["$", "$"], ["\\(", "\\)"]], de = { sup: "font-size:70%", sub: "font-size:70%", s: "text-decoration:line-through", u: "text-decoration:underline", b: "font-weight:bold", i: "font-style:italic", a: "cursor:pointer", span: "", em: "font-style:italic;font-weight:bold" }, fe = { sub: "0.3em", sup: "-0.6em" }, he = { sub: "-0.21em", sup: "0.42em" }, me = "​", pe = ["http:", "https:", "mailto:", "", void 0, ":"], ye = J.NEWLINES = /(\r\n?|\n)/g, ge = /(<[^<>]*>)/, ve = /<(\/?)([^ >]*)(\s+(.*))?>/i, be = /<br(\s+.*)?>/i;
            J.BR_TAG_ALL = /<br(\s+.*)?>/gi;
            var Ce = /(^|[\s"'])style\s*=\s*("([^"]*);?"|'([^']*);?')/i, Se = /(^|[\s"'])href\s*=\s*("([^"]*)"|'([^']*)')/i, we = /(^|[\s"'])target\s*=\s*("([^"\s]*)"|'([^'\s]*)')/i, Ee = /(^|[\s"'])popup\s*=\s*("([\w=,]*)"|'([\w=,]*)')/i;
            function xe($e, qe) {
              if (!$e) return null;
              var Xe = $e.match(qe), Ve = Xe && (Xe[3] || Xe[4]);
              return Ve && Me(Ve);
            }
            var Le = /(^|;)\s*color:/;
            J.plainText = function($e, qe) {
              for (var Xe = (qe = qe || {}).len !== void 0 && qe.len !== -1 ? qe.len : 1 / 0, Ve = qe.allowedTags !== void 0 ? qe.allowedTags : ["br"], Ze = $e.split(ge), Ge = [], ze = "", We = 0, at = 0; at < Ze.length; at++) {
                var ct = Ze[at], et = ct.match(ve), dt = et && et[2].toLowerCase();
                if (dt) Ve.indexOf(dt) !== -1 && (Ge.push(ct), ze = dt);
                else {
                  var Je = ct.length;
                  if (We + Je < Xe) Ge.push(ct), We += Je;
                  else if (We < Xe) {
                    var it = Xe - We;
                    ze && (ze !== "br" || it <= 3 || Je <= 3) && Ge.pop(), Xe > 3 ? Ge.push(ct.substr(0, it - 3) + "...") : Ge.push(ct.substr(0, it));
                    break;
                  }
                  ze = "";
                }
              }
              return Ge.join("");
            };
            var Qe = { mu: "μ", amp: "&", lt: "<", gt: ">", nbsp: " ", times: "×", plusmn: "±", deg: "°" }, ke = /&(#\d+|#x[\da-fA-F]+|[a-z]+);/g;
            function Me($e) {
              return $e.replace(ke, function(qe, Xe) {
                return (Xe.charAt(0) === "#" ? function(Ve) {
                  if (!(Ve > 1114111)) {
                    var Ze = String.fromCodePoint;
                    if (Ze) return Ze(Ve);
                    var Ge = String.fromCharCode;
                    return Ve <= 65535 ? Ge(Ve) : Ge(55232 + (Ve >> 10), Ve % 1024 + 56320);
                  }
                }(Xe.charAt(1) === "x" ? parseInt(Xe.substr(2), 16) : parseInt(Xe.substr(1), 10)) : Qe[Xe]) || qe;
              });
            }
            function Re($e) {
              var qe = encodeURI(decodeURI($e)), Xe = document.createElement("a"), Ve = document.createElement("a");
              Xe.href = $e, Ve.href = qe;
              var Ze = Xe.protocol, Ge = Ve.protocol;
              return pe.indexOf(Ze) !== -1 && pe.indexOf(Ge) !== -1 ? qe : "";
            }
            function De($e, qe, Xe) {
              var Ve, Ze, Ge, ze = Xe.horizontalAlign, We = Xe.verticalAlign || "top", at = $e.node().getBoundingClientRect(), ct = qe.node().getBoundingClientRect();
              return Ze = We === "bottom" ? function() {
                return at.bottom - Ve.height;
              } : We === "middle" ? function() {
                return at.top + (at.height - Ve.height) / 2;
              } : function() {
                return at.top;
              }, Ge = ze === "right" ? function() {
                return at.right - Ve.width;
              } : ze === "center" ? function() {
                return at.left + (at.width - Ve.width) / 2;
              } : function() {
                return at.left;
              }, function() {
                Ve = this.node().getBoundingClientRect();
                var et = Ge() - ct.left, dt = Ze() - ct.top, Je = Xe.gd || {};
                if (Xe.gd) {
                  Je._fullLayout._calcInverseTransform(Je);
                  var it = ne.apply3DTransform(Je._fullLayout._invTransform)(et, dt);
                  et = it[0], dt = it[1];
                }
                return this.style({ top: dt + "px", left: et + "px", "z-index": 1e3 }), this;
              };
            }
            J.convertEntities = Me, J.sanitizeHTML = function($e) {
              $e = $e.replace(ye, " ");
              for (var qe = document.createElement("p"), Xe = qe, Ve = [], Ze = $e.split(ge), Ge = 0; Ge < Ze.length; Ge++) {
                var ze = Ze[Ge], We = ze.match(ve), at = We && We[2].toLowerCase();
                if (at in de) if (We[1]) Ve.length && (Xe = Ve.pop());
                else {
                  var ct = We[4], et = xe(ct, Ce), dt = et ? { style: et } : {};
                  if (at === "a") {
                    var Je = xe(ct, Se);
                    if (Je) {
                      var it = Re(Je);
                      if (it) {
                        dt.href = it;
                        var st = xe(ct, we);
                        st && (dt.target = st);
                      }
                    }
                  }
                  var gt = document.createElement(at);
                  Xe.appendChild(gt), te.select(gt).attr(dt), Xe = gt, Ve.push(gt);
                }
                else Xe.appendChild(document.createTextNode(Me(ze)));
              }
              return qe.innerHTML;
            }, J.lineCount = function($e) {
              return $e.selectAll("tspan.line").size() || 1;
            }, J.positionText = function($e, qe, Xe) {
              return $e.each(function() {
                var Ve = te.select(this);
                function Ze(We, at) {
                  return at === void 0 ? (at = Ve.attr(We)) === null && (Ve.attr(We, 0), at = 0) : Ve.attr(We, at), at;
                }
                var Ge = Ze("x", qe), ze = Ze("y", Xe);
                this.nodeName === "text" && Ve.selectAll("tspan.line").attr({ x: Ge, y: ze });
              });
            };
            var Pe = "1px ";
            J.makeTextShadow = function($e) {
              return Pe + Pe + Pe + $e + ", -" + Pe + "-" + Pe + Pe + $e + ", " + Pe + "-" + Pe + Pe + $e + ", -" + Pe + Pe + Pe + $e;
            }, J.makeEditable = function($e, qe) {
              var Xe = qe.gd, Ve = qe.delegate, Ze = te.dispatch("edit", "input", "cancel"), Ge = Ve || $e;
              if ($e.style({ "pointer-events": Ve ? "none" : "all" }), $e.size() !== 1) throw new Error("boo");
              function ze() {
                var at, ct, et, dt, Je;
                at = te.select(Xe).select(".svg-container"), ct = at.append("div"), et = $e.node().style, dt = parseFloat(et.fontSize || 12), (Je = qe.text) === void 0 && (Je = $e.attr("data-unformatted")), ct.classed("plugin-editable editable", !0).style({ position: "absolute", "font-family": et.fontFamily || "Arial", "font-size": dt, color: qe.fill || et.fill || "black", opacity: 1, "background-color": qe.background || "transparent", outline: "#ffffff33 1px solid", margin: [-dt / 8 + 1, 0, 0, -1].join("px ") + "px", padding: "0", "box-sizing": "border-box" }).attr({ contenteditable: !0 }).text(Je).call(De($e, at, qe)).on("blur", function() {
                  Xe._editing = !1, $e.text(this.textContent).style({ opacity: 1 });
                  var gt, bt = te.select(this).attr("class");
                  (gt = bt ? "." + bt.split(" ")[0] + "-math-group" : "[class*=-math-group]") && te.select($e.node().parentNode).select(gt).style({ opacity: 0 });
                  var yt = this.textContent;
                  te.select(this).transition().duration(0).remove(), te.select(document).on("mouseup", null), Ze.edit.call($e, yt);
                }).on("focus", function() {
                  var gt = this;
                  Xe._editing = !0, te.select(document).on("mouseup", function() {
                    if (te.event.target === gt) return !1;
                    document.activeElement === ct.node() && ct.node().blur();
                  });
                }).on("keyup", function() {
                  te.event.which === 27 ? (Xe._editing = !1, $e.style({ opacity: 1 }), te.select(this).style({ opacity: 0 }).on("blur", function() {
                    return !1;
                  }).transition().remove(), Ze.cancel.call($e, this.textContent)) : (Ze.input.call($e, this.textContent), te.select(this).call(De($e, at, qe)));
                }).on("keydown", function() {
                  te.event.which === 13 && this.blur();
                }).call(We), $e.style({ opacity: 0 });
                var it, st = Ge.attr("class");
                (it = st ? "." + st.split(" ")[0] + "-math-group" : "[class*=-math-group]") && te.select($e.node().parentNode).select(it).style({ opacity: 0 });
              }
              function We(at) {
                var ct = at.node(), et = document.createRange();
                et.selectNodeContents(ct);
                var dt = window.getSelection();
                dt.removeAllRanges(), dt.addRange(et), ct.focus();
              }
              return qe.immediate ? ze() : Ge.on("click", ze), te.rebind($e, Ze, "on");
            };
          }, 64025: function(G, J) {
            var Z = {};
            function te(ne) {
              ne && ne.timer !== null && (clearTimeout(ne.timer), ne.timer = null);
            }
            J.throttle = function(ne, ee, ae) {
              var oe = Z[ne], se = Date.now();
              if (!oe) {
                for (var ce in Z) Z[ce].ts < se - 6e4 && delete Z[ce];
                oe = Z[ne] = { ts: 0, timer: null };
              }
              function le() {
                ae(), oe.ts = Date.now(), oe.onDone && (oe.onDone(), oe.onDone = null);
              }
              te(oe), se > oe.ts + ee ? le() : oe.timer = setTimeout(function() {
                le(), oe.timer = null;
              }, ee);
            }, J.done = function(ne) {
              var ee = Z[ne];
              return ee && ee.timer ? new Promise(function(ae) {
                var oe = ee.onDone;
                ee.onDone = function() {
                  oe && oe(), ae(), ee.onDone = null;
                };
              }) : Promise.resolve();
            }, J.clear = function(ne) {
              if (ne) te(Z[ne]), delete Z[ne];
              else for (var ee in Z) J.clear(ee);
            };
          }, 8083: function(G, J, Z) {
            var te = Z(10721);
            G.exports = function(ne, ee) {
              if (ne > 0) return Math.log(ne) / Math.LN10;
              var ae = Math.log(Math.min(ee[0], ee[1])) / Math.LN10;
              return te(ae) || (ae = Math.log(Math.max(ee[0], ee[1])) / Math.LN10 - 6), ae;
            };
          }, 11577: function(G, J, Z) {
            var te = G.exports = {}, ne = Z(74285).locationmodeToLayer, ee = Z(48640).N4;
            te.getTopojsonName = function(ae) {
              return [ae.scope.replace(/ /g, "-"), "_", ae.resolution.toString(), "m"].join("");
            }, te.getTopojsonPath = function(ae, oe) {
              return ae + oe + ".json";
            }, te.getTopojsonFeatures = function(ae, oe) {
              var se = ne[ae.locationmode], ce = oe.objects[se];
              return ee(oe, ce).features;
            };
          }, 44611: function(G) {
            G.exports = { moduleType: "locale", name: "en-US", dictionary: { "Click to enter Colorscale title": "Click to enter Colorscale title" }, format: { date: "%m/%d/%Y" } };
          }, 30227: function(G) {
            G.exports = { moduleType: "locale", name: "en", dictionary: { "Click to enter Colorscale title": "Click to enter Colourscale title" }, format: { days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], shortDays: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], months: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"], shortMonths: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"], periods: ["AM", "PM"], dateTime: "%a %b %e %X %Y", date: "%d/%m/%Y", time: "%H:%M:%S", decimal: ".", thousands: ",", grouping: [3], currency: ["$", ""], year: "%Y", month: "%b %Y", dayMonth: "%b %-d", dayMonthYear: "%b %-d, %Y" } };
          }, 56037: function(G, J, Z) {
            var te = Z(33626);
            G.exports = function(ne) {
              for (var ee, ae, oe = te.layoutArrayContainers, se = te.layoutArrayRegexes, ce = ne.split("[")[0], le = 0; le < se.length; le++) if ((ae = ne.match(se[le])) && ae.index === 0) {
                ee = ae[0];
                break;
              }
              if (ee || (ee = oe[oe.indexOf(ce)]), !ee) return !1;
              var ue = ne.substr(ee.length);
              return ue ? !!(ae = ue.match(/^\[(0|[1-9][0-9]*)\](\.(.+))?$/)) && { array: ee, index: Number(ae[1]), property: ae[3] || "" } : { array: ee, index: "", property: "" };
            };
          }, 13582: function(G, J, Z) {
            var te = Z(93049).extendFlat, ne = Z(56174), ee = { valType: "flaglist", extras: ["none"], flags: ["calc", "clearAxisTypes", "plot", "style", "markerSize", "colorbars"] }, ae = { valType: "flaglist", extras: ["none"], flags: ["calc", "plot", "legend", "ticks", "axrange", "layoutstyle", "modebar", "camera", "arraydraw", "colorbars"] }, oe = ee.flags.slice().concat(["fullReplot"]), se = ae.flags.slice().concat("layoutReplot");
            function ce(de) {
              for (var fe = {}, he = 0; he < de.length; he++) fe[de[he]] = !1;
              return fe;
            }
            function le(de, fe, he) {
              var me = te({}, de);
              for (var pe in me) {
                var ye = me[pe];
                ne(ye) && (me[pe] = ue(ye, fe, 0, pe));
              }
              return he === "from-root" && (me.editType = fe), me;
            }
            function ue(de, fe, he, me) {
              if (de.valType) {
                var pe = te({}, de);
                if (pe.editType = fe, Array.isArray(de.items)) {
                  pe.items = new Array(de.items.length);
                  for (var ye = 0; ye < de.items.length; ye++) pe.items[ye] = ue(de.items[ye], fe);
                }
                return pe;
              }
              return le(de, fe, me.charAt(0) === "_" ? "nested" : "from-root");
            }
            G.exports = { traces: ee, layout: ae, traceFlags: function() {
              return ce(oe);
            }, layoutFlags: function() {
              return ce(se);
            }, update: function(de, fe) {
              var he = fe.editType;
              if (he && he !== "none") for (var me = he.split("+"), pe = 0; pe < me.length; pe++) de[me[pe]] = !0;
            }, overrideAll: le };
          }, 10887: function(G, J, Z) {
            var te = Z(10721), ne = Z(36472), ee = Z(33626), ae = Z(34809), oe = Z(44122), se = Z(5975), ce = Z(78766), le = se.cleanId, ue = se.getFromTrace, de = ee.traceIs;
            function fe(Se, we) {
              var Ee = Se[we], xe = we.charAt(0);
              Ee && Ee !== "paper" && (Se[we] = le(Ee, xe, !0));
            }
            function he(Se) {
              function we(Ee, xe) {
                var Le = Se[Ee], Qe = Se.title && Se.title[xe];
                Le && !Qe && (Se.title || (Se.title = {}), Se.title[xe] = Se[Ee], delete Se[Ee]);
              }
              Se && (typeof Se.title != "string" && typeof Se.title != "number" || (Se.title = { text: Se.title }), we("titlefont", "font"), we("titleposition", "position"), we("titleside", "side"), we("titleoffset", "offset"));
            }
            function me(Se) {
              if (!ae.isPlainObject(Se)) return !1;
              var we = Se.name;
              return delete Se.name, delete Se.showlegend, (typeof we == "string" || typeof we == "number") && String(we);
            }
            function pe(Se, we, Ee, xe) {
              if (Ee && !xe) return Se;
              if (xe && !Ee || !Se.trim()) return we;
              if (!we.trim()) return Se;
              var Le, Qe = Math.min(Se.length, we.length);
              for (Le = 0; Le < Qe && Se.charAt(Le) === we.charAt(Le); Le++) ;
              return Se.substr(0, Le).trim();
            }
            function ye(Se) {
              var we = "middle", Ee = "center";
              return typeof Se == "string" && (Se.indexOf("top") !== -1 ? we = "top" : Se.indexOf("bottom") !== -1 && (we = "bottom"), Se.indexOf("left") !== -1 ? Ee = "left" : Se.indexOf("right") !== -1 && (Ee = "right")), we + " " + Ee;
            }
            function ge(Se, we) {
              return we in Se && typeof Se[we] == "object" && Object.keys(Se[we]).length === 0;
            }
            J.clearPromiseQueue = function(Se) {
              Array.isArray(Se._promises) && Se._promises.length > 0 && ae.log("Clearing previous rejected promises from queue."), Se._promises = [];
            }, J.cleanLayout = function(Se) {
              var we, Ee;
              Se || (Se = {}), Se.xaxis1 && (Se.xaxis || (Se.xaxis = Se.xaxis1), delete Se.xaxis1), Se.yaxis1 && (Se.yaxis || (Se.yaxis = Se.yaxis1), delete Se.yaxis1), Se.scene1 && (Se.scene || (Se.scene = Se.scene1), delete Se.scene1);
              var xe = (oe.subplotsRegistry.cartesian || {}).attrRegex, Le = (oe.subplotsRegistry.polar || {}).attrRegex, Qe = (oe.subplotsRegistry.ternary || {}).attrRegex, ke = (oe.subplotsRegistry.gl3d || {}).attrRegex, Me = Object.keys(Se);
              for (we = 0; we < Me.length; we++) {
                var Re = Me[we];
                if (xe && xe.test(Re)) {
                  var De = Se[Re];
                  De.anchor && De.anchor !== "free" && (De.anchor = le(De.anchor)), De.overlaying && (De.overlaying = le(De.overlaying)), De.type || (De.isdate ? De.type = "date" : De.islog ? De.type = "log" : De.isdate === !1 && De.islog === !1 && (De.type = "linear")), De.autorange !== "withzero" && De.autorange !== "tozero" || (De.autorange = !0, De.rangemode = "tozero"), De.insiderange && delete De.range, delete De.islog, delete De.isdate, delete De.categories, ge(De, "domain") && delete De.domain, De.autotick !== void 0 && (De.tickmode === void 0 && (De.tickmode = De.autotick ? "auto" : "linear"), delete De.autotick), he(De);
                } else if (Le && Le.test(Re)) he(Se[Re].radialaxis);
                else if (Qe && Qe.test(Re)) {
                  var Pe = Se[Re];
                  he(Pe.aaxis), he(Pe.baxis), he(Pe.caxis);
                } else if (ke && ke.test(Re)) {
                  var $e = Se[Re], qe = $e.cameraposition;
                  if (Array.isArray(qe) && qe[0].length === 4) {
                    var Xe = qe[0], Ve = qe[1], Ze = qe[2], Ge = ne([], Xe), ze = [];
                    for (Ee = 0; Ee < 3; ++Ee) ze[Ee] = Ve[Ee] + Ze * Ge[2 + 4 * Ee];
                    $e.camera = { eye: { x: ze[0], y: ze[1], z: ze[2] }, center: { x: Ve[0], y: Ve[1], z: Ve[2] }, up: { x: 0, y: 0, z: 1 } }, delete $e.cameraposition;
                  }
                  he($e.xaxis), he($e.yaxis), he($e.zaxis);
                }
              }
              var We = Array.isArray(Se.annotations) ? Se.annotations.length : 0;
              for (we = 0; we < We; we++) {
                var at = Se.annotations[we];
                ae.isPlainObject(at) && (at.ref && (at.ref === "paper" ? (at.xref = "paper", at.yref = "paper") : at.ref === "data" && (at.xref = "x", at.yref = "y"), delete at.ref), fe(at, "xref"), fe(at, "yref"));
              }
              var ct = Array.isArray(Se.shapes) ? Se.shapes.length : 0;
              for (we = 0; we < ct; we++) {
                var et = Se.shapes[we];
                ae.isPlainObject(et) && (fe(et, "xref"), fe(et, "yref"));
              }
              var dt = Array.isArray(Se.images) ? Se.images.length : 0;
              for (we = 0; we < dt; we++) {
                var Je = Se.images[we];
                ae.isPlainObject(Je) && (fe(Je, "xref"), fe(Je, "yref"));
              }
              var it = Se.legend;
              return it && (it.x > 3 ? (it.x = 1.02, it.xanchor = "left") : it.x < -2 && (it.x = -0.02, it.xanchor = "right"), it.y > 3 ? (it.y = 1.02, it.yanchor = "bottom") : it.y < -2 && (it.y = -0.02, it.yanchor = "top")), he(Se), Se.dragmode === "rotate" && (Se.dragmode = "orbit"), ce.clean(Se), Se.template && Se.template.layout && J.cleanLayout(Se.template.layout), Se;
            }, J.cleanData = function(Se) {
              for (var we = 0; we < Se.length; we++) {
                var Ee, xe = Se[we];
                if (xe.type === "histogramy" && "xbins" in xe && !("ybins" in xe) && (xe.ybins = xe.xbins, delete xe.xbins), xe.error_y && "opacity" in xe.error_y) {
                  var Le = ce.defaults, Qe = xe.error_y.color || (de(xe, "bar") ? ce.defaultLine : Le[we % Le.length]);
                  xe.error_y.color = ce.addOpacity(ce.rgb(Qe), ce.opacity(Qe) * xe.error_y.opacity), delete xe.error_y.opacity;
                }
                if ("bardir" in xe && (xe.bardir !== "h" || !de(xe, "bar") && xe.type.substr(0, 9) !== "histogram" || (xe.orientation = "h", J.swapXYData(xe)), delete xe.bardir), xe.type === "histogramy" && J.swapXYData(xe), xe.type !== "histogramx" && xe.type !== "histogramy" || (xe.type = "histogram"), "scl" in xe && !("colorscale" in xe) && (xe.colorscale = xe.scl, delete xe.scl), "reversescl" in xe && !("reversescale" in xe) && (xe.reversescale = xe.reversescl, delete xe.reversescl), xe.xaxis && (xe.xaxis = le(xe.xaxis, "x")), xe.yaxis && (xe.yaxis = le(xe.yaxis, "y")), de(xe, "gl3d") && xe.scene && (xe.scene = oe.subplotsRegistry.gl3d.cleanId(xe.scene)), !de(xe, "pie-like") && !de(xe, "bar-like")) if (Array.isArray(xe.textposition)) for (Ee = 0; Ee < xe.textposition.length; Ee++) xe.textposition[Ee] = ye(xe.textposition[Ee]);
                else xe.textposition && (xe.textposition = ye(xe.textposition));
                var ke = ee.getModule(xe);
                if (ke && ke.colorbar) {
                  var Me = ke.colorbar.container, Re = Me ? xe[Me] : xe;
                  Re && Re.colorscale && (Re.colorscale === "YIGnBu" && (Re.colorscale = "YlGnBu"), Re.colorscale === "YIOrRd" && (Re.colorscale = "YlOrRd"));
                }
                if (xe.type === "surface" && ae.isPlainObject(xe.contours)) {
                  var De = ["x", "y", "z"];
                  for (Ee = 0; Ee < De.length; Ee++) {
                    var Pe = xe.contours[De[Ee]];
                    ae.isPlainObject(Pe) && (Pe.highlightColor && (Pe.highlightcolor = Pe.highlightColor, delete Pe.highlightColor), Pe.highlightWidth && (Pe.highlightwidth = Pe.highlightWidth, delete Pe.highlightWidth));
                  }
                }
                if (xe.type === "candlestick" || xe.type === "ohlc") {
                  var $e = (xe.increasing || {}).showlegend !== !1, qe = (xe.decreasing || {}).showlegend !== !1, Xe = me(xe.increasing), Ve = me(xe.decreasing);
                  if (Xe !== !1 && Ve !== !1) {
                    var Ze = pe(Xe, Ve, $e, qe);
                    Ze && (xe.name = Ze);
                  } else !Xe && !Ve || xe.name || (xe.name = Xe || Ve);
                }
                if (Array.isArray(xe.transforms)) {
                  var Ge = xe.transforms;
                  for (Ee = 0; Ee < Ge.length; Ee++) {
                    var ze = Ge[Ee];
                    if (ae.isPlainObject(ze)) switch (ze.type) {
                      case "filter":
                        ze.filtersrc && (ze.target = ze.filtersrc, delete ze.filtersrc), ze.calendar && (ze.valuecalendar || (ze.valuecalendar = ze.calendar), delete ze.calendar);
                        break;
                      case "groupby":
                        if (ze.styles = ze.styles || ze.style, ze.styles && !Array.isArray(ze.styles)) {
                          var We = ze.styles, at = Object.keys(We);
                          ze.styles = [];
                          for (var ct = 0; ct < at.length; ct++) ze.styles.push({ target: at[ct], value: We[at[ct]] });
                        }
                    }
                  }
                }
                ge(xe, "line") && delete xe.line, "marker" in xe && (ge(xe.marker, "line") && delete xe.marker.line, ge(xe, "marker") && delete xe.marker), ce.clean(xe), xe.autobinx && (delete xe.autobinx, delete xe.xbins), xe.autobiny && (delete xe.autobiny, delete xe.ybins), he(xe), xe.colorbar && he(xe.colorbar), xe.marker && xe.marker.colorbar && he(xe.marker.colorbar), xe.line && xe.line.colorbar && he(xe.line.colorbar), xe.aaxis && he(xe.aaxis), xe.baxis && he(xe.baxis);
              }
            }, J.swapXYData = function(Se) {
              var we;
              if (ae.swapAttrs(Se, ["?", "?0", "d?", "?bins", "nbins?", "autobin?", "?src", "error_?"]), Array.isArray(Se.z) && Array.isArray(Se.z[0]) && (Se.transpose ? delete Se.transpose : Se.transpose = !0), Se.error_x && Se.error_y) {
                var Ee = Se.error_y, xe = "copy_ystyle" in Ee ? Ee.copy_ystyle : !(Ee.color || Ee.thickness || Ee.width);
                ae.swapAttrs(Se, ["error_?.copy_ystyle"]), xe && ae.swapAttrs(Se, ["error_?.color", "error_?.thickness", "error_?.width"]);
              }
              if (typeof Se.hoverinfo == "string") {
                var Le = Se.hoverinfo.split("+");
                for (we = 0; we < Le.length; we++) Le[we] === "x" ? Le[we] = "y" : Le[we] === "y" && (Le[we] = "x");
                Se.hoverinfo = Le.join("+");
              }
            }, J.coerceTraceIndices = function(Se, we) {
              if (te(we)) return [we];
              if (!Array.isArray(we) || !we.length) return Se.data.map(function(Le, Qe) {
                return Qe;
              });
              if (Array.isArray(we)) {
                for (var Ee = [], xe = 0; xe < we.length; xe++) ae.isIndex(we[xe], Se.data.length) ? Ee.push(we[xe]) : ae.warn("trace index (", we[xe], ") is not a number or is out of bounds");
                return Ee;
              }
              return we;
            }, J.manageArrayContainers = function(Se, we, Ee) {
              var xe = Se.obj, Le = Se.parts, Qe = Le.length, ke = Le[Qe - 1], Me = te(ke);
              if (Me && we === null) {
                var Re = Le.slice(0, Qe - 1).join(".");
                ae.nestedProperty(xe, Re).get().splice(ke, 1);
              } else Me && Se.get() === void 0 && Se.get() === void 0 && (Ee[Se.astr] = null), Se.set(we);
            };
            var ve = /(\.[^\[\]\.]+|\[[^\[\]\.]+\])$/;
            function be(Se) {
              var we = Se.search(ve);
              if (we > 0) return Se.substr(0, we);
            }
            J.hasParent = function(Se, we) {
              for (var Ee = be(we); Ee; ) {
                if (Ee in Se) return !0;
                Ee = be(Ee);
              }
              return !1;
            };
            var Ce = ["x", "y", "z"];
            J.clearAxisTypes = function(Se, we, Ee) {
              for (var xe = 0; xe < we.length; xe++) for (var Le = Se._fullData[xe], Qe = 0; Qe < 3; Qe++) {
                var ke = ue(Se, Le, Ce[Qe]);
                if (ke && ke.type !== "log") {
                  var Me = ke._name, Re = ke._id.substr(1);
                  if (Re.substr(0, 5) === "scene") {
                    if (Ee[Re] !== void 0) continue;
                    Me = Re + "." + Me;
                  }
                  var De = Me + ".type";
                  Ee[Me] === void 0 && Ee[De] === void 0 && ae.nestedProperty(Se.layout, De).set(null);
                }
              }
            };
          }, 90742: function(G, J, Z) {
            var te = Z(31420);
            J._doPlot = te._doPlot, J.newPlot = te.newPlot, J.restyle = te.restyle, J.relayout = te.relayout, J.redraw = te.redraw, J.update = te.update, J._guiRestyle = te._guiRestyle, J._guiRelayout = te._guiRelayout, J._guiUpdate = te._guiUpdate, J._storeDirectGUIEdit = te._storeDirectGUIEdit, J.react = te.react, J.extendTraces = te.extendTraces, J.prependTraces = te.prependTraces, J.addTraces = te.addTraces, J.deleteTraces = te.deleteTraces, J.moveTraces = te.moveTraces, J.purge = te.purge, J.addFrames = te.addFrames, J.deleteFrames = te.deleteFrames, J.animate = te.animate, J.setPlotConfig = te.setPlotConfig;
            var ne = Z(95425).getGraphDiv, ee = Z(28231).eraseActiveShape;
            J.deleteActiveShape = function(oe) {
              return ee(ne(oe));
            }, J.toImage = Z(80491), J.validate = Z(2466), J.downloadImage = Z(26452);
            var ae = Z(53853);
            J.makeTemplate = ae.makeTemplate, J.validateTemplate = ae.validateTemplate;
          }, 85844: function(G, J, Z) {
            var te = Z(56174), ne = Z(4969), ee = Z(48636), ae = Z(98813).sorterAsc, oe = Z(33626);
            J.containerArrayMatch = Z(56037);
            var se = J.isAddVal = function(le) {
              return le === "add" || te(le);
            }, ce = J.isRemoveVal = function(le) {
              return le === null || le === "remove";
            };
            J.applyContainerArrayChanges = function(le, ue, de, fe, he) {
              var me = ue.astr, pe = oe.getComponentMethod(me, "supplyLayoutDefaults"), ye = oe.getComponentMethod(me, "draw"), ge = oe.getComponentMethod(me, "drawOne"), ve = fe.replot || fe.recalc || pe === ne || ye === ne, be = le.layout, Ce = le._fullLayout;
              if (de[""]) {
                Object.keys(de).length > 1 && ee.warn("Full array edits are incompatible with other edits", me);
                var Se = de[""][""];
                if (ce(Se)) ue.set(null);
                else {
                  if (!Array.isArray(Se)) return ee.warn("Unrecognized full array edit value", me, Se), !0;
                  ue.set(Se);
                }
                return !ve && (pe(be, Ce), ye(le), !0);
              }
              var we, Ee, xe, Le, Qe, ke, Me, Re, De = Object.keys(de).map(Number).sort(ae), Pe = ue.get(), $e = Pe || [], qe = he(Ce, me).get(), Xe = [], Ve = -1, Ze = $e.length;
              for (we = 0; we < De.length; we++) if (Le = de[xe = De[we]], Qe = Object.keys(Le), ke = Le[""], Me = se(ke), xe < 0 || xe > $e.length - (Me ? 0 : 1)) ee.warn("index out of range", me, xe);
              else if (ke !== void 0) Qe.length > 1 && ee.warn("Insertion & removal are incompatible with edits to the same index.", me, xe), ce(ke) ? Xe.push(xe) : Me ? (ke === "add" && (ke = {}), $e.splice(xe, 0, ke), qe && qe.splice(xe, 0, {})) : ee.warn("Unrecognized full object edit value", me, xe, ke), Ve === -1 && (Ve = xe);
              else for (Ee = 0; Ee < Qe.length; Ee++) Re = me + "[" + xe + "].", he($e[xe], Qe[Ee], Re).set(Le[Qe[Ee]]);
              for (we = Xe.length - 1; we >= 0; we--) $e.splice(Xe[we], 1), qe && qe.splice(Xe[we], 1);
              if ($e.length ? Pe || ue.set($e) : ue.set(null), ve) return !1;
              if (pe(be, Ce), ge !== ne) {
                var Ge;
                if (Ve === -1) Ge = De;
                else {
                  for (Ze = Math.max($e.length, Ze), Ge = [], we = 0; we < De.length && !((xe = De[we]) >= Ve); we++) Ge.push(xe);
                  for (we = Ve; we < Ze; we++) Ge.push(we);
                }
                for (we = 0; we < Ge.length; we++) ge(le, Ge[we]);
              } else ye(le);
              return !0;
            };
          }, 31420: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(39784), ae = Z(34809), oe = ae.nestedProperty, se = Z(68596), ce = Z(40486), le = Z(33626), ue = Z(57297), de = Z(44122), fe = Z(29714), he = Z(90259), me = Z(25829), pe = Z(62203), ye = Z(78766), ge = Z(95284).initInteractions, ve = Z(62972), be = Z(44844).clearOutline, Ce = Z(24452).dfltConfig, Se = Z(85844), we = Z(10887), Ee = Z(71817), xe = Z(13582), Le = Z(54826).AX_NAME_PATTERN, Qe = 0;
            function ke(Ct) {
              var Kt = Ct._fullLayout;
              Kt._redrawFromAutoMarginCount ? Kt._redrawFromAutoMarginCount-- : Ct.emit("plotly_afterplot");
            }
            function Me(Ct, Kt) {
              try {
                Ct._fullLayout._paper.style("background", Kt);
              } catch (Vt) {
                ae.error(Vt);
              }
            }
            function Re(Ct, Kt) {
              Me(Ct, ye.combine(Kt, "white"));
            }
            function De(Ct, Kt) {
              if (!Ct._context) {
                Ct._context = ae.extendDeep({}, Ce);
                var Vt = te.select("base");
                Ct._context._baseUrl = Vt.size() && Vt.attr("href") ? window.location.href.split("#")[0] : "";
              }
              var Yt, sn, hn, Qt = Ct._context;
              if (Kt) {
                for (sn = Object.keys(Kt), Yt = 0; Yt < sn.length; Yt++) (hn = sn[Yt]) !== "editable" && hn !== "edits" && hn in Qt && (hn === "setBackground" && Kt[hn] === "opaque" ? Qt[hn] = Re : Qt[hn] = Kt[hn]);
                Kt.plot3dPixelRatio && !Qt.plotGlPixelRatio && (Qt.plotGlPixelRatio = Qt.plot3dPixelRatio);
                var Ut = Kt.editable;
                if (Ut !== void 0) for (Qt.editable = Ut, sn = Object.keys(Qt.edits), Yt = 0; Yt < sn.length; Yt++) Qt.edits[sn[Yt]] = Ut;
                if (Kt.edits) for (sn = Object.keys(Kt.edits), Yt = 0; Yt < sn.length; Yt++) (hn = sn[Yt]) in Qt.edits && (Qt.edits[hn] = Kt.edits[hn]);
                Qt._exportedPlot = Kt._exportedPlot;
              }
              Qt.staticPlot && (Qt.editable = !1, Qt.edits = {}, Qt.autosizable = !1, Qt.scrollZoom = !1, Qt.doubleClick = !1, Qt.showTips = !1, Qt.showLink = !1, Qt.displayModeBar = !1), Qt.displayModeBar !== "hover" || ee || (Qt.displayModeBar = !0), Qt.setBackground !== "transparent" && typeof Qt.setBackground == "function" || (Qt.setBackground = Me), Qt._hasZeroHeight = Qt._hasZeroHeight || Ct.clientHeight === 0, Qt._hasZeroWidth = Qt._hasZeroWidth || Ct.clientWidth === 0;
              var Wt = Qt.scrollZoom, rn = Qt._scrollZoom = {};
              if (Wt === !0) rn.cartesian = 1, rn.gl3d = 1, rn.geo = 1, rn.mapbox = 1, rn.map = 1;
              else if (typeof Wt == "string") {
                var vn = Wt.split("+");
                for (Yt = 0; Yt < vn.length; Yt++) rn[vn[Yt]] = 1;
              } else Wt !== !1 && (rn.gl3d = 1, rn.geo = 1, rn.mapbox = 1, rn.map = 1);
            }
            function Pe(Ct, Kt) {
              var Vt, Yt, sn = Kt + 1, hn = [];
              for (Vt = 0; Vt < Ct.length; Vt++) (Yt = Ct[Vt]) < 0 ? hn.push(sn + Yt) : hn.push(Yt);
              return hn;
            }
            function $e(Ct, Kt, Vt) {
              var Yt, sn;
              for (Yt = 0; Yt < Kt.length; Yt++) {
                if ((sn = Kt[Yt]) !== parseInt(sn, 10)) throw new Error("all values in " + Vt + " must be integers");
                if (sn >= Ct.data.length || sn < -Ct.data.length) throw new Error(Vt + " must be valid indices for gd.data.");
                if (Kt.indexOf(sn, Yt + 1) > -1 || sn >= 0 && Kt.indexOf(-Ct.data.length + sn) > -1 || sn < 0 && Kt.indexOf(Ct.data.length + sn) > -1) throw new Error("each index in " + Vt + " must be unique.");
              }
            }
            function qe(Ct, Kt, Vt) {
              if (!Array.isArray(Ct.data)) throw new Error("gd.data must be an array.");
              if (Kt === void 0) throw new Error("currentIndices is a required argument.");
              if (Array.isArray(Kt) || (Kt = [Kt]), $e(Ct, Kt, "currentIndices"), Vt === void 0 || Array.isArray(Vt) || (Vt = [Vt]), Vt !== void 0 && $e(Ct, Vt, "newIndices"), Vt !== void 0 && Kt.length !== Vt.length) throw new Error("current and new indices must be of equal length.");
            }
            function Xe(Ct, Kt, Vt, Yt, sn) {
              (function(Ht, jt, cn, wn) {
                var Rn = ae.isPlainObject(wn);
                if (!Array.isArray(Ht.data)) throw new Error("gd.data must be an array");
                if (!ae.isPlainObject(jt)) throw new Error("update must be a key:value object");
                if (cn === void 0) throw new Error("indices must be an integer or array of integers");
                for (var Pn in $e(Ht, cn, "indices"), jt) {
                  if (!Array.isArray(jt[Pn]) || jt[Pn].length !== cn.length) throw new Error("attribute " + Pn + " must be an array of length equal to indices array length");
                  if (Rn && (!(Pn in wn) || !Array.isArray(wn[Pn]) || wn[Pn].length !== jt[Pn].length)) throw new Error("when maxPoints is set as a key:value object it must contain a 1:1 corrispondence with the keys and number of traces in the update object");
                }
              })(Ct, Kt, Vt, Yt);
              for (var hn = function(Ht, jt, cn, wn) {
                var Rn, Pn, Ie, rt, ft, pt = ae.isPlainObject(wn), wt = [];
                for (var Bt in Array.isArray(cn) || (cn = [cn]), cn = Pe(cn, Ht.data.length - 1), jt) for (var At = 0; At < cn.length; At++) {
                  if (Rn = Ht.data[cn[At]], Pn = (Ie = oe(Rn, Bt)).get(), rt = jt[Bt][At], !ae.isArrayOrTypedArray(rt)) throw new Error("attribute: " + Bt + " index: " + At + " must be an array");
                  if (!ae.isArrayOrTypedArray(Pn)) throw new Error("cannot extend missing or non-array attribute: " + Bt);
                  if (Pn.constructor !== rt.constructor) throw new Error("cannot extend array with an array of a different type: " + Bt);
                  ft = pt ? wn[Bt][At] : wn, ne(ft) || (ft = -1), wt.push({ prop: Ie, target: Pn, insert: rt, maxp: Math.floor(ft) });
                }
                return wt;
              }(Ct, Kt, Vt, Yt), Qt = {}, Ut = {}, Wt = 0; Wt < hn.length; Wt++) {
                var rn = hn[Wt].prop, vn = hn[Wt].maxp, dn = sn(hn[Wt].target, hn[Wt].insert, vn);
                rn.set(dn[0]), Array.isArray(Qt[rn.astr]) || (Qt[rn.astr] = []), Qt[rn.astr].push(dn[1]), Array.isArray(Ut[rn.astr]) || (Ut[rn.astr] = []), Ut[rn.astr].push(hn[Wt].target.length);
              }
              return { update: Qt, maxPoints: Ut };
            }
            function Ve(Ct, Kt) {
              var Vt = new Ct.constructor(Ct.length + Kt.length);
              return Vt.set(Ct), Vt.set(Kt, Ct.length), Vt;
            }
            function Ze(Ct, Kt, Vt, Yt) {
              Ct = ae.getGraphDiv(Ct), we.clearPromiseQueue(Ct);
              var sn = {};
              if (typeof Kt == "string") sn[Kt] = Vt;
              else {
                if (!ae.isPlainObject(Kt)) return ae.warn("Restyle fail.", Kt, Vt, Yt), Promise.reject();
                sn = ae.extendFlat({}, Kt), Yt === void 0 && (Yt = Vt);
              }
              Object.keys(sn).length && (Ct.changed = !0);
              var hn = we.coerceTraceIndices(Ct, Yt), Qt = at(Ct, sn, hn), Ut = Qt.flags;
              Ut.calc && (Ct.calcdata = void 0), Ut.clearAxisTypes && we.clearAxisTypes(Ct, hn, {});
              var Wt = [];
              Ut.fullReplot ? Wt.push(J._doPlot) : (Wt.push(de.previousPromises), de.supplyDefaults(Ct), Ut.markerSize && (de.doCalcdata(Ct), Je(Wt)), Ut.style && Wt.push(Ee.doTraceStyle), Ut.colorbars && Wt.push(Ee.doColorBars), Wt.push(ke)), Wt.push(de.rehover, de.redrag, de.reselect), ce.add(Ct, Ze, [Ct, Qt.undoit, Qt.traces], Ze, [Ct, Qt.redoit, Qt.traces]);
              var rn = ae.syncOrAsync(Wt, Ct);
              return rn && rn.then || (rn = Promise.resolve()), rn.then(function() {
                return Ct.emit("plotly_restyle", Qt.eventData), Ct;
              });
            }
            function Ge(Ct) {
              return Ct === void 0 ? null : Ct;
            }
            function ze(Ct, Kt) {
              return Kt ? function(Vt, Yt, sn) {
                var hn = oe(Vt, Yt), Qt = hn.set;
                return hn.set = function(Ut) {
                  We((sn || "") + Yt, hn.get(), Ut, Ct), Qt(Ut);
                }, hn;
              } : oe;
            }
            function We(Ct, Kt, Vt, Yt) {
              if (Array.isArray(Kt) || Array.isArray(Vt)) for (var sn = Array.isArray(Kt) ? Kt : [], hn = Array.isArray(Vt) ? Vt : [], Qt = Math.max(sn.length, hn.length), Ut = 0; Ut < Qt; Ut++) We(Ct + "[" + Ut + "]", sn[Ut], hn[Ut], Yt);
              else if (ae.isPlainObject(Kt) || ae.isPlainObject(Vt)) {
                var Wt = ae.isPlainObject(Kt) ? Kt : {}, rn = ae.isPlainObject(Vt) ? Vt : {}, vn = ae.extendFlat({}, Wt, rn);
                for (var dn in vn) We(Ct + "." + dn, Wt[dn], rn[dn], Yt);
              } else Yt[Ct] === void 0 && (Yt[Ct] = Ge(Kt));
            }
            function at(Ct, Kt, Vt) {
              var Yt, sn = Ct._fullLayout, hn = Ct._fullData, Qt = Ct.data, Ut = sn._guiEditing, Wt = ze(sn._preGUI, Ut), rn = ae.extendDeepAll({}, Kt);
              ct(Kt);
              var vn, dn = xe.traceFlags(), Ht = {}, jt = {};
              function cn() {
                return Vt.map(function() {
                });
              }
              function wn(Yr) {
                var jr = fe.id2name(Yr);
                vn.indexOf(jr) === -1 && vn.push(jr);
              }
              function Rn(Yr) {
                return "LAYOUT" + Yr + ".autorange";
              }
              function Pn(Yr) {
                return "LAYOUT" + Yr + ".range";
              }
              function Ie(Yr) {
                for (var jr = Yr; jr < hn.length; jr++) if (hn[jr]._input === Qt[Yr]) return hn[jr];
              }
              function rt(Yr, jr, Mr) {
                if (Array.isArray(Yr)) Yr.forEach(function(gi) {
                  rt(gi, jr, Mr);
                });
                else if (!(Yr in Kt) && !we.hasParent(Kt, Yr)) {
                  var Wr;
                  if (Yr.substr(0, 6) === "LAYOUT") Wr = Wt(Ct.layout, Yr.replace("LAYOUT", ""));
                  else {
                    var Bi = Vt[Mr];
                    Wr = ze(sn._tracePreGUI[Ie(Bi)._fullInput.uid], Ut)(Qt[Bi], Yr);
                  }
                  Yr in jt || (jt[Yr] = cn()), jt[Yr][Mr] === void 0 && (jt[Yr][Mr] = Ge(Wr.get())), jr !== void 0 && Wr.set(jr);
                }
              }
              function ft(Yr) {
                return function(jr) {
                  return hn[jr][Yr];
                };
              }
              function pt(Yr) {
                return function(jr, Mr) {
                  return jr === !1 ? hn[Vt[Mr]][Yr] : null;
                };
              }
              for (var wt in Kt) {
                if (we.hasParent(Kt, wt)) throw new Error("cannot set " + wt + " and a parent attribute simultaneously");
                var Bt, At, Dt, Ot, un, yn, _n = Kt[wt];
                if (wt !== "autobinx" && wt !== "autobiny" || (wt = wt.charAt(wt.length - 1) + "bins", _n = Array.isArray(_n) ? _n.map(pt(wt)) : _n === !1 ? Vt.map(ft(wt)) : null), Ht[wt] = _n, wt.substr(0, 6) !== "LAYOUT") {
                  for (jt[wt] = cn(), Yt = 0; Yt < Vt.length; Yt++) if (Bt = Qt[Vt[Yt]], At = Ie(Vt[Yt]), Ot = (Dt = ze(sn._tracePreGUI[At._fullInput.uid], Ut)(Bt, wt)).get(), (un = Array.isArray(_n) ? _n[Yt % _n.length] : _n) !== void 0) {
                    var gn = Dt.parts[Dt.parts.length - 1], bn = wt.substr(0, wt.length - gn.length - 1), an = bn ? bn + "." : "", Zt = bn ? oe(At, bn).get() : At;
                    if ((yn = ue.getTraceValObject(At, Dt.parts)) && yn.impliedEdits && un !== null) for (var Mn in yn.impliedEdits) rt(ae.relativeAttr(wt, Mn), yn.impliedEdits[Mn], Yt);
                    else if (gn !== "thicknessmode" && gn !== "lenmode" || Ot === un || un !== "fraction" && un !== "pixels" || !Zt) {
                      if (wt === "type" && (un === "pie" != (Ot === "pie") || un === "funnelarea" != (Ot === "funnelarea"))) {
                        var Ln = "x", In = "y";
                        un !== "bar" && Ot !== "bar" || Bt.orientation !== "h" || (Ln = "y", In = "x"), ae.swapAttrs(Bt, ["?", "?src"], "labels", Ln), ae.swapAttrs(Bt, ["d?", "?0"], "label", Ln), ae.swapAttrs(Bt, ["?", "?src"], "values", In), Ot === "pie" || Ot === "funnelarea" ? (oe(Bt, "marker.color").set(oe(Bt, "marker.colors").get()), sn._pielayer.selectAll("g.trace").remove()) : le.traceIs(Bt, "cartesian") && oe(Bt, "marker.colors").set(oe(Bt, "marker.color").get());
                      }
                    } else {
                      var kn = sn._size, er = Zt.orient, jn = er === "top" || er === "bottom";
                      if (gn === "thicknessmode") {
                        var ur = jn ? kn.h : kn.w;
                        rt(an + "thickness", Zt.thickness * (un === "fraction" ? 1 / ur : ur), Yt);
                      } else {
                        var Kn = jn ? kn.w : kn.h;
                        rt(an + "len", Zt.len * (un === "fraction" ? 1 / Kn : Kn), Yt);
                      }
                    }
                    if (jt[wt][Yt] = Ge(Ot), ["swapxy", "swapxyaxes", "orientation", "orientationaxes"].indexOf(wt) !== -1) {
                      if (wt === "orientation") {
                        Dt.set(un);
                        var Jn = Bt.x && !Bt.y ? "h" : "v";
                        if ((Dt.get() || Jn) === At.orientation) continue;
                      } else wt === "orientationaxes" && (Bt.orientation = { v: "h", h: "v" }[At.orientation]);
                      we.swapXYData(Bt), dn.calc = dn.clearAxisTypes = !0;
                    } else de.dataArrayContainers.indexOf(Dt.parts[0]) !== -1 ? (we.manageArrayContainers(Dt, un, jt), dn.calc = !0) : (yn ? yn.arrayOk && !le.traceIs(At, "regl") && (ae.isArrayOrTypedArray(un) || ae.isArrayOrTypedArray(Ot)) ? dn.calc = !0 : xe.update(dn, yn) : dn.calc = !0, Dt.set(un));
                  }
                  if (["swapxyaxes", "orientationaxes"].indexOf(wt) !== -1 && fe.swap(Ct, Vt), wt === "orientationaxes") {
                    var Lr = oe(Ct.layout, "hovermode"), mr = Lr.get();
                    mr === "x" ? Lr.set("y") : mr === "y" ? Lr.set("x") : mr === "x unified" ? Lr.set("y unified") : mr === "y unified" && Lr.set("x unified");
                  }
                  if (["orientation", "type"].indexOf(wt) !== -1) {
                    for (vn = [], Yt = 0; Yt < Vt.length; Yt++) {
                      var Ar = Qt[Vt[Yt]];
                      le.traceIs(Ar, "cartesian") && (wn(Ar.xaxis || "x"), wn(Ar.yaxis || "y"));
                    }
                    rt(vn.map(Rn), !0, 0), rt(vn.map(Pn), [0, 1], 0);
                  }
                } else Dt = Wt(Ct.layout, wt.replace("LAYOUT", "")), jt[wt] = [Ge(Dt.get())], Dt.set(Array.isArray(_n) ? _n[0] : _n), dn.calc = !0;
              }
              return (dn.calc || dn.plot) && (dn.fullReplot = !0), { flags: dn, undoit: jt, redoit: Ht, traces: Vt, eventData: ae.extendDeepNoArrays([], [rn, Vt]) };
            }
            function ct(Ct) {
              var Kt, Vt, Yt, sn = ae.counterRegex("axis", ".title", !1, !1), hn = /colorbar\.title$/, Qt = Object.keys(Ct);
              for (Kt = 0; Kt < Qt.length; Kt++) Vt = Qt[Kt], Yt = Ct[Vt], Vt !== "title" && !sn.test(Vt) && !hn.test(Vt) || typeof Yt != "string" && typeof Yt != "number" ? Vt.indexOf("titlefont") > -1 && Vt.indexOf("grouptitlefont") === -1 ? Ut(Vt, Vt.replace("titlefont", "title.font")) : Vt.indexOf("titleposition") > -1 ? Ut(Vt, Vt.replace("titleposition", "title.position")) : Vt.indexOf("titleside") > -1 ? Ut(Vt, Vt.replace("titleside", "title.side")) : Vt.indexOf("titleoffset") > -1 && Ut(Vt, Vt.replace("titleoffset", "title.offset")) : Ut(Vt, Vt.replace("title", "title.text"));
              function Ut(Wt, rn) {
                Ct[rn] = Ct[Wt], delete Ct[Wt];
              }
            }
            function et(Ct, Kt, Vt) {
              Ct = ae.getGraphDiv(Ct), we.clearPromiseQueue(Ct);
              var Yt = {};
              if (typeof Kt == "string") Yt[Kt] = Vt;
              else {
                if (!ae.isPlainObject(Kt)) return ae.warn("Relayout fail.", Kt, Vt), Promise.reject();
                Yt = ae.extendFlat({}, Kt);
              }
              Object.keys(Yt).length && (Ct.changed = !0);
              var sn = bt(Ct, Yt), hn = sn.flags;
              hn.calc && (Ct.calcdata = void 0);
              var Qt = [de.previousPromises];
              hn.layoutReplot ? Qt.push(Ee.layoutReplot) : Object.keys(Yt).length && (dt(Ct, hn, sn) || de.supplyDefaults(Ct), hn.legend && Qt.push(Ee.doLegend), hn.layoutstyle && Qt.push(Ee.layoutStyles), hn.axrange && Je(Qt, sn.rangesAltered), hn.ticks && Qt.push(Ee.doTicksRelayout), hn.modebar && Qt.push(Ee.doModeBar), hn.camera && Qt.push(Ee.doCamera), hn.colorbars && Qt.push(Ee.doColorBars), Qt.push(ke)), Qt.push(de.rehover, de.redrag, de.reselect), ce.add(Ct, et, [Ct, sn.undoit], et, [Ct, sn.redoit]);
              var Ut = ae.syncOrAsync(Qt, Ct);
              return Ut && Ut.then || (Ut = Promise.resolve(Ct)), Ut.then(function() {
                return Ct.emit("plotly_relayout", sn.eventData), Ct;
              });
            }
            function dt(Ct, Kt, Vt) {
              var Yt, sn, hn = Ct._fullLayout;
              if (!Kt.axrange) return !1;
              for (var Qt in Kt) if (Qt !== "axrange" && Kt[Qt]) return !1;
              var Ut = function(jt, cn) {
                return ae.coerce(Yt, sn, me, jt, cn);
              }, Wt = {};
              for (var rn in Vt.rangesAltered) {
                var vn = fe.id2name(rn);
                if (Yt = Ct.layout[vn], sn = hn[vn], he(Yt, sn, Ut, Wt), sn._matchGroup) {
                  for (var dn in sn._matchGroup) if (dn !== rn) {
                    var Ht = hn[fe.id2name(dn)];
                    Ht.autorange = sn.autorange, Ht.range = sn.range.slice(), Ht._input.range = sn.range.slice();
                  }
                }
              }
              return !0;
            }
            function Je(Ct, Kt) {
              var Vt = Kt ? function(Yt) {
                var sn = [];
                for (var hn in Kt) {
                  var Qt = fe.getFromId(Yt, hn);
                  if (sn.push(hn), (Qt.ticklabelposition || "").indexOf("inside") !== -1 && Qt._anchorAxis && sn.push(Qt._anchorAxis._id), Qt._matchGroup) for (var Ut in Qt._matchGroup) Kt[Ut] || sn.push(Ut);
                }
                return fe.draw(Yt, sn, { skipTitle: !0 });
              } : function(Yt) {
                return fe.draw(Yt, "redraw");
              };
              Ct.push(be, Ee.doAutoRangeAndConstraints, Vt, Ee.drawData, Ee.finalDraw);
            }
            var it = /^[xyz]axis[0-9]*\.range(\[[0|1]\])?$/, st = /^[xyz]axis[0-9]*\.autorange$/, gt = /^[xyz]axis[0-9]*\.domain(\[[0|1]\])?$/;
            function bt(Ct, Kt) {
              var Vt, Yt, sn, hn = Ct.layout, Qt = Ct._fullLayout, Ut = Qt._guiEditing, Wt = ze(Qt._preGUI, Ut), rn = Object.keys(Kt), vn = fe.list(Ct), dn = ae.extendDeepAll({}, Kt), Ht = {};
              for (ct(Kt), rn = Object.keys(Kt), Yt = 0; Yt < rn.length; Yt++) if (rn[Yt].indexOf("allaxes") === 0) {
                for (sn = 0; sn < vn.length; sn++) {
                  var jt = vn[sn]._id.substr(1), cn = jt.indexOf("scene") !== -1 ? jt + "." : "", wn = rn[Yt].replace("allaxes", cn + vn[sn]._name);
                  Kt[wn] || (Kt[wn] = Kt[rn[Yt]]);
                }
                delete Kt[rn[Yt]];
              }
              var Rn = xe.layoutFlags(), Pn = {}, Ie = {};
              function rt(gi, Zr) {
                if (Array.isArray(gi)) gi.forEach(function(ri) {
                  rt(ri, Zr);
                });
                else if (!(gi in Kt) && !we.hasParent(Kt, gi)) {
                  var fi = Wt(hn, gi);
                  gi in Ie || (Ie[gi] = Ge(fi.get())), Zr !== void 0 && fi.set(Zr);
                }
              }
              var ft, pt = {};
              function wt(gi) {
                var Zr = fe.name2id(gi.split(".")[0]);
                return pt[Zr] = 1, Zr;
              }
              for (var Bt in Kt) {
                if (we.hasParent(Kt, Bt)) throw new Error("cannot set " + Bt + " and a parent attribute simultaneously");
                for (var At = Wt(hn, Bt), Dt = Kt[Bt], Ot = At.parts.length - 1; Ot > 0 && typeof At.parts[Ot] != "string"; ) Ot--;
                var un = At.parts[Ot], yn = At.parts[Ot - 1] + "." + un, _n = At.parts.slice(0, Ot).join("."), gn = oe(Ct.layout, _n).get(), bn = oe(Qt, _n).get(), an = At.get();
                if (Dt !== void 0) {
                  Pn[Bt] = Dt, Ie[Bt] = un === "reverse" ? Dt : Ge(an);
                  var Zt = ue.getLayoutValObject(Qt, At.parts);
                  if (Zt && Zt.impliedEdits && Dt !== null) for (var Mn in Zt.impliedEdits) rt(ae.relativeAttr(Bt, Mn), Zt.impliedEdits[Mn]);
                  if (["width", "height"].indexOf(Bt) !== -1) if (Dt) {
                    rt("autosize", null);
                    var Ln = Bt === "height" ? "width" : "height";
                    rt(Ln, Qt[Ln]);
                  } else Qt[Bt] = Ct._initialAutoSize[Bt];
                  else if (Bt === "autosize") rt("width", Dt ? null : Qt.width), rt("height", Dt ? null : Qt.height);
                  else if (yn.match(it)) wt(yn), oe(Qt, _n + "._inputRange").set(null);
                  else if (yn.match(st)) {
                    wt(yn), oe(Qt, _n + "._inputRange").set(null);
                    var In = oe(Qt, _n).get();
                    In._inputDomain && (In._input.domain = In._inputDomain.slice());
                  } else yn.match(gt) && oe(Qt, _n + "._inputDomain").set(null);
                  if (un === "type") {
                    ft = gn;
                    var kn = bn.type === "linear" && Dt === "log", er = bn.type === "log" && Dt === "linear";
                    if (kn || er) {
                      if (ft && ft.range) if (bn.autorange) kn && (ft.range = ft.range[1] > ft.range[0] ? [1, 2] : [2, 1]);
                      else {
                        var jn = ft.range[0], ur = ft.range[1];
                        kn ? (jn <= 0 && ur <= 0 && rt(_n + ".autorange", !0), jn <= 0 ? jn = ur / 1e6 : ur <= 0 && (ur = jn / 1e6), rt(_n + ".range[0]", Math.log(jn) / Math.LN10), rt(_n + ".range[1]", Math.log(ur) / Math.LN10)) : (rt(_n + ".range[0]", Math.pow(10, jn)), rt(_n + ".range[1]", Math.pow(10, ur)));
                      }
                      else rt(_n + ".autorange", !0);
                      Array.isArray(Qt._subplots.polar) && Qt._subplots.polar.length && Qt[At.parts[0]] && At.parts[1] === "radialaxis" && delete Qt[At.parts[0]]._subplot.viewInitial["radialaxis.range"], le.getComponentMethod("annotations", "convertCoords")(Ct, bn, Dt, rt), le.getComponentMethod("images", "convertCoords")(Ct, bn, Dt, rt);
                    } else rt(_n + ".autorange", !0), rt(_n + ".range", null);
                    oe(Qt, _n + "._inputRange").set(null);
                  } else if (un.match(Le)) {
                    var Kn = oe(Qt, Bt).get(), Jn = (Dt || {}).type;
                    Jn && Jn !== "-" || (Jn = "linear"), le.getComponentMethod("annotations", "convertCoords")(Ct, Kn, Jn, rt), le.getComponentMethod("images", "convertCoords")(Ct, Kn, Jn, rt);
                  }
                  var Lr = Se.containerArrayMatch(Bt);
                  if (Lr) {
                    Vt = Lr.array, Yt = Lr.index;
                    var mr = Lr.property, Ar = Zt || { editType: "calc" };
                    Yt !== "" && mr === "" && (Se.isAddVal(Dt) ? Ie[Bt] = null : Se.isRemoveVal(Dt) ? Ie[Bt] = (oe(hn, Vt).get() || [])[Yt] : ae.warn("unrecognized full object value", Kt)), xe.update(Rn, Ar), Ht[Vt] || (Ht[Vt] = {});
                    var Yr = Ht[Vt][Yt];
                    Yr || (Yr = Ht[Vt][Yt] = {}), Yr[mr] = Dt, delete Kt[Bt];
                  } else un === "reverse" ? (gn.range ? gn.range.reverse() : (rt(_n + ".autorange", !0), gn.range = [1, 0]), bn.autorange ? Rn.calc = !0 : Rn.plot = !0) : (Bt === "dragmode" && (Dt === !1 && an !== !1 || Dt !== !1 && an === !1) || Qt._has("scatter-like") && Qt._has("regl") && Bt === "dragmode" && (Dt === "lasso" || Dt === "select") && an !== "lasso" && an !== "select" || Qt._has("gl2d") ? Rn.plot = !0 : Zt ? xe.update(Rn, Zt) : Rn.calc = !0, At.set(Dt));
                }
              }
              for (Vt in Ht) Se.applyContainerArrayChanges(Ct, Wt(hn, Vt), Ht[Vt], Rn, Wt) || (Rn.plot = !0);
              for (var jr in pt) {
                var Mr = (ft = fe.getFromId(Ct, jr)) && ft._constraintGroup;
                if (Mr) for (var Wr in Rn.calc = !0, Mr) pt[Wr] || (fe.getFromId(Ct, Wr)._constraintShrinkable = !0);
              }
              (yt(Ct) || Kt.height || Kt.width) && (Rn.plot = !0);
              var Bi = Qt.shapes;
              for (Yt = 0; Yt < Bi.length; Yt++) if (Bi[Yt].showlegend) {
                Rn.calc = !0;
                break;
              }
              return (Rn.plot || Rn.calc) && (Rn.layoutReplot = !0), { flags: Rn, rangesAltered: pt, undoit: Ie, redoit: Pn, eventData: dn };
            }
            function yt(Ct) {
              var Kt = Ct._fullLayout, Vt = Kt.width, Yt = Kt.height;
              return Ct.layout.autosize && de.plotAutoSize(Ct, Ct.layout, Kt), Kt.width !== Vt || Kt.height !== Yt;
            }
            function St(Ct, Kt, Vt, Yt) {
              Ct = ae.getGraphDiv(Ct), we.clearPromiseQueue(Ct), ae.isPlainObject(Kt) || (Kt = {}), ae.isPlainObject(Vt) || (Vt = {}), Object.keys(Kt).length && (Ct.changed = !0), Object.keys(Vt).length && (Ct.changed = !0);
              var sn = we.coerceTraceIndices(Ct, Yt), hn = at(Ct, ae.extendFlat({}, Kt), sn), Qt = hn.flags, Ut = bt(Ct, ae.extendFlat({}, Vt)), Wt = Ut.flags;
              (Qt.calc || Wt.calc) && (Ct.calcdata = void 0), Qt.clearAxisTypes && we.clearAxisTypes(Ct, sn, Vt);
              var rn = [];
              Wt.layoutReplot ? rn.push(Ee.layoutReplot) : Qt.fullReplot ? rn.push(J._doPlot) : (rn.push(de.previousPromises), dt(Ct, Wt, Ut) || de.supplyDefaults(Ct), Qt.style && rn.push(Ee.doTraceStyle), (Qt.colorbars || Wt.colorbars) && rn.push(Ee.doColorBars), Wt.legend && rn.push(Ee.doLegend), Wt.layoutstyle && rn.push(Ee.layoutStyles), Wt.axrange && Je(rn, Ut.rangesAltered), Wt.ticks && rn.push(Ee.doTicksRelayout), Wt.modebar && rn.push(Ee.doModeBar), Wt.camera && rn.push(Ee.doCamera), rn.push(ke)), rn.push(de.rehover, de.redrag, de.reselect), ce.add(Ct, St, [Ct, hn.undoit, Ut.undoit, hn.traces], St, [Ct, hn.redoit, Ut.redoit, hn.traces]);
              var vn = ae.syncOrAsync(rn, Ct);
              return vn && vn.then || (vn = Promise.resolve(Ct)), vn.then(function() {
                return Ct.emit("plotly_update", { data: hn.eventData, layout: Ut.eventData }), Ct;
              });
            }
            function Lt(Ct) {
              return function(Kt) {
                Kt._fullLayout._guiEditing = !0;
                var Vt = Ct.apply(null, arguments);
                return Kt._fullLayout._guiEditing = !1, Vt;
              };
            }
            var Rt = [{ pattern: /^hiddenlabels/, attr: "legend.uirevision" }, { pattern: /^((x|y)axis\d*)\.((auto)?range|title\.text)/ }, { pattern: /axis\d*\.showspikes$/, attr: "modebar.uirevision" }, { pattern: /(hover|drag)mode$/, attr: "modebar.uirevision" }, { pattern: /^(scene\d*)\.camera/ }, { pattern: /^(geo\d*)\.(projection|center|fitbounds)/ }, { pattern: /^(ternary\d*\.[abc]axis)\.(min|title\.text)$/ }, { pattern: /^(polar\d*\.radialaxis)\.((auto)?range|angle|title\.text)/ }, { pattern: /^(polar\d*\.angularaxis)\.rotation/ }, { pattern: /^(mapbox\d*)\.(center|zoom|bearing|pitch)/ }, { pattern: /^(map\d*)\.(center|zoom|bearing|pitch)/ }, { pattern: /^legend\.(x|y)$/, attr: "editrevision" }, { pattern: /^(shapes|annotations)/, attr: "editrevision" }, { pattern: /^title\.text$/, attr: "editrevision" }], vt = [{ pattern: /^selectedpoints$/, attr: "selectionrevision" }, { pattern: /(^|value\.)visible$/, attr: "legend.uirevision" }, { pattern: /^dimensions\[\d+\]\.constraintrange/ }, { pattern: /^node\.(x|y|groups)/ }, { pattern: /^level$/ }, { pattern: /(^|value\.)name$/ }, { pattern: /colorbar\.title\.text$/ }, { pattern: /colorbar\.(x|y)$/, attr: "editrevision" }];
            function $t(Ct, Kt) {
              for (var Vt = 0; Vt < Kt.length; Vt++) {
                var Yt = Kt[Vt], sn = Ct.match(Yt.pattern);
                if (sn) {
                  var hn = sn[1] || "";
                  return { head: hn, tail: Ct.substr(hn.length + 1), attr: Yt.attr };
                }
              }
            }
            function Pt(Ct, Kt) {
              var Vt = oe(Kt, Ct).get();
              if (Vt !== void 0) return Vt;
              var Yt = Ct.split(".");
              for (Yt.pop(); Yt.length > 1; ) if (Yt.pop(), (Vt = oe(Kt, Yt.join(".") + ".uirevision").get()) !== void 0) return Vt;
              return Kt.uirevision;
            }
            function Xt(Ct, Kt) {
              for (var Vt = 0; Vt < Kt.length; Vt++) if (Kt[Vt]._fullInput.uid === Ct) return Vt;
              return -1;
            }
            function Nt(Ct, Kt, Vt) {
              for (var Yt = 0; Yt < Kt.length; Yt++) if (Kt[Yt].uid === Ct) return Yt;
              return !Kt[Vt] || Kt[Vt].uid ? -1 : Vt;
            }
            function Gt(Ct, Kt) {
              var Vt = ae.isPlainObject(Ct), Yt = Array.isArray(Ct);
              return Vt || Yt ? (Vt && ae.isPlainObject(Kt) || Yt && Array.isArray(Kt)) && JSON.stringify(Ct) === JSON.stringify(Kt) : Ct === Kt;
            }
            function Ft(Ct, Kt, Vt, Yt) {
              var sn, hn, Qt, Ut = Yt.getValObject, Wt = Yt.flags, rn = Yt.immutable, vn = Yt.inArray, dn = Yt.arrayIndex;
              function Ht() {
                var gn = sn.editType;
                vn && gn.indexOf("arraydraw") !== -1 ? ae.pushUnique(Wt.arrays[vn], dn) : (xe.update(Wt, sn), gn !== "none" && Wt.nChanges++, Yt.transition && sn.anim && Wt.nChangesAnim++, (it.test(Qt) || st.test(Qt)) && (Wt.rangesAltered[Vt[0]] = 1), gt.test(Qt) && oe(Kt, "_inputDomain").set(null), hn === "datarevision" && (Wt.newDataRevision = 1));
              }
              function jt(gn) {
                return gn.valType === "data_array" || gn.arrayOk;
              }
              for (hn in Ct) {
                if (Wt.calc && !Yt.transition) return;
                var cn = Ct[hn], wn = Kt[hn], Rn = Vt.concat(hn);
                if (Qt = Rn.join("."), hn.charAt(0) !== "_" && typeof cn != "function" && cn !== wn) {
                  if ((hn === "tick0" || hn === "dtick") && Vt[0] !== "geo") {
                    var Pn = Kt.tickmode;
                    if (Pn === "auto" || Pn === "array" || !Pn) continue;
                  }
                  if ((hn !== "range" || !Kt.autorange) && (hn !== "zmin" && hn !== "zmax" || Kt.type !== "contourcarpet") && (sn = Ut(Rn)) && (!sn._compareAsJSON || JSON.stringify(cn) !== JSON.stringify(wn))) {
                    var Ie, rt = sn.valType, ft = jt(sn), pt = Array.isArray(cn), wt = Array.isArray(wn);
                    if (pt && wt) {
                      var Bt = "_input_" + hn, At = Ct[Bt], Dt = Kt[Bt];
                      if (Array.isArray(At) && At === Dt) continue;
                    }
                    if (wn === void 0) ft && pt ? Wt.calc = !0 : Ht();
                    else if (sn._isLinkedToArray) {
                      var Ot = [], un = !1;
                      vn || (Wt.arrays[hn] = Ot);
                      var yn = Math.min(cn.length, wn.length), _n = Math.max(cn.length, wn.length);
                      if (yn !== _n) {
                        if (sn.editType !== "arraydraw") {
                          Ht();
                          continue;
                        }
                        un = !0;
                      }
                      for (Ie = 0; Ie < yn; Ie++) Ft(cn[Ie], wn[Ie], Rn.concat(Ie), ae.extendFlat({ inArray: hn, arrayIndex: Ie }, Yt));
                      if (un) for (Ie = yn; Ie < _n; Ie++) Ot.push(Ie);
                    } else !rt && ae.isPlainObject(cn) ? Ft(cn, wn, Rn, Yt) : ft ? pt && wt ? (rn && (Wt.calc = !0), (rn || Yt.newDataRevision) && Ht()) : pt !== wt ? Wt.calc = !0 : Ht() : pt && wt && cn.length === wn.length && String(cn) === String(wn) || Ht();
                  }
                }
              }
              for (hn in Kt) if (!(hn in Ct) && hn.charAt(0) !== "_" && typeof Kt[hn] != "function") {
                if (jt(sn = Ut(Vt.concat(hn))) && Array.isArray(Kt[hn])) return void (Wt.calc = !0);
                Ht();
              }
            }
            function Tt(Ct, Kt) {
              var Vt;
              for (Vt in Ct) if (Vt.charAt(0) !== "_") {
                var Yt = Ct[Vt], sn = Kt[Vt];
                if (Yt !== sn) if (ae.isPlainObject(Yt) && ae.isPlainObject(sn)) {
                  if (Tt(Yt, sn)) return !0;
                } else {
                  if (!Array.isArray(Yt) || !Array.isArray(sn) || Yt.length !== sn.length) return !0;
                  for (var hn = 0; hn < Yt.length; hn++) if (Yt[hn] !== sn[hn] && (!ae.isPlainObject(Yt[hn]) || !ae.isPlainObject(sn[hn]) || Tt(Yt[hn], sn[hn])))
                    return !0;
                }
              }
            }
            function It(Ct) {
              var Kt = Ct._fullLayout, Vt = Ct.getBoundingClientRect();
              if (!ae.equalDomRects(Vt, Kt._lastBBox)) {
                var Yt = Kt._invTransform = ae.inverseTransformMatrix(ae.getFullTransformMatrix(Ct));
                Kt._invScaleX = Math.sqrt(Yt[0][0] * Yt[0][0] + Yt[0][1] * Yt[0][1] + Yt[0][2] * Yt[0][2]), Kt._invScaleY = Math.sqrt(Yt[1][0] * Yt[1][0] + Yt[1][1] * Yt[1][1] + Yt[1][2] * Yt[1][2]), Kt._lastBBox = Vt;
              }
            }
            J.animate = function(Ct, Kt, Vt) {
              if (Ct = ae.getGraphDiv(Ct), !ae.isPlotDiv(Ct)) throw new Error("This element is not a Plotly plot: " + Ct + ". It's likely that you've failed to create a plot before animating it. For more details, see https://plotly.com/javascript/animations/");
              var Yt = Ct._transitionData;
              Yt._frameQueue || (Yt._frameQueue = []);
              var sn = (Vt = de.supplyAnimationDefaults(Vt)).transition, hn = Vt.frame;
              function Qt(rn) {
                return Array.isArray(sn) ? rn >= sn.length ? sn[0] : sn[rn] : sn;
              }
              function Ut(rn) {
                return Array.isArray(hn) ? rn >= hn.length ? hn[0] : hn[rn] : hn;
              }
              function Wt(rn, vn) {
                var dn = 0;
                return function() {
                  if (rn && ++dn === vn) return rn();
                };
              }
              return Yt._frameWaitingCnt === void 0 && (Yt._frameWaitingCnt = 0), new Promise(function(rn, vn) {
                function dn() {
                  Ct.emit("plotly_animating"), Yt._lastFrameAt = -1 / 0, Yt._timeToNext = 0, Yt._runningTransitions = 0, Yt._currentFrame = null;
                  var Bt = function() {
                    Yt._animationRaf = window.requestAnimationFrame(Bt), Date.now() - Yt._lastFrameAt > Yt._timeToNext && function() {
                      Yt._currentFrame && Yt._currentFrame.onComplete && Yt._currentFrame.onComplete();
                      var At = Yt._currentFrame = Yt._frameQueue.shift();
                      if (At) {
                        var Dt = At.name ? At.name.toString() : null;
                        Ct._fullLayout._currentFrame = Dt, Yt._lastFrameAt = Date.now(), Yt._timeToNext = At.frameOpts.duration, de.transition(Ct, At.frame.data, At.frame.layout, we.coerceTraceIndices(Ct, At.frame.traces), At.frameOpts, At.transitionOpts).then(function() {
                          At.onComplete && At.onComplete();
                        }), Ct.emit("plotly_animatingframe", { name: Dt, frame: At.frame, animation: { frame: At.frameOpts, transition: At.transitionOpts } });
                      } else Ct.emit("plotly_animated"), window.cancelAnimationFrame(Yt._animationRaf), Yt._animationRaf = null;
                    }();
                  };
                  Bt();
                }
                var Ht, jt, cn = 0;
                function wn(Bt) {
                  return Array.isArray(sn) ? cn >= sn.length ? Bt.transitionOpts = sn[cn] : Bt.transitionOpts = sn[0] : Bt.transitionOpts = sn, cn++, Bt;
                }
                var Rn = [], Pn = Kt == null, Ie = Array.isArray(Kt);
                if (Pn || Ie || !ae.isPlainObject(Kt)) {
                  if (Pn || ["string", "number"].indexOf(typeof Kt) !== -1) for (Ht = 0; Ht < Yt._frames.length; Ht++) (jt = Yt._frames[Ht]) && (Pn || String(jt.group) === String(Kt)) && Rn.push({ type: "byname", name: String(jt.name), data: wn({ name: jt.name }) });
                  else if (Ie) for (Ht = 0; Ht < Kt.length; Ht++) {
                    var rt = Kt[Ht];
                    ["number", "string"].indexOf(typeof rt) !== -1 ? (rt = String(rt), Rn.push({ type: "byname", name: rt, data: wn({ name: rt }) })) : ae.isPlainObject(rt) && Rn.push({ type: "object", data: wn(ae.extendFlat({}, rt)) });
                  }
                } else Rn.push({ type: "object", data: wn(ae.extendFlat({}, Kt)) });
                for (Ht = 0; Ht < Rn.length; Ht++) if ((jt = Rn[Ht]).type === "byname" && !Yt._frameHash[jt.data.name]) return ae.warn('animate failure: frame not found: "' + jt.data.name + '"'), void vn();
                ["next", "immediate"].indexOf(Vt.mode) !== -1 && function() {
                  if (Yt._frameQueue.length !== 0) {
                    for (; Yt._frameQueue.length; ) {
                      var Bt = Yt._frameQueue.pop();
                      Bt.onInterrupt && Bt.onInterrupt();
                    }
                    Ct.emit("plotly_animationinterrupted", []);
                  }
                }(), Vt.direction === "reverse" && Rn.reverse();
                var ft = Ct._fullLayout._currentFrame;
                if (ft && Vt.fromcurrent) {
                  var pt = -1;
                  for (Ht = 0; Ht < Rn.length; Ht++) if ((jt = Rn[Ht]).type === "byname" && jt.name === ft) {
                    pt = Ht;
                    break;
                  }
                  if (pt > 0 && pt < Rn.length - 1) {
                    var wt = [];
                    for (Ht = 0; Ht < Rn.length; Ht++) jt = Rn[Ht], (Rn[Ht].type !== "byname" || Ht > pt) && wt.push(jt);
                    Rn = wt;
                  }
                }
                Rn.length > 0 ? function(Bt) {
                  if (Bt.length !== 0) {
                    for (var At = 0; At < Bt.length; At++) {
                      var Dt;
                      Dt = Bt[At].type === "byname" ? de.computeFrame(Ct, Bt[At].name) : Bt[At].data;
                      var Ot = Ut(At), un = Qt(At);
                      un.duration = Math.min(un.duration, Ot.duration);
                      var yn = { frame: Dt, name: Bt[At].name, frameOpts: Ot, transitionOpts: un };
                      At === Bt.length - 1 && (yn.onComplete = Wt(rn, 2), yn.onInterrupt = vn), Yt._frameQueue.push(yn);
                    }
                    Vt.mode === "immediate" && (Yt._lastFrameAt = -1 / 0), Yt._animationRaf || dn();
                  }
                }(Rn) : (Ct.emit("plotly_animated"), rn());
              });
            }, J.addFrames = function(Ct, Kt, Vt) {
              if (Ct = ae.getGraphDiv(Ct), Kt == null) return Promise.resolve();
              if (!ae.isPlotDiv(Ct)) throw new Error("This element is not a Plotly plot: " + Ct + ". It's likely that you've failed to create a plot before adding frames. For more details, see https://plotly.com/javascript/animations/");
              var Yt, sn, hn, Qt, Ut = Ct._transitionData._frames, Wt = Ct._transitionData._frameHash;
              if (!Array.isArray(Kt)) throw new Error("addFrames failure: frameList must be an Array of frame definitions" + Kt);
              var rn = Ut.length + 2 * Kt.length, vn = [], dn = {};
              for (Yt = Kt.length - 1; Yt >= 0; Yt--) if (ae.isPlainObject(Kt[Yt])) {
                var Ht = Kt[Yt].name, jt = (Wt[Ht] || dn[Ht] || {}).name, cn = Kt[Yt].name, wn = Wt[jt] || dn[jt];
                jt && cn && typeof cn == "number" && wn && Qe < 5 && (Qe++, ae.warn('addFrames: overwriting frame "' + (Wt[jt] || dn[jt]).name + '" with a frame whose name of type "number" also equates to "' + jt + '". This is valid but may potentially lead to unexpected behavior since all plotly.js frame names are stored internally as strings.'), Qe === 5 && ae.warn("addFrames: This API call has yielded too many of these warnings. For the rest of this call, further warnings about numeric frame names will be suppressed.")), dn[Ht] = { name: Ht }, vn.push({ frame: de.supplyFrameDefaults(Kt[Yt]), index: Vt && Vt[Yt] !== void 0 && Vt[Yt] !== null ? Vt[Yt] : rn + Yt });
              }
              vn.sort(function(Bt, At) {
                return Bt.index > At.index ? -1 : Bt.index < At.index ? 1 : 0;
              });
              var Rn = [], Pn = [], Ie = Ut.length;
              for (Yt = vn.length - 1; Yt >= 0; Yt--) {
                if (typeof (sn = vn[Yt].frame).name == "number" && ae.warn("Warning: addFrames accepts frames with numeric names, but the numbers areimplicitly cast to strings"), !sn.name) for (; Wt[sn.name = "frame " + Ct._transitionData._counter++]; ) ;
                if (Wt[sn.name]) {
                  for (hn = 0; hn < Ut.length && (Ut[hn] || {}).name !== sn.name; hn++) ;
                  Rn.push({ type: "replace", index: hn, value: sn }), Pn.unshift({ type: "replace", index: hn, value: Ut[hn] });
                } else Qt = Math.max(0, Math.min(vn[Yt].index, Ie)), Rn.push({ type: "insert", index: Qt, value: sn }), Pn.unshift({ type: "delete", index: Qt }), Ie++;
              }
              var rt = de.modifyFrames, ft = de.modifyFrames, pt = [Ct, Pn], wt = [Ct, Rn];
              return ce && ce.add(Ct, rt, pt, ft, wt), de.modifyFrames(Ct, Rn);
            }, J.deleteFrames = function(Ct, Kt) {
              if (Ct = ae.getGraphDiv(Ct), !ae.isPlotDiv(Ct)) throw new Error("This element is not a Plotly plot: " + Ct);
              var Vt, Yt, sn = Ct._transitionData._frames, hn = [], Qt = [];
              if (!Kt) for (Kt = [], Vt = 0; Vt < sn.length; Vt++) Kt.push(Vt);
              for ((Kt = Kt.slice()).sort(), Vt = Kt.length - 1; Vt >= 0; Vt--) Yt = Kt[Vt], hn.push({ type: "delete", index: Yt }), Qt.unshift({ type: "insert", index: Yt, value: sn[Yt] });
              var Ut = de.modifyFrames, Wt = de.modifyFrames, rn = [Ct, Qt], vn = [Ct, hn];
              return ce && ce.add(Ct, Ut, rn, Wt, vn), de.modifyFrames(Ct, hn);
            }, J.addTraces = function Ct(Kt, Vt, Yt) {
              Kt = ae.getGraphDiv(Kt);
              var sn, hn, Qt = [], Ut = J.deleteTraces, Wt = Ct, rn = [Kt, Qt], vn = [Kt, Vt];
              for (function(dn, Ht, jt) {
                var cn, wn;
                if (!Array.isArray(dn.data)) throw new Error("gd.data must be an array.");
                if (Ht === void 0) throw new Error("traces must be defined.");
                for (Array.isArray(Ht) || (Ht = [Ht]), cn = 0; cn < Ht.length; cn++) if (typeof (wn = Ht[cn]) != "object" || Array.isArray(wn) || wn === null) throw new Error("all values in traces array must be non-array objects");
                if (jt === void 0 || Array.isArray(jt) || (jt = [jt]), jt !== void 0 && jt.length !== Ht.length) throw new Error("if indices is specified, traces.length must equal indices.length");
              }(Kt, Vt, Yt), Array.isArray(Vt) || (Vt = [Vt]), Vt = Vt.map(function(dn) {
                return ae.extendFlat({}, dn);
              }), we.cleanData(Vt), sn = 0; sn < Vt.length; sn++) Kt.data.push(Vt[sn]);
              for (sn = 0; sn < Vt.length; sn++) Qt.push(-Vt.length + sn);
              if (Yt === void 0) return hn = J.redraw(Kt), ce.add(Kt, Ut, rn, Wt, vn), hn;
              Array.isArray(Yt) || (Yt = [Yt]);
              try {
                qe(Kt, Qt, Yt);
              } catch (dn) {
                throw Kt.data.splice(Kt.data.length - Vt.length, Vt.length), dn;
              }
              return ce.startSequence(Kt), ce.add(Kt, Ut, rn, Wt, vn), hn = J.moveTraces(Kt, Qt, Yt), ce.stopSequence(Kt), hn;
            }, J.deleteTraces = function Ct(Kt, Vt) {
              Kt = ae.getGraphDiv(Kt);
              var Yt, sn, hn = [], Qt = J.addTraces, Ut = Ct, Wt = [Kt, hn, Vt], rn = [Kt, Vt];
              if (Vt === void 0) throw new Error("indices must be an integer or array of integers.");
              for (Array.isArray(Vt) || (Vt = [Vt]), $e(Kt, Vt, "indices"), (Vt = Pe(Vt, Kt.data.length - 1)).sort(ae.sorterDes), Yt = 0; Yt < Vt.length; Yt += 1) sn = Kt.data.splice(Vt[Yt], 1)[0], hn.push(sn);
              var vn = J.redraw(Kt);
              return ce.add(Kt, Qt, Wt, Ut, rn), vn;
            }, J.extendTraces = function Ct(Kt, Vt, Yt, sn) {
              var hn = Xe(Kt = ae.getGraphDiv(Kt), Vt, Yt, sn, function(Wt, rn, vn) {
                var dn, Ht;
                if (ae.isTypedArray(Wt)) if (vn < 0) {
                  var jt = new Wt.constructor(0), cn = Ve(Wt, rn);
                  vn < 0 ? (dn = cn, Ht = jt) : (dn = jt, Ht = cn);
                } else if (dn = new Wt.constructor(vn), Ht = new Wt.constructor(Wt.length + rn.length - vn), vn === rn.length) dn.set(rn), Ht.set(Wt);
                else if (vn < rn.length) {
                  var wn = rn.length - vn;
                  dn.set(rn.subarray(wn)), Ht.set(Wt), Ht.set(rn.subarray(0, wn), Wt.length);
                } else {
                  var Rn = vn - rn.length, Pn = Wt.length - Rn;
                  dn.set(Wt.subarray(Pn)), dn.set(rn, Rn), Ht.set(Wt.subarray(0, Pn));
                }
                else dn = Wt.concat(rn), Ht = vn >= 0 && vn < dn.length ? dn.splice(0, dn.length - vn) : [];
                return [dn, Ht];
              }), Qt = J.redraw(Kt), Ut = [Kt, hn.update, Yt, hn.maxPoints];
              return ce.add(Kt, J.prependTraces, Ut, Ct, arguments), Qt;
            }, J.moveTraces = function Ct(Kt, Vt, Yt) {
              var sn, hn = [], Qt = [], Ut = Ct, Wt = Ct, rn = [Kt = ae.getGraphDiv(Kt), Yt, Vt], vn = [Kt, Vt, Yt];
              if (qe(Kt, Vt, Yt), Vt = Array.isArray(Vt) ? Vt : [Vt], Yt === void 0) for (Yt = [], sn = 0; sn < Vt.length; sn++) Yt.push(-Vt.length + sn);
              for (Yt = Array.isArray(Yt) ? Yt : [Yt], Vt = Pe(Vt, Kt.data.length - 1), Yt = Pe(Yt, Kt.data.length - 1), sn = 0; sn < Kt.data.length; sn++) Vt.indexOf(sn) === -1 && hn.push(Kt.data[sn]);
              for (sn = 0; sn < Vt.length; sn++) Qt.push({ newIndex: Yt[sn], trace: Kt.data[Vt[sn]] });
              for (Qt.sort(function(Ht, jt) {
                return Ht.newIndex - jt.newIndex;
              }), sn = 0; sn < Qt.length; sn += 1) hn.splice(Qt[sn].newIndex, 0, Qt[sn].trace);
              Kt.data = hn;
              var dn = J.redraw(Kt);
              return ce.add(Kt, Ut, rn, Wt, vn), dn;
            }, J.prependTraces = function Ct(Kt, Vt, Yt, sn) {
              var hn = Xe(Kt = ae.getGraphDiv(Kt), Vt, Yt, sn, function(Wt, rn, vn) {
                var dn, Ht;
                if (ae.isTypedArray(Wt)) if (vn <= 0) {
                  var jt = new Wt.constructor(0), cn = Ve(rn, Wt);
                  vn < 0 ? (dn = cn, Ht = jt) : (dn = jt, Ht = cn);
                } else if (dn = new Wt.constructor(vn), Ht = new Wt.constructor(Wt.length + rn.length - vn), vn === rn.length) dn.set(rn), Ht.set(Wt);
                else if (vn < rn.length) {
                  var wn = rn.length - vn;
                  dn.set(rn.subarray(0, wn)), Ht.set(rn.subarray(wn)), Ht.set(Wt, wn);
                } else {
                  var Rn = vn - rn.length;
                  dn.set(rn), dn.set(Wt.subarray(0, Rn), rn.length), Ht.set(Wt.subarray(Rn));
                }
                else dn = rn.concat(Wt), Ht = vn >= 0 && vn < dn.length ? dn.splice(vn, dn.length) : [];
                return [dn, Ht];
              }), Qt = J.redraw(Kt), Ut = [Kt, hn.update, Yt, hn.maxPoints];
              return ce.add(Kt, J.extendTraces, Ut, Ct, arguments), Qt;
            }, J.newPlot = function(Ct, Kt, Vt, Yt) {
              return Ct = ae.getGraphDiv(Ct), de.cleanPlot([], {}, Ct._fullData || [], Ct._fullLayout || {}), de.purge(Ct), J._doPlot(Ct, Kt, Vt, Yt);
            }, J._doPlot = function(Ct, Kt, Vt, Yt) {
              var sn;
              if (Ct = ae.getGraphDiv(Ct), se.init(Ct), ae.isPlainObject(Kt)) {
                var hn = Kt;
                Kt = hn.data, Vt = hn.layout, Yt = hn.config, sn = hn.frames;
              }
              if (se.triggerHandler(Ct, "plotly_beforeplot", [Kt, Vt, Yt]) === !1) return Promise.reject();
              Kt || Vt || ae.isPlotDiv(Ct) || ae.warn("Calling _doPlot as if redrawing but this container doesn't yet have a plot.", Ct), De(Ct, Yt), Vt || (Vt = {}), te.select(Ct).classed("js-plotly-plot", !0), pe.makeTester(), Array.isArray(Ct._promises) || (Ct._promises = []);
              var Qt = (Ct.data || []).length === 0 && Array.isArray(Kt);
              Array.isArray(Kt) && (we.cleanData(Kt), Qt ? Ct.data = Kt : Ct.data.push.apply(Ct.data, Kt), Ct.empty = !1), Ct.layout && !Qt || (Ct.layout = we.cleanLayout(Vt)), de.supplyDefaults(Ct);
              var Ut = Ct._fullLayout, Wt = Ut._has("cartesian");
              Ut._replotting = !0, (Qt || Ut._shouldCreateBgLayer) && (function(Pn) {
                var Ie = te.select(Pn), rt = Pn._fullLayout;
                if (rt._calcInverseTransform = It, rt._calcInverseTransform(Pn), rt._container = Ie.selectAll(".plot-container").data([0]), rt._container.enter().insert("div", ":first-child").classed("plot-container", !0).classed("plotly", !0).style({ width: "100%", height: "100%" }), rt._paperdiv = rt._container.selectAll(".svg-container").data([0]), rt._paperdiv.enter().append("div").classed("user-select-none", !0).classed("svg-container", !0).style("position", "relative"), rt._glcontainer = rt._paperdiv.selectAll(".gl-container").data([{}]), rt._glcontainer.enter().append("div").classed("gl-container", !0), rt._paperdiv.selectAll(".main-svg").remove(), rt._paperdiv.select(".modebar-container").remove(), rt._paper = rt._paperdiv.insert("svg", ":first-child").classed("main-svg", !0), rt._toppaper = rt._paperdiv.append("svg").classed("main-svg", !0), rt._modebardiv = rt._paperdiv.append("div"), delete rt._modeBar, rt._hoverpaper = rt._paperdiv.append("svg").classed("main-svg", !0), !rt._uid) {
                  var ft = {};
                  te.selectAll("defs").each(function() {
                    this.id && (ft[this.id.split("-")[1]] = 1);
                  }), rt._uid = ae.randstr(ft);
                }
                rt._paperdiv.selectAll(".main-svg").attr(ve.svgAttrs), rt._defs = rt._paper.append("defs").attr("id", "defs-" + rt._uid), rt._clips = rt._defs.append("g").classed("clips", !0), rt._topdefs = rt._toppaper.append("defs").attr("id", "topdefs-" + rt._uid), rt._topclips = rt._topdefs.append("g").classed("clips", !0), rt._bgLayer = rt._paper.append("g").classed("bglayer", !0), rt._draggers = rt._paper.append("g").classed("draglayer", !0);
                var pt = rt._paper.append("g").classed("layer-below", !0);
                rt._imageLowerLayer = pt.append("g").classed("imagelayer", !0), rt._shapeLowerLayer = pt.append("g").classed("shapelayer", !0), rt._cartesianlayer = rt._paper.append("g").classed("cartesianlayer", !0), rt._polarlayer = rt._paper.append("g").classed("polarlayer", !0), rt._smithlayer = rt._paper.append("g").classed("smithlayer", !0), rt._ternarylayer = rt._paper.append("g").classed("ternarylayer", !0), rt._geolayer = rt._paper.append("g").classed("geolayer", !0), rt._funnelarealayer = rt._paper.append("g").classed("funnelarealayer", !0), rt._pielayer = rt._paper.append("g").classed("pielayer", !0), rt._iciclelayer = rt._paper.append("g").classed("iciclelayer", !0), rt._treemaplayer = rt._paper.append("g").classed("treemaplayer", !0), rt._sunburstlayer = rt._paper.append("g").classed("sunburstlayer", !0), rt._indicatorlayer = rt._toppaper.append("g").classed("indicatorlayer", !0), rt._glimages = rt._paper.append("g").classed("glimages", !0);
                var wt = rt._toppaper.append("g").classed("layer-above", !0);
                rt._imageUpperLayer = wt.append("g").classed("imagelayer", !0), rt._shapeUpperLayer = wt.append("g").classed("shapelayer", !0), rt._selectionLayer = rt._toppaper.append("g").classed("selectionlayer", !0), rt._infolayer = rt._toppaper.append("g").classed("infolayer", !0), rt._menulayer = rt._toppaper.append("g").classed("menulayer", !0), rt._zoomlayer = rt._toppaper.append("g").classed("zoomlayer", !0), rt._hoverlayer = rt._hoverpaper.append("g").classed("hoverlayer", !0), rt._modebardiv.classed("modebar-container", !0).style("position", "absolute").style("top", "0px").style("right", "0px"), Pn.emit("plotly_framework");
              }(Ct), Ut._shouldCreateBgLayer && delete Ut._shouldCreateBgLayer), pe.initGradients(Ct), pe.initPatterns(Ct), Qt && fe.saveShowSpikeInitial(Ct);
              var rn = !Ct.calcdata || Ct.calcdata.length !== (Ct._fullData || []).length;
              rn && de.doCalcdata(Ct);
              for (var vn = 0; vn < Ct.calcdata.length; vn++) Ct.calcdata[vn][0].trace = Ct._fullData[vn];
              Ct._context.responsive ? Ct._responsiveChartHandler || (Ct._responsiveChartHandler = function() {
                ae.isHidden(Ct) || de.resize(Ct);
              }, window.addEventListener("resize", Ct._responsiveChartHandler)) : ae.clearResponsive(Ct);
              var dn = ae.extendFlat({}, Ut._size), Ht = 0;
              function jt() {
                if (de.clearAutoMarginIds(Ct), Ee.drawMarginPushers(Ct), fe.allowAutoMargin(Ct), Ct._fullLayout.title.text && Ct._fullLayout.title.automargin && de.allowAutoMargin(Ct, "title.automargin"), Ut._has("pie")) for (var Pn = Ct._fullData, Ie = 0; Ie < Pn.length; Ie++) {
                  var rt = Pn[Ie];
                  rt.type === "pie" && rt.automargin && de.allowAutoMargin(Ct, "pie." + rt.uid + ".automargin");
                }
                return de.doAutoMargin(Ct), de.previousPromises(Ct);
              }
              function cn() {
                Ct._transitioning || (Ee.doAutoRangeAndConstraints(Ct), Qt && fe.saveRangeInitial(Ct), le.getComponentMethod("rangeslider", "calcAutorange")(Ct));
              }
              var wn = [de.previousPromises, function() {
                if (sn) return J.addFrames(Ct, sn);
              }, function Pn() {
                for (var Ie = Ut._basePlotModules, rt = 0; rt < Ie.length; rt++) Ie[rt].drawFramework && Ie[rt].drawFramework(Ct);
                !Ut._glcanvas && Ut._has("gl") && (Ut._glcanvas = Ut._glcontainer.selectAll(".gl-canvas").data([{ key: "contextLayer", context: !0, pick: !1 }, { key: "focusLayer", context: !1, pick: !1 }, { key: "pickLayer", context: !1, pick: !0 }], function(Bt) {
                  return Bt.key;
                }), Ut._glcanvas.enter().append("canvas").attr("class", function(Bt) {
                  return "gl-canvas gl-canvas-" + Bt.key.replace("Layer", "");
                }).style({ position: "absolute", top: 0, left: 0, overflow: "visible", "pointer-events": "none" }));
                var ft = Ct._context.plotGlPixelRatio;
                if (Ut._glcanvas) {
                  Ut._glcanvas.attr("width", Ut.width * ft).attr("height", Ut.height * ft).style("width", Ut.width + "px").style("height", Ut.height + "px");
                  var pt = Ut._glcanvas.data()[0].regl;
                  if (pt && (Math.floor(Ut.width * ft) !== pt._gl.drawingBufferWidth || Math.floor(Ut.height * ft) !== pt._gl.drawingBufferHeight)) {
                    var wt = "WebGL context buffer and canvas dimensions do not match due to browser/WebGL bug.";
                    if (!Ht) return ae.log(wt + " Clearing graph and plotting again."), de.cleanPlot([], {}, Ct._fullData, Ut), de.supplyDefaults(Ct), Ut = Ct._fullLayout, de.doCalcdata(Ct), Ht++, Pn();
                    ae.error(wt);
                  }
                }
                return Ut.modebar.orientation === "h" ? Ut._modebardiv.style("height", null).style("width", "100%") : Ut._modebardiv.style("width", null).style("height", Ut.height + "px"), de.previousPromises(Ct);
              }, jt, function() {
                if (de.didMarginChange(dn, Ut._size)) return ae.syncOrAsync([jt, Ee.layoutStyles], Ct);
              }];
              Wt && wn.push(function() {
                if (rn) return ae.syncOrAsync([le.getComponentMethod("shapes", "calcAutorange"), le.getComponentMethod("annotations", "calcAutorange"), cn], Ct);
                cn();
              }), wn.push(Ee.layoutStyles), Wt && wn.push(function() {
                return fe.draw(Ct, Qt ? "" : "redraw");
              }, function(Pn) {
                var Ie = Pn._fullLayout._insideTickLabelsUpdaterange;
                if (Ie) return Pn._fullLayout._insideTickLabelsUpdaterange = void 0, et(Pn, Ie).then(function() {
                  fe.saveRangeInitial(Pn, !0);
                });
              }), wn.push(Ee.drawData, Ee.finalDraw, ge, de.addLinks, de.rehover, de.redrag, de.reselect, de.doAutoMargin, de.previousPromises);
              var Rn = ae.syncOrAsync(wn, Ct);
              return Rn && Rn.then || (Rn = Promise.resolve()), Rn.then(function() {
                return ke(Ct), Ct;
              });
            }, J.purge = function(Ct) {
              var Kt = (Ct = ae.getGraphDiv(Ct))._fullLayout || {}, Vt = Ct._fullData || [];
              return de.cleanPlot([], {}, Vt, Kt), de.purge(Ct), se.purge(Ct), Kt._container && Kt._container.remove(), delete Ct._context, Ct;
            }, J.react = function(Ct, Kt, Vt, Yt) {
              var sn, hn;
              Ct = ae.getGraphDiv(Ct), we.clearPromiseQueue(Ct);
              var Qt = Ct._fullData, Ut = Ct._fullLayout;
              if (ae.isPlotDiv(Ct) && Qt && Ut) {
                if (ae.isPlainObject(Kt)) {
                  var Wt = Kt;
                  Kt = Wt.data, Vt = Wt.layout, Yt = Wt.config, sn = Wt.frames;
                }
                var rn = !1;
                if (Yt) {
                  var vn = ae.extendDeep({}, Ct._context);
                  Ct._context = void 0, De(Ct, Yt), rn = Tt(vn, Ct._context);
                }
                Ct.data = Kt || [], we.cleanData(Ct.data), Ct.layout = Vt || {}, we.cleanLayout(Ct.layout), function(_n, gn, bn, an) {
                  var Zt, Mn, Ln, In, kn, er, jn, ur, Kn, Jn, Lr = an._preGUI, mr = [], Ar = {}, Yr = {};
                  for (Zt in Lr) {
                    if (kn = $t(Zt, Rt)) {
                      if (Kn = kn.head, Jn = kn.tail, Mn = kn.attr || Kn + ".uirevision", (In = (Ln = oe(an, Mn).get()) && Pt(Mn, gn)) && In === Ln) {
                        if ((er = Lr[Zt]) === null && (er = void 0), Gt(ur = (jn = oe(gn, Zt)).get(), er)) {
                          ur === void 0 && Jn === "autorange" && mr.push(Kn), jn.set(Ge(oe(an, Zt).get()));
                          continue;
                        }
                        if (Jn === "autorange" || Jn.substr(0, 6) === "range[") {
                          var jr = Lr[Kn + ".range[0]"], Mr = Lr[Kn + ".range[1]"], Wr = Lr[Kn + ".autorange"];
                          if (Wr || Wr === null && jr === null && Mr === null) {
                            if (!(Kn in Ar)) {
                              var Bi = oe(gn, Kn).get();
                              Ar[Kn] = Bi && (Bi.autorange || Bi.autorange !== !1 && (!Bi.range || Bi.range.length !== 2));
                            }
                            if (Ar[Kn]) {
                              jn.set(Ge(oe(an, Zt).get()));
                              continue;
                            }
                          }
                        }
                      }
                    } else ae.warn("unrecognized GUI edit: " + Zt);
                    delete Lr[Zt], kn && kn.tail.substr(0, 6) === "range[" && (Yr[kn.head] = 1);
                  }
                  for (var gi = 0; gi < mr.length; gi++) {
                    var Zr = mr[gi];
                    if (Yr[Zr]) {
                      var fi = oe(gn, Zr).get();
                      fi && delete fi.autorange;
                    }
                  }
                  var ri = an._tracePreGUI;
                  for (var bi in ri) {
                    var Zi, $i = ri[bi], Br = null;
                    for (Zt in $i) {
                      if (!Br) {
                        var Pr = Xt(bi, bn);
                        if (Pr < 0) {
                          delete ri[bi];
                          break;
                        }
                        var ei = Nt(bi, _n, (Zi = bn[Pr]._fullInput).index);
                        if (ei < 0) {
                          delete ri[bi];
                          break;
                        }
                        Br = _n[ei];
                      }
                      if (kn = $t(Zt, vt)) {
                        if (kn.attr ? In = (Ln = oe(an, kn.attr).get()) && Pt(kn.attr, gn) : (Ln = Zi.uirevision, (In = Br.uirevision) === void 0 && (In = gn.uirevision)), In && In === Ln && ((er = $i[Zt]) === null && (er = void 0), Gt(ur = (jn = oe(Br, Zt)).get(), er))) {
                          jn.set(Ge(oe(Zi, Zt).get()));
                          continue;
                        }
                      } else ae.warn("unrecognized GUI edit: " + Zt + " in trace uid " + bi);
                      delete $i[Zt];
                    }
                  }
                }(Ct.data, Ct.layout, Qt, Ut), de.supplyDefaults(Ct, { skipUpdateCalc: !0 });
                var dn = Ct._fullData, Ht = Ct._fullLayout, jt = Ht.datarevision === void 0, cn = Ht.transition, wn = function(_n, gn, bn, an, Zt) {
                  var Mn = xe.layoutFlags();
                  return Mn.arrays = {}, Mn.rangesAltered = {}, Mn.nChanges = 0, Mn.nChangesAnim = 0, Ft(gn, bn, [], { getValObject: function(Ln) {
                    return ue.getLayoutValObject(bn, Ln);
                  }, flags: Mn, immutable: an, transition: Zt, gd: _n }), (Mn.plot || Mn.calc) && (Mn.layoutReplot = !0), Zt && Mn.nChanges && Mn.nChangesAnim && (Mn.anim = Mn.nChanges === Mn.nChangesAnim ? "all" : "some"), Mn;
                }(Ct, Ut, Ht, jt, cn), Rn = wn.newDataRevision, Pn = function(_n, gn, bn, an, Zt, Mn) {
                  var Ln = gn.length === bn.length;
                  if (!Zt && !Ln) return { fullReplot: !0, calc: !0 };
                  var In, kn, er = xe.traceFlags();
                  er.arrays = {}, er.nChanges = 0, er.nChangesAnim = 0;
                  var jn = { getValObject: function(Kn) {
                    var Jn = ue.getTraceValObject(kn, Kn);
                    return !kn._module.animatable && Jn.anim && (Jn.anim = !1), Jn;
                  }, flags: er, immutable: an, transition: Zt, newDataRevision: Mn, gd: _n }, ur = {};
                  for (In = 0; In < gn.length; In++) if (bn[In]) {
                    if (kn = bn[In]._fullInput, de.hasMakesDataTransform(kn) && (kn = bn[In]), ur[kn.uid]) continue;
                    ur[kn.uid] = 1, Ft(gn[In]._fullInput, kn, [], jn);
                  }
                  return (er.calc || er.plot) && (er.fullReplot = !0), Zt && er.nChanges && er.nChangesAnim && (er.anim = er.nChanges === er.nChangesAnim && Ln ? "all" : "some"), er;
                }(Ct, Qt, dn, jt, cn, Rn);
                if (yt(Ct) && (wn.layoutReplot = !0), Pn.calc || wn.calc) {
                  Ct.calcdata = void 0;
                  for (var Ie = Object.getOwnPropertyNames(Ht), rt = 0; rt < Ie.length; rt++) {
                    var ft = Ie[rt], pt = ft.substring(0, 5);
                    if (pt === "xaxis" || pt === "yaxis") {
                      var wt = Ht[ft]._emptyCategories;
                      wt && wt();
                    }
                  }
                } else de.supplyDefaultsUpdateCalc(Ct.calcdata, dn);
                var Bt = [];
                if (sn && (Ct._transitionData = {}, de.createTransitionData(Ct), Bt.push(function() {
                  return J.addFrames(Ct, sn);
                })), Ht.transition && !rn && (Pn.anim || wn.anim)) wn.ticks && Bt.push(Ee.doTicksRelayout), de.doCalcdata(Ct), Ee.doAutoRangeAndConstraints(Ct), Bt.push(function() {
                  return de.transitionFromReact(Ct, Pn, wn, Ut);
                });
                else if (Pn.fullReplot || wn.layoutReplot || rn) Ct._fullLayout._skipDefaults = !0, Bt.push(J._doPlot);
                else {
                  for (var At in wn.arrays) {
                    var Dt = wn.arrays[At];
                    if (Dt.length) {
                      var Ot = le.getComponentMethod(At, "drawOne");
                      if (Ot !== ae.noop) for (var un = 0; un < Dt.length; un++) Ot(Ct, Dt[un]);
                      else {
                        var yn = le.getComponentMethod(At, "draw");
                        if (yn === ae.noop) throw new Error("cannot draw components: " + At);
                        yn(Ct);
                      }
                    }
                  }
                  Bt.push(de.previousPromises), Pn.style && Bt.push(Ee.doTraceStyle), (Pn.colorbars || wn.colorbars) && Bt.push(Ee.doColorBars), wn.legend && Bt.push(Ee.doLegend), wn.layoutstyle && Bt.push(Ee.layoutStyles), wn.axrange && Je(Bt), wn.ticks && Bt.push(Ee.doTicksRelayout), wn.modebar && Bt.push(Ee.doModeBar), wn.camera && Bt.push(Ee.doCamera), Bt.push(ke);
                }
                Bt.push(de.rehover, de.redrag, de.reselect), (hn = ae.syncOrAsync(Bt, Ct)) && hn.then || (hn = Promise.resolve(Ct));
              } else hn = J.newPlot(Ct, Kt, Vt, Yt);
              return hn.then(function() {
                return Ct.emit("plotly_react", { data: Kt, layout: Vt }), Ct;
              });
            }, J.redraw = function(Ct) {
              if (Ct = ae.getGraphDiv(Ct), !ae.isPlotDiv(Ct)) throw new Error("This element is not a Plotly plot: " + Ct);
              return we.cleanData(Ct.data), we.cleanLayout(Ct.layout), Ct.calcdata = void 0, J._doPlot(Ct).then(function() {
                return Ct.emit("plotly_redraw"), Ct;
              });
            }, J.relayout = et, J.restyle = Ze, J.setPlotConfig = function(Ct) {
              return ae.extendFlat(Ce, Ct);
            }, J.update = St, J._guiRelayout = Lt(et), J._guiRestyle = Lt(Ze), J._guiUpdate = Lt(St), J._storeDirectGUIEdit = function(Ct, Kt, Vt) {
              for (var Yt in Vt) We(Yt, oe(Ct, Yt).get(), Vt[Yt], Kt);
            };
          }, 24452: function(G) {
            var J = { staticPlot: { valType: "boolean", dflt: !1 }, typesetMath: { valType: "boolean", dflt: !0 }, plotlyServerURL: { valType: "string", dflt: "" }, editable: { valType: "boolean", dflt: !1 }, edits: { annotationPosition: { valType: "boolean", dflt: !1 }, annotationTail: { valType: "boolean", dflt: !1 }, annotationText: { valType: "boolean", dflt: !1 }, axisTitleText: { valType: "boolean", dflt: !1 }, colorbarPosition: { valType: "boolean", dflt: !1 }, colorbarTitleText: { valType: "boolean", dflt: !1 }, legendPosition: { valType: "boolean", dflt: !1 }, legendText: { valType: "boolean", dflt: !1 }, shapePosition: { valType: "boolean", dflt: !1 }, titleText: { valType: "boolean", dflt: !1 } }, editSelection: { valType: "boolean", dflt: !0 }, autosizable: { valType: "boolean", dflt: !1 }, responsive: { valType: "boolean", dflt: !1 }, fillFrame: { valType: "boolean", dflt: !1 }, frameMargins: { valType: "number", dflt: 0, min: 0, max: 0.5 }, scrollZoom: { valType: "flaglist", flags: ["cartesian", "gl3d", "geo", "mapbox", "map"], extras: [!0, !1], dflt: "gl3d+geo+map" }, doubleClick: { valType: "enumerated", values: [!1, "reset", "autosize", "reset+autosize"], dflt: "reset+autosize" }, doubleClickDelay: { valType: "number", dflt: 300, min: 0 }, showAxisDragHandles: { valType: "boolean", dflt: !0 }, showAxisRangeEntryBoxes: { valType: "boolean", dflt: !0 }, showTips: { valType: "boolean", dflt: !0 }, showLink: { valType: "boolean", dflt: !1 }, linkText: { valType: "string", dflt: "Edit chart", noBlank: !0 }, sendData: { valType: "boolean", dflt: !0 }, showSources: { valType: "any", dflt: !1 }, displayModeBar: { valType: "enumerated", values: ["hover", !0, !1], dflt: "hover" }, showSendToCloud: { valType: "boolean", dflt: !1 }, showEditInChartStudio: { valType: "boolean", dflt: !1 }, modeBarButtonsToRemove: { valType: "any", dflt: [] }, modeBarButtonsToAdd: { valType: "any", dflt: [] }, modeBarButtons: { valType: "any", dflt: !1 }, toImageButtonOptions: { valType: "any", dflt: {} }, displaylogo: { valType: "boolean", dflt: !0 }, watermark: { valType: "boolean", dflt: !1 }, plotGlPixelRatio: { valType: "number", dflt: 2, min: 1, max: 4 }, setBackground: { valType: "any", dflt: "transparent" }, topojsonURL: { valType: "string", noBlank: !0, dflt: "https://cdn.plot.ly/" }, mapboxAccessToken: { valType: "string", dflt: null }, logging: { valType: "integer", min: 0, max: 2, dflt: 1 }, notifyOnLogging: { valType: "integer", min: 0, max: 2, dflt: 0 }, queueLength: { valType: "integer", min: 0, dflt: 0 }, globalTransforms: { valType: "any", dflt: [] }, locale: { valType: "string", dflt: "en-US" }, locales: { valType: "any", dflt: {} } }, Z = {};
            (function te(ne, ee) {
              for (var ae in ne) {
                var oe = ne[ae];
                oe.valType ? ee[ae] = oe.dflt : (ee[ae] || (ee[ae] = {}), te(oe, ee[ae]));
              }
            })(J, Z), G.exports = { configAttributes: J, dfltConfig: Z };
          }, 57297: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(9829), ae = Z(6704), oe = Z(58935), se = Z(49722), ce = Z(24452).configAttributes, le = Z(13582), ue = ne.extendDeepAll, de = ne.isPlainObject, fe = ne.isArrayOrTypedArray, he = ne.nestedProperty, me = ne.valObjectMeta, pe = "_isSubplotObj", ye = "_isLinkedToArray", ge = "_deprecated", ve = [pe, ye, "_arrayAttrRegexps", ge];
            function be(Le, Qe, ke) {
              if (!Le) return !1;
              if (Le._isLinkedToArray) {
                if (Ce(Qe[ke])) ke++;
                else if (ke < Qe.length) return !1;
              }
              for (; ke < Qe.length; ke++) {
                var Me = Le[Qe[ke]];
                if (!de(Me) || (Le = Me, ke === Qe.length - 1)) break;
                if (Le._isLinkedToArray) {
                  if (!Ce(Qe[++ke])) return !1;
                } else if (Le.valType === "info_array") {
                  var Re = Qe[++ke];
                  if (!Ce(Re)) return !1;
                  var De = Le.items;
                  if (Array.isArray(De)) {
                    if (Re >= De.length) return !1;
                    if (Le.dimensions === 2) {
                      if (ke++, Qe.length === ke) return Le;
                      var Pe = Qe[ke];
                      if (!Ce(Pe)) return !1;
                      Le = De[Re][Pe];
                    } else Le = De[Re];
                  } else Le = De;
                }
              }
              return Le;
            }
            function Ce(Le) {
              return Le === Math.round(Le) && Le >= 0;
            }
            function Se() {
              var Le, Qe, ke = {};
              for (Le in ue(ke, ae), te.subplotsRegistry) if ((Qe = te.subplotsRegistry[Le]).layoutAttributes) if (Array.isArray(Qe.attr)) for (var Me = 0; Me < Qe.attr.length; Me++) Ee(ke, Qe, Qe.attr[Me]);
              else Ee(ke, Qe, Qe.attr === "subplot" ? Qe.name : Qe.attr);
              for (Le in te.componentsRegistry) {
                var Re = (Qe = te.componentsRegistry[Le]).schema;
                if (Re && (Re.subplots || Re.layout)) {
                  var De = Re.subplots;
                  if (De && De.xaxis && !De.yaxis) for (var Pe in De.xaxis) delete ke.yaxis[Pe];
                  delete ke.xaxis.shift, delete ke.xaxis.autoshift;
                } else Qe.name === "colorscale" ? ue(ke, Qe.layoutAttributes) : Qe.layoutAttributes && xe(ke, Qe.layoutAttributes, Qe.name);
              }
              return { layoutAttributes: we(ke) };
            }
            function we(Le) {
              return function(Qe) {
                J.crawl(Qe, function(ke, Me, Re) {
                  J.isValObject(ke) ? ke.arrayOk !== !0 && ke.valType !== "data_array" || (Re[Me + "src"] = { valType: "string", editType: "none" }) : de(ke) && (ke.role = "object");
                });
              }(Le), function(Qe) {
                J.crawl(Qe, function(ke, Me, Re) {
                  if (ke) {
                    var De = ke[ye];
                    De && (delete ke[ye], Re[Me] = { items: {} }, Re[Me].items[De] = ke, Re[Me].role = "object");
                  }
                });
              }(Le), function(Qe) {
                (function ke(Me) {
                  for (var Re in Me) if (de(Me[Re])) ke(Me[Re]);
                  else if (Array.isArray(Me[Re])) for (var De = 0; De < Me[Re].length; De++) ke(Me[Re][De]);
                  else Me[Re] instanceof RegExp && (Me[Re] = Me[Re].toString());
                })(Qe);
              }(Le), Le;
            }
            function Ee(Le, Qe, ke) {
              var Me = he(Le, ke), Re = ue({}, Qe.layoutAttributes);
              Re[pe] = !0, Me.set(Re);
            }
            function xe(Le, Qe, ke) {
              var Me = he(Le, ke);
              Me.set(ue(Me.get() || {}, Qe));
            }
            J.IS_SUBPLOT_OBJ = pe, J.IS_LINKED_TO_ARRAY = ye, J.DEPRECATED = ge, J.UNDERSCORE_ATTRS = ve, J.get = function() {
              var Le = {};
              te.allTypes.forEach(function(Me) {
                Le[Me] = function(Re) {
                  var De, Pe;
                  Pe = (De = te.modules[Re]._module).basePlotModule;
                  var $e = { type: null }, qe = ue({}, ee), Xe = ue({}, De.attributes);
                  J.crawl(Xe, function(Ge, ze, We, at, ct) {
                    he(qe, ct).set(void 0), Ge === void 0 && he(Xe, ct).set(void 0);
                  }), ue($e, qe), te.traceIs(Re, "noOpacity") && delete $e.opacity, te.traceIs(Re, "showLegend") || (delete $e.showlegend, delete $e.legendgroup), te.traceIs(Re, "noHover") && (delete $e.hoverinfo, delete $e.hoverlabel), De.selectPoints || delete $e.selectedpoints, ue($e, Xe), Pe.attributes && ue($e, Pe.attributes), $e.type = Re;
                  var Ve = { meta: De.meta || {}, categories: De.categories || {}, animatable: !!De.animatable, type: Re, attributes: we($e) };
                  if (De.layoutAttributes) {
                    var Ze = {};
                    ue(Ze, De.layoutAttributes), Ve.layoutAttributes = we(Ze);
                  }
                  return De.animatable || J.crawl(Ve, function(Ge) {
                    J.isValObject(Ge) && "anim" in Ge && delete Ge.anim;
                  }), Ve;
                }(Me);
              });
              var Qe, ke = {};
              return Object.keys(te.transformsRegistry).forEach(function(Me) {
                ke[Me] = function(Re) {
                  var De = te.transformsRegistry[Re], Pe = ue({}, De.attributes);
                  return Object.keys(te.componentsRegistry).forEach(function($e) {
                    var qe = te.componentsRegistry[$e];
                    qe.schema && qe.schema.transforms && qe.schema.transforms[Re] && Object.keys(qe.schema.transforms[Re]).forEach(function(Xe) {
                      xe(Pe, qe.schema.transforms[Re][Xe], Xe);
                    });
                  }), { attributes: we(Pe) };
                }(Me);
              }), { defs: { valObjects: me, metaKeys: ve.concat(["description", "role", "editType", "impliedEdits"]), editType: { traces: le.traces, layout: le.layout }, impliedEdits: {} }, traces: Le, layout: Se(), transforms: ke, frames: (Qe = { frames: ue({}, oe) }, we(Qe), Qe.frames), animation: we(se), config: we(ce) };
            }, J.crawl = function(Le, Qe, ke, Me) {
              var Re = ke || 0;
              Me = Me || "", Object.keys(Le).forEach(function(De) {
                var Pe = Le[De];
                if (ve.indexOf(De) === -1) {
                  var $e = (Me ? Me + "." : "") + De;
                  Qe(Pe, De, Le, Re, $e), J.isValObject(Pe) || de(Pe) && De !== "impliedEdits" && J.crawl(Pe, Qe, Re + 1, $e);
                }
              });
            }, J.isValObject = function(Le) {
              return Le && Le.valType !== void 0;
            }, J.findArrayAttributes = function(Le) {
              var Qe, ke, Me = [], Re = [], De = [];
              function Pe(Ge, ze, We, at) {
                Re = Re.slice(0, at).concat([ze]), De = De.slice(0, at).concat([Ge && Ge._isLinkedToArray]), Ge && (Ge.valType === "data_array" || Ge.arrayOk === !0) && (Re[at - 1] !== "colorbar" || ze !== "ticktext" && ze !== "tickvals") && $e(Qe, 0, "");
              }
              function $e(Ge, ze, We) {
                var at = Ge[Re[ze]], ct = We + Re[ze];
                if (ze === Re.length - 1) fe(at) && Me.push(ke + ct);
                else if (De[ze]) {
                  if (Array.isArray(at)) for (var et = 0; et < at.length; et++) de(at[et]) && $e(at[et], ze + 1, ct + "[" + et + "].");
                } else de(at) && $e(at, ze + 1, ct + ".");
              }
              Qe = Le, ke = "", J.crawl(ee, Pe), Le._module && Le._module.attributes && J.crawl(Le._module.attributes, Pe);
              var qe = Le.transforms;
              if (qe) for (var Xe = 0; Xe < qe.length; Xe++) {
                var Ve = qe[Xe], Ze = Ve._module;
                Ze && (ke = "transforms[" + Xe + "].", Qe = Ve, J.crawl(Ze.attributes, Pe));
              }
              return Me;
            }, J.getTraceValObject = function(Le, Qe) {
              var ke, Me, Re = Qe[0], De = 1;
              if (Re === "transforms") {
                if (Qe.length === 1) return ee.transforms;
                var Pe = Le.transforms;
                if (!Array.isArray(Pe) || !Pe.length) return !1;
                var $e = Qe[1];
                if (!Ce($e) || $e >= Pe.length) return !1;
                Me = (ke = (te.transformsRegistry[Pe[$e].type] || {}).attributes) && ke[Qe[2]], De = 3;
              } else {
                var qe = Le._module;
                if (qe || (qe = (te.modules[Le.type || ee.type.dflt] || {})._module), !qe) return !1;
                if (!(Me = (ke = qe.attributes) && ke[Re])) {
                  var Xe = qe.basePlotModule;
                  Xe && Xe.attributes && (Me = Xe.attributes[Re]);
                }
                Me || (Me = ee[Re]);
              }
              return be(Me, Qe, De);
            }, J.getLayoutValObject = function(Le, Qe) {
              var ke = function(Me, Re) {
                var De, Pe, $e, qe, Xe = Me._basePlotModules;
                if (Xe) {
                  var Ve;
                  for (De = 0; De < Xe.length; De++) {
                    if (($e = Xe[De]).attrRegex && $e.attrRegex.test(Re)) {
                      if ($e.layoutAttrOverrides) return $e.layoutAttrOverrides;
                      !Ve && $e.layoutAttributes && (Ve = $e.layoutAttributes);
                    }
                    var Ze = $e.baseLayoutAttrOverrides;
                    if (Ze && Re in Ze) return Ze[Re];
                  }
                  if (Ve) return Ve;
                }
                var Ge = Me._modules;
                if (Ge) {
                  for (De = 0; De < Ge.length; De++) if ((qe = Ge[De].layoutAttributes) && Re in qe) return qe[Re];
                }
                for (Pe in te.componentsRegistry) {
                  if (($e = te.componentsRegistry[Pe]).name === "colorscale" && Re.indexOf("coloraxis") === 0) return $e.layoutAttributes[Re];
                  if (!$e.schema && Re === $e.name) return $e.layoutAttributes;
                }
                return Re in ae && ae[Re];
              }(Le, Qe[0]);
              return be(ke, Qe, 1);
            };
          }, 78032: function(G, J, Z) {
            var te = Z(34809), ne = Z(9829), ee = "templateitemname", ae = { name: { valType: "string", editType: "none" } };
            function oe(ce) {
              return ce && typeof ce == "string";
            }
            function se(ce) {
              var le = ce.length - 1;
              return ce.charAt(le) !== "s" && te.warn("bad argument to arrayDefaultKey: " + ce), ce.substr(0, ce.length - 1) + "defaults";
            }
            ae[ee] = { valType: "string", editType: "calc" }, J.templatedArray = function(ce, le) {
              return le._isLinkedToArray = ce, le.name = ae.name, le[ee] = ae[ee], le;
            }, J.traceTemplater = function(ce) {
              var le, ue, de = {};
              for (le in ce) ue = ce[le], Array.isArray(ue) && ue.length && (de[le] = 0);
              return { newTrace: function(fe) {
                var he = { type: le = te.coerce(fe, {}, ne, "type"), _template: null };
                if (le in de) {
                  ue = ce[le];
                  var me = de[le] % ue.length;
                  de[le]++, he._template = ue[me];
                }
                return he;
              } };
            }, J.newContainer = function(ce, le, ue) {
              var de = ce._template, fe = de && (de[le] || ue && de[ue]);
              return te.isPlainObject(fe) || (fe = null), ce[le] = { _template: fe };
            }, J.arrayTemplater = function(ce, le, ue) {
              var de = ce._template, fe = de && de[se(le)], he = de && de[le];
              Array.isArray(he) && he.length || (he = []);
              var me = {};
              return { newItem: function(pe) {
                var ye = { name: pe.name, _input: pe }, ge = ye[ee] = pe[ee];
                if (!oe(ge)) return ye._template = fe, ye;
                for (var ve = 0; ve < he.length; ve++) {
                  var be = he[ve];
                  if (be.name === ge) return me[ge] = 1, ye._template = be, ye;
                }
                return ye[ue] = pe[ue] || !1, ye._template = !1, ye;
              }, defaultItems: function() {
                for (var pe = [], ye = 0; ye < he.length; ye++) {
                  var ge = he[ye], ve = ge.name;
                  if (oe(ve) && !me[ve]) {
                    var be = { _template: ge, name: ve, _input: { _templateitemname: ve } };
                    be[ee] = ge[ee], pe.push(be), me[ve] = 1;
                  }
                }
                return pe;
              } };
            }, J.arrayDefaultKey = se, J.arrayEditor = function(ce, le, ue) {
              var de = (te.nestedProperty(ce, le).get() || []).length, fe = ue._index, he = fe >= de && (ue._input || {})._templateitemname;
              he && (fe = de);
              var me, pe = le + "[" + fe + "]";
              function ye() {
                me = {}, he && (me[pe] = {}, me[pe][ee] = he);
              }
              function ge(be, Ce) {
                he ? te.nestedProperty(me[pe], be).set(Ce) : me[pe + "." + be] = Ce;
              }
              function ve() {
                var be = me;
                return ye(), be;
              }
              return ye(), { modifyBase: function(be, Ce) {
                me[be] = Ce;
              }, modifyItem: ge, getUpdateObj: ve, applyUpdate: function(be, Ce) {
                be && ge(be, Ce);
                var Se = ve();
                for (var we in Se) te.nestedProperty(ce, we).set(Se[we]);
              } };
            };
          }, 71817: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(44122), ae = Z(34809), oe = Z(30635), se = Z(34823), ce = Z(78766), le = Z(62203), ue = Z(17240), de = Z(95433), fe = Z(29714), he = Z(4530), me = Z(84391), pe = me.enforce, ye = me.clean, ge = Z(32919).doAutoRange, ve = "start", be = Z(54826).zindexSeparator;
            function Ce(Qe, ke, Me) {
              for (var Re = 0; Re < Me.length; Re++) {
                var De = Me[Re][0], Pe = Me[Re][1];
                if (!(De[0] >= Qe[1] || De[1] <= Qe[0]) && Pe[0] < ke[1] && Pe[1] > ke[0]) return !0;
              }
              return !1;
            }
            function Se(Qe) {
              var ke, Me, Re, De, Pe, $e, qe = Qe._fullLayout, Xe = qe._size, Ve = Xe.p, Ze = fe.list(Qe, "", !0);
              if (qe._paperdiv.style({ width: Qe._context.responsive && qe.autosize && !Qe._context._hasZeroWidth && !Qe.layout.width ? "100%" : qe.width + "px", height: Qe._context.responsive && qe.autosize && !Qe._context._hasZeroHeight && !Qe.layout.height ? "100%" : qe.height + "px" }).selectAll(".main-svg").call(le.setSize, qe.width, qe.height), Qe._context.setBackground(Qe, qe.paper_bgcolor), J.drawMainTitle(Qe), de.manage(Qe), !qe._has("cartesian")) return ee.previousPromises(Qe);
              function Ge(dn, Ht, jt) {
                var cn = dn._lw / 2;
                return dn._id.charAt(0) === "x" ? Ht ? jt === "top" ? Ht._offset - Ve - cn : Ht._offset + Ht._length + Ve + cn : Xe.t + Xe.h * (1 - (dn.position || 0)) + cn % 1 : Ht ? jt === "right" ? Ht._offset + Ht._length + Ve + cn : Ht._offset - Ve - cn : Xe.l + Xe.w * (dn.position || 0) + cn % 1;
              }
              for (ke = 0; ke < Ze.length; ke++) {
                var ze = (De = Ze[ke])._anchorAxis;
                De._linepositions = {}, De._lw = le.crispRound(Qe, De.linewidth, 1), De._mainLinePosition = Ge(De, ze, De.side), De._mainMirrorPosition = De.mirror && ze ? Ge(De, ze, he.OPPOSITE_SIDE[De.side]) : null;
              }
              var We = [], at = [], ct = [], et = ce.opacity(qe.paper_bgcolor) === 1 && ce.opacity(qe.plot_bgcolor) === 1 && qe.paper_bgcolor === qe.plot_bgcolor;
              for (Me in qe._plots) if ((Re = qe._plots[Me]).mainplot) Re.bg && Re.bg.remove(), Re.bg = void 0;
              else {
                var dt = Re.xaxis.domain, Je = Re.yaxis.domain, it = Re.plotgroup;
                if (Ce(dt, Je, ct) && Me.indexOf(be) === -1) {
                  var st = it.node(), gt = Re.bg = ae.ensureSingle(it, "rect", "bg");
                  st.insertBefore(gt.node(), st.childNodes[0]), at.push(Me);
                } else it.select("rect.bg").remove(), ct.push([dt, Je]), et || (We.push(Me), at.push(Me));
              }
              var bt, yt, St, Lt, Rt, vt, $t, Pt, Xt, Nt, Gt, Ft, Tt, It = qe._bgLayer.selectAll(".bg").data(We);
              for (It.enter().append("rect").classed("bg", !0), It.exit().remove(), It.each(function(dn) {
                qe._plots[dn].bg = te.select(this);
              }), ke = 0; ke < at.length; ke++) Re = qe._plots[at[ke]], Pe = Re.xaxis, $e = Re.yaxis, Re.bg && Pe._offset !== void 0 && $e._offset !== void 0 && Re.bg.call(le.setRect, Pe._offset - Ve, $e._offset - Ve, Pe._length + 2 * Ve, $e._length + 2 * Ve).call(ce.fill, qe.plot_bgcolor).style("stroke-width", 0);
              if (!qe._hasOnlyLargeSploms) for (Me in qe._plots) {
                Re = qe._plots[Me], Pe = Re.xaxis, $e = Re.yaxis;
                var Ct, Kt, Vt = Re.clipId = "clip" + qe._uid + Me + "plot", Yt = ae.ensureSingleById(qe._clips, "clipPath", Vt, function(dn) {
                  dn.classed("plotclip", !0).append("rect");
                });
                Re.clipRect = Yt.select("rect").attr({ width: Pe._length, height: $e._length }), le.setTranslate(Re.plot, Pe._offset, $e._offset), Re._hasClipOnAxisFalse ? (Ct = null, Kt = Vt) : (Ct = Vt, Kt = null), le.setClipUrl(Re.plot, Ct, Qe), Re.layerClipId = Kt;
              }
              function sn(dn) {
                return "M" + bt + "," + dn + "H" + yt;
              }
              function hn(dn) {
                return "M" + Pe._offset + "," + dn + "h" + Pe._length;
              }
              function Qt(dn) {
                return "M" + dn + "," + Pt + "V" + $t;
              }
              function Ut(dn) {
                return $e._shift !== void 0 && (dn += $e._shift), "M" + dn + "," + $e._offset + "v" + $e._length;
              }
              function Wt(dn, Ht, jt) {
                if (!dn.showline || Me !== dn._mainSubplot) return "";
                if (!dn._anchorAxis) return jt(dn._mainLinePosition);
                var cn = Ht(dn._mainLinePosition);
                return dn.mirror && (cn += Ht(dn._mainMirrorPosition)), cn;
              }
              for (Me in qe._plots) {
                Re = qe._plots[Me], Pe = Re.xaxis, $e = Re.yaxis;
                var rn = "M0,0";
                we(Pe, Me) && (Rt = xe(Pe, "left", $e, Ze), bt = Pe._offset - (Rt ? Ve + Rt : 0), vt = xe(Pe, "right", $e, Ze), yt = Pe._offset + Pe._length + (vt ? Ve + vt : 0), St = Ge(Pe, $e, "bottom"), Lt = Ge(Pe, $e, "top"), !(Tt = !Pe._anchorAxis || Me !== Pe._mainSubplot) || Pe.mirror !== "allticks" && Pe.mirror !== "all" || (Pe._linepositions[Me] = [St, Lt]), rn = Wt(Pe, sn, hn), Tt && Pe.showline && (Pe.mirror === "all" || Pe.mirror === "allticks") && (rn += sn(St) + sn(Lt)), Re.xlines.style("stroke-width", Pe._lw + "px").call(ce.stroke, Pe.showline ? Pe.linecolor : "rgba(0,0,0,0)")), Re.xlines.attr("d", rn);
                var vn = "M0,0";
                we($e, Me) && (Gt = xe($e, "bottom", Pe, Ze), $t = $e._offset + $e._length + (Gt ? Ve : 0), Ft = xe($e, "top", Pe, Ze), Pt = $e._offset - (Ft ? Ve : 0), Xt = Ge($e, Pe, "left"), Nt = Ge($e, Pe, "right"), !(Tt = !$e._anchorAxis || Me !== $e._mainSubplot) || $e.mirror !== "allticks" && $e.mirror !== "all" || ($e._linepositions[Me] = [Xt, Nt]), vn = Wt($e, Qt, Ut), Tt && $e.showline && ($e.mirror === "all" || $e.mirror === "allticks") && (vn += Qt(Xt) + Qt(Nt)), Re.ylines.style("stroke-width", $e._lw + "px").call(ce.stroke, $e.showline ? $e.linecolor : "rgba(0,0,0,0)")), Re.ylines.attr("d", vn);
              }
              return fe.makeClipPaths(Qe), ee.previousPromises(Qe);
            }
            function we(Qe, ke) {
              return (Qe.ticks || Qe.showline) && (ke === Qe._mainSubplot || Qe.mirror === "all" || Qe.mirror === "allticks");
            }
            function Ee(Qe, ke, Me) {
              if (!Me.showline || !Me._lw) return !1;
              if (Me.mirror === "all" || Me.mirror === "allticks") return !0;
              var Re = Me._anchorAxis;
              if (!Re) return !1;
              var De = he.FROM_BL[ke];
              return Me.side === ke ? Re.domain[De] === Qe.domain[De] : Me.mirror && Re.domain[1 - De] === Qe.domain[1 - De];
            }
            function xe(Qe, ke, Me, Re) {
              if (Ee(Qe, ke, Me)) return Me._lw;
              for (var De = 0; De < Re.length; De++) {
                var Pe = Re[De];
                if (Pe._mainAxis === Me._mainAxis && Ee(Qe, ke, Pe)) return Pe._lw;
              }
              return 0;
            }
            function Le(Qe) {
              return Qe === "top" ? he.CAP_SHIFT + 0.3 + "em" : Qe === "bottom" ? "-0.3em" : he.MID_SHIFT + "em";
            }
            J.layoutStyles = function(Qe) {
              return ae.syncOrAsync([ee.doAutoMargin, Se], Qe);
            }, J.drawMainTitle = function(Qe) {
              var ke = Qe._fullLayout.title, Me = Qe._fullLayout, Re = function(ct) {
                var et = ct.title, dt = "middle";
                return ae.isRightAnchor(et) ? dt = "end" : ae.isLeftAnchor(et) && (dt = ve), dt;
              }(Me), De = function(ct) {
                var et = ct.title, dt = "0em";
                return ae.isTopAnchor(et) ? dt = he.CAP_SHIFT + "em" : ae.isMiddleAnchor(et) && (dt = he.MID_SHIFT + "em"), dt;
              }(Me), Pe = function(ct, et) {
                var dt = ct.title, Je = ct._size, it = 0;
                return et !== "0em" && et ? et === he.CAP_SHIFT + "em" && (it = dt.pad.t) : it = -dt.pad.b, dt.y === "auto" ? Je.t / 2 : dt.yref === "paper" ? Je.t + Je.h - Je.h * dt.y + it : ct.height - ct.height * dt.y + it;
              }(Me, De), $e = function(ct, et) {
                var dt = ct.title, Je = ct._size, it = 0;
                return et === ve ? it = dt.pad.l : et === "end" && (it = -dt.pad.r), dt.xref === "paper" ? Je.l + Je.w * dt.x + it : ct.width * dt.x + it;
              }(Me, Re);
              if (ue.draw(Qe, "gtitle", { propContainer: Me, propName: "title.text", subtitlePropName: "title.subtitle.text", placeholder: Me._dfltTitle.plot, subtitlePlaceholder: Me._dfltTitle.subtitle, attributes: { x: $e, y: Pe, "text-anchor": Re, dy: De } }), ke.text && ke.automargin) {
                var qe = te.selectAll(".gtitle"), Xe = le.bBox(te.selectAll(".g-gtitle").node()).height, Ve = function(ct, et, dt) {
                  var Je = et.y, it = et.yanchor, st = Je > 0.5 ? "t" : "b", gt = ct._fullLayout.margin[st], bt = 0;
                  return et.yref === "paper" ? bt = dt + et.pad.t + et.pad.b : et.yref === "container" && (bt = function(yt, St, Lt, Rt, vt) {
                    var $t = 0;
                    return Lt === "middle" && ($t += vt / 2), yt === "t" ? (Lt === "top" && ($t += vt), $t += Rt - St * Rt) : (Lt === "bottom" && ($t += vt), $t += St * Rt), $t;
                  }(st, Je, it, ct._fullLayout.height, dt) + et.pad.t + et.pad.b), bt > gt ? bt : 0;
                }(Qe, ke, Xe);
                if (Ve > 0) {
                  (function(ct, et, dt, Je) {
                    var it = "title.automargin", st = ct._fullLayout.title, gt = st.y > 0.5 ? "t" : "b", bt = { x: st.x, y: st.y, t: 0, b: 0 }, yt = {};
                    st.yref === "paper" && function(St, Lt, Rt, vt, $t) {
                      var Pt = Lt.yref === "paper" ? St._fullLayout._size.h : St._fullLayout.height, Xt = ae.isTopAnchor(Lt) ? vt : vt - $t, Nt = Rt === "b" ? Pt - Xt : Xt;
                      return !(ae.isTopAnchor(Lt) && Rt === "t" || ae.isBottomAnchor(Lt) && Rt === "b") && Nt < $t;
                    }(ct, st, gt, et, Je) ? bt[gt] = dt : st.yref === "container" && (yt[gt] = dt, ct._fullLayout._reservedMargin[it] = yt), ee.allowAutoMargin(ct, it), ee.autoMargin(ct, it, bt);
                  })(Qe, Pe, Ve, Xe), qe.attr({ x: $e, y: Pe, "text-anchor": Re, dy: Le(ke.yanchor) }).call(oe.positionText, $e, Pe);
                  var Ze = (ke.text.match(oe.BR_TAG_ALL) || []).length;
                  if (Ze) {
                    var Ge = he.LINE_SPACING * Ze + he.MID_SHIFT;
                    ke.y === 0 && (Ge = -Ge), qe.selectAll(".line").each(function() {
                      var ct = +this.getAttribute("dy").slice(0, -2) - Ge + "em";
                      this.setAttribute("dy", ct);
                    });
                  }
                  var ze = te.selectAll(".gtitle-subtitle");
                  if (ze.node()) {
                    var We = qe.node().getBBox(), at = We.y + We.height + ue.SUBTITLE_PADDING_EM * ke.subtitle.font.size;
                    ze.attr({ x: $e, y: at, "text-anchor": Re, dy: Le(ke.yanchor) }).call(oe.positionText, $e, at);
                  }
                }
              }
            }, J.doTraceStyle = function(Qe) {
              var ke, Me = Qe.calcdata, Re = [];
              for (ke = 0; ke < Me.length; ke++) {
                var De = Me[ke], Pe = De[0] || {}, $e = Pe.trace || {}, qe = $e._module || {}, Xe = qe.arraysToCalcdata;
                Xe && Xe(De, $e);
                var Ve = qe.editStyle;
                Ve && Re.push({ fn: Ve, cd0: Pe });
              }
              if (Re.length) {
                for (ke = 0; ke < Re.length; ke++) {
                  var Ze = Re[ke];
                  Ze.fn(Qe, Ze.cd0);
                }
                se(Qe), J.redrawReglTraces(Qe);
              }
              return ee.style(Qe), ne.getComponentMethod("legend", "draw")(Qe), ee.previousPromises(Qe);
            }, J.doColorBars = function(Qe) {
              return ne.getComponentMethod("colorbar", "draw")(Qe), ee.previousPromises(Qe);
            }, J.layoutReplot = function(Qe) {
              var ke = Qe.layout;
              return Qe.layout = void 0, ne.call("_doPlot", Qe, "", ke);
            }, J.doLegend = function(Qe) {
              return ne.getComponentMethod("legend", "draw")(Qe), ee.previousPromises(Qe);
            }, J.doTicksRelayout = function(Qe) {
              return fe.draw(Qe, "redraw"), Qe._fullLayout._hasOnlyLargeSploms && (ne.subplotsRegistry.splom.updateGrid(Qe), se(Qe), J.redrawReglTraces(Qe)), J.drawMainTitle(Qe), ee.previousPromises(Qe);
            }, J.doModeBar = function(Qe) {
              var ke = Qe._fullLayout;
              de.manage(Qe);
              for (var Me = 0; Me < ke._basePlotModules.length; Me++) {
                var Re = ke._basePlotModules[Me].updateFx;
                Re && Re(Qe);
              }
              return ee.previousPromises(Qe);
            }, J.doCamera = function(Qe) {
              for (var ke = Qe._fullLayout, Me = ke._subplots.gl3d, Re = 0; Re < Me.length; Re++) {
                var De = ke[Me[Re]];
                De._scene.setViewport(De);
              }
            }, J.drawData = function(Qe) {
              var ke = Qe._fullLayout;
              se(Qe);
              for (var Me = ke._basePlotModules, Re = 0; Re < Me.length; Re++) Me[Re].plot(Qe);
              return J.redrawReglTraces(Qe), ee.style(Qe), ne.getComponentMethod("selections", "draw")(Qe), ne.getComponentMethod("shapes", "draw")(Qe), ne.getComponentMethod("annotations", "draw")(Qe), ne.getComponentMethod("images", "draw")(Qe), ke._replotting = !1, ee.previousPromises(Qe);
            }, J.redrawReglTraces = function(Qe) {
              var ke = Qe._fullLayout;
              if (ke._has("regl")) {
                var Me, Re, De = Qe._fullData, Pe = [], $e = [];
                for (ke._hasOnlyLargeSploms && ke._splomGrid.draw(), Me = 0; Me < De.length; Me++) {
                  var qe = De[Me];
                  qe.visible === !0 && qe._length !== 0 && (qe.type === "splom" ? ke._splomScenes[qe.uid].draw() : qe.type === "scattergl" ? ae.pushUnique(Pe, qe.xaxis + qe.yaxis) : qe.type === "scatterpolargl" && ae.pushUnique($e, qe.subplot));
                }
                for (Me = 0; Me < Pe.length; Me++) (Re = ke._plots[Pe[Me]])._scene && Re._scene.draw();
                for (Me = 0; Me < $e.length; Me++) (Re = ke[$e[Me]]._subplot)._scene && Re._scene.draw();
              }
            }, J.doAutoRangeAndConstraints = function(Qe) {
              for (var ke, Me = fe.list(Qe, "", !0), Re = {}, De = 0; De < Me.length; De++) if (!Re[(ke = Me[De])._id]) {
                Re[ke._id] = 1, ye(Qe, ke), ge(Qe, ke);
                var Pe = ke._matchGroup;
                if (Pe) for (var $e in Pe) {
                  var qe = fe.getFromId(Qe, $e);
                  ge(Qe, qe, ke.range), Re[$e] = 1;
                }
              }
              pe(Qe);
            }, J.finalDraw = function(Qe) {
              ne.getComponentMethod("rangeslider", "draw")(Qe), ne.getComponentMethod("rangeselector", "draw")(Qe);
            }, J.drawMarginPushers = function(Qe) {
              ne.getComponentMethod("legend", "draw")(Qe), ne.getComponentMethod("rangeselector", "draw")(Qe), ne.getComponentMethod("sliders", "draw")(Qe), ne.getComponentMethod("updatemenus", "draw")(Qe), ne.getComponentMethod("colorbar", "draw")(Qe);
            };
          }, 53853: function(G, J, Z) {
            var te = Z(34809), ne = te.isPlainObject, ee = Z(57297), ae = Z(44122), oe = Z(9829), se = Z(78032), ce = Z(24452).dfltConfig;
            function le(ge, ve) {
              ge = te.extendDeep({}, ge);
              var be, Ce, Se = Object.keys(ge).sort();
              function we(ke, Me, Re) {
                if (ne(Me) && ne(ke)) le(ke, Me);
                else if (Array.isArray(Me) && Array.isArray(ke)) {
                  var De = se.arrayTemplater({ _template: ge }, Re);
                  for (Ce = 0; Ce < Me.length; Ce++) {
                    var Pe = Me[Ce], $e = De.newItem(Pe)._template;
                    $e && le($e, Pe);
                  }
                  var qe = De.defaultItems();
                  for (Ce = 0; Ce < qe.length; Ce++) Me.push(qe[Ce]._template);
                  for (Ce = 0; Ce < Me.length; Ce++) delete Me[Ce].templateitemname;
                }
              }
              for (be = 0; be < Se.length; be++) {
                var Ee = Se[be], xe = ge[Ee];
                if (Ee in ve ? we(xe, ve[Ee], Ee) : ve[Ee] = xe, ue(Ee) === Ee) for (var Le in ve) {
                  var Qe = ue(Le);
                  Le === Qe || Qe !== Ee || Le in ge || we(xe, ve[Le], Ee);
                }
              }
            }
            function ue(ge) {
              return ge.replace(/[0-9]+$/, "");
            }
            function de(ge, ve, be, Ce, Se) {
              var we = Se && be(Se);
              for (var Ee in ge) {
                var xe = ge[Ee], Le = me(ge, Ee, Ce), Qe = me(ge, Ee, Se), ke = be(Qe);
                if (!ke) {
                  var Me = ue(Ee);
                  Me !== Ee && (ke = be(Qe = me(ge, Me, Se)));
                }
                if (!(we && we === ke || !ke || ke._noTemplating || ke.valType === "data_array" || ke.arrayOk && Array.isArray(xe))) if (!ke.valType && ne(xe)) de(xe, ve, be, Le, Qe);
                else if (ke._isLinkedToArray && Array.isArray(xe)) for (var Re = !1, De = 0, Pe = {}, $e = 0; $e < xe.length; $e++) {
                  var qe = xe[$e];
                  if (ne(qe)) {
                    var Xe = qe.name;
                    if (Xe) Pe[Xe] || (de(qe, ve, be, me(xe, De, Le), me(xe, De, Qe)), De++, Pe[Xe] = 1);
                    else if (!Re) {
                      var Ve = me(ge, se.arrayDefaultKey(Ee), Ce), Ze = me(xe, De, Le);
                      de(qe, ve, be, Ze, me(xe, De, Qe));
                      var Ge = te.nestedProperty(ve, Ze);
                      te.nestedProperty(ve, Ve).set(Ge.get()), Ge.set(null), Re = !0;
                    }
                  }
                }
                else te.nestedProperty(ve, Le).set(xe);
              }
            }
            function fe(ge, ve) {
              return ee.getLayoutValObject(ge, te.nestedProperty({}, ve).parts);
            }
            function he(ge, ve) {
              return ee.getTraceValObject(ge, te.nestedProperty({}, ve).parts);
            }
            function me(ge, ve, be) {
              return be ? Array.isArray(ge) ? be + "[" + ve + "]" : be + "." + ve : ve;
            }
            function pe(ge) {
              for (var ve = 0; ve < ge.length; ve++) if (ne(ge[ve])) return !0;
            }
            function ye(ge) {
              var ve;
              switch (ge.code) {
                case "data":
                  ve = "The template has no key data.";
                  break;
                case "layout":
                  ve = "The template has no key layout.";
                  break;
                case "missing":
                  ve = ge.path ? "There are no templates for item " + ge.path + " with name " + ge.templateitemname : "There are no templates for trace " + ge.index + ", of type " + ge.traceType + ".";
                  break;
                case "unused":
                  ve = ge.path ? "The template item at " + ge.path + " was not used in constructing the plot." : ge.dataCount ? "Some of the templates of type " + ge.traceType + " were not used. The template has " + ge.templateCount + " traces, the data only has " + ge.dataCount + " of this type." : "The template has " + ge.templateCount + " traces of type " + ge.traceType + " but there are none in the data.";
                  break;
                case "reused":
                  ve = "Some of the templates of type " + ge.traceType + " were used more than once. The template has " + ge.templateCount + " traces, the data has " + ge.dataCount + " of this type.";
              }
              return ge.msg = ve, ge;
            }
            J.makeTemplate = function(ge) {
              ge = te.isPlainObject(ge) ? ge : te.getGraphDiv(ge), ge = te.extendDeep({ _context: ce }, { data: ge.data, layout: ge.layout }), ae.supplyDefaults(ge);
              var ve = ge.data || [], be = ge.layout || {};
              be._basePlotModules = ge._fullLayout._basePlotModules, be._modules = ge._fullLayout._modules;
              var Ce = { data: {}, layout: {} };
              ve.forEach(function(De) {
                var Pe = {};
                de(De, Pe, he.bind(null, De));
                var $e = te.coerce(De, {}, oe, "type"), qe = Ce.data[$e];
                qe || (qe = Ce.data[$e] = []), qe.push(Pe);
              }), de(be, Ce.layout, fe.bind(null, be)), delete Ce.layout.template;
              var Se = be.template;
              if (ne(Se)) {
                var we, Ee, xe, Le, Qe, ke, Me = Se.layout;
                ne(Me) && le(Me, Ce.layout);
                var Re = Se.data;
                if (ne(Re)) {
                  for (Ee in Ce.data) if (xe = Re[Ee], Array.isArray(xe)) {
                    for (ke = (Qe = Ce.data[Ee]).length, Le = xe.length, we = 0; we < ke; we++) le(xe[we % Le], Qe[we]);
                    for (we = ke; we < Le; we++) Qe.push(te.extendDeep({}, xe[we]));
                  }
                  for (Ee in Re) Ee in Ce.data || (Ce.data[Ee] = te.extendDeep([], Re[Ee]));
                }
              }
              return Ce;
            }, J.validateTemplate = function(ge, ve) {
              var be = te.extendDeep({}, { _context: ce, data: ge.data, layout: ge.layout }), Ce = be.layout || {};
              ne(ve) || (ve = Ce.template || {});
              var Se = ve.layout, we = ve.data, Ee = [];
              be.layout = Ce, be.layout.template = ve, ae.supplyDefaults(be);
              var xe = be._fullLayout, Le = be._fullData, Qe = {};
              if (ne(Se) ? (function qe(Xe, Ve) {
                for (var Ze in Xe) if (Ze.charAt(0) !== "_" && ne(Xe[Ze])) {
                  var Ge, ze = ue(Ze), We = [];
                  for (Ge = 0; Ge < Ve.length; Ge++) We.push(me(Xe, Ze, Ve[Ge])), ze !== Ze && We.push(me(Xe, ze, Ve[Ge]));
                  for (Ge = 0; Ge < We.length; Ge++) Qe[We[Ge]] = 1;
                  qe(Xe[Ze], We);
                }
              }(xe, ["layout"]), function qe(Xe, Ve) {
                for (var Ze in Xe) if (Ze.indexOf("defaults") === -1 && ne(Xe[Ze])) {
                  var Ge = me(Xe, Ze, Ve);
                  Qe[Ge] ? qe(Xe[Ze], Ge) : Ee.push({ code: "unused", path: Ge });
                }
              }(Se, "layout")) : Ee.push({ code: "layout" }), ne(we)) {
                for (var ke, Me = {}, Re = 0; Re < Le.length; Re++) {
                  var De = Le[Re];
                  Me[ke = De.type] = (Me[ke] || 0) + 1, De._fullInput._template || Ee.push({ code: "missing", index: De._fullInput.index, traceType: ke });
                }
                for (ke in we) {
                  var Pe = we[ke].length, $e = Me[ke] || 0;
                  Pe > $e ? Ee.push({ code: "unused", traceType: ke, templateCount: Pe, dataCount: $e }) : $e > Pe && Ee.push({ code: "reused", traceType: ke, templateCount: Pe, dataCount: $e });
                }
              } else Ee.push({ code: "data" });
              if (function qe(Xe, Ve) {
                for (var Ze in Xe) if (Ze.charAt(0) !== "_") {
                  var Ge = Xe[Ze], ze = me(Xe, Ze, Ve);
                  ne(Ge) ? (Array.isArray(Xe) && Ge._template === !1 && Ge.templateitemname && Ee.push({ code: "missing", path: ze, templateitemname: Ge.templateitemname }), qe(Ge, ze)) : Array.isArray(Ge) && pe(Ge) && qe(Ge, ze);
                }
              }({ data: Le, layout: xe }, ""), Ee.length) return Ee.map(ye);
            };
          }, 80491: function(G, J, Z) {
            var te = Z(10721), ne = Z(31420), ee = Z(44122), ae = Z(34809), oe = Z(84619), se = Z(6243), ce = Z(72914), le = Z(29697).version, ue = { format: { valType: "enumerated", values: ["png", "jpeg", "webp", "svg", "full-json"], dflt: "png" }, width: { valType: "number", min: 1 }, height: { valType: "number", min: 1 }, scale: { valType: "number", min: 0, dflt: 1 }, setBackground: { valType: "any", dflt: !1 }, imageDataOnly: { valType: "boolean", dflt: !1 } };
            G.exports = function(de, fe) {
              var he, me, pe, ye;
              function ge($e) {
                return !($e in fe) || ae.validate(fe[$e], ue[$e]);
              }
              if (fe = fe || {}, ae.isPlainObject(de) ? (he = de.data || [], me = de.layout || {}, pe = de.config || {}, ye = {}) : (de = ae.getGraphDiv(de), he = ae.extendDeep([], de.data), me = ae.extendDeep({}, de.layout), pe = de._context, ye = de._fullLayout || {}), !ge("width") && fe.width !== null || !ge("height") && fe.height !== null) throw new Error("Height and width should be pixel values.");
              if (!ge("format")) throw new Error("Export format is not " + ae.join2(ue.format.values, ", ", " or ") + ".");
              var ve = {};
              function be($e, qe) {
                return ae.coerce(fe, ve, ue, $e, qe);
              }
              var Ce = be("format"), Se = be("width"), we = be("height"), Ee = be("scale"), xe = be("setBackground"), Le = be("imageDataOnly"), Qe = document.createElement("div");
              Qe.style.position = "absolute", Qe.style.left = "-5000px", document.body.appendChild(Qe);
              var ke = ae.extendFlat({}, me);
              Se ? ke.width = Se : fe.width === null && te(ye.width) && (ke.width = ye.width), we ? ke.height = we : fe.height === null && te(ye.height) && (ke.height = ye.height);
              var Me = ae.extendFlat({}, pe, { _exportedPlot: !0, staticPlot: !0, setBackground: xe }), Re = oe.getRedrawFunc(Qe);
              function De() {
                return new Promise(function($e) {
                  setTimeout($e, oe.getDelay(Qe._fullLayout));
                });
              }
              function Pe() {
                return new Promise(function($e, qe) {
                  var Xe = se(Qe, Ce, Ee), Ve = Qe._fullLayout.width, Ze = Qe._fullLayout.height;
                  function Ge() {
                    ne.purge(Qe), document.body.removeChild(Qe);
                  }
                  if (Ce === "full-json") {
                    var ze = ee.graphJson(Qe, !1, "keepdata", "object", !0, !0);
                    return ze.version = le, ze = JSON.stringify(ze), Ge(), $e(Le ? ze : oe.encodeJSON(ze));
                  }
                  if (Ge(), Ce === "svg") return $e(Le ? Xe : oe.encodeSVG(Xe));
                  var We = document.createElement("canvas");
                  We.id = ae.randstr(), ce({ format: Ce, width: Ve, height: Ze, scale: Ee, canvas: We, svg: Xe, promise: !0 }).then($e).catch(qe);
                });
              }
              return new Promise(function($e, qe) {
                ne.newPlot(Qe, he, ke, Me).then(Re).then(De).then(Pe).then(function(Xe) {
                  $e(function(Ve) {
                    return Le ? Ve.replace(oe.IMAGE_URL_PREFIX, "") : Ve;
                  }(Xe));
                }).catch(function(Xe) {
                  qe(Xe);
                });
              });
            };
          }, 2466: function(G, J, Z) {
            var te = Z(34809), ne = Z(44122), ee = Z(57297), ae = Z(24452).dfltConfig, oe = te.isPlainObject, se = Array.isArray, ce = te.isArrayOrTypedArray;
            function le(ge, ve, be, Ce, Se, we) {
              we = we || [];
              for (var Ee = Object.keys(ge), xe = 0; xe < Ee.length; xe++) {
                var Le = Ee[xe];
                if (Le !== "transforms") {
                  var Qe = we.slice();
                  Qe.push(Le);
                  var ke = ge[Le], Me = ve[Le], Re = me(be, Le), De = (Re || {}).valType, Pe = De === "info_array", $e = De === "colorscale", qe = (Re || {}).items;
                  if (he(be, Le)) if (oe(ke) && oe(Me) && De !== "any") le(ke, Me, Re, Ce, Se, Qe);
                  else if (Pe && se(ke)) {
                    ke.length > Me.length && Ce.push(fe("unused", Se, Qe.concat(Me.length)));
                    var Xe, Ve, Ze, Ge, ze, We = Me.length, at = Array.isArray(qe);
                    if (at && (We = Math.min(We, qe.length)), Re.dimensions === 2) for (Ve = 0; Ve < We; Ve++) if (se(ke[Ve])) {
                      ke[Ve].length > Me[Ve].length && Ce.push(fe("unused", Se, Qe.concat(Ve, Me[Ve].length)));
                      var ct = Me[Ve].length;
                      for (Xe = 0; Xe < (at ? Math.min(ct, qe[Ve].length) : ct); Xe++) Ze = at ? qe[Ve][Xe] : qe, Ge = ke[Ve][Xe], ze = Me[Ve][Xe], te.validate(Ge, Ze) ? ze !== Ge && ze !== +Ge && Ce.push(fe("dynamic", Se, Qe.concat(Ve, Xe), Ge, ze)) : Ce.push(fe("value", Se, Qe.concat(Ve, Xe), Ge));
                    } else Ce.push(fe("array", Se, Qe.concat(Ve), ke[Ve]));
                    else for (Ve = 0; Ve < We; Ve++) Ze = at ? qe[Ve] : qe, Ge = ke[Ve], ze = Me[Ve], te.validate(Ge, Ze) ? ze !== Ge && ze !== +Ge && Ce.push(fe("dynamic", Se, Qe.concat(Ve), Ge, ze)) : Ce.push(fe("value", Se, Qe.concat(Ve), Ge));
                  } else if (Re.items && !Pe && se(ke)) {
                    var et, dt, Je = qe[Object.keys(qe)[0]], it = [];
                    for (et = 0; et < Me.length; et++) {
                      var st = Me[et]._index || et;
                      if ((dt = Qe.slice()).push(st), oe(ke[st]) && oe(Me[et])) {
                        it.push(st);
                        var gt = ke[st], bt = Me[et];
                        oe(gt) && gt.visible !== !1 && bt.visible === !1 ? Ce.push(fe("invisible", Se, dt)) : le(gt, bt, Je, Ce, Se, dt);
                      }
                    }
                    for (et = 0; et < ke.length; et++) (dt = Qe.slice()).push(et), oe(ke[et]) ? it.indexOf(et) === -1 && Ce.push(fe("unused", Se, dt)) : Ce.push(fe("object", Se, dt, ke[et]));
                  } else !oe(ke) && oe(Me) ? Ce.push(fe("object", Se, Qe, ke)) : ce(ke) || !ce(Me) || Pe || $e ? Le in ve ? te.validate(ke, Re) ? Re.valType === "enumerated" && (Re.coerceNumber && ke !== +Me || ke !== Me) && Ce.push(fe("dynamic", Se, Qe, ke, Me)) : Ce.push(fe("value", Se, Qe, ke)) : Ce.push(fe("unused", Se, Qe, ke)) : Ce.push(fe("array", Se, Qe, ke));
                  else Ce.push(fe("schema", Se, Qe));
                }
              }
              return Ce;
            }
            G.exports = function(ge, ve) {
              ge === void 0 && (ge = []), ve === void 0 && (ve = {});
              var be, Ce, Se = ee.get(), we = [], Ee = { _context: te.extendFlat({}, ae) };
              se(ge) ? (Ee.data = te.extendDeep([], ge), be = ge) : (Ee.data = [], be = [], we.push(fe("array", "data"))), oe(ve) ? (Ee.layout = te.extendDeep({}, ve), Ce = ve) : (Ee.layout = {}, Ce = {}, arguments.length > 1 && we.push(fe("object", "layout"))), ne.supplyDefaults(Ee);
              for (var xe = Ee._fullData, Le = be.length, Qe = 0; Qe < Le; Qe++) {
                var ke = be[Qe], Me = ["data", Qe];
                if (oe(ke)) {
                  var Re = xe[Qe], De = Re.type, Pe = Se.traces[De].attributes;
                  Pe.type = { valType: "enumerated", values: [De] }, Re.visible === !1 && ke.visible !== !1 && we.push(fe("invisible", Me)), le(ke, Re, Pe, we, Me);
                  var $e = ke.transforms, qe = Re.transforms;
                  if ($e) {
                    se($e) || we.push(fe("array", Me, ["transforms"])), Me.push("transforms");
                    for (var Xe = 0; Xe < $e.length; Xe++) {
                      var Ve = ["transforms", Xe], Ze = $e[Xe].type;
                      if (oe($e[Xe])) {
                        var Ge = Se.transforms[Ze] ? Se.transforms[Ze].attributes : {};
                        Ge.type = { valType: "enumerated", values: Object.keys(Se.transforms) }, le($e[Xe], qe[Xe], Ge, we, Me, Ve);
                      } else we.push(fe("object", Me, Ve));
                    }
                  }
                } else we.push(fe("object", Me));
              }
              var ze = Ee._fullLayout, We = function(at, ct) {
                for (var et = at.layout.layoutAttributes, dt = 0; dt < ct.length; dt++) {
                  var Je = ct[dt], it = at.traces[Je.type], st = it.layoutAttributes;
                  st && (Je.subplot ? te.extendFlat(et[it.attributes.subplot.dflt], st) : te.extendFlat(et, st));
                }
                return et;
              }(Se, xe);
              return le(Ce, ze, We, we, "layout"), we.length === 0 ? void 0 : we;
            };
            var ue = { object: function(ge, ve) {
              return (ge === "layout" && ve === "" ? "The layout argument" : ge[0] === "data" && ve === "" ? "Trace " + ge[1] + " in the data argument" : de(ge) + "key " + ve) + " must be linked to an object container";
            }, array: function(ge, ve) {
              return (ge === "data" ? "The data argument" : de(ge) + "key " + ve) + " must be linked to an array container";
            }, schema: function(ge, ve) {
              return de(ge) + "key " + ve + " is not part of the schema";
            }, unused: function(ge, ve, be) {
              var Ce = oe(be) ? "container" : "key";
              return de(ge) + Ce + " " + ve + " did not get coerced";
            }, dynamic: function(ge, ve, be, Ce) {
              return [de(ge) + "key", ve, "(set to '" + be + "')", "got reset to", "'" + Ce + "'", "during defaults."].join(" ");
            }, invisible: function(ge, ve) {
              return (ve ? de(ge) + "item " + ve : "Trace " + ge[1]) + " got defaulted to be not visible";
            }, value: function(ge, ve, be) {
              return [de(ge) + "key " + ve, "is set to an invalid value (" + be + ")"].join(" ");
            } };
            function de(ge) {
              return se(ge) ? "In data trace " + ge[1] + ", " : "In " + ge + ", ";
            }
            function fe(ge, ve, be, Ce, Se) {
              var we, Ee;
              be = be || "", se(ve) ? (we = ve[0], Ee = ve[1]) : (we = ve, Ee = null);
              var xe = function(Qe) {
                if (!se(Qe)) return String(Qe);
                for (var ke = "", Me = 0; Me < Qe.length; Me++) {
                  var Re = Qe[Me];
                  typeof Re == "number" ? ke = ke.substr(0, ke.length - 1) + "[" + Re + "]" : ke += Re, Me < Qe.length - 1 && (ke += ".");
                }
                return ke;
              }(be), Le = ue[ge](ve, xe, Ce, Se);
              return te.log(Le), { code: ge, container: we, trace: Ee, path: be, astr: xe, msg: Le };
            }
            function he(ge, ve) {
              var be = ye(ve), Ce = be.keyMinusId, Se = be.id;
              return !!(Ce in ge && ge[Ce]._isSubplotObj && Se) || ve in ge;
            }
            function me(ge, ve) {
              return ve in ge ? ge[ve] : ge[ye(ve).keyMinusId];
            }
            var pe = te.counterRegex("([a-z]+)");
            function ye(ge) {
              var ve = ge.match(pe);
              return { keyMinusId: ve && ve[1], id: ve && ve[2] };
            }
          }, 49722: function(G) {
            G.exports = { mode: { valType: "enumerated", dflt: "afterall", values: ["immediate", "next", "afterall"] }, direction: { valType: "enumerated", values: ["forward", "reverse"], dflt: "forward" }, fromcurrent: { valType: "boolean", dflt: !1 }, frame: { duration: { valType: "number", min: 0, dflt: 500 }, redraw: { valType: "boolean", dflt: !0 } }, transition: { duration: { valType: "number", min: 0, dflt: 500, editType: "none" }, easing: { valType: "enumerated", dflt: "cubic-in-out", values: ["linear", "quad", "cubic", "sin", "exp", "circle", "elastic", "back", "bounce", "linear-in", "quad-in", "cubic-in", "sin-in", "exp-in", "circle-in", "elastic-in", "back-in", "bounce-in", "linear-out", "quad-out", "cubic-out", "sin-out", "exp-out", "circle-out", "elastic-out", "back-out", "bounce-out", "linear-in-out", "quad-in-out", "cubic-in-out", "sin-in-out", "exp-in-out", "circle-in-out", "elastic-in-out", "back-in-out", "bounce-in-out"], editType: "none" }, ordering: { valType: "enumerated", values: ["layout first", "traces first"], dflt: "layout first", editType: "none" } } };
          }, 59008: function(G, J, Z) {
            var te = Z(34809), ne = Z(78032);
            G.exports = function(ee, ae, oe) {
              var se, ce, le = oe.name, ue = oe.inclusionAttr || "visible", de = ae[le], fe = te.isArrayOrTypedArray(ee[le]) ? ee[le] : [], he = ae[le] = [], me = ne.arrayTemplater(ae, le, ue);
              for (se = 0; se < fe.length; se++) {
                var pe = fe[se];
                te.isPlainObject(pe) ? ce = me.newItem(pe) : (ce = me.newItem({}))[ue] = !1, ce._index = se, ce[ue] !== !1 && oe.handleItemDefaults(pe, ce, ae, oe), he.push(ce);
              }
              var ye = me.defaultItems();
              for (se = 0; se < ye.length; se++) (ce = ye[se])._index = he.length, oe.handleItemDefaults({}, ce, ae, oe, {}), he.push(ce);
              if (te.isArrayOrTypedArray(de)) {
                var ge = Math.min(de.length, he.length);
                for (se = 0; se < ge; se++) te.relinkPrivateKeys(he[se], de[se]);
              }
              return he;
            };
          }, 9829: function(G, J, Z) {
            var te = Z(80337), ne = Z(70192);
            G.exports = { type: { valType: "enumerated", values: [], dflt: "scatter", editType: "calc+clearAxisTypes", _noTemplating: !0 }, visible: { valType: "enumerated", values: [!0, !1, "legendonly"], dflt: !0, editType: "calc" }, showlegend: { valType: "boolean", dflt: !0, editType: "style" }, legend: { valType: "subplotid", dflt: "legend", editType: "style" }, legendgroup: { valType: "string", dflt: "", editType: "style" }, legendgrouptitle: { text: { valType: "string", dflt: "", editType: "style" }, font: te({ editType: "style" }), editType: "style" }, legendrank: { valType: "number", dflt: 1e3, editType: "style" }, legendwidth: { valType: "number", min: 0, editType: "style" }, opacity: { valType: "number", min: 0, max: 1, dflt: 1, editType: "style" }, name: { valType: "string", editType: "style" }, uid: { valType: "string", editType: "plot", anim: !0 }, ids: { valType: "data_array", editType: "calc", anim: !0 }, customdata: { valType: "data_array", editType: "calc" }, meta: { valType: "any", arrayOk: !0, editType: "plot" }, selectedpoints: { valType: "any", editType: "calc" }, hoverinfo: { valType: "flaglist", flags: ["x", "y", "z", "text", "name"], extras: ["all", "none", "skip"], arrayOk: !0, dflt: "all", editType: "none" }, hoverlabel: ne.hoverlabel, stream: { token: { valType: "string", noBlank: !0, strict: !0, editType: "calc" }, maxpoints: { valType: "number", min: 0, max: 1e4, dflt: 500, editType: "calc" }, editType: "calc" }, transforms: { _isLinkedToArray: "transform", editType: "calc" }, uirevision: { valType: "any", editType: "none" } };
          }, 40528: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = ne.dateTime2ms, ae = ne.incrementMonth, oe = Z(63821).ONEAVGMONTH;
            G.exports = function(se, ce, le, ue) {
              if (ce.type !== "date") return { vals: ue };
              var de = se[le + "periodalignment"];
              if (!de) return { vals: ue };
              var fe, he = se[le + "period"];
              if (te(he)) {
                if ((he = +he) <= 0) return { vals: ue };
              } else if (typeof he == "string" && he.charAt(0) === "M") {
                var me = +he.substring(1);
                if (!(me > 0 && Math.round(me) === me)) return { vals: ue };
                fe = me;
              }
              for (var pe = ce.calendar, ye = de === "start", ge = de === "end", ve = se[le + "period0"], be = ee(ve, pe) || 0, Ce = [], Se = [], we = [], Ee = ue.length, xe = 0; xe < Ee; xe++) {
                var Le, Qe, ke, Me = ue[xe];
                if (fe) {
                  for (Le = Math.round((Me - be) / (fe * oe)), ke = ae(be, fe * Le, pe); ke > Me; ) ke = ae(ke, -fe, pe);
                  for (; ke <= Me; ) ke = ae(ke, fe, pe);
                  Qe = ae(ke, -fe, pe);
                } else {
                  for (ke = be + (Le = Math.round((Me - be) / he)) * he; ke > Me; ) ke -= he;
                  for (; ke <= Me; ) ke += he;
                  Qe = ke - he;
                }
                Ce[xe] = ye ? Qe : ge ? ke : (Qe + ke) / 2, Se[xe] = Qe, we[xe] = ke;
              }
              return { vals: Ce, starts: Se, ends: we };
            };
          }, 55126: function(G) {
            G.exports = { xaxis: { valType: "subplotid", dflt: "x", editType: "calc+clearAxisTypes" }, yaxis: { valType: "subplotid", dflt: "y", editType: "calc+clearAxisTypes" } };
          }, 32919: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(34809), ae = Z(63821).FP_SAFE, oe = Z(33626), se = Z(62203), ce = Z(5975), le = ce.getFromId, ue = ce.isLinked;
            function de(xe, Le) {
              var Qe, ke, Me = [], Re = xe._fullLayout, De = he(Re, Le, 0), Pe = he(Re, Le, 1), $e = pe(xe, Le), qe = $e.min, Xe = $e.max;
              if (qe.length === 0 || Xe.length === 0) return ee.simpleMap(Le.range, Le.r2l);
              var Ve = qe[0].val, Ze = Xe[0].val;
              for (Qe = 1; Qe < qe.length && Ve === Ze; Qe++) Ve = Math.min(Ve, qe[Qe].val);
              for (Qe = 1; Qe < Xe.length && Ve === Ze; Qe++) Ze = Math.max(Ze, Xe[Qe].val);
              var Ge = Le.autorange, ze = Ge === "reversed" || Ge === "min reversed" || Ge === "max reversed";
              if (!ze && Le.range) {
                var We = ee.simpleMap(Le.range, Le.r2l);
                ze = We[1] < We[0];
              }
              Le.autorange === "reversed" && (Le.autorange = !0);
              var at, ct, et, dt, Je, it, st = Le.rangemode, gt = st === "tozero", bt = st === "nonnegative", yt = Le._length, St = yt / 10, Lt = 0;
              for (Qe = 0; Qe < qe.length; Qe++) for (at = qe[Qe], ke = 0; ke < Xe.length; ke++) (it = (ct = Xe[ke]).val - at.val - fe(Le, at.val, ct.val)) > 0 && ((Je = yt - De(at) - Pe(ct)) > St ? it / Je > Lt && (et = at, dt = ct, Lt = it / Je) : it / yt > Lt && (et = { val: at.val, nopad: 1 }, dt = { val: ct.val, nopad: 1 }, Lt = it / yt));
              if (Ve === Ze) {
                var Rt = Ve - 1, vt = Ve + 1;
                if (gt) if (Ve === 0) Me = [0, 1];
                else {
                  var $t = (Ve > 0 ? Xe : qe).reduce(function(Xt, Nt) {
                    return Math.max(Xt, Pe(Nt));
                  }, 0), Pt = Ve / (1 - Math.min(0.5, $t / yt));
                  Me = Ve > 0 ? [0, Pt] : [Pt, 0];
                }
                else Me = bt ? [Math.max(0, Rt), Math.max(1, vt)] : [Rt, vt];
              } else gt ? (et.val >= 0 && (et = { val: 0, nopad: 1 }), dt.val <= 0 && (dt = { val: 0, nopad: 1 })) : bt && (et.val - Lt * De(et) < 0 && (et = { val: 0, nopad: 1 }), dt.val <= 0 && (dt = { val: 1, nopad: 1 })), Lt = (dt.val - et.val - fe(Le, at.val, ct.val)) / (yt - De(et) - Pe(dt)), Me = [et.val - Lt * De(et), dt.val + Lt * Pe(dt)];
              return Me = Ee(Me, Le), Le.limitRange && Le.limitRange(), ze && Me.reverse(), ee.simpleMap(Me, Le.l2r || Number);
            }
            function fe(xe, Le, Qe) {
              var ke = 0;
              if (xe.rangebreaks) for (var Me = xe.locateBreaks(Le, Qe), Re = 0; Re < Me.length; Re++) {
                var De = Me[Re];
                ke += De.max - De.min;
              }
              return ke;
            }
            function he(xe, Le, Qe) {
              var ke = 0.05 * Le._length, Me = Le._anchorAxis || {};
              if ((Le.ticklabelposition || "").indexOf("inside") !== -1 || (Me.ticklabelposition || "").indexOf("inside") !== -1) {
                var Re = Le.isReversed();
                if (!Re) {
                  var De = ee.simpleMap(Le.range, Le.r2l);
                  Re = De[1] < De[0];
                }
                Re && (Qe = !Qe);
              }
              var Pe = 0;
              return ue(xe, Le._id) || (Pe = function($e, qe, Xe) {
                var Ve = 0, Ze = qe._id.charAt(0) === "x";
                for (var Ge in $e._plots) {
                  var ze = $e._plots[Ge];
                  if (qe._id === ze.xaxis._id || qe._id === ze.yaxis._id) {
                    var We = (Ze ? ze.yaxis : ze.xaxis) || {};
                    if ((We.ticklabelposition || "").indexOf("inside") !== -1 && (!Xe && (We.side === "left" || We.side === "bottom") || Xe && (We.side === "top" || We.side === "right"))) {
                      if (We._vals) {
                        var at = ee.deg2rad(We._tickAngles[We._id + "tick"] || 0), ct = Math.abs(Math.cos(at)), et = Math.abs(Math.sin(at));
                        if (!We._vals[0].bb) {
                          var dt = We._id + "tick";
                          We._selections[dt].each(function(bt) {
                            var yt = te.select(this);
                            yt.select(".text-math-group").empty() && (bt.bb = se.bBox(yt.node()));
                          });
                        }
                        for (var Je = 0; Je < We._vals.length; Je++) {
                          var it = We._vals[Je].bb;
                          if (it) {
                            var st = 2 * me + it.width, gt = 2 * me + it.height;
                            Ve = Math.max(Ve, Ze ? Math.max(st * ct, gt * et) : Math.max(gt * ct, st * et));
                          }
                        }
                      }
                      We.ticks === "inside" && We.ticklabelposition === "inside" && (Ve += We.ticklen || 0);
                    }
                  }
                }
                return Ve;
              }(xe, Le, Qe)), ke = Math.max(Pe, ke), Le.constrain === "domain" && Le._inputDomain && (ke *= (Le._inputDomain[1] - Le._inputDomain[0]) / (Le.domain[1] - Le.domain[0])), function($e) {
                return $e.nopad ? 0 : $e.pad + ($e.extrapad ? ke : Pe);
              };
            }
            G.exports = { applyAutorangeOptions: Ee, getAutoRange: de, makePadFn: he, doAutoRange: function(xe, Le, Qe) {
              if (Le.setScale(), Le.autorange) {
                Le.range = Qe ? Qe.slice() : de(xe, Le), Le._r = Le.range.slice(), Le._rl = ee.simpleMap(Le._r, Le.r2l);
                var ke = Le._input, Me = {};
                Me[Le._attr + ".range"] = Le.range, Me[Le._attr + ".autorange"] = Le.autorange, oe.call("_storeDirectGUIEdit", xe.layout, xe._fullLayout._preGUI, Me), ke.range = Le.range.slice(), ke.autorange = Le.autorange;
              }
              var Re = Le._anchorAxis;
              if (Re && Re.rangeslider) {
                var De = Re.rangeslider[Le._name];
                De && De.rangemode === "auto" && (De.range = de(xe, Le)), Re._input.rangeslider[Le._name] = ee.extendFlat({}, De);
              }
            }, findExtremes: function(xe, Le, Qe) {
              Qe || (Qe = {}), xe._m || xe.setScale();
              var ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze = [], Ge = [], ze = Le.length, We = Qe.padded || !1, at = Qe.tozero && (xe.type === "linear" || xe.type === "-"), ct = xe.type === "log", et = !1, dt = Qe.vpadLinearized || !1;
              function Je(Rt) {
                if (Array.isArray(Rt)) return et = !0, function($t) {
                  return Math.max(Number(Rt[$t] || 0), 0);
                };
                var vt = Math.max(Number(Rt || 0), 0);
                return function() {
                  return vt;
                };
              }
              var it = Je((xe._m > 0 ? Qe.ppadplus : Qe.ppadminus) || Qe.ppad || 0), st = Je((xe._m > 0 ? Qe.ppadminus : Qe.ppadplus) || Qe.ppad || 0), gt = Je(Qe.vpadplus || Qe.vpad), bt = Je(Qe.vpadminus || Qe.vpad);
              if (!et) {
                if (Xe = 1 / 0, Ve = -1 / 0, ct) for (ke = 0; ke < ze; ke++) (Me = Le[ke]) < Xe && Me > 0 && (Xe = Me), Me > Ve && Me < ae && (Ve = Me);
                else for (ke = 0; ke < ze; ke++) (Me = Le[ke]) < Xe && Me > -ae && (Xe = Me), Me > Ve && Me < ae && (Ve = Me);
                Le = [Xe, Ve], ze = 2;
              }
              var yt = { tozero: at, extrapad: We };
              function St(Rt) {
                Re = Le[Rt], ne(Re) && ($e = it(Rt), qe = st(Rt), dt ? (De = xe.c2l(Re) - bt(Rt), Pe = xe.c2l(Re) + gt(Rt)) : (Xe = Re - bt(Rt), Ve = Re + gt(Rt), ct && Xe < Ve / 10 && (Xe = Ve / 10), De = xe.c2l(Xe), Pe = xe.c2l(Ve)), at && (De = Math.min(0, De), Pe = Math.max(0, Pe)), be(De) && ye(Ze, De, qe, yt), be(Pe) && ge(Ge, Pe, $e, yt));
              }
              var Lt = Math.min(6, ze);
              for (ke = 0; ke < Lt; ke++) St(ke);
              for (ke = ze - 1; ke >= Lt; ke--) St(ke);
              return { min: Ze, max: Ge, opts: Qe };
            }, concatExtremes: pe };
            var me = 3;
            function pe(xe, Le, Qe) {
              var ke, Me, Re, De = Le._id, Pe = xe._fullData, $e = xe._fullLayout, qe = [], Xe = [];
              function Ve(at, ct) {
                for (ke = 0; ke < ct.length; ke++) {
                  var et = at[ct[ke]], dt = (et._extremes || {})[De];
                  if (et.visible === !0 && dt) {
                    for (Me = 0; Me < dt.min.length; Me++) Re = dt.min[Me], ye(qe, Re.val, Re.pad, { extrapad: Re.extrapad });
                    for (Me = 0; Me < dt.max.length; Me++) Re = dt.max[Me], ge(Xe, Re.val, Re.pad, { extrapad: Re.extrapad });
                  }
                }
              }
              if (Ve(Pe, Le._traceIndices), Ve($e.annotations || [], Le._annIndices || []), Ve($e.shapes || [], Le._shapeIndices || []), Le._matchGroup && !Qe) {
                for (var Ze in Le._matchGroup) if (Ze !== Le._id) {
                  var Ge = le(xe, Ze), ze = pe(xe, Ge, !0), We = Le._length / Ge._length;
                  for (Me = 0; Me < ze.min.length; Me++) Re = ze.min[Me], ye(qe, Re.val, Re.pad * We, { extrapad: Re.extrapad });
                  for (Me = 0; Me < ze.max.length; Me++) Re = ze.max[Me], ge(Xe, Re.val, Re.pad * We, { extrapad: Re.extrapad });
                }
              }
              return { min: qe, max: Xe };
            }
            function ye(xe, Le, Qe, ke) {
              ve(xe, Le, Qe, ke, Ce);
            }
            function ge(xe, Le, Qe, ke) {
              ve(xe, Le, Qe, ke, Se);
            }
            function ve(xe, Le, Qe, ke, Me) {
              for (var Re = ke.tozero, De = ke.extrapad, Pe = !0, $e = 0; $e < xe.length && Pe; $e++) {
                var qe = xe[$e];
                if (Me(qe.val, Le) && qe.pad >= Qe && (qe.extrapad || !De)) {
                  Pe = !1;
                  break;
                }
                Me(Le, qe.val) && qe.pad <= Qe && (De || !qe.extrapad) && (xe.splice($e, 1), $e--);
              }
              if (Pe) {
                var Xe = Re && Le === 0;
                xe.push({ val: Le, pad: Xe ? 0 : Qe, extrapad: !Xe && De });
              }
            }
            function be(xe) {
              return ne(xe) && Math.abs(xe) < ae;
            }
            function Ce(xe, Le) {
              return xe <= Le;
            }
            function Se(xe, Le) {
              return xe >= Le;
            }
            function we(xe, Le, Qe) {
              return Le === void 0 || Qe === void 0 || (Le = xe.d2l(Le)) < xe.d2l(Qe);
            }
            function Ee(xe, Le) {
              if (!Le || !Le.autorangeoptions) return xe;
              var Qe = xe[0], ke = xe[1], Me = Le.autorangeoptions.include;
              if (Me !== void 0) {
                var Re = Le.d2l(Qe), De = Le.d2l(ke);
                ee.isArrayOrTypedArray(Me) || (Me = [Me]);
                for (var Pe = 0; Pe < Me.length; Pe++) {
                  var $e = Le.d2l(Me[Pe]);
                  Re >= $e && (Re = $e, Qe = $e), De <= $e && (De = $e, ke = $e);
                }
              }
              return Qe = function(qe, Xe) {
                var Ve = Xe.autorangeoptions;
                return Ve && Ve.minallowed !== void 0 && we(Xe, Ve.minallowed, Ve.maxallowed) ? Ve.minallowed : Ve && Ve.clipmin !== void 0 && we(Xe, Ve.clipmin, Ve.clipmax) ? Math.max(qe, Xe.d2l(Ve.clipmin)) : qe;
              }(Qe, Le), ke = function(qe, Xe) {
                var Ve = Xe.autorangeoptions;
                return Ve && Ve.maxallowed !== void 0 && we(Xe, Ve.minallowed, Ve.maxallowed) ? Ve.maxallowed : Ve && Ve.clipmax !== void 0 && we(Xe, Ve.clipmin, Ve.clipmax) ? Math.min(qe, Xe.d2l(Ve.clipmax)) : qe;
              }(ke, Le), [Qe, ke];
            }
          }, 75511: function(G) {
            G.exports = function(J, Z, te) {
              var ne, ee;
              if (te) {
                var ae = Z === "reversed" || Z === "min reversed" || Z === "max reversed";
                ne = te[ae ? 1 : 0], ee = te[ae ? 0 : 1];
              }
              var oe = J("autorangeoptions.minallowed", ee === null ? ne : void 0), se = J("autorangeoptions.maxallowed", ne === null ? ee : void 0);
              oe === void 0 && J("autorangeoptions.clipmin"), se === void 0 && J("autorangeoptions.clipmax"), J("autorangeoptions.include");
            };
          }, 29714: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(44122), ae = Z(33626), oe = Z(34809), se = oe.strTranslate, ce = Z(30635), le = Z(17240), ue = Z(78766), de = Z(62203), fe = Z(25829), he = Z(68599), me = Z(63821), pe = me.ONEMAXYEAR, ye = me.ONEAVGYEAR, ge = me.ONEMINYEAR, ve = me.ONEMAXQUARTER, be = me.ONEAVGQUARTER, Ce = me.ONEMINQUARTER, Se = me.ONEMAXMONTH, we = me.ONEAVGMONTH, Ee = me.ONEMINMONTH, xe = me.ONEWEEK, Le = me.ONEDAY, Qe = Le / 2, ke = me.ONEHOUR, Me = me.ONEMIN, Re = me.ONESEC, De = me.ONEMILLI, Pe = me.ONEMICROSEC, $e = me.MINUS_SIGN, qe = me.BADNUM, Xe = { K: "zeroline" }, Ve = { K: "gridline", L: "path" }, Ze = { K: "minor-gridline", L: "path" }, Ge = { K: "tick", L: "path" }, ze = { K: "tick", L: "text" }, We = { width: ["x", "r", "l", "xl", "xr"], height: ["y", "t", "b", "yt", "yb"], right: ["r", "xr"], left: ["l", "xl"], top: ["t", "yt"], bottom: ["b", "yb"] }, at = Z(4530), ct = at.MID_SHIFT, et = at.CAP_SHIFT, dt = at.LINE_SPACING, Je = at.OPPOSITE_SIDE, it = G.exports = {};
            it.setConvert = Z(19091);
            var st = Z(9666), gt = Z(5975), bt = gt.idSort, yt = gt.isLinked;
            it.id2name = gt.id2name, it.name2id = gt.name2id, it.cleanId = gt.cleanId, it.list = gt.list, it.listIds = gt.listIds, it.getFromId = gt.getFromId, it.getFromTrace = gt.getFromTrace;
            var St = Z(32919);
            it.getAutoRange = St.getAutoRange, it.findExtremes = St.findExtremes;
            var Lt = 1e-4;
            function Rt(an) {
              var Zt = (an[1] - an[0]) * Lt;
              return [an[0] - Zt, an[1] + Zt];
            }
            it.coerceRef = function(an, Zt, Mn, Ln, In, kn) {
              var er = Ln.charAt(Ln.length - 1), jn = Mn._fullLayout._subplots[er + "axis"], ur = Ln + "ref", Kn = {};
              return In || (In = jn[0] || (typeof kn == "string" ? kn : kn[0])), kn || (kn = In), jn = jn.concat(jn.map(function(Jn) {
                return Jn + " domain";
              })), Kn[ur] = { valType: "enumerated", values: jn.concat(kn ? typeof kn == "string" ? [kn] : kn : []), dflt: In }, oe.coerce(an, Zt, Kn, ur);
            }, it.getRefType = function(an) {
              return an === void 0 ? an : an === "paper" ? "paper" : an === "pixel" ? "pixel" : /( domain)$/.test(an) ? "domain" : "range";
            }, it.coercePosition = function(an, Zt, Mn, Ln, In, kn) {
              var er, jn;
              if (it.getRefType(Ln) !== "range") er = oe.ensureNumber, jn = Mn(In, kn);
              else {
                var ur = it.getFromId(Zt, Ln);
                jn = Mn(In, kn = ur.fraction2r(kn)), er = ur.cleanPos;
              }
              an[In] = er(jn);
            }, it.cleanPosition = function(an, Zt, Mn) {
              return (Mn === "paper" || Mn === "pixel" ? oe.ensureNumber : it.getFromId(Zt, Mn).cleanPos)(an);
            }, it.redrawComponents = function(an, Zt) {
              Zt = Zt || it.listIds(an);
              var Mn = an._fullLayout;
              function Ln(In, kn, er, jn) {
                for (var ur = ae.getComponentMethod(In, kn), Kn = {}, Jn = 0; Jn < Zt.length; Jn++) for (var Lr = Mn[it.id2name(Zt[Jn])][er], mr = 0; mr < Lr.length; mr++) {
                  var Ar = Lr[mr];
                  if (!Kn[Ar] && (ur(an, Ar), Kn[Ar] = 1, jn)) return;
                }
              }
              Ln("annotations", "drawOne", "_annIndices"), Ln("shapes", "drawOne", "_shapeIndices"), Ln("images", "draw", "_imgIndices", !0), Ln("selections", "drawOne", "_selectionIndices");
            };
            var vt = it.getDataConversions = function(an, Zt, Mn, Ln) {
              var In, kn = Mn === "x" || Mn === "y" || Mn === "z" ? Mn : Ln;
              if (oe.isArrayOrTypedArray(kn)) {
                if (In = { type: st(Ln, void 0, { autotypenumbers: an._fullLayout.autotypenumbers }), _categories: [] }, it.setConvert(In), In.type === "category") for (var er = 0; er < Ln.length; er++) In.d2c(Ln[er]);
              } else In = it.getFromTrace(an, Zt, kn);
              return In ? { d2c: In.d2c, c2d: In.c2d } : kn === "ids" ? { d2c: Pt, c2d: Pt } : { d2c: $t, c2d: $t };
            };
            function $t(an) {
              return +an;
            }
            function Pt(an) {
              return String(an);
            }
            function Xt(an, Zt) {
              return Math.abs((an / Zt + 0.5) % 1 - 0.5) < 1e-3;
            }
            function Nt(an, Zt) {
              return Math.abs(an / Zt - 1) < 1e-3;
            }
            function Gt(an) {
              return +an.substring(1);
            }
            function Ft(an, Zt) {
              return an.rangebreaks && (Zt = Zt.filter(function(Mn) {
                return an.maskBreaks(Mn.x) !== qe;
              })), Zt;
            }
            function Tt(an) {
              var Zt = an._mainAxis, Mn = [];
              if (Zt._vals) {
                for (var Ln = 0; Ln < Zt._vals.length; Ln++) if (!Zt._vals[Ln].noTick) {
                  var In = Zt.l2p(Zt._vals[Ln].x), kn = an.p2l(In), er = it.tickText(an, kn);
                  Zt._vals[Ln].minor && (er.minor = !0, er.text = ""), Mn.push(er);
                }
              }
              return Ft(an, Mn);
            }
            function It(an, Zt) {
              var Mn = Rt(oe.simpleMap(an.range, an.r2l)), Ln = Math.min(Mn[0], Mn[1]), In = Math.max(Mn[0], Mn[1]), kn = an.type === "category" ? an.d2l_noadd : an.d2l;
              an.type === "log" && String(an.dtick).charAt(0) !== "L" && (an.dtick = "L" + Math.pow(10, Math.floor(Math.min(an.range[0], an.range[1])) - 1));
              for (var er = [], jn = 0; jn <= 1; jn++) if ((Zt === void 0 || !(Zt && jn || Zt === !1 && !jn)) && (!jn || an.minor)) {
                var ur = jn ? an.minor.tickvals : an.tickvals, Kn = jn ? [] : an.ticktext;
                if (ur) {
                  oe.isArrayOrTypedArray(Kn) || (Kn = []);
                  for (var Jn = 0; Jn < ur.length; Jn++) {
                    var Lr = kn(ur[Jn]);
                    if (Lr > Ln && Lr < In) {
                      var mr = it.tickText(an, Lr, !1, String(Kn[Jn]));
                      jn && (mr.minor = !0, mr.text = ""), er.push(mr);
                    }
                  }
                }
              }
              return Ft(an, er);
            }
            it.getDataToCoordFunc = function(an, Zt, Mn, Ln) {
              return vt(an, Zt, Mn, Ln).d2c;
            }, it.counterLetter = function(an) {
              var Zt = an.charAt(0);
              return Zt === "x" ? "y" : Zt === "y" ? "x" : void 0;
            }, it.minDtick = function(an, Zt, Mn, Ln) {
              ["log", "category", "multicategory"].indexOf(an.type) === -1 && Ln ? an._minDtick === void 0 ? (an._minDtick = Zt, an._forceTick0 = Mn) : an._minDtick && ((an._minDtick / Zt + 1e-6) % 1 < 2e-6 && ((Mn - an._forceTick0) / Zt % 1 + 1.000001) % 1 < 2e-6 ? (an._minDtick = Zt, an._forceTick0 = Mn) : ((Zt / an._minDtick + 1e-6) % 1 > 2e-6 || ((Mn - an._forceTick0) / an._minDtick % 1 + 1.000001) % 1 > 2e-6) && (an._minDtick = 0)) : an._minDtick = 0;
            }, it.saveRangeInitial = function(an, Zt) {
              for (var Mn = it.list(an, "", !0), Ln = !1, In = 0; In < Mn.length; In++) {
                var kn = Mn[In], er = kn._rangeInitial0 === void 0 && kn._rangeInitial1 === void 0, jn = er || kn.range[0] !== kn._rangeInitial0 || kn.range[1] !== kn._rangeInitial1, ur = kn.autorange;
                (er && ur !== !0 || Zt && jn) && (kn._rangeInitial0 = ur === "min" || ur === "max reversed" ? void 0 : kn.range[0], kn._rangeInitial1 = ur === "max" || ur === "min reversed" ? void 0 : kn.range[1], kn._autorangeInitial = ur, Ln = !0);
              }
              return Ln;
            }, it.saveShowSpikeInitial = function(an, Zt) {
              for (var Mn = it.list(an, "", !0), Ln = !1, In = "on", kn = 0; kn < Mn.length; kn++) {
                var er = Mn[kn], jn = er._showSpikeInitial === void 0, ur = jn || er.showspikes !== er._showspikes;
                (jn || Zt && ur) && (er._showSpikeInitial = er.showspikes, Ln = !0), In !== "on" || er.showspikes || (In = "off");
              }
              return an._fullLayout._cartesianSpikesEnabled = In, Ln;
            }, it.autoBin = function(an, Zt, Mn, Ln, In, kn) {
              var er, jn = oe.aggNums(Math.min, null, an), ur = oe.aggNums(Math.max, null, an);
              if (Zt.type === "category" || Zt.type === "multicategory") return { start: jn - 0.5, end: ur + 0.5, size: Math.max(1, Math.round(kn) || 1), _dataSpan: ur - jn };
              if (In || (In = Zt.calendar), er = Zt.type === "log" ? { type: "linear", range: [jn, ur] } : { type: Zt.type, range: oe.simpleMap([jn, ur], Zt.c2r, 0, In), calendar: In }, it.setConvert(er), kn = kn && he.dtick(kn, er.type)) er.dtick = kn, er.tick0 = he.tick0(void 0, er.type, In);
              else {
                var Kn;
                if (Mn) Kn = (ur - jn) / Mn;
                else {
                  var Jn = oe.distinctVals(an), Lr = Math.pow(10, Math.floor(Math.log(Jn.minDiff) / Math.LN10)), mr = Lr * oe.roundUp(Jn.minDiff / Lr, [0.9, 1.9, 4.9, 9.9], !0);
                  Kn = Math.max(mr, 2 * oe.stdev(an) / Math.pow(an.length, Ln ? 0.25 : 0.4)), ne(Kn) || (Kn = 1);
                }
                it.autoTicks(er, Kn);
              }
              var Ar, Yr = er.dtick, jr = it.tickIncrement(it.tickFirst(er), Yr, "reverse", In);
              if (typeof Yr == "number") jr = function(Mr, Wr, Bi, gi, Zr) {
                var fi = 0, ri = 0, bi = 0, Zi = 0;
                function $i(Ci) {
                  return (1 + 100 * (Ci - Mr) / Bi.dtick) % 100 < 2;
                }
                for (var Br = 0; Br < Wr.length; Br++) Wr[Br] % 1 == 0 ? bi++ : ne(Wr[Br]) || Zi++, $i(Wr[Br]) && fi++, $i(Wr[Br] + Bi.dtick / 2) && ri++;
                var Pr = Wr.length - Zi;
                if (bi === Pr && Bi.type !== "date") Bi.dtick < 1 ? Mr = gi - 0.5 * Bi.dtick : (Mr -= 0.5) + Bi.dtick < gi && (Mr += Bi.dtick);
                else if (ri < 0.1 * Pr && (fi > 0.3 * Pr || $i(gi) || $i(Zr))) {
                  var ei = Bi.dtick / 2;
                  Mr += Mr + ei < gi ? ei : -ei;
                }
                return Mr;
              }(jr, an, er, jn, ur), Ar = jr + (1 + Math.floor((ur - jr) / Yr)) * Yr;
              else for (er.dtick.charAt(0) === "M" && (jr = function(Mr, Wr, Bi, gi, Zr) {
                var fi = oe.findExactDates(Wr, Zr);
                if (fi.exactDays > 0.8) {
                  var ri = Number(Bi.substr(1));
                  fi.exactYears > 0.8 && ri % 12 == 0 ? Mr = it.tickIncrement(Mr, "M6", "reverse") + 1.5 * Le : fi.exactMonths > 0.8 ? Mr = it.tickIncrement(Mr, "M1", "reverse") + 15.5 * Le : Mr -= Qe;
                  var bi = it.tickIncrement(Mr, Bi);
                  if (bi <= gi) return bi;
                }
                return Mr;
              }(jr, an, Yr, jn, In)), Ar = jr; Ar <= ur; ) Ar = it.tickIncrement(Ar, Yr, !1, In);
              return { start: Zt.c2r(jr, 0, In), end: Zt.c2r(Ar, 0, In), size: Yr, _dataSpan: ur - jn };
            }, it.prepMinorTicks = function(an, Zt, Mn) {
              if (!Zt.minor.dtick) {
                delete an.dtick;
                var Ln, In = Zt.dtick && ne(Zt._tmin);
                if (In) {
                  var kn = it.tickIncrement(Zt._tmin, Zt.dtick, !0);
                  Ln = [Zt._tmin, 0.99 * kn + 0.01 * Zt._tmin];
                } else {
                  var er = oe.simpleMap(Zt.range, Zt.r2l);
                  Ln = [er[0], 0.8 * er[0] + 0.2 * er[1]];
                }
                if (an.range = oe.simpleMap(Ln, Zt.l2r), an._isMinor = !0, it.prepTicks(an, Mn), In) {
                  var jn = ne(Zt.dtick), ur = ne(an.dtick), Kn = jn ? Zt.dtick : +Zt.dtick.substring(1), Jn = ur ? an.dtick : +an.dtick.substring(1);
                  jn && ur ? Xt(Kn, Jn) ? Kn === 2 * xe && Jn === 2 * Le && (an.dtick = xe) : Kn === 2 * xe && Jn === 3 * Le ? an.dtick = xe : Kn !== xe || (Zt._input.minor || {}).nticks ? Nt(Kn / Jn, 2.5) ? an.dtick = Kn / 2 : an.dtick = Kn : an.dtick = Le : String(Zt.dtick).charAt(0) === "M" ? ur ? an.dtick = "M1" : Xt(Kn, Jn) ? Kn >= 12 && Jn === 2 && (an.dtick = "M3") : an.dtick = Zt.dtick : String(an.dtick).charAt(0) === "L" ? String(Zt.dtick).charAt(0) === "L" ? Xt(Kn, Jn) || (an.dtick = Nt(Kn / Jn, 2.5) ? Zt.dtick / 2 : Zt.dtick) : an.dtick = "D1" : an.dtick === "D2" && +Zt.dtick > 1 && (an.dtick = 1);
                }
                an.range = Zt.range;
              }
              Zt.minor._tick0Init === void 0 && (an.tick0 = Zt.tick0);
            }, it.prepTicks = function(an, Zt) {
              var Mn = oe.simpleMap(an.range, an.r2l, void 0, void 0, Zt);
              if (an.tickmode === "auto" || !an.dtick) {
                var Ln, In = an.nticks;
                In || (an.type === "category" || an.type === "multicategory" ? (Ln = an.tickfont ? oe.bigFont(an.tickfont.size || 12) : 15, In = an._length / Ln) : (Ln = an._id.charAt(0) === "y" ? 40 : 80, In = oe.constrain(an._length / Ln, 4, 9) + 1), an._name === "radialaxis" && (In *= 2)), an.minor && an.minor.tickmode !== "array" || an.tickmode === "array" && (In *= 100), an._roughDTick = Math.abs(Mn[1] - Mn[0]) / In, it.autoTicks(an, an._roughDTick), an._minDtick > 0 && an.dtick < 2 * an._minDtick && (an.dtick = an._minDtick, an.tick0 = an.l2r(an._forceTick0));
              }
              an.ticklabelmode === "period" && function(kn) {
                var er;
                function jn() {
                  return !(ne(kn.dtick) || kn.dtick.charAt(0) !== "M");
                }
                var ur = jn(), Kn = it.getTickFormat(kn);
                if (Kn) {
                  var Jn = kn._dtickInit !== kn.dtick;
                  /%[fLQsSMX]/.test(Kn) || (/%[HI]/.test(Kn) ? (er = ke, Jn && !ur && kn.dtick < ke && (kn.dtick = ke)) : /%p/.test(Kn) ? (er = Qe, Jn && !ur && kn.dtick < Qe && (kn.dtick = Qe)) : /%[Aadejuwx]/.test(Kn) ? (er = Le, Jn && !ur && kn.dtick < Le && (kn.dtick = Le)) : /%[UVW]/.test(Kn) ? (er = xe, Jn && !ur && kn.dtick < xe && (kn.dtick = xe)) : /%[Bbm]/.test(Kn) ? (er = we, Jn && (ur ? Gt(kn.dtick) < 1 : kn.dtick < Ee) && (kn.dtick = "M1")) : /%[q]/.test(Kn) ? (er = be, Jn && (ur ? Gt(kn.dtick) < 3 : kn.dtick < Ce) && (kn.dtick = "M3")) : /%[Yy]/.test(Kn) && (er = ye, Jn && (ur ? Gt(kn.dtick) < 12 : kn.dtick < ge) && (kn.dtick = "M12")));
                }
                (ur = jn()) && kn.tick0 === kn._dowTick0 && (kn.tick0 = kn._rawTick0), kn._definedDelta = er;
              }(an), an.tick0 || (an.tick0 = an.type === "date" ? "2000-01-01" : 0), an.type === "date" && an.dtick < 0.1 && (an.dtick = 0.1), Wt(an);
            }, it.calcTicks = function(an, Zt) {
              for (var Mn, Ln, In, kn, er = an.type, jn = an.calendar, ur = an.ticklabelstep, Kn = an.ticklabelmode === "period", Jn = an.range[0] > an.range[1], Lr = !an.ticklabelindex || oe.isArrayOrTypedArray(an.ticklabelindex) ? an.ticklabelindex : [an.ticklabelindex], mr = oe.simpleMap(an.range, an.r2l, void 0, void 0, Zt), Ar = mr[1] < mr[0], Yr = Math.min(mr[0], mr[1]), jr = Math.max(mr[0], mr[1]), Mr = Math.max(1e3, an._length || 0), Wr = [], Bi = [], gi = [], Zr = [], fi = [], ri = an.minor && (an.minor.ticks || an.minor.showgrid), bi = 1; bi >= (ri ? 0 : 1); bi--) {
                var Zi = !bi;
                bi ? (an._dtickInit = an.dtick, an._tick0Init = an.tick0) : (an.minor._dtickInit = an.minor.dtick, an.minor._tick0Init = an.minor.tick0);
                var $i = bi ? an : oe.extendFlat({}, an, an.minor);
                if (Zi ? it.prepMinorTicks($i, an, Zt) : it.prepTicks($i, Zt), $i.tickmode !== "array") if ($i.tickmode !== "sync") {
                  var Br = Rt(mr), Pr = Br[0], ei = Br[1], Ci = ne($i.dtick), oa = er === "log" && !(Ci || $i.dtick.charAt(0) === "L"), ka = it.tickFirst($i, Zt);
                  if (bi) {
                    if (an._tmin = ka, ka < Pr !== Ar) break;
                    er !== "category" && er !== "multicategory" || (ei = Ar ? Math.max(-0.5, ei) : Math.min(an._categories.length - 0.5, ei));
                  }
                  var aa, hi, Gr = null, Ca = ka;
                  bi && (Ci ? hi = an.dtick : er === "date" ? typeof an.dtick == "string" && an.dtick.charAt(0) === "M" && (hi = we * an.dtick.substring(1)) : hi = an._roughDTick, aa = Math.round((an.r2l(Ca) - an.r2l(an.tick0)) / hi) - 1);
                  var Sr = $i.dtick;
                  for ($i.rangebreaks && $i._tick0Init !== $i.tick0 && (Ca = yn(Ca, an), Ar || (Ca = it.tickIncrement(Ca, Sr, !Ar, jn))), bi && Kn && (Ca = it.tickIncrement(Ca, Sr, !Ar, jn), aa--); Ar ? Ca >= ei : Ca <= ei; Ca = it.tickIncrement(Ca, Sr, Ar, jn)) {
                    if (bi && aa++, $i.rangebreaks && !Ar) {
                      if (Ca < Pr) continue;
                      if ($i.maskBreaks(Ca) === qe && yn(Ca, $i) >= jr) break;
                    }
                    if (gi.length > Mr || Ca === Gr) break;
                    Gr = Ca;
                    var Dr = { value: Ca };
                    bi ? (oa && Ca !== (0 | Ca) && (Dr.simpleLabel = !0), ur > 1 && aa % ur && (Dr.skipLabel = !0), gi.push(Dr)) : (Dr.minor = !0, Zr.push(Dr));
                  }
                } else gi = [], Wr = Tt(an);
                else bi ? (gi = [], Wr = It(an, !Zi)) : (Zr = [], Bi = It(an, !Zi));
              }
              if (!Zr || Zr.length < 2 ? Lr = !1 : (Mn = (Zr[1].value - Zr[0].value) * (Jn ? -1 : 1), Ln = an.tickformat, (/%f/.test(Ln) ? Mn >= Pe : /%L/.test(Ln) ? Mn >= De : /%[SX]/.test(Ln) ? Mn >= Re : /%M/.test(Ln) ? Mn >= Me : /%[HI]/.test(Ln) ? Mn >= ke : /%p/.test(Ln) ? Mn >= Qe : /%[Aadejuwx]/.test(Ln) ? Mn >= Le : /%[UVW]/.test(Ln) ? Mn >= xe : /%[Bbm]/.test(Ln) ? Mn >= Ee : /%[q]/.test(Ln) ? Mn >= Ce : !/%[Yy]/.test(Ln) || Mn >= ge) || (Lr = !1)), Lr) {
                var si = gi.concat(Zr);
                Kn && gi.length && (si = si.slice(1)), (si = si.sort(function(Er, gr) {
                  return Er.value - gr.value;
                }).filter(function(Er, gr, $r) {
                  return gr === 0 || Er.value !== $r[gr - 1].value;
                })).map(function(Er, gr) {
                  return Er.minor !== void 0 || Er.skipLabel ? null : gr;
                }).filter(function(Er) {
                  return Er !== null;
                }).forEach(function(Er) {
                  Lr.map(function(gr) {
                    var $r = Er + gr;
                    $r >= 0 && $r < si.length && oe.pushUnique(fi, si[$r]);
                  });
                });
              } else fi = gi;
              if (ri && !(an.minor.ticks === "inside" && an.ticks === "outside" || an.minor.ticks === "outside" && an.ticks === "inside")) {
                for (var zr = gi.map(function(Er) {
                  return Er.value;
                }), di = [], Da = 0; Da < Zr.length; Da++) {
                  var la = Zr[Da], wa = la.value;
                  if (zr.indexOf(wa) === -1) {
                    for (var Wa = !1, Qr = 0; !Wa && Qr < gi.length; Qr++) 1e7 + gi[Qr].value === 1e7 + wa && (Wa = !0);
                    Wa || di.push(la);
                  }
                }
                Zr = di;
              }
              if (Kn && function(Er, gr, $r) {
                for (var li = 0; li < Er.length; li++) {
                  var Qi = Er[li].value, Ai = li, Li = li + 1;
                  li < Er.length - 1 ? (Ai = li, Li = li + 1) : li > 0 ? (Ai = li - 1, Li = li) : (Ai = li, Li = li);
                  var Ni, Pi = Er[Ai].value, Oi = Er[Li].value, ki = Math.abs(Oi - Pi), Mi = $r || ki, Vi = 0;
                  Mi >= ge ? Vi = ki >= ge && ki <= pe ? ki : ye : $r === be && Mi >= Ce ? Vi = ki >= Ce && ki <= ve ? ki : be : Mi >= Ee ? Vi = ki >= Ee && ki <= Se ? ki : we : $r === xe && Mi >= xe ? Vi = xe : Mi >= Le ? Vi = Le : $r === Qe && Mi >= Qe ? Vi = Qe : $r === ke && Mi >= ke && (Vi = ke), Vi >= ki && (Vi = ki, Ni = !0);
                  var ua = Qi + Vi;
                  if (gr.rangebreaks && Vi > 0) {
                    for (var Ki = 0, Fi = 0; Fi < 84; Fi++) {
                      var na = (Fi + 0.5) / 84;
                      gr.maskBreaks(Qi * (1 - na) + na * ua) !== qe && Ki++;
                    }
                    (Vi *= Ki / 84) || (Er[li].drop = !0), Ni && ki > xe && (Vi = ki);
                  }
                  (Vi > 0 || li === 0) && (Er[li].periodX = Qi + Vi / 2);
                }
              }(fi, an, an._definedDelta), an.rangebreaks) {
                var Ur = an._id.charAt(0) === "y", rr = 1;
                an.tickmode === "auto" && (rr = an.tickfont ? an.tickfont.size : 12);
                var vr = NaN;
                for (In = gi.length - 1; In > -1; In--) if (gi[In].drop) gi.splice(In, 1);
                else {
                  gi[In].value = yn(gi[In].value, an);
                  var Rr = an.c2p(gi[In].value);
                  (Ur ? vr > Rr - rr : vr < Rr + rr) ? gi.splice(Ar ? In + 1 : In, 1) : vr = Rr;
                }
              }
              un(an) && Math.abs(mr[1] - mr[0]) === 360 && gi.pop(), an._tmax = (gi[gi.length - 1] || {}).value, an._prevDateHead = "", an._inCalcTicks = !0;
              var Vr, Xr = function(Er) {
                Er.text = "", an._prevDateHead = kn;
              };
              function Jr(Er, gr) {
                var $r = it.tickText(Er, gr.value, !1, gr.simpleLabel), li = gr.periodX;
                return li !== void 0 && ($r.periodX = li, (li > jr || li < Yr) && (li > jr && ($r.periodX = jr), li < Yr && ($r.periodX = Yr), Xr($r))), $r;
              }
              for (gi = gi.concat(Zr), In = 0; In < gi.length; In++) {
                var ti = gi[In].minor, Hr = gi[In].value;
                ti ? ((Vr = Lr && fi.indexOf(gi[In]) !== -1 ? Jr(an, gi[In]) : { x: Hr }).minor = !0, Bi.push(Vr)) : (kn = an._prevDateHead, Vr = Jr(an, gi[In]), (gi[In].skipLabel || Lr && fi.indexOf(gi[In]) === -1) && Xr(Vr), Wr.push(Vr));
              }
              return Wr = Wr.concat(Bi), an._inCalcTicks = !1, Kn && Wr.length && (Wr[0].noTick = !0), Wr;
            };
            var Ct = [2, 5, 10], Kt = [1, 2, 3, 6, 12], Vt = [1, 2, 5, 10, 15, 30], Yt = [1, 2, 3, 7, 14], sn = [-0.046, 0, 0.301, 0.477, 0.602, 0.699, 0.778, 0.845, 0.903, 0.954, 1], hn = [-0.301, 0, 0.301, 0.699, 1], Qt = [15, 30, 45, 90, 180];
            function Ut(an, Zt, Mn) {
              return Zt * oe.roundUp(an / Zt, Mn);
            }
            function Wt(an) {
              var Zt = an.dtick;
              if (an._tickexponent = 0, ne(Zt) || typeof Zt == "string" || (Zt = 1), an.type !== "category" && an.type !== "multicategory" || (an._tickround = null), an.type === "date") {
                var Mn = an.r2l(an.tick0), Ln = an.l2r(Mn).replace(/(^-|i)/g, ""), In = Ln.length;
                if (String(Zt).charAt(0) === "M") In > 10 || Ln.substr(5) !== "01-01" ? an._tickround = "d" : an._tickround = +Zt.substr(1) % 12 == 0 ? "y" : "m";
                else if (Zt >= Le && In <= 10 || Zt >= 15 * Le) an._tickround = "d";
                else if (Zt >= Me && In <= 16 || Zt >= ke) an._tickround = "M";
                else if (Zt >= Re && In <= 19 || Zt >= Me) an._tickround = "S";
                else {
                  var kn = an.l2r(Mn + Zt).replace(/^-/, "").length;
                  an._tickround = Math.max(In, kn) - 20, an._tickround < 0 && (an._tickround = 4);
                }
              } else if (ne(Zt) || Zt.charAt(0) === "L") {
                var er = an.range.map(an.r2d || Number);
                ne(Zt) || (Zt = Number(Zt.substr(1))), an._tickround = 2 - Math.floor(Math.log(Zt) / Math.LN10 + 0.01);
                var jn = Math.max(Math.abs(er[0]), Math.abs(er[1])), ur = Math.floor(Math.log(jn) / Math.LN10 + 0.01), Kn = an.minexponent === void 0 ? 3 : an.minexponent;
                Math.abs(ur) > Kn && (dn(an.exponentformat) && !Ht(ur) ? an._tickexponent = 3 * Math.round((ur - 1) / 3) : an._tickexponent = ur);
              } else an._tickround = null;
            }
            function rn(an, Zt, Mn) {
              var Ln = an.tickfont || {};
              return { x: Zt, dx: 0, dy: 0, text: Mn || "", fontSize: Ln.size, font: Ln.family, fontWeight: Ln.weight, fontStyle: Ln.style, fontVariant: Ln.variant, fontTextcase: Ln.textcase, fontLineposition: Ln.lineposition, fontShadow: Ln.shadow, fontColor: Ln.color };
            }
            it.autoTicks = function(an, Zt, Mn) {
              var Ln;
              function In(Lr) {
                return Math.pow(Lr, Math.floor(Math.log(Zt) / Math.LN10));
              }
              if (an.type === "date") {
                an.tick0 = oe.dateTick0(an.calendar, 0);
                var kn = 2 * Zt;
                if (kn > ye) Zt /= ye, Ln = In(10), an.dtick = "M" + 12 * Ut(Zt, Ln, Ct);
                else if (kn > we) Zt /= we, an.dtick = "M" + Ut(Zt, 1, Kt);
                else if (kn > Le) {
                  if (an.dtick = Ut(Zt, Le, an._hasDayOfWeekBreaks ? [1, 2, 7, 14] : Yt), !Mn) {
                    var er = it.getTickFormat(an), jn = an.ticklabelmode === "period";
                    jn && (an._rawTick0 = an.tick0), /%[uVW]/.test(er) ? an.tick0 = oe.dateTick0(an.calendar, 2) : an.tick0 = oe.dateTick0(an.calendar, 1), jn && (an._dowTick0 = an.tick0);
                  }
                } else kn > ke ? an.dtick = Ut(Zt, ke, Kt) : kn > Me ? an.dtick = Ut(Zt, Me, Vt) : kn > Re ? an.dtick = Ut(Zt, Re, Vt) : (Ln = In(10), an.dtick = Ut(Zt, Ln, Ct));
              } else if (an.type === "log") {
                an.tick0 = 0;
                var ur = oe.simpleMap(an.range, an.r2l);
                if (an._isMinor && (Zt *= 1.5), Zt > 0.7) an.dtick = Math.ceil(Zt);
                else if (Math.abs(ur[1] - ur[0]) < 1) {
                  var Kn = 1.5 * Math.abs((ur[1] - ur[0]) / Zt);
                  Zt = Math.abs(Math.pow(10, ur[1]) - Math.pow(10, ur[0])) / Kn, Ln = In(10), an.dtick = "L" + Ut(Zt, Ln, Ct);
                } else an.dtick = Zt > 0.3 ? "D2" : "D1";
              } else an.type === "category" || an.type === "multicategory" ? (an.tick0 = 0, an.dtick = Math.ceil(Math.max(Zt, 1))) : un(an) ? (an.tick0 = 0, Ln = 1, an.dtick = Ut(Zt, Ln, Qt)) : (an.tick0 = 0, Ln = In(10), an.dtick = Ut(Zt, Ln, Ct));
              if (an.dtick === 0 && (an.dtick = 1), !ne(an.dtick) && typeof an.dtick != "string") {
                var Jn = an.dtick;
                throw an.dtick = 1, "ax.dtick error: " + String(Jn);
              }
            }, it.tickIncrement = function(an, Zt, Mn, Ln) {
              var In = Mn ? -1 : 1;
              if (ne(Zt)) return oe.increment(an, In * Zt);
              var kn = Zt.charAt(0), er = In * Number(Zt.substr(1));
              if (kn === "M") return oe.incrementMonth(an, er, Ln);
              if (kn === "L") return Math.log(Math.pow(10, an) + er) / Math.LN10;
              if (kn === "D") {
                var jn = Zt === "D2" ? hn : sn, ur = an + 0.01 * In, Kn = oe.roundUp(oe.mod(ur, 1), jn, Mn);
                return Math.floor(ur) + Math.log(te.round(Math.pow(10, Kn), 1)) / Math.LN10;
              }
              throw "unrecognized dtick " + String(Zt);
            }, it.tickFirst = function(an, Zt) {
              var Mn = an.r2l || Number, Ln = oe.simpleMap(an.range, Mn, void 0, void 0, Zt), In = Ln[1] < Ln[0], kn = In ? Math.floor : Math.ceil, er = Rt(Ln)[0], jn = an.dtick, ur = Mn(an.tick0);
              if (ne(jn)) {
                var Kn = kn((er - ur) / jn) * jn + ur;
                return an.type !== "category" && an.type !== "multicategory" || (Kn = oe.constrain(Kn, 0, an._categories.length - 1)), Kn;
              }
              var Jn = jn.charAt(0), Lr = Number(jn.substr(1));
              if (Jn === "M") {
                for (var mr, Ar, Yr, jr = 0, Mr = ur; jr < 10; ) {
                  if (((mr = it.tickIncrement(Mr, jn, In, an.calendar)) - er) * (Mr - er) <= 0) return In ? Math.min(Mr, mr) : Math.max(Mr, mr);
                  Ar = (er - (Mr + mr) / 2) / (mr - Mr), Yr = Jn + (Math.abs(Math.round(Ar)) || 1) * Lr, Mr = it.tickIncrement(Mr, Yr, Ar < 0 ? !In : In, an.calendar), jr++;
                }
                return oe.error("tickFirst did not converge", an), Mr;
              }
              if (Jn === "L") return Math.log(kn((Math.pow(10, er) - ur) / Lr) * Lr + ur) / Math.LN10;
              if (Jn === "D") {
                var Wr = jn === "D2" ? hn : sn, Bi = oe.roundUp(oe.mod(er, 1), Wr, In);
                return Math.floor(er) + Math.log(te.round(Math.pow(10, Bi), 1)) / Math.LN10;
              }
              throw "unrecognized dtick " + String(jn);
            }, it.tickText = function(an, Zt, Mn, Ln) {
              var In, kn = rn(an, Zt), er = an.tickmode === "array", jn = Mn || er, ur = an.type, Kn = ur === "category" ? an.d2l_noadd : an.d2l, Jn = function(Mr) {
                var Wr = an.l2p(Mr);
                return Wr >= 0 && Wr <= an._length ? Mr : null;
              };
              if (er && oe.isArrayOrTypedArray(an.ticktext)) {
                var Lr = oe.simpleMap(an.range, an.r2l), mr = (Math.abs(Lr[1] - Lr[0]) - (an._lBreaks || 0)) / 1e4;
                for (In = 0; In < an.ticktext.length && !(Math.abs(Zt - Kn(an.tickvals[In])) < mr); In++) ;
                if (In < an.ticktext.length) return kn.text = String(an.ticktext[In]), kn.xbnd = [Jn(kn.x - 0.5), Jn(kn.x + an.dtick - 0.5)], kn;
              }
              function Ar(Mr) {
                if (Mr === void 0) return !0;
                if (Mn) return Mr === "none";
                var Wr = { first: an._tmin, last: an._tmax }[Mr];
                return Mr !== "all" && Zt !== Wr;
              }
              var Yr = Mn ? "never" : an.exponentformat !== "none" && Ar(an.showexponent) ? "hide" : "";
              if (ur === "date" ? function(Mr, Wr, Bi, gi) {
                var Zr = Mr._tickround, fi = Bi && Mr.hoverformat || it.getTickFormat(Mr);
                (gi = !fi && gi) && (Zr = ne(Zr) ? 4 : { y: "m", m: "d", d: "M", M: "S", S: 4 }[Zr]);
                var ri, bi = oe.formatDate(Wr.x, fi, Zr, Mr._dateFormat, Mr.calendar, Mr._extraFormat), Zi = bi.indexOf(`
`);
                if (Zi !== -1 && (ri = bi.substr(Zi + 1), bi = bi.substr(0, Zi)), gi && (ri === void 0 || bi !== "00:00:00" && bi !== "00:00" ? bi.length === 8 && (bi = bi.replace(/:00$/, "")) : (bi = ri, ri = "")), ri) if (Bi) Zr === "d" ? bi += ", " + ri : bi = ri + (bi ? ", " + bi : "");
                else if (Mr._inCalcTicks && Mr._prevDateHead === ri) {
                  var $i = _n(Mr), Br = Mr._trueSide || Mr.side;
                  (!$i && Br === "top" || $i && Br === "bottom") && (bi += "<br> ");
                } else Mr._prevDateHead = ri, bi += "<br>" + ri;
                Wr.text = bi;
              }(an, kn, Mn, jn) : ur === "log" ? function(Mr, Wr, Bi, gi, Zr) {
                var fi = Mr.dtick, ri = Wr.x, bi = Mr.tickformat, Zi = typeof fi == "string" && fi.charAt(0);
                if (Zr === "never" && (Zr = ""), gi && Zi !== "L" && (fi = "L3", Zi = "L"), bi || Zi === "L") Wr.text = jt(Math.pow(10, ri), Mr, Zr, gi);
                else if (ne(fi) || Zi === "D" && oe.mod(ri + 0.01, 1) < 0.1) {
                  var $i = Math.round(ri), Br = Math.abs($i), Pr = Mr.exponentformat;
                  Pr === "power" || dn(Pr) && Ht($i) ? (Wr.text = $i === 0 ? 1 : $i === 1 ? "10" : "10<sup>" + ($i > 1 ? "" : $e) + Br + "</sup>", Wr.fontSize *= 1.25) : (Pr === "e" || Pr === "E") && Br > 2 ? Wr.text = "1" + Pr + ($i > 0 ? "+" : $e) + Br : (Wr.text = jt(Math.pow(10, ri), Mr, "", "fakehover"), fi === "D1" && Mr._id.charAt(0) === "y" && (Wr.dy -= Wr.fontSize / 6));
                } else {
                  if (Zi !== "D") throw "unrecognized dtick " + String(fi);
                  Wr.text = String(Math.round(Math.pow(10, oe.mod(ri, 1)))), Wr.fontSize *= 0.75;
                }
                if (Mr.dtick === "D1") {
                  var ei = String(Wr.text).charAt(0);
                  ei !== "0" && ei !== "1" || (Mr._id.charAt(0) === "y" ? Wr.dx -= Wr.fontSize / 4 : (Wr.dy += Wr.fontSize / 2, Wr.dx += (Mr.range[1] > Mr.range[0] ? 1 : -1) * Wr.fontSize * (ri < 0 ? 0.5 : 0.25)));
                }
              }(an, kn, 0, jn, Yr) : ur === "category" ? function(Mr, Wr) {
                var Bi = Mr._categories[Math.round(Wr.x)];
                Bi === void 0 && (Bi = ""), Wr.text = String(Bi);
              }(an, kn) : ur === "multicategory" ? function(Mr, Wr, Bi) {
                var gi = Math.round(Wr.x), Zr = Mr._categories[gi] || [], fi = Zr[1] === void 0 ? "" : String(Zr[1]), ri = Zr[0] === void 0 ? "" : String(Zr[0]);
                Bi ? Wr.text = ri + " - " + fi : (Wr.text = fi, Wr.text2 = ri);
              }(an, kn, Mn) : un(an) ? function(Mr, Wr, Bi, gi, Zr) {
                if (Mr.thetaunit !== "radians" || Bi) Wr.text = jt(Wr.x, Mr, Zr, gi);
                else {
                  var fi = Wr.x / 180;
                  if (fi === 0) Wr.text = "0";
                  else {
                    var ri = function(Zi) {
                      function $i(Ci, oa) {
                        return Math.abs(Ci - oa) <= 1e-6;
                      }
                      var Br = function(Ci) {
                        for (var oa = 1; !$i(Math.round(Ci * oa) / oa, Ci); ) oa *= 10;
                        return oa;
                      }(Zi), Pr = Zi * Br, ei = Math.abs(function Ci(oa, ka) {
                        return $i(ka, 0) ? oa : Ci(ka, oa % ka);
                      }(Pr, Br));
                      return [Math.round(Pr / ei), Math.round(Br / ei)];
                    }(fi);
                    if (ri[1] >= 100) Wr.text = jt(oe.deg2rad(Wr.x), Mr, Zr, gi);
                    else {
                      var bi = Wr.x < 0;
                      ri[1] === 1 ? ri[0] === 1 ? Wr.text = "π" : Wr.text = ri[0] + "π" : Wr.text = ["<sup>", ri[0], "</sup>", "⁄", "<sub>", ri[1], "</sub>", "π"].join(""), bi && (Wr.text = $e + Wr.text);
                    }
                  }
                }
              }(an, kn, Mn, jn, Yr) : function(Mr, Wr, Bi, gi, Zr) {
                Zr === "never" ? Zr = "" : Mr.showexponent === "all" && Math.abs(Wr.x / Mr.dtick) < 1e-6 && (Zr = "hide"), Wr.text = jt(Wr.x, Mr, Zr, gi);
              }(an, kn, 0, jn, Yr), Ln || (an.tickprefix && !Ar(an.showtickprefix) && (kn.text = an.tickprefix + kn.text), an.ticksuffix && !Ar(an.showticksuffix) && (kn.text += an.ticksuffix)), an.labelalias && an.labelalias.hasOwnProperty(kn.text)) {
                var jr = an.labelalias[kn.text];
                typeof jr == "string" && (kn.text = jr);
              }
              return (an.tickson === "boundaries" || an.showdividers) && (kn.xbnd = [Jn(kn.x - 0.5), Jn(kn.x + an.dtick - 0.5)]), kn;
            }, it.hoverLabelText = function(an, Zt, Mn) {
              Mn && (an = oe.extendFlat({}, an, { hoverformat: Mn }));
              var Ln = oe.isArrayOrTypedArray(Zt) ? Zt[0] : Zt, In = oe.isArrayOrTypedArray(Zt) ? Zt[1] : void 0;
              if (In !== void 0 && In !== Ln) return it.hoverLabelText(an, Ln, Mn) + " - " + it.hoverLabelText(an, In, Mn);
              var kn = an.type === "log" && Ln <= 0, er = it.tickText(an, an.c2l(kn ? -Ln : Ln), "hover").text;
              return kn ? Ln === 0 ? "0" : $e + er : er;
            };
            var vn = ["f", "p", "n", "μ", "m", "", "k", "M", "G", "T"];
            function dn(an) {
              return an === "SI" || an === "B";
            }
            function Ht(an) {
              return an > 14 || an < -15;
            }
            function jt(an, Zt, Mn, Ln) {
              var In = an < 0, kn = Zt._tickround, er = Mn || Zt.exponentformat || "B", jn = Zt._tickexponent, ur = it.getTickFormat(Zt), Kn = Zt.separatethousands;
              if (Ln) {
                var Jn = { exponentformat: er, minexponent: Zt.minexponent, dtick: Zt.showexponent === "none" ? Zt.dtick : ne(an) && Math.abs(an) || 1, range: Zt.showexponent === "none" ? Zt.range.map(Zt.r2d) : [0, an || 1] };
                Wt(Jn), kn = (Number(Jn._tickround) || 0) + 4, jn = Jn._tickexponent, Zt.hoverformat && (ur = Zt.hoverformat);
              }
              if (ur) return Zt._numFormat(ur)(an).replace(/-/g, $e);
              var Lr, mr = Math.pow(10, -kn) / 2;
              if (er === "none" && (jn = 0), (an = Math.abs(an)) < mr) an = "0", In = !1;
              else {
                if (an += mr, jn && (an *= Math.pow(10, -jn), kn += jn), kn === 0) an = String(Math.floor(an));
                else if (kn < 0) {
                  an = (an = String(Math.round(an))).substr(0, an.length + kn);
                  for (var Ar = kn; Ar < 0; Ar++) an += "0";
                } else {
                  var Yr = (an = String(an)).indexOf(".") + 1;
                  Yr && (an = an.substr(0, Yr + kn).replace(/\.?0+$/, ""));
                }
                an = oe.numSeparate(an, Zt._separators, Kn);
              }
              return jn && er !== "hide" && (dn(er) && Ht(jn) && (er = "power"), Lr = jn < 0 ? $e + -jn : er !== "power" ? "+" + jn : String(jn), er === "e" || er === "E" ? an += er + Lr : er === "power" ? an += "×10<sup>" + Lr + "</sup>" : er === "B" && jn === 9 ? an += "B" : dn(er) && (an += vn[jn / 3 + 5])), In ? $e + an : an;
            }
            function cn(an, Zt) {
              if (an) {
                var Mn = Object.keys(We).reduce(function(Ln, In) {
                  return Zt.indexOf(In) !== -1 && We[In].forEach(function(kn) {
                    Ln[kn] = 1;
                  }), Ln;
                }, {});
                Object.keys(an).forEach(function(Ln) {
                  Mn[Ln] || (Ln.length === 1 ? an[Ln] = 0 : delete an[Ln]);
                });
              }
            }
            function wn(an, Zt) {
              for (var Mn = [], Ln = {}, In = 0; In < Zt.length; In++) {
                var kn = Zt[In];
                Ln[kn.text2] ? Ln[kn.text2].push(kn.x) : Ln[kn.text2] = [kn.x];
              }
              for (var er in Ln) Mn.push(rn(an, oe.interp(Ln[er], 0.5), er));
              return Mn;
            }
            function Rn(an) {
              return an.periodX !== void 0 ? an.periodX : an.x;
            }
            function Pn(an) {
              return [an.text, an.x, an.axInfo, an.font, an.fontSize, an.fontColor].join("_");
            }
            function Ie(an) {
              var Zt = an.title.font.size, Mn = (an.title.text.match(ce.BR_TAG_ALL) || []).length;
              return an.title.hasOwnProperty("standoff") ? Zt * (et + Mn * dt) : Mn ? Zt * (Mn + 1) * dt : Zt;
            }
            function rt(an, Zt) {
              var Mn = an.l2p(Zt);
              return Mn > 1 && Mn < an._length - 1;
            }
            function ft(an) {
              var Zt = te.select(an), Mn = Zt.select(".text-math-group");
              return Mn.empty() ? Zt.select("text") : Mn;
            }
            function pt(an) {
              return an._id + ".automargin";
            }
            function wt(an) {
              return pt(an) + ".mirror";
            }
            function Bt(an) {
              return an._id + ".rangeslider";
            }
            function At(an, Zt) {
              for (var Mn = 0; Mn < Zt.length; Mn++) an.indexOf(Zt[Mn]) === -1 && an.push(Zt[Mn]);
            }
            function Dt(an, Zt, Mn) {
              var Ln, In, kn = [], er = [], jn = an.layout;
              for (Ln = 0; Ln < Zt.length; Ln++) kn.push(it.getFromId(an, Zt[Ln]));
              for (Ln = 0; Ln < Mn.length; Ln++) er.push(it.getFromId(an, Mn[Ln]));
              var ur = Object.keys(fe), Kn = ["anchor", "domain", "overlaying", "position", "side", "tickangle", "editType"], Jn = ["linear", "log"];
              for (Ln = 0; Ln < ur.length; Ln++) {
                var Lr = ur[Ln], mr = kn[0][Lr], Ar = er[0][Lr], Yr = !0, jr = !1, Mr = !1;
                if (Lr.charAt(0) !== "_" && typeof mr != "function" && Kn.indexOf(Lr) === -1) {
                  for (In = 1; In < kn.length && Yr; In++) {
                    var Wr = kn[In][Lr];
                    Lr === "type" && Jn.indexOf(mr) !== -1 && Jn.indexOf(Wr) !== -1 && mr !== Wr ? jr = !0 : Wr !== mr && (Yr = !1);
                  }
                  for (In = 1; In < er.length && Yr; In++) {
                    var Bi = er[In][Lr];
                    Lr === "type" && Jn.indexOf(Ar) !== -1 && Jn.indexOf(Bi) !== -1 && Ar !== Bi ? Mr = !0 : er[In][Lr] !== Ar && (Yr = !1);
                  }
                  Yr && (jr && (jn[kn[0]._name].type = "linear"), Mr && (jn[er[0]._name].type = "linear"), Ot(jn, Lr, kn, er, an._fullLayout._dfltTitle));
                }
              }
              for (Ln = 0; Ln < an._fullLayout.annotations.length; Ln++) {
                var gi = an._fullLayout.annotations[Ln];
                Zt.indexOf(gi.xref) !== -1 && Mn.indexOf(gi.yref) !== -1 && oe.swapAttrs(jn.annotations[Ln], ["?"]);
              }
            }
            function Ot(an, Zt, Mn, Ln, In) {
              var kn, er = oe.nestedProperty, jn = er(an[Mn[0]._name], Zt).get(), ur = er(an[Ln[0]._name], Zt).get();
              for (Zt === "title" && (jn && jn.text === In.x && (jn.text = In.y), ur && ur.text === In.y && (ur.text = In.x)), kn = 0; kn < Mn.length; kn++) er(an, Mn[kn]._name + "." + Zt).set(ur);
              for (kn = 0; kn < Ln.length; kn++) er(an, Ln[kn]._name + "." + Zt).set(jn);
            }
            function un(an) {
              return an._id === "angularaxis";
            }
            function yn(an, Zt) {
              for (var Mn = Zt._rangebreaks.length, Ln = 0; Ln < Mn; Ln++) {
                var In = Zt._rangebreaks[Ln];
                if (an >= In.min && an < In.max) return In.max;
              }
              return an;
            }
            function _n(an) {
              return (an.ticklabelposition || "").indexOf("inside") !== -1;
            }
            function gn(an, Zt) {
              _n(an._anchorAxis || {}) && an._hideCounterAxisInsideTickLabels && an._hideCounterAxisInsideTickLabels(Zt);
            }
            function bn(an, Zt, Mn, Ln) {
              var In, kn = an.anchor === "free" || an.overlaying !== void 0 && an.overlaying !== !1 ? an.overlaying : an._id;
              In = Ln ? an.side === "right" ? Zt : -Zt : Zt, kn in Mn || (Mn[kn] = {}), an.side in Mn[kn] || (Mn[kn][an.side] = 0), Mn[kn][an.side] += In;
            }
            it.getTickFormat = function(an) {
              var Zt, Mn, Ln, In, kn, er, jn, ur;
              function Kn(mr) {
                return typeof mr != "string" ? mr : Number(mr.replace("M", "")) * we;
              }
              function Jn(mr, Ar) {
                var Yr = ["L", "D"];
                if (typeof mr == typeof Ar) {
                  if (typeof mr == "number") return mr - Ar;
                  var jr = Yr.indexOf(mr.charAt(0)), Mr = Yr.indexOf(Ar.charAt(0));
                  return jr === Mr ? Number(mr.replace(/(L|D)/g, "")) - Number(Ar.replace(/(L|D)/g, "")) : jr - Mr;
                }
                return typeof mr == "number" ? 1 : -1;
              }
              function Lr(mr, Ar) {
                var Yr = Ar[0] === null, jr = Ar[1] === null, Mr = Jn(mr, Ar[0]) >= 0, Wr = Jn(mr, Ar[1]) <= 0;
                return (Yr || Mr) && (jr || Wr);
              }
              if (an.tickformatstops && an.tickformatstops.length > 0) switch (an.type) {
                case "date":
                case "linear":
                  for (Zt = 0; Zt < an.tickformatstops.length; Zt++) if ((Ln = an.tickformatstops[Zt]).enabled && (In = an.dtick, kn = Ln.dtickrange, er = void 0, jn = void 0, ur = void 0, er = Kn, jn = kn[0], ur = kn[1], (!jn && typeof jn != "number" || er(jn) <= er(In)) && (!ur && typeof ur != "number" || er(ur) >= er(In)))) {
                    Mn = Ln;
                    break;
                  }
                  break;
                case "log":
                  for (Zt = 0; Zt < an.tickformatstops.length; Zt++) if ((Ln = an.tickformatstops[Zt]).enabled && Lr(an.dtick, Ln.dtickrange)) {
                    Mn = Ln;
                    break;
                  }
              }
              return Mn ? Mn.value : an.tickformat;
            }, it.getSubplots = function(an, Zt) {
              var Mn = an._fullLayout._subplots, Ln = Mn.cartesian.concat(Mn.gl2d || []), In = Zt ? it.findSubplotsWithAxis(Ln, Zt) : Ln;
              return In.sort(function(kn, er) {
                var jn = kn.substr(1).split("y"), ur = er.substr(1).split("y");
                return jn[0] === ur[0] ? +jn[1] - +ur[1] : +jn[0] - +ur[0];
              }), In;
            }, it.findSubplotsWithAxis = function(an, Zt) {
              for (var Mn = new RegExp(Zt._id.charAt(0) === "x" ? "^" + Zt._id + "y" : Zt._id + "$"), Ln = [], In = 0; In < an.length; In++) {
                var kn = an[In];
                Mn.test(kn) && Ln.push(kn);
              }
              return Ln;
            }, it.makeClipPaths = function(an) {
              var Zt = an._fullLayout;
              if (!Zt._hasOnlyLargeSploms) {
                var Mn, Ln, In = { _offset: 0, _length: Zt.width, _id: "" }, kn = { _offset: 0, _length: Zt.height, _id: "" }, er = it.list(an, "x", !0), jn = it.list(an, "y", !0), ur = [];
                for (Mn = 0; Mn < er.length; Mn++) for (ur.push({ x: er[Mn], y: kn }), Ln = 0; Ln < jn.length; Ln++) Mn === 0 && ur.push({ x: In, y: jn[Ln] }), ur.push({ x: er[Mn], y: jn[Ln] });
                var Kn = Zt._clips.selectAll(".axesclip").data(ur, function(Jn) {
                  return Jn.x._id + Jn.y._id;
                });
                Kn.enter().append("clipPath").classed("axesclip", !0).attr("id", function(Jn) {
                  return "clip" + Zt._uid + Jn.x._id + Jn.y._id;
                }).append("rect"), Kn.exit().remove(), Kn.each(function(Jn) {
                  te.select(this).select("rect").attr({ x: Jn.x._offset || 0, y: Jn.y._offset || 0, width: Jn.x._length || 1, height: Jn.y._length || 1 });
                });
              }
            }, it.draw = function(an, Zt, Mn) {
              var Ln = an._fullLayout;
              Zt === "redraw" && Ln._paper.selectAll("g.subplot").each(function(jn) {
                var ur = jn[0], Kn = Ln._plots[ur];
                if (Kn) {
                  var Jn = Kn.xaxis, Lr = Kn.yaxis;
                  Kn.xaxislayer.selectAll("." + Jn._id + "tick").remove(), Kn.yaxislayer.selectAll("." + Lr._id + "tick").remove(), Kn.xaxislayer.selectAll("." + Jn._id + "tick2").remove(), Kn.yaxislayer.selectAll("." + Lr._id + "tick2").remove(), Kn.xaxislayer.selectAll("." + Jn._id + "divider").remove(), Kn.yaxislayer.selectAll("." + Lr._id + "divider").remove(), Kn.minorGridlayer && Kn.minorGridlayer.selectAll("path").remove(), Kn.gridlayer && Kn.gridlayer.selectAll("path").remove(), Kn.zerolinelayer && Kn.zerolinelayer.selectAll("path").remove(), Ln._infolayer.select(".g-" + Jn._id + "title").remove(), Ln._infolayer.select(".g-" + Lr._id + "title").remove();
                }
              });
              var In = Zt && Zt !== "redraw" ? Zt : it.listIds(an), kn = it.list(an).filter(function(jn) {
                return jn.autoshift;
              }).map(function(jn) {
                return jn.overlaying;
              });
              In.map(function(jn) {
                var ur = it.getFromId(an, jn);
                if (ur.tickmode === "sync" && ur.overlaying) {
                  var Kn = In.findIndex(function(Jn) {
                    return Jn === ur.overlaying;
                  });
                  Kn >= 0 && In.unshift(In.splice(Kn, 1).shift());
                }
              });
              var er = { false: { left: 0, right: 0 } };
              return oe.syncOrAsync(In.map(function(jn) {
                return function() {
                  if (jn) {
                    var ur = it.getFromId(an, jn);
                    Mn || (Mn = {}), Mn.axShifts = er, Mn.overlayingShiftedAx = kn;
                    var Kn = it.drawOne(an, ur, Mn);
                    return ur._shiftPusher && bn(ur, ur._fullDepth || 0, er, !0), ur._r = ur.range.slice(), ur._rl = oe.simpleMap(ur._r, ur.r2l), Kn;
                  }
                };
              }));
            }, it.drawOne = function(an, Zt, Mn) {
              var Ln, In, kn, er = (Mn = Mn || {}).axShifts || {}, jn = Mn.overlayingShiftedAx || [];
              Zt.setScale();
              var ur = an._fullLayout, Kn = Zt._id, Jn = Kn.charAt(0), Lr = it.counterLetter(Kn), mr = ur._plots[Zt._mainSubplot];
              if (mr) {
                if (Zt._shiftPusher = Zt.autoshift || jn.indexOf(Zt._id) !== -1 || jn.indexOf(Zt.overlaying) !== -1, Zt._shiftPusher & Zt.anchor === "free") {
                  var Ar = Zt.linewidth / 2 || 0;
                  Zt.ticks === "inside" && (Ar += Zt.ticklen), bn(Zt, Ar, er, !0), bn(Zt, Zt.shift || 0, er, !1);
                }
                Mn.skipTitle === !0 && Zt._shift !== void 0 || (Zt._shift = function(Er, gr) {
                  return Er.autoshift ? gr[Er.overlaying][Er.side] : Er.shift || 0;
                }(Zt, er));
                var Yr = mr[Jn + "axislayer"], jr = Zt._mainLinePosition, Mr = jr += Zt._shift, Wr = Zt._mainMirrorPosition, Bi = Zt._vals = it.calcTicks(Zt), gi = [Zt.mirror, Mr, Wr].join("_");
                for (Ln = 0; Ln < Bi.length; Ln++) Bi[Ln].axInfo = gi;
                Zt._selections = {}, Zt._tickAngles && (Zt._prevTickAngles = Zt._tickAngles), Zt._tickAngles = {}, Zt._depth = null;
                var Zr = {};
                if (Zt.visible) {
                  var fi, ri, bi = it.makeTransTickFn(Zt), Zi = it.makeTransTickLabelFn(Zt), $i = Zt.ticks === "inside", Br = Zt.ticks === "outside";
                  if (Zt.tickson === "boundaries") {
                    var Pr = function(Er, gr) {
                      var $r, li = [], Qi = function(Ai, Li) {
                        var Ni = Ai.xbnd[Li];
                        Ni !== null && li.push(oe.extendFlat({}, Ai, { x: Ni }));
                      };
                      if (gr.length) {
                        for ($r = 0; $r < gr.length; $r++) Qi(gr[$r], 0);
                        Qi(gr[$r - 1], 1);
                      }
                      return li;
                    }(0, Bi);
                    ri = it.clipEnds(Zt, Pr), fi = $i ? ri : Pr;
                  } else ri = it.clipEnds(Zt, Bi), fi = $i && Zt.ticklabelmode !== "period" ? ri : Bi;
                  var ei, Ci = Zt._gridVals = ri, oa = function(Er, gr) {
                    var $r, li, Qi = [], Ai = gr.length && gr[gr.length - 1].x < gr[0].x, Li = function(Pi, Oi) {
                      var ki = Pi.xbnd[Oi];
                      ki !== null && Qi.push(oe.extendFlat({}, Pi, { x: ki }));
                    };
                    if (Er.showdividers && gr.length) {
                      for ($r = 0; $r < gr.length; $r++) {
                        var Ni = gr[$r];
                        Ni.text2 !== li && Li(Ni, Ai ? 1 : 0), li = Ni.text2;
                      }
                      Li(gr[$r - 1], Ai ? 0 : 1);
                    }
                    return Qi;
                  }(Zt, Bi);
                  if (!ur._hasOnlyLargeSploms) {
                    var ka = Zt._subplotsWith, aa = {};
                    for (Ln = 0; Ln < ka.length; Ln++) {
                      In = ka[Ln];
                      var hi = (kn = ur._plots[In])[Lr + "axis"], Gr = hi._mainAxis._id;
                      if (!aa[Gr]) {
                        aa[Gr] = 1;
                        var Ca = Jn === "x" ? "M0," + hi._offset + "v" + hi._length : "M" + hi._offset + ",0h" + hi._length;
                        it.drawGrid(an, Zt, { vals: Ci, counterAxis: hi, layer: kn.gridlayer.select("." + Kn), minorLayer: kn.minorGridlayer.select("." + Kn), path: Ca, transFn: bi }), it.drawZeroLine(an, Zt, { counterAxis: hi, layer: kn.zerolinelayer, path: Ca, transFn: bi });
                      }
                    }
                  }
                  var Sr = it.getTickSigns(Zt), Dr = it.getTickSigns(Zt, "minor");
                  if (Zt.ticks || Zt.minor && Zt.minor.ticks) {
                    var si, zr, di, Da, la = it.makeTickPath(Zt, Mr, Sr[2]), wa = it.makeTickPath(Zt, Mr, Dr[2], { minor: !0 });
                    if (Zt._anchorAxis && Zt.mirror && Zt.mirror !== !0 ? (si = it.makeTickPath(Zt, Wr, Sr[3]), zr = it.makeTickPath(Zt, Wr, Dr[3], { minor: !0 }), di = la + si, Da = wa + zr) : (si = "", zr = "", di = la, Da = wa), Zt.showdividers && Br && Zt.tickson === "boundaries") {
                      var Wa = {};
                      for (Ln = 0; Ln < oa.length; Ln++) Wa[oa[Ln].x] = 1;
                      ei = function(Er) {
                        return Wa[Er.x] ? si : di;
                      };
                    } else ei = function(Er) {
                      return Er.minor ? Da : di;
                    };
                  }
                  if (it.drawTicks(an, Zt, { vals: fi, layer: Yr, path: ei, transFn: bi }), Zt.mirror === "allticks") {
                    var Qr = Object.keys(Zt._linepositions || {});
                    for (Ln = 0; Ln < Qr.length; Ln++) {
                      In = Qr[Ln], kn = ur._plots[In];
                      var Ur = Zt._linepositions[In] || [], rr = Ur[0], vr = Ur[1], Rr = Ur[2], Vr = it.makeTickPath(Zt, rr, Rr ? Sr[0] : Dr[0], { minor: Rr }) + it.makeTickPath(Zt, vr, Rr ? Sr[1] : Dr[1], { minor: Rr });
                      it.drawTicks(an, Zt, { vals: fi, layer: kn[Jn + "axislayer"], path: Vr, transFn: bi });
                    }
                  }
                  var Xr = [];
                  if (Xr.push(function() {
                    return it.drawLabels(an, Zt, { vals: Bi, layer: Yr, plotinfo: kn, transFn: Zi, labelFns: it.makeLabelFns(Zt, Mr) });
                  }), Zt.type === "multicategory") {
                    var Jr = { x: 2, y: 10 }[Jn];
                    Xr.push(function() {
                      var Er = { x: "height", y: "width" }[Jn], gr = Hr()[Er] + Jr + (Zt._tickAngles[Kn + "tick"] ? Zt.tickfont.size * dt : 0);
                      return it.drawLabels(an, Zt, { vals: wn(Zt, Bi), layer: Yr, cls: Kn + "tick2", repositionOnUpdate: !0, secondary: !0, transFn: bi, labelFns: it.makeLabelFns(Zt, Mr + gr * Sr[4]) });
                    }), Xr.push(function() {
                      return Zt._depth = Sr[4] * (Hr("tick2")[Zt.side] - Mr), function(Er, gr, $r) {
                        var li = gr._id + "divider", Qi = $r.vals, Ai = $r.layer.selectAll("path." + li).data(Qi, Pn);
                        Ai.exit().remove(), Ai.enter().insert("path", ":first-child").classed(li, 1).classed("crisp", 1).call(ue.stroke, gr.dividercolor).style("stroke-width", de.crispRound(Er, gr.dividerwidth, 1) + "px"), Ai.attr("transform", $r.transFn).attr("d", $r.path);
                      }(an, Zt, { vals: oa, layer: Yr, path: it.makeTickPath(Zt, Mr, Sr[4], { len: Zt._depth }), transFn: bi });
                    });
                  } else Zt.title.hasOwnProperty("standoff") && Xr.push(function() {
                    Zt._depth = Sr[4] * (Hr()[Zt.side] - Mr);
                  });
                  var ti = ae.getComponentMethod("rangeslider", "isVisible")(Zt);
                  return Mn.skipTitle || ti && Zt.side === "bottom" || Xr.push(function() {
                    return function(Er, gr) {
                      var $r, li = Er._fullLayout, Qi = gr._id, Ai = Qi.charAt(0), Li = gr.title.font.size, Ni = (gr.title.text.match(ce.BR_TAG_ALL) || []).length;
                      if (gr.title.hasOwnProperty("standoff")) gr.side === "bottom" || gr.side === "right" ? $r = gr._depth + gr.title.standoff + Li * et : gr.side !== "top" && gr.side !== "left" || ($r = gr._depth + gr.title.standoff + Li * (ct + Ni * dt));
                      else {
                        var Pi = _n(gr);
                        if (gr.type === "multicategory") $r = gr._depth;
                        else {
                          var Oi = 1.5 * Li;
                          Pi && (Oi = 0.5 * Li, gr.ticks === "outside" && (Oi += gr.ticklen)), $r = 10 + Oi + (gr.linewidth ? gr.linewidth - 1 : 0);
                        }
                        Pi || ($r += Ai === "x" ? gr.side === "top" ? Li * (gr.showticklabels ? 1 : 0) : Li * (gr.showticklabels ? 1.5 : 0.5) : gr.side === "right" ? Li * (gr.showticklabels ? 1 : 0.5) : Li * (gr.showticklabels ? 0.5 : 0));
                      }
                      var ki, Mi, Vi, ua, Ki = it.getPxPosition(Er, gr);
                      if (Ai === "x" ? (Mi = gr._offset + gr._length / 2, Vi = gr.side === "top" ? Ki - $r : Ki + $r) : (Vi = gr._offset + gr._length / 2, Mi = gr.side === "right" ? Ki + $r : Ki - $r, ki = { rotate: "-90", offset: 0 }), gr.type !== "multicategory") {
                        var Fi = gr._selections[gr._id + "tick"];
                        if (ua = { selection: Fi, side: gr.side }, Fi && Fi.node() && Fi.node().parentNode) {
                          var na = de.getTranslate(Fi.node().parentNode);
                          ua.offsetLeft = na.x, ua.offsetTop = na.y;
                        }
                        gr.title.hasOwnProperty("standoff") && (ua.pad = 0);
                      }
                      return gr._titleStandoff = $r, le.draw(Er, Qi + "title", { propContainer: gr, propName: gr._name + ".title.text", placeholder: li._dfltTitle[Ai], avoid: ua, transform: ki, attributes: { x: Mi, y: Vi, "text-anchor": "middle" } });
                    }(an, Zt);
                  }), Xr.push(function() {
                    var Er, gr, $r, li, Qi = Zt.side.charAt(0), Ai = Je[Zt.side].charAt(0), Li = it.getPxPosition(an, Zt), Ni = Br ? Zt.ticklen : 0;
                    (Zt.automargin || ti || Zt._shiftPusher) && (Zt.type === "multicategory" ? Er = Hr("tick2") : (Er = Hr(), Jn === "x" && Qi === "b" && (Zt._depth = Math.max(Er.width > 0 ? Er.bottom - Li : 0, Ni))));
                    var Pi = 0, Oi = 0;
                    if (Zt._shiftPusher && (Pi = Math.max(Ni, Er.height > 0 ? Qi === "l" ? Li - Er.left : Er.right - Li : 0), Zt.title.text !== ur._dfltTitle[Jn] && (Oi = (Zt._titleStandoff || 0) + (Zt._titleScoot || 0), Qi === "l" && (Oi += Ie(Zt))), Zt._fullDepth = Math.max(Pi, Oi)), Zt.automargin) {
                      gr = { x: 0, y: 0, r: 0, l: 0, t: 0, b: 0 };
                      var ki = [0, 1], Mi = typeof Zt._shift == "number" ? Zt._shift : 0;
                      if (Jn === "x") {
                        if (Qi === "b" ? gr[Qi] = Zt._depth : (gr[Qi] = Zt._depth = Math.max(Er.width > 0 ? Li - Er.top : 0, Ni), ki.reverse()), Er.width > 0) {
                          var Vi = Er.right - (Zt._offset + Zt._length);
                          Vi > 0 && (gr.xr = 1, gr.r = Vi);
                          var ua = Zt._offset - Er.left;
                          ua > 0 && (gr.xl = 0, gr.l = ua);
                        }
                      } else if (Qi === "l" ? (Zt._depth = Math.max(Er.height > 0 ? Li - Er.left : 0, Ni), gr[Qi] = Zt._depth - Mi) : (Zt._depth = Math.max(Er.height > 0 ? Er.right - Li : 0, Ni), gr[Qi] = Zt._depth + Mi, ki.reverse()), Er.height > 0) {
                        var Ki = Er.bottom - (Zt._offset + Zt._length);
                        Ki > 0 && (gr.yb = 0, gr.b = Ki);
                        var Fi = Zt._offset - Er.top;
                        Fi > 0 && (gr.yt = 1, gr.t = Fi);
                      }
                      gr[Lr] = Zt.anchor === "free" ? Zt.position : Zt._anchorAxis.domain[ki[0]], Zt.title.text !== ur._dfltTitle[Jn] && (gr[Qi] += Ie(Zt) + (Zt.title.standoff || 0)), Zt.mirror && Zt.anchor !== "free" && (($r = { x: 0, y: 0, r: 0, l: 0, t: 0, b: 0 })[Ai] = Zt.linewidth, Zt.mirror && Zt.mirror !== !0 && ($r[Ai] += Ni), Zt.mirror === !0 || Zt.mirror === "ticks" ? $r[Lr] = Zt._anchorAxis.domain[ki[1]] : Zt.mirror !== "all" && Zt.mirror !== "allticks" || ($r[Lr] = [Zt._counterDomainMin, Zt._counterDomainMax][ki[1]]));
                    }
                    ti && (li = ae.getComponentMethod("rangeslider", "autoMarginOpts")(an, Zt)), typeof Zt.automargin == "string" && (cn(gr, Zt.automargin), cn($r, Zt.automargin)), ee.autoMargin(an, pt(Zt), gr), ee.autoMargin(an, wt(Zt), $r), ee.autoMargin(an, Bt(Zt), li);
                  }), oe.syncOrAsync(Xr);
                }
              }
              function Hr(Er) {
                var gr = Kn + (Er || "tick");
                return Zr[gr] || (Zr[gr] = function($r, li, Qi) {
                  var Ai, Li, Ni, Pi;
                  if ($r._selections[li].size()) Ai = 1 / 0, Li = -1 / 0, Ni = 1 / 0, Pi = -1 / 0, $r._selections[li].each(function() {
                    var ki = ft(this), Mi = de.bBox(ki.node().parentNode);
                    Ai = Math.min(Ai, Mi.top), Li = Math.max(Li, Mi.bottom), Ni = Math.min(Ni, Mi.left), Pi = Math.max(Pi, Mi.right);
                  });
                  else {
                    var Oi = it.makeLabelFns($r, Qi);
                    Ai = Li = Oi.yFn({ dx: 0, dy: 0, fontSize: 0 }), Ni = Pi = Oi.xFn({ dx: 0, dy: 0, fontSize: 0 });
                  }
                  return { top: Ai, bottom: Li, left: Ni, right: Pi, height: Li - Ai, width: Pi - Ni };
                }(Zt, gr, Mr)), Zr[gr];
              }
            }, it.getTickSigns = function(an, Zt) {
              var Mn = an._id.charAt(0), Ln = { x: "top", y: "right" }[Mn], In = an.side === Ln ? 1 : -1, kn = [-1, 1, In, -In];
              return (Zt ? (an.minor || {}).ticks : an.ticks) !== "inside" == (Mn === "x") && (kn = kn.map(function(er) {
                return -er;
              })), an.side && kn.push({ l: -1, t: -1, r: 1, b: 1 }[an.side.charAt(0)]), kn;
            }, it.makeTransTickFn = function(an) {
              return an._id.charAt(0) === "x" ? function(Zt) {
                return se(an._offset + an.l2p(Zt.x), 0);
              } : function(Zt) {
                return se(0, an._offset + an.l2p(Zt.x));
              };
            }, it.makeTransTickLabelFn = function(an) {
              var Zt = function(Jn) {
                var Lr = Jn.ticklabelposition || "", mr = function(bi) {
                  return Lr.indexOf(bi) !== -1;
                }, Ar = mr("top"), Yr = mr("left"), jr = mr("right"), Mr = mr("bottom"), Wr = mr("inside"), Bi = Mr || Yr || Ar || jr;
                if (!Bi && !Wr) return [0, 0];
                var gi = Jn.side, Zr = Bi ? (Jn.tickwidth || 0) / 2 : 0, fi = 3, ri = Jn.tickfont ? Jn.tickfont.size : 12;
                return (Mr || Ar) && (Zr += ri * et, fi += (Jn.linewidth || 0) / 2), (Yr || jr) && (Zr += (Jn.linewidth || 0) / 2, fi += 3), Wr && gi === "top" && (fi -= ri * (1 - et)), (Yr || Ar) && (Zr = -Zr), gi !== "bottom" && gi !== "right" || (fi = -fi), [Bi ? Zr : 0, Wr ? fi : 0];
              }(an), Mn = an.ticklabelshift || 0, Ln = an.ticklabelstandoff || 0, In = Zt[0], kn = Zt[1], er = an.range[0] > an.range[1], jn = an.ticklabelposition && an.ticklabelposition.indexOf("inside") !== -1, ur = !jn;
              if (Mn && (Mn *= er ? -1 : 1), Ln) {
                var Kn = an.side;
                Ln *= jn && (Kn === "top" || Kn === "left") || ur && (Kn === "bottom" || Kn === "right") ? 1 : -1;
              }
              return an._id.charAt(0) === "x" ? function(Jn) {
                return se(In + an._offset + an.l2p(Rn(Jn)) + Mn, kn + Ln);
              } : function(Jn) {
                return se(kn + Ln, In + an._offset + an.l2p(Rn(Jn)) + Mn);
              };
            }, it.makeTickPath = function(an, Zt, Mn, Ln) {
              Ln || (Ln = {});
              var In = Ln.minor;
              if (In && !an.minor) return "";
              var kn = Ln.len !== void 0 ? Ln.len : In ? an.minor.ticklen : an.ticklen, er = an._id.charAt(0), jn = (an.linewidth || 1) / 2;
              return er === "x" ? "M0," + (Zt + jn * Mn) + "v" + kn * Mn : "M" + (Zt + jn * Mn) + ",0h" + kn * Mn;
            }, it.makeLabelFns = function(an, Zt, Mn) {
              var Ln = an.ticklabelposition || "", In = function(Pr) {
                return Ln.indexOf(Pr) !== -1;
              }, kn = In("top"), er = In("left"), jn = In("right"), ur = In("bottom") || er || kn || jn, Kn = In("inside"), Jn = Ln === "inside" && an.ticks === "inside" || !Kn && an.ticks === "outside" && an.tickson !== "boundaries", Lr = 0, mr = 0, Ar = Jn ? an.ticklen : 0;
              if (Kn ? Ar *= -1 : ur && (Ar = 0), Jn && (Lr += Ar, Mn)) {
                var Yr = oe.deg2rad(Mn);
                Lr = Ar * Math.cos(Yr) + 1, mr = Ar * Math.sin(Yr);
              }
              an.showticklabels && (Jn || an.showline) && (Lr += 0.2 * an.tickfont.size);
              var jr, Mr, Wr, Bi, gi, Zr = { labelStandoff: Lr += (an.linewidth || 1) / 2 * (Kn ? -1 : 1), labelShift: mr }, fi = 0, ri = an.side, bi = an._id.charAt(0), Zi = an.tickangle;
              if (bi === "x") Bi = (gi = !Kn && ri === "bottom" || Kn && ri === "top") ? 1 : -1, Kn && (Bi *= -1), jr = mr * Bi, Mr = Zt + Lr * Bi, Wr = gi ? 1 : -0.2, Math.abs(Zi) === 90 && (Kn ? Wr += ct : Wr = Zi === -90 && ri === "bottom" ? et : Zi === 90 && ri === "top" ? ct : 0.5, fi = ct / 2 * (Zi / 90)), Zr.xFn = function(Pr) {
                return Pr.dx + jr + fi * Pr.fontSize;
              }, Zr.yFn = function(Pr) {
                return Pr.dy + Mr + Pr.fontSize * Wr;
              }, Zr.anchorFn = function(Pr, ei) {
                if (ur) {
                  if (er) return "end";
                  if (jn) return "start";
                }
                return ne(ei) && ei !== 0 && ei !== 180 ? ei * Bi < 0 !== Kn ? "end" : "start" : "middle";
              }, Zr.heightFn = function(Pr, ei, Ci) {
                return ei < -60 || ei > 60 ? -0.5 * Ci : an.side === "top" !== Kn ? -Ci : 0;
              };
              else if (bi === "y") {
                if (Bi = (gi = !Kn && ri === "left" || Kn && ri === "right") ? 1 : -1, Kn && (Bi *= -1), jr = Lr, Mr = mr * Bi, Wr = 0, Kn || Math.abs(Zi) !== 90 || (Wr = Zi === -90 && ri === "left" || Zi === 90 && ri === "right" ? et : 0.5), Kn) {
                  var $i = ne(Zi) ? +Zi : 0;
                  if ($i !== 0) {
                    var Br = oe.deg2rad($i);
                    fi = Math.abs(Math.sin(Br)) * et * Bi, Wr = 0;
                  }
                }
                Zr.xFn = function(Pr) {
                  return Pr.dx + Zt - (jr + Pr.fontSize * Wr) * Bi + fi * Pr.fontSize;
                }, Zr.yFn = function(Pr) {
                  return Pr.dy + Mr + Pr.fontSize * ct;
                }, Zr.anchorFn = function(Pr, ei) {
                  return ne(ei) && Math.abs(ei) === 90 ? "middle" : gi ? "end" : "start";
                }, Zr.heightFn = function(Pr, ei, Ci) {
                  return an.side === "right" && (ei *= -1), ei < -30 ? -Ci : ei < 30 ? -0.5 * Ci : 0;
                };
              }
              return Zr;
            }, it.drawTicks = function(an, Zt, Mn) {
              Mn = Mn || {};
              var Ln = Zt._id + "tick", In = [].concat(Zt.minor && Zt.minor.ticks ? Mn.vals.filter(function(er) {
                return er.minor && !er.noTick;
              }) : []).concat(Zt.ticks ? Mn.vals.filter(function(er) {
                return !er.minor && !er.noTick;
              }) : []), kn = Mn.layer.selectAll("path." + Ln).data(In, Pn);
              kn.exit().remove(), kn.enter().append("path").classed(Ln, 1).classed("ticks", 1).classed("crisp", Mn.crisp !== !1).each(function(er) {
                return ue.stroke(te.select(this), er.minor ? Zt.minor.tickcolor : Zt.tickcolor);
              }).style("stroke-width", function(er) {
                return de.crispRound(an, er.minor ? Zt.minor.tickwidth : Zt.tickwidth, 1) + "px";
              }).attr("d", Mn.path).style("display", null), gn(Zt, [Ge]), kn.attr("transform", Mn.transFn);
            }, it.drawGrid = function(an, Zt, Mn) {
              if (Mn = Mn || {}, Zt.tickmode !== "sync") {
                var Ln = Zt._id + "grid", In = Zt.minor && Zt.minor.showgrid, kn = In ? Mn.vals.filter(function(Wr) {
                  return Wr.minor;
                }) : [], er = Zt.showgrid ? Mn.vals.filter(function(Wr) {
                  return !Wr.minor;
                }) : [], jn = Mn.counterAxis;
                if (jn && it.shouldShowZeroLine(an, Zt, jn)) for (var ur = Zt.tickmode === "array", Kn = 0; Kn < er.length; Kn++) {
                  var Jn = er[Kn].x;
                  if (ur ? !Jn : Math.abs(Jn) < Zt.dtick / 100) {
                    if (er = er.slice(0, Kn).concat(er.slice(Kn + 1)), !ur) break;
                    Kn--;
                  }
                }
                Zt._gw = de.crispRound(an, Zt.gridwidth, 1);
                for (var Lr = In ? de.crispRound(an, Zt.minor.gridwidth, 1) : 0, mr = Mn.layer, Ar = Mn.minorLayer, Yr = 1; Yr >= 0; Yr--) {
                  var jr = Yr ? mr : Ar;
                  if (jr) {
                    var Mr = jr.selectAll("path." + Ln).data(Yr ? er : kn, Pn);
                    Mr.exit().remove(), Mr.enter().append("path").classed(Ln, 1).classed("crisp", Mn.crisp !== !1), Mr.attr("transform", Mn.transFn).attr("d", Mn.path).each(function(Wr) {
                      return ue.stroke(te.select(this), Wr.minor ? Zt.minor.gridcolor : Zt.gridcolor || "#ddd");
                    }).style("stroke-dasharray", function(Wr) {
                      return de.dashStyle(Wr.minor ? Zt.minor.griddash : Zt.griddash, Wr.minor ? Zt.minor.gridwidth : Zt.gridwidth);
                    }).style("stroke-width", function(Wr) {
                      return (Wr.minor ? Lr : Zt._gw) + "px";
                    }).style("display", null), typeof Mn.path == "function" && Mr.attr("d", Mn.path);
                  }
                }
                gn(Zt, [Ve, Ze]);
              }
            }, it.drawZeroLine = function(an, Zt, Mn) {
              Mn = Mn || Mn;
              var Ln = Zt._id + "zl", In = it.shouldShowZeroLine(an, Zt, Mn.counterAxis), kn = Mn.layer.selectAll("path." + Ln).data(In ? [{ x: 0, id: Zt._id }] : []);
              kn.exit().remove(), kn.enter().append("path").classed(Ln, 1).classed("zl", 1).classed("crisp", Mn.crisp !== !1).each(function() {
                Mn.layer.selectAll("path").sort(function(er, jn) {
                  return bt(er.id, jn.id);
                });
              }), kn.attr("transform", Mn.transFn).attr("d", Mn.path).call(ue.stroke, Zt.zerolinecolor || ue.defaultLine).style("stroke-width", de.crispRound(an, Zt.zerolinewidth, Zt._gw || 1) + "px").style("display", null), gn(Zt, [Xe]);
            }, it.drawLabels = function(an, Zt, Mn) {
              Mn = Mn || {};
              var Ln = an._fullLayout, In = Zt._id, kn = Mn.cls || In + "tick", er = Mn.vals.filter(function(aa) {
                return aa.text;
              }), jn = Mn.labelFns, ur = Mn.secondary ? 0 : Zt.tickangle, Kn = (Zt._prevTickAngles || {})[kn], Jn = Mn.layer.selectAll("g." + kn).data(Zt.showticklabels ? er : [], Pn), Lr = [];
              function mr(aa, hi) {
                aa.each(function(Gr) {
                  var Ca = te.select(this), Sr = Ca.select(".text-math-group"), Dr = jn.anchorFn(Gr, hi), si = Mn.transFn.call(Ca.node(), Gr) + (ne(hi) && +hi != 0 ? " rotate(" + hi + "," + jn.xFn(Gr) + "," + (jn.yFn(Gr) - Gr.fontSize / 2) + ")" : ""), zr = ce.lineCount(Ca), di = dt * Gr.fontSize, Da = jn.heightFn(Gr, ne(hi) ? +hi : 0, (zr - 1) * di);
                  if (Da && (si += se(0, Da)), Sr.empty()) {
                    var la = Ca.select("text");
                    la.attr({ transform: si, "text-anchor": Dr }), la.style("opacity", 1), Zt._adjustTickLabelsOverflow && Zt._adjustTickLabelsOverflow();
                  } else {
                    var wa = de.bBox(Sr.node()).width * { end: -0.5, start: 0.5 }[Dr];
                    Sr.attr("transform", si + se(wa, 0));
                  }
                });
              }
              Jn.enter().append("g").classed(kn, 1).append("text").attr("text-anchor", "middle").each(function(aa) {
                var hi = te.select(this), Gr = an._promises.length;
                hi.call(ce.positionText, jn.xFn(aa), jn.yFn(aa)).call(de.font, { family: aa.font, size: aa.fontSize, color: aa.fontColor, weight: aa.fontWeight, style: aa.fontStyle, variant: aa.fontVariant, textcase: aa.fontTextcase, lineposition: aa.fontLineposition, shadow: aa.fontShadow }).text(aa.text).call(ce.convertToTspans, an), an._promises[Gr] ? Lr.push(an._promises.pop().then(function() {
                  mr(hi, ur);
                })) : mr(hi, ur);
              }), gn(Zt, [ze]), Jn.exit().remove(), Mn.repositionOnUpdate && Jn.each(function(aa) {
                te.select(this).select("text").call(ce.positionText, jn.xFn(aa), jn.yFn(aa));
              }), Zt._adjustTickLabelsOverflow = function() {
                var aa = Zt.ticklabeloverflow;
                if (aa && aa !== "allow") {
                  var hi = aa.indexOf("hide") !== -1, Gr = Zt._id.charAt(0) === "x", Ca = 0, Sr = Gr ? an._fullLayout.width : an._fullLayout.height;
                  if (aa.indexOf("domain") !== -1) {
                    var Dr = oe.simpleMap(Zt.range, Zt.r2l);
                    Ca = Zt.l2p(Dr[0]) + Zt._offset, Sr = Zt.l2p(Dr[1]) + Zt._offset;
                  }
                  var si = Math.min(Ca, Sr), zr = Math.max(Ca, Sr), di = Zt.side, Da = 1 / 0, la = -1 / 0;
                  for (var wa in Jn.each(function(Ur) {
                    var rr = te.select(this);
                    if (rr.select(".text-math-group").empty()) {
                      var vr = de.bBox(rr.node()), Rr = 0;
                      Gr ? (vr.right > zr || vr.left < si) && (Rr = 1) : (vr.bottom > zr || vr.top + (Zt.tickangle ? 0 : Ur.fontSize / 4) < si) && (Rr = 1);
                      var Vr = rr.select("text");
                      Rr ? hi && Vr.style("opacity", 0) : (Vr.style("opacity", 1), Da = di === "bottom" || di === "right" ? Math.min(Da, Gr ? vr.top : vr.left) : -1 / 0, la = di === "top" || di === "left" ? Math.max(la, Gr ? vr.bottom : vr.right) : 1 / 0);
                    }
                  }), Ln._plots) {
                    var Wa = Ln._plots[wa];
                    if (Zt._id === Wa.xaxis._id || Zt._id === Wa.yaxis._id) {
                      var Qr = Gr ? Wa.yaxis : Wa.xaxis;
                      Qr && (Qr["_visibleLabelMin_" + Zt._id] = Da, Qr["_visibleLabelMax_" + Zt._id] = la);
                    }
                  }
                }
              }, Zt._hideCounterAxisInsideTickLabels = function(aa) {
                var hi = Zt._id.charAt(0) === "x", Gr = [];
                for (var Ca in Ln._plots) {
                  var Sr = Ln._plots[Ca];
                  Zt._id !== Sr.xaxis._id && Zt._id !== Sr.yaxis._id || Gr.push(hi ? Sr.yaxis : Sr.xaxis);
                }
                Gr.forEach(function(Dr, si) {
                  Dr && _n(Dr) && (aa || [Xe, Ze, Ve, Ge, ze]).forEach(function(zr) {
                    var di = zr.K === "tick" && zr.L === "text" && Zt.ticklabelmode === "period", Da = Ln._plots[Zt._mainSubplot];
                    (zr.K === Xe.K ? Da.zerolinelayer.selectAll("." + Zt._id + "zl") : zr.K === Ze.K ? Da.minorGridlayer.selectAll("." + Zt._id) : zr.K === Ve.K ? Da.gridlayer.selectAll("." + Zt._id) : Da[Zt._id.charAt(0) + "axislayer"]).each(function() {
                      var la = te.select(this);
                      zr.L && (la = la.selectAll(zr.L)), la.each(function(wa) {
                        var Wa = Zt.l2p(di ? Rn(wa) : wa.x) + Zt._offset, Qr = te.select(this);
                        Wa < Zt["_visibleLabelMax_" + Dr._id] && Wa > Zt["_visibleLabelMin_" + Dr._id] ? Qr.style("display", "none") : zr.K !== "tick" || si || Qr.style("display", null);
                      });
                    });
                  });
                });
              }, mr(Jn, Kn + 1 ? Kn : ur);
              var Ar = null;
              Zt._selections && (Zt._selections[kn] = Jn);
              var Yr = [function() {
                return Lr.length && Promise.all(Lr);
              }];
              Zt.automargin && Ln._redrawFromAutoMarginCount && Kn === 90 ? (Ar = Kn, Yr.push(function() {
                mr(Jn, Kn);
              })) : Yr.push(function() {
                if (mr(Jn, ur), er.length && Zt.autotickangles && (Zt.type !== "log" || String(Zt.dtick).charAt(0) !== "D")) {
                  Ar = Zt.autotickangles[0];
                  var aa, hi = 0, Gr = [], Ca = 1;
                  Jn.each(function(Hr) {
                    hi = Math.max(hi, Hr.fontSize);
                    var Er = Zt.l2p(Hr.x), gr = ft(this), $r = de.bBox(gr.node());
                    Ca = Math.max(Ca, ce.lineCount(gr)), Gr.push({ top: 0, bottom: 10, height: 10, left: Er - $r.width / 2, right: Er + $r.width / 2 + 2, width: $r.width + 2 });
                  });
                  var Sr = (Zt.tickson === "boundaries" || Zt.showdividers) && !Mn.secondary, Dr = er.length, si = Math.abs((er[Dr - 1].x - er[0].x) * Zt._m) / (Dr - 1), zr = Sr ? si / 2 : si, di = Sr ? Zt.ticklen : 1.25 * hi * Ca, Da = zr / Math.sqrt(Math.pow(zr, 2) + Math.pow(di, 2)), la = Zt.autotickangles.map(function(Hr) {
                    return Hr * Math.PI / 180;
                  }), wa = la.find(function(Hr) {
                    return Math.abs(Math.cos(Hr)) <= Da;
                  });
                  wa === void 0 && (wa = la.reduce(function(Hr, Er) {
                    return Math.abs(Math.cos(Hr)) < Math.abs(Math.cos(Er)) ? Hr : Er;
                  }, la[0]));
                  var Wa = wa * (180 / Math.PI);
                  if (Sr) {
                    var Qr = 2;
                    for (Zt.ticks && (Qr += Zt.tickwidth / 2), aa = 0; aa < Gr.length; aa++) {
                      var Ur = er[aa].xbnd, rr = Gr[aa];
                      if (Ur[0] !== null && rr.left - Zt.l2p(Ur[0]) < Qr || Ur[1] !== null && Zt.l2p(Ur[1]) - rr.right < Qr) {
                        Ar = Wa;
                        break;
                      }
                    }
                  } else {
                    var vr = Zt.ticklabelposition || "", Rr = function(Hr) {
                      return vr.indexOf(Hr) !== -1;
                    }, Vr = Rr("top"), Xr = Rr("left"), Jr = Rr("right"), ti = Rr("bottom") || Xr || Vr || Jr ? (Zt.tickwidth || 0) + 6 : 0;
                    for (aa = 0; aa < Gr.length - 1; aa++) if (oe.bBoxIntersect(Gr[aa], Gr[aa + 1], ti)) {
                      Ar = Wa;
                      break;
                    }
                  }
                  Ar && mr(Jn, Ar);
                }
              }), Zt._tickAngles && Yr.push(function() {
                Zt._tickAngles[kn] = Ar === null ? ne(ur) ? ur : 0 : Ar;
              });
              var jr = function() {
                var aa = 0, hi = 0;
                return Jn.each(function(Gr, Ca) {
                  var Sr, Dr = ft(this);
                  Dr.select(".text-math-group").empty() && (Zt._vals[Ca] && (Sr = Zt._vals[Ca].bb || de.bBox(Dr.node()), Zt._vals[Ca].bb = Sr), aa = Math.max(aa, Sr.width), hi = Math.max(hi, Sr.height));
                }), { labelsMaxW: aa, labelsMaxH: hi };
              }, Mr = Zt._anchorAxis;
              if (Mr && (Mr.autorange || Mr.insiderange) && _n(Zt) && !yt(Ln, Zt._id) && (Ln._insideTickLabelsUpdaterange || (Ln._insideTickLabelsUpdaterange = {}), Mr.autorange && (Ln._insideTickLabelsUpdaterange[Mr._name + ".autorange"] = Mr.autorange, Yr.push(jr)), Mr.insiderange)) {
                var Wr = jr(), Bi = Zt._id.charAt(0) === "y" ? Wr.labelsMaxW : Wr.labelsMaxH;
                Bi += 6, Zt.ticklabelposition === "inside" && (Bi += Zt.ticklen || 0);
                var gi = Zt.side === "right" || Zt.side === "top" ? 1 : -1, Zr = gi === 1 ? 1 : 0, fi = gi === 1 ? 0 : 1, ri = [];
                ri[fi] = Mr.range[fi];
                var bi = Mr.range, Zi = Mr.r2p(bi[Zr]), $i = Mr.r2p(bi[fi]), Br = Ln._insideTickLabelsUpdaterange[Mr._name + ".range"];
                if (Br) {
                  var Pr = Mr.r2p(Br[Zr]), ei = Mr.r2p(Br[fi]), Ci = gi * (Zt._id.charAt(0) === "y" ? 1 : -1);
                  Ci * Zi < Ci * Pr && (Zi = Pr, ri[Zr] = bi[Zr] = Br[Zr]), Ci * $i > Ci * ei && ($i = ei, ri[fi] = bi[fi] = Br[fi]);
                }
                var oa = Math.abs($i - Zi);
                oa - Bi > 0 ? Bi *= 1 + Bi / (oa -= Bi) : Bi = 0, Zt._id.charAt(0) !== "y" && (Bi = -Bi), ri[Zr] = Mr.p2r(Mr.r2p(bi[Zr]) + gi * Bi), Mr.autorange === "min" || Mr.autorange === "max reversed" ? (ri[0] = null, Mr._rangeInitial0 = void 0, Mr._rangeInitial1 = void 0) : Mr.autorange !== "max" && Mr.autorange !== "min reversed" || (ri[1] = null, Mr._rangeInitial0 = void 0, Mr._rangeInitial1 = void 0), Ln._insideTickLabelsUpdaterange[Mr._name + ".range"] = ri;
              }
              var ka = oe.syncOrAsync(Yr);
              return ka && ka.then && an._promises.push(ka), ka;
            }, it.getPxPosition = function(an, Zt) {
              var Mn, Ln = an._fullLayout._size, In = Zt._id.charAt(0), kn = Zt.side;
              return Zt.anchor !== "free" ? Mn = Zt._anchorAxis : In === "x" ? Mn = { _offset: Ln.t + (1 - (Zt.position || 0)) * Ln.h, _length: 0 } : In === "y" && (Mn = { _offset: Ln.l + (Zt.position || 0) * Ln.w + Zt._shift, _length: 0 }), kn === "top" || kn === "left" ? Mn._offset : kn === "bottom" || kn === "right" ? Mn._offset + Mn._length : void 0;
            }, it.shouldShowZeroLine = function(an, Zt, Mn) {
              var Ln = oe.simpleMap(Zt.range, Zt.r2l);
              return Ln[0] * Ln[1] <= 0 && Zt.zeroline && (Zt.type === "linear" || Zt.type === "-") && !(Zt.rangebreaks && Zt.maskBreaks(0) === qe) && (rt(Zt, 0) || !function(In, kn, er, jn) {
                var ur = er._mainAxis;
                if (ur) {
                  var Kn = In._fullLayout, Jn = kn._id.charAt(0), Lr = it.counterLetter(kn._id), mr = kn._offset + (Math.abs(jn[0]) < Math.abs(jn[1]) == (Jn === "x") ? 0 : kn._length), Ar = Kn._plots[er._mainSubplot];
                  if (!(Ar.mainplotinfo || Ar).overlays.length) return Wr(er);
                  for (var Yr = it.list(In, Lr), jr = 0; jr < Yr.length; jr++) {
                    var Mr = Yr[jr];
                    if (Mr._mainAxis === ur && Wr(Mr)) return !0;
                  }
                }
                function Wr(Bi) {
                  if (!Bi.showline || !Bi.linewidth) return !1;
                  var gi = Math.max((Bi.linewidth + kn.zerolinewidth) / 2, 1);
                  function Zr(bi) {
                    return typeof bi == "number" && Math.abs(bi - mr) < gi;
                  }
                  if (Zr(Bi._mainLinePosition) || Zr(Bi._mainMirrorPosition)) return !0;
                  var fi = Bi._linepositions || {};
                  for (var ri in fi) if (Zr(fi[ri][0]) || Zr(fi[ri][1])) return !0;
                }
              }(an, Zt, Mn, Ln) || function(In, kn) {
                for (var er = In._fullData, jn = kn._mainSubplot, ur = kn._id.charAt(0), Kn = 0; Kn < er.length; Kn++) {
                  var Jn = er[Kn];
                  if (Jn.visible === !0 && Jn.xaxis + Jn.yaxis === jn && (ae.traceIs(Jn, "bar-like") && Jn.orientation === { x: "h", y: "v" }[ur] || Jn.fill && Jn.fill.charAt(Jn.fill.length - 1) === ur))
                    return !0;
                }
                return !1;
              }(an, Zt));
            }, it.clipEnds = function(an, Zt) {
              return Zt.filter(function(Mn) {
                return rt(an, Mn.x);
              });
            }, it.allowAutoMargin = function(an) {
              for (var Zt = it.list(an, "", !0), Mn = 0; Mn < Zt.length; Mn++) {
                var Ln = Zt[Mn];
                Ln.automargin && (ee.allowAutoMargin(an, pt(Ln)), Ln.mirror && ee.allowAutoMargin(an, wt(Ln))), ae.getComponentMethod("rangeslider", "isVisible")(Ln) && ee.allowAutoMargin(an, Bt(Ln));
              }
            }, it.swap = function(an, Zt) {
              for (var Mn = function(In, kn) {
                var er, jn, ur = [];
                for (er = 0; er < kn.length; er++) {
                  var Kn = [], Jn = In._fullData[kn[er]].xaxis, Lr = In._fullData[kn[er]].yaxis;
                  if (Jn && Lr) {
                    for (jn = 0; jn < ur.length; jn++) ur[jn].x.indexOf(Jn) === -1 && ur[jn].y.indexOf(Lr) === -1 || Kn.push(jn);
                    if (Kn.length) {
                      var mr, Ar = ur[Kn[0]];
                      if (Kn.length > 1) for (jn = 1; jn < Kn.length; jn++) mr = ur[Kn[jn]], At(Ar.x, mr.x), At(Ar.y, mr.y);
                      At(Ar.x, [Jn]), At(Ar.y, [Lr]);
                    } else ur.push({ x: [Jn], y: [Lr] });
                  }
                }
                return ur;
              }(an, Zt), Ln = 0; Ln < Mn.length; Ln++) Dt(an, Mn[Ln].x, Mn[Ln].y);
            };
          }, 9666: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(63821).BADNUM, ae = ne.isArrayOrTypedArray, oe = ne.isDateTime, se = ne.cleanNumber, ce = Math.round;
            function le(de, fe) {
              return fe ? te(de) : typeof de == "number";
            }
            function ue(de) {
              return Math.max(1, (de - 1) / 1e3);
            }
            G.exports = function(de, fe, he) {
              var me = de, pe = he.noMultiCategory;
              if (ae(me) && !me.length) return "-";
              if (!pe && function(Ce) {
                return ae(Ce[0]) && ae(Ce[1]);
              }(me)) return "multicategory";
              if (pe && Array.isArray(me[0])) {
                for (var ye = [], ge = 0; ge < me.length; ge++) if (ae(me[ge])) for (var ve = 0; ve < me[ge].length; ve++) ye.push(me[ge][ve]);
                me = ye;
              }
              if (function(Ce, Se) {
                for (var we = Ce.length, Ee = ue(we), xe = 0, Le = 0, Qe = {}, ke = 0; ke < we; ke += Ee) {
                  var Me = Ce[ce(ke)], Re = String(Me);
                  Qe[Re] || (Qe[Re] = 1, oe(Me, Se) && xe++, te(Me) && Le++);
                }
                return xe > 2 * Le;
              }(me, fe)) return "date";
              var be = he.autotypenumbers !== "strict";
              return function(Ce, Se) {
                for (var we = Ce.length, Ee = ue(we), xe = 0, Le = 0, Qe = {}, ke = 0; ke < we; ke += Ee) {
                  var Me = Ce[ce(ke)], Re = String(Me);
                  if (!Qe[Re]) {
                    Qe[Re] = 1;
                    var De = typeof Me;
                    De === "boolean" ? Le++ : (Se ? se(Me) !== ee : De === "number") ? xe++ : De === "string" && Le++;
                  }
                }
                return Le > 2 * xe;
              }(me, be) ? "category" : function(Ce, Se) {
                for (var we = Ce.length, Ee = 0; Ee < we; Ee++) if (le(Ce[Ee], Se)) return !0;
                return !1;
              }(me, be) ? "linear" : "-";
            };
          }, 97655: function(G, J, Z) {
            var te = Z(10721), ne = Z(33626), ee = Z(34809), ae = Z(78032), oe = Z(59008), se = Z(25829), ce = Z(22777), le = Z(87433), ue = Z(12036), de = Z(54616), fe = Z(46473), he = Z(97405), me = Z(90259), pe = Z(19091), ye = Z(54826).WEEKDAY_PATTERN, ge = Z(54826).HOUR_PATTERN;
            function ve(Se, we, Ee) {
              function xe($e, qe) {
                return ee.coerce(Se, we, se.rangebreaks, $e, qe);
              }
              if (xe("enabled")) {
                var Le = xe("bounds");
                if (Le && Le.length >= 2) {
                  var Qe, ke, Me = "";
                  if (Le.length === 2) {
                    for (Qe = 0; Qe < 2; Qe++) if (ke = Ce(Le[Qe])) {
                      Me = ye;
                      break;
                    }
                  }
                  var Re = xe("pattern", Me);
                  if (Re === ye) for (Qe = 0; Qe < 2; Qe++) (ke = Ce(Le[Qe])) && (we.bounds[Qe] = Le[Qe] = ke - 1);
                  if (Re) for (Qe = 0; Qe < 2; Qe++) switch (ke = Le[Qe], Re) {
                    case ye:
                      if (!te(ke) || (ke = +ke) !== Math.floor(ke) || ke < 0 || ke >= 7) return void (we.enabled = !1);
                      we.bounds[Qe] = Le[Qe] = ke;
                      break;
                    case ge:
                      if (!te(ke) || (ke = +ke) < 0 || ke > 24) return void (we.enabled = !1);
                      we.bounds[Qe] = Le[Qe] = ke;
                  }
                  if (Ee.autorange === !1) {
                    var De = Ee.range;
                    if (De[0] < De[1]) {
                      if (Le[0] < De[0] && Le[1] > De[1]) return void (we.enabled = !1);
                    } else if (Le[0] > De[0] && Le[1] < De[1]) return void (we.enabled = !1);
                  }
                } else {
                  var Pe = xe("values");
                  if (!Pe || !Pe.length) return void (we.enabled = !1);
                  xe("dvalue");
                }
              }
            }
            G.exports = function(Se, we, Ee, xe, Le) {
              var Qe, ke = xe.letter, Me = xe.font || {}, Re = xe.splomStash || {}, De = Ee("visible", !xe.visibleDflt), Pe = we._template || {}, $e = we.type || Pe.type || "-";
              $e === "date" && (ne.getComponentMethod("calendars", "handleDefaults")(Se, we, "calendar", xe.calendar), xe.noTicklabelmode || (Qe = Ee("ticklabelmode"))), xe.noTicklabelindex || $e !== "date" && $e !== "linear" || Ee("ticklabelindex");
              var qe = "";
              xe.noTicklabelposition && $e !== "multicategory" || (qe = ee.coerce(Se, we, { ticklabelposition: { valType: "enumerated", dflt: "outside", values: Qe === "period" ? ["outside", "inside"] : ke === "x" ? ["outside", "inside", "outside left", "inside left", "outside right", "inside right"] : ["outside", "inside", "outside top", "inside top", "outside bottom", "inside bottom"] } }, "ticklabelposition")), xe.noTicklabeloverflow || Ee("ticklabeloverflow", qe.indexOf("inside") !== -1 ? "hide past domain" : $e === "category" || $e === "multicategory" ? "allow" : "hide past div"), pe(we, Le), me(Se, we, Ee, xe), fe(Se, we, Ee, xe), $e === "category" || xe.noHover || Ee("hoverformat");
              var Xe = Ee("color"), Ve = Xe !== se.color.dflt ? Xe : Me.color, Ze = Re.label || Le._dfltTitle[ke];
              if (de(Se, we, Ee, $e, xe), !De) return we;
              Ee("title.text", Ze), ee.coerceFont(Ee, "title.font", Me, { overrideDflt: { size: ee.bigFont(Me.size), color: Ve } }), ce(Se, we, Ee, $e);
              var Ge = xe.hasMinor;
              if (Ge && (ae.newContainer(we, "minor"), ce(Se, we, Ee, $e, { isMinor: !0 })), ue(Se, we, Ee, $e, xe), le(Se, we, Ee, xe), Ge) {
                var ze = xe.isMinor;
                xe.isMinor = !0, le(Se, we, Ee, xe), xe.isMinor = ze;
              }
              he(Se, we, Ee, { dfltColor: Xe, bgColor: xe.bgColor, showGrid: xe.showGrid, hasMinor: Ge, attributes: se }), !Ge || we.minor.ticks || we.minor.showgrid || delete we.minor, (we.showline || we.ticks) && Ee("mirror");
              var We, at = $e === "multicategory";
              if (xe.noTickson || $e !== "category" && !at || !we.ticks && !we.showgrid || (at && (We = "boundaries"), Ee("tickson", We) === "boundaries" && delete we.ticklabelposition), at && Ee("showdividers") && (Ee("dividercolor"), Ee("dividerwidth")), $e === "date") if (oe(Se, we, { name: "rangebreaks", inclusionAttr: "enabled", handleItemDefaults: ve }), we.rangebreaks.length) {
                for (var ct = 0; ct < we.rangebreaks.length; ct++) if (we.rangebreaks[ct].pattern === ye) {
                  we._hasDayOfWeekBreaks = !0;
                  break;
                }
                if (pe(we, Le), Le._has("scattergl") || Le._has("splom")) for (var et = 0; et < xe.data.length; et++) {
                  var dt = xe.data[et];
                  dt.type !== "scattergl" && dt.type !== "splom" || (dt.visible = !1, ee.warn(dt.type + " traces do not work on axes with rangebreaks. Setting trace " + dt.index + " to `visible: false`."));
                }
              } else delete we.rangebreaks;
              return we;
            };
            var be = { sun: 1, mon: 2, tue: 3, wed: 4, thu: 5, fri: 6, sat: 7 };
            function Ce(Se) {
              if (typeof Se == "string") return be[Se.substr(0, 3).toLowerCase()];
            }
          }, 80712: function(G, J, Z) {
            var te = Z(87296), ne = te.FORMAT_LINK, ee = te.DATE_FORMAT_LINK;
            function ae(se, ce) {
              return ["Sets the " + se + " formatting rule" + (ce ? "for `" + ce + "` " : ""), "using d3 formatting mini-languages", "which are very similar to those in Python. For numbers, see: " + ne + "."].join(" ");
            }
            function oe(se, ce) {
              return ae(se, ce) + [" And for dates see: " + ee + ".", "We add two items to d3's date formatter:", "*%h* for half of the year as a decimal number as well as", "*%{n}f* for fractional seconds", "with n digits. For example, *2016-10-13 09:15:23.456* with tickformat", "*%H~%M~%S.%2f* would display *09~15~23.46*"].join(" ");
            }
            G.exports = { axisHoverFormat: function(se, ce) {
              return { valType: "string", dflt: "", editType: "none", description: (ce ? ae : oe)("hover text", se) + ["By default the values are formatted using " + (ce ? "generic number format" : "`" + se + "axis.hoverformat`") + "."].join(" ") };
            }, descriptionOnlyNumbers: ae, descriptionWithDates: oe };
          }, 5975: function(G, J, Z) {
            var te = Z(33626), ne = Z(54826);
            function ee(ae, oe) {
              if (oe && oe.length) {
                for (var se = 0; se < oe.length; se++) if (oe[se][ae]) return !0;
              }
              return !1;
            }
            J.id2name = function(ae) {
              if (typeof ae == "string" && ae.match(ne.AX_ID_PATTERN)) {
                var oe = ae.split(" ")[0].substr(1);
                return oe === "1" && (oe = ""), ae.charAt(0) + "axis" + oe;
              }
            }, J.name2id = function(ae) {
              if (ae.match(ne.AX_NAME_PATTERN)) {
                var oe = ae.substr(5);
                return oe === "1" && (oe = ""), ae.charAt(0) + oe;
              }
            }, J.cleanId = function(ae, oe, se) {
              var ce = /( domain)$/.test(ae);
              if (typeof ae == "string" && ae.match(ne.AX_ID_PATTERN) && (!oe || ae.charAt(0) === oe) && (!ce || se)) {
                var le = ae.split(" ")[0].substr(1).replace(/^0+/, "");
                return le === "1" && (le = ""), ae.charAt(0) + le + (ce && se ? " domain" : "");
              }
            }, J.list = function(ae, oe, se) {
              var ce = ae._fullLayout;
              if (!ce) return [];
              var le, ue = J.listIds(ae, oe), de = new Array(ue.length);
              for (le = 0; le < ue.length; le++) {
                var fe = ue[le];
                de[le] = ce[fe.charAt(0) + "axis" + fe.substr(1)];
              }
              if (!se) {
                var he = ce._subplots.gl3d || [];
                for (le = 0; le < he.length; le++) {
                  var me = ce[he[le]];
                  oe ? de.push(me[oe + "axis"]) : de.push(me.xaxis, me.yaxis, me.zaxis);
                }
              }
              return de;
            }, J.listIds = function(ae, oe) {
              var se = ae._fullLayout;
              if (!se) return [];
              var ce = se._subplots;
              return oe ? ce[oe + "axis"] : ce.xaxis.concat(ce.yaxis);
            }, J.getFromId = function(ae, oe, se) {
              var ce = ae._fullLayout;
              return oe = oe === void 0 || typeof oe != "string" ? oe : oe.replace(" domain", ""), se === "x" ? oe = oe.replace(/y[0-9]*/, "") : se === "y" && (oe = oe.replace(/x[0-9]*/, "")), ce[J.id2name(oe)];
            }, J.getFromTrace = function(ae, oe, se) {
              var ce = ae._fullLayout, le = null;
              if (te.traceIs(oe, "gl3d")) {
                var ue = oe.scene;
                ue.substr(0, 5) === "scene" && (le = ce[ue][se + "axis"]);
              } else le = J.getFromId(ae, oe[se + "axis"] || se);
              return le;
            }, J.idSort = function(ae, oe) {
              var se = ae.charAt(0), ce = oe.charAt(0);
              return se !== ce ? se > ce ? 1 : -1 : +(ae.substr(1) || 1) - +(oe.substr(1) || 1);
            }, J.ref2id = function(ae) {
              return !!/^[xyz]/.test(ae) && ae.split(" ")[0];
            }, J.isLinked = function(ae, oe) {
              return ee(oe, ae._axisMatchGroups) || ee(oe, ae._axisConstraintGroups);
            };
          }, 46473: function(G, J, Z) {
            var te = Z(87800).isTypedArraySpec;
            G.exports = function(ne, ee, ae, oe) {
              if (ee.type === "category") {
                var se, ce = ne.categoryarray, le = Array.isArray(ce) && ce.length > 0 || te(ce);
                le && (se = "array");
                var ue, de = ae("categoryorder", se);
                de === "array" && (ue = ae("categoryarray")), le || de !== "array" || (de = ee.categoryorder = "trace"), de === "trace" ? ee._initialCategories = [] : de === "array" ? ee._initialCategories = ue.slice() : (ue = function(fe, he) {
                  var me, pe, ye, ge = he.dataAttr || fe._id.charAt(0), ve = {};
                  if (he.axData) me = he.axData;
                  else for (me = [], pe = 0; pe < he.data.length; pe++) {
                    var be = he.data[pe];
                    be[ge + "axis"] === fe._id && me.push(be);
                  }
                  for (pe = 0; pe < me.length; pe++) {
                    var Ce = me[pe][ge];
                    for (ye = 0; ye < Ce.length; ye++) {
                      var Se = Ce[ye];
                      Se != null && (ve[Se] = 1);
                    }
                  }
                  return Object.keys(ve);
                }(ee, oe).sort(), de === "category ascending" ? ee._initialCategories = ue : de === "category descending" && (ee._initialCategories = ue.reverse()));
              }
            };
          }, 68599: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(63821), ae = ee.ONEDAY, oe = ee.ONEWEEK;
            J.dtick = function(se, ce) {
              var le = ce === "log", ue = ce === "date", de = ce === "category", fe = ue ? ae : 1;
              if (!se) return fe;
              if (te(se)) return (se = Number(se)) <= 0 ? fe : de ? Math.max(1, Math.round(se)) : ue ? Math.max(0.1, se) : se;
              if (typeof se != "string" || !ue && !le) return fe;
              var he = se.charAt(0), me = se.substr(1);
              return (me = te(me) ? Number(me) : 0) <= 0 || !(ue && he === "M" && me === Math.round(me) || le && he === "L" || le && he === "D" && (me === 1 || me === 2)) ? fe : se;
            }, J.tick0 = function(se, ce, le, ue) {
              return ce === "date" ? ne.cleanDate(se, ne.dateTick0(le, ue % oe == 0 ? 1 : 0)) : ue !== "D1" && ue !== "D2" ? te(se) ? Number(se) : 0 : void 0;
            };
          }, 54826: function(G, J, Z) {
            var te = Z(90694).counter;
            G.exports = { idRegex: { x: te("x", "( domain)?"), y: te("y", "( domain)?") }, attrRegex: te("[xy]axis"), xAxisMatch: te("xaxis"), yAxisMatch: te("yaxis"), AX_ID_PATTERN: /^[xyz][0-9]*( domain)?$/, AX_NAME_PATTERN: /^[xyz]axis[0-9]*$/, SUBPLOT_PATTERN: /^x([0-9]*)y([0-9]*)$/, HOUR_PATTERN: "hour", WEEKDAY_PATTERN: "day of week", MINDRAG: 8, MINZOOM: 20, DRAGGERSIZE: 20, REDRAWDELAY: 50, DFLTRANGEX: [-1, 6], DFLTRANGEY: [-1, 4], traceLayerClasses: ["imagelayer", "heatmaplayer", "contourcarpetlayer", "contourlayer", "funnellayer", "waterfalllayer", "barlayer", "carpetlayer", "violinlayer", "boxlayer", "ohlclayer", "scattercarpetlayer", "scatterlayer"], clipOnAxisFalseQuery: [".scatterlayer", ".barlayer", ".funnellayer", ".waterfalllayer"], layerValue2layerClass: { "above traces": "above", "below traces": "below" }, zindexSeparator: "z" };
          }, 84391: function(G, J, Z) {
            var te = Z(34809), ne = Z(32919), ee = Z(5975).id2name, ae = Z(25829), oe = Z(67611), se = Z(19091), ce = Z(63821).ALMOST_EQUAL, le = Z(4530).FROM_BL;
            function ue(ge, ve, be) {
              var Ce = be.axIds, Se = be.layoutOut, we = be.hasImage, Ee = Se._axisConstraintGroups, xe = Se._axisMatchGroups, Le = ve._id, Qe = Le.charAt(0), ke = ((Se._splomAxes || {})[Qe] || {})[Le] || {}, Me = ve._id, Re = Me.charAt(0) === "x";
              function De(Je, it) {
                return te.coerce(ge, ve, ae, Je, it);
              }
              ve._matchGroup = null, ve._constraintGroup = null, De("constrain", we ? "domain" : "range"), te.coerce(ge, ve, { constraintoward: { valType: "enumerated", values: Re ? ["left", "center", "right"] : ["bottom", "middle", "top"], dflt: Re ? "center" : "middle" } }, "constraintoward");
              var Pe, $e, qe = ve.type, Xe = [];
              for (Pe = 0; Pe < Ce.length; Pe++) ($e = Ce[Pe]) !== Me && Se[ee($e)].type === qe && Xe.push($e);
              var Ve = fe(Ee, Me);
              if (Ve) {
                var Ze = [];
                for (Pe = 0; Pe < Xe.length; Pe++) Ve[$e = Xe[Pe]] || Ze.push($e);
                Xe = Ze;
              }
              var Ge, ze, We = Xe.length;
              We && (ge.matches || ke.matches) && (Ge = te.coerce(ge, ve, { matches: { valType: "enumerated", values: Xe, dflt: Xe.indexOf(ke.matches) !== -1 ? ke.matches : void 0 } }, "matches"));
              var at = we && !Re ? ve.anchor : void 0;
              if (We && !Ge && (ge.scaleanchor || at) && (ze = te.coerce(ge, ve, { scaleanchor: { valType: "enumerated", values: Xe.concat([!1]) } }, "scaleanchor", at)), Ge) {
                ve._matchGroup = he(xe, Me, Ge, 1);
                var ct = Se[ee(Ge)], et = de(Se, ve) / de(Se, ct);
                Re !== (Ge.charAt(0) === "x") && (et = (Re ? "x" : "y") + et), he(Ee, Me, Ge, et);
              } else ge.matches && Ce.indexOf(ge.matches) !== -1 && te.warn("ignored " + ve._name + '.matches: "' + ge.matches + '" to avoid an infinite loop');
              if (ze) {
                var dt = De("scaleratio");
                dt || (dt = ve.scaleratio = 1), he(Ee, Me, ze, dt);
              } else ge.scaleanchor && Ce.indexOf(ge.scaleanchor) !== -1 && te.warn("ignored " + ve._name + '.scaleanchor: "' + ge.scaleanchor + '" to avoid either an infinite loop and possibly inconsistent scaleratios, or because this axis declares a *matches* constraint.');
            }
            function de(ge, ve) {
              var be = ve.domain;
              return be || (be = ge[ee(ve.overlaying)].domain), be[1] - be[0];
            }
            function fe(ge, ve) {
              for (var be = 0; be < ge.length; be++) if (ge[be][ve]) return ge[be];
              return null;
            }
            function he(ge, ve, be, Ce) {
              var Se, we, Ee, xe, Le, Qe = fe(ge, ve);
              Qe === null ? ((Qe = {})[ve] = 1, Le = ge.length, ge.push(Qe)) : Le = ge.indexOf(Qe);
              var ke = Object.keys(Qe);
              for (Se = 0; Se < ge.length; Se++) if (Ee = ge[Se], Se !== Le && Ee[be]) {
                var Me = Ee[be];
                for (we = 0; we < ke.length; we++) Ee[xe = ke[we]] = me(Me, me(Ce, Qe[xe]));
                return void ge.splice(Le, 1);
              }
              if (Ce !== 1) for (we = 0; we < ke.length; we++) {
                var Re = ke[we];
                Qe[Re] = me(Ce, Qe[Re]);
              }
              Qe[be] = 1;
            }
            function me(ge, ve) {
              var be, Ce, Se = "", we = "";
              typeof ge == "string" && (be = (Se = ge.match(/^[xy]*/)[0]).length, ge = +ge.substr(be)), typeof ve == "string" && (Ce = (we = ve.match(/^[xy]*/)[0]).length, ve = +ve.substr(Ce));
              var Ee = ge * ve;
              return be || Ce ? be && Ce && Se.charAt(0) !== we.charAt(0) ? be === Ce ? Ee : (be > Ce ? Se.substr(Ce) : we.substr(be)) + Ee : Se + we + ge * ve : Ee;
            }
            function pe(ge, ve) {
              for (var be = ve._size, Ce = be.h / be.w, Se = {}, we = Object.keys(ge), Ee = 0; Ee < we.length; Ee++) {
                var xe = we[Ee], Le = ge[xe];
                if (typeof Le == "string") {
                  var Qe = Le.match(/^[xy]*/)[0], ke = Qe.length;
                  Le = +Le.substr(ke);
                  for (var Me = Qe.charAt(0) === "y" ? Ce : 1 / Ce, Re = 0; Re < ke; Re++) Le *= Me;
                }
                Se[xe] = Le;
              }
              return Se;
            }
            function ye(ge, ve) {
              var be = ge._inputDomain, Ce = le[ge.constraintoward], Se = be[0] + (be[1] - be[0]) * Ce;
              ge.domain = ge._input.domain = [Se + (be[0] - Se) / ve, Se + (be[1] - Se) / ve], ge.setScale();
            }
            J.handleDefaults = function(ge, ve, be) {
              var Ce, Se, we, Ee, xe, Le, Qe, ke, Me = be.axIds, Re = be.axHasImage, De = ve._axisConstraintGroups = [], Pe = ve._axisMatchGroups = [];
              for (Ce = 0; Ce < Me.length; Ce++) ue(xe = ge[Ee = ee(Me[Ce])], Le = ve[Ee], { axIds: Me, layoutOut: ve, hasImage: Re[Ee] });
              function $e(ct, et) {
                for (Ce = 0; Ce < ct.length; Ce++) for (we in Se = ct[Ce]) ve[ee(we)][et] = Se;
              }
              for ($e(Pe, "_matchGroup"), Ce = 0; Ce < De.length; Ce++) for (we in Se = De[Ce]) if ((Le = ve[ee(we)]).fixedrange) {
                for (var qe in Se) {
                  var Xe = ee(qe);
                  (ge[Xe] || {}).fixedrange === !1 && te.warn("fixedrange was specified as false for axis " + Xe + " but was overridden because another axis in its constraint group has fixedrange true"), ve[Xe].fixedrange = !0;
                }
                break;
              }
              for (Ce = 0; Ce < De.length; ) {
                for (we in Se = De[Ce]) {
                  (Le = ve[ee(we)])._matchGroup && Object.keys(Le._matchGroup).length === Object.keys(Se).length && (De.splice(Ce, 1), Ce--);
                  break;
                }
                Ce++;
              }
              $e(De, "_constraintGroup");
              var Ve = ["constrain", "range", "autorange", "rangemode", "rangebreaks", "categoryorder", "categoryarray"], Ze = !1, Ge = !1;
              function ze() {
                ke = Le[Qe], Qe === "rangebreaks" && (Ge = Le._hasDayOfWeekBreaks);
              }
              for (Ce = 0; Ce < Pe.length; Ce++) {
                Se = Pe[Ce];
                for (var We = 0; We < Ve.length; We++) {
                  var at;
                  for (we in Qe = Ve[We], ke = null, Se) if (xe = ge[Ee = ee(we)], Le = ve[Ee], Qe in Le) {
                    if (!Le.matches && (at = Le, Qe in xe)) {
                      ze();
                      break;
                    }
                    ke === null && Qe in xe && ze();
                  }
                  if (Qe === "range" && ke && xe.range && xe.range.length === 2 && xe.range[0] !== null && xe.range[1] !== null && (Ze = !0), Qe === "autorange" && ke === null && Ze && (ke = !1), ke === null && Qe in at && (ke = at[Qe]), ke !== null) for (we in Se) (Le = ve[ee(we)])[Qe] = Qe === "range" ? ke.slice() : ke, Qe === "rangebreaks" && (Le._hasDayOfWeekBreaks = Ge, se(Le, ve));
                }
              }
            }, J.enforce = function(ge) {
              var ve, be, Ce, Se, we, Ee, xe, Le, Qe = ge._fullLayout, ke = Qe._axisConstraintGroups || [];
              for (ve = 0; ve < ke.length; ve++) {
                Ce = pe(ke[ve], Qe);
                var Me = Object.keys(Ce), Re = 1 / 0, De = 0, Pe = 1 / 0, $e = {}, qe = {}, Xe = !1;
                for (be = 0; be < Me.length; be++) qe[Se = Me[be]] = we = Qe[ee(Se)], we._inputDomain ? we.domain = we._inputDomain.slice() : we._inputDomain = we.domain.slice(), we._inputRange || (we._inputRange = we.range.slice()), we.setScale(), $e[Se] = Ee = Math.abs(we._m) / Ce[Se], Re = Math.min(Re, Ee), we.constrain !== "domain" && we._constraintShrinkable || (Pe = Math.min(Pe, Ee)), delete we._constraintShrinkable, De = Math.max(De, Ee), we.constrain === "domain" && (Xe = !0);
                if (!(Re > ce * De) || Xe) {
                  for (be = 0; be < Me.length; be++) if (Ee = $e[Se = Me[be]], xe = (we = qe[Se]).constrain, Ee !== Pe || xe === "domain") if (Le = Ee / Pe, xe === "range") oe(we, Le);
                  else {
                    var Ve = we._inputDomain, Ze = (we.domain[1] - we.domain[0]) / (Ve[1] - Ve[0]), Ge = (we.r2l(we.range[1]) - we.r2l(we.range[0])) / (we.r2l(we._inputRange[1]) - we.r2l(we._inputRange[0]));
                    if ((Le /= Ze) * Ge < 1) {
                      we.domain = we._input.domain = Ve.slice(), oe(we, Le);
                      continue;
                    }
                    if (Ge < 1 && (we.range = we._input.range = we._inputRange.slice(), Le *= Ge), we.autorange) {
                      var ze = we.r2l(we.range[0]), We = we.r2l(we.range[1]), at = (ze + We) / 2, ct = at, et = at, dt = Math.abs(We - at), Je = at - dt * Le * 1.0001, it = at + dt * Le * 1.0001, st = ne.makePadFn(Qe, we, 0), gt = ne.makePadFn(Qe, we, 1);
                      ye(we, Le);
                      var bt, yt, St = Math.abs(we._m), Lt = ne.concatExtremes(ge, we), Rt = Lt.min, vt = Lt.max;
                      for (yt = 0; yt < Rt.length; yt++) (bt = Rt[yt].val - st(Rt[yt]) / St) > Je && bt < ct && (ct = bt);
                      for (yt = 0; yt < vt.length; yt++) (bt = vt[yt].val + gt(vt[yt]) / St) < it && bt > et && (et = bt);
                      Le /= (et - ct) / (2 * dt), ct = we.l2r(ct), et = we.l2r(et), we.range = we._input.range = ze < We ? [ct, et] : [et, ct];
                    }
                    ye(we, Le);
                  }
                }
              }
            }, J.getAxisGroup = function(ge, ve) {
              for (var be = ge._axisMatchGroups, Ce = 0; Ce < be.length; Ce++) if (be[Ce][ve]) return "g" + Ce;
              return ve;
            }, J.clean = function(ge, ve) {
              if (ve._inputDomain) {
                for (var be = !1, Ce = ve._id, Se = ge._fullLayout._axisConstraintGroups, we = 0; we < Se.length; we++) if (Se[we][Ce]) {
                  be = !0;
                  break;
                }
                be && ve.constrain === "domain" || (ve._input.domain = ve.domain = ve._inputDomain, delete ve._inputDomain);
              }
            };
          }, 51680: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = ne.numberFormat, ae = Z(65657), oe = Z(74043), se = Z(33626), ce = ne.strTranslate, le = Z(30635), ue = Z(78766), de = Z(62203), fe = Z(32141), he = Z(29714), me = Z(27983), pe = Z(14751), ye = Z(70414), ge = ye.selectingOrDrawing, ve = ye.freeMode, be = Z(4530).FROM_TL, Ce = Z(34823), Se = Z(71817).redrawReglTraces, we = Z(44122), Ee = Z(5975).getFromId, xe = Z(44844).prepSelect, Le = Z(44844).clearOutline, Qe = Z(44844).selectOnClick, ke = Z(67611), Me = Z(54826), Re = Me.MINDRAG, De = Me.MINZOOM, Pe = !0;
            function $e(bt, yt, St, Lt) {
              var Rt = ne.ensureSingle(bt.draglayer, yt, St, function(vt) {
                vt.classed("drag", !0).style({ fill: "transparent", "stroke-width": 0 }).attr("data-subplot", bt.id);
              });
              return Rt.call(me, Lt), Rt.node();
            }
            function qe(bt, yt, St, Lt, Rt, vt, $t) {
              var Pt = $e(bt, "rect", yt, St);
              return te.select(Pt).call(de.setRect, Lt, Rt, vt, $t), Pt;
            }
            function Xe(bt, yt) {
              for (var St = 0; St < bt.length; St++) if (!bt[St].fixedrange) return yt;
              return "";
            }
            function Ve(bt, yt, St, Lt, Rt) {
              for (var vt = 0; vt < bt.length; vt++) {
                var $t = bt[vt];
                if (!$t.fixedrange) if ($t.rangebreaks) {
                  var Pt = $t._id.charAt(0) === "y", Xt = Pt ? 1 - yt : yt, Nt = Pt ? 1 - St : St;
                  Lt[$t._name + ".range[0]"] = $t.l2r($t.p2l(Xt * $t._length)), Lt[$t._name + ".range[1]"] = $t.l2r($t.p2l(Nt * $t._length));
                } else {
                  var Gt = $t._rl[0], Ft = $t._rl[1] - Gt;
                  Lt[$t._name + ".range[0]"] = $t.l2r(Gt + Ft * yt), Lt[$t._name + ".range[1]"] = $t.l2r(Gt + Ft * St);
                }
              }
              if (Rt && Rt.length) {
                var Tt = (yt + (1 - St)) / 2;
                Ve(Rt, Tt, 1 - Tt, Lt, []);
              }
            }
            function Ze(bt, yt) {
              for (var St = 0; St < bt.length; St++) {
                var Lt = bt[St];
                if (!Lt.fixedrange) {
                  if (Lt.rangebreaks) {
                    var Rt = Lt._length, vt = (Lt.p2l(0 + yt) - Lt.p2l(0) + (Lt.p2l(Rt + yt) - Lt.p2l(Rt))) / 2;
                    Lt.range = [Lt.l2r(Lt._rl[0] - vt), Lt.l2r(Lt._rl[1] - vt)];
                  } else Lt.range = [Lt.l2r(Lt._rl[0] - yt / Lt._m), Lt.l2r(Lt._rl[1] - yt / Lt._m)];
                  Lt.limitRange && Lt.limitRange();
                }
              }
            }
            function Ge(bt) {
              return 1 - (bt >= 0 ? Math.min(bt, 0.9) : 1 / (1 / Math.max(bt, -0.3) + 3.222));
            }
            function ze(bt, yt, St, Lt, Rt) {
              return bt.append("path").attr("class", "zoombox").style({ fill: yt > 0.2 ? "rgba(0,0,0,0)" : "rgba(255,255,255,0)", "stroke-width": 0 }).attr("transform", ce(St, Lt)).attr("d", Rt + "Z");
            }
            function We(bt, yt, St) {
              return bt.append("path").attr("class", "zoombox-corners").style({ fill: ue.background, stroke: ue.defaultLine, "stroke-width": 1, opacity: 0 }).attr("transform", ce(yt, St)).attr("d", "M0,0Z");
            }
            function at(bt, yt, St, Lt, Rt, vt) {
              bt.attr("d", Lt + "M" + St.l + "," + St.t + "v" + St.h + "h" + St.w + "v-" + St.h + "h-" + St.w + "Z"), ct(bt, yt, Rt, vt);
            }
            function ct(bt, yt, St, Lt) {
              St || (bt.transition().style("fill", Lt > 0.2 ? "rgba(0,0,0,0.4)" : "rgba(255,255,255,0.3)").duration(200), yt.transition().style("opacity", 1).duration(200));
            }
            function et(bt) {
              te.select(bt).selectAll(".zoombox,.js-zoombox-backdrop,.js-zoombox-menu,.zoombox-corners").remove();
            }
            function dt(bt) {
              Pe && bt.data && bt._context.showTips && (ne.notifier(ne._(bt, "Double-click to zoom back out"), "long"), Pe = !1);
            }
            function Je(bt) {
              var yt = Math.floor(Math.min(bt.b - bt.t, bt.r - bt.l, De) / 2);
              return "M" + (bt.l - 3.5) + "," + (bt.t - 0.5 + yt) + "h3v" + -yt + "h" + yt + "v-3h-" + (yt + 3) + "ZM" + (bt.r + 3.5) + "," + (bt.t - 0.5 + yt) + "h-3v" + -yt + "h" + -yt + "v-3h" + (yt + 3) + "ZM" + (bt.r + 3.5) + "," + (bt.b + 0.5 - yt) + "h-3v" + yt + "h" + -yt + "v3h" + (yt + 3) + "ZM" + (bt.l - 3.5) + "," + (bt.b + 0.5 - yt) + "h3v" + yt + "h" + yt + "v3h-" + (yt + 3) + "Z";
            }
            function it(bt, yt, St, Lt, Rt) {
              for (var vt, $t, Pt, Xt, Nt = !1, Gt = {}, Ft = {}, Tt = (Rt || {}).xaHash, It = (Rt || {}).yaHash, Ct = 0; Ct < yt.length; Ct++) {
                var Kt = yt[Ct];
                for (vt in St) if (Kt[vt]) {
                  for (Pt in Kt) Rt && (Tt[Pt] || It[Pt]) || (Pt.charAt(0) === "x" ? St : Lt)[Pt] || (Gt[Pt] = vt);
                  for ($t in Lt) Rt && (Tt[$t] || It[$t]) || !Kt[$t] || (Nt = !0);
                }
                for ($t in Lt) if (Kt[$t]) for (Xt in Kt) Rt && (Tt[Xt] || It[Xt]) || (Xt.charAt(0) === "x" ? St : Lt)[Xt] || (Ft[Xt] = $t);
              }
              Nt && (ne.extendFlat(Gt, Ft), Ft = {});
              var Vt = {}, Yt = [];
              for (Pt in Gt) {
                var sn = Ee(bt, Pt);
                Yt.push(sn), Vt[sn._id] = sn;
              }
              var hn = {}, Qt = [];
              for (Xt in Ft) {
                var Ut = Ee(bt, Xt);
                Qt.push(Ut), hn[Ut._id] = Ut;
              }
              return { xaHash: Vt, yaHash: hn, xaxes: Yt, yaxes: Qt, xLinks: Gt, yLinks: Ft, isSubplotConstrained: Nt };
            }
            function st(bt, yt) {
              if (oe) {
                var St = bt.onwheel !== void 0 ? "wheel" : "mousewheel";
                bt._onwheel && bt.removeEventListener(St, bt._onwheel), bt._onwheel = yt, bt.addEventListener(St, yt, { passive: !1 });
              } else bt.onwheel !== void 0 ? bt.onwheel = yt : bt.onmousewheel !== void 0 ? bt.onmousewheel = yt : bt.isAddedWheelEvent || (bt.isAddedWheelEvent = !0, bt.addEventListener("wheel", yt, { passive: !1 }));
            }
            function gt(bt) {
              var yt = [];
              for (var St in bt) yt.push(bt[St]);
              return yt;
            }
            G.exports = { makeDragBox: function(bt, yt, St, Lt, Rt, vt, $t, Pt) {
              var Xt, Nt, Gt, Ft, Tt, It, Ct, Kt, Vt, Yt, sn, hn, Qt, Ut, Wt, rn, vn, dn, Ht, jt, cn, wn, Rn, Pn = bt._fullLayout._zoomlayer, Ie = $t + Pt === "nsew", rt = ($t + Pt).length === 1;
              function ft() {
                if (Xt = yt.xaxis, Nt = yt.yaxis, Vt = Xt._length, Yt = Nt._length, Ct = Xt._offset, Kt = Nt._offset, (Gt = {})[Xt._id] = Xt, (Ft = {})[Nt._id] = Nt, $t && Pt) for (var Zr = yt.overlays, fi = 0; fi < Zr.length; fi++) {
                  var ri = Zr[fi].xaxis;
                  Gt[ri._id] = ri;
                  var bi = Zr[fi].yaxis;
                  Ft[bi._id] = bi;
                }
                Tt = gt(Gt), It = gt(Ft), Qt = Xe(Tt, Pt), Ut = Xe(It, $t), Wt = !Ut && !Qt, hn = it(bt, bt._fullLayout._axisMatchGroups, Gt, Ft);
                var Zi = (sn = it(bt, bt._fullLayout._axisConstraintGroups, Gt, Ft, hn)).isSubplotConstrained || hn.isSubplotConstrained;
                rn = Pt || Zi, vn = $t || Zi;
                var $i = bt._fullLayout;
                dn = $i._has("scattergl"), Ht = $i._has("splom"), jt = $i._has("svg");
              }
              St += yt.yaxis._shift, ft();
              var pt = function(Zr, fi, ri) {
                return Zr ? Zr === "nsew" ? ri ? "" : fi === "pan" ? "move" : "crosshair" : Zr.toLowerCase() + "-resize" : "pointer";
              }(Ut + Qt, bt._fullLayout.dragmode, Ie), wt = qe(yt, $t + Pt + "drag", pt, St, Lt, Rt, vt);
              if (Wt && !Ie) return wt.onmousedown = null, wt.style.pointerEvents = "none", wt;
              var Bt, At, Dt, Ot, un, yn, _n, gn, bn, an, Zt = { element: wt, gd: bt, plotinfo: yt };
              function Mn() {
                Zt.plotinfo.selection = !1, Le(bt);
              }
              function Ln(Zr, fi) {
                var ri = Zt.gd;
                if (ri._fullLayout._activeShapeIndex >= 0) ri._fullLayout._deactivateShape(ri);
                else {
                  var bi = ri._fullLayout.clickmode;
                  if (et(ri), Zr !== 2 || rt || Yr(), Ie) bi.indexOf("select") > -1 && Qe(fi, ri, Tt, It, yt.id, Zt), bi.indexOf("event") > -1 && fe.click(ri, fi, yt.id);
                  else if (Zr === 1 && rt) {
                    var Zi = $t ? Nt : Xt, $i = $t === "s" || Pt === "w" ? 0 : 1, Br = Zi._name + ".range[" + $i + "]", Pr = function(oa, ka) {
                      var aa, hi = oa.range[ka], Gr = Math.abs(hi - oa.range[1 - ka]);
                      return oa.type === "date" ? hi : oa.type === "log" ? (aa = Math.ceil(Math.max(0, -Math.log(Gr) / Math.LN10)) + 3, ee("." + aa + "g")(Math.pow(10, hi))) : (aa = Math.floor(Math.log(Math.abs(hi)) / Math.LN10) - Math.floor(Math.log(Gr) / Math.LN10) + 4, ee("." + String(aa) + "g")(hi));
                    }(Zi, $i), ei = "left", Ci = "middle";
                    if (Zi.fixedrange) return;
                    $t ? (Ci = $t === "n" ? "top" : "bottom", Zi.side === "right" && (ei = "right")) : Pt === "e" && (ei = "right"), ri._context.showAxisRangeEntryBoxes && te.select(wt).call(le.makeEditable, { gd: ri, immediate: !0, background: ri._fullLayout.paper_bgcolor, text: String(Pr), fill: Zi.tickfont ? Zi.tickfont.color : "#444", horizontalAlign: ei, verticalAlign: Ci }).on("edit", function(oa) {
                      var ka = Zi.d2r(oa);
                      ka !== void 0 && se.call("_guiRelayout", ri, Br, ka);
                    });
                  }
                }
              }
              function In(Zr, fi) {
                if (bt._transitioningWithDuration) return !1;
                var ri = Math.max(0, Math.min(Vt, wn * Zr + Bt)), bi = Math.max(0, Math.min(Yt, Rn * fi + At)), Zi = Math.abs(ri - Bt), $i = Math.abs(bi - At);
                function Br() {
                  _n = "", Dt.r = Dt.l, Dt.t = Dt.b, bn.attr("d", "M0,0Z");
                }
                if (Dt.l = Math.min(Bt, ri), Dt.r = Math.max(Bt, ri), Dt.t = Math.min(At, bi), Dt.b = Math.max(At, bi), sn.isSubplotConstrained) Zi > De || $i > De ? (_n = "xy", Zi / Vt > $i / Yt ? ($i = Zi * Yt / Vt, At > bi ? Dt.t = At - $i : Dt.b = At + $i) : (Zi = $i * Vt / Yt, Bt > ri ? Dt.l = Bt - Zi : Dt.r = Bt + Zi), bn.attr("d", Je(Dt))) : Br();
                else if (hn.isSubplotConstrained) if (Zi > De || $i > De) {
                  _n = "xy";
                  var Pr = Math.min(Dt.l / Vt, (Yt - Dt.b) / Yt), ei = Math.max(Dt.r / Vt, (Yt - Dt.t) / Yt);
                  Dt.l = Pr * Vt, Dt.r = ei * Vt, Dt.b = (1 - Pr) * Yt, Dt.t = (1 - ei) * Yt, bn.attr("d", Je(Dt));
                } else Br();
                else !Ut || $i < Math.min(Math.max(0.6 * Zi, Re), De) ? Zi < Re || !Qt ? Br() : (Dt.t = 0, Dt.b = Yt, _n = "x", bn.attr("d", function(Ci, oa) {
                  return "M" + (Ci.l - 0.5) + "," + (oa - De - 0.5) + "h-3v" + (2 * De + 1) + "h3ZM" + (Ci.r + 0.5) + "," + (oa - De - 0.5) + "h3v" + (2 * De + 1) + "h-3Z";
                }(Dt, At))) : !Qt || Zi < Math.min(0.6 * $i, De) ? (Dt.l = 0, Dt.r = Vt, _n = "y", bn.attr("d", function(Ci, oa) {
                  return "M" + (oa - De - 0.5) + "," + (Ci.t - 0.5) + "v-3h" + (2 * De + 1) + "v3ZM" + (oa - De - 0.5) + "," + (Ci.b + 0.5) + "v3h" + (2 * De + 1) + "v-3Z";
                }(Dt, Bt))) : (_n = "xy", bn.attr("d", Je(Dt)));
                Dt.w = Dt.r - Dt.l, Dt.h = Dt.b - Dt.t, _n && (an = !0), bt._dragged = an, at(gn, bn, Dt, un, yn, Ot), kn(), bt.emit("plotly_relayouting", cn), yn = !0;
              }
              function kn() {
                cn = {}, _n !== "xy" && _n !== "x" || (Ve(Tt, Dt.l / Vt, Dt.r / Vt, cn, sn.xaxes), mr("x", cn)), _n !== "xy" && _n !== "y" || (Ve(It, (Yt - Dt.b) / Yt, (Yt - Dt.t) / Yt, cn, sn.yaxes), mr("y", cn));
              }
              function er() {
                kn(), et(bt), jr(), dt(bt);
              }
              Zt.prepFn = function(Zr, fi, ri) {
                var bi = Zt.dragmode, Zi = bt._fullLayout.dragmode;
                Zi !== bi && (Zt.dragmode = Zi), ft(), wn = bt._fullLayout._invScaleX, Rn = bt._fullLayout._invScaleY, Wt || (Ie ? Zr.shiftKey ? Zi === "pan" ? Zi = "zoom" : ge(Zi) || (Zi = "pan") : Zr.ctrlKey && (Zi = "pan") : Zi = "pan"), ve(Zi) ? Zt.minDrag = 1 : Zt.minDrag = void 0, ge(Zi) ? (Zt.xaxes = Tt, Zt.yaxes = It, xe(Zr, fi, ri, Zt, Zi)) : (Zt.clickFn = Ln, ge(bi) && Mn(), Wt || (Zi === "zoom" ? (Zt.moveFn = In, Zt.doneFn = er, Zt.minDrag = 1, function($i, Br, Pr) {
                  var ei = wt.getBoundingClientRect();
                  Bt = Br - ei.left, At = Pr - ei.top, bt._fullLayout._calcInverseTransform(bt);
                  var Ci = ne.apply3DTransform(bt._fullLayout._invTransform)(Bt, At);
                  Bt = Ci[0], At = Ci[1], Dt = { l: Bt, r: Bt, w: 0, t: At, b: At, h: 0 }, Ot = bt._hmpixcount ? bt._hmlumcount / bt._hmpixcount : ae(bt._fullLayout.plot_bgcolor).getLuminance(), yn = !1, _n = "xy", an = !1, gn = ze(Pn, Ot, Ct, Kt, un = "M0,0H" + Vt + "V" + Yt + "H0V0"), bn = We(Pn, Ct, Kt);
                }(0, fi, ri)) : Zi === "pan" && (Zt.moveFn = Lr, Zt.doneFn = jr))), bt._fullLayout._redrag = function() {
                  var $i = bt._dragdata;
                  if ($i && $i.element === wt) {
                    var Br = bt._fullLayout.dragmode;
                    ge(Br) || (ft(), Mr([0, 0, Vt, Yt]), Zt.moveFn($i.dx, $i.dy));
                  }
                };
              }, pe.init(Zt);
              var jn = [0, 0, Vt, Yt], ur = null, Kn = Me.REDRAWDELAY, Jn = yt.mainplot ? bt._fullLayout._plots[yt.mainplot] : yt;
              function Lr(Zr, fi) {
                if (Zr *= wn, fi *= Rn, !bt._transitioningWithDuration) {
                  if (bt._fullLayout._replotting = !0, Qt === "ew" || Ut === "ns") {
                    var ri = Qt ? -Zr : 0, bi = Ut ? -fi : 0;
                    if (hn.isSubplotConstrained) {
                      if (Qt && Ut) {
                        var Zi = (Zr / Vt - fi / Yt) / 2;
                        ri = -(Zr = Zi * Vt), bi = -(fi = -Zi * Yt);
                      }
                      Ut ? ri = -bi * Vt / Yt : bi = -ri * Yt / Vt;
                    }
                    return Qt && (Ze(Tt, Zr), mr("x")), Ut && (Ze(It, fi), mr("y")), Mr([ri, bi, Vt, Yt]), Ar(), void bt.emit("plotly_relayouting", cn);
                  }
                  var $i, Br, Pr = Qt === "w" == (Ut === "n") ? 1 : -1;
                  if (Qt && Ut && (sn.isSubplotConstrained || hn.isSubplotConstrained)) {
                    var ei = (Zr / Vt + Pr * fi / Yt) / 2;
                    Zr = ei * Vt, fi = Pr * ei * Yt;
                  }
                  if (Qt === "w" ? Zr = aa(Tt, 0, Zr) : Qt === "e" ? Zr = aa(Tt, 1, -Zr) : Qt || (Zr = 0), Ut === "n" ? fi = aa(It, 1, fi) : Ut === "s" ? fi = aa(It, 0, -fi) : Ut || (fi = 0), $i = Qt === "w" ? Zr : 0, Br = Ut === "n" ? fi : 0, sn.isSubplotConstrained && !hn.isSubplotConstrained || hn.isSubplotConstrained && Qt && Ut && Pr > 0) {
                    var Ci;
                    if (hn.isSubplotConstrained || !Qt && Ut.length === 1) {
                      for (Ci = 0; Ci < Tt.length; Ci++) Tt[Ci].range = Tt[Ci]._r.slice(), ke(Tt[Ci], 1 - fi / Yt);
                      $i = (Zr = fi * Vt / Yt) / 2;
                    }
                    if (hn.isSubplotConstrained || !Ut && Qt.length === 1) {
                      for (Ci = 0; Ci < It.length; Ci++) It[Ci].range = It[Ci]._r.slice(), ke(It[Ci], 1 - Zr / Vt);
                      Br = (fi = Zr * Yt / Vt) / 2;
                    }
                  }
                  hn.isSubplotConstrained && Ut || mr("x"), hn.isSubplotConstrained && Qt || mr("y");
                  var oa = Vt - Zr, ka = Yt - fi;
                  !hn.isSubplotConstrained || Qt && Ut || (Qt ? (Br = $i ? 0 : Zr * Yt / Vt, ka = oa * Yt / Vt) : ($i = Br ? 0 : fi * Vt / Yt, oa = ka * Vt / Yt)), Mr([$i, Br, oa, ka]), Ar(), bt.emit("plotly_relayouting", cn);
                }
                function aa(hi, Gr, Ca) {
                  for (var Sr, Dr, si = 1 - Gr, zr = 0; zr < hi.length; zr++) {
                    var di = hi[zr];
                    if (!di.fixedrange) {
                      Sr = di, Dr = di._rl[si] + (di._rl[Gr] - di._rl[si]) / Ge(Ca / di._length);
                      var Da = di.l2r(Dr);
                      Da !== !1 && Da !== void 0 && (di.range[Gr] = Da);
                    }
                  }
                  return Sr._length * (Sr._rl[Gr] - Dr) / (Sr._rl[Gr] - Sr._rl[si]);
                }
              }
              function mr(Zr, fi) {
                for (var ri = hn.isSubplotConstrained ? { x: It, y: Tt }[Zr] : hn[Zr + "axes"], bi = hn.isSubplotConstrained ? { x: Tt, y: It }[Zr] : [], Zi = 0; Zi < ri.length; Zi++) {
                  var $i = ri[Zi], Br = $i._id, Pr = hn.xLinks[Br] || hn.yLinks[Br], ei = bi[0] || Gt[Pr] || Ft[Pr];
                  ei && (fi ? (fi[$i._name + ".range[0]"] = fi[ei._name + ".range[0]"], fi[$i._name + ".range[1]"] = fi[ei._name + ".range[1]"]) : $i.range = ei.range.slice());
                }
              }
              function Ar() {
                var Zr, fi = [];
                function ri(Br) {
                  for (Zr = 0; Zr < Br.length; Zr++) Br[Zr].fixedrange || fi.push(Br[Zr]._id);
                }
                function bi(Br, Pr) {
                  for (Zr = 0; Zr < Br.length; Zr++) {
                    var ei = Br[Zr], Ci = ei[Pr];
                    ei.fixedrange || Ci.tickmode !== "sync" || fi.push(Ci._id);
                  }
                }
                for (rn && (ri(Tt), ri(sn.xaxes), ri(hn.xaxes), bi(yt.overlays, "xaxis")), vn && (ri(It), ri(sn.yaxes), ri(hn.yaxes), bi(yt.overlays, "yaxis")), cn = {}, Zr = 0; Zr < fi.length; Zr++) {
                  var Zi = fi[Zr], $i = Ee(bt, Zi);
                  he.drawOne(bt, $i, { skipTitle: !0 }), cn[$i._name + ".range[0]"] = $i.range[0], cn[$i._name + ".range[1]"] = $i.range[1];
                }
                he.redrawComponents(bt, fi);
              }
              function Yr() {
                if (!bt._transitioningWithDuration) {
                  var Zr = bt._context.doubleClick, fi = [];
                  Qt && (fi = fi.concat(Tt)), Ut && (fi = fi.concat(It)), hn.xaxes && (fi = fi.concat(hn.xaxes)), hn.yaxes && (fi = fi.concat(hn.yaxes));
                  var ri, bi, Zi = {};
                  if (Zr === "reset+autosize") for (Zr = "autosize", bi = 0; bi < fi.length; bi++) {
                    var $i = (ri = fi[bi])._rangeInitial0, Br = ri._rangeInitial1, Pr = $i !== void 0 || Br !== void 0;
                    if (Pr && ($i !== void 0 && $i !== ri.range[0] || Br !== void 0 && Br !== ri.range[1]) || !Pr && ri.autorange !== !0) {
                      Zr = "reset";
                      break;
                    }
                  }
                  if (Zr === "autosize") for (bi = 0; bi < fi.length; bi++) (ri = fi[bi]).fixedrange || (Zi[ri._name + ".autorange"] = !0);
                  else if (Zr === "reset") {
                    for ((Qt || sn.isSubplotConstrained) && (fi = fi.concat(sn.xaxes)), Ut && !sn.isSubplotConstrained && (fi = fi.concat(sn.yaxes)), sn.isSubplotConstrained && (Qt ? Ut || (fi = fi.concat(It)) : fi = fi.concat(Tt)), bi = 0; bi < fi.length; bi++) if (!(ri = fi[bi]).fixedrange) {
                      var ei = ri._name, Ci = ri._autorangeInitial;
                      ri._rangeInitial0 === void 0 && ri._rangeInitial1 === void 0 ? Zi[ei + ".autorange"] = !0 : ri._rangeInitial0 === void 0 ? (Zi[ei + ".autorange"] = Ci, Zi[ei + ".range"] = [null, ri._rangeInitial1]) : ri._rangeInitial1 === void 0 ? (Zi[ei + ".range"] = [ri._rangeInitial0, null], Zi[ei + ".autorange"] = Ci) : Zi[ei + ".range"] = [ri._rangeInitial0, ri._rangeInitial1];
                    }
                  }
                  bt.emit("plotly_doubleclick", null), se.call("_guiRelayout", bt, Zi);
                }
              }
              function jr() {
                Mr([0, 0, Vt, Yt]), ne.syncOrAsync([we.previousPromises, function() {
                  bt._fullLayout._replotting = !1, se.call("_guiRelayout", bt, cn);
                }], bt);
              }
              function Mr(Zr) {
                var fi, ri, bi, Zi, $i = bt._fullLayout, Br = $i._plots, Pr = $i._subplots.cartesian;
                if (Ht && se.subplotsRegistry.splom.drag(bt), dn) {
                  for (fi = 0; fi < Pr.length; fi++) if (bi = (ri = Br[Pr[fi]]).xaxis, Zi = ri.yaxis, ri._scene) {
                    bi.limitRange && bi.limitRange(), Zi.limitRange && Zi.limitRange();
                    var ei = ne.simpleMap(bi.range, bi.r2l), Ci = ne.simpleMap(Zi.range, Zi.r2l);
                    ri._scene.update({ range: [ei[0], Ci[0], ei[1], Ci[1]] });
                  }
                }
                if ((Ht || dn) && (Ce(bt), Se(bt)), jt) {
                  var oa = Zr[2] / Xt._length, ka = Zr[3] / Nt._length;
                  for (fi = 0; fi < Pr.length; fi++) {
                    bi = (ri = Br[Pr[fi]]).xaxis, Zi = ri.yaxis;
                    var aa, hi, Gr, Ca, Sr = (rn || hn.isSubplotConstrained) && !bi.fixedrange && Gt[bi._id], Dr = (vn || hn.isSubplotConstrained) && !Zi.fixedrange && Ft[Zi._id];
                    if (Sr ? (aa = oa, Gr = Pt || hn.isSubplotConstrained ? Zr[0] : gi(bi, aa)) : hn.xaHash[bi._id] ? (aa = oa, Gr = Zr[0] * bi._length / Xt._length) : hn.yaHash[bi._id] ? (aa = ka, Gr = Ut === "ns" ? -Zr[1] * bi._length / Nt._length : gi(bi, aa, { n: "top", s: "bottom" }[Ut])) : Gr = Bi(bi, aa = Wr(bi, oa, ka)), aa > 1 && (bi.maxallowed !== void 0 && rn === (bi.range[0] < bi.range[1] ? "e" : "w") || bi.minallowed !== void 0 && rn === (bi.range[0] < bi.range[1] ? "w" : "e")) && (aa = 1, Gr = 0), Dr ? (hi = ka, Ca = $t || hn.isSubplotConstrained ? Zr[1] : gi(Zi, hi)) : hn.yaHash[Zi._id] ? (hi = ka, Ca = Zr[1] * Zi._length / Nt._length) : hn.xaHash[Zi._id] ? (hi = oa, Ca = Qt === "ew" ? -Zr[0] * Zi._length / Xt._length : gi(Zi, hi, { e: "right", w: "left" }[Qt])) : Ca = Bi(Zi, hi = Wr(Zi, oa, ka)), hi > 1 && (Zi.maxallowed !== void 0 && vn === (Zi.range[0] < Zi.range[1] ? "n" : "s") || Zi.minallowed !== void 0 && vn === (Zi.range[0] < Zi.range[1] ? "s" : "n")) && (hi = 1, Ca = 0), aa || hi) {
                      aa || (aa = 1), hi || (hi = 1);
                      var si = bi._offset - Gr / aa, zr = Zi._offset - Ca / hi;
                      ri.clipRect.call(de.setTranslate, Gr, Ca).call(de.setScale, aa, hi), ri.plot.call(de.setTranslate, si, zr).call(de.setScale, 1 / aa, 1 / hi), aa === ri.xScaleFactor && hi === ri.yScaleFactor || (de.setPointGroupScale(ri.zoomScalePts, aa, hi), de.setTextPointsScale(ri.zoomScaleTxt, aa, hi)), de.hideOutsideRangePoints(ri.clipOnAxisFalseTraces, ri), ri.xScaleFactor = aa, ri.yScaleFactor = hi;
                    }
                  }
                }
              }
              function Wr(Zr, fi, ri) {
                return Zr.fixedrange ? 0 : rn && sn.xaHash[Zr._id] ? fi : vn && (sn.isSubplotConstrained ? sn.xaHash : sn.yaHash)[Zr._id] ? ri : 0;
              }
              function Bi(Zr, fi) {
                return fi ? (Zr.range = Zr._r.slice(), ke(Zr, fi), gi(Zr, fi)) : 0;
              }
              function gi(Zr, fi, ri) {
                return Zr._length * (1 - fi) * be[ri || Zr.constraintoward || "middle"];
              }
              return $t.length * Pt.length != 1 && st(wt, function(Zr) {
                if (bt._context._scrollZoom.cartesian || bt._fullLayout._enablescrollzoom) {
                  if (Mn(), bt._transitioningWithDuration) return Zr.preventDefault(), void Zr.stopPropagation();
                  ft(), clearTimeout(ur);
                  var fi = -Zr.deltaY;
                  if (isFinite(fi) || (fi = Zr.wheelDelta / 10), isFinite(fi)) {
                    var ri, bi = Math.exp(-Math.min(Math.max(fi, -20), 20) / 200), Zi = Jn.draglayer.select(".nsewdrag").node().getBoundingClientRect(), $i = (Zr.clientX - Zi.left) / Zi.width, Br = (Zi.bottom - Zr.clientY) / Zi.height;
                    if (rn) {
                      for (Pt || ($i = 0.5), ri = 0; ri < Tt.length; ri++) Pr(Tt[ri], $i, bi);
                      mr("x"), jn[2] *= bi, jn[0] += jn[2] * $i * (1 / bi - 1);
                    }
                    if (vn) {
                      for ($t || (Br = 0.5), ri = 0; ri < It.length; ri++) Pr(It[ri], Br, bi);
                      mr("y"), jn[3] *= bi, jn[1] += jn[3] * (1 - Br) * (1 / bi - 1);
                    }
                    Mr(jn), Ar(), bt.emit("plotly_relayouting", cn), ur = setTimeout(function() {
                      bt._fullLayout && (jn = [0, 0, Vt, Yt], jr());
                    }, Kn), Zr.preventDefault();
                  } else ne.log("Did not find wheel motion attributes: ", Zr);
                }
                function Pr(ei, Ci, oa) {
                  if (!ei.fixedrange) {
                    var ka = ne.simpleMap(ei.range, ei.r2l), aa = ka[0] + (ka[1] - ka[0]) * Ci;
                    ei.range = ka.map(function(hi) {
                      return ei.l2r(aa + (hi - aa) * oa);
                    });
                  }
                }
              }), wt;
            }, makeDragger: $e, makeRectDragger: qe, makeZoombox: ze, makeCorners: We, updateZoombox: at, xyCorners: Je, transitionZoombox: ct, removeZoombox: et, showDoubleClickNotifier: dt, attachWheelEventHandler: st };
          }, 95284: function(G, J, Z) {
            var te = Z(45568), ne = Z(32141), ee = Z(14751), ae = Z(27983), oe = Z(51680).makeDragBox, se = Z(54826).DRAGGERSIZE;
            J.initInteractions = function(ce) {
              var le = ce._fullLayout;
              if (ce._context.staticPlot) te.select(ce).selectAll(".drag").remove();
              else if (le._has("cartesian") || le._has("splom")) {
                Object.keys(le._plots || {}).sort(function(de, fe) {
                  if ((le._plots[de].mainplot && !0) === (le._plots[fe].mainplot && !0)) {
                    var he = de.split("y"), me = fe.split("y");
                    return he[0] === me[0] ? Number(he[1] || 1) - Number(me[1] || 1) : Number(he[0] || 1) - Number(me[0] || 1);
                  }
                  return le._plots[de].mainplot ? 1 : -1;
                }).forEach(function(de) {
                  var fe = le._plots[de], he = fe.xaxis, me = fe.yaxis;
                  if (!fe.mainplot) {
                    var pe = oe(ce, fe, he._offset, me._offset, he._length, me._length, "ns", "ew");
                    pe.onmousemove = function(ve) {
                      ce._fullLayout._rehover = function() {
                        ce._fullLayout._hoversubplot === de && ce._fullLayout._plots[de] && ne.hover(ce, ve, de);
                      }, ne.hover(ce, ve, de), ce._fullLayout._lasthover = pe, ce._fullLayout._hoversubplot = de;
                    }, pe.onmouseout = function(ve) {
                      ce._dragging || (ce._fullLayout._hoversubplot = null, ee.unhover(ce, ve));
                    }, ce._context.showAxisDragHandles && (oe(ce, fe, he._offset - se, me._offset - se, se, se, "n", "w"), oe(ce, fe, he._offset + he._length, me._offset - se, se, se, "n", "e"), oe(ce, fe, he._offset - se, me._offset + me._length, se, se, "s", "w"), oe(ce, fe, he._offset + he._length, me._offset + me._length, se, se, "s", "e"));
                  }
                  if (ce._context.showAxisDragHandles) {
                    if (de === he._mainSubplot) {
                      var ye = he._mainLinePosition;
                      he.side === "top" && (ye -= se), oe(ce, fe, he._offset + 0.1 * he._length, ye, 0.8 * he._length, se, "", "ew"), oe(ce, fe, he._offset, ye, 0.1 * he._length, se, "", "w"), oe(ce, fe, he._offset + 0.9 * he._length, ye, 0.1 * he._length, se, "", "e");
                    }
                    if (de === me._mainSubplot) {
                      var ge = me._mainLinePosition;
                      me.side !== "right" && (ge -= se), oe(ce, fe, ge, me._offset + 0.1 * me._length, se, 0.8 * me._length, "ns", ""), oe(ce, fe, ge, me._offset + 0.9 * me._length, se, 0.1 * me._length, "s", ""), oe(ce, fe, ge, me._offset, se, 0.1 * me._length, "n", "");
                    }
                  }
                });
                var ue = le._hoverlayer.node();
                ue.onmousemove = function(de) {
                  de.target = ce._fullLayout._lasthover, ne.hover(ce, de, le._hoversubplot);
                }, ue.onclick = function(de) {
                  de.target = ce._fullLayout._lasthover, ne.click(ce, de);
                }, ue.onmousedown = function(de) {
                  ce._fullLayout._lasthover.onmousedown(de);
                }, J.updateFx(ce);
              }
            }, J.updateFx = function(ce) {
              var le = ce._fullLayout, ue = le.dragmode === "pan" ? "move" : "crosshair";
              ae(le._draggers, ue);
            };
          }, 20706: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(5975);
            G.exports = function(ae) {
              return function(oe, se) {
                var ce = oe[ae];
                if (Array.isArray(ce)) for (var le = te.subplotsRegistry.cartesian, ue = le.idRegex, de = se._subplots, fe = de.xaxis, he = de.yaxis, me = de.cartesian, pe = se._has("cartesian") || se._has("gl2d"), ye = 0; ye < ce.length; ye++) {
                  var ge = ce[ye];
                  if (ne.isPlainObject(ge)) {
                    var ve = ee.cleanId(ge.xref, "x", !1), be = ee.cleanId(ge.yref, "y", !1), Ce = ue.x.test(ve), Se = ue.y.test(be);
                    if (Ce || Se) {
                      pe || ne.pushUnique(se._basePlotModules, le);
                      var we = !1;
                      Ce && fe.indexOf(ve) === -1 && (fe.push(ve), we = !0), Se && he.indexOf(be) === -1 && (he.push(be), we = !0), we && Ce && Se && me.push(ve + be);
                    }
                  }
                }
              };
            };
          }, 37703: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(34809), ae = Z(44122), oe = Z(62203), se = Z(4173).eV, ce = Z(5975), le = Z(54826), ue = Z(62972), de = ee.ensureSingle;
            function fe(ve, be, Ce) {
              return ee.ensureSingle(ve, be, Ce, function(Se) {
                Se.datum(Ce);
              });
            }
            var he = le.zindexSeparator;
            function me(ve, be, Ce, Se, we) {
              for (var Ee, xe, Le, Qe = le.traceLayerClasses, ke = ve._fullLayout, Me = ke._zindices, Re = ke._modules, De = [], Pe = [], $e = 0; $e < Me.length; $e++) for (var qe = Me[$e], Xe = 0; Xe < Re.length; Xe++) {
                var Ve = (Ee = Re[Xe]).name, Ze = ne.modules[Ve].categories;
                if (Ze.svg) {
                  var Ge = Ee.layerName || Ve + "layer", ze = Ge + ($e ? Number($e) + 1 : ""), We = Ee.plot;
                  Le = (xe = se(Ce, We, qe))[0], Ce = xe[1], Le.length && De.push({ i: Qe.indexOf(Ge), zindex: $e, className: ze, plotMethod: We, cdModule: Le }), Ze.zoomScale && Pe.push("." + ze);
                }
              }
              De.sort(function(et, dt) {
                return (et.zindex || 0) - (dt.zindex || 0) || et.i - dt.i;
              });
              var at = be.plot.selectAll("g.mlayer").data(De, function(et) {
                return et.className;
              });
              if (at.enter().append("g").attr("class", function(et) {
                return et.className;
              }).classed("mlayer", !0).classed("rangeplot", be.isRangePlot), at.exit().remove(), at.order(), at.each(function(et) {
                var dt = te.select(this), Je = et.className;
                et.plotMethod(ve, be, et.cdModule, dt, Se, we), le.clipOnAxisFalseQuery.indexOf("." + Je) === -1 && oe.setClipUrl(dt, be.layerClipId, ve);
              }), ke._has("scattergl") && (Ee = ne.getModule("scattergl"), Le = se(Ce, Ee)[0], Ee.plot(ve, be, Le)), !ve._context.staticPlot && (be._hasClipOnAxisFalse && (be.clipOnAxisFalseTraces = be.plot.selectAll(le.clipOnAxisFalseQuery.join(",")).selectAll(".trace")), Pe.length)) {
                var ct = be.plot.selectAll(Pe.join(",")).selectAll(".trace");
                be.zoomScalePts = ct.selectAll("path.point"), be.zoomScaleTxt = ct.selectAll(".textpoint");
              }
            }
            function pe(ve, be) {
              var Ce = ve._fullLayout, Se = be.plotgroup, we = be.id, Ee = we.indexOf(he) !== -1, xe = le.layerValue2layerClass[be.xaxis.layer], Le = le.layerValue2layerClass[be.yaxis.layer], Qe = Ce._hasOnlyLargeSploms;
              if (!be.mainplot || Ce._zindices.length > 1) if (Qe) be.xlines = de(Se, "path", "xlines-above"), be.ylines = de(Se, "path", "ylines-above"), be.xaxislayer = de(Se, "g", "xaxislayer-above"), be.yaxislayer = de(Se, "g", "yaxislayer-above");
              else {
                if (!Ee) {
                  var ke = de(Se, "g", "layer-subplot");
                  be.shapelayer = de(ke, "g", "shapelayer"), be.imagelayer = de(ke, "g", "imagelayer"), be.minorGridlayer = de(Se, "g", "minor-gridlayer"), be.gridlayer = de(Se, "g", "gridlayer"), be.zerolinelayer = de(Se, "g", "zerolinelayer");
                  var Me = de(Se, "g", "layer-between");
                  be.shapelayerBetween = de(Me, "g", "shapelayer"), be.imagelayerBetween = de(Me, "g", "imagelayer"), de(Se, "path", "xlines-below"), de(Se, "path", "ylines-below"), be.overlinesBelow = de(Se, "g", "overlines-below"), de(Se, "g", "xaxislayer-below"), de(Se, "g", "yaxislayer-below"), be.overaxesBelow = de(Se, "g", "overaxes-below");
                }
                be.overplot = de(Se, "g", "overplot"), be.plot = de(be.overplot, "g", we), Ee || (be.xlines = de(Se, "path", "xlines-above"), be.ylines = de(Se, "path", "ylines-above"), be.overlinesAbove = de(Se, "g", "overlines-above"), de(Se, "g", "xaxislayer-above"), de(Se, "g", "yaxislayer-above"), be.overaxesAbove = de(Se, "g", "overaxes-above"), be.xlines = Se.select(".xlines-" + xe), be.ylines = Se.select(".ylines-" + Le), be.xaxislayer = Se.select(".xaxislayer-" + xe), be.yaxislayer = Se.select(".yaxislayer-" + Le));
              }
              else {
                var Re = be.mainplotinfo, De = Re.plotgroup, Pe = we + "-x", $e = we + "-y";
                be.minorGridlayer = Re.minorGridlayer, be.gridlayer = Re.gridlayer, be.zerolinelayer = Re.zerolinelayer, de(Re.overlinesBelow, "path", Pe), de(Re.overlinesBelow, "path", $e), de(Re.overaxesBelow, "g", Pe), de(Re.overaxesBelow, "g", $e), be.plot = de(Re.overplot, "g", we), de(Re.overlinesAbove, "path", Pe), de(Re.overlinesAbove, "path", $e), de(Re.overaxesAbove, "g", Pe), de(Re.overaxesAbove, "g", $e), be.xlines = De.select(".overlines-" + xe).select("." + Pe), be.ylines = De.select(".overlines-" + Le).select("." + $e), be.xaxislayer = De.select(".overaxes-" + xe).select("." + Pe), be.yaxislayer = De.select(".overaxes-" + Le).select("." + $e);
              }
              Ee || (Qe || (fe(be.minorGridlayer, "g", be.xaxis._id), fe(be.minorGridlayer, "g", be.yaxis._id), be.minorGridlayer.selectAll("g").map(function(qe) {
                return qe[0];
              }).sort(ce.idSort), fe(be.gridlayer, "g", be.xaxis._id), fe(be.gridlayer, "g", be.yaxis._id), be.gridlayer.selectAll("g").map(function(qe) {
                return qe[0];
              }).sort(ce.idSort)), be.xlines.style("fill", "none").classed("crisp", !0), be.ylines.style("fill", "none").classed("crisp", !0));
            }
            function ye(ve, be) {
              if (ve) {
                var Ce = {};
                for (var Se in ve.each(function(Le) {
                  var Qe = Le[0];
                  te.select(this).remove(), ge(Qe, be), Ce[Qe] = !0;
                }), be._plots) for (var we = be._plots[Se].overlays || [], Ee = 0; Ee < we.length; Ee++) {
                  var xe = we[Ee];
                  Ce[xe.id] && xe.plot.selectAll(".trace").remove();
                }
              }
            }
            function ge(ve, be) {
              be._draggers.selectAll("g." + ve).remove(), be._defs.select("#clip" + be._uid + ve + "plot").remove();
            }
            J.name = "cartesian", J.attr = ["xaxis", "yaxis"], J.idRoot = ["x", "y"], J.idRegex = le.idRegex, J.attrRegex = le.attrRegex, J.attributes = Z(55126), J.layoutAttributes = Z(25829), J.supplyLayoutDefaults = Z(74098), J.transitionAxes = Z(84982), J.finalizeSubplots = function(ve, be) {
              var Ce, Se, we, Ee = be._subplots, xe = Ee.xaxis, Le = Ee.yaxis, Qe = Ee.cartesian, ke = Qe.concat(Ee.gl2d || []), Me = {}, Re = {};
              for (Ce = 0; Ce < ke.length; Ce++) {
                var De = ke[Ce].split("y");
                Me[De[0]] = 1, Re["y" + De[1]] = 1;
              }
              for (Ce = 0; Ce < xe.length; Ce++) Me[Se = xe[Ce]] || (we = (ve[ce.id2name(Se)] || {}).anchor, le.idRegex.y.test(we) || (we = "y"), Qe.push(Se + we), ke.push(Se + we), Re[we] || (Re[we] = 1, ee.pushUnique(Le, we)));
              for (Ce = 0; Ce < Le.length; Ce++) Re[we = Le[Ce]] || (Se = (ve[ce.id2name(we)] || {}).anchor, le.idRegex.x.test(Se) || (Se = "x"), Qe.push(Se + we), ke.push(Se + we), Me[Se] || (Me[Se] = 1, ee.pushUnique(xe, Se)));
              if (!ke.length) {
                for (var Pe in Se = "", we = "", ve) le.attrRegex.test(Pe) && (Pe.charAt(0) === "x" ? (!Se || +Pe.substr(5) < +Se.substr(5)) && (Se = Pe) : (!we || +Pe.substr(5) < +we.substr(5)) && (we = Pe));
                Se = Se ? ce.name2id(Se) : "x", we = we ? ce.name2id(we) : "y", xe.push(Se), Le.push(we), Qe.push(Se + we);
              }
            }, J.plot = function(ve, be, Ce, Se) {
              var we, Ee = ve._fullLayout, xe = Ee._subplots.cartesian, Le = ve.calcdata;
              if (!Array.isArray(be)) for (be = [], we = 0; we < Le.length; we++) be.push(we);
              for (var Qe = Ee._zindices, ke = 0; ke < Qe.length; ke++) {
                var Me = Qe[ke];
                for (we = 0; we < xe.length; we++) {
                  var Re = xe[we], De = Ee._plots[Re];
                  if (ke > 0) {
                    var Pe = De.id;
                    if (Pe.indexOf(he) !== -1) continue;
                    Pe += he + (ke + 1), De = ee.extendFlat({}, De, { id: Pe, plot: Ee._cartesianlayer.selectAll(".subplot").select("." + Pe) });
                  }
                  for (var $e, qe = [], Xe = 0; Xe < Le.length; Xe++) {
                    var Ve = Le[Xe], Ze = Ve[0].trace;
                    Me === (Ze.zorder || 0) && Ze.xaxis + Ze.yaxis === Re && ((be.indexOf(Ze.index) !== -1 || Ze.carpet) && ($e && $e[0].trace.xaxis + $e[0].trace.yaxis === Re && ["tonextx", "tonexty", "tonext"].indexOf(Ze.fill) !== -1 && qe.indexOf($e) === -1 && qe.push($e), qe.push(Ve)), $e = Ve);
                  }
                  me(ve, De, qe, Ce, Se);
                }
              }
            }, J.clean = function(ve, be, Ce, Se) {
              var we, Ee, xe, Le = Se._plots || {}, Qe = be._plots || {}, ke = Se._subplots || {};
              if (Se._hasOnlyLargeSploms && !be._hasOnlyLargeSploms) for (xe in Le) (we = Le[xe]).plotgroup && we.plotgroup.remove();
              var Me = Se._has && Se._has("gl"), Re = be._has && be._has("gl");
              if (Me && !Re) for (xe in Le) (we = Le[xe])._scene && we._scene.destroy();
              if (ke.xaxis && ke.yaxis) {
                var De = ce.listIds({ _fullLayout: Se });
                for (Ee = 0; Ee < De.length; Ee++) {
                  var Pe = De[Ee];
                  be[ce.id2name(Pe)] || Se._infolayer.selectAll(".g-" + Pe + "title").remove();
                }
              }
              var $e = Se._has && Se._has("cartesian"), qe = be._has && be._has("cartesian");
              if ($e && !qe) ye(Se._cartesianlayer.selectAll(".subplot"), Se), Se._defs.selectAll(".axesclip").remove(), delete Se._axisConstraintGroups, delete Se._axisMatchGroups;
              else if (ke.cartesian) for (Ee = 0; Ee < ke.cartesian.length; Ee++) {
                var Xe = ke.cartesian[Ee];
                if (Xe.indexOf(he) === -1 && !Qe[Xe]) {
                  var Ve = "." + Xe + ",." + Xe + "-x,." + Xe + "-y";
                  Se._cartesianlayer.selectAll(Ve).remove(), ge(Xe, Se);
                }
              }
            }, J.drawFramework = function(ve) {
              var be, Ce = ve._fullLayout, Se = ve.calcdata, we = {};
              for (be = 0; be < Se.length; be++) {
                var Ee = Se[be][0], xe = Ee.trace.zorder || 0;
                we[xe] || (we[xe] = []), we[xe].push(Ee);
              }
              var Le = Object.keys(we).map(Number).sort(ee.sorterAsc);
              Le.length || (Le = [0]), Ce._zindices = Le;
              var Qe = function($e) {
                var qe, Xe, Ve, Ze, Ge, ze, We = $e._fullLayout, at = We._zindices.length, ct = We._subplots.cartesian, et = ct.length, dt = [], Je = [];
                for (qe = 0; qe < et; qe++) {
                  Ve = ct[qe], Ge = (Ze = We._plots[Ve]).xaxis, ze = Ze.yaxis;
                  var it = Ge._mainAxis, st = ze._mainAxis, gt = it._id + st._id, bt = We._plots[gt];
                  Ze.overlays = [], gt !== Ve && bt ? (Ze.mainplot = gt, Ze.mainplotinfo = bt, Je.push(Ve)) : (Ze.mainplot = void 0, Ze.mainplotinfo = void 0, dt.push(Ve));
                }
                for (qe = 0; qe < Je.length; qe++) Ve = Je[qe], (Ze = We._plots[Ve]).mainplotinfo.overlays.push(Ze);
                var yt = dt.concat(Je), St = [];
                for (qe = 0; qe < et; qe++) {
                  Ve = yt[qe], Ge = (Ze = We._plots[Ve]).xaxis, ze = Ze.yaxis;
                  for (var Lt = [], Rt = 1; Rt <= at; Rt++) {
                    var vt = "";
                    for (Rt > 1 && (vt += he + Rt), Lt.push(Ve + vt), Xe = 0; Xe < Ze.overlays.length; Xe++) Lt.push(Ze.overlays[Xe].id + vt);
                  }
                  Lt = Lt.concat([Ge.layer, ze.layer, Ge.overlaying || "", ze.overlaying || ""]), St.push(Lt);
                }
                return St;
              }(ve), ke = Qe.length, Me = [];
              for (be = 0; be < ke; be++) Me[be] = Qe[be].slice();
              for (var Re = 1; Re < Le.length; Re++) {
                var De = [];
                for (be = 0; be < ke; be++) De[be] = Qe[be].slice(), De[be][0] += he + (Re + 1);
                Me = Me.concat(De);
              }
              var Pe = Ce._cartesianlayer.selectAll(".subplot").data(Me, String);
              Pe.enter().append("g").attr("class", function($e) {
                return "subplot " + $e[0];
              }), Pe.order(), Pe.exit().call(ye, Ce), Pe.each(function($e) {
                var qe = $e[0], Xe = qe.indexOf(he), Ve = Xe !== -1, Ze = Ve ? qe.slice(0, Xe) : qe, Ge = Ce._plots[qe];
                Ge || (Ge = ee.extendFlat({}, Ce._plots[Ze])) && (Ge.id = qe, Ce._plots[qe] = Ge, Ce._subplots.cartesian.push(qe)), Ge && (Ge.plotgroup = te.select(this), pe(ve, Ge), Ve || (Ge.draglayer = de(Ce._draggers, "g", qe)));
              });
            }, J.rangePlot = function(ve, be, Ce) {
              pe(ve, be), me(ve, be, Ce), ae.style(ve);
            }, J.toSVG = function(ve) {
              var be = ve._fullLayout._glimages, Ce = te.select(ve).selectAll(".svg-container");
              Ce.filter(function(Se, we) {
                return we === Ce.size() - 1;
              }).selectAll(".gl-canvas-context, .gl-canvas-focus").each(function() {
                var Se = this, we = Se.toDataURL("image/png");
                be.append("svg:image").attr({ xmlns: ue.svg, "xlink:href": we, preserveAspectRatio: "none", x: 0, y: 0, width: Se.style.width, height: Se.style.height });
              });
            }, J.updateFx = Z(95284).updateFx;
          }, 25829: function(G, J, Z) {
            var te = Z(80337), ne = Z(10229), ee = Z(94850).T, ae = Z(93049).extendFlat, oe = Z(78032).templatedArray, se = Z(80712).descriptionWithDates, ce = Z(63821).ONEDAY, le = Z(54826), ue = le.HOUR_PATTERN, de = le.WEEKDAY_PATTERN, fe = { valType: "enumerated", values: ["auto", "linear", "array"], editType: "ticks", impliedEdits: { tick0: void 0, dtick: void 0 } }, he = ae({}, fe, { values: fe.values.slice().concat(["sync"]) });
            function me(Qe) {
              return { valType: "integer", min: 0, dflt: Qe ? 5 : 0, editType: "ticks" };
            }
            var pe = { valType: "any", editType: "ticks", impliedEdits: { tickmode: "linear" } }, ye = { valType: "any", editType: "ticks", impliedEdits: { tickmode: "linear" } }, ge = { valType: "data_array", editType: "ticks" }, ve = { valType: "enumerated", values: ["outside", "inside", ""], editType: "ticks" };
            function be(Qe) {
              var ke = { valType: "number", min: 0, editType: "ticks" };
              return Qe || (ke.dflt = 5), ke;
            }
            function Ce(Qe) {
              var ke = { valType: "number", min: 0, editType: "ticks" };
              return Qe || (ke.dflt = 1), ke;
            }
            var Se = { valType: "color", dflt: ne.defaultLine, editType: "ticks" }, we = { valType: "color", dflt: ne.lightLine, editType: "ticks" };
            function Ee(Qe) {
              var ke = { valType: "number", min: 0, editType: "ticks" };
              return Qe || (ke.dflt = 1), ke;
            }
            var xe = ae({}, ee, { editType: "ticks" }), Le = { valType: "boolean", editType: "ticks" };
            G.exports = { visible: { valType: "boolean", editType: "plot" }, color: { valType: "color", dflt: ne.defaultLine, editType: "ticks" }, title: { text: { valType: "string", editType: "ticks" }, font: te({ editType: "ticks" }), standoff: { valType: "number", min: 0, editType: "ticks" }, editType: "ticks" }, type: { valType: "enumerated", values: ["-", "linear", "log", "date", "category", "multicategory"], dflt: "-", editType: "calc", _noTemplating: !0 }, autotypenumbers: { valType: "enumerated", values: ["convert types", "strict"], dflt: "convert types", editType: "calc" }, autorange: { valType: "enumerated", values: [!0, !1, "reversed", "min reversed", "max reversed", "min", "max"], dflt: !0, editType: "axrange", impliedEdits: { "range[0]": void 0, "range[1]": void 0 } }, autorangeoptions: { minallowed: { valType: "any", editType: "plot", impliedEdits: { "range[0]": void 0, "range[1]": void 0 } }, maxallowed: { valType: "any", editType: "plot", impliedEdits: { "range[0]": void 0, "range[1]": void 0 } }, clipmin: { valType: "any", editType: "plot", impliedEdits: { "range[0]": void 0, "range[1]": void 0 } }, clipmax: { valType: "any", editType: "plot", impliedEdits: { "range[0]": void 0, "range[1]": void 0 } }, include: { valType: "any", arrayOk: !0, editType: "plot", impliedEdits: { "range[0]": void 0, "range[1]": void 0 } }, editType: "plot" }, rangemode: { valType: "enumerated", values: ["normal", "tozero", "nonnegative"], dflt: "normal", editType: "plot" }, range: { valType: "info_array", items: [{ valType: "any", editType: "axrange", impliedEdits: { "^autorange": !1 }, anim: !0 }, { valType: "any", editType: "axrange", impliedEdits: { "^autorange": !1 }, anim: !0 }], editType: "axrange", impliedEdits: { autorange: !1 }, anim: !0 }, minallowed: { valType: "any", editType: "plot", impliedEdits: { "^autorange": !1 } }, maxallowed: { valType: "any", editType: "plot", impliedEdits: { "^autorange": !1 } }, fixedrange: { valType: "boolean", dflt: !1, editType: "calc" }, insiderange: { valType: "info_array", items: [{ valType: "any", editType: "plot" }, { valType: "any", editType: "plot" }], editType: "plot" }, scaleanchor: { valType: "enumerated", values: [le.idRegex.x.toString(), le.idRegex.y.toString(), !1], editType: "plot" }, scaleratio: { valType: "number", min: 0, dflt: 1, editType: "plot" }, constrain: { valType: "enumerated", values: ["range", "domain"], editType: "plot" }, constraintoward: { valType: "enumerated", values: ["left", "center", "right", "top", "middle", "bottom"], editType: "plot" }, matches: { valType: "enumerated", values: [le.idRegex.x.toString(), le.idRegex.y.toString()], editType: "calc" }, rangebreaks: oe("rangebreak", { enabled: { valType: "boolean", dflt: !0, editType: "calc" }, bounds: { valType: "info_array", items: [{ valType: "any", editType: "calc" }, { valType: "any", editType: "calc" }], editType: "calc" }, pattern: { valType: "enumerated", values: [de, ue, ""], editType: "calc" }, values: { valType: "info_array", freeLength: !0, editType: "calc", items: { valType: "any", editType: "calc" } }, dvalue: { valType: "number", editType: "calc", min: 0, dflt: ce }, editType: "calc" }), tickmode: he, nticks: me(), tick0: pe, dtick: ye, ticklabelstep: { valType: "integer", min: 1, dflt: 1, editType: "ticks" }, tickvals: ge, ticktext: { valType: "data_array", editType: "ticks" }, ticks: ve, tickson: { valType: "enumerated", values: ["labels", "boundaries"], dflt: "labels", editType: "ticks" }, ticklabelmode: { valType: "enumerated", values: ["instant", "period"], dflt: "instant", editType: "ticks" }, ticklabelposition: { valType: "enumerated", values: ["outside", "inside", "outside top", "inside top", "outside left", "inside left", "outside right", "inside right", "outside bottom", "inside bottom"], dflt: "outside", editType: "calc" }, ticklabeloverflow: { valType: "enumerated", values: ["allow", "hide past div", "hide past domain"], editType: "calc" }, ticklabelshift: { valType: "integer", dflt: 0, editType: "ticks" }, ticklabelstandoff: { valType: "integer", dflt: 0, editType: "ticks" }, ticklabelindex: { valType: "integer", arrayOk: !0, editType: "calc" }, mirror: { valType: "enumerated", values: [!0, "ticks", !1, "all", "allticks"], dflt: !1, editType: "ticks+layoutstyle" }, ticklen: be(), tickwidth: Ce(), tickcolor: Se, showticklabels: { valType: "boolean", dflt: !0, editType: "ticks" }, labelalias: { valType: "any", dflt: !1, editType: "ticks" }, automargin: { valType: "flaglist", flags: ["height", "width", "left", "right", "top", "bottom"], extras: [!0, !1], dflt: !1, editType: "ticks" }, showspikes: { valType: "boolean", dflt: !1, editType: "modebar" }, spikecolor: { valType: "color", dflt: null, editType: "none" }, spikethickness: { valType: "number", dflt: 3, editType: "none" }, spikedash: ae({}, ee, { dflt: "dash", editType: "none" }), spikemode: { valType: "flaglist", flags: ["toaxis", "across", "marker"], dflt: "toaxis", editType: "none" }, spikesnap: { valType: "enumerated", values: ["data", "cursor", "hovered data"], dflt: "hovered data", editType: "none" }, tickfont: te({ editType: "ticks" }), tickangle: { valType: "angle", dflt: "auto", editType: "ticks" }, autotickangles: { valType: "info_array", freeLength: !0, items: { valType: "angle" }, dflt: [0, 30, 90], editType: "ticks" }, tickprefix: { valType: "string", dflt: "", editType: "ticks" }, showtickprefix: { valType: "enumerated", values: ["all", "first", "last", "none"], dflt: "all", editType: "ticks" }, ticksuffix: { valType: "string", dflt: "", editType: "ticks" }, showticksuffix: { valType: "enumerated", values: ["all", "first", "last", "none"], dflt: "all", editType: "ticks" }, showexponent: { valType: "enumerated", values: ["all", "first", "last", "none"], dflt: "all", editType: "ticks" }, exponentformat: { valType: "enumerated", values: ["none", "e", "E", "power", "SI", "B"], dflt: "B", editType: "ticks" }, minexponent: { valType: "number", dflt: 3, min: 0, editType: "ticks" }, separatethousands: { valType: "boolean", dflt: !1, editType: "ticks" }, tickformat: { valType: "string", dflt: "", editType: "ticks", description: se("tick label") }, tickformatstops: oe("tickformatstop", { enabled: { valType: "boolean", dflt: !0, editType: "ticks" }, dtickrange: { valType: "info_array", items: [{ valType: "any", editType: "ticks" }, { valType: "any", editType: "ticks" }], editType: "ticks" }, value: { valType: "string", dflt: "", editType: "ticks" }, editType: "ticks" }), hoverformat: { valType: "string", dflt: "", editType: "none", description: se("hover text") }, showline: { valType: "boolean", dflt: !1, editType: "ticks+layoutstyle" }, linecolor: { valType: "color", dflt: ne.defaultLine, editType: "layoutstyle" }, linewidth: { valType: "number", min: 0, dflt: 1, editType: "ticks+layoutstyle" }, showgrid: Le, gridcolor: we, gridwidth: Ee(), griddash: xe, zeroline: { valType: "boolean", editType: "ticks" }, zerolinecolor: { valType: "color", dflt: ne.defaultLine, editType: "ticks" }, zerolinewidth: { valType: "number", dflt: 1, editType: "ticks" }, showdividers: { valType: "boolean", dflt: !0, editType: "ticks" }, dividercolor: { valType: "color", dflt: ne.defaultLine, editType: "ticks" }, dividerwidth: { valType: "number", dflt: 1, editType: "ticks" }, anchor: { valType: "enumerated", values: ["free", le.idRegex.x.toString(), le.idRegex.y.toString()], editType: "plot" }, side: { valType: "enumerated", values: ["top", "bottom", "left", "right"], editType: "plot" }, overlaying: { valType: "enumerated", values: ["free", le.idRegex.x.toString(), le.idRegex.y.toString()], editType: "plot" }, minor: { tickmode: fe, nticks: me("minor"), tick0: pe, dtick: ye, tickvals: ge, ticks: ve, ticklen: be("minor"), tickwidth: Ce("minor"), tickcolor: Se, gridcolor: we, gridwidth: Ee("minor"), griddash: xe, showgrid: Le, editType: "ticks" }, layer: { valType: "enumerated", values: ["above traces", "below traces"], dflt: "above traces", editType: "plot" }, domain: { valType: "info_array", items: [{ valType: "number", min: 0, max: 1, editType: "plot" }, { valType: "number", min: 0, max: 1, editType: "plot" }], dflt: [0, 1], editType: "plot" }, position: { valType: "number", min: 0, max: 1, dflt: 0, editType: "plot" }, autoshift: { valType: "boolean", dflt: !1, editType: "plot" }, shift: { valType: "number", editType: "plot" }, categoryorder: { valType: "enumerated", values: ["trace", "category ascending", "category descending", "array", "total ascending", "total descending", "min ascending", "min descending", "max ascending", "max descending", "sum ascending", "sum descending", "mean ascending", "mean descending", "geometric mean ascending", "geometric mean descending", "median ascending", "median descending"], dflt: "trace", editType: "calc" }, categoryarray: { valType: "data_array", editType: "calc" }, uirevision: { valType: "any", editType: "none" }, editType: "calc", _deprecated: { autotick: { valType: "boolean", editType: "ticks" }, title: { valType: "string", editType: "ticks" }, titlefont: te({ editType: "ticks" }) } };
          }, 74098: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(36040).isUnifiedHover, ae = Z(45265), oe = Z(78032), se = Z(6704), ce = Z(25829), le = Z(4392), ue = Z(97655), de = Z(84391), fe = Z(40957), he = Z(5975), me = he.id2name, pe = he.name2id, ye = Z(54826).AX_ID_PATTERN, ge = Z(33626), ve = ge.traceIs, be = ge.getComponentMethod;
            function Ce(Se, we, Ee) {
              Array.isArray(Se[we]) ? Se[we].push(Ee) : Se[we] = [Ee];
            }
            G.exports = function(Se, we, Ee) {
              var xe, Le, Qe = we.autotypenumbers, ke = {}, Me = {}, Re = {}, De = {}, Pe = {}, $e = {}, qe = {}, Xe = {}, Ve = {}, Ze = {};
              for (xe = 0; xe < Ee.length; xe++) {
                var Ge = Ee[xe];
                if (ve(Ge, "cartesian") || ve(Ge, "gl2d")) {
                  var ze, We;
                  if (Ge.xaxis) ze = me(Ge.xaxis), Ce(ke, ze, Ge);
                  else if (Ge.xaxes) for (Le = 0; Le < Ge.xaxes.length; Le++) Ce(ke, me(Ge.xaxes[Le]), Ge);
                  if (Ge.yaxis) We = me(Ge.yaxis), Ce(ke, We, Ge);
                  else if (Ge.yaxes) for (Le = 0; Le < Ge.yaxes.length; Le++) Ce(ke, me(Ge.yaxes[Le]), Ge);
                  Ge.type === "funnel" ? Ge.orientation === "h" ? (ze && (Me[ze] = !0), We && (qe[We] = !0)) : We && (Re[We] = !0) : Ge.type === "image" ? (We && (Xe[We] = !0), ze && (Xe[ze] = !0)) : (We && (Pe[We] = !0, $e[We] = !0), ve(Ge, "carpet") && (Ge.type !== "carpet" || Ge._cheater) || ze && (De[ze] = !0)), Ge.type === "carpet" && Ge._cheater && ze && (Me[ze] = !0), ve(Ge, "2dMap") && (Ve[ze] = !0, Ve[We] = !0), ve(Ge, "oriented") && (Ze[Ge.orientation === "h" ? We : ze] = !0);
                }
              }
              var at = we._subplots, ct = at.xaxis, et = at.yaxis, dt = te.simpleMap(ct, me), Je = te.simpleMap(et, me), it = dt.concat(Je), st = ne.background;
              ct.length && et.length && (st = te.coerce(Se, we, se, "plot_bgcolor"));
              var gt, bt, yt, St, Lt, Rt = ne.combine(st, we.paper_bgcolor);
              function vt() {
                var rt = ke[gt] || [];
                Lt._traceIndices = rt.map(function(ft) {
                  return ft._expandedIndex;
                }), Lt._annIndices = [], Lt._shapeIndices = [], Lt._selectionIndices = [], Lt._imgIndices = [], Lt._subplotsWith = [], Lt._counterAxes = [], Lt._name = Lt._attr = gt, Lt._id = bt;
              }
              function $t(rt, ft) {
                return te.coerce(St, Lt, ce, rt, ft);
              }
              function Pt(rt, ft) {
                return te.coerce2(St, Lt, ce, rt, ft);
              }
              function Xt(rt) {
                return rt === "x" ? et : ct;
              }
              function Nt(rt, ft) {
                for (var pt = rt === "x" ? dt : Je, wt = [], Bt = 0; Bt < pt.length; Bt++) {
                  var At = pt[Bt];
                  At === ft || (Se[At] || {}).overlaying || wt.push(pe(At));
                }
                return wt;
              }
              var Gt = { x: Xt("x"), y: Xt("y") }, Ft = Gt.x.concat(Gt.y), Tt = {}, It = [];
              function Ct() {
                var rt = St.matches;
                ye.test(rt) && Ft.indexOf(rt) === -1 && (Tt[rt] = St.type, It = Object.keys(Tt));
              }
              var Kt = ae(Se, we), Vt = ee(Kt);
              for (xe = 0; xe < it.length; xe++) {
                gt = it[xe], bt = pe(gt), yt = gt.charAt(0), te.isPlainObject(Se[gt]) || (Se[gt] = {}), St = Se[gt], Lt = oe.newContainer(we, gt, yt + "axis"), vt();
                var Yt = yt === "x" && !De[gt] && Me[gt] || yt === "y" && !Pe[gt] && Re[gt], sn = yt === "y" && (!$e[gt] && qe[gt] || Xe[gt]), hn = { hasMinor: !0, letter: yt, font: we.font, outerTicks: Ve[gt], showGrid: !Ze[gt], data: ke[gt] || [], bgColor: Rt, calendar: we.calendar, automargin: !0, visibleDflt: Yt, reverseDflt: sn, autotypenumbersDflt: Qe, splomStash: ((we._splomAxes || {})[yt] || {})[bt], noAutotickangles: yt === "y" };
                $t("uirevision", we.uirevision), le(St, Lt, $t, hn), ue(St, Lt, $t, hn, we);
                var Qt = Vt && yt === Kt.charAt(0), Ut = Pt("spikecolor", Vt ? Lt.color : void 0), Wt = Pt("spikethickness", Vt ? 1.5 : void 0), rn = Pt("spikedash", Vt ? "dot" : void 0), vn = Pt("spikemode", Vt ? "across" : void 0), dn = Pt("spikesnap");
                $t("showspikes", !!(Qt || Ut || Wt || rn || vn || dn)) || (delete Lt.spikecolor, delete Lt.spikethickness, delete Lt.spikedash, delete Lt.spikemode, delete Lt.spikesnap);
                var Ht = me(St.overlaying), jt = [0, 1];
                if (we[Ht] !== void 0) {
                  var cn = me(we[Ht].anchor);
                  we[cn] !== void 0 && (jt = we[cn].domain);
                }
                fe(St, Lt, $t, { letter: yt, counterAxes: Gt[yt], overlayableAxes: Nt(yt, gt), grid: we.grid, overlayingDomain: jt }), $t("title.standoff"), Ct(), Lt._input = St;
              }
              for (xe = 0; xe < It.length; ) {
                bt = It[xe++], yt = (gt = me(bt)).charAt(0), te.isPlainObject(Se[gt]) || (Se[gt] = {}), St = Se[gt], Lt = oe.newContainer(we, gt, yt + "axis"), vt();
                var wn = { letter: yt, font: we.font, outerTicks: Ve[gt], showGrid: !Ze[gt], data: [], bgColor: Rt, calendar: we.calendar, automargin: !0, visibleDflt: !1, reverseDflt: !1, autotypenumbersDflt: Qe, splomStash: ((we._splomAxes || {})[yt] || {})[bt] };
                $t("uirevision", we.uirevision), Lt.type = Tt[bt] || "linear", ue(St, Lt, $t, wn, we), fe(St, Lt, $t, { letter: yt, counterAxes: Gt[yt], overlayableAxes: Nt(yt, gt), grid: we.grid }), $t("fixedrange"), Ct(), Lt._input = St;
              }
              var Rn = be("rangeslider", "handleDefaults"), Pn = be("rangeselector", "handleDefaults");
              for (xe = 0; xe < dt.length; xe++) gt = dt[xe], St = Se[gt], Lt = we[gt], Rn(Se, we, gt), Lt.type === "date" && Pn(St, Lt, we, Je, Lt.calendar), $t("fixedrange");
              for (xe = 0; xe < Je.length; xe++) {
                gt = Je[xe], St = Se[gt], Lt = we[gt];
                var Ie = we[me(Lt.anchor)];
                $t("fixedrange", be("rangeslider", "isVisible")(Ie));
              }
              de.handleDefaults(Se, we, { axIds: Ft.concat(It).sort(he.idSort), axHasImage: Xe });
            };
          }, 97405: function(G, J, Z) {
            var te = Z(65657).mix, ne = Z(10229), ee = Z(34809);
            G.exports = function(ae, oe, se, ce) {
              var le = (ce = ce || {}).dfltColor;
              function ue(Se, we) {
                return ee.coerce2(ae, oe, ce.attributes, Se, we);
              }
              var de = ue("linecolor", le), fe = ue("linewidth");
              se("showline", ce.showLine || !!de || !!fe) || (delete oe.linecolor, delete oe.linewidth);
              var he = ue("gridcolor", te(le, ce.bgColor, ce.blend || ne.lightFraction).toRgbString()), me = ue("gridwidth"), pe = ue("griddash");
              if (se("showgrid", ce.showGrid || !!he || !!me || !!pe) || (delete oe.gridcolor, delete oe.gridwidth, delete oe.griddash), ce.hasMinor) {
                var ye = ue("minor.gridcolor", te(oe.gridcolor, ce.bgColor, 67).toRgbString()), ge = ue("minor.gridwidth", oe.gridwidth || 1), ve = ue("minor.griddash", oe.griddash || "solid");
                se("minor.showgrid", !!ye || !!ge || !!ve) || (delete oe.minor.gridcolor, delete oe.minor.gridwidth, delete oe.minor.griddash);
              }
              if (!ce.noZeroLine) {
                var be = ue("zerolinecolor", le), Ce = ue("zerolinewidth");
                se("zeroline", ce.showGrid || !!be || !!Ce) || (delete oe.zerolinecolor, delete oe.zerolinewidth);
              }
            };
          }, 40957: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809);
            G.exports = function(ee, ae, oe, se) {
              var ce, le, ue, de, fe, he, me = se.counterAxes || [], pe = se.overlayableAxes || [], ye = se.letter, ge = se.grid, ve = se.overlayingDomain;
              ge && (le = ge._domains[ye][ge._axisMap[ae._id]], ce = ge._anchors[ae._id], le && (ue = ge[ye + "side"].split(" ")[0], de = ge.domain[ye][ue === "right" || ue === "top" ? 1 : 0])), le = le || [0, 1], ce = ce || (te(ee.position) ? "free" : me[0] || "free"), ue = ue || (ye === "x" ? "bottom" : "left"), de = de || 0, fe = 0, he = !1;
              var be = ne.coerce(ee, ae, { anchor: { valType: "enumerated", values: ["free"].concat(me), dflt: ce } }, "anchor"), Ce = ne.coerce(ee, ae, { side: { valType: "enumerated", values: ye === "x" ? ["bottom", "top"] : ["left", "right"], dflt: ue } }, "side");
              be === "free" && (ye === "y" && (oe("autoshift") && (de = Ce === "left" ? ve[0] : ve[1], he = !ae.automargin || ae.automargin, fe = Ce === "left" ? -3 : 3), oe("shift", fe)), oe("position", de)), oe("automargin", he);
              var Se = !1;
              if (pe.length && (Se = ne.coerce(ee, ae, { overlaying: { valType: "enumerated", values: [!1].concat(pe), dflt: !1 } }, "overlaying")), !Se) {
                var we = oe("domain", le);
                we[0] > we[1] - 244140625e-12 && (ae.domain = le), ne.noneOrAll(ee.domain, ae.domain, le), ae.tickmode === "sync" && (ae.tickmode = "auto");
              }
              return oe("layer"), ae;
            };
          }, 54616: function(G, J, Z) {
            var te = Z(87703);
            G.exports = function(ne, ee, ae, oe, se) {
              se || (se = {});
              var ce = se.tickSuffixDflt, le = te(ne);
              ae("tickprefix") && ae("showtickprefix", le), ae("ticksuffix", ce) && ae("showticksuffix", le);
            };
          }, 90259: function(G, J, Z) {
            var te = Z(75511);
            G.exports = function(ne, ee, ae, oe) {
              var se = ee._template || {}, ce = ee.type || se.type || "-";
              ae("minallowed"), ae("maxallowed");
              var le, ue = ae("range");
              ue || oe.noInsiderange || ce === "log" || (!(le = ae("insiderange")) || le[0] !== null && le[1] !== null || (ee.insiderange = !1, le = void 0), le && (ue = ae("range", le)));
              var de, fe = ee.getAutorangeDflt(ue, oe), he = ae("autorange", fe);
              !ue || (ue[0] !== null || ue[1] !== null) && (ue[0] !== null && ue[1] !== null || he !== "reversed" && he !== !0) && (ue[0] === null || he !== "min" && he !== "max reversed") && (ue[1] === null || he !== "max" && he !== "min reversed") || (ue = void 0, delete ee.range, ee.autorange = !0, de = !0), de || (he = ae("autorange", fe = ee.getAutorangeDflt(ue, oe))), he && (te(ae, he, ue), ce !== "linear" && ce !== "-" || ae("rangemode")), ee.cleanRange();
            };
          }, 67611: function(G, J, Z) {
            var te = Z(4530).FROM_BL;
            G.exports = function(ne, ee, ae) {
              ae === void 0 && (ae = te[ne.constraintoward || "center"]);
              var oe = [ne.r2l(ne.range[0]), ne.r2l(ne.range[1])], se = oe[0] + (oe[1] - oe[0]) * ae;
              ne.range = ne._input.range = [ne.l2r(se + (oe[0] - se) * ee), ne.l2r(se + (oe[1] - se) * ee)], ne.setScale();
            };
          }, 19091: function(G, J, Z) {
            var te = Z(45568), ne = Z(42696).aL, ee = Z(34809), ae = ee.numberFormat, oe = Z(10721), se = ee.cleanNumber, ce = ee.ms2DateTime, le = ee.dateTime2ms, ue = ee.ensureNumber, de = ee.isArrayOrTypedArray, fe = Z(63821), he = fe.FP_SAFE, me = fe.BADNUM, pe = fe.LOG_CLIP, ye = fe.ONEWEEK, ge = fe.ONEDAY, ve = fe.ONEHOUR, be = fe.ONEMIN, Ce = fe.ONESEC, Se = Z(5975), we = Z(54826), Ee = we.HOUR_PATTERN, xe = we.WEEKDAY_PATTERN;
            function Le(ke) {
              return Math.pow(10, ke);
            }
            function Qe(ke) {
              return ke != null;
            }
            G.exports = function(ke, Me) {
              Me = Me || {};
              var Re = ke._id || "x", De = Re.charAt(0);
              function Pe(it, st) {
                if (it > 0) return Math.log(it) / Math.LN10;
                if (it <= 0 && st && ke.range && ke.range.length === 2) {
                  var gt = ke.range[0], bt = ke.range[1];
                  return 0.5 * (gt + bt - 2 * pe * Math.abs(gt - bt));
                }
                return me;
              }
              function $e(it, st, gt, bt) {
                if ((bt || {}).msUTC && oe(it)) return +it;
                var yt = le(it, gt || ke.calendar);
                if (yt === me) {
                  if (!oe(it)) return me;
                  it = +it;
                  var St = Math.floor(10 * ee.mod(it + 0.05, 1)), Lt = Math.round(it - St / 10);
                  yt = le(new Date(Lt)) + St / 10;
                }
                return yt;
              }
              function qe(it, st, gt) {
                return ce(it, st, gt || ke.calendar);
              }
              function Xe(it) {
                return ke._categories[Math.round(it)];
              }
              function Ve(it) {
                if (Qe(it)) {
                  if (ke._categoriesMap === void 0 && (ke._categoriesMap = {}), ke._categoriesMap[it] !== void 0) return ke._categoriesMap[it];
                  ke._categories.push(typeof it == "number" ? String(it) : it);
                  var st = ke._categories.length - 1;
                  return ke._categoriesMap[it] = st, st;
                }
                return me;
              }
              function Ze(it) {
                if (ke._categoriesMap) return ke._categoriesMap[it];
              }
              function Ge(it) {
                var st = Ze(it);
                return st !== void 0 ? st : oe(it) ? +it : void 0;
              }
              function ze(it) {
                return oe(it) ? +it : Ze(it);
              }
              function We(it, st, gt) {
                return te.round(gt + st * it, 2);
              }
              function at(it, st, gt) {
                return (it - gt) / st;
              }
              var ct = function(it) {
                return oe(it) ? We(it, ke._m, ke._b) : me;
              }, et = function(it) {
                return at(it, ke._m, ke._b);
              };
              if (ke.rangebreaks) {
                var dt = De === "y";
                ct = function(it) {
                  if (!oe(it)) return me;
                  var st = ke._rangebreaks.length;
                  if (!st) return We(it, ke._m, ke._b);
                  var gt = dt;
                  ke.range[0] > ke.range[1] && (gt = !gt);
                  for (var bt = gt ? -1 : 1, yt = bt * it, St = 0, Lt = 0; Lt < st; Lt++) {
                    var Rt = bt * ke._rangebreaks[Lt].min, vt = bt * ke._rangebreaks[Lt].max;
                    if (yt < Rt) break;
                    if (!(yt > vt)) {
                      St = yt < (Rt + vt) / 2 ? Lt : Lt + 1;
                      break;
                    }
                    St = Lt + 1;
                  }
                  var $t = ke._B[St] || 0;
                  return isFinite($t) ? We(it, ke._m2, $t) : 0;
                }, et = function(it) {
                  var st = ke._rangebreaks.length;
                  if (!st) return at(it, ke._m, ke._b);
                  for (var gt = 0, bt = 0; bt < st && !(it < ke._rangebreaks[bt].pmin); bt++) it > ke._rangebreaks[bt].pmax && (gt = bt + 1);
                  return at(it, ke._m2, ke._B[gt]);
                };
              }
              ke.c2l = ke.type === "log" ? Pe : ue, ke.l2c = ke.type === "log" ? Le : ue, ke.l2p = ct, ke.p2l = et, ke.c2p = ke.type === "log" ? function(it, st) {
                return ct(Pe(it, st));
              } : ct, ke.p2c = ke.type === "log" ? function(it) {
                return Le(et(it));
              } : et, ["linear", "-"].indexOf(ke.type) !== -1 ? (ke.d2r = ke.r2d = ke.d2c = ke.r2c = ke.d2l = ke.r2l = se, ke.c2d = ke.c2r = ke.l2d = ke.l2r = ue, ke.d2p = ke.r2p = function(it) {
                return ke.l2p(se(it));
              }, ke.p2d = ke.p2r = et, ke.cleanPos = ue) : ke.type === "log" ? (ke.d2r = ke.d2l = function(it, st) {
                return Pe(se(it), st);
              }, ke.r2d = ke.r2c = function(it) {
                return Le(se(it));
              }, ke.d2c = ke.r2l = se, ke.c2d = ke.l2r = ue, ke.c2r = Pe, ke.l2d = Le, ke.d2p = function(it, st) {
                return ke.l2p(ke.d2r(it, st));
              }, ke.p2d = function(it) {
                return Le(et(it));
              }, ke.r2p = function(it) {
                return ke.l2p(se(it));
              }, ke.p2r = et, ke.cleanPos = ue) : ke.type === "date" ? (ke.d2r = ke.r2d = ee.identity, ke.d2c = ke.r2c = ke.d2l = ke.r2l = $e, ke.c2d = ke.c2r = ke.l2d = ke.l2r = qe, ke.d2p = ke.r2p = function(it, st, gt) {
                return ke.l2p($e(it, 0, gt));
              }, ke.p2d = ke.p2r = function(it, st, gt) {
                return qe(et(it), st, gt);
              }, ke.cleanPos = function(it) {
                return ee.cleanDate(it, me, ke.calendar);
              }) : ke.type === "category" ? (ke.d2c = ke.d2l = Ve, ke.r2d = ke.c2d = ke.l2d = Xe, ke.d2r = ke.d2l_noadd = Ge, ke.r2c = function(it) {
                var st = ze(it);
                return st !== void 0 ? st : ke.fraction2r(0.5);
              }, ke.l2r = ke.c2r = ue, ke.r2l = ze, ke.d2p = function(it) {
                return ke.l2p(ke.r2c(it));
              }, ke.p2d = function(it) {
                return Xe(et(it));
              }, ke.r2p = ke.d2p, ke.p2r = et, ke.cleanPos = function(it) {
                return typeof it == "string" && it !== "" ? it : ue(it);
              }) : ke.type === "multicategory" && (ke.r2d = ke.c2d = ke.l2d = Xe, ke.d2r = ke.d2l_noadd = Ge, ke.r2c = function(it) {
                var st = Ge(it);
                return st !== void 0 ? st : ke.fraction2r(0.5);
              }, ke.r2c_just_indices = Ze, ke.l2r = ke.c2r = ue, ke.r2l = Ge, ke.d2p = function(it) {
                return ke.l2p(ke.r2c(it));
              }, ke.p2d = function(it) {
                return Xe(et(it));
              }, ke.r2p = ke.d2p, ke.p2r = et, ke.cleanPos = function(it) {
                return Array.isArray(it) || typeof it == "string" && it !== "" ? it : ue(it);
              }, ke.setupMultiCategory = function(it) {
                var st, gt, bt = ke._traceIndices, yt = ke._matchGroup;
                if (yt && ke._categories.length === 0) {
                  for (var St in yt) if (St !== Re) {
                    var Lt = Me[Se.id2name(St)];
                    bt = bt.concat(Lt._traceIndices);
                  }
                }
                var Rt = [[0, {}], [0, {}]], vt = [];
                for (st = 0; st < bt.length; st++) {
                  var $t = it[bt[st]];
                  if (De in $t) {
                    var Pt = $t[De], Xt = $t._length || ee.minRowLength(Pt);
                    if (de(Pt[0]) && de(Pt[1])) for (gt = 0; gt < Xt; gt++) {
                      var Nt = Pt[0][gt], Gt = Pt[1][gt];
                      Qe(Nt) && Qe(Gt) && (vt.push([Nt, Gt]), Nt in Rt[0][1] || (Rt[0][1][Nt] = Rt[0][0]++), Gt in Rt[1][1] || (Rt[1][1][Gt] = Rt[1][0]++));
                    }
                  }
                }
                for (vt.sort(function(Ft, Tt) {
                  var It = Rt[0][1], Ct = It[Ft[0]] - It[Tt[0]];
                  if (Ct) return Ct;
                  var Kt = Rt[1][1];
                  return Kt[Ft[1]] - Kt[Tt[1]];
                }), st = 0; st < vt.length; st++) Ve(vt[st]);
              }), ke.fraction2r = function(it) {
                var st = ke.r2l(ke.range[0]), gt = ke.r2l(ke.range[1]);
                return ke.l2r(st + it * (gt - st));
              }, ke.r2fraction = function(it) {
                var st = ke.r2l(ke.range[0]), gt = ke.r2l(ke.range[1]);
                return (ke.r2l(it) - st) / (gt - st);
              }, ke.limitRange = function(it) {
                var st = ke.minallowed, gt = ke.maxallowed;
                if (st !== void 0 || gt !== void 0) {
                  it || (it = "range");
                  var bt = ee.nestedProperty(ke, it).get(), yt = ee.simpleMap(bt, ke.r2l), St = yt[1] < yt[0];
                  St && yt.reverse();
                  var Lt = ee.simpleMap([st, gt], ke.r2l);
                  if (st !== void 0 && yt[0] < Lt[0] && (bt[St ? 1 : 0] = st), gt !== void 0 && yt[1] > Lt[1] && (bt[St ? 0 : 1] = gt), bt[0] === bt[1]) {
                    var Rt = ke.l2r(st), vt = ke.l2r(gt);
                    if (st !== void 0) {
                      var $t = Rt + 1;
                      gt !== void 0 && ($t = Math.min($t, vt)), bt[St ? 1 : 0] = $t;
                    }
                    if (gt !== void 0) {
                      var Pt = vt + 1;
                      st !== void 0 && (Pt = Math.max(Pt, Rt)), bt[St ? 0 : 1] = Pt;
                    }
                  }
                }
              }, ke.cleanRange = function(it, st) {
                ke._cleanRange(it, st), ke.limitRange(it);
              }, ke._cleanRange = function(it, st) {
                st || (st = {}), it || (it = "range");
                var gt, bt, yt = ee.nestedProperty(ke, it).get();
                if (bt = (bt = ke.type === "date" ? ee.dfltRange(ke.calendar) : De === "y" ? we.DFLTRANGEY : ke._name === "realaxis" ? [0, 1] : st.dfltRange || we.DFLTRANGEX).slice(), ke.rangemode !== "tozero" && ke.rangemode !== "nonnegative" || (bt[0] = 0), yt && yt.length === 2) {
                  var St = yt[0] === null, Lt = yt[1] === null;
                  for (ke.type !== "date" || ke.autorange || (yt[0] = ee.cleanDate(yt[0], me, ke.calendar), yt[1] = ee.cleanDate(yt[1], me, ke.calendar)), gt = 0; gt < 2; gt++) if (ke.type === "date") {
                    if (!ee.isDateTime(yt[gt], ke.calendar)) {
                      ke[it] = bt;
                      break;
                    }
                    if (ke.r2l(yt[0]) === ke.r2l(yt[1])) {
                      var Rt = ee.constrain(ke.r2l(yt[0]), ee.MIN_MS + 1e3, ee.MAX_MS - 1e3);
                      yt[0] = ke.l2r(Rt - 1e3), yt[1] = ke.l2r(Rt + 1e3);
                      break;
                    }
                  } else {
                    if (!oe(yt[gt])) {
                      if (St || Lt || !oe(yt[1 - gt])) {
                        ke[it] = bt;
                        break;
                      }
                      yt[gt] = yt[1 - gt] * (gt ? 10 : 0.1);
                    }
                    if (yt[gt] < -he ? yt[gt] = -he : yt[gt] > he && (yt[gt] = he), yt[0] === yt[1]) {
                      var vt = Math.max(1, Math.abs(1e-6 * yt[0]));
                      yt[0] -= vt, yt[1] += vt;
                    }
                  }
                } else ee.nestedProperty(ke, it).set(bt);
              }, ke.setScale = function(it) {
                var st = Me._size;
                if (ke.overlaying) {
                  var gt = Se.getFromId({ _fullLayout: Me }, ke.overlaying);
                  ke.domain = gt.domain;
                }
                var bt = it && ke._r ? "_r" : "range", yt = ke.calendar;
                ke.cleanRange(bt);
                var St, Lt, Rt = ke.r2l(ke[bt][0], yt), vt = ke.r2l(ke[bt][1], yt), $t = De === "y";
                if ($t ? (ke._offset = st.t + (1 - ke.domain[1]) * st.h, ke._length = st.h * (ke.domain[1] - ke.domain[0]), ke._m = ke._length / (Rt - vt), ke._b = -ke._m * vt) : (ke._offset = st.l + ke.domain[0] * st.w, ke._length = st.w * (ke.domain[1] - ke.domain[0]), ke._m = ke._length / (vt - Rt), ke._b = -ke._m * Rt), ke._rangebreaks = [], ke._lBreaks = 0, ke._m2 = 0, ke._B = [], ke.rangebreaks && (ke._rangebreaks = ke.locateBreaks(Math.min(Rt, vt), Math.max(Rt, vt)), ke._rangebreaks.length)) {
                  for (St = 0; St < ke._rangebreaks.length; St++) Lt = ke._rangebreaks[St], ke._lBreaks += Math.abs(Lt.max - Lt.min);
                  var Pt = $t;
                  Rt > vt && (Pt = !Pt), Pt && ke._rangebreaks.reverse();
                  var Xt = Pt ? -1 : 1;
                  for (ke._m2 = Xt * ke._length / (Math.abs(vt - Rt) - ke._lBreaks), ke._B.push(-ke._m2 * ($t ? vt : Rt)), St = 0; St < ke._rangebreaks.length; St++) Lt = ke._rangebreaks[St], ke._B.push(ke._B[ke._B.length - 1] - Xt * ke._m2 * (Lt.max - Lt.min));
                  for (St = 0; St < ke._rangebreaks.length; St++) (Lt = ke._rangebreaks[St]).pmin = ct(Lt.min), Lt.pmax = ct(Lt.max);
                }
                if (!isFinite(ke._m) || !isFinite(ke._b) || ke._length < 0) throw Me._replotting = !1, new Error("Something went wrong with axis scaling");
              }, ke.maskBreaks = function(it) {
                var st, gt, bt, yt, St, Lt = ke.rangebreaks || [];
                Lt._cachedPatterns || (Lt._cachedPatterns = Lt.map(function(Nt) {
                  return Nt.enabled && Nt.bounds ? ee.simpleMap(Nt.bounds, Nt.pattern ? se : ke.d2c) : null;
                })), Lt._cachedValues || (Lt._cachedValues = Lt.map(function(Nt) {
                  return Nt.enabled && Nt.values ? ee.simpleMap(Nt.values, ke.d2c).sort(ee.sorterAsc) : null;
                }));
                for (var Rt = 0; Rt < Lt.length; Rt++) {
                  var vt = Lt[Rt];
                  if (vt.enabled) {
                    if (vt.bounds) {
                      var $t = vt.pattern;
                      switch (gt = (st = Lt._cachedPatterns[Rt])[0], bt = st[1], $t) {
                        case xe:
                          yt = (St = new Date(it)).getUTCDay(), gt > bt && (bt += 7, yt < gt && (yt += 7));
                          break;
                        case Ee:
                          yt = (St = new Date(it)).getUTCHours() + (St.getUTCMinutes() / 60 + St.getUTCSeconds() / 3600 + St.getUTCMilliseconds() / 36e5), gt > bt && (bt += 24, yt < gt && (yt += 24));
                          break;
                        case "":
                          yt = it;
                      }
                      if (yt >= gt && yt < bt) return me;
                    } else for (var Pt = Lt._cachedValues[Rt], Xt = 0; Xt < Pt.length; Xt++) if (bt = (gt = Pt[Xt]) + vt.dvalue, it >= gt && it < bt) return me;
                  }
                }
                return it;
              }, ke.locateBreaks = function(it, st) {
                var gt, bt, yt, St, Lt = [];
                if (!ke.rangebreaks) return Lt;
                var Rt = ke.rangebreaks.slice().sort(function(Kt, Vt) {
                  return Kt.pattern === xe && Vt.pattern === Ee ? -1 : Vt.pattern === xe && Kt.pattern === Ee ? 1 : 0;
                }), vt = function(Kt, Vt) {
                  if ((Kt = ee.constrain(Kt, it, st)) !== (Vt = ee.constrain(Vt, it, st))) {
                    for (var Yt = !0, sn = 0; sn < Lt.length; sn++) {
                      var hn = Lt[sn];
                      Kt < hn.max && Vt >= hn.min && (Kt < hn.min && (hn.min = Kt), Vt > hn.max && (hn.max = Vt), Yt = !1);
                    }
                    Yt && Lt.push({ min: Kt, max: Vt });
                  }
                };
                for (gt = 0; gt < Rt.length; gt++) {
                  var $t = Rt[gt];
                  if ($t.enabled) if ($t.bounds) {
                    var Pt = it, Xt = st;
                    $t.pattern && (Pt = Math.floor(Pt)), yt = (bt = ee.simpleMap($t.bounds, $t.pattern ? se : ke.r2l))[0], St = bt[1];
                    var Nt, Gt, Ft = new Date(Pt);
                    switch ($t.pattern) {
                      case xe:
                        Gt = ye, Nt = (St - yt + (St < yt ? 7 : 0)) * ge, Pt += yt * ge - (Ft.getUTCDay() * ge + Ft.getUTCHours() * ve + Ft.getUTCMinutes() * be + Ft.getUTCSeconds() * Ce + Ft.getUTCMilliseconds());
                        break;
                      case Ee:
                        Gt = ge, Nt = (St - yt + (St < yt ? 24 : 0)) * ve, Pt += yt * ve - (Ft.getUTCHours() * ve + Ft.getUTCMinutes() * be + Ft.getUTCSeconds() * Ce + Ft.getUTCMilliseconds());
                        break;
                      default:
                        Pt = Math.min(bt[0], bt[1]), Nt = Gt = (Xt = Math.max(bt[0], bt[1])) - Pt;
                    }
                    for (var Tt = Pt; Tt < Xt; Tt += Gt) vt(Tt, Tt + Nt);
                  } else for (var It = ee.simpleMap($t.values, ke.d2c), Ct = 0; Ct < It.length; Ct++) vt(yt = It[Ct], St = yt + $t.dvalue);
                }
                return Lt.sort(function(Kt, Vt) {
                  return Kt.min - Vt.min;
                }), Lt;
              }, ke.makeCalcdata = function(it, st, gt) {
                var bt, yt, St, Lt, Rt = ke.type, vt = Rt === "date" && it[st + "calendar"];
                if (st in it) {
                  if (bt = it[st], Lt = it._length || ee.minRowLength(bt), ee.isTypedArray(bt) && (Rt === "linear" || Rt === "log")) {
                    if (Lt === bt.length) return bt;
                    if (bt.subarray) return bt.subarray(0, Lt);
                  }
                  if (Rt === "multicategory") return function(Xt, Nt) {
                    for (var Gt = new Array(Nt), Ft = 0; Ft < Nt; Ft++) {
                      var Tt = (Xt[0] || [])[Ft], It = (Xt[1] || [])[Ft];
                      Gt[Ft] = Ze([Tt, It]);
                    }
                    return Gt;
                  }(bt, Lt);
                  for (yt = new Array(Lt), St = 0; St < Lt; St++) yt[St] = ke.d2c(bt[St], 0, vt, gt);
                } else {
                  var $t = st + "0" in it ? ke.d2c(it[st + "0"], 0, vt) : 0, Pt = it["d" + st] ? Number(it["d" + st]) : 1;
                  for (bt = it[{ x: "y", y: "x" }[st]], Lt = it._length || bt.length, yt = new Array(Lt), St = 0; St < Lt; St++) yt[St] = $t + St * Pt;
                }
                if (ke.rangebreaks) for (St = 0; St < Lt; St++) yt[St] = ke.maskBreaks(yt[St]);
                return yt;
              }, ke.isValidRange = function(it, st) {
                return Array.isArray(it) && it.length === 2 && (st && it[0] === null || oe(ke.r2l(it[0]))) && (st && it[1] === null || oe(ke.r2l(it[1])));
              }, ke.getAutorangeDflt = function(it, st) {
                var gt = !ke.isValidRange(it, "nullOk");
                return gt && st && st.reverseDflt ? gt = "reversed" : it && (it[0] === null && it[1] === null ? gt = !0 : it[0] === null && it[1] !== null ? gt = "min" : it[0] !== null && it[1] === null && (gt = "max")), gt;
              }, ke.isReversed = function() {
                var it = ke.autorange;
                return it === "reversed" || it === "min reversed" || it === "max reversed";
              }, ke.isPtWithinRange = function(it, st) {
                var gt = ke.c2l(it[De], null, st), bt = ke.r2l(ke.range[0]), yt = ke.r2l(ke.range[1]);
                return bt < yt ? bt <= gt && gt <= yt : yt <= gt && gt <= bt;
              }, ke._emptyCategories = function() {
                ke._categories = [], ke._categoriesMap = {};
              }, ke.clearCalc = function() {
                var it = ke._matchGroup;
                if (it) {
                  var st = null, gt = null;
                  for (var bt in it) {
                    var yt = Me[Se.id2name(bt)];
                    if (yt._categories) {
                      st = yt._categories, gt = yt._categoriesMap;
                      break;
                    }
                  }
                  st && gt ? (ke._categories = st, ke._categoriesMap = gt) : ke._emptyCategories();
                } else ke._emptyCategories();
                if (ke._initialCategories) for (var St = 0; St < ke._initialCategories.length; St++) Ve(ke._initialCategories[St]);
              }, ke.sortByInitialCategories = function() {
                var it = [];
                if (ke._emptyCategories(), ke._initialCategories) for (var st = 0; st < ke._initialCategories.length; st++) Ve(ke._initialCategories[st]);
                it = it.concat(ke._traceIndices);
                var gt = ke._matchGroup;
                for (var bt in gt) if (Re !== bt) {
                  var yt = Me[Se.id2name(bt)];
                  yt._categories = ke._categories, yt._categoriesMap = ke._categoriesMap, it = it.concat(yt._traceIndices);
                }
                return it;
              };
              var Je = Me._d3locale;
              ke.type === "date" && (ke._dateFormat = Je ? Je.timeFormat : ne, ke._extraFormat = Me._extraFormat), ke._separators = Me.separators, ke._numFormat = Je ? Je.numberFormat : ae, delete ke._minDtick, delete ke._forceTick0;
            };
          }, 87703: function(G) {
            G.exports = function(J) {
              var Z = ["showexponent", "showtickprefix", "showticksuffix"].filter(function(te) {
                return J[te] !== void 0;
              });
              if (Z.every(function(te) {
                return J[te] === J[Z[0]];
              }) || Z.length === 1) return J[Z[0]];
            };
          }, 12036: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766).contrast, ee = Z(25829), ae = Z(87703), oe = Z(59008);
            function se(ce, le) {
              function ue(de, fe) {
                return te.coerce(ce, le, ee.tickformatstops, de, fe);
              }
              ue("enabled") && (ue("dtickrange"), ue("value"));
            }
            G.exports = function(ce, le, ue, de, fe) {
              fe || (fe = {});
              var he = ue("labelalias");
              te.isPlainObject(he) || delete le.labelalias;
              var me = ae(ce);
              if (ue("showticklabels")) {
                fe.noTicklabelshift || ue("ticklabelshift"), fe.noTicklabelstandoff || ue("ticklabelstandoff");
                var pe = fe.font || {}, ye = le.color, ge = (le.ticklabelposition || "").indexOf("inside") !== -1 ? ne(fe.bgColor) : ye && ye !== ee.color.dflt ? ye : pe.color;
                if (te.coerceFont(ue, "tickfont", pe, { overrideDflt: { color: ge } }), fe.noTicklabelstep || de === "multicategory" || de === "log" || ue("ticklabelstep"), !fe.noAng) {
                  var ve = ue("tickangle");
                  fe.noAutotickangles || ve !== "auto" || ue("autotickangles");
                }
                if (de !== "category") {
                  var be = ue("tickformat");
                  oe(ce, le, { name: "tickformatstops", inclusionAttr: "enabled", handleItemDefaults: se }), le.tickformatstops.length || delete le.tickformatstops, fe.noExp || be || de === "date" || (ue("showexponent", me), ue("exponentformat"), ue("minexponent"), ue("separatethousands"));
                }
              }
            };
          }, 87433: function(G, J, Z) {
            var te = Z(34809), ne = Z(25829);
            G.exports = function(ee, ae, oe, se) {
              var ce = se.isMinor, le = ce ? ee.minor || {} : ee, ue = ce ? ae.minor : ae, de = ce ? ne.minor : ne, fe = ce ? "minor." : "", he = te.coerce2(le, ue, de, "ticklen", ce ? 0.6 * (ae.ticklen || 5) : void 0), me = te.coerce2(le, ue, de, "tickwidth", ce ? ae.tickwidth || 1 : void 0), pe = te.coerce2(le, ue, de, "tickcolor", (ce ? ae.tickcolor : void 0) || ue.color);
              oe(fe + "ticks", !ce && se.outerTicks || he || me || pe ? "outside" : "") || (delete ue.ticklen, delete ue.tickwidth, delete ue.tickcolor);
            };
          }, 22777: function(G, J, Z) {
            var te = Z(68599), ne = Z(34809).isArrayOrTypedArray, ee = Z(87800).isTypedArraySpec, ae = Z(87800).decodeTypedArraySpec;
            G.exports = function(oe, se, ce, le, ue) {
              ue || (ue = {});
              var de = ue.isMinor, fe = de ? oe.minor || {} : oe, he = de ? se.minor : se, me = de ? "minor." : "";
              function pe(Se) {
                var we = fe[Se];
                return ee(we) && (we = ae(we)), we !== void 0 ? we : (he._template || {})[Se];
              }
              var ye = pe("tick0"), ge = pe("dtick"), ve = pe("tickvals"), be = ce(me + "tickmode", ne(ve) ? "array" : ge ? "linear" : "auto");
              if (be === "auto" || be === "sync") ce(me + "nticks");
              else if (be === "linear") {
                var Ce = he.dtick = te.dtick(ge, le);
                he.tick0 = te.tick0(ye, le, se.calendar, Ce);
              } else le !== "multicategory" && (ce(me + "tickvals") === void 0 ? he.tickmode = "auto" : de || ce("ticktext"));
            };
          }, 84982: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(34809), ae = Z(62203), oe = Z(29714);
            G.exports = function(se, ce, le, ue) {
              var de = se._fullLayout;
              if (ce.length !== 0) {
                var fe, he, me, pe;
                ue && (fe = ue());
                var ye = te.ease(le.easing);
                return se._transitionData._interruptCallbacks.push(function() {
                  return window.cancelAnimationFrame(pe), pe = null, function() {
                    for (var be = {}, Ce = 0; Ce < ce.length; Ce++) {
                      var Se = ce[Ce], we = Se.plotinfo.xaxis, Ee = Se.plotinfo.yaxis;
                      Se.xr0 && (be[we._name + ".range"] = Se.xr0.slice()), Se.yr0 && (be[Ee._name + ".range"] = Se.yr0.slice());
                    }
                    return ne.call("relayout", se, be).then(function() {
                      for (var xe = 0; xe < ce.length; xe++) ge(ce[xe].plotinfo);
                    });
                  }();
                }), he = Date.now(), pe = window.requestAnimationFrame(function be() {
                  me = Date.now();
                  for (var Ce = Math.min(1, (me - he) / le.duration), Se = ye(Ce), we = 0; we < ce.length; we++) ve(ce[we], Se);
                  me - he > le.duration ? (function() {
                    for (var Ee = {}, xe = 0; xe < ce.length; xe++) {
                      var Le = ce[xe], Qe = Le.plotinfo.xaxis, ke = Le.plotinfo.yaxis;
                      Le.xr1 && (Ee[Qe._name + ".range"] = Le.xr1.slice()), Le.yr1 && (Ee[ke._name + ".range"] = Le.yr1.slice());
                    }
                    fe && fe(), ne.call("relayout", se, Ee).then(function() {
                      for (var Me = 0; Me < ce.length; Me++) ge(ce[Me].plotinfo);
                    });
                  }(), pe = window.cancelAnimationFrame(be)) : pe = window.requestAnimationFrame(be);
                }), Promise.resolve();
              }
              function ge(be) {
                var Ce = be.xaxis, Se = be.yaxis;
                de._defs.select("#" + be.clipId + "> rect").call(ae.setTranslate, 0, 0).call(ae.setScale, 1, 1), be.plot.call(ae.setTranslate, Ce._offset, Se._offset).call(ae.setScale, 1, 1);
                var we = be.plot.selectAll(".scatterlayer .trace");
                we.selectAll(".point").call(ae.setPointGroupScale, 1, 1), we.selectAll(".textpoint").call(ae.setTextPointsScale, 1, 1), we.call(ae.hideOutsideRangePoints, be);
              }
              function ve(be, Ce) {
                var Se = be.plotinfo, we = Se.xaxis, Ee = Se.yaxis, xe = we._length, Le = Ee._length, Qe = !!be.xr1, ke = !!be.yr1, Me = [];
                if (Qe) {
                  var Re = ee.simpleMap(be.xr0, we.r2l), De = ee.simpleMap(be.xr1, we.r2l), Pe = Re[1] - Re[0], $e = De[1] - De[0];
                  Me[0] = (Re[0] * (1 - Ce) + Ce * De[0] - Re[0]) / (Re[1] - Re[0]) * xe, Me[2] = xe * (1 - Ce + Ce * $e / Pe), we.range[0] = we.l2r(Re[0] * (1 - Ce) + Ce * De[0]), we.range[1] = we.l2r(Re[1] * (1 - Ce) + Ce * De[1]);
                } else Me[0] = 0, Me[2] = xe;
                if (ke) {
                  var qe = ee.simpleMap(be.yr0, Ee.r2l), Xe = ee.simpleMap(be.yr1, Ee.r2l), Ve = qe[1] - qe[0], Ze = Xe[1] - Xe[0];
                  Me[1] = (qe[1] * (1 - Ce) + Ce * Xe[1] - qe[1]) / (qe[0] - qe[1]) * Le, Me[3] = Le * (1 - Ce + Ce * Ze / Ve), Ee.range[0] = we.l2r(qe[0] * (1 - Ce) + Ce * Xe[0]), Ee.range[1] = Ee.l2r(qe[1] * (1 - Ce) + Ce * Xe[1]);
                } else Me[1] = 0, Me[3] = Le;
                oe.drawOne(se, we, { skipTitle: !0 }), oe.drawOne(se, Ee, { skipTitle: !0 }), oe.redrawComponents(se, [we._id, Ee._id]);
                var Ge = Qe ? xe / Me[2] : 1, ze = ke ? Le / Me[3] : 1, We = Qe ? Me[0] : 0, at = ke ? Me[1] : 0, ct = Qe ? Me[0] / Me[2] * xe : 0, et = ke ? Me[1] / Me[3] * Le : 0, dt = we._offset - ct, Je = Ee._offset - et;
                Se.clipRect.call(ae.setTranslate, We, at).call(ae.setScale, 1 / Ge, 1 / ze), Se.plot.call(ae.setTranslate, dt, Je).call(ae.setScale, Ge, ze), ae.setPointGroupScale(Se.zoomScalePts, 1 / Ge, 1 / ze), ae.setTextPointsScale(Se.zoomScaleTxt, 1 / Ge, 1 / ze);
              }
              oe.redrawComponents(se);
            };
          }, 4392: function(G, J, Z) {
            var te = Z(33626).traceIs, ne = Z(9666);
            function ee(oe) {
              return { v: "x", h: "y" }[oe.orientation || "v"];
            }
            function ae(oe, se) {
              var ce = ee(oe), le = te(oe, "box-violin"), ue = te(oe._fullInput || {}, "candlestick");
              return le && !ue && se === ce && oe[ce] === void 0 && oe[ce + "0"] === void 0;
            }
            G.exports = function(oe, se, ce, le) {
              ce("autotypenumbers", le.autotypenumbersDflt), ce("type", (le.splomStash || {}).type) === "-" && (function(ue, de) {
                if (ue.type === "-") {
                  var fe, he = ue._id, me = he.charAt(0);
                  he.indexOf("scene") !== -1 && (he = me);
                  var pe = function(Ee, xe, Le) {
                    for (var Qe = 0; Qe < Ee.length; Qe++) {
                      var ke = Ee[Qe];
                      if (ke.type === "splom" && ke._length > 0 && (ke["_" + Le + "axes"] || {})[xe] || (ke[Le + "axis"] || Le) === xe && (ae(ke, Le) || (ke[Le] || []).length || ke[Le + "0"]))
                        return ke;
                    }
                  }(de, he, me);
                  if (pe) if (pe.type !== "histogram" || me !== { v: "y", h: "x" }[pe.orientation || "v"]) {
                    var ye = me + "calendar", ge = pe[ye], ve = { noMultiCategory: !te(pe, "cartesian") || te(pe, "noMultiCategory") };
                    if (pe.type === "box" && pe._hasPreCompStats && me === { h: "x", v: "y" }[pe.orientation || "v"] && (ve.noMultiCategory = !0), ve.autotypenumbers = ue.autotypenumbers, ae(pe, me)) {
                      var be = ee(pe), Ce = [];
                      for (fe = 0; fe < de.length; fe++) {
                        var Se = de[fe];
                        te(Se, "box-violin") && (Se[me + "axis"] || me) === he && (Se[be] !== void 0 ? Ce.push(Se[be][0]) : Se.name !== void 0 ? Ce.push(Se.name) : Ce.push("text"), Se[ye] !== ge && (ge = void 0));
                      }
                      ue.type = ne(Ce, ge, ve);
                    } else if (pe.type === "splom") {
                      var we = pe.dimensions[pe._axesDim[he]];
                      we.visible && (ue.type = ne(we.values, ge, ve));
                    } else ue.type = ne(pe[me] || [pe[me + "0"]], ge, ve);
                  } else ue.type = "linear";
                }
              }(se, le.data), se.type === "-" ? se.type = "linear" : oe.type = se.type);
            };
          }, 90251: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809);
            function ee(ce, le, ue) {
              var de, fe, he, me = !1;
              if (le.type === "data") de = ce._fullData[le.traces !== null ? le.traces[0] : 0];
              else {
                if (le.type !== "layout") return !1;
                de = ce._fullLayout;
              }
              return fe = ne.nestedProperty(de, le.prop).get(), (he = ue[le.type] = ue[le.type] || {}).hasOwnProperty(le.prop) && he[le.prop] !== fe && (me = !0), he[le.prop] = fe, { changed: me, value: fe };
            }
            function ae(ce, le) {
              var ue = [], de = le[0], fe = {};
              if (typeof de == "string") fe[de] = le[1];
              else {
                if (!ne.isPlainObject(de)) return ue;
                fe = de;
              }
              return se(fe, function(he, me, pe) {
                ue.push({ type: "layout", prop: he, value: pe });
              }, "", 0), ue;
            }
            function oe(ce, le) {
              var ue, de, fe, he, me = [];
              if (de = le[0], fe = le[1], ue = le[2], he = {}, typeof de == "string") he[de] = fe;
              else {
                if (!ne.isPlainObject(de)) return me;
                he = de, ue === void 0 && (ue = fe);
              }
              return ue === void 0 && (ue = null), se(he, function(pe, ye, ge) {
                var ve, be;
                if (Array.isArray(ge)) {
                  be = ge.slice();
                  var Ce = Math.min(be.length, ce.data.length);
                  ue && (Ce = Math.min(Ce, ue.length)), ve = [];
                  for (var Se = 0; Se < Ce; Se++) ve[Se] = ue ? ue[Se] : Se;
                } else be = ge, ve = ue ? ue.slice() : null;
                if (ve === null) Array.isArray(be) && (be = be[0]);
                else if (Array.isArray(ve)) {
                  if (!Array.isArray(be)) {
                    var we = be;
                    be = [];
                    for (var Ee = 0; Ee < ve.length; Ee++) be[Ee] = we;
                  }
                  be.length = Math.min(ve.length, be.length);
                }
                me.push({ type: "data", prop: pe, traces: ve, value: be });
              }, "", 0), me;
            }
            function se(ce, le, ue, de) {
              Object.keys(ce).forEach(function(fe) {
                var he = ce[fe];
                if (fe[0] !== "_") {
                  var me = ue + (de > 0 ? "." : "") + fe;
                  ne.isPlainObject(he) ? se(he, le, me, de + 1) : le(me, fe, he);
                }
              });
            }
            J.manageCommandObserver = function(ce, le, ue, de) {
              var fe = {}, he = !0;
              le && le._commandObserver && (fe = le._commandObserver), fe.cache || (fe.cache = {}), fe.lookupTable = {};
              var me = J.hasSimpleAPICommandBindings(ce, ue, fe.lookupTable);
              if (le && le._commandObserver) {
                if (me) return fe;
                if (le._commandObserver.remove) return le._commandObserver.remove(), le._commandObserver = null, fe;
              }
              if (me) {
                ee(ce, me, fe.cache), fe.check = function() {
                  if (he) {
                    var ge = ee(ce, me, fe.cache);
                    return ge.changed && de && fe.lookupTable[ge.value] !== void 0 && (fe.disable(), Promise.resolve(de({ value: ge.value, type: me.type, prop: me.prop, traces: me.traces, index: fe.lookupTable[ge.value] })).then(fe.enable, fe.enable)), ge.changed;
                  }
                };
                for (var pe = ["plotly_relayout", "plotly_redraw", "plotly_restyle", "plotly_update", "plotly_animatingframe", "plotly_afterplot"], ye = 0; ye < pe.length; ye++) ce._internalOn(pe[ye], fe.check);
                fe.remove = function() {
                  for (var ge = 0; ge < pe.length; ge++) ce._removeInternalListener(pe[ge], fe.check);
                };
              } else ne.log("Unable to automatically bind plot updates to API command"), fe.lookupTable = {}, fe.remove = function() {
              };
              return fe.disable = function() {
                he = !1;
              }, fe.enable = function() {
                he = !0;
              }, le && (le._commandObserver = fe), fe;
            }, J.hasSimpleAPICommandBindings = function(ce, le, ue) {
              var de, fe, he = le.length;
              for (de = 0; de < he; de++) {
                var me, pe = le[de], ye = pe.method, ge = pe.args;
                if (Array.isArray(ge) || (ge = []), !ye) return !1;
                var ve = J.computeAPICommandBindings(ce, ye, ge);
                if (ve.length !== 1) return !1;
                if (fe) {
                  if ((me = ve[0]).type !== fe.type || me.prop !== fe.prop) return !1;
                  if (Array.isArray(fe.traces)) {
                    if (!Array.isArray(me.traces)) return !1;
                    me.traces.sort();
                    for (var be = 0; be < fe.traces.length; be++) if (fe.traces[be] !== me.traces[be]) return !1;
                  } else if (me.prop !== fe.prop) return !1;
                } else fe = ve[0], Array.isArray(fe.traces) && fe.traces.sort();
                var Ce = (me = ve[0]).value;
                if (Array.isArray(Ce)) {
                  if (Ce.length !== 1) return !1;
                  Ce = Ce[0];
                }
                ue && (ue[Ce] = de);
              }
              return fe;
            }, J.executeAPICommand = function(ce, le, ue) {
              if (le === "skip") return Promise.resolve();
              var de = te.apiMethodRegistry[le], fe = [ce];
              Array.isArray(ue) || (ue = []);
              for (var he = 0; he < ue.length; he++) fe.push(ue[he]);
              return de.apply(null, fe).catch(function(me) {
                return ne.warn("API call to Plotly." + le + " rejected.", me), Promise.reject(me);
              });
            }, J.computeAPICommandBindings = function(ce, le, ue) {
              var de;
              switch (Array.isArray(ue) || (ue = []), le) {
                case "restyle":
                  de = oe(ce, ue);
                  break;
                case "relayout":
                  de = ae(0, ue);
                  break;
                case "update":
                  de = oe(ce, [ue[0], ue[2]]).concat(ae(0, [ue[1]]));
                  break;
                case "animate":
                  de = function(fe, he) {
                    return Array.isArray(he[0]) && he[0].length === 1 && ["string", "number"].indexOf(typeof he[0][0]) !== -1 ? [{ type: "layout", prop: "_currentFrame", value: he[0][0].toString() }] : [];
                  }(0, ue);
                  break;
                default:
                  de = [];
              }
              return de;
            };
          }, 13792: function(G, J, Z) {
            var te = Z(93049).extendFlat;
            J.u = function(ne, ee) {
              ee = ee || {};
              var ae = { valType: "info_array", editType: (ne = ne || {}).editType, items: [{ valType: "number", min: 0, max: 1, editType: ne.editType }, { valType: "number", min: 0, max: 1, editType: ne.editType }], dflt: [0, 1] }, oe = (ne.name && ne.name, ne.trace, ee.description && ee.description, { x: te({}, ae, {}), y: te({}, ae, {}), editType: ne.editType });
              return ne.noGridCell || (oe.row = { valType: "integer", min: 0, dflt: 0, editType: ne.editType }, oe.column = { valType: "integer", min: 0, dflt: 0, editType: ne.editType }), oe;
            }, J.N = function(ne, ee, ae, oe) {
              var se = oe && oe.x || [0, 1], ce = oe && oe.y || [0, 1], le = ee.grid;
              if (le) {
                var ue = ae("domain.column");
                ue !== void 0 && (ue < le.columns ? se = le._domains.x[ue] : delete ne.domain.column);
                var de = ae("domain.row");
                de !== void 0 && (de < le.rows ? ce = le._domains.y[de] : delete ne.domain.row);
              }
              var fe = ae("domain.x", se), he = ae("domain.y", ce);
              fe[0] < fe[1] || (ne.domain.x = se.slice()), he[0] < he[1] || (ne.domain.y = ce.slice());
            };
          }, 80337: function(G) {
            G.exports = function(J) {
              var Z = J.variantValues, te = J.editType, ne = J.colorEditType;
              ne === void 0 && (ne = te);
              var ee = { editType: te, valType: "integer", min: 1, max: 1e3, extras: ["normal", "bold"], dflt: "normal" };
              J.noNumericWeightValues && (ee.valType = "enumerated", ee.values = ee.extras, ee.extras = void 0, ee.min = void 0, ee.max = void 0);
              var ae = { family: { valType: "string", noBlank: !0, strict: !0, editType: te }, size: { valType: "number", min: 1, editType: te }, color: { valType: "color", editType: ne }, weight: ee, style: { editType: te, valType: "enumerated", values: ["normal", "italic"], dflt: "normal" }, variant: J.noFontVariant ? void 0 : { editType: te, valType: "enumerated", values: Z || ["normal", "small-caps", "all-small-caps", "all-petite-caps", "petite-caps", "unicase"], dflt: "normal" }, textcase: J.noFontTextcase ? void 0 : { editType: te, valType: "enumerated", values: ["normal", "word caps", "upper", "lower"], dflt: "normal" }, lineposition: J.noFontLineposition ? void 0 : { editType: te, valType: "flaglist", flags: ["under", "over", "through"], extras: ["none"], dflt: "none" }, shadow: J.noFontShadow ? void 0 : { editType: te, valType: "string", dflt: J.autoShadowDflt ? "auto" : "none" }, editType: te };
              return J.autoSize && (ae.size.dflt = "auto"), J.autoColor && (ae.color.dflt = "auto"), J.arrayOk && (ae.family.arrayOk = !0, ae.weight.arrayOk = !0, ae.style.arrayOk = !0, J.noFontVariant || (ae.variant.arrayOk = !0), J.noFontTextcase || (ae.textcase.arrayOk = !0), J.noFontLineposition || (ae.lineposition.arrayOk = !0), J.noFontShadow || (ae.shadow.arrayOk = !0), ae.size.arrayOk = !0, ae.color.arrayOk = !0), ae;
            };
          }, 58935: function(G) {
            G.exports = { _isLinkedToArray: "frames_entry", group: { valType: "string" }, name: { valType: "string" }, traces: { valType: "any" }, baseframe: { valType: "string" }, data: { valType: "any" }, layout: { valType: "any" } };
          }, 74285: function(G, J) {
            J.projNames = { airy: "airy", aitoff: "aitoff", "albers usa": "albersUsa", albers: "albers", august: "august", "azimuthal equal area": "azimuthalEqualArea", "azimuthal equidistant": "azimuthalEquidistant", baker: "baker", bertin1953: "bertin1953", boggs: "boggs", bonne: "bonne", bottomley: "bottomley", bromley: "bromley", collignon: "collignon", "conic conformal": "conicConformal", "conic equal area": "conicEqualArea", "conic equidistant": "conicEquidistant", craig: "craig", craster: "craster", "cylindrical equal area": "cylindricalEqualArea", "cylindrical stereographic": "cylindricalStereographic", eckert1: "eckert1", eckert2: "eckert2", eckert3: "eckert3", eckert4: "eckert4", eckert5: "eckert5", eckert6: "eckert6", eisenlohr: "eisenlohr", "equal earth": "equalEarth", equirectangular: "equirectangular", fahey: "fahey", "foucaut sinusoidal": "foucautSinusoidal", foucaut: "foucaut", ginzburg4: "ginzburg4", ginzburg5: "ginzburg5", ginzburg6: "ginzburg6", ginzburg8: "ginzburg8", ginzburg9: "ginzburg9", gnomonic: "gnomonic", "gringorten quincuncial": "gringortenQuincuncial", gringorten: "gringorten", guyou: "guyou", hammer: "hammer", hill: "hill", homolosine: "homolosine", hufnagel: "hufnagel", hyperelliptical: "hyperelliptical", kavrayskiy7: "kavrayskiy7", lagrange: "lagrange", larrivee: "larrivee", laskowski: "laskowski", loximuthal: "loximuthal", mercator: "mercator", miller: "miller", mollweide: "mollweide", "mt flat polar parabolic": "mtFlatPolarParabolic", "mt flat polar quartic": "mtFlatPolarQuartic", "mt flat polar sinusoidal": "mtFlatPolarSinusoidal", "natural earth": "naturalEarth", "natural earth1": "naturalEarth1", "natural earth2": "naturalEarth2", "nell hammer": "nellHammer", nicolosi: "nicolosi", orthographic: "orthographic", patterson: "patterson", "peirce quincuncial": "peirceQuincuncial", polyconic: "polyconic", "rectangular polyconic": "rectangularPolyconic", robinson: "robinson", satellite: "satellite", "sinu mollweide": "sinuMollweide", sinusoidal: "sinusoidal", stereographic: "stereographic", times: "times", "transverse mercator": "transverseMercator", "van der grinten": "vanDerGrinten", "van der grinten2": "vanDerGrinten2", "van der grinten3": "vanDerGrinten3", "van der grinten4": "vanDerGrinten4", wagner4: "wagner4", wagner6: "wagner6", wiechel: "wiechel", "winkel tripel": "winkel3", winkel3: "winkel3" }, J.axesNames = ["lonaxis", "lataxis"], J.lonaxisSpan = { orthographic: 180, "azimuthal equal area": 360, "azimuthal equidistant": 360, "conic conformal": 180, gnomonic: 160, stereographic: 180, "transverse mercator": 180, "*": 360 }, J.lataxisSpan = { "conic conformal": 150, stereographic: 179.5, "*": 180 }, J.scopeDefaults = { world: { lonaxisRange: [-180, 180], lataxisRange: [-90, 90], projType: "equirectangular", projRotate: [0, 0, 0] }, usa: { lonaxisRange: [-180, -50], lataxisRange: [15, 80], projType: "albers usa" }, europe: { lonaxisRange: [-30, 60], lataxisRange: [30, 85], projType: "conic conformal", projRotate: [15, 0, 0], projParallels: [0, 60] }, asia: { lonaxisRange: [22, 160], lataxisRange: [-15, 55], projType: "mercator", projRotate: [0, 0, 0] }, africa: { lonaxisRange: [-30, 60], lataxisRange: [-40, 40], projType: "mercator", projRotate: [0, 0, 0] }, "north america": { lonaxisRange: [-180, -45], lataxisRange: [5, 85], projType: "conic conformal", projRotate: [-100, 0, 0], projParallels: [29.5, 45.5] }, "south america": { lonaxisRange: [-100, -30], lataxisRange: [-60, 15], projType: "mercator", projRotate: [0, 0, 0] } }, J.clipPad = 1e-3, J.precision = 0.1, J.landColor = "#F0DC82", J.waterColor = "#3399FF", J.locationmodeToLayer = { "ISO-3": "countries", "USA-states": "subunits", "country names": "countries" }, J.sphereSVG = { type: "Sphere" }, J.fillLayers = { ocean: 1, land: 1, lakes: 1 }, J.lineLayers = { subunits: 1, countries: 1, coastlines: 1, rivers: 1, frame: 1 }, J.layers = ["bg", "ocean", "land", "lakes", "subunits", "countries", "coastlines", "rivers", "lataxis", "lonaxis", "frame", "backplot", "frontplot"], J.layersForChoropleth = ["bg", "ocean", "land", "subunits", "countries", "coastlines", "lataxis", "lonaxis", "frame", "backplot", "rivers", "lakes", "frontplot"], J.layerNameToAdjective = { ocean: "ocean", land: "land", lakes: "lake", subunits: "subunit", countries: "country", coastlines: "coastline", rivers: "river", frame: "frame" };
          }, 6493: function(G, J, Z) {
            var te = Z(45568), ne = Z(70884), ee = ne.geoPath, ae = ne.geoDistance, oe = Z(75987), se = Z(33626), ce = Z(34809), le = ce.strTranslate, ue = Z(78766), de = Z(62203), fe = Z(32141), he = Z(44122), me = Z(29714), pe = Z(32919).getAutoRange, ye = Z(14751), ge = Z(44844).prepSelect, ve = Z(44844).clearOutline, be = Z(44844).selectOnClick, Ce = Z(14309), Se = Z(74285), we = Z(3994), Ee = Z(11577), xe = Z(48640).N4;
            function Le(Me) {
              this.id = Me.id, this.graphDiv = Me.graphDiv, this.container = Me.container, this.topojsonURL = Me.topojsonURL, this.isStatic = Me.staticPlot, this.topojsonName = null, this.topojson = null, this.projection = null, this.scope = null, this.viewInitial = null, this.fitScale = null, this.bounds = null, this.midPt = null, this.hasChoropleth = !1, this.traceHash = {}, this.layers = {}, this.basePaths = {}, this.dataPaths = {}, this.dataPoints = {}, this.clipDef = null, this.clipRect = null, this.bgRect = null, this.makeFramework();
            }
            var Qe = Le.prototype;
            function ke(Me, Re) {
              var De = Se.clipPad, Pe = Me[0] + De, $e = Me[1] - De, qe = Re[0] + De, Xe = Re[1] - De;
              Pe > 0 && $e < 0 && ($e += 360);
              var Ve = ($e - Pe) / 4;
              return { type: "Polygon", coordinates: [[[Pe, qe], [Pe, Xe], [Pe + Ve, Xe], [Pe + 2 * Ve, Xe], [Pe + 3 * Ve, Xe], [$e, Xe], [$e, qe], [$e - Ve, qe], [$e - 2 * Ve, qe], [$e - 3 * Ve, qe], [Pe, qe]]] };
            }
            G.exports = function(Me) {
              return new Le(Me);
            }, Qe.plot = function(Me, Re, De, Pe) {
              var $e = this;
              if (Pe) return $e.update(Me, Re, !0);
              $e._geoCalcData = Me, $e._fullLayout = Re;
              var qe = Re[this.id], Xe = [], Ve = !1;
              for (var Ze in Se.layerNameToAdjective) if (Ze !== "frame" && qe["show" + Ze]) {
                Ve = !0;
                break;
              }
              for (var Ge = !1, ze = 0; ze < Me.length; ze++) {
                var We = Me[0][0].trace;
                We._geo = $e, We.locationmode && (Ve = !0);
                var at = We.marker;
                if (at) {
                  var ct = at.angle, et = at.angleref;
                  (ct || et === "north" || et === "previous") && (Ge = !0);
                }
              }
              if (this._hasMarkerAngles = Ge, Ve) {
                var dt = Ee.getTopojsonName(qe);
                $e.topojson !== null && dt === $e.topojsonName || ($e.topojsonName = dt, PlotlyGeoAssets.topojson[$e.topojsonName] === void 0 && Xe.push($e.fetchTopojson()));
              }
              Xe = Xe.concat(we.fetchTraceGeoData(Me)), De.push(new Promise(function(Je, it) {
                Promise.all(Xe).then(function() {
                  $e.topojson = PlotlyGeoAssets.topojson[$e.topojsonName], $e.update(Me, Re), Je();
                }).catch(it);
              }));
            }, Qe.fetchTopojson = function() {
              var Me = this, Re = Ee.getTopojsonPath(Me.topojsonURL, Me.topojsonName);
              return new Promise(function(De, Pe) {
                te.json(Re, function($e, qe) {
                  if ($e) return $e.status === 404 ? Pe(new Error(["plotly.js could not find topojson file at", Re + ".", "Make sure the *topojsonURL* plot config option", "is set properly."].join(" "))) : Pe(new Error(["unexpected error while fetching topojson file at", Re].join(" ")));
                  PlotlyGeoAssets.topojson[Me.topojsonName] = qe, De();
                });
              });
            }, Qe.update = function(Me, Re, De) {
              var Pe = Re[this.id];
              this.hasChoropleth = !1;
              for (var $e = 0; $e < Me.length; $e++) {
                var qe = Me[$e], Xe = qe[0].trace;
                Xe.type === "choropleth" && (this.hasChoropleth = !0), Xe.visible === !0 && Xe._length > 0 && Xe._module.calcGeoJSON(qe, Re);
              }
              if (!De) {
                if (this.updateProjection(Me, Re)) return;
                this.viewInitial && this.scope === Pe.scope || this.saveViewInitial(Pe);
              }
              this.scope = Pe.scope, this.updateBaseLayers(Re, Pe), this.updateDims(Re, Pe), this.updateFx(Re, Pe), he.generalUpdatePerTraceModule(this.graphDiv, this, Me, Pe);
              var Ve = this.layers.frontplot.select(".scatterlayer");
              this.dataPoints.point = Ve.selectAll(".point"), this.dataPoints.text = Ve.selectAll("text"), this.dataPaths.line = Ve.selectAll(".js-line");
              var Ze = this.layers.backplot.select(".choroplethlayer");
              this.dataPaths.choropleth = Ze.selectAll("path"), this._render();
            }, Qe.updateProjection = function(Me, Re) {
              var De = this.graphDiv, Pe = Re[this.id], $e = Re._size, qe = Pe.domain, Xe = Pe.projection, Ve = Pe.lonaxis, Ze = Pe.lataxis, Ge = Ve._ax, ze = Ze._ax, We = this.projection = function(Ft) {
                var Tt = Ft.projection, It = Tt.type, Ct = Se.projNames[It];
                Ct = "geo" + ce.titleCase(Ct);
                for (var Kt = (ne[Ct] || oe[Ct])(), Vt = Ft._isSatellite ? 180 * Math.acos(1 / Tt.distance) / Math.PI : Ft._isClipped ? Se.lonaxisSpan[It] / 2 : null, Yt = ["center", "rotate", "parallels", "clipExtent"], sn = function(Ut) {
                  return Ut ? Kt : [];
                }, hn = 0; hn < Yt.length; hn++) {
                  var Qt = Yt[hn];
                  typeof Kt[Qt] != "function" && (Kt[Qt] = sn);
                }
                return Kt.isLonLatOverEdges = function(Ut) {
                  if (Kt(Ut) === null) return !0;
                  if (Vt) {
                    var Wt = Kt.rotate();
                    return ae(Ut, [-Wt[0], -Wt[1]]) > Vt * Math.PI / 180;
                  }
                  return !1;
                }, Kt.getPath = function() {
                  return ee().projection(Kt);
                }, Kt.getBounds = function(Ut) {
                  return Kt.getPath().bounds(Ut);
                }, Kt.precision(Se.precision), Ft._isSatellite && Kt.tilt(Tt.tilt).distance(Tt.distance), Vt && Kt.clipAngle(Vt - Se.clipPad), Kt;
              }(Pe), at = [[$e.l + $e.w * qe.x[0], $e.t + $e.h * (1 - qe.y[1])], [$e.l + $e.w * qe.x[1], $e.t + $e.h * (1 - qe.y[0])]], ct = Pe.center || {}, et = Xe.rotation || {}, dt = Ve.range || [], Je = Ze.range || [];
              if (Pe.fitbounds) {
                Ge._length = at[1][0] - at[0][0], ze._length = at[1][1] - at[0][1], Ge.range = pe(De, Ge), ze.range = pe(De, ze);
                var it = (Ge.range[0] + Ge.range[1]) / 2, st = (ze.range[0] + ze.range[1]) / 2;
                if (Pe._isScoped) ct = { lon: it, lat: st };
                else if (Pe._isClipped) {
                  ct = { lon: it, lat: st }, et = { lon: it, lat: st, roll: et.roll };
                  var gt = Xe.type, bt = Se.lonaxisSpan[gt] / 2 || 180, yt = Se.lataxisSpan[gt] / 2 || 90;
                  dt = [it - bt, it + bt], Je = [st - yt, st + yt];
                } else ct = { lon: it, lat: st }, et = { lon: it, lat: et.lat, roll: et.roll };
              }
              We.center([ct.lon - et.lon, ct.lat - et.lat]).rotate([-et.lon, -et.lat, et.roll]).parallels(Xe.parallels);
              var St = ke(dt, Je);
              We.fitExtent(at, St);
              var Lt = this.bounds = We.getBounds(St), Rt = this.fitScale = We.scale(), vt = We.translate();
              if (Pe.fitbounds) {
                var $t = We.getBounds(ke(Ge.range, ze.range)), Pt = Math.min((Lt[1][0] - Lt[0][0]) / ($t[1][0] - $t[0][0]), (Lt[1][1] - Lt[0][1]) / ($t[1][1] - $t[0][1]));
                isFinite(Pt) ? We.scale(Pt * Rt) : ce.warn("Something went wrong during" + this.id + "fitbounds computations.");
              } else We.scale(Xe.scale * Rt);
              var Xt = this.midPt = [(Lt[0][0] + Lt[1][0]) / 2, (Lt[0][1] + Lt[1][1]) / 2];
              if (We.translate([vt[0] + (Xt[0] - vt[0]), vt[1] + (Xt[1] - vt[1])]).clipExtent(Lt), Pe._isAlbersUsa) {
                var Nt = We([ct.lon, ct.lat]), Gt = We.translate();
                We.translate([Gt[0] - (Nt[0] - Gt[0]), Gt[1] - (Nt[1] - Gt[1])]);
              }
            }, Qe.updateBaseLayers = function(Me, Re) {
              var De = this, Pe = De.topojson, $e = De.layers, qe = De.basePaths;
              function Xe(We) {
                return We === "lonaxis" || We === "lataxis";
              }
              function Ve(We) {
                return !!Se.lineLayers[We];
              }
              function Ze(We) {
                return !!Se.fillLayers[We];
              }
              var Ge = (this.hasChoropleth ? Se.layersForChoropleth : Se.layers).filter(function(We) {
                return Ve(We) || Ze(We) ? Re["show" + We] : !Xe(We) || Re[We].showgrid;
              }), ze = De.framework.selectAll(".layer").data(Ge, String);
              ze.exit().each(function(We) {
                delete $e[We], delete qe[We], te.select(this).remove();
              }), ze.enter().append("g").attr("class", function(We) {
                return "layer " + We;
              }).each(function(We) {
                var at = $e[We] = te.select(this);
                We === "bg" ? De.bgRect = at.append("rect").style("pointer-events", "all") : Xe(We) ? qe[We] = at.append("path").style("fill", "none") : We === "backplot" ? at.append("g").classed("choroplethlayer", !0) : We === "frontplot" ? at.append("g").classed("scatterlayer", !0) : Ve(We) ? qe[We] = at.append("path").style("fill", "none").style("stroke-miterlimit", 2) : Ze(We) && (qe[We] = at.append("path").style("stroke", "none"));
              }), ze.order(), ze.each(function(We) {
                var at = qe[We], ct = Se.layerNameToAdjective[We];
                We === "frame" ? at.datum(Se.sphereSVG) : Ve(We) || Ze(We) ? at.datum(xe(Pe, Pe.objects[We])) : Xe(We) && at.datum(function(et, dt, Je) {
                  var it, st, gt, bt = dt[et], yt = Se.scopeDefaults[dt.scope];
                  et === "lonaxis" ? (it = yt.lonaxisRange, st = yt.lataxisRange, gt = function(Gt, Ft) {
                    return [Gt, Ft];
                  }) : et === "lataxis" && (it = yt.lataxisRange, st = yt.lonaxisRange, gt = function(Gt, Ft) {
                    return [Ft, Gt];
                  });
                  var St = { type: "linear", range: [it[0], it[1] - 1e-6], tick0: bt.tick0, dtick: bt.dtick };
                  me.setConvert(St, Je);
                  var Lt = me.calcTicks(St);
                  dt.isScoped || et !== "lonaxis" || Lt.pop();
                  for (var Rt = Lt.length, vt = new Array(Rt), $t = 0; $t < Rt; $t++) for (var Pt = Lt[$t].x, Xt = vt[$t] = [], Nt = st[0]; Nt < st[1] + 2.5; Nt += 2.5) Xt.push(gt(Pt, Nt));
                  return { type: "MultiLineString", coordinates: vt };
                }(We, Re, Me)).call(ue.stroke, Re[We].gridcolor).call(de.dashLine, Re[We].griddash, Re[We].gridwidth), Ve(We) ? at.call(ue.stroke, Re[ct + "color"]).call(de.dashLine, "", Re[ct + "width"]) : Ze(We) && at.call(ue.fill, Re[ct + "color"]);
              });
            }, Qe.updateDims = function(Me, Re) {
              var De = this.bounds, Pe = (Re.framewidth || 0) / 2, $e = De[0][0] - Pe, qe = De[0][1] - Pe, Xe = De[1][0] - $e + Pe, Ve = De[1][1] - qe + Pe;
              de.setRect(this.clipRect, $e, qe, Xe, Ve), this.bgRect.call(de.setRect, $e, qe, Xe, Ve).call(ue.fill, Re.bgcolor), this.xaxis._offset = $e, this.xaxis._length = Xe, this.yaxis._offset = qe, this.yaxis._length = Ve;
            }, Qe.updateFx = function(Me, Re) {
              var De = this, Pe = De.graphDiv, $e = De.bgRect, qe = Me.dragmode, Xe = Me.clickmode;
              if (!De.isStatic) {
                var Ve = { element: De.bgRect.node(), gd: Pe, plotinfo: { id: De.id, xaxis: De.xaxis, yaxis: De.yaxis, fillRangeItems: function(Ge, ze) {
                  ze.isRect ? (Ge.range = {})[De.id] = [Ze([ze.xmin, ze.ymin]), Ze([ze.xmax, ze.ymax])] : (Ge.lassoPoints = {})[De.id] = ze.map(Ze);
                } }, xaxes: [De.xaxis], yaxes: [De.yaxis], subplot: De.id, clickFn: function(Ge) {
                  Ge === 2 && ve(Pe);
                } };
                qe === "pan" ? ($e.node().onmousedown = null, $e.call(Ce(De, Re)), $e.on("dblclick.zoom", function() {
                  var Ge = De.viewInitial, ze = {};
                  for (var We in Ge) ze[De.id + "." + We] = Ge[We];
                  se.call("_guiRelayout", Pe, ze), Pe.emit("plotly_doubleclick", null);
                }), Pe._context._scrollZoom.geo || $e.on("wheel.zoom", null)) : qe !== "select" && qe !== "lasso" || ($e.on(".zoom", null), Ve.prepFn = function(Ge, ze, We) {
                  ge(Ge, ze, We, Ve, qe);
                }, ye.init(Ve)), $e.on("mousemove", function() {
                  var Ge = De.projection.invert(ce.getPositionFromD3Event());
                  if (!Ge) return ye.unhover(Pe, te.event);
                  De.xaxis.p2c = function() {
                    return Ge[0];
                  }, De.yaxis.p2c = function() {
                    return Ge[1];
                  }, fe.hover(Pe, te.event, De.id);
                }), $e.on("mouseout", function() {
                  Pe._dragging || ye.unhover(Pe, te.event);
                }), $e.on("click", function() {
                  qe !== "select" && qe !== "lasso" && (Xe.indexOf("select") > -1 && be(te.event, Pe, [De.xaxis], [De.yaxis], De.id, Ve), Xe.indexOf("event") > -1 && fe.click(Pe, te.event));
                });
              }
              function Ze(Ge) {
                return De.projection.invert([Ge[0] + De.xaxis._offset, Ge[1] + De.yaxis._offset]);
              }
            }, Qe.makeFramework = function() {
              var Me = this, Re = Me.graphDiv, De = Re._fullLayout, Pe = "clip" + De._uid + Me.id;
              Me.clipDef = De._clips.append("clipPath").attr("id", Pe), Me.clipRect = Me.clipDef.append("rect"), Me.framework = te.select(Me.container).append("g").attr("class", "geo " + Me.id).call(de.setClipUrl, Pe, Re), Me.project = function($e) {
                var qe = Me.projection($e);
                return qe ? [qe[0] - Me.xaxis._offset, qe[1] - Me.yaxis._offset] : [null, null];
              }, Me.xaxis = { _id: "x", c2p: function($e) {
                return Me.project($e)[0];
              } }, Me.yaxis = { _id: "y", c2p: function($e) {
                return Me.project($e)[1];
              } }, Me.mockAxis = { type: "linear", showexponent: "all", exponentformat: "B" }, me.setConvert(Me.mockAxis, De);
            }, Qe.saveViewInitial = function(Me) {
              var Re, De = Me.center || {}, Pe = Me.projection, $e = Pe.rotation || {};
              this.viewInitial = { fitbounds: Me.fitbounds, "projection.scale": Pe.scale }, Re = Me._isScoped ? { "center.lon": De.lon, "center.lat": De.lat } : Me._isClipped ? { "projection.rotation.lon": $e.lon, "projection.rotation.lat": $e.lat } : { "center.lon": De.lon, "center.lat": De.lat, "projection.rotation.lon": $e.lon }, ce.extendFlat(this.viewInitial, Re);
            }, Qe.render = function(Me) {
              this._hasMarkerAngles && Me ? this.plot(this._geoCalcData, this._fullLayout, [], !0) : this._render();
            }, Qe._render = function() {
              var Me, Re = this.projection, De = Re.getPath();
              function Pe(qe) {
                var Xe = Re(qe.lonlat);
                return Xe ? le(Xe[0], Xe[1]) : null;
              }
              function $e(qe) {
                return Re.isLonLatOverEdges(qe.lonlat) ? "none" : null;
              }
              for (Me in this.basePaths) this.basePaths[Me].attr("d", De);
              for (Me in this.dataPaths) this.dataPaths[Me].attr("d", function(qe) {
                return De(qe.geojson);
              });
              for (Me in this.dataPoints) this.dataPoints[Me].attr("display", $e).attr("transform", Pe);
            };
          }, 47544: function(G, J, Z) {
            var te = Z(4173).fX, ne = Z(34809).counterRegex, ee = Z(6493), ae = "geo", oe = ne(ae), se = {};
            se[ae] = { valType: "subplotid", dflt: ae, editType: "calc" }, G.exports = { attr: ae, name: ae, idRoot: ae, idRegex: oe, attrRegex: oe, attributes: se, layoutAttributes: Z(42194), supplyLayoutDefaults: Z(31653), plot: function(ce) {
              for (var le = ce._fullLayout, ue = ce.calcdata, de = le._subplots[ae], fe = 0; fe < de.length; fe++) {
                var he = de[fe], me = te(ue, ae, he), pe = le[he]._subplot;
                pe || (pe = ee({ id: he, graphDiv: ce, container: le._geolayer.node(), topojsonURL: ce._context.topojsonURL, staticPlot: ce._context.staticPlot }), le[he]._subplot = pe), pe.plot(me, le, ce._promises);
              }
            }, updateFx: function(ce) {
              for (var le = ce._fullLayout, ue = le._subplots[ae], de = 0; de < ue.length; de++) {
                var fe = le[ue[de]];
                fe._subplot.updateFx(le, fe);
              }
            }, clean: function(ce, le, ue, de) {
              for (var fe = de._subplots[ae] || [], he = 0; he < fe.length; he++) {
                var me = fe[he], pe = de[me]._subplot;
                !le[me] && pe && (pe.framework.remove(), pe.clipDef.remove());
              }
            } };
          }, 42194: function(G, J, Z) {
            var te = Z(10229), ne = Z(13792).u, ee = Z(94850).T, ae = Z(74285), oe = Z(13582).overrideAll, se = Z(62994), ce = { range: { valType: "info_array", items: [{ valType: "number" }, { valType: "number" }] }, showgrid: { valType: "boolean", dflt: !1 }, tick0: { valType: "number", dflt: 0 }, dtick: { valType: "number" }, gridcolor: { valType: "color", dflt: te.lightLine }, gridwidth: { valType: "number", min: 0, dflt: 1 }, griddash: ee };
            (G.exports = oe({ domain: ne({ name: "geo" }, {}), fitbounds: { valType: "enumerated", values: [!1, "locations", "geojson"], dflt: !1, editType: "plot" }, resolution: { valType: "enumerated", values: [110, 50], dflt: 110, coerceNumber: !0 }, scope: { valType: "enumerated", values: se(ae.scopeDefaults), dflt: "world" }, projection: { type: { valType: "enumerated", values: se(ae.projNames) }, rotation: { lon: { valType: "number" }, lat: { valType: "number" }, roll: { valType: "number" } }, tilt: { valType: "number", dflt: 0 }, distance: { valType: "number", min: 1.001, dflt: 2 }, parallels: { valType: "info_array", items: [{ valType: "number" }, { valType: "number" }] }, scale: { valType: "number", min: 0, dflt: 1 } }, center: { lon: { valType: "number" }, lat: { valType: "number" } }, visible: { valType: "boolean", dflt: !0 }, showcoastlines: { valType: "boolean" }, coastlinecolor: { valType: "color", dflt: te.defaultLine }, coastlinewidth: { valType: "number", min: 0, dflt: 1 }, showland: { valType: "boolean", dflt: !1 }, landcolor: { valType: "color", dflt: ae.landColor }, showocean: { valType: "boolean", dflt: !1 }, oceancolor: { valType: "color", dflt: ae.waterColor }, showlakes: { valType: "boolean", dflt: !1 }, lakecolor: { valType: "color", dflt: ae.waterColor }, showrivers: { valType: "boolean", dflt: !1 }, rivercolor: { valType: "color", dflt: ae.waterColor }, riverwidth: { valType: "number", min: 0, dflt: 1 }, showcountries: { valType: "boolean" }, countrycolor: { valType: "color", dflt: te.defaultLine }, countrywidth: { valType: "number", min: 0, dflt: 1 }, showsubunits: { valType: "boolean" }, subunitcolor: { valType: "color", dflt: te.defaultLine }, subunitwidth: { valType: "number", min: 0, dflt: 1 }, showframe: { valType: "boolean" }, framecolor: { valType: "color", dflt: te.defaultLine }, framewidth: { valType: "number", min: 0, dflt: 1 }, bgcolor: { valType: "color", dflt: te.background }, lonaxis: ce, lataxis: ce }, "plot", "from-root")).uirevision = { valType: "any", editType: "none" };
          }, 31653: function(G, J, Z) {
            var te = Z(34809), ne = Z(4448), ee = Z(4173).KO, ae = Z(74285), oe = Z(42194), se = ae.axesNames;
            function ce(le, ue, de, fe) {
              var he = ee(fe.fullData, "geo", fe.id).map(function(et) {
                return et._expandedIndex;
              }), me = de("resolution"), pe = de("scope"), ye = ae.scopeDefaults[pe], ge = de("projection.type", ye.projType), ve = ue._isAlbersUsa = ge === "albers usa";
              ve && (pe = ue.scope = "usa");
              var be = ue._isScoped = pe !== "world", Ce = ue._isSatellite = ge === "satellite", Se = ue._isConic = ge.indexOf("conic") !== -1 || ge === "albers", we = ue._isClipped = !!ae.lonaxisSpan[ge];
              if (le.visible === !1) {
                var Ee = te.extendDeep({}, ue._template);
                Ee.showcoastlines = !1, Ee.showcountries = !1, Ee.showframe = !1, Ee.showlakes = !1, Ee.showland = !1, Ee.showocean = !1, Ee.showrivers = !1, Ee.showsubunits = !1, Ee.lonaxis && (Ee.lonaxis.showgrid = !1), Ee.lataxis && (Ee.lataxis.showgrid = !1), ue._template = Ee;
              }
              for (var xe = de("visible"), Le = 0; Le < se.length; Le++) {
                var Qe, ke = se[Le], Me = [30, 10][Le];
                if (be) Qe = ye[ke + "Range"];
                else {
                  var Re = ae[ke + "Span"], De = (Re[ge] || Re["*"]) / 2, Pe = de("projection.rotation." + ke.substr(0, 3), ye.projRotate[Le]);
                  Qe = [Pe - De, Pe + De];
                }
                var $e = de(ke + ".range", Qe);
                de(ke + ".tick0"), de(ke + ".dtick", Me), de(ke + ".showgrid", !!xe && void 0) && (de(ke + ".gridcolor"), de(ke + ".gridwidth"), de(ke + ".griddash")), ue[ke]._ax = { type: "linear", _id: ke.slice(0, 3), _traceIndices: he, setScale: te.identity, c2l: te.identity, r2l: te.identity, autorange: !0, range: $e.slice(), _m: 1, _input: {} };
              }
              var qe = ue.lonaxis.range, Xe = ue.lataxis.range, Ve = qe[0], Ze = qe[1];
              Ve > 0 && Ze < 0 && (Ze += 360);
              var Ge, ze, We, at = (Ve + Ze) / 2;
              if (!ve) {
                var ct = be ? ye.projRotate : [at, 0, 0];
                Ge = de("projection.rotation.lon", ct[0]), de("projection.rotation.lat", ct[1]), de("projection.rotation.roll", ct[2]), de("showcoastlines", !be && xe) && (de("coastlinecolor"), de("coastlinewidth")), de("showocean", !!xe && void 0) && de("oceancolor");
              }
              ve ? (ze = -96.6, We = 38.7) : (ze = be ? at : Ge, We = (Xe[0] + Xe[1]) / 2), de("center.lon", ze), de("center.lat", We), Ce && (de("projection.tilt"), de("projection.distance")), Se && de("projection.parallels", ye.projParallels || [0, 60]), de("projection.scale"), de("showland", !!xe && void 0) && de("landcolor"), de("showlakes", !!xe && void 0) && de("lakecolor"), de("showrivers", !!xe && void 0) && (de("rivercolor"), de("riverwidth")), de("showcountries", be && pe !== "usa" && xe) && (de("countrycolor"), de("countrywidth")), (pe === "usa" || pe === "north america" && me === 50) && (de("showsubunits", xe), de("subunitcolor"), de("subunitwidth")), be || de("showframe", xe) && (de("framecolor"), de("framewidth")), de("bgcolor"), de("fitbounds") && (delete ue.projection.scale, be ? (delete ue.center.lon, delete ue.center.lat) : we ? (delete ue.center.lon, delete ue.center.lat, delete ue.projection.rotation.lon, delete ue.projection.rotation.lat, delete ue.lonaxis.range, delete ue.lataxis.range) : (delete ue.center.lon, delete ue.center.lat, delete ue.projection.rotation.lon));
            }
            G.exports = function(le, ue, de) {
              ne(le, ue, de, { type: "geo", attributes: oe, handleDefaults: ce, fullData: de, partition: "y" });
            };
          }, 14309: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(33626), ae = Math.PI / 180, oe = 180 / Math.PI, se = { cursor: "pointer" }, ce = { cursor: "auto" };
            function le(be, Ce) {
              return te.behavior.zoom().translate(Ce.translate()).scale(Ce.scale());
            }
            function ue(be, Ce, Se) {
              var we = be.id, Ee = be.graphDiv, xe = Ee.layout, Le = xe[we], Qe = Ee._fullLayout, ke = Qe[we], Me = {}, Re = {};
              function De(Pe, $e) {
                Me[we + "." + Pe] = ne.nestedProperty(Le, Pe).get(), ee.call("_storeDirectGUIEdit", xe, Qe._preGUI, Me);
                var qe = ne.nestedProperty(ke, Pe);
                qe.get() !== $e && (qe.set($e), ne.nestedProperty(Le, Pe).set($e), Re[we + "." + Pe] = $e);
              }
              Se(De), De("projection.scale", Ce.scale() / be.fitScale), De("fitbounds", !1), Ee.emit("plotly_relayout", Re);
            }
            function de(be, Ce) {
              var Se = le(0, Ce);
              function we(Ee) {
                var xe = Ce.invert(be.midPt);
                Ee("center.lon", xe[0]), Ee("center.lat", xe[1]);
              }
              return Se.on("zoomstart", function() {
                te.select(this).style(se);
              }).on("zoom", function() {
                Ce.scale(te.event.scale).translate(te.event.translate), be.render(!0);
                var Ee = Ce.invert(be.midPt);
                be.graphDiv.emit("plotly_relayouting", { "geo.projection.scale": Ce.scale() / be.fitScale, "geo.center.lon": Ee[0], "geo.center.lat": Ee[1] });
              }).on("zoomend", function() {
                te.select(this).style(ce), ue(be, Ce, we);
              }), Se;
            }
            function fe(be, Ce) {
              var Se, we, Ee, xe, Le, Qe, ke, Me, Re, De = le(0, Ce);
              function Pe(qe) {
                return Ce.invert(qe);
              }
              function $e(qe) {
                var Xe = Ce.rotate(), Ve = Ce.invert(be.midPt);
                qe("projection.rotation.lon", -Xe[0]), qe("center.lon", Ve[0]), qe("center.lat", Ve[1]);
              }
              return De.on("zoomstart", function() {
                te.select(this).style(se), Se = te.mouse(this), we = Ce.rotate(), Ee = Ce.translate(), xe = we, Le = Pe(Se);
              }).on("zoom", function() {
                if (Qe = te.mouse(this), function(Ve) {
                  var Ze = Pe(Ve);
                  if (!Ze) return !0;
                  var Ge = Ce(Ze);
                  return Math.abs(Ge[0] - Ve[0]) > 2 || Math.abs(Ge[1] - Ve[1]) > 2;
                }(Se)) return De.scale(Ce.scale()), void De.translate(Ce.translate());
                Ce.scale(te.event.scale), Ce.translate([Ee[0], te.event.translate[1]]), Le ? Pe(Qe) && (Me = Pe(Qe), ke = [xe[0] + (Me[0] - Le[0]), we[1], we[2]], Ce.rotate(ke), xe = ke) : Le = Pe(Se = Qe), Re = !0, be.render(!0);
                var qe = Ce.rotate(), Xe = Ce.invert(be.midPt);
                be.graphDiv.emit("plotly_relayouting", { "geo.projection.scale": Ce.scale() / be.fitScale, "geo.center.lon": Xe[0], "geo.center.lat": Xe[1], "geo.projection.rotation.lon": -qe[0] });
              }).on("zoomend", function() {
                te.select(this).style(ce), Re && ue(be, Ce, $e);
              }), De;
            }
            function he(be, Ce) {
              var Se;
              Ce.rotate(), Ce.scale();
              var we = le(0, Ce), Ee = function(ke) {
                for (var Me = 0, Re = arguments.length, De = []; ++Me < Re; ) De.push(arguments[Me]);
                var Pe = te.dispatch.apply(null, De);
                return Pe.of = function($e, qe) {
                  return function(Xe) {
                    var Ve;
                    try {
                      Ve = Xe.sourceEvent = te.event, Xe.target = ke, te.event = Xe, Pe[Xe.type].apply($e, qe);
                    } finally {
                      te.event = Ve;
                    }
                  };
                }, Pe;
              }(we, "zoomstart", "zoom", "zoomend"), xe = 0, Le = we.on;
              function Qe(ke) {
                var Me = Ce.rotate();
                ke("projection.rotation.lon", -Me[0]), ke("projection.rotation.lat", -Me[1]);
              }
              return we.on("zoomstart", function() {
                te.select(this).style(se);
                var ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze, Ge, ze = te.mouse(this), We = Ce.rotate(), at = We, ct = Ce.translate(), et = (Me = 0.5 * (ke = We)[0] * ae, Re = 0.5 * ke[1] * ae, De = 0.5 * ke[2] * ae, Pe = Math.sin(Me), $e = Math.cos(Me), qe = Math.sin(Re), Xe = Math.cos(Re), Ve = Math.sin(De), [$e * Xe * (Ze = Math.cos(De)) + Pe * qe * Ve, Pe * Xe * Ze - $e * qe * Ve, $e * qe * Ze + Pe * Xe * Ve, $e * Xe * Ve - Pe * qe * Ze]);
                Se = me(Ce, ze), Le.call(we, "zoom", function() {
                  var dt, Je, it, st, gt, bt, yt, St, Lt, Rt, vt = te.mouse(this);
                  if (Ce.scale(te.event.scale), Se) {
                    if (me(Ce, vt)) {
                      Ce.rotate(We).translate(ct);
                      var $t = me(Ce, vt), Pt = function(Gt, Ft) {
                        if (Gt && Ft) {
                          var Tt = function(Vt, Yt) {
                            return [Vt[1] * Yt[2] - Vt[2] * Yt[1], Vt[2] * Yt[0] - Vt[0] * Yt[2], Vt[0] * Yt[1] - Vt[1] * Yt[0]];
                          }(Gt, Ft), It = Math.sqrt(ve(Tt, Tt)), Ct = 0.5 * Math.acos(Math.max(-1, Math.min(1, ve(Gt, Ft)))), Kt = Math.sin(Ct) / It;
                          return It && [Math.cos(Ct), Tt[2] * Kt, -Tt[1] * Kt, Tt[0] * Kt];
                        }
                      }(Se, $t), Xt = function(Gt) {
                        return [Math.atan2(2 * (Gt[0] * Gt[1] + Gt[2] * Gt[3]), 1 - 2 * (Gt[1] * Gt[1] + Gt[2] * Gt[2])) * oe, Math.asin(Math.max(-1, Math.min(1, 2 * (Gt[0] * Gt[2] - Gt[3] * Gt[1])))) * oe, Math.atan2(2 * (Gt[0] * Gt[3] + Gt[1] * Gt[2]), 1 - 2 * (Gt[2] * Gt[2] + Gt[3] * Gt[3])) * oe];
                      }((it = (dt = et)[0], st = dt[1], gt = dt[2], bt = dt[3], [it * (yt = (Je = Pt)[0]) - st * (St = Je[1]) - gt * (Lt = Je[2]) - bt * (Rt = Je[3]), it * St + st * yt + gt * Rt - bt * Lt, it * Lt - st * Rt + gt * yt + bt * St, it * Rt + st * Lt - gt * St + bt * yt])), Nt = function(Gt, Ft, Tt) {
                        var It = ge(Ft, 2, Gt[0]);
                        It = ge(It, 1, Gt[1]), It = ge(It, 0, Gt[2] - Tt[2]);
                        var Ct, Kt, Vt = Ft[0], Yt = Ft[1], sn = Ft[2], hn = It[0], Qt = It[1], Ut = It[2], Wt = Math.atan2(Yt, Vt) * oe, rn = Math.sqrt(Vt * Vt + Yt * Yt);
                        Math.abs(Qt) > rn ? (Kt = (Qt > 0 ? 90 : -90) - Wt, Ct = 0) : (Kt = Math.asin(Qt / rn) * oe - Wt, Ct = Math.sqrt(rn * rn - Qt * Qt));
                        var vn = 180 - Kt - 2 * Wt, dn = (Math.atan2(Ut, hn) - Math.atan2(sn, Ct)) * oe, Ht = (Math.atan2(Ut, hn) - Math.atan2(sn, -Ct)) * oe;
                        return pe(Tt[0], Tt[1], Kt, dn) <= pe(Tt[0], Tt[1], vn, Ht) ? [Kt, dn, Tt[2]] : [vn, Ht, Tt[2]];
                      }(Xt, Se, at);
                      isFinite(Nt[0]) && isFinite(Nt[1]) && isFinite(Nt[2]) || (Nt = at), Ce.rotate(Nt), at = Nt;
                    }
                  } else Se = me(Ce, ze = vt);
                  Ee.of(this, arguments)({ type: "zoom" });
                }), Ge = Ee.of(this, arguments), xe++ || Ge({ type: "zoomstart" });
              }).on("zoomend", function() {
                var ke;
                te.select(this).style(ce), Le.call(we, "zoom", null), ke = Ee.of(this, arguments), --xe || ke({ type: "zoomend" }), ue(be, Ce, Qe);
              }).on("zoom.redraw", function() {
                be.render(!0);
                var ke = Ce.rotate();
                be.graphDiv.emit("plotly_relayouting", { "geo.projection.scale": Ce.scale() / be.fitScale, "geo.projection.rotation.lon": -ke[0], "geo.projection.rotation.lat": -ke[1] });
              }), te.rebind(we, Ee, "on");
            }
            function me(be, Ce) {
              var Se = be.invert(Ce);
              return Se && isFinite(Se[0]) && isFinite(Se[1]) && function(we) {
                var Ee = we[0] * ae, xe = we[1] * ae, Le = Math.cos(xe);
                return [Le * Math.cos(Ee), Le * Math.sin(Ee), Math.sin(xe)];
              }(Se);
            }
            function pe(be, Ce, Se, we) {
              var Ee = ye(Se - be), xe = ye(we - Ce);
              return Math.sqrt(Ee * Ee + xe * xe);
            }
            function ye(be) {
              return (be % 360 + 540) % 360 - 180;
            }
            function ge(be, Ce, Se) {
              var we = Se * ae, Ee = be.slice(), xe = Ce === 0 ? 1 : 0, Le = Ce === 2 ? 1 : 2, Qe = Math.cos(we), ke = Math.sin(we);
              return Ee[xe] = be[xe] * Qe - be[Le] * ke, Ee[Le] = be[Le] * Qe + be[xe] * ke, Ee;
            }
            function ve(be, Ce) {
              for (var Se = 0, we = 0, Ee = be.length; we < Ee; ++we) Se += be[we] * Ce[we];
              return Se;
            }
            G.exports = function(be, Ce) {
              var Se = be.projection;
              return (Ce._isScoped ? de : Ce._isClipped ? he : fe)(be, Se);
            };
          }, 4173: function(G, J, Z) {
            var te = Z(33626), ne = Z(54826).SUBPLOT_PATTERN;
            J.fX = function(ee, ae, oe) {
              var se = te.subplotsRegistry[ae];
              if (!se) return [];
              for (var ce = se.attr, le = [], ue = 0; ue < ee.length; ue++) {
                var de = ee[ue];
                de[0].trace[ce] === oe && le.push(de);
              }
              return le;
            }, J.eV = function(ee, ae, oe) {
              var se, ce = [], le = [];
              if (!(se = typeof ae == "string" ? te.getModule(ae).plot : typeof ae == "function" ? ae : ae.plot)) return [ce, ee];
              for (var ue = oe, de = 0; de < ee.length; de++) {
                var fe = ee[de], he = fe[0].trace, me = he.zorder !== void 0;
                he.visible === !0 && he._length !== 0 && (!he._module || he._module.plot !== se || me && he.zorder !== ue ? le.push(fe) : ce.push(fe));
              }
              return [ce, le];
            }, J.KO = function(ee, ae, oe) {
              if (!te.subplotsRegistry[ae]) return [];
              var se, ce, le, ue = te.subplotsRegistry[ae].attr, de = [];
              if (ae === "gl2d") {
                var fe = oe.match(ne);
                ce = "x" + fe[1], le = "y" + fe[2];
              }
              for (var he = 0; he < ee.length; he++) se = ee[he], ae === "gl2d" && te.traceIs(se, "gl2d") ? se[ue[0]] === ce && se[ue[1]] === le && de.push(se) : se[ue] === oe && de.push(se);
              return de;
            };
          }, 77055: function(G, J, Z) {
            var te = Z(99978), ne = Z(20573), ee = Z(44039), ae = Z(54826), oe = Z(74043);
            function se(ce, le) {
              this.element = ce, this.plot = le, this.mouseListener = null, this.wheelListener = null, this.lastInputTime = Date.now(), this.lastPos = [0, 0], this.boxEnabled = !1, this.boxInited = !1, this.boxStart = [0, 0], this.boxEnd = [0, 0], this.dragStart = [0, 0];
            }
            G.exports = function(ce) {
              var le = ce.mouseContainer, ue = ce.glplot, de = new se(le, ue);
              function fe() {
                ce.xaxis.autorange = !1, ce.yaxis.autorange = !1;
              }
              function he(me, pe, ye) {
                var ge, ve, be = ce.calcDataBox(), Ce = ue.viewBox, Se = de.lastPos[0], we = de.lastPos[1], Ee = ae.MINDRAG * ue.pixelRatio, xe = ae.MINZOOM * ue.pixelRatio;
                function Le(Pe, $e, qe) {
                  var Xe = Math.min($e, qe), Ve = Math.max($e, qe);
                  Xe !== Ve ? (be[Pe] = Xe, be[Pe + 2] = Ve, de.dataBox = be, ce.setRanges(be)) : (ce.selectBox.selectBox = [0, 0, 1, 1], ce.glplot.setDirty());
                }
                switch (pe *= ue.pixelRatio, ye *= ue.pixelRatio, ye = Ce[3] - Ce[1] - ye, ce.fullLayout.dragmode) {
                  case "zoom":
                    if (me) {
                      var Qe = pe / (Ce[2] - Ce[0]) * (be[2] - be[0]) + be[0], ke = ye / (Ce[3] - Ce[1]) * (be[3] - be[1]) + be[1];
                      de.boxInited || (de.boxStart[0] = Qe, de.boxStart[1] = ke, de.dragStart[0] = pe, de.dragStart[1] = ye), de.boxEnd[0] = Qe, de.boxEnd[1] = ke, de.boxInited = !0, de.boxEnabled || de.boxStart[0] === de.boxEnd[0] && de.boxStart[1] === de.boxEnd[1] || (de.boxEnabled = !0);
                      var Me = Math.abs(de.dragStart[0] - pe) < xe, Re = Math.abs(de.dragStart[1] - ye) < xe;
                      if (!function() {
                        for (var Pe = ce.graphDiv._fullLayout._axisConstraintGroups, $e = ce.xaxis._id, qe = ce.yaxis._id, Xe = 0; Xe < Pe.length; Xe++) if (Pe[Xe][$e] !== -1) {
                          if (Pe[Xe][qe] !== -1) return !0;
                          break;
                        }
                        return !1;
                      }() || Me && Re) Me && (de.boxEnd[0] = de.boxStart[0]), Re && (de.boxEnd[1] = de.boxStart[1]);
                      else {
                        ge = de.boxEnd[0] - de.boxStart[0], ve = de.boxEnd[1] - de.boxStart[1];
                        var De = (be[3] - be[1]) / (be[2] - be[0]);
                        Math.abs(ge * De) > Math.abs(ve) ? (de.boxEnd[1] = de.boxStart[1] + Math.abs(ge) * De * (ve >= 0 ? 1 : -1), de.boxEnd[1] < be[1] ? (de.boxEnd[1] = be[1], de.boxEnd[0] = de.boxStart[0] + (be[1] - de.boxStart[1]) / Math.abs(De)) : de.boxEnd[1] > be[3] && (de.boxEnd[1] = be[3], de.boxEnd[0] = de.boxStart[0] + (be[3] - de.boxStart[1]) / Math.abs(De))) : (de.boxEnd[0] = de.boxStart[0] + Math.abs(ve) / De * (ge >= 0 ? 1 : -1), de.boxEnd[0] < be[0] ? (de.boxEnd[0] = be[0], de.boxEnd[1] = de.boxStart[1] + (be[0] - de.boxStart[0]) * Math.abs(De)) : de.boxEnd[0] > be[2] && (de.boxEnd[0] = be[2], de.boxEnd[1] = de.boxStart[1] + (be[2] - de.boxStart[0]) * Math.abs(De)));
                      }
                    } else de.boxEnabled ? (ge = de.boxStart[0] !== de.boxEnd[0], ve = de.boxStart[1] !== de.boxEnd[1], ge || ve ? (ge && (Le(0, de.boxStart[0], de.boxEnd[0]), ce.xaxis.autorange = !1), ve && (Le(1, de.boxStart[1], de.boxEnd[1]), ce.yaxis.autorange = !1), ce.relayoutCallback()) : ce.glplot.setDirty(), de.boxEnabled = !1, de.boxInited = !1) : de.boxInited && (de.boxInited = !1);
                    break;
                  case "pan":
                    de.boxEnabled = !1, de.boxInited = !1, me ? (de.panning || (de.dragStart[0] = pe, de.dragStart[1] = ye), Math.abs(de.dragStart[0] - pe) < Ee && (pe = de.dragStart[0]), Math.abs(de.dragStart[1] - ye) < Ee && (ye = de.dragStart[1]), ge = (Se - pe) * (be[2] - be[0]) / (ue.viewBox[2] - ue.viewBox[0]), ve = (we - ye) * (be[3] - be[1]) / (ue.viewBox[3] - ue.viewBox[1]), be[0] += ge, be[2] += ge, be[1] += ve, be[3] += ve, ce.setRanges(be), de.panning = !0, de.lastInputTime = Date.now(), fe(), ce.cameraChanged(), ce.handleAnnotations()) : de.panning && (de.panning = !1, ce.relayoutCallback());
                }
                de.lastPos[0] = pe, de.lastPos[1] = ye;
              }
              return de.mouseListener = te(le, he), le.addEventListener("touchstart", function(me) {
                var pe = ee(me.changedTouches[0], le);
                he(0, pe[0], pe[1]), he(1, pe[0], pe[1]), me.preventDefault();
              }, !!oe && { passive: !1 }), le.addEventListener("touchmove", function(me) {
                me.preventDefault();
                var pe = ee(me.changedTouches[0], le);
                he(1, pe[0], pe[1]), me.preventDefault();
              }, !!oe && { passive: !1 }), le.addEventListener("touchend", function(me) {
                he(0, de.lastPos[0], de.lastPos[1]), me.preventDefault();
              }, !!oe && { passive: !1 }), de.wheelListener = ne(le, function(me, pe) {
                if (!ce.scrollZoom) return !1;
                var ye = ce.calcDataBox(), ge = ue.viewBox, ve = de.lastPos[0], be = de.lastPos[1], Ce = Math.exp(5 * pe / (ge[3] - ge[1])), Se = ve / (ge[2] - ge[0]) * (ye[2] - ye[0]) + ye[0], we = be / (ge[3] - ge[1]) * (ye[3] - ye[1]) + ye[1];
                return ye[0] = (ye[0] - Se) * Ce + Se, ye[2] = (ye[2] - Se) * Ce + Se, ye[1] = (ye[1] - we) * Ce + we, ye[3] = (ye[3] - we) * Ce + we, ce.setRanges(ye), de.lastInputTime = Date.now(), fe(), ce.cameraChanged(), ce.handleAnnotations(), ce.relayoutCallback(), !0;
              }, !0), de;
            };
          }, 10749: function(G, J, Z) {
            var te = Z(29714), ne = Z(55010);
            function ee(se) {
              this.scene = se, this.gl = se.gl, this.pixelRatio = se.pixelRatio, this.screenBox = [0, 0, 1, 1], this.viewBox = [0, 0, 1, 1], this.dataBox = [-1, -1, 1, 1], this.borderLineEnable = [!1, !1, !1, !1], this.borderLineWidth = [1, 1, 1, 1], this.borderLineColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.ticks = [[], []], this.tickEnable = [!0, !0, !1, !1], this.tickPad = [15, 15, 15, 15], this.tickAngle = [0, 0, 0, 0], this.tickColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.tickMarkLength = [0, 0, 0, 0], this.tickMarkWidth = [0, 0, 0, 0], this.tickMarkColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.labels = ["x", "y"], this.labelEnable = [!0, !0, !1, !1], this.labelAngle = [0, Math.PI / 2, 0, 3 * Math.PI / 2], this.labelPad = [15, 15, 15, 15], this.labelSize = [12, 12], this.labelFont = ["sans-serif", "sans-serif"], this.labelColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.title = "", this.titleEnable = !0, this.titleCenter = [0, 0, 0, 0], this.titleAngle = 0, this.titleColor = [0, 0, 0, 1], this.titleFont = "sans-serif", this.titleSize = 18, this.gridLineEnable = [!0, !0], this.gridLineColor = [[0, 0, 0, 0.5], [0, 0, 0, 0.5]], this.gridLineWidth = [1, 1], this.zeroLineEnable = [!0, !0], this.zeroLineWidth = [1, 1], this.zeroLineColor = [[0, 0, 0, 1], [0, 0, 0, 1]], this.borderColor = [0, 0, 0, 0], this.backgroundColor = [0, 0, 0, 0], this.static = this.scene.staticPlot;
            }
            var ae = ee.prototype, oe = ["xaxis", "yaxis"];
            ae.merge = function(se) {
              var ce, le, ue, de, fe, he, me, pe, ye, ge, ve;
              for (this.titleEnable = !1, this.backgroundColor = ne(se.plot_bgcolor), ge = 0; ge < 2; ++ge) {
                var be = (ce = oe[ge]).charAt(0);
                for (ue = (le = se[this.scene[ce]._name]).title.text === this.scene.fullLayout._dfltTitle[be] ? "" : le.title.text, ve = 0; ve <= 2; ve += 2) this.labelEnable[ge + ve] = !1, this.labels[ge + ve] = ue, this.labelColor[ge + ve] = ne(le.title.font.color), this.labelFont[ge + ve] = le.title.font.family, this.labelSize[ge + ve] = le.title.font.size, this.labelPad[ge + ve] = this.getLabelPad(ce, le), this.tickEnable[ge + ve] = !1, this.tickColor[ge + ve] = ne((le.tickfont || {}).color), this.tickAngle[ge + ve] = le.tickangle === "auto" ? 0 : Math.PI * -le.tickangle / 180, this.tickPad[ge + ve] = this.getTickPad(le), this.tickMarkLength[ge + ve] = 0, this.tickMarkWidth[ge + ve] = le.tickwidth || 0, this.tickMarkColor[ge + ve] = ne(le.tickcolor), this.borderLineEnable[ge + ve] = !1, this.borderLineColor[ge + ve] = ne(le.linecolor), this.borderLineWidth[ge + ve] = le.linewidth || 0;
                me = this.hasSharedAxis(le), fe = this.hasAxisInDfltPos(ce, le) && !me, he = this.hasAxisInAltrPos(ce, le) && !me, de = le.mirror || !1, pe = me ? String(de).indexOf("all") !== -1 : !!de, ye = me ? de === "allticks" : String(de).indexOf("ticks") !== -1, fe ? this.labelEnable[ge] = !0 : he && (this.labelEnable[ge + 2] = !0), fe ? this.tickEnable[ge] = le.showticklabels : he && (this.tickEnable[ge + 2] = le.showticklabels), (fe || pe) && (this.borderLineEnable[ge] = le.showline), (he || pe) && (this.borderLineEnable[ge + 2] = le.showline), (fe || ye) && (this.tickMarkLength[ge] = this.getTickMarkLength(le)), (he || ye) && (this.tickMarkLength[ge + 2] = this.getTickMarkLength(le)), this.gridLineEnable[ge] = le.showgrid, this.gridLineColor[ge] = ne(le.gridcolor), this.gridLineWidth[ge] = le.gridwidth, this.zeroLineEnable[ge] = le.zeroline, this.zeroLineColor[ge] = ne(le.zerolinecolor), this.zeroLineWidth[ge] = le.zerolinewidth;
              }
            }, ae.hasSharedAxis = function(se) {
              var ce = this.scene, le = ce.fullLayout._subplots.gl2d;
              return te.findSubplotsWithAxis(le, se).indexOf(ce.id) !== 0;
            }, ae.hasAxisInDfltPos = function(se, ce) {
              var le = ce.side;
              return se === "xaxis" ? le === "bottom" : se === "yaxis" ? le === "left" : void 0;
            }, ae.hasAxisInAltrPos = function(se, ce) {
              var le = ce.side;
              return se === "xaxis" ? le === "top" : se === "yaxis" ? le === "right" : void 0;
            }, ae.getLabelPad = function(se, ce) {
              var le = 1.5, ue = ce.title.font.size, de = ce.showticklabels;
              return se === "xaxis" ? ce.side === "top" ? ue * (le + (de ? 1 : 0)) - 10 : ue * (le + (de ? 0.5 : 0)) - 10 : se === "yaxis" ? ce.side === "right" ? 10 + ue * (le + (de ? 1 : 0.5)) : 10 + ue * (le + (de ? 0.5 : 0)) : void 0;
            }, ae.getTickPad = function(se) {
              return se.ticks === "outside" ? 10 + se.ticklen : 15;
            }, ae.getTickMarkLength = function(se) {
              if (!se.ticks) return 0;
              var ce = se.ticklen;
              return se.ticks === "inside" ? -ce : ce;
            }, G.exports = function(se) {
              return new ee(se);
            };
          }, 24585: function(G, J, Z) {
            var te = Z(13582).overrideAll, ne = Z(27672), ee = Z(6704), ae = Z(62972), oe = Z(54826), se = Z(37703), ce = Z(6811), le = Z(4173).KO;
            J.name = "gl2d", J.attr = ["xaxis", "yaxis"], J.idRoot = ["x", "y"], J.idRegex = oe.idRegex, J.attrRegex = oe.attrRegex, J.attributes = Z(55126), J.supplyLayoutDefaults = function(ue, de, fe) {
              de._has("cartesian") || se.supplyLayoutDefaults(ue, de, fe);
            }, J.layoutAttrOverrides = te(se.layoutAttributes, "plot", "from-root"), J.baseLayoutAttrOverrides = te({ plot_bgcolor: ee.plot_bgcolor, hoverlabel: ce.hoverlabel }, "plot", "nested"), J.plot = function(ue) {
              for (var de = ue._fullLayout, fe = ue._fullData, he = de._subplots.gl2d, me = 0; me < he.length; me++) {
                var pe = he[me], ye = de._plots[pe], ge = le(fe, "gl2d", pe), ve = ye._scene2d;
                ve === void 0 && (ve = new ne({ id: pe, graphDiv: ue, container: ue.querySelector(".gl-container"), staticPlot: ue._context.staticPlot, plotGlPixelRatio: ue._context.plotGlPixelRatio }, de), ye._scene2d = ve), ve.plot(ge, ue.calcdata, de, ue.layout);
              }
            }, J.clean = function(ue, de, fe, he) {
              for (var me = he._subplots.gl2d || [], pe = 0; pe < me.length; pe++) {
                var ye = me[pe], ge = he._plots[ye];
                ge._scene2d && le(ue, "gl2d", ye).length === 0 && (ge._scene2d.destroy(), delete he._plots[ye]);
              }
              se.clean.apply(this, arguments);
            }, J.drawFramework = function(ue) {
              ue._context.staticPlot || se.drawFramework(ue);
            }, J.toSVG = function(ue) {
              for (var de = ue._fullLayout, fe = de._subplots.gl2d, he = 0; he < fe.length; he++) {
                var me = de._plots[fe[he]]._scene2d, pe = me.toImage("png");
                de._glimages.append("svg:image").attr({ xmlns: ae.svg, "xlink:href": pe, x: 0, y: 0, width: "100%", height: "100%", preserveAspectRatio: "none" }), me.destroy();
              }
            }, J.updateFx = function(ue) {
              for (var de = ue._fullLayout, fe = de._subplots.gl2d, he = 0; he < fe.length; he++) de._plots[fe[he]]._scene2d.updateFx(de.dragmode);
            };
          }, 27672: function(G, J, Z) {
            var te, ne, ee = Z(33626), ae = Z(29714), oe = Z(32141), se = Z(99098).gl_plot2d, ce = Z(99098).gl_spikes2d, le = Z(99098).gl_select_box, ue = Z(22248), de = Z(10749), fe = Z(77055), he = Z(97464), me = Z(84391), pe = me.enforce, ye = me.clean, ge = Z(32919).doAutoRange, ve = Z(70414), be = ve.drawMode, Ce = ve.selectMode, Se = ["xaxis", "yaxis"], we = Z(54826).SUBPLOT_PATTERN;
            function Ee(Le, Qe) {
              this.container = Le.container, this.graphDiv = Le.graphDiv, this.pixelRatio = Le.plotGlPixelRatio || window.devicePixelRatio, this.id = Le.id, this.staticPlot = !!Le.staticPlot, this.scrollZoom = this.graphDiv._context._scrollZoom.cartesian, this.fullData = null, this.updateRefs(Qe), this.makeFramework(), this.stopped || (this.glplotOptions = de(this), this.glplotOptions.merge(Qe), this.glplot = se(this.glplotOptions), this.camera = fe(this), this.traces = {}, this.spikes = ce(this.glplot), this.selectBox = le(this.glplot, { innerFill: !1, outerFill: !0 }), this.lastButtonState = 0, this.pickResult = null, this.isMouseOver = !0, this.stopped = !1, this.redraw = this.draw.bind(this), this.redraw());
            }
            G.exports = Ee;
            var xe = Ee.prototype;
            xe.makeFramework = function() {
              if (this.staticPlot) {
                if (!(ne || (te = document.createElement("canvas"), ne = ue({ canvas: te, preserveDrawingBuffer: !1, premultipliedAlpha: !0, antialias: !0 })))) throw new Error("Error creating static canvas/context for image server");
                this.canvas = te, this.gl = ne;
              } else {
                var Le = this.container.querySelector(".gl-canvas-focus"), Qe = ue({ canvas: Le, preserveDrawingBuffer: !0, premultipliedAlpha: !0 });
                if (!Qe) return he(this), void (this.stopped = !0);
                this.canvas = Le, this.gl = Qe;
              }
              var ke = this.canvas;
              ke.style.width = "100%", ke.style.height = "100%", ke.style.position = "absolute", ke.style.top = "0px", ke.style.left = "0px", ke.style["pointer-events"] = "none", this.updateSize(ke);
              var Me = this.svgContainer = document.createElementNS("http://www.w3.org/2000/svg", "svg");
              Me.style.position = "absolute", Me.style.top = Me.style.left = "0px", Me.style.width = Me.style.height = "100%", Me.style["z-index"] = 20, Me.style["pointer-events"] = "none";
              var Re = this.mouseContainer = document.createElement("div");
              Re.style.position = "absolute", Re.style["pointer-events"] = "auto", this.pickCanvas = this.container.querySelector(".gl-canvas-pick");
              var De = this.container;
              De.appendChild(Me), De.appendChild(Re);
              var Pe = this;
              Re.addEventListener("mouseout", function() {
                Pe.isMouseOver = !1, Pe.unhover();
              }), Re.addEventListener("mouseover", function() {
                Pe.isMouseOver = !0;
              });
            }, xe.toImage = function(Le) {
              Le || (Le = "png"), this.stopped = !0, this.staticPlot && this.container.appendChild(te), this.updateSize(this.canvas);
              var Qe = this.glplot.gl, ke = Qe.drawingBufferWidth, Me = Qe.drawingBufferHeight;
              Qe.clearColor(1, 1, 1, 0), Qe.clear(Qe.COLOR_BUFFER_BIT | Qe.DEPTH_BUFFER_BIT), this.glplot.setDirty(), this.glplot.draw(), Qe.bindFramebuffer(Qe.FRAMEBUFFER, null);
              var Re = new Uint8Array(ke * Me * 4);
              Qe.readPixels(0, 0, ke, Me, Qe.RGBA, Qe.UNSIGNED_BYTE, Re);
              for (var De = 0, Pe = Me - 1; De < Pe; ++De, --Pe) for (var $e = 0; $e < ke; ++$e) for (var qe = 0; qe < 4; ++qe) {
                var Xe = Re[4 * (ke * De + $e) + qe];
                Re[4 * (ke * De + $e) + qe] = Re[4 * (ke * Pe + $e) + qe], Re[4 * (ke * Pe + $e) + qe] = Xe;
              }
              var Ve = document.createElement("canvas");
              Ve.width = ke, Ve.height = Me;
              var Ze, Ge = Ve.getContext("2d", { willReadFrequently: !0 }), ze = Ge.createImageData(ke, Me);
              switch (ze.data.set(Re), Ge.putImageData(ze, 0, 0), Le) {
                case "jpeg":
                  Ze = Ve.toDataURL("image/jpeg");
                  break;
                case "webp":
                  Ze = Ve.toDataURL("image/webp");
                  break;
                default:
                  Ze = Ve.toDataURL("image/png");
              }
              return this.staticPlot && this.container.removeChild(te), Ze;
            }, xe.updateSize = function(Le) {
              Le || (Le = this.canvas);
              var Qe = this.pixelRatio, ke = this.fullLayout, Me = ke.width, Re = ke.height, De = 0 | Math.ceil(Qe * Me), Pe = 0 | Math.ceil(Qe * Re);
              return Le.width === De && Le.height === Pe || (Le.width = De, Le.height = Pe), Le;
            }, xe.computeTickMarks = function() {
              this.xaxis.setScale(), this.yaxis.setScale();
              for (var Le = [ae.calcTicks(this.xaxis), ae.calcTicks(this.yaxis)], Qe = 0; Qe < 2; ++Qe) for (var ke = 0; ke < Le[Qe].length; ++ke) Le[Qe][ke].text = Le[Qe][ke].text + "";
              return Le;
            }, xe.updateRefs = function(Le) {
              this.fullLayout = Le;
              var Qe = this.id.match(we), ke = "xaxis" + Qe[1], Me = "yaxis" + Qe[2];
              this.xaxis = this.fullLayout[ke], this.yaxis = this.fullLayout[Me];
            }, xe.relayoutCallback = function() {
              var Le = this.graphDiv, Qe = this.xaxis, ke = this.yaxis, Me = Le.layout, Re = {}, De = Re[Qe._name + ".range"] = Qe.range.slice(), Pe = Re[ke._name + ".range"] = ke.range.slice();
              Re[Qe._name + ".autorange"] = Qe.autorange, Re[ke._name + ".autorange"] = ke.autorange, ee.call("_storeDirectGUIEdit", Le.layout, Le._fullLayout._preGUI, Re);
              var $e = Me[Qe._name];
              $e.range = De, $e.autorange = Qe.autorange;
              var qe = Me[ke._name];
              qe.range = Pe, qe.autorange = ke.autorange, Re.lastInputTime = this.camera.lastInputTime, Le.emit("plotly_relayout", Re);
            }, xe.cameraChanged = function() {
              var Le = this.camera;
              this.glplot.setDataBox(this.calcDataBox());
              var Qe = this.computeTickMarks();
              (function(ke, Me) {
                for (var Re = 0; Re < 2; ++Re) {
                  var De = ke[Re], Pe = Me[Re];
                  if (De.length !== Pe.length) return !0;
                  for (var $e = 0; $e < De.length; ++$e) if (De[$e].x !== Pe[$e].x) return !0;
                }
                return !1;
              })(Qe, this.glplotOptions.ticks) && (this.glplotOptions.ticks = Qe, this.glplotOptions.dataBox = Le.dataBox, this.glplot.update(this.glplotOptions), this.handleAnnotations());
            }, xe.handleAnnotations = function() {
              for (var Le = this.graphDiv, Qe = this.fullLayout.annotations, ke = 0; ke < Qe.length; ke++) {
                var Me = Qe[ke];
                Me.xref === this.xaxis._id && Me.yref === this.yaxis._id && ee.getComponentMethod("annotations", "drawOne")(Le, ke);
              }
            }, xe.destroy = function() {
              if (this.glplot) {
                var Le = this.traces;
                Le && Object.keys(Le).map(function(Qe) {
                  Le[Qe].dispose(), delete Le[Qe];
                }), this.glplot.dispose(), this.container.removeChild(this.svgContainer), this.container.removeChild(this.mouseContainer), this.fullData = null, this.glplot = null, this.stopped = !0, this.camera.mouseListener.enabled = !1, this.mouseContainer.removeEventListener("wheel", this.camera.wheelListener), this.camera = null;
              }
            }, xe.plot = function(Le, Qe, ke) {
              var Me = this.glplot;
              this.updateRefs(ke), this.xaxis.clearCalc(), this.yaxis.clearCalc(), this.updateTraces(Le, Qe), this.updateFx(ke.dragmode);
              var Re = ke.width, De = ke.height;
              this.updateSize(this.canvas);
              var Pe = this.glplotOptions;
              Pe.merge(ke), Pe.screenBox = [0, 0, Re, De];
              var $e = { _fullLayout: { _axisConstraintGroups: ke._axisConstraintGroups, xaxis: this.xaxis, yaxis: this.yaxis, _size: ke._size } };
              ye($e, this.xaxis), ye($e, this.yaxis);
              var qe, Xe, Ve = ke._size, Ze = this.xaxis.domain, Ge = this.yaxis.domain;
              for (Pe.viewBox = [Ve.l + Ze[0] * Ve.w, Ve.b + Ge[0] * Ve.h, Re - Ve.r - (1 - Ze[1]) * Ve.w, De - Ve.t - (1 - Ge[1]) * Ve.h], this.mouseContainer.style.width = Ve.w * (Ze[1] - Ze[0]) + "px", this.mouseContainer.style.height = Ve.h * (Ge[1] - Ge[0]) + "px", this.mouseContainer.height = Ve.h * (Ge[1] - Ge[0]), this.mouseContainer.style.left = Ve.l + Ze[0] * Ve.w + "px", this.mouseContainer.style.top = Ve.t + (1 - Ge[1]) * Ve.h + "px", Xe = 0; Xe < 2; ++Xe) (qe = this[Se[Xe]])._length = Pe.viewBox[Xe + 2] - Pe.viewBox[Xe], ge(this.graphDiv, qe), qe.setScale();
              pe($e), Pe.ticks = this.computeTickMarks(), Pe.dataBox = this.calcDataBox(), Pe.merge(ke), Me.update(Pe), this.glplot.draw();
            }, xe.calcDataBox = function() {
              var Le = this.xaxis, Qe = this.yaxis, ke = Le.range, Me = Qe.range, Re = Le.r2l, De = Qe.r2l;
              return [Re(ke[0]), De(Me[0]), Re(ke[1]), De(Me[1])];
            }, xe.setRanges = function(Le) {
              var Qe = this.xaxis, ke = this.yaxis, Me = Qe.l2r, Re = ke.l2r;
              Qe.range = [Me(Le[0]), Me(Le[2])], ke.range = [Re(Le[1]), Re(Le[3])];
            }, xe.updateTraces = function(Le, Qe) {
              var ke, Me, Re, De = Object.keys(this.traces);
              this.fullData = Le;
              e: for (ke = 0; ke < De.length; ke++) {
                var Pe = De[ke], $e = this.traces[Pe];
                for (Me = 0; Me < Le.length; Me++) if ((Re = Le[Me]).uid === Pe && Re.type === $e.type) continue e;
                $e.dispose(), delete this.traces[Pe];
              }
              for (ke = 0; ke < Le.length; ke++) {
                Re = Le[ke];
                var qe = Qe[ke], Xe = this.traces[Re.uid];
                Xe ? Xe.update(Re, qe) : (Xe = Re._module.plot(this, Re, qe), this.traces[Re.uid] = Xe);
              }
              this.glplot.objects.sort(function(Ve, Ze) {
                return Ve._trace.index - Ze._trace.index;
              });
            }, xe.updateFx = function(Le) {
              Ce(Le) || be(Le) ? (this.pickCanvas.style["pointer-events"] = "none", this.mouseContainer.style["pointer-events"] = "none") : (this.pickCanvas.style["pointer-events"] = "auto", this.mouseContainer.style["pointer-events"] = "auto"), this.mouseContainer.style.cursor = Le === "pan" ? "move" : Le === "zoom" ? "crosshair" : null;
            }, xe.emitPointAction = function(Le, Qe) {
              for (var ke, Me = Le.trace.uid, Re = Le.pointIndex, De = 0; De < this.fullData.length; De++) this.fullData[De].uid === Me && (ke = this.fullData[De]);
              var Pe = { x: Le.traceCoord[0], y: Le.traceCoord[1], curveNumber: ke.index, pointNumber: Re, data: ke._input, fullData: this.fullData, xaxis: this.xaxis, yaxis: this.yaxis };
              oe.appendArrayPointValue(Pe, ke, Re), this.graphDiv.emit(Qe, { points: [Pe] });
            }, xe.draw = function() {
              if (!this.stopped) {
                requestAnimationFrame(this.redraw);
                var Le = this.glplot, Qe = this.camera, ke = Qe.mouseListener, Me = this.lastButtonState === 1 && ke.buttons === 0, Re = this.fullLayout;
                this.lastButtonState = ke.buttons, this.cameraChanged();
                var De, Pe = ke.x * Le.pixelRatio, $e = this.canvas.height - Le.pixelRatio * ke.y;
                if (Qe.boxEnabled && Re.dragmode === "zoom") {
                  this.selectBox.enabled = !0;
                  for (var qe = this.selectBox.selectBox = [Math.min(Qe.boxStart[0], Qe.boxEnd[0]), Math.min(Qe.boxStart[1], Qe.boxEnd[1]), Math.max(Qe.boxStart[0], Qe.boxEnd[0]), Math.max(Qe.boxStart[1], Qe.boxEnd[1])], Xe = 0; Xe < 2; Xe++) Qe.boxStart[Xe] === Qe.boxEnd[Xe] && (qe[Xe] = Le.dataBox[Xe], qe[Xe + 2] = Le.dataBox[Xe + 2]);
                  Le.setDirty();
                } else if (!Qe.panning && this.isMouseOver) {
                  this.selectBox.enabled = !1;
                  var Ve = Re._size, Ze = this.xaxis.domain, Ge = this.yaxis.domain, ze = (De = Le.pick(Pe / Le.pixelRatio + Ve.l + Ze[0] * Ve.w, $e / Le.pixelRatio - (Ve.t + (1 - Ge[1]) * Ve.h))) && De.object._trace.handlePick(De);
                  if (ze && Me && this.emitPointAction(ze, "plotly_click"), De && De.object._trace.hoverinfo !== "skip" && Re.hovermode && ze && (!this.lastPickResult || this.lastPickResult.traceUid !== ze.trace.uid || this.lastPickResult.dataCoord[0] !== ze.dataCoord[0] || this.lastPickResult.dataCoord[1] !== ze.dataCoord[1])) {
                    var We = ze;
                    this.lastPickResult = { traceUid: ze.trace ? ze.trace.uid : null, dataCoord: ze.dataCoord.slice() }, this.spikes.update({ center: De.dataCoord }), We.screenCoord = [((Le.viewBox[2] - Le.viewBox[0]) * (De.dataCoord[0] - Le.dataBox[0]) / (Le.dataBox[2] - Le.dataBox[0]) + Le.viewBox[0]) / Le.pixelRatio, (this.canvas.height - (Le.viewBox[3] - Le.viewBox[1]) * (De.dataCoord[1] - Le.dataBox[1]) / (Le.dataBox[3] - Le.dataBox[1]) - Le.viewBox[1]) / Le.pixelRatio], this.emitPointAction(ze, "plotly_hover");
                    var at = this.fullData[We.trace.index] || {}, ct = We.pointIndex, et = oe.castHoverinfo(at, Re, ct);
                    if (et && et !== "all") {
                      var dt = et.split("+");
                      dt.indexOf("x") === -1 && (We.traceCoord[0] = void 0), dt.indexOf("y") === -1 && (We.traceCoord[1] = void 0), dt.indexOf("z") === -1 && (We.traceCoord[2] = void 0), dt.indexOf("text") === -1 && (We.textLabel = void 0), dt.indexOf("name") === -1 && (We.name = void 0);
                    }
                    oe.loneHover({ x: We.screenCoord[0], y: We.screenCoord[1], xLabel: this.hoverFormatter("xaxis", We.traceCoord[0]), yLabel: this.hoverFormatter("yaxis", We.traceCoord[1]), zLabel: We.traceCoord[2], text: We.textLabel, name: We.name, color: oe.castHoverOption(at, ct, "bgcolor") || We.color, borderColor: oe.castHoverOption(at, ct, "bordercolor"), fontFamily: oe.castHoverOption(at, ct, "font.family"), fontSize: oe.castHoverOption(at, ct, "font.size"), fontColor: oe.castHoverOption(at, ct, "font.color"), nameLength: oe.castHoverOption(at, ct, "namelength"), textAlign: oe.castHoverOption(at, ct, "align") }, { container: this.svgContainer, gd: this.graphDiv });
                  }
                }
                De || this.unhover(), Le.draw();
              }
            }, xe.unhover = function() {
              this.lastPickResult && (this.spikes.update({}), this.lastPickResult = null, this.graphDiv.emit("plotly_unhover"), oe.loneUnhover(this.svgContainer));
            }, xe.hoverFormatter = function(Le, Qe) {
              if (Qe !== void 0) {
                var ke = this[Le];
                return ae.tickText(ke, ke.c2l(Qe), "hover").text;
              }
            };
          }, 2487: function(G, J, Z) {
            var te = Z(13582).overrideAll, ne = Z(6811), ee = Z(20299), ae = Z(4173).KO, oe = Z(34809), se = Z(62972), ce = "gl3d", le = "scene";
            J.name = ce, J.attr = le, J.idRoot = le, J.idRegex = J.attrRegex = oe.counterRegex("scene"), J.attributes = Z(22597), J.layoutAttributes = Z(77168), J.baseLayoutAttrOverrides = te({ hoverlabel: ne.hoverlabel }, "plot", "nested"), J.supplyLayoutDefaults = Z(15250), J.plot = function(ue) {
              for (var de = ue._fullLayout, fe = ue._fullData, he = de._subplots[ce], me = 0; me < he.length; me++) {
                var pe = he[me], ye = ae(fe, ce, pe), ge = de[pe], ve = ge.camera, be = ge._scene;
                be || (be = new ee({ id: pe, graphDiv: ue, container: ue.querySelector(".gl-container"), staticPlot: ue._context.staticPlot, plotGlPixelRatio: ue._context.plotGlPixelRatio, camera: ve }, de), ge._scene = be), be.viewInitial || (be.viewInitial = { up: { x: ve.up.x, y: ve.up.y, z: ve.up.z }, eye: { x: ve.eye.x, y: ve.eye.y, z: ve.eye.z }, center: { x: ve.center.x, y: ve.center.y, z: ve.center.z } }), be.plot(ye, de, ue.layout);
              }
            }, J.clean = function(ue, de, fe, he) {
              for (var me = he._subplots[ce] || [], pe = 0; pe < me.length; pe++) {
                var ye = me[pe];
                !de[ye] && he[ye]._scene && (he[ye]._scene.destroy(), he._infolayer && he._infolayer.selectAll(".annotation-" + ye).remove());
              }
            }, J.toSVG = function(ue) {
              for (var de = ue._fullLayout, fe = de._subplots[ce], he = de._size, me = 0; me < fe.length; me++) {
                var pe = de[fe[me]], ye = pe.domain, ge = pe._scene, ve = ge.toImage("png");
                de._glimages.append("svg:image").attr({ xmlns: se.svg, "xlink:href": ve, x: he.l + he.w * ye.x[0], y: he.t + he.h * (1 - ye.y[1]), width: he.w * (ye.x[1] - ye.x[0]), height: he.h * (ye.y[1] - ye.y[0]), preserveAspectRatio: "none" }), ge.destroy();
              }
            }, J.cleanId = function(ue) {
              if (ue.match(/^scene[0-9]*$/)) {
                var de = ue.substr(5);
                return de === "1" && (de = ""), le + de;
              }
            }, J.updateFx = function(ue) {
              for (var de = ue._fullLayout, fe = de._subplots[ce], he = 0; he < fe.length; he++) de[fe[he]]._scene.updateFx(de.dragmode, de.hovermode);
            };
          }, 22597: function(G) {
            G.exports = { scene: { valType: "subplotid", dflt: "scene", editType: "calc+clearAxisTypes" } };
          }, 63397: function(G, J, Z) {
            var te = Z(78766), ne = Z(25829), ee = Z(93049).extendFlat, ae = Z(13582).overrideAll;
            G.exports = ae({ visible: ne.visible, showspikes: { valType: "boolean", dflt: !0 }, spikesides: { valType: "boolean", dflt: !0 }, spikethickness: { valType: "number", min: 0, dflt: 2 }, spikecolor: { valType: "color", dflt: te.defaultLine }, showbackground: { valType: "boolean", dflt: !1 }, backgroundcolor: { valType: "color", dflt: "rgba(204, 204, 204, 0.5)" }, showaxeslabels: { valType: "boolean", dflt: !0 }, color: ne.color, categoryorder: ne.categoryorder, categoryarray: ne.categoryarray, title: { text: ne.title.text, font: ne.title.font }, type: ee({}, ne.type, { values: ["-", "linear", "log", "date", "category"] }), autotypenumbers: ne.autotypenumbers, autorange: ne.autorange, autorangeoptions: { minallowed: ne.autorangeoptions.minallowed, maxallowed: ne.autorangeoptions.maxallowed, clipmin: ne.autorangeoptions.clipmin, clipmax: ne.autorangeoptions.clipmax, include: ne.autorangeoptions.include, editType: "plot" }, rangemode: ne.rangemode, minallowed: ne.minallowed, maxallowed: ne.maxallowed, range: ee({}, ne.range, { items: [{ valType: "any", editType: "plot", impliedEdits: { "^autorange": !1 } }, { valType: "any", editType: "plot", impliedEdits: { "^autorange": !1 } }], anim: !1 }), tickmode: ne.minor.tickmode, nticks: ne.nticks, tick0: ne.tick0, dtick: ne.dtick, tickvals: ne.tickvals, ticktext: ne.ticktext, ticks: ne.ticks, mirror: ne.mirror, ticklen: ne.ticklen, tickwidth: ne.tickwidth, tickcolor: ne.tickcolor, showticklabels: ne.showticklabels, labelalias: ne.labelalias, tickfont: ne.tickfont, tickangle: ne.tickangle, tickprefix: ne.tickprefix, showtickprefix: ne.showtickprefix, ticksuffix: ne.ticksuffix, showticksuffix: ne.showticksuffix, showexponent: ne.showexponent, exponentformat: ne.exponentformat, minexponent: ne.minexponent, separatethousands: ne.separatethousands, tickformat: ne.tickformat, tickformatstops: ne.tickformatstops, hoverformat: ne.hoverformat, showline: ne.showline, linecolor: ne.linecolor, linewidth: ne.linewidth, showgrid: ne.showgrid, gridcolor: ee({}, ne.gridcolor, { dflt: "rgb(204, 204, 204)" }), gridwidth: ne.gridwidth, zeroline: ne.zeroline, zerolinecolor: ne.zerolinecolor, zerolinewidth: ne.zerolinewidth, _deprecated: { title: ne._deprecated.title, titlefont: ne._deprecated.titlefont } }, "plot", "from-root");
          }, 34258: function(G, J, Z) {
            var te = Z(65657).mix, ne = Z(34809), ee = Z(78032), ae = Z(63397), oe = Z(4392), se = Z(97655), ce = ["xaxis", "yaxis", "zaxis"];
            G.exports = function(le, ue, de) {
              var fe, he;
              function me(ge, ve) {
                return ne.coerce(fe, he, ae, ge, ve);
              }
              for (var pe = 0; pe < ce.length; pe++) {
                var ye = ce[pe];
                fe = le[ye] || {}, (he = ee.newContainer(ue, ye))._id = ye[0] + de.scene, he._name = ye, oe(fe, he, me, de), se(fe, he, me, { font: de.font, letter: ye[0], data: de.data, showGrid: !0, noAutotickangles: !0, noTicklabelindex: !0, noTickson: !0, noTicklabelmode: !0, noTicklabelshift: !0, noTicklabelstandoff: !0, noTicklabelstep: !0, noTicklabelposition: !0, noTicklabeloverflow: !0, noInsiderange: !0, bgColor: de.bgColor, calendar: de.calendar }, de.fullLayout), me("gridcolor", te(he.color, de.bgColor, 72.72727272727273).toRgbString()), me("title.text", ye[0]), he.setScale = ne.noop, me("showspikes") && (me("spikesides"), me("spikethickness"), me("spikecolor", he.color)), me("showaxeslabels"), me("showbackground") && me("backgroundcolor");
              }
            };
          }, 95701: function(G, J, Z) {
            var te = Z(55010), ne = Z(34809), ee = ["xaxis", "yaxis", "zaxis"];
            function ae() {
              this.bounds = [[-10, -10, -10], [10, 10, 10]], this.ticks = [[], [], []], this.tickEnable = [!0, !0, !0], this.tickFont = ["sans-serif", "sans-serif", "sans-serif"], this.tickSize = [12, 12, 12], this.tickFontWeight = ["normal", "normal", "normal", "normal"], this.tickFontStyle = ["normal", "normal", "normal", "normal"], this.tickFontVariant = ["normal", "normal", "normal", "normal"], this.tickAngle = [0, 0, 0], this.tickColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.tickPad = [18, 18, 18], this.labels = ["x", "y", "z"], this.labelEnable = [!0, !0, !0], this.labelFont = ["Open Sans", "Open Sans", "Open Sans"], this.labelSize = [20, 20, 20], this.labelFontWeight = ["normal", "normal", "normal", "normal"], this.labelFontStyle = ["normal", "normal", "normal", "normal"], this.labelFontVariant = ["normal", "normal", "normal", "normal"], this.labelColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.labelPad = [30, 30, 30], this.lineEnable = [!0, !0, !0], this.lineMirror = [!1, !1, !1], this.lineWidth = [1, 1, 1], this.lineColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.lineTickEnable = [!0, !0, !0], this.lineTickMirror = [!1, !1, !1], this.lineTickLength = [10, 10, 10], this.lineTickWidth = [1, 1, 1], this.lineTickColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.gridEnable = [!0, !0, !0], this.gridWidth = [1, 1, 1], this.gridColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.zeroEnable = [!0, !0, !0], this.zeroLineColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.zeroLineWidth = [2, 2, 2], this.backgroundEnable = [!0, !0, !0], this.backgroundColor = [[0.8, 0.8, 0.8, 0.5], [0.8, 0.8, 0.8, 0.5], [0.8, 0.8, 0.8, 0.5]], this._defaultTickPad = this.tickPad.slice(), this._defaultLabelPad = this.labelPad.slice(), this._defaultLineTickLength = this.lineTickLength.slice();
            }
            ae.prototype.merge = function(oe, se) {
              for (var ce = this, le = 0; le < 3; ++le) {
                var ue = se[ee[le]];
                ue.visible ? (ce.labels[le] = oe._meta ? ne.templateString(ue.title.text, oe._meta) : ue.title.text, "font" in ue.title && (ue.title.font.color && (ce.labelColor[le] = te(ue.title.font.color)), ue.title.font.family && (ce.labelFont[le] = ue.title.font.family), ue.title.font.size && (ce.labelSize[le] = ue.title.font.size), ue.title.font.weight && (ce.labelFontWeight[le] = ue.title.font.weight), ue.title.font.style && (ce.labelFontStyle[le] = ue.title.font.style), ue.title.font.variant && (ce.labelFontVariant[le] = ue.title.font.variant)), "showline" in ue && (ce.lineEnable[le] = ue.showline), "linecolor" in ue && (ce.lineColor[le] = te(ue.linecolor)), "linewidth" in ue && (ce.lineWidth[le] = ue.linewidth), "showgrid" in ue && (ce.gridEnable[le] = ue.showgrid), "gridcolor" in ue && (ce.gridColor[le] = te(ue.gridcolor)), "gridwidth" in ue && (ce.gridWidth[le] = ue.gridwidth), ue.type === "log" ? ce.zeroEnable[le] = !1 : "zeroline" in ue && (ce.zeroEnable[le] = ue.zeroline), "zerolinecolor" in ue && (ce.zeroLineColor[le] = te(ue.zerolinecolor)), "zerolinewidth" in ue && (ce.zeroLineWidth[le] = ue.zerolinewidth), "ticks" in ue && ue.ticks ? ce.lineTickEnable[le] = !0 : ce.lineTickEnable[le] = !1, "ticklen" in ue && (ce.lineTickLength[le] = ce._defaultLineTickLength[le] = ue.ticklen), "tickcolor" in ue && (ce.lineTickColor[le] = te(ue.tickcolor)), "tickwidth" in ue && (ce.lineTickWidth[le] = ue.tickwidth), "tickangle" in ue && (ce.tickAngle[le] = ue.tickangle === "auto" ? -3600 : Math.PI * -ue.tickangle / 180), "showticklabels" in ue && (ce.tickEnable[le] = ue.showticklabels), "tickfont" in ue && (ue.tickfont.color && (ce.tickColor[le] = te(ue.tickfont.color)), ue.tickfont.family && (ce.tickFont[le] = ue.tickfont.family), ue.tickfont.size && (ce.tickSize[le] = ue.tickfont.size), ue.tickfont.weight && (ce.tickFontWeight[le] = ue.tickfont.weight), ue.tickfont.style && (ce.tickFontStyle[le] = ue.tickfont.style), ue.tickfont.variant && (ce.tickFontVariant[le] = ue.tickfont.variant)), "mirror" in ue ? ["ticks", "all", "allticks"].indexOf(ue.mirror) !== -1 ? (ce.lineTickMirror[le] = !0, ce.lineMirror[le] = !0) : ue.mirror === !0 ? (ce.lineTickMirror[le] = !1, ce.lineMirror[le] = !0) : (ce.lineTickMirror[le] = !1, ce.lineMirror[le] = !1) : ce.lineMirror[le] = !1, "showbackground" in ue && ue.showbackground !== !1 ? (ce.backgroundEnable[le] = !0, ce.backgroundColor[le] = te(ue.backgroundcolor)) : ce.backgroundEnable[le] = !1) : (ce.tickEnable[le] = !1, ce.labelEnable[le] = !1, ce.lineEnable[le] = !1, ce.lineTickEnable[le] = !1, ce.gridEnable[le] = !1, ce.zeroEnable[le] = !1, ce.backgroundEnable[le] = !1);
              }
            }, G.exports = function(oe, se) {
              var ce = new ae();
              return ce.merge(oe, se), ce;
            };
          }, 15250: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(33626), ae = Z(4448), oe = Z(34258), se = Z(77168), ce = Z(4173).KO, le = "gl3d";
            function ue(de, fe, he, me) {
              for (var pe = he("bgcolor"), ye = ne.combine(pe, me.paper_bgcolor), ge = ["up", "center", "eye"], ve = 0; ve < ge.length; ve++) he("camera." + ge[ve] + ".x"), he("camera." + ge[ve] + ".y"), he("camera." + ge[ve] + ".z");
              he("camera.projection.type");
              var be = !!he("aspectratio.x") && !!he("aspectratio.y") && !!he("aspectratio.z"), Ce = he("aspectmode", be ? "manual" : "auto");
              be || (de.aspectratio = fe.aspectratio = { x: 1, y: 1, z: 1 }, Ce === "manual" && (fe.aspectmode = "auto"), de.aspectmode = fe.aspectmode);
              var Se = ce(me.fullData, le, me.id);
              oe(de, fe, { font: me.font, scene: me.id, data: Se, bgColor: ye, calendar: me.calendar, autotypenumbersDflt: me.autotypenumbersDflt, fullLayout: me.fullLayout }), ee.getComponentMethod("annotations3d", "handleDefaults")(de, fe, me);
              var we = me.getDfltFromLayout("dragmode");
              if (we !== !1 && !we) if (we = "orbit", de.camera && de.camera.up) {
                var Ee = de.camera.up.x, xe = de.camera.up.y, Le = de.camera.up.z;
                Le !== 0 && (Ee && xe && Le ? Le / Math.sqrt(Ee * Ee + xe * xe + Le * Le) > 0.999 && (we = "turntable") : we = "turntable");
              } else we = "turntable";
              he("dragmode", we), he("hovermode", me.getDfltFromLayout("hovermode"));
            }
            G.exports = function(de, fe, he) {
              var me = fe._basePlotModules.length > 1;
              ae(de, fe, he, { type: le, attributes: se, handleDefaults: ue, fullLayout: fe, font: fe.font, fullData: he, getDfltFromLayout: function(pe) {
                if (!me) return te.validate(de[pe], se[pe]) ? de[pe] : void 0;
              }, autotypenumbersDflt: fe.autotypenumbers, paper_bgcolor: fe.paper_bgcolor, calendar: fe.calendar });
            };
          }, 77168: function(G, J, Z) {
            var te = Z(63397), ne = Z(13792).u, ee = Z(93049).extendFlat, ae = Z(34809).counterRegex;
            function oe(se, ce, le) {
              return { x: { valType: "number", dflt: se, editType: "camera" }, y: { valType: "number", dflt: ce, editType: "camera" }, z: { valType: "number", dflt: le, editType: "camera" }, editType: "camera" };
            }
            G.exports = { _arrayAttrRegexps: [ae("scene", ".annotations", !0)], bgcolor: { valType: "color", dflt: "rgba(0,0,0,0)", editType: "plot" }, camera: { up: ee(oe(0, 0, 1), {}), center: ee(oe(0, 0, 0), {}), eye: ee(oe(1.25, 1.25, 1.25), {}), projection: { type: { valType: "enumerated", values: ["perspective", "orthographic"], dflt: "perspective", editType: "calc" }, editType: "calc" }, editType: "camera" }, domain: ne({ name: "scene", editType: "plot" }), aspectmode: { valType: "enumerated", values: ["auto", "cube", "data", "manual"], dflt: "auto", editType: "plot", impliedEdits: { "aspectratio.x": void 0, "aspectratio.y": void 0, "aspectratio.z": void 0 } }, aspectratio: { x: { valType: "number", min: 0, editType: "plot", impliedEdits: { "^aspectmode": "manual" } }, y: { valType: "number", min: 0, editType: "plot", impliedEdits: { "^aspectmode": "manual" } }, z: { valType: "number", min: 0, editType: "plot", impliedEdits: { "^aspectmode": "manual" } }, editType: "plot", impliedEdits: { aspectmode: "manual" } }, xaxis: te, yaxis: te, zaxis: te, dragmode: { valType: "enumerated", values: ["orbit", "turntable", "zoom", "pan", !1], editType: "plot" }, hovermode: { valType: "enumerated", values: ["closest", !1], dflt: "closest", editType: "modebar" }, uirevision: { valType: "any", editType: "none" }, editType: "plot", _deprecated: { cameraposition: { valType: "info_array", editType: "camera" } } };
          }, 64087: function(G, J, Z) {
            var te = Z(55010), ne = ["xaxis", "yaxis", "zaxis"];
            function ee() {
              this.enabled = [!0, !0, !0], this.colors = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.drawSides = [!0, !0, !0], this.lineWidth = [1, 1, 1];
            }
            ee.prototype.merge = function(ae) {
              for (var oe = 0; oe < 3; ++oe) {
                var se = ae[ne[oe]];
                se.visible ? (this.enabled[oe] = se.showspikes, this.colors[oe] = te(se.spikecolor), this.drawSides[oe] = se.spikesides, this.lineWidth[oe] = se.spikethickness) : (this.enabled[oe] = !1, this.drawSides[oe] = !1);
              }
            }, G.exports = function(ae) {
              var oe = new ee();
              return oe.merge(ae), oe;
            };
          }, 32412: function(G, J, Z) {
            G.exports = function(oe) {
              for (var se = oe.axesOptions, ce = oe.glplot.axesPixels, le = oe.fullSceneLayout, ue = [[], [], []], de = 0; de < 3; ++de) {
                var fe = le[ee[de]];
                if (fe._length = (ce[de].hi - ce[de].lo) * ce[de].pixelsPerDataUnit / oe.dataScale[de], Math.abs(fe._length) === 1 / 0 || isNaN(fe._length)) ue[de] = [];
                else {
                  fe._input_range = fe.range.slice(), fe.range[0] = ce[de].lo / oe.dataScale[de], fe.range[1] = ce[de].hi / oe.dataScale[de], fe._m = 1 / (oe.dataScale[de] * ce[de].pixelsPerDataUnit), fe.range[0] === fe.range[1] && (fe.range[0] -= 1, fe.range[1] += 1);
                  var he = fe.tickmode;
                  if (fe.tickmode === "auto") {
                    fe.tickmode = "linear";
                    var me = fe.nticks || ne.constrain(fe._length / 40, 4, 9);
                    te.autoTicks(fe, Math.abs(fe.range[1] - fe.range[0]) / me);
                  }
                  for (var pe = te.calcTicks(fe, { msUTC: !0 }), ye = 0; ye < pe.length; ++ye) pe[ye].x = pe[ye].x * oe.dataScale[de], fe.type === "date" && (pe[ye].text = pe[ye].text.replace(/\<br\>/g, " "));
                  ue[de] = pe, fe.tickmode = he;
                }
              }
              for (se.ticks = ue, de = 0; de < 3; ++de) for (ae[de] = 0.5 * (oe.glplot.bounds[0][de] + oe.glplot.bounds[1][de]), ye = 0; ye < 2; ++ye) se.bounds[ye][de] = oe.glplot.bounds[ye][de];
              oe.contourLevels = function(ge) {
                for (var ve = new Array(3), be = 0; be < 3; ++be) {
                  for (var Ce = ge[be], Se = new Array(Ce.length), we = 0; we < Ce.length; ++we) Se[we] = Ce[we].x;
                  ve[be] = Se;
                }
                return ve;
              }(ue);
            };
            var te = Z(29714), ne = Z(34809), ee = ["xaxis", "yaxis", "zaxis"], ae = [0, 0, 0];
          }, 25802: function(G) {
            function J(Z, te) {
              var ne, ee, ae = [0, 0, 0, 0];
              for (ne = 0; ne < 4; ++ne) for (ee = 0; ee < 4; ++ee) ae[ee] += Z[4 * ne + ee] * te[ne];
              return ae;
            }
            G.exports = function(Z, te) {
              return J(Z.projection, J(Z.view, J(Z.model, [te[0], te[1], te[2], 1])));
            };
          }, 20299: function(G, J, Z) {
            var te, ne, ee = Z(99098).gl_plot3d, ae = ee.createCamera, oe = ee.createScene, se = Z(22248), ce = Z(74043), le = Z(33626), ue = Z(34809), de = ue.preserveDrawingBuffer(), fe = Z(29714), he = Z(32141), me = Z(55010), pe = Z(97464), ye = Z(25802), ge = Z(95701), ve = Z(64087), be = Z(32412), Ce = Z(32919).applyAutorangeOptions, Se = !1;
            function we(ke, Me) {
              var Re = document.createElement("div"), De = ke.container;
              this.graphDiv = ke.graphDiv;
              var Pe = document.createElementNS("http://www.w3.org/2000/svg", "svg");
              Pe.style.position = "absolute", Pe.style.top = Pe.style.left = "0px", Pe.style.width = Pe.style.height = "100%", Pe.style["z-index"] = 20, Pe.style["pointer-events"] = "none", Re.appendChild(Pe), this.svgContainer = Pe, Re.id = ke.id, Re.style.position = "absolute", Re.style.top = Re.style.left = "0px", Re.style.width = Re.style.height = "100%", De.appendChild(Re), this.fullLayout = Me, this.id = ke.id || "scene", this.fullSceneLayout = Me[this.id], this.plotArgs = [[], {}, {}], this.axesOptions = ge(Me, Me[this.id]), this.spikeOptions = ve(Me[this.id]), this.container = Re, this.staticMode = !!ke.staticPlot, this.pixelRatio = this.pixelRatio || ke.plotGlPixelRatio || 2, this.dataScale = [1, 1, 1], this.contourLevels = [[], [], []], this.convertAnnotations = le.getComponentMethod("annotations3d", "convert"), this.drawAnnotations = le.getComponentMethod("annotations3d", "draw"), this.initializeGLPlot();
            }
            var Ee = we.prototype;
            Ee.prepareOptions = function() {
              var ke = this, Me = { canvas: ke.canvas, gl: ke.gl, glOptions: { preserveDrawingBuffer: de, premultipliedAlpha: !0, antialias: !0 }, container: ke.container, axes: ke.axesOptions, spikes: ke.spikeOptions, pickRadius: 10, snapToData: !0, autoScale: !0, autoBounds: !1, cameraObject: ke.camera, pixelRatio: ke.pixelRatio };
              if (ke.staticMode) {
                if (!(ne || (te = document.createElement("canvas"), ne = se({ canvas: te, preserveDrawingBuffer: !0, premultipliedAlpha: !0, antialias: !0 })))) throw new Error("error creating static canvas/context for image server");
                Me.gl = ne, Me.canvas = te;
              }
              return Me;
            };
            var xe = !0;
            Ee.tryCreatePlot = function() {
              var ke = this, Me = ke.prepareOptions(), Re = !0;
              try {
                ke.glplot = oe(Me);
              } catch {
                if (ke.staticMode || !xe || de) Re = !1;
                else {
                  ue.warn(["webgl setup failed possibly due to", "false preserveDrawingBuffer config.", "The mobile/tablet device may not be detected by is-mobile module.", "Enabling preserveDrawingBuffer in second attempt to create webgl scene..."].join(" "));
                  try {
                    de = Me.glOptions.preserveDrawingBuffer = !0, ke.glplot = oe(Me);
                  } catch {
                    de = Me.glOptions.preserveDrawingBuffer = !1, Re = !1;
                  }
                }
              }
              return xe = !1, Re;
            }, Ee.initializeGLCamera = function() {
              var ke = this, Me = ke.fullSceneLayout.camera, Re = Me.projection.type === "orthographic";
              ke.camera = ae(ke.container, { center: [Me.center.x, Me.center.y, Me.center.z], eye: [Me.eye.x, Me.eye.y, Me.eye.z], up: [Me.up.x, Me.up.y, Me.up.z], _ortho: Re, zoomMin: 0.01, zoomMax: 100, mode: "orbit" });
            }, Ee.initializeGLPlot = function() {
              var ke = this;
              if (ke.initializeGLCamera(), !ke.tryCreatePlot()) return pe(ke);
              ke.traces = {}, ke.make4thDimension();
              var Me = ke.graphDiv, Re = Me.layout, De = function() {
                var $e = {};
                return ke.isCameraChanged(Re) && ($e[ke.id + ".camera"] = ke.getCamera()), ke.isAspectChanged(Re) && ($e[ke.id + ".aspectratio"] = ke.glplot.getAspectratio(), Re[ke.id].aspectmode !== "manual" && (ke.fullSceneLayout.aspectmode = Re[ke.id].aspectmode = $e[ke.id + ".aspectmode"] = "manual")), $e;
              }, Pe = function($e) {
                if ($e.fullSceneLayout.dragmode !== !1) {
                  var qe = De();
                  $e.saveLayout(Re), $e.graphDiv.emit("plotly_relayout", qe);
                }
              };
              return ke.glplot.canvas && (ke.glplot.canvas.addEventListener("mouseup", function() {
                Pe(ke);
              }), ke.glplot.canvas.addEventListener("touchstart", function() {
                Se = !0;
              }), ke.glplot.canvas.addEventListener("wheel", function($e) {
                if (Me._context._scrollZoom.gl3d) {
                  if (ke.camera._ortho) {
                    var qe = $e.deltaX > $e.deltaY ? 1.1 : 0.9090909090909091, Xe = ke.glplot.getAspectratio();
                    ke.glplot.setAspectratio({ x: qe * Xe.x, y: qe * Xe.y, z: qe * Xe.z });
                  }
                  Pe(ke);
                }
              }, !!ce && { passive: !1 }), ke.glplot.canvas.addEventListener("mousemove", function() {
                if (ke.fullSceneLayout.dragmode !== !1 && ke.camera.mouseListener.buttons !== 0) {
                  var $e = De();
                  ke.graphDiv.emit("plotly_relayouting", $e);
                }
              }), ke.staticMode || ke.glplot.canvas.addEventListener("webglcontextlost", function($e) {
                Me && Me.emit && Me.emit("plotly_webglcontextlost", { event: $e, layer: ke.id });
              }, !1)), ke.glplot.oncontextloss = function() {
                ke.recoverContext();
              }, ke.glplot.onrender = function() {
                ke.render();
              }, !0;
            }, Ee.render = function() {
              var ke, Me = this, Re = Me.graphDiv, De = Me.svgContainer, Pe = Me.container.getBoundingClientRect();
              Re._fullLayout._calcInverseTransform(Re);
              var $e = Re._fullLayout._invScaleX, qe = Re._fullLayout._invScaleY, Xe = Pe.width * $e, Ve = Pe.height * qe;
              De.setAttributeNS(null, "viewBox", "0 0 " + Xe + " " + Ve), De.setAttributeNS(null, "width", Xe), De.setAttributeNS(null, "height", Ve), be(Me), Me.glplot.axes.update(Me.axesOptions);
              for (var Ze = Object.keys(Me.traces), Ge = null, ze = Me.glplot.selection, We = 0; We < Ze.length; ++We) (ke = Me.traces[Ze[We]]).data.hoverinfo !== "skip" && ke.handlePick(ze) && (Ge = ke), ke.setContourLevels && ke.setContourLevels();
              function at(vt, $t, Pt) {
                var Xt = Me.fullSceneLayout[vt + "axis"];
                return Xt.type !== "log" && ($t = Xt.d2l($t)), fe.hoverLabelText(Xt, $t, Pt);
              }
              if (Ge !== null) {
                var ct = ye(Me.glplot.cameraParams, ze.dataCoordinate);
                ke = Ge.data;
                var et, dt = Re._fullData[ke.index], Je = ze.index, it = { xLabel: at("x", ze.traceCoordinate[0], ke.xhoverformat), yLabel: at("y", ze.traceCoordinate[1], ke.yhoverformat), zLabel: at("z", ze.traceCoordinate[2], ke.zhoverformat) }, st = he.castHoverinfo(dt, Me.fullLayout, Je), gt = (st || "").split("+"), bt = st && st === "all";
                dt.hovertemplate || bt || (gt.indexOf("x") === -1 && (it.xLabel = void 0), gt.indexOf("y") === -1 && (it.yLabel = void 0), gt.indexOf("z") === -1 && (it.zLabel = void 0), gt.indexOf("text") === -1 && (ze.textLabel = void 0), gt.indexOf("name") === -1 && (Ge.name = void 0));
                var yt = [];
                ke.type === "cone" || ke.type === "streamtube" ? (it.uLabel = at("x", ze.traceCoordinate[3], ke.uhoverformat), (bt || gt.indexOf("u") !== -1) && yt.push("u: " + it.uLabel), it.vLabel = at("y", ze.traceCoordinate[4], ke.vhoverformat), (bt || gt.indexOf("v") !== -1) && yt.push("v: " + it.vLabel), it.wLabel = at("z", ze.traceCoordinate[5], ke.whoverformat), (bt || gt.indexOf("w") !== -1) && yt.push("w: " + it.wLabel), it.normLabel = ze.traceCoordinate[6].toPrecision(3), (bt || gt.indexOf("norm") !== -1) && yt.push("norm: " + it.normLabel), ke.type === "streamtube" && (it.divergenceLabel = ze.traceCoordinate[7].toPrecision(3), (bt || gt.indexOf("divergence") !== -1) && yt.push("divergence: " + it.divergenceLabel)), ze.textLabel && yt.push(ze.textLabel), et = yt.join("<br>")) : ke.type === "isosurface" || ke.type === "volume" ? (it.valueLabel = fe.hoverLabelText(Me._mockAxis, Me._mockAxis.d2l(ze.traceCoordinate[3]), ke.valuehoverformat), yt.push("value: " + it.valueLabel), ze.textLabel && yt.push(ze.textLabel), et = yt.join("<br>")) : et = ze.textLabel;
                var St = { x: ze.traceCoordinate[0], y: ze.traceCoordinate[1], z: ze.traceCoordinate[2], data: dt._input, fullData: dt, curveNumber: dt.index, pointNumber: Je };
                he.appendArrayPointValue(St, dt, Je), ke._module.eventData && (St = dt._module.eventData(St, ze, dt, {}, Je));
                var Lt = { points: [St] };
                if (Me.fullSceneLayout.hovermode) {
                  var Rt = [];
                  he.loneHover({ trace: dt, x: (0.5 + 0.5 * ct[0] / ct[3]) * Xe, y: (0.5 - 0.5 * ct[1] / ct[3]) * Ve, xLabel: it.xLabel, yLabel: it.yLabel, zLabel: it.zLabel, text: et, name: Ge.name, color: he.castHoverOption(dt, Je, "bgcolor") || Ge.color, borderColor: he.castHoverOption(dt, Je, "bordercolor"), fontFamily: he.castHoverOption(dt, Je, "font.family"), fontSize: he.castHoverOption(dt, Je, "font.size"), fontColor: he.castHoverOption(dt, Je, "font.color"), nameLength: he.castHoverOption(dt, Je, "namelength"), textAlign: he.castHoverOption(dt, Je, "align"), hovertemplate: ue.castOption(dt, Je, "hovertemplate"), hovertemplateLabels: ue.extendFlat({}, St, it), eventData: [St] }, { container: De, gd: Re, inOut_bbox: Rt }), St.bbox = Rt[0];
                }
                ze.distance < 5 && (ze.buttons || Se) ? Re.emit("plotly_click", Lt) : Re.emit("plotly_hover", Lt), this.oldEventData = Lt;
              } else he.loneUnhover(De), this.oldEventData && Re.emit("plotly_unhover", this.oldEventData), this.oldEventData = void 0;
              Me.drawAnnotations(Me);
            }, Ee.recoverContext = function() {
              var ke = this;
              ke.glplot.dispose();
              var Me = function() {
                ke.glplot.gl.isContextLost() ? requestAnimationFrame(Me) : ke.initializeGLPlot() ? ke.plot.apply(ke, ke.plotArgs) : ue.error("Catastrophic and unrecoverable WebGL error. Context lost.");
              };
              requestAnimationFrame(Me);
            };
            var Le = ["xaxis", "yaxis", "zaxis"];
            function Qe(ke, Me, Re) {
              for (var De = ke.fullSceneLayout, Pe = 0; Pe < 3; Pe++) {
                var $e = Le[Pe], qe = $e.charAt(0), Xe = De[$e], Ve = Me[qe], Ze = Me[qe + "calendar"], Ge = Me["_" + qe + "length"];
                if (ue.isArrayOrTypedArray(Ve)) for (var ze, We = 0; We < (Ge || Ve.length); We++) if (ue.isArrayOrTypedArray(Ve[We])) for (var at = 0; at < Ve[We].length; ++at) ze = Xe.d2l(Ve[We][at], 0, Ze), !isNaN(ze) && isFinite(ze) && (Re[0][Pe] = Math.min(Re[0][Pe], ze), Re[1][Pe] = Math.max(Re[1][Pe], ze));
                else ze = Xe.d2l(Ve[We], 0, Ze), !isNaN(ze) && isFinite(ze) && (Re[0][Pe] = Math.min(Re[0][Pe], ze), Re[1][Pe] = Math.max(Re[1][Pe], ze));
                else Re[0][Pe] = Math.min(Re[0][Pe], 0), Re[1][Pe] = Math.max(Re[1][Pe], Ge - 1);
              }
            }
            Ee.plot = function(ke, Me, Re) {
              var De = this;
              if (De.plotArgs = [ke, Me, Re], !De.glplot.contextLost) {
                var Pe, $e, qe, Xe, Ve, Ze, Ge = Me[De.id], ze = Re[De.id];
                De.fullLayout = Me, De.fullSceneLayout = Ge, De.axesOptions.merge(Me, Ge), De.spikeOptions.merge(Ge), De.setViewport(Ge), De.updateFx(Ge.dragmode, Ge.hovermode), De.camera.enableWheel = De.graphDiv._context._scrollZoom.gl3d, De.glplot.setClearColor(me(Ge.bgcolor)), De.setConvert(Ve), ke ? Array.isArray(ke) || (ke = [ke]) : ke = [];
                var We = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]];
                for (qe = 0; qe < ke.length; ++qe) (Pe = ke[qe]).visible === !0 && Pe._length !== 0 && Qe(this, Pe, We);
                (function(Vt, Yt) {
                  for (var sn = Vt.fullSceneLayout, hn = sn.annotations || [], Qt = 0; Qt < 3; Qt++) for (var Ut = Le[Qt], Wt = Ut.charAt(0), rn = sn[Ut], vn = 0; vn < hn.length; vn++) {
                    var dn = hn[vn];
                    if (dn.visible) {
                      var Ht = rn.r2l(dn[Wt]);
                      !isNaN(Ht) && isFinite(Ht) && (Yt[0][Qt] = Math.min(Yt[0][Qt], Ht), Yt[1][Qt] = Math.max(Yt[1][Qt], Ht));
                    }
                  }
                })(this, We);
                var at = [1, 1, 1];
                for (Xe = 0; Xe < 3; ++Xe) We[1][Xe] === We[0][Xe] ? at[Xe] = 1 : at[Xe] = 1 / (We[1][Xe] - We[0][Xe]);
                for (De.dataScale = at, De.convertAnnotations(this), qe = 0; qe < ke.length; ++qe) (Pe = ke[qe]).visible === !0 && Pe._length !== 0 && (($e = De.traces[Pe.uid]) ? $e.data.type === Pe.type ? $e.update(Pe) : ($e.dispose(), $e = Pe._module.plot(this, Pe), De.traces[Pe.uid] = $e) : ($e = Pe._module.plot(this, Pe), De.traces[Pe.uid] = $e), $e.name = Pe.name);
                var ct = Object.keys(De.traces);
                e: for (qe = 0; qe < ct.length; ++qe) {
                  for (Xe = 0; Xe < ke.length; ++Xe) if (ke[Xe].uid === ct[qe] && ke[Xe].visible === !0 && ke[Xe]._length !== 0) continue e;
                  ($e = De.traces[ct[qe]]).dispose(), delete De.traces[ct[qe]];
                }
                De.glplot.objects.sort(function(Vt, Yt) {
                  return Vt._trace.data.index - Yt._trace.data.index;
                });
                var et, dt = [[0, 0, 0], [0, 0, 0]], Je = [], it = {};
                for (qe = 0; qe < 3; ++qe) {
                  var st;
                  if ((Ze = (Ve = Ge[Le[qe]]).type) in it ? (it[Ze].acc *= at[qe], it[Ze].count += 1) : it[Ze] = { acc: at[qe], count: 1 }, Ve.autorange) {
                    dt[0][qe] = 1 / 0, dt[1][qe] = -1 / 0;
                    var gt = De.glplot.objects, bt = De.fullSceneLayout.annotations || [], yt = Ve._name.charAt(0);
                    for (Xe = 0; Xe < gt.length; Xe++) {
                      var St = gt[Xe], Lt = St.bounds, Rt = St._trace.data._pad || 0;
                      St.constructor.name === "ErrorBars" && Ve._lowerLogErrorBound ? dt[0][qe] = Math.min(dt[0][qe], Ve._lowerLogErrorBound) : dt[0][qe] = Math.min(dt[0][qe], Lt[0][qe] / at[qe] - Rt), dt[1][qe] = Math.max(dt[1][qe], Lt[1][qe] / at[qe] + Rt);
                    }
                    for (Xe = 0; Xe < bt.length; Xe++) {
                      var vt = bt[Xe];
                      if (vt.visible) {
                        var $t = Ve.r2l(vt[yt]);
                        dt[0][qe] = Math.min(dt[0][qe], $t), dt[1][qe] = Math.max(dt[1][qe], $t);
                      }
                    }
                    if ("rangemode" in Ve && Ve.rangemode === "tozero" && (dt[0][qe] = Math.min(dt[0][qe], 0), dt[1][qe] = Math.max(dt[1][qe], 0)), dt[0][qe] > dt[1][qe]) dt[0][qe] = -1, dt[1][qe] = 1;
                    else {
                      var Pt = dt[1][qe] - dt[0][qe];
                      dt[0][qe] -= Pt / 32, dt[1][qe] += Pt / 32;
                    }
                    if (st = [dt[0][qe], dt[1][qe]], st = Ce(st, Ve), dt[0][qe] = st[0], dt[1][qe] = st[1], Ve.isReversed()) {
                      var Xt = dt[0][qe];
                      dt[0][qe] = dt[1][qe], dt[1][qe] = Xt;
                    }
                  } else st = Ve.range, dt[0][qe] = Ve.r2l(st[0]), dt[1][qe] = Ve.r2l(st[1]);
                  dt[0][qe] === dt[1][qe] && (dt[0][qe] -= 1, dt[1][qe] += 1), Je[qe] = dt[1][qe] - dt[0][qe], Ve.range = [dt[0][qe], dt[1][qe]], Ve.limitRange(), De.glplot.setBounds(qe, { min: Ve.range[0] * at[qe], max: Ve.range[1] * at[qe] });
                }
                var Nt = Ge.aspectmode;
                if (Nt === "cube") et = [1, 1, 1];
                else if (Nt === "manual") {
                  var Gt = Ge.aspectratio;
                  et = [Gt.x, Gt.y, Gt.z];
                } else {
                  if (Nt !== "auto" && Nt !== "data") throw new Error("scene.js aspectRatio was not one of the enumerated types");
                  var Ft = [1, 1, 1];
                  for (qe = 0; qe < 3; ++qe) {
                    var Tt = it[Ze = (Ve = Ge[Le[qe]]).type];
                    Ft[qe] = Math.pow(Tt.acc, 1 / Tt.count) / at[qe];
                  }
                  et = Nt === "data" || Math.max.apply(null, Ft) / Math.min.apply(null, Ft) <= 4 ? Ft : [1, 1, 1];
                }
                Ge.aspectratio.x = ze.aspectratio.x = et[0], Ge.aspectratio.y = ze.aspectratio.y = et[1], Ge.aspectratio.z = ze.aspectratio.z = et[2], De.glplot.setAspectratio(Ge.aspectratio), De.viewInitial.aspectratio || (De.viewInitial.aspectratio = { x: Ge.aspectratio.x, y: Ge.aspectratio.y, z: Ge.aspectratio.z }), De.viewInitial.aspectmode || (De.viewInitial.aspectmode = Ge.aspectmode);
                var It = Ge.domain || null, Ct = Me._size || null;
                if (It && Ct) {
                  var Kt = De.container.style;
                  Kt.position = "absolute", Kt.left = Ct.l + It.x[0] * Ct.w + "px", Kt.top = Ct.t + (1 - It.y[1]) * Ct.h + "px", Kt.width = Ct.w * (It.x[1] - It.x[0]) + "px", Kt.height = Ct.h * (It.y[1] - It.y[0]) + "px";
                }
                De.glplot.redraw();
              }
            }, Ee.destroy = function() {
              var ke = this;
              ke.glplot && (ke.camera.mouseListener.enabled = !1, ke.container.removeEventListener("wheel", ke.camera.wheelListener), ke.camera = null, ke.glplot.dispose(), ke.container.parentNode.removeChild(ke.container), ke.glplot = null);
            }, Ee.getCamera = function() {
              var ke, Me = this;
              return Me.camera.view.recalcMatrix(Me.camera.view.lastT()), { up: { x: (ke = Me.camera).up[0], y: ke.up[1], z: ke.up[2] }, center: { x: ke.center[0], y: ke.center[1], z: ke.center[2] }, eye: { x: ke.eye[0], y: ke.eye[1], z: ke.eye[2] }, projection: { type: ke._ortho === !0 ? "orthographic" : "perspective" } };
            }, Ee.setViewport = function(ke) {
              var Me, Re = this, De = ke.camera;
              Re.camera.lookAt.apply(this, [[(Me = De).eye.x, Me.eye.y, Me.eye.z], [Me.center.x, Me.center.y, Me.center.z], [Me.up.x, Me.up.y, Me.up.z]]), Re.glplot.setAspectratio(ke.aspectratio), De.projection.type === "orthographic" !== Re.camera._ortho && (Re.glplot.redraw(), Re.glplot.clearRGBA(), Re.glplot.dispose(), Re.initializeGLPlot());
            }, Ee.isCameraChanged = function(ke) {
              var Me = this.getCamera(), Re = ue.nestedProperty(ke, this.id + ".camera").get();
              function De(Xe, Ve, Ze, Ge) {
                var ze = ["up", "center", "eye"], We = ["x", "y", "z"];
                return Ve[ze[Ze]] && Xe[ze[Ze]][We[Ge]] === Ve[ze[Ze]][We[Ge]];
              }
              var Pe = !1;
              if (Re === void 0) Pe = !0;
              else {
                for (var $e = 0; $e < 3; $e++) for (var qe = 0; qe < 3; qe++) if (!De(Me, Re, $e, qe)) {
                  Pe = !0;
                  break;
                }
                (!Re.projection || Me.projection && Me.projection.type !== Re.projection.type) && (Pe = !0);
              }
              return Pe;
            }, Ee.isAspectChanged = function(ke) {
              var Me = this.glplot.getAspectratio(), Re = ue.nestedProperty(ke, this.id + ".aspectratio").get();
              return Re === void 0 || Re.x !== Me.x || Re.y !== Me.y || Re.z !== Me.z;
            }, Ee.saveLayout = function(ke) {
              var Me, Re, De, Pe, $e, qe, Xe = this, Ve = Xe.fullLayout, Ze = Xe.isCameraChanged(ke), Ge = Xe.isAspectChanged(ke), ze = Ze || Ge;
              if (ze) {
                var We = {};
                Ze && (Me = Xe.getCamera(), De = (Re = ue.nestedProperty(ke, Xe.id + ".camera")).get(), We[Xe.id + ".camera"] = De), Ge && (Pe = Xe.glplot.getAspectratio(), qe = ($e = ue.nestedProperty(ke, Xe.id + ".aspectratio")).get(), We[Xe.id + ".aspectratio"] = qe), le.call("_storeDirectGUIEdit", ke, Ve._preGUI, We), Ze && (Re.set(Me), ue.nestedProperty(Ve, Xe.id + ".camera").set(Me)), Ge && ($e.set(Pe), ue.nestedProperty(Ve, Xe.id + ".aspectratio").set(Pe), Xe.glplot.redraw());
              }
              return ze;
            }, Ee.updateFx = function(ke, Me) {
              var Re = this, De = Re.camera;
              if (De) if (ke === "orbit") De.mode = "orbit", De.keyBindingMode = "rotate";
              else if (ke === "turntable") {
                De.up = [0, 0, 1], De.mode = "turntable", De.keyBindingMode = "rotate";
                var Pe = Re.graphDiv, $e = Pe._fullLayout, qe = Re.fullSceneLayout.camera, Xe = qe.up.x, Ve = qe.up.y, Ze = qe.up.z;
                if (Ze / Math.sqrt(Xe * Xe + Ve * Ve + Ze * Ze) < 0.999) {
                  var Ge = Re.id + ".camera.up", ze = { x: 0, y: 0, z: 1 }, We = {};
                  We[Ge] = ze;
                  var at = Pe.layout;
                  le.call("_storeDirectGUIEdit", at, $e._preGUI, We), qe.up = ze, ue.nestedProperty(at, Ge).set(ze);
                }
              } else De.keyBindingMode = ke;
              Re.fullSceneLayout.hovermode = Me;
            }, Ee.toImage = function(ke) {
              var Me = this;
              ke || (ke = "png"), Me.staticMode && Me.container.appendChild(te), Me.glplot.redraw();
              var Re = Me.glplot.gl, De = Re.drawingBufferWidth, Pe = Re.drawingBufferHeight;
              Re.bindFramebuffer(Re.FRAMEBUFFER, null);
              var $e = new Uint8Array(De * Pe * 4);
              Re.readPixels(0, 0, De, Pe, Re.RGBA, Re.UNSIGNED_BYTE, $e), function(Ge, ze, We) {
                for (var at = 0, ct = We - 1; at < ct; ++at, --ct) for (var et = 0; et < ze; ++et) for (var dt = 0; dt < 4; ++dt) {
                  var Je = 4 * (ze * at + et) + dt, it = 4 * (ze * ct + et) + dt, st = Ge[Je];
                  Ge[Je] = Ge[it], Ge[it] = st;
                }
              }($e, De, Pe), function(Ge, ze, We) {
                for (var at = 0; at < We; ++at) for (var ct = 0; ct < ze; ++ct) {
                  var et = 4 * (ze * at + ct), dt = Ge[et + 3];
                  if (dt > 0) for (var Je = 255 / dt, it = 0; it < 3; ++it) Ge[et + it] = Math.min(Je * Ge[et + it], 255);
                }
              }($e, De, Pe);
              var qe = document.createElement("canvas");
              qe.width = De, qe.height = Pe;
              var Xe, Ve = qe.getContext("2d", { willReadFrequently: !0 }), Ze = Ve.createImageData(De, Pe);
              switch (Ze.data.set($e), Ve.putImageData(Ze, 0, 0), ke) {
                case "jpeg":
                  Xe = qe.toDataURL("image/jpeg");
                  break;
                case "webp":
                  Xe = qe.toDataURL("image/webp");
                  break;
                default:
                  Xe = qe.toDataURL("image/png");
              }
              return Me.staticMode && Me.container.removeChild(te), Xe;
            }, Ee.setConvert = function() {
              for (var ke = 0; ke < 3; ke++) {
                var Me = this.fullSceneLayout[Le[ke]];
                fe.setConvert(Me, this.fullLayout), Me.setScale = ue.noop;
              }
            }, Ee.make4thDimension = function() {
              var ke = this, Me = ke.graphDiv._fullLayout;
              ke._mockAxis = { type: "linear", showexponent: "all", exponentformat: "B" }, fe.setConvert(ke._mockAxis, Me);
            }, G.exports = we;
          }, 88239: function(G) {
            G.exports = function(J, Z, te, ne) {
              ne = ne || J.length;
              for (var ee = new Array(ne), ae = 0; ae < ne; ae++) ee[ae] = [J[ae], Z[ae], te[ae]];
              return ee;
            };
          }, 6704: function(G, J, Z) {
            var te = Z(80337), ne = Z(49722), ee = Z(10229), ae = Z(64101), oe = Z(52307), se = Z(57891), ce = Z(93049).extendFlat, le = te({ editType: "calc" });
            le.family.dflt = '"Open Sans", verdana, arial, sans-serif', le.size.dflt = 12, le.color.dflt = ee.defaultLine, G.exports = { font: le, title: { text: { valType: "string", editType: "layoutstyle" }, font: te({ editType: "layoutstyle" }), subtitle: { text: { valType: "string", editType: "layoutstyle" }, font: te({ editType: "layoutstyle" }), editType: "layoutstyle" }, xref: { valType: "enumerated", dflt: "container", values: ["container", "paper"], editType: "layoutstyle" }, yref: { valType: "enumerated", dflt: "container", values: ["container", "paper"], editType: "layoutstyle" }, x: { valType: "number", min: 0, max: 1, dflt: 0.5, editType: "layoutstyle" }, y: { valType: "number", min: 0, max: 1, dflt: "auto", editType: "layoutstyle" }, xanchor: { valType: "enumerated", dflt: "auto", values: ["auto", "left", "center", "right"], editType: "layoutstyle" }, yanchor: { valType: "enumerated", dflt: "auto", values: ["auto", "top", "middle", "bottom"], editType: "layoutstyle" }, pad: ce(se({ editType: "layoutstyle" }), {}), automargin: { valType: "boolean", dflt: !1, editType: "plot" }, editType: "layoutstyle" }, uniformtext: { mode: { valType: "enumerated", values: [!1, "hide", "show"], dflt: !1, editType: "plot" }, minsize: { valType: "number", min: 0, dflt: 0, editType: "plot" }, editType: "plot" }, autosize: { valType: "boolean", dflt: !1, editType: "none" }, width: { valType: "number", min: 10, dflt: 700, editType: "plot" }, height: { valType: "number", min: 10, dflt: 450, editType: "plot" }, minreducedwidth: { valType: "number", min: 2, dflt: 64, editType: "plot" }, minreducedheight: { valType: "number", min: 2, dflt: 64, editType: "plot" }, margin: { l: { valType: "number", min: 0, dflt: 80, editType: "plot" }, r: { valType: "number", min: 0, dflt: 80, editType: "plot" }, t: { valType: "number", min: 0, dflt: 100, editType: "plot" }, b: { valType: "number", min: 0, dflt: 80, editType: "plot" }, pad: { valType: "number", min: 0, dflt: 0, editType: "plot" }, autoexpand: { valType: "boolean", dflt: !0, editType: "plot" }, editType: "plot" }, computed: { valType: "any", editType: "none" }, paper_bgcolor: { valType: "color", dflt: ee.background, editType: "plot" }, plot_bgcolor: { valType: "color", dflt: ee.background, editType: "layoutstyle" }, autotypenumbers: { valType: "enumerated", values: ["convert types", "strict"], dflt: "convert types", editType: "calc" }, separators: { valType: "string", editType: "plot" }, hidesources: { valType: "boolean", dflt: !1, editType: "plot" }, showlegend: { valType: "boolean", editType: "legend" }, colorway: { valType: "colorlist", dflt: ee.defaults, editType: "calc" }, datarevision: { valType: "any", editType: "calc" }, uirevision: { valType: "any", editType: "none" }, editrevision: { valType: "any", editType: "none" }, selectionrevision: { valType: "any", editType: "none" }, template: { valType: "any", editType: "calc" }, newshape: ae.newshape, activeshape: ae.activeshape, newselection: oe.newselection, activeselection: oe.activeselection, meta: { valType: "any", arrayOk: !0, editType: "plot" }, transition: ce({}, ne.transition, { editType: "none" }), _deprecated: { title: { valType: "string", editType: "layoutstyle" }, titlefont: te({ editType: "layoutstyle" }) } };
          }, 8814: function(G, J, Z) {
            var te = Z(62994), ne = Z(37071), ee = "https://basemaps.cartocdn.com/gl/positron-gl-style/style.json", ae = "https://basemaps.cartocdn.com/gl/dark-matter-gl-style/style.json", oe = "https://basemaps.cartocdn.com/gl/voyager-gl-style/style.json", se = { basic: oe, streets: oe, outdoors: oe, light: ee, dark: ae, satellite: Z(51962), "satellite-streets": ne, "open-street-map": { id: "osm", version: 8, sources: { "plotly-osm-tiles": { type: "raster", attribution: '© <a target="_blank" href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors', tiles: ["https://tile.openstreetmap.org/{z}/{x}/{y}.png"], tileSize: 256 } }, layers: [{ id: "plotly-osm-tiles", type: "raster", source: "plotly-osm-tiles", minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "white-bg": { id: "white-bg", version: 8, sources: {}, layers: [{ id: "white-bg", type: "background", paint: { "background-color": "#FFFFFF" }, minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "carto-positron": ee, "carto-darkmatter": ae, "carto-voyager": oe, "carto-positron-nolabels": "https://basemaps.cartocdn.com/gl/positron-nolabels-gl-style/style.json", "carto-darkmatter-nolabels": "https://basemaps.cartocdn.com/gl/dark-matter-nolabels-gl-style/style.json", "carto-voyager-nolabels": "https://basemaps.cartocdn.com/gl/voyager-nolabels-gl-style/style.json" }, ce = te(se);
            G.exports = { styleValueDflt: "basic", stylesMap: se, styleValuesMap: ce, traceLayerPrefix: "plotly-trace-layer-", layoutLayerPrefix: "plotly-layout-layer-", missingStyleErrorMsg: ["No valid maplibre style found, please set `map.style` to one of:", ce.join(", "), "or use a tile service."].join(`
`), mapOnErrorMsg: "Map error." };
          }, 4657: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              var ae = ne.split(" "), oe = ae[0], se = ae[1], ce = te.isArrayOrTypedArray(ee) ? te.mean(ee) : ee, le = 0.5 + ce / 100, ue = 1.5 + ce / 100, de = ["", ""], fe = [0, 0];
              switch (oe) {
                case "top":
                  de[0] = "top", fe[1] = -ue;
                  break;
                case "bottom":
                  de[0] = "bottom", fe[1] = ue;
              }
              switch (se) {
                case "left":
                  de[1] = "right", fe[0] = -le;
                  break;
                case "right":
                  de[1] = "left", fe[0] = le;
              }
              return { anchor: de[0] && de[1] ? de.join("-") : de[0] ? de[0] : de[1] ? de[1] : "center", offset: fe };
            };
          }, 34091: function(G, J, Z) {
            var te = Z(34809), ne = te.strTranslate, ee = te.strScale, ae = Z(4173).fX, oe = Z(62972), se = Z(45568), ce = Z(62203), le = Z(30635), ue = Z(38793), de = "map";
            J.name = de, J.attr = "subplot", J.idRoot = de, J.idRegex = J.attrRegex = te.counterRegex(de), J.attributes = { subplot: { valType: "subplotid", dflt: "map", editType: "calc" } }, J.layoutAttributes = Z(8257), J.supplyLayoutDefaults = Z(97446), J.plot = function(fe) {
              for (var he = fe._fullLayout, me = fe.calcdata, pe = he._subplots[de], ye = 0; ye < pe.length; ye++) {
                var ge = pe[ye], ve = ae(me, de, ge), be = he[ge], Ce = be._subplot;
                Ce || (Ce = new ue(fe, ge), he[ge]._subplot = Ce), Ce.viewInitial || (Ce.viewInitial = { center: te.extendFlat({}, be.center), zoom: be.zoom, bearing: be.bearing, pitch: be.pitch }), Ce.plot(ve, he, fe._promises);
              }
            }, J.clean = function(fe, he, me, pe) {
              for (var ye = pe._subplots[de] || [], ge = 0; ge < ye.length; ge++) {
                var ve = ye[ge];
                !he[ve] && pe[ve]._subplot && pe[ve]._subplot.destroy();
              }
            }, J.toSVG = function(fe) {
              for (var he = fe._fullLayout, me = he._subplots[de], pe = he._size, ye = 0; ye < me.length; ye++) {
                var ge = he[me[ye]], ve = ge.domain, be = ge._subplot.toImage("png");
                he._glimages.append("svg:image").attr({ xmlns: oe.svg, "xlink:href": be, x: pe.l + pe.w * ve.x[0], y: pe.t + pe.h * (1 - ve.y[1]), width: pe.w * (ve.x[1] - ve.x[0]), height: pe.h * (ve.y[1] - ve.y[0]), preserveAspectRatio: "none" });
                var Ce = se.select(ge._subplot.div).select(".maplibregl-ctrl-attrib").text().replace("Improve this map", ""), Se = he._glimages.append("g"), we = Se.append("text");
                we.text(Ce).classed("static-attribution", !0).attr({ "font-size": 12, "font-family": "Arial", color: "rgba(0, 0, 0, 0.75)", "text-anchor": "end", "data-unformatted": Ce });
                var Ee = ce.bBox(we.node()), xe = pe.w * (ve.x[1] - ve.x[0]);
                if (Ee.width > xe / 2) {
                  var Le = Ce.split("|").join("<br>");
                  we.text(Le).attr("data-unformatted", Le).call(le.convertToTspans, fe), Ee = ce.bBox(we.node());
                }
                we.attr("transform", ne(-3, 8 - Ee.height)), Se.insert("rect", ".static-attribution").attr({ x: -Ee.width - 6, y: -Ee.height - 3, width: Ee.width + 6, height: Ee.height + 3, fill: "rgba(255, 255, 255, 0.75)" });
                var Qe = 1;
                Ee.width + 6 > xe && (Qe = xe / (Ee.width + 6));
                var ke = [pe.l + pe.w * ve.x[1], pe.t + pe.h * (1 - ve.y[0])];
                Se.attr("transform", ne(ke[0], ke[1]) + ee(Qe));
              }
            }, J.updateFx = function(fe) {
              for (var he = fe._fullLayout, me = he._subplots[de], pe = 0; pe < me.length; pe++) he[me[pe]]._subplot.updateFx(he);
            };
          }, 33389: function(G, J, Z) {
            var te = Z(34809), ne = Z(30635).sanitizeHTML, ee = Z(4657), ae = Z(8814);
            function oe(ue, de) {
              this.subplot = ue, this.uid = ue.uid + "-" + de, this.index = de, this.idSource = "source-" + this.uid, this.idLayer = ae.layoutLayerPrefix + this.uid, this.sourceType = null, this.source = null, this.layerType = null, this.below = null, this.visible = !1;
            }
            var se = oe.prototype;
            function ce(ue) {
              if (!ue.visible) return !1;
              var de = ue.source;
              if (Array.isArray(de) && de.length > 0) {
                for (var fe = 0; fe < de.length; fe++) if (typeof de[fe] != "string" || de[fe].length === 0) return !1;
                return !0;
              }
              return te.isPlainObject(de) || typeof de == "string" && de.length > 0;
            }
            function le(ue) {
              var de = {}, fe = {};
              switch (ue.type) {
                case "circle":
                  te.extendFlat(fe, { "circle-radius": ue.circle.radius, "circle-color": ue.color, "circle-opacity": ue.opacity });
                  break;
                case "line":
                  te.extendFlat(fe, { "line-width": ue.line.width, "line-color": ue.color, "line-opacity": ue.opacity, "line-dasharray": ue.line.dash });
                  break;
                case "fill":
                  te.extendFlat(fe, { "fill-color": ue.color, "fill-outline-color": ue.fill.outlinecolor, "fill-opacity": ue.opacity });
                  break;
                case "symbol":
                  var he = ue.symbol, me = ee(he.textposition, he.iconsize);
                  te.extendFlat(de, { "icon-image": he.icon + "-15", "icon-size": he.iconsize / 10, "text-field": he.text, "text-size": he.textfont.size, "text-anchor": me.anchor, "text-offset": me.offset, "symbol-placement": he.placement }), te.extendFlat(fe, { "icon-color": ue.color, "text-color": he.textfont.color, "text-opacity": ue.opacity });
                  break;
                case "raster":
                  te.extendFlat(fe, { "raster-fade-duration": 0, "raster-opacity": ue.opacity });
              }
              return { layout: de, paint: fe };
            }
            se.update = function(ue) {
              this.visible ? this.needsNewImage(ue) ? this.updateImage(ue) : this.needsNewSource(ue) ? (this.removeLayer(), this.updateSource(ue), this.updateLayer(ue)) : this.needsNewLayer(ue) ? this.updateLayer(ue) : this.updateStyle(ue) : (this.updateSource(ue), this.updateLayer(ue)), this.visible = ce(ue);
            }, se.needsNewImage = function(ue) {
              return this.subplot.map.getSource(this.idSource) && this.sourceType === "image" && ue.sourcetype === "image" && (this.source !== ue.source || JSON.stringify(this.coordinates) !== JSON.stringify(ue.coordinates));
            }, se.needsNewSource = function(ue) {
              return this.sourceType !== ue.sourcetype || JSON.stringify(this.source) !== JSON.stringify(ue.source) || this.layerType !== ue.type;
            }, se.needsNewLayer = function(ue) {
              return this.layerType !== ue.type || this.below !== this.subplot.belowLookup["layout-" + this.index];
            }, se.lookupBelow = function() {
              return this.subplot.belowLookup["layout-" + this.index];
            }, se.updateImage = function(ue) {
              this.subplot.map.getSource(this.idSource).updateImage({ url: ue.source, coordinates: ue.coordinates });
              var de = this.findFollowingMapLayerId(this.lookupBelow());
              de !== null && this.subplot.map.moveLayer(this.idLayer, de);
            }, se.updateSource = function(ue) {
              var de = this.subplot.map;
              if (de.getSource(this.idSource) && de.removeSource(this.idSource), this.sourceType = ue.sourcetype, this.source = ue.source, ce(ue)) {
                var fe = function(he) {
                  var me, pe = he.sourcetype, ye = he.source, ge = { type: pe };
                  return pe === "geojson" ? me = "data" : pe === "vector" ? me = typeof ye == "string" ? "url" : "tiles" : pe === "raster" ? (me = "tiles", ge.tileSize = 256) : pe === "image" && (me = "url", ge.coordinates = he.coordinates), ge[me] = ye, he.sourceattribution && (ge.attribution = ne(he.sourceattribution)), ge;
                }(ue);
                de.addSource(this.idSource, fe);
              }
            }, se.findFollowingMapLayerId = function(ue) {
              if (ue === "traces") for (var de = this.subplot.getMapLayers(), fe = 0; fe < de.length; fe++) {
                var he = de[fe].id;
                if (typeof he == "string" && he.indexOf(ae.traceLayerPrefix) === 0) {
                  ue = he;
                  break;
                }
              }
              return ue;
            }, se.updateLayer = function(ue) {
              var de = this.subplot, fe = le(ue), he = this.lookupBelow(), me = this.findFollowingMapLayerId(he);
              this.removeLayer(), ce(ue) && de.addLayer({ id: this.idLayer, source: this.idSource, "source-layer": ue.sourcelayer || "", type: ue.type, minzoom: ue.minzoom, maxzoom: ue.maxzoom, layout: fe.layout, paint: fe.paint }, me), this.layerType = ue.type, this.below = he;
            }, se.updateStyle = function(ue) {
              if (ce(ue)) {
                var de = le(ue);
                this.subplot.setOptions(this.idLayer, "setLayoutProperty", de.layout), this.subplot.setOptions(this.idLayer, "setPaintProperty", de.paint);
              }
            }, se.removeLayer = function() {
              var ue = this.subplot.map;
              ue.getLayer(this.idLayer) && ue.removeLayer(this.idLayer);
            }, se.dispose = function() {
              var ue = this.subplot.map;
              ue.getLayer(this.idLayer) && ue.removeLayer(this.idLayer), ue.getSource(this.idSource) && ue.removeSource(this.idSource);
            }, G.exports = function(ue, de, fe) {
              var he = new oe(ue, de);
              return he.update(fe), he;
            };
          }, 8257: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766).defaultLine, ee = Z(13792).u, ae = Z(80337), oe = Z(36640).textposition, se = Z(13582).overrideAll, ce = Z(78032).templatedArray, le = Z(8814), ue = ae({ noFontVariant: !0, noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0 });
            ue.family.dflt = "Open Sans Regular, Arial Unicode MS Regular", (G.exports = se({ _arrayAttrRegexps: [te.counterRegex("map", ".layers", !0)], domain: ee({ name: "map" }), style: { valType: "any", values: le.styleValuesMap, dflt: le.styleValueDflt }, center: { lon: { valType: "number", dflt: 0 }, lat: { valType: "number", dflt: 0 } }, zoom: { valType: "number", dflt: 1 }, bearing: { valType: "number", dflt: 0 }, pitch: { valType: "number", dflt: 0 }, bounds: { west: { valType: "number" }, east: { valType: "number" }, south: { valType: "number" }, north: { valType: "number" } }, layers: ce("layer", { visible: { valType: "boolean", dflt: !0 }, sourcetype: { valType: "enumerated", values: ["geojson", "vector", "raster", "image"], dflt: "geojson" }, source: { valType: "any" }, sourcelayer: { valType: "string", dflt: "" }, sourceattribution: { valType: "string" }, type: { valType: "enumerated", values: ["circle", "line", "fill", "symbol", "raster"], dflt: "circle" }, coordinates: { valType: "any" }, below: { valType: "string" }, color: { valType: "color", dflt: ne }, opacity: { valType: "number", min: 0, max: 1, dflt: 1 }, minzoom: { valType: "number", min: 0, max: 24, dflt: 0 }, maxzoom: { valType: "number", min: 0, max: 24, dflt: 24 }, circle: { radius: { valType: "number", dflt: 15 } }, line: { width: { valType: "number", dflt: 2 }, dash: { valType: "data_array" } }, fill: { outlinecolor: { valType: "color", dflt: ne } }, symbol: { icon: { valType: "string", dflt: "marker" }, iconsize: { valType: "number", dflt: 10 }, text: { valType: "string", dflt: "" }, placement: { valType: "enumerated", values: ["point", "line", "line-center"], dflt: "point" }, textfont: ue, textposition: te.extendFlat({}, oe, { arrayOk: !1 }) } }) }, "plot", "from-root")).uirevision = { valType: "any", editType: "none" };
          }, 97446: function(G, J, Z) {
            var te = Z(34809), ne = Z(4448), ee = Z(59008), ae = Z(8257);
            function oe(ce, le, ue) {
              ue("style"), ue("center.lon"), ue("center.lat"), ue("zoom"), ue("bearing"), ue("pitch");
              var de = ue("bounds.west"), fe = ue("bounds.east"), he = ue("bounds.south"), me = ue("bounds.north");
              de !== void 0 && fe !== void 0 && he !== void 0 && me !== void 0 || delete le.bounds, ee(ce, le, { name: "layers", handleItemDefaults: se }), le._input = ce;
            }
            function se(ce, le) {
              function ue(pe, ye) {
                return te.coerce(ce, le, ae.layers, pe, ye);
              }
              if (ue("visible")) {
                var de, fe = ue("sourcetype"), he = fe === "raster" || fe === "image";
                ue("source"), ue("sourceattribution"), fe === "vector" && ue("sourcelayer"), fe === "image" && ue("coordinates"), he && (de = "raster");
                var me = ue("type", de);
                he && me !== "raster" && (me = le.type = "raster", te.log("Source types *raster* and *image* must drawn *raster* layer type.")), ue("below"), ue("color"), ue("opacity"), ue("minzoom"), ue("maxzoom"), me === "circle" && ue("circle.radius"), me === "line" && (ue("line.width"), ue("line.dash")), me === "fill" && ue("fill.outlinecolor"), me === "symbol" && (ue("symbol.icon"), ue("symbol.iconsize"), ue("symbol.text"), te.coerceFont(ue, "symbol.textfont", void 0, { noFontVariant: !0, noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0 }), ue("symbol.textposition"), ue("symbol.placement"));
              }
            }
            G.exports = function(ce, le, ue) {
              ne(ce, le, ue, { type: "map", attributes: ae, handleDefaults: oe, partition: "y" });
            };
          }, 38793: function(G, J, Z) {
            var te = Z(89380), ne = Z(34809), ee = Z(3994), ae = Z(33626), oe = Z(29714), se = Z(14751), ce = Z(32141), le = Z(70414), ue = le.drawMode, de = le.selectMode, fe = Z(44844).prepSelect, he = Z(44844).clearOutline, me = Z(44844).clearSelectionsCache, pe = Z(44844).selectOnClick, ye = Z(8814), ge = Z(33389);
            function ve(Ee, xe) {
              this.id = xe, this.gd = Ee;
              var Le = Ee._fullLayout, Qe = Ee._context;
              this.container = Le._glcontainer.node(), this.isStatic = Qe.staticPlot, this.uid = Le._uid + "-" + this.id, this.div = null, this.xaxis = null, this.yaxis = null, this.createFramework(Le), this.map = null, this.styleObj = null, this.traceHash = {}, this.layerList = [], this.belowLookup = {}, this.dragging = !1, this.wheeling = !1;
            }
            var be = ve.prototype;
            be.plot = function(Ee, xe, Le) {
              var Qe, ke = this;
              Qe = ke.map ? new Promise(function(Me, Re) {
                ke.updateMap(Ee, xe, Me, Re);
              }) : new Promise(function(Me, Re) {
                ke.createMap(Ee, xe, Me, Re);
              }), Le.push(Qe);
            }, be.createMap = function(Ee, xe, Le, Qe) {
              var ke = this, Me = xe[ke.id], Re = ke.styleObj = Se(Me.style), De = Me.bounds, Pe = De ? [[De.west, De.south], [De.east, De.north]] : null, $e = ke.map = new te.Map({ container: ke.div, style: Re.style, center: we(Me.center), zoom: Me.zoom, bearing: Me.bearing, pitch: Me.pitch, maxBounds: Pe, interactive: !ke.isStatic, preserveDrawingBuffer: ke.isStatic, doubleClickZoom: !1, boxZoom: !1, attributionControl: !1 }).addControl(new te.AttributionControl({ compact: !0 })), qe = {};
              $e.on("styleimagemissing", function(Ve) {
                var Ze = Ve.id;
                if (!qe[Ze] && Ze.includes("-15")) {
                  qe[Ze] = !0;
                  var Ge = new Image(15, 15);
                  Ge.onload = function() {
                    $e.addImage(Ze, Ge);
                  }, Ge.crossOrigin = "Anonymous", Ge.src = "https://unpkg.com/maki@2.1.0/icons/" + Ze + ".svg";
                }
              }), $e.setTransformRequest(function(Ve) {
                return { url: Ve = (Ve = (Ve = Ve.replace("https://fonts.openmaptiles.org/Open Sans Extrabold", "https://fonts.openmaptiles.org/Open Sans Extra Bold")).replace("https://tiles.basemaps.cartocdn.com/fonts/Open Sans Extrabold", "https://fonts.openmaptiles.org/Open Sans Extra Bold")).replace("https://fonts.openmaptiles.org/Open Sans Regular,Arial Unicode MS Regular", "https://fonts.openmaptiles.org/Klokantech Noto Sans Regular") };
              }), $e._canvas.style.left = "0px", $e._canvas.style.top = "0px", ke.rejectOnError(Qe), ke.isStatic || ke.initFx(Ee, xe);
              var Xe = [];
              Xe.push(new Promise(function(Ve) {
                $e.once("load", Ve);
              })), Xe = Xe.concat(ee.fetchTraceGeoData(Ee)), Promise.all(Xe).then(function() {
                ke.fillBelowLookup(Ee, xe), ke.updateData(Ee), ke.updateLayout(xe), ke.resolveOnRender(Le);
              }).catch(Qe);
            }, be.updateMap = function(Ee, xe, Le, Qe) {
              var ke = this, Me = ke.map, Re = xe[this.id];
              ke.rejectOnError(Qe);
              var De = [], Pe = Se(Re.style);
              JSON.stringify(ke.styleObj) !== JSON.stringify(Pe) && (ke.styleObj = Pe, Me.setStyle(Pe.style), ke.traceHash = {}, De.push(new Promise(function($e) {
                Me.once("styledata", $e);
              }))), De = De.concat(ee.fetchTraceGeoData(Ee)), Promise.all(De).then(function() {
                ke.fillBelowLookup(Ee, xe), ke.updateData(Ee), ke.updateLayout(xe), ke.resolveOnRender(Le);
              }).catch(Qe);
            }, be.fillBelowLookup = function(Ee, xe) {
              var Le, Qe, ke = xe[this.id].layers, Me = this.belowLookup = {}, Re = !1;
              for (Le = 0; Le < Ee.length; Le++) {
                var De = Ee[Le][0].trace, Pe = De._module;
                typeof De.below == "string" ? Qe = De.below : Pe.getBelow && (Qe = Pe.getBelow(De, this)), Qe === "" && (Re = !0), Me["trace-" + De.uid] = Qe || "";
              }
              for (Le = 0; Le < ke.length; Le++) {
                var $e = ke[Le];
                Qe = typeof $e.below == "string" ? $e.below : Re ? "traces" : "", Me["layout-" + Le] = Qe;
              }
              var qe, Xe, Ve = {};
              for (qe in Me) Ve[Qe = Me[qe]] ? Ve[Qe].push(qe) : Ve[Qe] = [qe];
              for (Qe in Ve) {
                var Ze = Ve[Qe];
                if (Ze.length > 1) for (Le = 0; Le < Ze.length; Le++) (qe = Ze[Le]).indexOf("trace-") === 0 ? (Xe = qe.split("trace-")[1], this.traceHash[Xe] && (this.traceHash[Xe].below = null)) : qe.indexOf("layout-") === 0 && (Xe = qe.split("layout-")[1], this.layerList[Xe] && (this.layerList[Xe].below = null));
              }
            };
            var Ce = { choroplethmap: 0, densitymap: 1, scattermap: 2 };
            function Se(Ee) {
              var xe = {};
              return ne.isPlainObject(Ee) ? (xe.id = Ee.id, xe.style = Ee) : typeof Ee == "string" ? (xe.id = Ee, ye.stylesMap[Ee] ? xe.style = ye.stylesMap[Ee] : xe.style = Ee) : (xe.id = ye.styleValueDflt, xe.style = function(Le) {
                return ye.styleUrlPrefix + Le + "-" + ye.styleUrlSuffix;
              }(ye.styleValueDflt)), xe.transition = { duration: 0, delay: 0 }, xe;
            }
            function we(Ee) {
              return [Ee.lon, Ee.lat];
            }
            be.updateData = function(Ee) {
              var xe, Le, Qe, ke, Me = this.traceHash, Re = Ee.slice().sort(function(Xe, Ve) {
                return Ce[Xe[0].trace.type] - Ce[Ve[0].trace.type];
              });
              for (Qe = 0; Qe < Re.length; Qe++) {
                var De = Re[Qe], Pe = !1;
                (xe = Me[(Le = De[0].trace).uid]) && (xe.type === Le.type ? (xe.update(De), Pe = !0) : xe.dispose()), !Pe && Le._module && (Me[Le.uid] = Le._module.plot(this, De));
              }
              var $e = Object.keys(Me);
              e: for (Qe = 0; Qe < $e.length; Qe++) {
                var qe = $e[Qe];
                for (ke = 0; ke < Ee.length; ke++) if (qe === (Le = Ee[ke][0].trace).uid) continue e;
                (xe = Me[qe]).dispose(), delete Me[qe];
              }
            }, be.updateLayout = function(Ee) {
              var xe = this.map, Le = Ee[this.id];
              this.dragging || this.wheeling || (xe.setCenter(we(Le.center)), xe.setZoom(Le.zoom), xe.setBearing(Le.bearing), xe.setPitch(Le.pitch)), this.updateLayers(Ee), this.updateFramework(Ee), this.updateFx(Ee), this.map.resize(), this.gd._context._scrollZoom.map ? xe.scrollZoom.enable() : xe.scrollZoom.disable();
            }, be.resolveOnRender = function(Ee) {
              var xe = this.map;
              xe.on("render", function Le() {
                xe.loaded() && (xe.off("render", Le), setTimeout(Ee, 10));
              });
            }, be.rejectOnError = function(Ee) {
              var xe = this.map;
              function Le() {
                Ee(new Error(ye.mapOnErrorMsg));
              }
              xe.once("error", Le), xe.once("style.error", Le), xe.once("source.error", Le), xe.once("tile.error", Le), xe.once("layer.error", Le);
            }, be.createFramework = function(Ee) {
              var xe = this, Le = xe.div = document.createElement("div");
              Le.id = xe.uid, Le.style.position = "absolute", xe.container.appendChild(Le), xe.xaxis = { _id: "x", c2p: function(Qe) {
                return xe.project(Qe).x;
              } }, xe.yaxis = { _id: "y", c2p: function(Qe) {
                return xe.project(Qe).y;
              } }, xe.updateFramework(Ee), xe.mockAxis = { type: "linear", showexponent: "all", exponentformat: "B" }, oe.setConvert(xe.mockAxis, Ee);
            }, be.initFx = function(Ee, xe) {
              var Le = this, Qe = Le.gd, ke = Le.map;
              function Me() {
                ce.loneUnhover(xe._hoverlayer);
              }
              function Re() {
                var De = Le.getView();
                Qe.emit("plotly_relayouting", Le.getViewEditsWithDerived(De));
              }
              ke.on("moveend", function(De) {
                if (Le.map) {
                  var Pe = Qe._fullLayout;
                  if (De.originalEvent || Le.wheeling) {
                    var $e = Pe[Le.id];
                    ae.call("_storeDirectGUIEdit", Qe.layout, Pe._preGUI, Le.getViewEdits($e));
                    var qe = Le.getView();
                    $e._input.center = $e.center = qe.center, $e._input.zoom = $e.zoom = qe.zoom, $e._input.bearing = $e.bearing = qe.bearing, $e._input.pitch = $e.pitch = qe.pitch, Qe.emit("plotly_relayout", Le.getViewEditsWithDerived(qe));
                  }
                  De.originalEvent && De.originalEvent.type === "mouseup" ? Le.dragging = !1 : Le.wheeling && (Le.wheeling = !1), Pe && Pe._rehover && Pe._rehover();
                }
              }), ke.on("wheel", function() {
                Le.wheeling = !0;
              }), ke.on("mousemove", function(De) {
                var Pe = Le.div.getBoundingClientRect(), $e = [De.originalEvent.offsetX, De.originalEvent.offsetY];
                De.target.getBoundingClientRect = function() {
                  return Pe;
                }, Le.xaxis.p2c = function() {
                  return ke.unproject($e).lng;
                }, Le.yaxis.p2c = function() {
                  return ke.unproject($e).lat;
                }, Qe._fullLayout._rehover = function() {
                  Qe._fullLayout._hoversubplot === Le.id && Qe._fullLayout[Le.id] && ce.hover(Qe, De, Le.id);
                }, ce.hover(Qe, De, Le.id), Qe._fullLayout._hoversubplot = Le.id;
              }), ke.on("dragstart", function() {
                Le.dragging = !0, Me();
              }), ke.on("zoomstart", Me), ke.on("mouseout", function() {
                Qe._fullLayout._hoversubplot = null;
              }), ke.on("drag", Re), ke.on("zoom", Re), ke.on("dblclick", function() {
                var De = Qe._fullLayout[Le.id];
                ae.call("_storeDirectGUIEdit", Qe.layout, Qe._fullLayout._preGUI, Le.getViewEdits(De));
                var Pe = Le.viewInitial;
                ke.setCenter(we(Pe.center)), ke.setZoom(Pe.zoom), ke.setBearing(Pe.bearing), ke.setPitch(Pe.pitch);
                var $e = Le.getView();
                De._input.center = De.center = $e.center, De._input.zoom = De.zoom = $e.zoom, De._input.bearing = De.bearing = $e.bearing, De._input.pitch = De.pitch = $e.pitch, Qe.emit("plotly_doubleclick", null), Qe.emit("plotly_relayout", Le.getViewEditsWithDerived($e));
              }), Le.clearOutline = function() {
                me(Le.dragOptions), he(Le.dragOptions.gd);
              }, Le.onClickInPanFn = function(De) {
                return function(Pe) {
                  var $e = Qe._fullLayout.clickmode;
                  $e.indexOf("select") > -1 && pe(Pe.originalEvent, Qe, [Le.xaxis], [Le.yaxis], Le.id, De), $e.indexOf("event") > -1 && ce.click(Qe, Pe.originalEvent);
                };
              };
            }, be.updateFx = function(Ee) {
              var xe = this, Le = xe.map, Qe = xe.gd;
              if (!xe.isStatic) {
                var ke, Me = Ee.dragmode;
                ke = function(Pe, $e) {
                  $e.isRect ? (Pe.range = {})[xe.id] = [De([$e.xmin, $e.ymin]), De([$e.xmax, $e.ymax])] : (Pe.lassoPoints = {})[xe.id] = $e.map(De);
                };
                var Re = xe.dragOptions;
                xe.dragOptions = ne.extendDeep(Re || {}, { dragmode: Ee.dragmode, element: xe.div, gd: Qe, plotinfo: { id: xe.id, domain: Ee[xe.id].domain, xaxis: xe.xaxis, yaxis: xe.yaxis, fillRangeItems: ke }, xaxes: [xe.xaxis], yaxes: [xe.yaxis], subplot: xe.id }), Le.off("click", xe.onClickInPanHandler), de(Me) || ue(Me) ? (Le.dragPan.disable(), Le.on("zoomstart", xe.clearOutline), xe.dragOptions.prepFn = function(Pe, $e, qe) {
                  fe(Pe, $e, qe, xe.dragOptions, Me);
                }, se.init(xe.dragOptions)) : (Le.dragPan.enable(), Le.off("zoomstart", xe.clearOutline), xe.div.onmousedown = null, xe.div.ontouchstart = null, xe.div.removeEventListener("touchstart", xe.div._ontouchstart), xe.onClickInPanHandler = xe.onClickInPanFn(xe.dragOptions), Le.on("click", xe.onClickInPanHandler));
              }
              function De(Pe) {
                var $e = xe.map.unproject(Pe);
                return [$e.lng, $e.lat];
              }
            }, be.updateFramework = function(Ee) {
              var xe = Ee[this.id].domain, Le = Ee._size, Qe = this.div.style;
              Qe.width = Le.w * (xe.x[1] - xe.x[0]) + "px", Qe.height = Le.h * (xe.y[1] - xe.y[0]) + "px", Qe.left = Le.l + xe.x[0] * Le.w + "px", Qe.top = Le.t + (1 - xe.y[1]) * Le.h + "px", this.xaxis._offset = Le.l + xe.x[0] * Le.w, this.xaxis._length = Le.w * (xe.x[1] - xe.x[0]), this.yaxis._offset = Le.t + (1 - xe.y[1]) * Le.h, this.yaxis._length = Le.h * (xe.y[1] - xe.y[0]);
            }, be.updateLayers = function(Ee) {
              var xe, Le = Ee[this.id].layers, Qe = this.layerList;
              if (Le.length !== Qe.length) {
                for (xe = 0; xe < Qe.length; xe++) Qe[xe].dispose();
                for (Qe = this.layerList = [], xe = 0; xe < Le.length; xe++) Qe.push(ge(this, xe, Le[xe]));
              } else for (xe = 0; xe < Le.length; xe++) Qe[xe].update(Le[xe]);
            }, be.destroy = function() {
              this.map && (this.map.remove(), this.map = null, this.container.removeChild(this.div));
            }, be.toImage = function() {
              return this.map.stop(), this.map.getCanvas().toDataURL();
            }, be.setOptions = function(Ee, xe, Le) {
              for (var Qe in Le) this.map[xe](Ee, Qe, Le[Qe]);
            }, be.getMapLayers = function() {
              return this.map.getStyle().layers;
            }, be.addLayer = function(Ee, xe) {
              var Le = this.map;
              if (typeof xe == "string") {
                if (xe === "") return void Le.addLayer(Ee, xe);
                for (var Qe = this.getMapLayers(), ke = 0; ke < Qe.length; ke++) if (xe === Qe[ke].id) return void Le.addLayer(Ee, xe);
                ne.warn(["Trying to add layer with *below* value", xe, "referencing a layer that does not exist", "or that does not yet exist."].join(" "));
              }
              Le.addLayer(Ee);
            }, be.project = function(Ee) {
              return this.map.project(new te.LngLat(Ee[0], Ee[1]));
            }, be.getView = function() {
              var Ee = this.map, xe = Ee.getCenter(), Le = { lon: xe.lng, lat: xe.lat }, Qe = Ee.getCanvas(), ke = parseInt(Qe.style.width), Me = parseInt(Qe.style.height);
              return { center: Le, zoom: Ee.getZoom(), bearing: Ee.getBearing(), pitch: Ee.getPitch(), _derived: { coordinates: [Ee.unproject([0, 0]).toArray(), Ee.unproject([ke, 0]).toArray(), Ee.unproject([ke, Me]).toArray(), Ee.unproject([0, Me]).toArray()] } };
            }, be.getViewEdits = function(Ee) {
              for (var xe = this.id, Le = ["center", "zoom", "bearing", "pitch"], Qe = {}, ke = 0; ke < Le.length; ke++) {
                var Me = Le[ke];
                Qe[xe + "." + Me] = Ee[Me];
              }
              return Qe;
            }, be.getViewEditsWithDerived = function(Ee) {
              var xe = this.id, Le = this.getViewEdits(Ee);
              return Le[xe + "._derived"] = Ee._derived, Le;
            }, G.exports = ve;
          }, 44245: function(G, J, Z) {
            var te = Z(62994), ne = "1.13.4", ee = '© <a target="_blank" href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors', ae = ['© <a target="_blank" href="https://carto.com/">Carto</a>', ee].join(" "), oe = ['Map tiles by <a target="_blank" href="https://stamen.com">Stamen Design</a>', 'under <a target="_blank" href="https://creativecommons.org/licenses/by/3.0">CC BY 3.0</a>', "|", 'Data by <a target="_blank" href="https://openstreetmap.org">OpenStreetMap</a> contributors', 'under <a target="_blank" href="https://www.openstreetmap.org/copyright">ODbL</a>'].join(" "), se = { "open-street-map": { id: "osm", version: 8, sources: { "plotly-osm-tiles": { type: "raster", attribution: ee, tiles: ["https://a.tile.openstreetmap.org/{z}/{x}/{y}.png", "https://b.tile.openstreetmap.org/{z}/{x}/{y}.png"], tileSize: 256 } }, layers: [{ id: "plotly-osm-tiles", type: "raster", source: "plotly-osm-tiles", minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "white-bg": { id: "white-bg", version: 8, sources: {}, layers: [{ id: "white-bg", type: "background", paint: { "background-color": "#FFFFFF" }, minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "carto-positron": { id: "carto-positron", version: 8, sources: { "plotly-carto-positron": { type: "raster", attribution: ae, tiles: ["https://cartodb-basemaps-c.global.ssl.fastly.net/light_all/{z}/{x}/{y}.png"], tileSize: 256 } }, layers: [{ id: "plotly-carto-positron", type: "raster", source: "plotly-carto-positron", minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "carto-darkmatter": { id: "carto-darkmatter", version: 8, sources: { "plotly-carto-darkmatter": { type: "raster", attribution: ae, tiles: ["https://cartodb-basemaps-c.global.ssl.fastly.net/dark_all/{z}/{x}/{y}.png"], tileSize: 256 } }, layers: [{ id: "plotly-carto-darkmatter", type: "raster", source: "plotly-carto-darkmatter", minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "stamen-terrain": { id: "stamen-terrain", version: 8, sources: { "plotly-stamen-terrain": { type: "raster", attribution: oe, tiles: ["https://tiles.stadiamaps.com/tiles/stamen_terrain/{z}/{x}/{y}.png?api_key="], tileSize: 256 } }, layers: [{ id: "plotly-stamen-terrain", type: "raster", source: "plotly-stamen-terrain", minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "stamen-toner": { id: "stamen-toner", version: 8, sources: { "plotly-stamen-toner": { type: "raster", attribution: oe, tiles: ["https://tiles.stadiamaps.com/tiles/stamen_toner/{z}/{x}/{y}.png?api_key="], tileSize: 256 } }, layers: [{ id: "plotly-stamen-toner", type: "raster", source: "plotly-stamen-toner", minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" }, "stamen-watercolor": { id: "stamen-watercolor", version: 8, sources: { "plotly-stamen-watercolor": { type: "raster", attribution: ['Map tiles by <a target="_blank" href="https://stamen.com">Stamen Design</a>', 'under <a target="_blank" href="https://creativecommons.org/licenses/by/3.0">CC BY 3.0</a>', "|", 'Data by <a target="_blank" href="https://openstreetmap.org">OpenStreetMap</a> contributors', 'under <a target="_blank" href="https://creativecommons.org/licenses/by-sa/3.0">CC BY SA</a>'].join(" "), tiles: ["https://tiles.stadiamaps.com/tiles/stamen_watercolor/{z}/{x}/{y}.jpg?api_key="], tileSize: 256 } }, layers: [{ id: "plotly-stamen-watercolor", type: "raster", source: "plotly-stamen-watercolor", minzoom: 0, maxzoom: 22 }], glyphs: "https://fonts.openmaptiles.org/{fontstack}/{range}.pbf" } }, ce = te(se);
            G.exports = { requiredVersion: ne, styleUrlPrefix: "mapbox://styles/mapbox/", styleUrlSuffix: "v9", styleValuesMapbox: ["basic", "streets", "outdoors", "light", "dark", "satellite", "satellite-streets"], styleValueDflt: "basic", stylesNonMapbox: se, styleValuesNonMapbox: ce, traceLayerPrefix: "plotly-trace-layer-", layoutLayerPrefix: "plotly-layout-layer-", wrongVersionErrorMsg: ["Your custom plotly.js bundle is not using the correct mapbox-gl version", "Please install @plotly/mapbox-gl@" + ne + "."].join(`
`), noAccessTokenErrorMsg: ["Missing Mapbox access token.", "Mapbox trace type require a Mapbox access token to be registered.", "For example:", "  Plotly.newPlot(gd, data, layout, { mapboxAccessToken: 'my-access-token' });", "More info here: https://www.mapbox.com/help/define-access-token/"].join(`
`), missingStyleErrorMsg: ["No valid mapbox style found, please set `mapbox.style` to one of:", ce.join(", "), "or register a Mapbox access token to use a Mapbox-served style."].join(`
`), multipleTokensErrorMsg: ["Set multiple mapbox access token across different mapbox subplot,", "using first token found as mapbox-gl does not allow multipleaccess tokens on the same page."].join(`
`), mapOnErrorMsg: "Mapbox error.", mapboxLogo: { path0: "m 10.5,1.24 c -5.11,0 -9.25,4.15 -9.25,9.25 0,5.1 4.15,9.25 9.25,9.25 5.1,0 9.25,-4.15 9.25,-9.25 0,-5.11 -4.14,-9.25 -9.25,-9.25 z m 4.39,11.53 c -1.93,1.93 -4.78,2.31 -6.7,2.31 -0.7,0 -1.41,-0.05 -2.1,-0.16 0,0 -1.02,-5.64 2.14,-8.81 0.83,-0.83 1.95,-1.28 3.13,-1.28 1.27,0 2.49,0.51 3.39,1.42 1.84,1.84 1.89,4.75 0.14,6.52 z", path1: "M 10.5,-0.01 C 4.7,-0.01 0,4.7 0,10.49 c 0,5.79 4.7,10.5 10.5,10.5 5.8,0 10.5,-4.7 10.5,-10.5 C 20.99,4.7 16.3,-0.01 10.5,-0.01 Z m 0,19.75 c -5.11,0 -9.25,-4.15 -9.25,-9.25 0,-5.1 4.14,-9.26 9.25,-9.26 5.11,0 9.25,4.15 9.25,9.25 0,5.13 -4.14,9.26 -9.25,9.26 z", path2: "M 14.74,6.25 C 12.9,4.41 9.98,4.35 8.23,6.1 5.07,9.27 6.09,14.91 6.09,14.91 c 0,0 5.64,1.02 8.81,-2.14 C 16.64,11 16.59,8.09 14.74,6.25 Z m -2.27,4.09 -0.91,1.87 -0.9,-1.87 -1.86,-0.91 1.86,-0.9 0.9,-1.87 0.91,1.87 1.86,0.9 z", polygon: "11.56,12.21 10.66,10.34 8.8,9.43 10.66,8.53 11.56,6.66 12.47,8.53 14.33,9.43 12.47,10.34" }, styleRules: { map: "overflow:hidden;position:relative;", "missing-css": "display:none;", canary: "background-color:salmon;", "ctrl-bottom-left": "position: absolute; pointer-events: none; z-index: 2; bottom: 0; left: 0;", "ctrl-bottom-right": "position: absolute; pointer-events: none; z-index: 2; right: 0; bottom: 0;", ctrl: "clear: both; pointer-events: auto; transform: translate(0, 0);", "ctrl-attrib.mapboxgl-compact .mapboxgl-ctrl-attrib-inner": "display: none;", "ctrl-attrib.mapboxgl-compact:hover .mapboxgl-ctrl-attrib-inner": "display: block; margin-top:2px", "ctrl-attrib.mapboxgl-compact:hover": "padding: 2px 24px 2px 4px; visibility: visible; margin-top: 6px;", "ctrl-attrib.mapboxgl-compact::after": `content: ""; cursor: pointer; position: absolute; background-image: url('data:image/svg+xml;charset=utf-8,%3Csvg viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"%3E %3Cpath fill="%23333333" fill-rule="evenodd" d="M4,10a6,6 0 1,0 12,0a6,6 0 1,0 -12,0 M9,7a1,1 0 1,0 2,0a1,1 0 1,0 -2,0 M9,10a1,1 0 1,1 2,0l0,3a1,1 0 1,1 -2,0"/%3E %3C/svg%3E'); background-color: rgba(255, 255, 255, 0.5); width: 24px; height: 24px; box-sizing: border-box; border-radius: 12px;`, "ctrl-attrib.mapboxgl-compact": "min-height: 20px; padding: 0; margin: 10px; position: relative; background-color: #fff; border-radius: 3px 12px 12px 3px;", "ctrl-bottom-right > .mapboxgl-ctrl-attrib.mapboxgl-compact::after": "bottom: 0; right: 0", "ctrl-bottom-left > .mapboxgl-ctrl-attrib.mapboxgl-compact::after": "bottom: 0; left: 0", "ctrl-bottom-left .mapboxgl-ctrl": "margin: 0 0 10px 10px; float: left;", "ctrl-bottom-right .mapboxgl-ctrl": "margin: 0 10px 10px 0; float: right;", "ctrl-attrib": "color: rgba(0, 0, 0, 0.75); text-decoration: none; font-size: 12px", "ctrl-attrib a": "color: rgba(0, 0, 0, 0.75); text-decoration: none; font-size: 12px", "ctrl-attrib a:hover": "color: inherit; text-decoration: underline;", "ctrl-attrib .mapbox-improve-map": "font-weight: bold; margin-left: 2px;", "attrib-empty": "display: none;", "ctrl-logo": `display:block; width: 21px; height: 21px; background-image: url('data:image/svg+xml;charset=utf-8,%3C?xml version="1.0" encoding="utf-8"?%3E %3Csvg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 21 21" style="enable-background:new 0 0 21 21;" xml:space="preserve"%3E%3Cg transform="translate(0,0.01)"%3E%3Cpath d="m 10.5,1.24 c -5.11,0 -9.25,4.15 -9.25,9.25 0,5.1 4.15,9.25 9.25,9.25 5.1,0 9.25,-4.15 9.25,-9.25 0,-5.11 -4.14,-9.25 -9.25,-9.25 z m 4.39,11.53 c -1.93,1.93 -4.78,2.31 -6.7,2.31 -0.7,0 -1.41,-0.05 -2.1,-0.16 0,0 -1.02,-5.64 2.14,-8.81 0.83,-0.83 1.95,-1.28 3.13,-1.28 1.27,0 2.49,0.51 3.39,1.42 1.84,1.84 1.89,4.75 0.14,6.52 z" style="opacity:0.9;fill:%23ffffff;enable-background:new" class="st0"/%3E%3Cpath d="M 10.5,-0.01 C 4.7,-0.01 0,4.7 0,10.49 c 0,5.79 4.7,10.5 10.5,10.5 5.8,0 10.5,-4.7 10.5,-10.5 C 20.99,4.7 16.3,-0.01 10.5,-0.01 Z m 0,19.75 c -5.11,0 -9.25,-4.15 -9.25,-9.25 0,-5.1 4.14,-9.26 9.25,-9.26 5.11,0 9.25,4.15 9.25,9.25 0,5.13 -4.14,9.26 -9.25,9.26 z" style="opacity:0.35;enable-background:new" class="st1"/%3E%3Cpath d="M 14.74,6.25 C 12.9,4.41 9.98,4.35 8.23,6.1 5.07,9.27 6.09,14.91 6.09,14.91 c 0,0 5.64,1.02 8.81,-2.14 C 16.64,11 16.59,8.09 14.74,6.25 Z m -2.27,4.09 -0.91,1.87 -0.9,-1.87 -1.86,-0.91 1.86,-0.9 0.9,-1.87 0.91,1.87 1.86,0.9 z" style="opacity:0.35;enable-background:new" class="st1"/%3E%3Cpolygon points="11.56,12.21 10.66,10.34 8.8,9.43 10.66,8.53 11.56,6.66 12.47,8.53 14.33,9.43 12.47,10.34 " style="opacity:0.9;fill:%23ffffff;enable-background:new" class="st0"/%3E%3C/g%3E%3C/svg%3E')` } };
          }, 2178: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              var ae = ne.split(" "), oe = ae[0], se = ae[1], ce = te.isArrayOrTypedArray(ee) ? te.mean(ee) : ee, le = 0.5 + ce / 100, ue = 1.5 + ce / 100, de = ["", ""], fe = [0, 0];
              switch (oe) {
                case "top":
                  de[0] = "top", fe[1] = -ue;
                  break;
                case "bottom":
                  de[0] = "bottom", fe[1] = ue;
              }
              switch (se) {
                case "left":
                  de[1] = "right", fe[0] = -le;
                  break;
                case "right":
                  de[1] = "left", fe[0] = le;
              }
              return { anchor: de[0] && de[1] ? de.join("-") : de[0] ? de[0] : de[1] ? de[1] : "center", offset: fe };
            };
          }, 68192: function(G, J, Z) {
            var te = Z(32280), ne = Z(34809), ee = ne.strTranslate, ae = ne.strScale, oe = Z(4173).fX, se = Z(62972), ce = Z(45568), le = Z(62203), ue = Z(30635), de = Z(5417), fe = "mapbox", he = J.constants = Z(44245);
            J.name = fe, J.attr = "subplot", J.idRoot = fe, J.idRegex = J.attrRegex = ne.counterRegex(fe);
            var me = ["mapbox subplots and traces are deprecated!", "Please consider switching to `map` subplots and traces.", "Learn more at: https://plotly.com/javascript/maplibre-migration/"].join(" ");
            J.attributes = { subplot: { valType: "subplotid", dflt: "mapbox", editType: "calc" } }, J.layoutAttributes = Z(67514), J.supplyLayoutDefaults = Z(86989);
            var pe = !0;
            function ye(ge) {
              return typeof ge == "string" && (he.styleValuesMapbox.indexOf(ge) !== -1 || ge.indexOf("mapbox://") === 0 || ge.indexOf("stamen") === 0);
            }
            J.plot = function(ge) {
              pe && (pe = !1, ne.warn(me));
              var ve = ge._fullLayout, be = ge.calcdata, Ce = ve._subplots[fe];
              if (te.version !== he.requiredVersion) throw new Error(he.wrongVersionErrorMsg);
              var Se = function(ke, Me) {
                var Re = ke._fullLayout;
                if (ke._context.mapboxAccessToken === "") return "";
                for (var De = [], Pe = [], $e = !1, qe = !1, Xe = 0; Xe < Me.length; Xe++) {
                  var Ve = Re[Me[Xe]], Ze = Ve.accesstoken;
                  ye(Ve.style) && (Ze ? ne.pushUnique(De, Ze) : (ye(Ve._input.style) && (ne.error("Uses Mapbox map style, but did not set an access token."), $e = !0), qe = !0)), Ze && ne.pushUnique(Pe, Ze);
                }
                if (qe) {
                  var Ge = $e ? he.noAccessTokenErrorMsg : he.missingStyleErrorMsg;
                  throw ne.error(Ge), new Error(Ge);
                }
                return De.length ? (De.length > 1 && ne.warn(he.multipleTokensErrorMsg), De[0]) : (Pe.length && ne.log(["Listed mapbox access token(s)", Pe.join(","), "but did not use a Mapbox map style, ignoring token(s)."].join(" ")), "");
              }(ge, Ce);
              te.accessToken = Se;
              for (var we = 0; we < Ce.length; we++) {
                var Ee = Ce[we], xe = oe(be, fe, Ee), Le = ve[Ee], Qe = Le._subplot;
                Qe || (Qe = new de(ge, Ee), ve[Ee]._subplot = Qe), Qe.viewInitial || (Qe.viewInitial = { center: ne.extendFlat({}, Le.center), zoom: Le.zoom, bearing: Le.bearing, pitch: Le.pitch }), Qe.plot(xe, ve, ge._promises);
              }
            }, J.clean = function(ge, ve, be, Ce) {
              for (var Se = Ce._subplots[fe] || [], we = 0; we < Se.length; we++) {
                var Ee = Se[we];
                !ve[Ee] && Ce[Ee]._subplot && Ce[Ee]._subplot.destroy();
              }
            }, J.toSVG = function(ge) {
              for (var ve = ge._fullLayout, be = ve._subplots[fe], Ce = ve._size, Se = 0; Se < be.length; Se++) {
                var we = ve[be[Se]], Ee = we.domain, xe = we._subplot.toImage("png");
                ve._glimages.append("svg:image").attr({ xmlns: se.svg, "xlink:href": xe, x: Ce.l + Ce.w * Ee.x[0], y: Ce.t + Ce.h * (1 - Ee.y[1]), width: Ce.w * (Ee.x[1] - Ee.x[0]), height: Ce.h * (Ee.y[1] - Ee.y[0]), preserveAspectRatio: "none" });
                var Le = ce.select(we._subplot.div);
                if (Le.select(".mapboxgl-ctrl-logo").node().offsetParent !== null) {
                  var Qe = ve._glimages.append("g");
                  Qe.attr("transform", ee(Ce.l + Ce.w * Ee.x[0] + 10, Ce.t + Ce.h * (1 - Ee.y[0]) - 31)), Qe.append("path").attr("d", he.mapboxLogo.path0).style({ opacity: 0.9, fill: "#ffffff", "enable-background": "new" }), Qe.append("path").attr("d", he.mapboxLogo.path1).style("opacity", 0.35).style("enable-background", "new"), Qe.append("path").attr("d", he.mapboxLogo.path2).style("opacity", 0.35).style("enable-background", "new"), Qe.append("polygon").attr("points", he.mapboxLogo.polygon).style({ opacity: 0.9, fill: "#ffffff", "enable-background": "new" });
                }
                var ke = Le.select(".mapboxgl-ctrl-attrib").text().replace("Improve this map", ""), Me = ve._glimages.append("g"), Re = Me.append("text");
                Re.text(ke).classed("static-attribution", !0).attr({ "font-size": 12, "font-family": "Arial", color: "rgba(0, 0, 0, 0.75)", "text-anchor": "end", "data-unformatted": ke });
                var De = le.bBox(Re.node()), Pe = Ce.w * (Ee.x[1] - Ee.x[0]);
                if (De.width > Pe / 2) {
                  var $e = ke.split("|").join("<br>");
                  Re.text($e).attr("data-unformatted", $e).call(ue.convertToTspans, ge), De = le.bBox(Re.node());
                }
                Re.attr("transform", ee(-3, 8 - De.height)), Me.insert("rect", ".static-attribution").attr({ x: -De.width - 6, y: -De.height - 3, width: De.width + 6, height: De.height + 3, fill: "rgba(255, 255, 255, 0.75)" });
                var qe = 1;
                De.width + 6 > Pe && (qe = Pe / (De.width + 6));
                var Xe = [Ce.l + Ce.w * Ee.x[1], Ce.t + Ce.h * (1 - Ee.y[0])];
                Me.attr("transform", ee(Xe[0], Xe[1]) + ae(qe));
              }
            }, J.updateFx = function(ge) {
              for (var ve = ge._fullLayout, be = ve._subplots[fe], Ce = 0; Ce < be.length; Ce++) ve[be[Ce]]._subplot.updateFx(ve);
            };
          }, 51276: function(G, J, Z) {
            var te = Z(34809), ne = Z(30635).sanitizeHTML, ee = Z(2178), ae = Z(44245);
            function oe(ue, de) {
              this.subplot = ue, this.uid = ue.uid + "-" + de, this.index = de, this.idSource = "source-" + this.uid, this.idLayer = ae.layoutLayerPrefix + this.uid, this.sourceType = null, this.source = null, this.layerType = null, this.below = null, this.visible = !1;
            }
            var se = oe.prototype;
            function ce(ue) {
              if (!ue.visible) return !1;
              var de = ue.source;
              if (Array.isArray(de) && de.length > 0) {
                for (var fe = 0; fe < de.length; fe++) if (typeof de[fe] != "string" || de[fe].length === 0) return !1;
                return !0;
              }
              return te.isPlainObject(de) || typeof de == "string" && de.length > 0;
            }
            function le(ue) {
              var de = {}, fe = {};
              switch (ue.type) {
                case "circle":
                  te.extendFlat(fe, { "circle-radius": ue.circle.radius, "circle-color": ue.color, "circle-opacity": ue.opacity });
                  break;
                case "line":
                  te.extendFlat(fe, { "line-width": ue.line.width, "line-color": ue.color, "line-opacity": ue.opacity, "line-dasharray": ue.line.dash });
                  break;
                case "fill":
                  te.extendFlat(fe, { "fill-color": ue.color, "fill-outline-color": ue.fill.outlinecolor, "fill-opacity": ue.opacity });
                  break;
                case "symbol":
                  var he = ue.symbol, me = ee(he.textposition, he.iconsize);
                  te.extendFlat(de, { "icon-image": he.icon + "-15", "icon-size": he.iconsize / 10, "text-field": he.text, "text-size": he.textfont.size, "text-anchor": me.anchor, "text-offset": me.offset, "symbol-placement": he.placement }), te.extendFlat(fe, { "icon-color": ue.color, "text-color": he.textfont.color, "text-opacity": ue.opacity });
                  break;
                case "raster":
                  te.extendFlat(fe, { "raster-fade-duration": 0, "raster-opacity": ue.opacity });
              }
              return { layout: de, paint: fe };
            }
            se.update = function(ue) {
              this.visible ? this.needsNewImage(ue) ? this.updateImage(ue) : this.needsNewSource(ue) ? (this.removeLayer(), this.updateSource(ue), this.updateLayer(ue)) : this.needsNewLayer(ue) ? this.updateLayer(ue) : this.updateStyle(ue) : (this.updateSource(ue), this.updateLayer(ue)), this.visible = ce(ue);
            }, se.needsNewImage = function(ue) {
              return this.subplot.map.getSource(this.idSource) && this.sourceType === "image" && ue.sourcetype === "image" && (this.source !== ue.source || JSON.stringify(this.coordinates) !== JSON.stringify(ue.coordinates));
            }, se.needsNewSource = function(ue) {
              return this.sourceType !== ue.sourcetype || JSON.stringify(this.source) !== JSON.stringify(ue.source) || this.layerType !== ue.type;
            }, se.needsNewLayer = function(ue) {
              return this.layerType !== ue.type || this.below !== this.subplot.belowLookup["layout-" + this.index];
            }, se.lookupBelow = function() {
              return this.subplot.belowLookup["layout-" + this.index];
            }, se.updateImage = function(ue) {
              this.subplot.map.getSource(this.idSource).updateImage({ url: ue.source, coordinates: ue.coordinates });
              var de = this.findFollowingMapboxLayerId(this.lookupBelow());
              de !== null && this.subplot.map.moveLayer(this.idLayer, de);
            }, se.updateSource = function(ue) {
              var de = this.subplot.map;
              if (de.getSource(this.idSource) && de.removeSource(this.idSource), this.sourceType = ue.sourcetype, this.source = ue.source, ce(ue)) {
                var fe = function(he) {
                  var me, pe = he.sourcetype, ye = he.source, ge = { type: pe };
                  return pe === "geojson" ? me = "data" : pe === "vector" ? me = typeof ye == "string" ? "url" : "tiles" : pe === "raster" ? (me = "tiles", ge.tileSize = 256) : pe === "image" && (me = "url", ge.coordinates = he.coordinates), ge[me] = ye, he.sourceattribution && (ge.attribution = ne(he.sourceattribution)), ge;
                }(ue);
                de.addSource(this.idSource, fe);
              }
            }, se.findFollowingMapboxLayerId = function(ue) {
              if (ue === "traces") for (var de = this.subplot.getMapLayers(), fe = 0; fe < de.length; fe++) {
                var he = de[fe].id;
                if (typeof he == "string" && he.indexOf(ae.traceLayerPrefix) === 0) {
                  ue = he;
                  break;
                }
              }
              return ue;
            }, se.updateLayer = function(ue) {
              var de = this.subplot, fe = le(ue), he = this.lookupBelow(), me = this.findFollowingMapboxLayerId(he);
              this.removeLayer(), ce(ue) && de.addLayer({ id: this.idLayer, source: this.idSource, "source-layer": ue.sourcelayer || "", type: ue.type, minzoom: ue.minzoom, maxzoom: ue.maxzoom, layout: fe.layout, paint: fe.paint }, me), this.layerType = ue.type, this.below = he;
            }, se.updateStyle = function(ue) {
              if (ce(ue)) {
                var de = le(ue);
                this.subplot.setOptions(this.idLayer, "setLayoutProperty", de.layout), this.subplot.setOptions(this.idLayer, "setPaintProperty", de.paint);
              }
            }, se.removeLayer = function() {
              var ue = this.subplot.map;
              ue.getLayer(this.idLayer) && ue.removeLayer(this.idLayer);
            }, se.dispose = function() {
              var ue = this.subplot.map;
              ue.getLayer(this.idLayer) && ue.removeLayer(this.idLayer), ue.getSource(this.idSource) && ue.removeSource(this.idSource);
            }, G.exports = function(ue, de, fe) {
              var he = new oe(ue, de);
              return he.update(fe), he;
            };
          }, 67514: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766).defaultLine, ee = Z(13792).u, ae = Z(80337), oe = Z(36640).textposition, se = Z(13582).overrideAll, ce = Z(78032).templatedArray, le = Z(44245), ue = ae({ noFontVariant: !0, noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0 });
            ue.family.dflt = "Open Sans Regular, Arial Unicode MS Regular", (G.exports = se({ _arrayAttrRegexps: [te.counterRegex("mapbox", ".layers", !0)], domain: ee({ name: "mapbox" }), accesstoken: { valType: "string", noBlank: !0, strict: !0 }, style: { valType: "any", values: le.styleValuesMapbox.concat(le.styleValuesNonMapbox), dflt: le.styleValueDflt }, center: { lon: { valType: "number", dflt: 0 }, lat: { valType: "number", dflt: 0 } }, zoom: { valType: "number", dflt: 1 }, bearing: { valType: "number", dflt: 0 }, pitch: { valType: "number", dflt: 0 }, bounds: { west: { valType: "number" }, east: { valType: "number" }, south: { valType: "number" }, north: { valType: "number" } }, layers: ce("layer", { visible: { valType: "boolean", dflt: !0 }, sourcetype: { valType: "enumerated", values: ["geojson", "vector", "raster", "image"], dflt: "geojson" }, source: { valType: "any" }, sourcelayer: { valType: "string", dflt: "" }, sourceattribution: { valType: "string" }, type: { valType: "enumerated", values: ["circle", "line", "fill", "symbol", "raster"], dflt: "circle" }, coordinates: { valType: "any" }, below: { valType: "string" }, color: { valType: "color", dflt: ne }, opacity: { valType: "number", min: 0, max: 1, dflt: 1 }, minzoom: { valType: "number", min: 0, max: 24, dflt: 0 }, maxzoom: { valType: "number", min: 0, max: 24, dflt: 24 }, circle: { radius: { valType: "number", dflt: 15 } }, line: { width: { valType: "number", dflt: 2 }, dash: { valType: "data_array" } }, fill: { outlinecolor: { valType: "color", dflt: ne } }, symbol: { icon: { valType: "string", dflt: "marker" }, iconsize: { valType: "number", dflt: 10 }, text: { valType: "string", dflt: "" }, placement: { valType: "enumerated", values: ["point", "line", "line-center"], dflt: "point" }, textfont: ue, textposition: te.extendFlat({}, oe, { arrayOk: !1 }) } }) }, "plot", "from-root")).uirevision = { valType: "any", editType: "none" };
          }, 86989: function(G, J, Z) {
            var te = Z(34809), ne = Z(4448), ee = Z(59008), ae = Z(67514);
            function oe(ce, le, ue, de) {
              ue("accesstoken", de.accessToken), ue("style"), ue("center.lon"), ue("center.lat"), ue("zoom"), ue("bearing"), ue("pitch");
              var fe = ue("bounds.west"), he = ue("bounds.east"), me = ue("bounds.south"), pe = ue("bounds.north");
              fe !== void 0 && he !== void 0 && me !== void 0 && pe !== void 0 || delete le.bounds, ee(ce, le, { name: "layers", handleItemDefaults: se }), le._input = ce;
            }
            function se(ce, le) {
              function ue(pe, ye) {
                return te.coerce(ce, le, ae.layers, pe, ye);
              }
              if (ue("visible")) {
                var de, fe = ue("sourcetype"), he = fe === "raster" || fe === "image";
                ue("source"), ue("sourceattribution"), fe === "vector" && ue("sourcelayer"), fe === "image" && ue("coordinates"), he && (de = "raster");
                var me = ue("type", de);
                he && me !== "raster" && (me = le.type = "raster", te.log("Source types *raster* and *image* must drawn *raster* layer type.")), ue("below"), ue("color"), ue("opacity"), ue("minzoom"), ue("maxzoom"), me === "circle" && ue("circle.radius"), me === "line" && (ue("line.width"), ue("line.dash")), me === "fill" && ue("fill.outlinecolor"), me === "symbol" && (ue("symbol.icon"), ue("symbol.iconsize"), ue("symbol.text"), te.coerceFont(ue, "symbol.textfont", void 0, { noFontVariant: !0, noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0 }), ue("symbol.textposition"), ue("symbol.placement"));
              }
            }
            G.exports = function(ce, le, ue) {
              ne(ce, le, ue, { type: "mapbox", attributes: ae, handleDefaults: oe, partition: "y", accessToken: le._mapboxAccessToken });
            };
          }, 5417: function(G, J, Z) {
            var te = Z(32280), ne = Z(34809), ee = Z(3994), ae = Z(33626), oe = Z(29714), se = Z(14751), ce = Z(32141), le = Z(70414), ue = le.drawMode, de = le.selectMode, fe = Z(44844).prepSelect, he = Z(44844).clearOutline, me = Z(44844).clearSelectionsCache, pe = Z(44844).selectOnClick, ye = Z(44245), ge = Z(51276);
            function ve(xe, Le) {
              this.id = Le, this.gd = xe;
              var Qe = xe._fullLayout, ke = xe._context;
              this.container = Qe._glcontainer.node(), this.isStatic = ke.staticPlot, this.uid = Qe._uid + "-" + this.id, this.div = null, this.xaxis = null, this.yaxis = null, this.createFramework(Qe), this.map = null, this.accessToken = null, this.styleObj = null, this.traceHash = {}, this.layerList = [], this.belowLookup = {}, this.dragging = !1, this.wheeling = !1;
            }
            var be = ve.prototype;
            be.plot = function(xe, Le, Qe) {
              var ke, Me = this, Re = Le[Me.id];
              Me.map && Re.accesstoken !== Me.accessToken && (Me.map.remove(), Me.map = null, Me.styleObj = null, Me.traceHash = {}, Me.layerList = []), ke = Me.map ? new Promise(function(De, Pe) {
                Me.updateMap(xe, Le, De, Pe);
              }) : new Promise(function(De, Pe) {
                Me.createMap(xe, Le, De, Pe);
              }), Qe.push(ke);
            }, be.createMap = function(xe, Le, Qe, ke) {
              var Me = this, Re = Le[Me.id], De = Me.styleObj = Se(Re.style, Le);
              Me.accessToken = Re.accesstoken;
              var Pe = Re.bounds, $e = Pe ? [[Pe.west, Pe.south], [Pe.east, Pe.north]] : null, qe = Me.map = new te.Map({ container: Me.div, style: De.style, center: Ee(Re.center), zoom: Re.zoom, bearing: Re.bearing, pitch: Re.pitch, maxBounds: $e, interactive: !Me.isStatic, preserveDrawingBuffer: Me.isStatic, doubleClickZoom: !1, boxZoom: !1, attributionControl: !1 }).addControl(new te.AttributionControl({ compact: !0 }));
              qe._canvas.style.left = "0px", qe._canvas.style.top = "0px", Me.rejectOnError(ke), Me.isStatic || Me.initFx(xe, Le);
              var Xe = [];
              Xe.push(new Promise(function(Ve) {
                qe.once("load", Ve);
              })), Xe = Xe.concat(ee.fetchTraceGeoData(xe)), Promise.all(Xe).then(function() {
                Me.fillBelowLookup(xe, Le), Me.updateData(xe), Me.updateLayout(Le), Me.resolveOnRender(Qe);
              }).catch(ke);
            }, be.updateMap = function(xe, Le, Qe, ke) {
              var Me = this, Re = Me.map, De = Le[this.id];
              Me.rejectOnError(ke);
              var Pe = [], $e = Se(De.style, Le);
              JSON.stringify(Me.styleObj) !== JSON.stringify($e) && (Me.styleObj = $e, Re.setStyle($e.style), Me.traceHash = {}, Pe.push(new Promise(function(qe) {
                Re.once("styledata", qe);
              }))), Pe = Pe.concat(ee.fetchTraceGeoData(xe)), Promise.all(Pe).then(function() {
                Me.fillBelowLookup(xe, Le), Me.updateData(xe), Me.updateLayout(Le), Me.resolveOnRender(Qe);
              }).catch(ke);
            }, be.fillBelowLookup = function(xe, Le) {
              var Qe, ke, Me = Le[this.id].layers, Re = this.belowLookup = {}, De = !1;
              for (Qe = 0; Qe < xe.length; Qe++) {
                var Pe = xe[Qe][0].trace, $e = Pe._module;
                typeof Pe.below == "string" ? ke = Pe.below : $e.getBelow && (ke = $e.getBelow(Pe, this)), ke === "" && (De = !0), Re["trace-" + Pe.uid] = ke || "";
              }
              for (Qe = 0; Qe < Me.length; Qe++) {
                var qe = Me[Qe];
                ke = typeof qe.below == "string" ? qe.below : De ? "traces" : "", Re["layout-" + Qe] = ke;
              }
              var Xe, Ve, Ze = {};
              for (Xe in Re) Ze[ke = Re[Xe]] ? Ze[ke].push(Xe) : Ze[ke] = [Xe];
              for (ke in Ze) {
                var Ge = Ze[ke];
                if (Ge.length > 1) for (Qe = 0; Qe < Ge.length; Qe++) (Xe = Ge[Qe]).indexOf("trace-") === 0 ? (Ve = Xe.split("trace-")[1], this.traceHash[Ve] && (this.traceHash[Ve].below = null)) : Xe.indexOf("layout-") === 0 && (Ve = Xe.split("layout-")[1], this.layerList[Ve] && (this.layerList[Ve].below = null));
              }
            };
            var Ce = { choroplethmapbox: 0, densitymapbox: 1, scattermapbox: 2 };
            function Se(xe, Le) {
              var Qe = {};
              if (ne.isPlainObject(xe)) Qe.id = xe.id, Qe.style = xe;
              else if (typeof xe == "string") if (Qe.id = xe, ye.styleValuesMapbox.indexOf(xe) !== -1) Qe.style = we(xe);
              else if (ye.stylesNonMapbox[xe]) {
                Qe.style = ye.stylesNonMapbox[xe];
                var ke = Qe.style.sources["plotly-" + xe], Me = ke ? ke.tiles : void 0;
                Me && Me[0] && Me[0].slice(-9) === "?api_key=" && (Me[0] += Le._mapboxAccessToken);
              } else Qe.style = xe;
              else Qe.id = ye.styleValueDflt, Qe.style = we(ye.styleValueDflt);
              return Qe.transition = { duration: 0, delay: 0 }, Qe;
            }
            function we(xe) {
              return ye.styleUrlPrefix + xe + "-" + ye.styleUrlSuffix;
            }
            function Ee(xe) {
              return [xe.lon, xe.lat];
            }
            be.updateData = function(xe) {
              var Le, Qe, ke, Me, Re = this.traceHash, De = xe.slice().sort(function(Ve, Ze) {
                return Ce[Ve[0].trace.type] - Ce[Ze[0].trace.type];
              });
              for (ke = 0; ke < De.length; ke++) {
                var Pe = De[ke], $e = !1;
                (Le = Re[(Qe = Pe[0].trace).uid]) && (Le.type === Qe.type ? (Le.update(Pe), $e = !0) : Le.dispose()), !$e && Qe._module && (Re[Qe.uid] = Qe._module.plot(this, Pe));
              }
              var qe = Object.keys(Re);
              e: for (ke = 0; ke < qe.length; ke++) {
                var Xe = qe[ke];
                for (Me = 0; Me < xe.length; Me++) if (Xe === (Qe = xe[Me][0].trace).uid) continue e;
                (Le = Re[Xe]).dispose(), delete Re[Xe];
              }
            }, be.updateLayout = function(xe) {
              var Le = this.map, Qe = xe[this.id];
              this.dragging || this.wheeling || (Le.setCenter(Ee(Qe.center)), Le.setZoom(Qe.zoom), Le.setBearing(Qe.bearing), Le.setPitch(Qe.pitch)), this.updateLayers(xe), this.updateFramework(xe), this.updateFx(xe), this.map.resize(), this.gd._context._scrollZoom.mapbox ? Le.scrollZoom.enable() : Le.scrollZoom.disable();
            }, be.resolveOnRender = function(xe) {
              var Le = this.map;
              Le.on("render", function Qe() {
                Le.loaded() && (Le.off("render", Qe), setTimeout(xe, 10));
              });
            }, be.rejectOnError = function(xe) {
              var Le = this.map;
              function Qe() {
                xe(new Error(ye.mapOnErrorMsg));
              }
              Le.once("error", Qe), Le.once("style.error", Qe), Le.once("source.error", Qe), Le.once("tile.error", Qe), Le.once("layer.error", Qe);
            }, be.createFramework = function(xe) {
              var Le = this, Qe = Le.div = document.createElement("div");
              Qe.id = Le.uid, Qe.style.position = "absolute", Le.container.appendChild(Qe), Le.xaxis = { _id: "x", c2p: function(ke) {
                return Le.project(ke).x;
              } }, Le.yaxis = { _id: "y", c2p: function(ke) {
                return Le.project(ke).y;
              } }, Le.updateFramework(xe), Le.mockAxis = { type: "linear", showexponent: "all", exponentformat: "B" }, oe.setConvert(Le.mockAxis, xe);
            }, be.initFx = function(xe, Le) {
              var Qe = this, ke = Qe.gd, Me = Qe.map;
              function Re() {
                ce.loneUnhover(Le._hoverlayer);
              }
              function De() {
                var Pe = Qe.getView();
                ke.emit("plotly_relayouting", Qe.getViewEditsWithDerived(Pe));
              }
              Me.on("moveend", function(Pe) {
                if (Qe.map) {
                  var $e = ke._fullLayout;
                  if (Pe.originalEvent || Qe.wheeling) {
                    var qe = $e[Qe.id];
                    ae.call("_storeDirectGUIEdit", ke.layout, $e._preGUI, Qe.getViewEdits(qe));
                    var Xe = Qe.getView();
                    qe._input.center = qe.center = Xe.center, qe._input.zoom = qe.zoom = Xe.zoom, qe._input.bearing = qe.bearing = Xe.bearing, qe._input.pitch = qe.pitch = Xe.pitch, ke.emit("plotly_relayout", Qe.getViewEditsWithDerived(Xe));
                  }
                  Pe.originalEvent && Pe.originalEvent.type === "mouseup" ? Qe.dragging = !1 : Qe.wheeling && (Qe.wheeling = !1), $e._rehover && $e._rehover();
                }
              }), Me.on("wheel", function() {
                Qe.wheeling = !0;
              }), Me.on("mousemove", function(Pe) {
                var $e = Qe.div.getBoundingClientRect(), qe = [Pe.originalEvent.offsetX, Pe.originalEvent.offsetY];
                Pe.target.getBoundingClientRect = function() {
                  return $e;
                }, Qe.xaxis.p2c = function() {
                  return Me.unproject(qe).lng;
                }, Qe.yaxis.p2c = function() {
                  return Me.unproject(qe).lat;
                }, ke._fullLayout._rehover = function() {
                  ke._fullLayout._hoversubplot === Qe.id && ke._fullLayout[Qe.id] && ce.hover(ke, Pe, Qe.id);
                }, ce.hover(ke, Pe, Qe.id), ke._fullLayout._hoversubplot = Qe.id;
              }), Me.on("dragstart", function() {
                Qe.dragging = !0, Re();
              }), Me.on("zoomstart", Re), Me.on("mouseout", function() {
                ke._fullLayout._hoversubplot = null;
              }), Me.on("drag", De), Me.on("zoom", De), Me.on("dblclick", function() {
                var Pe = ke._fullLayout[Qe.id];
                ae.call("_storeDirectGUIEdit", ke.layout, ke._fullLayout._preGUI, Qe.getViewEdits(Pe));
                var $e = Qe.viewInitial;
                Me.setCenter(Ee($e.center)), Me.setZoom($e.zoom), Me.setBearing($e.bearing), Me.setPitch($e.pitch);
                var qe = Qe.getView();
                Pe._input.center = Pe.center = qe.center, Pe._input.zoom = Pe.zoom = qe.zoom, Pe._input.bearing = Pe.bearing = qe.bearing, Pe._input.pitch = Pe.pitch = qe.pitch, ke.emit("plotly_doubleclick", null), ke.emit("plotly_relayout", Qe.getViewEditsWithDerived(qe));
              }), Qe.clearOutline = function() {
                me(Qe.dragOptions), he(Qe.dragOptions.gd);
              }, Qe.onClickInPanFn = function(Pe) {
                return function($e) {
                  var qe = ke._fullLayout.clickmode;
                  qe.indexOf("select") > -1 && pe($e.originalEvent, ke, [Qe.xaxis], [Qe.yaxis], Qe.id, Pe), qe.indexOf("event") > -1 && ce.click(ke, $e.originalEvent);
                };
              };
            }, be.updateFx = function(xe) {
              var Le = this, Qe = Le.map, ke = Le.gd;
              if (!Le.isStatic) {
                var Me, Re = xe.dragmode;
                Me = function($e, qe) {
                  qe.isRect ? ($e.range = {})[Le.id] = [Pe([qe.xmin, qe.ymin]), Pe([qe.xmax, qe.ymax])] : ($e.lassoPoints = {})[Le.id] = qe.map(Pe);
                };
                var De = Le.dragOptions;
                Le.dragOptions = ne.extendDeep(De || {}, { dragmode: xe.dragmode, element: Le.div, gd: ke, plotinfo: { id: Le.id, domain: xe[Le.id].domain, xaxis: Le.xaxis, yaxis: Le.yaxis, fillRangeItems: Me }, xaxes: [Le.xaxis], yaxes: [Le.yaxis], subplot: Le.id }), Qe.off("click", Le.onClickInPanHandler), de(Re) || ue(Re) ? (Qe.dragPan.disable(), Qe.on("zoomstart", Le.clearOutline), Le.dragOptions.prepFn = function($e, qe, Xe) {
                  fe($e, qe, Xe, Le.dragOptions, Re);
                }, se.init(Le.dragOptions)) : (Qe.dragPan.enable(), Qe.off("zoomstart", Le.clearOutline), Le.div.onmousedown = null, Le.div.ontouchstart = null, Le.div.removeEventListener("touchstart", Le.div._ontouchstart), Le.onClickInPanHandler = Le.onClickInPanFn(Le.dragOptions), Qe.on("click", Le.onClickInPanHandler));
              }
              function Pe($e) {
                var qe = Le.map.unproject($e);
                return [qe.lng, qe.lat];
              }
            }, be.updateFramework = function(xe) {
              var Le = xe[this.id].domain, Qe = xe._size, ke = this.div.style;
              ke.width = Qe.w * (Le.x[1] - Le.x[0]) + "px", ke.height = Qe.h * (Le.y[1] - Le.y[0]) + "px", ke.left = Qe.l + Le.x[0] * Qe.w + "px", ke.top = Qe.t + (1 - Le.y[1]) * Qe.h + "px", this.xaxis._offset = Qe.l + Le.x[0] * Qe.w, this.xaxis._length = Qe.w * (Le.x[1] - Le.x[0]), this.yaxis._offset = Qe.t + (1 - Le.y[1]) * Qe.h, this.yaxis._length = Qe.h * (Le.y[1] - Le.y[0]);
            }, be.updateLayers = function(xe) {
              var Le, Qe = xe[this.id].layers, ke = this.layerList;
              if (Qe.length !== ke.length) {
                for (Le = 0; Le < ke.length; Le++) ke[Le].dispose();
                for (ke = this.layerList = [], Le = 0; Le < Qe.length; Le++) ke.push(ge(this, Le, Qe[Le]));
              } else for (Le = 0; Le < Qe.length; Le++) ke[Le].update(Qe[Le]);
            }, be.destroy = function() {
              this.map && (this.map.remove(), this.map = null, this.container.removeChild(this.div));
            }, be.toImage = function() {
              return this.map.stop(), this.map.getCanvas().toDataURL();
            }, be.setOptions = function(xe, Le, Qe) {
              for (var ke in Qe) this.map[Le](xe, ke, Qe[ke]);
            }, be.getMapLayers = function() {
              return this.map.getStyle().layers;
            }, be.addLayer = function(xe, Le) {
              var Qe = this.map;
              if (typeof Le == "string") {
                if (Le === "") return void Qe.addLayer(xe, Le);
                for (var ke = this.getMapLayers(), Me = 0; Me < ke.length; Me++) if (Le === ke[Me].id) return void Qe.addLayer(xe, Le);
                ne.warn(["Trying to add layer with *below* value", Le, "referencing a layer that does not exist", "or that does not yet exist."].join(" "));
              }
              Qe.addLayer(xe);
            }, be.project = function(xe) {
              return this.map.project(new te.LngLat(xe[0], xe[1]));
            }, be.getView = function() {
              var xe = this.map, Le = xe.getCenter(), Qe = { lon: Le.lng, lat: Le.lat }, ke = xe.getCanvas(), Me = parseInt(ke.style.width), Re = parseInt(ke.style.height);
              return { center: Qe, zoom: xe.getZoom(), bearing: xe.getBearing(), pitch: xe.getPitch(), _derived: { coordinates: [xe.unproject([0, 0]).toArray(), xe.unproject([Me, 0]).toArray(), xe.unproject([Me, Re]).toArray(), xe.unproject([0, Re]).toArray()] } };
            }, be.getViewEdits = function(xe) {
              for (var Le = this.id, Qe = ["center", "zoom", "bearing", "pitch"], ke = {}, Me = 0; Me < Qe.length; Me++) {
                var Re = Qe[Me];
                ke[Le + "." + Re] = xe[Re];
              }
              return ke;
            }, be.getViewEditsWithDerived = function(xe) {
              var Le = this.id, Qe = this.getViewEdits(xe);
              return Qe[Le + "._derived"] = xe._derived, Qe;
            }, G.exports = ve;
          }, 57891: function(G) {
            G.exports = function(J) {
              var Z = J.editType;
              return { t: { valType: "number", dflt: 0, editType: Z }, r: { valType: "number", dflt: 0, editType: Z }, b: { valType: "number", dflt: 0, editType: Z }, l: { valType: "number", dflt: 0, editType: Z }, editType: Z };
            };
          }, 44122: function(G, J, Z) {
            var te = Z(45568), ne = Z(42696).de, ee = Z(36464).OE, ae = Z(10721), oe = Z(93229), se = Z(33626), ce = Z(57297), le = Z(78032), ue = Z(34809), de = Z(78766), fe = Z(63821).BADNUM, he = Z(5975), me = Z(78534).clearOutline, pe = Z(26667), ye = Z(49722), ge = Z(58935), ve = Z(4173).eV, be = ue.relinkPrivateKeys, Ce = ue._, Se = G.exports = {};
            ue.extendFlat(Se, se), Se.attributes = Z(9829), Se.attributes.type.values = Se.allTypes, Se.fontAttrs = Z(80337), Se.layoutAttributes = Z(6704);
            var we = Se.transformsRegistry, Ee = Z(90251);
            Se.executeAPICommand = Ee.executeAPICommand, Se.computeAPICommandBindings = Ee.computeAPICommandBindings, Se.manageCommandObserver = Ee.manageCommandObserver, Se.hasSimpleAPICommandBindings = Ee.hasSimpleAPICommandBindings, Se.redrawText = function(Ge) {
              return Ge = ue.getGraphDiv(Ge), new Promise(function(ze) {
                setTimeout(function() {
                  Ge._fullLayout && (se.getComponentMethod("annotations", "draw")(Ge), se.getComponentMethod("legend", "draw")(Ge), se.getComponentMethod("colorbar", "draw")(Ge), ze(Se.previousPromises(Ge)));
                }, 300);
              });
            }, Se.resize = function(Ge) {
              var ze;
              Ge = ue.getGraphDiv(Ge);
              var We = new Promise(function(at, ct) {
                Ge && !ue.isHidden(Ge) || ct(new Error("Resize must be passed a displayed plot div element.")), Ge._redrawTimer && clearTimeout(Ge._redrawTimer), Ge._resolveResize && (ze = Ge._resolveResize), Ge._resolveResize = at, Ge._redrawTimer = setTimeout(function() {
                  if (!Ge.layout || Ge.layout.width && Ge.layout.height || ue.isHidden(Ge)) at(Ge);
                  else {
                    delete Ge.layout.width, delete Ge.layout.height;
                    var et = Ge.changed;
                    Ge.autoplay = !0, se.call("relayout", Ge, { autosize: !0 }).then(function() {
                      Ge.changed = et, Ge._resolveResize === at && (delete Ge._resolveResize, at(Ge));
                    });
                  }
                }, 100);
              });
              return ze && ze(We), We;
            }, Se.previousPromises = function(Ge) {
              if ((Ge._promises || []).length) return Promise.all(Ge._promises).then(function() {
                Ge._promises = [];
              });
            }, Se.addLinks = function(Ge) {
              if (Ge._context.showLink || Ge._context.showSources) {
                var ze = Ge._fullLayout, We = ue.ensureSingle(ze._paper, "text", "js-plot-link-container", function(it) {
                  it.style({ "font-family": '"Open Sans", Arial, sans-serif', "font-size": "12px", fill: de.defaultLine, "pointer-events": "all" }).each(function() {
                    var st = te.select(this);
                    st.append("tspan").classed("js-link-to-tool", !0), st.append("tspan").classed("js-link-spacer", !0), st.append("tspan").classed("js-sourcelinks", !0);
                  });
                }), at = We.node(), ct = { y: ze._paper.attr("height") - 9 };
                document.body.contains(at) && at.getComputedTextLength() >= ze.width - 20 ? (ct["text-anchor"] = "start", ct.x = 5) : (ct["text-anchor"] = "end", ct.x = ze._paper.attr("width") - 7), We.attr(ct);
                var et = We.select(".js-link-to-tool"), dt = We.select(".js-link-spacer"), Je = We.select(".js-sourcelinks");
                Ge._context.showSources && Ge._context.showSources(Ge), Ge._context.showLink && function(it, st) {
                  st.text("");
                  var gt = st.append("a").attr({ "xlink:xlink:href": "#", class: "link--impt link--embedview", "font-weight": "bold" }).text(it._context.linkText + " »");
                  if (it._context.sendData) gt.on("click", function() {
                    Se.sendDataToCloud(it);
                  });
                  else {
                    var bt = window.location.pathname.split("/"), yt = window.location.search;
                    gt.attr({ "xlink:xlink:show": "new", "xlink:xlink:href": "/" + bt[2].split(".")[0] + "/" + bt[1] + yt });
                  }
                }(Ge, et), dt.text(et.text() && Je.text() ? " - " : "");
              }
            }, Se.sendDataToCloud = function(Ge) {
              var ze = (window.PLOTLYENV || {}).BASE_URL || Ge._context.plotlyServerURL;
              if (ze) {
                Ge.emit("plotly_beforeexport");
                var We = te.select(Ge).append("div").attr("id", "hiddenform").style("display", "none"), at = We.append("form").attr({ action: ze + "/external", method: "post", target: "_blank" });
                return at.append("input").attr({ type: "text", name: "data" }).node().value = Se.graphJson(Ge, !1, "keepdata"), at.node().submit(), We.remove(), Ge.emit("plotly_afterexport"), !1;
              }
            };
            var xe = ["days", "shortDays", "months", "shortMonths", "periods", "dateTime", "date", "time", "decimal", "thousands", "grouping", "currency"], Le = ["year", "month", "dayMonth", "dayMonthYear"];
            function Qe(Ge, ze) {
              var We = Ge._context.locale;
              We || (We = "en-US");
              var at = !1, ct = {};
              function et(bt) {
                for (var yt = !0, St = 0; St < ze.length; St++) {
                  var Lt = ze[St];
                  ct[Lt] || (bt[Lt] ? ct[Lt] = bt[Lt] : yt = !1);
                }
                yt && (at = !0);
              }
              for (var dt = 0; dt < 2; dt++) {
                for (var Je = Ge._context.locales, it = 0; it < 2; it++) {
                  var st = (Je[We] || {}).format;
                  if (st && (et(st), at)) break;
                  Je = se.localeRegistry;
                }
                var gt = We.split("-")[0];
                if (at || gt === We) break;
                We = gt;
              }
              return at || et(se.localeRegistry.en.format), ct;
            }
            function ke(Ge, ze) {
              var We = { _fullLayout: ze }, at = Ge._id.charAt(0) === "x", ct = Ge._mainAxis._anchorAxis, et = "", dt = "", Je = "";
              if (ct && (Je = ct._mainAxis._id, et = at ? Ge._id + Je : Je + Ge._id), !et || !ze._plots[et]) {
                et = "";
                for (var it = Ge._counterAxes, st = 0; st < it.length; st++) {
                  var gt = it[st], bt = at ? Ge._id + gt : gt + Ge._id;
                  dt || (dt = bt);
                  var yt = he.getFromId(We, gt);
                  if (Je && yt.overlaying === Je) {
                    et = bt;
                    break;
                  }
                }
              }
              return et || dt;
            }
            function Me(Ge) {
              var ze = Ge.transforms;
              if (Array.isArray(ze) && ze.length) for (var We = 0; We < ze.length; We++) {
                var at = ze[We], ct = at._module || we[at.type];
                if (ct && ct.makesData) return !0;
              }
              return !1;
            }
            function Re(Ge, ze, We, at) {
              for (var ct = Ge.transforms, et = [Ge], dt = 0; dt < ct.length; dt++) {
                var Je = ct[dt], it = we[Je.type];
                it && it.transform && (et = it.transform(et, { transform: Je, fullTrace: Ge, fullData: ze, layout: We, fullLayout: at, transformIndex: dt }));
              }
              return et;
            }
            function De(Ge) {
              return typeof Ge == "string" && Ge.substr(Ge.length - 2) === "px" && parseFloat(Ge);
            }
            function Pe(Ge) {
              var ze = Ge.margin;
              if (!Ge._size) {
                var We = Ge._size = { l: Math.round(ze.l), r: Math.round(ze.r), t: Math.round(ze.t), b: Math.round(ze.b), p: Math.round(ze.pad) };
                We.w = Math.round(Ge.width) - We.l - We.r, We.h = Math.round(Ge.height) - We.t - We.b;
              }
              Ge._pushmargin || (Ge._pushmargin = {}), Ge._pushmarginIds || (Ge._pushmarginIds = {}), Ge._reservedMargin || (Ge._reservedMargin = {});
            }
            Se.supplyDefaults = function(Ge, ze) {
              var We = ze && ze.skipUpdateCalc, at = Ge._fullLayout || {};
              if (at._skipDefaults) delete at._skipDefaults;
              else {
                var ct, et = Ge._fullLayout = {}, dt = Ge.layout || {}, Je = Ge._fullData || [], it = Ge._fullData = [], st = Ge.data || [], gt = Ge.calcdata || [], bt = Ge._context || {};
                Ge._transitionData || Se.createTransitionData(Ge), et._dfltTitle = { plot: Ce(Ge, "Click to enter Plot title"), subtitle: Ce(Ge, "Click to enter Plot subtitle"), x: Ce(Ge, "Click to enter X axis title"), y: Ce(Ge, "Click to enter Y axis title"), colorbar: Ce(Ge, "Click to enter Colorscale title"), annotation: Ce(Ge, "new text") }, et._traceWord = Ce(Ge, "trace");
                var yt = Qe(Ge, xe);
                if (et._mapboxAccessToken = bt.mapboxAccessToken, at._initialAutoSizeIsDone) {
                  var St = at.width, Lt = at.height;
                  Se.supplyLayoutGlobalDefaults(dt, et, yt), dt.width || (et.width = St), dt.height || (et.height = Lt), Se.sanitizeMargins(et);
                } else {
                  Se.supplyLayoutGlobalDefaults(dt, et, yt);
                  var Rt = !dt.width || !dt.height, vt = et.autosize, $t = bt.autosizable;
                  Rt && (vt || $t) ? Se.plotAutoSize(Ge, dt, et) : Rt && Se.sanitizeMargins(et), !vt && Rt && (dt.width = et.width, dt.height = et.height);
                }
                et._d3locale = function(rn, vn) {
                  return rn.decimal = vn.charAt(0), rn.thousands = vn.charAt(1), { numberFormat: function(dn) {
                    try {
                      dn = ee(rn).format(ue.adjustFormat(dn));
                    } catch {
                      return ue.warnBadFormat(dn), ue.noFormat;
                    }
                    return dn;
                  }, timeFormat: ne(rn).utcFormat };
                }(yt, et.separators), et._extraFormat = Qe(Ge, Le), et._initialAutoSizeIsDone = !0, et._dataLength = st.length, et._modules = [], et._visibleModules = [], et._basePlotModules = [];
                var Pt = et._subplots = function() {
                  var rn, vn, dn = se.collectableSubplotTypes, Ht = {};
                  if (!dn) {
                    dn = [];
                    var jt = se.subplotsRegistry;
                    for (var cn in jt) {
                      var wn = jt[cn].attr;
                      if (wn && (dn.push(cn), Array.isArray(wn))) for (vn = 0; vn < wn.length; vn++) ue.pushUnique(dn, wn[vn]);
                    }
                  }
                  for (rn = 0; rn < dn.length; rn++) Ht[dn[rn]] = [];
                  return Ht;
                }(), Xt = et._splomAxes = { x: {}, y: {} }, Nt = et._splomSubplots = {};
                et._splomGridDflt = {}, et._scatterStackOpts = {}, et._firstScatter = {}, et._alignmentOpts = {}, et._colorAxes = {}, et._requestRangeslider = {}, et._traceUids = function(rn, vn) {
                  var dn, Ht, jt = vn.length, cn = [];
                  for (dn = 0; dn < rn.length; dn++) {
                    var wn = rn[dn]._fullInput;
                    wn !== Ht && cn.push(wn), Ht = wn;
                  }
                  var Rn = cn.length, Pn = new Array(jt), Ie = {};
                  function rt(wt, Bt) {
                    Pn[Bt] = wt, Ie[wt] = 1;
                  }
                  function ft(wt, Bt) {
                    if (wt && typeof wt == "string" && !Ie[wt]) return rt(wt, Bt), !0;
                  }
                  for (dn = 0; dn < jt; dn++) {
                    var pt = vn[dn].uid;
                    typeof pt == "number" && (pt = String(pt)), ft(pt, dn) || dn < Rn && ft(cn[dn].uid, dn) || rt(ue.randstr(Ie), dn);
                  }
                  return Pn;
                }(Je, st), et._globalTransforms = (Ge._context || {}).globalTransforms, Se.supplyDataDefaults(st, it, dt, et);
                var Gt = Object.keys(Xt.x), Ft = Object.keys(Xt.y);
                if (Gt.length > 1 && Ft.length > 1) {
                  for (se.getComponentMethod("grid", "sizeDefaults")(dt, et), ct = 0; ct < Gt.length; ct++) ue.pushUnique(Pt.xaxis, Gt[ct]);
                  for (ct = 0; ct < Ft.length; ct++) ue.pushUnique(Pt.yaxis, Ft[ct]);
                  for (var Tt in Nt) ue.pushUnique(Pt.cartesian, Tt);
                }
                if (et._has = Se._hasPlotType.bind(et), Je.length === it.length) for (ct = 0; ct < it.length; ct++) be(it[ct], Je[ct]);
                Se.supplyLayoutModuleDefaults(dt, et, it, Ge._transitionData);
                var It = et._visibleModules, Ct = [];
                for (ct = 0; ct < It.length; ct++) {
                  var Kt = It[ct].crossTraceDefaults;
                  Kt && ue.pushUnique(Ct, Kt);
                }
                for (ct = 0; ct < Ct.length; ct++) Ct[ct](it, et);
                et._hasOnlyLargeSploms = et._basePlotModules.length === 1 && et._basePlotModules[0].name === "splom" && Gt.length > 15 && Ft.length > 15 && et.shapes.length === 0 && et.images.length === 0, Se.linkSubplots(it, et, Je, at), Se.cleanPlot(it, et, Je, at);
                var Vt = !(!at._has || !at._has("gl2d")), Yt = !(!et._has || !et._has("gl2d")), sn = !(!at._has || !at._has("cartesian")) || Vt, hn = !(!et._has || !et._has("cartesian")) || Yt;
                sn && !hn ? at._bgLayer.remove() : hn && !sn && (et._shouldCreateBgLayer = !0), at._zoomlayer && !Ge._dragging && me({ _fullLayout: at }), function(rn, vn) {
                  var dn, Ht = [];
                  vn.meta && (dn = vn._meta = { meta: vn.meta, layout: { meta: vn.meta } });
                  for (var jt = 0; jt < rn.length; jt++) {
                    var cn = rn[jt];
                    cn.meta ? Ht[cn.index] = cn._meta = { meta: cn.meta } : vn.meta && (cn._meta = { meta: vn.meta }), vn.meta && (cn._meta.layout = { meta: vn.meta });
                  }
                  Ht.length && (dn || (dn = vn._meta = {}), dn.data = Ht);
                }(it, et), be(et, at), se.getComponentMethod("colorscale", "crossTraceDefaults")(it, et), et._preGUI || (et._preGUI = {}), et._tracePreGUI || (et._tracePreGUI = {});
                var Qt, Ut = et._tracePreGUI, Wt = {};
                for (Qt in Ut) Wt[Qt] = "old";
                for (ct = 0; ct < it.length; ct++) Wt[Qt = it[ct]._fullInput.uid] || (Ut[Qt] = {}), Wt[Qt] = "new";
                for (Qt in Wt) Wt[Qt] === "old" && delete Ut[Qt];
                Pe(et), se.getComponentMethod("rangeslider", "makeData")(et), We || gt.length !== it.length || Se.supplyDefaultsUpdateCalc(gt, it);
              }
            }, Se.supplyDefaultsUpdateCalc = function(Ge, ze) {
              for (var We = 0; We < ze.length; We++) {
                var at = ze[We], ct = (Ge[We] || [])[0];
                if (ct && ct.trace) {
                  var et = ct.trace;
                  if (et._hasCalcTransform) {
                    var dt, Je, it, st = et._arrayAttrs;
                    for (dt = 0; dt < st.length; dt++) Je = st[dt], it = ue.nestedProperty(et, Je).get().slice(), ue.nestedProperty(at, Je).set(it);
                  }
                  ct.trace = at;
                }
              }
            }, Se.createTransitionData = function(Ge) {
              Ge._transitionData || (Ge._transitionData = {}), Ge._transitionData._frames || (Ge._transitionData._frames = []), Ge._transitionData._frameHash || (Ge._transitionData._frameHash = {}), Ge._transitionData._counter || (Ge._transitionData._counter = 0), Ge._transitionData._interruptCallbacks || (Ge._transitionData._interruptCallbacks = []);
            }, Se._hasPlotType = function(Ge) {
              var ze, We = this._basePlotModules || [];
              for (ze = 0; ze < We.length; ze++) if (We[ze].name === Ge) return !0;
              var at = this._modules || [];
              for (ze = 0; ze < at.length; ze++) {
                var ct = at[ze].name;
                if (ct === Ge) return !0;
                var et = se.modules[ct];
                if (et && et.categories[Ge]) return !0;
              }
              return !1;
            }, Se.cleanPlot = function(Ge, ze, We, at) {
              var ct, et, dt = at._basePlotModules || [];
              for (ct = 0; ct < dt.length; ct++) {
                var Je = dt[ct];
                Je.clean && Je.clean(Ge, ze, We, at);
              }
              var it = at._has && at._has("gl"), st = ze._has && ze._has("gl");
              it && !st && at._glcontainer !== void 0 && (at._glcontainer.selectAll(".gl-canvas").remove(), at._glcontainer.selectAll(".no-webgl").remove(), at._glcanvas = null);
              var gt = !!at._infolayer;
              e: for (ct = 0; ct < We.length; ct++) {
                var bt = We[ct].uid;
                for (et = 0; et < Ge.length; et++) if (bt === Ge[et].uid) continue e;
                gt && at._infolayer.select(".cb" + bt).remove();
              }
            }, Se.linkSubplots = function(Ge, ze, We, at) {
              var ct, et, dt = at._plots || {}, Je = ze._plots = {}, it = ze._subplots, st = { _fullData: Ge, _fullLayout: ze }, gt = it.cartesian.concat(it.gl2d || []);
              for (ct = 0; ct < gt.length; ct++) {
                var bt, yt = gt[ct], St = dt[yt], Lt = he.getFromId(st, yt, "x"), Rt = he.getFromId(st, yt, "y");
                for (St ? bt = Je[yt] = St : (bt = Je[yt] = {}).id = yt, Lt._counterAxes.push(Rt._id), Rt._counterAxes.push(Lt._id), Lt._subplotsWith.push(yt), Rt._subplotsWith.push(yt), bt.xaxis = Lt, bt.yaxis = Rt, bt._hasClipOnAxisFalse = !1, et = 0; et < Ge.length; et++) {
                  var vt = Ge[et];
                  if (vt.xaxis === bt.xaxis._id && vt.yaxis === bt.yaxis._id && vt.cliponaxis === !1) {
                    bt._hasClipOnAxisFalse = !0;
                    break;
                  }
                }
              }
              var $t, Pt = he.list(st, null, !0);
              for (ct = 0; ct < Pt.length; ct++) {
                var Xt = null;
                ($t = Pt[ct]).overlaying && (Xt = he.getFromId(st, $t.overlaying)) && Xt.overlaying && ($t.overlaying = !1, Xt = null), $t._mainAxis = Xt || $t, Xt && ($t.domain = Xt.domain.slice()), $t._anchorAxis = $t.anchor === "free" ? null : he.getFromId(st, $t.anchor);
              }
              for (ct = 0; ct < Pt.length; ct++) if (($t = Pt[ct])._counterAxes.sort(he.idSort), $t._subplotsWith.sort(ue.subplotSort), $t._mainSubplot = ke($t, ze), $t._counterAxes.length && ($t.spikemode && $t.spikemode.indexOf("across") !== -1 || $t.automargin && $t.mirror && $t.anchor !== "free" || se.getComponentMethod("rangeslider", "isVisible")($t))) {
                var Nt = 1, Gt = 0;
                for (et = 0; et < $t._counterAxes.length; et++) {
                  var Ft = he.getFromId(st, $t._counterAxes[et]);
                  Nt = Math.min(Nt, Ft.domain[0]), Gt = Math.max(Gt, Ft.domain[1]);
                }
                Nt < Gt && ($t._counterDomainMin = Nt, $t._counterDomainMax = Gt);
              }
            }, Se.clearExpandedTraceDefaultColors = function(Ge) {
              var ze, We, at;
              for (We = [], (ze = Ge._module._colorAttrs) || (Ge._module._colorAttrs = ze = [], ce.crawl(Ge._module.attributes, function(ct, et, dt, Je) {
                We[Je] = et, We.length = Je + 1, ct.valType === "color" && ct.dflt === void 0 && ze.push(We.join("."));
              })), at = 0; at < ze.length; at++) ue.nestedProperty(Ge, "_input." + ze[at]).get() || ue.nestedProperty(Ge, ze[at]).set(null);
            }, Se.supplyDataDefaults = function(Ge, ze, We, at) {
              var ct, et, dt, Je = at._modules, it = at._visibleModules, st = at._basePlotModules, gt = 0, bt = 0;
              function yt(Tt) {
                ze.push(Tt);
                var It = Tt._module;
                It && (ue.pushUnique(Je, It), Tt.visible === !0 && ue.pushUnique(it, It), ue.pushUnique(st, Tt._module.basePlotModule), gt++, Tt._input.visible !== !1 && bt++);
              }
              at._transformModules = [];
              var St = {}, Lt = [], Rt = (We.template || {}).data || {}, vt = le.traceTemplater(Rt);
              for (ct = 0; ct < Ge.length; ct++) {
                if (dt = Ge[ct], (et = vt.newTrace(dt)).uid = at._traceUids[ct], Se.supplyTraceDefaults(dt, et, bt, at, ct), et.index = ct, et._input = dt, et._expandedIndex = gt, et.transforms && et.transforms.length) for (var $t = dt.visible !== !1 && et.visible === !1, Pt = Re(et, ze, We, at), Xt = 0; Xt < Pt.length; Xt++) {
                  var Nt = Pt[Xt], Gt = { _template: et._template, type: et.type, uid: et.uid + Xt };
                  $t && Nt.visible === !1 && delete Nt.visible, Se.supplyTraceDefaults(Nt, Gt, gt, at, ct), be(Gt, Nt), Gt.index = ct, Gt._input = dt, Gt._fullInput = et, Gt._expandedIndex = gt, Gt._expandedInput = Nt, yt(Gt);
                }
                else et._fullInput = et, et._expandedInput = et, yt(et);
                se.traceIs(et, "carpetAxis") && (St[et.carpet] = et), se.traceIs(et, "carpetDependent") && Lt.push(ct);
              }
              for (ct = 0; ct < Lt.length; ct++) if ((et = ze[Lt[ct]]).visible) {
                var Ft = St[et.carpet];
                et._carpet = Ft, Ft && Ft.visible ? (et.xaxis = Ft.xaxis, et.yaxis = Ft.yaxis) : et.visible = !1;
              }
            }, Se.supplyAnimationDefaults = function(Ge) {
              var ze;
              Ge = Ge || {};
              var We = {};
              function at(ct, et) {
                return ue.coerce(Ge || {}, We, ye, ct, et);
              }
              if (at("mode"), at("direction"), at("fromcurrent"), Array.isArray(Ge.frame)) for (We.frame = [], ze = 0; ze < Ge.frame.length; ze++) We.frame[ze] = Se.supplyAnimationFrameDefaults(Ge.frame[ze] || {});
              else We.frame = Se.supplyAnimationFrameDefaults(Ge.frame || {});
              if (Array.isArray(Ge.transition)) for (We.transition = [], ze = 0; ze < Ge.transition.length; ze++) We.transition[ze] = Se.supplyAnimationTransitionDefaults(Ge.transition[ze] || {});
              else We.transition = Se.supplyAnimationTransitionDefaults(Ge.transition || {});
              return We;
            }, Se.supplyAnimationFrameDefaults = function(Ge) {
              var ze = {};
              function We(at, ct) {
                return ue.coerce(Ge || {}, ze, ye.frame, at, ct);
              }
              return We("duration"), We("redraw"), ze;
            }, Se.supplyAnimationTransitionDefaults = function(Ge) {
              var ze = {};
              function We(at, ct) {
                return ue.coerce(Ge || {}, ze, ye.transition, at, ct);
              }
              return We("duration"), We("easing"), ze;
            }, Se.supplyFrameDefaults = function(Ge) {
              var ze = {};
              function We(at, ct) {
                return ue.coerce(Ge, ze, ge, at, ct);
              }
              return We("group"), We("name"), We("traces"), We("baseframe"), We("data"), We("layout"), ze;
            }, Se.supplyTraceDefaults = function(Ge, ze, We, at, ct) {
              var et, dt = at.colorway || de.defaults, Je = dt[We % dt.length];
              function it(Xt, Nt) {
                return ue.coerce(Ge, ze, Se.attributes, Xt, Nt);
              }
              var st = it("visible");
              it("type"), it("name", at._traceWord + " " + ct), it("uirevision", at.uirevision);
              var gt = Se.getModule(ze);
              if (ze._module = gt, gt) {
                var bt = gt.basePlotModule, yt = bt.attr, St = bt.attributes;
                if (yt && St) {
                  var Lt = at._subplots, Rt = "";
                  if (st || bt.name !== "gl2d") {
                    if (Array.isArray(yt)) for (et = 0; et < yt.length; et++) {
                      var vt = yt[et], $t = ue.coerce(Ge, ze, St, vt);
                      Lt[vt] && ue.pushUnique(Lt[vt], $t), Rt += $t;
                    }
                    else Rt = ue.coerce(Ge, ze, St, yt);
                    Lt[bt.name] && ue.pushUnique(Lt[bt.name], Rt);
                  }
                }
              }
              if (st) {
                if (it("customdata"), it("ids"), it("meta"), se.traceIs(ze, "showLegend") ? (ue.coerce(Ge, ze, gt.attributes.showlegend ? gt.attributes : Se.attributes, "showlegend"), it("legend"), it("legendwidth"), it("legendgroup"), it("legendgrouptitle.text"), it("legendrank"), ze._dfltShowLegend = !0) : ze._dfltShowLegend = !1, gt && gt.supplyDefaults(Ge, ze, Je, at), se.traceIs(ze, "noOpacity") || it("opacity"), se.traceIs(ze, "notLegendIsolatable") && (ze.visible = !!ze.visible), se.traceIs(ze, "noHover") || (ze.hovertemplate || ue.coerceHoverinfo(Ge, ze, at), ze.type !== "parcats" && se.getComponentMethod("fx", "supplyDefaults")(Ge, ze, Je, at)), gt && gt.selectPoints) {
                  var Pt = it("selectedpoints");
                  ue.isTypedArray(Pt) && (ze.selectedpoints = Array.from(Pt));
                }
                Se.supplyTransformDefaults(Ge, ze, at);
              }
              return ze;
            }, Se.hasMakesDataTransform = Me, Se.supplyTransformDefaults = function(Ge, ze, We) {
              if (ze._length || Me(Ge)) {
                var at = We._globalTransforms || [], ct = We._transformModules || [];
                if (Array.isArray(Ge.transforms) || at.length !== 0) for (var et = Ge.transforms || [], dt = at.concat(et), Je = ze.transforms = [], it = 0; it < dt.length; it++) {
                  var st, gt = dt[it], bt = gt.type, yt = we[bt], St = !(gt._module && gt._module === yt), Lt = yt && typeof yt.transform == "function";
                  yt || ue.warn("Unrecognized transform type " + bt + "."), yt && yt.supplyDefaults && (St || Lt) ? ((st = yt.supplyDefaults(gt, ze, We, Ge)).type = bt, st._module = yt, ue.pushUnique(ct, yt)) : st = ue.extendFlat({}, gt), Je.push(st);
                }
              }
            }, Se.supplyLayoutGlobalDefaults = function(Ge, ze, We) {
              function at(gt, bt) {
                return ue.coerce(Ge, ze, Se.layoutAttributes, gt, bt);
              }
              var ct = Ge.template;
              ue.isPlainObject(ct) && (ze.template = ct, ze._template = ct.layout, ze._dataTemplate = ct.data), at("autotypenumbers");
              var et = ue.coerceFont(at, "font"), dt = et.size;
              ue.coerceFont(at, "title.font", et, { overrideDflt: { size: Math.round(1.4 * dt) } }), at("title.text", ze._dfltTitle.plot), at("title.xref");
              var Je = at("title.yref");
              at("title.pad.t"), at("title.pad.r"), at("title.pad.b"), at("title.pad.l");
              var it = at("title.automargin");
              at("title.x"), at("title.xanchor"), at("title.y"), at("title.yanchor"), at("title.subtitle.text", ze._dfltTitle.subtitle), ue.coerceFont(at, "title.subtitle.font", et, { overrideDflt: { size: Math.round(0.7 * ze.title.font.size) } }), it && (Je === "paper" && (ze.title.y !== 0 && (ze.title.y = 1), ze.title.yanchor === "auto" && (ze.title.yanchor = ze.title.y === 0 ? "top" : "bottom")), Je === "container" && (ze.title.y === "auto" && (ze.title.y = 1), ze.title.yanchor === "auto" && (ze.title.yanchor = ze.title.y < 0.5 ? "bottom" : "top"))), at("uniformtext.mode") && at("uniformtext.minsize"), at("autosize", !(Ge.width && Ge.height)), at("width"), at("height"), at("minreducedwidth"), at("minreducedheight"), at("margin.l"), at("margin.r"), at("margin.t"), at("margin.b"), at("margin.pad"), at("margin.autoexpand"), Ge.width && Ge.height && Se.sanitizeMargins(ze), se.getComponentMethod("grid", "sizeDefaults")(Ge, ze), at("paper_bgcolor"), at("separators", We.decimal + We.thousands), at("hidesources"), at("colorway"), at("datarevision");
              var st = at("uirevision");
              at("editrevision", st), at("selectionrevision", st), se.getComponentMethod("modebar", "supplyLayoutDefaults")(Ge, ze), se.getComponentMethod("shapes", "supplyDrawNewShapeDefaults")(Ge, ze, at), se.getComponentMethod("selections", "supplyDrawNewSelectionDefaults")(Ge, ze, at), at("meta"), ue.isPlainObject(Ge.transition) && (at("transition.duration"), at("transition.easing"), at("transition.ordering")), se.getComponentMethod("calendars", "handleDefaults")(Ge, ze, "calendar"), se.getComponentMethod("fx", "supplyLayoutGlobalDefaults")(Ge, ze, at), ue.coerce(Ge, ze, pe, "scattermode");
            }, Se.plotAutoSize = function(Ge, ze, We) {
              var at, ct, et = Ge._context || {}, dt = et.frameMargins, Je = ue.isPlotDiv(Ge);
              if (Je && Ge.emit("plotly_autosize"), et.fillFrame) at = window.innerWidth, ct = window.innerHeight, document.body.style.overflow = "hidden";
              else {
                var it = Je ? window.getComputedStyle(Ge) : {};
                if (at = De(it.width) || De(it.maxWidth) || We.width, ct = De(it.height) || De(it.maxHeight) || We.height, ae(dt) && dt > 0) {
                  var st = 1 - 2 * dt;
                  at = Math.round(st * at), ct = Math.round(st * ct);
                }
              }
              var gt = Se.layoutAttributes.width.min, bt = Se.layoutAttributes.height.min;
              at < gt && (at = gt), ct < bt && (ct = bt);
              var yt = !ze.width && Math.abs(We.width - at) > 1, St = !ze.height && Math.abs(We.height - ct) > 1;
              (St || yt) && (yt && (We.width = at), St && (We.height = ct)), Ge._initialAutoSize || (Ge._initialAutoSize = { width: at, height: ct }), Se.sanitizeMargins(We);
            }, Se.supplyLayoutModuleDefaults = function(Ge, ze, We, at) {
              var ct, et, dt, Je = se.componentsRegistry, it = ze._basePlotModules, st = se.subplotsRegistry.cartesian;
              for (ct in Je) (dt = Je[ct]).includeBasePlot && dt.includeBasePlot(Ge, ze);
              for (var gt in it.length || it.push(st), ze._has("cartesian") && (se.getComponentMethod("grid", "contentDefaults")(Ge, ze), st.finalizeSubplots(Ge, ze)), ze._subplots) ze._subplots[gt].sort(ue.subplotSort);
              for (et = 0; et < it.length; et++) (dt = it[et]).supplyLayoutDefaults && dt.supplyLayoutDefaults(Ge, ze, We);
              var bt = ze._modules;
              for (et = 0; et < bt.length; et++) (dt = bt[et]).supplyLayoutDefaults && dt.supplyLayoutDefaults(Ge, ze, We);
              var yt = ze._transformModules;
              for (et = 0; et < yt.length; et++) (dt = yt[et]).supplyLayoutDefaults && dt.supplyLayoutDefaults(Ge, ze, We, at);
              for (ct in Je) (dt = Je[ct]).supplyLayoutDefaults && dt.supplyLayoutDefaults(Ge, ze, We);
            }, Se.purge = function(Ge) {
              var ze = Ge._fullLayout || {};
              ze._glcontainer !== void 0 && (ze._glcontainer.selectAll(".gl-canvas").remove(), ze._glcontainer.remove(), ze._glcanvas = null), ze._modeBar && ze._modeBar.destroy(), Ge._transitionData && (Ge._transitionData._interruptCallbacks && (Ge._transitionData._interruptCallbacks.length = 0), Ge._transitionData._animationRaf && window.cancelAnimationFrame(Ge._transitionData._animationRaf)), ue.clearThrottle(), ue.clearResponsive(Ge), delete Ge.data, delete Ge.layout, delete Ge._fullData, delete Ge._fullLayout, delete Ge.calcdata, delete Ge.empty, delete Ge.fid, delete Ge.undoqueue, delete Ge.undonum, delete Ge.autoplay, delete Ge.changed, delete Ge._promises, delete Ge._redrawTimer, delete Ge._hmlumcount, delete Ge._hmpixcount, delete Ge._transitionData, delete Ge._transitioning, delete Ge._initialAutoSize, delete Ge._transitioningWithDuration, delete Ge._dragging, delete Ge._dragged, delete Ge._dragdata, delete Ge._hoverdata, delete Ge._snapshotInProgress, delete Ge._editing, delete Ge._mouseDownTime, delete Ge._legendMouseDownTime, Ge.removeAllListeners && Ge.removeAllListeners();
            }, Se.style = function(Ge) {
              var ze, We = Ge._fullLayout._visibleModules, at = [];
              for (ze = 0; ze < We.length; ze++) {
                var ct = We[ze];
                ct.style && ue.pushUnique(at, ct.style);
              }
              for (ze = 0; ze < at.length; ze++) at[ze](Ge);
            }, Se.sanitizeMargins = function(Ge) {
              if (Ge && Ge.margin) {
                var ze, We = Ge.width, at = Ge.height, ct = Ge.margin, et = We - (ct.l + ct.r), dt = at - (ct.t + ct.b);
                et < 0 && (ze = (We - 1) / (ct.l + ct.r), ct.l = Math.floor(ze * ct.l), ct.r = Math.floor(ze * ct.r)), dt < 0 && (ze = (at - 1) / (ct.t + ct.b), ct.t = Math.floor(ze * ct.t), ct.b = Math.floor(ze * ct.b));
              }
            }, Se.clearAutoMarginIds = function(Ge) {
              Ge._fullLayout._pushmarginIds = {};
            }, Se.allowAutoMargin = function(Ge, ze) {
              Ge._fullLayout._pushmarginIds[ze] = 1;
            }, Se.autoMargin = function(Ge, ze, We) {
              var at = Ge._fullLayout, ct = at.width, et = at.height, dt = at.margin, Je = at.minreducedwidth, it = at.minreducedheight, st = ue.constrain(ct - dt.l - dt.r, 2, Je), gt = ue.constrain(et - dt.t - dt.b, 2, it), bt = Math.max(0, ct - st), yt = Math.max(0, et - gt), St = at._pushmargin, Lt = at._pushmarginIds;
              if (dt.autoexpand !== !1) {
                if (We) {
                  var Rt = We.pad;
                  if (Rt === void 0 && (Rt = Math.min(12, dt.l, dt.r, dt.t, dt.b)), bt) {
                    var vt = (We.l + We.r) / bt;
                    vt > 1 && (We.l /= vt, We.r /= vt);
                  }
                  if (yt) {
                    var $t = (We.t + We.b) / yt;
                    $t > 1 && (We.t /= $t, We.b /= $t);
                  }
                  var Pt = We.xl !== void 0 ? We.xl : We.x, Xt = We.xr !== void 0 ? We.xr : We.x, Nt = We.yt !== void 0 ? We.yt : We.y, Gt = We.yb !== void 0 ? We.yb : We.y;
                  St[ze] = { l: { val: Pt, size: We.l + Rt }, r: { val: Xt, size: We.r + Rt }, b: { val: Gt, size: We.b + Rt }, t: { val: Nt, size: We.t + Rt } }, Lt[ze] = 1;
                } else delete St[ze], delete Lt[ze];
                if (!at._replotting) return Se.doAutoMargin(Ge);
              }
            }, Se.doAutoMargin = function(Ge) {
              var ze = Ge._fullLayout, We = ze.width, at = ze.height;
              ze._size || (ze._size = {}), Pe(ze);
              var ct = ze._size, et = ze.margin, dt = { t: 0, b: 0, l: 0, r: 0 }, Je = ue.extendFlat({}, ct), it = et.l, st = et.r, gt = et.t, bt = et.b, yt = ze._pushmargin, St = ze._pushmarginIds, Lt = ze.minreducedwidth, Rt = ze.minreducedheight;
              if (et.autoexpand !== !1) {
                for (var vt in yt) St[vt] || delete yt[vt];
                var $t = Ge._fullLayout._reservedMargin;
                for (var Pt in $t) for (var Xt in $t[Pt]) {
                  var Nt = $t[Pt][Xt];
                  dt[Xt] = Math.max(dt[Xt], Nt);
                }
                for (var Gt in yt.base = { l: { val: 0, size: it }, r: { val: 1, size: st }, t: { val: 1, size: gt }, b: { val: 0, size: bt } }, dt) {
                  var Ft = 0;
                  for (var Tt in yt) Tt !== "base" && ae(yt[Tt][Gt].size) && (Ft = yt[Tt][Gt].size > Ft ? yt[Tt][Gt].size : Ft);
                  var It = Math.max(0, et[Gt] - Ft);
                  dt[Gt] = Math.max(0, dt[Gt] - It);
                }
                for (var Ct in yt) {
                  var Kt = yt[Ct].l || {}, Vt = yt[Ct].b || {}, Yt = Kt.val, sn = Kt.size, hn = Vt.val, Qt = Vt.size, Ut = We - dt.r - dt.l, Wt = at - dt.t - dt.b;
                  for (var rn in yt) {
                    if (ae(sn) && yt[rn].r) {
                      var vn = yt[rn].r.val, dn = yt[rn].r.size;
                      if (vn > Yt) {
                        var Ht = (sn * vn + (dn - Ut) * Yt) / (vn - Yt), jt = (dn * (1 - Yt) + (sn - Ut) * (1 - vn)) / (vn - Yt);
                        Ht + jt > it + st && (it = Ht, st = jt);
                      }
                    }
                    if (ae(Qt) && yt[rn].t) {
                      var cn = yt[rn].t.val, wn = yt[rn].t.size;
                      if (cn > hn) {
                        var Rn = (Qt * cn + (wn - Wt) * hn) / (cn - hn), Pn = (wn * (1 - hn) + (Qt - Wt) * (1 - cn)) / (cn - hn);
                        Rn + Pn > bt + gt && (bt = Rn, gt = Pn);
                      }
                    }
                  }
                }
              }
              var Ie = ue.constrain(We - et.l - et.r, 2, Lt), rt = ue.constrain(at - et.t - et.b, 2, Rt), ft = Math.max(0, We - Ie), pt = Math.max(0, at - rt);
              if (ft) {
                var wt = (it + st) / ft;
                wt > 1 && (it /= wt, st /= wt);
              }
              if (pt) {
                var Bt = (bt + gt) / pt;
                Bt > 1 && (bt /= Bt, gt /= Bt);
              }
              if (ct.l = Math.round(it) + dt.l, ct.r = Math.round(st) + dt.r, ct.t = Math.round(gt) + dt.t, ct.b = Math.round(bt) + dt.b, ct.p = Math.round(et.pad), ct.w = Math.round(We) - ct.l - ct.r, ct.h = Math.round(at) - ct.t - ct.b, !ze._replotting && (Se.didMarginChange(Je, ct) || function(Dt) {
                if ("_redrawFromAutoMarginCount" in Dt._fullLayout) return !1;
                var Ot = he.list(Dt, "", !0);
                for (var un in Ot) if (Ot[un].autoshift || Ot[un].shift) return !0;
                return !1;
              }(Ge))) {
                "_redrawFromAutoMarginCount" in ze ? ze._redrawFromAutoMarginCount++ : ze._redrawFromAutoMarginCount = 1;
                var At = 3 * (1 + Object.keys(St).length);
                if (ze._redrawFromAutoMarginCount < At) return se.call("_doPlot", Ge);
                ze._size = Je, ue.warn("Too many auto-margin redraws.");
              }
              (function(Dt) {
                var Ot = he.list(Dt, "", !0);
                ["_adjustTickLabelsOverflow", "_hideCounterAxisInsideTickLabels"].forEach(function(un) {
                  for (var yn = 0; yn < Ot.length; yn++) {
                    var _n = Ot[yn][un];
                    _n && _n();
                  }
                });
              })(Ge);
            };
            var $e = ["l", "r", "t", "b", "p", "w", "h"];
            function qe(Ge, ze, We) {
              var at = !1, ct = [Se.previousPromises, function() {
                if (Ge._transitionData) return Ge._transitioning = !1, function(dt) {
                  var Je = Promise.resolve();
                  if (!dt) return Je;
                  for (; dt.length; ) Je = Je.then(dt.shift());
                  return Je;
                }(Ge._transitionData._interruptCallbacks);
              }, We.prepareFn, Se.rehover, Se.reselect, function() {
                return Ge.emit("plotly_transitioning", []), new Promise(function(dt) {
                  Ge._transitioning = !0, ze.duration > 0 && (Ge._transitioningWithDuration = !0), Ge._transitionData._interruptCallbacks.push(function() {
                    at = !0;
                  }), We.redraw && Ge._transitionData._interruptCallbacks.push(function() {
                    return se.call("redraw", Ge);
                  }), Ge._transitionData._interruptCallbacks.push(function() {
                    Ge.emit("plotly_transitioninterrupted", []);
                  });
                  var Je = 0, it = 0;
                  function st() {
                    return Je++, function() {
                      var gt;
                      it++, at || it !== Je || (gt = dt, Ge._transitionData && (function(bt) {
                        if (bt) for (; bt.length; ) bt.shift();
                      }(Ge._transitionData._interruptCallbacks), Promise.resolve().then(function() {
                        if (We.redraw) return se.call("redraw", Ge);
                      }).then(function() {
                        Ge._transitioning = !1, Ge._transitioningWithDuration = !1, Ge.emit("plotly_transitioned", []);
                      }).then(gt)));
                    };
                  }
                  We.runFn(st), setTimeout(st());
                });
              }], et = ue.syncOrAsync(ct, Ge);
              return et && et.then || (et = Promise.resolve()), et.then(function() {
                return Ge;
              });
            }
            Se.didMarginChange = function(Ge, ze) {
              for (var We = 0; We < $e.length; We++) {
                var at = $e[We], ct = Ge[at], et = ze[at];
                if (!ae(ct) || Math.abs(et - ct) > 1) return !0;
              }
              return !1;
            }, Se.graphJson = function(Ge, ze, We, at, ct, et) {
              (ct && ze && !Ge._fullData || ct && !ze && !Ge._fullLayout) && Se.supplyDefaults(Ge);
              var dt = ct ? Ge._fullData : Ge.data, Je = ct ? Ge._fullLayout : Ge.layout, it = (Ge._transitionData || {})._frames;
              function st(yt, St) {
                if (typeof yt == "function") return St ? "_function_" : null;
                if (ue.isPlainObject(yt)) {
                  var Lt, Rt = {};
                  return Object.keys(yt).sort().forEach(function(Xt) {
                    if (["_", "["].indexOf(Xt.charAt(0)) === -1) if (typeof yt[Xt] != "function") {
                      if (We === "keepdata") {
                        if (Xt.substr(Xt.length - 3) === "src") return;
                      } else if (We === "keepstream") {
                        if (typeof (Lt = yt[Xt + "src"]) == "string" && Lt.indexOf(":") > 0 && !ue.isPlainObject(yt.stream)) return;
                      } else if (We !== "keepall" && typeof (Lt = yt[Xt + "src"]) == "string" && Lt.indexOf(":") > 0) return;
                      Rt[Xt] = st(yt[Xt], St);
                    } else St && (Rt[Xt] = "_function");
                  }), Rt;
                }
                var vt = Array.isArray(yt), $t = ue.isTypedArray(yt);
                if ((vt || $t) && yt.dtype && yt.shape) {
                  var Pt = yt.bdata;
                  return st({ dtype: yt.dtype, shape: yt.shape, bdata: ue.isArrayBuffer(Pt) ? oe.encode(Pt) : Pt }, St);
                }
                return vt ? yt.map(function(Xt) {
                  return st(Xt, St);
                }) : $t ? ue.simpleMap(yt, ue.identity) : ue.isJSDate(yt) ? ue.ms2DateTimeLocal(+yt) : yt;
              }
              var gt = { data: (dt || []).map(function(yt) {
                var St = st(yt);
                return ze && delete St.fit, St;
              }) };
              if (!ze && (gt.layout = st(Je), ct)) {
                var bt = Je._size;
                gt.layout.computed = { margin: { b: bt.b, l: bt.l, r: bt.r, t: bt.t } };
              }
              return it && (gt.frames = st(it)), et && (gt.config = st(Ge._context, !0)), at === "object" ? gt : JSON.stringify(gt);
            }, Se.modifyFrames = function(Ge, ze) {
              var We, at, ct, et = Ge._transitionData._frames, dt = Ge._transitionData._frameHash;
              for (We = 0; We < ze.length; We++) switch ((at = ze[We]).type) {
                case "replace":
                  ct = at.value;
                  var Je = (et[at.index] || {}).name, it = ct.name;
                  et[at.index] = dt[it] = ct, it !== Je && (delete dt[Je], dt[it] = ct);
                  break;
                case "insert":
                  dt[(ct = at.value).name] = ct, et.splice(at.index, 0, ct);
                  break;
                case "delete":
                  delete dt[(ct = et[at.index]).name], et.splice(at.index, 1);
              }
              return Promise.resolve();
            }, Se.computeFrame = function(Ge, ze) {
              var We, at, ct, et, dt = Ge._transitionData._frameHash;
              if (!ze) throw new Error("computeFrame must be given a string frame name");
              var Je = dt[ze.toString()];
              if (!Je) return !1;
              for (var it = [Je], st = [Je.name]; Je.baseframe && (Je = dt[Je.baseframe.toString()]) && st.indexOf(Je.name) === -1; ) it.push(Je), st.push(Je.name);
              for (var gt = {}; Je = it.pop(); ) if (Je.layout && (gt.layout = Se.extendLayout(gt.layout, Je.layout)), Je.data) {
                if (gt.data || (gt.data = []), !(at = Je.traces)) for (at = [], We = 0; We < Je.data.length; We++) at[We] = We;
                for (gt.traces || (gt.traces = []), We = 0; We < Je.data.length; We++) (ct = at[We]) != null && ((et = gt.traces.indexOf(ct)) === -1 && (et = gt.data.length, gt.traces[et] = ct), gt.data[et] = Se.extendTrace(gt.data[et], Je.data[We]));
              }
              return gt;
            }, Se.recomputeFrameHash = function(Ge) {
              for (var ze = Ge._transitionData._frameHash = {}, We = Ge._transitionData._frames, at = 0; at < We.length; at++) {
                var ct = We[at];
                ct && ct.name && (ze[ct.name] = ct);
              }
            }, Se.extendObjectWithContainers = function(Ge, ze, We) {
              var at, ct, et, dt, Je, it, st, gt = ue.extendDeepNoArrays({}, ze || {}), bt = ue.expandObjectPaths(gt), yt = {};
              if (We && We.length) for (et = 0; et < We.length; et++) (ct = (at = ue.nestedProperty(bt, We[et])).get()) === void 0 ? ue.nestedProperty(yt, We[et]).set(null) : (at.set(null), ue.nestedProperty(yt, We[et]).set(ct));
              if (Ge = ue.extendDeepNoArrays(Ge || {}, bt), We && We.length) {
                for (et = 0; et < We.length; et++) if (it = ue.nestedProperty(yt, We[et]).get()) {
                  for (st = (Je = ue.nestedProperty(Ge, We[et])).get(), Array.isArray(st) || (st = [], Je.set(st)), dt = 0; dt < it.length; dt++) {
                    var St = it[dt];
                    st[dt] = St === null ? null : Se.extendObjectWithContainers(st[dt], St);
                  }
                  Je.set(st);
                }
              }
              return Ge;
            }, Se.dataArrayContainers = ["transforms", "dimensions"], Se.layoutArrayContainers = se.layoutArrayContainers, Se.extendTrace = function(Ge, ze) {
              return Se.extendObjectWithContainers(Ge, ze, Se.dataArrayContainers);
            }, Se.extendLayout = function(Ge, ze) {
              return Se.extendObjectWithContainers(Ge, ze, Se.layoutArrayContainers);
            }, Se.transition = function(Ge, ze, We, at, ct, et) {
              var dt = { redraw: ct.redraw }, Je = {}, it = [];
              return dt.prepareFn = function() {
                for (var st = Array.isArray(ze) ? ze.length : 0, gt = at.slice(0, st), bt = 0; bt < gt.length; bt++) {
                  var yt = gt[bt], St = Ge._fullData[yt]._module;
                  if (St) {
                    if (St.animatable) {
                      var Lt = St.basePlotModule.name;
                      Je[Lt] || (Je[Lt] = []), Je[Lt].push(yt);
                    }
                    Ge.data[gt[bt]] = Se.extendTrace(Ge.data[gt[bt]], ze[bt]);
                  }
                }
                var Rt = ue.expandObjectPaths(ue.extendDeepNoArrays({}, We)), vt = /^[xy]axis[0-9]*$/;
                for (var $t in Rt) vt.test($t) && delete Rt[$t].range;
                Se.extendLayout(Ge.layout, Rt), delete Ge.calcdata, Se.supplyDefaults(Ge), Se.doCalcdata(Ge);
                var Pt = ue.expandObjectPaths(We);
                if (Pt) {
                  var Xt = Ge._fullLayout._plots;
                  for (var Nt in Xt) {
                    var Gt = Xt[Nt], Ft = Gt.xaxis, Tt = Gt.yaxis, It = Ft.range.slice(), Ct = Tt.range.slice(), Kt = null, Vt = null, Yt = null, sn = null;
                    Array.isArray(Pt[Ft._name + ".range"]) ? Kt = Pt[Ft._name + ".range"].slice() : Array.isArray((Pt[Ft._name] || {}).range) && (Kt = Pt[Ft._name].range.slice()), Array.isArray(Pt[Tt._name + ".range"]) ? Vt = Pt[Tt._name + ".range"].slice() : Array.isArray((Pt[Tt._name] || {}).range) && (Vt = Pt[Tt._name].range.slice()), It && Kt && (Ft.r2l(It[0]) !== Ft.r2l(Kt[0]) || Ft.r2l(It[1]) !== Ft.r2l(Kt[1])) && (Yt = { xr0: It, xr1: Kt }), Ct && Vt && (Tt.r2l(Ct[0]) !== Tt.r2l(Vt[0]) || Tt.r2l(Ct[1]) !== Tt.r2l(Vt[1])) && (sn = { yr0: Ct, yr1: Vt }), (Yt || sn) && it.push(ue.extendFlat({ plotinfo: Gt }, Yt, sn));
                  }
                }
                return Promise.resolve();
              }, dt.runFn = function(st) {
                var gt, bt, yt = Ge._fullLayout._basePlotModules, St = it.length;
                if (We) for (bt = 0; bt < yt.length; bt++) yt[bt].transitionAxes && yt[bt].transitionAxes(Ge, it, et, st);
                for (var Lt in St ? ((gt = ue.extendFlat({}, et)).duration = 0, delete Je.cartesian) : gt = et, Je) {
                  var Rt = Je[Lt];
                  Ge._fullData[Rt[0]]._module.basePlotModule.plot(Ge, Rt, gt, st);
                }
              }, qe(Ge, et, dt);
            }, Se.transitionFromReact = function(Ge, ze, We, at) {
              var ct = Ge._fullLayout, et = ct.transition, dt = {}, Je = [];
              return dt.prepareFn = function() {
                var it = ct._plots;
                for (var st in dt.redraw = !1, ze.anim === "some" && (dt.redraw = !0), We.anim === "some" && (dt.redraw = !0), it) {
                  var gt = it[st], bt = gt.xaxis, yt = gt.yaxis, St = at[bt._name].range.slice(), Lt = at[yt._name].range.slice(), Rt = bt.range.slice(), vt = yt.range.slice();
                  bt.setScale(), yt.setScale();
                  var $t = null, Pt = null;
                  bt.r2l(St[0]) === bt.r2l(Rt[0]) && bt.r2l(St[1]) === bt.r2l(Rt[1]) || ($t = { xr0: St, xr1: Rt }), yt.r2l(Lt[0]) === yt.r2l(vt[0]) && yt.r2l(Lt[1]) === yt.r2l(vt[1]) || (Pt = { yr0: Lt, yr1: vt }), ($t || Pt) && Je.push(ue.extendFlat({ plotinfo: gt }, $t, Pt));
                }
                return Promise.resolve();
              }, dt.runFn = function(it) {
                for (var st, gt, bt, yt = Ge._fullData, St = Ge._fullLayout._basePlotModules, Lt = [], Rt = 0; Rt < yt.length; Rt++) Lt.push(Rt);
                function vt() {
                  if (Ge._fullLayout) for (var Pt = 0; Pt < St.length; Pt++) St[Pt].transitionAxes && St[Pt].transitionAxes(Ge, Je, st, it);
                }
                function $t() {
                  if (Ge._fullLayout) for (var Pt = 0; Pt < St.length; Pt++) St[Pt].plot(Ge, bt, gt, it);
                }
                Je.length && ze.anim ? et.ordering === "traces first" ? (st = ue.extendFlat({}, et, { duration: 0 }), bt = Lt, gt = et, setTimeout(vt, et.duration), $t()) : (st = et, bt = null, gt = ue.extendFlat({}, et, { duration: 0 }), setTimeout($t, st.duration), vt()) : Je.length ? (st = et, vt()) : ze.anim && (bt = Lt, gt = et, $t());
              }, qe(Ge, et, dt);
            }, Se.doCalcdata = function(Ge, ze) {
              var We, at, ct, et, dt = he.list(Ge), Je = Ge._fullData, it = Ge._fullLayout, st = new Array(Je.length), gt = (Ge.calcdata || []).slice();
              for (Ge.calcdata = st, it._numBoxes = 0, it._numViolins = 0, it._violinScaleGroupStats = {}, Ge._hmpixcount = 0, Ge._hmlumcount = 0, it._piecolormap = {}, it._sunburstcolormap = {}, it._treemapcolormap = {}, it._iciclecolormap = {}, it._funnelareacolormap = {}, ct = 0; ct < Je.length; ct++) Array.isArray(ze) && ze.indexOf(ct) === -1 && (st[ct] = gt[ct]);
              for (ct = 0; ct < Je.length; ct++) (We = Je[ct])._arrayAttrs = ce.findArrayAttributes(We), We._extremes = {};
              var bt = it._subplots.polar || [];
              for (ct = 0; ct < bt.length; ct++) dt.push(it[bt[ct]].radialaxis, it[bt[ct]].angularaxis);
              for (var yt in it._colorAxes) {
                var St = it[yt];
                St.cauto !== !1 && (delete St.cmin, delete St.cmax);
              }
              var Lt = !1;
              function Rt(Pt) {
                if (We = Je[Pt], at = We._module, We.visible === !0 && We.transforms) {
                  if (at && at.calc) {
                    var Xt = at.calc(Ge, We);
                    Xt[0] && Xt[0].t && Xt[0].t._scene && delete Xt[0].t._scene.dirty;
                  }
                  for (et = 0; et < We.transforms.length; et++) {
                    var Nt = We.transforms[et];
                    (at = we[Nt.type]) && at.calcTransform && (We._hasCalcTransform = !0, Lt = !0, at.calcTransform(Ge, We, Nt));
                  }
                }
              }
              function vt(Pt, Xt) {
                if (We = Je[Pt], !!(at = We._module).isContainer === Xt) {
                  var Nt = [];
                  if (We.visible === !0 && We._length !== 0) {
                    delete We._indexToPoints;
                    var Gt = We.transforms || [];
                    for (et = Gt.length - 1; et >= 0; et--) if (Gt[et].enabled) {
                      We._indexToPoints = Gt[et]._indexToPoints;
                      break;
                    }
                    at && at.calc && (Nt = at.calc(Ge, We));
                  }
                  Array.isArray(Nt) && Nt[0] || (Nt = [{ x: fe, y: fe }]), Nt[0].t || (Nt[0].t = {}), Nt[0].trace = We, st[Pt] = Nt;
                }
              }
              for (Ve(dt, Je, it), ct = 0; ct < Je.length; ct++) vt(ct, !0);
              for (ct = 0; ct < Je.length; ct++) Rt(ct);
              for (Lt && Ve(dt, Je, it), ct = 0; ct < Je.length; ct++) vt(ct, !0);
              for (ct = 0; ct < Je.length; ct++) vt(ct, !1);
              Ze(Ge);
              var $t = function(Pt, Xt) {
                var Nt, Gt, Ft, Tt, It, Ct = [];
                function Kt(un, yn, _n) {
                  var gn = yn._id.charAt(0);
                  if (un === "histogram2dcontour") {
                    var bn = yn._counterAxes[0], an = he.getFromId(Xt, bn), Zt = gn === "x" || bn === "x" && an.type === "category", Mn = gn === "y" || bn === "y" && an.type === "category";
                    return function(Ln, In) {
                      return Ln === 0 || In === 0 || Zt && Ln === _n[In].length - 1 || Mn && In === _n.length - 1 ? -1 : (gn === "y" ? In : Ln) - 1;
                    };
                  }
                  return function(Ln, In) {
                    return gn === "y" ? In : Ln;
                  };
                }
                var Vt = { min: function(un) {
                  return ue.aggNums(Math.min, null, un);
                }, max: function(un) {
                  return ue.aggNums(Math.max, null, un);
                }, sum: function(un) {
                  return ue.aggNums(function(yn, _n) {
                    return yn + _n;
                  }, null, un);
                }, total: function(un) {
                  return ue.aggNums(function(yn, _n) {
                    return yn + _n;
                  }, null, un);
                }, mean: function(un) {
                  return ue.mean(un);
                }, "geometric mean": function(un) {
                  return ue.geometricMean(un);
                }, median: function(un) {
                  return ue.median(un);
                } };
                function Yt(un, yn) {
                  return un[1] - yn[1];
                }
                function sn(un, yn) {
                  return yn[1] - un[1];
                }
                for (Nt = 0; Nt < Pt.length; Nt++) {
                  var hn = Pt[Nt];
                  if (hn.type === "category") {
                    var Qt = hn.categoryorder.match(Xe);
                    if (Qt) {
                      var Ut = Qt[1], Wt = Qt[2], rn = hn._id.charAt(0), vn = rn === "x", dn = [];
                      for (Gt = 0; Gt < hn._categories.length; Gt++) dn.push([hn._categories[Gt], []]);
                      for (Gt = 0; Gt < hn._traceIndices.length; Gt++) {
                        var Ht = hn._traceIndices[Gt], jt = Xt._fullData[Ht];
                        if (jt.visible === !0) {
                          var cn = jt.type;
                          se.traceIs(jt, "histogram") && (delete jt._xautoBinFinished, delete jt._yautoBinFinished);
                          var wn = cn === "splom", Rn = cn === "scattergl", Pn = Xt.calcdata[Ht];
                          for (Ft = 0; Ft < Pn.length; Ft++) {
                            var Ie, rt, ft = Pn[Ft];
                            if (wn) {
                              var pt = jt._axesDim[hn._id];
                              if (!vn) {
                                var wt = jt._diag[pt][0];
                                wt && (hn = Xt._fullLayout[he.id2name(wt)]);
                              }
                              var Bt = ft.trace.dimensions[pt].values;
                              for (Tt = 0; Tt < Bt.length; Tt++) for (Ie = hn._categoriesMap[Bt[Tt]], It = 0; It < ft.trace.dimensions.length; It++) if (It !== pt) {
                                var At = ft.trace.dimensions[It];
                                dn[Ie][1].push(At.values[Tt]);
                              }
                            } else if (Rn) {
                              for (Tt = 0; Tt < ft.t.x.length; Tt++) vn ? (Ie = ft.t.x[Tt], rt = ft.t.y[Tt]) : (Ie = ft.t.y[Tt], rt = ft.t.x[Tt]), dn[Ie][1].push(rt);
                              ft.t && ft.t._scene && delete ft.t._scene.dirty;
                            } else if (ft.hasOwnProperty("z")) {
                              rt = ft.z;
                              var Dt = Kt(jt.type, hn, rt);
                              for (Tt = 0; Tt < rt.length; Tt++) for (It = 0; It < rt[Tt].length; It++) (Ie = Dt(It, Tt)) + 1 && dn[Ie][1].push(rt[Tt][It]);
                            } else for ((Ie = ft.p) === void 0 && (Ie = ft[rn]), (rt = ft.s) === void 0 && (rt = ft.v), rt === void 0 && (rt = vn ? ft.y : ft.x), Array.isArray(rt) || (rt = rt === void 0 ? [] : [rt]), Tt = 0; Tt < rt.length; Tt++) dn[Ie][1].push(rt[Tt]);
                          }
                        }
                      }
                      hn._categoriesValue = dn;
                      var Ot = [];
                      for (Gt = 0; Gt < dn.length; Gt++) Ot.push([dn[Gt][0], Vt[Ut](dn[Gt][1])]);
                      Ot.sort(Wt === "descending" ? sn : Yt), hn._categoriesAggregatedValue = Ot, hn._initialCategories = Ot.map(function(un) {
                        return un[0];
                      }), Ct = Ct.concat(hn.sortByInitialCategories());
                    }
                  }
                }
                return Ct;
              }(dt, Ge);
              if ($t.length) {
                for (it._numBoxes = 0, it._numViolins = 0, ct = 0; ct < $t.length; ct++) vt($t[ct], !0);
                for (ct = 0; ct < $t.length; ct++) vt($t[ct], !1);
                Ze(Ge);
              }
              se.getComponentMethod("fx", "calc")(Ge), se.getComponentMethod("errorbars", "calc")(Ge);
            };
            var Xe = /(total|sum|min|max|mean|geometric mean|median) (ascending|descending)/;
            function Ve(Ge, ze, We) {
              var at = {};
              function ct(it) {
                it.clearCalc(), it.type === "multicategory" && it.setupMultiCategory(ze), at[it._id] = 1;
              }
              ue.simpleMap(Ge, ct);
              for (var et = We._axisMatchGroups || [], dt = 0; dt < et.length; dt++) for (var Je in et[dt]) at[Je] || ct(We[he.id2name(Je)]);
            }
            function Ze(Ge) {
              var ze, We, at, ct = Ge._fullLayout, et = ct._visibleModules, dt = {};
              for (We = 0; We < et.length; We++) {
                var Je = et[We], it = Je.crossTraceCalc;
                if (it) {
                  var st = Je.basePlotModule.name;
                  dt[st] ? ue.pushUnique(dt[st], it) : dt[st] = [it];
                }
              }
              for (at in dt) {
                var gt = dt[at], bt = ct._subplots[at];
                if (Array.isArray(bt)) for (ze = 0; ze < bt.length; ze++) {
                  var yt = bt[ze], St = at === "cartesian" ? ct._plots[yt] : ct[yt];
                  for (We = 0; We < gt.length; We++) gt[We](Ge, St, yt);
                }
                else for (We = 0; We < gt.length; We++) gt[We](Ge);
              }
            }
            Se.rehover = function(Ge) {
              Ge._fullLayout._rehover && Ge._fullLayout._rehover();
            }, Se.redrag = function(Ge) {
              Ge._fullLayout._redrag && Ge._fullLayout._redrag();
            }, Se.reselect = function(Ge) {
              var ze = Ge._fullLayout, We = (Ge.layout || {}).selections, at = ze._previousSelections;
              ze._previousSelections = We;
              var ct = ze._reselect || JSON.stringify(We) !== JSON.stringify(at);
              se.getComponentMethod("selections", "reselect")(Ge, ct);
            }, Se.generalUpdatePerTraceModule = function(Ge, ze, We, at) {
              var ct, et = ze.traceHash, dt = {};
              for (ct = 0; ct < We.length; ct++) {
                var Je = We[ct], it = Je[0].trace;
                it.visible && (dt[it.type] = dt[it.type] || [], dt[it.type].push(Je));
              }
              for (var st in et) if (!dt[st]) {
                var gt = et[st][0];
                gt[0].trace.visible = !1, dt[st] = [gt];
              }
              for (var bt in dt) {
                var yt = dt[bt];
                yt[0][0].trace._module.plot(Ge, ze, ue.filterVisible(yt), at);
              }
              ze.traceHash = dt;
            }, Se.plotBasePlot = function(Ge, ze, We, at, ct) {
              var et = se.getModule(Ge), dt = ve(ze.calcdata, et)[0];
              et.plot(ze, dt, at, ct);
            }, Se.cleanBasePlot = function(Ge, ze, We, at, ct) {
              var et = ct._has && ct._has(Ge), dt = We._has && We._has(Ge);
              et && !dt && ct["_" + Ge + "layer"].selectAll("g.trace").remove();
            };
          }, 26484: function(G) {
            G.exports = { attr: "subplot", name: "polar", axisNames: ["angularaxis", "radialaxis"], axisName2dataArray: { angularaxis: "theta", radialaxis: "r" }, layerNames: ["draglayer", "plotbg", "backplot", "angular-grid", "radial-grid", "frontplot", "angular-line", "radial-line", "angular-axis", "radial-axis"], radialDragBoxSize: 50, angularDragBoxSize: 30, cornerLen: 25, cornerHalfWidth: 2, MINDRAG: 8, MINZOOM: 20, OFFEDGE: 20 };
          }, 95928: function(G, J, Z) {
            var te = Z(34809), ne = Z(80899).tester, ee = te.findIndexOfMin, ae = te.isAngleInsideSector, oe = te.angleDelta, se = te.angleDist;
            function ce(fe, he, me, pe) {
              var ye, ge, ve = pe[0], be = pe[1], Ce = ue(Math.sin(he) - Math.sin(fe)), Se = ue(Math.cos(he) - Math.cos(fe)), we = Math.tan(me), Ee = ue(1 / we), xe = Ce / Se, Le = be - xe * ve;
              return Ee ? Ce && Se ? ge = we * (ye = Le / (we - xe)) : Se ? (ye = be * Ee, ge = be) : (ye = ve, ge = ve * we) : Ce && Se ? (ye = 0, ge = Le) : Se ? (ye = 0, ge = be) : ye = ge = NaN, [ye, ge];
            }
            function le(fe, he, me, pe) {
              return te.isFullCircle([he, me]) ? function(ye, ge) {
                var ve, be = ge.length, Ce = new Array(be + 1);
                for (ve = 0; ve < be; ve++) {
                  var Se = ge[ve];
                  Ce[ve] = [ye * Math.cos(Se), ye * Math.sin(Se)];
                }
                return Ce[ve] = Ce[0].slice(), Ce;
              }(fe, pe) : function(ye, ge, ve, be) {
                var Ce, Se, we = be.length, Ee = [];
                function xe(qe) {
                  return [ye * Math.cos(qe), ye * Math.sin(qe)];
                }
                function Le(qe, Xe, Ve) {
                  return ce(qe, Xe, Ve, xe(qe));
                }
                function Qe(qe) {
                  return te.mod(qe, we);
                }
                function ke(qe) {
                  return ae(qe, [ge, ve]);
                }
                var Me = ee(be, function(qe) {
                  return ke(qe) ? se(qe, ge) : 1 / 0;
                }), Re = Le(be[Me], be[Qe(Me - 1)], ge);
                for (Ee.push(Re), Ce = Me, Se = 0; Se < we; Ce++, Se++) {
                  var De = be[Qe(Ce)];
                  if (!ke(De)) break;
                  Ee.push(xe(De));
                }
                var Pe = ee(be, function(qe) {
                  return ke(qe) ? se(qe, ve) : 1 / 0;
                }), $e = Le(be[Pe], be[Qe(Pe + 1)], ve);
                return Ee.push($e), Ee.push([0, 0]), Ee.push(Ee[0].slice()), Ee;
              }(fe, he, me, pe);
            }
            function ue(fe) {
              return Math.abs(fe) > 1e-10 ? fe : 0;
            }
            function de(fe, he, me) {
              he = he || 0, me = me || 0;
              for (var pe = fe.length, ye = new Array(pe), ge = 0; ge < pe; ge++) {
                var ve = fe[ge];
                ye[ge] = [he + ve[0], me - ve[1]];
              }
              return ye;
            }
            G.exports = { isPtInsidePolygon: function(fe, he, me, pe, ye) {
              if (!ae(he, pe)) return !1;
              var ge, ve;
              me[0] < me[1] ? (ge = me[0], ve = me[1]) : (ge = me[1], ve = me[0]);
              var be = ne(le(ge, pe[0], pe[1], ye)), Ce = ne(le(ve, pe[0], pe[1], ye)), Se = [fe * Math.cos(he), fe * Math.sin(he)];
              return Ce.contains(Se) && !be.contains(Se);
            }, findPolygonOffset: function(fe, he, me, pe) {
              for (var ye = 1 / 0, ge = 1 / 0, ve = le(fe, he, me, pe), be = 0; be < ve.length; be++) {
                var Ce = ve[be];
                ye = Math.min(ye, Ce[0]), ge = Math.min(ge, -Ce[1]);
              }
              return [ye, ge];
            }, findEnclosingVertexAngles: function(fe, he) {
              var me = ee(he, function(ye) {
                var ge = oe(ye, fe);
                return ge > 0 ? ge : 1 / 0;
              }), pe = te.mod(me + 1, he.length);
              return [he[me], he[pe]];
            }, findIntersectionXY: ce, findXYatLength: function(fe, he, me, pe) {
              var ye = -he * me, ge = he * he + 1, ve = 2 * (he * ye - me), be = ye * ye + me * me - fe * fe, Ce = Math.sqrt(ve * ve - 4 * ge * be), Se = (-ve + Ce) / (2 * ge), we = (-ve - Ce) / (2 * ge);
              return [[Se, he * Se + ye + pe], [we, he * we + ye + pe]];
            }, clampTiny: ue, pathPolygon: function(fe, he, me, pe, ye, ge) {
              return "M" + de(le(fe, he, me, pe), ye, ge).join("L");
            }, pathPolygonAnnulus: function(fe, he, me, pe, ye, ge, ve) {
              var be, Ce;
              fe < he ? (be = fe, Ce = he) : (be = he, Ce = fe);
              var Se = de(le(be, me, pe, ye), ge, ve);
              return "M" + de(le(Ce, me, pe, ye), ge, ve).reverse().join("L") + "M" + Se.join("L");
            } };
          }, 31645: function(G, J, Z) {
            var te = Z(4173).fX, ne = Z(34809).counterRegex, ee = Z(35785), ae = Z(26484), oe = ae.attr, se = ae.name, ce = ne(se), le = {};
            le[oe] = { valType: "subplotid", dflt: se, editType: "calc" }, G.exports = { attr: oe, name: se, idRoot: se, idRegex: ce, attrRegex: ce, attributes: le, layoutAttributes: Z(42219), supplyLayoutDefaults: Z(84588), plot: function(ue) {
              for (var de = ue._fullLayout, fe = ue.calcdata, he = de._subplots[se], me = 0; me < he.length; me++) {
                var pe = he[me], ye = te(fe, se, pe), ge = de[pe]._subplot;
                ge || (ge = ee(ue, pe), de[pe]._subplot = ge), ge.plot(ye, de, ue._promises);
              }
            }, clean: function(ue, de, fe, he) {
              for (var me = he._subplots[se] || [], pe = he._has && he._has("gl"), ye = de._has && de._has("gl"), ge = pe && !ye, ve = 0; ve < me.length; ve++) {
                var be = me[ve], Ce = he[be]._subplot;
                if (!de[be] && Ce) for (var Se in Ce.framework.remove(), Ce.layers["radial-axis-title"].remove(), Ce.clipPaths) Ce.clipPaths[Se].remove();
                ge && Ce._scene && (Ce._scene.destroy(), Ce._scene = null);
              }
            }, toSVG: Z(37703).toSVG };
          }, 42219: function(G, J, Z) {
            var te = Z(10229), ne = Z(25829), ee = Z(13792).u, ae = Z(34809).extendFlat, oe = Z(13582).overrideAll, se = oe({ color: ne.color, showline: ae({}, ne.showline, { dflt: !0 }), linecolor: ne.linecolor, linewidth: ne.linewidth, showgrid: ae({}, ne.showgrid, { dflt: !0 }), gridcolor: ne.gridcolor, gridwidth: ne.gridwidth, griddash: ne.griddash }, "plot", "from-root"), ce = oe({ tickmode: ne.minor.tickmode, nticks: ne.nticks, tick0: ne.tick0, dtick: ne.dtick, tickvals: ne.tickvals, ticktext: ne.ticktext, ticks: ne.ticks, ticklen: ne.ticklen, tickwidth: ne.tickwidth, tickcolor: ne.tickcolor, ticklabelstep: ne.ticklabelstep, showticklabels: ne.showticklabels, labelalias: ne.labelalias, showtickprefix: ne.showtickprefix, tickprefix: ne.tickprefix, showticksuffix: ne.showticksuffix, ticksuffix: ne.ticksuffix, showexponent: ne.showexponent, exponentformat: ne.exponentformat, minexponent: ne.minexponent, separatethousands: ne.separatethousands, tickfont: ne.tickfont, tickangle: ne.tickangle, tickformat: ne.tickformat, tickformatstops: ne.tickformatstops, layer: ne.layer }, "plot", "from-root"), le = { visible: ae({}, ne.visible, { dflt: !0 }), type: ae({}, ne.type, { values: ["-", "linear", "log", "date", "category"] }), autotypenumbers: ne.autotypenumbers, autorangeoptions: { minallowed: ne.autorangeoptions.minallowed, maxallowed: ne.autorangeoptions.maxallowed, clipmin: ne.autorangeoptions.clipmin, clipmax: ne.autorangeoptions.clipmax, include: ne.autorangeoptions.include, editType: "plot" }, autorange: ae({}, ne.autorange, { editType: "plot" }), rangemode: { valType: "enumerated", values: ["tozero", "nonnegative", "normal"], dflt: "tozero", editType: "calc" }, minallowed: ae({}, ne.minallowed, { editType: "plot" }), maxallowed: ae({}, ne.maxallowed, { editType: "plot" }), range: ae({}, ne.range, { items: [{ valType: "any", editType: "plot", impliedEdits: { "^autorange": !1 } }, { valType: "any", editType: "plot", impliedEdits: { "^autorange": !1 } }], editType: "plot" }), categoryorder: ne.categoryorder, categoryarray: ne.categoryarray, angle: { valType: "angle", editType: "plot" }, autotickangles: ne.autotickangles, side: { valType: "enumerated", values: ["clockwise", "counterclockwise"], dflt: "clockwise", editType: "plot" }, title: { text: ae({}, ne.title.text, { editType: "plot", dflt: "" }), font: ae({}, ne.title.font, { editType: "plot" }), editType: "plot" }, hoverformat: ne.hoverformat, uirevision: { valType: "any", editType: "none" }, editType: "calc", _deprecated: { title: ne._deprecated.title, titlefont: ne._deprecated.titlefont } };
            ae(le, se, ce);
            var ue = { visible: ae({}, ne.visible, { dflt: !0 }), type: { valType: "enumerated", values: ["-", "linear", "category"], dflt: "-", editType: "calc", _noTemplating: !0 }, autotypenumbers: ne.autotypenumbers, categoryorder: ne.categoryorder, categoryarray: ne.categoryarray, thetaunit: { valType: "enumerated", values: ["radians", "degrees"], dflt: "degrees", editType: "calc" }, period: { valType: "number", editType: "calc", min: 0 }, direction: { valType: "enumerated", values: ["counterclockwise", "clockwise"], dflt: "counterclockwise", editType: "calc" }, rotation: { valType: "angle", editType: "calc" }, hoverformat: ne.hoverformat, uirevision: { valType: "any", editType: "none" }, editType: "calc" };
            ae(ue, se, ce), G.exports = { domain: ee({ name: "polar", editType: "plot" }), sector: { valType: "info_array", items: [{ valType: "number", editType: "plot" }, { valType: "number", editType: "plot" }], dflt: [0, 360], editType: "plot" }, hole: { valType: "number", min: 0, max: 1, dflt: 0, editType: "plot" }, bgcolor: { valType: "color", editType: "plot", dflt: te.background }, radialaxis: le, angularaxis: ue, gridshape: { valType: "enumerated", values: ["circular", "linear"], dflt: "circular", editType: "plot" }, uirevision: { valType: "any", editType: "none" }, editType: "calc" };
          }, 84588: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(78032), ae = Z(4448), oe = Z(4173).KO, se = Z(22777), ce = Z(87433), le = Z(12036), ue = Z(54616), de = Z(46473), fe = Z(97405), he = Z(75511), me = Z(9666), pe = Z(42219), ye = Z(51937), ge = Z(26484), ve = ge.axisNames;
            function be(Se, we, Ee, xe) {
              var Le = Ee("bgcolor");
              xe.bgColor = ne.combine(Le, xe.paper_bgcolor);
              var Qe = Ee("sector");
              Ee("hole");
              var ke, Me = oe(xe.fullData, ge.name, xe.id), Re = xe.layoutOut;
              function De($t, Pt) {
                return Ee(ke + "." + $t, Pt);
              }
              for (var Pe = 0; Pe < ve.length; Pe++) {
                ke = ve[Pe], te.isPlainObject(Se[ke]) || (Se[ke] = {});
                var $e = Se[ke], qe = ee.newContainer(we, ke);
                qe._id = qe._name = ke, qe._attr = xe.id + "." + ke, qe._traceIndices = Me.map(function($t) {
                  return $t._expandedIndex;
                });
                var Xe = ge.axisName2dataArray[ke], Ve = Ce($e, qe, De, Me, Xe, xe);
                de($e, qe, De, { axData: Me, dataAttr: Xe });
                var Ze = De("visible");
                switch (ye(qe, we, Re), De("uirevision", we.uirevision), qe._m = 1, ke) {
                  case "radialaxis":
                    De("minallowed"), De("maxallowed");
                    var Ge, ze = De("range"), We = qe.getAutorangeDflt(ze), at = De("autorange", We);
                    !ze || (ze[0] !== null || ze[1] !== null) && (ze[0] !== null && ze[1] !== null || at !== "reversed" && at !== !0) && (ze[0] === null || at !== "min" && at !== "max reversed") && (ze[1] === null || at !== "max" && at !== "min reversed") || (ze = void 0, delete qe.range, qe.autorange = !0, Ge = !0), Ge || (at = De("autorange", We = qe.getAutorangeDflt(ze))), $e.autorange = at, at && (he(De, at, ze), Ve !== "linear" && Ve !== "-" || De("rangemode"), qe.isReversed() && (qe._m = -1)), qe.cleanRange("range", { dfltRange: [0, 1] });
                    break;
                  case "angularaxis":
                    if (Ve === "date") {
                      te.log("Polar plots do not support date angular axes yet.");
                      for (var ct = 0; ct < Me.length; ct++) Me[ct].visible = !1;
                      Ve = $e.type = qe.type = "linear";
                    }
                    De(Ve === "linear" ? "thetaunit" : "period");
                    var et = De("direction");
                    De("rotation", { counterclockwise: 0, clockwise: 90 }[et]);
                }
                if (ue($e, qe, De, qe.type, { tickSuffixDflt: qe.thetaunit === "degrees" ? "°" : void 0 }), Ze) {
                  var dt, Je, it, st, gt, bt, yt, St, Lt, Rt, vt = xe.font || {};
                  Je = (dt = De("color")) === $e.color ? dt : vt.color, it = vt.size, st = vt.family, gt = vt.weight, bt = vt.style, yt = vt.variant, St = vt.textcase, Lt = vt.lineposition, Rt = vt.shadow, se($e, qe, De, qe.type), le($e, qe, De, qe.type, { font: { weight: gt, style: bt, variant: yt, textcase: St, lineposition: Lt, shadow: Rt, color: Je, size: it, family: st }, noAutotickangles: ke === "angularaxis", noTicklabelshift: !0, noTicklabelstandoff: !0 }), ce($e, qe, De, { outerTicks: !0 }), fe($e, qe, De, { dfltColor: dt, bgColor: xe.bgColor, blend: 60, showLine: !0, showGrid: !0, noZeroLine: !0, attributes: pe[ke] }), De("layer"), ke === "radialaxis" && (De("side"), De("angle", Qe[0]), De("title.text"), te.coerceFont(De, "title.font", { weight: gt, style: bt, variant: yt, textcase: St, lineposition: Lt, shadow: Rt, color: Je, size: te.bigFont(it), family: st }));
                }
                Ve !== "category" && De("hoverformat"), qe._input = $e;
              }
              we.angularaxis.type === "category" && Ee("gridshape");
            }
            function Ce(Se, we, Ee, xe, Le, Qe) {
              var ke = Ee("autotypenumbers", Qe.autotypenumbersDflt);
              if (Ee("type") === "-") {
                for (var Me, Re = 0; Re < xe.length; Re++) if (xe[Re].visible) {
                  Me = xe[Re];
                  break;
                }
                Me && Me[Le] && (we.type = me(Me[Le], "gregorian", { noMultiCategory: !0, autotypenumbers: ke })), we.type === "-" ? we.type = "linear" : Se.type = we.type;
              }
              return we.type;
            }
            G.exports = function(Se, we, Ee) {
              ae(Se, we, Ee, { type: ge.name, attributes: pe, handleDefaults: be, font: we.font, autotypenumbersDflt: we.autotypenumbers, paper_bgcolor: we.paper_bgcolor, fullData: Ee, layoutOut: we });
            };
          }, 35785: function(G, J, Z) {
            var te = Z(45568), ne = Z(65657), ee = Z(33626), ae = Z(34809), oe = ae.strRotate, se = ae.strTranslate, ce = Z(78766), le = Z(62203), ue = Z(44122), de = Z(29714), fe = Z(19091), he = Z(51937), me = Z(32919).doAutoRange, pe = Z(51680), ye = Z(14751), ge = Z(32141), ve = Z(17240), be = Z(44844).prepSelect, Ce = Z(44844).selectOnClick, Se = Z(44844).clearOutline, we = Z(27983), Ee = Z(34823), xe = Z(71817).redrawReglTraces, Le = Z(4530).MID_SHIFT, Qe = Z(26484), ke = Z(95928), Me = Z(52007), Re = Me.smith, De = Me.reactanceArc, Pe = Me.resistanceArc, $e = Me.smithTransform, qe = ae._, Xe = ae.mod, Ve = ae.deg2rad, Ze = ae.rad2deg;
            function Ge(et, dt, Je) {
              this.isSmith = Je || !1, this.id = dt, this.gd = et, this._hasClipOnAxisFalse = null, this.vangles = null, this.radialAxisAngle = null, this.traceHash = {}, this.layers = {}, this.clipPaths = {}, this.clipIds = {}, this.viewInitial = {};
              var it = et._fullLayout, st = "clip" + it._uid + dt;
              this.clipIds.forTraces = st + "-for-traces", this.clipPaths.forTraces = it._clips.append("clipPath").attr("id", this.clipIds.forTraces), this.clipPaths.forTraces.append("path"), this.framework = it["_" + (Je ? "smith" : "polar") + "layer"].append("g").attr("class", dt), this.getHole = function(gt) {
                return this.isSmith ? 0 : gt.hole;
              }, this.getSector = function(gt) {
                return this.isSmith ? [0, 360] : gt.sector;
              }, this.getRadial = function(gt) {
                return this.isSmith ? gt.realaxis : gt.radialaxis;
              }, this.getAngular = function(gt) {
                return this.isSmith ? gt.imaginaryaxis : gt.angularaxis;
              }, Je || (this.radialTickLayout = null, this.angularTickLayout = null);
            }
            var ze = Ge.prototype;
            function We(et) {
              var dt = et.ticks + String(et.ticklen) + String(et.showticklabels);
              return "side" in et && (dt += et.side), dt;
            }
            function at(et, dt) {
              return dt[ae.findIndexOfMin(dt, function(Je) {
                return ae.angleDist(et, Je);
              })];
            }
            function ct(et, dt, Je) {
              return dt ? (et.attr("display", null), et.attr(Je)) : et && et.attr("display", "none"), et;
            }
            G.exports = function(et, dt, Je) {
              return new Ge(et, dt, Je);
            }, ze.plot = function(et, dt) {
              for (var Je = this, it = dt[Je.id], st = !1, gt = 0; gt < et.length; gt++) if (et[gt][0].trace.cliponaxis === !1) {
                st = !0;
                break;
              }
              Je._hasClipOnAxisFalse = st, Je.updateLayers(dt, it), Je.updateLayout(dt, it), ue.generalUpdatePerTraceModule(Je.gd, Je, et, it), Je.updateFx(dt, it), Je.isSmith && (delete it.realaxis.range, delete it.imaginaryaxis.range);
            }, ze.updateLayers = function(et, dt) {
              var Je = this, it = Je.isSmith, st = Je.layers, gt = Je.getRadial(dt), bt = Je.getAngular(dt), yt = Qe.layerNames, St = yt.indexOf("frontplot"), Lt = yt.slice(0, St), Rt = bt.layer === "below traces", vt = gt.layer === "below traces";
              Rt && Lt.push("angular-line"), vt && Lt.push("radial-line"), Rt && Lt.push("angular-axis"), vt && Lt.push("radial-axis"), Lt.push("frontplot"), Rt || Lt.push("angular-line"), vt || Lt.push("radial-line"), Rt || Lt.push("angular-axis"), vt || Lt.push("radial-axis");
              var $t = (it ? "smith" : "polar") + "sublayer", Pt = Je.framework.selectAll("." + $t).data(Lt, String);
              Pt.enter().append("g").attr("class", function(Xt) {
                return $t + " " + Xt;
              }).each(function(Xt) {
                var Nt = st[Xt] = te.select(this);
                switch (Xt) {
                  case "frontplot":
                    it || Nt.append("g").classed("barlayer", !0), Nt.append("g").classed("scatterlayer", !0);
                    break;
                  case "backplot":
                    Nt.append("g").classed("maplayer", !0);
                    break;
                  case "plotbg":
                    st.bg = Nt.append("path");
                    break;
                  case "radial-grid":
                  case "angular-grid":
                    Nt.style("fill", "none");
                    break;
                  case "radial-line":
                    Nt.append("line").style("fill", "none");
                    break;
                  case "angular-line":
                    Nt.append("path").style("fill", "none");
                }
              }), Pt.order();
            }, ze.updateLayout = function(et, dt) {
              var Je = this, it = Je.layers, st = et._size, gt = Je.getRadial(dt), bt = Je.getAngular(dt), yt = dt.domain.x, St = dt.domain.y;
              Je.xOffset = st.l + st.w * yt[0], Je.yOffset = st.t + st.h * (1 - St[1]);
              var Lt = Je.xLength = st.w * (yt[1] - yt[0]), Rt = Je.yLength = st.h * (St[1] - St[0]), vt = Je.getSector(dt);
              Je.sectorInRad = vt.map(Ve);
              var $t, Pt, Xt, Nt, Gt, Ft = Je.sectorBBox = function(jt) {
                var cn, wn = jt[0], Rn = jt[1] - wn, Pn = Xe(wn, 360), Ie = Pn + Rn, rt = Math.cos(Ve(Pn)), ft = Math.sin(Ve(Pn)), pt = Math.cos(Ve(Ie)), wt = Math.sin(Ve(Ie));
                return cn = Pn <= 90 && Ie >= 90 || Pn > 90 && Ie >= 450 ? 1 : ft <= 0 && wt <= 0 ? 0 : Math.max(ft, wt), [Pn <= 180 && Ie >= 180 || Pn > 180 && Ie >= 540 ? -1 : rt >= 0 && pt >= 0 ? 0 : Math.min(rt, pt), Pn <= 270 && Ie >= 270 || Pn > 270 && Ie >= 630 ? -1 : ft >= 0 && wt >= 0 ? 0 : Math.min(ft, wt), Ie >= 360 ? 1 : rt <= 0 && pt <= 0 ? 0 : Math.max(rt, pt), cn];
              }(vt), Tt = Ft[2] - Ft[0], It = Ft[3] - Ft[1], Ct = Rt / Lt, Kt = Math.abs(It / Tt);
              Ct > Kt ? ($t = Lt, Gt = (Rt - (Pt = Lt * Kt)) / st.h / 2, Xt = [yt[0], yt[1]], Nt = [St[0] + Gt, St[1] - Gt]) : (Pt = Rt, Gt = (Lt - ($t = Rt / Kt)) / st.w / 2, Xt = [yt[0] + Gt, yt[1] - Gt], Nt = [St[0], St[1]]), Je.xLength2 = $t, Je.yLength2 = Pt, Je.xDomain2 = Xt, Je.yDomain2 = Nt;
              var Vt, Yt = Je.xOffset2 = st.l + st.w * Xt[0], sn = Je.yOffset2 = st.t + st.h * (1 - Nt[1]), hn = Je.radius = $t / Tt, Qt = Je.innerRadius = Je.getHole(dt) * hn, Ut = Je.cx = Yt - hn * Ft[0], Wt = Je.cy = sn + hn * Ft[3], rn = Je.cxx = Ut - Yt, vn = Je.cyy = Wt - sn, dn = gt.side;
              dn === "counterclockwise" ? (Vt = dn, dn = "top") : dn === "clockwise" && (Vt = dn, dn = "bottom"), Je.radialAxis = Je.mockAxis(et, dt, gt, { _id: "x", side: dn, _trueSide: Vt, domain: [Qt / st.w, hn / st.w] }), Je.angularAxis = Je.mockAxis(et, dt, bt, { side: "right", domain: [0, Math.PI], autorange: !1 }), Je.doAutoRange(et, dt), Je.updateAngularAxis(et, dt), Je.updateRadialAxis(et, dt), Je.updateRadialAxisTitle(et, dt), Je.xaxis = Je.mockCartesianAxis(et, dt, { _id: "x", domain: Xt }), Je.yaxis = Je.mockCartesianAxis(et, dt, { _id: "y", domain: Nt });
              var Ht = Je.pathSubplot();
              Je.clipPaths.forTraces.select("path").attr("d", Ht).attr("transform", se(rn, vn)), it.frontplot.attr("transform", se(Yt, sn)).call(le.setClipUrl, Je._hasClipOnAxisFalse ? null : Je.clipIds.forTraces, Je.gd), it.bg.attr("d", Ht).attr("transform", se(Ut, Wt)).call(ce.fill, dt.bgcolor);
            }, ze.mockAxis = function(et, dt, Je, it) {
              var st = ae.extendFlat({}, Je, it);
              return he(st, dt, et), st;
            }, ze.mockCartesianAxis = function(et, dt, Je) {
              var it = this, st = it.isSmith, gt = Je._id, bt = ae.extendFlat({ type: "linear" }, Je);
              fe(bt, et);
              var yt = { x: [0, 2], y: [1, 3] };
              return bt.setRange = function() {
                var St = it.sectorBBox, Lt = yt[gt], Rt = it.radialAxis._rl, vt = (Rt[1] - Rt[0]) / (1 - it.getHole(dt));
                bt.range = [St[Lt[0]] * vt, St[Lt[1]] * vt];
              }, bt.isPtWithinRange = gt !== "x" || st ? function() {
                return !0;
              } : function(St) {
                return it.isPtInside(St);
              }, bt.setRange(), bt.setScale(), bt;
            }, ze.doAutoRange = function(et, dt) {
              var Je = this, it = Je.gd, st = Je.radialAxis, gt = Je.getRadial(dt);
              me(it, st);
              var bt = st.range;
              if (gt.range = bt.slice(), gt._input.range = bt.slice(), st._rl = [st.r2l(bt[0], null, "gregorian"), st.r2l(bt[1], null, "gregorian")], st.minallowed !== void 0) {
                var yt = st.r2l(st.minallowed);
                st._rl[0] > st._rl[1] ? st._rl[1] = Math.max(st._rl[1], yt) : st._rl[0] = Math.max(st._rl[0], yt);
              }
              if (st.maxallowed !== void 0) {
                var St = st.r2l(st.maxallowed);
                st._rl[0] < st._rl[1] ? st._rl[1] = Math.min(st._rl[1], St) : st._rl[0] = Math.min(st._rl[0], St);
              }
            }, ze.updateRadialAxis = function(et, dt) {
              var Je = this, it = Je.gd, st = Je.layers, gt = Je.radius, bt = Je.innerRadius, yt = Je.cx, St = Je.cy, Lt = Je.getRadial(dt), Rt = Xe(Je.getSector(dt)[0], 360), vt = Je.radialAxis, $t = bt < gt, Pt = Je.isSmith;
              Pt || (Je.fillViewInitialKey("radialaxis.angle", Lt.angle), Je.fillViewInitialKey("radialaxis.range", vt.range.slice()), vt.setGeometry()), vt.tickangle === "auto" && Rt > 90 && Rt <= 270 && (vt.tickangle = 180);
              var Xt = Pt ? function(sn) {
                var hn = $e(Je, Re([sn.x, 0]));
                return se(hn[0] - yt, hn[1] - St);
              } : function(sn) {
                return se(vt.l2p(sn.x) + bt, 0);
              }, Nt = Pt ? function(sn) {
                return Pe(Je, sn.x, -1 / 0, 1 / 0);
              } : function(sn) {
                return Je.pathArc(vt.r2p(sn.x) + bt);
              }, Gt = We(Lt);
              if (Je.radialTickLayout !== Gt && (st["radial-axis"].selectAll(".xtick").remove(), Je.radialTickLayout = Gt), $t) {
                vt.setScale();
                var Ft = 0, Tt = Pt ? (vt.tickvals || []).filter(function(sn) {
                  return sn >= 0;
                }).map(function(sn) {
                  return de.tickText(vt, sn, !0, !1);
                }) : de.calcTicks(vt), It = Pt ? Tt : de.clipEnds(vt, Tt), Ct = de.getTickSigns(vt)[2];
                Pt && ((vt.ticks === "top" && vt.side === "bottom" || vt.ticks === "bottom" && vt.side === "top") && (Ct = -Ct), vt.ticks === "top" && vt.side === "top" && (Ft = -vt.ticklen), vt.ticks === "bottom" && vt.side === "bottom" && (Ft = vt.ticklen)), de.drawTicks(it, vt, { vals: Tt, layer: st["radial-axis"], path: de.makeTickPath(vt, 0, Ct), transFn: Xt, crisp: !1 }), de.drawGrid(it, vt, { vals: It, layer: st["radial-grid"], path: Nt, transFn: ae.noop, crisp: !1 }), de.drawLabels(it, vt, { vals: Tt, layer: st["radial-axis"], transFn: Xt, labelFns: de.makeLabelFns(vt, Ft) });
              }
              var Kt = Je.radialAxisAngle = Je.vangles ? Ze(at(Ve(Lt.angle), Je.vangles)) : Lt.angle, Vt = se(yt, St), Yt = Vt + oe(-Kt);
              ct(st["radial-axis"], $t && (Lt.showticklabels || Lt.ticks), { transform: Yt }), ct(st["radial-grid"], $t && Lt.showgrid, { transform: Pt ? "" : Vt }), ct(st["radial-line"].select("line"), $t && Lt.showline, { x1: Pt ? -gt : bt, y1: 0, x2: gt, y2: 0, transform: Yt }).attr("stroke-width", Lt.linewidth).call(ce.stroke, Lt.linecolor);
            }, ze.updateRadialAxisTitle = function(et, dt, Je) {
              if (!this.isSmith) {
                var it = this, st = it.gd, gt = it.radius, bt = it.cx, yt = it.cy, St = it.getRadial(dt), Lt = it.id + "title", Rt = 0;
                if (St.title) {
                  var vt = le.bBox(it.layers["radial-axis"].node()).height, $t = St.title.font.size, Pt = St.side;
                  Rt = Pt === "top" ? $t : Pt === "counterclockwise" ? -(vt + 0.4 * $t) : vt + 0.8 * $t;
                }
                var Xt = Je !== void 0 ? Je : it.radialAxisAngle, Nt = Ve(Xt), Gt = Math.cos(Nt), Ft = Math.sin(Nt), Tt = bt + gt / 2 * Gt + Rt * Ft, It = yt - gt / 2 * Ft + Rt * Gt;
                it.layers["radial-axis-title"] = ve.draw(st, Lt, { propContainer: St, propName: it.id + ".radialaxis.title", placeholder: qe(st, "Click to enter radial axis title"), attributes: { x: Tt, y: It, "text-anchor": "middle" }, transform: { rotate: -Xt } });
              }
            }, ze.updateAngularAxis = function(et, dt) {
              var Je = this, it = Je.gd, st = Je.layers, gt = Je.radius, bt = Je.innerRadius, yt = Je.cx, St = Je.cy, Lt = Je.getAngular(dt), Rt = Je.angularAxis, vt = Je.isSmith;
              vt || (Je.fillViewInitialKey("angularaxis.rotation", Lt.rotation), Rt.setGeometry(), Rt.setScale());
              var $t = vt ? function(sn) {
                var hn = $e(Je, Re([0, sn.x]));
                return Math.atan2(hn[0] - yt, hn[1] - St) - Math.PI / 2;
              } : function(sn) {
                return Rt.t2g(sn.x);
              };
              Rt.type === "linear" && Rt.thetaunit === "radians" && (Rt.tick0 = Ze(Rt.tick0), Rt.dtick = Ze(Rt.dtick));
              var Pt = function(sn) {
                return se(yt + gt * Math.cos(sn), St - gt * Math.sin(sn));
              }, Xt = vt ? function(sn) {
                var hn = $e(Je, Re([0, sn.x]));
                return se(hn[0], hn[1]);
              } : function(sn) {
                return Pt($t(sn));
              }, Nt = vt ? function(sn) {
                var hn = $e(Je, Re([0, sn.x])), Qt = Math.atan2(hn[0] - yt, hn[1] - St) - Math.PI / 2;
                return se(hn[0], hn[1]) + oe(-Ze(Qt));
              } : function(sn) {
                var hn = $t(sn);
                return Pt(hn) + oe(-Ze(hn));
              }, Gt = vt ? function(sn) {
                return De(Je, sn.x, 0, 1 / 0);
              } : function(sn) {
                var hn = $t(sn), Qt = Math.cos(hn), Ut = Math.sin(hn);
                return "M" + [yt + bt * Qt, St - bt * Ut] + "L" + [yt + gt * Qt, St - gt * Ut];
              }, Ft = de.makeLabelFns(Rt, 0).labelStandoff, Tt = { xFn: function(sn) {
                var hn = $t(sn);
                return Math.cos(hn) * Ft;
              }, yFn: function(sn) {
                var hn = $t(sn), Qt = Math.sin(hn) > 0 ? 0.2 : 1;
                return -Math.sin(hn) * (Ft + sn.fontSize * Qt) + Math.abs(Math.cos(hn)) * (sn.fontSize * Le);
              }, anchorFn: function(sn) {
                var hn = $t(sn), Qt = Math.cos(hn);
                return Math.abs(Qt) < 0.1 ? "middle" : Qt > 0 ? "start" : "end";
              }, heightFn: function(sn, hn, Qt) {
                var Ut = $t(sn);
                return -0.5 * (1 + Math.sin(Ut)) * Qt;
              } }, It = We(Lt);
              Je.angularTickLayout !== It && (st["angular-axis"].selectAll("." + Rt._id + "tick").remove(), Je.angularTickLayout = It);
              var Ct, Kt = vt ? [1 / 0].concat(Rt.tickvals || []).map(function(sn) {
                return de.tickText(Rt, sn, !0, !1);
              }) : de.calcTicks(Rt);
              if (vt && (Kt[0].text = "∞", Kt[0].fontSize *= 1.75), dt.gridshape === "linear" ? (Ct = Kt.map($t), ae.angleDelta(Ct[0], Ct[1]) < 0 && (Ct = Ct.slice().reverse())) : Ct = null, Je.vangles = Ct, Rt.type === "category" && (Kt = Kt.filter(function(sn) {
                return ae.isAngleInsideSector($t(sn), Je.sectorInRad);
              })), Rt.visible) {
                var Vt = Rt.ticks === "inside" ? -1 : 1, Yt = (Rt.linewidth || 1) / 2;
                de.drawTicks(it, Rt, { vals: Kt, layer: st["angular-axis"], path: "M" + Vt * Yt + ",0h" + Vt * Rt.ticklen, transFn: Nt, crisp: !1 }), de.drawGrid(it, Rt, { vals: Kt, layer: st["angular-grid"], path: Gt, transFn: ae.noop, crisp: !1 }), de.drawLabels(it, Rt, { vals: Kt, layer: st["angular-axis"], repositionOnUpdate: !0, transFn: Xt, labelFns: Tt });
              }
              ct(st["angular-line"].select("path"), Lt.showline, { d: Je.pathSubplot(), transform: se(yt, St) }).attr("stroke-width", Lt.linewidth).call(ce.stroke, Lt.linecolor);
            }, ze.updateFx = function(et, dt) {
              this.gd._context.staticPlot || (!this.isSmith && (this.updateAngularDrag(et), this.updateRadialDrag(et, dt, 0), this.updateRadialDrag(et, dt, 1)), this.updateHoverAndMainDrag(et));
            }, ze.updateHoverAndMainDrag = function(et) {
              var dt, Je, it = this, st = it.isSmith, gt = it.gd, bt = it.layers, yt = et._zoomlayer, St = Qe.MINZOOM, Lt = Qe.OFFEDGE, Rt = it.radius, vt = it.innerRadius, $t = it.cx, Pt = it.cy, Xt = it.cxx, Nt = it.cyy, Gt = it.sectorInRad, Ft = it.vangles, Tt = it.radialAxis, It = ke.clampTiny, Ct = ke.findXYatLength, Kt = ke.findEnclosingVertexAngles, Vt = Qe.cornerHalfWidth, Yt = Qe.cornerLen / 2, sn = pe.makeDragger(bt, "path", "maindrag", et.dragmode === !1 ? "none" : "crosshair");
              te.select(sn).attr("d", it.pathSubplot()).attr("transform", se($t, Pt)), sn.onmousemove = function(_n) {
                ge.hover(gt, _n, it.id), gt._fullLayout._lasthover = sn, gt._fullLayout._hoversubplot = it.id;
              }, sn.onmouseout = function(_n) {
                gt._dragging || ye.unhover(gt, _n);
              };
              var hn, Qt, Ut, Wt, rn, vn, dn, Ht, jt, cn = { element: sn, gd: gt, subplot: it.id, plotinfo: { id: it.id, xaxis: it.xaxis, yaxis: it.yaxis }, xaxes: [it.xaxis], yaxes: [it.yaxis] };
              function wn(_n, gn) {
                return Math.sqrt(_n * _n + gn * gn);
              }
              function Rn(_n, gn) {
                return wn(_n - Xt, gn - Nt);
              }
              function Pn(_n, gn) {
                return Math.atan2(Nt - gn, _n - Xt);
              }
              function Ie(_n, gn) {
                return [_n * Math.cos(gn), _n * Math.sin(-gn)];
              }
              function rt(_n, gn) {
                if (_n === 0) return it.pathSector(2 * Vt);
                var bn = Yt / _n, an = gn - bn, Zt = gn + bn, Mn = Math.max(0, Math.min(_n, Rt)), Ln = Mn - Vt, In = Mn + Vt;
                return "M" + Ie(Ln, an) + "A" + [Ln, Ln] + " 0,0,0 " + Ie(Ln, Zt) + "L" + Ie(In, Zt) + "A" + [In, In] + " 0,0,1 " + Ie(In, an) + "Z";
              }
              function ft(_n, gn, bn) {
                if (_n === 0) return it.pathSector(2 * Vt);
                var an, Zt, Mn = Ie(_n, gn), Ln = Ie(_n, bn), In = It((Mn[0] + Ln[0]) / 2), kn = It((Mn[1] + Ln[1]) / 2);
                if (In && kn) {
                  var er = kn / In, jn = -1 / er, ur = Ct(Vt, er, In, kn);
                  an = Ct(Yt, jn, ur[0][0], ur[0][1]), Zt = Ct(Yt, jn, ur[1][0], ur[1][1]);
                } else {
                  var Kn, Jn;
                  kn ? (Kn = Yt, Jn = Vt) : (Kn = Vt, Jn = Yt), an = [[In - Kn, kn - Jn], [In + Kn, kn - Jn]], Zt = [[In - Kn, kn + Jn], [In + Kn, kn + Jn]];
                }
                return "M" + an.join("L") + "L" + Zt.reverse().join("L") + "Z";
              }
              function pt(_n, gn) {
                return gn = Math.max(Math.min(gn, Rt), vt), _n < Lt ? _n = 0 : Rt - _n < Lt ? _n = Rt : gn < Lt ? gn = 0 : Rt - gn < Lt && (gn = Rt), Math.abs(gn - _n) > St ? (_n < gn ? (Ut = _n, Wt = gn) : (Ut = gn, Wt = _n), !0) : (Ut = null, Wt = null, !1);
              }
              function wt(_n, gn) {
                _n = _n || rn, gn = gn || "M0,0Z", Ht.attr("d", _n), jt.attr("d", gn), pe.transitionZoombox(Ht, jt, vn, dn), vn = !0;
                var bn = {};
                un(bn), gt.emit("plotly_relayouting", bn);
              }
              function Bt(_n, gn) {
                var bn, an, Zt = hn + (_n *= dt), Mn = Qt + (gn *= Je), Ln = Rn(hn, Qt), In = Math.min(Rn(Zt, Mn), Rt), kn = Pn(hn, Qt);
                pt(Ln, In) && (bn = rn + it.pathSector(Wt), Ut && (bn += it.pathSector(Ut)), an = rt(Ut, kn) + rt(Wt, kn)), wt(bn, an);
              }
              function At(_n, gn, bn, an) {
                var Zt = ke.findIntersectionXY(bn, an, bn, [_n - Xt, Nt - gn]);
                return wn(Zt[0], Zt[1]);
              }
              function Dt(_n, gn) {
                var bn, an, Zt = hn + _n, Mn = Qt + gn, Ln = Pn(hn, Qt), In = Pn(Zt, Mn), kn = Kt(Ln, Ft), er = Kt(In, Ft);
                pt(At(hn, Qt, kn[0], kn[1]), Math.min(At(Zt, Mn, er[0], er[1]), Rt)) && (bn = rn + it.pathSector(Wt), Ut && (bn += it.pathSector(Ut)), an = [ft(Ut, kn[0], kn[1]), ft(Wt, kn[0], kn[1])].join(" ")), wt(bn, an);
              }
              function Ot() {
                if (pe.removeZoombox(gt), Ut !== null && Wt !== null) {
                  var _n = {};
                  un(_n), pe.showDoubleClickNotifier(gt), ee.call("_guiRelayout", gt, _n);
                }
              }
              function un(_n) {
                var gn = Tt._rl, bn = (gn[1] - gn[0]) / (1 - vt / Rt) / Rt, an = [gn[0] + (Ut - vt) * bn, gn[0] + (Wt - vt) * bn];
                _n[it.id + ".radialaxis.range"] = an;
              }
              function yn(_n, gn) {
                var bn = gt._fullLayout.clickmode;
                if (pe.removeZoombox(gt), _n === 2) {
                  var an = {};
                  for (var Zt in it.viewInitial) an[it.id + "." + Zt] = it.viewInitial[Zt];
                  gt.emit("plotly_doubleclick", null), ee.call("_guiRelayout", gt, an);
                }
                bn.indexOf("select") > -1 && _n === 1 && Ce(gn, gt, [it.xaxis], [it.yaxis], it.id, cn), bn.indexOf("event") > -1 && ge.click(gt, gn, it.id);
              }
              cn.prepFn = function(_n, gn, bn) {
                var an = gt._fullLayout.dragmode, Zt = sn.getBoundingClientRect();
                gt._fullLayout._calcInverseTransform(gt);
                var Mn = gt._fullLayout._invTransform;
                dt = gt._fullLayout._invScaleX, Je = gt._fullLayout._invScaleY;
                var Ln = ae.apply3DTransform(Mn)(gn - Zt.left, bn - Zt.top);
                if (hn = Ln[0], Qt = Ln[1], Ft) {
                  var In = ke.findPolygonOffset(Rt, Gt[0], Gt[1], Ft);
                  hn += Xt + In[0], Qt += Nt + In[1];
                }
                switch (an) {
                  case "zoom":
                    cn.clickFn = yn, st || (cn.moveFn = Ft ? Dt : Bt, cn.doneFn = Ot, function() {
                      Ut = null, Wt = null, rn = it.pathSubplot(), vn = !1;
                      var kn = gt._fullLayout[it.id];
                      dn = ne(kn.bgcolor).getLuminance(), (Ht = pe.makeZoombox(yt, dn, $t, Pt, rn)).attr("fill-rule", "evenodd"), jt = pe.makeCorners(yt, $t, Pt), Se(gt);
                    }());
                    break;
                  case "select":
                  case "lasso":
                    be(_n, gn, bn, cn, an);
                }
              }, ye.init(cn);
            }, ze.updateRadialDrag = function(et, dt, Je) {
              var it = this, st = it.gd, gt = it.layers, bt = it.radius, yt = it.innerRadius, St = it.cx, Lt = it.cy, Rt = it.radialAxis, vt = Qe.radialDragBoxSize, $t = vt / 2;
              if (Rt.visible) {
                var Pt, Xt, Nt, Gt = Ve(it.radialAxisAngle), Ft = Rt._rl, Tt = Ft[0], It = Ft[1], Ct = Ft[Je], Kt = 0.75 * (Ft[1] - Ft[0]) / (1 - it.getHole(dt)) / bt;
                Je ? (Pt = St + (bt + $t) * Math.cos(Gt), Xt = Lt - (bt + $t) * Math.sin(Gt), Nt = "radialdrag") : (Pt = St + (yt - $t) * Math.cos(Gt), Xt = Lt - (yt - $t) * Math.sin(Gt), Nt = "radialdrag-inner");
                var Vt, Yt, sn, hn = pe.makeRectDragger(gt, Nt, "crosshair", -$t, -$t, vt, vt), Qt = { element: hn, gd: st };
                et.dragmode === !1 && (Qt.dragmode = !1), ct(te.select(hn), Rt.visible && yt < bt, { transform: se(Pt, Xt) }), Qt.prepFn = function() {
                  Vt = null, Yt = null, sn = null, Qt.moveFn = Ut, Qt.doneFn = Wt, Se(st);
                }, Qt.clampFn = function(dn, Ht) {
                  return Math.sqrt(dn * dn + Ht * Ht) < Qe.MINDRAG && (dn = 0, Ht = 0), [dn, Ht];
                }, ye.init(Qt);
              }
              function Ut(dn, Ht) {
                if (Vt) Vt(dn, Ht);
                else {
                  var jt = [dn, -Ht], cn = [Math.cos(Gt), Math.sin(Gt)], wn = Math.abs(ae.dot(jt, cn) / Math.sqrt(ae.dot(jt, jt)));
                  isNaN(wn) || (Vt = wn < 0.5 ? rn : vn);
                }
                var Rn = {};
                (function(Pn) {
                  Yt !== null ? Pn[it.id + ".radialaxis.angle"] = Yt : sn !== null && (Pn[it.id + ".radialaxis.range[" + Je + "]"] = sn);
                })(Rn), st.emit("plotly_relayouting", Rn);
              }
              function Wt() {
                Yt !== null ? ee.call("_guiRelayout", st, it.id + ".radialaxis.angle", Yt) : sn !== null && ee.call("_guiRelayout", st, it.id + ".radialaxis.range[" + Je + "]", sn);
              }
              function rn(dn, Ht) {
                if (Je !== 0) {
                  var jt = Pt + dn, cn = Xt + Ht;
                  Yt = Math.atan2(Lt - cn, jt - St), it.vangles && (Yt = at(Yt, it.vangles)), Yt = Ze(Yt);
                  var wn = se(St, Lt) + oe(-Yt);
                  gt["radial-axis"].attr("transform", wn), gt["radial-line"].select("line").attr("transform", wn);
                  var Rn = it.gd._fullLayout, Pn = Rn[it.id];
                  it.updateRadialAxisTitle(Rn, Pn, Yt);
                }
              }
              function vn(dn, Ht) {
                var jt = ae.dot([dn, -Ht], [Math.cos(Gt), Math.sin(Gt)]);
                if (sn = Ct - Kt * jt, Kt > 0 == (Je ? sn > Tt : sn < It)) {
                  var cn = st._fullLayout, wn = cn[it.id];
                  Rt.range[Je] = sn, Rt._rl[Je] = sn, it.updateRadialAxis(cn, wn), it.xaxis.setRange(), it.xaxis.setScale(), it.yaxis.setRange(), it.yaxis.setScale();
                  var Rn = !1;
                  for (var Pn in it.traceHash) {
                    var Ie = it.traceHash[Pn], rt = ae.filterVisible(Ie);
                    Ie[0][0].trace._module.plot(st, it, rt, wn), ee.traceIs(Pn, "gl") && rt.length && (Rn = !0);
                  }
                  Rn && (Ee(st), xe(st));
                } else sn = null;
              }
            }, ze.updateAngularDrag = function(et) {
              var dt = this, Je = dt.gd, it = dt.layers, st = dt.radius, gt = dt.angularAxis, bt = dt.cx, yt = dt.cy, St = dt.cxx, Lt = dt.cyy, Rt = Qe.angularDragBoxSize, vt = pe.makeDragger(it, "path", "angulardrag", et.dragmode === !1 ? "none" : "move"), $t = { element: vt, gd: Je };
              function Pt(Qt, Ut) {
                return Math.atan2(Lt + Rt - Ut, Qt - St - Rt);
              }
              et.dragmode === !1 ? $t.dragmode = !1 : te.select(vt).attr("d", dt.pathAnnulus(st, st + Rt)).attr("transform", se(bt, yt)).call(we, "move");
              var Xt, Nt, Gt, Ft, Tt, It, Ct = it.frontplot.select(".scatterlayer").selectAll(".trace"), Kt = Ct.selectAll(".point"), Vt = Ct.selectAll(".textpoint");
              function Yt(Qt, Ut) {
                var Wt = dt.gd._fullLayout, rn = Wt[dt.id], vn = Pt(Xt + Qt * et._invScaleX, Nt + Ut * et._invScaleY), dn = Ze(vn - It);
                if (Ft = Gt + dn, it.frontplot.attr("transform", se(dt.xOffset2, dt.yOffset2) + oe([-dn, St, Lt])), dt.vangles) {
                  Tt = dt.radialAxisAngle + dn;
                  var Ht = se(bt, yt) + oe(-dn), jt = se(bt, yt) + oe(-Tt);
                  it.bg.attr("transform", Ht), it["radial-grid"].attr("transform", Ht), it["radial-axis"].attr("transform", jt), it["radial-line"].select("line").attr("transform", jt), dt.updateRadialAxisTitle(Wt, rn, Tt);
                } else dt.clipPaths.forTraces.select("path").attr("transform", se(St, Lt) + oe(dn));
                Kt.each(function() {
                  var rt = te.select(this), ft = le.getTranslate(rt);
                  rt.attr("transform", se(ft.x, ft.y) + oe([dn]));
                }), Vt.each(function() {
                  var rt = te.select(this), ft = rt.select("text"), pt = le.getTranslate(rt);
                  rt.attr("transform", oe([dn, ft.attr("x"), ft.attr("y")]) + se(pt.x, pt.y));
                }), gt.rotation = ae.modHalf(Ft, 360), dt.updateAngularAxis(Wt, rn), dt._hasClipOnAxisFalse && !ae.isFullCircle(dt.sectorInRad) && Ct.call(le.hideOutsideRangePoints, dt);
                var cn = !1;
                for (var wn in dt.traceHash) if (ee.traceIs(wn, "gl")) {
                  var Rn = dt.traceHash[wn], Pn = ae.filterVisible(Rn);
                  Rn[0][0].trace._module.plot(Je, dt, Pn, rn), Pn.length && (cn = !0);
                }
                cn && (Ee(Je), xe(Je));
                var Ie = {};
                sn(Ie), Je.emit("plotly_relayouting", Ie);
              }
              function sn(Qt) {
                Qt[dt.id + ".angularaxis.rotation"] = Ft, dt.vangles && (Qt[dt.id + ".radialaxis.angle"] = Tt);
              }
              function hn() {
                Vt.select("text").attr("transform", null);
                var Qt = {};
                sn(Qt), ee.call("_guiRelayout", Je, Qt);
              }
              $t.prepFn = function(Qt, Ut, Wt) {
                var rn = et[dt.id];
                Gt = rn.angularaxis.rotation;
                var vn = vt.getBoundingClientRect();
                Xt = Ut - vn.left, Nt = Wt - vn.top, Je._fullLayout._calcInverseTransform(Je);
                var dn = ae.apply3DTransform(et._invTransform)(Xt, Nt);
                Xt = dn[0], Nt = dn[1], It = Pt(Xt, Nt), $t.moveFn = Yt, $t.doneFn = hn, Se(Je);
              }, dt.vangles && !ae.isFullCircle(dt.sectorInRad) && ($t.prepFn = ae.noop, we(te.select(vt), null)), ye.init($t);
            }, ze.isPtInside = function(et) {
              if (this.isSmith) return !0;
              var dt = this.sectorInRad, Je = this.vangles, it = this.angularAxis.c2g(et.theta), st = this.radialAxis, gt = st.c2l(et.r), bt = st._rl;
              return (Je ? ke.isPtInsidePolygon : ae.isPtInsideSector)(gt, it, bt, dt, Je);
            }, ze.pathArc = function(et) {
              var dt = this.sectorInRad, Je = this.vangles;
              return (Je ? ke.pathPolygon : ae.pathArc)(et, dt[0], dt[1], Je);
            }, ze.pathSector = function(et) {
              var dt = this.sectorInRad, Je = this.vangles;
              return (Je ? ke.pathPolygon : ae.pathSector)(et, dt[0], dt[1], Je);
            }, ze.pathAnnulus = function(et, dt) {
              var Je = this.sectorInRad, it = this.vangles;
              return (it ? ke.pathPolygonAnnulus : ae.pathAnnulus)(et, dt, Je[0], Je[1], it);
            }, ze.pathSubplot = function() {
              var et = this.innerRadius, dt = this.radius;
              return et ? this.pathAnnulus(et, dt) : this.pathSector(dt);
            }, ze.fillViewInitialKey = function(et, dt) {
              et in this.viewInitial || (this.viewInitial[et] = dt);
            };
          }, 51937: function(G, J, Z) {
            var te = Z(34809), ne = Z(19091), ee = te.deg2rad, ae = te.rad2deg;
            G.exports = function(oe, se, ce) {
              switch (ne(oe, ce), oe._id) {
                case "x":
                case "radialaxis":
                  (function(le, ue) {
                    var de = ue._subplot;
                    le.setGeometry = function() {
                      var fe = le._rl[0], he = le._rl[1], me = de.innerRadius, pe = (de.radius - me) / (he - fe), ye = me / pe, ge = fe > he ? function(ve) {
                        return ve <= 0;
                      } : function(ve) {
                        return ve >= 0;
                      };
                      le.c2g = function(ve) {
                        var be = le.c2l(ve) - fe;
                        return (ge(be) ? be : 0) + ye;
                      }, le.g2c = function(ve) {
                        return le.l2c(ve + fe - ye);
                      }, le.g2p = function(ve) {
                        return ve * pe;
                      }, le.c2p = function(ve) {
                        return le.g2p(le.c2g(ve));
                      };
                    };
                  })(oe, se);
                  break;
                case "angularaxis":
                  (function(le, ue) {
                    var de = le.type;
                    if (de === "linear") {
                      var fe = le.d2c, he = le.c2d;
                      le.d2c = function(me, pe) {
                        return function(ye, ge) {
                          return ge === "degrees" ? ee(ye) : ye;
                        }(fe(me), pe);
                      }, le.c2d = function(me, pe) {
                        return he(function(ye, ge) {
                          return ge === "degrees" ? ae(ye) : ye;
                        }(me, pe));
                      };
                    }
                    le.makeCalcdata = function(me, pe) {
                      var ye, ge, ve = me[pe], be = me._length, Ce = function(Le) {
                        return le.d2c(Le, me.thetaunit);
                      };
                      if (ve) for (ye = new Array(be), ge = 0; ge < be; ge++) ye[ge] = Ce(ve[ge]);
                      else {
                        var Se = pe + "0", we = "d" + pe, Ee = Se in me ? Ce(me[Se]) : 0, xe = me[we] ? Ce(me[we]) : (le.period || 2 * Math.PI) / be;
                        for (ye = new Array(be), ge = 0; ge < be; ge++) ye[ge] = Ee + ge * xe;
                      }
                      return ye;
                    }, le.setGeometry = function() {
                      var me, pe, ye, ge, ve = ue.sector, be = ve.map(ee), Ce = { clockwise: -1, counterclockwise: 1 }[le.direction], Se = ee(le.rotation), we = function(Qe) {
                        return Ce * Qe + Se;
                      }, Ee = function(Qe) {
                        return (Qe - Se) / Ce;
                      };
                      switch (de) {
                        case "linear":
                          pe = me = te.identity, ge = ee, ye = ae, le.range = te.isFullCircle(be) ? [ve[0], ve[0] + 360] : be.map(Ee).map(ae);
                          break;
                        case "category":
                          var xe = le._categories.length, Le = le.period ? Math.max(le.period, xe) : xe;
                          Le === 0 && (Le = 1), pe = ge = function(Qe) {
                            return 2 * Qe * Math.PI / Le;
                          }, me = ye = function(Qe) {
                            return Qe * Le / Math.PI / 2;
                          }, le.range = [0, Le];
                      }
                      le.c2g = function(Qe) {
                        return we(pe(Qe));
                      }, le.g2c = function(Qe) {
                        return me(Ee(Qe));
                      }, le.t2g = function(Qe) {
                        return we(ge(Qe));
                      }, le.g2t = function(Qe) {
                        return ye(Ee(Qe));
                      };
                    };
                  })(oe, se);
              }
            };
          }, 70951: function(G) {
            G.exports = { attr: "subplot", name: "smith", axisNames: ["realaxis", "imaginaryaxis"], axisName2dataArray: { imaginaryaxis: "imag", realaxis: "real" } };
          }, 52007: function(G) {
            function J(ee) {
              return ee < 0 ? -1 : ee > 0 ? 1 : 0;
            }
            function Z(ee) {
              var ae = ee[0], oe = ee[1];
              if (!isFinite(ae) || !isFinite(oe)) return [1, 0];
              var se = (ae + 1) * (ae + 1) + oe * oe;
              return [(ae * ae + oe * oe - 1) / se, 2 * oe / se];
            }
            function te(ee, ae) {
              var oe = ae[0], se = ae[1];
              return [oe * ee.radius + ee.cx, -se * ee.radius + ee.cy];
            }
            function ne(ee, ae) {
              return ae * ee.radius;
            }
            G.exports = { smith: Z, reactanceArc: function(ee, ae, oe, se) {
              var ce = te(ee, Z([oe, ae])), le = ce[0], ue = ce[1], de = te(ee, Z([se, ae])), fe = de[0], he = de[1];
              if (ae === 0) return ["M" + le + "," + ue, "L" + fe + "," + he].join(" ");
              var me = ne(ee, 1 / Math.abs(ae));
              return ["M" + le + "," + ue, "A" + me + "," + me + " 0 0," + (ae < 0 ? 1 : 0) + " " + fe + "," + he].join(" ");
            }, resistanceArc: function(ee, ae, oe, se) {
              var ce = ne(ee, 1 / (ae + 1)), le = te(ee, Z([ae, oe])), ue = le[0], de = le[1], fe = te(ee, Z([ae, se])), he = fe[0], me = fe[1];
              if (J(oe) !== J(se)) {
                var pe = te(ee, Z([ae, 0]));
                return ["M" + ue + "," + de, "A" + ce + "," + ce + " 0 0," + (0 < oe ? 0 : 1) + " " + pe[0] + "," + pe[1], "A" + ce + "," + ce + " 0 0," + (se < 0 ? 0 : 1) + he + "," + me].join(" ");
              }
              return ["M" + ue + "," + de, "A" + ce + "," + ce + " 0 0," + (se < oe ? 0 : 1) + " " + he + "," + me].join(" ");
            }, smithTransform: te };
          }, 50358: function(G, J, Z) {
            var te = Z(4173).fX, ne = Z(34809).counterRegex, ee = Z(35785), ae = Z(70951), oe = ae.attr, se = ae.name, ce = ne(se), le = {};
            le[oe] = { valType: "subplotid", dflt: se, editType: "calc" }, G.exports = { attr: oe, name: se, idRoot: se, idRegex: ce, attrRegex: ce, attributes: le, layoutAttributes: Z(93288), supplyLayoutDefaults: Z(31359), plot: function(ue) {
              for (var de = ue._fullLayout, fe = ue.calcdata, he = de._subplots[se], me = 0; me < he.length; me++) {
                var pe = he[me], ye = te(fe, se, pe), ge = de[pe]._subplot;
                ge || (ge = ee(ue, pe, !0), de[pe]._subplot = ge), ge.plot(ye, de, ue._promises);
              }
            }, clean: function(ue, de, fe, he) {
              for (var me = he._subplots[se] || [], pe = 0; pe < me.length; pe++) {
                var ye = me[pe], ge = he[ye]._subplot;
                if (!de[ye] && ge) for (var ve in ge.framework.remove(), ge.clipPaths) ge.clipPaths[ve].remove();
              }
            }, toSVG: Z(37703).toSVG };
          }, 93288: function(G, J, Z) {
            var te = Z(10229), ne = Z(25829), ee = Z(13792).u, ae = Z(34809).extendFlat, oe = Z(13582).overrideAll, se = oe({ color: ne.color, showline: ae({}, ne.showline, { dflt: !0 }), linecolor: ne.linecolor, linewidth: ne.linewidth, showgrid: ae({}, ne.showgrid, { dflt: !0 }), gridcolor: ne.gridcolor, gridwidth: ne.gridwidth, griddash: ne.griddash }, "plot", "from-root"), ce = oe({ ticklen: ne.ticklen, tickwidth: ae({}, ne.tickwidth, { dflt: 2 }), tickcolor: ne.tickcolor, showticklabels: ne.showticklabels, labelalias: ne.labelalias, showtickprefix: ne.showtickprefix, tickprefix: ne.tickprefix, showticksuffix: ne.showticksuffix, ticksuffix: ne.ticksuffix, tickfont: ne.tickfont, tickformat: ne.tickformat, hoverformat: ne.hoverformat, layer: ne.layer }, "plot", "from-root"), le = ae({ visible: ae({}, ne.visible, { dflt: !0 }), tickvals: { dflt: [0.2, 0.5, 1, 2, 5], valType: "data_array", editType: "plot" }, tickangle: ae({}, ne.tickangle, { dflt: 90 }), ticks: { valType: "enumerated", values: ["top", "bottom", ""], editType: "ticks" }, side: { valType: "enumerated", values: ["top", "bottom"], dflt: "top", editType: "plot" }, editType: "calc" }, se, ce), ue = ae({ visible: ae({}, ne.visible, { dflt: !0 }), tickvals: { valType: "data_array", editType: "plot" }, ticks: ne.ticks, editType: "calc" }, se, ce);
            G.exports = { domain: ee({ name: "smith", editType: "plot" }), bgcolor: { valType: "color", editType: "plot", dflt: te.background }, realaxis: le, imaginaryaxis: ue, editType: "calc" };
          }, 31359: function(G, J, Z) {
            var te, ne, ee, ae = Z(34809), oe = Z(78766), se = Z(78032), ce = Z(4448), le = Z(4173).KO, ue = Z(54616), de = Z(12036), fe = Z(97405), he = Z(19091), me = Z(93288), pe = Z(70951), ye = pe.axisNames, ge = (te = function(be) {
              return ae.isTypedArray(be) && (be = Array.from(be)), be.slice().reverse().map(function(Ce) {
                return -Ce;
              }).concat([0]).concat(be);
            }, ne = String, ee = {}, function(be) {
              var Ce = ne ? ne(be) : be;
              if (Ce in ee) return ee[Ce];
              var Se = te(be);
              return ee[Ce] = Se, Se;
            });
            function ve(be, Ce, Se, we) {
              var Ee = Se("bgcolor");
              we.bgColor = oe.combine(Ee, we.paper_bgcolor);
              var xe, Le = le(we.fullData, pe.name, we.id), Qe = we.layoutOut;
              function ke(ze, We) {
                return Se(xe + "." + ze, We);
              }
              for (var Me = 0; Me < ye.length; Me++) {
                xe = ye[Me], ae.isPlainObject(be[xe]) || (be[xe] = {});
                var Re = be[xe], De = se.newContainer(Ce, xe);
                De._id = De._name = xe, De._attr = we.id + "." + xe, De._traceIndices = Le.map(function(ze) {
                  return ze._expandedIndex;
                });
                var Pe = ke("visible");
                if (De.type = "linear", he(De, Qe), ue(Re, De, ke, De.type), Pe) {
                  var $e, qe, Xe, Ve, Ze = xe === "realaxis";
                  Ze && ke("side"), Ze ? ke("tickvals") : ke("tickvals", ge(Ce.realaxis.tickvals || me.realaxis.tickvals.dflt)), ae.isTypedArray(De.tickvals) && (De.tickvals = Array.from(De.tickvals));
                  var Ge = we.font || {};
                  Pe && (qe = ($e = ke("color")) === Re.color ? $e : Ge.color, Xe = Ge.size, Ve = Ge.family), de(Re, De, ke, De.type, { noAutotickangles: !0, noTicklabelshift: !0, noTicklabelstandoff: !0, noTicklabelstep: !0, noAng: !Ze, noExp: !0, font: { color: qe, size: Xe, family: Ve } }), ae.coerce2(be, Ce, me, xe + ".ticklen"), ae.coerce2(be, Ce, me, xe + ".tickwidth"), ae.coerce2(be, Ce, me, xe + ".tickcolor", Ce.color), ke("ticks") || (delete Ce[xe].ticklen, delete Ce[xe].tickwidth, delete Ce[xe].tickcolor), fe(Re, De, ke, { dfltColor: $e, bgColor: we.bgColor, blend: 60, showLine: !0, showGrid: !0, noZeroLine: !0, attributes: me[xe] }), ke("layer");
                }
                ke("hoverformat"), delete De.type, De._input = Re;
              }
            }
            G.exports = function(be, Ce, Se) {
              ce(be, Ce, Se, { noUirevision: !0, type: pe.name, attributes: me, handleDefaults: ve, font: Ce.font, paper_bgcolor: Ce.paper_bgcolor, fullData: Se, layoutOut: Ce });
            };
          }, 4448: function(G, J, Z) {
            var te = Z(34809), ne = Z(78032), ee = Z(13792).N;
            G.exports = function(ae, oe, se, ce) {
              var le, ue, de = ce.type, fe = ce.attributes, he = ce.handleDefaults, me = ce.partition || "x", pe = oe._subplots[de], ye = pe.length, ge = ye && pe[0].replace(/\d+$/, "");
              function ve(we, Ee) {
                return te.coerce(le, ue, fe, we, Ee);
              }
              for (var be = 0; be < ye; be++) {
                var Ce = pe[be];
                le = ae[Ce] ? ae[Ce] : ae[Ce] = {}, ue = ne.newContainer(oe, Ce, ge), ce.noUirevision || ve("uirevision", oe.uirevision);
                var Se = {};
                Se[me] = [be / ye, (be + 1) / ye], ee(ue, oe, ve, Se), ce.id = Ce, he(le, ue, ve, ce);
              }
            };
          }, 3208: function(G, J, Z) {
            var te = Z(87296);
            function ne(ee) {
              var ae = ee.description ? " " + ee.description : "", oe = ee.keys || [];
              if (oe.length > 0) {
                for (var se = [], ce = 0; ce < oe.length; ce++) se[ce] = "`" + oe[ce] + "`";
                ae += "Finally, the template string has access to ", ae = oe.length === 1 ? ae + "variable " + se[0] : ae + "variables " + se.slice(0, -1).join(", ") + " and " + se.slice(-1) + ".";
              }
              return ae;
            }
            te.FORMAT_LINK, te.DATE_FORMAT_LINK, J.rb = function(ee, ae) {
              ee = ee || {}, ne(ae = ae || {});
              var oe = { valType: "string", dflt: "", editType: ee.editType || "none" };
              return ee.arrayOk !== !1 && (oe.arrayOk = !0), oe;
            }, J.ay = function(ee, ae) {
              ee = ee || {}, ne(ae = ae || {});
              var oe = { valType: "string", dflt: "", editType: ee.editType || "calc" };
              return ee.arrayOk !== !1 && (oe.arrayOk = !0), oe;
            }, J.LF = function(ee, ae) {
              return ae = ae || {}, (ee = ee || {}).newshape, ne(ae), { valType: "string", dflt: "", editType: ee.editType || "arraydraw" };
            };
          }, 7638: function(G, J, Z) {
            var te = Z(83637), ne = Z(4173).fX, ee = Z(34809).counterRegex, ae = "ternary";
            J.name = ae;
            var oe = J.attr = "subplot";
            J.idRoot = ae, J.idRegex = J.attrRegex = ee(ae), (J.attributes = {})[oe] = { valType: "subplotid", dflt: "ternary", editType: "calc" }, J.layoutAttributes = Z(77416), J.supplyLayoutDefaults = Z(25247), J.plot = function(se) {
              for (var ce = se._fullLayout, le = se.calcdata, ue = ce._subplots[ae], de = 0; de < ue.length; de++) {
                var fe = ue[de], he = ne(le, ae, fe), me = ce[fe]._subplot;
                me || (me = new te({ id: fe, graphDiv: se, container: ce._ternarylayer.node() }, ce), ce[fe]._subplot = me), me.plot(he, ce, se._promises);
              }
            }, J.clean = function(se, ce, le, ue) {
              for (var de = ue._subplots[ae] || [], fe = 0; fe < de.length; fe++) {
                var he = de[fe], me = ue[he]._subplot;
                !ce[he] && me && (me.plotContainer.remove(), me.clipDef.remove(), me.clipDefRelative.remove(), me.layers["a-title"].remove(), me.layers["b-title"].remove(), me.layers["c-title"].remove());
              }
            }, J.updateFx = function(se) {
              var ce = se._fullLayout;
              ce._ternarylayer.selectAll("g.toplevel").style("cursor", ce.dragmode === "pan" ? "move" : "crosshair");
            };
          }, 77416: function(G, J, Z) {
            var te = Z(10229), ne = Z(13792).u, ee = Z(25829), ae = Z(13582).overrideAll, oe = Z(93049).extendFlat, se = { title: { text: ee.title.text, font: ee.title.font }, color: ee.color, tickmode: ee.minor.tickmode, nticks: oe({}, ee.nticks, { dflt: 6, min: 1 }), tick0: ee.tick0, dtick: ee.dtick, tickvals: ee.tickvals, ticktext: ee.ticktext, ticks: ee.ticks, ticklen: ee.ticklen, tickwidth: ee.tickwidth, tickcolor: ee.tickcolor, ticklabelstep: ee.ticklabelstep, showticklabels: ee.showticklabels, labelalias: ee.labelalias, showtickprefix: ee.showtickprefix, tickprefix: ee.tickprefix, showticksuffix: ee.showticksuffix, ticksuffix: ee.ticksuffix, showexponent: ee.showexponent, exponentformat: ee.exponentformat, minexponent: ee.minexponent, separatethousands: ee.separatethousands, tickfont: ee.tickfont, tickangle: ee.tickangle, tickformat: ee.tickformat, tickformatstops: ee.tickformatstops, hoverformat: ee.hoverformat, showline: oe({}, ee.showline, { dflt: !0 }), linecolor: ee.linecolor, linewidth: ee.linewidth, showgrid: oe({}, ee.showgrid, { dflt: !0 }), gridcolor: ee.gridcolor, gridwidth: ee.gridwidth, griddash: ee.griddash, layer: ee.layer, min: { valType: "number", dflt: 0, min: 0 }, _deprecated: { title: ee._deprecated.title, titlefont: ee._deprecated.titlefont } }, ce = G.exports = ae({ domain: ne({ name: "ternary" }), bgcolor: { valType: "color", dflt: te.background }, sum: { valType: "number", dflt: 1, min: 0 }, aaxis: se, baxis: se, caxis: se }, "plot", "from-root");
            ce.uirevision = { valType: "any", editType: "none" }, ce.aaxis.uirevision = ce.baxis.uirevision = ce.caxis.uirevision = { valType: "any", editType: "none" };
          }, 25247: function(G, J, Z) {
            var te = Z(78766), ne = Z(78032), ee = Z(34809), ae = Z(4448), oe = Z(12036), se = Z(54616), ce = Z(87433), le = Z(22777), ue = Z(97405), de = Z(77416), fe = ["aaxis", "baxis", "caxis"];
            function he(pe, ye, ge, ve) {
              var be, Ce, Se, we = ge("bgcolor"), Ee = ge("sum");
              ve.bgColor = te.combine(we, ve.paper_bgcolor);
              for (var xe = 0; xe < fe.length; xe++) Ce = pe[be = fe[xe]] || {}, (Se = ne.newContainer(ye, be))._name = be, me(Ce, Se, ve, ye);
              var Le = ye.aaxis, Qe = ye.baxis, ke = ye.caxis;
              Le.min + Qe.min + ke.min >= Ee && (Le.min = 0, Qe.min = 0, ke.min = 0, pe.aaxis && delete pe.aaxis.min, pe.baxis && delete pe.baxis.min, pe.caxis && delete pe.caxis.min);
            }
            function me(pe, ye, ge, ve) {
              var be = de[ye._name];
              function Ce(Qe, ke) {
                return ee.coerce(pe, ye, be, Qe, ke);
              }
              Ce("uirevision", ve.uirevision), ye.type = "linear";
              var Se = Ce("color"), we = Se !== be.color.dflt ? Se : ge.font.color, Ee = ye._name.charAt(0).toUpperCase(), xe = "Component " + Ee, Le = Ce("title.text", xe);
              ye._hovertitle = Le === xe ? Le : Ee, ee.coerceFont(Ce, "title.font", ge.font, { overrideDflt: { size: ee.bigFont(ge.font.size), color: we } }), Ce("min"), le(pe, ye, Ce, "linear"), se(pe, ye, Ce, "linear"), oe(pe, ye, Ce, "linear", { noAutotickangles: !0, noTicklabelshift: !0, noTicklabelstandoff: !0 }), ce(pe, ye, Ce, { outerTicks: !0 }), Ce("showticklabels") && (ee.coerceFont(Ce, "tickfont", ge.font, { overrideDflt: { color: we } }), Ce("tickangle"), Ce("tickformat")), ue(pe, ye, Ce, { dfltColor: Se, bgColor: ge.bgColor, blend: 60, showLine: !0, showGrid: !0, noZeroLine: !0, attributes: be }), Ce("hoverformat"), Ce("layer");
            }
            G.exports = function(pe, ye, ge) {
              ae(pe, ye, ge, { type: "ternary", attributes: de, handleDefaults: he, font: ye.font, paper_bgcolor: ye.paper_bgcolor });
            };
          }, 83637: function(G, J, Z) {
            var te = Z(45568), ne = Z(65657), ee = Z(33626), ae = Z(34809), oe = ae.strTranslate, se = ae._, ce = Z(78766), le = Z(62203), ue = Z(19091), de = Z(93049).extendFlat, fe = Z(44122), he = Z(29714), me = Z(14751), pe = Z(32141), ye = Z(70414), ge = ye.freeMode, ve = ye.rectMode, be = Z(17240), Ce = Z(44844).prepSelect, Se = Z(44844).selectOnClick, we = Z(44844).clearOutline, Ee = Z(44844).clearSelectionsCache, xe = Z(54826);
            function Le(Xe, Ve) {
              this.id = Xe.id, this.graphDiv = Xe.graphDiv, this.init(Ve), this.makeFramework(Ve), this.updateFx(Ve), this.aTickLayout = null, this.bTickLayout = null, this.cTickLayout = null;
            }
            G.exports = Le;
            var Qe = Le.prototype;
            Qe.init = function(Xe) {
              this.container = Xe._ternarylayer, this.defs = Xe._defs, this.layoutId = Xe._uid, this.traceHash = {}, this.layers = {};
            }, Qe.plot = function(Xe, Ve) {
              var Ze = this, Ge = Ve[Ze.id], ze = Ve._size;
              Ze._hasClipOnAxisFalse = !1;
              for (var We = 0; We < Xe.length; We++) if (Xe[We][0].trace.cliponaxis === !1) {
                Ze._hasClipOnAxisFalse = !0;
                break;
              }
              Ze.updateLayers(Ge), Ze.adjustLayout(Ge, ze), fe.generalUpdatePerTraceModule(Ze.graphDiv, Ze, Xe, Ge), Ze.layers.plotbg.select("path").call(ce.fill, Ge.bgcolor);
            }, Qe.makeFramework = function(Xe) {
              var Ve = this, Ze = Ve.graphDiv, Ge = Xe[Ve.id], ze = Ve.clipId = "clip" + Ve.layoutId + Ve.id, We = Ve.clipIdRelative = "clip-relative" + Ve.layoutId + Ve.id;
              Ve.clipDef = ae.ensureSingleById(Xe._clips, "clipPath", ze, function(at) {
                at.append("path").attr("d", "M0,0Z");
              }), Ve.clipDefRelative = ae.ensureSingleById(Xe._clips, "clipPath", We, function(at) {
                at.append("path").attr("d", "M0,0Z");
              }), Ve.plotContainer = ae.ensureSingle(Ve.container, "g", Ve.id), Ve.updateLayers(Ge), le.setClipUrl(Ve.layers.backplot, ze, Ze), le.setClipUrl(Ve.layers.grids, ze, Ze);
            }, Qe.updateFx = function(Xe) {
              Xe._ternarylayer.selectAll("g.toplevel").style("cursor", Xe.dragmode === "pan" ? "move" : "crosshair");
            }, Qe.updateLayers = function(Xe) {
              var Ve = this.layers, Ze = ["draglayer", "plotbg", "backplot", "grids"];
              Xe.aaxis.layer === "below traces" && Ze.push("aaxis", "aline"), Xe.baxis.layer === "below traces" && Ze.push("baxis", "bline"), Xe.caxis.layer === "below traces" && Ze.push("caxis", "cline"), Ze.push("frontplot"), Xe.aaxis.layer === "above traces" && Ze.push("aaxis", "aline"), Xe.baxis.layer === "above traces" && Ze.push("baxis", "bline"), Xe.caxis.layer === "above traces" && Ze.push("caxis", "cline");
              var Ge = this.plotContainer.selectAll("g.toplevel").data(Ze, String), ze = ["agrid", "bgrid", "cgrid"];
              Ge.enter().append("g").attr("class", function(We) {
                return "toplevel " + We;
              }).each(function(We) {
                var at = te.select(this);
                Ve[We] = at, We === "frontplot" ? at.append("g").classed("scatterlayer", !0) : We === "backplot" ? at.append("g").classed("maplayer", !0) : We === "plotbg" ? at.append("path").attr("d", "M0,0Z") : We === "aline" || We === "bline" || We === "cline" ? at.append("path") : We === "grids" && ze.forEach(function(ct) {
                  Ve[ct] = at.append("g").classed("grid " + ct, !0);
                });
              }), Ge.order();
            };
            var ke = Math.sqrt(4 / 3);
            Qe.adjustLayout = function(Xe, Ve) {
              var Ze, Ge, ze, We, at, ct, et = this, dt = Xe.domain, Je = (dt.x[0] + dt.x[1]) / 2, it = (dt.y[0] + dt.y[1]) / 2, st = dt.x[1] - dt.x[0], gt = dt.y[1] - dt.y[0], bt = st * Ve.w, yt = gt * Ve.h, St = Xe.sum, Lt = Xe.aaxis.min, Rt = Xe.baxis.min, vt = Xe.caxis.min;
              bt > ke * yt ? ze = (We = yt) * ke : We = (ze = bt) / ke, at = st * ze / bt, ct = gt * We / yt, Ze = Ve.l + Ve.w * Je - ze / 2, Ge = Ve.t + Ve.h * (1 - it) - We / 2, et.x0 = Ze, et.y0 = Ge, et.w = ze, et.h = We, et.sum = St, et.xaxis = { type: "linear", range: [Lt + 2 * vt - St, St - Lt - 2 * Rt], domain: [Je - at / 2, Je + at / 2], _id: "x" }, ue(et.xaxis, et.graphDiv._fullLayout), et.xaxis.setScale(), et.xaxis.isPtWithinRange = function(Vt) {
                return Vt.a >= et.aaxis.range[0] && Vt.a <= et.aaxis.range[1] && Vt.b >= et.baxis.range[1] && Vt.b <= et.baxis.range[0] && Vt.c >= et.caxis.range[1] && Vt.c <= et.caxis.range[0];
              }, et.yaxis = { type: "linear", range: [Lt, St - Rt - vt], domain: [it - ct / 2, it + ct / 2], _id: "y" }, ue(et.yaxis, et.graphDiv._fullLayout), et.yaxis.setScale(), et.yaxis.isPtWithinRange = function() {
                return !0;
              };
              var $t = et.yaxis.domain[0], Pt = et.aaxis = de({}, Xe.aaxis, { range: [Lt, St - Rt - vt], side: "left", tickangle: (+Xe.aaxis.tickangle || 0) - 30, domain: [$t, $t + ct * ke], anchor: "free", position: 0, _id: "y", _length: ze });
              ue(Pt, et.graphDiv._fullLayout), Pt.setScale();
              var Xt = et.baxis = de({}, Xe.baxis, { range: [St - Lt - vt, Rt], side: "bottom", domain: et.xaxis.domain, anchor: "free", position: 0, _id: "x", _length: ze });
              ue(Xt, et.graphDiv._fullLayout), Xt.setScale();
              var Nt = et.caxis = de({}, Xe.caxis, { range: [St - Lt - Rt, vt], side: "right", tickangle: (+Xe.caxis.tickangle || 0) + 30, domain: [$t, $t + ct * ke], anchor: "free", position: 0, _id: "y", _length: ze });
              ue(Nt, et.graphDiv._fullLayout), Nt.setScale();
              var Gt = "M" + Ze + "," + (Ge + We) + "h" + ze + "l-" + ze / 2 + ",-" + We + "Z";
              et.clipDef.select("path").attr("d", Gt), et.layers.plotbg.select("path").attr("d", Gt);
              var Ft = "M0," + We + "h" + ze + "l-" + ze / 2 + ",-" + We + "Z";
              et.clipDefRelative.select("path").attr("d", Ft);
              var Tt = oe(Ze, Ge);
              et.plotContainer.selectAll(".scatterlayer,.maplayer").attr("transform", Tt), et.clipDefRelative.select("path").attr("transform", null);
              var It = oe(Ze - Xt._offset, Ge + We);
              et.layers.baxis.attr("transform", It), et.layers.bgrid.attr("transform", It);
              var Ct = oe(Ze + ze / 2, Ge) + "rotate(30)" + oe(0, -Pt._offset);
              et.layers.aaxis.attr("transform", Ct), et.layers.agrid.attr("transform", Ct);
              var Kt = oe(Ze + ze / 2, Ge) + "rotate(-30)" + oe(0, -Nt._offset);
              et.layers.caxis.attr("transform", Kt), et.layers.cgrid.attr("transform", Kt), et.drawAxes(!0), et.layers.aline.select("path").attr("d", Pt.showline ? "M" + Ze + "," + (Ge + We) + "l" + ze / 2 + ",-" + We : "M0,0").call(ce.stroke, Pt.linecolor || "#000").style("stroke-width", (Pt.linewidth || 0) + "px"), et.layers.bline.select("path").attr("d", Xt.showline ? "M" + Ze + "," + (Ge + We) + "h" + ze : "M0,0").call(ce.stroke, Xt.linecolor || "#000").style("stroke-width", (Xt.linewidth || 0) + "px"), et.layers.cline.select("path").attr("d", Nt.showline ? "M" + (Ze + ze / 2) + "," + Ge + "l" + ze / 2 + "," + We : "M0,0").call(ce.stroke, Nt.linecolor || "#000").style("stroke-width", (Nt.linewidth || 0) + "px"), et.graphDiv._context.staticPlot || et.initInteractions(), le.setClipUrl(et.layers.frontplot, et._hasClipOnAxisFalse ? null : et.clipId, et.graphDiv);
            }, Qe.drawAxes = function(Xe) {
              var Ve = this, Ze = Ve.graphDiv, Ge = Ve.id.substr(7) + "title", ze = Ve.layers, We = Ve.aaxis, at = Ve.baxis, ct = Ve.caxis;
              if (Ve.drawAx(We), Ve.drawAx(at), Ve.drawAx(ct), Xe) {
                var et = Math.max(We.showticklabels ? We.tickfont.size / 2 : 0, (ct.showticklabels ? 0.75 * ct.tickfont.size : 0) + (ct.ticks === "outside" ? 0.87 * ct.ticklen : 0)), dt = (at.showticklabels ? at.tickfont.size : 0) + (at.ticks === "outside" ? at.ticklen : 0) + 3;
                ze["a-title"] = be.draw(Ze, "a" + Ge, { propContainer: We, propName: Ve.id + ".aaxis.title", placeholder: se(Ze, "Click to enter Component A title"), attributes: { x: Ve.x0 + Ve.w / 2, y: Ve.y0 - We.title.font.size / 3 - et, "text-anchor": "middle" } }), ze["b-title"] = be.draw(Ze, "b" + Ge, { propContainer: at, propName: Ve.id + ".baxis.title", placeholder: se(Ze, "Click to enter Component B title"), attributes: { x: Ve.x0 - dt, y: Ve.y0 + Ve.h + 0.83 * at.title.font.size + dt, "text-anchor": "middle" } }), ze["c-title"] = be.draw(Ze, "c" + Ge, { propContainer: ct, propName: Ve.id + ".caxis.title", placeholder: se(Ze, "Click to enter Component C title"), attributes: { x: Ve.x0 + Ve.w + dt, y: Ve.y0 + Ve.h + 0.83 * ct.title.font.size + dt, "text-anchor": "middle" } });
              }
            }, Qe.drawAx = function(Xe) {
              var Ve, Ze = this, Ge = Ze.graphDiv, ze = Xe._name, We = ze.charAt(0), at = Xe._id, ct = Ze.layers[ze], et = We + "tickLayout", dt = (Ve = Xe).ticks + String(Ve.ticklen) + String(Ve.showticklabels);
              Ze[et] !== dt && (ct.selectAll("." + at + "tick").remove(), Ze[et] = dt), Xe.setScale();
              var Je = he.calcTicks(Xe), it = he.clipEnds(Xe, Je), st = he.makeTransTickFn(Xe), gt = he.getTickSigns(Xe)[2], bt = ae.deg2rad(30), yt = gt * (Xe.linewidth || 1) / 2, St = gt * Xe.ticklen, Lt = Ze.w, Rt = Ze.h, vt = We === "b" ? "M0," + yt + "l" + Math.sin(bt) * St + "," + Math.cos(bt) * St : "M" + yt + ",0l" + Math.cos(bt) * St + "," + -Math.sin(bt) * St, $t = { a: "M0,0l" + Rt + ",-" + Lt / 2, b: "M0,0l-" + Lt / 2 + ",-" + Rt, c: "M0,0l-" + Rt + "," + Lt / 2 }[We];
              he.drawTicks(Ge, Xe, { vals: Xe.ticks === "inside" ? it : Je, layer: ct, path: vt, transFn: st, crisp: !1 }), he.drawGrid(Ge, Xe, { vals: it, layer: Ze.layers[We + "grid"], path: $t, transFn: st, crisp: !1 }), he.drawLabels(Ge, Xe, { vals: Je, layer: ct, transFn: st, labelFns: he.makeLabelFns(Xe, 0, 30) });
            };
            var Me = xe.MINZOOM / 2 + 0.87, Re = "m-0.87,.5h" + Me + "v3h-" + (Me + 5.2) + "l" + (Me / 2 + 2.6) + ",-" + (0.87 * Me + 4.5) + "l2.6,1.5l-" + Me / 2 + "," + 0.87 * Me + "Z", De = "m0.87,.5h-" + Me + "v3h" + (Me + 5.2) + "l-" + (Me / 2 + 2.6) + ",-" + (0.87 * Me + 4.5) + "l-2.6,1.5l" + Me / 2 + "," + 0.87 * Me + "Z", Pe = "m0,1l" + Me / 2 + "," + 0.87 * Me + "l2.6,-1.5l-" + (Me / 2 + 2.6) + ",-" + (0.87 * Me + 4.5) + "l-" + (Me / 2 + 2.6) + "," + (0.87 * Me + 4.5) + "l2.6,1.5l" + Me / 2 + ",-" + 0.87 * Me + "Z", $e = !0;
            function qe(Xe) {
              te.select(Xe).selectAll(".zoombox,.js-zoombox-backdrop,.js-zoombox-menu,.zoombox-corners").remove();
            }
            Qe.clearOutline = function() {
              Ee(this.dragOptions), we(this.dragOptions.gd);
            }, Qe.initInteractions = function() {
              var Xe, Ve, Ze, Ge, ze, We, at, ct, et, dt, Je, it, st = this, gt = st.layers.plotbg.select("path").node(), bt = st.graphDiv, yt = bt._fullLayout._zoomlayer;
              function St(Ft) {
                var Tt = {};
                return Tt[st.id + ".aaxis.min"] = Ft.a, Tt[st.id + ".baxis.min"] = Ft.b, Tt[st.id + ".caxis.min"] = Ft.c, Tt;
              }
              function Lt(Ft, Tt) {
                var It = bt._fullLayout.clickmode;
                qe(bt), Ft === 2 && (bt.emit("plotly_doubleclick", null), ee.call("_guiRelayout", bt, St({ a: 0, b: 0, c: 0 }))), It.indexOf("select") > -1 && Ft === 1 && Se(Tt, bt, [st.xaxis], [st.yaxis], st.id, st.dragOptions), It.indexOf("event") > -1 && pe.click(bt, Tt, st.id);
              }
              function Rt(Ft, Tt) {
                return 1 - Tt / st.h;
              }
              function vt(Ft, Tt) {
                return 1 - (Ft + (st.h - Tt) / Math.sqrt(3)) / st.w;
              }
              function $t(Ft, Tt) {
                return (Ft - (st.h - Tt) / Math.sqrt(3)) / st.w;
              }
              function Pt(Ft, Tt) {
                var It = Ze + Ft * Xe, Ct = Ge + Tt * Ve, Kt = Math.max(0, Math.min(1, Rt(0, Ge), Rt(0, Ct))), Vt = Math.max(0, Math.min(1, vt(Ze, Ge), vt(It, Ct))), Yt = Math.max(0, Math.min(1, $t(Ze, Ge), $t(It, Ct))), sn = (Kt / 2 + Yt) * st.w, hn = (1 - Kt / 2 - Vt) * st.w, Qt = (sn + hn) / 2, Ut = hn - sn, Wt = (1 - Kt) * st.h, rn = Wt - Ut / ke;
                Ut < xe.MINZOOM ? (at = ze, Je.attr("d", et), it.attr("d", "M0,0Z")) : (at = { a: ze.a + Kt * We, b: ze.b + Vt * We, c: ze.c + Yt * We }, Je.attr("d", et + "M" + sn + "," + Wt + "H" + hn + "L" + Qt + "," + rn + "L" + sn + "," + Wt + "Z"), it.attr("d", "M" + Ze + "," + Ge + "m0.5,0.5h5v-2h-5v-5h-2v5h-5v2h5v5h2ZM" + sn + "," + Wt + Re + "M" + hn + "," + Wt + De + "M" + Qt + "," + rn + Pe)), dt || (Je.transition().style("fill", ct > 0.2 ? "rgba(0,0,0,0.4)" : "rgba(255,255,255,0.3)").duration(200), it.transition().style("opacity", 1).duration(200), dt = !0), bt.emit("plotly_relayouting", St(at));
              }
              function Xt() {
                qe(bt), at !== ze && (ee.call("_guiRelayout", bt, St(at)), $e && bt.data && bt._context.showTips && (ae.notifier(se(bt, "Double-click to zoom back out"), "long"), $e = !1));
              }
              function Nt(Ft, Tt) {
                var It = Ft / st.xaxis._m, Ct = Tt / st.yaxis._m, Kt = [(at = { a: ze.a - Ct, b: ze.b + (It + Ct) / 2, c: ze.c - (It - Ct) / 2 }).a, at.b, at.c].sort(ae.sorterAsc), Vt = Kt.indexOf(at.a), Yt = Kt.indexOf(at.b), sn = Kt.indexOf(at.c);
                Kt[0] < 0 && (Kt[1] + Kt[0] / 2 < 0 ? (Kt[2] += Kt[0] + Kt[1], Kt[0] = Kt[1] = 0) : (Kt[2] += Kt[0] / 2, Kt[1] += Kt[0] / 2, Kt[0] = 0), at = { a: Kt[Vt], b: Kt[Yt], c: Kt[sn] }, Tt = (ze.a - at.a) * st.yaxis._m, Ft = (ze.c - at.c - ze.b + at.b) * st.xaxis._m);
                var hn = oe(st.x0 + Ft, st.y0 + Tt);
                st.plotContainer.selectAll(".scatterlayer,.maplayer").attr("transform", hn);
                var Qt = oe(-Ft, -Tt);
                st.clipDefRelative.select("path").attr("transform", Qt), st.aaxis.range = [at.a, st.sum - at.b - at.c], st.baxis.range = [st.sum - at.a - at.c, at.b], st.caxis.range = [st.sum - at.a - at.b, at.c], st.drawAxes(!1), st._hasClipOnAxisFalse && st.plotContainer.select(".scatterlayer").selectAll(".trace").call(le.hideOutsideRangePoints, st), bt.emit("plotly_relayouting", St(at));
              }
              function Gt() {
                ee.call("_guiRelayout", bt, St(at));
              }
              this.dragOptions = { element: gt, gd: bt, plotinfo: { id: st.id, domain: bt._fullLayout[st.id].domain, xaxis: st.xaxis, yaxis: st.yaxis }, subplot: st.id, prepFn: function(Ft, Tt, It) {
                st.dragOptions.xaxes = [st.xaxis], st.dragOptions.yaxes = [st.yaxis], Xe = bt._fullLayout._invScaleX, Ve = bt._fullLayout._invScaleY;
                var Ct = st.dragOptions.dragmode = bt._fullLayout.dragmode;
                ge(Ct) ? st.dragOptions.minDrag = 1 : st.dragOptions.minDrag = void 0, Ct === "zoom" ? (st.dragOptions.moveFn = Pt, st.dragOptions.clickFn = Lt, st.dragOptions.doneFn = Xt, function(Kt, Vt, Yt) {
                  var sn = gt.getBoundingClientRect();
                  Ze = Vt - sn.left, Ge = Yt - sn.top, bt._fullLayout._calcInverseTransform(bt);
                  var hn = bt._fullLayout._invTransform, Qt = ae.apply3DTransform(hn)(Ze, Ge);
                  Ze = Qt[0], Ge = Qt[1], ze = { a: st.aaxis.range[0], b: st.baxis.range[1], c: st.caxis.range[1] }, at = ze, We = st.aaxis.range[1] - ze.a, ct = ne(st.graphDiv._fullLayout[st.id].bgcolor).getLuminance(), et = "M0," + st.h + "L" + st.w / 2 + ", 0L" + st.w + "," + st.h + "Z", dt = !1, Je = yt.append("path").attr("class", "zoombox").attr("transform", oe(st.x0, st.y0)).style({ fill: ct > 0.2 ? "rgba(0,0,0,0)" : "rgba(255,255,255,0)", "stroke-width": 0 }).attr("d", et), it = yt.append("path").attr("class", "zoombox-corners").attr("transform", oe(st.x0, st.y0)).style({ fill: ce.background, stroke: ce.defaultLine, "stroke-width": 1, opacity: 0 }).attr("d", "M0,0Z"), st.clearOutline(bt);
                }(0, Tt, It)) : Ct === "pan" ? (st.dragOptions.moveFn = Nt, st.dragOptions.clickFn = Lt, st.dragOptions.doneFn = Gt, ze = { a: st.aaxis.range[0], b: st.baxis.range[1], c: st.caxis.range[1] }, at = ze, st.clearOutline(bt)) : (ve(Ct) || ge(Ct)) && Ce(Ft, Tt, It, st.dragOptions, Ct);
              } }, gt.onmousemove = function(Ft) {
                pe.hover(bt, Ft, st.id), bt._fullLayout._lasthover = gt, bt._fullLayout._hoversubplot = st.id;
              }, gt.onmouseout = function(Ft) {
                bt._dragging || me.unhover(bt, Ft);
              }, me.init(this.dragOptions);
            };
          }, 33626: function(G, J, Z) {
            var te = Z(48636), ne = Z(4969), ee = Z(36539), ae = Z(56174), oe = Z(95425).addStyleRule, se = Z(93049), ce = Z(9829), le = Z(6704), ue = se.extendFlat, de = se.extendDeepAll;
            function fe(Se) {
              var we = Se.name, Ee = Se.categories, xe = Se.meta;
              if (J.modules[we]) te.log("Type " + we + " already registered");
              else {
                J.subplotsRegistry[Se.basePlotModule.name] || function($e) {
                  var qe = $e.name;
                  if (J.subplotsRegistry[qe]) te.log("Plot type " + qe + " already registered.");
                  else for (var Xe in ye($e), J.subplotsRegistry[qe] = $e, J.componentsRegistry) be(Xe, $e.name);
                }(Se.basePlotModule);
                for (var Le = {}, Qe = 0; Qe < Ee.length; Qe++) Le[Ee[Qe]] = !0, J.allCategories[Ee[Qe]] = !0;
                for (var ke in J.modules[we] = { _module: Se, categories: Le }, xe && Object.keys(xe).length && (J.modules[we].meta = xe), J.allTypes.push(we), J.componentsRegistry) ge(ke, we);
                Se.layoutAttributes && ue(J.traceLayoutAttributes, Se.layoutAttributes);
                var Me = Se.basePlotModule, Re = Me.name;
                if (Re === "mapbox") {
                  var De = Me.constants.styleRules;
                  for (var Pe in De) oe(".js-plotly-plot .plotly .mapboxgl-" + Pe, De[Pe]);
                }
                Re === "map" && Z(96144), Re !== "geo" && Re !== "mapbox" && Re !== "map" || window.PlotlyGeoAssets !== void 0 || (window.PlotlyGeoAssets = { topojson: {} });
              }
            }
            function he(Se) {
              if (typeof Se.name != "string") throw new Error("Component module *name* must be a string.");
              var we = Se.name;
              for (var Ee in J.componentsRegistry[we] = Se, Se.layoutAttributes && (Se.layoutAttributes._isLinkedToArray && ee(J.layoutArrayContainers, we), ye(Se)), J.modules) ge(we, Ee);
              for (var xe in J.subplotsRegistry) be(we, xe);
              for (var Le in J.transformsRegistry) ve(we, Le);
              Se.schema && Se.schema.layout && de(le, Se.schema.layout);
            }
            function me(Se) {
              if (typeof Se.name != "string") throw new Error("Transform module *name* must be a string.");
              var we = "Transform module " + Se.name, Ee = typeof Se.transform == "function", xe = typeof Se.calcTransform == "function";
              if (!Ee && !xe) throw new Error(we + " is missing a *transform* or *calcTransform* method.");
              for (var Le in Ee && xe && te.log([we + " has both a *transform* and *calcTransform* methods.", "Please note that all *transform* methods are executed", "before all *calcTransform* methods."].join(" ")), ae(Se.attributes) || te.log(we + " registered without an *attributes* object."), typeof Se.supplyDefaults != "function" && te.log(we + " registered without a *supplyDefaults* method."), J.transformsRegistry[Se.name] = Se, J.componentsRegistry) ve(Le, Se.name);
            }
            function pe(Se) {
              var we = Se.name, Ee = we.split("-")[0], xe = Se.dictionary, Le = Se.format, Qe = xe && Object.keys(xe).length, ke = Le && Object.keys(Le).length, Me = J.localeRegistry, Re = Me[we];
              if (Re || (Me[we] = Re = {}), Ee !== we) {
                var De = Me[Ee];
                De || (Me[Ee] = De = {}), Qe && De.dictionary === Re.dictionary && (De.dictionary = xe), ke && De.format === Re.format && (De.format = Le);
              }
              Qe && (Re.dictionary = xe), ke && (Re.format = Le);
            }
            function ye(Se) {
              if (Se.layoutAttributes) {
                var we = Se.layoutAttributes._arrayAttrRegexps;
                if (we) for (var Ee = 0; Ee < we.length; Ee++) ee(J.layoutArrayRegexes, we[Ee]);
              }
            }
            function ge(Se, we) {
              var Ee = J.componentsRegistry[Se].schema;
              if (Ee && Ee.traces) {
                var xe = Ee.traces[we];
                xe && de(J.modules[we]._module.attributes, xe);
              }
            }
            function ve(Se, we) {
              var Ee = J.componentsRegistry[Se].schema;
              if (Ee && Ee.transforms) {
                var xe = Ee.transforms[we];
                xe && de(J.transformsRegistry[we].attributes, xe);
              }
            }
            function be(Se, we) {
              var Ee = J.componentsRegistry[Se].schema;
              if (Ee && Ee.subplots) {
                var xe = J.subplotsRegistry[we], Le = xe.layoutAttributes, Qe = xe.attr === "subplot" ? xe.name : xe.attr;
                Array.isArray(Qe) && (Qe = Qe[0]);
                var ke = Ee.subplots[Qe];
                Le && ke && de(Le, ke);
              }
            }
            function Ce(Se) {
              return typeof Se == "object" && (Se = Se.type), Se;
            }
            J.modules = {}, J.allCategories = {}, J.allTypes = [], J.subplotsRegistry = {}, J.transformsRegistry = {}, J.componentsRegistry = {}, J.layoutArrayContainers = [], J.layoutArrayRegexes = [], J.traceLayoutAttributes = {}, J.localeRegistry = {}, J.apiMethodRegistry = {}, J.collectableSubplotTypes = null, J.register = function(Se) {
              if (J.collectableSubplotTypes = null, !Se) throw new Error("No argument passed to Plotly.register.");
              Se && !Array.isArray(Se) && (Se = [Se]);
              for (var we = 0; we < Se.length; we++) {
                var Ee = Se[we];
                if (!Ee) throw new Error("Invalid module was attempted to be registered!");
                switch (Ee.moduleType) {
                  case "trace":
                    fe(Ee);
                    break;
                  case "transform":
                    me(Ee);
                    break;
                  case "component":
                    he(Ee);
                    break;
                  case "locale":
                    pe(Ee);
                    break;
                  case "apiMethod":
                    var xe = Ee.name;
                    J.apiMethodRegistry[xe] = Ee.fn;
                    break;
                  default:
                    throw new Error("Invalid module was attempted to be registered!");
                }
              }
            }, J.getModule = function(Se) {
              var we = J.modules[Ce(Se)];
              return !!we && we._module;
            }, J.traceIs = function(Se, we) {
              if ((Se = Ce(Se)) === "various") return !1;
              var Ee = J.modules[Se];
              return Ee || (Se && te.log("Unrecognized trace type " + Se + "."), Ee = J.modules[ce.type.dflt]), !!Ee.categories[we];
            }, J.getTransformIndices = function(Se, we) {
              for (var Ee = [], xe = Se.transforms || [], Le = 0; Le < xe.length; Le++) xe[Le].type === we && Ee.push(Le);
              return Ee;
            }, J.hasTransform = function(Se, we) {
              for (var Ee = Se.transforms || [], xe = 0; xe < Ee.length; xe++) if (Ee[xe].type === we) return !0;
              return !1;
            }, J.getComponentMethod = function(Se, we) {
              var Ee = J.componentsRegistry[Se];
              return Ee && Ee[we] || ne;
            }, J.call = function() {
              var Se = arguments[0], we = [].slice.call(arguments, 1);
              return J.apiMethodRegistry[Se].apply(null, we);
            };
          }, 3164: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = ne.extendFlat, ae = ne.extendDeep;
            function oe(se) {
              var ce;
              switch (se) {
                case "themes__thumb":
                  ce = { autosize: !0, width: 150, height: 150, title: { text: "" }, showlegend: !1, margin: { l: 5, r: 5, t: 5, b: 5, pad: 0 }, annotations: [] };
                  break;
                case "thumbnail":
                  ce = { title: { text: "" }, hidesources: !0, showlegend: !1, borderwidth: 0, bordercolor: "", margin: { l: 1, r: 1, t: 1, b: 1, pad: 0 }, annotations: [] };
                  break;
                default:
                  ce = {};
              }
              return ce;
            }
            G.exports = function(se, ce) {
              var le, ue, de = se.data, fe = se.layout, he = ae([], de), me = ae({}, fe, oe(ce.tileClass)), pe = se._context || {};
              if (ce.width && (me.width = ce.width), ce.height && (me.height = ce.height), ce.tileClass === "thumbnail" || ce.tileClass === "themes__thumb") {
                me.annotations = [];
                var ye = Object.keys(me);
                for (le = 0; le < ye.length; le++) ue = ye[le], ["xaxis", "yaxis", "zaxis"].indexOf(ue.slice(0, 5)) > -1 && (me[ye[le]].title = { text: "" });
                for (le = 0; le < he.length; le++) {
                  var ge = he[le];
                  ge.showscale = !1, ge.marker && (ge.marker.showscale = !1), te.traceIs(ge, "pie-like") && (ge.textposition = "none");
                }
              }
              if (Array.isArray(ce.annotations)) for (le = 0; le < ce.annotations.length; le++) me.annotations.push(ce.annotations[le]);
              var ve = Object.keys(me).filter(function(Ee) {
                return Ee.match(/^scene\d*$/);
              });
              if (ve.length) {
                var be = {};
                for (ce.tileClass === "thumbnail" && (be = { title: { text: "" }, showaxeslabels: !1, showticklabels: !1, linetickenable: !1 }), le = 0; le < ve.length; le++) {
                  var Ce = me[ve[le]];
                  Ce.xaxis || (Ce.xaxis = {}), Ce.yaxis || (Ce.yaxis = {}), Ce.zaxis || (Ce.zaxis = {}), ee(Ce.xaxis, be), ee(Ce.yaxis, be), ee(Ce.zaxis, be), Ce._scene = null;
                }
              }
              var Se = document.createElement("div");
              ce.tileClass && (Se.className = ce.tileClass);
              var we = { gd: Se, td: Se, layout: me, data: he, config: { staticPlot: ce.staticPlot === void 0 || ce.staticPlot, plotGlPixelRatio: ce.plotGlPixelRatio === void 0 ? 2 : ce.plotGlPixelRatio, displaylogo: ce.displaylogo || !1, showLink: ce.showLink || !1, showTips: ce.showTips || !1, mapboxAccessToken: pe.mapboxAccessToken } };
              return ce.setBackground !== "transparent" && (we.config.setBackground = ce.setBackground || "opaque"), we.gd.defaultLayout = oe(ce.tileClass), we;
            };
          }, 26452: function(G, J, Z) {
            var te = Z(34809), ne = Z(80491), ee = Z(33353), ae = Z(84619);
            G.exports = function(oe, se) {
              var ce;
              return te.isPlainObject(oe) || (ce = te.getGraphDiv(oe)), (se = se || {}).format = se.format || "png", se.width = se.width || null, se.height = se.height || null, se.imageDataOnly = !0, new Promise(function(le, ue) {
                ce && ce._snapshotInProgress && ue(new Error("Snapshotting already in progress.")), te.isIE() && se.format !== "svg" && ue(new Error(ae.MSG_IE_BAD_FORMAT)), ce && (ce._snapshotInProgress = !0);
                var de = ne(oe, se), fe = se.filename || oe.fn || "newplot";
                fe += "." + se.format.replace("-", "."), de.then(function(he) {
                  return ce && (ce._snapshotInProgress = !1), ee(he, fe, se.format);
                }).then(function(he) {
                  le(he);
                }).catch(function(he) {
                  ce && (ce._snapshotInProgress = !1), ue(he);
                });
              });
            };
          }, 33353: function(G, J, Z) {
            var te = Z(34809), ne = Z(84619);
            G.exports = function(ee, ae, oe) {
              var se = document.createElement("a"), ce = "download" in se;
              return new Promise(function(le, ue) {
                var de, fe;
                if (te.isIE()) return de = ne.createBlob(ee, "svg"), window.navigator.msSaveBlob(de, ae), de = null, le(ae);
                if (ce) return de = ne.createBlob(ee, oe), fe = ne.createObjectURL(de), se.href = fe, se.download = ae, document.body.appendChild(se), se.click(), document.body.removeChild(se), ne.revokeObjectURL(fe), de = null, le(ae);
                if (te.isSafari()) {
                  var he = oe === "svg" ? "," : ";base64,";
                  return ne.octetStream(he + encodeURIComponent(ee)), le(ae);
                }
                ue(new Error("download error"));
              });
            };
          }, 84619: function(G, J, Z) {
            var te = Z(33626);
            J.getDelay = function(ee) {
              return ee._has && (ee._has("gl3d") || ee._has("gl2d") || ee._has("mapbox") || ee._has("map")) ? 500 : 0;
            }, J.getRedrawFunc = function(ee) {
              return function() {
                te.getComponentMethod("colorbar", "draw")(ee);
              };
            }, J.encodeSVG = function(ee) {
              return "data:image/svg+xml," + encodeURIComponent(ee);
            }, J.encodeJSON = function(ee) {
              return "data:application/json," + encodeURIComponent(ee);
            };
            var ne = window.URL || window.webkitURL;
            J.createObjectURL = function(ee) {
              return ne.createObjectURL(ee);
            }, J.revokeObjectURL = function(ee) {
              return ne.revokeObjectURL(ee);
            }, J.createBlob = function(ee, ae) {
              if (ae === "svg") return new window.Blob([ee], { type: "image/svg+xml;charset=utf-8" });
              if (ae === "full-json") return new window.Blob([ee], { type: "application/json;charset=utf-8" });
              var oe = function(se) {
                for (var ce = se.length, le = new ArrayBuffer(ce), ue = new Uint8Array(le), de = 0; de < ce; de++) ue[de] = se.charCodeAt(de);
                return le;
              }(window.atob(ee));
              return new window.Blob([oe], { type: "image/" + ae });
            }, J.octetStream = function(ee) {
              document.location.href = "data:application/octet-stream" + ee;
            }, J.IMAGE_URL_PREFIX = /^data:image\/\w+;base64,/, J.MSG_IE_BAD_FORMAT = "Sorry IE does not support downloading from canvas. Try {format:'svg'} instead.";
          }, 6170: function(G, J, Z) {
            var te = Z(84619), ne = { getDelay: te.getDelay, getRedrawFunc: te.getRedrawFunc, clone: Z(3164), toSVG: Z(6243), svgToImg: Z(72914), toImage: Z(76896), downloadImage: Z(26452) };
            G.exports = ne;
          }, 72914: function(G, J, Z) {
            var te = Z(34809), ne = Z(7683).EventEmitter, ee = Z(84619);
            G.exports = function(ae) {
              var oe = ae.emitter || new ne(), se = new Promise(function(ce, le) {
                var ue = window.Image, de = ae.svg, fe = ae.format || "png";
                if (te.isIE() && fe !== "svg") {
                  var he = new Error(ee.MSG_IE_BAD_FORMAT);
                  return le(he), ae.promise ? se : oe.emit("error", he);
                }
                var me, pe, ye = ae.canvas, ge = ae.scale || 1, ve = ae.width || 300, be = ae.height || 150, Ce = ge * ve, Se = ge * be, we = ye.getContext("2d", { willReadFrequently: !0 }), Ee = new ue();
                fe === "svg" || te.isSafari() ? pe = ee.encodeSVG(de) : (me = ee.createBlob(de, "svg"), pe = ee.createObjectURL(me)), ye.width = Ce, ye.height = Se, Ee.onload = function() {
                  var xe;
                  switch (me = null, ee.revokeObjectURL(pe), fe !== "svg" && we.drawImage(Ee, 0, 0, Ce, Se), fe) {
                    case "jpeg":
                      xe = ye.toDataURL("image/jpeg");
                      break;
                    case "png":
                      xe = ye.toDataURL("image/png");
                      break;
                    case "webp":
                      xe = ye.toDataURL("image/webp");
                      break;
                    case "svg":
                      xe = pe;
                      break;
                    default:
                      var Le = "Image format is not jpeg, png, svg or webp.";
                      if (le(new Error(Le)), !ae.promise) return oe.emit("error", Le);
                  }
                  ce(xe), ae.promise || oe.emit("success", xe);
                }, Ee.onerror = function(xe) {
                  if (me = null, ee.revokeObjectURL(pe), le(xe), !ae.promise) return oe.emit("error", xe);
                }, Ee.src = pe;
              });
              return ae.promise ? se : oe;
            };
          }, 76896: function(G, J, Z) {
            var te = Z(7683).EventEmitter, ne = Z(33626), ee = Z(34809), ae = Z(84619), oe = Z(3164), se = Z(6243), ce = Z(72914);
            G.exports = function(le, ue) {
              var de = new te(), fe = oe(le, { format: "png" }), he = fe.gd;
              he.style.position = "absolute", he.style.left = "-5000px", document.body.appendChild(he);
              var me = ae.getRedrawFunc(he);
              return ne.call("_doPlot", he, fe.data, fe.layout, fe.config).then(me).then(function() {
                var pe = ae.getDelay(he._fullLayout);
                setTimeout(function() {
                  var ye = se(he), ge = document.createElement("canvas");
                  ge.id = ee.randstr(), (de = ce({ format: ue.format, width: he._fullLayout.width, height: he._fullLayout.height, canvas: ge, emitter: de, svg: ye })).clean = function() {
                    he && document.body.removeChild(he);
                  };
                }, pe);
              }).catch(function(pe) {
                de.emit("error", pe);
              }), de;
            };
          }, 6243: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(78766), oe = Z(62972), se = /"/g, ce = "TOBESTRIPPED", le = new RegExp('("' + ce + ")|(" + ce + '")', "g");
            G.exports = function(ue, de, fe) {
              var he, me = ue._fullLayout, pe = me._paper, ye = me._toppaper, ge = me.width, ve = me.height;
              pe.insert("rect", ":first-child").call(ee.setRect, 0, 0, ge, ve).call(ae.fill, me.paper_bgcolor);
              var be = me._basePlotModules || [];
              for (he = 0; he < be.length; he++) {
                var Ce = be[he];
                Ce.toSVG && Ce.toSVG(ue);
              }
              if (ye) {
                var Se = ye.node().childNodes, we = Array.prototype.slice.call(Se);
                for (he = 0; he < we.length; he++) {
                  var Ee = we[he];
                  Ee.childNodes.length && pe.node().appendChild(Ee);
                }
              }
              me._draggers && me._draggers.remove(), pe.node().style.background = "", pe.selectAll("text").attr({ "data-unformatted": null, "data-math": null }).each(function() {
                var Le = te.select(this);
                if (this.style.visibility !== "hidden" && this.style.display !== "none") {
                  Le.style({ visibility: null, display: null });
                  var Qe = this.style.fontFamily;
                  Qe && Qe.indexOf('"') !== -1 && Le.style("font-family", Qe.replace(se, ce));
                  var ke = this.style.fontWeight;
                  !ke || ke !== "normal" && ke !== "400" || Le.style("font-weight", void 0);
                  var Me = this.style.fontStyle;
                  Me && Me === "normal" && Le.style("font-style", void 0);
                  var Re = this.style.fontVariant;
                  Re && Re === "normal" && Le.style("font-variant", void 0);
                } else Le.remove();
              }), pe.selectAll(".gradient_filled,.pattern_filled").each(function() {
                var Le = te.select(this), Qe = this.style.fill;
                Qe && Qe.indexOf("url(") !== -1 && Le.style("fill", Qe.replace(se, ce));
                var ke = this.style.stroke;
                ke && ke.indexOf("url(") !== -1 && Le.style("stroke", ke.replace(se, ce));
              }), de !== "pdf" && de !== "eps" || pe.selectAll("#MathJax_SVG_glyphs path").attr("stroke-width", 0), pe.node().setAttributeNS(oe.xmlns, "xmlns", oe.svg), pe.node().setAttributeNS(oe.xmlns, "xmlns:xlink", oe.xlink), de === "svg" && fe && (pe.attr("width", fe * ge), pe.attr("height", fe * ve), pe.attr("viewBox", "0 0 " + ge + " " + ve));
              var xe = new window.XMLSerializer().serializeToString(pe.node());
              return xe = (xe = (xe = function(Le) {
                var Qe = te.select("body").append("div").style({ display: "none" }).html(""), ke = Le.replace(/(&[^;]*;)/gi, function(Me) {
                  return Me === "&lt;" ? "&#60;" : Me === "&rt;" ? "&#62;" : Me.indexOf("<") !== -1 || Me.indexOf(">") !== -1 ? "" : Qe.html(Me).text();
                });
                return Qe.remove(), ke;
              }(xe)).replace(/&(?!\w+;|\#[0-9]+;| \#x[0-9A-F]+;)/g, "&amp;")).replace(le, "'"), ne.isIE() && (xe = (xe = (xe = xe.replace(/"/gi, "'")).replace(/(\('#)([^']*)('\))/gi, '("#$2")')).replace(/(\\')/gi, '"')), xe;
            };
          }, 35374: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              for (var ae = 0; ae < ne.length; ae++) ne[ae].i = ae;
              te.mergeArray(ee.text, ne, "tx"), te.mergeArray(ee.hovertext, ne, "htx");
              var oe = ee.marker;
              if (oe) {
                te.mergeArray(oe.opacity, ne, "mo", !0), te.mergeArray(oe.color, ne, "mc");
                var se = oe.line;
                se && (te.mergeArray(se.color, ne, "mlc"), te.mergeArrayCastPositive(se.width, ne, "mlw"));
              }
            };
          }, 81481: function(G, J, Z) {
            var te = Z(36640), ne = Z(80712).axisHoverFormat, ee = Z(3208).rb, ae = Z(3208).ay, oe = Z(87163), se = Z(80337), ce = Z(56155), le = Z(94850).k, ue = Z(93049).extendFlat, de = se({ editType: "calc", arrayOk: !0, colorEditType: "style" }), fe = ue({}, te.marker.line.width, { dflt: 0 }), he = ue({ width: fe, editType: "calc" }, oe("marker.line")), me = ue({ line: he, editType: "calc" }, oe("marker"), { opacity: { valType: "number", arrayOk: !0, dflt: 1, min: 0, max: 1, editType: "style" }, pattern: le, cornerradius: { valType: "any", editType: "calc" } });
            G.exports = { x: te.x, x0: te.x0, dx: te.dx, y: te.y, y0: te.y0, dy: te.dy, xperiod: te.xperiod, yperiod: te.yperiod, xperiod0: te.xperiod0, yperiod0: te.yperiod0, xperiodalignment: te.xperiodalignment, yperiodalignment: te.yperiodalignment, xhoverformat: ne("x"), yhoverformat: ne("y"), text: te.text, texttemplate: ae({ editType: "plot" }, { keys: ce.eventDataKeys }), hovertext: te.hovertext, hovertemplate: ee({}, { keys: ce.eventDataKeys }), textposition: { valType: "enumerated", values: ["inside", "outside", "auto", "none"], dflt: "auto", arrayOk: !0, editType: "calc" }, insidetextanchor: { valType: "enumerated", values: ["end", "middle", "start"], dflt: "end", editType: "plot" }, textangle: { valType: "angle", dflt: "auto", editType: "plot" }, textfont: ue({}, de, {}), insidetextfont: ue({}, de, {}), outsidetextfont: ue({}, de, {}), constraintext: { valType: "enumerated", values: ["inside", "outside", "both", "none"], dflt: "both", editType: "calc" }, cliponaxis: ue({}, te.cliponaxis, {}), orientation: { valType: "enumerated", values: ["v", "h"], editType: "calc+clearAxisTypes" }, base: { valType: "any", dflt: null, arrayOk: !0, editType: "calc" }, offset: { valType: "number", dflt: null, arrayOk: !0, editType: "calc" }, width: { valType: "number", dflt: null, min: 0, arrayOk: !0, editType: "calc" }, marker: me, offsetgroup: te.offsetgroup, alignmentgroup: te.alignmentgroup, selected: { marker: { opacity: te.selected.marker.opacity, color: te.selected.marker.color, editType: "style" }, textfont: te.selected.textfont, editType: "style" }, unselected: { marker: { opacity: te.unselected.marker.opacity, color: te.unselected.marker.color, editType: "style" }, textfont: te.unselected.textfont, editType: "style" }, zorder: te.zorder, _deprecated: { bardir: { valType: "enumerated", editType: "calc", values: ["v", "h"] } } };
          }, 67565: function(G, J, Z) {
            var te = Z(29714), ne = Z(40528), ee = Z(65477).hasColorscale, ae = Z(28379), oe = Z(35374), se = Z(48861);
            G.exports = function(ce, le) {
              var ue, de, fe, he, me, pe, ye = te.getFromId(ce, le.xaxis || "x"), ge = te.getFromId(ce, le.yaxis || "y"), ve = { msUTC: !(!le.base && le.base !== 0) };
              le.orientation === "h" ? (ue = ye.makeCalcdata(le, "x", ve), fe = ge.makeCalcdata(le, "y"), he = ne(le, ge, "y", fe), me = !!le.yperiodalignment, pe = "y") : (ue = ge.makeCalcdata(le, "y", ve), fe = ye.makeCalcdata(le, "x"), he = ne(le, ye, "x", fe), me = !!le.xperiodalignment, pe = "x"), de = he.vals;
              for (var be = Math.min(de.length, ue.length), Ce = new Array(be), Se = 0; Se < be; Se++) Ce[Se] = { p: de[Se], s: ue[Se] }, me && (Ce[Se].orig_p = fe[Se], Ce[Se][pe + "End"] = he.ends[Se], Ce[Se][pe + "Start"] = he.starts[Se]), le.ids && (Ce[Se].id = String(le.ids[Se]));
              return ee(le, "marker") && ae(ce, le, { vals: le.marker.color, containerStr: "marker", cLetter: "c" }), ee(le, "marker.line") && ae(ce, le, { vals: le.marker.line.color, containerStr: "marker.line", cLetter: "c" }), oe(Ce, le), se(Ce, le), Ce;
            };
          }, 56155: function(G) {
            G.exports = { TEXTPAD: 3, eventDataKeys: ["value", "label"] };
          }, 24782: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809).isArrayOrTypedArray, ee = Z(63821).BADNUM, ae = Z(33626), oe = Z(29714), se = Z(84391).getAxisGroup, ce = Z(2880);
            function le(be, Ce, Se, we, Ee) {
              if (we.length) {
                var xe, Le, Qe, ke;
                switch (function(Me, Re) {
                  var De, Pe;
                  for (De = 0; De < Re.length; De++) {
                    var $e, qe = Re[De], Xe = qe[0].trace, Ve = Xe.type === "funnel" ? Xe._base : Xe.base, Ze = Xe.orientation === "h" ? Xe.xcalendar : Xe.ycalendar, Ge = Me.type === "category" || Me.type === "multicategory" ? function() {
                      return null;
                    } : Me.d2c;
                    if (ne(Ve)) {
                      for (Pe = 0; Pe < Math.min(Ve.length, qe.length); Pe++) $e = Ge(Ve[Pe], 0, Ze), te($e) ? (qe[Pe].b = +$e, qe[Pe].hasB = 1) : qe[Pe].b = 0;
                      for (; Pe < qe.length; Pe++) qe[Pe].b = 0;
                    } else {
                      $e = Ge(Ve, 0, Ze);
                      var ze = te($e);
                      for ($e = ze ? $e : 0, Pe = 0; Pe < qe.length; Pe++) qe[Pe].b = $e, ze && (qe[Pe].hasB = 1);
                    }
                  }
                }(Se, we), Ee.mode) {
                  case "overlay":
                    ue(Ce, Se, we, Ee);
                    break;
                  case "group":
                    for (xe = [], Le = [], Qe = 0; Qe < we.length; Qe++) (ke = we[Qe])[0].trace.offset === void 0 ? Le.push(ke) : xe.push(ke);
                    Le.length && function(Me, Re, De, Pe, $e) {
                      var qe = new ce(Pe, { posAxis: Re, sepNegVal: !1, overlapNoMerge: !$e.norm });
                      (function(Xe, Ve, Ze, Ge) {
                        for (var ze = Xe._fullLayout, We = Ze.positions, at = Ze.distinctPositions, ct = Ze.minDiff, et = Ze.traces, dt = et.length, Je = We.length !== at.length, it = ct * (1 - Ge.gap), st = se(ze, Ve._id) + et[0][0].trace.orientation, gt = ze._alignmentOpts[st] || {}, bt = 0; bt < dt; bt++) {
                          var yt, St, Lt = et[bt], Rt = Lt[0].trace, vt = gt[Rt.alignmentgroup] || {}, $t = Object.keys(vt.offsetGroups || {}).length, Pt = (yt = $t ? it / $t : Je ? it / dt : it) * (1 - (Ge.groupgap || 0));
                          St = $t ? ((2 * Rt._offsetIndex + 1 - $t) * yt - Pt) / 2 : Je ? ((2 * bt + 1 - dt) * yt - Pt) / 2 : -Pt / 2;
                          var Xt = Lt[0].t;
                          Xt.barwidth = Pt, Xt.poffset = St, Xt.bargroupwidth = it, Xt.bardelta = ct;
                        }
                        Ze.binWidth = et[0][0].t.barwidth / 100, fe(Ze), he(Ve, Ze), me(Ve, Ze, Je);
                      })(Me, Re, qe, $e), function(Xe, Ve) {
                        for (var Ze = Xe.traces, Ge = 0; Ge < Ze.length; Ge++) {
                          var ze = Ze[Ge];
                          if (ze[0].trace.base === void 0) for (var We = new ce([ze], { posAxis: Ve, sepNegVal: !0, overlapNoMerge: !0 }), at = 0; at < ze.length; at++) {
                            var ct = ze[at];
                            if (ct.p !== ee) {
                              var et = We.put(ct.p, ct.b + ct.s);
                              et && (ct.b = et);
                            }
                          }
                        }
                      }(qe, Re), $e.norm ? (ye(qe), ge(De, qe, $e)) : pe(De, qe);
                    }(be, Ce, Se, Le, Ee), xe.length && ue(Ce, Se, xe, Ee);
                    break;
                  case "stack":
                  case "relative":
                    for (xe = [], Le = [], Qe = 0; Qe < we.length; Qe++) (ke = we[Qe])[0].trace.base === void 0 ? Le.push(ke) : xe.push(ke);
                    (function(Me) {
                      if (!(Me.length < 2)) {
                        var Re, De, Pe, $e, qe, Xe;
                        for (Re = 0; Re < Me.length && ($e = (De = Me[Re][0].trace).marker ? De.marker.cornerradius : void 0) === void 0; Re++) ;
                        if ($e !== void 0) for (qe = te($e) ? +$e : +$e.slice(0, -1), Xe = te($e) ? "px" : "%", Re = 0; Re < Me.length; Re++) (Pe = Me[Re][0].t).cornerradiusvalue = qe, Pe.cornerradiusform = Xe;
                      }
                    })(Le), Le.length && function(Me, Re, De, Pe, $e) {
                      var qe = new ce(Pe, { posAxis: Re, sepNegVal: $e.mode === "relative", overlapNoMerge: !($e.norm || $e.mode === "stack" || $e.mode === "relative") });
                      de(Re, qe, $e), function(ze, We, at) {
                        var ct, et, dt, Je, it, st, gt = ve(ze), bt = We.traces;
                        for (Je = 0; Je < bt.length; Je++) if ((et = (ct = bt[Je])[0].trace).type === "funnel") for (it = 0; it < ct.length; it++) (st = ct[it]).s !== ee && We.put(st.p, -0.5 * st.s);
                        for (Je = 0; Je < bt.length; Je++) {
                          dt = (et = (ct = bt[Je])[0].trace).type === "funnel";
                          var yt = [];
                          for (it = 0; it < ct.length; it++) if ((st = ct[it]).s !== ee) {
                            var St;
                            St = dt ? st.s : st.s + st.b;
                            var Lt = We.put(st.p, St), Rt = Lt + St;
                            st.b = Lt, st[gt] = Rt, at.norm || (yt.push(Rt), st.hasB && yt.push(Lt));
                          }
                          at.norm || (et._extremes[ze._id] = oe.findExtremes(ze, yt, { tozero: !0, padded: !0 }));
                        }
                      }(De, qe, $e);
                      for (var Xe = 0; Xe < Pe.length; Xe++) for (var Ve = Pe[Xe], Ze = 0; Ze < Ve.length; Ze++) {
                        var Ge = Ve[Ze];
                        Ge.s !== ee && Ge.b + Ge.s === qe.get(Ge.p, Ge.s) && (Ge._outmost = !0);
                      }
                      $e.norm && ge(De, qe, $e);
                    }(0, Ce, Se, Le, Ee), xe.length && ue(Ce, Se, xe, Ee);
                }
                (function(Me) {
                  var Re, De, Pe, $e, qe, Xe, Ve;
                  for (Re = 0; Re < Me.length; Re++) Pe = (De = Me[Re])[0].trace, ($e = De[0].t).cornerradiusvalue === void 0 && (qe = Pe.marker ? Pe.marker.cornerradius : void 0) !== void 0 && (Xe = te(qe) ? +qe : +qe.slice(0, -1), Ve = te(qe) ? "px" : "%", $e.cornerradiusvalue = Xe, $e.cornerradiusform = Ve);
                })(we), function(Me, Re) {
                  var De, Pe, $e, qe = ve(Re), Xe = {}, Ve = 1 / 0, Ze = -1 / 0;
                  for (De = 0; De < Me.length; De++) for ($e = Me[De], Pe = 0; Pe < $e.length; Pe++) {
                    var Ge = $e[Pe].p;
                    te(Ge) && (Ve = Math.min(Ve, Ge), Ze = Math.max(Ze, Ge));
                  }
                  var ze = 1e4 / (Ze - Ve), We = Xe.round = function(Rt) {
                    return String(Math.round(ze * (Rt - Ve)));
                  }, at = {}, ct = {}, et = Me.some(function(Rt) {
                    var vt = Rt[0].trace;
                    return "marker" in vt && vt.marker.cornerradius;
                  });
                  for (De = 0; De < Me.length; De++) {
                    ($e = Me[De])[0].t.extents = Xe;
                    var dt = $e[0].t.poffset, Je = ne(dt);
                    for (Pe = 0; Pe < $e.length; Pe++) {
                      var it = $e[Pe], st = it[qe] - it.w / 2;
                      if (te(st)) {
                        var gt = it[qe] + it.w / 2, bt = We(it.p);
                        Xe[bt] ? Xe[bt] = [Math.min(st, Xe[bt][0]), Math.max(gt, Xe[bt][1])] : Xe[bt] = [st, gt];
                      }
                      if (it.p0 = it.p + (Je ? dt[Pe] : dt), it.p1 = it.p0 + it.w, it.s0 = it.b, it.s1 = it.s0 + it.s, et) {
                        var yt = Math.min(it.s0, it.s1) || 0, St = Math.max(it.s0, it.s1) || 0, Lt = it[qe];
                        at[Lt] = Lt in at ? Math.min(at[Lt], yt) : yt, ct[Lt] = Lt in ct ? Math.max(ct[Lt], St) : St;
                      }
                    }
                  }
                  et && function(Rt, vt, $t, Pt) {
                    for (var Xt = ve(Pt), Nt = 0; Nt < Rt.length; Nt++) for (var Gt = Rt[Nt], Ft = 0; Ft < Gt.length; Ft++) {
                      var Tt = Gt[Ft], It = Tt[Xt];
                      Tt._sMin = vt[It], Tt._sMax = $t[It];
                    }
                  }(Me, at, ct, Re);
                }(we, Ce);
              }
            }
            function ue(be, Ce, Se, we) {
              for (var Ee = 0; Ee < Se.length; Ee++) {
                var xe = Se[Ee], Le = new ce([xe], { posAxis: be, sepNegVal: !1, overlapNoMerge: !we.norm });
                de(be, Le, we), we.norm ? (ye(Le), ge(Ce, Le, we)) : pe(Ce, Le);
              }
            }
            function de(be, Ce, Se) {
              for (var we = Ce.minDiff, Ee = Ce.traces, xe = we * (1 - Se.gap), Le = xe * (1 - (Se.groupgap || 0)), Qe = -Le / 2, ke = 0; ke < Ee.length; ke++) {
                var Me = Ee[ke][0].t;
                Me.barwidth = Le, Me.poffset = Qe, Me.bargroupwidth = xe, Me.bardelta = we;
              }
              Ce.binWidth = Ee[0][0].t.barwidth / 100, fe(Ce), he(be, Ce), me(be, Ce);
            }
            function fe(be) {
              var Ce, Se, we = be.traces;
              for (Ce = 0; Ce < we.length; Ce++) {
                var Ee, xe = we[Ce], Le = xe[0], Qe = Le.trace, ke = Le.t, Me = Qe._offset || Qe.offset, Re = ke.poffset;
                if (ne(Me)) {
                  for (Ee = Array.prototype.slice.call(Me, 0, xe.length), Se = 0; Se < Ee.length; Se++) te(Ee[Se]) || (Ee[Se] = Re);
                  for (Se = Ee.length; Se < xe.length; Se++) Ee.push(Re);
                  ke.poffset = Ee;
                } else Me !== void 0 && (ke.poffset = Me);
                var De = Qe._width || Qe.width, Pe = ke.barwidth;
                if (ne(De)) {
                  var $e = Array.prototype.slice.call(De, 0, xe.length);
                  for (Se = 0; Se < $e.length; Se++) te($e[Se]) || ($e[Se] = Pe);
                  for (Se = $e.length; Se < xe.length; Se++) $e.push(Pe);
                  if (ke.barwidth = $e, Me === void 0) {
                    for (Ee = [], Se = 0; Se < xe.length; Se++) Ee.push(Re + (Pe - $e[Se]) / 2);
                    ke.poffset = Ee;
                  }
                } else De !== void 0 && (ke.barwidth = De, Me === void 0 && (ke.poffset = Re + (Pe - De) / 2));
              }
            }
            function he(be, Ce) {
              for (var Se = Ce.traces, we = ve(be), Ee = 0; Ee < Se.length; Ee++) for (var xe = Se[Ee], Le = xe[0].t, Qe = Le.poffset, ke = ne(Qe), Me = Le.barwidth, Re = ne(Me), De = 0; De < xe.length; De++) {
                var Pe = xe[De], $e = Pe.w = Re ? Me[De] : Me;
                Pe.p === void 0 && (Pe.p = Pe[we], Pe["orig_" + we] = Pe[we]);
                var qe = (ke ? Qe[De] : Qe) + $e / 2;
                Pe[we] = Pe.p + qe;
              }
            }
            function me(be, Ce, Se) {
              var we = Ce.traces, Ee = Ce.minDiff / 2;
              oe.minDtick(be, Ce.minDiff, Ce.distinctPositions[0], Se);
              for (var xe = 0; xe < we.length; xe++) {
                var Le, Qe, ke, Me, Re = we[xe], De = Re[0], Pe = De.trace, $e = [];
                for (Me = 0; Me < Re.length; Me++) Qe = (Le = Re[Me]).p - Ee, ke = Le.p + Ee, $e.push(Qe, ke);
                if (Pe.width || Pe.offset) {
                  var qe = De.t, Xe = qe.poffset, Ve = qe.barwidth, Ze = ne(Xe), Ge = ne(Ve);
                  for (Me = 0; Me < Re.length; Me++) {
                    Le = Re[Me];
                    var ze = Ze ? Xe[Me] : Xe, We = Ge ? Ve[Me] : Ve;
                    ke = (Qe = Le.p + ze) + We, $e.push(Qe, ke);
                  }
                }
                Pe._extremes[be._id] = oe.findExtremes(be, $e, { padded: !1 });
              }
            }
            function pe(be, Ce) {
              for (var Se = Ce.traces, we = ve(be), Ee = 0; Ee < Se.length; Ee++) {
                for (var xe = Se[Ee], Le = xe[0].trace, Qe = Le.type === "scatter", ke = Le.orientation === "v", Me = [], Re = !1, De = 0; De < xe.length; De++) {
                  var Pe = xe[De], $e = Qe ? 0 : Pe.b, qe = Qe ? ke ? Pe.y : Pe.x : $e + Pe.s;
                  Pe[we] = qe, Me.push(qe), Pe.hasB && Me.push($e), Pe.hasB && Pe.b || (Re = !0);
                }
                Le._extremes[be._id] = oe.findExtremes(be, Me, { tozero: Re, padded: !0 });
              }
            }
            function ye(be) {
              for (var Ce = be.traces, Se = 0; Se < Ce.length; Se++) for (var we = Ce[Se], Ee = 0; Ee < we.length; Ee++) {
                var xe = we[Ee];
                xe.s !== ee && be.put(xe.p, xe.b + xe.s);
              }
            }
            function ge(be, Ce, Se) {
              var we = Ce.traces, Ee = ve(be), xe = Se.norm === "fraction" ? 1 : 100, Le = xe / 1e9, Qe = be.l2c(be.c2l(0)), ke = Se.mode === "stack" ? xe : Qe;
              function Me(at) {
                return te(be.c2l(at)) && (at < Qe - Le || at > ke + Le || !te(Qe));
              }
              for (var Re = 0; Re < we.length; Re++) {
                for (var De = we[Re], Pe = De[0].trace, $e = [], qe = !1, Xe = !1, Ve = 0; Ve < De.length; Ve++) {
                  var Ze = De[Ve];
                  if (Ze.s !== ee) {
                    var Ge = Math.abs(xe / Ce.get(Ze.p, Ze.s));
                    Ze.b *= Ge, Ze.s *= Ge;
                    var ze = Ze.b, We = ze + Ze.s;
                    Ze[Ee] = We, $e.push(We), Xe = Xe || Me(We), Ze.hasB && ($e.push(ze), Xe = Xe || Me(ze)), Ze.hasB && Ze.b || (qe = !0);
                  }
                }
                Pe._extremes[be._id] = oe.findExtremes(be, $e, { tozero: qe, padded: Xe });
              }
            }
            function ve(be) {
              return be._id.charAt(0);
            }
            G.exports = { crossTraceCalc: function(be, Ce) {
              for (var Se = Ce.xaxis, we = Ce.yaxis, Ee = be._fullLayout, xe = be._fullData, Le = be.calcdata, Qe = [], ke = [], Me = 0; Me < xe.length; Me++) {
                var Re = xe[Me];
                if (Re.visible === !0 && ae.traceIs(Re, "bar") && Re.xaxis === Se._id && Re.yaxis === we._id && (Re.orientation === "h" ? Qe.push(Le[Me]) : ke.push(Le[Me]), Re._computePh)) for (var De = be.calcdata[Me], Pe = 0; Pe < De.length; Pe++) typeof De[Pe].ph0 == "function" && (De[Pe].ph0 = De[Pe].ph0()), typeof De[Pe].ph1 == "function" && (De[Pe].ph1 = De[Pe].ph1());
              }
              var $e = { xCat: Se.type === "category" || Se.type === "multicategory", yCat: we.type === "category" || we.type === "multicategory", mode: Ee.barmode, norm: Ee.barnorm, gap: Ee.bargap, groupgap: Ee.bargroupgap };
              le(be, Se, we, ke, $e), le(be, we, Se, Qe, $e);
            }, setGroupPositions: le };
          }, 17550: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(78766), ae = Z(33626), oe = Z(99867), se = Z(99669), ce = Z(59760), le = Z(36301), ue = Z(81481), de = ne.coerceFont;
            function fe(me) {
              if (te(me)) {
                if ((me = +me) >= 0) return me;
              } else if (typeof me == "string" && (me = me.trim()).slice(-1) === "%" && te(me.slice(0, -1)) && (me = +me.slice(0, -1)) >= 0) return me + "%";
            }
            function he(me, pe, ye, ge, ve, be) {
              var Ce = (be = be || {}).moduleHasSelected !== !1, Se = be.moduleHasUnselected !== !1, we = be.moduleHasConstrain !== !1, Ee = be.moduleHasCliponaxis !== !1, xe = be.moduleHasTextangle !== !1, Le = be.moduleHasInsideanchor !== !1, Qe = !!be.hasPathbar, ke = Array.isArray(ve) || ve === "auto", Me = ke || ve === "inside", Re = ke || ve === "outside";
              if (Me || Re) {
                var De = de(ge, "textfont", ye.font), Pe = ne.extendFlat({}, De), $e = !(me.textfont && me.textfont.color);
                if ($e && delete Pe.color, de(ge, "insidetextfont", Pe), Qe) {
                  var qe = ne.extendFlat({}, De);
                  $e && delete qe.color, de(ge, "pathbar.textfont", qe);
                }
                Re && de(ge, "outsidetextfont", De), Ce && ge("selected.textfont.color"), Se && ge("unselected.textfont.color"), we && ge("constraintext"), Ee && ge("cliponaxis"), xe && ge("textangle"), ge("texttemplate");
              }
              Me && Le && ge("insidetextanchor");
            }
            G.exports = { supplyDefaults: function(me, pe, ye, ge) {
              function ve(we, Ee) {
                return ne.coerce(me, pe, ue, we, Ee);
              }
              if (oe(me, pe, ge, ve)) {
                se(me, pe, ge, ve), ve("xhoverformat"), ve("yhoverformat"), ve("zorder"), ve("orientation", pe.x && !pe.y ? "h" : "v"), ve("base"), ve("offset"), ve("width"), ve("text"), ve("hovertext"), ve("hovertemplate");
                var be = ve("textposition");
                he(me, 0, ge, ve, be, { moduleHasSelected: !0, moduleHasUnselected: !0, moduleHasConstrain: !0, moduleHasCliponaxis: !0, moduleHasTextangle: !0, moduleHasInsideanchor: !0 }), ce(me, pe, ve, ye, ge);
                var Ce = (pe.marker.line || {}).color, Se = ae.getComponentMethod("errorbars", "supplyDefaults");
                Se(me, pe, Ce || ee.defaultLine, { axis: "y" }), Se(me, pe, Ce || ee.defaultLine, { axis: "x", inherit: "y" }), ne.coerceSelectionMarkerOpacity(pe, ve);
              } else pe.visible = !1;
            }, crossTraceDefaults: function(me, pe) {
              var ye, ge;
              function ve(Se, we) {
                return ne.coerce(ge._input, ge, ue, Se, we);
              }
              for (var be = 0; be < me.length; be++) if ((ge = me[be]).type === "bar") {
                ye = ge._input;
                var Ce = ve("marker.cornerradius", pe.barcornerradius);
                ge.marker && (ge.marker.cornerradius = fe(Ce)), pe.barmode === "group" && le(ye, ge, pe, ve);
              }
            }, handleText: he, validateCornerradius: fe };
          }, 59541: function(G) {
            G.exports = function(J, Z, te) {
              return J.x = "xVal" in Z ? Z.xVal : Z.x, J.y = "yVal" in Z ? Z.yVal : Z.y, Z.xa && (J.xaxis = Z.xa), Z.ya && (J.yaxis = Z.ya), te.orientation === "h" ? (J.label = J.y, J.value = J.x) : (J.label = J.x, J.value = J.y), J;
            };
          }, 42843: function(G, J, Z) {
            var te = Z(10721), ne = Z(65657), ee = Z(34809).isArrayOrTypedArray;
            J.coerceString = function(ae, oe, se) {
              if (typeof oe == "string") {
                if (oe || !ae.noBlank) return oe;
              } else if ((typeof oe == "number" || oe === !0) && !ae.strict) return String(oe);
              return se !== void 0 ? se : ae.dflt;
            }, J.coerceNumber = function(ae, oe, se) {
              if (te(oe)) {
                oe = +oe;
                var ce = ae.min, le = ae.max;
                if (!(ce !== void 0 && oe < ce || le !== void 0 && oe > le)) return oe;
              }
              return se !== void 0 ? se : ae.dflt;
            }, J.coerceColor = function(ae, oe, se) {
              return ne(oe).isValid() ? oe : se !== void 0 ? se : ae.dflt;
            }, J.coerceEnumerated = function(ae, oe, se) {
              return ae.coerceNumber && (oe = +oe), ae.values.indexOf(oe) !== -1 ? oe : se !== void 0 ? se : ae.dflt;
            }, J.getValue = function(ae, oe) {
              var se;
              return ee(ae) ? oe < ae.length && (se = ae[oe]) : se = ae, se;
            }, J.getLineWidth = function(ae, oe) {
              return 0 < oe.mlw ? oe.mlw : ee(ae.marker.line.width) ? 0 : ae.marker.line.width;
            };
          }, 91664: function(G, J, Z) {
            var te = Z(32141), ne = Z(33626), ee = Z(78766), ae = Z(34809).fillText, oe = Z(42843).getLineWidth, se = Z(29714).hoverLabelText, ce = Z(63821).BADNUM;
            function le(de, fe, he, me, pe) {
              var ye, ge, ve, be, Ce, Se, we, Ee = de.cd, xe = Ee[0].trace, Le = Ee[0].t, Qe = me === "closest", ke = xe.type === "waterfall", Me = de.maxHoverDistance, Re = de.maxSpikeDistance;
              xe.orientation === "h" ? (ye = he, ge = fe, ve = "y", be = "x", Ce = ct, Se = We) : (ye = fe, ge = he, ve = "x", be = "y", Se = ct, Ce = We);
              var De = xe[ve + "period"], Pe = Qe || De;
              function $e(yt) {
                return Xe(yt, -1);
              }
              function qe(yt) {
                return Xe(yt, 1);
              }
              function Xe(yt, St) {
                var Lt = yt.w;
                return yt[ve] + St * Lt / 2;
              }
              function Ve(yt) {
                return yt[ve + "End"] - yt[ve + "Start"];
              }
              var Ze = Qe ? $e : De ? function(yt) {
                return yt.p - Ve(yt) / 2;
              } : function(yt) {
                return Math.min($e(yt), yt.p - Le.bardelta / 2);
              }, Ge = Qe ? qe : De ? function(yt) {
                return yt.p + Ve(yt) / 2;
              } : function(yt) {
                return Math.max(qe(yt), yt.p + Le.bardelta / 2);
              };
              function ze(yt, St, Lt) {
                return pe.finiteRange && (Lt = 0), te.inbox(yt - ye, St - ye, Lt + Math.min(1, Math.abs(St - yt) / we) - 1);
              }
              function We(yt) {
                return ze(Ze(yt), Ge(yt), Me);
              }
              function at(yt) {
                var St = yt[be];
                if (ke) {
                  var Lt = Math.abs(yt.rawS) || 0;
                  ge > 0 ? St += Lt : ge < 0 && (St -= Lt);
                }
                return St;
              }
              function ct(yt) {
                var St = ge, Lt = yt.b, Rt = at(yt);
                return te.inbox(Lt - St, Rt - St, Me + (Rt - St) / (Rt - Lt) - 1);
              }
              var et = de[ve + "a"], dt = de[be + "a"];
              we = Math.abs(et.r2c(et.range[1]) - et.r2c(et.range[0]));
              var Je = te.getDistanceFunction(me, Ce, Se, function(yt) {
                return (Ce(yt) + Se(yt)) / 2;
              });
              if (te.getClosest(Ee, Je, de), de.index !== !1 && Ee[de.index].p !== ce) {
                Pe || (Ze = function(yt) {
                  return Math.min($e(yt), yt.p - Le.bargroupwidth / 2);
                }, Ge = function(yt) {
                  return Math.max(qe(yt), yt.p + Le.bargroupwidth / 2);
                });
                var it = Ee[de.index], st = xe.base ? it.b + it.s : it.s;
                de[be + "0"] = de[be + "1"] = dt.c2p(it[be], !0), de[be + "LabelVal"] = st;
                var gt = Le.extents[Le.extents.round(it.p)];
                de[ve + "0"] = et.c2p(Qe ? Ze(it) : gt[0], !0), de[ve + "1"] = et.c2p(Qe ? Ge(it) : gt[1], !0);
                var bt = it.orig_p !== void 0;
                return de[ve + "LabelVal"] = bt ? it.orig_p : it.p, de.labelLabel = se(et, de[ve + "LabelVal"], xe[ve + "hoverformat"]), de.valueLabel = se(dt, de[be + "LabelVal"], xe[be + "hoverformat"]), de.baseLabel = se(dt, it.b, xe[be + "hoverformat"]), de.spikeDistance = (function(yt) {
                  var St = ge, Lt = yt.b, Rt = at(yt);
                  return te.inbox(Lt - St, Rt - St, Re + (Rt - St) / (Rt - Lt) - 1);
                }(it) + function(yt) {
                  return ze($e(yt), qe(yt), Re);
                }(it)) / 2, de[ve + "Spike"] = et.c2p(it.p, !0), ae(it, xe, de), de.hovertemplate = xe.hovertemplate, de;
              }
            }
            function ue(de, fe) {
              var he = fe.mcc || de.marker.color, me = fe.mlcc || de.marker.line.color, pe = oe(de, fe);
              return ee.opacity(he) ? he : ee.opacity(me) && pe ? me : void 0;
            }
            G.exports = { hoverPoints: function(de, fe, he, me, pe) {
              var ye = le(de, fe, he, me, pe);
              if (ye) {
                var ge = ye.cd, ve = ge[0].trace, be = ge[ye.index];
                return ye.color = ue(ve, be), ne.getComponentMethod("errorbars", "hoverInfo")(be, ve, ye), [ye];
              }
            }, hoverOnBars: le, getTraceColor: ue };
          }, 58218: function(G, J, Z) {
            G.exports = { attributes: Z(81481), layoutAttributes: Z(25412), supplyDefaults: Z(17550).supplyDefaults, crossTraceDefaults: Z(17550).crossTraceDefaults, supplyLayoutDefaults: Z(78931), calc: Z(67565), crossTraceCalc: Z(24782).crossTraceCalc, colorbar: Z(21146), arraysToCalcdata: Z(35374), plot: Z(32995).plot, style: Z(6851).style, styleOnSelect: Z(6851).styleOnSelect, hoverPoints: Z(91664).hoverPoints, eventData: Z(59541), selectPoints: Z(88384), moduleType: "trace", name: "bar", basePlotModule: Z(37703), categories: ["bar-like", "cartesian", "svg", "bar", "oriented", "errorBarsOK", "showLegend", "zoomScale"], animatable: !0, meta: {} };
          }, 25412: function(G) {
            G.exports = { barmode: { valType: "enumerated", values: ["stack", "group", "overlay", "relative"], dflt: "group", editType: "calc" }, barnorm: { valType: "enumerated", values: ["", "fraction", "percent"], dflt: "", editType: "calc" }, bargap: { valType: "number", min: 0, max: 1, editType: "calc" }, bargroupgap: { valType: "number", min: 0, max: 1, dflt: 0, editType: "calc" }, barcornerradius: { valType: "any", editType: "calc" } };
          }, 78931: function(G, J, Z) {
            var te = Z(33626), ne = Z(29714), ee = Z(34809), ae = Z(25412), oe = Z(17550).validateCornerradius;
            G.exports = function(se, ce, le) {
              function ue(Ce, Se) {
                return ee.coerce(se, ce, ae, Ce, Se);
              }
              for (var de = !1, fe = !1, he = !1, me = {}, pe = ue("barmode"), ye = 0; ye < le.length; ye++) {
                var ge = le[ye];
                if (te.traceIs(ge, "bar") && ge.visible) {
                  if (de = !0, pe === "group") {
                    var ve = ge.xaxis + ge.yaxis;
                    me[ve] && (he = !0), me[ve] = !0;
                  }
                  ge.visible && ge.type === "histogram" && ne.getFromId({ _fullLayout: ce }, ge[ge.orientation === "v" ? "xaxis" : "yaxis"]).type !== "category" && (fe = !0);
                }
              }
              if (de) {
                pe !== "overlay" && ue("barnorm"), ue("bargap", fe && !he ? 0 : 0.2), ue("bargroupgap");
                var be = ue("barcornerradius");
                ce.barcornerradius = oe(be);
              } else delete ce.barmode;
            };
          }, 32995: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(34809), ae = Z(30635), oe = Z(78766), se = Z(62203), ce = Z(33626), le = Z(29714).tickText, ue = Z(84102), de = ue.recordMinTextSize, fe = ue.clearMinTextSize, he = Z(6851), me = Z(42843), pe = Z(56155), ye = Z(81481), ge = ye.text, ve = ye.textposition, be = Z(36040).appendArrayPointValue, Ce = pe.TEXTPAD;
            function Se(Pe) {
              return Pe.id;
            }
            function we(Pe) {
              if (Pe.ids) return Se;
            }
            function Ee(Pe) {
              return (Pe > 0) - (Pe < 0);
            }
            function xe(Pe, $e) {
              return Pe < $e ? 1 : -1;
            }
            function Le(Pe, $e, qe, Xe) {
              var Ve;
              return !$e.uniformtext.mode && Qe(qe) ? (Xe && (Ve = Xe()), Pe.transition().duration(qe.duration).ease(qe.easing).each("end", function() {
                Ve && Ve();
              }).each("interrupt", function() {
                Ve && Ve();
              })) : Pe;
            }
            function Qe(Pe) {
              return Pe && Pe.duration > 0;
            }
            function ke(Pe, $e, qe, Xe, Ve) {
              return !(Pe < 0 || $e < 0) && (qe <= Pe && Xe <= $e || qe <= $e && Xe <= Pe || (Ve ? Pe >= qe * ($e / Xe) : $e >= Xe * (Pe / qe)));
            }
            function Me(Pe) {
              return Pe === "auto" ? 0 : Pe;
            }
            function Re(Pe, $e) {
              var qe = Math.PI / 180 * $e, Xe = Math.abs(Math.sin(qe)), Ve = Math.abs(Math.cos(qe));
              return { x: Pe.width * Ve + Pe.height * Xe, y: Pe.width * Xe + Pe.height * Ve };
            }
            function De(Pe, $e, qe, Xe, Ve, Ze) {
              var Ge = !!Ze.isHorizontal, ze = !!Ze.constrained, We = Ze.angle || 0, at = Ze.anchor, ct = at === "end", et = at === "start", dt = ((Ze.leftToRight || 0) + 1) / 2, Je = 1 - dt, it = Ze.hasB, st = Ze.r, gt = Ze.overhead, bt = Ve.width, yt = Ve.height, St = Math.abs($e - Pe), Lt = Math.abs(Xe - qe), Rt = St > 2 * Ce && Lt > 2 * Ce ? Ce : 0;
              St -= 2 * Rt, Lt -= 2 * Rt;
              var vt = Me(We);
              We !== "auto" || bt <= St && yt <= Lt || !(bt > St || yt > Lt) || (bt > Lt || yt > St) && bt < yt == St < Lt || (vt += 90);
              var $t, Pt, Xt = Re(Ve, vt);
              if (st && st - gt > Ce) {
                var Nt = function(sn, hn, Qt, Ut, Wt, rn, vn, dn, Ht) {
                  var jt, cn, wn, Rn, Pn = Math.max(0, Math.abs(hn - sn) - 2 * Ce), Ie = Math.max(0, Math.abs(Ut - Qt) - 2 * Ce), rt = rn - Ce, ft = vn ? rt - Math.sqrt(rt * rt - (rt - vn) * (rt - vn)) : rt, pt = Ht ? 2 * rt : dn ? rt - vn : 2 * ft, wt = Ht ? 2 * rt : dn ? 2 * ft : rt - vn;
                  return Wt.y / Wt.x >= Ie / (Pn - pt) ? Rn = Ie / Wt.y : Wt.y / Wt.x <= (Ie - wt) / Pn ? Rn = Pn / Wt.x : !Ht && dn ? (jt = Wt.x * Wt.x + Wt.y * Wt.y / 4, wn = (Pn - rt) * (Pn - rt) + (Ie / 2 - rt) * (Ie / 2 - rt) - rt * rt, Rn = (-(cn = -2 * Wt.x * (Pn - rt) - Wt.y * (Ie / 2 - rt)) + Math.sqrt(cn * cn - 4 * jt * wn)) / (2 * jt)) : Ht ? (jt = (Wt.x * Wt.x + Wt.y * Wt.y) / 4, wn = (Pn / 2 - rt) * (Pn / 2 - rt) + (Ie / 2 - rt) * (Ie / 2 - rt) - rt * rt, Rn = (-(cn = -Wt.x * (Pn / 2 - rt) - Wt.y * (Ie / 2 - rt)) + Math.sqrt(cn * cn - 4 * jt * wn)) / (2 * jt)) : (jt = Wt.x * Wt.x / 4 + Wt.y * Wt.y, wn = (Pn / 2 - rt) * (Pn / 2 - rt) + (Ie - rt) * (Ie - rt) - rt * rt, Rn = (-(cn = -Wt.x * (Pn / 2 - rt) - 2 * Wt.y * (Ie - rt)) + Math.sqrt(cn * cn - 4 * jt * wn)) / (2 * jt)), { scale: Rn = Math.min(1, Rn), pad: dn ? Math.max(0, rt - Math.sqrt(Math.max(0, rt * rt - (rt - (Ie - Wt.y * Rn) / 2) * (rt - (Ie - Wt.y * Rn) / 2))) - vn) : Math.max(0, rt - Math.sqrt(Math.max(0, rt * rt - (rt - (Pn - Wt.x * Rn) / 2) * (rt - (Pn - Wt.x * Rn) / 2))) - vn) };
                }(Pe, $e, qe, Xe, Xt, st, gt, Ge, it);
                $t = Nt.scale, Pt = Nt.pad;
              } else $t = 1, ze && ($t = Math.min(1, St / Xt.x, Lt / Xt.y)), Pt = 0;
              var Gt = Ve.left * Je + Ve.right * dt, Ft = (Ve.top + Ve.bottom) / 2, Tt = (Pe + Ce) * Je + ($e - Ce) * dt, It = (qe + Xe) / 2, Ct = 0, Kt = 0;
              if (et || ct) {
                var Vt = (Ge ? Xt.x : Xt.y) / 2;
                st && (ct || it) && (Rt += Pt);
                var Yt = Ge ? xe(Pe, $e) : xe(qe, Xe);
                Ge ? et ? (Tt = Pe + Yt * Rt, Ct = -Yt * Vt) : (Tt = $e - Yt * Rt, Ct = Yt * Vt) : et ? (It = qe + Yt * Rt, Kt = -Yt * Vt) : (It = Xe - Yt * Rt, Kt = Yt * Vt);
              }
              return { textX: Gt, textY: Ft, targetX: Tt, targetY: It, anchorX: Ct, anchorY: Kt, scale: $t, rotate: vt };
            }
            G.exports = { plot: function(Pe, $e, qe, Xe, Ve, Ze) {
              var Ge = $e.xaxis, ze = $e.yaxis, We = Pe._fullLayout, at = Pe._context.staticPlot;
              Ve || (Ve = { mode: We.barmode, norm: We.barmode, gap: We.bargap, groupgap: We.bargroupgap }, fe("bar", We));
              var ct = ee.makeTraceGroups(Xe, qe, "trace bars").each(function(et) {
                var dt = te.select(this), Je = et[0].trace, it = et[0].t, st = Je.type === "waterfall", gt = Je.type === "funnel", bt = Je.type === "histogram", yt = Je.type === "bar", St = yt || gt, Lt = 0;
                st && Je.connector.visible && Je.connector.mode === "between" && (Lt = Je.connector.line.width / 2);
                var Rt = Je.orientation === "h", vt = Qe(Ve), $t = ee.ensureSingle(dt, "g", "points"), Pt = we(Je), Xt = $t.selectAll("g.point").data(ee.identity, Pt);
                Xt.enter().append("g").classed("point", !0), Xt.exit().remove(), Xt.each(function(Gt, Ft) {
                  var Tt, It, Ct = te.select(this), Kt = function(_n, gn, bn, an) {
                    var Zt = [], Mn = [], Ln = an ? gn : bn, In = an ? bn : gn;
                    return Zt[0] = Ln.c2p(_n.s0, !0), Mn[0] = In.c2p(_n.p0, !0), Zt[1] = Ln.c2p(_n.s1, !0), Mn[1] = In.c2p(_n.p1, !0), an ? [Zt, Mn] : [Mn, Zt];
                  }(Gt, Ge, ze, Rt), Vt = Kt[0][0], Yt = Kt[0][1], sn = Kt[1][0], hn = Kt[1][1], Qt = (Rt ? Yt - Vt : hn - sn) == 0;
                  if (Qt && St && me.getLineWidth(Je, Gt) && (Qt = !1), Qt || (Qt = !(ne(Vt) && ne(Yt) && ne(sn) && ne(hn))), Gt.isBlank = Qt, Qt && (Rt ? Yt = Vt : hn = sn), Lt && !Qt && (Rt ? (Vt -= xe(Vt, Yt) * Lt, Yt += xe(Vt, Yt) * Lt) : (sn -= xe(sn, hn) * Lt, hn += xe(sn, hn) * Lt)), Je.type === "waterfall") {
                    if (!Qt) {
                      var Ut = Je[Gt.dir].marker;
                      Tt = Ut.line.width, It = Ut.color;
                    }
                  } else Tt = me.getLineWidth(Je, Gt), It = Gt.mc || Je.marker.color;
                  function Wt(_n) {
                    var gn = te.round(Tt / 2 % 1, 2);
                    return Ve.gap === 0 && Ve.groupgap === 0 ? te.round(Math.round(_n) - gn, 2) : _n;
                  }
                  var rn = oe.opacity(It) < 1 || Tt > 0.01 ? Wt : function(_n, gn, bn) {
                    return bn && _n === gn ? _n : Math.abs(_n - gn) >= 2 ? Wt(_n) : _n > gn ? Math.ceil(_n) : Math.floor(_n);
                  };
                  Pe._context.staticPlot || (Vt = rn(Vt, Yt, Rt), Yt = rn(Yt, Vt, Rt), sn = rn(sn, hn, !Rt), hn = rn(hn, sn, !Rt));
                  var vn, dn = Rt ? Ge.c2p : ze.c2p;
                  vn = Gt.s0 > 0 ? Gt._sMax : Gt.s0 < 0 ? Gt._sMin : Gt.s1 > 0 ? Gt._sMax : Gt._sMin;
                  var Ht, jt, cn = yt || bt ? function(_n, gn) {
                    if (!_n) return 0;
                    var bn, an = Math.abs(Rt ? hn - sn : Yt - Vt), Zt = Math.abs(Rt ? Yt - Vt : hn - sn), Mn = rn(Math.abs(dn(vn, !0) - dn(0, !0))), Ln = Gt.hasB ? Math.min(an / 2, Zt / 2) : Math.min(an / 2, Mn);
                    return bn = gn === "%" ? an * (Math.min(50, _n) / 100) : _n, rn(Math.max(Math.min(bn, Ln), 0));
                  }(it.cornerradiusvalue, it.cornerradiusform) : 0, wn = "M" + Vt + "," + sn + "V" + hn + "H" + Yt + "V" + sn + "Z", Rn = 0;
                  if (cn && Gt.s) {
                    var Pn = Ee(Gt.s0) === 0 || Ee(Gt.s) === Ee(Gt.s0) ? Gt.s1 : Gt.s0;
                    if ((Rn = rn(Gt.hasB ? 0 : Math.abs(dn(vn, !0) - dn(Pn, !0)))) < cn) {
                      var Ie = xe(Vt, Yt), rt = xe(sn, hn), ft = Ie === -rt ? 1 : 0;
                      if (Rt) if (Gt.hasB) Ht = "M" + (Vt + cn * Ie) + "," + sn + "A " + cn + "," + cn + " 0 0 " + ft + " " + Vt + "," + (sn + cn * rt) + "V" + (hn - cn * rt) + "A " + cn + "," + cn + " 0 0 " + ft + " " + (Vt + cn * Ie) + "," + hn + "H" + (Yt - cn * Ie) + "A " + cn + "," + cn + " 0 0 " + ft + " " + Yt + "," + (hn - cn * rt) + "V" + (sn + cn * rt) + "A " + cn + "," + cn + " 0 0 " + ft + " " + (Yt - cn * Ie) + "," + sn + "Z";
                      else {
                        var pt = (jt = Math.abs(Yt - Vt) + Rn) < cn ? cn - Math.sqrt(jt * (2 * cn - jt)) : 0, wt = Rn > 0 ? Math.sqrt(Rn * (2 * cn - Rn)) : 0, Bt = Ie > 0 ? Math.max : Math.min;
                        Ht = "M" + Vt + "," + sn + "V" + (hn - pt * rt) + "H" + Bt(Yt - (cn - Rn) * Ie, Vt) + "A " + cn + "," + cn + " 0 0 " + ft + " " + Yt + "," + (hn - cn * rt - wt) + "V" + (sn + cn * rt + wt) + "A " + cn + "," + cn + " 0 0 " + ft + " " + Bt(Yt - (cn - Rn) * Ie, Vt) + "," + (sn + pt * rt) + "Z";
                      }
                      else if (Gt.hasB) Ht = "M" + (Vt + cn * Ie) + "," + sn + "A " + cn + "," + cn + " 0 0 " + ft + " " + Vt + "," + (sn + cn * rt) + "V" + (hn - cn * rt) + "A " + cn + "," + cn + " 0 0 " + ft + " " + (Vt + cn * Ie) + "," + hn + "H" + (Yt - cn * Ie) + "A " + cn + "," + cn + " 0 0 " + ft + " " + Yt + "," + (hn - cn * rt) + "V" + (sn + cn * rt) + "A " + cn + "," + cn + " 0 0 " + ft + " " + (Yt - cn * Ie) + "," + sn + "Z";
                      else {
                        var At = (jt = Math.abs(hn - sn) + Rn) < cn ? cn - Math.sqrt(jt * (2 * cn - jt)) : 0, Dt = Rn > 0 ? Math.sqrt(Rn * (2 * cn - Rn)) : 0, Ot = rt > 0 ? Math.max : Math.min;
                        Ht = "M" + (Vt + At * Ie) + "," + sn + "V" + Ot(hn - (cn - Rn) * rt, sn) + "A " + cn + "," + cn + " 0 0 " + ft + " " + (Vt + cn * Ie - Dt) + "," + hn + "H" + (Yt - cn * Ie + Dt) + "A " + cn + "," + cn + " 0 0 " + ft + " " + (Yt - At * Ie) + "," + Ot(hn - (cn - Rn) * rt, sn) + "V" + sn + "Z";
                      }
                    } else Ht = wn;
                  } else Ht = wn;
                  var un = Le(ee.ensureSingle(Ct, "path"), We, Ve, Ze);
                  if (un.style("vector-effect", at ? "none" : "non-scaling-stroke").attr("d", isNaN((Yt - Vt) * (hn - sn)) || Qt && Pe._context.staticPlot ? "M0,0Z" : Ht).call(se.setClipUrl, $e.layerClipId, Pe), !We.uniformtext.mode && vt) {
                    var yn = se.makePointStyleFns(Je);
                    se.singlePointStyle(Gt, un, Je, yn, Pe);
                  }
                  (function(_n, gn, bn, an, Zt, Mn, Ln, In, kn, er, jn, ur, Kn) {
                    var Jn, Lr = gn.xaxis, mr = gn.yaxis, Ar = _n._fullLayout;
                    function Yr(Wa, Qr, Ur) {
                      return ee.ensureSingle(Wa, "text").text(Qr).attr({ class: "bartext bartext-" + Jn, "text-anchor": "middle", "data-notex": 1 }).call(se.font, Ur).call(ae.convertToTspans, _n);
                    }
                    var jr = an[0].trace, Mr = jr.orientation === "h", Wr = function(Wa, Qr, Ur, rr, vr) {
                      var Rr, Vr = Qr[0].trace;
                      return Rr = Vr.texttemplate ? function(Xr, Jr, ti, Hr, Er) {
                        var gr = Jr[0].trace, $r = ee.castOption(gr, ti, "texttemplate");
                        if (!$r) return "";
                        var li, Qi, Ai, Li, Ni = gr.type === "histogram", Pi = gr.type === "waterfall", Oi = gr.type === "funnel", ki = gr.orientation === "h";
                        function Mi(Tn) {
                          return le(Li, Li.c2l(Tn), !0).text;
                        }
                        ki ? (li = "y", Qi = Er, Ai = "x", Li = Hr) : (li = "x", Qi = Hr, Ai = "y", Li = Er);
                        var Vi, ua = Jr[ti], Ki = {};
                        Ki.label = ua.p, Ki.labelLabel = Ki[li + "Label"] = (Vi = ua.p, le(Qi, Qi.c2l(Vi), !0).text);
                        var Fi = ee.castOption(gr, ua.i, "text");
                        (Fi === 0 || Fi) && (Ki.text = Fi), Ki.value = ua.s, Ki.valueLabel = Ki[Ai + "Label"] = Mi(ua.s);
                        var na = {};
                        be(na, gr, ua.i), (Ni || na.x === void 0) && (na.x = ki ? Ki.value : Ki.label), (Ni || na.y === void 0) && (na.y = ki ? Ki.label : Ki.value), (Ni || na.xLabel === void 0) && (na.xLabel = ki ? Ki.valueLabel : Ki.labelLabel), (Ni || na.yLabel === void 0) && (na.yLabel = ki ? Ki.labelLabel : Ki.valueLabel), Pi && (Ki.delta = +ua.rawS || ua.s, Ki.deltaLabel = Mi(Ki.delta), Ki.final = ua.v, Ki.finalLabel = Mi(Ki.final), Ki.initial = Ki.final - Ki.delta, Ki.initialLabel = Mi(Ki.initial)), Oi && (Ki.value = ua.s, Ki.valueLabel = Mi(Ki.value), Ki.percentInitial = ua.begR, Ki.percentInitialLabel = ee.formatPercent(ua.begR), Ki.percentPrevious = ua.difR, Ki.percentPreviousLabel = ee.formatPercent(ua.difR), Ki.percentTotal = ua.sumR, Ki.percenTotalLabel = ee.formatPercent(ua.sumR));
                        var nn = ee.castOption(gr, ua.i, "customdata");
                        return nn && (Ki.customdata = nn), ee.texttemplateString($r, Ki, Xr._d3locale, na, Ki, gr._meta || {});
                      }(Wa, Qr, Ur, rr, vr) : Vr.textinfo ? function(Xr, Jr, ti, Hr) {
                        var Er = Xr[0].trace, gr = Er.orientation === "h", $r = Er.type === "waterfall", li = Er.type === "funnel";
                        function Qi(nn) {
                          return le(gr ? ti : Hr, +nn, !0).text;
                        }
                        var Ai, Li, Ni = Er.textinfo, Pi = Xr[Jr], Oi = Ni.split("+"), ki = [], Mi = function(nn) {
                          return Oi.indexOf(nn) !== -1;
                        };
                        if (Mi("label") && ki.push((Li = Xr[Jr].p, le(gr ? Hr : ti, Li, !0).text)), Mi("text") && ((Ai = ee.castOption(Er, Pi.i, "text")) === 0 || Ai) && ki.push(Ai), $r) {
                          var Vi = +Pi.rawS || Pi.s, ua = Pi.v, Ki = ua - Vi;
                          Mi("initial") && ki.push(Qi(Ki)), Mi("delta") && ki.push(Qi(Vi)), Mi("final") && ki.push(Qi(ua));
                        }
                        if (li) {
                          Mi("value") && ki.push(Qi(Pi.s));
                          var Fi = 0;
                          Mi("percent initial") && Fi++, Mi("percent previous") && Fi++, Mi("percent total") && Fi++;
                          var na = Fi > 1;
                          Mi("percent initial") && (Ai = ee.formatPercent(Pi.begR), na && (Ai += " of initial"), ki.push(Ai)), Mi("percent previous") && (Ai = ee.formatPercent(Pi.difR), na && (Ai += " of previous"), ki.push(Ai)), Mi("percent total") && (Ai = ee.formatPercent(Pi.sumR), na && (Ai += " of total"), ki.push(Ai));
                        }
                        return ki.join("<br>");
                      }(Qr, Ur, rr, vr) : me.getValue(Vr.text, Ur), me.coerceString(ge, Rr);
                    }(Ar, an, Zt, Lr, mr);
                    Jn = function(Wa, Qr) {
                      var Ur = me.getValue(Wa.textposition, Qr);
                      return me.coerceEnumerated(ve, Ur);
                    }(jr, Zt);
                    var Bi = ur.mode === "stack" || ur.mode === "relative", gi = an[Zt], Zr = !Bi || gi._outmost, fi = gi.hasB, ri = er && er - jn > Ce;
                    if (Wr && Jn !== "none" && (!gi.isBlank && Mn !== Ln && In !== kn || Jn !== "auto" && Jn !== "inside")) {
                      var bi = Ar.font, Zi = he.getBarColor(an[Zt], jr), $i = he.getInsideTextFont(jr, Zt, bi, Zi), Br = he.getOutsideTextFont(jr, Zt, bi), Pr = jr.insidetextanchor || "end", ei = bn.datum();
                      Mr ? Lr.type === "log" && ei.s0 <= 0 && (Mn = Lr.range[0] < Lr.range[1] ? 0 : Lr._length) : mr.type === "log" && ei.s0 <= 0 && (In = mr.range[0] < mr.range[1] ? mr._length : 0);
                      var Ci, oa, ka, aa, hi, Gr = Math.abs(Ln - Mn), Ca = Math.abs(kn - In), Sr = Gr - 2 * Ce, Dr = Ca - 2 * Ce;
                      if (Jn === "outside" && (Zr || gi.hasB || (Jn = "inside")), Jn === "auto") if (Zr) {
                        Jn = "inside", Ci = Yr(bn, Wr, hi = ee.ensureUniformFontSize(_n, $i)), ka = (oa = se.bBox(Ci.node())).width, aa = oa.height;
                        var si, zr = ka > 0 && aa > 0;
                        si = ri ? fi ? ke(Sr - 2 * er, Dr, ka, aa, Mr) || ke(Sr, Dr - 2 * er, ka, aa, Mr) : Mr ? ke(Sr - (er - jn), Dr, ka, aa, Mr) || ke(Sr, Dr - 2 * (er - jn), ka, aa, Mr) : ke(Sr, Dr - (er - jn), ka, aa, Mr) || ke(Sr - 2 * (er - jn), Dr, ka, aa, Mr) : ke(Sr, Dr, ka, aa, Mr), zr && si ? Jn = "inside" : (Jn = "outside", Ci.remove(), Ci = null);
                      } else Jn = "inside";
                      if (!Ci) {
                        var di = (Ci = Yr(bn, Wr, hi = ee.ensureUniformFontSize(_n, Jn === "outside" ? Br : $i))).attr("transform");
                        if (Ci.attr("transform", ""), ka = (oa = se.bBox(Ci.node())).width, aa = oa.height, Ci.attr("transform", di), ka <= 0 || aa <= 0) return void Ci.remove();
                      }
                      var Da, la = jr.textangle;
                      Da = Jn === "outside" ? function(Wa, Qr, Ur, rr, vr, Rr) {
                        var Vr, Xr = !!Rr.isHorizontal, Jr = !!Rr.constrained, ti = Rr.angle || 0, Hr = vr.width, Er = vr.height, gr = Math.abs(Qr - Wa), $r = Math.abs(rr - Ur);
                        Vr = Xr ? $r > 2 * Ce ? Ce : 0 : gr > 2 * Ce ? Ce : 0;
                        var li = 1;
                        Jr && (li = Xr ? Math.min(1, $r / Er) : Math.min(1, gr / Hr));
                        var Qi = Me(ti), Ai = Re(vr, Qi), Li = (Xr ? Ai.x : Ai.y) / 2, Ni = (vr.left + vr.right) / 2, Pi = (vr.top + vr.bottom) / 2, Oi = (Wa + Qr) / 2, ki = (Ur + rr) / 2, Mi = 0, Vi = 0, ua = Xr ? xe(Qr, Wa) : xe(Ur, rr);
                        return Xr ? (Oi = Qr - ua * Vr, Mi = ua * Li) : (ki = rr + ua * Vr, Vi = -ua * Li), { textX: Ni, textY: Pi, targetX: Oi, targetY: ki, anchorX: Mi, anchorY: Vi, scale: li, rotate: Qi };
                      }(Mn, Ln, In, kn, oa, { isHorizontal: Mr, constrained: jr.constraintext === "both" || jr.constraintext === "outside", angle: la }) : De(Mn, Ln, In, kn, oa, { isHorizontal: Mr, constrained: jr.constraintext === "both" || jr.constraintext === "inside", angle: la, anchor: Pr, hasB: fi, r: er, overhead: jn }), Da.fontSize = hi.size, de(jr.type === "histogram" ? "bar" : jr.type, Da, Ar), gi.transform = Da;
                      var wa = Le(Ci, Ar, ur, Kn);
                      ee.setTransormAndDisplay(wa, Da);
                    } else bn.select("text").remove();
                  })(Pe, $e, Ct, et, Ft, Vt, Yt, sn, hn, cn, Rn, Ve, Ze), $e.layerClipId && se.hideOutsideRangePoint(Gt, Ct.select("text"), Ge, ze, Je.xcalendar, Je.ycalendar);
                });
                var Nt = Je.cliponaxis === !1;
                se.setClipUrl(dt, Nt ? null : $e.layerClipId, Pe);
              });
              ce.getComponentMethod("errorbars", "plot")(Pe, ct, $e, Ve);
            }, toMoveInsideBar: De };
          }, 88384: function(G) {
            function J(Z, te, ne, ee, ae) {
              var oe = te.c2p(ee ? Z.s0 : Z.p0, !0), se = te.c2p(ee ? Z.s1 : Z.p1, !0), ce = ne.c2p(ee ? Z.p0 : Z.s0, !0), le = ne.c2p(ee ? Z.p1 : Z.s1, !0);
              return ae ? [(oe + se) / 2, (ce + le) / 2] : ee ? [se, (ce + le) / 2] : [(oe + se) / 2, le];
            }
            G.exports = function(Z, te) {
              var ne, ee = Z.cd, ae = Z.xaxis, oe = Z.yaxis, se = ee[0].trace, ce = se.type === "funnel", le = se.orientation === "h", ue = [];
              if (te === !1) for (ne = 0; ne < ee.length; ne++) ee[ne].selected = 0;
              else for (ne = 0; ne < ee.length; ne++) {
                var de = ee[ne], fe = "ct" in de ? de.ct : J(de, ae, oe, le, ce);
                te.contains(fe, !1, ne, Z) ? (ue.push({ pointNumber: ne, x: ae.c2d(de.x), y: oe.c2d(de.y) }), de.selected = 1) : de.selected = 0;
              }
              return ue;
            };
          }, 2880: function(G, J, Z) {
            G.exports = ne;
            var te = Z(34809).distinctVals;
            function ne(ee, ae) {
              this.traces = ee, this.sepNegVal = ae.sepNegVal, this.overlapNoMerge = ae.overlapNoMerge;
              for (var oe = 1 / 0, se = ae.posAxis._id.charAt(0), ce = [], le = 0; le < ee.length; le++) {
                for (var ue = ee[le], de = 0; de < ue.length; de++) {
                  var fe = ue[de], he = fe.p;
                  he === void 0 && (he = fe[se]), he !== void 0 && ce.push(he);
                }
                ue[0] && ue[0].width1 && (oe = Math.min(ue[0].width1, oe));
              }
              this.positions = ce;
              var me = te(ce);
              this.distinctPositions = me.vals, me.vals.length === 1 && oe !== 1 / 0 ? this.minDiff = oe : this.minDiff = Math.min(me.minDiff, oe);
              var pe = (ae.posAxis || {}).type;
              pe !== "category" && pe !== "multicategory" || (this.minDiff = 1), this.binWidth = this.minDiff, this.bins = {};
            }
            ne.prototype.put = function(ee, ae) {
              var oe = this.getLabel(ee, ae), se = this.bins[oe] || 0;
              return this.bins[oe] = se + ae, se;
            }, ne.prototype.get = function(ee, ae) {
              var oe = this.getLabel(ee, ae);
              return this.bins[oe] || 0;
            }, ne.prototype.getLabel = function(ee, ae) {
              return (ae < 0 && this.sepNegVal ? "v" : "^") + (this.overlapNoMerge ? ee : Math.round(ee / this.binWidth));
            };
          }, 6851: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(62203), ae = Z(34809), oe = Z(33626), se = Z(84102).resizeText, ce = Z(81481), le = ce.textfont, ue = ce.insidetextfont, de = ce.outsidetextfont, fe = Z(42843);
            function he(Se, we, Ee) {
              ee.pointStyle(Se.selectAll("path"), we, Ee), me(Se, we, Ee);
            }
            function me(Se, we, Ee) {
              Se.selectAll("text").each(function(xe) {
                var Le = te.select(this), Qe = ae.ensureUniformFontSize(Ee, pe(Le, xe, we, Ee));
                ee.font(Le, Qe);
              });
            }
            function pe(Se, we, Ee, xe) {
              var Le = xe._fullLayout.font, Qe = Ee.textfont;
              if (Se.classed("bartext-inside")) {
                var ke = Ce(we, Ee);
                Qe = ge(Ee, we.i, Le, ke);
              } else Se.classed("bartext-outside") && (Qe = ve(Ee, we.i, Le));
              return Qe;
            }
            function ye(Se, we, Ee) {
              return be(le, Se.textfont, we, Ee);
            }
            function ge(Se, we, Ee, xe) {
              var Le = ye(Se, we, Ee);
              return (Se._input.textfont === void 0 || Se._input.textfont.color === void 0 || Array.isArray(Se.textfont.color) && Se.textfont.color[we] === void 0) && (Le = { color: ne.contrast(xe), family: Le.family, size: Le.size, weight: Le.weight, style: Le.style, variant: Le.variant, textcase: Le.textcase, lineposition: Le.lineposition, shadow: Le.shadow }), be(ue, Se.insidetextfont, we, Le);
            }
            function ve(Se, we, Ee) {
              var xe = ye(Se, we, Ee);
              return be(de, Se.outsidetextfont, we, xe);
            }
            function be(Se, we, Ee, xe) {
              we = we || {};
              var Le = fe.getValue(we.family, Ee), Qe = fe.getValue(we.size, Ee), ke = fe.getValue(we.color, Ee), Me = fe.getValue(we.weight, Ee), Re = fe.getValue(we.style, Ee), De = fe.getValue(we.variant, Ee), Pe = fe.getValue(we.textcase, Ee), $e = fe.getValue(we.lineposition, Ee), qe = fe.getValue(we.shadow, Ee);
              return { family: fe.coerceString(Se.family, Le, xe.family), size: fe.coerceNumber(Se.size, Qe, xe.size), color: fe.coerceColor(Se.color, ke, xe.color), weight: fe.coerceString(Se.weight, Me, xe.weight), style: fe.coerceString(Se.style, Re, xe.style), variant: fe.coerceString(Se.variant, De, xe.variant), textcase: fe.coerceString(Se.variant, Pe, xe.textcase), lineposition: fe.coerceString(Se.variant, $e, xe.lineposition), shadow: fe.coerceString(Se.variant, qe, xe.shadow) };
            }
            function Ce(Se, we) {
              return we.type === "waterfall" ? we[Se.dir].marker.color : Se.mcc || Se.mc || we.marker.color;
            }
            G.exports = { style: function(Se) {
              var we = te.select(Se).selectAll('g[class^="barlayer"]').selectAll("g.trace");
              se(Se, we, "bar");
              var Ee = we.size(), xe = Se._fullLayout;
              we.style("opacity", function(Le) {
                return Le[0].trace.opacity;
              }).each(function(Le) {
                (xe.barmode === "stack" && Ee > 1 || xe.bargap === 0 && xe.bargroupgap === 0 && !Le[0].trace.marker.line.width) && te.select(this).attr("shape-rendering", "crispEdges");
              }), we.selectAll("g.points").each(function(Le) {
                he(te.select(this), Le[0].trace, Se);
              }), oe.getComponentMethod("errorbars", "style")(we);
            }, styleTextPoints: me, styleOnSelect: function(Se, we, Ee) {
              var xe = we[0].trace;
              xe.selectedpoints ? function(Le, Qe, ke) {
                ee.selectedPointStyle(Le.selectAll("path"), Qe), function(Me, Re, De) {
                  Me.each(function(Pe) {
                    var $e, qe = te.select(this);
                    if (Pe.selected) {
                      $e = ae.ensureUniformFontSize(De, pe(qe, Pe, Re, De));
                      var Xe = Re.selected.textfont && Re.selected.textfont.color;
                      Xe && ($e.color = Xe), ee.font(qe, $e);
                    } else ee.selectedTextStyle(qe, Re);
                  });
                }(Le.selectAll("text"), Qe, ke);
              }(Ee, xe, Se) : (he(Ee, xe, Se), oe.getComponentMethod("errorbars", "style")(Ee));
            }, getInsideTextFont: ge, getOutsideTextFont: ve, getBarColor: Ce, resizeText: se };
          }, 59760: function(G, J, Z) {
            var te = Z(78766), ne = Z(65477).hasColorscale, ee = Z(39356), ae = Z(34809).coercePattern;
            G.exports = function(oe, se, ce, le, ue) {
              var de = ce("marker.color", le), fe = ne(oe, "marker");
              fe && ee(oe, se, ue, ce, { prefix: "marker.", cLetter: "c" }), ce("marker.line.color", te.defaultLine), ne(oe, "marker.line") && ee(oe, se, ue, ce, { prefix: "marker.line.", cLetter: "c" }), ce("marker.line.width"), ce("marker.opacity"), ae(ce, "marker.pattern", de, fe), ce("selected.marker.color"), ce("unselected.marker.color");
            };
          }, 84102: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809);
            function ee(ae) {
              return "_" + ae + "Text_minsize";
            }
            G.exports = { recordMinTextSize: function(ae, oe, se) {
              if (se.uniformtext.mode) {
                var ce = ee(ae), le = se.uniformtext.minsize, ue = oe.scale * oe.fontSize;
                oe.hide = ue < le, se[ce] = se[ce] || 1 / 0, oe.hide || (se[ce] = Math.min(se[ce], Math.max(ue, le)));
              }
            }, clearMinTextSize: function(ae, oe) {
              oe[ee(ae)] = void 0;
            }, resizeText: function(ae, oe, se) {
              var ce = ae._fullLayout, le = ce["_" + se + "Text_minsize"];
              if (le) {
                var ue, de = ce.uniformtext.mode === "hide";
                switch (se) {
                  case "funnelarea":
                  case "pie":
                  case "sunburst":
                    ue = "g.slice";
                    break;
                  case "treemap":
                  case "icicle":
                    ue = "g.slice, g.pathbar";
                    break;
                  default:
                    ue = "g.points > g.point";
                }
                oe.selectAll(ue).each(function(fe) {
                  var he = fe.transform;
                  if (he) {
                    he.scale = de && he.hide ? 0 : le / he.fontSize;
                    var me = te.select(this).select("text");
                    ne.setTransormAndDisplay(me, he);
                  }
                });
              }
            } };
          }, 32225: function(G, J, Z) {
            var te, ne = Z(3208).rb, ee = Z(93049).extendFlat, ae = Z(8738), oe = Z(81481);
            G.exports = { r: ae.r, theta: ae.theta, r0: ae.r0, dr: ae.dr, theta0: ae.theta0, dtheta: ae.dtheta, thetaunit: ae.thetaunit, base: ee({}, oe.base, {}), offset: ee({}, oe.offset, {}), width: ee({}, oe.width, {}), text: ee({}, oe.text, {}), hovertext: ee({}, oe.hovertext, {}), marker: (te = ee({}, oe.marker), delete te.cornerradius, te), hoverinfo: ae.hoverinfo, hovertemplate: ne(), selected: oe.selected, unselected: oe.unselected };
          }, 27941: function(G, J, Z) {
            var te = Z(65477).hasColorscale, ne = Z(28379), ee = Z(34809).isArrayOrTypedArray, ae = Z(35374), oe = Z(24782).setGroupPositions, se = Z(48861), ce = Z(33626).traceIs, le = Z(34809).extendFlat;
            G.exports = { calc: function(ue, de) {
              for (var fe = ue._fullLayout, he = de.subplot, me = fe[he].radialaxis, pe = fe[he].angularaxis, ye = me.makeCalcdata(de, "r"), ge = pe.makeCalcdata(de, "theta"), ve = de._length, be = new Array(ve), Ce = ye, Se = ge, we = 0; we < ve; we++) be[we] = { p: Se[we], s: Ce[we] };
              function Ee(xe) {
                var Le = de[xe];
                Le !== void 0 && (de["_" + xe] = ee(Le) ? pe.makeCalcdata(de, xe) : pe.d2c(Le, de.thetaunit));
              }
              return pe.type === "linear" && (Ee("width"), Ee("offset")), te(de, "marker") && ne(ue, de, { vals: de.marker.color, containerStr: "marker", cLetter: "c" }), te(de, "marker.line") && ne(ue, de, { vals: de.marker.line.color, containerStr: "marker.line", cLetter: "c" }), ae(be, de), se(be, de), be;
            }, crossTraceCalc: function(ue, de, fe) {
              for (var he = ue.calcdata, me = [], pe = 0; pe < he.length; pe++) {
                var ye = he[pe], ge = ye[0].trace;
                ge.visible === !0 && ce(ge, "bar") && ge.subplot === fe && me.push(ye);
              }
              var ve = le({}, de.radialaxis, { _id: "x" }), be = de.angularaxis;
              oe(ue, be, ve, me, { mode: de.barmode, norm: de.barnorm, gap: de.bargap, groupgap: de.bargroupgap });
            } };
          }, 77318: function(G, J, Z) {
            var te = Z(34809), ne = Z(73749).handleRThetaDefaults, ee = Z(59760), ae = Z(32225);
            G.exports = function(oe, se, ce, le) {
              function ue(de, fe) {
                return te.coerce(oe, se, ae, de, fe);
              }
              ne(oe, se, le, ue) ? (ue("thetaunit"), ue("base"), ue("offset"), ue("width"), ue("text"), ue("hovertext"), ue("hovertemplate"), ee(oe, se, ue, ce, le), te.coerceSelectionMarkerOpacity(se, ue)) : se.visible = !1;
            };
          }, 83080: function(G, J, Z) {
            var te = Z(32141), ne = Z(34809), ee = Z(91664).getTraceColor, ae = ne.fillText, oe = Z(29709).makeHoverPointText, se = Z(95928).isPtInsidePolygon;
            G.exports = function(ce, le, ue) {
              var de = ce.cd, fe = de[0].trace, he = ce.subplot, me = he.radialAxis, pe = he.angularAxis, ye = he.vangles, ge = ye ? se : ne.isPtInsideSector, ve = ce.maxHoverDistance, be = pe._period || 2 * Math.PI, Ce = Math.abs(me.g2p(Math.sqrt(le * le + ue * ue))), Se = Math.atan2(ue, le);
              if (me.range[0] > me.range[1] && (Se += Math.PI), te.getClosest(de, function(xe) {
                return ge(Ce, Se, [xe.rp0, xe.rp1], [xe.thetag0, xe.thetag1], ye) ? ve + Math.min(1, Math.abs(xe.thetag1 - xe.thetag0) / be) - 1 + (xe.rp1 - Ce) / (xe.rp1 - xe.rp0) - 1 : 1 / 0;
              }, ce), ce.index !== !1) {
                var we = de[ce.index];
                ce.x0 = ce.x1 = we.ct[0], ce.y0 = ce.y1 = we.ct[1];
                var Ee = ne.extendFlat({}, we, { r: we.s, theta: we.p });
                return ae(we, fe, ce), oe(Ee, fe, he, ce), ce.hovertemplate = fe.hovertemplate, ce.color = ee(fe, we), ce.xLabelVal = ce.yLabelVal = void 0, we.s < 0 && (ce.idealAlign = "left"), [ce];
              }
            };
          }, 89362: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "barpolar", basePlotModule: Z(31645), categories: ["polar", "bar", "showLegend"], attributes: Z(32225), layoutAttributes: Z(42956), supplyDefaults: Z(77318), supplyLayoutDefaults: Z(60507), calc: Z(27941).calc, crossTraceCalc: Z(27941).crossTraceCalc, plot: Z(11627), colorbar: Z(21146), formatLabels: Z(33368), style: Z(6851).style, styleOnSelect: Z(6851).styleOnSelect, hoverPoints: Z(83080), selectPoints: Z(88384), meta: {} };
          }, 42956: function(G) {
            G.exports = { barmode: { valType: "enumerated", values: ["stack", "overlay"], dflt: "stack", editType: "calc" }, bargap: { valType: "number", dflt: 0.1, min: 0, max: 1, editType: "calc" } };
          }, 60507: function(G, J, Z) {
            var te = Z(34809), ne = Z(42956);
            G.exports = function(ee, ae, oe) {
              var se, ce = {};
              function le(fe, he) {
                return te.coerce(ee[se] || {}, ae[se], ne, fe, he);
              }
              for (var ue = 0; ue < oe.length; ue++) {
                var de = oe[ue];
                de.type === "barpolar" && de.visible === !0 && (ce[se = de.subplot] || (le("barmode"), le("bargap"), ce[se] = 1));
              }
            };
          }, 11627: function(G, J, Z) {
            var te = Z(45568), ne = Z(10721), ee = Z(34809), ae = Z(62203), oe = Z(95928);
            G.exports = function(se, ce, le) {
              var ue = se._context.staticPlot, de = ce.xaxis, fe = ce.yaxis, he = ce.radialAxis, me = ce.angularAxis, pe = function(ge) {
                var ve = ge.cxx, be = ge.cyy;
                return ge.vangles ? function(Ce, Se, we, Ee) {
                  var xe, Le;
                  ee.angleDelta(we, Ee) > 0 ? (xe = we, Le = Ee) : (xe = Ee, Le = we);
                  var Qe = [oe.findEnclosingVertexAngles(xe, ge.vangles)[0], (xe + Le) / 2, oe.findEnclosingVertexAngles(Le, ge.vangles)[1]];
                  return oe.pathPolygonAnnulus(Ce, Se, xe, Le, Qe, ve, be);
                } : function(Ce, Se, we, Ee) {
                  return ee.pathAnnulus(Ce, Se, we, Ee, ve, be);
                };
              }(ce), ye = ce.layers.frontplot.select("g.barlayer");
              ee.makeTraceGroups(ye, le, "trace bars").each(function() {
                var ge = te.select(this), ve = ee.ensureSingle(ge, "g", "points").selectAll("g.point").data(ee.identity);
                ve.enter().append("g").style("vector-effect", ue ? "none" : "non-scaling-stroke").style("stroke-miterlimit", 2).classed("point", !0), ve.exit().remove(), ve.each(function(be) {
                  var Ce, Se = te.select(this), we = be.rp0 = he.c2p(be.s0), Ee = be.rp1 = he.c2p(be.s1), xe = be.thetag0 = me.c2g(be.p0), Le = be.thetag1 = me.c2g(be.p1);
                  if (ne(we) && ne(Ee) && ne(xe) && ne(Le) && we !== Ee && xe !== Le) {
                    var Qe = he.c2g(be.s1), ke = (xe + Le) / 2;
                    be.ct = [de.c2p(Qe * Math.cos(ke)), fe.c2p(Qe * Math.sin(ke))], Ce = pe(we, Ee, xe, Le);
                  } else Ce = "M0,0Z";
                  ee.ensureSingle(Se, "path").attr("d", Ce);
                }), ae.setClipUrl(ge, ce._hasClipOnAxisFalse ? ce.clipIds.forTraces : null, se);
              });
            };
          }, 64625: function(G, J, Z) {
            var te = Z(19326), ne = Z(36640), ee = Z(81481), ae = Z(10229), oe = Z(80712).axisHoverFormat, se = Z(3208).rb, ce = Z(93049).extendFlat, le = ne.marker, ue = le.line;
            G.exports = { y: { valType: "data_array", editType: "calc+clearAxisTypes" }, x: { valType: "data_array", editType: "calc+clearAxisTypes" }, x0: { valType: "any", editType: "calc+clearAxisTypes" }, y0: { valType: "any", editType: "calc+clearAxisTypes" }, dx: { valType: "number", editType: "calc" }, dy: { valType: "number", editType: "calc" }, xperiod: ne.xperiod, yperiod: ne.yperiod, xperiod0: ne.xperiod0, yperiod0: ne.yperiod0, xperiodalignment: ne.xperiodalignment, yperiodalignment: ne.yperiodalignment, xhoverformat: oe("x"), yhoverformat: oe("y"), name: { valType: "string", editType: "calc+clearAxisTypes" }, q1: { valType: "data_array", editType: "calc+clearAxisTypes" }, median: { valType: "data_array", editType: "calc+clearAxisTypes" }, q3: { valType: "data_array", editType: "calc+clearAxisTypes" }, lowerfence: { valType: "data_array", editType: "calc" }, upperfence: { valType: "data_array", editType: "calc" }, notched: { valType: "boolean", editType: "calc" }, notchwidth: { valType: "number", min: 0, max: 0.5, dflt: 0.25, editType: "calc" }, notchspan: { valType: "data_array", editType: "calc" }, boxpoints: { valType: "enumerated", values: ["all", "outliers", "suspectedoutliers", !1], editType: "calc" }, jitter: { valType: "number", min: 0, max: 1, editType: "calc" }, pointpos: { valType: "number", min: -2, max: 2, editType: "calc" }, sdmultiple: { valType: "number", min: 0, editType: "calc", dflt: 1 }, sizemode: { valType: "enumerated", values: ["quartiles", "sd"], editType: "calc", dflt: "quartiles" }, boxmean: { valType: "enumerated", values: [!0, "sd", !1], editType: "calc" }, mean: { valType: "data_array", editType: "calc" }, sd: { valType: "data_array", editType: "calc" }, orientation: { valType: "enumerated", values: ["v", "h"], editType: "calc+clearAxisTypes" }, quartilemethod: { valType: "enumerated", values: ["linear", "exclusive", "inclusive"], dflt: "linear", editType: "calc" }, width: { valType: "number", min: 0, dflt: 0, editType: "calc" }, marker: { outliercolor: { valType: "color", dflt: "rgba(0, 0, 0, 0)", editType: "style" }, symbol: ce({}, le.symbol, { arrayOk: !1, editType: "plot" }), opacity: ce({}, le.opacity, { arrayOk: !1, dflt: 1, editType: "style" }), angle: ce({}, le.angle, { arrayOk: !1, editType: "calc" }), size: ce({}, le.size, { arrayOk: !1, editType: "calc" }), color: ce({}, le.color, { arrayOk: !1, editType: "style" }), line: { color: ce({}, ue.color, { arrayOk: !1, dflt: ae.defaultLine, editType: "style" }), width: ce({}, ue.width, { arrayOk: !1, dflt: 0, editType: "style" }), outliercolor: { valType: "color", editType: "style" }, outlierwidth: { valType: "number", min: 0, dflt: 1, editType: "style" }, editType: "style" }, editType: "plot" }, line: { color: { valType: "color", editType: "style" }, width: { valType: "number", min: 0, dflt: 2, editType: "style" }, editType: "plot" }, fillcolor: te(), whiskerwidth: { valType: "number", min: 0, max: 1, dflt: 0.5, editType: "calc" }, showwhiskers: { valType: "boolean", editType: "calc" }, offsetgroup: ee.offsetgroup, alignmentgroup: ee.alignmentgroup, selected: { marker: ne.selected.marker, editType: "style" }, unselected: { marker: ne.unselected.marker, editType: "style" }, text: ce({}, ne.text, {}), hovertext: ce({}, ne.hovertext, {}), hovertemplate: se({}), hoveron: { valType: "flaglist", flags: ["boxes", "points"], dflt: "boxes+points", editType: "style" }, zorder: ne.zorder };
          }, 89429: function(G, J, Z) {
            var te = Z(10721), ne = Z(29714), ee = Z(40528), ae = Z(34809), oe = Z(63821).BADNUM, se = ae._;
            G.exports = function(ge, ve) {
              var be, Ce, Se, we, Ee, xe, Le, Qe = ge._fullLayout, ke = ne.getFromId(ge, ve.xaxis || "x"), Me = ne.getFromId(ge, ve.yaxis || "y"), Re = [], De = ve.type === "violin" ? "_numViolins" : "_numBoxes";
              ve.orientation === "h" ? (Se = ke, we = "x", Ee = Me, xe = "y", Le = !!ve.yperiodalignment) : (Se = Me, we = "y", Ee = ke, xe = "x", Le = !!ve.xperiodalignment);
              var Pe, $e, qe, Xe, Ve, Ze, Ge = function(Wt, rn, vn, dn) {
                var Ht, jt = rn + "0" in Wt;
                if (rn in Wt || jt && "d" + rn in Wt) {
                  var cn = vn.makeCalcdata(Wt, rn);
                  return [ee(Wt, vn, rn, cn).vals, cn];
                }
                Ht = jt ? Wt[rn + "0"] : "name" in Wt && (vn.type === "category" || te(Wt.name) && ["linear", "log"].indexOf(vn.type) !== -1 || ae.isDateTime(Wt.name) && vn.type === "date") ? Wt.name : dn;
                for (var wn = vn.type === "multicategory" ? vn.r2c_just_indices(Ht) : vn.d2c(Ht, 0, Wt[rn + "calendar"]), Rn = Wt._length, Pn = new Array(Rn), Ie = 0; Ie < Rn; Ie++) Pn[Ie] = wn;
                return [Pn];
              }(ve, xe, Ee, Qe[De]), ze = Ge[0], We = Ge[1], at = ae.distinctVals(ze, Ee), ct = at.vals, et = at.minDiff / 2, dt = (ve.boxpoints || ve.points) === "all" ? ae.identity : function(Wt) {
                return Wt.v < Pe.lf || Wt.v > Pe.uf;
              };
              if (ve._hasPreCompStats) {
                var Je = ve[we], it = function(Wt) {
                  return Se.d2c((ve[Wt] || [])[be]);
                }, st = 1 / 0, gt = -1 / 0;
                for (be = 0; be < ve._length; be++) {
                  var bt = ze[be];
                  if (te(bt)) {
                    if ((Pe = {}).pos = Pe[xe] = bt, Le && We && (Pe.orig_p = We[be]), Pe.q1 = it("q1"), Pe.med = it("median"), Pe.q3 = it("q3"), $e = [], Je && ae.isArrayOrTypedArray(Je[be])) for (Ce = 0; Ce < Je[be].length; Ce++) (Ze = Se.d2c(Je[be][Ce])) !== oe && (le(Ve = { v: Ze, i: [be, Ce] }, ve, [be, Ce]), $e.push(Ve));
                    if (Pe.pts = $e.sort(ue), Xe = (qe = Pe[we] = $e.map(de)).length, Pe.med !== oe && Pe.q1 !== oe && Pe.q3 !== oe && Pe.med >= Pe.q1 && Pe.q3 >= Pe.med) {
                      var yt = it("lowerfence");
                      Pe.lf = yt !== oe && yt <= Pe.q1 ? yt : fe(Pe, qe, Xe);
                      var St = it("upperfence");
                      Pe.uf = St !== oe && St >= Pe.q3 ? St : he(Pe, qe, Xe);
                      var Lt = it("mean");
                      Pe.mean = Lt !== oe ? Lt : Xe ? ae.mean(qe, Xe) : (Pe.q1 + Pe.q3) / 2;
                      var Rt = it("sd");
                      Pe.sd = Lt !== oe && Rt >= 0 ? Rt : Xe ? ae.stdev(qe, Xe, Pe.mean) : Pe.q3 - Pe.q1, Pe.lo = me(Pe), Pe.uo = pe(Pe);
                      var vt = it("notchspan");
                      vt = vt !== oe && vt > 0 ? vt : ye(Pe, Xe), Pe.ln = Pe.med - vt, Pe.un = Pe.med + vt;
                      var $t = Pe.lf, Pt = Pe.uf;
                      ve.boxpoints && qe.length && ($t = Math.min($t, qe[0]), Pt = Math.max(Pt, qe[Xe - 1])), ve.notched && ($t = Math.min($t, Pe.ln), Pt = Math.max(Pt, Pe.un)), Pe.min = $t, Pe.max = Pt;
                    } else {
                      var Xt;
                      ae.warn(["Invalid input - make sure that q1 <= median <= q3", "q1 = " + Pe.q1, "median = " + Pe.med, "q3 = " + Pe.q3].join(`
`)), Xt = Pe.med !== oe ? Pe.med : Pe.q1 !== oe ? Pe.q3 !== oe ? (Pe.q1 + Pe.q3) / 2 : Pe.q1 : Pe.q3 !== oe ? Pe.q3 : 0, Pe.med = Xt, Pe.q1 = Pe.q3 = Xt, Pe.lf = Pe.uf = Xt, Pe.mean = Pe.sd = Xt, Pe.ln = Pe.un = Xt, Pe.min = Pe.max = Xt;
                    }
                    st = Math.min(st, Pe.min), gt = Math.max(gt, Pe.max), Pe.pts2 = $e.filter(dt), Re.push(Pe);
                  }
                }
                ve._extremes[Se._id] = ne.findExtremes(Se, [st, gt], { padded: !0 });
              } else {
                var Nt = Se.makeCalcdata(ve, we), Gt = function(Wt, rn) {
                  for (var vn = Wt.length, dn = new Array(vn + 1), Ht = 0; Ht < vn; Ht++) dn[Ht] = Wt[Ht] - rn;
                  return dn[vn] = Wt[vn - 1] + rn, dn;
                }(ct, et), Ft = ct.length, Tt = function(Wt) {
                  for (var rn = new Array(Wt), vn = 0; vn < Wt; vn++) rn[vn] = [];
                  return rn;
                }(Ft);
                for (be = 0; be < ve._length; be++) if (Ze = Nt[be], te(Ze)) {
                  var It = ae.findBin(ze[be], Gt);
                  It >= 0 && It < Ft && (le(Ve = { v: Ze, i: be }, ve, be), Tt[It].push(Ve));
                }
                var Ct = 1 / 0, Kt = -1 / 0, Vt = ve.quartilemethod, Yt = Vt === "exclusive", sn = Vt === "inclusive";
                for (be = 0; be < Ft; be++) if (Tt[be].length > 0) {
                  var hn, Qt;
                  (Pe = {}).pos = Pe[xe] = ct[be], $e = Pe.pts = Tt[be].sort(ue), Xe = (qe = Pe[we] = $e.map(de)).length, Pe.min = qe[0], Pe.max = qe[Xe - 1], Pe.mean = ae.mean(qe, Xe), Pe.sd = ae.stdev(qe, Xe, Pe.mean) * ve.sdmultiple, Pe.med = ae.interp(qe, 0.5), Xe % 2 && (Yt || sn) ? (Yt ? (hn = qe.slice(0, Xe / 2), Qt = qe.slice(Xe / 2 + 1)) : sn && (hn = qe.slice(0, Xe / 2 + 1), Qt = qe.slice(Xe / 2)), Pe.q1 = ae.interp(hn, 0.5), Pe.q3 = ae.interp(Qt, 0.5)) : (Pe.q1 = ae.interp(qe, 0.25), Pe.q3 = ae.interp(qe, 0.75)), Pe.lf = fe(Pe, qe, Xe), Pe.uf = he(Pe, qe, Xe), Pe.lo = me(Pe), Pe.uo = pe(Pe);
                  var Ut = ye(Pe, Xe);
                  Pe.ln = Pe.med - Ut, Pe.un = Pe.med + Ut, Ct = Math.min(Ct, Pe.ln), Kt = Math.max(Kt, Pe.un), Pe.pts2 = $e.filter(dt), Re.push(Pe);
                }
                ve.notched && ae.isTypedArray(Nt) && (Nt = Array.from(Nt)), ve._extremes[Se._id] = ne.findExtremes(Se, ve.notched ? Nt.concat([Ct, Kt]) : Nt, { padded: !0 });
              }
              return function(Wt, rn) {
                if (ae.isArrayOrTypedArray(rn.selectedpoints)) for (var vn = 0; vn < Wt.length; vn++) {
                  for (var dn = Wt[vn].pts || [], Ht = {}, jt = 0; jt < dn.length; jt++) Ht[dn[jt].i] = jt;
                  ae.tagSelected(dn, rn, Ht);
                }
              }(Re, ve), Re.length > 0 ? (Re[0].t = { num: Qe[De], dPos: et, posLetter: xe, valLetter: we, labels: { med: se(ge, "median:"), min: se(ge, "min:"), q1: se(ge, "q1:"), q3: se(ge, "q3:"), max: se(ge, "max:"), mean: ve.boxmean === "sd" || ve.sizemode === "sd" ? se(ge, "mean ± σ:").replace("σ", ve.sdmultiple === 1 ? "σ" : ve.sdmultiple + "σ") : se(ge, "mean:"), lf: se(ge, "lower fence:"), uf: se(ge, "upper fence:") } }, Qe[De]++, Re) : [{ t: { empty: !0 } }];
            };
            var ce = { text: "tx", hovertext: "htx" };
            function le(ge, ve, be) {
              for (var Ce in ce) ae.isArrayOrTypedArray(ve[Ce]) && (Array.isArray(be) ? ae.isArrayOrTypedArray(ve[Ce][be[0]]) && (ge[ce[Ce]] = ve[Ce][be[0]][be[1]]) : ge[ce[Ce]] = ve[Ce][be]);
            }
            function ue(ge, ve) {
              return ge.v - ve.v;
            }
            function de(ge) {
              return ge.v;
            }
            function fe(ge, ve, be) {
              return be === 0 ? ge.q1 : Math.min(ge.q1, ve[Math.min(ae.findBin(2.5 * ge.q1 - 1.5 * ge.q3, ve, !0) + 1, be - 1)]);
            }
            function he(ge, ve, be) {
              return be === 0 ? ge.q3 : Math.max(ge.q3, ve[Math.max(ae.findBin(2.5 * ge.q3 - 1.5 * ge.q1, ve), 0)]);
            }
            function me(ge) {
              return 4 * ge.q1 - 3 * ge.q3;
            }
            function pe(ge) {
              return 4 * ge.q3 - 3 * ge.q1;
            }
            function ye(ge, ve) {
              return ve === 0 ? 0 : 1.57 * (ge.q3 - ge.q1) / Math.sqrt(ve);
            }
          }, 81606: function(G, J, Z) {
            var te = Z(29714), ne = Z(34809), ee = Z(84391).getAxisGroup, ae = ["v", "h"];
            function oe(se, ce, le, ue) {
              var de, fe, he, me = ce.calcdata, pe = ce._fullLayout, ye = ue._id, ge = ye.charAt(0), ve = [], be = 0;
              for (de = 0; de < le.length; de++) for (he = me[le[de]], fe = 0; fe < he.length; fe++) ve.push(ue.c2l(he[fe].pos, !0)), be += (he[fe].pts2 || []).length;
              if (ve.length) {
                var Ce = ne.distinctVals(ve);
                ue.type !== "category" && ue.type !== "multicategory" || (Ce.minDiff = 1);
                var Se = Ce.minDiff / 2;
                te.minDtick(ue, Ce.minDiff, Ce.vals[0], !0);
                var we = pe[se === "violin" ? "_numViolins" : "_numBoxes"], Ee = pe[se + "mode"] === "group" && we > 1, xe = 1 - pe[se + "gap"], Le = 1 - pe[se + "groupgap"];
                for (de = 0; de < le.length; de++) {
                  var Qe, ke, Me, Re, De, Pe, $e = (he = me[le[de]])[0].trace, qe = he[0].t, Xe = $e.width, Ve = $e.side;
                  if (Xe) Qe = ke = Re = Xe / 2, Me = 0;
                  else if (Qe = Se, Ee) {
                    var Ze = ee(pe, ue._id) + $e.orientation, Ge = (pe._alignmentOpts[Ze] || {})[$e.alignmentgroup] || {}, ze = Object.keys(Ge.offsetGroups || {}).length, We = ze || we;
                    ke = Qe * xe * Le / We, Me = 2 * Qe * (((ze ? $e._offsetIndex : qe.num) + 0.5) / We - 0.5) * xe, Re = Qe * xe / We;
                  } else ke = Qe * xe * Le, Me = 0, Re = Qe;
                  qe.dPos = Qe, qe.bPos = Me, qe.bdPos = ke, qe.wHover = Re;
                  var at, ct, et, dt, Je, it, st = Me + ke, gt = !!Xe;
                  if (Ve === "positive" ? (De = Qe * (Xe ? 1 : 0.5), at = st, Pe = at = Me) : Ve === "negative" ? (De = at = Me, Pe = Qe * (Xe ? 1 : 0.5), ct = st) : (De = Pe = Qe, at = ct = st), ($e.boxpoints || $e.points) && be > 0) {
                    var bt = $e.pointpos, yt = $e.jitter, St = $e.marker.size / 2, Lt = 0;
                    bt + yt >= 0 && ((Lt = st * (bt + yt)) > De ? (gt = !0, Je = St, et = Lt) : Lt > at && (Je = St, et = De)), Lt <= De && (et = De);
                    var Rt = 0;
                    bt - yt <= 0 && ((Rt = -st * (bt - yt)) > Pe ? (gt = !0, it = St, dt = Rt) : Rt > ct && (it = St, dt = Pe)), Rt <= Pe && (dt = Pe);
                  } else et = De, dt = Pe;
                  var vt = new Array(he.length);
                  for (fe = 0; fe < he.length; fe++) vt[fe] = he[fe].pos;
                  $e._extremes[ye] = te.findExtremes(ue, vt, { padded: gt, vpadminus: dt, vpadplus: et, vpadLinearized: !0, ppadminus: { x: it, y: Je }[ge], ppadplus: { x: Je, y: it }[ge] });
                }
              }
            }
            G.exports = { crossTraceCalc: function(se, ce) {
              for (var le = se.calcdata, ue = ce.xaxis, de = ce.yaxis, fe = 0; fe < ae.length; fe++) {
                for (var he = ae[fe], me = he === "h" ? de : ue, pe = [], ye = 0; ye < le.length; ye++) {
                  var ge = le[ye], ve = ge[0].t, be = ge[0].trace;
                  be.visible !== !0 || be.type !== "box" && be.type !== "candlestick" || ve.empty || (be.orientation || "v") !== he || be.xaxis !== ue._id || be.yaxis !== de._id || pe.push(ye);
                }
                oe("box", se, pe, me);
              }
            }, setPositionOffset: oe };
          }, 62294: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626), ee = Z(78766), ae = Z(99669), oe = Z(36301), se = Z(9666), ce = Z(64625);
            function le(de, fe, he, me) {
              function pe(Ve) {
                var Ze = 0;
                return Ve && Ve.length && (Ze += 1, te.isArrayOrTypedArray(Ve[0]) && Ve[0].length && (Ze += 1)), Ze;
              }
              function ye(Ve) {
                return te.validate(de[Ve], ce[Ve]);
              }
              var ge, ve = he("y"), be = he("x");
              if (fe.type === "box") {
                var Ce = he("q1"), Se = he("median"), we = he("q3");
                fe._hasPreCompStats = Ce && Ce.length && Se && Se.length && we && we.length, ge = Math.min(te.minRowLength(Ce), te.minRowLength(Se), te.minRowLength(we));
              }
              var Ee, xe, Le = pe(ve), Qe = pe(be), ke = Le && te.minRowLength(ve), Me = Qe && te.minRowLength(be), Re = me.calendar, De = { autotypenumbers: me.autotypenumbers };
              if (fe._hasPreCompStats) switch (String(Qe) + String(Le)) {
                case "00":
                  var Pe = ye("x0") || ye("dx");
                  Ee = !ye("y0") && !ye("dy") || Pe ? "v" : "h", xe = ge;
                  break;
                case "10":
                  Ee = "v", xe = Math.min(ge, Me);
                  break;
                case "20":
                  Ee = "h", xe = Math.min(ge, be.length);
                  break;
                case "01":
                  Ee = "h", xe = Math.min(ge, ke);
                  break;
                case "02":
                  Ee = "v", xe = Math.min(ge, ve.length);
                  break;
                case "12":
                  Ee = "v", xe = Math.min(ge, Me, ve.length);
                  break;
                case "21":
                  Ee = "h", xe = Math.min(ge, be.length, ke);
                  break;
                case "11":
                  xe = 0;
                  break;
                case "22":
                  var $e, qe = !1;
                  for ($e = 0; $e < be.length; $e++) if (se(be[$e], Re, De) === "category") {
                    qe = !0;
                    break;
                  }
                  if (qe) Ee = "v", xe = Math.min(ge, Me, ve.length);
                  else {
                    for ($e = 0; $e < ve.length; $e++) if (se(ve[$e], Re, De) === "category") {
                      qe = !0;
                      break;
                    }
                    qe ? (Ee = "h", xe = Math.min(ge, be.length, ke)) : (Ee = "v", xe = Math.min(ge, Me, ve.length));
                  }
              }
              else Le > 0 ? (Ee = "v", xe = Qe > 0 ? Math.min(Me, ke) : Math.min(ke)) : Qe > 0 ? (Ee = "h", xe = Math.min(Me)) : xe = 0;
              if (xe) {
                fe._length = xe;
                var Xe = he("orientation", Ee);
                fe._hasPreCompStats ? Xe === "v" && Qe === 0 ? (he("x0", 0), he("dx", 1)) : Xe === "h" && Le === 0 && (he("y0", 0), he("dy", 1)) : Xe === "v" && Qe === 0 ? he("x0") : Xe === "h" && Le === 0 && he("y0"), ne.getComponentMethod("calendars", "handleTraceDefaults")(de, fe, ["x", "y"], me);
              } else fe.visible = !1;
            }
            function ue(de, fe, he, me) {
              var pe = me.prefix, ye = te.coerce2(de, fe, ce, "marker.outliercolor"), ge = he("marker.line.outliercolor"), ve = "outliers";
              fe._hasPreCompStats ? ve = "all" : (ye || ge) && (ve = "suspectedoutliers");
              var be = he(pe + "points", ve);
              be ? (he("jitter", be === "all" ? 0.3 : 0), he("pointpos", be === "all" ? -1.5 : 0), he("marker.symbol"), he("marker.opacity"), he("marker.size"), he("marker.angle"), he("marker.color", fe.line.color), he("marker.line.color"), he("marker.line.width"), be === "suspectedoutliers" && (he("marker.line.outliercolor", fe.marker.color), he("marker.line.outlierwidth")), he("selected.marker.color"), he("unselected.marker.color"), he("selected.marker.size"), he("unselected.marker.size"), he("text"), he("hovertext")) : delete fe.marker;
              var Ce = he("hoveron");
              Ce !== "all" && Ce.indexOf("points") === -1 || he("hovertemplate"), te.coerceSelectionMarkerOpacity(fe, he);
            }
            G.exports = { supplyDefaults: function(de, fe, he, me) {
              function pe(xe, Le) {
                return te.coerce(de, fe, ce, xe, Le);
              }
              if (le(de, fe, pe, me), fe.visible !== !1) {
                ae(de, fe, me, pe), pe("xhoverformat"), pe("yhoverformat");
                var ye = fe._hasPreCompStats;
                ye && (pe("lowerfence"), pe("upperfence")), pe("line.color", (de.marker || {}).color || he), pe("line.width"), pe("fillcolor", ee.addOpacity(fe.line.color, 0.5));
                var ge = !1;
                if (ye) {
                  var ve = pe("mean"), be = pe("sd");
                  ve && ve.length && (ge = !0, be && be.length && (ge = "sd"));
                }
                pe("whiskerwidth");
                var Ce, Se = pe("sizemode");
                Se === "quartiles" && (Ce = pe("boxmean", ge)), pe("showwhiskers", Se === "quartiles"), Se !== "sd" && Ce !== "sd" || pe("sdmultiple"), pe("width"), pe("quartilemethod");
                var we = !1;
                if (ye) {
                  var Ee = pe("notchspan");
                  Ee && Ee.length && (we = !0);
                } else te.validate(de.notchwidth, ce.notchwidth) && (we = !0);
                pe("notched", we) && pe("notchwidth"), ue(de, fe, pe, { prefix: "box" }), pe("zorder");
              }
            }, crossTraceDefaults: function(de, fe) {
              var he, me;
              function pe(ve) {
                return te.coerce(me._input, me, ce, ve);
              }
              for (var ye = 0; ye < de.length; ye++) {
                var ge = (me = de[ye]).type;
                ge !== "box" && ge !== "violin" || (he = me._input, fe[ge + "mode"] === "group" && oe(he, me, fe, pe));
              }
            }, handleSampleDefaults: le, handlePointsDefaults: ue };
          }, 76429: function(G) {
            G.exports = function(J, Z) {
              return Z.hoverOnBox && (J.hoverOnBox = Z.hoverOnBox), "xVal" in Z && (J.x = Z.xVal), "yVal" in Z && (J.y = Z.yVal), Z.xa && (J.xaxis = Z.xa), Z.ya && (J.yaxis = Z.ya), J;
            };
          }, 11448: function(G, J, Z) {
            var te = Z(29714), ne = Z(34809), ee = Z(32141), ae = Z(78766), oe = ne.fillText;
            function se(le, ue, de, fe) {
              var he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke = le.cd, Me = le.xa, Re = le.ya, De = ke[0].trace, Pe = ke[0].t, $e = De.type === "violin", qe = Pe.bdPos, Xe = Pe.wHover, Ve = function(Xt) {
                return pe.c2l(Xt.pos) + Pe.bPos - pe.c2l(ve);
              };
              $e && De.side !== "both" ? (De.side === "positive" && (Se = function(Xt) {
                var Nt = Ve(Xt);
                return ee.inbox(Nt, Nt + Xe, we);
              }, xe = qe, Le = 0), De.side === "negative" && (Se = function(Xt) {
                var Nt = Ve(Xt);
                return ee.inbox(Nt - Xe, Nt, we);
              }, xe = 0, Le = qe)) : (Se = function(Xt) {
                var Nt = Ve(Xt);
                return ee.inbox(Nt - Xe, Nt + Xe, we);
              }, xe = Le = qe), Qe = $e ? function(Xt) {
                return ee.inbox(Xt.span[0] - ge, Xt.span[1] - ge, we);
              } : function(Xt) {
                return ee.inbox(Xt.min - ge, Xt.max - ge, we);
              }, De.orientation === "h" ? (ge = ue, ve = de, be = Qe, Ce = Se, he = "y", pe = Re, me = "x", ye = Me) : (ge = de, ve = ue, be = Se, Ce = Qe, he = "x", pe = Me, me = "y", ye = Re);
              var Ze = Math.min(1, qe / Math.abs(pe.r2c(pe.range[1]) - pe.r2c(pe.range[0])));
              function Ge(Xt) {
                return (be(Xt) + Ce(Xt)) / 2;
              }
              we = le.maxHoverDistance - Ze, Ee = le.maxSpikeDistance - Ze;
              var ze = ee.getDistanceFunction(fe, be, Ce, Ge);
              if (ee.getClosest(ke, ze, le), le.index === !1) return [];
              var We = ke[le.index], at = De.line.color, ct = (De.marker || {}).color;
              ae.opacity(at) && De.line.width ? le.color = at : ae.opacity(ct) && De.boxpoints ? le.color = ct : le.color = De.fillcolor, le[he + "0"] = pe.c2p(We.pos + Pe.bPos - Le, !0), le[he + "1"] = pe.c2p(We.pos + Pe.bPos + xe, !0), le[he + "LabelVal"] = We.orig_p !== void 0 ? We.orig_p : We.pos;
              var et = he + "Spike";
              le.spikeDistance = Ge(We) * Ee / we, le[et] = pe.c2p(We.pos, !0);
              var dt = De.boxmean || De.sizemode === "sd" || (De.meanline || {}).visible, Je = De.boxpoints || De.points, it = Je && dt ? ["max", "uf", "q3", "med", "mean", "q1", "lf", "min"] : Je && !dt ? ["max", "uf", "q3", "med", "q1", "lf", "min"] : !Je && dt ? ["max", "q3", "med", "mean", "q1", "min"] : ["max", "q3", "med", "q1", "min"], st = ye.range[1] < ye.range[0];
              De.orientation === (st ? "v" : "h") && it.reverse();
              for (var gt = le.spikeDistance, bt = le[et], yt = [], St = 0; St < it.length; St++) {
                var Lt = it[St];
                if (Lt in We) {
                  var Rt = We[Lt], vt = ye.c2p(Rt, !0), $t = ne.extendFlat({}, le);
                  $t.attr = Lt, $t[me + "0"] = $t[me + "1"] = vt, $t[me + "LabelVal"] = Rt, $t[me + "Label"] = (Pe.labels ? Pe.labels[Lt] + " " : "") + te.hoverLabelText(ye, Rt, De[me + "hoverformat"]), $t.hoverOnBox = !0, Lt !== "mean" || !("sd" in We) || De.boxmean !== "sd" && De.sizemode !== "sd" || ($t[me + "err"] = We.sd), $t.hovertemplate = !1, yt.push($t);
                }
              }
              le.name = "", le.spikeDistance = void 0, le[et] = void 0;
              for (var Pt = 0; Pt < yt.length; Pt++) yt[Pt].attr !== "med" ? (yt[Pt].name = "", yt[Pt].spikeDistance = void 0, yt[Pt][et] = void 0) : (yt[Pt].spikeDistance = gt, yt[Pt][et] = bt);
              return yt;
            }
            function ce(le, ue, de) {
              for (var fe, he, me, pe = le.cd, ye = le.xa, ge = le.ya, ve = pe[0].trace, be = ye.c2p(ue), Ce = ge.c2p(de), Se = ee.quadrature(function($e) {
                var qe = Math.max(3, $e.mrc || 0);
                return Math.max(Math.abs(ye.c2p($e.x) - be) - qe, 1 - 3 / qe);
              }, function($e) {
                var qe = Math.max(3, $e.mrc || 0);
                return Math.max(Math.abs(ge.c2p($e.y) - Ce) - qe, 1 - 3 / qe);
              }), we = !1, Ee = 0; Ee < pe.length; Ee++) {
                he = pe[Ee];
                for (var xe = 0; xe < (he.pts || []).length; xe++) {
                  var Le = Se(me = he.pts[xe]);
                  Le <= le.distance && (le.distance = Le, we = [Ee, xe]);
                }
              }
              if (!we) return !1;
              me = (he = pe[we[0]]).pts[we[1]];
              var Qe = ye.c2p(me.x, !0), ke = ge.c2p(me.y, !0), Me = me.mrc || 1;
              fe = ne.extendFlat({}, le, { index: me.i, color: (ve.marker || {}).color, name: ve.name, x0: Qe - Me, x1: Qe + Me, y0: ke - Me, y1: ke + Me, spikeDistance: le.distance, hovertemplate: ve.hovertemplate });
              var Re, De = he.orig_p, Pe = De !== void 0 ? De : he.pos;
              return ve.orientation === "h" ? (Re = ge, fe.xLabelVal = me.x, fe.yLabelVal = Pe) : (Re = ye, fe.xLabelVal = Pe, fe.yLabelVal = me.y), fe[Re._id.charAt(0) + "Spike"] = Re.c2p(he.pos, !0), oe(me, ve, fe), fe;
            }
            G.exports = { hoverPoints: function(le, ue, de, fe) {
              var he, me = le.cd[0].trace.hoveron, pe = [];
              return me.indexOf("boxes") !== -1 && (pe = pe.concat(se(le, ue, de, fe))), me.indexOf("points") !== -1 && (he = ce(le, ue, de)), fe === "closest" ? he ? [he] : pe : (he && pe.push(he), pe);
            }, hoverOnBoxes: se, hoverOnPoints: ce };
          }, 53794: function(G, J, Z) {
            G.exports = { attributes: Z(64625), layoutAttributes: Z(64636), supplyDefaults: Z(62294).supplyDefaults, crossTraceDefaults: Z(62294).crossTraceDefaults, supplyLayoutDefaults: Z(65067).supplyLayoutDefaults, calc: Z(89429), crossTraceCalc: Z(81606).crossTraceCalc, plot: Z(95419).plot, style: Z(59979).style, styleOnSelect: Z(59979).styleOnSelect, hoverPoints: Z(11448).hoverPoints, eventData: Z(76429), selectPoints: Z(72488), moduleType: "trace", name: "box", basePlotModule: Z(37703), categories: ["cartesian", "svg", "symbols", "oriented", "box-violin", "showLegend", "boxLayout", "zoomScale"], meta: {} };
          }, 64636: function(G) {
            G.exports = { boxmode: { valType: "enumerated", values: ["group", "overlay"], dflt: "overlay", editType: "calc" }, boxgap: { valType: "number", min: 0, max: 1, dflt: 0.3, editType: "calc" }, boxgroupgap: { valType: "number", min: 0, max: 1, dflt: 0.3, editType: "calc" } };
          }, 65067: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(64636);
            function ae(oe, se, ce, le, ue) {
              for (var de = ue + "Layout", fe = !1, he = 0; he < ce.length; he++) {
                var me = ce[he];
                if (te.traceIs(me, de)) {
                  fe = !0;
                  break;
                }
              }
              fe && (le(ue + "mode"), le(ue + "gap"), le(ue + "groupgap"));
            }
            G.exports = { supplyLayoutDefaults: function(oe, se, ce) {
              ae(0, 0, ce, function(le, ue) {
                return ne.coerce(oe, se, ee, le, ue);
              }, "box");
            }, _supply: ae };
          }, 95419: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203);
            function ae(ce, le, ue, de, fe) {
              var he, me, pe = ue.orientation === "h", ye = le.val, ge = le.pos, ve = !!ge.rangebreaks, be = de.bPos, Ce = de.wdPos || 0, Se = de.bPosPxOffset || 0, we = ue.whiskerwidth || 0, Ee = ue.showwhiskers !== !1, xe = ue.notched || !1, Le = xe ? 1 - 2 * ue.notchwidth : 1;
              Array.isArray(de.bdPos) ? (he = de.bdPos[0], me = de.bdPos[1]) : (he = de.bdPos, me = de.bdPos);
              var Qe = ce.selectAll("path.box").data(ue.type !== "violin" || ue.box.visible ? ne.identity : []);
              Qe.enter().append("path").style("vector-effect", fe ? "none" : "non-scaling-stroke").attr("class", "box"), Qe.exit().remove(), Qe.each(function(ke) {
                if (ke.empty) return te.select(this).attr("d", "M0,0Z");
                var Me = ge.c2l(ke.pos + be, !0), Re = ge.l2p(Me - he) + Se, De = ge.l2p(Me + me) + Se, Pe = ve ? (Re + De) / 2 : ge.l2p(Me) + Se, $e = ue.whiskerwidth, qe = ve ? Re * $e + (1 - $e) * Pe : ge.l2p(Me - Ce) + Se, Xe = ve ? De * $e + (1 - $e) * Pe : ge.l2p(Me + Ce) + Se, Ve = ge.l2p(Me - he * Le) + Se, Ze = ge.l2p(Me + me * Le) + Se, Ge = ue.sizemode === "sd", ze = ye.c2p(Ge ? ke.mean - ke.sd : ke.q1, !0), We = Ge ? ye.c2p(ke.mean + ke.sd, !0) : ye.c2p(ke.q3, !0), at = ne.constrain(Ge ? ye.c2p(ke.mean, !0) : ye.c2p(ke.med, !0), Math.min(ze, We) + 1, Math.max(ze, We) - 1), ct = ke.lf === void 0 || ue.boxpoints === !1 || Ge, et = ye.c2p(ct ? ke.min : ke.lf, !0), dt = ye.c2p(ct ? ke.max : ke.uf, !0), Je = ye.c2p(ke.ln, !0), it = ye.c2p(ke.un, !0);
                pe ? te.select(this).attr("d", "M" + at + "," + Ve + "V" + Ze + "M" + ze + "," + Re + "V" + De + (xe ? "H" + Je + "L" + at + "," + Ze + "L" + it + "," + De : "") + "H" + We + "V" + Re + (xe ? "H" + it + "L" + at + "," + Ve + "L" + Je + "," + Re : "") + "Z" + (Ee ? "M" + ze + "," + Pe + "H" + et + "M" + We + "," + Pe + "H" + dt + (we === 0 ? "" : "M" + et + "," + qe + "V" + Xe + "M" + dt + "," + qe + "V" + Xe) : "")) : te.select(this).attr("d", "M" + Ve + "," + at + "H" + Ze + "M" + Re + "," + ze + "H" + De + (xe ? "V" + Je + "L" + Ze + "," + at + "L" + De + "," + it : "") + "V" + We + "H" + Re + (xe ? "V" + it + "L" + Ve + "," + at + "L" + Re + "," + Je : "") + "Z" + (Ee ? "M" + Pe + "," + ze + "V" + et + "M" + Pe + "," + We + "V" + dt + (we === 0 ? "" : "M" + qe + "," + et + "H" + Xe + "M" + qe + "," + dt + "H" + Xe) : ""));
              });
            }
            function oe(ce, le, ue, de) {
              var fe = le.x, he = le.y, me = de.bdPos, pe = de.bPos, ye = ue.boxpoints || ue.points;
              ne.seedPseudoRandom();
              var ge = ce.selectAll("g.points").data(ye ? function(be) {
                return be.forEach(function(Ce) {
                  Ce.t = de, Ce.trace = ue;
                }), be;
              } : []);
              ge.enter().append("g").attr("class", "points"), ge.exit().remove();
              var ve = ge.selectAll("path").data(function(be) {
                var Ce, Se, we = be.pts2, Ee = Math.max((be.max - be.min) / 10, be.q3 - be.q1), xe = 1e-9 * Ee, Le = 0.01 * Ee, Qe = [], ke = 0;
                if (ue.jitter) {
                  if (Ee === 0) for (ke = 1, Qe = new Array(we.length), Ce = 0; Ce < we.length; Ce++) Qe[Ce] = 1;
                  else for (Ce = 0; Ce < we.length; Ce++) {
                    var Me = Math.max(0, Ce - 5), Re = we[Me].v, De = Math.min(we.length - 1, Ce + 5), Pe = we[De].v;
                    ye !== "all" && (we[Ce].v < be.lf ? Pe = Math.min(Pe, be.lf) : Re = Math.max(Re, be.uf));
                    var $e = Math.sqrt(Le * (De - Me) / (Pe - Re + xe)) || 0;
                    $e = ne.constrain(Math.abs($e), 0, 1), Qe.push($e), ke = Math.max($e, ke);
                  }
                  Se = 2 * ue.jitter / (ke || 1);
                }
                for (Ce = 0; Ce < we.length; Ce++) {
                  var qe = we[Ce], Xe = qe.v, Ve = ue.jitter ? Se * Qe[Ce] * (ne.pseudoRandom() - 0.5) : 0, Ze = be.pos + pe + me * (ue.pointpos + Ve);
                  ue.orientation === "h" ? (qe.y = Ze, qe.x = Xe) : (qe.x = Ze, qe.y = Xe), ye === "suspectedoutliers" && Xe < be.uo && Xe > be.lo && (qe.so = !0);
                }
                return we;
              });
              ve.enter().append("path").classed("point", !0), ve.exit().remove(), ve.call(ee.translatePoints, fe, he);
            }
            function se(ce, le, ue, de) {
              var fe, he, me = le.val, pe = le.pos, ye = !!pe.rangebreaks, ge = de.bPos, ve = de.bPosPxOffset || 0, be = ue.boxmean || (ue.meanline || {}).visible;
              Array.isArray(de.bdPos) ? (fe = de.bdPos[0], he = de.bdPos[1]) : (fe = de.bdPos, he = de.bdPos);
              var Ce = ce.selectAll("path.mean").data(ue.type === "box" && ue.boxmean || ue.type === "violin" && ue.box.visible && ue.meanline.visible ? ne.identity : []);
              Ce.enter().append("path").attr("class", "mean").style({ fill: "none", "vector-effect": "non-scaling-stroke" }), Ce.exit().remove(), Ce.each(function(Se) {
                var we = pe.c2l(Se.pos + ge, !0), Ee = pe.l2p(we - fe) + ve, xe = pe.l2p(we + he) + ve, Le = ye ? (Ee + xe) / 2 : pe.l2p(we) + ve, Qe = me.c2p(Se.mean, !0), ke = me.c2p(Se.mean - Se.sd, !0), Me = me.c2p(Se.mean + Se.sd, !0);
                ue.orientation === "h" ? te.select(this).attr("d", "M" + Qe + "," + Ee + "V" + xe + (be === "sd" ? "m0,0L" + ke + "," + Le + "L" + Qe + "," + Ee + "L" + Me + "," + Le + "Z" : "")) : te.select(this).attr("d", "M" + Ee + "," + Qe + "H" + xe + (be === "sd" ? "m0,0L" + Le + "," + ke + "L" + Ee + "," + Qe + "L" + Le + "," + Me + "Z" : ""));
              });
            }
            G.exports = { plot: function(ce, le, ue, de) {
              var fe = ce._context.staticPlot, he = le.xaxis, me = le.yaxis;
              ne.makeTraceGroups(de, ue, "trace boxes").each(function(pe) {
                var ye, ge, ve = te.select(this), be = pe[0], Ce = be.t, Se = be.trace;
                Ce.wdPos = Ce.bdPos * Se.whiskerwidth, Se.visible !== !0 || Ce.empty ? ve.remove() : (Se.orientation === "h" ? (ye = me, ge = he) : (ye = he, ge = me), ae(ve, { pos: ye, val: ge }, Se, Ce, fe), oe(ve, { x: he, y: me }, Se, Ce), se(ve, { pos: ye, val: ge }, Se, Ce));
              });
            }, plotBoxAndWhiskers: ae, plotPoints: oe, plotBoxMean: se };
          }, 72488: function(G) {
            G.exports = function(J, Z) {
              var te, ne, ee = J.cd, ae = J.xaxis, oe = J.yaxis, se = [];
              if (Z === !1) for (te = 0; te < ee.length; te++) for (ne = 0; ne < (ee[te].pts || []).length; ne++) ee[te].pts[ne].selected = 0;
              else for (te = 0; te < ee.length; te++) for (ne = 0; ne < (ee[te].pts || []).length; ne++) {
                var ce = ee[te].pts[ne], le = ae.c2p(ce.x), ue = oe.c2p(ce.y);
                Z.contains([le, ue], null, ce.i, J) ? (se.push({ pointNumber: ce.i, x: ae.c2d(ce.x), y: oe.c2d(ce.y) }), ce.selected = 1) : ce.selected = 0;
              }
              return se;
            };
          }, 59979: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(62203);
            G.exports = { style: function(ae, oe, se) {
              var ce = se || te.select(ae).selectAll("g.trace.boxes");
              ce.style("opacity", function(le) {
                return le[0].trace.opacity;
              }), ce.each(function(le) {
                var ue = te.select(this), de = le[0].trace, fe = de.line.width;
                function he(ye, ge, ve, be) {
                  ye.style("stroke-width", ge + "px").call(ne.stroke, ve).call(ne.fill, be);
                }
                var me = ue.selectAll("path.box");
                if (de.type === "candlestick") me.each(function(ye) {
                  if (!ye.empty) {
                    var ge = te.select(this), ve = de[ye.dir];
                    he(ge, ve.line.width, ve.line.color, ve.fillcolor), ge.style("opacity", de.selectedpoints && !ye.selected ? 0.3 : 1);
                  }
                });
                else {
                  he(me, fe, de.line.color, de.fillcolor), ue.selectAll("path.mean").style({ "stroke-width": fe, "stroke-dasharray": 2 * fe + "px," + fe + "px" }).call(ne.stroke, de.line.color);
                  var pe = ue.selectAll("path.point");
                  ee.pointStyle(pe, de, ae);
                }
              });
            }, styleOnSelect: function(ae, oe, se) {
              var ce = oe[0].trace, le = se.selectAll("path.point");
              ce.selectedpoints ? ee.selectedPointStyle(le, ce) : ee.pointStyle(le, ce, ae);
            } };
          }, 24319: function(G, J, Z) {
            var te = Z(34809).extendFlat, ne = Z(80712).axisHoverFormat, ee = Z(86706), ae = Z(64625);
            function oe(se) {
              return { line: { color: te({}, ae.line.color, { dflt: se }), width: ae.line.width, editType: "style" }, fillcolor: ae.fillcolor, editType: "style" };
            }
            G.exports = { xperiod: ee.xperiod, xperiod0: ee.xperiod0, xperiodalignment: ee.xperiodalignment, xhoverformat: ne("x"), yhoverformat: ne("y"), x: ee.x, open: ee.open, high: ee.high, low: ee.low, close: ee.close, line: { width: te({}, ae.line.width, {}), editType: "style" }, increasing: oe(ee.increasing.line.color.dflt), decreasing: oe(ee.decreasing.line.color.dflt), text: ee.text, hovertext: ee.hovertext, whiskerwidth: te({}, ae.whiskerwidth, { dflt: 0 }), hoverlabel: ee.hoverlabel, zorder: ae.zorder };
          }, 63679: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(40528), ae = Z(95694).calcCommon;
            function oe(se, ce, le, ue) {
              return { min: le, q1: Math.min(se, ue), med: ue, q3: Math.max(se, ue), max: ce };
            }
            G.exports = function(se, ce) {
              var le = se._fullLayout, ue = ne.getFromId(se, ce.xaxis), de = ne.getFromId(se, ce.yaxis), fe = ue.makeCalcdata(ce, "x"), he = ee(ce, ue, "x", fe).vals, me = ae(se, ce, fe, he, de, oe);
              return me.length ? (te.extendFlat(me[0].t, { num: le._numBoxes, dPos: te.distinctVals(he).minDiff / 2, posLetter: "x", valLetter: "y" }), le._numBoxes++, me) : [{ t: { empty: !0 } }];
            };
          }, 57336: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(28270), ae = Z(99669), oe = Z(24319);
            function se(ce, le, ue, de) {
              var fe = ue(de + ".line.color");
              ue(de + ".line.width", le.line.width), ue(de + ".fillcolor", ne.addOpacity(fe, 0.5));
            }
            G.exports = function(ce, le, ue, de) {
              function fe(he, me) {
                return te.coerce(ce, le, oe, he, me);
              }
              ee(ce, le, fe, de) ? (ae(ce, le, de, fe, { x: !0 }), fe("xhoverformat"), fe("yhoverformat"), fe("line.width"), se(0, le, fe, "increasing"), se(0, le, fe, "decreasing"), fe("text"), fe("hovertext"), fe("whiskerwidth"), de._requestRangeslider[le.xaxis] = !0, fe("zorder")) : le.visible = !1;
            };
          }, 51252: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "candlestick", basePlotModule: Z(37703), categories: ["cartesian", "svg", "showLegend", "candlestick", "boxLayout"], meta: {}, attributes: Z(24319), layoutAttributes: Z(64636), supplyLayoutDefaults: Z(65067).supplyLayoutDefaults, crossTraceCalc: Z(81606).crossTraceCalc, supplyDefaults: Z(57336), calc: Z(63679), plot: Z(95419).plot, layerName: "boxlayer", style: Z(59979).style, hoverPoints: Z(93245).hoverPoints, selectPoints: Z(49343) };
          }, 8432: function(G, J, Z) {
            var te = Z(6038), ne = Z(78032);
            G.exports = function(ee, ae, oe, se, ce) {
              se("a") || (se("da"), se("a0")), se("b") || (se("db"), se("b0")), function(le, ue, de, fe) {
                ["aaxis", "baxis"].forEach(function(he) {
                  var me = he.charAt(0), pe = le[he] || {}, ye = ne.newContainer(ue, he), ge = { noAutotickangles: !0, noTicklabelshift: !0, noTicklabelstandoff: !0, noTicklabelstep: !0, tickfont: "x", id: me + "axis", letter: me, font: ue.font, name: he, data: le[me], calendar: ue.calendar, dfltColor: fe, bgColor: de.paper_bgcolor, autotypenumbersDflt: de.autotypenumbers, fullLayout: de };
                  te(pe, ye, ge), ye._categories = ye._categories || [], le[he] || pe.type === "-" || (le[he] = { type: pe.type });
                });
              }(ee, ae, oe, ce);
            };
          }, 97052: function(G, J, Z) {
            var te = Z(34809).isArrayOrTypedArray;
            function ne(ee, ae) {
              if (!te(ee) || ae >= 10) return null;
              for (var oe = 1 / 0, se = -1 / 0, ce = ee.length, le = 0; le < ce; le++) {
                var ue = ee[le];
                if (te(ue)) {
                  var de = ne(ue, ae + 1);
                  de && (oe = Math.min(de[0], oe), se = Math.max(de[1], se));
                } else oe = Math.min(ue, oe), se = Math.max(ue, se);
              }
              return [oe, se];
            }
            G.exports = function(ee) {
              return ne(ee, 0);
            };
          }, 43745: function(G, J, Z) {
            var te = Z(80337), ne = Z(86961), ee = Z(10229), ae = te({ editType: "calc" }), oe = Z(36640).zorder;
            ae.family.dflt = '"Open Sans", verdana, arial, sans-serif', ae.size.dflt = 12, ae.color.dflt = ee.defaultLine, G.exports = { carpet: { valType: "string", editType: "calc" }, x: { valType: "data_array", editType: "calc+clearAxisTypes" }, y: { valType: "data_array", editType: "calc+clearAxisTypes" }, a: { valType: "data_array", editType: "calc" }, a0: { valType: "number", dflt: 0, editType: "calc" }, da: { valType: "number", dflt: 1, editType: "calc" }, b: { valType: "data_array", editType: "calc" }, b0: { valType: "number", dflt: 0, editType: "calc" }, db: { valType: "number", dflt: 1, editType: "calc" }, cheaterslope: { valType: "number", dflt: 1, editType: "calc" }, aaxis: ne, baxis: ne, font: ae, color: { valType: "color", dflt: ee.defaultLine, editType: "plot" }, transforms: void 0, zorder: oe };
          }, 94903: function(G, J, Z) {
            var te = Z(34809).isArrayOrTypedArray;
            G.exports = function(ne, ee, ae, oe) {
              var se, ce, le, ue, de, fe, he, me, pe, ye, ge, ve, be, Ce = te(ae) ? "a" : "b", Se = (Ce === "a" ? ne.aaxis : ne.baxis).smoothing, we = Ce === "a" ? ne.a2i : ne.b2j, Ee = Ce === "a" ? ae : oe, xe = Ce === "a" ? oe : ae, Le = Ce === "a" ? ee.a.length : ee.b.length, Qe = Ce === "a" ? ee.b.length : ee.a.length, ke = Math.floor(Ce === "a" ? ne.b2j(xe) : ne.a2i(xe)), Me = Ce === "a" ? function(at) {
                return ne.evalxy([], at, ke);
              } : function(at) {
                return ne.evalxy([], ke, at);
              };
              Se && (le = Math.max(0, Math.min(Qe - 2, ke)), ue = ke - le, ce = Ce === "a" ? function(at, ct) {
                return ne.dxydi([], at, le, ct, ue);
              } : function(at, ct) {
                return ne.dxydj([], le, at, ue, ct);
              });
              var Re = we(Ee[0]), De = we(Ee[1]), Pe = Re < De ? 1 : -1, $e = 1e-8 * (De - Re), qe = Pe > 0 ? Math.floor : Math.ceil, Xe = Pe > 0 ? Math.ceil : Math.floor, Ve = Pe > 0 ? Math.min : Math.max, Ze = Pe > 0 ? Math.max : Math.min, Ge = qe(Re + $e), ze = Xe(De - $e), We = [[he = Me(Re)]];
              for (se = Ge; se * Pe < ze * Pe; se += Pe) de = [], ge = Ze(Re, se), be = (ve = Ve(De, se + Pe)) - ge, fe = Math.max(0, Math.min(Le - 2, Math.floor(0.5 * (ge + ve)))), me = Me(ve), Se && (pe = ce(fe, ge - fe), ye = ce(fe, ve - fe), de.push([he[0] + pe[0] / 3 * be, he[1] + pe[1] / 3 * be]), de.push([me[0] - ye[0] / 3 * be, me[1] - ye[1] / 3 * be])), de.push(me), We.push(de), he = me;
              return We;
            };
          }, 86961: function(G, J, Z) {
            var te = Z(80337), ne = Z(10229), ee = Z(25829), ae = Z(80712).descriptionWithDates, oe = Z(13582).overrideAll, se = Z(94850).T, ce = Z(93049).extendFlat;
            G.exports = { color: { valType: "color", editType: "calc" }, smoothing: { valType: "number", dflt: 1, min: 0, max: 1.3, editType: "calc" }, title: { text: { valType: "string", dflt: "", editType: "calc" }, font: te({ editType: "calc" }), offset: { valType: "number", dflt: 10, editType: "calc" }, editType: "calc" }, type: { valType: "enumerated", values: ["-", "linear", "date", "category"], dflt: "-", editType: "calc" }, autotypenumbers: ee.autotypenumbers, autorange: { valType: "enumerated", values: [!0, !1, "reversed"], dflt: !0, editType: "calc" }, rangemode: { valType: "enumerated", values: ["normal", "tozero", "nonnegative"], dflt: "normal", editType: "calc" }, range: { valType: "info_array", editType: "calc", items: [{ valType: "any", editType: "calc" }, { valType: "any", editType: "calc" }] }, fixedrange: { valType: "boolean", dflt: !1, editType: "calc" }, cheatertype: { valType: "enumerated", values: ["index", "value"], dflt: "value", editType: "calc" }, tickmode: { valType: "enumerated", values: ["linear", "array"], dflt: "array", editType: "calc" }, nticks: { valType: "integer", min: 0, dflt: 0, editType: "calc" }, tickvals: { valType: "data_array", editType: "calc" }, ticktext: { valType: "data_array", editType: "calc" }, showticklabels: { valType: "enumerated", values: ["start", "end", "both", "none"], dflt: "start", editType: "calc" }, labelalias: ce({}, ee.labelalias, { editType: "calc" }), tickfont: te({ editType: "calc" }), tickangle: { valType: "angle", dflt: "auto", editType: "calc" }, tickprefix: { valType: "string", dflt: "", editType: "calc" }, showtickprefix: { valType: "enumerated", values: ["all", "first", "last", "none"], dflt: "all", editType: "calc" }, ticksuffix: { valType: "string", dflt: "", editType: "calc" }, showticksuffix: { valType: "enumerated", values: ["all", "first", "last", "none"], dflt: "all", editType: "calc" }, showexponent: { valType: "enumerated", values: ["all", "first", "last", "none"], dflt: "all", editType: "calc" }, exponentformat: { valType: "enumerated", values: ["none", "e", "E", "power", "SI", "B"], dflt: "B", editType: "calc" }, minexponent: { valType: "number", dflt: 3, min: 0, editType: "calc" }, separatethousands: { valType: "boolean", dflt: !1, editType: "calc" }, tickformat: { valType: "string", dflt: "", editType: "calc", description: ae("tick label") }, tickformatstops: oe(ee.tickformatstops, "calc", "from-root"), categoryorder: { valType: "enumerated", values: ["trace", "category ascending", "category descending", "array"], dflt: "trace", editType: "calc" }, categoryarray: { valType: "data_array", editType: "calc" }, labelpadding: { valType: "integer", dflt: 10, editType: "calc" }, labelprefix: { valType: "string", editType: "calc" }, labelsuffix: { valType: "string", dflt: "", editType: "calc" }, showline: { valType: "boolean", dflt: !1, editType: "calc" }, linecolor: { valType: "color", dflt: ne.defaultLine, editType: "calc" }, linewidth: { valType: "number", min: 0, dflt: 1, editType: "calc" }, gridcolor: { valType: "color", editType: "calc" }, gridwidth: { valType: "number", min: 0, dflt: 1, editType: "calc" }, griddash: ce({}, se, { editType: "calc" }), showgrid: { valType: "boolean", dflt: !0, editType: "calc" }, minorgridcount: { valType: "integer", min: 0, dflt: 0, editType: "calc" }, minorgridwidth: { valType: "number", min: 0, dflt: 1, editType: "calc" }, minorgriddash: ce({}, se, { editType: "calc" }), minorgridcolor: { valType: "color", dflt: ne.lightLine, editType: "calc" }, startline: { valType: "boolean", editType: "calc" }, startlinecolor: { valType: "color", editType: "calc" }, startlinewidth: { valType: "number", dflt: 1, editType: "calc" }, endline: { valType: "boolean", editType: "calc" }, endlinewidth: { valType: "number", dflt: 1, editType: "calc" }, endlinecolor: { valType: "color", editType: "calc" }, tick0: { valType: "number", min: 0, dflt: 0, editType: "calc" }, dtick: { valType: "number", min: 0, dflt: 1, editType: "calc" }, arraytick0: { valType: "integer", min: 0, dflt: 0, editType: "calc" }, arraydtick: { valType: "integer", min: 1, dflt: 1, editType: "calc" }, _deprecated: { title: { valType: "string", editType: "calc" }, titlefont: te({ editType: "calc" }), titleoffset: { valType: "number", dflt: 10, editType: "calc" } }, editType: "calc" };
          }, 6038: function(G, J, Z) {
            var te = Z(43745), ne = Z(78766).addOpacity, ee = Z(33626), ae = Z(34809), oe = Z(22777), se = Z(12036), ce = Z(54616), le = Z(46473), ue = Z(19091), de = Z(9666);
            G.exports = function(fe, he, me) {
              var pe = me.letter, ye = me.font || {}, ge = te[pe + "axis"];
              function ve(Pe, $e) {
                return ae.coerce(fe, he, ge, Pe, $e);
              }
              function be(Pe, $e) {
                return ae.coerce2(fe, he, ge, Pe, $e);
              }
              me.name && (he._name = me.name, he._id = me.name), ve("autotypenumbers", me.autotypenumbersDflt);
              var Ce = ve("type");
              Ce === "-" && (me.data && function(Pe, $e) {
                if (Pe.type === "-") {
                  var qe = Pe._id.charAt(0), Xe = Pe[qe + "calendar"];
                  Pe.type = de($e, Xe, { autotypenumbers: Pe.autotypenumbers });
                }
              }(he, me.data), he.type === "-" ? he.type = "linear" : Ce = fe.type = he.type), ve("smoothing"), ve("cheatertype"), ve("showticklabels"), ve("labelprefix", pe + " = "), ve("labelsuffix"), ve("showtickprefix"), ve("showticksuffix"), ve("separatethousands"), ve("tickformat"), ve("exponentformat"), ve("minexponent"), ve("showexponent"), ve("categoryorder"), ve("tickmode"), ve("tickvals"), ve("ticktext"), ve("tick0"), ve("dtick"), he.tickmode === "array" && (ve("arraytick0"), ve("arraydtick")), ve("labelpadding"), he._hovertitle = pe, Ce === "date" && ee.getComponentMethod("calendars", "handleDefaults")(fe, he, "calendar", me.calendar), ue(he, me.fullLayout), he.c2p = ae.identity;
              var Se = ve("color", me.dfltColor), we = Se === fe.color ? Se : ye.color;
              ve("title.text") && (ae.coerceFont(ve, "title.font", ye, { overrideDflt: { size: ae.bigFont(ye.size), color: we } }), ve("title.offset")), ve("tickangle"), ve("autorange", !he.isValidRange(fe.range)) && ve("rangemode"), ve("range"), he.cleanRange(), ve("fixedrange"), oe(fe, he, ve, Ce), ce(fe, he, ve, Ce, me), se(fe, he, ve, Ce, me), le(fe, he, ve, { data: me.data, dataAttr: pe });
              var Ee = be("gridcolor", ne(Se, 0.3)), xe = be("gridwidth"), Le = be("griddash"), Qe = ve("showgrid");
              Qe || (delete he.gridcolor, delete he.gridwidth, delete he.griddash);
              var ke = be("startlinecolor", Se), Me = be("startlinewidth", xe);
              ve("startline", he.showgrid || !!ke || !!Me) || (delete he.startlinecolor, delete he.startlinewidth);
              var Re = be("endlinecolor", Se), De = be("endlinewidth", xe);
              return ve("endline", he.showgrid || !!Re || !!De) || (delete he.endlinecolor, delete he.endlinewidth), Qe ? (ve("minorgridcount"), ve("minorgridwidth", xe), ve("minorgriddash", Le), ve("minorgridcolor", ne(Ee, 0.06)), he.minorgridcount || (delete he.minorgridwidth, delete he.minorgriddash, delete he.minorgridcolor)) : (delete he.gridcolor, delete he.gridwidth, delete he.griddash), he.showticklabels === "none" && (delete he.tickfont, delete he.tickangle, delete he.showexponent, delete he.exponentformat, delete he.minexponent, delete he.tickformat, delete he.showticksuffix, delete he.showtickprefix), he.showticksuffix || delete he.ticksuffix, he.showtickprefix || delete he.tickprefix, ve("tickmode"), he;
            };
          }, 67525: function(G, J, Z) {
            var te = Z(29714), ne = Z(34809).isArray1D, ee = Z(89992), ae = Z(97052), oe = Z(4753), se = Z(93923), ce = Z(39373), le = Z(93877), ue = Z(13007), de = Z(87869), fe = Z(76842);
            G.exports = function(he, me) {
              var pe = te.getFromId(he, me.xaxis), ye = te.getFromId(he, me.yaxis), ge = me.aaxis, ve = me.baxis, be = me.x, Ce = me.y, Se = [];
              be && ne(be) && Se.push("x"), Ce && ne(Ce) && Se.push("y"), Se.length && de(me, ge, ve, "a", "b", Se);
              var we = me._a = me._a || me.a, Ee = me._b = me._b || me.b;
              be = me._x || me.x, Ce = me._y || me.y;
              var xe = {};
              if (me._cheater) {
                var Le = ge.cheatertype === "index" ? we.length : we, Qe = ve.cheatertype === "index" ? Ee.length : Ee;
                be = ee(Le, Qe, me.cheaterslope);
              }
              me._x = be = le(be), me._y = Ce = le(Ce), ue(be, we, Ee), ue(Ce, we, Ee), fe(me), me.setScale();
              var ke = ae(be), Me = ae(Ce), Re = 0.5 * (ke[1] - ke[0]), De = 0.5 * (ke[1] + ke[0]), Pe = 0.5 * (Me[1] - Me[0]), $e = 0.5 * (Me[1] + Me[0]), qe = 1.3;
              return ke = [De - Re * qe, De + Re * qe], Me = [$e - Pe * qe, $e + Pe * qe], me._extremes[pe._id] = te.findExtremes(pe, ke, { padded: !0 }), me._extremes[ye._id] = te.findExtremes(ye, Me, { padded: !0 }), oe(me, "a", "b"), oe(me, "b", "a"), se(me, ge), se(me, ve), xe.clipsegments = ce(me._xctrl, me._yctrl, ge, ve), xe.x = be, xe.y = Ce, xe.a = we, xe.b = Ee, [xe];
            };
          }, 39373: function(G) {
            G.exports = function(J, Z, te, ne) {
              var ee, ae, oe, se = [], ce = !!te.smoothing, le = !!ne.smoothing, ue = J[0].length - 1, de = J.length - 1;
              for (ee = 0, ae = [], oe = []; ee <= ue; ee++) ae[ee] = J[0][ee], oe[ee] = Z[0][ee];
              for (se.push({ x: ae, y: oe, bicubic: ce }), ee = 0, ae = [], oe = []; ee <= de; ee++) ae[ee] = J[ee][ue], oe[ee] = Z[ee][ue];
              for (se.push({ x: ae, y: oe, bicubic: le }), ee = ue, ae = [], oe = []; ee >= 0; ee--) ae[ue - ee] = J[de][ee], oe[ue - ee] = Z[de][ee];
              for (se.push({ x: ae, y: oe, bicubic: ce }), ee = de, ae = [], oe = []; ee >= 0; ee--) ae[de - ee] = J[ee][0], oe[de - ee] = Z[ee][0];
              return se.push({ x: ae, y: oe, bicubic: le }), se;
            };
          }, 4753: function(G, J, Z) {
            var te = Z(29714), ne = Z(93049).extendFlat;
            G.exports = function(ee, ae, oe) {
              var se, ce, le, ue, de, fe, he, me, pe, ye, ge, ve, be, Ce, Se = ee["_" + ae], we = ee[ae + "axis"], Ee = we._gridlines = [], xe = we._minorgridlines = [], Le = we._boundarylines = [], Qe = ee["_" + oe], ke = ee[oe + "axis"];
              we.tickmode === "array" && (we.tickvals = Se.slice());
              var Me = ee._xctrl, Re = ee._yctrl, De = Me[0].length, Pe = Me.length, $e = ee._a.length, qe = ee._b.length;
              te.prepTicks(we), we.tickmode === "array" && delete we.tickvals;
              var Xe = we.smoothing ? 3 : 1;
              function Ve(Ge) {
                var ze, We, at, ct, et, dt, Je, it, st, gt, bt, yt, St = [], Lt = [], Rt = {};
                if (ae === "b") for (We = ee.b2j(Ge), at = Math.floor(Math.max(0, Math.min(qe - 2, We))), ct = We - at, Rt.length = qe, Rt.crossLength = $e, Rt.xy = function(vt) {
                  return ee.evalxy([], vt, We);
                }, Rt.dxy = function(vt, $t) {
                  return ee.dxydi([], vt, at, $t, ct);
                }, ze = 0; ze < $e; ze++) dt = Math.min($e - 2, ze), Je = ze - dt, it = ee.evalxy([], ze, We), ke.smoothing && ze > 0 && (st = ee.dxydi([], ze - 1, at, 0, ct), St.push(et[0] + st[0] / 3), Lt.push(et[1] + st[1] / 3), gt = ee.dxydi([], ze - 1, at, 1, ct), St.push(it[0] - gt[0] / 3), Lt.push(it[1] - gt[1] / 3)), St.push(it[0]), Lt.push(it[1]), et = it;
                else for (ze = ee.a2i(Ge), dt = Math.floor(Math.max(0, Math.min($e - 2, ze))), Je = ze - dt, Rt.length = $e, Rt.crossLength = qe, Rt.xy = function(vt) {
                  return ee.evalxy([], ze, vt);
                }, Rt.dxy = function(vt, $t) {
                  return ee.dxydj([], dt, vt, Je, $t);
                }, We = 0; We < qe; We++) at = Math.min(qe - 2, We), ct = We - at, it = ee.evalxy([], ze, We), ke.smoothing && We > 0 && (bt = ee.dxydj([], dt, We - 1, Je, 0), St.push(et[0] + bt[0] / 3), Lt.push(et[1] + bt[1] / 3), yt = ee.dxydj([], dt, We - 1, Je, 1), St.push(it[0] - yt[0] / 3), Lt.push(it[1] - yt[1] / 3)), St.push(it[0]), Lt.push(it[1]), et = it;
                return Rt.axisLetter = ae, Rt.axis = we, Rt.crossAxis = ke, Rt.value = Ge, Rt.constvar = oe, Rt.index = me, Rt.x = St, Rt.y = Lt, Rt.smoothing = ke.smoothing, Rt;
              }
              function Ze(Ge) {
                var ze, We, at, ct, et, dt = [], Je = [], it = {};
                if (it.length = Se.length, it.crossLength = Qe.length, ae === "b") for (at = Math.max(0, Math.min(qe - 2, Ge)), et = Math.min(1, Math.max(0, Ge - at)), it.xy = function(st) {
                  return ee.evalxy([], st, Ge);
                }, it.dxy = function(st, gt) {
                  return ee.dxydi([], st, at, gt, et);
                }, ze = 0; ze < De; ze++) dt[ze] = Me[Ge * Xe][ze], Je[ze] = Re[Ge * Xe][ze];
                else for (We = Math.max(0, Math.min($e - 2, Ge)), ct = Math.min(1, Math.max(0, Ge - We)), it.xy = function(st) {
                  return ee.evalxy([], Ge, st);
                }, it.dxy = function(st, gt) {
                  return ee.dxydj([], We, st, ct, gt);
                }, ze = 0; ze < Pe; ze++) dt[ze] = Me[ze][Ge * Xe], Je[ze] = Re[ze][Ge * Xe];
                return it.axisLetter = ae, it.axis = we, it.crossAxis = ke, it.value = Se[Ge], it.constvar = oe, it.index = Ge, it.x = dt, it.y = Je, it.smoothing = ke.smoothing, it;
              }
              if (we.tickmode === "array") {
                for (ue = 5e-15, fe = (de = [Math.floor((Se.length - 1 - we.arraytick0) / we.arraydtick * (1 + ue)), Math.ceil(-we.arraytick0 / we.arraydtick / (1 + ue))].sort(function(Ge, ze) {
                  return Ge - ze;
                }))[0] - 1, he = de[1] + 1, me = fe; me < he; me++) (ce = we.arraytick0 + we.arraydtick * me) < 0 || ce > Se.length - 1 || Ee.push(ne(Ze(ce), { color: we.gridcolor, width: we.gridwidth, dash: we.griddash }));
                for (me = fe; me < he; me++) if (le = we.arraytick0 + we.arraydtick * me, ge = Math.min(le + we.arraydtick, Se.length - 1), !(le < 0 || le > Se.length - 1 || ge < 0 || ge > Se.length - 1)) for (ve = Se[le], be = Se[ge], se = 0; se < we.minorgridcount; se++) (Ce = ge - le) <= 0 || (ye = ve + (be - ve) * (se + 1) / (we.minorgridcount + 1) * (we.arraydtick / Ce)) < Se[0] || ye > Se[Se.length - 1] || xe.push(ne(Ve(ye), { color: we.minorgridcolor, width: we.minorgridwidth, dash: we.minorgriddash }));
                we.startline && Le.push(ne(Ze(0), { color: we.startlinecolor, width: we.startlinewidth })), we.endline && Le.push(ne(Ze(Se.length - 1), { color: we.endlinecolor, width: we.endlinewidth }));
              } else {
                for (ue = 5e-15, fe = (de = [Math.floor((Se[Se.length - 1] - we.tick0) / we.dtick * (1 + ue)), Math.ceil((Se[0] - we.tick0) / we.dtick / (1 + ue))].sort(function(Ge, ze) {
                  return Ge - ze;
                }))[0], he = de[1], me = fe; me <= he; me++) pe = we.tick0 + we.dtick * me, Ee.push(ne(Ve(pe), { color: we.gridcolor, width: we.gridwidth, dash: we.griddash }));
                for (me = fe - 1; me < he + 1; me++) for (pe = we.tick0 + we.dtick * me, se = 0; se < we.minorgridcount; se++) (ye = pe + we.dtick * (se + 1) / (we.minorgridcount + 1)) < Se[0] || ye > Se[Se.length - 1] || xe.push(ne(Ve(ye), { color: we.minorgridcolor, width: we.minorgridwidth, dash: we.minorgriddash }));
                we.startline && Le.push(ne(Ve(Se[0]), { color: we.startlinecolor, width: we.startlinewidth })), we.endline && Le.push(ne(Ve(Se[Se.length - 1]), { color: we.endlinecolor, width: we.endlinewidth }));
              }
            };
          }, 93923: function(G, J, Z) {
            var te = Z(29714), ne = Z(93049).extendFlat;
            G.exports = function(ee, ae) {
              var oe, se, ce, le = ae._labels = [], ue = ae._gridlines;
              for (oe = 0; oe < ue.length; oe++) ce = ue[oe], ["start", "both"].indexOf(ae.showticklabels) !== -1 && (se = te.tickText(ae, ce.value), ne(se, { prefix: void 0, suffix: void 0, endAnchor: !0, xy: ce.xy(0), dxy: ce.dxy(0, 0), axis: ce.axis, length: ce.crossAxis.length, font: ce.axis.tickfont, isFirst: oe === 0, isLast: oe === ue.length - 1 }), le.push(se)), ["end", "both"].indexOf(ae.showticklabels) !== -1 && (se = te.tickText(ae, ce.value), ne(se, { endAnchor: !1, xy: ce.xy(ce.crossLength - 1), dxy: ce.dxy(ce.crossLength - 2, 1), axis: ce.axis, length: ce.crossAxis.length, font: ce.axis.tickfont, isFirst: oe === 0, isLast: oe === ue.length - 1 }), le.push(se));
            };
          }, 49109: function(G) {
            G.exports = function(J, Z, te, ne) {
              var ee = J[0] - Z[0], ae = J[1] - Z[1], oe = te[0] - Z[0], se = te[1] - Z[1], ce = Math.pow(ee * ee + ae * ae, 0.25), le = Math.pow(oe * oe + se * se, 0.25), ue = (le * le * ee - ce * ce * oe) * ne, de = (le * le * ae - ce * ce * se) * ne, fe = le * (ce + le) * 3, he = ce * (ce + le) * 3;
              return [[Z[0] + (fe && ue / fe), Z[1] + (fe && de / fe)], [Z[0] - (he && ue / he), Z[1] - (he && de / he)]];
            };
          }, 89992: function(G, J, Z) {
            var te = Z(34809).isArrayOrTypedArray;
            G.exports = function(ne, ee, ae) {
              var oe, se, ce, le, ue, de, fe = [], he = te(ne) ? ne.length : ne, me = te(ee) ? ee.length : ee, pe = te(ne) ? ne : null, ye = te(ee) ? ee : null;
              pe && (ce = (pe.length - 1) / (pe[pe.length - 1] - pe[0]) / (he - 1)), ye && (le = (ye.length - 1) / (ye[ye.length - 1] - ye[0]) / (me - 1));
              var ge = 1 / 0, ve = -1 / 0;
              for (se = 0; se < me; se++) for (fe[se] = [], ue = ye ? (ye[se] - ye[0]) * le : se / (me - 1), oe = 0; oe < he; oe++) de = (pe ? (pe[oe] - pe[0]) * ce : oe / (he - 1)) - ue * ae, ge = Math.min(de, ge), ve = Math.max(de, ve), fe[se][oe] = de;
              var be = 1 / (ve - ge), Ce = -ge * be;
              for (se = 0; se < me; se++) for (oe = 0; oe < he; oe++) fe[se][oe] = be * fe[se][oe] + Ce;
              return fe;
            };
          }, 57075: function(G, J, Z) {
            var te = Z(49109), ne = Z(34809).ensureArray;
            function ee(ae, oe, se) {
              var ce = -0.5 * se[0] + 1.5 * oe[0], le = -0.5 * se[1] + 1.5 * oe[1];
              return [(2 * ce + ae[0]) / 3, (2 * le + ae[1]) / 3];
            }
            G.exports = function(ae, oe, se, ce, le, ue) {
              var de, fe, he, me, pe, ye, ge, ve, be, Ce, Se = se[0].length, we = se.length, Ee = le ? 3 * Se - 2 : Se, xe = ue ? 3 * we - 2 : we;
              for (ae = ne(ae, xe), oe = ne(oe, xe), he = 0; he < xe; he++) ae[he] = ne(ae[he], Ee), oe[he] = ne(oe[he], Ee);
              for (fe = 0, me = 0; fe < we; fe++, me += ue ? 3 : 1) for (pe = ae[me], ye = oe[me], ge = se[fe], ve = ce[fe], de = 0, he = 0; de < Se; de++, he += le ? 3 : 1) pe[he] = ge[de], ye[he] = ve[de];
              if (le) for (fe = 0, me = 0; fe < we; fe++, me += ue ? 3 : 1) {
                for (de = 1, he = 3; de < Se - 1; de++, he += 3) be = te([se[fe][de - 1], ce[fe][de - 1]], [se[fe][de], ce[fe][de]], [se[fe][de + 1], ce[fe][de + 1]], le), ae[me][he - 1] = be[0][0], oe[me][he - 1] = be[0][1], ae[me][he + 1] = be[1][0], oe[me][he + 1] = be[1][1];
                Ce = ee([ae[me][0], oe[me][0]], [ae[me][2], oe[me][2]], [ae[me][3], oe[me][3]]), ae[me][1] = Ce[0], oe[me][1] = Ce[1], Ce = ee([ae[me][Ee - 1], oe[me][Ee - 1]], [ae[me][Ee - 3], oe[me][Ee - 3]], [ae[me][Ee - 4], oe[me][Ee - 4]]), ae[me][Ee - 2] = Ce[0], oe[me][Ee - 2] = Ce[1];
              }
              if (ue) for (he = 0; he < Ee; he++) {
                for (me = 3; me < xe - 3; me += 3) be = te([ae[me - 3][he], oe[me - 3][he]], [ae[me][he], oe[me][he]], [ae[me + 3][he], oe[me + 3][he]], ue), ae[me - 1][he] = be[0][0], oe[me - 1][he] = be[0][1], ae[me + 1][he] = be[1][0], oe[me + 1][he] = be[1][1];
                Ce = ee([ae[0][he], oe[0][he]], [ae[2][he], oe[2][he]], [ae[3][he], oe[3][he]]), ae[1][he] = Ce[0], oe[1][he] = Ce[1], Ce = ee([ae[xe - 1][he], oe[xe - 1][he]], [ae[xe - 3][he], oe[xe - 3][he]], [ae[xe - 4][he], oe[xe - 4][he]]), ae[xe - 2][he] = Ce[0], oe[xe - 2][he] = Ce[1];
              }
              if (le && ue) for (me = 1; me < xe; me += (me + 1) % 3 == 0 ? 2 : 1) {
                for (he = 3; he < Ee - 3; he += 3) be = te([ae[me][he - 3], oe[me][he - 3]], [ae[me][he], oe[me][he]], [ae[me][he + 3], oe[me][he + 3]], le), ae[me][he - 1] = 0.5 * (ae[me][he - 1] + be[0][0]), oe[me][he - 1] = 0.5 * (oe[me][he - 1] + be[0][1]), ae[me][he + 1] = 0.5 * (ae[me][he + 1] + be[1][0]), oe[me][he + 1] = 0.5 * (oe[me][he + 1] + be[1][1]);
                Ce = ee([ae[me][0], oe[me][0]], [ae[me][2], oe[me][2]], [ae[me][3], oe[me][3]]), ae[me][1] = 0.5 * (ae[me][1] + Ce[0]), oe[me][1] = 0.5 * (oe[me][1] + Ce[1]), Ce = ee([ae[me][Ee - 1], oe[me][Ee - 1]], [ae[me][Ee - 3], oe[me][Ee - 3]], [ae[me][Ee - 4], oe[me][Ee - 4]]), ae[me][Ee - 2] = 0.5 * (ae[me][Ee - 2] + Ce[0]), oe[me][Ee - 2] = 0.5 * (oe[me][Ee - 2] + Ce[1]);
              }
              return [ae, oe];
            };
          }, 45923: function(G) {
            G.exports = { RELATIVE_CULL_TOLERANCE: 1e-6 };
          }, 39848: function(G) {
            G.exports = function(J, Z, te) {
              return Z && te ? function(ne, ee, ae, oe, se) {
                var ce, le, ue, de, fe, he;
                ne || (ne = []), ee *= 3, ae *= 3;
                var me = oe * oe, pe = 1 - oe, ye = pe * pe, ge = pe * oe * 2, ve = -3 * ye, be = 3 * (ye - ge), Ce = 3 * (ge - me), Se = 3 * me, we = se * se, Ee = we * se, xe = 1 - se, Le = xe * xe, Qe = Le * xe;
                for (he = 0; he < J.length; he++) ce = ve * (fe = J[he])[ae][ee] + be * fe[ae][ee + 1] + Ce * fe[ae][ee + 2] + Se * fe[ae][ee + 3], le = ve * fe[ae + 1][ee] + be * fe[ae + 1][ee + 1] + Ce * fe[ae + 1][ee + 2] + Se * fe[ae + 1][ee + 3], ue = ve * fe[ae + 2][ee] + be * fe[ae + 2][ee + 1] + Ce * fe[ae + 2][ee + 2] + Se * fe[ae + 2][ee + 3], de = ve * fe[ae + 3][ee] + be * fe[ae + 3][ee + 1] + Ce * fe[ae + 3][ee + 2] + Se * fe[ae + 3][ee + 3], ne[he] = Qe * ce + 3 * (Le * se * le + xe * we * ue) + Ee * de;
                return ne;
              } : Z ? function(ne, ee, ae, oe, se) {
                var ce, le, ue, de;
                ne || (ne = []), ee *= 3;
                var fe = oe * oe, he = 1 - oe, me = he * he, pe = he * oe * 2, ye = -3 * me, ge = 3 * (me - pe), ve = 3 * (pe - fe), be = 3 * fe, Ce = 1 - se;
                for (ue = 0; ue < J.length; ue++) ce = ye * (de = J[ue])[ae][ee] + ge * de[ae][ee + 1] + ve * de[ae][ee + 2] + be * de[ae][ee + 3], le = ye * de[ae + 1][ee] + ge * de[ae + 1][ee + 1] + ve * de[ae + 1][ee + 2] + be * de[ae + 1][ee + 3], ne[ue] = Ce * ce + se * le;
                return ne;
              } : te ? function(ne, ee, ae, oe, se) {
                var ce, le, ue, de, fe, he;
                ne || (ne = []), ae *= 3;
                var me = se * se, pe = me * se, ye = 1 - se, ge = ye * ye, ve = ge * ye;
                for (fe = 0; fe < J.length; fe++) ce = (he = J[fe])[ae][ee + 1] - he[ae][ee], le = he[ae + 1][ee + 1] - he[ae + 1][ee], ue = he[ae + 2][ee + 1] - he[ae + 2][ee], de = he[ae + 3][ee + 1] - he[ae + 3][ee], ne[fe] = ve * ce + 3 * (ge * se * le + ye * me * ue) + pe * de;
                return ne;
              } : function(ne, ee, ae, oe, se) {
                var ce, le, ue, de;
                ne || (ne = []);
                var fe = 1 - se;
                for (ue = 0; ue < J.length; ue++) ce = (de = J[ue])[ae][ee + 1] - de[ae][ee], le = de[ae + 1][ee + 1] - de[ae + 1][ee], ne[ue] = fe * ce + se * le;
                return ne;
              };
            };
          }, 41839: function(G) {
            G.exports = function(J, Z, te) {
              return Z && te ? function(ne, ee, ae, oe, se) {
                var ce, le, ue, de, fe, he;
                ne || (ne = []), ee *= 3, ae *= 3;
                var me = oe * oe, pe = me * oe, ye = 1 - oe, ge = ye * ye, ve = ge * ye, be = se * se, Ce = 1 - se, Se = Ce * Ce, we = Ce * se * 2, Ee = -3 * Se, xe = 3 * (Se - we), Le = 3 * (we - be), Qe = 3 * be;
                for (he = 0; he < J.length; he++) ce = Ee * (fe = J[he])[ae][ee] + xe * fe[ae + 1][ee] + Le * fe[ae + 2][ee] + Qe * fe[ae + 3][ee], le = Ee * fe[ae][ee + 1] + xe * fe[ae + 1][ee + 1] + Le * fe[ae + 2][ee + 1] + Qe * fe[ae + 3][ee + 1], ue = Ee * fe[ae][ee + 2] + xe * fe[ae + 1][ee + 2] + Le * fe[ae + 2][ee + 2] + Qe * fe[ae + 3][ee + 2], de = Ee * fe[ae][ee + 3] + xe * fe[ae + 1][ee + 3] + Le * fe[ae + 2][ee + 3] + Qe * fe[ae + 3][ee + 3], ne[he] = ve * ce + 3 * (ge * oe * le + ye * me * ue) + pe * de;
                return ne;
              } : Z ? function(ne, ee, ae, oe, se) {
                var ce, le, ue, de, fe, he;
                ne || (ne = []), ee *= 3;
                var me = se * se, pe = me * se, ye = 1 - se, ge = ye * ye, ve = ge * ye;
                for (fe = 0; fe < J.length; fe++) ce = (he = J[fe])[ae + 1][ee] - he[ae][ee], le = he[ae + 1][ee + 1] - he[ae][ee + 1], ue = he[ae + 1][ee + 2] - he[ae][ee + 2], de = he[ae + 1][ee + 3] - he[ae][ee + 3], ne[fe] = ve * ce + 3 * (ge * se * le + ye * me * ue) + pe * de;
                return ne;
              } : te ? function(ne, ee, ae, oe, se) {
                var ce, le, ue, de;
                ne || (ne = []), ae *= 3;
                var fe = 1 - oe, he = se * se, me = 1 - se, pe = me * me, ye = me * se * 2, ge = -3 * pe, ve = 3 * (pe - ye), be = 3 * (ye - he), Ce = 3 * he;
                for (ue = 0; ue < J.length; ue++) ce = ge * (de = J[ue])[ae][ee] + ve * de[ae + 1][ee] + be * de[ae + 2][ee] + Ce * de[ae + 3][ee], le = ge * de[ae][ee + 1] + ve * de[ae + 1][ee + 1] + be * de[ae + 2][ee + 1] + Ce * de[ae + 3][ee + 1], ne[ue] = fe * ce + oe * le;
                return ne;
              } : function(ne, ee, ae, oe, se) {
                var ce, le, ue, de;
                ne || (ne = []);
                var fe = 1 - oe;
                for (ue = 0; ue < J.length; ue++) ce = (de = J[ue])[ae + 1][ee] - de[ae][ee], le = de[ae + 1][ee + 1] - de[ae][ee + 1], ne[ue] = fe * ce + oe * le;
                return ne;
              };
            };
          }, 13828: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              var ae = Z - 2, oe = te - 2;
              return ne && ee ? function(se, ce, le) {
                var ue, de, fe, he, me, pe;
                se || (se = []);
                var ye = Math.max(0, Math.min(Math.floor(ce), ae)), ge = Math.max(0, Math.min(Math.floor(le), oe)), ve = Math.max(0, Math.min(1, ce - ye)), be = Math.max(0, Math.min(1, le - ge));
                ye *= 3, ge *= 3;
                var Ce = ve * ve, Se = Ce * ve, we = 1 - ve, Ee = we * we, xe = Ee * we, Le = be * be, Qe = Le * be, ke = 1 - be, Me = ke * ke, Re = Me * ke;
                for (pe = 0; pe < J.length; pe++) ue = xe * (me = J[pe])[ge][ye] + 3 * (Ee * ve * me[ge][ye + 1] + we * Ce * me[ge][ye + 2]) + Se * me[ge][ye + 3], de = xe * me[ge + 1][ye] + 3 * (Ee * ve * me[ge + 1][ye + 1] + we * Ce * me[ge + 1][ye + 2]) + Se * me[ge + 1][ye + 3], fe = xe * me[ge + 2][ye] + 3 * (Ee * ve * me[ge + 2][ye + 1] + we * Ce * me[ge + 2][ye + 2]) + Se * me[ge + 2][ye + 3], he = xe * me[ge + 3][ye] + 3 * (Ee * ve * me[ge + 3][ye + 1] + we * Ce * me[ge + 3][ye + 2]) + Se * me[ge + 3][ye + 3], se[pe] = Re * ue + 3 * (Me * be * de + ke * Le * fe) + Qe * he;
                return se;
              } : ne ? function(se, ce, le) {
                se || (se = []);
                var ue, de, fe, he, me, pe, ye = Math.max(0, Math.min(Math.floor(ce), ae)), ge = Math.max(0, Math.min(Math.floor(le), oe)), ve = Math.max(0, Math.min(1, ce - ye)), be = Math.max(0, Math.min(1, le - ge));
                ye *= 3;
                var Ce = ve * ve, Se = Ce * ve, we = 1 - ve, Ee = we * we, xe = Ee * we, Le = 1 - be;
                for (me = 0; me < J.length; me++) ue = Le * (pe = J[me])[ge][ye] + be * pe[ge + 1][ye], de = Le * pe[ge][ye + 1] + be * pe[ge + 1][ye + 1], fe = Le * pe[ge][ye + 2] + be * pe[ge + 1][ye + 1], he = Le * pe[ge][ye + 3] + be * pe[ge + 1][ye + 1], se[me] = xe * ue + 3 * (Ee * ve * de + we * Ce * fe) + Se * he;
                return se;
              } : ee ? function(se, ce, le) {
                se || (se = []);
                var ue, de, fe, he, me, pe, ye = Math.max(0, Math.min(Math.floor(ce), ae)), ge = Math.max(0, Math.min(Math.floor(le), oe)), ve = Math.max(0, Math.min(1, ce - ye)), be = Math.max(0, Math.min(1, le - ge));
                ge *= 3;
                var Ce = be * be, Se = Ce * be, we = 1 - be, Ee = we * we, xe = Ee * we, Le = 1 - ve;
                for (me = 0; me < J.length; me++) ue = Le * (pe = J[me])[ge][ye] + ve * pe[ge][ye + 1], de = Le * pe[ge + 1][ye] + ve * pe[ge + 1][ye + 1], fe = Le * pe[ge + 2][ye] + ve * pe[ge + 2][ye + 1], he = Le * pe[ge + 3][ye] + ve * pe[ge + 3][ye + 1], se[me] = xe * ue + 3 * (Ee * be * de + we * Ce * fe) + Se * he;
                return se;
              } : function(se, ce, le) {
                se || (se = []);
                var ue, de, fe, he, me = Math.max(0, Math.min(Math.floor(ce), ae)), pe = Math.max(0, Math.min(Math.floor(le), oe)), ye = Math.max(0, Math.min(1, ce - me)), ge = Math.max(0, Math.min(1, le - pe)), ve = 1 - ge, be = 1 - ye;
                for (fe = 0; fe < J.length; fe++) ue = be * (he = J[fe])[pe][me] + ye * he[pe][me + 1], de = be * he[pe + 1][me] + ye * he[pe + 1][me + 1], se[fe] = ve * ue + ge * de;
                return se;
              };
            };
          }, 13254: function(G, J, Z) {
            var te = Z(34809), ne = Z(10820), ee = Z(8432), ae = Z(43745), oe = Z(10229);
            G.exports = function(se, ce, le, ue) {
              function de(he, me) {
                return te.coerce(se, ce, ae, he, me);
              }
              ce._clipPathId = "clip" + ce.uid + "carpet";
              var fe = de("color", oe.defaultLine);
              te.coerceFont(de, "font", ue.font), de("carpet"), ee(se, ce, ue, de, fe), ce.a && ce.b ? (ce.a.length < 3 && (ce.aaxis.smoothing = 0), ce.b.length < 3 && (ce.baxis.smoothing = 0), ne(se, ce, de) || (ce.visible = !1), ce._cheater && de("cheaterslope"), de("zorder")) : ce.visible = !1;
            };
          }, 48050: function(G, J, Z) {
            G.exports = { attributes: Z(43745), supplyDefaults: Z(13254), plot: Z(87947), calc: Z(67525), animatable: !0, isContainer: !0, moduleType: "trace", name: "carpet", basePlotModule: Z(37703), categories: ["cartesian", "svg", "carpet", "carpetAxis", "notLegendIsolatable", "noMultiCategory", "noHover", "noSortingByValue"], meta: {} };
          }, 26571: function(G) {
            G.exports = function(J, Z) {
              for (var te, ne = J._fullData.length, ee = 0; ee < ne; ee++) {
                var ae = J._fullData[ee];
                if (ae.index !== Z.index && ae.type === "carpet" && (te || (te = ae), ae.carpet === Z.carpet)) return ae;
              }
              return te;
            };
          }, 3685: function(G) {
            G.exports = function(J, Z, te) {
              if (J.length === 0) return "";
              var ne, ee = [], ae = te ? 3 : 1;
              for (ne = 0; ne < J.length; ne += ae) ee.push(J[ne] + "," + Z[ne]), te && ne < J.length - ae && (ee.push("C"), ee.push([J[ne + 1] + "," + Z[ne + 1], J[ne + 2] + "," + Z[ne + 2] + " "].join(" ")));
              return ee.join(te ? "" : "L");
            };
          }, 6720: function(G, J, Z) {
            var te = Z(34809).isArrayOrTypedArray;
            G.exports = function(ne, ee, ae) {
              var oe;
              for (te(ne) ? ne.length > ee.length && (ne = ne.slice(0, ee.length)) : ne = [], oe = 0; oe < ee.length; oe++) ne[oe] = ae(ee[oe]);
              return ne;
            };
          }, 33163: function(G) {
            G.exports = function(J, Z, te, ne, ee, ae) {
              var oe = ee[0] * J.dpdx(Z), se = ee[1] * J.dpdy(te), ce = 1, le = 1;
              if (ae) {
                var ue = Math.sqrt(ee[0] * ee[0] + ee[1] * ee[1]), de = Math.sqrt(ae[0] * ae[0] + ae[1] * ae[1]), fe = (ee[0] * ae[0] + ee[1] * ae[1]) / ue / de;
                le = Math.max(0, fe);
              }
              var he = 180 * Math.atan2(se, oe) / Math.PI;
              return he < -90 ? (he += 180, ce = -ce) : he > 90 && (he -= 180, ce = -ce), { angle: he, flip: ce, p: J.c2p(ne, Z, te), offsetMultplier: le };
            };
          }, 87947: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(6720), ae = Z(3685), oe = Z(33163), se = Z(30635), ce = Z(34809), le = ce.strRotate, ue = ce.strTranslate, de = Z(4530);
            function fe(ge, ve, be, Ce, Se, we, Ee) {
              var xe = "const-" + Se + "-lines", Le = be.selectAll("." + xe).data(we);
              Le.enter().append("path").classed(xe, !0).style("vector-effect", Ee ? "none" : "non-scaling-stroke"), Le.each(function(Qe) {
                var ke = Qe, Me = ke.x, Re = ke.y, De = ee([], Me, ge.c2p), Pe = ee([], Re, ve.c2p), $e = "M" + ae(De, Pe, ke.smoothing);
                te.select(this).attr("d", $e).style("stroke-width", ke.width).style("stroke", ke.color).style("stroke-dasharray", ne.dashStyle(ke.dash, ke.width)).style("fill", "none");
              }), Le.exit().remove();
            }
            function he(ge, ve, be, Ce, Se, we, Ee, xe) {
              var Le = we.selectAll("text." + xe).data(Ee);
              Le.enter().append("text").classed(xe, !0);
              var Qe = 0, ke = {};
              return Le.each(function(Me, Re) {
                var De;
                if (Me.axis.tickangle === "auto") De = oe(Ce, ve, be, Me.xy, Me.dxy);
                else {
                  var Pe = (Me.axis.tickangle + 180) * Math.PI / 180;
                  De = oe(Ce, ve, be, Me.xy, [Math.cos(Pe), Math.sin(Pe)]);
                }
                Re || (ke = { angle: De.angle, flip: De.flip });
                var $e = (Me.endAnchor ? -1 : 1) * De.flip, qe = te.select(this).attr({ "text-anchor": $e > 0 ? "start" : "end", "data-notex": 1 }).call(ne.font, Me.font).text(Me.text).call(se.convertToTspans, ge), Xe = ne.bBox(this);
                qe.attr("transform", ue(De.p[0], De.p[1]) + le(De.angle) + ue(Me.axis.labelpadding * $e, 0.3 * Xe.height)), Qe = Math.max(Qe, Xe.width + Me.axis.labelpadding);
              }), Le.exit().remove(), ke.maxExtent = Qe, ke;
            }
            G.exports = function(ge, ve, be, Ce) {
              var Se = ge._context.staticPlot, we = ve.xaxis, Ee = ve.yaxis, xe = ge._fullLayout._clips;
              ce.makeTraceGroups(Ce, be, "trace").each(function(Le) {
                var Qe = te.select(this), ke = Le[0], Me = ke.trace, Re = Me.aaxis, De = Me.baxis, Pe = ce.ensureSingle(Qe, "g", "minorlayer"), $e = ce.ensureSingle(Qe, "g", "majorlayer"), qe = ce.ensureSingle(Qe, "g", "boundarylayer"), Xe = ce.ensureSingle(Qe, "g", "labellayer");
                Qe.style("opacity", Me.opacity), fe(we, Ee, $e, 0, "a", Re._gridlines, !0), fe(we, Ee, $e, 0, "b", De._gridlines, !0), fe(we, Ee, Pe, 0, "a", Re._minorgridlines, !0), fe(we, Ee, Pe, 0, "b", De._minorgridlines, !0), fe(we, Ee, qe, 0, "a-boundary", Re._boundarylines, Se), fe(we, Ee, qe, 0, "b-boundary", De._boundarylines, Se);
                var Ve = he(ge, we, Ee, Me, 0, Xe, Re._labels, "a-label"), Ze = he(ge, we, Ee, Me, 0, Xe, De._labels, "b-label");
                (function(Ge, ze, We, at, ct, et, dt, Je) {
                  var it, st, gt, bt, yt = ce.aggNums(Math.min, null, We.a), St = ce.aggNums(Math.max, null, We.a), Lt = ce.aggNums(Math.min, null, We.b), Rt = ce.aggNums(Math.max, null, We.b);
                  it = 0.5 * (yt + St), st = Lt, gt = We.ab2xy(it, st, !0), bt = We.dxyda_rough(it, st), dt.angle === void 0 && ce.extendFlat(dt, oe(We, ct, et, gt, We.dxydb_rough(it, st))), ye(Ge, ze, We, 0, gt, bt, We.aaxis, ct, et, dt, "a-title"), it = yt, st = 0.5 * (Lt + Rt), gt = We.ab2xy(it, st, !0), bt = We.dxydb_rough(it, st), Je.angle === void 0 && ce.extendFlat(Je, oe(We, ct, et, gt, We.dxyda_rough(it, st))), ye(Ge, ze, We, 0, gt, bt, We.baxis, ct, et, Je, "b-title");
                })(ge, Xe, Me, 0, we, Ee, Ve, Ze), function(Ge, ze, We, at, ct) {
                  var et, dt, Je, it, st = We.select("#" + Ge._clipPathId);
                  st.size() || (st = We.append("clipPath").classed("carpetclip", !0));
                  var gt = ce.ensureSingle(st, "path", "carpetboundary"), bt = ze.clipsegments, yt = [];
                  for (it = 0; it < bt.length; it++) et = bt[it], dt = ee([], et.x, at.c2p), Je = ee([], et.y, ct.c2p), yt.push(ae(dt, Je, et.bicubic));
                  var St = "M" + yt.join("L") + "Z";
                  st.attr("id", Ge._clipPathId), gt.attr("d", St);
                }(Me, ke, xe, we, Ee);
              });
            };
            var me = de.LINE_SPACING, pe = (1 - de.MID_SHIFT) / me + 1;
            function ye(ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke) {
              var Me = [];
              Ee.title.text && Me.push(Ee.title.text);
              var Re = ve.selectAll("text." + ke).data(Me), De = Qe.maxExtent;
              Re.enter().append("text").classed(ke, !0), Re.each(function() {
                var Pe = oe(be, xe, Le, Se, we);
                ["start", "both"].indexOf(Ee.showticklabels) === -1 && (De = 0);
                var $e = Ee.title.font.size;
                De += $e + Ee.title.offset;
                var qe = (Qe.angle + (Qe.flip < 0 ? 180 : 0) - Pe.angle + 450) % 360, Xe = qe > 90 && qe < 270, Ve = te.select(this);
                Ve.text(Ee.title.text).call(se.convertToTspans, ge), Xe && (De = (-se.lineCount(Ve) + pe) * me * $e - De), Ve.attr("transform", ue(Pe.p[0], Pe.p[1]) + le(Pe.angle) + ue(0, De)).attr("text-anchor", "middle").call(ne.font, Ee.title.font);
              }), Re.exit().remove();
            }
          }, 76842: function(G, J, Z) {
            var te = Z(45923), ne = Z(98813).findBin, ee = Z(57075), ae = Z(13828), oe = Z(39848), se = Z(41839);
            G.exports = function(ce) {
              var le = ce._a, ue = ce._b, de = le.length, fe = ue.length, he = ce.aaxis, me = ce.baxis, pe = le[0], ye = le[de - 1], ge = ue[0], ve = ue[fe - 1], be = le[le.length - 1] - le[0], Ce = ue[ue.length - 1] - ue[0], Se = be * te.RELATIVE_CULL_TOLERANCE, we = Ce * te.RELATIVE_CULL_TOLERANCE;
              pe -= Se, ye += Se, ge -= we, ve += we, ce.isVisible = function(Ee, xe) {
                return Ee > pe && Ee < ye && xe > ge && xe < ve;
              }, ce.isOccluded = function(Ee, xe) {
                return Ee < pe || Ee > ye || xe < ge || xe > ve;
              }, ce.setScale = function() {
                var Ee = ce._x, xe = ce._y, Le = ee(ce._xctrl, ce._yctrl, Ee, xe, he.smoothing, me.smoothing);
                ce._xctrl = Le[0], ce._yctrl = Le[1], ce.evalxy = ae([ce._xctrl, ce._yctrl], de, fe, he.smoothing, me.smoothing), ce.dxydi = oe([ce._xctrl, ce._yctrl], he.smoothing, me.smoothing), ce.dxydj = se([ce._xctrl, ce._yctrl], he.smoothing, me.smoothing);
              }, ce.i2a = function(Ee) {
                var xe = Math.max(0, Math.floor(Ee[0]), de - 2), Le = Ee[0] - xe;
                return (1 - Le) * le[xe] + Le * le[xe + 1];
              }, ce.j2b = function(Ee) {
                var xe = Math.max(0, Math.floor(Ee[1]), de - 2), Le = Ee[1] - xe;
                return (1 - Le) * ue[xe] + Le * ue[xe + 1];
              }, ce.ij2ab = function(Ee) {
                return [ce.i2a(Ee[0]), ce.j2b(Ee[1])];
              }, ce.a2i = function(Ee) {
                var xe = Math.max(0, Math.min(ne(Ee, le), de - 2)), Le = le[xe], Qe = le[xe + 1];
                return Math.max(0, Math.min(de - 1, xe + (Ee - Le) / (Qe - Le)));
              }, ce.b2j = function(Ee) {
                var xe = Math.max(0, Math.min(ne(Ee, ue), fe - 2)), Le = ue[xe], Qe = ue[xe + 1];
                return Math.max(0, Math.min(fe - 1, xe + (Ee - Le) / (Qe - Le)));
              }, ce.ab2ij = function(Ee) {
                return [ce.a2i(Ee[0]), ce.b2j(Ee[1])];
              }, ce.i2c = function(Ee, xe) {
                return ce.evalxy([], Ee, xe);
              }, ce.ab2xy = function(Ee, xe, Le) {
                if (!Le && (Ee < le[0] || Ee > le[de - 1] | xe < ue[0] || xe > ue[fe - 1])) return [!1, !1];
                var Qe = ce.a2i(Ee), ke = ce.b2j(xe), Me = ce.evalxy([], Qe, ke);
                if (Le) {
                  var Re, De, Pe, $e, qe = 0, Xe = 0, Ve = [];
                  Ee < le[0] ? (Re = 0, De = 0, qe = (Ee - le[0]) / (le[1] - le[0])) : Ee > le[de - 1] ? (Re = de - 2, De = 1, qe = (Ee - le[de - 1]) / (le[de - 1] - le[de - 2])) : De = Qe - (Re = Math.max(0, Math.min(de - 2, Math.floor(Qe)))), xe < ue[0] ? (Pe = 0, $e = 0, Xe = (xe - ue[0]) / (ue[1] - ue[0])) : xe > ue[fe - 1] ? (Pe = fe - 2, $e = 1, Xe = (xe - ue[fe - 1]) / (ue[fe - 1] - ue[fe - 2])) : $e = ke - (Pe = Math.max(0, Math.min(fe - 2, Math.floor(ke)))), qe && (ce.dxydi(Ve, Re, Pe, De, $e), Me[0] += Ve[0] * qe, Me[1] += Ve[1] * qe), Xe && (ce.dxydj(Ve, Re, Pe, De, $e), Me[0] += Ve[0] * Xe, Me[1] += Ve[1] * Xe);
                }
                return Me;
              }, ce.c2p = function(Ee, xe, Le) {
                return [xe.c2p(Ee[0]), Le.c2p(Ee[1])];
              }, ce.p2x = function(Ee, xe, Le) {
                return [xe.p2c(Ee[0]), Le.p2c(Ee[1])];
              }, ce.dadi = function(Ee) {
                var xe = Math.max(0, Math.min(le.length - 2, Ee));
                return le[xe + 1] - le[xe];
              }, ce.dbdj = function(Ee) {
                var xe = Math.max(0, Math.min(ue.length - 2, Ee));
                return ue[xe + 1] - ue[xe];
              }, ce.dxyda = function(Ee, xe, Le, Qe) {
                var ke = ce.dxydi(null, Ee, xe, Le, Qe), Me = ce.dadi(Ee, Le);
                return [ke[0] / Me, ke[1] / Me];
              }, ce.dxydb = function(Ee, xe, Le, Qe) {
                var ke = ce.dxydj(null, Ee, xe, Le, Qe), Me = ce.dbdj(xe, Qe);
                return [ke[0] / Me, ke[1] / Me];
              }, ce.dxyda_rough = function(Ee, xe, Le) {
                var Qe = be * (Le || 0.1), ke = ce.ab2xy(Ee + Qe, xe, !0), Me = ce.ab2xy(Ee - Qe, xe, !0);
                return [0.5 * (ke[0] - Me[0]) / Qe, 0.5 * (ke[1] - Me[1]) / Qe];
              }, ce.dxydb_rough = function(Ee, xe, Le) {
                var Qe = Ce * (Le || 0.1), ke = ce.ab2xy(Ee, xe + Qe, !0), Me = ce.ab2xy(Ee, xe - Qe, !0);
                return [0.5 * (ke[0] - Me[0]) / Qe, 0.5 * (ke[1] - Me[1]) / Qe];
              }, ce.dpdx = function(Ee) {
                return Ee._m;
              }, ce.dpdy = function(Ee) {
                return Ee._m;
              };
            };
          }, 13007: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee, ae) {
              var oe, se, ce, le = [], ue = [], de = ne[0].length, fe = ne.length;
              function he(Ge, ze) {
                var We, at = 0, ct = 0;
                return Ge > 0 && (We = ne[ze][Ge - 1]) !== void 0 && (ct++, at += We), Ge < de - 1 && (We = ne[ze][Ge + 1]) !== void 0 && (ct++, at += We), ze > 0 && (We = ne[ze - 1][Ge]) !== void 0 && (ct++, at += We), ze < fe - 1 && (We = ne[ze + 1][Ge]) !== void 0 && (ct++, at += We), at / Math.max(1, ct);
              }
              var me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le = 0;
              for (oe = 0; oe < de; oe++) for (se = 0; se < fe; se++) ne[se][oe] === void 0 && (le.push(oe), ue.push(se), ne[se][oe] = he(oe, se)), Le = Math.max(Le, Math.abs(ne[se][oe]));
              if (!le.length) return ne;
              var Qe = 0, ke = 0, Me = le.length;
              do {
                for (Qe = 0, ce = 0; ce < Me; ce++) {
                  oe = le[ce], se = ue[ce];
                  var Re, De, Pe, $e, qe, Xe, Ve = 0, Ze = 0;
                  oe === 0 ? (Pe = ee[qe = Math.min(de - 1, 2)], $e = ee[1], Re = ne[se][qe], Ze += (De = ne[se][1]) + (De - Re) * (ee[0] - $e) / ($e - Pe), Ve++) : oe === de - 1 && (Pe = ee[qe = Math.max(0, de - 3)], $e = ee[de - 2], Re = ne[se][qe], Ze += (De = ne[se][de - 2]) + (De - Re) * (ee[de - 1] - $e) / ($e - Pe), Ve++), (oe === 0 || oe === de - 1) && se > 0 && se < fe - 1 && (me = ae[se + 1] - ae[se], Ze += ((pe = ae[se] - ae[se - 1]) * ne[se + 1][oe] + me * ne[se - 1][oe]) / (pe + me), Ve++), se === 0 ? (Pe = ae[Xe = Math.min(fe - 1, 2)], $e = ae[1], Re = ne[Xe][oe], Ze += (De = ne[1][oe]) + (De - Re) * (ae[0] - $e) / ($e - Pe), Ve++) : se === fe - 1 && (Pe = ae[Xe = Math.max(0, fe - 3)], $e = ae[fe - 2], Re = ne[Xe][oe], Ze += (De = ne[fe - 2][oe]) + (De - Re) * (ae[fe - 1] - $e) / ($e - Pe), Ve++), (se === 0 || se === fe - 1) && oe > 0 && oe < de - 1 && (me = ee[oe + 1] - ee[oe], Ze += ((pe = ee[oe] - ee[oe - 1]) * ne[se][oe + 1] + me * ne[se][oe - 1]) / (pe + me), Ve++), Ve ? Ze /= Ve : (ye = ee[oe + 1] - ee[oe], ge = ee[oe] - ee[oe - 1], Se = (ve = ae[se + 1] - ae[se]) * (be = ae[se] - ae[se - 1]) * (ve + be), Ze = ((Ce = ye * ge * (ye + ge)) * (be * ne[se + 1][oe] + ve * ne[se - 1][oe]) + Se * (ge * ne[se][oe + 1] + ye * ne[se][oe - 1])) / (Se * (ge + ye) + Ce * (be + ve))), Qe += (Ee = (we = Ze - ne[se][oe]) / Le) * Ee, xe = Ve ? 0 : 0.85, ne[se][oe] += we * (1 + xe);
                }
                Qe = Math.sqrt(Qe);
              } while (ke++ < 100 && Qe > 1e-5);
              return te.log("Smoother converged to", Qe, "after", ke, "iterations"), ne;
            };
          }, 10820: function(G, J, Z) {
            var te = Z(34809).isArray1D;
            G.exports = function(ne, ee, ae) {
              var oe = ae("x"), se = oe && oe.length, ce = ae("y"), le = ce && ce.length;
              if (!se && !le) return !1;
              if (ee._cheater = !oe, se && !te(oe) || le && !te(ce)) ee._length = null;
              else {
                var ue = se ? oe.length : 1 / 0;
                le && (ue = Math.min(ue, ce.length)), ee.a && ee.a.length && (ue = Math.min(ue, ee.a.length)), ee.b && ee.b.length && (ue = Math.min(ue, ee.b.length)), ee._length = ue;
              }
              return !0;
            };
          }, 92802: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(6893), ee = Z(87163), ae = Z(9829), oe = Z(10229).defaultLine, se = Z(93049).extendFlat, ce = ne.marker.line;
            G.exports = se({ locations: { valType: "data_array", editType: "calc" }, locationmode: ne.locationmode, z: { valType: "data_array", editType: "calc" }, geojson: se({}, ne.geojson, {}), featureidkey: ne.featureidkey, text: se({}, ne.text, {}), hovertext: se({}, ne.hovertext, {}), marker: { line: { color: se({}, ce.color, { dflt: oe }), width: se({}, ce.width, { dflt: 1 }), editType: "calc" }, opacity: { valType: "number", arrayOk: !0, min: 0, max: 1, dflt: 1, editType: "style" }, editType: "calc" }, selected: { marker: { opacity: ne.selected.marker.opacity, editType: "plot" }, editType: "plot" }, unselected: { marker: { opacity: ne.unselected.marker.opacity, editType: "plot" }, editType: "plot" }, hoverinfo: se({}, ae.hoverinfo, { editType: "calc", flags: ["location", "z", "text", "name"] }), hovertemplate: te(), showlegend: se({}, ae.showlegend, { dflt: !1 }) }, ee("", { cLetter: "z", editTypeOverride: "calc" }));
          }, 12702: function(G, J, Z) {
            var te = Z(10721), ne = Z(63821).BADNUM, ee = Z(28379), ae = Z(99203), oe = Z(48861);
            function se(ce) {
              return ce && typeof ce == "string";
            }
            G.exports = function(ce, le) {
              var ue, de = le._length, fe = new Array(de);
              ue = le.geojson ? function(ge) {
                return se(ge) || te(ge);
              } : se;
              for (var he = 0; he < de; he++) {
                var me = fe[he] = {}, pe = le.locations[he], ye = le.z[he];
                ue(pe) && te(ye) ? (me.loc = pe, me.z = ye) : (me.loc = null, me.z = ne), me.index = he;
              }
              return ae(fe, le), ee(ce, le, { vals: le.z, containerStr: "", cLetter: "z" }), oe(fe, le), fe;
            };
          }, 51893: function(G, J, Z) {
            var te = Z(34809), ne = Z(39356), ee = Z(92802);
            G.exports = function(ae, oe, se, ce) {
              function le(me, pe) {
                return te.coerce(ae, oe, ee, me, pe);
              }
              var ue = le("locations"), de = le("z");
              if (ue && ue.length && te.isArrayOrTypedArray(de) && de.length) {
                oe._length = Math.min(ue.length, de.length);
                var fe, he = le("geojson");
                (typeof he == "string" && he !== "" || te.isPlainObject(he)) && (fe = "geojson-id"), le("locationmode", fe) === "geojson-id" && le("featureidkey"), le("text"), le("hovertext"), le("hovertemplate"), le("marker.line.width") && le("marker.line.color"), le("marker.opacity"), ne(ae, oe, ce, le, { prefix: "", cLetter: "z" }), te.coerceSelectionMarkerOpacity(oe, le);
              } else oe.visible = !1;
            };
          }, 38414: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              J.location = Z.location, J.z = Z.z;
              var ae = ne[ee];
              return ae.fIn && ae.fIn.properties && (J.properties = ae.fIn.properties), J.ct = ae.ct, J;
            };
          }, 94125: function(G, J, Z) {
            var te = Z(29714), ne = Z(92802), ee = Z(34809).fillText;
            G.exports = function(ae, oe, se) {
              var ce, le, ue, de, fe = ae.cd, he = fe[0].trace, me = ae.subplot, pe = [oe, se], ye = [oe + 360, se];
              for (le = 0; le < fe.length; le++) if (de = !1, (ce = fe[le])._polygons) {
                for (ue = 0; ue < ce._polygons.length; ue++) ce._polygons[ue].contains(pe) && (de = !de), ce._polygons[ue].contains(ye) && (de = !de);
                if (de) break;
              }
              if (de && ce) return ae.x0 = ae.x1 = ae.xa.c2p(ce.ct), ae.y0 = ae.y1 = ae.ya.c2p(ce.ct), ae.index = ce.index, ae.location = ce.loc, ae.z = ce.z, ae.zLabel = te.tickText(me.mockAxis, me.mockAxis.c2l(ce.z), "hover").text, ae.hovertemplate = ce.hovertemplate, function(ge, ve, be) {
                if (!ve.hovertemplate) {
                  var Ce = be.hi || ve.hoverinfo, Se = String(be.loc), we = Ce === "all" ? ne.hoverinfo.flags : Ce.split("+"), Ee = we.indexOf("name") !== -1, xe = we.indexOf("location") !== -1, Le = we.indexOf("z") !== -1, Qe = we.indexOf("text") !== -1, ke = [];
                  !Ee && xe ? ge.nameOverride = Se : (Ee && (ge.nameOverride = ve.name), xe && ke.push(Se)), Le && ke.push(ge.zLabel), Qe && ee(be, ve, ke), ge.extraText = ke.join("<br>");
                }
              }(ae, he, ce), [ae];
            };
          }, 58075: function(G, J, Z) {
            G.exports = { attributes: Z(92802), supplyDefaults: Z(51893), colorbar: Z(12431), calc: Z(12702), calcGeoJSON: Z(4700).calcGeoJSON, plot: Z(4700).plot, style: Z(59342).style, styleOnSelect: Z(59342).styleOnSelect, hoverPoints: Z(94125), eventData: Z(38414), selectPoints: Z(43727), moduleType: "trace", name: "choropleth", basePlotModule: Z(47544), categories: ["geo", "noOpacity", "showLegend"], meta: {} };
          }, 4700: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(3994), ae = Z(11577).getTopojsonFeatures, oe = Z(32919).findExtremes, se = Z(59342).style;
            G.exports = { calcGeoJSON: function(ce, le) {
              for (var ue = ce[0].trace, de = le[ue.geo], fe = de._subplot, he = ue.locationmode, me = ue._length, pe = he === "geojson-id" ? ee.extractTraceFeature(ce) : ae(ue, fe.topojson), ye = [], ge = [], ve = 0; ve < me; ve++) {
                var be = ce[ve], Ce = he === "geojson-id" ? be.fOut : ee.locationToFeature(he, be.loc, pe);
                if (Ce) {
                  be.geojson = Ce, be.ct = Ce.properties.ct, be._polygons = ee.feature2polygons(Ce);
                  var Se = ee.computeBbox(Ce);
                  ye.push(Se[0], Se[2]), ge.push(Se[1], Se[3]);
                } else be.geojson = null;
              }
              if (de.fitbounds === "geojson" && he === "geojson-id") {
                var we = ee.computeBbox(ee.getTraceGeojson(ue));
                ye = [we[0], we[2]], ge = [we[1], we[3]];
              }
              var Ee = { padded: !0 };
              ue._extremes.lon = oe(de.lonaxis._ax, ye, Ee), ue._extremes.lat = oe(de.lataxis._ax, ge, Ee);
            }, plot: function(ce, le, ue) {
              var de = le.layers.backplot.select(".choroplethlayer");
              ne.makeTraceGroups(de, ue, "trace choropleth").each(function(fe) {
                var he = te.select(this).selectAll("path.choroplethlocation").data(ne.identity);
                he.enter().append("path").classed("choroplethlocation", !0), he.exit().remove(), se(ce, fe);
              });
            } };
          }, 43727: function(G) {
            G.exports = function(J, Z) {
              var te, ne, ee, ae, oe, se = J.cd, ce = J.xaxis, le = J.yaxis, ue = [];
              if (Z === !1) for (te = 0; te < se.length; te++) se[te].selected = 0;
              else for (te = 0; te < se.length; te++) (ee = (ne = se[te]).ct) && (ae = ce.c2p(ee), oe = le.c2p(ee), Z.contains([ae, oe], null, te, J) ? (ue.push({ pointNumber: te, lon: ee[0], lat: ee[1] }), ne.selected = 1) : ne.selected = 0);
              return ue;
            };
          }, 59342: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(62203), ae = Z(88856);
            function oe(se, ce) {
              var le = ce[0].trace, ue = ce[0].node3.selectAll(".choroplethlocation"), de = le.marker || {}, fe = de.line || {}, he = ae.makeColorScaleFuncFromTrace(le);
              ue.each(function(me) {
                te.select(this).attr("fill", he(me.z)).call(ne.stroke, me.mlc || fe.color).call(ee.dashLine, "", me.mlw || fe.width || 0).style("opacity", de.opacity);
              }), ee.selectedPointStyle(ue, le);
            }
            G.exports = { style: function(se, ce) {
              ce && oe(0, ce);
            }, styleOnSelect: function(se, ce) {
              var le = ce[0].node3, ue = ce[0].trace;
              ue.selectedpoints ? ee.selectedPointStyle(le.selectAll(".choroplethlocation"), ue) : oe(0, ce);
            } };
          }, 34770: function(G, J, Z) {
            var te = Z(92802), ne = Z(87163), ee = Z(3208).rb, ae = Z(9829), oe = Z(93049).extendFlat;
            G.exports = oe({ locations: { valType: "data_array", editType: "calc" }, z: { valType: "data_array", editType: "calc" }, geojson: { valType: "any", editType: "calc" }, featureidkey: oe({}, te.featureidkey, {}), below: { valType: "string", editType: "plot" }, text: te.text, hovertext: te.hovertext, marker: { line: { color: oe({}, te.marker.line.color, { editType: "plot" }), width: oe({}, te.marker.line.width, { editType: "plot" }), editType: "calc" }, opacity: oe({}, te.marker.opacity, { editType: "plot" }), editType: "calc" }, selected: { marker: { opacity: oe({}, te.selected.marker.opacity, { editType: "plot" }), editType: "plot" }, editType: "plot" }, unselected: { marker: { opacity: oe({}, te.unselected.marker.opacity, { editType: "plot" }), editType: "plot" }, editType: "plot" }, hoverinfo: te.hoverinfo, hovertemplate: ee({}, { keys: ["properties"] }), showlegend: oe({}, ae.showlegend, { dflt: !1 }) }, ne("", { cLetter: "z", editTypeOverride: "calc" }));
          }, 40980: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(88856), ae = Z(62203), oe = Z(39532).makeBlank, se = Z(3994);
            function ce(le) {
              var ue, de = le[0].trace, fe = de._opts;
              if (de.selectedpoints) {
                for (var he = ae.makeSelectedPointStyleFns(de), me = 0; me < le.length; me++) {
                  var pe = le[me];
                  pe.fOut && (pe.fOut.properties.mo2 = he.selectedOpacityFn(pe));
                }
                ue = { type: "identity", property: "mo2" };
              } else ue = ne.isArrayOrTypedArray(de.marker.opacity) ? { type: "identity", property: "mo" } : de.marker.opacity;
              return ne.extendFlat(fe.fill.paint, { "fill-opacity": ue }), ne.extendFlat(fe.line.paint, { "line-opacity": ue }), fe;
            }
            G.exports = { convert: function(le) {
              var ue = le[0].trace, de = ue.visible === !0 && ue._length !== 0, fe = { layout: { visibility: "none" }, paint: {} }, he = { layout: { visibility: "none" }, paint: {} }, me = ue._opts = { fill: fe, line: he, geojson: oe() };
              if (!de) return me;
              var pe = se.extractTraceFeature(le);
              if (!pe) return me;
              var ye, ge, ve, be = ee.makeColorScaleFuncFromTrace(ue), Ce = ue.marker, Se = Ce.line || {};
              ne.isArrayOrTypedArray(Ce.opacity) && (ye = function(ke) {
                var Me = ke.mo;
                return te(Me) ? +ne.constrain(Me, 0, 1) : 0;
              }), ne.isArrayOrTypedArray(Se.color) && (ge = function(ke) {
                return ke.mlc;
              }), ne.isArrayOrTypedArray(Se.width) && (ve = function(ke) {
                return ke.mlw;
              });
              for (var we = 0; we < le.length; we++) {
                var Ee = le[we], xe = Ee.fOut;
                if (xe) {
                  var Le = xe.properties;
                  Le.fc = be(Ee.z), ye && (Le.mo = ye(Ee)), ge && (Le.mlc = ge(Ee)), ve && (Le.mlw = ve(Ee)), Ee.ct = Le.ct, Ee._polygons = se.feature2polygons(xe);
                }
              }
              var Qe = ye ? { type: "identity", property: "mo" } : Ce.opacity;
              return ne.extendFlat(fe.paint, { "fill-color": { type: "identity", property: "fc" }, "fill-opacity": Qe }), ne.extendFlat(he.paint, { "line-color": ge ? { type: "identity", property: "mlc" } : Se.color, "line-width": ve ? { type: "identity", property: "mlw" } : Se.width, "line-opacity": Qe }), fe.layout.visibility = "visible", he.layout.visibility = "visible", me.geojson = { type: "FeatureCollection", features: pe }, ce(le), me;
            }, convertOnSelect: ce };
          }, 94149: function(G, J, Z) {
            var te = Z(34809), ne = Z(39356), ee = Z(34770);
            G.exports = function(ae, oe, se, ce) {
              function le(he, me) {
                return te.coerce(ae, oe, ee, he, me);
              }
              var ue = le("locations"), de = le("z"), fe = le("geojson");
              te.isArrayOrTypedArray(ue) && ue.length && te.isArrayOrTypedArray(de) && de.length && (typeof fe == "string" && fe !== "" || te.isPlainObject(fe)) ? (le("featureidkey"), oe._length = Math.min(ue.length, de.length), le("below"), le("text"), le("hovertext"), le("hovertemplate"), le("marker.line.width") && le("marker.line.color"), le("marker.opacity"), ne(ae, oe, ce, le, { prefix: "", cLetter: "z" }), te.coerceSelectionMarkerOpacity(oe, le)) : oe.visible = !1;
            };
          }, 9419: function(G, J, Z) {
            G.exports = { attributes: Z(34770), supplyDefaults: Z(94149), colorbar: Z(12431), calc: Z(12702), plot: Z(30316), hoverPoints: Z(94125), eventData: Z(38414), selectPoints: Z(43727), styleOnSelect: function(te, ne) {
              ne && ne[0].trace._glTrace.updateOnSelect(ne);
            }, getBelow: function(te, ne) {
              for (var ee = ne.getMapLayers(), ae = ee.length - 2; ae >= 0; ae--) {
                var oe = ee[ae].id;
                if (typeof oe == "string" && oe.indexOf("water") === 0) {
                  for (var se = ae + 1; se < ee.length; se++) if (typeof (oe = ee[se].id) == "string" && oe.indexOf("plotly-") === -1) return oe;
                }
              }
            }, moduleType: "trace", name: "choroplethmap", basePlotModule: Z(34091), categories: ["map", "gl", "noOpacity", "showLegend"], meta: { hr_name: "choropleth_map" } };
          }, 30316: function(G, J, Z) {
            var te = Z(40980).convert, ne = Z(40980).convertOnSelect, ee = Z(8814).traceLayerPrefix;
            function ae(se, ce) {
              this.type = "choroplethmap", this.subplot = se, this.uid = ce, this.sourceId = "source-" + ce, this.layerList = [["fill", ee + ce + "-fill"], ["line", ee + ce + "-line"]], this.below = null;
            }
            var oe = ae.prototype;
            oe.update = function(se) {
              this._update(te(se)), se[0].trace._glTrace = this;
            }, oe.updateOnSelect = function(se) {
              this._update(ne(se));
            }, oe._update = function(se) {
              var ce = this.subplot, le = this.layerList, ue = ce.belowLookup["trace-" + this.uid];
              ce.map.getSource(this.sourceId).setData(se.geojson), ue !== this.below && (this._removeLayers(), this._addLayers(se, ue), this.below = ue);
              for (var de = 0; de < le.length; de++) {
                var fe = le[de], he = fe[0], me = fe[1], pe = se[he];
                ce.setOptions(me, "setLayoutProperty", pe.layout), pe.layout.visibility === "visible" && ce.setOptions(me, "setPaintProperty", pe.paint);
              }
            }, oe._addLayers = function(se, ce) {
              for (var le = this.subplot, ue = this.layerList, de = this.sourceId, fe = 0; fe < ue.length; fe++) {
                var he = ue[fe], me = he[0], pe = se[me];
                le.addLayer({ type: me, id: he[1], source: de, layout: pe.layout, paint: pe.paint }, ce);
              }
            }, oe._removeLayers = function() {
              for (var se = this.subplot.map, ce = this.layerList, le = ce.length - 1; le >= 0; le--) se.removeLayer(ce[le][1]);
            }, oe.dispose = function() {
              var se = this.subplot.map;
              this._removeLayers(), se.removeSource(this.sourceId);
            }, G.exports = function(se, ce) {
              var le = ce[0].trace, ue = new ae(se, le.uid), de = ue.sourceId, fe = te(ce), he = ue.below = se.belowLookup["trace-" + le.uid];
              return se.map.addSource(de, { type: "geojson", data: fe.geojson }), ue._addLayers(fe, he), ce[0].trace._glTrace = ue, ue;
            };
          }, 86227: function(G, J, Z) {
            var te = Z(92802), ne = Z(87163), ee = Z(3208).rb, ae = Z(9829), oe = Z(93049).extendFlat;
            G.exports = oe({ locations: { valType: "data_array", editType: "calc" }, z: { valType: "data_array", editType: "calc" }, geojson: { valType: "any", editType: "calc" }, featureidkey: oe({}, te.featureidkey, {}), below: { valType: "string", editType: "plot" }, text: te.text, hovertext: te.hovertext, marker: { line: { color: oe({}, te.marker.line.color, { editType: "plot" }), width: oe({}, te.marker.line.width, { editType: "plot" }), editType: "calc" }, opacity: oe({}, te.marker.opacity, { editType: "plot" }), editType: "calc" }, selected: { marker: { opacity: oe({}, te.selected.marker.opacity, { editType: "plot" }), editType: "plot" }, editType: "plot" }, unselected: { marker: { opacity: oe({}, te.unselected.marker.opacity, { editType: "plot" }), editType: "plot" }, editType: "plot" }, hoverinfo: te.hoverinfo, hovertemplate: ee({}, { keys: ["properties"] }), showlegend: oe({}, ae.showlegend, { dflt: !1 }) }, ne("", { cLetter: "z", editTypeOverride: "calc" }));
          }, 51335: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(88856), ae = Z(62203), oe = Z(39532).makeBlank, se = Z(3994);
            function ce(le) {
              var ue, de = le[0].trace, fe = de._opts;
              if (de.selectedpoints) {
                for (var he = ae.makeSelectedPointStyleFns(de), me = 0; me < le.length; me++) {
                  var pe = le[me];
                  pe.fOut && (pe.fOut.properties.mo2 = he.selectedOpacityFn(pe));
                }
                ue = { type: "identity", property: "mo2" };
              } else ue = ne.isArrayOrTypedArray(de.marker.opacity) ? { type: "identity", property: "mo" } : de.marker.opacity;
              return ne.extendFlat(fe.fill.paint, { "fill-opacity": ue }), ne.extendFlat(fe.line.paint, { "line-opacity": ue }), fe;
            }
            G.exports = { convert: function(le) {
              var ue = le[0].trace, de = ue.visible === !0 && ue._length !== 0, fe = { layout: { visibility: "none" }, paint: {} }, he = { layout: { visibility: "none" }, paint: {} }, me = ue._opts = { fill: fe, line: he, geojson: oe() };
              if (!de) return me;
              var pe = se.extractTraceFeature(le);
              if (!pe) return me;
              var ye, ge, ve, be = ee.makeColorScaleFuncFromTrace(ue), Ce = ue.marker, Se = Ce.line || {};
              ne.isArrayOrTypedArray(Ce.opacity) && (ye = function(ke) {
                var Me = ke.mo;
                return te(Me) ? +ne.constrain(Me, 0, 1) : 0;
              }), ne.isArrayOrTypedArray(Se.color) && (ge = function(ke) {
                return ke.mlc;
              }), ne.isArrayOrTypedArray(Se.width) && (ve = function(ke) {
                return ke.mlw;
              });
              for (var we = 0; we < le.length; we++) {
                var Ee = le[we], xe = Ee.fOut;
                if (xe) {
                  var Le = xe.properties;
                  Le.fc = be(Ee.z), ye && (Le.mo = ye(Ee)), ge && (Le.mlc = ge(Ee)), ve && (Le.mlw = ve(Ee)), Ee.ct = Le.ct, Ee._polygons = se.feature2polygons(xe);
                }
              }
              var Qe = ye ? { type: "identity", property: "mo" } : Ce.opacity;
              return ne.extendFlat(fe.paint, { "fill-color": { type: "identity", property: "fc" }, "fill-opacity": Qe }), ne.extendFlat(he.paint, { "line-color": ge ? { type: "identity", property: "mlc" } : Se.color, "line-width": ve ? { type: "identity", property: "mlw" } : Se.width, "line-opacity": Qe }), fe.layout.visibility = "visible", he.layout.visibility = "visible", me.geojson = { type: "FeatureCollection", features: pe }, ce(le), me;
            }, convertOnSelect: ce };
          }, 8244: function(G, J, Z) {
            var te = Z(34809), ne = Z(39356), ee = Z(86227);
            G.exports = function(ae, oe, se, ce) {
              function le(he, me) {
                return te.coerce(ae, oe, ee, he, me);
              }
              var ue = le("locations"), de = le("z"), fe = le("geojson");
              te.isArrayOrTypedArray(ue) && ue.length && te.isArrayOrTypedArray(de) && de.length && (typeof fe == "string" && fe !== "" || te.isPlainObject(fe)) ? (le("featureidkey"), oe._length = Math.min(ue.length, de.length), le("below"), le("text"), le("hovertext"), le("hovertemplate"), le("marker.line.width") && le("marker.line.color"), le("marker.opacity"), ne(ae, oe, ce, le, { prefix: "", cLetter: "z" }), te.coerceSelectionMarkerOpacity(oe, le)) : oe.visible = !1;
            };
          }, 28128: function(G, J, Z) {
            G.exports = { attributes: Z(86227), supplyDefaults: Z(8244), colorbar: Z(12431), calc: Z(12702), plot: Z(33501), hoverPoints: Z(94125), eventData: Z(38414), selectPoints: Z(43727), styleOnSelect: function(te, ne) {
              ne && ne[0].trace._glTrace.updateOnSelect(ne);
            }, getBelow: function(te, ne) {
              for (var ee = ne.getMapLayers(), ae = ee.length - 2; ae >= 0; ae--) {
                var oe = ee[ae].id;
                if (typeof oe == "string" && oe.indexOf("water") === 0) {
                  for (var se = ae + 1; se < ee.length; se++) if (typeof (oe = ee[se].id) == "string" && oe.indexOf("plotly-") === -1) return oe;
                }
              }
            }, moduleType: "trace", name: "choroplethmapbox", basePlotModule: Z(68192), categories: ["mapbox", "gl", "noOpacity", "showLegend"], meta: { hr_name: "choropleth_mapbox" } };
          }, 33501: function(G, J, Z) {
            var te = Z(51335).convert, ne = Z(51335).convertOnSelect, ee = Z(44245).traceLayerPrefix;
            function ae(se, ce) {
              this.type = "choroplethmapbox", this.subplot = se, this.uid = ce, this.sourceId = "source-" + ce, this.layerList = [["fill", ee + ce + "-fill"], ["line", ee + ce + "-line"]], this.below = null;
            }
            var oe = ae.prototype;
            oe.update = function(se) {
              this._update(te(se)), se[0].trace._glTrace = this;
            }, oe.updateOnSelect = function(se) {
              this._update(ne(se));
            }, oe._update = function(se) {
              var ce = this.subplot, le = this.layerList, ue = ce.belowLookup["trace-" + this.uid];
              ce.map.getSource(this.sourceId).setData(se.geojson), ue !== this.below && (this._removeLayers(), this._addLayers(se, ue), this.below = ue);
              for (var de = 0; de < le.length; de++) {
                var fe = le[de], he = fe[0], me = fe[1], pe = se[he];
                ce.setOptions(me, "setLayoutProperty", pe.layout), pe.layout.visibility === "visible" && ce.setOptions(me, "setPaintProperty", pe.paint);
              }
            }, oe._addLayers = function(se, ce) {
              for (var le = this.subplot, ue = this.layerList, de = this.sourceId, fe = 0; fe < ue.length; fe++) {
                var he = ue[fe], me = he[0], pe = se[me];
                le.addLayer({ type: me, id: he[1], source: de, layout: pe.layout, paint: pe.paint }, ce);
              }
            }, oe._removeLayers = function() {
              for (var se = this.subplot.map, ce = this.layerList, le = ce.length - 1; le >= 0; le--) se.removeLayer(ce[le][1]);
            }, oe.dispose = function() {
              var se = this.subplot.map;
              this._removeLayers(), se.removeSource(this.sourceId);
            }, G.exports = function(se, ce) {
              var le = ce[0].trace, ue = new ae(se, le.uid), de = ue.sourceId, fe = te(ce), he = ue.below = se.belowLookup["trace-" + le.uid];
              return se.map.addSource(de, { type: "geojson", data: fe.geojson }), ue._addLayers(fe, he), ce[0].trace._glTrace = ue, ue;
            };
          }, 49865: function(G, J, Z) {
            var te = Z(87163), ne = Z(80712).axisHoverFormat, ee = Z(3208).rb, ae = Z(42450), oe = Z(9829), se = Z(93049).extendFlat, ce = { x: { valType: "data_array", editType: "calc+clearAxisTypes" }, y: { valType: "data_array", editType: "calc+clearAxisTypes" }, z: { valType: "data_array", editType: "calc+clearAxisTypes" }, u: { valType: "data_array", editType: "calc" }, v: { valType: "data_array", editType: "calc" }, w: { valType: "data_array", editType: "calc" }, sizemode: { valType: "enumerated", values: ["scaled", "absolute", "raw"], editType: "calc", dflt: "scaled" }, sizeref: { valType: "number", editType: "calc", min: 0 }, anchor: { valType: "enumerated", editType: "calc", values: ["tip", "tail", "cm", "center"], dflt: "cm" }, text: { valType: "string", dflt: "", arrayOk: !0, editType: "calc" }, hovertext: { valType: "string", dflt: "", arrayOk: !0, editType: "calc" }, hovertemplate: ee({ editType: "calc" }, { keys: ["norm"] }), uhoverformat: ne("u", 1), vhoverformat: ne("v", 1), whoverformat: ne("w", 1), xhoverformat: ne("x"), yhoverformat: ne("y"), zhoverformat: ne("z"), showlegend: se({}, oe.showlegend, { dflt: !1 }) };
            se(ce, te("", { colorAttr: "u/v/w norm", showScaleDflt: !0, editTypeOverride: "calc" })), ["opacity", "lightposition", "lighting"].forEach(function(le) {
              ce[le] = ae[le];
            }), ce.hoverinfo = se({}, oe.hoverinfo, { editType: "calc", flags: ["x", "y", "z", "u", "v", "w", "norm", "text", "name"], dflt: "x+y+z+norm+text+name" }), ce.transforms = void 0, G.exports = ce;
          }, 93805: function(G, J, Z) {
            var te = Z(28379);
            G.exports = function(ne, ee) {
              for (var ae = ee.u, oe = ee.v, se = ee.w, ce = Math.min(ee.x.length, ee.y.length, ee.z.length, ae.length, oe.length, se.length), le = -1 / 0, ue = 1 / 0, de = 0; de < ce; de++) {
                var fe = ae[de], he = oe[de], me = se[de], pe = Math.sqrt(fe * fe + he * he + me * me);
                le = Math.max(le, pe), ue = Math.min(ue, pe);
              }
              ee._len = ce, ee._normMax = le, te(ne, ee, { vals: [ue, le], containerStr: "", cLetter: "c" });
            };
          }, 49393: function(G, J, Z) {
            var te = Z(99098).gl_cone3d, ne = Z(99098).gl_cone3d.createConeMesh, ee = Z(34809).simpleMap, ae = Z(46998).parseColorScale, oe = Z(88856).extractOpts, se = Z(34809).isArrayOrTypedArray, ce = Z(88239);
            function le(pe, ye) {
              this.scene = pe, this.uid = ye, this.mesh = null, this.data = null;
            }
            var ue = le.prototype;
            ue.handlePick = function(pe) {
              if (pe.object === this.mesh) {
                var ye = pe.index = pe.data.index, ge = this.data.x[ye], ve = this.data.y[ye], be = this.data.z[ye], Ce = this.data.u[ye], Se = this.data.v[ye], we = this.data.w[ye];
                pe.traceCoordinate = [ge, ve, be, Ce, Se, we, Math.sqrt(Ce * Ce + Se * Se + we * we)];
                var Ee = this.data.hovertext || this.data.text;
                return se(Ee) && Ee[ye] !== void 0 ? pe.textLabel = Ee[ye] : Ee && (pe.textLabel = Ee), !0;
              }
            };
            var de = { xaxis: 0, yaxis: 1, zaxis: 2 }, fe = { tip: 1, tail: 0, cm: 0.25, center: 0.5 }, he = { tip: 1, tail: 1, cm: 0.75, center: 0.5 };
            function me(pe, ye) {
              var ge = pe.fullSceneLayout, ve = pe.dataScale, be = {};
              function Ce(Le, Qe) {
                var ke = ge[Qe], Me = ve[de[Qe]];
                return ee(Le, function(Re) {
                  return ke.d2l(Re) * Me;
                });
              }
              be.vectors = ce(Ce(ye.u, "xaxis"), Ce(ye.v, "yaxis"), Ce(ye.w, "zaxis"), ye._len), be.positions = ce(Ce(ye.x, "xaxis"), Ce(ye.y, "yaxis"), Ce(ye.z, "zaxis"), ye._len);
              var Se = oe(ye);
              be.colormap = ae(ye), be.vertexIntensityBounds = [Se.min / ye._normMax, Se.max / ye._normMax], be.coneOffset = fe[ye.anchor];
              var we = ye.sizemode;
              we === "scaled" ? be.coneSize = ye.sizeref || 0.5 : we === "absolute" ? be.coneSize = ye.sizeref && ye._normMax ? ye.sizeref / ye._normMax : 0.5 : we === "raw" && (be.coneSize = ye.sizeref), be.coneSizemode = we;
              var Ee = te(be), xe = ye.lightposition;
              return Ee.lightPosition = [xe.x, xe.y, xe.z], Ee.ambient = ye.lighting.ambient, Ee.diffuse = ye.lighting.diffuse, Ee.specular = ye.lighting.specular, Ee.roughness = ye.lighting.roughness, Ee.fresnel = ye.lighting.fresnel, Ee.opacity = ye.opacity, ye._pad = he[ye.anchor] * Ee.vectorScale * Ee.coneScale * ye._normMax, Ee;
            }
            ue.update = function(pe) {
              this.data = pe;
              var ye = me(this.scene, pe);
              this.mesh.update(ye);
            }, ue.dispose = function() {
              this.scene.glplot.remove(this.mesh), this.mesh.dispose();
            }, G.exports = function(pe, ye) {
              var ge = pe.glplot.gl, ve = me(pe, ye), be = ne(ge, ve), Ce = new le(pe, ye.uid);
              return Ce.mesh = be, Ce.data = ye, be._trace = Ce, pe.glplot.add(be), Ce;
            };
          }, 17326: function(G, J, Z) {
            var te = Z(34809), ne = Z(39356), ee = Z(49865);
            G.exports = function(ae, oe, se, ce) {
              function le(ge, ve) {
                return te.coerce(ae, oe, ee, ge, ve);
              }
              var ue = le("u"), de = le("v"), fe = le("w"), he = le("x"), me = le("y"), pe = le("z");
              if (ue && ue.length && de && de.length && fe && fe.length && he && he.length && me && me.length && pe && pe.length) {
                var ye = le("sizemode");
                le("sizeref", ye === "raw" ? 1 : 0.5), le("anchor"), le("lighting.ambient"), le("lighting.diffuse"), le("lighting.specular"), le("lighting.roughness"), le("lighting.fresnel"), le("lightposition.x"), le("lightposition.y"), le("lightposition.z"), ne(ae, oe, ce, le, { prefix: "", cLetter: "c" }), le("text"), le("hovertext"), le("hovertemplate"), le("uhoverformat"), le("vhoverformat"), le("whoverformat"), le("xhoverformat"), le("yhoverformat"), le("zhoverformat"), oe._length = null;
              } else oe.visible = !1;
            };
          }, 47050: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "cone", basePlotModule: Z(2487), categories: ["gl3d", "showLegend"], attributes: Z(49865), supplyDefaults: Z(17326), colorbar: { min: "cmin", max: "cmax" }, calc: Z(93805), plot: Z(49393), eventData: function(te, ne) {
              return te.norm = ne.traceCoordinate[6], te;
            }, meta: {} };
          }, 52240: function(G, J, Z) {
            var te = Z(81658), ne = Z(36640), ee = Z(80712), ae = ee.axisHoverFormat, oe = ee.descriptionOnlyNumbers, se = Z(87163), ce = Z(94850).T, le = Z(80337), ue = Z(93049).extendFlat, de = Z(20726), fe = de.COMPARISON_OPS2, he = de.INTERVAL_OPS, me = ne.line;
            G.exports = ue({ z: te.z, x: te.x, x0: te.x0, dx: te.dx, y: te.y, y0: te.y0, dy: te.dy, xperiod: te.xperiod, yperiod: te.yperiod, xperiod0: ne.xperiod0, yperiod0: ne.yperiod0, xperiodalignment: te.xperiodalignment, yperiodalignment: te.yperiodalignment, text: te.text, hovertext: te.hovertext, transpose: te.transpose, xtype: te.xtype, ytype: te.ytype, xhoverformat: ae("x"), yhoverformat: ae("y"), zhoverformat: ae("z", 1), hovertemplate: te.hovertemplate, texttemplate: ue({}, te.texttemplate, {}), textfont: ue({}, te.textfont, {}), hoverongaps: te.hoverongaps, connectgaps: ue({}, te.connectgaps, {}), fillcolor: { valType: "color", editType: "calc" }, autocontour: { valType: "boolean", dflt: !0, editType: "calc", impliedEdits: { "contours.start": void 0, "contours.end": void 0, "contours.size": void 0 } }, ncontours: { valType: "integer", dflt: 15, min: 1, editType: "calc" }, contours: { type: { valType: "enumerated", values: ["levels", "constraint"], dflt: "levels", editType: "calc" }, start: { valType: "number", dflt: null, editType: "plot", impliedEdits: { "^autocontour": !1 } }, end: { valType: "number", dflt: null, editType: "plot", impliedEdits: { "^autocontour": !1 } }, size: { valType: "number", dflt: null, min: 0, editType: "plot", impliedEdits: { "^autocontour": !1 } }, coloring: { valType: "enumerated", values: ["fill", "heatmap", "lines", "none"], dflt: "fill", editType: "calc" }, showlines: { valType: "boolean", dflt: !0, editType: "plot" }, showlabels: { valType: "boolean", dflt: !1, editType: "plot" }, labelfont: le({ editType: "plot", colorEditType: "style" }), labelformat: { valType: "string", dflt: "", editType: "plot", description: oe("contour label") }, operation: { valType: "enumerated", values: [].concat(fe).concat(he), dflt: "=", editType: "calc" }, value: { valType: "any", dflt: 0, editType: "calc" }, editType: "calc", impliedEdits: { autocontour: !1 } }, line: { color: ue({}, me.color, { editType: "style+colorbars" }), width: { valType: "number", min: 0, editType: "style+colorbars" }, dash: ce, smoothing: ue({}, me.smoothing, {}), editType: "plot" }, zorder: ne.zorder }, se("", { cLetter: "z", autoColorDflt: !1, editTypeOverride: "calc" }));
          }, 40352: function(G, J, Z) {
            var te = Z(88856), ne = Z(51670), ee = Z(62475), ae = Z(48715);
            G.exports = function(oe, se) {
              var ce = ne(oe, se), le = ce[0].z;
              ee(se, le);
              var ue, de = se.contours, fe = te.extractOpts(se);
              if (de.coloring === "heatmap" && fe.auto && se.autocontour === !1) {
                var he = de.start, me = ae(de), pe = de.size || 1, ye = Math.floor((me - he) / pe) + 1;
                isFinite(pe) || (pe = 1, ye = 1);
                var ge = he - pe / 2;
                ue = [ge, ge + ye * pe];
              } else ue = le;
              return te.calc(oe, se, { vals: ue, cLetter: "z" }), ce;
            };
          }, 49886: function(G) {
            G.exports = function(J, Z) {
              var te, ne = J[0], ee = ne.z;
              switch (Z.type) {
                case "levels":
                  var ae = Math.min(ee[0][0], ee[0][1]);
                  for (te = 0; te < J.length; te++) {
                    var oe = J[te];
                    oe.prefixBoundary = !oe.edgepaths.length && (ae > oe.level || oe.starts.length && ae === oe.level);
                  }
                  break;
                case "constraint":
                  if (ne.prefixBoundary = !1, ne.edgepaths.length) return;
                  var se = ne.x.length, ce = ne.y.length, le = -1 / 0, ue = 1 / 0;
                  for (te = 0; te < ce; te++) ue = Math.min(ue, ee[te][0]), ue = Math.min(ue, ee[te][se - 1]), le = Math.max(le, ee[te][0]), le = Math.max(le, ee[te][se - 1]);
                  for (te = 1; te < se - 1; te++) ue = Math.min(ue, ee[0][te]), ue = Math.min(ue, ee[ce - 1][te]), le = Math.max(le, ee[0][te]), le = Math.max(le, ee[ce - 1][te]);
                  var de, fe, he = Z.value;
                  switch (Z._operation) {
                    case ">":
                      he > le && (ne.prefixBoundary = !0);
                      break;
                    case "<":
                      (he < ue || ne.starts.length && he === ue) && (ne.prefixBoundary = !0);
                      break;
                    case "[]":
                      de = Math.min(he[0], he[1]), ((fe = Math.max(he[0], he[1])) < ue || de > le || ne.starts.length && fe === ue) && (ne.prefixBoundary = !0);
                      break;
                    case "][":
                      de = Math.min(he[0], he[1]), fe = Math.max(he[0], he[1]), de < ue && fe > le && (ne.prefixBoundary = !0);
                  }
              }
            };
          }, 92697: function(G, J, Z) {
            var te = Z(88856), ne = Z(16438), ee = Z(48715);
            G.exports = { min: "zmin", max: "zmax", calc: function(ae, oe, se) {
              var ce = oe.contours, le = oe.line, ue = ce.size || 1, de = ce.coloring, fe = ne(oe, { isColorbar: !0 });
              if (de === "heatmap") {
                var he = te.extractOpts(oe);
                se._fillgradient = he.reversescale ? te.flipScale(he.colorscale) : he.colorscale, se._zrange = [he.min, he.max];
              } else de === "fill" && (se._fillcolor = fe);
              se._line = { color: de === "lines" ? fe : le.color, width: ce.showlines !== !1 ? le.width : 0, dash: le.dash }, se._levels = { start: ce.start, end: ee(ce), size: ue };
            } };
          }, 53156: function(G) {
            G.exports = { BOTTOMSTART: [1, 9, 13, 104, 713], TOPSTART: [4, 6, 7, 104, 713], LEFTSTART: [8, 12, 14, 208, 1114], RIGHTSTART: [2, 3, 11, 208, 1114], NEWDELTA: [null, [-1, 0], [0, -1], [-1, 0], [1, 0], null, [0, -1], [-1, 0], [0, 1], [0, 1], null, [0, 1], [1, 0], [1, 0], [0, -1]], CHOOSESADDLE: { 104: [4, 1], 208: [2, 8], 713: [7, 13], 1114: [11, 14] }, SADDLEREMAINDER: { 1: 4, 2: 8, 4: 1, 7: 13, 8: 2, 11: 14, 13: 7, 14: 11 }, LABELDISTANCE: 2, LABELINCREASE: 10, LABELMIN: 3, LABELMAX: 10, LABELOPTIMIZER: { EDGECOST: 1, ANGLECOST: 1, NEIGHBORCOST: 5, SAMELEVELFACTOR: 10, SAMELEVELDISTANCE: 5, MAXCOST: 100, INITIALSEARCHPOINTS: 10, ITERATIONS: 5 } };
          }, 29503: function(G, J, Z) {
            var te = Z(10721), ne = Z(20576), ee = Z(78766), ae = ee.addOpacity, oe = ee.opacity, se = Z(20726), ce = Z(34809).isArrayOrTypedArray, le = se.CONSTRAINT_REDUCTION, ue = se.COMPARISON_OPS2;
            G.exports = function(de, fe, he, me, pe, ye) {
              var ge, ve, be, Ce = fe.contours, Se = he("contours.operation");
              Ce._operation = le[Se], function(we, Ee) {
                var xe;
                ue.indexOf(Ee.operation) === -1 ? (we("contours.value", [0, 1]), ce(Ee.value) ? Ee.value.length > 2 ? Ee.value = Ee.value.slice(2) : Ee.length === 0 ? Ee.value = [0, 1] : Ee.length < 2 ? (xe = parseFloat(Ee.value[0]), Ee.value = [xe, xe + 1]) : Ee.value = [parseFloat(Ee.value[0]), parseFloat(Ee.value[1])] : te(Ee.value) && (xe = parseFloat(Ee.value), Ee.value = [xe, xe + 1])) : (we("contours.value", 0), te(Ee.value) || (ce(Ee.value) ? Ee.value = parseFloat(Ee.value[0]) : Ee.value = 0));
              }(he, Ce), Se === "=" ? ge = Ce.showlines = !0 : (ge = he("contours.showlines"), be = he("fillcolor", ae((de.line || {}).color || pe, 0.5))), ge && (ve = he("line.color", be && oe(be) ? ae(fe.fillcolor, 1) : pe), he("line.width", 2), he("line.dash")), he("line.smoothing"), ne(he, me, ve, ye);
            };
          }, 22783: function(G, J, Z) {
            var te = Z(20726), ne = Z(10721);
            function ee(se, ce) {
              var le, ue = Array.isArray(ce);
              function de(fe) {
                return ne(fe) ? +fe : null;
              }
              return te.COMPARISON_OPS2.indexOf(se) !== -1 ? le = de(ue ? ce[0] : ce) : te.INTERVAL_OPS.indexOf(se) !== -1 ? le = ue ? [de(ce[0]), de(ce[1])] : [de(ce), de(ce)] : te.SET_OPS.indexOf(se) !== -1 && (le = ue ? ce.map(de) : [de(ce)]), le;
            }
            function ae(se) {
              return function(ce) {
                ce = ee(se, ce);
                var le = Math.min(ce[0], ce[1]), ue = Math.max(ce[0], ce[1]);
                return { start: le, end: ue, size: ue - le };
              };
            }
            function oe(se) {
              return function(ce) {
                return { start: ce = ee(se, ce), end: 1 / 0, size: 1 / 0 };
              };
            }
            G.exports = { "[]": ae("[]"), "][": ae("]["), ">": oe(">"), "<": oe("<"), "=": oe("=") };
          }, 47495: function(G) {
            G.exports = function(J, Z, te, ne) {
              var ee = ne("contours.start"), ae = ne("contours.end"), oe = ee === !1 || ae === !1, se = te("contours.size");
              !(oe ? Z.autocontour = !0 : te("autocontour", !1)) && se || te("ncontours");
            };
          }, 1999: function(G, J, Z) {
            var te = Z(34809);
            function ne(ee) {
              return te.extendFlat({}, ee, { edgepaths: te.extendDeep([], ee.edgepaths), paths: te.extendDeep([], ee.paths), starts: te.extendDeep([], ee.starts) });
            }
            G.exports = function(ee, ae) {
              var oe, se, ce, le = function(fe) {
                return fe.reverse();
              }, ue = function(fe) {
                return fe;
              };
              switch (ae) {
                case "=":
                case "<":
                  return ee;
                case ">":
                  for (ee.length !== 1 && te.warn("Contour data invalid for the specified inequality operation."), se = ee[0], oe = 0; oe < se.edgepaths.length; oe++) se.edgepaths[oe] = le(se.edgepaths[oe]);
                  for (oe = 0; oe < se.paths.length; oe++) se.paths[oe] = le(se.paths[oe]);
                  for (oe = 0; oe < se.starts.length; oe++) se.starts[oe] = le(se.starts[oe]);
                  return ee;
                case "][":
                  var de = le;
                  le = ue, ue = de;
                case "[]":
                  for (ee.length !== 2 && te.warn("Contour data invalid for the specified inequality range operation."), se = ne(ee[0]), ce = ne(ee[1]), oe = 0; oe < se.edgepaths.length; oe++) se.edgepaths[oe] = le(se.edgepaths[oe]);
                  for (oe = 0; oe < se.paths.length; oe++) se.paths[oe] = le(se.paths[oe]);
                  for (oe = 0; oe < se.starts.length; oe++) se.starts[oe] = le(se.starts[oe]);
                  for (; ce.edgepaths.length; ) se.edgepaths.push(ue(ce.edgepaths.shift()));
                  for (; ce.paths.length; ) se.paths.push(ue(ce.paths.shift()));
                  for (; ce.starts.length; ) se.starts.push(ue(ce.starts.shift()));
                  return [se];
              }
            };
          }, 57543: function(G, J, Z) {
            var te = Z(34809), ne = Z(86073), ee = Z(99669), ae = Z(29503), oe = Z(47495), se = Z(39889), ce = Z(63814), le = Z(52240);
            G.exports = function(ue, de, fe, he) {
              function me(ye, ge) {
                return te.coerce(ue, de, le, ye, ge);
              }
              if (ne(ue, de, me, he)) {
                ee(ue, de, he, me), me("xhoverformat"), me("yhoverformat"), me("text"), me("hovertext"), me("hoverongaps"), me("hovertemplate");
                var pe = me("contours.type") === "constraint";
                me("connectgaps", te.isArray1D(de.z)), pe ? ae(ue, de, me, he, fe) : (oe(ue, de, me, function(ye) {
                  return te.coerce2(ue, de, le, ye);
                }), se(ue, de, me, he)), de.contours && de.contours.coloring === "heatmap" && ce(me, he), me("zorder");
              } else de.visible = !1;
            };
          }, 86828: function(G, J, Z) {
            var te = Z(34809), ne = Z(22783), ee = Z(48715);
            G.exports = function(ae, oe, se) {
              for (var ce = ae.type === "constraint" ? ne[ae._operation](ae.value) : ae, le = ce.size, ue = [], de = ee(ce), fe = se.trace._carpetTrace, he = fe ? { xaxis: fe.aaxis, yaxis: fe.baxis, x: se.a, y: se.b } : { xaxis: oe.xaxis, yaxis: oe.yaxis, x: se.x, y: se.y }, me = ce.start; me < de; me += le) if (ue.push(te.extendFlat({ level: me, crossings: {}, starts: [], edgepaths: [], paths: [], z: se.z, smoothing: se.trace.line.smoothing }, he)), ue.length > 1e3) {
                te.warn("Too many contours, clipping at 1000", ae);
                break;
              }
              return ue;
            };
          }, 48715: function(G) {
            G.exports = function(J) {
              return J.end + J.size / 1e6;
            };
          }, 27657: function(G, J, Z) {
            var te = Z(34809), ne = Z(53156);
            function ee(se, ce, le, ue) {
              return Math.abs(se[0] - ce[0]) < le && Math.abs(se[1] - ce[1]) < ue;
            }
            function ae(se, ce, le, ue, de) {
              var fe, he = ce.join(","), me = se.crossings[he], pe = function(st, gt, bt) {
                var yt = 0, St = 0;
                return st > 20 && gt ? st === 208 || st === 1114 ? yt = bt[0] === 0 ? 1 : -1 : St = bt[1] === 0 ? 1 : -1 : ne.BOTTOMSTART.indexOf(st) !== -1 ? St = 1 : ne.LEFTSTART.indexOf(st) !== -1 ? yt = 1 : ne.TOPSTART.indexOf(st) !== -1 ? St = -1 : yt = -1, [yt, St];
              }(me, le, ce), ye = [oe(se, ce, [-pe[0], -pe[1]])], ge = se.z.length, ve = se.z[0].length, be = ce.slice(), Ce = pe.slice();
              for (fe = 0; fe < 1e4; fe++) {
                if (me > 20 ? (me = ne.CHOOSESADDLE[me][(pe[0] || pe[1]) < 0 ? 0 : 1], se.crossings[he] = ne.SADDLEREMAINDER[me]) : delete se.crossings[he], !(pe = ne.NEWDELTA[me])) {
                  te.log("Found bad marching index:", me, ce, se.level);
                  break;
                }
                ye.push(oe(se, ce, pe)), ce[0] += pe[0], ce[1] += pe[1], he = ce.join(","), ee(ye[ye.length - 1], ye[ye.length - 2], ue, de) && ye.pop();
                var Se = pe[0] && (ce[0] < 0 || ce[0] > ve - 2) || pe[1] && (ce[1] < 0 || ce[1] > ge - 2);
                if (ce[0] === be[0] && ce[1] === be[1] && pe[0] === Ce[0] && pe[1] === Ce[1] || le && Se) break;
                me = se.crossings[he];
              }
              fe === 1e4 && te.log("Infinite loop in contour?");
              var we, Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze, Ge = ee(ye[0], ye[ye.length - 1], ue, de), ze = 0, We = 0.2 * se.smoothing, at = [], ct = 0;
              for (fe = 1; fe < ye.length; fe++) qe = ye[fe], Xe = ye[fe - 1], Ve = qe[2] - Xe[2], Ze = qe[3] - Xe[3], ze += Me = Math.sqrt(Ve * Ve + Ze * Ze), at.push(Me);
              var et = ze / at.length * We;
              function dt(st) {
                return ye[st % ye.length];
              }
              for (fe = ye.length - 2; fe >= ct; fe--) if ((we = at[fe]) < et) {
                for (xe = 0, Ee = fe - 1; Ee >= ct && we + at[Ee] < et; Ee--) we += at[Ee];
                if (Ge && fe === ye.length - 2) for (xe = 0; xe < Ee && we + at[xe] < et; xe++) we += at[xe];
                Qe = fe - Ee + xe + 1, ke = Math.floor((fe + Ee + xe + 2) / 2), Le = Ge || fe !== ye.length - 2 ? Ge || Ee !== -1 ? Qe % 2 ? dt(ke) : [(dt(ke)[0] + dt(ke + 1)[0]) / 2, (dt(ke)[1] + dt(ke + 1)[1]) / 2] : ye[0] : ye[ye.length - 1], ye.splice(Ee + 1, fe - Ee + 1, Le), fe = Ee + 1, xe && (ct = xe), Ge && (fe === ye.length - 2 ? ye[xe] = ye[ye.length - 1] : fe === 0 && (ye[ye.length - 1] = ye[0]));
              }
              for (ye.splice(0, ct), fe = 0; fe < ye.length; fe++) ye[fe].length = 2;
              if (!(ye.length < 2)) if (Ge) ye.pop(), se.paths.push(ye);
              else {
                le || te.log("Unclosed interior contour?", se.level, be.join(","), ye.join("L"));
                var Je = !1;
                for (Re = 0; Re < se.edgepaths.length; Re++) if (Pe = se.edgepaths[Re], !Je && ee(Pe[0], ye[ye.length - 1], ue, de)) {
                  ye.pop(), Je = !0;
                  var it = !1;
                  for (De = 0; De < se.edgepaths.length; De++) if (ee(($e = se.edgepaths[De])[$e.length - 1], ye[0], ue, de)) {
                    it = !0, ye.shift(), se.edgepaths.splice(Re, 1), De === Re ? se.paths.push(ye.concat($e)) : (De > Re && De--, se.edgepaths[De] = $e.concat(ye, Pe));
                    break;
                  }
                  it || (se.edgepaths[Re] = ye.concat(Pe));
                }
                for (Re = 0; Re < se.edgepaths.length && !Je; Re++) ee((Pe = se.edgepaths[Re])[Pe.length - 1], ye[0], ue, de) && (ye.shift(), se.edgepaths[Re] = Pe.concat(ye), Je = !0);
                Je || se.edgepaths.push(ye);
              }
            }
            function oe(se, ce, le) {
              var ue = ce[0] + Math.max(le[0], 0), de = ce[1] + Math.max(le[1], 0), fe = se.z[de][ue], he = se.xaxis, me = se.yaxis;
              if (le[1]) {
                var pe = (se.level - fe) / (se.z[de][ue + 1] - fe), ye = (pe !== 1 ? (1 - pe) * he.c2l(se.x[ue]) : 0) + (pe !== 0 ? pe * he.c2l(se.x[ue + 1]) : 0);
                return [he.c2p(he.l2c(ye), !0), me.c2p(se.y[de], !0), ue + pe, de];
              }
              var ge = (se.level - fe) / (se.z[de + 1][ue] - fe), ve = (ge !== 1 ? (1 - ge) * me.c2l(se.y[de]) : 0) + (ge !== 0 ? ge * me.c2l(se.y[de + 1]) : 0);
              return [he.c2p(se.x[ue], !0), me.c2p(me.l2c(ve), !0), ue, de + ge];
            }
            G.exports = function(se, ce, le) {
              var ue, de, fe, he;
              for (ce = ce || 0.01, le = le || 0.01, de = 0; de < se.length; de++) {
                for (fe = se[de], he = 0; he < fe.starts.length; he++) ae(fe, fe.starts[he], "edge", ce, le);
                for (ue = 0; Object.keys(fe.crossings).length && ue < 1e4; ) ue++, ae(fe, Object.keys(fe.crossings)[0].split(",").map(Number), void 0, ce, le);
                ue === 1e4 && te.log("Infinite loop in contour?");
              }
            };
          }, 29815: function(G, J, Z) {
            var te = Z(78766), ne = Z(93125);
            G.exports = function(ee, ae, oe, se, ce) {
              ce || (ce = {}), ce.isContour = !0;
              var le = ne(ee, ae, oe, se, ce);
              return le && le.forEach(function(ue) {
                var de = ue.trace;
                de.contours.type === "constraint" && (de.fillcolor && te.opacity(de.fillcolor) ? ue.color = te.addOpacity(de.fillcolor, 1) : de.contours.showlines && te.opacity(de.line.color) && (ue.color = te.addOpacity(de.line.color, 1)));
              }), le;
            };
          }, 91405: function(G, J, Z) {
            G.exports = { attributes: Z(52240), supplyDefaults: Z(57543), calc: Z(40352), plot: Z(8850).plot, style: Z(1328), colorbar: Z(92697), hoverPoints: Z(29815), moduleType: "trace", name: "contour", basePlotModule: Z(37703), categories: ["cartesian", "svg", "2dMap", "contour", "showLegend"], meta: {} };
          }, 20576: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee, ae, oe) {
              if (oe || (oe = {}), ne("contours.showlabels")) {
                var se = ee.font;
                te.coerceFont(ne, "contours.labelfont", se, { overrideDflt: { color: ae } }), ne("contours.labelformat");
              }
              oe.hasHover !== !1 && ne("zhoverformat");
            };
          }, 16438: function(G, J, Z) {
            var te = Z(45568), ne = Z(88856), ee = Z(48715);
            G.exports = function(ae) {
              var oe = ae.contours, se = oe.start, ce = ee(oe), le = oe.size || 1, ue = Math.floor((ce - se) / le) + 1, de = oe.coloring === "lines" ? 0 : 1, fe = ne.extractOpts(ae);
              isFinite(le) || (le = 1, ue = 1);
              var he, me, pe = fe.reversescale ? ne.flipScale(fe.colorscale) : fe.colorscale, ye = pe.length, ge = new Array(ye), ve = new Array(ye), be = fe.min, Ce = fe.max;
              if (oe.coloring === "heatmap") {
                for (me = 0; me < ye; me++) he = pe[me], ge[me] = he[0] * (Ce - be) + be, ve[me] = he[1];
                var Se = te.extent([be, Ce, oe.start, oe.start + le * (ue - 1)]), we = Se[be < Ce ? 0 : 1], Ee = Se[be < Ce ? 1 : 0];
                we !== be && (ge.splice(0, 0, we), ve.splice(0, 0, ve[0])), Ee !== Ce && (ge.push(Ee), ve.push(ve[ve.length - 1]));
              } else {
                var xe = ae._input && typeof ae._input.zmin == "number" && typeof ae._input.zmax == "number";
                for (xe && (se <= be || ce >= Ce) && (se <= be && (se = be), ce >= Ce && (ce = Ce), ue = Math.floor((ce - se) / le) + 1, de = 0), me = 0; me < ye; me++) he = pe[me], ge[me] = (he[0] * (ue + de - 1) - de / 2) * le + se, ve[me] = he[1];
                (xe || ae.autocontour) && (ge[0] > be && (ge.unshift(be), ve.unshift(ve[0])), ge[ge.length - 1] < Ce && (ge.push(Ce), ve.push(ve[ve.length - 1])));
              }
              return ne.makeColorScaleFunc({ domain: ge, range: ve }, { noNumericCheck: !0 });
            };
          }, 83545: function(G, J, Z) {
            var te = Z(53156);
            function ne(ee, ae) {
              var oe = (ae[0][0] > ee ? 0 : 1) + (ae[0][1] > ee ? 0 : 2) + (ae[1][1] > ee ? 0 : 4) + (ae[1][0] > ee ? 0 : 8);
              return oe === 5 || oe === 10 ? ee > (ae[0][0] + ae[0][1] + ae[1][0] + ae[1][1]) / 4 ? oe === 5 ? 713 : 1114 : oe === 5 ? 104 : 208 : oe === 15 ? 0 : oe;
            }
            G.exports = function(ee) {
              var ae, oe, se, ce, le, ue, de, fe, he, me = ee[0].z, pe = me.length, ye = me[0].length, ge = pe === 2 || ye === 2;
              for (oe = 0; oe < pe - 1; oe++) for (ce = [], oe === 0 && (ce = ce.concat(te.BOTTOMSTART)), oe === pe - 2 && (ce = ce.concat(te.TOPSTART)), ae = 0; ae < ye - 1; ae++) for (se = ce.slice(), ae === 0 && (se = se.concat(te.LEFTSTART)), ae === ye - 2 && (se = se.concat(te.RIGHTSTART)), le = ae + "," + oe, ue = [[me[oe][ae], me[oe][ae + 1]], [me[oe + 1][ae], me[oe + 1][ae + 1]]], he = 0; he < ee.length; he++) (de = ne((fe = ee[he]).level, ue)) && (fe.crossings[le] = de, se.indexOf(de) !== -1 && (fe.starts.push([ae, oe]), ge && se.indexOf(de, se.indexOf(de) + 1) !== -1 && fe.starts.push([ae, oe])));
            };
          }, 8850: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(88856), oe = Z(30635), se = Z(29714), ce = Z(19091), le = Z(19236), ue = Z(83545), de = Z(27657), fe = Z(86828), he = Z(1999), me = Z(49886), pe = Z(53156), ye = pe.LABELOPTIMIZER;
            function ge(Ce, Se) {
              var we, Ee, xe, Le, Qe, ke, Me, Re = "", De = 0, Pe = Ce.edgepaths.map(function(Ge, ze) {
                return ze;
              }), $e = !0;
              function qe(Ge) {
                return Math.abs(Ge[1] - Se[2][1]) < 0.01;
              }
              function Xe(Ge) {
                return Math.abs(Ge[0] - Se[0][0]) < 0.01;
              }
              function Ve(Ge) {
                return Math.abs(Ge[0] - Se[2][0]) < 0.01;
              }
              for (; Pe.length; ) {
                for (ke = ee.smoothopen(Ce.edgepaths[De], Ce.smoothing), Re += $e ? ke : ke.replace(/^M/, "L"), Pe.splice(Pe.indexOf(De), 1), we = Ce.edgepaths[De][Ce.edgepaths[De].length - 1], Le = -1, xe = 0; xe < 4; xe++) {
                  if (!we) {
                    ne.log("Missing end?", De, Ce);
                    break;
                  }
                  for (Me = we, Math.abs(Me[1] - Se[0][1]) < 0.01 && !Ve(we) ? Ee = Se[1] : Xe(we) ? Ee = Se[0] : qe(we) ? Ee = Se[3] : Ve(we) && (Ee = Se[2]), Qe = 0; Qe < Ce.edgepaths.length; Qe++) {
                    var Ze = Ce.edgepaths[Qe][0];
                    Math.abs(we[0] - Ee[0]) < 0.01 ? Math.abs(we[0] - Ze[0]) < 0.01 && (Ze[1] - we[1]) * (Ee[1] - Ze[1]) >= 0 && (Ee = Ze, Le = Qe) : Math.abs(we[1] - Ee[1]) < 0.01 ? Math.abs(we[1] - Ze[1]) < 0.01 && (Ze[0] - we[0]) * (Ee[0] - Ze[0]) >= 0 && (Ee = Ze, Le = Qe) : ne.log("endpt to newendpt is not vert. or horz.", we, Ee, Ze);
                  }
                  if (we = Ee, Le >= 0) break;
                  Re += "L" + Ee;
                }
                if (Le === Ce.edgepaths.length) {
                  ne.log("unclosed perimeter path");
                  break;
                }
                De = Le, ($e = Pe.indexOf(De) === -1) && (De = Pe[0], Re += "Z");
              }
              for (De = 0; De < Ce.paths.length; De++) Re += ee.smoothclosed(Ce.paths[De], Ce.smoothing);
              return Re;
            }
            function ve(Ce, Se, we, Ee) {
              var xe = Se.width / 2, Le = Se.height / 2, Qe = Ce.x, ke = Ce.y, Me = Ce.theta, Re = Math.cos(Me) * xe, De = Math.sin(Me) * xe, Pe = (Qe > Ee.center ? Ee.right - Qe : Qe - Ee.left) / (Re + Math.abs(Math.sin(Me) * Le)), $e = (ke > Ee.middle ? Ee.bottom - ke : ke - Ee.top) / (Math.abs(De) + Math.cos(Me) * Le);
              if (Pe < 1 || $e < 1) return 1 / 0;
              var qe = ye.EDGECOST * (1 / (Pe - 1) + 1 / ($e - 1));
              qe += ye.ANGLECOST * Me * Me;
              for (var Xe = Qe - Re, Ve = ke - De, Ze = Qe + Re, Ge = ke + De, ze = 0; ze < we.length; ze++) {
                var We = we[ze], at = Math.cos(We.theta) * We.width / 2, ct = Math.sin(We.theta) * We.width / 2, et = 2 * ne.segmentDistance(Xe, Ve, Ze, Ge, We.x - at, We.y - ct, We.x + at, We.y + ct) / (Se.height + We.height), dt = We.level === Se.level, Je = dt ? ye.SAMELEVELDISTANCE : 1;
                if (et <= Je) return 1 / 0;
                qe += ye.NEIGHBORCOST * (dt ? ye.SAMELEVELFACTOR : 1) / (et - Je);
              }
              return qe;
            }
            function be(Ce) {
              var Se, we, Ee = Ce.trace._emptypoints, xe = [], Le = Ce.z.length, Qe = Ce.z[0].length, ke = [];
              for (Se = 0; Se < Qe; Se++) ke.push(1);
              for (Se = 0; Se < Le; Se++) xe.push(ke.slice());
              for (Se = 0; Se < Ee.length; Se++) xe[(we = Ee[Se])[0]][we[1]] = 0;
              return Ce.zmask = xe, xe;
            }
            J.plot = function(Ce, Se, we, Ee) {
              var xe = Se.xaxis, Le = Se.yaxis;
              ne.makeTraceGroups(Ee, we, "contour").each(function(Qe) {
                var ke = te.select(this), Me = Qe[0], Re = Me.trace, De = Me.x, Pe = Me.y, $e = Re.contours, qe = fe($e, Se, Me), Xe = ne.ensureSingle(ke, "g", "heatmapcoloring"), Ve = [];
                $e.coloring === "heatmap" && (Ve = [Qe]), le(Ce, Se, Ve, Xe), ue(qe), de(qe);
                var Ze = xe.c2p(De[0], !0), Ge = xe.c2p(De[De.length - 1], !0), ze = Le.c2p(Pe[0], !0), We = Le.c2p(Pe[Pe.length - 1], !0), at = [[Ze, We], [Ge, We], [Ge, ze], [Ze, ze]], ct = qe;
                $e.type === "constraint" && (ct = he(qe, $e._operation)), function(et, dt, Je) {
                  var it = ne.ensureSingle(et, "g", "contourbg").selectAll("path").data(Je.coloring === "fill" ? [0] : []);
                  it.enter().append("path"), it.exit().remove(), it.attr("d", "M" + dt.join("L") + "Z").style("stroke", "none");
                }(ke, at, $e), function(et, dt, Je, it) {
                  var st = it.coloring === "fill" || it.type === "constraint" && it._operation !== "=", gt = "M" + Je.join("L") + "Z";
                  st && me(dt, it);
                  var bt = ne.ensureSingle(et, "g", "contourfill").selectAll("path").data(st ? dt : []);
                  bt.enter().append("path"), bt.exit().remove(), bt.each(function(yt) {
                    var St = (yt.prefixBoundary ? gt : "") + ge(yt, Je);
                    St ? te.select(this).attr("d", St).style("stroke", "none") : te.select(this).remove();
                  });
                }(ke, ct, at, $e), function(et, dt, Je, it, st) {
                  var gt = Je._context.staticPlot, bt = ne.ensureSingle(et, "g", "contourlines"), yt = st.showlines !== !1, St = st.showlabels, Lt = yt && St, Rt = J.createLines(bt, yt || St, dt, gt), vt = J.createLineClip(bt, Lt, Je, it.trace.uid), $t = et.selectAll("g.contourlabels").data(St ? [0] : []);
                  if ($t.exit().remove(), $t.enter().append("g").classed("contourlabels", !0), St) {
                    var Pt = [], Xt = [];
                    ne.clearLocationCache();
                    var Nt = J.labelFormatter(Je, it), Gt = ee.tester.append("text").attr("data-notex", 1).call(ee.font, st.labelfont), Ft = dt[0].xaxis, Tt = dt[0].yaxis, It = Ft._length, Ct = Tt._length, Kt = Ft.range, Vt = Tt.range, Yt = ne.aggNums(Math.min, null, it.x), sn = ne.aggNums(Math.max, null, it.x), hn = ne.aggNums(Math.min, null, it.y), Qt = ne.aggNums(Math.max, null, it.y), Ut = Math.max(Ft.c2p(Yt, !0), 0), Wt = Math.min(Ft.c2p(sn, !0), It), rn = Math.max(Tt.c2p(Qt, !0), 0), vn = Math.min(Tt.c2p(hn, !0), Ct), dn = {};
                    Kt[0] < Kt[1] ? (dn.left = Ut, dn.right = Wt) : (dn.left = Wt, dn.right = Ut), Vt[0] < Vt[1] ? (dn.top = rn, dn.bottom = vn) : (dn.top = vn, dn.bottom = rn), dn.middle = (dn.top + dn.bottom) / 2, dn.center = (dn.left + dn.right) / 2, Pt.push([[dn.left, dn.top], [dn.right, dn.top], [dn.right, dn.bottom], [dn.left, dn.bottom]]);
                    var Ht = Math.sqrt(It * It + Ct * Ct), jt = pe.LABELDISTANCE * Ht / Math.max(1, dt.length / pe.LABELINCREASE);
                    Rt.each(function(cn) {
                      var wn = J.calcTextOpts(cn.level, Nt, Gt, Je);
                      te.select(this).selectAll("path").each(function() {
                        var Rn = ne.getVisibleSegment(this, dn, wn.height / 2);
                        if (Rn && !(Rn.len < (wn.width + wn.height) * pe.LABELMIN)) for (var Pn = Math.min(Math.ceil(Rn.len / jt), pe.LABELMAX), Ie = 0; Ie < Pn; Ie++) {
                          var rt = J.findBestTextLocation(this, Rn, wn, Xt, dn);
                          if (!rt) break;
                          J.addLabelData(rt, wn, Xt, Pt);
                        }
                      });
                    }), Gt.remove(), J.drawLabels($t, Xt, Je, vt, Lt ? Pt : null);
                  }
                  St && !yt && Rt.remove();
                }(ke, qe, Ce, Me, $e), function(et, dt, Je, it, st) {
                  var gt = it.trace, bt = Je._fullLayout._clips, yt = "clip" + gt.uid, St = bt.selectAll("#" + yt).data(gt.connectgaps ? [] : [0]);
                  if (St.enter().append("clipPath").classed("contourclip", !0).attr("id", yt), St.exit().remove(), gt.connectgaps === !1) {
                    var Lt = { level: 0.9, crossings: {}, starts: [], edgepaths: [], paths: [], xaxis: dt.xaxis, yaxis: dt.yaxis, x: it.x, y: it.y, z: be(it), smoothing: 0 };
                    ue([Lt]), de([Lt]), me([Lt], { type: "levels" }), ne.ensureSingle(St, "path", "").attr("d", (Lt.prefixBoundary ? "M" + st.join("L") + "Z" : "") + ge(Lt, st));
                  } else yt = null;
                  ee.setClipUrl(et, yt, Je);
                }(ke, Se, Ce, Me, at);
              });
            }, J.createLines = function(Ce, Se, we, Ee) {
              var xe = we[0].smoothing, Le = Ce.selectAll("g.contourlevel").data(Se ? we : []);
              if (Le.exit().remove(), Le.enter().append("g").classed("contourlevel", !0), Se) {
                var Qe = Le.selectAll("path.openline").data(function(Me) {
                  return Me.pedgepaths || Me.edgepaths;
                });
                Qe.exit().remove(), Qe.enter().append("path").classed("openline", !0), Qe.attr("d", function(Me) {
                  return ee.smoothopen(Me, xe);
                }).style("stroke-miterlimit", 1).style("vector-effect", Ee ? "none" : "non-scaling-stroke");
                var ke = Le.selectAll("path.closedline").data(function(Me) {
                  return Me.ppaths || Me.paths;
                });
                ke.exit().remove(), ke.enter().append("path").classed("closedline", !0), ke.attr("d", function(Me) {
                  return ee.smoothclosed(Me, xe);
                }).style("stroke-miterlimit", 1).style("vector-effect", Ee ? "none" : "non-scaling-stroke");
              }
              return Le;
            }, J.createLineClip = function(Ce, Se, we, Ee) {
              var xe = Se ? "clipline" + Ee : null, Le = we._fullLayout._clips.selectAll("#" + xe).data(Se ? [0] : []);
              return Le.exit().remove(), Le.enter().append("clipPath").classed("contourlineclip", !0).attr("id", xe), ee.setClipUrl(Ce, xe, we), Le;
            }, J.labelFormatter = function(Ce, Se) {
              var we = Ce._fullLayout, Ee = Se.trace, xe = Ee.contours, Le = { type: "linear", _id: "ycontour", showexponent: "all", exponentformat: "B" };
              if (xe.labelformat) Le.tickformat = xe.labelformat, ce(Le, we);
              else {
                var Qe = ae.extractOpts(Ee);
                if (Qe && Qe.colorbar && Qe.colorbar._axis) Le = Qe.colorbar._axis;
                else {
                  if (xe.type === "constraint") {
                    var ke = xe.value;
                    ne.isArrayOrTypedArray(ke) ? Le.range = [ke[0], ke[ke.length - 1]] : Le.range = [ke, ke];
                  } else Le.range = [xe.start, xe.end], Le.nticks = (xe.end - xe.start) / xe.size;
                  Le.range[0] === Le.range[1] && (Le.range[1] += Le.range[0] || 1), Le.nticks || (Le.nticks = 1e3), ce(Le, we), se.prepTicks(Le), Le._tmin = null, Le._tmax = null;
                }
              }
              return function(Me) {
                return se.tickText(Le, Me).text;
              };
            }, J.calcTextOpts = function(Ce, Se, we, Ee) {
              var xe = Se(Ce);
              we.text(xe).call(oe.convertToTspans, Ee);
              var Le = we.node(), Qe = ee.bBox(Le, !0);
              return { text: xe, width: Qe.width, height: Qe.height, fontSize: +Le.style["font-size"].replace("px", ""), level: Ce, dy: (Qe.top + Qe.bottom) / 2 };
            }, J.findBestTextLocation = function(Ce, Se, we, Ee, xe) {
              var Le, Qe, ke, Me, Re, De = we.width;
              Se.isClosed ? (Qe = Se.len / ye.INITIALSEARCHPOINTS, Le = Se.min + Qe / 2, ke = Se.max) : (Qe = (Se.len - De) / (ye.INITIALSEARCHPOINTS + 1), Le = Se.min + Qe + De / 2, ke = Se.max - (Qe + De) / 2);
              for (var Pe = 1 / 0, $e = 0; $e < ye.ITERATIONS; $e++) {
                for (var qe = Le; qe < ke; qe += Qe) {
                  var Xe = ne.getTextLocation(Ce, Se.total, qe, De), Ve = ve(Xe, we, Ee, xe);
                  Ve < Pe && (Pe = Ve, Re = Xe, Me = qe);
                }
                if (Pe > 2 * ye.MAXCOST) break;
                $e && (Qe /= 2), ke = (Le = Me - Qe / 2) + 1.5 * Qe;
              }
              if (Pe <= ye.MAXCOST) return Re;
            }, J.addLabelData = function(Ce, Se, we, Ee) {
              var xe = Se.fontSize, Le = Se.width + xe / 3, Qe = Math.max(0, Se.height - xe / 3), ke = Ce.x, Me = Ce.y, Re = Ce.theta, De = Math.sin(Re), Pe = Math.cos(Re), $e = function(Xe, Ve) {
                return [ke + Xe * Pe - Ve * De, Me + Xe * De + Ve * Pe];
              }, qe = [$e(-Le / 2, -Qe / 2), $e(-Le / 2, Qe / 2), $e(Le / 2, Qe / 2), $e(Le / 2, -Qe / 2)];
              we.push({ text: Se.text, x: ke, y: Me, dy: Se.dy, theta: Re, level: Se.level, width: Le, height: Qe }), Ee.push(qe);
            }, J.drawLabels = function(Ce, Se, we, Ee, xe) {
              var Le = Ce.selectAll("text").data(Se, function(Me) {
                return Me.text + "," + Me.x + "," + Me.y + "," + Me.theta;
              });
              if (Le.exit().remove(), Le.enter().append("text").attr({ "data-notex": 1, "text-anchor": "middle" }).each(function(Me) {
                var Re = Me.x + Math.sin(Me.theta) * Me.dy, De = Me.y - Math.cos(Me.theta) * Me.dy;
                te.select(this).text(Me.text).attr({ x: Re, y: De, transform: "rotate(" + 180 * Me.theta / Math.PI + " " + Re + " " + De + ")" }).call(oe.convertToTspans, we);
              }), xe) {
                for (var Qe = "", ke = 0; ke < xe.length; ke++) Qe += "M" + xe[ke].join("L") + "Z";
                ne.ensureSingle(Ee, "path", "").attr("d", Qe);
              }
            };
          }, 62475: function(G, J, Z) {
            var te = Z(29714), ne = Z(34809);
            function ee(ae, oe, se) {
              var ce = { type: "linear", range: [ae, oe] };
              return te.autoTicks(ce, (oe - ae) / (se || 15)), ce;
            }
            G.exports = function(ae, oe) {
              var se = ae.contours;
              if (ae.autocontour) {
                var ce = ae.zmin, le = ae.zmax;
                (ae.zauto || ce === void 0) && (ce = ne.aggNums(Math.min, null, oe)), (ae.zauto || le === void 0) && (le = ne.aggNums(Math.max, null, oe));
                var ue = ee(ce, le, ae.ncontours);
                se.size = ue.dtick, se.start = te.tickFirst(ue), ue.range.reverse(), se.end = te.tickFirst(ue), se.start === ce && (se.start += se.size), se.end === le && (se.end -= se.size), se.start > se.end && (se.start = se.end = (se.start + se.end) / 2), ae._input.contours || (ae._input.contours = {}), ne.extendFlat(ae._input.contours, { start: se.start, end: se.end, size: se.size }), ae._input.autocontour = !0;
              } else if (se.type !== "constraint") {
                var de, fe = se.start, he = se.end, me = ae._input.contours;
                fe > he && (se.start = me.start = he, he = se.end = me.end = fe, fe = se.start), se.size > 0 || (de = fe === he ? 1 : ee(fe, he, ae.ncontours).dtick, me.size = se.size = de);
              }
            };
          }, 1328: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(12774), ae = Z(16438);
            G.exports = function(oe) {
              var se = te.select(oe).selectAll("g.contour");
              se.style("opacity", function(ce) {
                return ce[0].trace.opacity;
              }), se.each(function(ce) {
                var le = te.select(this), ue = ce[0].trace, de = ue.contours, fe = ue.line, he = de.size || 1, me = de.start, pe = de.type === "constraint", ye = !pe && de.coloring === "lines", ge = !pe && de.coloring === "fill", ve = ye || ge ? ae(ue) : null;
                le.selectAll("g.contourlevel").each(function(Se) {
                  te.select(this).selectAll("path").call(ne.lineGroupStyle, fe.width, ye ? ve(Se.level) : fe.color, fe.dash);
                });
                var be = de.labelfont;
                if (le.selectAll("g.contourlabels text").each(function(Se) {
                  ne.font(te.select(this), { weight: be.weight, style: be.style, variant: be.variant, textcase: be.textcase, lineposition: be.lineposition, shadow: be.shadow, family: be.family, size: be.size, color: be.color || (ye ? ve(Se.level) : fe.color) });
                }), pe) le.selectAll("g.contourfill path").style("fill", ue.fillcolor);
                else if (ge) {
                  var Ce;
                  le.selectAll("g.contourfill path").style("fill", function(Se) {
                    return Ce === void 0 && (Ce = Se.level), ve(Se.level + 0.5 * he);
                  }), Ce === void 0 && (Ce = me), le.selectAll("g.contourbg path").style("fill", ve(Ce - 0.5 * he));
                }
              }), ee(oe);
            };
          }, 39889: function(G, J, Z) {
            var te = Z(39356), ne = Z(20576);
            G.exports = function(ee, ae, oe, se, ce) {
              var le, ue = oe("contours.coloring"), de = "";
              ue === "fill" && (le = oe("contours.showlines")), le !== !1 && (ue !== "lines" && (de = oe("line.color", "#000")), oe("line.width", 0.5), oe("line.dash")), ue !== "none" && (ee.showlegend !== !0 && (ae.showlegend = !1), ae._dfltShowLegend = !1, te(ee, ae, se, oe, { prefix: "", cLetter: "z" })), oe("line.smoothing"), ne(oe, se, de, ce);
            };
          }, 66365: function(G, J, Z) {
            var te = Z(81658), ne = Z(52240), ee = Z(87163), ae = Z(93049).extendFlat, oe = ne.contours;
            G.exports = ae({ carpet: { valType: "string", editType: "calc" }, z: te.z, a: te.x, a0: te.x0, da: te.dx, b: te.y, b0: te.y0, db: te.dy, text: te.text, hovertext: te.hovertext, transpose: te.transpose, atype: te.xtype, btype: te.ytype, fillcolor: ne.fillcolor, autocontour: ne.autocontour, ncontours: ne.ncontours, contours: { type: oe.type, start: oe.start, end: oe.end, size: oe.size, coloring: { valType: "enumerated", values: ["fill", "lines", "none"], dflt: "fill", editType: "calc" }, showlines: oe.showlines, showlabels: oe.showlabels, labelfont: oe.labelfont, labelformat: oe.labelformat, operation: oe.operation, value: oe.value, editType: "calc", impliedEdits: { autocontour: !1 } }, line: { color: ne.line.color, width: ne.line.width, dash: ne.line.dash, smoothing: ne.line.smoothing, editType: "plot" }, zorder: ne.zorder, transforms: void 0 }, ee("", { cLetter: "z", autoColorDflt: !1 }));
          }, 80849: function(G, J, Z) {
            var te = Z(28379), ne = Z(34809), ee = Z(87869), ae = Z(93877), oe = Z(69295), se = Z(78106), ce = Z(80924), le = Z(50538), ue = Z(26571), de = Z(62475);
            G.exports = function(fe, he) {
              var me = he._carpetTrace = ue(fe, he);
              if (me && me.visible && me.visible !== "legendonly") {
                if (!he.a || !he.b) {
                  var pe = fe.data[me.index], ye = fe.data[he.index];
                  ye.a || (ye.a = pe.a), ye.b || (ye.b = pe.b), le(ye, he, he._defaultColor, fe._fullLayout);
                }
                var ge = function(ve, be) {
                  var Ce, Se, we, Ee, xe, Le, Qe, ke = be._carpetTrace, Me = ke.aaxis, Re = ke.baxis;
                  Me._minDtick = 0, Re._minDtick = 0, ne.isArray1D(be.z) && ee(be, Me, Re, "a", "b", ["z"]), Ce = be._a = be._a || be.a, Ee = be._b = be._b || be.b, Ce = Ce ? Me.makeCalcdata(be, "_a") : [], Ee = Ee ? Re.makeCalcdata(be, "_b") : [], Se = be.a0 || 0, we = be.da || 1, xe = be.b0 || 0, Le = be.db || 1, Qe = be._z = ae(be._z || be.z, be.transpose), be._emptypoints = se(Qe), oe(Qe, be._emptypoints);
                  var De = ne.maxRowLength(Qe), Pe = be.xtype === "scaled" ? "" : Ce, $e = ce(be, Pe, Se, we, De, Me), qe = be.ytype === "scaled" ? "" : Ee, Xe = { a: $e, b: ce(be, qe, xe, Le, Qe.length, Re), z: Qe };
                  return be.contours.type === "levels" && be.contours.coloring !== "none" && te(ve, be, { vals: Qe, containerStr: "", cLetter: "z" }), [Xe];
                }(fe, he);
                return de(he, he._z), ge;
              }
            };
          }, 50538: function(G, J, Z) {
            var te = Z(34809), ne = Z(86073), ee = Z(66365), ae = Z(29503), oe = Z(47495), se = Z(39889);
            G.exports = function(ce, le, ue, de) {
              function fe(he, me) {
                return te.coerce(ce, le, ee, he, me);
              }
              if (fe("carpet"), ce.a && ce.b) {
                if (!ne(ce, le, fe, de, "a", "b")) return void (le.visible = !1);
                fe("text"), fe("contours.type") === "constraint" ? ae(ce, le, fe, de, ue, { hasHover: !1 }) : (oe(ce, le, fe, function(he) {
                  return te.coerce2(ce, le, ee, he);
                }), se(ce, le, fe, de, { hasHover: !1 }));
              } else le._defaultColor = ue, le._length = null;
              fe("zorder");
            };
          }, 34406: function(G, J, Z) {
            G.exports = { attributes: Z(66365), supplyDefaults: Z(50538), colorbar: Z(92697), calc: Z(80849), plot: Z(71815), style: Z(1328), moduleType: "trace", name: "contourcarpet", basePlotModule: Z(37703), categories: ["cartesian", "svg", "carpet", "contour", "symbols", "showLegend", "hasLines", "carpetDependent", "noHover", "noSortingByValue"], meta: {} };
          }, 71815: function(G, J, Z) {
            var te = Z(45568), ne = Z(6720), ee = Z(3685), ae = Z(62203), oe = Z(34809), se = Z(83545), ce = Z(27657), le = Z(8850), ue = Z(53156), de = Z(1999), fe = Z(86828), he = Z(49886), me = Z(26571), pe = Z(94903);
            function ye(be, Ce, Se) {
              var we = be.getPointAtLength(Ce), Ee = be.getPointAtLength(Se), xe = Ee.x - we.x, Le = Ee.y - we.y, Qe = Math.sqrt(xe * xe + Le * Le);
              return [xe / Qe, Le / Qe];
            }
            function ge(be) {
              var Ce = Math.sqrt(be[0] * be[0] + be[1] * be[1]);
              return [be[0] / Ce, be[1] / Ce];
            }
            function ve(be, Ce) {
              var Se = Math.abs(be[0] * Ce[0] + be[1] * Ce[1]);
              return Math.sqrt(1 - Se * Se) / Se;
            }
            G.exports = function(be, Ce, Se, we) {
              var Ee = Ce.xaxis, xe = Ce.yaxis;
              oe.makeTraceGroups(we, Se, "contour").each(function(Le) {
                var Qe = te.select(this), ke = Le[0], Me = ke.trace, Re = Me._carpetTrace = me(be, Me), De = be.calcdata[Re.index][0];
                if (Re.visible && Re.visible !== "legendonly") {
                  var Pe = ke.a, $e = ke.b, qe = Me.contours, Xe = fe(qe, Ce, ke), Ve = qe.type === "constraint", Ze = qe._operation, Ge = Ve ? Ze === "=" ? "lines" : "fill" : qe.coloring, ze = [[Pe[0], $e[$e.length - 1]], [Pe[Pe.length - 1], $e[$e.length - 1]], [Pe[Pe.length - 1], $e[0]], [Pe[0], $e[0]]];
                  se(Xe);
                  var We = 1e-8 * (Pe[Pe.length - 1] - Pe[0]), at = 1e-8 * ($e[$e.length - 1] - $e[0]);
                  ce(Xe, We, at);
                  var ct, et, dt, Je, it = Xe;
                  qe.type === "constraint" && (it = de(Xe, Ze)), function(yt, St) {
                    var Lt, Rt, vt, $t, Pt, Xt, Nt, Gt, Ft;
                    for (Lt = 0; Lt < yt.length; Lt++) {
                      for (Pt = ($t = yt[Lt]).pedgepaths = [], Xt = $t.ppaths = [], Rt = 0; Rt < $t.edgepaths.length; Rt++) {
                        for (Ft = $t.edgepaths[Rt], Nt = [], vt = 0; vt < Ft.length; vt++) Nt[vt] = St(Ft[vt]);
                        Pt.push(Nt);
                      }
                      for (Rt = 0; Rt < $t.paths.length; Rt++) {
                        for (Ft = $t.paths[Rt], Gt = [], vt = 0; vt < Ft.length; vt++) Gt[vt] = St(Ft[vt]);
                        Xt.push(Gt);
                      }
                    }
                  }(Xe, bt);
                  var st = [];
                  for (Je = De.clipsegments.length - 1; Je >= 0; Je--) ct = De.clipsegments[Je], et = ne([], ct.x, Ee.c2p), dt = ne([], ct.y, xe.c2p), et.reverse(), dt.reverse(), st.push(ee(et, dt, ct.bicubic));
                  var gt = "M" + st.join("L") + "Z";
                  (function(yt, St, Lt, Rt, vt, $t) {
                    var Pt, Xt, Nt, Gt, Ft = oe.ensureSingle(yt, "g", "contourbg").selectAll("path").data($t !== "fill" || vt ? [] : [0]);
                    Ft.enter().append("path"), Ft.exit().remove();
                    var Tt = [];
                    for (Gt = 0; Gt < St.length; Gt++) Pt = St[Gt], Xt = ne([], Pt.x, Lt.c2p), Nt = ne([], Pt.y, Rt.c2p), Tt.push(ee(Xt, Nt, Pt.bicubic));
                    Ft.attr("d", "M" + Tt.join("L") + "Z").style("stroke", "none");
                  })(Qe, De.clipsegments, Ee, xe, Ve, Ge), function(yt, St, Lt, Rt, vt, $t, Pt, Xt, Nt, Gt, Ft) {
                    var Tt = Gt === "fill";
                    Tt && he(vt, yt.contours);
                    var It = oe.ensureSingle(St, "g", "contourfill").selectAll("path").data(Tt ? vt : []);
                    It.enter().append("path"), It.exit().remove(), It.each(function(Ct) {
                      var Kt = (Ct.prefixBoundary ? Ft : "") + function(Vt, Yt, sn, hn, Qt, Ut, Wt, rn) {
                        var vn, dn, Ht, jt, cn, wn, Rn, Pn = "", Ie = Yt.edgepaths.map(function(_n, gn) {
                          return gn;
                        }), rt = !0, ft = 1e-4 * Math.abs(sn[0][0] - sn[2][0]), pt = 1e-4 * Math.abs(sn[0][1] - sn[2][1]);
                        function wt(_n) {
                          return Math.abs(_n[1] - sn[0][1]) < pt;
                        }
                        function Bt(_n) {
                          return Math.abs(_n[1] - sn[2][1]) < pt;
                        }
                        function At(_n) {
                          return Math.abs(_n[0] - sn[0][0]) < ft;
                        }
                        function Dt(_n) {
                          return Math.abs(_n[0] - sn[2][0]) < ft;
                        }
                        function Ot(_n, gn) {
                          var bn, an, Zt, Mn, Ln = "";
                          for (wt(_n) && !Dt(_n) || Bt(_n) && !At(_n) ? (Mn = Qt.aaxis, Zt = pe(Qt, Ut, [_n[0], gn[0]], 0.5 * (_n[1] + gn[1]))) : (Mn = Qt.baxis, Zt = pe(Qt, Ut, 0.5 * (_n[0] + gn[0]), [_n[1], gn[1]])), bn = 1; bn < Zt.length; bn++) for (Ln += Mn.smoothing ? "C" : "L", an = 0; an < Zt[bn].length; an++) {
                            var In = Zt[bn][an];
                            Ln += [Wt.c2p(In[0]), rn.c2p(In[1])] + " ";
                          }
                          return Ln;
                        }
                        for (vn = 0, dn = null; Ie.length; ) {
                          var un = Yt.edgepaths[vn][0];
                          for (dn && (Pn += Ot(dn, un)), Rn = ae.smoothopen(Yt.edgepaths[vn].map(hn), Yt.smoothing), Pn += rt ? Rn : Rn.replace(/^M/, "L"), Ie.splice(Ie.indexOf(vn), 1), dn = Yt.edgepaths[vn][Yt.edgepaths[vn].length - 1], cn = -1, jt = 0; jt < 4; jt++) {
                            if (!dn) {
                              oe.log("Missing end?", vn, Yt);
                              break;
                            }
                            for (wt(dn) && !Dt(dn) ? Ht = sn[1] : At(dn) ? Ht = sn[0] : Bt(dn) ? Ht = sn[3] : Dt(dn) && (Ht = sn[2]), wn = 0; wn < Yt.edgepaths.length; wn++) {
                              var yn = Yt.edgepaths[wn][0];
                              Math.abs(dn[0] - Ht[0]) < ft ? Math.abs(dn[0] - yn[0]) < ft && (yn[1] - dn[1]) * (Ht[1] - yn[1]) >= 0 && (Ht = yn, cn = wn) : Math.abs(dn[1] - Ht[1]) < pt ? Math.abs(dn[1] - yn[1]) < pt && (yn[0] - dn[0]) * (Ht[0] - yn[0]) >= 0 && (Ht = yn, cn = wn) : oe.log("endpt to newendpt is not vert. or horz.", dn, Ht, yn);
                            }
                            if (cn >= 0) break;
                            Pn += Ot(dn, Ht), dn = Ht;
                          }
                          if (cn === Yt.edgepaths.length) {
                            oe.log("unclosed perimeter path");
                            break;
                          }
                          vn = cn, (rt = Ie.indexOf(vn) === -1) && (vn = Ie[0], Pn += Ot(dn, Ht) + "Z", dn = null);
                        }
                        for (vn = 0; vn < Yt.paths.length; vn++) Pn += ae.smoothclosed(Yt.paths[vn].map(hn), Yt.smoothing);
                        return Pn;
                      }(0, Ct, $t, Pt, Xt, Nt, Lt, Rt);
                      Kt ? te.select(this).attr("d", Kt).style("stroke", "none") : te.select(this).remove();
                    });
                  }(Me, Qe, Ee, xe, it, ze, bt, Re, De, Ge, gt), function(yt, St, Lt, Rt, vt, $t, Pt) {
                    var Xt = Lt._context.staticPlot, Nt = oe.ensureSingle(yt, "g", "contourlines"), Gt = vt.showlines !== !1, Ft = vt.showlabels, Tt = Gt && Ft, It = le.createLines(Nt, Gt || Ft, St, Xt), Ct = le.createLineClip(Nt, Tt, Lt, Rt.trace.uid), Kt = yt.selectAll("g.contourlabels").data(Ft ? [0] : []);
                    if (Kt.exit().remove(), Kt.enter().append("g").classed("contourlabels", !0), Ft) {
                      var Vt = $t.xaxis, Yt = $t.yaxis, sn = Vt._length, hn = Yt._length, Qt = [[[0, 0], [sn, 0], [sn, hn], [0, hn]]], Ut = [];
                      oe.clearLocationCache();
                      var Wt = le.labelFormatter(Lt, Rt), rn = ae.tester.append("text").attr("data-notex", 1).call(ae.font, vt.labelfont), vn = { left: 0, right: sn, center: sn / 2, top: 0, bottom: hn, middle: hn / 2 }, dn = Math.sqrt(sn * sn + hn * hn), Ht = ue.LABELDISTANCE * dn / Math.max(1, St.length / ue.LABELINCREASE);
                      It.each(function(jt) {
                        var cn = le.calcTextOpts(jt.level, Wt, rn, Lt);
                        te.select(this).selectAll("path").each(function(wn) {
                          var Rn = this, Pn = oe.getVisibleSegment(Rn, vn, cn.height / 2);
                          if (Pn && (function(pt, wt, Bt, At, Dt, Ot) {
                            for (var un, yn = 0; yn < Bt.pedgepaths.length; yn++) wt === Bt.pedgepaths[yn] && (un = Bt.edgepaths[yn]);
                            if (un) {
                              var _n = Dt.a[0], gn = Dt.a[Dt.a.length - 1], bn = Dt.b[0], an = Dt.b[Dt.b.length - 1], Zt = ye(pt, 0, 1), Mn = ye(pt, At.total, At.total - 1), Ln = kn(un[0], Zt), In = At.total - kn(un[un.length - 1], Mn);
                              At.min < Ln && (At.min = Ln), At.max > In && (At.max = In), At.len = At.max - At.min;
                            }
                            function kn(er, jn) {
                              var ur, Kn = 0, Jn = 0.1;
                              return (Math.abs(er[0] - _n) < Jn || Math.abs(er[0] - gn) < Jn) && (ur = ge(Dt.dxydb_rough(er[0], er[1], Jn)), Kn = Math.max(Kn, Ot * ve(jn, ur) / 2)), (Math.abs(er[1] - bn) < Jn || Math.abs(er[1] - an) < Jn) && (ur = ge(Dt.dxyda_rough(er[0], er[1], Jn)), Kn = Math.max(Kn, Ot * ve(jn, ur) / 2)), Kn;
                            }
                          }(Rn, wn, jt, Pn, Pt, cn.height), !(Pn.len < (cn.width + cn.height) * ue.LABELMIN))) for (var Ie = Math.min(Math.ceil(Pn.len / Ht), ue.LABELMAX), rt = 0; rt < Ie; rt++) {
                            var ft = le.findBestTextLocation(Rn, Pn, cn, Ut, vn);
                            if (!ft) break;
                            le.addLabelData(ft, cn, Ut, Qt);
                          }
                        });
                      }), rn.remove(), le.drawLabels(Kt, Ut, Lt, Ct, Tt ? Qt : null);
                    }
                    Ft && !Gt && It.remove();
                  }(Qe, Xe, be, ke, qe, Ce, Re), ae.setClipUrl(Qe, Re._clipPathId, be);
                }
                function bt(yt) {
                  var St = Re.ab2xy(yt[0], yt[1], !0);
                  return [Ee.c2p(St[0]), xe.c2p(St[1])];
                }
              });
            };
          }, 70690: function(G, J, Z) {
            var te = Z(87163), ne = Z(3208).rb, ee = Z(9829), ae = Z(71388), oe = Z(93049).extendFlat;
            G.exports = oe({ lon: ae.lon, lat: ae.lat, z: { valType: "data_array", editType: "calc" }, radius: { valType: "number", editType: "plot", arrayOk: !0, min: 1, dflt: 30 }, below: { valType: "string", editType: "plot" }, text: ae.text, hovertext: ae.hovertext, hoverinfo: oe({}, ee.hoverinfo, { flags: ["lon", "lat", "z", "text", "name"] }), hovertemplate: ne(), showlegend: oe({}, ee.showlegend, { dflt: !1 }) }, te("", { cLetter: "z", editTypeOverride: "calc" }));
          }, 91582: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809).isArrayOrTypedArray, ee = Z(63821).BADNUM, ae = Z(28379), oe = Z(34809)._;
            G.exports = function(se, ce) {
              for (var le = ce._length, ue = new Array(le), de = ce.z, fe = ne(de) && de.length, he = 0; he < le; he++) {
                var me = ue[he] = {}, pe = ce.lon[he], ye = ce.lat[he];
                if (me.lonlat = te(pe) && te(ye) ? [+pe, +ye] : [ee, ee], fe) {
                  var ge = de[he];
                  me.z = te(ge) ? ge : ee;
                }
              }
              return ae(se, ce, { vals: fe ? de : [0, 1], containerStr: "", cLetter: "z" }), le && (ue[0].t = { labels: { lat: oe(se, "lat:") + " ", lon: oe(se, "lon:") + " " } }), ue;
            };
          }, 95012: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(78766), ae = Z(88856), oe = Z(63821).BADNUM, se = Z(39532).makeBlank;
            G.exports = function(ce) {
              var le = ce[0].trace, ue = le.visible === !0 && le._length !== 0, de = le._opts = { heatmap: { layout: { visibility: "none" }, paint: {} }, geojson: se() };
              if (!ue) return de;
              var fe, he = [], me = le.z, pe = le.radius, ye = ne.isArrayOrTypedArray(me) && me.length, ge = ne.isArrayOrTypedArray(pe);
              for (fe = 0; fe < ce.length; fe++) {
                var ve = ce[fe], be = ve.lonlat;
                if (be[0] !== oe) {
                  var Ce = {};
                  if (ye) {
                    var Se = ve.z;
                    Ce.z = Se !== oe ? Se : 0;
                  }
                  ge && (Ce.r = te(pe[fe]) && pe[fe] > 0 ? +pe[fe] : 0), he.push({ type: "Feature", geometry: { type: "Point", coordinates: be }, properties: Ce });
                }
              }
              var we = ae.extractOpts(le), Ee = we.reversescale ? ae.flipScale(we.colorscale) : we.colorscale, xe = Ee[0][1], Le = ["interpolate", ["linear"], ["heatmap-density"], 0, ee.opacity(xe) < 1 ? xe : ee.addOpacity(xe, 0)];
              for (fe = 1; fe < Ee.length; fe++) Le.push(Ee[fe][0], Ee[fe][1]);
              var Qe = ["interpolate", ["linear"], ["get", "z"], we.min, 0, we.max, 1];
              return ne.extendFlat(de.heatmap.paint, { "heatmap-weight": ye ? Qe : 1 / (we.max - we.min), "heatmap-color": Le, "heatmap-radius": ge ? { type: "identity", property: "r" } : le.radius, "heatmap-opacity": le.opacity }), de.geojson = { type: "FeatureCollection", features: he }, de.heatmap.layout.visibility = "visible", de;
            };
          }, 9653: function(G, J, Z) {
            var te = Z(34809), ne = Z(39356), ee = Z(70690);
            G.exports = function(ae, oe, se, ce) {
              function le(he, me) {
                return te.coerce(ae, oe, ee, he, me);
              }
              var ue = le("lon") || [], de = le("lat") || [], fe = Math.min(ue.length, de.length);
              fe ? (oe._length = fe, le("z"), le("radius"), le("below"), le("text"), le("hovertext"), le("hovertemplate"), ne(ae, oe, ce, le, { prefix: "", cLetter: "z" })) : oe.visible = !1;
            };
          }, 16302: function(G) {
            G.exports = function(J, Z) {
              return J.lon = Z.lon, J.lat = Z.lat, J.z = Z.z, J;
            };
          }, 28045: function(G, J, Z) {
            var te = Z(29714), ne = Z(67275).hoverPoints, ee = Z(67275).getExtraText;
            G.exports = function(ae, oe, se) {
              var ce = ne(ae, oe, se);
              if (ce) {
                var le = ce[0], ue = le.cd, de = ue[0].trace, fe = ue[le.index];
                if (delete le.color, "z" in fe) {
                  var he = le.subplot.mockAxis;
                  le.z = fe.z, le.zLabel = te.tickText(he, he.c2l(fe.z), "hover").text;
                }
                return le.extraText = ee(de, fe, ue[0].t.labels), [le];
              }
            };
          }, 91995: function(G, J, Z) {
            G.exports = { attributes: Z(70690), supplyDefaults: Z(9653), colorbar: Z(12431), formatLabels: Z(66762), calc: Z(91582), plot: Z(99932), hoverPoints: Z(28045), eventData: Z(16302), getBelow: function(te, ne) {
              for (var ee = ne.getMapLayers(), ae = 0; ae < ee.length; ae++) {
                var oe = ee[ae], se = oe.id;
                if (oe.type === "symbol" && typeof se == "string" && se.indexOf("plotly-") === -1) return se;
              }
            }, moduleType: "trace", name: "densitymap", basePlotModule: Z(34091), categories: ["map", "gl", "showLegend"], meta: { hr_name: "density_map" } };
          }, 99932: function(G, J, Z) {
            var te = Z(95012), ne = Z(8814).traceLayerPrefix;
            function ee(oe, se) {
              this.type = "densitymap", this.subplot = oe, this.uid = se, this.sourceId = "source-" + se, this.layerList = [["heatmap", ne + se + "-heatmap"]], this.below = null;
            }
            var ae = ee.prototype;
            ae.update = function(oe) {
              var se = this.subplot, ce = this.layerList, le = te(oe), ue = se.belowLookup["trace-" + this.uid];
              se.map.getSource(this.sourceId).setData(le.geojson), ue !== this.below && (this._removeLayers(), this._addLayers(le, ue), this.below = ue);
              for (var de = 0; de < ce.length; de++) {
                var fe = ce[de], he = fe[0], me = fe[1], pe = le[he];
                se.setOptions(me, "setLayoutProperty", pe.layout), pe.layout.visibility === "visible" && se.setOptions(me, "setPaintProperty", pe.paint);
              }
            }, ae._addLayers = function(oe, se) {
              for (var ce = this.subplot, le = this.layerList, ue = this.sourceId, de = 0; de < le.length; de++) {
                var fe = le[de], he = fe[0], me = oe[he];
                ce.addLayer({ type: he, id: fe[1], source: ue, layout: me.layout, paint: me.paint }, se);
              }
            }, ae._removeLayers = function() {
              for (var oe = this.subplot.map, se = this.layerList, ce = se.length - 1; ce >= 0; ce--) oe.removeLayer(se[ce][1]);
            }, ae.dispose = function() {
              var oe = this.subplot.map;
              this._removeLayers(), oe.removeSource(this.sourceId);
            }, G.exports = function(oe, se) {
              var ce = se[0].trace, le = new ee(oe, ce.uid), ue = le.sourceId, de = te(se), fe = le.below = oe.belowLookup["trace-" + ce.uid];
              return oe.map.addSource(ue, { type: "geojson", data: de.geojson }), le._addLayers(de, fe), le;
            };
          }, 17347: function(G, J, Z) {
            var te = Z(87163), ne = Z(3208).rb, ee = Z(9829), ae = Z(95833), oe = Z(93049).extendFlat;
            G.exports = oe({ lon: ae.lon, lat: ae.lat, z: { valType: "data_array", editType: "calc" }, radius: { valType: "number", editType: "plot", arrayOk: !0, min: 1, dflt: 30 }, below: { valType: "string", editType: "plot" }, text: ae.text, hovertext: ae.hovertext, hoverinfo: oe({}, ee.hoverinfo, { flags: ["lon", "lat", "z", "text", "name"] }), hovertemplate: ne(), showlegend: oe({}, ee.showlegend, { dflt: !1 }) }, te("", { cLetter: "z", editTypeOverride: "calc" }));
          }, 60675: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809).isArrayOrTypedArray, ee = Z(63821).BADNUM, ae = Z(28379), oe = Z(34809)._;
            G.exports = function(se, ce) {
              for (var le = ce._length, ue = new Array(le), de = ce.z, fe = ne(de) && de.length, he = 0; he < le; he++) {
                var me = ue[he] = {}, pe = ce.lon[he], ye = ce.lat[he];
                if (me.lonlat = te(pe) && te(ye) ? [+pe, +ye] : [ee, ee], fe) {
                  var ge = de[he];
                  me.z = te(ge) ? ge : ee;
                }
              }
              return ae(se, ce, { vals: fe ? de : [0, 1], containerStr: "", cLetter: "z" }), le && (ue[0].t = { labels: { lat: oe(se, "lat:") + " ", lon: oe(se, "lon:") + " " } }), ue;
            };
          }, 78391: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(78766), ae = Z(88856), oe = Z(63821).BADNUM, se = Z(39532).makeBlank;
            G.exports = function(ce) {
              var le = ce[0].trace, ue = le.visible === !0 && le._length !== 0, de = le._opts = { heatmap: { layout: { visibility: "none" }, paint: {} }, geojson: se() };
              if (!ue) return de;
              var fe, he = [], me = le.z, pe = le.radius, ye = ne.isArrayOrTypedArray(me) && me.length, ge = ne.isArrayOrTypedArray(pe);
              for (fe = 0; fe < ce.length; fe++) {
                var ve = ce[fe], be = ve.lonlat;
                if (be[0] !== oe) {
                  var Ce = {};
                  if (ye) {
                    var Se = ve.z;
                    Ce.z = Se !== oe ? Se : 0;
                  }
                  ge && (Ce.r = te(pe[fe]) && pe[fe] > 0 ? +pe[fe] : 0), he.push({ type: "Feature", geometry: { type: "Point", coordinates: be }, properties: Ce });
                }
              }
              var we = ae.extractOpts(le), Ee = we.reversescale ? ae.flipScale(we.colorscale) : we.colorscale, xe = Ee[0][1], Le = ["interpolate", ["linear"], ["heatmap-density"], 0, ee.opacity(xe) < 1 ? xe : ee.addOpacity(xe, 0)];
              for (fe = 1; fe < Ee.length; fe++) Le.push(Ee[fe][0], Ee[fe][1]);
              var Qe = ["interpolate", ["linear"], ["get", "z"], we.min, 0, we.max, 1];
              return ne.extendFlat(de.heatmap.paint, { "heatmap-weight": ye ? Qe : 1 / (we.max - we.min), "heatmap-color": Le, "heatmap-radius": ge ? { type: "identity", property: "r" } : le.radius, "heatmap-opacity": le.opacity }), de.geojson = { type: "FeatureCollection", features: he }, de.heatmap.layout.visibility = "visible", de;
            };
          }, 1892: function(G, J, Z) {
            var te = Z(34809), ne = Z(39356), ee = Z(17347);
            G.exports = function(ae, oe, se, ce) {
              function le(he, me) {
                return te.coerce(ae, oe, ee, he, me);
              }
              var ue = le("lon") || [], de = le("lat") || [], fe = Math.min(ue.length, de.length);
              fe ? (oe._length = fe, le("z"), le("radius"), le("below"), le("text"), le("hovertext"), le("hovertemplate"), ne(ae, oe, ce, le, { prefix: "", cLetter: "z" })) : oe.visible = !1;
            };
          }, 8919: function(G) {
            G.exports = function(J, Z) {
              return J.lon = Z.lon, J.lat = Z.lat, J.z = Z.z, J;
            };
          }, 54478: function(G, J, Z) {
            var te = Z(29714), ne = Z(18016).hoverPoints, ee = Z(18016).getExtraText;
            G.exports = function(ae, oe, se) {
              var ce = ne(ae, oe, se);
              if (ce) {
                var le = ce[0], ue = le.cd, de = ue[0].trace, fe = ue[le.index];
                if (delete le.color, "z" in fe) {
                  var he = le.subplot.mockAxis;
                  le.z = fe.z, le.zLabel = te.tickText(he, he.c2l(fe.z), "hover").text;
                }
                return le.extraText = ee(de, fe, ue[0].t.labels), [le];
              }
            };
          }, 81264: function(G, J, Z) {
            G.exports = { attributes: Z(17347), supplyDefaults: Z(1892), colorbar: Z(12431), formatLabels: Z(69009), calc: Z(60675), plot: Z(5165), hoverPoints: Z(54478), eventData: Z(8919), getBelow: function(te, ne) {
              for (var ee = ne.getMapLayers(), ae = 0; ae < ee.length; ae++) {
                var oe = ee[ae], se = oe.id;
                if (oe.type === "symbol" && typeof se == "string" && se.indexOf("plotly-") === -1) return se;
              }
            }, moduleType: "trace", name: "densitymapbox", basePlotModule: Z(68192), categories: ["mapbox", "gl", "showLegend"], meta: { hr_name: "density_mapbox" } };
          }, 5165: function(G, J, Z) {
            var te = Z(78391), ne = Z(44245).traceLayerPrefix;
            function ee(oe, se) {
              this.type = "densitymapbox", this.subplot = oe, this.uid = se, this.sourceId = "source-" + se, this.layerList = [["heatmap", ne + se + "-heatmap"]], this.below = null;
            }
            var ae = ee.prototype;
            ae.update = function(oe) {
              var se = this.subplot, ce = this.layerList, le = te(oe), ue = se.belowLookup["trace-" + this.uid];
              se.map.getSource(this.sourceId).setData(le.geojson), ue !== this.below && (this._removeLayers(), this._addLayers(le, ue), this.below = ue);
              for (var de = 0; de < ce.length; de++) {
                var fe = ce[de], he = fe[0], me = fe[1], pe = le[he];
                se.setOptions(me, "setLayoutProperty", pe.layout), pe.layout.visibility === "visible" && se.setOptions(me, "setPaintProperty", pe.paint);
              }
            }, ae._addLayers = function(oe, se) {
              for (var ce = this.subplot, le = this.layerList, ue = this.sourceId, de = 0; de < le.length; de++) {
                var fe = le[de], he = fe[0], me = oe[he];
                ce.addLayer({ type: he, id: fe[1], source: ue, layout: me.layout, paint: me.paint }, se);
              }
            }, ae._removeLayers = function() {
              for (var oe = this.subplot.map, se = this.layerList, ce = se.length - 1; ce >= 0; ce--) oe.removeLayer(se[ce][1]);
            }, ae.dispose = function() {
              var oe = this.subplot.map;
              this._removeLayers(), oe.removeSource(this.sourceId);
            }, G.exports = function(oe, se) {
              var ce = se[0].trace, le = new ee(oe, ce.uid), ue = le.sourceId, de = te(se), fe = le.below = oe.belowLookup["trace-" + ce.uid];
              return oe.map.addSource(ue, { type: "geojson", data: de.geojson }), le._addLayers(de, fe), le;
            };
          }, 43179: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              for (var ae = 0; ae < ne.length; ae++) ne[ae].i = ae;
              te.mergeArray(ee.text, ne, "tx"), te.mergeArray(ee.hovertext, ne, "htx");
              var oe = ee.marker;
              if (oe) {
                te.mergeArray(oe.opacity, ne, "mo"), te.mergeArray(oe.color, ne, "mc");
                var se = oe.line;
                se && (te.mergeArray(se.color, ne, "mlc"), te.mergeArrayCastPositive(se.width, ne, "mlw"));
              }
            };
          }, 62824: function(G, J, Z) {
            var te, ne = Z(81481), ee = Z(36640).line, ae = Z(9829), oe = Z(80712).axisHoverFormat, se = Z(3208).rb, ce = Z(3208).ay, le = Z(87948), ue = Z(93049).extendFlat, de = Z(78766);
            G.exports = { x: ne.x, x0: ne.x0, dx: ne.dx, y: ne.y, y0: ne.y0, dy: ne.dy, xperiod: ne.xperiod, yperiod: ne.yperiod, xperiod0: ne.xperiod0, yperiod0: ne.yperiod0, xperiodalignment: ne.xperiodalignment, yperiodalignment: ne.yperiodalignment, xhoverformat: oe("x"), yhoverformat: oe("y"), hovertext: ne.hovertext, hovertemplate: se({}, { keys: le.eventDataKeys }), hoverinfo: ue({}, ae.hoverinfo, { flags: ["name", "x", "y", "text", "percent initial", "percent previous", "percent total"] }), textinfo: { valType: "flaglist", flags: ["label", "text", "percent initial", "percent previous", "percent total", "value"], extras: ["none"], editType: "plot", arrayOk: !1 }, texttemplate: ce({ editType: "plot" }, { keys: le.eventDataKeys.concat(["label", "value"]) }), text: ne.text, textposition: ne.textposition, insidetextanchor: ue({}, ne.insidetextanchor, { dflt: "middle" }), textangle: ue({}, ne.textangle, { dflt: 0 }), textfont: ne.textfont, insidetextfont: ne.insidetextfont, outsidetextfont: ne.outsidetextfont, constraintext: ne.constraintext, cliponaxis: ne.cliponaxis, orientation: ue({}, ne.orientation, {}), offset: ue({}, ne.offset, { arrayOk: !1 }), width: ue({}, ne.width, { arrayOk: !1 }), marker: (te = ue({}, ne.marker), delete te.pattern, delete te.cornerradius, te), connector: { fillcolor: { valType: "color", editType: "style" }, line: { color: ue({}, ee.color, { dflt: de.defaultLine }), width: ue({}, ee.width, { dflt: 0, editType: "plot" }), dash: ee.dash, editType: "style" }, visible: { valType: "boolean", dflt: !0, editType: "plot" }, editType: "plot" }, offsetgroup: ne.offsetgroup, alignmentgroup: ne.alignmentgroup, zorder: ne.zorder };
          }, 28152: function(G, J, Z) {
            var te = Z(29714), ne = Z(40528), ee = Z(43179), ae = Z(48861), oe = Z(63821).BADNUM;
            function se(ce) {
              return ce === oe ? 0 : ce;
            }
            G.exports = function(ce, le) {
              var ue, de, fe, he, me, pe, ye, ge, ve = te.getFromId(ce, le.xaxis || "x"), be = te.getFromId(ce, le.yaxis || "y");
              le.orientation === "h" ? (ue = ve.makeCalcdata(le, "x"), fe = be.makeCalcdata(le, "y"), he = ne(le, be, "y", fe), me = !!le.yperiodalignment, pe = "y") : (ue = be.makeCalcdata(le, "y"), fe = ve.makeCalcdata(le, "x"), he = ne(le, ve, "x", fe), me = !!le.xperiodalignment, pe = "x"), de = he.vals;
              var Ce, Se = Math.min(de.length, ue.length), we = new Array(Se);
              for (le._base = [], ye = 0; ye < Se; ye++) {
                ue[ye] < 0 && (ue[ye] = oe);
                var Ee = !1;
                ue[ye] !== oe && ye + 1 < Se && ue[ye + 1] !== oe && (Ee = !0), ge = we[ye] = { p: de[ye], s: ue[ye], cNext: Ee }, le._base[ye] = -0.5 * ge.s, me && (we[ye].orig_p = fe[ye], we[ye][pe + "End"] = he.ends[ye], we[ye][pe + "Start"] = he.starts[ye]), le.ids && (ge.id = String(le.ids[ye])), ye === 0 && (we[0].vTotal = 0), we[0].vTotal += se(ge.s), ge.begR = se(ge.s) / se(we[0].s);
              }
              for (ye = 0; ye < Se; ye++) (ge = we[ye]).s !== oe && (ge.sumR = ge.s / we[0].vTotal, ge.difR = Ce !== void 0 ? ge.s / Ce : 1, Ce = ge.s);
              return ee(we, le), ae(we, le), we;
            };
          }, 87948: function(G) {
            G.exports = { eventDataKeys: ["percentInitial", "percentPrevious", "percentTotal"] };
          }, 82539: function(G, J, Z) {
            var te = Z(24782).setGroupPositions;
            G.exports = function(ne, ee) {
              var ae, oe, se = ne._fullLayout, ce = ne._fullData, le = ne.calcdata, ue = ee.xaxis, de = ee.yaxis, fe = [], he = [], me = [];
              for (oe = 0; oe < ce.length; oe++) {
                var pe = ce[oe], ye = pe.orientation === "h";
                pe.visible === !0 && pe.xaxis === ue._id && pe.yaxis === de._id && pe.type === "funnel" && (ae = le[oe], ye ? me.push(ae) : he.push(ae), fe.push(ae));
              }
              var ge = { mode: se.funnelmode, norm: se.funnelnorm, gap: se.funnelgap, groupgap: se.funnelgroupgap };
              for (te(ne, ue, de, he, ge), te(ne, de, ue, me, ge), oe = 0; oe < fe.length; oe++) {
                ae = fe[oe];
                for (var ve = 0; ve < ae.length; ve++) ve + 1 < ae.length && (ae[ve].nextP0 = ae[ve + 1].p0, ae[ve].nextS0 = ae[ve + 1].s0, ae[ve].nextP1 = ae[ve + 1].p1, ae[ve].nextS1 = ae[ve + 1].s1);
              }
            };
          }, 30495: function(G, J, Z) {
            var te = Z(34809), ne = Z(36301), ee = Z(17550).handleText, ae = Z(99867), oe = Z(99669), se = Z(62824), ce = Z(78766);
            G.exports = { supplyDefaults: function(le, ue, de, fe) {
              function he(ge, ve) {
                return te.coerce(le, ue, se, ge, ve);
              }
              if (ae(le, ue, fe, he)) {
                oe(le, ue, fe, he), he("xhoverformat"), he("yhoverformat"), he("orientation", ue.y && !ue.x ? "v" : "h"), he("offset"), he("width");
                var me = he("text");
                he("hovertext"), he("hovertemplate");
                var pe = he("textposition");
                ee(le, ue, fe, he, pe, { moduleHasSelected: !1, moduleHasUnselected: !1, moduleHasConstrain: !0, moduleHasCliponaxis: !0, moduleHasTextangle: !0, moduleHasInsideanchor: !0 }), ue.textposition === "none" || ue.texttemplate || he("textinfo", te.isArrayOrTypedArray(me) ? "text+value" : "value");
                var ye = he("marker.color", de);
                he("marker.line.color", ce.defaultLine), he("marker.line.width"), he("connector.visible") && (he("connector.fillcolor", function(ge) {
                  var ve = te.isArrayOrTypedArray(ge) ? "#000" : ge;
                  return ce.addOpacity(ve, 0.5 * ce.opacity(ve));
                }(ye)), he("connector.line.width") && (he("connector.line.color"), he("connector.line.dash"))), he("zorder");
              } else ue.visible = !1;
            }, crossTraceDefaults: function(le, ue) {
              var de, fe;
              function he(pe) {
                return te.coerce(fe._input, fe, se, pe);
              }
              if (ue.funnelmode === "group") for (var me = 0; me < le.length; me++) de = (fe = le[me])._input, ne(de, fe, ue, he);
            } };
          }, 29412: function(G) {
            G.exports = function(J, Z) {
              return J.x = "xVal" in Z ? Z.xVal : Z.x, J.y = "yVal" in Z ? Z.yVal : Z.y, "percentInitial" in Z && (J.percentInitial = Z.percentInitial), "percentPrevious" in Z && (J.percentPrevious = Z.percentPrevious), "percentTotal" in Z && (J.percentTotal = Z.percentTotal), Z.xa && (J.xaxis = Z.xa), Z.ya && (J.yaxis = Z.ya), J;
            };
          }, 27759: function(G, J, Z) {
            var te = Z(78766).opacity, ne = Z(91664).hoverOnBars, ee = Z(34809).formatPercent;
            G.exports = function(ae, oe, se, ce, le) {
              var ue = ne(ae, oe, se, ce, le);
              if (ue) {
                var de = ue.cd, fe = de[0].trace, he = fe.orientation === "h", me = de[ue.index];
                ue[(he ? "x" : "y") + "LabelVal"] = me.s, ue.percentInitial = me.begR, ue.percentInitialLabel = ee(me.begR, 1), ue.percentPrevious = me.difR, ue.percentPreviousLabel = ee(me.difR, 1), ue.percentTotal = me.sumR, ue.percentTotalLabel = ee(me.sumR, 1);
                var pe = me.hi || fe.hoverinfo, ye = [];
                if (pe && pe !== "none" && pe !== "skip") {
                  var ge = pe === "all", ve = pe.split("+"), be = function(Ce) {
                    return ge || ve.indexOf(Ce) !== -1;
                  };
                  be("percent initial") && ye.push(ue.percentInitialLabel + " of initial"), be("percent previous") && ye.push(ue.percentPreviousLabel + " of previous"), be("percent total") && ye.push(ue.percentTotalLabel + " of total");
                }
                return ue.extraText = ye.join("<br>"), ue.color = function(Ce, Se) {
                  var we = Ce.marker, Ee = Se.mc || we.color, xe = Se.mlc || we.line.color, Le = Se.mlw || we.line.width;
                  return te(Ee) ? Ee : te(xe) && Le ? xe : void 0;
                }(fe, me), [ue];
              }
            };
          }, 52213: function(G, J, Z) {
            G.exports = { attributes: Z(62824), layoutAttributes: Z(93795), supplyDefaults: Z(30495).supplyDefaults, crossTraceDefaults: Z(30495).crossTraceDefaults, supplyLayoutDefaults: Z(34980), calc: Z(28152), crossTraceCalc: Z(82539), plot: Z(83482), style: Z(7240).style, hoverPoints: Z(27759), eventData: Z(29412), selectPoints: Z(88384), moduleType: "trace", name: "funnel", basePlotModule: Z(37703), categories: ["bar-like", "cartesian", "svg", "oriented", "showLegend", "zoomScale"], meta: {} };
          }, 93795: function(G) {
            G.exports = { funnelmode: { valType: "enumerated", values: ["stack", "group", "overlay"], dflt: "stack", editType: "calc" }, funnelgap: { valType: "number", min: 0, max: 1, editType: "calc" }, funnelgroupgap: { valType: "number", min: 0, max: 1, dflt: 0, editType: "calc" } };
          }, 34980: function(G, J, Z) {
            var te = Z(34809), ne = Z(93795);
            G.exports = function(ee, ae, oe) {
              var se = !1;
              function ce(de, fe) {
                return te.coerce(ee, ae, ne, de, fe);
              }
              for (var le = 0; le < oe.length; le++) {
                var ue = oe[le];
                if (ue.visible && ue.type === "funnel") {
                  se = !0;
                  break;
                }
              }
              se && (ce("funnelmode"), ce("funnelgap", 0.2), ce("funnelgroupgap"));
            };
          }, 83482: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(63821).BADNUM, oe = Z(32995), se = Z(84102).clearMinTextSize;
            function ce(le, ue, de, fe) {
              var he = [], me = [], pe = fe ? ue : de, ye = fe ? de : ue;
              return he[0] = pe.c2p(le.s0, !0), me[0] = ye.c2p(le.p0, !0), he[1] = pe.c2p(le.s1, !0), me[1] = ye.c2p(le.p1, !0), he[2] = pe.c2p(le.nextS0, !0), me[2] = ye.c2p(le.nextP0, !0), he[3] = pe.c2p(le.nextS1, !0), me[3] = ye.c2p(le.nextP1, !0), fe ? [he, me] : [me, he];
            }
            G.exports = function(le, ue, de, fe) {
              var he = le._fullLayout;
              se("funnel", he), function(me, pe, ye, ge) {
                var ve = pe.xaxis, be = pe.yaxis;
                ne.makeTraceGroups(ge, ye, "trace bars").each(function(Ce) {
                  var Se = te.select(this), we = Ce[0].trace, Ee = ne.ensureSingle(Se, "g", "regions");
                  if (we.connector && we.connector.visible) {
                    var xe = we.orientation === "h", Le = Ee.selectAll("g.region").data(ne.identity);
                    Le.enter().append("g").classed("region", !0), Le.exit().remove();
                    var Qe = Le.size();
                    Le.each(function(ke, Me) {
                      if (Me === Qe - 1 || ke.cNext) {
                        var Re = ce(ke, ve, be, xe), De = Re[0], Pe = Re[1], $e = "";
                        De[0] !== ae && Pe[0] !== ae && De[1] !== ae && Pe[1] !== ae && De[2] !== ae && Pe[2] !== ae && De[3] !== ae && Pe[3] !== ae && ($e += xe ? "M" + De[0] + "," + Pe[1] + "L" + De[2] + "," + Pe[2] + "H" + De[3] + "L" + De[1] + "," + Pe[1] + "Z" : "M" + De[1] + "," + Pe[1] + "L" + De[2] + "," + Pe[3] + "V" + Pe[2] + "L" + De[1] + "," + Pe[0] + "Z"), $e === "" && ($e = "M0,0Z"), ne.ensureSingle(te.select(this), "path").attr("d", $e).call(ee.setClipUrl, pe.layerClipId, me);
                      }
                    });
                  } else Ee.remove();
                });
              }(le, ue, de, fe), function(me, pe, ye, ge) {
                var ve = pe.xaxis, be = pe.yaxis;
                ne.makeTraceGroups(ge, ye, "trace bars").each(function(Ce) {
                  var Se = te.select(this), we = Ce[0].trace, Ee = ne.ensureSingle(Se, "g", "lines");
                  if (we.connector && we.connector.visible && we.connector.line.width) {
                    var xe = we.orientation === "h", Le = Ee.selectAll("g.line").data(ne.identity);
                    Le.enter().append("g").classed("line", !0), Le.exit().remove();
                    var Qe = Le.size();
                    Le.each(function(ke, Me) {
                      if (Me === Qe - 1 || ke.cNext) {
                        var Re = ce(ke, ve, be, xe), De = Re[0], Pe = Re[1], $e = "";
                        De[3] !== void 0 && Pe[3] !== void 0 && (xe ? ($e += "M" + De[0] + "," + Pe[1] + "L" + De[2] + "," + Pe[2], $e += "M" + De[1] + "," + Pe[1] + "L" + De[3] + "," + Pe[2]) : ($e += "M" + De[1] + "," + Pe[1] + "L" + De[2] + "," + Pe[3], $e += "M" + De[1] + "," + Pe[0] + "L" + De[2] + "," + Pe[2])), $e === "" && ($e = "M0,0Z"), ne.ensureSingle(te.select(this), "path").attr("d", $e).call(ee.setClipUrl, pe.layerClipId, me);
                      }
                    });
                  } else Ee.remove();
                });
              }(le, ue, de, fe), oe.plot(le, ue, de, fe, { mode: he.funnelmode, norm: he.funnelmode, gap: he.funnelgap, groupgap: he.funnelgroupgap });
            };
          }, 7240: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(78766), ae = Z(20438).DESELECTDIM, oe = Z(6851), se = Z(84102).resizeText, ce = oe.styleTextPoints;
            G.exports = { style: function(le, ue, de) {
              var fe = de || te.select(le).selectAll('g[class^="funnellayer"]').selectAll("g.trace");
              se(le, fe, "funnel"), fe.style("opacity", function(he) {
                return he[0].trace.opacity;
              }), fe.each(function(he) {
                var me = te.select(this), pe = he[0].trace;
                me.selectAll(".point > path").each(function(ye) {
                  if (!ye.isBlank) {
                    var ge = pe.marker;
                    te.select(this).call(ee.fill, ye.mc || ge.color).call(ee.stroke, ye.mlc || ge.line.color).call(ne.dashLine, ge.line.dash, ye.mlw || ge.line.width).style("opacity", pe.selectedpoints && !ye.selected ? ae : 1);
                  }
                }), ce(me, pe, le), me.selectAll(".regions").each(function() {
                  te.select(this).selectAll("path").style("stroke-width", 0).call(ee.fill, pe.connector.fillcolor);
                }), me.selectAll(".lines").each(function() {
                  var ye = pe.connector.line;
                  ne.lineGroupStyle(te.select(this).selectAll("path"), ye.width, ye.color, ye.dash);
                });
              });
            } };
          }, 63447: function(G, J, Z) {
            var te = Z(55412), ne = Z(9829), ee = Z(13792).u, ae = Z(3208).rb, oe = Z(3208).ay, se = Z(93049).extendFlat;
            G.exports = { labels: te.labels, label0: te.label0, dlabel: te.dlabel, values: te.values, marker: { colors: te.marker.colors, line: { color: se({}, te.marker.line.color, { dflt: null }), width: se({}, te.marker.line.width, { dflt: 1 }), editType: "calc" }, pattern: te.marker.pattern, editType: "calc" }, text: te.text, hovertext: te.hovertext, scalegroup: se({}, te.scalegroup, {}), textinfo: se({}, te.textinfo, { flags: ["label", "text", "value", "percent"] }), texttemplate: oe({ editType: "plot" }, { keys: ["label", "color", "value", "text", "percent"] }), hoverinfo: se({}, ne.hoverinfo, { flags: ["label", "text", "value", "percent", "name"] }), hovertemplate: ae({}, { keys: ["label", "color", "value", "text", "percent"] }), textposition: se({}, te.textposition, { values: ["inside", "none"], dflt: "inside" }), textfont: te.textfont, insidetextfont: te.insidetextfont, title: { text: te.title.text, font: te.title.font, position: se({}, te.title.position, { values: ["top left", "top center", "top right"], dflt: "top center" }), editType: "plot" }, domain: ee({ name: "funnelarea", trace: !0, editType: "calc" }), aspectratio: { valType: "number", min: 0, dflt: 1, editType: "plot" }, baseratio: { valType: "number", min: 0, max: 1, dflt: 0.333, editType: "plot" } };
          }, 86817: function(G, J, Z) {
            var te = Z(44122);
            J.name = "funnelarea", J.plot = function(ne, ee, ae, oe) {
              te.plotBasePlot(J.name, ne, ee, ae, oe);
            }, J.clean = function(ne, ee, ae, oe) {
              te.cleanBasePlot(J.name, ne, ee, ae, oe);
            };
          }, 2807: function(G, J, Z) {
            var te = Z(44148);
            G.exports = { calc: function(ne, ee) {
              return te.calc(ne, ee);
            }, crossTraceCalc: function(ne) {
              te.crossTraceCalc(ne, { type: "funnelarea" });
            } };
          }, 79824: function(G, J, Z) {
            var te = Z(34809), ne = Z(63447), ee = Z(13792).N, ae = Z(17550).handleText, oe = Z(46979).handleLabelsAndValues, se = Z(46979).handleMarkerDefaults;
            G.exports = function(ce, le, ue, de) {
              function fe(Se, we) {
                return te.coerce(ce, le, ne, Se, we);
              }
              var he = fe("labels"), me = fe("values"), pe = oe(he, me), ye = pe.len;
              if (le._hasLabels = pe.hasLabels, le._hasValues = pe.hasValues, !le._hasLabels && le._hasValues && (fe("label0"), fe("dlabel")), ye) {
                le._length = ye, se(ce, le, de, fe), fe("scalegroup");
                var ge, ve = fe("text"), be = fe("texttemplate");
                if (be || (ge = fe("textinfo", Array.isArray(ve) ? "text+percent" : "percent")), fe("hovertext"), fe("hovertemplate"), be || ge && ge !== "none") {
                  var Ce = fe("textposition");
                  ae(ce, le, de, fe, Ce, { moduleHasSelected: !1, moduleHasUnselected: !1, moduleHasConstrain: !1, moduleHasCliponaxis: !1, moduleHasTextangle: !1, moduleHasInsideanchor: !1 });
                } else ge === "none" && fe("textposition", "none");
                ee(le, de, fe), fe("title.text") && (fe("title.position"), te.coerceFont(fe, "title.font", de.font)), fe("aspectratio"), fe("baseratio");
              } else le.visible = !1;
            };
          }, 91132: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "funnelarea", basePlotModule: Z(86817), categories: ["pie-like", "funnelarea", "showLegend"], attributes: Z(63447), layoutAttributes: Z(10270), supplyDefaults: Z(79824), supplyLayoutDefaults: Z(69161), calc: Z(2807).calc, crossTraceCalc: Z(2807).crossTraceCalc, plot: Z(96673), style: Z(13757), styleOne: Z(32891), meta: {} };
          }, 10270: function(G, J, Z) {
            var te = Z(4031).hiddenlabels;
            G.exports = { hiddenlabels: te, funnelareacolorway: { valType: "colorlist", editType: "calc" }, extendfunnelareacolors: { valType: "boolean", dflt: !0, editType: "calc" } };
          }, 69161: function(G, J, Z) {
            var te = Z(34809), ne = Z(10270);
            G.exports = function(ee, ae) {
              function oe(se, ce) {
                return te.coerce(ee, ae, ne, se, ce);
              }
              oe("hiddenlabels"), oe("funnelareacolorway", ae.colorway), oe("extendfunnelareacolors");
            };
          }, 96673: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(34809), ae = ee.strScale, oe = ee.strTranslate, se = Z(30635), ce = Z(32995).toMoveInsideBar, le = Z(84102), ue = le.recordMinTextSize, de = le.clearMinTextSize, fe = Z(37252), he = Z(35734), me = he.attachFxHandlers, pe = he.determineInsideTextFont, ye = he.layoutAreas, ge = he.prerenderTitles, ve = he.positionTitleOutside, be = he.formatSliceLabel;
            function Ce(Se, we) {
              return "l" + (we[0] - Se[0]) + "," + (we[1] - Se[1]);
            }
            G.exports = function(Se, we) {
              var Ee = Se._context.staticPlot, xe = Se._fullLayout;
              de("funnelarea", xe), ge(we, Se), ye(we, xe._size), ee.makeTraceGroups(xe._funnelarealayer, we, "trace").each(function(Le) {
                var Qe = te.select(this), ke = Le[0], Me = ke.trace;
                (function(Re) {
                  if (Re.length) {
                    var De = Re[0], Pe = De.trace, $e = Pe.aspectratio, qe = Pe.baseratio;
                    qe > 0.999 && (qe = 0.999);
                    var Xe, Ve, Ze, Ge = Math.pow(qe, 2), ze = De.vTotal, We = ze, at = ze * Ge / (1 - Ge) / ze, ct = [];
                    for (ct.push(Nt()), Ve = Re.length - 1; Ve > -1; Ve--) if (!(Ze = Re[Ve]).hidden) {
                      var et = Ze.v / We;
                      at += et, ct.push(Nt());
                    }
                    var dt = 1 / 0, Je = -1 / 0;
                    for (Ve = 0; Ve < ct.length; Ve++) Xe = ct[Ve], dt = Math.min(dt, Xe[1]), Je = Math.max(Je, Xe[1]);
                    for (Ve = 0; Ve < ct.length; Ve++) ct[Ve][1] -= (Je + dt) / 2;
                    var it = ct[ct.length - 1][0], st = De.r, gt = (Je - dt) / 2, bt = st / it, yt = st / gt * $e;
                    for (De.r = yt * gt, Ve = 0; Ve < ct.length; Ve++) ct[Ve][0] *= bt, ct[Ve][1] *= yt;
                    var St, Lt, Rt = [-(Xe = ct[0])[0], Xe[1]], vt = [Xe[0], Xe[1]], $t = 0;
                    for (Ve = Re.length - 1; Ve > -1; Ve--) if (!(Ze = Re[Ve]).hidden) {
                      var Pt = ct[$t += 1][0], Xt = ct[$t][1];
                      Ze.TL = [-Pt, Xt], Ze.TR = [Pt, Xt], Ze.BL = Rt, Ze.BR = vt, Ze.pxmid = (St = Ze.TR, Lt = Ze.BR, [0.5 * (St[0] + Lt[0]), 0.5 * (St[1] + Lt[1])]), Rt = Ze.TL, vt = Ze.TR;
                    }
                  }
                  function Nt() {
                    var Gt, Ft = { x: Gt = Math.sqrt(at), y: -Gt };
                    return [Ft.x, Ft.y];
                  }
                })(Le), Qe.each(function() {
                  var Re = te.select(this).selectAll("g.slice").data(Le);
                  Re.enter().append("g").classed("slice", !0), Re.exit().remove(), Re.each(function(Pe, $e) {
                    if (Pe.hidden) te.select(this).selectAll("path,g").remove();
                    else {
                      Pe.pointNumber = Pe.i, Pe.curveNumber = Me.index;
                      var qe = ke.cx, Xe = ke.cy, Ve = te.select(this), Ze = Ve.selectAll("path.surface").data([Pe]);
                      Ze.enter().append("path").classed("surface", !0).style({ "pointer-events": Ee ? "none" : "all" }), Ve.call(me, Se, Le);
                      var Ge = "M" + (qe + Pe.TR[0]) + "," + (Xe + Pe.TR[1]) + Ce(Pe.TR, Pe.BR) + Ce(Pe.BR, Pe.BL) + Ce(Pe.BL, Pe.TL) + "Z";
                      Ze.attr("d", Ge), be(Se, Pe, ke);
                      var ze = fe.castOption(Me.textposition, Pe.pts), We = Ve.selectAll("g.slicetext").data(Pe.text && ze !== "none" ? [0] : []);
                      We.enter().append("g").classed("slicetext", !0), We.exit().remove(), We.each(function() {
                        var at = ee.ensureSingle(te.select(this), "text", "", function(bt) {
                          bt.attr("data-notex", 1);
                        }), ct = ee.ensureUniformFontSize(Se, pe(Me, Pe, xe.font));
                        at.text(Pe.text).attr({ class: "slicetext", transform: "", "text-anchor": "middle" }).call(ne.font, ct).call(se.convertToTspans, Se);
                        var et, dt, Je, it = ne.bBox(at.node()), st = Math.min(Pe.BL[1], Pe.BR[1]) + Xe, gt = Math.max(Pe.TL[1], Pe.TR[1]) + Xe;
                        dt = Math.max(Pe.TL[0], Pe.BL[0]) + qe, Je = Math.min(Pe.TR[0], Pe.BR[0]) + qe, (et = ce(dt, Je, st, gt, it, { isHorizontal: !0, constrained: !0, angle: 0, anchor: "middle" })).fontSize = ct.size, ue(Me.type, et, xe), Le[$e].transform = et, ee.setTransormAndDisplay(at, et);
                      });
                    }
                  });
                  var De = te.select(this).selectAll("g.titletext").data(Me.title.text ? [0] : []);
                  De.enter().append("g").classed("titletext", !0), De.exit().remove(), De.each(function() {
                    var Pe = ee.ensureSingle(te.select(this), "text", "", function(Xe) {
                      Xe.attr("data-notex", 1);
                    }), $e = Me.title.text;
                    Me._meta && ($e = ee.templateString($e, Me._meta)), Pe.text($e).attr({ class: "titletext", transform: "", "text-anchor": "middle" }).call(ne.font, Me.title.font).call(se.convertToTspans, Se);
                    var qe = ve(ke, xe._size);
                    Pe.attr("transform", oe(qe.x, qe.y) + ae(Math.min(1, qe.scale)) + oe(qe.tx, qe.ty));
                  });
                });
              });
            };
          }, 13757: function(G, J, Z) {
            var te = Z(45568), ne = Z(32891), ee = Z(84102).resizeText;
            G.exports = function(ae) {
              var oe = ae._fullLayout._funnelarealayer.selectAll(".trace");
              ee(ae, oe, "funnelarea"), oe.each(function(se) {
                var ce = se[0].trace, le = te.select(this);
                le.style({ opacity: ce.opacity }), le.selectAll("path.surface").each(function(ue) {
                  te.select(this).call(ne, ue, ce, ae);
                });
              });
            };
          }, 81658: function(G, J, Z) {
            var te = Z(36640), ne = Z(9829), ee = Z(80337), ae = Z(80712).axisHoverFormat, oe = Z(3208).rb, se = Z(3208).ay, ce = Z(87163), le = Z(93049).extendFlat;
            G.exports = le({ z: { valType: "data_array", editType: "calc" }, x: le({}, te.x, { impliedEdits: { xtype: "array" } }), x0: le({}, te.x0, { impliedEdits: { xtype: "scaled" } }), dx: le({}, te.dx, { impliedEdits: { xtype: "scaled" } }), y: le({}, te.y, { impliedEdits: { ytype: "array" } }), y0: le({}, te.y0, { impliedEdits: { ytype: "scaled" } }), dy: le({}, te.dy, { impliedEdits: { ytype: "scaled" } }), xperiod: le({}, te.xperiod, { impliedEdits: { xtype: "scaled" } }), yperiod: le({}, te.yperiod, { impliedEdits: { ytype: "scaled" } }), xperiod0: le({}, te.xperiod0, { impliedEdits: { xtype: "scaled" } }), yperiod0: le({}, te.yperiod0, { impliedEdits: { ytype: "scaled" } }), xperiodalignment: le({}, te.xperiodalignment, { impliedEdits: { xtype: "scaled" } }), yperiodalignment: le({}, te.yperiodalignment, { impliedEdits: { ytype: "scaled" } }), text: { valType: "data_array", editType: "calc" }, hovertext: { valType: "data_array", editType: "calc" }, transpose: { valType: "boolean", dflt: !1, editType: "calc" }, xtype: { valType: "enumerated", values: ["array", "scaled"], editType: "calc+clearAxisTypes" }, ytype: { valType: "enumerated", values: ["array", "scaled"], editType: "calc+clearAxisTypes" }, zsmooth: { valType: "enumerated", values: ["fast", "best", !1], dflt: !1, editType: "calc" }, hoverongaps: { valType: "boolean", dflt: !0, editType: "none" }, connectgaps: { valType: "boolean", editType: "calc" }, xgap: { valType: "number", dflt: 0, min: 0, editType: "plot" }, ygap: { valType: "number", dflt: 0, min: 0, editType: "plot" }, xhoverformat: ae("x"), yhoverformat: ae("y"), zhoverformat: ae("z", 1), hovertemplate: oe(), texttemplate: se({ arrayOk: !1, editType: "plot" }, { keys: ["x", "y", "z", "text"] }), textfont: ee({ editType: "plot", autoSize: !0, autoColor: !0, colorEditType: "style" }), showlegend: le({}, ne.showlegend, { dflt: !1 }), zorder: te.zorder }, { transforms: void 0 }, ce("", { cLetter: "z", autoColorDflt: !1 }));
          }, 51670: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(29714), ae = Z(40528), oe = Z(19226), se = Z(28379), ce = Z(87869), le = Z(93877), ue = Z(69295), de = Z(78106), fe = Z(80924), he = Z(63821).BADNUM;
            function me(pe) {
              for (var ye = [], ge = pe.length, ve = 0; ve < ge; ve++) {
                var be = pe[ve];
                be !== he && ye.push(be);
              }
              return ye;
            }
            G.exports = function(pe, ye) {
              var ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me = ee.getFromId(pe, ye.xaxis || "x"), Re = ee.getFromId(pe, ye.yaxis || "y"), De = te.traceIs(ye, "contour"), Pe = te.traceIs(ye, "histogram"), $e = te.traceIs(ye, "gl2d"), qe = De ? "best" : ye.zsmooth;
              if (Me._minDtick = 0, Re._minDtick = 0, Pe) Ce = (ke = oe(pe, ye)).orig_x, ge = ke.x, ve = ke.x0, be = ke.dx, xe = ke.orig_y, Se = ke.y, we = ke.y0, Ee = ke.dy, Le = ke.z;
              else {
                var Xe = ye.z;
                ne.isArray1D(Xe) ? (ce(ye, Me, Re, "x", "y", ["z"]), ge = ye._x, Se = ye._y, Xe = ye._z) : (Ce = ye.x ? Me.makeCalcdata(ye, "x") : [], xe = ye.y ? Re.makeCalcdata(ye, "y") : [], ge = ae(ye, Me, "x", Ce).vals, Se = ae(ye, Re, "y", xe).vals, ye._x = ge, ye._y = Se), ve = ye.x0, be = ye.dx, we = ye.y0, Ee = ye.dy, Le = le(Xe, ye, Me, Re);
              }
              function Ve(Je) {
                qe = ye._input.zsmooth = ye.zsmooth = !1, ne.warn('cannot use zsmooth: "fast": ' + Je);
              }
              function Ze(Je) {
                if (Je.length > 1) {
                  var it = (Je[Je.length - 1] - Je[0]) / (Je.length - 1), st = Math.abs(it / 100);
                  for (Qe = 0; Qe < Je.length - 1; Qe++) if (Math.abs(Je[Qe + 1] - Je[Qe] - it) > st) return !1;
                }
                return !0;
              }
              (Me.rangebreaks || Re.rangebreaks) && (Le = function(Je, it, st) {
                for (var gt = [], bt = -1, yt = 0; yt < st.length; yt++) if (it[yt] !== he) {
                  gt[++bt] = [];
                  for (var St = 0; St < st[yt].length; St++) Je[St] !== he && gt[bt].push(st[yt][St]);
                }
                return gt;
              }(ge, Se, Le), Pe || (ge = me(ge), Se = me(Se), ye._x = ge, ye._y = Se)), Pe || !De && !ye.connectgaps || (ye._emptypoints = de(Le), ue(Le, ye._emptypoints)), ye._islinear = !1, Me.type === "log" || Re.type === "log" ? qe === "fast" && Ve("log axis found") : Ze(ge) ? Ze(Se) ? ye._islinear = !0 : qe === "fast" && Ve("y scale is not linear") : qe === "fast" && Ve("x scale is not linear");
              var Ge = ne.maxRowLength(Le), ze = ye.xtype === "scaled" ? "" : ge, We = fe(ye, ze, ve, be, Ge, Me), at = ye.ytype === "scaled" ? "" : Se, ct = fe(ye, at, we, Ee, Le.length, Re);
              $e || (ye._extremes[Me._id] = ee.findExtremes(Me, We), ye._extremes[Re._id] = ee.findExtremes(Re, ct));
              var et = { x: We, y: ct, z: Le, text: ye._text || ye.text, hovertext: ye._hovertext || ye.hovertext };
              if (ye.xperiodalignment && Ce && (et.orig_x = Ce), ye.yperiodalignment && xe && (et.orig_y = xe), ze && ze.length === We.length - 1 && (et.xCenter = ze), at && at.length === ct.length - 1 && (et.yCenter = at), Pe && (et.xRanges = ke.xRanges, et.yRanges = ke.yRanges, et.pts = ke.pts), De || se(pe, ye, { vals: Le, cLetter: "z" }), De && ye.contours && ye.contours.coloring === "heatmap") {
                var dt = { type: ye.type === "contour" ? "heatmap" : "histogram2d", xcalendar: ye.xcalendar, ycalendar: ye.ycalendar };
                et.xfill = fe(dt, ze, ve, be, Ge, Me), et.yfill = fe(dt, at, we, Ee, Le.length, Re);
              }
              return [et];
            };
          }, 93877: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(63821).BADNUM;
            G.exports = function(ae, oe, se, ce) {
              var le, ue, de, fe, he, me;
              function pe(Se) {
                if (te(Se)) return +Se;
              }
              if (oe && oe.transpose) {
                for (le = 0, he = 0; he < ae.length; he++) le = Math.max(le, ae[he].length);
                if (le === 0) return !1;
                de = function(Se) {
                  return Se.length;
                }, fe = function(Se, we, Ee) {
                  return (Se[Ee] || [])[we];
                };
              } else le = ae.length, de = function(Se, we) {
                return Se[we].length;
              }, fe = function(Se, we, Ee) {
                return (Se[we] || [])[Ee];
              };
              var ye = function(Se, we, Ee) {
                return we === ee || Ee === ee ? ee : fe(Se, we, Ee);
              };
              function ge(Se) {
                if (oe && oe.type !== "carpet" && oe.type !== "contourcarpet" && Se && Se.type === "category" && oe["_" + Se._id.charAt(0)].length) {
                  var we = Se._id.charAt(0), Ee = {}, xe = oe["_" + we + "CategoryMap"] || oe[we];
                  for (he = 0; he < xe.length; he++) Ee[xe[he]] = he;
                  return function(Le) {
                    var Qe = Ee[Se._categories[Le]];
                    return Qe + 1 ? Qe : ee;
                  };
                }
                return ne.identity;
              }
              var ve = ge(se), be = ge(ce);
              ce && ce.type === "category" && (le = ce._categories.length);
              var Ce = new Array(le);
              for (he = 0; he < le; he++) for (ue = se && se.type === "category" ? se._categories.length : de(ae, he), Ce[he] = new Array(ue), me = 0; me < ue; me++) Ce[he][me] = pe(ye(ae, be(he), ve(me)));
              return Ce;
            };
          }, 12431: function(G) {
            G.exports = { min: "zmin", max: "zmax" };
          }, 87869: function(G, J, Z) {
            var te = Z(34809), ne = Z(63821).BADNUM, ee = Z(40528);
            G.exports = function(ae, oe, se, ce, le, ue) {
              var de = ae._length, fe = oe.makeCalcdata(ae, ce), he = se.makeCalcdata(ae, le);
              fe = ee(ae, oe, ce, fe).vals, he = ee(ae, se, le, he).vals;
              var me, pe, ye, ge, ve = ae.text, be = ve !== void 0 && te.isArray1D(ve), Ce = ae.hovertext, Se = Ce !== void 0 && te.isArray1D(Ce), we = te.distinctVals(fe), Ee = we.vals, xe = te.distinctVals(he), Le = xe.vals, Qe = [], ke = Le.length, Me = Ee.length;
              for (me = 0; me < ue.length; me++) Qe[me] = te.init2dArray(ke, Me);
              be && (ye = te.init2dArray(ke, Me)), Se && (ge = te.init2dArray(ke, Me));
              var Re = te.init2dArray(ke, Me);
              for (me = 0; me < de; me++) if (fe[me] !== ne && he[me] !== ne) {
                var De = te.findBin(fe[me] + we.minDiff / 2, Ee), Pe = te.findBin(he[me] + xe.minDiff / 2, Le);
                for (pe = 0; pe < ue.length; pe++) {
                  var $e = ae[ue[pe]];
                  Qe[pe][Pe][De] = $e[me], Re[Pe][De] = me;
                }
                be && (ye[Pe][De] = ve[me]), Se && (ge[Pe][De] = Ce[me]);
              }
              for (ae["_" + ce] = Ee, ae["_" + le] = Le, pe = 0; pe < ue.length; pe++) ae["_" + ue[pe]] = Qe[pe];
              be && (ae._text = ye), Se && (ae._hovertext = ge), oe && oe.type === "category" && (ae["_" + ce + "CategoryMap"] = Ee.map(function(qe) {
                return oe._categories[qe];
              })), se && se.type === "category" && (ae["_" + le + "CategoryMap"] = Le.map(function(qe) {
                return se._categories[qe];
              })), ae._after2before = Re;
            };
          }, 52813: function(G, J, Z) {
            var te = Z(34809), ne = Z(86073), ee = Z(63814), ae = Z(99669), oe = Z(44143), se = Z(39356), ce = Z(81658);
            G.exports = function(le, ue, de, fe) {
              function he(me, pe) {
                return te.coerce(le, ue, ce, me, pe);
              }
              ne(le, ue, he, fe) ? (ae(le, ue, fe, he), he("xhoverformat"), he("yhoverformat"), he("text"), he("hovertext"), he("hovertemplate"), ee(he, fe), oe(le, ue, he, fe), he("hoverongaps"), he("connectgaps", te.isArray1D(ue.z) && ue.zsmooth !== !1), se(le, ue, fe, he, { prefix: "", cLetter: "z" }), he("zorder")) : ue.visible = !1;
            };
          }, 78106: function(G, J, Z) {
            var te = Z(34809).maxRowLength;
            G.exports = function(ne) {
              var ee, ae, oe, se, ce, le, ue, de, fe = [], he = {}, me = [], pe = ne[0], ye = [], ge = [0, 0, 0], ve = te(ne);
              for (ae = 0; ae < ne.length; ae++) for (ee = ye, ye = pe, pe = ne[ae + 1] || [], oe = 0; oe < ve; oe++) ye[oe] === void 0 && ((le = (ye[oe - 1] !== void 0 ? 1 : 0) + (ye[oe + 1] !== void 0 ? 1 : 0) + (ee[oe] !== void 0 ? 1 : 0) + (pe[oe] !== void 0 ? 1 : 0)) ? (ae === 0 && le++, oe === 0 && le++, ae === ne.length - 1 && le++, oe === ye.length - 1 && le++, le < 4 && (he[[ae, oe]] = [ae, oe, le]), fe.push([ae, oe, le])) : me.push([ae, oe]));
              for (; me.length; ) {
                for (ue = {}, de = !1, ce = me.length - 1; ce >= 0; ce--) (le = ((he[[(ae = (se = me[ce])[0]) - 1, oe = se[1]]] || ge)[2] + (he[[ae + 1, oe]] || ge)[2] + (he[[ae, oe - 1]] || ge)[2] + (he[[ae, oe + 1]] || ge)[2]) / 20) && (ue[se] = [ae, oe, le], me.splice(ce, 1), de = !0);
                if (!de) throw "findEmpties iterated with no new neighbors";
                for (se in ue) he[se] = ue[se], fe.push(ue[se]);
              }
              return fe.sort(function(be, Ce) {
                return Ce[2] - be[2];
              });
            };
          }, 93125: function(G, J, Z) {
            var te = Z(32141), ne = Z(34809), ee = ne.isArrayOrTypedArray, ae = Z(29714), oe = Z(88856).extractOpts;
            G.exports = function(se, ce, le, ue, de) {
              de || (de = {});
              var fe, he, me, pe, ye = de.isContour, ge = se.cd[0], ve = ge.trace, be = se.xa, Ce = se.ya, Se = ge.x, we = ge.y, Ee = ge.z, xe = ge.xCenter, Le = ge.yCenter, Qe = ge.zmask, ke = ve.zhoverformat, Me = Se, Re = we;
              if (se.index !== !1) {
                try {
                  me = Math.round(se.index[1]), pe = Math.round(se.index[0]);
                } catch {
                  return void ne.error("Error hovering on heatmap, pointNumber must be [row,col], found:", se.index);
                }
                if (me < 0 || me >= Ee[0].length || pe < 0 || pe > Ee.length) return;
              } else {
                if (te.inbox(ce - Se[0], ce - Se[Se.length - 1], 0) > 0 || te.inbox(le - we[0], le - we[we.length - 1], 0) > 0) return;
                if (ye) {
                  var De;
                  for (Me = [2 * Se[0] - Se[1]], De = 1; De < Se.length; De++) Me.push((Se[De] + Se[De - 1]) / 2);
                  for (Me.push([2 * Se[Se.length - 1] - Se[Se.length - 2]]), Re = [2 * we[0] - we[1]], De = 1; De < we.length; De++) Re.push((we[De] + we[De - 1]) / 2);
                  Re.push([2 * we[we.length - 1] - we[we.length - 2]]);
                }
                me = Math.max(0, Math.min(Me.length - 2, ne.findBin(ce, Me))), pe = Math.max(0, Math.min(Re.length - 2, ne.findBin(le, Re)));
              }
              var Pe, $e, qe = be.c2p(Se[me]), Xe = be.c2p(Se[me + 1]), Ve = Ce.c2p(we[pe]), Ze = Ce.c2p(we[pe + 1]);
              ye ? (Pe = ge.orig_x || Se, $e = ge.orig_y || we, Xe = qe, fe = Pe[me], Ze = Ve, he = $e[pe]) : (Pe = ge.orig_x || xe || Se, $e = ge.orig_y || Le || we, fe = xe ? Pe[me] : (Pe[me] + Pe[me + 1]) / 2, he = Le ? $e[pe] : ($e[pe] + $e[pe + 1]) / 2, be && be.type === "category" && (fe = Se[me]), Ce && Ce.type === "category" && (he = we[pe]), ve.zsmooth && (qe = Xe = be.c2p(fe), Ve = Ze = Ce.c2p(he)));
              var Ge = Ee[pe][me];
              if (Qe && !Qe[pe][me] && (Ge = void 0), Ge !== void 0 || ve.hoverongaps) {
                var ze;
                ee(ge.hovertext) && ee(ge.hovertext[pe]) ? ze = ge.hovertext[pe][me] : ee(ge.text) && ee(ge.text[pe]) && (ze = ge.text[pe][me]);
                var We = oe(ve), at = { type: "linear", range: [We.min, We.max], hoverformat: ke, _separators: be._separators, _numFormat: be._numFormat }, ct = ae.tickText(at, Ge, "hover").text;
                return [ne.extendFlat(se, { index: ve._after2before ? ve._after2before[pe][me] : [pe, me], distance: se.maxHoverDistance, spikeDistance: se.maxSpikeDistance, x0: qe, x1: Xe, y0: Ve, y1: Ze, xLabelVal: fe, yLabelVal: he, zLabelVal: Ge, zLabel: ct, text: ze })];
              }
            };
          }, 29251: function(G, J, Z) {
            G.exports = { attributes: Z(81658), supplyDefaults: Z(52813), calc: Z(51670), plot: Z(19236), colorbar: Z(12431), style: Z(12774), hoverPoints: Z(93125), moduleType: "trace", name: "heatmap", basePlotModule: Z(37703), categories: ["cartesian", "svg", "2dMap", "showLegend"], meta: {} };
          }, 69295: function(G, J, Z) {
            var te = Z(34809), ne = [[-1, 0], [1, 0], [0, -1], [0, 1]];
            function ee(oe) {
              return 0.5 - 0.25 * Math.min(1, 0.5 * oe);
            }
            function ae(oe, se, ce) {
              var le, ue, de, fe, he, me, pe, ye, ge, ve, be, Ce, Se, we = 0;
              for (fe = 0; fe < se.length; fe++) {
                for (ue = (le = se[fe])[0], de = le[1], be = oe[ue][de], ve = 0, ge = 0, he = 0; he < 4; he++) (pe = oe[ue + (me = ne[he])[0]]) && (ye = pe[de + me[1]]) !== void 0 && (ve === 0 ? Ce = Se = ye : (Ce = Math.min(Ce, ye), Se = Math.max(Se, ye)), ge++, ve += ye);
                if (ge === 0) throw "iterateInterp2d order is wrong: no defined neighbors";
                oe[ue][de] = ve / ge, be === void 0 ? ge < 4 && (we = 1) : (oe[ue][de] = (1 + ce) * oe[ue][de] - ce * be, Se > Ce && (we = Math.max(we, Math.abs(oe[ue][de] - be) / (Se - Ce))));
              }
              return we;
            }
            G.exports = function(oe, se) {
              var ce, le = 1;
              for (ae(oe, se), ce = 0; ce < se.length && !(se[ce][2] < 4); ce++) ;
              for (se = se.slice(ce), ce = 0; ce < 100 && le > 0.01; ce++) le = ae(oe, se, ee(le));
              return le > 0.01 && te.log("interp2d didn't converge quickly", le), oe;
            };
          }, 63814: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              ne("texttemplate");
              var ae = te.extendFlat({}, ee.font, { color: "auto", size: "auto" });
              te.coerceFont(ne, "textfont", ae);
            };
          }, 80924: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809).isArrayOrTypedArray;
            G.exports = function(ee, ae, oe, se, ce, le) {
              var ue, de, fe, he = [], me = te.traceIs(ee, "contour"), pe = te.traceIs(ee, "histogram"), ye = te.traceIs(ee, "gl2d");
              if (ne(ae) && ae.length > 1 && !pe && le.type !== "category") {
                var ge = ae.length;
                if (!(ge <= ce)) return me ? ae.slice(0, ce) : ae.slice(0, ce + 1);
                if (me || ye) he = Array.from(ae).slice(0, ce);
                else if (ce === 1) he = le.type === "log" ? [0.5 * ae[0], 2 * ae[0]] : [ae[0] - 0.5, ae[0] + 0.5];
                else if (le.type === "log") {
                  for (he = [Math.pow(ae[0], 1.5) / Math.pow(ae[1], 0.5)], fe = 1; fe < ge; fe++) he.push(Math.sqrt(ae[fe - 1] * ae[fe]));
                  he.push(Math.pow(ae[ge - 1], 1.5) / Math.pow(ae[ge - 2], 0.5));
                } else {
                  for (he = [1.5 * ae[0] - 0.5 * ae[1]], fe = 1; fe < ge; fe++) he.push(0.5 * (ae[fe - 1] + ae[fe]));
                  he.push(1.5 * ae[ge - 1] - 0.5 * ae[ge - 2]);
                }
                if (ge < ce) {
                  var ve, be = he[he.length - 1];
                  if (le.type === "log") for (ve = be / he[he.length - 2], fe = ge; fe < ce; fe++) be *= ve, he.push(be);
                  else for (ve = be - he[he.length - 2], fe = ge; fe < ce; fe++) be += ve, he.push(be);
                }
              } else {
                var Ce = ee[le._id.charAt(0) + "calendar"];
                for (ue = pe ? le.r2c(oe, 0, Ce) : ne(ae) && ae.length === 1 ? ae[0] : oe === void 0 ? 0 : (le.type === "log" ? le.d2c : le.r2c)(oe, 0, Ce), de = se || 1, fe = me || ye ? 0 : -0.5; fe < ce; fe++) he.push(ue + de * fe);
              }
              return he;
            };
          }, 19236: function(G, J, Z) {
            var te = Z(45568), ne = Z(65657), ee = Z(33626), ae = Z(62203), oe = Z(29714), se = Z(34809), ce = Z(30635), le = Z(15294), ue = Z(78766), de = Z(88856).extractOpts, fe = Z(88856).makeColorScaleFuncFromTrace, he = Z(62972), me = Z(4530).LINE_SPACING, pe = Z(95544), ye = Z(1837).STYLE, ge = "heatmap-label";
            function ve(Ee) {
              return Ee.selectAll("g." + ge);
            }
            function be(Ee) {
              ve(Ee).remove();
            }
            function Ce(Ee, xe) {
              var Le = xe.length - 2, Qe = se.constrain(se.findBin(Ee, xe), 0, Le), ke = xe[Qe], Me = xe[Qe + 1], Re = se.constrain(Qe + (Ee - ke) / (Me - ke) - 0.5, 0, Le), De = Math.round(Re), Pe = Math.abs(Re - De);
              return Re && Re !== Le && Pe ? { bin0: De, frac: Pe, bin1: Math.round(De + Pe / (Re - De)) } : { bin0: De, bin1: De, frac: 0 };
            }
            function Se(Ee, xe) {
              var Le = xe.length - 1, Qe = se.constrain(se.findBin(Ee, xe), 0, Le), ke = xe[Qe], Me = (Ee - ke) / (xe[Qe + 1] - ke) || 0;
              return Me <= 0 ? { bin0: Qe, bin1: Qe, frac: 0 } : Me < 0.5 ? { bin0: Qe, bin1: Qe + 1, frac: Me } : { bin0: Qe + 1, bin1: Qe, frac: 1 - Me };
            }
            function we(Ee, xe, Le) {
              Ee[xe] = Le[0], Ee[xe + 1] = Le[1], Ee[xe + 2] = Le[2], Ee[xe + 3] = Math.round(255 * Le[3]);
            }
            G.exports = function(Ee, xe, Le, Qe) {
              var ke = xe.xaxis, Me = xe.yaxis;
              se.makeTraceGroups(Qe, Le, "hm").each(function(Re) {
                var De, Pe, $e, qe, Xe, Ve, Ze, Ge, ze = te.select(this), We = Re[0], at = We.trace, ct = at.xgap || 0, et = at.ygap || 0, dt = We.z, Je = We.x, it = We.y, st = We.xCenter, gt = We.yCenter, bt = ee.traceIs(at, "contour"), yt = bt ? "best" : at.zsmooth, St = dt.length, Lt = se.maxRowLength(dt), Rt = !1, vt = !1;
                for (Ve = 0; De === void 0 && Ve < Je.length - 1; ) De = ke.c2p(Je[Ve]), Ve++;
                for (Ve = Je.length - 1; Pe === void 0 && Ve > 0; ) Pe = ke.c2p(Je[Ve]), Ve--;
                for (Pe < De && ($e = Pe, Pe = De, De = $e, Rt = !0), Ve = 0; qe === void 0 && Ve < it.length - 1; ) qe = Me.c2p(it[Ve]), Ve++;
                for (Ve = it.length - 1; Xe === void 0 && Ve > 0; ) Xe = Me.c2p(it[Ve]), Ve--;
                Xe < qe && ($e = qe, qe = Xe, Xe = $e, vt = !0), bt && (st = Je, gt = it, Je = We.xfill, it = We.yfill);
                var $t = "default";
                if (yt ? $t = yt === "best" ? "smooth" : "fast" : at._islinear && ct === 0 && et === 0 && pe() && ($t = "fast"), $t !== "fast") {
                  var Pt = yt === "best" ? 0 : 0.5;
                  De = Math.max(-Pt * ke._length, De), Pe = Math.min((1 + Pt) * ke._length, Pe), qe = Math.max(-Pt * Me._length, qe), Xe = Math.min((1 + Pt) * Me._length, Xe);
                }
                var Xt, Nt, Gt = Math.round(Pe - De), Ft = Math.round(Xe - qe);
                if (De >= ke._length || Pe <= 0 || qe >= Me._length || Xe <= 0) return ze.selectAll("image").data([]).exit().remove(), void be(ze);
                $t === "fast" ? (Xt = Lt, Nt = St) : (Xt = Gt, Nt = Ft);
                var Tt = document.createElement("canvas");
                Tt.width = Xt, Tt.height = Nt;
                var It, Ct, Kt = Tt.getContext("2d", { willReadFrequently: !0 }), Vt = fe(at, { noNumericCheck: !0, returnArray: !0 });
                $t === "fast" ? (It = Rt ? function(Ca) {
                  return Lt - 1 - Ca;
                } : se.identity, Ct = vt ? function(Ca) {
                  return St - 1 - Ca;
                } : se.identity) : (It = function(Ca) {
                  return se.constrain(Math.round(ke.c2p(Je[Ca]) - De), 0, Gt);
                }, Ct = function(Ca) {
                  return se.constrain(Math.round(Me.c2p(it[Ca]) - qe), 0, Ft);
                });
                var Yt, sn, hn, Qt, Ut = Ct(0), Wt = [Ut, Ut], rn = Rt ? 0 : 1, vn = vt ? 0 : 1, dn = 0, Ht = 0, jt = 0, cn = 0;
                function wn(Ca, Sr) {
                  if (Ca !== void 0) {
                    var Dr = Vt(Ca);
                    return Dr[0] = Math.round(Dr[0]), Dr[1] = Math.round(Dr[1]), Dr[2] = Math.round(Dr[2]), dn += Sr, Ht += Dr[0] * Sr, jt += Dr[1] * Sr, cn += Dr[2] * Sr, Dr;
                  }
                  return [0, 0, 0, 0];
                }
                function Rn(Ca, Sr, Dr, si) {
                  var zr = Ca[Dr.bin0];
                  if (zr === void 0) return wn(void 0, 1);
                  var di, Da = Ca[Dr.bin1], la = Sr[Dr.bin0], wa = Sr[Dr.bin1], Wa = Da - zr || 0, Qr = la - zr || 0;
                  return di = Da === void 0 ? wa === void 0 ? 0 : la === void 0 ? 2 * (wa - zr) : 2 * (2 * wa - la - zr) / 3 : wa === void 0 ? la === void 0 ? 0 : 2 * (2 * zr - Da - la) / 3 : la === void 0 ? 2 * (2 * wa - Da - zr) / 3 : wa + zr - Da - la, wn(zr + Dr.frac * Wa + si.frac * (Qr + Dr.frac * di));
                }
                if ($t !== "default") {
                  var Pn, Ie = 0;
                  try {
                    Pn = new Uint8Array(Xt * Nt * 4);
                  } catch {
                    Pn = new Array(Xt * Nt * 4);
                  }
                  if ($t === "smooth") {
                    var rt, ft, pt, wt = st || Je, Bt = gt || it, At = new Array(wt.length), Dt = new Array(Bt.length), Ot = new Array(Gt), un = st ? Se : Ce, yn = gt ? Se : Ce;
                    for (Ve = 0; Ve < wt.length; Ve++) At[Ve] = Math.round(ke.c2p(wt[Ve]) - De);
                    for (Ve = 0; Ve < Bt.length; Ve++) Dt[Ve] = Math.round(Me.c2p(Bt[Ve]) - qe);
                    for (Ve = 0; Ve < Gt; Ve++) Ot[Ve] = un(Ve, At);
                    for (Ze = 0; Ze < Ft; Ze++) for (ft = dt[(rt = yn(Ze, Dt)).bin0], pt = dt[rt.bin1], Ve = 0; Ve < Gt; Ve++, Ie += 4) we(Pn, Ie, Qt = Rn(ft, pt, Ot[Ve], rt));
                  } else for (Ze = 0; Ze < St; Ze++) for (hn = dt[Ze], Wt = Ct(Ze), Ve = 0; Ve < Lt; Ve++) Qt = wn(hn[Ve], 1), we(Pn, Ie = 4 * (Wt * Lt + It(Ve)), Qt);
                  var _n = Kt.createImageData(Xt, Nt);
                  try {
                    _n.data.set(Pn);
                  } catch {
                    var gn = _n.data, bn = gn.length;
                    for (Ze = 0; Ze < bn; Ze++) gn[Ze] = Pn[Ze];
                  }
                  Kt.putImageData(_n, 0, 0);
                } else {
                  var an = Math.floor(ct / 2), Zt = Math.floor(et / 2);
                  for (Ze = 0; Ze < St; Ze++) if (hn = dt[Ze], Wt.reverse(), Wt[vn] = Ct(Ze + 1), Wt[0] !== Wt[1] && Wt[0] !== void 0 && Wt[1] !== void 0) for (Yt = [sn = It(0), sn], Ve = 0; Ve < Lt; Ve++) Yt.reverse(), Yt[rn] = It(Ve + 1), Yt[0] !== Yt[1] && Yt[0] !== void 0 && Yt[1] !== void 0 && (Qt = wn(hn[Ve], (Yt[1] - Yt[0]) * (Wt[1] - Wt[0])), Kt.fillStyle = "rgba(" + Qt.join(",") + ")", Kt.fillRect(Yt[0] + an, Wt[0] + Zt, Yt[1] - Yt[0] - ct, Wt[1] - Wt[0] - et));
                }
                Ht = Math.round(Ht / dn), jt = Math.round(jt / dn), cn = Math.round(cn / dn);
                var Mn = ne("rgb(" + Ht + "," + jt + "," + cn + ")");
                Ee._hmpixcount = (Ee._hmpixcount || 0) + dn, Ee._hmlumcount = (Ee._hmlumcount || 0) + dn * Mn.getLuminance();
                var Ln = ze.selectAll("image").data(Re);
                Ln.enter().append("svg:image").attr({ xmlns: he.svg, preserveAspectRatio: "none" }), Ln.attr({ height: Ft, width: Gt, x: De, y: qe, "xlink:href": Tt.toDataURL("image/png") }), $t !== "fast" || yt || Ln.attr("style", ye), be(ze);
                var In = at.texttemplate;
                if (In) {
                  var kn = de(at), er = { type: "linear", range: [kn.min, kn.max], _separators: ke._separators, _numFormat: ke._numFormat }, jn = at.type === "histogram2dcontour", ur = at.type === "contour", Kn = ur ? St - 1 : St, Jn = ur ? 1 : 0, Lr = ur ? Lt - 1 : Lt, mr = [];
                  for (Ve = ur ? 1 : 0; Ve < Kn; Ve++) {
                    var Ar;
                    if (ur) Ar = We.y[Ve];
                    else if (jn) {
                      if (Ve === 0 || Ve === St - 1) continue;
                      Ar = We.y[Ve];
                    } else if (We.yCenter) Ar = We.yCenter[Ve];
                    else {
                      if (Ve + 1 === St && We.y[Ve + 1] === void 0) continue;
                      Ar = (We.y[Ve] + We.y[Ve + 1]) / 2;
                    }
                    var Yr = Math.round(Me.c2p(Ar));
                    if (!(0 > Yr || Yr > Me._length)) for (Ze = Jn; Ze < Lr; Ze++) {
                      var jr;
                      if (ur) jr = We.x[Ze];
                      else if (jn) {
                        if (Ze === 0 || Ze === Lt - 1) continue;
                        jr = We.x[Ze];
                      } else if (We.xCenter) jr = We.xCenter[Ze];
                      else {
                        if (Ze + 1 === Lt && We.x[Ze + 1] === void 0) continue;
                        jr = (We.x[Ze] + We.x[Ze + 1]) / 2;
                      }
                      var Mr = Math.round(ke.c2p(jr));
                      if (!(0 > Mr || Mr > ke._length)) {
                        var Wr = le({ x: jr, y: Ar }, at, Ee._fullLayout);
                        Wr.x = jr, Wr.y = Ar;
                        var Bi = We.z[Ve][Ze];
                        Bi === void 0 ? (Wr.z = "", Wr.zLabel = "") : (Wr.z = Bi, Wr.zLabel = oe.tickText(er, Bi, "hover").text);
                        var gi = We.text && We.text[Ve] && We.text[Ve][Ze];
                        gi !== void 0 && gi !== !1 || (gi = ""), Wr.text = gi;
                        var Zr = se.texttemplateString(In, Wr, Ee._fullLayout._d3locale, Wr, at._meta || {});
                        if (Zr) {
                          var fi = Zr.split("<br>"), ri = fi.length, bi = 0;
                          for (Ge = 0; Ge < ri; Ge++) bi = Math.max(bi, fi[Ge].length);
                          mr.push({ l: ri, c: bi, t: Zr, x: Mr, y: Yr, z: Bi });
                        }
                      }
                    }
                  }
                  var Zi = at.textfont, $i = Zi.size, Br = Ee._fullLayout.font.size;
                  if (!$i || $i === "auto") {
                    var Pr = 1 / 0, ei = 1 / 0, Ci = 0, oa = 0;
                    for (Ge = 0; Ge < mr.length; Ge++) {
                      var ka = mr[Ge];
                      if (Ci = Math.max(Ci, ka.l), oa = Math.max(oa, ka.c), Ge < mr.length - 1) {
                        var aa = mr[Ge + 1], hi = Math.abs(aa.x - ka.x), Gr = Math.abs(aa.y - ka.y);
                        hi && (Pr = Math.min(Pr, hi)), Gr && (ei = Math.min(ei, Gr));
                      }
                    }
                    isFinite(Pr) && isFinite(ei) ? (Pr -= ct, ei -= et, Pr /= oa, ei /= Ci, Pr /= me / 2, ei /= me, $i = Math.min(Math.floor(Pr), Math.floor(ei), Br)) : $i = Br;
                  }
                  if ($i <= 0 || !isFinite($i)) return;
                  ve(ze).data(mr).enter().append("g").classed(ge, 1).append("text").attr("text-anchor", "middle").each(function(Ca) {
                    var Sr = te.select(this), Dr = Zi.color;
                    Dr && Dr !== "auto" || (Dr = ue.contrast(Ca.z === void 0 ? Ee._fullLayout.plot_bgcolor : "rgba(" + Vt(Ca.z).join() + ")")), Sr.attr("data-notex", 1).call(ce.positionText, function(si) {
                      return si.x;
                    }(Ca), function(si) {
                      return si.y - $i * (si.l * me / 2 - 1);
                    }(Ca)).call(ae.font, { family: Zi.family, size: $i, color: Dr, weight: Zi.weight, style: Zi.style, variant: Zi.variant, textcase: Zi.textcase, lineposition: Zi.lineposition, shadow: Zi.shadow }).text(Ca.t).call(ce.convertToTspans, Ee);
                  });
                }
              });
            };
          }, 12774: function(G, J, Z) {
            var te = Z(45568);
            G.exports = function(ne) {
              te.select(ne).selectAll(".hm image").style("opacity", function(ee) {
                return ee.trace.opacity;
              });
            };
          }, 44143: function(G) {
            G.exports = function(J, Z, te) {
              te("zsmooth") === !1 && (te("xgap"), te("ygap")), te("zhoverformat");
            };
          }, 86073: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(33626);
            function ae(oe, se) {
              var ce = se(oe);
              return (ce ? se(oe + "type", "array") : "scaled") === "scaled" && (se(oe + "0"), se("d" + oe)), ce;
            }
            G.exports = function(oe, se, ce, le, ue, de) {
              var fe, he, me = ce("z");
              if (ue = ue || "x", de = de || "y", me === void 0 || !me.length) return 0;
              if (ne.isArray1D(me)) {
                fe = ce(ue), he = ce(de);
                var pe = ne.minRowLength(fe), ye = ne.minRowLength(he);
                if (pe === 0 || ye === 0) return 0;
                se._length = Math.min(pe, ye, me.length);
              } else {
                if (fe = ae(ue, ce), he = ae(de, ce), !function(ge) {
                  for (var ve, be = !0, Ce = !1, Se = !1, we = 0; we < ge.length; we++) {
                    if (ve = ge[we], !ne.isArrayOrTypedArray(ve)) {
                      be = !1;
                      break;
                    }
                    ve.length > 0 && (Ce = !0);
                    for (var Ee = 0; Ee < ve.length; Ee++) if (te(ve[Ee])) {
                      Se = !0;
                      break;
                    }
                  }
                  return be && Ce && Se;
                }(me)) return 0;
                ce("transpose"), se._length = null;
              }
              return oe.type === "heatmapgl" || ee.getComponentMethod("calendars", "handleTraceDefaults")(oe, se, [ue, de], le), !0;
            };
          }, 29751: function(G, J, Z) {
            for (var te = Z(81658), ne = Z(87163), ee = Z(93049).extendFlat, ae = Z(13582).overrideAll, oe = ["z", "x", "x0", "dx", "y", "y0", "dy", "text", "transpose", "xtype", "ytype"], se = {}, ce = 0; ce < oe.length; ce++) {
              var le = oe[ce];
              se[le] = te[le];
            }
            se.zsmooth = { valType: "enumerated", values: ["fast", !1], dflt: "fast", editType: "calc" }, ee(se, ne("", { cLetter: "z", autoColorDflt: !1 })), G.exports = ae(se, "calc", "nested");
          }, 89987: function(G, J, Z) {
            var te = Z(99098).gl_heatmap2d, ne = Z(29714), ee = Z(55010);
            function ae(se, ce) {
              this.scene = se, this.uid = ce, this.type = "heatmapgl", this.name = "", this.hoverinfo = "all", this.xData = [], this.yData = [], this.zData = [], this.textLabels = [], this.idToIndex = [], this.bounds = [0, 0, 0, 0], this.options = { zsmooth: "fast", z: [], x: [], y: [], shape: [0, 0], colorLevels: [0], colorValues: [0, 0, 0, 1] }, this.heatmap = te(se.glplot, this.options), this.heatmap._trace = this;
            }
            var oe = ae.prototype;
            oe.handlePick = function(se) {
              var ce = this.options, le = ce.shape, ue = se.pointId, de = ue % le[0], fe = Math.floor(ue / le[0]), he = ue;
              return { trace: this, dataCoord: se.dataCoord, traceCoord: [ce.x[de], ce.y[fe], ce.z[he]], textLabel: this.textLabels[ue], name: this.name, pointIndex: [fe, de], hoverinfo: this.hoverinfo };
            }, oe.update = function(se, ce) {
              var le = ce[0];
              this.index = se.index, this.name = se.name, this.hoverinfo = se.hoverinfo;
              var ue = le.z;
              this.options.z = [].concat.apply([], ue);
              var de = ue[0].length, fe = ue.length;
              this.options.shape = [de, fe], this.options.x = le.x, this.options.y = le.y, this.options.zsmooth = se.zsmooth;
              var he = function(ve) {
                for (var be = ve.colorscale, Ce = ve.zmin, Se = ve.zmax, we = be.length, Ee = new Array(we), xe = new Array(4 * we), Le = 0; Le < we; Le++) {
                  var Qe = be[Le], ke = ee(Qe[1]);
                  Ee[Le] = Ce + Qe[0] * (Se - Ce);
                  for (var Me = 0; Me < 4; Me++) xe[4 * Le + Me] = ke[Me];
                }
                return { colorLevels: Ee, colorValues: xe };
              }(se);
              this.options.colorLevels = he.colorLevels, this.options.colorValues = he.colorValues, this.textLabels = [].concat.apply([], se.text), this.heatmap.update(this.options);
              var me, pe, ye = this.scene.xaxis, ge = this.scene.yaxis;
              se.zsmooth === !1 && (me = { ppad: le.x[1] - le.x[0] }, pe = { ppad: le.y[1] - le.y[0] }), se._extremes[ye._id] = ne.findExtremes(ye, le.x, me), se._extremes[ge._id] = ne.findExtremes(ge, le.y, pe);
            }, oe.dispose = function() {
              this.heatmap.dispose();
            }, G.exports = function(se, ce, le) {
              var ue = new ae(se, ce.uid);
              return ue.update(ce, le), ue;
            };
          }, 51312: function(G, J, Z) {
            var te = Z(34809), ne = Z(86073), ee = Z(39356), ae = Z(29751);
            G.exports = function(oe, se, ce, le) {
              function ue(de, fe) {
                return te.coerce(oe, se, ae, de, fe);
              }
              ne(oe, se, ue, le) ? (ue("text"), ue("zsmooth"), ee(oe, se, le, ue, { prefix: "", cLetter: "z" })) : se.visible = !1;
            };
          }, 72892: function(G, J, Z) {
            G.exports = { attributes: Z(29751), supplyDefaults: Z(51312), colorbar: Z(12431), calc: Z(51670), plot: Z(89987), moduleType: "trace", name: "heatmapgl", basePlotModule: Z(24585), categories: ["gl", "gl2d", "2dMap"], meta: {} };
          }, 16160: function(G, J, Z) {
            var te = Z(81481), ne = Z(80712).axisHoverFormat, ee = Z(3208).rb, ae = Z(3208).ay, oe = Z(80337), se = Z(64766), ce = Z(39732), le = Z(93049).extendFlat;
            G.exports = { x: { valType: "data_array", editType: "calc+clearAxisTypes" }, y: { valType: "data_array", editType: "calc+clearAxisTypes" }, xhoverformat: ne("x"), yhoverformat: ne("y"), text: le({}, te.text, {}), hovertext: le({}, te.hovertext, {}), orientation: te.orientation, histfunc: { valType: "enumerated", values: ["count", "sum", "avg", "min", "max"], dflt: "count", editType: "calc" }, histnorm: { valType: "enumerated", values: ["", "percent", "probability", "density", "probability density"], dflt: "", editType: "calc" }, cumulative: { enabled: { valType: "boolean", dflt: !1, editType: "calc" }, direction: { valType: "enumerated", values: ["increasing", "decreasing"], dflt: "increasing", editType: "calc" }, currentbin: { valType: "enumerated", values: ["include", "exclude", "half"], dflt: "include", editType: "calc" }, editType: "calc" }, nbinsx: { valType: "integer", min: 0, dflt: 0, editType: "calc" }, xbins: se("x", !0), nbinsy: { valType: "integer", min: 0, dflt: 0, editType: "calc" }, ybins: se("y", !0), autobinx: { valType: "boolean", dflt: null, editType: "calc" }, autobiny: { valType: "boolean", dflt: null, editType: "calc" }, bingroup: { valType: "string", dflt: "", editType: "calc" }, hovertemplate: ee({}, { keys: ce.eventDataKeys }), texttemplate: ae({ arrayOk: !1, editType: "plot" }, { keys: ["label", "value"] }), textposition: le({}, te.textposition, { arrayOk: !1 }), textfont: oe({ arrayOk: !1, editType: "plot", colorEditType: "style" }), outsidetextfont: oe({ arrayOk: !1, editType: "plot", colorEditType: "style" }), insidetextfont: oe({ arrayOk: !1, editType: "plot", colorEditType: "style" }), insidetextanchor: te.insidetextanchor, textangle: te.textangle, cliponaxis: te.cliponaxis, constraintext: te.constraintext, marker: te.marker, offsetgroup: te.offsetgroup, alignmentgroup: te.alignmentgroup, selected: te.selected, unselected: te.unselected, _deprecated: { bardir: te._deprecated.bardir }, zorder: te.zorder };
          }, 48198: function(G) {
            G.exports = function(J, Z) {
              for (var te = J.length, ne = 0, ee = 0; ee < te; ee++) Z[ee] ? (J[ee] /= Z[ee], ne += J[ee]) : J[ee] = null;
              return ne;
            };
          }, 64766: function(G) {
            G.exports = function(J, Z) {
              return { start: { valType: "any", editType: "calc" }, end: { valType: "any", editType: "calc" }, size: { valType: "any", editType: "calc" }, editType: "calc" };
            };
          }, 34870: function(G, J, Z) {
            var te = Z(10721);
            G.exports = { count: function(ne, ee, ae) {
              return ae[ne]++, 1;
            }, sum: function(ne, ee, ae, oe) {
              var se = oe[ee];
              return te(se) ? (se = Number(se), ae[ne] += se, se) : 0;
            }, avg: function(ne, ee, ae, oe, se) {
              var ce = oe[ee];
              return te(ce) && (ce = Number(ce), ae[ne] += ce, se[ne]++), 0;
            }, min: function(ne, ee, ae, oe) {
              var se = oe[ee];
              if (te(se)) {
                if (se = Number(se), !te(ae[ne])) return ae[ne] = se, se;
                if (ae[ne] > se) {
                  var ce = se - ae[ne];
                  return ae[ne] = se, ce;
                }
              }
              return 0;
            }, max: function(ne, ee, ae, oe) {
              var se = oe[ee];
              if (te(se)) {
                if (se = Number(se), !te(ae[ne])) return ae[ne] = se, se;
                if (ae[ne] < se) {
                  var ce = se - ae[ne];
                  return ae[ne] = se, ce;
                }
              }
              return 0;
            } };
          }, 64852: function(G, J, Z) {
            var te = Z(63821), ne = te.ONEAVGYEAR, ee = te.ONEAVGMONTH, ae = te.ONEDAY, oe = te.ONEHOUR, se = te.ONEMIN, ce = te.ONESEC, le = Z(29714).tickIncrement;
            function ue(me, pe, ye, ge) {
              if (me * pe <= 0) return 1 / 0;
              for (var ve = Math.abs(pe - me), be = ye.type === "date", Ce = de(ve, be), Se = 0; Se < 10; Se++) {
                var we = de(80 * Ce, be);
                if (Ce === we || !fe(we, me, pe, be, ye, ge)) break;
                Ce = we;
              }
              return Ce;
            }
            function de(me, pe) {
              return pe && me > ce ? me > ae ? me > 1.1 * ne ? ne : me > 1.1 * ee ? ee : ae : me > oe ? oe : me > se ? se : ce : Math.pow(10, Math.floor(Math.log(me) / Math.LN10));
            }
            function fe(me, pe, ye, ge, ve, be) {
              if (ge && me > ae) {
                var Ce = he(pe, ve, be), Se = he(ye, ve, be), we = me === ne ? 0 : 1;
                return Ce[we] !== Se[we];
              }
              return Math.floor(ye / me) - Math.floor(pe / me) > 0.1;
            }
            function he(me, pe, ye) {
              var ge = pe.c2d(me, ne, ye).split("-");
              return ge[0] === "" && (ge.unshift(), ge[0] = "-" + ge[0]), ge;
            }
            G.exports = function(me, pe, ye, ge, ve) {
              var be, Ce, Se = -1.1 * pe, we = -0.1 * pe, Ee = me - we, xe = ye[0], Le = ye[1], Qe = Math.min(ue(xe + we, xe + Ee, ge, ve), ue(Le + we, Le + Ee, ge, ve)), ke = Math.min(ue(xe + Se, xe + we, ge, ve), ue(Le + Se, Le + we, ge, ve));
              if (Qe > ke && ke < Math.abs(Le - xe) / 4e3 ? (be = Qe, Ce = !1) : (be = Math.min(Qe, ke), Ce = !0), ge.type === "date" && be > ae) {
                var Me = be === ne ? 1 : 6, Re = be === ne ? "M12" : "M1";
                return function(De, Pe) {
                  var $e = ge.c2d(De, ne, ve), qe = $e.indexOf("-", Me);
                  qe > 0 && ($e = $e.substr(0, qe));
                  var Xe = ge.d2c($e, 0, ve);
                  if (Xe < De) {
                    var Ve = le(Xe, Re, !1, ve);
                    (Xe + Ve) / 2 < De + me && (Xe = Ve);
                  }
                  return Pe && Ce ? le(Xe, Re, !0, ve) : Xe;
                };
              }
              return function(De, Pe) {
                var $e = be * Math.round(De / be);
                return $e + be / 10 < De && $e + 0.9 * be < De + me && ($e += be), Pe && Ce && ($e -= be), $e;
              };
            };
          }, 53616: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(33626), ae = Z(29714), oe = Z(35374), se = Z(34870), ce = Z(58665), le = Z(48198), ue = Z(64852);
            function de(fe, he, me, pe, ye) {
              var ge, ve, be, Ce, Se, we, Ee, xe = pe + "bins", Le = fe._fullLayout, Qe = he["_" + pe + "bingroup"], ke = Le._histogramBinOpts[Qe], Me = Le.barmode === "overlay", Re = function(vt) {
                return me.r2c(vt, 0, Ce);
              }, De = function(vt) {
                return me.c2r(vt, 0, Ce);
              }, Pe = me.type === "date" ? function(vt) {
                return vt || vt === 0 ? ne.cleanDate(vt, null, Ce) : null;
              } : function(vt) {
                return te(vt) ? Number(vt) : null;
              };
              function $e(vt, $t, Pt) {
                $t[vt + "Found"] ? ($t[vt] = Pe($t[vt]), $t[vt] === null && ($t[vt] = Pt[vt])) : (we[vt] = $t[vt] = Pt[vt], ne.nestedProperty(ve[0], xe + "." + vt).set(Pt[vt]));
              }
              if (he["_" + pe + "autoBinFinished"]) delete he["_" + pe + "autoBinFinished"];
              else {
                ve = ke.traces;
                var qe = [], Xe = !0, Ve = !1, Ze = !1;
                for (ge = 0; ge < ve.length; ge++) if ((be = ve[ge]).visible) {
                  var Ge = ke.dirs[ge];
                  Se = be["_" + Ge + "pos0"] = me.makeCalcdata(be, Ge), qe = ne.concat(qe, Se), delete be["_" + pe + "autoBinFinished"], he.visible === !0 && (Xe ? Xe = !1 : (delete be._autoBin, be["_" + pe + "autoBinFinished"] = 1), ee.traceIs(be, "2dMap") && (Ve = !0), be.type === "histogram2dcontour" && (Ze = !0));
                }
                Ce = ve[0][pe + "calendar"];
                var ze = ae.autoBin(qe, me, ke.nbins, Ve, Ce, ke.sizeFound && ke.size), We = ve[0]._autoBin = {};
                if (we = We[ke.dirs[0]] = {}, Ze && (ke.size || (ze.start = De(ae.tickIncrement(Re(ze.start), ze.size, !0, Ce))), ke.end === void 0 && (ze.end = De(ae.tickIncrement(Re(ze.end), ze.size, !1, Ce)))), Me && !ee.traceIs(he, "2dMap") && ze._dataSpan === 0 && me.type !== "category" && me.type !== "multicategory" && he.bingroup === "" && he.xbins === void 0) {
                  if (ye) return [ze, Se, !0];
                  ze = function(vt, $t, Pt, Xt, Nt) {
                    var Gt, Ft, Tt, It = vt._fullLayout, Ct = function(Ht, jt) {
                      for (var cn = jt.xaxis, wn = jt.yaxis, Rn = jt.orientation, Pn = [], Ie = Ht._fullData, rt = 0; rt < Ie.length; rt++) {
                        var ft = Ie[rt];
                        ft.type === "histogram" && ft.visible === !0 && ft.orientation === Rn && ft.xaxis === cn && ft.yaxis === wn && Pn.push(ft);
                      }
                      return Pn;
                    }(vt, $t), Kt = !1, Vt = 1 / 0, Yt = [$t];
                    for (Gt = 0; Gt < Ct.length; Gt++) if ((Ft = Ct[Gt]) === $t) Kt = !0;
                    else if (Kt) {
                      var sn = de(vt, Ft, Pt, Xt, !0), hn = sn[0], Qt = sn[2];
                      Ft["_" + Xt + "autoBinFinished"] = 1, Ft["_" + Xt + "pos0"] = sn[1], Qt ? Yt.push(Ft) : Vt = Math.min(Vt, hn.size);
                    } else Tt = It._histogramBinOpts[Ft["_" + Xt + "bingroup"]], Vt = Math.min(Vt, Tt.size || Ft[Nt].size);
                    var Ut = new Array(Yt.length);
                    for (Gt = 0; Gt < Yt.length; Gt++) for (var Wt = Yt[Gt]["_" + Xt + "pos0"], rn = 0; rn < Wt.length; rn++) if (Wt[rn] !== void 0) {
                      Ut[Gt] = Wt[rn];
                      break;
                    }
                    for (isFinite(Vt) || (Vt = ne.distinctVals(Ut).minDiff), Gt = 0; Gt < Yt.length; Gt++) {
                      var vn = (Ft = Yt[Gt])[Xt + "calendar"], dn = { start: Pt.c2r(Ut[Gt] - Vt / 2, 0, vn), end: Pt.c2r(Ut[Gt] + Vt / 2, 0, vn), size: Vt };
                      Ft._input[Nt] = Ft[Nt] = dn, (Tt = It._histogramBinOpts[Ft["_" + Xt + "bingroup"]]) && ne.extendFlat(Tt, dn);
                    }
                    return $t[Nt];
                  }(fe, he, me, pe, xe);
                }
                (Ee = be.cumulative || {}).enabled && Ee.currentbin !== "include" && (Ee.direction === "decreasing" ? ze.start = De(ae.tickIncrement(Re(ze.start), ze.size, !0, Ce)) : ze.end = De(ae.tickIncrement(Re(ze.end), ze.size, !1, Ce))), ke.size = ze.size, ke.sizeFound || (we.size = ze.size, ne.nestedProperty(ve[0], xe + ".size").set(ze.size)), $e("start", ke, ze), $e("end", ke, ze);
              }
              Se = he["_" + pe + "pos0"], delete he["_" + pe + "pos0"];
              var at = he._input[xe] || {}, ct = ne.extendFlat({}, ke), et = ke.start, dt = me.r2l(at.start), Je = dt !== void 0;
              if ((ke.startFound || Je) && dt !== me.r2l(et)) {
                var it = Je ? dt : ne.aggNums(Math.min, null, Se), st = { type: me.type === "category" || me.type === "multicategory" ? "linear" : me.type, r2l: me.r2l, dtick: ke.size, tick0: et, calendar: Ce, range: [it, ae.tickIncrement(it, ke.size, !1, Ce)].map(me.l2r) }, gt = ae.tickFirst(st);
                gt > me.r2l(it) && (gt = ae.tickIncrement(gt, ke.size, !0, Ce)), ct.start = me.l2r(gt), Je || ne.nestedProperty(he, xe + ".start").set(ct.start);
              }
              var bt = ke.end, yt = me.r2l(at.end), St = yt !== void 0;
              if ((ke.endFound || St) && yt !== me.r2l(bt)) {
                var Lt = St ? yt : ne.aggNums(Math.max, null, Se);
                ct.end = me.l2r(Lt), St || ne.nestedProperty(he, xe + ".start").set(ct.end);
              }
              var Rt = "autobin" + pe;
              return he._input[Rt] === !1 && (he._input[xe] = ne.extendFlat({}, he[xe] || {}), delete he._input[Rt], delete he[Rt]), [ct, Se];
            }
            G.exports = { calc: function(fe, he) {
              var me, pe, ye, ge, ve = [], be = [], Ce = he.orientation === "h", Se = ae.getFromId(fe, Ce ? he.yaxis : he.xaxis), we = Ce ? "y" : "x", Ee = { x: "y", y: "x" }[we], xe = he[we + "calendar"], Le = he.cumulative, Qe = de(fe, he, Se, we), ke = Qe[0], Me = Qe[1], Re = typeof ke.size == "string", De = [], Pe = Re ? De : ke, $e = [], qe = [], Xe = [], Ve = 0, Ze = he.histnorm, Ge = he.histfunc, ze = Ze.indexOf("density") !== -1;
              Le.enabled && ze && (Ze = Ze.replace(/ ?density$/, ""), ze = !1);
              var We, at = Ge === "max" || Ge === "min" ? null : 0, ct = se.count, et = ce[Ze], dt = !1, Je = function(Ft) {
                return Se.r2c(Ft, 0, xe);
              };
              for (ne.isArrayOrTypedArray(he[Ee]) && Ge !== "count" && (We = he[Ee], dt = Ge === "avg", ct = se[Ge]), me = Je(ke.start), ye = Je(ke.end) + (me - ae.tickIncrement(me, ke.size, !1, xe)) / 1e6; me < ye && ve.length < 1e6 && (pe = ae.tickIncrement(me, ke.size, !1, xe), ve.push((me + pe) / 2), be.push(at), Xe.push([]), De.push(me), ze && $e.push(1 / (pe - me)), dt && qe.push(0), !(pe <= me)); ) me = pe;
              De.push(me), Re || Se.type !== "date" || (Pe = { start: Je(Pe.start), end: Je(Pe.end), size: Pe.size }), fe._fullLayout._roundFnOpts || (fe._fullLayout._roundFnOpts = {});
              var it = he["_" + we + "bingroup"], st = { leftGap: 1 / 0, rightGap: 1 / 0 };
              it && (fe._fullLayout._roundFnOpts[it] || (fe._fullLayout._roundFnOpts[it] = st), st = fe._fullLayout._roundFnOpts[it]);
              var gt, bt = be.length, yt = !0, St = st.leftGap, Lt = st.rightGap, Rt = {};
              for (me = 0; me < Me.length; me++) {
                var vt = Me[me];
                (ge = ne.findBin(vt, Pe)) >= 0 && ge < bt && (Ve += ct(ge, me, be, We, qe), yt && Xe[ge].length && vt !== Me[Xe[ge][0]] && (yt = !1), Xe[ge].push(me), Rt[me] = ge, St = Math.min(St, vt - De[ge]), Lt = Math.min(Lt, De[ge + 1] - vt));
              }
              st.leftGap = St, st.rightGap = Lt, yt || (gt = function(Ft, Tt) {
                return function() {
                  var It = fe._fullLayout._roundFnOpts[it];
                  return ue(It.leftGap, It.rightGap, De, Se, xe)(Ft, Tt);
                };
              }), dt && (Ve = le(be, qe)), et && et(be, Ve, $e), Le.enabled && function(Ft, Tt, It) {
                var Ct, Kt, Vt;
                function Yt(hn) {
                  Vt = Ft[hn], Ft[hn] /= 2;
                }
                function sn(hn) {
                  Kt = Ft[hn], Ft[hn] = Vt + Kt / 2, Vt += Kt;
                }
                if (It === "half") if (Tt === "increasing") for (Yt(0), Ct = 1; Ct < Ft.length; Ct++) sn(Ct);
                else for (Yt(Ft.length - 1), Ct = Ft.length - 2; Ct >= 0; Ct--) sn(Ct);
                else if (Tt === "increasing") {
                  for (Ct = 1; Ct < Ft.length; Ct++) Ft[Ct] += Ft[Ct - 1];
                  It === "exclude" && (Ft.unshift(0), Ft.pop());
                } else {
                  for (Ct = Ft.length - 2; Ct >= 0; Ct--) Ft[Ct] += Ft[Ct + 1];
                  It === "exclude" && (Ft.push(0), Ft.shift());
                }
              }(be, Le.direction, Le.currentbin);
              var $t = Math.min(ve.length, be.length), Pt = [], Xt = 0, Nt = $t - 1;
              for (me = 0; me < $t; me++) if (be[me]) {
                Xt = me;
                break;
              }
              for (me = $t - 1; me >= Xt; me--) if (be[me]) {
                Nt = me;
                break;
              }
              for (me = Xt; me <= Nt; me++) if (te(ve[me]) && te(be[me])) {
                var Gt = { p: ve[me], s: be[me], b: 0 };
                Le.enabled || (Gt.pts = Xe[me], yt ? Gt.ph0 = Gt.ph1 = Xe[me].length ? Me[Xe[me][0]] : ve[me] : (he._computePh = !0, Gt.ph0 = gt(De[me]), Gt.ph1 = gt(De[me + 1], !0))), Pt.push(Gt);
              }
              return Pt.length === 1 && (Pt[0].width1 = ae.tickIncrement(Pt[0].p, ke.size, !1, xe) - Pt[0].p), oe(Pt, he), ne.isArrayOrTypedArray(he.selectedpoints) && ne.tagSelected(Pt, he, Rt), Pt;
            }, calcAllAutoBins: de };
          }, 39732: function(G) {
            G.exports = { eventDataKeys: ["binNumber"] };
          }, 83380: function(G, J, Z) {
            var te = Z(34809), ne = Z(5975), ee = Z(33626).traceIs, ae = Z(36301), oe = Z(17550).validateCornerradius, se = te.nestedProperty, ce = Z(84391).getAxisGroup, le = [{ aStr: { x: "xbins.start", y: "ybins.start" }, name: "start" }, { aStr: { x: "xbins.end", y: "ybins.end" }, name: "end" }, { aStr: { x: "xbins.size", y: "ybins.size" }, name: "size" }, { aStr: { x: "nbinsx", y: "nbinsy" }, name: "nbins" }], ue = ["x", "y"];
            G.exports = function(de, fe) {
              var he, me, pe, ye, ge, ve, be, Ce = fe._histogramBinOpts = {}, Se = [], we = {}, Ee = [];
              function xe(at, ct) {
                return te.coerce(he._input, he, he._module.attributes, at, ct);
              }
              function Le(at) {
                return at.orientation === "v" ? "x" : "y";
              }
              function Qe(at, ct, et) {
                var dt = at.uid + "__" + et;
                ct || (ct = dt);
                var Je = function(bt, yt) {
                  return ne.getFromTrace({ _fullLayout: fe }, bt, yt).type;
                }(at, et), it = at[et + "calendar"] || "", st = Ce[ct], gt = !0;
                st && (Je === st.axType && it === st.calendar ? (gt = !1, st.traces.push(at), st.dirs.push(et)) : (ct = dt, Je !== st.axType && te.warn(["Attempted to group the bins of trace", at.index, "set on a", "type:" + Je, "axis", "with bins on", "type:" + st.axType, "axis."].join(" ")), it !== st.calendar && te.warn(["Attempted to group the bins of trace", at.index, "set with a", it, "calendar", "with bins", st.calendar ? "on a " + st.calendar + " calendar" : "w/o a set calendar"].join(" ")))), gt && (Ce[ct] = { traces: [at], dirs: [et], axType: Je, calendar: at[et + "calendar"] || "" }), at["_" + et + "bingroup"] = ct;
              }
              for (ge = 0; ge < de.length; ge++) if (he = de[ge], ee(he, "histogram")) {
                if (Se.push(he), delete he._xautoBinFinished, delete he._yautoBinFinished, he.type === "histogram") {
                  var ke = xe("marker.cornerradius", fe.barcornerradius);
                  he.marker && (he.marker.cornerradius = oe(ke));
                }
                ee(he, "2dMap") || ae(he._input, he, fe, xe);
              }
              var Me = fe._alignmentOpts || {};
              for (ge = 0; ge < Se.length; ge++) {
                if (he = Se[ge], pe = "", !ee(he, "2dMap")) {
                  if (ye = Le(he), fe.barmode === "group" && he.alignmentgroup) {
                    var Re = he[ye + "axis"], De = ce(fe, Re) + he.orientation;
                    (Me[De] || {})[he.alignmentgroup] && (pe = De);
                  }
                  pe || fe.barmode === "overlay" || (pe = ce(fe, he.xaxis) + ce(fe, he.yaxis) + Le(he));
                }
                pe ? (we[pe] || (we[pe] = []), we[pe].push(he)) : Ee.push(he);
              }
              for (pe in we) if ((me = we[pe]).length !== 1) {
                var Pe = !1;
                for (me.length && (he = me[0], Pe = xe("bingroup")), pe = Pe || pe, ge = 0; ge < me.length; ge++) {
                  var $e = (he = me[ge])._input.bingroup;
                  $e && $e !== pe && te.warn(["Trace", he.index, "must match", "within bingroup", pe + ".", "Ignoring its bingroup:", $e, "setting."].join(" ")), he.bingroup = pe, Qe(he, pe, Le(he));
                }
              } else Ee.push(me[0]);
              for (ge = 0; ge < Ee.length; ge++) {
                he = Ee[ge];
                var qe = xe("bingroup");
                if (ee(he, "2dMap")) for (be = 0; be < 2; be++) {
                  var Xe = xe((ye = ue[be]) + "bingroup", qe ? qe + "__" + ye : null);
                  Qe(he, Xe, ye);
                }
                else Qe(he, qe, Le(he));
              }
              for (pe in Ce) {
                var Ve = Ce[pe];
                for (me = Ve.traces, ve = 0; ve < le.length; ve++) {
                  var Ze, Ge, ze = le[ve], We = ze.name;
                  if (We !== "nbins" || !Ve.sizeFound) {
                    for (ge = 0; ge < me.length; ge++) {
                      if (he = me[ge], ye = Ve.dirs[ge], Ze = ze.aStr[ye], se(he._input, Ze).get() !== void 0) {
                        Ve[We] = xe(Ze), Ve[We + "Found"] = !0;
                        break;
                      }
                      (Ge = (he._autoBin || {})[ye] || {})[We] && se(he, Ze).set(Ge[We]);
                    }
                    if (We === "start" || We === "end") for (; ge < me.length; ge++) (he = me[ge])["_" + ye + "bingroup"] && xe(Ze, (Ge = (he._autoBin || {})[ye] || {})[We]);
                    We !== "nbins" || Ve.sizeFound || Ve.nbinsFound || (he = me[0], Ve[We] = xe(Ze));
                  }
                }
              }
            };
          }, 85079: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(78766), ae = Z(17550).handleText, oe = Z(59760), se = Z(16160);
            G.exports = function(ce, le, ue, de) {
              function fe(we, Ee) {
                return ne.coerce(ce, le, se, we, Ee);
              }
              var he = fe("x"), me = fe("y");
              fe("cumulative.enabled") && (fe("cumulative.direction"), fe("cumulative.currentbin")), fe("text");
              var pe = fe("textposition");
              ae(ce, le, de, fe, pe, { moduleHasSelected: !0, moduleHasUnselected: !0, moduleHasConstrain: !0, moduleHasCliponaxis: !0, moduleHasTextangle: !0, moduleHasInsideanchor: !0 }), fe("hovertext"), fe("hovertemplate"), fe("xhoverformat"), fe("yhoverformat");
              var ye = fe("orientation", me && !he ? "h" : "v"), ge = ye === "v" ? "x" : "y", ve = ye === "v" ? "y" : "x", be = he && me ? Math.min(ne.minRowLength(he) && ne.minRowLength(me)) : ne.minRowLength(le[ge] || []);
              if (be) {
                le._length = be, te.getComponentMethod("calendars", "handleTraceDefaults")(ce, le, ["x", "y"], de), le[ve] && fe("histfunc"), fe("histnorm"), fe("autobin" + ge), oe(ce, le, fe, ue, de), ne.coerceSelectionMarkerOpacity(le, fe);
                var Ce = (le.marker.line || {}).color, Se = te.getComponentMethod("errorbars", "supplyDefaults");
                Se(ce, le, Ce || ee.defaultLine, { axis: "y" }), Se(ce, le, Ce || ee.defaultLine, { axis: "x", inherit: "y" }), fe("zorder");
              } else le.visible = !1;
            };
          }, 82604: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              if (J.x = "xVal" in Z ? Z.xVal : Z.x, J.y = "yVal" in Z ? Z.yVal : Z.y, "zLabelVal" in Z && (J.z = Z.zLabelVal), Z.xa && (J.xaxis = Z.xa), Z.ya && (J.yaxis = Z.ya), !(te.cumulative || {}).enabled) {
                var ae, oe = Array.isArray(ee) ? ne[0].pts[ee[0]][ee[1]] : ne[ee].pts;
                if (J.pointNumbers = oe, J.binNumber = J.pointNumber, delete J.pointNumber, delete J.pointIndex, te._indexToPoints) {
                  ae = [];
                  for (var se = 0; se < oe.length; se++) ae = ae.concat(te._indexToPoints[oe[se]]);
                } else ae = oe;
                J.pointIndices = ae;
              }
              return J;
            };
          }, 20487: function(G, J, Z) {
            var te = Z(91664).hoverPoints, ne = Z(29714).hoverLabelText;
            G.exports = function(ee, ae, oe, se, ce) {
              var le = te(ee, ae, oe, se, ce);
              if (le) {
                var ue = (ee = le[0]).cd[ee.index], de = ee.cd[0].trace;
                if (!de.cumulative.enabled) {
                  var fe = de.orientation === "h" ? "y" : "x";
                  ee[fe + "Label"] = ne(ee[fe + "a"], [ue.ph0, ue.ph1], de[fe + "hoverformat"]);
                }
                return le;
              }
            };
          }, 74461: function(G, J, Z) {
            G.exports = { attributes: Z(16160), layoutAttributes: Z(25412), supplyDefaults: Z(85079), crossTraceDefaults: Z(83380), supplyLayoutDefaults: Z(78931), calc: Z(53616).calc, crossTraceCalc: Z(24782).crossTraceCalc, plot: Z(32995).plot, layerName: "barlayer", style: Z(6851).style, styleOnSelect: Z(6851).styleOnSelect, colorbar: Z(21146), hoverPoints: Z(20487), selectPoints: Z(88384), eventData: Z(82604), moduleType: "trace", name: "histogram", basePlotModule: Z(37703), categories: ["bar-like", "cartesian", "svg", "bar", "histogram", "oriented", "errorBarsOK", "showLegend"], meta: {} };
          }, 58665: function(G) {
            G.exports = { percent: function(J, Z) {
              for (var te = J.length, ne = 100 / Z, ee = 0; ee < te; ee++) J[ee] *= ne;
            }, probability: function(J, Z) {
              for (var te = J.length, ne = 0; ne < te; ne++) J[ne] /= Z;
            }, density: function(J, Z, te, ne) {
              var ee = J.length;
              ne = ne || 1;
              for (var ae = 0; ae < ee; ae++) J[ae] *= te[ae] * ne;
            }, "probability density": function(J, Z, te, ne) {
              var ee = J.length;
              ne && (Z /= ne);
              for (var ae = 0; ae < ee; ae++) J[ae] *= te[ae] / Z;
            } };
          }, 9310: function(G, J, Z) {
            var te = Z(16160), ne = Z(64766), ee = Z(81658), ae = Z(9829), oe = Z(80712).axisHoverFormat, se = Z(3208).rb, ce = Z(3208).ay, le = Z(87163), ue = Z(93049).extendFlat;
            G.exports = ue({ x: te.x, y: te.y, z: { valType: "data_array", editType: "calc" }, marker: { color: { valType: "data_array", editType: "calc" }, editType: "calc" }, histnorm: te.histnorm, histfunc: te.histfunc, nbinsx: te.nbinsx, xbins: ne("x"), nbinsy: te.nbinsy, ybins: ne("y"), autobinx: te.autobinx, autobiny: te.autobiny, bingroup: ue({}, te.bingroup, {}), xbingroup: ue({}, te.bingroup, {}), ybingroup: ue({}, te.bingroup, {}), xgap: ee.xgap, ygap: ee.ygap, zsmooth: ee.zsmooth, xhoverformat: oe("x"), yhoverformat: oe("y"), zhoverformat: oe("z", 1), hovertemplate: se({}, { keys: "z" }), texttemplate: ce({ arrayOk: !1, editType: "plot" }, { keys: "z" }), textfont: ee.textfont, showlegend: ue({}, ae.showlegend, { dflt: !1 }) }, le("", { cLetter: "z", autoColorDflt: !1 }));
          }, 19226: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(34870), ae = Z(58665), oe = Z(48198), se = Z(64852), ce = Z(53616).calcAllAutoBins;
            function le(fe, he, me, pe) {
              var ye, ge = new Array(fe);
              if (pe) for (ye = 0; ye < fe; ye++) ge[ye] = 1 / (he[ye + 1] - he[ye]);
              else {
                var ve = 1 / me;
                for (ye = 0; ye < fe; ye++) ge[ye] = ve;
              }
              return ge;
            }
            function ue(fe, he) {
              return { start: fe(he.start), end: fe(he.end), size: he.size };
            }
            function de(fe, he, me, pe, ye, ge) {
              var ve, be = fe.length - 1, Ce = new Array(be), Se = se(me, pe, fe, ye, ge);
              for (ve = 0; ve < be; ve++) {
                var we = (he || [])[ve];
                Ce[ve] = we === void 0 ? [Se(fe[ve]), Se(fe[ve + 1], !0)] : [we, we];
              }
              return Ce;
            }
            G.exports = function(fe, he) {
              var me, pe, ye, ge, ve = ne.getFromId(fe, he.xaxis), be = ne.getFromId(fe, he.yaxis), Ce = he.xcalendar, Se = he.ycalendar, we = function(wn) {
                return ve.r2c(wn, 0, Ce);
              }, Ee = function(wn) {
                return be.r2c(wn, 0, Se);
              }, xe = ce(fe, he, ve, "x"), Le = xe[0], Qe = xe[1], ke = ce(fe, he, be, "y"), Me = ke[0], Re = ke[1], De = he._length;
              Qe.length > De && Qe.splice(De, Qe.length - De), Re.length > De && Re.splice(De, Re.length - De);
              var Pe = [], $e = [], qe = [], Xe = typeof Le.size == "string", Ve = typeof Me.size == "string", Ze = [], Ge = [], ze = Xe ? Ze : Le, We = Ve ? Ge : Me, at = 0, ct = [], et = [], dt = he.histnorm, Je = he.histfunc, it = dt.indexOf("density") !== -1, st = Je === "max" || Je === "min" ? null : 0, gt = ee.count, bt = ae[dt], yt = !1, St = [], Lt = [], Rt = "z" in he ? he.z : "marker" in he && Array.isArray(he.marker.color) ? he.marker.color : "";
              Rt && Je !== "count" && (yt = Je === "avg", gt = ee[Je]);
              var vt = Le.size, $t = we(Le.start), Pt = we(Le.end) + ($t - ne.tickIncrement($t, vt, !1, Ce)) / 1e6;
              for (me = $t; me < Pt; me = ne.tickIncrement(me, vt, !1, Ce)) $e.push(st), Ze.push(me), yt && qe.push(0);
              Ze.push(me);
              var Xt, Nt = $e.length, Gt = (me - $t) / Nt, Ft = (Xt = $t + Gt / 2, ve.c2r(Xt, 0, Ce)), Tt = Me.size, It = Ee(Me.start), Ct = Ee(Me.end) + (It - ne.tickIncrement(It, Tt, !1, Se)) / 1e6;
              for (me = It; me < Ct; me = ne.tickIncrement(me, Tt, !1, Se)) {
                Pe.push($e.slice()), Ge.push(me);
                var Kt = new Array(Nt);
                for (pe = 0; pe < Nt; pe++) Kt[pe] = [];
                et.push(Kt), yt && ct.push(qe.slice());
              }
              Ge.push(me);
              var Vt = Pe.length, Yt = (me - It) / Vt, sn = function(wn) {
                return be.c2r(wn, 0, Se);
              }(It + Yt / 2);
              it && (St = le($e.length, ze, Gt, Xe), Lt = le(Pe.length, We, Yt, Ve)), Xe || ve.type !== "date" || (ze = ue(we, ze)), Ve || be.type !== "date" || (We = ue(Ee, We));
              var hn = !0, Qt = !0, Ut = new Array(Nt), Wt = new Array(Vt), rn = 1 / 0, vn = 1 / 0, dn = 1 / 0, Ht = 1 / 0;
              for (me = 0; me < De; me++) {
                var jt = Qe[me], cn = Re[me];
                ye = te.findBin(jt, ze), ge = te.findBin(cn, We), ye >= 0 && ye < Nt && ge >= 0 && ge < Vt && (at += gt(ye, me, Pe[ge], Rt, ct[ge]), et[ge][ye].push(me), hn && (Ut[ye] === void 0 ? Ut[ye] = jt : Ut[ye] !== jt && (hn = !1)), Qt && (Wt[ge] === void 0 ? Wt[ge] = cn : Wt[ge] !== cn && (Qt = !1)), rn = Math.min(rn, jt - Ze[ye]), vn = Math.min(vn, Ze[ye + 1] - jt), dn = Math.min(dn, cn - Ge[ge]), Ht = Math.min(Ht, Ge[ge + 1] - cn));
              }
              if (yt) for (ge = 0; ge < Vt; ge++) at += oe(Pe[ge], ct[ge]);
              if (bt) for (ge = 0; ge < Vt; ge++) bt(Pe[ge], at, St, Lt[ge]);
              return { x: Qe, xRanges: de(Ze, hn && Ut, rn, vn, ve, Ce), x0: Ft, dx: Gt, y: Re, yRanges: de(Ge, Qt && Wt, dn, Ht, be, Se), y0: sn, dy: Yt, z: Pe, pts: et };
            };
          }, 29097: function(G, J, Z) {
            var te = Z(34809), ne = Z(77134), ee = Z(44143), ae = Z(39356), oe = Z(63814), se = Z(9310);
            G.exports = function(ce, le, ue, de) {
              function fe(he, me) {
                return te.coerce(ce, le, se, he, me);
              }
              ne(ce, le, fe, de), le.visible !== !1 && (ee(ce, le, fe, de), ae(ce, le, de, fe, { prefix: "", cLetter: "z" }), fe("hovertemplate"), oe(fe, de), fe("xhoverformat"), fe("yhoverformat"));
            };
          }, 1873: function(G, J, Z) {
            var te = Z(93125), ne = Z(29714).hoverLabelText;
            G.exports = function(ee, ae, oe, se, ce) {
              var le = te(ee, ae, oe, se, ce);
              if (le) {
                var ue = (ee = le[0]).index, de = ue[0], fe = ue[1], he = ee.cd[0], me = he.trace, pe = he.xRanges[fe], ye = he.yRanges[de];
                return ee.xLabel = ne(ee.xa, [pe[0], pe[1]], me.xhoverformat), ee.yLabel = ne(ee.ya, [ye[0], ye[1]], me.yhoverformat), le;
              }
            };
          }, 66143: function(G, J, Z) {
            G.exports = { attributes: Z(9310), supplyDefaults: Z(29097), crossTraceDefaults: Z(83380), calc: Z(51670), plot: Z(19236), layerName: "heatmaplayer", colorbar: Z(12431), style: Z(12774), hoverPoints: Z(1873), eventData: Z(82604), moduleType: "trace", name: "histogram2d", basePlotModule: Z(37703), categories: ["cartesian", "svg", "2dMap", "histogram", "showLegend"], meta: {} };
          }, 77134: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809);
            G.exports = function(ee, ae, oe, se) {
              var ce = oe("x"), le = oe("y"), ue = ne.minRowLength(ce), de = ne.minRowLength(le);
              ue && de ? (ae._length = Math.min(ue, de), te.getComponentMethod("calendars", "handleTraceDefaults")(ee, ae, ["x", "y"], se), (oe("z") || oe("marker.color")) && oe("histfunc"), oe("histnorm"), oe("autobinx"), oe("autobiny")) : ae.visible = !1;
            };
          }, 85018: function(G, J, Z) {
            var te = Z(9310), ne = Z(52240), ee = Z(87163), ae = Z(80712).axisHoverFormat, oe = Z(93049).extendFlat;
            G.exports = oe({ x: te.x, y: te.y, z: te.z, marker: te.marker, histnorm: te.histnorm, histfunc: te.histfunc, nbinsx: te.nbinsx, xbins: te.xbins, nbinsy: te.nbinsy, ybins: te.ybins, autobinx: te.autobinx, autobiny: te.autobiny, bingroup: te.bingroup, xbingroup: te.xbingroup, ybingroup: te.ybingroup, autocontour: ne.autocontour, ncontours: ne.ncontours, contours: ne.contours, line: { color: ne.line.color, width: oe({}, ne.line.width, { dflt: 0.5 }), dash: ne.line.dash, smoothing: ne.line.smoothing, editType: "plot" }, xhoverformat: ae("x"), yhoverformat: ae("y"), zhoverformat: ae("z", 1), hovertemplate: te.hovertemplate, texttemplate: ne.texttemplate, textfont: ne.textfont }, ee("", { cLetter: "z", editTypeOverride: "calc" }));
          }, 49389: function(G, J, Z) {
            var te = Z(34809), ne = Z(77134), ee = Z(47495), ae = Z(39889), oe = Z(63814), se = Z(85018);
            G.exports = function(ce, le, ue, de) {
              function fe(he, me) {
                return te.coerce(ce, le, se, he, me);
              }
              ne(ce, le, fe, de), le.visible !== !1 && (ee(ce, le, fe, function(he) {
                return te.coerce2(ce, le, se, he);
              }), ae(ce, le, fe, de), fe("xhoverformat"), fe("yhoverformat"), fe("hovertemplate"), le.contours && le.contours.coloring === "heatmap" && oe(fe, de));
            };
          }, 81955: function(G, J, Z) {
            G.exports = { attributes: Z(85018), supplyDefaults: Z(49389), crossTraceDefaults: Z(83380), calc: Z(40352), plot: Z(8850).plot, layerName: "contourlayer", style: Z(1328), colorbar: Z(92697), hoverPoints: Z(29815), moduleType: "trace", name: "histogram2dcontour", basePlotModule: Z(37703), categories: ["cartesian", "svg", "2dMap", "contour", "histogram", "showLegend"], meta: {} };
          }, 12505: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(87163), ae = Z(13792).u, oe = Z(55412), se = Z(56708), ce = Z(71856), le = Z(43236), ue = Z(93049).extendFlat, de = Z(94850).k;
            G.exports = { labels: se.labels, parents: se.parents, values: se.values, branchvalues: se.branchvalues, count: se.count, level: se.level, maxdepth: se.maxdepth, tiling: { orientation: { valType: "enumerated", values: ["v", "h"], dflt: "h", editType: "plot" }, flip: ce.tiling.flip, pad: { valType: "number", min: 0, dflt: 0, editType: "plot" }, editType: "calc" }, marker: ue({ colors: se.marker.colors, line: se.marker.line, pattern: de, editType: "calc" }, ee("marker", { colorAttr: "colors", anim: !1 })), leaf: se.leaf, pathbar: ce.pathbar, text: oe.text, textinfo: se.textinfo, texttemplate: ne({ editType: "plot" }, { keys: le.eventDataKeys.concat(["label", "value"]) }), hovertext: oe.hovertext, hoverinfo: se.hoverinfo, hovertemplate: te({}, { keys: le.eventDataKeys }), textfont: oe.textfont, insidetextfont: oe.insidetextfont, outsidetextfont: ce.outsidetextfont, textposition: ce.textposition, sort: oe.sort, root: se.root, domain: ae({ name: "icicle", trace: !0, editType: "calc" }) };
          }, 63387: function(G, J, Z) {
            var te = Z(44122);
            J.name = "icicle", J.plot = function(ne, ee, ae, oe) {
              te.plotBasePlot(J.name, ne, ee, ae, oe);
            }, J.clean = function(ne, ee, ae, oe) {
              te.cleanBasePlot(J.name, ne, ee, ae, oe);
            };
          }, 36349: function(G, J, Z) {
            var te = Z(14852);
            J._ = function(ne, ee) {
              return te.calc(ne, ee);
            }, J.t = function(ne) {
              return te._runCrossTraceCalc("icicle", ne);
            };
          }, 17918: function(G, J, Z) {
            var te = Z(34809), ne = Z(12505), ee = Z(78766), ae = Z(13792).N, oe = Z(17550).handleText, se = Z(56155).TEXTPAD, ce = Z(46979).handleMarkerDefaults, le = Z(88856), ue = le.hasColorscale, de = le.handleDefaults;
            G.exports = function(fe, he, me, pe) {
              function ye(Ee, xe) {
                return te.coerce(fe, he, ne, Ee, xe);
              }
              var ge = ye("labels"), ve = ye("parents");
              if (ge && ge.length && ve && ve.length) {
                var be = ye("values");
                be && be.length ? ye("branchvalues") : ye("count"), ye("level"), ye("maxdepth"), ye("tiling.orientation"), ye("tiling.flip"), ye("tiling.pad");
                var Ce = ye("text");
                ye("texttemplate"), he.texttemplate || ye("textinfo", te.isArrayOrTypedArray(Ce) ? "text+label" : "label"), ye("hovertext"), ye("hovertemplate");
                var Se = ye("pathbar.visible");
                oe(fe, he, pe, ye, "auto", { hasPathbar: Se, moduleHasSelected: !1, moduleHasUnselected: !1, moduleHasConstrain: !1, moduleHasCliponaxis: !1, moduleHasTextangle: !1, moduleHasInsideanchor: !1 }), ye("textposition"), ce(fe, he, pe, ye);
                var we = he._hasColorscale = ue(fe, "marker", "colors") || (fe.marker || {}).coloraxis;
                we && de(fe, he, pe, ye, { prefix: "marker.", cLetter: "c" }), ye("leaf.opacity", we ? 1 : 0.7), he._hovered = { marker: { line: { width: 2, color: ee.contrast(pe.paper_bgcolor) } } }, Se && (ye("pathbar.thickness", he.pathbar.textfont.size + 2 * se), ye("pathbar.side"), ye("pathbar.edgeshape")), ye("sort"), ye("root.color"), ae(he, pe, ye), he._length = null;
              } else he.visible = !1;
            };
          }, 23593: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(30635), oe = Z(29316), se = Z(50579).styleOne, ce = Z(43236), le = Z(33108), ue = Z(44691), de = Z(19718).formatSliceLabel, fe = !1;
            G.exports = function(he, me, pe, ye, ge) {
              var ve = ge.width, be = ge.height, Ce = ge.viewX, Se = ge.viewY, we = ge.pathSlice, Ee = ge.toMoveInsideSlice, xe = ge.strTransform, Le = ge.hasTransition, Qe = ge.handleSlicesExit, ke = ge.makeUpdateSliceInterpolator, Me = ge.makeUpdateTextInterpolator, Re = ge.prevEntry, De = he._context.staticPlot, Pe = he._fullLayout, $e = me[0].trace, qe = $e.textposition.indexOf("left") !== -1, Xe = $e.textposition.indexOf("right") !== -1, Ve = $e.textposition.indexOf("bottom") !== -1, Ze = oe(pe, [ve, be], { flipX: $e.tiling.flip.indexOf("x") > -1, flipY: $e.tiling.flip.indexOf("y") > -1, orientation: $e.tiling.orientation, pad: { inner: $e.tiling.pad }, maxDepth: $e._maxDepth }).descendants(), Ge = 1 / 0, ze = -1 / 0;
              Ze.forEach(function(dt) {
                var Je = dt.depth;
                Je >= $e._maxDepth ? (dt.x0 = dt.x1 = (dt.x0 + dt.x1) / 2, dt.y0 = dt.y1 = (dt.y0 + dt.y1) / 2) : (Ge = Math.min(Ge, Je), ze = Math.max(ze, Je));
              }), ye = ye.data(Ze, le.getPtId), $e._maxVisibleLayers = isFinite(ze) ? ze - Ge + 1 : 0, ye.enter().append("g").classed("slice", !0), Qe(ye, fe, {}, [ve, be], we), ye.order();
              var We = null;
              if (Le && Re) {
                var at = le.getPtId(Re);
                ye.each(function(dt) {
                  We === null && le.getPtId(dt) === at && (We = { x0: dt.x0, x1: dt.x1, y0: dt.y0, y1: dt.y1 });
                });
              }
              var ct = function() {
                return We || { x0: 0, x1: ve, y0: 0, y1: be };
              }, et = ye;
              return Le && (et = et.transition().each("end", function() {
                var dt = te.select(this);
                le.setSliceCursor(dt, he, { hideOnRoot: !0, hideOnLeaves: !1, isTransitioning: !1 });
              })), et.each(function(dt) {
                dt._x0 = Ce(dt.x0), dt._x1 = Ce(dt.x1), dt._y0 = Se(dt.y0), dt._y1 = Se(dt.y1), dt._hoverX = Ce(dt.x1 - $e.tiling.pad), dt._hoverY = Se(Ve ? dt.y1 - $e.tiling.pad / 2 : dt.y0 + $e.tiling.pad / 2);
                var Je = te.select(this), it = ne.ensureSingle(Je, "path", "surface", function(yt) {
                  yt.style("pointer-events", De ? "none" : "all");
                });
                Le ? it.transition().attrTween("d", function(yt) {
                  var St = ke(yt, fe, ct(), [ve, be], { orientation: $e.tiling.orientation, flipX: $e.tiling.flip.indexOf("x") > -1, flipY: $e.tiling.flip.indexOf("y") > -1 });
                  return function(Lt) {
                    return we(St(Lt));
                  };
                }) : it.attr("d", we), Je.call(ue, pe, he, me, { styleOne: se, eventDataKeys: ce.eventDataKeys, transitionTime: ce.CLICK_TRANSITION_TIME, transitionEasing: ce.CLICK_TRANSITION_EASING }).call(le.setSliceCursor, he, { isTransitioning: he._transitioning }), it.call(se, dt, $e, he, { hovered: !1 }), dt.x0 === dt.x1 || dt.y0 === dt.y1 ? dt._text = "" : dt._text = de(dt, pe, $e, me, Pe) || "";
                var st = ne.ensureSingle(Je, "g", "slicetext"), gt = ne.ensureSingle(st, "text", "", function(yt) {
                  yt.attr("data-notex", 1);
                }), bt = ne.ensureUniformFontSize(he, le.determineTextFont($e, dt, Pe.font));
                gt.text(dt._text || " ").classed("slicetext", !0).attr("text-anchor", Xe ? "end" : qe ? "start" : "middle").call(ee.font, bt).call(ae.convertToTspans, he), dt.textBB = ee.bBox(gt.node()), dt.transform = Ee(dt, { fontSize: bt.size }), dt.transform.fontSize = bt.size, Le ? gt.transition().attrTween("transform", function(yt) {
                  var St = Me(yt, fe, ct(), [ve, be]);
                  return function(Lt) {
                    return xe(St(Lt));
                  };
                }) : gt.attr("transform", xe(dt));
              }), We;
            };
          }, 36858: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "icicle", basePlotModule: Z(63387), categories: [], animatable: !0, attributes: Z(12505), layoutAttributes: Z(60052), supplyDefaults: Z(17918), supplyLayoutDefaults: Z(11747), calc: Z(36349)._, crossTraceCalc: Z(36349).t, plot: Z(1395), style: Z(50579).style, colorbar: Z(21146), meta: {} };
          }, 60052: function(G) {
            G.exports = { iciclecolorway: { valType: "colorlist", editType: "calc" }, extendiciclecolors: { valType: "boolean", dflt: !0, editType: "calc" } };
          }, 11747: function(G, J, Z) {
            var te = Z(34809), ne = Z(60052);
            G.exports = function(ee, ae) {
              function oe(se, ce) {
                return te.coerce(ee, ae, ne, se, ce);
              }
              oe("iciclecolorway", ae.colorway), oe("extendiciclecolors");
            };
          }, 29316: function(G, J, Z) {
            var te = Z(92264), ne = Z(36141);
            G.exports = function(ee, ae, oe) {
              var se = oe.flipX, ce = oe.flipY, le = oe.orientation === "h", ue = oe.maxDepth, de = ae[0], fe = ae[1];
              ue && (de = (ee.height + 1) * ae[0] / Math.min(ee.height + 1, ue), fe = (ee.height + 1) * ae[1] / Math.min(ee.height + 1, ue));
              var he = te.partition().padding(oe.pad.inner).size(le ? [ae[1], de] : [ae[0], fe])(ee);
              return (le || se || ce) && ne(he, ae, { swapXY: le, flipX: se, flipY: ce }), he;
            };
          }, 1395: function(G, J, Z) {
            var te = Z(41567), ne = Z(23593);
            G.exports = function(ee, ae, oe, se) {
              return te(ee, ae, oe, se, { type: "icicle", drawDescendants: ne });
            };
          }, 50579: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(34809), ae = Z(84102).resizeText, oe = Z(72043);
            function se(ce, le, ue, de) {
              var fe = le.data.data, he = !le.children, me = fe.i, pe = ee.castOption(ue, me, "marker.line.color") || ne.defaultLine, ye = ee.castOption(ue, me, "marker.line.width") || 0;
              ce.call(oe, le, ue, de).style("stroke-width", ye).call(ne.stroke, pe).style("opacity", he ? ue.leaf.opacity : null);
            }
            G.exports = { style: function(ce) {
              var le = ce._fullLayout._iciclelayer.selectAll(".trace");
              ae(ce, le, "icicle"), le.each(function(ue) {
                var de = te.select(this), fe = ue[0].trace;
                de.style("opacity", fe.opacity), de.selectAll("path.surface").each(function(he) {
                  te.select(this).call(se, he, fe, ce);
                });
              });
            }, styleOne: se };
          }, 22153: function(G, J, Z) {
            for (var te = Z(9829), ne = Z(36640).zorder, ee = Z(3208).rb, ae = Z(93049).extendFlat, oe = Z(42939).colormodel, se = ["rgb", "rgba", "rgba256", "hsl", "hsla"], ce = [], le = [], ue = 0; ue < se.length; ue++) {
              var de = oe[se[ue]];
              ce.push("For the `" + se[ue] + "` colormodel, it is [" + (de.zminDflt || de.min).join(", ") + "]."), le.push("For the `" + se[ue] + "` colormodel, it is [" + (de.zmaxDflt || de.max).join(", ") + "].");
            }
            G.exports = ae({ source: { valType: "string", editType: "calc" }, z: { valType: "data_array", editType: "calc" }, colormodel: { valType: "enumerated", values: se, editType: "calc" }, zsmooth: { valType: "enumerated", values: ["fast", !1], dflt: !1, editType: "plot" }, zmin: { valType: "info_array", items: [{ valType: "number", editType: "calc" }, { valType: "number", editType: "calc" }, { valType: "number", editType: "calc" }, { valType: "number", editType: "calc" }], editType: "calc" }, zmax: { valType: "info_array", items: [{ valType: "number", editType: "calc" }, { valType: "number", editType: "calc" }, { valType: "number", editType: "calc" }, { valType: "number", editType: "calc" }], editType: "calc" }, x0: { valType: "any", dflt: 0, editType: "calc+clearAxisTypes" }, y0: { valType: "any", dflt: 0, editType: "calc+clearAxisTypes" }, dx: { valType: "number", dflt: 1, editType: "calc" }, dy: { valType: "number", dflt: 1, editType: "calc" }, text: { valType: "data_array", editType: "plot" }, hovertext: { valType: "data_array", editType: "plot" }, hoverinfo: ae({}, te.hoverinfo, { flags: ["x", "y", "z", "color", "name", "text"], dflt: "x+y+z+text+name" }), hovertemplate: ee({}, { keys: ["z", "color", "colormodel"] }), zorder: ne, transforms: void 0 });
          }, 31181: function(G, J, Z) {
            var te = Z(34809), ne = Z(42939), ee = Z(10721), ae = Z(29714), oe = Z(34809).maxRowLength, se = Z(96315).p;
            function ce(ue, de, fe, he) {
              return function(me) {
                return te.constrain((me - ue) * de, fe, he);
              };
            }
            function le(ue, de) {
              return function(fe) {
                return te.constrain(fe, ue, de);
              };
            }
            G.exports = function(ue, de) {
              var fe, he;
              if (de._hasZ) fe = de.z.length, he = oe(de.z);
              else if (de._hasSource) {
                var me = se(de.source);
                fe = me.height, he = me.width;
              }
              var pe, ye = ae.getFromId(ue, de.xaxis || "x"), ge = ae.getFromId(ue, de.yaxis || "y"), ve = ye.d2c(de.x0) - de.dx / 2, be = ge.d2c(de.y0) - de.dy / 2, Ce = [ve, ve + he * de.dx], Se = [be, be + fe * de.dy];
              if (ye && ye.type === "log") for (pe = 0; pe < he; pe++) Ce.push(ve + pe * de.dx);
              if (ge && ge.type === "log") for (pe = 0; pe < fe; pe++) Se.push(be + pe * de.dy);
              return de._extremes[ye._id] = ae.findExtremes(ye, Ce), de._extremes[ge._id] = ae.findExtremes(ge, Se), de._scaler = function(we) {
                var Ee = ne.colormodel[we.colormodel], xe = (Ee.colormodel || we.colormodel).length;
                we._sArray = [];
                for (var Le = 0; Le < xe; Le++) Ee.min[Le] !== we.zmin[Le] || Ee.max[Le] !== we.zmax[Le] ? we._sArray.push(ce(we.zmin[Le], (Ee.max[Le] - Ee.min[Le]) / (we.zmax[Le] - we.zmin[Le]), Ee.min[Le], Ee.max[Le])) : we._sArray.push(le(Ee.min[Le], Ee.max[Le]));
                return function(Qe) {
                  for (var ke = Qe.slice(0, xe), Me = 0; Me < xe; Me++) {
                    var Re = ke[Me];
                    if (!ee(Re)) return !1;
                    ke[Me] = we._sArray[Me](Re);
                  }
                  return ke;
                };
              }(de), [{ x0: ve, y0: be, z: de.z, w: he, h: fe }];
            };
          }, 42939: function(G) {
            G.exports = { colormodel: { rgb: { min: [0, 0, 0], max: [255, 255, 255], fmt: function(J) {
              return J.slice(0, 3);
            }, suffix: ["", "", ""] }, rgba: { min: [0, 0, 0, 0], max: [255, 255, 255, 1], fmt: function(J) {
              return J.slice(0, 4);
            }, suffix: ["", "", "", ""] }, rgba256: { colormodel: "rgba", zminDflt: [0, 0, 0, 0], zmaxDflt: [255, 255, 255, 255], min: [0, 0, 0, 0], max: [255, 255, 255, 1], fmt: function(J) {
              return J.slice(0, 4);
            }, suffix: ["", "", "", ""] }, hsl: { min: [0, 0, 0], max: [360, 100, 100], fmt: function(J) {
              var Z = J.slice(0, 3);
              return Z[1] = Z[1] + "%", Z[2] = Z[2] + "%", Z;
            }, suffix: ["°", "%", "%"] }, hsla: { min: [0, 0, 0, 0], max: [360, 100, 100, 1], fmt: function(J) {
              var Z = J.slice(0, 4);
              return Z[1] = Z[1] + "%", Z[2] = Z[2] + "%", Z;
            }, suffix: ["°", "%", "%", ""] } } };
          }, 82766: function(G, J, Z) {
            var te = Z(34809), ne = Z(22153), ee = Z(42939), ae = Z(84619).IMAGE_URL_PREFIX;
            G.exports = function(oe, se) {
              function ce(de, fe) {
                return te.coerce(oe, se, ne, de, fe);
              }
              ce("source"), se.source && !se.source.match(ae) && delete se.source, se._hasSource = !!se.source;
              var le, ue = ce("z");
              se._hasZ = !(ue === void 0 || !ue.length || !ue[0] || !ue[0].length), se._hasZ || se._hasSource ? (ce("x0"), ce("y0"), ce("dx"), ce("dy"), se._hasZ ? (ce("colormodel", "rgb"), ce("zmin", (le = ee.colormodel[se.colormodel]).zminDflt || le.min), ce("zmax", le.zmaxDflt || le.max)) : se._hasSource && (se.colormodel = "rgba256", le = ee.colormodel[se.colormodel], se.zmin = le.zminDflt, se.zmax = le.zmaxDflt), ce("zsmooth"), ce("text"), ce("hovertext"), ce("hovertemplate"), se._length = null, ce("zorder")) : se.visible = !1;
            };
          }, 45461: function(G) {
            G.exports = function(J, Z) {
              return "xVal" in Z && (J.x = Z.xVal), "yVal" in Z && (J.y = Z.yVal), Z.xa && (J.xaxis = Z.xa), Z.ya && (J.yaxis = Z.ya), J.color = Z.color, J.colormodel = Z.trace.colormodel, J.z || (J.z = Z.color), J;
            };
          }, 96315: function(G, J, Z) {
            var te = Z(19490), ne = Z(84619).IMAGE_URL_PREFIX, ee = Z(45708).Buffer;
            J.p = function(ae) {
              var oe = ae.replace(ne, ""), se = new ee(oe, "base64");
              return te(se);
            };
          }, 57328: function(G, J, Z) {
            var te = Z(32141), ne = Z(34809), ee = ne.isArrayOrTypedArray, ae = Z(42939);
            G.exports = function(oe, se, ce) {
              var le = oe.cd[0], ue = le.trace, de = oe.xa, fe = oe.ya;
              if (!(te.inbox(se - le.x0, se - (le.x0 + le.w * ue.dx), 0) > 0 || te.inbox(ce - le.y0, ce - (le.y0 + le.h * ue.dy), 0) > 0)) {
                var he, me = Math.floor((se - le.x0) / ue.dx), pe = Math.floor(Math.abs(ce - le.y0) / ue.dy);
                if (ue._hasZ ? he = le.z[pe][me] : ue._hasSource && (he = ue._canvas.el.getContext("2d", { willReadFrequently: !0 }).getImageData(me, pe, 1, 1).data), he) {
                  var ye, ge = le.hi || ue.hoverinfo;
                  if (ge) {
                    var ve = ge.split("+");
                    ve.indexOf("all") !== -1 && (ve = ["color"]), ve.indexOf("color") !== -1 && (ye = !0);
                  }
                  var be, Ce = ae.colormodel[ue.colormodel], Se = Ce.colormodel || ue.colormodel, we = Se.length, Ee = ue._scaler(he), xe = Ce.suffix, Le = [];
                  (ue.hovertemplate || ye) && (Le.push("[" + [Ee[0] + xe[0], Ee[1] + xe[1], Ee[2] + xe[2]].join(", ")), we === 4 && Le.push(", " + Ee[3] + xe[3]), Le.push("]"), Le = Le.join(""), oe.extraText = Se.toUpperCase() + ": " + Le), ee(ue.hovertext) && ee(ue.hovertext[pe]) ? be = ue.hovertext[pe][me] : ee(ue.text) && ee(ue.text[pe]) && (be = ue.text[pe][me]);
                  var Qe = fe.c2p(le.y0 + (pe + 0.5) * ue.dy), ke = le.x0 + (me + 0.5) * ue.dx, Me = le.y0 + (pe + 0.5) * ue.dy, Re = "[" + he.slice(0, ue.colormodel.length).join(", ") + "]";
                  return [ne.extendFlat(oe, { index: [pe, me], x0: de.c2p(le.x0 + me * ue.dx), x1: de.c2p(le.x0 + (me + 1) * ue.dx), y0: Qe, y1: Qe, color: Ee, xVal: ke, xLabelVal: ke, yVal: Me, yLabelVal: Me, zLabelVal: Re, text: be, hovertemplateLabels: { zLabel: Re, colorLabel: Le, "color[0]Label": Ee[0] + xe[0], "color[1]Label": Ee[1] + xe[1], "color[2]Label": Ee[2] + xe[2], "color[3]Label": Ee[3] + xe[3] } })];
                }
              }
            };
          }, 92106: function(G, J, Z) {
            G.exports = { attributes: Z(22153), supplyDefaults: Z(82766), calc: Z(31181), plot: Z(36899), style: Z(67555), hoverPoints: Z(57328), eventData: Z(45461), moduleType: "trace", name: "image", basePlotModule: Z(37703), categories: ["cartesian", "svg", "2dMap", "noSortingByValue"], animatable: !1, meta: {} };
          }, 36899: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = ne.strTranslate, ae = Z(62972), oe = Z(42939), se = Z(95544), ce = Z(1837).STYLE;
            G.exports = function(le, ue, de, fe) {
              var he = ue.xaxis, me = ue.yaxis, pe = !le._context._exportedPlot && se();
              ne.makeTraceGroups(fe, de, "im").each(function(ye) {
                var ge = te.select(this), ve = ye[0], be = ve.trace, Ce = (be.zsmooth === "fast" || be.zsmooth === !1 && pe) && !be._hasZ && be._hasSource && he.type === "linear" && me.type === "linear";
                be._realImage = Ce;
                var Se, we, Ee, xe, Le, Qe, ke = ve.z, Me = ve.x0, Re = ve.y0, De = ve.w, Pe = ve.h, $e = be.dx, qe = be.dy;
                for (Qe = 0; Se === void 0 && Qe < De; ) Se = he.c2p(Me + Qe * $e), Qe++;
                for (Qe = De; we === void 0 && Qe > 0; ) we = he.c2p(Me + Qe * $e), Qe--;
                for (Qe = 0; xe === void 0 && Qe < Pe; ) xe = me.c2p(Re + Qe * qe), Qe++;
                for (Qe = Pe; Le === void 0 && Qe > 0; ) Le = me.c2p(Re + Qe * qe), Qe--;
                we < Se && (Ee = we, we = Se, Se = Ee), Le < xe && (Ee = xe, xe = Le, Le = Ee), Ce || (Se = Math.max(-0.5 * he._length, Se), we = Math.min(1.5 * he._length, we), xe = Math.max(-0.5 * me._length, xe), Le = Math.min(1.5 * me._length, Le));
                var Xe = Math.round(we - Se), Ve = Math.round(Le - xe);
                if (Xe <= 0 || Ve <= 0) ge.selectAll("image").data([]).exit().remove();
                else {
                  var Ze = ge.selectAll("image").data([ye]);
                  Ze.enter().append("svg:image").attr({ xmlns: ae.svg, preserveAspectRatio: "none" }), Ze.exit().remove();
                  var Ge = be.zsmooth === !1 ? ce : "";
                  if (Ce) {
                    var ze = ne.simpleMap(he.range, he.r2l), We = ne.simpleMap(me.range, me.r2l), at = ze[1] < ze[0], ct = We[1] > We[0];
                    if (at || ct) {
                      var et = Se + Xe / 2, dt = xe + Ve / 2;
                      Ge += "transform:" + ee(et + "px", dt + "px") + "scale(" + (at ? -1 : 1) + "," + (ct ? -1 : 1) + ")" + ee(-et + "px", -dt + "px") + ";";
                    }
                  }
                  Ze.attr("style", Ge);
                  var Je = new Promise(function(st) {
                    if (be._hasZ) st();
                    else if (be._hasSource) if (be._canvas && be._canvas.el.width === De && be._canvas.el.height === Pe && be._canvas.source === be.source) st();
                    else {
                      var gt = document.createElement("canvas");
                      gt.width = De, gt.height = Pe;
                      var bt = gt.getContext("2d", { willReadFrequently: !0 });
                      be._image = be._image || new Image();
                      var yt = be._image;
                      yt.onload = function() {
                        bt.drawImage(yt, 0, 0), be._canvas = { el: gt, source: be.source }, st();
                      }, yt.setAttribute("src", be.source);
                    }
                  }).then(function() {
                    var st, gt;
                    if (be._hasZ) gt = it(function(yt, St) {
                      var Lt = ke[St][yt];
                      return ne.isTypedArray(Lt) && (Lt = Array.from(Lt)), Lt;
                    }), st = gt.toDataURL("image/png");
                    else if (be._hasSource) if (Ce) st = be.source;
                    else {
                      var bt = be._canvas.el.getContext("2d", { willReadFrequently: !0 }).getImageData(0, 0, De, Pe).data;
                      gt = it(function(yt, St) {
                        var Lt = 4 * (St * De + yt);
                        return [bt[Lt], bt[Lt + 1], bt[Lt + 2], bt[Lt + 3]];
                      }), st = gt.toDataURL("image/png");
                    }
                    Ze.attr({ "xlink:href": st, height: Ve, width: Xe, x: Se, y: xe });
                  });
                  le._promises.push(Je);
                }
                function it(st) {
                  var gt = document.createElement("canvas");
                  gt.width = Xe, gt.height = Ve;
                  var bt, yt = gt.getContext("2d", { willReadFrequently: !0 }), St = function(Tt) {
                    return ne.constrain(Math.round(he.c2p(Me + Tt * $e) - Se), 0, Xe);
                  }, Lt = function(Tt) {
                    return ne.constrain(Math.round(me.c2p(Re + Tt * qe) - xe), 0, Ve);
                  }, Rt = oe.colormodel[be.colormodel], vt = Rt.colormodel || be.colormodel, $t = Rt.fmt;
                  for (Qe = 0; Qe < ve.w; Qe++) {
                    var Pt = St(Qe), Xt = St(Qe + 1);
                    if (Xt !== Pt && !isNaN(Xt) && !isNaN(Pt)) for (var Nt = 0; Nt < ve.h; Nt++) {
                      var Gt = Lt(Nt), Ft = Lt(Nt + 1);
                      Ft === Gt || isNaN(Ft) || isNaN(Gt) || !st(Qe, Nt) || (bt = be._scaler(st(Qe, Nt)), yt.fillStyle = bt ? vt + "(" + $t(bt).join(",") + ")" : "rgba(0,0,0,0)", yt.fillRect(Pt, Gt, Xt - Pt, Ft - Gt));
                    }
                  }
                  return gt;
                }
              });
            };
          }, 67555: function(G, J, Z) {
            var te = Z(45568);
            G.exports = function(ne) {
              te.select(ne).selectAll(".im image").style("opacity", function(ee) {
                return ee[0].trace.opacity;
              });
            };
          }, 95485: function(G, J, Z) {
            var te = Z(93049).extendFlat, ne = Z(93049).extendDeep, ee = Z(13582).overrideAll, ae = Z(80337), oe = Z(10229), se = Z(13792).u, ce = Z(25829), le = Z(78032).templatedArray, ue = Z(20909), de = Z(80712).descriptionOnlyNumbers, fe = ae({ editType: "plot", colorEditType: "plot" }), he = { color: { valType: "color", editType: "plot" }, line: { color: { valType: "color", dflt: oe.defaultLine, editType: "plot" }, width: { valType: "number", min: 0, dflt: 0, editType: "plot" }, editType: "calc" }, thickness: { valType: "number", min: 0, max: 1, dflt: 1, editType: "plot" }, editType: "calc" }, me = { valType: "info_array", items: [{ valType: "number", editType: "plot" }, { valType: "number", editType: "plot" }], editType: "plot" }, pe = le("step", ne({}, he, { range: me }));
            G.exports = { mode: { valType: "flaglist", editType: "calc", flags: ["number", "delta", "gauge"], dflt: "number" }, value: { valType: "number", editType: "calc", anim: !0 }, align: { valType: "enumerated", values: ["left", "center", "right"], editType: "plot" }, domain: se({ name: "indicator", trace: !0, editType: "calc" }), title: { text: { valType: "string", editType: "plot" }, align: { valType: "enumerated", values: ["left", "center", "right"], editType: "plot" }, font: te({}, fe, {}), editType: "plot" }, number: { valueformat: { valType: "string", dflt: "", editType: "plot", description: de("value") }, font: te({}, fe, {}), prefix: { valType: "string", dflt: "", editType: "plot" }, suffix: { valType: "string", dflt: "", editType: "plot" }, editType: "plot" }, delta: { reference: { valType: "number", editType: "calc" }, position: { valType: "enumerated", values: ["top", "bottom", "left", "right"], dflt: "bottom", editType: "plot" }, relative: { valType: "boolean", editType: "plot", dflt: !1 }, valueformat: { valType: "string", editType: "plot", description: de("value") }, increasing: { symbol: { valType: "string", dflt: ue.INCREASING.SYMBOL, editType: "plot" }, color: { valType: "color", dflt: ue.INCREASING.COLOR, editType: "plot" }, editType: "plot" }, decreasing: { symbol: { valType: "string", dflt: ue.DECREASING.SYMBOL, editType: "plot" }, color: { valType: "color", dflt: ue.DECREASING.COLOR, editType: "plot" }, editType: "plot" }, font: te({}, fe, {}), prefix: { valType: "string", dflt: "", editType: "plot" }, suffix: { valType: "string", dflt: "", editType: "plot" }, editType: "calc" }, gauge: { shape: { valType: "enumerated", editType: "plot", dflt: "angular", values: ["angular", "bullet"] }, bar: ne({}, he, { color: { dflt: "green" } }), bgcolor: { valType: "color", editType: "plot" }, bordercolor: { valType: "color", dflt: oe.defaultLine, editType: "plot" }, borderwidth: { valType: "number", min: 0, dflt: 1, editType: "plot" }, axis: ee({ range: me, visible: te({}, ce.visible, { dflt: !0 }), tickmode: ce.minor.tickmode, nticks: ce.nticks, tick0: ce.tick0, dtick: ce.dtick, tickvals: ce.tickvals, ticktext: ce.ticktext, ticks: te({}, ce.ticks, { dflt: "outside" }), ticklen: ce.ticklen, tickwidth: ce.tickwidth, tickcolor: ce.tickcolor, ticklabelstep: ce.ticklabelstep, showticklabels: ce.showticklabels, labelalias: ce.labelalias, tickfont: ae({}), tickangle: ce.tickangle, tickformat: ce.tickformat, tickformatstops: ce.tickformatstops, tickprefix: ce.tickprefix, showtickprefix: ce.showtickprefix, ticksuffix: ce.ticksuffix, showticksuffix: ce.showticksuffix, separatethousands: ce.separatethousands, exponentformat: ce.exponentformat, minexponent: ce.minexponent, showexponent: ce.showexponent, editType: "plot" }, "plot"), steps: pe, threshold: { line: { color: te({}, he.line.color, {}), width: te({}, he.line.width, { dflt: 1 }), editType: "plot" }, thickness: te({}, he.thickness, { dflt: 0.85 }), value: { valType: "number", editType: "calc", dflt: !1 }, editType: "plot" }, editType: "plot" } };
          }, 47751: function(G, J, Z) {
            var te = Z(44122);
            J.name = "indicator", J.plot = function(ne, ee, ae, oe) {
              te.plotBasePlot(J.name, ne, ee, ae, oe);
            }, J.clean = function(ne, ee, ae, oe) {
              te.cleanBasePlot(J.name, ne, ee, ae, oe);
            };
          }, 98385: function(G) {
            G.exports = { calc: function(J, Z) {
              var te = [], ne = Z.value;
              typeof Z._lastValue != "number" && (Z._lastValue = Z.value);
              var ee = Z._lastValue, ae = ee;
              return Z._hasDelta && typeof Z.delta.reference == "number" && (ae = Z.delta.reference), te[0] = { y: ne, lastY: ee, delta: ne - ae, relativeDelta: (ne - ae) / ae }, te;
            } };
          }, 74807: function(G) {
            G.exports = { defaultNumberFontSize: 80, bulletNumberDomainSize: 0.25, bulletPadding: 0.025, innerRadius: 0.75, valueThickness: 0.5, titlePadding: 5, horizontalPadding: 10 };
          }, 79306: function(G, J, Z) {
            var te = Z(34809), ne = Z(95485), ee = Z(13792).N, ae = Z(78032), oe = Z(59008), se = Z(74807), ce = Z(22777), le = Z(87433), ue = Z(12036), de = Z(54616);
            function fe(he, me) {
              function pe(ye, ge) {
                return te.coerce(he, me, ne.gauge.steps, ye, ge);
              }
              pe("color"), pe("line.color"), pe("line.width"), pe("range"), pe("thickness");
            }
            G.exports = { supplyDefaults: function(he, me, pe, ye) {
              function ge($e, qe) {
                return te.coerce(he, me, ne, $e, qe);
              }
              ee(me, ye, ge), ge("mode"), me._hasNumber = me.mode.indexOf("number") !== -1, me._hasDelta = me.mode.indexOf("delta") !== -1, me._hasGauge = me.mode.indexOf("gauge") !== -1;
              var ve = ge("value");
              me._range = [0, typeof ve == "number" ? 1.5 * ve : 1];
              var be, Ce, Se = new Array(2);
              if (me._hasNumber) {
                ge("number.valueformat");
                var we = te.extendFlat({}, ye.font);
                we.size = void 0, te.coerceFont(ge, "number.font", we), me.number.font.size === void 0 && (me.number.font.size = se.defaultNumberFontSize, Se[0] = !0), ge("number.prefix"), ge("number.suffix"), be = me.number.font.size;
              }
              if (me._hasDelta) {
                var Ee = te.extendFlat({}, ye.font);
                Ee.size = void 0, te.coerceFont(ge, "delta.font", Ee), me.delta.font.size === void 0 && (me.delta.font.size = (me._hasNumber ? 0.5 : 1) * (be || se.defaultNumberFontSize), Se[1] = !0), ge("delta.reference", me.value), ge("delta.relative"), ge("delta.valueformat", me.delta.relative ? "2%" : ""), ge("delta.increasing.symbol"), ge("delta.increasing.color"), ge("delta.decreasing.symbol"), ge("delta.decreasing.color"), ge("delta.position"), ge("delta.prefix"), ge("delta.suffix"), Ce = me.delta.font.size;
              }
              me._scaleNumbers = (!me._hasNumber || Se[0]) && (!me._hasDelta || Se[1]) || !1;
              var xe, Le, Qe, ke, Me = te.extendFlat({}, ye.font);
              function Re($e, qe) {
                return te.coerce(xe, Le, ne.gauge, $e, qe);
              }
              function De($e, qe) {
                return te.coerce(Qe, ke, ne.gauge.axis, $e, qe);
              }
              if (Me.size = 0.25 * (be || Ce || se.defaultNumberFontSize), te.coerceFont(ge, "title.font", Me), ge("title.text"), me._hasGauge) {
                (xe = he.gauge) || (xe = {}), Le = ae.newContainer(me, "gauge"), Re("shape"), (me._isBullet = me.gauge.shape === "bullet") || ge("title.align", "center"), (me._isAngular = me.gauge.shape === "angular") || ge("align", "center"), Re("bgcolor", ye.paper_bgcolor), Re("borderwidth"), Re("bordercolor"), Re("bar.color"), Re("bar.line.color"), Re("bar.line.width"), Re("bar.thickness", se.valueThickness * (me.gauge.shape === "bullet" ? 0.5 : 1)), oe(xe, Le, { name: "steps", handleItemDefaults: fe }), Re("threshold.value"), Re("threshold.thickness"), Re("threshold.line.width"), Re("threshold.line.color"), Qe = {}, xe && (Qe = xe.axis || {}), ke = ae.newContainer(Le, "axis"), De("visible"), me._range = De("range", me._range);
                var Pe = { font: ye.font, noAutotickangles: !0, outerTicks: !0, noTicklabelshift: !0, noTicklabelstandoff: !0 };
                ce(Qe, ke, De, "linear"), de(Qe, ke, De, "linear", Pe), ue(Qe, ke, De, "linear", Pe), le(Qe, ke, De, Pe);
              } else ge("title.align", "center"), ge("align", "center"), me._isAngular = me._isBullet = !1;
              me._length = null;
            } };
          }, 25638: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "indicator", basePlotModule: Z(47751), categories: ["svg", "noOpacity", "noHover"], animatable: !0, attributes: Z(95485), supplyDefaults: Z(79306).supplyDefaults, calc: Z(98385).calc, plot: Z(37095), meta: {} };
          }, 37095: function(G, J, Z) {
            var te = Z(45568), ne = Z(88640).GW, ee = Z(88640).Dj, ae = Z(34809), oe = ae.strScale, se = ae.strTranslate, ce = ae.rad2deg, le = Z(4530).MID_SHIFT, ue = Z(62203), de = Z(74807), fe = Z(30635), he = Z(29714), me = Z(97655), pe = Z(40957), ye = Z(25829), ge = Z(78766), ve = { left: "start", center: "middle", right: "end" }, be = { left: 0, center: 0.5, right: 1 }, Ce = /[yzafpnµmkMGTPEZY]/;
            function Se(ke) {
              return ke && ke.duration > 0;
            }
            function we(ke) {
              ke.each(function(Me) {
                ge.stroke(te.select(this), Me.line.color);
              }).each(function(Me) {
                ge.fill(te.select(this), Me.color);
              }).style("stroke-width", function(Me) {
                return Me.line.width;
              });
            }
            function Ee(ke, Me, Re) {
              var De = ke._fullLayout, Pe = ae.extendFlat({ type: "linear", ticks: "outside", range: Re, showline: !0 }, Me), $e = { type: "linear", _id: "x" + Me._id }, qe = { letter: "x", font: De.font, noAutotickangles: !0, noHover: !0, noTickson: !0 };
              function Xe(Ve, Ze) {
                return ae.coerce(Pe, $e, ye, Ve, Ze);
              }
              return me(Pe, $e, Xe, qe, De), pe(Pe, $e, Xe, qe), $e;
            }
            function xe(ke, Me, Re) {
              return [Math.min(Me / ke.width, Re / ke.height), ke, Me + "x" + Re];
            }
            function Le(ke, Me, Re, De) {
              var Pe = document.createElementNS("http://www.w3.org/2000/svg", "text"), $e = te.select(Pe);
              return $e.text(ke).attr("x", 0).attr("y", 0).attr("text-anchor", Re).attr("data-unformatted", ke).call(fe.convertToTspans, De).call(ue.font, Me), ue.bBox($e.node());
            }
            function Qe(ke, Me, Re, De, Pe, $e) {
              var qe = "_cache" + Me;
              ke[qe] && ke[qe].key === Pe || (ke[qe] = { key: Pe, value: Re });
              var Xe = ae.aggNums($e, null, [ke[qe].value, De], 2);
              return ke[qe].value = Xe, Xe;
            }
            G.exports = function(ke, Me, Re, De) {
              var Pe, $e = ke._fullLayout;
              Se(Re) && De && (Pe = De()), ae.makeTraceGroups($e._indicatorlayer, Me, "trace").each(function(qe) {
                var Xe, Ve, Ze, Ge, ze, We = qe[0].trace, at = te.select(this), ct = We._hasGauge, et = We._isAngular, dt = We._isBullet, Je = We.domain, it = { w: $e._size.w * (Je.x[1] - Je.x[0]), h: $e._size.h * (Je.y[1] - Je.y[0]), l: $e._size.l + $e._size.w * Je.x[0], r: $e._size.r + $e._size.w * (1 - Je.x[1]), t: $e._size.t + $e._size.h * (1 - Je.y[1]), b: $e._size.b + $e._size.h * Je.y[0] }, st = it.l + it.w / 2, gt = it.t + it.h / 2, bt = Math.min(it.w / 2, it.h), yt = de.innerRadius * bt, St = We.align || "center";
                if (Ve = gt, ct) {
                  if (et && (Xe = st, Ve = gt + bt / 2, Ze = function(Gt) {
                    return function(Ft, Tt) {
                      return [Tt / Math.sqrt(Ft.width / 2 * (Ft.width / 2) + Ft.height * Ft.height), Ft, Tt];
                    }(Gt, 0.9 * yt);
                  }), dt) {
                    var Lt = de.bulletPadding, Rt = 1 - de.bulletNumberDomainSize + Lt;
                    Xe = it.l + (Rt + (1 - Rt) * be[St]) * it.w, Ze = function(Gt) {
                      return xe(Gt, (de.bulletNumberDomainSize - Lt) * it.w, it.h);
                    };
                  }
                } else Xe = it.l + be[St] * it.w, Ze = function(Gt) {
                  return xe(Gt, it.w, it.h);
                };
                (function(Gt, Ft, Tt, It) {
                  var Ct, Kt, Vt, Yt = Tt[0].trace, sn = It.numbersX, hn = It.numbersY, Qt = Yt.align || "center", Ut = ve[Qt], Wt = It.transitionOpts, rn = It.onComplete, vn = ae.ensureSingle(Ft, "g", "numbers"), dn = [];
                  Yt._hasNumber && dn.push("number"), Yt._hasDelta && (dn.push("delta"), Yt.delta.position === "left" && dn.reverse());
                  var Ht = vn.selectAll("text").data(dn);
                  function jt(pt, wt, Bt, At) {
                    if (!pt.match("s") || Bt >= 0 == At >= 0 || wt(Bt).slice(-1).match(Ce) || wt(At).slice(-1).match(Ce)) return wt;
                    var Dt = pt.slice().replace("s", "f").replace(/\d+/, function(un) {
                      return parseInt(un) - 1;
                    }), Ot = Ee(Gt, { tickformat: Dt });
                    return function(un) {
                      return Math.abs(un) < 1 ? he.tickText(Ot, un).text : wt(un);
                    };
                  }
                  Ht.enter().append("text"), Ht.attr("text-anchor", function() {
                    return Ut;
                  }).attr("class", function(pt) {
                    return pt;
                  }).attr("x", null).attr("y", null).attr("dx", null).attr("dy", null), Ht.exit().remove();
                  var cn, wn = Yt.mode + Yt.align;
                  if (Yt._hasDelta && (cn = function() {
                    var pt = Ee(Gt, { tickformat: Yt.delta.valueformat }, Yt._range);
                    pt.setScale(), he.prepTicks(pt);
                    var wt = function(gn) {
                      return he.tickText(pt, gn).text;
                    }, Bt = Yt.delta.suffix, At = Yt.delta.prefix, Dt = function(gn) {
                      return Yt.delta.relative ? gn.relativeDelta : gn.delta;
                    }, Ot = function(gn, bn) {
                      return gn === 0 || typeof gn != "number" || isNaN(gn) ? "-" : (gn > 0 ? Yt.delta.increasing.symbol : Yt.delta.decreasing.symbol) + At + bn(gn) + Bt;
                    }, un = function(gn) {
                      return gn.delta >= 0 ? Yt.delta.increasing.color : Yt.delta.decreasing.color;
                    };
                    Yt._deltaLastValue === void 0 && (Yt._deltaLastValue = Dt(Tt[0]));
                    var yn = vn.select("text.delta");
                    function _n() {
                      yn.text(Ot(Dt(Tt[0]), wt)).call(ge.fill, un(Tt[0])).call(fe.convertToTspans, Gt);
                    }
                    return yn.call(ue.font, Yt.delta.font).call(ge.fill, un({ delta: Yt._deltaLastValue })), Se(Wt) ? yn.transition().duration(Wt.duration).ease(Wt.easing).tween("text", function() {
                      var gn = te.select(this), bn = Dt(Tt[0]), an = Yt._deltaLastValue, Zt = jt(Yt.delta.valueformat, wt, an, bn), Mn = ee(an, bn);
                      return Yt._deltaLastValue = bn, function(Ln) {
                        gn.text(Ot(Mn(Ln), Zt)), gn.call(ge.fill, un({ delta: Mn(Ln) }));
                      };
                    }).each("end", function() {
                      _n(), rn && rn();
                    }).each("interrupt", function() {
                      _n(), rn && rn();
                    }) : _n(), Kt = Le(Ot(Dt(Tt[0]), wt), Yt.delta.font, Ut, Gt), yn;
                  }(), wn += Yt.delta.position + Yt.delta.font.size + Yt.delta.font.family + Yt.delta.valueformat, wn += Yt.delta.increasing.symbol + Yt.delta.decreasing.symbol, Vt = Kt), Yt._hasNumber && (function() {
                    var pt = Ee(Gt, { tickformat: Yt.number.valueformat }, Yt._range);
                    pt.setScale(), he.prepTicks(pt);
                    var wt = function(un) {
                      return he.tickText(pt, un).text;
                    }, Bt = Yt.number.suffix, At = Yt.number.prefix, Dt = vn.select("text.number");
                    function Ot() {
                      var un = typeof Tt[0].y == "number" ? At + wt(Tt[0].y) + Bt : "-";
                      Dt.text(un).call(ue.font, Yt.number.font).call(fe.convertToTspans, Gt);
                    }
                    Se(Wt) ? Dt.transition().duration(Wt.duration).ease(Wt.easing).each("end", function() {
                      Ot(), rn && rn();
                    }).each("interrupt", function() {
                      Ot(), rn && rn();
                    }).attrTween("text", function() {
                      var un = te.select(this), yn = ee(Tt[0].lastY, Tt[0].y);
                      Yt._lastValue = Tt[0].y;
                      var _n = jt(Yt.number.valueformat, wt, Tt[0].lastY, Tt[0].y);
                      return function(gn) {
                        un.text(At + _n(yn(gn)) + Bt);
                      };
                    }) : Ot(), Ct = Le(At + wt(Tt[0].y) + Bt, Yt.number.font, Ut, Gt);
                  }(), wn += Yt.number.font.size + Yt.number.font.family + Yt.number.valueformat + Yt.number.suffix + Yt.number.prefix, Vt = Ct), Yt._hasDelta && Yt._hasNumber) {
                    var Rn, Pn, Ie = [(Ct.left + Ct.right) / 2, (Ct.top + Ct.bottom) / 2], rt = [(Kt.left + Kt.right) / 2, (Kt.top + Kt.bottom) / 2], ft = 0.75 * Yt.delta.font.size;
                    Yt.delta.position === "left" && (Rn = Qe(Yt, "deltaPos", 0, -1 * (Ct.width * be[Yt.align] + Kt.width * (1 - be[Yt.align]) + ft), wn, Math.min), Pn = Ie[1] - rt[1], Vt = { width: Ct.width + Kt.width + ft, height: Math.max(Ct.height, Kt.height), left: Kt.left + Rn, right: Ct.right, top: Math.min(Ct.top, Kt.top + Pn), bottom: Math.max(Ct.bottom, Kt.bottom + Pn) }), Yt.delta.position === "right" && (Rn = Qe(Yt, "deltaPos", 0, Ct.width * (1 - be[Yt.align]) + Kt.width * be[Yt.align] + ft, wn, Math.max), Pn = Ie[1] - rt[1], Vt = { width: Ct.width + Kt.width + ft, height: Math.max(Ct.height, Kt.height), left: Ct.left, right: Kt.right + Rn, top: Math.min(Ct.top, Kt.top + Pn), bottom: Math.max(Ct.bottom, Kt.bottom + Pn) }), Yt.delta.position === "bottom" && (Rn = null, Pn = Kt.height, Vt = { width: Math.max(Ct.width, Kt.width), height: Ct.height + Kt.height, left: Math.min(Ct.left, Kt.left), right: Math.max(Ct.right, Kt.right), top: Ct.bottom - Ct.height, bottom: Ct.bottom + Kt.height }), Yt.delta.position === "top" && (Rn = null, Pn = Ct.top, Vt = { width: Math.max(Ct.width, Kt.width), height: Ct.height + Kt.height, left: Math.min(Ct.left, Kt.left), right: Math.max(Ct.right, Kt.right), top: Ct.bottom - Ct.height - Kt.height, bottom: Ct.bottom }), cn.attr({ dx: Rn, dy: Pn });
                  }
                  (Yt._hasNumber || Yt._hasDelta) && vn.attr("transform", function() {
                    var pt = It.numbersScaler(Vt);
                    wn += pt[2];
                    var wt, Bt = Qe(Yt, "numbersScale", 1, pt[0], wn, Math.min);
                    Yt._scaleNumbers || (Bt = 1), wt = Yt._isAngular ? hn - Bt * Vt.bottom : hn - Bt * (Vt.top + Vt.bottom) / 2, Yt._numbersTop = Bt * Vt.top + wt;
                    var At = Vt[Qt];
                    Qt === "center" && (At = (Vt.left + Vt.right) / 2);
                    var Dt = sn - Bt * At;
                    return Dt = Qe(Yt, "numbersTranslate", 0, Dt, wn, Math.max), se(Dt, wt) + oe(Bt);
                  });
                })(ke, at, qe, { numbersX: Xe, numbersY: Ve, numbersScaler: Ze, transitionOpts: Re, onComplete: Pe }), ct && (Ge = { range: We.gauge.axis.range, color: We.gauge.bgcolor, line: { color: We.gauge.bordercolor, width: 0 }, thickness: 1 }, ze = { range: We.gauge.axis.range, color: "rgba(0, 0, 0, 0)", line: { color: We.gauge.bordercolor, width: We.gauge.borderwidth }, thickness: 1 });
                var vt = at.selectAll("g.angular").data(et ? qe : []);
                vt.exit().remove();
                var $t = at.selectAll("g.angularaxis").data(et ? qe : []);
                $t.exit().remove(), et && function(Gt, Ft, Tt, It) {
                  var Ct, Kt, Vt, Yt, sn = Tt[0].trace, hn = It.size, Qt = It.radius, Ut = It.innerRadius, Wt = It.gaugeBg, rn = It.gaugeOutline, vn = [hn.l + hn.w / 2, hn.t + hn.h / 2 + Qt / 2], dn = It.gauge, Ht = It.layer, jt = It.transitionOpts, cn = It.onComplete, wn = Math.PI / 2;
                  function Rn(Ln) {
                    var In = sn.gauge.axis.range[0], kn = (Ln - In) / (sn.gauge.axis.range[1] - In) * Math.PI - wn;
                    return kn < -wn ? -wn : kn > wn ? wn : kn;
                  }
                  function Pn(Ln) {
                    return te.svg.arc().innerRadius((Ut + Qt) / 2 - Ln / 2 * (Qt - Ut)).outerRadius((Ut + Qt) / 2 + Ln / 2 * (Qt - Ut)).startAngle(-wn);
                  }
                  function Ie(Ln) {
                    Ln.attr("d", function(In) {
                      return Pn(In.thickness).startAngle(Rn(In.range[0])).endAngle(Rn(In.range[1]))();
                    });
                  }
                  dn.enter().append("g").classed("angular", !0), dn.attr("transform", se(vn[0], vn[1])), Ht.enter().append("g").classed("angularaxis", !0).classed("crisp", !0), Ht.selectAll("g.xangularaxistick,path,text").remove(), (Ct = Ee(Gt, sn.gauge.axis)).type = "linear", Ct.range = sn.gauge.axis.range, Ct._id = "xangularaxis", Ct.ticklabeloverflow = "allow", Ct.setScale();
                  var rt = function(Ln) {
                    return (Ct.range[0] - Ln.x) / (Ct.range[1] - Ct.range[0]) * Math.PI + Math.PI;
                  }, ft = {}, pt = he.makeLabelFns(Ct, 0).labelStandoff;
                  ft.xFn = function(Ln) {
                    var In = rt(Ln);
                    return Math.cos(In) * pt;
                  }, ft.yFn = function(Ln) {
                    var In = rt(Ln), kn = Math.sin(In) > 0 ? 0.2 : 1;
                    return -Math.sin(In) * (pt + Ln.fontSize * kn) + Math.abs(Math.cos(In)) * (Ln.fontSize * le);
                  }, ft.anchorFn = function(Ln) {
                    var In = rt(Ln), kn = Math.cos(In);
                    return Math.abs(kn) < 0.1 ? "middle" : kn > 0 ? "start" : "end";
                  }, ft.heightFn = function(Ln, In, kn) {
                    var er = rt(Ln);
                    return -0.5 * (1 + Math.sin(er)) * kn;
                  };
                  var wt = function(Ln) {
                    return se(vn[0] + Qt * Math.cos(Ln), vn[1] - Qt * Math.sin(Ln));
                  };
                  if (Vt = function(Ln) {
                    return wt(rt(Ln));
                  }, Kt = he.calcTicks(Ct), Yt = he.getTickSigns(Ct)[2], Ct.visible) {
                    Yt = Ct.ticks === "inside" ? -1 : 1;
                    var Bt = (Ct.linewidth || 1) / 2;
                    he.drawTicks(Gt, Ct, { vals: Kt, layer: Ht, path: "M" + Yt * Bt + ",0h" + Yt * Ct.ticklen, transFn: function(Ln) {
                      var In = rt(Ln);
                      return wt(In) + "rotate(" + -ce(In) + ")";
                    } }), he.drawLabels(Gt, Ct, { vals: Kt, layer: Ht, transFn: Vt, labelFns: ft });
                  }
                  var At = [Wt].concat(sn.gauge.steps), Dt = dn.selectAll("g.bg-arc").data(At);
                  Dt.enter().append("g").classed("bg-arc", !0).append("path"), Dt.select("path").call(Ie).call(we), Dt.exit().remove();
                  var Ot = Pn(sn.gauge.bar.thickness), un = dn.selectAll("g.value-arc").data([sn.gauge.bar]);
                  un.enter().append("g").classed("value-arc", !0).append("path");
                  var yn, _n, gn, bn = un.select("path");
                  Se(jt) ? (bn.transition().duration(jt.duration).ease(jt.easing).each("end", function() {
                    cn && cn();
                  }).each("interrupt", function() {
                    cn && cn();
                  }).attrTween("d", (yn = Ot, _n = Rn(Tt[0].lastY), gn = Rn(Tt[0].y), function() {
                    var Ln = ne(_n, gn);
                    return function(In) {
                      return yn.endAngle(Ln(In))();
                    };
                  })), sn._lastValue = Tt[0].y) : bn.attr("d", typeof Tt[0].y == "number" ? Ot.endAngle(Rn(Tt[0].y)) : "M0,0Z"), bn.call(we), un.exit().remove(), At = [];
                  var an = sn.gauge.threshold.value;
                  (an || an === 0) && At.push({ range: [an, an], color: sn.gauge.threshold.color, line: { color: sn.gauge.threshold.line.color, width: sn.gauge.threshold.line.width }, thickness: sn.gauge.threshold.thickness });
                  var Zt = dn.selectAll("g.threshold-arc").data(At);
                  Zt.enter().append("g").classed("threshold-arc", !0).append("path"), Zt.select("path").call(Ie).call(we), Zt.exit().remove();
                  var Mn = dn.selectAll("g.gauge-outline").data([rn]);
                  Mn.enter().append("g").classed("gauge-outline", !0).append("path"), Mn.select("path").call(Ie).call(we), Mn.exit().remove();
                }(ke, 0, qe, { radius: bt, innerRadius: yt, gauge: vt, layer: $t, size: it, gaugeBg: Ge, gaugeOutline: ze, transitionOpts: Re, onComplete: Pe });
                var Pt = at.selectAll("g.bullet").data(dt ? qe : []);
                Pt.exit().remove();
                var Xt = at.selectAll("g.bulletaxis").data(dt ? qe : []);
                Xt.exit().remove(), dt && function(Gt, Ft, Tt, It) {
                  var Ct, Kt, Vt, Yt, sn, hn = Tt[0].trace, Qt = It.gauge, Ut = It.layer, Wt = It.gaugeBg, rn = It.gaugeOutline, vn = It.size, dn = hn.domain, Ht = It.transitionOpts, jt = It.onComplete;
                  Qt.enter().append("g").classed("bullet", !0), Qt.attr("transform", se(vn.l, vn.t)), Ut.enter().append("g").classed("bulletaxis", !0).classed("crisp", !0), Ut.selectAll("g.xbulletaxistick,path,text").remove();
                  var cn = vn.h, wn = hn.gauge.bar.thickness * cn, Rn = dn.x[0], Pn = dn.x[0] + (dn.x[1] - dn.x[0]) * (hn._hasNumber || hn._hasDelta ? 1 - de.bulletNumberDomainSize : 1);
                  function Ie(Dt) {
                    Dt.attr("width", function(Ot) {
                      return Math.max(0, Ct.c2p(Ot.range[1]) - Ct.c2p(Ot.range[0]));
                    }).attr("x", function(Ot) {
                      return Ct.c2p(Ot.range[0]);
                    }).attr("y", function(Ot) {
                      return 0.5 * (1 - Ot.thickness) * cn;
                    }).attr("height", function(Ot) {
                      return Ot.thickness * cn;
                    });
                  }
                  (Ct = Ee(Gt, hn.gauge.axis))._id = "xbulletaxis", Ct.domain = [Rn, Pn], Ct.setScale(), Kt = he.calcTicks(Ct), Vt = he.makeTransTickFn(Ct), Yt = he.getTickSigns(Ct)[2], sn = vn.t + vn.h, Ct.visible && (he.drawTicks(Gt, Ct, { vals: Ct.ticks === "inside" ? he.clipEnds(Ct, Kt) : Kt, layer: Ut, path: he.makeTickPath(Ct, sn, Yt), transFn: Vt }), he.drawLabels(Gt, Ct, { vals: Kt, layer: Ut, transFn: Vt, labelFns: he.makeLabelFns(Ct, sn) }));
                  var rt = [Wt].concat(hn.gauge.steps), ft = Qt.selectAll("g.bg-bullet").data(rt);
                  ft.enter().append("g").classed("bg-bullet", !0).append("rect"), ft.select("rect").call(Ie).call(we), ft.exit().remove();
                  var pt = Qt.selectAll("g.value-bullet").data([hn.gauge.bar]);
                  pt.enter().append("g").classed("value-bullet", !0).append("rect"), pt.select("rect").attr("height", wn).attr("y", (cn - wn) / 2).call(we), Se(Ht) ? pt.select("rect").transition().duration(Ht.duration).ease(Ht.easing).each("end", function() {
                    jt && jt();
                  }).each("interrupt", function() {
                    jt && jt();
                  }).attr("width", Math.max(0, Ct.c2p(Math.min(hn.gauge.axis.range[1], Tt[0].y)))) : pt.select("rect").attr("width", typeof Tt[0].y == "number" ? Math.max(0, Ct.c2p(Math.min(hn.gauge.axis.range[1], Tt[0].y))) : 0), pt.exit().remove();
                  var wt = Tt.filter(function() {
                    return hn.gauge.threshold.value || hn.gauge.threshold.value === 0;
                  }), Bt = Qt.selectAll("g.threshold-bullet").data(wt);
                  Bt.enter().append("g").classed("threshold-bullet", !0).append("line"), Bt.select("line").attr("x1", Ct.c2p(hn.gauge.threshold.value)).attr("x2", Ct.c2p(hn.gauge.threshold.value)).attr("y1", (1 - hn.gauge.threshold.thickness) / 2 * cn).attr("y2", (1 - (1 - hn.gauge.threshold.thickness) / 2) * cn).call(ge.stroke, hn.gauge.threshold.line.color).style("stroke-width", hn.gauge.threshold.line.width), Bt.exit().remove();
                  var At = Qt.selectAll("g.gauge-outline").data([rn]);
                  At.enter().append("g").classed("gauge-outline", !0).append("rect"), At.select("rect").call(Ie).call(we), At.exit().remove();
                }(ke, 0, qe, { gauge: Pt, layer: Xt, size: it, gaugeBg: Ge, gaugeOutline: ze, transitionOpts: Re, onComplete: Pe });
                var Nt = at.selectAll("text.title").data(qe);
                Nt.exit().remove(), Nt.enter().append("text").classed("title", !0), Nt.attr("text-anchor", function() {
                  return dt ? ve.right : ve[We.title.align];
                }).text(We.title.text).call(ue.font, We.title.font).call(fe.convertToTspans, ke), Nt.attr("transform", function() {
                  var Gt, Ft = it.l + it.w * be[We.title.align], Tt = de.titlePadding, It = ue.bBox(Nt.node());
                  return ct ? (et && (Gt = We.gauge.axis.visible ? ue.bBox($t.node()).top - Tt - It.bottom : it.t + it.h / 2 - bt / 2 - It.bottom - Tt), dt && (Gt = Ve - (It.top + It.bottom) / 2, Ft = it.l - de.bulletPadding * it.w)) : Gt = We._numbersTop - Tt - It.bottom, se(Ft, Gt);
                });
              });
            };
          }, 70252: function(G, J, Z) {
            var te = Z(87163), ne = Z(80712).axisHoverFormat, ee = Z(3208).rb, ae = Z(42450), oe = Z(9829), se = Z(93049).extendFlat, ce = Z(13582).overrideAll, le = G.exports = ce(se({ x: { valType: "data_array" }, y: { valType: "data_array" }, z: { valType: "data_array" }, value: { valType: "data_array" }, isomin: { valType: "number" }, isomax: { valType: "number" }, surface: { show: { valType: "boolean", dflt: !0 }, count: { valType: "integer", dflt: 2, min: 1 }, fill: { valType: "number", min: 0, max: 1, dflt: 1 }, pattern: { valType: "flaglist", flags: ["A", "B", "C", "D", "E"], extras: ["all", "odd", "even"], dflt: "all" } }, spaceframe: { show: { valType: "boolean", dflt: !1 }, fill: { valType: "number", min: 0, max: 1, dflt: 0.15 } }, slices: { x: { show: { valType: "boolean", dflt: !1 }, locations: { valType: "data_array", dflt: [] }, fill: { valType: "number", min: 0, max: 1, dflt: 1 } }, y: { show: { valType: "boolean", dflt: !1 }, locations: { valType: "data_array", dflt: [] }, fill: { valType: "number", min: 0, max: 1, dflt: 1 } }, z: { show: { valType: "boolean", dflt: !1 }, locations: { valType: "data_array", dflt: [] }, fill: { valType: "number", min: 0, max: 1, dflt: 1 } } }, caps: { x: { show: { valType: "boolean", dflt: !0 }, fill: { valType: "number", min: 0, max: 1, dflt: 1 } }, y: { show: { valType: "boolean", dflt: !0 }, fill: { valType: "number", min: 0, max: 1, dflt: 1 } }, z: { show: { valType: "boolean", dflt: !0 }, fill: { valType: "number", min: 0, max: 1, dflt: 1 } } }, text: { valType: "string", dflt: "", arrayOk: !0 }, hovertext: { valType: "string", dflt: "", arrayOk: !0 }, hovertemplate: ee(), xhoverformat: ne("x"), yhoverformat: ne("y"), zhoverformat: ne("z"), valuehoverformat: ne("value", 1), showlegend: se({}, oe.showlegend, { dflt: !1 }) }, te("", { colorAttr: "`value`", showScaleDflt: !0, editTypeOverride: "calc" }), { opacity: ae.opacity, lightposition: ae.lightposition, lighting: ae.lighting, flatshading: ae.flatshading, contour: ae.contour, hoverinfo: se({}, oe.hoverinfo) }), "calc", "nested");
            le.flatshading.dflt = !0, le.lighting.facenormalsepsilon.dflt = 0, le.x.editType = le.y.editType = le.z.editType = le.value.editType = "calc+clearAxisTypes", le.transforms = void 0;
          }, 58988: function(G, J, Z) {
            var te = Z(28379), ne = Z(36402).processGrid, ee = Z(36402).filter;
            G.exports = function(ae, oe) {
              oe._len = Math.min(oe.x.length, oe.y.length, oe.z.length, oe.value.length), oe._x = ee(oe.x, oe._len), oe._y = ee(oe.y, oe._len), oe._z = ee(oe.z, oe._len), oe._value = ee(oe.value, oe._len);
              var se = ne(oe);
              oe._gridFill = se.fill, oe._Xs = se.Xs, oe._Ys = se.Ys, oe._Zs = se.Zs, oe._len = se.len;
              for (var ce = 1 / 0, le = -1 / 0, ue = 0; ue < oe._len; ue++) {
                var de = oe._value[ue];
                ce = Math.min(ce, de), le = Math.max(le, de);
              }
              oe._minValues = ce, oe._maxValues = le, oe._vMin = oe.isomin === void 0 || oe.isomin === null ? ce : oe.isomin, oe._vMax = oe.isomax === void 0 || oe.isomax === null ? le : oe.isomax, te(ae, oe, { vals: [oe._vMin, oe._vMax], containerStr: "", cLetter: "c" });
            };
          }, 91370: function(G, J, Z) {
            var te = Z(99098).gl_mesh3d, ne = Z(46998).parseColorScale, ee = Z(34809).isArrayOrTypedArray, ae = Z(55010), oe = Z(88856).extractOpts, se = Z(88239), ce = function(he, me) {
              for (var pe = me.length - 1; pe > 0; pe--) {
                var ye = Math.min(me[pe], me[pe - 1]), ge = Math.max(me[pe], me[pe - 1]);
                if (ge > ye && ye < he && he <= ge) return { id: pe, distRatio: (ge - he) / (ge - ye) };
              }
              return { id: 0, distRatio: 0 };
            };
            function le(he, me, pe) {
              this.scene = he, this.uid = pe, this.mesh = me, this.name = "", this.data = null, this.showContour = !1;
            }
            var ue = le.prototype;
            ue.handlePick = function(he) {
              if (he.object === this.mesh) {
                var me = he.data.index, pe = this.data._meshX[me], ye = this.data._meshY[me], ge = this.data._meshZ[me], ve = this.data._Ys.length, be = this.data._Zs.length, Ce = ce(pe, this.data._Xs).id, Se = ce(ye, this.data._Ys).id, we = ce(ge, this.data._Zs).id, Ee = he.index = we + be * Se + be * ve * Ce;
                he.traceCoordinate = [this.data._meshX[Ee], this.data._meshY[Ee], this.data._meshZ[Ee], this.data._value[Ee]];
                var xe = this.data.hovertext || this.data.text;
                return ee(xe) && xe[Ee] !== void 0 ? he.textLabel = xe[Ee] : xe && (he.textLabel = xe), !0;
              }
            }, ue.update = function(he) {
              var me = this.scene, pe = me.fullSceneLayout;
              function ye(be, Ce, Se, we) {
                return Ce.map(function(Ee) {
                  return be.d2l(Ee, 0, we) * Se;
                });
              }
              this.data = fe(he);
              var ge = { positions: se(ye(pe.xaxis, he._meshX, me.dataScale[0], he.xcalendar), ye(pe.yaxis, he._meshY, me.dataScale[1], he.ycalendar), ye(pe.zaxis, he._meshZ, me.dataScale[2], he.zcalendar)), cells: se(he._meshI, he._meshJ, he._meshK), lightPosition: [he.lightposition.x, he.lightposition.y, he.lightposition.z], ambient: he.lighting.ambient, diffuse: he.lighting.diffuse, specular: he.lighting.specular, roughness: he.lighting.roughness, fresnel: he.lighting.fresnel, vertexNormalsEpsilon: he.lighting.vertexnormalsepsilon, faceNormalsEpsilon: he.lighting.facenormalsepsilon, opacity: he.opacity, contourEnable: he.contour.show, contourColor: ae(he.contour.color).slice(0, 3), contourWidth: he.contour.width, useFacetNormals: he.flatshading }, ve = oe(he);
              ge.vertexIntensity = he._meshIntensity, ge.vertexIntensityBounds = [ve.min, ve.max], ge.colormap = ne(he), this.mesh.update(ge);
            }, ue.dispose = function() {
              this.scene.glplot.remove(this.mesh), this.mesh.dispose();
            };
            var de = ["xyz", "xzy", "yxz", "yzx", "zxy", "zyx"];
            function fe(he) {
              he._meshI = [], he._meshJ = [], he._meshK = [];
              var me, pe, ye, ge, ve, be, Ce, Se = he.surface.show, we = he.spaceframe.show, Ee = he.surface.fill, xe = he.spaceframe.fill, Le = !1, Qe = !1, ke = 0, Me = he._Xs, Re = he._Ys, De = he._Zs, Pe = Me.length, $e = Re.length, qe = De.length, Xe = de.indexOf(he._gridFill.replace(/-/g, "").replace(/\+/g, "")), Ve = function(Ut, Wt, rn) {
                switch (Xe) {
                  case 5:
                    return rn + qe * Wt + qe * $e * Ut;
                  case 4:
                    return rn + qe * Ut + qe * Pe * Wt;
                  case 3:
                    return Wt + $e * rn + $e * qe * Ut;
                  case 2:
                    return Wt + $e * Ut + $e * Pe * rn;
                  case 1:
                    return Ut + Pe * rn + Pe * qe * Wt;
                  default:
                    return Ut + Pe * Wt + Pe * $e * rn;
                }
              }, Ze = he._minValues, Ge = he._maxValues, ze = he._vMin, We = he._vMax;
              function at(Ut, Wt, rn) {
                for (var vn = be.length, dn = pe; dn < vn; dn++) if (Ut === ye[dn] && Wt === ge[dn] && rn === ve[dn]) return dn;
                return -1;
              }
              function ct() {
                pe = me;
              }
              function et() {
                ye = [], ge = [], ve = [], be = [], me = 0, ct();
              }
              function dt(Ut, Wt, rn, vn) {
                return ye.push(Ut), ge.push(Wt), ve.push(rn), be.push(vn), ++me - 1;
              }
              function Je(Ut, Wt, rn) {
                for (var vn = [], dn = 0; dn < Ut.length; dn++) vn[dn] = Ut[dn] * (1 - rn) + rn * Wt[dn];
                return vn;
              }
              function it(Ut) {
                Ce = Ut;
              }
              function st(Ut, Wt) {
                return Ut === "all" || Ut === null || Ut.indexOf(Wt) > -1;
              }
              function gt(Ut, Wt) {
                return Ut === null ? Wt : Ut;
              }
              function bt(Ut, Wt, rn) {
                ct();
                var vn, dn, Ht, jt = [Wt], cn = [rn];
                if (Ce >= 1) jt = [Wt], cn = [rn];
                else if (Ce > 0) {
                  var wn = function(At, Dt) {
                    var Ot = At[0], un = At[1], yn = At[2], _n = function(kn, er, jn) {
                      for (var ur = [], Kn = 0; Kn < kn.length; Kn++) ur[Kn] = (kn[Kn] + er[Kn] + jn[Kn]) / 3;
                      return ur;
                    }(Ot, un, yn), gn = Math.sqrt(1 - Ce), bn = Je(_n, Ot, gn), an = Je(_n, un, gn), Zt = Je(_n, yn, gn), Mn = Dt[0], Ln = Dt[1], In = Dt[2];
                    return { xyzv: [[Ot, un, an], [an, bn, Ot], [un, yn, Zt], [Zt, an, un], [yn, Ot, bn], [bn, Zt, yn]], abc: [[Mn, Ln, -1], [-1, -1, Mn], [Ln, In, -1], [-1, -1, Ln], [In, Mn, -1], [-1, -1, In]] };
                  }(Wt, rn);
                  jt = wn.xyzv, cn = wn.abc;
                }
                for (var Rn = 0; Rn < jt.length; Rn++) {
                  Wt = jt[Rn], rn = cn[Rn];
                  for (var Pn = [], Ie = 0; Ie < 3; Ie++) {
                    var rt = Wt[Ie][0], ft = Wt[Ie][1], pt = Wt[Ie][2], wt = Wt[Ie][3], Bt = rn[Ie] > -1 ? rn[Ie] : at(rt, ft, pt);
                    Pn[Ie] = Bt > -1 ? Bt : dt(rt, ft, pt, gt(Ut, wt));
                  }
                  vn = Pn[0], dn = Pn[1], Ht = Pn[2], he._meshI.push(vn), he._meshJ.push(dn), he._meshK.push(Ht), ++ke;
                }
              }
              function yt(Ut, Wt, rn, vn) {
                var dn = Ut[3];
                dn < rn && (dn = rn), dn > vn && (dn = vn);
                for (var Ht = (Ut[3] - dn) / (Ut[3] - Wt[3] + 1e-9), jt = [], cn = 0; cn < 4; cn++) jt[cn] = (1 - Ht) * Ut[cn] + Ht * Wt[cn];
                return jt;
              }
              function St(Ut, Wt, rn) {
                return Ut >= Wt && Ut <= rn;
              }
              function Lt(Ut) {
                var Wt = 1e-3 * (We - ze);
                return Ut >= ze - Wt && Ut <= We + Wt;
              }
              function Rt(Ut) {
                for (var Wt = [], rn = 0; rn < 4; rn++) {
                  var vn = Ut[rn];
                  Wt.push([he._x[vn], he._y[vn], he._z[vn], he._value[vn]]);
                }
                return Wt;
              }
              var vt = 3;
              function $t(Ut, Wt, rn, vn, dn, Ht) {
                Ht || (Ht = 1), rn = [-1, -1, -1];
                var jt = !1, cn = [St(Wt[0][3], vn, dn), St(Wt[1][3], vn, dn), St(Wt[2][3], vn, dn)];
                if (!cn[0] && !cn[1] && !cn[2]) return !1;
                var wn = function(Pn, Ie, rt) {
                  return Lt(Ie[0][3]) && Lt(Ie[1][3]) && Lt(Ie[2][3]) ? (bt(Pn, Ie, rt), !0) : Ht < vt && $t(Pn, Ie, rt, ze, We, ++Ht);
                };
                if (cn[0] && cn[1] && cn[2]) return wn(Ut, Wt, rn) || jt;
                var Rn = !1;
                return [[0, 1, 2], [2, 0, 1], [1, 2, 0]].forEach(function(Pn) {
                  if (cn[Pn[0]] && cn[Pn[1]] && !cn[Pn[2]]) {
                    var Ie = Wt[Pn[0]], rt = Wt[Pn[1]], ft = Wt[Pn[2]], pt = yt(ft, Ie, vn, dn), wt = yt(ft, rt, vn, dn);
                    jt = wn(Ut, [wt, pt, Ie], [-1, -1, rn[Pn[0]]]) || jt, jt = wn(Ut, [Ie, rt, wt], [rn[Pn[0]], rn[Pn[1]], -1]) || jt, Rn = !0;
                  }
                }), Rn || [[0, 1, 2], [1, 2, 0], [2, 0, 1]].forEach(function(Pn) {
                  if (cn[Pn[0]] && !cn[Pn[1]] && !cn[Pn[2]]) {
                    var Ie = Wt[Pn[0]], rt = Wt[Pn[1]], ft = Wt[Pn[2]], pt = yt(rt, Ie, vn, dn), wt = yt(ft, Ie, vn, dn);
                    jt = wn(Ut, [wt, pt, Ie], [-1, -1, rn[Pn[0]]]) || jt, Rn = !0;
                  }
                }), jt;
              }
              function Pt(Ut, Wt, rn, vn) {
                var dn = !1, Ht = Rt(Wt), jt = [St(Ht[0][3], rn, vn), St(Ht[1][3], rn, vn), St(Ht[2][3], rn, vn), St(Ht[3][3], rn, vn)];
                if (!(jt[0] || jt[1] || jt[2] || jt[3])) return dn;
                if (jt[0] && jt[1] && jt[2] && jt[3]) return Qe && (dn = function(wn, Rn, Pn) {
                  var Ie = function(rt, ft, pt) {
                    bt(wn, [Rn[rt], Rn[ft], Rn[pt]], [Pn[rt], Pn[ft], Pn[pt]]);
                  };
                  Ie(0, 1, 2), Ie(3, 0, 1), Ie(2, 3, 0), Ie(1, 2, 3);
                }(Ut, Ht, Wt) || dn), dn;
                var cn = !1;
                return [[0, 1, 2, 3], [3, 0, 1, 2], [2, 3, 0, 1], [1, 2, 3, 0]].forEach(function(wn) {
                  if (jt[wn[0]] && jt[wn[1]] && jt[wn[2]] && !jt[wn[3]]) {
                    var Rn = Ht[wn[0]], Pn = Ht[wn[1]], Ie = Ht[wn[2]], rt = Ht[wn[3]];
                    if (Qe) dn = bt(Ut, [Rn, Pn, Ie], [Wt[wn[0]], Wt[wn[1]], Wt[wn[2]]]) || dn;
                    else {
                      var ft = yt(rt, Rn, rn, vn), pt = yt(rt, Pn, rn, vn), wt = yt(rt, Ie, rn, vn);
                      dn = bt(null, [ft, pt, wt], [-1, -1, -1]) || dn;
                    }
                    cn = !0;
                  }
                }), cn || ([[0, 1, 2, 3], [1, 2, 3, 0], [2, 3, 0, 1], [3, 0, 1, 2], [0, 2, 3, 1], [1, 3, 2, 0]].forEach(function(wn) {
                  if (jt[wn[0]] && jt[wn[1]] && !jt[wn[2]] && !jt[wn[3]]) {
                    var Rn = Ht[wn[0]], Pn = Ht[wn[1]], Ie = Ht[wn[2]], rt = Ht[wn[3]], ft = yt(Ie, Rn, rn, vn), pt = yt(Ie, Pn, rn, vn), wt = yt(rt, Pn, rn, vn), Bt = yt(rt, Rn, rn, vn);
                    Qe ? (dn = bt(Ut, [Rn, Bt, ft], [Wt[wn[0]], -1, -1]) || dn, dn = bt(Ut, [Pn, pt, wt], [Wt[wn[1]], -1, -1]) || dn) : dn = function(At, Dt, Ot) {
                      var un = function(yn, _n, gn) {
                        bt(null, [Dt[yn], Dt[_n], Dt[gn]], [Ot[yn], Ot[_n], Ot[gn]]);
                      };
                      un(0, 1, 2), un(2, 3, 0);
                    }(0, [ft, pt, wt, Bt], [-1, -1, -1, -1]) || dn, cn = !0;
                  }
                }), cn || [[0, 1, 2, 3], [1, 2, 3, 0], [2, 3, 0, 1], [3, 0, 1, 2]].forEach(function(wn) {
                  if (jt[wn[0]] && !jt[wn[1]] && !jt[wn[2]] && !jt[wn[3]]) {
                    var Rn = Ht[wn[0]], Pn = Ht[wn[1]], Ie = Ht[wn[2]], rt = Ht[wn[3]], ft = yt(Pn, Rn, rn, vn), pt = yt(Ie, Rn, rn, vn), wt = yt(rt, Rn, rn, vn);
                    Qe ? (dn = bt(Ut, [Rn, ft, pt], [Wt[wn[0]], -1, -1]) || dn, dn = bt(Ut, [Rn, pt, wt], [Wt[wn[0]], -1, -1]) || dn, dn = bt(Ut, [Rn, wt, ft], [Wt[wn[0]], -1, -1]) || dn) : dn = bt(null, [ft, pt, wt], [-1, -1, -1]) || dn, cn = !0;
                  }
                })), dn;
              }
              function Xt(Ut, Wt, rn, vn, dn, Ht, jt, cn, wn, Rn, Pn) {
                var Ie = !1;
                return Le && (st(Ut, "A") && (Ie = Pt(null, [Wt, rn, vn, Ht], Rn, Pn) || Ie), st(Ut, "B") && (Ie = Pt(null, [rn, vn, dn, wn], Rn, Pn) || Ie), st(Ut, "C") && (Ie = Pt(null, [rn, Ht, jt, wn], Rn, Pn) || Ie), st(Ut, "D") && (Ie = Pt(null, [vn, Ht, cn, wn], Rn, Pn) || Ie), st(Ut, "E") && (Ie = Pt(null, [rn, vn, Ht, wn], Rn, Pn) || Ie)), Qe && (Ie = Pt(Ut, [rn, vn, Ht, wn], Rn, Pn) || Ie), Ie;
              }
              function Nt(Ut, Wt, rn, vn, dn, Ht, jt, cn) {
                return [cn[0] === !0 || $t(Ut, Rt([Wt, rn, vn]), [Wt, rn, vn], Ht, jt), cn[1] === !0 || $t(Ut, Rt([vn, dn, Wt]), [vn, dn, Wt], Ht, jt)];
              }
              function Gt(Ut, Wt, rn, vn, dn, Ht, jt, cn, wn) {
                return cn ? Nt(Ut, Wt, rn, dn, vn, Ht, jt, wn) : Nt(Ut, rn, dn, vn, Wt, Ht, jt, wn);
              }
              function Ft(Ut, Wt, rn, vn, dn, Ht, jt) {
                var cn, wn, Rn, Pn, Ie = !1, rt = function() {
                  Ie = $t(Ut, [cn, wn, Rn], [-1, -1, -1], dn, Ht) || Ie, Ie = $t(Ut, [Rn, Pn, cn], [-1, -1, -1], dn, Ht) || Ie;
                }, ft = jt[0], pt = jt[1], wt = jt[2];
                return ft && (cn = Je(Rt([Ve(Wt, rn - 0, vn - 0)])[0], Rt([Ve(Wt - 1, rn - 0, vn - 0)])[0], ft), wn = Je(Rt([Ve(Wt, rn - 0, vn - 1)])[0], Rt([Ve(Wt - 1, rn - 0, vn - 1)])[0], ft), Rn = Je(Rt([Ve(Wt, rn - 1, vn - 1)])[0], Rt([Ve(Wt - 1, rn - 1, vn - 1)])[0], ft), Pn = Je(Rt([Ve(Wt, rn - 1, vn - 0)])[0], Rt([Ve(Wt - 1, rn - 1, vn - 0)])[0], ft), rt()), pt && (cn = Je(Rt([Ve(Wt - 0, rn, vn - 0)])[0], Rt([Ve(Wt - 0, rn - 1, vn - 0)])[0], pt), wn = Je(Rt([Ve(Wt - 0, rn, vn - 1)])[0], Rt([Ve(Wt - 0, rn - 1, vn - 1)])[0], pt), Rn = Je(Rt([Ve(Wt - 1, rn, vn - 1)])[0], Rt([Ve(Wt - 1, rn - 1, vn - 1)])[0], pt), Pn = Je(Rt([Ve(Wt - 1, rn, vn - 0)])[0], Rt([Ve(Wt - 1, rn - 1, vn - 0)])[0], pt), rt()), wt && (cn = Je(Rt([Ve(Wt - 0, rn - 0, vn)])[0], Rt([Ve(Wt - 0, rn - 0, vn - 1)])[0], wt), wn = Je(Rt([Ve(Wt - 0, rn - 1, vn)])[0], Rt([Ve(Wt - 0, rn - 1, vn - 1)])[0], wt), Rn = Je(Rt([Ve(Wt - 1, rn - 1, vn)])[0], Rt([Ve(Wt - 1, rn - 1, vn - 1)])[0], wt), Pn = Je(Rt([Ve(Wt - 1, rn - 0, vn)])[0], Rt([Ve(Wt - 1, rn - 0, vn - 1)])[0], wt), rt()), Ie;
              }
              function Tt(Ut, Wt, rn, vn, dn, Ht, jt, cn, wn, Rn, Pn, Ie) {
                var rt = Ut;
                return Ie ? (Le && Ut === "even" && (rt = null), Xt(rt, Wt, rn, vn, dn, Ht, jt, cn, wn, Rn, Pn)) : (Le && Ut === "odd" && (rt = null), Xt(rt, wn, cn, jt, Ht, dn, vn, rn, Wt, Rn, Pn));
              }
              function It(Ut, Wt, rn, vn, dn) {
                for (var Ht = [], jt = 0, cn = 0; cn < Wt.length; cn++) for (var wn = Wt[cn], Rn = 1; Rn < qe; Rn++) for (var Pn = 1; Pn < $e; Pn++) Ht.push(Gt(Ut, Ve(wn, Pn - 1, Rn - 1), Ve(wn, Pn - 1, Rn), Ve(wn, Pn, Rn - 1), Ve(wn, Pn, Rn), rn, vn, (wn + Pn + Rn) % 2, dn && dn[jt] ? dn[jt] : [])), jt++;
                return Ht;
              }
              function Ct(Ut, Wt, rn, vn, dn) {
                for (var Ht = [], jt = 0, cn = 0; cn < Wt.length; cn++) for (var wn = Wt[cn], Rn = 1; Rn < Pe; Rn++) for (var Pn = 1; Pn < qe; Pn++) Ht.push(Gt(Ut, Ve(Rn - 1, wn, Pn - 1), Ve(Rn, wn, Pn - 1), Ve(Rn - 1, wn, Pn), Ve(Rn, wn, Pn), rn, vn, (Rn + wn + Pn) % 2, dn && dn[jt] ? dn[jt] : [])), jt++;
                return Ht;
              }
              function Kt(Ut, Wt, rn, vn, dn) {
                for (var Ht = [], jt = 0, cn = 0; cn < Wt.length; cn++) for (var wn = Wt[cn], Rn = 1; Rn < $e; Rn++) for (var Pn = 1; Pn < Pe; Pn++) Ht.push(Gt(Ut, Ve(Pn - 1, Rn - 1, wn), Ve(Pn - 1, Rn, wn), Ve(Pn, Rn - 1, wn), Ve(Pn, Rn, wn), rn, vn, (Pn + Rn + wn) % 2, dn && dn[jt] ? dn[jt] : [])), jt++;
                return Ht;
              }
              function Vt(Ut, Wt, rn) {
                for (var vn = 1; vn < qe; vn++) for (var dn = 1; dn < $e; dn++) for (var Ht = 1; Ht < Pe; Ht++) Tt(Ut, Ve(Ht - 1, dn - 1, vn - 1), Ve(Ht - 1, dn - 1, vn), Ve(Ht - 1, dn, vn - 1), Ve(Ht - 1, dn, vn), Ve(Ht, dn - 1, vn - 1), Ve(Ht, dn - 1, vn), Ve(Ht, dn, vn - 1), Ve(Ht, dn, vn), Wt, rn, (Ht + dn + vn) % 2);
              }
              function Yt(Ut, Wt, rn, vn, dn, Ht) {
                for (var jt = [], cn = 0, wn = 0; wn < Wt.length; wn++) for (var Rn = Wt[wn], Pn = 1; Pn < qe; Pn++) for (var Ie = 1; Ie < $e; Ie++) jt.push(Ft(Ut, Rn, Ie, Pn, rn, vn, dn[wn], Ht && Ht[cn] && Ht[cn])), cn++;
                return jt;
              }
              function sn(Ut, Wt, rn, vn, dn, Ht) {
                for (var jt = [], cn = 0, wn = 0; wn < Wt.length; wn++) for (var Rn = Wt[wn], Pn = 1; Pn < Pe; Pn++) for (var Ie = 1; Ie < qe; Ie++) jt.push(Ft(Ut, Pn, Rn, Ie, rn, vn, dn[wn], Ht && Ht[cn] && Ht[cn])), cn++;
                return jt;
              }
              function hn(Ut, Wt, rn, vn, dn, Ht) {
                for (var jt = [], cn = 0, wn = 0; wn < Wt.length; wn++) for (var Rn = Wt[wn], Pn = 1; Pn < $e; Pn++) for (var Ie = 1; Ie < Pe; Ie++) jt.push(Ft(Ut, Ie, Pn, Rn, rn, vn, dn[wn], Ht && Ht[cn] && Ht[cn])), cn++;
                return jt;
              }
              function Qt(Ut, Wt) {
                for (var rn = [], vn = Ut; vn < Wt; vn++) rn.push(vn);
                return rn;
              }
              return function() {
                et(), function() {
                  for (var wn = 0; wn < Pe; wn++) for (var Rn = 0; Rn < $e; Rn++) for (var Pn = 0; Pn < qe; Pn++) {
                    var Ie = Ve(wn, Rn, Pn);
                    dt(he._x[Ie], he._y[Ie], he._z[Ie], he._value[Ie]);
                  }
                }();
                var Ut = null;
                if (we && xe && (it(xe), Qe = !0, Vt(Ut, ze, We), Qe = !1), Se && Ee) {
                  it(Ee);
                  for (var Wt = he.surface.pattern, rn = he.surface.count, vn = 0; vn < rn; vn++) {
                    var dn = rn === 1 ? 0.5 : vn / (rn - 1), Ht = (1 - dn) * ze + dn * We, jt = Math.abs(Ht - Ze) > Math.abs(Ht - Ge) ? [Ze, Ht] : [Ht, Ge];
                    Le = !0, Vt(Wt, jt[0], jt[1]), Le = !1;
                  }
                }
                var cn = [[Math.min(ze, Ge), Math.max(ze, Ge)], [Math.min(Ze, We), Math.max(Ze, We)]];
                ["x", "y", "z"].forEach(function(wn) {
                  for (var Rn = [], Pn = 0; Pn < cn.length; Pn++) {
                    var Ie = 0, rt = cn[Pn][0], ft = cn[Pn][1], pt = he.slices[wn];
                    if (pt.show && pt.fill) {
                      it(pt.fill);
                      var wt = [], Bt = [], At = [];
                      if (pt.locations.length) for (var Dt = 0; Dt < pt.locations.length; Dt++) {
                        var Ot = ce(pt.locations[Dt], wn === "x" ? Me : wn === "y" ? Re : De);
                        Ot.distRatio === 0 ? wt.push(Ot.id) : Ot.id > 0 && (Bt.push(Ot.id), wn === "x" ? At.push([Ot.distRatio, 0, 0]) : wn === "y" ? At.push([0, Ot.distRatio, 0]) : At.push([0, 0, Ot.distRatio]));
                      }
                      else wt = Qt(1, wn === "x" ? Pe - 1 : wn === "y" ? $e - 1 : qe - 1);
                      Bt.length > 0 && (Rn[Ie] = wn === "x" ? Yt(Ut, Bt, rt, ft, At, Rn[Ie]) : wn === "y" ? sn(Ut, Bt, rt, ft, At, Rn[Ie]) : hn(Ut, Bt, rt, ft, At, Rn[Ie]), Ie++), wt.length > 0 && (Rn[Ie] = wn === "x" ? It(Ut, wt, rt, ft, Rn[Ie]) : wn === "y" ? Ct(Ut, wt, rt, ft, Rn[Ie]) : Kt(Ut, wt, rt, ft, Rn[Ie]), Ie++);
                    }
                    var un = he.caps[wn];
                    un.show && un.fill && (it(un.fill), Rn[Ie] = wn === "x" ? It(Ut, [0, Pe - 1], rt, ft, Rn[Ie]) : wn === "y" ? Ct(Ut, [0, $e - 1], rt, ft, Rn[Ie]) : Kt(Ut, [0, qe - 1], rt, ft, Rn[Ie]), Ie++);
                  }
                }), ke === 0 && et(), he._meshX = ye, he._meshY = ge, he._meshZ = ve, he._meshIntensity = be, he._Xs = Me, he._Ys = Re, he._Zs = De;
              }(), he;
            }
            G.exports = { findNearestOnAxis: ce, generateIsoMeshes: fe, createIsosurfaceTrace: function(he, me) {
              var pe = he.glplot.gl, ye = te({ gl: pe }), ge = new le(he, ye, me.uid);
              return ye._trace = ge, ge.update(me), he.glplot.add(ye), ge;
            } };
          }, 44731: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626), ee = Z(70252), ae = Z(39356);
            function oe(se, ce, le, ue, de) {
              var fe = de("isomin"), he = de("isomax");
              he != null && fe != null && fe > he && (ce.isomin = null, ce.isomax = null);
              var me = de("x"), pe = de("y"), ye = de("z"), ge = de("value");
              me && me.length && pe && pe.length && ye && ye.length && ge && ge.length ? (ne.getComponentMethod("calendars", "handleTraceDefaults")(se, ce, ["x", "y", "z"], ue), de("valuehoverformat"), ["x", "y", "z"].forEach(function(ve) {
                de(ve + "hoverformat");
                var be = "caps." + ve;
                de(be + ".show") && de(be + ".fill");
                var Ce = "slices." + ve;
                de(Ce + ".show") && (de(Ce + ".fill"), de(Ce + ".locations"));
              }), de("spaceframe.show") && de("spaceframe.fill"), de("surface.show") && (de("surface.count"), de("surface.fill"), de("surface.pattern")), de("contour.show") && (de("contour.color"), de("contour.width")), ["text", "hovertext", "hovertemplate", "lighting.ambient", "lighting.diffuse", "lighting.specular", "lighting.roughness", "lighting.fresnel", "lighting.vertexnormalsepsilon", "lighting.facenormalsepsilon", "lightposition.x", "lightposition.y", "lightposition.z", "flatshading", "opacity"].forEach(function(ve) {
                de(ve);
              }), ae(se, ce, ue, de, { prefix: "", cLetter: "c" }), ce._length = null) : ce.visible = !1;
            }
            G.exports = { supplyDefaults: function(se, ce, le, ue) {
              oe(se, ce, 0, ue, function(de, fe) {
                return te.coerce(se, ce, ee, de, fe);
              });
            }, supplyIsoDefaults: oe };
          }, 75297: function(G, J, Z) {
            G.exports = { attributes: Z(70252), supplyDefaults: Z(44731).supplyDefaults, calc: Z(58988), colorbar: { min: "cmin", max: "cmax" }, plot: Z(91370).createIsosurfaceTrace, moduleType: "trace", name: "isosurface", basePlotModule: Z(2487), categories: ["gl3d", "showLegend"], meta: {} };
          }, 42450: function(G, J, Z) {
            var te = Z(87163), ne = Z(80712).axisHoverFormat, ee = Z(3208).rb, ae = Z(16131), oe = Z(9829), se = Z(93049).extendFlat;
            G.exports = se({ x: { valType: "data_array", editType: "calc+clearAxisTypes" }, y: { valType: "data_array", editType: "calc+clearAxisTypes" }, z: { valType: "data_array", editType: "calc+clearAxisTypes" }, i: { valType: "data_array", editType: "calc" }, j: { valType: "data_array", editType: "calc" }, k: { valType: "data_array", editType: "calc" }, text: { valType: "string", dflt: "", arrayOk: !0, editType: "calc" }, hovertext: { valType: "string", dflt: "", arrayOk: !0, editType: "calc" }, hovertemplate: ee({ editType: "calc" }), xhoverformat: ne("x"), yhoverformat: ne("y"), zhoverformat: ne("z"), delaunayaxis: { valType: "enumerated", values: ["x", "y", "z"], dflt: "z", editType: "calc" }, alphahull: { valType: "number", dflt: -1, editType: "calc" }, intensity: { valType: "data_array", editType: "calc" }, intensitymode: { valType: "enumerated", values: ["vertex", "cell"], dflt: "vertex", editType: "calc" }, color: { valType: "color", editType: "calc" }, vertexcolor: { valType: "data_array", editType: "calc" }, facecolor: { valType: "data_array", editType: "calc" }, transforms: void 0 }, te("", { colorAttr: "`intensity`", showScaleDflt: !0, editTypeOverride: "calc" }), { opacity: ae.opacity, flatshading: { valType: "boolean", dflt: !1, editType: "calc" }, contour: { show: se({}, ae.contours.x.show, {}), color: ae.contours.x.color, width: ae.contours.x.width, editType: "calc" }, lightposition: { x: se({}, ae.lightposition.x, { dflt: 1e5 }), y: se({}, ae.lightposition.y, { dflt: 1e5 }), z: se({}, ae.lightposition.z, { dflt: 0 }), editType: "calc" }, lighting: se({ vertexnormalsepsilon: { valType: "number", min: 0, max: 1, dflt: 1e-12, editType: "calc" }, facenormalsepsilon: { valType: "number", min: 0, max: 1, dflt: 1e-6, editType: "calc" }, editType: "calc" }, ae.lighting), hoverinfo: se({}, oe.hoverinfo, { editType: "calc" }), showlegend: se({}, oe.showlegend, { dflt: !1 }) });
          }, 44878: function(G, J, Z) {
            var te = Z(28379);
            G.exports = function(ne, ee) {
              ee.intensity && te(ne, ee, { vals: ee.intensity, containerStr: "", cLetter: "c" });
            };
          }, 82836: function(G, J, Z) {
            var te = Z(99098).gl_mesh3d, ne = Z(99098).delaunay_triangulate, ee = Z(99098).alpha_shape, ae = Z(99098).convex_hull, oe = Z(46998).parseColorScale, se = Z(34809).isArrayOrTypedArray, ce = Z(55010), le = Z(88856).extractOpts, ue = Z(88239);
            function de(ge, ve, be) {
              this.scene = ge, this.uid = be, this.mesh = ve, this.name = "", this.color = "#fff", this.data = null, this.showContour = !1;
            }
            var fe = de.prototype;
            function he(ge) {
              for (var ve = [], be = ge.length, Ce = 0; Ce < be; Ce++) ve[Ce] = ce(ge[Ce]);
              return ve;
            }
            function me(ge, ve, be, Ce) {
              for (var Se = [], we = ve.length, Ee = 0; Ee < we; Ee++) Se[Ee] = ge.d2l(ve[Ee], 0, Ce) * be;
              return Se;
            }
            function pe(ge) {
              for (var ve = [], be = ge.length, Ce = 0; Ce < be; Ce++) ve[Ce] = Math.round(ge[Ce]);
              return ve;
            }
            function ye(ge, ve) {
              for (var be = ge.length, Ce = 0; Ce < be; Ce++) if (ge[Ce] <= -0.5 || ge[Ce] >= ve - 0.5) return !1;
              return !0;
            }
            fe.handlePick = function(ge) {
              if (ge.object === this.mesh) {
                var ve = ge.index = ge.data.index;
                ge.data._cellCenter ? ge.traceCoordinate = ge.data.dataCoordinate : ge.traceCoordinate = [this.data.x[ve], this.data.y[ve], this.data.z[ve]];
                var be = this.data.hovertext || this.data.text;
                return se(be) && be[ve] !== void 0 ? ge.textLabel = be[ve] : be && (ge.textLabel = be), !0;
              }
            }, fe.update = function(ge) {
              var ve = this.scene, be = ve.fullSceneLayout;
              this.data = ge;
              var Ce, Se = ge.x.length, we = ue(me(be.xaxis, ge.x, ve.dataScale[0], ge.xcalendar), me(be.yaxis, ge.y, ve.dataScale[1], ge.ycalendar), me(be.zaxis, ge.z, ve.dataScale[2], ge.zcalendar));
              if (ge.i && ge.j && ge.k) {
                if (ge.i.length !== ge.j.length || ge.j.length !== ge.k.length || !ye(ge.i, Se) || !ye(ge.j, Se) || !ye(ge.k, Se)) return;
                Ce = ue(pe(ge.i), pe(ge.j), pe(ge.k));
              } else Ce = ge.alphahull === 0 ? ae(we) : ge.alphahull > 0 ? ee(ge.alphahull, we) : function(Qe, ke) {
                for (var Me = ["x", "y", "z"].indexOf(Qe), Re = [], De = ke.length, Pe = 0; Pe < De; Pe++) Re[Pe] = [ke[Pe][(Me + 1) % 3], ke[Pe][(Me + 2) % 3]];
                return ne(Re);
              }(ge.delaunayaxis, we);
              var Ee = { positions: we, cells: Ce, lightPosition: [ge.lightposition.x, ge.lightposition.y, ge.lightposition.z], ambient: ge.lighting.ambient, diffuse: ge.lighting.diffuse, specular: ge.lighting.specular, roughness: ge.lighting.roughness, fresnel: ge.lighting.fresnel, vertexNormalsEpsilon: ge.lighting.vertexnormalsepsilon, faceNormalsEpsilon: ge.lighting.facenormalsepsilon, opacity: ge.opacity, contourEnable: ge.contour.show, contourColor: ce(ge.contour.color).slice(0, 3), contourWidth: ge.contour.width, useFacetNormals: ge.flatshading };
              if (ge.intensity) {
                var xe = le(ge);
                this.color = "#fff";
                var Le = ge.intensitymode;
                Ee[Le + "Intensity"] = ge.intensity, Ee[Le + "IntensityBounds"] = [xe.min, xe.max], Ee.colormap = oe(ge);
              } else ge.vertexcolor ? (this.color = ge.vertexcolor[0], Ee.vertexColors = he(ge.vertexcolor)) : ge.facecolor ? (this.color = ge.facecolor[0], Ee.cellColors = he(ge.facecolor)) : (this.color = ge.color, Ee.meshColor = ce(ge.color));
              this.mesh.update(Ee);
            }, fe.dispose = function() {
              this.scene.glplot.remove(this.mesh), this.mesh.dispose();
            }, G.exports = function(ge, ve) {
              var be = ge.glplot.gl, Ce = te({ gl: be }), Se = new de(ge, Ce, ve.uid);
              return Ce._trace = Se, Se.update(ve), ge.glplot.add(Ce), Se;
            };
          }, 13573: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(39356), ae = Z(42450);
            G.exports = function(oe, se, ce, le) {
              function ue(fe, he) {
                return ne.coerce(oe, se, ae, fe, he);
              }
              function de(fe) {
                var he = fe.map(function(me) {
                  var pe = ue(me);
                  return pe && ne.isArrayOrTypedArray(pe) ? pe : null;
                });
                return he.every(function(me) {
                  return me && me.length === he[0].length;
                }) && he;
              }
              de(["x", "y", "z"]) ? (de(["i", "j", "k"]), (!se.i || se.j && se.k) && (!se.j || se.k && se.i) && (!se.k || se.i && se.j) ? (te.getComponentMethod("calendars", "handleTraceDefaults")(oe, se, ["x", "y", "z"], le), ["lighting.ambient", "lighting.diffuse", "lighting.specular", "lighting.roughness", "lighting.fresnel", "lighting.vertexnormalsepsilon", "lighting.facenormalsepsilon", "lightposition.x", "lightposition.y", "lightposition.z", "flatshading", "alphahull", "delaunayaxis", "opacity"].forEach(function(fe) {
                ue(fe);
              }), ue("contour.show") && (ue("contour.color"), ue("contour.width")), "intensity" in oe ? (ue("intensity"), ue("intensitymode"), ee(oe, se, le, ue, { prefix: "", cLetter: "c" })) : (se.showscale = !1, "facecolor" in oe ? ue("facecolor") : "vertexcolor" in oe ? ue("vertexcolor") : ue("color", ce)), ue("text"), ue("hovertext"), ue("hovertemplate"), ue("xhoverformat"), ue("yhoverformat"), ue("zhoverformat"), se._length = null) : se.visible = !1) : se.visible = !1;
            };
          }, 58859: function(G, J, Z) {
            G.exports = { attributes: Z(42450), supplyDefaults: Z(13573), calc: Z(44878), colorbar: { min: "cmin", max: "cmax" }, plot: Z(82836), moduleType: "trace", name: "mesh3d", basePlotModule: Z(2487), categories: ["gl3d", "showLegend"], meta: {} };
          }, 86706: function(G, J, Z) {
            var te = Z(34809).extendFlat, ne = Z(36640), ee = Z(80712).axisHoverFormat, ae = Z(94850).T, oe = Z(70192), se = Z(20909), ce = se.INCREASING.COLOR, le = se.DECREASING.COLOR, ue = ne.line;
            function de(fe) {
              return { line: { color: te({}, ue.color, { dflt: fe }), width: ue.width, dash: ae, editType: "style" }, editType: "style" };
            }
            G.exports = { xperiod: ne.xperiod, xperiod0: ne.xperiod0, xperiodalignment: ne.xperiodalignment, xhoverformat: ee("x"), yhoverformat: ee("y"), x: { valType: "data_array", editType: "calc+clearAxisTypes" }, open: { valType: "data_array", editType: "calc" }, high: { valType: "data_array", editType: "calc" }, low: { valType: "data_array", editType: "calc" }, close: { valType: "data_array", editType: "calc" }, line: { width: te({}, ue.width, {}), dash: te({}, ae, {}), editType: "style" }, increasing: de(ce), decreasing: de(le), text: { valType: "string", dflt: "", arrayOk: !0, editType: "calc" }, hovertext: { valType: "string", dflt: "", arrayOk: !0, editType: "calc" }, tickwidth: { valType: "number", min: 0, max: 0.5, dflt: 0.3, editType: "calc" }, hoverlabel: te({}, oe.hoverlabel, { split: { valType: "boolean", dflt: !1, editType: "style" } }), zorder: ne.zorder };
          }, 95694: function(G, J, Z) {
            var te = Z(34809), ne = te._, ee = Z(29714), ae = Z(40528), oe = Z(63821).BADNUM;
            function se(le, ue, de, fe) {
              return { o: le, h: ue, l: de, c: fe };
            }
            function ce(le, ue, de, fe, he, me) {
              for (var pe = he.makeCalcdata(ue, "open"), ye = he.makeCalcdata(ue, "high"), ge = he.makeCalcdata(ue, "low"), ve = he.makeCalcdata(ue, "close"), be = te.isArrayOrTypedArray(ue.text), Ce = te.isArrayOrTypedArray(ue.hovertext), Se = !0, we = null, Ee = !!ue.xperiodalignment, xe = [], Le = 0; Le < fe.length; Le++) {
                var Qe = fe[Le], ke = pe[Le], Me = ye[Le], Re = ge[Le], De = ve[Le];
                if (Qe !== oe && ke !== oe && Me !== oe && Re !== oe && De !== oe) {
                  De === ke ? we !== null && De !== we && (Se = De > we) : Se = De > ke, we = De;
                  var Pe = me(ke, Me, Re, De);
                  Pe.pos = Qe, Pe.yc = (ke + De) / 2, Pe.i = Le, Pe.dir = Se ? "increasing" : "decreasing", Pe.x = Pe.pos, Pe.y = [Re, Me], Ee && (Pe.orig_p = de[Le]), be && (Pe.tx = ue.text[Le]), Ce && (Pe.htx = ue.hovertext[Le]), xe.push(Pe);
                } else xe.push({ pos: Qe, empty: !0 });
              }
              return ue._extremes[he._id] = ee.findExtremes(he, te.concat(ge, ye), { padded: !0 }), xe.length && (xe[0].t = { labels: { open: ne(le, "open:") + " ", high: ne(le, "high:") + " ", low: ne(le, "low:") + " ", close: ne(le, "close:") + " " } }), xe;
            }
            G.exports = { calc: function(le, ue) {
              var de = ee.getFromId(le, ue.xaxis), fe = ee.getFromId(le, ue.yaxis), he = function(ve, be, Ce) {
                var Se = Ce._minDiff;
                if (!Se) {
                  var we, Ee = ve._fullData, xe = [];
                  for (Se = 1 / 0, we = 0; we < Ee.length; we++) {
                    var Le = Ee[we];
                    if (Le.type === "ohlc" && Le.visible === !0 && Le.xaxis === be._id) {
                      xe.push(Le);
                      var Qe = be.makeCalcdata(Le, "x");
                      Le._origX = Qe;
                      var ke = ae(Ce, be, "x", Qe).vals;
                      Le._xcalc = ke;
                      var Me = te.distinctVals(ke).minDiff;
                      Me && isFinite(Me) && (Se = Math.min(Se, Me));
                    }
                  }
                  for (Se === 1 / 0 && (Se = 1), we = 0; we < xe.length; we++) xe[we]._minDiff = Se;
                }
                return Se * Ce.tickwidth;
              }(le, de, ue), me = ue._minDiff;
              ue._minDiff = null;
              var pe = ue._origX;
              ue._origX = null;
              var ye = ue._xcalc;
              ue._xcalc = null;
              var ge = ce(le, ue, pe, ye, fe, se);
              return ue._extremes[de._id] = ee.findExtremes(de, ye, { vpad: me / 2 }), ge.length ? (te.extendFlat(ge[0].t, { wHover: me / 2, tickLen: he }), ge) : [{ t: { empty: !0 } }];
            }, calcCommon: ce };
          }, 22629: function(G, J, Z) {
            var te = Z(34809), ne = Z(28270), ee = Z(99669), ae = Z(86706);
            function oe(se, ce, le, ue) {
              le(ue + ".line.color"), le(ue + ".line.width", ce.line.width), le(ue + ".line.dash", ce.line.dash);
            }
            G.exports = function(se, ce, le, ue) {
              function de(fe, he) {
                return te.coerce(se, ce, ae, fe, he);
              }
              ne(se, ce, de, ue) ? (ee(se, ce, ue, de, { x: !0 }), de("xhoverformat"), de("yhoverformat"), de("line.width"), de("line.dash"), oe(0, ce, de, "increasing"), oe(0, ce, de, "decreasing"), de("text"), de("hovertext"), de("tickwidth"), ue._requestRangeslider[ce.xaxis] = !0, de("zorder")) : ce.visible = !1;
            };
          }, 93245: function(G, J, Z) {
            var te = Z(29714), ne = Z(34809), ee = Z(32141), ae = Z(78766), oe = Z(34809).fillText, se = Z(20909), ce = { increasing: se.INCREASING.SYMBOL, decreasing: se.DECREASING.SYMBOL };
            function le(fe, he, me, pe) {
              var ye, ge, ve = fe.cd, be = fe.xa, Ce = ve[0].trace, Se = ve[0].t, we = Ce.type, Ee = we === "ohlc" ? "l" : "min", xe = we === "ohlc" ? "h" : "max", Le = Se.bPos || 0, Qe = function(Ge) {
                return Ge.pos + Le - he;
              }, ke = Se.bdPos || Se.tickLen, Me = Se.wHover, Re = Math.min(1, ke / Math.abs(be.r2c(be.range[1]) - be.r2c(be.range[0])));
              function De(Ge) {
                var ze = Qe(Ge);
                return ee.inbox(ze - Me, ze + Me, ye);
              }
              function Pe(Ge) {
                var ze = Ge[Ee], We = Ge[xe];
                return ze === We || ee.inbox(ze - me, We - me, ye);
              }
              function $e(Ge) {
                return (De(Ge) + Pe(Ge)) / 2;
              }
              ye = fe.maxHoverDistance - Re, ge = fe.maxSpikeDistance - Re;
              var qe = ee.getDistanceFunction(pe, De, Pe, $e);
              if (ee.getClosest(ve, qe, fe), fe.index === !1) return null;
              var Xe = ve[fe.index];
              if (Xe.empty) return null;
              var Ve = Ce[Xe.dir], Ze = Ve.line.color;
              return ae.opacity(Ze) && Ve.line.width ? fe.color = Ze : fe.color = Ve.fillcolor, fe.x0 = be.c2p(Xe.pos + Le - ke, !0), fe.x1 = be.c2p(Xe.pos + Le + ke, !0), fe.xLabelVal = Xe.orig_p !== void 0 ? Xe.orig_p : Xe.pos, fe.spikeDistance = $e(Xe) * ge / ye, fe.xSpike = be.c2p(Xe.pos, !0), fe;
            }
            function ue(fe, he, me, pe) {
              var ye = fe.cd, ge = fe.ya, ve = ye[0].trace, be = ye[0].t, Ce = [], Se = le(fe, he, me, pe);
              if (!Se) return [];
              var we = ye[Se.index].hi || ve.hoverinfo, Ee = we.split("+");
              if (we !== "all" && Ee.indexOf("y") === -1) return [];
              for (var xe = ["high", "open", "close", "low"], Le = {}, Qe = 0; Qe < xe.length; Qe++) {
                var ke, Me = xe[Qe], Re = ve[Me][Se.index], De = ge.c2p(Re, !0);
                Re in Le ? (ke = Le[Re]).yLabel += "<br>" + be.labels[Me] + te.hoverLabelText(ge, Re, ve.yhoverformat) : ((ke = ne.extendFlat({}, Se)).y0 = ke.y1 = De, ke.yLabelVal = Re, ke.yLabel = be.labels[Me] + te.hoverLabelText(ge, Re, ve.yhoverformat), ke.name = "", Ce.push(ke), Le[Re] = ke);
              }
              return Ce;
            }
            function de(fe, he, me, pe) {
              var ye = fe.cd, ge = fe.ya, ve = ye[0].trace, be = ye[0].t, Ce = le(fe, he, me, pe);
              if (!Ce) return [];
              var Se = ye[Ce.index], we = Ce.index = Se.i, Ee = Se.dir;
              function xe(Pe) {
                return be.labels[Pe] + te.hoverLabelText(ge, ve[Pe][we], ve.yhoverformat);
              }
              var Le = Se.hi || ve.hoverinfo, Qe = Le.split("+"), ke = Le === "all", Me = ke || Qe.indexOf("y") !== -1, Re = ke || Qe.indexOf("text") !== -1, De = Me ? [xe("open"), xe("high"), xe("low"), xe("close") + "  " + ce[Ee]] : [];
              return Re && oe(Se, ve, De), Ce.extraText = De.join("<br>"), Ce.y0 = Ce.y1 = ge.c2p(Se.yc, !0), [Ce];
            }
            G.exports = { hoverPoints: function(fe, he, me, pe) {
              return fe.cd[0].trace.hoverlabel.split ? ue(fe, he, me, pe) : de(fe, he, me, pe);
            }, hoverSplit: ue, hoverOnPoints: de };
          }, 12683: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "ohlc", basePlotModule: Z(37703), categories: ["cartesian", "svg", "showLegend"], meta: {}, attributes: Z(86706), supplyDefaults: Z(22629), calc: Z(95694).calc, plot: Z(38956), style: Z(57406), hoverPoints: Z(93245).hoverPoints, selectPoints: Z(49343) };
          }, 28270: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809);
            G.exports = function(ee, ae, oe, se) {
              var ce = oe("x"), le = oe("open"), ue = oe("high"), de = oe("low"), fe = oe("close");
              if (oe("hoverlabel.split"), te.getComponentMethod("calendars", "handleTraceDefaults")(ee, ae, ["x"], se), le && ue && de && fe) {
                var he = Math.min(le.length, ue.length, de.length, fe.length);
                return ce && (he = Math.min(he, ne.minRowLength(ce))), ae._length = he, he;
              }
            };
          }, 38956: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809);
            G.exports = function(ee, ae, oe, se) {
              var ce = ae.yaxis, le = ae.xaxis, ue = !!le.rangebreaks;
              ne.makeTraceGroups(se, oe, "trace ohlc").each(function(de) {
                var fe = te.select(this), he = de[0], me = he.t;
                if (he.trace.visible !== !0 || me.empty) fe.remove();
                else {
                  var pe = me.tickLen, ye = fe.selectAll("path").data(ne.identity);
                  ye.enter().append("path"), ye.exit().remove(), ye.attr("d", function(ge) {
                    if (ge.empty) return "M0,0Z";
                    var ve = le.c2p(ge.pos - pe, !0), be = le.c2p(ge.pos + pe, !0), Ce = ue ? (ve + be) / 2 : le.c2p(ge.pos, !0);
                    return "M" + ve + "," + ce.c2p(ge.o, !0) + "H" + Ce + "M" + Ce + "," + ce.c2p(ge.h, !0) + "V" + ce.c2p(ge.l, !0) + "M" + be + "," + ce.c2p(ge.c, !0) + "H" + Ce;
                  });
                }
              });
            };
          }, 49343: function(G) {
            G.exports = function(J, Z) {
              var te, ne = J.cd, ee = J.xaxis, ae = J.yaxis, oe = [], se = ne[0].t.bPos || 0;
              if (Z === !1) for (te = 0; te < ne.length; te++) ne[te].selected = 0;
              else for (te = 0; te < ne.length; te++) {
                var ce = ne[te];
                Z.contains([ee.c2p(ce.pos + se), ae.c2p(ce.yc)], null, ce.i, J) ? (oe.push({ pointNumber: ce.i, x: ee.c2d(ce.pos), y: ae.c2d(ce.yc) }), ce.selected = 1) : ce.selected = 0;
              }
              return oe;
            };
          }, 57406: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(78766);
            G.exports = function(ae, oe, se) {
              var ce = se || te.select(ae).selectAll("g.ohlclayer").selectAll("g.trace");
              ce.style("opacity", function(le) {
                return le[0].trace.opacity;
              }), ce.each(function(le) {
                var ue = le[0].trace;
                te.select(this).selectAll("path").each(function(de) {
                  if (!de.empty) {
                    var fe = ue[de.dir].line;
                    te.select(this).style("fill", "none").call(ee.stroke, fe.color).call(ne.dashLine, fe.dash, fe.width).style("opacity", ue.selectedpoints && !de.selected ? 0.3 : 1);
                  }
                });
              });
            };
          }, 11660: function(G, J, Z) {
            var te = Z(93049).extendFlat, ne = Z(9829), ee = Z(80337), ae = Z(87163), oe = Z(3208).rb, se = Z(13792).u, ce = te({ editType: "calc" }, ae("line", { editTypeOverride: "calc" }), { shape: { valType: "enumerated", values: ["linear", "hspline"], dflt: "linear", editType: "plot" }, hovertemplate: oe({ editType: "plot", arrayOk: !1 }, { keys: ["count", "probability"] }) });
            G.exports = { domain: se({ name: "parcats", trace: !0, editType: "calc" }), hoverinfo: te({}, ne.hoverinfo, { flags: ["count", "probability"], editType: "plot", arrayOk: !1 }), hoveron: { valType: "enumerated", values: ["category", "color", "dimension"], dflt: "category", editType: "plot" }, hovertemplate: oe({ editType: "plot", arrayOk: !1 }, { keys: ["count", "probability", "category", "categorycount", "colorcount", "bandcolorcount"] }), arrangement: { valType: "enumerated", values: ["perpendicular", "freeform", "fixed"], dflt: "perpendicular", editType: "plot" }, bundlecolors: { valType: "boolean", dflt: !0, editType: "plot" }, sortpaths: { valType: "enumerated", values: ["forward", "backward"], dflt: "forward", editType: "plot" }, labelfont: ee({ editType: "calc" }), tickfont: ee({ autoShadowDflt: !0, editType: "calc" }), dimensions: { _isLinkedToArray: "dimension", label: { valType: "string", editType: "calc" }, categoryorder: { valType: "enumerated", values: ["trace", "category ascending", "category descending", "array"], dflt: "trace", editType: "calc" }, categoryarray: { valType: "data_array", editType: "calc" }, ticktext: { valType: "data_array", editType: "calc" }, values: { valType: "data_array", dflt: [], editType: "calc" }, displayindex: { valType: "integer", editType: "calc" }, editType: "calc", visible: { valType: "boolean", dflt: !0, editType: "calc" } }, line: ce, counts: { valType: "number", min: 0, dflt: 1, arrayOk: !0, editType: "calc" }, customdata: void 0, hoverlabel: void 0, ids: void 0, legend: void 0, legendgroup: void 0, legendrank: void 0, opacity: void 0, selectedpoints: void 0, showlegend: void 0 };
          }, 83260: function(G, J, Z) {
            var te = Z(4173).eV, ne = Z(37822), ee = "parcats";
            J.name = ee, J.plot = function(ae, oe, se, ce) {
              var le = te(ae.calcdata, ee);
              if (le.length) {
                var ue = le[0];
                ne(ae, ue, se, ce);
              }
            }, J.clean = function(ae, oe, se, ce) {
              var le = ce._has && ce._has("parcats"), ue = oe._has && oe._has("parcats");
              le && !ue && ce._paperdiv.selectAll(".parcats").remove();
            };
          }, 95564: function(G, J, Z) {
            var te = Z(71293).wrap, ne = Z(65477).hasColorscale, ee = Z(28379), ae = Z(48965), oe = Z(62203), se = Z(34809), ce = Z(10721);
            function le(fe, he, me) {
              fe.valueInds.push(he), fe.count += me;
            }
            function ue(fe, he, me) {
              return { categoryInds: fe, color: he, rawColor: me, valueInds: [], count: 0 };
            }
            function de(fe, he, me) {
              fe.valueInds.push(he), fe.count += me;
            }
            G.exports = function(fe, he) {
              var me = se.filterVisible(he.dimensions);
              if (me.length === 0) return [];
              var pe, ye, ge, ve = me.map(function(We) {
                var at;
                if (We.categoryorder === "trace") at = null;
                else if (We.categoryorder === "array") at = We.categoryarray;
                else {
                  at = ae(We.values);
                  for (var ct = !0, et = 0; et < at.length; et++) if (!ce(at[et])) {
                    ct = !1;
                    break;
                  }
                  at.sort(ct ? se.sorterAsc : void 0), We.categoryorder === "category descending" && (at = at.reverse());
                }
                return function(dt, Je) {
                  Je = Je == null ? [] : Je.map(function(Rt) {
                    return Rt;
                  });
                  var it = {}, st = {}, gt = [];
                  Je.forEach(function(Rt, vt) {
                    it[Rt] = 0, st[Rt] = vt;
                  });
                  for (var bt = 0; bt < dt.length; bt++) {
                    var yt, St = dt[bt];
                    it[St] === void 0 ? (it[St] = 1, yt = Je.push(St) - 1, st[St] = yt) : (it[St]++, yt = st[St]), gt.push(yt);
                  }
                  var Lt = Je.map(function(Rt) {
                    return it[Rt];
                  });
                  return { uniqueValues: Je, uniqueCounts: Lt, inds: gt };
                }(We.values, at);
              });
              pe = se.isArrayOrTypedArray(he.counts) ? he.counts : [he.counts], function(We) {
                var at, ct = We.map(function(et) {
                  return et.displayindex;
                });
                if (function(et) {
                  for (var dt = new Array(et.length), Je = 0; Je < et.length; Je++) {
                    if (et[Je] < 0 || et[Je] >= et.length || dt[et[Je]] !== void 0) return !1;
                    dt[et[Je]] = !0;
                  }
                  return !0;
                }(ct)) for (at = 0; at < We.length; at++) We[at]._displayindex = We[at].displayindex;
                else for (at = 0; at < We.length; at++) We[at]._displayindex = at;
              }(me), me.forEach(function(We, at) {
                (function(ct, et) {
                  ct._categoryarray = et.uniqueValues, ct.ticktext === null || ct.ticktext === void 0 ? ct._ticktext = [] : ct._ticktext = ct.ticktext.slice();
                  for (var dt = ct._ticktext.length; dt < et.uniqueValues.length; dt++) ct._ticktext.push(et.uniqueValues[dt]);
                })(We, ve[at]);
              });
              var be, Ce = he.line;
              Ce ? (ne(he, "line") && ee(fe, he, { vals: he.line.color, containerStr: "line", cLetter: "c" }), be = oe.tryColorscale(Ce)) : be = se.identity;
              var Se, we, Ee, xe, Le, Qe = me[0].values.length, ke = {}, Me = ve.map(function(We) {
                return We.inds;
              });
              for (ge = 0, Se = 0; Se < Qe; Se++) {
                var Re = [];
                for (we = 0; we < Me.length; we++) Re.push(Me[we][Se]);
                ye = pe[Se % pe.length], ge += ye;
                var De = (Ee = Se, xe = void 0, Le = void 0, se.isArrayOrTypedArray(Ce.color) ? Le = xe = Ce.color[Ee % Ce.color.length] : xe = Ce.color, { color: be(xe), rawColor: Le }), Pe = Re + "-" + De.rawColor;
                ke[Pe] === void 0 && (ke[Pe] = ue(Re, De.color, De.rawColor)), de(ke[Pe], Se, ye);
              }
              var $e, qe = me.map(function(We, at) {
                return /* @__PURE__ */ function(ct, et, dt, Je, it) {
                  return { dimensionInd: ct, containerInd: et, displayInd: dt, dimensionLabel: Je, count: it, categories: [], dragX: null };
                }(at, We._index, We._displayindex, We.label, ge);
              });
              for (Se = 0; Se < Qe; Se++) for (ye = pe[Se % pe.length], we = 0; we < qe.length; we++) {
                var Xe = qe[we].containerInd, Ve = ve[we].inds[Se], Ze = qe[we].categories;
                if (Ze[Ve] === void 0) {
                  var Ge = he.dimensions[Xe]._categoryarray[Ve], ze = he.dimensions[Xe]._ticktext[Ve];
                  Ze[Ve] = { dimensionInd: we, categoryInd: $e = Ve, categoryValue: Ge, displayInd: $e, categoryLabel: ze, valueInds: [], count: 0, dragY: null };
                }
                le(Ze[Ve], Se, ye);
              }
              return te(function(We, at, ct) {
                var et = We.map(function(dt) {
                  return dt.categories.length;
                }).reduce(function(dt, Je) {
                  return Math.max(dt, Je);
                });
                return { dimensions: We, paths: at, trace: void 0, maxCats: et, count: ct };
              }(qe, ke, ge));
            };
          }, 62651: function(G, J, Z) {
            var te = Z(34809), ne = Z(65477).hasColorscale, ee = Z(39356), ae = Z(13792).N, oe = Z(59008), se = Z(11660), ce = Z(63197), le = Z(87800).isTypedArraySpec;
            function ue(de, fe) {
              function he(Ce, Se) {
                return te.coerce(de, fe, se.dimensions, Ce, Se);
              }
              var me = he("values"), pe = he("visible");
              if (me && me.length || (pe = fe.visible = !1), pe) {
                he("label"), he("displayindex", fe._index);
                var ye, ge = de.categoryarray, ve = te.isArrayOrTypedArray(ge) && ge.length > 0 || le(ge);
                ve && (ye = "array");
                var be = he("categoryorder", ye);
                be === "array" ? (he("categoryarray"), he("ticktext")) : (delete de.categoryarray, delete de.ticktext), ve || be !== "array" || (fe.categoryorder = "trace");
              }
            }
            G.exports = function(de, fe, he, me) {
              function pe(be, Ce) {
                return te.coerce(de, fe, se, be, Ce);
              }
              var ye = oe(de, fe, { name: "dimensions", handleItemDefaults: ue }), ge = function(be, Ce, Se, we, Ee) {
                Ee("line.shape"), Ee("line.hovertemplate");
                var xe = Ee("line.color", we.colorway[0]);
                if (ne(be, "line") && te.isArrayOrTypedArray(xe)) {
                  if (xe.length) return Ee("line.colorscale"), ee(be, Ce, we, Ee, { prefix: "line.", cLetter: "c" }), xe.length;
                  Ce.line.color = Se;
                }
                return 1 / 0;
              }(de, fe, he, me, pe);
              ae(fe, me, pe), Array.isArray(ye) && ye.length || (fe.visible = !1), ce(fe, ye, "values", ge), pe("hoveron"), pe("hovertemplate"), pe("arrangement"), pe("bundlecolors"), pe("sortpaths"), pe("counts");
              var ve = me.font;
              te.coerceFont(pe, "labelfont", ve, { overrideDflt: { size: Math.round(ve.size) } }), te.coerceFont(pe, "tickfont", ve, { autoShadowDflt: !0, overrideDflt: { size: Math.round(ve.size / 1.2) } });
            };
          }, 6305: function(G, J, Z) {
            G.exports = { attributes: Z(11660), supplyDefaults: Z(62651), calc: Z(95564), plot: Z(37822), colorbar: { container: "line", min: "cmin", max: "cmax" }, moduleType: "trace", name: "parcats", basePlotModule: Z(83260), categories: ["noOpacity"], meta: {} };
          }, 27219: function(G, J, Z) {
            var te = Z(45568), ne = Z(88640).Dj, ee = Z(31420), ae = Z(32141), oe = Z(34809), se = oe.strTranslate, ce = Z(62203), le = Z(65657), ue = Z(30635);
            function de(ct, et, dt, Je) {
              var it = et._context.staticPlot, st = ct.map(Ze.bind(0, et, dt)), gt = Je.selectAll("g.parcatslayer").data([null]);
              gt.enter().append("g").attr("class", "parcatslayer").style("pointer-events", it ? "none" : "all");
              var bt = gt.selectAll("g.trace.parcats").data(st, fe), yt = bt.enter().append("g").attr("class", "trace parcats");
              bt.attr("transform", function(Nt) {
                return se(Nt.x, Nt.y);
              }), yt.append("g").attr("class", "paths");
              var St = bt.select("g.paths").selectAll("path.path").data(function(Nt) {
                return Nt.paths;
              }, fe);
              St.attr("fill", function(Nt) {
                return Nt.model.color;
              });
              var Lt = St.enter().append("path").attr("class", "path").attr("stroke-opacity", 0).attr("fill", function(Nt) {
                return Nt.model.color;
              }).attr("fill-opacity", 0);
              Ce(Lt), St.attr("d", function(Nt) {
                return Nt.svgD;
              }), Lt.empty() || St.sort(me), St.exit().remove(), St.on("mouseover", pe).on("mouseout", ye).on("click", be), yt.append("g").attr("class", "dimensions");
              var Rt = bt.select("g.dimensions").selectAll("g.dimension").data(function(Nt) {
                return Nt.dimensions;
              }, fe);
              Rt.enter().append("g").attr("class", "dimension"), Rt.attr("transform", function(Nt) {
                return se(Nt.x, 0);
              }), Rt.exit().remove();
              var vt = Rt.selectAll("g.category").data(function(Nt) {
                return Nt.categories;
              }, fe), $t = vt.enter().append("g").attr("class", "category");
              vt.attr("transform", function(Nt) {
                return se(0, Nt.y);
              }), $t.append("rect").attr("class", "catrect").attr("pointer-events", "none"), vt.select("rect.catrect").attr("fill", "none").attr("width", function(Nt) {
                return Nt.width;
              }).attr("height", function(Nt) {
                return Nt.height;
              }), we($t);
              var Pt = vt.selectAll("rect.bandrect").data(function(Nt) {
                return Nt.bands;
              }, fe);
              Pt.each(function() {
                oe.raiseToTop(this);
              }), Pt.attr("fill", function(Nt) {
                return Nt.color;
              });
              var Xt = Pt.enter().append("rect").attr("class", "bandrect").attr("stroke-opacity", 0).attr("fill", function(Nt) {
                return Nt.color;
              }).attr("fill-opacity", 0);
              Pt.attr("fill", function(Nt) {
                return Nt.color;
              }).attr("width", function(Nt) {
                return Nt.width;
              }).attr("height", function(Nt) {
                return Nt.height;
              }).attr("y", function(Nt) {
                return Nt.y;
              }).attr("cursor", function(Nt) {
                return Nt.parcatsViewModel.arrangement === "fixed" ? "default" : Nt.parcatsViewModel.arrangement === "perpendicular" ? "ns-resize" : "move";
              }), Ee(Xt), Pt.exit().remove(), $t.append("text").attr("class", "catlabel").attr("pointer-events", "none"), vt.select("text.catlabel").attr("text-anchor", function(Nt) {
                return he(Nt) ? "start" : "end";
              }).attr("alignment-baseline", "middle").style("fill", "rgb(0, 0, 0)").attr("x", function(Nt) {
                return he(Nt) ? Nt.width + 5 : -5;
              }).attr("y", function(Nt) {
                return Nt.height / 2;
              }).text(function(Nt) {
                return Nt.model.categoryLabel;
              }).each(function(Nt) {
                ce.font(te.select(this), Nt.parcatsViewModel.categorylabelfont), ue.convertToTspans(te.select(this), et);
              }), $t.append("text").attr("class", "dimlabel"), vt.select("text.dimlabel").attr("text-anchor", "middle").attr("alignment-baseline", "baseline").attr("cursor", function(Nt) {
                return Nt.parcatsViewModel.arrangement === "fixed" ? "default" : "ew-resize";
              }).attr("x", function(Nt) {
                return Nt.width / 2;
              }).attr("y", -5).text(function(Nt, Gt) {
                return Gt === 0 ? Nt.parcatsViewModel.model.dimensions[Nt.model.dimensionInd].dimensionLabel : null;
              }).each(function(Nt) {
                ce.font(te.select(this), Nt.parcatsViewModel.labelfont);
              }), vt.selectAll("rect.bandrect").on("mouseover", Me).on("mouseout", Re), vt.exit().remove(), Rt.call(te.behavior.drag().origin(function(Nt) {
                return { x: Nt.x, y: 0 };
              }).on("dragstart", De).on("drag", Pe).on("dragend", $e)), bt.each(function(Nt) {
                Nt.traceSelection = te.select(this), Nt.pathSelection = te.select(this).selectAll("g.paths").selectAll("path.path"), Nt.dimensionSelection = te.select(this).selectAll("g.dimensions").selectAll("g.dimension");
              }), bt.exit().remove();
            }
            function fe(ct) {
              return ct.key;
            }
            function he(ct) {
              var et = ct.parcatsViewModel.dimensions.length, dt = ct.parcatsViewModel.dimensions[et - 1].model.dimensionInd;
              return ct.model.dimensionInd === dt;
            }
            function me(ct, et) {
              return ct.model.rawColor > et.model.rawColor ? 1 : ct.model.rawColor < et.model.rawColor ? -1 : 0;
            }
            function pe(ct) {
              if (!ct.parcatsViewModel.dragDimension && ct.parcatsViewModel.hoverinfoItems.indexOf("skip") === -1) {
                oe.raiseToTop(this), Se(te.select(this));
                var et = ge(ct), dt = ve(ct);
                if (ct.parcatsViewModel.graphDiv.emit("plotly_hover", { points: et, event: te.event, constraints: dt }), ct.parcatsViewModel.hoverinfoItems.indexOf("none") === -1) {
                  var Je, it, st, gt = te.mouse(this)[0], bt = ct.parcatsViewModel.graphDiv, yt = ct.parcatsViewModel.trace, St = bt._fullLayout, Lt = St._paperdiv.node().getBoundingClientRect(), Rt = ct.parcatsViewModel.graphDiv.getBoundingClientRect();
                  for (st = 0; st < ct.leftXs.length - 1; st++) if (ct.leftXs[st] + ct.dimWidths[st] - 2 <= gt && gt <= ct.leftXs[st + 1] + 2) {
                    var vt = ct.parcatsViewModel.dimensions[st], $t = ct.parcatsViewModel.dimensions[st + 1];
                    Je = (vt.x + vt.width + $t.x) / 2, it = (ct.topYs[st] + ct.topYs[st + 1] + ct.height) / 2;
                    break;
                  }
                  var Pt = ct.parcatsViewModel.x + Je, Xt = ct.parcatsViewModel.y + it, Nt = le.mostReadable(ct.model.color, ["black", "white"]), Gt = ct.model.count, Ft = Gt / ct.parcatsViewModel.model.count, Tt = { countLabel: Gt, probabilityLabel: Ft.toFixed(3) }, It = [];
                  ct.parcatsViewModel.hoverinfoItems.indexOf("count") !== -1 && It.push(["Count:", Tt.countLabel].join(" ")), ct.parcatsViewModel.hoverinfoItems.indexOf("probability") !== -1 && It.push(["P:", Tt.probabilityLabel].join(" "));
                  var Ct = It.join("<br>"), Kt = te.mouse(bt)[0];
                  ae.loneHover({ trace: yt, x: Pt - Lt.left + Rt.left, y: Xt - Lt.top + Rt.top, text: Ct, color: ct.model.color, borderColor: "black", fontFamily: 'Monaco, "Courier New", monospace', fontSize: 10, fontColor: Nt, idealAlign: Kt < Pt ? "right" : "left", hovertemplate: (yt.line || {}).hovertemplate, hovertemplateLabels: Tt, eventData: [{ data: yt._input, fullData: yt, count: Gt, probability: Ft }] }, { container: St._hoverlayer.node(), outerContainer: St._paper.node(), gd: bt });
                }
              }
            }
            function ye(ct) {
              if (!ct.parcatsViewModel.dragDimension && (Ce(te.select(this)), ae.loneUnhover(ct.parcatsViewModel.graphDiv._fullLayout._hoverlayer.node()), ct.parcatsViewModel.pathSelection.sort(me), ct.parcatsViewModel.hoverinfoItems.indexOf("skip") === -1)) {
                var et = ge(ct), dt = ve(ct);
                ct.parcatsViewModel.graphDiv.emit("plotly_unhover", { points: et, event: te.event, constraints: dt });
              }
            }
            function ge(ct) {
              for (var et = [], dt = qe(ct.parcatsViewModel), Je = 0; Je < ct.model.valueInds.length; Je++) {
                var it = ct.model.valueInds[Je];
                et.push({ curveNumber: dt, pointNumber: it });
              }
              return et;
            }
            function ve(ct) {
              for (var et = {}, dt = ct.parcatsViewModel.model.dimensions, Je = 0; Je < dt.length; Je++) {
                var it = dt[Je], st = it.categories[ct.model.categoryInds[Je]];
                et[it.containerInd] = st.categoryValue;
              }
              return ct.model.rawColor !== void 0 && (et.color = ct.model.rawColor), et;
            }
            function be(ct) {
              if (ct.parcatsViewModel.hoverinfoItems.indexOf("skip") === -1) {
                var et = ge(ct), dt = ve(ct);
                ct.parcatsViewModel.graphDiv.emit("plotly_click", { points: et, event: te.event, constraints: dt });
              }
            }
            function Ce(ct) {
              ct.attr("fill", function(et) {
                return et.model.color;
              }).attr("fill-opacity", 0.6).attr("stroke", "lightgray").attr("stroke-width", 0.2).attr("stroke-opacity", 1);
            }
            function Se(ct) {
              ct.attr("fill-opacity", 0.8).attr("stroke", function(et) {
                return le.mostReadable(et.model.color, ["black", "white"]);
              }).attr("stroke-width", 0.3);
            }
            function we(ct) {
              ct.select("rect.catrect").attr("stroke", "black").attr("stroke-width", 1).attr("stroke-opacity", 1);
            }
            function Ee(ct) {
              ct.attr("stroke", "black").attr("stroke-width", 0.2).attr("stroke-opacity", 1).attr("fill-opacity", 1);
            }
            function xe(ct) {
              var et = ct.parcatsViewModel.pathSelection, dt = ct.categoryViewModel.model.dimensionInd, Je = ct.categoryViewModel.model.categoryInd;
              return et.filter(function(it) {
                return it.model.categoryInds[dt] === Je && it.model.color === ct.color;
              });
            }
            function Le(ct, et, dt) {
              var Je = te.select(ct).datum(), it = Je.categoryViewModel.model, st = Je.parcatsViewModel.graphDiv, gt = te.select(ct.parentNode).selectAll("rect.bandrect"), bt = [];
              gt.each(function(St) {
                xe(St).each(function(Lt) {
                  Array.prototype.push.apply(bt, ge(Lt));
                });
              });
              var yt = {};
              yt[it.dimensionInd] = it.categoryValue, st.emit(et, { points: bt, event: dt, constraints: yt });
            }
            function Qe(ct, et, dt) {
              var Je = te.select(ct).datum(), it = Je.categoryViewModel.model, st = Je.parcatsViewModel.graphDiv, gt = xe(Je), bt = [];
              gt.each(function(St) {
                Array.prototype.push.apply(bt, ge(St));
              });
              var yt = {};
              yt[it.dimensionInd] = it.categoryValue, Je.rawColor !== void 0 && (yt.color = Je.rawColor), st.emit(et, { points: bt, event: dt, constraints: yt });
            }
            function ke(ct, et, dt) {
              ct._fullLayout._calcInverseTransform(ct);
              var Je, it, st = ct._fullLayout._invScaleX, gt = ct._fullLayout._invScaleY, bt = te.select(dt.parentNode).select("rect.catrect"), yt = bt.node().getBoundingClientRect(), St = bt.datum(), Lt = St.parcatsViewModel, Rt = Lt.model.dimensions[St.model.dimensionInd], vt = Lt.trace, $t = yt.top + yt.height / 2;
              Lt.dimensions.length > 1 && Rt.displayInd === Lt.dimensions.length - 1 ? (Je = yt.left, it = "left") : (Je = yt.left + yt.width, it = "right");
              var Pt = St.model.count, Xt = St.model.categoryLabel, Nt = Pt / St.parcatsViewModel.model.count, Gt = { countLabel: Pt, categoryLabel: Xt, probabilityLabel: Nt.toFixed(3) }, Ft = [];
              St.parcatsViewModel.hoverinfoItems.indexOf("count") !== -1 && Ft.push(["Count:", Gt.countLabel].join(" ")), St.parcatsViewModel.hoverinfoItems.indexOf("probability") !== -1 && Ft.push(["P(" + Gt.categoryLabel + "):", Gt.probabilityLabel].join(" "));
              var Tt = Ft.join("<br>");
              return { trace: vt, x: st * (Je - et.left), y: gt * ($t - et.top), text: Tt, color: "lightgray", borderColor: "black", fontFamily: 'Monaco, "Courier New", monospace', fontSize: 12, fontColor: "black", idealAlign: it, hovertemplate: vt.hovertemplate, hovertemplateLabels: Gt, eventData: [{ data: vt._input, fullData: vt, count: Pt, category: Xt, probability: Nt }] };
            }
            function Me(ct) {
              if (!ct.parcatsViewModel.dragDimension && ct.parcatsViewModel.hoverinfoItems.indexOf("skip") === -1) {
                if (te.mouse(this)[1] < -1) return;
                var et, dt = ct.parcatsViewModel.graphDiv, Je = dt._fullLayout, it = Je._paperdiv.node().getBoundingClientRect(), st = ct.parcatsViewModel.hoveron, gt = this;
                st === "color" ? (function(bt) {
                  var yt = te.select(bt).datum(), St = xe(yt);
                  Se(St), St.each(function() {
                    oe.raiseToTop(this);
                  }), te.select(bt.parentNode).selectAll("rect.bandrect").filter(function(Lt) {
                    return Lt.color === yt.color;
                  }).each(function() {
                    oe.raiseToTop(this), te.select(this).attr("stroke", "black").attr("stroke-width", 1.5);
                  });
                }(gt), Qe(gt, "plotly_hover", te.event)) : (function(bt) {
                  te.select(bt.parentNode).selectAll("rect.bandrect").each(function(yt) {
                    var St = xe(yt);
                    Se(St), St.each(function() {
                      oe.raiseToTop(this);
                    });
                  }), te.select(bt.parentNode).select("rect.catrect").attr("stroke", "black").attr("stroke-width", 2.5);
                }(gt), Le(gt, "plotly_hover", te.event)), ct.parcatsViewModel.hoverinfoItems.indexOf("none") === -1 && (st === "category" ? et = ke(dt, it, gt) : st === "color" ? et = function(bt, yt, St) {
                  bt._fullLayout._calcInverseTransform(bt);
                  var Lt, Rt, vt = bt._fullLayout._invScaleX, $t = bt._fullLayout._invScaleY, Pt = St.getBoundingClientRect(), Xt = te.select(St).datum(), Nt = Xt.categoryViewModel, Gt = Nt.parcatsViewModel, Ft = Gt.model.dimensions[Nt.model.dimensionInd], Tt = Gt.trace, It = Pt.y + Pt.height / 2;
                  Gt.dimensions.length > 1 && Ft.displayInd === Gt.dimensions.length - 1 ? (Lt = Pt.left, Rt = "left") : (Lt = Pt.left + Pt.width, Rt = "right");
                  var Ct = Nt.model.categoryLabel, Kt = Xt.parcatsViewModel.model.count, Vt = 0;
                  Xt.categoryViewModel.bands.forEach(function(Ht) {
                    Ht.color === Xt.color && (Vt += Ht.count);
                  });
                  var Yt = Nt.model.count, sn = 0;
                  Gt.pathSelection.each(function(Ht) {
                    Ht.model.color === Xt.color && (sn += Ht.model.count);
                  });
                  var hn = Vt / Kt, Qt = Vt / sn, Ut = Vt / Yt, Wt = { countLabel: Vt, categoryLabel: Ct, probabilityLabel: hn.toFixed(3) }, rn = [];
                  Nt.parcatsViewModel.hoverinfoItems.indexOf("count") !== -1 && rn.push(["Count:", Wt.countLabel].join(" ")), Nt.parcatsViewModel.hoverinfoItems.indexOf("probability") !== -1 && (rn.push("P(color ∩ " + Ct + "): " + Wt.probabilityLabel), rn.push("P(" + Ct + " | color): " + Qt.toFixed(3)), rn.push("P(color | " + Ct + "): " + Ut.toFixed(3)));
                  var vn = rn.join("<br>"), dn = le.mostReadable(Xt.color, ["black", "white"]);
                  return { trace: Tt, x: vt * (Lt - yt.left), y: $t * (It - yt.top), text: vn, color: Xt.color, borderColor: "black", fontFamily: 'Monaco, "Courier New", monospace', fontColor: dn, fontSize: 10, idealAlign: Rt, hovertemplate: Tt.hovertemplate, hovertemplateLabels: Wt, eventData: [{ data: Tt._input, fullData: Tt, category: Ct, count: Kt, probability: hn, categorycount: Yt, colorcount: sn, bandcolorcount: Vt }] };
                }(dt, it, gt) : st === "dimension" && (et = function(bt, yt, St) {
                  var Lt = [];
                  return te.select(St.parentNode.parentNode).selectAll("g.category").select("rect.catrect").each(function() {
                    Lt.push(ke(bt, yt, this));
                  }), Lt;
                }(dt, it, gt)), et && ae.loneHover(et, { container: Je._hoverlayer.node(), outerContainer: Je._paper.node(), gd: dt }));
              }
            }
            function Re(ct) {
              var et = ct.parcatsViewModel;
              et.dragDimension || (Ce(et.pathSelection), we(et.dimensionSelection.selectAll("g.category")), Ee(et.dimensionSelection.selectAll("g.category").selectAll("rect.bandrect")), ae.loneUnhover(et.graphDiv._fullLayout._hoverlayer.node()), et.pathSelection.sort(me), et.hoverinfoItems.indexOf("skip") !== -1) || (ct.parcatsViewModel.hoveron === "color" ? Qe(this, "plotly_unhover", te.event) : Le(this, "plotly_unhover", te.event));
            }
            function De(ct) {
              ct.parcatsViewModel.arrangement !== "fixed" && (ct.dragDimensionDisplayInd = ct.model.displayInd, ct.initialDragDimensionDisplayInds = ct.parcatsViewModel.model.dimensions.map(function(et) {
                return et.displayInd;
              }), ct.dragHasMoved = !1, ct.dragCategoryDisplayInd = null, te.select(this).selectAll("g.category").select("rect.catrect").each(function(et) {
                var dt = te.mouse(this)[0], Je = te.mouse(this)[1];
                -2 <= dt && dt <= et.width + 2 && -2 <= Je && Je <= et.height + 2 && (ct.dragCategoryDisplayInd = et.model.displayInd, ct.initialDragCategoryDisplayInds = ct.model.categories.map(function(it) {
                  return it.displayInd;
                }), et.model.dragY = et.y, oe.raiseToTop(this.parentNode), te.select(this.parentNode).selectAll("rect.bandrect").each(function(it) {
                  it.y < Je && Je <= it.y + it.height && (ct.potentialClickBand = this);
                }));
              }), ct.parcatsViewModel.dragDimension = ct, ae.loneUnhover(ct.parcatsViewModel.graphDiv._fullLayout._hoverlayer.node()));
            }
            function Pe(ct) {
              if (ct.parcatsViewModel.arrangement !== "fixed" && (ct.dragHasMoved = !0, ct.dragDimensionDisplayInd !== null)) {
                var et = ct.dragDimensionDisplayInd, dt = et - 1, Je = et + 1, it = ct.parcatsViewModel.dimensions[et];
                if (ct.dragCategoryDisplayInd !== null) {
                  var st = it.categories[ct.dragCategoryDisplayInd];
                  st.model.dragY += te.event.dy;
                  var gt = st.model.dragY, bt = st.model.displayInd, yt = it.categories, St = yt[bt - 1], Lt = yt[bt + 1];
                  St !== void 0 && gt < St.y + St.height / 2 && (st.model.displayInd = St.model.displayInd, St.model.displayInd = bt), Lt !== void 0 && gt + st.height > Lt.y + Lt.height / 2 && (st.model.displayInd = Lt.model.displayInd, Lt.model.displayInd = bt), ct.dragCategoryDisplayInd = st.model.displayInd;
                }
                if (ct.dragCategoryDisplayInd === null || ct.parcatsViewModel.arrangement === "freeform") {
                  it.model.dragX = te.event.x;
                  var Rt = ct.parcatsViewModel.dimensions[dt], vt = ct.parcatsViewModel.dimensions[Je];
                  Rt !== void 0 && it.model.dragX < Rt.x + Rt.width && (it.model.displayInd = Rt.model.displayInd, Rt.model.displayInd = et), vt !== void 0 && it.model.dragX + it.width > vt.x && (it.model.displayInd = vt.model.displayInd, vt.model.displayInd = ct.dragDimensionDisplayInd), ct.dragDimensionDisplayInd = it.model.displayInd;
                }
                We(ct.parcatsViewModel), ze(ct.parcatsViewModel), Ve(ct.parcatsViewModel), Xe(ct.parcatsViewModel);
              }
            }
            function $e(ct) {
              if (ct.parcatsViewModel.arrangement !== "fixed" && ct.dragDimensionDisplayInd !== null) {
                te.select(this).selectAll("text").attr("font-weight", "normal");
                var et = {}, dt = qe(ct.parcatsViewModel), Je = ct.parcatsViewModel.model.dimensions.map(function(Lt) {
                  return Lt.displayInd;
                }), it = ct.initialDragDimensionDisplayInds.some(function(Lt, Rt) {
                  return Lt !== Je[Rt];
                });
                it && Je.forEach(function(Lt, Rt) {
                  var vt = ct.parcatsViewModel.model.dimensions[Rt].containerInd;
                  et["dimensions[" + vt + "].displayindex"] = Lt;
                });
                var st = !1;
                if (ct.dragCategoryDisplayInd !== null) {
                  var gt = ct.model.categories.map(function(Lt) {
                    return Lt.displayInd;
                  });
                  if (st = ct.initialDragCategoryDisplayInds.some(function(Lt, Rt) {
                    return Lt !== gt[Rt];
                  })) {
                    var bt = ct.model.categories.slice().sort(function(Lt, Rt) {
                      return Lt.displayInd - Rt.displayInd;
                    }), yt = bt.map(function(Lt) {
                      return Lt.categoryValue;
                    }), St = bt.map(function(Lt) {
                      return Lt.categoryLabel;
                    });
                    et["dimensions[" + ct.model.containerInd + "].categoryarray"] = [yt], et["dimensions[" + ct.model.containerInd + "].ticktext"] = [St], et["dimensions[" + ct.model.containerInd + "].categoryorder"] = "array";
                  }
                }
                ct.parcatsViewModel.hoverinfoItems.indexOf("skip") === -1 && !ct.dragHasMoved && ct.potentialClickBand && (ct.parcatsViewModel.hoveron === "color" ? Qe(ct.potentialClickBand, "plotly_click", te.event.sourceEvent) : Le(ct.potentialClickBand, "plotly_click", te.event.sourceEvent)), ct.model.dragX = null, ct.dragCategoryDisplayInd !== null && (ct.parcatsViewModel.dimensions[ct.dragDimensionDisplayInd].categories[ct.dragCategoryDisplayInd].model.dragY = null, ct.dragCategoryDisplayInd = null), ct.dragDimensionDisplayInd = null, ct.parcatsViewModel.dragDimension = null, ct.dragHasMoved = null, ct.potentialClickBand = null, We(ct.parcatsViewModel), ze(ct.parcatsViewModel), te.transition().duration(300).ease("cubic-in-out").each(function() {
                  Ve(ct.parcatsViewModel, !0), Xe(ct.parcatsViewModel, !0);
                }).each("end", function() {
                  (it || st) && ee.restyle(ct.parcatsViewModel.graphDiv, et, [dt]);
                });
              }
            }
            function qe(ct) {
              for (var et, dt = ct.graphDiv._fullData, Je = 0; Je < dt.length; Je++) if (ct.key === dt[Je].uid) {
                et = Je;
                break;
              }
              return et;
            }
            function Xe(ct, et) {
              var dt;
              et === void 0 && (et = !1), ct.pathSelection.data(function(Je) {
                return Je.paths;
              }, fe), (dt = ct.pathSelection, et ? dt.transition() : dt).attr("d", function(Je) {
                return Je.svgD;
              });
            }
            function Ve(ct, et) {
              function dt(gt) {
                return et ? gt.transition() : gt;
              }
              et === void 0 && (et = !1), ct.dimensionSelection.data(function(gt) {
                return gt.dimensions;
              }, fe);
              var Je = ct.dimensionSelection.selectAll("g.category").data(function(gt) {
                return gt.categories;
              }, fe);
              dt(ct.dimensionSelection).attr("transform", function(gt) {
                return se(gt.x, 0);
              }), dt(Je).attr("transform", function(gt) {
                return se(0, gt.y);
              }), Je.select(".dimlabel").text(function(gt, bt) {
                return bt === 0 ? gt.parcatsViewModel.model.dimensions[gt.model.dimensionInd].dimensionLabel : null;
              }), Je.select(".catlabel").attr("text-anchor", function(gt) {
                return he(gt) ? "start" : "end";
              }).attr("x", function(gt) {
                return he(gt) ? gt.width + 5 : -5;
              }).each(function(gt) {
                var bt, yt;
                he(gt) ? (bt = gt.width + 5, yt = "start") : (bt = -5, yt = "end"), te.select(this).selectAll("tspan").attr("x", bt).attr("text-anchor", yt);
              });
              var it = Je.selectAll("rect.bandrect").data(function(gt) {
                return gt.bands;
              }, fe), st = it.enter().append("rect").attr("class", "bandrect").attr("cursor", "move").attr("stroke-opacity", 0).attr("fill", function(gt) {
                return gt.color;
              }).attr("fill-opacity", 0);
              it.attr("fill", function(gt) {
                return gt.color;
              }).attr("width", function(gt) {
                return gt.width;
              }).attr("height", function(gt) {
                return gt.height;
              }).attr("y", function(gt) {
                return gt.y;
              }), Ee(st), it.each(function() {
                oe.raiseToTop(this);
              }), it.exit().remove();
            }
            function Ze(ct, et, dt) {
              var Je, it = dt[0], st = et.margin || { l: 80, r: 80, t: 100, b: 80 }, gt = it.trace, bt = gt.domain, yt = et.width, St = et.height, Lt = Math.floor(yt * (bt.x[1] - bt.x[0])), Rt = Math.floor(St * (bt.y[1] - bt.y[0])), vt = bt.x[0] * yt + st.l, $t = et.height - bt.y[1] * et.height + st.t, Pt = gt.line.shape;
              Je = gt.hoverinfo === "all" ? ["count", "probability"] : (gt.hoverinfo || "").split("+");
              var Xt = { trace: gt, key: gt.uid, model: it, x: vt, y: $t, width: Lt, height: Rt, hoveron: gt.hoveron, hoverinfoItems: Je, arrangement: gt.arrangement, bundlecolors: gt.bundlecolors, sortpaths: gt.sortpaths, labelfont: gt.labelfont, categorylabelfont: gt.tickfont, pathShape: Pt, dragDimension: null, margin: st, paths: [], dimensions: [], graphDiv: ct, traceSelection: null, pathSelection: null, dimensionSelection: null };
              return it.dimensions && (We(Xt), ze(Xt)), Xt;
            }
            function Ge(ct, et, dt, Je, it) {
              var st, gt, bt = [], yt = [];
              for (gt = 0; gt < dt.length - 1; gt++) st = ne(dt[gt] + ct[gt], ct[gt + 1]), bt.push(st(it)), yt.push(st(1 - it));
              var St = "M " + ct[0] + "," + et[0];
              for (St += "l" + dt[0] + ",0 ", gt = 1; gt < dt.length; gt++) St += "C" + bt[gt - 1] + "," + et[gt - 1] + " " + yt[gt - 1] + "," + et[gt] + " " + ct[gt] + "," + et[gt], St += "l" + dt[gt] + ",0 ";
              for (St += "l0," + Je + " ", St += "l -" + dt[dt.length - 1] + ",0 ", gt = dt.length - 2; gt >= 0; gt--) St += "C" + yt[gt] + "," + (et[gt + 1] + Je) + " " + bt[gt] + "," + (et[gt] + Je) + " " + (ct[gt] + dt[gt]) + "," + (et[gt] + Je), St += "l-" + dt[gt] + ",0 ";
              return St + "Z";
            }
            function ze(ct) {
              var et = ct.dimensions, dt = ct.model, Je = et.map(function(Wt) {
                return Wt.categories.map(function(rn) {
                  return rn.y;
                });
              }), it = ct.model.dimensions.map(function(Wt) {
                return Wt.categories.map(function(rn) {
                  return rn.displayInd;
                });
              }), st = ct.model.dimensions.map(function(Wt) {
                return Wt.displayInd;
              }), gt = ct.dimensions.map(function(Wt) {
                return Wt.model.dimensionInd;
              }), bt = et.map(function(Wt) {
                return Wt.x;
              }), yt = et.map(function(Wt) {
                return Wt.width;
              }), St = [];
              for (var Lt in dt.paths) dt.paths.hasOwnProperty(Lt) && St.push(dt.paths[Lt]);
              function Rt(Wt) {
                var rn = Wt.categoryInds.map(function(vn, dn) {
                  return it[dn][vn];
                });
                return gt.map(function(vn) {
                  return rn[vn];
                });
              }
              St.sort(function(Wt, rn) {
                var vn = Rt(Wt), dn = Rt(rn);
                return ct.sortpaths === "backward" && (vn.reverse(), dn.reverse()), vn.push(Wt.valueInds[0]), dn.push(rn.valueInds[0]), ct.bundlecolors && (vn.unshift(Wt.rawColor), dn.unshift(rn.rawColor)), vn < dn ? -1 : vn > dn ? 1 : 0;
              });
              for (var vt = new Array(St.length), $t = et[0].model.count, Pt = et[0].categories.map(function(Wt) {
                return Wt.height;
              }).reduce(function(Wt, rn) {
                return Wt + rn;
              }), Xt = 0; Xt < St.length; Xt++) {
                var Nt, Gt = St[Xt];
                Nt = $t > 0 ? Pt * (Gt.count / $t) : 0;
                for (var Ft, Tt = new Array(Je.length), It = 0; It < Gt.categoryInds.length; It++) {
                  var Ct = Gt.categoryInds[It], Kt = it[It][Ct], Vt = st[It];
                  Tt[Vt] = Je[Vt][Kt], Je[Vt][Kt] += Nt;
                  var Yt = ct.dimensions[Vt].categories[Kt], sn = Yt.bands.length, hn = Yt.bands[sn - 1];
                  if (hn === void 0 || Gt.rawColor !== hn.rawColor) {
                    var Qt = hn === void 0 ? 0 : hn.y + hn.height;
                    Yt.bands.push({ key: Qt, color: Gt.color, rawColor: Gt.rawColor, height: Nt, width: Yt.width, count: Gt.count, y: Qt, categoryViewModel: Yt, parcatsViewModel: ct });
                  } else {
                    var Ut = Yt.bands[sn - 1];
                    Ut.height += Nt, Ut.count += Gt.count;
                  }
                }
                Ft = ct.pathShape === "hspline" ? Ge(bt, Tt, yt, Nt, 0.5) : Ge(bt, Tt, yt, Nt, 0), vt[Xt] = { key: Gt.valueInds[0], model: Gt, height: Nt, leftXs: bt, topYs: Tt, dimWidths: yt, svgD: Ft, parcatsViewModel: ct };
              }
              ct.paths = vt;
            }
            function We(ct) {
              var et = ct.model.dimensions.map(function(gt) {
                return { displayInd: gt.displayInd, dimensionInd: gt.dimensionInd };
              });
              et.sort(function(gt, bt) {
                return gt.displayInd - bt.displayInd;
              });
              var dt = [];
              for (var Je in et) {
                var it = et[Je].dimensionInd, st = ct.model.dimensions[it];
                dt.push(at(ct, st));
              }
              ct.dimensions = dt;
            }
            function at(ct, et) {
              var dt, Je = ct.model.dimensions.length, it = et.displayInd;
              dt = 40 + (Je > 1 ? (ct.width - 80 - 16) / (Je - 1) : 0) * it;
              var st, gt, bt, yt, St, Lt = [], Rt = ct.model.maxCats, vt = et.categories.length, $t = et.count, Pt = ct.height - 8 * (Rt - 1), Xt = 8 * (Rt - vt) / 2, Nt = et.categories.map(function(Gt) {
                return { displayInd: Gt.displayInd, categoryInd: Gt.categoryInd };
              });
              for (Nt.sort(function(Gt, Ft) {
                return Gt.displayInd - Ft.displayInd;
              }), St = 0; St < vt; St++) yt = Nt[St].categoryInd, gt = et.categories[yt], st = $t > 0 ? gt.count / $t * Pt : 0, bt = { key: gt.valueInds[0], model: gt, width: 16, height: st, y: gt.dragY !== null ? gt.dragY : Xt, bands: [], parcatsViewModel: ct }, Xt = Xt + st + 8, Lt.push(bt);
              return { key: et.dimensionInd, x: et.dragX !== null ? et.dragX : dt, y: 0, width: 16, model: et, categories: Lt, parcatsViewModel: ct, dragCategoryDisplayInd: null, dragDimensionDisplayInd: null, initialDragDimensionDisplayInds: null, initialDragCategoryDisplayInds: null, dragHasMoved: null, potentialClickBand: null };
            }
            G.exports = function(ct, et, dt, Je) {
              de(dt, ct, Je, et);
            };
          }, 37822: function(G, J, Z) {
            var te = Z(27219);
            G.exports = function(ne, ee, ae, oe) {
              var se = ne._fullLayout, ce = se._paper, le = se._size;
              te(ne, ce, ee, { width: le.w, height: le.h, margin: { t: le.t, r: le.r, b: le.b, l: le.l } }, ae, oe);
            };
          }, 59549: function(G, J, Z) {
            var te = Z(87163), ne = Z(25829), ee = Z(80337), ae = Z(13792).u, oe = Z(93049).extendFlat, se = Z(78032).templatedArray;
            G.exports = { domain: ae({ name: "parcoords", trace: !0, editType: "plot" }), labelangle: { valType: "angle", dflt: 0, editType: "plot" }, labelside: { valType: "enumerated", values: ["top", "bottom"], dflt: "top", editType: "plot" }, labelfont: ee({ editType: "plot" }), tickfont: ee({ autoShadowDflt: !0, editType: "plot" }), rangefont: ee({ editType: "plot" }), dimensions: se("dimension", { label: { valType: "string", editType: "plot" }, tickvals: oe({}, ne.tickvals, { editType: "plot" }), ticktext: oe({}, ne.ticktext, { editType: "plot" }), tickformat: oe({}, ne.tickformat, { editType: "plot" }), visible: { valType: "boolean", dflt: !0, editType: "plot" }, range: { valType: "info_array", items: [{ valType: "number", editType: "plot" }, { valType: "number", editType: "plot" }], editType: "plot" }, constraintrange: { valType: "info_array", freeLength: !0, dimensions: "1-2", items: [{ valType: "any", editType: "plot" }, { valType: "any", editType: "plot" }], editType: "plot" }, multiselect: { valType: "boolean", dflt: !0, editType: "plot" }, values: { valType: "data_array", editType: "calc" }, editType: "calc" }), line: oe({ editType: "calc" }, te("line", { colorscaleDflt: "Viridis", autoColorDflt: !1, editTypeOverride: "calc" })), unselected: { line: { color: { valType: "color", dflt: "#7f7f7f", editType: "plot" }, opacity: { valType: "number", min: 0, max: 1, dflt: "auto", editType: "plot" }, editType: "plot" }, editType: "plot" } };
          }, 23245: function(G, J, Z) {
            var te = Z(77911), ne = Z(45568), ee = Z(71293).keyFun, ae = Z(71293).repeat, oe = Z(34809).sorterAsc, se = Z(34809).strTranslate, ce = te.bar.snapRatio;
            function le(ke, Me) {
              return ke * (1 - ce) + Me * ce;
            }
            var ue = te.bar.snapClose;
            function de(ke, Me) {
              return ke * (1 - ue) + Me * ue;
            }
            function fe(ke, Me, Re, De) {
              if (function(at, ct) {
                for (var et = 0; et < ct.length; et++) if (at >= ct[et][0] && at <= ct[et][1]) return !0;
                return !1;
              }(Re, De)) return Re;
              var Pe = ke ? -1 : 1, $e = 0, qe = Me.length - 1;
              if (Pe < 0) {
                var Xe = $e;
                $e = qe, qe = Xe;
              }
              for (var Ve = Me[$e], Ze = Ve, Ge = $e; Pe * Ge < Pe * qe; Ge += Pe) {
                var ze = Ge + Pe, We = Me[ze];
                if (Pe * Re < Pe * de(Ve, We)) return le(Ve, Ze);
                if (Pe * Re < Pe * We || ze === qe) return le(We, Ve);
                Ze = Ve, Ve = We;
              }
            }
            function he(ke) {
              ke.attr("x", -te.bar.captureWidth / 2).attr("width", te.bar.captureWidth);
            }
            function me(ke) {
              ke.attr("visibility", "visible").style("visibility", "visible").attr("fill", "yellow").attr("opacity", 0);
            }
            function pe(ke) {
              if (!ke.brush.filterSpecified) return "0," + ke.height;
              for (var Me, Re, De, Pe = ye(ke.brush.filter.getConsolidated(), ke.height), $e = [0], qe = Pe.length ? Pe[0][0] : null, Xe = 0; Xe < Pe.length; Xe++) Re = (Me = Pe[Xe])[1] - Me[0], $e.push(qe), $e.push(Re), (De = Xe + 1) < Pe.length && (qe = Pe[De][0] - Me[1]);
              return $e.push(ke.height), $e;
            }
            function ye(ke, Me) {
              return ke.map(function(Re) {
                return Re.map(function(De) {
                  return Math.max(0, De * Me);
                }).sort(oe);
              });
            }
            function ge() {
              ne.select(document.body).style("cursor", null);
            }
            function ve(ke) {
              ke.attr("stroke-dasharray", pe);
            }
            function be(ke, Me) {
              var Re = ne.select(ke).selectAll(".highlight, .highlight-shadow");
              ve(Me ? Re.transition().duration(te.bar.snapDuration).each("end", Me) : Re);
            }
            function Ce(ke, Me) {
              var Re, De = ke.brush, Pe = NaN, $e = {};
              if (De.filterSpecified) {
                var qe = ke.height, Xe = De.filter.getConsolidated(), Ve = ye(Xe, qe), Ze = NaN, Ge = NaN, ze = NaN;
                for (Re = 0; Re <= Ve.length; Re++) {
                  var We = Ve[Re];
                  if (We && We[0] <= Me && Me <= We[1]) {
                    Ze = Re;
                    break;
                  }
                  if (Ge = Re ? Re - 1 : NaN, We && We[0] > Me) {
                    ze = Re;
                    break;
                  }
                }
                if (Pe = Ze, isNaN(Pe) && (Pe = isNaN(Ge) || isNaN(ze) ? isNaN(Ge) ? ze : Ge : Me - Ve[Ge][1] < Ve[ze][0] - Me ? Ge : ze), !isNaN(Pe)) {
                  var at = Ve[Pe], ct = function(it, st) {
                    var gt = te.bar.handleHeight;
                    if (!(st > it[1] + gt || st < it[0] - gt)) return st >= 0.9 * it[1] + 0.1 * it[0] ? "n" : st <= 0.9 * it[0] + 0.1 * it[1] ? "s" : "ns";
                  }(at, Me);
                  ct && ($e.interval = Xe[Pe], $e.intervalPix = at, $e.region = ct);
                }
              }
              if (ke.ordinal && !$e.region) {
                var et = ke.unitTickvals, dt = ke.unitToPaddedPx.invert(Me);
                for (Re = 0; Re < et.length; Re++) {
                  var Je = [0.25 * et[Math.max(Re - 1, 0)] + 0.75 * et[Re], 0.25 * et[Math.min(Re + 1, et.length - 1)] + 0.75 * et[Re]];
                  if (dt >= Je[0] && dt <= Je[1]) {
                    $e.clickableOrdinalRange = Je;
                    break;
                  }
                }
              }
              return $e;
            }
            function Se(ke, Me) {
              ne.event.sourceEvent.stopPropagation();
              var Re = Me.height - ne.mouse(ke)[1] - 2 * te.verticalPadding, De = Me.brush.svgBrush;
              De.wasDragged = !0, De._dragging = !0, De.grabbingBar ? De.newExtent = [Re - De.grabPoint, Re + De.barLength - De.grabPoint].map(Me.unitToPaddedPx.invert) : De.newExtent = [De.startExtent, Me.unitToPaddedPx.invert(Re)].sort(oe), Me.brush.filterSpecified = !0, De.extent = De.stayingIntervals.concat([De.newExtent]), De.brushCallback(Me), be(ke.parentNode);
            }
            function we(ke, Me) {
              var Re = Ce(Me, Me.height - ne.mouse(ke)[1] - 2 * te.verticalPadding), De = "crosshair";
              Re.clickableOrdinalRange ? De = "pointer" : Re.region && (De = Re.region + "-resize"), ne.select(document.body).style("cursor", De);
            }
            function Ee(ke) {
              ke.on("mousemove", function(Me) {
                ne.event.preventDefault(), Me.parent.inBrushDrag || we(this, Me);
              }).on("mouseleave", function(Me) {
                Me.parent.inBrushDrag || ge();
              }).call(ne.behavior.drag().on("dragstart", function(Me) {
                (function(Re, De) {
                  ne.event.sourceEvent.stopPropagation();
                  var Pe = De.height - ne.mouse(Re)[1] - 2 * te.verticalPadding, $e = De.unitToPaddedPx.invert(Pe), qe = De.brush, Xe = Ce(De, Pe), Ve = Xe.interval, Ze = qe.svgBrush;
                  if (Ze.wasDragged = !1, Ze.grabbingBar = Xe.region === "ns", Ze.grabbingBar) {
                    var Ge = Ve.map(De.unitToPaddedPx);
                    Ze.grabPoint = Pe - Ge[0] - te.verticalPadding, Ze.barLength = Ge[1] - Ge[0];
                  }
                  Ze.clickableOrdinalRange = Xe.clickableOrdinalRange, Ze.stayingIntervals = De.multiselect && qe.filterSpecified ? qe.filter.getConsolidated() : [], Ve && (Ze.stayingIntervals = Ze.stayingIntervals.filter(function(ze) {
                    return ze[0] !== Ve[0] && ze[1] !== Ve[1];
                  })), Ze.startExtent = Xe.region ? Ve[Xe.region === "s" ? 1 : 0] : $e, De.parent.inBrushDrag = !0, Ze.brushStartCallback();
                })(this, Me);
              }).on("drag", function(Me) {
                Se(this, Me);
              }).on("dragend", function(Me) {
                (function(Re, De) {
                  var Pe = De.brush, $e = Pe.filter, qe = Pe.svgBrush;
                  qe._dragging || (we(Re, De), Se(Re, De), De.brush.svgBrush.wasDragged = !1), qe._dragging = !1, ne.event.sourceEvent.stopPropagation();
                  var Xe = qe.grabbingBar;
                  if (qe.grabbingBar = !1, qe.grabLocation = void 0, De.parent.inBrushDrag = !1, ge(), !qe.wasDragged) return qe.wasDragged = void 0, qe.clickableOrdinalRange ? Pe.filterSpecified && De.multiselect ? qe.extent.push(qe.clickableOrdinalRange) : (qe.extent = [qe.clickableOrdinalRange], Pe.filterSpecified = !0) : Xe ? (qe.extent = qe.stayingIntervals, qe.extent.length === 0 && Le(Pe)) : Le(Pe), qe.brushCallback(De), be(Re.parentNode), void qe.brushEndCallback(Pe.filterSpecified ? $e.getConsolidated() : []);
                  var Ve = function() {
                    $e.set($e.getConsolidated());
                  };
                  if (De.ordinal) {
                    var Ze = De.unitTickvals;
                    Ze[Ze.length - 1] < Ze[0] && Ze.reverse(), qe.newExtent = [fe(0, Ze, qe.newExtent[0], qe.stayingIntervals), fe(1, Ze, qe.newExtent[1], qe.stayingIntervals)];
                    var Ge = qe.newExtent[1] > qe.newExtent[0];
                    qe.extent = qe.stayingIntervals.concat(Ge ? [qe.newExtent] : []), qe.extent.length || Le(Pe), qe.brushCallback(De), Ge ? be(Re.parentNode, Ve) : (Ve(), be(Re.parentNode));
                  } else Ve();
                  qe.brushEndCallback(Pe.filterSpecified ? $e.getConsolidated() : []);
                })(this, Me);
              }));
            }
            function xe(ke, Me) {
              return ke[0] - Me[0];
            }
            function Le(ke) {
              ke.filterSpecified = !1, ke.svgBrush.extent = [[-1 / 0, 1 / 0]];
            }
            function Qe(ke) {
              for (var Me, Re = ke.slice(), De = [], Pe = Re.shift(); Pe; ) {
                for (Me = Pe.slice(); (Pe = Re.shift()) && Pe[0] <= Me[1]; ) Me[1] = Math.max(Me[1], Pe[1]);
                De.push(Me);
              }
              return De.length === 1 && De[0][0] > De[0][1] && (De = []), De;
            }
            G.exports = { makeBrush: function(ke, Me, Re, De, Pe, $e) {
              var qe, Xe = /* @__PURE__ */ function() {
                var Ve, Ze, Ge = [];
                return { set: function(ze) {
                  (Ge = ze.map(function(We) {
                    return We.slice().sort(oe);
                  }).sort(xe)).length === 1 && Ge[0][0] === -1 / 0 && Ge[0][1] === 1 / 0 && (Ge = [[0, -1]]), Ve = Qe(Ge), Ze = Ge.reduce(function(We, at) {
                    return [Math.min(We[0], at[0]), Math.max(We[1], at[1])];
                  }, [1 / 0, -1 / 0]);
                }, get: function() {
                  return Ge.slice();
                }, getConsolidated: function() {
                  return Ve;
                }, getBounds: function() {
                  return Ze;
                } };
              }();
              return Xe.set(Re), { filter: Xe, filterSpecified: Me, svgBrush: { extent: [], brushStartCallback: De, brushCallback: (qe = Pe, function(Ve) {
                var Ze = Ve.brush, Ge = function(We) {
                  return We.svgBrush.extent.map(function(at) {
                    return at.slice();
                  });
                }(Ze), ze = Ge.slice();
                Ze.filter.set(ze), qe();
              }), brushEndCallback: $e } };
            }, ensureAxisBrush: function(ke, Me, Re) {
              var De = ke.selectAll("." + te.cn.axisBrush).data(ae, ee);
              De.enter().append("g").classed(te.cn.axisBrush, !0), function(Pe, $e, qe) {
                var Xe = qe._context.staticPlot, Ve = Pe.selectAll(".background").data(ae);
                Ve.enter().append("rect").classed("background", !0).call(he).call(me).style("pointer-events", Xe ? "none" : "auto").attr("transform", se(0, te.verticalPadding)), Ve.call(Ee).attr("height", function(ze) {
                  return ze.height - te.verticalPadding;
                });
                var Ze = Pe.selectAll(".highlight-shadow").data(ae);
                Ze.enter().append("line").classed("highlight-shadow", !0).attr("x", -te.bar.width / 2).attr("stroke-width", te.bar.width + te.bar.strokeWidth).attr("stroke", $e).attr("opacity", te.bar.strokeOpacity).attr("stroke-linecap", "butt"), Ze.attr("y1", function(ze) {
                  return ze.height;
                }).call(ve);
                var Ge = Pe.selectAll(".highlight").data(ae);
                Ge.enter().append("line").classed("highlight", !0).attr("x", -te.bar.width / 2).attr("stroke-width", te.bar.width - te.bar.strokeWidth).attr("stroke", te.bar.fillColor).attr("opacity", te.bar.fillOpacity).attr("stroke-linecap", "butt"), Ge.attr("y1", function(ze) {
                  return ze.height;
                }).call(ve);
              }(De, Me, Re);
            }, cleanRanges: function(ke, Me) {
              if (Array.isArray(ke[0]) ? (ke = ke.map(function(De) {
                return De.sort(oe);
              }), ke = Me.multiselect ? Qe(ke.sort(xe)) : [ke[0]]) : ke = [ke.sort(oe)], Me.tickvals) {
                var Re = Me.tickvals.slice().sort(oe);
                if (!(ke = ke.map(function(De) {
                  var Pe = [fe(0, Re, De[0], []), fe(1, Re, De[1], [])];
                  if (Pe[1] > Pe[0]) return Pe;
                }).filter(function(De) {
                  return De;
                })).length) return;
              }
              return ke.length > 1 ? ke : ke[0];
            } };
          }, 79846: function(G, J, Z) {
            G.exports = { attributes: Z(59549), supplyDefaults: Z(12842), calc: Z(20113), colorbar: { container: "line", min: "cmin", max: "cmax" }, moduleType: "trace", name: "parcoords", basePlotModule: Z(67207), categories: ["gl", "regl", "noOpacity", "noHover"], meta: {} };
          }, 67207: function(G, J, Z) {
            var te = Z(45568), ne = Z(4173).eV, ee = Z(58823), ae = Z(62972);
            J.name = "parcoords", J.plot = function(oe) {
              var se = ne(oe.calcdata, "parcoords")[0];
              se.length && ee(oe, se);
            }, J.clean = function(oe, se, ce, le) {
              var ue = le._has && le._has("parcoords"), de = se._has && se._has("parcoords");
              ue && !de && (le._paperdiv.selectAll(".parcoords").remove(), le._glimages.selectAll("*").remove());
            }, J.toSVG = function(oe) {
              var se = oe._fullLayout._glimages, ce = te.select(oe).selectAll(".svg-container");
              ce.filter(function(le, ue) {
                return ue === ce.size() - 1;
              }).selectAll(".gl-canvas-context, .gl-canvas-focus").each(function() {
                var le = this, ue = le.toDataURL("image/png");
                se.append("svg:image").attr({ xmlns: ae.svg, "xlink:href": ue, preserveAspectRatio: "none", x: 0, y: 0, width: le.style.width, height: le.style.height });
              }), window.setTimeout(function() {
                te.selectAll("#filterBarPattern").attr("id", "filterBarPattern");
              }, 60);
            };
          }, 20113: function(G, J, Z) {
            var te = Z(34809).isArrayOrTypedArray, ne = Z(88856), ee = Z(71293).wrap;
            G.exports = function(ae, oe) {
              var se, ce;
              return ne.hasColorscale(oe, "line") && te(oe.line.color) ? (se = oe.line.color, ce = ne.extractOpts(oe.line).colorscale, ne.calc(ae, oe, { vals: se, containerStr: "line", cLetter: "c" })) : (se = function(le) {
                for (var ue = new Array(le), de = 0; de < le; de++) ue[de] = 0.5;
                return ue;
              }(oe._length), ce = [[0, oe.line.color], [1, oe.line.color]]), ee({ lineColor: se, cscale: ce });
            };
          }, 77911: function(G) {
            G.exports = { maxDimensionCount: 60, overdrag: 45, verticalPadding: 2, tickDistance: 50, canvasPixelRatio: 1, blockLineCount: 5e3, layers: ["contextLineLayer", "focusLineLayer", "pickLineLayer"], axisTitleOffset: 28, axisExtentOffset: 10, bar: { width: 4, captureWidth: 10, fillColor: "magenta", fillOpacity: 1, snapDuration: 150, snapRatio: 0.25, snapClose: 0.01, strokeOpacity: 1, strokeWidth: 1, handleHeight: 8, handleOpacity: 1, handleOverlap: 0 }, cn: { axisExtentText: "axis-extent-text", parcoordsLineLayers: "parcoords-line-layers", parcoordsLineLayer: "parcoords-lines", parcoords: "parcoords", parcoordsControlView: "parcoords-control-view", yAxis: "y-axis", axisOverlays: "axis-overlays", axis: "axis", axisHeading: "axis-heading", axisTitle: "axis-title", axisExtent: "axis-extent", axisExtentTop: "axis-extent-top", axisExtentTopText: "axis-extent-top-text", axisExtentBottom: "axis-extent-bottom", axisExtentBottomText: "axis-extent-bottom-text", axisBrush: "axis-brush" }, id: { filterBarPattern: "filter-bar-pattern" } };
          }, 12842: function(G, J, Z) {
            var te = Z(34809), ne = Z(65477).hasColorscale, ee = Z(39356), ae = Z(13792).N, oe = Z(59008), se = Z(29714), ce = Z(59549), le = Z(23245), ue = Z(77911).maxDimensionCount, de = Z(63197);
            function fe(he, me, pe, ye) {
              function ge(we, Ee) {
                return te.coerce(he, me, ce.dimensions, we, Ee);
              }
              var ve = ge("values"), be = ge("visible");
              if (ve && ve.length || (be = me.visible = !1), be) {
                ge("label"), ge("tickvals"), ge("ticktext"), ge("tickformat");
                var Ce = ge("range");
                me._ax = { _id: "y", type: "linear", showexponent: "all", exponentformat: "B", range: Ce }, se.setConvert(me._ax, ye.layout), ge("multiselect");
                var Se = ge("constraintrange");
                Se && (me.constraintrange = le.cleanRanges(Se, me));
              }
            }
            G.exports = function(he, me, pe, ye) {
              function ge(we, Ee) {
                return te.coerce(he, me, ce, we, Ee);
              }
              var ve = he.dimensions;
              Array.isArray(ve) && ve.length > ue && (te.log("parcoords traces support up to " + ue + " dimensions at the moment"), ve.splice(ue));
              var be = oe(he, me, { name: "dimensions", layout: ye, handleItemDefaults: fe }), Ce = function(we, Ee, xe, Le, Qe) {
                var ke = Qe("line.color", xe);
                if (ne(we, "line") && te.isArrayOrTypedArray(ke)) {
                  if (ke.length) return Qe("line.colorscale"), ee(we, Ee, Le, Qe, { prefix: "line.", cLetter: "c" }), ke.length;
                  Ee.line.color = xe;
                }
                return 1 / 0;
              }(he, me, pe, ye, ge);
              ae(me, ye, ge), Array.isArray(be) && be.length || (me.visible = !1), de(me, be, "values", Ce);
              var Se = te.extendFlat({}, ye.font, { size: Math.round(ye.font.size / 1.2) });
              te.coerceFont(ge, "labelfont", Se), te.coerceFont(ge, "tickfont", Se, { autoShadowDflt: !0 }), te.coerceFont(ge, "rangefont", Se), ge("labelangle"), ge("labelside"), ge("unselected.line.color"), ge("unselected.line.opacity");
            };
          }, 62935: function(G, J, Z) {
            var te = Z(34809).isTypedArray;
            J.convertTypedArray = function(ne) {
              return te(ne) ? Array.prototype.slice.call(ne) : ne;
            }, J.isOrdinal = function(ne) {
              return !!ne.tickvals;
            }, J.isVisible = function(ne) {
              return ne.visible || !("visible" in ne);
            };
          }, 83910: function(G, J, Z) {
            var te = Z(79846);
            te.plot = Z(58823), G.exports = te;
          }, 1293: function(G, J, Z) {
            var te = ["precision highp float;", "", "varying vec4 fragColor;", "", "attribute vec4 p01_04, p05_08, p09_12, p13_16,", "               p17_20, p21_24, p25_28, p29_32,", "               p33_36, p37_40, p41_44, p45_48,", "               p49_52, p53_56, p57_60, colors;", "", "uniform mat4 dim0A, dim1A, dim0B, dim1B, dim0C, dim1C, dim0D, dim1D,", "             loA, hiA, loB, hiB, loC, hiC, loD, hiD;", "", "uniform vec2 resolution, viewBoxPos, viewBoxSize;", "uniform float maskHeight;", "uniform float drwLayer; // 0: context, 1: focus, 2: pick", "uniform vec4 contextColor;", "uniform sampler2D maskTexture, palette;", "", "bool isPick    = (drwLayer > 1.5);", "bool isContext = (drwLayer < 0.5);", "", "const vec4 ZEROS = vec4(0.0, 0.0, 0.0, 0.0);", "const vec4 UNITS = vec4(1.0, 1.0, 1.0, 1.0);", "", "float val(mat4 p, mat4 v) {", "    return dot(matrixCompMult(p, v) * UNITS, UNITS);", "}", "", "float axisY(float ratio, mat4 A, mat4 B, mat4 C, mat4 D) {", "    float y1 = val(A, dim0A) + val(B, dim0B) + val(C, dim0C) + val(D, dim0D);", "    float y2 = val(A, dim1A) + val(B, dim1B) + val(C, dim1C) + val(D, dim1D);", "    return y1 * (1.0 - ratio) + y2 * ratio;", "}", "", "int iMod(int a, int b) {", "    return a - b * (a / b);", "}", "", "bool fOutside(float p, float lo, float hi) {", "    return (lo < hi) && (lo > p || p > hi);", "}", "", "bool vOutside(vec4 p, vec4 lo, vec4 hi) {", "    return (", "        fOutside(p[0], lo[0], hi[0]) ||", "        fOutside(p[1], lo[1], hi[1]) ||", "        fOutside(p[2], lo[2], hi[2]) ||", "        fOutside(p[3], lo[3], hi[3])", "    );", "}", "", "bool mOutside(mat4 p, mat4 lo, mat4 hi) {", "    return (", "        vOutside(p[0], lo[0], hi[0]) ||", "        vOutside(p[1], lo[1], hi[1]) ||", "        vOutside(p[2], lo[2], hi[2]) ||", "        vOutside(p[3], lo[3], hi[3])", "    );", "}", "", "bool outsideBoundingBox(mat4 A, mat4 B, mat4 C, mat4 D) {", "    return mOutside(A, loA, hiA) ||", "           mOutside(B, loB, hiB) ||", "           mOutside(C, loC, hiC) ||", "           mOutside(D, loD, hiD);", "}", "", "bool outsideRasterMask(mat4 A, mat4 B, mat4 C, mat4 D) {", "    mat4 pnts[4];", "    pnts[0] = A;", "    pnts[1] = B;", "    pnts[2] = C;", "    pnts[3] = D;", "", "    for(int i = 0; i < 4; ++i) {", "        for(int j = 0; j < 4; ++j) {", "            for(int k = 0; k < 4; ++k) {", "                if(0 == iMod(", "                    int(255.0 * texture2D(maskTexture,", "                        vec2(", "                            (float(i * 2 + j / 2) + 0.5) / 8.0,", "                            (pnts[i][j][k] * (maskHeight - 1.0) + 1.0) / maskHeight", "                        ))[3]", "                    ) / int(pow(2.0, float(iMod(j * 4 + k, 8)))),", "                    2", "                )) return true;", "            }", "        }", "    }", "    return false;", "}", "", "vec4 position(bool isContext, float v, mat4 A, mat4 B, mat4 C, mat4 D) {", "    float x = 0.5 * sign(v) + 0.5;", "    float y = axisY(x, A, B, C, D);", "    float z = 1.0 - abs(v);", "", "    z += isContext ? 0.0 : 2.0 * float(", "        outsideBoundingBox(A, B, C, D) ||", "        outsideRasterMask(A, B, C, D)", "    );", "", "    return vec4(", "        2.0 * (vec2(x, y) * viewBoxSize + viewBoxPos) / resolution - 1.0,", "        z,", "        1.0", "    );", "}", "", "void main() {", "    mat4 A = mat4(p01_04, p05_08, p09_12, p13_16);", "    mat4 B = mat4(p17_20, p21_24, p25_28, p29_32);", "    mat4 C = mat4(p33_36, p37_40, p41_44, p45_48);", "    mat4 D = mat4(p49_52, p53_56, p57_60, ZEROS);", "", "    float v = colors[3];", "", "    gl_Position = position(isContext, v, A, B, C, D);", "", "    fragColor =", "        isContext ? vec4(contextColor) :", "        isPick ? vec4(colors.rgb, 1.0) : texture2D(palette, vec2(abs(v), 0.5));", "}"].join(`
`), ne = ["precision highp float;", "", "varying vec4 fragColor;", "", "void main() {", "    gl_FragColor = fragColor;", "}"].join(`
`), ee = Z(77911).maxDimensionCount, ae = Z(34809), oe = 1e-6, se = new Uint8Array(4), ce = new Uint8Array(4), le = { shape: [256, 1], format: "rgba", type: "uint8", mag: "nearest", min: "nearest" };
            function ue(be, Ce, Se, we, Ee) {
              var xe = be._gl;
              xe.enable(xe.SCISSOR_TEST), xe.scissor(Ce, Se, we, Ee), be.clear({ color: [0, 0, 0, 0], depth: 1 });
            }
            function de(be, Ce, Se, we, Ee, xe) {
              var Le = xe.key;
              Se.drawCompleted || (function(Qe) {
                Qe.read({ x: 0, y: 0, width: 1, height: 1, data: se });
              }(be), Se.drawCompleted = !0), function Qe(ke) {
                var Me = Math.min(we, Ee - ke * we);
                ke === 0 && (window.cancelAnimationFrame(Se.currentRafs[Le]), delete Se.currentRafs[Le], ue(be, xe.scissorX, xe.scissorY, xe.scissorWidth, xe.viewBoxSize[1])), Se.clearOnly || (xe.count = 2 * Me, xe.offset = 2 * ke * we, Ce(xe), ke * we + Me < Ee && (Se.currentRafs[Le] = window.requestAnimationFrame(function() {
                  Qe(ke + 1);
                })), Se.drawCompleted = !1);
              }(0);
            }
            function fe(be, Ce) {
              for (var Se = new Array(256), we = 0; we < 256; we++) Se[we] = be(we / 255).concat(Ce);
              return Se;
            }
            function he(be, Ce) {
              return (be >>> 8 * Ce) % 256 / 255;
            }
            function me(be, Ce, Se) {
              for (var we = new Array(8 * Ce), Ee = 0, xe = 0; xe < Ce; xe++) for (var Le = 0; Le < 2; Le++) for (var Qe = 0; Qe < 4; Qe++) {
                var ke = 4 * be + Qe, Me = Se[64 * xe + ke];
                ke === 63 && Le === 0 && (Me *= -1), we[Ee++] = Me;
              }
              return we;
            }
            function pe(be) {
              var Ce = "0" + be;
              return Ce.substr(Ce.length - 2);
            }
            function ye(be) {
              return be < ee ? "p" + pe(be + 1) + "_" + pe(be + 4) : "colors";
            }
            function ge(be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e) {
              for (var qe = [[], []], Xe = 0; Xe < 64; Xe++) qe[0][Xe] = Xe === Ee ? 1 : 0, qe[1][Xe] = Xe === xe ? 1 : 0;
              Le *= $e, Qe *= $e, ke *= $e, Me *= $e;
              var Ve = be.lines.canvasOverdrag * $e, Ze = be.domain, Ge = be.canvasWidth * $e, ze = be.canvasHeight * $e, We = be.pad.l * $e, at = be.pad.b * $e, ct = be.layoutHeight * $e, et = be.layoutWidth * $e, dt = be.deselectedLines.color, Je = be.deselectedLines.opacity;
              return ae.extendFlat({ key: Re, resolution: [Ge, ze], viewBoxPos: [Le + Ve, Qe], viewBoxSize: [ke, Me], i0: Ee, i1: xe, dim0A: qe[0].slice(0, 16), dim0B: qe[0].slice(16, 32), dim0C: qe[0].slice(32, 48), dim0D: qe[0].slice(48, 64), dim1A: qe[1].slice(0, 16), dim1B: qe[1].slice(16, 32), dim1C: qe[1].slice(32, 48), dim1D: qe[1].slice(48, 64), drwLayer: De, contextColor: [dt[0] / 255, dt[1] / 255, dt[2] / 255, Je !== "auto" ? dt[3] * Je : Math.max(0.00392156862745098, Math.pow(1 / be.lines.color.length, 0.3333333333333333))], scissorX: (we === Ce ? 0 : Le + Ve) + (We - Ve) + et * Ze.x[0], scissorWidth: (we === Se ? Ge - Le + Ve : ke + 0.5) + (we === Ce ? Le + Ve : 0), scissorY: Qe + at + ct * Ze.y[0], scissorHeight: Me, viewportX: We - Ve + et * Ze.x[0], viewportY: at + ct * Ze.y[0], viewportWidth: Ge, viewportHeight: ze }, Pe);
            }
            function ve(be) {
              var Ce = 2047, Se = Math.max(0, Math.floor(be[0] * Ce), 0), we = Math.min(Ce, Math.ceil(be[1] * Ce), Ce);
              return [Math.min(Se, we), Math.max(Se, we)];
            }
            G.exports = function(be, Ce) {
              var Se, we, Ee, xe, Le, Qe = Ce.context, ke = Ce.pick, Me = Ce.regl, Re = Me._gl, De = Re.getParameter(Re.ALIASED_LINE_WIDTH_RANGE), Pe = Math.max(De[0], Math.min(De[1], Ce.viewModel.plotGlPixelRatio)), $e = { currentRafs: {}, drawCompleted: !0, clearOnly: !1 }, qe = function(ze) {
                for (var We = {}, at = 0; at <= ee; at += 4) We[ye(at)] = ze.buffer({ usage: "dynamic", type: "float", data: new Uint8Array(0) });
                return We;
              }(Me), Xe = Me.texture(le), Ve = [];
              Ge(Ce);
              var Ze = Me({ profile: !1, blend: { enable: Qe, func: { srcRGB: "src alpha", dstRGB: "one minus src alpha", srcAlpha: 1, dstAlpha: 1 }, equation: { rgb: "add", alpha: "add" }, color: [0, 0, 0, 0] }, depth: { enable: !Qe, mask: !0, func: "less", range: [0, 1] }, cull: { enable: !0, face: "back" }, scissor: { enable: !0, box: { x: Me.prop("scissorX"), y: Me.prop("scissorY"), width: Me.prop("scissorWidth"), height: Me.prop("scissorHeight") } }, viewport: { x: Me.prop("viewportX"), y: Me.prop("viewportY"), width: Me.prop("viewportWidth"), height: Me.prop("viewportHeight") }, dither: !1, vert: te, frag: ne, primitive: "lines", lineWidth: Pe, attributes: qe, uniforms: { resolution: Me.prop("resolution"), viewBoxPos: Me.prop("viewBoxPos"), viewBoxSize: Me.prop("viewBoxSize"), dim0A: Me.prop("dim0A"), dim1A: Me.prop("dim1A"), dim0B: Me.prop("dim0B"), dim1B: Me.prop("dim1B"), dim0C: Me.prop("dim0C"), dim1C: Me.prop("dim1C"), dim0D: Me.prop("dim0D"), dim1D: Me.prop("dim1D"), loA: Me.prop("loA"), hiA: Me.prop("hiA"), loB: Me.prop("loB"), hiB: Me.prop("hiB"), loC: Me.prop("loC"), hiC: Me.prop("hiC"), loD: Me.prop("loD"), hiD: Me.prop("hiD"), palette: Xe, contextColor: Me.prop("contextColor"), maskTexture: Me.prop("maskTexture"), drwLayer: Me.prop("drwLayer"), maskHeight: Me.prop("maskHeight") }, offset: Me.prop("offset"), count: Me.prop("count") });
              function Ge(ze) {
                Se = ze.model, we = ze.viewModel, Ee = we.dimensions.slice(), xe = Ee[0] ? Ee[0].values.length : 0;
                var We = Se.lines, at = ke ? We.color.map(function(et, dt) {
                  return dt / We.color.length;
                }) : We.color, ct = function(et, dt, Je) {
                  for (var it, st = new Array(et * (ee + 4)), gt = 0, bt = 0; bt < et; bt++) {
                    for (var yt = 0; yt < ee; yt++) st[gt++] = yt < dt.length ? dt[yt].paddedUnitValues[bt] : 0.5;
                    st[gt++] = he(bt, 2), st[gt++] = he(bt, 1), st[gt++] = he(bt, 0), st[gt++] = (it = Je[bt], Math.max(oe, Math.min(0.999999, it)));
                  }
                  return st;
                }(xe, Ee, at);
                (function(et, dt, Je) {
                  for (var it = 0; it <= ee; it += 4) et[ye(it)](me(it / 4, dt, Je));
                })(qe, xe, ct), Qe || ke || (Xe = Me.texture(ae.extendFlat({ data: fe(Se.unitToColor, 255) }, le)));
              }
              return { render: function(ze, We, at) {
                var ct, et, dt, Je = ze.length, it = 1 / 0, st = -1 / 0;
                for (ct = 0; ct < Je; ct++) ze[ct].dim0.canvasX < it && (it = ze[ct].dim0.canvasX, et = ct), ze[ct].dim1.canvasX > st && (st = ze[ct].dim1.canvasX, dt = ct);
                Je === 0 && ue(Me, 0, 0, Se.canvasWidth, Se.canvasHeight);
                var gt = function(Nt) {
                  var Gt, Ft, Tt, It = [[], []];
                  for (Tt = 0; Tt < 64; Tt++) {
                    var Ct = !Nt && Tt < Ee.length ? Ee[Tt].brush.filter.getBounds() : [-1 / 0, 1 / 0];
                    It[0][Tt] = Ct[0], It[1][Tt] = Ct[1];
                  }
                  var Kt = new Array(16384);
                  for (Gt = 0; Gt < 16384; Gt++) Kt[Gt] = 255;
                  if (!Nt) for (Gt = 0; Gt < Ee.length; Gt++) {
                    var Vt = Gt % 8, Yt = (Gt - Vt) / 8, sn = Math.pow(2, Vt), hn = Ee[Gt].brush.filter.get();
                    if (!(hn.length < 2)) {
                      var Qt = ve(hn[0])[1];
                      for (Ft = 1; Ft < hn.length; Ft++) {
                        var Ut = ve(hn[Ft]);
                        for (Tt = Qt + 1; Tt < Ut[0]; Tt++) Kt[8 * Tt + Yt] &= ~sn;
                        Qt = Math.max(Qt, Ut[1]);
                      }
                    }
                  }
                  var Wt = { shape: [8, 2048], format: "alpha", type: "uint8", mag: "nearest", min: "nearest", data: Kt };
                  return Le ? Le(Wt) : Le = Me.texture(Wt), { maskTexture: Le, maskHeight: 2048, loA: It[0].slice(0, 16), loB: It[0].slice(16, 32), loC: It[0].slice(32, 48), loD: It[0].slice(48, 64), hiA: It[1].slice(0, 16), hiB: It[1].slice(16, 32), hiC: It[1].slice(32, 48), hiD: It[1].slice(48, 64) };
                }(Qe);
                for (ct = 0; ct < Je; ct++) {
                  var bt = ze[ct], yt = bt.dim0.crossfilterDimensionIndex, St = bt.dim1.crossfilterDimensionIndex, Lt = bt.canvasX, Rt = bt.canvasY, vt = Lt + bt.panelSizeX, $t = bt.plotGlPixelRatio;
                  if (We || !Ve[yt] || Ve[yt][0] !== Lt || Ve[yt][1] !== vt) {
                    Ve[yt] = [Lt, vt];
                    var Pt = ge(Se, et, dt, ct, yt, St, Lt, Rt, bt.panelSizeX, bt.panelSizeY, bt.dim0.crossfilterDimensionIndex, Qe ? 0 : ke ? 2 : 1, gt, $t);
                    $e.clearOnly = at;
                    var Xt = We ? Se.lines.blockLineCount : xe;
                    de(Me, Ze, $e, Xt, xe, Pt);
                  }
                }
              }, readPixel: function(ze, We) {
                return Me.read({ x: ze, y: We, width: 1, height: 1, data: ce }), ce;
              }, readPixels: function(ze, We, at, ct) {
                var et = new Uint8Array(4 * at * ct);
                return Me.read({ x: ze, y: We, width: at, height: ct, data: et }), et;
              }, destroy: function() {
                for (var ze in be.style["pointer-events"] = "none", Xe.destroy(), Le && Le.destroy(), qe) qe[ze].destroy();
              }, update: Ge };
            };
          }, 63197: function(G) {
            G.exports = function(J, Z, te, ne) {
              var ee, ae;
              for (ne || (ne = 1 / 0), ee = 0; ee < Z.length; ee++) (ae = Z[ee]).visible && (ne = Math.min(ne, ae[te].length));
              for (ne === 1 / 0 && (ne = 0), J._length = ne, ee = 0; ee < Z.length; ee++) (ae = Z[ee]).visible && (ae._length = ne);
              return ne;
            };
          }, 16019: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = ne.isArrayOrTypedArray, ae = ne.numberFormat, oe = Z(16401), se = Z(29714), ce = ne.strRotate, le = ne.strTranslate, ue = Z(30635), de = Z(62203), fe = Z(88856), he = Z(71293), me = he.keyFun, pe = he.repeat, ye = he.unwrap, ge = Z(62935), ve = Z(77911), be = Z(23245), Ce = Z(1293);
            function Se(Ze, Ge, ze) {
              return ne.aggNums(Ze, null, Ge, ze);
            }
            function we(Ze, Ge) {
              return xe(Se(Math.min, Ze, Ge), Se(Math.max, Ze, Ge));
            }
            function Ee(Ze) {
              var Ge = Ze.range;
              return Ge ? xe(Ge[0], Ge[1]) : we(Ze.values, Ze._length);
            }
            function xe(Ze, Ge) {
              return !isNaN(Ze) && isFinite(Ze) || (Ze = 0), !isNaN(Ge) && isFinite(Ge) || (Ge = 0), Ze === Ge && (Ze === 0 ? (Ze -= 1, Ge += 1) : (Ze *= 0.9, Ge *= 1.1)), [Ze, Ge];
            }
            function Le(Ze, Ge, ze, We, at) {
              var ct, et, dt = Ee(ze);
              return We ? te.scale.ordinal().domain(We.map((ct = ae(ze.tickformat), et = at, et ? function(Je, it) {
                var st = et[it];
                return st ?? ct(Je);
              } : ct))).range(We.map(function(Je) {
                var it = (Je - dt[0]) / (dt[1] - dt[0]);
                return Ze - Ge + it * (2 * Ge - Ze);
              })) : te.scale.linear().domain(dt).range([Ze - Ge, Ge]);
            }
            function Qe(Ze) {
              if (Ze.tickvals) {
                var Ge = Ee(Ze);
                return te.scale.ordinal().domain(Ze.tickvals).range(Ze.tickvals.map(function(ze) {
                  return (ze - Ge[0]) / (Ge[1] - Ge[0]);
                }));
              }
            }
            function ke(Ze) {
              var Ge = Ze.map(function(at) {
                return at[0];
              }), ze = Ze.map(function(at) {
                var ct = oe(at[1]);
                return te.rgb("rgb(" + ct[0] + "," + ct[1] + "," + ct[2] + ")");
              }), We = "rgb".split("").map(function(at) {
                return te.scale.linear().clamp(!0).domain(Ge).range(ze.map((ct = at, function(et) {
                  return et[ct];
                })));
                var ct;
              });
              return function(at) {
                return We.map(function(ct) {
                  return ct(at);
                });
              };
            }
            function Me(Ze) {
              return Ze.dimensions.some(function(Ge) {
                return Ge.brush.filterSpecified;
              });
            }
            function Re(Ze, Ge, ze) {
              var We = ye(Ge), at = We.trace, ct = ge.convertTypedArray(We.lineColor), et = at.line, dt = { color: oe(at.unselected.line.color), opacity: at.unselected.line.opacity }, Je = fe.extractOpts(et), it = Je.reversescale ? fe.flipScale(We.cscale) : We.cscale, st = at.domain, gt = at.dimensions, bt = Ze.width, yt = at.labelangle, St = at.labelside, Lt = at.labelfont, Rt = at.tickfont, vt = at.rangefont, $t = ne.extendDeepNoArrays({}, et, { color: ct.map(te.scale.linear().domain(Ee({ values: ct, range: [Je.min, Je.max], _length: at._length }))), blockLineCount: ve.blockLineCount, canvasOverdrag: ve.overdrag * ve.canvasPixelRatio }), Pt = Math.floor(bt * (st.x[1] - st.x[0])), Xt = Math.floor(Ze.height * (st.y[1] - st.y[0])), Nt = Ze.margin || { l: 80, r: 80, t: 100, b: 80 }, Gt = Pt, Ft = Xt;
              return { key: ze, colCount: gt.filter(ge.isVisible).length, dimensions: gt, tickDistance: ve.tickDistance, unitToColor: ke(it), lines: $t, deselectedLines: dt, labelAngle: yt, labelSide: St, labelFont: Lt, tickFont: Rt, rangeFont: vt, layoutWidth: bt, layoutHeight: Ze.height, domain: st, translateX: st.x[0] * bt, translateY: Ze.height - st.y[1] * Ze.height, pad: Nt, canvasWidth: Gt * ve.canvasPixelRatio + 2 * $t.canvasOverdrag, canvasHeight: Ft * ve.canvasPixelRatio, width: Gt, height: Ft, canvasPixelRatio: ve.canvasPixelRatio };
            }
            function De(Ze, Ge, ze) {
              var We = ze.width, at = ze.height, ct = ze.dimensions, et = ze.canvasPixelRatio, dt = function(bt) {
                return We * bt / Math.max(1, ze.colCount - 1);
              }, Je = ve.verticalPadding / at, it = function(bt, yt) {
                return te.scale.linear().range([yt, bt - yt]);
              }(at, ve.verticalPadding), st = { key: ze.key, xScale: dt, model: ze, inBrushDrag: !1 }, gt = {};
              return st.dimensions = ct.filter(ge.isVisible).map(function(bt, yt) {
                var St = function(Vt, Yt) {
                  return te.scale.linear().domain(Ee(Vt)).range([Yt, 1 - Yt]);
                }(bt, Je), Lt = gt[bt.label];
                gt[bt.label] = (Lt || 0) + 1;
                var Rt = bt.label + (Lt ? "__" + Lt : ""), vt = bt.constraintrange, $t = vt && vt.length;
                $t && !ee(vt[0]) && (vt = [vt]);
                var Pt = $t ? vt.map(function(Vt) {
                  return Vt.map(St);
                }) : [[-1 / 0, 1 / 0]], Xt = bt.values;
                Xt.length > bt._length && (Xt = Xt.slice(0, bt._length));
                var Nt, Gt = bt.tickvals;
                function Ft(Vt, Yt) {
                  return { val: Vt, text: Nt[Yt] };
                }
                function Tt(Vt, Yt) {
                  return Vt.val - Yt.val;
                }
                if (ee(Gt) && Gt.length) {
                  ne.isTypedArray(Gt) && (Gt = Array.from(Gt)), Nt = bt.ticktext, ee(Nt) && Nt.length ? Nt.length > Gt.length ? Nt = Nt.slice(0, Gt.length) : Gt.length > Nt.length && (Gt = Gt.slice(0, Nt.length)) : Nt = Gt.map(ae(bt.tickformat));
                  for (var It = 1; It < Gt.length; It++) if (Gt[It] < Gt[It - 1]) {
                    for (var Ct = Gt.map(Ft).sort(Tt), Kt = 0; Kt < Gt.length; Kt++) Gt[Kt] = Ct[Kt].val, Nt[Kt] = Ct[Kt].text;
                    break;
                  }
                } else Gt = void 0;
                return Xt = ge.convertTypedArray(Xt), { key: Rt, label: bt.label, tickFormat: bt.tickformat, tickvals: Gt, ticktext: Nt, ordinal: ge.isOrdinal(bt), multiselect: bt.multiselect, xIndex: yt, crossfilterDimensionIndex: yt, visibleIndex: bt._index, height: at, values: Xt, paddedUnitValues: Xt.map(St), unitTickvals: Gt && Gt.map(St), xScale: dt, x: dt(yt), canvasX: dt(yt) * et, unitToPaddedPx: it, domainScale: Le(at, ve.verticalPadding, bt, Gt, Nt), ordinalScale: Qe(bt), parent: st, model: ze, brush: be.makeBrush(Ze, $t, Pt, function() {
                  Ze.linePickActive(!1);
                }, function() {
                  var Vt = st;
                  Vt.focusLayer && Vt.focusLayer.render(Vt.panels, !0);
                  var Yt = Me(Vt);
                  !Ze.contextShown() && Yt ? (Vt.contextLayer && Vt.contextLayer.render(Vt.panels, !0), Ze.contextShown(!0)) : Ze.contextShown() && !Yt && (Vt.contextLayer && Vt.contextLayer.render(Vt.panels, !0, !0), Ze.contextShown(!1));
                }, function(Vt) {
                  if (st.focusLayer.render(st.panels, !0), st.pickLayer && st.pickLayer.render(st.panels, !0), Ze.linePickActive(!0), Ge && Ge.filterChanged) {
                    var Yt = St.invert, sn = Vt.map(function(hn) {
                      return hn.map(Yt).sort(ne.sorterAsc);
                    }).sort(function(hn, Qt) {
                      return hn[0] - Qt[0];
                    });
                    Ge.filterChanged(st.key, bt._index, sn);
                  }
                }) };
              }), st;
            }
            function Pe(Ze) {
              Ze.classed(ve.cn.axisExtentText, !0).attr("text-anchor", "middle").style("cursor", "default");
            }
            function $e(Ze, Ge) {
              var ze = Ge === "top" ? 1 : -1, We = Ze * Math.PI / 180;
              return { dir: ze, dx: Math.sin(We), dy: Math.cos(We), degrees: Ze };
            }
            function qe(Ze, Ge, ze) {
              for (var We = Ge.panels || (Ge.panels = []), at = Ze.data(), ct = 0; ct < at.length - 1; ct++) {
                var et = We[ct] || (We[ct] = {}), dt = at[ct], Je = at[ct + 1];
                et.dim0 = dt, et.dim1 = Je, et.canvasX = dt.canvasX, et.panelSizeX = Je.canvasX - dt.canvasX, et.panelSizeY = Ge.model.canvasHeight, et.y = 0, et.canvasY = 0, et.plotGlPixelRatio = ze;
              }
            }
            function Xe(Ze, Ge) {
              return se.tickText(Ze._ax, Ge, !1).text;
            }
            function Ve(Ze, Ge) {
              if (Ze.ordinal) return "";
              var ze = Ze.domainScale.domain(), We = ze[Ge ? ze.length - 1 : 0];
              return Xe(Ze.model.dimensions[Ze.visibleIndex], We);
            }
            G.exports = function(Ze, Ge, ze, We) {
              var at = Ze._context.staticPlot, ct = Ze._fullLayout, et = ct._toppaper, dt = ct._glcontainer, Je = Ze._context.plotGlPixelRatio, it = Ze._fullLayout.paper_bgcolor;
              (function(Vt) {
                for (var Yt = 0; Yt < Vt.length; Yt++) for (var sn = 0; sn < Vt[Yt].length; sn++) for (var hn = Vt[Yt][sn].trace, Qt = hn.dimensions, Ut = 0; Ut < Qt.length; Ut++) {
                  var Wt = Qt[Ut].values, rn = Qt[Ut]._ax;
                  rn && (rn.range ? rn.range = xe(rn.range[0], rn.range[1]) : rn.range = we(Wt, hn._length), rn.dtick || (rn.dtick = 0.01 * (Math.abs(rn.range[1] - rn.range[0]) || 1)), rn.tickformat = Qt[Ut].tickformat, se.calcTicks(rn), rn.cleanRange());
                }
              })(Ge);
              var st, gt, bt = (st = !0, gt = !1, { linePickActive: function(Vt) {
                return arguments.length ? st = !!Vt : st;
              }, contextShown: function(Vt) {
                return arguments.length ? gt = !!Vt : gt;
              } }), yt = Ge.filter(function(Vt) {
                return ye(Vt).trace.visible;
              }).map(Re.bind(0, ze)).map(De.bind(0, bt, We));
              dt.each(function(Vt, Yt) {
                return ne.extendFlat(Vt, yt[Yt]);
              });
              var St = dt.selectAll(".gl-canvas").each(function(Vt) {
                Vt.viewModel = yt[0], Vt.viewModel.plotGlPixelRatio = Je, Vt.viewModel.paperColor = it, Vt.model = Vt.viewModel ? Vt.viewModel.model : null;
              }), Lt = null;
              St.filter(function(Vt) {
                return Vt.pick;
              }).style("pointer-events", at ? "none" : "auto").on("mousemove", function(Vt) {
                if (bt.linePickActive() && Vt.lineLayer && We && We.hover) {
                  var Yt = te.event, sn = this.width, hn = this.height, Qt = te.mouse(this), Ut = Qt[0], Wt = Qt[1];
                  if (Ut < 0 || Wt < 0 || Ut >= sn || Wt >= hn) return;
                  var rn = Vt.lineLayer.readPixel(Ut, hn - 1 - Wt), vn = rn[3] !== 0, dn = vn ? rn[2] + 256 * (rn[1] + 256 * rn[0]) : null, Ht = { x: Ut, y: Wt, clientX: Yt.clientX, clientY: Yt.clientY, dataIndex: Vt.model.key, curveNumber: dn };
                  dn !== Lt && (vn ? We.hover(Ht) : We.unhover && We.unhover(Ht), Lt = dn);
                }
              }), St.style("opacity", function(Vt) {
                return Vt.pick ? 0 : 1;
              }), et.style("background", "rgba(255, 255, 255, 0)");
              var Rt = et.selectAll("." + ve.cn.parcoords).data(yt, me);
              Rt.exit().remove(), Rt.enter().append("g").classed(ve.cn.parcoords, !0).style("shape-rendering", "crispEdges").style("pointer-events", "none"), Rt.attr("transform", function(Vt) {
                return le(Vt.model.translateX, Vt.model.translateY);
              });
              var vt = Rt.selectAll("." + ve.cn.parcoordsControlView).data(pe, me);
              vt.enter().append("g").classed(ve.cn.parcoordsControlView, !0), vt.attr("transform", function(Vt) {
                return le(Vt.model.pad.l, Vt.model.pad.t);
              });
              var $t = vt.selectAll("." + ve.cn.yAxis).data(function(Vt) {
                return Vt.dimensions;
              }, me);
              $t.enter().append("g").classed(ve.cn.yAxis, !0), vt.each(function(Vt) {
                qe($t, Vt, Je);
              }), St.each(function(Vt) {
                if (Vt.viewModel) {
                  !Vt.lineLayer || We ? Vt.lineLayer = Ce(this, Vt) : Vt.lineLayer.update(Vt), (Vt.key || Vt.key === 0) && (Vt.viewModel[Vt.key] = Vt.lineLayer);
                  var Yt = !Vt.context || We;
                  Vt.lineLayer.render(Vt.viewModel.panels, Yt);
                }
              }), $t.attr("transform", function(Vt) {
                return le(Vt.xScale(Vt.xIndex), 0);
              }), $t.call(te.behavior.drag().origin(function(Vt) {
                return Vt;
              }).on("drag", function(Vt) {
                var Yt = Vt.parent;
                bt.linePickActive(!1), Vt.x = Math.max(-ve.overdrag, Math.min(Vt.model.width + ve.overdrag, te.event.x)), Vt.canvasX = Vt.x * Vt.model.canvasPixelRatio, $t.sort(function(sn, hn) {
                  return sn.x - hn.x;
                }).each(function(sn, hn) {
                  sn.xIndex = hn, sn.x = Vt === sn ? sn.x : sn.xScale(sn.xIndex), sn.canvasX = sn.x * sn.model.canvasPixelRatio;
                }), qe($t, Yt, Je), $t.filter(function(sn) {
                  return Math.abs(Vt.xIndex - sn.xIndex) !== 0;
                }).attr("transform", function(sn) {
                  return le(sn.xScale(sn.xIndex), 0);
                }), te.select(this).attr("transform", le(Vt.x, 0)), $t.each(function(sn, hn, Qt) {
                  Qt === Vt.parent.key && (Yt.dimensions[hn] = sn);
                }), Yt.contextLayer && Yt.contextLayer.render(Yt.panels, !1, !Me(Yt)), Yt.focusLayer.render && Yt.focusLayer.render(Yt.panels);
              }).on("dragend", function(Vt) {
                var Yt = Vt.parent;
                Vt.x = Vt.xScale(Vt.xIndex), Vt.canvasX = Vt.x * Vt.model.canvasPixelRatio, qe($t, Yt, Je), te.select(this).attr("transform", function(sn) {
                  return le(sn.x, 0);
                }), Yt.contextLayer && Yt.contextLayer.render(Yt.panels, !1, !Me(Yt)), Yt.focusLayer && Yt.focusLayer.render(Yt.panels), Yt.pickLayer && Yt.pickLayer.render(Yt.panels, !0), bt.linePickActive(!0), We && We.axesMoved && We.axesMoved(Yt.key, Yt.dimensions.map(function(sn) {
                  return sn.crossfilterDimensionIndex;
                }));
              })), $t.exit().remove();
              var Pt = $t.selectAll("." + ve.cn.axisOverlays).data(pe, me);
              Pt.enter().append("g").classed(ve.cn.axisOverlays, !0), Pt.selectAll("." + ve.cn.axis).remove();
              var Xt = Pt.selectAll("." + ve.cn.axis).data(pe, me);
              Xt.enter().append("g").classed(ve.cn.axis, !0), Xt.each(function(Vt) {
                var Yt = Vt.model.height / Vt.model.tickDistance, sn = Vt.domainScale, hn = sn.domain();
                te.select(this).call(te.svg.axis().orient("left").tickSize(4).outerTickSize(2).ticks(Yt, Vt.tickFormat).tickValues(Vt.ordinal ? hn : null).tickFormat(function(Qt) {
                  return ge.isOrdinal(Vt) ? Qt : Xe(Vt.model.dimensions[Vt.visibleIndex], Qt);
                }).scale(sn)), de.font(Xt.selectAll("text"), Vt.model.tickFont);
              }), Xt.selectAll(".domain, .tick>line").attr("fill", "none").attr("stroke", "black").attr("stroke-opacity", 0.25).attr("stroke-width", "1px"), Xt.selectAll("text").style("cursor", "default");
              var Nt = Pt.selectAll("." + ve.cn.axisHeading).data(pe, me);
              Nt.enter().append("g").classed(ve.cn.axisHeading, !0);
              var Gt = Nt.selectAll("." + ve.cn.axisTitle).data(pe, me);
              Gt.enter().append("text").classed(ve.cn.axisTitle, !0).attr("text-anchor", "middle").style("cursor", "ew-resize").style("pointer-events", at ? "none" : "auto"), Gt.text(function(Vt) {
                return Vt.label;
              }).each(function(Vt) {
                var Yt = te.select(this);
                de.font(Yt, Vt.model.labelFont), ue.convertToTspans(Yt, Ze);
              }).attr("transform", function(Vt) {
                var Yt = $e(Vt.model.labelAngle, Vt.model.labelSide), sn = ve.axisTitleOffset;
                return (Yt.dir > 0 ? "" : le(0, 2 * sn + Vt.model.height)) + ce(Yt.degrees) + le(-sn * Yt.dx, -sn * Yt.dy);
              }).attr("text-anchor", function(Vt) {
                var Yt = $e(Vt.model.labelAngle, Vt.model.labelSide);
                return 2 * Math.abs(Yt.dx) > Math.abs(Yt.dy) ? Yt.dir * Yt.dx < 0 ? "start" : "end" : "middle";
              });
              var Ft = Pt.selectAll("." + ve.cn.axisExtent).data(pe, me);
              Ft.enter().append("g").classed(ve.cn.axisExtent, !0);
              var Tt = Ft.selectAll("." + ve.cn.axisExtentTop).data(pe, me);
              Tt.enter().append("g").classed(ve.cn.axisExtentTop, !0), Tt.attr("transform", le(0, -ve.axisExtentOffset));
              var It = Tt.selectAll("." + ve.cn.axisExtentTopText).data(pe, me);
              It.enter().append("text").classed(ve.cn.axisExtentTopText, !0).call(Pe), It.text(function(Vt) {
                return Ve(Vt, !0);
              }).each(function(Vt) {
                de.font(te.select(this), Vt.model.rangeFont);
              });
              var Ct = Ft.selectAll("." + ve.cn.axisExtentBottom).data(pe, me);
              Ct.enter().append("g").classed(ve.cn.axisExtentBottom, !0), Ct.attr("transform", function(Vt) {
                return le(0, Vt.model.height + ve.axisExtentOffset);
              });
              var Kt = Ct.selectAll("." + ve.cn.axisExtentBottomText).data(pe, me);
              Kt.enter().append("text").classed(ve.cn.axisExtentBottomText, !0).attr("dy", "0.75em").call(Pe), Kt.text(function(Vt) {
                return Ve(Vt, !1);
              }).each(function(Vt) {
                de.font(te.select(this), Vt.model.rangeFont);
              }), be.ensureAxisBrush(Pt, it, Ze);
            };
          }, 58823: function(G, J, Z) {
            var te = Z(16019), ne = Z(22459), ee = Z(62935).isVisible, ae = {};
            function oe(se, ce, le) {
              var ue = ce.indexOf(le), de = se.indexOf(ue);
              return de === -1 && (de += ce.length), de;
            }
            (G.exports = function(se, ce) {
              var le = se._fullLayout;
              if (ne(se, [], ae)) {
                var ue = {}, de = {}, fe = {}, he = {}, me = le._size;
                ce.forEach(function(pe, ye) {
                  var ge = pe[0].trace;
                  fe[ye] = ge.index;
                  var ve = he[ye] = ge._fullInput.index;
                  ue[ye] = se.data[ve].dimensions, de[ye] = se.data[ve].dimensions.slice();
                }), te(se, ce, { width: me.w, height: me.h, margin: { t: me.t, r: me.r, b: me.b, l: me.l } }, { filterChanged: function(pe, ye, ge) {
                  var ve = de[pe][ye], be = ge.map(function(Le) {
                    return Le.slice();
                  }), Ce = "dimensions[" + ye + "].constraintrange", Se = le._tracePreGUI[se._fullData[fe[pe]]._fullInput.uid];
                  if (Se[Ce] === void 0) {
                    var we = ve.constraintrange;
                    Se[Ce] = we || null;
                  }
                  var Ee = se._fullData[fe[pe]].dimensions[ye];
                  be.length ? (be.length === 1 && (be = be[0]), ve.constraintrange = be, Ee.constraintrange = be.slice(), be = [be]) : (delete ve.constraintrange, delete Ee.constraintrange, be = null);
                  var xe = {};
                  xe[Ce] = be, se.emit("plotly_restyle", [xe, [he[pe]]]);
                }, hover: function(pe) {
                  se.emit("plotly_hover", pe);
                }, unhover: function(pe) {
                  se.emit("plotly_unhover", pe);
                }, axesMoved: function(pe, ye) {
                  var ge = /* @__PURE__ */ function(ve, be) {
                    return function(Ce, Se) {
                      return oe(ve, be, Ce) - oe(ve, be, Se);
                    };
                  }(ye, de[pe].filter(ee));
                  ue[pe].sort(ge), de[pe].filter(function(ve) {
                    return !ee(ve);
                  }).sort(function(ve) {
                    return de[pe].indexOf(ve);
                  }).forEach(function(ve) {
                    ue[pe].splice(ue[pe].indexOf(ve), 1), ue[pe].splice(de[pe].indexOf(ve), 0, ve);
                  }), se.emit("plotly_restyle", [{ dimensions: [ue[pe]] }, [he[pe]]]);
                } });
              }
            }).reglPrecompiled = ae;
          }, 55412: function(G, J, Z) {
            var te = Z(9829), ne = Z(13792).u, ee = Z(80337), ae = Z(10229), oe = Z(3208).rb, se = Z(3208).ay, ce = Z(93049).extendFlat, le = Z(94850).k, ue = ee({ editType: "plot", arrayOk: !0, colorEditType: "plot" });
            G.exports = { labels: { valType: "data_array", editType: "calc" }, label0: { valType: "number", dflt: 0, editType: "calc" }, dlabel: { valType: "number", dflt: 1, editType: "calc" }, values: { valType: "data_array", editType: "calc" }, marker: { colors: { valType: "data_array", editType: "calc" }, line: { color: { valType: "color", dflt: ae.defaultLine, arrayOk: !0, editType: "style" }, width: { valType: "number", min: 0, dflt: 0, arrayOk: !0, editType: "style" }, editType: "calc" }, pattern: le, editType: "calc" }, text: { valType: "data_array", editType: "plot" }, hovertext: { valType: "string", dflt: "", arrayOk: !0, editType: "style" }, scalegroup: { valType: "string", dflt: "", editType: "calc" }, textinfo: { valType: "flaglist", flags: ["label", "text", "value", "percent"], extras: ["none"], editType: "calc" }, hoverinfo: ce({}, te.hoverinfo, { flags: ["label", "text", "value", "percent", "name"] }), hovertemplate: oe({}, { keys: ["label", "color", "value", "percent", "text"] }), texttemplate: se({ editType: "plot" }, { keys: ["label", "color", "value", "percent", "text"] }), textposition: { valType: "enumerated", values: ["inside", "outside", "auto", "none"], dflt: "auto", arrayOk: !0, editType: "plot" }, textfont: ce({}, ue, {}), insidetextorientation: { valType: "enumerated", values: ["horizontal", "radial", "tangential", "auto"], dflt: "auto", editType: "plot" }, insidetextfont: ce({}, ue, {}), outsidetextfont: ce({}, ue, {}), automargin: { valType: "boolean", dflt: !1, editType: "plot" }, title: { text: { valType: "string", dflt: "", editType: "plot" }, font: ce({}, ue, {}), position: { valType: "enumerated", values: ["top left", "top center", "top right", "middle center", "bottom left", "bottom center", "bottom right"], editType: "plot" }, editType: "plot" }, domain: ne({ name: "pie", trace: !0, editType: "calc" }), hole: { valType: "number", min: 0, max: 1, dflt: 0, editType: "calc" }, sort: { valType: "boolean", dflt: !0, editType: "calc" }, direction: { valType: "enumerated", values: ["clockwise", "counterclockwise"], dflt: "counterclockwise", editType: "calc" }, rotation: { valType: "angle", dflt: 0, editType: "calc" }, pull: { valType: "number", min: 0, max: 1, dflt: 0, arrayOk: !0, editType: "calc" }, _deprecated: { title: { valType: "string", dflt: "", editType: "calc" }, titlefont: ce({}, ue, {}), titleposition: { valType: "enumerated", values: ["top left", "top center", "top right", "middle center", "bottom left", "bottom center", "bottom right"], editType: "calc" } } };
          }, 96052: function(G, J, Z) {
            var te = Z(44122);
            J.name = "pie", J.plot = function(ne, ee, ae, oe) {
              te.plotBasePlot(J.name, ne, ee, ae, oe);
            }, J.clean = function(ne, ee, ae, oe) {
              te.cleanBasePlot(J.name, ne, ee, ae, oe);
            };
          }, 44148: function(G, J, Z) {
            var te = Z(10721), ne = Z(65657), ee = Z(78766), ae = {};
            function oe(ce) {
              return function(le, ue) {
                return !!le && !!(le = ne(le)).isValid() && (le = ee.addOpacity(le, le.getAlpha()), ce[ue] || (ce[ue] = le), le);
              };
            }
            function se(ce, le) {
              var ue, de = JSON.stringify(ce), fe = le[de];
              if (!fe) {
                for (fe = ce.slice(), ue = 0; ue < ce.length; ue++) fe.push(ne(ce[ue]).lighten(20).toHexString());
                for (ue = 0; ue < ce.length; ue++) fe.push(ne(ce[ue]).darken(20).toHexString());
                le[de] = fe;
              }
              return fe;
            }
            G.exports = { calc: function(ce, le) {
              var ue, de, fe = [], he = ce._fullLayout, me = he.hiddenlabels || [], pe = le.labels, ye = le.marker.colors || [], ge = le.values, ve = le._length, be = le._hasValues && ve;
              if (le.dlabel) for (pe = new Array(ve), ue = 0; ue < ve; ue++) pe[ue] = String(le.label0 + ue * le.dlabel);
              var Ce = {}, Se = oe(he["_" + le.type + "colormap"]), we = 0, Ee = !1;
              for (ue = 0; ue < ve; ue++) {
                var xe, Le, Qe;
                if (be) {
                  if (xe = ge[ue], !te(xe)) continue;
                  xe = +xe;
                } else xe = 1;
                (Le = pe[ue]) !== void 0 && Le !== "" || (Le = ue);
                var ke = Ce[Le = String(Le)];
                ke === void 0 ? (Ce[Le] = fe.length, (Qe = me.indexOf(Le) !== -1) || (we += xe), fe.push({ v: xe, label: Le, color: Se(ye[ue], Le), i: ue, pts: [ue], hidden: Qe })) : (Ee = !0, (de = fe[ke]).v += xe, de.pts.push(ue), de.hidden || (we += xe), de.color === !1 && ye[ue] && (de.color = Se(ye[ue], Le)));
              }
              return fe = fe.filter(function(Me) {
                return Me.v >= 0;
              }), (le.type === "funnelarea" ? Ee : le.sort) && fe.sort(function(Me, Re) {
                return Re.v - Me.v;
              }), fe[0] && (fe[0].vTotal = we), fe;
            }, crossTraceCalc: function(ce, le) {
              var ue = (le || {}).type;
              ue || (ue = "pie");
              var de = ce._fullLayout, fe = ce.calcdata, he = de[ue + "colorway"], me = de["_" + ue + "colormap"];
              de["extend" + ue + "colors"] && (he = se(he, ae));
              for (var pe = 0, ye = 0; ye < fe.length; ye++) {
                var ge = fe[ye];
                if (ge[0].trace.type === ue) for (var ve = 0; ve < ge.length; ve++) {
                  var be = ge[ve];
                  be.color === !1 && (me[be.label] ? be.color = me[be.label] : (me[be.label] = be.color = he[pe % he.length], pe++));
                }
              }
            }, makePullColorFn: oe, generateExtendedColors: se };
          }, 46979: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(55412), ae = Z(13792).N, oe = Z(17550).handleText, se = Z(34809).coercePattern;
            function ce(ue, de) {
              var fe = ne.isArrayOrTypedArray(ue), he = ne.isArrayOrTypedArray(de), me = Math.min(fe ? ue.length : 1 / 0, he ? de.length : 1 / 0);
              if (isFinite(me) || (me = 0), me && he) {
                for (var pe, ye = 0; ye < me; ye++) {
                  var ge = de[ye];
                  if (te(ge) && ge > 0) {
                    pe = !0;
                    break;
                  }
                }
                pe || (me = 0);
              }
              return { hasLabels: fe, hasValues: he, len: me };
            }
            function le(ue, de, fe, he, me) {
              he("marker.line.width") && he("marker.line.color", me ? void 0 : fe.paper_bgcolor);
              var pe = he("marker.colors");
              se(he, "marker.pattern", pe), ue.marker && !de.marker.pattern.fgcolor && (de.marker.pattern.fgcolor = ue.marker.colors), de.marker.pattern.bgcolor || (de.marker.pattern.bgcolor = fe.paper_bgcolor);
            }
            G.exports = { handleLabelsAndValues: ce, handleMarkerDefaults: le, supplyDefaults: function(ue, de, fe, he) {
              function me(Ee, xe) {
                return ne.coerce(ue, de, ee, Ee, xe);
              }
              var pe = ce(me("labels"), me("values")), ye = pe.len;
              if (de._hasLabels = pe.hasLabels, de._hasValues = pe.hasValues, !de._hasLabels && de._hasValues && (me("label0"), me("dlabel")), ye) {
                de._length = ye, le(ue, de, he, me, !0), me("scalegroup");
                var ge, ve = me("text"), be = me("texttemplate");
                if (be || (ge = me("textinfo", ne.isArrayOrTypedArray(ve) ? "text+percent" : "percent")), me("hovertext"), me("hovertemplate"), be || ge && ge !== "none") {
                  var Ce = me("textposition");
                  oe(ue, de, he, me, Ce, { moduleHasSelected: !1, moduleHasUnselected: !1, moduleHasConstrain: !1, moduleHasCliponaxis: !1, moduleHasTextangle: !1, moduleHasInsideanchor: !1 }), (Array.isArray(Ce) || Ce === "auto" || Ce === "outside") && me("automargin"), (Ce === "inside" || Ce === "auto" || Array.isArray(Ce)) && me("insidetextorientation");
                } else ge === "none" && me("textposition", "none");
                ae(de, he, me);
                var Se = me("hole");
                if (me("title.text")) {
                  var we = me("title.position", Se ? "middle center" : "top center");
                  Se || we !== "middle center" || (de.title.position = "top center"), ne.coerceFont(me, "title.font", he.font);
                }
                me("sort"), me("direction"), me("rotation"), me("pull");
              } else de.visible = !1;
            } };
          }, 50568: function(G, J, Z) {
            var te = Z(36040).appendArrayMultiPointValues;
            G.exports = function(ne, ee) {
              var ae = { curveNumber: ee.index, pointNumbers: ne.pts, data: ee._input, fullData: ee, label: ne.label, color: ne.color, value: ne.v, percent: ne.percent, text: ne.text, bbox: ne.bbox, v: ne.v };
              return ne.pts.length === 1 && (ae.pointNumber = ae.i = ne.pts[0]), te(ae, ee, ne.pts), ee.type === "funnelarea" && (delete ae.v, delete ae.i), ae;
            };
          }, 75067: function(G, J, Z) {
            var te = Z(62203), ne = Z(78766);
            G.exports = function(ee, ae, oe, se) {
              var ce = oe.marker.pattern;
              ce && ce.shape ? te.pointStyle(ee, oe, se, ae) : ne.fill(ee, ae.color);
            };
          }, 37252: function(G, J, Z) {
            var te = Z(34809);
            function ne(ee) {
              return ee.indexOf("e") !== -1 ? ee.replace(/[.]?0+e/, "e") : ee.indexOf(".") !== -1 ? ee.replace(/[.]?0+$/, "") : ee;
            }
            J.formatPiePercent = function(ee, ae) {
              var oe = ne((100 * ee).toPrecision(3));
              return te.numSeparate(oe, ae) + "%";
            }, J.formatPieValue = function(ee, ae) {
              var oe = ne(ee.toPrecision(10));
              return te.numSeparate(oe, ae);
            }, J.getFirstFilled = function(ee, ae) {
              if (te.isArrayOrTypedArray(ee)) for (var oe = 0; oe < ae.length; oe++) {
                var se = ee[ae[oe]];
                if (se || se === 0 || se === "") return se;
              }
            }, J.castOption = function(ee, ae) {
              return te.isArrayOrTypedArray(ee) ? J.getFirstFilled(ee, ae) : ee || void 0;
            }, J.getRotationAngle = function(ee) {
              return (ee === "auto" ? 0 : ee) * Math.PI / 180;
            };
          }, 49913: function(G, J, Z) {
            G.exports = { attributes: Z(55412), supplyDefaults: Z(46979).supplyDefaults, supplyLayoutDefaults: Z(13464), layoutAttributes: Z(4031), calc: Z(44148).calc, crossTraceCalc: Z(44148).crossTraceCalc, plot: Z(35734).plot, style: Z(140), styleOne: Z(32891), moduleType: "trace", name: "pie", basePlotModule: Z(96052), categories: ["pie-like", "pie", "showLegend"], meta: {} };
          }, 4031: function(G) {
            G.exports = { hiddenlabels: { valType: "data_array", editType: "calc" }, piecolorway: { valType: "colorlist", editType: "calc" }, extendpiecolors: { valType: "boolean", dflt: !0, editType: "calc" } };
          }, 13464: function(G, J, Z) {
            var te = Z(34809), ne = Z(4031);
            G.exports = function(ee, ae) {
              function oe(se, ce) {
                return te.coerce(ee, ae, ne, se, ce);
              }
              oe("hiddenlabels"), oe("piecolorway", ae.colorway), oe("extendpiecolors");
            };
          }, 35734: function(G, J, Z) {
            var te = Z(45568), ne = Z(44122), ee = Z(32141), ae = Z(78766), oe = Z(62203), se = Z(34809), ce = se.strScale, le = se.strTranslate, ue = Z(30635), de = Z(84102), fe = de.recordMinTextSize, he = de.clearMinTextSize, me = Z(56155).TEXTPAD, pe = Z(37252), ye = Z(50568), ge = Z(34809).isValidTextValue;
            function ve(Ze, Ge, ze) {
              var We = ze[0], at = We.cx, ct = We.cy, et = We.trace, dt = et.type === "funnelarea";
              "_hasHoverLabel" in et || (et._hasHoverLabel = !1), "_hasHoverEvent" in et || (et._hasHoverEvent = !1), Ze.on("mouseover", function(Je) {
                var it = Ge._fullLayout, st = Ge._fullData[et.index];
                if (!Ge._dragging && it.hovermode !== !1) {
                  var gt = st.hoverinfo;
                  if (Array.isArray(gt) && (gt = ee.castHoverinfo({ hoverinfo: [pe.castOption(gt, Je.pts)], _module: et._module }, it, 0)), gt === "all" && (gt = "label+text+value+percent+name"), st.hovertemplate || gt !== "none" && gt !== "skip" && gt) {
                    var bt = Je.rInscribed || 0, yt = at + Je.pxmid[0] * (1 - bt), St = ct + Je.pxmid[1] * (1 - bt), Lt = it.separators, Rt = [];
                    if (gt && gt.indexOf("label") !== -1 && Rt.push(Je.label), Je.text = pe.castOption(st.hovertext || st.text, Je.pts), gt && gt.indexOf("text") !== -1) {
                      var vt = Je.text;
                      se.isValidTextValue(vt) && Rt.push(vt);
                    }
                    Je.value = Je.v, Je.valueLabel = pe.formatPieValue(Je.v, Lt), gt && gt.indexOf("value") !== -1 && Rt.push(Je.valueLabel), Je.percent = Je.v / We.vTotal, Je.percentLabel = pe.formatPiePercent(Je.percent, Lt), gt && gt.indexOf("percent") !== -1 && Rt.push(Je.percentLabel);
                    var $t = st.hoverlabel, Pt = $t.font, Xt = [];
                    ee.loneHover({ trace: et, x0: yt - bt * We.r, x1: yt + bt * We.r, y: St, _x0: dt ? at + Je.TL[0] : yt - bt * We.r, _x1: dt ? at + Je.TR[0] : yt + bt * We.r, _y0: dt ? ct + Je.TL[1] : St - bt * We.r, _y1: dt ? ct + Je.BL[1] : St + bt * We.r, text: Rt.join("<br>"), name: st.hovertemplate || gt.indexOf("name") !== -1 ? st.name : void 0, idealAlign: Je.pxmid[0] < 0 ? "left" : "right", color: pe.castOption($t.bgcolor, Je.pts) || Je.color, borderColor: pe.castOption($t.bordercolor, Je.pts), fontFamily: pe.castOption(Pt.family, Je.pts), fontSize: pe.castOption(Pt.size, Je.pts), fontColor: pe.castOption(Pt.color, Je.pts), nameLength: pe.castOption($t.namelength, Je.pts), textAlign: pe.castOption($t.align, Je.pts), hovertemplate: pe.castOption(st.hovertemplate, Je.pts), hovertemplateLabels: Je, eventData: [ye(Je, st)] }, { container: it._hoverlayer.node(), outerContainer: it._paper.node(), gd: Ge, inOut_bbox: Xt }), Je.bbox = Xt[0], et._hasHoverLabel = !0;
                  }
                  et._hasHoverEvent = !0, Ge.emit("plotly_hover", { points: [ye(Je, st)], event: te.event });
                }
              }), Ze.on("mouseout", function(Je) {
                var it = Ge._fullLayout, st = Ge._fullData[et.index], gt = te.select(this).datum();
                et._hasHoverEvent && (Je.originalEvent = te.event, Ge.emit("plotly_unhover", { points: [ye(gt, st)], event: te.event }), et._hasHoverEvent = !1), et._hasHoverLabel && (ee.loneUnhover(it._hoverlayer.node()), et._hasHoverLabel = !1);
              }), Ze.on("click", function(Je) {
                var it = Ge._fullLayout, st = Ge._fullData[et.index];
                Ge._dragging || it.hovermode === !1 || (Ge._hoverdata = [ye(Je, st)], ee.click(Ge, te.event));
              });
            }
            function be(Ze, Ge, ze) {
              var We = pe.castOption(Ze.insidetextfont.color, Ge.pts);
              !We && Ze._input.textfont && (We = pe.castOption(Ze._input.textfont.color, Ge.pts));
              var at = pe.castOption(Ze.insidetextfont.family, Ge.pts) || pe.castOption(Ze.textfont.family, Ge.pts) || ze.family, ct = pe.castOption(Ze.insidetextfont.size, Ge.pts) || pe.castOption(Ze.textfont.size, Ge.pts) || ze.size, et = pe.castOption(Ze.insidetextfont.weight, Ge.pts) || pe.castOption(Ze.textfont.weight, Ge.pts) || ze.weight, dt = pe.castOption(Ze.insidetextfont.style, Ge.pts) || pe.castOption(Ze.textfont.style, Ge.pts) || ze.style, Je = pe.castOption(Ze.insidetextfont.variant, Ge.pts) || pe.castOption(Ze.textfont.variant, Ge.pts) || ze.variant, it = pe.castOption(Ze.insidetextfont.textcase, Ge.pts) || pe.castOption(Ze.textfont.textcase, Ge.pts) || ze.textcase, st = pe.castOption(Ze.insidetextfont.lineposition, Ge.pts) || pe.castOption(Ze.textfont.lineposition, Ge.pts) || ze.lineposition, gt = pe.castOption(Ze.insidetextfont.shadow, Ge.pts) || pe.castOption(Ze.textfont.shadow, Ge.pts) || ze.shadow;
              return { color: We || ae.contrast(Ge.color), family: at, size: ct, weight: et, style: dt, variant: Je, textcase: it, lineposition: st, shadow: gt };
            }
            function Ce(Ze, Ge) {
              for (var ze, We, at = 0; at < Ze.length; at++) if ((We = (ze = Ze[at][0]).trace).title.text) {
                var ct = We.title.text;
                We._meta && (ct = se.templateString(ct, We._meta));
                var et = oe.tester.append("text").attr("data-notex", 1).text(ct).call(oe.font, We.title.font).call(ue.convertToTspans, Ge), dt = oe.bBox(et.node(), !0);
                ze.titleBox = { width: dt.width, height: dt.height }, et.remove();
              }
            }
            function Se(Ze, Ge, ze) {
              var We = ze.r || Ge.rpx1, at = Ge.rInscribed;
              if (Ge.startangle === Ge.stopangle) return { rCenter: 1 - at, scale: 0, rotate: 0, textPosAngle: 0 };
              var ct, et = Ge.ring, dt = et === 1 && Math.abs(Ge.startangle - Ge.stopangle) === 2 * Math.PI, Je = Ge.halfangle, it = Ge.midangle, st = ze.trace.insidetextorientation, gt = st === "horizontal", bt = st === "tangential", yt = st === "radial", St = st === "auto", Lt = [];
              if (!St) {
                var Rt, vt = function(Ft, Tt) {
                  if (function(Vt, Yt) {
                    var sn = Vt.startangle, hn = Vt.stopangle;
                    return sn > Yt && Yt > hn || sn < Yt && Yt < hn;
                  }(Ge, Ft)) {
                    var It = Math.abs(Ft - Ge.startangle), Ct = Math.abs(Ft - Ge.stopangle), Kt = It < Ct ? It : Ct;
                    (ct = Tt === "tan" ? Ee(Ze, We, et, Kt, 0) : we(Ze, We, et, Kt, Math.PI / 2)).textPosAngle = Ft, Lt.push(ct);
                  }
                };
                if (gt || bt) {
                  for (Rt = 4; Rt >= -4; Rt -= 2) vt(Math.PI * Rt, "tan");
                  for (Rt = 4; Rt >= -4; Rt -= 2) vt(Math.PI * (Rt + 1), "tan");
                }
                if (gt || yt) {
                  for (Rt = 4; Rt >= -4; Rt -= 2) vt(Math.PI * (Rt + 1.5), "rad");
                  for (Rt = 4; Rt >= -4; Rt -= 2) vt(Math.PI * (Rt + 0.5), "rad");
                }
              }
              if (dt || St || gt) {
                var $t = Math.sqrt(Ze.width * Ze.width + Ze.height * Ze.height);
                if ((ct = { scale: at * We * 2 / $t, rCenter: 1 - at, rotate: 0 }).textPosAngle = (Ge.startangle + Ge.stopangle) / 2, ct.scale >= 1) return ct;
                Lt.push(ct);
              }
              (St || yt) && ((ct = we(Ze, We, et, Je, it)).textPosAngle = (Ge.startangle + Ge.stopangle) / 2, Lt.push(ct)), (St || bt) && ((ct = Ee(Ze, We, et, Je, it)).textPosAngle = (Ge.startangle + Ge.stopangle) / 2, Lt.push(ct));
              for (var Pt = 0, Xt = 0, Nt = 0; Nt < Lt.length; Nt++) {
                var Gt = Lt[Nt].scale;
                if (Xt < Gt && (Xt = Gt, Pt = Nt), !St && Xt >= 1) break;
              }
              return Lt[Pt];
            }
            function we(Ze, Ge, ze, We, at) {
              Ge = Math.max(0, Ge - 2 * me);
              var ct = Ze.width / Ze.height, et = Qe(ct, We, Ge, ze);
              return { scale: 2 * et / Ze.height, rCenter: xe(ct, et / Ge), rotate: Le(at) };
            }
            function Ee(Ze, Ge, ze, We, at) {
              Ge = Math.max(0, Ge - 2 * me);
              var ct = Ze.height / Ze.width, et = Qe(ct, We, Ge, ze);
              return { scale: 2 * et / Ze.width, rCenter: xe(ct, et / Ge), rotate: Le(at + Math.PI / 2) };
            }
            function xe(Ze, Ge) {
              return Math.cos(Ge) - Ze * Ge;
            }
            function Le(Ze) {
              return (180 / Math.PI * Ze + 720) % 180 - 90;
            }
            function Qe(Ze, Ge, ze, We) {
              var at = Ze + 1 / (2 * Math.tan(Ge));
              return ze * Math.min(1 / (Math.sqrt(at * at + 0.5) + at), We / (Math.sqrt(Ze * Ze + We / 2) + Ze));
            }
            function ke(Ze, Ge) {
              return Ze.v !== Ge.vTotal || Ge.trace.hole ? Math.min(1 / (1 + 1 / Math.sin(Ze.halfangle)), Ze.ring / 2) : 1;
            }
            function Me(Ze, Ge) {
              var ze = Ge.pxmid[0], We = Ge.pxmid[1], at = Ze.width / 2, ct = Ze.height / 2;
              return ze < 0 && (at *= -1), We < 0 && (ct *= -1), { scale: 1, rCenter: 1, rotate: 0, x: at + Math.abs(ct) * (at > 0 ? 1 : -1) / 2, y: ct / (1 + ze * ze / (We * We)), outside: !0 };
            }
            function Re(Ze, Ge) {
              var ze, We, at, ct = Ze.trace, et = { x: Ze.cx, y: Ze.cy }, dt = { tx: 0, ty: 0 };
              dt.ty += ct.title.font.size, at = Pe(ct), ct.title.position.indexOf("top") !== -1 ? (et.y -= (1 + at) * Ze.r, dt.ty -= Ze.titleBox.height) : ct.title.position.indexOf("bottom") !== -1 && (et.y += (1 + at) * Ze.r);
              var Je, it = Ze.r / ((Je = Ze.trace.aspectratio) === void 0 ? 1 : Je), st = Ge.w * (ct.domain.x[1] - ct.domain.x[0]) / 2;
              return ct.title.position.indexOf("left") !== -1 ? (st += it, et.x -= (1 + at) * it, dt.tx += Ze.titleBox.width / 2) : ct.title.position.indexOf("center") !== -1 ? st *= 2 : ct.title.position.indexOf("right") !== -1 && (st += it, et.x += (1 + at) * it, dt.tx -= Ze.titleBox.width / 2), ze = st / Ze.titleBox.width, We = De(Ze, Ge) / Ze.titleBox.height, { x: et.x, y: et.y, scale: Math.min(ze, We), tx: dt.tx, ty: dt.ty };
            }
            function De(Ze, Ge) {
              var ze = Ze.trace, We = Ge.h * (ze.domain.y[1] - ze.domain.y[0]);
              return Math.min(Ze.titleBox.height, We / 2);
            }
            function Pe(Ze) {
              var Ge, ze = Ze.pull;
              if (!ze) return 0;
              if (se.isArrayOrTypedArray(ze)) for (ze = 0, Ge = 0; Ge < Ze.pull.length; Ge++) Ze.pull[Ge] > ze && (ze = Ze.pull[Ge]);
              return ze;
            }
            function $e(Ze, Ge) {
              for (var ze = [], We = 0; We < Ze.length; We++) {
                var at = Ze[We][0], ct = at.trace, et = ct.domain, dt = Ge.w * (et.x[1] - et.x[0]), Je = Ge.h * (et.y[1] - et.y[0]);
                ct.title.text && ct.title.position !== "middle center" && (Je -= De(at, Ge));
                var it = dt / 2, st = Je / 2;
                ct.type !== "funnelarea" || ct.scalegroup || (st /= ct.aspectratio), at.r = Math.min(it, st) / (1 + Pe(ct)), at.cx = Ge.l + Ge.w * (ct.domain.x[1] + ct.domain.x[0]) / 2, at.cy = Ge.t + Ge.h * (1 - ct.domain.y[0]) - Je / 2, ct.title.text && ct.title.position.indexOf("bottom") !== -1 && (at.cy -= De(at, Ge)), ct.scalegroup && ze.indexOf(ct.scalegroup) === -1 && ze.push(ct.scalegroup);
              }
              (function(gt, bt) {
                for (var yt, St, Lt, Rt = 0; Rt < bt.length; Rt++) {
                  var vt = 1 / 0, $t = bt[Rt];
                  for (St = 0; St < gt.length; St++) if ((Lt = (yt = gt[St][0]).trace).scalegroup === $t) {
                    var Pt;
                    if (Lt.type === "pie") Pt = yt.r * yt.r;
                    else if (Lt.type === "funnelarea") {
                      var Xt, Nt;
                      Lt.aspectratio > 1 ? Nt = (Xt = yt.r) / Lt.aspectratio : Xt = (Nt = yt.r) * Lt.aspectratio, Pt = (Xt *= (1 + Lt.baseratio) / 2) * Nt;
                    }
                    vt = Math.min(vt, Pt / yt.vTotal);
                  }
                  for (St = 0; St < gt.length; St++) if ((Lt = (yt = gt[St][0]).trace).scalegroup === $t) {
                    var Gt = vt * yt.vTotal;
                    Lt.type === "funnelarea" && (Gt /= (1 + Lt.baseratio) / 2, Gt /= Lt.aspectratio), yt.r = Math.sqrt(Gt);
                  }
                }
              })(Ze, ze);
            }
            function qe(Ze, Ge) {
              return [Ze * Math.sin(Ge), -Ze * Math.cos(Ge)];
            }
            function Xe(Ze, Ge, ze) {
              var We = Ze._fullLayout, at = ze.trace, ct = at.texttemplate, et = at.textinfo;
              if (!ct && et && et !== "none") {
                var dt, Je = et.split("+"), it = function(Pt) {
                  return Je.indexOf(Pt) !== -1;
                }, st = it("label"), gt = it("text"), bt = it("value"), yt = it("percent"), St = We.separators;
                if (dt = st ? [Ge.label] : [], gt) {
                  var Lt = pe.getFirstFilled(at.text, Ge.pts);
                  ge(Lt) && dt.push(Lt);
                }
                bt && dt.push(pe.formatPieValue(Ge.v, St)), yt && dt.push(pe.formatPiePercent(Ge.v / ze.vTotal, St)), Ge.text = dt.join("<br>");
              }
              if (ct) {
                var Rt = se.castOption(at, Ge.i, "texttemplate");
                if (Rt) {
                  var vt = function(Pt) {
                    return { label: Pt.label, value: Pt.v, valueLabel: pe.formatPieValue(Pt.v, We.separators), percent: Pt.v / ze.vTotal, percentLabel: pe.formatPiePercent(Pt.v / ze.vTotal, We.separators), color: Pt.color, text: Pt.text, customdata: se.castOption(at, Pt.i, "customdata") };
                  }(Ge), $t = pe.getFirstFilled(at.text, Ge.pts);
                  (ge($t) || $t === "") && (vt.text = $t), Ge.text = se.texttemplateString(Rt, vt, Ze._fullLayout._d3locale, vt, at._meta || {});
                } else Ge.text = "";
              }
            }
            function Ve(Ze, Ge) {
              var ze = Ze.rotate * Math.PI / 180, We = Math.cos(ze), at = Math.sin(ze), ct = (Ge.left + Ge.right) / 2, et = (Ge.top + Ge.bottom) / 2;
              Ze.textX = ct * We - et * at, Ze.textY = ct * at + et * We, Ze.noCenter = !0;
            }
            G.exports = { plot: function(Ze, Ge) {
              var ze = Ze._context.staticPlot, We = Ze._fullLayout, at = We._size;
              he("pie", We), Ce(Ge, Ze), $e(Ge, at);
              var ct = se.makeTraceGroups(We._pielayer, Ge, "trace").each(function(et) {
                var dt = te.select(this), Je = et[0], it = Je.trace;
                (function(st) {
                  var gt, bt, yt, St = st[0], Lt = St.r, Rt = St.trace, vt = pe.getRotationAngle(Rt.rotation), $t = 2 * Math.PI / St.vTotal, Pt = "px0", Xt = "px1";
                  if (Rt.direction === "counterclockwise") {
                    for (gt = 0; gt < st.length && st[gt].hidden; gt++) ;
                    if (gt === st.length) return;
                    vt += $t * st[gt].v, $t *= -1, Pt = "px1", Xt = "px0";
                  }
                  for (yt = qe(Lt, vt), gt = 0; gt < st.length; gt++) (bt = st[gt]).hidden || (bt[Pt] = yt, bt.startangle = vt, vt += $t * bt.v / 2, bt.pxmid = qe(Lt, vt), bt.midangle = vt, yt = qe(Lt, vt += $t * bt.v / 2), bt.stopangle = vt, bt[Xt] = yt, bt.largeArc = bt.v > St.vTotal / 2 ? 1 : 0, bt.halfangle = Math.PI * Math.min(bt.v / St.vTotal, 0.5), bt.ring = 1 - Rt.hole, bt.rInscribed = ke(bt, St));
                })(et), dt.attr("stroke-linejoin", "round"), dt.each(function() {
                  var st = te.select(this).selectAll("g.slice").data(et);
                  st.enter().append("g").classed("slice", !0), st.exit().remove();
                  var gt = [[[], []], [[], []]], bt = !1;
                  st.each(function(Xt, Nt) {
                    if (Xt.hidden) te.select(this).selectAll("path,g").remove();
                    else {
                      Xt.pointNumber = Xt.i, Xt.curveNumber = it.index, gt[Xt.pxmid[1] < 0 ? 0 : 1][Xt.pxmid[0] < 0 ? 0 : 1].push(Xt);
                      var Gt = Je.cx, Ft = Je.cy, Tt = te.select(this), It = Tt.selectAll("path.surface").data([Xt]);
                      if (It.enter().append("path").classed("surface", !0).style({ "pointer-events": ze ? "none" : "all" }), Tt.call(ve, Ze, et), it.pull) {
                        var Ct = +pe.castOption(it.pull, Xt.pts) || 0;
                        Ct > 0 && (Gt += Ct * Xt.pxmid[0], Ft += Ct * Xt.pxmid[1]);
                      }
                      Xt.cxFinal = Gt, Xt.cyFinal = Ft;
                      var Kt = it.hole;
                      if (Xt.v === Je.vTotal) {
                        var Vt = "M" + (Gt + Xt.px0[0]) + "," + (Ft + Xt.px0[1]) + Ut(Xt.px0, Xt.pxmid, !0, 1) + Ut(Xt.pxmid, Xt.px0, !0, 1) + "Z";
                        Kt ? It.attr("d", "M" + (Gt + Kt * Xt.px0[0]) + "," + (Ft + Kt * Xt.px0[1]) + Ut(Xt.px0, Xt.pxmid, !1, Kt) + Ut(Xt.pxmid, Xt.px0, !1, Kt) + "Z" + Vt) : It.attr("d", Vt);
                      } else {
                        var Yt = Ut(Xt.px0, Xt.px1, !0, 1);
                        if (Kt) {
                          var sn = 1 - Kt;
                          It.attr("d", "M" + (Gt + Kt * Xt.px1[0]) + "," + (Ft + Kt * Xt.px1[1]) + Ut(Xt.px1, Xt.px0, !1, Kt) + "l" + sn * Xt.px0[0] + "," + sn * Xt.px0[1] + Yt + "Z");
                        } else It.attr("d", "M" + Gt + "," + Ft + "l" + Xt.px0[0] + "," + Xt.px0[1] + Yt + "Z");
                      }
                      Xe(Ze, Xt, Je);
                      var hn = pe.castOption(it.textposition, Xt.pts), Qt = Tt.selectAll("g.slicetext").data(Xt.text && hn !== "none" ? [0] : []);
                      Qt.enter().append("g").classed("slicetext", !0), Qt.exit().remove(), Qt.each(function() {
                        var Wt = se.ensureSingle(te.select(this), "text", "", function(Rn) {
                          Rn.attr("data-notex", 1);
                        }), rn = se.ensureUniformFontSize(Ze, hn === "outside" ? function(Rn, Pn, Ie) {
                          return { color: pe.castOption(Rn.outsidetextfont.color, Pn.pts) || pe.castOption(Rn.textfont.color, Pn.pts) || Ie.color, family: pe.castOption(Rn.outsidetextfont.family, Pn.pts) || pe.castOption(Rn.textfont.family, Pn.pts) || Ie.family, size: pe.castOption(Rn.outsidetextfont.size, Pn.pts) || pe.castOption(Rn.textfont.size, Pn.pts) || Ie.size, weight: pe.castOption(Rn.outsidetextfont.weight, Pn.pts) || pe.castOption(Rn.textfont.weight, Pn.pts) || Ie.weight, style: pe.castOption(Rn.outsidetextfont.style, Pn.pts) || pe.castOption(Rn.textfont.style, Pn.pts) || Ie.style, variant: pe.castOption(Rn.outsidetextfont.variant, Pn.pts) || pe.castOption(Rn.textfont.variant, Pn.pts) || Ie.variant, textcase: pe.castOption(Rn.outsidetextfont.textcase, Pn.pts) || pe.castOption(Rn.textfont.textcase, Pn.pts) || Ie.textcase, lineposition: pe.castOption(Rn.outsidetextfont.lineposition, Pn.pts) || pe.castOption(Rn.textfont.lineposition, Pn.pts) || Ie.lineposition, shadow: pe.castOption(Rn.outsidetextfont.shadow, Pn.pts) || pe.castOption(Rn.textfont.shadow, Pn.pts) || Ie.shadow };
                        }(it, Xt, We.font) : be(it, Xt, We.font));
                        Wt.text(Xt.text).attr({ class: "slicetext", transform: "", "text-anchor": "middle" }).call(oe.font, rn).call(ue.convertToTspans, Ze);
                        var vn, dn = oe.bBox(Wt.node());
                        if (hn === "outside") vn = Me(dn, Xt);
                        else if (vn = Se(dn, Xt, Je), hn === "auto" && vn.scale < 1) {
                          var Ht = se.ensureUniformFontSize(Ze, it.outsidetextfont);
                          Wt.call(oe.font, Ht), vn = Me(dn = oe.bBox(Wt.node()), Xt);
                        }
                        var jt = vn.textPosAngle, cn = jt === void 0 ? Xt.pxmid : qe(Je.r, jt);
                        if (vn.targetX = Gt + cn[0] * vn.rCenter + (vn.x || 0), vn.targetY = Ft + cn[1] * vn.rCenter + (vn.y || 0), Ve(vn, dn), vn.outside) {
                          var wn = vn.targetY;
                          Xt.yLabelMin = wn - dn.height / 2, Xt.yLabelMid = wn, Xt.yLabelMax = wn + dn.height / 2, Xt.labelExtraX = 0, Xt.labelExtraY = 0, bt = !0;
                        }
                        vn.fontSize = rn.size, fe(it.type, vn, We), et[Nt].transform = vn, se.setTransormAndDisplay(Wt, vn);
                      });
                    }
                    function Ut(Wt, rn, vn, dn) {
                      var Ht = dn * (rn[0] - Wt[0]), jt = dn * (rn[1] - Wt[1]);
                      return "a" + dn * Je.r + "," + dn * Je.r + " 0 " + Xt.largeArc + (vn ? " 1 " : " 0 ") + Ht + "," + jt;
                    }
                  });
                  var yt = te.select(this).selectAll("g.titletext").data(it.title.text ? [0] : []);
                  if (yt.enter().append("g").classed("titletext", !0), yt.exit().remove(), yt.each(function() {
                    var Xt, Nt = se.ensureSingle(te.select(this), "text", "", function(Ft) {
                      Ft.attr("data-notex", 1);
                    }), Gt = it.title.text;
                    it._meta && (Gt = se.templateString(Gt, it._meta)), Nt.text(Gt).attr({ class: "titletext", transform: "", "text-anchor": "middle" }).call(oe.font, it.title.font).call(ue.convertToTspans, Ze), Xt = it.title.position === "middle center" ? function(Ft) {
                      var Tt = Math.sqrt(Ft.titleBox.width * Ft.titleBox.width + Ft.titleBox.height * Ft.titleBox.height);
                      return { x: Ft.cx, y: Ft.cy, scale: Ft.trace.hole * Ft.r * 2 / Tt, tx: 0, ty: -Ft.titleBox.height / 2 + Ft.trace.title.font.size };
                    }(Je) : Re(Je, at), Nt.attr("transform", le(Xt.x, Xt.y) + ce(Math.min(1, Xt.scale)) + le(Xt.tx, Xt.ty));
                  }), bt && function(Xt, Nt) {
                    var Gt, Ft, Tt, It, Ct, Kt, Vt, Yt, sn, hn, Qt, Ut, Wt;
                    function rn(jt, cn) {
                      return jt.pxmid[1] - cn.pxmid[1];
                    }
                    function vn(jt, cn) {
                      return cn.pxmid[1] - jt.pxmid[1];
                    }
                    function dn(jt, cn) {
                      cn || (cn = {});
                      var wn, Rn, Pn, Ie, rt = cn.labelExtraY + (Ft ? cn.yLabelMax : cn.yLabelMin), ft = Ft ? jt.yLabelMin : jt.yLabelMax, pt = Ft ? jt.yLabelMax : jt.yLabelMin, wt = jt.cyFinal + Ct(jt.px0[1], jt.px1[1]), Bt = rt - ft;
                      if (Bt * Vt > 0 && (jt.labelExtraY = Bt), se.isArrayOrTypedArray(Nt.pull)) for (Rn = 0; Rn < hn.length; Rn++) (Pn = hn[Rn]) === jt || (pe.castOption(Nt.pull, jt.pts) || 0) >= (pe.castOption(Nt.pull, Pn.pts) || 0) || ((jt.pxmid[1] - Pn.pxmid[1]) * Vt > 0 ? (Bt = Pn.cyFinal + Ct(Pn.px0[1], Pn.px1[1]) - ft - jt.labelExtraY) * Vt > 0 && (jt.labelExtraY += Bt) : (pt + jt.labelExtraY - wt) * Vt > 0 && (wn = 3 * Kt * Math.abs(Rn - hn.indexOf(jt)), (Ie = Pn.cxFinal + It(Pn.px0[0], Pn.px1[0]) + wn - (jt.cxFinal + jt.pxmid[0]) - jt.labelExtraX) * Kt > 0 && (jt.labelExtraX += Ie)));
                    }
                    for (Ft = 0; Ft < 2; Ft++) for (Tt = Ft ? rn : vn, Ct = Ft ? Math.max : Math.min, Vt = Ft ? 1 : -1, Gt = 0; Gt < 2; Gt++) {
                      for (It = Gt ? Math.max : Math.min, Kt = Gt ? 1 : -1, (Yt = Xt[Ft][Gt]).sort(Tt), sn = Xt[1 - Ft][Gt], hn = sn.concat(Yt), Ut = [], Qt = 0; Qt < Yt.length; Qt++) Yt[Qt].yLabelMid !== void 0 && Ut.push(Yt[Qt]);
                      for (Wt = !1, Qt = 0; Ft && Qt < sn.length; Qt++) if (sn[Qt].yLabelMid !== void 0) {
                        Wt = sn[Qt];
                        break;
                      }
                      for (Qt = 0; Qt < Ut.length; Qt++) {
                        var Ht = Qt && Ut[Qt - 1];
                        Wt && !Qt && (Ht = Wt), dn(Ut[Qt], Ht);
                      }
                    }
                  }(gt, it), function(Xt, Nt) {
                    Xt.each(function(Gt) {
                      var Ft = te.select(this);
                      if (Gt.labelExtraX || Gt.labelExtraY) {
                        var Tt = Ft.select("g.slicetext text");
                        Gt.transform.targetX += Gt.labelExtraX, Gt.transform.targetY += Gt.labelExtraY, se.setTransormAndDisplay(Tt, Gt.transform);
                        var It = Gt.cxFinal + Gt.pxmid[0], Ct = "M" + It + "," + (Gt.cyFinal + Gt.pxmid[1]), Kt = (Gt.yLabelMax - Gt.yLabelMin) * (Gt.pxmid[0] < 0 ? -1 : 1) / 4;
                        if (Gt.labelExtraX) {
                          var Vt = Gt.labelExtraX * Gt.pxmid[1] / Gt.pxmid[0], Yt = Gt.yLabelMid + Gt.labelExtraY - (Gt.cyFinal + Gt.pxmid[1]);
                          Math.abs(Vt) > Math.abs(Yt) ? Ct += "l" + Yt * Gt.pxmid[0] / Gt.pxmid[1] + "," + Yt + "H" + (It + Gt.labelExtraX + Kt) : Ct += "l" + Gt.labelExtraX + "," + Vt + "v" + (Yt - Vt) + "h" + Kt;
                        } else Ct += "V" + (Gt.yLabelMid + Gt.labelExtraY) + "h" + Kt;
                        se.ensureSingle(Ft, "path", "textline").call(ae.stroke, Nt.outsidetextfont.color).attr({ "stroke-width": Math.min(2, Nt.outsidetextfont.size / 8), d: Ct, fill: "none" });
                      } else Ft.select("path.textline").remove();
                    });
                  }(st, it), bt && it.automargin) {
                    var St = oe.bBox(dt.node()), Lt = it.domain, Rt = at.w * (Lt.x[1] - Lt.x[0]), vt = at.h * (Lt.y[1] - Lt.y[0]), $t = (0.5 * Rt - Je.r) / at.w, Pt = (0.5 * vt - Je.r) / at.h;
                    ne.autoMargin(Ze, "pie." + it.uid + ".automargin", { xl: Lt.x[0] - $t, xr: Lt.x[1] + $t, yb: Lt.y[0] - Pt, yt: Lt.y[1] + Pt, l: Math.max(Je.cx - Je.r - St.left, 0), r: Math.max(St.right - (Je.cx + Je.r), 0), b: Math.max(St.bottom - (Je.cy + Je.r), 0), t: Math.max(Je.cy - Je.r - St.top, 0), pad: 5 });
                  }
                });
              });
              setTimeout(function() {
                ct.selectAll("tspan").each(function() {
                  var et = te.select(this);
                  et.attr("dy") && et.attr("dy", et.attr("dy"));
                });
              }, 0);
            }, formatSliceLabel: Xe, transformInsideText: Se, determineInsideTextFont: be, positionTitleOutside: Re, prerenderTitles: Ce, layoutAreas: $e, attachFxHandlers: ve, computeTransform: Ve };
          }, 140: function(G, J, Z) {
            var te = Z(45568), ne = Z(32891), ee = Z(84102).resizeText;
            G.exports = function(ae) {
              var oe = ae._fullLayout._pielayer.selectAll(".trace");
              ee(ae, oe, "pie"), oe.each(function(se) {
                var ce = se[0].trace, le = te.select(this);
                le.style({ opacity: ce.opacity }), le.selectAll("path.surface").each(function(ue) {
                  te.select(this).call(ne, ue, ce, ae);
                });
              });
            };
          }, 32891: function(G, J, Z) {
            var te = Z(78766), ne = Z(37252).castOption, ee = Z(75067);
            G.exports = function(ae, oe, se, ce) {
              var le = se.marker.line, ue = ne(le.color, oe.pts) || te.defaultLine, de = ne(le.width, oe.pts) || 0;
              ae.call(ee, oe, se, ce).style("stroke-width", de).call(te.stroke, ue);
            };
          }, 36961: function(G, J, Z) {
            var te = Z(36640);
            G.exports = { x: te.x, y: te.y, xy: { valType: "data_array", editType: "calc" }, indices: { valType: "data_array", editType: "calc" }, xbounds: { valType: "data_array", editType: "calc" }, ybounds: { valType: "data_array", editType: "calc" }, text: te.text, marker: { color: { valType: "color", arrayOk: !1, editType: "calc" }, opacity: { valType: "number", min: 0, max: 1, dflt: 1, arrayOk: !1, editType: "calc" }, blend: { valType: "boolean", dflt: null, editType: "calc" }, sizemin: { valType: "number", min: 0.1, max: 2, dflt: 0.5, editType: "calc" }, sizemax: { valType: "number", min: 0.1, dflt: 20, editType: "calc" }, border: { color: { valType: "color", arrayOk: !1, editType: "calc" }, arearatio: { valType: "number", min: 0, max: 1, dflt: 0, editType: "calc" }, editType: "calc" }, editType: "calc" }, transforms: void 0 };
          }, 71593: function(G, J, Z) {
            var te = Z(99098).gl_pointcloud2d, ne = Z(34809).isArrayOrTypedArray, ee = Z(55010), ae = Z(32919).findExtremes, oe = Z(11539);
            function se(le, ue) {
              this.scene = le, this.uid = ue, this.type = "pointcloud", this.pickXData = [], this.pickYData = [], this.xData = [], this.yData = [], this.textLabels = [], this.color = "rgb(0, 0, 0)", this.name = "", this.hoverinfo = "all", this.idToIndex = new Int32Array(0), this.bounds = [0, 0, 0, 0], this.pointcloudOptions = { positions: new Float32Array(0), idToIndex: this.idToIndex, sizemin: 0.5, sizemax: 12, color: [0, 0, 0, 1], areaRatio: 1, borderColor: [0, 0, 0, 1] }, this.pointcloud = te(le.glplot, this.pointcloudOptions), this.pointcloud._trace = this;
            }
            var ce = se.prototype;
            ce.handlePick = function(le) {
              var ue = this.idToIndex[le.pointId];
              return { trace: this, dataCoord: le.dataCoord, traceCoord: this.pickXYData ? [this.pickXYData[2 * ue], this.pickXYData[2 * ue + 1]] : [this.pickXData[ue], this.pickYData[ue]], textLabel: ne(this.textLabels) ? this.textLabels[ue] : this.textLabels, color: this.color, name: this.name, pointIndex: ue, hoverinfo: this.hoverinfo };
            }, ce.update = function(le) {
              this.index = le.index, this.textLabels = le.text, this.name = le.name, this.hoverinfo = le.hoverinfo, this.bounds = [1 / 0, 1 / 0, -1 / 0, -1 / 0], this.updateFast(le), this.color = oe(le, {});
            }, ce.updateFast = function(le) {
              var ue, de, fe, he, me, pe, ye = this.xData = this.pickXData = le.x, ge = this.yData = this.pickYData = le.y, ve = this.pickXYData = le.xy, be = le.xbounds && le.ybounds, Ce = le.indices, Se = this.bounds;
              if (ve) {
                if (fe = ve, ue = ve.length >>> 1, be) Se[0] = le.xbounds[0], Se[2] = le.xbounds[1], Se[1] = le.ybounds[0], Se[3] = le.ybounds[1];
                else for (pe = 0; pe < ue; pe++) he = fe[2 * pe], me = fe[2 * pe + 1], he < Se[0] && (Se[0] = he), he > Se[2] && (Se[2] = he), me < Se[1] && (Se[1] = me), me > Se[3] && (Se[3] = me);
                if (Ce) de = Ce;
                else for (de = new Int32Array(ue), pe = 0; pe < ue; pe++) de[pe] = pe;
              } else for (ue = ye.length, fe = new Float32Array(2 * ue), de = new Int32Array(ue), pe = 0; pe < ue; pe++) he = ye[pe], me = ge[pe], de[pe] = pe, fe[2 * pe] = he, fe[2 * pe + 1] = me, he < Se[0] && (Se[0] = he), he > Se[2] && (Se[2] = he), me < Se[1] && (Se[1] = me), me > Se[3] && (Se[3] = me);
              this.idToIndex = de, this.pointcloudOptions.idToIndex = de, this.pointcloudOptions.positions = fe;
              var we = ee(le.marker.color), Ee = ee(le.marker.border.color), xe = le.opacity * le.marker.opacity;
              we[3] *= xe, this.pointcloudOptions.color = we;
              var Le = le.marker.blend;
              Le === null && (Le = ye.length < 100 || ge.length < 100), this.pointcloudOptions.blend = Le, Ee[3] *= xe, this.pointcloudOptions.borderColor = Ee;
              var Qe = le.marker.sizemin, ke = Math.max(le.marker.sizemax, le.marker.sizemin);
              this.pointcloudOptions.sizeMin = Qe, this.pointcloudOptions.sizeMax = ke, this.pointcloudOptions.areaRatio = le.marker.border.arearatio, this.pointcloud.update(this.pointcloudOptions);
              var Me = this.scene.xaxis, Re = this.scene.yaxis, De = ke / 2 || 0.5;
              le._extremes[Me._id] = ae(Me, [Se[0], Se[2]], { ppad: De }), le._extremes[Re._id] = ae(Re, [Se[1], Se[3]], { ppad: De });
            }, ce.dispose = function() {
              this.pointcloud.dispose();
            }, G.exports = function(le, ue) {
              var de = new se(le, ue.uid);
              return de.update(ue), de;
            };
          }, 75526: function(G, J, Z) {
            var te = Z(34809), ne = Z(36961);
            G.exports = function(ee, ae, oe) {
              function se(ce, le) {
                return te.coerce(ee, ae, ne, ce, le);
              }
              se("x"), se("y"), se("xbounds"), se("ybounds"), ee.xy && ee.xy instanceof Float32Array && (ae.xy = ee.xy), ee.indices && ee.indices instanceof Int32Array && (ae.indices = ee.indices), se("text"), se("marker.color", oe), se("marker.opacity"), se("marker.blend"), se("marker.sizemin"), se("marker.sizemax"), se("marker.border.color", oe), se("marker.border.arearatio"), ae._length = null;
            };
          }, 15186: function(G, J, Z) {
            G.exports = { attributes: Z(36961), supplyDefaults: Z(75526), calc: Z(37593), plot: Z(71593), moduleType: "trace", name: "pointcloud", basePlotModule: Z(24585), categories: ["gl", "gl2d", "showLegend"], meta: {} };
          }, 33795: function(G, J, Z) {
            var te = Z(80337), ne = Z(9829), ee = Z(10229), ae = Z(70192), oe = Z(13792).u, se = Z(3208).rb, ce = Z(87163), le = Z(78032).templatedArray, ue = Z(80712).descriptionOnlyNumbers, de = Z(93049).extendFlat, fe = Z(13582).overrideAll;
            (G.exports = fe({ hoverinfo: de({}, ne.hoverinfo, { flags: [], arrayOk: !1 }), hoverlabel: ae.hoverlabel, domain: oe({ name: "sankey", trace: !0 }), orientation: { valType: "enumerated", values: ["v", "h"], dflt: "h" }, valueformat: { valType: "string", dflt: ".3s", description: ue("value") }, valuesuffix: { valType: "string", dflt: "" }, arrangement: { valType: "enumerated", values: ["snap", "perpendicular", "freeform", "fixed"], dflt: "snap" }, textfont: te({ autoShadowDflt: !0 }), customdata: void 0, node: { label: { valType: "data_array", dflt: [] }, groups: { valType: "info_array", impliedEdits: { x: [], y: [] }, dimensions: 2, freeLength: !0, dflt: [], items: { valType: "number", editType: "calc" } }, x: { valType: "data_array", dflt: [] }, y: { valType: "data_array", dflt: [] }, color: { valType: "color", arrayOk: !0 }, customdata: { valType: "data_array", editType: "calc" }, line: { color: { valType: "color", dflt: ee.defaultLine, arrayOk: !0 }, width: { valType: "number", min: 0, dflt: 0.5, arrayOk: !0 } }, pad: { valType: "number", arrayOk: !1, min: 0, dflt: 20 }, thickness: { valType: "number", arrayOk: !1, min: 1, dflt: 20 }, hoverinfo: { valType: "enumerated", values: ["all", "none", "skip"], dflt: "all" }, hoverlabel: ae.hoverlabel, hovertemplate: se({}, { keys: ["value", "label"] }), align: { valType: "enumerated", values: ["justify", "left", "right", "center"], dflt: "justify" } }, link: { arrowlen: { valType: "number", min: 0, dflt: 0 }, label: { valType: "data_array", dflt: [] }, color: { valType: "color", arrayOk: !0 }, hovercolor: { valType: "color", arrayOk: !0 }, customdata: { valType: "data_array", editType: "calc" }, line: { color: { valType: "color", dflt: ee.defaultLine, arrayOk: !0 }, width: { valType: "number", min: 0, dflt: 0, arrayOk: !0 } }, source: { valType: "data_array", dflt: [] }, target: { valType: "data_array", dflt: [] }, value: { valType: "data_array", dflt: [] }, hoverinfo: { valType: "enumerated", values: ["all", "none", "skip"], dflt: "all" }, hoverlabel: ae.hoverlabel, hovertemplate: se({}, { keys: ["value", "label"] }), colorscales: le("concentrationscales", { editType: "calc", label: { valType: "string", editType: "calc", dflt: "" }, cmax: { valType: "number", editType: "calc", dflt: 1 }, cmin: { valType: "number", editType: "calc", dflt: 0 }, colorscale: de(ce().colorscale, { dflt: [[0, "white"], [1, "black"]] }) }) } }, "calc", "nested")).transforms = void 0;
          }, 42229: function(G, J, Z) {
            var te = Z(13582).overrideAll, ne = Z(4173).eV, ee = Z(16506), ae = Z(6811), oe = Z(27983), se = Z(14751), ce = Z(44844).prepSelect, le = Z(34809), ue = Z(33626), de = "sankey";
            function fe(he, me) {
              var pe = he._fullData[me], ye = he._fullLayout, ge = ye.dragmode, ve = ye.dragmode === "pan" ? "move" : "crosshair", be = pe._bgRect;
              if (be && ge !== "pan" && ge !== "zoom") {
                oe(be, ve);
                var Ce = { _id: "x", c2p: le.identity, _offset: pe._sankey.translateX, _length: pe._sankey.width }, Se = { _id: "y", c2p: le.identity, _offset: pe._sankey.translateY, _length: pe._sankey.height }, we = { gd: he, element: be.node(), plotinfo: { id: me, xaxis: Ce, yaxis: Se, fillRangeItems: le.noop }, subplot: me, xaxes: [Ce], yaxes: [Se], doneFnCompleted: function(Ee) {
                  var xe, Le = he._fullData[me], Qe = Le.node.groups.slice(), ke = [];
                  function Me($e) {
                    for (var qe = Le._sankey.graph.nodes, Xe = 0; Xe < qe.length; Xe++) if (qe[Xe].pointNumber === $e) return qe[Xe];
                  }
                  for (var Re = 0; Re < Ee.length; Re++) {
                    var De = Me(Ee[Re].pointNumber);
                    if (De) if (De.group) {
                      for (var Pe = 0; Pe < De.childrenNodes.length; Pe++) ke.push(De.childrenNodes[Pe].pointNumber);
                      Qe[De.pointNumber - Le.node._count] = !1;
                    } else ke.push(De.pointNumber);
                  }
                  xe = Qe.filter(Boolean).concat([ke]), ue.call("_guiRestyle", he, { "node.groups": [xe] }, me);
                }, prepFn: function(Ee, xe, Le) {
                  ce(Ee, xe, Le, we, ge);
                } };
                se.init(we);
              }
            }
            J.name = de, J.baseLayoutAttrOverrides = te({ hoverlabel: ae.hoverlabel }, "plot", "nested"), J.plot = function(he) {
              var me = ne(he.calcdata, de)[0];
              ee(he, me), J.updateFx(he);
            }, J.clean = function(he, me, pe, ye) {
              var ge = ye._has && ye._has(de), ve = me._has && me._has(de);
              ge && !ve && (ye._paperdiv.selectAll(".sankey").remove(), ye._paperdiv.selectAll(".bgsankey").remove());
            }, J.updateFx = function(he) {
              for (var me = 0; me < he._fullData.length; me++) fe(he, me);
            };
          }, 22915: function(G, J, Z) {
            var te = Z(26381), ne = Z(34809), ee = Z(71293).wrap, ae = ne.isArrayOrTypedArray, oe = ne.isIndex, se = Z(88856);
            G.exports = function(ce, le) {
              var ue = function(de) {
                var fe, he = de.node, me = de.link, pe = [], ye = ae(me.color), ge = ae(me.hovercolor), ve = ae(me.customdata), be = {}, Ce = {}, Se = me.colorscales.length;
                for (fe = 0; fe < Se; fe++) {
                  var we = me.colorscales[fe], Ee = se.extractScale(we, { cLetter: "c" }), xe = se.makeColorScaleFunc(Ee);
                  Ce[we.label] = xe;
                }
                var Le = 0;
                for (fe = 0; fe < me.value.length; fe++) me.source[fe] > Le && (Le = me.source[fe]), me.target[fe] > Le && (Le = me.target[fe]);
                var Qe, ke = Le + 1;
                de.node._count = ke;
                var Me = de.node.groups, Re = {};
                for (fe = 0; fe < Me.length; fe++) {
                  var De = Me[fe];
                  for (Qe = 0; Qe < De.length; Qe++) {
                    var Pe = De[Qe], $e = ke + fe;
                    Re.hasOwnProperty(Pe) ? ne.warn("Node " + Pe + " is already part of a group.") : Re[Pe] = $e;
                  }
                }
                var qe = { source: [], target: [] };
                for (fe = 0; fe < me.value.length; fe++) {
                  var Xe = me.value[fe], Ve = me.source[fe], Ze = me.target[fe];
                  if (Xe > 0 && oe(Ve, ke) && oe(Ze, ke) && (!Re.hasOwnProperty(Ve) || !Re.hasOwnProperty(Ze) || Re[Ve] !== Re[Ze])) {
                    Re.hasOwnProperty(Ze) && (Ze = Re[Ze]), Re.hasOwnProperty(Ve) && (Ve = Re[Ve]), Ze = +Ze, be[Ve = +Ve] = be[Ze] = !0;
                    var Ge = "";
                    me.label && me.label[fe] && (Ge = me.label[fe]);
                    var ze = null;
                    Ge && Ce.hasOwnProperty(Ge) && (ze = Ce[Ge]), pe.push({ pointNumber: fe, label: Ge, color: ye ? me.color[fe] : me.color, hovercolor: ge ? me.hovercolor[fe] : me.hovercolor, customdata: ve ? me.customdata[fe] : me.customdata, concentrationscale: ze, source: Ve, target: Ze, value: +Xe }), qe.source.push(Ve), qe.target.push(Ze);
                  }
                }
                var We = ke + Me.length, at = ae(he.color), ct = ae(he.customdata), et = [];
                for (fe = 0; fe < We; fe++) if (be[fe]) {
                  var dt = he.label[fe];
                  et.push({ group: fe > ke - 1, childrenNodes: [], pointNumber: fe, label: dt, color: at ? he.color[fe] : he.color, customdata: ct ? he.customdata[fe] : he.customdata });
                }
                var Je = !1;
                return function(it, st, gt) {
                  for (var bt = ne.init2dArray(it, 0), yt = 0; yt < Math.min(st.length, gt.length); yt++) if (ne.isIndex(st[yt], it) && ne.isIndex(gt[yt], it)) {
                    if (st[yt] === gt[yt]) return !0;
                    bt[st[yt]].push(gt[yt]);
                  }
                  return te(bt).components.some(function(St) {
                    return St.length > 1;
                  });
                }(We, qe.source, qe.target) && (Je = !0), { circular: Je, links: pe, nodes: et, groups: Me, groupLookup: Re };
              }(le);
              return ee({ circular: ue.circular, _nodes: ue.nodes, _links: ue.links, _groups: ue.groups, _groupLookup: ue.groupLookup });
            };
          }, 21541: function(G) {
            G.exports = { nodeTextOffsetHorizontal: 4, nodeTextOffsetVertical: 3, nodePadAcross: 10, sankeyIterations: 50, forceIterations: 5, forceTicksPerFrame: 10, duration: 500, ease: "linear", cn: { sankey: "sankey", sankeyLinks: "sankey-links", sankeyLink: "sankey-link", sankeyNodeSet: "sankey-node-set", sankeyNode: "sankey-node", nodeRect: "node-rect", nodeLabel: "node-label" } };
          }, 67940: function(G, J, Z) {
            var te = Z(34809), ne = Z(33795), ee = Z(78766), ae = Z(65657), oe = Z(13792).N, se = Z(26430), ce = Z(78032), le = Z(59008);
            function ue(de, fe) {
              function he(me, pe) {
                return te.coerce(de, fe, ne.link.colorscales, me, pe);
              }
              he("label"), he("cmin"), he("cmax"), he("colorscale");
            }
            G.exports = function(de, fe, he, me) {
              function pe(Me, Re) {
                return te.coerce(de, fe, ne, Me, Re);
              }
              var ye = te.extendDeep(me.hoverlabel, de.hoverlabel), ge = de.node, ve = ce.newContainer(fe, "node");
              function be(Me, Re) {
                return te.coerce(ge, ve, ne.node, Me, Re);
              }
              be("label"), be("groups"), be("x"), be("y"), be("pad"), be("thickness"), be("line.color"), be("line.width"), be("hoverinfo", de.hoverinfo), se(ge, ve, be, ye), be("hovertemplate"), be("align");
              var Ce = me.colorway;
              be("color", ve.label.map(function(Me, Re) {
                return ee.addOpacity(function(De) {
                  return Ce[De % Ce.length];
                }(Re), 0.8);
              })), be("customdata");
              var Se = de.link || {}, we = ce.newContainer(fe, "link");
              function Ee(Me, Re) {
                return te.coerce(Se, we, ne.link, Me, Re);
              }
              Ee("label"), Ee("arrowlen"), Ee("source"), Ee("target"), Ee("value"), Ee("line.color"), Ee("line.width"), Ee("hoverinfo", de.hoverinfo), se(Se, we, Ee, ye), Ee("hovertemplate");
              var xe, Le = ae(me.paper_bgcolor).getLuminance() < 0.333, Qe = Ee("color", Le ? "rgba(255, 255, 255, 0.6)" : "rgba(0, 0, 0, 0.2)");
              function ke(Me) {
                var Re = ae(Me);
                if (!Re.isValid()) return Me;
                var De = Re.getAlpha();
                return De <= 0.8 ? Re.setAlpha(De + 0.2) : Re = Le ? Re.brighten() : Re.darken(), Re.toRgbString();
              }
              Ee("hovercolor", Array.isArray(Qe) ? Qe.map(ke) : ke(Qe)), Ee("customdata"), le(Se, we, { name: "colorscales", handleItemDefaults: ue }), oe(fe, me, pe), pe("orientation"), pe("valueformat"), pe("valuesuffix"), ve.x.length && ve.y.length && (xe = "freeform"), pe("arrangement", xe), te.coerceFont(pe, "textfont", me.font, { autoShadowDflt: !0 }), fe._length = null;
            };
          }, 71760: function(G, J, Z) {
            G.exports = { attributes: Z(33795), supplyDefaults: Z(67940), calc: Z(22915), plot: Z(16506), moduleType: "trace", name: "sankey", basePlotModule: Z(42229), selectPoints: Z(74670), categories: ["noOpacity"], meta: {} };
          }, 16506: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = ne.numberFormat, ae = Z(90958), oe = Z(32141), se = Z(78766), ce = Z(21541).cn, le = ne._;
            function ue(Se) {
              return Se !== "";
            }
            function de(Se, we) {
              return Se.filter(function(Ee) {
                return Ee.key === we.traceId;
              });
            }
            function fe(Se, we) {
              te.select(Se).select("path").style("fill-opacity", we), te.select(Se).select("rect").style("fill-opacity", we);
            }
            function he(Se) {
              te.select(Se).select("text.name").style("fill", "black");
            }
            function me(Se) {
              return function(we) {
                return Se.node.sourceLinks.indexOf(we.link) !== -1 || Se.node.targetLinks.indexOf(we.link) !== -1;
              };
            }
            function pe(Se) {
              return function(we) {
                return we.node.sourceLinks.indexOf(Se.link) !== -1 || we.node.targetLinks.indexOf(Se.link) !== -1;
              };
            }
            function ye(Se, we, Ee) {
              we && Ee && de(Ee, we).selectAll("." + ce.sankeyLink).filter(me(we)).call(ve.bind(0, we, Ee, !1));
            }
            function ge(Se, we, Ee) {
              we && Ee && de(Ee, we).selectAll("." + ce.sankeyLink).filter(me(we)).call(be.bind(0, we, Ee, !1));
            }
            function ve(Se, we, Ee, xe) {
              xe.style("fill", function(Le) {
                if (!Le.link.concentrationscale) return Le.tinyColorHoverHue;
              }).style("fill-opacity", function(Le) {
                if (!Le.link.concentrationscale) return Le.tinyColorHoverAlpha;
              }), xe.each(function(Le) {
                var Qe = Le.link.label;
                Qe !== "" && de(we, Se).selectAll("." + ce.sankeyLink).filter(function(ke) {
                  return ke.link.label === Qe;
                }).style("fill", function(ke) {
                  if (!ke.link.concentrationscale) return ke.tinyColorHoverHue;
                }).style("fill-opacity", function(ke) {
                  if (!ke.link.concentrationscale) return ke.tinyColorHoverAlpha;
                });
              }), Ee && de(we, Se).selectAll("." + ce.sankeyNode).filter(pe(Se)).call(ye);
            }
            function be(Se, we, Ee, xe) {
              xe.style("fill", function(Le) {
                return Le.tinyColorHue;
              }).style("fill-opacity", function(Le) {
                return Le.tinyColorAlpha;
              }), xe.each(function(Le) {
                var Qe = Le.link.label;
                Qe !== "" && de(we, Se).selectAll("." + ce.sankeyLink).filter(function(ke) {
                  return ke.link.label === Qe;
                }).style("fill", function(ke) {
                  return ke.tinyColorHue;
                }).style("fill-opacity", function(ke) {
                  return ke.tinyColorAlpha;
                });
              }), Ee && de(we, Se).selectAll(ce.sankeyNode).filter(pe(Se)).call(ge);
            }
            function Ce(Se, we) {
              var Ee = Se.hoverlabel || {}, xe = ne.nestedProperty(Ee, we).get();
              return !Array.isArray(xe) && xe;
            }
            G.exports = function(Se, we) {
              for (var Ee = Se._fullLayout, xe = Ee._paper, Le = Ee._size, Qe = 0; Qe < Se._fullData.length; Qe++) if (Se._fullData[Qe].visible && Se._fullData[Qe].type === ce.sankey && !Se._fullData[Qe]._viewInitial) {
                var ke = Se._fullData[Qe].node;
                Se._fullData[Qe]._viewInitial = { node: { groups: ke.groups.slice(), x: ke.x.slice(), y: ke.y.slice() } };
              }
              var Me = le(Se, "source:") + " ", Re = le(Se, "target:") + " ", De = le(Se, "concentration:") + " ", Pe = le(Se, "incoming flow count:") + " ", $e = le(Se, "outgoing flow count:") + " ";
              ae(Se, xe, we, { width: Le.w, height: Le.h, margin: { t: Le.t, r: Le.r, b: Le.b, l: Le.l } }, { linkEvents: { hover: function(qe, Xe, Ve) {
                Se._fullLayout.hovermode !== !1 && (te.select(qe).call(ve.bind(0, Xe, Ve, !0)), Xe.link.trace.link.hoverinfo !== "skip" && (Xe.link.fullData = Xe.link.trace, Se.emit("plotly_hover", { event: te.event, points: [Xe.link] })));
              }, follow: function(qe, Xe) {
                if (Se._fullLayout.hovermode !== !1) {
                  var Ve = Xe.link.trace.link;
                  if (Ve.hoverinfo !== "none" && Ve.hoverinfo !== "skip") {
                    for (var Ze = [], Ge = 0, ze = 0; ze < Xe.flow.links.length; ze++) {
                      var We = Xe.flow.links[ze];
                      if (Se._fullLayout.hovermode !== "closest" || Xe.link.pointNumber === We.pointNumber) {
                        Xe.link.pointNumber === We.pointNumber && (Ge = ze), We.fullData = We.trace, Ve = Xe.link.trace.link;
                        var at = et(We), ct = { valueLabel: ee(Xe.valueFormat)(We.value) + Xe.valueSuffix };
                        Ze.push({ x: at[0], y: at[1], name: ct.valueLabel, text: [We.label || "", Me + We.source.label, Re + We.target.label, We.concentrationscale ? De + ee("%0.2f")(We.flow.labelConcentration) : ""].filter(ue).join("<br>"), color: Ce(Ve, "bgcolor") || se.addOpacity(We.color, 1), borderColor: Ce(Ve, "bordercolor"), fontFamily: Ce(Ve, "font.family"), fontSize: Ce(Ve, "font.size"), fontColor: Ce(Ve, "font.color"), fontWeight: Ce(Ve, "font.weight"), fontStyle: Ce(Ve, "font.style"), fontVariant: Ce(Ve, "font.variant"), fontTextcase: Ce(Ve, "font.textcase"), fontLineposition: Ce(Ve, "font.lineposition"), fontShadow: Ce(Ve, "font.shadow"), nameLength: Ce(Ve, "namelength"), textAlign: Ce(Ve, "align"), idealAlign: te.event.x < at[0] ? "right" : "left", hovertemplate: Ve.hovertemplate, hovertemplateLabels: ct, eventData: [We] });
                      }
                    }
                    oe.loneHover(Ze, { container: Ee._hoverlayer.node(), outerContainer: Ee._paper.node(), gd: Se, anchorIndex: Ge }).each(function() {
                      Xe.link.concentrationscale || fe(this, 0.65), he(this);
                    });
                  }
                }
                function et(dt) {
                  var Je, it;
                  dt.circular ? (Je = (dt.circularPathData.leftInnerExtent + dt.circularPathData.rightInnerExtent) / 2, it = dt.circularPathData.verticalFullExtent) : (Je = (dt.source.x1 + dt.target.x0) / 2, it = (dt.y0 + dt.y1) / 2);
                  var st = [Je, it];
                  return dt.trace.orientation === "v" && st.reverse(), st[0] += Xe.parent.translateX, st[1] += Xe.parent.translateY, st;
                }
              }, unhover: function(qe, Xe, Ve) {
                Se._fullLayout.hovermode !== !1 && (te.select(qe).call(be.bind(0, Xe, Ve, !0)), Xe.link.trace.link.hoverinfo !== "skip" && (Xe.link.fullData = Xe.link.trace, Se.emit("plotly_unhover", { event: te.event, points: [Xe.link] })), oe.loneUnhover(Ee._hoverlayer.node()));
              }, select: function(qe, Xe) {
                var Ve = Xe.link;
                Ve.originalEvent = te.event, Se._hoverdata = [Ve], oe.click(Se, { target: !0 });
              } }, nodeEvents: { hover: function(qe, Xe, Ve) {
                Se._fullLayout.hovermode !== !1 && (te.select(qe).call(ye, Xe, Ve), Xe.node.trace.node.hoverinfo !== "skip" && (Xe.node.fullData = Xe.node.trace, Se.emit("plotly_hover", { event: te.event, points: [Xe.node] })));
              }, follow: function(qe, Xe) {
                if (Se._fullLayout.hovermode !== !1) {
                  var Ve = Xe.node.trace.node;
                  if (Ve.hoverinfo !== "none" && Ve.hoverinfo !== "skip") {
                    var Ze = te.select(qe).select("." + ce.nodeRect), Ge = Se._fullLayout._paperdiv.node().getBoundingClientRect(), ze = Ze.node().getBoundingClientRect(), We = ze.left - 2 - Ge.left, at = ze.right + 2 - Ge.left, ct = ze.top + ze.height / 4 - Ge.top, et = { valueLabel: ee(Xe.valueFormat)(Xe.node.value) + Xe.valueSuffix };
                    Xe.node.fullData = Xe.node.trace, Se._fullLayout._calcInverseTransform(Se);
                    var dt = Se._fullLayout._invScaleX, Je = Se._fullLayout._invScaleY, it = oe.loneHover({ x0: dt * We, x1: dt * at, y: Je * ct, name: ee(Xe.valueFormat)(Xe.node.value) + Xe.valueSuffix, text: [Xe.node.label, Pe + Xe.node.targetLinks.length, $e + Xe.node.sourceLinks.length].filter(ue).join("<br>"), color: Ce(Ve, "bgcolor") || Xe.tinyColorHue, borderColor: Ce(Ve, "bordercolor"), fontFamily: Ce(Ve, "font.family"), fontSize: Ce(Ve, "font.size"), fontColor: Ce(Ve, "font.color"), fontWeight: Ce(Ve, "font.weight"), fontStyle: Ce(Ve, "font.style"), fontVariant: Ce(Ve, "font.variant"), fontTextcase: Ce(Ve, "font.textcase"), fontLineposition: Ce(Ve, "font.lineposition"), fontShadow: Ce(Ve, "font.shadow"), nameLength: Ce(Ve, "namelength"), textAlign: Ce(Ve, "align"), idealAlign: "left", hovertemplate: Ve.hovertemplate, hovertemplateLabels: et, eventData: [Xe.node] }, { container: Ee._hoverlayer.node(), outerContainer: Ee._paper.node(), gd: Se });
                    fe(it, 0.85), he(it);
                  }
                }
              }, unhover: function(qe, Xe, Ve) {
                Se._fullLayout.hovermode !== !1 && (te.select(qe).call(ge, Xe, Ve), Xe.node.trace.node.hoverinfo !== "skip" && (Xe.node.fullData = Xe.node.trace, Se.emit("plotly_unhover", { event: te.event, points: [Xe.node] })), oe.loneUnhover(Ee._hoverlayer.node()));
              }, select: function(qe, Xe, Ve) {
                var Ze = Xe.node;
                Ze.originalEvent = te.event, Se._hoverdata = [Ze], te.select(qe).call(ge, Xe, Ve), oe.click(Se, { target: !0 });
              } } });
            };
          }, 90958: function(G, J, Z) {
            var te = Z(32702), ne = Z(88640).Dj, ee = Z(45568), ae = Z(62369), oe = Z(68735), se = Z(21541), ce = Z(65657), le = Z(78766), ue = Z(62203), de = Z(34809), fe = de.strTranslate, he = de.strRotate, me = Z(71293), pe = me.keyFun, ye = me.repeat, ge = me.unwrap, ve = Z(30635), be = Z(33626), Ce = Z(4530), Se = Ce.CAP_SHIFT, we = Ce.LINE_SPACING;
            function Ee(ze, We, at) {
              var ct, et = ge(We), dt = et.trace, Je = dt.domain, it = dt.orientation === "h", st = dt.node.pad, gt = dt.node.thickness, bt = { justify: ae.sankeyJustify, left: ae.sankeyLeft, right: ae.sankeyRight, center: ae.sankeyCenter }[dt.node.align], yt = ze.width * (Je.x[1] - Je.x[0]), St = ze.height * (Je.y[1] - Je.y[0]), Lt = et._nodes, Rt = et._links, vt = et.circular;
              (ct = vt ? oe.sankeyCircular().circularLinkGap(0) : ae.sankey()).iterations(se.sankeyIterations).size(it ? [yt, St] : [St, yt]).nodeWidth(gt).nodePadding(st).nodeId(function(Vt) {
                return Vt.pointNumber;
              }).nodeAlign(bt).nodes(Lt).links(Rt);
              var $t, Pt, Xt, Nt = ct();
              for (var Gt in ct.nodePadding() < st && de.warn("node.pad was reduced to ", ct.nodePadding(), " to fit within the figure."), et._groupLookup) {
                var Ft, Tt = parseInt(et._groupLookup[Gt]);
                for ($t = 0; $t < Nt.nodes.length; $t++) if (Nt.nodes[$t].pointNumber === Tt) {
                  Ft = Nt.nodes[$t];
                  break;
                }
                if (Ft) {
                  var It = { pointNumber: parseInt(Gt), x0: Ft.x0, x1: Ft.x1, y0: Ft.y0, y1: Ft.y1, partOfGroup: !0, sourceLinks: [], targetLinks: [] };
                  Nt.nodes.unshift(It), Ft.childrenNodes.unshift(It);
                }
              }
              if (function() {
                for ($t = 0; $t < Nt.nodes.length; $t++) {
                  var Vt, Yt, sn = Nt.nodes[$t], hn = {};
                  for (Pt = 0; Pt < sn.targetLinks.length; Pt++) Vt = (Yt = sn.targetLinks[Pt]).source.pointNumber + ":" + Yt.target.pointNumber, hn.hasOwnProperty(Vt) || (hn[Vt] = []), hn[Vt].push(Yt);
                  var Qt = Object.keys(hn);
                  for (Pt = 0; Pt < Qt.length; Pt++) {
                    var Ut = hn[Vt = Qt[Pt]], Wt = 0, rn = {};
                    for (Xt = 0; Xt < Ut.length; Xt++) rn[(Yt = Ut[Xt]).label] || (rn[Yt.label] = 0), rn[Yt.label] += Yt.value, Wt += Yt.value;
                    for (Xt = 0; Xt < Ut.length; Xt++) (Yt = Ut[Xt]).flow = { value: Wt, labelConcentration: rn[Yt.label] / Wt, concentration: Yt.value / Wt, links: Ut }, Yt.concentrationscale && (Yt.color = ce(Yt.concentrationscale(Yt.flow.labelConcentration)));
                  }
                  var vn = 0;
                  for (Pt = 0; Pt < sn.sourceLinks.length; Pt++) vn += sn.sourceLinks[Pt].value;
                  for (Pt = 0; Pt < sn.sourceLinks.length; Pt++) (Yt = sn.sourceLinks[Pt]).concentrationOut = Yt.value / vn;
                  var dn = 0;
                  for (Pt = 0; Pt < sn.targetLinks.length; Pt++) dn += sn.targetLinks[Pt].value;
                  for (Pt = 0; Pt < sn.targetLinks.length; Pt++) (Yt = sn.targetLinks[Pt]).concenrationIn = Yt.value / dn;
                }
              }(), dt.node.x.length && dt.node.y.length) {
                for ($t = 0; $t < Math.min(dt.node.x.length, dt.node.y.length, Nt.nodes.length); $t++) if (dt.node.x[$t] && dt.node.y[$t]) {
                  var Ct = [dt.node.x[$t] * yt, dt.node.y[$t] * St];
                  Nt.nodes[$t].x0 = Ct[0] - gt / 2, Nt.nodes[$t].x1 = Ct[0] + gt / 2;
                  var Kt = Nt.nodes[$t].y1 - Nt.nodes[$t].y0;
                  Nt.nodes[$t].y0 = Ct[1] - Kt / 2, Nt.nodes[$t].y1 = Ct[1] + Kt / 2;
                }
                dt.arrangement === "snap" && function(Vt) {
                  var Yt, sn, hn = Vt.map(function(vn, dn) {
                    return { x0: vn.x0, index: dn };
                  }).sort(function(vn, dn) {
                    return vn.x0 - dn.x0;
                  }), Qt = [], Ut = -1, Wt = -1 / 0;
                  for ($t = 0; $t < hn.length; $t++) {
                    var rn = Vt[hn[$t].index];
                    rn.x0 > Wt + gt && (Ut += 1, Yt = rn.x0), Wt = rn.x0, Qt[Ut] || (Qt[Ut] = []), Qt[Ut].push(rn), sn = Yt - rn.x0, rn.x0 += sn, rn.x1 += sn;
                  }
                  return Qt;
                }(Lt = Nt.nodes).forEach(function(Vt) {
                  var Yt, sn, hn, Qt = 0, Ut = Vt.length;
                  for (Vt.sort(function(Wt, rn) {
                    return Wt.y0 - rn.y0;
                  }), hn = 0; hn < Ut; ++hn) (Yt = Vt[hn]).y0 >= Qt || (sn = Qt - Yt.y0) > 1e-6 && (Yt.y0 += sn, Yt.y1 += sn), Qt = Yt.y1 + st;
                }), ct.update(Nt);
              }
              return { circular: vt, key: at, trace: dt, guid: de.randstr(), horizontal: it, width: yt, height: St, nodePad: dt.node.pad, nodeLineColor: dt.node.line.color, nodeLineWidth: dt.node.line.width, linkLineColor: dt.link.line.color, linkLineWidth: dt.link.line.width, linkArrowLength: dt.link.arrowlen, valueFormat: dt.valueformat, valueSuffix: dt.valuesuffix, textFont: dt.textfont, translateX: Je.x[0] * ze.width + ze.margin.l, translateY: ze.height - Je.y[1] * ze.height + ze.margin.t, dragParallel: it ? St : yt, dragPerpendicular: it ? yt : St, arrangement: dt.arrangement, sankey: ct, graph: Nt, forceLayouts: {}, interactionState: { dragInProgress: !1, hovered: !1 } };
            }
            function xe(ze, We, at) {
              var ct = ce(We.color), et = ce(We.hovercolor), dt = We.source.label + "|" + We.target.label + "__" + at;
              return We.trace = ze.trace, We.curveNumber = ze.trace.index, { circular: ze.circular, key: dt, traceId: ze.key, pointNumber: We.pointNumber, link: We, tinyColorHue: le.tinyRGB(ct), tinyColorAlpha: ct.getAlpha(), tinyColorHoverHue: le.tinyRGB(et), tinyColorHoverAlpha: et.getAlpha(), linkPath: Le, linkLineColor: ze.linkLineColor, linkLineWidth: ze.linkLineWidth, linkArrowLength: ze.linkArrowLength, valueFormat: ze.valueFormat, valueSuffix: ze.valueSuffix, sankey: ze.sankey, parent: ze, interactionState: ze.interactionState, flow: We.flow };
            }
            function Le() {
              return function(ze) {
                var We = ze.linkArrowLength;
                if (ze.link.circular) return function($t, Pt) {
                  var Xt = $t.width / 2, Nt = $t.circularPathData;
                  return $t.circularLinkType === "top" ? "M " + (Nt.targetX - Pt) + " " + (Nt.targetY + Xt) + " L" + (Nt.rightInnerExtent - Pt) + " " + (Nt.targetY + Xt) + "A" + (Nt.rightLargeArcRadius + Xt) + " " + (Nt.rightSmallArcRadius + Xt) + " 0 0 1 " + (Nt.rightFullExtent - Xt - Pt) + " " + (Nt.targetY - Nt.rightSmallArcRadius) + "L" + (Nt.rightFullExtent - Xt - Pt) + " " + Nt.verticalRightInnerExtent + "A" + (Nt.rightLargeArcRadius + Xt) + " " + (Nt.rightLargeArcRadius + Xt) + " 0 0 1 " + (Nt.rightInnerExtent - Pt) + " " + (Nt.verticalFullExtent - Xt) + "L" + Nt.leftInnerExtent + " " + (Nt.verticalFullExtent - Xt) + "A" + (Nt.leftLargeArcRadius + Xt) + " " + (Nt.leftLargeArcRadius + Xt) + " 0 0 1 " + (Nt.leftFullExtent + Xt) + " " + Nt.verticalLeftInnerExtent + "L" + (Nt.leftFullExtent + Xt) + " " + (Nt.sourceY - Nt.leftSmallArcRadius) + "A" + (Nt.leftLargeArcRadius + Xt) + " " + (Nt.leftSmallArcRadius + Xt) + " 0 0 1 " + Nt.leftInnerExtent + " " + (Nt.sourceY + Xt) + "L" + Nt.sourceX + " " + (Nt.sourceY + Xt) + "L" + Nt.sourceX + " " + (Nt.sourceY - Xt) + "L" + Nt.leftInnerExtent + " " + (Nt.sourceY - Xt) + "A" + (Nt.leftLargeArcRadius - Xt) + " " + (Nt.leftSmallArcRadius - Xt) + " 0 0 0 " + (Nt.leftFullExtent - Xt) + " " + (Nt.sourceY - Nt.leftSmallArcRadius) + "L" + (Nt.leftFullExtent - Xt) + " " + Nt.verticalLeftInnerExtent + "A" + (Nt.leftLargeArcRadius - Xt) + " " + (Nt.leftLargeArcRadius - Xt) + " 0 0 0 " + Nt.leftInnerExtent + " " + (Nt.verticalFullExtent + Xt) + "L" + (Nt.rightInnerExtent - Pt) + " " + (Nt.verticalFullExtent + Xt) + "A" + (Nt.rightLargeArcRadius - Xt) + " " + (Nt.rightLargeArcRadius - Xt) + " 0 0 0 " + (Nt.rightFullExtent + Xt - Pt) + " " + Nt.verticalRightInnerExtent + "L" + (Nt.rightFullExtent + Xt - Pt) + " " + (Nt.targetY - Nt.rightSmallArcRadius) + "A" + (Nt.rightLargeArcRadius - Xt) + " " + (Nt.rightSmallArcRadius - Xt) + " 0 0 0 " + (Nt.rightInnerExtent - Pt) + " " + (Nt.targetY - Xt) + "L" + (Nt.targetX - Pt) + " " + (Nt.targetY - Xt) + (Pt > 0 ? "L" + Nt.targetX + " " + Nt.targetY : "") + "Z" : "M " + (Nt.targetX - Pt) + " " + (Nt.targetY - Xt) + " L" + (Nt.rightInnerExtent - Pt) + " " + (Nt.targetY - Xt) + "A" + (Nt.rightLargeArcRadius + Xt) + " " + (Nt.rightSmallArcRadius + Xt) + " 0 0 0 " + (Nt.rightFullExtent - Xt - Pt) + " " + (Nt.targetY + Nt.rightSmallArcRadius) + "L" + (Nt.rightFullExtent - Xt - Pt) + " " + Nt.verticalRightInnerExtent + "A" + (Nt.rightLargeArcRadius + Xt) + " " + (Nt.rightLargeArcRadius + Xt) + " 0 0 0 " + (Nt.rightInnerExtent - Pt) + " " + (Nt.verticalFullExtent + Xt) + "L" + Nt.leftInnerExtent + " " + (Nt.verticalFullExtent + Xt) + "A" + (Nt.leftLargeArcRadius + Xt) + " " + (Nt.leftLargeArcRadius + Xt) + " 0 0 0 " + (Nt.leftFullExtent + Xt) + " " + Nt.verticalLeftInnerExtent + "L" + (Nt.leftFullExtent + Xt) + " " + (Nt.sourceY + Nt.leftSmallArcRadius) + "A" + (Nt.leftLargeArcRadius + Xt) + " " + (Nt.leftSmallArcRadius + Xt) + " 0 0 0 " + Nt.leftInnerExtent + " " + (Nt.sourceY - Xt) + "L" + Nt.sourceX + " " + (Nt.sourceY - Xt) + "L" + Nt.sourceX + " " + (Nt.sourceY + Xt) + "L" + Nt.leftInnerExtent + " " + (Nt.sourceY + Xt) + "A" + (Nt.leftLargeArcRadius - Xt) + " " + (Nt.leftSmallArcRadius - Xt) + " 0 0 1 " + (Nt.leftFullExtent - Xt) + " " + (Nt.sourceY + Nt.leftSmallArcRadius) + "L" + (Nt.leftFullExtent - Xt) + " " + Nt.verticalLeftInnerExtent + "A" + (Nt.leftLargeArcRadius - Xt) + " " + (Nt.leftLargeArcRadius - Xt) + " 0 0 1 " + Nt.leftInnerExtent + " " + (Nt.verticalFullExtent - Xt) + "L" + (Nt.rightInnerExtent - Pt) + " " + (Nt.verticalFullExtent - Xt) + "A" + (Nt.rightLargeArcRadius - Xt) + " " + (Nt.rightLargeArcRadius - Xt) + " 0 0 1 " + (Nt.rightFullExtent + Xt - Pt) + " " + Nt.verticalRightInnerExtent + "L" + (Nt.rightFullExtent + Xt - Pt) + " " + (Nt.targetY + Nt.rightSmallArcRadius) + "A" + (Nt.rightLargeArcRadius - Xt) + " " + (Nt.rightSmallArcRadius - Xt) + " 0 0 1 " + (Nt.rightInnerExtent - Pt) + " " + (Nt.targetY + Xt) + "L" + (Nt.targetX - Pt) + " " + (Nt.targetY + Xt) + (Pt > 0 ? "L" + Nt.targetX + " " + Nt.targetY : "") + "Z";
                }(ze.link, We);
                var at = Math.abs((ze.link.target.x0 - ze.link.source.x1) / 2);
                We > at && (We = at);
                var ct = ze.link.source.x1, et = ze.link.target.x0 - We, dt = ne(ct, et), Je = dt(0.5), it = dt(0.5), st = ze.link.y0 - ze.link.width / 2, gt = ze.link.y0 + ze.link.width / 2, bt = ze.link.y1 - ze.link.width / 2, yt = ze.link.y1 + ze.link.width / 2, St = "M" + ct + "," + st, Lt = "C" + Je + "," + st + " " + it + "," + bt + " " + et + "," + bt, Rt = "C" + it + "," + yt + " " + Je + "," + gt + " " + ct + "," + gt, vt = We > 0 ? "L" + (et + We) + "," + (bt + ze.link.width / 2) : "";
                return St + Lt + (vt += "L" + et + "," + yt) + Rt + "Z";
              };
            }
            function Qe(ze, We) {
              var at = ce(We.color), ct = se.nodePadAcross, et = ze.nodePad / 2;
              We.dx = We.x1 - We.x0, We.dy = We.y1 - We.y0;
              var dt = We.dx, Je = Math.max(0.5, We.dy), it = "node_" + We.pointNumber;
              return We.group && (it = de.randstr()), We.trace = ze.trace, We.curveNumber = ze.trace.index, { index: We.pointNumber, key: it, partOfGroup: We.partOfGroup || !1, group: We.group, traceId: ze.key, trace: ze.trace, node: We, nodePad: ze.nodePad, nodeLineColor: ze.nodeLineColor, nodeLineWidth: ze.nodeLineWidth, textFont: ze.textFont, size: ze.horizontal ? ze.height : ze.width, visibleWidth: Math.ceil(dt), visibleHeight: Je, zoneX: -ct, zoneY: -et, zoneWidth: dt + 2 * ct, zoneHeight: Je + 2 * et, labelY: ze.horizontal ? We.dy / 2 + 1 : We.dx / 2 + 1, left: We.originalLayer === 1, sizeAcross: ze.width, forceLayouts: ze.forceLayouts, horizontal: ze.horizontal, darkBackground: at.getBrightness() <= 128, tinyColorHue: le.tinyRGB(at), tinyColorAlpha: at.getAlpha(), valueFormat: ze.valueFormat, valueSuffix: ze.valueSuffix, sankey: ze.sankey, graph: ze.graph, arrangement: ze.arrangement, uniqueNodeLabelPathId: [ze.guid, ze.key, it].join("_"), interactionState: ze.interactionState, figure: ze };
            }
            function ke(ze) {
              ze.attr("transform", function(We) {
                return fe(We.node.x0.toFixed(3), We.node.y0.toFixed(3));
              });
            }
            function Me(ze) {
              ze.call(ke);
            }
            function Re(ze, We) {
              ze.call(Me), We.attr("d", Le());
            }
            function De(ze) {
              ze.attr("width", function(We) {
                return We.node.x1 - We.node.x0;
              }).attr("height", function(We) {
                return We.visibleHeight;
              });
            }
            function Pe(ze) {
              return ze.link.width > 1 || ze.linkLineWidth > 0;
            }
            function $e(ze) {
              return fe(ze.translateX, ze.translateY) + (ze.horizontal ? "matrix(1 0 0 1 0 0)" : "matrix(0 1 1 0 0 0)");
            }
            function qe(ze, We, at) {
              ze.on(".basic", null).on("mouseover.basic", function(ct) {
                ct.interactionState.dragInProgress || ct.partOfGroup || (at.hover(this, ct, We), ct.interactionState.hovered = [this, ct]);
              }).on("mousemove.basic", function(ct) {
                ct.interactionState.dragInProgress || ct.partOfGroup || (at.follow(this, ct), ct.interactionState.hovered = [this, ct]);
              }).on("mouseout.basic", function(ct) {
                ct.interactionState.dragInProgress || ct.partOfGroup || (at.unhover(this, ct, We), ct.interactionState.hovered = !1);
              }).on("click.basic", function(ct) {
                ct.interactionState.hovered && (at.unhover(this, ct, We), ct.interactionState.hovered = !1), ct.interactionState.dragInProgress || ct.partOfGroup || at.select(this, ct, We);
              });
            }
            function Xe(ze, We, at, ct) {
              var et = ee.behavior.drag().origin(function(dt) {
                return { x: dt.node.x0 + dt.visibleWidth / 2, y: dt.node.y0 + dt.visibleHeight / 2 };
              }).on("dragstart", function(dt) {
                if (dt.arrangement !== "fixed" && (de.ensureSingle(ct._fullLayout._infolayer, "g", "dragcover", function(it) {
                  ct._fullLayout._dragCover = it;
                }), de.raiseToTop(this), dt.interactionState.dragInProgress = dt.node, Ze(dt.node), dt.interactionState.hovered && (at.nodeEvents.unhover.apply(0, dt.interactionState.hovered), dt.interactionState.hovered = !1), dt.arrangement === "snap")) {
                  var Je = dt.traceId + "|" + dt.key;
                  dt.forceLayouts[Je] ? dt.forceLayouts[Je].alpha(1) : function(it, st, gt, bt) {
                    (function(St) {
                      for (var Lt = 0; Lt < St.length; Lt++) St[Lt].y = (St[Lt].y0 + St[Lt].y1) / 2, St[Lt].x = (St[Lt].x0 + St[Lt].x1) / 2;
                    })(gt.graph.nodes);
                    var yt = gt.graph.nodes.filter(function(St) {
                      return St.originalX === gt.node.originalX;
                    }).filter(function(St) {
                      return !St.partOfGroup;
                    });
                    gt.forceLayouts[st] = te.forceSimulation(yt).alphaDecay(0).force("collide", te.forceCollide().radius(function(St) {
                      return St.dy / 2 + gt.nodePad / 2;
                    }).strength(1).iterations(se.forceIterations)).force("constrain", /* @__PURE__ */ function(St, Lt, Rt, vt) {
                      return function() {
                        for (var $t = 0, Pt = 0; Pt < Rt.length; Pt++) {
                          var Xt = Rt[Pt];
                          Xt === vt.interactionState.dragInProgress ? (Xt.x = Xt.lastDraggedX, Xt.y = Xt.lastDraggedY) : (Xt.vx = (Xt.originalX - Xt.x) / se.forceTicksPerFrame, Xt.y = Math.min(vt.size - Xt.dy / 2, Math.max(Xt.dy / 2, Xt.y))), $t = Math.max($t, Math.abs(Xt.vx), Math.abs(Xt.vy));
                        }
                        !vt.interactionState.dragInProgress && $t < 0.1 && vt.forceLayouts[Lt].alpha() > 0 && vt.forceLayouts[Lt].alpha(0);
                      };
                    }(0, st, yt, gt)).stop();
                  }(0, Je, dt), function(it, st, gt, bt, yt) {
                    window.requestAnimationFrame(function St() {
                      var Lt;
                      for (Lt = 0; Lt < se.forceTicksPerFrame; Lt++) gt.forceLayouts[bt].tick();
                      if (function(vt) {
                        for (var $t = 0; $t < vt.length; $t++) vt[$t].y0 = vt[$t].y - vt[$t].dy / 2, vt[$t].y1 = vt[$t].y0 + vt[$t].dy, vt[$t].x0 = vt[$t].x - vt[$t].dx / 2, vt[$t].x1 = vt[$t].x0 + vt[$t].dx;
                      }(gt.graph.nodes), gt.sankey.update(gt.graph), Re(it.filter(Ge(gt)), st), gt.forceLayouts[bt].alpha() > 0) window.requestAnimationFrame(St);
                      else {
                        var Rt = gt.node.originalX;
                        gt.node.x0 = Rt - gt.visibleWidth / 2, gt.node.x1 = Rt + gt.visibleWidth / 2, Ve(gt, yt);
                      }
                    });
                  }(ze, We, dt, Je, ct);
                }
              }).on("drag", function(dt) {
                if (dt.arrangement !== "fixed") {
                  var Je = ee.event.x, it = ee.event.y;
                  dt.arrangement === "snap" ? (dt.node.x0 = Je - dt.visibleWidth / 2, dt.node.x1 = Je + dt.visibleWidth / 2, dt.node.y0 = it - dt.visibleHeight / 2, dt.node.y1 = it + dt.visibleHeight / 2) : (dt.arrangement === "freeform" && (dt.node.x0 = Je - dt.visibleWidth / 2, dt.node.x1 = Je + dt.visibleWidth / 2), it = Math.max(0, Math.min(dt.size - dt.visibleHeight / 2, it)), dt.node.y0 = it - dt.visibleHeight / 2, dt.node.y1 = it + dt.visibleHeight / 2), Ze(dt.node), dt.arrangement !== "snap" && (dt.sankey.update(dt.graph), Re(ze.filter(Ge(dt)), We));
                }
              }).on("dragend", function(dt) {
                if (dt.arrangement !== "fixed") {
                  dt.interactionState.dragInProgress = !1;
                  for (var Je = 0; Je < dt.node.childrenNodes.length; Je++) dt.node.childrenNodes[Je].x = dt.node.x, dt.node.childrenNodes[Je].y = dt.node.y;
                  dt.arrangement !== "snap" && Ve(dt, ct);
                }
              });
              ze.on(".drag", null).call(et);
            }
            function Ve(ze, We) {
              for (var at = [], ct = [], et = 0; et < ze.graph.nodes.length; et++) {
                var dt = (ze.graph.nodes[et].x0 + ze.graph.nodes[et].x1) / 2, Je = (ze.graph.nodes[et].y0 + ze.graph.nodes[et].y1) / 2;
                at.push(dt / ze.figure.width), ct.push(Je / ze.figure.height);
              }
              be.call("_guiRestyle", We, { "node.x": [at], "node.y": [ct] }, ze.trace.index).then(function() {
                We._fullLayout._dragCover && We._fullLayout._dragCover.remove();
              });
            }
            function Ze(ze) {
              ze.lastDraggedX = ze.x0 + ze.dx / 2, ze.lastDraggedY = ze.y0 + ze.dy / 2;
            }
            function Ge(ze) {
              return function(We) {
                return We.node.originalX === ze.node.originalX;
              };
            }
            G.exports = function(ze, We, at, ct, et) {
              var dt = ze._context.staticPlot, Je = !1;
              de.ensureSingle(ze._fullLayout._infolayer, "g", "first-render", function() {
                Je = !0;
              });
              var it = ze._fullLayout._dragCover, st = at.filter(function($t) {
                return ge($t).trace.visible;
              }).map(Ee.bind(null, ct)), gt = We.selectAll("." + se.cn.sankey).data(st, pe);
              gt.exit().remove(), gt.enter().append("g").classed(se.cn.sankey, !0).style("box-sizing", "content-box").style("position", "absolute").style("left", 0).style("shape-rendering", "geometricPrecision").style("pointer-events", dt ? "none" : "auto").attr("transform", $e), gt.each(function($t, Pt) {
                ze._fullData[Pt]._sankey = $t;
                var Xt = "bgsankey-" + $t.trace.uid + "-" + Pt;
                de.ensureSingle(ze._fullLayout._draggers, "rect", Xt), ze._fullData[Pt]._bgRect = ee.select("." + Xt), ze._fullData[Pt]._bgRect.style("pointer-events", dt ? "none" : "all").attr("width", $t.width).attr("height", $t.height).attr("x", $t.translateX).attr("y", $t.translateY).classed("bgsankey", !0).style({ fill: "transparent", "stroke-width": 0 });
              }), gt.transition().ease(se.ease).duration(se.duration).attr("transform", $e);
              var bt = gt.selectAll("." + se.cn.sankeyLinks).data(ye, pe);
              bt.enter().append("g").classed(se.cn.sankeyLinks, !0).style("fill", "none");
              var yt = bt.selectAll("." + se.cn.sankeyLink).data(function($t) {
                return $t.graph.links.filter(function(Pt) {
                  return Pt.value;
                }).map(xe.bind(null, $t));
              }, pe);
              yt.enter().append("path").classed(se.cn.sankeyLink, !0).call(qe, gt, et.linkEvents), yt.style("stroke", function($t) {
                return Pe($t) ? le.tinyRGB(ce($t.linkLineColor)) : $t.tinyColorHue;
              }).style("stroke-opacity", function($t) {
                return Pe($t) ? le.opacity($t.linkLineColor) : $t.tinyColorAlpha;
              }).style("fill", function($t) {
                return $t.tinyColorHue;
              }).style("fill-opacity", function($t) {
                return $t.tinyColorAlpha;
              }).style("stroke-width", function($t) {
                return Pe($t) ? $t.linkLineWidth : 1;
              }).attr("d", Le()), yt.style("opacity", function() {
                return ze._context.staticPlot || Je || it ? 1 : 0;
              }).transition().ease(se.ease).duration(se.duration).style("opacity", 1), yt.exit().transition().ease(se.ease).duration(se.duration).style("opacity", 0).remove();
              var St = gt.selectAll("." + se.cn.sankeyNodeSet).data(ye, pe);
              St.enter().append("g").classed(se.cn.sankeyNodeSet, !0), St.style("cursor", function($t) {
                switch ($t.arrangement) {
                  case "fixed":
                    return "default";
                  case "perpendicular":
                    return "ns-resize";
                  default:
                    return "move";
                }
              });
              var Lt = St.selectAll("." + se.cn.sankeyNode).data(function($t) {
                var Pt = $t.graph.nodes;
                return function(Xt) {
                  var Nt, Gt = [];
                  for (Nt = 0; Nt < Xt.length; Nt++) Xt[Nt].originalX = (Xt[Nt].x0 + Xt[Nt].x1) / 2, Xt[Nt].originalY = (Xt[Nt].y0 + Xt[Nt].y1) / 2, Gt.indexOf(Xt[Nt].originalX) === -1 && Gt.push(Xt[Nt].originalX);
                  for (Gt.sort(function(Ft, Tt) {
                    return Ft - Tt;
                  }), Nt = 0; Nt < Xt.length; Nt++) Xt[Nt].originalLayerIndex = Gt.indexOf(Xt[Nt].originalX), Xt[Nt].originalLayer = Xt[Nt].originalLayerIndex / (Gt.length - 1);
                }(Pt), Pt.map(Qe.bind(null, $t));
              }, pe);
              Lt.enter().append("g").classed(se.cn.sankeyNode, !0).call(ke).style("opacity", function($t) {
                return !ze._context.staticPlot && !Je || $t.partOfGroup ? 0 : 1;
              }), Lt.call(qe, gt, et.nodeEvents).call(Xe, yt, et, ze), Lt.transition().ease(se.ease).duration(se.duration).call(ke).style("opacity", function($t) {
                return $t.partOfGroup ? 0 : 1;
              }), Lt.exit().transition().ease(se.ease).duration(se.duration).style("opacity", 0).remove();
              var Rt = Lt.selectAll("." + se.cn.nodeRect).data(ye);
              Rt.enter().append("rect").classed(se.cn.nodeRect, !0).call(De), Rt.style("stroke-width", function($t) {
                return $t.nodeLineWidth;
              }).style("stroke", function($t) {
                return le.tinyRGB(ce($t.nodeLineColor));
              }).style("stroke-opacity", function($t) {
                return le.opacity($t.nodeLineColor);
              }).style("fill", function($t) {
                return $t.tinyColorHue;
              }).style("fill-opacity", function($t) {
                return $t.tinyColorAlpha;
              }), Rt.transition().ease(se.ease).duration(se.duration).call(De);
              var vt = Lt.selectAll("." + se.cn.nodeLabel).data(ye);
              vt.enter().append("text").classed(se.cn.nodeLabel, !0).style("cursor", "default"), vt.attr("data-notex", 1).text(function($t) {
                return $t.node.label;
              }).each(function($t) {
                var Pt = ee.select(this);
                ue.font(Pt, $t.textFont), ve.convertToTspans(Pt, ze);
              }).attr("text-anchor", function($t) {
                return $t.horizontal && $t.left ? "end" : "start";
              }).attr("transform", function($t) {
                var Pt = ee.select(this), Xt = ve.lineCount(Pt), Nt = $t.textFont.size * ((Xt - 1) * we - Se), Gt = $t.nodeLineWidth / 2 + 3, Ft = (($t.horizontal ? $t.visibleHeight : $t.visibleWidth) - Nt) / 2;
                $t.horizontal && ($t.left ? Gt = -Gt : Gt += $t.visibleWidth);
                var Tt = $t.horizontal ? "" : "scale(-1,1)" + he(90);
                return fe($t.horizontal ? Gt : Ft, $t.horizontal ? Ft : Gt) + Tt;
              }), vt.transition().ease(se.ease).duration(se.duration);
            };
          }, 74670: function(G) {
            G.exports = function(J, Z) {
              for (var te = [], ne = J.cd[0].trace, ee = ne._sankey.graph.nodes, ae = 0; ae < ee.length; ae++) {
                var oe = ee[ae];
                if (!oe.partOfGroup) {
                  var se = [(oe.x0 + oe.x1) / 2, (oe.y0 + oe.y1) / 2];
                  ne.orientation === "v" && se.reverse(), Z && Z.contains(se, !1, ae, J) && te.push({ pointNumber: oe.pointNumber });
                }
              }
              return te;
            };
          }, 99203: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              for (var ae = 0; ae < ne.length; ae++) ne[ae].i = ae;
              te.mergeArray(ee.text, ne, "tx"), te.mergeArray(ee.texttemplate, ne, "txt"), te.mergeArray(ee.hovertext, ne, "htx"), te.mergeArray(ee.customdata, ne, "data"), te.mergeArray(ee.textposition, ne, "tp"), ee.textfont && (te.mergeArrayCastPositive(ee.textfont.size, ne, "ts"), te.mergeArray(ee.textfont.color, ne, "tc"), te.mergeArray(ee.textfont.family, ne, "tf"), te.mergeArray(ee.textfont.weight, ne, "tw"), te.mergeArray(ee.textfont.style, ne, "ty"), te.mergeArray(ee.textfont.variant, ne, "tv"), te.mergeArray(ee.textfont.textcase, ne, "tC"), te.mergeArray(ee.textfont.lineposition, ne, "tE"), te.mergeArray(ee.textfont.shadow, ne, "tS"));
              var oe = ee.marker;
              if (oe) {
                te.mergeArrayCastPositive(oe.size, ne, "ms"), te.mergeArrayCastPositive(oe.opacity, ne, "mo"), te.mergeArray(oe.symbol, ne, "mx"), te.mergeArray(oe.angle, ne, "ma"), te.mergeArray(oe.standoff, ne, "mf"), te.mergeArray(oe.color, ne, "mc");
                var se = oe.line;
                oe.line && (te.mergeArray(se.color, ne, "mlc"), te.mergeArrayCastPositive(se.width, ne, "mlw"));
                var ce = oe.gradient;
                ce && ce.type !== "none" && (te.mergeArray(ce.type, ne, "mgt"), te.mergeArray(ce.color, ne, "mgc"));
              }
            };
          }, 36640: function(G, J, Z) {
            var te = Z(80712).axisHoverFormat, ne = Z(3208).ay, ee = Z(3208).rb, ae = Z(87163), oe = Z(80337), se = Z(94850).T, ce = Z(94850).k, le = Z(62203), ue = Z(32660), de = Z(93049).extendFlat, fe = Z(19326);
            G.exports = { x: { valType: "data_array", editType: "calc+clearAxisTypes", anim: !0 }, x0: { valType: "any", dflt: 0, editType: "calc+clearAxisTypes", anim: !0 }, dx: { valType: "number", dflt: 1, editType: "calc", anim: !0 }, y: { valType: "data_array", editType: "calc+clearAxisTypes", anim: !0 }, y0: { valType: "any", dflt: 0, editType: "calc+clearAxisTypes", anim: !0 }, dy: { valType: "number", dflt: 1, editType: "calc", anim: !0 }, xperiod: { valType: "any", dflt: 0, editType: "calc" }, yperiod: { valType: "any", dflt: 0, editType: "calc" }, xperiod0: { valType: "any", editType: "calc" }, yperiod0: { valType: "any", editType: "calc" }, xperiodalignment: { valType: "enumerated", values: ["start", "middle", "end"], dflt: "middle", editType: "calc" }, yperiodalignment: { valType: "enumerated", values: ["start", "middle", "end"], dflt: "middle", editType: "calc" }, xhoverformat: te("x"), yhoverformat: te("y"), offsetgroup: { valType: "string", dflt: "", editType: "calc" }, alignmentgroup: { valType: "string", dflt: "", editType: "calc" }, stackgroup: { valType: "string", dflt: "", editType: "calc" }, orientation: { valType: "enumerated", values: ["v", "h"], editType: "calc" }, groupnorm: { valType: "enumerated", values: ["", "fraction", "percent"], dflt: "", editType: "calc" }, stackgaps: { valType: "enumerated", values: ["infer zero", "interpolate"], dflt: "infer zero", editType: "calc" }, text: { valType: "string", dflt: "", arrayOk: !0, editType: "calc" }, texttemplate: ne({}, {}), hovertext: { valType: "string", dflt: "", arrayOk: !0, editType: "style" }, mode: { valType: "flaglist", flags: ["lines", "markers", "text"], extras: ["none"], editType: "calc" }, hoveron: { valType: "flaglist", flags: ["points", "fills"], editType: "style" }, hovertemplate: ee({}, { keys: ue.eventDataKeys }), line: { color: { valType: "color", editType: "style", anim: !0 }, width: { valType: "number", min: 0, dflt: 2, editType: "style", anim: !0 }, shape: { valType: "enumerated", values: ["linear", "spline", "hv", "vh", "hvh", "vhv"], dflt: "linear", editType: "plot" }, smoothing: { valType: "number", min: 0, max: 1.3, dflt: 1, editType: "plot" }, dash: de({}, se, { editType: "style" }), backoff: { valType: "number", min: 0, dflt: "auto", arrayOk: !0, editType: "plot" }, simplify: { valType: "boolean", dflt: !0, editType: "plot" }, editType: "plot" }, connectgaps: { valType: "boolean", dflt: !1, editType: "calc" }, cliponaxis: { valType: "boolean", dflt: !0, editType: "plot" }, fill: { valType: "enumerated", values: ["none", "tozeroy", "tozerox", "tonexty", "tonextx", "toself", "tonext"], editType: "calc" }, fillcolor: fe(!0), fillgradient: de({ type: { valType: "enumerated", values: ["radial", "horizontal", "vertical", "none"], dflt: "none", editType: "calc" }, start: { valType: "number", editType: "calc" }, stop: { valType: "number", editType: "calc" }, colorscale: { valType: "colorscale", editType: "style" }, editType: "calc" }), fillpattern: ce, marker: de({ symbol: { valType: "enumerated", values: le.symbolList, dflt: "circle", arrayOk: !0, editType: "style" }, opacity: { valType: "number", min: 0, max: 1, arrayOk: !0, editType: "style", anim: !0 }, angle: { valType: "angle", dflt: 0, arrayOk: !0, editType: "plot", anim: !1 }, angleref: { valType: "enumerated", values: ["previous", "up"], dflt: "up", editType: "plot", anim: !1 }, standoff: { valType: "number", min: 0, dflt: 0, arrayOk: !0, editType: "plot", anim: !0 }, size: { valType: "number", min: 0, dflt: 6, arrayOk: !0, editType: "calc", anim: !0 }, maxdisplayed: { valType: "number", min: 0, dflt: 0, editType: "plot" }, sizeref: { valType: "number", dflt: 1, editType: "calc" }, sizemin: { valType: "number", min: 0, dflt: 0, editType: "calc" }, sizemode: { valType: "enumerated", values: ["diameter", "area"], dflt: "diameter", editType: "calc" }, line: de({ width: { valType: "number", min: 0, arrayOk: !0, editType: "style", anim: !0 }, editType: "calc" }, ae("marker.line", { anim: !0 })), gradient: { type: { valType: "enumerated", values: ["radial", "horizontal", "vertical", "none"], arrayOk: !0, dflt: "none", editType: "calc" }, color: { valType: "color", arrayOk: !0, editType: "calc" }, editType: "calc" }, editType: "calc" }, ae("marker", { anim: !0 })), selected: { marker: { opacity: { valType: "number", min: 0, max: 1, editType: "style" }, color: { valType: "color", editType: "style" }, size: { valType: "number", min: 0, editType: "style" }, editType: "style" }, textfont: { color: { valType: "color", editType: "style" }, editType: "style" }, editType: "style" }, unselected: { marker: { opacity: { valType: "number", min: 0, max: 1, editType: "style" }, color: { valType: "color", editType: "style" }, size: { valType: "number", min: 0, editType: "style" }, editType: "style" }, textfont: { color: { valType: "color", editType: "style" }, editType: "style" }, editType: "style" }, textposition: { valType: "enumerated", values: ["top left", "top center", "top right", "middle left", "middle center", "middle right", "bottom left", "bottom center", "bottom right"], dflt: "middle center", arrayOk: !0, editType: "calc" }, textfont: oe({ editType: "calc", colorEditType: "style", arrayOk: !0 }), zorder: { valType: "integer", dflt: 0, editType: "plot" } };
          }, 26544: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(29714), ae = Z(40528), oe = Z(63821).BADNUM, se = Z(64726), ce = Z(77272), le = Z(99203), ue = Z(48861);
            function de(ye, ge, ve, be, Ce, Se, we) {
              var Ee = ge._length, xe = ye._fullLayout, Le = ve._id, Qe = be._id, ke = xe._firstScatter[me(ge)] === ge.uid, Me = (pe(ge, xe, ve, be) || {}).orientation, Re = ge.fill;
              ve._minDtick = 0, be._minDtick = 0;
              var De = { padded: !0 }, Pe = { padded: !0 };
              we && (De.ppad = Pe.ppad = we);
              var $e = Ee < 2 || Ce[0] !== Ce[Ee - 1] || Se[0] !== Se[Ee - 1];
              $e && (Re === "tozerox" || Re === "tonextx" && (ke || Me === "h")) ? De.tozero = !0 : (ge.error_y || {}).visible || Re !== "tonexty" && Re !== "tozeroy" && (se.hasMarkers(ge) || se.hasText(ge)) || (De.padded = !1, De.ppad = 0), $e && (Re === "tozeroy" || Re === "tonexty" && (ke || Me === "v")) ? Pe.tozero = !0 : Re !== "tonextx" && Re !== "tozerox" || (Pe.padded = !1), Le && (ge._extremes[Le] = ee.findExtremes(ve, Ce, De)), Qe && (ge._extremes[Qe] = ee.findExtremes(be, Se, Pe));
            }
            function fe(ye, ge) {
              if (se.hasMarkers(ye)) {
                var ve, be = ye.marker, Ce = 1.6 * (ye.marker.sizeref || 1);
                if (ve = ye.marker.sizemode === "area" ? function(Le) {
                  return Math.max(Math.sqrt((Le || 0) / Ce), 3);
                } : function(Le) {
                  return Math.max((Le || 0) / Ce, 3);
                }, ne.isArrayOrTypedArray(be.size)) {
                  var Se = { type: "linear" };
                  ee.setConvert(Se);
                  for (var we = Se.makeCalcdata(ye.marker, "size"), Ee = new Array(ge), xe = 0; xe < ge; xe++) Ee[xe] = ve(we[xe]);
                  return Ee;
                }
                return ve(be.size);
              }
            }
            function he(ye, ge) {
              var ve = me(ge), be = ye._firstScatter;
              be[ve] || (be[ve] = ge.uid);
            }
            function me(ye) {
              var ge = ye.stackgroup;
              return ye.xaxis + ye.yaxis + ye.type + (ge ? "-" + ge : "");
            }
            function pe(ye, ge, ve, be) {
              var Ce = ye.stackgroup;
              if (Ce) {
                var Se = ge._scatterStackOpts[ve._id + be._id][Ce], we = Se.orientation === "v" ? be : ve;
                return we.type === "linear" || we.type === "log" ? Se : void 0;
              }
            }
            G.exports = { calc: function(ye, ge) {
              var ve, be, Ce, Se, we, Ee, xe = ye._fullLayout, Le = ge._xA = ee.getFromId(ye, ge.xaxis || "x", "x"), Qe = ge._yA = ee.getFromId(ye, ge.yaxis || "y", "y"), ke = Le.makeCalcdata(ge, "x"), Me = Qe.makeCalcdata(ge, "y"), Re = ae(ge, Le, "x", ke), De = ae(ge, Qe, "y", Me), Pe = Re.vals, $e = De.vals, qe = ge._length, Xe = new Array(qe), Ve = ge.ids, Ze = pe(ge, xe, Le, Qe), Ge = !1;
              he(xe, ge);
              var ze, We = "x", at = "y";
              Ze ? (ne.pushUnique(Ze.traceIndices, ge._expandedIndex), (ve = Ze.orientation === "v") ? (at = "s", ze = "x") : (We = "s", ze = "y"), we = Ze.stackgaps === "interpolate") : de(ye, ge, Le, Qe, Pe, $e, fe(ge, qe));
              var ct = !!ge.xperiodalignment, et = !!ge.yperiodalignment;
              for (be = 0; be < qe; be++) {
                var dt = Xe[be] = {}, Je = te(Pe[be]), it = te($e[be]);
                Je && it ? (dt[We] = Pe[be], dt[at] = $e[be], ct && (dt.orig_x = ke[be], dt.xEnd = Re.ends[be], dt.xStart = Re.starts[be]), et && (dt.orig_y = Me[be], dt.yEnd = De.ends[be], dt.yStart = De.starts[be])) : Ze && (ve ? Je : it) ? (dt[ze] = ve ? Pe[be] : $e[be], dt.gap = !0, we ? (dt.s = oe, Ge = !0) : dt.s = 0) : dt[We] = dt[at] = oe, Ve && (dt.id = String(Ve[be]));
              }
              if (le(Xe, ge), ce(ye, ge), ue(Xe, ge), Ze) {
                for (be = 0; be < Xe.length; ) Xe[be][ze] === oe ? Xe.splice(be, 1) : be++;
                if (ne.sort(Xe, function(yt, St) {
                  return yt[ze] - St[ze] || yt.i - St.i;
                }), Ge) {
                  for (be = 0; be < Xe.length - 1 && Xe[be].gap; ) be++;
                  for ((Ee = Xe[be].s) || (Ee = Xe[be].s = 0), Ce = 0; Ce < be; Ce++) Xe[Ce].s = Ee;
                  for (Se = Xe.length - 1; Se > be && Xe[Se].gap; ) Se--;
                  for (Ee = Xe[Se].s, Ce = Xe.length - 1; Ce > Se; Ce--) Xe[Ce].s = Ee;
                  for (; be < Se; ) if (Xe[++be].gap) {
                    for (Ce = be + 1; Xe[Ce].gap; ) Ce++;
                    for (var st = Xe[be - 1][ze], gt = Xe[be - 1].s, bt = (Xe[Ce].s - gt) / (Xe[Ce][ze] - st); be < Ce; ) Xe[be].s = gt + (Xe[be][ze] - st) * bt, be++;
                  }
                }
              }
              return Xe;
            }, calcMarkerSize: fe, calcAxisExpansion: de, setFirstScatter: he, getStackOpts: pe };
          }, 48861: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              te.isArrayOrTypedArray(ee.selectedpoints) && te.tagSelected(ne, ee);
            };
          }, 77272: function(G, J, Z) {
            var te = Z(65477).hasColorscale, ne = Z(28379), ee = Z(64726);
            G.exports = function(ae, oe) {
              ee.hasLines(oe) && te(oe, "line") && ne(ae, oe, { vals: oe.line.color, containerStr: "line", cLetter: "c" }), ee.hasMarkers(oe) && (te(oe, "marker") && ne(ae, oe, { vals: oe.marker.color, containerStr: "marker", cLetter: "c" }), te(oe, "marker.line") && ne(ae, oe, { vals: oe.marker.line.color, containerStr: "marker.line", cLetter: "c" }));
            };
          }, 32660: function(G) {
            G.exports = { PTS_LINESONLY: 20, minTolerance: 0.2, toleranceGrowth: 10, maxScreensAway: 20, eventDataKeys: [] };
          }, 75603: function(G, J, Z) {
            var te = Z(26544), ne = Z(24782).setGroupPositions;
            function ee(ae, oe, se, ce, le, ue, de) {
              le[ce] = !0;
              var fe = { i: null, gap: !0, s: 0 };
              if (fe[de] = se, ae.splice(oe, 0, fe), oe && se === ae[oe - 1][de]) {
                var he = ae[oe - 1];
                fe.s = he.s, fe.i = he.i, fe.gap = he.gap;
              } else ue && (fe.s = function(me, pe, ye, ge) {
                var ve = me[pe - 1], be = me[pe + 1];
                return be ? ve ? ve.s + (be.s - ve.s) * (ye - ve[ge]) / (be[ge] - ve[ge]) : be.s : ve.s;
              }(ae, oe, se, de));
              oe || (ae[0].t = ae[1].t, ae[0].trace = ae[1].trace, delete ae[1].t, delete ae[1].trace);
            }
            G.exports = function(ae, oe) {
              ae._fullLayout.scattermode === "group" && function(ze, We) {
                for (var at = We.xaxis, ct = We.yaxis, et = ze._fullLayout, dt = ze._fullData, Je = ze.calcdata, it = [], st = [], gt = 0; gt < dt.length; gt++) {
                  var bt = dt[gt];
                  bt.visible === !0 && bt.type === "scatter" && bt.xaxis === at._id && bt.yaxis === ct._id && (bt.orientation === "h" ? it.push(Je[gt]) : bt.orientation === "v" && st.push(Je[gt]));
                }
                var yt = { mode: et.scattermode, gap: et.scattergap };
                ne(ze, at, ct, st, yt), ne(ze, ct, at, it, yt);
              }(ae, oe);
              var se = oe.xaxis, ce = oe.yaxis, le = se._id + ce._id, ue = ae._fullLayout._scatterStackOpts[le];
              if (ue) {
                var de, fe, he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe = ae.calcdata;
                for (var ke in ue) {
                  var Me = (Ce = ue[ke]).traceIndices;
                  if (Me.length) {
                    for (Se = Ce.stackgaps === "interpolate", we = Ce.groupnorm, Ce.orientation === "v" ? (Ee = "x", xe = "y") : (Ee = "y", xe = "x"), Le = new Array(Me.length), de = 0; de < Le.length; de++) Le[de] = !1;
                    ye = Qe[Me[0]];
                    var Re = new Array(ye.length);
                    for (de = 0; de < ye.length; de++) Re[de] = ye[de][Ee];
                    for (de = 1; de < Me.length; de++) {
                      for (pe = Qe[Me[de]], fe = he = 0; fe < pe.length; fe++) {
                        for (ge = pe[fe][Ee]; ge > Re[he] && he < Re.length; he++) ee(pe, fe, Re[he], de, Le, Se, Ee), fe++;
                        if (ge !== Re[he]) {
                          for (me = 0; me < de; me++) ee(Qe[Me[me]], he, ge, me, Le, Se, Ee);
                          Re.splice(he, 0, ge);
                        }
                        he++;
                      }
                      for (; he < Re.length; he++) ee(pe, fe, Re[he], de, Le, Se, Ee), fe++;
                    }
                    var De = Re.length;
                    for (fe = 0; fe < ye.length; fe++) {
                      for (ve = ye[fe][xe] = ye[fe].s, de = 1; de < Me.length; de++) (pe = Qe[Me[de]])[0].trace._rawLength = pe[0].trace._length, pe[0].trace._length = De, ve += pe[fe].s, pe[fe][xe] = ve;
                      if (we) for (be = (we === "fraction" ? ve : ve / 100) || 1, de = 0; de < Me.length; de++) {
                        var Pe = Qe[Me[de]][fe];
                        Pe[xe] /= be, Pe.sNorm = Pe.s / be;
                      }
                    }
                    for (de = 0; de < Me.length; de++) {
                      var $e = (pe = Qe[Me[de]])[0].trace, qe = te.calcMarkerSize($e, $e._rawLength), Xe = Array.isArray(qe);
                      if (qe && Le[de] || Xe) {
                        var Ve = qe;
                        for (qe = new Array(De), fe = 0; fe < De; fe++) qe[fe] = pe[fe].gap ? 0 : Xe ? Ve[pe[fe].i] : Ve;
                      }
                      var Ze = new Array(De), Ge = new Array(De);
                      for (fe = 0; fe < De; fe++) Ze[fe] = pe[fe].x, Ge[fe] = pe[fe].y;
                      te.calcAxisExpansion(ae, $e, se, ce, Ze, Ge, qe), pe[0].t.orientation = Ce.orientation;
                    }
                  }
                }
              }
            };
          }, 53044: function(G, J, Z) {
            var te = Z(34809), ne = Z(36301), ee = Z(36640);
            G.exports = function(ae, oe) {
              var se, ce, le;
              function ue(pe) {
                return te.coerce(ce._input, ce, ee, pe);
              }
              if (oe.scattermode === "group") for (le = 0; le < ae.length; le++) (ce = ae[le]).type === "scatter" && (se = ce._input, ne(se, ce, oe, ue));
              for (le = 0; le < ae.length; le++) {
                var de = ae[le];
                if (de.type === "scatter") {
                  var fe = de.fill;
                  if (fe !== "none" && fe !== "toself" && (de.opacity = void 0, fe === "tonexty" || fe === "tonextx")) for (var he = le - 1; he >= 0; he--) {
                    var me = ae[he];
                    if (me.type === "scatter" && me.xaxis === de.xaxis && me.yaxis === de.yaxis) {
                      me.opacity = void 0;
                      break;
                    }
                  }
                }
              }
            };
          }, 40247: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626), ee = Z(36640), ae = Z(32660), oe = Z(64726), se = Z(99867), ce = Z(99669), le = Z(382), ue = Z(24272), de = Z(98168), fe = Z(91602), he = Z(663), me = Z(54114), pe = Z(34809).coercePattern;
            G.exports = function(ye, ge, ve, be) {
              function Ce(Me, Re) {
                return te.coerce(ye, ge, ee, Me, Re);
              }
              var Se = se(ye, ge, be, Ce);
              if (Se || (ge.visible = !1), ge.visible) {
                ce(ye, ge, be, Ce), Ce("xhoverformat"), Ce("yhoverformat"), Ce("zorder");
                var we = le(ye, ge, be, Ce);
                be.scattermode === "group" && ge.orientation === void 0 && Ce("orientation", "v");
                var Ee = !we && Se < ae.PTS_LINESONLY ? "lines+markers" : "lines";
                Ce("text"), Ce("hovertext"), Ce("mode", Ee), oe.hasMarkers(ge) && ue(ye, ge, ve, be, Ce, { gradient: !0 }), oe.hasLines(ge) && (de(ye, ge, ve, be, Ce, { backoff: !0 }), fe(ye, ge, Ce), Ce("connectgaps"), Ce("line.simplify")), oe.hasText(ge) && (Ce("texttemplate"), he(ye, ge, be, Ce));
                var xe = [];
                (oe.hasMarkers(ge) || oe.hasText(ge)) && (Ce("cliponaxis"), Ce("marker.maxdisplayed"), xe.push("points")), Ce("fill", we ? we.fillDflt : "none"), ge.fill !== "none" && (me(ye, ge, ve, Ce, { moduleHasFillgradient: !0 }), oe.hasLines(ge) || fe(ye, ge, Ce), pe(Ce, "fillpattern", ge.fillcolor, !1));
                var Le = (ge.line || {}).color, Qe = (ge.marker || {}).color;
                ge.fill !== "tonext" && ge.fill !== "toself" || xe.push("fills"), Ce("hoveron", xe.join("+") || "points"), ge.hoveron !== "fills" && Ce("hovertemplate");
                var ke = ne.getComponentMethod("errorbars", "supplyDefaults");
                ke(ye, ge, Le || Qe || ve, { axis: "y" }), ke(ye, ge, Le || Qe || ve, { axis: "x", inherit: "y" }), te.coerceSelectionMarkerOpacity(ge, Ce);
              }
            };
          }, 19326: function(G) {
            G.exports = function(J) {
              return { valType: "color", editType: "style", anim: !0 };
            };
          }, 54114: function(G, J, Z) {
            var te = Z(78766), ne = Z(34809).isArrayOrTypedArray;
            G.exports = function(ee, ae, oe, se, ce) {
              ce || (ce = {});
              var le, ue = !1;
              if (ae.marker) {
                var de = ae.marker.color, fe = (ae.marker.line || {}).color;
                de && !ne(de) ? ue = de : fe && !ne(fe) && (ue = fe);
              }
              if (ce.moduleHasFillgradient && se("fillgradient.type") !== "none") {
                se("fillgradient.start"), se("fillgradient.stop");
                var he = se("fillgradient.colorscale");
                he && (le = function(me) {
                  for (var pe = te.interpolate(me[0][1], me[1][1], 0.5), ye = 2; ye < me.length; ye++) {
                    var ge = te.interpolate(me[ye - 1][1], me[ye][1], 0.5);
                    pe = te.interpolate(pe, ge, me[ye - 1][0] / me[ye][0]);
                  }
                  return pe;
                }(he));
              }
              se("fillcolor", te.addOpacity((ae.line || {}).color || ue || le || oe, 0.5));
            };
          }, 15294: function(G, J, Z) {
            var te = Z(29714);
            G.exports = function(ne, ee, ae) {
              var oe = {}, se = { _fullLayout: ae }, ce = te.getFromTrace(se, ee, "x"), le = te.getFromTrace(se, ee, "y"), ue = ne.orig_x;
              ue === void 0 && (ue = ne.x);
              var de = ne.orig_y;
              return de === void 0 && (de = ne.y), oe.xLabel = te.tickText(ce, ce.c2l(ue), !0).text, oe.yLabel = te.tickText(le, le.c2l(de), !0).text, oe;
            };
          }, 11539: function(G, J, Z) {
            var te = Z(78766), ne = Z(64726);
            G.exports = function(ee, ae) {
              var oe, se;
              if (ee.mode === "lines") return (oe = ee.line.color) && te.opacity(oe) ? oe : ee.fillcolor;
              if (ee.mode === "none") return ee.fill ? ee.fillcolor : "";
              var ce = ae.mcc || (ee.marker || {}).color, le = ae.mlcc || ((ee.marker || {}).line || {}).color;
              return (se = ce && te.opacity(ce) ? ce : le && te.opacity(le) && (ae.mlw || ((ee.marker || {}).line || {}).width) ? le : "") ? te.opacity(se) < 0.3 ? te.addOpacity(se, 0.3) : se : (oe = (ee.line || {}).color) && te.opacity(oe) && ne.hasLines(ee) && ee.line.width ? oe : ee.fillcolor;
            };
          }, 36301: function(G, J, Z) {
            var te = Z(84391).getAxisGroup;
            G.exports = function(ne, ee, ae, oe) {
              var se = ee.orientation, ce = ee[{ v: "x", h: "y" }[se] + "axis"], le = te(ae, ce) + se, ue = ae._alignmentOpts || {}, de = oe("alignmentgroup"), fe = ue[le];
              fe || (fe = ue[le] = {});
              var he = fe[de];
              he ? he.traces.push(ee) : he = fe[de] = { traces: [ee], alignmentIndex: Object.keys(fe).length, offsetGroups: {} };
              var me = oe("offsetgroup"), pe = he.offsetGroups, ye = pe[me];
              me && (ye || (ye = pe[me] = { offsetIndex: Object.keys(pe).length }), ee._offsetIndex = ye.offsetIndex);
            };
          }, 37255: function(G, J, Z) {
            var te = Z(34809), ne = Z(32141), ee = Z(33626), ae = Z(11539), oe = Z(78766), se = te.fillText;
            G.exports = function(ce, le, ue, de) {
              var fe = ce.cd, he = fe[0].trace, me = ce.xa, pe = ce.ya, ye = me.c2p(le), ge = pe.c2p(ue), ve = [ye, ge], be = he.hoveron || "", Ce = he.mode.indexOf("markers") !== -1 ? 3 : 0.5, Se = !!he.xperiodalignment, we = !!he.yperiodalignment;
              if (be.indexOf("points") !== -1) {
                var Ee = function(Ze) {
                  var Ge = Math.max(Ce, Ze.mrc || 0), ze = me.c2p(Ze.x) - ye, We = pe.c2p(Ze.y) - ge;
                  return Math.max(Math.sqrt(ze * ze + We * We) - Ge, 1 - Ce / Ge);
                }, xe = ne.getDistanceFunction(de, function(Ze) {
                  if (Se) {
                    var Ge = me.c2p(Ze.xStart), ze = me.c2p(Ze.xEnd);
                    return ye >= Math.min(Ge, ze) && ye <= Math.max(Ge, ze) ? 0 : 1 / 0;
                  }
                  var We = Math.max(3, Ze.mrc || 0), at = 1 - 1 / We, ct = Math.abs(me.c2p(Ze.x) - ye);
                  return ct < We ? at * ct / We : ct - We + at;
                }, function(Ze) {
                  if (we) {
                    var Ge = pe.c2p(Ze.yStart), ze = pe.c2p(Ze.yEnd);
                    return ge >= Math.min(Ge, ze) && ge <= Math.max(Ge, ze) ? 0 : 1 / 0;
                  }
                  var We = Math.max(3, Ze.mrc || 0), at = 1 - 1 / We, ct = Math.abs(pe.c2p(Ze.y) - ge);
                  return ct < We ? at * ct / We : ct - We + at;
                }, Ee);
                if (ne.getClosest(fe, xe, ce), ce.index !== !1) {
                  var Le = fe[ce.index], Qe = me.c2p(Le.x, !0), ke = pe.c2p(Le.y, !0), Me = Le.mrc || 1;
                  ce.index = Le.i;
                  var Re = fe[0].t.orientation, De = Re && (Le.sNorm || Le.s), Pe = Re === "h" ? De : Le.orig_x !== void 0 ? Le.orig_x : Le.x, $e = Re === "v" ? De : Le.orig_y !== void 0 ? Le.orig_y : Le.y;
                  return te.extendFlat(ce, { color: ae(he, Le), x0: Qe - Me, x1: Qe + Me, xLabelVal: Pe, y0: ke - Me, y1: ke + Me, yLabelVal: $e, spikeDistance: Ee(Le), hovertemplate: he.hovertemplate }), se(Le, he, ce), ee.getComponentMethod("errorbars", "hoverInfo")(Le, he, ce), [ce];
                }
              }
              function qe(Ze) {
                if (!Ze) return !1;
                var Ge = Ze.node();
                try {
                  var ze = new DOMPoint(ve[0], ve[1]);
                  return Ge.isPointInFill(ze);
                } catch {
                  var We = Ge.ownerSVGElement.createSVGPoint();
                  return We.x = ve[0], We.y = ve[1], Ge.isPointInFill(We);
                }
              }
              if (be.indexOf("fills") !== -1 && he._fillElement && qe(he._fillElement) && !qe(he._fillExclusionElement)) {
                var Xe = function(Ze) {
                  var Ge, ze, We, at, ct, et, dt, Je, it, st = [], gt = 1 / 0, bt = -1 / 0, yt = 1 / 0, St = -1 / 0;
                  for (Ge = 0; Ge < Ze.length; Ge++) {
                    var Lt = Ze[Ge];
                    Lt.contains(ve) && (st.push(Lt), yt = Math.min(yt, Lt.ymin), St = Math.max(St, Lt.ymax));
                  }
                  if (st.length === 0) return null;
                  for (ze = ((yt = Math.max(yt, 0)) + (St = Math.min(St, pe._length))) / 2, Ge = 0; Ge < st.length; Ge++) for (at = st[Ge].pts, We = 1; We < at.length; We++) (Je = at[We - 1][1]) > ze != (it = at[We][1]) >= ze && (et = at[We - 1][0], dt = at[We][0], it - Je && (ct = et + (dt - et) * (ze - Je) / (it - Je), gt = Math.min(gt, ct), bt = Math.max(bt, ct)));
                  return { x0: gt = Math.max(gt, 0), x1: bt = Math.min(bt, me._length), y0: ze, y1: ze };
                }(he._polygons);
                Xe === null && (Xe = { x0: ve[0], x1: ve[0], y0: ve[1], y1: ve[1] });
                var Ve = oe.defaultLine;
                return oe.opacity(he.fillcolor) ? Ve = he.fillcolor : oe.opacity((he.line || {}).color) && (Ve = he.line.color), te.extendFlat(ce, { distance: ce.maxHoverDistance, x0: Xe.x0, x1: Xe.x1, y0: Xe.y0, y1: Xe.y1, color: Ve, hovertemplate: !1 }), delete ce.index, he.text && !te.isArrayOrTypedArray(he.text) ? ce.text = String(he.text) : ce.text = he.name, [ce];
              }
            };
          }, 69693: function(G, J, Z) {
            var te = Z(64726);
            G.exports = { hasLines: te.hasLines, hasMarkers: te.hasMarkers, hasText: te.hasText, isBubble: te.isBubble, attributes: Z(36640), layoutAttributes: Z(26667), supplyDefaults: Z(40247), crossTraceDefaults: Z(53044), supplyLayoutDefaults: Z(12332), calc: Z(26544).calc, crossTraceCalc: Z(75603), arraysToCalcdata: Z(99203), plot: Z(36098), colorbar: Z(21146), formatLabels: Z(15294), style: Z(9408).style, styleOnSelect: Z(9408).styleOnSelect, hoverPoints: Z(37255), selectPoints: Z(32665), animatable: !0, moduleType: "trace", name: "scatter", basePlotModule: Z(37703), categories: ["cartesian", "svg", "symbols", "errorBarsOK", "showLegend", "scatter-like", "zoomScale"], meta: {} };
          }, 26667: function(G) {
            G.exports = { scattermode: { valType: "enumerated", values: ["group", "overlay"], dflt: "overlay", editType: "calc" }, scattergap: { valType: "number", min: 0, max: 1, editType: "calc" } };
          }, 12332: function(G, J, Z) {
            var te = Z(34809), ne = Z(26667);
            G.exports = function(ee, ae) {
              var oe, se = ae.barmode === "group";
              ae.scattermode === "group" && (oe = se ? ae.bargap : 0.2, te.coerce(ee, ae, ne, "scattergap", oe));
            };
          }, 98168: function(G, J, Z) {
            var te = Z(34809).isArrayOrTypedArray, ne = Z(65477).hasColorscale, ee = Z(39356);
            G.exports = function(ae, oe, se, ce, le, ue) {
              ue || (ue = {});
              var de = (ae.marker || {}).color;
              de && de._inputArray && (de = de._inputArray), le("line.color", se), ne(ae, "line") ? ee(ae, oe, ce, le, { prefix: "line.", cLetter: "c" }) : le("line.color", !te(de) && de || se), le("line.width"), ue.noDash || le("line.dash"), ue.backoff && le("line.backoff");
            };
          }, 5525: function(G, J, Z) {
            var te = Z(62203), ne = Z(63821), ee = ne.BADNUM, ae = ne.LOG_CLIP, oe = ae + 0.5, se = ae - 0.5, ce = Z(34809), le = ce.segmentsIntersect, ue = ce.constrain, de = Z(32660);
            G.exports = function(fe, he) {
              var me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e = he.trace || {}, qe = he.xaxis, Xe = he.yaxis, Ve = qe.type === "log", Ze = Xe.type === "log", Ge = qe._length, ze = Xe._length, We = he.backoff, at = $e.marker, ct = he.connectGaps, et = he.baseTolerance, dt = he.shape, Je = dt === "linear", it = $e.fill && $e.fill !== "none", st = [], gt = de.minTolerance, bt = fe.length, yt = new Array(bt), St = 0;
              function Lt(Bt) {
                var At = fe[Bt];
                if (!At) return !1;
                var Dt = he.linearized ? qe.l2p(At.x) : qe.c2p(At.x), Ot = he.linearized ? Xe.l2p(At.y) : Xe.c2p(At.y);
                if (Dt === ee) {
                  if (Ve && (Dt = qe.c2p(At.x, !0)), Dt === ee) return !1;
                  Ze && Ot === ee && (Dt *= Math.abs(qe._m * ze * (qe._m > 0 ? oe : se) / (Xe._m * Ge * (Xe._m > 0 ? oe : se)))), Dt *= 1e3;
                }
                if (Ot === ee) {
                  if (Ze && (Ot = Xe.c2p(At.y, !0)), Ot === ee) return !1;
                  Ot *= 1e3;
                }
                return [Dt, Ot];
              }
              function Rt(Bt, At, Dt, Ot) {
                var un = Dt - Bt, yn = Ot - At, _n = 0.5 - Bt, gn = 0.5 - At, bn = un * un + yn * yn, an = un * _n + yn * gn;
                if (an > 0 && an < bn) {
                  var Zt = _n * yn - gn * un;
                  if (Zt * Zt < bn) return !0;
                }
              }
              function vt(Bt, At) {
                var Dt = Bt[0] / Ge, Ot = Bt[1] / ze, un = Math.max(0, -Dt, Dt - 1, -Ot, Ot - 1);
                return un && De !== void 0 && Rt(Dt, Ot, De, Pe) && (un = 0), un && At && Rt(Dt, Ot, At[0] / Ge, At[1] / ze) && (un = 0), (1 + de.toleranceGrowth * un) * et;
              }
              function $t(Bt, At) {
                var Dt = Bt[0] - At[0], Ot = Bt[1] - At[1];
                return Math.sqrt(Dt * Dt + Ot * Ot);
              }
              var Pt, Xt, Nt, Gt, Ft, Tt, It, Ct = de.maxScreensAway, Kt = -Ge * Ct, Vt = Ge * (1 + Ct), Yt = -ze * Ct, sn = ze * (1 + Ct), hn = [[Kt, Yt, Vt, Yt], [Vt, Yt, Vt, sn], [Vt, sn, Kt, sn], [Kt, sn, Kt, Yt]];
              function Qt(Bt) {
                if (Bt[0] < Kt || Bt[0] > Vt || Bt[1] < Yt || Bt[1] > sn) return [ue(Bt[0], Kt, Vt), ue(Bt[1], Yt, sn)];
              }
              function Ut(Bt, At) {
                return Bt[0] === At[0] && (Bt[0] === Kt || Bt[0] === Vt) || Bt[1] === At[1] && (Bt[1] === Yt || Bt[1] === sn) || void 0;
              }
              function Wt(Bt, At, Dt) {
                return function(Ot, un) {
                  var yn = Qt(Ot), _n = Qt(un), gn = [];
                  if (yn && _n && Ut(yn, _n)) return gn;
                  yn && gn.push(yn), _n && gn.push(_n);
                  var bn = 2 * ce.constrain((Ot[Bt] + un[Bt]) / 2, At, Dt) - ((yn || Ot)[Bt] + (_n || un)[Bt]);
                  return bn && ((yn && _n ? bn > 0 == yn[Bt] > _n[Bt] ? yn : _n : yn || _n)[Bt] += bn), gn;
                };
              }
              function rn(Bt) {
                var At = Bt[0], Dt = Bt[1], Ot = At === yt[St - 1][0], un = Dt === yt[St - 1][1];
                if (!Ot || !un) if (St > 1) {
                  var yn = At === yt[St - 2][0], _n = Dt === yt[St - 2][1];
                  Ot && (At === Kt || At === Vt) && yn ? _n ? St-- : yt[St - 1] = Bt : un && (Dt === Yt || Dt === sn) && _n ? yn ? St-- : yt[St - 1] = Bt : yt[St++] = Bt;
                } else yt[St++] = Bt;
              }
              function vn(Bt) {
                yt[St - 1][0] !== Bt[0] && yt[St - 1][1] !== Bt[1] && rn([Nt, Gt]), rn(Bt), Ft = null, Nt = Gt = 0;
              }
              dt === "linear" || dt === "spline" ? It = function(Bt, At) {
                for (var Dt = [], Ot = 0, un = 0; un < 4; un++) {
                  var yn = hn[un], _n = le(Bt[0], Bt[1], At[0], At[1], yn[0], yn[1], yn[2], yn[3]);
                  _n && (!Ot || Math.abs(_n.x - Dt[0][0]) > 1 || Math.abs(_n.y - Dt[0][1]) > 1) && (_n = [_n.x, _n.y], Ot && $t(_n, Bt) < $t(Dt[0], Bt) ? Dt.unshift(_n) : Dt.push(_n), Ot++);
                }
                return Dt;
              } : dt === "hv" || dt === "vh" ? It = function(Bt, At) {
                var Dt = [], Ot = Qt(Bt), un = Qt(At);
                return Ot && un && Ut(Ot, un) || (Ot && Dt.push(Ot), un && Dt.push(un)), Dt;
              } : dt === "hvh" ? It = Wt(0, Kt, Vt) : dt === "vhv" && (It = Wt(1, Yt, sn));
              var dn = ce.isArrayOrTypedArray(at);
              function Ht(Bt) {
                if (Bt && We && (Bt.i = me, Bt.d = fe, Bt.trace = $e, Bt.marker = dn ? at[Bt.i] : at, Bt.backoff = We), De = Bt[0] / Ge, Pe = Bt[1] / ze, Pt = Bt[0] < Kt ? Kt : Bt[0] > Vt ? Vt : 0, Xt = Bt[1] < Yt ? Yt : Bt[1] > sn ? sn : 0, Pt || Xt) {
                  if (St) if (Ft) {
                    var At = It(Ft, Bt);
                    At.length > 1 && (vn(At[0]), yt[St++] = At[1]);
                  } else Tt = It(yt[St - 1], Bt)[0], yt[St++] = Tt;
                  else yt[St++] = [Pt || Bt[0], Xt || Bt[1]];
                  var Dt = yt[St - 1];
                  Pt && Xt && (Dt[0] !== Pt || Dt[1] !== Xt) ? (Ft && (Nt !== Pt && Gt !== Xt ? rn(Nt && Gt ? (Ot = Ft, yn = (un = Bt)[0] - Ot[0], _n = (un[1] - Ot[1]) / yn, (Ot[1] * un[0] - un[1] * Ot[0]) / yn > 0 ? [_n > 0 ? Kt : Vt, sn] : [_n > 0 ? Vt : Kt, Yt]) : [Nt || Pt, Gt || Xt]) : Nt && Gt && rn([Nt, Gt])), rn([Pt, Xt])) : Nt - Pt && Gt - Xt && rn([Pt || Nt, Xt || Gt]), Ft = Bt, Nt = Pt, Gt = Xt;
                } else Ft && vn(It(Ft, Bt)[0]), yt[St++] = Bt;
                var Ot, un, yn, _n;
              }
              for (me = 0; me < bt; me++) if (pe = Lt(me)) {
                for (St = 0, Ft = null, Ht(pe), me++; me < bt; me++) {
                  if (!(ge = Lt(me))) {
                    if (ct) continue;
                    break;
                  }
                  if (Je && he.simplify) {
                    var jt = Lt(me + 1);
                    if (Ee = $t(ge, pe), it && (St === 0 || St === bt - 1) || !(Ee < vt(ge, jt) * gt)) {
                      for (Se = [(ge[0] - pe[0]) / Ee, (ge[1] - pe[1]) / Ee], ve = pe, xe = Ee, Le = ke = Me = 0, Ce = !1, ye = ge, me++; me < fe.length; me++) {
                        if (be = jt, jt = Lt(me + 1), !be) {
                          if (ct) continue;
                          break;
                        }
                        if (Re = (we = [be[0] - pe[0], be[1] - pe[1]])[0] * Se[1] - we[1] * Se[0], ke = Math.min(ke, Re), (Me = Math.max(Me, Re)) - ke > vt(be, jt)) break;
                        ye = be, (Qe = we[0] * Se[0] + we[1] * Se[1]) > xe ? (xe = Qe, ge = be, Ce = !1) : Qe < Le && (Le = Qe, ve = be, Ce = !0);
                      }
                      if (Ce ? (Ht(ge), ye !== ve && Ht(ve)) : (ve !== pe && Ht(ve), ye !== ge && Ht(ge)), Ht(ye), me >= fe.length || !be) break;
                      Ht(be), pe = be;
                    }
                  } else Ht(ge);
                }
                Ft && rn([Nt || Ft[0], Gt || Ft[1]]), st.push(yt.slice(0, St));
              }
              var cn = dt.slice(dt.length - 1);
              if (We && cn !== "h" && cn !== "v") {
                for (var wn = !1, Rn = -1, Pn = [], Ie = 0; Ie < st.length; Ie++) for (var rt = 0; rt < st[Ie].length - 1; rt++) {
                  var ft = st[Ie][rt], pt = st[Ie][rt + 1], wt = te.applyBackoff(pt, ft);
                  wt[0] === pt[0] && wt[1] === pt[1] || (wn = !0), Pn[Rn + 1] || (Pn[++Rn] = [ft, [wt[0], wt[1]]]);
                }
                return wn ? Pn : st;
              }
              return st;
            };
          }, 91602: function(G) {
            G.exports = function(J, Z, te) {
              te("line.shape") === "spline" && te("line.smoothing");
            };
          }, 17210: function(G) {
            var J = { tonextx: 1, tonexty: 1, tonext: 1 };
            G.exports = function(Z, te, ne) {
              var ee, ae, oe, se, ce, le = {}, ue = !1, de = -1, fe = 0, he = -1;
              for (ae = 0; ae < ne.length; ae++) (oe = (ee = ne[ae][0].trace).stackgroup || "") ? oe in le ? ce = le[oe] : (ce = le[oe] = fe, fe++) : ee.fill in J && he >= 0 ? ce = he : (ce = he = fe, fe++), ce < de && (ue = !0), ee._groupIndex = de = ce;
              var me = ne.slice();
              ue && me.sort(function(ye, ge) {
                var ve = ye[0].trace, be = ge[0].trace;
                return ve._groupIndex - be._groupIndex || ve.index - be.index;
              });
              var pe = {};
              for (ae = 0; ae < me.length; ae++) oe = (ee = me[ae][0].trace).stackgroup || "", ee.visible === !0 ? (ee._nexttrace = null, ee.fill in J && (se = pe[oe], ee._prevtrace = se || null, se && (se._nexttrace = ee)), ee._ownfill = ee.fill && (ee.fill.substr(0, 6) === "tozero" || ee.fill === "toself" || ee.fill.substr(0, 2) === "to" && !ee._prevtrace), pe[oe] = ee) : ee._prevtrace = ee._nexttrace = ee._ownfill = null;
              return me;
            };
          }, 92527: function(G, J, Z) {
            var te = Z(10721);
            G.exports = function(ne, ee) {
              ee || (ee = 2);
              var ae = ne.marker, oe = ae.sizeref || 1, se = ae.sizemin || 0, ce = ae.sizemode === "area" ? function(le) {
                return Math.sqrt(le / oe);
              } : function(le) {
                return le / oe;
              };
              return function(le) {
                var ue = ce(le / ee);
                return te(ue) && ue > 0 ? Math.max(ue, se) : 0;
              };
            };
          }, 21146: function(G) {
            G.exports = { container: "marker", min: "cmin", max: "cmax" };
          }, 24272: function(G, J, Z) {
            var te = Z(78766), ne = Z(65477).hasColorscale, ee = Z(39356), ae = Z(64726);
            G.exports = function(oe, se, ce, le, ue, de) {
              var fe = ae.isBubble(oe), he = (oe.line || {}).color;
              de = de || {}, he && (ce = he), ue("marker.symbol"), ue("marker.opacity", fe ? 0.7 : 1), ue("marker.size"), de.noAngle || (ue("marker.angle"), de.noAngleRef || ue("marker.angleref"), de.noStandOff || ue("marker.standoff")), ue("marker.color", ce), ne(oe, "marker") && ee(oe, se, le, ue, { prefix: "marker.", cLetter: "c" }), de.noSelect || (ue("selected.marker.color"), ue("unselected.marker.color"), ue("selected.marker.size"), ue("unselected.marker.size")), de.noLine || (ue("marker.line.color", he && !Array.isArray(he) && se.marker.color !== he ? he : fe ? te.background : te.defaultLine), ne(oe, "marker.line") && ee(oe, se, le, ue, { prefix: "marker.line.", cLetter: "c" }), ue("marker.line.width", fe ? 1 : 0)), fe && (ue("marker.sizeref"), ue("marker.sizemin"), ue("marker.sizemode")), de.gradient && ue("marker.gradient.type") !== "none" && ue("marker.gradient.color");
            };
          }, 99669: function(G, J, Z) {
            var te = Z(34809).dateTick0, ne = Z(63821).ONEWEEK;
            function ee(ae, oe) {
              return te(oe, ae % ne == 0 ? 1 : 0);
            }
            G.exports = function(ae, oe, se, ce, le) {
              if (le || (le = { x: !0, y: !0 }), le.x) {
                var ue = ce("xperiod");
                ue && (ce("xperiod0", ee(ue, oe.xcalendar)), ce("xperiodalignment"));
              }
              if (le.y) {
                var de = ce("yperiod");
                de && (ce("yperiod0", ee(de, oe.ycalendar)), ce("yperiodalignment"));
              }
            };
          }, 36098: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(34809), ae = ee.ensureSingle, oe = ee.identity, se = Z(62203), ce = Z(64726), le = Z(5525), ue = Z(17210), de = Z(80899).tester;
            function fe(he, me, pe, ye, ge, ve, be) {
              var Ce, Se = he._context.staticPlot;
              (function(rn, vn, dn, Ht, jt) {
                var cn = dn.xaxis, wn = dn.yaxis, Rn = te.extent(ee.simpleMap(cn.range, cn.r2c)), Pn = te.extent(ee.simpleMap(wn.range, wn.r2c)), Ie = Ht[0].trace;
                if (ce.hasMarkers(Ie)) {
                  var rt = Ie.marker.maxdisplayed;
                  if (rt !== 0) {
                    var ft = Ht.filter(function(At) {
                      return At.x >= Rn[0] && At.x <= Rn[1] && At.y >= Pn[0] && At.y <= Pn[1];
                    }), pt = Math.ceil(ft.length / rt), wt = 0;
                    jt.forEach(function(At, Dt) {
                      var Ot = At[0].trace;
                      ce.hasMarkers(Ot) && Ot.marker.maxdisplayed > 0 && Dt < vn && wt++;
                    });
                    var Bt = Math.round(wt * pt / 3 + Math.floor(wt / 3) * pt / 7.1);
                    Ht.forEach(function(At) {
                      delete At.vis;
                    }), ft.forEach(function(At, Dt) {
                      Math.round((Dt + Bt) % pt) === 0 && (At.vis = !0);
                    });
                  }
                }
              })(0, me, pe, ye, ge);
              var we = !!be && be.duration > 0;
              function Ee(rn) {
                return we ? rn.transition() : rn;
              }
              var xe = pe.xaxis, Le = pe.yaxis, Qe = ye[0].trace, ke = Qe.line, Me = te.select(ve), Re = ae(Me, "g", "errorbars"), De = ae(Me, "g", "lines"), Pe = ae(Me, "g", "points"), $e = ae(Me, "g", "text");
              if (ne.getComponentMethod("errorbars", "plot")(he, Re, pe, be), Qe.visible === !0) {
                var qe, Xe;
                Ee(Me).style("opacity", Qe.opacity);
                var Ve, Ze, Ge = Qe.fill.charAt(Qe.fill.length - 1);
                Ge !== "x" && Ge !== "y" && (Ge = ""), Ge === "y" ? (Ve = 1, Ze = Le.c2p(0, !0)) : Ge === "x" && (Ve = 0, Ze = xe.c2p(0, !0)), ye[0][pe.isRangePlot ? "nodeRangePlot3" : "node3"] = Me;
                var ze, We, at = "", ct = [], et = Qe._prevtrace, dt = null, Je = null;
                et && (at = et._prevRevpath || "", Xe = et._nextFill, ct = et._ownPolygons, dt = et._fillsegments, Je = et._fillElement);
                var it, st, gt, bt, yt, St, Lt = "", Rt = "", vt = [];
                Qe._polygons = [];
                var $t = [], Pt = [], Xt = ee.noop;
                if (qe = Qe._ownFill, ce.hasLines(Qe) || Qe.fill !== "none") {
                  Xe && Xe.datum(ye), ["hv", "vh", "hvh", "vhv"].indexOf(ke.shape) !== -1 ? (it = se.steps(ke.shape), st = se.steps(ke.shape.split("").reverse().join(""))) : it = st = ke.shape === "spline" ? function(rn) {
                    var vn = rn[rn.length - 1];
                    return rn.length > 1 && rn[0][0] === vn[0] && rn[0][1] === vn[1] ? se.smoothclosed(rn.slice(1), ke.smoothing) : se.smoothopen(rn, ke.smoothing);
                  } : function(rn) {
                    return "M" + rn.join("L");
                  }, gt = function(rn) {
                    return st(rn.reverse());
                  }, Pt = le(ye, { xaxis: xe, yaxis: Le, trace: Qe, connectGaps: Qe.connectgaps, baseTolerance: Math.max(ke.width || 1, 3) / 4, shape: ke.shape, backoff: ke.backoff, simplify: ke.simplify, fill: Qe.fill }), $t = new Array(Pt.length);
                  var Nt = 0;
                  for (Ce = 0; Ce < Pt.length; Ce++) {
                    var Gt, Ft = Pt[Ce];
                    Gt && Ge ? Gt.push.apply(Gt, Ft) : (Gt = Ft.slice(), $t[Nt] = Gt, Nt++);
                  }
                  Qe._fillElement = null, Qe._fillExclusionElement = Je, Qe._fillsegments = $t.slice(0, Nt), $t = Qe._fillsegments, Pt.length && (bt = Pt[0][0].slice(), St = (yt = Pt[Pt.length - 1])[yt.length - 1].slice()), Xt = function(rn) {
                    return function(vn) {
                      if (ze = it(vn), We = gt(vn), Lt ? Ge ? (Lt += "L" + ze.substr(1), Rt = We + "L" + Rt.substr(1)) : (Lt += "Z" + ze, Rt = We + "Z" + Rt) : (Lt = ze, Rt = We), ce.hasLines(Qe)) {
                        var dn = te.select(this);
                        if (dn.datum(ye), rn) Ee(dn.style("opacity", 0).attr("d", ze).call(se.lineGroupStyle)).style("opacity", 1);
                        else {
                          var Ht = Ee(dn);
                          Ht.attr("d", ze), se.singleLineStyle(ye, Ht);
                        }
                      }
                    };
                  };
                }
                var Tt = De.selectAll(".js-line").data(Pt);
                Ee(Tt.exit()).style("opacity", 0).remove(), Tt.each(Xt(!1)), Tt.enter().append("path").classed("js-line", !0).style("vector-effect", Se ? "none" : "non-scaling-stroke").call(se.lineGroupStyle).each(Xt(!0)), se.setClipUrl(Tt, pe.layerClipId, he);
                var It = function() {
                  var rn = new Array($t.length);
                  for (Ce = 0; Ce < $t.length; Ce++) rn[Ce] = de($t[Ce]);
                  return rn;
                }, Ct = function(rn) {
                  var vn, dn;
                  if (rn && rn.length !== 0) {
                    for (vn = new Array(rn.length - 1 + $t.length), dn = 0; dn < rn.length - 1; dn++) vn[dn] = de(rn[dn]);
                    var Ht = rn[rn.length - 1].slice();
                    for (Ht.reverse(), dn = 0; dn < $t.length; dn++) vn[rn.length - 1 + dn] = de($t[dn].concat(Ht));
                  } else for (vn = new Array($t.length), dn = 0; dn < $t.length; dn++) {
                    var jt = $t[dn][0].slice(), cn = $t[dn][$t[dn].length - 1].slice();
                    jt[Ve] = cn[Ve] = Ze;
                    var wn = [cn, jt].concat($t[dn]);
                    vn[dn] = de(wn);
                  }
                  return vn;
                };
                Pt.length ? (qe ? (qe.datum(ye), bt && St && (Ge ? (bt[Ve] = St[Ve] = Ze, Ee(qe).attr("d", "M" + St + "L" + bt + "L" + Lt.substr(1)).call(se.singleFillStyle, he), vt = Ct(null)) : (Ee(qe).attr("d", Lt + "Z").call(se.singleFillStyle, he), vt = It())), Qe._polygons = vt, Qe._fillElement = qe) : Xe && (Qe.fill.substr(0, 6) === "tonext" && Lt && at ? (Qe.fill === "tonext" ? (Ee(Xe).attr("d", Lt + "Z" + at + "Z").call(se.singleFillStyle, he), vt = It(), Qe._polygons = vt.concat(ct)) : (Ee(Xe).attr("d", Lt + "L" + at.substr(1) + "Z").call(se.singleFillStyle, he), vt = Ct(dt), Qe._polygons = vt), Qe._fillElement = Xe) : Vt(Xe)), Qe._prevRevpath = Rt) : (qe ? Vt(qe) : Xe && Vt(Xe), Qe._prevRevpath = null), Qe._ownPolygons = vt, Pe.datum(ye), $e.datum(ye), function(rn, vn, dn) {
                  var Ht, jt = dn[0].trace, cn = ce.hasMarkers(jt), wn = ce.hasText(jt), Rn = Ut(jt), Pn = Wt, Ie = Wt;
                  if (cn || wn) {
                    var rt = oe, ft = jt.stackgroup, pt = ft && he._fullLayout._scatterStackOpts[xe._id + Le._id][ft].stackgaps === "infer zero";
                    jt.marker.maxdisplayed || jt._needsCull ? rt = pt ? sn : Yt : ft && !pt && (rt = hn), cn && (Pn = rt), wn && (Ie = rt);
                  }
                  var wt, Bt = (Ht = rn.selectAll("path.point").data(Pn, Rn)).enter().append("path").classed("point", !0);
                  we && Bt.call(se.pointStyle, jt, he).call(se.translatePoints, xe, Le).style("opacity", 0).transition().style("opacity", 1), Ht.order(), cn && (wt = se.makePointStyleFns(jt)), Ht.each(function(At) {
                    var Dt = te.select(this), Ot = Ee(Dt);
                    se.translatePoint(At, Ot, xe, Le) ? (se.singlePointStyle(At, Ot, jt, wt, he), pe.layerClipId && se.hideOutsideRangePoint(At, Ot, xe, Le, jt.xcalendar, jt.ycalendar), jt.customdata && Dt.classed("plotly-customdata", At.data !== null && At.data !== void 0)) : Ot.remove();
                  }), we ? Ht.exit().transition().style("opacity", 0).remove() : Ht.exit().remove(), (Ht = vn.selectAll("g").data(Ie, Rn)).enter().append("g").classed("textpoint", !0).append("text"), Ht.order(), Ht.each(function(At) {
                    var Dt = te.select(this), Ot = Ee(Dt.select("text"));
                    se.translatePoint(At, Ot, xe, Le) ? pe.layerClipId && se.hideOutsideRangePoint(At, Dt, xe, Le, jt.xcalendar, jt.ycalendar) : Dt.remove();
                  }), Ht.selectAll("text").call(se.textPointStyle, jt, he).each(function(At) {
                    var Dt = xe.c2p(At.x), Ot = Le.c2p(At.y);
                    te.select(this).selectAll("tspan.line").each(function() {
                      Ee(te.select(this)).attr({ x: Dt, y: Ot });
                    });
                  }), Ht.exit().remove();
                }(Pe, $e, ye);
                var Kt = Qe.cliponaxis === !1 ? null : pe.layerClipId;
                se.setClipUrl(Pe, Kt, he), se.setClipUrl($e, Kt, he);
              }
              function Vt(rn) {
                Ee(rn).attr("d", "M0,0Z");
              }
              function Yt(rn) {
                return rn.filter(function(vn) {
                  return !vn.gap && vn.vis;
                });
              }
              function sn(rn) {
                return rn.filter(function(vn) {
                  return vn.vis;
                });
              }
              function hn(rn) {
                return rn.filter(function(vn) {
                  return !vn.gap;
                });
              }
              function Qt(rn) {
                return rn.id;
              }
              function Ut(rn) {
                if (rn.ids) return Qt;
              }
              function Wt() {
                return !1;
              }
            }
            G.exports = function(he, me, pe, ye, ge, ve) {
              var be, Ce, Se = !ge, we = !!ge && ge.duration > 0, Ee = ue(he, me, pe);
              (be = ye.selectAll("g.trace").data(Ee, function(xe) {
                return xe[0].trace.uid;
              })).enter().append("g").attr("class", function(xe) {
                return "trace scatter trace" + xe[0].trace.uid;
              }).style("stroke-miterlimit", 2), be.order(), function(xe, Le, Qe) {
                Le.each(function(ke) {
                  var Me = ae(te.select(this), "g", "fills");
                  se.setClipUrl(Me, Qe.layerClipId, xe);
                  var Re = ke[0].trace, De = [];
                  Re._ownfill && De.push("_ownFill"), Re._nexttrace && De.push("_nextFill");
                  var Pe = Me.selectAll("g").data(De, oe);
                  Pe.enter().append("g"), Pe.exit().each(function($e) {
                    Re[$e] = null;
                  }).remove(), Pe.order().each(function($e) {
                    Re[$e] = ae(te.select(this), "path", "js-fill");
                  });
                });
              }(he, be, me), we ? (ve && (Ce = ve()), te.transition().duration(ge.duration).ease(ge.easing).each("end", function() {
                Ce && Ce();
              }).each("interrupt", function() {
                Ce && Ce();
              }).each(function() {
                ye.selectAll("g.trace").each(function(xe, Le) {
                  fe(he, Le, me, xe, Ee, this, ge);
                });
              })) : be.each(function(xe, Le) {
                fe(he, Le, me, xe, Ee, this, ge);
              }), Se && be.exit().remove(), ye.selectAll("path:not([d])").remove();
            };
          }, 32665: function(G, J, Z) {
            var te = Z(64726);
            G.exports = function(ne, ee) {
              var ae, oe, se, ce, le = ne.cd, ue = ne.xaxis, de = ne.yaxis, fe = [], he = le[0].trace;
              if (!te.hasMarkers(he) && !te.hasText(he)) return [];
              if (ee === !1) for (ae = 0; ae < le.length; ae++) le[ae].selected = 0;
              else for (ae = 0; ae < le.length; ae++) oe = le[ae], se = ue.c2p(oe.x), ce = de.c2p(oe.y), oe.i !== null && ee.contains([se, ce], !1, ae, ne) ? (fe.push({ pointNumber: oe.i, x: ue.c2d(oe.x), y: de.c2d(oe.y) }), oe.selected = 1) : oe.selected = 0;
              return fe;
            };
          }, 382: function(G) {
            var J = ["orientation", "groupnorm", "stackgaps"];
            G.exports = function(Z, te, ne, ee) {
              var ae = ne._scatterStackOpts, oe = ee("stackgroup");
              if (oe) {
                var se = te.xaxis + te.yaxis, ce = ae[se];
                ce || (ce = ae[se] = {});
                var le = ce[oe], ue = !1;
                le ? le.traces.push(te) : (le = ce[oe] = { traceIndices: [], traces: [te] }, ue = !0);
                for (var de = { orientation: te.x && !te.y ? "h" : "v" }, fe = 0; fe < J.length; fe++) {
                  var he = J[fe], me = he + "Found";
                  if (!le[me]) {
                    var pe = Z[he] !== void 0, ye = he === "orientation";
                    if ((pe || ue) && (le[he] = ee(he, de[he]), ye && (le.fillDflt = le[he] === "h" ? "tonextx" : "tonexty"), pe && (le[me] = !0, !ue && (delete le.traces[0][he], ye)))) for (var ge = 0; ge < le.traces.length - 1; ge++) {
                      var ve = le.traces[ge];
                      ve._input.fill !== ve.fill && (ve.fill = le.fillDflt);
                    }
                  }
                }
                return le;
              }
            };
          }, 9408: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(33626);
            function ae(se, ce, le) {
              ne.pointStyle(se.selectAll("path.point"), ce, le);
            }
            function oe(se, ce, le) {
              ne.textPointStyle(se.selectAll("text"), ce, le);
            }
            G.exports = { style: function(se) {
              var ce = te.select(se).selectAll("g.trace.scatter");
              ce.style("opacity", function(le) {
                return le[0].trace.opacity;
              }), ce.selectAll("g.points").each(function(le) {
                ae(te.select(this), le.trace || le[0].trace, se);
              }), ce.selectAll("g.text").each(function(le) {
                oe(te.select(this), le.trace || le[0].trace, se);
              }), ce.selectAll("g.trace path.js-line").call(ne.lineGroupStyle), ce.selectAll("g.trace path.js-fill").call(ne.fillGroupStyle, se, !1), ee.getComponentMethod("errorbars", "style")(ce);
            }, stylePoints: ae, styleText: oe, styleOnSelect: function(se, ce, le) {
              var ue = ce[0].trace;
              ue.selectedpoints ? (ne.selectedPointStyle(le.selectAll("path.point"), ue), ne.selectedTextStyle(le.selectAll("text"), ue)) : (ae(le, ue, se), oe(le, ue, se));
            } };
          }, 64726: function(G, J, Z) {
            var te = Z(34809), ne = Z(87800).isTypedArraySpec;
            G.exports = { hasLines: function(ee) {
              return ee.visible && ee.mode && ee.mode.indexOf("lines") !== -1;
            }, hasMarkers: function(ee) {
              return ee.visible && (ee.mode && ee.mode.indexOf("markers") !== -1 || ee.type === "splom");
            }, hasText: function(ee) {
              return ee.visible && ee.mode && ee.mode.indexOf("text") !== -1;
            }, isBubble: function(ee) {
              var ae = ee.marker;
              return te.isPlainObject(ae) && (te.isArrayOrTypedArray(ae.size) || ne(ae.size));
            } };
          }, 663: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee, ae, oe, se) {
              se = se || {}, oe("textposition"), te.coerceFont(oe, "textfont", se.font || ae.font, se), se.noSelect || (oe("selected.textfont.color"), oe("unselected.textfont.color"));
            };
          }, 99867: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626);
            G.exports = function(ee, ae, oe, se) {
              var ce, le = se("x"), ue = se("y");
              if (ne.getComponentMethod("calendars", "handleTraceDefaults")(ee, ae, ["x", "y"], oe), le) {
                var de = te.minRowLength(le);
                ue ? ce = Math.min(de, te.minRowLength(ue)) : (ce = de, se("y0"), se("dy"));
              } else {
                if (!ue) return 0;
                ce = te.minRowLength(ue), se("x0"), se("dx");
              }
              return ae._length = ce, ce;
            };
          }, 14117: function(G, J, Z) {
            var te = Z(36640), ne = Z(80337), ee = Z(87163), ae = Z(80712).axisHoverFormat, oe = Z(3208).rb, se = Z(3208).ay, ce = Z(9829), le = Z(84770), ue = Z(49467), de = Z(93049).extendFlat, fe = Z(13582).overrideAll, he = Z(62994), me = te.line, pe = te.marker, ye = pe.line, ge = de({ width: me.width, dash: { valType: "enumerated", values: he(le), dflt: "solid" } }, ee("line")), ve = G.exports = fe({ x: te.x, y: te.y, z: { valType: "data_array" }, text: de({}, te.text, {}), texttemplate: se({}, {}), hovertext: de({}, te.hovertext, {}), hovertemplate: oe(), xhoverformat: ae("x"), yhoverformat: ae("y"), zhoverformat: ae("z"), mode: de({}, te.mode, { dflt: "lines+markers" }), surfaceaxis: { valType: "enumerated", values: [-1, 0, 1, 2], dflt: -1 }, surfacecolor: { valType: "color" }, projection: { x: { show: { valType: "boolean", dflt: !1 }, opacity: { valType: "number", min: 0, max: 1, dflt: 1 }, scale: { valType: "number", min: 0, max: 10, dflt: 2 / 3 } }, y: { show: { valType: "boolean", dflt: !1 }, opacity: { valType: "number", min: 0, max: 1, dflt: 1 }, scale: { valType: "number", min: 0, max: 10, dflt: 2 / 3 } }, z: { show: { valType: "boolean", dflt: !1 }, opacity: { valType: "number", min: 0, max: 1, dflt: 1 }, scale: { valType: "number", min: 0, max: 10, dflt: 2 / 3 } } }, connectgaps: te.connectgaps, line: ge, marker: de({ symbol: { valType: "enumerated", values: he(ue), dflt: "circle", arrayOk: !0 }, size: de({}, pe.size, { dflt: 8 }), sizeref: pe.sizeref, sizemin: pe.sizemin, sizemode: pe.sizemode, opacity: de({}, pe.opacity, { arrayOk: !1 }), colorbar: pe.colorbar, line: de({ width: de({}, ye.width, { arrayOk: !1 }) }, ee("marker.line")) }, ee("marker")), textposition: de({}, te.textposition, { dflt: "top center" }), textfont: ne({ noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0, editType: "calc", colorEditType: "style", arrayOk: !0, variantValues: ["normal", "small-caps"] }), opacity: ce.opacity, hoverinfo: de({}, ce.hoverinfo) }, "calc", "nested");
            ve.x.editType = ve.y.editType = ve.z.editType = "calc+clearAxisTypes";
          }, 37593: function(G, J, Z) {
            var te = Z(99203), ne = Z(77272);
            G.exports = function(ee, ae) {
              var oe = [{ x: !1, y: !1, trace: ae, t: {} }];
              return te(oe, ae), ne(ee, ae), oe;
            };
          }, 95447: function(G, J, Z) {
            var te = Z(33626);
            function ne(ee, ae, oe, se) {
              if (!ae || !ae.visible) return null;
              for (var ce = te.getComponentMethod("errorbars", "makeComputeError")(ae), le = new Array(ee.length), ue = 0; ue < ee.length; ue++) {
                var de = ce(+ee[ue], ue);
                if (se.type === "log") {
                  var fe = se.c2l(ee[ue]), he = ee[ue] - de[0], me = ee[ue] + de[1];
                  if (le[ue] = [(se.c2l(he, !0) - fe) * oe, (se.c2l(me, !0) - fe) * oe], he > 0) {
                    var pe = se.c2l(he);
                    se._lowerLogErrorBound || (se._lowerLogErrorBound = pe), se._lowerErrorBound = Math.min(se._lowerLogErrorBound, pe);
                  }
                } else le[ue] = [-de[0] * oe, de[1] * oe];
              }
              return le;
            }
            G.exports = function(ee, ae, oe) {
              var se = [ne(ee.x, ee.error_x, ae[0], oe.xaxis), ne(ee.y, ee.error_y, ae[1], oe.yaxis), ne(ee.z, ee.error_z, ae[2], oe.zaxis)], ce = function(me) {
                for (var pe = 0; pe < me.length; pe++) if (me[pe]) return me[pe].length;
                return 0;
              }(se);
              if (ce === 0) return null;
              for (var le = new Array(ce), ue = 0; ue < ce; ue++) {
                for (var de = [[0, 0, 0], [0, 0, 0]], fe = 0; fe < 3; fe++) if (se[fe]) for (var he = 0; he < 2; he++) de[he][fe] = se[fe][ue][he];
                le[ue] = de;
              }
              return le;
            };
          }, 16533: function(G, J, Z) {
            var te = Z(99098).gl_line3d, ne = Z(99098).gl_scatter3d, ee = Z(99098).gl_error3d, ae = Z(99098).gl_mesh3d, oe = Z(99098).delaunay_triangulate, se = Z(34809), ce = Z(55010), le = Z(46998).formatColor, ue = Z(92527), de = Z(84770), fe = Z(49467), he = Z(29714), me = Z(36040).appendArrayPointValue, pe = Z(95447);
            function ye(Le, Qe) {
              this.scene = Le, this.uid = Qe, this.linePlot = null, this.scatterPlot = null, this.errorBars = null, this.textMarkers = null, this.delaunayMesh = null, this.color = null, this.mode = "", this.dataPoints = [], this.axesBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.textLabels = null, this.data = null;
            }
            var ge = ye.prototype;
            function ve(Le) {
              return Le == null ? 0 : Le.indexOf("left") > -1 ? -1 : Le.indexOf("right") > -1 ? 1 : 0;
            }
            function be(Le) {
              return Le == null ? 0 : Le.indexOf("top") > -1 ? -1 : Le.indexOf("bottom") > -1 ? 1 : 0;
            }
            function Ce(Le, Qe) {
              return Qe(4 * Le);
            }
            function Se(Le) {
              return fe[Le];
            }
            function we(Le, Qe, ke, Me, Re) {
              var De = null;
              if (se.isArrayOrTypedArray(Le)) {
                De = [];
                for (var Pe = 0; Pe < Qe; Pe++) Le[Pe] === void 0 ? De[Pe] = Me : De[Pe] = ke(Le[Pe], Re);
              } else De = ke(Le, se.identity);
              return De;
            }
            function Ee(Le) {
              if (se.isArrayOrTypedArray(Le)) {
                var Qe = Le[0];
                return se.isArrayOrTypedArray(Qe) && (Le = Qe), "rgb(" + Le.slice(0, 3).map(function(ke) {
                  return Math.round(255 * ke);
                }) + ")";
              }
              return null;
            }
            function xe(Le) {
              return se.isArrayOrTypedArray(Le) ? Le.length === 4 && typeof Le[0] == "number" ? Ee(Le) : Le.map(Ee) : null;
            }
            ge.handlePick = function(Le) {
              if (Le.object && (Le.object === this.linePlot || Le.object === this.delaunayMesh || Le.object === this.textMarkers || Le.object === this.scatterPlot)) {
                var Qe = Le.index = Le.data.index;
                return Le.object.highlight && Le.object.highlight(null), this.scatterPlot && (Le.object = this.scatterPlot, this.scatterPlot.highlight(Le.data)), Le.textLabel = "", this.textLabels && (se.isArrayOrTypedArray(this.textLabels) ? (this.textLabels[Qe] || this.textLabels[Qe] === 0) && (Le.textLabel = this.textLabels[Qe]) : Le.textLabel = this.textLabels), Le.traceCoordinate = [this.data.x[Qe], this.data.y[Qe], this.data.z[Qe]], !0;
              }
            }, ge.update = function(Le) {
              var Qe, ke, Me, Re, De = this.scene.glplot.gl, Pe = de.solid;
              this.data = Le;
              var $e = function(Ve, Ze) {
                var Ge, ze, We, at, ct, et, dt = [], Je = Ve.fullSceneLayout, it = Ve.dataScale, st = Je.xaxis, gt = Je.yaxis, bt = Je.zaxis, yt = Ze.marker, St = Ze.line, Lt = Ze.x || [], Rt = Ze.y || [], vt = Ze.z || [], $t = Lt.length, Pt = Ze.xcalendar, Xt = Ze.ycalendar, Nt = Ze.zcalendar;
                for (ct = 0; ct < $t; ct++) Ge = st.d2l(Lt[ct], 0, Pt) * it[0], ze = gt.d2l(Rt[ct], 0, Xt) * it[1], We = bt.d2l(vt[ct], 0, Nt) * it[2], dt[ct] = [Ge, ze, We];
                if (Array.isArray(Ze.text)) et = Ze.text;
                else if (se.isTypedArray(Ze.text)) et = Array.from(Ze.text);
                else if (Ze.text !== void 0) for (et = new Array($t), ct = 0; ct < $t; ct++) et[ct] = Ze.text;
                function Gt(vn, dn) {
                  var Ht = Je[vn];
                  return he.tickText(Ht, Ht.d2l(dn), !0).text;
                }
                var Ft = Ze.texttemplate;
                if (Ft) {
                  var Tt = Ve.fullLayout._d3locale, It = Array.isArray(Ft), Ct = It ? Math.min(Ft.length, $t) : $t, Kt = It ? function(vn) {
                    return Ft[vn];
                  } : function() {
                    return Ft;
                  };
                  for (et = new Array(Ct), ct = 0; ct < Ct; ct++) {
                    var Vt = { x: Lt[ct], y: Rt[ct], z: vt[ct] }, Yt = { xLabel: Gt("xaxis", Lt[ct]), yLabel: Gt("yaxis", Rt[ct]), zLabel: Gt("zaxis", vt[ct]) }, sn = {};
                    me(sn, Ze, ct);
                    var hn = Ze._meta || {};
                    et[ct] = se.texttemplateString(Kt(ct), Yt, Tt, sn, Vt, hn);
                  }
                }
                if (at = { position: dt, mode: Ze.mode, text: et }, "line" in Ze && (at.lineColor = le(St, 1, $t), at.lineWidth = St.width, at.lineDashes = St.dash), "marker" in Ze) {
                  var Qt = ue(Ze);
                  at.scatterColor = le(yt, 1, $t), at.scatterSize = we(yt.size, $t, Ce, 20, Qt), at.scatterMarker = we(yt.symbol, $t, Se, "●"), at.scatterLineWidth = yt.line.width, at.scatterLineColor = le(yt.line, 1, $t), at.scatterAngle = 0;
                }
                "textposition" in Ze && (at.textOffset = function(vn) {
                  var dn = [0, 0];
                  if (Array.isArray(vn)) for (var Ht = 0; Ht < vn.length; Ht++) dn[Ht] = [0, 0], vn[Ht] && (dn[Ht][0] = ve(vn[Ht]), dn[Ht][1] = be(vn[Ht]));
                  else dn[0] = ve(vn), dn[1] = be(vn);
                  return dn;
                }(Ze.textposition), at.textColor = le(Ze.textfont, 1, $t), at.textSize = we(Ze.textfont.size, $t, se.identity, 12), at.textFontFamily = Ze.textfont.family, at.textFontWeight = Ze.textfont.weight, at.textFontStyle = Ze.textfont.style, at.textFontVariant = Ze.textfont.variant, at.textAngle = 0);
                var Ut = ["x", "y", "z"];
                for (at.project = [!1, !1, !1], at.projectScale = [1, 1, 1], at.projectOpacity = [1, 1, 1], ct = 0; ct < 3; ++ct) {
                  var Wt = Ze.projection[Ut[ct]];
                  (at.project[ct] = Wt.show) && (at.projectOpacity[ct] = Wt.opacity, at.projectScale[ct] = Wt.scale);
                }
                at.errorBounds = pe(Ze, it, Je);
                var rn = function(vn) {
                  for (var dn = [0, 0, 0], Ht = [[0, 0, 0], [0, 0, 0], [0, 0, 0]], jt = [1, 1, 1], cn = 0; cn < 3; cn++) {
                    var wn = vn[cn];
                    wn && wn.copy_zstyle !== !1 && vn[2].visible !== !1 && (wn = vn[2]), wn && wn.visible && (dn[cn] = wn.width / 2, Ht[cn] = ce(wn.color), jt[cn] = wn.thickness);
                  }
                  return { capSize: dn, color: Ht, lineWidth: jt };
                }([Ze.error_x, Ze.error_y, Ze.error_z]);
                return at.errorColor = rn.color, at.errorLineWidth = rn.lineWidth, at.errorCapSize = rn.capSize, at.delaunayAxis = Ze.surfaceaxis, at.delaunayColor = ce(Ze.surfacecolor), at;
              }(this.scene, Le);
              "mode" in $e && (this.mode = $e.mode), "lineDashes" in $e && $e.lineDashes in de && (Pe = de[$e.lineDashes]), this.color = xe($e.scatterColor) || xe($e.lineColor), this.dataPoints = $e.position, Qe = { gl: this.scene.glplot.gl, position: $e.position, color: $e.lineColor, lineWidth: $e.lineWidth || 1, dashes: Pe[0], dashScale: Pe[1], opacity: Le.opacity, connectGaps: Le.connectgaps }, this.mode.indexOf("lines") !== -1 ? this.linePlot ? this.linePlot.update(Qe) : (this.linePlot = te(Qe), this.linePlot._trace = this, this.scene.glplot.add(this.linePlot)) : this.linePlot && (this.scene.glplot.remove(this.linePlot), this.linePlot.dispose(), this.linePlot = null);
              var qe = Le.opacity;
              if (Le.marker && Le.marker.opacity !== void 0 && (qe *= Le.marker.opacity), ke = { gl: this.scene.glplot.gl, position: $e.position, color: $e.scatterColor, size: $e.scatterSize, glyph: $e.scatterMarker, opacity: qe, orthographic: !0, lineWidth: $e.scatterLineWidth, lineColor: $e.scatterLineColor, project: $e.project, projectScale: $e.projectScale, projectOpacity: $e.projectOpacity }, this.mode.indexOf("markers") !== -1 ? this.scatterPlot ? this.scatterPlot.update(ke) : (this.scatterPlot = ne(ke), this.scatterPlot._trace = this, this.scatterPlot.highlightScale = 1, this.scene.glplot.add(this.scatterPlot)) : this.scatterPlot && (this.scene.glplot.remove(this.scatterPlot), this.scatterPlot.dispose(), this.scatterPlot = null), Re = { gl: this.scene.glplot.gl, position: $e.position, glyph: $e.text, color: $e.textColor, size: $e.textSize, angle: $e.textAngle, alignment: $e.textOffset, font: $e.textFontFamily, fontWeight: $e.textFontWeight, fontStyle: $e.textFontStyle, fontVariant: $e.textFontVariant, orthographic: !0, lineWidth: 0, project: !1, opacity: Le.opacity }, this.textLabels = Le.hovertext || Le.text, this.mode.indexOf("text") !== -1 ? this.textMarkers ? this.textMarkers.update(Re) : (this.textMarkers = ne(Re), this.textMarkers._trace = this, this.textMarkers.highlightScale = 1, this.scene.glplot.add(this.textMarkers)) : this.textMarkers && (this.scene.glplot.remove(this.textMarkers), this.textMarkers.dispose(), this.textMarkers = null), Me = { gl: this.scene.glplot.gl, position: $e.position, color: $e.errorColor, error: $e.errorBounds, lineWidth: $e.errorLineWidth, capSize: $e.errorCapSize, opacity: Le.opacity }, this.errorBars ? $e.errorBounds ? this.errorBars.update(Me) : (this.scene.glplot.remove(this.errorBars), this.errorBars.dispose(), this.errorBars = null) : $e.errorBounds && (this.errorBars = ee(Me), this.errorBars._trace = this, this.scene.glplot.add(this.errorBars)), $e.delaunayAxis >= 0) {
                var Xe = function(Ve, Ze, Ge) {
                  var ze, We = (Ge + 1) % 3, at = (Ge + 2) % 3, ct = [], et = [];
                  for (ze = 0; ze < Ve.length; ++ze) {
                    var dt = Ve[ze];
                    !isNaN(dt[We]) && isFinite(dt[We]) && !isNaN(dt[at]) && isFinite(dt[at]) && (ct.push([dt[We], dt[at]]), et.push(ze));
                  }
                  var Je = oe(ct);
                  for (ze = 0; ze < Je.length; ++ze) for (var it = Je[ze], st = 0; st < it.length; ++st) it[st] = et[it[st]];
                  return { positions: Ve, cells: Je, meshColor: Ze };
                }($e.position, $e.delaunayColor, $e.delaunayAxis);
                Xe.opacity = Le.opacity, this.delaunayMesh ? this.delaunayMesh.update(Xe) : (Xe.gl = De, this.delaunayMesh = ae(Xe), this.delaunayMesh._trace = this, this.scene.glplot.add(this.delaunayMesh));
              } else this.delaunayMesh && (this.scene.glplot.remove(this.delaunayMesh), this.delaunayMesh.dispose(), this.delaunayMesh = null);
            }, ge.dispose = function() {
              this.linePlot && (this.scene.glplot.remove(this.linePlot), this.linePlot.dispose()), this.scatterPlot && (this.scene.glplot.remove(this.scatterPlot), this.scatterPlot.dispose()), this.errorBars && (this.scene.glplot.remove(this.errorBars), this.errorBars.dispose()), this.textMarkers && (this.scene.glplot.remove(this.textMarkers), this.textMarkers.dispose()), this.delaunayMesh && (this.scene.glplot.remove(this.delaunayMesh), this.delaunayMesh.dispose());
            }, G.exports = function(Le, Qe) {
              var ke = new ye(Le, Qe.uid);
              return ke.update(Qe), ke;
            };
          }, 82418: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(64726), ae = Z(24272), oe = Z(98168), se = Z(663), ce = Z(14117);
            G.exports = function(le, ue, de, fe) {
              function he(Se, we) {
                return ne.coerce(le, ue, ce, Se, we);
              }
              var me = function(Se, we, Ee, xe) {
                var Le = 0, Qe = Ee("x"), ke = Ee("y"), Me = Ee("z");
                return te.getComponentMethod("calendars", "handleTraceDefaults")(Se, we, ["x", "y", "z"], xe), Qe && ke && Me && (Le = Math.min(Qe.length, ke.length, Me.length), we._length = we._xlength = we._ylength = we._zlength = Le), Le;
              }(le, ue, he, fe);
              if (me) {
                he("text"), he("hovertext"), he("hovertemplate"), he("xhoverformat"), he("yhoverformat"), he("zhoverformat"), he("mode"), ee.hasMarkers(ue) && ae(le, ue, de, fe, he, { noSelect: !0, noAngle: !0 }), ee.hasLines(ue) && (he("connectgaps"), oe(le, ue, de, fe, he)), ee.hasText(ue) && (he("texttemplate"), se(le, ue, fe, he, { noSelect: !0, noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0 }));
                var pe = (ue.line || {}).color, ye = (ue.marker || {}).color;
                he("surfaceaxis") >= 0 && he("surfacecolor", pe || ye);
                for (var ge = ["x", "y", "z"], ve = 0; ve < 3; ++ve) {
                  var be = "projection." + ge[ve];
                  he(be + ".show") && (he(be + ".opacity"), he(be + ".scale"));
                }
                var Ce = te.getComponentMethod("errorbars", "supplyDefaults");
                Ce(le, ue, pe || ye || de, { axis: "z" }), Ce(le, ue, pe || ye || de, { axis: "y", inherit: "z" }), Ce(le, ue, pe || ye || de, { axis: "x", inherit: "z" });
              } else ue.visible = !1;
            };
          }, 17822: function(G, J, Z) {
            G.exports = { plot: Z(16533), attributes: Z(14117), markerSymbols: Z(49467), supplyDefaults: Z(82418), colorbar: [{ container: "marker", min: "cmin", max: "cmax" }, { container: "line", min: "cmin", max: "cmax" }], calc: Z(37593), moduleType: "trace", name: "scatter3d", basePlotModule: Z(2487), categories: ["gl3d", "symbols", "showLegend", "scatter-like"], meta: {} };
          }, 54637: function(G, J, Z) {
            var te = Z(19326), ne = Z(36640), ee = Z(9829), ae = Z(3208).rb, oe = Z(3208).ay, se = Z(87163), ce = Z(93049).extendFlat, le = ne.marker, ue = ne.line, de = le.line;
            G.exports = { carpet: { valType: "string", editType: "calc" }, a: { valType: "data_array", editType: "calc" }, b: { valType: "data_array", editType: "calc" }, mode: ce({}, ne.mode, { dflt: "markers" }), text: ce({}, ne.text, {}), texttemplate: oe({ editType: "plot" }, { keys: ["a", "b", "text"] }), hovertext: ce({}, ne.hovertext, {}), line: { color: ue.color, width: ue.width, dash: ue.dash, backoff: ue.backoff, shape: ce({}, ue.shape, { values: ["linear", "spline"] }), smoothing: ue.smoothing, editType: "calc" }, connectgaps: ne.connectgaps, fill: ce({}, ne.fill, { values: ["none", "toself", "tonext"], dflt: "none" }), fillcolor: te(), marker: ce({ symbol: le.symbol, opacity: le.opacity, maxdisplayed: le.maxdisplayed, angle: le.angle, angleref: le.angleref, standoff: le.standoff, size: le.size, sizeref: le.sizeref, sizemin: le.sizemin, sizemode: le.sizemode, line: ce({ width: de.width, editType: "calc" }, se("marker.line")), gradient: le.gradient, editType: "calc" }, se("marker")), textfont: ne.textfont, textposition: ne.textposition, selected: ne.selected, unselected: ne.unselected, hoverinfo: ce({}, ee.hoverinfo, { flags: ["a", "b", "text", "name"] }), hoveron: ne.hoveron, hovertemplate: ae(), zorder: ne.zorder };
          }, 68001: function(G, J, Z) {
            var te = Z(10721), ne = Z(77272), ee = Z(99203), ae = Z(48861), oe = Z(26544).calcMarkerSize, se = Z(26571);
            G.exports = function(ce, le) {
              var ue = le._carpetTrace = se(ce, le);
              if (ue && ue.visible && ue.visible !== "legendonly") {
                var de;
                le.xaxis = ue.xaxis, le.yaxis = ue.yaxis;
                var fe, he, me = le._length, pe = new Array(me), ye = !1;
                for (de = 0; de < me; de++) if (fe = le.a[de], he = le.b[de], te(fe) && te(he)) {
                  var ge = ue.ab2xy(+fe, +he, !0), ve = ue.isVisible(+fe, +he);
                  ve || (ye = !0), pe[de] = { x: ge[0], y: ge[1], a: fe, b: he, vis: ve };
                } else pe[de] = { x: !1, y: !1 };
                return le._needsCull = ye, pe[0].carpet = ue, pe[0].trace = le, oe(le, me), ne(ce, le), ee(pe, le), ae(pe, le), pe;
              }
            };
          }, 16986: function(G, J, Z) {
            var te = Z(34809), ne = Z(32660), ee = Z(64726), ae = Z(24272), oe = Z(98168), se = Z(91602), ce = Z(663), le = Z(54114), ue = Z(54637);
            G.exports = function(de, fe, he, me) {
              function pe(Ce, Se) {
                return te.coerce(de, fe, ue, Ce, Se);
              }
              pe("carpet"), fe.xaxis = "x", fe.yaxis = "y";
              var ye = pe("a"), ge = pe("b"), ve = Math.min(ye.length, ge.length);
              if (ve) {
                fe._length = ve, pe("text"), pe("texttemplate"), pe("hovertext"), pe("mode", ve < ne.PTS_LINESONLY ? "lines+markers" : "lines"), ee.hasMarkers(fe) && ae(de, fe, he, me, pe, { gradient: !0 }), ee.hasLines(fe) && (oe(de, fe, he, me, pe, { backoff: !0 }), se(de, fe, pe), pe("connectgaps")), ee.hasText(fe) && ce(de, fe, me, pe);
                var be = [];
                (ee.hasMarkers(fe) || ee.hasText(fe)) && (pe("marker.maxdisplayed"), be.push("points")), pe("fill"), fe.fill !== "none" && (le(de, fe, he, pe), ee.hasLines(fe) || se(de, fe, pe)), fe.fill !== "tonext" && fe.fill !== "toself" || be.push("fills"), pe("hoveron", be.join("+") || "points") !== "fills" && pe("hovertemplate"), pe("zorder"), te.coerceSelectionMarkerOpacity(fe, pe);
              } else fe.visible = !1;
            };
          }, 68289: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              var ae = ne[ee];
              return J.a = ae.a, J.b = ae.b, J.y = ae.y, J;
            };
          }, 32709: function(G) {
            G.exports = function(J, Z) {
              var te = {}, ne = Z._carpet, ee = ne.ab2ij([J.a, J.b]), ae = Math.floor(ee[0]), oe = ee[0] - ae, se = Math.floor(ee[1]), ce = ee[1] - se, le = ne.evalxy([], ae, se, oe, ce);
              return te.yLabel = le[1].toFixed(3), te;
            };
          }, 59420: function(G, J, Z) {
            var te = Z(37255), ne = Z(34809).fillText;
            G.exports = function(ee, ae, oe, se) {
              var ce = te(ee, ae, oe, se);
              if (ce && ce[0].index !== !1) {
                var le = ce[0];
                if (le.index === void 0) {
                  var ue = 1 - le.y0 / ee.ya._length, de = ee.xa._length, fe = de * ue / 2, he = de - fe;
                  return le.x0 = Math.max(Math.min(le.x0, he), fe), le.x1 = Math.max(Math.min(le.x1, he), fe), ce;
                }
                var me = le.cd[le.index];
                le.a = me.a, le.b = me.b, le.xLabelVal = void 0, le.yLabelVal = void 0;
                var pe = le.trace, ye = pe._carpet, ge = pe._module.formatLabels(me, pe);
                le.yLabel = ge.yLabel, delete le.text;
                var ve = [];
                if (!pe.hovertemplate) {
                  var be = (me.hi || pe.hoverinfo).split("+");
                  be.indexOf("all") !== -1 && (be = ["a", "b", "text"]), be.indexOf("a") !== -1 && Ce(ye.aaxis, me.a), be.indexOf("b") !== -1 && Ce(ye.baxis, me.b), ve.push("y: " + le.yLabel), be.indexOf("text") !== -1 && ne(me, pe, ve), le.extraText = ve.join("<br>");
                }
                return ce;
              }
              function Ce(Se, we) {
                var Ee;
                Ee = Se.labelprefix && Se.labelprefix.length > 0 ? Se.labelprefix.replace(/ = $/, "") : Se._hovertitle, ve.push(Ee + ": " + we.toFixed(3) + Se.labelsuffix);
              }
            };
          }, 56534: function(G, J, Z) {
            G.exports = { attributes: Z(54637), supplyDefaults: Z(16986), colorbar: Z(21146), formatLabels: Z(32709), calc: Z(68001), plot: Z(64535), style: Z(9408).style, styleOnSelect: Z(9408).styleOnSelect, hoverPoints: Z(59420), selectPoints: Z(32665), eventData: Z(68289), moduleType: "trace", name: "scattercarpet", basePlotModule: Z(37703), categories: ["svg", "carpet", "symbols", "showLegend", "carpetDependent", "zoomScale"], meta: {} };
          }, 64535: function(G, J, Z) {
            var te = Z(36098), ne = Z(29714), ee = Z(62203);
            G.exports = function(ae, oe, se, ce) {
              var le, ue, de, fe = se[0][0].carpet, he = ne.getFromId(ae, fe.xaxis || "x"), me = ne.getFromId(ae, fe.yaxis || "y"), pe = { xaxis: he, yaxis: me, plot: oe.plot };
              for (le = 0; le < se.length; le++) (ue = se[le][0].trace)._xA = he, ue._yA = me;
              for (te(ae, pe, se, ce), le = 0; le < se.length; le++) ue = se[le][0].trace, de = ce.selectAll("g.trace" + ue.uid + " .js-line"), ee.setClipUrl(de, se[le][0].carpet._clipPathId, ae);
            };
          }, 6893: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(19326), ae = Z(36640), oe = Z(9829), se = Z(87163), ce = Z(94850).T, le = Z(93049).extendFlat, ue = Z(13582).overrideAll, de = ae.marker, fe = ae.line, he = de.line;
            G.exports = ue({ lon: { valType: "data_array" }, lat: { valType: "data_array" }, locations: { valType: "data_array" }, locationmode: { valType: "enumerated", values: ["ISO-3", "USA-states", "country names", "geojson-id"], dflt: "ISO-3" }, geojson: { valType: "any", editType: "calc" }, featureidkey: { valType: "string", editType: "calc", dflt: "id" }, mode: le({}, ae.mode, { dflt: "markers" }), text: le({}, ae.text, {}), texttemplate: ne({ editType: "plot" }, { keys: ["lat", "lon", "location", "text"] }), hovertext: le({}, ae.hovertext, {}), textfont: ae.textfont, textposition: ae.textposition, line: { color: fe.color, width: fe.width, dash: ce }, connectgaps: ae.connectgaps, marker: le({ symbol: de.symbol, opacity: de.opacity, angle: de.angle, angleref: le({}, de.angleref, { values: ["previous", "up", "north"] }), standoff: de.standoff, size: de.size, sizeref: de.sizeref, sizemin: de.sizemin, sizemode: de.sizemode, colorbar: de.colorbar, line: le({ width: he.width }, se("marker.line")), gradient: de.gradient }, se("marker")), fill: { valType: "enumerated", values: ["none", "toself"], dflt: "none" }, fillcolor: ee(), selected: ae.selected, unselected: ae.unselected, hoverinfo: le({}, oe.hoverinfo, { flags: ["lon", "lat", "location", "text", "name"] }), hovertemplate: te() }, "calc", "nested");
          }, 75649: function(G, J, Z) {
            var te = Z(10721), ne = Z(63821).BADNUM, ee = Z(77272), ae = Z(99203), oe = Z(48861), se = Z(34809).isArrayOrTypedArray, ce = Z(34809)._;
            function le(ue) {
              return ue && typeof ue == "string";
            }
            G.exports = function(ue, de) {
              var fe, he = se(de.locations), me = he ? de.locations.length : de._length, pe = new Array(me);
              fe = de.geojson ? function(Se) {
                return le(Se) || te(Se);
              } : le;
              for (var ye = 0; ye < me; ye++) {
                var ge = pe[ye] = {};
                if (he) {
                  var ve = de.locations[ye];
                  ge.loc = fe(ve) ? ve : null;
                } else {
                  var be = de.lon[ye], Ce = de.lat[ye];
                  te(be) && te(Ce) ? ge.lonlat = [+be, +Ce] : ge.lonlat = [ne, ne];
                }
              }
              return ae(pe, de), ee(ue, de), oe(pe, de), me && (pe[0].t = { labels: { lat: ce(ue, "lat:") + " ", lon: ce(ue, "lon:") + " " } }), pe;
            };
          }, 27386: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(24272), ae = Z(98168), oe = Z(663), se = Z(54114), ce = Z(6893);
            G.exports = function(le, ue, de, fe) {
              function he(Ce, Se) {
                return te.coerce(le, ue, ce, Ce, Se);
              }
              var me, pe = he("locations");
              if (pe && pe.length) {
                var ye, ge = he("geojson");
                (typeof ge == "string" && ge !== "" || te.isPlainObject(ge)) && (ye = "geojson-id"), he("locationmode", ye) === "geojson-id" && he("featureidkey"), me = pe.length;
              } else {
                var ve = he("lon") || [], be = he("lat") || [];
                me = Math.min(ve.length, be.length);
              }
              me ? (ue._length = me, he("text"), he("hovertext"), he("hovertemplate"), he("mode"), ne.hasMarkers(ue) && ee(le, ue, de, fe, he, { gradient: !0 }), ne.hasLines(ue) && (ae(le, ue, de, fe, he), he("connectgaps")), ne.hasText(ue) && (he("texttemplate"), oe(le, ue, fe, he)), he("fill"), ue.fill !== "none" && se(le, ue, de, he), te.coerceSelectionMarkerOpacity(ue, he)) : ue.visible = !1;
            };
          }, 71873: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              J.lon = Z.lon, J.lat = Z.lat, J.location = Z.loc ? Z.loc : null;
              var ae = ne[ee];
              return ae.fIn && ae.fIn.properties && (J.properties = ae.fIn.properties), J;
            };
          }, 57413: function(G, J, Z) {
            var te = Z(29714);
            G.exports = function(ne, ee, ae) {
              var oe = {}, se = ae[ee.geo]._subplot.mockAxis, ce = ne.lonlat;
              return oe.lonLabel = te.tickText(se, se.c2l(ce[0]), !0).text, oe.latLabel = te.tickText(se, se.c2l(ce[1]), !0).text, oe;
            };
          }, 40636: function(G, J, Z) {
            var te = Z(32141), ne = Z(63821).BADNUM, ee = Z(11539), ae = Z(34809).fillText, oe = Z(6893);
            G.exports = function(se, ce, le) {
              var ue = se.cd, de = ue[0].trace, fe = se.xa, he = se.ya, me = se.subplot, pe = me.projection.isLonLatOverEdges, ye = me.project;
              if (te.getClosest(ue, function(Ee) {
                var xe = Ee.lonlat;
                if (xe[0] === ne || pe(xe)) return 1 / 0;
                var Le = ye(xe), Qe = ye([ce, le]), ke = Math.abs(Le[0] - Qe[0]), Me = Math.abs(Le[1] - Qe[1]), Re = Math.max(3, Ee.mrc || 0);
                return Math.max(Math.sqrt(ke * ke + Me * Me) - Re, 1 - 3 / Re);
              }, se), se.index !== !1) {
                var ge = ue[se.index], ve = ge.lonlat, be = [fe.c2p(ve), he.c2p(ve)], Ce = ge.mrc || 1;
                se.x0 = be[0] - Ce, se.x1 = be[0] + Ce, se.y0 = be[1] - Ce, se.y1 = be[1] + Ce, se.loc = ge.loc, se.lon = ve[0], se.lat = ve[1];
                var Se = {};
                Se[de.geo] = { _subplot: me };
                var we = de._module.formatLabels(ge, de, Se);
                return se.lonLabel = we.lonLabel, se.latLabel = we.latLabel, se.color = ee(de, ge), se.extraText = function(Ee, xe, Le, Qe) {
                  if (!Ee.hovertemplate) {
                    var ke = xe.hi || Ee.hoverinfo, Me = ke === "all" ? oe.hoverinfo.flags : ke.split("+"), Re = Me.indexOf("location") !== -1 && Array.isArray(Ee.locations), De = Me.indexOf("lon") !== -1, Pe = Me.indexOf("lat") !== -1, $e = Me.indexOf("text") !== -1, qe = [];
                    return Re ? qe.push(xe.loc) : De && Pe ? qe.push("(" + Xe(Le.latLabel) + ", " + Xe(Le.lonLabel) + ")") : De ? qe.push(Qe.lon + Xe(Le.lonLabel)) : Pe && qe.push(Qe.lat + Xe(Le.latLabel)), $e && ae(xe, Ee, qe), qe.join("<br>");
                  }
                  function Xe(Ve) {
                    return Ve + "°";
                  }
                }(de, ge, se, ue[0].t.labels), se.hovertemplate = de.hovertemplate, [se];
              }
            };
          }, 18070: function(G, J, Z) {
            G.exports = { attributes: Z(6893), supplyDefaults: Z(27386), colorbar: Z(21146), formatLabels: Z(57413), calc: Z(75649), calcGeoJSON: Z(48887).calcGeoJSON, plot: Z(48887).plot, style: Z(60367), styleOnSelect: Z(9408).styleOnSelect, hoverPoints: Z(40636), eventData: Z(71873), selectPoints: Z(45852), moduleType: "trace", name: "scattergeo", basePlotModule: Z(47544), categories: ["geo", "symbols", "showLegend", "scatter-like"], meta: {} };
          }, 48887: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(11577).getTopojsonFeatures, ae = Z(39532), oe = Z(3994), se = Z(32919).findExtremes, ce = Z(63821).BADNUM, le = Z(26544).calcMarkerSize, ue = Z(64726), de = Z(60367);
            G.exports = { calcGeoJSON: function(fe, he) {
              var me, pe, ye = fe[0].trace, ge = he[ye.geo], ve = ge._subplot, be = ye._length;
              if (ne.isArrayOrTypedArray(ye.locations)) {
                var Ce = ye.locationmode, Se = Ce === "geojson-id" ? oe.extractTraceFeature(fe) : ee(ye, ve.topojson);
                for (me = 0; me < be; me++) {
                  pe = fe[me];
                  var we = Ce === "geojson-id" ? pe.fOut : oe.locationToFeature(Ce, pe.loc, Se);
                  pe.lonlat = we ? we.properties.ct : [ce, ce];
                }
              }
              var Ee, xe, Le = { padded: !0 };
              if (ge.fitbounds === "geojson" && ye.locationmode === "geojson-id") {
                var Qe = oe.computeBbox(oe.getTraceGeojson(ye));
                Ee = [Qe[0], Qe[2]], xe = [Qe[1], Qe[3]];
              } else {
                for (Ee = new Array(be), xe = new Array(be), me = 0; me < be; me++) pe = fe[me], Ee[me] = pe.lonlat[0], xe[me] = pe.lonlat[1];
                Le.ppad = le(ye, be);
              }
              ye._extremes.lon = se(ge.lonaxis._ax, Ee, Le), ye._extremes.lat = se(ge.lataxis._ax, xe, Le);
            }, plot: function(fe, he, me) {
              var pe = he.layers.frontplot.select(".scatterlayer"), ye = ne.makeTraceGroups(pe, me, "trace scattergeo");
              function ge(ve, be) {
                ve.lonlat[0] === ce && te.select(be).remove();
              }
              ye.selectAll("*").remove(), ye.each(function(ve) {
                var be = te.select(this), Ce = ve[0].trace;
                if (ue.hasLines(Ce) || Ce.fill !== "none") {
                  var Se = ae.calcTraceToLineCoords(ve), we = Ce.fill !== "none" ? ae.makePolygon(Se) : ae.makeLine(Se);
                  be.selectAll("path.js-line").data([{ geojson: we, trace: Ce }]).enter().append("path").classed("js-line", !0).style("stroke-miterlimit", 2);
                }
                ue.hasMarkers(Ce) && be.selectAll("path.point").data(ne.identity).enter().append("path").classed("point", !0).each(function(Ee) {
                  ge(Ee, this);
                }), ue.hasText(Ce) && be.selectAll("g").data(ne.identity).enter().append("g").append("text").each(function(Ee) {
                  ge(Ee, this);
                }), de(fe, ve);
              });
            } };
          }, 45852: function(G, J, Z) {
            var te = Z(64726), ne = Z(63821).BADNUM;
            G.exports = function(ee, ae) {
              var oe, se, ce, le, ue, de = ee.cd, fe = ee.xaxis, he = ee.yaxis, me = [], pe = de[0].trace;
              if (!te.hasMarkers(pe) && !te.hasText(pe)) return [];
              if (ae === !1) for (ue = 0; ue < de.length; ue++) de[ue].selected = 0;
              else for (ue = 0; ue < de.length; ue++) (se = (oe = de[ue]).lonlat)[0] !== ne && (ce = fe.c2p(se), le = he.c2p(se), ae.contains([ce, le], null, ue, ee) ? (me.push({ pointNumber: ue, lon: se[0], lat: se[1] }), oe.selected = 1) : oe.selected = 0);
              return me;
            };
          }, 60367: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(78766), ae = Z(9408), oe = ae.stylePoints, se = ae.styleText;
            G.exports = function(ce, le) {
              le && function(ue, de) {
                var fe = de[0].trace, he = de[0].node3;
                he.style("opacity", de[0].trace.opacity), oe(he, fe, ue), se(he, fe, ue), he.selectAll("path.js-line").style("fill", "none").each(function(me) {
                  var pe = te.select(this), ye = me.trace, ge = ye.line || {};
                  pe.call(ee.stroke, ge.color).call(ne.dashLine, ge.dash || "", ge.width || 0), ye.fill !== "none" && pe.call(ee.fill, ye.fillcolor);
                });
              }(ce, le);
            };
          }, 92089: function(G, J, Z) {
            var te = Z(9829), ne = Z(80337), ee = Z(19326), ae = Z(36640), oe = Z(80712).axisHoverFormat, se = Z(87163), ce = Z(62994), le = Z(93049).extendFlat, ue = Z(13582).overrideAll, de = Z(29483).DASHES, fe = ae.line, he = ae.marker, me = he.line, pe = G.exports = ue({ x: ae.x, x0: ae.x0, dx: ae.dx, y: ae.y, y0: ae.y0, dy: ae.dy, xperiod: ae.xperiod, yperiod: ae.yperiod, xperiod0: ae.xperiod0, yperiod0: ae.yperiod0, xperiodalignment: ae.xperiodalignment, yperiodalignment: ae.yperiodalignment, xhoverformat: oe("x"), yhoverformat: oe("y"), text: ae.text, hovertext: ae.hovertext, textposition: ae.textposition, textfont: ne({ noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0, editType: "calc", colorEditType: "style", arrayOk: !0, noNumericWeightValues: !0, variantValues: ["normal", "small-caps"] }), mode: { valType: "flaglist", flags: ["lines", "markers", "text"], extras: ["none"] }, line: { color: fe.color, width: fe.width, shape: { valType: "enumerated", values: ["linear", "hv", "vh", "hvh", "vhv"], dflt: "linear", editType: "plot" }, dash: { valType: "enumerated", values: ce(de), dflt: "solid" } }, marker: le({}, se("marker"), { symbol: he.symbol, angle: he.angle, size: he.size, sizeref: he.sizeref, sizemin: he.sizemin, sizemode: he.sizemode, opacity: he.opacity, colorbar: he.colorbar, line: le({}, se("marker.line"), { width: me.width }) }), connectgaps: ae.connectgaps, fill: le({}, ae.fill, { dflt: "none" }), fillcolor: ee(), selected: { marker: ae.selected.marker, textfont: ae.selected.textfont }, unselected: { marker: ae.unselected.marker, textfont: ae.unselected.textfont }, opacity: te.opacity }, "calc", "nested");
            pe.x.editType = pe.y.editType = pe.x0.editType = pe.y0.editType = "calc+clearAxisTypes", pe.hovertemplate = ae.hovertemplate, pe.texttemplate = ae.texttemplate;
          }, 68258: function(G, J, Z) {
            var te = Z(36544);
            G.exports = { moduleType: "trace", name: "scattergl", basePlotModule: Z(37703), categories: ["gl", "regl", "cartesian", "symbols", "errorBarsOK", "showLegend", "scatter-like"], attributes: Z(92089), supplyDefaults: Z(86590), crossTraceDefaults: Z(53044), colorbar: Z(21146), formatLabels: Z(99185), calc: Z(15293), hoverPoints: te.hoverPoints, selectPoints: Z(17168), meta: {} };
          }, 15293: function(G, J, Z) {
            var te = Z(27549), ne = Z(34809), ee = Z(5975), ae = Z(32919).findExtremes, oe = Z(40528), se = Z(26544), ce = se.calcMarkerSize, le = se.calcAxisExpansion, ue = se.setFirstScatter, de = Z(77272), fe = Z(19937), he = Z(62336), me = Z(63821).BADNUM, pe = Z(29483).TOO_MANY_POINTS;
            function ye(ge, ve, be) {
              var Ce = ge._extremes[ve._id], Se = ae(ve, be._bnds, { padded: !0 });
              Ce.min = Ce.min.concat(Se.min), Ce.max = Ce.max.concat(Se.max);
            }
            G.exports = function(ge, ve) {
              var be, Ce = ge._fullLayout, Se = ve._xA = ee.getFromId(ge, ve.xaxis, "x"), we = ve._yA = ee.getFromId(ge, ve.yaxis, "y"), Ee = Ce._plots[ve.xaxis + ve.yaxis], xe = ve._length, Le = xe >= pe, Qe = 2 * xe, ke = {}, Me = Se.makeCalcdata(ve, "x"), Re = we.makeCalcdata(ve, "y"), De = oe(ve, Se, "x", Me), Pe = oe(ve, we, "y", Re), $e = De.vals, qe = Pe.vals;
              ve._x = $e, ve._y = qe, ve.xperiodalignment && (ve._origX = Me, ve._xStarts = De.starts, ve._xEnds = De.ends), ve.yperiodalignment && (ve._origY = Re, ve._yStarts = Pe.starts, ve._yEnds = Pe.ends);
              var Xe = new Array(Qe), Ve = new Array(xe);
              for (be = 0; be < xe; be++) Xe[2 * be] = $e[be] === me ? NaN : $e[be], Xe[2 * be + 1] = qe[be] === me ? NaN : qe[be], Ve[be] = be;
              if (Se.type === "log") for (be = 0; be < Qe; be += 2) Xe[be] = Se.c2l(Xe[be]);
              if (we.type === "log") for (be = 1; be < Qe; be += 2) Xe[be] = we.c2l(Xe[be]);
              Le && Se.type !== "log" && we.type !== "log" ? ke.tree = te(Xe) : ke.ids = Ve, de(ge, ve);
              var Ze, Ge = function(We, at, ct, et, dt, Je) {
                var it = fe.style(We, ct);
                if (it.marker && (it.marker.positions = et), it.line && et.length > 1 && ne.extendFlat(it.line, fe.linePositions(We, ct, et)), it.errorX || it.errorY) {
                  var st = fe.errorBarPositions(We, ct, et, dt, Je);
                  it.errorX && ne.extendFlat(it.errorX, st.x), it.errorY && ne.extendFlat(it.errorY, st.y);
                }
                return it.text && (ne.extendFlat(it.text, { positions: et }, fe.textPosition(We, ct, it.text, it.marker)), ne.extendFlat(it.textSel, { positions: et }, fe.textPosition(We, ct, it.text, it.markerSel)), ne.extendFlat(it.textUnsel, { positions: et }, fe.textPosition(We, ct, it.text, it.markerUnsel))), it;
              }(ge, 0, ve, Xe, $e, qe), ze = he(ge, Ee);
              return ue(Ce, ve), Le ? Ge.marker && (Ze = Ge.marker.sizeAvg || Math.max(Ge.marker.size, 3)) : Ze = ce(ve, xe), le(ge, ve, Se, we, $e, qe, Ze), Ge.errorX && ye(ve, Se, Ge.errorX), Ge.errorY && ye(ve, we, Ge.errorY), Ge.fill && !ze.fill2d && (ze.fill2d = !0), Ge.marker && !ze.scatter2d && (ze.scatter2d = !0), Ge.line && !ze.line2d && (ze.line2d = !0), !Ge.errorX && !Ge.errorY || ze.error2d || (ze.error2d = !0), Ge.text && !ze.glText && (ze.glText = !0), Ge.marker && (Ge.marker.snap = xe), ze.lineOptions.push(Ge.line), ze.errorXOptions.push(Ge.errorX), ze.errorYOptions.push(Ge.errorY), ze.fillOptions.push(Ge.fill), ze.markerOptions.push(Ge.marker), ze.markerSelectedOptions.push(Ge.markerSel), ze.markerUnselectedOptions.push(Ge.markerUnsel), ze.textOptions.push(Ge.text), ze.textSelectedOptions.push(Ge.textSel), ze.textUnselectedOptions.push(Ge.textUnsel), ze.selectBatch.push([]), ze.unselectBatch.push([]), ke._scene = ze, ke.index = ze.count, ke.x = $e, ke.y = qe, ke.positions = Xe, ze.count++, [{ x: !1, y: !1, t: ke, trace: ve }];
            };
          }, 29483: function(G) {
            G.exports = { TOO_MANY_POINTS: 1e5, SYMBOL_SDF_SIZE: 200, SYMBOL_SIZE: 20, SYMBOL_STROKE: 1, DOT_RE: /-dot/, OPEN_RE: /-open/, DASHES: { solid: [1], dot: [1, 1], dash: [4, 1], longdash: [8, 1], dashdot: [4, 1, 1, 1], longdashdot: [8, 1, 1, 1] } };
          }, 19937: function(G, J, Z) {
            var te = Z(10721), ne = Z(96021), ee = Z(162), ae = Z(33626), oe = Z(34809), se = oe.isArrayOrTypedArray, ce = Z(62203), le = Z(5975), ue = Z(46998).formatColor, de = Z(64726), fe = Z(92527), he = Z(4075), me = Z(29483), pe = Z(20438).DESELECTDIM, ye = { start: 1, left: 1, end: -1, right: -1, middle: 0, center: 0, bottom: 1, top: -1 }, ge = Z(36040).appendArrayPointValue;
            function ve(De, Pe) {
              var $e, qe = De._fullLayout, Xe = Pe._length, Ve = Pe.textfont, Ze = Pe.textposition, Ge = se(Ze) ? Ze : [Ze], ze = Ve.color, We = Ve.size, at = Ve.family, ct = Ve.weight, et = Ve.style, dt = Ve.variant, Je = {}, it = De._context.plotGlPixelRatio, st = Pe.texttemplate;
              if (st) {
                Je.text = [];
                var gt = qe._d3locale, bt = Array.isArray(st), yt = bt ? Math.min(st.length, Xe) : Xe, St = bt ? function(Nt) {
                  return st[Nt];
                } : function() {
                  return st;
                };
                for ($e = 0; $e < yt; $e++) {
                  var Lt = { i: $e }, Rt = Pe._module.formatLabels(Lt, Pe, qe), vt = {};
                  ge(vt, Pe, $e);
                  var $t = Pe._meta || {};
                  Je.text.push(oe.texttemplateString(St($e), Rt, gt, vt, Lt, $t));
                }
              } else se(Pe.text) && Pe.text.length < Xe ? Je.text = Pe.text.slice() : Je.text = Pe.text;
              if (se(Je.text)) for ($e = Je.text.length; $e < Xe; $e++) Je.text[$e] = "";
              for (Je.opacity = Pe.opacity, Je.font = {}, Je.align = [], Je.baseline = [], $e = 0; $e < Ge.length; $e++) {
                var Pt = Ge[$e].split(/\s+/);
                switch (Pt[1]) {
                  case "left":
                    Je.align.push("right");
                    break;
                  case "right":
                    Je.align.push("left");
                    break;
                  default:
                    Je.align.push(Pt[1]);
                }
                switch (Pt[0]) {
                  case "top":
                    Je.baseline.push("bottom");
                    break;
                  case "bottom":
                    Je.baseline.push("top");
                    break;
                  default:
                    Je.baseline.push(Pt[0]);
                }
              }
              if (se(ze)) for (Je.color = new Array(Xe), $e = 0; $e < Xe; $e++) Je.color[$e] = ze[$e];
              else Je.color = ze;
              if (se(We) || Array.isArray(at) || se(ct) || Array.isArray(et) || Array.isArray(dt)) for (Je.font = new Array(Xe), $e = 0; $e < Xe; $e++) {
                var Xt = Je.font[$e] = {};
                Xt.size = (oe.isTypedArray(We) ? We[$e] : se(We) ? te(We[$e]) ? We[$e] : 0 : We) * it, Xt.family = Array.isArray(at) ? at[$e] : at, Xt.weight = be(se(ct) ? ct[$e] : ct), Xt.style = Array.isArray(et) ? et[$e] : et, Xt.variant = Array.isArray(dt) ? dt[$e] : dt;
              }
              else Je.font = { size: We * it, family: at, weight: be(ct), style: et, variant: dt };
              return Je;
            }
            function be(De) {
              return De <= 1e3 ? De > 500 ? "bold" : "normal" : De;
            }
            function Ce(De, Pe) {
              var $e, qe, Xe = Pe._length, Ve = Pe.marker, Ze = {}, Ge = se(Ve.symbol), ze = se(Ve.angle), We = se(Ve.color), at = se(Ve.line.color), ct = se(Ve.opacity), et = se(Ve.size), dt = se(Ve.line.width);
              if (Ge || (qe = he.isOpenSymbol(Ve.symbol)), Ge || We || at || ct || ze) {
                Ze.symbols = new Array(Xe), Ze.angles = new Array(Xe), Ze.colors = new Array(Xe), Ze.borderColors = new Array(Xe);
                var Je = Ve.symbol, it = Ve.angle, st = ue(Ve, Ve.opacity, Xe), gt = ue(Ve.line, Ve.opacity, Xe);
                if (!se(gt[0])) {
                  var bt = gt;
                  for (gt = Array(Xe), $e = 0; $e < Xe; $e++) gt[$e] = bt;
                }
                if (!se(st[0])) {
                  var yt = st;
                  for (st = Array(Xe), $e = 0; $e < Xe; $e++) st[$e] = yt;
                }
                if (!se(Je)) {
                  var St = Je;
                  for (Je = Array(Xe), $e = 0; $e < Xe; $e++) Je[$e] = St;
                }
                if (!se(it)) {
                  var Lt = it;
                  for (it = Array(Xe), $e = 0; $e < Xe; $e++) it[$e] = Lt;
                }
                for (Ze.symbols = Je, Ze.angles = it, Ze.colors = st, Ze.borderColors = gt, $e = 0; $e < Xe; $e++) Ge && (qe = he.isOpenSymbol(Ve.symbol[$e])), qe && (gt[$e] = st[$e].slice(), st[$e] = st[$e].slice(), st[$e][3] = 0);
                for (Ze.opacity = Pe.opacity, Ze.markers = new Array(Xe), $e = 0; $e < Xe; $e++) Ze.markers[$e] = Re({ mx: Ze.symbols[$e], ma: Ze.angles[$e] }, Pe);
              } else qe ? (Ze.color = ee(Ve.color, "uint8"), Ze.color[3] = 0, Ze.borderColor = ee(Ve.color, "uint8")) : (Ze.color = ee(Ve.color, "uint8"), Ze.borderColor = ee(Ve.line.color, "uint8")), Ze.opacity = Pe.opacity * Ve.opacity, Ze.marker = Re({ mx: Ve.symbol, ma: Ve.angle }, Pe);
              var Rt, vt = fe(Pe, 1);
              if (et || dt) {
                var $t, Pt = Ze.sizes = new Array(Xe), Xt = Ze.borderSizes = new Array(Xe), Nt = 0;
                if (et) {
                  for ($e = 0; $e < Xe; $e++) Pt[$e] = vt(Ve.size[$e]), Nt += Pt[$e];
                  $t = Nt / Xe;
                } else for (Rt = vt(Ve.size), $e = 0; $e < Xe; $e++) Pt[$e] = Rt;
                if (dt) for ($e = 0; $e < Xe; $e++) Xt[$e] = Ve.line.width[$e];
                else for (Rt = Ve.line.width, $e = 0; $e < Xe; $e++) Xt[$e] = Rt;
                Ze.sizeAvg = $t;
              } else Ze.size = vt(Ve && Ve.size || 10), Ze.borderSizes = vt(Ve.line.width);
              return Ze;
            }
            function Se(De, Pe, $e) {
              var qe = Pe.marker, Xe = {};
              return $e && ($e.marker && $e.marker.symbol ? Xe = Ce(0, oe.extendFlat({}, qe, $e.marker)) : $e.marker && ($e.marker.size && (Xe.size = $e.marker.size), $e.marker.color && (Xe.colors = $e.marker.color), $e.marker.opacity !== void 0 && (Xe.opacity = $e.marker.opacity))), Xe;
            }
            function we(De, Pe, $e) {
              var qe = {};
              if (!$e) return qe;
              if ($e.textfont) {
                var Xe = { opacity: 1, text: Pe.text, texttemplate: Pe.texttemplate, textposition: Pe.textposition, textfont: oe.extendFlat({}, Pe.textfont) };
                $e.textfont && oe.extendFlat(Xe.textfont, $e.textfont), qe = ve(De, Xe);
              }
              return qe;
            }
            function Ee(De, Pe, $e) {
              var qe = { capSize: 2 * Pe.width * $e, lineWidth: Pe.thickness * $e, color: Pe.color };
              return Pe.copy_ystyle && (qe = De.error_y), qe;
            }
            var xe = me.SYMBOL_SDF_SIZE, Le = me.SYMBOL_SIZE, Qe = me.SYMBOL_STROKE, ke = {}, Me = ce.symbolFuncs[0](0.05 * Le);
            function Re(De, Pe) {
              var $e, qe, Xe = De.mx;
              if (Xe === "circle") return null;
              var Ve = ce.symbolNumber(Xe), Ze = ce.symbolFuncs[Ve % 100], Ge = !!ce.symbolNoDot[Ve % 100], ze = !!ce.symbolNoFill[Ve % 100], We = he.isDotSymbol(Xe);
              if (De.ma && (Xe += "_" + De.ma), ke[Xe]) return ke[Xe];
              var at = ce.getMarkerAngle(De, Pe);
              return $e = We && !Ge ? Ze(1.1 * Le, at) + Me : Ze(Le, at), qe = ne($e, { w: xe, h: xe, viewBox: [-Le, -Le, Le, Le], stroke: ze ? Qe : -Qe }), ke[Xe] = qe, qe || null;
            }
            G.exports = { style: function(De, Pe) {
              var $e, qe = { marker: void 0, markerSel: void 0, markerUnsel: void 0, line: void 0, fill: void 0, errorX: void 0, errorY: void 0, text: void 0, textSel: void 0, textUnsel: void 0 }, Xe = De._context.plotGlPixelRatio;
              if (Pe.visible !== !0) return qe;
              if (de.hasText(Pe) && (qe.text = ve(De, Pe), qe.textSel = we(De, Pe, Pe.selected), qe.textUnsel = we(De, Pe, Pe.unselected)), de.hasMarkers(Pe) && (qe.marker = Ce(0, Pe), qe.markerSel = Se(0, Pe, Pe.selected), qe.markerUnsel = Se(0, Pe, Pe.unselected), !Pe.unselected && se(Pe.marker.opacity))) {
                var Ve = Pe.marker.opacity;
                for (qe.markerUnsel.opacity = new Array(Ve.length), $e = 0; $e < Ve.length; $e++) qe.markerUnsel.opacity[$e] = pe * Ve[$e];
              }
              if (de.hasLines(Pe)) {
                qe.line = { overlay: !0, thickness: Pe.line.width * Xe, color: Pe.line.color, opacity: Pe.opacity };
                var Ze = (me.DASHES[Pe.line.dash] || [1]).slice();
                for ($e = 0; $e < Ze.length; ++$e) Ze[$e] *= Pe.line.width * Xe;
                qe.line.dashes = Ze;
              }
              return Pe.error_x && Pe.error_x.visible && (qe.errorX = Ee(Pe, Pe.error_x, Xe)), Pe.error_y && Pe.error_y.visible && (qe.errorY = Ee(Pe, Pe.error_y, Xe)), Pe.fill && Pe.fill !== "none" && (qe.fill = { closed: !0, fill: Pe.fillcolor, thickness: 0 }), qe;
            }, markerStyle: Ce, markerSelection: Se, linePositions: function(De, Pe, $e) {
              var qe, Xe, Ve = $e.length, Ze = Ve / 2;
              if (de.hasLines(Pe) && Ze) if (Pe.line.shape === "hv") {
                for (qe = [], Xe = 0; Xe < Ze - 1; Xe++) isNaN($e[2 * Xe]) || isNaN($e[2 * Xe + 1]) ? qe.push(NaN, NaN, NaN, NaN) : (qe.push($e[2 * Xe], $e[2 * Xe + 1]), isNaN($e[2 * Xe + 2]) || isNaN($e[2 * Xe + 3]) ? qe.push(NaN, NaN) : qe.push($e[2 * Xe + 2], $e[2 * Xe + 1]));
                qe.push($e[Ve - 2], $e[Ve - 1]);
              } else if (Pe.line.shape === "hvh") {
                for (qe = [], Xe = 0; Xe < Ze - 1; Xe++) if (isNaN($e[2 * Xe]) || isNaN($e[2 * Xe + 1]) || isNaN($e[2 * Xe + 2]) || isNaN($e[2 * Xe + 3])) isNaN($e[2 * Xe]) || isNaN($e[2 * Xe + 1]) ? qe.push(NaN, NaN) : qe.push($e[2 * Xe], $e[2 * Xe + 1]), qe.push(NaN, NaN);
                else {
                  var Ge = ($e[2 * Xe] + $e[2 * Xe + 2]) / 2;
                  qe.push($e[2 * Xe], $e[2 * Xe + 1], Ge, $e[2 * Xe + 1], Ge, $e[2 * Xe + 3]);
                }
                qe.push($e[Ve - 2], $e[Ve - 1]);
              } else if (Pe.line.shape === "vhv") {
                for (qe = [], Xe = 0; Xe < Ze - 1; Xe++) if (isNaN($e[2 * Xe]) || isNaN($e[2 * Xe + 1]) || isNaN($e[2 * Xe + 2]) || isNaN($e[2 * Xe + 3])) isNaN($e[2 * Xe]) || isNaN($e[2 * Xe + 1]) ? qe.push(NaN, NaN) : qe.push($e[2 * Xe], $e[2 * Xe + 1]), qe.push(NaN, NaN);
                else {
                  var ze = ($e[2 * Xe + 1] + $e[2 * Xe + 3]) / 2;
                  qe.push($e[2 * Xe], $e[2 * Xe + 1], $e[2 * Xe], ze, $e[2 * Xe + 2], ze);
                }
                qe.push($e[Ve - 2], $e[Ve - 1]);
              } else if (Pe.line.shape === "vh") {
                for (qe = [], Xe = 0; Xe < Ze - 1; Xe++) isNaN($e[2 * Xe]) || isNaN($e[2 * Xe + 1]) ? qe.push(NaN, NaN, NaN, NaN) : (qe.push($e[2 * Xe], $e[2 * Xe + 1]), isNaN($e[2 * Xe + 2]) || isNaN($e[2 * Xe + 3]) ? qe.push(NaN, NaN) : qe.push($e[2 * Xe], $e[2 * Xe + 3]));
                qe.push($e[Ve - 2], $e[Ve - 1]);
              } else qe = $e;
              var We = !1;
              for (Xe = 0; Xe < qe.length; Xe++) if (isNaN(qe[Xe])) {
                We = !0;
                break;
              }
              var at = We || qe.length > me.TOO_MANY_POINTS || de.hasMarkers(Pe) ? "rect" : "round";
              if (We && Pe.connectgaps) {
                var ct = qe[0], et = qe[1];
                for (Xe = 0; Xe < qe.length; Xe += 2) isNaN(qe[Xe]) || isNaN(qe[Xe + 1]) ? (qe[Xe] = ct, qe[Xe + 1] = et) : (ct = qe[Xe], et = qe[Xe + 1]);
              }
              return { join: at, positions: qe };
            }, errorBarPositions: function(De, Pe, $e, qe, Xe) {
              var Ve = ae.getComponentMethod("errorbars", "makeComputeError"), Ze = le.getFromId(De, Pe.xaxis, "x"), Ge = le.getFromId(De, Pe.yaxis, "y"), ze = $e.length / 2, We = {};
              function at(ct, et) {
                var dt = et._id.charAt(0), Je = Pe["error_" + dt];
                if (Je && Je.visible && (et.type === "linear" || et.type === "log")) {
                  for (var it = Ve(Je), st = { x: 0, y: 1 }[dt], gt = { x: [0, 1, 2, 3], y: [2, 3, 0, 1] }[dt], bt = new Float64Array(4 * ze), yt = 1 / 0, St = -1 / 0, Lt = 0, Rt = 0; Lt < ze; Lt++, Rt += 4) {
                    var vt = ct[Lt];
                    if (te(vt)) {
                      var $t = $e[2 * Lt + st], Pt = it(vt, Lt), Xt = Pt[0], Nt = Pt[1];
                      if (te(Xt) && te(Nt)) {
                        var Gt = vt - Xt, Ft = vt + Nt;
                        bt[Rt + gt[0]] = $t - et.c2l(Gt), bt[Rt + gt[1]] = et.c2l(Ft) - $t, bt[Rt + gt[2]] = 0, bt[Rt + gt[3]] = 0, yt = Math.min(yt, vt - Xt), St = Math.max(St, vt + Nt);
                      }
                    }
                  }
                  We[dt] = { positions: $e, errors: bt, _bnds: [yt, St] };
                }
              }
              return at(qe, Ze), at(Xe, Ge), We;
            }, textPosition: function(De, Pe, $e, qe) {
              var Xe, Ve = Pe._length, Ze = {};
              if (de.hasMarkers(Pe)) {
                var Ge = $e.font, ze = $e.align, We = $e.baseline;
                for (Ze.offset = new Array(Ve), Xe = 0; Xe < Ve; Xe++) {
                  var at = qe.sizes ? qe.sizes[Xe] : qe.size, ct = se(Ge) ? Ge[Xe].size : Ge.size, et = se(ze) ? ze.length > 1 ? ze[Xe] : ze[0] : ze, dt = se(We) ? We.length > 1 ? We[Xe] : We[0] : We, Je = ye[et], it = ye[dt], st = at ? at / 0.8 + 1 : 0, gt = -it * st - 0.5 * it;
                  Ze.offset[Xe] = [Je * st / ct, gt / ct];
                }
              }
              return Ze;
            } };
          }, 86590: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626), ee = Z(4075), ae = Z(92089), oe = Z(32660), se = Z(64726), ce = Z(99867), le = Z(99669), ue = Z(24272), de = Z(98168), fe = Z(54114), he = Z(663);
            G.exports = function(me, pe, ye, ge) {
              function ve(Qe, ke) {
                return te.coerce(me, pe, ae, Qe, ke);
              }
              var be = !!me.marker && ee.isOpenSymbol(me.marker.symbol), Ce = se.isBubble(me), Se = ce(me, pe, ge, ve);
              if (Se) {
                le(me, pe, ge, ve), ve("xhoverformat"), ve("yhoverformat");
                var we = Se < oe.PTS_LINESONLY ? "lines+markers" : "lines";
                ve("text"), ve("hovertext"), ve("hovertemplate"), ve("mode", we), se.hasMarkers(pe) && (ue(me, pe, ye, ge, ve, { noAngleRef: !0, noStandOff: !0 }), ve("marker.line.width", be || Ce ? 1 : 0)), se.hasLines(pe) && (ve("connectgaps"), de(me, pe, ye, ge, ve), ve("line.shape")), se.hasText(pe) && (ve("texttemplate"), he(me, pe, ge, ve, { noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0 }));
                var Ee = (pe.line || {}).color, xe = (pe.marker || {}).color;
                ve("fill"), pe.fill !== "none" && fe(me, pe, ye, ve);
                var Le = ne.getComponentMethod("errorbars", "supplyDefaults");
                Le(me, pe, Ee || xe || ye, { axis: "y" }), Le(me, pe, Ee || xe || ye, { axis: "x", inherit: "y" }), te.coerceSelectionMarkerOpacity(pe, ve);
              } else pe.visible = !1;
            };
          }, 85686: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(20438).DESELECTDIM;
            G.exports = { styleTextSelection: function(ae) {
              var oe, se, ce = ae[0], le = ce.trace, ue = ce.t, de = ue._scene, fe = ue.index, he = de.selectBatch[fe], me = de.unselectBatch[fe], pe = de.textOptions[fe], ye = de.textSelectedOptions[fe] || {}, ge = de.textUnselectedOptions[fe] || {}, ve = te.extendFlat({}, pe);
              if (he.length || me.length) {
                var be = ye.color, Ce = ge.color, Se = pe.color, we = te.isArrayOrTypedArray(Se);
                for (ve.color = new Array(le._length), oe = 0; oe < he.length; oe++) se = he[oe], ve.color[se] = be || (we ? Se[se] : Se);
                for (oe = 0; oe < me.length; oe++) {
                  se = me[oe];
                  var Ee = we ? Se[se] : Se;
                  ve.color[se] = Ce || (be ? Ee : ne.addOpacity(Ee, ee));
                }
              }
              de.glText[fe].update(ve);
            } };
          }, 99185: function(G, J, Z) {
            var te = Z(15294);
            G.exports = function(ne, ee, ae) {
              var oe = ne.i;
              return "x" in ne || (ne.x = ee._x[oe]), "y" in ne || (ne.y = ee._y[oe]), te(ne, ee, ae);
            };
          }, 4075: function(G, J, Z) {
            var te = Z(29483);
            J.isOpenSymbol = function(ne) {
              return typeof ne == "string" ? te.OPEN_RE.test(ne) : ne % 200 > 100;
            }, J.isDotSymbol = function(ne) {
              return typeof ne == "string" ? te.DOT_RE.test(ne) : ne > 200;
            };
          }, 36544: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(11539);
            function ae(oe, se, ce, le) {
              var ue = oe.xa, de = oe.ya, fe = oe.distance, he = oe.dxy, me = oe.index, pe = { pointNumber: me, x: se[me], y: ce[me] };
              pe.tx = ne.isArrayOrTypedArray(le.text) ? le.text[me] : le.text, pe.htx = Array.isArray(le.hovertext) ? le.hovertext[me] : le.hovertext, pe.data = Array.isArray(le.customdata) ? le.customdata[me] : le.customdata, pe.tp = Array.isArray(le.textposition) ? le.textposition[me] : le.textposition;
              var ye = le.textfont;
              ye && (pe.ts = ne.isArrayOrTypedArray(ye.size) ? ye.size[me] : ye.size, pe.tc = ne.isArrayOrTypedArray(ye.color) ? ye.color[me] : ye.color, pe.tf = Array.isArray(ye.family) ? ye.family[me] : ye.family, pe.tw = Array.isArray(ye.weight) ? ye.weight[me] : ye.weight, pe.ty = Array.isArray(ye.style) ? ye.style[me] : ye.style, pe.tv = Array.isArray(ye.variant) ? ye.variant[me] : ye.variant);
              var ge = le.marker;
              ge && (pe.ms = ne.isArrayOrTypedArray(ge.size) ? ge.size[me] : ge.size, pe.mo = ne.isArrayOrTypedArray(ge.opacity) ? ge.opacity[me] : ge.opacity, pe.mx = ne.isArrayOrTypedArray(ge.symbol) ? ge.symbol[me] : ge.symbol, pe.ma = ne.isArrayOrTypedArray(ge.angle) ? ge.angle[me] : ge.angle, pe.mc = ne.isArrayOrTypedArray(ge.color) ? ge.color[me] : ge.color);
              var ve = ge && ge.line;
              ve && (pe.mlc = Array.isArray(ve.color) ? ve.color[me] : ve.color, pe.mlw = ne.isArrayOrTypedArray(ve.width) ? ve.width[me] : ve.width);
              var be = ge && ge.gradient;
              be && be.type !== "none" && (pe.mgt = Array.isArray(be.type) ? be.type[me] : be.type, pe.mgc = Array.isArray(be.color) ? be.color[me] : be.color);
              var Ce = ue.c2p(pe.x, !0), Se = de.c2p(pe.y, !0), we = pe.mrc || 1, Ee = le.hoverlabel;
              Ee && (pe.hbg = Array.isArray(Ee.bgcolor) ? Ee.bgcolor[me] : Ee.bgcolor, pe.hbc = Array.isArray(Ee.bordercolor) ? Ee.bordercolor[me] : Ee.bordercolor, pe.hts = ne.isArrayOrTypedArray(Ee.font.size) ? Ee.font.size[me] : Ee.font.size, pe.htc = Array.isArray(Ee.font.color) ? Ee.font.color[me] : Ee.font.color, pe.htf = Array.isArray(Ee.font.family) ? Ee.font.family[me] : Ee.font.family, pe.hnl = ne.isArrayOrTypedArray(Ee.namelength) ? Ee.namelength[me] : Ee.namelength);
              var xe = le.hoverinfo;
              xe && (pe.hi = Array.isArray(xe) ? xe[me] : xe);
              var Le = le.hovertemplate;
              Le && (pe.ht = Array.isArray(Le) ? Le[me] : Le);
              var Qe = {};
              Qe[oe.index] = pe;
              var ke = le._origX, Me = le._origY, Re = ne.extendFlat({}, oe, { color: ee(le, pe), x0: Ce - we, x1: Ce + we, xLabelVal: ke ? ke[me] : pe.x, y0: Se - we, y1: Se + we, yLabelVal: Me ? Me[me] : pe.y, cd: Qe, distance: fe, spikeDistance: he, hovertemplate: pe.ht });
              return pe.htx ? Re.text = pe.htx : pe.tx ? Re.text = pe.tx : le.text && (Re.text = le.text), ne.fillText(pe, le, Re), te.getComponentMethod("errorbars", "hoverInfo")(pe, le, Re), Re;
            }
            G.exports = { hoverPoints: function(oe, se, ce, le) {
              var ue, de, fe, he, me, pe, ye, ge, ve, be, Ce = oe.cd, Se = Ce[0].t, we = Ce[0].trace, Ee = oe.xa, xe = oe.ya, Le = Se.x, Qe = Se.y, ke = Ee.c2p(se), Me = xe.c2p(ce), Re = oe.distance;
              if (Se.tree) {
                var De = Ee.p2c(ke - Re), Pe = Ee.p2c(ke + Re), $e = xe.p2c(Me - Re), qe = xe.p2c(Me + Re);
                ue = le === "x" ? Se.tree.range(Math.min(De, Pe), Math.min(xe._rl[0], xe._rl[1]), Math.max(De, Pe), Math.max(xe._rl[0], xe._rl[1])) : Se.tree.range(Math.min(De, Pe), Math.min($e, qe), Math.max(De, Pe), Math.max($e, qe));
              } else ue = Se.ids;
              var Xe = Re;
              if (le === "x") {
                var Ve = !!we.xperiodalignment, Ze = !!we.yperiodalignment;
                for (pe = 0; pe < ue.length; pe++) {
                  if (he = Le[de = ue[pe]], ye = Math.abs(Ee.c2p(he) - ke), Ve) {
                    var Ge = Ee.c2p(we._xStarts[de]), ze = Ee.c2p(we._xEnds[de]);
                    ye = ke >= Math.min(Ge, ze) && ke <= Math.max(Ge, ze) ? 0 : 1 / 0;
                  }
                  if (ye < Xe) {
                    if (Xe = ye, me = Qe[de], ge = xe.c2p(me) - Me, Ze) {
                      var We = xe.c2p(we._yStarts[de]), at = xe.c2p(we._yEnds[de]);
                      ge = Me >= Math.min(We, at) && Me <= Math.max(We, at) ? 0 : 1 / 0;
                    }
                    be = Math.sqrt(ye * ye + ge * ge), fe = ue[pe];
                  }
                }
              } else for (pe = ue.length - 1; pe > -1; pe--) he = Le[de = ue[pe]], me = Qe[de], ye = Ee.c2p(he) - ke, ge = xe.c2p(me) - Me, (ve = Math.sqrt(ye * ye + ge * ge)) < Xe && (Xe = be = ve, fe = de);
              return oe.index = fe, oe.distance = Xe, oe.dxy = be, fe === void 0 ? [oe] : [ae(oe, Le, Qe, we)];
            }, calcHover: ae };
          }, 52378: function(G, J, Z) {
            var te = Z(68258);
            te.plot = Z(47731), G.exports = te;
          }, 47731: function(G, J, Z) {
            var te = Z(62172), ne = Z(49478), ee = Z(29978), ae = Z(74024), oe = Z(34809), se = Z(70414).selectMode, ce = Z(22459), le = Z(64726), ue = Z(17210), de = Z(85686).styleTextSelection, fe = {};
            function he(me, pe, ye, ge) {
              var ve = me._size, be = me.width * ge, Ce = me.height * ge, Se = ve.l * ge, we = ve.b * ge, Ee = ve.r * ge, xe = ve.t * ge, Le = ve.w * ge, Qe = ve.h * ge;
              return [Se + pe.domain[0] * Le, we + ye.domain[0] * Qe, be - Ee - (1 - pe.domain[1]) * Le, Ce - xe - (1 - ye.domain[1]) * Qe];
            }
            (G.exports = function(me, pe, ye) {
              if (ye.length) {
                var ge, ve, be = me._fullLayout, Ce = pe._scene, Se = pe.xaxis, we = pe.yaxis;
                if (Ce) if (ce(me, ["ANGLE_instanced_arrays", "OES_element_index_uint"], fe)) {
                  var Ee = Ce.count, xe = be._glcanvas.data()[0].regl;
                  if (ue(me, pe, ye), Ce.dirty) {
                    if (!Ce.line2d && !Ce.error2d || Ce.scatter2d || Ce.fill2d || Ce.glText || xe.clear({}), Ce.error2d === !0 && (Ce.error2d = ee(xe)), Ce.line2d === !0 && (Ce.line2d = ne(xe)), Ce.scatter2d === !0 && (Ce.scatter2d = te(xe)), Ce.fill2d === !0 && (Ce.fill2d = ne(xe)), Ce.glText === !0) for (Ce.glText = new Array(Ee), ge = 0; ge < Ee; ge++) Ce.glText[ge] = new ae(xe);
                    if (Ce.glText) {
                      if (Ee > Ce.glText.length) {
                        var Le = Ee - Ce.glText.length;
                        for (ge = 0; ge < Le; ge++) Ce.glText.push(new ae(xe));
                      } else if (Ee < Ce.glText.length) {
                        var Qe = Ce.glText.length - Ee;
                        Ce.glText.splice(Ee, Qe).forEach(function(st) {
                          st.destroy();
                        });
                      }
                      for (ge = 0; ge < Ee; ge++) Ce.glText[ge].update(Ce.textOptions[ge]);
                    }
                    if (Ce.line2d && (Ce.line2d.update(Ce.lineOptions), Ce.lineOptions = Ce.lineOptions.map(function(st) {
                      if (st && st.positions) {
                        for (var gt = st.positions, bt = 0; bt < gt.length && (isNaN(gt[bt]) || isNaN(gt[bt + 1])); ) bt += 2;
                        for (var yt = gt.length - 2; yt > bt && (isNaN(gt[yt]) || isNaN(gt[yt + 1])); ) yt -= 2;
                        st.positions = gt.slice(bt, yt + 2);
                      }
                      return st;
                    }), Ce.line2d.update(Ce.lineOptions)), Ce.error2d) {
                      var ke = (Ce.errorXOptions || []).concat(Ce.errorYOptions || []);
                      Ce.error2d.update(ke);
                    }
                    Ce.scatter2d && Ce.scatter2d.update(Ce.markerOptions), Ce.fillOrder = oe.repeat(null, Ee), Ce.fill2d && (Ce.fillOptions = Ce.fillOptions.map(function(st, gt) {
                      var bt = ye[gt];
                      if (st && bt && bt[0] && bt[0].trace) {
                        var yt, St, Lt = bt[0], Rt = Lt.trace, vt = Lt.t, $t = Ce.lineOptions[gt], Pt = [];
                        Rt._ownfill && Pt.push(gt), Rt._nexttrace && Pt.push(gt + 1), Pt.length && (Ce.fillOrder[gt] = Pt);
                        var Xt, Nt, Gt = [], Ft = $t && $t.positions || vt.positions;
                        if (Rt.fill === "tozeroy") {
                          for (Xt = 0; Xt < Ft.length && isNaN(Ft[Xt + 1]); ) Xt += 2;
                          for (Nt = Ft.length - 2; Nt > Xt && isNaN(Ft[Nt + 1]); ) Nt -= 2;
                          Ft[Xt + 1] !== 0 && (Gt = [Ft[Xt], 0]), Gt = Gt.concat(Ft.slice(Xt, Nt + 2)), Ft[Nt + 1] !== 0 && (Gt = Gt.concat([Ft[Nt], 0]));
                        } else if (Rt.fill === "tozerox") {
                          for (Xt = 0; Xt < Ft.length && isNaN(Ft[Xt]); ) Xt += 2;
                          for (Nt = Ft.length - 2; Nt > Xt && isNaN(Ft[Nt]); ) Nt -= 2;
                          Ft[Xt] !== 0 && (Gt = [0, Ft[Xt + 1]]), Gt = Gt.concat(Ft.slice(Xt, Nt + 2)), Ft[Nt] !== 0 && (Gt = Gt.concat([0, Ft[Nt + 1]]));
                        } else if (Rt.fill === "toself" || Rt.fill === "tonext") {
                          for (Gt = [], yt = 0, st.splitNull = !0, St = 0; St < Ft.length; St += 2) (isNaN(Ft[St]) || isNaN(Ft[St + 1])) && ((Gt = Gt.concat(Ft.slice(yt, St))).push(Ft[yt], Ft[yt + 1]), Gt.push(null, null), yt = St + 2);
                          Gt = Gt.concat(Ft.slice(yt)), yt && Gt.push(Ft[yt], Ft[yt + 1]);
                        } else {
                          var Tt = Rt._nexttrace;
                          if (Tt) {
                            var It = Ce.lineOptions[gt + 1];
                            if (It) {
                              var Ct = It.positions;
                              if (Rt.fill === "tonexty") {
                                for (Gt = Ft.slice(), gt = Math.floor(Ct.length / 2); gt--; ) {
                                  var Kt = Ct[2 * gt], Vt = Ct[2 * gt + 1];
                                  isNaN(Kt) || isNaN(Vt) || Gt.push(Kt, Vt);
                                }
                                st.fill = Tt.fillcolor;
                              }
                            }
                          }
                        }
                        if (Rt._prevtrace && Rt._prevtrace.fill === "tonext") {
                          var Yt = Ce.lineOptions[gt - 1].positions, sn = Gt.length / 2, hn = [yt = sn];
                          for (St = 0; St < Yt.length; St += 2) (isNaN(Yt[St]) || isNaN(Yt[St + 1])) && (hn.push(St / 2 + sn + 1), yt = St + 2);
                          Gt = Gt.concat(Yt), st.hole = hn;
                        }
                        return st.fillmode = Rt.fill, st.opacity = Rt.opacity, st.positions = Gt, st;
                      }
                    }), Ce.fill2d.update(Ce.fillOptions));
                  }
                  var Me = be.dragmode, Re = se(Me), De = be.clickmode.indexOf("select") > -1;
                  for (ge = 0; ge < Ee; ge++) {
                    var Pe = ye[ge][0], $e = Pe.trace, qe = Pe.t, Xe = qe.index, Ve = $e._length, Ze = qe.x, Ge = qe.y;
                    if ($e.selectedpoints || Re || De) {
                      if (Re || (Re = !0), $e.selectedpoints) {
                        var ze = Ce.selectBatch[Xe] = oe.selIndices2selPoints($e), We = {};
                        for (ve = 0; ve < ze.length; ve++) We[ze[ve]] = 1;
                        var at = [];
                        for (ve = 0; ve < Ve; ve++) We[ve] || at.push(ve);
                        Ce.unselectBatch[Xe] = at;
                      }
                      var ct = qe.xpx = new Array(Ve), et = qe.ypx = new Array(Ve);
                      for (ve = 0; ve < Ve; ve++) ct[ve] = Se.c2p(Ze[ve]), et[ve] = we.c2p(Ge[ve]);
                    } else qe.xpx = qe.ypx = null;
                  }
                  if (Re) {
                    if (Ce.select2d || (Ce.select2d = te(be._glcanvas.data()[1].regl)), Ce.scatter2d) {
                      var dt = new Array(Ee);
                      for (ge = 0; ge < Ee; ge++) dt[ge] = Ce.selectBatch[ge].length || Ce.unselectBatch[ge].length ? Ce.markerUnselectedOptions[ge] : {};
                      Ce.scatter2d.update(dt);
                    }
                    Ce.select2d && (Ce.select2d.update(Ce.markerOptions), Ce.select2d.update(Ce.markerSelectedOptions)), Ce.glText && ye.forEach(function(st) {
                      var gt = ((st || [])[0] || {}).trace || {};
                      le.hasText(gt) && de(st);
                    });
                  } else Ce.scatter2d && Ce.scatter2d.update(Ce.markerOptions);
                  var Je = { viewport: he(be, Se, we, me._context.plotGlPixelRatio), range: [(Se._rl || Se.range)[0], (we._rl || we.range)[0], (Se._rl || Se.range)[1], (we._rl || we.range)[1]] }, it = oe.repeat(Je, Ce.count);
                  Ce.fill2d && Ce.fill2d.update(it), Ce.line2d && Ce.line2d.update(it), Ce.error2d && Ce.error2d.update(it.concat(it)), Ce.scatter2d && Ce.scatter2d.update(it), Ce.select2d && Ce.select2d.update(it), Ce.glText && Ce.glText.forEach(function(st) {
                    st.update(Je);
                  });
                } else Ce.init();
              }
            }).reglPrecompiled = fe;
          }, 62336: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              var ae = ee._scene, oe = { count: 0, dirty: !0, lineOptions: [], fillOptions: [], markerOptions: [], markerSelectedOptions: [], markerUnselectedOptions: [], errorXOptions: [], errorYOptions: [], textOptions: [], textSelectedOptions: [], textUnselectedOptions: [], selectBatch: [], unselectBatch: [] }, se = { fill2d: !1, scatter2d: !1, error2d: !1, line2d: !1, glText: !1, select2d: !1 };
              return ee._scene || ((ae = ee._scene = {}).init = function() {
                te.extendFlat(ae, se, oe);
              }, ae.init(), ae.update = function(ce) {
                var le = te.repeat(ce, ae.count);
                if (ae.fill2d && ae.fill2d.update(le), ae.scatter2d && ae.scatter2d.update(le), ae.line2d && ae.line2d.update(le), ae.error2d && ae.error2d.update(le.concat(le)), ae.select2d && ae.select2d.update(le), ae.glText) for (var ue = 0; ue < ae.count; ue++) ae.glText[ue].update(ce);
              }, ae.draw = function() {
                for (var ce = ae.count, le = ae.fill2d, ue = ae.error2d, de = ae.line2d, fe = ae.scatter2d, he = ae.glText, me = ae.select2d, pe = ae.selectBatch, ye = ae.unselectBatch, ge = 0; ge < ce; ge++) {
                  if (le && ae.fillOrder[ge] && le.draw(ae.fillOrder[ge]), de && ae.lineOptions[ge] && de.draw(ge), ue && (ae.errorXOptions[ge] && ue.draw(ge), ae.errorYOptions[ge] && ue.draw(ge + ce)), fe && ae.markerOptions[ge]) if (ye[ge].length) {
                    var ve = te.repeat([], ae.count);
                    ve[ge] = ye[ge], fe.draw(ve);
                  } else pe[ge].length || fe.draw(ge);
                  he[ge] && ae.textOptions[ge] && he[ge].render();
                }
                me && me.draw(pe), ae.dirty = !1;
              }, ae.destroy = function() {
                ae.fill2d && ae.fill2d.destroy && ae.fill2d.destroy(), ae.scatter2d && ae.scatter2d.destroy && ae.scatter2d.destroy(), ae.error2d && ae.error2d.destroy && ae.error2d.destroy(), ae.line2d && ae.line2d.destroy && ae.line2d.destroy(), ae.select2d && ae.select2d.destroy && ae.select2d.destroy(), ae.glText && ae.glText.forEach(function(ce) {
                  ce.destroy && ce.destroy();
                }), ae.lineOptions = null, ae.fillOptions = null, ae.markerOptions = null, ae.markerSelectedOptions = null, ae.markerUnselectedOptions = null, ae.errorXOptions = null, ae.errorYOptions = null, ae.textOptions = null, ae.textSelectedOptions = null, ae.textUnselectedOptions = null, ae.selectBatch = null, ae.unselectBatch = null, ee._scene = null;
              }), ae.dirty || te.extendFlat(ae, oe), ae;
            };
          }, 17168: function(G, J, Z) {
            var te = Z(64726), ne = Z(85686).styleTextSelection;
            G.exports = function(ee, ae) {
              var oe = ee.cd, se = ee.xaxis, ce = ee.yaxis, le = [], ue = oe[0].trace, de = oe[0].t, fe = ue._length, he = de.x, me = de.y, pe = de._scene, ye = de.index;
              if (!pe) return le;
              var ge = te.hasText(ue), ve = te.hasMarkers(ue), be = !ve && !ge;
              if (ue.visible !== !0 || be) return le;
              var Ce = [], Se = [];
              if (ae !== !1 && !ae.degenerate) for (var we = 0; we < fe; we++) ae.contains([de.xpx[we], de.ypx[we]], !1, we, ee) ? (Ce.push(we), le.push({ pointNumber: we, x: se.c2d(he[we]), y: ce.c2d(me[we]) })) : Se.push(we);
              if (ve) {
                var Ee = pe.scatter2d;
                if (Ce.length || Se.length) {
                  if (!pe.selectBatch[ye].length && !pe.unselectBatch[ye].length) {
                    var xe = new Array(pe.count);
                    xe[ye] = pe.markerUnselectedOptions[ye], Ee.update.apply(Ee, xe);
                  }
                } else {
                  var Le = new Array(pe.count);
                  Le[ye] = pe.markerOptions[ye], Ee.update.apply(Ee, Le);
                }
              }
              return pe.selectBatch[ye] = Ce, pe.unselectBatch[ye] = Se, ge && ne(oe), le;
            };
          }, 71388: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(19326), ae = Z(6893), oe = Z(36640), se = Z(8257), ce = Z(9829), le = Z(87163), ue = Z(93049).extendFlat, de = Z(13582).overrideAll, fe = Z(8257), he = ae.line, me = ae.marker;
            G.exports = de({ lon: ae.lon, lat: ae.lat, cluster: { enabled: { valType: "boolean" }, maxzoom: ue({}, fe.layers.maxzoom, {}), step: { valType: "number", arrayOk: !0, dflt: -1, min: -1 }, size: { valType: "number", arrayOk: !0, dflt: 20, min: 0 }, color: { valType: "color", arrayOk: !0 }, opacity: ue({}, me.opacity, { dflt: 1 }) }, mode: ue({}, oe.mode, { dflt: "markers" }), text: ue({}, oe.text, {}), texttemplate: ne({ editType: "plot" }, { keys: ["lat", "lon", "text"] }), hovertext: ue({}, oe.hovertext, {}), line: { color: he.color, width: he.width }, connectgaps: oe.connectgaps, marker: ue({ symbol: { valType: "string", dflt: "circle", arrayOk: !0 }, angle: { valType: "number", dflt: "auto", arrayOk: !0 }, allowoverlap: { valType: "boolean", dflt: !1 }, opacity: me.opacity, size: me.size, sizeref: me.sizeref, sizemin: me.sizemin, sizemode: me.sizemode }, le("marker")), fill: ae.fill, fillcolor: ee(), textfont: se.layers.symbol.textfont, textposition: se.layers.symbol.textposition, below: { valType: "string" }, selected: { marker: oe.selected.marker }, unselected: { marker: oe.unselected.marker }, hoverinfo: ue({}, ce.hoverinfo, { flags: ["lon", "lat", "text", "name"] }), hovertemplate: te() }, "calc", "nested");
          }, 13624: function(G) {
            var J = ["Metropolis Black Italic", "Metropolis Black", "Metropolis Bold Italic", "Metropolis Bold", "Metropolis Extra Bold Italic", "Metropolis Extra Bold", "Metropolis Extra Light Italic", "Metropolis Extra Light", "Metropolis Light Italic", "Metropolis Light", "Metropolis Medium Italic", "Metropolis Medium", "Metropolis Regular Italic", "Metropolis Regular", "Metropolis Semi Bold Italic", "Metropolis Semi Bold", "Metropolis Thin Italic", "Metropolis Thin", "Open Sans Bold Italic", "Open Sans Bold", "Open Sans Extrabold Italic", "Open Sans Extrabold", "Open Sans Italic", "Open Sans Light Italic", "Open Sans Light", "Open Sans Regular", "Open Sans Semibold Italic", "Open Sans Semibold", "Klokantech Noto Sans Bold", "Klokantech Noto Sans CJK Bold", "Klokantech Noto Sans CJK Regular", "Klokantech Noto Sans Italic", "Klokantech Noto Sans Regular"];
            G.exports = { isSupportedFont: function(Z) {
              return J.indexOf(Z) !== -1;
            } };
          }, 76717: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(63821).BADNUM, ae = Z(39532), oe = Z(88856), se = Z(62203), ce = Z(92527), le = Z(64726), ue = Z(13624).isSupportedFont, de = Z(4657), fe = Z(36040).appendArrayPointValue, he = Z(30635).NEWLINES, me = Z(30635).BR_TAG_ALL;
            function pe(Se) {
              return { type: Se, geojson: ae.makeBlank(), layout: { visibility: "none" }, filter: null, paint: {} };
            }
            function ye(Se, we) {
              return ne.isArrayOrTypedArray(Se) ? we ? function(Ee) {
                return te(Se[Ee]) ? +Se[Ee] : 0;
              } : function(Ee) {
                return Se[Ee];
              } : Se ? function() {
                return Se;
              } : ge;
            }
            function ge() {
              return "";
            }
            function ve(Se) {
              return Se[0] === ee;
            }
            function be(Se, we) {
              var Ee;
              if (ne.isArrayOrTypedArray(Se) && ne.isArrayOrTypedArray(we)) {
                Ee = ["step", ["get", "point_count"], Se[0]];
                for (var xe = 1; xe < Se.length; xe++) Ee.push(we[xe - 1], Se[xe]);
              } else Ee = Se;
              return Ee;
            }
            function Ce(Se) {
              var we = Se.textfont, Ee = we.family, xe = we.style, Le = we.weight, Qe = Ee.split(" "), ke = Qe[Qe.length - 1] === "Italic";
              ke && Qe.pop(), ke = ke || xe === "italic";
              var Me = Qe.join(" ");
              return Le === "bold" && Qe.indexOf("Bold") === -1 ? Me += " Bold" : Le <= 1e3 && (Qe[0] === "Metropolis" ? (Me = "Metropolis", Me += Le > 850 ? " Black" : Le > 750 ? " Extra Bold" : Le > 650 ? " Bold" : Le > 550 ? " Semi Bold" : Le > 450 ? " Medium" : Le > 350 ? " Regular" : Le > 250 ? " Light" : Le > 150 ? " Extra Light" : " Thin") : Qe.slice(0, 2).join(" ") === "Open Sans" ? (Me = "Open Sans", Me += Le > 750 ? " Extrabold" : Le > 650 ? " Bold" : Le > 550 ? " Semibold" : Le > 350 ? " Regular" : " Light") : Qe.slice(0, 3).join(" ") === "Klokantech Noto Sans" && (Me = "Klokantech Noto Sans", Qe[3] === "CJK" && (Me += " CJK"), Me += Le > 500 ? " Bold" : " Regular")), ke && (Me += " Italic"), Me === "Open Sans Regular Italic" ? Me = "Open Sans Italic" : Me === "Open Sans Regular Bold" ? Me = "Open Sans Bold" : Me === "Open Sans Regular Bold Italic" ? Me = "Open Sans Bold Italic" : Me === "Klokantech Noto Sans Regular Italic" && (Me = "Klokantech Noto Sans Italic"), ue(Me) || (Me = Ee), Me.split(", ");
            }
            G.exports = function(Se, we) {
              var Ee, xe = we[0].trace, Le = xe.visible === !0 && xe._length !== 0, Qe = xe.fill !== "none", ke = le.hasLines(xe), Me = le.hasMarkers(xe), Re = le.hasText(xe), De = Me && xe.marker.symbol === "circle", Pe = Me && xe.marker.symbol !== "circle", $e = xe.cluster && xe.cluster.enabled, qe = pe("fill"), Xe = pe("line"), Ve = pe("circle"), Ze = pe("symbol"), Ge = { fill: qe, line: Xe, circle: Ve, symbol: Ze };
              if (!Le) return Ge;
              if ((Qe || ke) && (Ee = ae.calcTraceToLineCoords(we)), Qe && (qe.geojson = ae.makePolygon(Ee), qe.layout.visibility = "visible", ne.extendFlat(qe.paint, { "fill-color": xe.fillcolor })), ke && (Xe.geojson = ae.makeLine(Ee), Xe.layout.visibility = "visible", ne.extendFlat(Xe.paint, { "line-width": xe.line.width, "line-color": xe.line.color, "line-opacity": xe.opacity })), De) {
                var ze = function(ct) {
                  var et, dt, Je, it, st = ct[0].trace, gt = st.marker, bt = st.selectedpoints, yt = ne.isArrayOrTypedArray(gt.color), St = ne.isArrayOrTypedArray(gt.size), Lt = ne.isArrayOrTypedArray(gt.opacity);
                  function Rt(Tt) {
                    return st.opacity * Tt;
                  }
                  yt && (dt = oe.hasColorscale(st, "marker") ? oe.makeColorScaleFuncFromTrace(gt) : ne.identity), St && (Je = ce(st)), Lt && (it = function(Tt) {
                    return Rt(te(Tt) ? +ne.constrain(Tt, 0, 1) : 0);
                  });
                  var vt, $t, Pt = [];
                  for (et = 0; et < ct.length; et++) {
                    var Xt = ct[et], Nt = Xt.lonlat;
                    if (!ve(Nt)) {
                      var Gt = {};
                      dt && (Gt.mcc = Xt.mcc = dt(Xt.mc)), Je && (Gt.mrc = Xt.mrc = Je(Xt.ms)), it && (Gt.mo = it(Xt.mo)), bt && (Gt.selected = Xt.selected || 0), Pt.push({ type: "Feature", id: et + 1, geometry: { type: "Point", coordinates: Nt }, properties: Gt });
                    }
                  }
                  if (bt) for (vt = se.makeSelectedPointStyleFns(st), et = 0; et < Pt.length; et++) {
                    var Ft = Pt[et].properties;
                    vt.selectedOpacityFn && (Ft.mo = Rt(vt.selectedOpacityFn(Ft))), vt.selectedColorFn && (Ft.mcc = vt.selectedColorFn(Ft)), vt.selectedSizeFn && (Ft.mrc = vt.selectedSizeFn(Ft));
                  }
                  return { geojson: { type: "FeatureCollection", features: Pt }, mcc: yt || vt && vt.selectedColorFn ? { type: "identity", property: "mcc" } : gt.color, mrc: St || vt && vt.selectedSizeFn ? { type: "identity", property: "mrc" } : ($t = gt.size, $t / 2), mo: Lt || vt && vt.selectedOpacityFn ? { type: "identity", property: "mo" } : Rt(gt.opacity) };
                }(we);
                Ve.geojson = ze.geojson, Ve.layout.visibility = "visible", $e && (Ve.filter = ["!", ["has", "point_count"]], Ge.cluster = { type: "circle", filter: ["has", "point_count"], layout: { visibility: "visible" }, paint: { "circle-color": be(xe.cluster.color, xe.cluster.step), "circle-radius": be(xe.cluster.size, xe.cluster.step), "circle-opacity": be(xe.cluster.opacity, xe.cluster.step) } }, Ge.clusterCount = { type: "symbol", filter: ["has", "point_count"], paint: {}, layout: { "text-field": "{point_count_abbreviated}", "text-font": Ce(xe), "text-size": 12 } }), ne.extendFlat(Ve.paint, { "circle-color": ze.mcc, "circle-radius": ze.mrc, "circle-opacity": ze.mo });
              }
              if (De && $e && (Ve.filter = ["!", ["has", "point_count"]]), (Pe || Re) && (Ze.geojson = function(ct, et) {
                for (var dt = et._fullLayout, Je = ct[0].trace, it = Je.marker || {}, st = it.symbol, gt = it.angle, bt = st !== "circle" ? ye(st) : ge, yt = gt !== "auto" ? ye(gt, !0) : ge, St = le.hasText(Je) ? ye(Je.text) : ge, Lt = [], Rt = 0; Rt < ct.length; Rt++) {
                  var vt = ct[Rt];
                  if (!ve(vt.lonlat)) {
                    var $t, Pt = Je.texttemplate;
                    if (Pt) {
                      var Xt = Array.isArray(Pt) ? Pt[Rt] || "" : Pt, Nt = Je._module.formatLabels(vt, Je, dt), Gt = {};
                      fe(Gt, Je, vt.i);
                      var Ft = Je._meta || {};
                      $t = ne.texttemplateString(Xt, Nt, dt._d3locale, Gt, vt, Ft);
                    } else $t = St(Rt);
                    $t && ($t = $t.replace(he, "").replace(me, `
`)), Lt.push({ type: "Feature", geometry: { type: "Point", coordinates: vt.lonlat }, properties: { symbol: bt(Rt), angle: yt(Rt), text: $t } });
                  }
                }
                return { type: "FeatureCollection", features: Lt };
              }(we, Se), ne.extendFlat(Ze.layout, { visibility: "visible", "icon-image": "{symbol}-15", "text-field": "{text}" }), Pe && (ne.extendFlat(Ze.layout, { "icon-size": xe.marker.size / 10 }), "angle" in xe.marker && xe.marker.angle !== "auto" && ne.extendFlat(Ze.layout, { "icon-rotate": { type: "identity", property: "angle" }, "icon-rotation-alignment": "map" }), Ze.layout["icon-allow-overlap"] = xe.marker.allowoverlap, ne.extendFlat(Ze.paint, { "icon-opacity": xe.opacity * xe.marker.opacity, "icon-color": xe.marker.color })), Re)) {
                var We = (xe.marker || {}).size, at = de(xe.textposition, We);
                ne.extendFlat(Ze.layout, { "text-size": xe.textfont.size, "text-anchor": at.anchor, "text-offset": at.offset, "text-font": Ce(xe) }), ne.extendFlat(Ze.paint, { "text-color": xe.textfont.color, "text-opacity": xe.opacity });
              }
              return Ge;
            };
          }, 57387: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(24272), ae = Z(98168), oe = Z(663), se = Z(54114), ce = Z(71388), le = Z(13624).isSupportedFont;
            G.exports = function(ue, de, fe, he) {
              function me(xe, Le) {
                return te.coerce(ue, de, ce, xe, Le);
              }
              function pe(xe, Le) {
                return te.coerce2(ue, de, ce, xe, Le);
              }
              var ye = function(xe, Le, Qe) {
                var ke = Qe("lon") || [], Me = Qe("lat") || [], Re = Math.min(ke.length, Me.length);
                return Le._length = Re, Re;
              }(0, de, me);
              if (ye) {
                if (me("text"), me("texttemplate"), me("hovertext"), me("hovertemplate"), me("mode"), me("below"), ne.hasMarkers(de)) {
                  ee(ue, de, fe, he, me, { noLine: !0, noAngle: !0 }), me("marker.allowoverlap"), me("marker.angle");
                  var ge = de.marker;
                  ge.symbol !== "circle" && (te.isArrayOrTypedArray(ge.size) && (ge.size = ge.size[0]), te.isArrayOrTypedArray(ge.color) && (ge.color = ge.color[0]));
                }
                ne.hasLines(de) && (ae(ue, de, fe, he, me, { noDash: !0 }), me("connectgaps"));
                var ve = pe("cluster.maxzoom"), be = pe("cluster.step"), Ce = pe("cluster.color", de.marker && de.marker.color || fe), Se = pe("cluster.size"), we = pe("cluster.opacity");
                if (me("cluster.enabled", ve !== !1 || be !== !1 || Ce !== !1 || Se !== !1 || we !== !1) || ne.hasText(de)) {
                  var Ee = he.font.family;
                  oe(ue, de, he, me, { noSelect: !0, noFontVariant: !0, noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0, font: { family: le(Ee) ? Ee : "Open Sans Regular", weight: he.font.weight, style: he.font.style, size: he.font.size, color: he.font.color } });
                }
                me("fill"), de.fill !== "none" && se(ue, de, fe, me), te.coerceSelectionMarkerOpacity(de, me);
              } else de.visible = !1;
            };
          }, 58240: function(G) {
            G.exports = function(J, Z) {
              return J.lon = Z.lon, J.lat = Z.lat, J;
            };
          }, 66762: function(G, J, Z) {
            var te = Z(29714);
            G.exports = function(ne, ee, ae) {
              var oe = {}, se = ae[ee.subplot]._subplot.mockAxis, ce = ne.lonlat;
              return oe.lonLabel = te.tickText(se, se.c2l(ce[0]), !0).text, oe.latLabel = te.tickText(se, se.c2l(ce[1]), !0).text, oe;
            };
          }, 67275: function(G, J, Z) {
            var te = Z(32141), ne = Z(34809), ee = Z(11539), ae = ne.fillText, oe = Z(63821).BADNUM, se = Z(8814).traceLayerPrefix;
            function ce(le, ue, de) {
              if (!le.hovertemplate) {
                var fe = (ue.hi || le.hoverinfo).split("+"), he = fe.indexOf("all") !== -1, me = fe.indexOf("lon") !== -1, pe = fe.indexOf("lat") !== -1, ye = ue.lonlat, ge = [];
                return he || me && pe ? ge.push("(" + ve(ye[1]) + ", " + ve(ye[0]) + ")") : me ? ge.push(de.lon + ve(ye[0])) : pe && ge.push(de.lat + ve(ye[1])), (he || fe.indexOf("text") !== -1) && ae(ue, le, ge), ge.join("<br>");
              }
              function ve(be) {
                return be + "°";
              }
            }
            G.exports = { hoverPoints: function(le, ue, de) {
              var fe = le.cd, he = fe[0].trace, me = le.xa, pe = le.ya, ye = le.subplot, ge = [], ve = se + he.uid + "-circle", be = he.cluster && he.cluster.enabled;
              if (be) {
                var Ce = ye.map.queryRenderedFeatures(null, { layers: [ve] });
                ge = Ce.map(function(Pe) {
                  return Pe.id;
                });
              }
              var Se = 360 * (ue >= 0 ? Math.floor((ue + 180) / 360) : Math.ceil((ue - 180) / 360)), we = ue - Se;
              if (te.getClosest(fe, function(Pe) {
                var $e = Pe.lonlat;
                if ($e[0] === oe || be && ge.indexOf(Pe.i + 1) === -1) return 1 / 0;
                var qe = ne.modHalf($e[0], 360), Xe = $e[1], Ve = ye.project([qe, Xe]), Ze = Ve.x - me.c2p([we, Xe]), Ge = Ve.y - pe.c2p([qe, de]), ze = Math.max(3, Pe.mrc || 0);
                return Math.max(Math.sqrt(Ze * Ze + Ge * Ge) - ze, 1 - 3 / ze);
              }, le), le.index !== !1) {
                var Ee = fe[le.index], xe = Ee.lonlat, Le = [ne.modHalf(xe[0], 360) + Se, xe[1]], Qe = me.c2p(Le), ke = pe.c2p(Le), Me = Ee.mrc || 1;
                le.x0 = Qe - Me, le.x1 = Qe + Me, le.y0 = ke - Me, le.y1 = ke + Me;
                var Re = {};
                Re[he.subplot] = { _subplot: ye };
                var De = he._module.formatLabels(Ee, he, Re);
                return le.lonLabel = De.lonLabel, le.latLabel = De.latLabel, le.color = ee(he, Ee), le.extraText = ce(he, Ee, fe[0].t.labels), le.hovertemplate = he.hovertemplate, [le];
              }
            }, getExtraText: ce };
          }, 30929: function(G, J, Z) {
            G.exports = { attributes: Z(71388), supplyDefaults: Z(57387), colorbar: Z(21146), formatLabels: Z(66762), calc: Z(75649), plot: Z(26126), hoverPoints: Z(67275).hoverPoints, eventData: Z(58240), selectPoints: Z(21501), styleOnSelect: function(te, ne) {
              ne && ne[0].trace._glTrace.update(ne);
            }, moduleType: "trace", name: "scattermap", basePlotModule: Z(34091), categories: ["map", "gl", "symbols", "showLegend", "scatter-like"], meta: {} };
          }, 26126: function(G, J, Z) {
            var te = Z(34809), ne = Z(76717), ee = Z(8814).traceLayerPrefix, ae = { cluster: ["cluster", "clusterCount", "circle"], nonCluster: ["fill", "line", "circle", "symbol"] };
            function oe(ce, le, ue, de) {
              this.type = "scattermap", this.subplot = ce, this.uid = le, this.clusterEnabled = ue, this.isHidden = de, this.sourceIds = { fill: "source-" + le + "-fill", line: "source-" + le + "-line", circle: "source-" + le + "-circle", symbol: "source-" + le + "-symbol", cluster: "source-" + le + "-circle", clusterCount: "source-" + le + "-circle" }, this.layerIds = { fill: ee + le + "-fill", line: ee + le + "-line", circle: ee + le + "-circle", symbol: ee + le + "-symbol", cluster: ee + le + "-cluster", clusterCount: ee + le + "-cluster-count" }, this.below = null;
            }
            var se = oe.prototype;
            se.addSource = function(ce, le, ue) {
              var de = { type: "geojson", data: le.geojson };
              ue && ue.enabled && te.extendFlat(de, { cluster: !0, clusterMaxZoom: ue.maxzoom });
              var fe = this.subplot.map.getSource(this.sourceIds[ce]);
              fe ? fe.setData(le.geojson) : this.subplot.map.addSource(this.sourceIds[ce], de);
            }, se.setSourceData = function(ce, le) {
              this.subplot.map.getSource(this.sourceIds[ce]).setData(le.geojson);
            }, se.addLayer = function(ce, le, ue) {
              var de = { type: le.type, id: this.layerIds[ce], source: this.sourceIds[ce], layout: le.layout, paint: le.paint };
              le.filter && (de.filter = le.filter);
              for (var fe, he = this.layerIds[ce], me = this.subplot.getMapLayers(), pe = 0; pe < me.length; pe++) if (me[pe].id === he) {
                fe = !0;
                break;
              }
              fe ? (this.subplot.setOptions(he, "setLayoutProperty", de.layout), de.layout.visibility === "visible" && this.subplot.setOptions(he, "setPaintProperty", de.paint)) : this.subplot.addLayer(de, ue);
            }, se.update = function(ce) {
              var le = ce[0].trace, ue = this.subplot, de = ue.map, fe = ne(ue.gd, ce), he = ue.belowLookup["trace-" + this.uid], me = !(!le.cluster || !le.cluster.enabled), pe = !!this.clusterEnabled, ye = this;
              function ge(we) {
                pe ? function(Ee) {
                  for (var xe = ae.cluster, Le = xe.length - 1; Le >= 0; Le--) {
                    var Qe = xe[Le];
                    de.removeLayer(ye.layerIds[Qe]);
                  }
                  Ee || de.removeSource(ye.sourceIds.circle);
                }(we) : function(Ee) {
                  for (var xe = ae.nonCluster, Le = xe.length - 1; Le >= 0; Le--) {
                    var Qe = xe[Le];
                    de.removeLayer(ye.layerIds[Qe]), Ee || de.removeSource(ye.sourceIds[Qe]);
                  }
                }(we);
              }
              function ve(we) {
                me ? function(Ee) {
                  Ee || ye.addSource("circle", fe.circle, le.cluster);
                  for (var xe = ae.cluster, Le = 0; Le < xe.length; Le++) {
                    var Qe = xe[Le], ke = fe[Qe];
                    ye.addLayer(Qe, ke, he);
                  }
                }(we) : function(Ee) {
                  for (var xe = ae.nonCluster, Le = 0; Le < xe.length; Le++) {
                    var Qe = xe[Le], ke = fe[Qe];
                    Ee || ye.addSource(Qe, ke), ye.addLayer(Qe, ke, he);
                  }
                }(we);
              }
              function be() {
                for (var we = me ? ae.cluster : ae.nonCluster, Ee = 0; Ee < we.length; Ee++) {
                  var xe = we[Ee], Le = fe[xe];
                  Le && (ue.setOptions(ye.layerIds[xe], "setLayoutProperty", Le.layout), Le.layout.visibility === "visible" && (xe !== "cluster" && ye.setSourceData(xe, Le), ue.setOptions(ye.layerIds[xe], "setPaintProperty", Le.paint)));
                }
              }
              var Ce = this.isHidden, Se = le.visible !== !0;
              Se ? Ce || ge() : Ce ? Se || ve() : pe !== me ? (ge(), ve()) : (this.below !== he && (ge(!0), ve(!0)), be()), this.clusterEnabled = me, this.isHidden = Se, this.below = he, ce[0].trace._glTrace = this;
            }, se.dispose = function() {
              for (var ce = this.subplot.map, le = this.clusterEnabled ? ae.cluster : ae.nonCluster, ue = le.length - 1; ue >= 0; ue--) {
                var de = le[ue];
                ce.removeLayer(this.layerIds[de]), ce.removeSource(this.sourceIds[de]);
              }
            }, G.exports = function(ce, le) {
              var ue, de, fe, he = le[0].trace, me = he.cluster && he.cluster.enabled, pe = he.visible !== !0, ye = new oe(ce, he.uid, me, pe), ge = ne(ce.gd, le), ve = ye.below = ce.belowLookup["trace-" + he.uid];
              if (me) for (ye.addSource("circle", ge.circle, he.cluster), ue = 0; ue < ae.cluster.length; ue++) fe = ge[de = ae.cluster[ue]], ye.addLayer(de, fe, ve);
              else for (ue = 0; ue < ae.nonCluster.length; ue++) fe = ge[de = ae.nonCluster[ue]], ye.addSource(de, fe, he.cluster), ye.addLayer(de, fe, ve);
              return le[0].trace._glTrace = ye, ye;
            };
          }, 21501: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(63821).BADNUM;
            G.exports = function(ae, oe) {
              var se, ce = ae.cd, le = ae.xaxis, ue = ae.yaxis, de = [], fe = ce[0].trace;
              if (!ne.hasMarkers(fe)) return [];
              if (oe === !1) for (se = 0; se < ce.length; se++) ce[se].selected = 0;
              else for (se = 0; se < ce.length; se++) {
                var he = ce[se], me = he.lonlat;
                if (me[0] !== ee) {
                  var pe = [te.modHalf(me[0], 360), me[1]], ye = [le.c2p(pe), ue.c2p(pe)];
                  oe.contains(ye, null, se, ae) ? (de.push({ pointNumber: se, lon: me[0], lat: me[1] }), he.selected = 1) : he.selected = 0;
                }
              }
              return de;
            };
          }, 95833: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(19326), ae = Z(6893), oe = Z(36640), se = Z(67514), ce = Z(9829), le = Z(87163), ue = Z(93049).extendFlat, de = Z(13582).overrideAll, fe = Z(67514), he = ae.line, me = ae.marker;
            G.exports = de({ lon: ae.lon, lat: ae.lat, cluster: { enabled: { valType: "boolean" }, maxzoom: ue({}, fe.layers.maxzoom, {}), step: { valType: "number", arrayOk: !0, dflt: -1, min: -1 }, size: { valType: "number", arrayOk: !0, dflt: 20, min: 0 }, color: { valType: "color", arrayOk: !0 }, opacity: ue({}, me.opacity, { dflt: 1 }) }, mode: ue({}, oe.mode, { dflt: "markers" }), text: ue({}, oe.text, {}), texttemplate: ne({ editType: "plot" }, { keys: ["lat", "lon", "text"] }), hovertext: ue({}, oe.hovertext, {}), line: { color: he.color, width: he.width }, connectgaps: oe.connectgaps, marker: ue({ symbol: { valType: "string", dflt: "circle", arrayOk: !0 }, angle: { valType: "number", dflt: "auto", arrayOk: !0 }, allowoverlap: { valType: "boolean", dflt: !1 }, opacity: me.opacity, size: me.size, sizeref: me.sizeref, sizemin: me.sizemin, sizemode: me.sizemode }, le("marker")), fill: ae.fill, fillcolor: ee(), textfont: se.layers.symbol.textfont, textposition: se.layers.symbol.textposition, below: { valType: "string" }, selected: { marker: oe.selected.marker }, unselected: { marker: oe.unselected.marker }, hoverinfo: ue({}, ce.hoverinfo, { flags: ["lon", "lat", "text", "name"] }), hovertemplate: te() }, "calc", "nested");
          }, 2795: function(G) {
            var J = ["Metropolis Black Italic", "Metropolis Black", "Metropolis Bold Italic", "Metropolis Bold", "Metropolis Extra Bold Italic", "Metropolis Extra Bold", "Metropolis Extra Light Italic", "Metropolis Extra Light", "Metropolis Light Italic", "Metropolis Light", "Metropolis Medium Italic", "Metropolis Medium", "Metropolis Regular Italic", "Metropolis Regular", "Metropolis Semi Bold Italic", "Metropolis Semi Bold", "Metropolis Thin Italic", "Metropolis Thin", "Open Sans Bold Italic", "Open Sans Bold", "Open Sans Extrabold Italic", "Open Sans Extrabold", "Open Sans Italic", "Open Sans Light Italic", "Open Sans Light", "Open Sans Regular", "Open Sans Semibold Italic", "Open Sans Semibold", "Klokantech Noto Sans Bold", "Klokantech Noto Sans CJK Bold", "Klokantech Noto Sans CJK Regular", "Klokantech Noto Sans Italic", "Klokantech Noto Sans Regular"];
            G.exports = { isSupportedFont: function(Z) {
              return J.indexOf(Z) !== -1;
            } };
          }, 27009: function(G, J, Z) {
            var te = Z(10721), ne = Z(34809), ee = Z(63821).BADNUM, ae = Z(39532), oe = Z(88856), se = Z(62203), ce = Z(92527), le = Z(64726), ue = Z(2795).isSupportedFont, de = Z(2178), fe = Z(36040).appendArrayPointValue, he = Z(30635).NEWLINES, me = Z(30635).BR_TAG_ALL;
            function pe(Se) {
              return { type: Se, geojson: ae.makeBlank(), layout: { visibility: "none" }, filter: null, paint: {} };
            }
            function ye(Se, we) {
              return ne.isArrayOrTypedArray(Se) ? we ? function(Ee) {
                return te(Se[Ee]) ? +Se[Ee] : 0;
              } : function(Ee) {
                return Se[Ee];
              } : Se ? function() {
                return Se;
              } : ge;
            }
            function ge() {
              return "";
            }
            function ve(Se) {
              return Se[0] === ee;
            }
            function be(Se, we) {
              var Ee;
              if (ne.isArrayOrTypedArray(Se) && ne.isArrayOrTypedArray(we)) {
                Ee = ["step", ["get", "point_count"], Se[0]];
                for (var xe = 1; xe < Se.length; xe++) Ee.push(we[xe - 1], Se[xe]);
              } else Ee = Se;
              return Ee;
            }
            function Ce(Se) {
              var we = Se.textfont, Ee = we.family, xe = we.style, Le = we.weight, Qe = Ee.split(" "), ke = Qe[Qe.length - 1] === "Italic";
              ke && Qe.pop(), ke = ke || xe === "italic";
              var Me = Qe.join(" ");
              return Le === "bold" && Qe.indexOf("Bold") === -1 ? Me += " Bold" : Le <= 1e3 && (Qe[0] === "Metropolis" ? (Me = "Metropolis", Me += Le > 850 ? " Black" : Le > 750 ? " Extra Bold" : Le > 650 ? " Bold" : Le > 550 ? " Semi Bold" : Le > 450 ? " Medium" : Le > 350 ? " Regular" : Le > 250 ? " Light" : Le > 150 ? " Extra Light" : " Thin") : Qe.slice(0, 2).join(" ") === "Open Sans" ? (Me = "Open Sans", Me += Le > 750 ? " Extrabold" : Le > 650 ? " Bold" : Le > 550 ? " Semibold" : Le > 350 ? " Regular" : " Light") : Qe.slice(0, 3).join(" ") === "Klokantech Noto Sans" && (Me = "Klokantech Noto Sans", Qe[3] === "CJK" && (Me += " CJK"), Me += Le > 500 ? " Bold" : " Regular")), ke && (Me += " Italic"), Me === "Open Sans Regular Italic" ? Me = "Open Sans Italic" : Me === "Open Sans Regular Bold" ? Me = "Open Sans Bold" : Me === "Open Sans Regular Bold Italic" ? Me = "Open Sans Bold Italic" : Me === "Klokantech Noto Sans Regular Italic" && (Me = "Klokantech Noto Sans Italic"), ue(Me) || (Me = Ee), Me.split(", ");
            }
            G.exports = function(Se, we) {
              var Ee, xe = we[0].trace, Le = xe.visible === !0 && xe._length !== 0, Qe = xe.fill !== "none", ke = le.hasLines(xe), Me = le.hasMarkers(xe), Re = le.hasText(xe), De = Me && xe.marker.symbol === "circle", Pe = Me && xe.marker.symbol !== "circle", $e = xe.cluster && xe.cluster.enabled, qe = pe("fill"), Xe = pe("line"), Ve = pe("circle"), Ze = pe("symbol"), Ge = { fill: qe, line: Xe, circle: Ve, symbol: Ze };
              if (!Le) return Ge;
              if ((Qe || ke) && (Ee = ae.calcTraceToLineCoords(we)), Qe && (qe.geojson = ae.makePolygon(Ee), qe.layout.visibility = "visible", ne.extendFlat(qe.paint, { "fill-color": xe.fillcolor })), ke && (Xe.geojson = ae.makeLine(Ee), Xe.layout.visibility = "visible", ne.extendFlat(Xe.paint, { "line-width": xe.line.width, "line-color": xe.line.color, "line-opacity": xe.opacity })), De) {
                var ze = function(ct) {
                  var et, dt, Je, it, st = ct[0].trace, gt = st.marker, bt = st.selectedpoints, yt = ne.isArrayOrTypedArray(gt.color), St = ne.isArrayOrTypedArray(gt.size), Lt = ne.isArrayOrTypedArray(gt.opacity);
                  function Rt(Tt) {
                    return st.opacity * Tt;
                  }
                  yt && (dt = oe.hasColorscale(st, "marker") ? oe.makeColorScaleFuncFromTrace(gt) : ne.identity), St && (Je = ce(st)), Lt && (it = function(Tt) {
                    return Rt(te(Tt) ? +ne.constrain(Tt, 0, 1) : 0);
                  });
                  var vt, $t, Pt = [];
                  for (et = 0; et < ct.length; et++) {
                    var Xt = ct[et], Nt = Xt.lonlat;
                    if (!ve(Nt)) {
                      var Gt = {};
                      dt && (Gt.mcc = Xt.mcc = dt(Xt.mc)), Je && (Gt.mrc = Xt.mrc = Je(Xt.ms)), it && (Gt.mo = it(Xt.mo)), bt && (Gt.selected = Xt.selected || 0), Pt.push({ type: "Feature", id: et + 1, geometry: { type: "Point", coordinates: Nt }, properties: Gt });
                    }
                  }
                  if (bt) for (vt = se.makeSelectedPointStyleFns(st), et = 0; et < Pt.length; et++) {
                    var Ft = Pt[et].properties;
                    vt.selectedOpacityFn && (Ft.mo = Rt(vt.selectedOpacityFn(Ft))), vt.selectedColorFn && (Ft.mcc = vt.selectedColorFn(Ft)), vt.selectedSizeFn && (Ft.mrc = vt.selectedSizeFn(Ft));
                  }
                  return { geojson: { type: "FeatureCollection", features: Pt }, mcc: yt || vt && vt.selectedColorFn ? { type: "identity", property: "mcc" } : gt.color, mrc: St || vt && vt.selectedSizeFn ? { type: "identity", property: "mrc" } : ($t = gt.size, $t / 2), mo: Lt || vt && vt.selectedOpacityFn ? { type: "identity", property: "mo" } : Rt(gt.opacity) };
                }(we);
                Ve.geojson = ze.geojson, Ve.layout.visibility = "visible", $e && (Ve.filter = ["!", ["has", "point_count"]], Ge.cluster = { type: "circle", filter: ["has", "point_count"], layout: { visibility: "visible" }, paint: { "circle-color": be(xe.cluster.color, xe.cluster.step), "circle-radius": be(xe.cluster.size, xe.cluster.step), "circle-opacity": be(xe.cluster.opacity, xe.cluster.step) } }, Ge.clusterCount = { type: "symbol", filter: ["has", "point_count"], paint: {}, layout: { "text-field": "{point_count_abbreviated}", "text-font": Ce(xe), "text-size": 12 } }), ne.extendFlat(Ve.paint, { "circle-color": ze.mcc, "circle-radius": ze.mrc, "circle-opacity": ze.mo });
              }
              if (De && $e && (Ve.filter = ["!", ["has", "point_count"]]), (Pe || Re) && (Ze.geojson = function(ct, et) {
                for (var dt = et._fullLayout, Je = ct[0].trace, it = Je.marker || {}, st = it.symbol, gt = it.angle, bt = st !== "circle" ? ye(st) : ge, yt = gt !== "auto" ? ye(gt, !0) : ge, St = le.hasText(Je) ? ye(Je.text) : ge, Lt = [], Rt = 0; Rt < ct.length; Rt++) {
                  var vt = ct[Rt];
                  if (!ve(vt.lonlat)) {
                    var $t, Pt = Je.texttemplate;
                    if (Pt) {
                      var Xt = Array.isArray(Pt) ? Pt[Rt] || "" : Pt, Nt = Je._module.formatLabels(vt, Je, dt), Gt = {};
                      fe(Gt, Je, vt.i);
                      var Ft = Je._meta || {};
                      $t = ne.texttemplateString(Xt, Nt, dt._d3locale, Gt, vt, Ft);
                    } else $t = St(Rt);
                    $t && ($t = $t.replace(he, "").replace(me, `
`)), Lt.push({ type: "Feature", geometry: { type: "Point", coordinates: vt.lonlat }, properties: { symbol: bt(Rt), angle: yt(Rt), text: $t } });
                  }
                }
                return { type: "FeatureCollection", features: Lt };
              }(we, Se), ne.extendFlat(Ze.layout, { visibility: "visible", "icon-image": "{symbol}-15", "text-field": "{text}" }), Pe && (ne.extendFlat(Ze.layout, { "icon-size": xe.marker.size / 10 }), "angle" in xe.marker && xe.marker.angle !== "auto" && ne.extendFlat(Ze.layout, { "icon-rotate": { type: "identity", property: "angle" }, "icon-rotation-alignment": "map" }), Ze.layout["icon-allow-overlap"] = xe.marker.allowoverlap, ne.extendFlat(Ze.paint, { "icon-opacity": xe.opacity * xe.marker.opacity, "icon-color": xe.marker.color })), Re)) {
                var We = (xe.marker || {}).size, at = de(xe.textposition, We);
                ne.extendFlat(Ze.layout, { "text-size": xe.textfont.size, "text-anchor": at.anchor, "text-offset": at.offset, "text-font": Ce(xe) }), ne.extendFlat(Ze.paint, { "text-color": xe.textfont.color, "text-opacity": xe.opacity });
              }
              return Ge;
            };
          }, 38302: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(24272), ae = Z(98168), oe = Z(663), se = Z(54114), ce = Z(95833), le = Z(2795).isSupportedFont;
            G.exports = function(ue, de, fe, he) {
              function me(xe, Le) {
                return te.coerce(ue, de, ce, xe, Le);
              }
              function pe(xe, Le) {
                return te.coerce2(ue, de, ce, xe, Le);
              }
              var ye = function(xe, Le, Qe) {
                var ke = Qe("lon") || [], Me = Qe("lat") || [], Re = Math.min(ke.length, Me.length);
                return Le._length = Re, Re;
              }(0, de, me);
              if (ye) {
                if (me("text"), me("texttemplate"), me("hovertext"), me("hovertemplate"), me("mode"), me("below"), ne.hasMarkers(de)) {
                  ee(ue, de, fe, he, me, { noLine: !0, noAngle: !0 }), me("marker.allowoverlap"), me("marker.angle");
                  var ge = de.marker;
                  ge.symbol !== "circle" && (te.isArrayOrTypedArray(ge.size) && (ge.size = ge.size[0]), te.isArrayOrTypedArray(ge.color) && (ge.color = ge.color[0]));
                }
                ne.hasLines(de) && (ae(ue, de, fe, he, me, { noDash: !0 }), me("connectgaps"));
                var ve = pe("cluster.maxzoom"), be = pe("cluster.step"), Ce = pe("cluster.color", de.marker && de.marker.color || fe), Se = pe("cluster.size"), we = pe("cluster.opacity");
                if (me("cluster.enabled", ve !== !1 || be !== !1 || Ce !== !1 || Se !== !1 || we !== !1) || ne.hasText(de)) {
                  var Ee = he.font.family;
                  oe(ue, de, he, me, { noSelect: !0, noFontVariant: !0, noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0, font: { family: le(Ee) ? Ee : "Open Sans Regular", weight: he.font.weight, style: he.font.style, size: he.font.size, color: he.font.color } });
                }
                me("fill"), de.fill !== "none" && se(ue, de, fe, me), te.coerceSelectionMarkerOpacity(de, me);
              } else de.visible = !1;
            };
          }, 68197: function(G) {
            G.exports = function(J, Z) {
              return J.lon = Z.lon, J.lat = Z.lat, J;
            };
          }, 69009: function(G, J, Z) {
            var te = Z(29714);
            G.exports = function(ne, ee, ae) {
              var oe = {}, se = ae[ee.subplot]._subplot.mockAxis, ce = ne.lonlat;
              return oe.lonLabel = te.tickText(se, se.c2l(ce[0]), !0).text, oe.latLabel = te.tickText(se, se.c2l(ce[1]), !0).text, oe;
            };
          }, 18016: function(G, J, Z) {
            var te = Z(32141), ne = Z(34809), ee = Z(11539), ae = ne.fillText, oe = Z(63821).BADNUM, se = Z(44245).traceLayerPrefix;
            function ce(le, ue, de) {
              if (!le.hovertemplate) {
                var fe = (ue.hi || le.hoverinfo).split("+"), he = fe.indexOf("all") !== -1, me = fe.indexOf("lon") !== -1, pe = fe.indexOf("lat") !== -1, ye = ue.lonlat, ge = [];
                return he || me && pe ? ge.push("(" + ve(ye[1]) + ", " + ve(ye[0]) + ")") : me ? ge.push(de.lon + ve(ye[0])) : pe && ge.push(de.lat + ve(ye[1])), (he || fe.indexOf("text") !== -1) && ae(ue, le, ge), ge.join("<br>");
              }
              function ve(be) {
                return be + "°";
              }
            }
            G.exports = { hoverPoints: function(le, ue, de) {
              var fe = le.cd, he = fe[0].trace, me = le.xa, pe = le.ya, ye = le.subplot, ge = [], ve = se + he.uid + "-circle", be = he.cluster && he.cluster.enabled;
              if (be) {
                var Ce = ye.map.queryRenderedFeatures(null, { layers: [ve] });
                ge = Ce.map(function(Pe) {
                  return Pe.id;
                });
              }
              var Se = 360 * (ue >= 0 ? Math.floor((ue + 180) / 360) : Math.ceil((ue - 180) / 360)), we = ue - Se;
              if (te.getClosest(fe, function(Pe) {
                var $e = Pe.lonlat;
                if ($e[0] === oe || be && ge.indexOf(Pe.i + 1) === -1) return 1 / 0;
                var qe = ne.modHalf($e[0], 360), Xe = $e[1], Ve = ye.project([qe, Xe]), Ze = Ve.x - me.c2p([we, Xe]), Ge = Ve.y - pe.c2p([qe, de]), ze = Math.max(3, Pe.mrc || 0);
                return Math.max(Math.sqrt(Ze * Ze + Ge * Ge) - ze, 1 - 3 / ze);
              }, le), le.index !== !1) {
                var Ee = fe[le.index], xe = Ee.lonlat, Le = [ne.modHalf(xe[0], 360) + Se, xe[1]], Qe = me.c2p(Le), ke = pe.c2p(Le), Me = Ee.mrc || 1;
                le.x0 = Qe - Me, le.x1 = Qe + Me, le.y0 = ke - Me, le.y1 = ke + Me;
                var Re = {};
                Re[he.subplot] = { _subplot: ye };
                var De = he._module.formatLabels(Ee, he, Re);
                return le.lonLabel = De.lonLabel, le.latLabel = De.latLabel, le.color = ee(he, Ee), le.extraText = ce(he, Ee, fe[0].t.labels), le.hovertemplate = he.hovertemplate, [le];
              }
            }, getExtraText: ce };
          }, 83866: function(G, J, Z) {
            G.exports = { attributes: Z(95833), supplyDefaults: Z(38302), colorbar: Z(21146), formatLabels: Z(69009), calc: Z(75649), plot: Z(20691), hoverPoints: Z(18016).hoverPoints, eventData: Z(68197), selectPoints: Z(60784), styleOnSelect: function(te, ne) {
              ne && ne[0].trace._glTrace.update(ne);
            }, moduleType: "trace", name: "scattermapbox", basePlotModule: Z(68192), categories: ["mapbox", "gl", "symbols", "showLegend", "scatter-like"], meta: {} };
          }, 20691: function(G, J, Z) {
            var te = Z(34809), ne = Z(27009), ee = Z(44245).traceLayerPrefix, ae = { cluster: ["cluster", "clusterCount", "circle"], nonCluster: ["fill", "line", "circle", "symbol"] };
            function oe(ce, le, ue, de) {
              this.type = "scattermapbox", this.subplot = ce, this.uid = le, this.clusterEnabled = ue, this.isHidden = de, this.sourceIds = { fill: "source-" + le + "-fill", line: "source-" + le + "-line", circle: "source-" + le + "-circle", symbol: "source-" + le + "-symbol", cluster: "source-" + le + "-circle", clusterCount: "source-" + le + "-circle" }, this.layerIds = { fill: ee + le + "-fill", line: ee + le + "-line", circle: ee + le + "-circle", symbol: ee + le + "-symbol", cluster: ee + le + "-cluster", clusterCount: ee + le + "-cluster-count" }, this.below = null;
            }
            var se = oe.prototype;
            se.addSource = function(ce, le, ue) {
              var de = { type: "geojson", data: le.geojson };
              ue && ue.enabled && te.extendFlat(de, { cluster: !0, clusterMaxZoom: ue.maxzoom });
              var fe = this.subplot.map.getSource(this.sourceIds[ce]);
              fe ? fe.setData(le.geojson) : this.subplot.map.addSource(this.sourceIds[ce], de);
            }, se.setSourceData = function(ce, le) {
              this.subplot.map.getSource(this.sourceIds[ce]).setData(le.geojson);
            }, se.addLayer = function(ce, le, ue) {
              var de = { type: le.type, id: this.layerIds[ce], source: this.sourceIds[ce], layout: le.layout, paint: le.paint };
              le.filter && (de.filter = le.filter);
              for (var fe, he = this.layerIds[ce], me = this.subplot.getMapLayers(), pe = 0; pe < me.length; pe++) if (me[pe].id === he) {
                fe = !0;
                break;
              }
              fe ? (this.subplot.setOptions(he, "setLayoutProperty", de.layout), de.layout.visibility === "visible" && this.subplot.setOptions(he, "setPaintProperty", de.paint)) : this.subplot.addLayer(de, ue);
            }, se.update = function(ce) {
              var le = ce[0].trace, ue = this.subplot, de = ue.map, fe = ne(ue.gd, ce), he = ue.belowLookup["trace-" + this.uid], me = !(!le.cluster || !le.cluster.enabled), pe = !!this.clusterEnabled, ye = this;
              function ge(we) {
                pe ? function(Ee) {
                  for (var xe = ae.cluster, Le = xe.length - 1; Le >= 0; Le--) {
                    var Qe = xe[Le];
                    de.removeLayer(ye.layerIds[Qe]);
                  }
                  Ee || de.removeSource(ye.sourceIds.circle);
                }(we) : function(Ee) {
                  for (var xe = ae.nonCluster, Le = xe.length - 1; Le >= 0; Le--) {
                    var Qe = xe[Le];
                    de.removeLayer(ye.layerIds[Qe]), Ee || de.removeSource(ye.sourceIds[Qe]);
                  }
                }(we);
              }
              function ve(we) {
                me ? function(Ee) {
                  Ee || ye.addSource("circle", fe.circle, le.cluster);
                  for (var xe = ae.cluster, Le = 0; Le < xe.length; Le++) {
                    var Qe = xe[Le], ke = fe[Qe];
                    ye.addLayer(Qe, ke, he);
                  }
                }(we) : function(Ee) {
                  for (var xe = ae.nonCluster, Le = 0; Le < xe.length; Le++) {
                    var Qe = xe[Le], ke = fe[Qe];
                    Ee || ye.addSource(Qe, ke), ye.addLayer(Qe, ke, he);
                  }
                }(we);
              }
              function be() {
                for (var we = me ? ae.cluster : ae.nonCluster, Ee = 0; Ee < we.length; Ee++) {
                  var xe = we[Ee], Le = fe[xe];
                  Le && (ue.setOptions(ye.layerIds[xe], "setLayoutProperty", Le.layout), Le.layout.visibility === "visible" && (xe !== "cluster" && ye.setSourceData(xe, Le), ue.setOptions(ye.layerIds[xe], "setPaintProperty", Le.paint)));
                }
              }
              var Ce = this.isHidden, Se = le.visible !== !0;
              Se ? Ce || ge() : Ce ? Se || ve() : pe !== me ? (ge(), ve()) : (this.below !== he && (ge(!0), ve(!0)), be()), this.clusterEnabled = me, this.isHidden = Se, this.below = he, ce[0].trace._glTrace = this;
            }, se.dispose = function() {
              for (var ce = this.subplot.map, le = this.clusterEnabled ? ae.cluster : ae.nonCluster, ue = le.length - 1; ue >= 0; ue--) {
                var de = le[ue];
                ce.removeLayer(this.layerIds[de]), ce.removeSource(this.sourceIds[de]);
              }
            }, G.exports = function(ce, le) {
              var ue, de, fe, he = le[0].trace, me = he.cluster && he.cluster.enabled, pe = he.visible !== !0, ye = new oe(ce, he.uid, me, pe), ge = ne(ce.gd, le), ve = ye.below = ce.belowLookup["trace-" + he.uid];
              if (me) for (ye.addSource("circle", ge.circle, he.cluster), ue = 0; ue < ae.cluster.length; ue++) fe = ge[de = ae.cluster[ue]], ye.addLayer(de, fe, ve);
              else for (ue = 0; ue < ae.nonCluster.length; ue++) fe = ge[de = ae.nonCluster[ue]], ye.addSource(de, fe, he.cluster), ye.addLayer(de, fe, ve);
              return le[0].trace._glTrace = ye, ye;
            };
          }, 60784: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(63821).BADNUM;
            G.exports = function(ae, oe) {
              var se, ce = ae.cd, le = ae.xaxis, ue = ae.yaxis, de = [], fe = ce[0].trace;
              if (!ne.hasMarkers(fe)) return [];
              if (oe === !1) for (se = 0; se < ce.length; se++) ce[se].selected = 0;
              else for (se = 0; se < ce.length; se++) {
                var he = ce[se], me = he.lonlat;
                if (me[0] !== ee) {
                  var pe = [te.modHalf(me[0], 360), me[1]], ye = [le.c2p(pe), ue.c2p(pe)];
                  oe.contains(ye, null, se, ae) ? (de.push({ pointNumber: se, lon: me[0], lat: me[1] }), he.selected = 1) : he.selected = 0;
                }
              }
              return de;
            };
          }, 8738: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(93049).extendFlat, ae = Z(19326), oe = Z(36640), se = Z(9829), ce = oe.line;
            G.exports = { mode: oe.mode, r: { valType: "data_array", editType: "calc+clearAxisTypes" }, theta: { valType: "data_array", editType: "calc+clearAxisTypes" }, r0: { valType: "any", dflt: 0, editType: "calc+clearAxisTypes" }, dr: { valType: "number", dflt: 1, editType: "calc" }, theta0: { valType: "any", dflt: 0, editType: "calc+clearAxisTypes" }, dtheta: { valType: "number", editType: "calc" }, thetaunit: { valType: "enumerated", values: ["radians", "degrees", "gradians"], dflt: "degrees", editType: "calc+clearAxisTypes" }, text: oe.text, texttemplate: ne({ editType: "plot" }, { keys: ["r", "theta", "text"] }), hovertext: oe.hovertext, line: { color: ce.color, width: ce.width, dash: ce.dash, backoff: ce.backoff, shape: ee({}, ce.shape, { values: ["linear", "spline"] }), smoothing: ce.smoothing, editType: "calc" }, connectgaps: oe.connectgaps, marker: oe.marker, cliponaxis: ee({}, oe.cliponaxis, { dflt: !1 }), textposition: oe.textposition, textfont: oe.textfont, fill: ee({}, oe.fill, { values: ["none", "toself", "tonext"], dflt: "none" }), fillcolor: ae(), hoverinfo: ee({}, se.hoverinfo, { flags: ["r", "theta", "text", "name"] }), hoveron: oe.hoveron, hovertemplate: te(), selected: oe.selected, unselected: oe.unselected };
          }, 13246: function(G, J, Z) {
            var te = Z(10721), ne = Z(63821).BADNUM, ee = Z(29714), ae = Z(77272), oe = Z(99203), se = Z(48861), ce = Z(26544).calcMarkerSize;
            G.exports = function(le, ue) {
              for (var de = le._fullLayout, fe = ue.subplot, he = de[fe].radialaxis, me = de[fe].angularaxis, pe = he.makeCalcdata(ue, "r"), ye = me.makeCalcdata(ue, "theta"), ge = ue._length, ve = new Array(ge), be = 0; be < ge; be++) {
                var Ce = pe[be], Se = ye[be], we = ve[be] = {};
                te(Ce) && te(Se) ? (we.r = Ce, we.theta = Se) : we.r = ne;
              }
              var Ee = ce(ue, ge);
              return ue._extremes.x = ee.findExtremes(he, pe, { ppad: Ee }), ae(le, ue), oe(ve, ue), se(ve, ue), ve;
            };
          }, 73749: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(24272), ae = Z(98168), oe = Z(91602), se = Z(663), ce = Z(54114), le = Z(32660).PTS_LINESONLY, ue = Z(8738);
            function de(fe, he, me, pe) {
              var ye, ge = pe("r"), ve = pe("theta");
              if (te.isTypedArray(ge) && (he.r = ge = Array.from(ge)), te.isTypedArray(ve) && (he.theta = ve = Array.from(ve)), ge) ve ? ye = Math.min(ge.length, ve.length) : (ye = ge.length, pe("theta0"), pe("dtheta"));
              else {
                if (!ve) return 0;
                ye = he.theta.length, pe("r0"), pe("dr");
              }
              return he._length = ye, ye;
            }
            G.exports = { handleRThetaDefaults: de, supplyDefaults: function(fe, he, me, pe) {
              function ye(be, Ce) {
                return te.coerce(fe, he, ue, be, Ce);
              }
              var ge = de(0, he, 0, ye);
              if (ge) {
                ye("thetaunit"), ye("mode", ge < le ? "lines+markers" : "lines"), ye("text"), ye("hovertext"), he.hoveron !== "fills" && ye("hovertemplate"), ne.hasMarkers(he) && ee(fe, he, me, pe, ye, { gradient: !0 }), ne.hasLines(he) && (ae(fe, he, me, pe, ye, { backoff: !0 }), oe(fe, he, ye), ye("connectgaps")), ne.hasText(he) && (ye("texttemplate"), se(fe, he, pe, ye));
                var ve = [];
                (ne.hasMarkers(he) || ne.hasText(he)) && (ye("cliponaxis"), ye("marker.maxdisplayed"), ve.push("points")), ye("fill"), he.fill !== "none" && (ce(fe, he, me, ye), ne.hasLines(he) || oe(fe, he, ye)), he.fill !== "tonext" && he.fill !== "toself" || ve.push("fills"), ye("hoveron", ve.join("+") || "points"), te.coerceSelectionMarkerOpacity(he, ye);
              } else he.visible = !1;
            } };
          }, 33368: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714);
            G.exports = function(ee, ae, oe) {
              var se, ce, le = {}, ue = oe[ae.subplot]._subplot;
              ue ? (se = ue.radialAxis, ce = ue.angularAxis) : (se = (ue = oe[ae.subplot]).radialaxis, ce = ue.angularaxis);
              var de = se.c2l(ee.r);
              le.rLabel = ne.tickText(se, de, !0).text;
              var fe = ce.thetaunit === "degrees" ? te.rad2deg(ee.theta) : ee.theta;
              return le.thetaLabel = ne.tickText(ce, fe, !0).text, le;
            };
          }, 29709: function(G, J, Z) {
            var te = Z(37255);
            function ne(ee, ae, oe, se) {
              var ce = oe.radialAxis, le = oe.angularAxis;
              ce._hovertitle = "r", le._hovertitle = "θ";
              var ue = {};
              ue[ae.subplot] = { _subplot: oe };
              var de = ae._module.formatLabels(ee, ae, ue);
              se.rLabel = de.rLabel, se.thetaLabel = de.thetaLabel;
              var fe = ee.hi || ae.hoverinfo, he = [];
              function me(ye, ge) {
                he.push(ye._hovertitle + ": " + ge);
              }
              if (!ae.hovertemplate) {
                var pe = fe.split("+");
                pe.indexOf("all") !== -1 && (pe = ["r", "theta", "text"]), pe.indexOf("r") !== -1 && me(ce, se.rLabel), pe.indexOf("theta") !== -1 && me(le, se.thetaLabel), pe.indexOf("text") !== -1 && se.text && (he.push(se.text), delete se.text), se.extraText = he.join("<br>");
              }
            }
            G.exports = { hoverPoints: function(ee, ae, oe, se) {
              var ce = te(ee, ae, oe, se);
              if (ce && ce[0].index !== !1) {
                var le = ce[0];
                if (le.index === void 0) return ce;
                var ue = ee.subplot, de = le.cd[le.index], fe = le.trace;
                if (ue.isPtInside(de)) return le.xLabelVal = void 0, le.yLabelVal = void 0, ne(de, fe, ue, le), le.hovertemplate = fe.hovertemplate, ce;
              }
            }, makeHoverPointText: ne };
          }, 66939: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "scatterpolar", basePlotModule: Z(31645), categories: ["polar", "symbols", "showLegend", "scatter-like"], attributes: Z(8738), supplyDefaults: Z(73749).supplyDefaults, colorbar: Z(21146), formatLabels: Z(33368), calc: Z(13246), plot: Z(43836), style: Z(9408).style, styleOnSelect: Z(9408).styleOnSelect, hoverPoints: Z(29709).hoverPoints, selectPoints: Z(32665), meta: {} };
          }, 43836: function(G, J, Z) {
            var te = Z(36098), ne = Z(63821).BADNUM;
            G.exports = function(ee, ae, oe) {
              for (var se = ae.layers.frontplot.select("g.scatterlayer"), ce = ae.xaxis, le = ae.yaxis, ue = { xaxis: ce, yaxis: le, plot: ae.framework, layerClipId: ae._hasClipOnAxisFalse ? ae.clipIds.forTraces : null }, de = ae.radialAxis, fe = ae.angularAxis, he = 0; he < oe.length; he++) for (var me = oe[he], pe = 0; pe < me.length; pe++) {
                pe === 0 && (me[0].trace._xA = ce, me[0].trace._yA = le);
                var ye = me[pe], ge = ye.r;
                if (ge === ne) ye.x = ye.y = ne;
                else {
                  var ve = de.c2g(ge), be = fe.c2g(ye.theta);
                  ye.x = ve * Math.cos(be), ye.y = ve * Math.sin(be);
                }
              }
              te(ee, ue, oe, se);
            };
          }, 58319: function(G, J, Z) {
            var te = Z(8738), ne = Z(92089), ee = Z(3208).ay;
            G.exports = { mode: te.mode, r: te.r, theta: te.theta, r0: te.r0, dr: te.dr, theta0: te.theta0, dtheta: te.dtheta, thetaunit: te.thetaunit, text: te.text, texttemplate: ee({ editType: "plot" }, { keys: ["r", "theta", "text"] }), hovertext: te.hovertext, hovertemplate: te.hovertemplate, line: { color: ne.line.color, width: ne.line.width, dash: ne.line.dash, editType: "calc" }, connectgaps: ne.connectgaps, marker: ne.marker, fill: ne.fill, fillcolor: ne.fillcolor, textposition: ne.textposition, textfont: ne.textfont, hoverinfo: te.hoverinfo, selected: te.selected, unselected: te.unselected };
          }, 25796: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "scatterpolargl", basePlotModule: Z(31645), categories: ["gl", "regl", "polar", "symbols", "showLegend", "scatter-like"], attributes: Z(58319), supplyDefaults: Z(14952), colorbar: Z(21146), formatLabels: Z(94015), calc: Z(71535), hoverPoints: Z(47522).hoverPoints, selectPoints: Z(17168), meta: {} };
          }, 71535: function(G, J, Z) {
            var te = Z(77272), ne = Z(26544).calcMarkerSize, ee = Z(19937), ae = Z(29714), oe = Z(29483).TOO_MANY_POINTS;
            G.exports = function(se, ce) {
              var le = se._fullLayout, ue = ce.subplot, de = le[ue].radialaxis, fe = le[ue].angularaxis, he = ce._r = de.makeCalcdata(ce, "r"), me = ce._theta = fe.makeCalcdata(ce, "theta"), pe = ce._length, ye = {};
              pe < he.length && (he = he.slice(0, pe)), pe < me.length && (me = me.slice(0, pe)), ye.r = he, ye.theta = me, te(se, ce);
              var ge, ve = ye.opts = ee.style(se, ce);
              return pe < oe ? ge = ne(ce, pe) : ve.marker && (ge = 2 * (ve.marker.sizeAvg || Math.max(ve.marker.size, 3))), ce._extremes.x = ae.findExtremes(de, he, { ppad: ge }), [{ x: !1, y: !1, t: ye, trace: ce }];
            };
          }, 14952: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(73749).handleRThetaDefaults, ae = Z(24272), oe = Z(98168), se = Z(663), ce = Z(54114), le = Z(32660).PTS_LINESONLY, ue = Z(58319);
            G.exports = function(de, fe, he, me) {
              function pe(ge, ve) {
                return te.coerce(de, fe, ue, ge, ve);
              }
              var ye = ee(de, fe, me, pe);
              ye ? (pe("thetaunit"), pe("mode", ye < le ? "lines+markers" : "lines"), pe("text"), pe("hovertext"), fe.hoveron !== "fills" && pe("hovertemplate"), ne.hasMarkers(fe) && ae(de, fe, he, me, pe, { noAngleRef: !0, noStandOff: !0 }), ne.hasLines(fe) && (oe(de, fe, he, me, pe), pe("connectgaps")), ne.hasText(fe) && (pe("texttemplate"), se(de, fe, me, pe, { noFontShadow: !0, noFontLineposition: !0, noFontTextcase: !0 })), pe("fill"), fe.fill !== "none" && ce(de, fe, he, pe), te.coerceSelectionMarkerOpacity(fe, pe)) : fe.visible = !1;
            };
          }, 94015: function(G, J, Z) {
            var te = Z(33368);
            G.exports = function(ne, ee, ae) {
              var oe = ne.i;
              return "r" in ne || (ne.r = ee._r[oe]), "theta" in ne || (ne.theta = ee._theta[oe]), te(ne, ee, ae);
            };
          }, 47522: function(G, J, Z) {
            var te = Z(36544), ne = Z(29709).makeHoverPointText;
            G.exports = { hoverPoints: function(ee, ae, oe, se) {
              var ce = ee.cd[0].t, le = ce.r, ue = ce.theta, de = te.hoverPoints(ee, ae, oe, se);
              if (de && de[0].index !== !1) {
                var fe = de[0];
                if (fe.index === void 0) return de;
                var he = ee.subplot, me = fe.cd[fe.index], pe = fe.trace;
                if (me.r = le[fe.index], me.theta = ue[fe.index], he.isPtInside(me)) return fe.xLabelVal = void 0, fe.yLabelVal = void 0, ne(me, pe, he, fe), de;
              }
            } };
          }, 23748: function(G, J, Z) {
            var te = Z(25796);
            te.plot = Z(54121), G.exports = te;
          }, 54121: function(G, J, Z) {
            var te = Z(27549), ne = Z(10721), ee = Z(47731), ae = Z(62336), oe = Z(19937), se = Z(34809), ce = Z(29483).TOO_MANY_POINTS;
            G.exports = function(le, ue, de) {
              if (de.length) {
                var fe = ue.radialAxis, he = ue.angularAxis, me = ae(le, ue);
                return de.forEach(function(pe) {
                  if (pe && pe[0] && pe[0].trace) {
                    var ye, ge = pe[0], ve = ge.trace, be = ge.t, Ce = ve._length, Se = be.r, we = be.theta, Ee = be.opts, xe = Se.slice(), Le = we.slice();
                    for (ye = 0; ye < Se.length; ye++) ue.isPtInside({ r: Se[ye], theta: we[ye] }) || (xe[ye] = NaN, Le[ye] = NaN);
                    var Qe = new Array(2 * Ce), ke = Array(Ce), Me = Array(Ce);
                    for (ye = 0; ye < Ce; ye++) {
                      var Re, De, Pe = xe[ye];
                      if (ne(Pe)) {
                        var $e = fe.c2g(Pe), qe = he.c2g(Le[ye], ve.thetaunit);
                        Re = $e * Math.cos(qe), De = $e * Math.sin(qe);
                      } else Re = De = NaN;
                      ke[ye] = Qe[2 * ye] = Re, Me[ye] = Qe[2 * ye + 1] = De;
                    }
                    be.tree = te(Qe), Ee.marker && Ce >= ce && (Ee.marker.cluster = be.tree), Ee.marker && (Ee.markerSel.positions = Ee.markerUnsel.positions = Ee.marker.positions = Qe), Ee.line && Qe.length > 1 && se.extendFlat(Ee.line, oe.linePositions(le, ve, Qe)), Ee.text && (se.extendFlat(Ee.text, { positions: Qe }, oe.textPosition(le, ve, Ee.text, Ee.marker)), se.extendFlat(Ee.textSel, { positions: Qe }, oe.textPosition(le, ve, Ee.text, Ee.markerSel)), se.extendFlat(Ee.textUnsel, { positions: Qe }, oe.textPosition(le, ve, Ee.text, Ee.markerUnsel))), Ee.fill && !me.fill2d && (me.fill2d = !0), Ee.marker && !me.scatter2d && (me.scatter2d = !0), Ee.line && !me.line2d && (me.line2d = !0), Ee.text && !me.glText && (me.glText = !0), me.lineOptions.push(Ee.line), me.fillOptions.push(Ee.fill), me.markerOptions.push(Ee.marker), me.markerSelectedOptions.push(Ee.markerSel), me.markerUnselectedOptions.push(Ee.markerUnsel), me.textOptions.push(Ee.text), me.textSelectedOptions.push(Ee.textSel), me.textUnselectedOptions.push(Ee.textUnsel), me.selectBatch.push([]), me.unselectBatch.push([]), be.x = ke, be.y = Me, be.rawx = ke, be.rawy = Me, be.r = Se, be.theta = we, be.positions = Qe, be._scene = me, be.index = me.count, me.count++;
                  }
                }), ee(le, ue, de);
              }
            }, G.exports.reglPrecompiled = {};
          }, 69595: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(93049).extendFlat, ae = Z(19326), oe = Z(36640), se = Z(9829), ce = oe.line;
            G.exports = { mode: oe.mode, real: { valType: "data_array", editType: "calc+clearAxisTypes" }, imag: { valType: "data_array", editType: "calc+clearAxisTypes" }, text: oe.text, texttemplate: ne({ editType: "plot" }, { keys: ["real", "imag", "text"] }), hovertext: oe.hovertext, line: { color: ce.color, width: ce.width, dash: ce.dash, backoff: ce.backoff, shape: ee({}, ce.shape, { values: ["linear", "spline"] }), smoothing: ce.smoothing, editType: "calc" }, connectgaps: oe.connectgaps, marker: oe.marker, cliponaxis: ee({}, oe.cliponaxis, { dflt: !1 }), textposition: oe.textposition, textfont: oe.textfont, fill: ee({}, oe.fill, { values: ["none", "toself", "tonext"], dflt: "none" }), fillcolor: ae(), hoverinfo: ee({}, se.hoverinfo, { flags: ["real", "imag", "text", "name"] }), hoveron: oe.hoveron, hovertemplate: te(), selected: oe.selected, unselected: oe.unselected };
          }, 44315: function(G, J, Z) {
            var te = Z(10721), ne = Z(63821).BADNUM, ee = Z(77272), ae = Z(99203), oe = Z(48861), se = Z(26544).calcMarkerSize;
            G.exports = function(ce, le) {
              for (var ue = ce._fullLayout, de = le.subplot, fe = ue[de].realaxis, he = ue[de].imaginaryaxis, me = fe.makeCalcdata(le, "real"), pe = he.makeCalcdata(le, "imag"), ye = le._length, ge = new Array(ye), ve = 0; ve < ye; ve++) {
                var be = me[ve], Ce = pe[ve], Se = ge[ve] = {};
                te(be) && te(Ce) ? (Se.real = be, Se.imag = Ce) : Se.real = ne;
              }
              return se(le, ye), ee(ce, le), ae(ge, le), oe(ge, le), ge;
            };
          }, 93788: function(G, J, Z) {
            var te = Z(34809), ne = Z(64726), ee = Z(24272), ae = Z(98168), oe = Z(91602), se = Z(663), ce = Z(54114), le = Z(32660).PTS_LINESONLY, ue = Z(69595);
            G.exports = function(de, fe, he, me) {
              function pe(ve, be) {
                return te.coerce(de, fe, ue, ve, be);
              }
              var ye = function(ve, be, Ce, Se) {
                var we, Ee = Se("real"), xe = Se("imag");
                return Ee && xe && (we = Math.min(Ee.length, xe.length)), te.isTypedArray(Ee) && (be.real = Ee = Array.from(Ee)), te.isTypedArray(xe) && (be.imag = xe = Array.from(xe)), be._length = we, we;
              }(0, fe, 0, pe);
              if (ye) {
                pe("mode", ye < le ? "lines+markers" : "lines"), pe("text"), pe("hovertext"), fe.hoveron !== "fills" && pe("hovertemplate"), ne.hasMarkers(fe) && ee(de, fe, he, me, pe, { gradient: !0 }), ne.hasLines(fe) && (ae(de, fe, he, me, pe, { backoff: !0 }), oe(de, fe, pe), pe("connectgaps")), ne.hasText(fe) && (pe("texttemplate"), se(de, fe, me, pe));
                var ge = [];
                (ne.hasMarkers(fe) || ne.hasText(fe)) && (pe("cliponaxis"), pe("marker.maxdisplayed"), ge.push("points")), pe("fill"), fe.fill !== "none" && (ce(de, fe, he, pe), ne.hasLines(fe) || oe(de, fe, pe)), fe.fill !== "tonext" && fe.fill !== "toself" || ge.push("fills"), pe("hoveron", ge.join("+") || "points"), te.coerceSelectionMarkerOpacity(fe, pe);
              } else fe.visible = !1;
            };
          }, 89419: function(G, J, Z) {
            var te = Z(29714);
            G.exports = function(ne, ee, ae) {
              var oe = {}, se = ae[ee.subplot]._subplot;
              return oe.realLabel = te.tickText(se.radialAxis, ne.real, !0).text, oe.imagLabel = te.tickText(se.angularAxis, ne.imag, !0).text, oe;
            };
          }, 64422: function(G, J, Z) {
            var te = Z(37255);
            function ne(ee, ae, oe, se) {
              var ce = oe.radialAxis, le = oe.angularAxis;
              ce._hovertitle = "real", le._hovertitle = "imag";
              var ue = {};
              ue[ae.subplot] = { _subplot: oe };
              var de = ae._module.formatLabels(ee, ae, ue);
              se.realLabel = de.realLabel, se.imagLabel = de.imagLabel;
              var fe = ee.hi || ae.hoverinfo, he = [];
              function me(ye, ge) {
                he.push(ye._hovertitle + ": " + ge);
              }
              if (!ae.hovertemplate) {
                var pe = fe.split("+");
                pe.indexOf("all") !== -1 && (pe = ["real", "imag", "text"]), pe.indexOf("real") !== -1 && me(ce, se.realLabel), pe.indexOf("imag") !== -1 && me(le, se.imagLabel), pe.indexOf("text") !== -1 && se.text && (he.push(se.text), delete se.text), se.extraText = he.join("<br>");
              }
            }
            G.exports = { hoverPoints: function(ee, ae, oe, se) {
              var ce = te(ee, ae, oe, se);
              if (ce && ce[0].index !== !1) {
                var le = ce[0];
                if (le.index === void 0) return ce;
                var ue = ee.subplot, de = le.cd[le.index], fe = le.trace;
                if (ue.isPtInside(de)) return le.xLabelVal = void 0, le.yLabelVal = void 0, ne(de, fe, ue, le), le.hovertemplate = fe.hovertemplate, ce;
              }
            }, makeHoverPointText: ne };
          }, 73304: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "scattersmith", basePlotModule: Z(50358), categories: ["smith", "symbols", "showLegend", "scatter-like"], attributes: Z(69595), supplyDefaults: Z(93788), colorbar: Z(21146), formatLabels: Z(89419), calc: Z(44315), plot: Z(6229), style: Z(9408).style, styleOnSelect: Z(9408).styleOnSelect, hoverPoints: Z(64422).hoverPoints, selectPoints: Z(32665), meta: {} };
          }, 6229: function(G, J, Z) {
            var te = Z(36098), ne = Z(63821).BADNUM, ee = Z(52007).smith;
            G.exports = function(ae, oe, se) {
              for (var ce = oe.layers.frontplot.select("g.scatterlayer"), le = oe.xaxis, ue = oe.yaxis, de = { xaxis: le, yaxis: ue, plot: oe.framework, layerClipId: oe._hasClipOnAxisFalse ? oe.clipIds.forTraces : null }, fe = 0; fe < se.length; fe++) for (var he = se[fe], me = 0; me < he.length; me++) {
                me === 0 && (he[0].trace._xA = le, he[0].trace._yA = ue);
                var pe = he[me], ye = pe.real;
                if (ye === ne) pe.x = pe.y = ne;
                else {
                  var ge = ee([ye, pe.imag]);
                  pe.x = ge[0], pe.y = ge[1];
                }
              }
              te(ae, de, se, ce);
            };
          }, 18483: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(19326), ae = Z(36640), oe = Z(9829), se = Z(87163), ce = Z(94850).T, le = Z(93049).extendFlat, ue = ae.marker, de = ae.line, fe = ue.line;
            G.exports = { a: { valType: "data_array", editType: "calc" }, b: { valType: "data_array", editType: "calc" }, c: { valType: "data_array", editType: "calc" }, sum: { valType: "number", dflt: 0, min: 0, editType: "calc" }, mode: le({}, ae.mode, { dflt: "markers" }), text: le({}, ae.text, {}), texttemplate: ne({ editType: "plot" }, { keys: ["a", "b", "c", "text"] }), hovertext: le({}, ae.hovertext, {}), line: { color: de.color, width: de.width, dash: ce, backoff: de.backoff, shape: le({}, de.shape, { values: ["linear", "spline"] }), smoothing: de.smoothing, editType: "calc" }, connectgaps: ae.connectgaps, cliponaxis: ae.cliponaxis, fill: le({}, ae.fill, { values: ["none", "toself", "tonext"], dflt: "none" }), fillcolor: ee(), marker: le({ symbol: ue.symbol, opacity: ue.opacity, angle: ue.angle, angleref: ue.angleref, standoff: ue.standoff, maxdisplayed: ue.maxdisplayed, size: ue.size, sizeref: ue.sizeref, sizemin: ue.sizemin, sizemode: ue.sizemode, line: le({ width: fe.width, editType: "calc" }, se("marker.line")), gradient: ue.gradient, editType: "calc" }, se("marker")), textfont: ae.textfont, textposition: ae.textposition, selected: ae.selected, unselected: ae.unselected, hoverinfo: le({}, oe.hoverinfo, { flags: ["a", "b", "c", "text", "name"] }), hoveron: ae.hoveron, hovertemplate: te() };
          }, 67091: function(G, J, Z) {
            var te = Z(10721), ne = Z(77272), ee = Z(99203), ae = Z(48861), oe = Z(26544).calcMarkerSize, se = ["a", "b", "c"], ce = { a: ["b", "c"], b: ["a", "c"], c: ["a", "b"] };
            G.exports = function(le, ue) {
              var de, fe, he, me, pe, ye, ge = le._fullLayout[ue.subplot].sum, ve = ue.sum || ge, be = { a: ue.a, b: ue.b, c: ue.c };
              for (de = 0; de < se.length; de++) if (!be[he = se[de]]) {
                for (pe = be[ce[he][0]], ye = be[ce[he][1]], me = new Array(pe.length), fe = 0; fe < pe.length; fe++) me[fe] = ve - pe[fe] - ye[fe];
                be[he] = me;
              }
              var Ce, Se, we, Ee, xe, Le, Qe = ue._length, ke = new Array(Qe);
              for (de = 0; de < Qe; de++) Ce = be.a[de], Se = be.b[de], we = be.c[de], te(Ce) && te(Se) && te(we) ? ((Ee = ge / ((Ce = +Ce) + (Se = +Se) + (we = +we))) != 1 && (Ce *= Ee, Se *= Ee, we *= Ee), Le = Ce, xe = we - Se, ke[de] = { x: xe, y: Le, a: Ce, b: Se, c: we }) : ke[de] = { x: !1, y: !1 };
              return oe(ue, Qe), ne(le, ue), ee(ke, ue), ae(ke, ue), ke;
            };
          }, 79028: function(G, J, Z) {
            var te = Z(34809), ne = Z(32660), ee = Z(64726), ae = Z(24272), oe = Z(98168), se = Z(91602), ce = Z(663), le = Z(54114), ue = Z(18483);
            G.exports = function(de, fe, he, me) {
              function pe(Se, we) {
                return te.coerce(de, fe, ue, Se, we);
              }
              var ye, ge = pe("a"), ve = pe("b"), be = pe("c");
              if (ge ? (ye = ge.length, ve ? (ye = Math.min(ye, ve.length), be && (ye = Math.min(ye, be.length))) : ye = be ? Math.min(ye, be.length) : 0) : ve && be && (ye = Math.min(ve.length, be.length)), ye) {
                fe._length = ye, pe("sum"), pe("text"), pe("hovertext"), fe.hoveron !== "fills" && pe("hovertemplate"), pe("mode", ye < ne.PTS_LINESONLY ? "lines+markers" : "lines"), ee.hasMarkers(fe) && ae(de, fe, he, me, pe, { gradient: !0 }), ee.hasLines(fe) && (oe(de, fe, he, me, pe, { backoff: !0 }), se(de, fe, pe), pe("connectgaps")), ee.hasText(fe) && (pe("texttemplate"), ce(de, fe, me, pe));
                var Ce = [];
                (ee.hasMarkers(fe) || ee.hasText(fe)) && (pe("cliponaxis"), pe("marker.maxdisplayed"), Ce.push("points")), pe("fill"), fe.fill !== "none" && (le(de, fe, he, pe), ee.hasLines(fe) || se(de, fe, pe)), fe.fill !== "tonext" && fe.fill !== "toself" || Ce.push("fills"), pe("hoveron", Ce.join("+") || "points"), te.coerceSelectionMarkerOpacity(fe, pe);
              } else fe.visible = !1;
            };
          }, 94343: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              if (Z.xa && (J.xaxis = Z.xa), Z.ya && (J.yaxis = Z.ya), ne[ee]) {
                var ae = ne[ee];
                J.a = ae.a, J.b = ae.b, J.c = ae.c;
              } else J.a = Z.a, J.b = Z.b, J.c = Z.c;
              return J;
            };
          }, 78995: function(G, J, Z) {
            var te = Z(29714);
            G.exports = function(ne, ee, ae) {
              var oe = {}, se = ae[ee.subplot]._subplot;
              return oe.aLabel = te.tickText(se.aaxis, ne.a, !0).text, oe.bLabel = te.tickText(se.baxis, ne.b, !0).text, oe.cLabel = te.tickText(se.caxis, ne.c, !0).text, oe;
            };
          }, 26558: function(G, J, Z) {
            var te = Z(37255);
            G.exports = function(ne, ee, ae, oe) {
              var se = te(ne, ee, ae, oe);
              if (se && se[0].index !== !1) {
                var ce = se[0];
                if (ce.index === void 0) {
                  var le = 1 - ce.y0 / ne.ya._length, ue = ne.xa._length, de = ue * le / 2, fe = ue - de;
                  return ce.x0 = Math.max(Math.min(ce.x0, fe), de), ce.x1 = Math.max(Math.min(ce.x1, fe), de), se;
                }
                var he = ce.cd[ce.index], me = ce.trace, pe = ce.subplot;
                ce.a = he.a, ce.b = he.b, ce.c = he.c, ce.xLabelVal = void 0, ce.yLabelVal = void 0;
                var ye = {};
                ye[me.subplot] = { _subplot: pe };
                var ge = me._module.formatLabels(he, me, ye);
                ce.aLabel = ge.aLabel, ce.bLabel = ge.bLabel, ce.cLabel = ge.cLabel;
                var ve = he.hi || me.hoverinfo, be = [];
                if (!me.hovertemplate) {
                  var Ce = ve.split("+");
                  Ce.indexOf("all") !== -1 && (Ce = ["a", "b", "c"]), Ce.indexOf("a") !== -1 && Se(pe.aaxis, ce.aLabel), Ce.indexOf("b") !== -1 && Se(pe.baxis, ce.bLabel), Ce.indexOf("c") !== -1 && Se(pe.caxis, ce.cLabel);
                }
                return ce.extraText = be.join("<br>"), ce.hovertemplate = me.hovertemplate, se;
              }
              function Se(we, Ee) {
                be.push(we._hovertitle + ": " + Ee);
              }
            };
          }, 12864: function(G, J, Z) {
            G.exports = { attributes: Z(18483), supplyDefaults: Z(79028), colorbar: Z(21146), formatLabels: Z(78995), calc: Z(67091), plot: Z(79005), style: Z(9408).style, styleOnSelect: Z(9408).styleOnSelect, hoverPoints: Z(26558), selectPoints: Z(32665), eventData: Z(94343), moduleType: "trace", name: "scatterternary", basePlotModule: Z(7638), categories: ["ternary", "symbols", "showLegend", "scatter-like"], meta: {} };
          }, 79005: function(G, J, Z) {
            var te = Z(36098);
            G.exports = function(ne, ee, ae) {
              var oe = ee.plotContainer;
              oe.select(".scatterlayer").selectAll("*").remove();
              for (var se = ee.xaxis, ce = ee.yaxis, le = { xaxis: se, yaxis: ce, plot: oe, layerClipId: ee._hasClipOnAxisFalse ? ee.clipIdRelative : null }, ue = ee.layers.frontplot.select("g.scatterlayer"), de = 0; de < ae.length; de++) {
                var fe = ae[de];
                fe.length && (fe[0].trace._xA = se, fe[0].trace._yA = ce);
              }
              te(ne, le, ae, ue);
            };
          }, 68697: function(G, J, Z) {
            var te = Z(36640), ne = Z(87163), ee = Z(80712).axisHoverFormat, ae = Z(3208).rb, oe = Z(92089), se = Z(54826).idRegex, ce = Z(78032).templatedArray, le = Z(93049).extendFlat, ue = te.marker, de = ue.line, fe = le(ne("marker.line", { editTypeOverride: "calc" }), { width: le({}, de.width, { editType: "calc" }), editType: "calc" }), he = le(ne("marker"), { symbol: ue.symbol, angle: ue.angle, size: le({}, ue.size, { editType: "markerSize" }), sizeref: ue.sizeref, sizemin: ue.sizemin, sizemode: ue.sizemode, opacity: ue.opacity, colorbar: ue.colorbar, line: fe, editType: "calc" });
            function me(pe) {
              return { valType: "info_array", freeLength: !0, editType: "calc", items: { valType: "subplotid", regex: se[pe], editType: "plot" } };
            }
            he.color.editType = he.cmin.editType = he.cmax.editType = "style", G.exports = { dimensions: ce("dimension", { visible: { valType: "boolean", dflt: !0, editType: "calc" }, label: { valType: "string", editType: "calc" }, values: { valType: "data_array", editType: "calc+clearAxisTypes" }, axis: { type: { valType: "enumerated", values: ["linear", "log", "date", "category"], editType: "calc+clearAxisTypes" }, matches: { valType: "boolean", dflt: !1, editType: "calc" }, editType: "calc+clearAxisTypes" }, editType: "calc+clearAxisTypes" }), text: le({}, oe.text, {}), hovertext: le({}, oe.hovertext, {}), hovertemplate: ae(), xhoverformat: ee("x"), yhoverformat: ee("y"), marker: he, xaxes: me("x"), yaxes: me("y"), diagonal: { visible: { valType: "boolean", dflt: !0, editType: "calc" }, editType: "calc" }, showupperhalf: { valType: "boolean", dflt: !0, editType: "calc" }, showlowerhalf: { valType: "boolean", dflt: !0, editType: "calc" }, selected: { marker: oe.selected.marker, editType: "calc" }, unselected: { marker: oe.unselected.marker, editType: "calc" }, opacity: oe.opacity };
          }, 86690: function(G, J, Z) {
            var te = Z(33626), ne = Z(83595);
            G.exports = { moduleType: "trace", name: "splom", categories: ["gl", "regl", "cartesian", "symbols", "showLegend", "scatter-like"], attributes: Z(68697), supplyDefaults: Z(52542), colorbar: Z(21146), calc: Z(55325), plot: Z(83027), hoverPoints: Z(25600).hoverPoints, selectPoints: Z(13392), editStyle: Z(27926), meta: {} }, te.register(ne);
          }, 571: function(G, J, Z) {
            var te = Z(49478), ne = Z(33626), ee = Z(22459), ae = Z(4173).eV, oe = Z(37703), se = Z(5975).getFromId, ce = Z(29714).shouldShowZeroLine, le = "splom", ue = {};
            function de(he, me, pe) {
              for (var ye = pe.matrixOptions.data.length, ge = me._visibleDims, ve = pe.viewOpts.ranges = new Array(ye), be = 0; be < ge.length; be++) {
                var Ce = ge[be], Se = ve[be] = new Array(4), we = se(he, me._diag[Ce][0]);
                we && (Se[0] = we.r2l(we.range[0]), Se[2] = we.r2l(we.range[1]));
                var Ee = se(he, me._diag[Ce][1]);
                Ee && (Se[1] = Ee.r2l(Ee.range[0]), Se[3] = Ee.r2l(Ee.range[1]));
              }
              pe.selectBatch.length || pe.unselectBatch.length ? pe.matrix.update({ ranges: ve }, { ranges: ve }) : pe.matrix.update({ ranges: ve });
            }
            function fe(he) {
              var me = he._fullLayout, pe = me._glcanvas.data()[0].regl, ye = me._splomGrid;
              ye || (ye = me._splomGrid = te(pe)), ye.update(function(ge) {
                var ve, be = ge._context.plotGlPixelRatio, Ce = ge._fullLayout, Se = Ce._size, we = [0, 0, Ce.width * be, Ce.height * be], Ee = {};
                function xe(We, at, ct, et, dt, Je) {
                  ct *= be, et *= be, dt *= be, Je *= be;
                  var it = at[We + "color"], st = at[We + "width"], gt = String(it + st);
                  gt in Ee ? Ee[gt].data.push(NaN, NaN, ct, et, dt, Je) : Ee[gt] = { data: [ct, et, dt, Je], join: "rect", thickness: st * be, color: it, viewport: we, range: we, overlay: !1 };
                }
                for (ve in Ce._splomSubplots) {
                  var Le, Qe, ke = Ce._plots[ve], Me = ke.xaxis, Re = ke.yaxis, De = Me._gridVals, Pe = Re._gridVals, $e = Me._offset, qe = Me._length, Xe = Re._length, Ve = Se.b + Re.domain[0] * Se.h, Ze = -Re._m, Ge = -Ze * Re.r2l(Re.range[0], Re.calendar);
                  if (Me.showgrid) for (ve = 0; ve < De.length; ve++) Le = $e + Me.l2p(De[ve].x), xe("grid", Me, Le, Ve, Le, Ve + Xe);
                  if (Re.showgrid) for (ve = 0; ve < Pe.length; ve++) xe("grid", Re, $e, Qe = Ve + Ge + Ze * Pe[ve].x, $e + qe, Qe);
                  ce(ge, Me, Re) && (Le = $e + Me.l2p(0), xe("zeroline", Me, Le, Ve, Le, Ve + Xe)), ce(ge, Re, Me) && xe("zeroline", Re, $e, Qe = Ve + Ge + 0, $e + qe, Qe);
                }
                var ze = [];
                for (ve in Ee) ze.push(Ee[ve]);
                return ze;
              }(he));
            }
            G.exports = { name: le, attr: oe.attr, attrRegex: oe.attrRegex, layoutAttributes: oe.layoutAttributes, supplyLayoutDefaults: oe.supplyLayoutDefaults, drawFramework: oe.drawFramework, plot: function(he) {
              var me = he._fullLayout, pe = ne.getModule(le), ye = ae(he.calcdata, pe)[0];
              ee(he, ["ANGLE_instanced_arrays", "OES_element_index_uint"], ue) && (me._hasOnlyLargeSploms && fe(he), pe.plot(he, {}, ye));
            }, drag: function(he) {
              var me = he.calcdata, pe = he._fullLayout;
              pe._hasOnlyLargeSploms && fe(he);
              for (var ye = 0; ye < me.length; ye++) {
                var ge = me[ye][0].trace, ve = pe._splomScenes[ge.uid];
                ge.type === "splom" && ve && ve.matrix && de(he, ge, ve);
              }
            }, updateGrid: fe, clean: function(he, me, pe, ye) {
              var ge, ve = {};
              if (ye._splomScenes) {
                for (ge = 0; ge < he.length; ge++) {
                  var be = he[ge];
                  be.type === "splom" && (ve[be.uid] = 1);
                }
                for (ge = 0; ge < pe.length; ge++) {
                  var Ce = pe[ge];
                  if (!ve[Ce.uid]) {
                    var Se = ye._splomScenes[Ce.uid];
                    Se && Se.destroy && Se.destroy(), ye._splomScenes[Ce.uid] = null, delete ye._splomScenes[Ce.uid];
                  }
                }
              }
              Object.keys(ye._splomScenes || {}).length === 0 && delete ye._splomScenes, ye._splomGrid && !me._hasOnlyLargeSploms && ye._hasOnlyLargeSploms && (ye._splomGrid.destroy(), ye._splomGrid = null, delete ye._splomGrid), oe.clean(he, me, pe, ye);
            }, updateFx: oe.updateFx, toSVG: oe.toSVG, reglPrecompiled: ue };
          }, 55325: function(G, J, Z) {
            var te = Z(34809), ne = Z(5975), ee = Z(26544).calcMarkerSize, ae = Z(26544).calcAxisExpansion, oe = Z(77272), se = Z(19937).markerSelection, ce = Z(19937).markerStyle, le = Z(78880), ue = Z(63821).BADNUM, de = Z(29483).TOO_MANY_POINTS;
            G.exports = function(fe, he) {
              var me, pe, ye, ge, ve, be, Ce = he.dimensions, Se = he._length, we = {}, Ee = we.cdata = [], xe = we.data = [], Le = he._visibleDims = [];
              function Qe(Me, Re) {
                for (var De = Me.makeCalcdata({ v: Re.values, vcalendar: he.calendar }, "v"), Pe = 0; Pe < De.length; Pe++) De[Pe] = De[Pe] === ue ? NaN : De[Pe];
                Ee.push(De), xe.push(Me.type === "log" ? te.simpleMap(De, Me.c2l) : De);
              }
              for (me = 0; me < Ce.length; me++) if ((ye = Ce[me]).visible) {
                if (ge = ne.getFromId(fe, he._diag[me][0]), ve = ne.getFromId(fe, he._diag[me][1]), ge && ve && ge.type !== ve.type) {
                  te.log("Skipping splom dimension " + me + " with conflicting axis types");
                  continue;
                }
                ge ? (Qe(ge, ye), ve && ve.type === "category" && (ve._categories = ge._categories.slice())) : Qe(ve, ye), Le.push(me);
              }
              for (oe(fe, he), te.extendFlat(we, ce(fe, he)), be = Ee.length * Se > de ? we.sizeAvg || Math.max(we.size, 3) : ee(he, Se), pe = 0; pe < Le.length; pe++) ye = Ce[me = Le[pe]], ge = ne.getFromId(fe, he._diag[me][0]) || {}, ve = ne.getFromId(fe, he._diag[me][1]) || {}, ae(fe, he, ge, ve, Ee[pe], Ee[pe], be);
              var ke = le(fe, he);
              return ke.matrix || (ke.matrix = !0), ke.matrixOptions = we, ke.selectedOptions = se(fe, he, he.selected), ke.unselectedOptions = se(fe, he, he.unselected), [{ x: !1, y: !1, t: {}, trace: he }];
            };
          }, 52542: function(G, J, Z) {
            var te = Z(34809), ne = Z(59008), ee = Z(68697), ae = Z(64726), oe = Z(24272), se = Z(63197), ce = Z(4075).isOpenSymbol;
            function le(ue, de) {
              function fe(me, pe) {
                return te.coerce(ue, de, ee.dimensions, me, pe);
              }
              fe("label");
              var he = fe("values");
              he && he.length ? fe("visible") : de.visible = !1, fe("axis.type"), fe("axis.matches");
            }
            G.exports = function(ue, de, fe, he) {
              function me(Se, we) {
                return te.coerce(ue, de, ee, Se, we);
              }
              var pe = ne(ue, de, { name: "dimensions", handleItemDefaults: le }), ye = me("diagonal.visible"), ge = me("showupperhalf"), ve = me("showlowerhalf");
              if (se(de, pe, "values") && (ye || ge || ve)) {
                me("text"), me("hovertext"), me("hovertemplate"), me("xhoverformat"), me("yhoverformat"), oe(ue, de, fe, he, me, { noAngleRef: !0, noStandOff: !0 });
                var be = ce(de.marker.symbol), Ce = ae.isBubble(de);
                me("marker.line.width", be || Ce ? 1 : 0), function(Se, we, Ee, xe) {
                  var Le, Qe, ke = we.dimensions, Me = ke.length, Re = we.showupperhalf, De = we.showlowerhalf, Pe = we.diagonal.visible, $e = new Array(Me), qe = new Array(Me);
                  for (Le = 0; Le < Me; Le++) {
                    var Xe = Le ? Le + 1 : "";
                    $e[Le] = "x" + Xe, qe[Le] = "y" + Xe;
                  }
                  var Ve = xe("xaxes", $e), Ze = xe("yaxes", qe), Ge = we._diag = new Array(Me);
                  we._xaxes = {}, we._yaxes = {};
                  var ze = [], We = [];
                  function at(yt, St, Lt, Rt) {
                    if (yt) {
                      var vt = yt.charAt(0), $t = Ee._splomAxes[vt];
                      if (we["_" + vt + "axes"][yt] = 1, Rt.push(yt), !(yt in $t)) {
                        var Pt = $t[yt] = {};
                        Lt && (Pt.label = Lt.label || "", Lt.visible && Lt.axis && (Lt.axis.type && (Pt.type = Lt.axis.type), Lt.axis.matches && (Pt.matches = St)));
                      }
                    }
                  }
                  var ct = !Pe && !De, et = !Pe && !Re;
                  for (we._axesDim = {}, Le = 0; Le < Me; Le++) {
                    var dt = ke[Le], Je = Le === 0, it = Le === Me - 1, st = Je && ct || it && et ? void 0 : Ve[Le], gt = Je && et || it && ct ? void 0 : Ze[Le];
                    at(st, gt, dt, ze), at(gt, st, dt, We), Ge[Le] = [st, gt], we._axesDim[st] = Le, we._axesDim[gt] = Le;
                  }
                  for (Le = 0; Le < ze.length; Le++) for (Qe = 0; Qe < We.length; Qe++) {
                    var bt = ze[Le] + We[Qe];
                    Le > Qe && Re || Le < Qe && De ? Ee._splomSubplots[bt] = 1 : Le !== Qe || !Pe && De && Re || (Ee._splomSubplots[bt] = 1);
                  }
                  (!De || !Pe && Re && De) && (Ee._splomGridDflt.xside = "bottom", Ee._splomGridDflt.yside = "left");
                }(0, de, he, me), te.coerceSelectionMarkerOpacity(de, me);
              } else de.visible = !1;
            };
          }, 27926: function(G, J, Z) {
            var te = Z(34809), ne = Z(77272), ee = Z(19937).markerStyle;
            G.exports = function(ae, oe) {
              var se = oe.trace, ce = ae._fullLayout._splomScenes[se.uid];
              if (ce) {
                ne(ae, se), te.extendFlat(ce.matrixOptions, ee(ae, se));
                var le = te.extendFlat({}, ce.matrixOptions, ce.viewOpts);
                ce.matrix.update(le, null);
              }
            };
          }, 52875: function(G, J) {
            J.getDimIndex = function(Z, te) {
              for (var ne = te._id, ee = { x: 0, y: 1 }[ne.charAt(0)], ae = Z._visibleDims, oe = 0; oe < ae.length; oe++) {
                var se = ae[oe];
                if (Z._diag[se][ee] === ne) return oe;
              }
              return !1;
            };
          }, 25600: function(G, J, Z) {
            var te = Z(52875), ne = Z(36544).calcHover, ee = Z(29714).getFromId, ae = Z(93049).extendFlat;
            function oe(se, ce, le, ue) {
              var de = se.cd[0].trace, fe = se.scene.matrixOptions.cdata, he = se.xa, me = se.ya, pe = he.c2p(ce), ye = me.c2p(le), ge = se.distance, ve = te.getDimIndex(de, he), be = te.getDimIndex(de, me);
              if (ve === !1 || be === !1) return [se];
              for (var Ce, Se, we = fe[ve], Ee = fe[be], xe = ge, Le = 0; Le < we.length; Le++) if (!ue || Le === se.index) {
                var Qe = we[Le], ke = Ee[Le], Me = he.c2p(Qe) - pe, Re = me.c2p(ke) - ye, De = Math.sqrt(Me * Me + Re * Re);
                (ue || De < xe) && (xe = Se = De, Ce = Le);
              }
              return se.index = Ce, se.distance = xe, se.dxy = Se, Ce === void 0 ? [se] : [ne(se, we, Ee, de)];
            }
            G.exports = { hoverPoints: function(se, ce, le, ue, de) {
              de || (de = {});
              var fe = (ue || "").charAt(0) === "x", he = (ue || "").charAt(0) === "y", me = oe(se, ce, le);
              if ((fe || he) && de.hoversubplots === "axis" && me[0]) for (var pe = (fe ? se.xa : se.ya)._subplotsWith, ye = de.gd, ge = ae({}, se), ve = 0; ve < pe.length; ve++) {
                var be = pe[ve];
                if (be !== se.xa._id + se.ya._id) {
                  he ? ge.xa = ee(ye, be, "x") : ge.ya = ee(ye, be, "y");
                  var Ce = oe(ge, ce, le, fe || he);
                  me = me.concat(Ce);
                }
              }
              return me;
            } };
          }, 91450: function(G, J, Z) {
            var te = Z(86690);
            te.basePlotModule = Z(571), G.exports = te;
          }, 83027: function(G, J, Z) {
            var te = Z(31239), ne = Z(34809), ee = Z(5975), ae = Z(70414).selectMode;
            function oe(se, ce) {
              var le, ue, de, fe, he, me = se._fullLayout, pe = me._size, ye = ce.trace, ge = ce.t, ve = me._splomScenes[ye.uid], be = ve.matrixOptions, Ce = be.cdata, Se = me._glcanvas.data()[0].regl, we = me.dragmode;
              if (Ce.length !== 0) {
                be.lower = ye.showupperhalf, be.upper = ye.showlowerhalf, be.diagonal = ye.diagonal.visible;
                var Ee = ye._visibleDims, xe = Ce.length, Le = ve.viewOpts = {};
                for (Le.ranges = new Array(xe), Le.domains = new Array(xe), he = 0; he < Ee.length; he++) {
                  de = Ee[he];
                  var Qe = Le.ranges[he] = new Array(4), ke = Le.domains[he] = new Array(4);
                  (le = ee.getFromId(se, ye._diag[de][0])) && (Qe[0] = le._rl[0], Qe[2] = le._rl[1], ke[0] = le.domain[0], ke[2] = le.domain[1]), (ue = ee.getFromId(se, ye._diag[de][1])) && (Qe[1] = ue._rl[0], Qe[3] = ue._rl[1], ke[1] = ue.domain[0], ke[3] = ue.domain[1]);
                }
                var Me = se._context.plotGlPixelRatio, Re = pe.l * Me, De = pe.b * Me, Pe = pe.w * Me, $e = pe.h * Me;
                Le.viewport = [Re, De, Pe + Re, $e + De], ve.matrix === !0 && (ve.matrix = te(Se));
                var qe = me.clickmode.indexOf("select") > -1, Xe = !0;
                if (ae(we) || ye.selectedpoints || qe) {
                  var Ve = ye._length;
                  if (ye.selectedpoints) {
                    ve.selectBatch = ye.selectedpoints;
                    var Ze = ye.selectedpoints, Ge = {};
                    for (de = 0; de < Ze.length; de++) Ge[Ze[de]] = !0;
                    var ze = [];
                    for (de = 0; de < Ve; de++) Ge[de] || ze.push(de);
                    ve.unselectBatch = ze;
                  }
                  var We = ge.xpx = new Array(xe), at = ge.ypx = new Array(xe);
                  for (he = 0; he < Ee.length; he++) {
                    if (de = Ee[he], le = ee.getFromId(se, ye._diag[de][0])) for (We[he] = new Array(Ve), fe = 0; fe < Ve; fe++) We[he][fe] = le.c2p(Ce[he][fe]);
                    if (ue = ee.getFromId(se, ye._diag[de][1])) for (at[he] = new Array(Ve), fe = 0; fe < Ve; fe++) at[he][fe] = ue.c2p(Ce[he][fe]);
                  }
                  if (ve.selectBatch.length || ve.unselectBatch.length) {
                    var ct = ne.extendFlat({}, be, ve.unselectedOptions, Le), et = ne.extendFlat({}, be, ve.selectedOptions, Le);
                    ve.matrix.update(ct, et), Xe = !1;
                  }
                } else ge.xpx = ge.ypx = null;
                if (Xe) {
                  var dt = ne.extendFlat({}, be, Le);
                  ve.matrix.update(dt, null);
                }
              }
            }
            G.exports = function(se, ce, le) {
              if (le.length) for (var ue = 0; ue < le.length; ue++) oe(se, le[ue][0]);
            };
          }, 78880: function(G, J, Z) {
            var te = Z(34809);
            G.exports = function(ne, ee) {
              var ae = ne._fullLayout, oe = ee.uid, se = ae._splomScenes;
              se || (se = ae._splomScenes = {});
              var ce = { dirty: !0, selectBatch: [], unselectBatch: [] }, le = se[ee.uid];
              return le || ((le = se[oe] = te.extendFlat({}, ce, { matrix: !1, selectBatch: [], unselectBatch: [] })).draw = function() {
                le.matrix && le.matrix.draw && (le.selectBatch.length || le.unselectBatch.length ? le.matrix.draw(le.unselectBatch, le.selectBatch) : le.matrix.draw()), le.dirty = !1;
              }, le.destroy = function() {
                le.matrix && le.matrix.destroy && le.matrix.destroy(), le.matrixOptions = null, le.selectBatch = null, le.unselectBatch = null, le = null;
              }), le.dirty || te.extendFlat(le, ce), le;
            };
          }, 13392: function(G, J, Z) {
            var te = Z(34809), ne = te.pushUnique, ee = Z(64726), ae = Z(52875);
            G.exports = function(oe, se) {
              var ce = oe.cd, le = ce[0].trace, ue = ce[0].t, de = oe.scene, fe = de.matrixOptions.cdata, he = oe.xaxis, me = oe.yaxis, pe = [];
              if (!de) return pe;
              var ye = !ee.hasMarkers(le) && !ee.hasText(le);
              if (le.visible !== !0 || ye) return pe;
              var ge = ae.getDimIndex(le, he), ve = ae.getDimIndex(le, me);
              if (ge === !1 || ve === !1) return pe;
              var be = ue.xpx[ge], Ce = ue.ypx[ve], Se = fe[ge], we = fe[ve], Ee = (oe.scene.selectBatch || []).slice(), xe = [];
              if (se !== !1 && !se.degenerate) for (var Le = 0; Le < Se.length; Le++) se.contains([be[Le], Ce[Le]], null, Le, oe) ? (pe.push({ pointNumber: Le, x: Se[Le], y: we[Le] }), ne(Ee, Le)) : Ee.indexOf(Le) !== -1 ? ne(Ee, Le) : xe.push(Le);
              var Qe = de.matrixOptions;
              return Ee.length || xe.length ? de.selectBatch.length || de.unselectBatch.length || de.matrix.update(de.unselectedOptions, te.extendFlat({}, Qe, de.selectedOptions, de.viewOpts)) : de.matrix.update(Qe, null), de.selectBatch = Ee, de.unselectBatch = xe, pe;
            };
          }, 14774: function(G, J, Z) {
            var te = Z(87163), ne = Z(80712).axisHoverFormat, ee = Z(3208).rb, ae = Z(42450), oe = Z(9829), se = Z(93049).extendFlat, ce = { x: { valType: "data_array", editType: "calc+clearAxisTypes" }, y: { valType: "data_array", editType: "calc+clearAxisTypes" }, z: { valType: "data_array", editType: "calc+clearAxisTypes" }, u: { valType: "data_array", editType: "calc" }, v: { valType: "data_array", editType: "calc" }, w: { valType: "data_array", editType: "calc" }, starts: { x: { valType: "data_array", editType: "calc" }, y: { valType: "data_array", editType: "calc" }, z: { valType: "data_array", editType: "calc" }, editType: "calc" }, maxdisplayed: { valType: "integer", min: 0, dflt: 1e3, editType: "calc" }, sizeref: { valType: "number", editType: "calc", min: 0, dflt: 1 }, text: { valType: "string", dflt: "", editType: "calc" }, hovertext: { valType: "string", dflt: "", editType: "calc" }, hovertemplate: ee({ editType: "calc" }, { keys: ["tubex", "tubey", "tubez", "tubeu", "tubev", "tubew", "norm", "divergence"] }), uhoverformat: ne("u", 1), vhoverformat: ne("v", 1), whoverformat: ne("w", 1), xhoverformat: ne("x"), yhoverformat: ne("y"), zhoverformat: ne("z"), showlegend: se({}, oe.showlegend, { dflt: !1 }) };
            se(ce, te("", { colorAttr: "u/v/w norm", showScaleDflt: !0, editTypeOverride: "calc" })), ["opacity", "lightposition", "lighting"].forEach(function(le) {
              ce[le] = ae[le];
            }), ce.hoverinfo = se({}, oe.hoverinfo, { editType: "calc", flags: ["x", "y", "z", "u", "v", "w", "norm", "divergence", "text", "name"], dflt: "x+y+z+norm+text+name" }), ce.transforms = void 0, G.exports = ce;
          }, 36402: function(G, J, Z) {
            var te = Z(34809), ne = Z(28379);
            function ee(se) {
              var ce, le, ue, de, fe, he, me, pe, ye, ge, ve, be, Ce = se._x, Se = se._y, we = se._z, Ee = se._len, xe = -1 / 0, Le = 1 / 0, Qe = -1 / 0, ke = 1 / 0, Me = -1 / 0, Re = 1 / 0, De = "";
              for (Ee && (me = Ce[0], ye = Se[0], ve = we[0]), Ee > 1 && (pe = Ce[Ee - 1], ge = Se[Ee - 1], be = we[Ee - 1]), ce = 0; ce < Ee; ce++) xe = Math.max(xe, Ce[ce]), Le = Math.min(Le, Ce[ce]), Qe = Math.max(Qe, Se[ce]), ke = Math.min(ke, Se[ce]), Me = Math.max(Me, we[ce]), Re = Math.min(Re, we[ce]), de || Ce[ce] === me || (de = !0, De += "x"), fe || Se[ce] === ye || (fe = !0, De += "y"), he || we[ce] === ve || (he = !0, De += "z");
              de || (De += "x"), fe || (De += "y"), he || (De += "z");
              var Pe = ae(se._x), $e = ae(se._y), qe = ae(se._z);
              De = (De = (De = De.replace("x", (me > pe ? "-" : "+") + "x")).replace("y", (ye > ge ? "-" : "+") + "y")).replace("z", (ve > be ? "-" : "+") + "z");
              var Xe = function() {
                Ee = 0, Pe = [], $e = [], qe = [];
              };
              (!Ee || Ee < Pe.length * $e.length * qe.length) && Xe();
              var Ve = function(vt) {
                return vt === "x" ? Ce : vt === "y" ? Se : we;
              }, Ze = function(vt) {
                return vt === "x" ? Pe : vt === "y" ? $e : qe;
              }, Ge = function(vt) {
                return vt[Ee - 1] < vt[0] ? -1 : 1;
              }, ze = Ve(De[1]), We = Ve(De[3]), at = Ve(De[5]), ct = Ze(De[1]).length, et = Ze(De[3]).length, dt = Ze(De[5]).length, Je = !1, it = function(vt, $t, Pt) {
                return ct * (et * vt + $t) + Pt;
              }, st = Ge(Ve(De[1])), gt = Ge(Ve(De[3])), bt = Ge(Ve(De[5]));
              for (ce = 0; ce < dt - 1; ce++) {
                for (le = 0; le < et - 1; le++) {
                  for (ue = 0; ue < ct - 1; ue++) {
                    var yt = it(ce, le, ue), St = it(ce, le, ue + 1), Lt = it(ce, le + 1, ue), Rt = it(ce + 1, le, ue);
                    if (ze[yt] * st < ze[St] * st && We[yt] * gt < We[Lt] * gt && at[yt] * bt < at[Rt] * bt || (Je = !0), Je) break;
                  }
                  if (Je) break;
                }
                if (Je) break;
              }
              return Je && (te.warn("Encountered arbitrary coordinates! Unable to input data grid."), Xe()), { xMin: Le, yMin: ke, zMin: Re, xMax: xe, yMax: Qe, zMax: Me, Xs: Pe, Ys: $e, Zs: qe, len: Ee, fill: De };
            }
            function ae(se) {
              return te.distinctVals(se).vals;
            }
            function oe(se, ce) {
              if (ce === void 0 && (ce = se.length), te.isTypedArray(se)) return se.subarray(0, ce);
              for (var le = [], ue = 0; ue < ce; ue++) le[ue] = +se[ue];
              return le;
            }
            G.exports = { calc: function(se, ce) {
              ce._len = Math.min(ce.u.length, ce.v.length, ce.w.length, ce.x.length, ce.y.length, ce.z.length), ce._u = oe(ce.u, ce._len), ce._v = oe(ce.v, ce._len), ce._w = oe(ce.w, ce._len), ce._x = oe(ce.x, ce._len), ce._y = oe(ce.y, ce._len), ce._z = oe(ce.z, ce._len);
              var le = ee(ce);
              ce._gridFill = le.fill, ce._Xs = le.Xs, ce._Ys = le.Ys, ce._Zs = le.Zs, ce._len = le.len;
              var ue, de, fe, he = 0;
              ce.starts && (ue = oe(ce.starts.x || []), de = oe(ce.starts.y || []), fe = oe(ce.starts.z || []), he = Math.min(ue.length, de.length, fe.length)), ce._startsX = ue || [], ce._startsY = de || [], ce._startsZ = fe || [];
              var me, pe = 0, ye = 1 / 0;
              for (me = 0; me < ce._len; me++) {
                var ge = ce._u[me], ve = ce._v[me], be = ce._w[me], Ce = Math.sqrt(ge * ge + ve * ve + be * be);
                pe = Math.max(pe, Ce), ye = Math.min(ye, Ce);
              }
              for (ne(se, ce, { vals: [ye, pe], containerStr: "", cLetter: "c" }), me = 0; me < he; me++) {
                var Se = ue[me];
                le.xMax = Math.max(le.xMax, Se), le.xMin = Math.min(le.xMin, Se);
                var we = de[me];
                le.yMax = Math.max(le.yMax, we), le.yMin = Math.min(le.yMin, we);
                var Ee = fe[me];
                le.zMax = Math.max(le.zMax, Ee), le.zMin = Math.min(le.zMin, Ee);
              }
              ce._slen = he, ce._normMax = pe, ce._xbnds = [le.xMin, le.xMax], ce._ybnds = [le.yMin, le.yMax], ce._zbnds = [le.zMin, le.zMax];
            }, filter: oe, processGrid: ee };
          }, 49280: function(G, J, Z) {
            var te = Z(99098).gl_streamtube3d, ne = te.createTubeMesh, ee = Z(34809), ae = Z(46998).parseColorScale, oe = Z(88856).extractOpts, se = Z(88239), ce = { xaxis: 0, yaxis: 1, zaxis: 2 };
            function le(me, pe) {
              this.scene = me, this.uid = pe, this.mesh = null, this.data = null;
            }
            var ue = le.prototype;
            function de(me) {
              var pe = me.length;
              return pe > 2 ? me.slice(1, pe - 1) : pe === 2 ? [(me[0] + me[1]) / 2] : me;
            }
            function fe(me) {
              var pe = me.length;
              return pe === 1 ? [0.5, 0.5] : [me[1] - me[0], me[pe - 1] - me[pe - 2]];
            }
            function he(me, pe) {
              var ye = me.fullSceneLayout, ge = me.dataScale, ve = pe._len, be = {};
              function Ce(ct, et) {
                var dt = ye[et], Je = ge[ce[et]];
                return ee.simpleMap(ct, function(it) {
                  return dt.d2l(it) * Je;
                });
              }
              if (be.vectors = se(Ce(pe._u, "xaxis"), Ce(pe._v, "yaxis"), Ce(pe._w, "zaxis"), ve), !ve) return { positions: [], cells: [] };
              var Se = Ce(pe._Xs, "xaxis"), we = Ce(pe._Ys, "yaxis"), Ee = Ce(pe._Zs, "zaxis");
              if (be.meshgrid = [Se, we, Ee], be.gridFill = pe._gridFill, pe._slen) be.startingPositions = se(Ce(pe._startsX, "xaxis"), Ce(pe._startsY, "yaxis"), Ce(pe._startsZ, "zaxis"));
              else {
                for (var xe = we[0], Le = de(Se), Qe = de(Ee), ke = new Array(Le.length * Qe.length), Me = 0, Re = 0; Re < Le.length; Re++) for (var De = 0; De < Qe.length; De++) ke[Me++] = [Le[Re], xe, Qe[De]];
                be.startingPositions = ke;
              }
              be.colormap = ae(pe), be.tubeSize = pe.sizeref, be.maxLength = pe.maxdisplayed;
              var Pe = Ce(pe._xbnds, "xaxis"), $e = Ce(pe._ybnds, "yaxis"), qe = Ce(pe._zbnds, "zaxis"), Xe = fe(Se), Ve = fe(we), Ze = fe(Ee), Ge = [[Pe[0] - Xe[0], $e[0] - Ve[0], qe[0] - Ze[0]], [Pe[1] + Xe[1], $e[1] + Ve[1], qe[1] + Ze[1]]], ze = te(be, Ge), We = oe(pe);
              ze.vertexIntensityBounds = [We.min / pe._normMax, We.max / pe._normMax];
              var at = pe.lightposition;
              return ze.lightPosition = [at.x, at.y, at.z], ze.ambient = pe.lighting.ambient, ze.diffuse = pe.lighting.diffuse, ze.specular = pe.lighting.specular, ze.roughness = pe.lighting.roughness, ze.fresnel = pe.lighting.fresnel, ze.opacity = pe.opacity, pe._pad = ze.tubeScale * pe.sizeref * 2, ze;
            }
            ue.handlePick = function(me) {
              var pe = this.scene.fullSceneLayout, ye = this.scene.dataScale;
              function ge(Ce, Se) {
                var we = pe[Se], Ee = ye[ce[Se]];
                return we.l2c(Ce) / Ee;
              }
              if (me.object === this.mesh) {
                var ve = me.data.position, be = me.data.velocity;
                return me.traceCoordinate = [ge(ve[0], "xaxis"), ge(ve[1], "yaxis"), ge(ve[2], "zaxis"), ge(be[0], "xaxis"), ge(be[1], "yaxis"), ge(be[2], "zaxis"), me.data.intensity * this.data._normMax, me.data.divergence], me.textLabel = this.data.hovertext || this.data.text, !0;
              }
            }, ue.update = function(me) {
              this.data = me;
              var pe = he(this.scene, me);
              this.mesh.update(pe);
            }, ue.dispose = function() {
              this.scene.glplot.remove(this.mesh), this.mesh.dispose();
            }, G.exports = function(me, pe) {
              var ye = me.glplot.gl, ge = he(me, pe), ve = ne(ye, ge), be = new le(me, pe.uid);
              return be.mesh = ve, be.data = pe, ve._trace = be, me.glplot.add(ve), be;
            };
          }, 52737: function(G, J, Z) {
            var te = Z(34809), ne = Z(39356), ee = Z(14774);
            G.exports = function(ae, oe, se, ce) {
              function le(ye, ge) {
                return te.coerce(ae, oe, ee, ye, ge);
              }
              var ue = le("u"), de = le("v"), fe = le("w"), he = le("x"), me = le("y"), pe = le("z");
              ue && ue.length && de && de.length && fe && fe.length && he && he.length && me && me.length && pe && pe.length ? (le("starts.x"), le("starts.y"), le("starts.z"), le("maxdisplayed"), le("sizeref"), le("lighting.ambient"), le("lighting.diffuse"), le("lighting.specular"), le("lighting.roughness"), le("lighting.fresnel"), le("lightposition.x"), le("lightposition.y"), le("lightposition.z"), ne(ae, oe, ce, le, { prefix: "", cLetter: "c" }), le("text"), le("hovertext"), le("hovertemplate"), le("uhoverformat"), le("vhoverformat"), le("whoverformat"), le("xhoverformat"), le("yhoverformat"), le("zhoverformat"), oe._length = null) : oe.visible = !1;
            };
          }, 51943: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "streamtube", basePlotModule: Z(2487), categories: ["gl3d", "showLegend"], attributes: Z(14774), supplyDefaults: Z(52737), colorbar: { min: "cmin", max: "cmax" }, calc: Z(36402).calc, plot: Z(49280), eventData: function(te, ne) {
              return te.tubex = te.x, te.tubey = te.y, te.tubez = te.z, te.tubeu = ne.traceCoordinate[3], te.tubev = ne.traceCoordinate[4], te.tubew = ne.traceCoordinate[5], te.norm = ne.traceCoordinate[6], te.divergence = ne.traceCoordinate[7], delete te.x, delete te.y, delete te.z, te;
            }, meta: {} };
          }, 56708: function(G, J, Z) {
            var te = Z(9829), ne = Z(3208).rb, ee = Z(3208).ay, ae = Z(87163), oe = Z(13792).u, se = Z(55412), ce = Z(2032), le = Z(93049).extendFlat, ue = Z(94850).k;
            G.exports = { labels: { valType: "data_array", editType: "calc" }, parents: { valType: "data_array", editType: "calc" }, values: { valType: "data_array", editType: "calc" }, branchvalues: { valType: "enumerated", values: ["remainder", "total"], dflt: "remainder", editType: "calc" }, count: { valType: "flaglist", flags: ["branches", "leaves"], dflt: "leaves", editType: "calc" }, level: { valType: "any", editType: "plot", anim: !0 }, maxdepth: { valType: "integer", editType: "plot", dflt: -1 }, marker: le({ colors: { valType: "data_array", editType: "calc" }, line: { color: le({}, se.marker.line.color, { dflt: null }), width: le({}, se.marker.line.width, { dflt: 1 }), editType: "calc" }, pattern: ue, editType: "calc" }, ae("marker", { colorAttr: "colors", anim: !1 })), leaf: { opacity: { valType: "number", editType: "style", min: 0, max: 1 }, editType: "plot" }, text: se.text, textinfo: { valType: "flaglist", flags: ["label", "text", "value", "current path", "percent root", "percent entry", "percent parent"], extras: ["none"], editType: "plot" }, texttemplate: ee({ editType: "plot" }, { keys: ce.eventDataKeys.concat(["label", "value"]) }), hovertext: se.hovertext, hoverinfo: le({}, te.hoverinfo, { flags: ["label", "text", "value", "name", "current path", "percent root", "percent entry", "percent parent"], dflt: "label+text+value+name" }), hovertemplate: ne({}, { keys: ce.eventDataKeys }), textfont: se.textfont, insidetextorientation: se.insidetextorientation, insidetextfont: se.insidetextfont, outsidetextfont: le({}, se.outsidetextfont, {}), rotation: { valType: "angle", dflt: 0, editType: "plot" }, sort: se.sort, root: { color: { valType: "color", editType: "calc", dflt: "rgba(0,0,0,0)" }, editType: "calc" }, domain: oe({ name: "sunburst", trace: !0, editType: "calc" }) };
          }, 14724: function(G, J, Z) {
            var te = Z(44122);
            J.name = "sunburst", J.plot = function(ne, ee, ae, oe) {
              te.plotBasePlot(J.name, ne, ee, ae, oe);
            }, J.clean = function(ne, ee, ae, oe) {
              te.cleanBasePlot(J.name, ne, ee, ae, oe);
            };
          }, 14852: function(G, J, Z) {
            var te = Z(92264), ne = Z(10721), ee = Z(34809), ae = Z(88856).makeColorScaleFuncFromTrace, oe = Z(44148).makePullColorFn, se = Z(44148).generateExtendedColors, ce = Z(88856).calc, le = Z(63821).ALMOST_EQUAL, ue = {}, de = {}, fe = {};
            function he(me, pe, ye) {
              var ge = 0, ve = me.children;
              if (ve) {
                for (var be = ve.length, Ce = 0; Ce < be; Ce++) ge += he(ve[Ce], pe, ye);
                ye.branches && ge++;
              } else ye.leaves && ge++;
              return me.value = me.data.data.value = ge, pe._values || (pe._values = []), pe._values[me.data.data.i] = ge, ge;
            }
            J.calc = function(me, pe) {
              var ye, ge, ve, be, Ce, Se, we = me._fullLayout, Ee = pe.ids, xe = ee.isArrayOrTypedArray(Ee), Le = pe.labels, Qe = pe.parents, ke = pe.values, Me = ee.isArrayOrTypedArray(ke), Re = [], De = {}, Pe = {}, $e = function(bt) {
                return bt || typeof bt == "number";
              }, qe = function(bt) {
                return !Me || ne(ke[bt]) && ke[bt] >= 0;
              };
              xe ? (ye = Math.min(Ee.length, Qe.length), ge = function(bt) {
                return $e(Ee[bt]) && qe(bt);
              }, ve = function(bt) {
                return String(Ee[bt]);
              }) : (ye = Math.min(Le.length, Qe.length), ge = function(bt) {
                return $e(Le[bt]) && qe(bt);
              }, ve = function(bt) {
                return String(Le[bt]);
              }), Me && (ye = Math.min(ye, ke.length));
              for (var Xe = 0; Xe < ye; Xe++) if (ge(Xe)) {
                var Ve = ve(Xe), Ze = $e(Qe[Xe]) ? String(Qe[Xe]) : "", Ge = { i: Xe, id: Ve, pid: Ze, label: $e(Le[Xe]) ? String(Le[Xe]) : "" };
                Me && (Ge.v = +ke[Xe]), Re.push(Ge), Ce = Ve, De[be = Ze] ? De[be].push(Ce) : De[be] = [Ce], Pe[Ce] = 1;
              }
              if (De[""]) {
                if (De[""].length > 1) {
                  for (var ze = ee.randstr(), We = 0; We < Re.length; We++) Re[We].pid === "" && (Re[We].pid = ze);
                  Re.unshift({ hasMultipleRoots: !0, id: ze, pid: "", label: "" });
                }
              } else {
                var at, ct = [];
                for (at in De) Pe[at] || ct.push(at);
                if (ct.length !== 1) return ee.warn(["Multiple implied roots, cannot build", pe.type, "hierarchy of", pe.name + ".", "These roots include:", ct.join(", ")].join(" "));
                at = ct[0], Re.unshift({ hasImpliedRoot: !0, id: at, pid: "", label: at });
              }
              try {
                Se = te.stratify().id(function(bt) {
                  return bt.id;
                }).parentId(function(bt) {
                  return bt.pid;
                })(Re);
              } catch (bt) {
                return ee.warn(["Failed to build", pe.type, "hierarchy of", pe.name + ".", "Error:", bt.message].join(" "));
              }
              var et = te.hierarchy(Se), dt = !1;
              if (Me) switch (pe.branchvalues) {
                case "remainder":
                  et.sum(function(bt) {
                    return bt.data.v;
                  });
                  break;
                case "total":
                  et.each(function(bt) {
                    var yt = bt.data.data, St = yt.v;
                    if (bt.children) {
                      var Lt = bt.children.reduce(function(Rt, vt) {
                        return Rt + vt.data.data.v;
                      }, 0);
                      if ((yt.hasImpliedRoot || yt.hasMultipleRoots) && (St = Lt), St < Lt * le) return dt = !0, ee.warn(["Total value for node", bt.data.data.id, "of", pe.name, "is smaller than the sum of its children.", `
parent value =`, St, `
children sum =`, Lt].join(" "));
                    }
                    bt.value = St;
                  });
              }
              else he(et, pe, { branches: pe.count.indexOf("branches") !== -1, leaves: pe.count.indexOf("leaves") !== -1 });
              if (!dt) {
                var Je, it;
                pe.sort && et.sort(function(bt, yt) {
                  return yt.value - bt.value;
                });
                var st = pe.marker.colors || [], gt = !!st.length;
                return pe._hasColorscale ? (gt || (st = Me ? pe.values : pe._values), ce(me, pe, { vals: st, containerStr: "marker", cLetter: "c" }), it = ae(pe.marker)) : Je = oe(we["_" + pe.type + "colormap"]), et.each(function(bt) {
                  var yt = bt.data.data;
                  yt.color = pe._hasColorscale ? it(st[yt.i]) : Je(st[yt.i], yt.id);
                }), Re[0].hierarchy = et, Re;
              }
            }, J._runCrossTraceCalc = function(me, pe) {
              var ye = pe._fullLayout, ge = pe.calcdata, ve = ye[me + "colorway"], be = ye["_" + me + "colormap"];
              ye["extend" + me + "colors"] && (ve = se(ve, me === "icicle" ? fe : me === "treemap" ? de : ue));
              var Ce, Se = 0;
              function we(Le) {
                var Qe = Le.data.data, ke = Qe.id;
                Qe.color === !1 && (be[ke] ? Qe.color = be[ke] : Le.parent ? Le.parent.parent ? Qe.color = Le.parent.data.data.color : (be[ke] = Qe.color = ve[Se % ve.length], Se++) : Qe.color = Ce);
              }
              for (var Ee = 0; Ee < ge.length; Ee++) {
                var xe = ge[Ee][0];
                xe.trace.type === me && xe.hierarchy && (Ce = xe.trace.root.color, xe.hierarchy.each(we));
              }
            }, J.crossTraceCalc = function(me) {
              return J._runCrossTraceCalc("sunburst", me);
            };
          }, 2032: function(G) {
            G.exports = { CLICK_TRANSITION_TIME: 750, CLICK_TRANSITION_EASING: "linear", eventDataKeys: ["currentPath", "root", "entry", "percentRoot", "percentEntry", "percentParent"] };
          }, 33459: function(G, J, Z) {
            var te = Z(34809), ne = Z(56708), ee = Z(13792).N, ae = Z(17550).handleText, oe = Z(46979).handleMarkerDefaults, se = Z(88856), ce = se.hasColorscale, le = se.handleDefaults;
            G.exports = function(ue, de, fe, he) {
              function me(Ce, Se) {
                return te.coerce(ue, de, ne, Ce, Se);
              }
              var pe = me("labels"), ye = me("parents");
              if (pe && pe.length && ye && ye.length) {
                var ge = me("values");
                ge && ge.length ? me("branchvalues") : me("count"), me("level"), me("maxdepth"), oe(ue, de, he, me);
                var ve = de._hasColorscale = ce(ue, "marker", "colors") || (ue.marker || {}).coloraxis;
                ve && le(ue, de, he, me, { prefix: "marker.", cLetter: "c" }), me("leaf.opacity", ve ? 1 : 0.7);
                var be = me("text");
                me("texttemplate"), de.texttemplate || me("textinfo", te.isArrayOrTypedArray(be) ? "text+label" : "label"), me("hovertext"), me("hovertemplate"), ae(ue, de, he, me, "auto", { moduleHasSelected: !1, moduleHasUnselected: !1, moduleHasConstrain: !1, moduleHasCliponaxis: !1, moduleHasTextangle: !1, moduleHasInsideanchor: !1 }), me("insidetextorientation"), me("sort"), me("rotation"), me("root.color"), ee(de, he, me), de._length = null;
              } else de.visible = !1;
            };
          }, 72043: function(G, J, Z) {
            var te = Z(62203), ne = Z(78766);
            G.exports = function(ee, ae, oe, se, ce) {
              var le = ae.data.data, ue = le.i, de = ce || le.color;
              if (ue >= 0) {
                ae.i = le.i;
                var fe = oe.marker;
                fe.pattern && fe.colors && fe.pattern.shape || (fe.color = de, ae.color = de), te.pointStyle(ee, oe, se, ae);
              } else ne.fill(ee, de);
            };
          }, 44691: function(G, J, Z) {
            var te = Z(45568), ne = Z(33626), ee = Z(36040).appendArrayPointValue, ae = Z(32141), oe = Z(34809), se = Z(68596), ce = Z(33108), le = Z(37252).formatPieValue;
            function ue(de, fe, he) {
              for (var me = de.data.data, pe = { curveNumber: fe.index, pointNumber: me.i, data: fe._input, fullData: fe }, ye = 0; ye < he.length; ye++) {
                var ge = he[ye];
                ge in de && (pe[ge] = de[ge]);
              }
              return "parentString" in de && !ce.isHierarchyRoot(de) && (pe.parent = de.parentString), ee(pe, fe, me.i), pe;
            }
            G.exports = function(de, fe, he, me, pe) {
              var ye = me[0], ge = ye.trace, ve = ye.hierarchy, be = ge.type === "sunburst", Ce = ge.type === "treemap" || ge.type === "icicle";
              "_hasHoverLabel" in ge || (ge._hasHoverLabel = !1), "_hasHoverEvent" in ge || (ge._hasHoverEvent = !1), de.on("mouseover", function(Se) {
                var we = he._fullLayout;
                if (!he._dragging && we.hovermode !== !1) {
                  var Ee, xe = he._fullData[ge.index], Le = Se.data.data, Qe = Le.i, ke = ce.isHierarchyRoot(Se), Me = ce.getParent(ve, Se), Re = ce.getValue(Se), De = function(st) {
                    return oe.castOption(xe, Qe, st);
                  }, Pe = De("hovertemplate"), $e = ae.castHoverinfo(xe, we, Qe), qe = we.separators;
                  if (Pe || $e && $e !== "none" && $e !== "skip") {
                    var Xe, Ve;
                    be && (Xe = ye.cx + Se.pxmid[0] * (1 - Se.rInscribed), Ve = ye.cy + Se.pxmid[1] * (1 - Se.rInscribed)), Ce && (Xe = Se._hoverX, Ve = Se._hoverY);
                    var Ze, Ge = {}, ze = [], We = [], at = function(st) {
                      return ze.indexOf(st) !== -1;
                    };
                    $e && (ze = $e === "all" ? xe._module.attributes.hoverinfo.flags : $e.split("+")), Ge.label = Le.label, at("label") && Ge.label && We.push(Ge.label), Le.hasOwnProperty("v") && (Ge.value = Le.v, Ge.valueLabel = le(Ge.value, qe), at("value") && We.push(Ge.valueLabel)), Ge.currentPath = Se.currentPath = ce.getPath(Se.data), at("current path") && !ke && We.push(Ge.currentPath);
                    var ct = [], et = function() {
                      ct.indexOf(Ze) === -1 && (We.push(Ze), ct.push(Ze));
                    };
                    Ge.percentParent = Se.percentParent = Re / ce.getValue(Me), Ge.parent = Se.parentString = ce.getPtLabel(Me), at("percent parent") && (Ze = ce.formatPercent(Ge.percentParent, qe) + " of " + Ge.parent, et()), Ge.percentEntry = Se.percentEntry = Re / ce.getValue(fe), Ge.entry = Se.entry = ce.getPtLabel(fe), !at("percent entry") || ke || Se.onPathbar || (Ze = ce.formatPercent(Ge.percentEntry, qe) + " of " + Ge.entry, et()), Ge.percentRoot = Se.percentRoot = Re / ce.getValue(ve), Ge.root = Se.root = ce.getPtLabel(ve), at("percent root") && !ke && (Ze = ce.formatPercent(Ge.percentRoot, qe) + " of " + Ge.root, et()), Ge.text = De("hovertext") || De("text"), at("text") && (Ze = Ge.text, oe.isValidTextValue(Ze) && We.push(Ze)), Ee = [ue(Se, xe, pe.eventDataKeys)];
                    var dt = { trace: xe, y: Ve, _x0: Se._x0, _x1: Se._x1, _y0: Se._y0, _y1: Se._y1, text: We.join("<br>"), name: Pe || at("name") ? xe.name : void 0, color: De("hoverlabel.bgcolor") || Le.color, borderColor: De("hoverlabel.bordercolor"), fontFamily: De("hoverlabel.font.family"), fontSize: De("hoverlabel.font.size"), fontColor: De("hoverlabel.font.color"), fontWeight: De("hoverlabel.font.weight"), fontStyle: De("hoverlabel.font.style"), fontVariant: De("hoverlabel.font.variant"), nameLength: De("hoverlabel.namelength"), textAlign: De("hoverlabel.align"), hovertemplate: Pe, hovertemplateLabels: Ge, eventData: Ee };
                    be && (dt.x0 = Xe - Se.rInscribed * Se.rpx1, dt.x1 = Xe + Se.rInscribed * Se.rpx1, dt.idealAlign = Se.pxmid[0] < 0 ? "left" : "right"), Ce && (dt.x = Xe, dt.idealAlign = Xe < 0 ? "left" : "right");
                    var Je = [];
                    ae.loneHover(dt, { container: we._hoverlayer.node(), outerContainer: we._paper.node(), gd: he, inOut_bbox: Je }), Ee[0].bbox = Je[0], ge._hasHoverLabel = !0;
                  }
                  if (Ce) {
                    var it = de.select("path.surface");
                    pe.styleOne(it, Se, xe, he, { hovered: !0 });
                  }
                  ge._hasHoverEvent = !0, he.emit("plotly_hover", { points: Ee || [ue(Se, xe, pe.eventDataKeys)], event: te.event });
                }
              }), de.on("mouseout", function(Se) {
                var we = he._fullLayout, Ee = he._fullData[ge.index], xe = te.select(this).datum();
                if (ge._hasHoverEvent && (Se.originalEvent = te.event, he.emit("plotly_unhover", { points: [ue(xe, Ee, pe.eventDataKeys)], event: te.event }), ge._hasHoverEvent = !1), ge._hasHoverLabel && (ae.loneUnhover(we._hoverlayer.node()), ge._hasHoverLabel = !1), Ce) {
                  var Le = de.select("path.surface");
                  pe.styleOne(Le, xe, Ee, he, { hovered: !1 });
                }
              }), de.on("click", function(Se) {
                var we = he._fullLayout, Ee = he._fullData[ge.index], xe = be && (ce.isHierarchyRoot(Se) || ce.isLeaf(Se)), Le = ce.getPtId(Se), Qe = ce.isEntry(Se) ? ce.findEntryWithChild(ve, Le) : ce.findEntryWithLevel(ve, Le), ke = ce.getPtId(Qe), Me = { points: [ue(Se, Ee, pe.eventDataKeys)], event: te.event };
                xe || (Me.nextLevel = ke);
                var Re = se.triggerHandler(he, "plotly_" + ge.type + "click", Me);
                if (Re !== !1 && we.hovermode && (he._hoverdata = [ue(Se, Ee, pe.eventDataKeys)], ae.click(he, te.event)), !xe && Re !== !1 && !he._dragging && !he._transitioning) {
                  ne.call("_storeDirectGUIEdit", Ee, we._tracePreGUI[Ee.uid], { level: Ee.level });
                  var De = { data: [{ level: ke }], traces: [ge.index] }, Pe = { frame: { redraw: !1, duration: pe.transitionTime }, transition: { duration: pe.transitionTime, easing: pe.transitionEasing }, mode: "immediate", fromcurrent: !0 };
                  ae.loneUnhover(we._hoverlayer.node()), ne.call("animate", he, De, Pe);
                }
              });
            };
          }, 33108: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(27983), ae = Z(37252);
            function oe(se) {
              return se.data.data.pid;
            }
            J.findEntryWithLevel = function(se, ce) {
              var le;
              return ce && se.eachAfter(function(ue) {
                if (J.getPtId(ue) === ce) return le = ue.copy();
              }), le || se;
            }, J.findEntryWithChild = function(se, ce) {
              var le;
              return se.eachAfter(function(ue) {
                for (var de = ue.children || [], fe = 0; fe < de.length; fe++) {
                  var he = de[fe];
                  if (J.getPtId(he) === ce) return le = ue.copy();
                }
              }), le || se;
            }, J.isEntry = function(se) {
              return !se.parent;
            }, J.isLeaf = function(se) {
              return !se.children;
            }, J.getPtId = function(se) {
              return se.data.data.id;
            }, J.getPtLabel = function(se) {
              return se.data.data.label;
            }, J.getValue = function(se) {
              return se.value;
            }, J.isHierarchyRoot = function(se) {
              return oe(se) === "";
            }, J.setSliceCursor = function(se, ce, le) {
              var ue = le.isTransitioning;
              if (!ue) {
                var de = se.datum();
                ue = le.hideOnRoot && J.isHierarchyRoot(de) || le.hideOnLeaves && J.isLeaf(de);
              }
              ee(se, ue ? null : "pointer");
            }, J.getInsideTextFontKey = function(se, ce, le, ue, de) {
              var fe = (de || {}).onPathbar ? "pathbar.textfont" : "insidetextfont", he = le.data.data.i;
              return te.castOption(ce, he, fe + "." + se) || te.castOption(ce, he, "textfont." + se) || ue.size;
            }, J.getOutsideTextFontKey = function(se, ce, le, ue) {
              var de = le.data.data.i;
              return te.castOption(ce, de, "outsidetextfont." + se) || te.castOption(ce, de, "textfont." + se) || ue.size;
            }, J.isOutsideText = function(se, ce) {
              return !se._hasColorscale && J.isHierarchyRoot(ce);
            }, J.determineTextFont = function(se, ce, le, ue) {
              return J.isOutsideText(se, ce) ? function(de, fe, he) {
                return { color: J.getOutsideTextFontKey("color", de, fe, he), family: J.getOutsideTextFontKey("family", de, fe, he), size: J.getOutsideTextFontKey("size", de, fe, he), weight: J.getOutsideTextFontKey("weight", de, fe, he), style: J.getOutsideTextFontKey("style", de, fe, he), variant: J.getOutsideTextFontKey("variant", de, fe, he), textcase: J.getOutsideTextFontKey("textcase", de, fe, he), lineposition: J.getOutsideTextFontKey("lineposition", de, fe, he), shadow: J.getOutsideTextFontKey("shadow", de, fe, he) };
              }(se, ce, le) : function(de, fe, he, me) {
                var pe = (me || {}).onPathbar, ye = fe.data.data, ge = ye.i, ve = te.castOption(de, ge, (pe ? "pathbar.textfont" : "insidetextfont") + ".color");
                return !ve && de._input.textfont && (ve = te.castOption(de._input, ge, "textfont.color")), { color: ve || ne.contrast(ye.color), family: J.getInsideTextFontKey("family", de, fe, he, me), size: J.getInsideTextFontKey("size", de, fe, he, me), weight: J.getInsideTextFontKey("weight", de, fe, he, me), style: J.getInsideTextFontKey("style", de, fe, he, me), variant: J.getInsideTextFontKey("variant", de, fe, he, me), textcase: J.getInsideTextFontKey("textcase", de, fe, he, me), lineposition: J.getInsideTextFontKey("lineposition", de, fe, he, me), shadow: J.getInsideTextFontKey("shadow", de, fe, he, me) };
              }(se, ce, le, ue);
            }, J.hasTransition = function(se) {
              return !!(se && se.duration > 0);
            }, J.getMaxDepth = function(se) {
              return se.maxdepth >= 0 ? se.maxdepth : 1 / 0;
            }, J.isHeader = function(se, ce) {
              return !(J.isLeaf(se) || se.depth === ce._maxDepth - 1);
            }, J.getParent = function(se, ce) {
              return J.findEntryWithLevel(se, oe(ce));
            }, J.listPath = function(se, ce) {
              var le = se.parent;
              if (!le) return [];
              var ue = ce ? [le.data[ce]] : [le];
              return J.listPath(le, ce).concat(ue);
            }, J.getPath = function(se) {
              return J.listPath(se, "label").join("/") + "/";
            }, J.formatValue = ae.formatPieValue, J.formatPercent = function(se, ce) {
              var le = te.formatPercent(se, 0);
              return le === "0%" && (le = ae.formatPiePercent(se, ce)), le;
            };
          }, 80809: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "sunburst", basePlotModule: Z(14724), categories: [], animatable: !0, attributes: Z(56708), layoutAttributes: Z(98959), supplyDefaults: Z(33459), supplyLayoutDefaults: Z(75816), calc: Z(14852).calc, crossTraceCalc: Z(14852).crossTraceCalc, plot: Z(19718).plot, style: Z(98972).style, colorbar: Z(21146), meta: {} };
          }, 98959: function(G) {
            G.exports = { sunburstcolorway: { valType: "colorlist", editType: "calc" }, extendsunburstcolors: { valType: "boolean", dflt: !0, editType: "calc" } };
          }, 75816: function(G, J, Z) {
            var te = Z(34809), ne = Z(98959);
            G.exports = function(ee, ae) {
              function oe(se, ce) {
                return te.coerce(ee, ae, ne, se, ce);
              }
              oe("sunburstcolorway", ae.colorway), oe("extendsunburstcolors");
            };
          }, 19718: function(G, J, Z) {
            var te = Z(45568), ne = Z(92264), ee = Z(88640).GW, ae = Z(62203), oe = Z(34809), se = Z(30635), ce = Z(84102), le = ce.recordMinTextSize, ue = ce.clearMinTextSize, de = Z(35734), fe = Z(37252).getRotationAngle, he = de.computeTransform, me = de.transformInsideText, pe = Z(98972).styleOne, ye = Z(6851).resizeText, ge = Z(44691), ve = Z(2032), be = Z(33108);
            function Ce(we, Ee, xe, Le) {
              var Qe = we._context.staticPlot, ke = we._fullLayout, Me = !ke.uniformtext.mode && be.hasTransition(Le), Re = te.select(xe).selectAll("g.slice"), De = Ee[0], Pe = De.trace, $e = De.hierarchy, qe = be.findEntryWithLevel($e, Pe.level), Xe = be.getMaxDepth(Pe), Ve = ke._size, Ze = Pe.domain, Ge = Ve.w * (Ze.x[1] - Ze.x[0]), ze = Ve.h * (Ze.y[1] - Ze.y[0]), We = 0.5 * Math.min(Ge, ze), at = De.cx = Ve.l + Ve.w * (Ze.x[1] + Ze.x[0]) / 2, ct = De.cy = Ve.t + Ve.h * (1 - Ze.y[0]) - ze / 2;
              if (!qe) return Re.remove();
              var et = null, dt = {};
              Me && Re.each(function(Ft) {
                dt[be.getPtId(Ft)] = { rpx0: Ft.rpx0, rpx1: Ft.rpx1, x0: Ft.x0, x1: Ft.x1, transform: Ft.transform }, !et && be.isEntry(Ft) && (et = Ft);
              });
              var Je = function(Ft) {
                return ne.partition().size([2 * Math.PI, Ft.height + 1])(Ft);
              }(qe).descendants(), it = qe.height + 1, st = 0, gt = Xe;
              De.hasMultipleRoots && be.isHierarchyRoot(qe) && (Je = Je.slice(1), it -= 1, st = 1, gt += 1), Je = Je.filter(function(Ft) {
                return Ft.y1 <= gt;
              });
              var bt = fe(Pe.rotation);
              bt && Je.forEach(function(Ft) {
                Ft.x0 += bt, Ft.x1 += bt;
              });
              var yt = Math.min(it, Xe), St = function(Ft) {
                return (Ft - st) / yt * We;
              }, Lt = function(Ft, Tt) {
                return [Ft * Math.cos(Tt), -Ft * Math.sin(Tt)];
              }, Rt = function(Ft) {
                return oe.pathAnnulus(Ft.rpx0, Ft.rpx1, Ft.x0, Ft.x1, at, ct);
              }, vt = function(Ft) {
                return at + Se(Ft)[0] * (Ft.transform.rCenter || 0) + (Ft.transform.x || 0);
              }, $t = function(Ft) {
                return ct + Se(Ft)[1] * (Ft.transform.rCenter || 0) + (Ft.transform.y || 0);
              };
              (Re = Re.data(Je, be.getPtId)).enter().append("g").classed("slice", !0), Me ? Re.exit().transition().each(function() {
                var Ft = te.select(this);
                Ft.select("path.surface").transition().attrTween("d", function(Tt) {
                  var It = function(Ct) {
                    var Kt, Vt = be.getPtId(Ct), Yt = dt[Vt], sn = dt[be.getPtId(qe)];
                    if (sn) {
                      var hn = (Ct.x1 > sn.x1 ? 2 * Math.PI : 0) + bt;
                      Kt = Ct.rpx1 < sn.rpx1 ? { x0: Ct.x0, x1: Ct.x1, rpx0: 0, rpx1: 0 } : { x0: hn, x1: hn, rpx0: Ct.rpx0, rpx1: Ct.rpx1 };
                    } else {
                      var Qt, Ut = be.getPtId(Ct.parent);
                      Re.each(function(Ht) {
                        if (be.getPtId(Ht) === Ut) return Qt = Ht;
                      });
                      var Wt, rn = Qt.children;
                      rn.forEach(function(Ht, jt) {
                        if (be.getPtId(Ht) === Vt) return Wt = jt;
                      });
                      var vn = rn.length, dn = ee(Qt.x0, Qt.x1);
                      Kt = { rpx0: We, rpx1: We, x0: dn(Wt / vn), x1: dn((Wt + 1) / vn) };
                    }
                    return ee(Yt, Kt);
                  }(Tt);
                  return function(Ct) {
                    return Rt(It(Ct));
                  };
                }), Ft.select("g.slicetext").attr("opacity", 0);
              }).remove() : Re.exit().remove(), Re.order();
              var Pt = null;
              if (Me && et) {
                var Xt = be.getPtId(et);
                Re.each(function(Ft) {
                  Pt === null && be.getPtId(Ft) === Xt && (Pt = Ft.x1);
                });
              }
              var Nt = Re;
              function Gt(Ft) {
                var Tt = Ft.parent, It = dt[be.getPtId(Tt)], Ct = {};
                if (It) {
                  var Kt = Tt.children, Vt = Kt.indexOf(Ft), Yt = Kt.length, sn = ee(It.x0, It.x1);
                  Ct.x0 = sn(Vt / Yt), Ct.x1 = sn(Vt / Yt);
                } else Ct.x0 = Ct.x1 = 0;
                return Ct;
              }
              Me && (Nt = Nt.transition().each("end", function() {
                var Ft = te.select(this);
                be.setSliceCursor(Ft, we, { hideOnRoot: !0, hideOnLeaves: !0, isTransitioning: !1 });
              })), Nt.each(function(Ft) {
                var Tt = te.select(this), It = oe.ensureSingle(Tt, "path", "surface", function(hn) {
                  hn.style("pointer-events", Qe ? "none" : "all");
                });
                Ft.rpx0 = St(Ft.y0), Ft.rpx1 = St(Ft.y1), Ft.xmid = (Ft.x0 + Ft.x1) / 2, Ft.pxmid = Lt(Ft.rpx1, Ft.xmid), Ft.midangle = -(Ft.xmid - Math.PI / 2), Ft.startangle = -(Ft.x0 - Math.PI / 2), Ft.stopangle = -(Ft.x1 - Math.PI / 2), Ft.halfangle = 0.5 * Math.min(oe.angleDelta(Ft.x0, Ft.x1) || Math.PI, Math.PI), Ft.ring = 1 - Ft.rpx0 / Ft.rpx1, Ft.rInscribed = function(hn) {
                  return hn.rpx0 === 0 && oe.isFullCircle([hn.x0, hn.x1]) ? 1 : Math.max(0, Math.min(1 / (1 + 1 / Math.sin(hn.halfangle)), hn.ring / 2));
                }(Ft), Me ? It.transition().attrTween("d", function(hn) {
                  var Qt = function(Ut) {
                    var Wt, rn = dt[be.getPtId(Ut)], vn = { x0: Ut.x0, x1: Ut.x1, rpx0: Ut.rpx0, rpx1: Ut.rpx1 };
                    if (rn) Wt = rn;
                    else if (et) if (Ut.parent) if (Pt) {
                      var dn = (Ut.x1 > Pt ? 2 * Math.PI : 0) + bt;
                      Wt = { x0: dn, x1: dn };
                    } else Wt = { rpx0: We, rpx1: We }, oe.extendFlat(Wt, Gt(Ut));
                    else Wt = { rpx0: 0, rpx1: 0 };
                    else Wt = { x0: bt, x1: bt };
                    return ee(Wt, vn);
                  }(hn);
                  return function(Ut) {
                    return Rt(Qt(Ut));
                  };
                }) : It.attr("d", Rt), Tt.call(ge, qe, we, Ee, { eventDataKeys: ve.eventDataKeys, transitionTime: ve.CLICK_TRANSITION_TIME, transitionEasing: ve.CLICK_TRANSITION_EASING }).call(be.setSliceCursor, we, { hideOnRoot: !0, hideOnLeaves: !0, isTransitioning: we._transitioning }), It.call(pe, Ft, Pe, we);
                var Ct = oe.ensureSingle(Tt, "g", "slicetext"), Kt = oe.ensureSingle(Ct, "text", "", function(hn) {
                  hn.attr("data-notex", 1);
                }), Vt = oe.ensureUniformFontSize(we, be.determineTextFont(Pe, Ft, ke.font));
                Kt.text(J.formatSliceLabel(Ft, qe, Pe, Ee, ke)).classed("slicetext", !0).attr("text-anchor", "middle").call(ae.font, Vt).call(se.convertToTspans, we);
                var Yt = ae.bBox(Kt.node());
                Ft.transform = me(Yt, Ft, De), Ft.transform.targetX = vt(Ft), Ft.transform.targetY = $t(Ft);
                var sn = function(hn, Qt) {
                  var Ut = hn.transform;
                  return he(Ut, Qt), Ut.fontSize = Vt.size, le(Pe.type, Ut, ke), oe.getTextTransform(Ut);
                };
                Me ? Kt.transition().attrTween("transform", function(hn) {
                  var Qt = function(Ut) {
                    var Wt, rn = dt[be.getPtId(Ut)], vn = Ut.transform;
                    if (rn) Wt = rn;
                    else if (Wt = { rpx1: Ut.rpx1, transform: { textPosAngle: vn.textPosAngle, scale: 0, rotate: vn.rotate, rCenter: vn.rCenter, x: vn.x, y: vn.y } }, et) if (Ut.parent) if (Pt) {
                      var dn = Ut.x1 > Pt ? 2 * Math.PI : 0;
                      Wt.x0 = Wt.x1 = dn;
                    } else oe.extendFlat(Wt, Gt(Ut));
                    else Wt.x0 = Wt.x1 = bt;
                    else Wt.x0 = Wt.x1 = bt;
                    var Ht = ee(Wt.transform.textPosAngle, Ut.transform.textPosAngle), jt = ee(Wt.rpx1, Ut.rpx1), cn = ee(Wt.x0, Ut.x0), wn = ee(Wt.x1, Ut.x1), Rn = ee(Wt.transform.scale, vn.scale), Pn = ee(Wt.transform.rotate, vn.rotate), Ie = vn.rCenter === 0 ? 3 : Wt.transform.rCenter === 0 ? 1 / 3 : 1, rt = ee(Wt.transform.rCenter, vn.rCenter);
                    return function(ft) {
                      var pt = jt(ft), wt = cn(ft), Bt = wn(ft), At = function(Ot) {
                        return rt(Math.pow(Ot, Ie));
                      }(ft), Dt = { pxmid: Lt(pt, (wt + Bt) / 2), rpx1: pt, transform: { textPosAngle: Ht(ft), rCenter: At, x: vn.x, y: vn.y } };
                      return le(Pe.type, vn, ke), { transform: { targetX: vt(Dt), targetY: $t(Dt), scale: Rn(ft), rotate: Pn(ft), rCenter: At } };
                    };
                  }(hn);
                  return function(Ut) {
                    return sn(Qt(Ut), Yt);
                  };
                }) : Kt.attr("transform", sn(Ft, Yt));
              });
            }
            function Se(we) {
              return Ee = we.rpx1, xe = we.transform.textPosAngle, [Ee * Math.sin(xe), -Ee * Math.cos(xe)];
              var Ee, xe;
            }
            J.plot = function(we, Ee, xe, Le) {
              var Qe, ke, Me = we._fullLayout, Re = Me._sunburstlayer, De = !xe, Pe = !Me.uniformtext.mode && be.hasTransition(xe);
              ue("sunburst", Me), (Qe = Re.selectAll("g.trace.sunburst").data(Ee, function($e) {
                return $e[0].trace.uid;
              })).enter().append("g").classed("trace", !0).classed("sunburst", !0).attr("stroke-linejoin", "round"), Qe.order(), Pe ? (Le && (ke = Le()), te.transition().duration(xe.duration).ease(xe.easing).each("end", function() {
                ke && ke();
              }).each("interrupt", function() {
                ke && ke();
              }).each(function() {
                Re.selectAll("g.trace").each(function($e) {
                  Ce(we, $e, this, xe);
                });
              })) : (Qe.each(function($e) {
                Ce(we, $e, this, xe);
              }), Me.uniformtext.mode && ye(we, Me._sunburstlayer.selectAll(".trace"), "sunburst")), De && Qe.exit().remove();
            }, J.formatSliceLabel = function(we, Ee, xe, Le, Qe) {
              var ke = xe.texttemplate, Me = xe.textinfo;
              if (!(ke || Me && Me !== "none")) return "";
              var Re = Qe.separators, De = Le[0], Pe = we.data.data, $e = De.hierarchy, qe = be.isHierarchyRoot(we), Xe = be.getParent($e, we), Ve = be.getValue(we);
              if (!ke) {
                var Ze, Ge = Me.split("+"), ze = function(gt) {
                  return Ge.indexOf(gt) !== -1;
                }, We = [];
                if (ze("label") && Pe.label && We.push(Pe.label), Pe.hasOwnProperty("v") && ze("value") && We.push(be.formatValue(Pe.v, Re)), !qe) {
                  ze("current path") && We.push(be.getPath(we.data));
                  var at = 0;
                  ze("percent parent") && at++, ze("percent entry") && at++, ze("percent root") && at++;
                  var ct = at > 1;
                  if (at) {
                    var et, dt = function(gt) {
                      Ze = be.formatPercent(et, Re), ct && (Ze += " of " + gt), We.push(Ze);
                    };
                    ze("percent parent") && !qe && (et = Ve / be.getValue(Xe), dt("parent")), ze("percent entry") && (et = Ve / be.getValue(Ee), dt("entry")), ze("percent root") && (et = Ve / be.getValue($e), dt("root"));
                  }
                }
                return ze("text") && (Ze = oe.castOption(xe, Pe.i, "text"), oe.isValidTextValue(Ze) && We.push(Ze)), We.join("<br>");
              }
              var Je = oe.castOption(xe, Pe.i, "texttemplate");
              if (!Je) return "";
              var it = {};
              Pe.label && (it.label = Pe.label), Pe.hasOwnProperty("v") && (it.value = Pe.v, it.valueLabel = be.formatValue(Pe.v, Re)), it.currentPath = be.getPath(we.data), qe || (it.percentParent = Ve / be.getValue(Xe), it.percentParentLabel = be.formatPercent(it.percentParent, Re), it.parent = be.getPtLabel(Xe)), it.percentEntry = Ve / be.getValue(Ee), it.percentEntryLabel = be.formatPercent(it.percentEntry, Re), it.entry = be.getPtLabel(Ee), it.percentRoot = Ve / be.getValue($e), it.percentRootLabel = be.formatPercent(it.percentRoot, Re), it.root = be.getPtLabel($e), Pe.hasOwnProperty("color") && (it.color = Pe.color);
              var st = oe.castOption(xe, Pe.i, "text");
              return (oe.isValidTextValue(st) || st === "") && (it.text = st), it.customdata = oe.castOption(xe, Pe.i, "customdata"), oe.texttemplateString(Je, it, Qe._d3locale, it, xe._meta || {});
            };
          }, 98972: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(34809), ae = Z(84102).resizeText, oe = Z(72043);
            function se(ce, le, ue, de) {
              var fe = le.data.data, he = !le.children, me = fe.i, pe = ee.castOption(ue, me, "marker.line.color") || ne.defaultLine, ye = ee.castOption(ue, me, "marker.line.width") || 0;
              ce.call(oe, le, ue, de).style("stroke-width", ye).call(ne.stroke, pe).style("opacity", he ? ue.leaf.opacity : null);
            }
            G.exports = { style: function(ce) {
              var le = ce._fullLayout._sunburstlayer.selectAll(".trace");
              ae(ce, le, "sunburst"), le.each(function(ue) {
                var de = te.select(this), fe = ue[0].trace;
                de.style("opacity", fe.opacity), de.selectAll("path.surface").each(function(he) {
                  te.select(this).call(se, he, fe, ce);
                });
              });
            }, styleOne: se };
          }, 16131: function(G, J, Z) {
            var te = Z(78766), ne = Z(87163), ee = Z(80712).axisHoverFormat, ae = Z(3208).rb, oe = Z(9829), se = Z(93049).extendFlat, ce = Z(13582).overrideAll;
            function le(de) {
              return { show: { valType: "boolean", dflt: !1 }, start: { valType: "number", dflt: null, editType: "plot" }, end: { valType: "number", dflt: null, editType: "plot" }, size: { valType: "number", dflt: null, min: 0, editType: "plot" }, project: { x: { valType: "boolean", dflt: !1 }, y: { valType: "boolean", dflt: !1 }, z: { valType: "boolean", dflt: !1 } }, color: { valType: "color", dflt: te.defaultLine }, usecolormap: { valType: "boolean", dflt: !1 }, width: { valType: "number", min: 1, max: 16, dflt: 2 }, highlight: { valType: "boolean", dflt: !0 }, highlightcolor: { valType: "color", dflt: te.defaultLine }, highlightwidth: { valType: "number", min: 1, max: 16, dflt: 2 } };
            }
            var ue = G.exports = ce(se({ z: { valType: "data_array" }, x: { valType: "data_array" }, y: { valType: "data_array" }, text: { valType: "string", dflt: "", arrayOk: !0 }, hovertext: { valType: "string", dflt: "", arrayOk: !0 }, hovertemplate: ae(), xhoverformat: ee("x"), yhoverformat: ee("y"), zhoverformat: ee("z"), connectgaps: { valType: "boolean", dflt: !1, editType: "calc" }, surfacecolor: { valType: "data_array" } }, ne("", { colorAttr: "z or surfacecolor", showScaleDflt: !0, autoColorDflt: !1, editTypeOverride: "calc" }), { contours: { x: le(), y: le(), z: le() }, hidesurface: { valType: "boolean", dflt: !1 }, lightposition: { x: { valType: "number", min: -1e5, max: 1e5, dflt: 10 }, y: { valType: "number", min: -1e5, max: 1e5, dflt: 1e4 }, z: { valType: "number", min: -1e5, max: 1e5, dflt: 0 } }, lighting: { ambient: { valType: "number", min: 0, max: 1, dflt: 0.8 }, diffuse: { valType: "number", min: 0, max: 1, dflt: 0.8 }, specular: { valType: "number", min: 0, max: 2, dflt: 0.05 }, roughness: { valType: "number", min: 0, max: 1, dflt: 0.5 }, fresnel: { valType: "number", min: 0, max: 5, dflt: 0.2 } }, opacity: { valType: "number", min: 0, max: 1, dflt: 1 }, opacityscale: { valType: "any", editType: "calc" }, _deprecated: { zauto: se({}, ne.zauto, {}), zmin: se({}, ne.zmin, {}), zmax: se({}, ne.zmax, {}) }, hoverinfo: se({}, oe.hoverinfo), showlegend: se({}, oe.showlegend, { dflt: !1 }) }), "calc", "nested");
            ue.x.editType = ue.y.editType = ue.z.editType = "calc+clearAxisTypes", ue.transforms = void 0;
          }, 53027: function(G, J, Z) {
            var te = Z(28379);
            G.exports = function(ne, ee) {
              ee.surfacecolor ? te(ne, ee, { vals: ee.surfacecolor, containerStr: "", cLetter: "c" }) : te(ne, ee, { vals: ee.z, containerStr: "", cLetter: "c" });
            };
          }, 27159: function(G, J, Z) {
            var te = Z(99098).gl_surface3d, ne = Z(99098).ndarray, ee = Z(99098).ndarray_linear_interpolate.d2, ae = Z(69295), oe = Z(78106), se = Z(34809).isArrayOrTypedArray, ce = Z(46998).parseColorScale, le = Z(55010), ue = Z(88856).extractOpts;
            function de(xe, Le, Qe) {
              this.scene = xe, this.uid = Qe, this.surface = Le, this.data = null, this.showContour = [!1, !1, !1], this.contourStart = [null, null, null], this.contourEnd = [null, null, null], this.contourSize = [0, 0, 0], this.minValues = [1 / 0, 1 / 0, 1 / 0], this.maxValues = [-1 / 0, -1 / 0, -1 / 0], this.dataScaleX = 1, this.dataScaleY = 1, this.refineData = !0, this.objectOffset = [0, 0, 0];
            }
            var fe = de.prototype;
            fe.getXat = function(xe, Le, Qe, ke) {
              var Me = se(this.data.x) ? se(this.data.x[0]) ? this.data.x[Le][xe] : this.data.x[xe] : xe;
              return Qe === void 0 ? Me : ke.d2l(Me, 0, Qe);
            }, fe.getYat = function(xe, Le, Qe, ke) {
              var Me = se(this.data.y) ? se(this.data.y[0]) ? this.data.y[Le][xe] : this.data.y[Le] : Le;
              return Qe === void 0 ? Me : ke.d2l(Me, 0, Qe);
            }, fe.getZat = function(xe, Le, Qe, ke) {
              var Me = this.data.z[Le][xe];
              return Me === null && this.data.connectgaps && this.data._interpolatedZ && (Me = this.data._interpolatedZ[Le][xe]), Qe === void 0 ? Me : ke.d2l(Me, 0, Qe);
            }, fe.handlePick = function(xe) {
              if (xe.object === this.surface) {
                var Le = (xe.data.index[0] - 1) / this.dataScaleX - 1, Qe = (xe.data.index[1] - 1) / this.dataScaleY - 1, ke = Math.max(Math.min(Math.round(Le), this.data.z[0].length - 1), 0), Me = Math.max(Math.min(Math.round(Qe), this.data._ylength - 1), 0);
                xe.index = [ke, Me], xe.traceCoordinate = [this.getXat(ke, Me), this.getYat(ke, Me), this.getZat(ke, Me)], xe.dataCoordinate = [this.getXat(ke, Me, this.data.xcalendar, this.scene.fullSceneLayout.xaxis), this.getYat(ke, Me, this.data.ycalendar, this.scene.fullSceneLayout.yaxis), this.getZat(ke, Me, this.data.zcalendar, this.scene.fullSceneLayout.zaxis)];
                for (var Re = 0; Re < 3; Re++)
                  xe.dataCoordinate[Re] != null && (xe.dataCoordinate[Re] *= this.scene.dataScale[Re]);
                var De = this.data.hovertext || this.data.text;
                return se(De) && De[Me] && De[Me][ke] !== void 0 ? xe.textLabel = De[Me][ke] : xe.textLabel = De || "", xe.data.dataCoordinate = xe.dataCoordinate.slice(), this.surface.highlight(xe.data), this.scene.glplot.spikes.position = xe.dataCoordinate, !0;
              }
            };
            var he = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223, 227, 229, 233, 239, 241, 251, 257, 263, 269, 271, 277, 281, 283, 293, 307, 311, 313, 317, 331, 337, 347, 349, 353, 359, 367, 373, 379, 383, 389, 397, 401, 409, 419, 421, 431, 433, 439, 443, 449, 457, 461, 463, 467, 479, 487, 491, 499, 503, 509, 521, 523, 541, 547, 557, 563, 569, 571, 577, 587, 593, 599, 601, 607, 613, 617, 619, 631, 641, 643, 647, 653, 659, 661, 673, 677, 683, 691, 701, 709, 719, 727, 733, 739, 743, 751, 757, 761, 769, 773, 787, 797, 809, 811, 821, 823, 827, 829, 839, 853, 857, 859, 863, 877, 881, 883, 887, 907, 911, 919, 929, 937, 941, 947, 953, 967, 971, 977, 983, 991, 997, 1009, 1013, 1019, 1021, 1031, 1033, 1039, 1049, 1051, 1061, 1063, 1069, 1087, 1091, 1093, 1097, 1103, 1109, 1117, 1123, 1129, 1151, 1153, 1163, 1171, 1181, 1187, 1193, 1201, 1213, 1217, 1223, 1229, 1231, 1237, 1249, 1259, 1277, 1279, 1283, 1289, 1291, 1297, 1301, 1303, 1307, 1319, 1321, 1327, 1361, 1367, 1373, 1381, 1399, 1409, 1423, 1427, 1429, 1433, 1439, 1447, 1451, 1453, 1459, 1471, 1481, 1483, 1487, 1489, 1493, 1499, 1511, 1523, 1531, 1543, 1549, 1553, 1559, 1567, 1571, 1579, 1583, 1597, 1601, 1607, 1609, 1613, 1619, 1621, 1627, 1637, 1657, 1663, 1667, 1669, 1693, 1697, 1699, 1709, 1721, 1723, 1733, 1741, 1747, 1753, 1759, 1777, 1783, 1787, 1789, 1801, 1811, 1823, 1831, 1847, 1861, 1867, 1871, 1873, 1877, 1879, 1889, 1901, 1907, 1913, 1931, 1933, 1949, 1951, 1973, 1979, 1987, 1993, 1997, 1999, 2003, 2011, 2017, 2027, 2029, 2039, 2053, 2063, 2069, 2081, 2083, 2087, 2089, 2099, 2111, 2113, 2129, 2131, 2137, 2141, 2143, 2153, 2161, 2179, 2203, 2207, 2213, 2221, 2237, 2239, 2243, 2251, 2267, 2269, 2273, 2281, 2287, 2293, 2297, 2309, 2311, 2333, 2339, 2341, 2347, 2351, 2357, 2371, 2377, 2381, 2383, 2389, 2393, 2399, 2411, 2417, 2423, 2437, 2441, 2447, 2459, 2467, 2473, 2477, 2503, 2521, 2531, 2539, 2543, 2549, 2551, 2557, 2579, 2591, 2593, 2609, 2617, 2621, 2633, 2647, 2657, 2659, 2663, 2671, 2677, 2683, 2687, 2689, 2693, 2699, 2707, 2711, 2713, 2719, 2729, 2731, 2741, 2749, 2753, 2767, 2777, 2789, 2791, 2797, 2801, 2803, 2819, 2833, 2837, 2843, 2851, 2857, 2861, 2879, 2887, 2897, 2903, 2909, 2917, 2927, 2939, 2953, 2957, 2963, 2969, 2971, 2999];
            function me(xe, Le) {
              if (xe < Le) return 0;
              for (var Qe = 0; Math.floor(xe % Le) === 0; ) xe /= Le, Qe++;
              return Qe;
            }
            function pe(xe) {
              for (var Le = [], Qe = 0; Qe < he.length; Qe++) {
                var ke = he[Qe];
                Le.push(me(xe, ke));
              }
              return Le;
            }
            function ye(xe) {
              for (var Le = pe(xe), Qe = xe, ke = 0; ke < he.length; ke++) if (Le[ke] > 0) {
                Qe = he[ke];
                break;
              }
              return Qe;
            }
            function ge(xe, Le) {
              if (!(xe < 1 || Le < 1)) {
                for (var Qe = pe(xe), ke = pe(Le), Me = 1, Re = 0; Re < he.length; Re++) Me *= Math.pow(he[Re], Math.max(Qe[Re], ke[Re]));
                return Me;
              }
            }
            fe.calcXnums = function(xe) {
              var Le, Qe = [];
              for (Le = 1; Le < xe; Le++) {
                var ke = this.getXat(Le - 1, 0), Me = this.getXat(Le, 0);
                Qe[Le - 1] = Me !== ke && ke != null && Me != null ? Math.abs(Me - ke) : 0;
              }
              var Re = 0;
              for (Le = 1; Le < xe; Le++) Re += Qe[Le - 1];
              for (Le = 1; Le < xe; Le++) Qe[Le - 1] === 0 ? Qe[Le - 1] = 1 : Qe[Le - 1] = Math.round(Re / Qe[Le - 1]);
              return Qe;
            }, fe.calcYnums = function(xe) {
              var Le, Qe = [];
              for (Le = 1; Le < xe; Le++) {
                var ke = this.getYat(0, Le - 1), Me = this.getYat(0, Le);
                Qe[Le - 1] = Me !== ke && ke != null && Me != null ? Math.abs(Me - ke) : 0;
              }
              var Re = 0;
              for (Le = 1; Le < xe; Le++) Re += Qe[Le - 1];
              for (Le = 1; Le < xe; Le++) Qe[Le - 1] === 0 ? Qe[Le - 1] = 1 : Qe[Le - 1] = Math.round(Re / Qe[Le - 1]);
              return Qe;
            };
            var ve = [1, 2, 4, 6, 12, 24, 36, 48, 60, 120, 180, 240, 360, 720, 840, 1260], be = ve[9], Ce = ve[13];
            function Se(xe, Le, Qe) {
              var ke = Qe[8] + Qe[2] * Le[0] + Qe[5] * Le[1];
              return xe[0] = (Qe[6] + Qe[0] * Le[0] + Qe[3] * Le[1]) / ke, xe[1] = (Qe[7] + Qe[1] * Le[0] + Qe[4] * Le[1]) / ke, xe;
            }
            function we(xe, Le, Qe) {
              return function(ke, Me, Re, De) {
                for (var Pe = [0, 0], $e = ke.shape[0], qe = ke.shape[1], Xe = 0; Xe < $e; Xe++) for (var Ve = 0; Ve < qe; Ve++) Re(Pe, [Xe, Ve], De), ke.set(Xe, Ve, ee(Me, Pe[0], Pe[1]));
              }(xe, Le, Se, Qe), xe;
            }
            function Ee(xe, Le) {
              for (var Qe = !1, ke = 0; ke < xe.length; ke++) if (Le === xe[ke]) {
                Qe = !0;
                break;
              }
              Qe === !1 && xe.push(Le);
            }
            fe.estimateScale = function(xe, Le) {
              for (var Qe = 1 + function(Me) {
                if (Me.length !== 0) {
                  for (var Re = 1, De = 0; De < Me.length; De++) Re = ge(Re, Me[De]);
                  return Re;
                }
              }(Le === 0 ? this.calcXnums(xe) : this.calcYnums(xe)); Qe < be; ) Qe *= 2;
              for (; Qe > Ce; ) Qe--, Qe /= ye(Qe), ++Qe < be && (Qe = Ce);
              var ke = Math.round(Qe / xe);
              return ke > 1 ? ke : 1;
            }, fe.refineCoords = function(xe) {
              for (var Le = this.dataScaleX, Qe = this.dataScaleY, ke = xe[0].shape[0], Me = xe[0].shape[1], Re = 0 | Math.floor(xe[0].shape[0] * Le + 1), De = 0 | Math.floor(xe[0].shape[1] * Qe + 1), Pe = 1 + ke + 1, $e = 1 + Me + 1, qe = ne(new Float32Array(Pe * $e), [Pe, $e]), Xe = [1 / Le, 0, 0, 0, 1 / Qe, 0, 0, 0, 1], Ve = 0; Ve < xe.length; ++Ve) {
                this.surface.padField(qe, xe[Ve]);
                var Ze = ne(new Float32Array(Re * De), [Re, De]);
                we(Ze, qe, Xe), xe[Ve] = Ze;
              }
            }, fe.setContourLevels = function() {
              var xe, Le, Qe, ke = [[], [], []], Me = [!1, !1, !1], Re = !1;
              for (xe = 0; xe < 3; ++xe) if (this.showContour[xe] && (Re = !0, this.contourSize[xe] > 0 && this.contourStart[xe] !== null && this.contourEnd[xe] !== null && this.contourEnd[xe] > this.contourStart[xe])) for (Me[xe] = !0, Le = this.contourStart[xe]; Le < this.contourEnd[xe]; Le += this.contourSize[xe]) Qe = Le * this.scene.dataScale[xe], Ee(ke[xe], Qe);
              if (Re) {
                var De = [[], [], []];
                for (xe = 0; xe < 3; ++xe) this.showContour[xe] && (De[xe] = Me[xe] ? ke[xe] : this.scene.contourLevels[xe]);
                this.surface.update({ levels: De });
              }
            }, fe.update = function(xe) {
              var Le, Qe, ke, Me, Re = this.scene, De = Re.fullSceneLayout, Pe = this.surface, $e = ce(xe), qe = Re.dataScale, Xe = xe.z[0].length, Ve = xe._ylength, Ze = Re.contourLevels;
              this.data = xe;
              var Ge = [];
              for (Le = 0; Le < 3; Le++) for (Ge[Le] = [], Qe = 0; Qe < Xe; Qe++) Ge[Le][Qe] = [];
              for (Qe = 0; Qe < Xe; Qe++) for (ke = 0; ke < Ve; ke++) Ge[0][Qe][ke] = this.getXat(Qe, ke, xe.xcalendar, De.xaxis), Ge[1][Qe][ke] = this.getYat(Qe, ke, xe.ycalendar, De.yaxis), Ge[2][Qe][ke] = this.getZat(Qe, ke, xe.zcalendar, De.zaxis);
              if (xe.connectgaps) for (xe._emptypoints = oe(Ge[2]), ae(Ge[2], xe._emptypoints), xe._interpolatedZ = [], Qe = 0; Qe < Xe; Qe++) for (xe._interpolatedZ[Qe] = [], ke = 0; ke < Ve; ke++) xe._interpolatedZ[Qe][ke] = Ge[2][Qe][ke];
              for (Le = 0; Le < 3; Le++) for (Qe = 0; Qe < Xe; Qe++) for (ke = 0; ke < Ve; ke++) (Me = Ge[Le][Qe][ke]) == null ? Ge[Le][Qe][ke] = NaN : Me = Ge[Le][Qe][ke] *= qe[Le];
              for (Le = 0; Le < 3; Le++) for (Qe = 0; Qe < Xe; Qe++) for (ke = 0; ke < Ve; ke++) (Me = Ge[Le][Qe][ke]) != null && (this.minValues[Le] > Me && (this.minValues[Le] = Me), this.maxValues[Le] < Me && (this.maxValues[Le] = Me));
              for (Le = 0; Le < 3; Le++) this.objectOffset[Le] = 0.5 * (this.minValues[Le] + this.maxValues[Le]);
              for (Le = 0; Le < 3; Le++) for (Qe = 0; Qe < Xe; Qe++) for (ke = 0; ke < Ve; ke++) (Me = Ge[Le][Qe][ke]) != null && (Ge[Le][Qe][ke] -= this.objectOffset[Le]);
              var ze = [ne(new Float32Array(Xe * Ve), [Xe, Ve]), ne(new Float32Array(Xe * Ve), [Xe, Ve]), ne(new Float32Array(Xe * Ve), [Xe, Ve])];
              for (Le = 0; Le < 3; Le++) for (Qe = 0; Qe < Xe; Qe++) for (ke = 0; ke < Ve; ke++) ze[Le].set(Qe, ke, Ge[Le][Qe][ke]);
              Ge = [];
              var We = { colormap: $e, levels: [[], [], []], showContour: [!0, !0, !0], showSurface: !xe.hidesurface, contourProject: [[!1, !1, !1], [!1, !1, !1], [!1, !1, !1]], contourWidth: [1, 1, 1], contourColor: [[1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1]], contourTint: [1, 1, 1], dynamicColor: [[1, 1, 1, 1], [1, 1, 1, 1], [1, 1, 1, 1]], dynamicWidth: [1, 1, 1], dynamicTint: [1, 1, 1], opacityscale: xe.opacityscale, opacity: xe.opacity }, at = ue(xe);
              if (We.intensityBounds = [at.min, at.max], xe.surfacecolor) {
                var ct = ne(new Float32Array(Xe * Ve), [Xe, Ve]);
                for (Qe = 0; Qe < Xe; Qe++) for (ke = 0; ke < Ve; ke++) ct.set(Qe, ke, xe.surfacecolor[ke][Qe]);
                ze.push(ct);
              } else We.intensityBounds[0] *= qe[2], We.intensityBounds[1] *= qe[2];
              (Ce < ze[0].shape[0] || Ce < ze[0].shape[1]) && (this.refineData = !1), this.refineData === !0 && (this.dataScaleX = this.estimateScale(ze[0].shape[0], 0), this.dataScaleY = this.estimateScale(ze[0].shape[1], 1), this.dataScaleX === 1 && this.dataScaleY === 1 || this.refineCoords(ze)), xe.surfacecolor && (We.intensity = ze.pop());
              var et = [!0, !0, !0], dt = ["x", "y", "z"];
              for (Le = 0; Le < 3; ++Le) {
                var Je = xe.contours[dt[Le]];
                et[Le] = Je.highlight, We.showContour[Le] = Je.show || Je.highlight, We.showContour[Le] && (We.contourProject[Le] = [Je.project.x, Je.project.y, Je.project.z], Je.show ? (this.showContour[Le] = !0, We.levels[Le] = Ze[Le], Pe.highlightColor[Le] = We.contourColor[Le] = le(Je.color), Je.usecolormap ? Pe.highlightTint[Le] = We.contourTint[Le] = 0 : Pe.highlightTint[Le] = We.contourTint[Le] = 1, We.contourWidth[Le] = Je.width, this.contourStart[Le] = Je.start, this.contourEnd[Le] = Je.end, this.contourSize[Le] = Je.size) : (this.showContour[Le] = !1, this.contourStart[Le] = null, this.contourEnd[Le] = null, this.contourSize[Le] = 0), Je.highlight && (We.dynamicColor[Le] = le(Je.highlightcolor), We.dynamicWidth[Le] = Je.highlightwidth));
              }
              (function(it) {
                var st = it[0].rgb, gt = it[it.length - 1].rgb;
                return st[0] === gt[0] && st[1] === gt[1] && st[2] === gt[2] && st[3] === gt[3];
              })($e) && (We.vertexColor = !0), We.objectOffset = this.objectOffset, We.coords = ze, Pe.update(We), Pe.visible = xe.visible, Pe.enableDynamic = et, Pe.enableHighlight = et, Pe.snapToData = !0, "lighting" in xe && (Pe.ambientLight = xe.lighting.ambient, Pe.diffuseLight = xe.lighting.diffuse, Pe.specularLight = xe.lighting.specular, Pe.roughness = xe.lighting.roughness, Pe.fresnel = xe.lighting.fresnel), "lightposition" in xe && (Pe.lightPosition = [xe.lightposition.x, xe.lightposition.y, xe.lightposition.z]);
            }, fe.dispose = function() {
              this.scene.glplot.remove(this.surface), this.surface.dispose();
            }, G.exports = function(xe, Le) {
              var Qe = xe.glplot.gl, ke = te({ gl: Qe }), Me = new de(xe, ke, Le.uid);
              return ke._trace = Me, Me.update(Le), xe.glplot.add(ke), Me;
            };
          }, 65444: function(G, J, Z) {
            var te = Z(33626), ne = Z(34809), ee = Z(39356), ae = Z(16131);
            function oe(ce, le, ue, de) {
              var fe = de("opacityscale");
              fe === "max" ? le.opacityscale = [[0, 0.1], [1, 1]] : fe === "min" ? le.opacityscale = [[0, 1], [1, 0.1]] : fe === "extremes" ? le.opacityscale = function(he, me) {
                for (var pe = [], ye = 0; ye < 32; ye++) {
                  var ge = ye / 31, ve = 0.1 + 0.9 * (1 - Math.pow(Math.sin(1 * ge * Math.PI), 2));
                  pe.push([ge, Math.max(0, Math.min(1, ve))]);
                }
                return pe;
              }() : function(he) {
                var me = 0;
                if (!Array.isArray(he) || he.length < 2 || !he[0] || !he[he.length - 1] || +he[0][0] != 0 || +he[he.length - 1][0] != 1) return !1;
                for (var pe = 0; pe < he.length; pe++) {
                  var ye = he[pe];
                  if (ye.length !== 2 || +ye[0] < me) return !1;
                  me = +ye[0];
                }
                return !0;
              }(fe) || (le.opacityscale = void 0);
            }
            function se(ce, le, ue) {
              le in ce && !(ue in ce) && (ce[ue] = ce[le]);
            }
            G.exports = { supplyDefaults: function(ce, le, ue, de) {
              var fe, he;
              function me(Ee, xe) {
                return ne.coerce(ce, le, ae, Ee, xe);
              }
              var pe = me("x"), ye = me("y"), ge = me("z");
              if (!ge || !ge.length || pe && pe.length < 1 || ye && ye.length < 1) le.visible = !1;
              else {
                le._xlength = Array.isArray(pe) && ne.isArrayOrTypedArray(pe[0]) ? ge.length : ge[0].length, le._ylength = ge.length, te.getComponentMethod("calendars", "handleTraceDefaults")(ce, le, ["x", "y", "z"], de), me("text"), me("hovertext"), me("hovertemplate"), me("xhoverformat"), me("yhoverformat"), me("zhoverformat"), ["lighting.ambient", "lighting.diffuse", "lighting.specular", "lighting.roughness", "lighting.fresnel", "lightposition.x", "lightposition.y", "lightposition.z", "hidesurface", "connectgaps", "opacity"].forEach(function(Ee) {
                  me(Ee);
                });
                var ve = me("surfacecolor"), be = ["x", "y", "z"];
                for (fe = 0; fe < 3; ++fe) {
                  var Ce = "contours." + be[fe], Se = me(Ce + ".show"), we = me(Ce + ".highlight");
                  if (Se || we) for (he = 0; he < 3; ++he) me(Ce + ".project." + be[he]);
                  Se && (me(Ce + ".color"), me(Ce + ".width"), me(Ce + ".usecolormap")), we && (me(Ce + ".highlightcolor"), me(Ce + ".highlightwidth")), me(Ce + ".start"), me(Ce + ".end"), me(Ce + ".size");
                }
                ve || (se(ce, "zmin", "cmin"), se(ce, "zmax", "cmax"), se(ce, "zauto", "cauto")), ee(ce, le, de, me, { prefix: "", cLetter: "c" }), oe(0, le, 0, me), le._length = null;
              }
            }, opacityscaleDefaults: oe };
          }, 95984: function(G, J, Z) {
            G.exports = { attributes: Z(16131), supplyDefaults: Z(65444).supplyDefaults, colorbar: { min: "cmin", max: "cmax" }, calc: Z(53027), plot: Z(27159), moduleType: "trace", name: "surface", basePlotModule: Z(2487), categories: ["gl3d", "2dMap", "showLegend"], meta: {} };
          }, 92294: function(G, J, Z) {
            var te = Z(50222), ne = Z(93049).extendFlat, ee = Z(13582).overrideAll, ae = Z(80337), oe = Z(13792).u, se = Z(80712).descriptionOnlyNumbers;
            (G.exports = ee({ domain: oe({ name: "table", trace: !0 }), columnwidth: { valType: "number", arrayOk: !0, dflt: null }, columnorder: { valType: "data_array" }, header: { values: { valType: "data_array", dflt: [] }, format: { valType: "data_array", dflt: [], description: se("cell value") }, prefix: { valType: "string", arrayOk: !0, dflt: null }, suffix: { valType: "string", arrayOk: !0, dflt: null }, height: { valType: "number", dflt: 28 }, align: ne({}, te.align, { arrayOk: !0 }), line: { width: { valType: "number", arrayOk: !0, dflt: 1 }, color: { valType: "color", arrayOk: !0, dflt: "grey" } }, fill: { color: { valType: "color", arrayOk: !0, dflt: "white" } }, font: ne({}, ae({ arrayOk: !0 })) }, cells: { values: { valType: "data_array", dflt: [] }, format: { valType: "data_array", dflt: [], description: se("cell value") }, prefix: { valType: "string", arrayOk: !0, dflt: null }, suffix: { valType: "string", arrayOk: !0, dflt: null }, height: { valType: "number", dflt: 20 }, align: ne({}, te.align, { arrayOk: !0 }), line: { width: { valType: "number", arrayOk: !0, dflt: 1 }, color: { valType: "color", arrayOk: !0, dflt: "grey" } }, fill: { color: { valType: "color", arrayOk: !0, dflt: "white" } }, font: ne({}, ae({ arrayOk: !0 })) } }, "calc", "from-root")).transforms = void 0;
          }, 82662: function(G, J, Z) {
            var te = Z(4173).eV, ne = Z(84576), ee = "table";
            J.name = ee, J.plot = function(ae) {
              var oe = te(ae.calcdata, ee)[0];
              oe.length && ne(ae, oe);
            }, J.clean = function(ae, oe, se, ce) {
              var le = ce._has && ce._has(ee), ue = oe._has && oe._has(ee);
              le && !ue && ce._paperdiv.selectAll(".table").remove();
            };
          }, 87522: function(G, J, Z) {
            var te = Z(71293).wrap;
            G.exports = function() {
              return te({});
            };
          }, 18426: function(G) {
            G.exports = { cellPad: 8, columnExtentOffset: 10, columnTitleOffset: 28, emptyHeaderHeight: 16, latexCheck: /^\$.*\$$/, goldenRatio: 1.618, lineBreaker: "<br>", maxDimensionCount: 60, overdrag: 45, releaseTransitionDuration: 120, releaseTransitionEase: "cubic-out", scrollbarCaptureWidth: 18, scrollbarHideDelay: 1e3, scrollbarHideDuration: 1e3, scrollbarOffset: 5, scrollbarWidth: 8, transitionDuration: 100, transitionEase: "cubic-out", uplift: 5, wrapSpacer: " ", wrapSplitCharacter: " ", cn: { table: "table", tableControlView: "table-control-view", scrollBackground: "scroll-background", yColumn: "y-column", columnBlock: "column-block", scrollAreaClip: "scroll-area-clip", scrollAreaClipRect: "scroll-area-clip-rect", columnBoundary: "column-boundary", columnBoundaryClippath: "column-boundary-clippath", columnBoundaryRect: "column-boundary-rect", columnCells: "column-cells", columnCell: "column-cell", cellRect: "cell-rect", cellText: "cell-text", cellTextHolder: "cell-text-holder", scrollbarKit: "scrollbar-kit", scrollbar: "scrollbar", scrollbarSlider: "scrollbar-slider", scrollbarGlyph: "scrollbar-glyph", scrollbarCaptureZone: "scrollbar-capture-zone" } };
          }, 21908: function(G, J, Z) {
            var te = Z(18426), ne = Z(93049).extendFlat, ee = Z(10721), ae = Z(87800).isTypedArray, oe = Z(87800).isArrayOrTypedArray;
            function se(me) {
              if (oe(me)) {
                for (var pe = 0, ye = 0; ye < me.length; ye++) pe = Math.max(pe, se(me[ye]));
                return pe;
              }
              return me;
            }
            function ce(me, pe) {
              return me + pe;
            }
            function le(me) {
              var pe, ye = me.slice(), ge = 1 / 0, ve = 0;
              for (pe = 0; pe < ye.length; pe++) ae(ye[pe]) ? ye[pe] = Array.from(ye[pe]) : oe(ye[pe]) || (ye[pe] = [ye[pe]]), ge = Math.min(ge, ye[pe].length), ve = Math.max(ve, ye[pe].length);
              if (ge !== ve) for (pe = 0; pe < ye.length; pe++) {
                var be = ve - ye[pe].length;
                be && (ye[pe] = ye[pe].concat(ue(be)));
              }
              return ye;
            }
            function ue(me) {
              for (var pe = new Array(me), ye = 0; ye < me; ye++) pe[ye] = "";
              return pe;
            }
            function de(me) {
              return me.calcdata.columns.reduce(function(pe, ye) {
                return ye.xIndex < me.xIndex ? pe + ye.columnWidth : pe;
              }, 0);
            }
            function fe(me, pe) {
              return Object.keys(me).map(function(ye) {
                return ne({}, me[ye], { auxiliaryBlocks: pe });
              });
            }
            function he(me, pe) {
              for (var ye, ge = {}, ve = 0, be = 0, Ce = { firstRowIndex: null, lastRowIndex: null, rows: [] }, Se = 0, we = 0, Ee = 0; Ee < me.length; Ee++) ye = me[Ee], Ce.rows.push({ rowIndex: Ee, rowHeight: ye }), ((be += ye) >= pe || Ee === me.length - 1) && (ge[ve] = Ce, Ce.key = we++, Ce.firstRowIndex = Se, Ce.lastRowIndex = Ee, Ce = { firstRowIndex: null, lastRowIndex: null, rows: [] }, ve += be, Se = Ee + 1, be = 0);
              return ge;
            }
            G.exports = function(me, pe) {
              var ye = le(pe.cells.values), ge = function(Ve) {
                return Ve.slice(pe.header.values.length, Ve.length);
              }, ve = le(pe.header.values);
              ve.length && !ve[0].length && (ve[0] = [""], ve = le(ve));
              var be = ve.concat(ge(ye).map(function() {
                return ue((ve[0] || [""]).length);
              })), Ce = pe.domain, Se = Math.floor(me._fullLayout._size.w * (Ce.x[1] - Ce.x[0])), we = Math.floor(me._fullLayout._size.h * (Ce.y[1] - Ce.y[0])), Ee = pe.header.values.length ? be[0].map(function() {
                return pe.header.height;
              }) : [te.emptyHeaderHeight], xe = ye.length ? ye[0].map(function() {
                return pe.cells.height;
              }) : [], Le = Ee.reduce(ce, 0), Qe = he(xe, we - Le + te.uplift), ke = fe(he(Ee, Le), []), Me = fe(Qe, ke), Re = {}, De = pe._fullInput.columnorder;
              oe(De) && (De = Array.from(De)), De = De.concat(ge(ye.map(function(Ve, Ze) {
                return Ze;
              })));
              var Pe = be.map(function(Ve, Ze) {
                var Ge = oe(pe.columnwidth) ? pe.columnwidth[Math.min(Ze, pe.columnwidth.length - 1)] : pe.columnwidth;
                return ee(Ge) ? Number(Ge) : 1;
              }), $e = Pe.reduce(ce, 0);
              Pe = Pe.map(function(Ve) {
                return Ve / $e * Se;
              });
              var qe = Math.max(se(pe.header.line.width), se(pe.cells.line.width)), Xe = { key: pe.uid + me._context.staticPlot, translateX: Ce.x[0] * me._fullLayout._size.w, translateY: me._fullLayout._size.h * (1 - Ce.y[1]), size: me._fullLayout._size, width: Se, maxLineWidth: qe, height: we, columnOrder: De, groupHeight: we, rowBlocks: Me, headerRowBlocks: ke, scrollY: 0, cells: ne({}, pe.cells, { values: ye }), headerCells: ne({}, pe.header, { values: be }), gdColumns: be.map(function(Ve) {
                return Ve[0];
              }), gdColumnsOriginalOrder: be.map(function(Ve) {
                return Ve[0];
              }), prevPages: [0, 0], scrollbarState: { scrollbarScrollInProgress: !1 }, columns: be.map(function(Ve, Ze) {
                var Ge = Re[Ve];
                return Re[Ve] = (Ge || 0) + 1, { key: Ve + "__" + Re[Ve], label: Ve, specIndex: Ze, xIndex: De[Ze], xScale: de, x: void 0, calcdata: void 0, columnWidth: Pe[Ze] };
              }) };
              return Xe.columns.forEach(function(Ve) {
                Ve.calcdata = Xe, Ve.x = de(Ve);
              }), Xe;
            };
          }, 49618: function(G, J, Z) {
            var te = Z(93049).extendFlat;
            J.splitToPanels = function(ne) {
              var ee = [0, 0], ae = te({}, ne, { key: "header", type: "header", page: 0, prevPages: ee, currentRepaint: [null, null], dragHandle: !0, values: ne.calcdata.headerCells.values[ne.specIndex], rowBlocks: ne.calcdata.headerRowBlocks, calcdata: te({}, ne.calcdata, { cells: ne.calcdata.headerCells }) });
              return [te({}, ne, { key: "cells1", type: "cells", page: 0, prevPages: ee, currentRepaint: [null, null], dragHandle: !1, values: ne.calcdata.cells.values[ne.specIndex], rowBlocks: ne.calcdata.rowBlocks }), te({}, ne, { key: "cells2", type: "cells", page: 1, prevPages: ee, currentRepaint: [null, null], dragHandle: !1, values: ne.calcdata.cells.values[ne.specIndex], rowBlocks: ne.calcdata.rowBlocks }), ae];
            }, J.splitToCells = function(ne) {
              var ee = function(ae) {
                var oe = ae.rowBlocks[ae.page], se = oe ? oe.rows[0].rowIndex : 0;
                return [se, oe ? se + oe.rows.length : 0];
              }(ne);
              return (ne.values || []).slice(ee[0], ee[1]).map(function(ae, oe) {
                return { keyWithinBlock: oe + (typeof ae == "string" && ae.match(/[<$&> ]/) ? "_keybuster_" + Math.random() : ""), key: ee[0] + oe, column: ne, calcdata: ne.calcdata, page: ne.page, rowBlocks: ne.rowBlocks, value: ae };
              });
            };
          }, 23281: function(G, J, Z) {
            var te = Z(34809), ne = Z(92294), ee = Z(13792).N;
            G.exports = function(ae, oe, se, ce) {
              function le(ue, de) {
                return te.coerce(ae, oe, ne, ue, de);
              }
              ee(oe, ce, le), le("columnwidth"), le("header.values"), le("header.format"), le("header.align"), le("header.prefix"), le("header.suffix"), le("header.height"), le("header.line.width"), le("header.line.color"), le("header.fill.color"), te.coerceFont(le, "header.font", ce.font), function(ue, de) {
                for (var fe = ue.columnorder || [], he = ue.header.values.length, me = fe.slice(0, he), pe = me.slice().sort(function(ve, be) {
                  return ve - be;
                }), ye = me.map(function(ve) {
                  return pe.indexOf(ve);
                }), ge = ye.length; ge < he; ge++) ye.push(ge);
                de("columnorder", ye);
              }(oe, le), le("cells.values"), le("cells.format"), le("cells.align"), le("cells.prefix"), le("cells.suffix"), le("cells.height"), le("cells.line.width"), le("cells.line.color"), le("cells.fill.color"), te.coerceFont(le, "cells.font", ce.font), oe._length = null;
            };
          }, 51671: function(G, J, Z) {
            G.exports = { attributes: Z(92294), supplyDefaults: Z(23281), calc: Z(87522), plot: Z(84576), moduleType: "trace", name: "table", basePlotModule: Z(82662), categories: ["noOpacity"], meta: {} };
          }, 84576: function(G, J, Z) {
            var te = Z(18426), ne = Z(45568), ee = Z(34809), ae = ee.numberFormat, oe = Z(71293), se = Z(62203), ce = Z(30635), le = Z(34809).raiseToTop, ue = Z(34809).strTranslate, de = Z(34809).cancelTransition, fe = Z(21908), he = Z(49618), me = Z(78766);
            function pe(We) {
              return Math.ceil(We.calcdata.maxLineWidth / 2);
            }
            function ye(We, at) {
              return "clip" + We._fullLayout._uid + "_scrollAreaBottomClip_" + at.key;
            }
            function ge(We, at) {
              return "clip" + We._fullLayout._uid + "_columnBoundaryClippath_" + at.calcdata.key + "_" + at.specIndex;
            }
            function ve(We) {
              return [].concat.apply([], We.map(function(at) {
                return at;
              })).map(function(at) {
                return at.__data__;
              });
            }
            function be(We, at, ct) {
              var et = We.selectAll("." + te.cn.scrollbarKit).data(oe.repeat, oe.keyFun);
              et.enter().append("g").classed(te.cn.scrollbarKit, !0).style("shape-rendering", "geometricPrecision"), et.each(function(gt) {
                var bt = gt.scrollbarState;
                bt.totalHeight = function(yt) {
                  var St = yt.rowBlocks;
                  return Ve(St, St.length - 1) + (St.length ? Ze(St[St.length - 1], 1 / 0) : 1);
                }(gt), bt.scrollableAreaHeight = gt.groupHeight - ke(gt), bt.currentlyVisibleHeight = Math.min(bt.totalHeight, bt.scrollableAreaHeight), bt.ratio = bt.currentlyVisibleHeight / bt.totalHeight, bt.barLength = Math.max(bt.ratio * bt.currentlyVisibleHeight, te.goldenRatio * te.scrollbarWidth), bt.barWiggleRoom = bt.currentlyVisibleHeight - bt.barLength, bt.wiggleRoom = Math.max(0, bt.totalHeight - bt.scrollableAreaHeight), bt.topY = bt.barWiggleRoom === 0 ? 0 : gt.scrollY / bt.wiggleRoom * bt.barWiggleRoom, bt.bottomY = bt.topY + bt.barLength, bt.dragMultiplier = bt.wiggleRoom / bt.barWiggleRoom;
              }).attr("transform", function(gt) {
                var bt = gt.width + te.scrollbarWidth / 2 + te.scrollbarOffset;
                return ue(bt, ke(gt));
              });
              var dt = et.selectAll("." + te.cn.scrollbar).data(oe.repeat, oe.keyFun);
              dt.enter().append("g").classed(te.cn.scrollbar, !0);
              var Je = dt.selectAll("." + te.cn.scrollbarSlider).data(oe.repeat, oe.keyFun);
              Je.enter().append("g").classed(te.cn.scrollbarSlider, !0), Je.attr("transform", function(gt) {
                return ue(0, gt.scrollbarState.topY || 0);
              });
              var it = Je.selectAll("." + te.cn.scrollbarGlyph).data(oe.repeat, oe.keyFun);
              it.enter().append("line").classed(te.cn.scrollbarGlyph, !0).attr("stroke", "black").attr("stroke-width", te.scrollbarWidth).attr("stroke-linecap", "round").attr("y1", te.scrollbarWidth / 2), it.attr("y2", function(gt) {
                return gt.scrollbarState.barLength - te.scrollbarWidth / 2;
              }).attr("stroke-opacity", function(gt) {
                return gt.columnDragInProgress || !gt.scrollbarState.barWiggleRoom || ct ? 0 : 0.4;
              }), it.transition().delay(0).duration(0), it.transition().delay(te.scrollbarHideDelay).duration(te.scrollbarHideDuration).attr("stroke-opacity", 0);
              var st = dt.selectAll("." + te.cn.scrollbarCaptureZone).data(oe.repeat, oe.keyFun);
              st.enter().append("line").classed(te.cn.scrollbarCaptureZone, !0).attr("stroke", "white").attr("stroke-opacity", 0.01).attr("stroke-width", te.scrollbarCaptureWidth).attr("stroke-linecap", "butt").attr("y1", 0).on("mousedown", function(gt) {
                var bt = ne.event.y, yt = this.getBoundingClientRect(), St = gt.scrollbarState, Lt = bt - yt.top, Rt = ne.scale.linear().domain([0, St.scrollableAreaHeight]).range([0, St.totalHeight]).clamp(!0);
                St.topY <= Lt && Lt <= St.bottomY || Re(at, We, null, Rt(Lt - St.barLength / 2))(gt);
              }).call(ne.behavior.drag().origin(function(gt) {
                return ne.event.stopPropagation(), gt.scrollbarState.scrollbarScrollInProgress = !0, gt;
              }).on("drag", Re(at, We)).on("dragend", function() {
              })), st.attr("y2", function(gt) {
                return gt.scrollbarState.scrollableAreaHeight;
              }), at._context.staticPlot && (it.remove(), st.remove());
            }
            function Ce(We, at, ct, et) {
              var dt = function(bt) {
                var yt = bt.selectAll("." + te.cn.columnCells).data(oe.repeat, oe.keyFun);
                return yt.enter().append("g").classed(te.cn.columnCells, !0), yt.exit().remove(), yt;
              }(ct), Je = function(bt) {
                var yt = bt.selectAll("." + te.cn.columnCell).data(he.splitToCells, function(St) {
                  return St.keyWithinBlock;
                });
                return yt.enter().append("g").classed(te.cn.columnCell, !0), yt.exit().remove(), yt;
              }(dt);
              (function(bt) {
                bt.each(function(yt, St) {
                  var Lt = yt.calcdata.cells.font, Rt = yt.column.specIndex, vt = { size: Ee(Lt.size, Rt, St), color: Ee(Lt.color, Rt, St), family: Ee(Lt.family, Rt, St), weight: Ee(Lt.weight, Rt, St), style: Ee(Lt.style, Rt, St), variant: Ee(Lt.variant, Rt, St), textcase: Ee(Lt.textcase, Rt, St), lineposition: Ee(Lt.lineposition, Rt, St), shadow: Ee(Lt.shadow, Rt, St) };
                  yt.rowNumber = yt.key, yt.align = Ee(yt.calcdata.cells.align, Rt, St), yt.cellBorderWidth = Ee(yt.calcdata.cells.line.width, Rt, St), yt.font = vt;
                });
              })(Je);
              var it = function(bt) {
                var yt = bt.selectAll("." + te.cn.cellRect).data(oe.repeat, function(St) {
                  return St.keyWithinBlock;
                });
                return yt.enter().append("rect").classed(te.cn.cellRect, !0), yt;
              }(Je);
              (function(bt) {
                bt.attr("width", function(yt) {
                  return yt.column.columnWidth;
                }).attr("stroke-width", function(yt) {
                  return yt.cellBorderWidth;
                }).each(function(yt) {
                  var St = ne.select(this);
                  me.stroke(St, Ee(yt.calcdata.cells.line.color, yt.column.specIndex, yt.rowNumber)), me.fill(St, Ee(yt.calcdata.cells.fill.color, yt.column.specIndex, yt.rowNumber));
                });
              })(it);
              var st = function(bt) {
                var yt = bt.selectAll("." + te.cn.cellTextHolder).data(oe.repeat, function(St) {
                  return St.keyWithinBlock;
                });
                return yt.enter().append("g").classed(te.cn.cellTextHolder, !0).style("shape-rendering", "geometricPrecision"), yt;
              }(Je), gt = function(bt) {
                var yt = bt.selectAll("." + te.cn.cellText).data(oe.repeat, function(St) {
                  return St.keyWithinBlock;
                });
                return yt.enter().append("text").classed(te.cn.cellText, !0).style("cursor", function() {
                  return "auto";
                }).on("mousedown", function() {
                  ne.event.stopPropagation();
                }), yt;
              }(st);
              (function(bt) {
                bt.each(function(yt) {
                  se.font(ne.select(this), yt.font);
                });
              })(gt), Se(gt, at, et, We), Xe(Je);
            }
            function Se(We, at, ct, et) {
              We.text(function(dt) {
                var Je = dt.column.specIndex, it = dt.rowNumber, st = dt.value, gt = typeof st == "string", bt = gt && st.match(/<br>/i), yt = !gt || bt;
                dt.mayHaveMarkup = gt && st.match(/[<&>]/);
                var St, Lt = typeof (St = st) == "string" && St.match(te.latexCheck);
                dt.latex = Lt;
                var Rt, vt, $t = Lt ? "" : Ee(dt.calcdata.cells.prefix, Je, it) || "", Pt = Lt ? "" : Ee(dt.calcdata.cells.suffix, Je, it) || "", Xt = Lt ? null : Ee(dt.calcdata.cells.format, Je, it) || null, Nt = $t + (Xt ? ae(Xt)(dt.value) : dt.value) + Pt;
                if (dt.wrappingNeeded = !dt.wrapped && !yt && !Lt && (Rt = we(Nt)), dt.cellHeightMayIncrease = bt || Lt || dt.mayHaveMarkup || (Rt === void 0 ? we(Nt) : Rt), dt.needsConvertToTspans = dt.mayHaveMarkup || dt.wrappingNeeded || dt.latex, dt.wrappingNeeded) {
                  var Gt = (te.wrapSplitCharacter === " " ? Nt.replace(/<a href=/gi, "<a_href=") : Nt).split(te.wrapSplitCharacter), Ft = te.wrapSplitCharacter === " " ? Gt.map(function(Tt) {
                    return Tt.replace(/<a_href=/gi, "<a href=");
                  }) : Gt;
                  dt.fragments = Ft.map(function(Tt) {
                    return { text: Tt, width: null };
                  }), dt.fragments.push({ fragment: te.wrapSpacer, width: null }), vt = Ft.join(te.lineBreaker) + te.lineBreaker + te.wrapSpacer;
                } else delete dt.fragments, vt = Nt;
                return vt;
              }).attr("dy", function(dt) {
                return dt.needsConvertToTspans ? 0 : "0.75em";
              }).each(function(dt) {
                var Je = this, it = ne.select(Je), st = dt.wrappingNeeded ? Pe : $e;
                dt.needsConvertToTspans ? ce.convertToTspans(it, et, st(ct, Je, at, et, dt)) : ne.select(Je.parentNode).attr("transform", function(gt) {
                  return ue(qe(gt), te.cellPad);
                }).attr("text-anchor", function(gt) {
                  return { left: "start", center: "middle", right: "end" }[gt.align];
                });
              });
            }
            function we(We) {
              return We.indexOf(te.wrapSplitCharacter) !== -1;
            }
            function Ee(We, at, ct) {
              if (ee.isArrayOrTypedArray(We)) {
                var et = We[Math.min(at, We.length - 1)];
                return ee.isArrayOrTypedArray(et) ? et[Math.min(ct, et.length - 1)] : et;
              }
              return We;
            }
            function xe(We, at, ct) {
              We.transition().ease(te.releaseTransitionEase).duration(te.releaseTransitionDuration).attr("transform", ue(at.x, ct));
            }
            function Le(We) {
              return We.type === "cells";
            }
            function Qe(We) {
              return We.type === "header";
            }
            function ke(We) {
              return (We.rowBlocks.length ? We.rowBlocks[0].auxiliaryBlocks : []).reduce(function(at, ct) {
                return at + Ze(ct, 1 / 0);
              }, 0);
            }
            function Me(We, at, ct) {
              var et = ve(at)[0];
              if (et !== void 0) {
                var dt = et.rowBlocks, Je = et.calcdata, it = Ve(dt, dt.length), st = et.calcdata.groupHeight - ke(et), gt = Je.scrollY = Math.max(0, Math.min(it - st, Je.scrollY)), bt = function(yt, St, Lt) {
                  for (var Rt = [], vt = 0, $t = 0; $t < yt.length; $t++) {
                    for (var Pt = yt[$t], Xt = Pt.rows, Nt = 0, Gt = 0; Gt < Xt.length; Gt++) Nt += Xt[Gt].rowHeight;
                    Pt.allRowsHeight = Nt, St < vt + Nt && St + Lt > vt && Rt.push($t), vt += Nt;
                  }
                  return Rt;
                }(dt, gt, st);
                bt.length === 1 && (bt[0] === dt.length - 1 ? bt.unshift(bt[0] - 1) : bt.push(bt[0] + 1)), bt[0] % 2 && bt.reverse(), at.each(function(yt, St) {
                  yt.page = bt[St], yt.scrollY = gt;
                }), at.attr("transform", function(yt) {
                  var St = Ve(yt.rowBlocks, yt.page) - yt.scrollY;
                  return ue(0, St);
                }), We && (De(We, ct, at, bt, et.prevPages, et, 0), De(We, ct, at, bt, et.prevPages, et, 1), be(ct, We));
              }
            }
            function Re(We, at, ct, et) {
              return function(dt) {
                var Je = dt.calcdata ? dt.calcdata : dt, it = at.filter(function(yt) {
                  return Je.key === yt.key;
                }), st = ct || Je.scrollbarState.dragMultiplier, gt = Je.scrollY;
                Je.scrollY = et === void 0 ? Je.scrollY + st * ne.event.dy : et;
                var bt = it.selectAll("." + te.cn.yColumn).selectAll("." + te.cn.columnBlock).filter(Le);
                return Me(We, bt, it), Je.scrollY === gt;
              };
            }
            function De(We, at, ct, et, dt, Je, it) {
              et[it] !== dt[it] && (clearTimeout(Je.currentRepaint[it]), Je.currentRepaint[it] = setTimeout(function() {
                var st = ct.filter(function(gt, bt) {
                  return bt === it && et[bt] !== dt[bt];
                });
                Ce(We, at, st, ct), dt[it] = et[it];
              }));
            }
            function Pe(We, at, ct, et) {
              return function() {
                var dt = ne.select(at.parentNode);
                dt.each(function(Je) {
                  var it = Je.fragments;
                  dt.selectAll("tspan.line").each(function(vt, $t) {
                    it[$t].width = this.getComputedTextLength();
                  });
                  var st, gt, bt = it[it.length - 1].width, yt = it.slice(0, -1), St = [], Lt = 0, Rt = Je.column.columnWidth - 2 * te.cellPad;
                  for (Je.value = ""; yt.length; ) Lt + (gt = (st = yt.shift()).width + bt) > Rt && (Je.value += St.join(te.wrapSpacer) + te.lineBreaker, St = [], Lt = 0), St.push(st.text), Lt += gt;
                  Lt && (Je.value += St.join(te.wrapSpacer)), Je.wrapped = !0;
                }), dt.selectAll("tspan.line").remove(), Se(dt.select("." + te.cn.cellText), ct, We, et), ne.select(at.parentNode.parentNode).call(Xe);
              };
            }
            function $e(We, at, ct, et, dt) {
              return function() {
                if (!dt.settledY) {
                  var Je = ne.select(at.parentNode), it = ze(dt), st = dt.key - it.firstRowIndex, gt = it.rows[st].rowHeight, bt = dt.cellHeightMayIncrease ? at.parentNode.getBoundingClientRect().height + 2 * te.cellPad : gt, yt = Math.max(bt, gt);
                  yt - it.rows[st].rowHeight && (it.rows[st].rowHeight = yt, We.selectAll("." + te.cn.columnCell).call(Xe), Me(null, We.filter(Le), 0), be(ct, et, !0)), Je.attr("transform", function() {
                    var St = this, Lt = St.parentNode.getBoundingClientRect(), Rt = ne.select(St.parentNode).select("." + te.cn.cellRect).node().getBoundingClientRect(), vt = St.transform.baseVal.consolidate(), $t = Rt.top - Lt.top + (vt ? vt.matrix.f : te.cellPad);
                    return ue(qe(dt, ne.select(St.parentNode).select("." + te.cn.cellTextHolder).node().getBoundingClientRect().width), $t);
                  }), dt.settledY = !0;
                }
              };
            }
            function qe(We, at) {
              switch (We.align) {
                case "left":
                default:
                  return te.cellPad;
                case "right":
                  return We.column.columnWidth - (at || 0) - te.cellPad;
                case "center":
                  return (We.column.columnWidth - (at || 0)) / 2;
              }
            }
            function Xe(We) {
              We.attr("transform", function(at) {
                var ct = at.rowBlocks[0].auxiliaryBlocks.reduce(function(dt, Je) {
                  return dt + Ze(Je, 1 / 0);
                }, 0), et = Ze(ze(at), at.key);
                return ue(0, et + ct);
              }).selectAll("." + te.cn.cellRect).attr("height", function(at) {
                return (ct = ze(at), et = at.key, ct.rows[et - ct.firstRowIndex]).rowHeight;
                var ct, et;
              });
            }
            function Ve(We, at) {
              for (var ct = 0, et = at - 1; et >= 0; et--) ct += Ge(We[et]);
              return ct;
            }
            function Ze(We, at) {
              for (var ct = 0, et = 0; et < We.rows.length && We.rows[et].rowIndex < at; et++) ct += We.rows[et].rowHeight;
              return ct;
            }
            function Ge(We) {
              var at = We.allRowsHeight;
              if (at !== void 0) return at;
              for (var ct = 0, et = 0; et < We.rows.length; et++) ct += We.rows[et].rowHeight;
              return We.allRowsHeight = ct, ct;
            }
            function ze(We) {
              return We.rowBlocks[We.page];
            }
            G.exports = function(We, at) {
              var ct = !We._context.staticPlot, et = We._fullLayout._paper.selectAll("." + te.cn.table).data(at.map(function(Pt) {
                var Xt = oe.unwrap(Pt).trace;
                return fe(We, Xt);
              }), oe.keyFun);
              et.exit().remove(), et.enter().append("g").classed(te.cn.table, !0).attr("overflow", "visible").style("box-sizing", "content-box").style("position", "absolute").style("left", 0).style("overflow", "visible").style("shape-rendering", "crispEdges").style("pointer-events", "all"), et.attr("width", function(Pt) {
                return Pt.width + Pt.size.l + Pt.size.r;
              }).attr("height", function(Pt) {
                return Pt.height + Pt.size.t + Pt.size.b;
              }).attr("transform", function(Pt) {
                return ue(Pt.translateX, Pt.translateY);
              });
              var dt = et.selectAll("." + te.cn.tableControlView).data(oe.repeat, oe.keyFun), Je = dt.enter().append("g").classed(te.cn.tableControlView, !0).style("box-sizing", "content-box");
              if (ct) {
                var it = "onwheel" in document ? "wheel" : "mousewheel";
                Je.on("mousemove", function(Pt) {
                  dt.filter(function(Xt) {
                    return Pt === Xt;
                  }).call(be, We);
                }).on(it, function(Pt) {
                  if (!Pt.scrollbarState.wheeling) {
                    Pt.scrollbarState.wheeling = !0;
                    var Xt = Pt.scrollY + ne.event.deltaY;
                    Re(We, dt, null, Xt)(Pt) || (ne.event.stopPropagation(), ne.event.preventDefault()), Pt.scrollbarState.wheeling = !1;
                  }
                }).call(be, We, !0);
              }
              dt.attr("transform", function(Pt) {
                return ue(Pt.size.l, Pt.size.t);
              });
              var st = dt.selectAll("." + te.cn.scrollBackground).data(oe.repeat, oe.keyFun);
              st.enter().append("rect").classed(te.cn.scrollBackground, !0).attr("fill", "none"), st.attr("width", function(Pt) {
                return Pt.width;
              }).attr("height", function(Pt) {
                return Pt.height;
              }), dt.each(function(Pt) {
                se.setClipUrl(ne.select(this), ye(We, Pt), We);
              });
              var gt = dt.selectAll("." + te.cn.yColumn).data(function(Pt) {
                return Pt.columns;
              }, oe.keyFun);
              gt.enter().append("g").classed(te.cn.yColumn, !0), gt.exit().remove(), gt.attr("transform", function(Pt) {
                return ue(Pt.x, 0);
              }), ct && gt.call(ne.behavior.drag().origin(function(Pt) {
                return xe(ne.select(this), Pt, -te.uplift), le(this), Pt.calcdata.columnDragInProgress = !0, be(dt.filter(function(Xt) {
                  return Pt.calcdata.key === Xt.key;
                }), We), Pt;
              }).on("drag", function(Pt) {
                var Xt = ne.select(this), Nt = function(Gt) {
                  return (Pt === Gt ? ne.event.x : Gt.x) + Gt.columnWidth / 2;
                };
                Pt.x = Math.max(-te.overdrag, Math.min(Pt.calcdata.width + te.overdrag - Pt.columnWidth, ne.event.x)), ve(gt).filter(function(Gt) {
                  return Gt.calcdata.key === Pt.calcdata.key;
                }).sort(function(Gt, Ft) {
                  return Nt(Gt) - Nt(Ft);
                }).forEach(function(Gt, Ft) {
                  Gt.xIndex = Ft, Gt.x = Pt === Gt ? Gt.x : Gt.xScale(Gt);
                }), gt.filter(function(Gt) {
                  return Pt !== Gt;
                }).transition().ease(te.transitionEase).duration(te.transitionDuration).attr("transform", function(Gt) {
                  return ue(Gt.x, 0);
                }), Xt.call(de).attr("transform", ue(Pt.x, -te.uplift));
              }).on("dragend", function(Pt) {
                var Xt = ne.select(this), Nt = Pt.calcdata;
                Pt.x = Pt.xScale(Pt), Pt.calcdata.columnDragInProgress = !1, xe(Xt, Pt, 0), function(Gt, Ft, Tt) {
                  var It = Ft.gdColumnsOriginalOrder;
                  Ft.gdColumns.sort(function(Ct, Kt) {
                    return Tt[It.indexOf(Ct)] - Tt[It.indexOf(Kt)];
                  }), Ft.columnorder = Tt, Gt.emit("plotly_restyle");
                }(We, Nt, Nt.columns.map(function(Gt) {
                  return Gt.xIndex;
                }));
              })), gt.each(function(Pt) {
                se.setClipUrl(ne.select(this), ge(We, Pt), We);
              });
              var bt = gt.selectAll("." + te.cn.columnBlock).data(he.splitToPanels, oe.keyFun);
              bt.enter().append("g").classed(te.cn.columnBlock, !0).attr("id", function(Pt) {
                return Pt.key;
              }), bt.style("cursor", function(Pt) {
                return Pt.dragHandle ? "ew-resize" : Pt.calcdata.scrollbarState.barWiggleRoom ? "ns-resize" : "default";
              });
              var yt = bt.filter(Qe), St = bt.filter(Le);
              ct && St.call(ne.behavior.drag().origin(function(Pt) {
                return ne.event.stopPropagation(), Pt;
              }).on("drag", Re(We, dt, -1)).on("dragend", function() {
              })), Ce(We, dt, yt, bt), Ce(We, dt, St, bt);
              var Lt = dt.selectAll("." + te.cn.scrollAreaClip).data(oe.repeat, oe.keyFun);
              Lt.enter().append("clipPath").classed(te.cn.scrollAreaClip, !0).attr("id", function(Pt) {
                return ye(We, Pt);
              });
              var Rt = Lt.selectAll("." + te.cn.scrollAreaClipRect).data(oe.repeat, oe.keyFun);
              Rt.enter().append("rect").classed(te.cn.scrollAreaClipRect, !0).attr("x", -te.overdrag).attr("y", -te.uplift).attr("fill", "none"), Rt.attr("width", function(Pt) {
                return Pt.width + 2 * te.overdrag;
              }).attr("height", function(Pt) {
                return Pt.height + te.uplift;
              }), gt.selectAll("." + te.cn.columnBoundary).data(oe.repeat, oe.keyFun).enter().append("g").classed(te.cn.columnBoundary, !0);
              var vt = gt.selectAll("." + te.cn.columnBoundaryClippath).data(oe.repeat, oe.keyFun);
              vt.enter().append("clipPath").classed(te.cn.columnBoundaryClippath, !0), vt.attr("id", function(Pt) {
                return ge(We, Pt);
              });
              var $t = vt.selectAll("." + te.cn.columnBoundaryRect).data(oe.repeat, oe.keyFun);
              $t.enter().append("rect").classed(te.cn.columnBoundaryRect, !0).attr("fill", "none"), $t.attr("width", function(Pt) {
                return Pt.columnWidth + 2 * pe(Pt);
              }).attr("height", function(Pt) {
                return Pt.calcdata.height + 2 * pe(Pt) + te.uplift;
              }).attr("x", function(Pt) {
                return -pe(Pt);
              }).attr("y", function(Pt) {
                return -pe(Pt);
              }), Me(null, St, dt);
            };
          }, 71856: function(G, J, Z) {
            var te = Z(3208).rb, ne = Z(3208).ay, ee = Z(87163), ae = Z(13792).u, oe = Z(55412), se = Z(56708), ce = Z(43236), le = Z(93049).extendFlat, ue = Z(94850).k;
            G.exports = { labels: se.labels, parents: se.parents, values: se.values, branchvalues: se.branchvalues, count: se.count, level: se.level, maxdepth: se.maxdepth, tiling: { packing: { valType: "enumerated", values: ["squarify", "binary", "dice", "slice", "slice-dice", "dice-slice"], dflt: "squarify", editType: "plot" }, squarifyratio: { valType: "number", min: 1, dflt: 1, editType: "plot" }, flip: { valType: "flaglist", flags: ["x", "y"], dflt: "", editType: "plot" }, pad: { valType: "number", min: 0, dflt: 3, editType: "plot" }, editType: "calc" }, marker: le({ pad: { t: { valType: "number", min: 0, editType: "plot" }, l: { valType: "number", min: 0, editType: "plot" }, r: { valType: "number", min: 0, editType: "plot" }, b: { valType: "number", min: 0, editType: "plot" }, editType: "calc" }, colors: se.marker.colors, pattern: ue, depthfade: { valType: "enumerated", values: [!0, !1, "reversed"], editType: "style" }, line: se.marker.line, cornerradius: { valType: "number", min: 0, dflt: 0, editType: "plot" }, editType: "calc" }, ee("marker", { colorAttr: "colors", anim: !1 })), pathbar: { visible: { valType: "boolean", dflt: !0, editType: "plot" }, side: { valType: "enumerated", values: ["top", "bottom"], dflt: "top", editType: "plot" }, edgeshape: { valType: "enumerated", values: [">", "<", "|", "/", "\\"], dflt: ">", editType: "plot" }, thickness: { valType: "number", min: 12, editType: "plot" }, textfont: le({}, oe.textfont, {}), editType: "calc" }, text: oe.text, textinfo: se.textinfo, texttemplate: ne({ editType: "plot" }, { keys: ce.eventDataKeys.concat(["label", "value"]) }), hovertext: oe.hovertext, hoverinfo: se.hoverinfo, hovertemplate: te({}, { keys: ce.eventDataKeys }), textfont: oe.textfont, insidetextfont: oe.insidetextfont, outsidetextfont: le({}, oe.outsidetextfont, {}), textposition: { valType: "enumerated", values: ["top left", "top center", "top right", "middle left", "middle center", "middle right", "bottom left", "bottom center", "bottom right"], dflt: "top left", editType: "plot" }, sort: oe.sort, root: se.root, domain: ae({ name: "treemap", trace: !0, editType: "calc" }) };
          }, 69784: function(G, J, Z) {
            var te = Z(44122);
            J.name = "treemap", J.plot = function(ne, ee, ae, oe) {
              te.plotBasePlot(J.name, ne, ee, ae, oe);
            }, J.clean = function(ne, ee, ae, oe) {
              te.cleanBasePlot(J.name, ne, ee, ae, oe);
            };
          }, 38848: function(G, J, Z) {
            var te = Z(14852);
            J._ = function(ne, ee) {
              return te.calc(ne, ee);
            }, J.t = function(ne) {
              return te._runCrossTraceCalc("treemap", ne);
            };
          }, 43236: function(G) {
            G.exports = { CLICK_TRANSITION_TIME: 750, CLICK_TRANSITION_EASING: "poly", eventDataKeys: ["currentPath", "root", "entry", "percentRoot", "percentEntry", "percentParent"], gapWithPathbar: 1 };
          }, 95719: function(G, J, Z) {
            var te = Z(34809), ne = Z(71856), ee = Z(78766), ae = Z(13792).N, oe = Z(17550).handleText, se = Z(56155).TEXTPAD, ce = Z(46979).handleMarkerDefaults, le = Z(88856), ue = le.hasColorscale, de = le.handleDefaults;
            G.exports = function(fe, he, me, pe) {
              function ye(xe, Le) {
                return te.coerce(fe, he, ne, xe, Le);
              }
              var ge = ye("labels"), ve = ye("parents");
              if (ge && ge.length && ve && ve.length) {
                var be = ye("values");
                be && be.length ? ye("branchvalues") : ye("count"), ye("level"), ye("maxdepth"), ye("tiling.packing") === "squarify" && ye("tiling.squarifyratio"), ye("tiling.flip"), ye("tiling.pad");
                var Ce = ye("text");
                ye("texttemplate"), he.texttemplate || ye("textinfo", te.isArrayOrTypedArray(Ce) ? "text+label" : "label"), ye("hovertext"), ye("hovertemplate");
                var Se = ye("pathbar.visible");
                oe(fe, he, pe, ye, "auto", { hasPathbar: Se, moduleHasSelected: !1, moduleHasUnselected: !1, moduleHasConstrain: !1, moduleHasCliponaxis: !1, moduleHasTextangle: !1, moduleHasInsideanchor: !1 }), ye("textposition");
                var we = he.textposition.indexOf("bottom") !== -1;
                ce(fe, he, pe, ye), (he._hasColorscale = ue(fe, "marker", "colors") || (fe.marker || {}).coloraxis) ? de(fe, he, pe, ye, { prefix: "marker.", cLetter: "c" }) : ye("marker.depthfade", !(he.marker.colors || []).length);
                var Ee = 2 * he.textfont.size;
                ye("marker.pad.t", we ? Ee / 4 : Ee), ye("marker.pad.l", Ee / 4), ye("marker.pad.r", Ee / 4), ye("marker.pad.b", we ? Ee : Ee / 4), ye("marker.cornerradius"), he._hovered = { marker: { line: { width: 2, color: ee.contrast(pe.paper_bgcolor) } } }, Se && (ye("pathbar.thickness", he.pathbar.textfont.size + 2 * se), ye("pathbar.side"), ye("pathbar.edgeshape")), ye("sort"), ye("root.color"), ae(he, pe, ye), he._length = null;
              } else he.visible = !1;
            };
          }, 41567: function(G, J, Z) {
            var te = Z(45568), ne = Z(33108), ee = Z(84102).clearMinTextSize, ae = Z(6851).resizeText, oe = Z(95709);
            G.exports = function(se, ce, le, ue, de) {
              var fe, he, me = de.type, pe = de.drawDescendants, ye = se._fullLayout, ge = ye["_" + me + "layer"], ve = !le;
              ee(me, ye), (fe = ge.selectAll("g.trace." + me).data(ce, function(be) {
                return be[0].trace.uid;
              })).enter().append("g").classed("trace", !0).classed(me, !0), fe.order(), !ye.uniformtext.mode && ne.hasTransition(le) ? (ue && (he = ue()), te.transition().duration(le.duration).ease(le.easing).each("end", function() {
                he && he();
              }).each("interrupt", function() {
                he && he();
              }).each(function() {
                ge.selectAll("g.trace").each(function(be) {
                  oe(se, be, this, le, pe);
                });
              })) : (fe.each(function(be) {
                oe(se, be, this, le, pe);
              }), ye.uniformtext.mode && ae(se, ge.selectAll(".trace"), me)), ve && fe.exit().remove();
            };
          }, 17010: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(30635), oe = Z(11995), se = Z(92080).styleOne, ce = Z(43236), le = Z(33108), ue = Z(44691), de = !0;
            G.exports = function(fe, he, me, pe, ye) {
              var ge = ye.barDifY, ve = ye.width, be = ye.height, Ce = ye.viewX, Se = ye.viewY, we = ye.pathSlice, Ee = ye.toMoveInsideSlice, xe = ye.strTransform, Le = ye.hasTransition, Qe = ye.handleSlicesExit, ke = ye.makeUpdateSliceInterpolator, Me = ye.makeUpdateTextInterpolator, Re = {}, De = fe._context.staticPlot, Pe = fe._fullLayout, $e = he[0], qe = $e.trace, Xe = $e.hierarchy, Ve = ve / qe._entryDepth, Ze = le.listPath(me.data, "id"), Ge = oe(Xe.copy(), [ve, be], { packing: "dice", pad: { inner: 0, top: 0, left: 0, right: 0, bottom: 0 } }).descendants();
              (Ge = Ge.filter(function(We) {
                var at = Ze.indexOf(We.data.id);
                return at !== -1 && (We.x0 = Ve * at, We.x1 = Ve * (at + 1), We.y0 = ge, We.y1 = ge + be, We.onPathbar = !0, !0);
              })).reverse(), (pe = pe.data(Ge, le.getPtId)).enter().append("g").classed("pathbar", !0), Qe(pe, de, Re, [ve, be], we), pe.order();
              var ze = pe;
              Le && (ze = ze.transition().each("end", function() {
                var We = te.select(this);
                le.setSliceCursor(We, fe, { hideOnRoot: !1, hideOnLeaves: !1, isTransitioning: !1 });
              })), ze.each(function(We) {
                We._x0 = Ce(We.x0), We._x1 = Ce(We.x1), We._y0 = Se(We.y0), We._y1 = Se(We.y1), We._hoverX = Ce(We.x1 - Math.min(ve, be) / 2), We._hoverY = Se(We.y1 - be / 2);
                var at = te.select(this), ct = ne.ensureSingle(at, "path", "surface", function(it) {
                  it.style("pointer-events", De ? "none" : "all");
                });
                Le ? ct.transition().attrTween("d", function(it) {
                  var st = ke(it, de, Re, [ve, be]);
                  return function(gt) {
                    return we(st(gt));
                  };
                }) : ct.attr("d", we), at.call(ue, me, fe, he, { styleOne: se, eventDataKeys: ce.eventDataKeys, transitionTime: ce.CLICK_TRANSITION_TIME, transitionEasing: ce.CLICK_TRANSITION_EASING }).call(le.setSliceCursor, fe, { hideOnRoot: !1, hideOnLeaves: !1, isTransitioning: fe._transitioning }), ct.call(se, We, qe, fe, { hovered: !1 }), We._text = (le.getPtLabel(We) || "").split("<br>").join(" ") || "";
                var et = ne.ensureSingle(at, "g", "slicetext"), dt = ne.ensureSingle(et, "text", "", function(it) {
                  it.attr("data-notex", 1);
                }), Je = ne.ensureUniformFontSize(fe, le.determineTextFont(qe, We, Pe.font, { onPathbar: !0 }));
                dt.text(We._text || " ").classed("slicetext", !0).attr("text-anchor", "start").call(ee.font, Je).call(ae.convertToTspans, fe), We.textBB = ee.bBox(dt.node()), We.transform = Ee(We, { fontSize: Je.size, onPathbar: !0 }), We.transform.fontSize = Je.size, Le ? dt.transition().attrTween("transform", function(it) {
                  var st = Me(it, de, Re, [ve, be]);
                  return function(gt) {
                    return xe(st(gt));
                  };
                }) : dt.attr("transform", xe(We));
              });
            };
          }, 50916: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(30635), oe = Z(11995), se = Z(92080).styleOne, ce = Z(43236), le = Z(33108), ue = Z(44691), de = Z(19718).formatSliceLabel, fe = !1;
            G.exports = function(he, me, pe, ye, ge) {
              var ve = ge.width, be = ge.height, Ce = ge.viewX, Se = ge.viewY, we = ge.pathSlice, Ee = ge.toMoveInsideSlice, xe = ge.strTransform, Le = ge.hasTransition, Qe = ge.handleSlicesExit, ke = ge.makeUpdateSliceInterpolator, Me = ge.makeUpdateTextInterpolator, Re = ge.prevEntry, De = he._context.staticPlot, Pe = he._fullLayout, $e = me[0].trace, qe = $e.textposition.indexOf("left") !== -1, Xe = $e.textposition.indexOf("right") !== -1, Ve = $e.textposition.indexOf("bottom") !== -1, Ze = !Ve && !$e.marker.pad.t || Ve && !$e.marker.pad.b, Ge = oe(pe, [ve, be], { packing: $e.tiling.packing, squarifyratio: $e.tiling.squarifyratio, flipX: $e.tiling.flip.indexOf("x") > -1, flipY: $e.tiling.flip.indexOf("y") > -1, pad: { inner: $e.tiling.pad, top: $e.marker.pad.t, left: $e.marker.pad.l, right: $e.marker.pad.r, bottom: $e.marker.pad.b } }).descendants(), ze = 1 / 0, We = -1 / 0;
              Ge.forEach(function(Je) {
                var it = Je.depth;
                it >= $e._maxDepth ? (Je.x0 = Je.x1 = (Je.x0 + Je.x1) / 2, Je.y0 = Je.y1 = (Je.y0 + Je.y1) / 2) : (ze = Math.min(ze, it), We = Math.max(We, it));
              }), ye = ye.data(Ge, le.getPtId), $e._maxVisibleLayers = isFinite(We) ? We - ze + 1 : 0, ye.enter().append("g").classed("slice", !0), Qe(ye, fe, {}, [ve, be], we), ye.order();
              var at = null;
              if (Le && Re) {
                var ct = le.getPtId(Re);
                ye.each(function(Je) {
                  at === null && le.getPtId(Je) === ct && (at = { x0: Je.x0, x1: Je.x1, y0: Je.y0, y1: Je.y1 });
                });
              }
              var et = function() {
                return at || { x0: 0, x1: ve, y0: 0, y1: be };
              }, dt = ye;
              return Le && (dt = dt.transition().each("end", function() {
                var Je = te.select(this);
                le.setSliceCursor(Je, he, { hideOnRoot: !0, hideOnLeaves: !1, isTransitioning: !1 });
              })), dt.each(function(Je) {
                var it = le.isHeader(Je, $e);
                Je._x0 = Ce(Je.x0), Je._x1 = Ce(Je.x1), Je._y0 = Se(Je.y0), Je._y1 = Se(Je.y1), Je._hoverX = Ce(Je.x1 - $e.marker.pad.r), Je._hoverY = Se(Ve ? Je.y1 - $e.marker.pad.b / 2 : Je.y0 + $e.marker.pad.t / 2);
                var st = te.select(this), gt = ne.ensureSingle(st, "path", "surface", function(vt) {
                  vt.style("pointer-events", De ? "none" : "all");
                });
                Le ? gt.transition().attrTween("d", function(vt) {
                  var $t = ke(vt, fe, et(), [ve, be]);
                  return function(Pt) {
                    return we($t(Pt));
                  };
                }) : gt.attr("d", we), st.call(ue, pe, he, me, { styleOne: se, eventDataKeys: ce.eventDataKeys, transitionTime: ce.CLICK_TRANSITION_TIME, transitionEasing: ce.CLICK_TRANSITION_EASING }).call(le.setSliceCursor, he, { isTransitioning: he._transitioning }), gt.call(se, Je, $e, he, { hovered: !1 }), Je.x0 === Je.x1 || Je.y0 === Je.y1 ? Je._text = "" : Je._text = it ? Ze ? "" : le.getPtLabel(Je) || "" : de(Je, pe, $e, me, Pe) || "";
                var bt = ne.ensureSingle(st, "g", "slicetext"), yt = ne.ensureSingle(bt, "text", "", function(vt) {
                  vt.attr("data-notex", 1);
                }), St = ne.ensureUniformFontSize(he, le.determineTextFont($e, Je, Pe.font)), Lt = Je._text || " ", Rt = it && Lt.indexOf("<br>") === -1;
                yt.text(Lt).classed("slicetext", !0).attr("text-anchor", Xe ? "end" : qe || Rt ? "start" : "middle").call(ee.font, St).call(ae.convertToTspans, he), Je.textBB = ee.bBox(yt.node()), Je.transform = Ee(Je, { fontSize: St.size, isHeader: it }), Je.transform.fontSize = St.size, Le ? yt.transition().attrTween("transform", function(vt) {
                  var $t = Me(vt, fe, et(), [ve, be]);
                  return function(Pt) {
                    return xe($t(Pt));
                  };
                }) : yt.attr("transform", xe(Je));
              }), at;
            };
          }, 36141: function(G) {
            G.exports = function J(Z, te, ne) {
              var ee;
              ne.swapXY && (ee = Z.x0, Z.x0 = Z.y0, Z.y0 = ee, ee = Z.x1, Z.x1 = Z.y1, Z.y1 = ee), ne.flipX && (ee = Z.x0, Z.x0 = te[0] - Z.x1, Z.x1 = te[0] - ee), ne.flipY && (ee = Z.y0, Z.y0 = te[1] - Z.y1, Z.y1 = te[1] - ee);
              var ae = Z.children;
              if (ae) for (var oe = 0; oe < ae.length; oe++) J(ae[oe], te, ne);
            };
          }, 47181: function(G, J, Z) {
            G.exports = { moduleType: "trace", name: "treemap", basePlotModule: Z(69784), categories: [], animatable: !0, attributes: Z(71856), layoutAttributes: Z(4219), supplyDefaults: Z(95719), supplyLayoutDefaults: Z(49852), calc: Z(38848)._, crossTraceCalc: Z(38848).t, plot: Z(64274), style: Z(92080).style, colorbar: Z(21146), meta: {} };
          }, 4219: function(G) {
            G.exports = { treemapcolorway: { valType: "colorlist", editType: "calc" }, extendtreemapcolors: { valType: "boolean", dflt: !0, editType: "calc" } };
          }, 49852: function(G, J, Z) {
            var te = Z(34809), ne = Z(4219);
            G.exports = function(ee, ae) {
              function oe(se, ce) {
                return te.coerce(ee, ae, ne, se, ce);
              }
              oe("treemapcolorway", ae.colorway), oe("extendtreemapcolors");
            };
          }, 11995: function(G, J, Z) {
            var te = Z(92264), ne = Z(36141);
            G.exports = function(ee, ae, oe) {
              var se, ce = oe.flipX, le = oe.flipY, ue = oe.packing === "dice-slice", de = oe.pad[le ? "bottom" : "top"], fe = oe.pad[ce ? "right" : "left"], he = oe.pad[ce ? "left" : "right"], me = oe.pad[le ? "top" : "bottom"];
              ue && (se = fe, fe = de, de = se, se = he, he = me, me = se);
              var pe = te.treemap().tile(function(ye, ge) {
                switch (ye) {
                  case "squarify":
                    return te.treemapSquarify.ratio(ge);
                  case "binary":
                    return te.treemapBinary;
                  case "dice":
                    return te.treemapDice;
                  case "slice":
                    return te.treemapSlice;
                  default:
                    return te.treemapSliceDice;
                }
              }(oe.packing, oe.squarifyratio)).paddingInner(oe.pad.inner).paddingLeft(fe).paddingRight(he).paddingTop(de).paddingBottom(me).size(ue ? [ae[1], ae[0]] : ae)(ee);
              return (ue || ce || le) && ne(pe, ae, { swapXY: ue, flipX: ce, flipY: le }), pe;
            };
          }, 64274: function(G, J, Z) {
            var te = Z(41567), ne = Z(50916);
            G.exports = function(ee, ae, oe, se) {
              return te(ee, ae, oe, se, { type: "treemap", drawDescendants: ne });
            };
          }, 95709: function(G, J, Z) {
            var te = Z(45568), ne = Z(88640).GW, ee = Z(33108), ae = Z(34809), oe = Z(56155).TEXTPAD, se = Z(32995).toMoveInsideBar, ce = Z(84102).recordMinTextSize, le = Z(43236), ue = Z(17010);
            function de(fe) {
              return ee.isHierarchyRoot(fe) ? "" : ee.getPtId(fe);
            }
            G.exports = function(fe, he, me, pe, ye) {
              var ge = fe._fullLayout, ve = he[0], be = ve.trace, Ce = be.type === "icicle", Se = ve.hierarchy, we = ee.findEntryWithLevel(Se, be.level), Ee = te.select(me), xe = Ee.selectAll("g.pathbar"), Le = Ee.selectAll("g.slice");
              if (!we) return xe.remove(), void Le.remove();
              var Qe = ee.isHierarchyRoot(we), ke = !ge.uniformtext.mode && ee.hasTransition(pe), Me = ee.getMaxDepth(be), Re = ge._size, De = be.domain, Pe = Re.w * (De.x[1] - De.x[0]), $e = Re.h * (De.y[1] - De.y[0]), qe = Pe, Xe = be.pathbar.thickness, Ve = be.marker.line.width + le.gapWithPathbar, Ze = be.pathbar.visible ? be.pathbar.side.indexOf("bottom") > -1 ? $e + Ve : -(Xe + Ve) : 0, Ge = { x0: qe, x1: qe, y0: Ze, y1: Ze + Xe }, ze = function(Qt, Ut, Wt) {
                var rn = be.tiling.pad, vn = function(cn) {
                  return cn - rn <= Ut.x0;
                }, dn = function(cn) {
                  return cn + rn >= Ut.x1;
                }, Ht = function(cn) {
                  return cn - rn <= Ut.y0;
                }, jt = function(cn) {
                  return cn + rn >= Ut.y1;
                };
                return Qt.x0 === Ut.x0 && Qt.x1 === Ut.x1 && Qt.y0 === Ut.y0 && Qt.y1 === Ut.y1 ? { x0: Qt.x0, x1: Qt.x1, y0: Qt.y0, y1: Qt.y1 } : { x0: vn(Qt.x0 - rn) ? 0 : dn(Qt.x0 - rn) ? Wt[0] : Qt.x0, x1: vn(Qt.x1 + rn) ? 0 : dn(Qt.x1 + rn) ? Wt[0] : Qt.x1, y0: Ht(Qt.y0 - rn) ? 0 : jt(Qt.y0 - rn) ? Wt[1] : Qt.y0, y1: Ht(Qt.y1 + rn) ? 0 : jt(Qt.y1 + rn) ? Wt[1] : Qt.y1 };
              }, We = null, at = {}, ct = {}, et = null, dt = function(Qt, Ut) {
                return Ut ? at[de(Qt)] : ct[de(Qt)];
              };
              ve.hasMultipleRoots && Qe && Me++, be._maxDepth = Me, be._backgroundColor = ge.paper_bgcolor, be._entryDepth = we.data.depth, be._atRootLevel = Qe;
              var Je = -Pe / 2 + Re.l + Re.w * (De.x[1] + De.x[0]) / 2, it = -$e / 2 + Re.t + Re.h * (1 - (De.y[1] + De.y[0]) / 2), st = function(Qt) {
                return Je + Qt;
              }, gt = function(Qt) {
                return it + Qt;
              }, bt = gt(0), yt = st(0), St = function(Qt) {
                return yt + Qt;
              }, Lt = function(Qt) {
                return bt + Qt;
              };
              function Rt(Qt, Ut) {
                return Qt + "," + Ut;
              }
              var vt = St(0), $t = function(Qt) {
                Qt.x = Math.max(vt, Qt.x);
              }, Pt = be.pathbar.edgeshape, Xt = be[Ce ? "tiling" : "marker"].pad, Nt = function(Qt) {
                return be.textposition.indexOf(Qt) !== -1;
              }, Gt = Nt("top"), Ft = Nt("left"), Tt = Nt("right"), It = Nt("bottom"), Ct = function(Qt, Ut) {
                var Wt = Qt.x0, rn = Qt.x1, vn = Qt.y0, dn = Qt.y1, Ht = Qt.textBB, jt = Gt || Ut.isHeader && !It ? "start" : It ? "end" : "middle", cn = Nt("right"), wn = Nt("left") || Ut.onPathbar ? -1 : cn ? 1 : 0;
                if (Ut.isHeader) {
                  if ((Wt += (Ce ? Xt : Xt.l) - oe) >= (rn -= (Ce ? Xt : Xt.r) - oe)) {
                    var Rn = (Wt + rn) / 2;
                    Wt = Rn, rn = Rn;
                  }
                  var Pn;
                  It ? vn < (Pn = dn - (Ce ? Xt : Xt.b)) && Pn < dn && (vn = Pn) : vn < (Pn = vn + (Ce ? Xt : Xt.t)) && Pn < dn && (dn = Pn);
                }
                var Ie = se(Wt, rn, vn, dn, Ht, { isHorizontal: !1, constrained: !0, angle: 0, anchor: jt, leftToRight: wn });
                return Ie.fontSize = Ut.fontSize, Ie.targetX = st(Ie.targetX), Ie.targetY = gt(Ie.targetY), isNaN(Ie.targetX) || isNaN(Ie.targetY) ? {} : (Wt !== rn && vn !== dn && ce(be.type, Ie, ge), { scale: Ie.scale, rotate: Ie.rotate, textX: Ie.textX, textY: Ie.textY, anchorX: Ie.anchorX, anchorY: Ie.anchorY, targetX: Ie.targetX, targetY: Ie.targetY });
              }, Kt = function(Qt, Ut) {
                for (var Wt, rn = 0, vn = Qt; !Wt && rn < Me; ) rn++, (vn = vn.parent) ? Wt = dt(vn, Ut) : rn = Me;
                return Wt || {};
              }, Vt = function(Qt, Ut, Wt, rn, vn) {
                var dn, Ht = dt(Qt, Ut);
                if (Ht) dn = Ht;
                else if (Ut) dn = Ge;
                else if (We) if (Qt.parent) {
                  var jt = et || Wt;
                  jt && !Ut ? dn = ze(Qt, jt, rn) : (dn = {}, ae.extendFlat(dn, Kt(Qt, Ut)));
                } else dn = ae.extendFlat({}, Qt), Ce && (vn.orientation === "h" ? vn.flipX ? dn.x0 = Qt.x1 : dn.x1 = 0 : vn.flipY ? dn.y0 = Qt.y1 : dn.y1 = 0);
                else dn = {};
                return ne(dn, { x0: Qt.x0, x1: Qt.x1, y0: Qt.y0, y1: Qt.y1 });
              }, Yt = function(Qt, Ut, Wt, rn) {
                var vn = dt(Qt, Ut), dn = {}, Ht = function(cn, wn, Rn, Pn) {
                  if (wn) return at[de(Se)] || Ge;
                  var Ie = ct[be.level] || Rn;
                  return function(rt) {
                    return rt.data.depth - we.data.depth < Me;
                  }(cn) ? ze(cn, Ie, Pn) : {};
                }(Qt, Ut, Wt, rn);
                ae.extendFlat(dn, { transform: Ct({ x0: Ht.x0, x1: Ht.x1, y0: Ht.y0, y1: Ht.y1, textBB: Qt.textBB, _text: Qt._text }, { isHeader: ee.isHeader(Qt, be) }) }), vn ? dn = vn : Qt.parent && ae.extendFlat(dn, Kt(Qt, Ut));
                var jt = Qt.transform;
                return Qt.x0 !== Qt.x1 && Qt.y0 !== Qt.y1 && ce(be.type, jt, ge), ne(dn, { transform: { scale: jt.scale, rotate: jt.rotate, textX: jt.textX, textY: jt.textY, anchorX: jt.anchorX, anchorY: jt.anchorY, targetX: jt.targetX, targetY: jt.targetY } });
              }, sn = function(Qt, Ut, Wt, rn, vn) {
                var dn = rn[0], Ht = rn[1];
                ke ? Qt.exit().transition().each(function() {
                  var jt = te.select(this);
                  jt.select("path.surface").transition().attrTween("d", function(cn) {
                    var wn = function(Rn, Pn, Ie, rt) {
                      var ft, pt = dt(Rn, Pn);
                      if (Pn) ft = Ge;
                      else {
                        var wt = dt(we, Pn);
                        ft = wt ? ze(Rn, wt, rt) : {};
                      }
                      return ne(pt, ft);
                    }(cn, Ut, 0, [dn, Ht]);
                    return function(Rn) {
                      return vn(wn(Rn));
                    };
                  }), jt.select("g.slicetext").attr("opacity", 0);
                }).remove() : Qt.exit().remove();
              }, hn = function(Qt) {
                var Ut = Qt.transform;
                return Qt.x0 !== Qt.x1 && Qt.y0 !== Qt.y1 && ce(be.type, Ut, ge), ae.getTextTransform({ textX: Ut.textX, textY: Ut.textY, anchorX: Ut.anchorX, anchorY: Ut.anchorY, targetX: Ut.targetX, targetY: Ut.targetY, scale: Ut.scale, rotate: Ut.rotate });
              };
              ke && (xe.each(function(Qt) {
                at[de(Qt)] = { x0: Qt.x0, x1: Qt.x1, y0: Qt.y0, y1: Qt.y1 }, Qt.transform && (at[de(Qt)].transform = { textX: Qt.transform.textX, textY: Qt.transform.textY, anchorX: Qt.transform.anchorX, anchorY: Qt.transform.anchorY, targetX: Qt.transform.targetX, targetY: Qt.transform.targetY, scale: Qt.transform.scale, rotate: Qt.transform.rotate });
              }), Le.each(function(Qt) {
                ct[de(Qt)] = { x0: Qt.x0, x1: Qt.x1, y0: Qt.y0, y1: Qt.y1 }, Qt.transform && (ct[de(Qt)].transform = { textX: Qt.transform.textX, textY: Qt.transform.textY, anchorX: Qt.transform.anchorX, anchorY: Qt.transform.anchorY, targetX: Qt.transform.targetX, targetY: Qt.transform.targetY, scale: Qt.transform.scale, rotate: Qt.transform.rotate }), !We && ee.isEntry(Qt) && (We = Qt);
              })), et = ye(fe, he, we, Le, { width: Pe, height: $e, viewX: st, viewY: gt, pathSlice: function(Qt) {
                var Ut = st(Qt.x0), Wt = st(Qt.x1), rn = gt(Qt.y0), vn = gt(Qt.y1), dn = Wt - Ut, Ht = vn - rn;
                if (!dn || !Ht) return "";
                var jt = be.marker.cornerradius || 0, cn = Math.min(jt, dn / 2, Ht / 2);
                cn && Qt.data && Qt.data.data && Qt.data.data.label && (Gt && (cn = Math.min(cn, Xt.t)), Ft && (cn = Math.min(cn, Xt.l)), Tt && (cn = Math.min(cn, Xt.r)), It && (cn = Math.min(cn, Xt.b)));
                var wn = function(Rn, Pn) {
                  return cn ? "a" + Rt(cn, cn) + " 0 0 1 " + Rt(Rn, Pn) : "";
                };
                return "M" + Rt(Ut, rn + cn) + wn(cn, -cn) + "L" + Rt(Wt - cn, rn) + wn(cn, cn) + "L" + Rt(Wt, vn - cn) + wn(-cn, cn) + "L" + Rt(Ut + cn, vn) + wn(-cn, -cn) + "Z";
              }, toMoveInsideSlice: Ct, prevEntry: We, makeUpdateSliceInterpolator: Vt, makeUpdateTextInterpolator: Yt, handleSlicesExit: sn, hasTransition: ke, strTransform: hn }), be.pathbar.visible ? ue(fe, he, we, xe, { barDifY: Ze, width: qe, height: Xe, viewX: St, viewY: Lt, pathSlice: function(Qt) {
                var Ut = St(Math.max(Math.min(Qt.x0, Qt.x0), 0)), Wt = St(Math.min(Math.max(Qt.x1, Qt.x1), qe)), rn = Lt(Qt.y0), vn = Lt(Qt.y1), dn = Xe / 2, Ht = {}, jt = {};
                Ht.x = Ut, jt.x = Wt, Ht.y = jt.y = (rn + vn) / 2;
                var cn = { x: Ut, y: rn }, wn = { x: Wt, y: rn }, Rn = { x: Wt, y: vn }, Pn = { x: Ut, y: vn };
                return Pt === ">" ? (cn.x -= dn, wn.x -= dn, Rn.x -= dn, Pn.x -= dn) : Pt === "/" ? (Rn.x -= dn, Pn.x -= dn, Ht.x -= dn / 2, jt.x -= dn / 2) : Pt === "\\" ? (cn.x -= dn, wn.x -= dn, Ht.x -= dn / 2, jt.x -= dn / 2) : Pt === "<" && (Ht.x -= dn, jt.x -= dn), $t(cn), $t(Pn), $t(Ht), $t(wn), $t(Rn), $t(jt), "M" + Rt(cn.x, cn.y) + "L" + Rt(wn.x, wn.y) + "L" + Rt(jt.x, jt.y) + "L" + Rt(Rn.x, Rn.y) + "L" + Rt(Pn.x, Pn.y) + "L" + Rt(Ht.x, Ht.y) + "Z";
              }, toMoveInsideSlice: Ct, makeUpdateSliceInterpolator: Vt, makeUpdateTextInterpolator: Yt, handleSlicesExit: sn, hasTransition: ke, strTransform: hn }) : xe.remove();
            };
          }, 92080: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(34809), ae = Z(33108), oe = Z(84102).resizeText, se = Z(72043);
            function ce(le, ue, de, fe, he) {
              var me, pe, ye = (he || {}).hovered, ge = ue.data.data, ve = ge.i, be = ge.color, Ce = ae.isHierarchyRoot(ue), Se = 1;
              if (ye) me = de._hovered.marker.line.color, pe = de._hovered.marker.line.width;
              else if (Ce && be === de.root.color) Se = 100, me = "rgba(0,0,0,0)", pe = 0;
              else if (me = ee.castOption(de, ve, "marker.line.color") || ne.defaultLine, pe = ee.castOption(de, ve, "marker.line.width") || 0, !de._hasColorscale && !ue.onPathbar) {
                var we = de.marker.depthfade;
                if (we) {
                  var Ee, xe = ne.combine(ne.addOpacity(de._backgroundColor, 0.75), be);
                  if (we === !0) {
                    var Le = ae.getMaxDepth(de);
                    Ee = isFinite(Le) ? ae.isLeaf(ue) ? 0 : de._maxVisibleLayers - (ue.data.depth - de._entryDepth) : ue.data.height + 1;
                  } else Ee = ue.data.depth - de._entryDepth, de._atRootLevel || Ee++;
                  if (Ee > 0) for (var Qe = 0; Qe < Ee; Qe++) {
                    var ke = 0.5 * Qe / Ee;
                    be = ne.combine(ne.addOpacity(xe, ke), be);
                  }
                }
              }
              le.call(se, ue, de, fe, be).style("stroke-width", pe).call(ne.stroke, me).style("opacity", Se);
            }
            G.exports = { style: function(le) {
              var ue = le._fullLayout._treemaplayer.selectAll(".trace");
              oe(le, ue, "treemap"), ue.each(function(de) {
                var fe = te.select(this), he = de[0].trace;
                fe.style("opacity", he.opacity), fe.selectAll("path.surface").each(function(me) {
                  te.select(this).call(ce, me, he, le, { hovered: !1 });
                });
              });
            }, styleOne: ce };
          }, 14711: function(G, J, Z) {
            var te = Z(64625), ne = Z(93049).extendFlat, ee = Z(80712).axisHoverFormat;
            G.exports = { y: te.y, x: te.x, x0: te.x0, y0: te.y0, xhoverformat: ee("x"), yhoverformat: ee("y"), name: ne({}, te.name, {}), orientation: ne({}, te.orientation, {}), bandwidth: { valType: "number", min: 0, editType: "calc" }, scalegroup: { valType: "string", dflt: "", editType: "calc" }, scalemode: { valType: "enumerated", values: ["width", "count"], dflt: "width", editType: "calc" }, spanmode: { valType: "enumerated", values: ["soft", "hard", "manual"], dflt: "soft", editType: "calc" }, span: { valType: "info_array", items: [{ valType: "any", editType: "calc" }, { valType: "any", editType: "calc" }], editType: "calc" }, line: { color: { valType: "color", editType: "style" }, width: { valType: "number", min: 0, dflt: 2, editType: "style" }, editType: "plot" }, fillcolor: te.fillcolor, points: ne({}, te.boxpoints, {}), jitter: ne({}, te.jitter, {}), pointpos: ne({}, te.pointpos, {}), width: ne({}, te.width, {}), marker: te.marker, text: te.text, hovertext: te.hovertext, hovertemplate: te.hovertemplate, quartilemethod: te.quartilemethod, box: { visible: { valType: "boolean", dflt: !1, editType: "plot" }, width: { valType: "number", min: 0, max: 1, dflt: 0.25, editType: "plot" }, fillcolor: { valType: "color", editType: "style" }, line: { color: { valType: "color", editType: "style" }, width: { valType: "number", min: 0, editType: "style" }, editType: "style" }, editType: "plot" }, meanline: { visible: { valType: "boolean", dflt: !1, editType: "plot" }, color: { valType: "color", editType: "style" }, width: { valType: "number", min: 0, editType: "style" }, editType: "plot" }, side: { valType: "enumerated", values: ["both", "positive", "negative"], dflt: "both", editType: "calc" }, offsetgroup: te.offsetgroup, alignmentgroup: te.alignmentgroup, selected: te.selected, unselected: te.unselected, hoveron: { valType: "flaglist", flags: ["violins", "points", "kde"], dflt: "violins+points+kde", extras: ["all"], editType: "style" }, zorder: te.zorder };
          }, 88759: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(89429), ae = Z(37881), oe = Z(63821).BADNUM;
            function se(le, ue, de) {
              var fe = ue.max - ue.min;
              if (!fe) return le.bandwidth ? le.bandwidth : 0;
              if (le.bandwidth) return Math.max(le.bandwidth, fe / 1e4);
              var he = de.length, me = te.stdev(de, he - 1, ue.mean);
              return Math.max(function(pe, ye, ge) {
                return 1.059 * Math.min(ye, ge / 1.349) * Math.pow(pe, -0.2);
              }(he, me, ue.q3 - ue.q1), fe / 100);
            }
            function ce(le, ue, de, fe) {
              var he, me = le.spanmode, pe = le.span || [], ye = [ue.min, ue.max], ge = [ue.min - 2 * fe, ue.max + 2 * fe];
              function ve(Ce) {
                var Se = pe[Ce], we = de.type === "multicategory" ? de.r2c(Se) : de.d2c(Se, 0, le[ue.valLetter + "calendar"]);
                return we === oe ? ge[Ce] : we;
              }
              var be = { type: "linear", range: he = me === "soft" ? ge : me === "hard" ? ye : [ve(0), ve(1)] };
              return ne.setConvert(be), be.cleanRange(), he;
            }
            G.exports = function(le, ue) {
              var de = ee(le, ue);
              if (de[0].t.empty) return de;
              for (var fe = le._fullLayout, he = ne.getFromId(le, ue[ue.orientation === "h" ? "xaxis" : "yaxis"]), me = 1 / 0, pe = -1 / 0, ye = 0, ge = 0, ve = 0; ve < de.length; ve++) {
                var be = de[ve], Ce = be.pts.map(ae.extractVal), Se = be.bandwidth = se(ue, be, Ce), we = be.span = ce(ue, be, he, Se);
                if (be.min === be.max && Se === 0) we = be.span = [be.min, be.max], be.density = [{ v: 1, t: we[0] }], be.bandwidth = Se, ye = Math.max(ye, 1);
                else {
                  var Ee = we[1] - we[0], xe = Math.ceil(Ee / (Se / 3)), Le = Ee / xe;
                  if (!isFinite(Le) || !isFinite(xe)) return te.error("Something went wrong with computing the violin span"), de[0].t.empty = !0, de;
                  var Qe = ae.makeKDE(be, ue, Ce);
                  be.density = new Array(xe);
                  for (var ke = 0, Me = we[0]; Me < we[1] + Le / 2; ke++, Me += Le) {
                    var Re = Qe(Me);
                    be.density[ke] = { v: Re, t: Me }, ye = Math.max(ye, Re);
                  }
                }
                ge = Math.max(ge, Ce.length), me = Math.min(me, we[0]), pe = Math.max(pe, we[1]);
              }
              var De = ne.findExtremes(he, [me, pe], { padded: !0 });
              if (ue._extremes[he._id] = De, ue.width) de[0].t.maxKDE = ye;
              else {
                var Pe = fe._violinScaleGroupStats, $e = ue.scalegroup, qe = Pe[$e];
                qe ? (qe.maxKDE = Math.max(qe.maxKDE, ye), qe.maxCount = Math.max(qe.maxCount, ge)) : Pe[$e] = { maxKDE: ye, maxCount: ge };
              }
              return de[0].t.labels.kde = te._(le, "kde:"), de;
            };
          }, 67316: function(G, J, Z) {
            var te = Z(81606).setPositionOffset, ne = ["v", "h"];
            G.exports = function(ee, ae) {
              for (var oe = ee.calcdata, se = ae.xaxis, ce = ae.yaxis, le = 0; le < ne.length; le++) {
                for (var ue = ne[le], de = ue === "h" ? ce : se, fe = [], he = 0; he < oe.length; he++) {
                  var me = oe[he], pe = me[0].t, ye = me[0].trace;
                  ye.visible !== !0 || ye.type !== "violin" || pe.empty || ye.orientation !== ue || ye.xaxis !== se._id || ye.yaxis !== ce._id || fe.push(he);
                }
                te("violin", ee, fe, de);
              }
            };
          }, 10864: function(G, J, Z) {
            var te = Z(34809), ne = Z(78766), ee = Z(62294), ae = Z(14711);
            G.exports = function(oe, se, ce, le) {
              function ue(Ee, xe) {
                return te.coerce(oe, se, ae, Ee, xe);
              }
              function de(Ee, xe) {
                return te.coerce2(oe, se, ae, Ee, xe);
              }
              if (ee.handleSampleDefaults(oe, se, ue, le), se.visible !== !1) {
                ue("bandwidth"), ue("side"), ue("width") || (ue("scalegroup", se.name), ue("scalemode"));
                var fe, he = ue("span");
                Array.isArray(he) && (fe = "manual"), ue("spanmode", fe);
                var me = ue("line.color", (oe.marker || {}).color || ce), pe = ue("line.width"), ye = ue("fillcolor", ne.addOpacity(se.line.color, 0.5));
                ee.handlePointsDefaults(oe, se, ue, { prefix: "" });
                var ge = de("box.width"), ve = de("box.fillcolor", ye), be = de("box.line.color", me), Ce = de("box.line.width", pe);
                ue("box.visible", !!(ge || ve || be || Ce)) || (se.box = { visible: !1 });
                var Se = de("meanline.color", me), we = de("meanline.width", pe);
                ue("meanline.visible", !!(Se || we)) || (se.meanline = { visible: !1 }), ue("quartilemethod"), ue("zorder");
              }
            };
          }, 37881: function(G, J, Z) {
            var te = Z(34809), ne = function(ee) {
              return 1 / Math.sqrt(2 * Math.PI) * Math.exp(-0.5 * ee * ee);
            };
            J.makeKDE = function(ee, ae, oe) {
              var se = oe.length, ce = ne, le = ee.bandwidth, ue = 1 / (se * le);
              return function(de) {
                for (var fe = 0, he = 0; he < se; he++) fe += ce((de - oe[he]) / le);
                return ue * fe;
              };
            }, J.getPositionOnKdePath = function(ee, ae, oe) {
              var se, ce;
              ae.orientation === "h" ? (se = "y", ce = "x") : (se = "x", ce = "y");
              var le = te.findPointOnPath(ee.path, oe, ce, { pathLength: ee.pathLength }), ue = ee.posCenterPx, de = le[se];
              return [de, ae.side === "both" ? 2 * ue - de : ue];
            }, J.getKdeValue = function(ee, ae, oe) {
              var se = ee.pts.map(J.extractVal);
              return J.makeKDE(ee, ae, se)(oe) / ee.posDensityScale;
            }, J.extractVal = function(ee) {
              return ee.v;
            };
          }, 16842: function(G, J, Z) {
            var te = Z(78766), ne = Z(34809), ee = Z(29714), ae = Z(11448), oe = Z(37881);
            G.exports = function(se, ce, le, ue, de) {
              de || (de = {});
              var fe, he, me = de.hoverLayer, pe = se.cd, ye = pe[0].trace, ge = ye.hoveron, ve = ge.indexOf("violins") !== -1, be = ge.indexOf("kde") !== -1, Ce = [];
              if (ve || be) {
                var Se = ae.hoverOnBoxes(se, ce, le, ue);
                if (be && Se.length > 0) {
                  var we, Ee, xe, Le, Qe, ke = se.xa, Me = se.ya;
                  ye.orientation === "h" ? (Qe = ce, we = "y", xe = Me, Ee = "x", Le = ke) : (Qe = le, we = "x", xe = ke, Ee = "y", Le = Me);
                  var Re = pe[se.index];
                  if (Qe >= Re.span[0] && Qe <= Re.span[1]) {
                    var De = ne.extendFlat({}, se), Pe = Le.c2p(Qe, !0), $e = oe.getKdeValue(Re, ye, Qe), qe = oe.getPositionOnKdePath(Re, ye, Pe), Xe = xe._offset, Ve = xe._length;
                    De[we + "0"] = qe[0], De[we + "1"] = qe[1], De[Ee + "0"] = De[Ee + "1"] = Pe, De[Ee + "Label"] = Ee + ": " + ee.hoverLabelText(Le, Qe, ye[Ee + "hoverformat"]) + ", " + pe[0].t.labels.kde + " " + $e.toFixed(3);
                    for (var Ze = 0, Ge = 0; Ge < Se.length; Ge++) if (Se[Ge].attr === "med") {
                      Ze = Ge;
                      break;
                    }
                    De.spikeDistance = Se[Ze].spikeDistance;
                    var ze = we + "Spike";
                    De[ze] = Se[Ze][ze], Se[Ze].spikeDistance = void 0, Se[Ze][ze] = void 0, De.hovertemplate = !1, Ce.push(De), (he = {})[we + "1"] = ne.constrain(Xe + qe[0], Xe, Xe + Ve), he[we + "2"] = ne.constrain(Xe + qe[1], Xe, Xe + Ve), he[Ee + "1"] = he[Ee + "2"] = Le._offset + Pe;
                  }
                }
                ve && (Ce = Ce.concat(Se));
              }
              ge.indexOf("points") !== -1 && (fe = ae.hoverOnPoints(se, ce, le));
              var We = me.selectAll(".violinline-" + ye.uid).data(he ? [0] : []);
              return We.enter().append("line").classed("violinline-" + ye.uid, !0).attr("stroke-width", 1.5), We.exit().remove(), We.attr(he).call(te.stroke, se.color), ue === "closest" ? fe ? [fe] : Ce : (fe && Ce.push(fe), Ce);
            };
          }, 37276: function(G, J, Z) {
            G.exports = { attributes: Z(14711), layoutAttributes: Z(84734), supplyDefaults: Z(10864), crossTraceDefaults: Z(62294).crossTraceDefaults, supplyLayoutDefaults: Z(55145), calc: Z(88759), crossTraceCalc: Z(67316), plot: Z(36769), style: Z(25117), styleOnSelect: Z(9408).styleOnSelect, hoverPoints: Z(16842), selectPoints: Z(72488), moduleType: "trace", name: "violin", basePlotModule: Z(37703), categories: ["cartesian", "svg", "symbols", "oriented", "box-violin", "showLegend", "violinLayout", "zoomScale"], meta: {} };
          }, 84734: function(G, J, Z) {
            var te = Z(64636), ne = Z(34809).extendFlat;
            G.exports = { violinmode: ne({}, te.boxmode, {}), violingap: ne({}, te.boxgap, {}), violingroupgap: ne({}, te.boxgroupgap, {}) };
          }, 55145: function(G, J, Z) {
            var te = Z(34809), ne = Z(84734), ee = Z(65067);
            G.exports = function(ae, oe, se) {
              ee._supply(ae, oe, se, function(ce, le) {
                return te.coerce(ae, oe, ne, ce, le);
              }, "violin");
            };
          }, 36769: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(95419), oe = Z(5525), se = Z(37881);
            G.exports = function(ce, le, ue, de) {
              var fe = ce._context.staticPlot, he = ce._fullLayout, me = le.xaxis, pe = le.yaxis;
              function ye(ge, ve) {
                var be = oe(ge, { xaxis: me, yaxis: pe, trace: ve, connectGaps: !0, baseTolerance: 0.75, shape: "spline", simplify: !0, linearized: !0 });
                return ee.smoothopen(be[0], 1);
              }
              ne.makeTraceGroups(de, ue, "trace violins").each(function(ge) {
                var ve = te.select(this), be = ge[0], Ce = be.t, Se = be.trace;
                if (Se.visible !== !0 || Ce.empty) ve.remove();
                else {
                  var we = Ce.bPos, Ee = Ce.bdPos, xe = le[Ce.valLetter + "axis"], Le = le[Ce.posLetter + "axis"], Qe = Se.side === "both", ke = Qe || Se.side === "positive", Me = Qe || Se.side === "negative", Re = ve.selectAll("path.violin").data(ne.identity);
                  Re.enter().append("path").style("vector-effect", fe ? "none" : "non-scaling-stroke").attr("class", "violin"), Re.exit().remove(), Re.each(function(Ge) {
                    var ze, We, at, ct, et, dt, Je, it, st = te.select(this), gt = Ge.density, bt = gt.length, yt = Le.c2l(Ge.pos + we, !0), St = Le.l2p(yt);
                    if (Se.width) ze = Ce.maxKDE / Ee;
                    else {
                      var Lt = he._violinScaleGroupStats[Se.scalegroup];
                      ze = Se.scalemode === "count" ? Lt.maxKDE / Ee * (Lt.maxCount / Ge.pts.length) : Lt.maxKDE / Ee;
                    }
                    if (ke) {
                      for (Je = new Array(bt), et = 0; et < bt; et++) (it = Je[et] = {})[Ce.posLetter] = yt + gt[et].v / ze, it[Ce.valLetter] = xe.c2l(gt[et].t, !0);
                      We = ye(Je, Se);
                    }
                    if (Me) {
                      for (Je = new Array(bt), dt = 0, et = bt - 1; dt < bt; dt++, et--) (it = Je[dt] = {})[Ce.posLetter] = yt - gt[et].v / ze, it[Ce.valLetter] = xe.c2l(gt[et].t, !0);
                      at = ye(Je, Se);
                    }
                    if (Qe) ct = We + "L" + at.substr(1) + "Z";
                    else {
                      var Rt = [St, xe.c2p(gt[0].t)], vt = [St, xe.c2p(gt[bt - 1].t)];
                      Se.orientation === "h" && (Rt.reverse(), vt.reverse()), ct = ke ? "M" + Rt + "L" + We.substr(1) + "L" + vt : "M" + vt + "L" + at.substr(1) + "L" + Rt;
                    }
                    st.attr("d", ct), Ge.posCenterPx = St, Ge.posDensityScale = ze * Ee, Ge.path = st.node(), Ge.pathLength = Ge.path.getTotalLength() / (Qe ? 2 : 1);
                  });
                  var De, Pe, $e, qe = Se.box, Xe = qe.width, Ve = (qe.line || {}).width;
                  Qe ? (De = Ee * Xe, Pe = 0) : ke ? (De = [0, Ee * Xe / 2], Pe = Ve * { x: 1, y: -1 }[Ce.posLetter]) : (De = [Ee * Xe / 2, 0], Pe = Ve * { x: -1, y: 1 }[Ce.posLetter]), ae.plotBoxAndWhiskers(ve, { pos: Le, val: xe }, Se, { bPos: we, bdPos: De, bPosPxOffset: Pe }), ae.plotBoxMean(ve, { pos: Le, val: xe }, Se, { bPos: we, bdPos: De, bPosPxOffset: Pe }), !Se.box.visible && Se.meanline.visible && ($e = ne.identity);
                  var Ze = ve.selectAll("path.meanline").data($e || []);
                  Ze.enter().append("path").attr("class", "meanline").style("fill", "none").style("vector-effect", fe ? "none" : "non-scaling-stroke"), Ze.exit().remove(), Ze.each(function(Ge) {
                    var ze = xe.c2p(Ge.mean, !0), We = se.getPositionOnKdePath(Ge, Se, ze);
                    te.select(this).attr("d", Se.orientation === "h" ? "M" + ze + "," + We[0] + "V" + We[1] : "M" + We[0] + "," + ze + "H" + We[1]);
                  }), ae.plotPoints(ve, { x: me, y: pe }, Se, Ce);
                }
              });
            };
          }, 25117: function(G, J, Z) {
            var te = Z(45568), ne = Z(78766), ee = Z(9408).stylePoints;
            G.exports = function(ae) {
              var oe = te.select(ae).selectAll("g.trace.violins");
              oe.style("opacity", function(se) {
                return se[0].trace.opacity;
              }), oe.each(function(se) {
                var ce = se[0].trace, le = te.select(this), ue = ce.box || {}, de = ue.line || {}, fe = ce.meanline || {}, he = fe.width;
                le.selectAll("path.violin").style("stroke-width", ce.line.width + "px").call(ne.stroke, ce.line.color).call(ne.fill, ce.fillcolor), le.selectAll("path.box").style("stroke-width", de.width + "px").call(ne.stroke, de.color).call(ne.fill, ue.fillcolor);
                var me = { "stroke-width": he + "px", "stroke-dasharray": 2 * he + "px," + he + "px" };
                le.selectAll("path.mean").style(me).call(ne.stroke, fe.color), le.selectAll("path.meanline").style(me).call(ne.stroke, fe.color), ee(le, ce, ae);
              });
            };
          }, 51526: function(G, J, Z) {
            var te = Z(87163), ne = Z(70252), ee = Z(16131), ae = Z(9829), oe = Z(93049).extendFlat, se = Z(13582).overrideAll, ce = G.exports = se(oe({ x: ne.x, y: ne.y, z: ne.z, value: ne.value, isomin: ne.isomin, isomax: ne.isomax, surface: ne.surface, spaceframe: { show: { valType: "boolean", dflt: !1 }, fill: { valType: "number", min: 0, max: 1, dflt: 1 } }, slices: ne.slices, caps: ne.caps, text: ne.text, hovertext: ne.hovertext, xhoverformat: ne.xhoverformat, yhoverformat: ne.yhoverformat, zhoverformat: ne.zhoverformat, valuehoverformat: ne.valuehoverformat, hovertemplate: ne.hovertemplate }, te("", { colorAttr: "`value`", showScaleDflt: !0, editTypeOverride: "calc" }), { colorbar: ne.colorbar, opacity: ne.opacity, opacityscale: ee.opacityscale, lightposition: ne.lightposition, lighting: ne.lighting, flatshading: ne.flatshading, contour: ne.contour, hoverinfo: oe({}, ae.hoverinfo), showlegend: oe({}, ae.showlegend, { dflt: !1 }) }), "calc", "nested");
            ce.x.editType = ce.y.editType = ce.z.editType = ce.value.editType = "calc+clearAxisTypes", ce.transforms = void 0;
          }, 96496: function(G, J, Z) {
            var te = Z(99098).gl_mesh3d, ne = Z(46998).parseColorScale, ee = Z(34809).isArrayOrTypedArray, ae = Z(55010), oe = Z(88856).extractOpts, se = Z(88239), ce = Z(91370).findNearestOnAxis, le = Z(91370).generateIsoMeshes;
            function ue(fe, he, me) {
              this.scene = fe, this.uid = me, this.mesh = he, this.name = "", this.data = null, this.showContour = !1;
            }
            var de = ue.prototype;
            de.handlePick = function(fe) {
              if (fe.object === this.mesh) {
                var he = fe.data.index, me = this.data._meshX[he], pe = this.data._meshY[he], ye = this.data._meshZ[he], ge = this.data._Ys.length, ve = this.data._Zs.length, be = ce(me, this.data._Xs).id, Ce = ce(pe, this.data._Ys).id, Se = ce(ye, this.data._Zs).id, we = fe.index = Se + ve * Ce + ve * ge * be;
                fe.traceCoordinate = [this.data._meshX[we], this.data._meshY[we], this.data._meshZ[we], this.data._value[we]];
                var Ee = this.data.hovertext || this.data.text;
                return ee(Ee) && Ee[we] !== void 0 ? fe.textLabel = Ee[we] : Ee && (fe.textLabel = Ee), !0;
              }
            }, de.update = function(fe) {
              var he = this.scene, me = he.fullSceneLayout;
              function pe(ve, be, Ce, Se) {
                return be.map(function(we) {
                  return ve.d2l(we, 0, Se) * Ce;
                });
              }
              this.data = le(fe);
              var ye = { positions: se(pe(me.xaxis, fe._meshX, he.dataScale[0], fe.xcalendar), pe(me.yaxis, fe._meshY, he.dataScale[1], fe.ycalendar), pe(me.zaxis, fe._meshZ, he.dataScale[2], fe.zcalendar)), cells: se(fe._meshI, fe._meshJ, fe._meshK), lightPosition: [fe.lightposition.x, fe.lightposition.y, fe.lightposition.z], ambient: fe.lighting.ambient, diffuse: fe.lighting.diffuse, specular: fe.lighting.specular, roughness: fe.lighting.roughness, fresnel: fe.lighting.fresnel, vertexNormalsEpsilon: fe.lighting.vertexnormalsepsilon, faceNormalsEpsilon: fe.lighting.facenormalsepsilon, opacity: fe.opacity, opacityscale: fe.opacityscale, contourEnable: fe.contour.show, contourColor: ae(fe.contour.color).slice(0, 3), contourWidth: fe.contour.width, useFacetNormals: fe.flatshading }, ge = oe(fe);
              ye.vertexIntensity = fe._meshIntensity, ye.vertexIntensityBounds = [ge.min, ge.max], ye.colormap = ne(fe), this.mesh.update(ye);
            }, de.dispose = function() {
              this.scene.glplot.remove(this.mesh), this.mesh.dispose();
            }, G.exports = function(fe, he) {
              var me = fe.glplot.gl, pe = te({ gl: me }), ye = new ue(fe, pe, he.uid);
              return pe._trace = ye, ye.update(he), fe.glplot.add(pe), ye;
            };
          }, 22385: function(G, J, Z) {
            var te = Z(34809), ne = Z(51526), ee = Z(44731).supplyIsoDefaults, ae = Z(65444).opacityscaleDefaults;
            G.exports = function(oe, se, ce, le) {
              function ue(de, fe) {
                return te.coerce(oe, se, ne, de, fe);
              }
              ee(oe, se, ce, le, ue), ae(oe, se, le, ue);
            };
          }, 75703: function(G, J, Z) {
            G.exports = { attributes: Z(51526), supplyDefaults: Z(22385), calc: Z(58988), colorbar: { min: "cmin", max: "cmax" }, plot: Z(96496), moduleType: "trace", name: "volume", basePlotModule: Z(2487), categories: ["gl3d", "showLegend"], meta: {} };
          }, 37832: function(G, J, Z) {
            var te = Z(81481), ne = Z(36640).line, ee = Z(9829), ae = Z(80712).axisHoverFormat, oe = Z(3208).rb, se = Z(3208).ay, ce = Z(82508), le = Z(93049).extendFlat, ue = Z(78766);
            function de(fe) {
              return { marker: { color: le({}, te.marker.color, { arrayOk: !1, editType: "style" }), line: { color: le({}, te.marker.line.color, { arrayOk: !1, editType: "style" }), width: le({}, te.marker.line.width, { arrayOk: !1, editType: "style" }), editType: "style" }, editType: "style" }, editType: "style" };
            }
            G.exports = { measure: { valType: "data_array", dflt: [], editType: "calc" }, base: { valType: "number", dflt: null, arrayOk: !1, editType: "calc" }, x: te.x, x0: te.x0, dx: te.dx, y: te.y, y0: te.y0, dy: te.dy, xperiod: te.xperiod, yperiod: te.yperiod, xperiod0: te.xperiod0, yperiod0: te.yperiod0, xperiodalignment: te.xperiodalignment, yperiodalignment: te.yperiodalignment, xhoverformat: ae("x"), yhoverformat: ae("y"), hovertext: te.hovertext, hovertemplate: oe({}, { keys: ce.eventDataKeys }), hoverinfo: le({}, ee.hoverinfo, { flags: ["name", "x", "y", "text", "initial", "delta", "final"] }), textinfo: { valType: "flaglist", flags: ["label", "text", "initial", "delta", "final"], extras: ["none"], editType: "plot", arrayOk: !1 }, texttemplate: se({ editType: "plot" }, { keys: ce.eventDataKeys.concat(["label"]) }), text: te.text, textposition: te.textposition, insidetextanchor: te.insidetextanchor, textangle: te.textangle, textfont: te.textfont, insidetextfont: te.insidetextfont, outsidetextfont: te.outsidetextfont, constraintext: te.constraintext, cliponaxis: te.cliponaxis, orientation: te.orientation, offset: te.offset, width: te.width, increasing: de(), decreasing: de(), totals: de(), connector: { line: { color: le({}, ne.color, { dflt: ue.defaultLine }), width: le({}, ne.width, { editType: "plot" }), dash: ne.dash, editType: "plot" }, mode: { valType: "enumerated", values: ["spanning", "between"], dflt: "between", editType: "plot" }, visible: { valType: "boolean", dflt: !0, editType: "plot" }, editType: "plot" }, offsetgroup: te.offsetgroup, alignmentgroup: te.alignmentgroup, zorder: te.zorder };
          }, 15e3: function(G, J, Z) {
            var te = Z(29714), ne = Z(40528), ee = Z(34809).mergeArray, ae = Z(48861), oe = Z(63821).BADNUM;
            function se(le) {
              return le === "a" || le === "absolute";
            }
            function ce(le) {
              return le === "t" || le === "total";
            }
            G.exports = function(le, ue) {
              var de, fe, he, me, pe, ye, ge = te.getFromId(le, ue.xaxis || "x"), ve = te.getFromId(le, ue.yaxis || "y");
              ue.orientation === "h" ? (de = ge.makeCalcdata(ue, "x"), he = ve.makeCalcdata(ue, "y"), me = ne(ue, ve, "y", he), pe = !!ue.yperiodalignment, ye = "y") : (de = ve.makeCalcdata(ue, "y"), he = ge.makeCalcdata(ue, "x"), me = ne(ue, ge, "x", he), pe = !!ue.xperiodalignment, ye = "x"), fe = me.vals;
              for (var be, Ce = Math.min(fe.length, de.length), Se = new Array(Ce), we = 0, Ee = !1, xe = 0; xe < Ce; xe++) {
                var Le = de[xe] || 0, Qe = !1;
                (de[xe] !== oe || ce(ue.measure[xe]) || se(ue.measure[xe])) && xe + 1 < Ce && (de[xe + 1] !== oe || ce(ue.measure[xe + 1]) || se(ue.measure[xe + 1])) && (Qe = !0);
                var ke = Se[xe] = { i: xe, p: fe[xe], s: Le, rawS: Le, cNext: Qe };
                se(ue.measure[xe]) ? (we = ke.s, ke.isSum = !0, ke.dir = "totals", ke.s = we) : ce(ue.measure[xe]) ? (ke.isSum = !0, ke.dir = "totals", ke.s = we) : (ke.isSum = !1, ke.dir = ke.rawS < 0 ? "decreasing" : "increasing", be = ke.s, ke.s = we + be, we += be), ke.dir === "totals" && (Ee = !0), pe && (Se[xe].orig_p = he[xe], Se[xe][ye + "End"] = me.ends[xe], Se[xe][ye + "Start"] = me.starts[xe]), ue.ids && (ke.id = String(ue.ids[xe])), ke.v = (ue.base || 0) + we;
              }
              return Se.length && (Se[0].hasTotals = Ee), ee(ue.text, Se, "tx"), ee(ue.hovertext, Se, "htx"), ae(Se, ue), Se;
            };
          }, 82508: function(G) {
            G.exports = { eventDataKeys: ["initial", "delta", "final"] };
          }, 9963: function(G, J, Z) {
            var te = Z(24782).setGroupPositions;
            G.exports = function(ne, ee) {
              var ae, oe, se = ne._fullLayout, ce = ne._fullData, le = ne.calcdata, ue = ee.xaxis, de = ee.yaxis, fe = [], he = [], me = [];
              for (oe = 0; oe < ce.length; oe++) {
                var pe = ce[oe];
                pe.visible === !0 && pe.xaxis === ue._id && pe.yaxis === de._id && pe.type === "waterfall" && (ae = le[oe], pe.orientation === "h" ? me.push(ae) : he.push(ae), fe.push(ae));
              }
              var ye = { mode: se.waterfallmode, norm: se.waterfallnorm, gap: se.waterfallgap, groupgap: se.waterfallgroupgap };
              for (te(ne, ue, de, he, ye), te(ne, de, ue, me, ye), oe = 0; oe < fe.length; oe++) {
                ae = fe[oe];
                for (var ge = 0; ge < ae.length; ge++) {
                  var ve = ae[ge];
                  ve.isSum === !1 && (ve.s0 += ge === 0 ? 0 : ae[ge - 1].s), ge + 1 < ae.length && (ae[ge].nextP0 = ae[ge + 1].p0, ae[ge].nextS0 = ae[ge + 1].s0);
                }
              }
            };
          }, 67199: function(G, J, Z) {
            var te = Z(34809), ne = Z(36301), ee = Z(17550).handleText, ae = Z(99867), oe = Z(99669), se = Z(37832), ce = Z(78766), le = Z(20909), ue = le.INCREASING.COLOR, de = le.DECREASING.COLOR;
            function fe(he, me, pe) {
              he(me + ".marker.color", pe), he(me + ".marker.line.color", ce.defaultLine), he(me + ".marker.line.width");
            }
            G.exports = { supplyDefaults: function(he, me, pe, ye) {
              function ge(be, Ce) {
                return te.coerce(he, me, se, be, Ce);
              }
              if (ae(he, me, ye, ge)) {
                oe(he, me, ye, ge), ge("xhoverformat"), ge("yhoverformat"), ge("measure"), ge("orientation", me.x && !me.y ? "h" : "v"), ge("base"), ge("offset"), ge("width"), ge("text"), ge("hovertext"), ge("hovertemplate");
                var ve = ge("textposition");
                ee(he, me, ye, ge, ve, { moduleHasSelected: !1, moduleHasUnselected: !1, moduleHasConstrain: !0, moduleHasCliponaxis: !0, moduleHasTextangle: !0, moduleHasInsideanchor: !0 }), me.textposition !== "none" && (ge("texttemplate"), me.texttemplate || ge("textinfo")), fe(ge, "increasing", ue), fe(ge, "decreasing", de), fe(ge, "totals", "#4499FF"), ge("connector.visible") && (ge("connector.mode"), ge("connector.line.width") && (ge("connector.line.color"), ge("connector.line.dash"))), ge("zorder");
              } else me.visible = !1;
            }, crossTraceDefaults: function(he, me) {
              var pe, ye;
              function ge(be) {
                return te.coerce(ye._input, ye, se, be);
              }
              if (me.waterfallmode === "group") for (var ve = 0; ve < he.length; ve++) pe = (ye = he[ve])._input, ne(pe, ye, me, ge);
            } };
          }, 64932: function(G) {
            G.exports = function(J, Z) {
              return J.x = "xVal" in Z ? Z.xVal : Z.x, J.y = "yVal" in Z ? Z.yVal : Z.y, "initial" in Z && (J.initial = Z.initial), "delta" in Z && (J.delta = Z.delta), "final" in Z && (J.final = Z.final), Z.xa && (J.xaxis = Z.xa), Z.ya && (J.yaxis = Z.ya), J;
            };
          }, 40943: function(G, J, Z) {
            var te = Z(29714).hoverLabelText, ne = Z(78766).opacity, ee = Z(91664).hoverOnBars, ae = Z(20909), oe = ae.INCREASING.SYMBOL, se = ae.DECREASING.SYMBOL;
            G.exports = function(ce, le, ue, de, fe) {
              var he = ee(ce, le, ue, de, fe);
              if (he) {
                var me = he.cd, pe = me[0].trace, ye = pe.orientation === "h", ge = ye ? "x" : "y", ve = ye ? ce.xa : ce.ya, be = me[he.index], Ce = be.isSum ? be.b + be.s : be.rawS;
                he.initial = be.b + be.s - Ce, he.delta = Ce, he.final = he.initial + he.delta;
                var Se = ke(Math.abs(he.delta));
                he.deltaLabel = Ce < 0 ? "(" + Se + ")" : Se, he.finalLabel = ke(he.final), he.initialLabel = ke(he.initial);
                var we = be.hi || pe.hoverinfo, Ee = [];
                if (we && we !== "none" && we !== "skip") {
                  var xe = we === "all", Le = we.split("+"), Qe = function(Me) {
                    return xe || Le.indexOf(Me) !== -1;
                  };
                  be.isSum || (!Qe("final") || Qe(ye ? "x" : "y") || Ee.push(he.finalLabel), Qe("delta") && (Ce < 0 ? Ee.push(he.deltaLabel + " " + se) : Ee.push(he.deltaLabel + " " + oe)), Qe("initial") && Ee.push("Initial: " + he.initialLabel));
                }
                return Ee.length && (he.extraText = Ee.join("<br>")), he.color = function(Me, Re) {
                  var De = Me[Re.dir].marker, Pe = De.color, $e = De.line.color, qe = De.line.width;
                  return ne(Pe) ? Pe : ne($e) && qe ? $e : void 0;
                }(pe, be), [he];
              }
              function ke(Me) {
                return te(ve, Me, pe[ge + "hoverformat"]);
              }
            };
          }, 38261: function(G, J, Z) {
            G.exports = { attributes: Z(37832), layoutAttributes: Z(579), supplyDefaults: Z(67199).supplyDefaults, crossTraceDefaults: Z(67199).crossTraceDefaults, supplyLayoutDefaults: Z(71492), calc: Z(15e3), crossTraceCalc: Z(9963), plot: Z(71130), style: Z(57256).style, hoverPoints: Z(40943), eventData: Z(64932), selectPoints: Z(88384), moduleType: "trace", name: "waterfall", basePlotModule: Z(37703), categories: ["bar-like", "cartesian", "svg", "oriented", "showLegend", "zoomScale"], meta: {} };
          }, 579: function(G) {
            G.exports = { waterfallmode: { valType: "enumerated", values: ["group", "overlay"], dflt: "group", editType: "calc" }, waterfallgap: { valType: "number", min: 0, max: 1, editType: "calc" }, waterfallgroupgap: { valType: "number", min: 0, max: 1, dflt: 0, editType: "calc" } };
          }, 71492: function(G, J, Z) {
            var te = Z(34809), ne = Z(579);
            G.exports = function(ee, ae, oe) {
              var se = !1;
              function ce(de, fe) {
                return te.coerce(ee, ae, ne, de, fe);
              }
              for (var le = 0; le < oe.length; le++) {
                var ue = oe[le];
                if (ue.visible && ue.type === "waterfall") {
                  se = !0;
                  break;
                }
              }
              se && (ce("waterfallmode"), ce("waterfallgap", 0.2), ce("waterfallgroupgap"));
            };
          }, 71130: function(G, J, Z) {
            var te = Z(45568), ne = Z(34809), ee = Z(62203), ae = Z(63821).BADNUM, oe = Z(32995), se = Z(84102).clearMinTextSize;
            G.exports = function(ce, le, ue, de) {
              var fe = ce._fullLayout;
              se("waterfall", fe), oe.plot(ce, le, ue, de, { mode: fe.waterfallmode, norm: fe.waterfallmode, gap: fe.waterfallgap, groupgap: fe.waterfallgroupgap }), function(he, me, pe, ye) {
                var ge = me.xaxis, ve = me.yaxis;
                ne.makeTraceGroups(ye, pe, "trace bars").each(function(be) {
                  var Ce = te.select(this), Se = be[0].trace, we = ne.ensureSingle(Ce, "g", "lines");
                  if (Se.connector && Se.connector.visible) {
                    var Ee = Se.orientation === "h", xe = Se.connector.mode, Le = we.selectAll("g.line").data(ne.identity);
                    Le.enter().append("g").classed("line", !0), Le.exit().remove();
                    var Qe = Le.size();
                    Le.each(function(ke, Me) {
                      if (Me === Qe - 1 || ke.cNext) {
                        var Re = function(qe, Xe, Ve, Ze) {
                          var Ge = [], ze = [], We = Ze ? Xe : Ve, at = Ze ? Ve : Xe;
                          return Ge[0] = We.c2p(qe.s0, !0), ze[0] = at.c2p(qe.p0, !0), Ge[1] = We.c2p(qe.s1, !0), ze[1] = at.c2p(qe.p1, !0), Ge[2] = We.c2p(qe.nextS0, !0), ze[2] = at.c2p(qe.nextP0, !0), Ze ? [Ge, ze] : [ze, Ge];
                        }(ke, ge, ve, Ee), De = Re[0], Pe = Re[1], $e = "";
                        De[0] !== ae && Pe[0] !== ae && De[1] !== ae && Pe[1] !== ae && (xe === "spanning" && !ke.isSum && Me > 0 && ($e += Ee ? "M" + De[0] + "," + Pe[1] + "V" + Pe[0] : "M" + De[1] + "," + Pe[0] + "H" + De[0]), xe !== "between" && (ke.isSum || Me < Qe - 1) && ($e += Ee ? "M" + De[1] + "," + Pe[0] + "V" + Pe[1] : "M" + De[0] + "," + Pe[1] + "H" + De[1]), De[2] !== ae && Pe[2] !== ae && ($e += Ee ? "M" + De[1] + "," + Pe[1] + "V" + Pe[2] : "M" + De[1] + "," + Pe[1] + "H" + De[2])), $e === "" && ($e = "M0,0Z"), ne.ensureSingle(te.select(this), "path").attr("d", $e).call(ee.setClipUrl, me.layerClipId, he);
                      }
                    });
                  } else we.remove();
                });
              }(ce, le, ue, de);
            };
          }, 57256: function(G, J, Z) {
            var te = Z(45568), ne = Z(62203), ee = Z(78766), ae = Z(20438).DESELECTDIM, oe = Z(6851), se = Z(84102).resizeText, ce = oe.styleTextPoints;
            G.exports = { style: function(le, ue, de) {
              var fe = de || te.select(le).selectAll('g[class^="waterfalllayer"]').selectAll("g.trace");
              se(le, fe, "waterfall"), fe.style("opacity", function(he) {
                return he[0].trace.opacity;
              }), fe.each(function(he) {
                var me = te.select(this), pe = he[0].trace;
                me.selectAll(".point > path").each(function(ye) {
                  if (!ye.isBlank) {
                    var ge = pe[ye.dir].marker;
                    te.select(this).call(ee.fill, ge.color).call(ee.stroke, ge.line.color).call(ne.dashLine, ge.line.dash, ge.line.width).style("opacity", pe.selectedpoints && !ye.selected ? ae : 1);
                  }
                }), ce(me, pe, le), me.selectAll(".lines").each(function() {
                  var ye = pe.connector.line;
                  ne.lineGroupStyle(te.select(this).selectAll("path"), ye.width, ye.color, ye.dash);
                });
              });
            } };
          }, 47908: function(G, J, Z) {
            var te = Z(29714), ne = Z(34809), ee = Z(57297), ae = Z(5086).z, oe = Z(63821).BADNUM;
            J.moduleType = "transform", J.name = "aggregate";
            var se = J.attributes = { enabled: { valType: "boolean", dflt: !0, editType: "calc" }, groups: { valType: "string", strict: !0, noBlank: !0, arrayOk: !0, dflt: "x", editType: "calc" }, aggregations: { _isLinkedToArray: "aggregation", target: { valType: "string", editType: "calc" }, func: { valType: "enumerated", values: ["count", "sum", "avg", "median", "mode", "rms", "stddev", "min", "max", "first", "last", "change", "range"], dflt: "first", editType: "calc" }, funcmode: { valType: "enumerated", values: ["sample", "population"], dflt: "sample", editType: "calc" }, enabled: { valType: "boolean", dflt: !0, editType: "calc" }, editType: "calc" }, editType: "calc" }, ce = se.aggregations;
            function le(he, me, pe, ye) {
              if (ye.enabled) {
                for (var ge = ye.target, ve = ne.nestedProperty(me, ge), be = ve.get(), Ce = function(Ee, xe) {
                  var Le = Ee.func, Qe = xe.d2c, ke = xe.c2d;
                  switch (Le) {
                    case "count":
                      return ue;
                    case "first":
                      return de;
                    case "last":
                      return fe;
                    case "sum":
                      return function(Me, Re) {
                        for (var De = 0, Pe = 0; Pe < Re.length; Pe++) {
                          var $e = Qe(Me[Re[Pe]]);
                          $e !== oe && (De += $e);
                        }
                        return ke(De);
                      };
                    case "avg":
                      return function(Me, Re) {
                        for (var De = 0, Pe = 0, $e = 0; $e < Re.length; $e++) {
                          var qe = Qe(Me[Re[$e]]);
                          qe !== oe && (De += qe, Pe++);
                        }
                        return Pe ? ke(De / Pe) : oe;
                      };
                    case "min":
                      return function(Me, Re) {
                        for (var De = 1 / 0, Pe = 0; Pe < Re.length; Pe++) {
                          var $e = Qe(Me[Re[Pe]]);
                          $e !== oe && (De = Math.min(De, $e));
                        }
                        return De === 1 / 0 ? oe : ke(De);
                      };
                    case "max":
                      return function(Me, Re) {
                        for (var De = -1 / 0, Pe = 0; Pe < Re.length; Pe++) {
                          var $e = Qe(Me[Re[Pe]]);
                          $e !== oe && (De = Math.max(De, $e));
                        }
                        return De === -1 / 0 ? oe : ke(De);
                      };
                    case "range":
                      return function(Me, Re) {
                        for (var De = 1 / 0, Pe = -1 / 0, $e = 0; $e < Re.length; $e++) {
                          var qe = Qe(Me[Re[$e]]);
                          qe !== oe && (De = Math.min(De, qe), Pe = Math.max(Pe, qe));
                        }
                        return Pe === -1 / 0 || De === 1 / 0 ? oe : ke(Pe - De);
                      };
                    case "change":
                      return function(Me, Re) {
                        var De = Qe(Me[Re[0]]), Pe = Qe(Me[Re[Re.length - 1]]);
                        return De === oe || Pe === oe ? oe : ke(Pe - De);
                      };
                    case "median":
                      return function(Me, Re) {
                        for (var De = [], Pe = 0; Pe < Re.length; Pe++) {
                          var $e = Qe(Me[Re[Pe]]);
                          $e !== oe && De.push($e);
                        }
                        if (!De.length) return oe;
                        De.sort(ne.sorterAsc);
                        var qe = (De.length - 1) / 2;
                        return ke((De[Math.floor(qe)] + De[Math.ceil(qe)]) / 2);
                      };
                    case "mode":
                      return function(Me, Re) {
                        for (var De = {}, Pe = 0, $e = oe, qe = 0; qe < Re.length; qe++) {
                          var Xe = Qe(Me[Re[qe]]);
                          if (Xe !== oe) {
                            var Ve = De[Xe] = (De[Xe] || 0) + 1;
                            Ve > Pe && (Pe = Ve, $e = Xe);
                          }
                        }
                        return Pe ? ke($e) : oe;
                      };
                    case "rms":
                      return function(Me, Re) {
                        for (var De = 0, Pe = 0, $e = 0; $e < Re.length; $e++) {
                          var qe = Qe(Me[Re[$e]]);
                          qe !== oe && (De += qe * qe, Pe++);
                        }
                        return Pe ? ke(Math.sqrt(De / Pe)) : oe;
                      };
                    case "stddev":
                      return function(Me, Re) {
                        var De, Pe = 0, $e = 0, qe = 1, Xe = oe;
                        for (De = 0; De < Re.length && Xe === oe; De++) Xe = Qe(Me[Re[De]]);
                        if (Xe === oe) return oe;
                        for (; De < Re.length; De++) {
                          var Ve = Qe(Me[Re[De]]);
                          if (Ve !== oe) {
                            var Ze = Ve - Xe;
                            Pe += Ze, $e += Ze * Ze, qe++;
                          }
                        }
                        var Ge = Ee.funcmode === "sample" ? qe - 1 : qe;
                        return Ge ? Math.sqrt(($e - Pe * Pe / qe) / Ge) : 0;
                      };
                  }
                }(ye, te.getDataConversions(he, me, ge, be)), Se = new Array(pe.length), we = 0; we < pe.length; we++) Se[we] = Ce(be, pe[we]);
                ve.set(Se), ye.func === "count" && ne.pushUnique(me._arrayAttrs, ge);
              }
            }
            function ue(he, me) {
              return me.length;
            }
            function de(he, me) {
              return he[me[0]];
            }
            function fe(he, me) {
              return he[me[me.length - 1]];
            }
            J.supplyDefaults = function(he, me) {
              var pe, ye = {};
              function ge(ke, Me) {
                return ne.coerce(he, ye, se, ke, Me);
              }
              if (!ge("enabled")) return ye;
              var ve = ee.findArrayAttributes(me), be = {};
              for (pe = 0; pe < ve.length; pe++) be[ve[pe]] = 1;
              var Ce = ge("groups");
              if (!Array.isArray(Ce)) {
                if (!be[Ce]) return ye.enabled = !1, ye;
                be[Ce] = 0;
              }
              var Se, we = he.aggregations || [], Ee = ye.aggregations = new Array(we.length);
              function xe(ke, Me) {
                return ne.coerce(we[pe], Se, ce, ke, Me);
              }
              for (pe = 0; pe < we.length; pe++) {
                Se = { _index: pe };
                var Le = xe("target"), Qe = xe("func");
                xe("enabled") && Le && (be[Le] || Qe === "count" && be[Le] === void 0) ? (Qe === "stddev" && xe("funcmode"), be[Le] = 0, Ee[pe] = Se) : Ee[pe] = { enabled: !1, _index: pe };
              }
              for (pe = 0; pe < ve.length; pe++) be[ve[pe]] && Ee.push({ target: ve[pe], func: ce.func.dflt, enabled: !0, _index: -1 });
              return ye;
            }, J.calcTransform = function(he, me, pe) {
              if (pe.enabled) {
                var ye = pe.groups, ge = ne.getTargetArray(me, { target: ye });
                if (ge) {
                  var ve, be, Ce, Se, we = {}, Ee = {}, xe = [], Le = ae(me.transforms, pe), Qe = ge.length;
                  for (me._length && (Qe = Math.min(Qe, me._length)), ve = 0; ve < Qe; ve++) (Ce = we[be = ge[ve]]) === void 0 ? (we[be] = xe.length, Se = [ve], xe.push(Se), Ee[we[be]] = Le(ve)) : (xe[Ce].push(ve), Ee[we[be]] = (Ee[we[be]] || []).concat(Le(ve)));
                  pe._indexToPoints = Ee;
                  var ke = pe.aggregations;
                  for (ve = 0; ve < ke.length; ve++) le(he, me, xe, ke[ve]);
                  typeof ye == "string" && le(he, me, xe, { target: ye, func: "first", enabled: !0 }), me._length = xe.length;
                }
              }
            };
          }, 42849: function(G, J, Z) {
            var te = Z(34809), ne = Z(33626), ee = Z(29714), ae = Z(5086).z, oe = Z(20726), se = oe.COMPARISON_OPS, ce = oe.INTERVAL_OPS, le = oe.SET_OPS;
            J.moduleType = "transform", J.name = "filter", J.attributes = { enabled: { valType: "boolean", dflt: !0, editType: "calc" }, target: { valType: "string", strict: !0, noBlank: !0, arrayOk: !0, dflt: "x", editType: "calc" }, operation: { valType: "enumerated", values: [].concat(se).concat(ce).concat(le), dflt: "=", editType: "calc" }, value: { valType: "any", dflt: 0, editType: "calc" }, preservegaps: { valType: "boolean", dflt: !1, editType: "calc" }, editType: "calc" }, J.supplyDefaults = function(ue) {
              var de = {};
              function fe(pe, ye) {
                return te.coerce(ue, de, J.attributes, pe, ye);
              }
              if (fe("enabled")) {
                var he = fe("target");
                if (te.isArrayOrTypedArray(he) && he.length === 0) return de.enabled = !1, de;
                fe("preservegaps"), fe("operation"), fe("value");
                var me = ne.getComponentMethod("calendars", "handleDefaults");
                me(ue, de, "valuecalendar", null), me(ue, de, "targetcalendar", null);
              }
              return de;
            }, J.calcTransform = function(ue, de, fe) {
              if (fe.enabled) {
                var he = te.getTargetArray(de, fe);
                if (he) {
                  var me = fe.target, pe = he.length;
                  de._length && (pe = Math.min(pe, de._length));
                  var ye = fe.targetcalendar, ge = de._arrayAttrs, ve = fe.preservegaps;
                  if (typeof me == "string") {
                    var be = te.nestedProperty(de, me + "calendar").get();
                    be && (ye = be);
                  }
                  var Ce, Se, we = function(Re, De, Pe) {
                    var $e = Re.operation, qe = Re.value, Xe = te.isArrayOrTypedArray(qe);
                    function Ve(We) {
                      return We.indexOf($e) !== -1;
                    }
                    var Ze, Ge = function(We) {
                      return De(We, 0, Re.valuecalendar);
                    }, ze = function(We) {
                      return De(We, 0, Pe);
                    };
                    switch (Ve(se) ? Ze = Ge(Xe ? qe[0] : qe) : Ve(ce) ? Ze = Xe ? [Ge(qe[0]), Ge(qe[1])] : [Ge(qe), Ge(qe)] : Ve(le) && (Ze = Xe ? qe.map(Ge) : [Ge(qe)]), $e) {
                      case "=":
                        return function(We) {
                          return ze(We) === Ze;
                        };
                      case "!=":
                        return function(We) {
                          return ze(We) !== Ze;
                        };
                      case "<":
                        return function(We) {
                          return ze(We) < Ze;
                        };
                      case "<=":
                        return function(We) {
                          return ze(We) <= Ze;
                        };
                      case ">":
                        return function(We) {
                          return ze(We) > Ze;
                        };
                      case ">=":
                        return function(We) {
                          return ze(We) >= Ze;
                        };
                      case "[]":
                        return function(We) {
                          var at = ze(We);
                          return at >= Ze[0] && at <= Ze[1];
                        };
                      case "()":
                        return function(We) {
                          var at = ze(We);
                          return at > Ze[0] && at < Ze[1];
                        };
                      case "[)":
                        return function(We) {
                          var at = ze(We);
                          return at >= Ze[0] && at < Ze[1];
                        };
                      case "(]":
                        return function(We) {
                          var at = ze(We);
                          return at > Ze[0] && at <= Ze[1];
                        };
                      case "][":
                        return function(We) {
                          var at = ze(We);
                          return at <= Ze[0] || at >= Ze[1];
                        };
                      case ")(":
                        return function(We) {
                          var at = ze(We);
                          return at < Ze[0] || at > Ze[1];
                        };
                      case "](":
                        return function(We) {
                          var at = ze(We);
                          return at <= Ze[0] || at > Ze[1];
                        };
                      case ")[":
                        return function(We) {
                          var at = ze(We);
                          return at < Ze[0] || at >= Ze[1];
                        };
                      case "{}":
                        return function(We) {
                          return Ze.indexOf(ze(We)) !== -1;
                        };
                      case "}{":
                        return function(We) {
                          return Ze.indexOf(ze(We)) === -1;
                        };
                    }
                  }(fe, ee.getDataToCoordFunc(ue, de, me, he), ye), Ee = {}, xe = {}, Le = 0;
                  ve ? (Ce = function(Re) {
                    Ee[Re.astr] = te.extendDeep([], Re.get()), Re.set(new Array(pe));
                  }, Se = function(Re, De) {
                    var Pe = Ee[Re.astr][De];
                    Re.get()[De] = Pe;
                  }) : (Ce = function(Re) {
                    Ee[Re.astr] = te.extendDeep([], Re.get()), Re.set([]);
                  }, Se = function(Re, De) {
                    var Pe = Ee[Re.astr][De];
                    Re.get().push(Pe);
                  }), Me(Ce);
                  for (var Qe = ae(de.transforms, fe), ke = 0; ke < pe; ke++) we(he[ke]) ? (Me(Se, ke), xe[Le++] = Qe(ke)) : ve && Le++;
                  fe._indexToPoints = xe, de._length = Le;
                }
              }
              function Me(Re, De) {
                for (var Pe = 0; Pe < ge.length; Pe++) Re(te.nestedProperty(de, ge[Pe]), De);
              }
            };
          }, 50453: function(G, J, Z) {
            var te = Z(34809), ne = Z(57297), ee = Z(44122), ae = Z(5086).z;
            function oe(se, ce) {
              var le, ue, de, fe, he, me, pe, ye, ge, ve, be = ce.transform, Ce = ce.transformIndex, Se = se.transforms[Ce].groups, we = ae(se.transforms, be);
              if (!te.isArrayOrTypedArray(Se) || Se.length === 0) return [se];
              var Ee = te.filterUnique(Se), xe = new Array(Ee.length), Le = Se.length, Qe = ne.findArrayAttributes(se), ke = be.styles || [], Me = {};
              for (le = 0; le < ke.length; le++) Me[ke[le].target] = ke[le].value;
              be.styles && (ve = te.keyedContainer(be, "styles", "target", "value.name"));
              var Re = {}, De = {};
              for (le = 0; le < Ee.length; le++) {
                Re[me = Ee[le]] = le, De[me] = 0, (pe = xe[le] = te.extendDeepNoArrays({}, se))._group = me, pe.transforms[Ce]._indexToPoints = {};
                var Pe = null;
                for (ve && (Pe = ve.get(me)), pe.name = Pe || Pe === "" ? Pe : te.templateString(be.nameformat, { trace: se.name, group: me }), ye = pe.transforms, pe.transforms = [], ue = 0; ue < ye.length; ue++) pe.transforms[ue] = te.extendDeepNoArrays({}, ye[ue]);
                for (ue = 0; ue < Qe.length; ue++) te.nestedProperty(pe, Qe[ue]).set([]);
              }
              for (de = 0; de < Qe.length; de++) {
                for (fe = Qe[de], ue = 0, ge = []; ue < Ee.length; ue++) ge[ue] = te.nestedProperty(xe[ue], fe).get();
                for (he = te.nestedProperty(se, fe).get(), ue = 0; ue < Le; ue++) ge[Re[Se[ue]]].push(he[ue]);
              }
              for (ue = 0; ue < Le; ue++) (pe = xe[Re[Se[ue]]]).transforms[Ce]._indexToPoints[De[Se[ue]]] = we(ue), De[Se[ue]]++;
              for (le = 0; le < Ee.length; le++) me = Ee[le], pe = xe[le], ee.clearExpandedTraceDefaultColors(pe), pe = te.extendDeepNoArrays(pe, Me[me] || {});
              return xe;
            }
            J.moduleType = "transform", J.name = "groupby", J.attributes = { enabled: { valType: "boolean", dflt: !0, editType: "calc" }, groups: { valType: "data_array", dflt: [], editType: "calc" }, nameformat: { valType: "string", editType: "calc" }, styles: { _isLinkedToArray: "style", target: { valType: "string", editType: "calc" }, value: { valType: "any", dflt: {}, editType: "calc", _compareAsJSON: !0 }, editType: "calc" }, editType: "calc" }, J.supplyDefaults = function(se, ce, le) {
              var ue, de = {};
              function fe(ge, ve) {
                return te.coerce(se, de, J.attributes, ge, ve);
              }
              if (!fe("enabled")) return de;
              fe("groups"), fe("nameformat", le._dataLength > 1 ? "%{group} (%{trace})" : "%{group}");
              var he = se.styles, me = de.styles = [];
              if (he) for (ue = 0; ue < he.length; ue++) {
                var pe = me[ue] = {};
                te.coerce(he[ue], me[ue], J.attributes.styles, "target");
                var ye = te.coerce(he[ue], me[ue], J.attributes.styles, "value");
                te.isPlainObject(ye) ? pe.value = te.extendDeep({}, ye) : ye && delete pe.value;
              }
              return de;
            }, J.transform = function(se, ce) {
              var le, ue, de, fe = [];
              for (ue = 0; ue < se.length; ue++) for (le = oe(se[ue], ce), de = 0; de < le.length; de++) fe.push(le[de]);
              return fe;
            };
          }, 5086: function(G, J) {
            J.z = function(Z, te) {
              for (var ne, ee, ae = 0; ae < Z.length && (ne = Z[ae]) !== te; ae++) ne._indexToPoints && ne.enabled !== !1 && (ee = ne._indexToPoints);
              var oe = ee ? function(se) {
                return ee[se];
              } : function(se) {
                return [se];
              };
              return oe;
            };
          }, 99855: function(G, J, Z) {
            var te = Z(34809), ne = Z(29714), ee = Z(5086).z, ae = Z(63821).BADNUM;
            J.moduleType = "transform", J.name = "sort", J.attributes = { enabled: { valType: "boolean", dflt: !0, editType: "calc" }, target: { valType: "string", strict: !0, noBlank: !0, arrayOk: !0, dflt: "x", editType: "calc" }, order: { valType: "enumerated", values: ["ascending", "descending"], dflt: "ascending", editType: "calc" }, editType: "calc" }, J.supplyDefaults = function(oe) {
              var se = {};
              function ce(le, ue) {
                return te.coerce(oe, se, J.attributes, le, ue);
              }
              return ce("enabled") && (ce("target"), ce("order")), se;
            }, J.calcTransform = function(oe, se, ce) {
              if (ce.enabled) {
                var le = te.getTargetArray(se, ce);
                if (le) {
                  var ue = ce.target, de = le.length;
                  se._length && (de = Math.min(de, se._length));
                  var fe, he, me = se._arrayAttrs, pe = function(Se, we, Ee, xe) {
                    var Le, Qe = new Array(xe), ke = new Array(xe);
                    for (Le = 0; Le < xe; Le++) Qe[Le] = { v: we[Le], i: Le };
                    for (Qe.sort(function(Me, Re) {
                      switch (Me.order) {
                        case "ascending":
                          return function(De, Pe) {
                            var $e = Re(De.v), qe = Re(Pe.v);
                            return $e === ae ? 1 : qe === ae ? -1 : $e - qe;
                          };
                        case "descending":
                          return function(De, Pe) {
                            var $e = Re(De.v), qe = Re(Pe.v);
                            return $e === ae ? 1 : qe === ae ? -1 : qe - $e;
                          };
                      }
                    }(Se, Ee)), Le = 0; Le < xe; Le++) ke[Le] = Qe[Le].i;
                    return ke;
                  }(ce, le, ne.getDataToCoordFunc(oe, se, ue, le), de), ye = ee(se.transforms, ce), ge = {};
                  for (fe = 0; fe < me.length; fe++) {
                    var ve = te.nestedProperty(se, me[fe]), be = ve.get(), Ce = new Array(de);
                    for (he = 0; he < de; he++) Ce[he] = be[pe[he]];
                    ve.set(Ce);
                  }
                  for (he = 0; he < de; he++) ge[he] = ye(pe[he]);
                  ce._indexToPoints = ge, se._length = de;
                }
              }
            };
          }, 29697: function(G, J) {
            J.version = "2.35.3";
          }, 99098: function(G, J, Z) {
            var te = Z(45708).Buffer, ne = Z(33282);
            (function() {
              var ee = { 1964: function(ce, le, ue) {
                ce.exports = { alpha_shape: ue(3502), convex_hull: ue(7352), delaunay_triangulate: ue(7642), gl_cone3d: ue(6405), gl_error3d: ue(9165), gl_heatmap2d: ue(2510), gl_line3d: ue(5714), gl_mesh3d: ue(7201), gl_plot2d: ue(1850), gl_plot3d: ue(4100), gl_pointcloud2d: ue(4696), gl_scatter3d: ue(8418), gl_select_box: ue(3161), gl_spikes2d: ue(4098), gl_streamtube3d: ue(7815), gl_surface3d: ue(9499), ndarray: ue(9618), ndarray_linear_interpolate: ue(4317) };
              }, 4793: function(ce, le, ue) {
                function de(Qt, Ut) {
                  for (var Wt = 0; Wt < Ut.length; Wt++) {
                    var rn = Ut[Wt];
                    rn.enumerable = rn.enumerable || !1, rn.configurable = !0, "value" in rn && (rn.writable = !0), Object.defineProperty(Qt, fe(rn.key), rn);
                  }
                }
                function fe(Qt) {
                  var Ut = function(Wt, rn) {
                    if (ye(Wt) != "object" || !Wt) return Wt;
                    var vn = Wt[Symbol.toPrimitive];
                    if (vn !== void 0) {
                      var dn = vn.call(Wt, "string");
                      if (ye(dn) != "object") return dn;
                      throw new TypeError("@@toPrimitive must return a primitive value.");
                    }
                    return String(Wt);
                  }(Qt);
                  return ye(Ut) == "symbol" ? Ut : Ut + "";
                }
                function he() {
                  try {
                    var Qt = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {
                    }));
                  } catch {
                  }
                  return (he = function() {
                    return !!Qt;
                  })();
                }
                function me(Qt) {
                  return me = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function(Ut) {
                    return Ut.__proto__ || Object.getPrototypeOf(Ut);
                  }, me(Qt);
                }
                function pe(Qt, Ut) {
                  return pe = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(Wt, rn) {
                    return Wt.__proto__ = rn, Wt;
                  }, pe(Qt, Ut);
                }
                function ye(Qt) {
                  return ye = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(Ut) {
                    return typeof Ut;
                  } : function(Ut) {
                    return Ut && typeof Symbol == "function" && Ut.constructor === Symbol && Ut !== Symbol.prototype ? "symbol" : typeof Ut;
                  }, ye(Qt);
                }
                var ge = ue(7507), ve = ue(3778), be = typeof Symbol == "function" && typeof Symbol.for == "function" ? Symbol.for("nodejs.util.inspect.custom") : null;
                le.hp = we, le.IS = 50;
                var Ce = 2147483647;
                function Se(Qt) {
                  if (Qt > Ce) throw new RangeError('The value "' + Qt + '" is invalid for option "size"');
                  var Ut = new Uint8Array(Qt);
                  return Object.setPrototypeOf(Ut, we.prototype), Ut;
                }
                function we(Qt, Ut, Wt) {
                  if (typeof Qt == "number") {
                    if (typeof Ut == "string") throw new TypeError('The "string" argument must be of type string. Received type number');
                    return Le(Qt);
                  }
                  return Ee(Qt, Ut, Wt);
                }
                function Ee(Qt, Ut, Wt) {
                  if (typeof Qt == "string") return function(dn, Ht) {
                    if (typeof Ht == "string" && Ht !== "" || (Ht = "utf8"), !we.isEncoding(Ht)) throw new TypeError("Unknown encoding: " + Ht);
                    var jt = 0 | Re(dn, Ht), cn = Se(jt), wn = cn.write(dn, Ht);
                    return wn !== jt && (cn = cn.slice(0, wn)), cn;
                  }(Qt, Ut);
                  if (ArrayBuffer.isView(Qt)) return function(dn) {
                    if (Kt(dn, Uint8Array)) {
                      var Ht = new Uint8Array(dn);
                      return ke(Ht.buffer, Ht.byteOffset, Ht.byteLength);
                    }
                    return Qe(dn);
                  }(Qt);
                  if (Qt == null) throw new TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type " + ye(Qt));
                  if (Kt(Qt, ArrayBuffer) || Qt && Kt(Qt.buffer, ArrayBuffer) || typeof SharedArrayBuffer < "u" && (Kt(Qt, SharedArrayBuffer) || Qt && Kt(Qt.buffer, SharedArrayBuffer))) return ke(Qt, Ut, Wt);
                  if (typeof Qt == "number") throw new TypeError('The "value" argument must not be of type number. Received type number');
                  var rn = Qt.valueOf && Qt.valueOf();
                  if (rn != null && rn !== Qt) return we.from(rn, Ut, Wt);
                  var vn = function(dn) {
                    if (we.isBuffer(dn)) {
                      var Ht = 0 | Me(dn.length), jt = Se(Ht);
                      return jt.length === 0 || dn.copy(jt, 0, 0, Ht), jt;
                    }
                    return dn.length !== void 0 ? typeof dn.length != "number" || Vt(dn.length) ? Se(0) : Qe(dn) : dn.type === "Buffer" && Array.isArray(dn.data) ? Qe(dn.data) : void 0;
                  }(Qt);
                  if (vn) return vn;
                  if (typeof Symbol < "u" && Symbol.toPrimitive != null && typeof Qt[Symbol.toPrimitive] == "function") return we.from(Qt[Symbol.toPrimitive]("string"), Ut, Wt);
                  throw new TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type " + ye(Qt));
                }
                function xe(Qt) {
                  if (typeof Qt != "number") throw new TypeError('"size" argument must be of type number');
                  if (Qt < 0) throw new RangeError('The value "' + Qt + '" is invalid for option "size"');
                }
                function Le(Qt) {
                  return xe(Qt), Se(Qt < 0 ? 0 : 0 | Me(Qt));
                }
                function Qe(Qt) {
                  for (var Ut = Qt.length < 0 ? 0 : 0 | Me(Qt.length), Wt = Se(Ut), rn = 0; rn < Ut; rn += 1) Wt[rn] = 255 & Qt[rn];
                  return Wt;
                }
                function ke(Qt, Ut, Wt) {
                  if (Ut < 0 || Qt.byteLength < Ut) throw new RangeError('"offset" is outside of buffer bounds');
                  if (Qt.byteLength < Ut + (Wt || 0)) throw new RangeError('"length" is outside of buffer bounds');
                  var rn;
                  return rn = Ut === void 0 && Wt === void 0 ? new Uint8Array(Qt) : Wt === void 0 ? new Uint8Array(Qt, Ut) : new Uint8Array(Qt, Ut, Wt), Object.setPrototypeOf(rn, we.prototype), rn;
                }
                function Me(Qt) {
                  if (Qt >= Ce) throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x" + Ce.toString(16) + " bytes");
                  return 0 | Qt;
                }
                function Re(Qt, Ut) {
                  if (we.isBuffer(Qt)) return Qt.length;
                  if (ArrayBuffer.isView(Qt) || Kt(Qt, ArrayBuffer)) return Qt.byteLength;
                  if (typeof Qt != "string") throw new TypeError('The "string" argument must be one of type string, Buffer, or ArrayBuffer. Received type ' + ye(Qt));
                  var Wt = Qt.length, rn = arguments.length > 2 && arguments[2] === !0;
                  if (!rn && Wt === 0) return 0;
                  for (var vn = !1; ; ) switch (Ut) {
                    case "ascii":
                    case "latin1":
                    case "binary":
                      return Wt;
                    case "utf8":
                    case "utf-8":
                      return Tt(Qt).length;
                    case "ucs2":
                    case "ucs-2":
                    case "utf16le":
                    case "utf-16le":
                      return 2 * Wt;
                    case "hex":
                      return Wt >>> 1;
                    case "base64":
                      return It(Qt).length;
                    default:
                      if (vn) return rn ? -1 : Tt(Qt).length;
                      Ut = ("" + Ut).toLowerCase(), vn = !0;
                  }
                }
                function De(Qt, Ut, Wt) {
                  var rn = !1;
                  if ((Ut === void 0 || Ut < 0) && (Ut = 0), Ut > this.length || ((Wt === void 0 || Wt > this.length) && (Wt = this.length), Wt <= 0) || (Wt >>>= 0) <= (Ut >>>= 0)) return "";
                  for (Qt || (Qt = "utf8"); ; ) switch (Qt) {
                    case "hex":
                      return Je(this, Ut, Wt);
                    case "utf8":
                    case "utf-8":
                      return at(this, Ut, Wt);
                    case "ascii":
                      return et(this, Ut, Wt);
                    case "latin1":
                    case "binary":
                      return dt(this, Ut, Wt);
                    case "base64":
                      return We(this, Ut, Wt);
                    case "ucs2":
                    case "ucs-2":
                    case "utf16le":
                    case "utf-16le":
                      return it(this, Ut, Wt);
                    default:
                      if (rn) throw new TypeError("Unknown encoding: " + Qt);
                      Qt = (Qt + "").toLowerCase(), rn = !0;
                  }
                }
                function Pe(Qt, Ut, Wt) {
                  var rn = Qt[Ut];
                  Qt[Ut] = Qt[Wt], Qt[Wt] = rn;
                }
                function $e(Qt, Ut, Wt, rn, vn) {
                  if (Qt.length === 0) return -1;
                  if (typeof Wt == "string" ? (rn = Wt, Wt = 0) : Wt > 2147483647 ? Wt = 2147483647 : Wt < -2147483648 && (Wt = -2147483648), Vt(Wt = +Wt) && (Wt = vn ? 0 : Qt.length - 1), Wt < 0 && (Wt = Qt.length + Wt), Wt >= Qt.length) {
                    if (vn) return -1;
                    Wt = Qt.length - 1;
                  } else if (Wt < 0) {
                    if (!vn) return -1;
                    Wt = 0;
                  }
                  if (typeof Ut == "string" && (Ut = we.from(Ut, rn)), we.isBuffer(Ut)) return Ut.length === 0 ? -1 : qe(Qt, Ut, Wt, rn, vn);
                  if (typeof Ut == "number") return Ut &= 255, typeof Uint8Array.prototype.indexOf == "function" ? vn ? Uint8Array.prototype.indexOf.call(Qt, Ut, Wt) : Uint8Array.prototype.lastIndexOf.call(Qt, Ut, Wt) : qe(Qt, [Ut], Wt, rn, vn);
                  throw new TypeError("val must be string, number or Buffer");
                }
                function qe(Qt, Ut, Wt, rn, vn) {
                  var dn, Ht = 1, jt = Qt.length, cn = Ut.length;
                  if (rn !== void 0 && ((rn = String(rn).toLowerCase()) === "ucs2" || rn === "ucs-2" || rn === "utf16le" || rn === "utf-16le")) {
                    if (Qt.length < 2 || Ut.length < 2) return -1;
                    Ht = 2, jt /= 2, cn /= 2, Wt /= 2;
                  }
                  function wn(rt, ft) {
                    return Ht === 1 ? rt[ft] : rt.readUInt16BE(ft * Ht);
                  }
                  if (vn) {
                    var Rn = -1;
                    for (dn = Wt; dn < jt; dn++) if (wn(Qt, dn) === wn(Ut, Rn === -1 ? 0 : dn - Rn)) {
                      if (Rn === -1 && (Rn = dn), dn - Rn + 1 === cn) return Rn * Ht;
                    } else Rn !== -1 && (dn -= dn - Rn), Rn = -1;
                  } else for (Wt + cn > jt && (Wt = jt - cn), dn = Wt; dn >= 0; dn--) {
                    for (var Pn = !0, Ie = 0; Ie < cn; Ie++) if (wn(Qt, dn + Ie) !== wn(Ut, Ie)) {
                      Pn = !1;
                      break;
                    }
                    if (Pn) return dn;
                  }
                  return -1;
                }
                function Xe(Qt, Ut, Wt, rn) {
                  Wt = Number(Wt) || 0;
                  var vn = Qt.length - Wt;
                  rn ? (rn = Number(rn)) > vn && (rn = vn) : rn = vn;
                  var dn, Ht = Ut.length;
                  for (rn > Ht / 2 && (rn = Ht / 2), dn = 0; dn < rn; ++dn) {
                    var jt = parseInt(Ut.substr(2 * dn, 2), 16);
                    if (Vt(jt)) return dn;
                    Qt[Wt + dn] = jt;
                  }
                  return dn;
                }
                function Ve(Qt, Ut, Wt, rn) {
                  return Ct(Tt(Ut, Qt.length - Wt), Qt, Wt, rn);
                }
                function Ze(Qt, Ut, Wt, rn) {
                  return Ct(function(vn) {
                    for (var dn = [], Ht = 0; Ht < vn.length; ++Ht) dn.push(255 & vn.charCodeAt(Ht));
                    return dn;
                  }(Ut), Qt, Wt, rn);
                }
                function Ge(Qt, Ut, Wt, rn) {
                  return Ct(It(Ut), Qt, Wt, rn);
                }
                function ze(Qt, Ut, Wt, rn) {
                  return Ct(function(vn, dn) {
                    for (var Ht, jt, cn, wn = [], Rn = 0; Rn < vn.length && !((dn -= 2) < 0); ++Rn) jt = (Ht = vn.charCodeAt(Rn)) >> 8, cn = Ht % 256, wn.push(cn), wn.push(jt);
                    return wn;
                  }(Ut, Qt.length - Wt), Qt, Wt, rn);
                }
                function We(Qt, Ut, Wt) {
                  return Ut === 0 && Wt === Qt.length ? ge.fromByteArray(Qt) : ge.fromByteArray(Qt.slice(Ut, Wt));
                }
                function at(Qt, Ut, Wt) {
                  Wt = Math.min(Qt.length, Wt);
                  for (var rn = [], vn = Ut; vn < Wt; ) {
                    var dn = Qt[vn], Ht = null, jt = dn > 239 ? 4 : dn > 223 ? 3 : dn > 191 ? 2 : 1;
                    if (vn + jt <= Wt) {
                      var cn = void 0, wn = void 0, Rn = void 0, Pn = void 0;
                      switch (jt) {
                        case 1:
                          dn < 128 && (Ht = dn);
                          break;
                        case 2:
                          (192 & (cn = Qt[vn + 1])) == 128 && (Pn = (31 & dn) << 6 | 63 & cn) > 127 && (Ht = Pn);
                          break;
                        case 3:
                          cn = Qt[vn + 1], wn = Qt[vn + 2], (192 & cn) == 128 && (192 & wn) == 128 && (Pn = (15 & dn) << 12 | (63 & cn) << 6 | 63 & wn) > 2047 && (Pn < 55296 || Pn > 57343) && (Ht = Pn);
                          break;
                        case 4:
                          cn = Qt[vn + 1], wn = Qt[vn + 2], Rn = Qt[vn + 3], (192 & cn) == 128 && (192 & wn) == 128 && (192 & Rn) == 128 && (Pn = (15 & dn) << 18 | (63 & cn) << 12 | (63 & wn) << 6 | 63 & Rn) > 65535 && Pn < 1114112 && (Ht = Pn);
                      }
                    }
                    Ht === null ? (Ht = 65533, jt = 1) : Ht > 65535 && (Ht -= 65536, rn.push(Ht >>> 10 & 1023 | 55296), Ht = 56320 | 1023 & Ht), rn.push(Ht), vn += jt;
                  }
                  return function(Ie) {
                    var rt = Ie.length;
                    if (rt <= ct) return String.fromCharCode.apply(String, Ie);
                    for (var ft = "", pt = 0; pt < rt; ) ft += String.fromCharCode.apply(String, Ie.slice(pt, pt += ct));
                    return ft;
                  }(rn);
                }
                we.TYPED_ARRAY_SUPPORT = function() {
                  try {
                    var Qt = new Uint8Array(1), Ut = { foo: function() {
                      return 42;
                    } };
                    return Object.setPrototypeOf(Ut, Uint8Array.prototype), Object.setPrototypeOf(Qt, Ut), Qt.foo() === 42;
                  } catch {
                    return !1;
                  }
                }(), we.TYPED_ARRAY_SUPPORT || typeof console > "u" || typeof console.error != "function" || console.error("This browser lacks typed array (Uint8Array) support which is required by `buffer` v5.x. Use `buffer` v4.x if you require old browser support."), Object.defineProperty(we.prototype, "parent", { enumerable: !0, get: function() {
                  if (we.isBuffer(this)) return this.buffer;
                } }), Object.defineProperty(we.prototype, "offset", { enumerable: !0, get: function() {
                  if (we.isBuffer(this)) return this.byteOffset;
                } }), we.poolSize = 8192, we.from = function(Qt, Ut, Wt) {
                  return Ee(Qt, Ut, Wt);
                }, Object.setPrototypeOf(we.prototype, Uint8Array.prototype), Object.setPrototypeOf(we, Uint8Array), we.alloc = function(Qt, Ut, Wt) {
                  return function(rn, vn, dn) {
                    return xe(rn), rn <= 0 ? Se(rn) : vn !== void 0 ? typeof dn == "string" ? Se(rn).fill(vn, dn) : Se(rn).fill(vn) : Se(rn);
                  }(Qt, Ut, Wt);
                }, we.allocUnsafe = function(Qt) {
                  return Le(Qt);
                }, we.allocUnsafeSlow = function(Qt) {
                  return Le(Qt);
                }, we.isBuffer = function(Qt) {
                  return Qt != null && Qt._isBuffer === !0 && Qt !== we.prototype;
                }, we.compare = function(Qt, Ut) {
                  if (Kt(Qt, Uint8Array) && (Qt = we.from(Qt, Qt.offset, Qt.byteLength)), Kt(Ut, Uint8Array) && (Ut = we.from(Ut, Ut.offset, Ut.byteLength)), !we.isBuffer(Qt) || !we.isBuffer(Ut)) throw new TypeError('The "buf1", "buf2" arguments must be one of type Buffer or Uint8Array');
                  if (Qt === Ut) return 0;
                  for (var Wt = Qt.length, rn = Ut.length, vn = 0, dn = Math.min(Wt, rn); vn < dn; ++vn) if (Qt[vn] !== Ut[vn]) {
                    Wt = Qt[vn], rn = Ut[vn];
                    break;
                  }
                  return Wt < rn ? -1 : rn < Wt ? 1 : 0;
                }, we.isEncoding = function(Qt) {
                  switch (String(Qt).toLowerCase()) {
                    case "hex":
                    case "utf8":
                    case "utf-8":
                    case "ascii":
                    case "latin1":
                    case "binary":
                    case "base64":
                    case "ucs2":
                    case "ucs-2":
                    case "utf16le":
                    case "utf-16le":
                      return !0;
                    default:
                      return !1;
                  }
                }, we.concat = function(Qt, Ut) {
                  if (!Array.isArray(Qt)) throw new TypeError('"list" argument must be an Array of Buffers');
                  if (Qt.length === 0) return we.alloc(0);
                  var Wt;
                  if (Ut === void 0) for (Ut = 0, Wt = 0; Wt < Qt.length; ++Wt) Ut += Qt[Wt].length;
                  var rn = we.allocUnsafe(Ut), vn = 0;
                  for (Wt = 0; Wt < Qt.length; ++Wt) {
                    var dn = Qt[Wt];
                    if (Kt(dn, Uint8Array)) vn + dn.length > rn.length ? (we.isBuffer(dn) || (dn = we.from(dn)), dn.copy(rn, vn)) : Uint8Array.prototype.set.call(rn, dn, vn);
                    else {
                      if (!we.isBuffer(dn)) throw new TypeError('"list" argument must be an Array of Buffers');
                      dn.copy(rn, vn);
                    }
                    vn += dn.length;
                  }
                  return rn;
                }, we.byteLength = Re, we.prototype._isBuffer = !0, we.prototype.swap16 = function() {
                  var Qt = this.length;
                  if (Qt % 2 != 0) throw new RangeError("Buffer size must be a multiple of 16-bits");
                  for (var Ut = 0; Ut < Qt; Ut += 2) Pe(this, Ut, Ut + 1);
                  return this;
                }, we.prototype.swap32 = function() {
                  var Qt = this.length;
                  if (Qt % 4 != 0) throw new RangeError("Buffer size must be a multiple of 32-bits");
                  for (var Ut = 0; Ut < Qt; Ut += 4) Pe(this, Ut, Ut + 3), Pe(this, Ut + 1, Ut + 2);
                  return this;
                }, we.prototype.swap64 = function() {
                  var Qt = this.length;
                  if (Qt % 8 != 0) throw new RangeError("Buffer size must be a multiple of 64-bits");
                  for (var Ut = 0; Ut < Qt; Ut += 8) Pe(this, Ut, Ut + 7), Pe(this, Ut + 1, Ut + 6), Pe(this, Ut + 2, Ut + 5), Pe(this, Ut + 3, Ut + 4);
                  return this;
                }, we.prototype.toString = function() {
                  var Qt = this.length;
                  return Qt === 0 ? "" : arguments.length === 0 ? at(this, 0, Qt) : De.apply(this, arguments);
                }, we.prototype.toLocaleString = we.prototype.toString, we.prototype.equals = function(Qt) {
                  if (!we.isBuffer(Qt)) throw new TypeError("Argument must be a Buffer");
                  return this === Qt || we.compare(this, Qt) === 0;
                }, we.prototype.inspect = function() {
                  var Qt = "", Ut = le.IS;
                  return Qt = this.toString("hex", 0, Ut).replace(/(.{2})/g, "$1 ").trim(), this.length > Ut && (Qt += " ... "), "<Buffer " + Qt + ">";
                }, be && (we.prototype[be] = we.prototype.inspect), we.prototype.compare = function(Qt, Ut, Wt, rn, vn) {
                  if (Kt(Qt, Uint8Array) && (Qt = we.from(Qt, Qt.offset, Qt.byteLength)), !we.isBuffer(Qt)) throw new TypeError('The "target" argument must be one of type Buffer or Uint8Array. Received type ' + ye(Qt));
                  if (Ut === void 0 && (Ut = 0), Wt === void 0 && (Wt = Qt ? Qt.length : 0), rn === void 0 && (rn = 0), vn === void 0 && (vn = this.length), Ut < 0 || Wt > Qt.length || rn < 0 || vn > this.length) throw new RangeError("out of range index");
                  if (rn >= vn && Ut >= Wt) return 0;
                  if (rn >= vn) return -1;
                  if (Ut >= Wt) return 1;
                  if (this === Qt) return 0;
                  for (var dn = (vn >>>= 0) - (rn >>>= 0), Ht = (Wt >>>= 0) - (Ut >>>= 0), jt = Math.min(dn, Ht), cn = this.slice(rn, vn), wn = Qt.slice(Ut, Wt), Rn = 0; Rn < jt; ++Rn) if (cn[Rn] !== wn[Rn]) {
                    dn = cn[Rn], Ht = wn[Rn];
                    break;
                  }
                  return dn < Ht ? -1 : Ht < dn ? 1 : 0;
                }, we.prototype.includes = function(Qt, Ut, Wt) {
                  return this.indexOf(Qt, Ut, Wt) !== -1;
                }, we.prototype.indexOf = function(Qt, Ut, Wt) {
                  return $e(this, Qt, Ut, Wt, !0);
                }, we.prototype.lastIndexOf = function(Qt, Ut, Wt) {
                  return $e(this, Qt, Ut, Wt, !1);
                }, we.prototype.write = function(Qt, Ut, Wt, rn) {
                  if (Ut === void 0) rn = "utf8", Wt = this.length, Ut = 0;
                  else if (Wt === void 0 && typeof Ut == "string") rn = Ut, Wt = this.length, Ut = 0;
                  else {
                    if (!isFinite(Ut)) throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");
                    Ut >>>= 0, isFinite(Wt) ? (Wt >>>= 0, rn === void 0 && (rn = "utf8")) : (rn = Wt, Wt = void 0);
                  }
                  var vn = this.length - Ut;
                  if ((Wt === void 0 || Wt > vn) && (Wt = vn), Qt.length > 0 && (Wt < 0 || Ut < 0) || Ut > this.length) throw new RangeError("Attempt to write outside buffer bounds");
                  rn || (rn = "utf8");
                  for (var dn = !1; ; ) switch (rn) {
                    case "hex":
                      return Xe(this, Qt, Ut, Wt);
                    case "utf8":
                    case "utf-8":
                      return Ve(this, Qt, Ut, Wt);
                    case "ascii":
                    case "latin1":
                    case "binary":
                      return Ze(this, Qt, Ut, Wt);
                    case "base64":
                      return Ge(this, Qt, Ut, Wt);
                    case "ucs2":
                    case "ucs-2":
                    case "utf16le":
                    case "utf-16le":
                      return ze(this, Qt, Ut, Wt);
                    default:
                      if (dn) throw new TypeError("Unknown encoding: " + rn);
                      rn = ("" + rn).toLowerCase(), dn = !0;
                  }
                }, we.prototype.toJSON = function() {
                  return { type: "Buffer", data: Array.prototype.slice.call(this._arr || this, 0) };
                };
                var ct = 4096;
                function et(Qt, Ut, Wt) {
                  var rn = "";
                  Wt = Math.min(Qt.length, Wt);
                  for (var vn = Ut; vn < Wt; ++vn) rn += String.fromCharCode(127 & Qt[vn]);
                  return rn;
                }
                function dt(Qt, Ut, Wt) {
                  var rn = "";
                  Wt = Math.min(Qt.length, Wt);
                  for (var vn = Ut; vn < Wt; ++vn) rn += String.fromCharCode(Qt[vn]);
                  return rn;
                }
                function Je(Qt, Ut, Wt) {
                  var rn = Qt.length;
                  (!Ut || Ut < 0) && (Ut = 0), (!Wt || Wt < 0 || Wt > rn) && (Wt = rn);
                  for (var vn = "", dn = Ut; dn < Wt; ++dn) vn += Yt[Qt[dn]];
                  return vn;
                }
                function it(Qt, Ut, Wt) {
                  for (var rn = Qt.slice(Ut, Wt), vn = "", dn = 0; dn < rn.length - 1; dn += 2) vn += String.fromCharCode(rn[dn] + 256 * rn[dn + 1]);
                  return vn;
                }
                function st(Qt, Ut, Wt) {
                  if (Qt % 1 != 0 || Qt < 0) throw new RangeError("offset is not uint");
                  if (Qt + Ut > Wt) throw new RangeError("Trying to access beyond buffer length");
                }
                function gt(Qt, Ut, Wt, rn, vn, dn) {
                  if (!we.isBuffer(Qt)) throw new TypeError('"buffer" argument must be a Buffer instance');
                  if (Ut > vn || Ut < dn) throw new RangeError('"value" argument is out of bounds');
                  if (Wt + rn > Qt.length) throw new RangeError("Index out of range");
                }
                function bt(Qt, Ut, Wt, rn, vn) {
                  Xt(Ut, rn, vn, Qt, Wt, 7);
                  var dn = Number(Ut & BigInt(4294967295));
                  Qt[Wt++] = dn, dn >>= 8, Qt[Wt++] = dn, dn >>= 8, Qt[Wt++] = dn, dn >>= 8, Qt[Wt++] = dn;
                  var Ht = Number(Ut >> BigInt(32) & BigInt(4294967295));
                  return Qt[Wt++] = Ht, Ht >>= 8, Qt[Wt++] = Ht, Ht >>= 8, Qt[Wt++] = Ht, Ht >>= 8, Qt[Wt++] = Ht, Wt;
                }
                function yt(Qt, Ut, Wt, rn, vn) {
                  Xt(Ut, rn, vn, Qt, Wt, 7);
                  var dn = Number(Ut & BigInt(4294967295));
                  Qt[Wt + 7] = dn, dn >>= 8, Qt[Wt + 6] = dn, dn >>= 8, Qt[Wt + 5] = dn, dn >>= 8, Qt[Wt + 4] = dn;
                  var Ht = Number(Ut >> BigInt(32) & BigInt(4294967295));
                  return Qt[Wt + 3] = Ht, Ht >>= 8, Qt[Wt + 2] = Ht, Ht >>= 8, Qt[Wt + 1] = Ht, Ht >>= 8, Qt[Wt] = Ht, Wt + 8;
                }
                function St(Qt, Ut, Wt, rn, vn, dn) {
                  if (Wt + rn > Qt.length) throw new RangeError("Index out of range");
                  if (Wt < 0) throw new RangeError("Index out of range");
                }
                function Lt(Qt, Ut, Wt, rn, vn) {
                  return Ut = +Ut, Wt >>>= 0, vn || St(Qt, 0, Wt, 4), ve.write(Qt, Ut, Wt, rn, 23, 4), Wt + 4;
                }
                function Rt(Qt, Ut, Wt, rn, vn) {
                  return Ut = +Ut, Wt >>>= 0, vn || St(Qt, 0, Wt, 8), ve.write(Qt, Ut, Wt, rn, 52, 8), Wt + 8;
                }
                we.prototype.slice = function(Qt, Ut) {
                  var Wt = this.length;
                  (Qt = ~~Qt) < 0 ? (Qt += Wt) < 0 && (Qt = 0) : Qt > Wt && (Qt = Wt), (Ut = Ut === void 0 ? Wt : ~~Ut) < 0 ? (Ut += Wt) < 0 && (Ut = 0) : Ut > Wt && (Ut = Wt), Ut < Qt && (Ut = Qt);
                  var rn = this.subarray(Qt, Ut);
                  return Object.setPrototypeOf(rn, we.prototype), rn;
                }, we.prototype.readUintLE = we.prototype.readUIntLE = function(Qt, Ut, Wt) {
                  Qt >>>= 0, Ut >>>= 0, Wt || st(Qt, Ut, this.length);
                  for (var rn = this[Qt], vn = 1, dn = 0; ++dn < Ut && (vn *= 256); ) rn += this[Qt + dn] * vn;
                  return rn;
                }, we.prototype.readUintBE = we.prototype.readUIntBE = function(Qt, Ut, Wt) {
                  Qt >>>= 0, Ut >>>= 0, Wt || st(Qt, Ut, this.length);
                  for (var rn = this[Qt + --Ut], vn = 1; Ut > 0 && (vn *= 256); ) rn += this[Qt + --Ut] * vn;
                  return rn;
                }, we.prototype.readUint8 = we.prototype.readUInt8 = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 1, this.length), this[Qt];
                }, we.prototype.readUint16LE = we.prototype.readUInt16LE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 2, this.length), this[Qt] | this[Qt + 1] << 8;
                }, we.prototype.readUint16BE = we.prototype.readUInt16BE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 2, this.length), this[Qt] << 8 | this[Qt + 1];
                }, we.prototype.readUint32LE = we.prototype.readUInt32LE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 4, this.length), (this[Qt] | this[Qt + 1] << 8 | this[Qt + 2] << 16) + 16777216 * this[Qt + 3];
                }, we.prototype.readUint32BE = we.prototype.readUInt32BE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 4, this.length), 16777216 * this[Qt] + (this[Qt + 1] << 16 | this[Qt + 2] << 8 | this[Qt + 3]);
                }, we.prototype.readBigUInt64LE = sn(function(Qt) {
                  Nt(Qt >>>= 0, "offset");
                  var Ut = this[Qt], Wt = this[Qt + 7];
                  Ut !== void 0 && Wt !== void 0 || Gt(Qt, this.length - 8);
                  var rn = Ut + this[++Qt] * Math.pow(2, 8) + this[++Qt] * Math.pow(2, 16) + this[++Qt] * Math.pow(2, 24), vn = this[++Qt] + this[++Qt] * Math.pow(2, 8) + this[++Qt] * Math.pow(2, 16) + Wt * Math.pow(2, 24);
                  return BigInt(rn) + (BigInt(vn) << BigInt(32));
                }), we.prototype.readBigUInt64BE = sn(function(Qt) {
                  Nt(Qt >>>= 0, "offset");
                  var Ut = this[Qt], Wt = this[Qt + 7];
                  Ut !== void 0 && Wt !== void 0 || Gt(Qt, this.length - 8);
                  var rn = Ut * Math.pow(2, 24) + this[++Qt] * Math.pow(2, 16) + this[++Qt] * Math.pow(2, 8) + this[++Qt], vn = this[++Qt] * Math.pow(2, 24) + this[++Qt] * Math.pow(2, 16) + this[++Qt] * Math.pow(2, 8) + Wt;
                  return (BigInt(rn) << BigInt(32)) + BigInt(vn);
                }), we.prototype.readIntLE = function(Qt, Ut, Wt) {
                  Qt >>>= 0, Ut >>>= 0, Wt || st(Qt, Ut, this.length);
                  for (var rn = this[Qt], vn = 1, dn = 0; ++dn < Ut && (vn *= 256); ) rn += this[Qt + dn] * vn;
                  return rn >= (vn *= 128) && (rn -= Math.pow(2, 8 * Ut)), rn;
                }, we.prototype.readIntBE = function(Qt, Ut, Wt) {
                  Qt >>>= 0, Ut >>>= 0, Wt || st(Qt, Ut, this.length);
                  for (var rn = Ut, vn = 1, dn = this[Qt + --rn]; rn > 0 && (vn *= 256); ) dn += this[Qt + --rn] * vn;
                  return dn >= (vn *= 128) && (dn -= Math.pow(2, 8 * Ut)), dn;
                }, we.prototype.readInt8 = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 1, this.length), 128 & this[Qt] ? -1 * (255 - this[Qt] + 1) : this[Qt];
                }, we.prototype.readInt16LE = function(Qt, Ut) {
                  Qt >>>= 0, Ut || st(Qt, 2, this.length);
                  var Wt = this[Qt] | this[Qt + 1] << 8;
                  return 32768 & Wt ? 4294901760 | Wt : Wt;
                }, we.prototype.readInt16BE = function(Qt, Ut) {
                  Qt >>>= 0, Ut || st(Qt, 2, this.length);
                  var Wt = this[Qt + 1] | this[Qt] << 8;
                  return 32768 & Wt ? 4294901760 | Wt : Wt;
                }, we.prototype.readInt32LE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 4, this.length), this[Qt] | this[Qt + 1] << 8 | this[Qt + 2] << 16 | this[Qt + 3] << 24;
                }, we.prototype.readInt32BE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 4, this.length), this[Qt] << 24 | this[Qt + 1] << 16 | this[Qt + 2] << 8 | this[Qt + 3];
                }, we.prototype.readBigInt64LE = sn(function(Qt) {
                  Nt(Qt >>>= 0, "offset");
                  var Ut = this[Qt], Wt = this[Qt + 7];
                  Ut !== void 0 && Wt !== void 0 || Gt(Qt, this.length - 8);
                  var rn = this[Qt + 4] + this[Qt + 5] * Math.pow(2, 8) + this[Qt + 6] * Math.pow(2, 16) + (Wt << 24);
                  return (BigInt(rn) << BigInt(32)) + BigInt(Ut + this[++Qt] * Math.pow(2, 8) + this[++Qt] * Math.pow(2, 16) + this[++Qt] * Math.pow(2, 24));
                }), we.prototype.readBigInt64BE = sn(function(Qt) {
                  Nt(Qt >>>= 0, "offset");
                  var Ut = this[Qt], Wt = this[Qt + 7];
                  Ut !== void 0 && Wt !== void 0 || Gt(Qt, this.length - 8);
                  var rn = (Ut << 24) + this[++Qt] * Math.pow(2, 16) + this[++Qt] * Math.pow(2, 8) + this[++Qt];
                  return (BigInt(rn) << BigInt(32)) + BigInt(this[++Qt] * Math.pow(2, 24) + this[++Qt] * Math.pow(2, 16) + this[++Qt] * Math.pow(2, 8) + Wt);
                }), we.prototype.readFloatLE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 4, this.length), ve.read(this, Qt, !0, 23, 4);
                }, we.prototype.readFloatBE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 4, this.length), ve.read(this, Qt, !1, 23, 4);
                }, we.prototype.readDoubleLE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 8, this.length), ve.read(this, Qt, !0, 52, 8);
                }, we.prototype.readDoubleBE = function(Qt, Ut) {
                  return Qt >>>= 0, Ut || st(Qt, 8, this.length), ve.read(this, Qt, !1, 52, 8);
                }, we.prototype.writeUintLE = we.prototype.writeUIntLE = function(Qt, Ut, Wt, rn) {
                  Qt = +Qt, Ut >>>= 0, Wt >>>= 0, rn || gt(this, Qt, Ut, Wt, Math.pow(2, 8 * Wt) - 1, 0);
                  var vn = 1, dn = 0;
                  for (this[Ut] = 255 & Qt; ++dn < Wt && (vn *= 256); ) this[Ut + dn] = Qt / vn & 255;
                  return Ut + Wt;
                }, we.prototype.writeUintBE = we.prototype.writeUIntBE = function(Qt, Ut, Wt, rn) {
                  Qt = +Qt, Ut >>>= 0, Wt >>>= 0, rn || gt(this, Qt, Ut, Wt, Math.pow(2, 8 * Wt) - 1, 0);
                  var vn = Wt - 1, dn = 1;
                  for (this[Ut + vn] = 255 & Qt; --vn >= 0 && (dn *= 256); ) this[Ut + vn] = Qt / dn & 255;
                  return Ut + Wt;
                }, we.prototype.writeUint8 = we.prototype.writeUInt8 = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 1, 255, 0), this[Ut] = 255 & Qt, Ut + 1;
                }, we.prototype.writeUint16LE = we.prototype.writeUInt16LE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 2, 65535, 0), this[Ut] = 255 & Qt, this[Ut + 1] = Qt >>> 8, Ut + 2;
                }, we.prototype.writeUint16BE = we.prototype.writeUInt16BE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 2, 65535, 0), this[Ut] = Qt >>> 8, this[Ut + 1] = 255 & Qt, Ut + 2;
                }, we.prototype.writeUint32LE = we.prototype.writeUInt32LE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 4, 4294967295, 0), this[Ut + 3] = Qt >>> 24, this[Ut + 2] = Qt >>> 16, this[Ut + 1] = Qt >>> 8, this[Ut] = 255 & Qt, Ut + 4;
                }, we.prototype.writeUint32BE = we.prototype.writeUInt32BE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 4, 4294967295, 0), this[Ut] = Qt >>> 24, this[Ut + 1] = Qt >>> 16, this[Ut + 2] = Qt >>> 8, this[Ut + 3] = 255 & Qt, Ut + 4;
                }, we.prototype.writeBigUInt64LE = sn(function(Qt) {
                  return bt(this, Qt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, BigInt(0), BigInt("0xffffffffffffffff"));
                }), we.prototype.writeBigUInt64BE = sn(function(Qt) {
                  return yt(this, Qt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, BigInt(0), BigInt("0xffffffffffffffff"));
                }), we.prototype.writeIntLE = function(Qt, Ut, Wt, rn) {
                  if (Qt = +Qt, Ut >>>= 0, !rn) {
                    var vn = Math.pow(2, 8 * Wt - 1);
                    gt(this, Qt, Ut, Wt, vn - 1, -vn);
                  }
                  var dn = 0, Ht = 1, jt = 0;
                  for (this[Ut] = 255 & Qt; ++dn < Wt && (Ht *= 256); ) Qt < 0 && jt === 0 && this[Ut + dn - 1] !== 0 && (jt = 1), this[Ut + dn] = (Qt / Ht >> 0) - jt & 255;
                  return Ut + Wt;
                }, we.prototype.writeIntBE = function(Qt, Ut, Wt, rn) {
                  if (Qt = +Qt, Ut >>>= 0, !rn) {
                    var vn = Math.pow(2, 8 * Wt - 1);
                    gt(this, Qt, Ut, Wt, vn - 1, -vn);
                  }
                  var dn = Wt - 1, Ht = 1, jt = 0;
                  for (this[Ut + dn] = 255 & Qt; --dn >= 0 && (Ht *= 256); ) Qt < 0 && jt === 0 && this[Ut + dn + 1] !== 0 && (jt = 1), this[Ut + dn] = (Qt / Ht >> 0) - jt & 255;
                  return Ut + Wt;
                }, we.prototype.writeInt8 = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 1, 127, -128), Qt < 0 && (Qt = 255 + Qt + 1), this[Ut] = 255 & Qt, Ut + 1;
                }, we.prototype.writeInt16LE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 2, 32767, -32768), this[Ut] = 255 & Qt, this[Ut + 1] = Qt >>> 8, Ut + 2;
                }, we.prototype.writeInt16BE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 2, 32767, -32768), this[Ut] = Qt >>> 8, this[Ut + 1] = 255 & Qt, Ut + 2;
                }, we.prototype.writeInt32LE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 4, 2147483647, -2147483648), this[Ut] = 255 & Qt, this[Ut + 1] = Qt >>> 8, this[Ut + 2] = Qt >>> 16, this[Ut + 3] = Qt >>> 24, Ut + 4;
                }, we.prototype.writeInt32BE = function(Qt, Ut, Wt) {
                  return Qt = +Qt, Ut >>>= 0, Wt || gt(this, Qt, Ut, 4, 2147483647, -2147483648), Qt < 0 && (Qt = 4294967295 + Qt + 1), this[Ut] = Qt >>> 24, this[Ut + 1] = Qt >>> 16, this[Ut + 2] = Qt >>> 8, this[Ut + 3] = 255 & Qt, Ut + 4;
                }, we.prototype.writeBigInt64LE = sn(function(Qt) {
                  return bt(this, Qt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, -BigInt("0x8000000000000000"), BigInt("0x7fffffffffffffff"));
                }), we.prototype.writeBigInt64BE = sn(function(Qt) {
                  return yt(this, Qt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, -BigInt("0x8000000000000000"), BigInt("0x7fffffffffffffff"));
                }), we.prototype.writeFloatLE = function(Qt, Ut, Wt) {
                  return Lt(this, Qt, Ut, !0, Wt);
                }, we.prototype.writeFloatBE = function(Qt, Ut, Wt) {
                  return Lt(this, Qt, Ut, !1, Wt);
                }, we.prototype.writeDoubleLE = function(Qt, Ut, Wt) {
                  return Rt(this, Qt, Ut, !0, Wt);
                }, we.prototype.writeDoubleBE = function(Qt, Ut, Wt) {
                  return Rt(this, Qt, Ut, !1, Wt);
                }, we.prototype.copy = function(Qt, Ut, Wt, rn) {
                  if (!we.isBuffer(Qt)) throw new TypeError("argument should be a Buffer");
                  if (Wt || (Wt = 0), rn || rn === 0 || (rn = this.length), Ut >= Qt.length && (Ut = Qt.length), Ut || (Ut = 0), rn > 0 && rn < Wt && (rn = Wt), rn === Wt || Qt.length === 0 || this.length === 0) return 0;
                  if (Ut < 0) throw new RangeError("targetStart out of bounds");
                  if (Wt < 0 || Wt >= this.length) throw new RangeError("Index out of range");
                  if (rn < 0) throw new RangeError("sourceEnd out of bounds");
                  rn > this.length && (rn = this.length), Qt.length - Ut < rn - Wt && (rn = Qt.length - Ut + Wt);
                  var vn = rn - Wt;
                  return this === Qt && typeof Uint8Array.prototype.copyWithin == "function" ? this.copyWithin(Ut, Wt, rn) : Uint8Array.prototype.set.call(Qt, this.subarray(Wt, rn), Ut), vn;
                }, we.prototype.fill = function(Qt, Ut, Wt, rn) {
                  if (typeof Qt == "string") {
                    if (typeof Ut == "string" ? (rn = Ut, Ut = 0, Wt = this.length) : typeof Wt == "string" && (rn = Wt, Wt = this.length), rn !== void 0 && typeof rn != "string") throw new TypeError("encoding must be a string");
                    if (typeof rn == "string" && !we.isEncoding(rn)) throw new TypeError("Unknown encoding: " + rn);
                    if (Qt.length === 1) {
                      var vn = Qt.charCodeAt(0);
                      (rn === "utf8" && vn < 128 || rn === "latin1") && (Qt = vn);
                    }
                  } else typeof Qt == "number" ? Qt &= 255 : typeof Qt == "boolean" && (Qt = Number(Qt));
                  if (Ut < 0 || this.length < Ut || this.length < Wt) throw new RangeError("Out of range index");
                  if (Wt <= Ut) return this;
                  var dn;
                  if (Ut >>>= 0, Wt = Wt === void 0 ? this.length : Wt >>> 0, Qt || (Qt = 0), typeof Qt == "number") for (dn = Ut; dn < Wt; ++dn) this[dn] = Qt;
                  else {
                    var Ht = we.isBuffer(Qt) ? Qt : we.from(Qt, rn), jt = Ht.length;
                    if (jt === 0) throw new TypeError('The value "' + Qt + '" is invalid for argument "value"');
                    for (dn = 0; dn < Wt - Ut; ++dn) this[dn + Ut] = Ht[dn % jt];
                  }
                  return this;
                };
                var vt = {};
                function $t(Qt, Ut, Wt) {
                  vt[Qt] = function(rn) {
                    function vn() {
                      var jt;
                      return function(cn, wn) {
                        if (!(cn instanceof wn)) throw new TypeError("Cannot call a class as a function");
                      }(this, vn), jt = function(cn, wn, Rn) {
                        return wn = me(wn), function(Pn, Ie) {
                          if (Ie && (ye(Ie) == "object" || typeof Ie == "function")) return Ie;
                          if (Ie !== void 0) throw new TypeError("Derived constructors may only return object or undefined");
                          return function(rt) {
                            if (rt === void 0) throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
                            return rt;
                          }(Pn);
                        }(cn, he() ? Reflect.construct(wn, [], me(cn).constructor) : wn.apply(cn, Rn));
                      }(this, vn), Object.defineProperty(jt, "message", { value: Ut.apply(jt, arguments), writable: !0, configurable: !0 }), jt.name = "".concat(jt.name, " [").concat(Qt, "]"), jt.stack, delete jt.name, jt;
                    }
                    return function(jt, cn) {
                      if (typeof cn != "function" && cn !== null) throw new TypeError("Super expression must either be null or a function");
                      jt.prototype = Object.create(cn && cn.prototype, { constructor: { value: jt, writable: !0, configurable: !0 } }), Object.defineProperty(jt, "prototype", { writable: !1 }), cn && pe(jt, cn);
                    }(vn, rn), dn = vn, (Ht = [{ key: "code", get: function() {
                      return Qt;
                    }, set: function(jt) {
                      Object.defineProperty(this, "code", { configurable: !0, enumerable: !0, value: jt, writable: !0 });
                    } }, { key: "toString", value: function() {
                      return "".concat(this.name, " [").concat(Qt, "]: ").concat(this.message);
                    } }]) && de(dn.prototype, Ht), Object.defineProperty(dn, "prototype", { writable: !1 }), dn;
                    var dn, Ht;
                  }(Wt);
                }
                function Pt(Qt) {
                  for (var Ut = "", Wt = Qt.length, rn = Qt[0] === "-" ? 1 : 0; Wt >= rn + 4; Wt -= 3) Ut = "_".concat(Qt.slice(Wt - 3, Wt)).concat(Ut);
                  return "".concat(Qt.slice(0, Wt)).concat(Ut);
                }
                function Xt(Qt, Ut, Wt, rn, vn, dn) {
                  if (Qt > Wt || Qt < Ut) {
                    var Ht, jt = typeof Ut == "bigint" ? "n" : "";
                    throw Ht = Ut === 0 || Ut === BigInt(0) ? ">= 0".concat(jt, " and < 2").concat(jt, " ** ").concat(8 * (dn + 1)).concat(jt) : ">= -(2".concat(jt, " ** ").concat(8 * (dn + 1) - 1).concat(jt, ") and < 2 ** ") + "".concat(8 * (dn + 1) - 1).concat(jt), new vt.ERR_OUT_OF_RANGE("value", Ht, Qt);
                  }
                  (function(cn, wn, Rn) {
                    Nt(wn, "offset"), cn[wn] !== void 0 && cn[wn + Rn] !== void 0 || Gt(wn, cn.length - (Rn + 1));
                  })(rn, vn, dn);
                }
                function Nt(Qt, Ut) {
                  if (typeof Qt != "number") throw new vt.ERR_INVALID_ARG_TYPE(Ut, "number", Qt);
                }
                function Gt(Qt, Ut, Wt) {
                  throw Math.floor(Qt) !== Qt ? (Nt(Qt, Wt), new vt.ERR_OUT_OF_RANGE("offset", "an integer", Qt)) : Ut < 0 ? new vt.ERR_BUFFER_OUT_OF_BOUNDS() : new vt.ERR_OUT_OF_RANGE("offset", ">= ".concat(0, " and <= ").concat(Ut), Qt);
                }
                $t("ERR_BUFFER_OUT_OF_BOUNDS", function(Qt) {
                  return Qt ? "".concat(Qt, " is outside of buffer bounds") : "Attempt to access memory outside buffer bounds";
                }, RangeError), $t("ERR_INVALID_ARG_TYPE", function(Qt, Ut) {
                  return 'The "'.concat(Qt, '" argument must be of type number. Received type ').concat(ye(Ut));
                }, TypeError), $t("ERR_OUT_OF_RANGE", function(Qt, Ut, Wt) {
                  var rn = 'The value of "'.concat(Qt, '" is out of range.'), vn = Wt;
                  return Number.isInteger(Wt) && Math.abs(Wt) > Math.pow(2, 32) ? vn = Pt(String(Wt)) : typeof Wt == "bigint" && (vn = String(Wt), (Wt > Math.pow(BigInt(2), BigInt(32)) || Wt < -Math.pow(BigInt(2), BigInt(32))) && (vn = Pt(vn)), vn += "n"), rn + " It must be ".concat(Ut, ". Received ").concat(vn);
                }, RangeError);
                var Ft = /[^+/0-9A-Za-z-_]/g;
                function Tt(Qt, Ut) {
                  var Wt;
                  Ut = Ut || 1 / 0;
                  for (var rn = Qt.length, vn = null, dn = [], Ht = 0; Ht < rn; ++Ht) {
                    if ((Wt = Qt.charCodeAt(Ht)) > 55295 && Wt < 57344) {
                      if (!vn) {
                        if (Wt > 56319) {
                          (Ut -= 3) > -1 && dn.push(239, 191, 189);
                          continue;
                        }
                        if (Ht + 1 === rn) {
                          (Ut -= 3) > -1 && dn.push(239, 191, 189);
                          continue;
                        }
                        vn = Wt;
                        continue;
                      }
                      if (Wt < 56320) {
                        (Ut -= 3) > -1 && dn.push(239, 191, 189), vn = Wt;
                        continue;
                      }
                      Wt = 65536 + (vn - 55296 << 10 | Wt - 56320);
                    } else vn && (Ut -= 3) > -1 && dn.push(239, 191, 189);
                    if (vn = null, Wt < 128) {
                      if ((Ut -= 1) < 0) break;
                      dn.push(Wt);
                    } else if (Wt < 2048) {
                      if ((Ut -= 2) < 0) break;
                      dn.push(Wt >> 6 | 192, 63 & Wt | 128);
                    } else if (Wt < 65536) {
                      if ((Ut -= 3) < 0) break;
                      dn.push(Wt >> 12 | 224, Wt >> 6 & 63 | 128, 63 & Wt | 128);
                    } else {
                      if (!(Wt < 1114112)) throw new Error("Invalid code point");
                      if ((Ut -= 4) < 0) break;
                      dn.push(Wt >> 18 | 240, Wt >> 12 & 63 | 128, Wt >> 6 & 63 | 128, 63 & Wt | 128);
                    }
                  }
                  return dn;
                }
                function It(Qt) {
                  return ge.toByteArray(function(Ut) {
                    if ((Ut = (Ut = Ut.split("=")[0]).trim().replace(Ft, "")).length < 2) return "";
                    for (; Ut.length % 4 != 0; ) Ut += "=";
                    return Ut;
                  }(Qt));
                }
                function Ct(Qt, Ut, Wt, rn) {
                  var vn;
                  for (vn = 0; vn < rn && !(vn + Wt >= Ut.length || vn >= Qt.length); ++vn) Ut[vn + Wt] = Qt[vn];
                  return vn;
                }
                function Kt(Qt, Ut) {
                  return Qt instanceof Ut || Qt != null && Qt.constructor != null && Qt.constructor.name != null && Qt.constructor.name === Ut.name;
                }
                function Vt(Qt) {
                  return Qt != Qt;
                }
                var Yt = function() {
                  for (var Qt = "0123456789abcdef", Ut = new Array(256), Wt = 0; Wt < 16; ++Wt) for (var rn = 16 * Wt, vn = 0; vn < 16; ++vn) Ut[rn + vn] = Qt[Wt] + Qt[vn];
                  return Ut;
                }();
                function sn(Qt) {
                  return typeof BigInt > "u" ? hn : Qt;
                }
                function hn() {
                  throw new Error("BigInt not supported");
                }
              }, 9216: function(ce) {
                ce.exports = fe, ce.exports.isMobile = fe, ce.exports.default = fe;
                var le = /(android|bb\d+|meego).+mobile|armv7l|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series[46]0|samsungbrowser.*mobile|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i, ue = /CrOS/, de = /android|ipad|playbook|silk/i;
                function fe(he) {
                  he || (he = {});
                  var me = he.ua;
                  if (me || typeof navigator > "u" || (me = navigator.userAgent), me && me.headers && typeof me.headers["user-agent"] == "string" && (me = me.headers["user-agent"]), typeof me != "string") return !1;
                  var pe = le.test(me) && !ue.test(me) || !!he.tablet && de.test(me);
                  return !pe && he.tablet && he.featureDetect && navigator && navigator.maxTouchPoints > 1 && me.indexOf("Macintosh") !== -1 && me.indexOf("Safari") !== -1 && (pe = !0), pe;
                }
              }, 6296: function(ce, le, ue) {
                ce.exports = function(ye) {
                  var ge = (ye = ye || {}).eye || [0, 0, 1], ve = ye.center || [0, 0, 0], be = ye.up || [0, 1, 0], Ce = ye.distanceLimits || [0, 1 / 0], Se = ye.mode || "turntable", we = de(), Ee = fe(), xe = he();
                  return we.setDistanceLimits(Ce[0], Ce[1]), we.lookAt(0, ge, ve, be), Ee.setDistanceLimits(Ce[0], Ce[1]), Ee.lookAt(0, ge, ve, be), xe.setDistanceLimits(Ce[0], Ce[1]), xe.lookAt(0, ge, ve, be), new me({ turntable: we, orbit: Ee, matrix: xe }, Se);
                };
                var de = ue(7261), fe = ue(9977), he = ue(4192);
                function me(ye, ge) {
                  this._controllerNames = Object.keys(ye), this._controllerList = this._controllerNames.map(function(ve) {
                    return ye[ve];
                  }), this._mode = ge, this._active = ye[ge], this._active || (this._mode = "turntable", this._active = ye.turntable), this.modes = this._controllerNames, this.computedMatrix = this._active.computedMatrix, this.computedEye = this._active.computedEye, this.computedUp = this._active.computedUp, this.computedCenter = this._active.computedCenter, this.computedRadius = this._active.computedRadius;
                }
                var pe = me.prototype;
                pe.flush = function(ye) {
                  for (var ge = this._controllerList, ve = 0; ve < ge.length; ++ve) ge[ve].flush(ye);
                }, pe.idle = function(ye) {
                  for (var ge = this._controllerList, ve = 0; ve < ge.length; ++ve) ge[ve].idle(ye);
                }, pe.lookAt = function(ye, ge, ve, be) {
                  for (var Ce = this._controllerList, Se = 0; Se < Ce.length; ++Se) Ce[Se].lookAt(ye, ge, ve, be);
                }, pe.rotate = function(ye, ge, ve, be) {
                  for (var Ce = this._controllerList, Se = 0; Se < Ce.length; ++Se) Ce[Se].rotate(ye, ge, ve, be);
                }, pe.pan = function(ye, ge, ve, be) {
                  for (var Ce = this._controllerList, Se = 0; Se < Ce.length; ++Se) Ce[Se].pan(ye, ge, ve, be);
                }, pe.translate = function(ye, ge, ve, be) {
                  for (var Ce = this._controllerList, Se = 0; Se < Ce.length; ++Se) Ce[Se].translate(ye, ge, ve, be);
                }, pe.setMatrix = function(ye, ge) {
                  for (var ve = this._controllerList, be = 0; be < ve.length; ++be) ve[be].setMatrix(ye, ge);
                }, pe.setDistanceLimits = function(ye, ge) {
                  for (var ve = this._controllerList, be = 0; be < ve.length; ++be) ve[be].setDistanceLimits(ye, ge);
                }, pe.setDistance = function(ye, ge) {
                  for (var ve = this._controllerList, be = 0; be < ve.length; ++be) ve[be].setDistance(ye, ge);
                }, pe.recalcMatrix = function(ye) {
                  this._active.recalcMatrix(ye);
                }, pe.getDistance = function(ye) {
                  return this._active.getDistance(ye);
                }, pe.getDistanceLimits = function(ye) {
                  return this._active.getDistanceLimits(ye);
                }, pe.lastT = function() {
                  return this._active.lastT();
                }, pe.setMode = function(ye) {
                  if (ye !== this._mode) {
                    var ge = this._controllerNames.indexOf(ye);
                    if (!(ge < 0)) {
                      var ve = this._active, be = this._controllerList[ge], Ce = Math.max(ve.lastT(), be.lastT());
                      ve.recalcMatrix(Ce), be.setMatrix(Ce, ve.computedMatrix), this._active = be, this._mode = ye, this.computedMatrix = this._active.computedMatrix, this.computedEye = this._active.computedEye, this.computedUp = this._active.computedUp, this.computedCenter = this._active.computedCenter, this.computedRadius = this._active.computedRadius;
                    }
                  }
                }, pe.getMode = function() {
                  return this._mode;
                };
              }, 7169: function(ce, le, ue) {
                var de = typeof WeakMap > "u" ? ue(1538) : WeakMap, fe = ue(2762), he = ue(8116), me = new de();
                ce.exports = function(pe) {
                  var ye = me.get(pe), ge = ye && (ye._triangleBuffer.handle || ye._triangleBuffer.buffer);
                  if (!ge || !pe.isBuffer(ge)) {
                    var ve = fe(pe, new Float32Array([-1, -1, -1, 4, 4, -1]));
                    (ye = he(pe, [{ buffer: ve, type: pe.FLOAT, size: 2 }]))._triangleBuffer = ve, me.set(pe, ye);
                  }
                  ye.bind(), pe.drawArrays(pe.TRIANGLES, 0, 3), ye.unbind();
                };
              }, 1085: function(ce, le, ue) {
                var de = ue(1371);
                ce.exports = function(fe, he, me) {
                  he = typeof he == "number" ? he : 1, me = me || ": ";
                  var pe = fe.split(/\r?\n/), ye = String(pe.length + he - 1).length;
                  return pe.map(function(ge, ve) {
                    var be = ve + he, Ce = String(be).length;
                    return de(be, ye - Ce) + me + ge;
                  }).join(`
`);
                };
              }, 3952: function(ce, le, ue) {
                ce.exports = function(he) {
                  var me = he.length;
                  if (me === 0) return [];
                  if (me === 1) return [0];
                  for (var pe = he[0].length, ye = [he[0]], ge = [0], ve = 1; ve < me; ++ve) if (ye.push(he[ve]), fe(ye, pe)) {
                    if (ge.push(ve), ge.length === pe + 1) return ge;
                  } else ye.pop();
                  return ge;
                };
                var de = ue(3250);
                function fe(he, me) {
                  for (var pe = new Array(me + 1), ye = 0; ye < he.length; ++ye) pe[ye] = he[ye];
                  for (ye = 0; ye <= he.length; ++ye) {
                    for (var ge = he.length; ge <= me; ++ge) {
                      for (var ve = new Array(me), be = 0; be < me; ++be) ve[be] = Math.pow(ge + 1 - ye, be);
                      pe[ge] = ve;
                    }
                    if (de.apply(void 0, pe)) return !0;
                  }
                  return !1;
                }
              }, 5995: function(ce, le, ue) {
                ce.exports = function(he, me) {
                  return de(me).filter(function(pe) {
                    for (var ye = new Array(pe.length), ge = 0; ge < pe.length; ++ge) ye[ge] = me[pe[ge]];
                    return fe(ye) * he < 1;
                  });
                };
                var de = ue(7642), fe = ue(6037);
              }, 3502: function(ce, le, ue) {
                ce.exports = function(he, me) {
                  return fe(de(he, me));
                };
                var de = ue(5995), fe = ue(9127);
              }, 6468: function(ce) {
                ce.exports = function(le) {
                  return atob(le);
                };
              }, 2642: function(ce, le, ue) {
                ce.exports = function(he, me) {
                  for (var pe = me.length, ye = new Array(pe + 1), ge = 0; ge < pe; ++ge) {
                    for (var ve = new Array(pe + 1), be = 0; be <= pe; ++be) ve[be] = he[be][ge];
                    ye[ge] = ve;
                  }
                  for (ye[pe] = new Array(pe + 1), ge = 0; ge <= pe; ++ge) ye[pe][ge] = 1;
                  var Ce = new Array(pe + 1);
                  for (ge = 0; ge < pe; ++ge) Ce[ge] = me[ge];
                  Ce[pe] = 1;
                  var Se = de(ye, Ce), we = fe(Se[pe + 1]);
                  we === 0 && (we = 1);
                  var Ee = new Array(pe + 1);
                  for (ge = 0; ge <= pe; ++ge) Ee[ge] = fe(Se[ge]) / we;
                  return Ee;
                };
                var de = ue(727);
                function fe(he) {
                  for (var me = 0, pe = 0; pe < he.length; ++pe) me += he[pe];
                  return me;
                }
              }, 7507: function(ce, le) {
                le.byteLength = function(ge) {
                  var ve = pe(ge), be = ve[0], Ce = ve[1];
                  return 3 * (be + Ce) / 4 - Ce;
                }, le.toByteArray = function(ge) {
                  var ve, be, Ce = pe(ge), Se = Ce[0], we = Ce[1], Ee = new fe(function(Qe, ke, Me) {
                    return 3 * (ke + Me) / 4 - Me;
                  }(0, Se, we)), xe = 0, Le = we > 0 ? Se - 4 : Se;
                  for (be = 0; be < Le; be += 4) ve = de[ge.charCodeAt(be)] << 18 | de[ge.charCodeAt(be + 1)] << 12 | de[ge.charCodeAt(be + 2)] << 6 | de[ge.charCodeAt(be + 3)], Ee[xe++] = ve >> 16 & 255, Ee[xe++] = ve >> 8 & 255, Ee[xe++] = 255 & ve;
                  return we === 2 && (ve = de[ge.charCodeAt(be)] << 2 | de[ge.charCodeAt(be + 1)] >> 4, Ee[xe++] = 255 & ve), we === 1 && (ve = de[ge.charCodeAt(be)] << 10 | de[ge.charCodeAt(be + 1)] << 4 | de[ge.charCodeAt(be + 2)] >> 2, Ee[xe++] = ve >> 8 & 255, Ee[xe++] = 255 & ve), Ee;
                }, le.fromByteArray = function(ge) {
                  for (var ve, be = ge.length, Ce = be % 3, Se = [], we = 16383, Ee = 0, xe = be - Ce; Ee < xe; Ee += we) Se.push(ye(ge, Ee, Ee + we > xe ? xe : Ee + we));
                  return Ce === 1 ? (ve = ge[be - 1], Se.push(ue[ve >> 2] + ue[ve << 4 & 63] + "==")) : Ce === 2 && (ve = (ge[be - 2] << 8) + ge[be - 1], Se.push(ue[ve >> 10] + ue[ve >> 4 & 63] + ue[ve << 2 & 63] + "=")), Se.join("");
                };
                for (var ue = [], de = [], fe = typeof Uint8Array < "u" ? Uint8Array : Array, he = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/", me = 0; me < 64; ++me) ue[me] = he[me], de[he.charCodeAt(me)] = me;
                function pe(ge) {
                  var ve = ge.length;
                  if (ve % 4 > 0) throw new Error("Invalid string. Length must be a multiple of 4");
                  var be = ge.indexOf("=");
                  return be === -1 && (be = ve), [be, be === ve ? 0 : 4 - be % 4];
                }
                function ye(ge, ve, be) {
                  for (var Ce, Se, we = [], Ee = ve; Ee < be; Ee += 3) Ce = (ge[Ee] << 16 & 16711680) + (ge[Ee + 1] << 8 & 65280) + (255 & ge[Ee + 2]), we.push(ue[(Se = Ce) >> 18 & 63] + ue[Se >> 12 & 63] + ue[Se >> 6 & 63] + ue[63 & Se]);
                  return we.join("");
                }
                de[45] = 62, de[95] = 63;
              }, 3865: function(ce, le, ue) {
                var de = ue(869);
                ce.exports = function(fe, he) {
                  return de(fe[0].mul(he[1]).add(he[0].mul(fe[1])), fe[1].mul(he[1]));
                };
              }, 1318: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0].mul(ue[1]).cmp(ue[0].mul(le[1]));
                };
              }, 8697: function(ce, le, ue) {
                var de = ue(869);
                ce.exports = function(fe, he) {
                  return de(fe[0].mul(he[1]), fe[1].mul(he[0]));
                };
              }, 7842: function(ce, le, ue) {
                var de = ue(6330), fe = ue(1533), he = ue(2651), me = ue(4387), pe = ue(869), ye = ue(8697);
                ce.exports = function ge(ve, be) {
                  if (de(ve)) return be ? ye(ve, ge(be)) : [ve[0].clone(), ve[1].clone()];
                  var Ce, Se, we = 0;
                  if (fe(ve)) Ce = ve.clone();
                  else if (typeof ve == "string") Ce = me(ve);
                  else {
                    if (ve === 0) return [he(0), he(1)];
                    if (ve === Math.floor(ve)) Ce = he(ve);
                    else {
                      for (; ve !== Math.floor(ve); ) ve *= Math.pow(2, 256), we -= 256;
                      Ce = he(ve);
                    }
                  }
                  if (de(be)) Ce.mul(be[1]), Se = be[0].clone();
                  else if (fe(be)) Se = be.clone();
                  else if (typeof be == "string") Se = me(be);
                  else if (be) if (be === Math.floor(be)) Se = he(be);
                  else {
                    for (; be !== Math.floor(be); ) be *= Math.pow(2, 256), we += 256;
                    Se = he(be);
                  }
                  else Se = he(1);
                  return we > 0 ? Ce = Ce.ushln(we) : we < 0 && (Se = Se.ushln(-we)), pe(Ce, Se);
                };
              }, 6330: function(ce, le, ue) {
                var de = ue(1533);
                ce.exports = function(fe) {
                  return Array.isArray(fe) && fe.length === 2 && de(fe[0]) && de(fe[1]);
                };
              }, 5716: function(ce, le, ue) {
                var de = ue(6859);
                ce.exports = function(fe) {
                  return fe.cmp(new de(0));
                };
              }, 1369: function(ce, le, ue) {
                var de = ue(5716);
                ce.exports = function(fe) {
                  var he = fe.length, me = fe.words, pe = 0;
                  if (he === 1) pe = me[0];
                  else if (he === 2) pe = me[0] + 67108864 * me[1];
                  else for (var ye = 0; ye < he; ye++) pe += me[ye] * Math.pow(67108864, ye);
                  return de(fe) * pe;
                };
              }, 4025: function(ce, le, ue) {
                var de = ue(2361), fe = ue(8828).countTrailingZeros;
                ce.exports = function(he) {
                  var me = fe(de.lo(he));
                  if (me < 32) return me;
                  var pe = fe(de.hi(he));
                  return pe > 20 ? 52 : pe + 32;
                };
              }, 1533: function(ce, le, ue) {
                ue(6859), ce.exports = function(de) {
                  return de && typeof de == "object" && !!de.words;
                };
              }, 2651: function(ce, le, ue) {
                var de = ue(6859), fe = ue(2361);
                ce.exports = function(he) {
                  var me = fe.exponent(he);
                  return me < 52 ? new de(he) : new de(he * Math.pow(2, 52 - me)).ushln(me - 52);
                };
              }, 869: function(ce, le, ue) {
                var de = ue(2651), fe = ue(5716);
                ce.exports = function(he, me) {
                  var pe = fe(he), ye = fe(me);
                  if (pe === 0) return [de(0), de(1)];
                  if (ye === 0) return [de(0), de(0)];
                  ye < 0 && (he = he.neg(), me = me.neg());
                  var ge = he.gcd(me);
                  return ge.cmpn(1) ? [he.div(ge), me.div(ge)] : [he, me];
                };
              }, 4387: function(ce, le, ue) {
                var de = ue(6859);
                ce.exports = function(fe) {
                  return new de(fe);
                };
              }, 6504: function(ce, le, ue) {
                var de = ue(869);
                ce.exports = function(fe, he) {
                  return de(fe[0].mul(he[0]), fe[1].mul(he[1]));
                };
              }, 7721: function(ce, le, ue) {
                var de = ue(5716);
                ce.exports = function(fe) {
                  return de(fe[0]) * de(fe[1]);
                };
              }, 5572: function(ce, le, ue) {
                var de = ue(869);
                ce.exports = function(fe, he) {
                  return de(fe[0].mul(he[1]).sub(fe[1].mul(he[0])), fe[1].mul(he[1]));
                };
              }, 946: function(ce, le, ue) {
                var de = ue(1369), fe = ue(4025);
                ce.exports = function(he) {
                  var me = he[0], pe = he[1];
                  if (me.cmpn(0) === 0) return 0;
                  var ye = me.abs().divmod(pe.abs()), ge = ye.div, ve = de(ge), be = ye.mod, Ce = me.negative !== pe.negative ? -1 : 1;
                  if (be.cmpn(0) === 0) return Ce * ve;
                  if (ve) {
                    var Se = fe(ve) + 4;
                    return Ce * (ve + (Ee = de(be.ushln(Se).divRound(pe))) * Math.pow(2, -Se));
                  }
                  var we = pe.bitLength() - be.bitLength() + 53, Ee = de(be.ushln(we).divRound(pe));
                  return we < 1023 ? Ce * Ee * Math.pow(2, -we) : Ce * (Ee *= Math.pow(2, -1023)) * Math.pow(2, 1023 - we);
                };
              }, 2478: function(ce) {
                function le(pe, ye, ge, ve, be) {
                  for (var Ce = be + 1; ve <= be; ) {
                    var Se = ve + be >>> 1, we = pe[Se];
                    (ge !== void 0 ? ge(we, ye) : we - ye) >= 0 ? (Ce = Se, be = Se - 1) : ve = Se + 1;
                  }
                  return Ce;
                }
                function ue(pe, ye, ge, ve, be) {
                  for (var Ce = be + 1; ve <= be; ) {
                    var Se = ve + be >>> 1, we = pe[Se];
                    (ge !== void 0 ? ge(we, ye) : we - ye) > 0 ? (Ce = Se, be = Se - 1) : ve = Se + 1;
                  }
                  return Ce;
                }
                function de(pe, ye, ge, ve, be) {
                  for (var Ce = ve - 1; ve <= be; ) {
                    var Se = ve + be >>> 1, we = pe[Se];
                    (ge !== void 0 ? ge(we, ye) : we - ye) < 0 ? (Ce = Se, ve = Se + 1) : be = Se - 1;
                  }
                  return Ce;
                }
                function fe(pe, ye, ge, ve, be) {
                  for (var Ce = ve - 1; ve <= be; ) {
                    var Se = ve + be >>> 1, we = pe[Se];
                    (ge !== void 0 ? ge(we, ye) : we - ye) <= 0 ? (Ce = Se, ve = Se + 1) : be = Se - 1;
                  }
                  return Ce;
                }
                function he(pe, ye, ge, ve, be) {
                  for (; ve <= be; ) {
                    var Ce = ve + be >>> 1, Se = pe[Ce], we = ge !== void 0 ? ge(Se, ye) : Se - ye;
                    if (we === 0) return Ce;
                    we <= 0 ? ve = Ce + 1 : be = Ce - 1;
                  }
                  return -1;
                }
                function me(pe, ye, ge, ve, be, Ce) {
                  return typeof ge == "function" ? Ce(pe, ye, ge, ve === void 0 ? 0 : 0 | ve, be === void 0 ? pe.length - 1 : 0 | be) : Ce(pe, ye, void 0, ge === void 0 ? 0 : 0 | ge, ve === void 0 ? pe.length - 1 : 0 | ve);
                }
                ce.exports = { ge: function(pe, ye, ge, ve, be) {
                  return me(pe, ye, ge, ve, be, le);
                }, gt: function(pe, ye, ge, ve, be) {
                  return me(pe, ye, ge, ve, be, ue);
                }, lt: function(pe, ye, ge, ve, be) {
                  return me(pe, ye, ge, ve, be, de);
                }, le: function(pe, ye, ge, ve, be) {
                  return me(pe, ye, ge, ve, be, fe);
                }, eq: function(pe, ye, ge, ve, be) {
                  return me(pe, ye, ge, ve, be, he);
                } };
              }, 8828: function(ce, le) {
                function ue(fe) {
                  var he = 32;
                  return (fe &= -fe) && he--, 65535 & fe && (he -= 16), 16711935 & fe && (he -= 8), 252645135 & fe && (he -= 4), 858993459 & fe && (he -= 2), 1431655765 & fe && (he -= 1), he;
                }
                le.INT_BITS = 32, le.INT_MAX = 2147483647, le.INT_MIN = -2147483648, le.sign = function(fe) {
                  return (fe > 0) - (fe < 0);
                }, le.abs = function(fe) {
                  var he = fe >> 31;
                  return (fe ^ he) - he;
                }, le.min = function(fe, he) {
                  return he ^ (fe ^ he) & -(fe < he);
                }, le.max = function(fe, he) {
                  return fe ^ (fe ^ he) & -(fe < he);
                }, le.isPow2 = function(fe) {
                  return !(fe & fe - 1 || !fe);
                }, le.log2 = function(fe) {
                  var he, me;
                  return he = (fe > 65535) << 4, he |= me = ((fe >>>= he) > 255) << 3, he |= me = ((fe >>>= me) > 15) << 2, (he |= me = ((fe >>>= me) > 3) << 1) | (fe >>>= me) >> 1;
                }, le.log10 = function(fe) {
                  return fe >= 1e9 ? 9 : fe >= 1e8 ? 8 : fe >= 1e7 ? 7 : fe >= 1e6 ? 6 : fe >= 1e5 ? 5 : fe >= 1e4 ? 4 : fe >= 1e3 ? 3 : fe >= 100 ? 2 : fe >= 10 ? 1 : 0;
                }, le.popCount = function(fe) {
                  return 16843009 * ((fe = (858993459 & (fe -= fe >>> 1 & 1431655765)) + (fe >>> 2 & 858993459)) + (fe >>> 4) & 252645135) >>> 24;
                }, le.countTrailingZeros = ue, le.nextPow2 = function(fe) {
                  return fe += fe === 0, --fe, fe |= fe >>> 1, fe |= fe >>> 2, fe |= fe >>> 4, fe |= fe >>> 8, 1 + (fe |= fe >>> 16);
                }, le.prevPow2 = function(fe) {
                  return fe |= fe >>> 1, fe |= fe >>> 2, fe |= fe >>> 4, fe |= fe >>> 8, (fe |= fe >>> 16) - (fe >>> 1);
                }, le.parity = function(fe) {
                  return fe ^= fe >>> 16, fe ^= fe >>> 8, fe ^= fe >>> 4, 27030 >>> (fe &= 15) & 1;
                };
                var de = new Array(256);
                (function(fe) {
                  for (var he = 0; he < 256; ++he) {
                    var me = he, pe = he, ye = 7;
                    for (me >>>= 1; me; me >>>= 1) pe <<= 1, pe |= 1 & me, --ye;
                    fe[he] = pe << ye & 255;
                  }
                })(de), le.reverse = function(fe) {
                  return de[255 & fe] << 24 | de[fe >>> 8 & 255] << 16 | de[fe >>> 16 & 255] << 8 | de[fe >>> 24 & 255];
                }, le.interleave2 = function(fe, he) {
                  return (fe = 1431655765 & ((fe = 858993459 & ((fe = 252645135 & ((fe = 16711935 & ((fe &= 65535) | fe << 8)) | fe << 4)) | fe << 2)) | fe << 1)) | (he = 1431655765 & ((he = 858993459 & ((he = 252645135 & ((he = 16711935 & ((he &= 65535) | he << 8)) | he << 4)) | he << 2)) | he << 1)) << 1;
                }, le.deinterleave2 = function(fe, he) {
                  return (fe = 65535 & ((fe = 16711935 & ((fe = 252645135 & ((fe = 858993459 & ((fe = fe >>> he & 1431655765) | fe >>> 1)) | fe >>> 2)) | fe >>> 4)) | fe >>> 16)) << 16 >> 16;
                }, le.interleave3 = function(fe, he, me) {
                  return fe = 1227133513 & ((fe = 3272356035 & ((fe = 251719695 & ((fe = 4278190335 & ((fe &= 1023) | fe << 16)) | fe << 8)) | fe << 4)) | fe << 2), (fe |= (he = 1227133513 & ((he = 3272356035 & ((he = 251719695 & ((he = 4278190335 & ((he &= 1023) | he << 16)) | he << 8)) | he << 4)) | he << 2)) << 1) | (me = 1227133513 & ((me = 3272356035 & ((me = 251719695 & ((me = 4278190335 & ((me &= 1023) | me << 16)) | me << 8)) | me << 4)) | me << 2)) << 2;
                }, le.deinterleave3 = function(fe, he) {
                  return (fe = 1023 & ((fe = 4278190335 & ((fe = 251719695 & ((fe = 3272356035 & ((fe = fe >>> he & 1227133513) | fe >>> 2)) | fe >>> 4)) | fe >>> 8)) | fe >>> 16)) << 22 >> 22;
                }, le.nextCombination = function(fe) {
                  var he = fe | fe - 1;
                  return he + 1 | (~he & -~he) - 1 >>> ue(fe) + 1;
                };
              }, 6859: function(ce, le, ue) {
                (function(de, fe) {
                  function he(Ve, Ze) {
                    if (!Ve) throw new Error(Ze || "Assertion failed");
                  }
                  function me(Ve, Ze) {
                    Ve.super_ = Ze;
                    var Ge = function() {
                    };
                    Ge.prototype = Ze.prototype, Ve.prototype = new Ge(), Ve.prototype.constructor = Ve;
                  }
                  function pe(Ve, Ze, Ge) {
                    if (pe.isBN(Ve)) return Ve;
                    this.negative = 0, this.words = null, this.length = 0, this.red = null, Ve !== null && (Ze !== "le" && Ze !== "be" || (Ge = Ze, Ze = 10), this._init(Ve || 0, Ze || 10, Ge || "be"));
                  }
                  var ye;
                  typeof de == "object" ? de.exports = pe : fe.BN = pe, pe.BN = pe, pe.wordSize = 26;
                  try {
                    ye = typeof window < "u" && window.Buffer !== void 0 ? window.Buffer : ue(7790).Buffer;
                  } catch {
                  }
                  function ge(Ve, Ze) {
                    var Ge = Ve.charCodeAt(Ze);
                    return Ge >= 65 && Ge <= 70 ? Ge - 55 : Ge >= 97 && Ge <= 102 ? Ge - 87 : Ge - 48 & 15;
                  }
                  function ve(Ve, Ze, Ge) {
                    var ze = ge(Ve, Ge);
                    return Ge - 1 >= Ze && (ze |= ge(Ve, Ge - 1) << 4), ze;
                  }
                  function be(Ve, Ze, Ge, ze) {
                    for (var We = 0, at = Math.min(Ve.length, Ge), ct = Ze; ct < at; ct++) {
                      var et = Ve.charCodeAt(ct) - 48;
                      We *= ze, We += et >= 49 ? et - 49 + 10 : et >= 17 ? et - 17 + 10 : et;
                    }
                    return We;
                  }
                  pe.isBN = function(Ve) {
                    return Ve instanceof pe || Ve !== null && typeof Ve == "object" && Ve.constructor.wordSize === pe.wordSize && Array.isArray(Ve.words);
                  }, pe.max = function(Ve, Ze) {
                    return Ve.cmp(Ze) > 0 ? Ve : Ze;
                  }, pe.min = function(Ve, Ze) {
                    return Ve.cmp(Ze) < 0 ? Ve : Ze;
                  }, pe.prototype._init = function(Ve, Ze, Ge) {
                    if (typeof Ve == "number") return this._initNumber(Ve, Ze, Ge);
                    if (typeof Ve == "object") return this._initArray(Ve, Ze, Ge);
                    Ze === "hex" && (Ze = 16), he(Ze === (0 | Ze) && Ze >= 2 && Ze <= 36);
                    var ze = 0;
                    (Ve = Ve.toString().replace(/\s+/g, ""))[0] === "-" && (ze++, this.negative = 1), ze < Ve.length && (Ze === 16 ? this._parseHex(Ve, ze, Ge) : (this._parseBase(Ve, Ze, ze), Ge === "le" && this._initArray(this.toArray(), Ze, Ge)));
                  }, pe.prototype._initNumber = function(Ve, Ze, Ge) {
                    Ve < 0 && (this.negative = 1, Ve = -Ve), Ve < 67108864 ? (this.words = [67108863 & Ve], this.length = 1) : Ve < 4503599627370496 ? (this.words = [67108863 & Ve, Ve / 67108864 & 67108863], this.length = 2) : (he(Ve < 9007199254740992), this.words = [67108863 & Ve, Ve / 67108864 & 67108863, 1], this.length = 3), Ge === "le" && this._initArray(this.toArray(), Ze, Ge);
                  }, pe.prototype._initArray = function(Ve, Ze, Ge) {
                    if (he(typeof Ve.length == "number"), Ve.length <= 0) return this.words = [0], this.length = 1, this;
                    this.length = Math.ceil(Ve.length / 3), this.words = new Array(this.length);
                    for (var ze = 0; ze < this.length; ze++) this.words[ze] = 0;
                    var We, at, ct = 0;
                    if (Ge === "be") for (ze = Ve.length - 1, We = 0; ze >= 0; ze -= 3) at = Ve[ze] | Ve[ze - 1] << 8 | Ve[ze - 2] << 16, this.words[We] |= at << ct & 67108863, this.words[We + 1] = at >>> 26 - ct & 67108863, (ct += 24) >= 26 && (ct -= 26, We++);
                    else if (Ge === "le") for (ze = 0, We = 0; ze < Ve.length; ze += 3) at = Ve[ze] | Ve[ze + 1] << 8 | Ve[ze + 2] << 16, this.words[We] |= at << ct & 67108863, this.words[We + 1] = at >>> 26 - ct & 67108863, (ct += 24) >= 26 && (ct -= 26, We++);
                    return this.strip();
                  }, pe.prototype._parseHex = function(Ve, Ze, Ge) {
                    this.length = Math.ceil((Ve.length - Ze) / 6), this.words = new Array(this.length);
                    for (var ze = 0; ze < this.length; ze++) this.words[ze] = 0;
                    var We, at = 0, ct = 0;
                    if (Ge === "be") for (ze = Ve.length - 1; ze >= Ze; ze -= 2) We = ve(Ve, Ze, ze) << at, this.words[ct] |= 67108863 & We, at >= 18 ? (at -= 18, ct += 1, this.words[ct] |= We >>> 26) : at += 8;
                    else for (ze = (Ve.length - Ze) % 2 == 0 ? Ze + 1 : Ze; ze < Ve.length; ze += 2) We = ve(Ve, Ze, ze) << at, this.words[ct] |= 67108863 & We, at >= 18 ? (at -= 18, ct += 1, this.words[ct] |= We >>> 26) : at += 8;
                    this.strip();
                  }, pe.prototype._parseBase = function(Ve, Ze, Ge) {
                    this.words = [0], this.length = 1;
                    for (var ze = 0, We = 1; We <= 67108863; We *= Ze) ze++;
                    ze--, We = We / Ze | 0;
                    for (var at = Ve.length - Ge, ct = at % ze, et = Math.min(at, at - ct) + Ge, dt = 0, Je = Ge; Je < et; Je += ze) dt = be(Ve, Je, Je + ze, Ze), this.imuln(We), this.words[0] + dt < 67108864 ? this.words[0] += dt : this._iaddn(dt);
                    if (ct !== 0) {
                      var it = 1;
                      for (dt = be(Ve, Je, Ve.length, Ze), Je = 0; Je < ct; Je++) it *= Ze;
                      this.imuln(it), this.words[0] + dt < 67108864 ? this.words[0] += dt : this._iaddn(dt);
                    }
                    this.strip();
                  }, pe.prototype.copy = function(Ve) {
                    Ve.words = new Array(this.length);
                    for (var Ze = 0; Ze < this.length; Ze++) Ve.words[Ze] = this.words[Ze];
                    Ve.length = this.length, Ve.negative = this.negative, Ve.red = this.red;
                  }, pe.prototype.clone = function() {
                    var Ve = new pe(null);
                    return this.copy(Ve), Ve;
                  }, pe.prototype._expand = function(Ve) {
                    for (; this.length < Ve; ) this.words[this.length++] = 0;
                    return this;
                  }, pe.prototype.strip = function() {
                    for (; this.length > 1 && this.words[this.length - 1] === 0; ) this.length--;
                    return this._normSign();
                  }, pe.prototype._normSign = function() {
                    return this.length === 1 && this.words[0] === 0 && (this.negative = 0), this;
                  }, pe.prototype.inspect = function() {
                    return (this.red ? "<BN-R: " : "<BN: ") + this.toString(16) + ">";
                  };
                  var Ce = ["", "0", "00", "000", "0000", "00000", "000000", "0000000", "00000000", "000000000", "0000000000", "00000000000", "000000000000", "0000000000000", "00000000000000", "000000000000000", "0000000000000000", "00000000000000000", "000000000000000000", "0000000000000000000", "00000000000000000000", "000000000000000000000", "0000000000000000000000", "00000000000000000000000", "000000000000000000000000", "0000000000000000000000000"], Se = [0, 0, 25, 16, 12, 11, 10, 9, 8, 8, 7, 7, 7, 7, 6, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5], we = [0, 0, 33554432, 43046721, 16777216, 48828125, 60466176, 40353607, 16777216, 43046721, 1e7, 19487171, 35831808, 62748517, 7529536, 11390625, 16777216, 24137569, 34012224, 47045881, 64e6, 4084101, 5153632, 6436343, 7962624, 9765625, 11881376, 14348907, 17210368, 20511149, 243e5, 28629151, 33554432, 39135393, 45435424, 52521875, 60466176];
                  function Ee(Ve, Ze, Ge) {
                    Ge.negative = Ze.negative ^ Ve.negative;
                    var ze = Ve.length + Ze.length | 0;
                    Ge.length = ze, ze = ze - 1 | 0;
                    var We = 0 | Ve.words[0], at = 0 | Ze.words[0], ct = We * at, et = 67108863 & ct, dt = ct / 67108864 | 0;
                    Ge.words[0] = et;
                    for (var Je = 1; Je < ze; Je++) {
                      for (var it = dt >>> 26, st = 67108863 & dt, gt = Math.min(Je, Ze.length - 1), bt = Math.max(0, Je - Ve.length + 1); bt <= gt; bt++) {
                        var yt = Je - bt | 0;
                        it += (ct = (We = 0 | Ve.words[yt]) * (at = 0 | Ze.words[bt]) + st) / 67108864 | 0, st = 67108863 & ct;
                      }
                      Ge.words[Je] = 0 | st, dt = 0 | it;
                    }
                    return dt !== 0 ? Ge.words[Je] = 0 | dt : Ge.length--, Ge.strip();
                  }
                  pe.prototype.toString = function(Ve, Ze) {
                    var Ge;
                    if (Ze = 0 | Ze || 1, (Ve = Ve || 10) === 16 || Ve === "hex") {
                      Ge = "";
                      for (var ze = 0, We = 0, at = 0; at < this.length; at++) {
                        var ct = this.words[at], et = (16777215 & (ct << ze | We)).toString(16);
                        Ge = (We = ct >>> 24 - ze & 16777215) != 0 || at !== this.length - 1 ? Ce[6 - et.length] + et + Ge : et + Ge, (ze += 2) >= 26 && (ze -= 26, at--);
                      }
                      for (We !== 0 && (Ge = We.toString(16) + Ge); Ge.length % Ze != 0; ) Ge = "0" + Ge;
                      return this.negative !== 0 && (Ge = "-" + Ge), Ge;
                    }
                    if (Ve === (0 | Ve) && Ve >= 2 && Ve <= 36) {
                      var dt = Se[Ve], Je = we[Ve];
                      Ge = "";
                      var it = this.clone();
                      for (it.negative = 0; !it.isZero(); ) {
                        var st = it.modn(Je).toString(Ve);
                        Ge = (it = it.idivn(Je)).isZero() ? st + Ge : Ce[dt - st.length] + st + Ge;
                      }
                      for (this.isZero() && (Ge = "0" + Ge); Ge.length % Ze != 0; ) Ge = "0" + Ge;
                      return this.negative !== 0 && (Ge = "-" + Ge), Ge;
                    }
                    he(!1, "Base should be between 2 and 36");
                  }, pe.prototype.toNumber = function() {
                    var Ve = this.words[0];
                    return this.length === 2 ? Ve += 67108864 * this.words[1] : this.length === 3 && this.words[2] === 1 ? Ve += 4503599627370496 + 67108864 * this.words[1] : this.length > 2 && he(!1, "Number can only safely store up to 53 bits"), this.negative !== 0 ? -Ve : Ve;
                  }, pe.prototype.toJSON = function() {
                    return this.toString(16);
                  }, pe.prototype.toBuffer = function(Ve, Ze) {
                    return he(ye !== void 0), this.toArrayLike(ye, Ve, Ze);
                  }, pe.prototype.toArray = function(Ve, Ze) {
                    return this.toArrayLike(Array, Ve, Ze);
                  }, pe.prototype.toArrayLike = function(Ve, Ze, Ge) {
                    var ze = this.byteLength(), We = Ge || Math.max(1, ze);
                    he(ze <= We, "byte array longer than desired length"), he(We > 0, "Requested array length <= 0"), this.strip();
                    var at, ct, et = Ze === "le", dt = new Ve(We), Je = this.clone();
                    if (et) {
                      for (ct = 0; !Je.isZero(); ct++) at = Je.andln(255), Je.iushrn(8), dt[ct] = at;
                      for (; ct < We; ct++) dt[ct] = 0;
                    } else {
                      for (ct = 0; ct < We - ze; ct++) dt[ct] = 0;
                      for (ct = 0; !Je.isZero(); ct++) at = Je.andln(255), Je.iushrn(8), dt[We - ct - 1] = at;
                    }
                    return dt;
                  }, Math.clz32 ? pe.prototype._countBits = function(Ve) {
                    return 32 - Math.clz32(Ve);
                  } : pe.prototype._countBits = function(Ve) {
                    var Ze = Ve, Ge = 0;
                    return Ze >= 4096 && (Ge += 13, Ze >>>= 13), Ze >= 64 && (Ge += 7, Ze >>>= 7), Ze >= 8 && (Ge += 4, Ze >>>= 4), Ze >= 2 && (Ge += 2, Ze >>>= 2), Ge + Ze;
                  }, pe.prototype._zeroBits = function(Ve) {
                    if (Ve === 0) return 26;
                    var Ze = Ve, Ge = 0;
                    return !(8191 & Ze) && (Ge += 13, Ze >>>= 13), !(127 & Ze) && (Ge += 7, Ze >>>= 7), !(15 & Ze) && (Ge += 4, Ze >>>= 4), !(3 & Ze) && (Ge += 2, Ze >>>= 2), !(1 & Ze) && Ge++, Ge;
                  }, pe.prototype.bitLength = function() {
                    var Ve = this.words[this.length - 1], Ze = this._countBits(Ve);
                    return 26 * (this.length - 1) + Ze;
                  }, pe.prototype.zeroBits = function() {
                    if (this.isZero()) return 0;
                    for (var Ve = 0, Ze = 0; Ze < this.length; Ze++) {
                      var Ge = this._zeroBits(this.words[Ze]);
                      if (Ve += Ge, Ge !== 26) break;
                    }
                    return Ve;
                  }, pe.prototype.byteLength = function() {
                    return Math.ceil(this.bitLength() / 8);
                  }, pe.prototype.toTwos = function(Ve) {
                    return this.negative !== 0 ? this.abs().inotn(Ve).iaddn(1) : this.clone();
                  }, pe.prototype.fromTwos = function(Ve) {
                    return this.testn(Ve - 1) ? this.notn(Ve).iaddn(1).ineg() : this.clone();
                  }, pe.prototype.isNeg = function() {
                    return this.negative !== 0;
                  }, pe.prototype.neg = function() {
                    return this.clone().ineg();
                  }, pe.prototype.ineg = function() {
                    return this.isZero() || (this.negative ^= 1), this;
                  }, pe.prototype.iuor = function(Ve) {
                    for (; this.length < Ve.length; ) this.words[this.length++] = 0;
                    for (var Ze = 0; Ze < Ve.length; Ze++) this.words[Ze] = this.words[Ze] | Ve.words[Ze];
                    return this.strip();
                  }, pe.prototype.ior = function(Ve) {
                    return he((this.negative | Ve.negative) == 0), this.iuor(Ve);
                  }, pe.prototype.or = function(Ve) {
                    return this.length > Ve.length ? this.clone().ior(Ve) : Ve.clone().ior(this);
                  }, pe.prototype.uor = function(Ve) {
                    return this.length > Ve.length ? this.clone().iuor(Ve) : Ve.clone().iuor(this);
                  }, pe.prototype.iuand = function(Ve) {
                    var Ze;
                    Ze = this.length > Ve.length ? Ve : this;
                    for (var Ge = 0; Ge < Ze.length; Ge++) this.words[Ge] = this.words[Ge] & Ve.words[Ge];
                    return this.length = Ze.length, this.strip();
                  }, pe.prototype.iand = function(Ve) {
                    return he((this.negative | Ve.negative) == 0), this.iuand(Ve);
                  }, pe.prototype.and = function(Ve) {
                    return this.length > Ve.length ? this.clone().iand(Ve) : Ve.clone().iand(this);
                  }, pe.prototype.uand = function(Ve) {
                    return this.length > Ve.length ? this.clone().iuand(Ve) : Ve.clone().iuand(this);
                  }, pe.prototype.iuxor = function(Ve) {
                    var Ze, Ge;
                    this.length > Ve.length ? (Ze = this, Ge = Ve) : (Ze = Ve, Ge = this);
                    for (var ze = 0; ze < Ge.length; ze++) this.words[ze] = Ze.words[ze] ^ Ge.words[ze];
                    if (this !== Ze) for (; ze < Ze.length; ze++) this.words[ze] = Ze.words[ze];
                    return this.length = Ze.length, this.strip();
                  }, pe.prototype.ixor = function(Ve) {
                    return he((this.negative | Ve.negative) == 0), this.iuxor(Ve);
                  }, pe.prototype.xor = function(Ve) {
                    return this.length > Ve.length ? this.clone().ixor(Ve) : Ve.clone().ixor(this);
                  }, pe.prototype.uxor = function(Ve) {
                    return this.length > Ve.length ? this.clone().iuxor(Ve) : Ve.clone().iuxor(this);
                  }, pe.prototype.inotn = function(Ve) {
                    he(typeof Ve == "number" && Ve >= 0);
                    var Ze = 0 | Math.ceil(Ve / 26), Ge = Ve % 26;
                    this._expand(Ze), Ge > 0 && Ze--;
                    for (var ze = 0; ze < Ze; ze++) this.words[ze] = 67108863 & ~this.words[ze];
                    return Ge > 0 && (this.words[ze] = ~this.words[ze] & 67108863 >> 26 - Ge), this.strip();
                  }, pe.prototype.notn = function(Ve) {
                    return this.clone().inotn(Ve);
                  }, pe.prototype.setn = function(Ve, Ze) {
                    he(typeof Ve == "number" && Ve >= 0);
                    var Ge = Ve / 26 | 0, ze = Ve % 26;
                    return this._expand(Ge + 1), this.words[Ge] = Ze ? this.words[Ge] | 1 << ze : this.words[Ge] & ~(1 << ze), this.strip();
                  }, pe.prototype.iadd = function(Ve) {
                    var Ze, Ge, ze;
                    if (this.negative !== 0 && Ve.negative === 0) return this.negative = 0, Ze = this.isub(Ve), this.negative ^= 1, this._normSign();
                    if (this.negative === 0 && Ve.negative !== 0) return Ve.negative = 0, Ze = this.isub(Ve), Ve.negative = 1, Ze._normSign();
                    this.length > Ve.length ? (Ge = this, ze = Ve) : (Ge = Ve, ze = this);
                    for (var We = 0, at = 0; at < ze.length; at++) Ze = (0 | Ge.words[at]) + (0 | ze.words[at]) + We, this.words[at] = 67108863 & Ze, We = Ze >>> 26;
                    for (; We !== 0 && at < Ge.length; at++) Ze = (0 | Ge.words[at]) + We, this.words[at] = 67108863 & Ze, We = Ze >>> 26;
                    if (this.length = Ge.length, We !== 0) this.words[this.length] = We, this.length++;
                    else if (Ge !== this) for (; at < Ge.length; at++) this.words[at] = Ge.words[at];
                    return this;
                  }, pe.prototype.add = function(Ve) {
                    var Ze;
                    return Ve.negative !== 0 && this.negative === 0 ? (Ve.negative = 0, Ze = this.sub(Ve), Ve.negative ^= 1, Ze) : Ve.negative === 0 && this.negative !== 0 ? (this.negative = 0, Ze = Ve.sub(this), this.negative = 1, Ze) : this.length > Ve.length ? this.clone().iadd(Ve) : Ve.clone().iadd(this);
                  }, pe.prototype.isub = function(Ve) {
                    if (Ve.negative !== 0) {
                      Ve.negative = 0;
                      var Ze = this.iadd(Ve);
                      return Ve.negative = 1, Ze._normSign();
                    }
                    if (this.negative !== 0) return this.negative = 0, this.iadd(Ve), this.negative = 1, this._normSign();
                    var Ge, ze, We = this.cmp(Ve);
                    if (We === 0) return this.negative = 0, this.length = 1, this.words[0] = 0, this;
                    We > 0 ? (Ge = this, ze = Ve) : (Ge = Ve, ze = this);
                    for (var at = 0, ct = 0; ct < ze.length; ct++) at = (Ze = (0 | Ge.words[ct]) - (0 | ze.words[ct]) + at) >> 26, this.words[ct] = 67108863 & Ze;
                    for (; at !== 0 && ct < Ge.length; ct++) at = (Ze = (0 | Ge.words[ct]) + at) >> 26, this.words[ct] = 67108863 & Ze;
                    if (at === 0 && ct < Ge.length && Ge !== this) for (; ct < Ge.length; ct++) this.words[ct] = Ge.words[ct];
                    return this.length = Math.max(this.length, ct), Ge !== this && (this.negative = 1), this.strip();
                  }, pe.prototype.sub = function(Ve) {
                    return this.clone().isub(Ve);
                  };
                  var xe = function(Ve, Ze, Ge) {
                    var ze, We, at, ct = Ve.words, et = Ze.words, dt = Ge.words, Je = 0, it = 0 | ct[0], st = 8191 & it, gt = it >>> 13, bt = 0 | ct[1], yt = 8191 & bt, St = bt >>> 13, Lt = 0 | ct[2], Rt = 8191 & Lt, vt = Lt >>> 13, $t = 0 | ct[3], Pt = 8191 & $t, Xt = $t >>> 13, Nt = 0 | ct[4], Gt = 8191 & Nt, Ft = Nt >>> 13, Tt = 0 | ct[5], It = 8191 & Tt, Ct = Tt >>> 13, Kt = 0 | ct[6], Vt = 8191 & Kt, Yt = Kt >>> 13, sn = 0 | ct[7], hn = 8191 & sn, Qt = sn >>> 13, Ut = 0 | ct[8], Wt = 8191 & Ut, rn = Ut >>> 13, vn = 0 | ct[9], dn = 8191 & vn, Ht = vn >>> 13, jt = 0 | et[0], cn = 8191 & jt, wn = jt >>> 13, Rn = 0 | et[1], Pn = 8191 & Rn, Ie = Rn >>> 13, rt = 0 | et[2], ft = 8191 & rt, pt = rt >>> 13, wt = 0 | et[3], Bt = 8191 & wt, At = wt >>> 13, Dt = 0 | et[4], Ot = 8191 & Dt, un = Dt >>> 13, yn = 0 | et[5], _n = 8191 & yn, gn = yn >>> 13, bn = 0 | et[6], an = 8191 & bn, Zt = bn >>> 13, Mn = 0 | et[7], Ln = 8191 & Mn, In = Mn >>> 13, kn = 0 | et[8], er = 8191 & kn, jn = kn >>> 13, ur = 0 | et[9], Kn = 8191 & ur, Jn = ur >>> 13;
                    Ge.negative = Ve.negative ^ Ze.negative, Ge.length = 19;
                    var Lr = (Je + (ze = Math.imul(st, cn)) | 0) + ((8191 & (We = (We = Math.imul(st, wn)) + Math.imul(gt, cn) | 0)) << 13) | 0;
                    Je = ((at = Math.imul(gt, wn)) + (We >>> 13) | 0) + (Lr >>> 26) | 0, Lr &= 67108863, ze = Math.imul(yt, cn), We = (We = Math.imul(yt, wn)) + Math.imul(St, cn) | 0, at = Math.imul(St, wn);
                    var mr = (Je + (ze = ze + Math.imul(st, Pn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, Ie) | 0) + Math.imul(gt, Pn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, Ie) | 0) + (We >>> 13) | 0) + (mr >>> 26) | 0, mr &= 67108863, ze = Math.imul(Rt, cn), We = (We = Math.imul(Rt, wn)) + Math.imul(vt, cn) | 0, at = Math.imul(vt, wn), ze = ze + Math.imul(yt, Pn) | 0, We = (We = We + Math.imul(yt, Ie) | 0) + Math.imul(St, Pn) | 0, at = at + Math.imul(St, Ie) | 0;
                    var Ar = (Je + (ze = ze + Math.imul(st, ft) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, pt) | 0) + Math.imul(gt, ft) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, pt) | 0) + (We >>> 13) | 0) + (Ar >>> 26) | 0, Ar &= 67108863, ze = Math.imul(Pt, cn), We = (We = Math.imul(Pt, wn)) + Math.imul(Xt, cn) | 0, at = Math.imul(Xt, wn), ze = ze + Math.imul(Rt, Pn) | 0, We = (We = We + Math.imul(Rt, Ie) | 0) + Math.imul(vt, Pn) | 0, at = at + Math.imul(vt, Ie) | 0, ze = ze + Math.imul(yt, ft) | 0, We = (We = We + Math.imul(yt, pt) | 0) + Math.imul(St, ft) | 0, at = at + Math.imul(St, pt) | 0;
                    var Yr = (Je + (ze = ze + Math.imul(st, Bt) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, At) | 0) + Math.imul(gt, Bt) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, At) | 0) + (We >>> 13) | 0) + (Yr >>> 26) | 0, Yr &= 67108863, ze = Math.imul(Gt, cn), We = (We = Math.imul(Gt, wn)) + Math.imul(Ft, cn) | 0, at = Math.imul(Ft, wn), ze = ze + Math.imul(Pt, Pn) | 0, We = (We = We + Math.imul(Pt, Ie) | 0) + Math.imul(Xt, Pn) | 0, at = at + Math.imul(Xt, Ie) | 0, ze = ze + Math.imul(Rt, ft) | 0, We = (We = We + Math.imul(Rt, pt) | 0) + Math.imul(vt, ft) | 0, at = at + Math.imul(vt, pt) | 0, ze = ze + Math.imul(yt, Bt) | 0, We = (We = We + Math.imul(yt, At) | 0) + Math.imul(St, Bt) | 0, at = at + Math.imul(St, At) | 0;
                    var jr = (Je + (ze = ze + Math.imul(st, Ot) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, un) | 0) + Math.imul(gt, Ot) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, un) | 0) + (We >>> 13) | 0) + (jr >>> 26) | 0, jr &= 67108863, ze = Math.imul(It, cn), We = (We = Math.imul(It, wn)) + Math.imul(Ct, cn) | 0, at = Math.imul(Ct, wn), ze = ze + Math.imul(Gt, Pn) | 0, We = (We = We + Math.imul(Gt, Ie) | 0) + Math.imul(Ft, Pn) | 0, at = at + Math.imul(Ft, Ie) | 0, ze = ze + Math.imul(Pt, ft) | 0, We = (We = We + Math.imul(Pt, pt) | 0) + Math.imul(Xt, ft) | 0, at = at + Math.imul(Xt, pt) | 0, ze = ze + Math.imul(Rt, Bt) | 0, We = (We = We + Math.imul(Rt, At) | 0) + Math.imul(vt, Bt) | 0, at = at + Math.imul(vt, At) | 0, ze = ze + Math.imul(yt, Ot) | 0, We = (We = We + Math.imul(yt, un) | 0) + Math.imul(St, Ot) | 0, at = at + Math.imul(St, un) | 0;
                    var Mr = (Je + (ze = ze + Math.imul(st, _n) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, gn) | 0) + Math.imul(gt, _n) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, gn) | 0) + (We >>> 13) | 0) + (Mr >>> 26) | 0, Mr &= 67108863, ze = Math.imul(Vt, cn), We = (We = Math.imul(Vt, wn)) + Math.imul(Yt, cn) | 0, at = Math.imul(Yt, wn), ze = ze + Math.imul(It, Pn) | 0, We = (We = We + Math.imul(It, Ie) | 0) + Math.imul(Ct, Pn) | 0, at = at + Math.imul(Ct, Ie) | 0, ze = ze + Math.imul(Gt, ft) | 0, We = (We = We + Math.imul(Gt, pt) | 0) + Math.imul(Ft, ft) | 0, at = at + Math.imul(Ft, pt) | 0, ze = ze + Math.imul(Pt, Bt) | 0, We = (We = We + Math.imul(Pt, At) | 0) + Math.imul(Xt, Bt) | 0, at = at + Math.imul(Xt, At) | 0, ze = ze + Math.imul(Rt, Ot) | 0, We = (We = We + Math.imul(Rt, un) | 0) + Math.imul(vt, Ot) | 0, at = at + Math.imul(vt, un) | 0, ze = ze + Math.imul(yt, _n) | 0, We = (We = We + Math.imul(yt, gn) | 0) + Math.imul(St, _n) | 0, at = at + Math.imul(St, gn) | 0;
                    var Wr = (Je + (ze = ze + Math.imul(st, an) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, Zt) | 0) + Math.imul(gt, an) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, Zt) | 0) + (We >>> 13) | 0) + (Wr >>> 26) | 0, Wr &= 67108863, ze = Math.imul(hn, cn), We = (We = Math.imul(hn, wn)) + Math.imul(Qt, cn) | 0, at = Math.imul(Qt, wn), ze = ze + Math.imul(Vt, Pn) | 0, We = (We = We + Math.imul(Vt, Ie) | 0) + Math.imul(Yt, Pn) | 0, at = at + Math.imul(Yt, Ie) | 0, ze = ze + Math.imul(It, ft) | 0, We = (We = We + Math.imul(It, pt) | 0) + Math.imul(Ct, ft) | 0, at = at + Math.imul(Ct, pt) | 0, ze = ze + Math.imul(Gt, Bt) | 0, We = (We = We + Math.imul(Gt, At) | 0) + Math.imul(Ft, Bt) | 0, at = at + Math.imul(Ft, At) | 0, ze = ze + Math.imul(Pt, Ot) | 0, We = (We = We + Math.imul(Pt, un) | 0) + Math.imul(Xt, Ot) | 0, at = at + Math.imul(Xt, un) | 0, ze = ze + Math.imul(Rt, _n) | 0, We = (We = We + Math.imul(Rt, gn) | 0) + Math.imul(vt, _n) | 0, at = at + Math.imul(vt, gn) | 0, ze = ze + Math.imul(yt, an) | 0, We = (We = We + Math.imul(yt, Zt) | 0) + Math.imul(St, an) | 0, at = at + Math.imul(St, Zt) | 0;
                    var Bi = (Je + (ze = ze + Math.imul(st, Ln) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, In) | 0) + Math.imul(gt, Ln) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, In) | 0) + (We >>> 13) | 0) + (Bi >>> 26) | 0, Bi &= 67108863, ze = Math.imul(Wt, cn), We = (We = Math.imul(Wt, wn)) + Math.imul(rn, cn) | 0, at = Math.imul(rn, wn), ze = ze + Math.imul(hn, Pn) | 0, We = (We = We + Math.imul(hn, Ie) | 0) + Math.imul(Qt, Pn) | 0, at = at + Math.imul(Qt, Ie) | 0, ze = ze + Math.imul(Vt, ft) | 0, We = (We = We + Math.imul(Vt, pt) | 0) + Math.imul(Yt, ft) | 0, at = at + Math.imul(Yt, pt) | 0, ze = ze + Math.imul(It, Bt) | 0, We = (We = We + Math.imul(It, At) | 0) + Math.imul(Ct, Bt) | 0, at = at + Math.imul(Ct, At) | 0, ze = ze + Math.imul(Gt, Ot) | 0, We = (We = We + Math.imul(Gt, un) | 0) + Math.imul(Ft, Ot) | 0, at = at + Math.imul(Ft, un) | 0, ze = ze + Math.imul(Pt, _n) | 0, We = (We = We + Math.imul(Pt, gn) | 0) + Math.imul(Xt, _n) | 0, at = at + Math.imul(Xt, gn) | 0, ze = ze + Math.imul(Rt, an) | 0, We = (We = We + Math.imul(Rt, Zt) | 0) + Math.imul(vt, an) | 0, at = at + Math.imul(vt, Zt) | 0, ze = ze + Math.imul(yt, Ln) | 0, We = (We = We + Math.imul(yt, In) | 0) + Math.imul(St, Ln) | 0, at = at + Math.imul(St, In) | 0;
                    var gi = (Je + (ze = ze + Math.imul(st, er) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, jn) | 0) + Math.imul(gt, er) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, jn) | 0) + (We >>> 13) | 0) + (gi >>> 26) | 0, gi &= 67108863, ze = Math.imul(dn, cn), We = (We = Math.imul(dn, wn)) + Math.imul(Ht, cn) | 0, at = Math.imul(Ht, wn), ze = ze + Math.imul(Wt, Pn) | 0, We = (We = We + Math.imul(Wt, Ie) | 0) + Math.imul(rn, Pn) | 0, at = at + Math.imul(rn, Ie) | 0, ze = ze + Math.imul(hn, ft) | 0, We = (We = We + Math.imul(hn, pt) | 0) + Math.imul(Qt, ft) | 0, at = at + Math.imul(Qt, pt) | 0, ze = ze + Math.imul(Vt, Bt) | 0, We = (We = We + Math.imul(Vt, At) | 0) + Math.imul(Yt, Bt) | 0, at = at + Math.imul(Yt, At) | 0, ze = ze + Math.imul(It, Ot) | 0, We = (We = We + Math.imul(It, un) | 0) + Math.imul(Ct, Ot) | 0, at = at + Math.imul(Ct, un) | 0, ze = ze + Math.imul(Gt, _n) | 0, We = (We = We + Math.imul(Gt, gn) | 0) + Math.imul(Ft, _n) | 0, at = at + Math.imul(Ft, gn) | 0, ze = ze + Math.imul(Pt, an) | 0, We = (We = We + Math.imul(Pt, Zt) | 0) + Math.imul(Xt, an) | 0, at = at + Math.imul(Xt, Zt) | 0, ze = ze + Math.imul(Rt, Ln) | 0, We = (We = We + Math.imul(Rt, In) | 0) + Math.imul(vt, Ln) | 0, at = at + Math.imul(vt, In) | 0, ze = ze + Math.imul(yt, er) | 0, We = (We = We + Math.imul(yt, jn) | 0) + Math.imul(St, er) | 0, at = at + Math.imul(St, jn) | 0;
                    var Zr = (Je + (ze = ze + Math.imul(st, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(st, Jn) | 0) + Math.imul(gt, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(gt, Jn) | 0) + (We >>> 13) | 0) + (Zr >>> 26) | 0, Zr &= 67108863, ze = Math.imul(dn, Pn), We = (We = Math.imul(dn, Ie)) + Math.imul(Ht, Pn) | 0, at = Math.imul(Ht, Ie), ze = ze + Math.imul(Wt, ft) | 0, We = (We = We + Math.imul(Wt, pt) | 0) + Math.imul(rn, ft) | 0, at = at + Math.imul(rn, pt) | 0, ze = ze + Math.imul(hn, Bt) | 0, We = (We = We + Math.imul(hn, At) | 0) + Math.imul(Qt, Bt) | 0, at = at + Math.imul(Qt, At) | 0, ze = ze + Math.imul(Vt, Ot) | 0, We = (We = We + Math.imul(Vt, un) | 0) + Math.imul(Yt, Ot) | 0, at = at + Math.imul(Yt, un) | 0, ze = ze + Math.imul(It, _n) | 0, We = (We = We + Math.imul(It, gn) | 0) + Math.imul(Ct, _n) | 0, at = at + Math.imul(Ct, gn) | 0, ze = ze + Math.imul(Gt, an) | 0, We = (We = We + Math.imul(Gt, Zt) | 0) + Math.imul(Ft, an) | 0, at = at + Math.imul(Ft, Zt) | 0, ze = ze + Math.imul(Pt, Ln) | 0, We = (We = We + Math.imul(Pt, In) | 0) + Math.imul(Xt, Ln) | 0, at = at + Math.imul(Xt, In) | 0, ze = ze + Math.imul(Rt, er) | 0, We = (We = We + Math.imul(Rt, jn) | 0) + Math.imul(vt, er) | 0, at = at + Math.imul(vt, jn) | 0;
                    var fi = (Je + (ze = ze + Math.imul(yt, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(yt, Jn) | 0) + Math.imul(St, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(St, Jn) | 0) + (We >>> 13) | 0) + (fi >>> 26) | 0, fi &= 67108863, ze = Math.imul(dn, ft), We = (We = Math.imul(dn, pt)) + Math.imul(Ht, ft) | 0, at = Math.imul(Ht, pt), ze = ze + Math.imul(Wt, Bt) | 0, We = (We = We + Math.imul(Wt, At) | 0) + Math.imul(rn, Bt) | 0, at = at + Math.imul(rn, At) | 0, ze = ze + Math.imul(hn, Ot) | 0, We = (We = We + Math.imul(hn, un) | 0) + Math.imul(Qt, Ot) | 0, at = at + Math.imul(Qt, un) | 0, ze = ze + Math.imul(Vt, _n) | 0, We = (We = We + Math.imul(Vt, gn) | 0) + Math.imul(Yt, _n) | 0, at = at + Math.imul(Yt, gn) | 0, ze = ze + Math.imul(It, an) | 0, We = (We = We + Math.imul(It, Zt) | 0) + Math.imul(Ct, an) | 0, at = at + Math.imul(Ct, Zt) | 0, ze = ze + Math.imul(Gt, Ln) | 0, We = (We = We + Math.imul(Gt, In) | 0) + Math.imul(Ft, Ln) | 0, at = at + Math.imul(Ft, In) | 0, ze = ze + Math.imul(Pt, er) | 0, We = (We = We + Math.imul(Pt, jn) | 0) + Math.imul(Xt, er) | 0, at = at + Math.imul(Xt, jn) | 0;
                    var ri = (Je + (ze = ze + Math.imul(Rt, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(Rt, Jn) | 0) + Math.imul(vt, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(vt, Jn) | 0) + (We >>> 13) | 0) + (ri >>> 26) | 0, ri &= 67108863, ze = Math.imul(dn, Bt), We = (We = Math.imul(dn, At)) + Math.imul(Ht, Bt) | 0, at = Math.imul(Ht, At), ze = ze + Math.imul(Wt, Ot) | 0, We = (We = We + Math.imul(Wt, un) | 0) + Math.imul(rn, Ot) | 0, at = at + Math.imul(rn, un) | 0, ze = ze + Math.imul(hn, _n) | 0, We = (We = We + Math.imul(hn, gn) | 0) + Math.imul(Qt, _n) | 0, at = at + Math.imul(Qt, gn) | 0, ze = ze + Math.imul(Vt, an) | 0, We = (We = We + Math.imul(Vt, Zt) | 0) + Math.imul(Yt, an) | 0, at = at + Math.imul(Yt, Zt) | 0, ze = ze + Math.imul(It, Ln) | 0, We = (We = We + Math.imul(It, In) | 0) + Math.imul(Ct, Ln) | 0, at = at + Math.imul(Ct, In) | 0, ze = ze + Math.imul(Gt, er) | 0, We = (We = We + Math.imul(Gt, jn) | 0) + Math.imul(Ft, er) | 0, at = at + Math.imul(Ft, jn) | 0;
                    var bi = (Je + (ze = ze + Math.imul(Pt, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(Pt, Jn) | 0) + Math.imul(Xt, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(Xt, Jn) | 0) + (We >>> 13) | 0) + (bi >>> 26) | 0, bi &= 67108863, ze = Math.imul(dn, Ot), We = (We = Math.imul(dn, un)) + Math.imul(Ht, Ot) | 0, at = Math.imul(Ht, un), ze = ze + Math.imul(Wt, _n) | 0, We = (We = We + Math.imul(Wt, gn) | 0) + Math.imul(rn, _n) | 0, at = at + Math.imul(rn, gn) | 0, ze = ze + Math.imul(hn, an) | 0, We = (We = We + Math.imul(hn, Zt) | 0) + Math.imul(Qt, an) | 0, at = at + Math.imul(Qt, Zt) | 0, ze = ze + Math.imul(Vt, Ln) | 0, We = (We = We + Math.imul(Vt, In) | 0) + Math.imul(Yt, Ln) | 0, at = at + Math.imul(Yt, In) | 0, ze = ze + Math.imul(It, er) | 0, We = (We = We + Math.imul(It, jn) | 0) + Math.imul(Ct, er) | 0, at = at + Math.imul(Ct, jn) | 0;
                    var Zi = (Je + (ze = ze + Math.imul(Gt, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(Gt, Jn) | 0) + Math.imul(Ft, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(Ft, Jn) | 0) + (We >>> 13) | 0) + (Zi >>> 26) | 0, Zi &= 67108863, ze = Math.imul(dn, _n), We = (We = Math.imul(dn, gn)) + Math.imul(Ht, _n) | 0, at = Math.imul(Ht, gn), ze = ze + Math.imul(Wt, an) | 0, We = (We = We + Math.imul(Wt, Zt) | 0) + Math.imul(rn, an) | 0, at = at + Math.imul(rn, Zt) | 0, ze = ze + Math.imul(hn, Ln) | 0, We = (We = We + Math.imul(hn, In) | 0) + Math.imul(Qt, Ln) | 0, at = at + Math.imul(Qt, In) | 0, ze = ze + Math.imul(Vt, er) | 0, We = (We = We + Math.imul(Vt, jn) | 0) + Math.imul(Yt, er) | 0, at = at + Math.imul(Yt, jn) | 0;
                    var $i = (Je + (ze = ze + Math.imul(It, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(It, Jn) | 0) + Math.imul(Ct, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(Ct, Jn) | 0) + (We >>> 13) | 0) + ($i >>> 26) | 0, $i &= 67108863, ze = Math.imul(dn, an), We = (We = Math.imul(dn, Zt)) + Math.imul(Ht, an) | 0, at = Math.imul(Ht, Zt), ze = ze + Math.imul(Wt, Ln) | 0, We = (We = We + Math.imul(Wt, In) | 0) + Math.imul(rn, Ln) | 0, at = at + Math.imul(rn, In) | 0, ze = ze + Math.imul(hn, er) | 0, We = (We = We + Math.imul(hn, jn) | 0) + Math.imul(Qt, er) | 0, at = at + Math.imul(Qt, jn) | 0;
                    var Br = (Je + (ze = ze + Math.imul(Vt, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(Vt, Jn) | 0) + Math.imul(Yt, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(Yt, Jn) | 0) + (We >>> 13) | 0) + (Br >>> 26) | 0, Br &= 67108863, ze = Math.imul(dn, Ln), We = (We = Math.imul(dn, In)) + Math.imul(Ht, Ln) | 0, at = Math.imul(Ht, In), ze = ze + Math.imul(Wt, er) | 0, We = (We = We + Math.imul(Wt, jn) | 0) + Math.imul(rn, er) | 0, at = at + Math.imul(rn, jn) | 0;
                    var Pr = (Je + (ze = ze + Math.imul(hn, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(hn, Jn) | 0) + Math.imul(Qt, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(Qt, Jn) | 0) + (We >>> 13) | 0) + (Pr >>> 26) | 0, Pr &= 67108863, ze = Math.imul(dn, er), We = (We = Math.imul(dn, jn)) + Math.imul(Ht, er) | 0, at = Math.imul(Ht, jn);
                    var ei = (Je + (ze = ze + Math.imul(Wt, Kn) | 0) | 0) + ((8191 & (We = (We = We + Math.imul(Wt, Jn) | 0) + Math.imul(rn, Kn) | 0)) << 13) | 0;
                    Je = ((at = at + Math.imul(rn, Jn) | 0) + (We >>> 13) | 0) + (ei >>> 26) | 0, ei &= 67108863;
                    var Ci = (Je + (ze = Math.imul(dn, Kn)) | 0) + ((8191 & (We = (We = Math.imul(dn, Jn)) + Math.imul(Ht, Kn) | 0)) << 13) | 0;
                    return Je = ((at = Math.imul(Ht, Jn)) + (We >>> 13) | 0) + (Ci >>> 26) | 0, Ci &= 67108863, dt[0] = Lr, dt[1] = mr, dt[2] = Ar, dt[3] = Yr, dt[4] = jr, dt[5] = Mr, dt[6] = Wr, dt[7] = Bi, dt[8] = gi, dt[9] = Zr, dt[10] = fi, dt[11] = ri, dt[12] = bi, dt[13] = Zi, dt[14] = $i, dt[15] = Br, dt[16] = Pr, dt[17] = ei, dt[18] = Ci, Je !== 0 && (dt[19] = Je, Ge.length++), Ge;
                  };
                  function Le(Ve, Ze, Ge) {
                    return new Qe().mulp(Ve, Ze, Ge);
                  }
                  function Qe(Ve, Ze) {
                    this.x = Ve, this.y = Ze;
                  }
                  Math.imul || (xe = Ee), pe.prototype.mulTo = function(Ve, Ze) {
                    var Ge, ze = this.length + Ve.length;
                    return Ge = this.length === 10 && Ve.length === 10 ? xe(this, Ve, Ze) : ze < 63 ? Ee(this, Ve, Ze) : ze < 1024 ? function(We, at, ct) {
                      ct.negative = at.negative ^ We.negative, ct.length = We.length + at.length;
                      for (var et = 0, dt = 0, Je = 0; Je < ct.length - 1; Je++) {
                        var it = dt;
                        dt = 0;
                        for (var st = 67108863 & et, gt = Math.min(Je, at.length - 1), bt = Math.max(0, Je - We.length + 1); bt <= gt; bt++) {
                          var yt = Je - bt, St = (0 | We.words[yt]) * (0 | at.words[bt]), Lt = 67108863 & St;
                          st = 67108863 & (Lt = Lt + st | 0), dt += (it = (it = it + (St / 67108864 | 0) | 0) + (Lt >>> 26) | 0) >>> 26, it &= 67108863;
                        }
                        ct.words[Je] = st, et = it, it = dt;
                      }
                      return et !== 0 ? ct.words[Je] = et : ct.length--, ct.strip();
                    }(this, Ve, Ze) : Le(this, Ve, Ze), Ge;
                  }, Qe.prototype.makeRBT = function(Ve) {
                    for (var Ze = new Array(Ve), Ge = pe.prototype._countBits(Ve) - 1, ze = 0; ze < Ve; ze++) Ze[ze] = this.revBin(ze, Ge, Ve);
                    return Ze;
                  }, Qe.prototype.revBin = function(Ve, Ze, Ge) {
                    if (Ve === 0 || Ve === Ge - 1) return Ve;
                    for (var ze = 0, We = 0; We < Ze; We++) ze |= (1 & Ve) << Ze - We - 1, Ve >>= 1;
                    return ze;
                  }, Qe.prototype.permute = function(Ve, Ze, Ge, ze, We, at) {
                    for (var ct = 0; ct < at; ct++) ze[ct] = Ze[Ve[ct]], We[ct] = Ge[Ve[ct]];
                  }, Qe.prototype.transform = function(Ve, Ze, Ge, ze, We, at) {
                    this.permute(at, Ve, Ze, Ge, ze, We);
                    for (var ct = 1; ct < We; ct <<= 1) for (var et = ct << 1, dt = Math.cos(2 * Math.PI / et), Je = Math.sin(2 * Math.PI / et), it = 0; it < We; it += et) for (var st = dt, gt = Je, bt = 0; bt < ct; bt++) {
                      var yt = Ge[it + bt], St = ze[it + bt], Lt = Ge[it + bt + ct], Rt = ze[it + bt + ct], vt = st * Lt - gt * Rt;
                      Rt = st * Rt + gt * Lt, Lt = vt, Ge[it + bt] = yt + Lt, ze[it + bt] = St + Rt, Ge[it + bt + ct] = yt - Lt, ze[it + bt + ct] = St - Rt, bt !== et && (vt = dt * st - Je * gt, gt = dt * gt + Je * st, st = vt);
                    }
                  }, Qe.prototype.guessLen13b = function(Ve, Ze) {
                    var Ge = 1 | Math.max(Ze, Ve), ze = 1 & Ge, We = 0;
                    for (Ge = Ge / 2 | 0; Ge; Ge >>>= 1) We++;
                    return 1 << We + 1 + ze;
                  }, Qe.prototype.conjugate = function(Ve, Ze, Ge) {
                    if (!(Ge <= 1)) for (var ze = 0; ze < Ge / 2; ze++) {
                      var We = Ve[ze];
                      Ve[ze] = Ve[Ge - ze - 1], Ve[Ge - ze - 1] = We, We = Ze[ze], Ze[ze] = -Ze[Ge - ze - 1], Ze[Ge - ze - 1] = -We;
                    }
                  }, Qe.prototype.normalize13b = function(Ve, Ze) {
                    for (var Ge = 0, ze = 0; ze < Ze / 2; ze++) {
                      var We = 8192 * Math.round(Ve[2 * ze + 1] / Ze) + Math.round(Ve[2 * ze] / Ze) + Ge;
                      Ve[ze] = 67108863 & We, Ge = We < 67108864 ? 0 : We / 67108864 | 0;
                    }
                    return Ve;
                  }, Qe.prototype.convert13b = function(Ve, Ze, Ge, ze) {
                    for (var We = 0, at = 0; at < Ze; at++) We += 0 | Ve[at], Ge[2 * at] = 8191 & We, We >>>= 13, Ge[2 * at + 1] = 8191 & We, We >>>= 13;
                    for (at = 2 * Ze; at < ze; ++at) Ge[at] = 0;
                    he(We === 0), he((-8192 & We) == 0);
                  }, Qe.prototype.stub = function(Ve) {
                    for (var Ze = new Array(Ve), Ge = 0; Ge < Ve; Ge++) Ze[Ge] = 0;
                    return Ze;
                  }, Qe.prototype.mulp = function(Ve, Ze, Ge) {
                    var ze = 2 * this.guessLen13b(Ve.length, Ze.length), We = this.makeRBT(ze), at = this.stub(ze), ct = new Array(ze), et = new Array(ze), dt = new Array(ze), Je = new Array(ze), it = new Array(ze), st = new Array(ze), gt = Ge.words;
                    gt.length = ze, this.convert13b(Ve.words, Ve.length, ct, ze), this.convert13b(Ze.words, Ze.length, Je, ze), this.transform(ct, at, et, dt, ze, We), this.transform(Je, at, it, st, ze, We);
                    for (var bt = 0; bt < ze; bt++) {
                      var yt = et[bt] * it[bt] - dt[bt] * st[bt];
                      dt[bt] = et[bt] * st[bt] + dt[bt] * it[bt], et[bt] = yt;
                    }
                    return this.conjugate(et, dt, ze), this.transform(et, dt, gt, at, ze, We), this.conjugate(gt, at, ze), this.normalize13b(gt, ze), Ge.negative = Ve.negative ^ Ze.negative, Ge.length = Ve.length + Ze.length, Ge.strip();
                  }, pe.prototype.mul = function(Ve) {
                    var Ze = new pe(null);
                    return Ze.words = new Array(this.length + Ve.length), this.mulTo(Ve, Ze);
                  }, pe.prototype.mulf = function(Ve) {
                    var Ze = new pe(null);
                    return Ze.words = new Array(this.length + Ve.length), Le(this, Ve, Ze);
                  }, pe.prototype.imul = function(Ve) {
                    return this.clone().mulTo(Ve, this);
                  }, pe.prototype.imuln = function(Ve) {
                    he(typeof Ve == "number"), he(Ve < 67108864);
                    for (var Ze = 0, Ge = 0; Ge < this.length; Ge++) {
                      var ze = (0 | this.words[Ge]) * Ve, We = (67108863 & ze) + (67108863 & Ze);
                      Ze >>= 26, Ze += ze / 67108864 | 0, Ze += We >>> 26, this.words[Ge] = 67108863 & We;
                    }
                    return Ze !== 0 && (this.words[Ge] = Ze, this.length++), this;
                  }, pe.prototype.muln = function(Ve) {
                    return this.clone().imuln(Ve);
                  }, pe.prototype.sqr = function() {
                    return this.mul(this);
                  }, pe.prototype.isqr = function() {
                    return this.imul(this.clone());
                  }, pe.prototype.pow = function(Ve) {
                    var Ze = function(at) {
                      for (var ct = new Array(at.bitLength()), et = 0; et < ct.length; et++) {
                        var dt = et / 26 | 0, Je = et % 26;
                        ct[et] = (at.words[dt] & 1 << Je) >>> Je;
                      }
                      return ct;
                    }(Ve);
                    if (Ze.length === 0) return new pe(1);
                    for (var Ge = this, ze = 0; ze < Ze.length && Ze[ze] === 0; ze++, Ge = Ge.sqr()) ;
                    if (++ze < Ze.length) for (var We = Ge.sqr(); ze < Ze.length; ze++, We = We.sqr()) Ze[ze] !== 0 && (Ge = Ge.mul(We));
                    return Ge;
                  }, pe.prototype.iushln = function(Ve) {
                    he(typeof Ve == "number" && Ve >= 0);
                    var Ze, Ge = Ve % 26, ze = (Ve - Ge) / 26, We = 67108863 >>> 26 - Ge << 26 - Ge;
                    if (Ge !== 0) {
                      var at = 0;
                      for (Ze = 0; Ze < this.length; Ze++) {
                        var ct = this.words[Ze] & We, et = (0 | this.words[Ze]) - ct << Ge;
                        this.words[Ze] = et | at, at = ct >>> 26 - Ge;
                      }
                      at && (this.words[Ze] = at, this.length++);
                    }
                    if (ze !== 0) {
                      for (Ze = this.length - 1; Ze >= 0; Ze--) this.words[Ze + ze] = this.words[Ze];
                      for (Ze = 0; Ze < ze; Ze++) this.words[Ze] = 0;
                      this.length += ze;
                    }
                    return this.strip();
                  }, pe.prototype.ishln = function(Ve) {
                    return he(this.negative === 0), this.iushln(Ve);
                  }, pe.prototype.iushrn = function(Ve, Ze, Ge) {
                    var ze;
                    he(typeof Ve == "number" && Ve >= 0), ze = Ze ? (Ze - Ze % 26) / 26 : 0;
                    var We = Ve % 26, at = Math.min((Ve - We) / 26, this.length), ct = 67108863 ^ 67108863 >>> We << We, et = Ge;
                    if (ze -= at, ze = Math.max(0, ze), et) {
                      for (var dt = 0; dt < at; dt++) et.words[dt] = this.words[dt];
                      et.length = at;
                    }
                    if (at !== 0) if (this.length > at) for (this.length -= at, dt = 0; dt < this.length; dt++) this.words[dt] = this.words[dt + at];
                    else this.words[0] = 0, this.length = 1;
                    var Je = 0;
                    for (dt = this.length - 1; dt >= 0 && (Je !== 0 || dt >= ze); dt--) {
                      var it = 0 | this.words[dt];
                      this.words[dt] = Je << 26 - We | it >>> We, Je = it & ct;
                    }
                    return et && Je !== 0 && (et.words[et.length++] = Je), this.length === 0 && (this.words[0] = 0, this.length = 1), this.strip();
                  }, pe.prototype.ishrn = function(Ve, Ze, Ge) {
                    return he(this.negative === 0), this.iushrn(Ve, Ze, Ge);
                  }, pe.prototype.shln = function(Ve) {
                    return this.clone().ishln(Ve);
                  }, pe.prototype.ushln = function(Ve) {
                    return this.clone().iushln(Ve);
                  }, pe.prototype.shrn = function(Ve) {
                    return this.clone().ishrn(Ve);
                  }, pe.prototype.ushrn = function(Ve) {
                    return this.clone().iushrn(Ve);
                  }, pe.prototype.testn = function(Ve) {
                    he(typeof Ve == "number" && Ve >= 0);
                    var Ze = Ve % 26, Ge = (Ve - Ze) / 26, ze = 1 << Ze;
                    return !(this.length <= Ge || !(this.words[Ge] & ze));
                  }, pe.prototype.imaskn = function(Ve) {
                    he(typeof Ve == "number" && Ve >= 0);
                    var Ze = Ve % 26, Ge = (Ve - Ze) / 26;
                    if (he(this.negative === 0, "imaskn works only with positive numbers"), this.length <= Ge) return this;
                    if (Ze !== 0 && Ge++, this.length = Math.min(Ge, this.length), Ze !== 0) {
                      var ze = 67108863 ^ 67108863 >>> Ze << Ze;
                      this.words[this.length - 1] &= ze;
                    }
                    return this.strip();
                  }, pe.prototype.maskn = function(Ve) {
                    return this.clone().imaskn(Ve);
                  }, pe.prototype.iaddn = function(Ve) {
                    return he(typeof Ve == "number"), he(Ve < 67108864), Ve < 0 ? this.isubn(-Ve) : this.negative !== 0 ? this.length === 1 && (0 | this.words[0]) < Ve ? (this.words[0] = Ve - (0 | this.words[0]), this.negative = 0, this) : (this.negative = 0, this.isubn(Ve), this.negative = 1, this) : this._iaddn(Ve);
                  }, pe.prototype._iaddn = function(Ve) {
                    this.words[0] += Ve;
                    for (var Ze = 0; Ze < this.length && this.words[Ze] >= 67108864; Ze++) this.words[Ze] -= 67108864, Ze === this.length - 1 ? this.words[Ze + 1] = 1 : this.words[Ze + 1]++;
                    return this.length = Math.max(this.length, Ze + 1), this;
                  }, pe.prototype.isubn = function(Ve) {
                    if (he(typeof Ve == "number"), he(Ve < 67108864), Ve < 0) return this.iaddn(-Ve);
                    if (this.negative !== 0) return this.negative = 0, this.iaddn(Ve), this.negative = 1, this;
                    if (this.words[0] -= Ve, this.length === 1 && this.words[0] < 0) this.words[0] = -this.words[0], this.negative = 1;
                    else for (var Ze = 0; Ze < this.length && this.words[Ze] < 0; Ze++) this.words[Ze] += 67108864, this.words[Ze + 1] -= 1;
                    return this.strip();
                  }, pe.prototype.addn = function(Ve) {
                    return this.clone().iaddn(Ve);
                  }, pe.prototype.subn = function(Ve) {
                    return this.clone().isubn(Ve);
                  }, pe.prototype.iabs = function() {
                    return this.negative = 0, this;
                  }, pe.prototype.abs = function() {
                    return this.clone().iabs();
                  }, pe.prototype._ishlnsubmul = function(Ve, Ze, Ge) {
                    var ze, We, at = Ve.length + Ge;
                    this._expand(at);
                    var ct = 0;
                    for (ze = 0; ze < Ve.length; ze++) {
                      We = (0 | this.words[ze + Ge]) + ct;
                      var et = (0 | Ve.words[ze]) * Ze;
                      ct = ((We -= 67108863 & et) >> 26) - (et / 67108864 | 0), this.words[ze + Ge] = 67108863 & We;
                    }
                    for (; ze < this.length - Ge; ze++) ct = (We = (0 | this.words[ze + Ge]) + ct) >> 26, this.words[ze + Ge] = 67108863 & We;
                    if (ct === 0) return this.strip();
                    for (he(ct === -1), ct = 0, ze = 0; ze < this.length; ze++) ct = (We = -(0 | this.words[ze]) + ct) >> 26, this.words[ze] = 67108863 & We;
                    return this.negative = 1, this.strip();
                  }, pe.prototype._wordDiv = function(Ve, Ze) {
                    var Ge = (this.length, Ve.length), ze = this.clone(), We = Ve, at = 0 | We.words[We.length - 1];
                    (Ge = 26 - this._countBits(at)) != 0 && (We = We.ushln(Ge), ze.iushln(Ge), at = 0 | We.words[We.length - 1]);
                    var ct, et = ze.length - We.length;
                    if (Ze !== "mod") {
                      (ct = new pe(null)).length = et + 1, ct.words = new Array(ct.length);
                      for (var dt = 0; dt < ct.length; dt++) ct.words[dt] = 0;
                    }
                    var Je = ze.clone()._ishlnsubmul(We, 1, et);
                    Je.negative === 0 && (ze = Je, ct && (ct.words[et] = 1));
                    for (var it = et - 1; it >= 0; it--) {
                      var st = 67108864 * (0 | ze.words[We.length + it]) + (0 | ze.words[We.length + it - 1]);
                      for (st = Math.min(st / at | 0, 67108863), ze._ishlnsubmul(We, st, it); ze.negative !== 0; ) st--, ze.negative = 0, ze._ishlnsubmul(We, 1, it), ze.isZero() || (ze.negative ^= 1);
                      ct && (ct.words[it] = st);
                    }
                    return ct && ct.strip(), ze.strip(), Ze !== "div" && Ge !== 0 && ze.iushrn(Ge), { div: ct || null, mod: ze };
                  }, pe.prototype.divmod = function(Ve, Ze, Ge) {
                    return he(!Ve.isZero()), this.isZero() ? { div: new pe(0), mod: new pe(0) } : this.negative !== 0 && Ve.negative === 0 ? (at = this.neg().divmod(Ve, Ze), Ze !== "mod" && (ze = at.div.neg()), Ze !== "div" && (We = at.mod.neg(), Ge && We.negative !== 0 && We.iadd(Ve)), { div: ze, mod: We }) : this.negative === 0 && Ve.negative !== 0 ? (at = this.divmod(Ve.neg(), Ze), Ze !== "mod" && (ze = at.div.neg()), { div: ze, mod: at.mod }) : this.negative & Ve.negative ? (at = this.neg().divmod(Ve.neg(), Ze), Ze !== "div" && (We = at.mod.neg(), Ge && We.negative !== 0 && We.isub(Ve)), { div: at.div, mod: We }) : Ve.length > this.length || this.cmp(Ve) < 0 ? { div: new pe(0), mod: this } : Ve.length === 1 ? Ze === "div" ? { div: this.divn(Ve.words[0]), mod: null } : Ze === "mod" ? { div: null, mod: new pe(this.modn(Ve.words[0])) } : { div: this.divn(Ve.words[0]), mod: new pe(this.modn(Ve.words[0])) } : this._wordDiv(Ve, Ze);
                    var ze, We, at;
                  }, pe.prototype.div = function(Ve) {
                    return this.divmod(Ve, "div", !1).div;
                  }, pe.prototype.mod = function(Ve) {
                    return this.divmod(Ve, "mod", !1).mod;
                  }, pe.prototype.umod = function(Ve) {
                    return this.divmod(Ve, "mod", !0).mod;
                  }, pe.prototype.divRound = function(Ve) {
                    var Ze = this.divmod(Ve);
                    if (Ze.mod.isZero()) return Ze.div;
                    var Ge = Ze.div.negative !== 0 ? Ze.mod.isub(Ve) : Ze.mod, ze = Ve.ushrn(1), We = Ve.andln(1), at = Ge.cmp(ze);
                    return at < 0 || We === 1 && at === 0 ? Ze.div : Ze.div.negative !== 0 ? Ze.div.isubn(1) : Ze.div.iaddn(1);
                  }, pe.prototype.modn = function(Ve) {
                    he(Ve <= 67108863);
                    for (var Ze = 67108864 % Ve, Ge = 0, ze = this.length - 1; ze >= 0; ze--) Ge = (Ze * Ge + (0 | this.words[ze])) % Ve;
                    return Ge;
                  }, pe.prototype.idivn = function(Ve) {
                    he(Ve <= 67108863);
                    for (var Ze = 0, Ge = this.length - 1; Ge >= 0; Ge--) {
                      var ze = (0 | this.words[Ge]) + 67108864 * Ze;
                      this.words[Ge] = ze / Ve | 0, Ze = ze % Ve;
                    }
                    return this.strip();
                  }, pe.prototype.divn = function(Ve) {
                    return this.clone().idivn(Ve);
                  }, pe.prototype.egcd = function(Ve) {
                    he(Ve.negative === 0), he(!Ve.isZero());
                    var Ze = this, Ge = Ve.clone();
                    Ze = Ze.negative !== 0 ? Ze.umod(Ve) : Ze.clone();
                    for (var ze = new pe(1), We = new pe(0), at = new pe(0), ct = new pe(1), et = 0; Ze.isEven() && Ge.isEven(); ) Ze.iushrn(1), Ge.iushrn(1), ++et;
                    for (var dt = Ge.clone(), Je = Ze.clone(); !Ze.isZero(); ) {
                      for (var it = 0, st = 1; !(Ze.words[0] & st) && it < 26; ++it, st <<= 1) ;
                      if (it > 0) for (Ze.iushrn(it); it-- > 0; ) (ze.isOdd() || We.isOdd()) && (ze.iadd(dt), We.isub(Je)), ze.iushrn(1), We.iushrn(1);
                      for (var gt = 0, bt = 1; !(Ge.words[0] & bt) && gt < 26; ++gt, bt <<= 1) ;
                      if (gt > 0) for (Ge.iushrn(gt); gt-- > 0; ) (at.isOdd() || ct.isOdd()) && (at.iadd(dt), ct.isub(Je)), at.iushrn(1), ct.iushrn(1);
                      Ze.cmp(Ge) >= 0 ? (Ze.isub(Ge), ze.isub(at), We.isub(ct)) : (Ge.isub(Ze), at.isub(ze), ct.isub(We));
                    }
                    return { a: at, b: ct, gcd: Ge.iushln(et) };
                  }, pe.prototype._invmp = function(Ve) {
                    he(Ve.negative === 0), he(!Ve.isZero());
                    var Ze = this, Ge = Ve.clone();
                    Ze = Ze.negative !== 0 ? Ze.umod(Ve) : Ze.clone();
                    for (var ze, We = new pe(1), at = new pe(0), ct = Ge.clone(); Ze.cmpn(1) > 0 && Ge.cmpn(1) > 0; ) {
                      for (var et = 0, dt = 1; !(Ze.words[0] & dt) && et < 26; ++et, dt <<= 1) ;
                      if (et > 0) for (Ze.iushrn(et); et-- > 0; ) We.isOdd() && We.iadd(ct), We.iushrn(1);
                      for (var Je = 0, it = 1; !(Ge.words[0] & it) && Je < 26; ++Je, it <<= 1) ;
                      if (Je > 0) for (Ge.iushrn(Je); Je-- > 0; ) at.isOdd() && at.iadd(ct), at.iushrn(1);
                      Ze.cmp(Ge) >= 0 ? (Ze.isub(Ge), We.isub(at)) : (Ge.isub(Ze), at.isub(We));
                    }
                    return (ze = Ze.cmpn(1) === 0 ? We : at).cmpn(0) < 0 && ze.iadd(Ve), ze;
                  }, pe.prototype.gcd = function(Ve) {
                    if (this.isZero()) return Ve.abs();
                    if (Ve.isZero()) return this.abs();
                    var Ze = this.clone(), Ge = Ve.clone();
                    Ze.negative = 0, Ge.negative = 0;
                    for (var ze = 0; Ze.isEven() && Ge.isEven(); ze++) Ze.iushrn(1), Ge.iushrn(1);
                    for (; ; ) {
                      for (; Ze.isEven(); ) Ze.iushrn(1);
                      for (; Ge.isEven(); ) Ge.iushrn(1);
                      var We = Ze.cmp(Ge);
                      if (We < 0) {
                        var at = Ze;
                        Ze = Ge, Ge = at;
                      } else if (We === 0 || Ge.cmpn(1) === 0) break;
                      Ze.isub(Ge);
                    }
                    return Ge.iushln(ze);
                  }, pe.prototype.invm = function(Ve) {
                    return this.egcd(Ve).a.umod(Ve);
                  }, pe.prototype.isEven = function() {
                    return (1 & this.words[0]) == 0;
                  }, pe.prototype.isOdd = function() {
                    return (1 & this.words[0]) == 1;
                  }, pe.prototype.andln = function(Ve) {
                    return this.words[0] & Ve;
                  }, pe.prototype.bincn = function(Ve) {
                    he(typeof Ve == "number");
                    var Ze = Ve % 26, Ge = (Ve - Ze) / 26, ze = 1 << Ze;
                    if (this.length <= Ge) return this._expand(Ge + 1), this.words[Ge] |= ze, this;
                    for (var We = ze, at = Ge; We !== 0 && at < this.length; at++) {
                      var ct = 0 | this.words[at];
                      We = (ct += We) >>> 26, ct &= 67108863, this.words[at] = ct;
                    }
                    return We !== 0 && (this.words[at] = We, this.length++), this;
                  }, pe.prototype.isZero = function() {
                    return this.length === 1 && this.words[0] === 0;
                  }, pe.prototype.cmpn = function(Ve) {
                    var Ze, Ge = Ve < 0;
                    if (this.negative !== 0 && !Ge) return -1;
                    if (this.negative === 0 && Ge) return 1;
                    if (this.strip(), this.length > 1) Ze = 1;
                    else {
                      Ge && (Ve = -Ve), he(Ve <= 67108863, "Number is too big");
                      var ze = 0 | this.words[0];
                      Ze = ze === Ve ? 0 : ze < Ve ? -1 : 1;
                    }
                    return this.negative !== 0 ? 0 | -Ze : Ze;
                  }, pe.prototype.cmp = function(Ve) {
                    if (this.negative !== 0 && Ve.negative === 0) return -1;
                    if (this.negative === 0 && Ve.negative !== 0) return 1;
                    var Ze = this.ucmp(Ve);
                    return this.negative !== 0 ? 0 | -Ze : Ze;
                  }, pe.prototype.ucmp = function(Ve) {
                    if (this.length > Ve.length) return 1;
                    if (this.length < Ve.length) return -1;
                    for (var Ze = 0, Ge = this.length - 1; Ge >= 0; Ge--) {
                      var ze = 0 | this.words[Ge], We = 0 | Ve.words[Ge];
                      if (ze !== We) {
                        ze < We ? Ze = -1 : ze > We && (Ze = 1);
                        break;
                      }
                    }
                    return Ze;
                  }, pe.prototype.gtn = function(Ve) {
                    return this.cmpn(Ve) === 1;
                  }, pe.prototype.gt = function(Ve) {
                    return this.cmp(Ve) === 1;
                  }, pe.prototype.gten = function(Ve) {
                    return this.cmpn(Ve) >= 0;
                  }, pe.prototype.gte = function(Ve) {
                    return this.cmp(Ve) >= 0;
                  }, pe.prototype.ltn = function(Ve) {
                    return this.cmpn(Ve) === -1;
                  }, pe.prototype.lt = function(Ve) {
                    return this.cmp(Ve) === -1;
                  }, pe.prototype.lten = function(Ve) {
                    return this.cmpn(Ve) <= 0;
                  }, pe.prototype.lte = function(Ve) {
                    return this.cmp(Ve) <= 0;
                  }, pe.prototype.eqn = function(Ve) {
                    return this.cmpn(Ve) === 0;
                  }, pe.prototype.eq = function(Ve) {
                    return this.cmp(Ve) === 0;
                  }, pe.red = function(Ve) {
                    return new qe(Ve);
                  }, pe.prototype.toRed = function(Ve) {
                    return he(!this.red, "Already a number in reduction context"), he(this.negative === 0, "red works only with positives"), Ve.convertTo(this)._forceRed(Ve);
                  }, pe.prototype.fromRed = function() {
                    return he(this.red, "fromRed works only with numbers in reduction context"), this.red.convertFrom(this);
                  }, pe.prototype._forceRed = function(Ve) {
                    return this.red = Ve, this;
                  }, pe.prototype.forceRed = function(Ve) {
                    return he(!this.red, "Already a number in reduction context"), this._forceRed(Ve);
                  }, pe.prototype.redAdd = function(Ve) {
                    return he(this.red, "redAdd works only with red numbers"), this.red.add(this, Ve);
                  }, pe.prototype.redIAdd = function(Ve) {
                    return he(this.red, "redIAdd works only with red numbers"), this.red.iadd(this, Ve);
                  }, pe.prototype.redSub = function(Ve) {
                    return he(this.red, "redSub works only with red numbers"), this.red.sub(this, Ve);
                  }, pe.prototype.redISub = function(Ve) {
                    return he(this.red, "redISub works only with red numbers"), this.red.isub(this, Ve);
                  }, pe.prototype.redShl = function(Ve) {
                    return he(this.red, "redShl works only with red numbers"), this.red.shl(this, Ve);
                  }, pe.prototype.redMul = function(Ve) {
                    return he(this.red, "redMul works only with red numbers"), this.red._verify2(this, Ve), this.red.mul(this, Ve);
                  }, pe.prototype.redIMul = function(Ve) {
                    return he(this.red, "redMul works only with red numbers"), this.red._verify2(this, Ve), this.red.imul(this, Ve);
                  }, pe.prototype.redSqr = function() {
                    return he(this.red, "redSqr works only with red numbers"), this.red._verify1(this), this.red.sqr(this);
                  }, pe.prototype.redISqr = function() {
                    return he(this.red, "redISqr works only with red numbers"), this.red._verify1(this), this.red.isqr(this);
                  }, pe.prototype.redSqrt = function() {
                    return he(this.red, "redSqrt works only with red numbers"), this.red._verify1(this), this.red.sqrt(this);
                  }, pe.prototype.redInvm = function() {
                    return he(this.red, "redInvm works only with red numbers"), this.red._verify1(this), this.red.invm(this);
                  }, pe.prototype.redNeg = function() {
                    return he(this.red, "redNeg works only with red numbers"), this.red._verify1(this), this.red.neg(this);
                  }, pe.prototype.redPow = function(Ve) {
                    return he(this.red && !Ve.red, "redPow(normalNum)"), this.red._verify1(this), this.red.pow(this, Ve);
                  };
                  var ke = { k256: null, p224: null, p192: null, p25519: null };
                  function Me(Ve, Ze) {
                    this.name = Ve, this.p = new pe(Ze, 16), this.n = this.p.bitLength(), this.k = new pe(1).iushln(this.n).isub(this.p), this.tmp = this._tmp();
                  }
                  function Re() {
                    Me.call(this, "k256", "ffffffff ffffffff ffffffff ffffffff ffffffff ffffffff fffffffe fffffc2f");
                  }
                  function De() {
                    Me.call(this, "p224", "ffffffff ffffffff ffffffff ffffffff 00000000 00000000 00000001");
                  }
                  function Pe() {
                    Me.call(this, "p192", "ffffffff ffffffff ffffffff fffffffe ffffffff ffffffff");
                  }
                  function $e() {
                    Me.call(this, "25519", "7fffffffffffffff ffffffffffffffff ffffffffffffffff ffffffffffffffed");
                  }
                  function qe(Ve) {
                    if (typeof Ve == "string") {
                      var Ze = pe._prime(Ve);
                      this.m = Ze.p, this.prime = Ze;
                    } else he(Ve.gtn(1), "modulus must be greater than 1"), this.m = Ve, this.prime = null;
                  }
                  function Xe(Ve) {
                    qe.call(this, Ve), this.shift = this.m.bitLength(), this.shift % 26 != 0 && (this.shift += 26 - this.shift % 26), this.r = new pe(1).iushln(this.shift), this.r2 = this.imod(this.r.sqr()), this.rinv = this.r._invmp(this.m), this.minv = this.rinv.mul(this.r).isubn(1).div(this.m), this.minv = this.minv.umod(this.r), this.minv = this.r.sub(this.minv);
                  }
                  Me.prototype._tmp = function() {
                    var Ve = new pe(null);
                    return Ve.words = new Array(Math.ceil(this.n / 13)), Ve;
                  }, Me.prototype.ireduce = function(Ve) {
                    var Ze, Ge = Ve;
                    do
                      this.split(Ge, this.tmp), Ze = (Ge = (Ge = this.imulK(Ge)).iadd(this.tmp)).bitLength();
                    while (Ze > this.n);
                    var ze = Ze < this.n ? -1 : Ge.ucmp(this.p);
                    return ze === 0 ? (Ge.words[0] = 0, Ge.length = 1) : ze > 0 ? Ge.isub(this.p) : Ge.strip !== void 0 ? Ge.strip() : Ge._strip(), Ge;
                  }, Me.prototype.split = function(Ve, Ze) {
                    Ve.iushrn(this.n, 0, Ze);
                  }, Me.prototype.imulK = function(Ve) {
                    return Ve.imul(this.k);
                  }, me(Re, Me), Re.prototype.split = function(Ve, Ze) {
                    for (var Ge = 4194303, ze = Math.min(Ve.length, 9), We = 0; We < ze; We++) Ze.words[We] = Ve.words[We];
                    if (Ze.length = ze, Ve.length <= 9) return Ve.words[0] = 0, void (Ve.length = 1);
                    var at = Ve.words[9];
                    for (Ze.words[Ze.length++] = at & Ge, We = 10; We < Ve.length; We++) {
                      var ct = 0 | Ve.words[We];
                      Ve.words[We - 10] = (ct & Ge) << 4 | at >>> 22, at = ct;
                    }
                    at >>>= 22, Ve.words[We - 10] = at, at === 0 && Ve.length > 10 ? Ve.length -= 10 : Ve.length -= 9;
                  }, Re.prototype.imulK = function(Ve) {
                    Ve.words[Ve.length] = 0, Ve.words[Ve.length + 1] = 0, Ve.length += 2;
                    for (var Ze = 0, Ge = 0; Ge < Ve.length; Ge++) {
                      var ze = 0 | Ve.words[Ge];
                      Ze += 977 * ze, Ve.words[Ge] = 67108863 & Ze, Ze = 64 * ze + (Ze / 67108864 | 0);
                    }
                    return Ve.words[Ve.length - 1] === 0 && (Ve.length--, Ve.words[Ve.length - 1] === 0 && Ve.length--), Ve;
                  }, me(De, Me), me(Pe, Me), me($e, Me), $e.prototype.imulK = function(Ve) {
                    for (var Ze = 0, Ge = 0; Ge < Ve.length; Ge++) {
                      var ze = 19 * (0 | Ve.words[Ge]) + Ze, We = 67108863 & ze;
                      ze >>>= 26, Ve.words[Ge] = We, Ze = ze;
                    }
                    return Ze !== 0 && (Ve.words[Ve.length++] = Ze), Ve;
                  }, pe._prime = function(Ve) {
                    if (ke[Ve]) return ke[Ve];
                    var Ze;
                    if (Ve === "k256") Ze = new Re();
                    else if (Ve === "p224") Ze = new De();
                    else if (Ve === "p192") Ze = new Pe();
                    else {
                      if (Ve !== "p25519") throw new Error("Unknown prime " + Ve);
                      Ze = new $e();
                    }
                    return ke[Ve] = Ze, Ze;
                  }, qe.prototype._verify1 = function(Ve) {
                    he(Ve.negative === 0, "red works only with positives"), he(Ve.red, "red works only with red numbers");
                  }, qe.prototype._verify2 = function(Ve, Ze) {
                    he((Ve.negative | Ze.negative) == 0, "red works only with positives"), he(Ve.red && Ve.red === Ze.red, "red works only with red numbers");
                  }, qe.prototype.imod = function(Ve) {
                    return this.prime ? this.prime.ireduce(Ve)._forceRed(this) : Ve.umod(this.m)._forceRed(this);
                  }, qe.prototype.neg = function(Ve) {
                    return Ve.isZero() ? Ve.clone() : this.m.sub(Ve)._forceRed(this);
                  }, qe.prototype.add = function(Ve, Ze) {
                    this._verify2(Ve, Ze);
                    var Ge = Ve.add(Ze);
                    return Ge.cmp(this.m) >= 0 && Ge.isub(this.m), Ge._forceRed(this);
                  }, qe.prototype.iadd = function(Ve, Ze) {
                    this._verify2(Ve, Ze);
                    var Ge = Ve.iadd(Ze);
                    return Ge.cmp(this.m) >= 0 && Ge.isub(this.m), Ge;
                  }, qe.prototype.sub = function(Ve, Ze) {
                    this._verify2(Ve, Ze);
                    var Ge = Ve.sub(Ze);
                    return Ge.cmpn(0) < 0 && Ge.iadd(this.m), Ge._forceRed(this);
                  }, qe.prototype.isub = function(Ve, Ze) {
                    this._verify2(Ve, Ze);
                    var Ge = Ve.isub(Ze);
                    return Ge.cmpn(0) < 0 && Ge.iadd(this.m), Ge;
                  }, qe.prototype.shl = function(Ve, Ze) {
                    return this._verify1(Ve), this.imod(Ve.ushln(Ze));
                  }, qe.prototype.imul = function(Ve, Ze) {
                    return this._verify2(Ve, Ze), this.imod(Ve.imul(Ze));
                  }, qe.prototype.mul = function(Ve, Ze) {
                    return this._verify2(Ve, Ze), this.imod(Ve.mul(Ze));
                  }, qe.prototype.isqr = function(Ve) {
                    return this.imul(Ve, Ve.clone());
                  }, qe.prototype.sqr = function(Ve) {
                    return this.mul(Ve, Ve);
                  }, qe.prototype.sqrt = function(Ve) {
                    if (Ve.isZero()) return Ve.clone();
                    var Ze = this.m.andln(3);
                    if (he(Ze % 2 == 1), Ze === 3) {
                      var Ge = this.m.add(new pe(1)).iushrn(2);
                      return this.pow(Ve, Ge);
                    }
                    for (var ze = this.m.subn(1), We = 0; !ze.isZero() && ze.andln(1) === 0; ) We++, ze.iushrn(1);
                    he(!ze.isZero());
                    var at = new pe(1).toRed(this), ct = at.redNeg(), et = this.m.subn(1).iushrn(1), dt = this.m.bitLength();
                    for (dt = new pe(2 * dt * dt).toRed(this); this.pow(dt, et).cmp(ct) !== 0; ) dt.redIAdd(ct);
                    for (var Je = this.pow(dt, ze), it = this.pow(Ve, ze.addn(1).iushrn(1)), st = this.pow(Ve, ze), gt = We; st.cmp(at) !== 0; ) {
                      for (var bt = st, yt = 0; bt.cmp(at) !== 0; yt++) bt = bt.redSqr();
                      he(yt < gt);
                      var St = this.pow(Je, new pe(1).iushln(gt - yt - 1));
                      it = it.redMul(St), Je = St.redSqr(), st = st.redMul(Je), gt = yt;
                    }
                    return it;
                  }, qe.prototype.invm = function(Ve) {
                    var Ze = Ve._invmp(this.m);
                    return Ze.negative !== 0 ? (Ze.negative = 0, this.imod(Ze).redNeg()) : this.imod(Ze);
                  }, qe.prototype.pow = function(Ve, Ze) {
                    if (Ze.isZero()) return new pe(1).toRed(this);
                    if (Ze.cmpn(1) === 0) return Ve.clone();
                    var Ge = new Array(16);
                    Ge[0] = new pe(1).toRed(this), Ge[1] = Ve;
                    for (var ze = 2; ze < Ge.length; ze++) Ge[ze] = this.mul(Ge[ze - 1], Ve);
                    var We = Ge[0], at = 0, ct = 0, et = Ze.bitLength() % 26;
                    for (et === 0 && (et = 26), ze = Ze.length - 1; ze >= 0; ze--) {
                      for (var dt = Ze.words[ze], Je = et - 1; Je >= 0; Je--) {
                        var it = dt >> Je & 1;
                        We !== Ge[0] && (We = this.sqr(We)), it !== 0 || at !== 0 ? (at <<= 1, at |= it, (++ct == 4 || ze === 0 && Je === 0) && (We = this.mul(We, Ge[at]), ct = 0, at = 0)) : ct = 0;
                      }
                      et = 26;
                    }
                    return We;
                  }, qe.prototype.convertTo = function(Ve) {
                    var Ze = Ve.umod(this.m);
                    return Ze === Ve ? Ze.clone() : Ze;
                  }, qe.prototype.convertFrom = function(Ve) {
                    var Ze = Ve.clone();
                    return Ze.red = null, Ze;
                  }, pe.mont = function(Ve) {
                    return new Xe(Ve);
                  }, me(Xe, qe), Xe.prototype.convertTo = function(Ve) {
                    return this.imod(Ve.ushln(this.shift));
                  }, Xe.prototype.convertFrom = function(Ve) {
                    var Ze = this.imod(Ve.mul(this.rinv));
                    return Ze.red = null, Ze;
                  }, Xe.prototype.imul = function(Ve, Ze) {
                    if (Ve.isZero() || Ze.isZero()) return Ve.words[0] = 0, Ve.length = 1, Ve;
                    var Ge = Ve.imul(Ze), ze = Ge.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m), We = Ge.isub(ze).iushrn(this.shift), at = We;
                    return We.cmp(this.m) >= 0 ? at = We.isub(this.m) : We.cmpn(0) < 0 && (at = We.iadd(this.m)), at._forceRed(this);
                  }, Xe.prototype.mul = function(Ve, Ze) {
                    if (Ve.isZero() || Ze.isZero()) return new pe(0)._forceRed(this);
                    var Ge = Ve.mul(Ze), ze = Ge.maskn(this.shift).mul(this.minv).imaskn(this.shift).mul(this.m), We = Ge.isub(ze).iushrn(this.shift), at = We;
                    return We.cmp(this.m) >= 0 ? at = We.isub(this.m) : We.cmpn(0) < 0 && (at = We.iadd(this.m)), at._forceRed(this);
                  }, Xe.prototype.invm = function(Ve) {
                    return this.imod(Ve._invmp(this.m).mul(this.r2))._forceRed(this);
                  };
                })(ce = ue.nmd(ce), this);
              }, 6204: function(ce) {
                ce.exports = function(le) {
                  var ue, de, fe, he = le.length, me = 0;
                  for (ue = 0; ue < he; ++ue) me += le[ue].length;
                  var pe = new Array(me), ye = 0;
                  for (ue = 0; ue < he; ++ue) {
                    var ge = le[ue], ve = ge.length;
                    for (de = 0; de < ve; ++de) {
                      var be = pe[ye++] = new Array(ve - 1), Ce = 0;
                      for (fe = 0; fe < ve; ++fe) fe !== de && (be[Ce++] = ge[fe]);
                      if (1 & de) {
                        var Se = be[1];
                        be[1] = be[0], be[0] = Se;
                      }
                    }
                  }
                  return pe;
                };
              }, 6867: function(ce, le, ue) {
                ce.exports = function(be, Ce, Se) {
                  switch (arguments.length) {
                    case 1:
                      return de = [], ge(we = be, we, ve, !0), de;
                    case 2:
                      return typeof Ce == "function" ? ge(be, be, Ce, !0) : function(Ee, xe) {
                        return de = [], ge(Ee, xe, ve, !1), de;
                      }(be, Ce);
                    case 3:
                      return ge(be, Ce, Se, !1);
                    default:
                      throw new Error("box-intersect: Invalid arguments");
                  }
                  var we;
                };
                var de, fe = ue(1888), he = ue(855), me = ue(7150);
                function pe(be, Ce) {
                  for (var Se = 0; Se < be; ++Se) if (!(Ce[Se] <= Ce[Se + be])) return !0;
                  return !1;
                }
                function ye(be, Ce, Se, we) {
                  for (var Ee = 0, xe = 0, Le = 0, Qe = be.length; Le < Qe; ++Le) {
                    var ke = be[Le];
                    if (!pe(Ce, ke)) {
                      for (var Me = 0; Me < 2 * Ce; ++Me) Se[Ee++] = ke[Me];
                      we[xe++] = Le;
                    }
                  }
                  return xe;
                }
                function ge(be, Ce, Se, we) {
                  var Ee = be.length, xe = Ce.length;
                  if (!(Ee <= 0 || xe <= 0)) {
                    var Le = be[0].length >>> 1;
                    if (!(Le <= 0)) {
                      var Qe, ke = fe.mallocDouble(2 * Le * Ee), Me = fe.mallocInt32(Ee);
                      if ((Ee = ye(be, Le, ke, Me)) > 0) {
                        if (Le === 1 && we) he.init(Ee), Qe = he.sweepComplete(Le, Se, 0, Ee, ke, Me, 0, Ee, ke, Me);
                        else {
                          var Re = fe.mallocDouble(2 * Le * xe), De = fe.mallocInt32(xe);
                          (xe = ye(Ce, Le, Re, De)) > 0 && (he.init(Ee + xe), Qe = Le === 1 ? he.sweepBipartite(Le, Se, 0, Ee, ke, Me, 0, xe, Re, De) : me(Le, Se, we, Ee, ke, Me, xe, Re, De), fe.free(Re), fe.free(De));
                        }
                        fe.free(ke), fe.free(Me);
                      }
                      return Qe;
                    }
                  }
                }
                function ve(be, Ce) {
                  de.push([be, Ce]);
                }
              }, 2455: function(ce, le) {
                function ue(de) {
                  return de ? function(fe, he, me, pe, ye, ge, ve, be, Ce, Se, we) {
                    return ye - pe > Ce - be ? function(Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe) {
                      for (var Xe = 2 * Ee, Ve = Qe, Ze = Xe * Qe; Ve < ke; ++Ve, Ze += Xe) {
                        var Ge = Me[xe + Ze], ze = Me[xe + Ze + Ee], We = Re[Ve];
                        e: for (var at = De, ct = Xe * De; at < Pe; ++at, ct += Xe) {
                          var et = $e[xe + ct], dt = $e[xe + ct + Ee], Je = qe[at];
                          if (!(dt < Ge || ze < et)) {
                            for (var it = xe + 1; it < Ee; ++it) {
                              var st = Me[it + Ze], gt = Me[it + Ee + Ze], bt = $e[it + ct], yt = $e[it + Ee + ct];
                              if (gt < bt || yt < st) continue e;
                            }
                            var St = Le(We, Je);
                            if (St !== void 0) return St;
                          }
                        }
                      }
                    }(fe, he, me, pe, ye, ge, ve, be, Ce, Se, we) : function(Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe) {
                      for (var Xe = 2 * Ee, Ve = De, Ze = Xe * De; Ve < Pe; ++Ve, Ze += Xe) {
                        var Ge = $e[xe + Ze], ze = $e[xe + Ze + Ee], We = qe[Ve];
                        e: for (var at = Qe, ct = Xe * Qe; at < ke; ++at, ct += Xe) {
                          var et = Me[xe + ct], dt = Me[xe + ct + Ee], Je = Re[at];
                          if (!(ze < et || dt < Ge)) {
                            for (var it = xe + 1; it < Ee; ++it) {
                              var st = Me[it + ct], gt = Me[it + Ee + ct], bt = $e[it + Ze], yt = $e[it + Ee + Ze];
                              if (gt < bt || yt < st) continue e;
                            }
                            var St = Le(Je, We);
                            if (St !== void 0) return St;
                          }
                        }
                      }
                    }(fe, he, me, pe, ye, ge, ve, be, Ce, Se, we);
                  } : function(fe, he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee) {
                    return ge - ye > Se - Ce ? pe ? function(xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe) {
                      for (var Ve = 2 * xe, Ze = ke, Ge = Ve * ke; Ze < Me; ++Ze, Ge += Ve) {
                        var ze = Re[Le + Ge], We = Re[Le + Ge + xe], at = De[Ze];
                        e: for (var ct = Pe, et = Ve * Pe; ct < $e; ++ct, et += Ve) {
                          var dt = qe[Le + et], Je = Xe[ct];
                          if (!(dt <= ze || We < dt)) {
                            for (var it = Le + 1; it < xe; ++it) {
                              var st = Re[it + Ge], gt = Re[it + xe + Ge], bt = qe[it + et], yt = qe[it + xe + et];
                              if (gt < bt || yt < st) continue e;
                            }
                            var St = Qe(Je, at);
                            if (St !== void 0) return St;
                          }
                        }
                      }
                    }(fe, he, me, ye, ge, ve, be, Ce, Se, we, Ee) : function(xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe) {
                      for (var Ve = 2 * xe, Ze = ke, Ge = Ve * ke; Ze < Me; ++Ze, Ge += Ve) {
                        var ze = Re[Le + Ge], We = Re[Le + Ge + xe], at = De[Ze];
                        e: for (var ct = Pe, et = Ve * Pe; ct < $e; ++ct, et += Ve) {
                          var dt = qe[Le + et], Je = Xe[ct];
                          if (!(dt < ze || We < dt)) {
                            for (var it = Le + 1; it < xe; ++it) {
                              var st = Re[it + Ge], gt = Re[it + xe + Ge], bt = qe[it + et], yt = qe[it + xe + et];
                              if (gt < bt || yt < st) continue e;
                            }
                            var St = Qe(at, Je);
                            if (St !== void 0) return St;
                          }
                        }
                      }
                    }(fe, he, me, ye, ge, ve, be, Ce, Se, we, Ee) : pe ? function(xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe) {
                      for (var Ve = 2 * xe, Ze = Pe, Ge = Ve * Pe; Ze < $e; ++Ze, Ge += Ve) {
                        var ze = qe[Le + Ge], We = Xe[Ze];
                        e: for (var at = ke, ct = Ve * ke; at < Me; ++at, ct += Ve) {
                          var et = Re[Le + ct], dt = Re[Le + ct + xe], Je = De[at];
                          if (!(ze <= et || dt < ze)) {
                            for (var it = Le + 1; it < xe; ++it) {
                              var st = Re[it + ct], gt = Re[it + xe + ct], bt = qe[it + Ge], yt = qe[it + xe + Ge];
                              if (gt < bt || yt < st) continue e;
                            }
                            var St = Qe(We, Je);
                            if (St !== void 0) return St;
                          }
                        }
                      }
                    }(fe, he, me, ye, ge, ve, be, Ce, Se, we, Ee) : function(xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe) {
                      for (var Ve = 2 * xe, Ze = Pe, Ge = Ve * Pe; Ze < $e; ++Ze, Ge += Ve) {
                        var ze = qe[Le + Ge], We = Xe[Ze];
                        e: for (var at = ke, ct = Ve * ke; at < Me; ++at, ct += Ve) {
                          var et = Re[Le + ct], dt = Re[Le + ct + xe], Je = De[at];
                          if (!(ze < et || dt < ze)) {
                            for (var it = Le + 1; it < xe; ++it) {
                              var st = Re[it + ct], gt = Re[it + xe + ct], bt = qe[it + Ge], yt = qe[it + xe + Ge];
                              if (gt < bt || yt < st) continue e;
                            }
                            var St = Qe(Je, We);
                            if (St !== void 0) return St;
                          }
                        }
                      }
                    }(fe, he, me, ye, ge, ve, be, Ce, Se, we, Ee);
                  };
                }
                le.partial = ue(!1), le.full = ue(!0);
              }, 7150: function(ce, le, ue) {
                ce.exports = function(Ve, Ze, Ge, ze, We, at, ct, et, dt) {
                  (function(rn, vn) {
                    var dn = 8 * fe.log2(vn + 1) * (rn + 1) | 0, Ht = fe.nextPow2(Me * dn);
                    De.length < Ht && (de.free(De), De = de.mallocInt32(Ht));
                    var jt = fe.nextPow2(Re * dn);
                    Pe.length < jt && (de.free(Pe), Pe = de.mallocDouble(jt));
                  })(Ve, ze + ct);
                  var Je, it = 0, st = 2 * Ve;
                  for ($e(it++, 0, 0, ze, 0, ct, Ge ? 16 : 0, -1 / 0, 1 / 0), Ge || $e(it++, 0, 0, ct, 0, ze, 1, -1 / 0, 1 / 0); it > 0; ) {
                    var gt = (it -= 1) * Me, bt = De[gt], yt = De[gt + 1], St = De[gt + 2], Lt = De[gt + 3], Rt = De[gt + 4], vt = De[gt + 5], $t = it * Re, Pt = Pe[$t], Xt = Pe[$t + 1], Nt = 1 & vt, Gt = !!(16 & vt), Ft = We, Tt = at, It = et, Ct = dt;
                    if (Nt && (Ft = et, Tt = dt, It = We, Ct = at), !(2 & vt && yt >= (St = xe(Ve, bt, yt, St, Ft, Tt, Xt)) || 4 & vt && (yt = Le(Ve, bt, yt, St, Ft, Tt, Pt)) >= St)) {
                      var Kt = St - yt, Vt = Rt - Lt;
                      if (Gt) {
                        if (Ve * Kt * (Kt + Vt) < Se) {
                          if ((Je = ye.scanComplete(Ve, bt, Ze, yt, St, Ft, Tt, Lt, Rt, It, Ct)) !== void 0) return Je;
                          continue;
                        }
                      } else {
                        if (Ve * Math.min(Kt, Vt) < be) {
                          if ((Je = me(Ve, bt, Ze, Nt, yt, St, Ft, Tt, Lt, Rt, It, Ct)) !== void 0) return Je;
                          continue;
                        }
                        if (Ve * Kt * Vt < Ce) {
                          if ((Je = ye.scanBipartite(Ve, bt, Ze, Nt, yt, St, Ft, Tt, Lt, Rt, It, Ct)) !== void 0) return Je;
                          continue;
                        }
                      }
                      var Yt = we(Ve, bt, yt, St, Ft, Tt, Pt, Xt);
                      if (yt < Yt) if (Ve * (Yt - yt) < be) {
                        if ((Je = pe(Ve, bt + 1, Ze, yt, Yt, Ft, Tt, Lt, Rt, It, Ct)) !== void 0) return Je;
                      } else if (bt === Ve - 2) {
                        if ((Je = Nt ? ye.sweepBipartite(Ve, Ze, Lt, Rt, It, Ct, yt, Yt, Ft, Tt) : ye.sweepBipartite(Ve, Ze, yt, Yt, Ft, Tt, Lt, Rt, It, Ct)) !== void 0) return Je;
                      } else $e(it++, bt + 1, yt, Yt, Lt, Rt, Nt, -1 / 0, 1 / 0), $e(it++, bt + 1, Lt, Rt, yt, Yt, 1 ^ Nt, -1 / 0, 1 / 0);
                      if (Yt < St) {
                        var sn = ge(Ve, bt, Lt, Rt, It, Ct), hn = It[st * sn + bt], Qt = Ee(Ve, bt, sn, Rt, It, Ct, hn);
                        if (Qt < Rt && $e(it++, bt, Yt, St, Qt, Rt, (4 | Nt) + (Gt ? 16 : 0), hn, Xt), Lt < sn && $e(it++, bt, Yt, St, Lt, sn, (2 | Nt) + (Gt ? 16 : 0), Pt, hn), sn + 1 === Qt) {
                          if ((Je = Gt ? Xe(Ve, bt, Ze, Yt, St, Ft, Tt, sn, It, Ct[sn]) : qe(Ve, bt, Ze, Nt, Yt, St, Ft, Tt, sn, It, Ct[sn])) !== void 0) return Je;
                        } else if (sn < Qt) {
                          var Ut;
                          if (Gt) {
                            if (Yt < (Ut = Qe(Ve, bt, Yt, St, Ft, Tt, hn))) {
                              var Wt = Ee(Ve, bt, Yt, Ut, Ft, Tt, hn);
                              if (bt === Ve - 2) {
                                if (Yt < Wt && (Je = ye.sweepComplete(Ve, Ze, Yt, Wt, Ft, Tt, sn, Qt, It, Ct)) !== void 0 || Wt < Ut && (Je = ye.sweepBipartite(Ve, Ze, Wt, Ut, Ft, Tt, sn, Qt, It, Ct)) !== void 0) return Je;
                              } else Yt < Wt && $e(it++, bt + 1, Yt, Wt, sn, Qt, 16, -1 / 0, 1 / 0), Wt < Ut && ($e(it++, bt + 1, Wt, Ut, sn, Qt, 0, -1 / 0, 1 / 0), $e(it++, bt + 1, sn, Qt, Wt, Ut, 1, -1 / 0, 1 / 0));
                            }
                          } else Yt < (Ut = Nt ? ke(Ve, bt, Yt, St, Ft, Tt, hn) : Qe(Ve, bt, Yt, St, Ft, Tt, hn)) && (bt === Ve - 2 ? Je = Nt ? ye.sweepBipartite(Ve, Ze, sn, Qt, It, Ct, Yt, Ut, Ft, Tt) : ye.sweepBipartite(Ve, Ze, Yt, Ut, Ft, Tt, sn, Qt, It, Ct) : ($e(it++, bt + 1, Yt, Ut, sn, Qt, Nt, -1 / 0, 1 / 0), $e(it++, bt + 1, sn, Qt, Yt, Ut, 1 ^ Nt, -1 / 0, 1 / 0)));
                        }
                      }
                    }
                  }
                };
                var de = ue(1888), fe = ue(8828), he = ue(2455), me = he.partial, pe = he.full, ye = ue(855), ge = ue(3545), ve = ue(8105), be = 128, Ce = 1 << 22, Se = 1 << 22, we = ve("!(lo>=p0)&&!(p1>=hi)"), Ee = ve("lo===p0"), xe = ve("lo<p0"), Le = ve("hi<=p0"), Qe = ve("lo<=p0&&p0<=hi"), ke = ve("lo<p0&&p0<=hi"), Me = 6, Re = 2, De = de.mallocInt32(1024), Pe = de.mallocDouble(1024);
                function $e(Ve, Ze, Ge, ze, We, at, ct, et, dt) {
                  var Je = Me * Ve;
                  De[Je] = Ze, De[Je + 1] = Ge, De[Je + 2] = ze, De[Je + 3] = We, De[Je + 4] = at, De[Je + 5] = ct;
                  var it = Re * Ve;
                  Pe[it] = et, Pe[it + 1] = dt;
                }
                function qe(Ve, Ze, Ge, ze, We, at, ct, et, dt, Je, it) {
                  var st = 2 * Ve, gt = dt * st, bt = Je[gt + Ze];
                  e: for (var yt = We, St = We * st; yt < at; ++yt, St += st) {
                    var Lt = ct[St + Ze], Rt = ct[St + Ze + Ve];
                    if (!(bt < Lt || Rt < bt || ze && bt === Lt)) {
                      for (var vt, $t = et[yt], Pt = Ze + 1; Pt < Ve; ++Pt) {
                        Lt = ct[St + Pt], Rt = ct[St + Pt + Ve];
                        var Xt = Je[gt + Pt], Nt = Je[gt + Pt + Ve];
                        if (Rt < Xt || Nt < Lt) continue e;
                      }
                      if ((vt = ze ? Ge(it, $t) : Ge($t, it)) !== void 0) return vt;
                    }
                  }
                }
                function Xe(Ve, Ze, Ge, ze, We, at, ct, et, dt, Je) {
                  var it = 2 * Ve, st = et * it, gt = dt[st + Ze];
                  e: for (var bt = ze, yt = ze * it; bt < We; ++bt, yt += it) {
                    var St = ct[bt];
                    if (St !== Je) {
                      var Lt = at[yt + Ze], Rt = at[yt + Ze + Ve];
                      if (!(gt < Lt || Rt < gt)) {
                        for (var vt = Ze + 1; vt < Ve; ++vt) {
                          Lt = at[yt + vt], Rt = at[yt + vt + Ve];
                          var $t = dt[st + vt], Pt = dt[st + vt + Ve];
                          if (Rt < $t || Pt < Lt) continue e;
                        }
                        var Xt = Ge(St, Je);
                        if (Xt !== void 0) return Xt;
                      }
                    }
                  }
                }
              }, 3545: function(ce, le, ue) {
                ce.exports = function(me, pe, ye, ge, ve, be) {
                  if (ge <= ye + 1) return ye;
                  for (var Ce = ye, Se = ge, we = ge + ye >>> 1, Ee = 2 * me, xe = we, Le = ve[Ee * we + pe]; Ce < Se; ) {
                    if (Se - Ce < fe) {
                      he(me, pe, Ce, Se, ve, be), Le = ve[Ee * we + pe];
                      break;
                    }
                    var Qe = Se - Ce, ke = Math.random() * Qe + Ce | 0, Me = ve[Ee * ke + pe], Re = Math.random() * Qe + Ce | 0, De = ve[Ee * Re + pe], Pe = Math.random() * Qe + Ce | 0, $e = ve[Ee * Pe + pe];
                    Me <= De ? $e >= De ? (xe = Re, Le = De) : Me >= $e ? (xe = ke, Le = Me) : (xe = Pe, Le = $e) : De >= $e ? (xe = Re, Le = De) : $e >= Me ? (xe = ke, Le = Me) : (xe = Pe, Le = $e);
                    for (var qe = Ee * (Se - 1), Xe = Ee * xe, Ve = 0; Ve < Ee; ++Ve, ++qe, ++Xe) {
                      var Ze = ve[qe];
                      ve[qe] = ve[Xe], ve[Xe] = Ze;
                    }
                    var Ge = be[Se - 1];
                    for (be[Se - 1] = be[xe], be[xe] = Ge, qe = Ee * (Se - 1), Xe = Ee * (xe = de(me, pe, Ce, Se - 1, ve, be, Le)), Ve = 0; Ve < Ee; ++Ve, ++qe, ++Xe) Ze = ve[qe], ve[qe] = ve[Xe], ve[Xe] = Ze;
                    if (Ge = be[Se - 1], be[Se - 1] = be[xe], be[xe] = Ge, we < xe) {
                      for (Se = xe - 1; Ce < Se && ve[Ee * (Se - 1) + pe] === Le; ) Se -= 1;
                      Se += 1;
                    } else {
                      if (!(xe < we)) break;
                      for (Ce = xe + 1; Ce < Se && ve[Ee * Ce + pe] === Le; ) Ce += 1;
                    }
                  }
                  return de(me, pe, ye, we, ve, be, ve[Ee * we + pe]);
                };
                var de = ue(8105)("lo<p0"), fe = 8;
                function he(me, pe, ye, ge, ve, be) {
                  for (var Ce = 2 * me, Se = Ce * (ye + 1) + pe, we = ye + 1; we < ge; ++we, Se += Ce) for (var Ee = ve[Se], xe = we, Le = Ce * (we - 1); xe > ye && ve[Le + pe] > Ee; --xe, Le -= Ce) {
                    for (var Qe = Le, ke = Le + Ce, Me = 0; Me < Ce; ++Me, ++Qe, ++ke) {
                      var Re = ve[Qe];
                      ve[Qe] = ve[ke], ve[ke] = Re;
                    }
                    var De = be[xe];
                    be[xe] = be[xe - 1], be[xe - 1] = De;
                  }
                }
              }, 8105: function(ce) {
                ce.exports = function(ue) {
                  return le[ue];
                };
                var le = { "lo===p0": function(ue, de, fe, he, me, pe, ye) {
                  for (var ge = 2 * ue, ve = ge * fe, be = ve, Ce = fe, Se = de, we = fe; he > we; ++we, ve += ge) if (me[ve + Se] === ye) if (Ce === we) Ce += 1, be += ge;
                  else {
                    for (var Ee = 0; ge > Ee; ++Ee) {
                      var xe = me[ve + Ee];
                      me[ve + Ee] = me[be], me[be++] = xe;
                    }
                    var Le = pe[we];
                    pe[we] = pe[Ce], pe[Ce++] = Le;
                  }
                  return Ce;
                }, "lo<p0": function(ue, de, fe, he, me, pe, ye) {
                  for (var ge = 2 * ue, ve = ge * fe, be = ve, Ce = fe, Se = de, we = fe; he > we; ++we, ve += ge) if (me[ve + Se] < ye) if (Ce === we) Ce += 1, be += ge;
                  else {
                    for (var Ee = 0; ge > Ee; ++Ee) {
                      var xe = me[ve + Ee];
                      me[ve + Ee] = me[be], me[be++] = xe;
                    }
                    var Le = pe[we];
                    pe[we] = pe[Ce], pe[Ce++] = Le;
                  }
                  return Ce;
                }, "lo<=p0": function(ue, de, fe, he, me, pe, ye) {
                  for (var ge = 2 * ue, ve = ge * fe, be = ve, Ce = fe, Se = ue + de, we = fe; he > we; ++we, ve += ge) if (me[ve + Se] <= ye) if (Ce === we) Ce += 1, be += ge;
                  else {
                    for (var Ee = 0; ge > Ee; ++Ee) {
                      var xe = me[ve + Ee];
                      me[ve + Ee] = me[be], me[be++] = xe;
                    }
                    var Le = pe[we];
                    pe[we] = pe[Ce], pe[Ce++] = Le;
                  }
                  return Ce;
                }, "hi<=p0": function(ue, de, fe, he, me, pe, ye) {
                  for (var ge = 2 * ue, ve = ge * fe, be = ve, Ce = fe, Se = ue + de, we = fe; he > we; ++we, ve += ge) if (me[ve + Se] <= ye) if (Ce === we) Ce += 1, be += ge;
                  else {
                    for (var Ee = 0; ge > Ee; ++Ee) {
                      var xe = me[ve + Ee];
                      me[ve + Ee] = me[be], me[be++] = xe;
                    }
                    var Le = pe[we];
                    pe[we] = pe[Ce], pe[Ce++] = Le;
                  }
                  return Ce;
                }, "lo<p0&&p0<=hi": function(ue, de, fe, he, me, pe, ye) {
                  for (var ge = 2 * ue, ve = ge * fe, be = ve, Ce = fe, Se = de, we = ue + de, Ee = fe; he > Ee; ++Ee, ve += ge) {
                    var xe = me[ve + Se], Le = me[ve + we];
                    if (xe < ye && ye <= Le) if (Ce === Ee) Ce += 1, be += ge;
                    else {
                      for (var Qe = 0; ge > Qe; ++Qe) {
                        var ke = me[ve + Qe];
                        me[ve + Qe] = me[be], me[be++] = ke;
                      }
                      var Me = pe[Ee];
                      pe[Ee] = pe[Ce], pe[Ce++] = Me;
                    }
                  }
                  return Ce;
                }, "lo<=p0&&p0<=hi": function(ue, de, fe, he, me, pe, ye) {
                  for (var ge = 2 * ue, ve = ge * fe, be = ve, Ce = fe, Se = de, we = ue + de, Ee = fe; he > Ee; ++Ee, ve += ge) {
                    var xe = me[ve + Se], Le = me[ve + we];
                    if (xe <= ye && ye <= Le) if (Ce === Ee) Ce += 1, be += ge;
                    else {
                      for (var Qe = 0; ge > Qe; ++Qe) {
                        var ke = me[ve + Qe];
                        me[ve + Qe] = me[be], me[be++] = ke;
                      }
                      var Me = pe[Ee];
                      pe[Ee] = pe[Ce], pe[Ce++] = Me;
                    }
                  }
                  return Ce;
                }, "!(lo>=p0)&&!(p1>=hi)": function(ue, de, fe, he, me, pe, ye, ge) {
                  for (var ve = 2 * ue, be = ve * fe, Ce = be, Se = fe, we = de, Ee = ue + de, xe = fe; he > xe; ++xe, be += ve) {
                    var Le = me[be + we], Qe = me[be + Ee];
                    if (!(Le >= ye || ge >= Qe)) if (Se === xe) Se += 1, Ce += ve;
                    else {
                      for (var ke = 0; ve > ke; ++ke) {
                        var Me = me[be + ke];
                        me[be + ke] = me[Ce], me[Ce++] = Me;
                      }
                      var Re = pe[xe];
                      pe[xe] = pe[Se], pe[Se++] = Re;
                    }
                  }
                  return Se;
                } };
              }, 1811: function(ce) {
                ce.exports = function(ve, be) {
                  be <= 4 * le ? ue(0, be - 1, ve) : ge(0, be - 1, ve);
                };
                var le = 32;
                function ue(ve, be, Ce) {
                  for (var Se = 2 * (ve + 1), we = ve + 1; we <= be; ++we) {
                    for (var Ee = Ce[Se++], xe = Ce[Se++], Le = we, Qe = Se - 2; Le-- > ve; ) {
                      var ke = Ce[Qe - 2], Me = Ce[Qe - 1];
                      if (ke < Ee || ke === Ee && Me < xe) break;
                      Ce[Qe] = ke, Ce[Qe + 1] = Me, Qe -= 2;
                    }
                    Ce[Qe] = Ee, Ce[Qe + 1] = xe;
                  }
                }
                function de(ve, be, Ce) {
                  be *= 2;
                  var Se = Ce[ve *= 2], we = Ce[ve + 1];
                  Ce[ve] = Ce[be], Ce[ve + 1] = Ce[be + 1], Ce[be] = Se, Ce[be + 1] = we;
                }
                function fe(ve, be, Ce) {
                  be *= 2, Ce[ve *= 2] = Ce[be], Ce[ve + 1] = Ce[be + 1];
                }
                function he(ve, be, Ce, Se) {
                  be *= 2, Ce *= 2;
                  var we = Se[ve *= 2], Ee = Se[ve + 1];
                  Se[ve] = Se[be], Se[ve + 1] = Se[be + 1], Se[be] = Se[Ce], Se[be + 1] = Se[Ce + 1], Se[Ce] = we, Se[Ce + 1] = Ee;
                }
                function me(ve, be, Ce, Se, we) {
                  be *= 2, we[ve *= 2] = we[be], we[be] = Ce, we[ve + 1] = we[be + 1], we[be + 1] = Se;
                }
                function pe(ve, be, Ce) {
                  be *= 2;
                  var Se = Ce[ve *= 2], we = Ce[be];
                  return !(Se < we) && (Se !== we || Ce[ve + 1] > Ce[be + 1]);
                }
                function ye(ve, be, Ce, Se) {
                  var we = Se[ve *= 2];
                  return we < be || we === be && Se[ve + 1] < Ce;
                }
                function ge(ve, be, Ce) {
                  var Se = (be - ve + 1) / 6 | 0, we = ve + Se, Ee = be - Se, xe = ve + be >> 1, Le = xe - Se, Qe = xe + Se, ke = we, Me = Le, Re = xe, De = Qe, Pe = Ee, $e = ve + 1, qe = be - 1, Xe = 0;
                  pe(ke, Me, Ce) && (Xe = ke, ke = Me, Me = Xe), pe(De, Pe, Ce) && (Xe = De, De = Pe, Pe = Xe), pe(ke, Re, Ce) && (Xe = ke, ke = Re, Re = Xe), pe(Me, Re, Ce) && (Xe = Me, Me = Re, Re = Xe), pe(ke, De, Ce) && (Xe = ke, ke = De, De = Xe), pe(Re, De, Ce) && (Xe = Re, Re = De, De = Xe), pe(Me, Pe, Ce) && (Xe = Me, Me = Pe, Pe = Xe), pe(Me, Re, Ce) && (Xe = Me, Me = Re, Re = Xe), pe(De, Pe, Ce) && (Xe = De, De = Pe, Pe = Xe);
                  for (var Ve = Ce[2 * Me], Ze = Ce[2 * Me + 1], Ge = Ce[2 * De], ze = Ce[2 * De + 1], We = 2 * ke, at = 2 * Re, ct = 2 * Pe, et = 2 * we, dt = 2 * xe, Je = 2 * Ee, it = 0; it < 2; ++it) {
                    var st = Ce[We + it], gt = Ce[at + it], bt = Ce[ct + it];
                    Ce[et + it] = st, Ce[dt + it] = gt, Ce[Je + it] = bt;
                  }
                  fe(Le, ve, Ce), fe(Qe, be, Ce);
                  for (var yt = $e; yt <= qe; ++yt) if (ye(yt, Ve, Ze, Ce)) yt !== $e && de(yt, $e, Ce), ++$e;
                  else if (!ye(yt, Ge, ze, Ce)) for (; ; ) {
                    if (ye(qe, Ge, ze, Ce)) {
                      ye(qe, Ve, Ze, Ce) ? (he(yt, $e, qe, Ce), ++$e, --qe) : (de(yt, qe, Ce), --qe);
                      break;
                    }
                    if (--qe < yt) break;
                  }
                  me(ve, $e - 1, Ve, Ze, Ce), me(be, qe + 1, Ge, ze, Ce), $e - 2 - ve <= le ? ue(ve, $e - 2, Ce) : ge(ve, $e - 2, Ce), be - (qe + 2) <= le ? ue(qe + 2, be, Ce) : ge(qe + 2, be, Ce), qe - $e <= le ? ue($e, qe, Ce) : ge($e, qe, Ce);
                }
              }, 855: function(ce, le, ue) {
                ce.exports = { init: function(Le) {
                  var Qe = fe.nextPow2(Le);
                  ye.length < Qe && (de.free(ye), ye = de.mallocInt32(Qe)), ge.length < Qe && (de.free(ge), ge = de.mallocInt32(Qe)), ve.length < Qe && (de.free(ve), ve = de.mallocInt32(Qe)), be.length < Qe && (de.free(be), be = de.mallocInt32(Qe)), Ce.length < Qe && (de.free(Ce), Ce = de.mallocInt32(Qe)), Se.length < Qe && (de.free(Se), Se = de.mallocInt32(Qe));
                  var ke = 8 * Qe;
                  we.length < ke && (de.free(we), we = de.mallocDouble(ke));
                }, sweepBipartite: function(Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe) {
                  for (var Ve = 0, Ze = 2 * Le, Ge = Le - 1, ze = Ze - 1, We = ke; We < Me; ++We) {
                    var at = De[We], ct = Ze * We;
                    we[Ve++] = Re[ct + Ge], we[Ve++] = -(at + 1), we[Ve++] = Re[ct + ze], we[Ve++] = at;
                  }
                  for (We = Pe; We < $e; ++We) {
                    at = Xe[We] + me;
                    var et = Ze * We;
                    we[Ve++] = qe[et + Ge], we[Ve++] = -at, we[Ve++] = qe[et + ze], we[Ve++] = at;
                  }
                  var dt = Ve >>> 1;
                  he(we, dt);
                  var Je = 0, it = 0;
                  for (We = 0; We < dt; ++We) {
                    var st = 0 | we[2 * We + 1];
                    if (st >= me) Ee(ve, be, it--, st = st - me | 0);
                    else if (st >= 0) Ee(ye, ge, Je--, st);
                    else if (st <= -me) {
                      st = -st - me | 0;
                      for (var gt = 0; gt < Je; ++gt) if ((bt = Qe(ye[gt], st)) !== void 0) return bt;
                      xe(ve, be, it++, st);
                    } else {
                      for (st = -st - 1 | 0, gt = 0; gt < it; ++gt) {
                        var bt;
                        if ((bt = Qe(st, ve[gt])) !== void 0) return bt;
                      }
                      xe(ye, ge, Je++, st);
                    }
                  }
                }, sweepComplete: function(Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe) {
                  for (var Ve = 0, Ze = 2 * Le, Ge = Le - 1, ze = Ze - 1, We = ke; We < Me; ++We) {
                    var at = De[We] + 1 << 1, ct = Ze * We;
                    we[Ve++] = Re[ct + Ge], we[Ve++] = -at, we[Ve++] = Re[ct + ze], we[Ve++] = at;
                  }
                  for (We = Pe; We < $e; ++We) {
                    at = Xe[We] + 1 << 1;
                    var et = Ze * We;
                    we[Ve++] = qe[et + Ge], we[Ve++] = 1 | -at, we[Ve++] = qe[et + ze], we[Ve++] = 1 | at;
                  }
                  var dt = Ve >>> 1;
                  he(we, dt);
                  var Je = 0, it = 0, st = 0;
                  for (We = 0; We < dt; ++We) {
                    var gt = 0 | we[2 * We + 1], bt = 1 & gt;
                    if (We < dt - 1 && gt >> 1 == we[2 * We + 3] >> 1 && (bt = 2, We += 1), gt < 0) {
                      for (var yt = -(gt >> 1) - 1, St = 0; St < st; ++St) if ((Lt = Qe(Ce[St], yt)) !== void 0) return Lt;
                      if (bt !== 0) {
                        for (St = 0; St < Je; ++St) if ((Lt = Qe(ye[St], yt)) !== void 0) return Lt;
                      }
                      if (bt !== 1) for (St = 0; St < it; ++St) {
                        var Lt;
                        if ((Lt = Qe(ve[St], yt)) !== void 0) return Lt;
                      }
                      bt === 0 ? xe(ye, ge, Je++, yt) : bt === 1 ? xe(ve, be, it++, yt) : bt === 2 && xe(Ce, Se, st++, yt);
                    } else yt = (gt >> 1) - 1, bt === 0 ? Ee(ye, ge, Je--, yt) : bt === 1 ? Ee(ve, be, it--, yt) : bt === 2 && Ee(Ce, Se, st--, yt);
                  }
                }, scanBipartite: function(Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze) {
                  var Ge = 0, ze = 2 * Le, We = Qe, at = Qe + Le, ct = 1, et = 1;
                  Me ? et = me : ct = me;
                  for (var dt = Re; dt < De; ++dt) {
                    var Je = dt + ct, it = ze * dt;
                    we[Ge++] = Pe[it + We], we[Ge++] = -Je, we[Ge++] = Pe[it + at], we[Ge++] = Je;
                  }
                  for (dt = qe; dt < Xe; ++dt) {
                    Je = dt + et;
                    var st = ze * dt;
                    we[Ge++] = Ve[st + We], we[Ge++] = -Je;
                  }
                  var gt = Ge >>> 1;
                  he(we, gt);
                  var bt = 0;
                  for (dt = 0; dt < gt; ++dt) {
                    var yt = 0 | we[2 * dt + 1];
                    if (yt < 0) {
                      var St = !1;
                      if ((Je = -yt) >= me ? (St = !Me, Je -= me) : (St = !!Me, Je -= 1), St) xe(ye, ge, bt++, Je);
                      else {
                        var Lt = Ze[Je], Rt = ze * Je, vt = Ve[Rt + Qe + 1], $t = Ve[Rt + Qe + 1 + Le];
                        e: for (var Pt = 0; Pt < bt; ++Pt) {
                          var Xt = ye[Pt], Nt = ze * Xt;
                          if (!($t < Pe[Nt + Qe + 1] || Pe[Nt + Qe + 1 + Le] < vt)) {
                            for (var Gt = Qe + 2; Gt < Le; ++Gt) if (Ve[Rt + Gt + Le] < Pe[Nt + Gt] || Pe[Nt + Gt + Le] < Ve[Rt + Gt]) continue e;
                            var Ft, Tt = $e[Xt];
                            if ((Ft = Me ? ke(Lt, Tt) : ke(Tt, Lt)) !== void 0) return Ft;
                          }
                        }
                      }
                    } else Ee(ye, ge, bt--, yt - ct);
                  }
                }, scanComplete: function(Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe, Ve) {
                  for (var Ze = 0, Ge = 2 * Le, ze = Qe, We = Qe + Le, at = Me; at < Re; ++at) {
                    var ct = at + me, et = Ge * at;
                    we[Ze++] = De[et + ze], we[Ze++] = -ct, we[Ze++] = De[et + We], we[Ze++] = ct;
                  }
                  for (at = $e; at < qe; ++at) {
                    ct = at + 1;
                    var dt = Ge * at;
                    we[Ze++] = Xe[dt + ze], we[Ze++] = -ct;
                  }
                  var Je = Ze >>> 1;
                  he(we, Je);
                  var it = 0;
                  for (at = 0; at < Je; ++at) {
                    var st = 0 | we[2 * at + 1];
                    if (st < 0) if ((ct = -st) >= me) ye[it++] = ct - me;
                    else {
                      var gt = Ve[ct -= 1], bt = Ge * ct, yt = Xe[bt + Qe + 1], St = Xe[bt + Qe + 1 + Le];
                      e: for (var Lt = 0; Lt < it; ++Lt) {
                        var Rt = ye[Lt], vt = Pe[Rt];
                        if (vt === gt) break;
                        var $t = Ge * Rt;
                        if (!(St < De[$t + Qe + 1] || De[$t + Qe + 1 + Le] < yt)) {
                          for (var Pt = Qe + 2; Pt < Le; ++Pt) if (Xe[bt + Pt + Le] < De[$t + Pt] || De[$t + Pt + Le] < Xe[bt + Pt]) continue e;
                          var Xt = ke(vt, gt);
                          if (Xt !== void 0) return Xt;
                        }
                      }
                    }
                    else {
                      for (ct = st - me, Lt = it - 1; Lt >= 0; --Lt) if (ye[Lt] === ct) {
                        for (Pt = Lt + 1; Pt < it; ++Pt) ye[Pt - 1] = ye[Pt];
                        break;
                      }
                      --it;
                    }
                  }
                } };
                var de = ue(1888), fe = ue(8828), he = ue(1811), me = 1 << 28, pe = 1024, ye = de.mallocInt32(pe), ge = de.mallocInt32(pe), ve = de.mallocInt32(pe), be = de.mallocInt32(pe), Ce = de.mallocInt32(pe), Se = de.mallocInt32(pe), we = de.mallocDouble(8192);
                function Ee(Le, Qe, ke, Me) {
                  var Re = Qe[Me], De = Le[ke - 1];
                  Le[Re] = De, Qe[De] = Re;
                }
                function xe(Le, Qe, ke, Me) {
                  Le[ke] = Me, Qe[Me] = ke;
                }
              }, 2538: function(ce, le, ue) {
                var de = ue(8902), fe = ue(5542), he = ue(2272), me = ue(5023);
                function pe(ve) {
                  return [Math.min(ve[0], ve[1]), Math.max(ve[0], ve[1])];
                }
                function ye(ve, be) {
                  return ve[0] - be[0] || ve[1] - be[1];
                }
                function ge(ve, be, Ce) {
                  return be in ve ? ve[be] : Ce;
                }
                ce.exports = function(ve, be, Ce) {
                  Array.isArray(be) ? (Ce = Ce || {}, be = be || []) : (Ce = be || {}, be = []);
                  var Se = !!ge(Ce, "delaunay", !0), we = !!ge(Ce, "interior", !0), Ee = !!ge(Ce, "exterior", !0), xe = !!ge(Ce, "infinity", !1);
                  if (!we && !Ee || ve.length === 0) return [];
                  var Le = de(ve, be);
                  if (Se || we !== Ee || xe) {
                    for (var Qe = fe(ve.length, function(Re) {
                      return Re.map(pe).sort(ye);
                    }(be)), ke = 0; ke < Le.length; ++ke) {
                      var Me = Le[ke];
                      Qe.addTriangle(Me[0], Me[1], Me[2]);
                    }
                    return Se && he(ve, Qe), Ee ? we ? xe ? me(Qe, 0, xe) : Qe.cells() : me(Qe, 1, xe) : me(Qe, -1);
                  }
                  return Le;
                };
              }, 2272: function(ce, le, ue) {
                var de = ue(2646)[4];
                function fe(he, me, pe, ye, ge, ve) {
                  var be = me.opposite(ye, ge);
                  if (!(be < 0)) {
                    if (ge < ye) {
                      var Ce = ye;
                      ye = ge, ge = Ce, Ce = ve, ve = be, be = Ce;
                    }
                    me.isConstraint(ye, ge) || de(he[ye], he[ge], he[ve], he[be]) < 0 && pe.push(ye, ge);
                  }
                }
                ue(2478), ce.exports = function(he, me) {
                  for (var pe = [], ye = he.length, ge = me.stars, ve = 0; ve < ye; ++ve) for (var be = ge[ve], Ce = 1; Ce < be.length; Ce += 2) if (!((xe = be[Ce]) < ve || me.isConstraint(ve, xe))) {
                    for (var Se = be[Ce - 1], we = -1, Ee = 1; Ee < be.length; Ee += 2) if (be[Ee - 1] === xe) {
                      we = be[Ee];
                      break;
                    }
                    we < 0 || de(he[ve], he[xe], he[Se], he[we]) < 0 && pe.push(ve, xe);
                  }
                  for (; pe.length > 0; ) {
                    for (var xe = pe.pop(), Le = (Se = -1, we = -1, be = ge[ve = pe.pop()], 1); Le < be.length; Le += 2) {
                      var Qe = be[Le - 1], ke = be[Le];
                      Qe === xe ? we = ke : ke === xe && (Se = Qe);
                    }
                    Se < 0 || we < 0 || de(he[ve], he[xe], he[Se], he[we]) >= 0 || (me.flip(ve, xe), fe(he, me, pe, Se, ve, we), fe(he, me, pe, ve, we, Se), fe(he, me, pe, we, xe, Se), fe(he, me, pe, xe, Se, we));
                  }
                };
              }, 5023: function(ce, le, ue) {
                var de, fe = ue(2478);
                function he(pe, ye, ge, ve, be, Ce, Se) {
                  this.cells = pe, this.neighbor = ye, this.flags = ve, this.constraint = ge, this.active = be, this.next = Ce, this.boundary = Se;
                }
                function me(pe, ye) {
                  return pe[0] - ye[0] || pe[1] - ye[1] || pe[2] - ye[2];
                }
                ce.exports = function(pe, ye, ge) {
                  var ve = function(Pe, $e) {
                    for (var qe = Pe.cells(), Xe = qe.length, Ve = 0; Ve < Xe; ++Ve) {
                      var Ze = (st = qe[Ve])[0], Ge = st[1], ze = st[2];
                      Ge < ze ? Ge < Ze && (st[0] = Ge, st[1] = ze, st[2] = Ze) : ze < Ze && (st[0] = ze, st[1] = Ze, st[2] = Ge);
                    }
                    qe.sort(me);
                    var We = new Array(Xe);
                    for (Ve = 0; Ve < We.length; ++Ve) We[Ve] = 0;
                    var at = [], ct = [], et = new Array(3 * Xe), dt = new Array(3 * Xe), Je = null;
                    $e && (Je = []);
                    var it = new he(qe, et, dt, We, at, ct, Je);
                    for (Ve = 0; Ve < Xe; ++Ve) for (var st = qe[Ve], gt = 0; gt < 3; ++gt) {
                      Ze = st[gt], Ge = st[(gt + 1) % 3];
                      var bt = et[3 * Ve + gt] = it.locate(Ge, Ze, Pe.opposite(Ge, Ze)), yt = dt[3 * Ve + gt] = Pe.isConstraint(Ze, Ge);
                      bt < 0 && (yt ? ct.push(Ve) : (at.push(Ve), We[Ve] = 1), $e && Je.push([Ge, Ze, -1]));
                    }
                    return it;
                  }(pe, ge);
                  if (ye === 0) return ge ? ve.cells.concat(ve.boundary) : ve.cells;
                  for (var be = 1, Ce = ve.active, Se = ve.next, we = ve.flags, Ee = ve.cells, xe = ve.constraint, Le = ve.neighbor; Ce.length > 0 || Se.length > 0; ) {
                    for (; Ce.length > 0; ) {
                      var Qe = Ce.pop();
                      if (we[Qe] !== -be) {
                        we[Qe] = be, Ee[Qe];
                        for (var ke = 0; ke < 3; ++ke) {
                          var Me = Le[3 * Qe + ke];
                          Me >= 0 && we[Me] === 0 && (xe[3 * Qe + ke] ? Se.push(Me) : (Ce.push(Me), we[Me] = be));
                        }
                      }
                    }
                    var Re = Se;
                    Se = Ce, Ce = Re, Se.length = 0, be = -be;
                  }
                  var De = function(Pe, $e, qe) {
                    for (var Xe = 0, Ve = 0; Ve < Pe.length; ++Ve) $e[Ve] === qe && (Pe[Xe++] = Pe[Ve]);
                    return Pe.length = Xe, Pe;
                  }(Ee, we, ye);
                  return ge ? De.concat(ve.boundary) : De;
                }, he.prototype.locate = (de = [0, 0, 0], function(pe, ye, ge) {
                  var ve = pe, be = ye, Ce = ge;
                  return ye < ge ? ye < pe && (ve = ye, be = ge, Ce = pe) : ge < pe && (ve = ge, be = pe, Ce = ye), ve < 0 ? -1 : (de[0] = ve, de[1] = be, de[2] = Ce, fe.eq(this.cells, de, me));
                });
              }, 8902: function(ce, le, ue) {
                var de = ue(2478), fe = ue(3250)[3];
                function he(Se, we, Ee, xe, Le) {
                  this.a = Se, this.b = we, this.idx = Ee, this.lowerIds = xe, this.upperIds = Le;
                }
                function me(Se, we, Ee, xe) {
                  this.a = Se, this.b = we, this.type = Ee, this.idx = xe;
                }
                function pe(Se, we) {
                  var Ee = Se.a[0] - we.a[0] || Se.a[1] - we.a[1] || Se.type - we.type;
                  return Ee || (Se.type !== 0 && (Ee = fe(Se.a, Se.b, we.b)) ? Ee : Se.idx - we.idx);
                }
                function ye(Se, we) {
                  return fe(Se.a, Se.b, we);
                }
                function ge(Se, we, Ee, xe, Le) {
                  for (var Qe = de.lt(we, xe, ye), ke = de.gt(we, xe, ye), Me = Qe; Me < ke; ++Me) {
                    for (var Re = we[Me], De = Re.lowerIds, Pe = De.length; Pe > 1 && fe(Ee[De[Pe - 2]], Ee[De[Pe - 1]], xe) > 0; ) Se.push([De[Pe - 1], De[Pe - 2], Le]), Pe -= 1;
                    De.length = Pe, De.push(Le);
                    var $e = Re.upperIds;
                    for (Pe = $e.length; Pe > 1 && fe(Ee[$e[Pe - 2]], Ee[$e[Pe - 1]], xe) < 0; ) Se.push([$e[Pe - 2], $e[Pe - 1], Le]), Pe -= 1;
                    $e.length = Pe, $e.push(Le);
                  }
                }
                function ve(Se, we) {
                  var Ee;
                  return (Ee = Se.a[0] < we.a[0] ? fe(Se.a, Se.b, we.a) : fe(we.b, we.a, Se.a)) ? Ee : (Ee = we.b[0] < Se.b[0] ? fe(Se.a, Se.b, we.b) : fe(we.b, we.a, Se.b)) || Se.idx - we.idx;
                }
                function be(Se, we, Ee) {
                  var xe = de.le(Se, Ee, ve), Le = Se[xe], Qe = Le.upperIds, ke = Qe[Qe.length - 1];
                  Le.upperIds = [ke], Se.splice(xe + 1, 0, new he(Ee.a, Ee.b, Ee.idx, [ke], Qe));
                }
                function Ce(Se, we, Ee) {
                  var xe = Ee.a;
                  Ee.a = Ee.b, Ee.b = xe;
                  var Le = de.eq(Se, Ee, ve), Qe = Se[Le];
                  Se[Le - 1].upperIds = Qe.upperIds, Se.splice(Le, 1);
                }
                ce.exports = function(Se, we) {
                  for (var Ee = Se.length, xe = we.length, Le = [], Qe = 0; Qe < Ee; ++Qe) Le.push(new me(Se[Qe], null, 0, Qe));
                  for (Qe = 0; Qe < xe; ++Qe) {
                    var ke = we[Qe], Me = Se[ke[0]], Re = Se[ke[1]];
                    Me[0] < Re[0] ? Le.push(new me(Me, Re, 2, Qe), new me(Re, Me, 1, Qe)) : Me[0] > Re[0] && Le.push(new me(Re, Me, 2, Qe), new me(Me, Re, 1, Qe));
                  }
                  Le.sort(pe);
                  for (var De = Le[0].a[0] - (1 + Math.abs(Le[0].a[0])) * Math.pow(2, -52), Pe = [new he([De, 1], [De, 0], -1, [], [])], $e = [], qe = (Qe = 0, Le.length); Qe < qe; ++Qe) {
                    var Xe = Le[Qe], Ve = Xe.type;
                    Ve === 0 ? ge($e, Pe, Se, Xe.a, Xe.idx) : Ve === 2 ? be(Pe, 0, Xe) : Ce(Pe, 0, Xe);
                  }
                  return $e;
                };
              }, 5542: function(ce, le, ue) {
                var de = ue(2478);
                function fe(pe, ye) {
                  this.stars = pe, this.edges = ye;
                }
                ce.exports = function(pe, ye) {
                  for (var ge = new Array(pe), ve = 0; ve < pe; ++ve) ge[ve] = [];
                  return new fe(ge, ye);
                };
                var he = fe.prototype;
                function me(pe, ye, ge) {
                  for (var ve = 1, be = pe.length; ve < be; ve += 2) if (pe[ve - 1] === ye && pe[ve] === ge) return pe[ve - 1] = pe[be - 2], pe[ve] = pe[be - 1], void (pe.length = be - 2);
                }
                he.isConstraint = /* @__PURE__ */ function() {
                  var pe = [0, 0];
                  function ye(ge, ve) {
                    return ge[0] - ve[0] || ge[1] - ve[1];
                  }
                  return function(ge, ve) {
                    return pe[0] = Math.min(ge, ve), pe[1] = Math.max(ge, ve), de.eq(this.edges, pe, ye) >= 0;
                  };
                }(), he.removeTriangle = function(pe, ye, ge) {
                  var ve = this.stars;
                  me(ve[pe], ye, ge), me(ve[ye], ge, pe), me(ve[ge], pe, ye);
                }, he.addTriangle = function(pe, ye, ge) {
                  var ve = this.stars;
                  ve[pe].push(ye, ge), ve[ye].push(ge, pe), ve[ge].push(pe, ye);
                }, he.opposite = function(pe, ye) {
                  for (var ge = this.stars[ye], ve = 1, be = ge.length; ve < be; ve += 2) if (ge[ve] === pe) return ge[ve - 1];
                  return -1;
                }, he.flip = function(pe, ye) {
                  var ge = this.opposite(pe, ye), ve = this.opposite(ye, pe);
                  this.removeTriangle(pe, ye, ge), this.removeTriangle(ye, pe, ve), this.addTriangle(pe, ve, ge), this.addTriangle(ye, ge, ve);
                }, he.edges = function() {
                  for (var pe = this.stars, ye = [], ge = 0, ve = pe.length; ge < ve; ++ge) for (var be = pe[ge], Ce = 0, Se = be.length; Ce < Se; Ce += 2) ye.push([be[Ce], be[Ce + 1]]);
                  return ye;
                }, he.cells = function() {
                  for (var pe = this.stars, ye = [], ge = 0, ve = pe.length; ge < ve; ++ge) for (var be = pe[ge], Ce = 0, Se = be.length; Ce < Se; Ce += 2) {
                    var we = be[Ce], Ee = be[Ce + 1];
                    ge < Math.min(we, Ee) && ye.push([ge, we, Ee]);
                  }
                  return ye;
                };
              }, 2419: function(ce) {
                ce.exports = function(le) {
                  for (var ue = 1, de = 1; de < le.length; ++de) for (var fe = 0; fe < de; ++fe) if (le[de] < le[fe]) ue = -ue;
                  else if (le[fe] === le[de]) return 0;
                  return ue;
                };
              }, 3628: function(ce, le, ue) {
                var de = ue(1338), fe = ue(727);
                function he(ye, ge) {
                  for (var ve = 0, be = ye.length, Ce = 0; Ce < be; ++Ce) ve += ye[Ce] * ge[Ce];
                  return ve;
                }
                function me(ye) {
                  var ge = ye.length;
                  if (ge === 0) return [];
                  ye[0].length;
                  var ve = de([ye.length + 1, ye.length + 1], 1), be = de([ye.length + 1], 1);
                  ve[ge][ge] = 0;
                  for (var Ce = 0; Ce < ge; ++Ce) {
                    for (var Se = 0; Se <= Ce; ++Se) ve[Se][Ce] = ve[Ce][Se] = 2 * he(ye[Ce], ye[Se]);
                    be[Ce] = he(ye[Ce], ye[Ce]);
                  }
                  var we = fe(ve, be), Ee = 0, xe = we[ge + 1];
                  for (Ce = 0; Ce < xe.length; ++Ce) Ee += xe[Ce];
                  var Le = new Array(ge);
                  for (Ce = 0; Ce < ge; ++Ce) {
                    xe = we[Ce];
                    var Qe = 0;
                    for (Se = 0; Se < xe.length; ++Se) Qe += xe[Se];
                    Le[Ce] = Qe / Ee;
                  }
                  return Le;
                }
                function pe(ye) {
                  if (ye.length === 0) return [];
                  for (var ge = ye[0].length, ve = de([ge]), be = me(ye), Ce = 0; Ce < ye.length; ++Ce) for (var Se = 0; Se < ge; ++Se) ve[Se] += ye[Ce][Se] * be[Ce];
                  return ve;
                }
                pe.barycenetric = me, ce.exports = pe;
              }, 6037: function(ce, le, ue) {
                ce.exports = function(fe) {
                  for (var he = de(fe), me = 0, pe = 0; pe < fe.length; ++pe) for (var ye = fe[pe], ge = 0; ge < he.length; ++ge) me += Math.pow(ye[ge] - he[ge], 2);
                  return Math.sqrt(me / fe.length);
                };
                var de = ue(3628);
              }, 332: function(ce, le, ue) {
                ce.exports = function(ke, Me, Re) {
                  var De;
                  if (Re) {
                    De = Me;
                    for (var Pe = new Array(Me.length), $e = 0; $e < Me.length; ++$e) {
                      var qe = Me[$e];
                      Pe[$e] = [qe[0], qe[1], Re[$e]];
                    }
                    Me = Pe;
                  }
                  for (var Xe = function(Ve, Ze, Ge) {
                    var ze = we(Ve, [], Se(Ve));
                    return Le(Ze, ze, Ge), !!ze;
                  }(ke, Me, !!Re); Qe(ke, Me, !!Re); ) Xe = !0;
                  if (Re && Xe) for (De.length = 0, Re.length = 0, $e = 0; $e < Me.length; ++$e) qe = Me[$e], De.push([qe[0], qe[1]]), Re.push(qe[2]);
                  return Xe;
                };
                var de = ue(1755), fe = ue(6867), he = ue(1125), me = ue(7842), pe = ue(1318), ye = ue(946), ge = ue(5838), ve = ue(1278), be = ue(3637);
                function Ce(ke) {
                  var Me = ye(ke);
                  return [ve(Me, -1 / 0), ve(Me, 1 / 0)];
                }
                function Se(ke) {
                  for (var Me = new Array(ke.length), Re = 0; Re < ke.length; ++Re) {
                    var De = ke[Re];
                    Me[Re] = [ve(De[0], -1 / 0), ve(De[1], -1 / 0), ve(De[0], 1 / 0), ve(De[1], 1 / 0)];
                  }
                  return Me;
                }
                function we(ke, Me, Re) {
                  for (var De = Me.length, Pe = new de(De), $e = [], qe = 0; qe < Me.length; ++qe) {
                    var Xe = Me[qe], Ve = Ce(Xe[0]), Ze = Ce(Xe[1]);
                    $e.push([ve(Ve[0], -1 / 0), ve(Ze[0], -1 / 0), ve(Ve[1], 1 / 0), ve(Ze[1], 1 / 0)]);
                  }
                  fe($e, function(ct, et) {
                    Pe.link(ct, et);
                  });
                  var Ge = !0, ze = new Array(De);
                  for (qe = 0; qe < De; ++qe) (at = Pe.find(qe)) !== qe && (Ge = !1, ke[at] = [Math.min(ke[qe][0], ke[at][0]), Math.min(ke[qe][1], ke[at][1])]);
                  if (Ge) return null;
                  var We = 0;
                  for (qe = 0; qe < De; ++qe) {
                    var at;
                    (at = Pe.find(qe)) === qe ? (ze[qe] = We, ke[We++] = ke[qe]) : ze[qe] = -1;
                  }
                  for (ke.length = We, qe = 0; qe < De; ++qe) ze[qe] < 0 && (ze[qe] = ze[Pe.find(qe)]);
                  return ze;
                }
                function Ee(ke, Me) {
                  return ke[0] - Me[0] || ke[1] - Me[1];
                }
                function xe(ke, Me) {
                  return ke[0] - Me[0] || ke[1] - Me[1] || (ke[2] < Me[2] ? -1 : ke[2] > Me[2] ? 1 : 0);
                }
                function Le(ke, Me, Re) {
                  if (ke.length !== 0) {
                    if (Me) for (var De = 0; De < ke.length; ++De) {
                      var Pe = Me[(qe = ke[De])[0]], $e = Me[qe[1]];
                      qe[0] = Math.min(Pe, $e), qe[1] = Math.max(Pe, $e);
                    }
                    else for (De = 0; De < ke.length; ++De) {
                      var qe;
                      Pe = (qe = ke[De])[0], $e = qe[1], qe[0] = Math.min(Pe, $e), qe[1] = Math.max(Pe, $e);
                    }
                    Re ? ke.sort(xe) : ke.sort(Ee);
                    var Xe = 1;
                    for (De = 1; De < ke.length; ++De) {
                      var Ve = ke[De - 1], Ze = ke[De];
                      (Ze[0] !== Ve[0] || Ze[1] !== Ve[1] || Re && Ze[2] !== Ve[2]) && (ke[Xe++] = Ze);
                    }
                    ke.length = Xe;
                  }
                }
                function Qe(ke, Me, Re) {
                  var De = function(Ve, Ze) {
                    for (var Ge = new Array(Ze.length), ze = 0; ze < Ze.length; ++ze) {
                      var We = Ze[ze], at = Ve[We[0]], ct = Ve[We[1]];
                      Ge[ze] = [ve(Math.min(at[0], ct[0]), -1 / 0), ve(Math.min(at[1], ct[1]), -1 / 0), ve(Math.max(at[0], ct[0]), 1 / 0), ve(Math.max(at[1], ct[1]), 1 / 0)];
                    }
                    return Ge;
                  }(ke, Me), Pe = function(Ve, Ze, Ge) {
                    var ze = [];
                    return fe(Ge, function(We, at) {
                      var ct = Ze[We], et = Ze[at];
                      if (ct[0] !== et[0] && ct[0] !== et[1] && ct[1] !== et[0] && ct[1] !== et[1]) {
                        var dt = Ve[ct[0]], Je = Ve[ct[1]], it = Ve[et[0]], st = Ve[et[1]];
                        he(dt, Je, it, st) && ze.push([We, at]);
                      }
                    }), ze;
                  }(ke, Me, De), $e = function(Ve, Ze, Ge, ze) {
                    var We = [];
                    return fe(Ge, ze, function(at, ct) {
                      var et = Ze[at];
                      if (et[0] !== ct && et[1] !== ct) {
                        var dt = Ve[ct], Je = Ve[et[0]], it = Ve[et[1]];
                        he(Je, it, dt, dt) && We.push([at, ct]);
                      }
                    }), We;
                  }(ke, Me, De, Se(ke)), qe = function(Ve, Ze, Ge, ze, We) {
                    var at, ct, et = Ve.map(function(Ft) {
                      return [me(Ft[0]), me(Ft[1])];
                    });
                    for (at = 0; at < Ge.length; ++at) {
                      var dt = Ge[at];
                      ct = dt[0];
                      var Je = dt[1], it = Ze[ct], st = Ze[Je], gt = be(ge(Ve[it[0]]), ge(Ve[it[1]]), ge(Ve[st[0]]), ge(Ve[st[1]]));
                      if (gt) {
                        var bt = Ve.length;
                        Ve.push([ye(gt[0]), ye(gt[1])]), et.push(gt), ze.push([ct, bt], [Je, bt]);
                      }
                    }
                    for (ze.sort(function(Ft, Tt) {
                      if (Ft[0] !== Tt[0]) return Ft[0] - Tt[0];
                      var It = et[Ft[1]], Ct = et[Tt[1]];
                      return pe(It[0], Ct[0]) || pe(It[1], Ct[1]);
                    }), at = ze.length - 1; at >= 0; --at) {
                      var yt = Ze[ct = (Nt = ze[at])[0]], St = yt[0], Lt = yt[1], Rt = Ve[St], vt = Ve[Lt];
                      if ((Rt[0] - vt[0] || Rt[1] - vt[1]) < 0) {
                        var $t = St;
                        St = Lt, Lt = $t;
                      }
                      yt[0] = St;
                      var Pt, Xt = yt[1] = Nt[1];
                      for (We && (Pt = yt[2]); at > 0 && ze[at - 1][0] === ct; ) {
                        var Nt, Gt = (Nt = ze[--at])[1];
                        We ? Ze.push([Xt, Gt, Pt]) : Ze.push([Xt, Gt]), Xt = Gt;
                      }
                      We ? Ze.push([Xt, Lt, Pt]) : Ze.push([Xt, Lt]);
                    }
                    return et;
                  }(ke, Me, Pe, $e, Re), Xe = we(ke, qe);
                  return Le(Me, Xe, Re), !!Xe || Pe.length > 0 || $e.length > 0;
                }
              }, 3637: function(ce, le, ue) {
                ce.exports = function(be, Ce, Se, we) {
                  var Ee = pe(Ce, be), xe = pe(we, Se), Le = ve(Ee, xe);
                  if (me(Le) === 0) return null;
                  var Qe = ve(xe, pe(be, Se)), ke = fe(Qe, Le), Me = ge(Ee, ke);
                  return ye(be, Me);
                };
                var de = ue(6504), fe = ue(8697), he = ue(5572), me = ue(7721), pe = ue(544), ye = ue(2653), ge = ue(8987);
                function ve(be, Ce) {
                  return he(de(be[0], Ce[1]), de(be[1], Ce[0]));
                }
              }, 3642: function(ce) {
                ce.exports = { jet: [{ index: 0, rgb: [0, 0, 131] }, { index: 0.125, rgb: [0, 60, 170] }, { index: 0.375, rgb: [5, 255, 255] }, { index: 0.625, rgb: [255, 255, 0] }, { index: 0.875, rgb: [250, 0, 0] }, { index: 1, rgb: [128, 0, 0] }], hsv: [{ index: 0, rgb: [255, 0, 0] }, { index: 0.169, rgb: [253, 255, 2] }, { index: 0.173, rgb: [247, 255, 2] }, { index: 0.337, rgb: [0, 252, 4] }, { index: 0.341, rgb: [0, 252, 10] }, { index: 0.506, rgb: [1, 249, 255] }, { index: 0.671, rgb: [2, 0, 253] }, { index: 0.675, rgb: [8, 0, 253] }, { index: 0.839, rgb: [255, 0, 251] }, { index: 0.843, rgb: [255, 0, 245] }, { index: 1, rgb: [255, 0, 6] }], hot: [{ index: 0, rgb: [0, 0, 0] }, { index: 0.3, rgb: [230, 0, 0] }, { index: 0.6, rgb: [255, 210, 0] }, { index: 1, rgb: [255, 255, 255] }], spring: [{ index: 0, rgb: [255, 0, 255] }, { index: 1, rgb: [255, 255, 0] }], summer: [{ index: 0, rgb: [0, 128, 102] }, { index: 1, rgb: [255, 255, 102] }], autumn: [{ index: 0, rgb: [255, 0, 0] }, { index: 1, rgb: [255, 255, 0] }], winter: [{ index: 0, rgb: [0, 0, 255] }, { index: 1, rgb: [0, 255, 128] }], bone: [{ index: 0, rgb: [0, 0, 0] }, { index: 0.376, rgb: [84, 84, 116] }, { index: 0.753, rgb: [169, 200, 200] }, { index: 1, rgb: [255, 255, 255] }], copper: [{ index: 0, rgb: [0, 0, 0] }, { index: 0.804, rgb: [255, 160, 102] }, { index: 1, rgb: [255, 199, 127] }], greys: [{ index: 0, rgb: [0, 0, 0] }, { index: 1, rgb: [255, 255, 255] }], yignbu: [{ index: 0, rgb: [8, 29, 88] }, { index: 0.125, rgb: [37, 52, 148] }, { index: 0.25, rgb: [34, 94, 168] }, { index: 0.375, rgb: [29, 145, 192] }, { index: 0.5, rgb: [65, 182, 196] }, { index: 0.625, rgb: [127, 205, 187] }, { index: 0.75, rgb: [199, 233, 180] }, { index: 0.875, rgb: [237, 248, 217] }, { index: 1, rgb: [255, 255, 217] }], greens: [{ index: 0, rgb: [0, 68, 27] }, { index: 0.125, rgb: [0, 109, 44] }, { index: 0.25, rgb: [35, 139, 69] }, { index: 0.375, rgb: [65, 171, 93] }, { index: 0.5, rgb: [116, 196, 118] }, { index: 0.625, rgb: [161, 217, 155] }, { index: 0.75, rgb: [199, 233, 192] }, { index: 0.875, rgb: [229, 245, 224] }, { index: 1, rgb: [247, 252, 245] }], yiorrd: [{ index: 0, rgb: [128, 0, 38] }, { index: 0.125, rgb: [189, 0, 38] }, { index: 0.25, rgb: [227, 26, 28] }, { index: 0.375, rgb: [252, 78, 42] }, { index: 0.5, rgb: [253, 141, 60] }, { index: 0.625, rgb: [254, 178, 76] }, { index: 0.75, rgb: [254, 217, 118] }, { index: 0.875, rgb: [255, 237, 160] }, { index: 1, rgb: [255, 255, 204] }], bluered: [{ index: 0, rgb: [0, 0, 255] }, { index: 1, rgb: [255, 0, 0] }], rdbu: [{ index: 0, rgb: [5, 10, 172] }, { index: 0.35, rgb: [106, 137, 247] }, { index: 0.5, rgb: [190, 190, 190] }, { index: 0.6, rgb: [220, 170, 132] }, { index: 0.7, rgb: [230, 145, 90] }, { index: 1, rgb: [178, 10, 28] }], picnic: [{ index: 0, rgb: [0, 0, 255] }, { index: 0.1, rgb: [51, 153, 255] }, { index: 0.2, rgb: [102, 204, 255] }, { index: 0.3, rgb: [153, 204, 255] }, { index: 0.4, rgb: [204, 204, 255] }, { index: 0.5, rgb: [255, 255, 255] }, { index: 0.6, rgb: [255, 204, 255] }, { index: 0.7, rgb: [255, 153, 255] }, { index: 0.8, rgb: [255, 102, 204] }, { index: 0.9, rgb: [255, 102, 102] }, { index: 1, rgb: [255, 0, 0] }], rainbow: [{ index: 0, rgb: [150, 0, 90] }, { index: 0.125, rgb: [0, 0, 200] }, { index: 0.25, rgb: [0, 25, 255] }, { index: 0.375, rgb: [0, 152, 255] }, { index: 0.5, rgb: [44, 255, 150] }, { index: 0.625, rgb: [151, 255, 0] }, { index: 0.75, rgb: [255, 234, 0] }, { index: 0.875, rgb: [255, 111, 0] }, { index: 1, rgb: [255, 0, 0] }], portland: [{ index: 0, rgb: [12, 51, 131] }, { index: 0.25, rgb: [10, 136, 186] }, { index: 0.5, rgb: [242, 211, 56] }, { index: 0.75, rgb: [242, 143, 56] }, { index: 1, rgb: [217, 30, 30] }], blackbody: [{ index: 0, rgb: [0, 0, 0] }, { index: 0.2, rgb: [230, 0, 0] }, { index: 0.4, rgb: [230, 210, 0] }, { index: 0.7, rgb: [255, 255, 255] }, { index: 1, rgb: [160, 200, 255] }], earth: [{ index: 0, rgb: [0, 0, 130] }, { index: 0.1, rgb: [0, 180, 180] }, { index: 0.2, rgb: [40, 210, 40] }, { index: 0.4, rgb: [230, 230, 50] }, { index: 0.6, rgb: [120, 70, 20] }, { index: 1, rgb: [255, 255, 255] }], electric: [{ index: 0, rgb: [0, 0, 0] }, { index: 0.15, rgb: [30, 0, 100] }, { index: 0.4, rgb: [120, 0, 100] }, { index: 0.6, rgb: [160, 90, 0] }, { index: 0.8, rgb: [230, 200, 0] }, { index: 1, rgb: [255, 250, 220] }], alpha: [{ index: 0, rgb: [255, 255, 255, 0] }, { index: 1, rgb: [255, 255, 255, 1] }], viridis: [{ index: 0, rgb: [68, 1, 84] }, { index: 0.13, rgb: [71, 44, 122] }, { index: 0.25, rgb: [59, 81, 139] }, { index: 0.38, rgb: [44, 113, 142] }, { index: 0.5, rgb: [33, 144, 141] }, { index: 0.63, rgb: [39, 173, 129] }, { index: 0.75, rgb: [92, 200, 99] }, { index: 0.88, rgb: [170, 220, 50] }, { index: 1, rgb: [253, 231, 37] }], inferno: [{ index: 0, rgb: [0, 0, 4] }, { index: 0.13, rgb: [31, 12, 72] }, { index: 0.25, rgb: [85, 15, 109] }, { index: 0.38, rgb: [136, 34, 106] }, { index: 0.5, rgb: [186, 54, 85] }, { index: 0.63, rgb: [227, 89, 51] }, { index: 0.75, rgb: [249, 140, 10] }, { index: 0.88, rgb: [249, 201, 50] }, { index: 1, rgb: [252, 255, 164] }], magma: [{ index: 0, rgb: [0, 0, 4] }, { index: 0.13, rgb: [28, 16, 68] }, { index: 0.25, rgb: [79, 18, 123] }, { index: 0.38, rgb: [129, 37, 129] }, { index: 0.5, rgb: [181, 54, 122] }, { index: 0.63, rgb: [229, 80, 100] }, { index: 0.75, rgb: [251, 135, 97] }, { index: 0.88, rgb: [254, 194, 135] }, { index: 1, rgb: [252, 253, 191] }], plasma: [{ index: 0, rgb: [13, 8, 135] }, { index: 0.13, rgb: [75, 3, 161] }, { index: 0.25, rgb: [125, 3, 168] }, { index: 0.38, rgb: [168, 34, 150] }, { index: 0.5, rgb: [203, 70, 121] }, { index: 0.63, rgb: [229, 107, 93] }, { index: 0.75, rgb: [248, 148, 65] }, { index: 0.88, rgb: [253, 195, 40] }, { index: 1, rgb: [240, 249, 33] }], warm: [{ index: 0, rgb: [125, 0, 179] }, { index: 0.13, rgb: [172, 0, 187] }, { index: 0.25, rgb: [219, 0, 170] }, { index: 0.38, rgb: [255, 0, 130] }, { index: 0.5, rgb: [255, 63, 74] }, { index: 0.63, rgb: [255, 123, 0] }, { index: 0.75, rgb: [234, 176, 0] }, { index: 0.88, rgb: [190, 228, 0] }, { index: 1, rgb: [147, 255, 0] }], cool: [{ index: 0, rgb: [125, 0, 179] }, { index: 0.13, rgb: [116, 0, 218] }, { index: 0.25, rgb: [98, 74, 237] }, { index: 0.38, rgb: [68, 146, 231] }, { index: 0.5, rgb: [0, 204, 197] }, { index: 0.63, rgb: [0, 247, 146] }, { index: 0.75, rgb: [0, 255, 88] }, { index: 0.88, rgb: [40, 255, 8] }, { index: 1, rgb: [147, 255, 0] }], "rainbow-soft": [{ index: 0, rgb: [125, 0, 179] }, { index: 0.1, rgb: [199, 0, 180] }, { index: 0.2, rgb: [255, 0, 121] }, { index: 0.3, rgb: [255, 108, 0] }, { index: 0.4, rgb: [222, 194, 0] }, { index: 0.5, rgb: [150, 255, 0] }, { index: 0.6, rgb: [0, 255, 55] }, { index: 0.7, rgb: [0, 246, 150] }, { index: 0.8, rgb: [50, 167, 222] }, { index: 0.9, rgb: [103, 51, 235] }, { index: 1, rgb: [124, 0, 186] }], bathymetry: [{ index: 0, rgb: [40, 26, 44] }, { index: 0.13, rgb: [59, 49, 90] }, { index: 0.25, rgb: [64, 76, 139] }, { index: 0.38, rgb: [63, 110, 151] }, { index: 0.5, rgb: [72, 142, 158] }, { index: 0.63, rgb: [85, 174, 163] }, { index: 0.75, rgb: [120, 206, 163] }, { index: 0.88, rgb: [187, 230, 172] }, { index: 1, rgb: [253, 254, 204] }], cdom: [{ index: 0, rgb: [47, 15, 62] }, { index: 0.13, rgb: [87, 23, 86] }, { index: 0.25, rgb: [130, 28, 99] }, { index: 0.38, rgb: [171, 41, 96] }, { index: 0.5, rgb: [206, 67, 86] }, { index: 0.63, rgb: [230, 106, 84] }, { index: 0.75, rgb: [242, 149, 103] }, { index: 0.88, rgb: [249, 193, 135] }, { index: 1, rgb: [254, 237, 176] }], chlorophyll: [{ index: 0, rgb: [18, 36, 20] }, { index: 0.13, rgb: [25, 63, 41] }, { index: 0.25, rgb: [24, 91, 59] }, { index: 0.38, rgb: [13, 119, 72] }, { index: 0.5, rgb: [18, 148, 80] }, { index: 0.63, rgb: [80, 173, 89] }, { index: 0.75, rgb: [132, 196, 122] }, { index: 0.88, rgb: [175, 221, 162] }, { index: 1, rgb: [215, 249, 208] }], density: [{ index: 0, rgb: [54, 14, 36] }, { index: 0.13, rgb: [89, 23, 80] }, { index: 0.25, rgb: [110, 45, 132] }, { index: 0.38, rgb: [120, 77, 178] }, { index: 0.5, rgb: [120, 113, 213] }, { index: 0.63, rgb: [115, 151, 228] }, { index: 0.75, rgb: [134, 185, 227] }, { index: 0.88, rgb: [177, 214, 227] }, { index: 1, rgb: [230, 241, 241] }], "freesurface-blue": [{ index: 0, rgb: [30, 4, 110] }, { index: 0.13, rgb: [47, 14, 176] }, { index: 0.25, rgb: [41, 45, 236] }, { index: 0.38, rgb: [25, 99, 212] }, { index: 0.5, rgb: [68, 131, 200] }, { index: 0.63, rgb: [114, 156, 197] }, { index: 0.75, rgb: [157, 181, 203] }, { index: 0.88, rgb: [200, 208, 216] }, { index: 1, rgb: [241, 237, 236] }], "freesurface-red": [{ index: 0, rgb: [60, 9, 18] }, { index: 0.13, rgb: [100, 17, 27] }, { index: 0.25, rgb: [142, 20, 29] }, { index: 0.38, rgb: [177, 43, 27] }, { index: 0.5, rgb: [192, 87, 63] }, { index: 0.63, rgb: [205, 125, 105] }, { index: 0.75, rgb: [216, 162, 148] }, { index: 0.88, rgb: [227, 199, 193] }, { index: 1, rgb: [241, 237, 236] }], oxygen: [{ index: 0, rgb: [64, 5, 5] }, { index: 0.13, rgb: [106, 6, 15] }, { index: 0.25, rgb: [144, 26, 7] }, { index: 0.38, rgb: [168, 64, 3] }, { index: 0.5, rgb: [188, 100, 4] }, { index: 0.63, rgb: [206, 136, 11] }, { index: 0.75, rgb: [220, 174, 25] }, { index: 0.88, rgb: [231, 215, 44] }, { index: 1, rgb: [248, 254, 105] }], par: [{ index: 0, rgb: [51, 20, 24] }, { index: 0.13, rgb: [90, 32, 35] }, { index: 0.25, rgb: [129, 44, 34] }, { index: 0.38, rgb: [159, 68, 25] }, { index: 0.5, rgb: [182, 99, 19] }, { index: 0.63, rgb: [199, 134, 22] }, { index: 0.75, rgb: [212, 171, 35] }, { index: 0.88, rgb: [221, 210, 54] }, { index: 1, rgb: [225, 253, 75] }], phase: [{ index: 0, rgb: [145, 105, 18] }, { index: 0.13, rgb: [184, 71, 38] }, { index: 0.25, rgb: [186, 58, 115] }, { index: 0.38, rgb: [160, 71, 185] }, { index: 0.5, rgb: [110, 97, 218] }, { index: 0.63, rgb: [50, 123, 164] }, { index: 0.75, rgb: [31, 131, 110] }, { index: 0.88, rgb: [77, 129, 34] }, { index: 1, rgb: [145, 105, 18] }], salinity: [{ index: 0, rgb: [42, 24, 108] }, { index: 0.13, rgb: [33, 50, 162] }, { index: 0.25, rgb: [15, 90, 145] }, { index: 0.38, rgb: [40, 118, 137] }, { index: 0.5, rgb: [59, 146, 135] }, { index: 0.63, rgb: [79, 175, 126] }, { index: 0.75, rgb: [120, 203, 104] }, { index: 0.88, rgb: [193, 221, 100] }, { index: 1, rgb: [253, 239, 154] }], temperature: [{ index: 0, rgb: [4, 35, 51] }, { index: 0.13, rgb: [23, 51, 122] }, { index: 0.25, rgb: [85, 59, 157] }, { index: 0.38, rgb: [129, 79, 143] }, { index: 0.5, rgb: [175, 95, 130] }, { index: 0.63, rgb: [222, 112, 101] }, { index: 0.75, rgb: [249, 146, 66] }, { index: 0.88, rgb: [249, 196, 65] }, { index: 1, rgb: [232, 250, 91] }], turbidity: [{ index: 0, rgb: [34, 31, 27] }, { index: 0.13, rgb: [65, 50, 41] }, { index: 0.25, rgb: [98, 69, 52] }, { index: 0.38, rgb: [131, 89, 57] }, { index: 0.5, rgb: [161, 112, 59] }, { index: 0.63, rgb: [185, 140, 66] }, { index: 0.75, rgb: [202, 174, 88] }, { index: 0.88, rgb: [216, 209, 126] }, { index: 1, rgb: [233, 246, 171] }], "velocity-blue": [{ index: 0, rgb: [17, 32, 64] }, { index: 0.13, rgb: [35, 52, 116] }, { index: 0.25, rgb: [29, 81, 156] }, { index: 0.38, rgb: [31, 113, 162] }, { index: 0.5, rgb: [50, 144, 169] }, { index: 0.63, rgb: [87, 173, 176] }, { index: 0.75, rgb: [149, 196, 189] }, { index: 0.88, rgb: [203, 221, 211] }, { index: 1, rgb: [254, 251, 230] }], "velocity-green": [{ index: 0, rgb: [23, 35, 19] }, { index: 0.13, rgb: [24, 64, 38] }, { index: 0.25, rgb: [11, 95, 45] }, { index: 0.38, rgb: [39, 123, 35] }, { index: 0.5, rgb: [95, 146, 12] }, { index: 0.63, rgb: [152, 165, 18] }, { index: 0.75, rgb: [201, 186, 69] }, { index: 0.88, rgb: [233, 216, 137] }, { index: 1, rgb: [255, 253, 205] }], cubehelix: [{ index: 0, rgb: [0, 0, 0] }, { index: 0.07, rgb: [22, 5, 59] }, { index: 0.13, rgb: [60, 4, 105] }, { index: 0.2, rgb: [109, 1, 135] }, { index: 0.27, rgb: [161, 0, 147] }, { index: 0.33, rgb: [210, 2, 142] }, { index: 0.4, rgb: [251, 11, 123] }, { index: 0.47, rgb: [255, 29, 97] }, { index: 0.53, rgb: [255, 54, 69] }, { index: 0.6, rgb: [255, 85, 46] }, { index: 0.67, rgb: [255, 120, 34] }, { index: 0.73, rgb: [255, 157, 37] }, { index: 0.8, rgb: [241, 191, 57] }, { index: 0.87, rgb: [224, 220, 93] }, { index: 0.93, rgb: [218, 241, 142] }, { index: 1, rgb: [227, 253, 198] }] };
              }, 6729: function(ce, le, ue) {
                var de = ue(3642), fe = ue(395);
                function he(ye) {
                  return [ye[0] / 255, ye[1] / 255, ye[2] / 255, ye[3]];
                }
                function me(ye) {
                  for (var ge, ve = "#", be = 0; be < 3; ++be) ve += ("00" + (ge = (ge = ye[be]).toString(16))).substr(ge.length);
                  return ve;
                }
                function pe(ye) {
                  return "rgba(" + ye.join(",") + ")";
                }
                ce.exports = function(ye) {
                  var ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe;
                  if (ye || (ye = {}), xe = (ye.nshades || 72) - 1, Ee = ye.format || "hex", (we = ye.colormap) || (we = "jet"), typeof we == "string") {
                    if (we = we.toLowerCase(), !de[we]) throw Error(we + " not a supported colorscale");
                    Se = de[we];
                  } else {
                    if (!Array.isArray(we)) throw Error("unsupported colormap option", we);
                    Se = we.slice();
                  }
                  if (Se.length > xe + 1) throw new Error(we + " map requires nshades to be at least size " + Se.length);
                  Le = Array.isArray(ye.alpha) ? ye.alpha.length !== 2 ? [1, 1] : ye.alpha.slice() : typeof ye.alpha == "number" ? [ye.alpha, ye.alpha] : [1, 1], ge = Se.map(function(Pe) {
                    return Math.round(Pe.index * xe);
                  }), Le[0] = Math.min(Math.max(Le[0], 0), 1), Le[1] = Math.min(Math.max(Le[1], 0), 1);
                  var ke = Se.map(function(Pe, $e) {
                    var qe = Se[$e].index, Xe = Se[$e].rgb.slice();
                    return Xe.length === 4 && Xe[3] >= 0 && Xe[3] <= 1 || (Xe[3] = Le[0] + (Le[1] - Le[0]) * qe), Xe;
                  }), Me = [];
                  for (Qe = 0; Qe < ge.length - 1; ++Qe) {
                    Ce = ge[Qe + 1] - ge[Qe], ve = ke[Qe], be = ke[Qe + 1];
                    for (var Re = 0; Re < Ce; Re++) {
                      var De = Re / Ce;
                      Me.push([Math.round(fe(ve[0], be[0], De)), Math.round(fe(ve[1], be[1], De)), Math.round(fe(ve[2], be[2], De)), fe(ve[3], be[3], De)]);
                    }
                  }
                  return Me.push(Se[Se.length - 1].rgb.concat(Le[1])), Ee === "hex" ? Me = Me.map(me) : Ee === "rgbaString" ? Me = Me.map(pe) : Ee === "float" && (Me = Me.map(he)), Me;
                };
              }, 3140: function(ce, le, ue) {
                ce.exports = function(ge, ve, be, Ce) {
                  var Se = de(ve, be, Ce);
                  if (Se === 0) {
                    var we = fe(de(ge, ve, be)), Ee = fe(de(ge, ve, Ce));
                    if (we === Ee) {
                      if (we === 0) {
                        var xe = ye(ge, ve, be);
                        return xe === ye(ge, ve, Ce) ? 0 : xe ? 1 : -1;
                      }
                      return 0;
                    }
                    return Ee === 0 ? we > 0 || ye(ge, ve, Ce) ? -1 : 1 : we === 0 ? Ee > 0 || ye(ge, ve, be) ? 1 : -1 : fe(Ee - we);
                  }
                  var Le = de(ge, ve, be);
                  return Le > 0 ? Se > 0 && de(ge, ve, Ce) > 0 ? 1 : -1 : Le < 0 ? Se > 0 || de(ge, ve, Ce) > 0 ? 1 : -1 : de(ge, ve, Ce) > 0 || ye(ge, ve, be) ? 1 : -1;
                };
                var de = ue(3250), fe = ue(8572), he = ue(9362), me = ue(5382), pe = ue(8210);
                function ye(ge, ve, be) {
                  var Ce = he(ge[0], -ve[0]), Se = he(ge[1], -ve[1]), we = he(be[0], -ve[0]), Ee = he(be[1], -ve[1]), xe = pe(me(Ce, we), me(Se, Ee));
                  return xe[xe.length - 1] >= 0;
                }
              }, 8572: function(ce) {
                ce.exports = function(le) {
                  return le < 0 ? -1 : le > 0 ? 1 : 0;
                };
              }, 8507: function(ce) {
                ce.exports = function(de, fe) {
                  var he = de.length, me = de.length - fe.length;
                  if (me) return me;
                  switch (he) {
                    case 0:
                      return 0;
                    case 1:
                      return de[0] - fe[0];
                    case 2:
                      return de[0] + de[1] - fe[0] - fe[1] || le(de[0], de[1]) - le(fe[0], fe[1]);
                    case 3:
                      var pe = de[0] + de[1], ye = fe[0] + fe[1];
                      if (me = pe + de[2] - (ye + fe[2])) return me;
                      var ge = le(de[0], de[1]), ve = le(fe[0], fe[1]);
                      return le(ge, de[2]) - le(ve, fe[2]) || le(ge + de[2], pe) - le(ve + fe[2], ye);
                    case 4:
                      var be = de[0], Ce = de[1], Se = de[2], we = de[3], Ee = fe[0], xe = fe[1], Le = fe[2], Qe = fe[3];
                      return be + Ce + Se + we - (Ee + xe + Le + Qe) || le(be, Ce, Se, we) - le(Ee, xe, Le, Qe, Ee) || le(be + Ce, be + Se, be + we, Ce + Se, Ce + we, Se + we) - le(Ee + xe, Ee + Le, Ee + Qe, xe + Le, xe + Qe, Le + Qe) || le(be + Ce + Se, be + Ce + we, be + Se + we, Ce + Se + we) - le(Ee + xe + Le, Ee + xe + Qe, Ee + Le + Qe, xe + Le + Qe);
                    default:
                      for (var ke = de.slice().sort(ue), Me = fe.slice().sort(ue), Re = 0; Re < he; ++Re) if (me = ke[Re] - Me[Re]) return me;
                      return 0;
                  }
                };
                var le = Math.min;
                function ue(de, fe) {
                  return de - fe;
                }
              }, 3788: function(ce, le, ue) {
                var de = ue(8507), fe = ue(2419);
                ce.exports = function(he, me) {
                  return de(he, me) || fe(he) - fe(me);
                };
              }, 7352: function(ce, le, ue) {
                var de = ue(5721), fe = ue(4750), he = ue(2690);
                ce.exports = function(me) {
                  var pe = me.length;
                  if (pe === 0) return [];
                  if (pe === 1) return [[0]];
                  var ye = me[0].length;
                  return ye === 0 ? [] : ye === 1 ? de(me) : ye === 2 ? fe(me) : he(me, ye);
                };
              }, 5721: function(ce) {
                ce.exports = function(le) {
                  for (var ue = 0, de = 0, fe = 1; fe < le.length; ++fe) le[fe][0] < le[ue][0] && (ue = fe), le[fe][0] > le[de][0] && (de = fe);
                  return ue < de ? [[ue], [de]] : ue > de ? [[de], [ue]] : [[ue]];
                };
              }, 4750: function(ce, le, ue) {
                ce.exports = function(fe) {
                  var he = de(fe), me = he.length;
                  if (me <= 2) return [];
                  for (var pe = new Array(me), ye = he[me - 1], ge = 0; ge < me; ++ge) {
                    var ve = he[ge];
                    pe[ge] = [ye, ve], ye = ve;
                  }
                  return pe;
                };
                var de = ue(3090);
              }, 2690: function(ce, le, ue) {
                ce.exports = function(he, me) {
                  try {
                    return de(he, !0);
                  } catch {
                    var pe = fe(he);
                    if (pe.length <= me) return [];
                    var ye = function(ve, be) {
                      for (var Ce = ve.length, Se = new Array(Ce), we = 0; we < be.length; ++we) Se[we] = ve[be[we]];
                      var Ee = be.length;
                      for (we = 0; we < Ce; ++we) be.indexOf(we) < 0 && (Se[Ee++] = ve[we]);
                      return Se;
                    }(he, pe);
                    return function(ve, be) {
                      for (var Ce = ve.length, Se = be.length, we = 0; we < Ce; ++we) for (var Ee = ve[we], xe = 0; xe < Ee.length; ++xe) {
                        var Le = Ee[xe];
                        if (Le < Se) Ee[xe] = be[Le];
                        else {
                          Le -= Se;
                          for (var Qe = 0; Qe < Se; ++Qe) Le >= be[Qe] && (Le += 1);
                          Ee[xe] = Le;
                        }
                      }
                      return ve;
                    }(de(ye, !0), pe);
                  }
                };
                var de = ue(8954), fe = ue(3952);
              }, 4769: function(ce) {
                ce.exports = function(le, ue, de, fe, he, me) {
                  var pe = he - 1, ye = he * he, ge = pe * pe, ve = (1 + 2 * he) * ge, be = he * ge, Ce = ye * (3 - 2 * he), Se = ye * pe;
                  if (le.length) {
                    me || (me = new Array(le.length));
                    for (var we = le.length - 1; we >= 0; --we) me[we] = ve * le[we] + be * ue[we] + Ce * de[we] + Se * fe[we];
                    return me;
                  }
                  return ve * le + be * ue + Ce * de + Se * fe;
                }, ce.exports.derivative = function(le, ue, de, fe, he, me) {
                  var pe = 6 * he * he - 6 * he, ye = 3 * he * he - 4 * he + 1, ge = -6 * he * he + 6 * he, ve = 3 * he * he - 2 * he;
                  if (le.length) {
                    me || (me = new Array(le.length));
                    for (var be = le.length - 1; be >= 0; --be) me[be] = pe * le[be] + ye * ue[be] + ge * de[be] + ve * fe[be];
                    return me;
                  }
                  return pe * le + ye * ue + ge * de[be] + ve * fe;
                };
              }, 7642: function(ce, le, ue) {
                var de = ue(8954), fe = ue(1682);
                function he(pe, ye) {
                  this.point = pe, this.index = ye;
                }
                function me(pe, ye) {
                  for (var ge = pe.point, ve = ye.point, be = ge.length, Ce = 0; Ce < be; ++Ce) {
                    var Se = ve[Ce] - ge[Ce];
                    if (Se) return Se;
                  }
                  return 0;
                }
                ce.exports = function(pe, ye) {
                  var ge = pe.length;
                  if (ge === 0) return [];
                  var ve = pe[0].length;
                  if (ve < 1) return [];
                  if (ve === 1) return function(qe, Xe, Ve) {
                    if (qe === 1) return Ve ? [[-1, 0]] : [];
                    var Ze = Xe.map(function(ct, et) {
                      return [ct[0], et];
                    });
                    Ze.sort(function(ct, et) {
                      return ct[0] - et[0];
                    });
                    for (var Ge = new Array(qe - 1), ze = 1; ze < qe; ++ze) {
                      var We = Ze[ze - 1], at = Ze[ze];
                      Ge[ze - 1] = [We[1], at[1]];
                    }
                    return Ve && Ge.push([-1, Ge[0][1]], [Ge[qe - 1][1], -1]), Ge;
                  }(ge, pe, ye);
                  for (var be = new Array(ge), Ce = 1, Se = 0; Se < ge; ++Se) {
                    for (var we = pe[Se], Ee = new Array(ve + 1), xe = 0, Le = 0; Le < ve; ++Le) {
                      var Qe = we[Le];
                      Ee[Le] = Qe, xe += Qe * Qe;
                    }
                    Ee[ve] = xe, be[Se] = new he(Ee, Se), Ce = Math.max(xe, Ce);
                  }
                  fe(be, me), ge = be.length;
                  var ke = new Array(ge + ve + 1), Me = new Array(ge + ve + 1), Re = (ve + 1) * (ve + 1) * Ce, De = new Array(ve + 1);
                  for (Se = 0; Se <= ve; ++Se) De[Se] = 0;
                  for (De[ve] = Re, ke[0] = De.slice(), Me[0] = -1, Se = 0; Se <= ve; ++Se) (Ee = De.slice())[Se] = 1, ke[Se + 1] = Ee, Me[Se + 1] = -1;
                  for (Se = 0; Se < ge; ++Se) {
                    var Pe = be[Se];
                    ke[Se + ve + 1] = Pe.point, Me[Se + ve + 1] = Pe.index;
                  }
                  var $e = de(ke, !1);
                  if ($e = ye ? $e.filter(function(qe) {
                    for (var Xe = 0, Ve = 0; Ve <= ve; ++Ve) {
                      var Ze = Me[qe[Ve]];
                      if (Ze < 0 && ++Xe >= 2) return !1;
                      qe[Ve] = Ze;
                    }
                    return !0;
                  }) : $e.filter(function(qe) {
                    for (var Xe = 0; Xe <= ve; ++Xe) {
                      var Ve = Me[qe[Xe]];
                      if (Ve < 0) return !1;
                      qe[Xe] = Ve;
                    }
                    return !0;
                  }), 1 & ve) for (Se = 0; Se < $e.length; ++Se) Ee = (Pe = $e[Se])[0], Pe[0] = Pe[1], Pe[1] = Ee;
                  return $e;
                };
              }, 2361: function(ce) {
                var le = !1;
                if (typeof Float64Array < "u") {
                  var ue = new Float64Array(1), de = new Uint32Array(ue.buffer);
                  ue[0] = 1, le = !0, de[1] === 1072693248 ? (ce.exports = function(he) {
                    return ue[0] = he, [de[0], de[1]];
                  }, ce.exports.pack = function(he, me) {
                    return de[0] = he, de[1] = me, ue[0];
                  }, ce.exports.lo = function(he) {
                    return ue[0] = he, de[0];
                  }, ce.exports.hi = function(he) {
                    return ue[0] = he, de[1];
                  }) : de[0] === 1072693248 ? (ce.exports = function(he) {
                    return ue[0] = he, [de[1], de[0]];
                  }, ce.exports.pack = function(he, me) {
                    return de[1] = he, de[0] = me, ue[0];
                  }, ce.exports.lo = function(he) {
                    return ue[0] = he, de[1];
                  }, ce.exports.hi = function(he) {
                    return ue[0] = he, de[0];
                  }) : le = !1;
                }
                if (!le) {
                  var fe = new te(8);
                  ce.exports = function(he) {
                    return fe.writeDoubleLE(he, 0, !0), [fe.readUInt32LE(0, !0), fe.readUInt32LE(4, !0)];
                  }, ce.exports.pack = function(he, me) {
                    return fe.writeUInt32LE(he, 0, !0), fe.writeUInt32LE(me, 4, !0), fe.readDoubleLE(0, !0);
                  }, ce.exports.lo = function(he) {
                    return fe.writeDoubleLE(he, 0, !0), fe.readUInt32LE(0, !0);
                  }, ce.exports.hi = function(he) {
                    return fe.writeDoubleLE(he, 0, !0), fe.readUInt32LE(4, !0);
                  };
                }
                ce.exports.sign = function(he) {
                  return ce.exports.hi(he) >>> 31;
                }, ce.exports.exponent = function(he) {
                  return (ce.exports.hi(he) << 1 >>> 21) - 1023;
                }, ce.exports.fraction = function(he) {
                  var me = ce.exports.lo(he), pe = ce.exports.hi(he), ye = 1048575 & pe;
                  return 2146435072 & pe && (ye += 1048576), [me, ye];
                }, ce.exports.denormalized = function(he) {
                  return !(2146435072 & ce.exports.hi(he));
                };
              }, 1338: function(ce) {
                function le(ue, de, fe) {
                  var he = 0 | ue[fe];
                  if (he <= 0) return [];
                  var me, pe = new Array(he);
                  if (fe === ue.length - 1) for (me = 0; me < he; ++me) pe[me] = de;
                  else for (me = 0; me < he; ++me) pe[me] = le(ue, de, fe + 1);
                  return pe;
                }
                ce.exports = function(ue, de) {
                  switch (de === void 0 && (de = 0), typeof ue) {
                    case "number":
                      if (ue > 0) return function(fe, he) {
                        var me, pe;
                        for (me = new Array(fe), pe = 0; pe < fe; ++pe) me[pe] = he;
                        return me;
                      }(0 | ue, de);
                      break;
                    case "object":
                      if (typeof ue.length == "number") return le(ue, de, 0);
                  }
                  return [];
                };
              }, 3134: function(ce, le, ue) {
                ce.exports = function(fe, he) {
                  var me = fe.length;
                  if (typeof he != "number") {
                    he = 0;
                    for (var pe = 0; pe < me; ++pe) {
                      var ye = fe[pe];
                      he = Math.max(he, ye[0], ye[1]);
                    }
                    he = 1 + (0 | he);
                  }
                  he |= 0;
                  var ge = new Array(he);
                  for (pe = 0; pe < he; ++pe) ge[pe] = [];
                  for (pe = 0; pe < me; ++pe) ge[(ye = fe[pe])[0]].push(ye[1]), ge[ye[1]].push(ye[0]);
                  for (var ve = 0; ve < he; ++ve) de(ge[ve], function(be, Ce) {
                    return be - Ce;
                  });
                  return ge;
                };
                var de = ue(1682);
              }, 5033: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue || 0, he = de || 1;
                  return [[le[12] + le[0], le[13] + le[1], le[14] + le[2], le[15] + le[3]], [le[12] - le[0], le[13] - le[1], le[14] - le[2], le[15] - le[3]], [le[12] + le[4], le[13] + le[5], le[14] + le[6], le[15] + le[7]], [le[12] - le[4], le[13] - le[5], le[14] - le[6], le[15] - le[7]], [fe * le[12] + le[8], fe * le[13] + le[9], fe * le[14] + le[10], fe * le[15] + le[11]], [he * le[12] - le[8], he * le[13] - le[9], he * le[14] - le[10], he * le[15] - le[11]]];
                };
              }, 9215: function(ce, le, ue) {
                ce.exports = function(ge, ve, be) {
                  switch (arguments.length) {
                    case 0:
                      return new me([0], [0], 0);
                    case 1:
                      return typeof ge == "number" ? new me(Ce = ye(ge), Ce, 0) : new me(ge, ye(ge.length), 0);
                    case 2:
                      var Ce;
                      if (typeof ve == "number") return new me(ge, Ce = ye(ge.length), +ve);
                      be = 0;
                    case 3:
                      if (ge.length !== ve.length) throw new Error("state and velocity lengths must match");
                      return new me(ge, ve, be);
                  }
                };
                var de = ue(4769), fe = ue(2478);
                function he(ge, ve, be) {
                  return Math.min(ve, Math.max(ge, be));
                }
                function me(ge, ve, be) {
                  this.dimension = ge.length, this.bounds = [new Array(this.dimension), new Array(this.dimension)];
                  for (var Ce = 0; Ce < this.dimension; ++Ce) this.bounds[0][Ce] = -1 / 0, this.bounds[1][Ce] = 1 / 0;
                  this._state = ge.slice().reverse(), this._velocity = ve.slice().reverse(), this._time = [be], this._scratch = [ge.slice(), ge.slice(), ge.slice(), ge.slice(), ge.slice()];
                }
                var pe = me.prototype;
                function ye(ge) {
                  for (var ve = new Array(ge), be = 0; be < ge; ++be) ve[be] = 0;
                  return ve;
                }
                pe.flush = function(ge) {
                  var ve = fe.gt(this._time, ge) - 1;
                  ve <= 0 || (this._time.splice(0, ve), this._state.splice(0, ve * this.dimension), this._velocity.splice(0, ve * this.dimension));
                }, pe.curve = function(ge) {
                  var ve = this._time, be = ve.length, Ce = fe.le(ve, ge), Se = this._scratch[0], we = this._state, Ee = this._velocity, xe = this.dimension, Le = this.bounds;
                  if (Ce < 0) for (var Qe = xe - 1, ke = 0; ke < xe; ++ke, --Qe) Se[ke] = we[Qe];
                  else if (Ce >= be - 1) {
                    Qe = we.length - 1;
                    var Me = ge - ve[be - 1];
                    for (ke = 0; ke < xe; ++ke, --Qe) Se[ke] = we[Qe] + Me * Ee[Qe];
                  } else {
                    Qe = xe * (Ce + 1) - 1;
                    var Re = ve[Ce], De = ve[Ce + 1] - Re || 1, Pe = this._scratch[1], $e = this._scratch[2], qe = this._scratch[3], Xe = this._scratch[4], Ve = !0;
                    for (ke = 0; ke < xe; ++ke, --Qe) Pe[ke] = we[Qe], qe[ke] = Ee[Qe] * De, $e[ke] = we[Qe + xe], Xe[ke] = Ee[Qe + xe] * De, Ve = Ve && Pe[ke] === $e[ke] && qe[ke] === Xe[ke] && qe[ke] === 0;
                    if (Ve) for (ke = 0; ke < xe; ++ke) Se[ke] = Pe[ke];
                    else de(Pe, qe, $e, Xe, (ge - Re) / De, Se);
                  }
                  var Ze = Le[0], Ge = Le[1];
                  for (ke = 0; ke < xe; ++ke) Se[ke] = he(Ze[ke], Ge[ke], Se[ke]);
                  return Se;
                }, pe.dcurve = function(ge) {
                  var ve = this._time, be = ve.length, Ce = fe.le(ve, ge), Se = this._scratch[0], we = this._state, Ee = this._velocity, xe = this.dimension;
                  if (Ce >= be - 1) for (var Le = we.length - 1, Qe = (ve[be - 1], 0); Qe < xe; ++Qe, --Le) Se[Qe] = Ee[Le];
                  else {
                    Le = xe * (Ce + 1) - 1;
                    var ke = ve[Ce], Me = ve[Ce + 1] - ke || 1, Re = this._scratch[1], De = this._scratch[2], Pe = this._scratch[3], $e = this._scratch[4], qe = !0;
                    for (Qe = 0; Qe < xe; ++Qe, --Le) Re[Qe] = we[Le], Pe[Qe] = Ee[Le] * Me, De[Qe] = we[Le + xe], $e[Qe] = Ee[Le + xe] * Me, qe = qe && Re[Qe] === De[Qe] && Pe[Qe] === $e[Qe] && Pe[Qe] === 0;
                    if (qe) for (Qe = 0; Qe < xe; ++Qe) Se[Qe] = 0;
                    else for (de.derivative(Re, Pe, De, $e, (ge - ke) / Me, Se), Qe = 0; Qe < xe; ++Qe) Se[Qe] /= Me;
                  }
                  return Se;
                }, pe.lastT = function() {
                  var ge = this._time;
                  return ge[ge.length - 1];
                }, pe.stable = function() {
                  for (var ge = this._velocity, ve = ge.length, be = this.dimension - 1; be >= 0; --be) if (ge[--ve]) return !1;
                  return !0;
                }, pe.jump = function(ge) {
                  var ve = this.lastT(), be = this.dimension;
                  if (!(ge < ve || arguments.length !== be + 1)) {
                    var Ce = this._state, Se = this._velocity, we = Ce.length - this.dimension, Ee = this.bounds, xe = Ee[0], Le = Ee[1];
                    this._time.push(ve, ge);
                    for (var Qe = 0; Qe < 2; ++Qe) for (var ke = 0; ke < be; ++ke) Ce.push(Ce[we++]), Se.push(0);
                    for (this._time.push(ge), ke = be; ke > 0; --ke) Ce.push(he(xe[ke - 1], Le[ke - 1], arguments[ke])), Se.push(0);
                  }
                }, pe.push = function(ge) {
                  var ve = this.lastT(), be = this.dimension;
                  if (!(ge < ve || arguments.length !== be + 1)) {
                    var Ce = this._state, Se = this._velocity, we = Ce.length - this.dimension, Ee = ge - ve, xe = this.bounds, Le = xe[0], Qe = xe[1], ke = Ee > 1e-6 ? 1 / Ee : 0;
                    this._time.push(ge);
                    for (var Me = be; Me > 0; --Me) {
                      var Re = he(Le[Me - 1], Qe[Me - 1], arguments[Me]);
                      Ce.push(Re), Se.push((Re - Ce[we++]) * ke);
                    }
                  }
                }, pe.set = function(ge) {
                  var ve = this.dimension;
                  if (!(ge < this.lastT() || arguments.length !== ve + 1)) {
                    var be = this._state, Ce = this._velocity, Se = this.bounds, we = Se[0], Ee = Se[1];
                    this._time.push(ge);
                    for (var xe = ve; xe > 0; --xe) be.push(he(we[xe - 1], Ee[xe - 1], arguments[xe])), Ce.push(0);
                  }
                }, pe.move = function(ge) {
                  var ve = this.lastT(), be = this.dimension;
                  if (!(ge <= ve || arguments.length !== be + 1)) {
                    var Ce = this._state, Se = this._velocity, we = Ce.length - this.dimension, Ee = this.bounds, xe = Ee[0], Le = Ee[1], Qe = ge - ve, ke = Qe > 1e-6 ? 1 / Qe : 0;
                    this._time.push(ge);
                    for (var Me = be; Me > 0; --Me) {
                      var Re = arguments[Me];
                      Ce.push(he(xe[Me - 1], Le[Me - 1], Ce[we++] + Re)), Se.push(Re * ke);
                    }
                  }
                }, pe.idle = function(ge) {
                  var ve = this.lastT();
                  if (!(ge < ve)) {
                    var be = this.dimension, Ce = this._state, Se = this._velocity, we = Ce.length - be, Ee = this.bounds, xe = Ee[0], Le = Ee[1], Qe = ge - ve;
                    this._time.push(ge);
                    for (var ke = be - 1; ke >= 0; --ke) Ce.push(he(xe[ke], Le[ke], Ce[we] + Qe * Se[we])), Se.push(0), we += 1;
                  }
                };
              }, 3840: function(ce) {
                function le(we, Ee, xe, Le, Qe, ke) {
                  this._color = we, this.key = Ee, this.value = xe, this.left = Le, this.right = Qe, this._count = ke;
                }
                function ue(we) {
                  return new le(we._color, we.key, we.value, we.left, we.right, we._count);
                }
                function de(we, Ee) {
                  return new le(we, Ee.key, Ee.value, Ee.left, Ee.right, Ee._count);
                }
                function fe(we) {
                  we._count = 1 + (we.left ? we.left._count : 0) + (we.right ? we.right._count : 0);
                }
                function he(we, Ee) {
                  this._compare = we, this.root = Ee;
                }
                ce.exports = function(we) {
                  return new he(we || Se, null);
                };
                var me = he.prototype;
                function pe(we, Ee) {
                  var xe;
                  return Ee.left && (xe = pe(we, Ee.left)) ? xe : (xe = we(Ee.key, Ee.value)) || (Ee.right ? pe(we, Ee.right) : void 0);
                }
                function ye(we, Ee, xe, Le) {
                  if (Ee(we, Le.key) <= 0) {
                    var Qe;
                    if (Le.left && (Qe = ye(we, Ee, xe, Le.left)) || (Qe = xe(Le.key, Le.value))) return Qe;
                  }
                  if (Le.right) return ye(we, Ee, xe, Le.right);
                }
                function ge(we, Ee, xe, Le, Qe) {
                  var ke, Me = xe(we, Qe.key), Re = xe(Ee, Qe.key);
                  if (Me <= 0 && (Qe.left && (ke = ge(we, Ee, xe, Le, Qe.left)) || Re > 0 && (ke = Le(Qe.key, Qe.value))))
                    return ke;
                  if (Re > 0 && Qe.right) return ge(we, Ee, xe, Le, Qe.right);
                }
                function ve(we, Ee) {
                  this.tree = we, this._stack = Ee;
                }
                Object.defineProperty(me, "keys", { get: function() {
                  var we = [];
                  return this.forEach(function(Ee, xe) {
                    we.push(Ee);
                  }), we;
                } }), Object.defineProperty(me, "values", { get: function() {
                  var we = [];
                  return this.forEach(function(Ee, xe) {
                    we.push(xe);
                  }), we;
                } }), Object.defineProperty(me, "length", { get: function() {
                  return this.root ? this.root._count : 0;
                } }), me.insert = function(we, Ee) {
                  for (var xe = this._compare, Le = this.root, Qe = [], ke = []; Le; ) {
                    var Me = xe(we, Le.key);
                    Qe.push(Le), ke.push(Me), Le = Me <= 0 ? Le.left : Le.right;
                  }
                  Qe.push(new le(0, we, Ee, null, null, 1));
                  for (var Re = Qe.length - 2; Re >= 0; --Re) Le = Qe[Re], ke[Re] <= 0 ? Qe[Re] = new le(Le._color, Le.key, Le.value, Qe[Re + 1], Le.right, Le._count + 1) : Qe[Re] = new le(Le._color, Le.key, Le.value, Le.left, Qe[Re + 1], Le._count + 1);
                  for (Re = Qe.length - 1; Re > 1; --Re) {
                    var De = Qe[Re - 1];
                    if (Le = Qe[Re], De._color === 1 || Le._color === 1) break;
                    var Pe = Qe[Re - 2];
                    if (Pe.left === De) if (De.left === Le) {
                      if (!($e = Pe.right) || $e._color !== 0) {
                        Pe._color = 0, Pe.left = De.right, De._color = 1, De.right = Pe, Qe[Re - 2] = De, Qe[Re - 1] = Le, fe(Pe), fe(De), Re >= 3 && ((qe = Qe[Re - 3]).left === Pe ? qe.left = De : qe.right = De);
                        break;
                      }
                      De._color = 1, Pe.right = de(1, $e), Pe._color = 0, Re -= 1;
                    } else {
                      if (!($e = Pe.right) || $e._color !== 0) {
                        De.right = Le.left, Pe._color = 0, Pe.left = Le.right, Le._color = 1, Le.left = De, Le.right = Pe, Qe[Re - 2] = Le, Qe[Re - 1] = De, fe(Pe), fe(De), fe(Le), Re >= 3 && ((qe = Qe[Re - 3]).left === Pe ? qe.left = Le : qe.right = Le);
                        break;
                      }
                      De._color = 1, Pe.right = de(1, $e), Pe._color = 0, Re -= 1;
                    }
                    else if (De.right === Le) {
                      if (!($e = Pe.left) || $e._color !== 0) {
                        Pe._color = 0, Pe.right = De.left, De._color = 1, De.left = Pe, Qe[Re - 2] = De, Qe[Re - 1] = Le, fe(Pe), fe(De), Re >= 3 && ((qe = Qe[Re - 3]).right === Pe ? qe.right = De : qe.left = De);
                        break;
                      }
                      De._color = 1, Pe.left = de(1, $e), Pe._color = 0, Re -= 1;
                    } else {
                      var $e;
                      if (!($e = Pe.left) || $e._color !== 0) {
                        var qe;
                        De.left = Le.right, Pe._color = 0, Pe.right = Le.left, Le._color = 1, Le.right = De, Le.left = Pe, Qe[Re - 2] = Le, Qe[Re - 1] = De, fe(Pe), fe(De), fe(Le), Re >= 3 && ((qe = Qe[Re - 3]).right === Pe ? qe.right = Le : qe.left = Le);
                        break;
                      }
                      De._color = 1, Pe.left = de(1, $e), Pe._color = 0, Re -= 1;
                    }
                  }
                  return Qe[0]._color = 1, new he(xe, Qe[0]);
                }, me.forEach = function(we, Ee, xe) {
                  if (this.root) switch (arguments.length) {
                    case 1:
                      return pe(we, this.root);
                    case 2:
                      return ye(Ee, this._compare, we, this.root);
                    case 3:
                      return this._compare(Ee, xe) >= 0 ? void 0 : ge(Ee, xe, this._compare, we, this.root);
                  }
                }, Object.defineProperty(me, "begin", { get: function() {
                  for (var we = [], Ee = this.root; Ee; ) we.push(Ee), Ee = Ee.left;
                  return new ve(this, we);
                } }), Object.defineProperty(me, "end", { get: function() {
                  for (var we = [], Ee = this.root; Ee; ) we.push(Ee), Ee = Ee.right;
                  return new ve(this, we);
                } }), me.at = function(we) {
                  if (we < 0) return new ve(this, []);
                  for (var Ee = this.root, xe = []; ; ) {
                    if (xe.push(Ee), Ee.left) {
                      if (we < Ee.left._count) {
                        Ee = Ee.left;
                        continue;
                      }
                      we -= Ee.left._count;
                    }
                    if (!we) return new ve(this, xe);
                    if (we -= 1, !Ee.right || we >= Ee.right._count) break;
                    Ee = Ee.right;
                  }
                  return new ve(this, []);
                }, me.ge = function(we) {
                  for (var Ee = this._compare, xe = this.root, Le = [], Qe = 0; xe; ) {
                    var ke = Ee(we, xe.key);
                    Le.push(xe), ke <= 0 && (Qe = Le.length), xe = ke <= 0 ? xe.left : xe.right;
                  }
                  return Le.length = Qe, new ve(this, Le);
                }, me.gt = function(we) {
                  for (var Ee = this._compare, xe = this.root, Le = [], Qe = 0; xe; ) {
                    var ke = Ee(we, xe.key);
                    Le.push(xe), ke < 0 && (Qe = Le.length), xe = ke < 0 ? xe.left : xe.right;
                  }
                  return Le.length = Qe, new ve(this, Le);
                }, me.lt = function(we) {
                  for (var Ee = this._compare, xe = this.root, Le = [], Qe = 0; xe; ) {
                    var ke = Ee(we, xe.key);
                    Le.push(xe), ke > 0 && (Qe = Le.length), xe = ke <= 0 ? xe.left : xe.right;
                  }
                  return Le.length = Qe, new ve(this, Le);
                }, me.le = function(we) {
                  for (var Ee = this._compare, xe = this.root, Le = [], Qe = 0; xe; ) {
                    var ke = Ee(we, xe.key);
                    Le.push(xe), ke >= 0 && (Qe = Le.length), xe = ke < 0 ? xe.left : xe.right;
                  }
                  return Le.length = Qe, new ve(this, Le);
                }, me.find = function(we) {
                  for (var Ee = this._compare, xe = this.root, Le = []; xe; ) {
                    var Qe = Ee(we, xe.key);
                    if (Le.push(xe), Qe === 0) return new ve(this, Le);
                    xe = Qe <= 0 ? xe.left : xe.right;
                  }
                  return new ve(this, []);
                }, me.remove = function(we) {
                  var Ee = this.find(we);
                  return Ee ? Ee.remove() : this;
                }, me.get = function(we) {
                  for (var Ee = this._compare, xe = this.root; xe; ) {
                    var Le = Ee(we, xe.key);
                    if (Le === 0) return xe.value;
                    xe = Le <= 0 ? xe.left : xe.right;
                  }
                };
                var be = ve.prototype;
                function Ce(we, Ee) {
                  we.key = Ee.key, we.value = Ee.value, we.left = Ee.left, we.right = Ee.right, we._color = Ee._color, we._count = Ee._count;
                }
                function Se(we, Ee) {
                  return we < Ee ? -1 : we > Ee ? 1 : 0;
                }
                Object.defineProperty(be, "valid", { get: function() {
                  return this._stack.length > 0;
                } }), Object.defineProperty(be, "node", { get: function() {
                  return this._stack.length > 0 ? this._stack[this._stack.length - 1] : null;
                }, enumerable: !0 }), be.clone = function() {
                  return new ve(this.tree, this._stack.slice());
                }, be.remove = function() {
                  var we = this._stack;
                  if (we.length === 0) return this.tree;
                  var Ee = new Array(we.length), xe = we[we.length - 1];
                  Ee[Ee.length - 1] = new le(xe._color, xe.key, xe.value, xe.left, xe.right, xe._count);
                  for (var Le = we.length - 2; Le >= 0; --Le) (xe = we[Le]).left === we[Le + 1] ? Ee[Le] = new le(xe._color, xe.key, xe.value, Ee[Le + 1], xe.right, xe._count) : Ee[Le] = new le(xe._color, xe.key, xe.value, xe.left, Ee[Le + 1], xe._count);
                  if ((xe = Ee[Ee.length - 1]).left && xe.right) {
                    var Qe = Ee.length;
                    for (xe = xe.left; xe.right; ) Ee.push(xe), xe = xe.right;
                    var ke = Ee[Qe - 1];
                    for (Ee.push(new le(xe._color, ke.key, ke.value, xe.left, xe.right, xe._count)), Ee[Qe - 1].key = xe.key, Ee[Qe - 1].value = xe.value, Le = Ee.length - 2; Le >= Qe; --Le) xe = Ee[Le], Ee[Le] = new le(xe._color, xe.key, xe.value, xe.left, Ee[Le + 1], xe._count);
                    Ee[Qe - 1].left = Ee[Qe];
                  }
                  if ((xe = Ee[Ee.length - 1])._color === 0) {
                    var Me = Ee[Ee.length - 2];
                    for (Me.left === xe ? Me.left = null : Me.right === xe && (Me.right = null), Ee.pop(), Le = 0; Le < Ee.length; ++Le) Ee[Le]._count--;
                    return new he(this.tree._compare, Ee[0]);
                  }
                  if (xe.left || xe.right) {
                    for (xe.left ? Ce(xe, xe.left) : xe.right && Ce(xe, xe.right), xe._color = 1, Le = 0; Le < Ee.length - 1; ++Le) Ee[Le]._count--;
                    return new he(this.tree._compare, Ee[0]);
                  }
                  if (Ee.length === 1) return new he(this.tree._compare, null);
                  for (Le = 0; Le < Ee.length; ++Le) Ee[Le]._count--;
                  var Re = Ee[Ee.length - 2];
                  return function(De) {
                    for (var Pe, $e, qe, Xe, Ve = De.length - 1; Ve >= 0; --Ve) {
                      if (Pe = De[Ve], Ve === 0) return void (Pe._color = 1);
                      if (($e = De[Ve - 1]).left === Pe) {
                        if ((qe = $e.right).right && qe.right._color === 0) return Xe = (qe = $e.right = ue(qe)).right = ue(qe.right), $e.right = qe.left, qe.left = $e, qe.right = Xe, qe._color = $e._color, Pe._color = 1, $e._color = 1, Xe._color = 1, fe($e), fe(qe), Ve > 1 && ((Ze = De[Ve - 2]).left === $e ? Ze.left = qe : Ze.right = qe), void (De[Ve - 1] = qe);
                        if (qe.left && qe.left._color === 0) return Xe = (qe = $e.right = ue(qe)).left = ue(qe.left), $e.right = Xe.left, qe.left = Xe.right, Xe.left = $e, Xe.right = qe, Xe._color = $e._color, $e._color = 1, qe._color = 1, Pe._color = 1, fe($e), fe(qe), fe(Xe), Ve > 1 && ((Ze = De[Ve - 2]).left === $e ? Ze.left = Xe : Ze.right = Xe), void (De[Ve - 1] = Xe);
                        if (qe._color === 1) {
                          if ($e._color === 0) return $e._color = 1, void ($e.right = de(0, qe));
                          $e.right = de(0, qe);
                          continue;
                        }
                        qe = ue(qe), $e.right = qe.left, qe.left = $e, qe._color = $e._color, $e._color = 0, fe($e), fe(qe), Ve > 1 && ((Ze = De[Ve - 2]).left === $e ? Ze.left = qe : Ze.right = qe), De[Ve - 1] = qe, De[Ve] = $e, Ve + 1 < De.length ? De[Ve + 1] = Pe : De.push(Pe), Ve += 2;
                      } else {
                        if ((qe = $e.left).left && qe.left._color === 0) return Xe = (qe = $e.left = ue(qe)).left = ue(qe.left), $e.left = qe.right, qe.right = $e, qe.left = Xe, qe._color = $e._color, Pe._color = 1, $e._color = 1, Xe._color = 1, fe($e), fe(qe), Ve > 1 && ((Ze = De[Ve - 2]).right === $e ? Ze.right = qe : Ze.left = qe), void (De[Ve - 1] = qe);
                        if (qe.right && qe.right._color === 0) return Xe = (qe = $e.left = ue(qe)).right = ue(qe.right), $e.left = Xe.right, qe.right = Xe.left, Xe.right = $e, Xe.left = qe, Xe._color = $e._color, $e._color = 1, qe._color = 1, Pe._color = 1, fe($e), fe(qe), fe(Xe), Ve > 1 && ((Ze = De[Ve - 2]).right === $e ? Ze.right = Xe : Ze.left = Xe), void (De[Ve - 1] = Xe);
                        if (qe._color === 1) {
                          if ($e._color === 0) return $e._color = 1, void ($e.left = de(0, qe));
                          $e.left = de(0, qe);
                          continue;
                        }
                        var Ze;
                        qe = ue(qe), $e.left = qe.right, qe.right = $e, qe._color = $e._color, $e._color = 0, fe($e), fe(qe), Ve > 1 && ((Ze = De[Ve - 2]).right === $e ? Ze.right = qe : Ze.left = qe), De[Ve - 1] = qe, De[Ve] = $e, Ve + 1 < De.length ? De[Ve + 1] = Pe : De.push(Pe), Ve += 2;
                      }
                    }
                  }(Ee), Re.left === xe ? Re.left = null : Re.right = null, new he(this.tree._compare, Ee[0]);
                }, Object.defineProperty(be, "key", { get: function() {
                  if (this._stack.length > 0) return this._stack[this._stack.length - 1].key;
                }, enumerable: !0 }), Object.defineProperty(be, "value", { get: function() {
                  if (this._stack.length > 0) return this._stack[this._stack.length - 1].value;
                }, enumerable: !0 }), Object.defineProperty(be, "index", { get: function() {
                  var we = 0, Ee = this._stack;
                  if (Ee.length === 0) {
                    var xe = this.tree.root;
                    return xe ? xe._count : 0;
                  }
                  Ee[Ee.length - 1].left && (we = Ee[Ee.length - 1].left._count);
                  for (var Le = Ee.length - 2; Le >= 0; --Le) Ee[Le + 1] === Ee[Le].right && (++we, Ee[Le].left && (we += Ee[Le].left._count));
                  return we;
                }, enumerable: !0 }), be.next = function() {
                  var we = this._stack;
                  if (we.length !== 0) {
                    var Ee = we[we.length - 1];
                    if (Ee.right) for (Ee = Ee.right; Ee; ) we.push(Ee), Ee = Ee.left;
                    else for (we.pop(); we.length > 0 && we[we.length - 1].right === Ee; ) Ee = we[we.length - 1], we.pop();
                  }
                }, Object.defineProperty(be, "hasNext", { get: function() {
                  var we = this._stack;
                  if (we.length === 0) return !1;
                  if (we[we.length - 1].right) return !0;
                  for (var Ee = we.length - 1; Ee > 0; --Ee) if (we[Ee - 1].left === we[Ee]) return !0;
                  return !1;
                } }), be.update = function(we) {
                  var Ee = this._stack;
                  if (Ee.length === 0) throw new Error("Can't update empty node!");
                  var xe = new Array(Ee.length), Le = Ee[Ee.length - 1];
                  xe[xe.length - 1] = new le(Le._color, Le.key, we, Le.left, Le.right, Le._count);
                  for (var Qe = Ee.length - 2; Qe >= 0; --Qe) (Le = Ee[Qe]).left === Ee[Qe + 1] ? xe[Qe] = new le(Le._color, Le.key, Le.value, xe[Qe + 1], Le.right, Le._count) : xe[Qe] = new le(Le._color, Le.key, Le.value, Le.left, xe[Qe + 1], Le._count);
                  return new he(this.tree._compare, xe[0]);
                }, be.prev = function() {
                  var we = this._stack;
                  if (we.length !== 0) {
                    var Ee = we[we.length - 1];
                    if (Ee.left) for (Ee = Ee.left; Ee; ) we.push(Ee), Ee = Ee.right;
                    else for (we.pop(); we.length > 0 && we[we.length - 1].left === Ee; ) Ee = we[we.length - 1], we.pop();
                  }
                }, Object.defineProperty(be, "hasPrev", { get: function() {
                  var we = this._stack;
                  if (we.length === 0) return !1;
                  if (we[we.length - 1].left) return !0;
                  for (var Ee = we.length - 1; Ee > 0; --Ee) if (we[Ee - 1].right === we[Ee]) return !0;
                  return !1;
                } });
              }, 3837: function(ce, le, ue) {
                ce.exports = function(Pe, $e) {
                  var qe = new Se(Pe);
                  return qe.update($e), qe;
                };
                var de = ue(4935), fe = ue(501), he = ue(5304), me = ue(6429), pe = ue(6444), ye = new Float32Array([1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1]), ge = ArrayBuffer, ve = DataView;
                function be(Pe) {
                  return Array.isArray(Pe) || function($e) {
                    return ge.isView($e) && !($e instanceof ve);
                  }(Pe);
                }
                function Ce(Pe, $e) {
                  return Pe[0] = $e[0], Pe[1] = $e[1], Pe[2] = $e[2], Pe;
                }
                function Se(Pe) {
                  this.gl = Pe, this.pixelRatio = 1, this.bounds = [[-10, -10, -10], [10, 10, 10]], this.ticks = [[], [], []], this.autoTicks = !0, this.tickSpacing = [1, 1, 1], this.tickEnable = [!0, !0, !0], this.tickFont = ["sans-serif", "sans-serif", "sans-serif"], this.tickFontStyle = ["normal", "normal", "normal"], this.tickFontWeight = ["normal", "normal", "normal"], this.tickFontVariant = ["normal", "normal", "normal"], this.tickSize = [12, 12, 12], this.tickAngle = [0, 0, 0], this.tickAlign = ["auto", "auto", "auto"], this.tickColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.tickPad = [10, 10, 10], this.lastCubeProps = { cubeEdges: [0, 0, 0], axis: [0, 0, 0] }, this.labels = ["x", "y", "z"], this.labelEnable = [!0, !0, !0], this.labelFont = ["sans-serif", "sans-serif", "sans-serif"], this.labelFontStyle = ["normal", "normal", "normal"], this.labelFontWeight = ["normal", "normal", "normal"], this.labelFontVariant = ["normal", "normal", "normal"], this.labelSize = [20, 20, 20], this.labelAngle = [0, 0, 0], this.labelAlign = ["auto", "auto", "auto"], this.labelColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.labelPad = [10, 10, 10], this.lineEnable = [!0, !0, !0], this.lineMirror = [!1, !1, !1], this.lineWidth = [1, 1, 1], this.lineColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.lineTickEnable = [!0, !0, !0], this.lineTickMirror = [!1, !1, !1], this.lineTickLength = [0, 0, 0], this.lineTickWidth = [1, 1, 1], this.lineTickColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.gridEnable = [!0, !0, !0], this.gridWidth = [1, 1, 1], this.gridColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.zeroEnable = [!0, !0, !0], this.zeroLineColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.zeroLineWidth = [2, 2, 2], this.backgroundEnable = [!1, !1, !1], this.backgroundColor = [[0.8, 0.8, 0.8, 0.5], [0.8, 0.8, 0.8, 0.5], [0.8, 0.8, 0.8, 0.5]], this._firstInit = !0, this._text = null, this._lines = null, this._background = he(Pe);
                }
                var we = Se.prototype;
                function Ee() {
                  this.primalOffset = [0, 0, 0], this.primalMinor = [0, 0, 0], this.mirrorOffset = [0, 0, 0], this.mirrorMinor = [0, 0, 0];
                }
                we.update = function(Pe) {
                  function $e(st, gt, bt) {
                    if (bt in Pe) {
                      var yt, St = Pe[bt], Lt = this[bt];
                      (st ? be(St) && be(St[0]) : be(St)) ? this[bt] = yt = [gt(St[0]), gt(St[1]), gt(St[2])] : this[bt] = yt = [gt(St), gt(St), gt(St)];
                      for (var Rt = 0; Rt < 3; ++Rt) if (yt[Rt] !== Lt[Rt]) return !0;
                    }
                    return !1;
                  }
                  Pe = Pe || {};
                  var qe, Xe = $e.bind(this, !1, Number), Ve = $e.bind(this, !1, Boolean), Ze = $e.bind(this, !1, String), Ge = $e.bind(this, !0, function(st) {
                    if (be(st)) {
                      if (st.length === 3) return [+st[0], +st[1], +st[2], 1];
                      if (st.length === 4) return [+st[0], +st[1], +st[2], +st[3]];
                    }
                    return [0, 0, 0, 1];
                  }), ze = !1, We = !1;
                  if ("bounds" in Pe) for (var at = Pe.bounds, ct = 0; ct < 2; ++ct) for (var et = 0; et < 3; ++et) at[ct][et] !== this.bounds[ct][et] && (We = !0), this.bounds[ct][et] = at[ct][et];
                  if ("ticks" in Pe) for (qe = Pe.ticks, ze = !0, this.autoTicks = !1, ct = 0; ct < 3; ++ct) this.tickSpacing[ct] = 0;
                  else Xe("tickSpacing") && (this.autoTicks = !0, We = !0);
                  if (this._firstInit && ("ticks" in Pe || "tickSpacing" in Pe || (this.autoTicks = !0), We = !0, ze = !0, this._firstInit = !1), We && this.autoTicks && (qe = pe.create(this.bounds, this.tickSpacing), ze = !0), ze) {
                    for (ct = 0; ct < 3; ++ct) qe[ct].sort(function(st, gt) {
                      return st.x - gt.x;
                    });
                    pe.equal(qe, this.ticks) ? ze = !1 : this.ticks = qe;
                  }
                  Ve("tickEnable"), Ze("tickFont") && (ze = !0), Ze("tickFontStyle") && (ze = !0), Ze("tickFontWeight") && (ze = !0), Ze("tickFontVariant") && (ze = !0), Xe("tickSize"), Xe("tickAngle"), Xe("tickPad"), Ge("tickColor");
                  var dt = Ze("labels");
                  Ze("labelFont") && (dt = !0), Ze("labelFontStyle") && (dt = !0), Ze("labelFontWeight") && (dt = !0), Ze("labelFontVariant") && (dt = !0), Ve("labelEnable"), Xe("labelSize"), Xe("labelPad"), Ge("labelColor"), Ve("lineEnable"), Ve("lineMirror"), Xe("lineWidth"), Ge("lineColor"), Ve("lineTickEnable"), Ve("lineTickMirror"), Xe("lineTickLength"), Xe("lineTickWidth"), Ge("lineTickColor"), Ve("gridEnable"), Xe("gridWidth"), Ge("gridColor"), Ve("zeroEnable"), Ge("zeroLineColor"), Xe("zeroLineWidth"), Ve("backgroundEnable"), Ge("backgroundColor");
                  var Je = [{ family: this.labelFont[0], style: this.labelFontStyle[0], weight: this.labelFontWeight[0], variant: this.labelFontVariant[0] }, { family: this.labelFont[1], style: this.labelFontStyle[1], weight: this.labelFontWeight[1], variant: this.labelFontVariant[1] }, { family: this.labelFont[2], style: this.labelFontStyle[2], weight: this.labelFontWeight[2], variant: this.labelFontVariant[2] }], it = [{ family: this.tickFont[0], style: this.tickFontStyle[0], weight: this.tickFontWeight[0], variant: this.tickFontVariant[0] }, { family: this.tickFont[1], style: this.tickFontStyle[1], weight: this.tickFontWeight[1], variant: this.tickFontVariant[1] }, { family: this.tickFont[2], style: this.tickFontStyle[2], weight: this.tickFontWeight[2], variant: this.tickFontVariant[2] }];
                  this._text ? this._text && (dt || ze) && this._text.update(this.bounds, this.labels, Je, this.ticks, it) : this._text = de(this.gl, this.bounds, this.labels, Je, this.ticks, it), this._lines && ze && (this._lines.dispose(), this._lines = null), this._lines || (this._lines = fe(this.gl, this.bounds, this.ticks));
                };
                var xe = [new Ee(), new Ee(), new Ee()];
                function Le(Pe, $e, qe, Xe, Ve) {
                  for (var Ze = Pe.primalOffset, Ge = Pe.primalMinor, ze = Pe.mirrorOffset, We = Pe.mirrorMinor, at = Xe[$e], ct = 0; ct < 3; ++ct) if ($e !== ct) {
                    var et = Ze, dt = ze, Je = Ge, it = We;
                    at & 1 << ct && (et = ze, dt = Ze, Je = We, it = Ge), et[ct] = qe[0][ct], dt[ct] = qe[1][ct], Ve[ct] > 0 ? (Je[ct] = -1, it[ct] = 0) : (Je[ct] = 0, it[ct] = 1);
                  }
                }
                var Qe = [0, 0, 0], ke = { model: ye, view: ye, projection: ye, _ortho: !1 };
                we.isOpaque = function() {
                  return !0;
                }, we.isTransparent = function() {
                  return !1;
                }, we.drawTransparent = function(Pe) {
                };
                var Me = [0, 0, 0], Re = [0, 0, 0], De = [0, 0, 0];
                we.draw = function(Pe) {
                  Pe = Pe || ke;
                  for (var $e = this.gl, qe = Pe.model || ye, Xe = Pe.view || ye, Ve = Pe.projection || ye, Ze = this.bounds, Ge = Pe._ortho || !1, ze = me(qe, Xe, Ve, Ze, Ge), We = ze.cubeEdges, at = ze.axis, ct = Xe[12], et = Xe[13], dt = Xe[14], Je = Xe[15], it = (Ge ? 2 : 1) * this.pixelRatio * (Ve[3] * ct + Ve[7] * et + Ve[11] * dt + Ve[15] * Je) / $e.drawingBufferHeight, st = 0; st < 3; ++st) this.lastCubeProps.cubeEdges[st] = We[st], this.lastCubeProps.axis[st] = at[st];
                  var gt = xe;
                  for (st = 0; st < 3; ++st) Le(xe[st], st, this.bounds, We, at);
                  $e = this.gl;
                  var bt, yt, St, Lt = Qe;
                  for (st = 0; st < 3; ++st) this.backgroundEnable[st] ? Lt[st] = at[st] : Lt[st] = 0;
                  for (this._background.draw(qe, Xe, Ve, Ze, Lt, this.backgroundColor), this._lines.bind(qe, Xe, Ve, this), st = 0; st < 3; ++st) {
                    var Rt = [0, 0, 0];
                    at[st] > 0 ? Rt[st] = Ze[1][st] : Rt[st] = Ze[0][st];
                    for (var vt = 0; vt < 2; ++vt) {
                      var $t = (st + 1 + vt) % 3, Pt = (st + 1 + (1 ^ vt)) % 3;
                      this.gridEnable[$t] && this._lines.drawGrid($t, Pt, this.bounds, Rt, this.gridColor[$t], this.gridWidth[$t] * this.pixelRatio);
                    }
                    for (vt = 0; vt < 2; ++vt) $t = (st + 1 + vt) % 3, Pt = (st + 1 + (1 ^ vt)) % 3, this.zeroEnable[Pt] && Math.min(Ze[0][Pt], Ze[1][Pt]) <= 0 && Math.max(Ze[0][Pt], Ze[1][Pt]) >= 0 && this._lines.drawZero($t, Pt, this.bounds, Rt, this.zeroLineColor[Pt], this.zeroLineWidth[Pt] * this.pixelRatio);
                  }
                  for (st = 0; st < 3; ++st) {
                    this.lineEnable[st] && this._lines.drawAxisLine(st, this.bounds, gt[st].primalOffset, this.lineColor[st], this.lineWidth[st] * this.pixelRatio), this.lineMirror[st] && this._lines.drawAxisLine(st, this.bounds, gt[st].mirrorOffset, this.lineColor[st], this.lineWidth[st] * this.pixelRatio);
                    var Xt = Ce(Me, gt[st].primalMinor), Nt = Ce(Re, gt[st].mirrorMinor), Gt = this.lineTickLength;
                    for (vt = 0; vt < 3; ++vt) {
                      var Ft = it / qe[5 * vt];
                      Xt[vt] *= Gt[vt] * Ft, Nt[vt] *= Gt[vt] * Ft;
                    }
                    this.lineTickEnable[st] && this._lines.drawAxisTicks(st, gt[st].primalOffset, Xt, this.lineTickColor[st], this.lineTickWidth[st] * this.pixelRatio), this.lineTickMirror[st] && this._lines.drawAxisTicks(st, gt[st].mirrorOffset, Nt, this.lineTickColor[st], this.lineTickWidth[st] * this.pixelRatio);
                  }
                  function Tt(sn) {
                    (St = [0, 0, 0])[sn] = 1;
                  }
                  function It(sn, hn, Qt) {
                    var Ut = (sn + 1) % 3, Wt = (sn + 2) % 3, rn = hn[Ut], vn = hn[Wt], dn = Qt[Ut], Ht = Qt[Wt];
                    rn > 0 && Ht > 0 || rn > 0 && Ht < 0 || rn < 0 && Ht > 0 || rn < 0 && Ht < 0 ? Tt(Ut) : (vn > 0 && dn > 0 || vn > 0 && dn < 0 || vn < 0 && dn > 0 || vn < 0 && dn < 0) && Tt(Wt);
                  }
                  for (this._lines.unbind(), this._text.bind(qe, Xe, Ve, this.pixelRatio), st = 0; st < 3; ++st) {
                    var Ct = gt[st].primalMinor, Kt = gt[st].mirrorMinor, Vt = Ce(De, gt[st].primalOffset);
                    for (vt = 0; vt < 3; ++vt) this.lineTickEnable[st] && (Vt[vt] += it * Ct[vt] * Math.max(this.lineTickLength[vt], 0) / qe[5 * vt]);
                    var Yt = [0, 0, 0];
                    if (Yt[st] = 1, this.tickEnable[st]) {
                      for (this.tickAngle[st] === -3600 ? (this.tickAngle[st] = 0, this.tickAlign[st] = "auto") : this.tickAlign[st] = -1, yt = 1, (bt = [this.tickAlign[st], 0.5, yt])[0] === "auto" ? bt[0] = 0 : bt[0] = parseInt("" + bt[0]), St = [0, 0, 0], It(st, Ct, Kt), vt = 0; vt < 3; ++vt) Vt[vt] += it * Ct[vt] * this.tickPad[vt] / qe[5 * vt];
                      this._text.drawTicks(st, this.tickSize[st], this.tickAngle[st], Vt, this.tickColor[st], Yt, St, bt);
                    }
                    if (this.labelEnable[st]) {
                      for (yt = 0, St = [0, 0, 0], this.labels[st].length > 4 && (Tt(st), yt = 1), (bt = [this.labelAlign[st], 0.5, yt])[0] === "auto" ? bt[0] = 0 : bt[0] = parseInt("" + bt[0]), vt = 0; vt < 3; ++vt) Vt[vt] += it * Ct[vt] * this.labelPad[vt] / qe[5 * vt];
                      Vt[st] += 0.5 * (Ze[0][st] + Ze[1][st]), this._text.drawLabel(st, this.labelSize[st], this.labelAngle[st], Vt, this.labelColor[st], [0, 0, 0], St, bt);
                    }
                  }
                  this._text.unbind();
                }, we.dispose = function() {
                  this._text.dispose(), this._lines.dispose(), this._background.dispose(), this._lines = null, this._text = null, this._background = null, this.gl = null;
                };
              }, 5304: function(ce, le, ue) {
                ce.exports = function(ye) {
                  for (var ge = [], ve = [], be = 0, Ce = 0; Ce < 3; ++Ce) for (var Se = (Ce + 1) % 3, we = (Ce + 2) % 3, Ee = [0, 0, 0], xe = [0, 0, 0], Le = -1; Le <= 1; Le += 2) {
                    ve.push(be, be + 2, be + 1, be + 1, be + 2, be + 3), Ee[Ce] = Le, xe[Ce] = Le;
                    for (var Qe = -1; Qe <= 1; Qe += 2) {
                      Ee[Se] = Qe;
                      for (var ke = -1; ke <= 1; ke += 2) Ee[we] = ke, ge.push(Ee[0], Ee[1], Ee[2], xe[0], xe[1], xe[2]), be += 1;
                    }
                    var Me = Se;
                    Se = we, we = Me;
                  }
                  var Re = de(ye, new Float32Array(ge)), De = de(ye, new Uint16Array(ve), ye.ELEMENT_ARRAY_BUFFER), Pe = fe(ye, [{ buffer: Re, type: ye.FLOAT, size: 3, offset: 0, stride: 24 }, { buffer: Re, type: ye.FLOAT, size: 3, offset: 12, stride: 24 }], De), $e = he(ye);
                  return $e.attributes.position.location = 0, $e.attributes.normal.location = 1, new me(ye, Re, Pe, $e);
                };
                var de = ue(2762), fe = ue(8116), he = ue(1879).bg;
                function me(ye, ge, ve, be) {
                  this.gl = ye, this.buffer = ge, this.vao = ve, this.shader = be;
                }
                var pe = me.prototype;
                pe.draw = function(ye, ge, ve, be, Ce, Se) {
                  for (var we = !1, Ee = 0; Ee < 3; ++Ee) we = we || Ce[Ee];
                  if (we) {
                    var xe = this.gl;
                    xe.enable(xe.POLYGON_OFFSET_FILL), xe.polygonOffset(1, 2), this.shader.bind(), this.shader.uniforms = { model: ye, view: ge, projection: ve, bounds: be, enable: Ce, colors: Se }, this.vao.bind(), this.vao.draw(this.gl.TRIANGLES, 36), this.vao.unbind(), xe.disable(xe.POLYGON_OFFSET_FILL);
                  }
                }, pe.dispose = function() {
                  this.vao.dispose(), this.buffer.dispose(), this.shader.dispose();
                };
              }, 6429: function(ce, le, ue) {
                ce.exports = function(Qe, ke, Me, Re, De) {
                  fe(pe, ke, Qe), fe(pe, Me, pe);
                  for (var Pe = 0, $e = 0; $e < 2; ++$e) {
                    ve[2] = Re[$e][2];
                    for (var qe = 0; qe < 2; ++qe) {
                      ve[1] = Re[qe][1];
                      for (var Xe = 0; Xe < 2; ++Xe) ve[0] = Re[Xe][0], Ce(ye[Pe], ve, pe), Pe += 1;
                    }
                  }
                  var Ve = -1;
                  for ($e = 0; $e < 8; ++$e) {
                    for (var Ze = ye[$e][3], Ge = 0; Ge < 3; ++Ge) ge[$e][Ge] = ye[$e][Ge] / Ze;
                    De && (ge[$e][2] *= -1), Ze < 0 && (Ve < 0 || ge[$e][2] < ge[Ve][2]) && (Ve = $e);
                  }
                  if (Ve < 0) {
                    Ve = 0;
                    for (var ze = 0; ze < 3; ++ze) {
                      for (var We = (ze + 2) % 3, at = (ze + 1) % 3, ct = -1, et = -1, dt = 0; dt < 2; ++dt) {
                        var Je = (st = dt << ze) + (dt << We) + (1 - dt << at), it = st + (1 - dt << We) + (dt << at);
                        me(ge[st], ge[Je], ge[it], be) < 0 || (dt ? ct = 1 : et = 1);
                      }
                      if (ct < 0 || et < 0) et > ct && (Ve |= 1 << ze);
                      else {
                        for (dt = 0; dt < 2; ++dt) {
                          Je = (st = dt << ze) + (dt << We) + (1 - dt << at), it = st + (1 - dt << We) + (dt << at);
                          var st, gt = we([ye[st], ye[Je], ye[it], ye[st + (1 << We) + (1 << at)]]);
                          dt ? ct = gt : et = gt;
                        }
                        et > ct && (Ve |= 1 << ze);
                      }
                    }
                  }
                  var bt = 7 ^ Ve, yt = -1;
                  for ($e = 0; $e < 8; ++$e) $e !== Ve && $e !== bt && (yt < 0 || ge[yt][1] > ge[$e][1]) && (yt = $e);
                  var St = -1;
                  for ($e = 0; $e < 3; ++$e) (Rt = yt ^ 1 << $e) !== Ve && Rt !== bt && (St < 0 && (St = Rt), (at = ge[Rt])[0] < ge[St][0] && (St = Rt));
                  var Lt = -1;
                  for ($e = 0; $e < 3; ++$e) {
                    var Rt;
                    (Rt = yt ^ 1 << $e) !== Ve && Rt !== bt && Rt !== St && (Lt < 0 && (Lt = Rt), (at = ge[Rt])[0] > ge[Lt][0] && (Lt = Rt));
                  }
                  var vt = Ee;
                  vt[0] = vt[1] = vt[2] = 0, vt[de.log2(St ^ yt)] = yt & St, vt[de.log2(yt ^ Lt)] = yt & Lt;
                  var $t = 7 ^ Lt;
                  $t === Ve || $t === bt ? ($t = 7 ^ St, vt[de.log2(Lt ^ $t)] = $t & Lt) : vt[de.log2(St ^ $t)] = $t & St;
                  var Pt = xe, Xt = Ve;
                  for (ze = 0; ze < 3; ++ze) Pt[ze] = Xt & 1 << ze ? -1 : 1;
                  return Le;
                };
                var de = ue(8828), fe = ue(6760), he = ue(5202), me = ue(3250), pe = new Array(16), ye = new Array(8), ge = new Array(8), ve = new Array(3), be = [0, 0, 0];
                function Ce(Qe, ke, Me) {
                  for (var Re = 0; Re < 4; ++Re) {
                    Qe[Re] = Me[12 + Re];
                    for (var De = 0; De < 3; ++De) Qe[Re] += ke[De] * Me[4 * De + Re];
                  }
                }
                (function() {
                  for (var Qe = 0; Qe < 8; ++Qe) ye[Qe] = [1, 1, 1, 1], ge[Qe] = [1, 1, 1];
                })();
                var Se = [[0, 0, 1, 0, 0], [0, 0, -1, 1, 0], [0, -1, 0, 1, 0], [0, 1, 0, 1, 0], [-1, 0, 0, 1, 0], [1, 0, 0, 1, 0]];
                function we(Qe) {
                  for (var ke = 0; ke < Se.length; ++ke) if ((Qe = he.positive(Qe, Se[ke])).length < 3) return 0;
                  var Me = Qe[0], Re = Me[0] / Me[3], De = Me[1] / Me[3], Pe = 0;
                  for (ke = 1; ke + 1 < Qe.length; ++ke) {
                    var $e = Qe[ke], qe = Qe[ke + 1], Xe = $e[0] / $e[3] - Re, Ve = $e[1] / $e[3] - De, Ze = qe[0] / qe[3] - Re, Ge = qe[1] / qe[3] - De;
                    Pe += Math.abs(Xe * Ge - Ve * Ze);
                  }
                  return Pe;
                }
                var Ee = [1, 1, 1], xe = [0, 0, 0], Le = { cubeEdges: Ee, axis: xe };
              }, 501: function(ce, le, ue) {
                ce.exports = function(Ee, xe, Le) {
                  var Qe = [], ke = [0, 0, 0], Me = [0, 0, 0], Re = [0, 0, 0], De = [0, 0, 0];
                  Qe.push(0, 0, 1, 0, 1, 1, 0, 0, -1, 0, 0, -1, 0, 1, 1, 0, 1, -1);
                  for (var Pe = 0; Pe < 3; ++Pe) {
                    for (var $e = Qe.length / 3 | 0, qe = 0; qe < Le[Pe].length; ++qe) {
                      var Xe = +Le[Pe][qe].x;
                      Qe.push(Xe, 0, 1, Xe, 1, 1, Xe, 0, -1, Xe, 0, -1, Xe, 1, 1, Xe, 1, -1);
                    }
                    var Ve = Qe.length / 3 | 0;
                    ke[Pe] = $e, Me[Pe] = Ve - $e, $e = Qe.length / 3 | 0;
                    for (var Ze = 0; Ze < Le[Pe].length; ++Ze) Xe = +Le[Pe][Ze].x, Qe.push(Xe, 0, 1, Xe, 1, 1, Xe, 0, -1, Xe, 0, -1, Xe, 1, 1, Xe, 1, -1);
                    Ve = Qe.length / 3 | 0, Re[Pe] = $e, De[Pe] = Ve - $e;
                  }
                  var Ge = de(Ee, new Float32Array(Qe)), ze = fe(Ee, [{ buffer: Ge, type: Ee.FLOAT, size: 3, stride: 0, offset: 0 }]), We = he(Ee);
                  return We.attributes.position.location = 0, new Se(Ee, Ge, ze, We, Me, ke, De, Re);
                };
                var de = ue(2762), fe = ue(8116), he = ue(1879).n, me = [0, 0, 0], pe = [0, 0, 0], ye = [0, 0, 0], ge = [0, 0, 0], ve = [1, 1];
                function be(Ee) {
                  return Ee[0] = Ee[1] = Ee[2] = 0, Ee;
                }
                function Ce(Ee, xe) {
                  return Ee[0] = xe[0], Ee[1] = xe[1], Ee[2] = xe[2], Ee;
                }
                function Se(Ee, xe, Le, Qe, ke, Me, Re, De) {
                  this.gl = Ee, this.vertBuffer = xe, this.vao = Le, this.shader = Qe, this.tickCount = ke, this.tickOffset = Me, this.gridCount = Re, this.gridOffset = De;
                }
                var we = Se.prototype;
                we.bind = function(Ee, xe, Le) {
                  this.shader.bind(), this.shader.uniforms.model = Ee, this.shader.uniforms.view = xe, this.shader.uniforms.projection = Le, ve[0] = this.gl.drawingBufferWidth, ve[1] = this.gl.drawingBufferHeight, this.shader.uniforms.screenShape = ve, this.vao.bind();
                }, we.unbind = function() {
                  this.vao.unbind();
                }, we.drawAxisLine = function(Ee, xe, Le, Qe, ke) {
                  var Me = be(pe);
                  this.shader.uniforms.majorAxis = pe, Me[Ee] = xe[1][Ee] - xe[0][Ee], this.shader.uniforms.minorAxis = Me;
                  var Re, De = Ce(ge, Le);
                  De[Ee] += xe[0][Ee], this.shader.uniforms.offset = De, this.shader.uniforms.lineWidth = ke, this.shader.uniforms.color = Qe, (Re = be(ye))[(Ee + 2) % 3] = 1, this.shader.uniforms.screenAxis = Re, this.vao.draw(this.gl.TRIANGLES, 6), (Re = be(ye))[(Ee + 1) % 3] = 1, this.shader.uniforms.screenAxis = Re, this.vao.draw(this.gl.TRIANGLES, 6);
                }, we.drawAxisTicks = function(Ee, xe, Le, Qe, ke) {
                  if (this.tickCount[Ee]) {
                    var Me = be(me);
                    Me[Ee] = 1, this.shader.uniforms.majorAxis = Me, this.shader.uniforms.offset = xe, this.shader.uniforms.minorAxis = Le, this.shader.uniforms.color = Qe, this.shader.uniforms.lineWidth = ke;
                    var Re = be(ye);
                    Re[Ee] = 1, this.shader.uniforms.screenAxis = Re, this.vao.draw(this.gl.TRIANGLES, this.tickCount[Ee], this.tickOffset[Ee]);
                  }
                }, we.drawGrid = function(Ee, xe, Le, Qe, ke, Me) {
                  if (this.gridCount[Ee]) {
                    var Re = be(pe);
                    Re[xe] = Le[1][xe] - Le[0][xe], this.shader.uniforms.minorAxis = Re;
                    var De = Ce(ge, Qe);
                    De[xe] += Le[0][xe], this.shader.uniforms.offset = De;
                    var Pe = be(me);
                    Pe[Ee] = 1, this.shader.uniforms.majorAxis = Pe;
                    var $e = be(ye);
                    $e[Ee] = 1, this.shader.uniforms.screenAxis = $e, this.shader.uniforms.lineWidth = Me, this.shader.uniforms.color = ke, this.vao.draw(this.gl.TRIANGLES, this.gridCount[Ee], this.gridOffset[Ee]);
                  }
                }, we.drawZero = function(Ee, xe, Le, Qe, ke, Me) {
                  var Re = be(pe);
                  this.shader.uniforms.majorAxis = Re, Re[Ee] = Le[1][Ee] - Le[0][Ee], this.shader.uniforms.minorAxis = Re;
                  var De = Ce(ge, Qe);
                  De[Ee] += Le[0][Ee], this.shader.uniforms.offset = De;
                  var Pe = be(ye);
                  Pe[xe] = 1, this.shader.uniforms.screenAxis = Pe, this.shader.uniforms.lineWidth = Me, this.shader.uniforms.color = ke, this.vao.draw(this.gl.TRIANGLES, 6);
                }, we.dispose = function() {
                  this.vao.dispose(), this.vertBuffer.dispose(), this.shader.dispose();
                };
              }, 1879: function(ce, le, ue) {
                var de = ue(3236), fe = ue(9405), he = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position;

uniform mat4 model, view, projection;
uniform vec3 offset, majorAxis, minorAxis, screenAxis;
uniform float lineWidth;
uniform vec2 screenShape;

vec3 project(vec3 p) {
  vec4 pp = projection * (view * (model * vec4(p, 1.0)));
  return pp.xyz / max(pp.w, 0.0001);
}

void main() {
  vec3 major = position.x * majorAxis;
  vec3 minor = position.y * minorAxis;

  vec3 vPosition = major + minor + offset;
  vec3 pPosition = project(vPosition);
  vec3 offset = project(vPosition + screenAxis * position.z);

  vec2 screen = normalize((offset - pPosition).xy * screenShape) / screenShape;

  gl_Position = vec4(pPosition + vec3(0.5 * screen * lineWidth, 0), 1.0);
}
`]), me = de([`precision highp float;
#define GLSLIFY 1

uniform vec4 color;
void main() {
  gl_FragColor = color;
}`]);
                le.n = function(be) {
                  return fe(be, he, me, null, [{ name: "position", type: "vec3" }]);
                };
                var pe = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position;

uniform mat4 model, view, projection;
uniform vec3 offset, axis, alignDir, alignOpt;
uniform float scale, angle, pixelScale;
uniform vec2 resolution;

vec3 project(vec3 p) {
  vec4 pp = projection * (view * (model * vec4(p, 1.0)));
  return pp.xyz / max(pp.w, 0.0001);
}

float computeViewAngle(vec3 a, vec3 b) {
  vec3 A = project(a);
  vec3 B = project(b);

  return atan(
    (B.y - A.y) * resolution.y,
    (B.x - A.x) * resolution.x
  );
}

const float PI = 3.141592;
const float TWO_PI = 2.0 * PI;
const float HALF_PI = 0.5 * PI;
const float ONE_AND_HALF_PI = 1.5 * PI;

int option = int(floor(alignOpt.x + 0.001));
float hv_ratio =       alignOpt.y;
bool enableAlign =    (alignOpt.z != 0.0);

float mod_angle(float a) {
  return mod(a, PI);
}

float positive_angle(float a) {
  return mod_angle((a < 0.0) ?
    a + TWO_PI :
    a
  );
}

float look_upwards(float a) {
  float b = positive_angle(a);
  return ((b > HALF_PI) && (b <= ONE_AND_HALF_PI)) ?
    b - PI :
    b;
}

float look_horizontal_or_vertical(float a, float ratio) {
  // ratio controls the ratio between being horizontal to (vertical + horizontal)
  // if ratio is set to 0.5 then it is 50%, 50%.
  // when using a higher ratio e.g. 0.75 the result would
  // likely be more horizontal than vertical.

  float b = positive_angle(a);

  return
    (b < (      ratio) * HALF_PI) ? 0.0 :
    (b < (2.0 - ratio) * HALF_PI) ? -HALF_PI :
    (b < (2.0 + ratio) * HALF_PI) ? 0.0 :
    (b < (4.0 - ratio) * HALF_PI) ? HALF_PI :
                                    0.0;
}

float roundTo(float a, float b) {
  return float(b * floor((a + 0.5 * b) / b));
}

float look_round_n_directions(float a, int n) {
  float b = positive_angle(a);
  float div = TWO_PI / float(n);
  float c = roundTo(b, div);
  return look_upwards(c);
}

float applyAlignOption(float rawAngle, float delta) {
  return
    (option >  2) ? look_round_n_directions(rawAngle + delta, option) :       // option 3-n: round to n directions
    (option == 2) ? look_horizontal_or_vertical(rawAngle + delta, hv_ratio) : // horizontal or vertical
    (option == 1) ? rawAngle + delta :       // use free angle, and flip to align with one direction of the axis
    (option == 0) ? look_upwards(rawAngle) : // use free angle, and stay upwards
    (option ==-1) ? 0.0 :                    // useful for backward compatibility, all texts remains horizontal
                    rawAngle;                // otherwise return back raw input angle
}

bool isAxisTitle = (axis.x == 0.0) &&
                   (axis.y == 0.0) &&
                   (axis.z == 0.0);

void main() {
  //Compute world offset
  float axisDistance = position.z;
  vec3 dataPosition = axisDistance * axis + offset;

  float beta = angle; // i.e. user defined attributes for each tick

  float axisAngle;
  float clipAngle;
  float flip;

  if (enableAlign) {
    axisAngle = (isAxisTitle) ? HALF_PI :
                      computeViewAngle(dataPosition, dataPosition + axis);
    clipAngle = computeViewAngle(dataPosition, dataPosition + alignDir);

    axisAngle += (sin(axisAngle) < 0.0) ? PI : 0.0;
    clipAngle += (sin(clipAngle) < 0.0) ? PI : 0.0;

    flip = (dot(vec2(cos(axisAngle), sin(axisAngle)),
                vec2(sin(clipAngle),-cos(clipAngle))) > 0.0) ? 1.0 : 0.0;

    beta += applyAlignOption(clipAngle, flip * PI);
  }

  //Compute plane offset
  vec2 planeCoord = position.xy * pixelScale;

  mat2 planeXform = scale * mat2(
     cos(beta), sin(beta),
    -sin(beta), cos(beta)
  );

  vec2 viewOffset = 2.0 * planeXform * planeCoord / resolution;

  //Compute clip position
  vec3 clipPosition = project(dataPosition);

  //Apply text offset in clip coordinates
  clipPosition += vec3(viewOffset, 0.0);

  //Done
  gl_Position = vec4(clipPosition, 1.0);
}
`]), ye = de([`precision highp float;
#define GLSLIFY 1

uniform vec4 color;
void main() {
  gl_FragColor = color;
}`]);
                le.Q = function(be) {
                  return fe(be, pe, ye, null, [{ name: "position", type: "vec3" }]);
                };
                var ge = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position;
attribute vec3 normal;

uniform mat4 model, view, projection;
uniform vec3 enable;
uniform vec3 bounds[2];

varying vec3 colorChannel;

void main() {

  vec3 signAxis = sign(bounds[1] - bounds[0]);

  vec3 realNormal = signAxis * normal;

  if(dot(realNormal, enable) > 0.0) {
    vec3 minRange = min(bounds[0], bounds[1]);
    vec3 maxRange = max(bounds[0], bounds[1]);
    vec3 nPosition = mix(minRange, maxRange, 0.5 * (position + 1.0));
    gl_Position = projection * (view * (model * vec4(nPosition, 1.0)));
  } else {
    gl_Position = vec4(0,0,0,0);
  }

  colorChannel = abs(realNormal);
}
`]), ve = de([`precision highp float;
#define GLSLIFY 1

uniform vec4 colors[3];

varying vec3 colorChannel;

void main() {
  gl_FragColor = colorChannel.x * colors[0] +
                 colorChannel.y * colors[1] +
                 colorChannel.z * colors[2];
}`]);
                le.bg = function(be) {
                  return fe(be, ge, ve, null, [{ name: "position", type: "vec3" }, { name: "normal", type: "vec3" }]);
                };
              }, 4935: function(ce, le, ue) {
                ce.exports = function(Ce, Se, we, Ee, xe, Le) {
                  var Qe = de(Ce), ke = fe(Ce, [{ buffer: Qe, size: 3 }]), Me = me(Ce);
                  Me.attributes.position.location = 0;
                  var Re = new ge(Ce, Me, Qe, ke);
                  return Re.update(Se, we, Ee, xe, Le), Re;
                };
                var de = ue(2762), fe = ue(8116), he = ue(4359), me = ue(1879).Q, pe = window || ne.global || {}, ye = pe.__TEXT_CACHE || {};
                function ge(Ce, Se, we, Ee) {
                  this.gl = Ce, this.shader = Se, this.buffer = we, this.vao = Ee, this.tickOffset = this.tickCount = this.labelOffset = this.labelCount = null;
                }
                pe.__TEXT_CACHE = {};
                var ve = ge.prototype, be = [0, 0];
                ve.bind = function(Ce, Se, we, Ee) {
                  this.vao.bind(), this.shader.bind();
                  var xe = this.shader.uniforms;
                  xe.model = Ce, xe.view = Se, xe.projection = we, xe.pixelScale = Ee, be[0] = this.gl.drawingBufferWidth, be[1] = this.gl.drawingBufferHeight, this.shader.uniforms.resolution = be;
                }, ve.unbind = function() {
                  this.vao.unbind();
                }, ve.update = function(Ce, Se, we, Ee, xe) {
                  var Le = [];
                  function Qe(Ve, Ze, Ge, ze, We, at) {
                    var ct = [Ge.style, Ge.weight, Ge.variant, Ge.family].join("_"), et = ye[ct];
                    et || (et = ye[ct] = {});
                    var dt = et[Ze];
                    dt || (dt = et[Ze] = function(Rt, vt) {
                      try {
                        return he(Rt, vt);
                      } catch ($t) {
                        return console.warn('error vectorizing text:"' + Rt + '" error:', $t), { cells: [], positions: [] };
                      }
                    }(Ze, { triangles: !0, font: Ge.family, fontStyle: Ge.style, fontWeight: Ge.weight, fontVariant: Ge.variant, textAlign: "center", textBaseline: "middle", lineSpacing: We, styletags: at }));
                    for (var Je = (ze || 12) / 12, it = dt.positions, st = dt.cells, gt = 0, bt = st.length; gt < bt; ++gt) for (var yt = st[gt], St = 2; St >= 0; --St) {
                      var Lt = it[yt[St]];
                      Le.push(Je * Lt[0], -Je * Lt[1], Ve);
                    }
                  }
                  for (var ke = [0, 0, 0], Me = [0, 0, 0], Re = [0, 0, 0], De = [0, 0, 0], Pe = { breaklines: !0, bolds: !0, italics: !0, subscripts: !0, superscripts: !0 }, $e = 0; $e < 3; ++$e) {
                    Re[$e] = Le.length / 3 | 0, Qe(0.5 * (Ce[0][$e] + Ce[1][$e]), Se[$e], we[$e], 12, 1.25, Pe), De[$e] = (Le.length / 3 | 0) - Re[$e], ke[$e] = Le.length / 3 | 0;
                    for (var qe = 0; qe < Ee[$e].length; ++qe) if (Ee[$e][qe].text) {
                      var Xe = { family: Ee[$e][qe].font || xe[$e].family, style: xe[$e].fontStyle || xe[$e].style, weight: xe[$e].fontWeight || xe[$e].weight, variant: xe[$e].fontVariant || xe[$e].variant };
                      Qe(Ee[$e][qe].x, Ee[$e][qe].text, Xe, Ee[$e][qe].fontSize || 12, 1.25, Pe);
                    }
                    Me[$e] = (Le.length / 3 | 0) - ke[$e];
                  }
                  this.buffer.update(Le), this.tickOffset = ke, this.tickCount = Me, this.labelOffset = Re, this.labelCount = De;
                }, ve.drawTicks = function(Ce, Se, we, Ee, xe, Le, Qe, ke) {
                  this.tickCount[Ce] && (this.shader.uniforms.axis = Le, this.shader.uniforms.color = xe, this.shader.uniforms.angle = we, this.shader.uniforms.scale = Se, this.shader.uniforms.offset = Ee, this.shader.uniforms.alignDir = Qe, this.shader.uniforms.alignOpt = ke, this.vao.draw(this.gl.TRIANGLES, this.tickCount[Ce], this.tickOffset[Ce]));
                }, ve.drawLabel = function(Ce, Se, we, Ee, xe, Le, Qe, ke) {
                  this.labelCount[Ce] && (this.shader.uniforms.axis = Le, this.shader.uniforms.color = xe, this.shader.uniforms.angle = we, this.shader.uniforms.scale = Se, this.shader.uniforms.offset = Ee, this.shader.uniforms.alignDir = Qe, this.shader.uniforms.alignOpt = ke, this.vao.draw(this.gl.TRIANGLES, this.labelCount[Ce], this.labelOffset[Ce]));
                }, ve.dispose = function() {
                  this.shader.dispose(), this.vao.dispose(), this.buffer.dispose();
                };
              }, 6444: function(ce, le) {
                function ue(de, fe) {
                  var he = de + "", me = he.indexOf("."), pe = 0;
                  me >= 0 && (pe = he.length - me - 1);
                  var ye = Math.pow(10, pe), ge = Math.round(de * fe * ye), ve = ge + "";
                  if (ve.indexOf("e") >= 0) return ve;
                  var be = ge / ye, Ce = ge % ye;
                  ge < 0 ? (be = 0 | -Math.ceil(be), Ce = 0 | -Ce) : (be = 0 | Math.floor(be), Ce |= 0);
                  var Se = "" + be;
                  if (ge < 0 && (Se = "-" + Se), pe) {
                    for (var we = "" + Ce; we.length < pe; ) we = "0" + we;
                    return Se + "." + we;
                  }
                  return Se;
                }
                le.create = function(de, fe) {
                  for (var he = [], me = 0; me < 3; ++me) {
                    for (var pe = [], ye = (de[0][me], de[1][me], 0); ye * fe[me] <= de[1][me]; ++ye) pe.push({ x: ye * fe[me], text: ue(fe[me], ye) });
                    for (ye = -1; ye * fe[me] >= de[0][me]; --ye) pe.push({ x: ye * fe[me], text: ue(fe[me], ye) });
                    he.push(pe);
                  }
                  return he;
                }, le.equal = function(de, fe) {
                  for (var he = 0; he < 3; ++he) {
                    if (de[he].length !== fe[he].length) return !1;
                    for (var me = 0; me < de[he].length; ++me) {
                      var pe = de[he][me], ye = fe[he][me];
                      if (pe.x !== ye.x || pe.text !== ye.text || pe.font !== ye.font || pe.fontColor !== ye.fontColor || pe.fontSize !== ye.fontSize || pe.dx !== ye.dx || pe.dy !== ye.dy) return !1;
                    }
                  }
                  return !0;
                };
              }, 5445: function(ce, le, ue) {
                ce.exports = function(Le, Qe, ke, Me, Re) {
                  var De = Qe.model || ge, Pe = Qe.view || ge, $e = Qe.projection || ge, qe = Qe._ortho || !1, Xe = Le.bounds, Ve = (Re = Re || he(De, Pe, $e, Xe, qe)).axis;
                  me(ve, Pe, De), me(ve, $e, ve);
                  for (var Ze = Ee, Ge = 0; Ge < 3; ++Ge) Ze[Ge].lo = 1 / 0, Ze[Ge].hi = -1 / 0, Ze[Ge].pixelsPerDataUnit = 1 / 0;
                  var ze = de(pe(ve, ve));
                  pe(ve, ve);
                  for (var We = 0; We < 3; ++We) {
                    var at = (We + 1) % 3, ct = (We + 2) % 3, et = xe;
                    e: for (Ge = 0; Ge < 2; ++Ge) {
                      var dt = [];
                      if (Ve[We] < 0 != !!Ge) {
                        et[We] = Xe[Ge][We];
                        for (var Je = 0; Je < 2; ++Je) {
                          et[at] = Xe[Je ^ Ge][at];
                          for (var it = 0; it < 2; ++it) et[ct] = Xe[it ^ Je ^ Ge][ct], dt.push(et.slice());
                        }
                        var st = qe ? 5 : 4;
                        for (Je = st; Je === st; ++Je) {
                          if (dt.length === 0) continue e;
                          dt = fe.positive(dt, ze[Je]);
                        }
                        for (Je = 0; Je < dt.length; ++Je) {
                          ct = dt[Je];
                          var gt = we(xe, ve, ct, ke, Me);
                          for (it = 0; it < 3; ++it) Ze[it].lo = Math.min(Ze[it].lo, ct[it]), Ze[it].hi = Math.max(Ze[it].hi, ct[it]), it !== We && (Ze[it].pixelsPerDataUnit = Math.min(Ze[it].pixelsPerDataUnit, Math.abs(gt[it])));
                        }
                      }
                    }
                  }
                  return Ze;
                };
                var de = ue(5033), fe = ue(5202), he = ue(6429), me = ue(6760), pe = ue(5665), ye = ue(5352), ge = new Float32Array([1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1]), ve = new Float32Array(16);
                function be(Le, Qe, ke) {
                  this.lo = Le, this.hi = Qe, this.pixelsPerDataUnit = ke;
                }
                var Ce = [0, 0, 0, 1], Se = [0, 0, 0, 1];
                function we(Le, Qe, ke, Me, Re) {
                  for (var De = 0; De < 3; ++De) {
                    for (var Pe = Ce, $e = Se, qe = 0; qe < 3; ++qe) $e[qe] = Pe[qe] = ke[qe];
                    $e[3] = Pe[3] = 1, $e[De] += 1, ye($e, $e, Qe), $e[3] < 0 && (Le[De] = 1 / 0), Pe[De] -= 1, ye(Pe, Pe, Qe), Pe[3] < 0 && (Le[De] = 1 / 0);
                    var Xe = (Pe[0] / Pe[3] - $e[0] / $e[3]) * Me, Ve = (Pe[1] / Pe[3] - $e[1] / $e[3]) * Re;
                    Le[De] = 0.25 * Math.sqrt(Xe * Xe + Ve * Ve);
                  }
                  return Le;
                }
                var Ee = [new be(1 / 0, -1 / 0, 1 / 0), new be(1 / 0, -1 / 0, 1 / 0), new be(1 / 0, -1 / 0, 1 / 0)], xe = [0, 0, 0];
              }, 2762: function(ce, le, ue) {
                var de = ue(1888), fe = ue(5298), he = ue(9618), me = ["uint8", "uint8_clamped", "uint16", "uint32", "int8", "int16", "int32", "float32"];
                function pe(be, Ce, Se, we, Ee) {
                  this.gl = be, this.type = Ce, this.handle = Se, this.length = we, this.usage = Ee;
                }
                var ye = pe.prototype;
                function ge(be, Ce, Se, we, Ee, xe) {
                  var Le = Ee.length * Ee.BYTES_PER_ELEMENT;
                  if (xe < 0) return be.bufferData(Ce, Ee, we), Le;
                  if (Le + xe > Se) throw new Error("gl-buffer: If resizing buffer, must not specify offset");
                  return be.bufferSubData(Ce, xe, Ee), Se;
                }
                function ve(be, Ce) {
                  for (var Se = de.malloc(be.length, Ce), we = be.length, Ee = 0; Ee < we; ++Ee) Se[Ee] = be[Ee];
                  return Se;
                }
                ye.bind = function() {
                  this.gl.bindBuffer(this.type, this.handle);
                }, ye.unbind = function() {
                  this.gl.bindBuffer(this.type, null);
                }, ye.dispose = function() {
                  this.gl.deleteBuffer(this.handle);
                }, ye.update = function(be, Ce) {
                  if (typeof Ce != "number" && (Ce = -1), this.bind(), typeof be == "object" && be.shape !== void 0) {
                    var Se = be.dtype;
                    if (me.indexOf(Se) < 0 && (Se = "float32"), this.type === this.gl.ELEMENT_ARRAY_BUFFER && (Se = gl.getExtension("OES_element_index_uint") && Se !== "uint16" ? "uint32" : "uint16"), Se === be.dtype && function(Le, Qe) {
                      for (var ke = 1, Me = Qe.length - 1; Me >= 0; --Me) {
                        if (Qe[Me] !== ke) return !1;
                        ke *= Le[Me];
                      }
                      return !0;
                    }(be.shape, be.stride)) be.offset === 0 && be.data.length === be.shape[0] ? this.length = ge(this.gl, this.type, this.length, this.usage, be.data, Ce) : this.length = ge(this.gl, this.type, this.length, this.usage, be.data.subarray(be.offset, be.shape[0]), Ce);
                    else {
                      var we = de.malloc(be.size, Se), Ee = he(we, be.shape);
                      fe.assign(Ee, be), this.length = ge(this.gl, this.type, this.length, this.usage, Ce < 0 ? we : we.subarray(0, be.size), Ce), de.free(we);
                    }
                  } else if (Array.isArray(be)) {
                    var xe;
                    xe = this.type === this.gl.ELEMENT_ARRAY_BUFFER ? ve(be, "uint16") : ve(be, "float32"), this.length = ge(this.gl, this.type, this.length, this.usage, Ce < 0 ? xe : xe.subarray(0, be.length), Ce), de.free(xe);
                  } else if (typeof be == "object" && typeof be.length == "number") this.length = ge(this.gl, this.type, this.length, this.usage, be, Ce);
                  else {
                    if (typeof be != "number" && be !== void 0) throw new Error("gl-buffer: Invalid data type");
                    if (Ce >= 0) throw new Error("gl-buffer: Cannot specify offset when resizing buffer");
                    (be |= 0) <= 0 && (be = 1), this.gl.bufferData(this.type, 0 | be, this.usage), this.length = be;
                  }
                }, ce.exports = function(be, Ce, Se, we) {
                  if (Se = Se || be.ARRAY_BUFFER, we = we || be.DYNAMIC_DRAW, Se !== be.ARRAY_BUFFER && Se !== be.ELEMENT_ARRAY_BUFFER) throw new Error("gl-buffer: Invalid type for webgl buffer, must be either gl.ARRAY_BUFFER or gl.ELEMENT_ARRAY_BUFFER");
                  if (we !== be.DYNAMIC_DRAW && we !== be.STATIC_DRAW && we !== be.STREAM_DRAW) throw new Error("gl-buffer: Invalid usage for buffer, must be either gl.DYNAMIC_DRAW, gl.STATIC_DRAW or gl.STREAM_DRAW");
                  var Ee = be.createBuffer(), xe = new pe(be, Se, Ee, 0, we);
                  return xe.update(Ce), xe;
                };
              }, 6405: function(ce, le, ue) {
                var de = ue(2931);
                ce.exports = function(he, me) {
                  var pe = he.positions, ye = he.vectors, ge = { positions: [], vertexIntensity: [], vertexIntensityBounds: he.vertexIntensityBounds, vectors: [], cells: [], coneOffset: he.coneOffset, colormap: he.colormap };
                  if (he.positions.length === 0) return me && (me[0] = [0, 0, 0], me[1] = [0, 0, 0]), ge;
                  for (var ve = 0, be = 1 / 0, Ce = -1 / 0, Se = 1 / 0, we = -1 / 0, Ee = 1 / 0, xe = -1 / 0, Le = null, Qe = null, ke = [], Me = 1 / 0, Re = !1, De = he.coneSizemode === "raw", Pe = 0; Pe < pe.length; Pe++) {
                    var $e = pe[Pe];
                    be = Math.min($e[0], be), Ce = Math.max($e[0], Ce), Se = Math.min($e[1], Se), we = Math.max($e[1], we), Ee = Math.min($e[2], Ee), xe = Math.max($e[2], xe);
                    var qe = ye[Pe];
                    if (de.length(qe) > ve && (ve = de.length(qe)), Pe && !De) {
                      var Xe = 2 * de.distance(Le, $e) / (de.length(Qe) + de.length(qe));
                      Xe ? (Me = Math.min(Me, Xe), Re = !1) : Re = !0;
                    }
                    Re || (Le = $e, Qe = qe), ke.push(qe);
                  }
                  var Ve = [be, Se, Ee], Ze = [Ce, we, xe];
                  me && (me[0] = Ve, me[1] = Ze), ve === 0 && (ve = 1);
                  var Ge = 1 / ve;
                  isFinite(Me) || (Me = 1), ge.vectorScale = Me;
                  var ze = he.coneSize || (De ? 1 : 0.5);
                  he.absoluteConeSize && (ze = he.absoluteConeSize * Ge), ge.coneScale = ze, Pe = 0;
                  for (var We = 0; Pe < pe.length; Pe++) for (var at = ($e = pe[Pe])[0], ct = $e[1], et = $e[2], dt = ke[Pe], Je = de.length(dt) * Ge, it = 0; it < 8; it++) {
                    ge.positions.push([at, ct, et, We++]), ge.positions.push([at, ct, et, We++]), ge.positions.push([at, ct, et, We++]), ge.positions.push([at, ct, et, We++]), ge.positions.push([at, ct, et, We++]), ge.positions.push([at, ct, et, We++]), ge.vectors.push(dt), ge.vectors.push(dt), ge.vectors.push(dt), ge.vectors.push(dt), ge.vectors.push(dt), ge.vectors.push(dt), ge.vertexIntensity.push(Je, Je, Je), ge.vertexIntensity.push(Je, Je, Je);
                    var st = ge.positions.length;
                    ge.cells.push([st - 6, st - 5, st - 4], [st - 3, st - 2, st - 1]);
                  }
                  return ge;
                };
                var fe = ue(614);
                ce.exports.createMesh = ue(9060), ce.exports.createConeMesh = function(he, me) {
                  return ce.exports.createMesh(he, me, { shaders: fe, traceType: "cone" });
                };
              }, 9060: function(ce, le, ue) {
                var de = ue(9405), fe = ue(2762), he = ue(8116), me = ue(7766), pe = ue(6760), ye = ue(7608), ge = ue(9618), ve = ue(6729), be = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
                function Ce(we, Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e) {
                  this.gl = we, this.pixelRatio = 1, this.cells = [], this.positions = [], this.intensity = [], this.texture = Ee, this.dirty = !0, this.triShader = xe, this.pickShader = Le, this.trianglePositions = Qe, this.triangleVectors = ke, this.triangleColors = Re, this.triangleUVs = De, this.triangleIds = Me, this.triangleVAO = Pe, this.triangleCount = 0, this.pickId = 1, this.bounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], this.clipBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.lightPosition = [1e5, 1e5, 0], this.ambientLight = 0.8, this.diffuseLight = 0.8, this.specularLight = 2, this.roughness = 0.5, this.fresnel = 1.5, this.opacity = 1, this.traceType = $e, this.tubeScale = 1, this.coneScale = 2, this.vectorScale = 1, this.coneOffset = 0.25, this._model = be, this._view = be, this._projection = be, this._resolution = [1, 1];
                }
                var Se = Ce.prototype;
                Se.isOpaque = function() {
                  return this.opacity >= 1;
                }, Se.isTransparent = function() {
                  return this.opacity < 1;
                }, Se.pickSlots = 1, Se.setPickBase = function(we) {
                  this.pickId = we;
                }, Se.update = function(we) {
                  we = we || {};
                  var Ee = this.gl;
                  this.dirty = !0, "lightPosition" in we && (this.lightPosition = we.lightPosition), "opacity" in we && (this.opacity = we.opacity), "ambient" in we && (this.ambientLight = we.ambient), "diffuse" in we && (this.diffuseLight = we.diffuse), "specular" in we && (this.specularLight = we.specular), "roughness" in we && (this.roughness = we.roughness), "fresnel" in we && (this.fresnel = we.fresnel), we.tubeScale !== void 0 && (this.tubeScale = we.tubeScale), we.vectorScale !== void 0 && (this.vectorScale = we.vectorScale), we.coneScale !== void 0 && (this.coneScale = we.coneScale), we.coneOffset !== void 0 && (this.coneOffset = we.coneOffset), we.colormap && (this.texture.shape = [256, 256], this.texture.minFilter = Ee.LINEAR_MIPMAP_LINEAR, this.texture.magFilter = Ee.LINEAR, this.texture.setPixels(function(gt) {
                    for (var bt = ve({ colormap: gt, nshades: 256, format: "rgba" }), yt = new Uint8Array(1024), St = 0; St < 256; ++St) {
                      for (var Lt = bt[St], Rt = 0; Rt < 3; ++Rt) yt[4 * St + Rt] = Lt[Rt];
                      yt[4 * St + 3] = 255 * Lt[3];
                    }
                    return ge(yt, [256, 256, 4], [4, 0, 1]);
                  }(we.colormap)), this.texture.generateMipmap());
                  var xe = we.cells, Le = we.positions, Qe = we.vectors;
                  if (Le && xe && Qe) {
                    var ke = [], Me = [], Re = [], De = [], Pe = [];
                    this.cells = xe, this.positions = Le, this.vectors = Qe;
                    var $e = we.meshColor || [1, 1, 1, 1], qe = we.vertexIntensity, Xe = 1 / 0, Ve = -1 / 0;
                    if (qe) if (we.vertexIntensityBounds) Xe = +we.vertexIntensityBounds[0], Ve = +we.vertexIntensityBounds[1];
                    else for (var Ze = 0; Ze < qe.length; ++Ze) {
                      var Ge = qe[Ze];
                      Xe = Math.min(Xe, Ge), Ve = Math.max(Ve, Ge);
                    }
                    else for (Ze = 0; Ze < Le.length; ++Ze) Ge = Le[Ze][2], Xe = Math.min(Xe, Ge), Ve = Math.max(Ve, Ge);
                    for (this.intensity = qe || function(gt) {
                      for (var bt = gt.length, yt = new Array(bt), St = 0; St < bt; ++St) yt[St] = gt[St][2];
                      return yt;
                    }(Le), this.bounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], Ze = 0; Ze < Le.length; ++Ze) for (var ze = Le[Ze], We = 0; We < 3; ++We) !isNaN(ze[We]) && isFinite(ze[We]) && (this.bounds[0][We] = Math.min(this.bounds[0][We], ze[We]), this.bounds[1][We] = Math.max(this.bounds[1][We], ze[We]));
                    var at = 0;
                    e: for (Ze = 0; Ze < xe.length; ++Ze) {
                      var ct = xe[Ze];
                      if (ct.length === 3) {
                        for (We = 0; We < 3; ++We) {
                          ze = Le[dt = ct[We]];
                          for (var et = 0; et < 3; ++et) if (isNaN(ze[et]) || !isFinite(ze[et])) continue e;
                        }
                        for (We = 0; We < 3; ++We) {
                          var dt;
                          ze = Le[dt = ct[2 - We]], ke.push(ze[0], ze[1], ze[2], ze[3]);
                          var Je = Qe[dt];
                          Me.push(Je[0], Je[1], Je[2], Je[3] || 0);
                          var it, st = $e;
                          st.length === 3 ? Re.push(st[0], st[1], st[2], 1) : Re.push(st[0], st[1], st[2], st[3]), it = qe ? [(qe[dt] - Xe) / (Ve - Xe), 0] : [(ze[2] - Xe) / (Ve - Xe), 0], De.push(it[0], it[1]), Pe.push(Ze);
                        }
                        at += 1;
                      }
                    }
                    this.triangleCount = at, this.trianglePositions.update(ke), this.triangleVectors.update(Me), this.triangleColors.update(Re), this.triangleUVs.update(De), this.triangleIds.update(new Uint32Array(Pe));
                  }
                }, Se.drawTransparent = Se.draw = function(we) {
                  we = we || {};
                  for (var Ee = this.gl, xe = we.model || be, Le = we.view || be, Qe = we.projection || be, ke = [[-1e6, -1e6, -1e6], [1e6, 1e6, 1e6]], Me = 0; Me < 3; ++Me) ke[0][Me] = Math.max(ke[0][Me], this.clipBounds[0][Me]), ke[1][Me] = Math.min(ke[1][Me], this.clipBounds[1][Me]);
                  var Re = { model: xe, view: Le, projection: Qe, inverseModel: be.slice(), clipBounds: ke, kambient: this.ambientLight, kdiffuse: this.diffuseLight, kspecular: this.specularLight, roughness: this.roughness, fresnel: this.fresnel, eyePosition: [0, 0, 0], lightPosition: [0, 0, 0], opacity: this.opacity, tubeScale: this.tubeScale, vectorScale: this.vectorScale, coneScale: this.coneScale, coneOffset: this.coneOffset, texture: 0 };
                  Re.inverseModel = ye(Re.inverseModel, Re.model), Ee.disable(Ee.CULL_FACE), this.texture.bind(0);
                  var De = new Array(16);
                  for (pe(De, Re.view, Re.model), pe(De, Re.projection, De), ye(De, De), Me = 0; Me < 3; ++Me) Re.eyePosition[Me] = De[12 + Me] / De[15];
                  var Pe = De[15];
                  for (Me = 0; Me < 3; ++Me) Pe += this.lightPosition[Me] * De[4 * Me + 3];
                  for (Me = 0; Me < 3; ++Me) {
                    for (var $e = De[12 + Me], qe = 0; qe < 3; ++qe) $e += De[4 * qe + Me] * this.lightPosition[qe];
                    Re.lightPosition[Me] = $e / Pe;
                  }
                  if (this.triangleCount > 0) {
                    var Xe = this.triShader;
                    Xe.bind(), Xe.uniforms = Re, this.triangleVAO.bind(), Ee.drawArrays(Ee.TRIANGLES, 0, 3 * this.triangleCount), this.triangleVAO.unbind();
                  }
                }, Se.drawPick = function(we) {
                  we = we || {};
                  for (var Ee = this.gl, xe = we.model || be, Le = we.view || be, Qe = we.projection || be, ke = [[-1e6, -1e6, -1e6], [1e6, 1e6, 1e6]], Me = 0; Me < 3; ++Me) ke[0][Me] = Math.max(ke[0][Me], this.clipBounds[0][Me]), ke[1][Me] = Math.min(ke[1][Me], this.clipBounds[1][Me]);
                  this._model = [].slice.call(xe), this._view = [].slice.call(Le), this._projection = [].slice.call(Qe), this._resolution = [Ee.drawingBufferWidth, Ee.drawingBufferHeight];
                  var Re = { model: xe, view: Le, projection: Qe, clipBounds: ke, tubeScale: this.tubeScale, vectorScale: this.vectorScale, coneScale: this.coneScale, coneOffset: this.coneOffset, pickId: this.pickId / 255 }, De = this.pickShader;
                  De.bind(), De.uniforms = Re, this.triangleCount > 0 && (this.triangleVAO.bind(), Ee.drawArrays(Ee.TRIANGLES, 0, 3 * this.triangleCount), this.triangleVAO.unbind());
                }, Se.pick = function(we) {
                  if (!we || we.id !== this.pickId) return null;
                  var Ee = we.value[0] + 256 * we.value[1] + 65536 * we.value[2], xe = this.cells[Ee], Le = this.positions[xe[1]].slice(0, 3), Qe = { position: Le, dataCoordinate: Le, index: Math.floor(xe[1] / 48) };
                  return this.traceType === "cone" ? Qe.index = Math.floor(xe[1] / 48) : this.traceType === "streamtube" && (Qe.intensity = this.intensity[xe[1]], Qe.velocity = this.vectors[xe[1]].slice(0, 3), Qe.divergence = this.vectors[xe[1]][3], Qe.index = Ee), Qe;
                }, Se.dispose = function() {
                  this.texture.dispose(), this.triShader.dispose(), this.pickShader.dispose(), this.triangleVAO.dispose(), this.trianglePositions.dispose(), this.triangleVectors.dispose(), this.triangleColors.dispose(), this.triangleUVs.dispose(), this.triangleIds.dispose();
                }, ce.exports = function(we, Ee, xe) {
                  var Le = xe.shaders;
                  arguments.length === 1 && (we = (Ee = we).gl);
                  var Qe = function(Ve, Ze) {
                    var Ge = de(Ve, Ze.meshShader.vertex, Ze.meshShader.fragment, null, Ze.meshShader.attributes);
                    return Ge.attributes.position.location = 0, Ge.attributes.color.location = 2, Ge.attributes.uv.location = 3, Ge.attributes.vector.location = 4, Ge;
                  }(we, Le), ke = function(Ve, Ze) {
                    var Ge = de(Ve, Ze.pickShader.vertex, Ze.pickShader.fragment, null, Ze.pickShader.attributes);
                    return Ge.attributes.position.location = 0, Ge.attributes.id.location = 1, Ge.attributes.vector.location = 4, Ge;
                  }(we, Le), Me = me(we, ge(new Uint8Array([255, 255, 255, 255]), [1, 1, 4]));
                  Me.generateMipmap(), Me.minFilter = we.LINEAR_MIPMAP_LINEAR, Me.magFilter = we.LINEAR;
                  var Re = fe(we), De = fe(we), Pe = fe(we), $e = fe(we), qe = fe(we), Xe = new Ce(we, Me, Qe, ke, Re, De, qe, Pe, $e, he(we, [{ buffer: Re, type: we.FLOAT, size: 4 }, { buffer: qe, type: we.UNSIGNED_BYTE, size: 4, normalized: !0 }, { buffer: Pe, type: we.FLOAT, size: 4 }, { buffer: $e, type: we.FLOAT, size: 2 }, { buffer: De, type: we.FLOAT, size: 4 }]), xe.traceType || "cone");
                  return Xe.update(Ee), Xe;
                };
              }, 614: function(ce, le, ue) {
                var de = ue(3236), fe = de([`precision highp float;

precision highp float;
#define GLSLIFY 1

vec3 getOrthogonalVector(vec3 v) {
  // Return up-vector for only-z vector.
  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).
  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.
  // Assign z = 0, x = -b, y = a:
  // a*-b + b*a + c*0 = -ba + ba + 0 = 0
  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {
    return normalize(vec3(-v.y, v.x, 0.0));
  } else {
    return normalize(vec3(0.0, v.z, -v.y));
  }
}

// Calculate the cone vertex and normal at the given index.
//
// The returned vertex is for a cone with its top at origin and height of 1.0,
// pointing in the direction of the vector attribute.
//
// Each cone is made up of a top vertex, a center base vertex and base perimeter vertices.
// These vertices are used to make up the triangles of the cone by the following:
//   segment + 0 top vertex
//   segment + 1 perimeter vertex a+1
//   segment + 2 perimeter vertex a
//   segment + 3 center base vertex
//   segment + 4 perimeter vertex a
//   segment + 5 perimeter vertex a+1
// Where segment is the number of the radial segment * 6 and a is the angle at that radial segment.
// To go from index to segment, floor(index / 6)
// To go from segment to angle, 2*pi * (segment/segmentCount)
// To go from index to segment index, index - (segment*6)
//
vec3 getConePosition(vec3 d, float rawIndex, float coneOffset, out vec3 normal) {

  const float segmentCount = 8.0;

  float index = rawIndex - floor(rawIndex /
    (segmentCount * 6.0)) *
    (segmentCount * 6.0);

  float segment = floor(0.001 + index/6.0);
  float segmentIndex = index - (segment*6.0);

  normal = -normalize(d);

  if (segmentIndex > 2.99 && segmentIndex < 3.01) {
    return mix(vec3(0.0), -d, coneOffset);
  }

  float nextAngle = (
    (segmentIndex > 0.99 &&  segmentIndex < 1.01) ||
    (segmentIndex > 4.99 &&  segmentIndex < 5.01)
  ) ? 1.0 : 0.0;
  float angle = 2.0 * 3.14159 * ((segment + nextAngle) / segmentCount);

  vec3 v1 = mix(d, vec3(0.0), coneOffset);
  vec3 v2 = v1 - d;

  vec3 u = getOrthogonalVector(d);
  vec3 v = normalize(cross(u, d));

  vec3 x = u * cos(angle) * length(d)*0.25;
  vec3 y = v * sin(angle) * length(d)*0.25;
  vec3 v3 = v2 + x + y;
  if (segmentIndex < 3.0) {
    vec3 tx = u * sin(angle);
    vec3 ty = v * -cos(angle);
    vec3 tangent = tx + ty;
    normal = normalize(cross(v3 - v1, tangent));
  }

  if (segmentIndex == 0.0) {
    return mix(d, vec3(0.0), coneOffset);
  }
  return v3;
}

attribute vec3 vector;
attribute vec4 color, position;
attribute vec2 uv;

uniform float vectorScale, coneScale, coneOffset;
uniform mat4 model, view, projection, inverseModel;
uniform vec3 eyePosition, lightPosition;

varying vec3 f_normal, f_lightDirection, f_eyeDirection, f_data, f_position;
varying vec4 f_color;
varying vec2 f_uv;

void main() {
  // Scale the vector magnitude to stay constant with
  // model & view changes.
  vec3 normal;
  vec3 XYZ = getConePosition(mat3(model) * ((vectorScale * coneScale) * vector), position.w, coneOffset, normal);
  vec4 conePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);

  //Lighting geometry parameters
  vec4 cameraCoordinate = view * conePosition;
  cameraCoordinate.xyz /= cameraCoordinate.w;
  f_lightDirection = lightPosition - cameraCoordinate.xyz;
  f_eyeDirection   = eyePosition - cameraCoordinate.xyz;
  f_normal = normalize((vec4(normal, 0.0) * inverseModel).xyz);

  // vec4 m_position  = model * vec4(conePosition, 1.0);
  vec4 t_position  = view * conePosition;
  gl_Position      = projection * t_position;

  f_color          = color;
  f_data           = conePosition.xyz;
  f_position       = position.xyz;
  f_uv             = uv;
}
`]), he = de([`#extension GL_OES_standard_derivatives : enable

precision highp float;
#define GLSLIFY 1

float beckmannDistribution(float x, float roughness) {
  float NdotH = max(x, 0.0001);
  float cos2Alpha = NdotH * NdotH;
  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;
  float roughness2 = roughness * roughness;
  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;
  return exp(tan2Alpha / roughness2) / denom;
}

float cookTorranceSpecular(
  vec3 lightDirection,
  vec3 viewDirection,
  vec3 surfaceNormal,
  float roughness,
  float fresnel) {

  float VdotN = max(dot(viewDirection, surfaceNormal), 0.0);
  float LdotN = max(dot(lightDirection, surfaceNormal), 0.0);

  //Half angle vector
  vec3 H = normalize(lightDirection + viewDirection);

  //Geometric term
  float NdotH = max(dot(surfaceNormal, H), 0.0);
  float VdotH = max(dot(viewDirection, H), 0.000001);
  float LdotH = max(dot(lightDirection, H), 0.000001);
  float G1 = (2.0 * NdotH * VdotN) / VdotH;
  float G2 = (2.0 * NdotH * LdotN) / LdotH;
  float G = min(1.0, min(G1, G2));
  
  //Distribution term
  float D = beckmannDistribution(NdotH, roughness);

  //Fresnel term
  float F = pow(1.0 - VdotN, fresnel);

  //Multiply terms and done
  return  G * F * D / max(3.14159265 * VdotN, 0.000001);
}

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 clipBounds[2];
uniform float roughness, fresnel, kambient, kdiffuse, kspecular, opacity;
uniform sampler2D texture;

varying vec3 f_normal, f_lightDirection, f_eyeDirection, f_data, f_position;
varying vec4 f_color;
varying vec2 f_uv;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;
  vec3 N = normalize(f_normal);
  vec3 L = normalize(f_lightDirection);
  vec3 V = normalize(f_eyeDirection);

  if(gl_FrontFacing) {
    N = -N;
  }

  float specular = min(1.0, max(0.0, cookTorranceSpecular(L, V, N, roughness, fresnel)));
  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);

  vec4 surfaceColor = f_color * texture2D(texture, f_uv);
  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);

  gl_FragColor = litColor * opacity;
}
`]), me = de([`precision highp float;

precision highp float;
#define GLSLIFY 1

vec3 getOrthogonalVector(vec3 v) {
  // Return up-vector for only-z vector.
  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).
  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.
  // Assign z = 0, x = -b, y = a:
  // a*-b + b*a + c*0 = -ba + ba + 0 = 0
  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {
    return normalize(vec3(-v.y, v.x, 0.0));
  } else {
    return normalize(vec3(0.0, v.z, -v.y));
  }
}

// Calculate the cone vertex and normal at the given index.
//
// The returned vertex is for a cone with its top at origin and height of 1.0,
// pointing in the direction of the vector attribute.
//
// Each cone is made up of a top vertex, a center base vertex and base perimeter vertices.
// These vertices are used to make up the triangles of the cone by the following:
//   segment + 0 top vertex
//   segment + 1 perimeter vertex a+1
//   segment + 2 perimeter vertex a
//   segment + 3 center base vertex
//   segment + 4 perimeter vertex a
//   segment + 5 perimeter vertex a+1
// Where segment is the number of the radial segment * 6 and a is the angle at that radial segment.
// To go from index to segment, floor(index / 6)
// To go from segment to angle, 2*pi * (segment/segmentCount)
// To go from index to segment index, index - (segment*6)
//
vec3 getConePosition(vec3 d, float rawIndex, float coneOffset, out vec3 normal) {

  const float segmentCount = 8.0;

  float index = rawIndex - floor(rawIndex /
    (segmentCount * 6.0)) *
    (segmentCount * 6.0);

  float segment = floor(0.001 + index/6.0);
  float segmentIndex = index - (segment*6.0);

  normal = -normalize(d);

  if (segmentIndex > 2.99 && segmentIndex < 3.01) {
    return mix(vec3(0.0), -d, coneOffset);
  }

  float nextAngle = (
    (segmentIndex > 0.99 &&  segmentIndex < 1.01) ||
    (segmentIndex > 4.99 &&  segmentIndex < 5.01)
  ) ? 1.0 : 0.0;
  float angle = 2.0 * 3.14159 * ((segment + nextAngle) / segmentCount);

  vec3 v1 = mix(d, vec3(0.0), coneOffset);
  vec3 v2 = v1 - d;

  vec3 u = getOrthogonalVector(d);
  vec3 v = normalize(cross(u, d));

  vec3 x = u * cos(angle) * length(d)*0.25;
  vec3 y = v * sin(angle) * length(d)*0.25;
  vec3 v3 = v2 + x + y;
  if (segmentIndex < 3.0) {
    vec3 tx = u * sin(angle);
    vec3 ty = v * -cos(angle);
    vec3 tangent = tx + ty;
    normal = normalize(cross(v3 - v1, tangent));
  }

  if (segmentIndex == 0.0) {
    return mix(d, vec3(0.0), coneOffset);
  }
  return v3;
}

attribute vec4 vector;
attribute vec4 position;
attribute vec4 id;

uniform mat4 model, view, projection;
uniform float vectorScale, coneScale, coneOffset;

varying vec3 f_position;
varying vec4 f_id;

void main() {
  vec3 normal;
  vec3 XYZ = getConePosition(mat3(model) * ((vectorScale * coneScale) * vector.xyz), position.w, coneOffset, normal);
  vec4 conePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);
  gl_Position = projection * (view * conePosition);
  f_id        = id;
  f_position  = position.xyz;
}
`]), pe = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3  clipBounds[2];
uniform float pickId;

varying vec3 f_position;
varying vec4 f_id;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;

  gl_FragColor = vec4(pickId, f_id.xyz);
}`]);
                le.meshShader = { vertex: fe, fragment: he, attributes: [{ name: "position", type: "vec4" }, { name: "color", type: "vec4" }, { name: "uv", type: "vec2" }, { name: "vector", type: "vec3" }] }, le.pickShader = { vertex: me, fragment: pe, attributes: [{ name: "position", type: "vec4" }, { name: "id", type: "vec4" }, { name: "vector", type: "vec3" }] };
              }, 737: function(ce) {
                ce.exports = { 0: "NONE", 1: "ONE", 2: "LINE_LOOP", 3: "LINE_STRIP", 4: "TRIANGLES", 5: "TRIANGLE_STRIP", 6: "TRIANGLE_FAN", 256: "DEPTH_BUFFER_BIT", 512: "NEVER", 513: "LESS", 514: "EQUAL", 515: "LEQUAL", 516: "GREATER", 517: "NOTEQUAL", 518: "GEQUAL", 519: "ALWAYS", 768: "SRC_COLOR", 769: "ONE_MINUS_SRC_COLOR", 770: "SRC_ALPHA", 771: "ONE_MINUS_SRC_ALPHA", 772: "DST_ALPHA", 773: "ONE_MINUS_DST_ALPHA", 774: "DST_COLOR", 775: "ONE_MINUS_DST_COLOR", 776: "SRC_ALPHA_SATURATE", 1024: "STENCIL_BUFFER_BIT", 1028: "FRONT", 1029: "BACK", 1032: "FRONT_AND_BACK", 1280: "INVALID_ENUM", 1281: "INVALID_VALUE", 1282: "INVALID_OPERATION", 1285: "OUT_OF_MEMORY", 1286: "INVALID_FRAMEBUFFER_OPERATION", 2304: "CW", 2305: "CCW", 2849: "LINE_WIDTH", 2884: "CULL_FACE", 2885: "CULL_FACE_MODE", 2886: "FRONT_FACE", 2928: "DEPTH_RANGE", 2929: "DEPTH_TEST", 2930: "DEPTH_WRITEMASK", 2931: "DEPTH_CLEAR_VALUE", 2932: "DEPTH_FUNC", 2960: "STENCIL_TEST", 2961: "STENCIL_CLEAR_VALUE", 2962: "STENCIL_FUNC", 2963: "STENCIL_VALUE_MASK", 2964: "STENCIL_FAIL", 2965: "STENCIL_PASS_DEPTH_FAIL", 2966: "STENCIL_PASS_DEPTH_PASS", 2967: "STENCIL_REF", 2968: "STENCIL_WRITEMASK", 2978: "VIEWPORT", 3024: "DITHER", 3042: "BLEND", 3088: "SCISSOR_BOX", 3089: "SCISSOR_TEST", 3106: "COLOR_CLEAR_VALUE", 3107: "COLOR_WRITEMASK", 3317: "UNPACK_ALIGNMENT", 3333: "PACK_ALIGNMENT", 3379: "MAX_TEXTURE_SIZE", 3386: "MAX_VIEWPORT_DIMS", 3408: "SUBPIXEL_BITS", 3410: "RED_BITS", 3411: "GREEN_BITS", 3412: "BLUE_BITS", 3413: "ALPHA_BITS", 3414: "DEPTH_BITS", 3415: "STENCIL_BITS", 3553: "TEXTURE_2D", 4352: "DONT_CARE", 4353: "FASTEST", 4354: "NICEST", 5120: "BYTE", 5121: "UNSIGNED_BYTE", 5122: "SHORT", 5123: "UNSIGNED_SHORT", 5124: "INT", 5125: "UNSIGNED_INT", 5126: "FLOAT", 5386: "INVERT", 5890: "TEXTURE", 6401: "STENCIL_INDEX", 6402: "DEPTH_COMPONENT", 6406: "ALPHA", 6407: "RGB", 6408: "RGBA", 6409: "LUMINANCE", 6410: "LUMINANCE_ALPHA", 7680: "KEEP", 7681: "REPLACE", 7682: "INCR", 7683: "DECR", 7936: "VENDOR", 7937: "RENDERER", 7938: "VERSION", 9728: "NEAREST", 9729: "LINEAR", 9984: "NEAREST_MIPMAP_NEAREST", 9985: "LINEAR_MIPMAP_NEAREST", 9986: "NEAREST_MIPMAP_LINEAR", 9987: "LINEAR_MIPMAP_LINEAR", 10240: "TEXTURE_MAG_FILTER", 10241: "TEXTURE_MIN_FILTER", 10242: "TEXTURE_WRAP_S", 10243: "TEXTURE_WRAP_T", 10497: "REPEAT", 10752: "POLYGON_OFFSET_UNITS", 16384: "COLOR_BUFFER_BIT", 32769: "CONSTANT_COLOR", 32770: "ONE_MINUS_CONSTANT_COLOR", 32771: "CONSTANT_ALPHA", 32772: "ONE_MINUS_CONSTANT_ALPHA", 32773: "BLEND_COLOR", 32774: "FUNC_ADD", 32777: "BLEND_EQUATION_RGB", 32778: "FUNC_SUBTRACT", 32779: "FUNC_REVERSE_SUBTRACT", 32819: "UNSIGNED_SHORT_4_4_4_4", 32820: "UNSIGNED_SHORT_5_5_5_1", 32823: "POLYGON_OFFSET_FILL", 32824: "POLYGON_OFFSET_FACTOR", 32854: "RGBA4", 32855: "RGB5_A1", 32873: "TEXTURE_BINDING_2D", 32926: "SAMPLE_ALPHA_TO_COVERAGE", 32928: "SAMPLE_COVERAGE", 32936: "SAMPLE_BUFFERS", 32937: "SAMPLES", 32938: "SAMPLE_COVERAGE_VALUE", 32939: "SAMPLE_COVERAGE_INVERT", 32968: "BLEND_DST_RGB", 32969: "BLEND_SRC_RGB", 32970: "BLEND_DST_ALPHA", 32971: "BLEND_SRC_ALPHA", 33071: "CLAMP_TO_EDGE", 33170: "GENERATE_MIPMAP_HINT", 33189: "DEPTH_COMPONENT16", 33306: "DEPTH_STENCIL_ATTACHMENT", 33635: "UNSIGNED_SHORT_5_6_5", 33648: "MIRRORED_REPEAT", 33901: "ALIASED_POINT_SIZE_RANGE", 33902: "ALIASED_LINE_WIDTH_RANGE", 33984: "TEXTURE0", 33985: "TEXTURE1", 33986: "TEXTURE2", 33987: "TEXTURE3", 33988: "TEXTURE4", 33989: "TEXTURE5", 33990: "TEXTURE6", 33991: "TEXTURE7", 33992: "TEXTURE8", 33993: "TEXTURE9", 33994: "TEXTURE10", 33995: "TEXTURE11", 33996: "TEXTURE12", 33997: "TEXTURE13", 33998: "TEXTURE14", 33999: "TEXTURE15", 34e3: "TEXTURE16", 34001: "TEXTURE17", 34002: "TEXTURE18", 34003: "TEXTURE19", 34004: "TEXTURE20", 34005: "TEXTURE21", 34006: "TEXTURE22", 34007: "TEXTURE23", 34008: "TEXTURE24", 34009: "TEXTURE25", 34010: "TEXTURE26", 34011: "TEXTURE27", 34012: "TEXTURE28", 34013: "TEXTURE29", 34014: "TEXTURE30", 34015: "TEXTURE31", 34016: "ACTIVE_TEXTURE", 34024: "MAX_RENDERBUFFER_SIZE", 34041: "DEPTH_STENCIL", 34055: "INCR_WRAP", 34056: "DECR_WRAP", 34067: "TEXTURE_CUBE_MAP", 34068: "TEXTURE_BINDING_CUBE_MAP", 34069: "TEXTURE_CUBE_MAP_POSITIVE_X", 34070: "TEXTURE_CUBE_MAP_NEGATIVE_X", 34071: "TEXTURE_CUBE_MAP_POSITIVE_Y", 34072: "TEXTURE_CUBE_MAP_NEGATIVE_Y", 34073: "TEXTURE_CUBE_MAP_POSITIVE_Z", 34074: "TEXTURE_CUBE_MAP_NEGATIVE_Z", 34076: "MAX_CUBE_MAP_TEXTURE_SIZE", 34338: "VERTEX_ATTRIB_ARRAY_ENABLED", 34339: "VERTEX_ATTRIB_ARRAY_SIZE", 34340: "VERTEX_ATTRIB_ARRAY_STRIDE", 34341: "VERTEX_ATTRIB_ARRAY_TYPE", 34342: "CURRENT_VERTEX_ATTRIB", 34373: "VERTEX_ATTRIB_ARRAY_POINTER", 34466: "NUM_COMPRESSED_TEXTURE_FORMATS", 34467: "COMPRESSED_TEXTURE_FORMATS", 34660: "BUFFER_SIZE", 34661: "BUFFER_USAGE", 34816: "STENCIL_BACK_FUNC", 34817: "STENCIL_BACK_FAIL", 34818: "STENCIL_BACK_PASS_DEPTH_FAIL", 34819: "STENCIL_BACK_PASS_DEPTH_PASS", 34877: "BLEND_EQUATION_ALPHA", 34921: "MAX_VERTEX_ATTRIBS", 34922: "VERTEX_ATTRIB_ARRAY_NORMALIZED", 34930: "MAX_TEXTURE_IMAGE_UNITS", 34962: "ARRAY_BUFFER", 34963: "ELEMENT_ARRAY_BUFFER", 34964: "ARRAY_BUFFER_BINDING", 34965: "ELEMENT_ARRAY_BUFFER_BINDING", 34975: "VERTEX_ATTRIB_ARRAY_BUFFER_BINDING", 35040: "STREAM_DRAW", 35044: "STATIC_DRAW", 35048: "DYNAMIC_DRAW", 35632: "FRAGMENT_SHADER", 35633: "VERTEX_SHADER", 35660: "MAX_VERTEX_TEXTURE_IMAGE_UNITS", 35661: "MAX_COMBINED_TEXTURE_IMAGE_UNITS", 35663: "SHADER_TYPE", 35664: "FLOAT_VEC2", 35665: "FLOAT_VEC3", 35666: "FLOAT_VEC4", 35667: "INT_VEC2", 35668: "INT_VEC3", 35669: "INT_VEC4", 35670: "BOOL", 35671: "BOOL_VEC2", 35672: "BOOL_VEC3", 35673: "BOOL_VEC4", 35674: "FLOAT_MAT2", 35675: "FLOAT_MAT3", 35676: "FLOAT_MAT4", 35678: "SAMPLER_2D", 35680: "SAMPLER_CUBE", 35712: "DELETE_STATUS", 35713: "COMPILE_STATUS", 35714: "LINK_STATUS", 35715: "VALIDATE_STATUS", 35716: "INFO_LOG_LENGTH", 35717: "ATTACHED_SHADERS", 35718: "ACTIVE_UNIFORMS", 35719: "ACTIVE_UNIFORM_MAX_LENGTH", 35720: "SHADER_SOURCE_LENGTH", 35721: "ACTIVE_ATTRIBUTES", 35722: "ACTIVE_ATTRIBUTE_MAX_LENGTH", 35724: "SHADING_LANGUAGE_VERSION", 35725: "CURRENT_PROGRAM", 36003: "STENCIL_BACK_REF", 36004: "STENCIL_BACK_VALUE_MASK", 36005: "STENCIL_BACK_WRITEMASK", 36006: "FRAMEBUFFER_BINDING", 36007: "RENDERBUFFER_BINDING", 36048: "FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE", 36049: "FRAMEBUFFER_ATTACHMENT_OBJECT_NAME", 36050: "FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL", 36051: "FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE", 36053: "FRAMEBUFFER_COMPLETE", 36054: "FRAMEBUFFER_INCOMPLETE_ATTACHMENT", 36055: "FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT", 36057: "FRAMEBUFFER_INCOMPLETE_DIMENSIONS", 36061: "FRAMEBUFFER_UNSUPPORTED", 36064: "COLOR_ATTACHMENT0", 36096: "DEPTH_ATTACHMENT", 36128: "STENCIL_ATTACHMENT", 36160: "FRAMEBUFFER", 36161: "RENDERBUFFER", 36162: "RENDERBUFFER_WIDTH", 36163: "RENDERBUFFER_HEIGHT", 36164: "RENDERBUFFER_INTERNAL_FORMAT", 36168: "STENCIL_INDEX8", 36176: "RENDERBUFFER_RED_SIZE", 36177: "RENDERBUFFER_GREEN_SIZE", 36178: "RENDERBUFFER_BLUE_SIZE", 36179: "RENDERBUFFER_ALPHA_SIZE", 36180: "RENDERBUFFER_DEPTH_SIZE", 36181: "RENDERBUFFER_STENCIL_SIZE", 36194: "RGB565", 36336: "LOW_FLOAT", 36337: "MEDIUM_FLOAT", 36338: "HIGH_FLOAT", 36339: "LOW_INT", 36340: "MEDIUM_INT", 36341: "HIGH_INT", 36346: "SHADER_COMPILER", 36347: "MAX_VERTEX_UNIFORM_VECTORS", 36348: "MAX_VARYING_VECTORS", 36349: "MAX_FRAGMENT_UNIFORM_VECTORS", 37440: "UNPACK_FLIP_Y_WEBGL", 37441: "UNPACK_PREMULTIPLY_ALPHA_WEBGL", 37442: "CONTEXT_LOST_WEBGL", 37443: "UNPACK_COLORSPACE_CONVERSION_WEBGL", 37444: "BROWSER_DEFAULT_WEBGL" };
              }, 5171: function(ce, le, ue) {
                var de = ue(737);
                ce.exports = function(fe) {
                  return de[fe];
                };
              }, 9165: function(ce, le, ue) {
                ce.exports = function(Ce) {
                  var Se = Ce.gl, we = de(Se), Ee = fe(Se, [{ buffer: we, type: Se.FLOAT, size: 3, offset: 0, stride: 40 }, { buffer: we, type: Se.FLOAT, size: 4, offset: 12, stride: 40 }, { buffer: we, type: Se.FLOAT, size: 3, offset: 28, stride: 40 }]), xe = he(Se);
                  xe.attributes.position.location = 0, xe.attributes.color.location = 1, xe.attributes.offset.location = 2;
                  var Le = new pe(Se, we, Ee, xe);
                  return Le.update(Ce), Le;
                };
                var de = ue(2762), fe = ue(8116), he = ue(3436), me = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
                function pe(Ce, Se, we, Ee) {
                  this.gl = Ce, this.shader = Ee, this.buffer = Se, this.vao = we, this.pixelRatio = 1, this.bounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], this.clipBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.lineWidth = [1, 1, 1], this.capSize = [10, 10, 10], this.lineCount = [0, 0, 0], this.lineOffset = [0, 0, 0], this.opacity = 1, this.hasAlpha = !1;
                }
                var ye = pe.prototype;
                function ge(Ce, Se) {
                  for (var we = 0; we < 3; ++we) Ce[0][we] = Math.min(Ce[0][we], Se[we]), Ce[1][we] = Math.max(Ce[1][we], Se[we]);
                }
                ye.isOpaque = function() {
                  return !this.hasAlpha;
                }, ye.isTransparent = function() {
                  return this.hasAlpha;
                }, ye.drawTransparent = ye.draw = function(Ce) {
                  var Se = this.gl, we = this.shader.uniforms;
                  this.shader.bind();
                  var Ee = we.view = Ce.view || me, xe = we.projection = Ce.projection || me;
                  we.model = Ce.model || me, we.clipBounds = this.clipBounds, we.opacity = this.opacity;
                  var Le = Ee[12], Qe = Ee[13], ke = Ee[14], Me = Ee[15], Re = (Ce._ortho ? 2 : 1) * this.pixelRatio * (xe[3] * Le + xe[7] * Qe + xe[11] * ke + xe[15] * Me) / Se.drawingBufferHeight;
                  this.vao.bind();
                  for (var De = 0; De < 3; ++De) Se.lineWidth(this.lineWidth[De] * this.pixelRatio), we.capSize = this.capSize[De] * Re, this.lineCount[De] && Se.drawArrays(Se.LINES, this.lineOffset[De], this.lineCount[De]);
                  this.vao.unbind();
                };
                var ve = function() {
                  for (var Ce = new Array(3), Se = 0; Se < 3; ++Se) {
                    for (var we = [], Ee = 1; Ee <= 2; ++Ee) for (var xe = -1; xe <= 1; xe += 2) {
                      var Le = [0, 0, 0];
                      Le[(Ee + Se) % 3] = xe, we.push(Le);
                    }
                    Ce[Se] = we;
                  }
                  return Ce;
                }();
                function be(Ce, Se, we, Ee) {
                  for (var xe = ve[Ee], Le = 0; Le < xe.length; ++Le) {
                    var Qe = xe[Le];
                    Ce.push(Se[0], Se[1], Se[2], we[0], we[1], we[2], we[3], Qe[0], Qe[1], Qe[2]);
                  }
                  return xe.length;
                }
                ye.update = function(Ce) {
                  "lineWidth" in (Ce = Ce || {}) && (this.lineWidth = Ce.lineWidth, Array.isArray(this.lineWidth) || (this.lineWidth = [this.lineWidth, this.lineWidth, this.lineWidth])), "capSize" in Ce && (this.capSize = Ce.capSize, Array.isArray(this.capSize) || (this.capSize = [this.capSize, this.capSize, this.capSize])), this.hasAlpha = !1, "opacity" in Ce && (this.opacity = +Ce.opacity, this.opacity < 1 && (this.hasAlpha = !0));
                  var Se = Ce.color || [[0, 0, 0], [0, 0, 0], [0, 0, 0]], we = Ce.position, Ee = Ce.error;
                  if (Array.isArray(Se[0]) || (Se = [Se, Se, Se]), we && Ee) {
                    var xe = [], Le = we.length, Qe = 0;
                    this.bounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], this.lineCount = [0, 0, 0];
                    for (var ke = 0; ke < 3; ++ke) {
                      this.lineOffset[ke] = Qe;
                      e: for (var Me = 0; Me < Le; ++Me) {
                        for (var Re = we[Me], De = 0; De < 3; ++De) if (isNaN(Re[De]) || !isFinite(Re[De])) continue e;
                        var Pe, $e = Ee[Me], qe = Se[ke];
                        Array.isArray(qe[0]) && (qe = Se[Me]), qe.length === 3 ? qe = [qe[0], qe[1], qe[2], 1] : qe.length === 4 && (qe = [qe[0], qe[1], qe[2], qe[3]], !this.hasAlpha && qe[3] < 1 && (this.hasAlpha = !0)), !isNaN($e[0][ke]) && !isNaN($e[1][ke]) && ($e[0][ke] < 0 && ((Pe = Re.slice())[ke] += $e[0][ke], xe.push(Re[0], Re[1], Re[2], qe[0], qe[1], qe[2], qe[3], 0, 0, 0, Pe[0], Pe[1], Pe[2], qe[0], qe[1], qe[2], qe[3], 0, 0, 0), ge(this.bounds, Pe), Qe += 2 + be(xe, Pe, qe, ke)), $e[1][ke] > 0 && ((Pe = Re.slice())[ke] += $e[1][ke], xe.push(Re[0], Re[1], Re[2], qe[0], qe[1], qe[2], qe[3], 0, 0, 0, Pe[0], Pe[1], Pe[2], qe[0], qe[1], qe[2], qe[3], 0, 0, 0), ge(this.bounds, Pe), Qe += 2 + be(xe, Pe, qe, ke)));
                      }
                      this.lineCount[ke] = Qe - this.lineOffset[ke];
                    }
                    this.buffer.update(xe);
                  }
                }, ye.dispose = function() {
                  this.shader.dispose(), this.buffer.dispose(), this.vao.dispose();
                };
              }, 3436: function(ce, le, ue) {
                var de = ue(3236), fe = ue(9405), he = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position, offset;
attribute vec4 color;
uniform mat4 model, view, projection;
uniform float capSize;
varying vec4 fragColor;
varying vec3 fragPosition;

void main() {
  vec4 worldPosition  = model * vec4(position, 1.0);
  worldPosition       = (worldPosition / worldPosition.w) + vec4(capSize * offset, 0.0);
  gl_Position         = projection * (view * worldPosition);
  fragColor           = color;
  fragPosition        = position;
}`]), me = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 clipBounds[2];
uniform float opacity;
varying vec3 fragPosition;
varying vec4 fragColor;

void main() {
  if (
    outOfRange(clipBounds[0], clipBounds[1], fragPosition) ||
    fragColor.a * opacity == 0.
  ) discard;

  gl_FragColor = opacity * fragColor;
}`]);
                ce.exports = function(pe) {
                  return fe(pe, he, me, null, [{ name: "position", type: "vec3" }, { name: "color", type: "vec4" }, { name: "offset", type: "vec3" }]);
                };
              }, 2260: function(ce, le, ue) {
                var de = ue(7766);
                ce.exports = function(Le, Qe, ke, Me) {
                  fe || (fe = Le.FRAMEBUFFER_UNSUPPORTED, he = Le.FRAMEBUFFER_INCOMPLETE_ATTACHMENT, me = Le.FRAMEBUFFER_INCOMPLETE_DIMENSIONS, pe = Le.FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT);
                  var Re = Le.getExtension("WEBGL_draw_buffers");
                  if (!ye && Re && function(Ze, Ge) {
                    var ze = Ze.getParameter(Ge.MAX_COLOR_ATTACHMENTS_WEBGL);
                    ye = new Array(ze + 1);
                    for (var We = 0; We <= ze; ++We) {
                      for (var at = new Array(ze), ct = 0; ct < We; ++ct) at[ct] = Ze.COLOR_ATTACHMENT0 + ct;
                      for (ct = We; ct < ze; ++ct) at[ct] = Ze.NONE;
                      ye[We] = at;
                    }
                  }(Le, Re), Array.isArray(Qe) && (Me = ke, ke = 0 | Qe[1], Qe = 0 | Qe[0]), typeof Qe != "number") throw new Error("gl-fbo: Missing shape parameter");
                  var De = Le.getParameter(Le.MAX_RENDERBUFFER_SIZE);
                  if (Qe < 0 || Qe > De || ke < 0 || ke > De) throw new Error("gl-fbo: Parameters are too large for FBO");
                  var Pe = 1;
                  if ("color" in (Me = Me || {})) {
                    if ((Pe = Math.max(0 | Me.color, 0)) < 0) throw new Error("gl-fbo: Must specify a nonnegative number of colors");
                    if (Pe > 1) {
                      if (!Re) throw new Error("gl-fbo: Multiple draw buffer extension not supported");
                      if (Pe > Le.getParameter(Re.MAX_COLOR_ATTACHMENTS_WEBGL)) throw new Error("gl-fbo: Context does not support " + Pe + " draw buffers");
                    }
                  }
                  var $e = Le.UNSIGNED_BYTE, qe = Le.getExtension("OES_texture_float");
                  if (Me.float && Pe > 0) {
                    if (!qe) throw new Error("gl-fbo: Context does not support floating point textures");
                    $e = Le.FLOAT;
                  } else Me.preferFloat && Pe > 0 && qe && ($e = Le.FLOAT);
                  var Xe = !0;
                  "depth" in Me && (Xe = !!Me.depth);
                  var Ve = !1;
                  return "stencil" in Me && (Ve = !!Me.stencil), new we(Le, Qe, ke, $e, Pe, Xe, Ve, Re);
                };
                var fe, he, me, pe, ye = null;
                function ge(Le) {
                  return [Le.getParameter(Le.FRAMEBUFFER_BINDING), Le.getParameter(Le.RENDERBUFFER_BINDING), Le.getParameter(Le.TEXTURE_BINDING_2D)];
                }
                function ve(Le, Qe) {
                  Le.bindFramebuffer(Le.FRAMEBUFFER, Qe[0]), Le.bindRenderbuffer(Le.RENDERBUFFER, Qe[1]), Le.bindTexture(Le.TEXTURE_2D, Qe[2]);
                }
                function be(Le) {
                  switch (Le) {
                    case fe:
                      throw new Error("gl-fbo: Framebuffer unsupported");
                    case he:
                      throw new Error("gl-fbo: Framebuffer incomplete attachment");
                    case me:
                      throw new Error("gl-fbo: Framebuffer incomplete dimensions");
                    case pe:
                      throw new Error("gl-fbo: Framebuffer incomplete missing attachment");
                    default:
                      throw new Error("gl-fbo: Framebuffer failed for unspecified reason");
                  }
                }
                function Ce(Le, Qe, ke, Me, Re, De) {
                  if (!Me) return null;
                  var Pe = de(Le, Qe, ke, Re, Me);
                  return Pe.magFilter = Le.NEAREST, Pe.minFilter = Le.NEAREST, Pe.mipSamples = 1, Pe.bind(), Le.framebufferTexture2D(Le.FRAMEBUFFER, De, Le.TEXTURE_2D, Pe.handle, 0), Pe;
                }
                function Se(Le, Qe, ke, Me, Re) {
                  var De = Le.createRenderbuffer();
                  return Le.bindRenderbuffer(Le.RENDERBUFFER, De), Le.renderbufferStorage(Le.RENDERBUFFER, Me, Qe, ke), Le.framebufferRenderbuffer(Le.FRAMEBUFFER, Re, Le.RENDERBUFFER, De), De;
                }
                function we(Le, Qe, ke, Me, Re, De, Pe, $e) {
                  this.gl = Le, this._shape = [0 | Qe, 0 | ke], this._destroyed = !1, this._ext = $e, this.color = new Array(Re);
                  for (var qe = 0; qe < Re; ++qe) this.color[qe] = null;
                  this._color_rb = null, this.depth = null, this._depth_rb = null, this._colorType = Me, this._useDepth = De, this._useStencil = Pe;
                  var Xe = this, Ve = [0 | Qe, 0 | ke];
                  Object.defineProperties(Ve, { 0: { get: function() {
                    return Xe._shape[0];
                  }, set: function(Ze) {
                    return Xe.width = Ze;
                  } }, 1: { get: function() {
                    return Xe._shape[1];
                  }, set: function(Ze) {
                    return Xe.height = Ze;
                  } } }), this._shapeVector = Ve, function(Ze) {
                    var Ge = ge(Ze.gl), ze = Ze.gl, We = Ze.handle = ze.createFramebuffer(), at = Ze._shape[0], ct = Ze._shape[1], et = Ze.color.length, dt = Ze._ext, Je = Ze._useStencil, it = Ze._useDepth, st = Ze._colorType;
                    ze.bindFramebuffer(ze.FRAMEBUFFER, We);
                    for (var gt = 0; gt < et; ++gt) Ze.color[gt] = Ce(ze, at, ct, st, ze.RGBA, ze.COLOR_ATTACHMENT0 + gt);
                    et === 0 ? (Ze._color_rb = Se(ze, at, ct, ze.RGBA4, ze.COLOR_ATTACHMENT0), dt && dt.drawBuffersWEBGL(ye[0])) : et > 1 && dt.drawBuffersWEBGL(ye[et]);
                    var bt = ze.getExtension("WEBGL_depth_texture");
                    bt ? Je ? Ze.depth = Ce(ze, at, ct, bt.UNSIGNED_INT_24_8_WEBGL, ze.DEPTH_STENCIL, ze.DEPTH_STENCIL_ATTACHMENT) : it && (Ze.depth = Ce(ze, at, ct, ze.UNSIGNED_SHORT, ze.DEPTH_COMPONENT, ze.DEPTH_ATTACHMENT)) : it && Je ? Ze._depth_rb = Se(ze, at, ct, ze.DEPTH_STENCIL, ze.DEPTH_STENCIL_ATTACHMENT) : it ? Ze._depth_rb = Se(ze, at, ct, ze.DEPTH_COMPONENT16, ze.DEPTH_ATTACHMENT) : Je && (Ze._depth_rb = Se(ze, at, ct, ze.STENCIL_INDEX, ze.STENCIL_ATTACHMENT));
                    var yt = ze.checkFramebufferStatus(ze.FRAMEBUFFER);
                    if (yt !== ze.FRAMEBUFFER_COMPLETE) {
                      for (Ze._destroyed = !0, ze.bindFramebuffer(ze.FRAMEBUFFER, null), ze.deleteFramebuffer(Ze.handle), Ze.handle = null, Ze.depth && (Ze.depth.dispose(), Ze.depth = null), Ze._depth_rb && (ze.deleteRenderbuffer(Ze._depth_rb), Ze._depth_rb = null), gt = 0; gt < Ze.color.length; ++gt) Ze.color[gt].dispose(), Ze.color[gt] = null;
                      Ze._color_rb && (ze.deleteRenderbuffer(Ze._color_rb), Ze._color_rb = null), ve(ze, Ge), be(yt);
                    }
                    ve(ze, Ge);
                  }(this);
                }
                var Ee = we.prototype;
                function xe(Le, Qe, ke) {
                  if (Le._destroyed) throw new Error("gl-fbo: Can't resize destroyed FBO");
                  if (Le._shape[0] !== Qe || Le._shape[1] !== ke) {
                    var Me = Le.gl, Re = Me.getParameter(Me.MAX_RENDERBUFFER_SIZE);
                    if (Qe < 0 || Qe > Re || ke < 0 || ke > Re) throw new Error("gl-fbo: Can't resize FBO, invalid dimensions");
                    Le._shape[0] = Qe, Le._shape[1] = ke;
                    for (var De = ge(Me), Pe = 0; Pe < Le.color.length; ++Pe) Le.color[Pe].shape = Le._shape;
                    Le._color_rb && (Me.bindRenderbuffer(Me.RENDERBUFFER, Le._color_rb), Me.renderbufferStorage(Me.RENDERBUFFER, Me.RGBA4, Le._shape[0], Le._shape[1])), Le.depth && (Le.depth.shape = Le._shape), Le._depth_rb && (Me.bindRenderbuffer(Me.RENDERBUFFER, Le._depth_rb), Le._useDepth && Le._useStencil ? Me.renderbufferStorage(Me.RENDERBUFFER, Me.DEPTH_STENCIL, Le._shape[0], Le._shape[1]) : Le._useDepth ? Me.renderbufferStorage(Me.RENDERBUFFER, Me.DEPTH_COMPONENT16, Le._shape[0], Le._shape[1]) : Le._useStencil && Me.renderbufferStorage(Me.RENDERBUFFER, Me.STENCIL_INDEX, Le._shape[0], Le._shape[1])), Me.bindFramebuffer(Me.FRAMEBUFFER, Le.handle);
                    var $e = Me.checkFramebufferStatus(Me.FRAMEBUFFER);
                    $e !== Me.FRAMEBUFFER_COMPLETE && (Le.dispose(), ve(Me, De), be($e)), ve(Me, De);
                  }
                }
                Object.defineProperties(Ee, { shape: { get: function() {
                  return this._destroyed ? [0, 0] : this._shapeVector;
                }, set: function(Le) {
                  if (Array.isArray(Le) || (Le = [0 | Le, 0 | Le]), Le.length !== 2) throw new Error("gl-fbo: Shape vector must be length 2");
                  var Qe = 0 | Le[0], ke = 0 | Le[1];
                  return xe(this, Qe, ke), [Qe, ke];
                }, enumerable: !1 }, width: { get: function() {
                  return this._destroyed ? 0 : this._shape[0];
                }, set: function(Le) {
                  return xe(this, Le |= 0, this._shape[1]), Le;
                }, enumerable: !1 }, height: { get: function() {
                  return this._destroyed ? 0 : this._shape[1];
                }, set: function(Le) {
                  return Le |= 0, xe(this, this._shape[0], Le), Le;
                }, enumerable: !1 } }), Ee.bind = function() {
                  if (!this._destroyed) {
                    var Le = this.gl;
                    Le.bindFramebuffer(Le.FRAMEBUFFER, this.handle), Le.viewport(0, 0, this._shape[0], this._shape[1]);
                  }
                }, Ee.dispose = function() {
                  if (!this._destroyed) {
                    this._destroyed = !0;
                    var Le = this.gl;
                    Le.deleteFramebuffer(this.handle), this.handle = null, this.depth && (this.depth.dispose(), this.depth = null), this._depth_rb && (Le.deleteRenderbuffer(this._depth_rb), this._depth_rb = null);
                    for (var Qe = 0; Qe < this.color.length; ++Qe) this.color[Qe].dispose(), this.color[Qe] = null;
                    this._color_rb && (Le.deleteRenderbuffer(this._color_rb), this._color_rb = null);
                  }
                };
              }, 2992: function(ce, le, ue) {
                var de = ue(3387).sprintf, fe = ue(5171), he = ue(1848), me = ue(1085);
                ce.exports = function(pe, ye, ge) {
                  var ve = he(ye) || "of unknown name (see npm glsl-shader-name)", be = "unknown type";
                  ge !== void 0 && (be = ge === fe.FRAGMENT_SHADER ? "fragment" : "vertex");
                  for (var Ce = de(`Error compiling %s shader %s:
`, be, ve), Se = de("%s%s", Ce, pe), we = pe.split(`
`), Ee = {}, xe = 0; xe < we.length; xe++) {
                    var Le = we[xe];
                    if (Le !== "" && Le !== "\0") {
                      var Qe = parseInt(Le.split(":")[2]);
                      if (isNaN(Qe)) throw new Error(de("Could not parse error: %s", Le));
                      Ee[Qe] = Le;
                    }
                  }
                  var ke = me(ye).split(`
`);
                  for (xe = 0; xe < ke.length; xe++) if ((Ee[xe + 3] || Ee[xe + 2] || Ee[xe + 1]) && (Ce += ke[xe] + `
`, Ee[xe + 1])) {
                    var Me = Ee[xe + 1];
                    Me = Me.substr(Me.split(":", 3).join(":").length + 1).trim(), Ce += de(`^^^ %s

`, Me);
                  }
                  return { long: Ce.trim(), short: Se.trim() };
                };
              }, 2510: function(ce, le, ue) {
                ce.exports = function(Se, we) {
                  var Ee = Se.gl, xe = new ge(Se, me(Ee, ye.vertex, ye.fragment), me(Ee, ye.pickVertex, ye.pickFragment), pe(Ee), pe(Ee), pe(Ee), pe(Ee));
                  return xe.update(we), Se.addObject(xe), xe;
                };
                var de = ue(2478), fe = ue(7762), he = ue(1888), me = ue(9405), pe = ue(2762), ye = ue(6768);
                function ge(Se, we, Ee, xe, Le, Qe, ke) {
                  this.plot = Se, this.shader = we, this.pickShader = Ee, this.positionBuffer = xe, this.weightBuffer = Le, this.colorBuffer = Qe, this.idBuffer = ke, this.xData = [], this.yData = [], this.shape = [0, 0], this.bounds = [1 / 0, 1 / 0, -1 / 0, -1 / 0], this.pickOffset = 0;
                }
                var ve, be = ge.prototype, Ce = [0, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1];
                be.draw = (ve = [1, 0, 0, 0, 1, 0, 0, 0, 1], function() {
                  var Se = this.plot, we = this.shader, Ee = this.bounds, xe = this.numVertices;
                  if (!(xe <= 0)) {
                    var Le = Se.gl, Qe = Se.dataBox, ke = Ee[2] - Ee[0], Me = Ee[3] - Ee[1], Re = Qe[2] - Qe[0], De = Qe[3] - Qe[1];
                    ve[0] = 2 * ke / Re, ve[4] = 2 * Me / De, ve[6] = 2 * (Ee[0] - Qe[0]) / Re - 1, ve[7] = 2 * (Ee[1] - Qe[1]) / De - 1, we.bind();
                    var Pe = we.uniforms;
                    Pe.viewTransform = ve, Pe.shape = this.shape;
                    var $e = we.attributes;
                    this.positionBuffer.bind(), $e.position.pointer(), this.weightBuffer.bind(), $e.weight.pointer(Le.UNSIGNED_BYTE, !1), this.colorBuffer.bind(), $e.color.pointer(Le.UNSIGNED_BYTE, !0), Le.drawArrays(Le.TRIANGLES, 0, xe);
                  }
                }), be.drawPick = /* @__PURE__ */ function() {
                  var Se = [1, 0, 0, 0, 1, 0, 0, 0, 1], we = [0, 0, 0, 0];
                  return function(Ee) {
                    var xe = this.plot, Le = this.pickShader, Qe = this.bounds, ke = this.numVertices;
                    if (!(ke <= 0)) {
                      var Me = xe.gl, Re = xe.dataBox, De = Qe[2] - Qe[0], Pe = Qe[3] - Qe[1], $e = Re[2] - Re[0], qe = Re[3] - Re[1];
                      Se[0] = 2 * De / $e, Se[4] = 2 * Pe / qe, Se[6] = 2 * (Qe[0] - Re[0]) / $e - 1, Se[7] = 2 * (Qe[1] - Re[1]) / qe - 1;
                      for (var Xe = 0; Xe < 4; ++Xe) we[Xe] = Ee >> 8 * Xe & 255;
                      this.pickOffset = Ee, Le.bind();
                      var Ve = Le.uniforms;
                      Ve.viewTransform = Se, Ve.pickOffset = we, Ve.shape = this.shape;
                      var Ze = Le.attributes;
                      return this.positionBuffer.bind(), Ze.position.pointer(), this.weightBuffer.bind(), Ze.weight.pointer(Me.UNSIGNED_BYTE, !1), this.idBuffer.bind(), Ze.pickId.pointer(Me.UNSIGNED_BYTE, !1), Me.drawArrays(Me.TRIANGLES, 0, ke), Ee + this.shape[0] * this.shape[1];
                    }
                  };
                }(), be.pick = function(Se, we, Ee) {
                  var xe = this.pickOffset, Le = this.shape[0] * this.shape[1];
                  if (Ee < xe || Ee >= xe + Le) return null;
                  var Qe = Ee - xe, ke = this.xData, Me = this.yData;
                  return { object: this, pointId: Qe, dataCoord: [ke[Qe % this.shape[0]], Me[Qe / this.shape[0] | 0]] };
                }, be.update = function(Se) {
                  var we = (Se = Se || {}).shape || [0, 0], Ee = Se.x || fe(we[0]), xe = Se.y || fe(we[1]), Le = Se.z || new Float32Array(we[0] * we[1]), Qe = Se.zsmooth !== !1;
                  this.xData = Ee, this.yData = xe;
                  var ke, Me, Re, De, Pe = Se.colorLevels || [0], $e = Se.colorValues || [0, 0, 0, 1], qe = Pe.length, Xe = this.bounds;
                  Qe ? (ke = Xe[0] = Ee[0], Me = Xe[1] = xe[0], Re = Xe[2] = Ee[Ee.length - 1], De = Xe[3] = xe[xe.length - 1]) : (ke = Xe[0] = Ee[0] + (Ee[1] - Ee[0]) / 2, Me = Xe[1] = xe[0] + (xe[1] - xe[0]) / 2, Re = Xe[2] = Ee[Ee.length - 1] + (Ee[Ee.length - 1] - Ee[Ee.length - 2]) / 2, De = Xe[3] = xe[xe.length - 1] + (xe[xe.length - 1] - xe[xe.length - 2]) / 2);
                  var Ve = 1 / (Re - ke), Ze = 1 / (De - Me), Ge = we[0], ze = we[1];
                  this.shape = [Ge, ze];
                  var We = (Qe ? (Ge - 1) * (ze - 1) : Ge * ze) * (Ce.length >>> 1);
                  this.numVertices = We;
                  for (var at = he.mallocUint8(4 * We), ct = he.mallocFloat32(2 * We), et = he.mallocUint8(2 * We), dt = he.mallocUint32(We), Je = 0, it = Qe ? Ge - 1 : Ge, st = Qe ? ze - 1 : ze, gt = 0; gt < st; ++gt) {
                    var bt, yt;
                    Qe ? (bt = Ze * (xe[gt] - Me), yt = Ze * (xe[gt + 1] - Me)) : (bt = gt < ze - 1 ? Ze * (xe[gt] - (xe[gt + 1] - xe[gt]) / 2 - Me) : Ze * (xe[gt] - (xe[gt] - xe[gt - 1]) / 2 - Me), yt = gt < ze - 1 ? Ze * (xe[gt] + (xe[gt + 1] - xe[gt]) / 2 - Me) : Ze * (xe[gt] + (xe[gt] - xe[gt - 1]) / 2 - Me));
                    for (var St = 0; St < it; ++St) {
                      var Lt, Rt;
                      Qe ? (Lt = Ve * (Ee[St] - ke), Rt = Ve * (Ee[St + 1] - ke)) : (Lt = St < Ge - 1 ? Ve * (Ee[St] - (Ee[St + 1] - Ee[St]) / 2 - ke) : Ve * (Ee[St] - (Ee[St] - Ee[St - 1]) / 2 - ke), Rt = St < Ge - 1 ? Ve * (Ee[St] + (Ee[St + 1] - Ee[St]) / 2 - ke) : Ve * (Ee[St] + (Ee[St] - Ee[St - 1]) / 2 - ke));
                      for (var vt = 0; vt < Ce.length; vt += 2) {
                        var $t, Pt, Xt, Nt, Gt = Ce[vt], Ft = Ce[vt + 1], Tt = Le[Qe ? (gt + Ft) * Ge + (St + Gt) : gt * Ge + St], It = de.le(Pe, Tt);
                        if (It < 0) $t = $e[0], Pt = $e[1], Xt = $e[2], Nt = $e[3];
                        else if (It === qe - 1) $t = $e[4 * qe - 4], Pt = $e[4 * qe - 3], Xt = $e[4 * qe - 2], Nt = $e[4 * qe - 1];
                        else {
                          var Ct = (Tt - Pe[It]) / (Pe[It + 1] - Pe[It]), Kt = 1 - Ct, Vt = 4 * It, Yt = 4 * (It + 1);
                          $t = Kt * $e[Vt] + Ct * $e[Yt], Pt = Kt * $e[Vt + 1] + Ct * $e[Yt + 1], Xt = Kt * $e[Vt + 2] + Ct * $e[Yt + 2], Nt = Kt * $e[Vt + 3] + Ct * $e[Yt + 3];
                        }
                        at[4 * Je] = 255 * $t, at[4 * Je + 1] = 255 * Pt, at[4 * Je + 2] = 255 * Xt, at[4 * Je + 3] = 255 * Nt, ct[2 * Je] = 0.5 * Lt + 0.5 * Rt, ct[2 * Je + 1] = 0.5 * bt + 0.5 * yt, et[2 * Je] = Gt, et[2 * Je + 1] = Ft, dt[Je] = gt * Ge + St, Je += 1;
                      }
                    }
                  }
                  this.positionBuffer.update(ct), this.weightBuffer.update(et), this.colorBuffer.update(at), this.idBuffer.update(dt), he.free(ct), he.free(at), he.free(et), he.free(dt);
                }, be.dispose = function() {
                  this.shader.dispose(), this.pickShader.dispose(), this.positionBuffer.dispose(), this.weightBuffer.dispose(), this.colorBuffer.dispose(), this.idBuffer.dispose(), this.plot.removeObject(this);
                };
              }, 6768: function(ce, le, ue) {
                var de = ue(3236);
                ce.exports = { fragment: de([`precision lowp float;
#define GLSLIFY 1
varying vec4 fragColor;
void main() {
  gl_FragColor = vec4(fragColor.rgb * fragColor.a, fragColor.a);
}
`]), vertex: de([`precision mediump float;
#define GLSLIFY 1

attribute vec2 position;
attribute vec4 color;
attribute vec2 weight;

uniform vec2 shape;
uniform mat3 viewTransform;

varying vec4 fragColor;

void main() {
  vec3 vPosition = viewTransform * vec3( position + (weight-.5)/(shape-1.) , 1.0);
  fragColor = color;
  gl_Position = vec4(vPosition.xy, 0, vPosition.z);
}
`]), pickFragment: de([`precision mediump float;
#define GLSLIFY 1

varying vec4 fragId;
varying vec2 vWeight;

uniform vec2 shape;
uniform vec4 pickOffset;

void main() {
  vec2 d = step(.5, vWeight);
  vec4 id = fragId + pickOffset;
  id.x += d.x + d.y*shape.x;

  id.y += floor(id.x / 256.0);
  id.x -= floor(id.x / 256.0) * 256.0;

  id.z += floor(id.y / 256.0);
  id.y -= floor(id.y / 256.0) * 256.0;

  id.w += floor(id.z / 256.0);
  id.z -= floor(id.z / 256.0) * 256.0;

  gl_FragColor = id/255.;
}
`]), pickVertex: de([`precision mediump float;
#define GLSLIFY 1

attribute vec2 position;
attribute vec4 pickId;
attribute vec2 weight;

uniform vec2 shape;
uniform mat3 viewTransform;

varying vec4 fragId;
varying vec2 vWeight;

void main() {
  vWeight = weight;

  fragId = pickId;

  vec3 vPosition = viewTransform * vec3( position + (weight-.5)/(shape-1.) , 1.0);
  gl_Position = vec4(vPosition.xy, 0, vPosition.z);
}
`]) };
              }, 7319: function(ce, le, ue) {
                var de = ue(3236), fe = ue(9405), he = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position, nextPosition;
attribute float arcLength, lineWidth;
attribute vec4 color;

uniform vec2 screenShape;
uniform float pixelRatio;
uniform mat4 model, view, projection;

varying vec4 fragColor;
varying vec3 worldPosition;
varying float pixelArcLength;

vec4 project(vec3 p) {
  return projection * (view * (model * vec4(p, 1.0)));
}

void main() {
  vec4 startPoint = project(position);
  vec4 endPoint   = project(nextPosition);

  vec2 A = startPoint.xy / startPoint.w;
  vec2 B =   endPoint.xy /   endPoint.w;

  float clipAngle = atan(
    (B.y - A.y) * screenShape.y,
    (B.x - A.x) * screenShape.x
  );

  vec2 offset = 0.5 * pixelRatio * lineWidth * vec2(
    sin(clipAngle),
    -cos(clipAngle)
  ) / screenShape;

  gl_Position = vec4(startPoint.xy + startPoint.w * offset, startPoint.zw);

  worldPosition = position;
  pixelArcLength = arcLength;
  fragColor = color;
}
`]), me = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3      clipBounds[2];
uniform sampler2D dashTexture;
uniform float     dashScale;
uniform float     opacity;

varying vec3    worldPosition;
varying float   pixelArcLength;
varying vec4    fragColor;

void main() {
  if (
    outOfRange(clipBounds[0], clipBounds[1], worldPosition) ||
    fragColor.a * opacity == 0.
  ) discard;

  float dashWeight = texture2D(dashTexture, vec2(dashScale * pixelArcLength, 0)).r;
  if(dashWeight < 0.5) {
    discard;
  }
  gl_FragColor = fragColor * opacity;
}
`]), pe = de([`precision highp float;
#define GLSLIFY 1

#define FLOAT_MAX  1.70141184e38
#define FLOAT_MIN  1.17549435e-38

// https://github.com/mikolalysenko/glsl-read-float/blob/master/index.glsl
vec4 packFloat(float v) {
  float av = abs(v);

  //Handle special cases
  if(av < FLOAT_MIN) {
    return vec4(0.0, 0.0, 0.0, 0.0);
  } else if(v > FLOAT_MAX) {
    return vec4(127.0, 128.0, 0.0, 0.0) / 255.0;
  } else if(v < -FLOAT_MAX) {
    return vec4(255.0, 128.0, 0.0, 0.0) / 255.0;
  }

  vec4 c = vec4(0,0,0,0);

  //Compute exponent and mantissa
  float e = floor(log2(av));
  float m = av * pow(2.0, -e) - 1.0;

  //Unpack mantissa
  c[1] = floor(128.0 * m);
  m -= c[1] / 128.0;
  c[2] = floor(32768.0 * m);
  m -= c[2] / 32768.0;
  c[3] = floor(8388608.0 * m);

  //Unpack exponent
  float ebias = e + 127.0;
  c[0] = floor(ebias / 2.0);
  ebias -= c[0] * 2.0;
  c[1] += floor(ebias) * 128.0;

  //Unpack sign bit
  c[0] += 128.0 * step(0.0, -v);

  //Scale back to range
  return c / 255.0;
}

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform float pickId;
uniform vec3 clipBounds[2];

varying vec3 worldPosition;
varying float pixelArcLength;
varying vec4 fragColor;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], worldPosition)) discard;

  gl_FragColor = vec4(pickId/255.0, packFloat(pixelArcLength).xyz);
}`]), ye = [{ name: "position", type: "vec3" }, { name: "nextPosition", type: "vec3" }, { name: "arcLength", type: "float" }, { name: "lineWidth", type: "float" }, { name: "color", type: "vec4" }];
                le.createShader = function(ge) {
                  return fe(ge, he, me, null, ye);
                }, le.createPickShader = function(ge) {
                  return fe(ge, he, pe, null, ye);
                };
              }, 5714: function(ce, le, ue) {
                ce.exports = function(ke) {
                  var Me = ke.gl || ke.scene && ke.scene.gl, Re = be(Me);
                  Re.attributes.position.location = 0, Re.attributes.nextPosition.location = 1, Re.attributes.arcLength.location = 2, Re.attributes.lineWidth.location = 3, Re.attributes.color.location = 4;
                  var De = Ce(Me);
                  De.attributes.position.location = 0, De.attributes.nextPosition.location = 1, De.attributes.arcLength.location = 2, De.attributes.lineWidth.location = 3, De.attributes.color.location = 4;
                  for (var Pe = de(Me), $e = fe(Me, [{ buffer: Pe, size: 3, offset: 0, stride: 48 }, { buffer: Pe, size: 3, offset: 12, stride: 48 }, { buffer: Pe, size: 1, offset: 24, stride: 48 }, { buffer: Pe, size: 1, offset: 28, stride: 48 }, { buffer: Pe, size: 4, offset: 32, stride: 48 }]), qe = ge(new Array(1024), [256, 1, 4]), Xe = 0; Xe < 1024; ++Xe) qe.data[Xe] = 255;
                  var Ve = he(Me, qe);
                  Ve.wrap = Me.REPEAT;
                  var Ze = new Le(Me, Re, De, Pe, $e, Ve);
                  return Ze.update(ke), Ze;
                };
                var de = ue(2762), fe = ue(8116), he = ue(7766), me = new Uint8Array(4), pe = new Float32Array(me.buffer), ye = ue(2478), ge = ue(9618), ve = ue(7319), be = ve.createShader, Ce = ve.createPickShader, Se = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
                function we(ke, Me) {
                  for (var Re = 0, De = 0; De < 3; ++De) {
                    var Pe = ke[De] - Me[De];
                    Re += Pe * Pe;
                  }
                  return Math.sqrt(Re);
                }
                function Ee(ke) {
                  for (var Me = [[-1e6, -1e6, -1e6], [1e6, 1e6, 1e6]], Re = 0; Re < 3; ++Re) Me[0][Re] = Math.max(ke[0][Re], Me[0][Re]), Me[1][Re] = Math.min(ke[1][Re], Me[1][Re]);
                  return Me;
                }
                function xe(ke, Me, Re, De) {
                  this.arcLength = ke, this.position = Me, this.index = Re, this.dataCoordinate = De;
                }
                function Le(ke, Me, Re, De, Pe, $e) {
                  this.gl = ke, this.shader = Me, this.pickShader = Re, this.buffer = De, this.vao = Pe, this.clipBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.points = [], this.arcLength = [], this.vertexCount = 0, this.bounds = [[0, 0, 0], [0, 0, 0]], this.pickId = 0, this.lineWidth = 1, this.texture = $e, this.dashScale = 1, this.opacity = 1, this.hasAlpha = !1, this.dirty = !0, this.pixelRatio = 1;
                }
                var Qe = Le.prototype;
                Qe.isTransparent = function() {
                  return this.hasAlpha;
                }, Qe.isOpaque = function() {
                  return !this.hasAlpha;
                }, Qe.pickSlots = 1, Qe.setPickBase = function(ke) {
                  this.pickId = ke;
                }, Qe.drawTransparent = Qe.draw = function(ke) {
                  if (this.vertexCount) {
                    var Me = this.gl, Re = this.shader, De = this.vao;
                    Re.bind(), Re.uniforms = { model: ke.model || Se, view: ke.view || Se, projection: ke.projection || Se, clipBounds: Ee(this.clipBounds), dashTexture: this.texture.bind(), dashScale: this.dashScale / this.arcLength[this.arcLength.length - 1], opacity: this.opacity, screenShape: [Me.drawingBufferWidth, Me.drawingBufferHeight], pixelRatio: this.pixelRatio }, De.bind(), De.draw(Me.TRIANGLE_STRIP, this.vertexCount), De.unbind();
                  }
                }, Qe.drawPick = function(ke) {
                  if (this.vertexCount) {
                    var Me = this.gl, Re = this.pickShader, De = this.vao;
                    Re.bind(), Re.uniforms = { model: ke.model || Se, view: ke.view || Se, projection: ke.projection || Se, pickId: this.pickId, clipBounds: Ee(this.clipBounds), screenShape: [Me.drawingBufferWidth, Me.drawingBufferHeight], pixelRatio: this.pixelRatio }, De.bind(), De.draw(Me.TRIANGLE_STRIP, this.vertexCount), De.unbind();
                  }
                }, Qe.update = function(ke) {
                  var Me, Re;
                  this.dirty = !0;
                  var De = !!ke.connectGaps;
                  "dashScale" in ke && (this.dashScale = ke.dashScale), this.hasAlpha = !1, "opacity" in ke && (this.opacity = +ke.opacity, this.opacity < 1 && (this.hasAlpha = !0));
                  var Pe = [], $e = [], qe = [], Xe = 0, Ve = 0, Ze = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], Ge = ke.position || ke.positions;
                  if (Ge) {
                    var ze = ke.color || ke.colors || [0, 0, 0, 1], We = ke.lineWidth || 1, at = !1;
                    e: for (Me = 1; Me < Ge.length; ++Me) {
                      var ct, et, dt, Je = Ge[Me - 1], it = Ge[Me];
                      for ($e.push(Xe), qe.push(Je.slice()), Re = 0; Re < 3; ++Re) {
                        if (isNaN(Je[Re]) || isNaN(it[Re]) || !isFinite(Je[Re]) || !isFinite(it[Re])) {
                          if (!De && Pe.length > 0) {
                            for (var st = 0; st < 24; ++st) Pe.push(Pe[Pe.length - 12]);
                            Ve += 2, at = !0;
                          }
                          continue e;
                        }
                        Ze[0][Re] = Math.min(Ze[0][Re], Je[Re], it[Re]), Ze[1][Re] = Math.max(Ze[1][Re], Je[Re], it[Re]);
                      }
                      Array.isArray(ze[0]) ? (ct = ze.length > Me - 1 ? ze[Me - 1] : ze.length > 0 ? ze[ze.length - 1] : [0, 0, 0, 1], et = ze.length > Me ? ze[Me] : ze.length > 0 ? ze[ze.length - 1] : [0, 0, 0, 1]) : ct = et = ze, ct.length === 3 && (ct = [ct[0], ct[1], ct[2], 1]), et.length === 3 && (et = [et[0], et[1], et[2], 1]), !this.hasAlpha && ct[3] < 1 && (this.hasAlpha = !0), dt = Array.isArray(We) ? We.length > Me - 1 ? We[Me - 1] : We.length > 0 ? We[We.length - 1] : [0, 0, 0, 1] : We;
                      var gt = Xe;
                      if (Xe += we(Je, it), at) {
                        for (Re = 0; Re < 2; ++Re) Pe.push(Je[0], Je[1], Je[2], it[0], it[1], it[2], gt, dt, ct[0], ct[1], ct[2], ct[3]);
                        Ve += 2, at = !1;
                      }
                      Pe.push(Je[0], Je[1], Je[2], it[0], it[1], it[2], gt, dt, ct[0], ct[1], ct[2], ct[3], Je[0], Je[1], Je[2], it[0], it[1], it[2], gt, -dt, ct[0], ct[1], ct[2], ct[3], it[0], it[1], it[2], Je[0], Je[1], Je[2], Xe, -dt, et[0], et[1], et[2], et[3], it[0], it[1], it[2], Je[0], Je[1], Je[2], Xe, dt, et[0], et[1], et[2], et[3]), Ve += 4;
                    }
                  }
                  if (this.buffer.update(Pe), $e.push(Xe), qe.push(Ge[Ge.length - 1].slice()), this.bounds = Ze, this.vertexCount = Ve, this.points = qe, this.arcLength = $e, "dashes" in ke) {
                    var bt = ke.dashes.slice();
                    for (bt.unshift(0), Me = 1; Me < bt.length; ++Me) bt[Me] = bt[Me - 1] + bt[Me];
                    var yt = ge(new Array(1024), [256, 1, 4]);
                    for (Me = 0; Me < 256; ++Me) {
                      for (Re = 0; Re < 4; ++Re) yt.set(Me, 0, Re, 0);
                      1 & ye.le(bt, bt[bt.length - 1] * Me / 255) ? yt.set(Me, 0, 0, 0) : yt.set(Me, 0, 0, 255);
                    }
                    this.texture.setPixels(yt);
                  }
                }, Qe.dispose = function() {
                  this.shader.dispose(), this.vao.dispose(), this.buffer.dispose();
                }, Qe.pick = function(ke) {
                  if (!ke || ke.id !== this.pickId) return null;
                  var Me = function(Ge, ze, We, at) {
                    return me[0] = 0, me[1] = We, me[2] = ze, me[3] = Ge, pe[0];
                  }(ke.value[0], ke.value[1], ke.value[2]), Re = ye.le(this.arcLength, Me);
                  if (Re < 0) return null;
                  if (Re === this.arcLength.length - 1) return new xe(this.arcLength[this.arcLength.length - 1], this.points[this.points.length - 1].slice(), Re);
                  for (var De = this.points[Re], Pe = this.points[Math.min(Re + 1, this.points.length - 1)], $e = (Me - this.arcLength[Re]) / (this.arcLength[Re + 1] - this.arcLength[Re]), qe = 1 - $e, Xe = [0, 0, 0], Ve = 0; Ve < 3; ++Ve) Xe[Ve] = qe * De[Ve] + $e * Pe[Ve];
                  var Ze = Math.min($e < 0.5 ? Re : Re + 1, this.points.length - 1);
                  return new xe(Me, Xe, Ze, this.points[Ze]);
                };
              }, 1903: function(ce) {
                ce.exports = function(le) {
                  var ue = new Float32Array(16);
                  return ue[0] = le[0], ue[1] = le[1], ue[2] = le[2], ue[3] = le[3], ue[4] = le[4], ue[5] = le[5], ue[6] = le[6], ue[7] = le[7], ue[8] = le[8], ue[9] = le[9], ue[10] = le[10], ue[11] = le[11], ue[12] = le[12], ue[13] = le[13], ue[14] = le[14], ue[15] = le[15], ue;
                };
              }, 6864: function(ce) {
                ce.exports = function() {
                  var le = new Float32Array(16);
                  return le[0] = 1, le[1] = 0, le[2] = 0, le[3] = 0, le[4] = 0, le[5] = 1, le[6] = 0, le[7] = 0, le[8] = 0, le[9] = 0, le[10] = 1, le[11] = 0, le[12] = 0, le[13] = 0, le[14] = 0, le[15] = 1, le;
                };
              }, 9921: function(ce) {
                ce.exports = function(le) {
                  var ue = le[0], de = le[1], fe = le[2], he = le[3], me = le[4], pe = le[5], ye = le[6], ge = le[7], ve = le[8], be = le[9], Ce = le[10], Se = le[11], we = le[12], Ee = le[13], xe = le[14], Le = le[15];
                  return (ue * pe - de * me) * (Ce * Le - Se * xe) - (ue * ye - fe * me) * (be * Le - Se * Ee) + (ue * ge - he * me) * (be * xe - Ce * Ee) + (de * ye - fe * pe) * (ve * Le - Se * we) - (de * ge - he * pe) * (ve * xe - Ce * we) + (fe * ge - he * ye) * (ve * Ee - be * we);
                };
              }, 7399: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0], fe = ue[1], he = ue[2], me = ue[3], pe = de + de, ye = fe + fe, ge = he + he, ve = de * pe, be = fe * pe, Ce = fe * ye, Se = he * pe, we = he * ye, Ee = he * ge, xe = me * pe, Le = me * ye, Qe = me * ge;
                  return le[0] = 1 - Ce - Ee, le[1] = be + Qe, le[2] = Se - Le, le[3] = 0, le[4] = be - Qe, le[5] = 1 - ve - Ee, le[6] = we + xe, le[7] = 0, le[8] = Se + Le, le[9] = we - xe, le[10] = 1 - ve - Ce, le[11] = 0, le[12] = 0, le[13] = 0, le[14] = 0, le[15] = 1, le;
                };
              }, 6743: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2], pe = ue[3], ye = fe + fe, ge = he + he, ve = me + me, be = fe * ye, Ce = fe * ge, Se = fe * ve, we = he * ge, Ee = he * ve, xe = me * ve, Le = pe * ye, Qe = pe * ge, ke = pe * ve;
                  return le[0] = 1 - (we + xe), le[1] = Ce + ke, le[2] = Se - Qe, le[3] = 0, le[4] = Ce - ke, le[5] = 1 - (be + xe), le[6] = Ee + Le, le[7] = 0, le[8] = Se + Qe, le[9] = Ee - Le, le[10] = 1 - (be + we), le[11] = 0, le[12] = de[0], le[13] = de[1], le[14] = de[2], le[15] = 1, le;
                };
              }, 7894: function(ce) {
                ce.exports = function(le) {
                  return le[0] = 1, le[1] = 0, le[2] = 0, le[3] = 0, le[4] = 0, le[5] = 1, le[6] = 0, le[7] = 0, le[8] = 0, le[9] = 0, le[10] = 1, le[11] = 0, le[12] = 0, le[13] = 0, le[14] = 0, le[15] = 1, le;
                };
              }, 7608: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0], fe = ue[1], he = ue[2], me = ue[3], pe = ue[4], ye = ue[5], ge = ue[6], ve = ue[7], be = ue[8], Ce = ue[9], Se = ue[10], we = ue[11], Ee = ue[12], xe = ue[13], Le = ue[14], Qe = ue[15], ke = de * ye - fe * pe, Me = de * ge - he * pe, Re = de * ve - me * pe, De = fe * ge - he * ye, Pe = fe * ve - me * ye, $e = he * ve - me * ge, qe = be * xe - Ce * Ee, Xe = be * Le - Se * Ee, Ve = be * Qe - we * Ee, Ze = Ce * Le - Se * xe, Ge = Ce * Qe - we * xe, ze = Se * Qe - we * Le, We = ke * ze - Me * Ge + Re * Ze + De * Ve - Pe * Xe + $e * qe;
                  return We ? (We = 1 / We, le[0] = (ye * ze - ge * Ge + ve * Ze) * We, le[1] = (he * Ge - fe * ze - me * Ze) * We, le[2] = (xe * $e - Le * Pe + Qe * De) * We, le[3] = (Se * Pe - Ce * $e - we * De) * We, le[4] = (ge * Ve - pe * ze - ve * Xe) * We, le[5] = (de * ze - he * Ve + me * Xe) * We, le[6] = (Le * Re - Ee * $e - Qe * Me) * We, le[7] = (be * $e - Se * Re + we * Me) * We, le[8] = (pe * Ge - ye * Ve + ve * qe) * We, le[9] = (fe * Ve - de * Ge - me * qe) * We, le[10] = (Ee * Pe - xe * Re + Qe * ke) * We, le[11] = (Ce * Re - be * Pe - we * ke) * We, le[12] = (ye * Xe - pe * Ze - ge * qe) * We, le[13] = (de * Ze - fe * Xe + he * qe) * We, le[14] = (xe * Me - Ee * De - Le * ke) * We, le[15] = (be * De - Ce * Me + Se * ke) * We, le) : null;
                };
              }, 6582: function(ce, le, ue) {
                var de = ue(7894);
                ce.exports = function(fe, he, me, pe) {
                  var ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe = he[0], ke = he[1], Me = he[2], Re = pe[0], De = pe[1], Pe = pe[2], $e = me[0], qe = me[1], Xe = me[2];
                  return Math.abs(Qe - $e) < 1e-6 && Math.abs(ke - qe) < 1e-6 && Math.abs(Me - Xe) < 1e-6 ? de(fe) : (we = Qe - $e, Ee = ke - qe, xe = Me - Xe, ye = De * (xe *= Le = 1 / Math.sqrt(we * we + Ee * Ee + xe * xe)) - Pe * (Ee *= Le), ge = Pe * (we *= Le) - Re * xe, ve = Re * Ee - De * we, (Le = Math.sqrt(ye * ye + ge * ge + ve * ve)) ? (ye *= Le = 1 / Le, ge *= Le, ve *= Le) : (ye = 0, ge = 0, ve = 0), be = Ee * ve - xe * ge, Ce = xe * ye - we * ve, Se = we * ge - Ee * ye, (Le = Math.sqrt(be * be + Ce * Ce + Se * Se)) ? (be *= Le = 1 / Le, Ce *= Le, Se *= Le) : (be = 0, Ce = 0, Se = 0), fe[0] = ye, fe[1] = be, fe[2] = we, fe[3] = 0, fe[4] = ge, fe[5] = Ce, fe[6] = Ee, fe[7] = 0, fe[8] = ve, fe[9] = Se, fe[10] = xe, fe[11] = 0, fe[12] = -(ye * Qe + ge * ke + ve * Me), fe[13] = -(be * Qe + Ce * ke + Se * Me), fe[14] = -(we * Qe + Ee * ke + xe * Me), fe[15] = 1, fe);
                };
              }, 6760: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2], pe = ue[3], ye = ue[4], ge = ue[5], ve = ue[6], be = ue[7], Ce = ue[8], Se = ue[9], we = ue[10], Ee = ue[11], xe = ue[12], Le = ue[13], Qe = ue[14], ke = ue[15], Me = de[0], Re = de[1], De = de[2], Pe = de[3];
                  return le[0] = Me * fe + Re * ye + De * Ce + Pe * xe, le[1] = Me * he + Re * ge + De * Se + Pe * Le, le[2] = Me * me + Re * ve + De * we + Pe * Qe, le[3] = Me * pe + Re * be + De * Ee + Pe * ke, Me = de[4], Re = de[5], De = de[6], Pe = de[7], le[4] = Me * fe + Re * ye + De * Ce + Pe * xe, le[5] = Me * he + Re * ge + De * Se + Pe * Le, le[6] = Me * me + Re * ve + De * we + Pe * Qe, le[7] = Me * pe + Re * be + De * Ee + Pe * ke, Me = de[8], Re = de[9], De = de[10], Pe = de[11], le[8] = Me * fe + Re * ye + De * Ce + Pe * xe, le[9] = Me * he + Re * ge + De * Se + Pe * Le, le[10] = Me * me + Re * ve + De * we + Pe * Qe, le[11] = Me * pe + Re * be + De * Ee + Pe * ke, Me = de[12], Re = de[13], De = de[14], Pe = de[15], le[12] = Me * fe + Re * ye + De * Ce + Pe * xe, le[13] = Me * he + Re * ge + De * Se + Pe * Le, le[14] = Me * me + Re * ve + De * we + Pe * Qe, le[15] = Me * pe + Re * be + De * Ee + Pe * ke, le;
                };
              }, 4040: function(ce) {
                ce.exports = function(le, ue, de, fe, he, me, pe) {
                  var ye = 1 / (ue - de), ge = 1 / (fe - he), ve = 1 / (me - pe);
                  return le[0] = -2 * ye, le[1] = 0, le[2] = 0, le[3] = 0, le[4] = 0, le[5] = -2 * ge, le[6] = 0, le[7] = 0, le[8] = 0, le[9] = 0, le[10] = 2 * ve, le[11] = 0, le[12] = (ue + de) * ye, le[13] = (he + fe) * ge, le[14] = (pe + me) * ve, le[15] = 1, le;
                };
              }, 4772: function(ce) {
                ce.exports = function(le, ue, de, fe, he) {
                  var me = 1 / Math.tan(ue / 2), pe = 1 / (fe - he);
                  return le[0] = me / de, le[1] = 0, le[2] = 0, le[3] = 0, le[4] = 0, le[5] = me, le[6] = 0, le[7] = 0, le[8] = 0, le[9] = 0, le[10] = (he + fe) * pe, le[11] = -1, le[12] = 0, le[13] = 0, le[14] = 2 * he * fe * pe, le[15] = 0, le;
                };
              }, 6079: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze, Ge = fe[0], ze = fe[1], We = fe[2], at = Math.sqrt(Ge * Ge + ze * ze + We * We);
                  return Math.abs(at) < 1e-6 ? null : (Ge *= at = 1 / at, ze *= at, We *= at, he = Math.sin(de), pe = 1 - (me = Math.cos(de)), ye = ue[0], ge = ue[1], ve = ue[2], be = ue[3], Ce = ue[4], Se = ue[5], we = ue[6], Ee = ue[7], xe = ue[8], Le = ue[9], Qe = ue[10], ke = ue[11], Me = Ge * Ge * pe + me, Re = ze * Ge * pe + We * he, De = We * Ge * pe - ze * he, Pe = Ge * ze * pe - We * he, $e = ze * ze * pe + me, qe = We * ze * pe + Ge * he, Xe = Ge * We * pe + ze * he, Ve = ze * We * pe - Ge * he, Ze = We * We * pe + me, le[0] = ye * Me + Ce * Re + xe * De, le[1] = ge * Me + Se * Re + Le * De, le[2] = ve * Me + we * Re + Qe * De, le[3] = be * Me + Ee * Re + ke * De, le[4] = ye * Pe + Ce * $e + xe * qe, le[5] = ge * Pe + Se * $e + Le * qe, le[6] = ve * Pe + we * $e + Qe * qe, le[7] = be * Pe + Ee * $e + ke * qe, le[8] = ye * Xe + Ce * Ve + xe * Ze, le[9] = ge * Xe + Se * Ve + Le * Ze, le[10] = ve * Xe + we * Ve + Qe * Ze, le[11] = be * Xe + Ee * Ve + ke * Ze, ue !== le && (le[12] = ue[12], le[13] = ue[13], le[14] = ue[14], le[15] = ue[15]), le);
                };
              }, 5567: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = Math.sin(de), he = Math.cos(de), me = ue[4], pe = ue[5], ye = ue[6], ge = ue[7], ve = ue[8], be = ue[9], Ce = ue[10], Se = ue[11];
                  return ue !== le && (le[0] = ue[0], le[1] = ue[1], le[2] = ue[2], le[3] = ue[3], le[12] = ue[12], le[13] = ue[13], le[14] = ue[14], le[15] = ue[15]), le[4] = me * he + ve * fe, le[5] = pe * he + be * fe, le[6] = ye * he + Ce * fe, le[7] = ge * he + Se * fe, le[8] = ve * he - me * fe, le[9] = be * he - pe * fe, le[10] = Ce * he - ye * fe, le[11] = Se * he - ge * fe, le;
                };
              }, 2408: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = Math.sin(de), he = Math.cos(de), me = ue[0], pe = ue[1], ye = ue[2], ge = ue[3], ve = ue[8], be = ue[9], Ce = ue[10], Se = ue[11];
                  return ue !== le && (le[4] = ue[4], le[5] = ue[5], le[6] = ue[6], le[7] = ue[7], le[12] = ue[12], le[13] = ue[13], le[14] = ue[14], le[15] = ue[15]), le[0] = me * he - ve * fe, le[1] = pe * he - be * fe, le[2] = ye * he - Ce * fe, le[3] = ge * he - Se * fe, le[8] = me * fe + ve * he, le[9] = pe * fe + be * he, le[10] = ye * fe + Ce * he, le[11] = ge * fe + Se * he, le;
                };
              }, 7089: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = Math.sin(de), he = Math.cos(de), me = ue[0], pe = ue[1], ye = ue[2], ge = ue[3], ve = ue[4], be = ue[5], Ce = ue[6], Se = ue[7];
                  return ue !== le && (le[8] = ue[8], le[9] = ue[9], le[10] = ue[10], le[11] = ue[11], le[12] = ue[12], le[13] = ue[13], le[14] = ue[14], le[15] = ue[15]), le[0] = me * he + ve * fe, le[1] = pe * he + be * fe, le[2] = ye * he + Ce * fe, le[3] = ge * he + Se * fe, le[4] = ve * he - me * fe, le[5] = be * he - pe * fe, le[6] = Ce * he - ye * fe, le[7] = Se * he - ge * fe, le;
                };
              }, 2504: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = de[0], he = de[1], me = de[2];
                  return le[0] = ue[0] * fe, le[1] = ue[1] * fe, le[2] = ue[2] * fe, le[3] = ue[3] * fe, le[4] = ue[4] * he, le[5] = ue[5] * he, le[6] = ue[6] * he, le[7] = ue[7] * he, le[8] = ue[8] * me, le[9] = ue[9] * me, le[10] = ue[10] * me, le[11] = ue[11] * me, le[12] = ue[12], le[13] = ue[13], le[14] = ue[14], le[15] = ue[15], le;
                };
              }, 7656: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe, he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe = de[0], Le = de[1], Qe = de[2];
                  return ue === le ? (le[12] = ue[0] * xe + ue[4] * Le + ue[8] * Qe + ue[12], le[13] = ue[1] * xe + ue[5] * Le + ue[9] * Qe + ue[13], le[14] = ue[2] * xe + ue[6] * Le + ue[10] * Qe + ue[14], le[15] = ue[3] * xe + ue[7] * Le + ue[11] * Qe + ue[15]) : (fe = ue[0], he = ue[1], me = ue[2], pe = ue[3], ye = ue[4], ge = ue[5], ve = ue[6], be = ue[7], Ce = ue[8], Se = ue[9], we = ue[10], Ee = ue[11], le[0] = fe, le[1] = he, le[2] = me, le[3] = pe, le[4] = ye, le[5] = ge, le[6] = ve, le[7] = be, le[8] = Ce, le[9] = Se, le[10] = we, le[11] = Ee, le[12] = fe * xe + ye * Le + Ce * Qe + ue[12], le[13] = he * xe + ge * Le + Se * Qe + ue[13], le[14] = me * xe + ve * Le + we * Qe + ue[14], le[15] = pe * xe + be * Le + Ee * Qe + ue[15]), le;
                };
              }, 5665: function(ce) {
                ce.exports = function(le, ue) {
                  if (le === ue) {
                    var de = ue[1], fe = ue[2], he = ue[3], me = ue[6], pe = ue[7], ye = ue[11];
                    le[1] = ue[4], le[2] = ue[8], le[3] = ue[12], le[4] = de, le[6] = ue[9], le[7] = ue[13], le[8] = fe, le[9] = me, le[11] = ue[14], le[12] = he, le[13] = pe, le[14] = ye;
                  } else le[0] = ue[0], le[1] = ue[4], le[2] = ue[8], le[3] = ue[12], le[4] = ue[1], le[5] = ue[5], le[6] = ue[9], le[7] = ue[13], le[8] = ue[2], le[9] = ue[6], le[10] = ue[10], le[11] = ue[14], le[12] = ue[3], le[13] = ue[7], le[14] = ue[11], le[15] = ue[15];
                  return le;
                };
              }, 7626: function(ce, le, ue) {
                var de = ue(2642), fe = ue(9346);
                function he(ye, ge) {
                  for (var ve = [0, 0, 0, 0], be = 0; be < 4; ++be) for (var Ce = 0; Ce < 4; ++Ce) ve[Ce] += ye[4 * be + Ce] * ge[be];
                  return ve;
                }
                function me(ye, ge, ve, be, Ce) {
                  for (var Se = he(be, he(ve, he(ge, [ye[0], ye[1], ye[2], 1]))), we = 0; we < 3; ++we) Se[we] /= Se[3];
                  return [0.5 * Ce[0] * (1 + Se[0]), 0.5 * Ce[1] * (1 - Se[1])];
                }
                function pe(ye, ge) {
                  for (var ve = [0, 0, 0], be = 0; be < ye.length; ++be) for (var Ce = ye[be], Se = ge[be], we = 0; we < 3; ++we) ve[we] += Se * Ce[we];
                  return ve;
                }
                ce.exports = function(ye, ge, ve, be, Ce, Se) {
                  if (ye.length === 1) return [0, ye[0].slice()];
                  for (var we = new Array(ye.length), Ee = 0; Ee < ye.length; ++Ee) we[Ee] = me(ye[Ee], ve, be, Ce, Se);
                  var xe = 0, Le = 1 / 0;
                  for (Ee = 0; Ee < we.length; ++Ee) {
                    for (var Qe = 0, ke = 0; ke < 2; ++ke) Qe += Math.pow(we[Ee][ke] - ge[ke], 2);
                    Qe < Le && (Le = Qe, xe = Ee);
                  }
                  var Me = function(De, Pe) {
                    if (De.length === 2) {
                      for (var $e = 0, qe = 0, Xe = 0; Xe < 2; ++Xe) $e += Math.pow(Pe[Xe] - De[0][Xe], 2), qe += Math.pow(Pe[Xe] - De[1][Xe], 2);
                      return ($e = Math.sqrt($e)) + (qe = Math.sqrt(qe)) < 1e-6 ? [1, 0] : [qe / ($e + qe), $e / (qe + $e)];
                    }
                    if (De.length === 3) {
                      var Ve = [0, 0];
                      return fe(De[0], De[1], De[2], Pe, Ve), de(De, Ve);
                    }
                    return [];
                  }(we, ge), Re = 0;
                  for (Ee = 0; Ee < 3; ++Ee) {
                    if (Me[Ee] < -1e-3 || Me[Ee] > 1.0001) return null;
                    Re += Me[Ee];
                  }
                  return Math.abs(Re - 1) > 1e-3 ? null : [xe, pe(ye, Me), Me];
                };
              }, 840: function(ce, le, ue) {
                var de = ue(3236), fe = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position, normal;
attribute vec4 color;
attribute vec2 uv;

uniform mat4 model
           , view
           , projection
           , inverseModel;
uniform vec3 eyePosition
           , lightPosition;

varying vec3 f_normal
           , f_lightDirection
           , f_eyeDirection
           , f_data;
varying vec4 f_color;
varying vec2 f_uv;

vec4 project(vec3 p) {
  return projection * (view * (model * vec4(p, 1.0)));
}

void main() {
  gl_Position      = project(position);

  //Lighting geometry parameters
  vec4 cameraCoordinate = view * vec4(position , 1.0);
  cameraCoordinate.xyz /= cameraCoordinate.w;
  f_lightDirection = lightPosition - cameraCoordinate.xyz;
  f_eyeDirection   = eyePosition - cameraCoordinate.xyz;
  f_normal  = normalize((vec4(normal, 0.0) * inverseModel).xyz);

  f_color          = color;
  f_data           = position;
  f_uv             = uv;
}
`]), he = de([`#extension GL_OES_standard_derivatives : enable

precision highp float;
#define GLSLIFY 1

float beckmannDistribution(float x, float roughness) {
  float NdotH = max(x, 0.0001);
  float cos2Alpha = NdotH * NdotH;
  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;
  float roughness2 = roughness * roughness;
  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;
  return exp(tan2Alpha / roughness2) / denom;
}

float cookTorranceSpecular(
  vec3 lightDirection,
  vec3 viewDirection,
  vec3 surfaceNormal,
  float roughness,
  float fresnel) {

  float VdotN = max(dot(viewDirection, surfaceNormal), 0.0);
  float LdotN = max(dot(lightDirection, surfaceNormal), 0.0);

  //Half angle vector
  vec3 H = normalize(lightDirection + viewDirection);

  //Geometric term
  float NdotH = max(dot(surfaceNormal, H), 0.0);
  float VdotH = max(dot(viewDirection, H), 0.000001);
  float LdotH = max(dot(lightDirection, H), 0.000001);
  float G1 = (2.0 * NdotH * VdotN) / VdotH;
  float G2 = (2.0 * NdotH * LdotN) / LdotH;
  float G = min(1.0, min(G1, G2));
  
  //Distribution term
  float D = beckmannDistribution(NdotH, roughness);

  //Fresnel term
  float F = pow(1.0 - VdotN, fresnel);

  //Multiply terms and done
  return  G * F * D / max(3.14159265 * VdotN, 0.000001);
}

//#pragma glslify: beckmann = require(glsl-specular-beckmann) // used in gl-surface3d

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 clipBounds[2];
uniform float roughness
            , fresnel
            , kambient
            , kdiffuse
            , kspecular;
uniform sampler2D texture;

varying vec3 f_normal
           , f_lightDirection
           , f_eyeDirection
           , f_data;
varying vec4 f_color;
varying vec2 f_uv;

void main() {
  if (f_color.a == 0.0 ||
    outOfRange(clipBounds[0], clipBounds[1], f_data)
  ) discard;

  vec3 N = normalize(f_normal);
  vec3 L = normalize(f_lightDirection);
  vec3 V = normalize(f_eyeDirection);

  if(gl_FrontFacing) {
    N = -N;
  }

  float specular = min(1.0, max(0.0, cookTorranceSpecular(L, V, N, roughness, fresnel)));
  //float specular = max(0.0, beckmann(L, V, N, roughness)); // used in gl-surface3d

  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);

  vec4 surfaceColor = vec4(f_color.rgb, 1.0) * texture2D(texture, f_uv);
  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);

  gl_FragColor = litColor * f_color.a;
}
`]), me = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position;
attribute vec4 color;
attribute vec2 uv;

uniform mat4 model, view, projection;

varying vec4 f_color;
varying vec3 f_data;
varying vec2 f_uv;

void main() {
  gl_Position = projection * (view * (model * vec4(position, 1.0)));
  f_color = color;
  f_data  = position;
  f_uv    = uv;
}`]), pe = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 clipBounds[2];
uniform sampler2D texture;
uniform float opacity;

varying vec4 f_color;
varying vec3 f_data;
varying vec2 f_uv;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], f_data)) discard;

  gl_FragColor = f_color * texture2D(texture, f_uv) * opacity;
}`]), ye = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

attribute vec3 position;
attribute vec4 color;
attribute vec2 uv;
attribute float pointSize;

uniform mat4 model, view, projection;
uniform vec3 clipBounds[2];

varying vec4 f_color;
varying vec2 f_uv;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], position)) {

    gl_Position = vec4(0.0, 0.0 ,0.0 ,0.0);
  } else {
    gl_Position = projection * (view * (model * vec4(position, 1.0)));
  }
  gl_PointSize = pointSize;
  f_color = color;
  f_uv = uv;
}`]), ge = de([`precision highp float;
#define GLSLIFY 1

uniform sampler2D texture;
uniform float opacity;

varying vec4 f_color;
varying vec2 f_uv;

void main() {
  vec2 pointR = gl_PointCoord.xy - vec2(0.5, 0.5);
  if(dot(pointR, pointR) > 0.25) {
    discard;
  }
  gl_FragColor = f_color * texture2D(texture, f_uv) * opacity;
}`]), ve = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position;
attribute vec4 id;

uniform mat4 model, view, projection;

varying vec3 f_position;
varying vec4 f_id;

void main() {
  gl_Position = projection * (view * (model * vec4(position, 1.0)));
  f_id        = id;
  f_position  = position;
}`]), be = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3  clipBounds[2];
uniform float pickId;

varying vec3 f_position;
varying vec4 f_id;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;

  gl_FragColor = vec4(pickId, f_id.xyz);
}`]), Ce = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

attribute vec3  position;
attribute float pointSize;
attribute vec4  id;

uniform mat4 model, view, projection;
uniform vec3 clipBounds[2];

varying vec3 f_position;
varying vec4 f_id;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], position)) {

    gl_Position = vec4(0.0, 0.0, 0.0, 0.0);
  } else {
    gl_Position  = projection * (view * (model * vec4(position, 1.0)));
    gl_PointSize = pointSize;
  }
  f_id         = id;
  f_position   = position;
}`]), Se = de([`precision highp float;
#define GLSLIFY 1

attribute vec3 position;

uniform mat4 model, view, projection;

void main() {
  gl_Position = projection * (view * (model * vec4(position, 1.0)));
}`]), we = de([`precision highp float;
#define GLSLIFY 1

uniform vec3 contourColor;

void main() {
  gl_FragColor = vec4(contourColor, 1.0);
}
`]);
                le.meshShader = { vertex: fe, fragment: he, attributes: [{ name: "position", type: "vec3" }, { name: "normal", type: "vec3" }, { name: "color", type: "vec4" }, { name: "uv", type: "vec2" }] }, le.wireShader = { vertex: me, fragment: pe, attributes: [{ name: "position", type: "vec3" }, { name: "color", type: "vec4" }, { name: "uv", type: "vec2" }] }, le.pointShader = { vertex: ye, fragment: ge, attributes: [{ name: "position", type: "vec3" }, { name: "color", type: "vec4" }, { name: "uv", type: "vec2" }, { name: "pointSize", type: "float" }] }, le.pickShader = { vertex: ve, fragment: be, attributes: [{ name: "position", type: "vec3" }, { name: "id", type: "vec4" }] }, le.pointPickShader = { vertex: Ce, fragment: be, attributes: [{ name: "position", type: "vec3" }, { name: "pointSize", type: "float" }, { name: "id", type: "vec4" }] }, le.contourShader = { vertex: Se, fragment: we, attributes: [{ name: "position", type: "vec3" }] };
              }, 7201: function(ce, le, ue) {
                var de = ue(9405), fe = ue(2762), he = ue(8116), me = ue(7766), pe = ue(8406), ye = ue(6760), ge = ue(7608), ve = ue(9618), be = ue(6729), Ce = ue(7765), Se = ue(1888), we = ue(840), Ee = ue(7626), xe = we.meshShader, Le = we.wireShader, Qe = we.pointShader, ke = we.pickShader, Me = we.pointPickShader, Re = we.contourShader, De = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
                function Pe(ze, We, at, ct, et, dt, Je, it, st, gt, bt, yt, St, Lt, Rt, vt, $t, Pt, Xt, Nt, Gt, Ft, Tt, It, Ct, Kt, Vt) {
                  this.gl = ze, this.pixelRatio = 1, this.cells = [], this.positions = [], this.intensity = [], this.texture = We, this.dirty = !0, this.triShader = at, this.lineShader = ct, this.pointShader = et, this.pickShader = dt, this.pointPickShader = Je, this.contourShader = it, this.trianglePositions = st, this.triangleColors = bt, this.triangleNormals = St, this.triangleUVs = yt, this.triangleIds = gt, this.triangleVAO = Lt, this.triangleCount = 0, this.lineWidth = 1, this.edgePositions = Rt, this.edgeColors = $t, this.edgeUVs = Pt, this.edgeIds = vt, this.edgeVAO = Xt, this.edgeCount = 0, this.pointPositions = Nt, this.pointColors = Ft, this.pointUVs = Tt, this.pointSizes = It, this.pointIds = Gt, this.pointVAO = Ct, this.pointCount = 0, this.contourLineWidth = 1, this.contourPositions = Kt, this.contourVAO = Vt, this.contourCount = 0, this.contourColor = [0, 0, 0], this.contourEnable = !0, this.pickVertex = !0, this.pickId = 1, this.bounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], this.clipBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.lightPosition = [1e5, 1e5, 0], this.ambientLight = 0.8, this.diffuseLight = 0.8, this.specularLight = 2, this.roughness = 0.5, this.fresnel = 1.5, this.opacity = 1, this.hasAlpha = !1, this.opacityscale = !1, this._model = De, this._view = De, this._projection = De, this._resolution = [1, 1];
                }
                var $e = Pe.prototype;
                function qe(ze, We) {
                  if (!We || !We.length) return 1;
                  for (var at = 0; at < We.length; ++at) {
                    if (We.length < 2) return 1;
                    if (We[at][0] === ze) return We[at][1];
                    if (We[at][0] > ze && at > 0) {
                      var ct = (We[at][0] - ze) / (We[at][0] - We[at - 1][0]);
                      return We[at][1] * (1 - ct) + ct * We[at - 1][1];
                    }
                  }
                  return 1;
                }
                function Xe(ze) {
                  var We = de(ze, Qe.vertex, Qe.fragment);
                  return We.attributes.position.location = 0, We.attributes.color.location = 2, We.attributes.uv.location = 3, We.attributes.pointSize.location = 4, We;
                }
                function Ve(ze) {
                  var We = de(ze, ke.vertex, ke.fragment);
                  return We.attributes.position.location = 0, We.attributes.id.location = 1, We;
                }
                function Ze(ze) {
                  var We = de(ze, Me.vertex, Me.fragment);
                  return We.attributes.position.location = 0, We.attributes.id.location = 1, We.attributes.pointSize.location = 4, We;
                }
                function Ge(ze) {
                  var We = de(ze, Re.vertex, Re.fragment);
                  return We.attributes.position.location = 0, We;
                }
                $e.isOpaque = function() {
                  return !this.hasAlpha;
                }, $e.isTransparent = function() {
                  return this.hasAlpha;
                }, $e.pickSlots = 1, $e.setPickBase = function(ze) {
                  this.pickId = ze;
                }, $e.highlight = function(ze) {
                  if (ze && this.contourEnable) {
                    for (var We = Ce(this.cells, this.intensity, ze.intensity), at = We.cells, ct = We.vertexIds, et = We.vertexWeights, dt = at.length, Je = Se.mallocFloat32(6 * dt), it = 0, st = 0; st < dt; ++st) for (var gt = at[st], bt = 0; bt < 2; ++bt) {
                      var yt = gt[0];
                      gt.length === 2 && (yt = gt[bt]);
                      for (var St = ct[yt][0], Lt = ct[yt][1], Rt = et[yt], vt = 1 - Rt, $t = this.positions[St], Pt = this.positions[Lt], Xt = 0; Xt < 3; ++Xt) Je[it++] = Rt * $t[Xt] + vt * Pt[Xt];
                    }
                    this.contourCount = it / 3 | 0, this.contourPositions.update(Je.subarray(0, it)), Se.free(Je);
                  } else this.contourCount = 0;
                }, $e.update = function(ze) {
                  ze = ze || {};
                  var We = this.gl;
                  this.dirty = !0, "contourEnable" in ze && (this.contourEnable = ze.contourEnable), "contourColor" in ze && (this.contourColor = ze.contourColor), "lineWidth" in ze && (this.lineWidth = ze.lineWidth), "lightPosition" in ze && (this.lightPosition = ze.lightPosition), this.hasAlpha = !1, "opacity" in ze && (this.opacity = ze.opacity, this.opacity < 1 && (this.hasAlpha = !0)), "opacityscale" in ze && (this.opacityscale = ze.opacityscale, this.hasAlpha = !0), "ambient" in ze && (this.ambientLight = ze.ambient), "diffuse" in ze && (this.diffuseLight = ze.diffuse), "specular" in ze && (this.specularLight = ze.specular), "roughness" in ze && (this.roughness = ze.roughness), "fresnel" in ze && (this.fresnel = ze.fresnel), ze.texture ? (this.texture.dispose(), this.texture = me(We, ze.texture)) : ze.colormap && (this.texture.shape = [256, 256], this.texture.minFilter = We.LINEAR_MIPMAP_LINEAR, this.texture.magFilter = We.LINEAR, this.texture.setPixels(function(wt, Bt) {
                    for (var At = be({ colormap: wt, nshades: 256, format: "rgba" }), Dt = new Uint8Array(1024), Ot = 0; Ot < 256; ++Ot) {
                      for (var un = At[Ot], yn = 0; yn < 3; ++yn) Dt[4 * Ot + yn] = un[yn];
                      Dt[4 * Ot + 3] = Bt ? 255 * qe(Ot / 255, Bt) : 255 * un[3];
                    }
                    return ve(Dt, [256, 256, 4], [4, 0, 1]);
                  }(ze.colormap, this.opacityscale)), this.texture.generateMipmap());
                  var at = ze.cells, ct = ze.positions;
                  if (ct && at) {
                    var et = [], dt = [], Je = [], it = [], st = [], gt = [], bt = [], yt = [], St = [], Lt = [], Rt = [], vt = [], $t = [], Pt = [];
                    this.cells = at, this.positions = ct;
                    var Xt = ze.vertexNormals, Nt = ze.cellNormals, Gt = ze.vertexNormalsEpsilon === void 0 ? 1e-6 : ze.vertexNormalsEpsilon, Ft = ze.faceNormalsEpsilon === void 0 ? 1e-6 : ze.faceNormalsEpsilon;
                    ze.useFacetNormals && !Nt && (Nt = pe.faceNormals(at, ct, Ft)), Nt || Xt || (Xt = pe.vertexNormals(at, ct, Gt));
                    var Tt = ze.vertexColors, It = ze.cellColors, Ct = ze.meshColor || [1, 1, 1, 1], Kt = ze.vertexUVs, Vt = ze.vertexIntensity, Yt = ze.cellUVs, sn = ze.cellIntensity, hn = 1 / 0, Qt = -1 / 0;
                    if (!Kt && !Yt) if (Vt) if (ze.vertexIntensityBounds) hn = +ze.vertexIntensityBounds[0], Qt = +ze.vertexIntensityBounds[1];
                    else for (var Ut = 0; Ut < Vt.length; ++Ut) {
                      var Wt = Vt[Ut];
                      hn = Math.min(hn, Wt), Qt = Math.max(Qt, Wt);
                    }
                    else if (sn) if (ze.cellIntensityBounds) hn = +ze.cellIntensityBounds[0], Qt = +ze.cellIntensityBounds[1];
                    else for (Ut = 0; Ut < sn.length; ++Ut) Wt = sn[Ut], hn = Math.min(hn, Wt), Qt = Math.max(Qt, Wt);
                    else for (Ut = 0; Ut < ct.length; ++Ut) Wt = ct[Ut][2], hn = Math.min(hn, Wt), Qt = Math.max(Qt, Wt);
                    this.intensity = Vt || sn || function(wt) {
                      for (var Bt = wt.length, At = new Array(Bt), Dt = 0; Dt < Bt; ++Dt) At[Dt] = wt[Dt][2];
                      return At;
                    }(ct), this.pickVertex = !(sn || It);
                    var rn = ze.pointSizes, vn = ze.pointSize || 1;
                    for (this.bounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], Ut = 0; Ut < ct.length; ++Ut) for (var dn = ct[Ut], Ht = 0; Ht < 3; ++Ht) !isNaN(dn[Ht]) && isFinite(dn[Ht]) && (this.bounds[0][Ht] = Math.min(this.bounds[0][Ht], dn[Ht]), this.bounds[1][Ht] = Math.max(this.bounds[1][Ht], dn[Ht]));
                    var jt = 0, cn = 0, wn = 0;
                    e: for (Ut = 0; Ut < at.length; ++Ut) {
                      var Rn = at[Ut];
                      switch (Rn.length) {
                        case 1:
                          for (dn = ct[Ie = Rn[0]], Ht = 0; Ht < 3; ++Ht) if (isNaN(dn[Ht]) || !isFinite(dn[Ht])) continue e;
                          Lt.push(dn[0], dn[1], dn[2]), rt = Tt ? Tt[Ie] : It ? It[Ut] : Ct, this.opacityscale && Vt ? dt.push(rt[0], rt[1], rt[2], this.opacity * qe((Vt[Ie] - hn) / (Qt - hn), this.opacityscale)) : rt.length === 3 ? Rt.push(rt[0], rt[1], rt[2], this.opacity) : (Rt.push(rt[0], rt[1], rt[2], rt[3] * this.opacity), rt[3] < 1 && (this.hasAlpha = !0)), ft = Kt ? Kt[Ie] : Vt ? [(Vt[Ie] - hn) / (Qt - hn), 0] : Yt ? Yt[Ut] : sn ? [(sn[Ut] - hn) / (Qt - hn), 0] : [(dn[2] - hn) / (Qt - hn), 0], vt.push(ft[0], ft[1]), rn ? $t.push(rn[Ie]) : $t.push(vn), Pt.push(Ut), wn += 1;
                          break;
                        case 2:
                          for (Ht = 0; Ht < 2; ++Ht) {
                            dn = ct[Ie = Rn[Ht]];
                            for (var Pn = 0; Pn < 3; ++Pn) if (isNaN(dn[Pn]) || !isFinite(dn[Pn])) continue e;
                          }
                          for (Ht = 0; Ht < 2; ++Ht) dn = ct[Ie = Rn[Ht]], gt.push(dn[0], dn[1], dn[2]), rt = Tt ? Tt[Ie] : It ? It[Ut] : Ct, this.opacityscale && Vt ? dt.push(rt[0], rt[1], rt[2], this.opacity * qe((Vt[Ie] - hn) / (Qt - hn), this.opacityscale)) : rt.length === 3 ? bt.push(rt[0], rt[1], rt[2], this.opacity) : (bt.push(rt[0], rt[1], rt[2], rt[3] * this.opacity), rt[3] < 1 && (this.hasAlpha = !0)), ft = Kt ? Kt[Ie] : Vt ? [(Vt[Ie] - hn) / (Qt - hn), 0] : Yt ? Yt[Ut] : sn ? [(sn[Ut] - hn) / (Qt - hn), 0] : [(dn[2] - hn) / (Qt - hn), 0], yt.push(ft[0], ft[1]), St.push(Ut);
                          cn += 1;
                          break;
                        case 3:
                          for (Ht = 0; Ht < 3; ++Ht) for (dn = ct[Ie = Rn[Ht]], Pn = 0; Pn < 3; ++Pn) if (isNaN(dn[Pn]) || !isFinite(dn[Pn])) continue e;
                          for (Ht = 0; Ht < 3; ++Ht) {
                            var Ie, rt, ft, pt;
                            dn = ct[Ie = Rn[2 - Ht]], et.push(dn[0], dn[1], dn[2]), (rt = Tt ? Tt[Ie] : It ? It[Ut] : Ct) ? this.opacityscale && Vt ? dt.push(rt[0], rt[1], rt[2], this.opacity * qe((Vt[Ie] - hn) / (Qt - hn), this.opacityscale)) : rt.length === 3 ? dt.push(rt[0], rt[1], rt[2], this.opacity) : (dt.push(rt[0], rt[1], rt[2], rt[3] * this.opacity), rt[3] < 1 && (this.hasAlpha = !0)) : dt.push(0.5, 0.5, 0.5, 1), ft = Kt ? Kt[Ie] : Vt ? [(Vt[Ie] - hn) / (Qt - hn), 0] : Yt ? Yt[Ut] : sn ? [(sn[Ut] - hn) / (Qt - hn), 0] : [(dn[2] - hn) / (Qt - hn), 0], it.push(ft[0], ft[1]), pt = Xt ? Xt[Ie] : Nt[Ut], Je.push(pt[0], pt[1], pt[2]), st.push(Ut);
                          }
                          jt += 1;
                      }
                    }
                    this.pointCount = wn, this.edgeCount = cn, this.triangleCount = jt, this.pointPositions.update(Lt), this.pointColors.update(Rt), this.pointUVs.update(vt), this.pointSizes.update($t), this.pointIds.update(new Uint32Array(Pt)), this.edgePositions.update(gt), this.edgeColors.update(bt), this.edgeUVs.update(yt), this.edgeIds.update(new Uint32Array(St)), this.trianglePositions.update(et), this.triangleColors.update(dt), this.triangleUVs.update(it), this.triangleNormals.update(Je), this.triangleIds.update(new Uint32Array(st));
                  }
                }, $e.drawTransparent = $e.draw = function(ze) {
                  ze = ze || {};
                  for (var We = this.gl, at = ze.model || De, ct = ze.view || De, et = ze.projection || De, dt = [[-1e6, -1e6, -1e6], [1e6, 1e6, 1e6]], Je = 0; Je < 3; ++Je) dt[0][Je] = Math.max(dt[0][Je], this.clipBounds[0][Je]), dt[1][Je] = Math.min(dt[1][Je], this.clipBounds[1][Je]);
                  var it = { model: at, view: ct, projection: et, inverseModel: De.slice(), clipBounds: dt, kambient: this.ambientLight, kdiffuse: this.diffuseLight, kspecular: this.specularLight, roughness: this.roughness, fresnel: this.fresnel, eyePosition: [0, 0, 0], lightPosition: [0, 0, 0], contourColor: this.contourColor, texture: 0 };
                  it.inverseModel = ge(it.inverseModel, it.model), We.disable(We.CULL_FACE), this.texture.bind(0);
                  var st = new Array(16);
                  for (ye(st, it.view, it.model), ye(st, it.projection, st), ge(st, st), Je = 0; Je < 3; ++Je) it.eyePosition[Je] = st[12 + Je] / st[15];
                  var gt, bt = st[15];
                  for (Je = 0; Je < 3; ++Je) bt += this.lightPosition[Je] * st[4 * Je + 3];
                  for (Je = 0; Je < 3; ++Je) {
                    for (var yt = st[12 + Je], St = 0; St < 3; ++St) yt += st[4 * St + Je] * this.lightPosition[St];
                    it.lightPosition[Je] = yt / bt;
                  }
                  this.triangleCount > 0 && ((gt = this.triShader).bind(), gt.uniforms = it, this.triangleVAO.bind(), We.drawArrays(We.TRIANGLES, 0, 3 * this.triangleCount), this.triangleVAO.unbind()), this.edgeCount > 0 && this.lineWidth > 0 && ((gt = this.lineShader).bind(), gt.uniforms = it, this.edgeVAO.bind(), We.lineWidth(this.lineWidth * this.pixelRatio), We.drawArrays(We.LINES, 0, 2 * this.edgeCount), this.edgeVAO.unbind()), this.pointCount > 0 && ((gt = this.pointShader).bind(), gt.uniforms = it, this.pointVAO.bind(), We.drawArrays(We.POINTS, 0, this.pointCount), this.pointVAO.unbind()), this.contourEnable && this.contourCount > 0 && this.contourLineWidth > 0 && ((gt = this.contourShader).bind(), gt.uniforms = it, this.contourVAO.bind(), We.drawArrays(We.LINES, 0, this.contourCount), this.contourVAO.unbind());
                }, $e.drawPick = function(ze) {
                  ze = ze || {};
                  for (var We = this.gl, at = ze.model || De, ct = ze.view || De, et = ze.projection || De, dt = [[-1e6, -1e6, -1e6], [1e6, 1e6, 1e6]], Je = 0; Je < 3; ++Je) dt[0][Je] = Math.max(dt[0][Je], this.clipBounds[0][Je]), dt[1][Je] = Math.min(dt[1][Je], this.clipBounds[1][Je]);
                  this._model = [].slice.call(at), this._view = [].slice.call(ct), this._projection = [].slice.call(et), this._resolution = [We.drawingBufferWidth, We.drawingBufferHeight];
                  var it, st = { model: at, view: ct, projection: et, clipBounds: dt, pickId: this.pickId / 255 };
                  (it = this.pickShader).bind(), it.uniforms = st, this.triangleCount > 0 && (this.triangleVAO.bind(), We.drawArrays(We.TRIANGLES, 0, 3 * this.triangleCount), this.triangleVAO.unbind()), this.edgeCount > 0 && (this.edgeVAO.bind(), We.lineWidth(this.lineWidth * this.pixelRatio), We.drawArrays(We.LINES, 0, 2 * this.edgeCount), this.edgeVAO.unbind()), this.pointCount > 0 && ((it = this.pointPickShader).bind(), it.uniforms = st, this.pointVAO.bind(), We.drawArrays(We.POINTS, 0, this.pointCount), this.pointVAO.unbind());
                }, $e.pick = function(ze) {
                  if (!ze || ze.id !== this.pickId) return null;
                  for (var We = ze.value[0] + 256 * ze.value[1] + 65536 * ze.value[2], at = this.cells[We], ct = this.positions, et = new Array(at.length), dt = 0; dt < at.length; ++dt) et[dt] = ct[at[dt]];
                  var Je = ze.coord[0], it = ze.coord[1];
                  if (!this.pickVertex) {
                    var st = this.positions[at[0]], gt = this.positions[at[1]], bt = this.positions[at[2]], yt = [(st[0] + gt[0] + bt[0]) / 3, (st[1] + gt[1] + bt[1]) / 3, (st[2] + gt[2] + bt[2]) / 3];
                    return { _cellCenter: !0, position: [Je, it], index: We, cell: at, cellId: We, intensity: this.intensity[We], dataCoordinate: yt };
                  }
                  var St = Ee(et, [Je * this.pixelRatio, this._resolution[1] - it * this.pixelRatio], this._model, this._view, this._projection, this._resolution);
                  if (!St) return null;
                  var Lt = St[2], Rt = 0;
                  for (dt = 0; dt < at.length; ++dt) Rt += Lt[dt] * this.intensity[at[dt]];
                  return { position: St[1], index: at[St[0]], cell: at, cellId: We, intensity: Rt, dataCoordinate: this.positions[at[St[0]]] };
                }, $e.dispose = function() {
                  this.texture.dispose(), this.triShader.dispose(), this.lineShader.dispose(), this.pointShader.dispose(), this.pickShader.dispose(), this.pointPickShader.dispose(), this.triangleVAO.dispose(), this.trianglePositions.dispose(), this.triangleColors.dispose(), this.triangleUVs.dispose(), this.triangleNormals.dispose(), this.triangleIds.dispose(), this.edgeVAO.dispose(), this.edgePositions.dispose(), this.edgeColors.dispose(), this.edgeUVs.dispose(), this.edgeIds.dispose(), this.pointVAO.dispose(), this.pointPositions.dispose(), this.pointColors.dispose(), this.pointUVs.dispose(), this.pointSizes.dispose(), this.pointIds.dispose(), this.contourVAO.dispose(), this.contourPositions.dispose(), this.contourShader.dispose();
                }, ce.exports = function(ze, We) {
                  if (arguments.length === 1 && (ze = (We = ze).gl), !(ze.getExtension("OES_standard_derivatives") || ze.getExtension("MOZ_OES_standard_derivatives") || ze.getExtension("WEBKIT_OES_standard_derivatives"))) throw new Error("derivatives not supported");
                  var at = function(sn) {
                    var hn = de(sn, xe.vertex, xe.fragment);
                    return hn.attributes.position.location = 0, hn.attributes.color.location = 2, hn.attributes.uv.location = 3, hn.attributes.normal.location = 4, hn;
                  }(ze), ct = function(sn) {
                    var hn = de(sn, Le.vertex, Le.fragment);
                    return hn.attributes.position.location = 0, hn.attributes.color.location = 2, hn.attributes.uv.location = 3, hn;
                  }(ze), et = Xe(ze), dt = Ve(ze), Je = Ze(ze), it = Ge(ze), st = me(ze, ve(new Uint8Array([255, 255, 255, 255]), [1, 1, 4]));
                  st.generateMipmap(), st.minFilter = ze.LINEAR_MIPMAP_LINEAR, st.magFilter = ze.LINEAR;
                  var gt = fe(ze), bt = fe(ze), yt = fe(ze), St = fe(ze), Lt = fe(ze), Rt = he(ze, [{ buffer: gt, type: ze.FLOAT, size: 3 }, { buffer: Lt, type: ze.UNSIGNED_BYTE, size: 4, normalized: !0 }, { buffer: bt, type: ze.FLOAT, size: 4 }, { buffer: yt, type: ze.FLOAT, size: 2 }, { buffer: St, type: ze.FLOAT, size: 3 }]), vt = fe(ze), $t = fe(ze), Pt = fe(ze), Xt = fe(ze), Nt = he(ze, [{ buffer: vt, type: ze.FLOAT, size: 3 }, { buffer: Xt, type: ze.UNSIGNED_BYTE, size: 4, normalized: !0 }, { buffer: $t, type: ze.FLOAT, size: 4 }, { buffer: Pt, type: ze.FLOAT, size: 2 }]), Gt = fe(ze), Ft = fe(ze), Tt = fe(ze), It = fe(ze), Ct = fe(ze), Kt = he(ze, [{ buffer: Gt, type: ze.FLOAT, size: 3 }, { buffer: Ct, type: ze.UNSIGNED_BYTE, size: 4, normalized: !0 }, { buffer: Ft, type: ze.FLOAT, size: 4 }, { buffer: Tt, type: ze.FLOAT, size: 2 }, { buffer: It, type: ze.FLOAT, size: 1 }]), Vt = fe(ze), Yt = new Pe(ze, st, at, ct, et, dt, Je, it, gt, Lt, bt, yt, St, Rt, vt, Xt, $t, Pt, Nt, Gt, Ct, Ft, Tt, It, Kt, Vt, he(ze, [{ buffer: Vt, type: ze.FLOAT, size: 3 }]));
                  return Yt.update(We), Yt;
                };
              }, 8120: function(ce, le, ue) {
                ce.exports = function(ve) {
                  var be = ve.gl;
                  return new me(ve, de(be, [0, 0, 0, 1, 1, 0, 1, 1]), fe(be, he.boxVert, he.lineFrag));
                };
                var de = ue(2762), fe = ue(9405), he = ue(3603);
                function me(ve, be, Ce) {
                  this.plot = ve, this.vbo = be, this.shader = Ce;
                }
                var pe, ye, ge = me.prototype;
                ge.bind = function() {
                  var ve = this.shader;
                  this.vbo.bind(), this.shader.bind(), ve.attributes.coord.pointer(), ve.uniforms.screenBox = this.plot.screenBox;
                }, ge.drawBox = (pe = [0, 0], ye = [0, 0], function(ve, be, Ce, Se, we) {
                  var Ee = this.plot, xe = this.shader, Le = Ee.gl;
                  pe[0] = ve, pe[1] = be, ye[0] = Ce, ye[1] = Se, xe.uniforms.lo = pe, xe.uniforms.hi = ye, xe.uniforms.color = we, Le.drawArrays(Le.TRIANGLE_STRIP, 0, 4);
                }), ge.dispose = function() {
                  this.vbo.dispose(), this.shader.dispose();
                };
              }, 1913: function(ce, le, ue) {
                ce.exports = function(Ee) {
                  var xe = Ee.gl;
                  return new pe(Ee, de(xe), fe(xe, me.gridVert, me.gridFrag), fe(xe, me.tickVert, me.gridFrag));
                };
                var de = ue(2762), fe = ue(9405), he = ue(2478), me = ue(3603);
                function pe(Ee, xe, Le, Qe) {
                  this.plot = Ee, this.vbo = xe, this.shader = Le, this.tickShader = Qe, this.ticks = [[], []];
                }
                function ye(Ee, xe) {
                  return Ee - xe;
                }
                var ge, ve, be, Ce, Se, we = pe.prototype;
                we.draw = (ge = [0, 0], ve = [0, 0], be = [0, 0], function() {
                  for (var Ee = this.plot, xe = this.vbo, Le = this.shader, Qe = this.ticks, ke = Ee.gl, Me = Ee._tickBounds, Re = Ee.dataBox, De = Ee.viewBox, Pe = Ee.gridLineWidth, $e = Ee.gridLineColor, qe = Ee.gridLineEnable, Xe = Ee.pixelRatio, Ve = 0; Ve < 2; ++Ve) {
                    var Ze = Me[Ve], Ge = Me[Ve + 2] - Ze, ze = 0.5 * (Re[Ve + 2] + Re[Ve]), We = Re[Ve + 2] - Re[Ve];
                    ve[Ve] = 2 * Ge / We, ge[Ve] = 2 * (Ze - ze) / We;
                  }
                  Le.bind(), xe.bind(), Le.attributes.dataCoord.pointer(), Le.uniforms.dataShift = ge, Le.uniforms.dataScale = ve;
                  var at = 0;
                  for (Ve = 0; Ve < 2; ++Ve) {
                    be[0] = be[1] = 0, be[Ve] = 1, Le.uniforms.dataAxis = be, Le.uniforms.lineWidth = Pe[Ve] / (De[Ve + 2] - De[Ve]) * Xe, Le.uniforms.color = $e[Ve];
                    var ct = 6 * Qe[Ve].length;
                    qe[Ve] && ct && ke.drawArrays(ke.TRIANGLES, at, ct), at += ct;
                  }
                }), we.drawTickMarks = /* @__PURE__ */ function() {
                  var Ee = [0, 0], xe = [0, 0], Le = [1, 0], Qe = [0, 1], ke = [0, 0], Me = [0, 0];
                  return function() {
                    for (var Re = this.plot, De = this.vbo, Pe = this.tickShader, $e = this.ticks, qe = Re.gl, Xe = Re._tickBounds, Ve = Re.dataBox, Ze = Re.viewBox, Ge = Re.pixelRatio, ze = Re.screenBox, We = ze[2] - ze[0], at = ze[3] - ze[1], ct = Ze[2] - Ze[0], et = Ze[3] - Ze[1], dt = 0; dt < 2; ++dt) {
                      var Je = Xe[dt], it = Xe[dt + 2] - Je, st = 0.5 * (Ve[dt + 2] + Ve[dt]), gt = Ve[dt + 2] - Ve[dt];
                      xe[dt] = 2 * it / gt, Ee[dt] = 2 * (Je - st) / gt;
                    }
                    xe[0] *= ct / We, Ee[0] *= ct / We, xe[1] *= et / at, Ee[1] *= et / at, Pe.bind(), De.bind(), Pe.attributes.dataCoord.pointer();
                    var bt = Pe.uniforms;
                    bt.dataShift = Ee, bt.dataScale = xe;
                    var yt = Re.tickMarkLength, St = Re.tickMarkWidth, Lt = Re.tickMarkColor, Rt = 6 * $e[0].length, vt = Math.min(he.ge($e[0], (Ve[0] - Xe[0]) / (Xe[2] - Xe[0]), ye), $e[0].length), $t = Math.min(he.gt($e[0], (Ve[2] - Xe[0]) / (Xe[2] - Xe[0]), ye), $e[0].length), Pt = 0 + 6 * vt, Xt = 6 * Math.max(0, $t - vt), Nt = Math.min(he.ge($e[1], (Ve[1] - Xe[1]) / (Xe[3] - Xe[1]), ye), $e[1].length), Gt = Math.min(he.gt($e[1], (Ve[3] - Xe[1]) / (Xe[3] - Xe[1]), ye), $e[1].length), Ft = Rt + 6 * Nt, Tt = 6 * Math.max(0, Gt - Nt);
                    ke[0] = 2 * (Ze[0] - yt[1]) / We - 1, ke[1] = (Ze[3] + Ze[1]) / at - 1, Me[0] = yt[1] * Ge / We, Me[1] = St[1] * Ge / at, Tt && (bt.color = Lt[1], bt.tickScale = Me, bt.dataAxis = Qe, bt.screenOffset = ke, qe.drawArrays(qe.TRIANGLES, Ft, Tt)), ke[0] = (Ze[2] + Ze[0]) / We - 1, ke[1] = 2 * (Ze[1] - yt[0]) / at - 1, Me[0] = St[0] * Ge / We, Me[1] = yt[0] * Ge / at, Xt && (bt.color = Lt[0], bt.tickScale = Me, bt.dataAxis = Le, bt.screenOffset = ke, qe.drawArrays(qe.TRIANGLES, Pt, Xt)), ke[0] = 2 * (Ze[2] + yt[3]) / We - 1, ke[1] = (Ze[3] + Ze[1]) / at - 1, Me[0] = yt[3] * Ge / We, Me[1] = St[3] * Ge / at, Tt && (bt.color = Lt[3], bt.tickScale = Me, bt.dataAxis = Qe, bt.screenOffset = ke, qe.drawArrays(qe.TRIANGLES, Ft, Tt)), ke[0] = (Ze[2] + Ze[0]) / We - 1, ke[1] = 2 * (Ze[3] + yt[2]) / at - 1, Me[0] = St[2] * Ge / We, Me[1] = yt[2] * Ge / at, Xt && (bt.color = Lt[2], bt.tickScale = Me, bt.dataAxis = Le, bt.screenOffset = ke, qe.drawArrays(qe.TRIANGLES, Pt, Xt));
                  };
                }(), we.update = (Ce = [1, 1, -1, -1, 1, -1], Se = [1, -1, 1, 1, -1, -1], function(Ee) {
                  for (var xe = Ee.ticks, Le = Ee.bounds, Qe = new Float32Array(18 * (xe[0].length + xe[1].length)), ke = (this.plot.zeroLineEnable, 0), Me = [[], []], Re = 0; Re < 2; ++Re) for (var De = Me[Re], Pe = xe[Re], $e = Le[Re], qe = Le[Re + 2], Xe = 0; Xe < Pe.length; ++Xe) {
                    var Ve = (Pe[Xe].x - $e) / (qe - $e);
                    De.push(Ve);
                    for (var Ze = 0; Ze < 6; ++Ze) Qe[ke++] = Ve, Qe[ke++] = Ce[Ze], Qe[ke++] = Se[Ze];
                  }
                  this.ticks = Me, this.vbo.update(Qe);
                }), we.dispose = function() {
                  this.vbo.dispose(), this.shader.dispose(), this.tickShader.dispose();
                };
              }, 4747: function(ce, le, ue) {
                ce.exports = function(ve) {
                  var be = ve.gl;
                  return new me(ve, de(be, [-1, -1, -1, 1, 1, -1, 1, 1]), fe(be, he.lineVert, he.lineFrag));
                };
                var de = ue(2762), fe = ue(9405), he = ue(3603);
                function me(ve, be, Ce) {
                  this.plot = ve, this.vbo = be, this.shader = Ce;
                }
                var pe, ye, ge = me.prototype;
                ge.bind = function() {
                  var ve = this.shader;
                  this.vbo.bind(), this.shader.bind(), ve.attributes.coord.pointer(), ve.uniforms.screenBox = this.plot.screenBox;
                }, ge.drawLine = (pe = [0, 0], ye = [0, 0], function(ve, be, Ce, Se, we, Ee) {
                  var xe = this.plot, Le = this.shader, Qe = xe.gl;
                  pe[0] = ve, pe[1] = be, ye[0] = Ce, ye[1] = Se, Le.uniforms.start = pe, Le.uniforms.end = ye, Le.uniforms.width = we * xe.pixelRatio, Le.uniforms.color = Ee, Qe.drawArrays(Qe.TRIANGLE_STRIP, 0, 4);
                }), ge.dispose = function() {
                  this.vbo.dispose(), this.shader.dispose();
                };
              }, 3603: function(ce, le, ue) {
                var de = ue(3236), fe = de([`precision lowp float;
#define GLSLIFY 1
uniform vec4 color;
void main() {
  gl_FragColor = vec4(color.xyz * color.w, color.w);
}
`]);
                ce.exports = { lineVert: de([`precision mediump float;
#define GLSLIFY 1

attribute vec2 coord;

uniform vec4 screenBox;
uniform vec2 start, end;
uniform float width;

vec2 perp(vec2 v) {
  return vec2(v.y, -v.x);
}

vec2 screen(vec2 v) {
  return 2.0 * (v - screenBox.xy) / (screenBox.zw - screenBox.xy) - 1.0;
}

void main() {
  vec2 delta = normalize(perp(start - end));
  vec2 offset = mix(start, end, 0.5 * (coord.y+1.0));
  gl_Position = vec4(screen(offset + 0.5 * width * delta * coord.x), 0, 1);
}
`]), lineFrag: fe, textVert: de([`#define GLSLIFY 1
attribute vec3 textCoordinate;

uniform vec2 dataScale, dataShift, dataAxis, screenOffset, textScale;
uniform float angle;

void main() {
  float dataOffset  = textCoordinate.z;
  vec2 glyphOffset  = textCoordinate.xy;
  mat2 glyphMatrix = mat2(cos(angle), sin(angle), -sin(angle), cos(angle));
  vec2 screenCoordinate = dataAxis * (dataScale * dataOffset + dataShift) +
    glyphMatrix * glyphOffset * textScale + screenOffset;
  gl_Position = vec4(screenCoordinate, 0, 1);
}
`]), textFrag: fe, gridVert: de([`precision mediump float;
#define GLSLIFY 1

attribute vec3 dataCoord;

uniform vec2 dataAxis, dataShift, dataScale;
uniform float lineWidth;

void main() {
  vec2 pos = dataAxis * (dataScale * dataCoord.x + dataShift);
  pos += 10.0 * dataCoord.y * vec2(dataAxis.y, -dataAxis.x) + dataCoord.z * lineWidth;
  gl_Position = vec4(pos, 0, 1);
}
`]), gridFrag: fe, boxVert: de([`precision mediump float;
#define GLSLIFY 1

attribute vec2 coord;

uniform vec4 screenBox;
uniform vec2 lo, hi;

vec2 screen(vec2 v) {
  return 2.0 * (v - screenBox.xy) / (screenBox.zw - screenBox.xy) - 1.0;
}

void main() {
  gl_Position = vec4(screen(mix(lo, hi, coord)), 0, 1);
}
`]), tickVert: de([`precision mediump float;
#define GLSLIFY 1

attribute vec3 dataCoord;

uniform vec2 dataAxis, dataShift, dataScale, screenOffset, tickScale;

void main() {
  vec2 pos = dataAxis * (dataScale * dataCoord.x + dataShift);
  gl_Position = vec4(pos + tickScale*dataCoord.yz + screenOffset, 0, 1);
}
`]) };
              }, 2142: function(ce, le, ue) {
                ce.exports = function(xe) {
                  var Le = xe.gl;
                  return new ye(xe, de(Le), fe(Le, pe.textVert, pe.textFrag));
                };
                var de = ue(2762), fe = ue(9405), he = ue(529), me = ue(2478), pe = ue(3603);
                function ye(xe, Le, Qe) {
                  this.plot = xe, this.vbo = Le, this.shader = Qe, this.tickOffset = [[], []], this.tickX = [[], []], this.labelOffset = [0, 0], this.labelCount = [0, 0];
                }
                var ge, ve, be, Ce, Se, we, Ee = ye.prototype;
                Ee.drawTicks = (ge = [0, 0], ve = [0, 0], be = [0, 0], function(xe) {
                  var Le = this.plot, Qe = this.shader, ke = this.tickX[xe], Me = this.tickOffset[xe], Re = Le.gl, De = Le.viewBox, Pe = Le.dataBox, $e = Le.screenBox, qe = Le.pixelRatio, Xe = Le.tickEnable, Ve = Le.tickPad, Ze = Le.tickColor, Ge = Le.tickAngle, ze = Le.labelEnable, We = Le.labelPad, at = Le.labelColor, ct = Le.labelAngle, et = this.labelOffset[xe], dt = this.labelCount[xe], Je = me.lt(ke, Pe[xe]), it = me.le(ke, Pe[xe + 2]);
                  ge[0] = ge[1] = 0, ge[xe] = 1, ve[xe] = (De[2 + xe] + De[xe]) / ($e[2 + xe] - $e[xe]) - 1;
                  var st = 2 / $e[2 + (1 ^ xe)] - $e[1 ^ xe];
                  ve[1 ^ xe] = st * De[1 ^ xe] - 1, Xe[xe] && (ve[1 ^ xe] -= st * qe * Ve[xe], Je < it && Me[it] > Me[Je] && (Qe.uniforms.dataAxis = ge, Qe.uniforms.screenOffset = ve, Qe.uniforms.color = Ze[xe], Qe.uniforms.angle = Ge[xe], Re.drawArrays(Re.TRIANGLES, Me[Je], Me[it] - Me[Je]))), ze[xe] && dt && (ve[1 ^ xe] -= st * qe * We[xe], Qe.uniforms.dataAxis = be, Qe.uniforms.screenOffset = ve, Qe.uniforms.color = at[xe], Qe.uniforms.angle = ct[xe], Re.drawArrays(Re.TRIANGLES, et, dt)), ve[1 ^ xe] = st * De[2 + (1 ^ xe)] - 1, Xe[xe + 2] && (ve[1 ^ xe] += st * qe * Ve[xe + 2], Je < it && Me[it] > Me[Je] && (Qe.uniforms.dataAxis = ge, Qe.uniforms.screenOffset = ve, Qe.uniforms.color = Ze[xe + 2], Qe.uniforms.angle = Ge[xe + 2], Re.drawArrays(Re.TRIANGLES, Me[Je], Me[it] - Me[Je]))), ze[xe + 2] && dt && (ve[1 ^ xe] += st * qe * We[xe + 2], Qe.uniforms.dataAxis = be, Qe.uniforms.screenOffset = ve, Qe.uniforms.color = at[xe + 2], Qe.uniforms.angle = ct[xe + 2], Re.drawArrays(Re.TRIANGLES, et, dt));
                }), Ee.drawTitle = /* @__PURE__ */ function() {
                  var xe = [0, 0], Le = [0, 0];
                  return function() {
                    var Qe = this.plot, ke = this.shader, Me = Qe.gl, Re = Qe.screenBox, De = Qe.titleCenter, Pe = Qe.titleAngle, $e = Qe.titleColor, qe = Qe.pixelRatio;
                    if (this.titleCount) {
                      for (var Xe = 0; Xe < 2; ++Xe) Le[Xe] = 2 * (De[Xe] * qe - Re[Xe]) / (Re[2 + Xe] - Re[Xe]) - 1;
                      ke.bind(), ke.uniforms.dataAxis = xe, ke.uniforms.screenOffset = Le, ke.uniforms.angle = Pe, ke.uniforms.color = $e, Me.drawArrays(Me.TRIANGLES, this.titleOffset, this.titleCount);
                    }
                  };
                }(), Ee.bind = (Ce = [0, 0], Se = [0, 0], we = [0, 0], function() {
                  var xe = this.plot, Le = this.shader, Qe = xe._tickBounds, ke = xe.dataBox, Me = xe.screenBox, Re = xe.viewBox;
                  Le.bind();
                  for (var De = 0; De < 2; ++De) {
                    var Pe = Qe[De], $e = Qe[De + 2] - Pe, qe = 0.5 * (ke[De + 2] + ke[De]), Xe = ke[De + 2] - ke[De], Ve = Re[De], Ze = Re[De + 2] - Ve, Ge = Me[De], ze = Me[De + 2] - Ge;
                    Se[De] = 2 * $e / Xe * Ze / ze, Ce[De] = 2 * (Pe - qe) / Xe * Ze / ze;
                  }
                  we[1] = 2 * xe.pixelRatio / (Me[3] - Me[1]), we[0] = we[1] * (Me[3] - Me[1]) / (Me[2] - Me[0]), Le.uniforms.dataScale = Se, Le.uniforms.dataShift = Ce, Le.uniforms.textScale = we, this.vbo.bind(), Le.attributes.textCoordinate.pointer();
                }), Ee.update = function(xe) {
                  var Le, Qe, ke, Me, Re, De = [], Pe = xe.ticks, $e = xe.bounds;
                  for (Re = 0; Re < 2; ++Re) {
                    var qe = [Math.floor(De.length / 3)], Xe = [-1 / 0], Ve = Pe[Re];
                    for (Le = 0; Le < Ve.length; ++Le) {
                      var Ze = Ve[Le], Ge = Ze.x, ze = Ze.text, We = Ze.font || "sans-serif", at = Ze.fontStyle || "normal", ct = Ze.fontWeight || "normal", et = Ze.fontVariant || "normal";
                      Me = Ze.fontSize || 12;
                      for (var dt = 1 / ($e[Re + 2] - $e[Re]), Je = $e[Re], it = ze.split(`
`), st = 0; st < it.length; st++) for (ke = he(We, it[st], { fontStyle: at, fontWeight: ct, fontVariant: et }).data, Qe = 0; Qe < ke.length; Qe += 2) De.push(ke[Qe] * Me, -ke[Qe + 1] * Me - st * Me * 1.2, (Ge - Je) * dt);
                      qe.push(Math.floor(De.length / 3)), Xe.push(Ge);
                    }
                    this.tickOffset[Re] = qe, this.tickX[Re] = Xe;
                  }
                  for (Re = 0; Re < 2; ++Re) {
                    for (this.labelOffset[Re] = Math.floor(De.length / 3), ke = he(xe.labelFont[Re], xe.labels[Re], { fontStyle: xe.labelFontStyle[Re], fontWeight: xe.labelFontWeight[Re], fontVariant: xe.labelFontVariant[Re], textAlign: "center" }).data, Me = xe.labelSize[Re], Le = 0; Le < ke.length; Le += 2) De.push(ke[Le] * Me, -ke[Le + 1] * Me, 0);
                    this.labelCount[Re] = Math.floor(De.length / 3) - this.labelOffset[Re];
                  }
                  for (this.titleOffset = Math.floor(De.length / 3), ke = he(xe.titleFont, xe.title, { fontStyle: xe.titleFontStyle, fontWeight: xe.titleFontWeight, fontVariant: xe.titleFontVariant }).data, Me = xe.titleSize, Le = 0; Le < ke.length; Le += 2) De.push(ke[Le] * Me, -ke[Le + 1] * Me, 0);
                  this.titleCount = Math.floor(De.length / 3) - this.titleOffset, this.vbo.update(De);
                }, Ee.dispose = function() {
                  this.vbo.dispose(), this.shader.dispose();
                };
              }, 1850: function(ce, le, ue) {
                ce.exports = function(Ce) {
                  var Se = Ce.gl, we = new ye(Se, de(Se, [Se.drawingBufferWidth, Se.drawingBufferHeight]));
                  return we.grid = fe(we), we.text = he(we), we.line = me(we), we.box = pe(we), we.update(Ce), we;
                };
                var de = ue(3589), fe = ue(1913), he = ue(2142), me = ue(4747), pe = ue(8120);
                function ye(Ce, Se) {
                  this.gl = Ce, this.pickBuffer = Se, this.screenBox = [0, 0, Ce.drawingBufferWidth, Ce.drawingBufferHeight], this.viewBox = [0, 0, 0, 0], this.dataBox = [-10, -10, 10, 10], this.gridLineEnable = [!0, !0], this.gridLineWidth = [1, 1], this.gridLineColor = [[0, 0, 0, 1], [0, 0, 0, 1]], this.pixelRatio = 1, this.tickMarkLength = [0, 0, 0, 0], this.tickMarkWidth = [0, 0, 0, 0], this.tickMarkColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.tickPad = [15, 15, 15, 15], this.tickAngle = [0, 0, 0, 0], this.tickEnable = [!0, !0, !0, !0], this.tickColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.labelPad = [15, 15, 15, 15], this.labelAngle = [0, Math.PI / 2, 0, 3 * Math.PI / 2], this.labelEnable = [!0, !0, !0, !0], this.labelColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.titleCenter = [0, 0], this.titleEnable = !0, this.titleAngle = 0, this.titleColor = [0, 0, 0, 1], this.borderColor = [0, 0, 0, 0], this.backgroundColor = [0, 0, 0, 0], this.zeroLineEnable = [!0, !0], this.zeroLineWidth = [4, 4], this.zeroLineColor = [[0, 0, 0, 1], [0, 0, 0, 1]], this.borderLineEnable = [!0, !0, !0, !0], this.borderLineWidth = [2, 2, 2, 2], this.borderLineColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.grid = null, this.text = null, this.line = null, this.box = null, this.objects = [], this.overlays = [], this._tickBounds = [1 / 0, 1 / 0, -1 / 0, -1 / 0], this.static = !1, this.dirty = !1, this.pickDirty = !1, this.pickDelay = 120, this.pickRadius = 10, this._pickTimeout = null, this._drawPick = this.drawPick.bind(this), this._depthCounter = 0;
                }
                var ge = ye.prototype;
                function ve(Ce) {
                  for (var Se = Ce.slice(), we = 0; we < Se.length; ++we) Se[we] = Se[we].slice();
                  return Se;
                }
                function be(Ce, Se) {
                  return Ce.x - Se.x;
                }
                ge.setDirty = function() {
                  this.dirty = this.pickDirty = !0;
                }, ge.setOverlayDirty = function() {
                  this.dirty = !0;
                }, ge.nextDepthValue = function() {
                  return this._depthCounter++ / 65536;
                }, ge.draw = function() {
                  var Ce = this.gl, Se = this.screenBox, we = this.viewBox, Ee = this.dataBox, xe = this.pixelRatio, Le = this.grid, Qe = this.line, ke = this.text, Me = this.objects;
                  if (this._depthCounter = 0, this.pickDirty && (this._pickTimeout && clearTimeout(this._pickTimeout), this.pickDirty = !1, this._pickTimeout = setTimeout(this._drawPick, this.pickDelay)), this.dirty) {
                    if (this.dirty = !1, Ce.bindFramebuffer(Ce.FRAMEBUFFER, null), Ce.enable(Ce.SCISSOR_TEST), Ce.disable(Ce.DEPTH_TEST), Ce.depthFunc(Ce.LESS), Ce.depthMask(!1), Ce.enable(Ce.BLEND), Ce.blendEquation(Ce.FUNC_ADD, Ce.FUNC_ADD), Ce.blendFunc(Ce.ONE, Ce.ONE_MINUS_SRC_ALPHA), this.borderColor) {
                      Ce.scissor(Se[0], Se[1], Se[2] - Se[0], Se[3] - Se[1]);
                      var Re = this.borderColor;
                      Ce.clearColor(Re[0] * Re[3], Re[1] * Re[3], Re[2] * Re[3], Re[3]), Ce.clear(Ce.COLOR_BUFFER_BIT | Ce.DEPTH_BUFFER_BIT);
                    }
                    Ce.scissor(we[0], we[1], we[2] - we[0], we[3] - we[1]), Ce.viewport(we[0], we[1], we[2] - we[0], we[3] - we[1]);
                    var De = this.backgroundColor;
                    Ce.clearColor(De[0] * De[3], De[1] * De[3], De[2] * De[3], De[3]), Ce.clear(Ce.COLOR_BUFFER_BIT), Le.draw();
                    var Pe = this.zeroLineEnable, $e = this.zeroLineColor, qe = this.zeroLineWidth;
                    if (Pe[0] || Pe[1]) {
                      Qe.bind();
                      for (var Xe = 0; Xe < 2; ++Xe) if (Pe[Xe] && Ee[Xe] <= 0 && Ee[Xe + 2] >= 0) {
                        var Ve = Se[Xe] - Ee[Xe] * (Se[Xe + 2] - Se[Xe]) / (Ee[Xe + 2] - Ee[Xe]);
                        Xe === 0 ? Qe.drawLine(Ve, Se[1], Ve, Se[3], qe[Xe], $e[Xe]) : Qe.drawLine(Se[0], Ve, Se[2], Ve, qe[Xe], $e[Xe]);
                      }
                    }
                    for (Xe = 0; Xe < Me.length; ++Xe) Me[Xe].draw();
                    Ce.viewport(Se[0], Se[1], Se[2] - Se[0], Se[3] - Se[1]), Ce.scissor(Se[0], Se[1], Se[2] - Se[0], Se[3] - Se[1]), this.grid.drawTickMarks(), Qe.bind();
                    var Ze = this.borderLineEnable, Ge = this.borderLineWidth, ze = this.borderLineColor;
                    for (Ze[1] && Qe.drawLine(we[0], we[1] - 0.5 * Ge[1] * xe, we[0], we[3] + 0.5 * Ge[3] * xe, Ge[1], ze[1]), Ze[0] && Qe.drawLine(we[0] - 0.5 * Ge[0] * xe, we[1], we[2] + 0.5 * Ge[2] * xe, we[1], Ge[0], ze[0]), Ze[3] && Qe.drawLine(we[2], we[1] - 0.5 * Ge[1] * xe, we[2], we[3] + 0.5 * Ge[3] * xe, Ge[3], ze[3]), Ze[2] && Qe.drawLine(we[0] - 0.5 * Ge[0] * xe, we[3], we[2] + 0.5 * Ge[2] * xe, we[3], Ge[2], ze[2]), ke.bind(), Xe = 0; Xe < 2; ++Xe) ke.drawTicks(Xe);
                    this.titleEnable && ke.drawTitle();
                    var We = this.overlays;
                    for (Xe = 0; Xe < We.length; ++Xe) We[Xe].draw();
                    Ce.disable(Ce.SCISSOR_TEST), Ce.disable(Ce.BLEND), Ce.depthMask(!0);
                  }
                }, ge.drawPick = function() {
                  if (!this.static) {
                    var Ce = this.pickBuffer;
                    this.gl, this._pickTimeout = null, Ce.begin();
                    for (var Se = 1, we = this.objects, Ee = 0; Ee < we.length; ++Ee) Se = we[Ee].drawPick(Se);
                    Ce.end();
                  }
                }, ge.pick = function(Ce, Se) {
                  if (!this.static) {
                    var we = this.pixelRatio, Ee = this.pickPixelRatio, xe = this.viewBox, Le = 0 | Math.round((Ce - xe[0] / we) * Ee), Qe = 0 | Math.round((Se - xe[1] / we) * Ee), ke = this.pickBuffer.query(Le, Qe, this.pickRadius);
                    if (!ke) return null;
                    for (var Me = ke.id + (ke.value[0] << 8) + (ke.value[1] << 16) + (ke.value[2] << 24), Re = this.objects, De = 0; De < Re.length; ++De) {
                      var Pe = Re[De].pick(Le, Qe, Me);
                      if (Pe) return Pe;
                    }
                    return null;
                  }
                }, ge.setScreenBox = function(Ce) {
                  var Se = this.screenBox, we = this.pixelRatio;
                  Se[0] = 0 | Math.round(Ce[0] * we), Se[1] = 0 | Math.round(Ce[1] * we), Se[2] = 0 | Math.round(Ce[2] * we), Se[3] = 0 | Math.round(Ce[3] * we), this.setDirty();
                }, ge.setDataBox = function(Ce) {
                  var Se = this.dataBox;
                  (Se[0] !== Ce[0] || Se[1] !== Ce[1] || Se[2] !== Ce[2] || Se[3] !== Ce[3]) && (Se[0] = Ce[0], Se[1] = Ce[1], Se[2] = Ce[2], Se[3] = Ce[3], this.setDirty());
                }, ge.setViewBox = function(Ce) {
                  var Se = this.pixelRatio, we = this.viewBox;
                  we[0] = 0 | Math.round(Ce[0] * Se), we[1] = 0 | Math.round(Ce[1] * Se), we[2] = 0 | Math.round(Ce[2] * Se), we[3] = 0 | Math.round(Ce[3] * Se);
                  var Ee = this.pickPixelRatio;
                  this.pickBuffer.shape = [0 | Math.round((Ce[2] - Ce[0]) * Ee), 0 | Math.round((Ce[3] - Ce[1]) * Ee)], this.setDirty();
                }, ge.update = function(Ce) {
                  Ce = Ce || {};
                  var Se = this.gl;
                  this.pixelRatio = Ce.pixelRatio || 1;
                  var we = this.pixelRatio;
                  this.pickPixelRatio = Math.max(we, 1), this.setScreenBox(Ce.screenBox || [0, 0, Se.drawingBufferWidth / we, Se.drawingBufferHeight / we]), this.screenBox, this.setViewBox(Ce.viewBox || [0.125 * (this.screenBox[2] - this.screenBox[0]) / we, 0.125 * (this.screenBox[3] - this.screenBox[1]) / we, 0.875 * (this.screenBox[2] - this.screenBox[0]) / we, 0.875 * (this.screenBox[3] - this.screenBox[1]) / we]);
                  var Ee = this.viewBox, xe = (Ee[2] - Ee[0]) / (Ee[3] - Ee[1]);
                  this.setDataBox(Ce.dataBox || [-10, -10 / xe, 10, 10 / xe]), this.borderColor = Ce.borderColor !== !1 && (Ce.borderColor || [0, 0, 0, 0]).slice(), this.backgroundColor = (Ce.backgroundColor || [0, 0, 0, 0]).slice(), this.gridLineEnable = (Ce.gridLineEnable || [!0, !0]).slice(), this.gridLineWidth = (Ce.gridLineWidth || [1, 1]).slice(), this.gridLineColor = ve(Ce.gridLineColor || [[0.5, 0.5, 0.5, 1], [0.5, 0.5, 0.5, 1]]), this.zeroLineEnable = (Ce.zeroLineEnable || [!0, !0]).slice(), this.zeroLineWidth = (Ce.zeroLineWidth || [4, 4]).slice(), this.zeroLineColor = ve(Ce.zeroLineColor || [[0, 0, 0, 1], [0, 0, 0, 1]]), this.tickMarkLength = (Ce.tickMarkLength || [0, 0, 0, 0]).slice(), this.tickMarkWidth = (Ce.tickMarkWidth || [0, 0, 0, 0]).slice(), this.tickMarkColor = ve(Ce.tickMarkColor || [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]]), this.titleCenter = (Ce.titleCenter || [0.5 * (Ee[0] + Ee[2]) / we, (Ee[3] + 120) / we]).slice(), this.titleEnable = !("titleEnable" in Ce) || !!Ce.titleEnable, this.titleAngle = Ce.titleAngle || 0, this.titleColor = (Ce.titleColor || [0, 0, 0, 1]).slice(), this.labelPad = (Ce.labelPad || [15, 15, 15, 15]).slice(), this.labelAngle = (Ce.labelAngle || [0, Math.PI / 2, 0, 3 * Math.PI / 2]).slice(), this.labelEnable = (Ce.labelEnable || [!0, !0, !0, !0]).slice(), this.labelColor = ve(Ce.labelColor || [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]]), this.tickPad = (Ce.tickPad || [15, 15, 15, 15]).slice(), this.tickAngle = (Ce.tickAngle || [0, 0, 0, 0]).slice(), this.tickEnable = (Ce.tickEnable || [!0, !0, !0, !0]).slice(), this.tickColor = ve(Ce.tickColor || [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]]), this.borderLineEnable = (Ce.borderLineEnable || [!0, !0, !0, !0]).slice(), this.borderLineWidth = (Ce.borderLineWidth || [2, 2, 2, 2]).slice(), this.borderLineColor = ve(Ce.borderLineColor || [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]]);
                  var Le = Ce.ticks || [[], []], Qe = this._tickBounds;
                  Qe[0] = Qe[1] = 1 / 0, Qe[2] = Qe[3] = -1 / 0;
                  for (var ke = 0; ke < 2; ++ke) {
                    var Me = Le[ke].slice(0);
                    Me.length !== 0 && (Me.sort(be), Qe[ke] = Math.min(Qe[ke], Me[0].x), Qe[ke + 2] = Math.max(Qe[ke + 2], Me[Me.length - 1].x));
                  }
                  this.grid.update({ bounds: Qe, ticks: Le }), this.text.update({ bounds: Qe, ticks: Le, labels: Ce.labels || ["x", "y"], labelSize: Ce.labelSize || [12, 12], labelFont: Ce.labelFont || ["sans-serif", "sans-serif"], labelFontStyle: Ce.labelFontStyle || ["normal", "normal"], labelFontWeight: Ce.labelFontWeight || ["normal", "normal"], labelFontVariant: Ce.labelFontVariant || ["normal", "normal"], title: Ce.title || "", titleSize: Ce.titleSize || 18, titleFont: Ce.titleFont || "sans-serif", titleFontStyle: Ce.titleFontStyle || "normal", titleFontWeight: Ce.titleFontWeight || "normal", titleFontVariant: Ce.titleFontVariant || "normal" }), this.static = !!Ce.static, this.setDirty();
                }, ge.dispose = function() {
                  this.box.dispose(), this.grid.dispose(), this.text.dispose(), this.line.dispose();
                  for (var Ce = this.objects.length - 1; Ce >= 0; --Ce) this.objects[Ce].dispose();
                  for (this.objects.length = 0, Ce = this.overlays.length - 1; Ce >= 0; --Ce) this.overlays[Ce].dispose();
                  this.overlays.length = 0, this.gl = null;
                }, ge.addObject = function(Ce) {
                  this.objects.indexOf(Ce) < 0 && (this.objects.push(Ce), this.setDirty());
                }, ge.removeObject = function(Ce) {
                  for (var Se = this.objects, we = 0; we < Se.length; ++we) if (Se[we] === Ce) {
                    Se.splice(we, 1), this.setDirty();
                    break;
                  }
                }, ge.addOverlay = function(Ce) {
                  this.overlays.indexOf(Ce) < 0 && (this.overlays.push(Ce), this.setOverlayDirty());
                }, ge.removeOverlay = function(Ce) {
                  for (var Se = this.overlays, we = 0; we < Se.length; ++we) if (Se[we] === Ce) {
                    Se.splice(we, 1), this.setOverlayDirty();
                    break;
                  }
                };
              }, 4437: function(ce, le, ue) {
                ce.exports = function(ge, ve) {
                  ge = ge || document.body;
                  var be = [0.01, 1 / 0];
                  "distanceLimits" in (ve = ve || {}) && (be[0] = ve.distanceLimits[0], be[1] = ve.distanceLimits[1]), "zoomMin" in ve && (be[0] = ve.zoomMin), "zoomMax" in ve && (be[1] = ve.zoomMax);
                  var Ce = fe({ center: ve.center || [0, 0, 0], up: ve.up || [0, 1, 0], eye: ve.eye || [0, 0, 10], mode: ve.mode || "orbit", distanceLimits: be }), Se = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], we = 0, Ee = ge.clientWidth, xe = ge.clientHeight, Le = { keyBindingMode: "rotate", enableWheel: !0, view: Ce, element: ge, delay: ve.delay || 16, rotateSpeed: ve.rotateSpeed || 1, zoomSpeed: ve.zoomSpeed || 1, translateSpeed: ve.translateSpeed || 1, flipX: !!ve.flipX, flipY: !!ve.flipY, modes: Ce.modes, _ortho: ve._ortho || ve.projection && ve.projection.type === "orthographic" || !1, tick: function() {
                    var Qe = de(), ke = this.delay, Me = Qe - 2 * ke;
                    Ce.idle(Qe - ke), Ce.recalcMatrix(Me), Ce.flush(Qe - (100 + 2 * ke));
                    for (var Re = !0, De = Ce.computedMatrix, Pe = 0; Pe < 16; ++Pe) Re = Re && Se[Pe] === De[Pe], Se[Pe] = De[Pe];
                    var $e = ge.clientWidth === Ee && ge.clientHeight === xe;
                    return Ee = ge.clientWidth, xe = ge.clientHeight, Re ? !$e : (we = Math.exp(Ce.computedRadius[0]), !0);
                  }, lookAt: function(Qe, ke, Me) {
                    Ce.lookAt(Ce.lastT(), Qe, ke, Me);
                  }, rotate: function(Qe, ke, Me) {
                    Ce.rotate(Ce.lastT(), Qe, ke, Me);
                  }, pan: function(Qe, ke, Me) {
                    Ce.pan(Ce.lastT(), Qe, ke, Me);
                  }, translate: function(Qe, ke, Me) {
                    Ce.translate(Ce.lastT(), Qe, ke, Me);
                  } };
                  return Object.defineProperties(Le, { matrix: { get: function() {
                    return Ce.computedMatrix;
                  }, set: function(Qe) {
                    return Ce.setMatrix(Ce.lastT(), Qe), Ce.computedMatrix;
                  }, enumerable: !0 }, mode: { get: function() {
                    return Ce.getMode();
                  }, set: function(Qe) {
                    var ke = Ce.computedUp.slice(), Me = Ce.computedEye.slice(), Re = Ce.computedCenter.slice();
                    if (Ce.setMode(Qe), Qe === "turntable") {
                      var De = de();
                      Ce._active.lookAt(De, Me, Re, ke), Ce._active.lookAt(De + 500, Me, Re, [0, 0, 1]), Ce._active.flush(De);
                    }
                    return Ce.getMode();
                  }, enumerable: !0 }, center: { get: function() {
                    return Ce.computedCenter;
                  }, set: function(Qe) {
                    return Ce.lookAt(Ce.lastT(), null, Qe), Ce.computedCenter;
                  }, enumerable: !0 }, eye: { get: function() {
                    return Ce.computedEye;
                  }, set: function(Qe) {
                    return Ce.lookAt(Ce.lastT(), Qe), Ce.computedEye;
                  }, enumerable: !0 }, up: { get: function() {
                    return Ce.computedUp;
                  }, set: function(Qe) {
                    return Ce.lookAt(Ce.lastT(), null, null, Qe), Ce.computedUp;
                  }, enumerable: !0 }, distance: { get: function() {
                    return we;
                  }, set: function(Qe) {
                    return Ce.setDistance(Ce.lastT(), Qe), Qe;
                  }, enumerable: !0 }, distanceLimits: { get: function() {
                    return Ce.getDistanceLimits(be);
                  }, set: function(Qe) {
                    return Ce.setDistanceLimits(Qe), Qe;
                  }, enumerable: !0 } }), ge.addEventListener("contextmenu", function(Qe) {
                    return Qe.preventDefault(), !1;
                  }), Le._lastX = -1, Le._lastY = -1, Le._lastMods = { shift: !1, control: !1, alt: !1, meta: !1 }, Le.enableMouseListeners = function() {
                    function Qe(ke, Me, Re, De) {
                      var Pe = Le.keyBindingMode;
                      if (Pe !== !1) {
                        var $e = Pe === "rotate", qe = Pe === "pan", Xe = Pe === "zoom", Ve = !!De.control, Ze = !!De.alt, Ge = !!De.shift, ze = !!(1 & ke), We = !!(2 & ke), at = !!(4 & ke), ct = 1 / ge.clientHeight, et = ct * (Me - Le._lastX), dt = ct * (Re - Le._lastY), Je = Le.flipX ? 1 : -1, it = Le.flipY ? 1 : -1, st = Math.PI * Le.rotateSpeed, gt = de();
                        if (Le._lastX !== -1 && Le._lastY !== -1 && (($e && ze && !Ve && !Ze && !Ge || ze && !Ve && !Ze && Ge) && Ce.rotate(gt, Je * st * et, -it * st * dt, 0), (qe && ze && !Ve && !Ze && !Ge || We || ze && Ve && !Ze && !Ge) && Ce.pan(gt, -Le.translateSpeed * et * we, Le.translateSpeed * dt * we, 0), Xe && ze && !Ve && !Ze && !Ge || at || ze && !Ve && Ze && !Ge)) {
                          var bt = -Le.zoomSpeed * dt / window.innerHeight * (gt - Ce.lastT()) * 100;
                          Ce.pan(gt, 0, 0, we * (Math.exp(bt) - 1));
                        }
                        return Le._lastX = Me, Le._lastY = Re, Le._lastMods = De, !0;
                      }
                    }
                    Le.mouseListener = he(ge, Qe), ge.addEventListener("touchstart", function(ke) {
                      var Me = pe(ke.changedTouches[0], ge);
                      Qe(0, Me[0], Me[1], Le._lastMods), Qe(1, Me[0], Me[1], Le._lastMods);
                    }, !!ye && { passive: !0 }), ge.addEventListener("touchmove", function(ke) {
                      var Me = pe(ke.changedTouches[0], ge);
                      Qe(1, Me[0], Me[1], Le._lastMods), ke.preventDefault();
                    }, !!ye && { passive: !1 }), ge.addEventListener("touchend", function(ke) {
                      Qe(0, Le._lastX, Le._lastY, Le._lastMods);
                    }, !!ye && { passive: !0 }), Le.wheelListener = me(ge, function(ke, Me) {
                      if (Le.keyBindingMode !== !1 && Le.enableWheel) {
                        var Re = Le.flipX ? 1 : -1, De = Le.flipY ? 1 : -1, Pe = de();
                        if (Math.abs(ke) > Math.abs(Me)) Ce.rotate(Pe, 0, 0, -ke * Re * Math.PI * Le.rotateSpeed / window.innerWidth);
                        else if (!Le._ortho) {
                          var $e = -Le.zoomSpeed * De * Me / window.innerHeight * (Pe - Ce.lastT()) / 20;
                          Ce.pan(Pe, 0, 0, we * (Math.exp($e) - 1));
                        }
                      }
                    }, !0);
                  }, Le.enableMouseListeners(), Le;
                };
                var de = ue(3025), fe = ue(6296), he = ue(351), me = ue(8512), pe = ue(24), ye = ue(7520);
              }, 799: function(ce, le, ue) {
                var de = ue(3236), fe = ue(9405), he = de([`precision mediump float;
#define GLSLIFY 1
attribute vec2 position;
varying vec2 uv;
void main() {
  uv = position;
  gl_Position = vec4(position, 0, 1);
}`]), me = de([`precision mediump float;
#define GLSLIFY 1

uniform sampler2D accumBuffer;
varying vec2 uv;

void main() {
  vec4 accum = texture2D(accumBuffer, 0.5 * (uv + 1.0));
  gl_FragColor = min(vec4(1,1,1,1), accum);
}`]);
                ce.exports = function(pe) {
                  return fe(pe, he, me, null, [{ name: "position", type: "vec2" }]);
                };
              }, 4100: function(ce, le, ue) {
                var de = ue(4437), fe = ue(3837), he = ue(5445), me = ue(4449), pe = ue(3589), ye = ue(2260), ge = ue(7169), ve = ue(351), be = ue(4772), Ce = ue(4040), Se = ue(799), we = ue(9216)({ tablet: !0, featureDetect: !0 });
                function Ee() {
                  this.mouse = [-1, -1], this.screen = null, this.distance = 1 / 0, this.index = null, this.dataCoordinate = null, this.dataPosition = null, this.object = null, this.data = null;
                }
                function xe(Qe) {
                  var ke = Math.round(Math.log(Math.abs(Qe)) / Math.log(10));
                  if (ke < 0) {
                    var Me = Math.round(Math.pow(10, -ke));
                    return Math.ceil(Qe * Me) / Me;
                  }
                  return ke > 0 ? (Me = Math.round(Math.pow(10, ke)), Math.ceil(Qe / Me) * Me) : Math.ceil(Qe);
                }
                function Le(Qe) {
                  return typeof Qe != "boolean" || Qe;
                }
                ce.exports = { createScene: function(Qe) {
                  (Qe = Qe || {}).camera = Qe.camera || {};
                  var ke = Qe.canvas;
                  ke || (ke = document.createElement("canvas"), Qe.container ? Qe.container.appendChild(ke) : document.body.appendChild(ke));
                  var Me = Qe.gl;
                  if (Me || (Qe.glOptions && (we = !!Qe.glOptions.preserveDrawingBuffer), Me = function(Xt, Nt) {
                    var Gt = null;
                    try {
                      (Gt = Xt.getContext("webgl", Nt)) || (Gt = Xt.getContext("experimental-webgl", Nt));
                    } catch {
                      return null;
                    }
                    return Gt;
                  }(ke, Qe.glOptions || { premultipliedAlpha: !0, antialias: !0, preserveDrawingBuffer: we })), !Me) throw new Error("webgl not supported");
                  var Re = Qe.bounds || [[-10, -10, -10], [10, 10, 10]], De = new Ee(), Pe = ye(Me, Me.drawingBufferWidth, Me.drawingBufferHeight, { preferFloat: !we }), $e = Se(Me), qe = Qe.cameraObject && Qe.cameraObject._ortho === !0 || Qe.camera.projection && Qe.camera.projection.type === "orthographic" || !1, Xe = { eye: Qe.camera.eye || [2, 0, 0], center: Qe.camera.center || [0, 0, 0], up: Qe.camera.up || [0, 1, 0], zoomMin: Qe.camera.zoomMax || 0.1, zoomMax: Qe.camera.zoomMin || 100, mode: Qe.camera.mode || "turntable", _ortho: qe }, Ve = Qe.axes || {}, Ze = fe(Me, Ve);
                  Ze.enable = !Ve.disable;
                  var Ge = Qe.spikes || {}, ze = me(Me, Ge), We = [], at = [], ct = [], et = [], dt = !0, Je = !0, it = { view: null, projection: new Array(16), model: new Array(16), _ortho: !1 }, st = (Je = !0, [Me.drawingBufferWidth, Me.drawingBufferHeight]), gt = Qe.cameraObject || de(ke, Xe), bt = { gl: Me, contextLost: !1, pixelRatio: Qe.pixelRatio || 1, canvas: ke, selection: De, camera: gt, axes: Ze, axesPixels: null, spikes: ze, bounds: Re, objects: We, shape: st, aspect: Qe.aspectRatio || [1, 1, 1], pickRadius: Qe.pickRadius || 10, zNear: Qe.zNear || 0.01, zFar: Qe.zFar || 1e3, fovy: Qe.fovy || Math.PI / 4, clearColor: Qe.clearColor || [0, 0, 0, 0], autoResize: Le(Qe.autoResize), autoBounds: Le(Qe.autoBounds), autoScale: !!Qe.autoScale, autoCenter: Le(Qe.autoCenter), clipToBounds: Le(Qe.clipToBounds), snapToData: !!Qe.snapToData, onselect: Qe.onselect || null, onrender: Qe.onrender || null, onclick: Qe.onclick || null, cameraParams: it, oncontextloss: null, mouseListener: null, _stopped: !1, getAspectratio: function() {
                    return { x: this.aspect[0], y: this.aspect[1], z: this.aspect[2] };
                  }, setAspectratio: function(Xt) {
                    this.aspect[0] = Xt.x, this.aspect[1] = Xt.y, this.aspect[2] = Xt.z, Je = !0;
                  }, setBounds: function(Xt, Nt) {
                    this.bounds[0][Xt] = Nt.min, this.bounds[1][Xt] = Nt.max;
                  }, setClearColor: function(Xt) {
                    this.clearColor = Xt;
                  }, clearRGBA: function() {
                    this.gl.clearColor(this.clearColor[0], this.clearColor[1], this.clearColor[2], this.clearColor[3]), this.gl.clear(this.gl.COLOR_BUFFER_BIT | this.gl.DEPTH_BUFFER_BIT);
                  } }, yt = [Me.drawingBufferWidth / bt.pixelRatio | 0, Me.drawingBufferHeight / bt.pixelRatio | 0];
                  function St() {
                    if (!bt._stopped && bt.autoResize) {
                      var Xt = ke.parentNode, Nt = 1, Gt = 1;
                      Xt && Xt !== document.body ? (Nt = Xt.clientWidth, Gt = Xt.clientHeight) : (Nt = window.innerWidth, Gt = window.innerHeight);
                      var Ft = 0 | Math.ceil(Nt * bt.pixelRatio), Tt = 0 | Math.ceil(Gt * bt.pixelRatio);
                      if (Ft !== ke.width || Tt !== ke.height) {
                        ke.width = Ft, ke.height = Tt;
                        var It = ke.style;
                        It.position = It.position || "absolute", It.left = "0px", It.top = "0px", It.width = Nt + "px", It.height = Gt + "px", dt = !0;
                      }
                    }
                  }
                  function Lt() {
                    for (var Xt = We.length, Nt = et.length, Gt = 0; Gt < Nt; ++Gt) ct[Gt] = 0;
                    e: for (Gt = 0; Gt < Xt; ++Gt) {
                      var Ft = We[Gt], Tt = Ft.pickSlots;
                      if (Tt) {
                        for (var It = 0; It < Nt; ++It) if (ct[It] + Tt < 255) {
                          at[Gt] = It, Ft.setPickBase(ct[It] + 1), ct[It] += Tt;
                          continue e;
                        }
                        var Ct = pe(Me, st);
                        at[Gt] = Nt, et.push(Ct), ct.push(Tt), Ft.setPickBase(1), Nt += 1;
                      } else at[Gt] = -1;
                    }
                    for (; Nt > 0 && ct[Nt - 1] === 0; ) ct.pop(), et.pop().dispose();
                  }
                  function Rt() {
                    if (bt.contextLost) return !0;
                    Me.isContextLost() && (bt.contextLost = !0, bt.mouseListener.enabled = !1, bt.selection.object = null, bt.oncontextloss && bt.oncontextloss());
                  }
                  bt.autoResize && St(), window.addEventListener("resize", St), bt.update = function(Xt) {
                    bt._stopped || (dt = !0, Je = !0);
                  }, bt.add = function(Xt) {
                    bt._stopped || (Xt.axes = Ze, We.push(Xt), at.push(-1), dt = !0, Je = !0, Lt());
                  }, bt.remove = function(Xt) {
                    if (!bt._stopped) {
                      var Nt = We.indexOf(Xt);
                      Nt < 0 || (We.splice(Nt, 1), at.pop(), dt = !0, Je = !0, Lt());
                    }
                  }, bt.dispose = function() {
                    if (!bt._stopped && (bt._stopped = !0, window.removeEventListener("resize", St), ke.removeEventListener("webglcontextlost", Rt), bt.mouseListener.enabled = !1, !bt.contextLost)) {
                      Ze.dispose(), ze.dispose();
                      for (var Xt = 0; Xt < We.length; ++Xt) We[Xt].dispose();
                      for (Pe.dispose(), Xt = 0; Xt < et.length; ++Xt) et[Xt].dispose();
                      $e.dispose(), Me = null, Ze = null, ze = null, We = [];
                    }
                  }, bt._mouseRotating = !1, bt._prevButtons = 0, bt.enableMouseListeners = function() {
                    bt.mouseListener = ve(ke, function(Xt, Nt, Gt) {
                      if (!bt._stopped) {
                        var Ft = et.length, Tt = We.length, It = De.object;
                        De.distance = 1 / 0, De.mouse[0] = Nt, De.mouse[1] = Gt, De.object = null, De.screen = null, De.dataCoordinate = De.dataPosition = null;
                        var Ct = !1;
                        if (Xt && bt._prevButtons) bt._mouseRotating = !0;
                        else {
                          bt._mouseRotating && (Je = !0), bt._mouseRotating = !1;
                          for (var Kt = 0; Kt < Ft; ++Kt) {
                            var Vt = et[Kt].query(Nt, yt[1] - Gt - 1, bt.pickRadius);
                            if (Vt) {
                              if (Vt.distance > De.distance) continue;
                              for (var Yt = 0; Yt < Tt; ++Yt) {
                                var sn = We[Yt];
                                if (at[Yt] === Kt) {
                                  var hn = sn.pick(Vt);
                                  hn && (De.buttons = Xt, De.screen = Vt.coord, De.distance = Vt.distance, De.object = sn, De.index = hn.distance, De.dataPosition = hn.position, De.dataCoordinate = hn.dataCoordinate, De.data = hn, Ct = !0);
                                }
                              }
                            }
                          }
                        }
                        It && It !== De.object && (It.highlight && It.highlight(null), dt = !0), De.object && (De.object.highlight && De.object.highlight(De.data), dt = !0), (Ct = Ct || De.object !== It) && bt.onselect && bt.onselect(De), 1 & Xt && !(1 & bt._prevButtons) && bt.onclick && bt.onclick(De), bt._prevButtons = Xt;
                      }
                    });
                  }, ke.addEventListener("webglcontextlost", Rt);
                  var vt = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], $t = [vt[0].slice(), vt[1].slice()];
                  function Pt() {
                    if (!Rt()) {
                      St();
                      var Xt = bt.camera.tick();
                      it.view = bt.camera.matrix, dt = dt || Xt, Je = Je || Xt, Ze.pixelRatio = bt.pixelRatio, ze.pixelRatio = bt.pixelRatio;
                      var Nt = We.length, Gt = vt[0], Ft = vt[1];
                      Gt[0] = Gt[1] = Gt[2] = 1 / 0, Ft[0] = Ft[1] = Ft[2] = -1 / 0;
                      for (var Tt = 0; Tt < Nt; ++Tt) {
                        (vn = We[Tt]).pixelRatio = bt.pixelRatio, vn.axes = bt.axes, dt = dt || !!vn.dirty, Je = Je || !!vn.dirty;
                        var It = vn.bounds;
                        if (It) for (var Ct = It[0], Kt = It[1], Vt = 0; Vt < 3; ++Vt) Gt[Vt] = Math.min(Gt[Vt], Ct[Vt]), Ft[Vt] = Math.max(Ft[Vt], Kt[Vt]);
                      }
                      var Yt = bt.bounds;
                      if (bt.autoBounds) for (Vt = 0; Vt < 3; ++Vt) {
                        if (Ft[Vt] < Gt[Vt]) Gt[Vt] = -1, Ft[Vt] = 1;
                        else {
                          Gt[Vt] === Ft[Vt] && (Gt[Vt] -= 1, Ft[Vt] += 1);
                          var sn = 0.05 * (Ft[Vt] - Gt[Vt]);
                          Gt[Vt] = Gt[Vt] - sn, Ft[Vt] = Ft[Vt] + sn;
                        }
                        Yt[0][Vt] = Gt[Vt], Yt[1][Vt] = Ft[Vt];
                      }
                      var hn = !1;
                      for (Vt = 0; Vt < 3; ++Vt) hn = hn || $t[0][Vt] !== Yt[0][Vt] || $t[1][Vt] !== Yt[1][Vt], $t[0][Vt] = Yt[0][Vt], $t[1][Vt] = Yt[1][Vt];
                      if (Je = Je || hn, dt = dt || hn) {
                        if (hn) {
                          var Qt = [0, 0, 0];
                          for (Tt = 0; Tt < 3; ++Tt) Qt[Tt] = xe((Yt[1][Tt] - Yt[0][Tt]) / 10);
                          Ze.autoTicks ? Ze.update({ bounds: Yt, tickSpacing: Qt }) : Ze.update({ bounds: Yt });
                        }
                        var Ut = Me.drawingBufferWidth, Wt = Me.drawingBufferHeight;
                        for (st[0] = Ut, st[1] = Wt, yt[0] = 0 | Math.max(Ut / bt.pixelRatio, 1), yt[1] = 0 | Math.max(Wt / bt.pixelRatio, 1), function(dn, Ht) {
                          var jt = dn.bounds, cn = dn.cameraParams, wn = cn.projection, Rn = cn.model, Pn = dn.gl.drawingBufferWidth, Ie = dn.gl.drawingBufferHeight, rt = dn.zNear, ft = dn.zFar, pt = dn.fovy, wt = Pn / Ie;
                          Ht ? (Ce(wn, -wt, wt, -1, 1, rt, ft), cn._ortho = !0) : (be(wn, pt, wt, rt, ft), cn._ortho = !1);
                          for (var Bt = 0; Bt < 16; ++Bt) Rn[Bt] = 0;
                          Rn[15] = 1;
                          var At = 0;
                          for (Bt = 0; Bt < 3; ++Bt) At = Math.max(At, jt[1][Bt] - jt[0][Bt]);
                          for (Bt = 0; Bt < 3; ++Bt) dn.autoScale ? Rn[5 * Bt] = dn.aspect[Bt] / (jt[1][Bt] - jt[0][Bt]) : Rn[5 * Bt] = 1 / At, dn.autoCenter && (Rn[12 + Bt] = 0.5 * -Rn[5 * Bt] * (jt[0][Bt] + jt[1][Bt]));
                        }(bt, qe), Tt = 0; Tt < Nt; ++Tt) (vn = We[Tt]).axesBounds = Yt, bt.clipToBounds && (vn.clipBounds = Yt);
                        De.object && (bt.snapToData ? ze.position = De.dataCoordinate : ze.position = De.dataPosition, ze.bounds = Yt), Je && (Je = !1, function() {
                          if (!Rt()) {
                            Me.colorMask(!0, !0, !0, !0), Me.depthMask(!0), Me.disable(Me.BLEND), Me.enable(Me.DEPTH_TEST), Me.depthFunc(Me.LEQUAL);
                            for (var dn = We.length, Ht = et.length, jt = 0; jt < Ht; ++jt) {
                              var cn = et[jt];
                              cn.shape = yt, cn.begin();
                              for (var wn = 0; wn < dn; ++wn) if (at[wn] === jt) {
                                var Rn = We[wn];
                                Rn.drawPick && (Rn.pixelRatio = 1, Rn.drawPick(it));
                              }
                              cn.end();
                            }
                          }
                        }()), bt.axesPixels = he(bt.axes, it, Ut, Wt), bt.onrender && bt.onrender(), Me.bindFramebuffer(Me.FRAMEBUFFER, null), Me.viewport(0, 0, Ut, Wt), bt.clearRGBA(), Me.depthMask(!0), Me.colorMask(!0, !0, !0, !0), Me.enable(Me.DEPTH_TEST), Me.depthFunc(Me.LEQUAL), Me.disable(Me.BLEND), Me.disable(Me.CULL_FACE);
                        var rn = !1;
                        for (Ze.enable && (rn = rn || Ze.isTransparent(), Ze.draw(it)), ze.axes = Ze, De.object && ze.draw(it), Me.disable(Me.CULL_FACE), Tt = 0; Tt < Nt; ++Tt) (vn = We[Tt]).axes = Ze, vn.pixelRatio = bt.pixelRatio, vn.isOpaque && vn.isOpaque() && vn.draw(it), vn.isTransparent && vn.isTransparent() && (rn = !0);
                        if (rn) {
                          for (Pe.shape = st, Pe.bind(), Me.clear(Me.DEPTH_BUFFER_BIT), Me.colorMask(!1, !1, !1, !1), Me.depthMask(!0), Me.depthFunc(Me.LESS), Ze.enable && Ze.isTransparent() && Ze.drawTransparent(it), Tt = 0; Tt < Nt; ++Tt) (vn = We[Tt]).isOpaque && vn.isOpaque() && vn.draw(it);
                          for (Me.enable(Me.BLEND), Me.blendEquation(Me.FUNC_ADD), Me.blendFunc(Me.ONE, Me.ONE_MINUS_SRC_ALPHA), Me.colorMask(!0, !0, !0, !0), Me.depthMask(!1), Me.clearColor(0, 0, 0, 0), Me.clear(Me.COLOR_BUFFER_BIT), Ze.isTransparent() && Ze.drawTransparent(it), Tt = 0; Tt < Nt; ++Tt) {
                            var vn;
                            (vn = We[Tt]).isTransparent && vn.isTransparent() && vn.drawTransparent(it);
                          }
                          Me.bindFramebuffer(Me.FRAMEBUFFER, null), Me.blendFunc(Me.ONE, Me.ONE_MINUS_SRC_ALPHA), Me.disable(Me.DEPTH_TEST), $e.bind(), Pe.color[0].bind(0), $e.uniforms.accumBuffer = 0, ge(Me), Me.disable(Me.BLEND);
                        }
                        for (dt = !1, Tt = 0; Tt < Nt; ++Tt) We[Tt].dirty = !1;
                      }
                    }
                  }
                  return bt.enableMouseListeners(), function Xt() {
                    bt._stopped || bt.contextLost || (Pt(), requestAnimationFrame(Xt));
                  }(), bt.redraw = function() {
                    bt._stopped || (dt = !0, Pt());
                  }, bt;
                }, createCamera: de };
              }, 6640: function(ce, le, ue) {
                var de = ue(3236);
                le.pointVertex = de([`precision mediump float;
#define GLSLIFY 1

attribute vec2 position;

uniform mat3 matrix;
uniform float pointSize;
uniform float pointCloud;

highp float rand(vec2 co) {
  highp float a = 12.9898;
  highp float b = 78.233;
  highp float c = 43758.5453;
  highp float d = dot(co.xy, vec2(a, b));
  highp float e = mod(d, 3.14);
  return fract(sin(e) * c);
}

void main() {
  vec3 hgPosition = matrix * vec3(position, 1);
  gl_Position  = vec4(hgPosition.xy, 0, hgPosition.z);
    // if we don't jitter the point size a bit, overall point cloud
    // saturation 'jumps' on zooming, which is disturbing and confusing
  gl_PointSize = pointSize * ((19.5 + rand(position)) / 20.0);
  if(pointCloud != 0.0) { // pointCloud is truthy
    // get the same square surface as circle would be
    gl_PointSize *= 0.886;
  }
}`]), le.pointFragment = de([`precision mediump float;
#define GLSLIFY 1

uniform vec4 color, borderColor;
uniform float centerFraction;
uniform float pointCloud;

void main() {
  float radius;
  vec4 baseColor;
  if(pointCloud != 0.0) { // pointCloud is truthy
    if(centerFraction == 1.0) {
      gl_FragColor = color;
    } else {
      gl_FragColor = mix(borderColor, color, centerFraction);
    }
  } else {
    radius = length(2.0 * gl_PointCoord.xy - 1.0);
    if(radius > 1.0) {
      discard;
    }
    baseColor = mix(borderColor, color, step(radius, centerFraction));
    gl_FragColor = vec4(baseColor.rgb * baseColor.a, baseColor.a);
  }
}
`]), le.pickVertex = de([`precision mediump float;
#define GLSLIFY 1

attribute vec2 position;
attribute vec4 pickId;

uniform mat3 matrix;
uniform float pointSize;
uniform vec4 pickOffset;

varying vec4 fragId;

void main() {
  vec3 hgPosition = matrix * vec3(position, 1);
  gl_Position  = vec4(hgPosition.xy, 0, hgPosition.z);
  gl_PointSize = pointSize;

  vec4 id = pickId + pickOffset;
  id.y += floor(id.x / 256.0);
  id.x -= floor(id.x / 256.0) * 256.0;

  id.z += floor(id.y / 256.0);
  id.y -= floor(id.y / 256.0) * 256.0;

  id.w += floor(id.z / 256.0);
  id.z -= floor(id.z / 256.0) * 256.0;

  fragId = id;
}
`]), le.pickFragment = de([`precision mediump float;
#define GLSLIFY 1

varying vec4 fragId;

void main() {
  float radius = length(2.0 * gl_PointCoord.xy - 1.0);
  if(radius > 1.0) {
    discard;
  }
  gl_FragColor = fragId / 255.0;
}
`]);
              }, 4696: function(ce, le, ue) {
                var de = ue(9405), fe = ue(2762), he = ue(1888), me = ue(6640);
                function pe(be, Ce, Se, we, Ee) {
                  this.plot = be, this.offsetBuffer = Ce, this.pickBuffer = Se, this.shader = we, this.pickShader = Ee, this.sizeMin = 0.5, this.sizeMinCap = 2, this.sizeMax = 20, this.areaRatio = 1, this.pointCount = 0, this.color = [1, 0, 0, 1], this.borderColor = [0, 0, 0, 1], this.blend = !1, this.pickOffset = 0, this.points = null;
                }
                ce.exports = function(be, Ce) {
                  var Se = be.gl, we = new pe(be, fe(Se), fe(Se), de(Se, me.pointVertex, me.pointFragment), de(Se, me.pickVertex, me.pickFragment));
                  return we.update(Ce), be.addObject(we), we;
                };
                var ye, ge, ve = pe.prototype;
                ve.dispose = function() {
                  this.shader.dispose(), this.pickShader.dispose(), this.offsetBuffer.dispose(), this.pickBuffer.dispose(), this.plot.removeObject(this);
                }, ve.update = function(be) {
                  var Ce;
                  function Se(Me, Re) {
                    return Me in be ? be[Me] : Re;
                  }
                  be = be || {}, this.sizeMin = Se("sizeMin", 0.5), this.sizeMax = Se("sizeMax", 20), this.color = Se("color", [1, 0, 0, 1]).slice(), this.areaRatio = Se("areaRatio", 1), this.borderColor = Se("borderColor", [0, 0, 0, 1]).slice(), this.blend = Se("blend", !1);
                  var we = be.positions.length >>> 1, Ee = be.positions instanceof Float32Array, xe = be.idToIndex instanceof Int32Array && be.idToIndex.length >= we, Le = be.positions, Qe = Ee ? Le : he.mallocFloat32(Le.length), ke = xe ? be.idToIndex : he.mallocInt32(we);
                  if (Ee || Qe.set(Le), !xe) for (Qe.set(Le), Ce = 0; Ce < we; Ce++) ke[Ce] = Ce;
                  this.points = Le, this.offsetBuffer.update(Qe), this.pickBuffer.update(ke), Ee || he.free(Qe), xe || he.free(ke), this.pointCount = we, this.pickOffset = 0;
                }, ve.unifiedDraw = (ye = [1, 0, 0, 0, 1, 0, 0, 0, 1], ge = [0, 0, 0, 0], function(be) {
                  var Ce = be !== void 0, Se = Ce ? this.pickShader : this.shader, we = this.plot.gl, Ee = this.plot.dataBox;
                  if (this.pointCount === 0) return be;
                  var xe = Ee[2] - Ee[0], Le = Ee[3] - Ee[1], Qe = function(De, Pe) {
                    var $e, qe = 0, Xe = De.length >>> 1;
                    for ($e = 0; $e < Xe; $e++) {
                      var Ve = De[2 * $e], Ze = De[2 * $e + 1];
                      Ve >= Pe[0] && Ve <= Pe[2] && Ze >= Pe[1] && Ze <= Pe[3] && qe++;
                    }
                    return qe;
                  }(this.points, Ee), ke = this.plot.pickPixelRatio * Math.max(Math.min(this.sizeMinCap, this.sizeMin), Math.min(this.sizeMax, this.sizeMax / Math.pow(Qe, 0.33333)));
                  ye[0] = 2 / xe, ye[4] = 2 / Le, ye[6] = -2 * Ee[0] / xe - 1, ye[7] = -2 * Ee[1] / Le - 1, this.offsetBuffer.bind(), Se.bind(), Se.attributes.position.pointer(), Se.uniforms.matrix = ye, Se.uniforms.color = this.color, Se.uniforms.borderColor = this.borderColor, Se.uniforms.pointCloud = ke < 5, Se.uniforms.pointSize = ke, Se.uniforms.centerFraction = Math.min(1, Math.max(0, Math.sqrt(1 - this.areaRatio))), Ce && (ge[0] = 255 & be, ge[1] = be >> 8 & 255, ge[2] = be >> 16 & 255, ge[3] = be >> 24 & 255, this.pickBuffer.bind(), Se.attributes.pickId.pointer(we.UNSIGNED_BYTE), Se.uniforms.pickOffset = ge, this.pickOffset = be);
                  var Me = we.getParameter(we.BLEND), Re = we.getParameter(we.DITHER);
                  return Me && !this.blend && we.disable(we.BLEND), Re && we.disable(we.DITHER), we.drawArrays(we.POINTS, 0, this.pointCount), Me && !this.blend && we.enable(we.BLEND), Re && we.enable(we.DITHER), be + this.pointCount;
                }), ve.draw = ve.unifiedDraw, ve.drawPick = ve.unifiedDraw, ve.pick = function(be, Ce, Se) {
                  var we = this.pickOffset, Ee = this.pointCount;
                  if (Se < we || Se >= we + Ee) return null;
                  var xe = Se - we, Le = this.points;
                  return { object: this, pointId: xe, dataCoord: [Le[2 * xe], Le[2 * xe + 1]] };
                };
              }, 783: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he, me, pe, ye, ge, ve = ue[0], be = ue[1], Ce = ue[2], Se = ue[3], we = de[0], Ee = de[1], xe = de[2], Le = de[3];
                  return (me = ve * we + be * Ee + Ce * xe + Se * Le) < 0 && (me = -me, we = -we, Ee = -Ee, xe = -xe, Le = -Le), 1 - me > 1e-6 ? (he = Math.acos(me), pe = Math.sin(he), ye = Math.sin((1 - fe) * he) / pe, ge = Math.sin(fe * he) / pe) : (ye = 1 - fe, ge = fe), le[0] = ye * ve + ge * we, le[1] = ye * be + ge * Ee, le[2] = ye * Ce + ge * xe, le[3] = ye * Se + ge * Le, le;
                };
              }, 5964: function(ce) {
                ce.exports = function(le) {
                  return le || le === 0 ? le.toString() : "";
                };
              }, 9366: function(ce, le, ue) {
                var de = ue(4359);
                ce.exports = function(he, me, pe) {
                  var ye = [me.style, me.weight, me.variant, me.family].join("_"), ge = fe[ye];
                  if (ge || (ge = fe[ye] = {}), he in ge) return ge[he];
                  var ve = { textAlign: "center", textBaseline: "middle", lineHeight: 1, font: me.family, fontStyle: me.style, fontWeight: me.weight, fontVariant: me.variant, lineSpacing: 1.25, styletags: { breaklines: !0, bolds: !0, italics: !0, subscripts: !0, superscripts: !0 }, triangles: !0 }, be = de(he, ve);
                  ve.triangles = !1;
                  var Ce, Se, we = de(he, ve);
                  if (pe && pe !== 1) {
                    for (Ce = 0; Ce < be.positions.length; ++Ce) for (Se = 0; Se < be.positions[Ce].length; ++Se) be.positions[Ce][Se] /= pe;
                    for (Ce = 0; Ce < we.positions.length; ++Ce) for (Se = 0; Se < we.positions[Ce].length; ++Se) we.positions[Ce][Se] /= pe;
                  }
                  var Ee = [[1 / 0, 1 / 0], [-1 / 0, -1 / 0]], xe = we.positions.length;
                  for (Ce = 0; Ce < xe; ++Ce) {
                    var Le = we.positions[Ce];
                    for (Se = 0; Se < 2; ++Se) Ee[0][Se] = Math.min(Ee[0][Se], Le[Se]), Ee[1][Se] = Math.max(Ee[1][Se], Le[Se]);
                  }
                  return ge[he] = [be, we, Ee];
                };
                var fe = {};
              }, 1283: function(ce, le, ue) {
                var de = ue(9405), fe = ue(3236), he = fe([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

attribute vec3 position;
attribute vec4 color;
attribute vec2 glyph;
attribute vec4 id;

uniform vec4 highlightId;
uniform float highlightScale;
uniform mat4 model, view, projection;
uniform vec3 clipBounds[2];

varying vec4 interpColor;
varying vec4 pickId;
varying vec3 dataCoordinate;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], position)) {

    gl_Position = vec4(0,0,0,0);
  } else {
    float scale = 1.0;
    if(distance(highlightId, id) < 0.0001) {
      scale = highlightScale;
    }

    vec4 worldPosition = model * vec4(position, 1);
    vec4 viewPosition = view * worldPosition;
    viewPosition = viewPosition / viewPosition.w;
    vec4 clipPosition = projection * (viewPosition + scale * vec4(glyph.x, -glyph.y, 0, 0));

    gl_Position = clipPosition;
    interpColor = color;
    pickId = id;
    dataCoordinate = position;
  }
}`]), me = fe([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

attribute vec3 position;
attribute vec4 color;
attribute vec2 glyph;
attribute vec4 id;

uniform mat4 model, view, projection;
uniform vec2 screenSize;
uniform vec3 clipBounds[2];
uniform float highlightScale, pixelRatio;
uniform vec4 highlightId;

varying vec4 interpColor;
varying vec4 pickId;
varying vec3 dataCoordinate;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], position)) {

    gl_Position = vec4(0,0,0,0);
  } else {
    float scale = pixelRatio;
    if(distance(highlightId.bgr, id.bgr) < 0.001) {
      scale *= highlightScale;
    }

    vec4 worldPosition = model * vec4(position, 1.0);
    vec4 viewPosition = view * worldPosition;
    vec4 clipPosition = projection * viewPosition;
    clipPosition /= clipPosition.w;

    gl_Position = clipPosition + vec4(screenSize * scale * vec2(glyph.x, -glyph.y), 0.0, 0.0);
    interpColor = color;
    pickId = id;
    dataCoordinate = position;
  }
}`]), pe = fe([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

attribute vec3 position;
attribute vec4 color;
attribute vec2 glyph;
attribute vec4 id;

uniform float highlightScale;
uniform vec4 highlightId;
uniform vec3 axes[2];
uniform mat4 model, view, projection;
uniform vec2 screenSize;
uniform vec3 clipBounds[2];
uniform float scale, pixelRatio;

varying vec4 interpColor;
varying vec4 pickId;
varying vec3 dataCoordinate;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], position)) {

    gl_Position = vec4(0,0,0,0);
  } else {
    float lscale = pixelRatio * scale;
    if(distance(highlightId, id) < 0.0001) {
      lscale *= highlightScale;
    }

    vec4 clipCenter   = projection * (view * (model * vec4(position, 1)));
    vec3 dataPosition = position + 0.5*lscale*(axes[0] * glyph.x + axes[1] * glyph.y) * clipCenter.w * screenSize.y;
    vec4 clipPosition = projection * (view * (model * vec4(dataPosition, 1)));

    gl_Position = clipPosition;
    interpColor = color;
    pickId = id;
    dataCoordinate = dataPosition;
  }
}
`]), ye = fe([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 fragClipBounds[2];
uniform float opacity;

varying vec4 interpColor;
varying vec3 dataCoordinate;

void main() {
  if (
    outOfRange(fragClipBounds[0], fragClipBounds[1], dataCoordinate) ||
    interpColor.a * opacity == 0.
  ) discard;
  gl_FragColor = interpColor * opacity;
}
`]), ge = fe([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 fragClipBounds[2];
uniform float pickGroup;

varying vec4 pickId;
varying vec3 dataCoordinate;

void main() {
  if (outOfRange(fragClipBounds[0], fragClipBounds[1], dataCoordinate)) discard;

  gl_FragColor = vec4(pickGroup, pickId.bgr);
}`]), ve = [{ name: "position", type: "vec3" }, { name: "color", type: "vec4" }, { name: "glyph", type: "vec2" }, { name: "id", type: "vec4" }], be = { vertex: he, fragment: ye, attributes: ve }, Ce = { vertex: me, fragment: ye, attributes: ve }, Se = { vertex: pe, fragment: ye, attributes: ve }, we = { vertex: he, fragment: ge, attributes: ve }, Ee = { vertex: me, fragment: ge, attributes: ve }, xe = { vertex: pe, fragment: ge, attributes: ve };
                function Le(Qe, ke) {
                  var Me = de(Qe, ke), Re = Me.attributes;
                  return Re.position.location = 0, Re.color.location = 1, Re.glyph.location = 2, Re.id.location = 3, Me;
                }
                le.createPerspective = function(Qe) {
                  return Le(Qe, be);
                }, le.createOrtho = function(Qe) {
                  return Le(Qe, Ce);
                }, le.createProject = function(Qe) {
                  return Le(Qe, Se);
                }, le.createPickPerspective = function(Qe) {
                  return Le(Qe, we);
                }, le.createPickOrtho = function(Qe) {
                  return Le(Qe, Ee);
                }, le.createPickProject = function(Qe) {
                  return Le(Qe, xe);
                };
              }, 8418: function(ce, le, ue) {
                var de = ue(5219), fe = ue(2762), he = ue(8116), me = ue(1888), pe = ue(6760), ye = ue(1283), ge = ue(9366), ve = ue(5964), be = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1], Ce = ArrayBuffer, Se = DataView;
                function we(dt) {
                  return Array.isArray(dt) || function(Je) {
                    return Ce.isView(Je) && !(Je instanceof Se);
                  }(dt);
                }
                function Ee(dt, Je) {
                  var it = dt[0], st = dt[1], gt = dt[2], bt = dt[3];
                  return dt[0] = Je[0] * it + Je[4] * st + Je[8] * gt + Je[12] * bt, dt[1] = Je[1] * it + Je[5] * st + Je[9] * gt + Je[13] * bt, dt[2] = Je[2] * it + Je[6] * st + Je[10] * gt + Je[14] * bt, dt[3] = Je[3] * it + Je[7] * st + Je[11] * gt + Je[15] * bt, dt;
                }
                function xe(dt, Je, it, st) {
                  return Ee(st, st), Ee(st, st), Ee(st, st);
                }
                function Le(dt, Je) {
                  this.index = dt, this.dataCoordinate = this.position = Je;
                }
                function Qe(dt) {
                  return dt === !0 || dt > 1 ? 1 : dt;
                }
                function ke(dt, Je, it, st, gt, bt, yt, St, Lt, Rt, vt, $t) {
                  this.gl = dt, this.pixelRatio = 1, this.shader = Je, this.orthoShader = it, this.projectShader = st, this.pointBuffer = gt, this.colorBuffer = bt, this.glyphBuffer = yt, this.idBuffer = St, this.vao = Lt, this.vertexCount = 0, this.lineVertexCount = 0, this.opacity = 1, this.hasAlpha = !1, this.lineWidth = 0, this.projectScale = [0.6666666666666666, 0.6666666666666666, 0.6666666666666666], this.projectOpacity = [1, 1, 1], this.projectHasAlpha = !1, this.pickId = 0, this.pickPerspectiveShader = Rt, this.pickOrthoShader = vt, this.pickProjectShader = $t, this.points = [], this._selectResult = new Le(0, [0, 0, 0]), this.useOrtho = !0, this.bounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], this.axesProject = [!0, !0, !0], this.axesBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.highlightId = [1, 1, 1, 1], this.highlightScale = 2, this.clipBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.dirty = !0;
                }
                ce.exports = function(dt) {
                  var Je = dt.gl, it = ye.createPerspective(Je), st = ye.createOrtho(Je), gt = ye.createProject(Je), bt = ye.createPickPerspective(Je), yt = ye.createPickOrtho(Je), St = ye.createPickProject(Je), Lt = fe(Je), Rt = fe(Je), vt = fe(Je), $t = fe(Je), Pt = new ke(Je, it, st, gt, Lt, Rt, vt, $t, he(Je, [{ buffer: Lt, size: 3, type: Je.FLOAT }, { buffer: Rt, size: 4, type: Je.FLOAT }, { buffer: vt, size: 2, type: Je.FLOAT }, { buffer: $t, size: 4, type: Je.UNSIGNED_BYTE, normalized: !0 }]), bt, yt, St);
                  return Pt.update(dt), Pt;
                };
                var Me = ke.prototype;
                Me.pickSlots = 1, Me.setPickBase = function(dt) {
                  this.pickId = dt;
                }, Me.isTransparent = function() {
                  if (this.hasAlpha) return !0;
                  for (var dt = 0; dt < 3; ++dt) if (this.axesProject[dt] && this.projectHasAlpha) return !0;
                  return !1;
                }, Me.isOpaque = function() {
                  if (!this.hasAlpha) return !0;
                  for (var dt = 0; dt < 3; ++dt) if (this.axesProject[dt] && !this.projectHasAlpha) return !0;
                  return !1;
                };
                var Re = [0, 0], De = [0, 0, 0], Pe = [0, 0, 0], $e = [0, 0, 0, 1], qe = [0, 0, 0, 1], Xe = be.slice(), Ve = [0, 0, 0], Ze = [[0, 0, 0], [0, 0, 0]];
                function Ge(dt) {
                  return dt[0] = dt[1] = dt[2] = 0, dt;
                }
                function ze(dt, Je) {
                  return dt[0] = Je[0], dt[1] = Je[1], dt[2] = Je[2], dt[3] = 1, dt;
                }
                function We(dt, Je, it, st) {
                  return dt[0] = Je[0], dt[1] = Je[1], dt[2] = Je[2], dt[it] = st, dt;
                }
                var at = [[-1e8, -1e8, -1e8], [1e8, 1e8, 1e8]];
                function ct(dt, Je, it, st, gt, bt, yt) {
                  var St = it.gl;
                  if ((bt === it.projectHasAlpha || yt) && function(Rt, vt, $t, Pt) {
                    var Xt, Nt = vt.axesProject, Gt = vt.gl, Ft = Rt.uniforms, Tt = $t.model || be, It = $t.view || be, Ct = $t.projection || be, Kt = vt.axesBounds, Vt = function(Rn) {
                      for (var Pn = Ze, Ie = 0; Ie < 2; ++Ie) for (var rt = 0; rt < 3; ++rt) Pn[Ie][rt] = Math.max(Math.min(Rn[Ie][rt], 1e8), -1e8);
                      return Pn;
                    }(vt.clipBounds);
                    Xt = vt.axes && vt.axes.lastCubeProps ? vt.axes.lastCubeProps.axis : [1, 1, 1], Re[0] = 2 / Gt.drawingBufferWidth, Re[1] = 2 / Gt.drawingBufferHeight, Rt.bind(), Ft.view = It, Ft.projection = Ct, Ft.screenSize = Re, Ft.highlightId = vt.highlightId, Ft.highlightScale = vt.highlightScale, Ft.clipBounds = Vt, Ft.pickGroup = vt.pickId / 255, Ft.pixelRatio = Pt;
                    for (var Yt = 0; Yt < 3; ++Yt) if (Nt[Yt]) {
                      Ft.scale = vt.projectScale[Yt], Ft.opacity = vt.projectOpacity[Yt];
                      for (var sn = Xe, hn = 0; hn < 16; ++hn) sn[hn] = 0;
                      for (hn = 0; hn < 4; ++hn) sn[5 * hn] = 1;
                      sn[5 * Yt] = 0, Xt[Yt] < 0 ? sn[12 + Yt] = Kt[0][Yt] : sn[12 + Yt] = Kt[1][Yt], pe(sn, Tt, sn), Ft.model = sn;
                      var Qt = (Yt + 1) % 3, Ut = (Yt + 2) % 3, Wt = Ge(De), rn = Ge(Pe);
                      Wt[Qt] = 1, rn[Ut] = 1;
                      var vn = xe(0, 0, 0, ze($e, Wt)), dn = xe(0, 0, 0, ze(qe, rn));
                      if (Math.abs(vn[1]) > Math.abs(dn[1])) {
                        var Ht = vn;
                        vn = dn, dn = Ht, Ht = Wt, Wt = rn, rn = Ht;
                        var jt = Qt;
                        Qt = Ut, Ut = jt;
                      }
                      vn[0] < 0 && (Wt[Qt] = -1), dn[1] > 0 && (rn[Ut] = -1);
                      var cn = 0, wn = 0;
                      for (hn = 0; hn < 4; ++hn) cn += Math.pow(Tt[4 * Qt + hn], 2), wn += Math.pow(Tt[4 * Ut + hn], 2);
                      Wt[Qt] /= Math.sqrt(cn), rn[Ut] /= Math.sqrt(wn), Ft.axes[0] = Wt, Ft.axes[1] = rn, Ft.fragClipBounds[0] = We(Ve, Vt[0], Yt, -1e8), Ft.fragClipBounds[1] = We(Ve, Vt[1], Yt, 1e8), vt.vao.bind(), vt.vao.draw(Gt.TRIANGLES, vt.vertexCount), vt.lineWidth > 0 && (Gt.lineWidth(vt.lineWidth * Pt), vt.vao.draw(Gt.LINES, vt.lineVertexCount, vt.vertexCount)), vt.vao.unbind();
                    }
                  }(Je, it, st, gt), bt === it.hasAlpha || yt) {
                    dt.bind();
                    var Lt = dt.uniforms;
                    Lt.model = st.model || be, Lt.view = st.view || be, Lt.projection = st.projection || be, Re[0] = 2 / St.drawingBufferWidth, Re[1] = 2 / St.drawingBufferHeight, Lt.screenSize = Re, Lt.highlightId = it.highlightId, Lt.highlightScale = it.highlightScale, Lt.fragClipBounds = at, Lt.clipBounds = it.axes.bounds, Lt.opacity = it.opacity, Lt.pickGroup = it.pickId / 255, Lt.pixelRatio = gt, it.vao.bind(), it.vao.draw(St.TRIANGLES, it.vertexCount), it.lineWidth > 0 && (St.lineWidth(it.lineWidth * gt), it.vao.draw(St.LINES, it.lineVertexCount, it.vertexCount)), it.vao.unbind();
                  }
                }
                function et(dt, Je, it, st) {
                  var gt;
                  gt = we(dt) ? Je < dt.length ? dt[Je] : void 0 : dt, gt = ve(gt);
                  var bt = !0;
                  de(gt) && (gt = "▼", bt = !1);
                  var yt = it.family;
                  we(yt) && (yt = yt[Je]), yt || (yt = "normal");
                  var St = it.weight;
                  we(St) && (St = St[Je]), St || (St = "normal");
                  var Lt = it.style;
                  we(Lt) && (Lt = Lt[Je]), Lt || (Lt = "normal");
                  var Rt = it.variant;
                  we(Rt) && (Rt = Rt[Je]), Rt || (Rt = "normal");
                  var vt = ge(gt, { family: yt, weight: St, style: Lt, variant: Rt }, st);
                  return { mesh: (vt = ge(gt, it, st))[0], lines: vt[1], bounds: vt[2], visible: bt };
                }
                Me.draw = function(dt) {
                  ct(this.useOrtho ? this.orthoShader : this.shader, this.projectShader, this, dt, this.pixelRatio, !1, !1);
                }, Me.drawTransparent = function(dt) {
                  ct(this.useOrtho ? this.orthoShader : this.shader, this.projectShader, this, dt, this.pixelRatio, !0, !1);
                }, Me.drawPick = function(dt) {
                  ct(this.useOrtho ? this.pickOrthoShader : this.pickPerspectiveShader, this.pickProjectShader, this, dt, 1, !0, !0);
                }, Me.pick = function(dt) {
                  if (!dt || dt.id !== this.pickId) return null;
                  var Je = dt.value[2] + (dt.value[1] << 8) + (dt.value[0] << 16);
                  if (Je >= this.pointCount || Je < 0) return null;
                  var it = this.points[Je], st = this._selectResult;
                  st.index = Je;
                  for (var gt = 0; gt < 3; ++gt) st.position[gt] = st.dataCoordinate[gt] = it[gt];
                  return st;
                }, Me.highlight = function(dt) {
                  if (dt) {
                    var Je = dt.index, it = 255 & Je, st = Je >> 8 & 255, gt = Je >> 16 & 255;
                    this.highlightId = [it / 255, st / 255, gt / 255, 0];
                  } else this.highlightId = [1, 1, 1, 1];
                }, Me.update = function(dt) {
                  if ("perspective" in (dt = dt || {}) && (this.useOrtho = !dt.perspective), "orthographic" in dt && (this.useOrtho = !!dt.orthographic), "lineWidth" in dt && (this.lineWidth = dt.lineWidth), "project" in dt) if (we(dt.project)) this.axesProject = dt.project;
                  else {
                    var Je = !!dt.project;
                    this.axesProject = [Je, Je, Je];
                  }
                  if ("projectScale" in dt) if (we(dt.projectScale)) this.projectScale = dt.projectScale.slice();
                  else {
                    var it = +dt.projectScale;
                    this.projectScale = [it, it, it];
                  }
                  if (this.projectHasAlpha = !1, "projectOpacity" in dt) {
                    we(dt.projectOpacity) ? this.projectOpacity = dt.projectOpacity.slice() : (it = +dt.projectOpacity, this.projectOpacity = [it, it, it]);
                    for (var st = 0; st < 3; ++st) this.projectOpacity[st] = Qe(this.projectOpacity[st]), this.projectOpacity[st] < 1 && (this.projectHasAlpha = !0);
                  }
                  this.hasAlpha = !1, "opacity" in dt && (this.opacity = Qe(dt.opacity), this.opacity < 1 && (this.hasAlpha = !0)), this.dirty = !0;
                  var gt, bt, yt = dt.position, St = { family: dt.font || "normal", style: dt.fontStyle || "normal", weight: dt.fontWeight || "normal", variant: dt.fontVariant || "normal" }, Lt = dt.alignment || [0, 0];
                  if (Lt.length === 2) gt = Lt[0], bt = Lt[1];
                  else for (gt = [], bt = [], st = 0; st < Lt.length; ++st) gt[st] = Lt[st][0], bt[st] = Lt[st][1];
                  var Rt = [1 / 0, 1 / 0, 1 / 0], vt = [-1 / 0, -1 / 0, -1 / 0], $t = dt.glyph, Pt = dt.color, Xt = dt.size, Nt = dt.angle, Gt = dt.lineColor, Ft = -1, Tt = 0, It = 0, Ct = 0;
                  if (yt.length) {
                    Ct = yt.length;
                    e: for (st = 0; st < Ct; ++st) {
                      for (var Kt = yt[st], Vt = 0; Vt < 3; ++Vt) if (isNaN(Kt[Vt]) || !isFinite(Kt[Vt])) continue e;
                      var Yt = (Pn = et($t, st, St, this.pixelRatio)).mesh, sn = Pn.lines, hn = Pn.bounds;
                      Tt += 3 * Yt.cells.length, It += 2 * sn.edges.length;
                    }
                  }
                  var Qt = Tt + It, Ut = me.mallocFloat(3 * Qt), Wt = me.mallocFloat(4 * Qt), rn = me.mallocFloat(2 * Qt), vn = me.mallocUint32(Qt);
                  if (Qt > 0) {
                    var dn = 0, Ht = Tt, jt = [0, 0, 0, 1], cn = [0, 0, 0, 1], wn = we(Pt) && we(Pt[0]), Rn = we(Gt) && we(Gt[0]);
                    e: for (st = 0; st < Ct; ++st) {
                      for (Ft += 1, Kt = yt[st], Vt = 0; Vt < 3; ++Vt) {
                        if (isNaN(Kt[Vt]) || !isFinite(Kt[Vt])) continue e;
                        vt[Vt] = Math.max(vt[Vt], Kt[Vt]), Rt[Vt] = Math.min(Rt[Vt], Kt[Vt]);
                      }
                      Yt = (Pn = et($t, st, St, this.pixelRatio)).mesh, sn = Pn.lines, hn = Pn.bounds;
                      var Pn, Ie = Pn.visible;
                      if (Ie) if (we(Pt)) {
                        if ((rt = wn ? st < Pt.length ? Pt[st] : [0, 0, 0, 0] : Pt).length === 3) {
                          for (Vt = 0; Vt < 3; ++Vt) jt[Vt] = rt[Vt];
                          jt[3] = 1;
                        } else if (rt.length === 4) {
                          for (Vt = 0; Vt < 4; ++Vt) jt[Vt] = rt[Vt];
                          !this.hasAlpha && rt[3] < 1 && (this.hasAlpha = !0);
                        }
                      } else jt[0] = jt[1] = jt[2] = 0, jt[3] = 1;
                      else jt = [1, 1, 1, 0];
                      if (Ie) if (we(Gt)) {
                        var rt;
                        if ((rt = Rn ? st < Gt.length ? Gt[st] : [0, 0, 0, 0] : Gt).length === 3) {
                          for (Vt = 0; Vt < 3; ++Vt) cn[Vt] = rt[Vt];
                          cn[Vt] = 1;
                        } else if (rt.length === 4) {
                          for (Vt = 0; Vt < 4; ++Vt) cn[Vt] = rt[Vt];
                          !this.hasAlpha && rt[3] < 1 && (this.hasAlpha = !0);
                        }
                      } else cn[0] = cn[1] = cn[2] = 0, cn[3] = 1;
                      else cn = [1, 1, 1, 0];
                      var ft = 0.5;
                      Ie ? we(Xt) ? ft = st < Xt.length ? +Xt[st] : 12 : Xt ? ft = +Xt : this.useOrtho && (ft = 12) : ft = 0;
                      var pt = 0;
                      we(Nt) ? pt = st < Nt.length ? +Nt[st] : 0 : Nt && (pt = +Nt);
                      var wt = Math.cos(pt), Bt = Math.sin(pt);
                      for (Kt = yt[st], Vt = 0; Vt < 3; ++Vt) vt[Vt] = Math.max(vt[Vt], Kt[Vt]), Rt[Vt] = Math.min(Rt[Vt], Kt[Vt]);
                      var At = gt, Dt = bt;
                      At = 0, we(gt) ? At = st < gt.length ? gt[st] : 0 : gt && (At = gt), Dt = 0, we(bt) ? Dt = st < bt.length ? bt[st] : 0 : bt && (Dt = bt);
                      var Ot = [At *= At > 0 ? 1 - hn[0][0] : At < 0 ? 1 + hn[1][0] : 1, Dt *= Dt > 0 ? 1 - hn[0][1] : Dt < 0 ? 1 + hn[1][1] : 1], un = Yt.cells || [], yn = Yt.positions || [];
                      for (Vt = 0; Vt < un.length; ++Vt) for (var _n = un[Vt], gn = 0; gn < 3; ++gn) {
                        for (var bn = 0; bn < 3; ++bn) Ut[3 * dn + bn] = Kt[bn];
                        for (bn = 0; bn < 4; ++bn) Wt[4 * dn + bn] = jt[bn];
                        vn[dn] = Ft;
                        var an = yn[_n[gn]];
                        rn[2 * dn] = ft * (wt * an[0] - Bt * an[1] + Ot[0]), rn[2 * dn + 1] = ft * (Bt * an[0] + wt * an[1] + Ot[1]), dn += 1;
                      }
                      for (un = sn.edges, yn = sn.positions, Vt = 0; Vt < un.length; ++Vt) for (_n = un[Vt], gn = 0; gn < 2; ++gn) {
                        for (bn = 0; bn < 3; ++bn) Ut[3 * Ht + bn] = Kt[bn];
                        for (bn = 0; bn < 4; ++bn) Wt[4 * Ht + bn] = cn[bn];
                        vn[Ht] = Ft, an = yn[_n[gn]], rn[2 * Ht] = ft * (wt * an[0] - Bt * an[1] + Ot[0]), rn[2 * Ht + 1] = ft * (Bt * an[0] + wt * an[1] + Ot[1]), Ht += 1;
                      }
                    }
                  }
                  this.bounds = [Rt, vt], this.points = yt, this.pointCount = yt.length, this.vertexCount = Tt, this.lineVertexCount = It, this.pointBuffer.update(Ut), this.colorBuffer.update(Wt), this.glyphBuffer.update(rn), this.idBuffer.update(vn), me.free(Ut), me.free(Wt), me.free(rn), me.free(vn);
                }, Me.dispose = function() {
                  this.shader.dispose(), this.orthoShader.dispose(), this.pickPerspectiveShader.dispose(), this.pickOrthoShader.dispose(), this.vao.dispose(), this.pointBuffer.dispose(), this.colorBuffer.dispose(), this.glyphBuffer.dispose(), this.idBuffer.dispose();
                };
              }, 4298: function(ce, le, ue) {
                var de = ue(3236);
                le.boxVertex = de([`precision mediump float;
#define GLSLIFY 1

attribute vec2 vertex;

uniform vec2 cornerA, cornerB;

void main() {
  gl_Position = vec4(mix(cornerA, cornerB, vertex), 0, 1);
}
`]), le.boxFragment = de([`precision mediump float;
#define GLSLIFY 1

uniform vec4 color;

void main() {
  gl_FragColor = color;
}
`]);
              }, 3161: function(ce, le, ue) {
                var de = ue(9405), fe = ue(2762), he = ue(4298);
                function me(ye, ge, ve) {
                  this.plot = ye, this.boxBuffer = ge, this.boxShader = ve, this.enabled = !0, this.selectBox = [1 / 0, 1 / 0, -1 / 0, -1 / 0], this.borderColor = [0, 0, 0, 1], this.innerFill = !1, this.innerColor = [0, 0, 0, 0.25], this.outerFill = !0, this.outerColor = [0, 0, 0, 0.5], this.borderWidth = 10;
                }
                ce.exports = function(ye, ge) {
                  var ve = ye.gl, be = new me(ye, fe(ve, [0, 0, 0, 1, 1, 0, 1, 1]), de(ve, he.boxVertex, he.boxFragment));
                  return be.update(ge), ye.addOverlay(be), be;
                };
                var pe = me.prototype;
                pe.draw = function() {
                  if (this.enabled) {
                    var ye = this.plot, ge = this.selectBox, ve = this.borderWidth, be = (this.innerFill, this.innerColor), Ce = (this.outerFill, this.outerColor), Se = this.borderColor, we = ye.box, Ee = ye.screenBox, xe = ye.dataBox, Le = ye.viewBox, Qe = ye.pixelRatio, ke = (ge[0] - xe[0]) * (Le[2] - Le[0]) / (xe[2] - xe[0]) + Le[0], Me = (ge[1] - xe[1]) * (Le[3] - Le[1]) / (xe[3] - xe[1]) + Le[1], Re = (ge[2] - xe[0]) * (Le[2] - Le[0]) / (xe[2] - xe[0]) + Le[0], De = (ge[3] - xe[1]) * (Le[3] - Le[1]) / (xe[3] - xe[1]) + Le[1];
                    if (ke = Math.max(ke, Le[0]), Me = Math.max(Me, Le[1]), Re = Math.min(Re, Le[2]), De = Math.min(De, Le[3]), !(Re < ke || De < Me)) {
                      we.bind();
                      var Pe = Ee[2] - Ee[0], $e = Ee[3] - Ee[1];
                      if (this.outerFill && (we.drawBox(0, 0, Pe, Me, Ce), we.drawBox(0, Me, ke, De, Ce), we.drawBox(0, De, Pe, $e, Ce), we.drawBox(Re, Me, Pe, De, Ce)), this.innerFill && we.drawBox(ke, Me, Re, De, be), ve > 0) {
                        var qe = ve * Qe;
                        we.drawBox(ke - qe, Me - qe, Re + qe, Me + qe, Se), we.drawBox(ke - qe, De - qe, Re + qe, De + qe, Se), we.drawBox(ke - qe, Me - qe, ke + qe, De + qe, Se), we.drawBox(Re - qe, Me - qe, Re + qe, De + qe, Se);
                      }
                    }
                  }
                }, pe.update = function(ye) {
                  ye = ye || {}, this.innerFill = !!ye.innerFill, this.outerFill = !!ye.outerFill, this.innerColor = (ye.innerColor || [0, 0, 0, 0.5]).slice(), this.outerColor = (ye.outerColor || [0, 0, 0, 0.5]).slice(), this.borderColor = (ye.borderColor || [0, 0, 0, 1]).slice(), this.borderWidth = ye.borderWidth || 0, this.selectBox = (ye.selectBox || this.selectBox).slice();
                }, pe.dispose = function() {
                  this.boxBuffer.dispose(), this.boxShader.dispose(), this.plot.removeOverlay(this);
                };
              }, 3589: function(ce, le, ue) {
                ce.exports = function(ve, be) {
                  var Ce = be[0], Se = be[1];
                  return new ye(ve, de(ve, Ce, Se, {}), fe.mallocUint8(Ce * Se * 4));
                };
                var de = ue(2260), fe = ue(1888), he = ue(9618), me = ue(8828).nextPow2;
                function pe(ve, be, Ce, Se, we) {
                  this.coord = [ve, be], this.id = Ce, this.value = Se, this.distance = we;
                }
                function ye(ve, be, Ce) {
                  this.gl = ve, this.fbo = be, this.buffer = Ce, this._readTimeout = null;
                  var Se = this;
                  this._readCallback = function() {
                    Se.gl && (be.bind(), ve.readPixels(0, 0, be.shape[0], be.shape[1], ve.RGBA, ve.UNSIGNED_BYTE, Se.buffer), Se._readTimeout = null);
                  };
                }
                var ge = ye.prototype;
                Object.defineProperty(ge, "shape", { get: function() {
                  return this.gl ? this.fbo.shape.slice() : [0, 0];
                }, set: function(ve) {
                  if (this.gl) {
                    this.fbo.shape = ve;
                    var be = this.fbo.shape[0], Ce = this.fbo.shape[1];
                    if (Ce * be * 4 > this.buffer.length) {
                      fe.free(this.buffer);
                      for (var Se = this.buffer = fe.mallocUint8(me(Ce * be * 4)), we = 0; we < Ce * be * 4; ++we) Se[we] = 255;
                    }
                    return ve;
                  }
                } }), ge.begin = function() {
                  var ve = this.gl;
                  this.shape, ve && (this.fbo.bind(), ve.clearColor(1, 1, 1, 1), ve.clear(ve.COLOR_BUFFER_BIT | ve.DEPTH_BUFFER_BIT));
                }, ge.end = function() {
                  var ve = this.gl;
                  ve && (ve.bindFramebuffer(ve.FRAMEBUFFER, null), this._readTimeout || clearTimeout(this._readTimeout), this._readTimeout = setTimeout(this._readCallback, 1));
                }, ge.query = function(ve, be, Ce) {
                  if (!this.gl) return null;
                  var Se = this.fbo.shape.slice();
                  ve |= 0, be |= 0, typeof Ce != "number" && (Ce = 1);
                  var we = 0 | Math.min(Math.max(ve - Ce, 0), Se[0]), Ee = 0 | Math.min(Math.max(ve + Ce, 0), Se[0]), xe = 0 | Math.min(Math.max(be - Ce, 0), Se[1]), Le = 0 | Math.min(Math.max(be + Ce, 0), Se[1]);
                  if (Ee <= we || Le <= xe) return null;
                  var Qe = [Ee - we, Le - xe], ke = he(this.buffer, [Qe[0], Qe[1], 4], [4, 4 * Se[0], 1], 4 * (we + Se[0] * xe)), Me = function(Pe, $e, qe) {
                    for (var Xe = 1e8, Ve = -1, Ze = -1, Ge = Pe.shape[0], ze = Pe.shape[1], We = 0; We < Ge; We++) for (var at = 0; at < ze; at++) {
                      var ct = Pe.get(We, at, 0), et = Pe.get(We, at, 1), dt = Pe.get(We, at, 2), Je = Pe.get(We, at, 3);
                      if (ct < 255 || et < 255 || dt < 255 || Je < 255) {
                        var it = $e - We, st = qe - at, gt = it * it + st * st;
                        gt < Xe && (Xe = gt, Ve = We, Ze = at);
                      }
                    }
                    return [Ve, Ze, Xe];
                  }(ke.hi(Qe[0], Qe[1], 1), Ce, Ce), Re = Me[0], De = Me[1];
                  return Re < 0 || Math.pow(this.radius, 2) < Me[2] ? null : new pe(Re + we | 0, De + xe | 0, ke.get(Re, De, 0), [ke.get(Re, De, 1), ke.get(Re, De, 2), ke.get(Re, De, 3)], Math.sqrt(Me[2]));
                }, ge.dispose = function() {
                  this.gl && (this.fbo.dispose(), fe.free(this.buffer), this.gl = null, this._readTimeout && clearTimeout(this._readTimeout));
                };
              }, 9405: function(ce, le, ue) {
                var de = ue(3327), fe = ue(8731), he = ue(216), me = ue(5091), pe = ue(2145), ye = ue(8866);
                function ge(Ce) {
                  this.gl = Ce, this.gl.lastAttribCount = 0, this._vref = this._fref = this._relink = this.vertShader = this.fragShader = this.program = this.attributes = this.uniforms = this.types = null;
                }
                var ve = ge.prototype;
                function be(Ce, Se) {
                  return Ce.name < Se.name ? -1 : 1;
                }
                ve.bind = function() {
                  var Ce;
                  this.program || this._relink();
                  var Se = this.gl.getProgramParameter(this.program, this.gl.ACTIVE_ATTRIBUTES), we = this.gl.lastAttribCount;
                  if (Se > we) for (Ce = we; Ce < Se; Ce++) this.gl.enableVertexAttribArray(Ce);
                  else if (we > Se) for (Ce = Se; Ce < we; Ce++) this.gl.disableVertexAttribArray(Ce);
                  this.gl.lastAttribCount = Se, this.gl.useProgram(this.program);
                }, ve.dispose = function() {
                  for (var Ce = this.gl.lastAttribCount, Se = 0; Se < Ce; Se++) this.gl.disableVertexAttribArray(Se);
                  this.gl.lastAttribCount = 0, this._fref && this._fref.dispose(), this._vref && this._vref.dispose(), this.attributes = this.types = this.vertShader = this.fragShader = this.program = this._relink = this._fref = this._vref = null;
                }, ve.update = function(Ce, Se, we, Ee) {
                  if (!Se || arguments.length === 1) {
                    var xe = Ce;
                    Ce = xe.vertex, Se = xe.fragment, we = xe.uniforms, Ee = xe.attributes;
                  }
                  var Le = this, Qe = Le.gl, ke = Le._vref;
                  Le._vref = me.shader(Qe, Qe.VERTEX_SHADER, Ce), ke && ke.dispose(), Le.vertShader = Le._vref.shader;
                  var Me = this._fref;
                  if (Le._fref = me.shader(Qe, Qe.FRAGMENT_SHADER, Se), Me && Me.dispose(), Le.fragShader = Le._fref.shader, !we || !Ee) {
                    var Re = Qe.createProgram();
                    if (Qe.attachShader(Re, Le.fragShader), Qe.attachShader(Re, Le.vertShader), Qe.linkProgram(Re), !Qe.getProgramParameter(Re, Qe.LINK_STATUS)) {
                      var De = Qe.getProgramInfoLog(Re);
                      throw new ye(De, "Error linking program:" + De);
                    }
                    we = we || pe.uniforms(Qe, Re), Ee = Ee || pe.attributes(Qe, Re), Qe.deleteProgram(Re);
                  }
                  (Ee = Ee.slice()).sort(be);
                  var Pe, $e = [], qe = [], Xe = [];
                  for (Pe = 0; Pe < Ee.length; ++Pe) {
                    var Ve = Ee[Pe];
                    if (Ve.type.indexOf("mat") >= 0) {
                      for (var Ze = 0 | Ve.type.charAt(Ve.type.length - 1), Ge = new Array(Ze), ze = 0; ze < Ze; ++ze) Ge[ze] = Xe.length, qe.push(Ve.name + "[" + ze + "]"), typeof Ve.location == "number" ? Xe.push(Ve.location + ze) : Array.isArray(Ve.location) && Ve.location.length === Ze && typeof Ve.location[ze] == "number" ? Xe.push(0 | Ve.location[ze]) : Xe.push(-1);
                      $e.push({ name: Ve.name, type: Ve.type, locations: Ge });
                    } else $e.push({ name: Ve.name, type: Ve.type, locations: [Xe.length] }), qe.push(Ve.name), typeof Ve.location == "number" ? Xe.push(0 | Ve.location) : Xe.push(-1);
                  }
                  var We = 0;
                  for (Pe = 0; Pe < Xe.length; ++Pe) if (Xe[Pe] < 0) {
                    for (; Xe.indexOf(We) >= 0; ) We += 1;
                    Xe[Pe] = We;
                  }
                  var at = new Array(we.length);
                  function ct() {
                    Le.program = me.program(Qe, Le._vref, Le._fref, qe, Xe);
                    for (var et = 0; et < we.length; ++et) at[et] = Qe.getUniformLocation(Le.program, we[et].name);
                  }
                  ct(), Le._relink = ct, Le.types = { uniforms: he(we), attributes: he(Ee) }, Le.attributes = fe(Qe, Le, $e, Xe), Object.defineProperty(Le, "uniforms", de(Qe, Le, we, at));
                }, ce.exports = function(Ce, Se, we, Ee, xe) {
                  var Le = new ge(Ce);
                  return Le.update(Se, we, Ee, xe), Le;
                };
              }, 8866: function(ce) {
                function le(ue, de, fe) {
                  this.shortMessage = de || "", this.longMessage = fe || "", this.rawError = ue || "", this.message = "gl-shader: " + (de || ue || "") + (fe ? `
` + fe : ""), this.stack = new Error().stack;
                }
                le.prototype = new Error(), le.prototype.name = "GLError", le.prototype.constructor = le, ce.exports = le;
              }, 8731: function(ce, le, ue) {
                ce.exports = function(ge, ve, be, Ce) {
                  for (var Se = {}, we = 0, Ee = be.length; we < Ee; ++we) {
                    var xe = be[we], Le = xe.name, Qe = xe.type, ke = xe.locations;
                    switch (Qe) {
                      case "bool":
                      case "int":
                      case "float":
                        pe(ge, ve, ke[0], Ce, 1, Se, Le);
                        break;
                      default:
                        if (Qe.indexOf("vec") >= 0) {
                          if ((Me = Qe.charCodeAt(Qe.length - 1) - 48) < 2 || Me > 4) throw new de("", "Invalid data type for attribute " + Le + ": " + Qe);
                          pe(ge, ve, ke[0], Ce, Me, Se, Le);
                        } else {
                          if (!(Qe.indexOf("mat") >= 0)) throw new de("", "Unknown data type for attribute " + Le + ": " + Qe);
                          var Me;
                          if ((Me = Qe.charCodeAt(Qe.length - 1) - 48) < 2 || Me > 4) throw new de("", "Invalid data type for attribute " + Le + ": " + Qe);
                          ye(ge, ve, ke, Ce, Me, Se, Le);
                        }
                    }
                  }
                  return Se;
                };
                var de = ue(8866);
                function fe(ge, ve, be, Ce, Se, we) {
                  this._gl = ge, this._wrapper = ve, this._index = be, this._locations = Ce, this._dimension = Se, this._constFunc = we;
                }
                var he = fe.prototype;
                he.pointer = function(ge, ve, be, Ce) {
                  var Se = this, we = Se._gl, Ee = Se._locations[Se._index];
                  we.vertexAttribPointer(Ee, Se._dimension, ge || we.FLOAT, !!ve, be || 0, Ce || 0), we.enableVertexAttribArray(Ee);
                }, he.set = function(ge, ve, be, Ce) {
                  return this._constFunc(this._locations[this._index], ge, ve, be, Ce);
                }, Object.defineProperty(he, "location", { get: function() {
                  return this._locations[this._index];
                }, set: function(ge) {
                  return ge !== this._locations[this._index] && (this._locations[this._index] = 0 | ge, this._wrapper.program = null), 0 | ge;
                } });
                var me = [function(ge, ve, be) {
                  return be.length === void 0 ? ge.vertexAttrib1f(ve, be) : ge.vertexAttrib1fv(ve, be);
                }, function(ge, ve, be, Ce) {
                  return be.length === void 0 ? ge.vertexAttrib2f(ve, be, Ce) : ge.vertexAttrib2fv(ve, be);
                }, function(ge, ve, be, Ce, Se) {
                  return be.length === void 0 ? ge.vertexAttrib3f(ve, be, Ce, Se) : ge.vertexAttrib3fv(ve, be);
                }, function(ge, ve, be, Ce, Se, we) {
                  return be.length === void 0 ? ge.vertexAttrib4f(ve, be, Ce, Se, we) : ge.vertexAttrib4fv(ve, be);
                }];
                function pe(ge, ve, be, Ce, Se, we, Ee) {
                  var xe = me[Se], Le = new fe(ge, ve, be, Ce, Se, xe);
                  Object.defineProperty(we, Ee, { set: function(Qe) {
                    return ge.disableVertexAttribArray(Ce[be]), xe(ge, Ce[be], Qe), Qe;
                  }, get: function() {
                    return Le;
                  }, enumerable: !0 });
                }
                function ye(ge, ve, be, Ce, Se, we, Ee) {
                  for (var xe = new Array(Se), Le = new Array(Se), Qe = 0; Qe < Se; ++Qe) pe(ge, ve, be[Qe], Ce, Se, xe, Qe), Le[Qe] = xe[Qe];
                  Object.defineProperty(xe, "location", { set: function(Re) {
                    if (Array.isArray(Re)) for (var De = 0; De < Se; ++De) Le[De].location = Re[De];
                    else for (De = 0; De < Se; ++De) Le[De].location = Re + De;
                    return Re;
                  }, get: function() {
                    for (var Re = new Array(Se), De = 0; De < Se; ++De) Re[De] = Ce[be[De]];
                    return Re;
                  }, enumerable: !0 }), xe.pointer = function(Re, De, Pe, $e) {
                    Re = Re || ge.FLOAT, De = !!De, Pe = Pe || Se * Se, $e = $e || 0;
                    for (var qe = 0; qe < Se; ++qe) {
                      var Xe = Ce[be[qe]];
                      ge.vertexAttribPointer(Xe, Se, Re, De, Pe, $e + qe * Se), ge.enableVertexAttribArray(Xe);
                    }
                  };
                  var ke = new Array(Se), Me = ge["vertexAttrib" + Se + "fv"];
                  Object.defineProperty(we, Ee, { set: function(Re) {
                    for (var De = 0; De < Se; ++De) {
                      var Pe = Ce[be[De]];
                      if (ge.disableVertexAttribArray(Pe), Array.isArray(Re[0])) Me.call(ge, Pe, Re[De]);
                      else {
                        for (var $e = 0; $e < Se; ++$e) ke[$e] = Re[Se * De + $e];
                        Me.call(ge, Pe, ke);
                      }
                    }
                    return Re;
                  }, get: function() {
                    return xe;
                  }, enumerable: !0 });
                }
              }, 3327: function(ce, le, ue) {
                var de = ue(216), fe = ue(8866);
                function he(pe) {
                  return function() {
                    return pe;
                  };
                }
                function me(pe, ye) {
                  for (var ge = new Array(pe), ve = 0; ve < pe; ++ve) ge[ve] = ye;
                  return ge;
                }
                ce.exports = function(pe, ye, ge, ve) {
                  function be(xe) {
                    return function(Le) {
                      for (var Qe = Ce("", xe), ke = 0; ke < Qe.length; ++ke) {
                        var Me = Qe[ke], Re = Me[0], De = Me[1];
                        if (ve[De]) {
                          var Pe = Le;
                          if (typeof Re == "string" && (Re.indexOf(".") === 0 || Re.indexOf("[") === 0)) {
                            var $e = Re;
                            if (Re.indexOf(".") === 0 && ($e = Re.slice(1)), $e.indexOf("]") === $e.length - 1) {
                              var qe = $e.indexOf("["), Xe = $e.slice(0, qe), Ve = $e.slice(qe + 1, $e.length - 1);
                              Pe = Xe ? Le[Xe][Ve] : Le[Ve];
                            } else Pe = Le[$e];
                          }
                          var Ze, Ge = ge[De].type;
                          switch (Ge) {
                            case "bool":
                            case "int":
                            case "sampler2D":
                            case "samplerCube":
                              pe.uniform1i(ve[De], Pe);
                              break;
                            case "float":
                              pe.uniform1f(ve[De], Pe);
                              break;
                            default:
                              var ze = Ge.indexOf("vec");
                              if (!(0 <= ze && ze <= 1 && Ge.length === 4 + ze)) {
                                if (Ge.indexOf("mat") === 0 && Ge.length === 4) {
                                  if ((Ze = Ge.charCodeAt(Ge.length - 1) - 48) < 2 || Ze > 4) throw new fe("", "Invalid uniform dimension type for matrix " + name + ": " + Ge);
                                  pe["uniformMatrix" + Ze + "fv"](ve[De], !1, Pe);
                                  break;
                                }
                                throw new fe("", "Unknown uniform data type for " + name + ": " + Ge);
                              }
                              if ((Ze = Ge.charCodeAt(Ge.length - 1) - 48) < 2 || Ze > 4) throw new fe("", "Invalid data type");
                              switch (Ge.charAt(0)) {
                                case "b":
                                case "i":
                                  pe["uniform" + Ze + "iv"](ve[De], Pe);
                                  break;
                                case "v":
                                  pe["uniform" + Ze + "fv"](ve[De], Pe);
                                  break;
                                default:
                                  throw new fe("", "Unrecognized data type for vector " + name + ": " + Ge);
                              }
                          }
                        }
                      }
                    };
                  }
                  function Ce(xe, Le) {
                    if (typeof Le != "object") return [[xe, Le]];
                    var Qe = [];
                    for (var ke in Le) {
                      var Me = Le[ke], Re = xe;
                      parseInt(ke) + "" === ke ? Re += "[" + ke + "]" : Re += "." + ke, typeof Me == "object" ? Qe.push.apply(Qe, Ce(Re, Me)) : Qe.push([Re, Me]);
                    }
                    return Qe;
                  }
                  function Se(xe, Le, Qe) {
                    if (typeof Qe == "object") {
                      var ke = we(Qe);
                      Object.defineProperty(xe, Le, { get: he(ke), set: be(Qe), enumerable: !0, configurable: !1 });
                    } else ve[Qe] ? Object.defineProperty(xe, Le, { get: (Me = Qe, function(Re, De, Pe) {
                      return Re.getUniform(De.program, Pe[Me]);
                    }), set: be(Qe), enumerable: !0, configurable: !1 }) : xe[Le] = function(Re) {
                      switch (Re) {
                        case "bool":
                          return !1;
                        case "int":
                        case "sampler2D":
                        case "samplerCube":
                        case "float":
                          return 0;
                        default:
                          var De = Re.indexOf("vec");
                          if (0 <= De && De <= 1 && Re.length === 4 + De) {
                            if ((Pe = Re.charCodeAt(Re.length - 1) - 48) < 2 || Pe > 4) throw new fe("", "Invalid data type");
                            return Re.charAt(0) === "b" ? me(Pe, !1) : me(Pe, 0);
                          }
                          if (Re.indexOf("mat") === 0 && Re.length === 4) {
                            var Pe;
                            if ((Pe = Re.charCodeAt(Re.length - 1) - 48) < 2 || Pe > 4) throw new fe("", "Invalid uniform dimension type for matrix " + name + ": " + Re);
                            return me(Pe * Pe, 0);
                          }
                          throw new fe("", "Unknown uniform data type for " + name + ": " + Re);
                      }
                    }(ge[Qe].type);
                    var Me;
                  }
                  function we(xe) {
                    var Le;
                    if (Array.isArray(xe)) {
                      Le = new Array(xe.length);
                      for (var Qe = 0; Qe < xe.length; ++Qe) Se(Le, Qe, xe[Qe]);
                    } else for (var ke in Le = {}, xe) Se(Le, ke, xe[ke]);
                    return Le;
                  }
                  var Ee = de(ge, !0);
                  return { get: he(we(Ee)), set: be(Ee), enumerable: !0, configurable: !0 };
                };
              }, 216: function(ce) {
                ce.exports = function(le, ue) {
                  for (var de = {}, fe = 0; fe < le.length; ++fe) for (var he = le[fe].name.split("."), me = de, pe = 0; pe < he.length; ++pe) {
                    var ye = he[pe].split("[");
                    if (ye.length > 1) {
                      ye[0] in me || (me[ye[0]] = []), me = me[ye[0]];
                      for (var ge = 1; ge < ye.length; ++ge) {
                        var ve = parseInt(ye[ge]);
                        ge < ye.length - 1 || pe < he.length - 1 ? (ve in me || (ge < ye.length - 1 ? me[ve] = [] : me[ve] = {}), me = me[ve]) : me[ve] = ue ? fe : le[fe].type;
                      }
                    } else pe < he.length - 1 ? (ye[0] in me || (me[ye[0]] = {}), me = me[ye[0]]) : me[ye[0]] = ue ? fe : le[fe].type;
                  }
                  return de;
                };
              }, 2145: function(ce, le) {
                le.uniforms = function(he, me) {
                  for (var pe = he.getProgramParameter(me, he.ACTIVE_UNIFORMS), ye = [], ge = 0; ge < pe; ++ge) {
                    var ve = he.getActiveUniform(me, ge);
                    if (ve) {
                      var be = fe(he, ve.type);
                      if (ve.size > 1) for (var Ce = 0; Ce < ve.size; ++Ce) ye.push({ name: ve.name.replace("[0]", "[" + Ce + "]"), type: be });
                      else ye.push({ name: ve.name, type: be });
                    }
                  }
                  return ye;
                }, le.attributes = function(he, me) {
                  for (var pe = he.getProgramParameter(me, he.ACTIVE_ATTRIBUTES), ye = [], ge = 0; ge < pe; ++ge) {
                    var ve = he.getActiveAttrib(me, ge);
                    ve && ye.push({ name: ve.name, type: fe(he, ve.type) });
                  }
                  return ye;
                };
                var ue = { FLOAT: "float", FLOAT_VEC2: "vec2", FLOAT_VEC3: "vec3", FLOAT_VEC4: "vec4", INT: "int", INT_VEC2: "ivec2", INT_VEC3: "ivec3", INT_VEC4: "ivec4", BOOL: "bool", BOOL_VEC2: "bvec2", BOOL_VEC3: "bvec3", BOOL_VEC4: "bvec4", FLOAT_MAT2: "mat2", FLOAT_MAT3: "mat3", FLOAT_MAT4: "mat4", SAMPLER_2D: "sampler2D", SAMPLER_CUBE: "samplerCube" }, de = null;
                function fe(he, me) {
                  if (!de) {
                    var pe = Object.keys(ue);
                    de = {};
                    for (var ye = 0; ye < pe.length; ++ye) {
                      var ge = pe[ye];
                      de[he[ge]] = ue[ge];
                    }
                  }
                  return de[me];
                }
              }, 5091: function(ce, le, ue) {
                le.shader = function(be, Ce, Se) {
                  return ve(be).getShaderReference(Ce, Se);
                }, le.program = function(be, Ce, Se, we, Ee) {
                  return ve(be).getProgram(Ce, Se, we, Ee);
                };
                var de = ue(8866), fe = ue(2992), he = new (typeof WeakMap > "u" ? ue(606) : WeakMap)(), me = 0;
                function pe(be, Ce, Se, we, Ee, xe, Le) {
                  this.id = be, this.src = Ce, this.type = Se, this.shader = we, this.count = xe, this.programs = [], this.cache = Le;
                }
                function ye(be) {
                  this.gl = be, this.shaders = [{}, {}], this.programs = {};
                }
                pe.prototype.dispose = function() {
                  if (--this.count == 0) {
                    for (var be = this.cache, Ce = be.gl, Se = this.programs, we = 0, Ee = Se.length; we < Ee; ++we) {
                      var xe = be.programs[Se[we]];
                      xe && (delete be.programs[we], Ce.deleteProgram(xe));
                    }
                    Ce.deleteShader(this.shader), delete be.shaders[this.type === Ce.FRAGMENT_SHADER | 0][this.src];
                  }
                };
                var ge = ye.prototype;
                function ve(be) {
                  var Ce = he.get(be);
                  return Ce || (Ce = new ye(be), he.set(be, Ce)), Ce;
                }
                ge.getShaderReference = function(be, Ce) {
                  var Se = this.gl, we = this.shaders[be === Se.FRAGMENT_SHADER | 0], Ee = we[Ce];
                  if (Ee && Se.isShader(Ee.shader)) Ee.count += 1;
                  else {
                    var xe = function(Le, Qe, ke) {
                      var Me = Le.createShader(Qe);
                      if (Le.shaderSource(Me, ke), Le.compileShader(Me), !Le.getShaderParameter(Me, Le.COMPILE_STATUS)) {
                        var Re = Le.getShaderInfoLog(Me);
                        try {
                          var De = fe(Re, ke, Qe);
                        } catch (Pe) {
                          throw console.warn("Failed to format compiler error: " + Pe), new de(Re, `Error compiling shader:
` + Re);
                        }
                        throw new de(Re, De.short, De.long);
                      }
                      return Me;
                    }(Se, be, Ce);
                    Ee = we[Ce] = new pe(me++, Ce, be, xe, [], 1, this);
                  }
                  return Ee;
                }, ge.getProgram = function(be, Ce, Se, we) {
                  var Ee = [be.id, Ce.id, Se.join(":"), we.join(":")].join("@"), xe = this.programs[Ee];
                  return xe && this.gl.isProgram(xe) || (this.programs[Ee] = xe = function(Le, Qe, ke, Me, Re) {
                    var De = Le.createProgram();
                    Le.attachShader(De, Qe), Le.attachShader(De, ke);
                    for (var Pe = 0; Pe < Me.length; ++Pe) Le.bindAttribLocation(De, Re[Pe], Me[Pe]);
                    if (Le.linkProgram(De), !Le.getProgramParameter(De, Le.LINK_STATUS)) {
                      var $e = Le.getProgramInfoLog(De);
                      throw new de($e, "Error linking program: " + $e);
                    }
                    return De;
                  }(this.gl, be.shader, Ce.shader, Se, we), be.programs.push(Ee), Ce.programs.push(Ee)), xe;
                };
              }, 4098: function(ce) {
                function le(de) {
                  this.plot = de, this.enable = [!0, !0, !1, !1], this.width = [1, 1, 1, 1], this.color = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.center = [1 / 0, 1 / 0];
                }
                ce.exports = function(de, fe) {
                  var he = new le(de);
                  return he.update(fe), de.addOverlay(he), he;
                };
                var ue = le.prototype;
                ue.update = function(de) {
                  de = de || {}, this.enable = (de.enable || [!0, !0, !1, !1]).slice(), this.width = (de.width || [1, 1, 1, 1]).slice(), this.color = (de.color || [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]]).map(function(fe) {
                    return fe.slice();
                  }), this.center = (de.center || [1 / 0, 1 / 0]).slice(), this.plot.setOverlayDirty();
                }, ue.draw = function() {
                  var de = this.enable, fe = this.width, he = this.color, me = this.center, pe = this.plot, ye = pe.line, ge = pe.dataBox, ve = pe.viewBox;
                  if (ye.bind(), ge[0] <= me[0] && me[0] <= ge[2] && ge[1] <= me[1] && me[1] <= ge[3]) {
                    var be = ve[0] + (me[0] - ge[0]) / (ge[2] - ge[0]) * (ve[2] - ve[0]), Ce = ve[1] + (me[1] - ge[1]) / (ge[3] - ge[1]) * (ve[3] - ve[1]);
                    de[0] && ye.drawLine(be, Ce, ve[0], Ce, fe[0], he[0]), de[1] && ye.drawLine(be, Ce, be, ve[1], fe[1], he[1]), de[2] && ye.drawLine(be, Ce, ve[2], Ce, fe[2], he[2]), de[3] && ye.drawLine(be, Ce, be, ve[3], fe[3], he[3]);
                  }
                }, ue.dispose = function() {
                  this.plot.removeOverlay(this);
                };
              }, 1493: function(ce, le, ue) {
                var de = ue(3236), fe = ue(9405), he = de([`precision mediump float;
#define GLSLIFY 1

attribute vec3 position, color;
attribute float weight;

uniform mat4 model, view, projection;
uniform vec3 coordinates[3];
uniform vec4 colors[3];
uniform vec2 screenShape;
uniform float lineWidth;

varying vec4 fragColor;

void main() {
  vec3 vertexPosition = mix(coordinates[0],
    mix(coordinates[2], coordinates[1], 0.5 * (position + 1.0)), abs(position));

  vec4 clipPos = projection * (view * (model * vec4(vertexPosition, 1.0)));
  vec2 clipOffset = (projection * (view * (model * vec4(color, 0.0)))).xy;
  vec2 delta = weight * clipOffset * screenShape;
  vec2 lineOffset = normalize(vec2(delta.y, -delta.x)) / screenShape;

  gl_Position   = vec4(clipPos.xy + clipPos.w * 0.5 * lineWidth * lineOffset, clipPos.z, clipPos.w);
  fragColor     = color.x * colors[0] + color.y * colors[1] + color.z * colors[2];
}
`]), me = de([`precision mediump float;
#define GLSLIFY 1

varying vec4 fragColor;

void main() {
  gl_FragColor = fragColor;
}`]);
                ce.exports = function(pe) {
                  return fe(pe, he, me, null, [{ name: "position", type: "vec3" }, { name: "color", type: "vec3" }, { name: "weight", type: "float" }]);
                };
              }, 4449: function(ce, le, ue) {
                var de = ue(2762), fe = ue(8116), he = ue(1493);
                ce.exports = function(Ce, Se) {
                  var we = [];
                  function Ee(Me, Re, De, Pe, $e, qe) {
                    var Xe = [Me, Re, De, 0, 0, 0, 1];
                    Xe[Pe + 3] = 1, Xe[Pe] = $e, we.push.apply(we, Xe), Xe[6] = -1, we.push.apply(we, Xe), Xe[Pe] = qe, we.push.apply(we, Xe), we.push.apply(we, Xe), Xe[6] = 1, we.push.apply(we, Xe), Xe[Pe] = $e, we.push.apply(we, Xe);
                  }
                  Ee(0, 0, 0, 0, 0, 1), Ee(0, 0, 0, 1, 0, 1), Ee(0, 0, 0, 2, 0, 1), Ee(1, 0, 0, 1, -1, 1), Ee(1, 0, 0, 2, -1, 1), Ee(0, 1, 0, 0, -1, 1), Ee(0, 1, 0, 2, -1, 1), Ee(0, 0, 1, 0, -1, 1), Ee(0, 0, 1, 1, -1, 1);
                  var xe = de(Ce, we), Le = fe(Ce, [{ type: Ce.FLOAT, buffer: xe, size: 3, offset: 0, stride: 28 }, { type: Ce.FLOAT, buffer: xe, size: 3, offset: 12, stride: 28 }, { type: Ce.FLOAT, buffer: xe, size: 1, offset: 24, stride: 28 }]), Qe = he(Ce);
                  Qe.attributes.position.location = 0, Qe.attributes.color.location = 1, Qe.attributes.weight.location = 2;
                  var ke = new pe(Ce, xe, Le, Qe);
                  return ke.update(Se), ke;
                };
                var me = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1];
                function pe(Ce, Se, we, Ee) {
                  this.gl = Ce, this.buffer = Se, this.vao = we, this.shader = Ee, this.pixelRatio = 1, this.bounds = [[-1e3, -1e3, -1e3], [1e3, 1e3, 1e3]], this.position = [0, 0, 0], this.lineWidth = [2, 2, 2], this.colors = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.enabled = [!0, !0, !0], this.drawSides = [!0, !0, !0], this.axes = null;
                }
                var ye = pe.prototype, ge = [0, 0, 0], ve = [0, 0, 0], be = [0, 0];
                ye.isTransparent = function() {
                  return !1;
                }, ye.drawTransparent = function(Ce) {
                }, ye.draw = function(Ce) {
                  var Se = this.gl, we = this.vao, Ee = this.shader;
                  we.bind(), Ee.bind();
                  var xe, Le = Ce.model || me, Qe = Ce.view || me, ke = Ce.projection || me;
                  this.axes && (xe = this.axes.lastCubeProps.axis);
                  for (var Me = ge, Re = ve, De = 0; De < 3; ++De) xe && xe[De] < 0 ? (Me[De] = this.bounds[0][De], Re[De] = this.bounds[1][De]) : (Me[De] = this.bounds[1][De], Re[De] = this.bounds[0][De]);
                  for (be[0] = Se.drawingBufferWidth, be[1] = Se.drawingBufferHeight, Ee.uniforms.model = Le, Ee.uniforms.view = Qe, Ee.uniforms.projection = ke, Ee.uniforms.coordinates = [this.position, Me, Re], Ee.uniforms.colors = this.colors, Ee.uniforms.screenShape = be, De = 0; De < 3; ++De) Ee.uniforms.lineWidth = this.lineWidth[De] * this.pixelRatio, this.enabled[De] && (we.draw(Se.TRIANGLES, 6, 6 * De), this.drawSides[De] && we.draw(Se.TRIANGLES, 12, 18 + 12 * De));
                  we.unbind();
                }, ye.update = function(Ce) {
                  Ce && ("bounds" in Ce && (this.bounds = Ce.bounds), "position" in Ce && (this.position = Ce.position), "lineWidth" in Ce && (this.lineWidth = Ce.lineWidth), "colors" in Ce && (this.colors = Ce.colors), "enabled" in Ce && (this.enabled = Ce.enabled), "drawSides" in Ce && (this.drawSides = Ce.drawSides));
                }, ye.dispose = function() {
                  this.vao.dispose(), this.buffer.dispose(), this.shader.dispose();
                };
              }, 6740: function(ce, le, ue) {
                var de = ue(3236), fe = de([`precision highp float;

precision highp float;
#define GLSLIFY 1

vec3 getOrthogonalVector(vec3 v) {
  // Return up-vector for only-z vector.
  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).
  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.
  // Assign z = 0, x = -b, y = a:
  // a*-b + b*a + c*0 = -ba + ba + 0 = 0
  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {
    return normalize(vec3(-v.y, v.x, 0.0));
  } else {
    return normalize(vec3(0.0, v.z, -v.y));
  }
}

// Calculate the tube vertex and normal at the given index.
//
// The returned vertex is for a tube ring with its center at origin, radius of length(d), pointing in the direction of d.
//
// Each tube segment is made up of a ring of vertices.
// These vertices are used to make up the triangles of the tube by connecting them together in the vertex array.
// The indexes of tube segments run from 0 to 8.
//
vec3 getTubePosition(vec3 d, float index, out vec3 normal) {
  float segmentCount = 8.0;

  float angle = 2.0 * 3.14159 * (index / segmentCount);

  vec3 u = getOrthogonalVector(d);
  vec3 v = normalize(cross(u, d));

  vec3 x = u * cos(angle) * length(d);
  vec3 y = v * sin(angle) * length(d);
  vec3 v3 = x + y;

  normal = normalize(v3);

  return v3;
}

attribute vec4 vector;
attribute vec4 color, position;
attribute vec2 uv;

uniform float vectorScale, tubeScale;
uniform mat4 model, view, projection, inverseModel;
uniform vec3 eyePosition, lightPosition;

varying vec3 f_normal, f_lightDirection, f_eyeDirection, f_data, f_position;
varying vec4 f_color;
varying vec2 f_uv;

void main() {
  // Scale the vector magnitude to stay constant with
  // model & view changes.
  vec3 normal;
  vec3 XYZ = getTubePosition(mat3(model) * (tubeScale * vector.w * normalize(vector.xyz)), position.w, normal);
  vec4 tubePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);

  //Lighting geometry parameters
  vec4 cameraCoordinate = view * tubePosition;
  cameraCoordinate.xyz /= cameraCoordinate.w;
  f_lightDirection = lightPosition - cameraCoordinate.xyz;
  f_eyeDirection   = eyePosition - cameraCoordinate.xyz;
  f_normal = normalize((vec4(normal, 0.0) * inverseModel).xyz);

  // vec4 m_position  = model * vec4(tubePosition, 1.0);
  vec4 t_position  = view * tubePosition;
  gl_Position      = projection * t_position;

  f_color          = color;
  f_data           = tubePosition.xyz;
  f_position       = position.xyz;
  f_uv             = uv;
}
`]), he = de([`#extension GL_OES_standard_derivatives : enable

precision highp float;
#define GLSLIFY 1

float beckmannDistribution(float x, float roughness) {
  float NdotH = max(x, 0.0001);
  float cos2Alpha = NdotH * NdotH;
  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;
  float roughness2 = roughness * roughness;
  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;
  return exp(tan2Alpha / roughness2) / denom;
}

float cookTorranceSpecular(
  vec3 lightDirection,
  vec3 viewDirection,
  vec3 surfaceNormal,
  float roughness,
  float fresnel) {

  float VdotN = max(dot(viewDirection, surfaceNormal), 0.0);
  float LdotN = max(dot(lightDirection, surfaceNormal), 0.0);

  //Half angle vector
  vec3 H = normalize(lightDirection + viewDirection);

  //Geometric term
  float NdotH = max(dot(surfaceNormal, H), 0.0);
  float VdotH = max(dot(viewDirection, H), 0.000001);
  float LdotH = max(dot(lightDirection, H), 0.000001);
  float G1 = (2.0 * NdotH * VdotN) / VdotH;
  float G2 = (2.0 * NdotH * LdotN) / LdotH;
  float G = min(1.0, min(G1, G2));
  
  //Distribution term
  float D = beckmannDistribution(NdotH, roughness);

  //Fresnel term
  float F = pow(1.0 - VdotN, fresnel);

  //Multiply terms and done
  return  G * F * D / max(3.14159265 * VdotN, 0.000001);
}

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 clipBounds[2];
uniform float roughness, fresnel, kambient, kdiffuse, kspecular, opacity;
uniform sampler2D texture;

varying vec3 f_normal, f_lightDirection, f_eyeDirection, f_data, f_position;
varying vec4 f_color;
varying vec2 f_uv;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;
  vec3 N = normalize(f_normal);
  vec3 L = normalize(f_lightDirection);
  vec3 V = normalize(f_eyeDirection);

  if(gl_FrontFacing) {
    N = -N;
  }

  float specular = min(1.0, max(0.0, cookTorranceSpecular(L, V, N, roughness, fresnel)));
  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);

  vec4 surfaceColor = f_color * texture2D(texture, f_uv);
  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);

  gl_FragColor = litColor * opacity;
}
`]), me = de([`precision highp float;

precision highp float;
#define GLSLIFY 1

vec3 getOrthogonalVector(vec3 v) {
  // Return up-vector for only-z vector.
  // Return ax + by + cz = 0, a point that lies on the plane that has v as a normal and that isn't (0,0,0).
  // From the above if-statement we have ||a|| > 0  U  ||b|| > 0.
  // Assign z = 0, x = -b, y = a:
  // a*-b + b*a + c*0 = -ba + ba + 0 = 0
  if (v.x*v.x > v.z*v.z || v.y*v.y > v.z*v.z) {
    return normalize(vec3(-v.y, v.x, 0.0));
  } else {
    return normalize(vec3(0.0, v.z, -v.y));
  }
}

// Calculate the tube vertex and normal at the given index.
//
// The returned vertex is for a tube ring with its center at origin, radius of length(d), pointing in the direction of d.
//
// Each tube segment is made up of a ring of vertices.
// These vertices are used to make up the triangles of the tube by connecting them together in the vertex array.
// The indexes of tube segments run from 0 to 8.
//
vec3 getTubePosition(vec3 d, float index, out vec3 normal) {
  float segmentCount = 8.0;

  float angle = 2.0 * 3.14159 * (index / segmentCount);

  vec3 u = getOrthogonalVector(d);
  vec3 v = normalize(cross(u, d));

  vec3 x = u * cos(angle) * length(d);
  vec3 y = v * sin(angle) * length(d);
  vec3 v3 = x + y;

  normal = normalize(v3);

  return v3;
}

attribute vec4 vector;
attribute vec4 position;
attribute vec4 id;

uniform mat4 model, view, projection;
uniform float tubeScale;

varying vec3 f_position;
varying vec4 f_id;

void main() {
  vec3 normal;
  vec3 XYZ = getTubePosition(mat3(model) * (tubeScale * vector.w * normalize(vector.xyz)), position.w, normal);
  vec4 tubePosition = model * vec4(position.xyz, 1.0) + vec4(XYZ, 0.0);

  gl_Position = projection * (view * tubePosition);
  f_id        = id;
  f_position  = position.xyz;
}
`]), pe = de([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3  clipBounds[2];
uniform float pickId;

varying vec3 f_position;
varying vec4 f_id;

void main() {
  if (outOfRange(clipBounds[0], clipBounds[1], f_position)) discard;

  gl_FragColor = vec4(pickId, f_id.xyz);
}`]);
                le.meshShader = { vertex: fe, fragment: he, attributes: [{ name: "position", type: "vec4" }, { name: "color", type: "vec4" }, { name: "uv", type: "vec2" }, { name: "vector", type: "vec4" }] }, le.pickShader = { vertex: me, fragment: pe, attributes: [{ name: "position", type: "vec4" }, { name: "id", type: "vec4" }, { name: "vector", type: "vec4" }] };
              }, 7815: function(ce, le, ue) {
                var de = ue(2931), fe = ue(9970), he = ["xyz", "xzy", "yxz", "yzx", "zxy", "zyx"], me = function(be, Ce) {
                  var Se, we = be.length;
                  for (Se = 0; Se < we; Se++) {
                    var Ee = be[Se];
                    if (Ee === Ce) return Se;
                    if (Ee > Ce) return Se - 1;
                  }
                  return Se;
                }, pe = function(be, Ce, Se) {
                  return be < Ce ? Ce : be > Se ? Se : be;
                }, ye = function(be) {
                  var Ce = 1 / 0;
                  be.sort(function(xe, Le) {
                    return xe - Le;
                  });
                  for (var Se = be.length, we = 1; we < Se; we++) {
                    var Ee = Math.abs(be[we] - be[we - 1]);
                    Ee < Ce && (Ce = Ee);
                  }
                  return Ce;
                };
                ce.exports = function(be, Ce) {
                  var Se = be.startingPositions, we = be.maxLength || 1e3, Ee = be.tubeSize || 1, xe = be.absoluteTubeSize, Le = be.gridFill || "+x+y+z", Qe = {};
                  Le.indexOf("-x") !== -1 && (Qe.reversedX = !0), Le.indexOf("-y") !== -1 && (Qe.reversedY = !0), Le.indexOf("-z") !== -1 && (Qe.reversedZ = !0), Qe.filled = he.indexOf(Le.replace(/-/g, "").replace(/\+/g, ""));
                  var ke = be.getVelocity || function(Pt) {
                    return function(Xt, Nt, Gt) {
                      var Ft = Nt.vectors, Tt = Nt.meshgrid, It = Xt[0], Ct = Xt[1], Kt = Xt[2], Vt = Tt[0].length, Yt = Tt[1].length, sn = Tt[2].length, hn = me(Tt[0], It), Qt = me(Tt[1], Ct), Ut = me(Tt[2], Kt), Wt = hn + 1, rn = Qt + 1, vn = Ut + 1;
                      if (hn = pe(hn, 0, Vt - 1), Wt = pe(Wt, 0, Vt - 1), Qt = pe(Qt, 0, Yt - 1), rn = pe(rn, 0, Yt - 1), Ut = pe(Ut, 0, sn - 1), vn = pe(vn, 0, sn - 1), hn < 0 || Qt < 0 || Ut < 0 || Wt > Vt - 1 || rn > Yt - 1 || vn > sn - 1) return de.create();
                      var dn, Ht, jt, cn, wn, Rn, Pn = Tt[0][hn], Ie = Tt[0][Wt], rt = Tt[1][Qt], ft = Tt[1][rn], pt = Tt[2][Ut], wt = (It - Pn) / (Ie - Pn), Bt = (Ct - rt) / (ft - rt), At = (Kt - pt) / (Tt[2][vn] - pt);
                      switch (isFinite(wt) || (wt = 0.5), isFinite(Bt) || (Bt = 0.5), isFinite(At) || (At = 0.5), Gt.reversedX && (hn = Vt - 1 - hn, Wt = Vt - 1 - Wt), Gt.reversedY && (Qt = Yt - 1 - Qt, rn = Yt - 1 - rn), Gt.reversedZ && (Ut = sn - 1 - Ut, vn = sn - 1 - vn), Gt.filled) {
                        case 5:
                          wn = Ut, Rn = vn, jt = Qt * sn, cn = rn * sn, dn = hn * sn * Yt, Ht = Wt * sn * Yt;
                          break;
                        case 4:
                          wn = Ut, Rn = vn, dn = hn * sn, Ht = Wt * sn, jt = Qt * sn * Vt, cn = rn * sn * Vt;
                          break;
                        case 3:
                          jt = Qt, cn = rn, wn = Ut * Yt, Rn = vn * Yt, dn = hn * Yt * sn, Ht = Wt * Yt * sn;
                          break;
                        case 2:
                          jt = Qt, cn = rn, dn = hn * Yt, Ht = Wt * Yt, wn = Ut * Yt * Vt, Rn = vn * Yt * Vt;
                          break;
                        case 1:
                          dn = hn, Ht = Wt, wn = Ut * Vt, Rn = vn * Vt, jt = Qt * Vt * sn, cn = rn * Vt * sn;
                          break;
                        default:
                          dn = hn, Ht = Wt, jt = Qt * Vt, cn = rn * Vt, wn = Ut * Vt * Yt, Rn = vn * Vt * Yt;
                      }
                      var Dt = Ft[dn + jt + wn], Ot = Ft[dn + jt + Rn], un = Ft[dn + cn + wn], yn = Ft[dn + cn + Rn], _n = Ft[Ht + jt + wn], gn = Ft[Ht + jt + Rn], bn = Ft[Ht + cn + wn], an = Ft[Ht + cn + Rn], Zt = de.create(), Mn = de.create(), Ln = de.create(), In = de.create();
                      de.lerp(Zt, Dt, _n, wt), de.lerp(Mn, Ot, gn, wt), de.lerp(Ln, un, bn, wt), de.lerp(In, yn, an, wt);
                      var kn = de.create(), er = de.create();
                      de.lerp(kn, Zt, Ln, Bt), de.lerp(er, Mn, In, Bt);
                      var jn = de.create();
                      return de.lerp(jn, kn, er, At), jn;
                    }(Pt, be, Qe);
                  }, Me = be.getDivergence || function(Pt, Xt) {
                    var Nt = de.create(), Gt = 1e-4;
                    de.add(Nt, Pt, [Gt, 0, 0]);
                    var Ft = ke(Nt);
                    de.subtract(Ft, Ft, Xt), de.scale(Ft, Ft, 1 / Gt), de.add(Nt, Pt, [0, Gt, 0]);
                    var Tt = ke(Nt);
                    de.subtract(Tt, Tt, Xt), de.scale(Tt, Tt, 1 / Gt), de.add(Nt, Pt, [0, 0, Gt]);
                    var It = ke(Nt);
                    return de.subtract(It, It, Xt), de.scale(It, It, 1 / Gt), de.add(Nt, Ft, Tt), de.add(Nt, Nt, It), Nt;
                  }, Re = [], De = Ce[0][0], Pe = Ce[0][1], $e = Ce[0][2], qe = Ce[1][0], Xe = Ce[1][1], Ve = Ce[1][2], Ze = function(Pt) {
                    var Xt = Pt[0], Nt = Pt[1], Gt = Pt[2];
                    return !(Xt < De || Xt > qe || Nt < Pe || Nt > Xe || Gt < $e || Gt > Ve);
                  }, Ge = 10 * de.distance(Ce[0], Ce[1]) / we, ze = Ge * Ge, We = 1, at = 0, ct = Se.length;
                  ct > 1 && (We = function(Pt) {
                    for (var Xt = [], Nt = [], Gt = [], Ft = {}, Tt = {}, It = {}, Ct = Pt.length, Kt = 0; Kt < Ct; Kt++) {
                      var Vt = Pt[Kt], Yt = Vt[0], sn = Vt[1], hn = Vt[2];
                      Ft[Yt] || (Xt.push(Yt), Ft[Yt] = !0), Tt[sn] || (Nt.push(sn), Tt[sn] = !0), It[hn] || (Gt.push(hn), It[hn] = !0);
                    }
                    var Qt = ye(Xt), Ut = ye(Nt), Wt = ye(Gt), rn = Math.min(Qt, Ut, Wt);
                    return isFinite(rn) ? rn : 1;
                  }(Se));
                  for (var et = 0; et < ct; et++) {
                    var dt = de.create();
                    de.copy(dt, Se[et]);
                    var Je = [dt], it = [], st = ke(dt), gt = dt;
                    it.push(st);
                    var bt = [], yt = Me(dt, st), St = de.length(yt);
                    isFinite(St) && St > at && (at = St), bt.push(St), Re.push({ points: Je, velocities: it, divergences: bt });
                    for (var Lt = 0; Lt < 100 * we && Je.length < we && Ze(dt); ) {
                      Lt++;
                      var Rt = de.clone(st), vt = de.squaredLength(Rt);
                      if (vt === 0) break;
                      vt > ze && de.scale(Rt, Rt, Ge / Math.sqrt(vt)), de.add(Rt, Rt, dt), st = ke(Rt), de.squaredDistance(gt, Rt) - ze > -1e-4 * ze && (Je.push(Rt), gt = Rt, it.push(st), yt = Me(Rt, st), St = de.length(yt), isFinite(St) && St > at && (at = St), bt.push(St)), dt = Rt;
                    }
                  }
                  var $t = function(Pt, Xt, Nt, Gt) {
                    for (var Ft = 0, Tt = 0; Tt < Pt.length; Tt++) for (var It = Pt[Tt].velocities, Ct = 0; Ct < It.length; Ct++) Ft = Math.max(Ft, de.length(It[Ct]));
                    var Kt = Pt.map(function(dn) {
                      return function(Ht, jt, cn, wn) {
                        for (var Rn = Ht.points, Pn = Ht.velocities, Ie = Ht.divergences, rt = [], ft = [], pt = [], wt = [], Bt = [], At = [], Dt = 0, Ot = 0, un = fe.create(), yn = fe.create(), _n = 0; _n < Rn.length; _n++) {
                          var gn = Rn[_n], bn = Pn[_n], an = Ie[_n];
                          jt === 0 && (an = 0.05 * cn), Ot = de.length(bn) / wn, un = fe.create(), de.copy(un, bn), un[3] = an;
                          for (var Zt = 0; Zt < 8; Zt++) Bt[Zt] = [gn[0], gn[1], gn[2], Zt];
                          if (wt.length > 0) for (Zt = 0; Zt < 8; Zt++) {
                            var Mn = (Zt + 1) % 8;
                            rt.push(wt[Zt], Bt[Zt], Bt[Mn], Bt[Mn], wt[Mn], wt[Zt]), pt.push(yn, un, un, un, yn, yn), At.push(Dt, Ot, Ot, Ot, Dt, Dt);
                            var Ln = rt.length;
                            ft.push([Ln - 6, Ln - 5, Ln - 4], [Ln - 3, Ln - 2, Ln - 1]);
                          }
                          var In = wt;
                          wt = Bt, Bt = In;
                          var kn = yn;
                          yn = un, un = kn;
                          var er = Dt;
                          Dt = Ot, Ot = er;
                        }
                        return { positions: rt, cells: ft, vectors: pt, vertexIntensity: At };
                      }(dn, Nt, Gt, Ft);
                    }), Vt = [], Yt = [], sn = [], hn = [];
                    for (Tt = 0; Tt < Kt.length; Tt++) {
                      var Qt = Kt[Tt], Ut = Vt.length;
                      for (Vt = Vt.concat(Qt.positions), sn = sn.concat(Qt.vectors), hn = hn.concat(Qt.vertexIntensity), Ct = 0; Ct < Qt.cells.length; Ct++) {
                        var Wt = Qt.cells[Ct], rn = [];
                        Yt.push(rn);
                        for (var vn = 0; vn < Wt.length; vn++) rn.push(Wt[vn] + Ut);
                      }
                    }
                    return { positions: Vt, cells: Yt, vectors: sn, vertexIntensity: hn, colormap: Xt };
                  }(Re, be.colormap, at, We);
                  return xe ? $t.tubeScale = xe : (at === 0 && (at = 1), $t.tubeScale = 0.5 * Ee * We / at), $t;
                };
                var ge = ue(6740), ve = ue(6405).createMesh;
                ce.exports.createTubeMesh = function(be, Ce) {
                  return ve(be, Ce, { shaders: ge, traceType: "streamtube" });
                };
              }, 990: function(ce, le, ue) {
                var de = ue(9405), fe = ue(3236), he = fe([`precision highp float;
#define GLSLIFY 1

attribute vec4 uv;
attribute vec3 f;
attribute vec3 normal;

uniform vec3 objectOffset;
uniform mat4 model, view, projection, inverseModel;
uniform vec3 lightPosition, eyePosition;
uniform sampler2D colormap;

varying float value, kill;
varying vec3 worldCoordinate;
varying vec2 planeCoordinate;
varying vec3 lightDirection, eyeDirection, surfaceNormal;
varying vec4 vColor;

void main() {
  vec3 localCoordinate = vec3(uv.zw, f.x);
  worldCoordinate = objectOffset + localCoordinate;
  mat4 objectOffsetTranslation = mat4(1.0) + mat4(vec4(0), vec4(0), vec4(0), vec4(objectOffset, 0));
  vec4 worldPosition = (model * objectOffsetTranslation) * vec4(localCoordinate, 1.0);
  vec4 clipPosition = projection * (view * worldPosition);
  gl_Position = clipPosition;
  kill = f.y;
  value = f.z;
  planeCoordinate = uv.xy;

  vColor = texture2D(colormap, vec2(value, value));

  //Lighting geometry parameters
  vec4 cameraCoordinate = view * worldPosition;
  cameraCoordinate.xyz /= cameraCoordinate.w;
  lightDirection = lightPosition - cameraCoordinate.xyz;
  eyeDirection   = eyePosition - cameraCoordinate.xyz;
  surfaceNormal  = normalize((vec4(normal,0) * inverseModel).xyz);
}
`]), me = fe([`precision highp float;
#define GLSLIFY 1

float beckmannDistribution(float x, float roughness) {
  float NdotH = max(x, 0.0001);
  float cos2Alpha = NdotH * NdotH;
  float tan2Alpha = (cos2Alpha - 1.0) / cos2Alpha;
  float roughness2 = roughness * roughness;
  float denom = 3.141592653589793 * roughness2 * cos2Alpha * cos2Alpha;
  return exp(tan2Alpha / roughness2) / denom;
}

float beckmannSpecular(
  vec3 lightDirection,
  vec3 viewDirection,
  vec3 surfaceNormal,
  float roughness) {
  return beckmannDistribution(dot(surfaceNormal, normalize(lightDirection + viewDirection)), roughness);
}

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec3 lowerBound, upperBound;
uniform float contourTint;
uniform vec4 contourColor;
uniform sampler2D colormap;
uniform vec3 clipBounds[2];
uniform float roughness, fresnel, kambient, kdiffuse, kspecular, opacity;
uniform float vertexColor;

varying float value, kill;
varying vec3 worldCoordinate;
varying vec3 lightDirection, eyeDirection, surfaceNormal;
varying vec4 vColor;

void main() {
  if (
    kill > 0.0 ||
    vColor.a == 0.0 ||
    outOfRange(clipBounds[0], clipBounds[1], worldCoordinate)
  ) discard;

  vec3 N = normalize(surfaceNormal);
  vec3 V = normalize(eyeDirection);
  vec3 L = normalize(lightDirection);

  if(gl_FrontFacing) {
    N = -N;
  }

  float specular = max(beckmannSpecular(L, V, N, roughness), 0.);
  float diffuse  = min(kambient + kdiffuse * max(dot(N, L), 0.0), 1.0);

  //decide how to interpolate color — in vertex or in fragment
  vec4 surfaceColor =
    step(vertexColor, .5) * texture2D(colormap, vec2(value, value)) +
    step(.5, vertexColor) * vColor;

  vec4 litColor = surfaceColor.a * vec4(diffuse * surfaceColor.rgb + kspecular * vec3(1,1,1) * specular,  1.0);

  gl_FragColor = mix(litColor, contourColor, contourTint) * opacity;
}
`]), pe = fe([`precision highp float;
#define GLSLIFY 1

attribute vec4 uv;
attribute float f;

uniform vec3 objectOffset;
uniform mat3 permutation;
uniform mat4 model, view, projection;
uniform float height, zOffset;
uniform sampler2D colormap;

varying float value, kill;
varying vec3 worldCoordinate;
varying vec2 planeCoordinate;
varying vec3 lightDirection, eyeDirection, surfaceNormal;
varying vec4 vColor;

void main() {
  vec3 dataCoordinate = permutation * vec3(uv.xy, height);
  worldCoordinate = objectOffset + dataCoordinate;
  mat4 objectOffsetTranslation = mat4(1.0) + mat4(vec4(0), vec4(0), vec4(0), vec4(objectOffset, 0));
  vec4 worldPosition = (model * objectOffsetTranslation) * vec4(dataCoordinate, 1.0);

  vec4 clipPosition = projection * (view * worldPosition);
  clipPosition.z += zOffset;

  gl_Position = clipPosition;
  value = f + objectOffset.z;
  kill = -1.0;
  planeCoordinate = uv.zw;

  vColor = texture2D(colormap, vec2(value, value));

  //Don't do lighting for contours
  surfaceNormal   = vec3(1,0,0);
  eyeDirection    = vec3(0,1,0);
  lightDirection  = vec3(0,0,1);
}
`]), ye = fe([`precision highp float;
#define GLSLIFY 1

bool outOfRange(float a, float b, float p) {
  return ((p > max(a, b)) || 
          (p < min(a, b)));
}

bool outOfRange(vec2 a, vec2 b, vec2 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y));
}

bool outOfRange(vec3 a, vec3 b, vec3 p) {
  return (outOfRange(a.x, b.x, p.x) ||
          outOfRange(a.y, b.y, p.y) ||
          outOfRange(a.z, b.z, p.z));
}

bool outOfRange(vec4 a, vec4 b, vec4 p) {
  return outOfRange(a.xyz, b.xyz, p.xyz);
}

uniform vec2 shape;
uniform vec3 clipBounds[2];
uniform float pickId;

varying float value, kill;
varying vec3 worldCoordinate;
varying vec2 planeCoordinate;
varying vec3 surfaceNormal;

vec2 splitFloat(float v) {
  float vh = 255.0 * v;
  float upper = floor(vh);
  float lower = fract(vh);
  return vec2(upper / 255.0, floor(lower * 16.0) / 16.0);
}

void main() {
  if ((kill > 0.0) ||
      (outOfRange(clipBounds[0], clipBounds[1], worldCoordinate))) discard;

  vec2 ux = splitFloat(planeCoordinate.x / shape.x);
  vec2 uy = splitFloat(planeCoordinate.y / shape.y);
  gl_FragColor = vec4(pickId, ux.x, uy.x, ux.y + (uy.y/16.0));
}
`]);
                le.createShader = function(ge) {
                  var ve = de(ge, he, me, null, [{ name: "uv", type: "vec4" }, { name: "f", type: "vec3" }, { name: "normal", type: "vec3" }]);
                  return ve.attributes.uv.location = 0, ve.attributes.f.location = 1, ve.attributes.normal.location = 2, ve;
                }, le.createPickShader = function(ge) {
                  var ve = de(ge, he, ye, null, [{ name: "uv", type: "vec4" }, { name: "f", type: "vec3" }, { name: "normal", type: "vec3" }]);
                  return ve.attributes.uv.location = 0, ve.attributes.f.location = 1, ve.attributes.normal.location = 2, ve;
                }, le.createContourShader = function(ge) {
                  var ve = de(ge, pe, me, null, [{ name: "uv", type: "vec4" }, { name: "f", type: "float" }]);
                  return ve.attributes.uv.location = 0, ve.attributes.f.location = 1, ve;
                }, le.createPickContourShader = function(ge) {
                  var ve = de(ge, pe, ye, null, [{ name: "uv", type: "vec4" }, { name: "f", type: "float" }]);
                  return ve.attributes.uv.location = 0, ve.attributes.f.location = 1, ve;
                };
              }, 9499: function(ce, le, ue) {
                ce.exports = function(yt) {
                  var St = yt.gl, Lt = Qe(St), Rt = Me(St), vt = ke(St), $t = Re(St), Pt = fe(St), Xt = he(St, [{ buffer: Pt, size: 4, stride: De, offset: 0 }, { buffer: Pt, size: 3, stride: De, offset: 16 }, { buffer: Pt, size: 3, stride: De, offset: 28 }]), Nt = fe(St), Gt = he(St, [{ buffer: Nt, size: 4, stride: 20, offset: 0 }, { buffer: Nt, size: 1, stride: 20, offset: 16 }]), Ft = fe(St), Tt = he(St, [{ buffer: Ft, size: 2, type: St.FLOAT }]), It = me(St, 1, Ve, St.RGBA, St.UNSIGNED_BYTE);
                  It.minFilter = St.LINEAR, It.magFilter = St.LINEAR;
                  var Ct = new Ze(St, [0, 0], [[0, 0, 0], [0, 0, 0]], Lt, Rt, Pt, Xt, It, vt, $t, Nt, Gt, Ft, Tt, [0, 0, 0]), Kt = { levels: [[], [], []] };
                  for (var Vt in yt) Kt[Vt] = yt[Vt];
                  return Kt.colormap = Kt.colormap || "jet", Ct.update(Kt), Ct;
                };
                var de = ue(8828), fe = ue(2762), he = ue(8116), me = ue(7766), pe = ue(1888), ye = ue(6729), ge = ue(5298), ve = ue(9994), be = ue(9618), Ce = ue(3711), Se = ue(6760), we = ue(7608), Ee = ue(2478), xe = ue(6199), Le = ue(990), Qe = Le.createShader, ke = Le.createContourShader, Me = Le.createPickShader, Re = Le.createPickContourShader, De = 40, Pe = [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1], $e = [[0, 0], [0, 1], [1, 0], [1, 1], [1, 0], [0, 1]], qe = [[0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0]];
                function Xe(yt, St, Lt, Rt, vt) {
                  this.position = yt, this.index = St, this.uv = Lt, this.level = Rt, this.dataCoordinate = vt;
                }
                (function() {
                  for (var yt = 0; yt < 3; ++yt) {
                    var St = qe[yt], Lt = (yt + 2) % 3;
                    St[(yt + 1) % 3 + 0] = 1, St[Lt + 3] = 1, St[yt + 6] = 1;
                  }
                })();
                var Ve = 256;
                function Ze(yt, St, Lt, Rt, vt, $t, Pt, Xt, Nt, Gt, Ft, Tt, It, Ct, Kt) {
                  this.gl = yt, this.shape = St, this.bounds = Lt, this.objectOffset = Kt, this.intensityBounds = [], this._shader = Rt, this._pickShader = vt, this._coordinateBuffer = $t, this._vao = Pt, this._colorMap = Xt, this._contourShader = Nt, this._contourPickShader = Gt, this._contourBuffer = Ft, this._contourVAO = Tt, this._contourOffsets = [[], [], []], this._contourCounts = [[], [], []], this._vertexCount = 0, this._pickResult = new Xe([0, 0, 0], [0, 0], [0, 0], [0, 0, 0], [0, 0, 0]), this._dynamicBuffer = It, this._dynamicVAO = Ct, this._dynamicOffsets = [0, 0, 0], this._dynamicCounts = [0, 0, 0], this.contourWidth = [1, 1, 1], this.contourLevels = [[1], [1], [1]], this.contourTint = [0, 0, 0], this.contourColor = [[0.5, 0.5, 0.5, 1], [0.5, 0.5, 0.5, 1], [0.5, 0.5, 0.5, 1]], this.showContour = !0, this.showSurface = !0, this.enableHighlight = [!0, !0, !0], this.highlightColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.highlightTint = [1, 1, 1], this.highlightLevel = [-1, -1, -1], this.enableDynamic = [!0, !0, !0], this.dynamicLevel = [NaN, NaN, NaN], this.dynamicColor = [[0, 0, 0, 1], [0, 0, 0, 1], [0, 0, 0, 1]], this.dynamicTint = [1, 1, 1], this.dynamicWidth = [1, 1, 1], this.axesBounds = [[1 / 0, 1 / 0, 1 / 0], [-1 / 0, -1 / 0, -1 / 0]], this.surfaceProject = [!1, !1, !1], this.contourProject = [[!1, !1, !1], [!1, !1, !1], [!1, !1, !1]], this.colorBounds = [!1, !1], this._field = [be(pe.mallocFloat(1024), [0, 0]), be(pe.mallocFloat(1024), [0, 0]), be(pe.mallocFloat(1024), [0, 0])], this.pickId = 1, this.clipBounds = [[-1 / 0, -1 / 0, -1 / 0], [1 / 0, 1 / 0, 1 / 0]], this.snapToData = !1, this.pixelRatio = 1, this.opacity = 1, this.lightPosition = [10, 1e4, 0], this.ambientLight = 0.8, this.diffuseLight = 0.8, this.specularLight = 2, this.roughness = 0.5, this.fresnel = 1.5, this.vertexColor = 0, this.dirty = !0;
                }
                var Ge = Ze.prototype;
                Ge.genColormap = function(yt, St) {
                  var Lt = !1, Rt = ve([ye({ colormap: yt, nshades: Ve, format: "rgba" }).map(function(vt, $t) {
                    var Pt = St ? function(Xt, Nt) {
                      if (!Nt || !Nt.length) return 1;
                      for (var Gt = 0; Gt < Nt.length; ++Gt) {
                        if (Nt.length < 2) return 1;
                        if (Nt[Gt][0] === Xt) return Nt[Gt][1];
                        if (Nt[Gt][0] > Xt && Gt > 0) {
                          var Ft = (Nt[Gt][0] - Xt) / (Nt[Gt][0] - Nt[Gt - 1][0]);
                          return Nt[Gt][1] * (1 - Ft) + Ft * Nt[Gt - 1][1];
                        }
                      }
                      return 1;
                    }($t / 255, St) : vt[3];
                    return Pt < 1 && (Lt = !0), [vt[0], vt[1], vt[2], 255 * Pt];
                  })]);
                  return ge.divseq(Rt, 255), this.hasAlphaScale = Lt, Rt;
                }, Ge.isTransparent = function() {
                  return this.opacity < 1 || this.hasAlphaScale;
                }, Ge.isOpaque = function() {
                  return !this.isTransparent();
                }, Ge.pickSlots = 1, Ge.setPickBase = function(yt) {
                  this.pickId = yt;
                };
                var ze = [0, 0, 0], We = { showSurface: !1, showContour: !1, projections: [Pe.slice(), Pe.slice(), Pe.slice()], clipBounds: [[[0, 0, 0], [0, 0, 0]], [[0, 0, 0], [0, 0, 0]], [[0, 0, 0], [0, 0, 0]]] };
                function at(yt, St) {
                  var Lt, Rt, vt, $t = St.axes && St.axes.lastCubeProps.axis || ze, Pt = St.showSurface, Xt = St.showContour;
                  for (Lt = 0; Lt < 3; ++Lt) for (Pt = Pt || St.surfaceProject[Lt], Rt = 0; Rt < 3; ++Rt) Xt = Xt || St.contourProject[Lt][Rt];
                  for (Lt = 0; Lt < 3; ++Lt) {
                    var Nt = We.projections[Lt];
                    for (Rt = 0; Rt < 16; ++Rt) Nt[Rt] = 0;
                    for (Rt = 0; Rt < 4; ++Rt) Nt[5 * Rt] = 1;
                    Nt[5 * Lt] = 0, Nt[12 + Lt] = St.axesBounds[+($t[Lt] > 0)][Lt], Se(Nt, yt.model, Nt);
                    var Gt = We.clipBounds[Lt];
                    for (vt = 0; vt < 2; ++vt) for (Rt = 0; Rt < 3; ++Rt) Gt[vt][Rt] = yt.clipBounds[vt][Rt];
                    Gt[0][Lt] = -1e8, Gt[1][Lt] = 1e8;
                  }
                  return We.showSurface = Pt, We.showContour = Xt, We;
                }
                var ct = { model: Pe, view: Pe, projection: Pe, inverseModel: Pe.slice(), lowerBound: [0, 0, 0], upperBound: [0, 0, 0], colorMap: 0, clipBounds: [[0, 0, 0], [0, 0, 0]], height: 0, contourTint: 0, contourColor: [0, 0, 0, 1], permutation: [1, 0, 0, 0, 1, 0, 0, 0, 1], zOffset: -1e-4, objectOffset: [0, 0, 0], kambient: 1, kdiffuse: 1, kspecular: 1, lightPosition: [1e3, 1e3, 1e3], eyePosition: [0, 0, 0], roughness: 1, fresnel: 1, opacity: 1, vertexColor: 0 }, et = Pe.slice(), dt = [1, 0, 0, 0, 1, 0, 0, 0, 1];
                function Je(yt, St) {
                  yt = yt || {};
                  var Lt = this.gl;
                  Lt.disable(Lt.CULL_FACE), this._colorMap.bind(0);
                  var Rt = ct;
                  Rt.model = yt.model || Pe, Rt.view = yt.view || Pe, Rt.projection = yt.projection || Pe, Rt.lowerBound = [this.bounds[0][0], this.bounds[0][1], this.colorBounds[0] || this.bounds[0][2]], Rt.upperBound = [this.bounds[1][0], this.bounds[1][1], this.colorBounds[1] || this.bounds[1][2]], Rt.objectOffset = this.objectOffset, Rt.contourColor = this.contourColor[0], Rt.inverseModel = we(Rt.inverseModel, Rt.model);
                  for (var vt = 0; vt < 2; ++vt) for (var $t = Rt.clipBounds[vt], Pt = 0; Pt < 3; ++Pt) $t[Pt] = Math.min(Math.max(this.clipBounds[vt][Pt], -1e8), 1e8);
                  Rt.kambient = this.ambientLight, Rt.kdiffuse = this.diffuseLight, Rt.kspecular = this.specularLight, Rt.roughness = this.roughness, Rt.fresnel = this.fresnel, Rt.opacity = this.opacity, Rt.height = 0, Rt.permutation = dt, Rt.vertexColor = this.vertexColor;
                  var Xt = et;
                  for (Se(Xt, Rt.view, Rt.model), Se(Xt, Rt.projection, Xt), we(Xt, Xt), vt = 0; vt < 3; ++vt) Rt.eyePosition[vt] = Xt[12 + vt] / Xt[15];
                  var Nt = Xt[15];
                  for (vt = 0; vt < 3; ++vt) Nt += this.lightPosition[vt] * Xt[4 * vt + 3];
                  for (vt = 0; vt < 3; ++vt) {
                    var Gt = Xt[12 + vt];
                    for (Pt = 0; Pt < 3; ++Pt) Gt += Xt[4 * Pt + vt] * this.lightPosition[Pt];
                    Rt.lightPosition[vt] = Gt / Nt;
                  }
                  var Ft = at(Rt, this);
                  if (Ft.showSurface) {
                    for (this._shader.bind(), this._shader.uniforms = Rt, this._vao.bind(), this.showSurface && this._vertexCount && this._vao.draw(Lt.TRIANGLES, this._vertexCount), vt = 0; vt < 3; ++vt) this.surfaceProject[vt] && this.vertexCount && (this._shader.uniforms.model = Ft.projections[vt], this._shader.uniforms.clipBounds = Ft.clipBounds[vt], this._vao.draw(Lt.TRIANGLES, this._vertexCount));
                    this._vao.unbind();
                  }
                  if (Ft.showContour) {
                    var Tt = this._contourShader;
                    Rt.kambient = 1, Rt.kdiffuse = 0, Rt.kspecular = 0, Rt.opacity = 1, Tt.bind(), Tt.uniforms = Rt;
                    var It = this._contourVAO;
                    for (It.bind(), vt = 0; vt < 3; ++vt) for (Tt.uniforms.permutation = qe[vt], Lt.lineWidth(this.contourWidth[vt] * this.pixelRatio), Pt = 0; Pt < this.contourLevels[vt].length; ++Pt) Pt === this.highlightLevel[vt] ? (Tt.uniforms.contourColor = this.highlightColor[vt], Tt.uniforms.contourTint = this.highlightTint[vt]) : Pt !== 0 && Pt - 1 !== this.highlightLevel[vt] || (Tt.uniforms.contourColor = this.contourColor[vt], Tt.uniforms.contourTint = this.contourTint[vt]), this._contourCounts[vt][Pt] && (Tt.uniforms.height = this.contourLevels[vt][Pt], It.draw(Lt.LINES, this._contourCounts[vt][Pt], this._contourOffsets[vt][Pt]));
                    for (vt = 0; vt < 3; ++vt) for (Tt.uniforms.model = Ft.projections[vt], Tt.uniforms.clipBounds = Ft.clipBounds[vt], Pt = 0; Pt < 3; ++Pt) if (this.contourProject[vt][Pt]) {
                      Tt.uniforms.permutation = qe[Pt], Lt.lineWidth(this.contourWidth[Pt] * this.pixelRatio);
                      for (var Ct = 0; Ct < this.contourLevels[Pt].length; ++Ct) Ct === this.highlightLevel[Pt] ? (Tt.uniforms.contourColor = this.highlightColor[Pt], Tt.uniforms.contourTint = this.highlightTint[Pt]) : Ct !== 0 && Ct - 1 !== this.highlightLevel[Pt] || (Tt.uniforms.contourColor = this.contourColor[Pt], Tt.uniforms.contourTint = this.contourTint[Pt]), this._contourCounts[Pt][Ct] && (Tt.uniforms.height = this.contourLevels[Pt][Ct], It.draw(Lt.LINES, this._contourCounts[Pt][Ct], this._contourOffsets[Pt][Ct]));
                    }
                    for (It.unbind(), (It = this._dynamicVAO).bind(), vt = 0; vt < 3; ++vt) if (this._dynamicCounts[vt] !== 0) for (Tt.uniforms.model = Rt.model, Tt.uniforms.clipBounds = Rt.clipBounds, Tt.uniforms.permutation = qe[vt], Lt.lineWidth(this.dynamicWidth[vt] * this.pixelRatio), Tt.uniforms.contourColor = this.dynamicColor[vt], Tt.uniforms.contourTint = this.dynamicTint[vt], Tt.uniforms.height = this.dynamicLevel[vt], It.draw(Lt.LINES, this._dynamicCounts[vt], this._dynamicOffsets[vt]), Pt = 0; Pt < 3; ++Pt) this.contourProject[Pt][vt] && (Tt.uniforms.model = Ft.projections[Pt], Tt.uniforms.clipBounds = Ft.clipBounds[Pt], It.draw(Lt.LINES, this._dynamicCounts[vt], this._dynamicOffsets[vt]));
                    It.unbind();
                  }
                }
                Ge.draw = function(yt) {
                  return Je.call(this, yt, !1);
                }, Ge.drawTransparent = function(yt) {
                  return Je.call(this, yt, !0);
                };
                var it = { model: Pe, view: Pe, projection: Pe, inverseModel: Pe, clipBounds: [[0, 0, 0], [0, 0, 0]], height: 0, shape: [0, 0], pickId: 0, lowerBound: [0, 0, 0], upperBound: [0, 0, 0], zOffset: 0, objectOffset: [0, 0, 0], permutation: [1, 0, 0, 0, 1, 0, 0, 0, 1], lightPosition: [0, 0, 0], eyePosition: [0, 0, 0] };
                function st(yt, St) {
                  return Array.isArray(yt) ? [St(yt[0]), St(yt[1]), St(yt[2])] : [St(yt), St(yt), St(yt)];
                }
                function gt(yt) {
                  return Array.isArray(yt) ? yt.length === 3 ? [yt[0], yt[1], yt[2], 1] : [yt[0], yt[1], yt[2], yt[3]] : [0, 0, 0, 1];
                }
                function bt(yt) {
                  if (Array.isArray(yt)) {
                    if (Array.isArray(yt)) return [gt(yt[0]), gt(yt[1]), gt(yt[2])];
                    var St = gt(yt);
                    return [St.slice(), St.slice(), St.slice()];
                  }
                }
                Ge.drawPick = function(yt) {
                  yt = yt || {};
                  var St = this.gl;
                  St.disable(St.CULL_FACE);
                  var Lt = it;
                  Lt.model = yt.model || Pe, Lt.view = yt.view || Pe, Lt.projection = yt.projection || Pe, Lt.shape = this._field[2].shape, Lt.pickId = this.pickId / 255, Lt.lowerBound = this.bounds[0], Lt.upperBound = this.bounds[1], Lt.objectOffset = this.objectOffset, Lt.permutation = dt;
                  for (var Rt = 0; Rt < 2; ++Rt) for (var vt = Lt.clipBounds[Rt], $t = 0; $t < 3; ++$t) vt[$t] = Math.min(Math.max(this.clipBounds[Rt][$t], -1e8), 1e8);
                  var Pt = at(Lt, this);
                  if (Pt.showSurface) {
                    for (this._pickShader.bind(), this._pickShader.uniforms = Lt, this._vao.bind(), this._vao.draw(St.TRIANGLES, this._vertexCount), Rt = 0; Rt < 3; ++Rt) this.surfaceProject[Rt] && (this._pickShader.uniforms.model = Pt.projections[Rt], this._pickShader.uniforms.clipBounds = Pt.clipBounds[Rt], this._vao.draw(St.TRIANGLES, this._vertexCount));
                    this._vao.unbind();
                  }
                  if (Pt.showContour) {
                    var Xt = this._contourPickShader;
                    Xt.bind(), Xt.uniforms = Lt;
                    var Nt = this._contourVAO;
                    for (Nt.bind(), $t = 0; $t < 3; ++$t) for (St.lineWidth(this.contourWidth[$t] * this.pixelRatio), Xt.uniforms.permutation = qe[$t], Rt = 0; Rt < this.contourLevels[$t].length; ++Rt) this._contourCounts[$t][Rt] && (Xt.uniforms.height = this.contourLevels[$t][Rt], Nt.draw(St.LINES, this._contourCounts[$t][Rt], this._contourOffsets[$t][Rt]));
                    for (Rt = 0; Rt < 3; ++Rt) for (Xt.uniforms.model = Pt.projections[Rt], Xt.uniforms.clipBounds = Pt.clipBounds[Rt], $t = 0; $t < 3; ++$t) if (this.contourProject[Rt][$t]) {
                      Xt.uniforms.permutation = qe[$t], St.lineWidth(this.contourWidth[$t] * this.pixelRatio);
                      for (var Gt = 0; Gt < this.contourLevels[$t].length; ++Gt) this._contourCounts[$t][Gt] && (Xt.uniforms.height = this.contourLevels[$t][Gt], Nt.draw(St.LINES, this._contourCounts[$t][Gt], this._contourOffsets[$t][Gt]));
                    }
                    Nt.unbind();
                  }
                }, Ge.pick = function(yt) {
                  if (!yt || yt.id !== this.pickId) return null;
                  var St = this._field[2].shape, Lt = this._pickResult, Rt = St[0] * (yt.value[0] + (yt.value[2] >> 4) / 16) / 255, vt = Math.floor(Rt), $t = Rt - vt, Pt = St[1] * (yt.value[1] + (15 & yt.value[2]) / 16) / 255, Xt = Math.floor(Pt), Nt = Pt - Xt;
                  vt += 1, Xt += 1;
                  var Gt = Lt.position;
                  Gt[0] = Gt[1] = Gt[2] = 0;
                  for (var Ft = 0; Ft < 2; ++Ft) for (var Tt = Ft ? $t : 1 - $t, It = 0; It < 2; ++It) for (var Ct = vt + Ft, Kt = Xt + It, Vt = Tt * (It ? Nt : 1 - Nt), Yt = 0; Yt < 3; ++Yt) Gt[Yt] += this._field[Yt].get(Ct, Kt) * Vt;
                  for (var sn = this._pickResult.level, hn = 0; hn < 3; ++hn) if (sn[hn] = Ee.le(this.contourLevels[hn], Gt[hn]), sn[hn] < 0) this.contourLevels[hn].length > 0 && (sn[hn] = 0);
                  else if (sn[hn] < this.contourLevels[hn].length - 1) {
                    var Qt = this.contourLevels[hn][sn[hn]], Ut = this.contourLevels[hn][sn[hn] + 1];
                    Math.abs(Qt - Gt[hn]) > Math.abs(Ut - Gt[hn]) && (sn[hn] += 1);
                  }
                  for (Lt.index[0] = $t < 0.5 ? vt : vt + 1, Lt.index[1] = Nt < 0.5 ? Xt : Xt + 1, Lt.uv[0] = Rt / St[0], Lt.uv[1] = Pt / St[1], Yt = 0; Yt < 3; ++Yt) Lt.dataCoordinate[Yt] = this._field[Yt].get(Lt.index[0], Lt.index[1]);
                  return Lt;
                }, Ge.padField = function(yt, St) {
                  var Lt = St.shape.slice(), Rt = yt.shape.slice();
                  ge.assign(yt.lo(1, 1).hi(Lt[0], Lt[1]), St), ge.assign(yt.lo(1).hi(Lt[0], 1), St.hi(Lt[0], 1)), ge.assign(yt.lo(1, Rt[1] - 1).hi(Lt[0], 1), St.lo(0, Lt[1] - 1).hi(Lt[0], 1)), ge.assign(yt.lo(0, 1).hi(1, Lt[1]), St.hi(1)), ge.assign(yt.lo(Rt[0] - 1, 1).hi(1, Lt[1]), St.lo(Lt[0] - 1)), yt.set(0, 0, St.get(0, 0)), yt.set(0, Rt[1] - 1, St.get(0, Lt[1] - 1)), yt.set(Rt[0] - 1, 0, St.get(Lt[0] - 1, 0)), yt.set(Rt[0] - 1, Rt[1] - 1, St.get(Lt[0] - 1, Lt[1] - 1));
                }, Ge.update = function(yt) {
                  yt = yt || {}, this.objectOffset = yt.objectOffset || this.objectOffset, this.dirty = !0, "contourWidth" in yt && (this.contourWidth = st(yt.contourWidth, Number)), "showContour" in yt && (this.showContour = st(yt.showContour, Boolean)), "showSurface" in yt && (this.showSurface = !!yt.showSurface), "contourTint" in yt && (this.contourTint = st(yt.contourTint, Boolean)), "contourColor" in yt && (this.contourColor = bt(yt.contourColor)), "contourProject" in yt && (this.contourProject = st(yt.contourProject, function(ri) {
                    return st(ri, Boolean);
                  })), "surfaceProject" in yt && (this.surfaceProject = yt.surfaceProject), "dynamicColor" in yt && (this.dynamicColor = bt(yt.dynamicColor)), "dynamicTint" in yt && (this.dynamicTint = st(yt.dynamicTint, Number)), "dynamicWidth" in yt && (this.dynamicWidth = st(yt.dynamicWidth, Number)), "opacity" in yt && (this.opacity = yt.opacity), "opacityscale" in yt && (this.opacityscale = yt.opacityscale), "colorBounds" in yt && (this.colorBounds = yt.colorBounds), "vertexColor" in yt && (this.vertexColor = yt.vertexColor ? 1 : 0), "colormap" in yt && this._colorMap.setPixels(this.genColormap(yt.colormap, this.opacityscale));
                  var St = yt.field || yt.coords && yt.coords[2] || null, Lt = !1;
                  if (St || (St = this._field[2].shape[0] || this._field[2].shape[2] ? this._field[2].lo(1, 1).hi(this._field[2].shape[0] - 2, this._field[2].shape[1] - 2) : this._field[2].hi(0, 0)), "field" in yt || "coords" in yt) {
                    var Rt = (St.shape[0] + 2) * (St.shape[1] + 2);
                    Rt > this._field[2].data.length && (pe.freeFloat(this._field[2].data), this._field[2].data = pe.mallocFloat(de.nextPow2(Rt))), this._field[2] = be(this._field[2].data, [St.shape[0] + 2, St.shape[1] + 2]), this.padField(this._field[2], St), this.shape = St.shape.slice();
                    for (var vt = this.shape, $t = 0; $t < 2; ++$t) this._field[2].size > this._field[$t].data.length && (pe.freeFloat(this._field[$t].data), this._field[$t].data = pe.mallocFloat(this._field[2].size)), this._field[$t] = be(this._field[$t].data, [vt[0] + 2, vt[1] + 2]);
                    if (yt.coords) {
                      var Pt = yt.coords;
                      if (!Array.isArray(Pt) || Pt.length !== 3) throw new Error("gl-surface: invalid coordinates for x/y");
                      for ($t = 0; $t < 2; ++$t) {
                        var Xt = Pt[$t];
                        for (It = 0; It < 2; ++It) if (Xt.shape[It] !== vt[It]) throw new Error("gl-surface: coords have incorrect shape");
                        this.padField(this._field[$t], Xt);
                      }
                    } else if (yt.ticks) {
                      var Nt = yt.ticks;
                      if (!Array.isArray(Nt) || Nt.length !== 2) throw new Error("gl-surface: invalid ticks");
                      for ($t = 0; $t < 2; ++$t) {
                        var Gt = Nt[$t];
                        if ((Array.isArray(Gt) || Gt.length) && (Gt = be(Gt)), Gt.shape[0] !== vt[$t]) throw new Error("gl-surface: invalid tick length");
                        var Ft = be(Gt.data, vt);
                        Ft.stride[$t] = Gt.stride[0], Ft.stride[1 ^ $t] = 0, this.padField(this._field[$t], Ft);
                      }
                    } else {
                      for ($t = 0; $t < 2; ++$t) {
                        var Tt = [0, 0];
                        Tt[$t] = 1, this._field[$t] = be(this._field[$t].data, [vt[0] + 2, vt[1] + 2], Tt, 0);
                      }
                      this._field[0].set(0, 0, 0);
                      for (var It = 0; It < vt[0]; ++It) this._field[0].set(It + 1, 0, It);
                      for (this._field[0].set(vt[0] + 1, 0, vt[0] - 1), this._field[1].set(0, 0, 0), It = 0; It < vt[1]; ++It) this._field[1].set(0, It + 1, It);
                      this._field[1].set(0, vt[1] + 1, vt[1] - 1);
                    }
                    var Ct = this._field, Kt = be(pe.mallocFloat(3 * Ct[2].size * 2), [3, vt[0] + 2, vt[1] + 2, 2]);
                    for ($t = 0; $t < 3; ++$t) xe(Kt.pick($t), Ct[$t], "mirror");
                    var Vt = be(pe.mallocFloat(3 * Ct[2].size), [vt[0] + 2, vt[1] + 2, 3]);
                    for ($t = 0; $t < vt[0] + 2; ++$t) for (It = 0; It < vt[1] + 2; ++It) {
                      var Yt = Kt.get(0, $t, It, 0), sn = Kt.get(0, $t, It, 1), hn = Kt.get(1, $t, It, 0), Qt = Kt.get(1, $t, It, 1), Ut = Kt.get(2, $t, It, 0), Wt = Kt.get(2, $t, It, 1), rn = hn * Wt - Qt * Ut, vn = Ut * sn - Wt * Yt, dn = Yt * Qt - sn * hn, Ht = Math.sqrt(rn * rn + vn * vn + dn * dn);
                      Ht < 1e-8 ? (Ht = Math.max(Math.abs(rn), Math.abs(vn), Math.abs(dn))) < 1e-8 ? (dn = 1, vn = rn = 0, Ht = 1) : Ht = 1 / Ht : Ht = 1 / Math.sqrt(Ht), Vt.set($t, It, 0, rn * Ht), Vt.set($t, It, 1, vn * Ht), Vt.set($t, It, 2, dn * Ht);
                    }
                    pe.free(Kt.data);
                    var jt = [1 / 0, 1 / 0, 1 / 0], cn = [-1 / 0, -1 / 0, -1 / 0], wn = 1 / 0, Rn = -1 / 0, Pn = (vt[0] - 1) * (vt[1] - 1) * 6, Ie = pe.mallocFloat(de.nextPow2(10 * Pn)), rt = 0, ft = 0;
                    for ($t = 0; $t < vt[0] - 1; ++$t) e: for (It = 0; It < vt[1] - 1; ++It) {
                      for (var pt = 0; pt < 2; ++pt) for (var wt = 0; wt < 2; ++wt) for (var Bt = 0; Bt < 3; ++Bt) {
                        var At = this._field[Bt].get(1 + $t + pt, 1 + It + wt);
                        if (isNaN(At) || !isFinite(At)) continue e;
                      }
                      for (Bt = 0; Bt < 6; ++Bt) {
                        var Dt = $t + $e[Bt][0], Ot = It + $e[Bt][1], un = this._field[0].get(Dt + 1, Ot + 1), yn = this._field[1].get(Dt + 1, Ot + 1);
                        At = this._field[2].get(Dt + 1, Ot + 1), rn = Vt.get(Dt + 1, Ot + 1, 0), vn = Vt.get(Dt + 1, Ot + 1, 1), dn = Vt.get(Dt + 1, Ot + 1, 2), yt.intensity && (_n = yt.intensity.get(Dt, Ot));
                        var _n = yt.intensity ? yt.intensity.get(Dt, Ot) : At + this.objectOffset[2];
                        Ie[rt++] = Dt, Ie[rt++] = Ot, Ie[rt++] = un, Ie[rt++] = yn, Ie[rt++] = At, Ie[rt++] = 0, Ie[rt++] = _n, Ie[rt++] = rn, Ie[rt++] = vn, Ie[rt++] = dn, jt[0] = Math.min(jt[0], un + this.objectOffset[0]), jt[1] = Math.min(jt[1], yn + this.objectOffset[1]), jt[2] = Math.min(jt[2], At + this.objectOffset[2]), wn = Math.min(wn, _n), cn[0] = Math.max(cn[0], un + this.objectOffset[0]), cn[1] = Math.max(cn[1], yn + this.objectOffset[1]), cn[2] = Math.max(cn[2], At + this.objectOffset[2]), Rn = Math.max(Rn, _n), ft += 1;
                      }
                    }
                    for (yt.intensityBounds && (wn = +yt.intensityBounds[0], Rn = +yt.intensityBounds[1]), $t = 6; $t < rt; $t += 10) Ie[$t] = (Ie[$t] - wn) / (Rn - wn);
                    this._vertexCount = ft, this._coordinateBuffer.update(Ie.subarray(0, rt)), pe.freeFloat(Ie), pe.free(Vt.data), this.bounds = [jt, cn], this.intensity = yt.intensity || this._field[2], this.intensityBounds[0] === wn && this.intensityBounds[1] === Rn || (Lt = !0), this.intensityBounds = [wn, Rn];
                  }
                  if ("levels" in yt) {
                    var gn = yt.levels;
                    for (gn = Array.isArray(gn[0]) ? gn.slice() : [[], [], gn], $t = 0; $t < 3; ++$t) gn[$t] = gn[$t].slice(), gn[$t].sort(function(ri, bi) {
                      return ri - bi;
                    });
                    for ($t = 0; $t < 3; ++$t) for (It = 0; It < gn[$t].length; ++It) gn[$t][It] -= this.objectOffset[$t];
                    e: for ($t = 0; $t < 3; ++$t) {
                      if (gn[$t].length !== this.contourLevels[$t].length) {
                        Lt = !0;
                        break;
                      }
                      for (It = 0; It < gn[$t].length; ++It) if (gn[$t][It] !== this.contourLevels[$t][It]) {
                        Lt = !0;
                        break e;
                      }
                    }
                    this.contourLevels = gn;
                  }
                  if (Lt) {
                    Ct = this._field, vt = this.shape;
                    for (var bn = [], an = 0; an < 3; ++an) {
                      var Zt = this.contourLevels[an], Mn = [], Ln = [], In = [0, 0, 0];
                      for ($t = 0; $t < Zt.length; ++$t) {
                        var kn = Ce(this._field[an], Zt[$t]);
                        Mn.push(bn.length / 5 | 0), ft = 0;
                        e: for (It = 0; It < kn.cells.length; ++It) {
                          var er = kn.cells[It];
                          for (Bt = 0; Bt < 2; ++Bt) {
                            var jn = kn.positions[er[Bt]], ur = jn[0], Kn = 0 | Math.floor(ur), Jn = ur - Kn, Lr = jn[1], mr = 0 | Math.floor(Lr), Ar = Lr - mr, Yr = !1;
                            t: for (var jr = 0; jr < 3; ++jr) {
                              In[jr] = 0;
                              var Mr = (an + jr + 1) % 3;
                              for (pt = 0; pt < 2; ++pt) {
                                var Wr = pt ? Jn : 1 - Jn;
                                for (Dt = 0 | Math.min(Math.max(Kn + pt, 0), vt[0]), wt = 0; wt < 2; ++wt) {
                                  var Bi = wt ? Ar : 1 - Ar;
                                  if (Ot = 0 | Math.min(Math.max(mr + wt, 0), vt[1]), At = jr < 2 ? this._field[Mr].get(Dt, Ot) : (this.intensity.get(Dt, Ot) - this.intensityBounds[0]) / (this.intensityBounds[1] - this.intensityBounds[0]), !isFinite(At) || isNaN(At)) {
                                    Yr = !0;
                                    break t;
                                  }
                                  var gi = Wr * Bi;
                                  In[jr] += gi * At;
                                }
                              }
                            }
                            if (Yr) {
                              if (Bt > 0) {
                                for (var Zr = 0; Zr < 5; ++Zr) bn.pop();
                                ft -= 1;
                              }
                              continue e;
                            }
                            bn.push(In[0], In[1], jn[0], jn[1], In[2]), ft += 1;
                          }
                        }
                        Ln.push(ft);
                      }
                      this._contourOffsets[an] = Mn, this._contourCounts[an] = Ln;
                    }
                    var fi = pe.mallocFloat(bn.length);
                    for ($t = 0; $t < bn.length; ++$t) fi[$t] = bn[$t];
                    this._contourBuffer.update(fi), pe.freeFloat(fi);
                  }
                }, Ge.dispose = function() {
                  this._shader.dispose(), this._vao.dispose(), this._coordinateBuffer.dispose(), this._colorMap.dispose(), this._contourBuffer.dispose(), this._contourVAO.dispose(), this._contourShader.dispose(), this._contourPickShader.dispose(), this._dynamicBuffer.dispose(), this._dynamicVAO.dispose();
                  for (var yt = 0; yt < 3; ++yt) pe.freeFloat(this._field[yt].data);
                }, Ge.highlight = function(yt) {
                  var St, Lt;
                  if (!yt) return this._dynamicCounts = [0, 0, 0], this.dyanamicLevel = [NaN, NaN, NaN], void (this.highlightLevel = [-1, -1, -1]);
                  for (St = 0; St < 3; ++St) this.enableHighlight[St] ? this.highlightLevel[St] = yt.level[St] : this.highlightLevel[St] = -1;
                  for (Lt = this.snapToData ? yt.dataCoordinate : yt.position, St = 0; St < 3; ++St) Lt[St] -= this.objectOffset[St];
                  if (this.enableDynamic[0] && Lt[0] !== this.dynamicLevel[0] || this.enableDynamic[1] && Lt[1] !== this.dynamicLevel[1] || this.enableDynamic[2] && Lt[2] !== this.dynamicLevel[2]) {
                    for (var Rt = 0, vt = this.shape, $t = pe.mallocFloat(12 * vt[0] * vt[1]), Pt = 0; Pt < 3; ++Pt) if (this.enableDynamic[Pt]) {
                      this.dynamicLevel[Pt] = Lt[Pt];
                      var Xt = (Pt + 1) % 3, Nt = (Pt + 2) % 3, Gt = this._field[Pt], Ft = this._field[Xt], Tt = this._field[Nt], It = Ce(Gt, Lt[Pt]), Ct = It.cells, Kt = It.positions;
                      for (this._dynamicOffsets[Pt] = Rt, St = 0; St < Ct.length; ++St) for (var Vt = Ct[St], Yt = 0; Yt < 2; ++Yt) {
                        var sn = Kt[Vt[Yt]], hn = +sn[0], Qt = 0 | hn, Ut = 0 | Math.min(Qt + 1, vt[0]), Wt = hn - Qt, rn = 1 - Wt, vn = +sn[1], dn = 0 | vn, Ht = 0 | Math.min(dn + 1, vt[1]), jt = vn - dn, cn = 1 - jt, wn = rn * cn, Rn = rn * jt, Pn = Wt * cn, Ie = Wt * jt, rt = wn * Ft.get(Qt, dn) + Rn * Ft.get(Qt, Ht) + Pn * Ft.get(Ut, dn) + Ie * Ft.get(Ut, Ht), ft = wn * Tt.get(Qt, dn) + Rn * Tt.get(Qt, Ht) + Pn * Tt.get(Ut, dn) + Ie * Tt.get(Ut, Ht);
                        if (isNaN(rt) || isNaN(ft)) {
                          Yt && (Rt -= 1);
                          break;
                        }
                        $t[2 * Rt + 0] = rt, $t[2 * Rt + 1] = ft, Rt += 1;
                      }
                      this._dynamicCounts[Pt] = Rt - this._dynamicOffsets[Pt];
                    } else this.dynamicLevel[Pt] = NaN, this._dynamicCounts[Pt] = 0;
                    this._dynamicBuffer.update($t.subarray(0, 2 * Rt)), pe.freeFloat($t);
                  }
                };
              }, 7766: function(ce, le, ue) {
                var de = ue(9618), fe = ue(5298), he = ue(1888);
                ce.exports = function(Le) {
                  if (arguments.length <= 1) throw new Error("gl-texture2d: Missing arguments for texture2d constructor");
                  if (me || function(Me) {
                    me = [Me.LINEAR, Me.NEAREST_MIPMAP_LINEAR, Me.LINEAR_MIPMAP_NEAREST, Me.LINEAR_MIPMAP_NEAREST], pe = [Me.NEAREST, Me.LINEAR, Me.NEAREST_MIPMAP_NEAREST, Me.NEAREST_MIPMAP_LINEAR, Me.LINEAR_MIPMAP_NEAREST, Me.LINEAR_MIPMAP_LINEAR], ye = [Me.REPEAT, Me.CLAMP_TO_EDGE, Me.MIRRORED_REPEAT];
                  }(Le), typeof arguments[1] == "number") return xe(Le, arguments[1], arguments[2], arguments[3] || Le.RGBA, arguments[4] || Le.UNSIGNED_BYTE);
                  if (Array.isArray(arguments[1])) return xe(Le, 0 | arguments[1][0], 0 | arguments[1][1], arguments[2] || Le.RGBA, arguments[3] || Le.UNSIGNED_BYTE);
                  if (typeof arguments[1] == "object") {
                    var Qe = arguments[1], ke = ge(Qe) ? Qe : Qe.raw;
                    if (ke) return function(Me, Re, De, Pe, $e, qe) {
                      var Xe = Ee(Me);
                      return Me.texImage2D(Me.TEXTURE_2D, 0, $e, $e, qe, Re), new Ce(Me, Xe, De, Pe, $e, qe);
                    }(Le, ke, 0 | Qe.width, 0 | Qe.height, arguments[2] || Le.RGBA, arguments[3] || Le.UNSIGNED_BYTE);
                    if (Qe.shape && Qe.data && Qe.stride) return function(Me, Re) {
                      var De = Re.dtype, Pe = Re.shape.slice(), $e = Me.getParameter(Me.MAX_TEXTURE_SIZE);
                      if (Pe[0] < 0 || Pe[0] > $e || Pe[1] < 0 || Pe[1] > $e) throw new Error("gl-texture2d: Invalid texture size");
                      var qe = we(Pe, Re.stride.slice()), Xe = 0;
                      De === "float32" ? Xe = Me.FLOAT : De === "float64" ? (Xe = Me.FLOAT, qe = !1, De = "float32") : De === "uint8" ? Xe = Me.UNSIGNED_BYTE : (Xe = Me.UNSIGNED_BYTE, qe = !1, De = "uint8");
                      var Ve, Ze, Ge = 0;
                      if (Pe.length === 2) Ge = Me.LUMINANCE, Pe = [Pe[0], Pe[1], 1], Re = de(Re.data, Pe, [Re.stride[0], Re.stride[1], 1], Re.offset);
                      else {
                        if (Pe.length !== 3) throw new Error("gl-texture2d: Invalid shape for texture");
                        if (Pe[2] === 1) Ge = Me.ALPHA;
                        else if (Pe[2] === 2) Ge = Me.LUMINANCE_ALPHA;
                        else if (Pe[2] === 3) Ge = Me.RGB;
                        else {
                          if (Pe[2] !== 4) throw new Error("gl-texture2d: Invalid shape for pixel coords");
                          Ge = Me.RGBA;
                        }
                      }
                      Xe !== Me.FLOAT || Me.getExtension("OES_texture_float") || (Xe = Me.UNSIGNED_BYTE, qe = !1);
                      var ze = Re.size;
                      if (qe) Ve = Re.offset === 0 && Re.data.length === ze ? Re.data : Re.data.subarray(Re.offset, Re.offset + ze);
                      else {
                        var We = [Pe[2], Pe[2] * Pe[0], 1];
                        Ze = he.malloc(ze, De);
                        var at = de(Ze, Pe, We, 0);
                        De !== "float32" && De !== "float64" || Xe !== Me.UNSIGNED_BYTE ? fe.assign(at, Re) : ve(at, Re), Ve = Ze.subarray(0, ze);
                      }
                      var ct = Ee(Me);
                      return Me.texImage2D(Me.TEXTURE_2D, 0, Ge, Pe[0], Pe[1], 0, Ge, Xe, Ve), qe || he.free(Ze), new Ce(Me, ct, Pe[0], Pe[1], Ge, Xe);
                    }(Le, Qe);
                  }
                  throw new Error("gl-texture2d: Invalid arguments for texture2d constructor");
                };
                var me = null, pe = null, ye = null;
                function ge(Le) {
                  return typeof HTMLCanvasElement < "u" && Le instanceof HTMLCanvasElement || typeof HTMLImageElement < "u" && Le instanceof HTMLImageElement || typeof HTMLVideoElement < "u" && Le instanceof HTMLVideoElement || typeof ImageData < "u" && Le instanceof ImageData;
                }
                var ve = function(Le, Qe) {
                  fe.muls(Le, Qe, 255);
                };
                function be(Le, Qe, ke) {
                  var Me = Le.gl, Re = Me.getParameter(Me.MAX_TEXTURE_SIZE);
                  if (Qe < 0 || Qe > Re || ke < 0 || ke > Re) throw new Error("gl-texture2d: Invalid texture size");
                  return Le._shape = [Qe, ke], Le.bind(), Me.texImage2D(Me.TEXTURE_2D, 0, Le.format, Qe, ke, 0, Le.format, Le.type, null), Le._mipLevels = [0], Le;
                }
                function Ce(Le, Qe, ke, Me, Re, De) {
                  this.gl = Le, this.handle = Qe, this.format = Re, this.type = De, this._shape = [ke, Me], this._mipLevels = [0], this._magFilter = Le.NEAREST, this._minFilter = Le.NEAREST, this._wrapS = Le.CLAMP_TO_EDGE, this._wrapT = Le.CLAMP_TO_EDGE, this._anisoSamples = 1;
                  var Pe = this, $e = [this._wrapS, this._wrapT];
                  Object.defineProperties($e, [{ get: function() {
                    return Pe._wrapS;
                  }, set: function(Xe) {
                    return Pe.wrapS = Xe;
                  } }, { get: function() {
                    return Pe._wrapT;
                  }, set: function(Xe) {
                    return Pe.wrapT = Xe;
                  } }]), this._wrapVector = $e;
                  var qe = [this._shape[0], this._shape[1]];
                  Object.defineProperties(qe, [{ get: function() {
                    return Pe._shape[0];
                  }, set: function(Xe) {
                    return Pe.width = Xe;
                  } }, { get: function() {
                    return Pe._shape[1];
                  }, set: function(Xe) {
                    return Pe.height = Xe;
                  } }]), this._shapeVector = qe;
                }
                var Se = Ce.prototype;
                function we(Le, Qe) {
                  return Le.length === 3 ? Qe[2] === 1 && Qe[1] === Le[0] * Le[2] && Qe[0] === Le[2] : Qe[0] === 1 && Qe[1] === Le[0];
                }
                function Ee(Le) {
                  var Qe = Le.createTexture();
                  return Le.bindTexture(Le.TEXTURE_2D, Qe), Le.texParameteri(Le.TEXTURE_2D, Le.TEXTURE_MIN_FILTER, Le.NEAREST), Le.texParameteri(Le.TEXTURE_2D, Le.TEXTURE_MAG_FILTER, Le.NEAREST), Le.texParameteri(Le.TEXTURE_2D, Le.TEXTURE_WRAP_S, Le.CLAMP_TO_EDGE), Le.texParameteri(Le.TEXTURE_2D, Le.TEXTURE_WRAP_T, Le.CLAMP_TO_EDGE), Qe;
                }
                function xe(Le, Qe, ke, Me, Re) {
                  var De = Le.getParameter(Le.MAX_TEXTURE_SIZE);
                  if (Qe < 0 || Qe > De || ke < 0 || ke > De) throw new Error("gl-texture2d: Invalid texture shape");
                  if (Re === Le.FLOAT && !Le.getExtension("OES_texture_float")) throw new Error("gl-texture2d: Floating point textures not supported on this platform");
                  var Pe = Ee(Le);
                  return Le.texImage2D(Le.TEXTURE_2D, 0, Me, Qe, ke, 0, Me, Re, null), new Ce(Le, Pe, Qe, ke, Me, Re);
                }
                Object.defineProperties(Se, { minFilter: { get: function() {
                  return this._minFilter;
                }, set: function(Le) {
                  this.bind();
                  var Qe = this.gl;
                  if (this.type === Qe.FLOAT && me.indexOf(Le) >= 0 && (Qe.getExtension("OES_texture_float_linear") || (Le = Qe.NEAREST)), pe.indexOf(Le) < 0) throw new Error("gl-texture2d: Unknown filter mode " + Le);
                  return Qe.texParameteri(Qe.TEXTURE_2D, Qe.TEXTURE_MIN_FILTER, Le), this._minFilter = Le;
                } }, magFilter: { get: function() {
                  return this._magFilter;
                }, set: function(Le) {
                  this.bind();
                  var Qe = this.gl;
                  if (this.type === Qe.FLOAT && me.indexOf(Le) >= 0 && (Qe.getExtension("OES_texture_float_linear") || (Le = Qe.NEAREST)), pe.indexOf(Le) < 0) throw new Error("gl-texture2d: Unknown filter mode " + Le);
                  return Qe.texParameteri(Qe.TEXTURE_2D, Qe.TEXTURE_MAG_FILTER, Le), this._magFilter = Le;
                } }, mipSamples: { get: function() {
                  return this._anisoSamples;
                }, set: function(Le) {
                  var Qe = this._anisoSamples;
                  if (this._anisoSamples = 0 | Math.max(Le, 1), Qe !== this._anisoSamples) {
                    var ke = this.gl.getExtension("EXT_texture_filter_anisotropic");
                    ke && this.gl.texParameterf(this.gl.TEXTURE_2D, ke.TEXTURE_MAX_ANISOTROPY_EXT, this._anisoSamples);
                  }
                  return this._anisoSamples;
                } }, wrapS: { get: function() {
                  return this._wrapS;
                }, set: function(Le) {
                  if (this.bind(), ye.indexOf(Le) < 0) throw new Error("gl-texture2d: Unknown wrap mode " + Le);
                  return this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_S, Le), this._wrapS = Le;
                } }, wrapT: { get: function() {
                  return this._wrapT;
                }, set: function(Le) {
                  if (this.bind(), ye.indexOf(Le) < 0) throw new Error("gl-texture2d: Unknown wrap mode " + Le);
                  return this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_T, Le), this._wrapT = Le;
                } }, wrap: { get: function() {
                  return this._wrapVector;
                }, set: function(Le) {
                  if (Array.isArray(Le) || (Le = [Le, Le]), Le.length !== 2) throw new Error("gl-texture2d: Must specify wrap mode for rows and columns");
                  for (var Qe = 0; Qe < 2; ++Qe) if (ye.indexOf(Le[Qe]) < 0) throw new Error("gl-texture2d: Unknown wrap mode " + Le);
                  this._wrapS = Le[0], this._wrapT = Le[1];
                  var ke = this.gl;
                  return this.bind(), ke.texParameteri(ke.TEXTURE_2D, ke.TEXTURE_WRAP_S, this._wrapS), ke.texParameteri(ke.TEXTURE_2D, ke.TEXTURE_WRAP_T, this._wrapT), Le;
                } }, shape: { get: function() {
                  return this._shapeVector;
                }, set: function(Le) {
                  if (Array.isArray(Le)) {
                    if (Le.length !== 2) throw new Error("gl-texture2d: Invalid texture shape");
                  } else Le = [0 | Le, 0 | Le];
                  return be(this, 0 | Le[0], 0 | Le[1]), [0 | Le[0], 0 | Le[1]];
                } }, width: { get: function() {
                  return this._shape[0];
                }, set: function(Le) {
                  return be(this, Le |= 0, this._shape[1]), Le;
                } }, height: { get: function() {
                  return this._shape[1];
                }, set: function(Le) {
                  return Le |= 0, be(this, this._shape[0], Le), Le;
                } } }), Se.bind = function(Le) {
                  var Qe = this.gl;
                  return Le !== void 0 && Qe.activeTexture(Qe.TEXTURE0 + (0 | Le)), Qe.bindTexture(Qe.TEXTURE_2D, this.handle), Le !== void 0 ? 0 | Le : Qe.getParameter(Qe.ACTIVE_TEXTURE) - Qe.TEXTURE0;
                }, Se.dispose = function() {
                  this.gl.deleteTexture(this.handle);
                }, Se.generateMipmap = function() {
                  this.bind(), this.gl.generateMipmap(this.gl.TEXTURE_2D);
                  for (var Le = Math.min(this._shape[0], this._shape[1]), Qe = 0; Le > 0; ++Qe, Le >>>= 1) this._mipLevels.indexOf(Qe) < 0 && this._mipLevels.push(Qe);
                }, Se.setPixels = function(Le, Qe, ke, Me) {
                  var Re = this.gl;
                  this.bind(), Array.isArray(Qe) ? (Me = ke, ke = 0 | Qe[1], Qe = 0 | Qe[0]) : (Qe = Qe || 0, ke = ke || 0), Me = Me || 0;
                  var De = ge(Le) ? Le : Le.raw;
                  if (De) this._mipLevels.indexOf(Me) < 0 ? (Re.texImage2D(Re.TEXTURE_2D, 0, this.format, this.format, this.type, De), this._mipLevels.push(Me)) : Re.texSubImage2D(Re.TEXTURE_2D, Me, Qe, ke, this.format, this.type, De);
                  else {
                    if (!(Le.shape && Le.stride && Le.data)) throw new Error("gl-texture2d: Unsupported data type");
                    if (Le.shape.length < 2 || Qe + Le.shape[1] > this._shape[1] >>> Me || ke + Le.shape[0] > this._shape[0] >>> Me || Qe < 0 || ke < 0) throw new Error("gl-texture2d: Texture dimensions are out of bounds");
                    (function(Pe, $e, qe, Xe, Ve, Ze, Ge, ze) {
                      var We = ze.dtype, at = ze.shape.slice();
                      if (at.length < 2 || at.length > 3) throw new Error("gl-texture2d: Invalid ndarray, must be 2d or 3d");
                      var ct = 0, et = 0, dt = we(at, ze.stride.slice());
                      if (We === "float32" ? ct = Pe.FLOAT : We === "float64" ? (ct = Pe.FLOAT, dt = !1, We = "float32") : We === "uint8" ? ct = Pe.UNSIGNED_BYTE : (ct = Pe.UNSIGNED_BYTE, dt = !1, We = "uint8"), at.length === 2) et = Pe.LUMINANCE, at = [at[0], at[1], 1], ze = de(ze.data, at, [ze.stride[0], ze.stride[1], 1], ze.offset);
                      else {
                        if (at.length !== 3) throw new Error("gl-texture2d: Invalid shape for texture");
                        if (at[2] === 1) et = Pe.ALPHA;
                        else if (at[2] === 2) et = Pe.LUMINANCE_ALPHA;
                        else if (at[2] === 3) et = Pe.RGB;
                        else {
                          if (at[2] !== 4) throw new Error("gl-texture2d: Invalid shape for pixel coords");
                          et = Pe.RGBA;
                        }
                        at[2];
                      }
                      if (et !== Pe.LUMINANCE && et !== Pe.ALPHA || Ve !== Pe.LUMINANCE && Ve !== Pe.ALPHA || (et = Ve), et !== Ve) throw new Error("gl-texture2d: Incompatible texture format for setPixels");
                      var Je = ze.size, it = Ge.indexOf(Xe) < 0;
                      if (it && Ge.push(Xe), ct === Ze && dt) ze.offset === 0 && ze.data.length === Je ? it ? Pe.texImage2D(Pe.TEXTURE_2D, Xe, Ve, at[0], at[1], 0, Ve, Ze, ze.data) : Pe.texSubImage2D(Pe.TEXTURE_2D, Xe, $e, qe, at[0], at[1], Ve, Ze, ze.data) : it ? Pe.texImage2D(Pe.TEXTURE_2D, Xe, Ve, at[0], at[1], 0, Ve, Ze, ze.data.subarray(ze.offset, ze.offset + Je)) : Pe.texSubImage2D(Pe.TEXTURE_2D, Xe, $e, qe, at[0], at[1], Ve, Ze, ze.data.subarray(ze.offset, ze.offset + Je));
                      else {
                        var st;
                        st = Ze === Pe.FLOAT ? he.mallocFloat32(Je) : he.mallocUint8(Je);
                        var gt = de(st, at, [at[2], at[2] * at[0], 1]);
                        ct === Pe.FLOAT && Ze === Pe.UNSIGNED_BYTE ? ve(gt, ze) : fe.assign(gt, ze), it ? Pe.texImage2D(Pe.TEXTURE_2D, Xe, Ve, at[0], at[1], 0, Ve, Ze, st.subarray(0, Je)) : Pe.texSubImage2D(Pe.TEXTURE_2D, Xe, $e, qe, at[0], at[1], Ve, Ze, st.subarray(0, Je)), Ze === Pe.FLOAT ? he.freeFloat32(st) : he.freeUint8(st);
                      }
                    })(Re, Qe, ke, Me, this.format, this.type, this._mipLevels, Le);
                  }
                };
              }, 1433: function(ce) {
                ce.exports = function(le, ue, de) {
                  ue ? ue.bind() : le.bindBuffer(le.ELEMENT_ARRAY_BUFFER, null);
                  var fe = 0 | le.getParameter(le.MAX_VERTEX_ATTRIBS);
                  if (de) {
                    if (de.length > fe) throw new Error("gl-vao: Too many vertex attributes");
                    for (var he = 0; he < de.length; ++he) {
                      var me = de[he];
                      if (me.buffer) {
                        var pe = me.buffer, ye = me.size || 4, ge = me.type || le.FLOAT, ve = !!me.normalized, be = me.stride || 0, Ce = me.offset || 0;
                        pe.bind(), le.enableVertexAttribArray(he), le.vertexAttribPointer(he, ye, ge, ve, be, Ce);
                      } else {
                        if (typeof me == "number") le.vertexAttrib1f(he, me);
                        else if (me.length === 1) le.vertexAttrib1f(he, me[0]);
                        else if (me.length === 2) le.vertexAttrib2f(he, me[0], me[1]);
                        else if (me.length === 3) le.vertexAttrib3f(he, me[0], me[1], me[2]);
                        else {
                          if (me.length !== 4) throw new Error("gl-vao: Invalid vertex attribute");
                          le.vertexAttrib4f(he, me[0], me[1], me[2], me[3]);
                        }
                        le.disableVertexAttribArray(he);
                      }
                    }
                    for (; he < fe; ++he) le.disableVertexAttribArray(he);
                  } else for (le.bindBuffer(le.ARRAY_BUFFER, null), he = 0; he < fe; ++he) le.disableVertexAttribArray(he);
                };
              }, 870: function(ce, le, ue) {
                var de = ue(1433);
                function fe(he) {
                  this.gl = he, this._elements = null, this._attributes = null, this._elementsType = he.UNSIGNED_SHORT;
                }
                fe.prototype.bind = function() {
                  de(this.gl, this._elements, this._attributes);
                }, fe.prototype.update = function(he, me, pe) {
                  this._elements = me, this._attributes = he, this._elementsType = pe || this.gl.UNSIGNED_SHORT;
                }, fe.prototype.dispose = function() {
                }, fe.prototype.unbind = function() {
                }, fe.prototype.draw = function(he, me, pe) {
                  pe = pe || 0;
                  var ye = this.gl;
                  this._elements ? ye.drawElements(he, me, this._elementsType, pe) : ye.drawArrays(he, pe, me);
                }, ce.exports = function(he) {
                  return new fe(he);
                };
              }, 7518: function(ce, le, ue) {
                var de = ue(1433);
                function fe(me, pe, ye, ge, ve, be) {
                  this.location = me, this.dimension = pe, this.a = ye, this.b = ge, this.c = ve, this.d = be;
                }
                function he(me, pe, ye) {
                  this.gl = me, this._ext = pe, this.handle = ye, this._attribs = [], this._useElements = !1, this._elementsType = me.UNSIGNED_SHORT;
                }
                fe.prototype.bind = function(me) {
                  switch (this.dimension) {
                    case 1:
                      me.vertexAttrib1f(this.location, this.a);
                      break;
                    case 2:
                      me.vertexAttrib2f(this.location, this.a, this.b);
                      break;
                    case 3:
                      me.vertexAttrib3f(this.location, this.a, this.b, this.c);
                      break;
                    case 4:
                      me.vertexAttrib4f(this.location, this.a, this.b, this.c, this.d);
                  }
                }, he.prototype.bind = function() {
                  this._ext.bindVertexArrayOES(this.handle);
                  for (var me = 0; me < this._attribs.length; ++me) this._attribs[me].bind(this.gl);
                }, he.prototype.unbind = function() {
                  this._ext.bindVertexArrayOES(null);
                }, he.prototype.dispose = function() {
                  this._ext.deleteVertexArrayOES(this.handle);
                }, he.prototype.update = function(me, pe, ye) {
                  if (this.bind(), de(this.gl, pe, me), this.unbind(), this._attribs.length = 0, me) for (var ge = 0; ge < me.length; ++ge) {
                    var ve = me[ge];
                    typeof ve == "number" ? this._attribs.push(new fe(ge, 1, ve)) : Array.isArray(ve) && this._attribs.push(new fe(ge, ve.length, ve[0], ve[1], ve[2], ve[3]));
                  }
                  this._useElements = !!pe, this._elementsType = ye || this.gl.UNSIGNED_SHORT;
                }, he.prototype.draw = function(me, pe, ye) {
                  ye = ye || 0;
                  var ge = this.gl;
                  this._useElements ? ge.drawElements(me, pe, this._elementsType, ye) : ge.drawArrays(me, ye, pe);
                }, ce.exports = function(me, pe) {
                  return new he(me, pe, pe.createVertexArrayOES());
                };
              }, 8116: function(ce, le, ue) {
                var de = ue(7518), fe = ue(870);
                function he(me) {
                  this.bindVertexArrayOES = me.bindVertexArray.bind(me), this.createVertexArrayOES = me.createVertexArray.bind(me), this.deleteVertexArrayOES = me.deleteVertexArray.bind(me);
                }
                ce.exports = function(me, pe, ye, ge) {
                  var ve, be = me.createVertexArray ? new he(me) : me.getExtension("OES_vertex_array_object");
                  return (ve = be ? de(me, be) : fe(me)).update(pe, ye, ge), ve;
                };
              }, 5632: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] + de[0], le[1] = ue[1] + de[1], le[2] = ue[2] + de[2], le;
                };
              }, 8192: function(ce, le, ue) {
                ce.exports = function(me, pe) {
                  var ye = de(me[0], me[1], me[2]), ge = de(pe[0], pe[1], pe[2]);
                  fe(ye, ye), fe(ge, ge);
                  var ve = he(ye, ge);
                  return ve > 1 ? 0 : Math.acos(ve);
                };
                var de = ue(2825), fe = ue(3536), he = ue(244);
              }, 9226: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = Math.ceil(ue[0]), le[1] = Math.ceil(ue[1]), le[2] = Math.ceil(ue[2]), le;
                };
              }, 3126: function(ce) {
                ce.exports = function(le) {
                  var ue = new Float32Array(3);
                  return ue[0] = le[0], ue[1] = le[1], ue[2] = le[2], ue;
                };
              }, 3990: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = ue[0], le[1] = ue[1], le[2] = ue[2], le;
                };
              }, 1091: function(ce) {
                ce.exports = function() {
                  var le = new Float32Array(3);
                  return le[0] = 0, le[1] = 0, le[2] = 0, le;
                };
              }, 5911: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2], pe = de[0], ye = de[1], ge = de[2];
                  return le[0] = he * ge - me * ye, le[1] = me * pe - fe * ge, le[2] = fe * ye - he * pe, le;
                };
              }, 5455: function(ce, le, ue) {
                ce.exports = ue(7056);
              }, 7056: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0] - le[0], fe = ue[1] - le[1], he = ue[2] - le[2];
                  return Math.sqrt(de * de + fe * fe + he * he);
                };
              }, 4008: function(ce, le, ue) {
                ce.exports = ue(6690);
              }, 6690: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] / de[0], le[1] = ue[1] / de[1], le[2] = ue[2] / de[2], le;
                };
              }, 244: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] * ue[0] + le[1] * ue[1] + le[2] * ue[2];
                };
              }, 2613: function(ce) {
                ce.exports = 1e-6;
              }, 9922: function(ce, le, ue) {
                ce.exports = function(fe, he) {
                  var me = fe[0], pe = fe[1], ye = fe[2], ge = he[0], ve = he[1], be = he[2];
                  return Math.abs(me - ge) <= de * Math.max(1, Math.abs(me), Math.abs(ge)) && Math.abs(pe - ve) <= de * Math.max(1, Math.abs(pe), Math.abs(ve)) && Math.abs(ye - be) <= de * Math.max(1, Math.abs(ye), Math.abs(be));
                };
                var de = ue(2613);
              }, 9265: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] === ue[0] && le[1] === ue[1] && le[2] === ue[2];
                };
              }, 2681: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = Math.floor(ue[0]), le[1] = Math.floor(ue[1]), le[2] = Math.floor(ue[2]), le;
                };
              }, 5137: function(ce, le, ue) {
                ce.exports = function(fe, he, me, pe, ye, ge) {
                  var ve, be;
                  for (he || (he = 3), me || (me = 0), be = pe ? Math.min(pe * he + me, fe.length) : fe.length, ve = me; ve < be; ve += he) de[0] = fe[ve], de[1] = fe[ve + 1], de[2] = fe[ve + 2], ye(de, de, ge), fe[ve] = de[0], fe[ve + 1] = de[1], fe[ve + 2] = de[2];
                  return fe;
                };
                var de = ue(1091)();
              }, 2825: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = new Float32Array(3);
                  return fe[0] = le, fe[1] = ue, fe[2] = de, fe;
                };
              }, 2931: function(ce, le, ue) {
                ce.exports = { EPSILON: ue(2613), create: ue(1091), clone: ue(3126), angle: ue(8192), fromValues: ue(2825), copy: ue(3990), set: ue(1463), equals: ue(9922), exactEquals: ue(9265), add: ue(5632), subtract: ue(6843), sub: ue(2229), multiply: ue(5847), mul: ue(4505), divide: ue(6690), div: ue(4008), min: ue(8107), max: ue(7417), floor: ue(2681), ceil: ue(9226), round: ue(2447), scale: ue(6621), scaleAndAdd: ue(8489), distance: ue(7056), dist: ue(5455), squaredDistance: ue(2953), sqrDist: ue(6141), length: ue(1387), len: ue(868), squaredLength: ue(3066), sqrLen: ue(5486), negate: ue(5093), inverse: ue(811), normalize: ue(3536), dot: ue(244), cross: ue(5911), lerp: ue(6658), random: ue(7636), transformMat4: ue(5673), transformMat3: ue(492), transformQuat: ue(264), rotateX: ue(6894), rotateY: ue(109), rotateZ: ue(8692), forEach: ue(5137) };
              }, 811: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = 1 / ue[0], le[1] = 1 / ue[1], le[2] = 1 / ue[2], le;
                };
              }, 868: function(ce, le, ue) {
                ce.exports = ue(1387);
              }, 1387: function(ce) {
                ce.exports = function(le) {
                  var ue = le[0], de = le[1], fe = le[2];
                  return Math.sqrt(ue * ue + de * de + fe * fe);
                };
              }, 6658: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he = ue[0], me = ue[1], pe = ue[2];
                  return le[0] = he + fe * (de[0] - he), le[1] = me + fe * (de[1] - me), le[2] = pe + fe * (de[2] - pe), le;
                };
              }, 7417: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = Math.max(ue[0], de[0]), le[1] = Math.max(ue[1], de[1]), le[2] = Math.max(ue[2], de[2]), le;
                };
              }, 8107: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = Math.min(ue[0], de[0]), le[1] = Math.min(ue[1], de[1]), le[2] = Math.min(ue[2], de[2]), le;
                };
              }, 4505: function(ce, le, ue) {
                ce.exports = ue(5847);
              }, 5847: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] * de[0], le[1] = ue[1] * de[1], le[2] = ue[2] * de[2], le;
                };
              }, 5093: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = -ue[0], le[1] = -ue[1], le[2] = -ue[2], le;
                };
              }, 3536: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0], fe = ue[1], he = ue[2], me = de * de + fe * fe + he * he;
                  return me > 0 && (me = 1 / Math.sqrt(me), le[0] = ue[0] * me, le[1] = ue[1] * me, le[2] = ue[2] * me), le;
                };
              }, 7636: function(ce) {
                ce.exports = function(le, ue) {
                  ue = ue || 1;
                  var de = 2 * Math.random() * Math.PI, fe = 2 * Math.random() - 1, he = Math.sqrt(1 - fe * fe) * ue;
                  return le[0] = Math.cos(de) * he, le[1] = Math.sin(de) * he, le[2] = fe * ue, le;
                };
              }, 6894: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he = de[1], me = de[2], pe = ue[1] - he, ye = ue[2] - me, ge = Math.sin(fe), ve = Math.cos(fe);
                  return le[0] = ue[0], le[1] = he + pe * ve - ye * ge, le[2] = me + pe * ge + ye * ve, le;
                };
              }, 109: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he = de[0], me = de[2], pe = ue[0] - he, ye = ue[2] - me, ge = Math.sin(fe), ve = Math.cos(fe);
                  return le[0] = he + ye * ge + pe * ve, le[1] = ue[1], le[2] = me + ye * ve - pe * ge, le;
                };
              }, 8692: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he = de[0], me = de[1], pe = ue[0] - he, ye = ue[1] - me, ge = Math.sin(fe), ve = Math.cos(fe);
                  return le[0] = he + pe * ve - ye * ge, le[1] = me + pe * ge + ye * ve, le[2] = ue[2], le;
                };
              }, 2447: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = Math.round(ue[0]), le[1] = Math.round(ue[1]), le[2] = Math.round(ue[2]), le;
                };
              }, 6621: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] * de, le[1] = ue[1] * de, le[2] = ue[2] * de, le;
                };
              }, 8489: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  return le[0] = ue[0] + de[0] * fe, le[1] = ue[1] + de[1] * fe, le[2] = ue[2] + de[2] * fe, le;
                };
              }, 1463: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  return le[0] = ue, le[1] = de, le[2] = fe, le;
                };
              }, 6141: function(ce, le, ue) {
                ce.exports = ue(2953);
              }, 5486: function(ce, le, ue) {
                ce.exports = ue(3066);
              }, 2953: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0] - le[0], fe = ue[1] - le[1], he = ue[2] - le[2];
                  return de * de + fe * fe + he * he;
                };
              }, 3066: function(ce) {
                ce.exports = function(le) {
                  var ue = le[0], de = le[1], fe = le[2];
                  return ue * ue + de * de + fe * fe;
                };
              }, 2229: function(ce, le, ue) {
                ce.exports = ue(6843);
              }, 6843: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] - de[0], le[1] = ue[1] - de[1], le[2] = ue[2] - de[2], le;
                };
              }, 492: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2];
                  return le[0] = fe * de[0] + he * de[3] + me * de[6], le[1] = fe * de[1] + he * de[4] + me * de[7], le[2] = fe * de[2] + he * de[5] + me * de[8], le;
                };
              }, 5673: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2], pe = de[3] * fe + de[7] * he + de[11] * me + de[15];
                  return pe = pe || 1, le[0] = (de[0] * fe + de[4] * he + de[8] * me + de[12]) / pe, le[1] = (de[1] * fe + de[5] * he + de[9] * me + de[13]) / pe, le[2] = (de[2] * fe + de[6] * he + de[10] * me + de[14]) / pe, le;
                };
              }, 264: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2], pe = de[0], ye = de[1], ge = de[2], ve = de[3], be = ve * fe + ye * me - ge * he, Ce = ve * he + ge * fe - pe * me, Se = ve * me + pe * he - ye * fe, we = -pe * fe - ye * he - ge * me;
                  return le[0] = be * ve + we * -pe + Ce * -ge - Se * -ye, le[1] = Ce * ve + we * -ye + Se * -pe - be * -ge, le[2] = Se * ve + we * -ge + be * -ye - Ce * -pe, le;
                };
              }, 4361: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] + de[0], le[1] = ue[1] + de[1], le[2] = ue[2] + de[2], le[3] = ue[3] + de[3], le;
                };
              }, 2335: function(ce) {
                ce.exports = function(le) {
                  var ue = new Float32Array(4);
                  return ue[0] = le[0], ue[1] = le[1], ue[2] = le[2], ue[3] = le[3], ue;
                };
              }, 2933: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = ue[0], le[1] = ue[1], le[2] = ue[2], le[3] = ue[3], le;
                };
              }, 7536: function(ce) {
                ce.exports = function() {
                  var le = new Float32Array(4);
                  return le[0] = 0, le[1] = 0, le[2] = 0, le[3] = 0, le;
                };
              }, 4691: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0] - le[0], fe = ue[1] - le[1], he = ue[2] - le[2], me = ue[3] - le[3];
                  return Math.sqrt(de * de + fe * fe + he * he + me * me);
                };
              }, 1373: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] / de[0], le[1] = ue[1] / de[1], le[2] = ue[2] / de[2], le[3] = ue[3] / de[3], le;
                };
              }, 3750: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] * ue[0] + le[1] * ue[1] + le[2] * ue[2] + le[3] * ue[3];
                };
              }, 3390: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he = new Float32Array(4);
                  return he[0] = le, he[1] = ue, he[2] = de, he[3] = fe, he;
                };
              }, 9970: function(ce, le, ue) {
                ce.exports = { create: ue(7536), clone: ue(2335), fromValues: ue(3390), copy: ue(2933), set: ue(4578), add: ue(4361), subtract: ue(6860), multiply: ue(3576), divide: ue(1373), min: ue(2334), max: ue(160), scale: ue(9288), scaleAndAdd: ue(4844), distance: ue(4691), squaredDistance: ue(7960), length: ue(6808), squaredLength: ue(483), negate: ue(1498), inverse: ue(4494), normalize: ue(5177), dot: ue(3750), lerp: ue(2573), random: ue(9131), transformMat4: ue(5352), transformQuat: ue(4041) };
              }, 4494: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = 1 / ue[0], le[1] = 1 / ue[1], le[2] = 1 / ue[2], le[3] = 1 / ue[3], le;
                };
              }, 6808: function(ce) {
                ce.exports = function(le) {
                  var ue = le[0], de = le[1], fe = le[2], he = le[3];
                  return Math.sqrt(ue * ue + de * de + fe * fe + he * he);
                };
              }, 2573: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  var he = ue[0], me = ue[1], pe = ue[2], ye = ue[3];
                  return le[0] = he + fe * (de[0] - he), le[1] = me + fe * (de[1] - me), le[2] = pe + fe * (de[2] - pe), le[3] = ye + fe * (de[3] - ye), le;
                };
              }, 160: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = Math.max(ue[0], de[0]), le[1] = Math.max(ue[1], de[1]), le[2] = Math.max(ue[2], de[2]), le[3] = Math.max(ue[3], de[3]), le;
                };
              }, 2334: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = Math.min(ue[0], de[0]), le[1] = Math.min(ue[1], de[1]), le[2] = Math.min(ue[2], de[2]), le[3] = Math.min(ue[3], de[3]), le;
                };
              }, 3576: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] * de[0], le[1] = ue[1] * de[1], le[2] = ue[2] * de[2], le[3] = ue[3] * de[3], le;
                };
              }, 1498: function(ce) {
                ce.exports = function(le, ue) {
                  return le[0] = -ue[0], le[1] = -ue[1], le[2] = -ue[2], le[3] = -ue[3], le;
                };
              }, 5177: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0], fe = ue[1], he = ue[2], me = ue[3], pe = de * de + fe * fe + he * he + me * me;
                  return pe > 0 && (pe = 1 / Math.sqrt(pe), le[0] = de * pe, le[1] = fe * pe, le[2] = he * pe, le[3] = me * pe), le;
                };
              }, 9131: function(ce, le, ue) {
                var de = ue(5177), fe = ue(9288);
                ce.exports = function(he, me) {
                  return me = me || 1, he[0] = Math.random(), he[1] = Math.random(), he[2] = Math.random(), he[3] = Math.random(), de(he, he), fe(he, he, me), he;
                };
              }, 9288: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] * de, le[1] = ue[1] * de, le[2] = ue[2] * de, le[3] = ue[3] * de, le;
                };
              }, 4844: function(ce) {
                ce.exports = function(le, ue, de, fe) {
                  return le[0] = ue[0] + de[0] * fe, le[1] = ue[1] + de[1] * fe, le[2] = ue[2] + de[2] * fe, le[3] = ue[3] + de[3] * fe, le;
                };
              }, 4578: function(ce) {
                ce.exports = function(le, ue, de, fe, he) {
                  return le[0] = ue, le[1] = de, le[2] = fe, le[3] = he, le;
                };
              }, 7960: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[0] - le[0], fe = ue[1] - le[1], he = ue[2] - le[2], me = ue[3] - le[3];
                  return de * de + fe * fe + he * he + me * me;
                };
              }, 483: function(ce) {
                ce.exports = function(le) {
                  var ue = le[0], de = le[1], fe = le[2], he = le[3];
                  return ue * ue + de * de + fe * fe + he * he;
                };
              }, 6860: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le[0] = ue[0] - de[0], le[1] = ue[1] - de[1], le[2] = ue[2] - de[2], le[3] = ue[3] - de[3], le;
                };
              }, 5352: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2], pe = ue[3];
                  return le[0] = de[0] * fe + de[4] * he + de[8] * me + de[12] * pe, le[1] = de[1] * fe + de[5] * he + de[9] * me + de[13] * pe, le[2] = de[2] * fe + de[6] * he + de[10] * me + de[14] * pe, le[3] = de[3] * fe + de[7] * he + de[11] * me + de[15] * pe, le;
                };
              }, 4041: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = ue[0], he = ue[1], me = ue[2], pe = de[0], ye = de[1], ge = de[2], ve = de[3], be = ve * fe + ye * me - ge * he, Ce = ve * he + ge * fe - pe * me, Se = ve * me + pe * he - ye * fe, we = -pe * fe - ye * he - ge * me;
                  return le[0] = be * ve + we * -pe + Ce * -ge - Se * -ye, le[1] = Ce * ve + we * -ye + Se * -pe - be * -ge, le[2] = Se * ve + we * -ge + be * -ye - Ce * -pe, le[3] = ue[3], le;
                };
              }, 1848: function(ce, le, ue) {
                var de = ue(4905), fe = ue(6468);
                ce.exports = function(he) {
                  for (var me = Array.isArray(he) ? he : de(he), pe = 0; pe < me.length; pe++) {
                    var ye = me[pe];
                    if (ye.type === "preprocessor") {
                      var ge = ye.data.match(/\#define\s+SHADER_NAME(_B64)?\s+(.+)$/);
                      if (ge && ge[2]) {
                        var ve = ge[1], be = ge[2];
                        return (ve ? fe(be) : be).trim();
                      }
                    }
                  }
                };
              }, 5874: function(ce, le, ue) {
                ce.exports = function(Pe) {
                  var $e, qe, Xe, Ve = 0, Ze = 0, Ge = ye, ze = [], We = [], at = 1, ct = 0, et = 0, dt = !1, Je = !1, it = "", st = he, gt = de;
                  (Pe = Pe || {}).version === "300 es" && (st = pe, gt = me);
                  var bt = {}, yt = {};
                  for (Ve = 0; Ve < st.length; Ve++) bt[st[Ve]] = !0;
                  for (Ve = 0; Ve < gt.length; Ve++) yt[gt[Ve]] = !0;
                  return function(It) {
                    return We = [], It !== null ? function(Ct) {
                      var Kt;
                      for (Ve = 0, Ct.toString && (Ct = Ct.toString()), it += Ct.replace(/\r\n/g, `
`), Xe = it.length; $e = it[Ve], Ve < Xe; ) {
                        switch (Kt = Ve, Ge) {
                          case ve:
                            Ve = $t();
                            break;
                          case be:
                          case Ce:
                            Ve = vt();
                            break;
                          case Se:
                            Ve = Pt();
                            break;
                          case we:
                            Ve = Gt();
                            break;
                          case Re:
                            Ve = Nt();
                            break;
                          case Ee:
                            Ve = Ft();
                            break;
                          case ge:
                            Ve = Tt();
                            break;
                          case ke:
                            Ve = Rt();
                            break;
                          case ye:
                            Ve = Lt();
                        }
                        Kt !== Ve && (it[Kt] === `
` ? (ct = 0, ++at) : ++ct);
                      }
                      return Ze += Ve, it = it.slice(Ve), We;
                    }(It) : (ze.length && St(ze.join("")), Ge = Me, St("(eof)"), We);
                  };
                  function St(It) {
                    It.length && We.push({ type: De[Ge], data: It, position: et, line: at, column: ct });
                  }
                  function Lt() {
                    return ze = ze.length ? [] : ze, qe === "/" && $e === "*" ? (et = Ze + Ve - 1, Ge = ve, qe = $e, Ve + 1) : qe === "/" && $e === "/" ? (et = Ze + Ve - 1, Ge = be, qe = $e, Ve + 1) : $e === "#" ? (Ge = Ce, et = Ze + Ve, Ve) : /\s/.test($e) ? (Ge = ke, et = Ze + Ve, Ve) : (dt = /\d/.test($e), Je = /[^\w_]/.test($e), et = Ze + Ve, Ge = dt ? we : Je ? Se : ge, Ve);
                  }
                  function Rt() {
                    return /[^\s]/g.test($e) ? (St(ze.join("")), Ge = ye, Ve) : (ze.push($e), qe = $e, Ve + 1);
                  }
                  function vt() {
                    return $e !== "\r" && $e !== `
` || qe === "\\" ? (ze.push($e), qe = $e, Ve + 1) : (St(ze.join("")), Ge = ye, Ve);
                  }
                  function $t() {
                    return $e === "/" && qe === "*" ? (ze.push($e), St(ze.join("")), Ge = ye, Ve + 1) : (ze.push($e), qe = $e, Ve + 1);
                  }
                  function Pt() {
                    if (qe === "." && /\d/.test($e)) return Ge = Ee, Ve;
                    if (qe === "/" && $e === "*") return Ge = ve, Ve;
                    if (qe === "/" && $e === "/") return Ge = be, Ve;
                    if ($e === "." && ze.length) {
                      for (; Xt(ze); ) ;
                      return Ge = Ee, Ve;
                    }
                    if ($e === ";" || $e === ")" || $e === "(") {
                      if (ze.length) for (; Xt(ze); ) ;
                      return St($e), Ge = ye, Ve + 1;
                    }
                    var It = ze.length === 2 && $e !== "=";
                    if (/[\w_\d\s]/.test($e) || It) {
                      for (; Xt(ze); ) ;
                      return Ge = ye, Ve;
                    }
                    return ze.push($e), qe = $e, Ve + 1;
                  }
                  function Xt(It) {
                    for (var Ct, Kt, Vt = 0; ; ) {
                      if (Ct = fe.indexOf(It.slice(0, It.length + Vt).join("")), Kt = fe[Ct], Ct === -1) {
                        if (Vt-- + It.length > 0) continue;
                        Kt = It.slice(0, 1).join("");
                      }
                      return St(Kt), et += Kt.length, (ze = ze.slice(Kt.length)).length;
                    }
                  }
                  function Nt() {
                    return /[^a-fA-F0-9]/.test($e) ? (St(ze.join("")), Ge = ye, Ve) : (ze.push($e), qe = $e, Ve + 1);
                  }
                  function Gt() {
                    return $e === "." || /[eE]/.test($e) ? (ze.push($e), Ge = Ee, qe = $e, Ve + 1) : $e === "x" && ze.length === 1 && ze[0] === "0" ? (Ge = Re, ze.push($e), qe = $e, Ve + 1) : /[^\d]/.test($e) ? (St(ze.join("")), Ge = ye, Ve) : (ze.push($e), qe = $e, Ve + 1);
                  }
                  function Ft() {
                    return $e === "f" && (ze.push($e), qe = $e, Ve += 1), /[eE]/.test($e) ? (ze.push($e), qe = $e, Ve + 1) : ($e !== "-" && $e !== "+" || !/[eE]/.test(qe)) && /[^\d]/.test($e) ? (St(ze.join("")), Ge = ye, Ve) : (ze.push($e), qe = $e, Ve + 1);
                  }
                  function Tt() {
                    if (/[^\d\w_]/.test($e)) {
                      var It = ze.join("");
                      return Ge = yt[It] ? Qe : bt[It] ? Le : xe, St(ze.join("")), Ge = ye, Ve;
                    }
                    return ze.push($e), qe = $e, Ve + 1;
                  }
                };
                var de = ue(620), fe = ue(7827), he = ue(6852), me = ue(7932), pe = ue(3508), ye = 999, ge = 9999, ve = 0, be = 1, Ce = 2, Se = 3, we = 4, Ee = 5, xe = 6, Le = 7, Qe = 8, ke = 9, Me = 10, Re = 11, De = ["block-comment", "line-comment", "preprocessor", "operator", "integer", "float", "ident", "builtin", "keyword", "whitespace", "eof", "integer"];
              }, 3508: function(ce, le, ue) {
                var de = ue(6852);
                de = de.slice().filter(function(fe) {
                  return !/^(gl\_|texture)/.test(fe);
                }), ce.exports = de.concat(["gl_VertexID", "gl_InstanceID", "gl_Position", "gl_PointSize", "gl_FragCoord", "gl_FrontFacing", "gl_FragDepth", "gl_PointCoord", "gl_MaxVertexAttribs", "gl_MaxVertexUniformVectors", "gl_MaxVertexOutputVectors", "gl_MaxFragmentInputVectors", "gl_MaxVertexTextureImageUnits", "gl_MaxCombinedTextureImageUnits", "gl_MaxTextureImageUnits", "gl_MaxFragmentUniformVectors", "gl_MaxDrawBuffers", "gl_MinProgramTexelOffset", "gl_MaxProgramTexelOffset", "gl_DepthRangeParameters", "gl_DepthRange", "trunc", "round", "roundEven", "isnan", "isinf", "floatBitsToInt", "floatBitsToUint", "intBitsToFloat", "uintBitsToFloat", "packSnorm2x16", "unpackSnorm2x16", "packUnorm2x16", "unpackUnorm2x16", "packHalf2x16", "unpackHalf2x16", "outerProduct", "transpose", "determinant", "inverse", "texture", "textureSize", "textureProj", "textureLod", "textureOffset", "texelFetch", "texelFetchOffset", "textureProjOffset", "textureLodOffset", "textureProjLod", "textureProjLodOffset", "textureGrad", "textureGradOffset", "textureProjGrad", "textureProjGradOffset"]);
              }, 6852: function(ce) {
                ce.exports = ["abs", "acos", "all", "any", "asin", "atan", "ceil", "clamp", "cos", "cross", "dFdx", "dFdy", "degrees", "distance", "dot", "equal", "exp", "exp2", "faceforward", "floor", "fract", "gl_BackColor", "gl_BackLightModelProduct", "gl_BackLightProduct", "gl_BackMaterial", "gl_BackSecondaryColor", "gl_ClipPlane", "gl_ClipVertex", "gl_Color", "gl_DepthRange", "gl_DepthRangeParameters", "gl_EyePlaneQ", "gl_EyePlaneR", "gl_EyePlaneS", "gl_EyePlaneT", "gl_Fog", "gl_FogCoord", "gl_FogFragCoord", "gl_FogParameters", "gl_FragColor", "gl_FragCoord", "gl_FragData", "gl_FragDepth", "gl_FragDepthEXT", "gl_FrontColor", "gl_FrontFacing", "gl_FrontLightModelProduct", "gl_FrontLightProduct", "gl_FrontMaterial", "gl_FrontSecondaryColor", "gl_LightModel", "gl_LightModelParameters", "gl_LightModelProducts", "gl_LightProducts", "gl_LightSource", "gl_LightSourceParameters", "gl_MaterialParameters", "gl_MaxClipPlanes", "gl_MaxCombinedTextureImageUnits", "gl_MaxDrawBuffers", "gl_MaxFragmentUniformComponents", "gl_MaxLights", "gl_MaxTextureCoords", "gl_MaxTextureImageUnits", "gl_MaxTextureUnits", "gl_MaxVaryingFloats", "gl_MaxVertexAttribs", "gl_MaxVertexTextureImageUnits", "gl_MaxVertexUniformComponents", "gl_ModelViewMatrix", "gl_ModelViewMatrixInverse", "gl_ModelViewMatrixInverseTranspose", "gl_ModelViewMatrixTranspose", "gl_ModelViewProjectionMatrix", "gl_ModelViewProjectionMatrixInverse", "gl_ModelViewProjectionMatrixInverseTranspose", "gl_ModelViewProjectionMatrixTranspose", "gl_MultiTexCoord0", "gl_MultiTexCoord1", "gl_MultiTexCoord2", "gl_MultiTexCoord3", "gl_MultiTexCoord4", "gl_MultiTexCoord5", "gl_MultiTexCoord6", "gl_MultiTexCoord7", "gl_Normal", "gl_NormalMatrix", "gl_NormalScale", "gl_ObjectPlaneQ", "gl_ObjectPlaneR", "gl_ObjectPlaneS", "gl_ObjectPlaneT", "gl_Point", "gl_PointCoord", "gl_PointParameters", "gl_PointSize", "gl_Position", "gl_ProjectionMatrix", "gl_ProjectionMatrixInverse", "gl_ProjectionMatrixInverseTranspose", "gl_ProjectionMatrixTranspose", "gl_SecondaryColor", "gl_TexCoord", "gl_TextureEnvColor", "gl_TextureMatrix", "gl_TextureMatrixInverse", "gl_TextureMatrixInverseTranspose", "gl_TextureMatrixTranspose", "gl_Vertex", "greaterThan", "greaterThanEqual", "inversesqrt", "length", "lessThan", "lessThanEqual", "log", "log2", "matrixCompMult", "max", "min", "mix", "mod", "normalize", "not", "notEqual", "pow", "radians", "reflect", "refract", "sign", "sin", "smoothstep", "sqrt", "step", "tan", "texture2D", "texture2DLod", "texture2DProj", "texture2DProjLod", "textureCube", "textureCubeLod", "texture2DLodEXT", "texture2DProjLodEXT", "textureCubeLodEXT", "texture2DGradEXT", "texture2DProjGradEXT", "textureCubeGradEXT"];
              }, 7932: function(ce, le, ue) {
                var de = ue(620);
                ce.exports = de.slice().concat(["layout", "centroid", "smooth", "case", "mat2x2", "mat2x3", "mat2x4", "mat3x2", "mat3x3", "mat3x4", "mat4x2", "mat4x3", "mat4x4", "uvec2", "uvec3", "uvec4", "samplerCubeShadow", "sampler2DArray", "sampler2DArrayShadow", "isampler2D", "isampler3D", "isamplerCube", "isampler2DArray", "usampler2D", "usampler3D", "usamplerCube", "usampler2DArray", "coherent", "restrict", "readonly", "writeonly", "resource", "atomic_uint", "noperspective", "patch", "sample", "subroutine", "common", "partition", "active", "filter", "image1D", "image2D", "image3D", "imageCube", "iimage1D", "iimage2D", "iimage3D", "iimageCube", "uimage1D", "uimage2D", "uimage3D", "uimageCube", "image1DArray", "image2DArray", "iimage1DArray", "iimage2DArray", "uimage1DArray", "uimage2DArray", "image1DShadow", "image2DShadow", "image1DArrayShadow", "image2DArrayShadow", "imageBuffer", "iimageBuffer", "uimageBuffer", "sampler1DArray", "sampler1DArrayShadow", "isampler1D", "isampler1DArray", "usampler1D", "usampler1DArray", "isampler2DRect", "usampler2DRect", "samplerBuffer", "isamplerBuffer", "usamplerBuffer", "sampler2DMS", "isampler2DMS", "usampler2DMS", "sampler2DMSArray", "isampler2DMSArray", "usampler2DMSArray"]);
              }, 620: function(ce) {
                ce.exports = ["precision", "highp", "mediump", "lowp", "attribute", "const", "uniform", "varying", "break", "continue", "do", "for", "while", "if", "else", "in", "out", "inout", "float", "int", "uint", "void", "bool", "true", "false", "discard", "return", "mat2", "mat3", "mat4", "vec2", "vec3", "vec4", "ivec2", "ivec3", "ivec4", "bvec2", "bvec3", "bvec4", "sampler1D", "sampler2D", "sampler3D", "samplerCube", "sampler1DShadow", "sampler2DShadow", "struct", "asm", "class", "union", "enum", "typedef", "template", "this", "packed", "goto", "switch", "default", "inline", "noinline", "volatile", "public", "static", "extern", "external", "interface", "long", "short", "double", "half", "fixed", "unsigned", "input", "output", "hvec2", "hvec3", "hvec4", "dvec2", "dvec3", "dvec4", "fvec2", "fvec3", "fvec4", "sampler2DRect", "sampler3DRect", "sampler2DRectShadow", "sizeof", "cast", "namespace", "using"];
              }, 7827: function(ce) {
                ce.exports = ["<<=", ">>=", "++", "--", "<<", ">>", "<=", ">=", "==", "!=", "&&", "||", "+=", "-=", "*=", "/=", "%=", "&=", "^^", "^=", "|=", "(", ")", "[", "]", ".", "!", "~", "*", "/", "%", "+", "-", "<", ">", "&", "^", "|", "?", ":", "=", ",", ";", "{", "}"];
              }, 4905: function(ce, le, ue) {
                var de = ue(5874);
                ce.exports = function(fe, he) {
                  var me = de(he), pe = [];
                  return (pe = pe.concat(me(fe))).concat(me(null));
                };
              }, 3236: function(ce) {
                ce.exports = function(le) {
                  typeof le == "string" && (le = [le]);
                  for (var ue = [].slice.call(arguments, 1), de = [], fe = 0; fe < le.length - 1; fe++) de.push(le[fe], ue[fe] || "");
                  return de.push(le[fe]), de.join("");
                };
              }, 7520: function(ce, le, ue) {
                var de = ue(9507);
                ce.exports = de && function() {
                  var fe = !1;
                  try {
                    var he = Object.defineProperty({}, "passive", { get: function() {
                      fe = !0;
                    } });
                    window.addEventListener("test", null, he), window.removeEventListener("test", null, he);
                  } catch {
                    fe = !1;
                  }
                  return fe;
                }();
              }, 3778: function(ce, le) {
                le.read = function(ue, de, fe, he, me) {
                  var pe, ye, ge = 8 * me - he - 1, ve = (1 << ge) - 1, be = ve >> 1, Ce = -7, Se = fe ? me - 1 : 0, we = fe ? -1 : 1, Ee = ue[de + Se];
                  for (Se += we, pe = Ee & (1 << -Ce) - 1, Ee >>= -Ce, Ce += ge; Ce > 0; pe = 256 * pe + ue[de + Se], Se += we, Ce -= 8) ;
                  for (ye = pe & (1 << -Ce) - 1, pe >>= -Ce, Ce += he; Ce > 0; ye = 256 * ye + ue[de + Se], Se += we, Ce -= 8) ;
                  if (pe === 0) pe = 1 - be;
                  else {
                    if (pe === ve) return ye ? NaN : 1 / 0 * (Ee ? -1 : 1);
                    ye += Math.pow(2, he), pe -= be;
                  }
                  return (Ee ? -1 : 1) * ye * Math.pow(2, pe - he);
                }, le.write = function(ue, de, fe, he, me, pe) {
                  var ye, ge, ve, be = 8 * pe - me - 1, Ce = (1 << be) - 1, Se = Ce >> 1, we = me === 23 ? Math.pow(2, -24) - Math.pow(2, -77) : 0, Ee = he ? 0 : pe - 1, xe = he ? 1 : -1, Le = de < 0 || de === 0 && 1 / de < 0 ? 1 : 0;
                  for (de = Math.abs(de), isNaN(de) || de === 1 / 0 ? (ge = isNaN(de) ? 1 : 0, ye = Ce) : (ye = Math.floor(Math.log(de) / Math.LN2), de * (ve = Math.pow(2, -ye)) < 1 && (ye--, ve *= 2), (de += ye + Se >= 1 ? we / ve : we * Math.pow(2, 1 - Se)) * ve >= 2 && (ye++, ve /= 2), ye + Se >= Ce ? (ge = 0, ye = Ce) : ye + Se >= 1 ? (ge = (de * ve - 1) * Math.pow(2, me), ye += Se) : (ge = de * Math.pow(2, Se - 1) * Math.pow(2, me), ye = 0)); me >= 8; ue[fe + Ee] = 255 & ge, Ee += xe, ge /= 256, me -= 8) ;
                  for (ye = ye << me | ge, be += me; be > 0; ue[fe + Ee] = 255 & ye, Ee += xe, ye /= 256, be -= 8) ;
                  ue[fe + Ee - xe] |= 128 * Le;
                };
              }, 8954: function(ce, le, ue) {
                ce.exports = function(be, Ce) {
                  var Se = be.length;
                  if (Se === 0) throw new Error("Must have at least d+1 points");
                  var we = be[0].length;
                  if (Se <= we) throw new Error("Must input at least d+1 points");
                  var Ee = be.slice(0, we + 1), xe = de.apply(void 0, Ee);
                  if (xe === 0) throw new Error("Input not in general position");
                  for (var Le = new Array(we + 1), Qe = 0; Qe <= we; ++Qe) Le[Qe] = Qe;
                  xe < 0 && (Le[0] = 1, Le[1] = 0);
                  var ke = new he(Le, new Array(we + 1), !1), Me = ke.adjacent, Re = new Array(we + 2);
                  for (Qe = 0; Qe <= we; ++Qe) {
                    for (var De = Le.slice(), Pe = 0; Pe <= we; ++Pe) Pe === Qe && (De[Pe] = -1);
                    var $e = De[0];
                    De[0] = De[1], De[1] = $e;
                    var qe = new he(De, new Array(we + 1), !0);
                    Me[Qe] = qe, Re[Qe] = qe;
                  }
                  for (Re[we + 1] = ke, Qe = 0; Qe <= we; ++Qe) {
                    De = Me[Qe].vertices;
                    var Xe = Me[Qe].adjacent;
                    for (Pe = 0; Pe <= we; ++Pe) {
                      var Ve = De[Pe];
                      if (Ve < 0) Xe[Pe] = ke;
                      else for (var Ze = 0; Ze <= we; ++Ze) Me[Ze].vertices.indexOf(Ve) < 0 && (Xe[Pe] = Me[Ze]);
                    }
                  }
                  var Ge = new ge(we, Ee, Re), ze = !!Ce;
                  for (Qe = we + 1; Qe < Se; ++Qe) Ge.insert(be[Qe], ze);
                  return Ge.boundary();
                };
                var de = ue(3250), fe = ue(6803).Fw;
                function he(be, Ce, Se) {
                  this.vertices = be, this.adjacent = Ce, this.boundary = Se, this.lastVisited = -1;
                }
                function me(be, Ce, Se) {
                  this.vertices = be, this.cell = Ce, this.index = Se;
                }
                function pe(be, Ce) {
                  return fe(be.vertices, Ce.vertices);
                }
                he.prototype.flip = function() {
                  var be = this.vertices[0];
                  this.vertices[0] = this.vertices[1], this.vertices[1] = be;
                  var Ce = this.adjacent[0];
                  this.adjacent[0] = this.adjacent[1], this.adjacent[1] = Ce;
                };
                var ye = [];
                function ge(be, Ce, Se) {
                  this.dimension = be, this.vertices = Ce, this.simplices = Se, this.interior = Se.filter(function(Le) {
                    return !Le.boundary;
                  }), this.tuple = new Array(be + 1);
                  for (var we = 0; we <= be; ++we) this.tuple[we] = this.vertices[we];
                  var Ee, xe = ye[be];
                  xe || (xe = ye[be] = ((Ee = de[be + 1]) || (Ee = de), /* @__PURE__ */ function(Le) {
                    return function() {
                      var Qe = this.tuple;
                      return Le.apply(this, Qe);
                    };
                  }(Ee))), this.orient = xe;
                }
                var ve = ge.prototype;
                ve.handleBoundaryDegeneracy = function(be, Ce) {
                  var Se = this.dimension, we = this.vertices.length - 1, Ee = this.tuple, xe = this.vertices, Le = [be];
                  for (be.lastVisited = -we; Le.length > 0; ) for (var Qe = (be = Le.pop()).adjacent, ke = 0; ke <= Se; ++ke) {
                    var Me = Qe[ke];
                    if (Me.boundary && !(Me.lastVisited <= -we)) {
                      for (var Re = Me.vertices, De = 0; De <= Se; ++De) {
                        var Pe = Re[De];
                        Ee[De] = Pe < 0 ? Ce : xe[Pe];
                      }
                      var $e = this.orient();
                      if ($e > 0) return Me;
                      Me.lastVisited = -we, $e === 0 && Le.push(Me);
                    }
                  }
                  return null;
                }, ve.walk = function(be, Ce) {
                  var Se = this.vertices.length - 1, we = this.dimension, Ee = this.vertices, xe = this.tuple, Le = Ce ? this.interior.length * Math.random() | 0 : this.interior.length - 1, Qe = this.interior[Le];
                  e: for (; !Qe.boundary; ) {
                    for (var ke = Qe.vertices, Me = Qe.adjacent, Re = 0; Re <= we; ++Re) xe[Re] = Ee[ke[Re]];
                    for (Qe.lastVisited = Se, Re = 0; Re <= we; ++Re) {
                      var De = Me[Re];
                      if (!(De.lastVisited >= Se)) {
                        var Pe = xe[Re];
                        xe[Re] = be;
                        var $e = this.orient();
                        if (xe[Re] = Pe, $e < 0) {
                          Qe = De;
                          continue e;
                        }
                        De.boundary ? De.lastVisited = -Se : De.lastVisited = Se;
                      }
                    }
                    return;
                  }
                  return Qe;
                }, ve.addPeaks = function(be, Ce) {
                  var Se = this.vertices.length - 1, we = this.dimension, Ee = this.vertices, xe = this.tuple, Le = this.interior, Qe = this.simplices, ke = [Ce];
                  Ce.lastVisited = Se, Ce.vertices[Ce.vertices.indexOf(-1)] = Se, Ce.boundary = !1, Le.push(Ce);
                  for (var Me = []; ke.length > 0; ) {
                    var Re = (Ce = ke.pop()).vertices, De = Ce.adjacent, Pe = Re.indexOf(Se);
                    if (!(Pe < 0)) {
                      for (var $e = 0; $e <= we; ++$e) if ($e !== Pe) {
                        var qe = De[$e];
                        if (qe.boundary && !(qe.lastVisited >= Se)) {
                          var Xe = qe.vertices;
                          if (qe.lastVisited !== -Se) {
                            for (var Ve = 0, Ze = 0; Ze <= we; ++Ze) Xe[Ze] < 0 ? (Ve = Ze, xe[Ze] = be) : xe[Ze] = Ee[Xe[Ze]];
                            if (this.orient() > 0) {
                              Xe[Ve] = Se, qe.boundary = !1, Le.push(qe), ke.push(qe), qe.lastVisited = Se;
                              continue;
                            }
                            qe.lastVisited = -Se;
                          }
                          var Ge = qe.adjacent, ze = Re.slice(), We = De.slice(), at = new he(ze, We, !0);
                          Qe.push(at);
                          var ct = Ge.indexOf(Ce);
                          if (!(ct < 0)) for (Ge[ct] = at, We[Pe] = qe, ze[$e] = -1, We[$e] = Ce, De[$e] = at, at.flip(), Ze = 0; Ze <= we; ++Ze) {
                            var et = ze[Ze];
                            if (!(et < 0 || et === Se)) {
                              for (var dt = new Array(we - 1), Je = 0, it = 0; it <= we; ++it) {
                                var st = ze[it];
                                st < 0 || it === Ze || (dt[Je++] = st);
                              }
                              Me.push(new me(dt, at, Ze));
                            }
                          }
                        }
                      }
                    }
                  }
                  for (Me.sort(pe), $e = 0; $e + 1 < Me.length; $e += 2) {
                    var gt = Me[$e], bt = Me[$e + 1], yt = gt.index, St = bt.index;
                    yt < 0 || St < 0 || (gt.cell.adjacent[gt.index] = bt.cell, bt.cell.adjacent[bt.index] = gt.cell);
                  }
                }, ve.insert = function(be, Ce) {
                  var Se = this.vertices;
                  Se.push(be);
                  var we = this.walk(be, Ce);
                  if (we) {
                    for (var Ee = this.dimension, xe = this.tuple, Le = 0; Le <= Ee; ++Le) {
                      var Qe = we.vertices[Le];
                      xe[Le] = Qe < 0 ? be : Se[Qe];
                    }
                    var ke = this.orient(xe);
                    ke < 0 || (ke !== 0 || (we = this.handleBoundaryDegeneracy(we, be))) && this.addPeaks(be, we);
                  }
                }, ve.boundary = function() {
                  for (var be = this.dimension, Ce = [], Se = this.simplices, we = Se.length, Ee = 0; Ee < we; ++Ee) {
                    var xe = Se[Ee];
                    if (xe.boundary) {
                      for (var Le = new Array(be), Qe = xe.vertices, ke = 0, Me = 0, Re = 0; Re <= be; ++Re) Qe[Re] >= 0 ? Le[ke++] = Qe[Re] : Me = 1 & Re;
                      if (Me === (1 & be)) {
                        var De = Le[0];
                        Le[0] = Le[1], Le[1] = De;
                      }
                      Ce.push(Le);
                    }
                  }
                  return Ce;
                };
              }, 3352: function(ce, le, ue) {
                var de = ue(2478);
                function fe(ke, Me, Re, De, Pe) {
                  this.mid = ke, this.left = Me, this.right = Re, this.leftPoints = De, this.rightPoints = Pe, this.count = (Me ? Me.count : 0) + (Re ? Re.count : 0) + De.length;
                }
                ce.exports = function(ke) {
                  return ke && ke.length !== 0 ? new Le(xe(ke)) : new Le(null);
                };
                var he = fe.prototype;
                function me(ke, Me) {
                  ke.mid = Me.mid, ke.left = Me.left, ke.right = Me.right, ke.leftPoints = Me.leftPoints, ke.rightPoints = Me.rightPoints, ke.count = Me.count;
                }
                function pe(ke, Me) {
                  var Re = xe(Me);
                  ke.mid = Re.mid, ke.left = Re.left, ke.right = Re.right, ke.leftPoints = Re.leftPoints, ke.rightPoints = Re.rightPoints, ke.count = Re.count;
                }
                function ye(ke, Me) {
                  var Re = ke.intervals([]);
                  Re.push(Me), pe(ke, Re);
                }
                function ge(ke, Me) {
                  var Re = ke.intervals([]), De = Re.indexOf(Me);
                  return De < 0 ? 0 : (Re.splice(De, 1), pe(ke, Re), 1);
                }
                function ve(ke, Me, Re) {
                  for (var De = 0; De < ke.length && ke[De][0] <= Me; ++De) {
                    var Pe = Re(ke[De]);
                    if (Pe) return Pe;
                  }
                }
                function be(ke, Me, Re) {
                  for (var De = ke.length - 1; De >= 0 && ke[De][1] >= Me; --De) {
                    var Pe = Re(ke[De]);
                    if (Pe) return Pe;
                  }
                }
                function Ce(ke, Me) {
                  for (var Re = 0; Re < ke.length; ++Re) {
                    var De = Me(ke[Re]);
                    if (De) return De;
                  }
                }
                function Se(ke, Me) {
                  return ke - Me;
                }
                function we(ke, Me) {
                  return ke[0] - Me[0] || ke[1] - Me[1];
                }
                function Ee(ke, Me) {
                  return ke[1] - Me[1] || ke[0] - Me[0];
                }
                function xe(ke) {
                  if (ke.length === 0) return null;
                  for (var Me = [], Re = 0; Re < ke.length; ++Re) Me.push(ke[Re][0], ke[Re][1]);
                  Me.sort(Se);
                  var De = Me[Me.length >> 1], Pe = [], $e = [], qe = [];
                  for (Re = 0; Re < ke.length; ++Re) {
                    var Xe = ke[Re];
                    Xe[1] < De ? Pe.push(Xe) : De < Xe[0] ? $e.push(Xe) : qe.push(Xe);
                  }
                  var Ve = qe, Ze = qe.slice();
                  return Ve.sort(we), Ze.sort(Ee), new fe(De, xe(Pe), xe($e), Ve, Ze);
                }
                function Le(ke) {
                  this.root = ke;
                }
                he.intervals = function(ke) {
                  return ke.push.apply(ke, this.leftPoints), this.left && this.left.intervals(ke), this.right && this.right.intervals(ke), ke;
                }, he.insert = function(ke) {
                  var Me = this.count - this.leftPoints.length;
                  if (this.count += 1, ke[1] < this.mid) this.left ? 4 * (this.left.count + 1) > 3 * (Me + 1) ? ye(this, ke) : this.left.insert(ke) : this.left = xe([ke]);
                  else if (ke[0] > this.mid) this.right ? 4 * (this.right.count + 1) > 3 * (Me + 1) ? ye(this, ke) : this.right.insert(ke) : this.right = xe([ke]);
                  else {
                    var Re = de.ge(this.leftPoints, ke, we), De = de.ge(this.rightPoints, ke, Ee);
                    this.leftPoints.splice(Re, 0, ke), this.rightPoints.splice(De, 0, ke);
                  }
                }, he.remove = function(ke) {
                  var Me = this.count - this.leftPoints;
                  if (ke[1] < this.mid) return this.left ? 4 * (this.right ? this.right.count : 0) > 3 * (Me - 1) ? ge(this, ke) : ($e = this.left.remove(ke)) === 2 ? (this.left = null, this.count -= 1, 1) : ($e === 1 && (this.count -= 1), $e) : 0;
                  if (ke[0] > this.mid) return this.right ? 4 * (this.left ? this.left.count : 0) > 3 * (Me - 1) ? ge(this, ke) : ($e = this.right.remove(ke)) === 2 ? (this.right = null, this.count -= 1, 1) : ($e === 1 && (this.count -= 1), $e) : 0;
                  if (this.count === 1) return this.leftPoints[0] === ke ? 2 : 0;
                  if (this.leftPoints.length === 1 && this.leftPoints[0] === ke) {
                    if (this.left && this.right) {
                      for (var Re = this, De = this.left; De.right; ) Re = De, De = De.right;
                      if (Re === this) De.right = this.right;
                      else {
                        var Pe = this.left, $e = this.right;
                        Re.count -= De.count, Re.right = De.left, De.left = Pe, De.right = $e;
                      }
                      me(this, De), this.count = (this.left ? this.left.count : 0) + (this.right ? this.right.count : 0) + this.leftPoints.length;
                    } else this.left ? me(this, this.left) : me(this, this.right);
                    return 1;
                  }
                  for (Pe = de.ge(this.leftPoints, ke, we); Pe < this.leftPoints.length && this.leftPoints[Pe][0] === ke[0]; ++Pe) if (this.leftPoints[Pe] === ke) {
                    for (this.count -= 1, this.leftPoints.splice(Pe, 1), $e = de.ge(this.rightPoints, ke, Ee); $e < this.rightPoints.length && this.rightPoints[$e][1] === ke[1]; ++$e) if (this.rightPoints[$e] === ke) return this.rightPoints.splice($e, 1), 1;
                  }
                  return 0;
                }, he.queryPoint = function(ke, Me) {
                  return ke < this.mid ? this.left && (Re = this.left.queryPoint(ke, Me)) ? Re : ve(this.leftPoints, ke, Me) : ke > this.mid ? this.right && (Re = this.right.queryPoint(ke, Me)) ? Re : be(this.rightPoints, ke, Me) : Ce(this.leftPoints, Me);
                  var Re;
                }, he.queryInterval = function(ke, Me, Re) {
                  var De;
                  return ke < this.mid && this.left && (De = this.left.queryInterval(ke, Me, Re)) || Me > this.mid && this.right && (De = this.right.queryInterval(ke, Me, Re)) ? De : Me < this.mid ? ve(this.leftPoints, Me, Re) : ke > this.mid ? be(this.rightPoints, ke, Re) : Ce(this.leftPoints, Re);
                };
                var Qe = Le.prototype;
                Qe.insert = function(ke) {
                  this.root ? this.root.insert(ke) : this.root = new fe(ke[0], null, null, [ke], [ke]);
                }, Qe.remove = function(ke) {
                  if (this.root) {
                    var Me = this.root.remove(ke);
                    return Me === 2 && (this.root = null), Me !== 0;
                  }
                  return !1;
                }, Qe.queryPoint = function(ke, Me) {
                  if (this.root) return this.root.queryPoint(ke, Me);
                }, Qe.queryInterval = function(ke, Me, Re) {
                  if (ke <= Me && this.root) return this.root.queryInterval(ke, Me, Re);
                }, Object.defineProperty(Qe, "count", { get: function() {
                  return this.root ? this.root.count : 0;
                } }), Object.defineProperty(Qe, "intervals", { get: function() {
                  return this.root ? this.root.intervals([]) : [];
                } });
              }, 7762: function(ce) {
                ce.exports = function(le) {
                  for (var ue = new Array(le), de = 0; de < le; ++de) ue[de] = de;
                  return ue;
                };
              }, 9507: function(ce) {
                ce.exports = !0;
              }, 7163: function(ce) {
                function le(ue) {
                  return !!ue.constructor && typeof ue.constructor.isBuffer == "function" && ue.constructor.isBuffer(ue);
                }
                ce.exports = function(ue) {
                  return ue != null && (le(ue) || function(de) {
                    return typeof de.readFloatLE == "function" && typeof de.slice == "function" && le(de.slice(0, 0));
                  }(ue) || !!ue._isBuffer);
                };
              }, 5219: function(ce) {
                ce.exports = function(le) {
                  for (var ue, de = le.length, fe = 0; fe < de; fe++) if (((ue = le.charCodeAt(fe)) < 9 || ue > 13) && ue !== 32 && ue !== 133 && ue !== 160 && ue !== 5760 && ue !== 6158 && (ue < 8192 || ue > 8205) && ue !== 8232 && ue !== 8233 && ue !== 8239 && ue !== 8287 && ue !== 8288 && ue !== 12288 && ue !== 65279) return !1;
                  return !0;
                };
              }, 395: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le * (1 - de) + ue * de;
                };
              }, 2652: function(ce, le, ue) {
                var de = ue(4335), fe = ue(6864), he = ue(1903), me = ue(9921), pe = ue(7608), ye = ue(5665), ge = { length: ue(1387), normalize: ue(3536), dot: ue(244), cross: ue(5911) }, ve = fe(), be = fe(), Ce = [0, 0, 0, 0], Se = [[0, 0, 0], [0, 0, 0], [0, 0, 0]], we = [0, 0, 0];
                function Ee(xe, Le, Qe, ke, Me) {
                  xe[0] = Le[0] * ke + Qe[0] * Me, xe[1] = Le[1] * ke + Qe[1] * Me, xe[2] = Le[2] * ke + Qe[2] * Me;
                }
                ce.exports = function(xe, Le, Qe, ke, Me, Re) {
                  if (Le || (Le = [0, 0, 0]), Qe || (Qe = [0, 0, 0]), ke || (ke = [0, 0, 0]), Me || (Me = [0, 0, 0, 1]), Re || (Re = [0, 0, 0, 1]), !de(ve, xe) || (he(be, ve), be[3] = 0, be[7] = 0, be[11] = 0, be[15] = 1, Math.abs(me(be) < 1e-8))) return !1;
                  var De, Pe, $e, qe, Xe, Ve, Ze, Ge = ve[3], ze = ve[7], We = ve[11], at = ve[12], ct = ve[13], et = ve[14], dt = ve[15];
                  if (Ge !== 0 || ze !== 0 || We !== 0) {
                    if (Ce[0] = Ge, Ce[1] = ze, Ce[2] = We, Ce[3] = dt, !pe(be, be)) return !1;
                    ye(be, be), De = Me, $e = be, qe = (Pe = Ce)[0], Xe = Pe[1], Ve = Pe[2], Ze = Pe[3], De[0] = $e[0] * qe + $e[4] * Xe + $e[8] * Ve + $e[12] * Ze, De[1] = $e[1] * qe + $e[5] * Xe + $e[9] * Ve + $e[13] * Ze, De[2] = $e[2] * qe + $e[6] * Xe + $e[10] * Ve + $e[14] * Ze, De[3] = $e[3] * qe + $e[7] * Xe + $e[11] * Ve + $e[15] * Ze;
                  } else Me[0] = Me[1] = Me[2] = 0, Me[3] = 1;
                  if (Le[0] = at, Le[1] = ct, Le[2] = et, function(it, st) {
                    it[0][0] = st[0], it[0][1] = st[1], it[0][2] = st[2], it[1][0] = st[4], it[1][1] = st[5], it[1][2] = st[6], it[2][0] = st[8], it[2][1] = st[9], it[2][2] = st[10];
                  }(Se, ve), Qe[0] = ge.length(Se[0]), ge.normalize(Se[0], Se[0]), ke[0] = ge.dot(Se[0], Se[1]), Ee(Se[1], Se[1], Se[0], 1, -ke[0]), Qe[1] = ge.length(Se[1]), ge.normalize(Se[1], Se[1]), ke[0] /= Qe[1], ke[1] = ge.dot(Se[0], Se[2]), Ee(Se[2], Se[2], Se[0], 1, -ke[1]), ke[2] = ge.dot(Se[1], Se[2]), Ee(Se[2], Se[2], Se[1], 1, -ke[2]), Qe[2] = ge.length(Se[2]), ge.normalize(Se[2], Se[2]), ke[1] /= Qe[2], ke[2] /= Qe[2], ge.cross(we, Se[1], Se[2]), ge.dot(Se[0], we) < 0) for (var Je = 0; Je < 3; Je++) Qe[Je] *= -1, Se[Je][0] *= -1, Se[Je][1] *= -1, Se[Je][2] *= -1;
                  return Re[0] = 0.5 * Math.sqrt(Math.max(1 + Se[0][0] - Se[1][1] - Se[2][2], 0)), Re[1] = 0.5 * Math.sqrt(Math.max(1 - Se[0][0] + Se[1][1] - Se[2][2], 0)), Re[2] = 0.5 * Math.sqrt(Math.max(1 - Se[0][0] - Se[1][1] + Se[2][2], 0)), Re[3] = 0.5 * Math.sqrt(Math.max(1 + Se[0][0] + Se[1][1] + Se[2][2], 0)), Se[2][1] > Se[1][2] && (Re[0] = -Re[0]), Se[0][2] > Se[2][0] && (Re[1] = -Re[1]), Se[1][0] > Se[0][1] && (Re[2] = -Re[2]), !0;
                };
              }, 4335: function(ce) {
                ce.exports = function(le, ue) {
                  var de = ue[15];
                  if (de === 0) return !1;
                  for (var fe = 1 / de, he = 0; he < 16; he++) le[he] = ue[he] * fe;
                  return !0;
                };
              }, 7442: function(ce, le, ue) {
                var de = ue(6658), fe = ue(7182), he = ue(2652), me = ue(9921), pe = ue(8648), ye = be(), ge = be(), ve = be();
                function be() {
                  return { translate: Ce(), scale: Ce(1), skew: Ce(), perspective: [0, 0, 0, 1], quaternion: [0, 0, 0, 1] };
                }
                function Ce(Se) {
                  return [Se || 0, Se || 0, Se || 0];
                }
                ce.exports = function(Se, we, Ee, xe) {
                  if (me(we) === 0 || me(Ee) === 0) return !1;
                  var Le = he(we, ye.translate, ye.scale, ye.skew, ye.perspective, ye.quaternion), Qe = he(Ee, ge.translate, ge.scale, ge.skew, ge.perspective, ge.quaternion);
                  return !(!Le || !Qe || (de(ve.translate, ye.translate, ge.translate, xe), de(ve.skew, ye.skew, ge.skew, xe), de(ve.scale, ye.scale, ge.scale, xe), de(ve.perspective, ye.perspective, ge.perspective, xe), pe(ve.quaternion, ye.quaternion, ge.quaternion, xe), fe(Se, ve.translate, ve.scale, ve.skew, ve.perspective, ve.quaternion), 0));
                };
              }, 7182: function(ce, le, ue) {
                var de = { identity: ue(7894), translate: ue(7656), multiply: ue(6760), create: ue(6864), scale: ue(2504), fromRotationTranslation: ue(6743) }, fe = (de.create(), de.create());
                ce.exports = function(he, me, pe, ye, ge, ve) {
                  return de.identity(he), de.fromRotationTranslation(he, ve, me), he[3] = ge[0], he[7] = ge[1], he[11] = ge[2], he[15] = ge[3], de.identity(fe), ye[2] !== 0 && (fe[9] = ye[2], de.multiply(he, he, fe)), ye[1] !== 0 && (fe[9] = 0, fe[8] = ye[1], de.multiply(he, he, fe)), ye[0] !== 0 && (fe[8] = 0, fe[4] = ye[0], de.multiply(he, he, fe)), de.scale(he, he, pe), he;
                };
              }, 4192: function(ce, le, ue) {
                var de = ue(2478), fe = ue(7442), he = ue(7608), me = ue(5567), pe = ue(2408), ye = ue(7089), ge = ue(6582), ve = ue(7656), be = (ue(2504), ue(3536)), Ce = [0, 0, 0];
                function Se(xe) {
                  this._components = xe.slice(), this._time = [0], this.prevMatrix = xe.slice(), this.nextMatrix = xe.slice(), this.computedMatrix = xe.slice(), this.computedInverse = xe.slice(), this.computedEye = [0, 0, 0], this.computedUp = [0, 0, 0], this.computedCenter = [0, 0, 0], this.computedRadius = [0], this._limits = [-1 / 0, 1 / 0];
                }
                ce.exports = function(xe) {
                  return new Se((xe = xe || {}).matrix || [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1]);
                };
                var we = Se.prototype;
                we.recalcMatrix = function(xe) {
                  var Le = this._time, Qe = de.le(Le, xe), ke = this.computedMatrix;
                  if (!(Qe < 0)) {
                    var Me = this._components;
                    if (Qe === Le.length - 1) for (var Re = 16 * Qe, De = 0; De < 16; ++De) ke[De] = Me[Re++];
                    else {
                      var Pe = Le[Qe + 1] - Le[Qe], $e = (Re = 16 * Qe, this.prevMatrix), qe = !0;
                      for (De = 0; De < 16; ++De) $e[De] = Me[Re++];
                      var Xe = this.nextMatrix;
                      for (De = 0; De < 16; ++De) Xe[De] = Me[Re++], qe = qe && $e[De] === Xe[De];
                      if (Pe < 1e-6 || qe) for (De = 0; De < 16; ++De) ke[De] = $e[De];
                      else fe(ke, $e, Xe, (xe - Le[Qe]) / Pe);
                    }
                    var Ve = this.computedUp;
                    Ve[0] = ke[1], Ve[1] = ke[5], Ve[2] = ke[9], be(Ve, Ve);
                    var Ze = this.computedInverse;
                    he(Ze, ke);
                    var Ge = this.computedEye, ze = Ze[15];
                    Ge[0] = Ze[12] / ze, Ge[1] = Ze[13] / ze, Ge[2] = Ze[14] / ze;
                    var We = this.computedCenter, at = Math.exp(this.computedRadius[0]);
                    for (De = 0; De < 3; ++De) We[De] = Ge[De] - ke[2 + 4 * De] * at;
                  }
                }, we.idle = function(xe) {
                  if (!(xe < this.lastT())) {
                    for (var Le = this._components, Qe = Le.length - 16, ke = 0; ke < 16; ++ke) Le.push(Le[Qe++]);
                    this._time.push(xe);
                  }
                }, we.flush = function(xe) {
                  var Le = de.gt(this._time, xe) - 2;
                  Le < 0 || (this._time.splice(0, Le), this._components.splice(0, 16 * Le));
                }, we.lastT = function() {
                  return this._time[this._time.length - 1];
                }, we.lookAt = function(xe, Le, Qe, ke) {
                  this.recalcMatrix(xe), Le = Le || this.computedEye, Qe = Qe || Ce, ke = ke || this.computedUp, this.setMatrix(xe, ge(this.computedMatrix, Le, Qe, ke));
                  for (var Me = 0, Re = 0; Re < 3; ++Re) Me += Math.pow(Qe[Re] - Le[Re], 2);
                  Me = Math.log(Math.sqrt(Me)), this.computedRadius[0] = Me;
                }, we.rotate = function(xe, Le, Qe, ke) {
                  this.recalcMatrix(xe);
                  var Me = this.computedInverse;
                  Le && pe(Me, Me, Le), Qe && me(Me, Me, Qe), ke && ye(Me, Me, ke), this.setMatrix(xe, he(this.computedMatrix, Me));
                };
                var Ee = [0, 0, 0];
                we.pan = function(xe, Le, Qe, ke) {
                  Ee[0] = -(Le || 0), Ee[1] = -(Qe || 0), Ee[2] = -(ke || 0), this.recalcMatrix(xe);
                  var Me = this.computedInverse;
                  ve(Me, Me, Ee), this.setMatrix(xe, he(Me, Me));
                }, we.translate = function(xe, Le, Qe, ke) {
                  Ee[0] = Le || 0, Ee[1] = Qe || 0, Ee[2] = ke || 0, this.recalcMatrix(xe);
                  var Me = this.computedMatrix;
                  ve(Me, Me, Ee), this.setMatrix(xe, Me);
                }, we.setMatrix = function(xe, Le) {
                  if (!(xe < this.lastT())) {
                    this._time.push(xe);
                    for (var Qe = 0; Qe < 16; ++Qe) this._components.push(Le[Qe]);
                  }
                }, we.setDistance = function(xe, Le) {
                  this.computedRadius[0] = Le;
                }, we.setDistanceLimits = function(xe, Le) {
                  var Qe = this._limits;
                  Qe[0] = xe, Qe[1] = Le;
                }, we.getDistanceLimits = function(xe) {
                  var Le = this._limits;
                  return xe ? (xe[0] = Le[0], xe[1] = Le[1], xe) : Le;
                };
              }, 3090: function(ce, le, ue) {
                ce.exports = function(fe) {
                  var he = fe.length;
                  if (he < 3) {
                    for (var me = new Array(he), pe = 0; pe < he; ++pe) me[pe] = pe;
                    return he === 2 && fe[0][0] === fe[1][0] && fe[0][1] === fe[1][1] ? [0] : me;
                  }
                  var ye = new Array(he);
                  for (pe = 0; pe < he; ++pe) ye[pe] = pe;
                  ye.sort(function(Le, Qe) {
                    return fe[Le][0] - fe[Qe][0] || fe[Le][1] - fe[Qe][1];
                  });
                  var ge = [ye[0], ye[1]], ve = [ye[0], ye[1]];
                  for (pe = 2; pe < he; ++pe) {
                    for (var be = ye[pe], Ce = fe[be], Se = ge.length; Se > 1 && de(fe[ge[Se - 2]], fe[ge[Se - 1]], Ce) <= 0; ) Se -= 1, ge.pop();
                    for (ge.push(be), Se = ve.length; Se > 1 && de(fe[ve[Se - 2]], fe[ve[Se - 1]], Ce) >= 0; ) Se -= 1, ve.pop();
                    ve.push(be);
                  }
                  me = new Array(ve.length + ge.length - 2);
                  for (var we = 0, Ee = (pe = 0, ge.length); pe < Ee; ++pe) me[we++] = ge[pe];
                  for (var xe = ve.length - 2; xe > 0; --xe) me[we++] = ve[xe];
                  return me;
                };
                var de = ue(3250)[3];
              }, 351: function(ce, le, ue) {
                ce.exports = function(fe, he) {
                  he || (he = fe, fe = window);
                  var me = 0, pe = 0, ye = 0, ge = { shift: !1, alt: !1, control: !1, meta: !1 }, ve = !1;
                  function be(Re) {
                    var De = !1;
                    return "altKey" in Re && (De = De || Re.altKey !== ge.alt, ge.alt = !!Re.altKey), "shiftKey" in Re && (De = De || Re.shiftKey !== ge.shift, ge.shift = !!Re.shiftKey), "ctrlKey" in Re && (De = De || Re.ctrlKey !== ge.control, ge.control = !!Re.ctrlKey), "metaKey" in Re && (De = De || Re.metaKey !== ge.meta, ge.meta = !!Re.metaKey), De;
                  }
                  function Ce(Re, De) {
                    var Pe = de.x(De), $e = de.y(De);
                    "buttons" in De && (Re = 0 | De.buttons), (Re !== me || Pe !== pe || $e !== ye || be(De)) && (me = 0 | Re, pe = Pe || 0, ye = $e || 0, he && he(me, pe, ye, ge));
                  }
                  function Se(Re) {
                    Ce(0, Re);
                  }
                  function we() {
                    (me || pe || ye || ge.shift || ge.alt || ge.meta || ge.control) && (pe = ye = 0, me = 0, ge.shift = ge.alt = ge.control = ge.meta = !1, he && he(0, 0, 0, ge));
                  }
                  function Ee(Re) {
                    be(Re) && he && he(me, pe, ye, ge);
                  }
                  function xe(Re) {
                    de.buttons(Re) === 0 ? Ce(0, Re) : Ce(me, Re);
                  }
                  function Le(Re) {
                    Ce(me | de.buttons(Re), Re);
                  }
                  function Qe(Re) {
                    Ce(me & ~de.buttons(Re), Re);
                  }
                  function ke() {
                    ve || (ve = !0, fe.addEventListener("mousemove", xe), fe.addEventListener("mousedown", Le), fe.addEventListener("mouseup", Qe), fe.addEventListener("mouseleave", Se), fe.addEventListener("mouseenter", Se), fe.addEventListener("mouseout", Se), fe.addEventListener("mouseover", Se), fe.addEventListener("blur", we), fe.addEventListener("keyup", Ee), fe.addEventListener("keydown", Ee), fe.addEventListener("keypress", Ee), fe !== window && (window.addEventListener("blur", we), window.addEventListener("keyup", Ee), window.addEventListener("keydown", Ee), window.addEventListener("keypress", Ee)));
                  }
                  ke();
                  var Me = { element: fe };
                  return Object.defineProperties(Me, { enabled: { get: function() {
                    return ve;
                  }, set: function(Re) {
                    Re ? ke() : ve && (ve = !1, fe.removeEventListener("mousemove", xe), fe.removeEventListener("mousedown", Le), fe.removeEventListener("mouseup", Qe), fe.removeEventListener("mouseleave", Se), fe.removeEventListener("mouseenter", Se), fe.removeEventListener("mouseout", Se), fe.removeEventListener("mouseover", Se), fe.removeEventListener("blur", we), fe.removeEventListener("keyup", Ee), fe.removeEventListener("keydown", Ee), fe.removeEventListener("keypress", Ee), fe !== window && (window.removeEventListener("blur", we), window.removeEventListener("keyup", Ee), window.removeEventListener("keydown", Ee), window.removeEventListener("keypress", Ee)));
                  }, enumerable: !0 }, buttons: { get: function() {
                    return me;
                  }, enumerable: !0 }, x: { get: function() {
                    return pe;
                  }, enumerable: !0 }, y: { get: function() {
                    return ye;
                  }, enumerable: !0 }, mods: { get: function() {
                    return ge;
                  }, enumerable: !0 } }), Me;
                };
                var de = ue(4687);
              }, 24: function(ce) {
                var le = { left: 0, top: 0 };
                ce.exports = function(ue, de, fe) {
                  de = de || ue.currentTarget || ue.srcElement, Array.isArray(fe) || (fe = [0, 0]);
                  var he, me = ue.clientX || 0, pe = ue.clientY || 0, ye = (he = de) === window || he === document || he === document.body ? le : he.getBoundingClientRect();
                  return fe[0] = me - ye.left, fe[1] = pe - ye.top, fe;
                };
              }, 4687: function(ce, le) {
                function ue(de) {
                  return de.target || de.srcElement || window;
                }
                le.buttons = function(de) {
                  if (typeof de == "object") {
                    if ("buttons" in de) return de.buttons;
                    if ("which" in de) {
                      if ((fe = de.which) === 2) return 4;
                      if (fe === 3) return 2;
                      if (fe > 0) return 1 << fe - 1;
                    } else if ("button" in de) {
                      var fe;
                      if ((fe = de.button) === 1) return 4;
                      if (fe === 2) return 2;
                      if (fe >= 0) return 1 << fe;
                    }
                  }
                  return 0;
                }, le.element = ue, le.x = function(de) {
                  if (typeof de == "object") {
                    if ("offsetX" in de) return de.offsetX;
                    var fe = ue(de).getBoundingClientRect();
                    return de.clientX - fe.left;
                  }
                  return 0;
                }, le.y = function(de) {
                  if (typeof de == "object") {
                    if ("offsetY" in de) return de.offsetY;
                    var fe = ue(de).getBoundingClientRect();
                    return de.clientY - fe.top;
                  }
                  return 0;
                };
              }, 8512: function(ce, le, ue) {
                var de = ue(665);
                ce.exports = function(fe, he, me) {
                  typeof fe == "function" && (me = !!he, he = fe, fe = window);
                  var pe = de("ex", fe), ye = function(ge) {
                    me && ge.preventDefault();
                    var ve = ge.deltaX || 0, be = ge.deltaY || 0, Ce = ge.deltaZ || 0, Se = 1;
                    switch (ge.deltaMode) {
                      case 1:
                        Se = pe;
                        break;
                      case 2:
                        Se = window.innerHeight;
                    }
                    if (be *= Se, Ce *= Se, (ve *= Se) || be || Ce) return he(ve, be, Ce, ge);
                  };
                  return fe.addEventListener("wheel", ye), ye;
                };
              }, 2640: function(ce, le, ue) {
                var de = ue(1888);
                ce.exports = function(he) {
                  function me(Ce) {
                    throw new Error("ndarray-extract-contour: " + Ce);
                  }
                  typeof he != "object" && me("Must specify arguments");
                  var pe = he.order;
                  Array.isArray(pe) || me("Must specify order");
                  var ye = he.arrayArguments || 1;
                  ye < 1 && me("Must have at least one array argument"), (he.scalarArguments || 0) < 0 && me("Scalar arg count must be > 0"), typeof he.vertex != "function" && me("Must specify vertex creation function"), typeof he.cell != "function" && me("Must specify cell creation function"), typeof he.phase != "function" && me("Must specify phase function");
                  for (var ge = he.getters || [], ve = new Array(ye), be = 0; be < ye; ++be) ge.indexOf(be) >= 0 ? ve[be] = !0 : ve[be] = !1;
                  return function(Ce, Se, we, Ee, xe, Le) {
                    var Qe = [Le, xe].join(",");
                    return (0, fe[Qe])(Ce, Se, we, de.mallocUint32, de.freeUint32);
                  }(he.vertex, he.cell, he.phase, 0, pe, ve);
                };
                var fe = { "false,0,1": function(he, me, pe, ye, ge) {
                  return function(ve, be, Ce, Se) {
                    var we, Ee = 0 | ve.shape[0], xe = 0 | ve.shape[1], Le = ve.data, Qe = 0 | ve.offset, ke = 0 | ve.stride[0], Me = 0 | ve.stride[1], Re = Qe, De = 0 | -ke, Pe = 0, $e = 0 | -Me, qe = 0, Xe = -ke - Me | 0, Ve = 0, Ze = 0 | ke, Ge = Me - ke * Ee | 0, ze = 0, We = 0, at = 0, ct = 2 * Ee | 0, et = ye(ct), dt = ye(ct), Je = 0, it = 0, st = -1, gt = -1, bt = 0, yt = 0 | -Ee, St = 0 | Ee, Lt = 0, Rt = -Ee - 1 | 0, vt = Ee - 1 | 0, $t = 0, Pt = 0, Xt = 0;
                    for (ze = 0; ze < Ee; ++ze) et[Je++] = pe(Le[Re], be, Ce, Se), Re += Ze;
                    if (Re += Ge, xe > 0) {
                      if (We = 1, et[Je++] = pe(Le[Re], be, Ce, Se), Re += Ze, Ee > 0) for (ze = 1, we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), bt = et[Je + st], Lt = et[Je + yt], $t = et[Je + Rt], it === bt && it === Lt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, bt, Lt, $t, be, Ce, Se), Pt = dt[Je] = at++), Je += 1, Re += Ze, ze = 2; ze < Ee; ++ze) we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), bt = et[Je + st], Lt = et[Je + yt], $t = et[Je + Rt], it === bt && it === Lt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, bt, Lt, $t, be, Ce, Se), Pt = dt[Je] = at++, $t !== bt && me(dt[Je + st], Pt, Ve, Pe, $t, bt, be, Ce, Se)), Je += 1, Re += Ze;
                      for (Re += Ge, Je = 0, Xt = st, st = gt, gt = Xt, Xt = yt, yt = St, St = Xt, Xt = Rt, Rt = vt, vt = Xt, We = 2; We < xe; ++We) {
                        if (et[Je++] = pe(Le[Re], be, Ce, Se), Re += Ze, Ee > 0) for (ze = 1, we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), bt = et[Je + st], Lt = et[Je + yt], $t = et[Je + Rt], it === bt && it === Lt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, bt, Lt, $t, be, Ce, Se), Pt = dt[Je] = at++, $t !== Lt && me(dt[Je + yt], Pt, qe, Ve, Lt, $t, be, Ce, Se)), Je += 1, Re += Ze, ze = 2; ze < Ee; ++ze) we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), bt = et[Je + st], Lt = et[Je + yt], $t = et[Je + Rt], it === bt && it === Lt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, bt, Lt, $t, be, Ce, Se), Pt = dt[Je] = at++, $t !== Lt && me(dt[Je + yt], Pt, qe, Ve, Lt, $t, be, Ce, Se), $t !== bt && me(dt[Je + st], Pt, Ve, Pe, $t, bt, be, Ce, Se)), Je += 1, Re += Ze;
                        1 & We && (Je = 0), Xt = st, st = gt, gt = Xt, Xt = yt, yt = St, St = Xt, Xt = Rt, Rt = vt, vt = Xt, Re += Ge;
                      }
                    }
                    ge(dt), ge(et);
                  };
                }, "false,1,0": function(he, me, pe, ye, ge) {
                  return function(ve, be, Ce, Se) {
                    var we, Ee = 0 | ve.shape[0], xe = 0 | ve.shape[1], Le = ve.data, Qe = 0 | ve.offset, ke = 0 | ve.stride[0], Me = 0 | ve.stride[1], Re = Qe, De = 0 | -ke, Pe = 0, $e = 0 | -Me, qe = 0, Xe = -ke - Me | 0, Ve = 0, Ze = 0 | Me, Ge = ke - Me * xe | 0, ze = 0, We = 0, at = 0, ct = 2 * xe | 0, et = ye(ct), dt = ye(ct), Je = 0, it = 0, st = -1, gt = -1, bt = 0, yt = 0 | -xe, St = 0 | xe, Lt = 0, Rt = -xe - 1 | 0, vt = xe - 1 | 0, $t = 0, Pt = 0, Xt = 0;
                    for (We = 0; We < xe; ++We) et[Je++] = pe(Le[Re], be, Ce, Se), Re += Ze;
                    if (Re += Ge, Ee > 0) {
                      if (ze = 1, et[Je++] = pe(Le[Re], be, Ce, Se), Re += Ze, xe > 0) for (We = 1, we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), Lt = et[Je + yt], bt = et[Je + st], $t = et[Je + Rt], it === Lt && it === bt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, Lt, bt, $t, be, Ce, Se), Pt = dt[Je] = at++), Je += 1, Re += Ze, We = 2; We < xe; ++We) we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), Lt = et[Je + yt], bt = et[Je + st], $t = et[Je + Rt], it === Lt && it === bt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, Lt, bt, $t, be, Ce, Se), Pt = dt[Je] = at++, $t !== bt && me(dt[Je + st], Pt, qe, Ve, bt, $t, be, Ce, Se)), Je += 1, Re += Ze;
                      for (Re += Ge, Je = 0, Xt = yt, yt = St, St = Xt, Xt = st, st = gt, gt = Xt, Xt = Rt, Rt = vt, vt = Xt, ze = 2; ze < Ee; ++ze) {
                        if (et[Je++] = pe(Le[Re], be, Ce, Se), Re += Ze, xe > 0) for (We = 1, we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), Lt = et[Je + yt], bt = et[Je + st], $t = et[Je + Rt], it === Lt && it === bt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, Lt, bt, $t, be, Ce, Se), Pt = dt[Je] = at++, $t !== Lt && me(dt[Je + yt], Pt, Ve, Pe, $t, Lt, be, Ce, Se)), Je += 1, Re += Ze, We = 2; We < xe; ++We) we = Le[Re], it = et[Je] = pe(we, be, Ce, Se), Lt = et[Je + yt], bt = et[Je + st], $t = et[Je + Rt], it === Lt && it === bt && it === $t || (Pe = Le[Re + De], qe = Le[Re + $e], Ve = Le[Re + Xe], he(ze, We, we, Pe, qe, Ve, it, Lt, bt, $t, be, Ce, Se), Pt = dt[Je] = at++, $t !== bt && me(dt[Je + st], Pt, qe, Ve, bt, $t, be, Ce, Se), $t !== Lt && me(dt[Je + yt], Pt, Ve, Pe, $t, Lt, be, Ce, Se)), Je += 1, Re += Ze;
                        1 & ze && (Je = 0), Xt = yt, yt = St, St = Xt, Xt = st, st = gt, gt = Xt, Xt = Rt, Rt = vt, vt = Xt, Re += Ge;
                      }
                    }
                    ge(dt), ge(et);
                  };
                } };
              }, 6199: function(ce, le, ue) {
                var de = ue(1338), fe = { zero: function(Ee, xe, Le, Qe) {
                  var ke = Ee[0];
                  Qe |= 0;
                  var Me = 0, Re = Le[0];
                  for (Me = 0; Me < ke; ++Me) xe[Qe] = 0, Qe += Re;
                }, fdTemplate1: function(Ee, xe, Le, Qe, ke, Me, Re) {
                  var De = Ee[0], Pe = Le[0], $e = -1 * Pe, qe = Pe;
                  Qe |= 0, Re |= 0;
                  var Xe = 0, Ve = Pe, Ze = Me[0];
                  for (Xe = 0; Xe < De; ++Xe) ke[Re] = 0.5 * (xe[Qe + $e] - xe[Qe + qe]), Qe += Ve, Re += Ze;
                }, fdTemplate2: function(Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e) {
                  var qe = Ee[0], Xe = Ee[1], Ve = Le[0], Ze = Le[1], Ge = Me[0], ze = Me[1], We = Pe[0], at = Pe[1], ct = -1 * Ve, et = Ve, dt = -1 * Ze, Je = Ze;
                  Qe |= 0, Re |= 0, $e |= 0;
                  var it = 0, st = 0, gt = Ze, bt = Ve - Xe * Ze, yt = ze, St = Ge - Xe * ze, Lt = at, Rt = We - Xe * at;
                  for (st = 0; st < qe; ++st) {
                    for (it = 0; it < Xe; ++it) ke[Re] = 0.5 * (xe[Qe + ct] - xe[Qe + et]), De[$e] = 0.5 * (xe[Qe + dt] - xe[Qe + Je]), Qe += gt, Re += yt, $e += Lt;
                    Qe += bt, Re += St, $e += Rt;
                  }
                } }, he = { cdiff: function(Ee) {
                  var xe = {};
                  return function(Le, Qe, ke) {
                    var Me = Le.dtype, Re = Le.order, De = Qe.dtype, Pe = Qe.order, $e = ke.dtype, qe = ke.order, Xe = [Me, Re.join(), De, Pe.join(), $e, qe.join()].join(), Ve = xe[Xe];
                    return Ve || (xe[Xe] = Ve = Ee([Me, Re, De, Pe, $e, qe])), Ve(Le.shape.slice(0), Le.data, Le.stride, 0 | Le.offset, Qe.data, Qe.stride, 0 | Qe.offset, ke.data, ke.stride, 0 | ke.offset);
                  };
                }, zero: function(Ee) {
                  var xe = {};
                  return function(Le) {
                    var Qe = Le.dtype, ke = Le.order, Me = [Qe, ke.join()].join(), Re = xe[Me];
                    return Re || (xe[Me] = Re = Ee([Qe, ke])), Re(Le.shape.slice(0), Le.data, Le.stride, 0 | Le.offset);
                  };
                }, fdTemplate1: function(Ee) {
                  var xe = {};
                  return function(Le, Qe) {
                    var ke = Le.dtype, Me = Le.order, Re = Qe.dtype, De = Qe.order, Pe = [ke, Me.join(), Re, De.join()].join(), $e = xe[Pe];
                    return $e || (xe[Pe] = $e = Ee([ke, Me, Re, De])), $e(Le.shape.slice(0), Le.data, Le.stride, 0 | Le.offset, Qe.data, Qe.stride, 0 | Qe.offset);
                  };
                }, fdTemplate2: function(Ee) {
                  var xe = {};
                  return function(Le, Qe, ke) {
                    var Me = Le.dtype, Re = Le.order, De = Qe.dtype, Pe = Qe.order, $e = ke.dtype, qe = ke.order, Xe = [Me, Re.join(), De, Pe.join(), $e, qe.join()].join(), Ve = xe[Xe];
                    return Ve || (xe[Xe] = Ve = Ee([Me, Re, De, Pe, $e, qe])), Ve(Le.shape.slice(0), Le.data, Le.stride, 0 | Le.offset, Qe.data, Qe.stride, 0 | Qe.offset, ke.data, ke.stride, 0 | ke.offset);
                  };
                } };
                function me(Ee) {
                  return (0, he[Ee.funcName])(pe.bind(void 0, Ee));
                }
                function pe(Ee) {
                  return fe[Ee.funcName];
                }
                function ye(Ee) {
                  return me({ funcName: Ee.funcName });
                }
                var ge = {}, ve = {}, be = ye({ funcName: "cdiff" }), Ce = ye({ funcName: "zero" });
                function Se(Ee) {
                  return Ee in ge ? ge[Ee] : ge[Ee] = ye({ funcName: "fdTemplate" + Ee });
                }
                function we(Ee, xe, Le, Qe) {
                  return function(ke, Me) {
                    var Re = Me.shape.slice();
                    return Re[0] > 2 && Re[1] > 2 && Qe(Me.pick(-1, -1).lo(1, 1).hi(Re[0] - 2, Re[1] - 2), ke.pick(-1, -1, 0).lo(1, 1).hi(Re[0] - 2, Re[1] - 2), ke.pick(-1, -1, 1).lo(1, 1).hi(Re[0] - 2, Re[1] - 2)), Re[1] > 2 && (Le(Me.pick(0, -1).lo(1).hi(Re[1] - 2), ke.pick(0, -1, 1).lo(1).hi(Re[1] - 2)), xe(ke.pick(0, -1, 0).lo(1).hi(Re[1] - 2))), Re[1] > 2 && (Le(Me.pick(Re[0] - 1, -1).lo(1).hi(Re[1] - 2), ke.pick(Re[0] - 1, -1, 1).lo(1).hi(Re[1] - 2)), xe(ke.pick(Re[0] - 1, -1, 0).lo(1).hi(Re[1] - 2))), Re[0] > 2 && (Le(Me.pick(-1, 0).lo(1).hi(Re[0] - 2), ke.pick(-1, 0, 0).lo(1).hi(Re[0] - 2)), xe(ke.pick(-1, 0, 1).lo(1).hi(Re[0] - 2))), Re[0] > 2 && (Le(Me.pick(-1, Re[1] - 1).lo(1).hi(Re[0] - 2), ke.pick(-1, Re[1] - 1, 0).lo(1).hi(Re[0] - 2)), xe(ke.pick(-1, Re[1] - 1, 1).lo(1).hi(Re[0] - 2))), ke.set(0, 0, 0, 0), ke.set(0, 0, 1, 0), ke.set(Re[0] - 1, 0, 0, 0), ke.set(Re[0] - 1, 0, 1, 0), ke.set(0, Re[1] - 1, 0, 0), ke.set(0, Re[1] - 1, 1, 0), ke.set(Re[0] - 1, Re[1] - 1, 0, 0), ke.set(Re[0] - 1, Re[1] - 1, 1, 0), ke;
                  };
                }
                ce.exports = function(Ee, xe, Le) {
                  return Array.isArray(Le) || (Le = de(xe.dimension, typeof Le == "string" ? Le : "clamp")), xe.size === 0 ? Ee : xe.dimension === 0 ? (Ee.set(0), Ee) : function(Qe) {
                    var ke = Qe.join();
                    if (Pe = ve[ke]) return Pe;
                    for (var Me = Qe.length, Re = [be, Ce], De = 1; De <= Me; ++De) Re.push(Se(De));
                    var Pe = we.apply(void 0, Re);
                    return ve[ke] = Pe, Pe;
                  }(Le)(Ee, xe);
                };
              }, 4317: function(ce) {
                function le(he, me) {
                  var pe = Math.floor(me), ye = me - pe, ge = 0 <= pe && pe < he.shape[0], ve = 0 <= pe + 1 && pe + 1 < he.shape[0];
                  return (1 - ye) * (ge ? +he.get(pe) : 0) + ye * (ve ? +he.get(pe + 1) : 0);
                }
                function ue(he, me, pe) {
                  var ye = Math.floor(me), ge = me - ye, ve = 0 <= ye && ye < he.shape[0], be = 0 <= ye + 1 && ye + 1 < he.shape[0], Ce = Math.floor(pe), Se = pe - Ce, we = 0 <= Ce && Ce < he.shape[1], Ee = 0 <= Ce + 1 && Ce + 1 < he.shape[1], xe = ve && we ? he.get(ye, Ce) : 0, Le = ve && Ee ? he.get(ye, Ce + 1) : 0;
                  return (1 - Se) * ((1 - ge) * xe + ge * (be && we ? he.get(ye + 1, Ce) : 0)) + Se * ((1 - ge) * Le + ge * (be && Ee ? he.get(ye + 1, Ce + 1) : 0));
                }
                function de(he, me, pe, ye) {
                  var ge = Math.floor(me), ve = me - ge, be = 0 <= ge && ge < he.shape[0], Ce = 0 <= ge + 1 && ge + 1 < he.shape[0], Se = Math.floor(pe), we = pe - Se, Ee = 0 <= Se && Se < he.shape[1], xe = 0 <= Se + 1 && Se + 1 < he.shape[1], Le = Math.floor(ye), Qe = ye - Le, ke = 0 <= Le && Le < he.shape[2], Me = 0 <= Le + 1 && Le + 1 < he.shape[2], Re = be && Ee && ke ? he.get(ge, Se, Le) : 0, De = be && xe && ke ? he.get(ge, Se + 1, Le) : 0, Pe = Ce && Ee && ke ? he.get(ge + 1, Se, Le) : 0, $e = Ce && xe && ke ? he.get(ge + 1, Se + 1, Le) : 0, qe = be && Ee && Me ? he.get(ge, Se, Le + 1) : 0, Xe = be && xe && Me ? he.get(ge, Se + 1, Le + 1) : 0;
                  return (1 - Qe) * ((1 - we) * ((1 - ve) * Re + ve * Pe) + we * ((1 - ve) * De + ve * $e)) + Qe * ((1 - we) * ((1 - ve) * qe + ve * (Ce && Ee && Me ? he.get(ge + 1, Se, Le + 1) : 0)) + we * ((1 - ve) * Xe + ve * (Ce && xe && Me ? he.get(ge + 1, Se + 1, Le + 1) : 0)));
                }
                function fe(he) {
                  var me, pe, ye = 0 | he.shape.length, ge = new Array(ye), ve = new Array(ye), be = new Array(ye), Ce = new Array(ye);
                  for (me = 0; me < ye; ++me) pe = +arguments[me + 1], ge[me] = Math.floor(pe), ve[me] = pe - ge[me], be[me] = 0 <= ge[me] && ge[me] < he.shape[me], Ce[me] = 0 <= ge[me] + 1 && ge[me] + 1 < he.shape[me];
                  var Se, we, Ee, xe = 0;
                  e: for (me = 0; me < 1 << ye; ++me) {
                    for (we = 1, Ee = he.offset, Se = 0; Se < ye; ++Se) if (me & 1 << Se) {
                      if (!Ce[Se]) continue e;
                      we *= ve[Se], Ee += he.stride[Se] * (ge[Se] + 1);
                    } else {
                      if (!be[Se]) continue e;
                      we *= 1 - ve[Se], Ee += he.stride[Se] * ge[Se];
                    }
                    xe += we * he.data[Ee];
                  }
                  return xe;
                }
                ce.exports = function(he, me, pe, ye) {
                  switch (he.shape.length) {
                    case 0:
                      return 0;
                    case 1:
                      return le(he, me);
                    case 2:
                      return ue(he, me, pe);
                    case 3:
                      return de(he, me, pe, ye);
                    default:
                      return fe.apply(void 0, arguments);
                  }
                }, ce.exports.d1 = le, ce.exports.d2 = ue, ce.exports.d3 = de;
              }, 5298: function(ce, le) {
                var ue = { "float64,2,1,0": function() {
                  return function(pe, ye, ge, ve, be) {
                    var Ce = pe[0], Se = pe[1], we = pe[2], Ee = ge[0], xe = ge[1], Le = ge[2];
                    ve |= 0;
                    var Qe = 0, ke = 0, Me = 0, Re = Le, De = xe - we * Le, Pe = Ee - Se * xe;
                    for (Me = 0; Me < Ce; ++Me) {
                      for (ke = 0; ke < Se; ++ke) {
                        for (Qe = 0; Qe < we; ++Qe) ye[ve] /= be, ve += Re;
                        ve += De;
                      }
                      ve += Pe;
                    }
                  };
                }, "uint8,2,0,1,float64,2,1,0": function() {
                  return function(pe, ye, ge, ve, be, Ce, Se, we) {
                    for (var Ee = pe[0], xe = pe[1], Le = pe[2], Qe = ge[0], ke = ge[1], Me = ge[2], Re = Ce[0], De = Ce[1], Pe = Ce[2], $e = ve |= 0, qe = Se |= 0, Xe = 0 | pe[0]; Xe > 0; ) {
                      Xe < 64 ? (Ee = Xe, Xe = 0) : (Ee = 64, Xe -= 64);
                      for (var Ve = 0 | pe[1]; Ve > 0; ) {
                        Ve < 64 ? (xe = Ve, Ve = 0) : (xe = 64, Ve -= 64), ve = $e + Xe * Qe + Ve * ke, Se = qe + Xe * Re + Ve * De;
                        var Ze = 0, Ge = 0, ze = 0, We = Me, at = Qe - Le * Me, ct = ke - Ee * Qe, et = Pe, dt = Re - Le * Pe, Je = De - Ee * Re;
                        for (ze = 0; ze < xe; ++ze) {
                          for (Ge = 0; Ge < Ee; ++Ge) {
                            for (Ze = 0; Ze < Le; ++Ze) ye[ve] = be[Se] * we, ve += We, Se += et;
                            ve += at, Se += dt;
                          }
                          ve += ct, Se += Je;
                        }
                      }
                    }
                  };
                }, "float32,1,0,float32,1,0": function() {
                  return function(pe, ye, ge, ve, be, Ce, Se) {
                    var we = pe[0], Ee = pe[1], xe = ge[0], Le = ge[1], Qe = Ce[0], ke = Ce[1];
                    ve |= 0, Se |= 0;
                    var Me = 0, Re = 0, De = Le, Pe = xe - Ee * Le, $e = ke, qe = Qe - Ee * ke;
                    for (Re = 0; Re < we; ++Re) {
                      for (Me = 0; Me < Ee; ++Me) ye[ve] = be[Se], ve += De, Se += $e;
                      ve += Pe, Se += qe;
                    }
                  };
                }, "float32,1,0,float32,0,1": function() {
                  return function(pe, ye, ge, ve, be, Ce, Se) {
                    for (var we = pe[0], Ee = pe[1], xe = ge[0], Le = ge[1], Qe = Ce[0], ke = Ce[1], Me = ve |= 0, Re = Se |= 0, De = 0 | pe[1]; De > 0; ) {
                      De < 64 ? (Ee = De, De = 0) : (Ee = 64, De -= 64);
                      for (var Pe = 0 | pe[0]; Pe > 0; ) {
                        Pe < 64 ? (we = Pe, Pe = 0) : (we = 64, Pe -= 64), ve = Me + De * Le + Pe * xe, Se = Re + De * ke + Pe * Qe;
                        var $e = 0, qe = 0, Xe = Le, Ve = xe - Ee * Le, Ze = ke, Ge = Qe - Ee * ke;
                        for (qe = 0; qe < we; ++qe) {
                          for ($e = 0; $e < Ee; ++$e) ye[ve] = be[Se], ve += Xe, Se += Ze;
                          ve += Ve, Se += Ge;
                        }
                      }
                    }
                  };
                }, "uint8,2,0,1,uint8,1,2,0": function() {
                  return function(pe, ye, ge, ve, be, Ce, Se) {
                    for (var we = pe[0], Ee = pe[1], xe = pe[2], Le = ge[0], Qe = ge[1], ke = ge[2], Me = Ce[0], Re = Ce[1], De = Ce[2], Pe = ve |= 0, $e = Se |= 0, qe = 0 | pe[2]; qe > 0; ) {
                      qe < 64 ? (xe = qe, qe = 0) : (xe = 64, qe -= 64);
                      for (var Xe = 0 | pe[0]; Xe > 0; ) {
                        Xe < 64 ? (we = Xe, Xe = 0) : (we = 64, Xe -= 64);
                        for (var Ve = 0 | pe[1]; Ve > 0; ) {
                          Ve < 64 ? (Ee = Ve, Ve = 0) : (Ee = 64, Ve -= 64), ve = Pe + qe * ke + Xe * Le + Ve * Qe, Se = $e + qe * De + Xe * Me + Ve * Re;
                          var Ze = 0, Ge = 0, ze = 0, We = ke, at = Le - xe * ke, ct = Qe - we * Le, et = De, dt = Me - xe * De, Je = Re - we * Me;
                          for (ze = 0; ze < Ee; ++ze) {
                            for (Ge = 0; Ge < we; ++Ge) {
                              for (Ze = 0; Ze < xe; ++Ze) ye[ve] = be[Se], ve += We, Se += et;
                              ve += at, Se += dt;
                            }
                            ve += ct, Se += Je;
                          }
                        }
                      }
                    }
                  };
                }, "uint8,2,0,1,array,2,0,1": function() {
                  return function(pe, ye, ge, ve, be, Ce, Se) {
                    var we = pe[0], Ee = pe[1], xe = pe[2], Le = ge[0], Qe = ge[1], ke = ge[2], Me = Ce[0], Re = Ce[1], De = Ce[2];
                    ve |= 0, Se |= 0;
                    var Pe = 0, $e = 0, qe = 0, Xe = ke, Ve = Le - xe * ke, Ze = Qe - we * Le, Ge = De, ze = Me - xe * De, We = Re - we * Me;
                    for (qe = 0; qe < Ee; ++qe) {
                      for ($e = 0; $e < we; ++$e) {
                        for (Pe = 0; Pe < xe; ++Pe) ye[ve] = be[Se], ve += Xe, Se += Ge;
                        ve += Ve, Se += ze;
                      }
                      ve += Ze, Se += We;
                    }
                  };
                } }, de = function(pe, ye) {
                  var ge = ye.join(",");
                  return (0, ue[ge])();
                }, fe = { mul: function(pe) {
                  var ye = {};
                  return function(ge, ve, be) {
                    var Ce = ge.dtype, Se = ge.order, we = ve.dtype, Ee = ve.order, xe = be.dtype, Le = be.order, Qe = [Ce, Se.join(), we, Ee.join(), xe, Le.join()].join(), ke = ye[Qe];
                    return ke || (ye[Qe] = ke = pe([Ce, Se, we, Ee, xe, Le])), ke(ge.shape.slice(0), ge.data, ge.stride, 0 | ge.offset, ve.data, ve.stride, 0 | ve.offset, be.data, be.stride, 0 | be.offset);
                  };
                }, muls: function(pe) {
                  var ye = {};
                  return function(ge, ve, be) {
                    var Ce = ge.dtype, Se = ge.order, we = ve.dtype, Ee = ve.order, xe = [Ce, Se.join(), we, Ee.join()].join(), Le = ye[xe];
                    return Le || (ye[xe] = Le = pe([Ce, Se, we, Ee])), Le(ge.shape.slice(0), ge.data, ge.stride, 0 | ge.offset, ve.data, ve.stride, 0 | ve.offset, be);
                  };
                }, mulseq: function(pe) {
                  var ye = {};
                  return function(ge, ve) {
                    var be = ge.dtype, Ce = ge.order, Se = [be, Ce.join()].join(), we = ye[Se];
                    return we || (ye[Se] = we = pe([be, Ce])), we(ge.shape.slice(0), ge.data, ge.stride, 0 | ge.offset, ve);
                  };
                }, div: function(pe) {
                  var ye = {};
                  return function(ge, ve, be) {
                    var Ce = ge.dtype, Se = ge.order, we = ve.dtype, Ee = ve.order, xe = be.dtype, Le = be.order, Qe = [Ce, Se.join(), we, Ee.join(), xe, Le.join()].join(), ke = ye[Qe];
                    return ke || (ye[Qe] = ke = pe([Ce, Se, we, Ee, xe, Le])), ke(ge.shape.slice(0), ge.data, ge.stride, 0 | ge.offset, ve.data, ve.stride, 0 | ve.offset, be.data, be.stride, 0 | be.offset);
                  };
                }, divs: function(pe) {
                  var ye = {};
                  return function(ge, ve, be) {
                    var Ce = ge.dtype, Se = ge.order, we = ve.dtype, Ee = ve.order, xe = [Ce, Se.join(), we, Ee.join()].join(), Le = ye[xe];
                    return Le || (ye[xe] = Le = pe([Ce, Se, we, Ee])), Le(ge.shape.slice(0), ge.data, ge.stride, 0 | ge.offset, ve.data, ve.stride, 0 | ve.offset, be);
                  };
                }, divseq: function(pe) {
                  var ye = {};
                  return function(ge, ve) {
                    var be = ge.dtype, Ce = ge.order, Se = [be, Ce.join()].join(), we = ye[Se];
                    return we || (ye[Se] = we = pe([be, Ce])), we(ge.shape.slice(0), ge.data, ge.stride, 0 | ge.offset, ve);
                  };
                }, assign: function(pe) {
                  var ye = {};
                  return function(ge, ve) {
                    var be = ge.dtype, Ce = ge.order, Se = ve.dtype, we = ve.order, Ee = [be, Ce.join(), Se, we.join()].join(), xe = ye[Ee];
                    return xe || (ye[Ee] = xe = pe([be, Ce, Se, we])), xe(ge.shape.slice(0), ge.data, ge.stride, 0 | ge.offset, ve.data, ve.stride, 0 | ve.offset);
                  };
                } };
                function he(pe) {
                  return ye = { funcName: pe.funcName }, (0, fe[ye.funcName])(de.bind(void 0, ye));
                  var ye;
                }
                var me = { mul: "*", div: "/" };
                (function() {
                  for (var pe in me) le[pe] = he({ funcName: pe }), le[pe + "s"] = he({ funcName: pe + "s" }), le[pe + "seq"] = he({ funcName: pe + "seq" });
                })(), le.assign = he({ funcName: "assign" });
              }, 9994: function(ce, le, ue) {
                var de = ue(9618), fe = ue(8277);
                ce.exports = function(he, me) {
                  for (var pe = [], ye = he, ge = 1; Array.isArray(ye); ) pe.push(ye.length), ge *= ye.length, ye = ye[0];
                  return pe.length === 0 ? de() : (me || (me = de(new Float64Array(ge), pe)), fe(me, he), me);
                };
              }, 8277: function(ce) {
                ce.exports = /* @__PURE__ */ function(le) {
                  var ue = {};
                  return function(de, fe) {
                    var he = de.dtype, me = de.order, pe = [he, me.join()].join(), ye = ue[pe];
                    return ye || (ue[pe] = ye = le([he, me])), ye(de.shape.slice(0), de.data, de.stride, 0 | de.offset, fe);
                  };
                }((function() {
                  return function(le, ue, de, fe, he) {
                    var me = le[0], pe = le[1], ye = le[2], ge = de[0], ve = de[1], be = de[2], Ce = [0, 0, 0];
                    fe |= 0;
                    var Se = 0, we = 0, Ee = 0, xe = be, Le = ve - ye * be, Qe = ge - pe * ve;
                    for (Ee = 0; Ee < me; ++Ee) {
                      for (we = 0; we < pe; ++we) {
                        for (Se = 0; Se < ye; ++Se) {
                          var ke, Me = he;
                          for (ke = 0; ke < Ce.length - 1; ++ke) Me = Me[Ce[ke]];
                          ue[fe] = Me[Ce[Ce.length - 1]], fe += xe, ++Ce[2];
                        }
                        fe += Le, Ce[2] -= ye, ++Ce[1];
                      }
                      fe += Qe, Ce[1] -= pe, ++Ce[0];
                    }
                  };
                }).bind(void 0, { funcName: "convert" }));
              }, 7640: function(ce, le, ue) {
                var de = ue(1888);
                function fe(ye) {
                  return ye === "uint32" ? [de.mallocUint32, de.freeUint32] : null;
                }
                var he = { "uint32,1,0": function(ye, ge) {
                  return function(ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me) {
                    var Re, De, Pe, $e, qe, Xe, Ve, Ze, Ge = ve * we + Se, ze = ye(Le);
                    for (Re = ve + 1; Re <= be; ++Re) {
                      for (De = Re, Pe = Ge += we, qe = 0, Xe = Ge, $e = 0; $e < Le; ++$e) ze[qe++] = Ce[Xe], Xe += Qe;
                      e: for (; De-- > ve; ) {
                        qe = 0, Xe = Pe - we;
                        t: for ($e = 0; $e < Le; ++$e) {
                          if ((Ve = Ce[Xe]) < (Ze = ze[qe])) break e;
                          if (Ve > Ze) break t;
                          Xe += ke, qe += Me;
                        }
                        for (qe = Pe, Xe = Pe - we, $e = 0; $e < Le; ++$e) Ce[qe] = Ce[Xe], qe += Qe, Xe += Qe;
                        Pe -= we;
                      }
                      for (qe = Pe, Xe = 0, $e = 0; $e < Le; ++$e) Ce[qe] = ze[Xe++], qe += Qe;
                    }
                    ge(ze);
                  };
                } }, me = { "uint32,1,0": function(ye, ge, ve) {
                  return function be(Ce, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De) {
                    var Pe, $e, qe, Xe, Ve, Ze, Ge, ze, We, at, ct, et, dt, Je, it, st, gt, bt, yt, St, Lt, Rt, vt, $t, Pt, Xt = (Se - Ce + 1) / 6 | 0, Nt = Ce + Xt, Gt = Se - Xt, Ft = Ce + Se >> 1, Tt = Ft - Xt, It = Ft + Xt, Ct = Nt, Kt = Tt, Vt = Ft, Yt = It, sn = Gt, hn = Ce + 1, Qt = Se - 1, Ut = !0, Wt = 0, rn = 0, vn = 0, dn = ke, Ht = ge(dn), jt = ge(dn);
                    ct = xe * Ct, et = xe * Kt, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Ct, Ct = Kt, Kt = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Yt, et = xe * sn, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Yt, Yt = sn, sn = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Ct, et = xe * Vt, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Ct, Ct = Vt, Vt = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Kt, et = xe * Vt, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Kt, Kt = Vt, Vt = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Ct, et = xe * Yt, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Ct, Ct = Yt, Yt = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Vt, et = xe * Yt, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Vt, Vt = Yt, Yt = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Kt, et = xe * sn, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Kt, Kt = sn, sn = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Kt, et = xe * Vt, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Kt, Kt = Vt, Vt = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    ct = xe * Yt, et = xe * sn, Pt = Ee;
                    e: for (at = 0; at < ke; ++at) {
                      if (ze = et + Pt, (vn = we[Ge = ct + Pt] - we[ze]) > 0) {
                        $e = Yt, Yt = sn, sn = $e;
                        break e;
                      }
                      if (vn < 0) break e;
                      Pt += Re;
                    }
                    for (ct = xe * Ct, et = xe * Kt, dt = xe * Vt, Je = xe * Yt, it = xe * sn, st = xe * Nt, gt = xe * Ft, bt = xe * Gt, $t = 0, Pt = Ee, at = 0; at < ke; ++at) Ge = ct + Pt, ze = et + Pt, We = dt + Pt, yt = Je + Pt, St = it + Pt, Lt = st + Pt, Rt = gt + Pt, vt = bt + Pt, Ht[$t] = we[ze], jt[$t] = we[yt], Ut = Ut && Ht[$t] === jt[$t], qe = we[Ge], Xe = we[We], Ve = we[St], we[Lt] = qe, we[Rt] = Xe, we[vt] = Ve, ++$t, Pt += Me;
                    for (ct = xe * Tt, et = xe * Ce, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, we[Ge = ct + Pt] = we[ze], Pt += Me;
                    for (ct = xe * It, et = xe * Se, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, we[Ge = ct + Pt] = we[ze], Pt += Me;
                    if (Ut) for (Ze = hn; Ze <= Qt; ++Ze) {
                      for (Ge = Ee + Ze * xe, $t = 0, at = 0; at < ke && (vn = we[Ge] - Ht[$t]) == 0; ++at) $t += De, Ge += Re;
                      if (vn !== 0) if (vn < 0) {
                        if (Ze !== hn) for (ct = xe * Ze, et = xe * hn, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = Pe, Pt += Me;
                        ++hn;
                      } else for (; ; ) {
                        for (Ge = Ee + Qt * xe, $t = 0, at = 0; at < ke && (vn = we[Ge] - Ht[$t]) == 0; ++at) $t += De, Ge += Re;
                        if (!(vn > 0)) {
                          if (vn < 0) {
                            for (ct = xe * Ze, et = xe * hn, dt = xe * Qt, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, We = dt + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = we[We], we[We] = Pe, Pt += Me;
                            ++hn, --Qt;
                            break;
                          }
                          for (ct = xe * Ze, et = xe * Qt, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = Pe, Pt += Me;
                          --Qt;
                          break;
                        }
                        Qt--;
                      }
                    }
                    else for (Ze = hn; Ze <= Qt; ++Ze) {
                      for (Ge = Ee + Ze * xe, $t = 0, at = 0; at < ke && (Wt = we[Ge] - Ht[$t]) == 0; ++at) $t += De, Ge += Re;
                      if (Wt < 0) {
                        if (Ze !== hn) for (ct = xe * Ze, et = xe * hn, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = Pe, Pt += Me;
                        ++hn;
                      } else {
                        for (Ge = Ee + Ze * xe, $t = 0, at = 0; at < ke && (rn = we[Ge] - jt[$t]) == 0; ++at) $t += De, Ge += Re;
                        if (rn > 0) for (; ; ) {
                          for (Ge = Ee + Qt * xe, $t = 0, at = 0; at < ke && (vn = we[Ge] - jt[$t]) == 0; ++at) $t += De, Ge += Re;
                          if (!(vn > 0)) {
                            for (Ge = Ee + Qt * xe, $t = 0, at = 0; at < ke && (vn = we[Ge] - Ht[$t]) == 0; ++at) $t += De, Ge += Re;
                            if (vn < 0) {
                              for (ct = xe * Ze, et = xe * hn, dt = xe * Qt, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, We = dt + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = we[We], we[We] = Pe, Pt += Me;
                              ++hn, --Qt;
                            } else {
                              for (ct = xe * Ze, et = xe * Qt, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = Pe, Pt += Me;
                              --Qt;
                            }
                            break;
                          }
                          if (--Qt < Ze) break;
                        }
                      }
                    }
                    for (ct = xe * Ce, et = xe * (hn - 1), $t = 0, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, we[Ge = ct + Pt] = we[ze], we[ze] = Ht[$t], ++$t, Pt += Me;
                    for (ct = xe * Se, et = xe * (Qt + 1), $t = 0, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, we[Ge = ct + Pt] = we[ze], we[ze] = jt[$t], ++$t, Pt += Me;
                    if (hn - 2 - Ce <= 32 ? ye(Ce, hn - 2, we, Ee, xe, Le, Qe, ke, Me, Re, De) : be(Ce, hn - 2, we, Ee, xe, Le, Qe, ke, Me, Re, De), Se - (Qt + 2) <= 32 ? ye(Qt + 2, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De) : be(Qt + 2, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De), Ut) return ve(Ht), void ve(jt);
                    if (hn < Nt && Qt > Gt) {
                      e: for (; ; ) {
                        for (Ge = Ee + hn * xe, $t = 0, Pt = Ee, at = 0; at < ke; ++at) {
                          if (we[Ge] !== Ht[$t]) break e;
                          ++$t, Ge += Me;
                        }
                        ++hn;
                      }
                      e: for (; ; ) {
                        for (Ge = Ee + Qt * xe, $t = 0, Pt = Ee, at = 0; at < ke; ++at) {
                          if (we[Ge] !== jt[$t]) break e;
                          ++$t, Ge += Me;
                        }
                        --Qt;
                      }
                      for (Ze = hn; Ze <= Qt; ++Ze) {
                        for (Ge = Ee + Ze * xe, $t = 0, at = 0; at < ke && (Wt = we[Ge] - Ht[$t]) == 0; ++at) $t += De, Ge += Re;
                        if (Wt === 0) {
                          if (Ze !== hn) for (ct = xe * Ze, et = xe * hn, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = Pe, Pt += Me;
                          ++hn;
                        } else {
                          for (Ge = Ee + Ze * xe, $t = 0, at = 0; at < ke && (rn = we[Ge] - jt[$t]) == 0; ++at) $t += De, Ge += Re;
                          if (rn === 0) for (; ; ) {
                            for (Ge = Ee + Qt * xe, $t = 0, at = 0; at < ke && (vn = we[Ge] - jt[$t]) == 0; ++at) $t += De, Ge += Re;
                            if (vn !== 0) {
                              for (Ge = Ee + Qt * xe, $t = 0, at = 0; at < ke && (vn = we[Ge] - Ht[$t]) == 0; ++at) $t += De, Ge += Re;
                              if (vn < 0) {
                                for (ct = xe * Ze, et = xe * hn, dt = xe * Qt, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, We = dt + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = we[We], we[We] = Pe, Pt += Me;
                                ++hn, --Qt;
                              } else {
                                for (ct = xe * Ze, et = xe * Qt, Pt = Ee, at = 0; at < ke; ++at) ze = et + Pt, Pe = we[Ge = ct + Pt], we[Ge] = we[ze], we[ze] = Pe, Pt += Me;
                                --Qt;
                              }
                              break;
                            }
                            if (--Qt < Ze) break;
                          }
                        }
                      }
                    }
                    ve(Ht), ve(jt), Qt - hn <= 32 ? ye(hn, Qt, we, Ee, xe, Le, Qe, ke, Me, Re, De) : be(hn, Qt, we, Ee, xe, Le, Qe, ke, Me, Re, De);
                  };
                } }, pe = { "uint32,1,0": function(ye, ge) {
                  return function(ve) {
                    var be = ve.data, Ce = 0 | ve.offset, Se = ve.shape, we = ve.stride, Ee = 0 | we[0], xe = 0 | Se[0], Le = 0 | we[1], Qe = 0 | Se[1], ke = Le, Me = Le;
                    xe <= 32 ? ye(0, xe - 1, be, Ce, Ee, Le, xe, Qe, ke, Me, 1) : ge(0, xe - 1, be, Ce, Ee, Le, xe, Qe, ke, Me, 1);
                  };
                } };
                ce.exports = function(ye, ge) {
                  var ve = [ge, ye].join(","), be = pe[ve], Ce = function(we, Ee) {
                    var xe = fe(Ee), Le = [Ee, we].join(","), Qe = he[Le];
                    return xe ? Qe(xe[0], xe[1]) : Qe();
                  }(ye, ge), Se = function(we, Ee, xe) {
                    var Le = fe(Ee), Qe = [Ee, we].join(","), ke = me[Qe];
                    return we.length > 1 && Le ? ke(xe, Le[0], Le[1]) : ke(xe);
                  }(ye, ge, Ce);
                  return be(Ce, Se);
                };
              }, 446: function(ce, le, ue) {
                var de = ue(7640), fe = {};
                ce.exports = function(he) {
                  var me = he.order, pe = he.dtype, ye = [me, pe].join(":"), ge = fe[ye];
                  return ge || (fe[ye] = ge = de(me, pe)), ge(he), he;
                };
              }, 9618: function(ce, le, ue) {
                var de = ue(7163), fe = typeof Float64Array < "u";
                function he(ve, be) {
                  return ve[0] - be[0];
                }
                function me() {
                  var ve, be = this.stride, Ce = new Array(be.length);
                  for (ve = 0; ve < Ce.length; ++ve) Ce[ve] = [Math.abs(be[ve]), ve];
                  Ce.sort(he);
                  var Se = new Array(Ce.length);
                  for (ve = 0; ve < Se.length; ++ve) Se[ve] = Ce[ve][1];
                  return Se;
                }
                var pe = { T: function(ve) {
                  function be(Se) {
                    this.data = Se;
                  }
                  var Ce = be.prototype;
                  return Ce.dtype = ve, Ce.index = function() {
                    return -1;
                  }, Ce.size = 0, Ce.dimension = -1, Ce.shape = Ce.stride = Ce.order = [], Ce.lo = Ce.hi = Ce.transpose = Ce.step = function() {
                    return new be(this.data);
                  }, Ce.get = Ce.set = function() {
                  }, Ce.pick = function() {
                    return null;
                  }, function(Se) {
                    return new be(Se);
                  };
                }, 0: function(ve, be) {
                  function Ce(we, Ee) {
                    this.data = we, this.offset = Ee;
                  }
                  var Se = Ce.prototype;
                  return Se.dtype = ve, Se.index = function() {
                    return this.offset;
                  }, Se.dimension = 0, Se.size = 1, Se.shape = Se.stride = Se.order = [], Se.lo = Se.hi = Se.transpose = Se.step = function() {
                    return new Ce(this.data, this.offset);
                  }, Se.pick = function() {
                    return be(this.data);
                  }, Se.valueOf = Se.get = function() {
                    return ve === "generic" ? this.data.get(this.offset) : this.data[this.offset];
                  }, Se.set = function(we) {
                    return ve === "generic" ? this.data.set(this.offset, we) : this.data[this.offset] = we;
                  }, function(we, Ee, xe, Le) {
                    return new Ce(we, Le);
                  };
                }, 1: function(ve, be, Ce) {
                  function Se(Ee, xe, Le, Qe) {
                    this.data = Ee, this.shape = [xe], this.stride = [Le], this.offset = 0 | Qe;
                  }
                  var we = Se.prototype;
                  return we.dtype = ve, we.dimension = 1, Object.defineProperty(we, "size", { get: function() {
                    return this.shape[0];
                  } }), we.order = [0], we.set = function(Ee, xe) {
                    return ve === "generic" ? this.data.set(this.offset + this.stride[0] * Ee, xe) : this.data[this.offset + this.stride[0] * Ee] = xe;
                  }, we.get = function(Ee) {
                    return ve === "generic" ? this.data.get(this.offset + this.stride[0] * Ee) : this.data[this.offset + this.stride[0] * Ee];
                  }, we.index = function(Ee) {
                    return this.offset + this.stride[0] * Ee;
                  }, we.hi = function(Ee) {
                    return new Se(this.data, typeof Ee != "number" || Ee < 0 ? this.shape[0] : 0 | Ee, this.stride[0], this.offset);
                  }, we.lo = function(Ee) {
                    var xe = this.offset, Le = 0, Qe = this.shape[0], ke = this.stride[0];
                    return typeof Ee == "number" && Ee >= 0 && (xe += ke * (Le = 0 | Ee), Qe -= Le), new Se(this.data, Qe, ke, xe);
                  }, we.step = function(Ee) {
                    var xe = this.shape[0], Le = this.stride[0], Qe = this.offset, ke = 0, Me = Math.ceil;
                    return typeof Ee == "number" && ((ke = 0 | Ee) < 0 ? (Qe += Le * (xe - 1), xe = Me(-xe / ke)) : xe = Me(xe / ke), Le *= ke), new Se(this.data, xe, Le, Qe);
                  }, we.transpose = function(Ee) {
                    Ee = Ee === void 0 ? 0 : 0 | Ee;
                    var xe = this.shape, Le = this.stride;
                    return new Se(this.data, xe[Ee], Le[Ee], this.offset);
                  }, we.pick = function(Ee) {
                    var xe = [], Le = [], Qe = this.offset;
                    return typeof Ee == "number" && Ee >= 0 ? Qe = Qe + this.stride[0] * Ee | 0 : (xe.push(this.shape[0]), Le.push(this.stride[0])), (0, be[xe.length + 1])(this.data, xe, Le, Qe);
                  }, function(Ee, xe, Le, Qe) {
                    return new Se(Ee, xe[0], Le[0], Qe);
                  };
                }, 2: function(ve, be, Ce) {
                  function Se(Ee, xe, Le, Qe, ke, Me) {
                    this.data = Ee, this.shape = [xe, Le], this.stride = [Qe, ke], this.offset = 0 | Me;
                  }
                  var we = Se.prototype;
                  return we.dtype = ve, we.dimension = 2, Object.defineProperty(we, "size", { get: function() {
                    return this.shape[0] * this.shape[1];
                  } }), Object.defineProperty(we, "order", { get: function() {
                    return Math.abs(this.stride[0]) > Math.abs(this.stride[1]) ? [1, 0] : [0, 1];
                  } }), we.set = function(Ee, xe, Le) {
                    return ve === "generic" ? this.data.set(this.offset + this.stride[0] * Ee + this.stride[1] * xe, Le) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe] = Le;
                  }, we.get = function(Ee, xe) {
                    return ve === "generic" ? this.data.get(this.offset + this.stride[0] * Ee + this.stride[1] * xe) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe];
                  }, we.index = function(Ee, xe) {
                    return this.offset + this.stride[0] * Ee + this.stride[1] * xe;
                  }, we.hi = function(Ee, xe) {
                    return new Se(this.data, typeof Ee != "number" || Ee < 0 ? this.shape[0] : 0 | Ee, typeof xe != "number" || xe < 0 ? this.shape[1] : 0 | xe, this.stride[0], this.stride[1], this.offset);
                  }, we.lo = function(Ee, xe) {
                    var Le = this.offset, Qe = 0, ke = this.shape[0], Me = this.shape[1], Re = this.stride[0], De = this.stride[1];
                    return typeof Ee == "number" && Ee >= 0 && (Le += Re * (Qe = 0 | Ee), ke -= Qe), typeof xe == "number" && xe >= 0 && (Le += De * (Qe = 0 | xe), Me -= Qe), new Se(this.data, ke, Me, Re, De, Le);
                  }, we.step = function(Ee, xe) {
                    var Le = this.shape[0], Qe = this.shape[1], ke = this.stride[0], Me = this.stride[1], Re = this.offset, De = 0, Pe = Math.ceil;
                    return typeof Ee == "number" && ((De = 0 | Ee) < 0 ? (Re += ke * (Le - 1), Le = Pe(-Le / De)) : Le = Pe(Le / De), ke *= De), typeof xe == "number" && ((De = 0 | xe) < 0 ? (Re += Me * (Qe - 1), Qe = Pe(-Qe / De)) : Qe = Pe(Qe / De), Me *= De), new Se(this.data, Le, Qe, ke, Me, Re);
                  }, we.transpose = function(Ee, xe) {
                    Ee = Ee === void 0 ? 0 : 0 | Ee, xe = xe === void 0 ? 1 : 0 | xe;
                    var Le = this.shape, Qe = this.stride;
                    return new Se(this.data, Le[Ee], Le[xe], Qe[Ee], Qe[xe], this.offset);
                  }, we.pick = function(Ee, xe) {
                    var Le = [], Qe = [], ke = this.offset;
                    return typeof Ee == "number" && Ee >= 0 ? ke = ke + this.stride[0] * Ee | 0 : (Le.push(this.shape[0]), Qe.push(this.stride[0])), typeof xe == "number" && xe >= 0 ? ke = ke + this.stride[1] * xe | 0 : (Le.push(this.shape[1]), Qe.push(this.stride[1])), (0, be[Le.length + 1])(this.data, Le, Qe, ke);
                  }, function(Ee, xe, Le, Qe) {
                    return new Se(Ee, xe[0], xe[1], Le[0], Le[1], Qe);
                  };
                }, 3: function(ve, be, Ce) {
                  function Se(Ee, xe, Le, Qe, ke, Me, Re, De) {
                    this.data = Ee, this.shape = [xe, Le, Qe], this.stride = [ke, Me, Re], this.offset = 0 | De;
                  }
                  var we = Se.prototype;
                  return we.dtype = ve, we.dimension = 3, Object.defineProperty(we, "size", { get: function() {
                    return this.shape[0] * this.shape[1] * this.shape[2];
                  } }), Object.defineProperty(we, "order", { get: function() {
                    var Ee = Math.abs(this.stride[0]), xe = Math.abs(this.stride[1]), Le = Math.abs(this.stride[2]);
                    return Ee > xe ? xe > Le ? [2, 1, 0] : Ee > Le ? [1, 2, 0] : [1, 0, 2] : Ee > Le ? [2, 0, 1] : Le > xe ? [0, 1, 2] : [0, 2, 1];
                  } }), we.set = function(Ee, xe, Le, Qe) {
                    return ve === "generic" ? this.data.set(this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le, Qe) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le] = Qe;
                  }, we.get = function(Ee, xe, Le) {
                    return ve === "generic" ? this.data.get(this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le];
                  }, we.index = function(Ee, xe, Le) {
                    return this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le;
                  }, we.hi = function(Ee, xe, Le) {
                    return new Se(this.data, typeof Ee != "number" || Ee < 0 ? this.shape[0] : 0 | Ee, typeof xe != "number" || xe < 0 ? this.shape[1] : 0 | xe, typeof Le != "number" || Le < 0 ? this.shape[2] : 0 | Le, this.stride[0], this.stride[1], this.stride[2], this.offset);
                  }, we.lo = function(Ee, xe, Le) {
                    var Qe = this.offset, ke = 0, Me = this.shape[0], Re = this.shape[1], De = this.shape[2], Pe = this.stride[0], $e = this.stride[1], qe = this.stride[2];
                    return typeof Ee == "number" && Ee >= 0 && (Qe += Pe * (ke = 0 | Ee), Me -= ke), typeof xe == "number" && xe >= 0 && (Qe += $e * (ke = 0 | xe), Re -= ke), typeof Le == "number" && Le >= 0 && (Qe += qe * (ke = 0 | Le), De -= ke), new Se(this.data, Me, Re, De, Pe, $e, qe, Qe);
                  }, we.step = function(Ee, xe, Le) {
                    var Qe = this.shape[0], ke = this.shape[1], Me = this.shape[2], Re = this.stride[0], De = this.stride[1], Pe = this.stride[2], $e = this.offset, qe = 0, Xe = Math.ceil;
                    return typeof Ee == "number" && ((qe = 0 | Ee) < 0 ? ($e += Re * (Qe - 1), Qe = Xe(-Qe / qe)) : Qe = Xe(Qe / qe), Re *= qe), typeof xe == "number" && ((qe = 0 | xe) < 0 ? ($e += De * (ke - 1), ke = Xe(-ke / qe)) : ke = Xe(ke / qe), De *= qe), typeof Le == "number" && ((qe = 0 | Le) < 0 ? ($e += Pe * (Me - 1), Me = Xe(-Me / qe)) : Me = Xe(Me / qe), Pe *= qe), new Se(this.data, Qe, ke, Me, Re, De, Pe, $e);
                  }, we.transpose = function(Ee, xe, Le) {
                    Ee = Ee === void 0 ? 0 : 0 | Ee, xe = xe === void 0 ? 1 : 0 | xe, Le = Le === void 0 ? 2 : 0 | Le;
                    var Qe = this.shape, ke = this.stride;
                    return new Se(this.data, Qe[Ee], Qe[xe], Qe[Le], ke[Ee], ke[xe], ke[Le], this.offset);
                  }, we.pick = function(Ee, xe, Le) {
                    var Qe = [], ke = [], Me = this.offset;
                    return typeof Ee == "number" && Ee >= 0 ? Me = Me + this.stride[0] * Ee | 0 : (Qe.push(this.shape[0]), ke.push(this.stride[0])), typeof xe == "number" && xe >= 0 ? Me = Me + this.stride[1] * xe | 0 : (Qe.push(this.shape[1]), ke.push(this.stride[1])), typeof Le == "number" && Le >= 0 ? Me = Me + this.stride[2] * Le | 0 : (Qe.push(this.shape[2]), ke.push(this.stride[2])), (0, be[Qe.length + 1])(this.data, Qe, ke, Me);
                  }, function(Ee, xe, Le, Qe) {
                    return new Se(Ee, xe[0], xe[1], xe[2], Le[0], Le[1], Le[2], Qe);
                  };
                }, 4: function(ve, be, Ce) {
                  function Se(Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e) {
                    this.data = Ee, this.shape = [xe, Le, Qe, ke], this.stride = [Me, Re, De, Pe], this.offset = 0 | $e;
                  }
                  var we = Se.prototype;
                  return we.dtype = ve, we.dimension = 4, Object.defineProperty(we, "size", { get: function() {
                    return this.shape[0] * this.shape[1] * this.shape[2] * this.shape[3];
                  } }), Object.defineProperty(we, "order", { get: Ce }), we.set = function(Ee, xe, Le, Qe, ke) {
                    return ve === "generic" ? this.data.set(this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe, ke) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe] = ke;
                  }, we.get = function(Ee, xe, Le, Qe) {
                    return ve === "generic" ? this.data.get(this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe];
                  }, we.index = function(Ee, xe, Le, Qe) {
                    return this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe;
                  }, we.hi = function(Ee, xe, Le, Qe) {
                    return new Se(this.data, typeof Ee != "number" || Ee < 0 ? this.shape[0] : 0 | Ee, typeof xe != "number" || xe < 0 ? this.shape[1] : 0 | xe, typeof Le != "number" || Le < 0 ? this.shape[2] : 0 | Le, typeof Qe != "number" || Qe < 0 ? this.shape[3] : 0 | Qe, this.stride[0], this.stride[1], this.stride[2], this.stride[3], this.offset);
                  }, we.lo = function(Ee, xe, Le, Qe) {
                    var ke = this.offset, Me = 0, Re = this.shape[0], De = this.shape[1], Pe = this.shape[2], $e = this.shape[3], qe = this.stride[0], Xe = this.stride[1], Ve = this.stride[2], Ze = this.stride[3];
                    return typeof Ee == "number" && Ee >= 0 && (ke += qe * (Me = 0 | Ee), Re -= Me), typeof xe == "number" && xe >= 0 && (ke += Xe * (Me = 0 | xe), De -= Me), typeof Le == "number" && Le >= 0 && (ke += Ve * (Me = 0 | Le), Pe -= Me), typeof Qe == "number" && Qe >= 0 && (ke += Ze * (Me = 0 | Qe), $e -= Me), new Se(this.data, Re, De, Pe, $e, qe, Xe, Ve, Ze, ke);
                  }, we.step = function(Ee, xe, Le, Qe) {
                    var ke = this.shape[0], Me = this.shape[1], Re = this.shape[2], De = this.shape[3], Pe = this.stride[0], $e = this.stride[1], qe = this.stride[2], Xe = this.stride[3], Ve = this.offset, Ze = 0, Ge = Math.ceil;
                    return typeof Ee == "number" && ((Ze = 0 | Ee) < 0 ? (Ve += Pe * (ke - 1), ke = Ge(-ke / Ze)) : ke = Ge(ke / Ze), Pe *= Ze), typeof xe == "number" && ((Ze = 0 | xe) < 0 ? (Ve += $e * (Me - 1), Me = Ge(-Me / Ze)) : Me = Ge(Me / Ze), $e *= Ze), typeof Le == "number" && ((Ze = 0 | Le) < 0 ? (Ve += qe * (Re - 1), Re = Ge(-Re / Ze)) : Re = Ge(Re / Ze), qe *= Ze), typeof Qe == "number" && ((Ze = 0 | Qe) < 0 ? (Ve += Xe * (De - 1), De = Ge(-De / Ze)) : De = Ge(De / Ze), Xe *= Ze), new Se(this.data, ke, Me, Re, De, Pe, $e, qe, Xe, Ve);
                  }, we.transpose = function(Ee, xe, Le, Qe) {
                    Ee = Ee === void 0 ? 0 : 0 | Ee, xe = xe === void 0 ? 1 : 0 | xe, Le = Le === void 0 ? 2 : 0 | Le, Qe = Qe === void 0 ? 3 : 0 | Qe;
                    var ke = this.shape, Me = this.stride;
                    return new Se(this.data, ke[Ee], ke[xe], ke[Le], ke[Qe], Me[Ee], Me[xe], Me[Le], Me[Qe], this.offset);
                  }, we.pick = function(Ee, xe, Le, Qe) {
                    var ke = [], Me = [], Re = this.offset;
                    return typeof Ee == "number" && Ee >= 0 ? Re = Re + this.stride[0] * Ee | 0 : (ke.push(this.shape[0]), Me.push(this.stride[0])), typeof xe == "number" && xe >= 0 ? Re = Re + this.stride[1] * xe | 0 : (ke.push(this.shape[1]), Me.push(this.stride[1])), typeof Le == "number" && Le >= 0 ? Re = Re + this.stride[2] * Le | 0 : (ke.push(this.shape[2]), Me.push(this.stride[2])), typeof Qe == "number" && Qe >= 0 ? Re = Re + this.stride[3] * Qe | 0 : (ke.push(this.shape[3]), Me.push(this.stride[3])), (0, be[ke.length + 1])(this.data, ke, Me, Re);
                  }, function(Ee, xe, Le, Qe) {
                    return new Se(Ee, xe[0], xe[1], xe[2], xe[3], Le[0], Le[1], Le[2], Le[3], Qe);
                  };
                }, 5: function(ve, be, Ce) {
                  function Se(Ee, xe, Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe) {
                    this.data = Ee, this.shape = [xe, Le, Qe, ke, Me], this.stride = [Re, De, Pe, $e, qe], this.offset = 0 | Xe;
                  }
                  var we = Se.prototype;
                  return we.dtype = ve, we.dimension = 5, Object.defineProperty(we, "size", { get: function() {
                    return this.shape[0] * this.shape[1] * this.shape[2] * this.shape[3] * this.shape[4];
                  } }), Object.defineProperty(we, "order", { get: Ce }), we.set = function(Ee, xe, Le, Qe, ke, Me) {
                    return ve === "generic" ? this.data.set(this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe + this.stride[4] * ke, Me) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe + this.stride[4] * ke] = Me;
                  }, we.get = function(Ee, xe, Le, Qe, ke) {
                    return ve === "generic" ? this.data.get(this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe + this.stride[4] * ke) : this.data[this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe + this.stride[4] * ke];
                  }, we.index = function(Ee, xe, Le, Qe, ke) {
                    return this.offset + this.stride[0] * Ee + this.stride[1] * xe + this.stride[2] * Le + this.stride[3] * Qe + this.stride[4] * ke;
                  }, we.hi = function(Ee, xe, Le, Qe, ke) {
                    return new Se(this.data, typeof Ee != "number" || Ee < 0 ? this.shape[0] : 0 | Ee, typeof xe != "number" || xe < 0 ? this.shape[1] : 0 | xe, typeof Le != "number" || Le < 0 ? this.shape[2] : 0 | Le, typeof Qe != "number" || Qe < 0 ? this.shape[3] : 0 | Qe, typeof ke != "number" || ke < 0 ? this.shape[4] : 0 | ke, this.stride[0], this.stride[1], this.stride[2], this.stride[3], this.stride[4], this.offset);
                  }, we.lo = function(Ee, xe, Le, Qe, ke) {
                    var Me = this.offset, Re = 0, De = this.shape[0], Pe = this.shape[1], $e = this.shape[2], qe = this.shape[3], Xe = this.shape[4], Ve = this.stride[0], Ze = this.stride[1], Ge = this.stride[2], ze = this.stride[3], We = this.stride[4];
                    return typeof Ee == "number" && Ee >= 0 && (Me += Ve * (Re = 0 | Ee), De -= Re), typeof xe == "number" && xe >= 0 && (Me += Ze * (Re = 0 | xe), Pe -= Re), typeof Le == "number" && Le >= 0 && (Me += Ge * (Re = 0 | Le), $e -= Re), typeof Qe == "number" && Qe >= 0 && (Me += ze * (Re = 0 | Qe), qe -= Re), typeof ke == "number" && ke >= 0 && (Me += We * (Re = 0 | ke), Xe -= Re), new Se(this.data, De, Pe, $e, qe, Xe, Ve, Ze, Ge, ze, We, Me);
                  }, we.step = function(Ee, xe, Le, Qe, ke) {
                    var Me = this.shape[0], Re = this.shape[1], De = this.shape[2], Pe = this.shape[3], $e = this.shape[4], qe = this.stride[0], Xe = this.stride[1], Ve = this.stride[2], Ze = this.stride[3], Ge = this.stride[4], ze = this.offset, We = 0, at = Math.ceil;
                    return typeof Ee == "number" && ((We = 0 | Ee) < 0 ? (ze += qe * (Me - 1), Me = at(-Me / We)) : Me = at(Me / We), qe *= We), typeof xe == "number" && ((We = 0 | xe) < 0 ? (ze += Xe * (Re - 1), Re = at(-Re / We)) : Re = at(Re / We), Xe *= We), typeof Le == "number" && ((We = 0 | Le) < 0 ? (ze += Ve * (De - 1), De = at(-De / We)) : De = at(De / We), Ve *= We), typeof Qe == "number" && ((We = 0 | Qe) < 0 ? (ze += Ze * (Pe - 1), Pe = at(-Pe / We)) : Pe = at(Pe / We), Ze *= We), typeof ke == "number" && ((We = 0 | ke) < 0 ? (ze += Ge * ($e - 1), $e = at(-$e / We)) : $e = at($e / We), Ge *= We), new Se(this.data, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze, Ge, ze);
                  }, we.transpose = function(Ee, xe, Le, Qe, ke) {
                    Ee = Ee === void 0 ? 0 : 0 | Ee, xe = xe === void 0 ? 1 : 0 | xe, Le = Le === void 0 ? 2 : 0 | Le, Qe = Qe === void 0 ? 3 : 0 | Qe, ke = ke === void 0 ? 4 : 0 | ke;
                    var Me = this.shape, Re = this.stride;
                    return new Se(this.data, Me[Ee], Me[xe], Me[Le], Me[Qe], Me[ke], Re[Ee], Re[xe], Re[Le], Re[Qe], Re[ke], this.offset);
                  }, we.pick = function(Ee, xe, Le, Qe, ke) {
                    var Me = [], Re = [], De = this.offset;
                    return typeof Ee == "number" && Ee >= 0 ? De = De + this.stride[0] * Ee | 0 : (Me.push(this.shape[0]), Re.push(this.stride[0])), typeof xe == "number" && xe >= 0 ? De = De + this.stride[1] * xe | 0 : (Me.push(this.shape[1]), Re.push(this.stride[1])), typeof Le == "number" && Le >= 0 ? De = De + this.stride[2] * Le | 0 : (Me.push(this.shape[2]), Re.push(this.stride[2])), typeof Qe == "number" && Qe >= 0 ? De = De + this.stride[3] * Qe | 0 : (Me.push(this.shape[3]), Re.push(this.stride[3])), typeof ke == "number" && ke >= 0 ? De = De + this.stride[4] * ke | 0 : (Me.push(this.shape[4]), Re.push(this.stride[4])), (0, be[Me.length + 1])(this.data, Me, Re, De);
                  }, function(Ee, xe, Le, Qe) {
                    return new Se(Ee, xe[0], xe[1], xe[2], xe[3], xe[4], Le[0], Le[1], Le[2], Le[3], Le[4], Qe);
                  };
                } };
                function ye(ve, be) {
                  var Ce = be === -1 ? "T" : String(be), Se = pe[Ce];
                  return be === -1 ? Se(ve) : be === 0 ? Se(ve, ge[ve][0]) : Se(ve, ge[ve], me);
                }
                var ge = { generic: [], buffer: [], array: [], float32: [], float64: [], int8: [], int16: [], int32: [], uint8_clamped: [], uint8: [], uint16: [], uint32: [], bigint64: [], biguint64: [] };
                ce.exports = function(ve, be, Ce, Se) {
                  if (ve === void 0) return (0, ge.array[0])([]);
                  typeof ve == "number" && (ve = [ve]), be === void 0 && (be = [ve.length]);
                  var we = be.length;
                  if (Ce === void 0) {
                    Ce = new Array(we);
                    for (var Ee = we - 1, xe = 1; Ee >= 0; --Ee) Ce[Ee] = xe, xe *= be[Ee];
                  }
                  if (Se === void 0) for (Se = 0, Ee = 0; Ee < we; ++Ee) Ce[Ee] < 0 && (Se -= (be[Ee] - 1) * Ce[Ee]);
                  for (var Le = function(ke) {
                    if (de(ke)) return "buffer";
                    if (fe) switch (Object.prototype.toString.call(ke)) {
                      case "[object Float64Array]":
                        return "float64";
                      case "[object Float32Array]":
                        return "float32";
                      case "[object Int8Array]":
                        return "int8";
                      case "[object Int16Array]":
                        return "int16";
                      case "[object Int32Array]":
                        return "int32";
                      case "[object Uint8ClampedArray]":
                        return "uint8_clamped";
                      case "[object Uint8Array]":
                        return "uint8";
                      case "[object Uint16Array]":
                        return "uint16";
                      case "[object Uint32Array]":
                        return "uint32";
                      case "[object BigInt64Array]":
                        return "bigint64";
                      case "[object BigUint64Array]":
                        return "biguint64";
                    }
                    return Array.isArray(ke) ? "array" : "generic";
                  }(ve), Qe = ge[Le]; Qe.length <= we + 1; ) Qe.push(ye(Le, Qe.length - 1));
                  return (0, Qe[we + 1])(ve, be, Ce, Se);
                };
              }, 1278: function(ce, le, ue) {
                var de = ue(2361), fe = Math.pow(2, -1074), he = -1 >>> 0;
                ce.exports = function(me, pe) {
                  if (isNaN(me) || isNaN(pe)) return NaN;
                  if (me === pe) return me;
                  if (me === 0) return pe < 0 ? -fe : fe;
                  var ye = de.hi(me), ge = de.lo(me);
                  return pe > me == me > 0 ? ge === he ? (ye += 1, ge = 0) : ge += 1 : ge === 0 ? (ge = he, ye -= 1) : ge -= 1, de.pack(ge, ye);
                };
              }, 8406: function(ce, le) {
                le.vertexNormals = function(ue, de, fe) {
                  for (var he = de.length, me = new Array(he), pe = fe === void 0 ? 1e-6 : fe, ye = 0; ye < he; ++ye) me[ye] = [0, 0, 0];
                  for (ye = 0; ye < ue.length; ++ye) for (var ge = ue[ye], ve = 0, be = ge[ge.length - 1], Ce = ge[0], Se = 0; Se < ge.length; ++Se) {
                    ve = be, be = Ce, Ce = ge[(Se + 1) % ge.length];
                    for (var we = de[ve], Ee = de[be], xe = de[Ce], Le = new Array(3), Qe = 0, ke = new Array(3), Me = 0, Re = 0; Re < 3; ++Re) Le[Re] = we[Re] - Ee[Re], Qe += Le[Re] * Le[Re], ke[Re] = xe[Re] - Ee[Re], Me += ke[Re] * ke[Re];
                    if (Qe * Me > pe) {
                      var De = me[be], Pe = 1 / Math.sqrt(Qe * Me);
                      for (Re = 0; Re < 3; ++Re) {
                        var $e = (Re + 1) % 3, qe = (Re + 2) % 3;
                        De[Re] += Pe * (ke[$e] * Le[qe] - ke[qe] * Le[$e]);
                      }
                    }
                  }
                  for (ye = 0; ye < he; ++ye) {
                    De = me[ye];
                    var Xe = 0;
                    for (Re = 0; Re < 3; ++Re) Xe += De[Re] * De[Re];
                    if (Xe > pe) for (Pe = 1 / Math.sqrt(Xe), Re = 0; Re < 3; ++Re) De[Re] *= Pe;
                    else for (Re = 0; Re < 3; ++Re) De[Re] = 0;
                  }
                  return me;
                }, le.faceNormals = function(ue, de, fe) {
                  for (var he = ue.length, me = new Array(he), pe = fe === void 0 ? 1e-6 : fe, ye = 0; ye < he; ++ye) {
                    for (var ge = ue[ye], ve = new Array(3), be = 0; be < 3; ++be) ve[be] = de[ge[be]];
                    var Ce = new Array(3), Se = new Array(3);
                    for (be = 0; be < 3; ++be) Ce[be] = ve[1][be] - ve[0][be], Se[be] = ve[2][be] - ve[0][be];
                    var we = new Array(3), Ee = 0;
                    for (be = 0; be < 3; ++be) {
                      var xe = (be + 1) % 3, Le = (be + 2) % 3;
                      we[be] = Ce[xe] * Se[Le] - Ce[Le] * Se[xe], Ee += we[be] * we[be];
                    }
                    for (Ee = Ee > pe ? 1 / Math.sqrt(Ee) : 0, be = 0; be < 3; ++be) we[be] *= Ee;
                    me[ye] = we;
                  }
                  return me;
                };
              }, 4081: function(ce) {
                ce.exports = function(le, ue, de, fe, he, me, pe, ye, ge, ve) {
                  var be = ue + me + ve;
                  if (Ce > 0) {
                    var Ce = Math.sqrt(be + 1);
                    le[0] = 0.5 * (pe - ge) / Ce, le[1] = 0.5 * (ye - fe) / Ce, le[2] = 0.5 * (de - me) / Ce, le[3] = 0.5 * Ce;
                  } else {
                    var Se = Math.max(ue, me, ve);
                    Ce = Math.sqrt(2 * Se - be + 1), ue >= Se ? (le[0] = 0.5 * Ce, le[1] = 0.5 * (he + de) / Ce, le[2] = 0.5 * (ye + fe) / Ce, le[3] = 0.5 * (pe - ge) / Ce) : me >= Se ? (le[0] = 0.5 * (de + he) / Ce, le[1] = 0.5 * Ce, le[2] = 0.5 * (ge + pe) / Ce, le[3] = 0.5 * (ye - fe) / Ce) : (le[0] = 0.5 * (fe + ye) / Ce, le[1] = 0.5 * (pe + ge) / Ce, le[2] = 0.5 * Ce, le[3] = 0.5 * (de - he) / Ce);
                  }
                  return le;
                };
              }, 9977: function(ce, le, ue) {
                ce.exports = function(Se) {
                  var we = (Se = Se || {}).center || [0, 0, 0], Ee = Se.rotation || [0, 0, 0, 1], xe = Se.radius || 1;
                  we = [].slice.call(we, 0, 3), ve(Ee = [].slice.call(Ee, 0, 4), Ee);
                  var Le = new be(Ee, we, Math.log(xe));
                  return Le.setDistanceLimits(Se.zoomMin, Se.zoomMax), ("eye" in Se || "up" in Se) && Le.lookAt(0, Se.eye, Se.center, Se.up), Le;
                };
                var de = ue(9215), fe = ue(6582), he = ue(7399), me = ue(7608), pe = ue(4081);
                function ye(Se, we, Ee) {
                  return Math.sqrt(Math.pow(Se, 2) + Math.pow(we, 2) + Math.pow(Ee, 2));
                }
                function ge(Se, we, Ee, xe) {
                  return Math.sqrt(Math.pow(Se, 2) + Math.pow(we, 2) + Math.pow(Ee, 2) + Math.pow(xe, 2));
                }
                function ve(Se, we) {
                  var Ee = we[0], xe = we[1], Le = we[2], Qe = we[3], ke = ge(Ee, xe, Le, Qe);
                  ke > 1e-6 ? (Se[0] = Ee / ke, Se[1] = xe / ke, Se[2] = Le / ke, Se[3] = Qe / ke) : (Se[0] = Se[1] = Se[2] = 0, Se[3] = 1);
                }
                function be(Se, we, Ee) {
                  this.radius = de([Ee]), this.center = de(we), this.rotation = de(Se), this.computedRadius = this.radius.curve(0), this.computedCenter = this.center.curve(0), this.computedRotation = this.rotation.curve(0), this.computedUp = [0.1, 0, 0], this.computedEye = [0.1, 0, 0], this.computedMatrix = [0.1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], this.recalcMatrix(0);
                }
                var Ce = be.prototype;
                Ce.lastT = function() {
                  return Math.max(this.radius.lastT(), this.center.lastT(), this.rotation.lastT());
                }, Ce.recalcMatrix = function(Se) {
                  this.radius.curve(Se), this.center.curve(Se), this.rotation.curve(Se);
                  var we = this.computedRotation;
                  ve(we, we);
                  var Ee = this.computedMatrix;
                  he(Ee, we);
                  var xe = this.computedCenter, Le = this.computedEye, Qe = this.computedUp, ke = Math.exp(this.computedRadius[0]);
                  Le[0] = xe[0] + ke * Ee[2], Le[1] = xe[1] + ke * Ee[6], Le[2] = xe[2] + ke * Ee[10], Qe[0] = Ee[1], Qe[1] = Ee[5], Qe[2] = Ee[9];
                  for (var Me = 0; Me < 3; ++Me) {
                    for (var Re = 0, De = 0; De < 3; ++De) Re += Ee[Me + 4 * De] * Le[De];
                    Ee[12 + Me] = -Re;
                  }
                }, Ce.getMatrix = function(Se, we) {
                  this.recalcMatrix(Se);
                  var Ee = this.computedMatrix;
                  if (we) {
                    for (var xe = 0; xe < 16; ++xe) we[xe] = Ee[xe];
                    return we;
                  }
                  return Ee;
                }, Ce.idle = function(Se) {
                  this.center.idle(Se), this.radius.idle(Se), this.rotation.idle(Se);
                }, Ce.flush = function(Se) {
                  this.center.flush(Se), this.radius.flush(Se), this.rotation.flush(Se);
                }, Ce.pan = function(Se, we, Ee, xe) {
                  we = we || 0, Ee = Ee || 0, xe = xe || 0, this.recalcMatrix(Se);
                  var Le = this.computedMatrix, Qe = Le[1], ke = Le[5], Me = Le[9], Re = ye(Qe, ke, Me);
                  Qe /= Re, ke /= Re, Me /= Re;
                  var De = Le[0], Pe = Le[4], $e = Le[8], qe = De * Qe + Pe * ke + $e * Me, Xe = ye(De -= Qe * qe, Pe -= ke * qe, $e -= Me * qe);
                  De /= Xe, Pe /= Xe, $e /= Xe, Le[2], Le[6], Le[10];
                  var Ve = De * we + Qe * Ee, Ze = Pe * we + ke * Ee, Ge = $e * we + Me * Ee;
                  this.center.move(Se, Ve, Ze, Ge);
                  var ze = Math.exp(this.computedRadius[0]);
                  ze = Math.max(1e-4, ze + xe), this.radius.set(Se, Math.log(ze));
                }, Ce.rotate = function(Se, we, Ee, xe) {
                  this.recalcMatrix(Se), we = we || 0, Ee = Ee || 0;
                  var Le = this.computedMatrix, Qe = Le[0], ke = Le[4], Me = Le[8], Re = Le[1], De = Le[5], Pe = Le[9], $e = Le[2], qe = Le[6], Xe = Le[10], Ve = we * Qe + Ee * Re, Ze = we * ke + Ee * De, Ge = we * Me + Ee * Pe, ze = -(qe * Ge - Xe * Ze), We = -(Xe * Ve - $e * Ge), at = -($e * Ze - qe * Ve), ct = Math.sqrt(Math.max(0, 1 - Math.pow(ze, 2) - Math.pow(We, 2) - Math.pow(at, 2))), et = ge(ze, We, at, ct);
                  et > 1e-6 ? (ze /= et, We /= et, at /= et, ct /= et) : (ze = We = at = 0, ct = 1);
                  var dt = this.computedRotation, Je = dt[0], it = dt[1], st = dt[2], gt = dt[3], bt = Je * ct + gt * ze + it * at - st * We, yt = it * ct + gt * We + st * ze - Je * at, St = st * ct + gt * at + Je * We - it * ze, Lt = gt * ct - Je * ze - it * We - st * at;
                  if (xe) {
                    ze = $e, We = qe, at = Xe;
                    var Rt = Math.sin(xe) / ye(ze, We, at);
                    ze *= Rt, We *= Rt, at *= Rt, Lt = Lt * (ct = Math.cos(we)) - (bt = bt * ct + Lt * ze + yt * at - St * We) * ze - (yt = yt * ct + Lt * We + St * ze - bt * at) * We - (St = St * ct + Lt * at + bt * We - yt * ze) * at;
                  }
                  var vt = ge(bt, yt, St, Lt);
                  vt > 1e-6 ? (bt /= vt, yt /= vt, St /= vt, Lt /= vt) : (bt = yt = St = 0, Lt = 1), this.rotation.set(Se, bt, yt, St, Lt);
                }, Ce.lookAt = function(Se, we, Ee, xe) {
                  this.recalcMatrix(Se), Ee = Ee || this.computedCenter, we = we || this.computedEye, xe = xe || this.computedUp;
                  var Le = this.computedMatrix;
                  fe(Le, we, Ee, xe);
                  var Qe = this.computedRotation;
                  pe(Qe, Le[0], Le[1], Le[2], Le[4], Le[5], Le[6], Le[8], Le[9], Le[10]), ve(Qe, Qe), this.rotation.set(Se, Qe[0], Qe[1], Qe[2], Qe[3]);
                  for (var ke = 0, Me = 0; Me < 3; ++Me) ke += Math.pow(Ee[Me] - we[Me], 2);
                  this.radius.set(Se, 0.5 * Math.log(Math.max(ke, 1e-6))), this.center.set(Se, Ee[0], Ee[1], Ee[2]);
                }, Ce.translate = function(Se, we, Ee, xe) {
                  this.center.move(Se, we || 0, Ee || 0, xe || 0);
                }, Ce.setMatrix = function(Se, we) {
                  var Ee = this.computedRotation;
                  pe(Ee, we[0], we[1], we[2], we[4], we[5], we[6], we[8], we[9], we[10]), ve(Ee, Ee), this.rotation.set(Se, Ee[0], Ee[1], Ee[2], Ee[3]);
                  var xe = this.computedMatrix;
                  me(xe, we);
                  var Le = xe[15];
                  if (Math.abs(Le) > 1e-6) {
                    var Qe = xe[12] / Le, ke = xe[13] / Le, Me = xe[14] / Le;
                    this.recalcMatrix(Se);
                    var Re = Math.exp(this.computedRadius[0]);
                    this.center.set(Se, Qe - xe[2] * Re, ke - xe[6] * Re, Me - xe[10] * Re), this.radius.idle(Se);
                  } else this.center.idle(Se), this.radius.idle(Se);
                }, Ce.setDistance = function(Se, we) {
                  we > 0 && this.radius.set(Se, Math.log(we));
                }, Ce.setDistanceLimits = function(Se, we) {
                  Se = Se > 0 ? Math.log(Se) : -1 / 0, we = we > 0 ? Math.log(we) : 1 / 0, we = Math.max(we, Se), this.radius.bounds[0][0] = Se, this.radius.bounds[1][0] = we;
                }, Ce.getDistanceLimits = function(Se) {
                  var we = this.radius.bounds;
                  return Se ? (Se[0] = Math.exp(we[0][0]), Se[1] = Math.exp(we[1][0]), Se) : [Math.exp(we[0][0]), Math.exp(we[1][0])];
                }, Ce.toJSON = function() {
                  return this.recalcMatrix(this.lastT()), { center: this.computedCenter.slice(), rotation: this.computedRotation.slice(), distance: Math.log(this.computedRadius[0]), zoomMin: this.radius.bounds[0][0], zoomMax: this.radius.bounds[1][0] };
                }, Ce.fromJSON = function(Se) {
                  var we = this.lastT(), Ee = Se.center;
                  Ee && this.center.set(we, Ee[0], Ee[1], Ee[2]);
                  var xe = Se.rotation;
                  xe && this.rotation.set(we, xe[0], xe[1], xe[2], xe[3]);
                  var Le = Se.distance;
                  Le && Le > 0 && this.radius.set(we, Math.log(Le)), this.setDistanceLimits(Se.zoomMin, Se.zoomMax);
                };
              }, 1371: function(ce, le, ue) {
                var de = ue(3233);
                ce.exports = function(fe, he, me) {
                  return de(me = me !== void 0 ? me + "" : " ", he) + fe;
                };
              }, 3202: function(ce) {
                ce.exports = function(le, ue) {
                  ue || (ue = [0, ""]), le = String(le);
                  var de = parseFloat(le, 10);
                  return ue[0] = de, ue[1] = le.match(/[\d.\-\+]*\s*(.*)/)[1] || "", ue;
                };
              }, 3088: function(ce, le, ue) {
                ce.exports = function(fe, he) {
                  for (var me = 0 | he.length, pe = fe.length, ye = [new Array(me), new Array(me)], ge = 0; ge < me; ++ge) ye[0][ge] = [], ye[1][ge] = [];
                  for (ge = 0; ge < pe; ++ge) {
                    var ve = fe[ge];
                    ye[0][ve[0]].push(ve), ye[1][ve[1]].push(ve);
                  }
                  var be = [];
                  for (ge = 0; ge < me; ++ge) ye[0][ge].length + ye[1][ge].length === 0 && be.push([ge]);
                  function Ce(ke, Me) {
                    var Re = ye[Me][ke[Me]];
                    Re.splice(Re.indexOf(ke), 1);
                  }
                  function Se(ke, Me, Re) {
                    for (var De, Pe, $e, qe = 0; qe < 2; ++qe) if (ye[qe][Me].length > 0) {
                      De = ye[qe][Me][0], $e = qe;
                      break;
                    }
                    Pe = De[1 ^ $e];
                    for (var Xe = 0; Xe < 2; ++Xe) for (var Ve = ye[Xe][Me], Ze = 0; Ze < Ve.length; ++Ze) {
                      var Ge = Ve[Ze], ze = Ge[1 ^ Xe];
                      de(he[ke], he[Me], he[Pe], he[ze]) > 0 && (De = Ge, Pe = ze, $e = Xe);
                    }
                    return Re || De && Ce(De, $e), Pe;
                  }
                  function we(ke, Me) {
                    var Re = ye[Me][ke][0], De = [ke];
                    Ce(Re, Me);
                    for (var Pe = Re[1 ^ Me]; ; ) {
                      for (; Pe !== ke; ) De.push(Pe), Pe = Se(De[De.length - 2], Pe, !1);
                      if (ye[0][ke].length + ye[1][ke].length === 0) break;
                      var $e = De[De.length - 1], qe = ke, Xe = De[1], Ve = Se($e, qe, !0);
                      if (de(he[$e], he[qe], he[Xe], he[Ve]) < 0) break;
                      De.push(ke), Pe = Se($e, qe);
                    }
                    return De;
                  }
                  function Ee(ke, Me) {
                    return Me[1] === Me[Me.length - 1];
                  }
                  for (ge = 0; ge < me; ++ge) for (var xe = 0; xe < 2; ++xe) {
                    for (var Le = []; ye[xe][ge].length > 0; ) {
                      ye[0][ge].length;
                      var Qe = we(ge, xe);
                      Ee(0, Qe) ? Le.push.apply(Le, Qe) : (Le.length > 0 && be.push(Le), Le = Qe);
                    }
                    Le.length > 0 && be.push(Le);
                  }
                  return be;
                };
                var de = ue(3140);
              }, 5609: function(ce, le, ue) {
                ce.exports = function(fe, he) {
                  for (var me = de(fe, he.length), pe = new Array(he.length), ye = new Array(he.length), ge = [], ve = 0; ve < he.length; ++ve) {
                    var be = me[ve].length;
                    ye[ve] = be, pe[ve] = !0, be <= 1 && ge.push(ve);
                  }
                  for (; ge.length > 0; ) {
                    pe[xe = ge.pop()] = !1;
                    var Ce = me[xe];
                    for (ve = 0; ve < Ce.length; ++ve) {
                      var Se = Ce[ve];
                      --ye[Se] == 0 && ge.push(Se);
                    }
                  }
                  var we = new Array(he.length), Ee = [];
                  for (ve = 0; ve < he.length; ++ve) if (pe[ve]) {
                    var xe = Ee.length;
                    we[ve] = xe, Ee.push(he[ve]);
                  } else we[ve] = -1;
                  var Le = [];
                  for (ve = 0; ve < fe.length; ++ve) {
                    var Qe = fe[ve];
                    pe[Qe[0]] && pe[Qe[1]] && Le.push([we[Qe[0]], we[Qe[1]]]);
                  }
                  return [Le, Ee];
                };
                var de = ue(3134);
              }, 2095: function(ce, le, ue) {
                ce.exports = function(be, Ce) {
                  var Se = ge(be, Ce);
                  be = Se[0];
                  for (var we = (Ce = Se[1]).length, Ee = (be.length, de(be, Ce.length)), xe = 0; xe < we; ++xe) if (Ee[xe].length % 2 == 1) throw new Error("planar-graph-to-polyline: graph must be manifold");
                  var Le = fe(be, Ce), Qe = (Le = Le.filter(function(Xt) {
                    for (var Nt = Xt.length, Gt = [0], Ft = 0; Ft < Nt; ++Ft) {
                      var Tt = Ce[Xt[Ft]], It = Ce[Xt[(Ft + 1) % Nt]], Ct = me(-Tt[0], Tt[1]), Kt = me(-Tt[0], It[1]), Vt = me(It[0], Tt[1]), Yt = me(It[0], It[1]);
                      Gt = pe(Gt, pe(pe(Ct, Kt), pe(Vt, Yt)));
                    }
                    return Gt[Gt.length - 1] > 0;
                  })).length, ke = new Array(Qe), Me = new Array(Qe);
                  for (xe = 0; xe < Qe; ++xe) {
                    ke[xe] = xe;
                    var Re = new Array(Qe), De = Le[xe].map(function(Xt) {
                      return Ce[Xt];
                    }), Pe = he([De]), $e = 0;
                    e: for (var qe = 0; qe < Qe; ++qe) if (Re[qe] = 0, xe !== qe) {
                      for (var Xe = ($t = Le[qe]).length, Ve = 0; Ve < Xe; ++Ve) {
                        var Ze = Pe(Ce[$t[Ve]]);
                        if (Ze !== 0) {
                          Ze < 0 && (Re[qe] = 1, $e += 1);
                          continue e;
                        }
                      }
                      Re[qe] = 1, $e += 1;
                    }
                    Me[xe] = [$e, xe, Re];
                  }
                  for (Me.sort(function(Xt, Nt) {
                    return Nt[0] - Xt[0];
                  }), xe = 0; xe < Qe; ++xe) {
                    var Ge = (Re = Me[xe])[1], ze = Re[2];
                    for (qe = 0; qe < Qe; ++qe) ze[qe] && (ke[qe] = Ge);
                  }
                  var We = function(Xt) {
                    for (var Nt = new Array(Xt), Gt = 0; Gt < Xt; ++Gt) Nt[Gt] = [];
                    return Nt;
                  }(Qe);
                  for (xe = 0; xe < Qe; ++xe) We[xe].push(ke[xe]), We[ke[xe]].push(xe);
                  var at = {}, ct = ve(we, !1);
                  for (xe = 0; xe < Qe; ++xe) for (Xe = ($t = Le[xe]).length, qe = 0; qe < Xe; ++qe) {
                    var et = $t[qe], dt = $t[(qe + 1) % Xe], Je = Math.min(et, dt) + ":" + Math.max(et, dt);
                    if (Je in at) {
                      var it = at[Je];
                      We[it].push(xe), We[xe].push(it), ct[et] = ct[dt] = !0;
                    } else at[Je] = xe;
                  }
                  function st(Xt) {
                    for (var Nt = Xt.length, Gt = 0; Gt < Nt; ++Gt) if (!ct[Xt[Gt]]) return !1;
                    return !0;
                  }
                  var gt = [], bt = ve(Qe, -1);
                  for (xe = 0; xe < Qe; ++xe) ke[xe] !== xe || st(Le[xe]) ? bt[xe] = -1 : (gt.push(xe), bt[xe] = 0);
                  for (Se = []; gt.length > 0; ) {
                    var yt = gt.pop(), St = We[yt];
                    ye(St, function(Xt, Nt) {
                      return Xt - Nt;
                    });
                    var Lt, Rt = St.length, vt = bt[yt];
                    for (vt === 0 && (Lt = [$t = Le[yt]]), xe = 0; xe < Rt; ++xe) {
                      var $t, Pt = St[xe];
                      bt[Pt] >= 0 || (bt[Pt] = 1 ^ vt, gt.push(Pt), vt === 0 && (st($t = Le[Pt]) || ($t.reverse(), Lt.push($t))));
                    }
                    vt === 0 && Se.push(Lt);
                  }
                  return Se;
                };
                var de = ue(3134), fe = ue(3088), he = ue(5085), me = ue(5250), pe = ue(8210), ye = ue(1682), ge = ue(5609);
                function ve(be, Ce) {
                  for (var Se = new Array(be), we = 0; we < be; ++we) Se[we] = Ce;
                  return Se;
                }
              }, 5085: function(ce, le, ue) {
                ce.exports = function(ve) {
                  for (var be = ve.length, Ce = [], Se = [], we = 0; we < be; ++we) for (var Ee = ve[we], xe = Ee.length, Le = xe - 1, Qe = 0; Qe < xe; Le = Qe++) {
                    var ke = Ee[Le], Me = Ee[Qe];
                    ke[0] === Me[0] ? Se.push([ke, Me]) : Ce.push([ke, Me]);
                  }
                  if (Ce.length === 0) return Se.length === 0 ? ge : (Re = ye(Se), function($e) {
                    return Re($e[0], $e[1]) ? 0 : 1;
                  });
                  var Re, De = fe(Ce), Pe = /* @__PURE__ */ function($e, qe) {
                    return function(Xe) {
                      var Ve = me.le(qe, Xe[0]);
                      if (Ve < 0) return 1;
                      var Ze = $e[Ve];
                      if (!Ze) {
                        if (!(Ve > 0 && qe[Ve] === Xe[0])) return 1;
                        Ze = $e[Ve - 1];
                      }
                      for (var Ge = 1; Ze; ) {
                        var ze = Ze.key, We = de(Xe, ze[0], ze[1]);
                        if (ze[0][0] < ze[1][0]) if (We < 0) Ze = Ze.left;
                        else {
                          if (!(We > 0)) return 0;
                          Ge = -1, Ze = Ze.right;
                        }
                        else if (We > 0) Ze = Ze.left;
                        else {
                          if (!(We < 0)) return 0;
                          Ge = 1, Ze = Ze.right;
                        }
                      }
                      return Ge;
                    };
                  }(De.slabs, De.coordinates);
                  return Se.length === 0 ? Pe : /* @__PURE__ */ function($e, qe) {
                    return function(Xe) {
                      return $e(Xe[0], Xe[1]) ? 0 : qe(Xe);
                    };
                  }(ye(Se), Pe);
                };
                var de = ue(3250)[3], fe = ue(4209), he = ue(3352), me = ue(2478);
                function pe() {
                  return !0;
                }
                function ye(ve) {
                  for (var be = {}, Ce = 0; Ce < ve.length; ++Ce) {
                    var Se = ve[Ce], we = Se[0][0], Ee = Se[0][1], xe = Se[1][1], Le = [Math.min(Ee, xe), Math.max(Ee, xe)];
                    we in be ? be[we].push(Le) : be[we] = [Le];
                  }
                  var Qe = {}, ke = Object.keys(be);
                  for (Ce = 0; Ce < ke.length; ++Ce) {
                    var Me = be[ke[Ce]];
                    Qe[ke[Ce]] = he(Me);
                  }
                  return /* @__PURE__ */ function(Re) {
                    return function(De, Pe) {
                      var $e = Re[De];
                      return !!$e && !!$e.queryPoint(Pe, pe);
                    };
                  }(Qe);
                }
                function ge(ve) {
                  return 1;
                }
              }, 9346: function(ce) {
                var le = new Float64Array(4), ue = new Float64Array(4), de = new Float64Array(4);
                ce.exports = function(fe, he, me, pe, ye) {
                  le.length < pe.length && (le = new Float64Array(pe.length), ue = new Float64Array(pe.length), de = new Float64Array(pe.length));
                  for (var ge = 0; ge < pe.length; ++ge) le[ge] = fe[ge] - pe[ge], ue[ge] = he[ge] - fe[ge], de[ge] = me[ge] - fe[ge];
                  var ve = 0, be = 0, Ce = 0, Se = 0, we = 0, Ee = 0;
                  for (ge = 0; ge < pe.length; ++ge) {
                    var xe = ue[ge], Le = de[ge], Qe = le[ge];
                    ve += xe * xe, be += xe * Le, Ce += Le * Le, Se += Qe * xe, we += Qe * Le, Ee += Qe * Qe;
                  }
                  var ke, Me, Re, De, Pe, $e = Math.abs(ve * Ce - be * be), qe = be * we - Ce * Se, Xe = be * Se - ve * we;
                  if (qe + Xe <= $e) if (qe < 0) Xe < 0 && Se < 0 ? (Xe = 0, -Se >= ve ? (qe = 1, ke = ve + 2 * Se + Ee) : ke = Se * (qe = -Se / ve) + Ee) : (qe = 0, we >= 0 ? (Xe = 0, ke = Ee) : -we >= Ce ? (Xe = 1, ke = Ce + 2 * we + Ee) : ke = we * (Xe = -we / Ce) + Ee);
                  else if (Xe < 0) Xe = 0, Se >= 0 ? (qe = 0, ke = Ee) : -Se >= ve ? (qe = 1, ke = ve + 2 * Se + Ee) : ke = Se * (qe = -Se / ve) + Ee;
                  else {
                    var Ve = 1 / $e;
                    ke = (qe *= Ve) * (ve * qe + be * (Xe *= Ve) + 2 * Se) + Xe * (be * qe + Ce * Xe + 2 * we) + Ee;
                  }
                  else qe < 0 ? (Re = Ce + we) > (Me = be + Se) ? (De = Re - Me) >= (Pe = ve - 2 * be + Ce) ? (qe = 1, Xe = 0, ke = ve + 2 * Se + Ee) : ke = (qe = De / Pe) * (ve * qe + be * (Xe = 1 - qe) + 2 * Se) + Xe * (be * qe + Ce * Xe + 2 * we) + Ee : (qe = 0, Re <= 0 ? (Xe = 1, ke = Ce + 2 * we + Ee) : we >= 0 ? (Xe = 0, ke = Ee) : ke = we * (Xe = -we / Ce) + Ee) : Xe < 0 ? (Re = ve + Se) > (Me = be + we) ? (De = Re - Me) >= (Pe = ve - 2 * be + Ce) ? (Xe = 1, qe = 0, ke = Ce + 2 * we + Ee) : ke = (qe = 1 - (Xe = De / Pe)) * (ve * qe + be * Xe + 2 * Se) + Xe * (be * qe + Ce * Xe + 2 * we) + Ee : (Xe = 0, Re <= 0 ? (qe = 1, ke = ve + 2 * Se + Ee) : Se >= 0 ? (qe = 0, ke = Ee) : ke = Se * (qe = -Se / ve) + Ee) : (De = Ce + we - be - Se) <= 0 ? (qe = 0, Xe = 1, ke = Ce + 2 * we + Ee) : De >= (Pe = ve - 2 * be + Ce) ? (qe = 1, Xe = 0, ke = ve + 2 * Se + Ee) : ke = (qe = De / Pe) * (ve * qe + be * (Xe = 1 - qe) + 2 * Se) + Xe * (be * qe + Ce * Xe + 2 * we) + Ee;
                  var Ze = 1 - qe - Xe;
                  for (ge = 0; ge < pe.length; ++ge) ye[ge] = Ze * fe[ge] + qe * he[ge] + Xe * me[ge];
                  return ke < 0 ? 0 : ke;
                };
              }, 8648: function(ce, le, ue) {
                ce.exports = ue(783);
              }, 2653: function(ce, le, ue) {
                var de = ue(3865);
                ce.exports = function(fe, he) {
                  for (var me = fe.length, pe = new Array(me), ye = 0; ye < me; ++ye) pe[ye] = de(fe[ye], he[ye]);
                  return pe;
                };
              }, 5838: function(ce, le, ue) {
                ce.exports = function(fe) {
                  for (var he = new Array(fe.length), me = 0; me < fe.length; ++me) he[me] = de(fe[me]);
                  return he;
                };
                var de = ue(7842);
              }, 8987: function(ce, le, ue) {
                var de = ue(7842), fe = ue(6504);
                ce.exports = function(he, me) {
                  for (var pe = de(me), ye = he.length, ge = new Array(ye), ve = 0; ve < ye; ++ve) ge[ve] = fe(he[ve], pe);
                  return ge;
                };
              }, 544: function(ce, le, ue) {
                var de = ue(5572);
                ce.exports = function(fe, he) {
                  for (var me = fe.length, pe = new Array(me), ye = 0; ye < me; ++ye) pe[ye] = de(fe[ye], he[ye]);
                  return pe;
                };
              }, 5771: function(ce, le, ue) {
                var de = ue(8507), fe = ue(3788), he = ue(2419);
                ce.exports = function(me) {
                  me.sort(fe);
                  for (var pe = me.length, ye = 0, ge = 0; ge < pe; ++ge) {
                    var ve = me[ge], be = he(ve);
                    if (be !== 0) {
                      if (ye > 0) {
                        var Ce = me[ye - 1];
                        if (de(ve, Ce) === 0 && he(Ce) !== be) {
                          ye -= 1;
                          continue;
                        }
                      }
                      me[ye++] = ve;
                    }
                  }
                  return me.length = ye, me;
                };
              }, 3233: function(ce) {
                var le, ue = "";
                ce.exports = function(de, fe) {
                  if (typeof de != "string") throw new TypeError("expected a string");
                  if (fe === 1) return de;
                  if (fe === 2) return de + de;
                  var he = de.length * fe;
                  if (le !== de || le === void 0) le = de, ue = "";
                  else if (ue.length >= he) return ue.substr(0, he);
                  for (; he > ue.length && fe > 1; ) 1 & fe && (ue += de), fe >>= 1, de += de;
                  return ue = (ue += de).substr(0, he);
                };
              }, 3025: function(ce, le, ue) {
                ce.exports = ue.g.performance && ue.g.performance.now ? function() {
                  return performance.now();
                } : Date.now || function() {
                  return +/* @__PURE__ */ new Date();
                };
              }, 7004: function(ce) {
                ce.exports = function(le) {
                  for (var ue = le.length, de = le[le.length - 1], fe = ue, he = ue - 2; he >= 0; --he) {
                    var me = de;
                    (ge = (ye = le[he]) - ((de = me + ye) - me)) && (le[--fe] = de, de = ge);
                  }
                  var pe = 0;
                  for (he = fe; he < ue; ++he) {
                    var ye, ge;
                    (ge = (ye = de) - ((de = (me = le[he]) + ye) - me)) && (le[pe++] = ge);
                  }
                  return le[pe++] = de, le.length = pe, le;
                };
              }, 2962: function(ce, le, ue) {
                var de = ue(5250), fe = ue(8210), he = ue(3012), me = ue(7004);
                function pe(we, Ee, xe, Le) {
                  return function(Qe) {
                    return Le(we(xe(Qe[0][0], Qe[1][1]), xe(-Qe[0][1], Qe[1][0])));
                  };
                }
                function ye(we, Ee, xe, Le) {
                  return function(Qe) {
                    return Le(we(Ee(we(xe(Qe[1][1], Qe[2][2]), xe(-Qe[1][2], Qe[2][1])), Qe[0][0]), we(Ee(we(xe(Qe[1][0], Qe[2][2]), xe(-Qe[1][2], Qe[2][0])), -Qe[0][1]), Ee(we(xe(Qe[1][0], Qe[2][1]), xe(-Qe[1][1], Qe[2][0])), Qe[0][2]))));
                  };
                }
                function ge(we, Ee, xe, Le) {
                  return function(Qe) {
                    return Le(we(we(Ee(we(Ee(we(xe(Qe[2][2], Qe[3][3]), xe(-Qe[2][3], Qe[3][2])), Qe[1][1]), we(Ee(we(xe(Qe[2][1], Qe[3][3]), xe(-Qe[2][3], Qe[3][1])), -Qe[1][2]), Ee(we(xe(Qe[2][1], Qe[3][2]), xe(-Qe[2][2], Qe[3][1])), Qe[1][3]))), Qe[0][0]), Ee(we(Ee(we(xe(Qe[2][2], Qe[3][3]), xe(-Qe[2][3], Qe[3][2])), Qe[1][0]), we(Ee(we(xe(Qe[2][0], Qe[3][3]), xe(-Qe[2][3], Qe[3][0])), -Qe[1][2]), Ee(we(xe(Qe[2][0], Qe[3][2]), xe(-Qe[2][2], Qe[3][0])), Qe[1][3]))), -Qe[0][1])), we(Ee(we(Ee(we(xe(Qe[2][1], Qe[3][3]), xe(-Qe[2][3], Qe[3][1])), Qe[1][0]), we(Ee(we(xe(Qe[2][0], Qe[3][3]), xe(-Qe[2][3], Qe[3][0])), -Qe[1][1]), Ee(we(xe(Qe[2][0], Qe[3][1]), xe(-Qe[2][1], Qe[3][0])), Qe[1][3]))), Qe[0][2]), Ee(we(Ee(we(xe(Qe[2][1], Qe[3][2]), xe(-Qe[2][2], Qe[3][1])), Qe[1][0]), we(Ee(we(xe(Qe[2][0], Qe[3][2]), xe(-Qe[2][2], Qe[3][0])), -Qe[1][1]), Ee(we(xe(Qe[2][0], Qe[3][1]), xe(-Qe[2][1], Qe[3][0])), Qe[1][2]))), -Qe[0][3]))));
                  };
                }
                function ve(we, Ee, xe, Le) {
                  return function(Qe) {
                    return Le(we(we(Ee(we(we(Ee(we(Ee(we(xe(Qe[3][3], Qe[4][4]), xe(-Qe[3][4], Qe[4][3])), Qe[2][2]), we(Ee(we(xe(Qe[3][2], Qe[4][4]), xe(-Qe[3][4], Qe[4][2])), -Qe[2][3]), Ee(we(xe(Qe[3][2], Qe[4][3]), xe(-Qe[3][3], Qe[4][2])), Qe[2][4]))), Qe[1][1]), Ee(we(Ee(we(xe(Qe[3][3], Qe[4][4]), xe(-Qe[3][4], Qe[4][3])), Qe[2][1]), we(Ee(we(xe(Qe[3][1], Qe[4][4]), xe(-Qe[3][4], Qe[4][1])), -Qe[2][3]), Ee(we(xe(Qe[3][1], Qe[4][3]), xe(-Qe[3][3], Qe[4][1])), Qe[2][4]))), -Qe[1][2])), we(Ee(we(Ee(we(xe(Qe[3][2], Qe[4][4]), xe(-Qe[3][4], Qe[4][2])), Qe[2][1]), we(Ee(we(xe(Qe[3][1], Qe[4][4]), xe(-Qe[3][4], Qe[4][1])), -Qe[2][2]), Ee(we(xe(Qe[3][1], Qe[4][2]), xe(-Qe[3][2], Qe[4][1])), Qe[2][4]))), Qe[1][3]), Ee(we(Ee(we(xe(Qe[3][2], Qe[4][3]), xe(-Qe[3][3], Qe[4][2])), Qe[2][1]), we(Ee(we(xe(Qe[3][1], Qe[4][3]), xe(-Qe[3][3], Qe[4][1])), -Qe[2][2]), Ee(we(xe(Qe[3][1], Qe[4][2]), xe(-Qe[3][2], Qe[4][1])), Qe[2][3]))), -Qe[1][4]))), Qe[0][0]), Ee(we(we(Ee(we(Ee(we(xe(Qe[3][3], Qe[4][4]), xe(-Qe[3][4], Qe[4][3])), Qe[2][2]), we(Ee(we(xe(Qe[3][2], Qe[4][4]), xe(-Qe[3][4], Qe[4][2])), -Qe[2][3]), Ee(we(xe(Qe[3][2], Qe[4][3]), xe(-Qe[3][3], Qe[4][2])), Qe[2][4]))), Qe[1][0]), Ee(we(Ee(we(xe(Qe[3][3], Qe[4][4]), xe(-Qe[3][4], Qe[4][3])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][4]), xe(-Qe[3][4], Qe[4][0])), -Qe[2][3]), Ee(we(xe(Qe[3][0], Qe[4][3]), xe(-Qe[3][3], Qe[4][0])), Qe[2][4]))), -Qe[1][2])), we(Ee(we(Ee(we(xe(Qe[3][2], Qe[4][4]), xe(-Qe[3][4], Qe[4][2])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][4]), xe(-Qe[3][4], Qe[4][0])), -Qe[2][2]), Ee(we(xe(Qe[3][0], Qe[4][2]), xe(-Qe[3][2], Qe[4][0])), Qe[2][4]))), Qe[1][3]), Ee(we(Ee(we(xe(Qe[3][2], Qe[4][3]), xe(-Qe[3][3], Qe[4][2])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][3]), xe(-Qe[3][3], Qe[4][0])), -Qe[2][2]), Ee(we(xe(Qe[3][0], Qe[4][2]), xe(-Qe[3][2], Qe[4][0])), Qe[2][3]))), -Qe[1][4]))), -Qe[0][1])), we(Ee(we(we(Ee(we(Ee(we(xe(Qe[3][3], Qe[4][4]), xe(-Qe[3][4], Qe[4][3])), Qe[2][1]), we(Ee(we(xe(Qe[3][1], Qe[4][4]), xe(-Qe[3][4], Qe[4][1])), -Qe[2][3]), Ee(we(xe(Qe[3][1], Qe[4][3]), xe(-Qe[3][3], Qe[4][1])), Qe[2][4]))), Qe[1][0]), Ee(we(Ee(we(xe(Qe[3][3], Qe[4][4]), xe(-Qe[3][4], Qe[4][3])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][4]), xe(-Qe[3][4], Qe[4][0])), -Qe[2][3]), Ee(we(xe(Qe[3][0], Qe[4][3]), xe(-Qe[3][3], Qe[4][0])), Qe[2][4]))), -Qe[1][1])), we(Ee(we(Ee(we(xe(Qe[3][1], Qe[4][4]), xe(-Qe[3][4], Qe[4][1])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][4]), xe(-Qe[3][4], Qe[4][0])), -Qe[2][1]), Ee(we(xe(Qe[3][0], Qe[4][1]), xe(-Qe[3][1], Qe[4][0])), Qe[2][4]))), Qe[1][3]), Ee(we(Ee(we(xe(Qe[3][1], Qe[4][3]), xe(-Qe[3][3], Qe[4][1])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][3]), xe(-Qe[3][3], Qe[4][0])), -Qe[2][1]), Ee(we(xe(Qe[3][0], Qe[4][1]), xe(-Qe[3][1], Qe[4][0])), Qe[2][3]))), -Qe[1][4]))), Qe[0][2]), we(Ee(we(we(Ee(we(Ee(we(xe(Qe[3][2], Qe[4][4]), xe(-Qe[3][4], Qe[4][2])), Qe[2][1]), we(Ee(we(xe(Qe[3][1], Qe[4][4]), xe(-Qe[3][4], Qe[4][1])), -Qe[2][2]), Ee(we(xe(Qe[3][1], Qe[4][2]), xe(-Qe[3][2], Qe[4][1])), Qe[2][4]))), Qe[1][0]), Ee(we(Ee(we(xe(Qe[3][2], Qe[4][4]), xe(-Qe[3][4], Qe[4][2])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][4]), xe(-Qe[3][4], Qe[4][0])), -Qe[2][2]), Ee(we(xe(Qe[3][0], Qe[4][2]), xe(-Qe[3][2], Qe[4][0])), Qe[2][4]))), -Qe[1][1])), we(Ee(we(Ee(we(xe(Qe[3][1], Qe[4][4]), xe(-Qe[3][4], Qe[4][1])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][4]), xe(-Qe[3][4], Qe[4][0])), -Qe[2][1]), Ee(we(xe(Qe[3][0], Qe[4][1]), xe(-Qe[3][1], Qe[4][0])), Qe[2][4]))), Qe[1][2]), Ee(we(Ee(we(xe(Qe[3][1], Qe[4][2]), xe(-Qe[3][2], Qe[4][1])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][2]), xe(-Qe[3][2], Qe[4][0])), -Qe[2][1]), Ee(we(xe(Qe[3][0], Qe[4][1]), xe(-Qe[3][1], Qe[4][0])), Qe[2][2]))), -Qe[1][4]))), -Qe[0][3]), Ee(we(we(Ee(we(Ee(we(xe(Qe[3][2], Qe[4][3]), xe(-Qe[3][3], Qe[4][2])), Qe[2][1]), we(Ee(we(xe(Qe[3][1], Qe[4][3]), xe(-Qe[3][3], Qe[4][1])), -Qe[2][2]), Ee(we(xe(Qe[3][1], Qe[4][2]), xe(-Qe[3][2], Qe[4][1])), Qe[2][3]))), Qe[1][0]), Ee(we(Ee(we(xe(Qe[3][2], Qe[4][3]), xe(-Qe[3][3], Qe[4][2])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][3]), xe(-Qe[3][3], Qe[4][0])), -Qe[2][2]), Ee(we(xe(Qe[3][0], Qe[4][2]), xe(-Qe[3][2], Qe[4][0])), Qe[2][3]))), -Qe[1][1])), we(Ee(we(Ee(we(xe(Qe[3][1], Qe[4][3]), xe(-Qe[3][3], Qe[4][1])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][3]), xe(-Qe[3][3], Qe[4][0])), -Qe[2][1]), Ee(we(xe(Qe[3][0], Qe[4][1]), xe(-Qe[3][1], Qe[4][0])), Qe[2][3]))), Qe[1][2]), Ee(we(Ee(we(xe(Qe[3][1], Qe[4][2]), xe(-Qe[3][2], Qe[4][1])), Qe[2][0]), we(Ee(we(xe(Qe[3][0], Qe[4][2]), xe(-Qe[3][2], Qe[4][0])), -Qe[2][1]), Ee(we(xe(Qe[3][0], Qe[4][1]), xe(-Qe[3][1], Qe[4][0])), Qe[2][2]))), -Qe[1][3]))), Qe[0][4])))));
                  };
                }
                function be(we) {
                  return (we === 2 ? pe : we === 3 ? ye : we === 4 ? ge : we === 5 ? ve : void 0)(fe, he, de, me);
                }
                var Ce = [function() {
                  return [0];
                }, function(we) {
                  return [we[0][0]];
                }];
                function Se(we, Ee, xe, Le, Qe, ke, Me, Re) {
                  return function(De) {
                    switch (De.length) {
                      case 0:
                        return we(De);
                      case 1:
                        return Ee(De);
                      case 2:
                        return xe(De);
                      case 3:
                        return Le(De);
                      case 4:
                        return Qe(De);
                      case 5:
                        return ke(De);
                    }
                    var Pe = Me[De.length];
                    return Pe || (Pe = Me[De.length] = Re(De.length)), Pe(De);
                  };
                }
                (function() {
                  for (; Ce.length < 6; ) Ce.push(be(Ce.length));
                  ce.exports = Se.apply(void 0, Ce.concat([Ce, be]));
                  for (var we = 0; we < Ce.length; ++we) ce.exports[we] = Ce[we];
                })();
              }, 1944: function(ce, le, ue) {
                var de = ue(5250), fe = ue(8210);
                ce.exports = function(he, me) {
                  for (var pe = de(he[0], me[0]), ye = 1; ye < he.length; ++ye) pe = fe(pe, de(he[ye], me[ye]));
                  return pe;
                };
              }, 2646: function(ce, le, ue) {
                var de = ue(5250), fe = ue(8210), he = ue(8545), me = ue(3012);
                function pe(Ee) {
                  return (Ee === 3 ? ye : Ee === 4 ? ge : Ee === 5 ? ve : be)(fe, he, de, me);
                }
                function ye(Ee, xe, Le, Qe) {
                  return function(ke, Me, Re) {
                    var De = Le(ke[0], ke[0]), Pe = Qe(De, Me[0]), $e = Qe(De, Re[0]), qe = Le(Me[0], Me[0]), Xe = Qe(qe, ke[0]), Ve = Qe(qe, Re[0]), Ze = Le(Re[0], Re[0]), Ge = Qe(Ze, ke[0]), ze = Qe(Ze, Me[0]), We = Ee(xe(ze, Ve), xe(Xe, Pe)), at = xe(Ge, $e), ct = xe(We, at);
                    return ct[ct.length - 1];
                  };
                }
                function ge(Ee, xe, Le, Qe) {
                  return function(ke, Me, Re, De) {
                    var Pe = Ee(Le(ke[0], ke[0]), Le(ke[1], ke[1])), $e = Qe(Pe, Me[0]), qe = Qe(Pe, Re[0]), Xe = Qe(Pe, De[0]), Ve = Ee(Le(Me[0], Me[0]), Le(Me[1], Me[1])), Ze = Qe(Ve, ke[0]), Ge = Qe(Ve, Re[0]), ze = Qe(Ve, De[0]), We = Ee(Le(Re[0], Re[0]), Le(Re[1], Re[1])), at = Qe(We, ke[0]), ct = Qe(We, Me[0]), et = Qe(We, De[0]), dt = Ee(Le(De[0], De[0]), Le(De[1], De[1])), Je = Qe(dt, ke[0]), it = Qe(dt, Me[0]), st = Qe(dt, Re[0]), gt = Ee(Ee(Qe(xe(st, et), Me[1]), Ee(Qe(xe(it, ze), -Re[1]), Qe(xe(ct, Ge), De[1]))), Ee(Qe(xe(it, ze), ke[1]), Ee(Qe(xe(Je, Xe), -Me[1]), Qe(xe(Ze, $e), De[1])))), bt = Ee(Ee(Qe(xe(st, et), ke[1]), Ee(Qe(xe(Je, Xe), -Re[1]), Qe(xe(at, qe), De[1]))), Ee(Qe(xe(ct, Ge), ke[1]), Ee(Qe(xe(at, qe), -Me[1]), Qe(xe(Ze, $e), Re[1])))), yt = xe(gt, bt);
                    return yt[yt.length - 1];
                  };
                }
                function ve(Ee, xe, Le, Qe) {
                  return function(ke, Me, Re, De, Pe) {
                    var $e = Ee(Le(ke[0], ke[0]), Ee(Le(ke[1], ke[1]), Le(ke[2], ke[2]))), qe = Qe($e, Me[0]), Xe = Qe($e, Re[0]), Ve = Qe($e, De[0]), Ze = Qe($e, Pe[0]), Ge = Ee(Le(Me[0], Me[0]), Ee(Le(Me[1], Me[1]), Le(Me[2], Me[2]))), ze = Qe(Ge, ke[0]), We = Qe(Ge, Re[0]), at = Qe(Ge, De[0]), ct = Qe(Ge, Pe[0]), et = Ee(Le(Re[0], Re[0]), Ee(Le(Re[1], Re[1]), Le(Re[2], Re[2]))), dt = Qe(et, ke[0]), Je = Qe(et, Me[0]), it = Qe(et, De[0]), st = Qe(et, Pe[0]), gt = Ee(Le(De[0], De[0]), Ee(Le(De[1], De[1]), Le(De[2], De[2]))), bt = Qe(gt, ke[0]), yt = Qe(gt, Me[0]), St = Qe(gt, Re[0]), Lt = Qe(gt, Pe[0]), Rt = Ee(Le(Pe[0], Pe[0]), Ee(Le(Pe[1], Pe[1]), Le(Pe[2], Pe[2]))), vt = Qe(Rt, ke[0]), $t = Qe(Rt, Me[0]), Pt = Qe(Rt, Re[0]), Xt = Qe(Rt, De[0]), Nt = Ee(Ee(Ee(Qe(Ee(Qe(xe(Xt, Lt), Re[1]), Ee(Qe(xe(Pt, st), -De[1]), Qe(xe(St, it), Pe[1]))), Me[2]), Ee(Qe(Ee(Qe(xe(Xt, Lt), Me[1]), Ee(Qe(xe($t, ct), -De[1]), Qe(xe(yt, at), Pe[1]))), -Re[2]), Qe(Ee(Qe(xe(Pt, st), Me[1]), Ee(Qe(xe($t, ct), -Re[1]), Qe(xe(Je, We), Pe[1]))), De[2]))), Ee(Qe(Ee(Qe(xe(St, it), Me[1]), Ee(Qe(xe(yt, at), -Re[1]), Qe(xe(Je, We), De[1]))), -Pe[2]), Ee(Qe(Ee(Qe(xe(Xt, Lt), Me[1]), Ee(Qe(xe($t, ct), -De[1]), Qe(xe(yt, at), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(Xt, Lt), ke[1]), Ee(Qe(xe(vt, Ze), -De[1]), Qe(xe(bt, Ve), Pe[1]))), -Me[2])))), Ee(Ee(Qe(Ee(Qe(xe($t, ct), ke[1]), Ee(Qe(xe(vt, Ze), -Me[1]), Qe(xe(ze, qe), Pe[1]))), De[2]), Ee(Qe(Ee(Qe(xe(yt, at), ke[1]), Ee(Qe(xe(bt, Ve), -Me[1]), Qe(xe(ze, qe), De[1]))), -Pe[2]), Qe(Ee(Qe(xe(St, it), Me[1]), Ee(Qe(xe(yt, at), -Re[1]), Qe(xe(Je, We), De[1]))), ke[2]))), Ee(Qe(Ee(Qe(xe(St, it), ke[1]), Ee(Qe(xe(bt, Ve), -Re[1]), Qe(xe(dt, Xe), De[1]))), -Me[2]), Ee(Qe(Ee(Qe(xe(yt, at), ke[1]), Ee(Qe(xe(bt, Ve), -Me[1]), Qe(xe(ze, qe), De[1]))), Re[2]), Qe(Ee(Qe(xe(Je, We), ke[1]), Ee(Qe(xe(dt, Xe), -Me[1]), Qe(xe(ze, qe), Re[1]))), -De[2]))))), Gt = Ee(Ee(Ee(Qe(Ee(Qe(xe(Xt, Lt), Re[1]), Ee(Qe(xe(Pt, st), -De[1]), Qe(xe(St, it), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(Xt, Lt), ke[1]), Ee(Qe(xe(vt, Ze), -De[1]), Qe(xe(bt, Ve), Pe[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(Pt, st), ke[1]), Ee(Qe(xe(vt, Ze), -Re[1]), Qe(xe(dt, Xe), Pe[1]))), De[2]), Qe(Ee(Qe(xe(St, it), ke[1]), Ee(Qe(xe(bt, Ve), -Re[1]), Qe(xe(dt, Xe), De[1]))), -Pe[2]))), Ee(Ee(Qe(Ee(Qe(xe(Pt, st), Me[1]), Ee(Qe(xe($t, ct), -Re[1]), Qe(xe(Je, We), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(Pt, st), ke[1]), Ee(Qe(xe(vt, Ze), -Re[1]), Qe(xe(dt, Xe), Pe[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe($t, ct), ke[1]), Ee(Qe(xe(vt, Ze), -Me[1]), Qe(xe(ze, qe), Pe[1]))), Re[2]), Qe(Ee(Qe(xe(Je, We), ke[1]), Ee(Qe(xe(dt, Xe), -Me[1]), Qe(xe(ze, qe), Re[1]))), -Pe[2])))), Ft = xe(Nt, Gt);
                    return Ft[Ft.length - 1];
                  };
                }
                function be(Ee, xe, Le, Qe) {
                  return function(ke, Me, Re, De, Pe, $e) {
                    var qe = Ee(Ee(Le(ke[0], ke[0]), Le(ke[1], ke[1])), Ee(Le(ke[2], ke[2]), Le(ke[3], ke[3]))), Xe = Qe(qe, Me[0]), Ve = Qe(qe, Re[0]), Ze = Qe(qe, De[0]), Ge = Qe(qe, Pe[0]), ze = Qe(qe, $e[0]), We = Ee(Ee(Le(Me[0], Me[0]), Le(Me[1], Me[1])), Ee(Le(Me[2], Me[2]), Le(Me[3], Me[3]))), at = Qe(We, ke[0]), ct = Qe(We, Re[0]), et = Qe(We, De[0]), dt = Qe(We, Pe[0]), Je = Qe(We, $e[0]), it = Ee(Ee(Le(Re[0], Re[0]), Le(Re[1], Re[1])), Ee(Le(Re[2], Re[2]), Le(Re[3], Re[3]))), st = Qe(it, ke[0]), gt = Qe(it, Me[0]), bt = Qe(it, De[0]), yt = Qe(it, Pe[0]), St = Qe(it, $e[0]), Lt = Ee(Ee(Le(De[0], De[0]), Le(De[1], De[1])), Ee(Le(De[2], De[2]), Le(De[3], De[3]))), Rt = Qe(Lt, ke[0]), vt = Qe(Lt, Me[0]), $t = Qe(Lt, Re[0]), Pt = Qe(Lt, Pe[0]), Xt = Qe(Lt, $e[0]), Nt = Ee(Ee(Le(Pe[0], Pe[0]), Le(Pe[1], Pe[1])), Ee(Le(Pe[2], Pe[2]), Le(Pe[3], Pe[3]))), Gt = Qe(Nt, ke[0]), Ft = Qe(Nt, Me[0]), Tt = Qe(Nt, Re[0]), It = Qe(Nt, De[0]), Ct = Qe(Nt, $e[0]), Kt = Ee(Ee(Le($e[0], $e[0]), Le($e[1], $e[1])), Ee(Le($e[2], $e[2]), Le($e[3], $e[3]))), Vt = Qe(Kt, ke[0]), Yt = Qe(Kt, Me[0]), sn = Qe(Kt, Re[0]), hn = Qe(Kt, De[0]), Qt = Qe(Kt, Pe[0]), Ut = Ee(Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), De[1]), Ee(Qe(xe(hn, Xt), -Pe[1]), Qe(xe(It, Pt), $e[1]))), Re[2]), Qe(Ee(Qe(xe(Qt, Ct), Re[1]), Ee(Qe(xe(sn, St), -Pe[1]), Qe(xe(Tt, yt), $e[1]))), -De[2])), Ee(Qe(Ee(Qe(xe(hn, Xt), Re[1]), Ee(Qe(xe(sn, St), -De[1]), Qe(xe($t, bt), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(It, Pt), Re[1]), Ee(Qe(xe(Tt, yt), -De[1]), Qe(xe($t, bt), Pe[1]))), -$e[2]))), Me[3]), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), De[1]), Ee(Qe(xe(hn, Xt), -Pe[1]), Qe(xe(It, Pt), $e[1]))), Me[2]), Qe(Ee(Qe(xe(Qt, Ct), Me[1]), Ee(Qe(xe(Yt, Je), -Pe[1]), Qe(xe(Ft, dt), $e[1]))), -De[2])), Ee(Qe(Ee(Qe(xe(hn, Xt), Me[1]), Ee(Qe(xe(Yt, Je), -De[1]), Qe(xe(vt, et), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(It, Pt), Me[1]), Ee(Qe(xe(Ft, dt), -De[1]), Qe(xe(vt, et), Pe[1]))), -$e[2]))), -Re[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), Re[1]), Ee(Qe(xe(sn, St), -Pe[1]), Qe(xe(Tt, yt), $e[1]))), Me[2]), Qe(Ee(Qe(xe(Qt, Ct), Me[1]), Ee(Qe(xe(Yt, Je), -Pe[1]), Qe(xe(Ft, dt), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), Me[1]), Ee(Qe(xe(Yt, Je), -Re[1]), Qe(xe(gt, ct), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(Tt, yt), Me[1]), Ee(Qe(xe(Ft, dt), -Re[1]), Qe(xe(gt, ct), Pe[1]))), -$e[2]))), De[3]))), Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(hn, Xt), Re[1]), Ee(Qe(xe(sn, St), -De[1]), Qe(xe($t, bt), $e[1]))), Me[2]), Qe(Ee(Qe(xe(hn, Xt), Me[1]), Ee(Qe(xe(Yt, Je), -De[1]), Qe(xe(vt, et), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), Me[1]), Ee(Qe(xe(Yt, Je), -Re[1]), Qe(xe(gt, ct), $e[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), Me[1]), Ee(Qe(xe(vt, et), -Re[1]), Qe(xe(gt, ct), De[1]))), -$e[2]))), -Pe[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(It, Pt), Re[1]), Ee(Qe(xe(Tt, yt), -De[1]), Qe(xe($t, bt), Pe[1]))), Me[2]), Qe(Ee(Qe(xe(It, Pt), Me[1]), Ee(Qe(xe(Ft, dt), -De[1]), Qe(xe(vt, et), Pe[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(Tt, yt), Me[1]), Ee(Qe(xe(Ft, dt), -Re[1]), Qe(xe(gt, ct), Pe[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), Me[1]), Ee(Qe(xe(vt, et), -Re[1]), Qe(xe(gt, ct), De[1]))), -Pe[2]))), $e[3])), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), De[1]), Ee(Qe(xe(hn, Xt), -Pe[1]), Qe(xe(It, Pt), $e[1]))), Me[2]), Qe(Ee(Qe(xe(Qt, Ct), Me[1]), Ee(Qe(xe(Yt, Je), -Pe[1]), Qe(xe(Ft, dt), $e[1]))), -De[2])), Ee(Qe(Ee(Qe(xe(hn, Xt), Me[1]), Ee(Qe(xe(Yt, Je), -De[1]), Qe(xe(vt, et), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(It, Pt), Me[1]), Ee(Qe(xe(Ft, dt), -De[1]), Qe(xe(vt, et), Pe[1]))), -$e[2]))), ke[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), De[1]), Ee(Qe(xe(hn, Xt), -Pe[1]), Qe(xe(It, Pt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(Qt, Ct), ke[1]), Ee(Qe(xe(Vt, ze), -Pe[1]), Qe(xe(Gt, Ge), $e[1]))), -De[2])), Ee(Qe(Ee(Qe(xe(hn, Xt), ke[1]), Ee(Qe(xe(Vt, ze), -De[1]), Qe(xe(Rt, Ze), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(It, Pt), ke[1]), Ee(Qe(xe(Gt, Ge), -De[1]), Qe(xe(Rt, Ze), Pe[1]))), -$e[2]))), -Me[3])))), Ee(Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), Me[1]), Ee(Qe(xe(Yt, Je), -Pe[1]), Qe(xe(Ft, dt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(Qt, Ct), ke[1]), Ee(Qe(xe(Vt, ze), -Pe[1]), Qe(xe(Gt, Ge), $e[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Yt, Je), ke[1]), Ee(Qe(xe(Vt, ze), -Me[1]), Qe(xe(at, Xe), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(Ft, dt), ke[1]), Ee(Qe(xe(Gt, Ge), -Me[1]), Qe(xe(at, Xe), Pe[1]))), -$e[2]))), De[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(hn, Xt), Me[1]), Ee(Qe(xe(Yt, Je), -De[1]), Qe(xe(vt, et), $e[1]))), ke[2]), Qe(Ee(Qe(xe(hn, Xt), ke[1]), Ee(Qe(xe(Vt, ze), -De[1]), Qe(xe(Rt, Ze), $e[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Yt, Je), ke[1]), Ee(Qe(xe(Vt, ze), -Me[1]), Qe(xe(at, Xe), $e[1]))), De[2]), Qe(Ee(Qe(xe(vt, et), ke[1]), Ee(Qe(xe(Rt, Ze), -Me[1]), Qe(xe(at, Xe), De[1]))), -$e[2]))), -Pe[3])), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(It, Pt), Me[1]), Ee(Qe(xe(Ft, dt), -De[1]), Qe(xe(vt, et), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(It, Pt), ke[1]), Ee(Qe(xe(Gt, Ge), -De[1]), Qe(xe(Rt, Ze), Pe[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Ft, dt), ke[1]), Ee(Qe(xe(Gt, Ge), -Me[1]), Qe(xe(at, Xe), Pe[1]))), De[2]), Qe(Ee(Qe(xe(vt, et), ke[1]), Ee(Qe(xe(Rt, Ze), -Me[1]), Qe(xe(at, Xe), De[1]))), -Pe[2]))), $e[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(hn, Xt), Re[1]), Ee(Qe(xe(sn, St), -De[1]), Qe(xe($t, bt), $e[1]))), Me[2]), Qe(Ee(Qe(xe(hn, Xt), Me[1]), Ee(Qe(xe(Yt, Je), -De[1]), Qe(xe(vt, et), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), Me[1]), Ee(Qe(xe(Yt, Je), -Re[1]), Qe(xe(gt, ct), $e[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), Me[1]), Ee(Qe(xe(vt, et), -Re[1]), Qe(xe(gt, ct), De[1]))), -$e[2]))), ke[3]))), Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(hn, Xt), Re[1]), Ee(Qe(xe(sn, St), -De[1]), Qe(xe($t, bt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(hn, Xt), ke[1]), Ee(Qe(xe(Vt, ze), -De[1]), Qe(xe(Rt, Ze), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), ke[1]), Ee(Qe(xe(Vt, ze), -Re[1]), Qe(xe(st, Ve), $e[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), ke[1]), Ee(Qe(xe(Rt, Ze), -Re[1]), Qe(xe(st, Ve), De[1]))), -$e[2]))), -Me[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(hn, Xt), Me[1]), Ee(Qe(xe(Yt, Je), -De[1]), Qe(xe(vt, et), $e[1]))), ke[2]), Qe(Ee(Qe(xe(hn, Xt), ke[1]), Ee(Qe(xe(Vt, ze), -De[1]), Qe(xe(Rt, Ze), $e[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Yt, Je), ke[1]), Ee(Qe(xe(Vt, ze), -Me[1]), Qe(xe(at, Xe), $e[1]))), De[2]), Qe(Ee(Qe(xe(vt, et), ke[1]), Ee(Qe(xe(Rt, Ze), -Me[1]), Qe(xe(at, Xe), De[1]))), -$e[2]))), Re[3])), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(sn, St), Me[1]), Ee(Qe(xe(Yt, Je), -Re[1]), Qe(xe(gt, ct), $e[1]))), ke[2]), Qe(Ee(Qe(xe(sn, St), ke[1]), Ee(Qe(xe(Vt, ze), -Re[1]), Qe(xe(st, Ve), $e[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Yt, Je), ke[1]), Ee(Qe(xe(Vt, ze), -Me[1]), Qe(xe(at, Xe), $e[1]))), Re[2]), Qe(Ee(Qe(xe(gt, ct), ke[1]), Ee(Qe(xe(st, Ve), -Me[1]), Qe(xe(at, Xe), Re[1]))), -$e[2]))), -De[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe($t, bt), Me[1]), Ee(Qe(xe(vt, et), -Re[1]), Qe(xe(gt, ct), De[1]))), ke[2]), Qe(Ee(Qe(xe($t, bt), ke[1]), Ee(Qe(xe(Rt, Ze), -Re[1]), Qe(xe(st, Ve), De[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(vt, et), ke[1]), Ee(Qe(xe(Rt, Ze), -Me[1]), Qe(xe(at, Xe), De[1]))), Re[2]), Qe(Ee(Qe(xe(gt, ct), ke[1]), Ee(Qe(xe(st, Ve), -Me[1]), Qe(xe(at, Xe), Re[1]))), -De[2]))), $e[3]))))), Wt = Ee(Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), De[1]), Ee(Qe(xe(hn, Xt), -Pe[1]), Qe(xe(It, Pt), $e[1]))), Re[2]), Qe(Ee(Qe(xe(Qt, Ct), Re[1]), Ee(Qe(xe(sn, St), -Pe[1]), Qe(xe(Tt, yt), $e[1]))), -De[2])), Ee(Qe(Ee(Qe(xe(hn, Xt), Re[1]), Ee(Qe(xe(sn, St), -De[1]), Qe(xe($t, bt), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(It, Pt), Re[1]), Ee(Qe(xe(Tt, yt), -De[1]), Qe(xe($t, bt), Pe[1]))), -$e[2]))), ke[3]), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), De[1]), Ee(Qe(xe(hn, Xt), -Pe[1]), Qe(xe(It, Pt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(Qt, Ct), ke[1]), Ee(Qe(xe(Vt, ze), -Pe[1]), Qe(xe(Gt, Ge), $e[1]))), -De[2])), Ee(Qe(Ee(Qe(xe(hn, Xt), ke[1]), Ee(Qe(xe(Vt, ze), -De[1]), Qe(xe(Rt, Ze), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(It, Pt), ke[1]), Ee(Qe(xe(Gt, Ge), -De[1]), Qe(xe(Rt, Ze), Pe[1]))), -$e[2]))), -Re[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), Re[1]), Ee(Qe(xe(sn, St), -Pe[1]), Qe(xe(Tt, yt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(Qt, Ct), ke[1]), Ee(Qe(xe(Vt, ze), -Pe[1]), Qe(xe(Gt, Ge), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), ke[1]), Ee(Qe(xe(Vt, ze), -Re[1]), Qe(xe(st, Ve), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(Tt, yt), ke[1]), Ee(Qe(xe(Gt, Ge), -Re[1]), Qe(xe(st, Ve), Pe[1]))), -$e[2]))), De[3]))), Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(hn, Xt), Re[1]), Ee(Qe(xe(sn, St), -De[1]), Qe(xe($t, bt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(hn, Xt), ke[1]), Ee(Qe(xe(Vt, ze), -De[1]), Qe(xe(Rt, Ze), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), ke[1]), Ee(Qe(xe(Vt, ze), -Re[1]), Qe(xe(st, Ve), $e[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), ke[1]), Ee(Qe(xe(Rt, Ze), -Re[1]), Qe(xe(st, Ve), De[1]))), -$e[2]))), -Pe[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(It, Pt), Re[1]), Ee(Qe(xe(Tt, yt), -De[1]), Qe(xe($t, bt), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(It, Pt), ke[1]), Ee(Qe(xe(Gt, Ge), -De[1]), Qe(xe(Rt, Ze), Pe[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(Tt, yt), ke[1]), Ee(Qe(xe(Gt, Ge), -Re[1]), Qe(xe(st, Ve), Pe[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), ke[1]), Ee(Qe(xe(Rt, Ze), -Re[1]), Qe(xe(st, Ve), De[1]))), -Pe[2]))), $e[3])), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), Re[1]), Ee(Qe(xe(sn, St), -Pe[1]), Qe(xe(Tt, yt), $e[1]))), Me[2]), Qe(Ee(Qe(xe(Qt, Ct), Me[1]), Ee(Qe(xe(Yt, Je), -Pe[1]), Qe(xe(Ft, dt), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), Me[1]), Ee(Qe(xe(Yt, Je), -Re[1]), Qe(xe(gt, ct), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(Tt, yt), Me[1]), Ee(Qe(xe(Ft, dt), -Re[1]), Qe(xe(gt, ct), Pe[1]))), -$e[2]))), ke[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), Re[1]), Ee(Qe(xe(sn, St), -Pe[1]), Qe(xe(Tt, yt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(Qt, Ct), ke[1]), Ee(Qe(xe(Vt, ze), -Pe[1]), Qe(xe(Gt, Ge), $e[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(sn, St), ke[1]), Ee(Qe(xe(Vt, ze), -Re[1]), Qe(xe(st, Ve), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(Tt, yt), ke[1]), Ee(Qe(xe(Gt, Ge), -Re[1]), Qe(xe(st, Ve), Pe[1]))), -$e[2]))), -Me[3])))), Ee(Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Qt, Ct), Me[1]), Ee(Qe(xe(Yt, Je), -Pe[1]), Qe(xe(Ft, dt), $e[1]))), ke[2]), Qe(Ee(Qe(xe(Qt, Ct), ke[1]), Ee(Qe(xe(Vt, ze), -Pe[1]), Qe(xe(Gt, Ge), $e[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Yt, Je), ke[1]), Ee(Qe(xe(Vt, ze), -Me[1]), Qe(xe(at, Xe), $e[1]))), Pe[2]), Qe(Ee(Qe(xe(Ft, dt), ke[1]), Ee(Qe(xe(Gt, Ge), -Me[1]), Qe(xe(at, Xe), Pe[1]))), -$e[2]))), Re[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(sn, St), Me[1]), Ee(Qe(xe(Yt, Je), -Re[1]), Qe(xe(gt, ct), $e[1]))), ke[2]), Qe(Ee(Qe(xe(sn, St), ke[1]), Ee(Qe(xe(Vt, ze), -Re[1]), Qe(xe(st, Ve), $e[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Yt, Je), ke[1]), Ee(Qe(xe(Vt, ze), -Me[1]), Qe(xe(at, Xe), $e[1]))), Re[2]), Qe(Ee(Qe(xe(gt, ct), ke[1]), Ee(Qe(xe(st, Ve), -Me[1]), Qe(xe(at, Xe), Re[1]))), -$e[2]))), -Pe[3])), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Tt, yt), Me[1]), Ee(Qe(xe(Ft, dt), -Re[1]), Qe(xe(gt, ct), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(Tt, yt), ke[1]), Ee(Qe(xe(Gt, Ge), -Re[1]), Qe(xe(st, Ve), Pe[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Ft, dt), ke[1]), Ee(Qe(xe(Gt, Ge), -Me[1]), Qe(xe(at, Xe), Pe[1]))), Re[2]), Qe(Ee(Qe(xe(gt, ct), ke[1]), Ee(Qe(xe(st, Ve), -Me[1]), Qe(xe(at, Xe), Re[1]))), -Pe[2]))), $e[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(It, Pt), Re[1]), Ee(Qe(xe(Tt, yt), -De[1]), Qe(xe($t, bt), Pe[1]))), Me[2]), Qe(Ee(Qe(xe(It, Pt), Me[1]), Ee(Qe(xe(Ft, dt), -De[1]), Qe(xe(vt, et), Pe[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(Tt, yt), Me[1]), Ee(Qe(xe(Ft, dt), -Re[1]), Qe(xe(gt, ct), Pe[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), Me[1]), Ee(Qe(xe(vt, et), -Re[1]), Qe(xe(gt, ct), De[1]))), -Pe[2]))), ke[3]))), Ee(Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(It, Pt), Re[1]), Ee(Qe(xe(Tt, yt), -De[1]), Qe(xe($t, bt), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(It, Pt), ke[1]), Ee(Qe(xe(Gt, Ge), -De[1]), Qe(xe(Rt, Ze), Pe[1]))), -Re[2])), Ee(Qe(Ee(Qe(xe(Tt, yt), ke[1]), Ee(Qe(xe(Gt, Ge), -Re[1]), Qe(xe(st, Ve), Pe[1]))), De[2]), Qe(Ee(Qe(xe($t, bt), ke[1]), Ee(Qe(xe(Rt, Ze), -Re[1]), Qe(xe(st, Ve), De[1]))), -Pe[2]))), -Me[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe(It, Pt), Me[1]), Ee(Qe(xe(Ft, dt), -De[1]), Qe(xe(vt, et), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(It, Pt), ke[1]), Ee(Qe(xe(Gt, Ge), -De[1]), Qe(xe(Rt, Ze), Pe[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Ft, dt), ke[1]), Ee(Qe(xe(Gt, Ge), -Me[1]), Qe(xe(at, Xe), Pe[1]))), De[2]), Qe(Ee(Qe(xe(vt, et), ke[1]), Ee(Qe(xe(Rt, Ze), -Me[1]), Qe(xe(at, Xe), De[1]))), -Pe[2]))), Re[3])), Ee(Qe(Ee(Ee(Qe(Ee(Qe(xe(Tt, yt), Me[1]), Ee(Qe(xe(Ft, dt), -Re[1]), Qe(xe(gt, ct), Pe[1]))), ke[2]), Qe(Ee(Qe(xe(Tt, yt), ke[1]), Ee(Qe(xe(Gt, Ge), -Re[1]), Qe(xe(st, Ve), Pe[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(Ft, dt), ke[1]), Ee(Qe(xe(Gt, Ge), -Me[1]), Qe(xe(at, Xe), Pe[1]))), Re[2]), Qe(Ee(Qe(xe(gt, ct), ke[1]), Ee(Qe(xe(st, Ve), -Me[1]), Qe(xe(at, Xe), Re[1]))), -Pe[2]))), -De[3]), Qe(Ee(Ee(Qe(Ee(Qe(xe($t, bt), Me[1]), Ee(Qe(xe(vt, et), -Re[1]), Qe(xe(gt, ct), De[1]))), ke[2]), Qe(Ee(Qe(xe($t, bt), ke[1]), Ee(Qe(xe(Rt, Ze), -Re[1]), Qe(xe(st, Ve), De[1]))), -Me[2])), Ee(Qe(Ee(Qe(xe(vt, et), ke[1]), Ee(Qe(xe(Rt, Ze), -Me[1]), Qe(xe(at, Xe), De[1]))), Re[2]), Qe(Ee(Qe(xe(gt, ct), ke[1]), Ee(Qe(xe(st, Ve), -Me[1]), Qe(xe(at, Xe), Re[1]))), -De[2]))), Pe[3]))))), rn = xe(Ut, Wt);
                    return rn[rn.length - 1];
                  };
                }
                var Ce = [function() {
                  return 0;
                }, function() {
                  return 0;
                }, function() {
                  return 0;
                }];
                function Se(Ee) {
                  var xe = Ce[Ee.length];
                  return xe || (xe = Ce[Ee.length] = pe(Ee.length)), xe.apply(void 0, Ee);
                }
                function we(Ee, xe, Le, Qe, ke, Me, Re, De) {
                  return function(Pe, $e, qe, Xe, Ve, Ze) {
                    switch (arguments.length) {
                      case 0:
                      case 1:
                        return 0;
                      case 2:
                        return Qe(Pe, $e);
                      case 3:
                        return ke(Pe, $e, qe);
                      case 4:
                        return Me(Pe, $e, qe, Xe);
                      case 5:
                        return Re(Pe, $e, qe, Xe, Ve);
                      case 6:
                        return De(Pe, $e, qe, Xe, Ve, Ze);
                    }
                    for (var Ge = new Array(arguments.length), ze = 0; ze < arguments.length; ++ze) Ge[ze] = arguments[ze];
                    return Ee(Ge);
                  };
                }
                (function() {
                  for (; Ce.length <= 6; ) Ce.push(pe(Ce.length));
                  ce.exports = we.apply(void 0, [Se].concat(Ce));
                  for (var Ee = 0; Ee <= 6; ++Ee) ce.exports[Ee] = Ce[Ee];
                })();
              }, 727: function(ce, le, ue) {
                var de = ue(2962);
                function fe(Ce) {
                  return (Ce === 2 ? he : Ce === 3 ? me : Ce === 4 ? pe : Ce === 5 ? ye : ge)(Ce < 6 ? de[Ce] : de);
                }
                function he(Ce) {
                  return function(Se, we) {
                    return [Ce([[+we[0], +Se[0][1]], [+we[1], +Se[1][1]]]), Ce([[+Se[0][0], +we[0]], [+Se[1][0], +we[1]]]), Ce(Se)];
                  };
                }
                function me(Ce) {
                  return function(Se, we) {
                    return [Ce([[+we[0], +Se[0][1], +Se[0][2]], [+we[1], +Se[1][1], +Se[1][2]], [+we[2], +Se[2][1], +Se[2][2]]]), Ce([[+Se[0][0], +we[0], +Se[0][2]], [+Se[1][0], +we[1], +Se[1][2]], [+Se[2][0], +we[2], +Se[2][2]]]), Ce([[+Se[0][0], +Se[0][1], +we[0]], [+Se[1][0], +Se[1][1], +we[1]], [+Se[2][0], +Se[2][1], +we[2]]]), Ce(Se)];
                  };
                }
                function pe(Ce) {
                  return function(Se, we) {
                    return [Ce([[+we[0], +Se[0][1], +Se[0][2], +Se[0][3]], [+we[1], +Se[1][1], +Se[1][2], +Se[1][3]], [+we[2], +Se[2][1], +Se[2][2], +Se[2][3]], [+we[3], +Se[3][1], +Se[3][2], +Se[3][3]]]), Ce([[+Se[0][0], +we[0], +Se[0][2], +Se[0][3]], [+Se[1][0], +we[1], +Se[1][2], +Se[1][3]], [+Se[2][0], +we[2], +Se[2][2], +Se[2][3]], [+Se[3][0], +we[3], +Se[3][2], +Se[3][3]]]), Ce([[+Se[0][0], +Se[0][1], +we[0], +Se[0][3]], [+Se[1][0], +Se[1][1], +we[1], +Se[1][3]], [+Se[2][0], +Se[2][1], +we[2], +Se[2][3]], [+Se[3][0], +Se[3][1], +we[3], +Se[3][3]]]), Ce([[+Se[0][0], +Se[0][1], +Se[0][2], +we[0]], [+Se[1][0], +Se[1][1], +Se[1][2], +we[1]], [+Se[2][0], +Se[2][1], +Se[2][2], +we[2]], [+Se[3][0], +Se[3][1], +Se[3][2], +we[3]]]), Ce(Se)];
                  };
                }
                function ye(Ce) {
                  return function(Se, we) {
                    return [Ce([[+we[0], +Se[0][1], +Se[0][2], +Se[0][3], +Se[0][4]], [+we[1], +Se[1][1], +Se[1][2], +Se[1][3], +Se[1][4]], [+we[2], +Se[2][1], +Se[2][2], +Se[2][3], +Se[2][4]], [+we[3], +Se[3][1], +Se[3][2], +Se[3][3], +Se[3][4]], [+we[4], +Se[4][1], +Se[4][2], +Se[4][3], +Se[4][4]]]), Ce([[+Se[0][0], +we[0], +Se[0][2], +Se[0][3], +Se[0][4]], [+Se[1][0], +we[1], +Se[1][2], +Se[1][3], +Se[1][4]], [+Se[2][0], +we[2], +Se[2][2], +Se[2][3], +Se[2][4]], [+Se[3][0], +we[3], +Se[3][2], +Se[3][3], +Se[3][4]], [+Se[4][0], +we[4], +Se[4][2], +Se[4][3], +Se[4][4]]]), Ce([[+Se[0][0], +Se[0][1], +we[0], +Se[0][3], +Se[0][4]], [+Se[1][0], +Se[1][1], +we[1], +Se[1][3], +Se[1][4]], [+Se[2][0], +Se[2][1], +we[2], +Se[2][3], +Se[2][4]], [+Se[3][0], +Se[3][1], +we[3], +Se[3][3], +Se[3][4]], [+Se[4][0], +Se[4][1], +we[4], +Se[4][3], +Se[4][4]]]), Ce([[+Se[0][0], +Se[0][1], +Se[0][2], +we[0], +Se[0][4]], [+Se[1][0], +Se[1][1], +Se[1][2], +we[1], +Se[1][4]], [+Se[2][0], +Se[2][1], +Se[2][2], +we[2], +Se[2][4]], [+Se[3][0], +Se[3][1], +Se[3][2], +we[3], +Se[3][4]], [+Se[4][0], +Se[4][1], +Se[4][2], +we[4], +Se[4][4]]]), Ce([[+Se[0][0], +Se[0][1], +Se[0][2], +Se[0][3], +we[0]], [+Se[1][0], +Se[1][1], +Se[1][2], +Se[1][3], +we[1]], [+Se[2][0], +Se[2][1], +Se[2][2], +Se[2][3], +we[2]], [+Se[3][0], +Se[3][1], +Se[3][2], +Se[3][3], +we[3]], [+Se[4][0], +Se[4][1], +Se[4][2], +Se[4][3], +we[4]]]), Ce(Se)];
                  };
                }
                function ge(Ce) {
                  return function(Se, we) {
                    return [Ce([[+we[0], +Se[0][1], +Se[0][2], +Se[0][3], +Se[0][4], +Se[0][5]], [+we[1], +Se[1][1], +Se[1][2], +Se[1][3], +Se[1][4], +Se[1][5]], [+we[2], +Se[2][1], +Se[2][2], +Se[2][3], +Se[2][4], +Se[2][5]], [+we[3], +Se[3][1], +Se[3][2], +Se[3][3], +Se[3][4], +Se[3][5]], [+we[4], +Se[4][1], +Se[4][2], +Se[4][3], +Se[4][4], +Se[4][5]], [+we[5], +Se[5][1], +Se[5][2], +Se[5][3], +Se[5][4], +Se[5][5]]]), Ce([[+Se[0][0], +we[0], +Se[0][2], +Se[0][3], +Se[0][4], +Se[0][5]], [+Se[1][0], +we[1], +Se[1][2], +Se[1][3], +Se[1][4], +Se[1][5]], [+Se[2][0], +we[2], +Se[2][2], +Se[2][3], +Se[2][4], +Se[2][5]], [+Se[3][0], +we[3], +Se[3][2], +Se[3][3], +Se[3][4], +Se[3][5]], [+Se[4][0], +we[4], +Se[4][2], +Se[4][3], +Se[4][4], +Se[4][5]], [+Se[5][0], +we[5], +Se[5][2], +Se[5][3], +Se[5][4], +Se[5][5]]]), Ce([[+Se[0][0], +Se[0][1], +we[0], +Se[0][3], +Se[0][4], +Se[0][5]], [+Se[1][0], +Se[1][1], +we[1], +Se[1][3], +Se[1][4], +Se[1][5]], [+Se[2][0], +Se[2][1], +we[2], +Se[2][3], +Se[2][4], +Se[2][5]], [+Se[3][0], +Se[3][1], +we[3], +Se[3][3], +Se[3][4], +Se[3][5]], [+Se[4][0], +Se[4][1], +we[4], +Se[4][3], +Se[4][4], +Se[4][5]], [+Se[5][0], +Se[5][1], +we[5], +Se[5][3], +Se[5][4], +Se[5][5]]]), Ce([[+Se[0][0], +Se[0][1], +Se[0][2], +we[0], +Se[0][4], +Se[0][5]], [+Se[1][0], +Se[1][1], +Se[1][2], +we[1], +Se[1][4], +Se[1][5]], [+Se[2][0], +Se[2][1], +Se[2][2], +we[2], +Se[2][4], +Se[2][5]], [+Se[3][0], +Se[3][1], +Se[3][2], +we[3], +Se[3][4], +Se[3][5]], [+Se[4][0], +Se[4][1], +Se[4][2], +we[4], +Se[4][4], +Se[4][5]], [+Se[5][0], +Se[5][1], +Se[5][2], +we[5], +Se[5][4], +Se[5][5]]]), Ce([[+Se[0][0], +Se[0][1], +Se[0][2], +Se[0][3], +we[0], +Se[0][5]], [+Se[1][0], +Se[1][1], +Se[1][2], +Se[1][3], +we[1], +Se[1][5]], [+Se[2][0], +Se[2][1], +Se[2][2], +Se[2][3], +we[2], +Se[2][5]], [+Se[3][0], +Se[3][1], +Se[3][2], +Se[3][3], +we[3], +Se[3][5]], [+Se[4][0], +Se[4][1], +Se[4][2], +Se[4][3], +we[4], +Se[4][5]], [+Se[5][0], +Se[5][1], +Se[5][2], +Se[5][3], +we[5], +Se[5][5]]]), Ce([[+Se[0][0], +Se[0][1], +Se[0][2], +Se[0][3], +Se[0][4], +we[0]], [+Se[1][0], +Se[1][1], +Se[1][2], +Se[1][3], +Se[1][4], +we[1]], [+Se[2][0], +Se[2][1], +Se[2][2], +Se[2][3], +Se[2][4], +we[2]], [+Se[3][0], +Se[3][1], +Se[3][2], +Se[3][3], +Se[3][4], +we[3]], [+Se[4][0], +Se[4][1], +Se[4][2], +Se[4][3], +Se[4][4], +we[4]], [+Se[5][0], +Se[5][1], +Se[5][2], +Se[5][3], +Se[5][4], +we[5]]]), Ce(Se)];
                  };
                }
                var ve = [function() {
                  return [[0]];
                }, function(Ce, Se) {
                  return [[Se[0]], [Ce[0][0]]];
                }];
                function be(Ce, Se, we, Ee, xe, Le, Qe, ke) {
                  return function(Me, Re) {
                    switch (Me.length) {
                      case 0:
                        return Ce(Me, Re);
                      case 1:
                        return Se(Me, Re);
                      case 2:
                        return we(Me, Re);
                      case 3:
                        return Ee(Me, Re);
                      case 4:
                        return xe(Me, Re);
                      case 5:
                        return Le(Me, Re);
                    }
                    var De = Qe[Me.length];
                    return De || (De = Qe[Me.length] = ke(Me.length)), De(Me, Re);
                  };
                }
                (function() {
                  for (; ve.length < 6; ) ve.push(fe(ve.length));
                  ce.exports = be.apply(void 0, ve.concat([ve, fe]));
                  for (var Ce = 0; Ce < 6; ++Ce) ce.exports[Ce] = ve[Ce];
                })();
              }, 3250: function(ce, le, ue) {
                var de = ue(5250), fe = ue(8210), he = ue(3012), me = ue(8545);
                function pe(xe, Le, Qe, ke) {
                  return function(Me, Re, De) {
                    var Pe = xe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0]))), $e = xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), qe = ke(Pe, $e);
                    return qe[qe.length - 1];
                  };
                }
                function ye(xe, Le, Qe, ke) {
                  return function(Me, Re, De, Pe) {
                    var $e = xe(xe(Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), Re[2]), xe(Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), -De[2]), Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), Pe[2]))), xe(Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), Pe[2])))), qe = xe(xe(Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), Me[2]), xe(Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), -De[2]), Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), Pe[2]))), xe(Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), De[2])))), Xe = ke($e, qe);
                    return Xe[Xe.length - 1];
                  };
                }
                function ge(xe, Le, Qe, ke) {
                  return function(Me, Re, De, Pe, $e) {
                    var qe = xe(xe(xe(Qe(xe(Qe(xe(Le(Pe[1], $e[0]), Le(-$e[1], Pe[0])), De[2]), xe(Qe(xe(Le(De[1], $e[0]), Le(-$e[1], De[0])), -Pe[2]), Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), $e[2]))), Re[3]), xe(Qe(xe(Qe(xe(Le(Pe[1], $e[0]), Le(-$e[1], Pe[0])), Re[2]), xe(Qe(xe(Le(Re[1], $e[0]), Le(-$e[1], Re[0])), -Pe[2]), Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), $e[2]))), -De[3]), Qe(xe(Qe(xe(Le(De[1], $e[0]), Le(-$e[1], De[0])), Re[2]), xe(Qe(xe(Le(Re[1], $e[0]), Le(-$e[1], Re[0])), -De[2]), Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), $e[2]))), Pe[3]))), xe(Qe(xe(Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), Re[2]), xe(Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), -De[2]), Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), Pe[2]))), -$e[3]), xe(Qe(xe(Qe(xe(Le(Pe[1], $e[0]), Le(-$e[1], Pe[0])), Re[2]), xe(Qe(xe(Le(Re[1], $e[0]), Le(-$e[1], Re[0])), -Pe[2]), Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), $e[2]))), Me[3]), Qe(xe(Qe(xe(Le(Pe[1], $e[0]), Le(-$e[1], Pe[0])), Me[2]), xe(Qe(xe(Le(Me[1], $e[0]), Le(-$e[1], Me[0])), -Pe[2]), Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), $e[2]))), -Re[3])))), xe(xe(Qe(xe(Qe(xe(Le(Re[1], $e[0]), Le(-$e[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], $e[0]), Le(-$e[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), $e[2]))), Pe[3]), xe(Qe(xe(Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), Pe[2]))), -$e[3]), Qe(xe(Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), Re[2]), xe(Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), -De[2]), Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), Pe[2]))), Me[3]))), xe(Qe(xe(Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), Me[2]), xe(Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), -De[2]), Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), Pe[2]))), -Re[3]), xe(Qe(xe(Qe(xe(Le(Re[1], Pe[0]), Le(-Pe[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), Pe[2]))), De[3]), Qe(xe(Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), De[2]))), -Pe[3]))))), Xe = xe(xe(xe(Qe(xe(Qe(xe(Le(Pe[1], $e[0]), Le(-$e[1], Pe[0])), De[2]), xe(Qe(xe(Le(De[1], $e[0]), Le(-$e[1], De[0])), -Pe[2]), Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), $e[2]))), Me[3]), Qe(xe(Qe(xe(Le(Pe[1], $e[0]), Le(-$e[1], Pe[0])), Me[2]), xe(Qe(xe(Le(Me[1], $e[0]), Le(-$e[1], Me[0])), -Pe[2]), Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), $e[2]))), -De[3])), xe(Qe(xe(Qe(xe(Le(De[1], $e[0]), Le(-$e[1], De[0])), Me[2]), xe(Qe(xe(Le(Me[1], $e[0]), Le(-$e[1], Me[0])), -De[2]), Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), $e[2]))), Pe[3]), Qe(xe(Qe(xe(Le(De[1], Pe[0]), Le(-Pe[1], De[0])), Me[2]), xe(Qe(xe(Le(Me[1], Pe[0]), Le(-Pe[1], Me[0])), -De[2]), Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), Pe[2]))), -$e[3]))), xe(xe(Qe(xe(Qe(xe(Le(De[1], $e[0]), Le(-$e[1], De[0])), Re[2]), xe(Qe(xe(Le(Re[1], $e[0]), Le(-$e[1], Re[0])), -De[2]), Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), $e[2]))), Me[3]), Qe(xe(Qe(xe(Le(De[1], $e[0]), Le(-$e[1], De[0])), Me[2]), xe(Qe(xe(Le(Me[1], $e[0]), Le(-$e[1], Me[0])), -De[2]), Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), $e[2]))), -Re[3])), xe(Qe(xe(Qe(xe(Le(Re[1], $e[0]), Le(-$e[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], $e[0]), Le(-$e[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), $e[2]))), De[3]), Qe(xe(Qe(xe(Le(Re[1], De[0]), Le(-De[1], Re[0])), Me[2]), xe(Qe(xe(Le(Me[1], De[0]), Le(-De[1], Me[0])), -Re[2]), Qe(xe(Le(Me[1], Re[0]), Le(-Re[1], Me[0])), De[2]))), -$e[3])))), Ve = ke(qe, Xe);
                    return Ve[Ve.length - 1];
                  };
                }
                function ve(xe) {
                  return (xe === 3 ? pe : xe === 4 ? ye : ge)(fe, de, he, me);
                }
                var be = ve(3), Ce = ve(4), Se = [function() {
                  return 0;
                }, function() {
                  return 0;
                }, function(xe, Le) {
                  return Le[0] - xe[0];
                }, function(xe, Le, Qe) {
                  var ke, Me = (xe[1] - Qe[1]) * (Le[0] - Qe[0]), Re = (xe[0] - Qe[0]) * (Le[1] - Qe[1]), De = Me - Re;
                  if (Me > 0) {
                    if (Re <= 0) return De;
                    ke = Me + Re;
                  } else {
                    if (!(Me < 0) || Re >= 0) return De;
                    ke = -(Me + Re);
                  }
                  var Pe = 33306690738754716e-32 * ke;
                  return De >= Pe || De <= -Pe ? De : be(xe, Le, Qe);
                }, function(xe, Le, Qe, ke) {
                  var Me = xe[0] - ke[0], Re = Le[0] - ke[0], De = Qe[0] - ke[0], Pe = xe[1] - ke[1], $e = Le[1] - ke[1], qe = Qe[1] - ke[1], Xe = xe[2] - ke[2], Ve = Le[2] - ke[2], Ze = Qe[2] - ke[2], Ge = Re * qe, ze = De * $e, We = De * Pe, at = Me * qe, ct = Me * $e, et = Re * Pe, dt = Xe * (Ge - ze) + Ve * (We - at) + Ze * (ct - et), Je = 7771561172376103e-31 * ((Math.abs(Ge) + Math.abs(ze)) * Math.abs(Xe) + (Math.abs(We) + Math.abs(at)) * Math.abs(Ve) + (Math.abs(ct) + Math.abs(et)) * Math.abs(Ze));
                  return dt > Je || -dt > Je ? dt : Ce(xe, Le, Qe, ke);
                }];
                function we(xe) {
                  var Le = Se[xe.length];
                  return Le || (Le = Se[xe.length] = ve(xe.length)), Le.apply(void 0, xe);
                }
                function Ee(xe, Le, Qe, ke, Me, Re, De) {
                  return function(Pe, $e, qe, Xe, Ve) {
                    switch (arguments.length) {
                      case 0:
                      case 1:
                        return 0;
                      case 2:
                        return ke(Pe, $e);
                      case 3:
                        return Me(Pe, $e, qe);
                      case 4:
                        return Re(Pe, $e, qe, Xe);
                      case 5:
                        return De(Pe, $e, qe, Xe, Ve);
                    }
                    for (var Ze = new Array(arguments.length), Ge = 0; Ge < arguments.length; ++Ge) Ze[Ge] = arguments[Ge];
                    return xe(Ze);
                  };
                }
                (function() {
                  for (; Se.length <= 5; ) Se.push(ve(Se.length));
                  ce.exports = Ee.apply(void 0, [we].concat(Se));
                  for (var xe = 0; xe <= 5; ++xe) ce.exports[xe] = Se[xe];
                })();
              }, 5382: function(ce, le, ue) {
                var de = ue(8210), fe = ue(3012);
                ce.exports = function(he, me) {
                  if (he.length === 1) return fe(me, he[0]);
                  if (me.length === 1) return fe(he, me[0]);
                  if (he.length === 0 || me.length === 0) return [0];
                  var pe = [0];
                  if (he.length < me.length) for (var ye = 0; ye < he.length; ++ye) pe = de(pe, fe(me, he[ye]));
                  else for (ye = 0; ye < me.length; ++ye) pe = de(pe, fe(he, me[ye]));
                  return pe;
                };
              }, 3012: function(ce, le, ue) {
                var de = ue(5250), fe = ue(9362);
                ce.exports = function(he, me) {
                  var pe = he.length;
                  if (pe === 1) {
                    var ye = de(he[0], me);
                    return ye[0] ? ye : [ye[1]];
                  }
                  var ge = new Array(2 * pe), ve = [0.1, 0.1], be = [0.1, 0.1], Ce = 0;
                  de(he[0], me, ve), ve[0] && (ge[Ce++] = ve[0]);
                  for (var Se = 1; Se < pe; ++Se) {
                    de(he[Se], me, be);
                    var we = ve[1];
                    fe(we, be[0], ve), ve[0] && (ge[Ce++] = ve[0]);
                    var Ee = be[1], xe = ve[1], Le = Ee + xe, Qe = xe - (Le - Ee);
                    ve[1] = Le, Qe && (ge[Ce++] = Qe);
                  }
                  return ve[1] && (ge[Ce++] = ve[1]), Ce === 0 && (ge[Ce++] = 0), ge.length = Ce, ge;
                };
              }, 1125: function(ce, le, ue) {
                ce.exports = function(fe, he, me, pe) {
                  var ye = de(fe, me, pe), ge = de(he, me, pe);
                  if (ye > 0 && ge > 0 || ye < 0 && ge < 0) return !1;
                  var ve = de(me, fe, he), be = de(pe, fe, he);
                  return !(ve > 0 && be > 0 || ve < 0 && be < 0) && (ye !== 0 || ge !== 0 || ve !== 0 || be !== 0 || function(Ce, Se, we, Ee) {
                    for (var xe = 0; xe < 2; ++xe) {
                      var Le = Ce[xe], Qe = Se[xe], ke = Math.min(Le, Qe), Me = Math.max(Le, Qe), Re = we[xe], De = Ee[xe], Pe = Math.min(Re, De);
                      if (Math.max(Re, De) < ke || Me < Pe) return !1;
                    }
                    return !0;
                  }(fe, he, me, pe));
                };
                var de = ue(3250)[3];
              }, 8545: function(ce) {
                ce.exports = function(le, ue) {
                  var de = 0 | le.length, fe = 0 | ue.length;
                  if (de === 1 && fe === 1) return function(Pe, $e) {
                    var qe = Pe + $e, Xe = qe - Pe, Ve = Pe - (qe - Xe) + ($e - Xe);
                    return Ve ? [Ve, qe] : [qe];
                  }(le[0], -ue[0]);
                  var he, me, pe = new Array(de + fe), ye = 0, ge = 0, ve = 0, be = Math.abs, Ce = le[ge], Se = be(Ce), we = -ue[ve], Ee = be(we);
                  Se < Ee ? (me = Ce, (ge += 1) < de && (Se = be(Ce = le[ge]))) : (me = we, (ve += 1) < fe && (Ee = be(we = -ue[ve]))), ge < de && Se < Ee || ve >= fe ? (he = Ce, (ge += 1) < de && (Se = be(Ce = le[ge]))) : (he = we, (ve += 1) < fe && (Ee = be(we = -ue[ve])));
                  for (var xe, Le, Qe = he + me, ke = Qe - he, Me = me - ke, Re = Me, De = Qe; ge < de && ve < fe; ) Se < Ee ? (he = Ce, (ge += 1) < de && (Se = be(Ce = le[ge]))) : (he = we, (ve += 1) < fe && (Ee = be(we = -ue[ve]))), (Me = (me = Re) - (ke = (Qe = he + me) - he)) && (pe[ye++] = Me), Re = De - ((xe = De + Qe) - (Le = xe - De)) + (Qe - Le), De = xe;
                  for (; ge < de; ) (Me = (me = Re) - (ke = (Qe = (he = Ce) + me) - he)) && (pe[ye++] = Me), Re = De - ((xe = De + Qe) - (Le = xe - De)) + (Qe - Le), De = xe, (ge += 1) < de && (Ce = le[ge]);
                  for (; ve < fe; ) (Me = (me = Re) - (ke = (Qe = (he = we) + me) - he)) && (pe[ye++] = Me), Re = De - ((xe = De + Qe) - (Le = xe - De)) + (Qe - Le), De = xe, (ve += 1) < fe && (we = -ue[ve]);
                  return Re && (pe[ye++] = Re), De && (pe[ye++] = De), ye || (pe[ye++] = 0), pe.length = ye, pe;
                };
              }, 8210: function(ce) {
                ce.exports = function(le, ue) {
                  var de = 0 | le.length, fe = 0 | ue.length;
                  if (de === 1 && fe === 1) return function(Pe, $e) {
                    var qe = Pe + $e, Xe = qe - Pe, Ve = Pe - (qe - Xe) + ($e - Xe);
                    return Ve ? [Ve, qe] : [qe];
                  }(le[0], ue[0]);
                  var he, me, pe = new Array(de + fe), ye = 0, ge = 0, ve = 0, be = Math.abs, Ce = le[ge], Se = be(Ce), we = ue[ve], Ee = be(we);
                  Se < Ee ? (me = Ce, (ge += 1) < de && (Se = be(Ce = le[ge]))) : (me = we, (ve += 1) < fe && (Ee = be(we = ue[ve]))), ge < de && Se < Ee || ve >= fe ? (he = Ce, (ge += 1) < de && (Se = be(Ce = le[ge]))) : (he = we, (ve += 1) < fe && (Ee = be(we = ue[ve])));
                  for (var xe, Le, Qe = he + me, ke = Qe - he, Me = me - ke, Re = Me, De = Qe; ge < de && ve < fe; ) Se < Ee ? (he = Ce, (ge += 1) < de && (Se = be(Ce = le[ge]))) : (he = we, (ve += 1) < fe && (Ee = be(we = ue[ve]))), (Me = (me = Re) - (ke = (Qe = he + me) - he)) && (pe[ye++] = Me), Re = De - ((xe = De + Qe) - (Le = xe - De)) + (Qe - Le), De = xe;
                  for (; ge < de; ) (Me = (me = Re) - (ke = (Qe = (he = Ce) + me) - he)) && (pe[ye++] = Me), Re = De - ((xe = De + Qe) - (Le = xe - De)) + (Qe - Le), De = xe, (ge += 1) < de && (Ce = le[ge]);
                  for (; ve < fe; ) (Me = (me = Re) - (ke = (Qe = (he = we) + me) - he)) && (pe[ye++] = Me), Re = De - ((xe = De + Qe) - (Le = xe - De)) + (Qe - Le), De = xe, (ve += 1) < fe && (we = ue[ve]);
                  return Re && (pe[ye++] = Re), De && (pe[ye++] = De), ye || (pe[ye++] = 0), pe.length = ye, pe;
                };
              }, 9127: function(ce, le, ue) {
                ce.exports = function(he) {
                  return fe(de(he));
                };
                var de = ue(6204), fe = ue(5771);
              }, 7765: function(ce, le, ue) {
                ce.exports = function(pe, ye, ge, ve) {
                  if (ge = ge || 0, ve === void 0 && (ve = function(Qe) {
                    for (var ke = Qe.length, Me = 0, Re = 0; Re < ke; ++Re) Me = 0 | Math.max(Me, Qe[Re].length);
                    return Me - 1;
                  }(pe)), pe.length === 0 || ve < 1) return { cells: [], vertexIds: [], vertexWeights: [] };
                  var be = function(Qe, ke) {
                    for (var Me = Qe.length, Re = fe.mallocUint8(Me), De = 0; De < Me; ++De) Re[De] = Qe[De] < ke | 0;
                    return Re;
                  }(ye, +ge), Ce = function(Qe, ke) {
                    for (var Me = Qe.length, Re = ke * (ke + 1) / 2 * Me | 0, De = fe.mallocUint32(2 * Re), Pe = 0, $e = 0; $e < Me; ++$e) for (var qe = Qe[$e], Xe = (ke = qe.length, 0); Xe < ke; ++Xe) for (var Ve = 0; Ve < Xe; ++Ve) {
                      var Ze = qe[Ve], Ge = qe[Xe];
                      De[Pe++] = 0 | Math.min(Ze, Ge), De[Pe++] = 0 | Math.max(Ze, Ge);
                    }
                    he(de(De, [Pe / 2 | 0, 2]));
                    var ze = 2;
                    for ($e = 2; $e < Pe; $e += 2) De[$e - 2] === De[$e] && De[$e - 1] === De[$e + 1] || (De[ze++] = De[$e], De[ze++] = De[$e + 1]);
                    return de(De, [ze / 2 | 0, 2]);
                  }(pe, ve), Se = function(Qe, ke, Me, Re) {
                    for (var De = Qe.data, Pe = Qe.shape[0], $e = fe.mallocDouble(Pe), qe = 0, Xe = 0; Xe < Pe; ++Xe) {
                      var Ve = De[2 * Xe], Ze = De[2 * Xe + 1];
                      if (Me[Ve] !== Me[Ze]) {
                        var Ge = ke[Ve], ze = ke[Ze];
                        De[2 * qe] = Ve, De[2 * qe + 1] = Ze, $e[qe++] = (ze - Re) / (ze - Ge);
                      }
                    }
                    return Qe.shape[0] = qe, de($e, [qe]);
                  }(Ce, ye, be, +ge), we = function(Qe, ke) {
                    var Me = fe.mallocInt32(2 * ke), Re = Qe.shape[0], De = Qe.data;
                    Me[0] = 0;
                    for (var Pe = 0, $e = 0; $e < Re; ++$e) {
                      var qe = De[2 * $e];
                      if (qe !== Pe) {
                        for (Me[2 * Pe + 1] = $e; ++Pe < qe; ) Me[2 * Pe] = $e, Me[2 * Pe + 1] = $e;
                        Me[2 * Pe] = $e;
                      }
                    }
                    for (Me[2 * Pe + 1] = Re; ++Pe < ke; ) Me[2 * Pe] = Me[2 * Pe + 1] = Re;
                    return Me;
                  }(Ce, 0 | ye.length), Ee = me(ve)(pe, Ce.data, we, be), xe = function(Qe) {
                    for (var ke = 0 | Qe.shape[0], Me = Qe.data, Re = new Array(ke), De = 0; De < ke; ++De) Re[De] = [Me[2 * De], Me[2 * De + 1]];
                    return Re;
                  }(Ce), Le = [].slice.call(Se.data, 0, Se.shape[0]);
                  return fe.free(be), fe.free(Ce.data), fe.free(Se.data), fe.free(we), { cells: Ee, vertexIds: xe, vertexWeights: Le };
                };
                var de = ue(9618), fe = ue(1888), he = ue(446), me = ue(1570);
              }, 1570: function(ce) {
                ce.exports = function(ue) {
                  return le[ue]();
                };
                var le = [function() {
                  return function(ue, de, fe, he) {
                    for (var me = ue.length, pe = 0; pe < me; ++pe) ue[pe].length;
                    return [];
                  };
                }, function() {
                  function ue(de, fe, he, me) {
                    for (var pe = 0 | Math.min(he, me), ye = 0 | Math.max(he, me), ge = de[2 * pe], ve = de[2 * pe + 1]; ge < ve; ) {
                      var be = ge + ve >> 1, Ce = fe[2 * be + 1];
                      if (Ce === ye) return be;
                      ye < Ce ? ve = be : ge = be + 1;
                    }
                    return ge;
                  }
                  return function(de, fe, he, me) {
                    for (var pe = de.length, ye = [], ge = 0; ge < pe; ++ge) {
                      var ve = de[ge];
                      if (ve.length === 2) {
                        var be = (me[ve[0]] << 0) + (me[ve[1]] << 1);
                        if (be === 0 || be === 3) continue;
                        switch (be) {
                          case 0:
                          case 3:
                            break;
                          case 1:
                            ye.push([ue(he, fe, ve[0], ve[1])]);
                            break;
                          case 2:
                            ye.push([ue(he, fe, ve[1], ve[0])]);
                        }
                      }
                    }
                    return ye;
                  };
                }, function() {
                  function ue(de, fe, he, me) {
                    for (var pe = 0 | Math.min(he, me), ye = 0 | Math.max(he, me), ge = de[2 * pe], ve = de[2 * pe + 1]; ge < ve; ) {
                      var be = ge + ve >> 1, Ce = fe[2 * be + 1];
                      if (Ce === ye) return be;
                      ye < Ce ? ve = be : ge = be + 1;
                    }
                    return ge;
                  }
                  return function(de, fe, he, me) {
                    for (var pe = de.length, ye = [], ge = 0; ge < pe; ++ge) {
                      var ve = de[ge], be = ve.length;
                      if (be === 3) {
                        if ((Ce = (me[ve[0]] << 0) + (me[ve[1]] << 1) + (me[ve[2]] << 2)) == 0 || Ce === 7) continue;
                        switch (Ce) {
                          case 0:
                          case 7:
                            break;
                          case 1:
                            ye.push([ue(he, fe, ve[0], ve[2]), ue(he, fe, ve[0], ve[1])]);
                            break;
                          case 2:
                            ye.push([ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[1], ve[2])]);
                            break;
                          case 3:
                            ye.push([ue(he, fe, ve[0], ve[2]), ue(he, fe, ve[1], ve[2])]);
                            break;
                          case 4:
                            ye.push([ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[2], ve[0])]);
                            break;
                          case 5:
                            ye.push([ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[0], ve[1])]);
                            break;
                          case 6:
                            ye.push([ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[2], ve[0])]);
                        }
                      } else if (be === 2) {
                        var Ce;
                        if ((Ce = (me[ve[0]] << 0) + (me[ve[1]] << 1)) == 0 || Ce === 3) continue;
                        switch (Ce) {
                          case 0:
                          case 3:
                            break;
                          case 1:
                            ye.push([ue(he, fe, ve[0], ve[1])]);
                            break;
                          case 2:
                            ye.push([ue(he, fe, ve[1], ve[0])]);
                        }
                      }
                    }
                    return ye;
                  };
                }, function() {
                  function ue(de, fe, he, me) {
                    for (var pe = 0 | Math.min(he, me), ye = 0 | Math.max(he, me), ge = de[2 * pe], ve = de[2 * pe + 1]; ge < ve; ) {
                      var be = ge + ve >> 1, Ce = fe[2 * be + 1];
                      if (Ce === ye) return be;
                      ye < Ce ? ve = be : ge = be + 1;
                    }
                    return ge;
                  }
                  return function(de, fe, he, me) {
                    for (var pe = de.length, ye = [], ge = 0; ge < pe; ++ge) {
                      var ve = de[ge], be = ve.length;
                      if (be === 4) {
                        if ((Ce = (me[ve[0]] << 0) + (me[ve[1]] << 1) + (me[ve[2]] << 2) + (me[ve[3]] << 3)) == 0 || Ce === 15) continue;
                        switch (Ce) {
                          case 0:
                          case 15:
                            break;
                          case 1:
                            ye.push([ue(he, fe, ve[0], ve[1]), ue(he, fe, ve[0], ve[2]), ue(he, fe, ve[0], ve[3])]);
                            break;
                          case 2:
                            ye.push([ue(he, fe, ve[1], ve[2]), ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[1], ve[3])]);
                            break;
                          case 3:
                            ye.push([ue(he, fe, ve[1], ve[2]), ue(he, fe, ve[0], ve[2]), ue(he, fe, ve[0], ve[3])], [ue(he, fe, ve[1], ve[3]), ue(he, fe, ve[1], ve[2]), ue(he, fe, ve[0], ve[3])]);
                            break;
                          case 4:
                            ye.push([ue(he, fe, ve[2], ve[0]), ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[2], ve[3])]);
                            break;
                          case 5:
                            ye.push([ue(he, fe, ve[0], ve[1]), ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[0], ve[3])], [ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[2], ve[3]), ue(he, fe, ve[0], ve[3])]);
                            break;
                          case 6:
                            ye.push([ue(he, fe, ve[2], ve[0]), ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[1], ve[3])], [ue(he, fe, ve[2], ve[3]), ue(he, fe, ve[2], ve[0]), ue(he, fe, ve[1], ve[3])]);
                            break;
                          case 7:
                            ye.push([ue(he, fe, ve[0], ve[3]), ue(he, fe, ve[1], ve[3]), ue(he, fe, ve[2], ve[3])]);
                            break;
                          case 8:
                            ye.push([ue(he, fe, ve[3], ve[1]), ue(he, fe, ve[3], ve[0]), ue(he, fe, ve[3], ve[2])]);
                            break;
                          case 9:
                            ye.push([ue(he, fe, ve[3], ve[1]), ue(he, fe, ve[0], ve[1]), ue(he, fe, ve[0], ve[2])], [ue(he, fe, ve[3], ve[2]), ue(he, fe, ve[3], ve[1]), ue(he, fe, ve[0], ve[2])]);
                            break;
                          case 10:
                            ye.push([ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[3], ve[0]), ue(he, fe, ve[1], ve[2])], [ue(he, fe, ve[3], ve[0]), ue(he, fe, ve[3], ve[2]), ue(he, fe, ve[1], ve[2])]);
                            break;
                          case 11:
                            ye.push([ue(he, fe, ve[1], ve[2]), ue(he, fe, ve[0], ve[2]), ue(he, fe, ve[3], ve[2])]);
                            break;
                          case 12:
                            ye.push([ue(he, fe, ve[3], ve[0]), ue(he, fe, ve[2], ve[0]), ue(he, fe, ve[2], ve[1])], [ue(he, fe, ve[3], ve[1]), ue(he, fe, ve[3], ve[0]), ue(he, fe, ve[2], ve[1])]);
                            break;
                          case 13:
                            ye.push([ue(he, fe, ve[0], ve[1]), ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[3], ve[1])]);
                            break;
                          case 14:
                            ye.push([ue(he, fe, ve[2], ve[0]), ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[3], ve[0])]);
                        }
                      } else if (be === 3) {
                        if ((Ce = (me[ve[0]] << 0) + (me[ve[1]] << 1) + (me[ve[2]] << 2)) == 0 || Ce === 7) continue;
                        switch (Ce) {
                          case 0:
                          case 7:
                            break;
                          case 1:
                            ye.push([ue(he, fe, ve[0], ve[2]), ue(he, fe, ve[0], ve[1])]);
                            break;
                          case 2:
                            ye.push([ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[1], ve[2])]);
                            break;
                          case 3:
                            ye.push([ue(he, fe, ve[0], ve[2]), ue(he, fe, ve[1], ve[2])]);
                            break;
                          case 4:
                            ye.push([ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[2], ve[0])]);
                            break;
                          case 5:
                            ye.push([ue(he, fe, ve[2], ve[1]), ue(he, fe, ve[0], ve[1])]);
                            break;
                          case 6:
                            ye.push([ue(he, fe, ve[1], ve[0]), ue(he, fe, ve[2], ve[0])]);
                        }
                      } else if (be === 2) {
                        var Ce;
                        if ((Ce = (me[ve[0]] << 0) + (me[ve[1]] << 1)) == 0 || Ce === 3) continue;
                        switch (Ce) {
                          case 0:
                          case 3:
                            break;
                          case 1:
                            ye.push([ue(he, fe, ve[0], ve[1])]);
                            break;
                          case 2:
                            ye.push([ue(he, fe, ve[1], ve[0])]);
                        }
                      }
                    }
                    return ye;
                  };
                }];
              }, 6803: function(ce, le, ue) {
                ue(8828), ue(1755);
                function de(fe, he) {
                  var me = fe.length, pe = fe.length - he.length, ye = Math.min;
                  if (pe) return pe;
                  switch (me) {
                    case 0:
                      return 0;
                    case 1:
                      return fe[0] - he[0];
                    case 2:
                      return (be = fe[0] + fe[1] - he[0] - he[1]) || ye(fe[0], fe[1]) - ye(he[0], he[1]);
                    case 3:
                      var ge = fe[0] + fe[1], ve = he[0] + he[1];
                      if (be = ge + fe[2] - (ve + he[2])) return be;
                      var be, Ce = ye(fe[0], fe[1]), Se = ye(he[0], he[1]);
                      return (be = ye(Ce, fe[2]) - ye(Se, he[2])) || ye(Ce + fe[2], ge) - ye(Se + he[2], ve);
                    default:
                      var we = fe.slice(0);
                      we.sort();
                      var Ee = he.slice(0);
                      Ee.sort();
                      for (var xe = 0; xe < me; ++xe) if (pe = we[xe] - Ee[xe]) return pe;
                      return 0;
                  }
                }
                le.Fw = de;
              }, 3105: function(ce, le) {
                function ue(fe) {
                  var he = 32;
                  return (fe &= -fe) && he--, 65535 & fe && (he -= 16), 16711935 & fe && (he -= 8), 252645135 & fe && (he -= 4), 858993459 & fe && (he -= 2), 1431655765 & fe && (he -= 1), he;
                }
                le.INT_BITS = 32, le.INT_MAX = 2147483647, le.INT_MIN = -2147483648, le.sign = function(fe) {
                  return (fe > 0) - (fe < 0);
                }, le.abs = function(fe) {
                  var he = fe >> 31;
                  return (fe ^ he) - he;
                }, le.min = function(fe, he) {
                  return he ^ (fe ^ he) & -(fe < he);
                }, le.max = function(fe, he) {
                  return fe ^ (fe ^ he) & -(fe < he);
                }, le.isPow2 = function(fe) {
                  return !(fe & fe - 1 || !fe);
                }, le.log2 = function(fe) {
                  var he, me;
                  return he = (fe > 65535) << 4, he |= me = ((fe >>>= he) > 255) << 3, he |= me = ((fe >>>= me) > 15) << 2, (he |= me = ((fe >>>= me) > 3) << 1) | (fe >>>= me) >> 1;
                }, le.log10 = function(fe) {
                  return fe >= 1e9 ? 9 : fe >= 1e8 ? 8 : fe >= 1e7 ? 7 : fe >= 1e6 ? 6 : fe >= 1e5 ? 5 : fe >= 1e4 ? 4 : fe >= 1e3 ? 3 : fe >= 100 ? 2 : fe >= 10 ? 1 : 0;
                }, le.popCount = function(fe) {
                  return 16843009 * ((fe = (858993459 & (fe -= fe >>> 1 & 1431655765)) + (fe >>> 2 & 858993459)) + (fe >>> 4) & 252645135) >>> 24;
                }, le.countTrailingZeros = ue, le.nextPow2 = function(fe) {
                  return fe += fe === 0, --fe, fe |= fe >>> 1, fe |= fe >>> 2, fe |= fe >>> 4, fe |= fe >>> 8, 1 + (fe |= fe >>> 16);
                }, le.prevPow2 = function(fe) {
                  return fe |= fe >>> 1, fe |= fe >>> 2, fe |= fe >>> 4, fe |= fe >>> 8, (fe |= fe >>> 16) - (fe >>> 1);
                }, le.parity = function(fe) {
                  return fe ^= fe >>> 16, fe ^= fe >>> 8, fe ^= fe >>> 4, 27030 >>> (fe &= 15) & 1;
                };
                var de = new Array(256);
                (function(fe) {
                  for (var he = 0; he < 256; ++he) {
                    var me = he, pe = he, ye = 7;
                    for (me >>>= 1; me; me >>>= 1) pe <<= 1, pe |= 1 & me, --ye;
                    fe[he] = pe << ye & 255;
                  }
                })(de), le.reverse = function(fe) {
                  return de[255 & fe] << 24 | de[fe >>> 8 & 255] << 16 | de[fe >>> 16 & 255] << 8 | de[fe >>> 24 & 255];
                }, le.interleave2 = function(fe, he) {
                  return (fe = 1431655765 & ((fe = 858993459 & ((fe = 252645135 & ((fe = 16711935 & ((fe &= 65535) | fe << 8)) | fe << 4)) | fe << 2)) | fe << 1)) | (he = 1431655765 & ((he = 858993459 & ((he = 252645135 & ((he = 16711935 & ((he &= 65535) | he << 8)) | he << 4)) | he << 2)) | he << 1)) << 1;
                }, le.deinterleave2 = function(fe, he) {
                  return (fe = 65535 & ((fe = 16711935 & ((fe = 252645135 & ((fe = 858993459 & ((fe = fe >>> he & 1431655765) | fe >>> 1)) | fe >>> 2)) | fe >>> 4)) | fe >>> 16)) << 16 >> 16;
                }, le.interleave3 = function(fe, he, me) {
                  return fe = 1227133513 & ((fe = 3272356035 & ((fe = 251719695 & ((fe = 4278190335 & ((fe &= 1023) | fe << 16)) | fe << 8)) | fe << 4)) | fe << 2), (fe |= (he = 1227133513 & ((he = 3272356035 & ((he = 251719695 & ((he = 4278190335 & ((he &= 1023) | he << 16)) | he << 8)) | he << 4)) | he << 2)) << 1) | (me = 1227133513 & ((me = 3272356035 & ((me = 251719695 & ((me = 4278190335 & ((me &= 1023) | me << 16)) | me << 8)) | me << 4)) | me << 2)) << 2;
                }, le.deinterleave3 = function(fe, he) {
                  return (fe = 1023 & ((fe = 4278190335 & ((fe = 251719695 & ((fe = 3272356035 & ((fe = fe >>> he & 1227133513) | fe >>> 2)) | fe >>> 4)) | fe >>> 8)) | fe >>> 16)) << 22 >> 22;
                }, le.nextCombination = function(fe) {
                  var he = fe | fe - 1;
                  return he + 1 | (~he & -~he) - 1 >>> ue(fe) + 1;
                };
              }, 2014: function(ce, le, ue) {
                var de = ue(3105), fe = ue(4623);
                function he(Ce, Se) {
                  var we = Ce.length, Ee = Ce.length - Se.length, xe = Math.min;
                  if (Ee) return Ee;
                  switch (we) {
                    case 0:
                      return 0;
                    case 1:
                      return Ce[0] - Se[0];
                    case 2:
                      return (ke = Ce[0] + Ce[1] - Se[0] - Se[1]) || xe(Ce[0], Ce[1]) - xe(Se[0], Se[1]);
                    case 3:
                      var Le = Ce[0] + Ce[1], Qe = Se[0] + Se[1];
                      if (ke = Le + Ce[2] - (Qe + Se[2])) return ke;
                      var ke, Me = xe(Ce[0], Ce[1]), Re = xe(Se[0], Se[1]);
                      return (ke = xe(Me, Ce[2]) - xe(Re, Se[2])) || xe(Me + Ce[2], Le) - xe(Re + Se[2], Qe);
                    default:
                      var De = Ce.slice(0);
                      De.sort();
                      var Pe = Se.slice(0);
                      Pe.sort();
                      for (var $e = 0; $e < we; ++$e) if (Ee = De[$e] - Pe[$e]) return Ee;
                      return 0;
                  }
                }
                function me(Ce, Se) {
                  return he(Ce[0], Se[0]);
                }
                function pe(Ce, Se) {
                  if (Se) {
                    for (var we = Ce.length, Ee = new Array(we), xe = 0; xe < we; ++xe) Ee[xe] = [Ce[xe], Se[xe]];
                    for (Ee.sort(me), xe = 0; xe < we; ++xe) Ce[xe] = Ee[xe][0], Se[xe] = Ee[xe][1];
                    return Ce;
                  }
                  return Ce.sort(he), Ce;
                }
                function ye(Ce) {
                  if (Ce.length === 0) return [];
                  for (var Se = 1, we = Ce.length, Ee = 1; Ee < we; ++Ee) {
                    var xe = Ce[Ee];
                    if (he(xe, Ce[Ee - 1])) {
                      if (Ee === Se) {
                        Se++;
                        continue;
                      }
                      Ce[Se++] = xe;
                    }
                  }
                  return Ce.length = Se, Ce;
                }
                function ge(Ce, Se) {
                  for (var we = 0, Ee = Ce.length - 1, xe = -1; we <= Ee; ) {
                    var Le = we + Ee >> 1, Qe = he(Ce[Le], Se);
                    Qe <= 0 ? (Qe === 0 && (xe = Le), we = Le + 1) : Qe > 0 && (Ee = Le - 1);
                  }
                  return xe;
                }
                function ve(Ce, Se) {
                  for (var we = new Array(Ce.length), Ee = 0, xe = we.length; Ee < xe; ++Ee) we[Ee] = [];
                  for (var Le = [], Qe = (Ee = 0, Se.length); Ee < Qe; ++Ee) for (var ke = Se[Ee], Me = ke.length, Re = 1, De = 1 << Me; Re < De; ++Re) {
                    Le.length = de.popCount(Re);
                    for (var Pe = 0, $e = 0; $e < Me; ++$e) Re & 1 << $e && (Le[Pe++] = ke[$e]);
                    var qe = ge(Ce, Le);
                    if (!(qe < 0)) for (; we[qe++].push(Ee), !(qe >= Ce.length || he(Ce[qe], Le) !== 0); ) ;
                  }
                  return we;
                }
                function be(Ce, Se) {
                  if (Se < 0) return [];
                  for (var we = [], Ee = (1 << Se + 1) - 1, xe = 0; xe < Ce.length; ++xe) for (var Le = Ce[xe], Qe = Ee; Qe < 1 << Le.length; Qe = de.nextCombination(Qe)) {
                    for (var ke = new Array(Se + 1), Me = 0, Re = 0; Re < Le.length; ++Re) Qe & 1 << Re && (ke[Me++] = Le[Re]);
                    we.push(ke);
                  }
                  return pe(we);
                }
                le.dimension = function(Ce) {
                  for (var Se = 0, we = Math.max, Ee = 0, xe = Ce.length; Ee < xe; ++Ee) Se = we(Se, Ce[Ee].length);
                  return Se - 1;
                }, le.countVertices = function(Ce) {
                  for (var Se = -1, we = Math.max, Ee = 0, xe = Ce.length; Ee < xe; ++Ee) for (var Le = Ce[Ee], Qe = 0, ke = Le.length; Qe < ke; ++Qe) Se = we(Se, Le[Qe]);
                  return Se + 1;
                }, le.cloneCells = function(Ce) {
                  for (var Se = new Array(Ce.length), we = 0, Ee = Ce.length; we < Ee; ++we) Se[we] = Ce[we].slice(0);
                  return Se;
                }, le.compareCells = he, le.normalize = pe, le.unique = ye, le.findCell = ge, le.incidence = ve, le.dual = function(Ce, Se) {
                  if (!Se) return ve(ye(be(Ce, 0)), Ce);
                  for (var we = new Array(Se), Ee = 0; Ee < Se; ++Ee) we[Ee] = [];
                  Ee = 0;
                  for (var xe = Ce.length; Ee < xe; ++Ee) for (var Le = Ce[Ee], Qe = 0, ke = Le.length; Qe < ke; ++Qe) we[Le[Qe]].push(Ee);
                  return we;
                }, le.explode = function(Ce) {
                  for (var Se = [], we = 0, Ee = Ce.length; we < Ee; ++we) for (var xe = Ce[we], Le = 0 | xe.length, Qe = 1, ke = 1 << Le; Qe < ke; ++Qe) {
                    for (var Me = [], Re = 0; Re < Le; ++Re) Qe >>> Re & 1 && Me.push(xe[Re]);
                    Se.push(Me);
                  }
                  return pe(Se);
                }, le.skeleton = be, le.boundary = function(Ce) {
                  for (var Se = [], we = 0, Ee = Ce.length; we < Ee; ++we) for (var xe = Ce[we], Le = 0, Qe = xe.length; Le < Qe; ++Le) {
                    for (var ke = new Array(xe.length - 1), Me = 0, Re = 0; Me < Qe; ++Me) Me !== Le && (ke[Re++] = xe[Me]);
                    Se.push(ke);
                  }
                  return pe(Se);
                }, le.connectedComponents = function(Ce, Se) {
                  return Se ? function(we, Ee) {
                    for (var xe = new fe(Ee), Le = 0; Le < we.length; ++Le) for (var Qe = we[Le], ke = 0; ke < Qe.length; ++ke) for (var Me = ke + 1; Me < Qe.length; ++Me) xe.link(Qe[ke], Qe[Me]);
                    var Re = [], De = xe.ranks;
                    for (Le = 0; Le < De.length; ++Le) De[Le] = -1;
                    for (Le = 0; Le < we.length; ++Le) {
                      var Pe = xe.find(we[Le][0]);
                      De[Pe] < 0 ? (De[Pe] = Re.length, Re.push([we[Le].slice(0)])) : Re[De[Pe]].push(we[Le].slice(0));
                    }
                    return Re;
                  }(Ce, Se) : function(we) {
                    for (var Ee = ye(pe(be(we, 0))), xe = new fe(Ee.length), Le = 0; Le < we.length; ++Le) for (var Qe = we[Le], ke = 0; ke < Qe.length; ++ke) for (var Me = ge(Ee, [Qe[ke]]), Re = ke + 1; Re < Qe.length; ++Re) xe.link(Me, ge(Ee, [Qe[Re]]));
                    var De = [], Pe = xe.ranks;
                    for (Le = 0; Le < Pe.length; ++Le) Pe[Le] = -1;
                    for (Le = 0; Le < we.length; ++Le) {
                      var $e = xe.find(ge(Ee, [we[Le][0]]));
                      Pe[$e] < 0 ? (Pe[$e] = De.length, De.push([we[Le].slice(0)])) : De[Pe[$e]].push(we[Le].slice(0));
                    }
                    return De;
                  }(Ce);
                };
              }, 4623: function(ce) {
                function le(ue) {
                  this.roots = new Array(ue), this.ranks = new Array(ue);
                  for (var de = 0; de < ue; ++de) this.roots[de] = de, this.ranks[de] = 0;
                }
                ce.exports = le, le.prototype.length = function() {
                  return this.roots.length;
                }, le.prototype.makeSet = function() {
                  var ue = this.roots.length;
                  return this.roots.push(ue), this.ranks.push(0), ue;
                }, le.prototype.find = function(ue) {
                  for (var de = this.roots; de[ue] !== ue; ) {
                    var fe = de[ue];
                    de[ue] = de[fe], ue = fe;
                  }
                  return ue;
                }, le.prototype.link = function(ue, de) {
                  var fe = this.find(ue), he = this.find(de);
                  if (fe !== he) {
                    var me = this.ranks, pe = this.roots, ye = me[fe], ge = me[he];
                    ye < ge ? pe[fe] = he : ge < ye ? pe[he] = fe : (pe[he] = fe, ++me[fe]);
                  }
                };
              }, 5878: function(ce, le, ue) {
                ce.exports = function(he, me, pe) {
                  for (var ye = me.length, ge = he.length, ve = new Array(ye), be = new Array(ye), Ce = new Array(ye), Se = new Array(ye), we = 0; we < ye; ++we) ve[we] = be[we] = -1, Ce[we] = 1 / 0, Se[we] = !1;
                  for (we = 0; we < ge; ++we) {
                    var Ee = he[we];
                    if (Ee.length !== 2) throw new Error("Input must be a graph");
                    var xe = Ee[1], Le = Ee[0];
                    be[Le] !== -1 ? be[Le] = -2 : be[Le] = xe, ve[xe] !== -1 ? ve[xe] = -2 : ve[xe] = Le;
                  }
                  function Qe(et) {
                    if (Se[et]) return 1 / 0;
                    var dt, Je, it, st = ve[et], gt = be[et];
                    return st < 0 || gt < 0 ? 1 / 0 : (dt = me[et], Je = me[st], it = me[gt], Math.abs(de(dt, Je, it)) / Math.sqrt(Math.pow(Je[0] - it[0], 2) + Math.pow(Je[1] - it[1], 2)));
                  }
                  function ke(et, dt) {
                    var Je = Ve[et], it = Ve[dt];
                    Ve[et] = it, Ve[dt] = Je, Ze[Je] = dt, Ze[it] = et;
                  }
                  function Me(et) {
                    return Ce[Ve[et]];
                  }
                  function Re(et) {
                    return 1 & et ? et - 1 >> 1 : (et >> 1) - 1;
                  }
                  function De(et) {
                    for (var dt = Me(et); ; ) {
                      var Je = dt, it = 2 * et + 1, st = 2 * (et + 1), gt = et;
                      if (it < Ge) {
                        var bt = Me(it);
                        bt < Je && (gt = it, Je = bt);
                      }
                      if (st < Ge && Me(st) < Je && (gt = st), gt === et) return et;
                      ke(et, gt), et = gt;
                    }
                  }
                  function Pe(et) {
                    for (var dt = Me(et); et > 0; ) {
                      var Je = Re(et);
                      if (!(Je >= 0 && dt < Me(Je))) return et;
                      ke(et, Je), et = Je;
                    }
                  }
                  function $e() {
                    if (Ge > 0) {
                      var et = Ve[0];
                      return ke(0, Ge - 1), Ge -= 1, De(0), et;
                    }
                    return -1;
                  }
                  function qe(et, dt) {
                    var Je = Ve[et];
                    return Ce[Je] === dt ? et : (Ce[Je] = -1 / 0, Pe(et), $e(), Ce[Je] = dt, Pe((Ge += 1) - 1));
                  }
                  function Xe(et) {
                    if (!Se[et]) {
                      Se[et] = !0;
                      var dt = ve[et], Je = be[et];
                      ve[Je] >= 0 && (ve[Je] = dt), be[dt] >= 0 && (be[dt] = Je), Ze[dt] >= 0 && qe(Ze[dt], Qe(dt)), Ze[Je] >= 0 && qe(Ze[Je], Qe(Je));
                    }
                  }
                  var Ve = [], Ze = new Array(ye);
                  for (we = 0; we < ye; ++we) (Ce[we] = Qe(we)) < 1 / 0 ? (Ze[we] = Ve.length, Ve.push(we)) : Ze[we] = -1;
                  var Ge = Ve.length;
                  for (we = Ge >> 1; we >= 0; --we) De(we);
                  for (; ; ) {
                    var ze = $e();
                    if (ze < 0 || Ce[ze] > pe) break;
                    Xe(ze);
                  }
                  var We = [];
                  for (we = 0; we < ye; ++we) Se[we] || (Ze[we] = We.length, We.push(me[we].slice()));
                  function at(et, dt) {
                    if (et[dt] < 0) return dt;
                    var Je = dt, it = dt;
                    do {
                      var st = et[it];
                      if (!Se[it] || st < 0 || st === it || (st = et[it = st], !Se[it] || st < 0 || st === it)) break;
                      it = st, Je = et[Je];
                    } while (Je !== it);
                    for (var gt = dt; gt !== it; gt = et[gt]) et[gt] = it;
                    return it;
                  }
                  We.length;
                  var ct = [];
                  return he.forEach(function(et) {
                    var dt = at(ve, et[0]), Je = at(be, et[1]);
                    if (dt >= 0 && Je >= 0 && dt !== Je) {
                      var it = Ze[dt], st = Ze[Je];
                      it !== st && ct.push([it, st]);
                    }
                  }), fe.unique(fe.normalize(ct)), { positions: We, edges: ct };
                };
                var de = ue(3250), fe = ue(2014);
              }, 1303: function(ce, le, ue) {
                ce.exports = function(he, me) {
                  var pe, ye, ge, ve;
                  if (me[0][0] < me[1][0]) pe = me[0], ye = me[1];
                  else {
                    if (!(me[0][0] > me[1][0])) return fe(me, he);
                    pe = me[1], ye = me[0];
                  }
                  if (he[0][0] < he[1][0]) ge = he[0], ve = he[1];
                  else {
                    if (!(he[0][0] > he[1][0])) return -fe(he, me);
                    ge = he[1], ve = he[0];
                  }
                  var be = de(pe, ye, ve), Ce = de(pe, ye, ge);
                  if (be < 0) {
                    if (Ce <= 0) return be;
                  } else if (be > 0) {
                    if (Ce >= 0) return be;
                  } else if (Ce) return Ce;
                  if (be = de(ve, ge, ye), Ce = de(ve, ge, pe), be < 0) {
                    if (Ce <= 0) return be;
                  } else if (be > 0) {
                    if (Ce >= 0) return be;
                  } else if (Ce) return Ce;
                  return ye[0] - ve[0];
                };
                var de = ue(3250);
                function fe(he, me) {
                  var pe, ye, ge, ve;
                  if (me[0][0] < me[1][0]) pe = me[0], ye = me[1];
                  else {
                    if (!(me[0][0] > me[1][0])) {
                      var be = Math.min(he[0][1], he[1][1]), Ce = Math.max(he[0][1], he[1][1]), Se = Math.min(me[0][1], me[1][1]), we = Math.max(me[0][1], me[1][1]);
                      return Ce < Se ? Ce - Se : be > we ? be - we : Ce - we;
                    }
                    pe = me[1], ye = me[0];
                  }
                  he[0][1] < he[1][1] ? (ge = he[0], ve = he[1]) : (ge = he[1], ve = he[0]);
                  var Ee = de(ye, pe, ge);
                  return Ee || (Ee = de(ye, pe, ve)) || ve - ye;
                }
              }, 4209: function(ce, le, ue) {
                ce.exports = function(Ce) {
                  for (var Se = Ce.length, we = 2 * Se, Ee = new Array(we), xe = 0; xe < Se; ++xe) {
                    var Le = Ce[xe], Qe = Le[0][0] < Le[1][0];
                    Ee[2 * xe] = new be(Le[0][0], Le, Qe, xe), Ee[2 * xe + 1] = new be(Le[1][0], Le, !Qe, xe);
                  }
                  Ee.sort(function(Xe, Ve) {
                    var Ze = Xe.x - Ve.x;
                    return Ze || (Ze = Xe.create - Ve.create) || Math.min(Xe.segment[0][1], Xe.segment[1][1]) - Math.min(Ve.segment[0][1], Ve.segment[1][1]);
                  });
                  var ke = fe(me), Me = [], Re = [], De = [];
                  for (xe = 0; xe < we; ) {
                    for (var Pe = Ee[xe].x, $e = []; xe < we; ) {
                      var qe = Ee[xe];
                      if (qe.x !== Pe) break;
                      xe += 1, qe.segment[0][0] === qe.x && qe.segment[1][0] === qe.x ? qe.create && (qe.segment[0][1] < qe.segment[1][1] ? ($e.push(new ve(qe.segment[0][1], qe.index, !0, !0)), $e.push(new ve(qe.segment[1][1], qe.index, !1, !1))) : ($e.push(new ve(qe.segment[1][1], qe.index, !0, !1)), $e.push(new ve(qe.segment[0][1], qe.index, !1, !0)))) : ke = qe.create ? ke.insert(qe.segment, qe.index) : ke.remove(qe.segment);
                    }
                    Me.push(ke.root), Re.push(Pe), De.push($e);
                  }
                  return new pe(Me, Re, De);
                };
                var de = ue(2478), fe = ue(3840), he = ue(3250), me = ue(1303);
                function pe(Ce, Se, we) {
                  this.slabs = Ce, this.coordinates = Se, this.horizontal = we;
                }
                function ye(Ce, Se) {
                  return Ce.y - Se;
                }
                function ge(Ce, Se) {
                  for (var we = null; Ce; ) {
                    var Ee, xe, Le = Ce.key;
                    Le[0][0] < Le[1][0] ? (Ee = Le[0], xe = Le[1]) : (Ee = Le[1], xe = Le[0]);
                    var Qe = he(Ee, xe, Se);
                    if (Qe < 0) Ce = Ce.left;
                    else if (Qe > 0) if (Se[0] !== Le[1][0]) we = Ce, Ce = Ce.right;
                    else {
                      if (ke = ge(Ce.right, Se)) return ke;
                      Ce = Ce.left;
                    }
                    else {
                      if (Se[0] !== Le[1][0]) return Ce;
                      var ke;
                      if (ke = ge(Ce.right, Se)) return ke;
                      Ce = Ce.left;
                    }
                  }
                  return we;
                }
                function ve(Ce, Se, we, Ee) {
                  this.y = Ce, this.index = Se, this.start = we, this.closed = Ee;
                }
                function be(Ce, Se, we, Ee) {
                  this.x = Ce, this.segment = Se, this.create = we, this.index = Ee;
                }
                pe.prototype.castUp = function(Ce) {
                  var Se = de.le(this.coordinates, Ce[0]);
                  if (Se < 0) return -1;
                  this.slabs[Se];
                  var we = ge(this.slabs[Se], Ce), Ee = -1;
                  if (we && (Ee = we.value), this.coordinates[Se] === Ce[0]) {
                    var xe = null;
                    if (we && (xe = we.key), Se > 0) {
                      var Le = ge(this.slabs[Se - 1], Ce);
                      Le && (xe ? me(Le.key, xe) > 0 && (xe = Le.key, Ee = Le.value) : (Ee = Le.value, xe = Le.key));
                    }
                    var Qe = this.horizontal[Se];
                    if (Qe.length > 0) {
                      var ke = de.ge(Qe, Ce[1], ye);
                      if (ke < Qe.length) {
                        var Me = Qe[ke];
                        if (Ce[1] === Me.y) {
                          if (Me.closed) return Me.index;
                          for (; ke < Qe.length - 1 && Qe[ke + 1].y === Ce[1]; ) if ((Me = Qe[ke += 1]).closed) return Me.index;
                          if (Me.y === Ce[1] && !Me.start) {
                            if ((ke += 1) >= Qe.length) return Ee;
                            Me = Qe[ke];
                          }
                        }
                        if (Me.start) if (xe) {
                          var Re = he(xe[0], xe[1], [Ce[0], Me.y]);
                          xe[0][0] > xe[1][0] && (Re = -Re), Re > 0 && (Ee = Me.index);
                        } else Ee = Me.index;
                        else Me.y !== Ce[1] && (Ee = Me.index);
                      }
                    }
                  }
                  return Ee;
                };
              }, 5202: function(ce, le, ue) {
                var de = ue(1944), fe = ue(8210);
                function he(pe, ye) {
                  var ge = fe(de(pe, ye), [ye[ye.length - 1]]);
                  return ge[ge.length - 1];
                }
                function me(pe, ye, ge, ve) {
                  var be = -ye / (ve - ye);
                  be < 0 ? be = 0 : be > 1 && (be = 1);
                  for (var Ce = 1 - be, Se = pe.length, we = new Array(Se), Ee = 0; Ee < Se; ++Ee) we[Ee] = be * pe[Ee] + Ce * ge[Ee];
                  return we;
                }
                ce.exports = function(pe, ye) {
                  for (var ge = [], ve = [], be = he(pe[pe.length - 1], ye), Ce = pe[pe.length - 1], Se = pe[0], we = 0; we < pe.length; ++we, Ce = Se) {
                    var Ee = he(Se = pe[we], ye);
                    if (be < 0 && Ee > 0 || be > 0 && Ee < 0) {
                      var xe = me(Ce, Ee, Se, be);
                      ge.push(xe), ve.push(xe.slice());
                    }
                    Ee < 0 ? ve.push(Se.slice()) : Ee > 0 ? ge.push(Se.slice()) : (ge.push(Se.slice()), ve.push(Se.slice())), be = Ee;
                  }
                  return { positive: ge, negative: ve };
                }, ce.exports.positive = function(pe, ye) {
                  for (var ge = [], ve = he(pe[pe.length - 1], ye), be = pe[pe.length - 1], Ce = pe[0], Se = 0; Se < pe.length; ++Se, be = Ce) {
                    var we = he(Ce = pe[Se], ye);
                    (ve < 0 && we > 0 || ve > 0 && we < 0) && ge.push(me(be, we, Ce, ve)), we >= 0 && ge.push(Ce.slice()), ve = we;
                  }
                  return ge;
                }, ce.exports.negative = function(pe, ye) {
                  for (var ge = [], ve = he(pe[pe.length - 1], ye), be = pe[pe.length - 1], Ce = pe[0], Se = 0; Se < pe.length; ++Se, be = Ce) {
                    var we = he(Ce = pe[Se], ye);
                    (ve < 0 && we > 0 || ve > 0 && we < 0) && ge.push(me(be, we, Ce, ve)), we <= 0 && ge.push(Ce.slice()), ve = we;
                  }
                  return ge;
                };
              }, 3387: function(ce, le, ue) {
                var de;
                (function() {
                  var fe = { not_string: /[^s]/, not_bool: /[^t]/, not_type: /[^T]/, not_primitive: /[^v]/, number: /[diefg]/, numeric_arg: /[bcdiefguxX]/, json: /[j]/, not_json: /[^j]/, text: /^[^\x25]+/, modulo: /^\x25{2}/, placeholder: /^\x25(?:([1-9]\d*)\$|\(([^)]+)\))?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-gijostTuvxX])/, key: /^([a-z_][a-z_\d]*)/i, key_access: /^\.([a-z_][a-z_\d]*)/i, index_access: /^\[(\d+)\]/, sign: /^[+-]/ };
                  function he(ye) {
                    return function(ge, ve) {
                      var be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me = 1, Re = ge.length, De = "";
                      for (Ce = 0; Ce < Re; Ce++) if (typeof ge[Ce] == "string") De += ge[Ce];
                      else if (typeof ge[Ce] == "object") {
                        if ((we = ge[Ce]).keys) for (be = ve[Me], Se = 0; Se < we.keys.length; Se++) {
                          if (be == null) throw new Error(he('[sprintf] Cannot access property "%s" of undefined value "%s"', we.keys[Se], we.keys[Se - 1]));
                          be = be[we.keys[Se]];
                        }
                        else be = we.param_no ? ve[we.param_no] : ve[Me++];
                        if (fe.not_type.test(we.type) && fe.not_primitive.test(we.type) && be instanceof Function && (be = be()), fe.numeric_arg.test(we.type) && typeof be != "number" && isNaN(be)) throw new TypeError(he("[sprintf] expecting number but found %T", be));
                        switch (fe.number.test(we.type) && (Qe = be >= 0), we.type) {
                          case "b":
                            be = parseInt(be, 10).toString(2);
                            break;
                          case "c":
                            be = String.fromCharCode(parseInt(be, 10));
                            break;
                          case "d":
                          case "i":
                            be = parseInt(be, 10);
                            break;
                          case "j":
                            be = JSON.stringify(be, null, we.width ? parseInt(we.width) : 0);
                            break;
                          case "e":
                            be = we.precision ? parseFloat(be).toExponential(we.precision) : parseFloat(be).toExponential();
                            break;
                          case "f":
                            be = we.precision ? parseFloat(be).toFixed(we.precision) : parseFloat(be);
                            break;
                          case "g":
                            be = we.precision ? String(Number(be.toPrecision(we.precision))) : parseFloat(be);
                            break;
                          case "o":
                            be = (parseInt(be, 10) >>> 0).toString(8);
                            break;
                          case "s":
                            be = String(be), be = we.precision ? be.substring(0, we.precision) : be;
                            break;
                          case "t":
                            be = String(!!be), be = we.precision ? be.substring(0, we.precision) : be;
                            break;
                          case "T":
                            be = Object.prototype.toString.call(be).slice(8, -1).toLowerCase(), be = we.precision ? be.substring(0, we.precision) : be;
                            break;
                          case "u":
                            be = parseInt(be, 10) >>> 0;
                            break;
                          case "v":
                            be = be.valueOf(), be = we.precision ? be.substring(0, we.precision) : be;
                            break;
                          case "x":
                            be = (parseInt(be, 10) >>> 0).toString(16);
                            break;
                          case "X":
                            be = (parseInt(be, 10) >>> 0).toString(16).toUpperCase();
                        }
                        fe.json.test(we.type) ? De += be : (!fe.number.test(we.type) || Qe && !we.sign ? ke = "" : (ke = Qe ? "+" : "-", be = be.toString().replace(fe.sign, "")), xe = we.pad_char ? we.pad_char === "0" ? "0" : we.pad_char.charAt(1) : " ", Le = we.width - (ke + be).length, Ee = we.width && Le > 0 ? xe.repeat(Le) : "", De += we.align ? ke + be + Ee : xe === "0" ? ke + Ee + be : Ee + ke + be);
                      }
                      return De;
                    }(function(ge) {
                      if (pe[ge]) return pe[ge];
                      for (var ve, be = ge, Ce = [], Se = 0; be; ) {
                        if ((ve = fe.text.exec(be)) !== null) Ce.push(ve[0]);
                        else if ((ve = fe.modulo.exec(be)) !== null) Ce.push("%");
                        else {
                          if ((ve = fe.placeholder.exec(be)) === null) throw new SyntaxError("[sprintf] unexpected placeholder");
                          if (ve[2]) {
                            Se |= 1;
                            var we = [], Ee = ve[2], xe = [];
                            if ((xe = fe.key.exec(Ee)) === null) throw new SyntaxError("[sprintf] failed to parse named argument key");
                            for (we.push(xe[1]); (Ee = Ee.substring(xe[0].length)) !== ""; ) if ((xe = fe.key_access.exec(Ee)) !== null) we.push(xe[1]);
                            else {
                              if ((xe = fe.index_access.exec(Ee)) === null) throw new SyntaxError("[sprintf] failed to parse named argument key");
                              we.push(xe[1]);
                            }
                            ve[2] = we;
                          } else Se |= 2;
                          if (Se === 3) throw new Error("[sprintf] mixing positional and named placeholders is not (yet) supported");
                          Ce.push({ placeholder: ve[0], param_no: ve[1], keys: ve[2], sign: ve[3], pad_char: ve[4], align: ve[5], width: ve[6], precision: ve[7], type: ve[8] });
                        }
                        be = be.substring(ve[0].length);
                      }
                      return pe[ge] = Ce;
                    }(ye), arguments);
                  }
                  function me(ye, ge) {
                    return he.apply(null, [ye].concat(ge || []));
                  }
                  var pe = /* @__PURE__ */ Object.create(null);
                  le.sprintf = he, le.vsprintf = me, typeof window < "u" && (window.sprintf = he, window.vsprintf = me, (de = (function() {
                    return { sprintf: he, vsprintf: me };
                  }).call(le, ue, le, ce)) === void 0 || (ce.exports = de));
                })();
              }, 3711: function(ce, le, ue) {
                ce.exports = function(pe, ye) {
                  if (pe.dimension <= 0) return { positions: [], cells: [] };
                  if (pe.dimension === 1) return function(be, Ce) {
                    for (var Se = fe(be, Ce), we = Se.length, Ee = new Array(we), xe = new Array(we), Le = 0; Le < we; ++Le) Ee[Le] = [Se[Le]], xe[Le] = [Le];
                    return { positions: Ee, cells: xe };
                  }(pe, ye);
                  var ge = pe.order.join() + "-" + pe.dtype, ve = me[ge];
                  return ye = +ye || 0, ve || (ve = me[ge] = function(be, Ce) {
                    var Se = be.length + "d", we = he[Se];
                    if (we) return we(de, be, Ce);
                  }(pe.order, pe.dtype)), ve(pe, ye);
                };
                var de = ue(2640), fe = ue(781), he = { "2d": function(pe, ye, ge) {
                  var ve = pe({ order: ye, scalarArguments: 3, getters: ge === "generic" ? [0] : void 0, phase: function(be, Ce, Se, we) {
                    return be > we | 0;
                  }, vertex: function(be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De, Pe) {
                    var $e = (Le << 0) + (Qe << 1) + (ke << 2) + (Me << 3) | 0;
                    if ($e !== 0 && $e !== 15) switch ($e) {
                      case 0:
                      case 15:
                        Re.push([be - 0.5, Ce - 0.5]);
                        break;
                      case 1:
                        Re.push([be - 0.25 - 0.25 * (we + Se - 2 * Pe) / (Se - we), Ce - 0.25 - 0.25 * (Ee + Se - 2 * Pe) / (Se - Ee)]);
                        break;
                      case 2:
                        Re.push([be - 0.75 - 0.25 * (-we - Se + 2 * Pe) / (we - Se), Ce - 0.25 - 0.25 * (xe + we - 2 * Pe) / (we - xe)]);
                        break;
                      case 3:
                        Re.push([be - 0.5, Ce - 0.5 - 0.5 * (Ee + Se + xe + we - 4 * Pe) / (Se - Ee + we - xe)]);
                        break;
                      case 4:
                        Re.push([be - 0.25 - 0.25 * (xe + Ee - 2 * Pe) / (Ee - xe), Ce - 0.75 - 0.25 * (-Ee - Se + 2 * Pe) / (Ee - Se)]);
                        break;
                      case 5:
                        Re.push([be - 0.5 - 0.5 * (we + Se + xe + Ee - 4 * Pe) / (Se - we + Ee - xe), Ce - 0.5]);
                        break;
                      case 6:
                        Re.push([be - 0.5 - 0.25 * (-we - Se + xe + Ee) / (we - Se + Ee - xe), Ce - 0.5 - 0.25 * (-Ee - Se + xe + we) / (Ee - Se + we - xe)]);
                        break;
                      case 7:
                        Re.push([be - 0.75 - 0.25 * (xe + Ee - 2 * Pe) / (Ee - xe), Ce - 0.75 - 0.25 * (xe + we - 2 * Pe) / (we - xe)]);
                        break;
                      case 8:
                        Re.push([be - 0.75 - 0.25 * (-xe - Ee + 2 * Pe) / (xe - Ee), Ce - 0.75 - 0.25 * (-xe - we + 2 * Pe) / (xe - we)]);
                        break;
                      case 9:
                        Re.push([be - 0.5 - 0.25 * (we + Se + -xe - Ee) / (Se - we + xe - Ee), Ce - 0.5 - 0.25 * (Ee + Se + -xe - we) / (Se - Ee + xe - we)]);
                        break;
                      case 10:
                        Re.push([be - 0.5 - 0.5 * (-we - Se - xe - Ee + 4 * Pe) / (we - Se + xe - Ee), Ce - 0.5]);
                        break;
                      case 11:
                        Re.push([be - 0.25 - 0.25 * (-xe - Ee + 2 * Pe) / (xe - Ee), Ce - 0.75 - 0.25 * (Ee + Se - 2 * Pe) / (Se - Ee)]);
                        break;
                      case 12:
                        Re.push([be - 0.5, Ce - 0.5 - 0.5 * (-Ee - Se - xe - we + 4 * Pe) / (Ee - Se + xe - we)]);
                        break;
                      case 13:
                        Re.push([be - 0.75 - 0.25 * (we + Se - 2 * Pe) / (Se - we), Ce - 0.25 - 0.25 * (-xe - we + 2 * Pe) / (xe - we)]);
                        break;
                      case 14:
                        Re.push([be - 0.25 - 0.25 * (-we - Se + 2 * Pe) / (we - Se), Ce - 0.25 - 0.25 * (-Ee - Se + 2 * Pe) / (Ee - Se)]);
                    }
                  }, cell: function(be, Ce, Se, we, Ee, xe, Le, Qe, ke) {
                    Ee ? Qe.push([be, Ce]) : Qe.push([Ce, be]);
                  } });
                  return function(be, Ce) {
                    var Se = [], we = [];
                    return ve(be, Se, we, Ce), { positions: Se, cells: we };
                  };
                } }, me = {};
              }, 529: function(ce, le, ue) {
                ce.exports = function me(pe, ye, ge) {
                  var ve = (ge = ge || {}).fontStyle || "normal", be = ge.fontWeight || "normal", Ce = ge.fontVariant || "normal", Se = [ve, be, Ce, pe].join("_"), we = he[Se];
                  we || (we = he[Se] = { " ": { data: new Float32Array(0), shape: 0.2 } });
                  var Ee = we[ye];
                  if (!Ee) if (ye.length <= 1 || !/\d/.test(ye)) Ee = we[ye] = function(Xe) {
                    for (var Ve = Xe.cells, Ze = Xe.positions, Ge = new Float32Array(6 * Ve.length), ze = 0, We = 0, at = 0; at < Ve.length; ++at) for (var ct = Ve[at], et = 0; et < 3; ++et) {
                      var dt = Ze[ct[et]];
                      Ge[ze++] = dt[0], Ge[ze++] = dt[1] + 1.4, We = Math.max(dt[0], We);
                    }
                    return { data: Ge, shape: We };
                  }(de(ye, { triangles: !0, font: pe, fontStyle: ve, fontWeight: be, fontVariant: Ce, textAlign: ge.textAlign || "left", textBaseline: "alphabetic", styletags: { breaklines: !0, bolds: !0, italics: !0, subscripts: !0, superscripts: !0 } }));
                  else {
                    for (var xe = ye.split(/(\d|\s)/), Le = new Array(xe.length), Qe = 0, ke = 0, Me = 0; Me < xe.length; ++Me) Le[Me] = me(pe, xe[Me]), Qe += Le[Me].data.length, ke += Le[Me].shape, Me > 0 && (ke += 0.02);
                    var Re = new Float32Array(Qe), De = 0, Pe = -0.5 * ke;
                    for (Me = 0; Me < Le.length; ++Me) {
                      for (var $e = Le[Me].data, qe = 0; qe < $e.length; qe += 2) Re[De++] = $e[qe] + Pe, Re[De++] = $e[qe + 1];
                      Pe += Le[Me].shape + 0.02;
                    }
                    Ee = we[ye] = { data: Re, shape: ke };
                  }
                  return Ee;
                };
                var de = ue(4359), fe = window || ne.global || {}, he = fe.__TEXT_CACHE || {};
                fe.__TEXT_CACHE = {};
              }, 665: function(ce, le, ue) {
                var de = ue(3202);
                ce.exports = me;
                var fe = 96;
                function he(pe, ye) {
                  var ge = de(getComputedStyle(pe).getPropertyValue(ye));
                  return ge[0] * me(ge[1], pe);
                }
                function me(pe, ye) {
                  switch (ye = ye || document.body, pe = (pe || "px").trim().toLowerCase(), ye !== window && ye !== document || (ye = document.body), pe) {
                    case "%":
                      return ye.clientHeight / 100;
                    case "ch":
                    case "ex":
                      return function(ge, ve) {
                        var be = document.createElement("div");
                        be.style["font-size"] = "128" + ge, ve.appendChild(be);
                        var Ce = he(be, "font-size") / 128;
                        return ve.removeChild(be), Ce;
                      }(pe, ye);
                    case "em":
                      return he(ye, "font-size");
                    case "rem":
                      return he(document.body, "font-size");
                    case "vw":
                      return window.innerWidth / 100;
                    case "vh":
                      return window.innerHeight / 100;
                    case "vmin":
                      return Math.min(window.innerWidth, window.innerHeight) / 100;
                    case "vmax":
                      return Math.max(window.innerWidth, window.innerHeight) / 100;
                    case "in":
                      return fe;
                    case "cm":
                      return fe / 2.54;
                    case "mm":
                      return fe / 25.4;
                    case "pt":
                      return fe / 72;
                    case "pc":
                      return fe / 6;
                  }
                  return 1;
                }
              }, 7261: function(ce, le, ue) {
                ce.exports = function(Ee) {
                  var xe = (Ee = Ee || {}).center || [0, 0, 0], Le = Ee.up || [0, 1, 0], Qe = Ee.right || be(Le), ke = Ee.radius || 1, Me = Ee.theta || 0, Re = Ee.phi || 0;
                  if (xe = [].slice.call(xe, 0, 3), Le = [].slice.call(Le, 0, 3), pe(Le, Le), Qe = [].slice.call(Qe, 0, 3), pe(Qe, Qe), "eye" in Ee) {
                    var De = Ee.eye, Pe = [De[0] - xe[0], De[1] - xe[1], De[2] - xe[2]];
                    me(Qe, Pe, Le), ge(Qe[0], Qe[1], Qe[2]) < 1e-6 ? Qe = be(Le) : pe(Qe, Qe), ke = ge(Pe[0], Pe[1], Pe[2]);
                    var $e = ye(Le, Pe) / ke, qe = ye(Qe, Pe) / ke;
                    Re = Math.acos($e), Me = Math.acos(qe);
                  }
                  return ke = Math.log(ke), new Ce(Ee.zoomMin, Ee.zoomMax, xe, Le, Qe, ke, Me, Re);
                };
                var de = ue(9215), fe = ue(7608), he = ue(6079), me = ue(5911), pe = ue(3536), ye = ue(244);
                function ge(Ee, xe, Le) {
                  return Math.sqrt(Math.pow(Ee, 2) + Math.pow(xe, 2) + Math.pow(Le, 2));
                }
                function ve(Ee) {
                  return Math.min(1, Math.max(-1, Ee));
                }
                function be(Ee) {
                  var xe = Math.abs(Ee[0]), Le = Math.abs(Ee[1]), Qe = Math.abs(Ee[2]), ke = [0, 0, 0];
                  xe > Math.max(Le, Qe) ? ke[2] = 1 : Le > Math.max(xe, Qe) ? ke[0] = 1 : ke[1] = 1;
                  for (var Me = 0, Re = 0, De = 0; De < 3; ++De) Me += Ee[De] * Ee[De], Re += ke[De] * Ee[De];
                  for (De = 0; De < 3; ++De) ke[De] -= Re / Me * Ee[De];
                  return pe(ke, ke), ke;
                }
                function Ce(Ee, xe, Le, Qe, ke, Me, Re, De) {
                  this.center = de(Le), this.up = de(Qe), this.right = de(ke), this.radius = de([Me]), this.angle = de([Re, De]), this.angle.bounds = [[-1 / 0, -Math.PI / 2], [1 / 0, Math.PI / 2]], this.setDistanceLimits(Ee, xe), this.computedCenter = this.center.curve(0), this.computedUp = this.up.curve(0), this.computedRight = this.right.curve(0), this.computedRadius = this.radius.curve(0), this.computedAngle = this.angle.curve(0), this.computedToward = [0, 0, 0], this.computedEye = [0, 0, 0], this.computedMatrix = new Array(16);
                  for (var Pe = 0; Pe < 16; ++Pe) this.computedMatrix[Pe] = 0.5;
                  this.recalcMatrix(0);
                }
                var Se = Ce.prototype;
                Se.setDistanceLimits = function(Ee, xe) {
                  Ee = Ee > 0 ? Math.log(Ee) : -1 / 0, xe = xe > 0 ? Math.log(xe) : 1 / 0, xe = Math.max(xe, Ee), this.radius.bounds[0][0] = Ee, this.radius.bounds[1][0] = xe;
                }, Se.getDistanceLimits = function(Ee) {
                  var xe = this.radius.bounds[0];
                  return Ee ? (Ee[0] = Math.exp(xe[0][0]), Ee[1] = Math.exp(xe[1][0]), Ee) : [Math.exp(xe[0][0]), Math.exp(xe[1][0])];
                }, Se.recalcMatrix = function(Ee) {
                  this.center.curve(Ee), this.up.curve(Ee), this.right.curve(Ee), this.radius.curve(Ee), this.angle.curve(Ee);
                  for (var xe = this.computedUp, Le = this.computedRight, Qe = 0, ke = 0, Me = 0; Me < 3; ++Me) ke += xe[Me] * Le[Me], Qe += xe[Me] * xe[Me];
                  var Re = Math.sqrt(Qe), De = 0;
                  for (Me = 0; Me < 3; ++Me) Le[Me] -= xe[Me] * ke / Qe, De += Le[Me] * Le[Me], xe[Me] /= Re;
                  var Pe = Math.sqrt(De);
                  for (Me = 0; Me < 3; ++Me) Le[Me] /= Pe;
                  var $e = this.computedToward;
                  me($e, xe, Le), pe($e, $e);
                  var qe = Math.exp(this.computedRadius[0]), Xe = this.computedAngle[0], Ve = this.computedAngle[1], Ze = Math.cos(Xe), Ge = Math.sin(Xe), ze = Math.cos(Ve), We = Math.sin(Ve), at = this.computedCenter, ct = Ze * ze, et = Ge * ze, dt = We, Je = -Ze * We, it = -Ge * We, st = ze, gt = this.computedEye, bt = this.computedMatrix;
                  for (Me = 0; Me < 3; ++Me) {
                    var yt = ct * Le[Me] + et * $e[Me] + dt * xe[Me];
                    bt[4 * Me + 1] = Je * Le[Me] + it * $e[Me] + st * xe[Me], bt[4 * Me + 2] = yt, bt[4 * Me + 3] = 0;
                  }
                  var St = bt[1], Lt = bt[5], Rt = bt[9], vt = bt[2], $t = bt[6], Pt = bt[10], Xt = Lt * Pt - Rt * $t, Nt = Rt * vt - St * Pt, Gt = St * $t - Lt * vt, Ft = ge(Xt, Nt, Gt);
                  for (Xt /= Ft, Nt /= Ft, Gt /= Ft, bt[0] = Xt, bt[4] = Nt, bt[8] = Gt, Me = 0; Me < 3; ++Me) gt[Me] = at[Me] + bt[2 + 4 * Me] * qe;
                  for (Me = 0; Me < 3; ++Me) {
                    De = 0;
                    for (var Tt = 0; Tt < 3; ++Tt) De += bt[Me + 4 * Tt] * gt[Tt];
                    bt[12 + Me] = -De;
                  }
                  bt[15] = 1;
                }, Se.getMatrix = function(Ee, xe) {
                  this.recalcMatrix(Ee);
                  var Le = this.computedMatrix;
                  if (xe) {
                    for (var Qe = 0; Qe < 16; ++Qe) xe[Qe] = Le[Qe];
                    return xe;
                  }
                  return Le;
                };
                var we = [0, 0, 0];
                Se.rotate = function(Ee, xe, Le, Qe) {
                  if (this.angle.move(Ee, xe, Le), Qe) {
                    this.recalcMatrix(Ee);
                    var ke = this.computedMatrix;
                    we[0] = ke[2], we[1] = ke[6], we[2] = ke[10];
                    for (var Me = this.computedUp, Re = this.computedRight, De = this.computedToward, Pe = 0; Pe < 3; ++Pe) ke[4 * Pe] = Me[Pe], ke[4 * Pe + 1] = Re[Pe], ke[4 * Pe + 2] = De[Pe];
                    for (he(ke, ke, Qe, we), Pe = 0; Pe < 3; ++Pe) Me[Pe] = ke[4 * Pe], Re[Pe] = ke[4 * Pe + 1];
                    this.up.set(Ee, Me[0], Me[1], Me[2]), this.right.set(Ee, Re[0], Re[1], Re[2]);
                  }
                }, Se.pan = function(Ee, xe, Le, Qe) {
                  xe = xe || 0, Le = Le || 0, Qe = Qe || 0, this.recalcMatrix(Ee);
                  var ke = this.computedMatrix, Me = (Math.exp(this.computedRadius[0]), ke[1]), Re = ke[5], De = ke[9], Pe = ge(Me, Re, De);
                  Me /= Pe, Re /= Pe, De /= Pe;
                  var $e = ke[0], qe = ke[4], Xe = ke[8], Ve = $e * Me + qe * Re + Xe * De, Ze = ge($e -= Me * Ve, qe -= Re * Ve, Xe -= De * Ve), Ge = ($e /= Ze) * xe + Me * Le, ze = (qe /= Ze) * xe + Re * Le, We = (Xe /= Ze) * xe + De * Le;
                  this.center.move(Ee, Ge, ze, We);
                  var at = Math.exp(this.computedRadius[0]);
                  at = Math.max(1e-4, at + Qe), this.radius.set(Ee, Math.log(at));
                }, Se.translate = function(Ee, xe, Le, Qe) {
                  this.center.move(Ee, xe || 0, Le || 0, Qe || 0);
                }, Se.setMatrix = function(Ee, xe, Le, Qe) {
                  var ke = 1;
                  typeof Le == "number" && (ke = 0 | Le), (ke < 0 || ke > 3) && (ke = 1);
                  var Me = (ke + 2) % 3;
                  xe || (this.recalcMatrix(Ee), xe = this.computedMatrix);
                  var Re = xe[ke], De = xe[ke + 4], Pe = xe[ke + 8];
                  if (Qe) {
                    var $e = Math.abs(Re), qe = Math.abs(De), Xe = Math.abs(Pe), Ve = Math.max($e, qe, Xe);
                    $e === Ve ? (Re = Re < 0 ? -1 : 1, De = Pe = 0) : Xe === Ve ? (Pe = Pe < 0 ? -1 : 1, Re = De = 0) : (De = De < 0 ? -1 : 1, Re = Pe = 0);
                  } else {
                    var Ze = ge(Re, De, Pe);
                    Re /= Ze, De /= Ze, Pe /= Ze;
                  }
                  var Ge, ze, We = xe[Me], at = xe[Me + 4], ct = xe[Me + 8], et = We * Re + at * De + ct * Pe, dt = ge(We -= Re * et, at -= De * et, ct -= Pe * et), Je = De * (ct /= dt) - Pe * (at /= dt), it = Pe * (We /= dt) - Re * ct, st = Re * at - De * We, gt = ge(Je, it, st);
                  if (Je /= gt, it /= gt, st /= gt, this.center.jump(Ee, Vt, Yt, sn), this.radius.idle(Ee), this.up.jump(Ee, Re, De, Pe), this.right.jump(Ee, We, at, ct), ke === 2) {
                    var bt = xe[1], yt = xe[5], St = xe[9], Lt = bt * We + yt * at + St * ct, Rt = bt * Je + yt * it + St * st;
                    Ge = Xt < 0 ? -Math.PI / 2 : Math.PI / 2, ze = Math.atan2(Rt, Lt);
                  } else {
                    var vt = xe[2], $t = xe[6], Pt = xe[10], Xt = vt * Re + $t * De + Pt * Pe, Nt = vt * We + $t * at + Pt * ct, Gt = vt * Je + $t * it + Pt * st;
                    Ge = Math.asin(ve(Xt)), ze = Math.atan2(Gt, Nt);
                  }
                  this.angle.jump(Ee, ze, Ge), this.recalcMatrix(Ee);
                  var Ft = xe[2], Tt = xe[6], It = xe[10], Ct = this.computedMatrix;
                  fe(Ct, xe);
                  var Kt = Ct[15], Vt = Ct[12] / Kt, Yt = Ct[13] / Kt, sn = Ct[14] / Kt, hn = Math.exp(this.computedRadius[0]);
                  this.center.jump(Ee, Vt - Ft * hn, Yt - Tt * hn, sn - It * hn);
                }, Se.lastT = function() {
                  return Math.max(this.center.lastT(), this.up.lastT(), this.right.lastT(), this.radius.lastT(), this.angle.lastT());
                }, Se.idle = function(Ee) {
                  this.center.idle(Ee), this.up.idle(Ee), this.right.idle(Ee), this.radius.idle(Ee), this.angle.idle(Ee);
                }, Se.flush = function(Ee) {
                  this.center.flush(Ee), this.up.flush(Ee), this.right.flush(Ee), this.radius.flush(Ee), this.angle.flush(Ee);
                }, Se.setDistance = function(Ee, xe) {
                  xe > 0 && this.radius.set(Ee, Math.log(xe));
                }, Se.lookAt = function(Ee, xe, Le, Qe) {
                  this.recalcMatrix(Ee), xe = xe || this.computedEye, Le = Le || this.computedCenter;
                  var ke = (Qe = Qe || this.computedUp)[0], Me = Qe[1], Re = Qe[2], De = ge(ke, Me, Re);
                  if (!(De < 1e-6)) {
                    ke /= De, Me /= De, Re /= De;
                    var Pe = xe[0] - Le[0], $e = xe[1] - Le[1], qe = xe[2] - Le[2], Xe = ge(Pe, $e, qe);
                    if (!(Xe < 1e-6)) {
                      Pe /= Xe, $e /= Xe, qe /= Xe;
                      var Ve = this.computedRight, Ze = Ve[0], Ge = Ve[1], ze = Ve[2], We = ke * Ze + Me * Ge + Re * ze, at = ge(Ze -= We * ke, Ge -= We * Me, ze -= We * Re);
                      if (!(at < 0.01 && (at = ge(Ze = Me * qe - Re * $e, Ge = Re * Pe - ke * qe, ze = ke * $e - Me * Pe)) < 1e-6)) {
                        Ze /= at, Ge /= at, ze /= at, this.up.set(Ee, ke, Me, Re), this.right.set(Ee, Ze, Ge, ze), this.center.set(Ee, Le[0], Le[1], Le[2]), this.radius.set(Ee, Math.log(Xe));
                        var ct = Me * ze - Re * Ge, et = Re * Ze - ke * ze, dt = ke * Ge - Me * Ze, Je = ge(ct, et, dt), it = ke * Pe + Me * $e + Re * qe, st = Ze * Pe + Ge * $e + ze * qe, gt = (ct /= Je) * Pe + (et /= Je) * $e + (dt /= Je) * qe, bt = Math.asin(ve(it)), yt = Math.atan2(gt, st), St = this.angle._state, Lt = St[St.length - 1], Rt = St[St.length - 2];
                        Lt %= 2 * Math.PI;
                        var vt = Math.abs(Lt + 2 * Math.PI - yt), $t = Math.abs(Lt - yt), Pt = Math.abs(Lt - 2 * Math.PI - yt);
                        vt < $t && (Lt += 2 * Math.PI), Pt < $t && (Lt -= 2 * Math.PI), this.angle.jump(this.angle.lastT(), Lt, Rt), this.angle.set(Ee, yt, bt);
                      }
                    }
                  }
                };
              }, 5250: function(ce) {
                ce.exports = function(ue, de, fe) {
                  var he = ue * de, me = le * ue, pe = me - (me - ue), ye = ue - pe, ge = le * de, ve = ge - (ge - de), be = de - ve, Ce = ye * be - (he - pe * ve - ye * ve - pe * be);
                  return fe ? (fe[0] = Ce, fe[1] = he, fe) : [Ce, he];
                };
                var le = +(Math.pow(2, 27) + 1);
              }, 9362: function(ce) {
                ce.exports = function(le, ue, de) {
                  var fe = le + ue, he = fe - le, me = ue - he, pe = le - (fe - he);
                  return de ? (de[0] = pe + me, de[1] = fe, de) : [pe + me, fe];
                };
              }, 1888: function(ce, le, ue) {
                var de = ue(8828), fe = ue(1338), he = ue(4793).hp;
                ue.g.__TYPEDARRAY_POOL || (ue.g.__TYPEDARRAY_POOL = { UINT8: fe([32, 0]), UINT16: fe([32, 0]), UINT32: fe([32, 0]), BIGUINT64: fe([32, 0]), INT8: fe([32, 0]), INT16: fe([32, 0]), INT32: fe([32, 0]), BIGINT64: fe([32, 0]), FLOAT: fe([32, 0]), DOUBLE: fe([32, 0]), DATA: fe([32, 0]), UINT8C: fe([32, 0]), BUFFER: fe([32, 0]) });
                var me = typeof Uint8ClampedArray < "u", pe = typeof BigUint64Array < "u", ye = typeof BigInt64Array < "u", ge = ue.g.__TYPEDARRAY_POOL;
                ge.UINT8C || (ge.UINT8C = fe([32, 0])), ge.BIGUINT64 || (ge.BIGUINT64 = fe([32, 0])), ge.BIGINT64 || (ge.BIGINT64 = fe([32, 0])), ge.BUFFER || (ge.BUFFER = fe([32, 0]));
                var ve = ge.DATA, be = ge.BUFFER;
                function Ce(Ve) {
                  if (Ve) {
                    var Ze = Ve.length || Ve.byteLength, Ge = de.log2(Ze);
                    ve[Ge].push(Ve);
                  }
                }
                function Se(Ve) {
                  Ve = de.nextPow2(Ve);
                  var Ze = de.log2(Ve), Ge = ve[Ze];
                  return Ge.length > 0 ? Ge.pop() : new ArrayBuffer(Ve);
                }
                function we(Ve) {
                  return new Uint8Array(Se(Ve), 0, Ve);
                }
                function Ee(Ve) {
                  return new Uint16Array(Se(2 * Ve), 0, Ve);
                }
                function xe(Ve) {
                  return new Uint32Array(Se(4 * Ve), 0, Ve);
                }
                function Le(Ve) {
                  return new Int8Array(Se(Ve), 0, Ve);
                }
                function Qe(Ve) {
                  return new Int16Array(Se(2 * Ve), 0, Ve);
                }
                function ke(Ve) {
                  return new Int32Array(Se(4 * Ve), 0, Ve);
                }
                function Me(Ve) {
                  return new Float32Array(Se(4 * Ve), 0, Ve);
                }
                function Re(Ve) {
                  return new Float64Array(Se(8 * Ve), 0, Ve);
                }
                function De(Ve) {
                  return me ? new Uint8ClampedArray(Se(Ve), 0, Ve) : we(Ve);
                }
                function Pe(Ve) {
                  return pe ? new BigUint64Array(Se(8 * Ve), 0, Ve) : null;
                }
                function $e(Ve) {
                  return ye ? new BigInt64Array(Se(8 * Ve), 0, Ve) : null;
                }
                function qe(Ve) {
                  return new DataView(Se(Ve), 0, Ve);
                }
                function Xe(Ve) {
                  Ve = de.nextPow2(Ve);
                  var Ze = de.log2(Ve), Ge = be[Ze];
                  return Ge.length > 0 ? Ge.pop() : new he(Ve);
                }
                le.free = function(Ve) {
                  if (he.isBuffer(Ve)) be[de.log2(Ve.length)].push(Ve);
                  else {
                    if (Object.prototype.toString.call(Ve) !== "[object ArrayBuffer]" && (Ve = Ve.buffer), !Ve) return;
                    var Ze = Ve.length || Ve.byteLength, Ge = 0 | de.log2(Ze);
                    ve[Ge].push(Ve);
                  }
                }, le.freeUint8 = le.freeUint16 = le.freeUint32 = le.freeBigUint64 = le.freeInt8 = le.freeInt16 = le.freeInt32 = le.freeBigInt64 = le.freeFloat32 = le.freeFloat = le.freeFloat64 = le.freeDouble = le.freeUint8Clamped = le.freeDataView = function(Ve) {
                  Ce(Ve.buffer);
                }, le.freeArrayBuffer = Ce, le.freeBuffer = function(Ve) {
                  be[de.log2(Ve.length)].push(Ve);
                }, le.malloc = function(Ve, Ze) {
                  if (Ze === void 0 || Ze === "arraybuffer") return Se(Ve);
                  switch (Ze) {
                    case "uint8":
                      return we(Ve);
                    case "uint16":
                      return Ee(Ve);
                    case "uint32":
                      return xe(Ve);
                    case "int8":
                      return Le(Ve);
                    case "int16":
                      return Qe(Ve);
                    case "int32":
                      return ke(Ve);
                    case "float":
                    case "float32":
                      return Me(Ve);
                    case "double":
                    case "float64":
                      return Re(Ve);
                    case "uint8_clamped":
                      return De(Ve);
                    case "bigint64":
                      return $e(Ve);
                    case "biguint64":
                      return Pe(Ve);
                    case "buffer":
                      return Xe(Ve);
                    case "data":
                    case "dataview":
                      return qe(Ve);
                    default:
                      return null;
                  }
                  return null;
                }, le.mallocArrayBuffer = Se, le.mallocUint8 = we, le.mallocUint16 = Ee, le.mallocUint32 = xe, le.mallocInt8 = Le, le.mallocInt16 = Qe, le.mallocInt32 = ke, le.mallocFloat32 = le.mallocFloat = Me, le.mallocFloat64 = le.mallocDouble = Re, le.mallocUint8Clamped = De, le.mallocBigUint64 = Pe, le.mallocBigInt64 = $e, le.mallocDataView = qe, le.mallocBuffer = Xe, le.clearCache = function() {
                  for (var Ve = 0; Ve < 32; ++Ve) ge.UINT8[Ve].length = 0, ge.UINT16[Ve].length = 0, ge.UINT32[Ve].length = 0, ge.INT8[Ve].length = 0, ge.INT16[Ve].length = 0, ge.INT32[Ve].length = 0, ge.FLOAT[Ve].length = 0, ge.DOUBLE[Ve].length = 0, ge.BIGUINT64[Ve].length = 0, ge.BIGINT64[Ve].length = 0, ge.UINT8C[Ve].length = 0, ve[Ve].length = 0, be[Ve].length = 0;
                };
              }, 1755: function(ce) {
                function le(de) {
                  this.roots = new Array(de), this.ranks = new Array(de);
                  for (var fe = 0; fe < de; ++fe) this.roots[fe] = fe, this.ranks[fe] = 0;
                }
                ce.exports = le;
                var ue = le.prototype;
                Object.defineProperty(ue, "length", { get: function() {
                  return this.roots.length;
                } }), ue.makeSet = function() {
                  var de = this.roots.length;
                  return this.roots.push(de), this.ranks.push(0), de;
                }, ue.find = function(de) {
                  for (var fe = de, he = this.roots; he[de] !== de; ) de = he[de];
                  for (; he[fe] !== de; ) {
                    var me = he[fe];
                    he[fe] = de, fe = me;
                  }
                  return de;
                }, ue.link = function(de, fe) {
                  var he = this.find(de), me = this.find(fe);
                  if (he !== me) {
                    var pe = this.ranks, ye = this.roots, ge = pe[he], ve = pe[me];
                    ge < ve ? ye[he] = me : ve < ge ? ye[me] = he : (ye[me] = he, ++pe[he]);
                  }
                };
              }, 1682: function(ce) {
                ce.exports = function(le, ue, de) {
                  return le.length === 0 ? le : ue ? (de || le.sort(ue), function(fe, he) {
                    for (var me = 1, pe = fe.length, ye = fe[0], ge = fe[0], ve = 1; ve < pe; ++ve) if (ge = ye, he(ye = fe[ve], ge)) {
                      if (ve === me) {
                        me++;
                        continue;
                      }
                      fe[me++] = ye;
                    }
                    return fe.length = me, fe;
                  }(le, ue)) : (de || le.sort(), function(fe) {
                    for (var he = 1, me = fe.length, pe = fe[0], ye = fe[0], ge = 1; ge < me; ++ge, ye = pe) if (ye = pe, (pe = fe[ge]) !== ye) {
                      if (ge === he) {
                        he++;
                        continue;
                      }
                      fe[he++] = pe;
                    }
                    return fe.length = he, fe;
                  }(le));
                };
              }, 4359: function(ce, le, ue) {
                ce.exports = function(me, pe) {
                  return typeof pe == "object" && pe !== null || (pe = {}), de(me, pe.canvas || fe, pe.context || he, pe);
                };
                var de = ue(7718), fe = null, he = null;
                typeof document < "u" && ((fe = document.createElement("canvas")).width = 8192, fe.height = 1024, he = fe.getContext("2d"));
              }, 7718: function(ce, le, ue) {
                ce.exports = function(Pe, $e, qe, Xe) {
                  var Ve = 64, Ze = 1.25, Ge = { breaklines: !1, bolds: !1, italics: !1, subscripts: !1, superscripts: !1 };
                  return Xe && (Xe.size && Xe.size > 0 && (Ve = Xe.size), Xe.lineSpacing && Xe.lineSpacing > 0 && (Ze = Xe.lineSpacing), Xe.styletags && Xe.styletags.breaklines && (Ge.breaklines = !!Xe.styletags.breaklines), Xe.styletags && Xe.styletags.bolds && (Ge.bolds = !!Xe.styletags.bolds), Xe.styletags && Xe.styletags.italics && (Ge.italics = !!Xe.styletags.italics), Xe.styletags && Xe.styletags.subscripts && (Ge.subscripts = !!Xe.styletags.subscripts), Xe.styletags && Xe.styletags.superscripts && (Ge.superscripts = !!Xe.styletags.superscripts)), qe.font = [Xe.fontStyle, Xe.fontVariant, Xe.fontWeight, Ve + "px", Xe.font].filter(function(ze) {
                    return ze;
                  }).join(" "), qe.textAlign = "start", qe.textBaseline = "alphabetic", qe.direction = "ltr", De(function(ze, We, at, ct, et, dt) {
                    at = at.replace(/\n/g, ""), at = dt.breaklines === !0 ? at.replace(/\<br\>/g, `
`) : at.replace(/\<br\>/g, " ");
                    var Je = "", it = [];
                    for (yt = 0; yt < at.length; ++yt) it[yt] = Je;
                    dt.bolds === !0 && (it = ke(ge, ve, at, it)), dt.italics === !0 && (it = ke(be, Ce, at, it)), dt.superscripts === !0 && (it = ke(Se, Ee, at, it)), dt.subscripts === !0 && (it = ke(xe, Qe, at, it));
                    var st = [], gt = "";
                    for (yt = 0; yt < at.length; ++yt) it[yt] !== null && (gt += at[yt], st.push(it[yt]));
                    var bt, yt, St, Lt, Rt, vt = gt.split(`
`), $t = vt.length, Pt = Math.round(et * ct), Xt = ct, Nt = 2 * ct, Gt = 0, Ft = $t * Pt + Nt;
                    ze.height < Ft && (ze.height = Ft), We.fillStyle = "#000", We.fillRect(0, 0, ze.width, ze.height), We.fillStyle = "#fff";
                    var Tt = 0, It = "";
                    function Ct() {
                      if (It !== "") {
                        var Wt = We.measureText(It).width;
                        We.fillText(It, Xt + St, Nt + Lt), St += Wt;
                      }
                    }
                    function Kt() {
                      return Math.round(Rt) + "px ";
                    }
                    function Vt(Wt, rn) {
                      var vn = "" + We.font;
                      if (dt.subscripts === !0) {
                        var dn = Wt.indexOf(Le), Ht = rn.indexOf(Le), jt = dn > -1 ? parseInt(Wt[1 + dn]) : 0, cn = Ht > -1 ? parseInt(rn[1 + Ht]) : 0;
                        jt !== cn && (vn = vn.replace(Kt(), "?px "), Rt *= Math.pow(0.75, cn - jt), vn = vn.replace("?px ", Kt())), Lt += 0.25 * Pt * (cn - jt);
                      }
                      if (dt.superscripts === !0) {
                        var wn = Wt.indexOf(we), Rn = rn.indexOf(we), Pn = wn > -1 ? parseInt(Wt[1 + wn]) : 0, Ie = Rn > -1 ? parseInt(rn[1 + Rn]) : 0;
                        Pn !== Ie && (vn = vn.replace(Kt(), "?px "), Rt *= Math.pow(0.75, Ie - Pn), vn = vn.replace("?px ", Kt())), Lt -= 0.25 * Pt * (Ie - Pn);
                      }
                      if (dt.bolds === !0) {
                        var rt = Wt.indexOf(ve) > -1, ft = rn.indexOf(ve) > -1;
                        !rt && ft && (vn = pt ? vn.replace("italic ", "italic bold ") : "bold " + vn), rt && !ft && (vn = vn.replace("bold ", ""));
                      }
                      if (dt.italics === !0) {
                        var pt = Wt.indexOf(Ce) > -1, wt = rn.indexOf(Ce) > -1;
                        !pt && wt && (vn = "italic " + vn), pt && !wt && (vn = vn.replace("italic ", ""));
                      }
                      We.font = vn;
                    }
                    for (bt = 0; bt < $t; ++bt) {
                      var Yt = vt[bt] + `
`;
                      for (St = 0, Lt = bt * Pt, Rt = ct, It = "", yt = 0; yt < Yt.length; ++yt) {
                        var sn = yt + Tt < st.length ? st[yt + Tt] : st[st.length - 1];
                        Je === sn ? It += Yt[yt] : (Ct(), It = Yt[yt], sn !== void 0 && (Vt(Je, sn), Je = sn));
                      }
                      Ct(), Tt += Yt.length;
                      var hn = 0 | Math.round(St + 2 * Xt);
                      Gt < hn && (Gt = hn);
                    }
                    var Qt = Gt, Ut = Nt + Pt * $t;
                    return fe(We.getImageData(0, 0, Qt, Ut).data, [Ut, Qt, 4]).pick(-1, -1, 0).transpose(1, 0);
                  }($e, qe, Pe, Ve, Ze, Ge), Xe, Ve);
                }, ce.exports.processPixels = De;
                var de = ue(3711), fe = ue(9618), he = ue(5878), me = ue(332), pe = ue(2538), ye = ue(2095), ge = "b", ve = "b|", be = "i", Ce = "i|", Se = "sup", we = "+", Ee = "+1", xe = "sub", Le = "-", Qe = "-1";
                function ke(Pe, $e, qe, Xe) {
                  for (var Ve = "<" + Pe + ">", Ze = "</" + Pe + ">", Ge = Ve.length, ze = Ze.length, We = $e[0] === we || $e[0] === Le, at = 0, ct = -ze; at > -1 && (at = qe.indexOf(Ve, at)) !== -1 && (ct = qe.indexOf(Ze, at + Ge)) !== -1 && !(ct <= at); ) {
                    for (var et = at; et < ct + ze; ++et) if (et < at + Ge || et >= ct) Xe[et] = null, qe = qe.substr(0, et) + " " + qe.substr(et + 1);
                    else if (Xe[et] !== null) {
                      var dt = Xe[et].indexOf($e[0]);
                      dt === -1 ? Xe[et] += $e : We && (Xe[et] = Xe[et].substr(0, dt + 1) + (1 + parseInt(Xe[et][dt + 1])) + Xe[et].substr(dt + 2));
                    }
                    var Je = at + Ge, it = qe.substr(Je, ct - Je).indexOf(Ve);
                    at = it !== -1 ? it : ct + ze;
                  }
                  return Xe;
                }
                function Me(Pe, $e) {
                  var qe = de(Pe, 128);
                  return $e ? he(qe.cells, qe.positions, 0.25) : { edges: qe.cells, positions: qe.positions };
                }
                function Re(Pe, $e, qe, Xe) {
                  var Ve = Me(Pe, Xe), Ze = function(bt, yt, St) {
                    for (var Lt = yt.textAlign || "start", Rt = yt.textBaseline || "alphabetic", vt = [1 << 30, 1 << 30], $t = [0, 0], Pt = bt.length, Xt = 0; Xt < Pt; ++Xt) for (var Nt = bt[Xt], Gt = 0; Gt < 2; ++Gt) vt[Gt] = 0 | Math.min(vt[Gt], Nt[Gt]), $t[Gt] = 0 | Math.max($t[Gt], Nt[Gt]);
                    var Ft = 0;
                    switch (Lt) {
                      case "center":
                        Ft = -0.5 * (vt[0] + $t[0]);
                        break;
                      case "right":
                      case "end":
                        Ft = -$t[0];
                        break;
                      case "left":
                      case "start":
                        Ft = -vt[0];
                        break;
                      default:
                        throw new Error("vectorize-text: Unrecognized textAlign: '" + Lt + "'");
                    }
                    var Tt = 0;
                    switch (Rt) {
                      case "hanging":
                      case "top":
                        Tt = -vt[1];
                        break;
                      case "middle":
                        Tt = -0.5 * (vt[1] + $t[1]);
                        break;
                      case "alphabetic":
                      case "ideographic":
                        Tt = -3 * St;
                        break;
                      case "bottom":
                        Tt = -$t[1];
                        break;
                      default:
                        throw new Error("vectorize-text: Unrecoginized textBaseline: '" + Rt + "'");
                    }
                    var It = 1 / St;
                    return "lineHeight" in yt ? It *= +yt.lineHeight : "width" in yt ? It = yt.width / ($t[0] - vt[0]) : "height" in yt && (It = yt.height / ($t[1] - vt[1])), bt.map(function(Ct) {
                      return [It * (Ct[0] + Ft), It * (Ct[1] + Tt)];
                    });
                  }(Ve.positions, $e, qe), Ge = Ve.edges, ze = $e.orientation === "ccw";
                  if (me(Ze, Ge), $e.polygons || $e.polygon || $e.polyline) {
                    for (var We = ye(Ge, Ze), at = new Array(We.length), ct = 0; ct < We.length; ++ct) {
                      for (var et = We[ct], dt = new Array(et.length), Je = 0; Je < et.length; ++Je) {
                        for (var it = et[Je], st = new Array(it.length), gt = 0; gt < it.length; ++gt) st[gt] = Ze[it[gt]].slice();
                        ze && st.reverse(), dt[Je] = st;
                      }
                      at[ct] = dt;
                    }
                    return at;
                  }
                  return $e.triangles || $e.triangulate || $e.triangle ? { cells: pe(Ze, Ge, { delaunay: !1, exterior: !1, interior: !0 }), positions: Ze } : { edges: Ge, positions: Ze };
                }
                function De(Pe, $e, qe) {
                  try {
                    return Re(Pe, $e, qe, !0);
                  } catch {
                  }
                  try {
                    return Re(Pe, $e, qe, !1);
                  } catch {
                  }
                  return $e.polygons || $e.polyline || $e.polygon ? [] : $e.triangles || $e.triangulate || $e.triangle ? { cells: [], positions: [] } : { edges: [], positions: [] };
                }
              }, 1538: function(ce) {
                (function() {
                  if (typeof ses > "u" || !ses.ok || ses.ok()) {
                    typeof ses < "u" && (ses.weakMapPermitHostObjects = xe);
                    var le = !1;
                    if (typeof WeakMap == "function") {
                      var ue = WeakMap;
                      if (!(typeof navigator < "u" && /Firefox/.test(navigator.userAgent))) {
                        var de = new ue(), fe = Object.freeze({});
                        if (de.set(fe, 1), de.get(fe) === 1) return void (ce.exports = WeakMap);
                        le = !0;
                      }
                    }
                    var he = Object.getOwnPropertyNames, me = Object.defineProperty, pe = Object.isExtensible, ye = "weakmap:", ge = ye + "ident:" + Math.random() + "___";
                    if (typeof crypto < "u" && typeof crypto.getRandomValues == "function" && typeof ArrayBuffer == "function" && typeof Uint8Array == "function") {
                      var ve = new ArrayBuffer(25), be = new Uint8Array(ve);
                      crypto.getRandomValues(be), ge = ye + "rand:" + Array.prototype.map.call(be, function(Re) {
                        return (Re % 36).toString(36);
                      }).join("") + "___";
                    }
                    if (me(Object, "getOwnPropertyNames", { value: function(Re) {
                      return he(Re).filter(Le);
                    } }), "getPropertyNames" in Object) {
                      var Ce = Object.getPropertyNames;
                      me(Object, "getPropertyNames", { value: function(Re) {
                        return Ce(Re).filter(Le);
                      } });
                    }
                    (function() {
                      var Re = Object.freeze;
                      me(Object, "freeze", { value: function($e) {
                        return Qe($e), Re($e);
                      } });
                      var De = Object.seal;
                      me(Object, "seal", { value: function($e) {
                        return Qe($e), De($e);
                      } });
                      var Pe = Object.preventExtensions;
                      me(Object, "preventExtensions", { value: function($e) {
                        return Qe($e), Pe($e);
                      } });
                    })();
                    var Se = !1, we = 0, Ee = function() {
                      this instanceof Ee || Me();
                      var Re = [], De = [], Pe = we++;
                      return Object.create(Ee.prototype, { get___: { value: ke(function($e, qe) {
                        var Xe, Ve = Qe($e);
                        return Ve ? Pe in Ve ? Ve[Pe] : qe : (Xe = Re.indexOf($e)) >= 0 ? De[Xe] : qe;
                      }) }, has___: { value: ke(function($e) {
                        var qe = Qe($e);
                        return qe ? Pe in qe : Re.indexOf($e) >= 0;
                      }) }, set___: { value: ke(function($e, qe) {
                        var Xe, Ve = Qe($e);
                        return Ve ? Ve[Pe] = qe : (Xe = Re.indexOf($e)) >= 0 ? De[Xe] = qe : (Xe = Re.length, De[Xe] = qe, Re[Xe] = $e), this;
                      }) }, delete___: { value: ke(function($e) {
                        var qe, Xe, Ve = Qe($e);
                        return Ve ? Pe in Ve && delete Ve[Pe] : !((qe = Re.indexOf($e)) < 0 || (Xe = Re.length - 1, Re[qe] = void 0, De[qe] = De[Xe], Re[qe] = Re[Xe], Re.length = Xe, De.length = Xe, 0));
                      }) } });
                    };
                    Ee.prototype = Object.create(Object.prototype, { get: { value: function(Re, De) {
                      return this.get___(Re, De);
                    }, writable: !0, configurable: !0 }, has: { value: function(Re) {
                      return this.has___(Re);
                    }, writable: !0, configurable: !0 }, set: { value: function(Re, De) {
                      return this.set___(Re, De);
                    }, writable: !0, configurable: !0 }, delete: { value: function(Re) {
                      return this.delete___(Re);
                    }, writable: !0, configurable: !0 } }), typeof ue == "function" ? function() {
                      function Re() {
                        this instanceof Ee || Me();
                        var De, Pe = new ue(), $e = void 0, qe = !1;
                        return De = le ? function(Xe, Ve) {
                          return Pe.set(Xe, Ve), Pe.has(Xe) || ($e || ($e = new Ee()), $e.set(Xe, Ve)), this;
                        } : function(Xe, Ve) {
                          if (qe) try {
                            Pe.set(Xe, Ve);
                          } catch {
                            $e || ($e = new Ee()), $e.set___(Xe, Ve);
                          }
                          else Pe.set(Xe, Ve);
                          return this;
                        }, Object.create(Ee.prototype, { get___: { value: ke(function(Xe, Ve) {
                          return $e ? Pe.has(Xe) ? Pe.get(Xe) : $e.get___(Xe, Ve) : Pe.get(Xe, Ve);
                        }) }, has___: { value: ke(function(Xe) {
                          return Pe.has(Xe) || !!$e && $e.has___(Xe);
                        }) }, set___: { value: ke(De) }, delete___: { value: ke(function(Xe) {
                          var Ve = !!Pe.delete(Xe);
                          return $e && $e.delete___(Xe) || Ve;
                        }) }, permitHostObjects___: { value: ke(function(Xe) {
                          if (Xe !== xe) throw new Error("bogus call to permitHostObjects___");
                          qe = !0;
                        }) } });
                      }
                      le && typeof Proxy < "u" && (Proxy = void 0), Re.prototype = Ee.prototype, ce.exports = Re, Object.defineProperty(WeakMap.prototype, "constructor", { value: WeakMap, enumerable: !1, configurable: !0, writable: !0 });
                    }() : (typeof Proxy < "u" && (Proxy = void 0), ce.exports = Ee);
                  }
                  function xe(Re) {
                    Re.permitHostObjects___ && Re.permitHostObjects___(xe);
                  }
                  function Le(Re) {
                    return !(Re.substr(0, 8) == ye && Re.substr(Re.length - 3) === "___");
                  }
                  function Qe(Re) {
                    if (Re !== Object(Re)) throw new TypeError("Not an object: " + Re);
                    var De = Re[ge];
                    if (De && De.key === Re) return De;
                    if (pe(Re)) {
                      De = { key: Re };
                      try {
                        return me(Re, ge, { value: De, writable: !1, enumerable: !1, configurable: !1 }), De;
                      } catch {
                        return;
                      }
                    }
                  }
                  function ke(Re) {
                    return Re.prototype = null, Object.freeze(Re);
                  }
                  function Me() {
                    Se || typeof console > "u" || (Se = !0, console.warn("WeakMap should be invoked as new WeakMap(), not WeakMap(). This will be an error in the future."));
                  }
                })();
              }, 236: function(ce, le, ue) {
                var de = ue(8284);
                ce.exports = function() {
                  var fe = {};
                  return function(he) {
                    if ((typeof he != "object" || he === null) && typeof he != "function") throw new Error("Weakmap-shim: Key must be object");
                    var me = he.valueOf(fe);
                    return me && me.identity === fe ? me : de(he, fe);
                  };
                };
              }, 8284: function(ce) {
                ce.exports = function(le, ue) {
                  var de = { identity: ue }, fe = le.valueOf;
                  return Object.defineProperty(le, "valueOf", { value: function(he) {
                    return he !== ue ? fe.apply(this, arguments) : de;
                  }, writable: !0 }), de;
                };
              }, 606: function(ce, le, ue) {
                var de = ue(236);
                ce.exports = function() {
                  var fe = de();
                  return { get: function(he, me) {
                    var pe = fe(he);
                    return pe.hasOwnProperty("value") ? pe.value : me;
                  }, set: function(he, me) {
                    return fe(he).value = me, this;
                  }, has: function(he) {
                    return "value" in fe(he);
                  }, delete: function(he) {
                    return delete fe(he).value;
                  } };
                };
              }, 3349: function(ce) {
                ce.exports = /* @__PURE__ */ function(le) {
                  var ue = {};
                  return function(de, fe, he) {
                    var me = de.dtype, pe = de.order, ye = [me, pe.join()].join(), ge = ue[ye];
                    return ge || (ue[ye] = ge = le([me, pe])), ge(de.shape.slice(0), de.data, de.stride, 0 | de.offset, fe, he);
                  };
                }((function() {
                  return function(le, ue, de, fe, he, me) {
                    var pe = le[0], ye = de[0], ge = [0], ve = ye;
                    fe |= 0;
                    var be = 0, Ce = ye;
                    for (be = 0; be < pe; ++be) {
                      var Se = ue[fe] - me, we = ue[fe + ve] - me;
                      Se >= 0 != we >= 0 && he.push(ge[0] + 0.5 + 0.5 * (Se + we) / (Se - we)), fe += Ce, ++ge[0];
                    }
                  };
                }).bind(void 0, { funcName: "zeroCrossings" }));
              }, 781: function(ce, le, ue) {
                ce.exports = function(fe, he) {
                  var me = [];
                  return he = +he || 0, de(fe.hi(fe.shape[0] - 1), me, he), me;
                };
                var de = ue(3349);
              }, 7790: function() {
              } }, ae = {};
              function oe(ce) {
                var le = ae[ce];
                if (le !== void 0) return le.exports;
                var ue = ae[ce] = { id: ce, loaded: !1, exports: {} };
                return ee[ce].call(ue.exports, ue, ue.exports, oe), ue.loaded = !0, ue.exports;
              }
              oe.g = function() {
                if (typeof globalThis == "object") return globalThis;
                try {
                  return this || new Function("return this")();
                } catch {
                  if (typeof window == "object") return window;
                }
              }(), oe.nmd = function(ce) {
                return ce.paths = [], ce.children || (ce.children = []), ce;
              };
              var se = oe(1964);
              G.exports = se;
            })();
          }, 45708: function(G, J, Z) {
            function te(Tt, It) {
              for (var Ct = 0; Ct < It.length; Ct++) {
                var Kt = It[Ct];
                Kt.enumerable = Kt.enumerable || !1, Kt.configurable = !0, "value" in Kt && (Kt.writable = !0), Object.defineProperty(Tt, ne(Kt.key), Kt);
              }
            }
            function ne(Tt) {
              var It = function(Ct, Kt) {
                if (ce(Ct) != "object" || !Ct) return Ct;
                var Vt = Ct[Symbol.toPrimitive];
                if (Vt !== void 0) {
                  var Yt = Vt.call(Ct, "string");
                  if (ce(Yt) != "object") return Yt;
                  throw new TypeError("@@toPrimitive must return a primitive value.");
                }
                return String(Ct);
              }(Tt);
              return ce(It) == "symbol" ? It : It + "";
            }
            function ee() {
              try {
                var Tt = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {
                }));
              } catch {
              }
              return (ee = function() {
                return !!Tt;
              })();
            }
            function ae(Tt) {
              return ae = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function(It) {
                return It.__proto__ || Object.getPrototypeOf(It);
              }, ae(Tt);
            }
            function oe(Tt) {
              if (Tt === void 0) throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
              return Tt;
            }
            function se(Tt, It) {
              return se = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(Ct, Kt) {
                return Ct.__proto__ = Kt, Ct;
              }, se(Tt, It);
            }
            function ce(Tt) {
              return ce = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(It) {
                return typeof It;
              } : function(It) {
                return It && typeof Symbol == "function" && It.constructor === Symbol && It !== Symbol.prototype ? "symbol" : typeof It;
              }, ce(Tt);
            }
            var le = Z(76226), ue = Z(27415), de = typeof Symbol == "function" && typeof Symbol.for == "function" ? Symbol.for("nodejs.util.inspect.custom") : null;
            J.Buffer = me, J.SlowBuffer = function(Tt) {
              return +Tt != Tt && (Tt = 0), me.alloc(+Tt);
            }, J.INSPECT_MAX_BYTES = 50;
            var fe = 2147483647;
            function he(Tt) {
              if (Tt > fe) throw new RangeError('The value "' + Tt + '" is invalid for option "size"');
              var It = new Uint8Array(Tt);
              return Object.setPrototypeOf(It, me.prototype), It;
            }
            function me(Tt, It, Ct) {
              if (typeof Tt == "number") {
                if (typeof It == "string") throw new TypeError('The "string" argument must be of type string. Received type number');
                return ge(Tt);
              }
              return pe(Tt, It, Ct);
            }
            function pe(Tt, It, Ct) {
              if (typeof Tt == "string") return function(Yt, sn) {
                if (typeof sn == "string" && sn !== "" || (sn = "utf8"), !me.isEncoding(sn)) throw new TypeError("Unknown encoding: " + sn);
                var hn = 0 | Se(Yt, sn), Qt = he(hn), Ut = Qt.write(Yt, sn);
                return Ut !== hn && (Qt = Qt.slice(0, Ut)), Qt;
              }(Tt, It);
              if (ArrayBuffer.isView(Tt)) return function(Yt) {
                if (Pt(Yt, Uint8Array)) {
                  var sn = new Uint8Array(Yt);
                  return be(sn.buffer, sn.byteOffset, sn.byteLength);
                }
                return ve(Yt);
              }(Tt);
              if (Tt == null) throw new TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type " + ce(Tt));
              if (Pt(Tt, ArrayBuffer) || Tt && Pt(Tt.buffer, ArrayBuffer) || typeof SharedArrayBuffer < "u" && (Pt(Tt, SharedArrayBuffer) || Tt && Pt(Tt.buffer, SharedArrayBuffer))) return be(Tt, It, Ct);
              if (typeof Tt == "number") throw new TypeError('The "value" argument must not be of type number. Received type number');
              var Kt = Tt.valueOf && Tt.valueOf();
              if (Kt != null && Kt !== Tt) return me.from(Kt, It, Ct);
              var Vt = function(Yt) {
                if (me.isBuffer(Yt)) {
                  var sn = 0 | Ce(Yt.length), hn = he(sn);
                  return hn.length === 0 || Yt.copy(hn, 0, 0, sn), hn;
                }
                return Yt.length !== void 0 ? typeof Yt.length != "number" || Xt(Yt.length) ? he(0) : ve(Yt) : Yt.type === "Buffer" && Array.isArray(Yt.data) ? ve(Yt.data) : void 0;
              }(Tt);
              if (Vt) return Vt;
              if (typeof Symbol < "u" && Symbol.toPrimitive != null && typeof Tt[Symbol.toPrimitive] == "function") return me.from(Tt[Symbol.toPrimitive]("string"), It, Ct);
              throw new TypeError("The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type " + ce(Tt));
            }
            function ye(Tt) {
              if (typeof Tt != "number") throw new TypeError('"size" argument must be of type number');
              if (Tt < 0) throw new RangeError('The value "' + Tt + '" is invalid for option "size"');
            }
            function ge(Tt) {
              return ye(Tt), he(Tt < 0 ? 0 : 0 | Ce(Tt));
            }
            function ve(Tt) {
              for (var It = Tt.length < 0 ? 0 : 0 | Ce(Tt.length), Ct = he(It), Kt = 0; Kt < It; Kt += 1) Ct[Kt] = 255 & Tt[Kt];
              return Ct;
            }
            function be(Tt, It, Ct) {
              if (It < 0 || Tt.byteLength < It) throw new RangeError('"offset" is outside of buffer bounds');
              if (Tt.byteLength < It + (Ct || 0)) throw new RangeError('"length" is outside of buffer bounds');
              var Kt;
              return Kt = It === void 0 && Ct === void 0 ? new Uint8Array(Tt) : Ct === void 0 ? new Uint8Array(Tt, It) : new Uint8Array(Tt, It, Ct), Object.setPrototypeOf(Kt, me.prototype), Kt;
            }
            function Ce(Tt) {
              if (Tt >= fe) throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x" + fe.toString(16) + " bytes");
              return 0 | Tt;
            }
            function Se(Tt, It) {
              if (me.isBuffer(Tt)) return Tt.length;
              if (ArrayBuffer.isView(Tt) || Pt(Tt, ArrayBuffer)) return Tt.byteLength;
              if (typeof Tt != "string") throw new TypeError('The "string" argument must be one of type string, Buffer, or ArrayBuffer. Received type ' + ce(Tt));
              var Ct = Tt.length, Kt = arguments.length > 2 && arguments[2] === !0;
              if (!Kt && Ct === 0) return 0;
              for (var Vt = !1; ; ) switch (It) {
                case "ascii":
                case "latin1":
                case "binary":
                  return Ct;
                case "utf8":
                case "utf-8":
                  return Rt(Tt).length;
                case "ucs2":
                case "ucs-2":
                case "utf16le":
                case "utf-16le":
                  return 2 * Ct;
                case "hex":
                  return Ct >>> 1;
                case "base64":
                  return vt(Tt).length;
                default:
                  if (Vt) return Kt ? -1 : Rt(Tt).length;
                  It = ("" + It).toLowerCase(), Vt = !0;
              }
            }
            function we(Tt, It, Ct) {
              var Kt = !1;
              if ((It === void 0 || It < 0) && (It = 0), It > this.length || ((Ct === void 0 || Ct > this.length) && (Ct = this.length), Ct <= 0) || (Ct >>>= 0) <= (It >>>= 0)) return "";
              for (Tt || (Tt = "utf8"); ; ) switch (Tt) {
                case "hex":
                  return Ze(this, It, Ct);
                case "utf8":
                case "utf-8":
                  return $e(this, It, Ct);
                case "ascii":
                  return Xe(this, It, Ct);
                case "latin1":
                case "binary":
                  return Ve(this, It, Ct);
                case "base64":
                  return Pe(this, It, Ct);
                case "ucs2":
                case "ucs-2":
                case "utf16le":
                case "utf-16le":
                  return Ge(this, It, Ct);
                default:
                  if (Kt) throw new TypeError("Unknown encoding: " + Tt);
                  Tt = (Tt + "").toLowerCase(), Kt = !0;
              }
            }
            function Ee(Tt, It, Ct) {
              var Kt = Tt[It];
              Tt[It] = Tt[Ct], Tt[Ct] = Kt;
            }
            function xe(Tt, It, Ct, Kt, Vt) {
              if (Tt.length === 0) return -1;
              if (typeof Ct == "string" ? (Kt = Ct, Ct = 0) : Ct > 2147483647 ? Ct = 2147483647 : Ct < -2147483648 && (Ct = -2147483648), Xt(Ct = +Ct) && (Ct = Vt ? 0 : Tt.length - 1), Ct < 0 && (Ct = Tt.length + Ct), Ct >= Tt.length) {
                if (Vt) return -1;
                Ct = Tt.length - 1;
              } else if (Ct < 0) {
                if (!Vt) return -1;
                Ct = 0;
              }
              if (typeof It == "string" && (It = me.from(It, Kt)), me.isBuffer(It)) return It.length === 0 ? -1 : Le(Tt, It, Ct, Kt, Vt);
              if (typeof It == "number") return It &= 255, typeof Uint8Array.prototype.indexOf == "function" ? Vt ? Uint8Array.prototype.indexOf.call(Tt, It, Ct) : Uint8Array.prototype.lastIndexOf.call(Tt, It, Ct) : Le(Tt, [It], Ct, Kt, Vt);
              throw new TypeError("val must be string, number or Buffer");
            }
            function Le(Tt, It, Ct, Kt, Vt) {
              var Yt, sn = 1, hn = Tt.length, Qt = It.length;
              if (Kt !== void 0 && ((Kt = String(Kt).toLowerCase()) === "ucs2" || Kt === "ucs-2" || Kt === "utf16le" || Kt === "utf-16le")) {
                if (Tt.length < 2 || It.length < 2) return -1;
                sn = 2, hn /= 2, Qt /= 2, Ct /= 2;
              }
              function Ut(dn, Ht) {
                return sn === 1 ? dn[Ht] : dn.readUInt16BE(Ht * sn);
              }
              if (Vt) {
                var Wt = -1;
                for (Yt = Ct; Yt < hn; Yt++) if (Ut(Tt, Yt) === Ut(It, Wt === -1 ? 0 : Yt - Wt)) {
                  if (Wt === -1 && (Wt = Yt), Yt - Wt + 1 === Qt) return Wt * sn;
                } else Wt !== -1 && (Yt -= Yt - Wt), Wt = -1;
              } else for (Ct + Qt > hn && (Ct = hn - Qt), Yt = Ct; Yt >= 0; Yt--) {
                for (var rn = !0, vn = 0; vn < Qt; vn++) if (Ut(Tt, Yt + vn) !== Ut(It, vn)) {
                  rn = !1;
                  break;
                }
                if (rn) return Yt;
              }
              return -1;
            }
            function Qe(Tt, It, Ct, Kt) {
              Ct = Number(Ct) || 0;
              var Vt = Tt.length - Ct;
              Kt ? (Kt = Number(Kt)) > Vt && (Kt = Vt) : Kt = Vt;
              var Yt, sn = It.length;
              for (Kt > sn / 2 && (Kt = sn / 2), Yt = 0; Yt < Kt; ++Yt) {
                var hn = parseInt(It.substr(2 * Yt, 2), 16);
                if (Xt(hn)) return Yt;
                Tt[Ct + Yt] = hn;
              }
              return Yt;
            }
            function ke(Tt, It, Ct, Kt) {
              return $t(Rt(It, Tt.length - Ct), Tt, Ct, Kt);
            }
            function Me(Tt, It, Ct, Kt) {
              return $t(function(Vt) {
                for (var Yt = [], sn = 0; sn < Vt.length; ++sn) Yt.push(255 & Vt.charCodeAt(sn));
                return Yt;
              }(It), Tt, Ct, Kt);
            }
            function Re(Tt, It, Ct, Kt) {
              return $t(vt(It), Tt, Ct, Kt);
            }
            function De(Tt, It, Ct, Kt) {
              return $t(function(Vt, Yt) {
                for (var sn, hn, Qt, Ut = [], Wt = 0; Wt < Vt.length && !((Yt -= 2) < 0); ++Wt) hn = (sn = Vt.charCodeAt(Wt)) >> 8, Qt = sn % 256, Ut.push(Qt), Ut.push(hn);
                return Ut;
              }(It, Tt.length - Ct), Tt, Ct, Kt);
            }
            function Pe(Tt, It, Ct) {
              return It === 0 && Ct === Tt.length ? le.fromByteArray(Tt) : le.fromByteArray(Tt.slice(It, Ct));
            }
            function $e(Tt, It, Ct) {
              Ct = Math.min(Tt.length, Ct);
              for (var Kt = [], Vt = It; Vt < Ct; ) {
                var Yt = Tt[Vt], sn = null, hn = Yt > 239 ? 4 : Yt > 223 ? 3 : Yt > 191 ? 2 : 1;
                if (Vt + hn <= Ct) {
                  var Qt = void 0, Ut = void 0, Wt = void 0, rn = void 0;
                  switch (hn) {
                    case 1:
                      Yt < 128 && (sn = Yt);
                      break;
                    case 2:
                      (192 & (Qt = Tt[Vt + 1])) == 128 && (rn = (31 & Yt) << 6 | 63 & Qt) > 127 && (sn = rn);
                      break;
                    case 3:
                      Qt = Tt[Vt + 1], Ut = Tt[Vt + 2], (192 & Qt) == 128 && (192 & Ut) == 128 && (rn = (15 & Yt) << 12 | (63 & Qt) << 6 | 63 & Ut) > 2047 && (rn < 55296 || rn > 57343) && (sn = rn);
                      break;
                    case 4:
                      Qt = Tt[Vt + 1], Ut = Tt[Vt + 2], Wt = Tt[Vt + 3], (192 & Qt) == 128 && (192 & Ut) == 128 && (192 & Wt) == 128 && (rn = (15 & Yt) << 18 | (63 & Qt) << 12 | (63 & Ut) << 6 | 63 & Wt) > 65535 && rn < 1114112 && (sn = rn);
                  }
                }
                sn === null ? (sn = 65533, hn = 1) : sn > 65535 && (sn -= 65536, Kt.push(sn >>> 10 & 1023 | 55296), sn = 56320 | 1023 & sn), Kt.push(sn), Vt += hn;
              }
              return function(vn) {
                var dn = vn.length;
                if (dn <= qe) return String.fromCharCode.apply(String, vn);
                for (var Ht = "", jt = 0; jt < dn; ) Ht += String.fromCharCode.apply(String, vn.slice(jt, jt += qe));
                return Ht;
              }(Kt);
            }
            J.kMaxLength = fe, me.TYPED_ARRAY_SUPPORT = function() {
              try {
                var Tt = new Uint8Array(1), It = { foo: function() {
                  return 42;
                } };
                return Object.setPrototypeOf(It, Uint8Array.prototype), Object.setPrototypeOf(Tt, It), Tt.foo() === 42;
              } catch {
                return !1;
              }
            }(), me.TYPED_ARRAY_SUPPORT || typeof console > "u" || typeof console.error != "function" || console.error("This browser lacks typed array (Uint8Array) support which is required by `buffer` v5.x. Use `buffer` v4.x if you require old browser support."), Object.defineProperty(me.prototype, "parent", { enumerable: !0, get: function() {
              if (me.isBuffer(this)) return this.buffer;
            } }), Object.defineProperty(me.prototype, "offset", { enumerable: !0, get: function() {
              if (me.isBuffer(this)) return this.byteOffset;
            } }), me.poolSize = 8192, me.from = function(Tt, It, Ct) {
              return pe(Tt, It, Ct);
            }, Object.setPrototypeOf(me.prototype, Uint8Array.prototype), Object.setPrototypeOf(me, Uint8Array), me.alloc = function(Tt, It, Ct) {
              return function(Kt, Vt, Yt) {
                return ye(Kt), Kt <= 0 ? he(Kt) : Vt !== void 0 ? typeof Yt == "string" ? he(Kt).fill(Vt, Yt) : he(Kt).fill(Vt) : he(Kt);
              }(Tt, It, Ct);
            }, me.allocUnsafe = function(Tt) {
              return ge(Tt);
            }, me.allocUnsafeSlow = function(Tt) {
              return ge(Tt);
            }, me.isBuffer = function(Tt) {
              return Tt != null && Tt._isBuffer === !0 && Tt !== me.prototype;
            }, me.compare = function(Tt, It) {
              if (Pt(Tt, Uint8Array) && (Tt = me.from(Tt, Tt.offset, Tt.byteLength)), Pt(It, Uint8Array) && (It = me.from(It, It.offset, It.byteLength)), !me.isBuffer(Tt) || !me.isBuffer(It)) throw new TypeError('The "buf1", "buf2" arguments must be one of type Buffer or Uint8Array');
              if (Tt === It) return 0;
              for (var Ct = Tt.length, Kt = It.length, Vt = 0, Yt = Math.min(Ct, Kt); Vt < Yt; ++Vt) if (Tt[Vt] !== It[Vt]) {
                Ct = Tt[Vt], Kt = It[Vt];
                break;
              }
              return Ct < Kt ? -1 : Kt < Ct ? 1 : 0;
            }, me.isEncoding = function(Tt) {
              switch (String(Tt).toLowerCase()) {
                case "hex":
                case "utf8":
                case "utf-8":
                case "ascii":
                case "latin1":
                case "binary":
                case "base64":
                case "ucs2":
                case "ucs-2":
                case "utf16le":
                case "utf-16le":
                  return !0;
                default:
                  return !1;
              }
            }, me.concat = function(Tt, It) {
              if (!Array.isArray(Tt)) throw new TypeError('"list" argument must be an Array of Buffers');
              if (Tt.length === 0) return me.alloc(0);
              var Ct;
              if (It === void 0) for (It = 0, Ct = 0; Ct < Tt.length; ++Ct) It += Tt[Ct].length;
              var Kt = me.allocUnsafe(It), Vt = 0;
              for (Ct = 0; Ct < Tt.length; ++Ct) {
                var Yt = Tt[Ct];
                if (Pt(Yt, Uint8Array)) Vt + Yt.length > Kt.length ? (me.isBuffer(Yt) || (Yt = me.from(Yt)), Yt.copy(Kt, Vt)) : Uint8Array.prototype.set.call(Kt, Yt, Vt);
                else {
                  if (!me.isBuffer(Yt)) throw new TypeError('"list" argument must be an Array of Buffers');
                  Yt.copy(Kt, Vt);
                }
                Vt += Yt.length;
              }
              return Kt;
            }, me.byteLength = Se, me.prototype._isBuffer = !0, me.prototype.swap16 = function() {
              var Tt = this.length;
              if (Tt % 2 != 0) throw new RangeError("Buffer size must be a multiple of 16-bits");
              for (var It = 0; It < Tt; It += 2) Ee(this, It, It + 1);
              return this;
            }, me.prototype.swap32 = function() {
              var Tt = this.length;
              if (Tt % 4 != 0) throw new RangeError("Buffer size must be a multiple of 32-bits");
              for (var It = 0; It < Tt; It += 4) Ee(this, It, It + 3), Ee(this, It + 1, It + 2);
              return this;
            }, me.prototype.swap64 = function() {
              var Tt = this.length;
              if (Tt % 8 != 0) throw new RangeError("Buffer size must be a multiple of 64-bits");
              for (var It = 0; It < Tt; It += 8) Ee(this, It, It + 7), Ee(this, It + 1, It + 6), Ee(this, It + 2, It + 5), Ee(this, It + 3, It + 4);
              return this;
            }, me.prototype.toString = function() {
              var Tt = this.length;
              return Tt === 0 ? "" : arguments.length === 0 ? $e(this, 0, Tt) : we.apply(this, arguments);
            }, me.prototype.toLocaleString = me.prototype.toString, me.prototype.equals = function(Tt) {
              if (!me.isBuffer(Tt)) throw new TypeError("Argument must be a Buffer");
              return this === Tt || me.compare(this, Tt) === 0;
            }, me.prototype.inspect = function() {
              var Tt = "", It = J.INSPECT_MAX_BYTES;
              return Tt = this.toString("hex", 0, It).replace(/(.{2})/g, "$1 ").trim(), this.length > It && (Tt += " ... "), "<Buffer " + Tt + ">";
            }, de && (me.prototype[de] = me.prototype.inspect), me.prototype.compare = function(Tt, It, Ct, Kt, Vt) {
              if (Pt(Tt, Uint8Array) && (Tt = me.from(Tt, Tt.offset, Tt.byteLength)), !me.isBuffer(Tt)) throw new TypeError('The "target" argument must be one of type Buffer or Uint8Array. Received type ' + ce(Tt));
              if (It === void 0 && (It = 0), Ct === void 0 && (Ct = Tt ? Tt.length : 0), Kt === void 0 && (Kt = 0), Vt === void 0 && (Vt = this.length), It < 0 || Ct > Tt.length || Kt < 0 || Vt > this.length) throw new RangeError("out of range index");
              if (Kt >= Vt && It >= Ct) return 0;
              if (Kt >= Vt) return -1;
              if (It >= Ct) return 1;
              if (this === Tt) return 0;
              for (var Yt = (Vt >>>= 0) - (Kt >>>= 0), sn = (Ct >>>= 0) - (It >>>= 0), hn = Math.min(Yt, sn), Qt = this.slice(Kt, Vt), Ut = Tt.slice(It, Ct), Wt = 0; Wt < hn; ++Wt) if (Qt[Wt] !== Ut[Wt]) {
                Yt = Qt[Wt], sn = Ut[Wt];
                break;
              }
              return Yt < sn ? -1 : sn < Yt ? 1 : 0;
            }, me.prototype.includes = function(Tt, It, Ct) {
              return this.indexOf(Tt, It, Ct) !== -1;
            }, me.prototype.indexOf = function(Tt, It, Ct) {
              return xe(this, Tt, It, Ct, !0);
            }, me.prototype.lastIndexOf = function(Tt, It, Ct) {
              return xe(this, Tt, It, Ct, !1);
            }, me.prototype.write = function(Tt, It, Ct, Kt) {
              if (It === void 0) Kt = "utf8", Ct = this.length, It = 0;
              else if (Ct === void 0 && typeof It == "string") Kt = It, Ct = this.length, It = 0;
              else {
                if (!isFinite(It)) throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");
                It >>>= 0, isFinite(Ct) ? (Ct >>>= 0, Kt === void 0 && (Kt = "utf8")) : (Kt = Ct, Ct = void 0);
              }
              var Vt = this.length - It;
              if ((Ct === void 0 || Ct > Vt) && (Ct = Vt), Tt.length > 0 && (Ct < 0 || It < 0) || It > this.length) throw new RangeError("Attempt to write outside buffer bounds");
              Kt || (Kt = "utf8");
              for (var Yt = !1; ; ) switch (Kt) {
                case "hex":
                  return Qe(this, Tt, It, Ct);
                case "utf8":
                case "utf-8":
                  return ke(this, Tt, It, Ct);
                case "ascii":
                case "latin1":
                case "binary":
                  return Me(this, Tt, It, Ct);
                case "base64":
                  return Re(this, Tt, It, Ct);
                case "ucs2":
                case "ucs-2":
                case "utf16le":
                case "utf-16le":
                  return De(this, Tt, It, Ct);
                default:
                  if (Yt) throw new TypeError("Unknown encoding: " + Kt);
                  Kt = ("" + Kt).toLowerCase(), Yt = !0;
              }
            }, me.prototype.toJSON = function() {
              return { type: "Buffer", data: Array.prototype.slice.call(this._arr || this, 0) };
            };
            var qe = 4096;
            function Xe(Tt, It, Ct) {
              var Kt = "";
              Ct = Math.min(Tt.length, Ct);
              for (var Vt = It; Vt < Ct; ++Vt) Kt += String.fromCharCode(127 & Tt[Vt]);
              return Kt;
            }
            function Ve(Tt, It, Ct) {
              var Kt = "";
              Ct = Math.min(Tt.length, Ct);
              for (var Vt = It; Vt < Ct; ++Vt) Kt += String.fromCharCode(Tt[Vt]);
              return Kt;
            }
            function Ze(Tt, It, Ct) {
              var Kt = Tt.length;
              (!It || It < 0) && (It = 0), (!Ct || Ct < 0 || Ct > Kt) && (Ct = Kt);
              for (var Vt = "", Yt = It; Yt < Ct; ++Yt) Vt += Nt[Tt[Yt]];
              return Vt;
            }
            function Ge(Tt, It, Ct) {
              for (var Kt = Tt.slice(It, Ct), Vt = "", Yt = 0; Yt < Kt.length - 1; Yt += 2) Vt += String.fromCharCode(Kt[Yt] + 256 * Kt[Yt + 1]);
              return Vt;
            }
            function ze(Tt, It, Ct) {
              if (Tt % 1 != 0 || Tt < 0) throw new RangeError("offset is not uint");
              if (Tt + It > Ct) throw new RangeError("Trying to access beyond buffer length");
            }
            function We(Tt, It, Ct, Kt, Vt, Yt) {
              if (!me.isBuffer(Tt)) throw new TypeError('"buffer" argument must be a Buffer instance');
              if (It > Vt || It < Yt) throw new RangeError('"value" argument is out of bounds');
              if (Ct + Kt > Tt.length) throw new RangeError("Index out of range");
            }
            function at(Tt, It, Ct, Kt, Vt) {
              bt(It, Kt, Vt, Tt, Ct, 7);
              var Yt = Number(It & BigInt(4294967295));
              Tt[Ct++] = Yt, Yt >>= 8, Tt[Ct++] = Yt, Yt >>= 8, Tt[Ct++] = Yt, Yt >>= 8, Tt[Ct++] = Yt;
              var sn = Number(It >> BigInt(32) & BigInt(4294967295));
              return Tt[Ct++] = sn, sn >>= 8, Tt[Ct++] = sn, sn >>= 8, Tt[Ct++] = sn, sn >>= 8, Tt[Ct++] = sn, Ct;
            }
            function ct(Tt, It, Ct, Kt, Vt) {
              bt(It, Kt, Vt, Tt, Ct, 7);
              var Yt = Number(It & BigInt(4294967295));
              Tt[Ct + 7] = Yt, Yt >>= 8, Tt[Ct + 6] = Yt, Yt >>= 8, Tt[Ct + 5] = Yt, Yt >>= 8, Tt[Ct + 4] = Yt;
              var sn = Number(It >> BigInt(32) & BigInt(4294967295));
              return Tt[Ct + 3] = sn, sn >>= 8, Tt[Ct + 2] = sn, sn >>= 8, Tt[Ct + 1] = sn, sn >>= 8, Tt[Ct] = sn, Ct + 8;
            }
            function et(Tt, It, Ct, Kt, Vt, Yt) {
              if (Ct + Kt > Tt.length) throw new RangeError("Index out of range");
              if (Ct < 0) throw new RangeError("Index out of range");
            }
            function dt(Tt, It, Ct, Kt, Vt) {
              return It = +It, Ct >>>= 0, Vt || et(Tt, 0, Ct, 4), ue.write(Tt, It, Ct, Kt, 23, 4), Ct + 4;
            }
            function Je(Tt, It, Ct, Kt, Vt) {
              return It = +It, Ct >>>= 0, Vt || et(Tt, 0, Ct, 8), ue.write(Tt, It, Ct, Kt, 52, 8), Ct + 8;
            }
            me.prototype.slice = function(Tt, It) {
              var Ct = this.length;
              (Tt = ~~Tt) < 0 ? (Tt += Ct) < 0 && (Tt = 0) : Tt > Ct && (Tt = Ct), (It = It === void 0 ? Ct : ~~It) < 0 ? (It += Ct) < 0 && (It = 0) : It > Ct && (It = Ct), It < Tt && (It = Tt);
              var Kt = this.subarray(Tt, It);
              return Object.setPrototypeOf(Kt, me.prototype), Kt;
            }, me.prototype.readUintLE = me.prototype.readUIntLE = function(Tt, It, Ct) {
              Tt >>>= 0, It >>>= 0, Ct || ze(Tt, It, this.length);
              for (var Kt = this[Tt], Vt = 1, Yt = 0; ++Yt < It && (Vt *= 256); ) Kt += this[Tt + Yt] * Vt;
              return Kt;
            }, me.prototype.readUintBE = me.prototype.readUIntBE = function(Tt, It, Ct) {
              Tt >>>= 0, It >>>= 0, Ct || ze(Tt, It, this.length);
              for (var Kt = this[Tt + --It], Vt = 1; It > 0 && (Vt *= 256); ) Kt += this[Tt + --It] * Vt;
              return Kt;
            }, me.prototype.readUint8 = me.prototype.readUInt8 = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 1, this.length), this[Tt];
            }, me.prototype.readUint16LE = me.prototype.readUInt16LE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 2, this.length), this[Tt] | this[Tt + 1] << 8;
            }, me.prototype.readUint16BE = me.prototype.readUInt16BE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 2, this.length), this[Tt] << 8 | this[Tt + 1];
            }, me.prototype.readUint32LE = me.prototype.readUInt32LE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 4, this.length), (this[Tt] | this[Tt + 1] << 8 | this[Tt + 2] << 16) + 16777216 * this[Tt + 3];
            }, me.prototype.readUint32BE = me.prototype.readUInt32BE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 4, this.length), 16777216 * this[Tt] + (this[Tt + 1] << 16 | this[Tt + 2] << 8 | this[Tt + 3]);
            }, me.prototype.readBigUInt64LE = Gt(function(Tt) {
              yt(Tt >>>= 0, "offset");
              var It = this[Tt], Ct = this[Tt + 7];
              It !== void 0 && Ct !== void 0 || St(Tt, this.length - 8);
              var Kt = It + this[++Tt] * Math.pow(2, 8) + this[++Tt] * Math.pow(2, 16) + this[++Tt] * Math.pow(2, 24), Vt = this[++Tt] + this[++Tt] * Math.pow(2, 8) + this[++Tt] * Math.pow(2, 16) + Ct * Math.pow(2, 24);
              return BigInt(Kt) + (BigInt(Vt) << BigInt(32));
            }), me.prototype.readBigUInt64BE = Gt(function(Tt) {
              yt(Tt >>>= 0, "offset");
              var It = this[Tt], Ct = this[Tt + 7];
              It !== void 0 && Ct !== void 0 || St(Tt, this.length - 8);
              var Kt = It * Math.pow(2, 24) + this[++Tt] * Math.pow(2, 16) + this[++Tt] * Math.pow(2, 8) + this[++Tt], Vt = this[++Tt] * Math.pow(2, 24) + this[++Tt] * Math.pow(2, 16) + this[++Tt] * Math.pow(2, 8) + Ct;
              return (BigInt(Kt) << BigInt(32)) + BigInt(Vt);
            }), me.prototype.readIntLE = function(Tt, It, Ct) {
              Tt >>>= 0, It >>>= 0, Ct || ze(Tt, It, this.length);
              for (var Kt = this[Tt], Vt = 1, Yt = 0; ++Yt < It && (Vt *= 256); ) Kt += this[Tt + Yt] * Vt;
              return Kt >= (Vt *= 128) && (Kt -= Math.pow(2, 8 * It)), Kt;
            }, me.prototype.readIntBE = function(Tt, It, Ct) {
              Tt >>>= 0, It >>>= 0, Ct || ze(Tt, It, this.length);
              for (var Kt = It, Vt = 1, Yt = this[Tt + --Kt]; Kt > 0 && (Vt *= 256); ) Yt += this[Tt + --Kt] * Vt;
              return Yt >= (Vt *= 128) && (Yt -= Math.pow(2, 8 * It)), Yt;
            }, me.prototype.readInt8 = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 1, this.length), 128 & this[Tt] ? -1 * (255 - this[Tt] + 1) : this[Tt];
            }, me.prototype.readInt16LE = function(Tt, It) {
              Tt >>>= 0, It || ze(Tt, 2, this.length);
              var Ct = this[Tt] | this[Tt + 1] << 8;
              return 32768 & Ct ? 4294901760 | Ct : Ct;
            }, me.prototype.readInt16BE = function(Tt, It) {
              Tt >>>= 0, It || ze(Tt, 2, this.length);
              var Ct = this[Tt + 1] | this[Tt] << 8;
              return 32768 & Ct ? 4294901760 | Ct : Ct;
            }, me.prototype.readInt32LE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 4, this.length), this[Tt] | this[Tt + 1] << 8 | this[Tt + 2] << 16 | this[Tt + 3] << 24;
            }, me.prototype.readInt32BE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 4, this.length), this[Tt] << 24 | this[Tt + 1] << 16 | this[Tt + 2] << 8 | this[Tt + 3];
            }, me.prototype.readBigInt64LE = Gt(function(Tt) {
              yt(Tt >>>= 0, "offset");
              var It = this[Tt], Ct = this[Tt + 7];
              It !== void 0 && Ct !== void 0 || St(Tt, this.length - 8);
              var Kt = this[Tt + 4] + this[Tt + 5] * Math.pow(2, 8) + this[Tt + 6] * Math.pow(2, 16) + (Ct << 24);
              return (BigInt(Kt) << BigInt(32)) + BigInt(It + this[++Tt] * Math.pow(2, 8) + this[++Tt] * Math.pow(2, 16) + this[++Tt] * Math.pow(2, 24));
            }), me.prototype.readBigInt64BE = Gt(function(Tt) {
              yt(Tt >>>= 0, "offset");
              var It = this[Tt], Ct = this[Tt + 7];
              It !== void 0 && Ct !== void 0 || St(Tt, this.length - 8);
              var Kt = (It << 24) + this[++Tt] * Math.pow(2, 16) + this[++Tt] * Math.pow(2, 8) + this[++Tt];
              return (BigInt(Kt) << BigInt(32)) + BigInt(this[++Tt] * Math.pow(2, 24) + this[++Tt] * Math.pow(2, 16) + this[++Tt] * Math.pow(2, 8) + Ct);
            }), me.prototype.readFloatLE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 4, this.length), ue.read(this, Tt, !0, 23, 4);
            }, me.prototype.readFloatBE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 4, this.length), ue.read(this, Tt, !1, 23, 4);
            }, me.prototype.readDoubleLE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 8, this.length), ue.read(this, Tt, !0, 52, 8);
            }, me.prototype.readDoubleBE = function(Tt, It) {
              return Tt >>>= 0, It || ze(Tt, 8, this.length), ue.read(this, Tt, !1, 52, 8);
            }, me.prototype.writeUintLE = me.prototype.writeUIntLE = function(Tt, It, Ct, Kt) {
              Tt = +Tt, It >>>= 0, Ct >>>= 0, Kt || We(this, Tt, It, Ct, Math.pow(2, 8 * Ct) - 1, 0);
              var Vt = 1, Yt = 0;
              for (this[It] = 255 & Tt; ++Yt < Ct && (Vt *= 256); ) this[It + Yt] = Tt / Vt & 255;
              return It + Ct;
            }, me.prototype.writeUintBE = me.prototype.writeUIntBE = function(Tt, It, Ct, Kt) {
              Tt = +Tt, It >>>= 0, Ct >>>= 0, Kt || We(this, Tt, It, Ct, Math.pow(2, 8 * Ct) - 1, 0);
              var Vt = Ct - 1, Yt = 1;
              for (this[It + Vt] = 255 & Tt; --Vt >= 0 && (Yt *= 256); ) this[It + Vt] = Tt / Yt & 255;
              return It + Ct;
            }, me.prototype.writeUint8 = me.prototype.writeUInt8 = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 1, 255, 0), this[It] = 255 & Tt, It + 1;
            }, me.prototype.writeUint16LE = me.prototype.writeUInt16LE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 2, 65535, 0), this[It] = 255 & Tt, this[It + 1] = Tt >>> 8, It + 2;
            }, me.prototype.writeUint16BE = me.prototype.writeUInt16BE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 2, 65535, 0), this[It] = Tt >>> 8, this[It + 1] = 255 & Tt, It + 2;
            }, me.prototype.writeUint32LE = me.prototype.writeUInt32LE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 4, 4294967295, 0), this[It + 3] = Tt >>> 24, this[It + 2] = Tt >>> 16, this[It + 1] = Tt >>> 8, this[It] = 255 & Tt, It + 4;
            }, me.prototype.writeUint32BE = me.prototype.writeUInt32BE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 4, 4294967295, 0), this[It] = Tt >>> 24, this[It + 1] = Tt >>> 16, this[It + 2] = Tt >>> 8, this[It + 3] = 255 & Tt, It + 4;
            }, me.prototype.writeBigUInt64LE = Gt(function(Tt) {
              return at(this, Tt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, BigInt(0), BigInt("0xffffffffffffffff"));
            }), me.prototype.writeBigUInt64BE = Gt(function(Tt) {
              return ct(this, Tt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, BigInt(0), BigInt("0xffffffffffffffff"));
            }), me.prototype.writeIntLE = function(Tt, It, Ct, Kt) {
              if (Tt = +Tt, It >>>= 0, !Kt) {
                var Vt = Math.pow(2, 8 * Ct - 1);
                We(this, Tt, It, Ct, Vt - 1, -Vt);
              }
              var Yt = 0, sn = 1, hn = 0;
              for (this[It] = 255 & Tt; ++Yt < Ct && (sn *= 256); ) Tt < 0 && hn === 0 && this[It + Yt - 1] !== 0 && (hn = 1), this[It + Yt] = (Tt / sn >> 0) - hn & 255;
              return It + Ct;
            }, me.prototype.writeIntBE = function(Tt, It, Ct, Kt) {
              if (Tt = +Tt, It >>>= 0, !Kt) {
                var Vt = Math.pow(2, 8 * Ct - 1);
                We(this, Tt, It, Ct, Vt - 1, -Vt);
              }
              var Yt = Ct - 1, sn = 1, hn = 0;
              for (this[It + Yt] = 255 & Tt; --Yt >= 0 && (sn *= 256); ) Tt < 0 && hn === 0 && this[It + Yt + 1] !== 0 && (hn = 1), this[It + Yt] = (Tt / sn >> 0) - hn & 255;
              return It + Ct;
            }, me.prototype.writeInt8 = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 1, 127, -128), Tt < 0 && (Tt = 255 + Tt + 1), this[It] = 255 & Tt, It + 1;
            }, me.prototype.writeInt16LE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 2, 32767, -32768), this[It] = 255 & Tt, this[It + 1] = Tt >>> 8, It + 2;
            }, me.prototype.writeInt16BE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 2, 32767, -32768), this[It] = Tt >>> 8, this[It + 1] = 255 & Tt, It + 2;
            }, me.prototype.writeInt32LE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 4, 2147483647, -2147483648), this[It] = 255 & Tt, this[It + 1] = Tt >>> 8, this[It + 2] = Tt >>> 16, this[It + 3] = Tt >>> 24, It + 4;
            }, me.prototype.writeInt32BE = function(Tt, It, Ct) {
              return Tt = +Tt, It >>>= 0, Ct || We(this, Tt, It, 4, 2147483647, -2147483648), Tt < 0 && (Tt = 4294967295 + Tt + 1), this[It] = Tt >>> 24, this[It + 1] = Tt >>> 16, this[It + 2] = Tt >>> 8, this[It + 3] = 255 & Tt, It + 4;
            }, me.prototype.writeBigInt64LE = Gt(function(Tt) {
              return at(this, Tt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, -BigInt("0x8000000000000000"), BigInt("0x7fffffffffffffff"));
            }), me.prototype.writeBigInt64BE = Gt(function(Tt) {
              return ct(this, Tt, arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : 0, -BigInt("0x8000000000000000"), BigInt("0x7fffffffffffffff"));
            }), me.prototype.writeFloatLE = function(Tt, It, Ct) {
              return dt(this, Tt, It, !0, Ct);
            }, me.prototype.writeFloatBE = function(Tt, It, Ct) {
              return dt(this, Tt, It, !1, Ct);
            }, me.prototype.writeDoubleLE = function(Tt, It, Ct) {
              return Je(this, Tt, It, !0, Ct);
            }, me.prototype.writeDoubleBE = function(Tt, It, Ct) {
              return Je(this, Tt, It, !1, Ct);
            }, me.prototype.copy = function(Tt, It, Ct, Kt) {
              if (!me.isBuffer(Tt)) throw new TypeError("argument should be a Buffer");
              if (Ct || (Ct = 0), Kt || Kt === 0 || (Kt = this.length), It >= Tt.length && (It = Tt.length), It || (It = 0), Kt > 0 && Kt < Ct && (Kt = Ct), Kt === Ct || Tt.length === 0 || this.length === 0) return 0;
              if (It < 0) throw new RangeError("targetStart out of bounds");
              if (Ct < 0 || Ct >= this.length) throw new RangeError("Index out of range");
              if (Kt < 0) throw new RangeError("sourceEnd out of bounds");
              Kt > this.length && (Kt = this.length), Tt.length - It < Kt - Ct && (Kt = Tt.length - It + Ct);
              var Vt = Kt - Ct;
              return this === Tt && typeof Uint8Array.prototype.copyWithin == "function" ? this.copyWithin(It, Ct, Kt) : Uint8Array.prototype.set.call(Tt, this.subarray(Ct, Kt), It), Vt;
            }, me.prototype.fill = function(Tt, It, Ct, Kt) {
              if (typeof Tt == "string") {
                if (typeof It == "string" ? (Kt = It, It = 0, Ct = this.length) : typeof Ct == "string" && (Kt = Ct, Ct = this.length), Kt !== void 0 && typeof Kt != "string") throw new TypeError("encoding must be a string");
                if (typeof Kt == "string" && !me.isEncoding(Kt)) throw new TypeError("Unknown encoding: " + Kt);
                if (Tt.length === 1) {
                  var Vt = Tt.charCodeAt(0);
                  (Kt === "utf8" && Vt < 128 || Kt === "latin1") && (Tt = Vt);
                }
              } else typeof Tt == "number" ? Tt &= 255 : typeof Tt == "boolean" && (Tt = Number(Tt));
              if (It < 0 || this.length < It || this.length < Ct) throw new RangeError("Out of range index");
              if (Ct <= It) return this;
              var Yt;
              if (It >>>= 0, Ct = Ct === void 0 ? this.length : Ct >>> 0, Tt || (Tt = 0), typeof Tt == "number") for (Yt = It; Yt < Ct; ++Yt) this[Yt] = Tt;
              else {
                var sn = me.isBuffer(Tt) ? Tt : me.from(Tt, Kt), hn = sn.length;
                if (hn === 0) throw new TypeError('The value "' + Tt + '" is invalid for argument "value"');
                for (Yt = 0; Yt < Ct - It; ++Yt) this[Yt + It] = sn[Yt % hn];
              }
              return this;
            };
            var it = {};
            function st(Tt, It, Ct) {
              it[Tt] = function(Kt) {
                function Vt() {
                  var hn;
                  return function(Qt, Ut) {
                    if (!(Qt instanceof Ut)) throw new TypeError("Cannot call a class as a function");
                  }(this, Vt), hn = function(Qt, Ut, Wt) {
                    return Ut = ae(Ut), function(rn, vn) {
                      if (vn && (ce(vn) == "object" || typeof vn == "function")) return vn;
                      if (vn !== void 0) throw new TypeError("Derived constructors may only return object or undefined");
                      return oe(rn);
                    }(Qt, ee() ? Reflect.construct(Ut, [], ae(Qt).constructor) : Ut.apply(Qt, Wt));
                  }(this, Vt), Object.defineProperty(oe(hn), "message", { value: It.apply(oe(hn), arguments), writable: !0, configurable: !0 }), hn.name = "".concat(hn.name, " [").concat(Tt, "]"), hn.stack, delete hn.name, hn;
                }
                var Yt, sn;
                return function(hn, Qt) {
                  if (typeof Qt != "function" && Qt !== null) throw new TypeError("Super expression must either be null or a function");
                  hn.prototype = Object.create(Qt && Qt.prototype, { constructor: { value: hn, writable: !0, configurable: !0 } }), Object.defineProperty(hn, "prototype", { writable: !1 }), Qt && se(hn, Qt);
                }(Vt, Kt), Yt = Vt, (sn = [{ key: "code", get: function() {
                  return Tt;
                }, set: function(hn) {
                  Object.defineProperty(this, "code", { configurable: !0, enumerable: !0, value: hn, writable: !0 });
                } }, { key: "toString", value: function() {
                  return "".concat(this.name, " [").concat(Tt, "]: ").concat(this.message);
                } }]) && te(Yt.prototype, sn), Object.defineProperty(Yt, "prototype", { writable: !1 }), Vt;
              }(Ct);
            }
            function gt(Tt) {
              for (var It = "", Ct = Tt.length, Kt = Tt[0] === "-" ? 1 : 0; Ct >= Kt + 4; Ct -= 3) It = "_".concat(Tt.slice(Ct - 3, Ct)).concat(It);
              return "".concat(Tt.slice(0, Ct)).concat(It);
            }
            function bt(Tt, It, Ct, Kt, Vt, Yt) {
              if (Tt > Ct || Tt < It) {
                var sn, hn = typeof It == "bigint" ? "n" : "";
                throw sn = It === 0 || It === BigInt(0) ? ">= 0".concat(hn, " and < 2").concat(hn, " ** ").concat(8 * (Yt + 1)).concat(hn) : ">= -(2".concat(hn, " ** ").concat(8 * (Yt + 1) - 1).concat(hn, ") and < 2 ** ") + "".concat(8 * (Yt + 1) - 1).concat(hn), new it.ERR_OUT_OF_RANGE("value", sn, Tt);
              }
              (function(Qt, Ut, Wt) {
                yt(Ut, "offset"), Qt[Ut] !== void 0 && Qt[Ut + Wt] !== void 0 || St(Ut, Qt.length - (Wt + 1));
              })(Kt, Vt, Yt);
            }
            function yt(Tt, It) {
              if (typeof Tt != "number") throw new it.ERR_INVALID_ARG_TYPE(It, "number", Tt);
            }
            function St(Tt, It, Ct) {
              throw Math.floor(Tt) !== Tt ? (yt(Tt, Ct), new it.ERR_OUT_OF_RANGE("offset", "an integer", Tt)) : It < 0 ? new it.ERR_BUFFER_OUT_OF_BOUNDS() : new it.ERR_OUT_OF_RANGE("offset", ">= ".concat(0, " and <= ").concat(It), Tt);
            }
            st("ERR_BUFFER_OUT_OF_BOUNDS", function(Tt) {
              return Tt ? "".concat(Tt, " is outside of buffer bounds") : "Attempt to access memory outside buffer bounds";
            }, RangeError), st("ERR_INVALID_ARG_TYPE", function(Tt, It) {
              return 'The "'.concat(Tt, '" argument must be of type number. Received type ').concat(ce(It));
            }, TypeError), st("ERR_OUT_OF_RANGE", function(Tt, It, Ct) {
              var Kt = 'The value of "'.concat(Tt, '" is out of range.'), Vt = Ct;
              return Number.isInteger(Ct) && Math.abs(Ct) > Math.pow(2, 32) ? Vt = gt(String(Ct)) : typeof Ct == "bigint" && (Vt = String(Ct), (Ct > Math.pow(BigInt(2), BigInt(32)) || Ct < -Math.pow(BigInt(2), BigInt(32))) && (Vt = gt(Vt)), Vt += "n"), Kt + " It must be ".concat(It, ". Received ").concat(Vt);
            }, RangeError);
            var Lt = /[^+/0-9A-Za-z-_]/g;
            function Rt(Tt, It) {
              var Ct;
              It = It || 1 / 0;
              for (var Kt = Tt.length, Vt = null, Yt = [], sn = 0; sn < Kt; ++sn) {
                if ((Ct = Tt.charCodeAt(sn)) > 55295 && Ct < 57344) {
                  if (!Vt) {
                    if (Ct > 56319) {
                      (It -= 3) > -1 && Yt.push(239, 191, 189);
                      continue;
                    }
                    if (sn + 1 === Kt) {
                      (It -= 3) > -1 && Yt.push(239, 191, 189);
                      continue;
                    }
                    Vt = Ct;
                    continue;
                  }
                  if (Ct < 56320) {
                    (It -= 3) > -1 && Yt.push(239, 191, 189), Vt = Ct;
                    continue;
                  }
                  Ct = 65536 + (Vt - 55296 << 10 | Ct - 56320);
                } else Vt && (It -= 3) > -1 && Yt.push(239, 191, 189);
                if (Vt = null, Ct < 128) {
                  if ((It -= 1) < 0) break;
                  Yt.push(Ct);
                } else if (Ct < 2048) {
                  if ((It -= 2) < 0) break;
                  Yt.push(Ct >> 6 | 192, 63 & Ct | 128);
                } else if (Ct < 65536) {
                  if ((It -= 3) < 0) break;
                  Yt.push(Ct >> 12 | 224, Ct >> 6 & 63 | 128, 63 & Ct | 128);
                } else {
                  if (!(Ct < 1114112)) throw new Error("Invalid code point");
                  if ((It -= 4) < 0) break;
                  Yt.push(Ct >> 18 | 240, Ct >> 12 & 63 | 128, Ct >> 6 & 63 | 128, 63 & Ct | 128);
                }
              }
              return Yt;
            }
            function vt(Tt) {
              return le.toByteArray(function(It) {
                if ((It = (It = It.split("=")[0]).trim().replace(Lt, "")).length < 2) return "";
                for (; It.length % 4 != 0; ) It += "=";
                return It;
              }(Tt));
            }
            function $t(Tt, It, Ct, Kt) {
              var Vt;
              for (Vt = 0; Vt < Kt && !(Vt + Ct >= It.length || Vt >= Tt.length); ++Vt) It[Vt + Ct] = Tt[Vt];
              return Vt;
            }
            function Pt(Tt, It) {
              return Tt instanceof It || Tt != null && Tt.constructor != null && Tt.constructor.name != null && Tt.constructor.name === It.name;
            }
            function Xt(Tt) {
              return Tt != Tt;
            }
            var Nt = function() {
              for (var Tt = "0123456789abcdef", It = new Array(256), Ct = 0; Ct < 16; ++Ct) for (var Kt = 16 * Ct, Vt = 0; Vt < 16; ++Vt) It[Kt + Vt] = Tt[Ct] + Tt[Vt];
              return It;
            }();
            function Gt(Tt) {
              return typeof BigInt > "u" ? Ft : Tt;
            }
            function Ft() {
              throw new Error("BigInt not supported");
            }
          }, 13087: function(G) {
            G.exports = ne, G.exports.isMobile = ne, G.exports.default = ne;
            var J = /(android|bb\d+|meego).+mobile|armv7l|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series[46]0|samsungbrowser.*mobile|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i, Z = /CrOS/, te = /android|ipad|playbook|silk/i;
            function ne(ee) {
              ee || (ee = {});
              var ae = ee.ua;
              if (ae || typeof navigator > "u" || (ae = navigator.userAgent), ae && ae.headers && typeof ae.headers["user-agent"] == "string" && (ae = ae.headers["user-agent"]), typeof ae != "string") return !1;
              var oe = J.test(ae) && !Z.test(ae) || !!ee.tablet && te.test(ae);
              return !oe && ee.tablet && ee.featureDetect && navigator && navigator.maxTouchPoints > 1 && ae.indexOf("Macintosh") !== -1 && ae.indexOf("Safari") !== -1 && (oe = !0), oe;
            }
          }, 5955: function(G, J, Z) {
            var te = Z(22413), ne = Z.n(te), ee = Z(51070), ae = Z.n(ee), oe = Z(62133), se = Z.n(oe), ce = new URL(Z(77035), Z.b), le = new URL(Z(43470), Z.b), ue = new URL(Z(68164), Z.b), de = new URL(Z(64665), Z.b), fe = new URL(Z(4890), Z.b), he = new URL(Z(13363), Z.b), me = new URL(Z(13490), Z.b), pe = new URL(Z(47603), Z.b), ye = new URL(Z(13913), Z.b), ge = new URL(Z(91413), Z.b), ve = new URL(Z(64643), Z.b), be = new URL(Z(80216), Z.b), Ce = new URL(Z(61907), Z.b), Se = new URL(Z(68605), Z.b), we = new URL(Z(25446), Z.b), Ee = new URL(Z(56694), Z.b), xe = new URL(Z(24420), Z.b), Le = new URL(Z(75796), Z.b), Qe = new URL(Z(92228), Z.b), ke = new URL(Z(9819), Z.b), Me = new URL(Z(47695), Z.b), Re = new URL(Z(28869), Z.b), De = new URL(Z(30557), Z.b), Pe = new URL(Z(48460), Z.b), $e = new URL(Z(56539), Z.b), qe = new URL(Z(43737), Z.b), Xe = new URL(Z(47914), Z.b), Ve = new URL(Z(26117), Z.b), Ze = new URL(Z(66311), Z.b), Ge = ae()(ne()), ze = se()(ce), We = se()(le), at = se()(ue), ct = se()(de), et = se()(fe), dt = se()(he), Je = se()(me), it = se()(pe), st = se()(ye), gt = se()(ge), bt = se()(ve), yt = se()(be), St = se()(Ce), Lt = se()(Se), Rt = se()(we), vt = se()(Ee), $t = se()(xe), Pt = se()(Le), Xt = se()(Qe), Nt = se()(ke), Gt = se()(Me), Ft = se()(Re), Tt = se()(De), It = se()(Pe), Ct = se()($e), Kt = se()(qe), Vt = se()(Xe), Yt = se()(Ve), sn = se()(Ze);
            Ge.push([G.id, ".maplibregl-map{font:12px/20px Helvetica Neue,Arial,Helvetica,sans-serif;overflow:hidden;position:relative;-webkit-tap-highlight-color:rgb(0 0 0/0)}.maplibregl-canvas{left:0;position:absolute;top:0}.maplibregl-map:fullscreen{height:100%;width:100%}.maplibregl-ctrl-group button.maplibregl-ctrl-compass{touch-action:none}.maplibregl-canvas-container.maplibregl-interactive,.maplibregl-ctrl-group button.maplibregl-ctrl-compass{cursor:grab;-webkit-user-select:none;-moz-user-select:none;user-select:none}.maplibregl-canvas-container.maplibregl-interactive.maplibregl-track-pointer{cursor:pointer}.maplibregl-canvas-container.maplibregl-interactive:active,.maplibregl-ctrl-group button.maplibregl-ctrl-compass:active{cursor:grabbing}.maplibregl-canvas-container.maplibregl-touch-zoom-rotate,.maplibregl-canvas-container.maplibregl-touch-zoom-rotate .maplibregl-canvas{touch-action:pan-x pan-y}.maplibregl-canvas-container.maplibregl-touch-drag-pan,.maplibregl-canvas-container.maplibregl-touch-drag-pan .maplibregl-canvas{touch-action:pinch-zoom}.maplibregl-canvas-container.maplibregl-touch-zoom-rotate.maplibregl-touch-drag-pan,.maplibregl-canvas-container.maplibregl-touch-zoom-rotate.maplibregl-touch-drag-pan .maplibregl-canvas{touch-action:none}.maplibregl-canvas-container.maplibregl-touch-drag-pan.maplibregl-cooperative-gestures,.maplibregl-canvas-container.maplibregl-touch-drag-pan.maplibregl-cooperative-gestures .maplibregl-canvas{touch-action:pan-x pan-y}.maplibregl-ctrl-bottom-left,.maplibregl-ctrl-bottom-right,.maplibregl-ctrl-top-left,.maplibregl-ctrl-top-right{pointer-events:none;position:absolute;z-index:2}.maplibregl-ctrl-top-left{left:0;top:0}.maplibregl-ctrl-top-right{right:0;top:0}.maplibregl-ctrl-bottom-left{bottom:0;left:0}.maplibregl-ctrl-bottom-right{bottom:0;right:0}.maplibregl-ctrl{clear:both;pointer-events:auto;transform:translate(0)}.maplibregl-ctrl-top-left .maplibregl-ctrl{float:left;margin:10px 0 0 10px}.maplibregl-ctrl-top-right .maplibregl-ctrl{float:right;margin:10px 10px 0 0}.maplibregl-ctrl-bottom-left .maplibregl-ctrl{float:left;margin:0 0 10px 10px}.maplibregl-ctrl-bottom-right .maplibregl-ctrl{float:right;margin:0 10px 10px 0}.maplibregl-ctrl-group{background:#fff;border-radius:4px}.maplibregl-ctrl-group:not(:empty){box-shadow:0 0 0 2px rgba(0,0,0,.1)}@media (forced-colors:active){.maplibregl-ctrl-group:not(:empty){box-shadow:0 0 0 2px ButtonText}}.maplibregl-ctrl-group button{background-color:transparent;border:0;box-sizing:border-box;cursor:pointer;display:block;height:29px;outline:none;padding:0;width:29px}.maplibregl-ctrl-group button+button{border-top:1px solid #ddd}.maplibregl-ctrl button .maplibregl-ctrl-icon{background-position:50%;background-repeat:no-repeat;display:block;height:100%;width:100%}@media (forced-colors:active){.maplibregl-ctrl-icon{background-color:transparent}.maplibregl-ctrl-group button+button{border-top:1px solid ButtonText}}.maplibregl-ctrl button::-moz-focus-inner{border:0;padding:0}.maplibregl-ctrl-attrib-button:focus,.maplibregl-ctrl-group button:focus{box-shadow:0 0 2px 2px #0096ff}.maplibregl-ctrl button:disabled{cursor:not-allowed}.maplibregl-ctrl button:disabled .maplibregl-ctrl-icon{opacity:.25}.maplibregl-ctrl button:not(:disabled):hover{background-color:rgb(0 0 0/5%)}.maplibregl-ctrl-group button:focus:focus-visible{box-shadow:0 0 2px 2px #0096ff}.maplibregl-ctrl-group button:focus:not(:focus-visible){box-shadow:none}.maplibregl-ctrl-group button:focus:first-child{border-radius:4px 4px 0 0}.maplibregl-ctrl-group button:focus:last-child{border-radius:0 0 4px 4px}.maplibregl-ctrl-group button:focus:only-child{border-radius:inherit}.maplibregl-ctrl button.maplibregl-ctrl-zoom-out .maplibregl-ctrl-icon{background-image:url(" + ze + ")}.maplibregl-ctrl button.maplibregl-ctrl-zoom-in .maplibregl-ctrl-icon{background-image:url(" + We + ")}@media (forced-colors:active){.maplibregl-ctrl button.maplibregl-ctrl-zoom-out .maplibregl-ctrl-icon{background-image:url(" + at + ")}.maplibregl-ctrl button.maplibregl-ctrl-zoom-in .maplibregl-ctrl-icon{background-image:url(" + ct + ")}}@media (forced-colors:active) and (prefers-color-scheme:light){.maplibregl-ctrl button.maplibregl-ctrl-zoom-out .maplibregl-ctrl-icon{background-image:url(" + et + ")}.maplibregl-ctrl button.maplibregl-ctrl-zoom-in .maplibregl-ctrl-icon{background-image:url(" + dt + ")}}.maplibregl-ctrl button.maplibregl-ctrl-fullscreen .maplibregl-ctrl-icon{background-image:url(" + Je + ")}.maplibregl-ctrl button.maplibregl-ctrl-shrink .maplibregl-ctrl-icon{background-image:url(" + it + ")}@media (forced-colors:active){.maplibregl-ctrl button.maplibregl-ctrl-fullscreen .maplibregl-ctrl-icon{background-image:url(" + st + ")}.maplibregl-ctrl button.maplibregl-ctrl-shrink .maplibregl-ctrl-icon{background-image:url(" + gt + ")}}@media (forced-colors:active) and (prefers-color-scheme:light){.maplibregl-ctrl button.maplibregl-ctrl-fullscreen .maplibregl-ctrl-icon{background-image:url(" + bt + ")}.maplibregl-ctrl button.maplibregl-ctrl-shrink .maplibregl-ctrl-icon{background-image:url(" + it + ")}}.maplibregl-ctrl button.maplibregl-ctrl-compass .maplibregl-ctrl-icon{background-image:url(" + yt + ")}@media (forced-colors:active){.maplibregl-ctrl button.maplibregl-ctrl-compass .maplibregl-ctrl-icon{background-image:url(" + St + ")}}@media (forced-colors:active) and (prefers-color-scheme:light){.maplibregl-ctrl button.maplibregl-ctrl-compass .maplibregl-ctrl-icon{background-image:url(" + Lt + ")}}.maplibregl-ctrl button.maplibregl-ctrl-terrain .maplibregl-ctrl-icon{background-image:url(" + Rt + ")}.maplibregl-ctrl button.maplibregl-ctrl-terrain-enabled .maplibregl-ctrl-icon{background-image:url(" + vt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate .maplibregl-ctrl-icon{background-image:url(" + $t + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate:disabled .maplibregl-ctrl-icon{background-image:url(" + Pt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-active .maplibregl-ctrl-icon{background-image:url(" + Xt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-active-error .maplibregl-ctrl-icon{background-image:url(" + Nt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-background .maplibregl-ctrl-icon{background-image:url(" + Gt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-background-error .maplibregl-ctrl-icon{background-image:url(" + Ft + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-waiting .maplibregl-ctrl-icon{animation:maplibregl-spin 2s linear infinite}@media (forced-colors:active){.maplibregl-ctrl button.maplibregl-ctrl-geolocate .maplibregl-ctrl-icon{background-image:url(" + Tt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate:disabled .maplibregl-ctrl-icon{background-image:url(" + It + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-active .maplibregl-ctrl-icon{background-image:url(" + Xt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-active-error .maplibregl-ctrl-icon{background-image:url(" + Nt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-background .maplibregl-ctrl-icon{background-image:url(" + Gt + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate.maplibregl-ctrl-geolocate-background-error .maplibregl-ctrl-icon{background-image:url(" + Ft + ")}}@media (forced-colors:active) and (prefers-color-scheme:light){.maplibregl-ctrl button.maplibregl-ctrl-geolocate .maplibregl-ctrl-icon{background-image:url(" + Ct + ")}.maplibregl-ctrl button.maplibregl-ctrl-geolocate:disabled .maplibregl-ctrl-icon{background-image:url(" + Kt + ")}}@keyframes maplibregl-spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}a.maplibregl-ctrl-logo{background-image:url(" + Vt + ");background-repeat:no-repeat;cursor:pointer;display:block;height:23px;margin:0 0 -4px -4px;overflow:hidden;width:88px}a.maplibregl-ctrl-logo.maplibregl-compact{width:14px}@media (forced-colors:active){a.maplibregl-ctrl-logo{background-color:transparent;background-image:url(" + Vt + ")}}@media (forced-colors:active) and (prefers-color-scheme:light){a.maplibregl-ctrl-logo{background-image:url(" + Vt + ")}}.maplibregl-ctrl.maplibregl-ctrl-attrib{background-color:hsla(0,0%,100%,.5);margin:0;padding:0 5px}@media screen{.maplibregl-ctrl-attrib.maplibregl-compact{background-color:#fff;border-radius:12px;box-sizing:content-box;color:#000;margin:10px;min-height:20px;padding:2px 24px 2px 0;position:relative}.maplibregl-ctrl-attrib.maplibregl-compact-show{padding:2px 28px 2px 8px;visibility:visible}.maplibregl-ctrl-bottom-left>.maplibregl-ctrl-attrib.maplibregl-compact-show,.maplibregl-ctrl-top-left>.maplibregl-ctrl-attrib.maplibregl-compact-show{border-radius:12px;padding:2px 8px 2px 28px}.maplibregl-ctrl-attrib.maplibregl-compact .maplibregl-ctrl-attrib-inner{display:none}.maplibregl-ctrl-attrib-button{background-color:hsla(0,0%,100%,.5);background-image:url(" + Yt + ");border:0;border-radius:12px;box-sizing:border-box;cursor:pointer;display:none;height:24px;outline:none;position:absolute;right:0;top:0;width:24px}.maplibregl-ctrl-attrib summary.maplibregl-ctrl-attrib-button{-webkit-appearance:none;-moz-appearance:none;appearance:none;list-style:none}.maplibregl-ctrl-attrib summary.maplibregl-ctrl-attrib-button::-webkit-details-marker{display:none}.maplibregl-ctrl-bottom-left .maplibregl-ctrl-attrib-button,.maplibregl-ctrl-top-left .maplibregl-ctrl-attrib-button{left:0}.maplibregl-ctrl-attrib.maplibregl-compact .maplibregl-ctrl-attrib-button,.maplibregl-ctrl-attrib.maplibregl-compact-show .maplibregl-ctrl-attrib-inner{display:block}.maplibregl-ctrl-attrib.maplibregl-compact-show .maplibregl-ctrl-attrib-button{background-color:rgb(0 0 0/5%)}.maplibregl-ctrl-bottom-right>.maplibregl-ctrl-attrib.maplibregl-compact:after{bottom:0;right:0}.maplibregl-ctrl-top-right>.maplibregl-ctrl-attrib.maplibregl-compact:after{right:0;top:0}.maplibregl-ctrl-top-left>.maplibregl-ctrl-attrib.maplibregl-compact:after{left:0;top:0}.maplibregl-ctrl-bottom-left>.maplibregl-ctrl-attrib.maplibregl-compact:after{bottom:0;left:0}}@media screen and (forced-colors:active){.maplibregl-ctrl-attrib.maplibregl-compact:after{background-image:url(" + sn + ")}}@media screen and (forced-colors:active) and (prefers-color-scheme:light){.maplibregl-ctrl-attrib.maplibregl-compact:after{background-image:url(" + Yt + ')}}.maplibregl-ctrl-attrib a{color:rgba(0,0,0,.75);text-decoration:none}.maplibregl-ctrl-attrib a:hover{color:inherit;text-decoration:underline}.maplibregl-attrib-empty{display:none}.maplibregl-ctrl-scale{background-color:hsla(0,0%,100%,.75);border:2px solid #333;border-top:#333;box-sizing:border-box;color:#333;font-size:10px;padding:0 5px}.maplibregl-popup{display:flex;left:0;pointer-events:none;position:absolute;top:0;will-change:transform}.maplibregl-popup-anchor-top,.maplibregl-popup-anchor-top-left,.maplibregl-popup-anchor-top-right{flex-direction:column}.maplibregl-popup-anchor-bottom,.maplibregl-popup-anchor-bottom-left,.maplibregl-popup-anchor-bottom-right{flex-direction:column-reverse}.maplibregl-popup-anchor-left{flex-direction:row}.maplibregl-popup-anchor-right{flex-direction:row-reverse}.maplibregl-popup-tip{border:10px solid transparent;height:0;width:0;z-index:1}.maplibregl-popup-anchor-top .maplibregl-popup-tip{align-self:center;border-bottom-color:#fff;border-top:none}.maplibregl-popup-anchor-top-left .maplibregl-popup-tip{align-self:flex-start;border-bottom-color:#fff;border-left:none;border-top:none}.maplibregl-popup-anchor-top-right .maplibregl-popup-tip{align-self:flex-end;border-bottom-color:#fff;border-right:none;border-top:none}.maplibregl-popup-anchor-bottom .maplibregl-popup-tip{align-self:center;border-bottom:none;border-top-color:#fff}.maplibregl-popup-anchor-bottom-left .maplibregl-popup-tip{align-self:flex-start;border-bottom:none;border-left:none;border-top-color:#fff}.maplibregl-popup-anchor-bottom-right .maplibregl-popup-tip{align-self:flex-end;border-bottom:none;border-right:none;border-top-color:#fff}.maplibregl-popup-anchor-left .maplibregl-popup-tip{align-self:center;border-left:none;border-right-color:#fff}.maplibregl-popup-anchor-right .maplibregl-popup-tip{align-self:center;border-left-color:#fff;border-right:none}.maplibregl-popup-close-button{background-color:transparent;border:0;border-radius:0 3px 0 0;cursor:pointer;position:absolute;right:0;top:0}.maplibregl-popup-close-button:hover{background-color:rgb(0 0 0/5%)}.maplibregl-popup-content{background:#fff;border-radius:3px;box-shadow:0 1px 2px rgba(0,0,0,.1);padding:15px 10px;pointer-events:auto;position:relative}.maplibregl-popup-anchor-top-left .maplibregl-popup-content{border-top-left-radius:0}.maplibregl-popup-anchor-top-right .maplibregl-popup-content{border-top-right-radius:0}.maplibregl-popup-anchor-bottom-left .maplibregl-popup-content{border-bottom-left-radius:0}.maplibregl-popup-anchor-bottom-right .maplibregl-popup-content{border-bottom-right-radius:0}.maplibregl-popup-track-pointer{display:none}.maplibregl-popup-track-pointer *{pointer-events:none;-webkit-user-select:none;-moz-user-select:none;user-select:none}.maplibregl-map:hover .maplibregl-popup-track-pointer{display:flex}.maplibregl-map:active .maplibregl-popup-track-pointer{display:none}.maplibregl-marker{left:0;position:absolute;top:0;transition:opacity .2s;will-change:transform}.maplibregl-user-location-dot,.maplibregl-user-location-dot:before{background-color:#1da1f2;border-radius:50%;height:15px;width:15px}.maplibregl-user-location-dot:before{animation:maplibregl-user-location-dot-pulse 2s infinite;content:"";position:absolute}.maplibregl-user-location-dot:after{border:2px solid #fff;border-radius:50%;box-shadow:0 0 3px rgba(0,0,0,.35);box-sizing:border-box;content:"";height:19px;left:-2px;position:absolute;top:-2px;width:19px}@keyframes maplibregl-user-location-dot-pulse{0%{opacity:1;transform:scale(1)}70%{opacity:0;transform:scale(3)}to{opacity:0;transform:scale(1)}}.maplibregl-user-location-dot-stale{background-color:#aaa}.maplibregl-user-location-dot-stale:after{display:none}.maplibregl-user-location-accuracy-circle{background-color:#1da1f233;border-radius:100%;height:1px;width:1px}.maplibregl-crosshair,.maplibregl-crosshair .maplibregl-interactive,.maplibregl-crosshair .maplibregl-interactive:active{cursor:crosshair}.maplibregl-boxzoom{background:#fff;border:2px dotted #202020;height:0;left:0;opacity:.5;position:absolute;top:0;width:0}.maplibregl-cooperative-gesture-screen{align-items:center;background:rgba(0,0,0,.4);color:#fff;display:flex;font-size:1.4em;inset:0;justify-content:center;line-height:1.2;opacity:0;padding:1rem;pointer-events:none;position:absolute;transition:opacity 1s ease 1s;z-index:99999}.maplibregl-cooperative-gesture-screen.maplibregl-show{opacity:1;transition:opacity .05s}.maplibregl-cooperative-gesture-screen .maplibregl-mobile-message{display:none}@media (hover:none),(width <= 480px){.maplibregl-cooperative-gesture-screen .maplibregl-desktop-message{display:none}.maplibregl-cooperative-gesture-screen .maplibregl-mobile-message{display:block}}.maplibregl-pseudo-fullscreen{height:100%!important;left:0!important;position:fixed!important;top:0!important;width:100%!important;z-index:99999}', ""]), J.A = Ge;
          }, 68735: function(G, J, Z) {
            Z.r(J), Z.d(J, { sankeyCenter: function() {
              return de;
            }, sankeyCircular: function() {
              return Re;
            }, sankeyJustify: function() {
              return ue;
            }, sankeyLeft: function() {
              return ce;
            }, sankeyRight: function() {
              return le;
            } });
            var te = Z(29725), ne = Z(4575), ee = Z(48544), ae = Z(96143), oe = Z.n(ae);
            function se(Rt) {
              return Rt.target.depth;
            }
            function ce(Rt) {
              return Rt.depth;
            }
            function le(Rt, vt) {
              return vt - 1 - Rt.height;
            }
            function ue(Rt, vt) {
              return Rt.sourceLinks.length ? Rt.depth : vt - 1;
            }
            function de(Rt) {
              return Rt.targetLinks.length ? Rt.depth : Rt.sourceLinks.length ? (0, te.jk)(Rt.sourceLinks, se) - 1 : 0;
            }
            function fe(Rt) {
              return function() {
                return Rt;
              };
            }
            var he = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(Rt) {
              return typeof Rt;
            } : function(Rt) {
              return Rt && typeof Symbol == "function" && Rt.constructor === Symbol && Rt !== Symbol.prototype ? "symbol" : typeof Rt;
            };
            function me(Rt, vt) {
              return ye(Rt.source, vt.source) || Rt.index - vt.index;
            }
            function pe(Rt, vt) {
              return ye(Rt.target, vt.target) || Rt.index - vt.index;
            }
            function ye(Rt, vt) {
              return Rt.partOfCycle === vt.partOfCycle ? Rt.y0 - vt.y0 : Rt.circularLinkType === "top" || vt.circularLinkType === "bottom" ? -1 : 1;
            }
            function ge(Rt) {
              return Rt.value;
            }
            function ve(Rt) {
              return (Rt.y0 + Rt.y1) / 2;
            }
            function be(Rt) {
              return ve(Rt.source);
            }
            function Ce(Rt) {
              return ve(Rt.target);
            }
            function Se(Rt) {
              return Rt.index;
            }
            function we(Rt) {
              return Rt.nodes;
            }
            function Ee(Rt) {
              return Rt.links;
            }
            function xe(Rt, vt) {
              var $t = Rt.get(vt);
              if (!$t) throw new Error("missing: " + vt);
              return $t;
            }
            function Le(Rt, vt) {
              return vt(Rt);
            }
            var Qe = 25, ke = 10, Me = 0.3;
            function Re() {
              var Rt, vt, $t = 0, Pt = 0, Xt = 1, Nt = 1, Gt = 24, Ft = Se, Tt = ue, It = we, Ct = Ee, Kt = 32, Vt = 2, Yt = null;
              function sn() {
                var Qt = { nodes: It.apply(null, arguments), links: Ct.apply(null, arguments) };
                (function(Wt) {
                  Wt.nodes.forEach(function(vn, dn) {
                    vn.index = dn, vn.sourceLinks = [], vn.targetLinks = [];
                  });
                  var rn = (0, ne.Tj)(Wt.nodes, Ft);
                  Wt.links.forEach(function(vn, dn) {
                    vn.index = dn;
                    var Ht = vn.source, jt = vn.target;
                    (Ht === void 0 ? "undefined" : he(Ht)) !== "object" && (Ht = vn.source = xe(rn, Ht)), (jt === void 0 ? "undefined" : he(jt)) !== "object" && (jt = vn.target = xe(rn, jt)), Ht.sourceLinks.push(vn), jt.targetLinks.push(vn);
                  });
                })(Qt), function(Wt, rn, vn) {
                  var dn = 0;
                  if (vn === null) {
                    for (var Ht = [], jt = 0; jt < Wt.links.length; jt++) {
                      var cn = Wt.links[jt], wn = cn.source.index, Rn = cn.target.index;
                      Ht[wn] || (Ht[wn] = []), Ht[Rn] || (Ht[Rn] = []), Ht[wn].indexOf(Rn) === -1 && Ht[wn].push(Rn);
                    }
                    var Pn = oe()(Ht);
                    Pn.sort(function(ft, pt) {
                      return ft.length - pt.length;
                    });
                    var Ie = {};
                    for (jt = 0; jt < Pn.length; jt++) {
                      var rt = Pn[jt].slice(-2);
                      Ie[rt[0]] || (Ie[rt[0]] = {}), Ie[rt[0]][rt[1]] = !0;
                    }
                    Wt.links.forEach(function(ft) {
                      var pt = ft.target.index, wt = ft.source.index;
                      pt === wt || Ie[wt] && Ie[wt][pt] ? (ft.circular = !0, ft.circularLinkID = dn, dn += 1) : ft.circular = !1;
                    });
                  } else Wt.links.forEach(function(ft) {
                    ft.source[vn] < ft.target[vn] ? ft.circular = !1 : (ft.circular = !0, ft.circularLinkID = dn, dn += 1);
                  });
                }(Qt, 0, Yt), function(Wt) {
                  Wt.nodes.forEach(function(rn) {
                    rn.partOfCycle = !1, rn.value = Math.max((0, te.cz)(rn.sourceLinks, ge), (0, te.cz)(rn.targetLinks, ge)), rn.sourceLinks.forEach(function(vn) {
                      vn.circular && (rn.partOfCycle = !0, rn.circularLinkType = vn.circularLinkType);
                    }), rn.targetLinks.forEach(function(vn) {
                      vn.circular && (rn.partOfCycle = !0, rn.circularLinkType = vn.circularLinkType);
                    });
                  });
                }(Qt), function(Wt) {
                  var rn, vn, dn;
                  for (rn = Wt.nodes, vn = [], dn = 0; rn.length; ++dn, rn = vn, vn = []) rn.forEach(function(Ht) {
                    Ht.depth = dn, Ht.sourceLinks.forEach(function(jt) {
                      vn.indexOf(jt.target) < 0 && !jt.circular && vn.push(jt.target);
                    });
                  });
                  for (rn = Wt.nodes, vn = [], dn = 0; rn.length; ++dn, rn = vn, vn = []) rn.forEach(function(Ht) {
                    Ht.height = dn, Ht.targetLinks.forEach(function(jt) {
                      vn.indexOf(jt.source) < 0 && !jt.circular && vn.push(jt.source);
                    });
                  });
                  Wt.nodes.forEach(function(Ht) {
                    Ht.column = Math.floor(Tt.call(null, Ht, dn));
                  });
                }(Qt), De(Qt, Ft), function(Wt, rn, vn) {
                  var dn = (0, ne.$I)().key(function(Rn) {
                    return Rn.column;
                  }).sortKeys(te.V_).entries(Wt.nodes).map(function(Rn) {
                    return Rn.values;
                  });
                  (function(Rn) {
                    if (vt) {
                      var Pn = 1 / 0;
                      dn.forEach(function(pt) {
                        var wt = Nt * vt / (pt.length + 1);
                        Pn = wt < Pn ? wt : Pn;
                      }), Rt = Pn;
                    }
                    var Ie = (0, te.jk)(dn, function(pt) {
                      return (Nt - Pt - (pt.length - 1) * Rt) / (0, te.cz)(pt, ge);
                    });
                    Ie *= Me, Wt.links.forEach(function(pt) {
                      pt.width = pt.value * Ie;
                    });
                    var rt = function(pt) {
                      var wt = 0, Bt = 0, At = 0, Dt = 0, Ot = (0, te.T9)(pt.nodes, function(un) {
                        return un.column;
                      });
                      return pt.links.forEach(function(un) {
                        un.circular && (un.circularLinkType == "top" ? wt += un.width : Bt += un.width, un.target.column == 0 && (Dt += un.width), un.source.column == Ot && (At += un.width));
                      }), { top: wt = wt > 0 ? wt + Qe + ke : wt, bottom: Bt = Bt > 0 ? Bt + Qe + ke : Bt, left: Dt = Dt > 0 ? Dt + Qe + ke : Dt, right: At = At > 0 ? At + Qe + ke : At };
                    }(Wt), ft = function(pt, wt) {
                      var Bt = (0, te.T9)(pt.nodes, function(yn) {
                        return yn.column;
                      }), At = Xt - $t, Dt = Nt - Pt, Ot = At / (At + wt.right + wt.left), un = Dt / (Dt + wt.top + wt.bottom);
                      return $t = $t * Ot + wt.left, Xt = wt.right == 0 ? Xt : Xt * Ot, Pt = Pt * un + wt.top, Nt *= un, pt.nodes.forEach(function(yn) {
                        yn.x0 = $t + yn.column * ((Xt - $t - Gt) / Bt), yn.x1 = yn.x0 + Gt;
                      }), un;
                    }(Wt, rt);
                    Ie *= ft, Wt.links.forEach(function(pt) {
                      pt.width = pt.value * Ie;
                    }), dn.forEach(function(pt) {
                      var wt = pt.length;
                      pt.forEach(function(Bt, At) {
                        Bt.depth == dn.length - 1 && wt == 1 || Bt.depth == 0 && wt == 1 ? (Bt.y0 = Nt / 2 - Bt.value * Ie, Bt.y1 = Bt.y0 + Bt.value * Ie) : Bt.partOfCycle ? $e(Bt, Rn) == 0 ? (Bt.y0 = Nt / 2 + At, Bt.y1 = Bt.y0 + Bt.value * Ie) : Bt.circularLinkType == "top" ? (Bt.y0 = Pt + At, Bt.y1 = Bt.y0 + Bt.value * Ie) : (Bt.y0 = Nt - Bt.value * Ie - At, Bt.y1 = Bt.y0 + Bt.value * Ie) : rt.top == 0 || rt.bottom == 0 ? (Bt.y0 = (Nt - Pt) / wt * At, Bt.y1 = Bt.y0 + Bt.value * Ie) : (Bt.y0 = (Nt - Pt) / 2 - wt / 2 + At, Bt.y1 = Bt.y0 + Bt.value * Ie);
                      });
                    });
                  })(vn), wn();
                  for (var Ht = 1, jt = rn; jt > 0; --jt) cn(Ht *= 0.99, vn), wn();
                  function cn(Rn, Pn) {
                    var Ie = dn.length;
                    dn.forEach(function(rt) {
                      var ft = rt.length, pt = rt[0].depth;
                      rt.forEach(function(wt) {
                        var Bt;
                        if ((wt.sourceLinks.length || wt.targetLinks.length) && !(wt.partOfCycle && $e(wt, Pn) > 0)) if (pt == 0 && ft == 1) Bt = wt.y1 - wt.y0, wt.y0 = Nt / 2 - Bt / 2, wt.y1 = Nt / 2 + Bt / 2;
                        else if (pt == Ie - 1 && ft == 1) Bt = wt.y1 - wt.y0, wt.y0 = Nt / 2 - Bt / 2, wt.y1 = Nt / 2 + Bt / 2;
                        else {
                          var At = (0, te.i2)(wt.sourceLinks, Ce), Dt = (0, te.i2)(wt.targetLinks, be), Ot = ((At && Dt ? (At + Dt) / 2 : At || Dt) - ve(wt)) * Rn;
                          wt.y0 += Ot, wt.y1 += Ot;
                        }
                      });
                    });
                  }
                  function wn() {
                    dn.forEach(function(Rn) {
                      var Pn, Ie, rt, ft = Pt, pt = Rn.length;
                      for (Rn.sort(ye), rt = 0; rt < pt; ++rt) (Ie = ft - (Pn = Rn[rt]).y0) > 0 && (Pn.y0 += Ie, Pn.y1 += Ie), ft = Pn.y1 + Rt;
                      if ((Ie = ft - Rt - Nt) > 0) for (ft = Pn.y0 -= Ie, Pn.y1 -= Ie, rt = pt - 2; rt >= 0; --rt) (Ie = (Pn = Rn[rt]).y1 + Rt - ft) > 0 && (Pn.y0 -= Ie, Pn.y1 -= Ie), ft = Pn.y0;
                    });
                  }
                }(Qt, Kt, Ft), hn(Qt);
                for (var Ut = 0; Ut < 4; Ut++) gt(Qt, Nt, Ft), bt(Qt, 0, Ft), it(Qt, Pt, Nt, Ft), gt(Qt, Nt, Ft), bt(Qt, 0, Ft);
                return function(Wt, rn, vn) {
                  var dn = Wt.nodes, Ht = Wt.links, jt = !1, cn = !1;
                  if (Ht.forEach(function(Pn) {
                    Pn.circularLinkType == "top" ? jt = !0 : Pn.circularLinkType == "bottom" && (cn = !0);
                  }), jt == 0 || cn == 0) {
                    var wn = (0, te.jk)(dn, function(Pn) {
                      return Pn.y0;
                    }), Rn = (vn - rn) / ((0, te.T9)(dn, function(Pn) {
                      return Pn.y1;
                    }) - wn);
                    dn.forEach(function(Pn) {
                      var Ie = (Pn.y1 - Pn.y0) * Rn;
                      Pn.y0 = (Pn.y0 - wn) * Rn, Pn.y1 = Pn.y0 + Ie;
                    }), Ht.forEach(function(Pn) {
                      Pn.y0 = (Pn.y0 - wn) * Rn, Pn.y1 = (Pn.y1 - wn) * Rn, Pn.width = Pn.width * Rn;
                    });
                  }
                }(Qt, Pt, Nt), Ve(Qt, Vt, Nt, Ft), Qt;
              }
              function hn(Qt) {
                Qt.nodes.forEach(function(Ut) {
                  Ut.sourceLinks.sort(pe), Ut.targetLinks.sort(me);
                }), Qt.nodes.forEach(function(Ut) {
                  var Wt = Ut.y0, rn = Wt, vn = Ut.y1, dn = vn;
                  Ut.sourceLinks.forEach(function(Ht) {
                    Ht.circular ? (Ht.y0 = vn - Ht.width / 2, vn -= Ht.width) : (Ht.y0 = Wt + Ht.width / 2, Wt += Ht.width);
                  }), Ut.targetLinks.forEach(function(Ht) {
                    Ht.circular ? (Ht.y1 = dn - Ht.width / 2, dn -= Ht.width) : (Ht.y1 = rn + Ht.width / 2, rn += Ht.width);
                  });
                });
              }
              return sn.nodeId = function(Qt) {
                return arguments.length ? (Ft = typeof Qt == "function" ? Qt : fe(Qt), sn) : Ft;
              }, sn.nodeAlign = function(Qt) {
                return arguments.length ? (Tt = typeof Qt == "function" ? Qt : fe(Qt), sn) : Tt;
              }, sn.nodeWidth = function(Qt) {
                return arguments.length ? (Gt = +Qt, sn) : Gt;
              }, sn.nodePadding = function(Qt) {
                return arguments.length ? (Rt = +Qt, sn) : Rt;
              }, sn.nodes = function(Qt) {
                return arguments.length ? (It = typeof Qt == "function" ? Qt : fe(Qt), sn) : It;
              }, sn.links = function(Qt) {
                return arguments.length ? (Ct = typeof Qt == "function" ? Qt : fe(Qt), sn) : Ct;
              }, sn.size = function(Qt) {
                return arguments.length ? ($t = Pt = 0, Xt = +Qt[0], Nt = +Qt[1], sn) : [Xt - $t, Nt - Pt];
              }, sn.extent = function(Qt) {
                return arguments.length ? ($t = +Qt[0][0], Xt = +Qt[1][0], Pt = +Qt[0][1], Nt = +Qt[1][1], sn) : [[$t, Pt], [Xt, Nt]];
              }, sn.iterations = function(Qt) {
                return arguments.length ? (Kt = +Qt, sn) : Kt;
              }, sn.circularLinkGap = function(Qt) {
                return arguments.length ? (Vt = +Qt, sn) : Vt;
              }, sn.nodePaddingRatio = function(Qt) {
                return arguments.length ? (vt = +Qt, sn) : vt;
              }, sn.sortNodes = function(Qt) {
                return arguments.length ? (Yt = Qt, sn) : Yt;
              }, sn.update = function(Qt) {
                return De(Qt, Ft), hn(Qt), Qt.links.forEach(function(Ut) {
                  Ut.circular && (Ut.circularLinkType = Ut.y0 + Ut.y1 < Nt ? "top" : "bottom", Ut.source.circularLinkType = Ut.circularLinkType, Ut.target.circularLinkType = Ut.circularLinkType);
                }), gt(Qt, Nt, Ft, !1), bt(Qt, 0, Ft), Ve(Qt, Vt, Nt, Ft), Qt;
              }, sn;
            }
            function De(Rt, vt) {
              var $t = 0, Pt = 0;
              Rt.links.forEach(function(Xt) {
                Xt.circular && (Xt.source.circularLinkType || Xt.target.circularLinkType ? Xt.circularLinkType = Xt.source.circularLinkType ? Xt.source.circularLinkType : Xt.target.circularLinkType : Xt.circularLinkType = $t < Pt ? "top" : "bottom", Xt.circularLinkType == "top" ? $t += 1 : Pt += 1, Rt.nodes.forEach(function(Nt) {
                  Le(Nt, vt) != Le(Xt.source, vt) && Le(Nt, vt) != Le(Xt.target, vt) || (Nt.circularLinkType = Xt.circularLinkType);
                }));
              }), Rt.links.forEach(function(Xt) {
                Xt.circular && (Xt.source.circularLinkType == Xt.target.circularLinkType && (Xt.circularLinkType = Xt.source.circularLinkType), Lt(Xt, vt) && (Xt.circularLinkType = Xt.source.circularLinkType));
              });
            }
            function Pe(Rt) {
              var vt = Math.abs(Rt.y1 - Rt.y0), $t = Math.abs(Rt.target.x0 - Rt.source.x1);
              return Math.atan($t / vt);
            }
            function $e(Rt, vt) {
              var $t = 0;
              Rt.sourceLinks.forEach(function(Xt) {
                $t = Xt.circular && !Lt(Xt, vt) ? $t + 1 : $t;
              });
              var Pt = 0;
              return Rt.targetLinks.forEach(function(Xt) {
                Pt = Xt.circular && !Lt(Xt, vt) ? Pt + 1 : Pt;
              }), $t + Pt;
            }
            function qe(Rt) {
              var vt = Rt.source.sourceLinks, $t = 0;
              vt.forEach(function(Nt) {
                $t = Nt.circular ? $t + 1 : $t;
              });
              var Pt = Rt.target.targetLinks, Xt = 0;
              return Pt.forEach(function(Nt) {
                Xt = Nt.circular ? Xt + 1 : Xt;
              }), !($t > 1 || Xt > 1);
            }
            function Xe(Rt, vt, $t) {
              return Rt.sort(Ze), Rt.forEach(function(Pt, Xt) {
                var Nt, Gt, Ft = 0;
                if (Lt(Pt, $t) && qe(Pt)) Pt.circularPathData.verticalBuffer = Ft + Pt.width / 2;
                else {
                  for (var Tt = 0; Tt < Xt; Tt++) if (Nt = Rt[Xt], Gt = Rt[Tt], !(Nt.source.column < Gt.target.column || Nt.target.column > Gt.source.column)) {
                    var It = Rt[Tt].circularPathData.verticalBuffer + Rt[Tt].width / 2 + vt;
                    Ft = It > Ft ? It : Ft;
                  }
                  Pt.circularPathData.verticalBuffer = Ft + Pt.width / 2;
                }
              }), Rt;
            }
            function Ve(Rt, vt, $t, Pt) {
              var Xt = (0, te.jk)(Rt.links, function(Nt) {
                return Nt.source.y0;
              });
              Rt.links.forEach(function(Nt) {
                Nt.circular && (Nt.circularPathData = {});
              }), Xe(Rt.links.filter(function(Nt) {
                return Nt.circularLinkType == "top";
              }), vt, Pt), Xe(Rt.links.filter(function(Nt) {
                return Nt.circularLinkType == "bottom";
              }), vt, Pt), Rt.links.forEach(function(Nt) {
                if (Nt.circular) {
                  if (Nt.circularPathData.arcRadius = Nt.width + ke, Nt.circularPathData.leftNodeBuffer = 5, Nt.circularPathData.rightNodeBuffer = 5, Nt.circularPathData.sourceWidth = Nt.source.x1 - Nt.source.x0, Nt.circularPathData.sourceX = Nt.source.x0 + Nt.circularPathData.sourceWidth, Nt.circularPathData.targetX = Nt.target.x0, Nt.circularPathData.sourceY = Nt.y0, Nt.circularPathData.targetY = Nt.y1, Lt(Nt, Pt) && qe(Nt)) Nt.circularPathData.leftSmallArcRadius = ke + Nt.width / 2, Nt.circularPathData.leftLargeArcRadius = ke + Nt.width / 2, Nt.circularPathData.rightSmallArcRadius = ke + Nt.width / 2, Nt.circularPathData.rightLargeArcRadius = ke + Nt.width / 2, Nt.circularLinkType == "bottom" ? (Nt.circularPathData.verticalFullExtent = Nt.source.y1 + Qe + Nt.circularPathData.verticalBuffer, Nt.circularPathData.verticalLeftInnerExtent = Nt.circularPathData.verticalFullExtent - Nt.circularPathData.leftLargeArcRadius, Nt.circularPathData.verticalRightInnerExtent = Nt.circularPathData.verticalFullExtent - Nt.circularPathData.rightLargeArcRadius) : (Nt.circularPathData.verticalFullExtent = Nt.source.y0 - Qe - Nt.circularPathData.verticalBuffer, Nt.circularPathData.verticalLeftInnerExtent = Nt.circularPathData.verticalFullExtent + Nt.circularPathData.leftLargeArcRadius, Nt.circularPathData.verticalRightInnerExtent = Nt.circularPathData.verticalFullExtent + Nt.circularPathData.rightLargeArcRadius);
                  else {
                    var Gt = Nt.source.column, Ft = Nt.circularLinkType, Tt = Rt.links.filter(function(Kt) {
                      return Kt.source.column == Gt && Kt.circularLinkType == Ft;
                    });
                    Nt.circularLinkType == "bottom" ? Tt.sort(ze) : Tt.sort(Ge);
                    var It = 0;
                    Tt.forEach(function(Kt, Vt) {
                      Kt.circularLinkID == Nt.circularLinkID && (Nt.circularPathData.leftSmallArcRadius = ke + Nt.width / 2 + It, Nt.circularPathData.leftLargeArcRadius = ke + Nt.width / 2 + Vt * vt + It), It += Kt.width;
                    }), Gt = Nt.target.column, Tt = Rt.links.filter(function(Kt) {
                      return Kt.target.column == Gt && Kt.circularLinkType == Ft;
                    }), Nt.circularLinkType == "bottom" ? Tt.sort(at) : Tt.sort(We), It = 0, Tt.forEach(function(Kt, Vt) {
                      Kt.circularLinkID == Nt.circularLinkID && (Nt.circularPathData.rightSmallArcRadius = ke + Nt.width / 2 + It, Nt.circularPathData.rightLargeArcRadius = ke + Nt.width / 2 + Vt * vt + It), It += Kt.width;
                    }), Nt.circularLinkType == "bottom" ? (Nt.circularPathData.verticalFullExtent = Math.max($t, Nt.source.y1, Nt.target.y1) + Qe + Nt.circularPathData.verticalBuffer, Nt.circularPathData.verticalLeftInnerExtent = Nt.circularPathData.verticalFullExtent - Nt.circularPathData.leftLargeArcRadius, Nt.circularPathData.verticalRightInnerExtent = Nt.circularPathData.verticalFullExtent - Nt.circularPathData.rightLargeArcRadius) : (Nt.circularPathData.verticalFullExtent = Xt - Qe - Nt.circularPathData.verticalBuffer, Nt.circularPathData.verticalLeftInnerExtent = Nt.circularPathData.verticalFullExtent + Nt.circularPathData.leftLargeArcRadius, Nt.circularPathData.verticalRightInnerExtent = Nt.circularPathData.verticalFullExtent + Nt.circularPathData.rightLargeArcRadius);
                  }
                  Nt.circularPathData.leftInnerExtent = Nt.circularPathData.sourceX + Nt.circularPathData.leftNodeBuffer, Nt.circularPathData.rightInnerExtent = Nt.circularPathData.targetX - Nt.circularPathData.rightNodeBuffer, Nt.circularPathData.leftFullExtent = Nt.circularPathData.sourceX + Nt.circularPathData.leftLargeArcRadius + Nt.circularPathData.leftNodeBuffer, Nt.circularPathData.rightFullExtent = Nt.circularPathData.targetX - Nt.circularPathData.rightLargeArcRadius - Nt.circularPathData.rightNodeBuffer;
                }
                if (Nt.circular) Nt.path = function(Kt) {
                  return Kt.circularLinkType == "top" ? "M" + Kt.circularPathData.sourceX + " " + Kt.circularPathData.sourceY + " L" + Kt.circularPathData.leftInnerExtent + " " + Kt.circularPathData.sourceY + " A" + Kt.circularPathData.leftLargeArcRadius + " " + Kt.circularPathData.leftSmallArcRadius + " 0 0 0 " + Kt.circularPathData.leftFullExtent + " " + (Kt.circularPathData.sourceY - Kt.circularPathData.leftSmallArcRadius) + " L" + Kt.circularPathData.leftFullExtent + " " + Kt.circularPathData.verticalLeftInnerExtent + " A" + Kt.circularPathData.leftLargeArcRadius + " " + Kt.circularPathData.leftLargeArcRadius + " 0 0 0 " + Kt.circularPathData.leftInnerExtent + " " + Kt.circularPathData.verticalFullExtent + " L" + Kt.circularPathData.rightInnerExtent + " " + Kt.circularPathData.verticalFullExtent + " A" + Kt.circularPathData.rightLargeArcRadius + " " + Kt.circularPathData.rightLargeArcRadius + " 0 0 0 " + Kt.circularPathData.rightFullExtent + " " + Kt.circularPathData.verticalRightInnerExtent + " L" + Kt.circularPathData.rightFullExtent + " " + (Kt.circularPathData.targetY - Kt.circularPathData.rightSmallArcRadius) + " A" + Kt.circularPathData.rightLargeArcRadius + " " + Kt.circularPathData.rightSmallArcRadius + " 0 0 0 " + Kt.circularPathData.rightInnerExtent + " " + Kt.circularPathData.targetY + " L" + Kt.circularPathData.targetX + " " + Kt.circularPathData.targetY : "M" + Kt.circularPathData.sourceX + " " + Kt.circularPathData.sourceY + " L" + Kt.circularPathData.leftInnerExtent + " " + Kt.circularPathData.sourceY + " A" + Kt.circularPathData.leftLargeArcRadius + " " + Kt.circularPathData.leftSmallArcRadius + " 0 0 1 " + Kt.circularPathData.leftFullExtent + " " + (Kt.circularPathData.sourceY + Kt.circularPathData.leftSmallArcRadius) + " L" + Kt.circularPathData.leftFullExtent + " " + Kt.circularPathData.verticalLeftInnerExtent + " A" + Kt.circularPathData.leftLargeArcRadius + " " + Kt.circularPathData.leftLargeArcRadius + " 0 0 1 " + Kt.circularPathData.leftInnerExtent + " " + Kt.circularPathData.verticalFullExtent + " L" + Kt.circularPathData.rightInnerExtent + " " + Kt.circularPathData.verticalFullExtent + " A" + Kt.circularPathData.rightLargeArcRadius + " " + Kt.circularPathData.rightLargeArcRadius + " 0 0 1 " + Kt.circularPathData.rightFullExtent + " " + Kt.circularPathData.verticalRightInnerExtent + " L" + Kt.circularPathData.rightFullExtent + " " + (Kt.circularPathData.targetY + Kt.circularPathData.rightSmallArcRadius) + " A" + Kt.circularPathData.rightLargeArcRadius + " " + Kt.circularPathData.rightSmallArcRadius + " 0 0 1 " + Kt.circularPathData.rightInnerExtent + " " + Kt.circularPathData.targetY + " L" + Kt.circularPathData.targetX + " " + Kt.circularPathData.targetY;
                }(Nt);
                else {
                  var Ct = (0, ee.pq)().source(function(Kt) {
                    return [Kt.source.x0 + (Kt.source.x1 - Kt.source.x0), Kt.y0];
                  }).target(function(Kt) {
                    return [Kt.target.x0, Kt.y1];
                  });
                  Nt.path = Ct(Nt);
                }
              });
            }
            function Ze(Rt, vt) {
              return ct(Rt) == ct(vt) ? Rt.circularLinkType == "bottom" ? ze(Rt, vt) : Ge(Rt, vt) : ct(vt) - ct(Rt);
            }
            function Ge(Rt, vt) {
              return Rt.y0 - vt.y0;
            }
            function ze(Rt, vt) {
              return vt.y0 - Rt.y0;
            }
            function We(Rt, vt) {
              return Rt.y1 - vt.y1;
            }
            function at(Rt, vt) {
              return vt.y1 - Rt.y1;
            }
            function ct(Rt) {
              return Rt.target.column - Rt.source.column;
            }
            function et(Rt) {
              return Rt.target.x0 - Rt.source.x1;
            }
            function dt(Rt, vt) {
              var $t = Pe(Rt), Pt = et(vt) / Math.tan($t);
              return St(Rt) == "up" ? Rt.y1 + Pt : Rt.y1 - Pt;
            }
            function Je(Rt, vt) {
              var $t = Pe(Rt), Pt = et(vt) / Math.tan($t);
              return St(Rt) == "up" ? Rt.y1 - Pt : Rt.y1 + Pt;
            }
            function it(Rt, vt, $t, Pt) {
              Rt.links.forEach(function(Xt) {
                if (!Xt.circular && Xt.target.column - Xt.source.column > 1) {
                  var Nt = Xt.source.column + 1, Gt = Xt.target.column - 1, Ft = 1, Tt = Gt - Nt + 1;
                  for (Ft = 1; Nt <= Gt; Nt++, Ft++) Rt.nodes.forEach(function(It) {
                    if (It.column == Nt) {
                      var Ct, Kt = Ft / (Tt + 1), Vt = Math.pow(1 - Kt, 3), Yt = 3 * Kt * Math.pow(1 - Kt, 2), sn = 3 * Math.pow(Kt, 2) * (1 - Kt), hn = Math.pow(Kt, 3), Qt = Vt * Xt.y0 + Yt * Xt.y0 + sn * Xt.y1 + hn * Xt.y1, Ut = Qt - Xt.width / 2, Wt = Qt + Xt.width / 2;
                      Ut > It.y0 && Ut < It.y1 ? (Ct = It.y1 - Ut + 10, Ct = It.circularLinkType == "bottom" ? Ct : -Ct, It = st(It, Ct, vt, $t), Rt.nodes.forEach(function(rn) {
                        var vn, dn;
                        Le(rn, Pt) != Le(It, Pt) && rn.column == It.column && (dn = rn, (vn = It).y0 > dn.y0 && vn.y0 < dn.y1 || vn.y1 > dn.y0 && vn.y1 < dn.y1 || vn.y0 < dn.y0 && vn.y1 > dn.y1) && st(rn, Ct, vt, $t);
                      })) : (Wt > It.y0 && Wt < It.y1 || Ut < It.y0 && Wt > It.y1) && (Ct = Wt - It.y0 + 10, It = st(It, Ct, vt, $t), Rt.nodes.forEach(function(rn) {
                        Le(rn, Pt) != Le(It, Pt) && rn.column == It.column && rn.y0 < It.y1 && rn.y1 > It.y1 && st(rn, Ct, vt, $t);
                      }));
                    }
                  });
                }
              });
            }
            function st(Rt, vt, $t, Pt) {
              return Rt.y0 + vt >= $t && Rt.y1 + vt <= Pt && (Rt.y0 = Rt.y0 + vt, Rt.y1 = Rt.y1 + vt, Rt.targetLinks.forEach(function(Xt) {
                Xt.y1 = Xt.y1 + vt;
              }), Rt.sourceLinks.forEach(function(Xt) {
                Xt.y0 = Xt.y0 + vt;
              })), Rt;
            }
            function gt(Rt, vt, $t, Pt) {
              Rt.nodes.forEach(function(Xt) {
                Pt && Xt.y + (Xt.y1 - Xt.y0) > vt && (Xt.y = Xt.y - (Xt.y + (Xt.y1 - Xt.y0) - vt));
                var Nt = Rt.links.filter(function(Tt) {
                  return Le(Tt.source, $t) == Le(Xt, $t);
                }), Gt = Nt.length;
                Gt > 1 && Nt.sort(function(Tt, It) {
                  if (!Tt.circular && !It.circular) {
                    if (Tt.target.column == It.target.column || !yt(Tt, It)) return Tt.y1 - It.y1;
                    if (Tt.target.column > It.target.column) {
                      var Ct = Je(It, Tt);
                      return Tt.y1 - Ct;
                    }
                    if (It.target.column > Tt.target.column) return Je(Tt, It) - It.y1;
                  }
                  return Tt.circular && !It.circular ? Tt.circularLinkType == "top" ? -1 : 1 : It.circular && !Tt.circular ? It.circularLinkType == "top" ? 1 : -1 : Tt.circular && It.circular ? Tt.circularLinkType === It.circularLinkType && Tt.circularLinkType == "top" ? Tt.target.column === It.target.column ? Tt.target.y1 - It.target.y1 : It.target.column - Tt.target.column : Tt.circularLinkType === It.circularLinkType && Tt.circularLinkType == "bottom" ? Tt.target.column === It.target.column ? It.target.y1 - Tt.target.y1 : Tt.target.column - It.target.column : Tt.circularLinkType == "top" ? -1 : 1 : void 0;
                });
                var Ft = Xt.y0;
                Nt.forEach(function(Tt) {
                  Tt.y0 = Ft + Tt.width / 2, Ft += Tt.width;
                }), Nt.forEach(function(Tt, It) {
                  if (Tt.circularLinkType == "bottom") {
                    for (var Ct = It + 1, Kt = 0; Ct < Gt; Ct++) Kt += Nt[Ct].width;
                    Tt.y0 = Xt.y1 - Kt - Tt.width / 2;
                  }
                });
              });
            }
            function bt(Rt, vt, $t) {
              Rt.nodes.forEach(function(Pt) {
                var Xt = Rt.links.filter(function(Ft) {
                  return Le(Ft.target, $t) == Le(Pt, $t);
                }), Nt = Xt.length;
                Nt > 1 && Xt.sort(function(Ft, Tt) {
                  if (!Ft.circular && !Tt.circular) {
                    if (Ft.source.column == Tt.source.column || !yt(Ft, Tt)) return Ft.y0 - Tt.y0;
                    if (Tt.source.column < Ft.source.column) {
                      var It = dt(Tt, Ft);
                      return Ft.y0 - It;
                    }
                    if (Ft.source.column < Tt.source.column) return dt(Ft, Tt) - Tt.y0;
                  }
                  return Ft.circular && !Tt.circular ? Ft.circularLinkType == "top" ? -1 : 1 : Tt.circular && !Ft.circular ? Tt.circularLinkType == "top" ? 1 : -1 : Ft.circular && Tt.circular ? Ft.circularLinkType === Tt.circularLinkType && Ft.circularLinkType == "top" ? Ft.source.column === Tt.source.column ? Ft.source.y1 - Tt.source.y1 : Ft.source.column - Tt.source.column : Ft.circularLinkType === Tt.circularLinkType && Ft.circularLinkType == "bottom" ? Ft.source.column === Tt.source.column ? Ft.source.y1 - Tt.source.y1 : Tt.source.column - Ft.source.column : Ft.circularLinkType == "top" ? -1 : 1 : void 0;
                });
                var Gt = Pt.y0;
                Xt.forEach(function(Ft) {
                  Ft.y1 = Gt + Ft.width / 2, Gt += Ft.width;
                }), Xt.forEach(function(Ft, Tt) {
                  if (Ft.circularLinkType == "bottom") {
                    for (var It = Tt + 1, Ct = 0; It < Nt; It++) Ct += Xt[It].width;
                    Ft.y1 = Pt.y1 - Ct - Ft.width / 2;
                  }
                });
              });
            }
            function yt(Rt, vt) {
              return St(Rt) == St(vt);
            }
            function St(Rt) {
              return Rt.y0 - Rt.y1 > 0 ? "up" : "down";
            }
            function Lt(Rt, vt) {
              return Le(Rt.source, vt) == Le(Rt.target, vt);
            }
          }, 62369: function(G, J, Z) {
            Z.r(J), Z.d(J, { sankey: function() {
              return Se;
            }, sankeyCenter: function() {
              return ce;
            }, sankeyJustify: function() {
              return se;
            }, sankeyLeft: function() {
              return ae;
            }, sankeyLinkHorizontal: function() {
              return Le;
            }, sankeyRight: function() {
              return oe;
            } });
            var te = Z(29725), ne = Z(4575);
            function ee(Qe) {
              return Qe.target.depth;
            }
            function ae(Qe) {
              return Qe.depth;
            }
            function oe(Qe, ke) {
              return ke - 1 - Qe.height;
            }
            function se(Qe, ke) {
              return Qe.sourceLinks.length ? Qe.depth : ke - 1;
            }
            function ce(Qe) {
              return Qe.targetLinks.length ? Qe.depth : Qe.sourceLinks.length ? (0, te.jk)(Qe.sourceLinks, ee) - 1 : 0;
            }
            function le(Qe) {
              return function() {
                return Qe;
              };
            }
            function ue(Qe, ke) {
              return fe(Qe.source, ke.source) || Qe.index - ke.index;
            }
            function de(Qe, ke) {
              return fe(Qe.target, ke.target) || Qe.index - ke.index;
            }
            function fe(Qe, ke) {
              return Qe.y0 - ke.y0;
            }
            function he(Qe) {
              return Qe.value;
            }
            function me(Qe) {
              return (Qe.y0 + Qe.y1) / 2;
            }
            function pe(Qe) {
              return me(Qe.source) * Qe.value;
            }
            function ye(Qe) {
              return me(Qe.target) * Qe.value;
            }
            function ge(Qe) {
              return Qe.index;
            }
            function ve(Qe) {
              return Qe.nodes;
            }
            function be(Qe) {
              return Qe.links;
            }
            function Ce(Qe, ke) {
              var Me = Qe.get(ke);
              if (!Me) throw new Error("missing: " + ke);
              return Me;
            }
            function Se() {
              var Qe = 0, ke = 0, Me = 1, Re = 1, De = 24, Pe = 8, $e = ge, qe = se, Xe = ve, Ve = be, Ze = 32;
              function Ge() {
                var We = { nodes: Xe.apply(null, arguments), links: Ve.apply(null, arguments) };
                return function(at) {
                  at.nodes.forEach(function(et, dt) {
                    et.index = dt, et.sourceLinks = [], et.targetLinks = [];
                  });
                  var ct = (0, ne.Tj)(at.nodes, $e);
                  at.links.forEach(function(et, dt) {
                    et.index = dt;
                    var Je = et.source, it = et.target;
                    typeof Je != "object" && (Je = et.source = Ce(ct, Je)), typeof it != "object" && (it = et.target = Ce(ct, it)), Je.sourceLinks.push(et), it.targetLinks.push(et);
                  });
                }(We), function(at) {
                  at.nodes.forEach(function(ct) {
                    ct.value = Math.max((0, te.cz)(ct.sourceLinks, he), (0, te.cz)(ct.targetLinks, he));
                  });
                }(We), function(at) {
                  var ct, et, dt;
                  for (ct = at.nodes, et = [], dt = 0; ct.length; ++dt, ct = et, et = []) ct.forEach(function(it) {
                    it.depth = dt, it.sourceLinks.forEach(function(st) {
                      et.indexOf(st.target) < 0 && et.push(st.target);
                    });
                  });
                  for (ct = at.nodes, et = [], dt = 0; ct.length; ++dt, ct = et, et = []) ct.forEach(function(it) {
                    it.height = dt, it.targetLinks.forEach(function(st) {
                      et.indexOf(st.source) < 0 && et.push(st.source);
                    });
                  });
                  var Je = (Me - Qe - De) / (dt - 1);
                  at.nodes.forEach(function(it) {
                    it.x1 = (it.x0 = Qe + Math.max(0, Math.min(dt - 1, Math.floor(qe.call(null, it, dt)))) * Je) + De;
                  });
                }(We), function(at) {
                  var ct = (0, ne.$I)().key(function(gt) {
                    return gt.x0;
                  }).sortKeys(te.V_).entries(at.nodes).map(function(gt) {
                    return gt.values;
                  });
                  (function() {
                    var gt = (0, te.T9)(ct, function(St) {
                      return St.length;
                    }), bt = 0.6666666666666666 * (Re - ke) / (gt - 1);
                    Pe > bt && (Pe = bt);
                    var yt = (0, te.jk)(ct, function(St) {
                      return (Re - ke - (St.length - 1) * Pe) / (0, te.cz)(St, he);
                    });
                    ct.forEach(function(St) {
                      St.forEach(function(Lt, Rt) {
                        Lt.y1 = (Lt.y0 = Rt) + Lt.value * yt;
                      });
                    }), at.links.forEach(function(St) {
                      St.width = St.value * yt;
                    });
                  })(), st();
                  for (var et = 1, dt = Ze; dt > 0; --dt) it(et *= 0.99), st(), Je(et), st();
                  function Je(gt) {
                    ct.forEach(function(bt) {
                      bt.forEach(function(yt) {
                        if (yt.targetLinks.length) {
                          var St = ((0, te.cz)(yt.targetLinks, pe) / (0, te.cz)(yt.targetLinks, he) - me(yt)) * gt;
                          yt.y0 += St, yt.y1 += St;
                        }
                      });
                    });
                  }
                  function it(gt) {
                    ct.slice().reverse().forEach(function(bt) {
                      bt.forEach(function(yt) {
                        if (yt.sourceLinks.length) {
                          var St = ((0, te.cz)(yt.sourceLinks, ye) / (0, te.cz)(yt.sourceLinks, he) - me(yt)) * gt;
                          yt.y0 += St, yt.y1 += St;
                        }
                      });
                    });
                  }
                  function st() {
                    ct.forEach(function(gt) {
                      var bt, yt, St, Lt = ke, Rt = gt.length;
                      for (gt.sort(fe), St = 0; St < Rt; ++St) (yt = Lt - (bt = gt[St]).y0) > 0 && (bt.y0 += yt, bt.y1 += yt), Lt = bt.y1 + Pe;
                      if ((yt = Lt - Pe - Re) > 0) for (Lt = bt.y0 -= yt, bt.y1 -= yt, St = Rt - 2; St >= 0; --St) (yt = (bt = gt[St]).y1 + Pe - Lt) > 0 && (bt.y0 -= yt, bt.y1 -= yt), Lt = bt.y0;
                    });
                  }
                }(We), ze(We), We;
              }
              function ze(We) {
                We.nodes.forEach(function(at) {
                  at.sourceLinks.sort(de), at.targetLinks.sort(ue);
                }), We.nodes.forEach(function(at) {
                  var ct = at.y0, et = ct;
                  at.sourceLinks.forEach(function(dt) {
                    dt.y0 = ct + dt.width / 2, ct += dt.width;
                  }), at.targetLinks.forEach(function(dt) {
                    dt.y1 = et + dt.width / 2, et += dt.width;
                  });
                });
              }
              return Ge.update = function(We) {
                return ze(We), We;
              }, Ge.nodeId = function(We) {
                return arguments.length ? ($e = typeof We == "function" ? We : le(We), Ge) : $e;
              }, Ge.nodeAlign = function(We) {
                return arguments.length ? (qe = typeof We == "function" ? We : le(We), Ge) : qe;
              }, Ge.nodeWidth = function(We) {
                return arguments.length ? (De = +We, Ge) : De;
              }, Ge.nodePadding = function(We) {
                return arguments.length ? (Pe = +We, Ge) : Pe;
              }, Ge.nodes = function(We) {
                return arguments.length ? (Xe = typeof We == "function" ? We : le(We), Ge) : Xe;
              }, Ge.links = function(We) {
                return arguments.length ? (Ve = typeof We == "function" ? We : le(We), Ge) : Ve;
              }, Ge.size = function(We) {
                return arguments.length ? (Qe = ke = 0, Me = +We[0], Re = +We[1], Ge) : [Me - Qe, Re - ke];
              }, Ge.extent = function(We) {
                return arguments.length ? (Qe = +We[0][0], Me = +We[1][0], ke = +We[0][1], Re = +We[1][1], Ge) : [[Qe, ke], [Me, Re]];
              }, Ge.iterations = function(We) {
                return arguments.length ? (Ze = +We, Ge) : Ze;
              }, Ge;
            }
            var we = Z(48544);
            function Ee(Qe) {
              return [Qe.source.x1, Qe.y0];
            }
            function xe(Qe) {
              return [Qe.target.x0, Qe.y1];
            }
            function Le() {
              return (0, we.pq)().source(Ee).target(xe);
            }
          }, 45568: function(G, J, Z) {
            var te, ne;
            (function() {
              var ee = { version: "3.8.2" }, ae = [].slice, oe = function(Ae) {
                return ae.call(Ae);
              }, se = self.document;
              function ce(Ae) {
                return Ae && (Ae.ownerDocument || Ae.document || Ae).documentElement;
              }
              function le(Ae) {
                return Ae && (Ae.ownerDocument && Ae.ownerDocument.defaultView || Ae.document && Ae || Ae.defaultView);
              }
              if (se) try {
                oe(se.documentElement.childNodes)[0].nodeType;
              } catch {
                oe = function(Oe) {
                  for (var Be = Oe.length, Ue = new Array(Be); Be--; ) Ue[Be] = Oe[Be];
                  return Ue;
                };
              }
              if (Date.now || (Date.now = function() {
                return +/* @__PURE__ */ new Date();
              }), se) try {
                se.createElement("DIV").style.setProperty("opacity", 0, "");
              } catch {
                var ue = this.Element.prototype, de = ue.setAttribute, fe = ue.setAttributeNS, he = this.CSSStyleDeclaration.prototype, me = he.setProperty;
                ue.setAttribute = function(Oe, Be) {
                  de.call(this, Oe, Be + "");
                }, ue.setAttributeNS = function(Oe, Be, Ue) {
                  fe.call(this, Oe, Be, Ue + "");
                }, he.setProperty = function(Oe, Be, Ue) {
                  me.call(this, Oe, Be + "", Ue);
                };
              }
              function pe(Ae, Oe) {
                return Ae < Oe ? -1 : Ae > Oe ? 1 : Ae >= Oe ? 0 : NaN;
              }
              function ye(Ae) {
                return Ae === null ? NaN : +Ae;
              }
              function ge(Ae) {
                return !isNaN(Ae);
              }
              function ve(Ae) {
                return { left: function(Oe, Be, Ue, nt) {
                  for (arguments.length < 3 && (Ue = 0), arguments.length < 4 && (nt = Oe.length); Ue < nt; ) {
                    var ht = Ue + nt >>> 1;
                    Ae(Oe[ht], Be) < 0 ? Ue = ht + 1 : nt = ht;
                  }
                  return Ue;
                }, right: function(Oe, Be, Ue, nt) {
                  for (arguments.length < 3 && (Ue = 0), arguments.length < 4 && (nt = Oe.length); Ue < nt; ) {
                    var ht = Ue + nt >>> 1;
                    Ae(Oe[ht], Be) > 0 ? nt = ht : Ue = ht + 1;
                  }
                  return Ue;
                } };
              }
              ee.ascending = pe, ee.descending = function(Ae, Oe) {
                return Oe < Ae ? -1 : Oe > Ae ? 1 : Oe >= Ae ? 0 : NaN;
              }, ee.min = function(Ae, Oe) {
                var Be, Ue, nt = -1, ht = Ae.length;
                if (arguments.length === 1) {
                  for (; ++nt < ht; ) if ((Ue = Ae[nt]) != null && Ue >= Ue) {
                    Be = Ue;
                    break;
                  }
                  for (; ++nt < ht; ) (Ue = Ae[nt]) != null && Be > Ue && (Be = Ue);
                } else {
                  for (; ++nt < ht; ) if ((Ue = Oe.call(Ae, Ae[nt], nt)) != null && Ue >= Ue) {
                    Be = Ue;
                    break;
                  }
                  for (; ++nt < ht; ) (Ue = Oe.call(Ae, Ae[nt], nt)) != null && Be > Ue && (Be = Ue);
                }
                return Be;
              }, ee.max = function(Ae, Oe) {
                var Be, Ue, nt = -1, ht = Ae.length;
                if (arguments.length === 1) {
                  for (; ++nt < ht; ) if ((Ue = Ae[nt]) != null && Ue >= Ue) {
                    Be = Ue;
                    break;
                  }
                  for (; ++nt < ht; ) (Ue = Ae[nt]) != null && Ue > Be && (Be = Ue);
                } else {
                  for (; ++nt < ht; ) if ((Ue = Oe.call(Ae, Ae[nt], nt)) != null && Ue >= Ue) {
                    Be = Ue;
                    break;
                  }
                  for (; ++nt < ht; ) (Ue = Oe.call(Ae, Ae[nt], nt)) != null && Ue > Be && (Be = Ue);
                }
                return Be;
              }, ee.extent = function(Ae, Oe) {
                var Be, Ue, nt, ht = -1, Et = Ae.length;
                if (arguments.length === 1) {
                  for (; ++ht < Et; ) if ((Ue = Ae[ht]) != null && Ue >= Ue) {
                    Be = nt = Ue;
                    break;
                  }
                  for (; ++ht < Et; ) (Ue = Ae[ht]) != null && (Be > Ue && (Be = Ue), nt < Ue && (nt = Ue));
                } else {
                  for (; ++ht < Et; ) if ((Ue = Oe.call(Ae, Ae[ht], ht)) != null && Ue >= Ue) {
                    Be = nt = Ue;
                    break;
                  }
                  for (; ++ht < Et; ) (Ue = Oe.call(Ae, Ae[ht], ht)) != null && (Be > Ue && (Be = Ue), nt < Ue && (nt = Ue));
                }
                return [Be, nt];
              }, ee.sum = function(Ae, Oe) {
                var Be, Ue = 0, nt = Ae.length, ht = -1;
                if (arguments.length === 1) for (; ++ht < nt; ) ge(Be = +Ae[ht]) && (Ue += Be);
                else for (; ++ht < nt; ) ge(Be = +Oe.call(Ae, Ae[ht], ht)) && (Ue += Be);
                return Ue;
              }, ee.mean = function(Ae, Oe) {
                var Be, Ue = 0, nt = Ae.length, ht = -1, Et = nt;
                if (arguments.length === 1) for (; ++ht < nt; ) ge(Be = ye(Ae[ht])) ? Ue += Be : --Et;
                else for (; ++ht < nt; ) ge(Be = ye(Oe.call(Ae, Ae[ht], ht))) ? Ue += Be : --Et;
                if (Et) return Ue / Et;
              }, ee.quantile = function(Ae, Oe) {
                var Be = (Ae.length - 1) * Oe + 1, Ue = Math.floor(Be), nt = +Ae[Ue - 1], ht = Be - Ue;
                return ht ? nt + ht * (Ae[Ue] - nt) : nt;
              }, ee.median = function(Ae, Oe) {
                var Be, Ue = [], nt = Ae.length, ht = -1;
                if (arguments.length === 1) for (; ++ht < nt; ) ge(Be = ye(Ae[ht])) && Ue.push(Be);
                else for (; ++ht < nt; ) ge(Be = ye(Oe.call(Ae, Ae[ht], ht))) && Ue.push(Be);
                if (Ue.length) return ee.quantile(Ue.sort(pe), 0.5);
              }, ee.variance = function(Ae, Oe) {
                var Be, Ue, nt = Ae.length, ht = 0, Et = 0, Mt = -1, qt = 0;
                if (arguments.length === 1) for (; ++Mt < nt; ) ge(Be = ye(Ae[Mt])) && (Et += (Ue = Be - ht) * (Be - (ht += Ue / ++qt)));
                else for (; ++Mt < nt; ) ge(Be = ye(Oe.call(Ae, Ae[Mt], Mt))) && (Et += (Ue = Be - ht) * (Be - (ht += Ue / ++qt)));
                if (qt > 1) return Et / (qt - 1);
              }, ee.deviation = function() {
                var Ae = ee.variance.apply(this, arguments);
                return Ae && Math.sqrt(Ae);
              };
              var be = ve(pe);
              function Ce(Ae) {
                return Ae.length;
              }
              ee.bisectLeft = be.left, ee.bisect = ee.bisectRight = be.right, ee.bisector = function(Ae) {
                return ve(Ae.length === 1 ? function(Oe, Be) {
                  return pe(Ae(Oe), Be);
                } : Ae);
              }, ee.shuffle = function(Ae, Oe, Be) {
                (ht = arguments.length) < 3 && (Be = Ae.length, ht < 2 && (Oe = 0));
                for (var Ue, nt, ht = Be - Oe; ht; ) nt = Math.random() * ht-- | 0, Ue = Ae[ht + Oe], Ae[ht + Oe] = Ae[nt + Oe], Ae[nt + Oe] = Ue;
                return Ae;
              }, ee.permute = function(Ae, Oe) {
                for (var Be = Oe.length, Ue = new Array(Be); Be--; ) Ue[Be] = Ae[Oe[Be]];
                return Ue;
              }, ee.pairs = function(Ae) {
                for (var Oe = 0, Be = Ae.length - 1, Ue = Ae[0], nt = new Array(Be < 0 ? 0 : Be); Oe < Be; ) nt[Oe] = [Ue, Ue = Ae[++Oe]];
                return nt;
              }, ee.transpose = function(Ae) {
                if (!(nt = Ae.length)) return [];
                for (var Oe = -1, Be = ee.min(Ae, Ce), Ue = new Array(Be); ++Oe < Be; ) for (var nt, ht = -1, Et = Ue[Oe] = new Array(nt); ++ht < nt; ) Et[ht] = Ae[ht][Oe];
                return Ue;
              }, ee.zip = function() {
                return ee.transpose(arguments);
              }, ee.keys = function(Ae) {
                var Oe = [];
                for (var Be in Ae) Oe.push(Be);
                return Oe;
              }, ee.values = function(Ae) {
                var Oe = [];
                for (var Be in Ae) Oe.push(Ae[Be]);
                return Oe;
              }, ee.entries = function(Ae) {
                var Oe = [];
                for (var Be in Ae) Oe.push({ key: Be, value: Ae[Be] });
                return Oe;
              }, ee.merge = function(Ae) {
                for (var Oe, Be, Ue, nt = Ae.length, ht = -1, Et = 0; ++ht < nt; ) Et += Ae[ht].length;
                for (Be = new Array(Et); --nt >= 0; ) for (Oe = (Ue = Ae[nt]).length; --Oe >= 0; ) Be[--Et] = Ue[Oe];
                return Be;
              };
              var Se = Math.abs;
              function we(Ae, Oe) {
                for (var Be in Oe) Object.defineProperty(Ae.prototype, Be, { value: Oe[Be], enumerable: !1 });
              }
              function Ee() {
                this._ = /* @__PURE__ */ Object.create(null);
              }
              ee.range = function(Ae, Oe, Be) {
                if (arguments.length < 3 && (Be = 1, arguments.length < 2 && (Oe = Ae, Ae = 0)), (Oe - Ae) / Be == 1 / 0) throw new Error("infinite range");
                var Ue, nt = [], ht = function(Mt) {
                  for (var qt = 1; Mt * qt % 1; ) qt *= 10;
                  return qt;
                }(Se(Be)), Et = -1;
                if (Ae *= ht, Oe *= ht, (Be *= ht) < 0) for (; (Ue = Ae + Be * ++Et) > Oe; ) nt.push(Ue / ht);
                else for (; (Ue = Ae + Be * ++Et) < Oe; ) nt.push(Ue / ht);
                return nt;
              }, ee.map = function(Ae, Oe) {
                var Be = new Ee();
                if (Ae instanceof Ee) Ae.forEach(function(Mt, qt) {
                  Be.set(Mt, qt);
                });
                else if (Array.isArray(Ae)) {
                  var Ue, nt = -1, ht = Ae.length;
                  if (arguments.length === 1) for (; ++nt < ht; ) Be.set(nt, Ae[nt]);
                  else for (; ++nt < ht; ) Be.set(Oe.call(Ae, Ue = Ae[nt], nt), Ue);
                } else for (var Et in Ae) Be.set(Et, Ae[Et]);
                return Be;
              };
              var xe = "__proto__", Le = "\0";
              function Qe(Ae) {
                return (Ae += "") === xe || Ae[0] === Le ? Le + Ae : Ae;
              }
              function ke(Ae) {
                return (Ae += "")[0] === Le ? Ae.slice(1) : Ae;
              }
              function Me(Ae) {
                return Qe(Ae) in this._;
              }
              function Re(Ae) {
                return (Ae = Qe(Ae)) in this._ && delete this._[Ae];
              }
              function De() {
                var Ae = [];
                for (var Oe in this._) Ae.push(ke(Oe));
                return Ae;
              }
              function Pe() {
                var Ae = 0;
                for (var Oe in this._) ++Ae;
                return Ae;
              }
              function $e() {
                for (var Ae in this._) return !1;
                return !0;
              }
              function qe() {
                this._ = /* @__PURE__ */ Object.create(null);
              }
              function Xe(Ae) {
                return Ae;
              }
              function Ve(Ae, Oe, Be) {
                return function() {
                  var Ue = Be.apply(Oe, arguments);
                  return Ue === Oe ? Ae : Ue;
                };
              }
              function Ze(Ae, Oe) {
                if (Oe in Ae) return Oe;
                Oe = Oe.charAt(0).toUpperCase() + Oe.slice(1);
                for (var Be = 0, Ue = Ge.length; Be < Ue; ++Be) {
                  var nt = Ge[Be] + Oe;
                  if (nt in Ae) return nt;
                }
              }
              we(Ee, { has: Me, get: function(Ae) {
                return this._[Qe(Ae)];
              }, set: function(Ae, Oe) {
                return this._[Qe(Ae)] = Oe;
              }, remove: Re, keys: De, values: function() {
                var Ae = [];
                for (var Oe in this._) Ae.push(this._[Oe]);
                return Ae;
              }, entries: function() {
                var Ae = [];
                for (var Oe in this._) Ae.push({ key: ke(Oe), value: this._[Oe] });
                return Ae;
              }, size: Pe, empty: $e, forEach: function(Ae) {
                for (var Oe in this._) Ae.call(this, ke(Oe), this._[Oe]);
              } }), ee.nest = function() {
                var Ae, Oe, Be = {}, Ue = [], nt = [];
                function ht(Mt, qt, tn) {
                  if (tn >= Ue.length) return Oe ? Oe.call(Be, qt) : Ae ? qt.sort(Ae) : qt;
                  for (var xn, An, On, Bn, qn = -1, Vn = qt.length, sr = Ue[tn++], Un = new Ee(); ++qn < Vn; ) (Bn = Un.get(xn = sr(An = qt[qn]))) ? Bn.push(An) : Un.set(xn, [An]);
                  return Mt ? (An = Mt(), On = function(wr, Cr) {
                    An.set(wr, ht(Mt, Cr, tn));
                  }) : (An = {}, On = function(wr, Cr) {
                    An[wr] = ht(Mt, Cr, tn);
                  }), Un.forEach(On), An;
                }
                function Et(Mt, qt) {
                  if (qt >= Ue.length) return Mt;
                  var tn = [], xn = nt[qt++];
                  return Mt.forEach(function(An, On) {
                    tn.push({ key: An, values: Et(On, qt) });
                  }), xn ? tn.sort(function(An, On) {
                    return xn(An.key, On.key);
                  }) : tn;
                }
                return Be.map = function(Mt, qt) {
                  return ht(qt, Mt, 0);
                }, Be.entries = function(Mt) {
                  return Et(ht(ee.map, Mt, 0), 0);
                }, Be.key = function(Mt) {
                  return Ue.push(Mt), Be;
                }, Be.sortKeys = function(Mt) {
                  return nt[Ue.length - 1] = Mt, Be;
                }, Be.sortValues = function(Mt) {
                  return Ae = Mt, Be;
                }, Be.rollup = function(Mt) {
                  return Oe = Mt, Be;
                }, Be;
              }, ee.set = function(Ae) {
                var Oe = new qe();
                if (Ae) for (var Be = 0, Ue = Ae.length; Be < Ue; ++Be) Oe.add(Ae[Be]);
                return Oe;
              }, we(qe, { has: Me, add: function(Ae) {
                return this._[Qe(Ae += "")] = !0, Ae;
              }, remove: Re, values: De, size: Pe, empty: $e, forEach: function(Ae) {
                for (var Oe in this._) Ae.call(this, ke(Oe));
              } }), ee.behavior = {}, ee.rebind = function(Ae, Oe) {
                for (var Be, Ue = 1, nt = arguments.length; ++Ue < nt; ) Ae[Be = arguments[Ue]] = Ve(Ae, Oe, Oe[Be]);
                return Ae;
              };
              var Ge = ["webkit", "ms", "moz", "Moz", "o", "O"];
              function ze() {
              }
              function We() {
              }
              function at(Ae) {
                var Oe = [], Be = new Ee();
                function Ue() {
                  for (var nt, ht = Oe, Et = -1, Mt = ht.length; ++Et < Mt; ) (nt = ht[Et].on) && nt.apply(this, arguments);
                  return Ae;
                }
                return Ue.on = function(nt, ht) {
                  var Et, Mt = Be.get(nt);
                  return arguments.length < 2 ? Mt && Mt.on : (Mt && (Mt.on = null, Oe = Oe.slice(0, Et = Oe.indexOf(Mt)).concat(Oe.slice(Et + 1)), Be.remove(nt)), ht && Oe.push(Be.set(nt, { on: ht })), Ae);
                }, Ue;
              }
              function ct() {
                ee.event.preventDefault();
              }
              function et() {
                for (var Ae, Oe = ee.event; Ae = Oe.sourceEvent; ) Oe = Ae;
                return Oe;
              }
              function dt(Ae) {
                for (var Oe = new We(), Be = 0, Ue = arguments.length; ++Be < Ue; ) Oe[arguments[Be]] = at(Oe);
                return Oe.of = function(nt, ht) {
                  return function(Et) {
                    try {
                      var Mt = Et.sourceEvent = ee.event;
                      Et.target = Ae, ee.event = Et, Oe[Et.type].apply(nt, ht);
                    } finally {
                      ee.event = Mt;
                    }
                  };
                }, Oe;
              }
              ee.dispatch = function() {
                for (var Ae = new We(), Oe = -1, Be = arguments.length; ++Oe < Be; ) Ae[arguments[Oe]] = at(Ae);
                return Ae;
              }, We.prototype.on = function(Ae, Oe) {
                var Be = Ae.indexOf("."), Ue = "";
                if (Be >= 0 && (Ue = Ae.slice(Be + 1), Ae = Ae.slice(0, Be)), Ae) return arguments.length < 2 ? this[Ae].on(Ue) : this[Ae].on(Ue, Oe);
                if (arguments.length === 2) {
                  if (Oe == null) for (Ae in this) this.hasOwnProperty(Ae) && this[Ae].on(Ue, null);
                  return this;
                }
              }, ee.event = null, ee.requote = function(Ae) {
                return Ae.replace(Je, "\\$&");
              };
              var Je = /[\\\^\$\*\+\?\|\[\]\(\)\.\{\}]/g, it = {}.__proto__ ? function(Ae, Oe) {
                Ae.__proto__ = Oe;
              } : function(Ae, Oe) {
                for (var Be in Oe) Ae[Be] = Oe[Be];
              };
              function st(Ae) {
                return it(Ae, St), Ae;
              }
              var gt = function(Ae, Oe) {
                return Oe.querySelector(Ae);
              }, bt = function(Ae, Oe) {
                return Oe.querySelectorAll(Ae);
              }, yt = function(Ae, Oe) {
                var Be = Ae.matches || Ae[Ze(Ae, "matchesSelector")];
                return yt = function(Ue, nt) {
                  return Be.call(Ue, nt);
                }, yt(Ae, Oe);
              };
              typeof Sizzle == "function" && (gt = function(Ae, Oe) {
                return Sizzle(Ae, Oe)[0] || null;
              }, bt = Sizzle, yt = Sizzle.matchesSelector), ee.selection = function() {
                return ee.select(se.documentElement);
              };
              var St = ee.selection.prototype = [];
              function Lt(Ae) {
                return typeof Ae == "function" ? Ae : function() {
                  return gt(Ae, this);
                };
              }
              function Rt(Ae) {
                return typeof Ae == "function" ? Ae : function() {
                  return bt(Ae, this);
                };
              }
              St.select = function(Ae) {
                var Oe, Be, Ue, nt, ht = [];
                Ae = Lt(Ae);
                for (var Et = -1, Mt = this.length; ++Et < Mt; ) {
                  ht.push(Oe = []), Oe.parentNode = (Ue = this[Et]).parentNode;
                  for (var qt = -1, tn = Ue.length; ++qt < tn; ) (nt = Ue[qt]) ? (Oe.push(Be = Ae.call(nt, nt.__data__, qt, Et)), Be && "__data__" in nt && (Be.__data__ = nt.__data__)) : Oe.push(null);
                }
                return st(ht);
              }, St.selectAll = function(Ae) {
                var Oe, Be, Ue = [];
                Ae = Rt(Ae);
                for (var nt = -1, ht = this.length; ++nt < ht; ) for (var Et = this[nt], Mt = -1, qt = Et.length; ++Mt < qt; ) (Be = Et[Mt]) && (Ue.push(Oe = oe(Ae.call(Be, Be.__data__, Mt, nt))), Oe.parentNode = Be);
                return st(Ue);
              };
              var vt = "http://www.w3.org/1999/xhtml", $t = { svg: "http://www.w3.org/2000/svg", xhtml: vt, xlink: "http://www.w3.org/1999/xlink", xml: "http://www.w3.org/XML/1998/namespace", xmlns: "http://www.w3.org/2000/xmlns/" };
              function Pt(Ae, Oe) {
                return Ae = ee.ns.qualify(Ae), Oe == null ? Ae.local ? function() {
                  this.removeAttributeNS(Ae.space, Ae.local);
                } : function() {
                  this.removeAttribute(Ae);
                } : typeof Oe == "function" ? Ae.local ? function() {
                  var Be = Oe.apply(this, arguments);
                  Be == null ? this.removeAttributeNS(Ae.space, Ae.local) : this.setAttributeNS(Ae.space, Ae.local, Be);
                } : function() {
                  var Be = Oe.apply(this, arguments);
                  Be == null ? this.removeAttribute(Ae) : this.setAttribute(Ae, Be);
                } : Ae.local ? function() {
                  this.setAttributeNS(Ae.space, Ae.local, Oe);
                } : function() {
                  this.setAttribute(Ae, Oe);
                };
              }
              function Xt(Ae) {
                return Ae.trim().replace(/\s+/g, " ");
              }
              function Nt(Ae) {
                return new RegExp("(?:^|\\s+)" + ee.requote(Ae) + "(?:\\s+|$)", "g");
              }
              function Gt(Ae) {
                return (Ae + "").trim().split(/^|\s+/);
              }
              function Ft(Ae, Oe) {
                var Be = (Ae = Gt(Ae).map(Tt)).length;
                return typeof Oe == "function" ? function() {
                  for (var Ue = -1, nt = Oe.apply(this, arguments); ++Ue < Be; ) Ae[Ue](this, nt);
                } : function() {
                  for (var Ue = -1; ++Ue < Be; ) Ae[Ue](this, Oe);
                };
              }
              function Tt(Ae) {
                var Oe = Nt(Ae);
                return function(Be, Ue) {
                  if (nt = Be.classList) return Ue ? nt.add(Ae) : nt.remove(Ae);
                  var nt = Be.getAttribute("class") || "";
                  Ue ? (Oe.lastIndex = 0, Oe.test(nt) || Be.setAttribute("class", Xt(nt + " " + Ae))) : Be.setAttribute("class", Xt(nt.replace(Oe, " ")));
                };
              }
              function It(Ae, Oe, Be) {
                return Oe == null ? function() {
                  this.style.removeProperty(Ae);
                } : typeof Oe == "function" ? function() {
                  var Ue = Oe.apply(this, arguments);
                  Ue == null ? this.style.removeProperty(Ae) : this.style.setProperty(Ae, Ue, Be);
                } : function() {
                  this.style.setProperty(Ae, Oe, Be);
                };
              }
              function Ct(Ae, Oe) {
                return Oe == null ? function() {
                  delete this[Ae];
                } : typeof Oe == "function" ? function() {
                  var Be = Oe.apply(this, arguments);
                  Be == null ? delete this[Ae] : this[Ae] = Be;
                } : function() {
                  this[Ae] = Oe;
                };
              }
              function Kt(Ae) {
                return typeof Ae == "function" ? Ae : (Ae = ee.ns.qualify(Ae)).local ? function() {
                  return this.ownerDocument.createElementNS(Ae.space, Ae.local);
                } : function() {
                  var Oe = this.ownerDocument, Be = this.namespaceURI;
                  return Be === vt && Oe.documentElement.namespaceURI === vt ? Oe.createElement(Ae) : Oe.createElementNS(Be, Ae);
                };
              }
              function Vt() {
                var Ae = this.parentNode;
                Ae && Ae.removeChild(this);
              }
              function Yt(Ae) {
                return { __data__: Ae };
              }
              function sn(Ae) {
                return function() {
                  return yt(this, Ae);
                };
              }
              function hn(Ae) {
                return arguments.length || (Ae = pe), function(Oe, Be) {
                  return Oe && Be ? Ae(Oe.__data__, Be.__data__) : !Oe - !Be;
                };
              }
              function Qt(Ae, Oe) {
                for (var Be = 0, Ue = Ae.length; Be < Ue; Be++) for (var nt, ht = Ae[Be], Et = 0, Mt = ht.length; Et < Mt; Et++) (nt = ht[Et]) && Oe(nt, Et, Be);
                return Ae;
              }
              function Ut(Ae) {
                return it(Ae, Wt), Ae;
              }
              ee.ns = { prefix: $t, qualify: function(Ae) {
                var Oe = Ae.indexOf(":"), Be = Ae;
                return Oe >= 0 && (Be = Ae.slice(0, Oe)) !== "xmlns" && (Ae = Ae.slice(Oe + 1)), $t.hasOwnProperty(Be) ? { space: $t[Be], local: Ae } : Ae;
              } }, St.attr = function(Ae, Oe) {
                if (arguments.length < 2) {
                  if (typeof Ae == "string") {
                    var Be = this.node();
                    return (Ae = ee.ns.qualify(Ae)).local ? Be.getAttributeNS(Ae.space, Ae.local) : Be.getAttribute(Ae);
                  }
                  for (Oe in Ae) this.each(Pt(Oe, Ae[Oe]));
                  return this;
                }
                return this.each(Pt(Ae, Oe));
              }, St.classed = function(Ae, Oe) {
                if (arguments.length < 2) {
                  if (typeof Ae == "string") {
                    var Be = this.node(), Ue = (Ae = Gt(Ae)).length, nt = -1;
                    if (Oe = Be.classList) {
                      for (; ++nt < Ue; ) if (!Oe.contains(Ae[nt])) return !1;
                    } else for (Oe = Be.getAttribute("class"); ++nt < Ue; ) if (!Nt(Ae[nt]).test(Oe)) return !1;
                    return !0;
                  }
                  for (Oe in Ae) this.each(Ft(Oe, Ae[Oe]));
                  return this;
                }
                return this.each(Ft(Ae, Oe));
              }, St.style = function(Ae, Oe, Be) {
                var Ue = arguments.length;
                if (Ue < 3) {
                  if (typeof Ae != "string") {
                    for (Be in Ue < 2 && (Oe = ""), Ae) this.each(It(Be, Ae[Be], Oe));
                    return this;
                  }
                  if (Ue < 2) {
                    var nt = this.node();
                    return le(nt).getComputedStyle(nt, null).getPropertyValue(Ae);
                  }
                  Be = "";
                }
                return this.each(It(Ae, Oe, Be));
              }, St.property = function(Ae, Oe) {
                if (arguments.length < 2) {
                  if (typeof Ae == "string") return this.node()[Ae];
                  for (Oe in Ae) this.each(Ct(Oe, Ae[Oe]));
                  return this;
                }
                return this.each(Ct(Ae, Oe));
              }, St.text = function(Ae) {
                return arguments.length ? this.each(typeof Ae == "function" ? function() {
                  var Oe = Ae.apply(this, arguments);
                  this.textContent = Oe ?? "";
                } : Ae == null ? function() {
                  this.textContent = "";
                } : function() {
                  this.textContent = Ae;
                }) : this.node().textContent;
              }, St.html = function(Ae) {
                return arguments.length ? this.each(typeof Ae == "function" ? function() {
                  var Oe = Ae.apply(this, arguments);
                  this.innerHTML = Oe ?? "";
                } : Ae == null ? function() {
                  this.innerHTML = "";
                } : function() {
                  this.innerHTML = Ae;
                }) : this.node().innerHTML;
              }, St.append = function(Ae) {
                return Ae = Kt(Ae), this.select(function() {
                  return this.appendChild(Ae.apply(this, arguments));
                });
              }, St.insert = function(Ae, Oe) {
                return Ae = Kt(Ae), Oe = Lt(Oe), this.select(function() {
                  return this.insertBefore(Ae.apply(this, arguments), Oe.apply(this, arguments) || null);
                });
              }, St.remove = function() {
                return this.each(Vt);
              }, St.data = function(Ae, Oe) {
                var Be, Ue, nt = -1, ht = this.length;
                if (!arguments.length) {
                  for (Ae = new Array(ht = (Be = this[0]).length); ++nt < ht; ) (Ue = Be[nt]) && (Ae[nt] = Ue.__data__);
                  return Ae;
                }
                function Et(xn, An) {
                  var On, Bn, qn, Vn = xn.length, sr = An.length, Un = Math.min(Vn, sr), wr = new Array(sr), Cr = new Array(sr), Tr = new Array(Vn);
                  if (Oe) {
                    var ii, wi = new Ee(), Wi = new Array(Vn);
                    for (On = -1; ++On < Vn; ) (Bn = xn[On]) && (wi.has(ii = Oe.call(Bn, Bn.__data__, On)) ? Tr[On] = Bn : wi.set(ii, Bn), Wi[On] = ii);
                    for (On = -1; ++On < sr; ) (Bn = wi.get(ii = Oe.call(An, qn = An[On], On))) ? Bn !== !0 && (wr[On] = Bn, Bn.__data__ = qn) : Cr[On] = Yt(qn), wi.set(ii, !0);
                    for (On = -1; ++On < Vn; ) On in Wi && wi.get(Wi[On]) !== !0 && (Tr[On] = xn[On]);
                  } else {
                    for (On = -1; ++On < Un; ) Bn = xn[On], qn = An[On], Bn ? (Bn.__data__ = qn, wr[On] = Bn) : Cr[On] = Yt(qn);
                    for (; On < sr; ++On) Cr[On] = Yt(An[On]);
                    for (; On < Vn; ++On) Tr[On] = xn[On];
                  }
                  Cr.update = wr, Cr.parentNode = wr.parentNode = Tr.parentNode = xn.parentNode, Mt.push(Cr), qt.push(wr), tn.push(Tr);
                }
                var Mt = Ut([]), qt = st([]), tn = st([]);
                if (typeof Ae == "function") for (; ++nt < ht; ) Et(Be = this[nt], Ae.call(Be, Be.parentNode.__data__, nt));
                else for (; ++nt < ht; ) Et(Be = this[nt], Ae);
                return qt.enter = function() {
                  return Mt;
                }, qt.exit = function() {
                  return tn;
                }, qt;
              }, St.datum = function(Ae) {
                return arguments.length ? this.property("__data__", Ae) : this.property("__data__");
              }, St.filter = function(Ae) {
                var Oe, Be, Ue, nt = [];
                typeof Ae != "function" && (Ae = sn(Ae));
                for (var ht = 0, Et = this.length; ht < Et; ht++) {
                  nt.push(Oe = []), Oe.parentNode = (Be = this[ht]).parentNode;
                  for (var Mt = 0, qt = Be.length; Mt < qt; Mt++) (Ue = Be[Mt]) && Ae.call(Ue, Ue.__data__, Mt, ht) && Oe.push(Ue);
                }
                return st(nt);
              }, St.order = function() {
                for (var Ae = -1, Oe = this.length; ++Ae < Oe; ) for (var Be, Ue = this[Ae], nt = Ue.length - 1, ht = Ue[nt]; --nt >= 0; ) (Be = Ue[nt]) && (ht && ht !== Be.nextSibling && ht.parentNode.insertBefore(Be, ht), ht = Be);
                return this;
              }, St.sort = function(Ae) {
                Ae = hn.apply(this, arguments);
                for (var Oe = -1, Be = this.length; ++Oe < Be; ) this[Oe].sort(Ae);
                return this.order();
              }, St.each = function(Ae) {
                return Qt(this, function(Oe, Be, Ue) {
                  Ae.call(Oe, Oe.__data__, Be, Ue);
                });
              }, St.call = function(Ae) {
                var Oe = oe(arguments);
                return Ae.apply(Oe[0] = this, Oe), this;
              }, St.empty = function() {
                return !this.node();
              }, St.node = function() {
                for (var Ae = 0, Oe = this.length; Ae < Oe; Ae++) for (var Be = this[Ae], Ue = 0, nt = Be.length; Ue < nt; Ue++) {
                  var ht = Be[Ue];
                  if (ht) return ht;
                }
                return null;
              }, St.size = function() {
                var Ae = 0;
                return Qt(this, function() {
                  ++Ae;
                }), Ae;
              };
              var Wt = [];
              function rn(Ae, Oe, Be) {
                var Ue = "__on" + Ae, nt = Ae.indexOf("."), ht = dn;
                nt > 0 && (Ae = Ae.slice(0, nt));
                var Et = vn.get(Ae);
                function Mt() {
                  var qt = this[Ue];
                  qt && (this.removeEventListener(Ae, qt, qt.$), delete this[Ue]);
                }
                return Et && (Ae = Et, ht = Ht), nt ? Oe ? function() {
                  var qt = ht(Oe, oe(arguments));
                  Mt.call(this), this.addEventListener(Ae, this[Ue] = qt, qt.$ = Be), qt._ = Oe;
                } : Mt : Oe ? ze : function() {
                  var qt, tn = new RegExp("^__on([^.]+)" + ee.requote(Ae) + "$");
                  for (var xn in this) if (qt = xn.match(tn)) {
                    var An = this[xn];
                    this.removeEventListener(qt[1], An, An.$), delete this[xn];
                  }
                };
              }
              ee.selection.enter = Ut, ee.selection.enter.prototype = Wt, Wt.append = St.append, Wt.empty = St.empty, Wt.node = St.node, Wt.call = St.call, Wt.size = St.size, Wt.select = function(Ae) {
                for (var Oe, Be, Ue, nt, ht, Et = [], Mt = -1, qt = this.length; ++Mt < qt; ) {
                  Ue = (nt = this[Mt]).update, Et.push(Oe = []), Oe.parentNode = nt.parentNode;
                  for (var tn = -1, xn = nt.length; ++tn < xn; ) (ht = nt[tn]) ? (Oe.push(Ue[tn] = Be = Ae.call(nt.parentNode, ht.__data__, tn, Mt)), Be.__data__ = ht.__data__) : Oe.push(null);
                }
                return st(Et);
              }, Wt.insert = function(Ae, Oe) {
                var Be, Ue, nt;
                return arguments.length < 2 && (Be = this, Oe = function(ht, Et, Mt) {
                  var qt, tn = Be[Mt].update, xn = tn.length;
                  for (Mt != nt && (nt = Mt, Ue = 0), Et >= Ue && (Ue = Et + 1); !(qt = tn[Ue]) && ++Ue < xn; ) ;
                  return qt;
                }), St.insert.call(this, Ae, Oe);
              }, ee.select = function(Ae) {
                var Oe;
                return typeof Ae == "string" ? (Oe = [gt(Ae, se)]).parentNode = se.documentElement : (Oe = [Ae]).parentNode = ce(Ae), st([Oe]);
              }, ee.selectAll = function(Ae) {
                var Oe;
                return typeof Ae == "string" ? (Oe = oe(bt(Ae, se))).parentNode = se.documentElement : (Oe = oe(Ae)).parentNode = null, st([Oe]);
              }, St.on = function(Ae, Oe, Be) {
                var Ue = arguments.length;
                if (Ue < 3) {
                  if (typeof Ae != "string") {
                    for (Be in Ue < 2 && (Oe = !1), Ae) this.each(rn(Be, Ae[Be], Oe));
                    return this;
                  }
                  if (Ue < 2) return (Ue = this.node()["__on" + Ae]) && Ue._;
                  Be = !1;
                }
                return this.each(rn(Ae, Oe, Be));
              };
              var vn = ee.map({ mouseenter: "mouseover", mouseleave: "mouseout" });
              function dn(Ae, Oe) {
                return function(Be) {
                  var Ue = ee.event;
                  ee.event = Be, Oe[0] = this.__data__;
                  try {
                    Ae.apply(this, Oe);
                  } finally {
                    ee.event = Ue;
                  }
                };
              }
              function Ht(Ae, Oe) {
                var Be = dn(Ae, Oe);
                return function(Ue) {
                  var nt = this, ht = Ue.relatedTarget;
                  ht && (ht === nt || 8 & ht.compareDocumentPosition(nt)) || Be.call(nt, Ue);
                };
              }
              se && vn.forEach(function(Ae) {
                "on" + Ae in se && vn.remove(Ae);
              });
              var jt, cn = 0;
              function wn(Ae) {
                var Oe = ".dragsuppress-" + ++cn, Be = "click" + Oe, Ue = ee.select(le(Ae)).on("touchmove" + Oe, ct).on("dragstart" + Oe, ct).on("selectstart" + Oe, ct);
                if (jt == null && (jt = !("onselectstart" in Ae) && Ze(Ae.style, "userSelect")), jt) {
                  var nt = ce(Ae).style, ht = nt[jt];
                  nt[jt] = "none";
                }
                return function(Et) {
                  if (Ue.on(Oe, null), jt && (nt[jt] = ht), Et) {
                    var Mt = function() {
                      Ue.on(Be, null);
                    };
                    Ue.on(Be, function() {
                      ct(), Mt();
                    }, !0), setTimeout(Mt, 0);
                  }
                };
              }
              ee.mouse = function(Ae) {
                return Pn(Ae, et());
              };
              var Rn = this.navigator && /WebKit/.test(this.navigator.userAgent) ? -1 : 0;
              function Pn(Ae, Oe) {
                Oe.changedTouches && (Oe = Oe.changedTouches[0]);
                var Be = Ae.ownerSVGElement || Ae;
                if (Be.createSVGPoint) {
                  var Ue = Be.createSVGPoint();
                  if (Rn < 0) {
                    var nt = le(Ae);
                    if (nt.scrollX || nt.scrollY) {
                      var ht = (Be = ee.select("body").append("svg").style({ position: "absolute", top: 0, left: 0, margin: 0, padding: 0, border: "none" }, "important"))[0][0].getScreenCTM();
                      Rn = !(ht.f || ht.e), Be.remove();
                    }
                  }
                  return Rn ? (Ue.x = Oe.pageX, Ue.y = Oe.pageY) : (Ue.x = Oe.clientX, Ue.y = Oe.clientY), [(Ue = Ue.matrixTransform(Ae.getScreenCTM().inverse())).x, Ue.y];
                }
                var Et = Ae.getBoundingClientRect();
                return [Oe.clientX - Et.left - Ae.clientLeft, Oe.clientY - Et.top - Ae.clientTop];
              }
              function Ie() {
                return ee.event.changedTouches[0].identifier;
              }
              ee.touch = function(Ae, Oe, Be) {
                if (arguments.length < 3 && (Be = Oe, Oe = et().changedTouches), Oe) {
                  for (var Ue, nt = 0, ht = Oe.length; nt < ht; ++nt) if ((Ue = Oe[nt]).identifier === Be) return Pn(Ae, Ue);
                }
              }, ee.behavior.drag = function() {
                var Ae = dt(nt, "drag", "dragstart", "dragend"), Oe = null, Be = ht(ze, ee.mouse, le, "mousemove", "mouseup"), Ue = ht(Ie, ee.touch, Xe, "touchmove", "touchend");
                function nt() {
                  this.on("mousedown.drag", Be).on("touchstart.drag", Ue);
                }
                function ht(Et, Mt, qt, tn, xn) {
                  return function() {
                    var An, On = this, Bn = ee.event.target.correspondingElement || ee.event.target, qn = On.parentNode, Vn = Ae.of(On, arguments), sr = 0, Un = Et(), wr = ".drag" + (Un == null ? "" : "-" + Un), Cr = ee.select(qt(Bn)).on(tn + wr, function() {
                      var wi, Wi, Yi = Mt(qn, Un);
                      Yi && (wi = Yi[0] - ii[0], Wi = Yi[1] - ii[1], sr |= wi | Wi, ii = Yi, Vn({ type: "drag", x: Yi[0] + An[0], y: Yi[1] + An[1], dx: wi, dy: Wi }));
                    }).on(xn + wr, function() {
                      Mt(qn, Un) && (Cr.on(tn + wr, null).on(xn + wr, null), Tr(sr), Vn({ type: "dragend" }));
                    }), Tr = wn(Bn), ii = Mt(qn, Un);
                    An = Oe ? [(An = Oe.apply(On, arguments)).x - ii[0], An.y - ii[1]] : [0, 0], Vn({ type: "dragstart" });
                  };
                }
                return nt.origin = function(Et) {
                  return arguments.length ? (Oe = Et, nt) : Oe;
                }, ee.rebind(nt, Ae, "on");
              }, ee.touches = function(Ae, Oe) {
                return arguments.length < 2 && (Oe = et().touches), Oe ? oe(Oe).map(function(Be) {
                  var Ue = Pn(Ae, Be);
                  return Ue.identifier = Be.identifier, Ue;
                }) : [];
              };
              var rt = 1e-6, ft = rt * rt, pt = Math.PI, wt = 2 * pt, Bt = wt - rt, At = pt / 2, Dt = pt / 180, Ot = 180 / pt;
              function un(Ae) {
                return Ae > 1 ? At : Ae < -1 ? -At : Math.asin(Ae);
              }
              function yn(Ae) {
                return ((Ae = Math.exp(Ae)) + 1 / Ae) / 2;
              }
              var _n = Math.SQRT2;
              ee.interpolateZoom = function(Ae, Oe) {
                var Be, Ue, nt = Ae[0], ht = Ae[1], Et = Ae[2], Mt = Oe[0], qt = Oe[1], tn = Oe[2], xn = Mt - nt, An = qt - ht, On = xn * xn + An * An;
                if (On < ft) Ue = Math.log(tn / Et) / _n, Be = function(wr) {
                  return [nt + wr * xn, ht + wr * An, Et * Math.exp(_n * wr * Ue)];
                };
                else {
                  var Bn = Math.sqrt(On), qn = (tn * tn - Et * Et + 4 * On) / (2 * Et * 2 * Bn), Vn = (tn * tn - Et * Et - 4 * On) / (2 * tn * 2 * Bn), sr = Math.log(Math.sqrt(qn * qn + 1) - qn), Un = Math.log(Math.sqrt(Vn * Vn + 1) - Vn);
                  Ue = (Un - sr) / _n, Be = function(wr) {
                    var Cr, Tr = wr * Ue, ii = yn(sr), wi = Et / (2 * Bn) * (ii * (Cr = _n * Tr + sr, ((Cr = Math.exp(2 * Cr)) - 1) / (Cr + 1)) - function(Wi) {
                      return ((Wi = Math.exp(Wi)) - 1 / Wi) / 2;
                    }(sr));
                    return [nt + wi * xn, ht + wi * An, Et * ii / yn(_n * Tr + sr)];
                  };
                }
                return Be.duration = 1e3 * Ue, Be;
              }, ee.behavior.zoom = function() {
                var Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn = { x: 0, y: 0, k: 1 }, xn = [960, 500], An = an, On = 250, Bn = 0, qn = "mousedown.zoom", Vn = "mousemove.zoom", sr = "mouseup.zoom", Un = "touchstart.zoom", wr = dt(Cr, "zoomstart", "zoom", "zoomend");
                function Cr(Oa) {
                  Oa.on(qn, va).on(bn + ".zoom", Aa).on("dblclick.zoom", Xa).on(Un, qa);
                }
                function Tr(Oa) {
                  return [(Oa[0] - tn.x) / tn.k, (Oa[1] - tn.y) / tn.k];
                }
                function ii(Oa) {
                  tn.k = Math.max(An[0], Math.min(An[1], Oa));
                }
                function wi(Oa, Ya) {
                  Ya = function(Wo) {
                    return [Wo[0] * tn.k + tn.x, Wo[1] * tn.k + tn.y];
                  }(Ya), tn.x += Oa[0] - Ya[0], tn.y += Oa[1] - Ya[1];
                }
                function Wi(Oa, Ya, Wo, Ja) {
                  Oa.__chart__ = { x: tn.x, y: tn.y, k: tn.k }, ii(Math.pow(2, Ja)), wi(Oe = Ya, Wo), Oa = ee.select(Oa), On > 0 && (Oa = Oa.transition().duration(On)), Oa.call(Cr.event);
                }
                function Yi() {
                  Et && Et.domain(ht.range().map(function(Oa) {
                    return (Oa - tn.x) / tn.k;
                  }).map(ht.invert)), qt && qt.domain(Mt.range().map(function(Oa) {
                    return (Oa - tn.y) / tn.k;
                  }).map(Mt.invert));
                }
                function ra(Oa) {
                  Bn++ || Oa({ type: "zoomstart" });
                }
                function ha(Oa) {
                  Yi(), Oa({ type: "zoom", scale: tn.k, translate: [tn.x, tn.y] });
                }
                function ya(Oa) {
                  --Bn || (Oa({ type: "zoomend" }), Oe = null);
                }
                function va() {
                  var Oa = this, Ya = wr.of(Oa, arguments), Wo = 0, Ja = ee.select(le(Oa)).on(Vn, function() {
                    Wo = 1, wi(ee.mouse(Oa), Uo), ha(Ya);
                  }).on(sr, function() {
                    Ja.on(Vn, null).on(sr, null), ys(Wo), ya(Ya);
                  }), Uo = Tr(ee.mouse(Oa)), ys = wn(Oa);
                  a1.call(Oa), ra(Ya);
                }
                function qa() {
                  var Oa, Ya = this, Wo = wr.of(Ya, arguments), Ja = {}, Uo = 0, ys = ".zoom-" + ee.event.changedTouches[0].identifier, il = "touchmove" + ys, Is = "touchend" + ys, ml = [], nl = ee.select(Ya), Il = wn(Ya);
                  function Fl() {
                    var Dl = ee.touches(Ya);
                    return Oa = tn.k, Dl.forEach(function(Ml) {
                      Ml.identifier in Ja && (Ja[Ml.identifier] = Tr(Ml));
                    }), Dl;
                  }
                  function qc() {
                    var Dl = ee.event.target;
                    ee.select(Dl).on(il, ql).on(Is, jc), ml.push(Dl);
                    for (var Ml = ee.event.changedTouches, pc = 0, hl = Ml.length; pc < hl; ++pc) Ja[Ml[pc].identifier] = null;
                    var xl = Fl(), Bl = Date.now();
                    if (xl.length === 1) {
                      if (Bl - nt < 500) {
                        var al = xl[0];
                        Wi(Ya, al, Ja[al.identifier], Math.floor(Math.log(tn.k) / Math.LN2) + 1), ct();
                      }
                      nt = Bl;
                    } else if (xl.length > 1) {
                      al = xl[0];
                      var Oc = xl[1], nu = al[0] - Oc[0], Ic = al[1] - Oc[1];
                      Uo = nu * nu + Ic * Ic;
                    }
                  }
                  function ql() {
                    var Dl, Ml, pc, hl, xl = ee.touches(Ya);
                    a1.call(Ya);
                    for (var Bl = 0, al = xl.length; Bl < al; ++Bl, hl = null) if (pc = xl[Bl], hl = Ja[pc.identifier]) {
                      if (Ml) break;
                      Dl = pc, Ml = hl;
                    }
                    if (hl) {
                      var Oc = (Oc = pc[0] - Dl[0]) * Oc + (Oc = pc[1] - Dl[1]) * Oc, nu = Uo && Math.sqrt(Oc / Uo);
                      Dl = [(Dl[0] + pc[0]) / 2, (Dl[1] + pc[1]) / 2], Ml = [(Ml[0] + hl[0]) / 2, (Ml[1] + hl[1]) / 2], ii(nu * Oa);
                    }
                    nt = null, wi(Dl, Ml), ha(Wo);
                  }
                  function jc() {
                    if (ee.event.touches.length) {
                      for (var Dl = ee.event.changedTouches, Ml = 0, pc = Dl.length; Ml < pc; ++Ml) delete Ja[Dl[Ml].identifier];
                      for (var hl in Ja) return void Fl();
                    }
                    ee.selectAll(ml).on(ys, null), nl.on(qn, va).on(Un, qa), Il(), ya(Wo);
                  }
                  qc(), ra(Wo), nl.on(qn, null).on(Un, qc);
                }
                function Aa() {
                  var Oa = wr.of(this, arguments);
                  Ue ? clearTimeout(Ue) : (a1.call(this), Ae = Tr(Oe = Be || ee.mouse(this)), ra(Oa)), Ue = setTimeout(function() {
                    Ue = null, ya(Oa);
                  }, 50), ct(), ii(Math.pow(2, 2e-3 * gn()) * tn.k), wi(Oe, Ae), ha(Oa);
                }
                function Xa() {
                  var Oa = ee.mouse(this), Ya = Math.log(tn.k) / Math.LN2;
                  Wi(this, Oa, Tr(Oa), ee.event.shiftKey ? Math.ceil(Ya) - 1 : Math.floor(Ya) + 1);
                }
                return bn || (bn = "onwheel" in se ? (gn = function() {
                  return -ee.event.deltaY * (ee.event.deltaMode ? 120 : 1);
                }, "wheel") : "onmousewheel" in se ? (gn = function() {
                  return ee.event.wheelDelta;
                }, "mousewheel") : (gn = function() {
                  return -ee.event.detail;
                }, "MozMousePixelScroll")), Cr.event = function(Oa) {
                  Oa.each(function() {
                    var Ya = wr.of(this, arguments), Wo = tn;
                    $c ? ee.select(this).transition().each("start.zoom", function() {
                      tn = this.__chart__ || { x: 0, y: 0, k: 1 }, ra(Ya);
                    }).tween("zoom:zoom", function() {
                      var Ja = xn[0], Uo = xn[1], ys = Oe ? Oe[0] : Ja / 2, il = Oe ? Oe[1] : Uo / 2, Is = ee.interpolateZoom([(ys - tn.x) / tn.k, (il - tn.y) / tn.k, Ja / tn.k], [(ys - Wo.x) / Wo.k, (il - Wo.y) / Wo.k, Ja / Wo.k]);
                      return function(ml) {
                        var nl = Is(ml), Il = Ja / nl[2];
                        this.__chart__ = tn = { x: ys - nl[0] * Il, y: il - nl[1] * Il, k: Il }, ha(Ya);
                      };
                    }).each("interrupt.zoom", function() {
                      ya(Ya);
                    }).each("end.zoom", function() {
                      ya(Ya);
                    }) : (this.__chart__ = tn, ra(Ya), ha(Ya), ya(Ya));
                  });
                }, Cr.translate = function(Oa) {
                  return arguments.length ? (tn = { x: +Oa[0], y: +Oa[1], k: tn.k }, Yi(), Cr) : [tn.x, tn.y];
                }, Cr.scale = function(Oa) {
                  return arguments.length ? (tn = { x: tn.x, y: tn.y, k: null }, ii(+Oa), Yi(), Cr) : tn.k;
                }, Cr.scaleExtent = function(Oa) {
                  return arguments.length ? (An = Oa == null ? an : [+Oa[0], +Oa[1]], Cr) : An;
                }, Cr.center = function(Oa) {
                  return arguments.length ? (Be = Oa && [+Oa[0], +Oa[1]], Cr) : Be;
                }, Cr.size = function(Oa) {
                  return arguments.length ? (xn = Oa && [+Oa[0], +Oa[1]], Cr) : xn;
                }, Cr.duration = function(Oa) {
                  return arguments.length ? (On = +Oa, Cr) : On;
                }, Cr.x = function(Oa) {
                  return arguments.length ? (Et = Oa, ht = Oa.copy(), tn = { x: 0, y: 0, k: 1 }, Cr) : Et;
                }, Cr.y = function(Oa) {
                  return arguments.length ? (qt = Oa, Mt = Oa.copy(), tn = { x: 0, y: 0, k: 1 }, Cr) : qt;
                }, ee.rebind(Cr, wr, "on");
              };
              var gn, bn, an = [0, 1 / 0];
              function Zt() {
              }
              function Mn(Ae, Oe, Be) {
                return this instanceof Mn ? (this.h = +Ae, this.s = +Oe, void (this.l = +Be)) : arguments.length < 2 ? Ae instanceof Mn ? new Mn(Ae.h, Ae.s, Ae.l) : bi("" + Ae, Zi, Mn) : new Mn(Ae, Oe, Be);
              }
              ee.color = Zt, Zt.prototype.toString = function() {
                return this.rgb() + "";
              }, ee.hsl = Mn;
              var Ln = Mn.prototype = new Zt();
              function In(Ae, Oe, Be) {
                var Ue, nt;
                function ht(Et) {
                  return Math.round(255 * function(Mt) {
                    return Mt > 360 ? Mt -= 360 : Mt < 0 && (Mt += 360), Mt < 60 ? Ue + (nt - Ue) * Mt / 60 : Mt < 180 ? nt : Mt < 240 ? Ue + (nt - Ue) * (240 - Mt) / 60 : Ue;
                  }(Et));
                }
                return Ae = isNaN(Ae) ? 0 : (Ae %= 360) < 0 ? Ae + 360 : Ae, Oe = isNaN(Oe) || Oe < 0 ? 0 : Oe > 1 ? 1 : Oe, Ue = 2 * (Be = Be < 0 ? 0 : Be > 1 ? 1 : Be) - (nt = Be <= 0.5 ? Be * (1 + Oe) : Be + Oe - Be * Oe), new Bi(ht(Ae + 120), ht(Ae), ht(Ae - 120));
              }
              function kn(Ae, Oe, Be) {
                return this instanceof kn ? (this.h = +Ae, this.c = +Oe, void (this.l = +Be)) : arguments.length < 2 ? Ae instanceof kn ? new kn(Ae.h, Ae.c, Ae.l) : function(Ue, nt, ht) {
                  return Ue > 0 ? new kn(Math.atan2(ht, nt) * Ot, Math.sqrt(nt * nt + ht * ht), Ue) : new kn(NaN, NaN, Ue);
                }(Ae instanceof ur ? Ae.l : (Ae = $i((Ae = ee.rgb(Ae)).r, Ae.g, Ae.b)).l, Ae.a, Ae.b) : new kn(Ae, Oe, Be);
              }
              Ln.brighter = function(Ae) {
                return Ae = Math.pow(0.7, arguments.length ? Ae : 1), new Mn(this.h, this.s, this.l / Ae);
              }, Ln.darker = function(Ae) {
                return Ae = Math.pow(0.7, arguments.length ? Ae : 1), new Mn(this.h, this.s, Ae * this.l);
              }, Ln.rgb = function() {
                return In(this.h, this.s, this.l);
              }, ee.hcl = kn;
              var er = kn.prototype = new Zt();
              function jn(Ae, Oe, Be) {
                return isNaN(Ae) && (Ae = 0), isNaN(Oe) && (Oe = 0), new ur(Be, Math.cos(Ae *= Dt) * Oe, Math.sin(Ae) * Oe);
              }
              function ur(Ae, Oe, Be) {
                return this instanceof ur ? (this.l = +Ae, this.a = +Oe, void (this.b = +Be)) : arguments.length < 2 ? Ae instanceof ur ? new ur(Ae.l, Ae.a, Ae.b) : Ae instanceof kn ? jn(Ae.h, Ae.c, Ae.l) : $i((Ae = Bi(Ae)).r, Ae.g, Ae.b) : new ur(Ae, Oe, Be);
              }
              er.brighter = function(Ae) {
                return new kn(this.h, this.c, Math.min(100, this.l + Kn * (arguments.length ? Ae : 1)));
              }, er.darker = function(Ae) {
                return new kn(this.h, this.c, Math.max(0, this.l - Kn * (arguments.length ? Ae : 1)));
              }, er.rgb = function() {
                return jn(this.h, this.c, this.l).rgb();
              }, ee.lab = ur;
              var Kn = 18, Jn = 0.95047, Lr = 1, mr = 1.08883, Ar = ur.prototype = new Zt();
              function Yr(Ae, Oe, Be) {
                var Ue = (Ae + 16) / 116, nt = Ue + Oe / 500, ht = Ue - Be / 200;
                return new Bi(Wr(3.2404542 * (nt = jr(nt) * Jn) - 1.5371385 * (Ue = jr(Ue) * Lr) - 0.4985314 * (ht = jr(ht) * mr)), Wr(-0.969266 * nt + 1.8760108 * Ue + 0.041556 * ht), Wr(0.0556434 * nt - 0.2040259 * Ue + 1.0572252 * ht));
              }
              function jr(Ae) {
                return Ae > 0.206893034 ? Ae * Ae * Ae : (Ae - 0.13793103448275862) / 7.787037;
              }
              function Mr(Ae) {
                return Ae > 8856e-6 ? Math.pow(Ae, 0.3333333333333333) : 7.787037 * Ae + 0.13793103448275862;
              }
              function Wr(Ae) {
                return Math.round(255 * (Ae <= 304e-5 ? 12.92 * Ae : 1.055 * Math.pow(Ae, 0.4166666666666667) - 0.055));
              }
              function Bi(Ae, Oe, Be) {
                return this instanceof Bi ? (this.r = ~~Ae, this.g = ~~Oe, void (this.b = ~~Be)) : arguments.length < 2 ? Ae instanceof Bi ? new Bi(Ae.r, Ae.g, Ae.b) : bi("" + Ae, Bi, In) : new Bi(Ae, Oe, Be);
              }
              function gi(Ae) {
                return new Bi(Ae >> 16, Ae >> 8 & 255, 255 & Ae);
              }
              function Zr(Ae) {
                return gi(Ae) + "";
              }
              Ar.brighter = function(Ae) {
                return new ur(Math.min(100, this.l + Kn * (arguments.length ? Ae : 1)), this.a, this.b);
              }, Ar.darker = function(Ae) {
                return new ur(Math.max(0, this.l - Kn * (arguments.length ? Ae : 1)), this.a, this.b);
              }, Ar.rgb = function() {
                return Yr(this.l, this.a, this.b);
              }, ee.rgb = Bi;
              var fi = Bi.prototype = new Zt();
              function ri(Ae) {
                return Ae < 16 ? "0" + Math.max(0, Ae).toString(16) : Math.min(255, Ae).toString(16);
              }
              function bi(Ae, Oe, Be) {
                var Ue, nt, ht, Et = 0, Mt = 0, qt = 0;
                if (Ue = /([a-z]+)\((.*)\)/.exec(Ae = Ae.toLowerCase())) switch (nt = Ue[2].split(","), Ue[1]) {
                  case "hsl":
                    return Be(parseFloat(nt[0]), parseFloat(nt[1]) / 100, parseFloat(nt[2]) / 100);
                  case "rgb":
                    return Oe(Pr(nt[0]), Pr(nt[1]), Pr(nt[2]));
                }
                return (ht = ei.get(Ae)) ? Oe(ht.r, ht.g, ht.b) : (Ae == null || Ae.charAt(0) !== "#" || isNaN(ht = parseInt(Ae.slice(1), 16)) || (Ae.length === 4 ? (Et = (3840 & ht) >> 4, Et |= Et >> 4, Mt = 240 & ht, Mt |= Mt >> 4, qt = 15 & ht, qt |= qt << 4) : Ae.length === 7 && (Et = (16711680 & ht) >> 16, Mt = (65280 & ht) >> 8, qt = 255 & ht)), Oe(Et, Mt, qt));
              }
              function Zi(Ae, Oe, Be) {
                var Ue, nt, ht = Math.min(Ae /= 255, Oe /= 255, Be /= 255), Et = Math.max(Ae, Oe, Be), Mt = Et - ht, qt = (Et + ht) / 2;
                return Mt ? (nt = qt < 0.5 ? Mt / (Et + ht) : Mt / (2 - Et - ht), Ue = Ae == Et ? (Oe - Be) / Mt + (Oe < Be ? 6 : 0) : Oe == Et ? (Be - Ae) / Mt + 2 : (Ae - Oe) / Mt + 4, Ue *= 60) : (Ue = NaN, nt = qt > 0 && qt < 1 ? 0 : Ue), new Mn(Ue, nt, qt);
              }
              function $i(Ae, Oe, Be) {
                var Ue = Mr((0.4124564 * (Ae = Br(Ae)) + 0.3575761 * (Oe = Br(Oe)) + 0.1804375 * (Be = Br(Be))) / Jn), nt = Mr((0.2126729 * Ae + 0.7151522 * Oe + 0.072175 * Be) / Lr);
                return ur(116 * nt - 16, 500 * (Ue - nt), 200 * (nt - Mr((0.0193339 * Ae + 0.119192 * Oe + 0.9503041 * Be) / mr)));
              }
              function Br(Ae) {
                return (Ae /= 255) <= 0.04045 ? Ae / 12.92 : Math.pow((Ae + 0.055) / 1.055, 2.4);
              }
              function Pr(Ae) {
                var Oe = parseFloat(Ae);
                return Ae.charAt(Ae.length - 1) === "%" ? Math.round(2.55 * Oe) : Oe;
              }
              fi.brighter = function(Ae) {
                Ae = Math.pow(0.7, arguments.length ? Ae : 1);
                var Oe = this.r, Be = this.g, Ue = this.b, nt = 30;
                return Oe || Be || Ue ? (Oe && Oe < nt && (Oe = nt), Be && Be < nt && (Be = nt), Ue && Ue < nt && (Ue = nt), new Bi(Math.min(255, Oe / Ae), Math.min(255, Be / Ae), Math.min(255, Ue / Ae))) : new Bi(nt, nt, nt);
              }, fi.darker = function(Ae) {
                return new Bi((Ae = Math.pow(0.7, arguments.length ? Ae : 1)) * this.r, Ae * this.g, Ae * this.b);
              }, fi.hsl = function() {
                return Zi(this.r, this.g, this.b);
              }, fi.toString = function() {
                return "#" + ri(this.r) + ri(this.g) + ri(this.b);
              };
              var ei = ee.map({ aliceblue: 15792383, antiquewhite: 16444375, aqua: 65535, aquamarine: 8388564, azure: 15794175, beige: 16119260, bisque: 16770244, black: 0, blanchedalmond: 16772045, blue: 255, blueviolet: 9055202, brown: 10824234, burlywood: 14596231, cadetblue: 6266528, chartreuse: 8388352, chocolate: 13789470, coral: 16744272, cornflowerblue: 6591981, cornsilk: 16775388, crimson: 14423100, cyan: 65535, darkblue: 139, darkcyan: 35723, darkgoldenrod: 12092939, darkgray: 11119017, darkgreen: 25600, darkgrey: 11119017, darkkhaki: 12433259, darkmagenta: 9109643, darkolivegreen: 5597999, darkorange: 16747520, darkorchid: 10040012, darkred: 9109504, darksalmon: 15308410, darkseagreen: 9419919, darkslateblue: 4734347, darkslategray: 3100495, darkslategrey: 3100495, darkturquoise: 52945, darkviolet: 9699539, deeppink: 16716947, deepskyblue: 49151, dimgray: 6908265, dimgrey: 6908265, dodgerblue: 2003199, firebrick: 11674146, floralwhite: 16775920, forestgreen: 2263842, fuchsia: 16711935, gainsboro: 14474460, ghostwhite: 16316671, gold: 16766720, goldenrod: 14329120, gray: 8421504, green: 32768, greenyellow: 11403055, grey: 8421504, honeydew: 15794160, hotpink: 16738740, indianred: 13458524, indigo: 4915330, ivory: 16777200, khaki: 15787660, lavender: 15132410, lavenderblush: 16773365, lawngreen: 8190976, lemonchiffon: 16775885, lightblue: 11393254, lightcoral: 15761536, lightcyan: 14745599, lightgoldenrodyellow: 16448210, lightgray: 13882323, lightgreen: 9498256, lightgrey: 13882323, lightpink: 16758465, lightsalmon: 16752762, lightseagreen: 2142890, lightskyblue: 8900346, lightslategray: 7833753, lightslategrey: 7833753, lightsteelblue: 11584734, lightyellow: 16777184, lime: 65280, limegreen: 3329330, linen: 16445670, magenta: 16711935, maroon: 8388608, mediumaquamarine: 6737322, mediumblue: 205, mediumorchid: 12211667, mediumpurple: 9662683, mediumseagreen: 3978097, mediumslateblue: 8087790, mediumspringgreen: 64154, mediumturquoise: 4772300, mediumvioletred: 13047173, midnightblue: 1644912, mintcream: 16121850, mistyrose: 16770273, moccasin: 16770229, navajowhite: 16768685, navy: 128, oldlace: 16643558, olive: 8421376, olivedrab: 7048739, orange: 16753920, orangered: 16729344, orchid: 14315734, palegoldenrod: 15657130, palegreen: 10025880, paleturquoise: 11529966, palevioletred: 14381203, papayawhip: 16773077, peachpuff: 16767673, peru: 13468991, pink: 16761035, plum: 14524637, powderblue: 11591910, purple: 8388736, rebeccapurple: 6697881, red: 16711680, rosybrown: 12357519, royalblue: 4286945, saddlebrown: 9127187, salmon: 16416882, sandybrown: 16032864, seagreen: 3050327, seashell: 16774638, sienna: 10506797, silver: 12632256, skyblue: 8900331, slateblue: 6970061, slategray: 7372944, slategrey: 7372944, snow: 16775930, springgreen: 65407, steelblue: 4620980, tan: 13808780, teal: 32896, thistle: 14204888, tomato: 16737095, turquoise: 4251856, violet: 15631086, wheat: 16113331, white: 16777215, whitesmoke: 16119285, yellow: 16776960, yellowgreen: 10145074 });
              function Ci(Ae) {
                return typeof Ae == "function" ? Ae : function() {
                  return Ae;
                };
              }
              function oa(Ae) {
                return function(Oe, Be, Ue) {
                  return arguments.length === 2 && typeof Be == "function" && (Ue = Be, Be = null), ka(Oe, Be, Ae, Ue);
                };
              }
              function ka(Ae, Oe, Be, Ue) {
                var nt = {}, ht = ee.dispatch("beforesend", "progress", "load", "error"), Et = {}, Mt = new XMLHttpRequest(), qt = null;
                function tn() {
                  var xn, An = Mt.status;
                  if (!An && function(On) {
                    var Bn = On.responseType;
                    return Bn && Bn !== "text" ? On.response : On.responseText;
                  }(Mt) || An >= 200 && An < 300 || An === 304) {
                    try {
                      xn = Be.call(nt, Mt);
                    } catch (On) {
                      return void ht.error.call(nt, On);
                    }
                    ht.load.call(nt, xn);
                  } else ht.error.call(nt, Mt);
                }
                return self.XDomainRequest && !("withCredentials" in Mt) && /^(http(s)?:)?\/\//.test(Ae) && (Mt = new XDomainRequest()), "onload" in Mt ? Mt.onload = Mt.onerror = tn : Mt.onreadystatechange = function() {
                  Mt.readyState > 3 && tn();
                }, Mt.onprogress = function(xn) {
                  var An = ee.event;
                  ee.event = xn;
                  try {
                    ht.progress.call(nt, Mt);
                  } finally {
                    ee.event = An;
                  }
                }, nt.header = function(xn, An) {
                  return xn = (xn + "").toLowerCase(), arguments.length < 2 ? Et[xn] : (An == null ? delete Et[xn] : Et[xn] = An + "", nt);
                }, nt.mimeType = function(xn) {
                  return arguments.length ? (Oe = xn == null ? null : xn + "", nt) : Oe;
                }, nt.responseType = function(xn) {
                  return arguments.length ? (qt = xn, nt) : qt;
                }, nt.response = function(xn) {
                  return Be = xn, nt;
                }, ["get", "post"].forEach(function(xn) {
                  nt[xn] = function() {
                    return nt.send.apply(nt, [xn].concat(oe(arguments)));
                  };
                }), nt.send = function(xn, An, On) {
                  if (arguments.length === 2 && typeof An == "function" && (On = An, An = null), Mt.open(xn, Ae, !0), Oe == null || "accept" in Et || (Et.accept = Oe + ",*/*"), Mt.setRequestHeader) for (var Bn in Et) Mt.setRequestHeader(Bn, Et[Bn]);
                  return Oe != null && Mt.overrideMimeType && Mt.overrideMimeType(Oe), qt != null && (Mt.responseType = qt), On != null && nt.on("error", On).on("load", function(qn) {
                    On(null, qn);
                  }), ht.beforesend.call(nt, Mt), Mt.send(An ?? null), nt;
                }, nt.abort = function() {
                  return Mt.abort(), nt;
                }, ee.rebind(nt, ht, "on"), Ue == null ? nt : nt.get(function(xn) {
                  return xn.length === 1 ? function(An, On) {
                    xn(An == null ? On : null);
                  } : xn;
                }(Ue));
              }
              ei.forEach(function(Ae, Oe) {
                ei.set(Ae, gi(Oe));
              }), ee.functor = Ci, ee.xhr = oa(Xe), ee.dsv = function(Ae, Oe) {
                var Be = new RegExp('["' + Ae + `
]`), Ue = Ae.charCodeAt(0);
                function nt(tn, xn, An) {
                  arguments.length < 3 && (An = xn, xn = null);
                  var On = ka(tn, Oe, xn == null ? ht : Et(xn), An);
                  return On.row = function(Bn) {
                    return arguments.length ? On.response((xn = Bn) == null ? ht : Et(Bn)) : xn;
                  }, On;
                }
                function ht(tn) {
                  return nt.parse(tn.responseText);
                }
                function Et(tn) {
                  return function(xn) {
                    return nt.parse(xn.responseText, tn);
                  };
                }
                function Mt(tn) {
                  return tn.map(qt).join(Ae);
                }
                function qt(tn) {
                  return Be.test(tn) ? '"' + tn.replace(/\"/g, '""') + '"' : tn;
                }
                return nt.parse = function(tn, xn) {
                  var An;
                  return nt.parseRows(tn, function(On, Bn) {
                    if (An) return An(On, Bn - 1);
                    var qn = function(Vn) {
                      for (var sr = {}, Un = On.length, wr = 0; wr < Un; ++wr) sr[On[wr]] = Vn[wr];
                      return sr;
                    };
                    An = xn ? function(Vn, sr) {
                      return xn(qn(Vn), sr);
                    } : qn;
                  });
                }, nt.parseRows = function(tn, xn) {
                  var An, On, Bn = {}, qn = {}, Vn = [], sr = tn.length, Un = 0, wr = 0;
                  function Cr() {
                    if (Un >= sr) return qn;
                    if (On) return On = !1, Bn;
                    var ii = Un;
                    if (tn.charCodeAt(ii) === 34) {
                      for (var wi = ii; wi++ < sr; ) if (tn.charCodeAt(wi) === 34) {
                        if (tn.charCodeAt(wi + 1) !== 34) break;
                        ++wi;
                      }
                      return Un = wi + 2, (Wi = tn.charCodeAt(wi + 1)) === 13 ? (On = !0, tn.charCodeAt(wi + 2) === 10 && ++Un) : Wi === 10 && (On = !0), tn.slice(ii + 1, wi).replace(/""/g, '"');
                    }
                    for (; Un < sr; ) {
                      var Wi, Yi = 1;
                      if ((Wi = tn.charCodeAt(Un++)) === 10) On = !0;
                      else if (Wi === 13) On = !0, tn.charCodeAt(Un) === 10 && (++Un, ++Yi);
                      else if (Wi !== Ue) continue;
                      return tn.slice(ii, Un - Yi);
                    }
                    return tn.slice(ii);
                  }
                  for (; (An = Cr()) !== qn; ) {
                    for (var Tr = []; An !== Bn && An !== qn; ) Tr.push(An), An = Cr();
                    xn && (Tr = xn(Tr, wr++)) == null || Vn.push(Tr);
                  }
                  return Vn;
                }, nt.format = function(tn) {
                  if (Array.isArray(tn[0])) return nt.formatRows(tn);
                  var xn = new qe(), An = [];
                  return tn.forEach(function(On) {
                    for (var Bn in On) xn.has(Bn) || An.push(xn.add(Bn));
                  }), [An.map(qt).join(Ae)].concat(tn.map(function(On) {
                    return An.map(function(Bn) {
                      return qt(On[Bn]);
                    }).join(Ae);
                  })).join(`
`);
                }, nt.formatRows = function(tn) {
                  return tn.map(Mt).join(`
`);
                }, nt;
              }, ee.csv = ee.dsv(",", "text/csv"), ee.tsv = ee.dsv("	", "text/tab-separated-values");
              var aa, hi, Gr, Ca, Sr = this[Ze(this, "requestAnimationFrame")] || function(Ae) {
                setTimeout(Ae, 17);
              };
              function Dr(Ae, Oe, Be) {
                var Ue = arguments.length;
                Ue < 2 && (Oe = 0), Ue < 3 && (Be = Date.now());
                var nt = { c: Ae, t: Be + Oe, n: null };
                return hi ? hi.n = nt : aa = nt, hi = nt, Gr || (Ca = clearTimeout(Ca), Gr = 1, Sr(si)), nt;
              }
              function si() {
                var Ae = zr(), Oe = di() - Ae;
                Oe > 24 ? (isFinite(Oe) && (clearTimeout(Ca), Ca = setTimeout(si, Oe)), Gr = 0) : (Gr = 1, Sr(si));
              }
              function zr() {
                for (var Ae = Date.now(), Oe = aa; Oe; ) Ae >= Oe.t && Oe.c(Ae - Oe.t) && (Oe.c = null), Oe = Oe.n;
                return Ae;
              }
              function di() {
                for (var Ae, Oe = aa, Be = 1 / 0; Oe; ) Oe.c ? (Oe.t < Be && (Be = Oe.t), Oe = (Ae = Oe).n) : Oe = Ae ? Ae.n = Oe.n : aa = Oe.n;
                return hi = Ae, Be;
              }
              function Da(Ae) {
                return Ae[0];
              }
              function la(Ae) {
                return Ae[1];
              }
              function wa(Ae) {
                for (var Oe, Be, Ue, nt = Ae.length, ht = [0, 1], Et = 2, Mt = 2; Mt < nt; Mt++) {
                  for (; Et > 1 && (Oe = Ae[ht[Et - 2]], Be = Ae[ht[Et - 1]], Ue = Ae[Mt], (Be[0] - Oe[0]) * (Ue[1] - Oe[1]) - (Be[1] - Oe[1]) * (Ue[0] - Oe[0]) <= 0); ) --Et;
                  ht[Et++] = Mt;
                }
                return ht.slice(0, Et);
              }
              function Wa(Ae, Oe) {
                return Ae[0] - Oe[0] || Ae[1] - Oe[1];
              }
              ee.timer = function() {
                Dr.apply(this, arguments);
              }, ee.timer.flush = function() {
                zr(), di();
              }, ee.round = function(Ae, Oe) {
                return Oe ? Math.round(Ae * (Oe = Math.pow(10, Oe))) / Oe : Math.round(Ae);
              }, ee.geom = {}, ee.geom.hull = function(Ae) {
                var Oe = Da, Be = la;
                if (arguments.length) return Ue(Ae);
                function Ue(nt) {
                  if (nt.length < 3) return [];
                  var ht, Et = Ci(Oe), Mt = Ci(Be), qt = nt.length, tn = [], xn = [];
                  for (ht = 0; ht < qt; ht++) tn.push([+Et.call(this, nt[ht], ht), +Mt.call(this, nt[ht], ht), ht]);
                  for (tn.sort(Wa), ht = 0; ht < qt; ht++) xn.push([tn[ht][0], -tn[ht][1]]);
                  var An = wa(tn), On = wa(xn), Bn = On[0] === An[0], qn = On[On.length - 1] === An[An.length - 1], Vn = [];
                  for (ht = An.length - 1; ht >= 0; --ht) Vn.push(nt[tn[An[ht]][2]]);
                  for (ht = +Bn; ht < On.length - qn; ++ht) Vn.push(nt[tn[On[ht]][2]]);
                  return Vn;
                }
                return Ue.x = function(nt) {
                  return arguments.length ? (Oe = nt, Ue) : Oe;
                }, Ue.y = function(nt) {
                  return arguments.length ? (Be = nt, Ue) : Be;
                }, Ue;
              }, ee.geom.polygon = function(Ae) {
                return it(Ae, Qr), Ae;
              };
              var Qr = ee.geom.polygon.prototype = [];
              function Ur(Ae, Oe, Be) {
                return (Be[0] - Oe[0]) * (Ae[1] - Oe[1]) < (Be[1] - Oe[1]) * (Ae[0] - Oe[0]);
              }
              function rr(Ae, Oe, Be, Ue) {
                var nt = Ae[0], ht = Be[0], Et = Oe[0] - nt, Mt = Ue[0] - ht, qt = Ae[1], tn = Be[1], xn = Oe[1] - qt, An = Ue[1] - tn, On = (Mt * (qt - tn) - An * (nt - ht)) / (An * Et - Mt * xn);
                return [nt + On * Et, qt + On * xn];
              }
              function vr(Ae) {
                var Oe = Ae[0], Be = Ae[Ae.length - 1];
                return !(Oe[0] - Be[0] || Oe[1] - Be[1]);
              }
              Qr.area = function() {
                for (var Ae, Oe = -1, Be = this.length, Ue = this[Be - 1], nt = 0; ++Oe < Be; ) Ae = Ue, Ue = this[Oe], nt += Ae[1] * Ue[0] - Ae[0] * Ue[1];
                return 0.5 * nt;
              }, Qr.centroid = function(Ae) {
                var Oe, Be, Ue = -1, nt = this.length, ht = 0, Et = 0, Mt = this[nt - 1];
                for (arguments.length || (Ae = -1 / (6 * this.area())); ++Ue < nt; ) Oe = Mt, Mt = this[Ue], Be = Oe[0] * Mt[1] - Mt[0] * Oe[1], ht += (Oe[0] + Mt[0]) * Be, Et += (Oe[1] + Mt[1]) * Be;
                return [ht * Ae, Et * Ae];
              }, Qr.clip = function(Ae) {
                for (var Oe, Be, Ue, nt, ht, Et, Mt = vr(Ae), qt = -1, tn = this.length - vr(this), xn = this[tn - 1]; ++qt < tn; ) {
                  for (Oe = Ae.slice(), Ae.length = 0, nt = this[qt], ht = Oe[(Ue = Oe.length - Mt) - 1], Be = -1; ++Be < Ue; ) Ur(Et = Oe[Be], xn, nt) ? (Ur(ht, xn, nt) || Ae.push(rr(ht, Et, xn, nt)), Ae.push(Et)) : Ur(ht, xn, nt) && Ae.push(rr(ht, Et, xn, nt)), ht = Et;
                  Mt && Ae.push(Ae[0]), xn = nt;
                }
                return Ae;
              };
              var Rr, Vr, Xr, Jr, ti, Hr = [], Er = [];
              function gr() {
                Dn(this), this.edge = this.site = this.circle = null;
              }
              function $r(Ae) {
                var Oe = Hr.pop() || new gr();
                return Oe.site = Ae, Oe;
              }
              function li(Ae) {
                Vi(Ae), Xr.remove(Ae), Hr.push(Ae), Dn(Ae);
              }
              function Qi(Ae) {
                var Oe = Ae.circle, Be = Oe.x, Ue = Oe.cy, nt = { x: Be, y: Ue }, ht = Ae.P, Et = Ae.N, Mt = [Ae];
                li(Ae);
                for (var qt = ht; qt.circle && Se(Be - qt.circle.x) < rt && Se(Ue - qt.circle.cy) < rt; ) ht = qt.P, Mt.unshift(qt), li(qt), qt = ht;
                Mt.unshift(qt), Vi(qt);
                for (var tn = Et; tn.circle && Se(Be - tn.circle.x) < rt && Se(Ue - tn.circle.cy) < rt; ) Et = tn.N, Mt.push(tn), li(tn), tn = Et;
                Mt.push(tn), Vi(tn);
                var xn, An = Mt.length;
                for (xn = 1; xn < An; ++xn) tn = Mt[xn], qt = Mt[xn - 1], na(tn.edge, qt.site, tn.site, nt);
                qt = Mt[0], (tn = Mt[An - 1]).edge = Fi(qt.site, tn.site, null, nt), Mi(qt), Mi(tn);
              }
              function Ai(Ae) {
                for (var Oe, Be, Ue, nt, ht = Ae.x, Et = Ae.y, Mt = Xr._; Mt; ) if ((Ue = Li(Mt, Et) - ht) > rt) Mt = Mt.L;
                else {
                  if (!((nt = ht - Ni(Mt, Et)) > rt)) {
                    Ue > -1e-6 ? (Oe = Mt.P, Be = Mt) : nt > -1e-6 ? (Oe = Mt, Be = Mt.N) : Oe = Be = Mt;
                    break;
                  }
                  if (!Mt.R) {
                    Oe = Mt;
                    break;
                  }
                  Mt = Mt.R;
                }
                var qt = $r(Ae);
                if (Xr.insert(Oe, qt), Oe || Be) {
                  if (Oe === Be) return Vi(Oe), Be = $r(Oe.site), Xr.insert(qt, Be), qt.edge = Be.edge = Fi(Oe.site, qt.site), Mi(Oe), void Mi(Be);
                  if (Be) {
                    Vi(Oe), Vi(Be);
                    var tn = Oe.site, xn = tn.x, An = tn.y, On = Ae.x - xn, Bn = Ae.y - An, qn = Be.site, Vn = qn.x - xn, sr = qn.y - An, Un = 2 * (On * sr - Bn * Vn), wr = On * On + Bn * Bn, Cr = Vn * Vn + sr * sr, Tr = { x: (sr * wr - Bn * Cr) / Un + xn, y: (On * Cr - Vn * wr) / Un + An };
                    na(Be.edge, tn, qn, Tr), qt.edge = Fi(tn, Ae, null, Tr), Be.edge = Fi(Ae, qn, null, Tr), Mi(Oe), Mi(Be);
                  } else qt.edge = Fi(Oe.site, qt.site);
                }
              }
              function Li(Ae, Oe) {
                var Be = Ae.site, Ue = Be.x, nt = Be.y, ht = nt - Oe;
                if (!ht) return Ue;
                var Et = Ae.P;
                if (!Et) return -1 / 0;
                var Mt = (Be = Et.site).x, qt = Be.y, tn = qt - Oe;
                if (!tn) return Mt;
                var xn = Mt - Ue, An = 1 / ht - 1 / tn, On = xn / tn;
                return An ? (-On + Math.sqrt(On * On - 2 * An * (xn * xn / (-2 * tn) - qt + tn / 2 + nt - ht / 2))) / An + Ue : (Ue + Mt) / 2;
              }
              function Ni(Ae, Oe) {
                var Be = Ae.N;
                if (Be) return Li(Be, Oe);
                var Ue = Ae.site;
                return Ue.y === Oe ? Ue.x : 1 / 0;
              }
              function Pi(Ae) {
                this.site = Ae, this.edges = [];
              }
              function Oi(Ae, Oe) {
                return Oe.angle - Ae.angle;
              }
              function ki() {
                Dn(this), this.x = this.y = this.arc = this.site = this.cy = null;
              }
              function Mi(Ae) {
                var Oe = Ae.P, Be = Ae.N;
                if (Oe && Be) {
                  var Ue = Oe.site, nt = Ae.site, ht = Be.site;
                  if (Ue !== ht) {
                    var Et = nt.x, Mt = nt.y, qt = Ue.x - Et, tn = Ue.y - Mt, xn = ht.x - Et, An = 2 * (qt * (sr = ht.y - Mt) - tn * xn);
                    if (!(An >= -1e-12)) {
                      var On = qt * qt + tn * tn, Bn = xn * xn + sr * sr, qn = (sr * On - tn * Bn) / An, Vn = (qt * Bn - xn * On) / An, sr = Vn + Mt, Un = Er.pop() || new ki();
                      Un.arc = Ae, Un.site = nt, Un.x = qn + Et, Un.y = sr + Math.sqrt(qn * qn + Vn * Vn), Un.cy = sr, Ae.circle = Un;
                      for (var wr = null, Cr = ti._; Cr; ) if (Un.y < Cr.y || Un.y === Cr.y && Un.x <= Cr.x) {
                        if (!Cr.L) {
                          wr = Cr.P;
                          break;
                        }
                        Cr = Cr.L;
                      } else {
                        if (!Cr.R) {
                          wr = Cr;
                          break;
                        }
                        Cr = Cr.R;
                      }
                      ti.insert(wr, Un), wr || (Jr = Un);
                    }
                  }
                }
              }
              function Vi(Ae) {
                var Oe = Ae.circle;
                Oe && (Oe.P || (Jr = Oe.N), ti.remove(Oe), Er.push(Oe), Dn(Oe), Ae.circle = null);
              }
              function ua(Ae, Oe) {
                var Be = Ae.b;
                if (Be) return !0;
                var Ue, nt, ht = Ae.a, Et = Oe[0][0], Mt = Oe[1][0], qt = Oe[0][1], tn = Oe[1][1], xn = Ae.l, An = Ae.r, On = xn.x, Bn = xn.y, qn = An.x, Vn = An.y, sr = (On + qn) / 2, Un = (Bn + Vn) / 2;
                if (Vn === Bn) {
                  if (sr < Et || sr >= Mt) return;
                  if (On > qn) {
                    if (ht) {
                      if (ht.y >= tn) return;
                    } else ht = { x: sr, y: qt };
                    Be = { x: sr, y: tn };
                  } else {
                    if (ht) {
                      if (ht.y < qt) return;
                    } else ht = { x: sr, y: tn };
                    Be = { x: sr, y: qt };
                  }
                } else if (nt = Un - (Ue = (On - qn) / (Vn - Bn)) * sr, Ue < -1 || Ue > 1) if (On > qn) {
                  if (ht) {
                    if (ht.y >= tn) return;
                  } else ht = { x: (qt - nt) / Ue, y: qt };
                  Be = { x: (tn - nt) / Ue, y: tn };
                } else {
                  if (ht) {
                    if (ht.y < qt) return;
                  } else ht = { x: (tn - nt) / Ue, y: tn };
                  Be = { x: (qt - nt) / Ue, y: qt };
                }
                else if (Bn < Vn) {
                  if (ht) {
                    if (ht.x >= Mt) return;
                  } else ht = { x: Et, y: Ue * Et + nt };
                  Be = { x: Mt, y: Ue * Mt + nt };
                } else {
                  if (ht) {
                    if (ht.x < Et) return;
                  } else ht = { x: Mt, y: Ue * Mt + nt };
                  Be = { x: Et, y: Ue * Et + nt };
                }
                return Ae.a = ht, Ae.b = Be, !0;
              }
              function Ki(Ae, Oe) {
                this.l = Ae, this.r = Oe, this.a = this.b = null;
              }
              function Fi(Ae, Oe, Be, Ue) {
                var nt = new Ki(Ae, Oe);
                return Rr.push(nt), Be && na(nt, Ae, Oe, Be), Ue && na(nt, Oe, Ae, Ue), Vr[Ae.i].edges.push(new nn(nt, Ae, Oe)), Vr[Oe.i].edges.push(new nn(nt, Oe, Ae)), nt;
              }
              function na(Ae, Oe, Be, Ue) {
                Ae.a || Ae.b ? Ae.l === Be ? Ae.b = Ue : Ae.a = Ue : (Ae.a = Ue, Ae.l = Oe, Ae.r = Be);
              }
              function nn(Ae, Oe, Be) {
                var Ue = Ae.a, nt = Ae.b;
                this.edge = Ae, this.site = Oe, this.angle = Be ? Math.atan2(Be.y - Oe.y, Be.x - Oe.x) : Ae.l === Oe ? Math.atan2(nt.x - Ue.x, Ue.y - nt.y) : Math.atan2(Ue.x - nt.x, nt.y - Ue.y);
              }
              function Tn() {
                this._ = null;
              }
              function Dn(Ae) {
                Ae.U = Ae.C = Ae.L = Ae.R = Ae.P = Ae.N = null;
              }
              function Fn(Ae, Oe) {
                var Be = Oe, Ue = Oe.R, nt = Be.U;
                nt ? nt.L === Be ? nt.L = Ue : nt.R = Ue : Ae._ = Ue, Ue.U = nt, Be.U = Ue, Be.R = Ue.L, Be.R && (Be.R.U = Be), Ue.L = Be;
              }
              function fr(Ae, Oe) {
                var Be = Oe, Ue = Oe.L, nt = Be.U;
                nt ? nt.L === Be ? nt.L = Ue : nt.R = Ue : Ae._ = Ue, Ue.U = nt, Be.U = Ue, Be.L = Ue.R, Be.L && (Be.L.U = Be), Ue.R = Be;
              }
              function dr(Ae) {
                for (; Ae.L; ) Ae = Ae.L;
                return Ae;
              }
              function yr(Ae, Oe) {
                var Be, Ue, nt, ht = Ae.sort(ai).pop();
                for (Rr = [], Vr = new Array(Ae.length), Xr = new Tn(), ti = new Tn(); ; ) if (nt = Jr, ht && (!nt || ht.y < nt.y || ht.y === nt.y && ht.x < nt.x)) ht.x === Be && ht.y === Ue || (Vr[ht.i] = new Pi(ht), Ai(ht), Be = ht.x, Ue = ht.y), ht = Ae.pop();
                else {
                  if (!nt) break;
                  Qi(nt.arc);
                }
                Oe && (function(Mt) {
                  for (var qt, tn, xn, An, On, Bn = Rr, qn = (tn = Mt[0][0], xn = Mt[0][1], An = Mt[1][0], On = Mt[1][1], function(sr) {
                    var Un, wr = sr.a, Cr = sr.b, Tr = wr.x, ii = wr.y, wi = 0, Wi = 1, Yi = Cr.x - Tr, ra = Cr.y - ii;
                    if (Un = tn - Tr, Yi || !(Un > 0)) {
                      if (Un /= Yi, Yi < 0) {
                        if (Un < wi) return;
                        Un < Wi && (Wi = Un);
                      } else if (Yi > 0) {
                        if (Un > Wi) return;
                        Un > wi && (wi = Un);
                      }
                      if (Un = An - Tr, Yi || !(Un < 0)) {
                        if (Un /= Yi, Yi < 0) {
                          if (Un > Wi) return;
                          Un > wi && (wi = Un);
                        } else if (Yi > 0) {
                          if (Un < wi) return;
                          Un < Wi && (Wi = Un);
                        }
                        if (Un = xn - ii, ra || !(Un > 0)) {
                          if (Un /= ra, ra < 0) {
                            if (Un < wi) return;
                            Un < Wi && (Wi = Un);
                          } else if (ra > 0) {
                            if (Un > Wi) return;
                            Un > wi && (wi = Un);
                          }
                          if (Un = On - ii, ra || !(Un < 0)) {
                            if (Un /= ra, ra < 0) {
                              if (Un > Wi) return;
                              Un > wi && (wi = Un);
                            } else if (ra > 0) {
                              if (Un < wi) return;
                              Un < Wi && (Wi = Un);
                            }
                            return wi > 0 && (sr.a = { x: Tr + wi * Yi, y: ii + wi * ra }), Wi < 1 && (sr.b = { x: Tr + Wi * Yi, y: ii + Wi * ra }), sr;
                          }
                        }
                      }
                    }
                  }), Vn = Bn.length; Vn--; ) (!ua(qt = Bn[Vn], Mt) || !qn(qt) || Se(qt.a.x - qt.b.x) < rt && Se(qt.a.y - qt.b.y) < rt) && (qt.a = qt.b = null, Bn.splice(Vn, 1));
                }(Oe), function(Mt) {
                  for (var qt, tn, xn, An, On, Bn, qn, Vn, sr, Un, wr = Mt[0][0], Cr = Mt[1][0], Tr = Mt[0][1], ii = Mt[1][1], wi = Vr, Wi = wi.length; Wi--; ) if ((On = wi[Wi]) && On.prepare()) for (Vn = (qn = On.edges).length, Bn = 0; Bn < Vn; ) xn = (Un = qn[Bn].end()).x, An = Un.y, qt = (sr = qn[++Bn % Vn].start()).x, tn = sr.y, (Se(xn - qt) > rt || Se(An - tn) > rt) && (qn.splice(Bn, 0, new nn((Yi = On.site, ra = Un, ha = Se(xn - wr) < rt && ii - An > rt ? { x: wr, y: Se(qt - wr) < rt ? tn : ii } : Se(An - ii) < rt && Cr - xn > rt ? { x: Se(tn - ii) < rt ? qt : Cr, y: ii } : Se(xn - Cr) < rt && An - Tr > rt ? { x: Cr, y: Se(qt - Cr) < rt ? tn : Tr } : Se(An - Tr) < rt && xn - wr > rt ? { x: Se(tn - Tr) < rt ? qt : wr, y: Tr } : null, ya = void 0, (ya = new Ki(Yi, null)).a = ra, ya.b = ha, Rr.push(ya), ya), On.site, null)), ++Vn);
                  var Yi, ra, ha, ya;
                }(Oe));
                var Et = { cells: Vr, edges: Rr };
                return Xr = ti = Rr = Vr = null, Et;
              }
              function ai(Ae, Oe) {
                return Oe.y - Ae.y || Oe.x - Ae.x;
              }
              Pi.prototype.prepare = function() {
                for (var Ae, Oe = this.edges, Be = Oe.length; Be--; ) (Ae = Oe[Be].edge).b && Ae.a || Oe.splice(Be, 1);
                return Oe.sort(Oi), Oe.length;
              }, nn.prototype = { start: function() {
                return this.edge.l === this.site ? this.edge.a : this.edge.b;
              }, end: function() {
                return this.edge.l === this.site ? this.edge.b : this.edge.a;
              } }, Tn.prototype = { insert: function(Ae, Oe) {
                var Be, Ue, nt;
                if (Ae) {
                  if (Oe.P = Ae, Oe.N = Ae.N, Ae.N && (Ae.N.P = Oe), Ae.N = Oe, Ae.R) {
                    for (Ae = Ae.R; Ae.L; ) Ae = Ae.L;
                    Ae.L = Oe;
                  } else Ae.R = Oe;
                  Be = Ae;
                } else this._ ? (Ae = dr(this._), Oe.P = null, Oe.N = Ae, Ae.P = Ae.L = Oe, Be = Ae) : (Oe.P = Oe.N = null, this._ = Oe, Be = null);
                for (Oe.L = Oe.R = null, Oe.U = Be, Oe.C = !0, Ae = Oe; Be && Be.C; ) Be === (Ue = Be.U).L ? (nt = Ue.R) && nt.C ? (Be.C = nt.C = !1, Ue.C = !0, Ae = Ue) : (Ae === Be.R && (Fn(this, Be), Be = (Ae = Be).U), Be.C = !1, Ue.C = !0, fr(this, Ue)) : (nt = Ue.L) && nt.C ? (Be.C = nt.C = !1, Ue.C = !0, Ae = Ue) : (Ae === Be.L && (fr(this, Be), Be = (Ae = Be).U), Be.C = !1, Ue.C = !0, Fn(this, Ue)), Be = Ae.U;
                this._.C = !1;
              }, remove: function(Ae) {
                Ae.N && (Ae.N.P = Ae.P), Ae.P && (Ae.P.N = Ae.N), Ae.N = Ae.P = null;
                var Oe, Be, Ue, nt = Ae.U, ht = Ae.L, Et = Ae.R;
                if (Be = ht ? Et ? dr(Et) : ht : Et, nt ? nt.L === Ae ? nt.L = Be : nt.R = Be : this._ = Be, ht && Et ? (Ue = Be.C, Be.C = Ae.C, Be.L = ht, ht.U = Be, Be !== Et ? (nt = Be.U, Be.U = Ae.U, Ae = Be.R, nt.L = Ae, Be.R = Et, Et.U = Be) : (Be.U = nt, nt = Be, Ae = Be.R)) : (Ue = Ae.C, Ae = Be), Ae && (Ae.U = nt), !Ue) if (Ae && Ae.C) Ae.C = !1;
                else {
                  do {
                    if (Ae === this._) break;
                    if (Ae === nt.L) {
                      if ((Oe = nt.R).C && (Oe.C = !1, nt.C = !0, Fn(this, nt), Oe = nt.R), Oe.L && Oe.L.C || Oe.R && Oe.R.C) {
                        Oe.R && Oe.R.C || (Oe.L.C = !1, Oe.C = !0, fr(this, Oe), Oe = nt.R), Oe.C = nt.C, nt.C = Oe.R.C = !1, Fn(this, nt), Ae = this._;
                        break;
                      }
                    } else if ((Oe = nt.L).C && (Oe.C = !1, nt.C = !0, fr(this, nt), Oe = nt.L), Oe.L && Oe.L.C || Oe.R && Oe.R.C) {
                      Oe.L && Oe.L.C || (Oe.R.C = !1, Oe.C = !0, Fn(this, Oe), Oe = nt.L), Oe.C = nt.C, nt.C = Oe.L.C = !1, fr(this, nt), Ae = this._;
                      break;
                    }
                    Oe.C = !0, Ae = nt, nt = nt.U;
                  } while (!Ae.C);
                  Ae && (Ae.C = !1);
                }
              } }, ee.geom.voronoi = function(Ae) {
                var Oe = Da, Be = la, Ue = Oe, nt = Be, ht = Fr;
                if (Ae) return Et(Ae);
                function Et(qt) {
                  var tn = new Array(qt.length), xn = ht[0][0], An = ht[0][1], On = ht[1][0], Bn = ht[1][1];
                  return yr(Mt(qt), ht).cells.forEach(function(qn, Vn) {
                    var sr = qn.edges, Un = qn.site;
                    (tn[Vn] = sr.length ? sr.map(function(wr) {
                      var Cr = wr.start();
                      return [Cr.x, Cr.y];
                    }) : Un.x >= xn && Un.x <= On && Un.y >= An && Un.y <= Bn ? [[xn, Bn], [On, Bn], [On, An], [xn, An]] : []).point = qt[Vn];
                  }), tn;
                }
                function Mt(qt) {
                  return qt.map(function(tn, xn) {
                    return { x: Math.round(Ue(tn, xn) / rt) * rt, y: Math.round(nt(tn, xn) / rt) * rt, i: xn };
                  });
                }
                return Et.links = function(qt) {
                  return yr(Mt(qt)).edges.filter(function(tn) {
                    return tn.l && tn.r;
                  }).map(function(tn) {
                    return { source: qt[tn.l.i], target: qt[tn.r.i] };
                  });
                }, Et.triangles = function(qt) {
                  var tn = [];
                  return yr(Mt(qt)).cells.forEach(function(xn, An) {
                    for (var On, Bn, qn, Vn, sr = xn.site, Un = xn.edges.sort(Oi), wr = -1, Cr = Un.length, Tr = Un[Cr - 1].edge, ii = Tr.l === sr ? Tr.r : Tr.l; ++wr < Cr; ) On = ii, ii = (Tr = Un[wr].edge).l === sr ? Tr.r : Tr.l, An < On.i && An < ii.i && (qn = On, Vn = ii, ((Bn = sr).x - Vn.x) * (qn.y - Bn.y) - (Bn.x - qn.x) * (Vn.y - Bn.y) < 0) && tn.push([qt[An], qt[On.i], qt[ii.i]]);
                  }), tn;
                }, Et.x = function(qt) {
                  return arguments.length ? (Ue = Ci(Oe = qt), Et) : Oe;
                }, Et.y = function(qt) {
                  return arguments.length ? (nt = Ci(Be = qt), Et) : Be;
                }, Et.clipExtent = function(qt) {
                  return arguments.length ? (ht = qt ?? Fr, Et) : ht === Fr ? null : ht;
                }, Et.size = function(qt) {
                  return arguments.length ? Et.clipExtent(qt && [[0, 0], qt]) : ht === Fr ? null : ht && ht[1];
                }, Et;
              };
              var Fr = [[-1e6, -1e6], [1e6, 1e6]];
              function Ti(Ae) {
                return Ae.x;
              }
              function Si(Ae) {
                return Ae.y;
              }
              function Ei(Ae, Oe, Be, Ue, nt, ht) {
                if (!Ae(Oe, Be, Ue, nt, ht)) {
                  var Et = 0.5 * (Be + nt), Mt = 0.5 * (Ue + ht), qt = Oe.nodes;
                  qt[0] && Ei(Ae, qt[0], Be, Ue, Et, Mt), qt[1] && Ei(Ae, qt[1], Et, Ue, nt, Mt), qt[2] && Ei(Ae, qt[2], Be, Mt, Et, ht), qt[3] && Ei(Ae, qt[3], Et, Mt, nt, ht);
                }
              }
              function Fa(Ae, Oe) {
                Ae = ee.rgb(Ae), Oe = ee.rgb(Oe);
                var Be = Ae.r, Ue = Ae.g, nt = Ae.b, ht = Oe.r - Be, Et = Oe.g - Ue, Mt = Oe.b - nt;
                return function(qt) {
                  return "#" + ri(Math.round(Be + ht * qt)) + ri(Math.round(Ue + Et * qt)) + ri(Math.round(nt + Mt * qt));
                };
              }
              function $o(Ae, Oe) {
                var Be, Ue = {}, nt = {};
                for (Be in Ae) Be in Oe ? Ue[Be] = Gl(Ae[Be], Oe[Be]) : nt[Be] = Ae[Be];
                for (Be in Oe) Be in Ae || (nt[Be] = Oe[Be]);
                return function(ht) {
                  for (Be in Ue) nt[Be] = Ue[Be](ht);
                  return nt;
                };
              }
              function Gs(Ae, Oe) {
                return Ae = +Ae, Oe = +Oe, function(Be) {
                  return Ae * (1 - Be) + Oe * Be;
                };
              }
              function Al(Ae, Oe) {
                var Be, Ue, nt, ht = ul.lastIndex = gc.lastIndex = 0, Et = -1, Mt = [], qt = [];
                for (Ae += "", Oe += ""; (Be = ul.exec(Ae)) && (Ue = gc.exec(Oe)); ) (nt = Ue.index) > ht && (nt = Oe.slice(ht, nt), Mt[Et] ? Mt[Et] += nt : Mt[++Et] = nt), (Be = Be[0]) === (Ue = Ue[0]) ? Mt[Et] ? Mt[Et] += Ue : Mt[++Et] = Ue : (Mt[++Et] = null, qt.push({ i: Et, x: Gs(Be, Ue) })), ht = gc.lastIndex;
                return ht < Oe.length && (nt = Oe.slice(ht), Mt[Et] ? Mt[Et] += nt : Mt[++Et] = nt), Mt.length < 2 ? qt[0] ? (Oe = qt[0].x, function(tn) {
                  return Oe(tn) + "";
                }) : function() {
                  return Oe;
                } : (Oe = qt.length, function(tn) {
                  for (var xn, An = 0; An < Oe; ++An) Mt[(xn = qt[An]).i] = xn.x(tn);
                  return Mt.join("");
                });
              }
              ee.geom.delaunay = function(Ae) {
                return ee.geom.voronoi().triangles(Ae);
              }, ee.geom.quadtree = function(Ae, Oe, Be, Ue, nt) {
                var ht, Et = Da, Mt = la;
                if (ht = arguments.length) return Et = Ti, Mt = Si, ht === 3 && (nt = Be, Ue = Oe, Be = Oe = 0), qt(Ae);
                function qt(tn) {
                  var xn, An, On, Bn, qn, Vn, sr, Un, wr, Cr = Ci(Et), Tr = Ci(Mt);
                  if (Oe != null) Vn = Oe, sr = Be, Un = Ue, wr = nt;
                  else if (Un = wr = -(Vn = sr = 1 / 0), An = [], On = [], qn = tn.length, ht) for (Bn = 0; Bn < qn; ++Bn) (xn = tn[Bn]).x < Vn && (Vn = xn.x), xn.y < sr && (sr = xn.y), xn.x > Un && (Un = xn.x), xn.y > wr && (wr = xn.y), An.push(xn.x), On.push(xn.y);
                  else for (Bn = 0; Bn < qn; ++Bn) {
                    var ii = +Cr(xn = tn[Bn], Bn), wi = +Tr(xn, Bn);
                    ii < Vn && (Vn = ii), wi < sr && (sr = wi), ii > Un && (Un = ii), wi > wr && (wr = wi), An.push(ii), On.push(wi);
                  }
                  var Wi = Un - Vn, Yi = wr - sr;
                  function ra(va, qa, Aa, Xa, Oa, Ya, Wo, Ja) {
                    if (!isNaN(Aa) && !isNaN(Xa)) if (va.leaf) {
                      var Uo = va.x, ys = va.y;
                      if (Uo != null) if (Se(Uo - Aa) + Se(ys - Xa) < 0.01) ha(va, qa, Aa, Xa, Oa, Ya, Wo, Ja);
                      else {
                        var il = va.point;
                        va.x = va.y = va.point = null, ha(va, il, Uo, ys, Oa, Ya, Wo, Ja), ha(va, qa, Aa, Xa, Oa, Ya, Wo, Ja);
                      }
                      else va.x = Aa, va.y = Xa, va.point = qa;
                    } else ha(va, qa, Aa, Xa, Oa, Ya, Wo, Ja);
                  }
                  function ha(va, qa, Aa, Xa, Oa, Ya, Wo, Ja) {
                    var Uo = 0.5 * (Oa + Wo), ys = 0.5 * (Ya + Ja), il = Aa >= Uo, Is = Xa >= ys, ml = Is << 1 | il;
                    va.leaf = !1, il ? Oa = Uo : Wo = Uo, Is ? Ya = ys : Ja = ys, ra(va = va.nodes[ml] || (va.nodes[ml] = { leaf: !0, nodes: [], point: null, x: null, y: null }), qa, Aa, Xa, Oa, Ya, Wo, Ja);
                  }
                  Wi > Yi ? wr = sr + Wi : Un = Vn + Yi;
                  var ya = { leaf: !0, nodes: [], point: null, x: null, y: null, add: function(va) {
                    ra(ya, va, +Cr(va, ++Bn), +Tr(va, Bn), Vn, sr, Un, wr);
                  } };
                  if (ya.visit = function(va) {
                    Ei(va, ya, Vn, sr, Un, wr);
                  }, ya.find = function(va) {
                    return function(qa, Aa, Xa, Oa, Ya, Wo, Ja) {
                      var Uo, ys = 1 / 0;
                      return function il(Is, ml, nl, Il, Fl) {
                        if (!(ml > Wo || nl > Ja || Il < Oa || Fl < Ya)) {
                          if (qc = Is.point) {
                            var qc, ql = Aa - Is.x, jc = Xa - Is.y, Dl = ql * ql + jc * jc;
                            if (Dl < ys) {
                              var Ml = Math.sqrt(ys = Dl);
                              Oa = Aa - Ml, Ya = Xa - Ml, Wo = Aa + Ml, Ja = Xa + Ml, Uo = qc;
                            }
                          }
                          for (var pc = Is.nodes, hl = 0.5 * (ml + Il), xl = 0.5 * (nl + Fl), Bl = (Xa >= xl) << 1 | Aa >= hl, al = Bl + 4; Bl < al; ++Bl) if (Is = pc[3 & Bl]) switch (3 & Bl) {
                            case 0:
                              il(Is, ml, nl, hl, xl);
                              break;
                            case 1:
                              il(Is, hl, nl, Il, xl);
                              break;
                            case 2:
                              il(Is, ml, xl, hl, Fl);
                              break;
                            case 3:
                              il(Is, hl, xl, Il, Fl);
                          }
                        }
                      }(qa, Oa, Ya, Wo, Ja), Uo;
                    }(ya, va[0], va[1], Vn, sr, Un, wr);
                  }, Bn = -1, Oe == null) {
                    for (; ++Bn < qn; ) ra(ya, tn[Bn], An[Bn], On[Bn], Vn, sr, Un, wr);
                    --Bn;
                  } else tn.forEach(ya.add);
                  return An = On = tn = xn = null, ya;
                }
                return qt.x = function(tn) {
                  return arguments.length ? (Et = tn, qt) : Et;
                }, qt.y = function(tn) {
                  return arguments.length ? (Mt = tn, qt) : Mt;
                }, qt.extent = function(tn) {
                  return arguments.length ? (tn == null ? Oe = Be = Ue = nt = null : (Oe = +tn[0][0], Be = +tn[0][1], Ue = +tn[1][0], nt = +tn[1][1]), qt) : Oe == null ? null : [[Oe, Be], [Ue, nt]];
                }, qt.size = function(tn) {
                  return arguments.length ? (tn == null ? Oe = Be = Ue = nt = null : (Oe = Be = 0, Ue = +tn[0], nt = +tn[1]), qt) : Oe == null ? null : [Ue - Oe, nt - Be];
                }, qt;
              }, ee.interpolateRgb = Fa, ee.interpolateObject = $o, ee.interpolateNumber = Gs, ee.interpolateString = Al;
              var ul = /[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g, gc = new RegExp(ul.source, "g");
              function Gl(Ae, Oe) {
                for (var Be, Ue = ee.interpolators.length; --Ue >= 0 && !(Be = ee.interpolators[Ue](Ae, Oe)); ) ;
                return Be;
              }
              function Sl(Ae, Oe) {
                var Be, Ue = [], nt = [], ht = Ae.length, Et = Oe.length, Mt = Math.min(Ae.length, Oe.length);
                for (Be = 0; Be < Mt; ++Be) Ue.push(Gl(Ae[Be], Oe[Be]));
                for (; Be < ht; ++Be) nt[Be] = Ae[Be];
                for (; Be < Et; ++Be) nt[Be] = Oe[Be];
                return function(qt) {
                  for (Be = 0; Be < Mt; ++Be) nt[Be] = Ue[Be](qt);
                  return nt;
                };
              }
              ee.interpolate = Gl, ee.interpolators = [function(Ae, Oe) {
                var Be = typeof Oe;
                return (Be === "string" ? ei.has(Oe.toLowerCase()) || /^(#|rgb\(|hsl\()/i.test(Oe) ? Fa : Al : Oe instanceof Zt ? Fa : Array.isArray(Oe) ? Sl : Be === "object" && isNaN(Oe) ? $o : Gs)(Ae, Oe);
              }], ee.interpolateArray = Sl;
              var ls = function() {
                return Xe;
              }, vs = ee.map({ linear: ls, poly: function(Ae) {
                return function(Oe) {
                  return Math.pow(Oe, Ae);
                };
              }, quad: function() {
                return yc;
              }, cubic: function() {
                return gu;
              }, sin: function() {
                return Cu;
              }, exp: function() {
                return zc;
              }, circle: function() {
                return Kc;
              }, elastic: function(Ae, Oe) {
                var Be;
                return arguments.length < 2 && (Oe = 0.45), arguments.length ? Be = Oe / wt * Math.asin(1 / Ae) : (Ae = 1, Be = Oe / 4), function(Ue) {
                  return 1 + Ae * Math.pow(2, -10 * Ue) * Math.sin((Ue - Be) * wt / Oe);
                };
              }, back: function(Ae) {
                return Ae || (Ae = 1.70158), function(Oe) {
                  return Oe * Oe * ((Ae + 1) * Oe - Ae);
                };
              }, bounce: function() {
                return zi;
              } }), Vl = ee.map({ in: Xe, out: Ll, "in-out": Xl, "out-in": function(Ae) {
                return Xl(Ll(Ae));
              } });
              function Ll(Ae) {
                return function(Oe) {
                  return 1 - Ae(1 - Oe);
                };
              }
              function Xl(Ae) {
                return function(Oe) {
                  return 0.5 * (Oe < 0.5 ? Ae(2 * Oe) : 2 - Ae(2 - 2 * Oe));
                };
              }
              function yc(Ae) {
                return Ae * Ae;
              }
              function gu(Ae) {
                return Ae * Ae * Ae;
              }
              function Nc(Ae) {
                if (Ae <= 0) return 0;
                if (Ae >= 1) return 1;
                var Oe = Ae * Ae, Be = Oe * Ae;
                return 4 * (Ae < 0.5 ? Be : 3 * (Ae - Oe) + Be - 0.75);
              }
              function Cu(Ae) {
                return 1 - Math.cos(Ae * At);
              }
              function zc(Ae) {
                return Math.pow(2, 10 * (Ae - 1));
              }
              function Kc(Ae) {
                return 1 - Math.sqrt(1 - Ae * Ae);
              }
              function zi(Ae) {
                return Ae < 0.36363636363636365 ? 7.5625 * Ae * Ae : Ae < 0.7272727272727273 ? 7.5625 * (Ae -= 0.5454545454545454) * Ae + 0.75 : Ae < 0.9090909090909091 ? 7.5625 * (Ae -= 0.8181818181818182) * Ae + 0.9375 : 7.5625 * (Ae -= 0.9545454545454546) * Ae + 0.984375;
              }
              function Fu(Ae, Oe) {
                return Oe -= Ae, function(Be) {
                  return Math.round(Ae + Oe * Be);
                };
              }
              function d1(Ae) {
                var Oe, Be, Ue, nt = [Ae.a, Ae.b], ht = [Ae.c, Ae.d], Et = Ku(nt), Mt = Gu(nt, ht), qt = Ku(((Oe = ht)[0] += (Ue = -Mt) * (Be = nt)[0], Oe[1] += Ue * Be[1], Oe)) || 0;
                nt[0] * ht[1] < ht[0] * nt[1] && (nt[0] *= -1, nt[1] *= -1, Et *= -1, Mt *= -1), this.rotate = (Et ? Math.atan2(nt[1], nt[0]) : Math.atan2(-ht[0], ht[1])) * Ot, this.translate = [Ae.e, Ae.f], this.scale = [Et, qt], this.skew = qt ? Math.atan2(Mt, qt) * Ot : 0;
              }
              function Gu(Ae, Oe) {
                return Ae[0] * Oe[0] + Ae[1] * Oe[1];
              }
              function Ku(Ae) {
                var Oe = Math.sqrt(Gu(Ae, Ae));
                return Oe && (Ae[0] /= Oe, Ae[1] /= Oe), Oe;
              }
              ee.ease = function(Ae) {
                var Oe, Be = Ae.indexOf("-"), Ue = Be >= 0 ? Ae.slice(0, Be) : Ae, nt = Be >= 0 ? Ae.slice(Be + 1) : "in";
                return Ue = vs.get(Ue) || ls, nt = Vl.get(nt) || Xe, Oe = nt(Ue.apply(null, ae.call(arguments, 1))), function(ht) {
                  return ht <= 0 ? 0 : ht >= 1 ? 1 : Oe(ht);
                };
              }, ee.interpolateHcl = function(Ae, Oe) {
                Ae = ee.hcl(Ae), Oe = ee.hcl(Oe);
                var Be = Ae.h, Ue = Ae.c, nt = Ae.l, ht = Oe.h - Be, Et = Oe.c - Ue, Mt = Oe.l - nt;
                return isNaN(Et) && (Et = 0, Ue = isNaN(Ue) ? Oe.c : Ue), isNaN(ht) ? (ht = 0, Be = isNaN(Be) ? Oe.h : Be) : ht > 180 ? ht -= 360 : ht < -180 && (ht += 360), function(qt) {
                  return jn(Be + ht * qt, Ue + Et * qt, nt + Mt * qt) + "";
                };
              }, ee.interpolateHsl = function(Ae, Oe) {
                Ae = ee.hsl(Ae), Oe = ee.hsl(Oe);
                var Be = Ae.h, Ue = Ae.s, nt = Ae.l, ht = Oe.h - Be, Et = Oe.s - Ue, Mt = Oe.l - nt;
                return isNaN(Et) && (Et = 0, Ue = isNaN(Ue) ? Oe.s : Ue), isNaN(ht) ? (ht = 0, Be = isNaN(Be) ? Oe.h : Be) : ht > 180 ? ht -= 360 : ht < -180 && (ht += 360), function(qt) {
                  return In(Be + ht * qt, Ue + Et * qt, nt + Mt * qt) + "";
                };
              }, ee.interpolateLab = function(Ae, Oe) {
                Ae = ee.lab(Ae), Oe = ee.lab(Oe);
                var Be = Ae.l, Ue = Ae.a, nt = Ae.b, ht = Oe.l - Be, Et = Oe.a - Ue, Mt = Oe.b - nt;
                return function(qt) {
                  return Yr(Be + ht * qt, Ue + Et * qt, nt + Mt * qt) + "";
                };
              }, ee.interpolateRound = Fu, ee.transform = function(Ae) {
                var Oe = se.createElementNS(ee.ns.prefix.svg, "g");
                return (ee.transform = function(Be) {
                  if (Be != null) {
                    Oe.setAttribute("transform", Be);
                    var Ue = Oe.transform.baseVal.consolidate();
                  }
                  return new d1(Ue ? Ue.matrix : I1);
                })(Ae);
              }, d1.prototype.toString = function() {
                return "translate(" + this.translate + ")rotate(" + this.rotate + ")skewX(" + this.skew + ")scale(" + this.scale + ")";
              };
              var I1 = { a: 1, b: 0, c: 0, d: 1, e: 0, f: 0 };
              function f1(Ae) {
                return Ae.length ? Ae.pop() + "," : "";
              }
              function D1(Ae, Oe) {
                var Be = [], Ue = [];
                return Ae = ee.transform(Ae), Oe = ee.transform(Oe), function(nt, ht, Et, Mt) {
                  if (nt[0] !== ht[0] || nt[1] !== ht[1]) {
                    var qt = Et.push("translate(", null, ",", null, ")");
                    Mt.push({ i: qt - 4, x: Gs(nt[0], ht[0]) }, { i: qt - 2, x: Gs(nt[1], ht[1]) });
                  } else (ht[0] || ht[1]) && Et.push("translate(" + ht + ")");
                }(Ae.translate, Oe.translate, Be, Ue), function(nt, ht, Et, Mt) {
                  nt !== ht ? (nt - ht > 180 ? ht += 360 : ht - nt > 180 && (nt += 360), Mt.push({ i: Et.push(f1(Et) + "rotate(", null, ")") - 2, x: Gs(nt, ht) })) : ht && Et.push(f1(Et) + "rotate(" + ht + ")");
                }(Ae.rotate, Oe.rotate, Be, Ue), function(nt, ht, Et, Mt) {
                  nt !== ht ? Mt.push({ i: Et.push(f1(Et) + "skewX(", null, ")") - 2, x: Gs(nt, ht) }) : ht && Et.push(f1(Et) + "skewX(" + ht + ")");
                }(Ae.skew, Oe.skew, Be, Ue), function(nt, ht, Et, Mt) {
                  if (nt[0] !== ht[0] || nt[1] !== ht[1]) {
                    var qt = Et.push(f1(Et) + "scale(", null, ",", null, ")");
                    Mt.push({ i: qt - 4, x: Gs(nt[0], ht[0]) }, { i: qt - 2, x: Gs(nt[1], ht[1]) });
                  } else ht[0] === 1 && ht[1] === 1 || Et.push(f1(Et) + "scale(" + ht + ")");
                }(Ae.scale, Oe.scale, Be, Ue), Ae = Oe = null, function(nt) {
                  for (var ht, Et = -1, Mt = Ue.length; ++Et < Mt; ) Be[(ht = Ue[Et]).i] = ht.x(nt);
                  return Be.join("");
                };
              }
              function Hl(Ae, Oe) {
                return Oe = (Oe -= Ae = +Ae) || 1 / Oe, function(Be) {
                  return (Be - Ae) / Oe;
                };
              }
              function Kl(Ae, Oe) {
                return Oe = (Oe -= Ae = +Ae) || 1 / Oe, function(Be) {
                  return Math.max(0, Math.min(1, (Be - Ae) / Oe));
                };
              }
              function uu(Ae) {
                for (var Oe = Ae.source, Be = Ae.target, Ue = function(Et, Mt) {
                  if (Et === Mt) return Et;
                  for (var qt = Hs(Et), tn = Hs(Mt), xn = qt.pop(), An = tn.pop(), On = null; xn === An; ) On = xn, xn = qt.pop(), An = tn.pop();
                  return On;
                }(Oe, Be), nt = [Oe]; Oe !== Ue; ) Oe = Oe.parent, nt.push(Oe);
                for (var ht = nt.length; Be !== Ue; ) nt.splice(ht, 0, Be), Be = Be.parent;
                return nt;
              }
              function Hs(Ae) {
                for (var Oe = [], Be = Ae.parent; Be != null; ) Oe.push(Ae), Ae = Be, Be = Be.parent;
                return Oe.push(Ae), Oe;
              }
              function b1(Ae) {
                Ae.fixed |= 2;
              }
              function Wl(Ae) {
                Ae.fixed &= -7;
              }
              function zl(Ae) {
                Ae.fixed |= 4, Ae.px = Ae.x, Ae.py = Ae.y;
              }
              function os(Ae) {
                Ae.fixed &= -5;
              }
              function jo(Ae, Oe, Be) {
                var Ue = 0, nt = 0;
                if (Ae.charge = 0, !Ae.leaf) for (var ht, Et = Ae.nodes, Mt = Et.length, qt = -1; ++qt < Mt; ) (ht = Et[qt]) != null && (jo(ht, Oe, Be), Ae.charge += ht.charge, Ue += ht.charge * ht.cx, nt += ht.charge * ht.cy);
                if (Ae.point) {
                  Ae.leaf || (Ae.point.x += Math.random() - 0.5, Ae.point.y += Math.random() - 0.5);
                  var tn = Oe * Be[Ae.point.index];
                  Ae.charge += Ae.pointCharge = tn, Ue += tn * Ae.point.x, nt += tn * Ae.point.y;
                }
                Ae.cx = Ue / Ae.charge, Ae.cy = nt / Ae.charge;
              }
              ee.interpolateTransform = D1, ee.layout = {}, ee.layout.bundle = function() {
                return function(Ae) {
                  for (var Oe = [], Be = -1, Ue = Ae.length; ++Be < Ue; ) Oe.push(uu(Ae[Be]));
                  return Oe;
                };
              }, ee.layout.chord = function() {
                var Ae, Oe, Be, Ue, nt, ht, Et, Mt = {}, qt = 0;
                function tn() {
                  var An, On, Bn, qn, Vn, sr = {}, Un = [], wr = ee.range(Ue), Cr = [];
                  for (Ae = [], Oe = [], An = 0, qn = -1; ++qn < Ue; ) {
                    for (On = 0, Vn = -1; ++Vn < Ue; ) On += Be[qn][Vn];
                    Un.push(On), Cr.push(ee.range(Ue)), An += On;
                  }
                  for (nt && wr.sort(function(ya, va) {
                    return nt(Un[ya], Un[va]);
                  }), ht && Cr.forEach(function(ya, va) {
                    ya.sort(function(qa, Aa) {
                      return ht(Be[va][qa], Be[va][Aa]);
                    });
                  }), An = (wt - qt * Ue) / An, On = 0, qn = -1; ++qn < Ue; ) {
                    for (Bn = On, Vn = -1; ++Vn < Ue; ) {
                      var Tr = wr[qn], ii = Cr[Tr][Vn], wi = Be[Tr][ii], Wi = On, Yi = On += wi * An;
                      sr[Tr + "-" + ii] = { index: Tr, subindex: ii, startAngle: Wi, endAngle: Yi, value: wi };
                    }
                    Oe[Tr] = { index: Tr, startAngle: Bn, endAngle: On, value: Un[Tr] }, On += qt;
                  }
                  for (qn = -1; ++qn < Ue; ) for (Vn = qn - 1; ++Vn < Ue; ) {
                    var ra = sr[qn + "-" + Vn], ha = sr[Vn + "-" + qn];
                    (ra.value || ha.value) && Ae.push(ra.value < ha.value ? { source: ha, target: ra } : { source: ra, target: ha });
                  }
                  Et && xn();
                }
                function xn() {
                  Ae.sort(function(An, On) {
                    return Et((An.source.value + An.target.value) / 2, (On.source.value + On.target.value) / 2);
                  });
                }
                return Mt.matrix = function(An) {
                  return arguments.length ? (Ue = (Be = An) && Be.length, Ae = Oe = null, Mt) : Be;
                }, Mt.padding = function(An) {
                  return arguments.length ? (qt = An, Ae = Oe = null, Mt) : qt;
                }, Mt.sortGroups = function(An) {
                  return arguments.length ? (nt = An, Ae = Oe = null, Mt) : nt;
                }, Mt.sortSubgroups = function(An) {
                  return arguments.length ? (ht = An, Ae = null, Mt) : ht;
                }, Mt.sortChords = function(An) {
                  return arguments.length ? (Et = An, Ae && xn(), Mt) : Et;
                }, Mt.chords = function() {
                  return Ae || tn(), Ae;
                }, Mt.groups = function() {
                  return Oe || tn(), Oe;
                }, Mt;
              }, ee.layout.force = function() {
                var Ae, Oe, Be, Ue, nt, ht, Et = {}, Mt = ee.dispatch("start", "tick", "end"), qt = [1, 1], tn = 0.9, xn = uo, An = so, On = -30, Bn = So, qn = 0.1, Vn = 0.64, sr = [], Un = [];
                function wr(Tr) {
                  return function(ii, wi, Wi, Yi) {
                    if (ii.point !== Tr) {
                      var ra = ii.cx - Tr.x, ha = ii.cy - Tr.y, ya = Yi - wi, va = ra * ra + ha * ha;
                      if (ya * ya / Vn < va) {
                        if (va < Bn) {
                          var qa = ii.charge / va;
                          Tr.px -= ra * qa, Tr.py -= ha * qa;
                        }
                        return !0;
                      }
                      ii.point && va && va < Bn && (qa = ii.pointCharge / va, Tr.px -= ra * qa, Tr.py -= ha * qa);
                    }
                    return !ii.charge;
                  };
                }
                function Cr(Tr) {
                  Tr.px = ee.event.x, Tr.py = ee.event.y, Et.resume();
                }
                return Et.tick = function() {
                  if ((Be *= 0.99) < 5e-3) return Ae = null, Mt.end({ type: "end", alpha: Be = 0 }), !0;
                  var Tr, ii, wi, Wi, Yi, ra, ha, ya, va, qa = sr.length, Aa = Un.length;
                  for (ii = 0; ii < Aa; ++ii) Wi = (wi = Un[ii]).source, (ra = (ya = (Yi = wi.target).x - Wi.x) * ya + (va = Yi.y - Wi.y) * va) && (ya *= ra = Be * nt[ii] * ((ra = Math.sqrt(ra)) - Ue[ii]) / ra, va *= ra, Yi.x -= ya * (ha = Wi.weight + Yi.weight ? Wi.weight / (Wi.weight + Yi.weight) : 0.5), Yi.y -= va * ha, Wi.x += ya * (ha = 1 - ha), Wi.y += va * ha);
                  if ((ha = Be * qn) && (ya = qt[0] / 2, va = qt[1] / 2, ii = -1, ha)) for (; ++ii < qa; ) (wi = sr[ii]).x += (ya - wi.x) * ha, wi.y += (va - wi.y) * ha;
                  if (On) for (jo(Tr = ee.geom.quadtree(sr), Be, ht), ii = -1; ++ii < qa; ) (wi = sr[ii]).fixed || Tr.visit(wr(wi));
                  for (ii = -1; ++ii < qa; ) (wi = sr[ii]).fixed ? (wi.x = wi.px, wi.y = wi.py) : (wi.x -= (wi.px - (wi.px = wi.x)) * tn, wi.y -= (wi.py - (wi.py = wi.y)) * tn);
                  Mt.tick({ type: "tick", alpha: Be });
                }, Et.nodes = function(Tr) {
                  return arguments.length ? (sr = Tr, Et) : sr;
                }, Et.links = function(Tr) {
                  return arguments.length ? (Un = Tr, Et) : Un;
                }, Et.size = function(Tr) {
                  return arguments.length ? (qt = Tr, Et) : qt;
                }, Et.linkDistance = function(Tr) {
                  return arguments.length ? (xn = typeof Tr == "function" ? Tr : +Tr, Et) : xn;
                }, Et.distance = Et.linkDistance, Et.linkStrength = function(Tr) {
                  return arguments.length ? (An = typeof Tr == "function" ? Tr : +Tr, Et) : An;
                }, Et.friction = function(Tr) {
                  return arguments.length ? (tn = +Tr, Et) : tn;
                }, Et.charge = function(Tr) {
                  return arguments.length ? (On = typeof Tr == "function" ? Tr : +Tr, Et) : On;
                }, Et.chargeDistance = function(Tr) {
                  return arguments.length ? (Bn = Tr * Tr, Et) : Math.sqrt(Bn);
                }, Et.gravity = function(Tr) {
                  return arguments.length ? (qn = +Tr, Et) : qn;
                }, Et.theta = function(Tr) {
                  return arguments.length ? (Vn = Tr * Tr, Et) : Math.sqrt(Vn);
                }, Et.alpha = function(Tr) {
                  return arguments.length ? (Tr = +Tr, Be ? Tr > 0 ? Be = Tr : (Ae.c = null, Ae.t = NaN, Ae = null, Mt.end({ type: "end", alpha: Be = 0 })) : Tr > 0 && (Mt.start({ type: "start", alpha: Be = Tr }), Ae = Dr(Et.tick)), Et) : Be;
                }, Et.start = function() {
                  var Tr, ii, wi, Wi = sr.length, Yi = Un.length, ra = qt[0], ha = qt[1];
                  for (Tr = 0; Tr < Wi; ++Tr) (wi = sr[Tr]).index = Tr, wi.weight = 0;
                  for (Tr = 0; Tr < Yi; ++Tr) typeof (wi = Un[Tr]).source == "number" && (wi.source = sr[wi.source]), typeof wi.target == "number" && (wi.target = sr[wi.target]), ++wi.source.weight, ++wi.target.weight;
                  for (Tr = 0; Tr < Wi; ++Tr) wi = sr[Tr], isNaN(wi.x) && (wi.x = ya("x", ra)), isNaN(wi.y) && (wi.y = ya("y", ha)), isNaN(wi.px) && (wi.px = wi.x), isNaN(wi.py) && (wi.py = wi.y);
                  if (Ue = [], typeof xn == "function") for (Tr = 0; Tr < Yi; ++Tr) Ue[Tr] = +xn.call(this, Un[Tr], Tr);
                  else for (Tr = 0; Tr < Yi; ++Tr) Ue[Tr] = xn;
                  if (nt = [], typeof An == "function") for (Tr = 0; Tr < Yi; ++Tr) nt[Tr] = +An.call(this, Un[Tr], Tr);
                  else for (Tr = 0; Tr < Yi; ++Tr) nt[Tr] = An;
                  if (ht = [], typeof On == "function") for (Tr = 0; Tr < Wi; ++Tr) ht[Tr] = +On.call(this, sr[Tr], Tr);
                  else for (Tr = 0; Tr < Wi; ++Tr) ht[Tr] = On;
                  function ya(va, qa) {
                    if (!ii) {
                      for (ii = new Array(Wi), Ya = 0; Ya < Wi; ++Ya) ii[Ya] = [];
                      for (Ya = 0; Ya < Yi; ++Ya) {
                        var Aa = Un[Ya];
                        ii[Aa.source.index].push(Aa.target), ii[Aa.target.index].push(Aa.source);
                      }
                    }
                    for (var Xa, Oa = ii[Tr], Ya = -1, Wo = Oa.length; ++Ya < Wo; ) if (!isNaN(Xa = Oa[Ya][va])) return Xa;
                    return Math.random() * qa;
                  }
                  return Et.resume();
                }, Et.resume = function() {
                  return Et.alpha(0.1);
                }, Et.stop = function() {
                  return Et.alpha(0);
                }, Et.drag = function() {
                  if (Oe || (Oe = ee.behavior.drag().origin(Xe).on("dragstart.force", b1).on("drag.force", Cr).on("dragend.force", Wl)), !arguments.length) return Oe;
                  this.on("mouseover.force", zl).on("mouseout.force", os).call(Oe);
                }, ee.rebind(Et, Mt, "on");
              };
              var uo = 20, so = 1, So = 1 / 0;
              function Jc(Ae, Oe) {
                return ee.rebind(Ae, Oe, "sort", "children", "value"), Ae.nodes = Ae, Ae.links = zo, Ae;
              }
              function No(Ae, Oe) {
                for (var Be = [Ae]; (Ae = Be.pop()) != null; ) if (Oe(Ae), (nt = Ae.children) && (Ue = nt.length)) for (var Ue, nt; --Ue >= 0; ) Be.push(nt[Ue]);
              }
              function ro(Ae, Oe) {
                for (var Be = [Ae], Ue = []; (Ae = Be.pop()) != null; ) if (Ue.push(Ae), (ht = Ae.children) && (nt = ht.length)) for (var nt, ht, Et = -1; ++Et < nt; ) Be.push(ht[Et]);
                for (; (Ae = Ue.pop()) != null; ) Oe(Ae);
              }
              function Io(Ae) {
                return Ae.children;
              }
              function bs(Ae) {
                return Ae.value;
              }
              function ws(Ae, Oe) {
                return Oe.value - Ae.value;
              }
              function zo(Ae) {
                return ee.merge(Ae.map(function(Oe) {
                  return (Oe.children || []).map(function(Be) {
                    return { source: Oe, target: Be };
                  });
                }));
              }
              ee.layout.hierarchy = function() {
                var Ae = ws, Oe = Io, Be = bs;
                function Ue(nt) {
                  var ht, Et = [nt], Mt = [];
                  for (nt.depth = 0; (ht = Et.pop()) != null; ) if (Mt.push(ht), (tn = Oe.call(Ue, ht, ht.depth)) && (qt = tn.length)) {
                    for (var qt, tn, xn; --qt >= 0; ) Et.push(xn = tn[qt]), xn.parent = ht, xn.depth = ht.depth + 1;
                    Be && (ht.value = 0), ht.children = tn;
                  } else Be && (ht.value = +Be.call(Ue, ht, ht.depth) || 0), delete ht.children;
                  return ro(nt, function(An) {
                    var On, Bn;
                    Ae && (On = An.children) && On.sort(Ae), Be && (Bn = An.parent) && (Bn.value += An.value);
                  }), Mt;
                }
                return Ue.sort = function(nt) {
                  return arguments.length ? (Ae = nt, Ue) : Ae;
                }, Ue.children = function(nt) {
                  return arguments.length ? (Oe = nt, Ue) : Oe;
                }, Ue.value = function(nt) {
                  return arguments.length ? (Be = nt, Ue) : Be;
                }, Ue.revalue = function(nt) {
                  return Be && (No(nt, function(ht) {
                    ht.children && (ht.value = 0);
                  }), ro(nt, function(ht) {
                    var Et;
                    ht.children || (ht.value = +Be.call(Ue, ht, ht.depth) || 0), (Et = ht.parent) && (Et.value += ht.value);
                  })), nt;
                }, Ue;
              }, ee.layout.partition = function() {
                var Ae = ee.layout.hierarchy(), Oe = [1, 1];
                function Be(ht, Et, Mt, qt) {
                  var tn = ht.children;
                  if (ht.x = Et, ht.y = ht.depth * qt, ht.dx = Mt, ht.dy = qt, tn && (xn = tn.length)) {
                    var xn, An, On, Bn = -1;
                    for (Mt = ht.value ? Mt / ht.value : 0; ++Bn < xn; ) Be(An = tn[Bn], Et, On = An.value * Mt, qt), Et += On;
                  }
                }
                function Ue(ht) {
                  var Et = ht.children, Mt = 0;
                  if (Et && (qt = Et.length)) for (var qt, tn = -1; ++tn < qt; ) Mt = Math.max(Mt, Ue(Et[tn]));
                  return 1 + Mt;
                }
                function nt(ht, Et) {
                  var Mt = Ae.call(this, ht, Et);
                  return Be(Mt[0], 0, Oe[0], Oe[1] / Ue(Mt[0])), Mt;
                }
                return nt.size = function(ht) {
                  return arguments.length ? (Oe = ht, nt) : Oe;
                }, Jc(nt, Ae);
              }, ee.layout.pie = function() {
                var Ae = Number, Oe = Yo, Be = 0, Ue = wt, nt = 0;
                function ht(Et) {
                  var Mt, qt = Et.length, tn = Et.map(function(wr, Cr) {
                    return +Ae.call(ht, wr, Cr);
                  }), xn = +(typeof Be == "function" ? Be.apply(this, arguments) : Be), An = (typeof Ue == "function" ? Ue.apply(this, arguments) : Ue) - xn, On = Math.min(Math.abs(An) / qt, +(typeof nt == "function" ? nt.apply(this, arguments) : nt)), Bn = On * (An < 0 ? -1 : 1), qn = ee.sum(tn), Vn = qn ? (An - qt * Bn) / qn : 0, sr = ee.range(qt), Un = [];
                  return Oe != null && sr.sort(Oe === Yo ? function(wr, Cr) {
                    return tn[Cr] - tn[wr];
                  } : function(wr, Cr) {
                    return Oe(Et[wr], Et[Cr]);
                  }), sr.forEach(function(wr) {
                    Un[wr] = { data: Et[wr], value: Mt = tn[wr], startAngle: xn, endAngle: xn += Mt * Vn + Bn, padAngle: On };
                  }), Un;
                }
                return ht.value = function(Et) {
                  return arguments.length ? (Ae = Et, ht) : Ae;
                }, ht.sort = function(Et) {
                  return arguments.length ? (Oe = Et, ht) : Oe;
                }, ht.startAngle = function(Et) {
                  return arguments.length ? (Be = Et, ht) : Be;
                }, ht.endAngle = function(Et) {
                  return arguments.length ? (Ue = Et, ht) : Ue;
                }, ht.padAngle = function(Et) {
                  return arguments.length ? (nt = Et, ht) : nt;
                }, ht;
              };
              var Yo = {};
              function Ls(Ae) {
                return Ae.x;
              }
              function Ms(Ae) {
                return Ae.y;
              }
              function vi(Ae, Oe, Be) {
                Ae.y0 = Oe, Ae.y = Be;
              }
              ee.layout.stack = function() {
                var Ae = Xe, Oe = da, Be = Ji, Ue = vi, nt = Ls, ht = Ms;
                function Et(Mt, qt) {
                  if (!(On = Mt.length)) return Mt;
                  var tn = Mt.map(function(wr, Cr) {
                    return Ae.call(Et, wr, Cr);
                  }), xn = tn.map(function(wr) {
                    return wr.map(function(Cr, Tr) {
                      return [nt.call(Et, Cr, Tr), ht.call(Et, Cr, Tr)];
                    });
                  }), An = Oe.call(Et, xn, qt);
                  tn = ee.permute(tn, An), xn = ee.permute(xn, An);
                  var On, Bn, qn, Vn, sr = Be.call(Et, xn, qt), Un = tn[0].length;
                  for (qn = 0; qn < Un; ++qn) for (Ue.call(Et, tn[0][qn], Vn = sr[qn], xn[0][qn][1]), Bn = 1; Bn < On; ++Bn) Ue.call(Et, tn[Bn][qn], Vn += xn[Bn - 1][qn][1], xn[Bn][qn][1]);
                  return Mt;
                }
                return Et.values = function(Mt) {
                  return arguments.length ? (Ae = Mt, Et) : Ae;
                }, Et.order = function(Mt) {
                  return arguments.length ? (Oe = typeof Mt == "function" ? Mt : Ri.get(Mt) || da, Et) : Oe;
                }, Et.offset = function(Mt) {
                  return arguments.length ? (Be = typeof Mt == "function" ? Mt : ta.get(Mt) || Ji, Et) : Be;
                }, Et.x = function(Mt) {
                  return arguments.length ? (nt = Mt, Et) : nt;
                }, Et.y = function(Mt) {
                  return arguments.length ? (ht = Mt, Et) : ht;
                }, Et.out = function(Mt) {
                  return arguments.length ? (Ue = Mt, Et) : Ue;
                }, Et;
              };
              var Ri = ee.map({ "inside-out": function(Ae) {
                var Oe, Be, Ue = Ae.length, nt = Ae.map(Pa), ht = Ae.map(Ga), Et = ee.range(Ue).sort(function(An, On) {
                  return nt[An] - nt[On];
                }), Mt = 0, qt = 0, tn = [], xn = [];
                for (Oe = 0; Oe < Ue; ++Oe) Be = Et[Oe], Mt < qt ? (Mt += ht[Be], tn.push(Be)) : (qt += ht[Be], xn.push(Be));
                return xn.reverse().concat(tn);
              }, reverse: function(Ae) {
                return ee.range(Ae.length).reverse();
              }, default: da }), ta = ee.map({ silhouette: function(Ae) {
                var Oe, Be, Ue, nt = Ae.length, ht = Ae[0].length, Et = [], Mt = 0, qt = [];
                for (Be = 0; Be < ht; ++Be) {
                  for (Oe = 0, Ue = 0; Oe < nt; Oe++) Ue += Ae[Oe][Be][1];
                  Ue > Mt && (Mt = Ue), Et.push(Ue);
                }
                for (Be = 0; Be < ht; ++Be) qt[Be] = (Mt - Et[Be]) / 2;
                return qt;
              }, wiggle: function(Ae) {
                var Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn = Ae.length, An = Ae[0], On = An.length, Bn = [];
                for (Bn[0] = qt = tn = 0, Be = 1; Be < On; ++Be) {
                  for (Oe = 0, nt = 0; Oe < xn; ++Oe) nt += Ae[Oe][Be][1];
                  for (Oe = 0, ht = 0, Mt = An[Be][0] - An[Be - 1][0]; Oe < xn; ++Oe) {
                    for (Ue = 0, Et = (Ae[Oe][Be][1] - Ae[Oe][Be - 1][1]) / (2 * Mt); Ue < Oe; ++Ue) Et += (Ae[Ue][Be][1] - Ae[Ue][Be - 1][1]) / Mt;
                    ht += Et * Ae[Oe][Be][1];
                  }
                  Bn[Be] = qt -= nt ? ht / nt * Mt : 0, qt < tn && (tn = qt);
                }
                for (Be = 0; Be < On; ++Be) Bn[Be] -= tn;
                return Bn;
              }, expand: function(Ae) {
                var Oe, Be, Ue, nt = Ae.length, ht = Ae[0].length, Et = 1 / nt, Mt = [];
                for (Be = 0; Be < ht; ++Be) {
                  for (Oe = 0, Ue = 0; Oe < nt; Oe++) Ue += Ae[Oe][Be][1];
                  if (Ue) for (Oe = 0; Oe < nt; Oe++) Ae[Oe][Be][1] /= Ue;
                  else for (Oe = 0; Oe < nt; Oe++) Ae[Oe][Be][1] = Et;
                }
                for (Be = 0; Be < ht; ++Be) Mt[Be] = 0;
                return Mt;
              }, zero: Ji });
              function da(Ae) {
                return ee.range(Ae.length);
              }
              function Ji(Ae) {
                for (var Oe = -1, Be = Ae[0].length, Ue = []; ++Oe < Be; ) Ue[Oe] = 0;
                return Ue;
              }
              function Pa(Ae) {
                for (var Oe, Be = 1, Ue = 0, nt = Ae[0][1], ht = Ae.length; Be < ht; ++Be) (Oe = Ae[Be][1]) > nt && (Ue = Be, nt = Oe);
                return Ue;
              }
              function Ga(Ae) {
                return Ae.reduce(Ha, 0);
              }
              function Ha(Ae, Oe) {
                return Ae + Oe[1];
              }
              function Ba(Ae, Oe) {
                return ao(Ae, Math.ceil(Math.log(Oe.length) / Math.LN2 + 1));
              }
              function ao(Ae, Oe) {
                for (var Be = -1, Ue = +Ae[0], nt = (Ae[1] - Ue) / Oe, ht = []; ++Be <= Oe; ) ht[Be] = nt * Be + Ue;
                return ht;
              }
              function io(Ae) {
                return [ee.min(Ae), ee.max(Ae)];
              }
              function To(Ae, Oe) {
                return Ae.value - Oe.value;
              }
              function Mo(Ae, Oe) {
                var Be = Ae._pack_next;
                Ae._pack_next = Oe, Oe._pack_prev = Ae, Oe._pack_next = Be, Be._pack_prev = Oe;
              }
              function po(Ae, Oe) {
                Ae._pack_next = Oe, Oe._pack_prev = Ae;
              }
              function Vo(Ae, Oe) {
                var Be = Oe.x - Ae.x, Ue = Oe.y - Ae.y, nt = Ae.r + Oe.r;
                return 0.999 * nt * nt > Be * Be + Ue * Ue;
              }
              function Xo(Ae) {
                if ((Oe = Ae.children) && (qt = Oe.length)) {
                  var Oe, Be, Ue, nt, ht, Et, Mt, qt, tn = 1 / 0, xn = -1 / 0, An = 1 / 0, On = -1 / 0;
                  if (Oe.forEach(Go), (Be = Oe[0]).x = -Be.r, Be.y = 0, Cr(Be), qt > 1 && ((Ue = Oe[1]).x = Ue.r, Ue.y = 0, Cr(Ue), qt > 2)) for (Fo(Be, Ue, nt = Oe[2]), Cr(nt), Mo(Be, nt), Be._pack_prev = nt, Mo(nt, Ue), Ue = Be._pack_next, ht = 3; ht < qt; ht++) {
                    Fo(Be, Ue, nt = Oe[ht]);
                    var Bn = 0, qn = 1, Vn = 1;
                    for (Et = Ue._pack_next; Et !== Ue; Et = Et._pack_next, qn++) if (Vo(Et, nt)) {
                      Bn = 1;
                      break;
                    }
                    if (Bn == 1) for (Mt = Be._pack_prev; Mt !== Et._pack_prev && !Vo(Mt, nt); Mt = Mt._pack_prev, Vn++) ;
                    Bn ? (qn < Vn || qn == Vn && Ue.r < Be.r ? po(Be, Ue = Et) : po(Be = Mt, Ue), ht--) : (Mo(Be, nt), Ue = nt, Cr(nt));
                  }
                  var sr = (tn + xn) / 2, Un = (An + On) / 2, wr = 0;
                  for (ht = 0; ht < qt; ht++) (nt = Oe[ht]).x -= sr, nt.y -= Un, wr = Math.max(wr, nt.r + Math.sqrt(nt.x * nt.x + nt.y * nt.y));
                  Ae.r = wr, Oe.forEach(es);
                }
                function Cr(Tr) {
                  tn = Math.min(Tr.x - Tr.r, tn), xn = Math.max(Tr.x + Tr.r, xn), An = Math.min(Tr.y - Tr.r, An), On = Math.max(Tr.y + Tr.r, On);
                }
              }
              function Go(Ae) {
                Ae._pack_next = Ae._pack_prev = Ae;
              }
              function es(Ae) {
                delete Ae._pack_next, delete Ae._pack_prev;
              }
              function Ts(Ae, Oe, Be, Ue) {
                var nt = Ae.children;
                if (Ae.x = Oe += Ue * Ae.x, Ae.y = Be += Ue * Ae.y, Ae.r *= Ue, nt) for (var ht = -1, Et = nt.length; ++ht < Et; ) Ts(nt[ht], Oe, Be, Ue);
              }
              function Fo(Ae, Oe, Be) {
                var Ue = Ae.r + Be.r, nt = Oe.x - Ae.x, ht = Oe.y - Ae.y;
                if (Ue && (nt || ht)) {
                  var Et = Oe.r + Be.r, Mt = nt * nt + ht * ht, qt = 0.5 + ((Ue *= Ue) - (Et *= Et)) / (2 * Mt), tn = Math.sqrt(Math.max(0, 2 * Et * (Ue + Mt) - (Ue -= Mt) * Ue - Et * Et)) / (2 * Mt);
                  Be.x = Ae.x + qt * nt + tn * ht, Be.y = Ae.y + qt * ht - tn * nt;
                } else Be.x = Ae.x + Ue, Be.y = Ae.y;
              }
              function Oo(Ae, Oe) {
                return Ae.parent == Oe.parent ? 1 : 2;
              }
              function gs(Ae) {
                var Oe = Ae.children;
                return Oe.length ? Oe[0] : Ae.t;
              }
              function Bu(Ae) {
                var Oe, Be = Ae.children;
                return (Oe = Be.length) ? Be[Oe - 1] : Ae.t;
              }
              function Ql(Ae, Oe, Be) {
                var Ue = Be / (Oe.i - Ae.i);
                Oe.c -= Ue, Oe.s += Be, Ae.c += Ue, Oe.z += Be, Oe.m += Be;
              }
              function dl(Ae, Oe, Be) {
                return Ae.a.parent === Oe.parent ? Ae.a : Be;
              }
              function h1(Ae) {
                var Oe = Ae.children;
                return Oe && Oe.length ? h1(Oe[0]) : Ae;
              }
              function z1(Ae) {
                var Oe, Be = Ae.children;
                return Be && (Oe = Be.length) ? z1(Be[Oe - 1]) : Ae;
              }
              function J1(Ae) {
                return { x: Ae.x, y: Ae.y, dx: Ae.dx, dy: Ae.dy };
              }
              function Lu(Ae, Oe) {
                var Be = Ae.x + Oe[3], Ue = Ae.y + Oe[0], nt = Ae.dx - Oe[1] - Oe[3], ht = Ae.dy - Oe[0] - Oe[2];
                return nt < 0 && (Be += nt / 2, nt = 0), ht < 0 && (Ue += ht / 2, ht = 0), { x: Be, y: Ue, dx: nt, dy: ht };
              }
              function M1(Ae) {
                var Oe = Ae[0], Be = Ae[Ae.length - 1];
                return Oe < Be ? [Oe, Be] : [Be, Oe];
              }
              function ku(Ae) {
                return Ae.rangeExtent ? Ae.rangeExtent() : M1(Ae.range());
              }
              function Wu(Ae, Oe, Be, Ue) {
                var nt = Be(Ae[0], Ae[1]), ht = Ue(Oe[0], Oe[1]);
                return function(Et) {
                  return ht(nt(Et));
                };
              }
              function T1(Ae, Oe) {
                var Be, Ue = 0, nt = Ae.length - 1, ht = Ae[Ue], Et = Ae[nt];
                return Et < ht && (Be = Ue, Ue = nt, nt = Be, Be = ht, ht = Et, Et = Be), Ae[Ue] = Oe.floor(ht), Ae[nt] = Oe.ceil(Et), Ae;
              }
              function kl(Ae) {
                return Ae ? { floor: function(Oe) {
                  return Math.floor(Oe / Ae) * Ae;
                }, ceil: function(Oe) {
                  return Math.ceil(Oe / Ae) * Ae;
                } } : Yl;
              }
              ee.layout.histogram = function() {
                var Ae = !0, Oe = Number, Be = io, Ue = Ba;
                function nt(ht, Et) {
                  for (var Mt, qt, tn = [], xn = ht.map(Oe, this), An = Be.call(this, xn, Et), On = Ue.call(this, An, xn, Et), Bn = (Et = -1, xn.length), qn = On.length - 1, Vn = Ae ? 1 : 1 / Bn; ++Et < qn; ) (Mt = tn[Et] = []).dx = On[Et + 1] - (Mt.x = On[Et]), Mt.y = 0;
                  if (qn > 0) for (Et = -1; ++Et < Bn; ) (qt = xn[Et]) >= An[0] && qt <= An[1] && ((Mt = tn[ee.bisect(On, qt, 1, qn) - 1]).y += Vn, Mt.push(ht[Et]));
                  return tn;
                }
                return nt.value = function(ht) {
                  return arguments.length ? (Oe = ht, nt) : Oe;
                }, nt.range = function(ht) {
                  return arguments.length ? (Be = Ci(ht), nt) : Be;
                }, nt.bins = function(ht) {
                  return arguments.length ? (Ue = typeof ht == "number" ? function(Et) {
                    return ao(Et, ht);
                  } : Ci(ht), nt) : Ue;
                }, nt.frequency = function(ht) {
                  return arguments.length ? (Ae = !!ht, nt) : Ae;
                }, nt;
              }, ee.layout.pack = function() {
                var Ae, Oe = ee.layout.hierarchy().sort(To), Be = 0, Ue = [1, 1];
                function nt(ht, Et) {
                  var Mt = Oe.call(this, ht, Et), qt = Mt[0], tn = Ue[0], xn = Ue[1], An = Ae == null ? Math.sqrt : typeof Ae == "function" ? Ae : function() {
                    return Ae;
                  };
                  if (qt.x = qt.y = 0, ro(qt, function(Bn) {
                    Bn.r = +An(Bn.value);
                  }), ro(qt, Xo), Be) {
                    var On = Be * (Ae ? 1 : Math.max(2 * qt.r / tn, 2 * qt.r / xn)) / 2;
                    ro(qt, function(Bn) {
                      Bn.r += On;
                    }), ro(qt, Xo), ro(qt, function(Bn) {
                      Bn.r -= On;
                    });
                  }
                  return Ts(qt, tn / 2, xn / 2, Ae ? 1 : 1 / Math.max(2 * qt.r / tn, 2 * qt.r / xn)), Mt;
                }
                return nt.size = function(ht) {
                  return arguments.length ? (Ue = ht, nt) : Ue;
                }, nt.radius = function(ht) {
                  return arguments.length ? (Ae = ht == null || typeof ht == "function" ? ht : +ht, nt) : Ae;
                }, nt.padding = function(ht) {
                  return arguments.length ? (Be = +ht, nt) : Be;
                }, Jc(nt, Oe);
              }, ee.layout.tree = function() {
                var Ae = ee.layout.hierarchy().sort(null).value(null), Oe = Oo, Be = [1, 1], Ue = null;
                function nt(qt, tn) {
                  var xn = Ae.call(this, qt, tn), An = xn[0], On = function(Cr) {
                    for (var Tr, ii = { A: null, children: [Cr] }, wi = [ii]; (Tr = wi.pop()) != null; ) for (var Wi, Yi = Tr.children, ra = 0, ha = Yi.length; ra < ha; ++ra) wi.push((Yi[ra] = Wi = { _: Yi[ra], parent: Tr, children: (Wi = Yi[ra].children) && Wi.slice() || [], A: null, a: null, z: 0, m: 0, c: 0, s: 0, t: null, i: ra }).a = Wi);
                    return ii.children[0];
                  }(An);
                  if (ro(On, ht), On.parent.m = -On.z, No(On, Et), Ue) No(An, Mt);
                  else {
                    var Bn = An, qn = An, Vn = An;
                    No(An, function(Cr) {
                      Cr.x < Bn.x && (Bn = Cr), Cr.x > qn.x && (qn = Cr), Cr.depth > Vn.depth && (Vn = Cr);
                    });
                    var sr = Oe(Bn, qn) / 2 - Bn.x, Un = Be[0] / (qn.x + Oe(qn, Bn) / 2 + sr), wr = Be[1] / (Vn.depth || 1);
                    No(An, function(Cr) {
                      Cr.x = (Cr.x + sr) * Un, Cr.y = Cr.depth * wr;
                    });
                  }
                  return xn;
                }
                function ht(qt) {
                  var tn = qt.children, xn = qt.parent.children, An = qt.i ? xn[qt.i - 1] : null;
                  if (tn.length) {
                    (function(Bn) {
                      for (var qn, Vn = 0, sr = 0, Un = Bn.children, wr = Un.length; --wr >= 0; ) (qn = Un[wr]).z += Vn, qn.m += Vn, Vn += qn.s + (sr += qn.c);
                    })(qt);
                    var On = (tn[0].z + tn[tn.length - 1].z) / 2;
                    An ? (qt.z = An.z + Oe(qt._, An._), qt.m = qt.z - On) : qt.z = On;
                  } else An && (qt.z = An.z + Oe(qt._, An._));
                  qt.parent.A = function(Bn, qn, Vn) {
                    if (qn) {
                      for (var sr, Un = Bn, wr = Bn, Cr = qn, Tr = Un.parent.children[0], ii = Un.m, wi = wr.m, Wi = Cr.m, Yi = Tr.m; Cr = Bu(Cr), Un = gs(Un), Cr && Un; ) Tr = gs(Tr), (wr = Bu(wr)).a = Bn, (sr = Cr.z + Wi - Un.z - ii + Oe(Cr._, Un._)) > 0 && (Ql(dl(Cr, Bn, Vn), Bn, sr), ii += sr, wi += sr), Wi += Cr.m, ii += Un.m, Yi += Tr.m, wi += wr.m;
                      Cr && !Bu(wr) && (wr.t = Cr, wr.m += Wi - wi), Un && !gs(Tr) && (Tr.t = Un, Tr.m += ii - Yi, Vn = Bn);
                    }
                    return Vn;
                  }(qt, An, qt.parent.A || xn[0]);
                }
                function Et(qt) {
                  qt._.x = qt.z + qt.parent.m, qt.m += qt.parent.m;
                }
                function Mt(qt) {
                  qt.x *= Be[0], qt.y = qt.depth * Be[1];
                }
                return nt.separation = function(qt) {
                  return arguments.length ? (Oe = qt, nt) : Oe;
                }, nt.size = function(qt) {
                  return arguments.length ? (Ue = (Be = qt) == null ? Mt : null, nt) : Ue ? null : Be;
                }, nt.nodeSize = function(qt) {
                  return arguments.length ? (Ue = (Be = qt) == null ? null : Mt, nt) : Ue ? Be : null;
                }, Jc(nt, Ae);
              }, ee.layout.cluster = function() {
                var Ae = ee.layout.hierarchy().sort(null).value(null), Oe = Oo, Be = [1, 1], Ue = !1;
                function nt(ht, Et) {
                  var Mt, qt = Ae.call(this, ht, Et), tn = qt[0], xn = 0;
                  ro(tn, function(Vn) {
                    var sr = Vn.children;
                    sr && sr.length ? (Vn.x = function(Un) {
                      return Un.reduce(function(wr, Cr) {
                        return wr + Cr.x;
                      }, 0) / Un.length;
                    }(sr), Vn.y = function(Un) {
                      return 1 + ee.max(Un, function(wr) {
                        return wr.y;
                      });
                    }(sr)) : (Vn.x = Mt ? xn += Oe(Vn, Mt) : 0, Vn.y = 0, Mt = Vn);
                  });
                  var An = h1(tn), On = z1(tn), Bn = An.x - Oe(An, On) / 2, qn = On.x + Oe(On, An) / 2;
                  return ro(tn, Ue ? function(Vn) {
                    Vn.x = (Vn.x - tn.x) * Be[0], Vn.y = (tn.y - Vn.y) * Be[1];
                  } : function(Vn) {
                    Vn.x = (Vn.x - Bn) / (qn - Bn) * Be[0], Vn.y = (1 - (tn.y ? Vn.y / tn.y : 1)) * Be[1];
                  }), qt;
                }
                return nt.separation = function(ht) {
                  return arguments.length ? (Oe = ht, nt) : Oe;
                }, nt.size = function(ht) {
                  return arguments.length ? (Ue = (Be = ht) == null, nt) : Ue ? null : Be;
                }, nt.nodeSize = function(ht) {
                  return arguments.length ? (Ue = (Be = ht) != null, nt) : Ue ? Be : null;
                }, Jc(nt, Ae);
              }, ee.layout.treemap = function() {
                var Ae, Oe = ee.layout.hierarchy(), Be = Math.round, Ue = [1, 1], nt = null, ht = J1, Et = !1, Mt = "squarify", qt = 0.5 * (1 + Math.sqrt(5));
                function tn(Vn, sr) {
                  for (var Un, wr, Cr = -1, Tr = Vn.length; ++Cr < Tr; ) wr = (Un = Vn[Cr]).value * (sr < 0 ? 0 : sr), Un.area = isNaN(wr) || wr <= 0 ? 0 : wr;
                }
                function xn(Vn) {
                  var sr = Vn.children;
                  if (sr && sr.length) {
                    var Un, wr, Cr, Tr = ht(Vn), ii = [], wi = sr.slice(), Wi = 1 / 0, Yi = Mt === "slice" ? Tr.dx : Mt === "dice" ? Tr.dy : Mt === "slice-dice" ? 1 & Vn.depth ? Tr.dy : Tr.dx : Math.min(Tr.dx, Tr.dy);
                    for (tn(wi, Tr.dx * Tr.dy / Vn.value), ii.area = 0; (Cr = wi.length) > 0; ) ii.push(Un = wi[Cr - 1]), ii.area += Un.area, Mt !== "squarify" || (wr = On(ii, Yi)) <= Wi ? (wi.pop(), Wi = wr) : (ii.area -= ii.pop().area, Bn(ii, Yi, Tr, !1), Yi = Math.min(Tr.dx, Tr.dy), ii.length = ii.area = 0, Wi = 1 / 0);
                    ii.length && (Bn(ii, Yi, Tr, !0), ii.length = ii.area = 0), sr.forEach(xn);
                  }
                }
                function An(Vn) {
                  var sr = Vn.children;
                  if (sr && sr.length) {
                    var Un, wr = ht(Vn), Cr = sr.slice(), Tr = [];
                    for (tn(Cr, wr.dx * wr.dy / Vn.value), Tr.area = 0; Un = Cr.pop(); ) Tr.push(Un), Tr.area += Un.area, Un.z != null && (Bn(Tr, Un.z ? wr.dx : wr.dy, wr, !Cr.length), Tr.length = Tr.area = 0);
                    sr.forEach(An);
                  }
                }
                function On(Vn, sr) {
                  for (var Un, wr = Vn.area, Cr = 0, Tr = 1 / 0, ii = -1, wi = Vn.length; ++ii < wi; ) (Un = Vn[ii].area) && (Un < Tr && (Tr = Un), Un > Cr && (Cr = Un));
                  return sr *= sr, (wr *= wr) ? Math.max(sr * Cr * qt / wr, wr / (sr * Tr * qt)) : 1 / 0;
                }
                function Bn(Vn, sr, Un, wr) {
                  var Cr, Tr = -1, ii = Vn.length, wi = Un.x, Wi = Un.y, Yi = sr ? Be(Vn.area / sr) : 0;
                  if (sr == Un.dx) {
                    for ((wr || Yi > Un.dy) && (Yi = Un.dy); ++Tr < ii; ) (Cr = Vn[Tr]).x = wi, Cr.y = Wi, Cr.dy = Yi, wi += Cr.dx = Math.min(Un.x + Un.dx - wi, Yi ? Be(Cr.area / Yi) : 0);
                    Cr.z = !0, Cr.dx += Un.x + Un.dx - wi, Un.y += Yi, Un.dy -= Yi;
                  } else {
                    for ((wr || Yi > Un.dx) && (Yi = Un.dx); ++Tr < ii; ) (Cr = Vn[Tr]).x = wi, Cr.y = Wi, Cr.dx = Yi, Wi += Cr.dy = Math.min(Un.y + Un.dy - Wi, Yi ? Be(Cr.area / Yi) : 0);
                    Cr.z = !1, Cr.dy += Un.y + Un.dy - Wi, Un.x += Yi, Un.dx -= Yi;
                  }
                }
                function qn(Vn) {
                  var sr = Ae || Oe(Vn), Un = sr[0];
                  return Un.x = Un.y = 0, Un.value ? (Un.dx = Ue[0], Un.dy = Ue[1]) : Un.dx = Un.dy = 0, Ae && Oe.revalue(Un), tn([Un], Un.dx * Un.dy / Un.value), (Ae ? An : xn)(Un), Et && (Ae = sr), sr;
                }
                return qn.size = function(Vn) {
                  return arguments.length ? (Ue = Vn, qn) : Ue;
                }, qn.padding = function(Vn) {
                  if (!arguments.length) return nt;
                  function sr(wr) {
                    return Lu(wr, Vn);
                  }
                  var Un;
                  return ht = (nt = Vn) == null ? J1 : (Un = typeof Vn) == "function" ? function(wr) {
                    var Cr = Vn.call(qn, wr, wr.depth);
                    return Cr == null ? J1(wr) : Lu(wr, typeof Cr == "number" ? [Cr, Cr, Cr, Cr] : Cr);
                  } : (Un === "number" && (Vn = [Vn, Vn, Vn, Vn]), sr), qn;
                }, qn.round = function(Vn) {
                  return arguments.length ? (Be = Vn ? Math.round : Number, qn) : Be != Number;
                }, qn.sticky = function(Vn) {
                  return arguments.length ? (Et = Vn, Ae = null, qn) : Et;
                }, qn.ratio = function(Vn) {
                  return arguments.length ? (qt = Vn, qn) : qt;
                }, qn.mode = function(Vn) {
                  return arguments.length ? (Mt = Vn + "", qn) : Mt;
                }, Jc(qn, Oe);
              }, ee.random = { normal: function(Ae, Oe) {
                var Be = arguments.length;
                return Be < 2 && (Oe = 1), Be < 1 && (Ae = 0), function() {
                  var Ue, nt, ht;
                  do
                    ht = (Ue = 2 * Math.random() - 1) * Ue + (nt = 2 * Math.random() - 1) * nt;
                  while (!ht || ht > 1);
                  return Ae + Oe * Ue * Math.sqrt(-2 * Math.log(ht) / ht);
                };
              }, logNormal: function() {
                var Ae = ee.random.normal.apply(ee, arguments);
                return function() {
                  return Math.exp(Ae());
                };
              }, bates: function(Ae) {
                var Oe = ee.random.irwinHall(Ae);
                return function() {
                  return Oe() / Ae;
                };
              }, irwinHall: function(Ae) {
                return function() {
                  for (var Oe = 0, Be = 0; Be < Ae; Be++) Oe += Math.random();
                  return Oe;
                };
              } }, ee.scale = {};
              var Yl = { floor: Xe, ceil: Xe };
              function i1(Ae, Oe, Be, Ue) {
                var nt = [], ht = [], Et = 0, Mt = Math.min(Ae.length, Oe.length) - 1;
                for (Ae[Mt] < Ae[0] && (Ae = Ae.slice().reverse(), Oe = Oe.slice().reverse()); ++Et <= Mt; ) nt.push(Be(Ae[Et - 1], Ae[Et])), ht.push(Ue(Oe[Et - 1], Oe[Et]));
                return function(qt) {
                  var tn = ee.bisect(Ae, qt, 1, Mt) - 1;
                  return ht[tn](nt[tn](qt));
                };
              }
              function ic(Ae, Oe, Be, Ue) {
                var nt, ht;
                function Et() {
                  var qt = Math.min(Ae.length, Oe.length) > 2 ? i1 : Wu, tn = Ue ? Kl : Hl;
                  return nt = qt(Ae, Oe, tn, Be), ht = qt(Oe, Ae, tn, Gl), Mt;
                }
                function Mt(qt) {
                  return nt(qt);
                }
                return Mt.invert = function(qt) {
                  return ht(qt);
                }, Mt.domain = function(qt) {
                  return arguments.length ? (Ae = qt.map(Number), Et()) : Ae;
                }, Mt.range = function(qt) {
                  return arguments.length ? (Oe = qt, Et()) : Oe;
                }, Mt.rangeRound = function(qt) {
                  return Mt.range(qt).interpolate(Fu);
                }, Mt.clamp = function(qt) {
                  return arguments.length ? (Ue = qt, Et()) : Ue;
                }, Mt.interpolate = function(qt) {
                  return arguments.length ? (Be = qt, Et()) : Be;
                }, Mt.ticks = function(qt) {
                  return Zl(Ae, qt);
                }, Mt.tickFormat = function(qt, tn) {
                  return d3_scale_linearTickFormat(Ae, qt, tn);
                }, Mt.nice = function(qt) {
                  return xu(Ae, qt), Et();
                }, Mt.copy = function() {
                  return ic(Ae, Oe, Be, Ue);
                }, Et();
              }
              function iu(Ae, Oe) {
                return ee.rebind(Ae, Oe, "range", "rangeRound", "interpolate", "clamp");
              }
              function xu(Ae, Oe) {
                return T1(Ae, kl(vc(Ae, Oe)[2])), T1(Ae, kl(vc(Ae, Oe)[2])), Ae;
              }
              function vc(Ae, Oe) {
                Oe == null && (Oe = 10);
                var Be = M1(Ae), Ue = Be[1] - Be[0], nt = Math.pow(10, Math.floor(Math.log(Ue / Oe) / Math.LN10)), ht = Oe / Ue * nt;
                return ht <= 0.15 ? nt *= 10 : ht <= 0.35 ? nt *= 5 : ht <= 0.75 && (nt *= 2), Be[0] = Math.ceil(Be[0] / nt) * nt, Be[1] = Math.floor(Be[1] / nt) * nt + 0.5 * nt, Be[2] = nt, Be;
              }
              function Zl(Ae, Oe) {
                return ee.range.apply(ee, vc(Ae, Oe));
              }
              function Ec(Ae, Oe, Be, Ue) {
                function nt(Mt) {
                  return (Be ? Math.log(Mt < 0 ? 0 : Mt) : -Math.log(Mt > 0 ? 0 : -Mt)) / Math.log(Oe);
                }
                function ht(Mt) {
                  return Be ? Math.pow(Oe, Mt) : -Math.pow(Oe, -Mt);
                }
                function Et(Mt) {
                  return Ae(nt(Mt));
                }
                return Et.invert = function(Mt) {
                  return ht(Ae.invert(Mt));
                }, Et.domain = function(Mt) {
                  return arguments.length ? (Be = Mt[0] >= 0, Ae.domain((Ue = Mt.map(Number)).map(nt)), Et) : Ue;
                }, Et.base = function(Mt) {
                  return arguments.length ? (Oe = +Mt, Ae.domain(Ue.map(nt)), Et) : Oe;
                }, Et.nice = function() {
                  var Mt = T1(Ue.map(nt), Be ? Math : wu);
                  return Ae.domain(Mt), Ue = Mt.map(ht), Et;
                }, Et.ticks = function() {
                  var Mt = M1(Ue), qt = [], tn = Mt[0], xn = Mt[1], An = Math.floor(nt(tn)), On = Math.ceil(nt(xn)), Bn = Oe % 1 ? 2 : Oe;
                  if (isFinite(On - An)) {
                    if (Be) {
                      for (; An < On; An++) for (var qn = 1; qn < Bn; qn++) qt.push(ht(An) * qn);
                      qt.push(ht(An));
                    } else for (qt.push(ht(An)); An++ < On; ) for (qn = Bn - 1; qn > 0; qn--) qt.push(ht(An) * qn);
                    for (An = 0; qt[An] < tn; An++) ;
                    for (On = qt.length; qt[On - 1] > xn; On--) ;
                    qt = qt.slice(An, On);
                  }
                  return qt;
                }, Et.copy = function() {
                  return Ec(Ae.copy(), Oe, Be, Ue);
                }, iu(Et, Ae);
              }
              ee.scale.linear = function() {
                return ic([0, 1], [0, 1], Gl, !1);
              }, ee.scale.log = function() {
                return Ec(ee.scale.linear().domain([0, 1]), 10, !0, [1, 10]);
              };
              var wu = { floor: function(Ae) {
                return -Math.ceil(-Ae);
              }, ceil: function(Ae) {
                return -Math.floor(-Ae);
              } };
              function Pc(Ae, Oe, Be) {
                var Ue = du(Oe), nt = du(1 / Oe);
                function ht(Et) {
                  return Ae(Ue(Et));
                }
                return ht.invert = function(Et) {
                  return nt(Ae.invert(Et));
                }, ht.domain = function(Et) {
                  return arguments.length ? (Ae.domain((Be = Et.map(Number)).map(Ue)), ht) : Be;
                }, ht.ticks = function(Et) {
                  return Zl(Be, Et);
                }, ht.tickFormat = function(Et, Mt) {
                  return d3_scale_linearTickFormat(Be, Et, Mt);
                }, ht.nice = function(Et) {
                  return ht.domain(xu(Be, Et));
                }, ht.exponent = function(Et) {
                  return arguments.length ? (Ue = du(Oe = Et), nt = du(1 / Oe), Ae.domain(Be.map(Ue)), ht) : Oe;
                }, ht.copy = function() {
                  return Pc(Ae.copy(), Oe, Be);
                }, iu(ht, Ae);
              }
              function du(Ae) {
                return function(Oe) {
                  return Oe < 0 ? -Math.pow(-Oe, Ae) : Math.pow(Oe, Ae);
                };
              }
              function Ac(Ae, Oe) {
                var Be, Ue, nt;
                function ht(Mt) {
                  return Ue[((Be.get(Mt) || (Oe.t === "range" ? Be.set(Mt, Ae.push(Mt)) : NaN)) - 1) % Ue.length];
                }
                function Et(Mt, qt) {
                  return ee.range(Ae.length).map(function(tn) {
                    return Mt + qt * tn;
                  });
                }
                return ht.domain = function(Mt) {
                  if (!arguments.length) return Ae;
                  Ae = [], Be = new Ee();
                  for (var qt, tn = -1, xn = Mt.length; ++tn < xn; ) Be.has(qt = Mt[tn]) || Be.set(qt, Ae.push(qt));
                  return ht[Oe.t].apply(ht, Oe.a);
                }, ht.range = function(Mt) {
                  return arguments.length ? (Ue = Mt, nt = 0, Oe = { t: "range", a: arguments }, ht) : Ue;
                }, ht.rangePoints = function(Mt, qt) {
                  arguments.length < 2 && (qt = 0);
                  var tn = Mt[0], xn = Mt[1], An = Ae.length < 2 ? (tn = (tn + xn) / 2, 0) : (xn - tn) / (Ae.length - 1 + qt);
                  return Ue = Et(tn + An * qt / 2, An), nt = 0, Oe = { t: "rangePoints", a: arguments }, ht;
                }, ht.rangeRoundPoints = function(Mt, qt) {
                  arguments.length < 2 && (qt = 0);
                  var tn = Mt[0], xn = Mt[1], An = Ae.length < 2 ? (tn = xn = Math.round((tn + xn) / 2), 0) : (xn - tn) / (Ae.length - 1 + qt) | 0;
                  return Ue = Et(tn + Math.round(An * qt / 2 + (xn - tn - (Ae.length - 1 + qt) * An) / 2), An), nt = 0, Oe = { t: "rangeRoundPoints", a: arguments }, ht;
                }, ht.rangeBands = function(Mt, qt, tn) {
                  arguments.length < 2 && (qt = 0), arguments.length < 3 && (tn = qt);
                  var xn = Mt[1] < Mt[0], An = Mt[xn - 0], On = (Mt[1 - xn] - An) / (Ae.length - qt + 2 * tn);
                  return Ue = Et(An + On * tn, On), xn && Ue.reverse(), nt = On * (1 - qt), Oe = { t: "rangeBands", a: arguments }, ht;
                }, ht.rangeRoundBands = function(Mt, qt, tn) {
                  arguments.length < 2 && (qt = 0), arguments.length < 3 && (tn = qt);
                  var xn = Mt[1] < Mt[0], An = Mt[xn - 0], On = Mt[1 - xn], Bn = Math.floor((On - An) / (Ae.length - qt + 2 * tn));
                  return Ue = Et(An + Math.round((On - An - (Ae.length - qt) * Bn) / 2), Bn), xn && Ue.reverse(), nt = Math.round(Bn * (1 - qt)), Oe = { t: "rangeRoundBands", a: arguments }, ht;
                }, ht.rangeBand = function() {
                  return nt;
                }, ht.rangeExtent = function() {
                  return M1(Oe.a[0]);
                }, ht.copy = function() {
                  return Ac(Ae, Oe);
                }, ht.domain(Ae);
              }
              ee.scale.pow = function() {
                return Pc(ee.scale.linear(), 1, [0, 1]);
              }, ee.scale.sqrt = function() {
                return ee.scale.pow().exponent(0.5);
              }, ee.scale.ordinal = function() {
                return Ac([], { t: "range", a: [[]] });
              }, ee.scale.category10 = function() {
                return ee.scale.ordinal().range(yu);
              }, ee.scale.category20 = function() {
                return ee.scale.ordinal().range(au);
              }, ee.scale.category20b = function() {
                return ee.scale.ordinal().range(Qc);
              }, ee.scale.category20c = function() {
                return ee.scale.ordinal().range(Cc);
              };
              var yu = [2062260, 16744206, 2924588, 14034728, 9725885, 9197131, 14907330, 8355711, 12369186, 1556175].map(Zr), au = [2062260, 11454440, 16744206, 16759672, 2924588, 10018698, 14034728, 16750742, 9725885, 12955861, 9197131, 12885140, 14907330, 16234194, 8355711, 13092807, 12369186, 14408589, 1556175, 10410725].map(Zr), Qc = [3750777, 5395619, 7040719, 10264286, 6519097, 9216594, 11915115, 13556636, 9202993, 12426809, 15186514, 15190932, 8666169, 11356490, 14049643, 15177372, 8077683, 10834324, 13528509, 14589654].map(Zr), Cc = [3244733, 7057110, 10406625, 13032431, 15095053, 16616764, 16625259, 16634018, 3253076, 7652470, 10607003, 13101504, 7695281, 10394312, 12369372, 14342891, 6513507, 9868950, 12434877, 14277081].map(Zr);
              function Ol(Ae, Oe) {
                var Be;
                function Ue() {
                  var ht = 0, Et = Oe.length;
                  for (Be = []; ++ht < Et; ) Be[ht - 1] = ee.quantile(Ae, ht / Et);
                  return nt;
                }
                function nt(ht) {
                  if (!isNaN(ht = +ht)) return Oe[ee.bisect(Be, ht)];
                }
                return nt.domain = function(ht) {
                  return arguments.length ? (Ae = ht.map(ye).filter(ge).sort(pe), Ue()) : Ae;
                }, nt.range = function(ht) {
                  return arguments.length ? (Oe = ht, Ue()) : Oe;
                }, nt.quantiles = function() {
                  return Be;
                }, nt.invertExtent = function(ht) {
                  return (ht = Oe.indexOf(ht)) < 0 ? [NaN, NaN] : [ht > 0 ? Be[ht - 1] : Ae[0], ht < Be.length ? Be[ht] : Ae[Ae.length - 1]];
                }, nt.copy = function() {
                  return Ol(Ae, Oe);
                }, Ue();
              }
              function uc(Ae, Oe, Be) {
                var Ue, nt;
                function ht(Mt) {
                  return Be[Math.max(0, Math.min(nt, Math.floor(Ue * (Mt - Ae))))];
                }
                function Et() {
                  return Ue = Be.length / (Oe - Ae), nt = Be.length - 1, ht;
                }
                return ht.domain = function(Mt) {
                  return arguments.length ? (Ae = +Mt[0], Oe = +Mt[Mt.length - 1], Et()) : [Ae, Oe];
                }, ht.range = function(Mt) {
                  return arguments.length ? (Be = Mt, Et()) : Be;
                }, ht.invertExtent = function(Mt) {
                  return [Mt = (Mt = Be.indexOf(Mt)) < 0 ? NaN : Mt / Ue + Ae, Mt + 1 / Ue];
                }, ht.copy = function() {
                  return uc(Ae, Oe, Be);
                }, Et();
              }
              function cs(Ae, Oe) {
                function Be(Ue) {
                  if (Ue <= Ue) return Oe[ee.bisect(Ae, Ue)];
                }
                return Be.domain = function(Ue) {
                  return arguments.length ? (Ae = Ue, Be) : Ae;
                }, Be.range = function(Ue) {
                  return arguments.length ? (Oe = Ue, Be) : Oe;
                }, Be.invertExtent = function(Ue) {
                  return Ue = Oe.indexOf(Ue), [Ae[Ue - 1], Ae[Ue]];
                }, Be.copy = function() {
                  return cs(Ae, Oe);
                }, Be;
              }
              function Mu(Ae) {
                function Oe(Be) {
                  return +Be;
                }
                return Oe.invert = Oe, Oe.domain = Oe.range = function(Be) {
                  return arguments.length ? (Ae = Be.map(Oe), Oe) : Ae;
                }, Oe.ticks = function(Be) {
                  return Zl(Ae, Be);
                }, Oe.tickFormat = function(Be, Ue) {
                  return d3_scale_linearTickFormat(Ae, Be, Ue);
                }, Oe.copy = function() {
                  return Mu(Ae);
                }, Oe;
              }
              function Hu() {
                return 0;
              }
              ee.scale.quantile = function() {
                return Ol([], []);
              }, ee.scale.quantize = function() {
                return uc(0, 1, [0, 1]);
              }, ee.scale.threshold = function() {
                return cs([0.5], [0, 1]);
              }, ee.scale.identity = function() {
                return Mu([0, 1]);
              }, ee.svg = {}, ee.svg.arc = function() {
                var Ae = Gc, Oe = Su, Be = Hu, Ue = ou, nt = Lc, ht = Do, Et = Ru;
                function Mt() {
                  var tn = Math.max(0, +Ae.apply(this, arguments)), xn = Math.max(0, +Oe.apply(this, arguments)), An = nt.apply(this, arguments) - At, On = ht.apply(this, arguments) - At, Bn = Math.abs(On - An), qn = An > On ? 0 : 1;
                  if (xn < tn && (Vn = xn, xn = tn, tn = Vn), Bn >= Bt) return qt(xn, qn) + (tn ? qt(tn, 1 - qn) : "") + "Z";
                  var Vn, sr, Un, wr, Cr, Tr, ii, wi, Wi, Yi, ra, ha, ya = 0, va = 0, qa = [];
                  if ((wr = (+Et.apply(this, arguments) || 0) / 2) && (Un = Ue === ou ? Math.sqrt(tn * tn + xn * xn) : +Ue.apply(this, arguments), qn || (va *= -1), xn && (va = un(Un / xn * Math.sin(wr))), tn && (ya = un(Un / tn * Math.sin(wr)))), xn) {
                    Cr = xn * Math.cos(An + va), Tr = xn * Math.sin(An + va), ii = xn * Math.cos(On - va), wi = xn * Math.sin(On - va);
                    var Aa = Math.abs(On - An - 2 * va) <= pt ? 0 : 1;
                    if (va && dc(Cr, Tr, ii, wi) === qn ^ Aa) {
                      var Xa = (An + On) / 2;
                      Cr = xn * Math.cos(Xa), Tr = xn * Math.sin(Xa), ii = wi = null;
                    }
                  } else Cr = Tr = 0;
                  if (tn) {
                    Wi = tn * Math.cos(On - ya), Yi = tn * Math.sin(On - ya), ra = tn * Math.cos(An + ya), ha = tn * Math.sin(An + ya);
                    var Oa = Math.abs(An - On + 2 * ya) <= pt ? 0 : 1;
                    if (ya && dc(Wi, Yi, ra, ha) === 1 - qn ^ Oa) {
                      var Ya = (An + On) / 2;
                      Wi = tn * Math.cos(Ya), Yi = tn * Math.sin(Ya), ra = ha = null;
                    }
                  } else Wi = Yi = 0;
                  if (Bn > rt && (Vn = Math.min(Math.abs(xn - tn) / 2, +Be.apply(this, arguments))) > 1e-3) {
                    sr = tn < xn ^ qn ? 0 : 1;
                    var Wo = Vn, Ja = Vn;
                    if (Bn < pt) {
                      var Uo = ra == null ? [Wi, Yi] : ii == null ? [Cr, Tr] : rr([Cr, Tr], [ra, ha], [ii, wi], [Wi, Yi]), ys = Cr - Uo[0], il = Tr - Uo[1], Is = ii - Uo[0], ml = wi - Uo[1], nl = 1 / Math.sin(Math.acos((ys * Is + il * ml) / (Math.sqrt(ys * ys + il * il) * Math.sqrt(Is * Is + ml * ml))) / 2), Il = Math.sqrt(Uo[0] * Uo[0] + Uo[1] * Uo[1]);
                      Ja = Math.min(Vn, (tn - Il) / (nl - 1)), Wo = Math.min(Vn, (xn - Il) / (nl + 1));
                    }
                    if (ii != null) {
                      var Fl = vl(ra == null ? [Wi, Yi] : [ra, ha], [Cr, Tr], xn, Wo, qn), qc = vl([ii, wi], [Wi, Yi], xn, Wo, qn);
                      Vn === Wo ? qa.push("M", Fl[0], "A", Wo, ",", Wo, " 0 0,", sr, " ", Fl[1], "A", xn, ",", xn, " 0 ", 1 - qn ^ dc(Fl[1][0], Fl[1][1], qc[1][0], qc[1][1]), ",", qn, " ", qc[1], "A", Wo, ",", Wo, " 0 0,", sr, " ", qc[0]) : qa.push("M", Fl[0], "A", Wo, ",", Wo, " 0 1,", sr, " ", qc[0]);
                    } else qa.push("M", Cr, ",", Tr);
                    if (ra != null) {
                      var ql = vl([Cr, Tr], [ra, ha], tn, -Ja, qn), jc = vl([Wi, Yi], ii == null ? [Cr, Tr] : [ii, wi], tn, -Ja, qn);
                      Vn === Ja ? qa.push("L", jc[0], "A", Ja, ",", Ja, " 0 0,", sr, " ", jc[1], "A", tn, ",", tn, " 0 ", qn ^ dc(jc[1][0], jc[1][1], ql[1][0], ql[1][1]), ",", 1 - qn, " ", ql[1], "A", Ja, ",", Ja, " 0 0,", sr, " ", ql[0]) : qa.push("L", jc[0], "A", Ja, ",", Ja, " 0 0,", sr, " ", ql[0]);
                    } else qa.push("L", Wi, ",", Yi);
                  } else qa.push("M", Cr, ",", Tr), ii != null && qa.push("A", xn, ",", xn, " 0 ", Aa, ",", qn, " ", ii, ",", wi), qa.push("L", Wi, ",", Yi), ra != null && qa.push("A", tn, ",", tn, " 0 ", Oa, ",", 1 - qn, " ", ra, ",", ha);
                  return qa.push("Z"), qa.join("");
                }
                function qt(tn, xn) {
                  return "M0," + tn + "A" + tn + "," + tn + " 0 1," + xn + " 0," + -tn + "A" + tn + "," + tn + " 0 1," + xn + " 0," + tn;
                }
                return Mt.innerRadius = function(tn) {
                  return arguments.length ? (Ae = Ci(tn), Mt) : Ae;
                }, Mt.outerRadius = function(tn) {
                  return arguments.length ? (Oe = Ci(tn), Mt) : Oe;
                }, Mt.cornerRadius = function(tn) {
                  return arguments.length ? (Be = Ci(tn), Mt) : Be;
                }, Mt.padRadius = function(tn) {
                  return arguments.length ? (Ue = tn == ou ? ou : Ci(tn), Mt) : Ue;
                }, Mt.startAngle = function(tn) {
                  return arguments.length ? (nt = Ci(tn), Mt) : nt;
                }, Mt.endAngle = function(tn) {
                  return arguments.length ? (ht = Ci(tn), Mt) : ht;
                }, Mt.padAngle = function(tn) {
                  return arguments.length ? (Et = Ci(tn), Mt) : Et;
                }, Mt.centroid = function() {
                  var tn = (+Ae.apply(this, arguments) + +Oe.apply(this, arguments)) / 2, xn = (+nt.apply(this, arguments) + +ht.apply(this, arguments)) / 2 - At;
                  return [Math.cos(xn) * tn, Math.sin(xn) * tn];
                }, Mt;
              };
              var ou = "auto";
              function Gc(Ae) {
                return Ae.innerRadius;
              }
              function Su(Ae) {
                return Ae.outerRadius;
              }
              function Lc(Ae) {
                return Ae.startAngle;
              }
              function Do(Ae) {
                return Ae.endAngle;
              }
              function Ru(Ae) {
                return Ae && Ae.padAngle;
              }
              function dc(Ae, Oe, Be, Ue) {
                return (Ae - Be) * Oe - (Oe - Ue) * Ae > 0 ? 0 : 1;
              }
              function vl(Ae, Oe, Be, Ue, nt) {
                var ht = Ae[0] - Oe[0], Et = Ae[1] - Oe[1], Mt = (nt ? Ue : -Ue) / Math.sqrt(ht * ht + Et * Et), qt = Mt * Et, tn = -Mt * ht, xn = Ae[0] + qt, An = Ae[1] + tn, On = Oe[0] + qt, Bn = Oe[1] + tn, qn = (xn + On) / 2, Vn = (An + Bn) / 2, sr = On - xn, Un = Bn - An, wr = sr * sr + Un * Un, Cr = Be - Ue, Tr = xn * Bn - On * An, ii = (Un < 0 ? -1 : 1) * Math.sqrt(Math.max(0, Cr * Cr * wr - Tr * Tr)), wi = (Tr * Un - sr * ii) / wr, Wi = (-Tr * sr - Un * ii) / wr, Yi = (Tr * Un + sr * ii) / wr, ra = (-Tr * sr + Un * ii) / wr, ha = wi - qn, ya = Wi - Vn, va = Yi - qn, qa = ra - Vn;
                return ha * ha + ya * ya > va * va + qa * qa && (wi = Yi, Wi = ra), [[wi - qt, Wi - tn], [wi * Be / Cr, Wi * Be / Cr]];
              }
              function Eu() {
                return !0;
              }
              function fu(Ae) {
                var Oe = Da, Be = la, Ue = Eu, nt = bc, ht = nt.key, Et = 0.7;
                function Mt(qt) {
                  var tn, xn = [], An = [], On = -1, Bn = qt.length, qn = Ci(Oe), Vn = Ci(Be);
                  function sr() {
                    xn.push("M", nt(Ae(An), Et));
                  }
                  for (; ++On < Bn; ) Ue.call(this, tn = qt[On], On) ? An.push([+qn.call(this, tn, On), +Vn.call(this, tn, On)]) : An.length && (sr(), An = []);
                  return An.length && sr(), xn.length ? xn.join("") : null;
                }
                return Mt.x = function(qt) {
                  return arguments.length ? (Oe = qt, Mt) : Oe;
                }, Mt.y = function(qt) {
                  return arguments.length ? (Be = qt, Mt) : Be;
                }, Mt.defined = function(qt) {
                  return arguments.length ? (Ue = qt, Mt) : Ue;
                }, Mt.interpolate = function(qt) {
                  return arguments.length ? (ht = typeof qt == "function" ? nt = qt : (nt = ac.get(qt) || bc).key, Mt) : ht;
                }, Mt.tension = function(qt) {
                  return arguments.length ? (Et = qt, Mt) : Et;
                }, Mt;
              }
              ee.svg.line = function() {
                return fu(Xe);
              };
              var ac = ee.map({ linear: bc, "linear-closed": kc, step: function(Ae) {
                for (var Oe = 0, Be = Ae.length, Ue = Ae[0], nt = [Ue[0], ",", Ue[1]]; ++Oe < Be; ) nt.push("H", (Ue[0] + (Ue = Ae[Oe])[0]) / 2, "V", Ue[1]);
                return Be > 1 && nt.push("H", Ue[0]), nt.join("");
              }, "step-before": fc, "step-after": oc, basis: Tl, "basis-open": function(Ae) {
                if (Ae.length < 4) return bc(Ae);
                for (var Oe, Be = [], Ue = -1, nt = Ae.length, ht = [0], Et = [0]; ++Ue < 3; ) Oe = Ae[Ue], ht.push(Oe[0]), Et.push(Oe[1]);
                for (Be.push(sc(Rc, ht) + "," + sc(Rc, Et)), --Ue; ++Ue < nt; ) Oe = Ae[Ue], ht.shift(), ht.push(Oe[0]), Et.shift(), Et.push(Oe[1]), Zo(Be, ht, Et);
                return Be.join("");
              }, "basis-closed": function(Ae) {
                for (var Oe, Be, Ue = -1, nt = Ae.length, ht = nt + 4, Et = [], Mt = []; ++Ue < 4; ) Be = Ae[Ue % nt], Et.push(Be[0]), Mt.push(Be[1]);
                for (Oe = [sc(Rc, Et), ",", sc(Rc, Mt)], --Ue; ++Ue < ht; ) Be = Ae[Ue % nt], Et.shift(), Et.push(Be[0]), Mt.shift(), Mt.push(Be[1]), Zo(Oe, Et, Mt);
                return Oe.join("");
              }, bundle: function(Ae, Oe) {
                var Be = Ae.length - 1;
                if (Be) for (var Ue, nt, ht = Ae[0][0], Et = Ae[0][1], Mt = Ae[Be][0] - ht, qt = Ae[Be][1] - Et, tn = -1; ++tn <= Be; ) nt = tn / Be, (Ue = Ae[tn])[0] = Oe * Ue[0] + (1 - Oe) * (ht + nt * Mt), Ue[1] = Oe * Ue[1] + (1 - Oe) * (Et + nt * qt);
                return Tl(Ae);
              }, cardinal: function(Ae, Oe) {
                return Ae.length < 3 ? bc(Ae) : Ae[0] + ss(Ae, su(Ae, Oe));
              }, "cardinal-open": function(Ae, Oe) {
                return Ae.length < 4 ? bc(Ae) : Ae[1] + ss(Ae.slice(1, -1), su(Ae, Oe));
              }, "cardinal-closed": function(Ae, Oe) {
                return Ae.length < 3 ? kc(Ae) : Ae[0] + ss((Ae.push(Ae[0]), Ae), su([Ae[Ae.length - 2]].concat(Ae, [Ae[1]]), Oe));
              }, monotone: function(Ae) {
                return Ae.length < 3 ? bc(Ae) : Ae[0] + ss(Ae, function(Oe) {
                  for (var Be, Ue, nt, ht, Et = [], Mt = function(xn) {
                    for (var An = 0, On = xn.length - 1, Bn = [], qn = xn[0], Vn = xn[1], sr = Bn[0] = Ou(qn, Vn); ++An < On; ) Bn[An] = (sr + (sr = Ou(qn = Vn, Vn = xn[An + 1]))) / 2;
                    return Bn[An] = sr, Bn;
                  }(Oe), qt = -1, tn = Oe.length - 1; ++qt < tn; ) Be = Ou(Oe[qt], Oe[qt + 1]), Se(Be) < rt ? Mt[qt] = Mt[qt + 1] = 0 : (ht = (Ue = Mt[qt] / Be) * Ue + (nt = Mt[qt + 1] / Be) * nt) > 9 && (ht = 3 * Be / Math.sqrt(ht), Mt[qt] = ht * Ue, Mt[qt + 1] = ht * nt);
                  for (qt = -1; ++qt <= tn; ) ht = (Oe[Math.min(tn, qt + 1)][0] - Oe[Math.max(0, qt - 1)][0]) / (6 * (1 + Mt[qt] * Mt[qt])), Et.push([ht || 0, Mt[qt] * ht || 0]);
                  return Et;
                }(Ae));
              } });
              function bc(Ae) {
                return Ae.length > 1 ? Ae.join("L") : Ae + "Z";
              }
              function kc(Ae) {
                return Ae.join("L") + "Z";
              }
              function fc(Ae) {
                for (var Oe = 0, Be = Ae.length, Ue = Ae[0], nt = [Ue[0], ",", Ue[1]]; ++Oe < Be; ) nt.push("V", (Ue = Ae[Oe])[1], "H", Ue[0]);
                return nt.join("");
              }
              function oc(Ae) {
                for (var Oe = 0, Be = Ae.length, Ue = Ae[0], nt = [Ue[0], ",", Ue[1]]; ++Oe < Be; ) nt.push("H", (Ue = Ae[Oe])[0], "V", Ue[1]);
                return nt.join("");
              }
              function ss(Ae, Oe) {
                if (Oe.length < 1 || Ae.length != Oe.length && Ae.length != Oe.length + 2) return bc(Ae);
                var Be = Ae.length != Oe.length, Ue = "", nt = Ae[0], ht = Ae[1], Et = Oe[0], Mt = Et, qt = 1;
                if (Be && (Ue += "Q" + (ht[0] - 2 * Et[0] / 3) + "," + (ht[1] - 2 * Et[1] / 3) + "," + ht[0] + "," + ht[1], nt = Ae[1], qt = 2), Oe.length > 1) {
                  Mt = Oe[1], ht = Ae[qt], qt++, Ue += "C" + (nt[0] + Et[0]) + "," + (nt[1] + Et[1]) + "," + (ht[0] - Mt[0]) + "," + (ht[1] - Mt[1]) + "," + ht[0] + "," + ht[1];
                  for (var tn = 2; tn < Oe.length; tn++, qt++) ht = Ae[qt], Mt = Oe[tn], Ue += "S" + (ht[0] - Mt[0]) + "," + (ht[1] - Mt[1]) + "," + ht[0] + "," + ht[1];
                }
                if (Be) {
                  var xn = Ae[qt];
                  Ue += "Q" + (ht[0] + 2 * Mt[0] / 3) + "," + (ht[1] + 2 * Mt[1] / 3) + "," + xn[0] + "," + xn[1];
                }
                return Ue;
              }
              function su(Ae, Oe) {
                for (var Be, Ue = [], nt = (1 - Oe) / 2, ht = Ae[0], Et = Ae[1], Mt = 1, qt = Ae.length; ++Mt < qt; ) Be = ht, ht = Et, Et = Ae[Mt], Ue.push([nt * (Et[0] - Be[0]), nt * (Et[1] - Be[1])]);
                return Ue;
              }
              function Tl(Ae) {
                if (Ae.length < 3) return bc(Ae);
                var Oe = 1, Be = Ae.length, Ue = Ae[0], nt = Ue[0], ht = Ue[1], Et = [nt, nt, nt, (Ue = Ae[1])[0]], Mt = [ht, ht, ht, Ue[1]], qt = [nt, ",", ht, "L", sc(Rc, Et), ",", sc(Rc, Mt)];
                for (Ae.push(Ae[Be - 1]); ++Oe <= Be; ) Ue = Ae[Oe], Et.shift(), Et.push(Ue[0]), Mt.shift(), Mt.push(Ue[1]), Zo(qt, Et, Mt);
                return Ae.pop(), qt.push("L", Ue), qt.join("");
              }
              function sc(Ae, Oe) {
                return Ae[0] * Oe[0] + Ae[1] * Oe[1] + Ae[2] * Oe[2] + Ae[3] * Oe[3];
              }
              ac.forEach(function(Ae, Oe) {
                Oe.key = Ae, Oe.closed = /-closed$/.test(Ae);
              });
              var hc = [0, 2 / 3, 1 / 3, 0], Vc = [0, 1 / 3, 2 / 3, 0], Rc = [0, 1 / 6, 2 / 3, 1 / 6];
              function Zo(Ae, Oe, Be) {
                Ae.push("C", sc(hc, Oe), ",", sc(hc, Be), ",", sc(Vc, Oe), ",", sc(Vc, Be), ",", sc(Rc, Oe), ",", sc(Rc, Be));
              }
              function Ou(Ae, Oe) {
                return (Oe[1] - Ae[1]) / (Oe[0] - Ae[0]);
              }
              function Uc(Ae) {
                for (var Oe, Be, Ue, nt = -1, ht = Ae.length; ++nt < ht; ) Be = (Oe = Ae[nt])[0], Ue = Oe[1] - At, Oe[0] = Be * Math.cos(Ue), Oe[1] = Be * Math.sin(Ue);
                return Ae;
              }
              function El(Ae) {
                var Oe = Da, Be = Da, Ue = 0, nt = la, ht = Eu, Et = bc, Mt = Et.key, qt = Et, tn = "L", xn = 0.7;
                function An(On) {
                  var Bn, qn, Vn, sr = [], Un = [], wr = [], Cr = -1, Tr = On.length, ii = Ci(Oe), wi = Ci(Ue), Wi = Oe === Be ? function() {
                    return qn;
                  } : Ci(Be), Yi = Ue === nt ? function() {
                    return Vn;
                  } : Ci(nt);
                  function ra() {
                    sr.push("M", Et(Ae(wr), xn), tn, qt(Ae(Un.reverse()), xn), "Z");
                  }
                  for (; ++Cr < Tr; ) ht.call(this, Bn = On[Cr], Cr) ? (Un.push([qn = +ii.call(this, Bn, Cr), Vn = +wi.call(this, Bn, Cr)]), wr.push([+Wi.call(this, Bn, Cr), +Yi.call(this, Bn, Cr)])) : Un.length && (ra(), Un = [], wr = []);
                  return Un.length && ra(), sr.length ? sr.join("") : null;
                }
                return An.x = function(On) {
                  return arguments.length ? (Oe = Be = On, An) : Be;
                }, An.x0 = function(On) {
                  return arguments.length ? (Oe = On, An) : Oe;
                }, An.x1 = function(On) {
                  return arguments.length ? (Be = On, An) : Be;
                }, An.y = function(On) {
                  return arguments.length ? (Ue = nt = On, An) : nt;
                }, An.y0 = function(On) {
                  return arguments.length ? (Ue = On, An) : Ue;
                }, An.y1 = function(On) {
                  return arguments.length ? (nt = On, An) : nt;
                }, An.defined = function(On) {
                  return arguments.length ? (ht = On, An) : ht;
                }, An.interpolate = function(On) {
                  return arguments.length ? (Mt = typeof On == "function" ? Et = On : (Et = ac.get(On) || bc).key, qt = Et.reverse || Et, tn = Et.closed ? "M" : "L", An) : Mt;
                }, An.tension = function(On) {
                  return arguments.length ? (xn = On, An) : xn;
                }, An;
              }
              function eu(Ae) {
                return Ae.source;
              }
              function Wc(Ae) {
                return Ae.target;
              }
              function hu(Ae) {
                return Ae.radius;
              }
              function gd(Ae) {
                return [Ae.x, Ae.y];
              }
              function Jl() {
                return 64;
              }
              function Yc() {
                return "circle";
              }
              function nc(Ae) {
                var Oe = Math.sqrt(Ae / pt);
                return "M0," + Oe + "A" + Oe + "," + Oe + " 0 1,1 0," + -Oe + "A" + Oe + "," + Oe + " 0 1,1 0," + Oe + "Z";
              }
              ee.svg.line.radial = function() {
                var Ae = fu(Uc);
                return Ae.radius = Ae.x, delete Ae.x, Ae.angle = Ae.y, delete Ae.y, Ae;
              }, fc.reverse = oc, oc.reverse = fc, ee.svg.area = function() {
                return El(Xe);
              }, ee.svg.area.radial = function() {
                var Ae = El(Uc);
                return Ae.radius = Ae.x, delete Ae.x, Ae.innerRadius = Ae.x0, delete Ae.x0, Ae.outerRadius = Ae.x1, delete Ae.x1, Ae.angle = Ae.y, delete Ae.y, Ae.startAngle = Ae.y0, delete Ae.y0, Ae.endAngle = Ae.y1, delete Ae.y1, Ae;
              }, ee.svg.chord = function() {
                var Ae = eu, Oe = Wc, Be = hu, Ue = Lc, nt = Do;
                function ht(tn, xn) {
                  var An, On, Bn = Et(this, Ae, tn, xn), qn = Et(this, Oe, tn, xn);
                  return "M" + Bn.p0 + Mt(Bn.r, Bn.p1, Bn.a1 - Bn.a0) + (On = qn, ((An = Bn).a0 == On.a0 && An.a1 == On.a1 ? qt(Bn.r, Bn.p1, Bn.r, Bn.p0) : qt(Bn.r, Bn.p1, qn.r, qn.p0) + Mt(qn.r, qn.p1, qn.a1 - qn.a0) + qt(qn.r, qn.p1, Bn.r, Bn.p0)) + "Z");
                }
                function Et(tn, xn, An, On) {
                  var Bn = xn.call(tn, An, On), qn = Be.call(tn, Bn, On), Vn = Ue.call(tn, Bn, On) - At, sr = nt.call(tn, Bn, On) - At;
                  return { r: qn, a0: Vn, a1: sr, p0: [qn * Math.cos(Vn), qn * Math.sin(Vn)], p1: [qn * Math.cos(sr), qn * Math.sin(sr)] };
                }
                function Mt(tn, xn, An) {
                  return "A" + tn + "," + tn + " 0 " + +(An > pt) + ",1 " + xn;
                }
                function qt(tn, xn, An, On) {
                  return "Q 0,0 " + On;
                }
                return ht.radius = function(tn) {
                  return arguments.length ? (Be = Ci(tn), ht) : Be;
                }, ht.source = function(tn) {
                  return arguments.length ? (Ae = Ci(tn), ht) : Ae;
                }, ht.target = function(tn) {
                  return arguments.length ? (Oe = Ci(tn), ht) : Oe;
                }, ht.startAngle = function(tn) {
                  return arguments.length ? (Ue = Ci(tn), ht) : Ue;
                }, ht.endAngle = function(tn) {
                  return arguments.length ? (nt = Ci(tn), ht) : nt;
                }, ht;
              }, ee.svg.diagonal = function() {
                var Ae = eu, Oe = Wc, Be = gd;
                function Ue(nt, ht) {
                  var Et = Ae.call(this, nt, ht), Mt = Oe.call(this, nt, ht), qt = (Et.y + Mt.y) / 2, tn = [Et, { x: Et.x, y: qt }, { x: Mt.x, y: qt }, Mt];
                  return "M" + (tn = tn.map(Be))[0] + "C" + tn[1] + " " + tn[2] + " " + tn[3];
                }
                return Ue.source = function(nt) {
                  return arguments.length ? (Ae = Ci(nt), Ue) : Ae;
                }, Ue.target = function(nt) {
                  return arguments.length ? (Oe = Ci(nt), Ue) : Oe;
                }, Ue.projection = function(nt) {
                  return arguments.length ? (Be = nt, Ue) : Be;
                }, Ue;
              }, ee.svg.diagonal.radial = function() {
                var Ae = ee.svg.diagonal(), Oe = gd, Be = Ae.projection;
                return Ae.projection = function(Ue) {
                  return arguments.length ? Be(/* @__PURE__ */ function(nt) {
                    return function() {
                      var ht = nt.apply(this, arguments), Et = ht[0], Mt = ht[1] - At;
                      return [Et * Math.cos(Mt), Et * Math.sin(Mt)];
                    };
                  }(Oe = Ue)) : Oe;
                }, Ae;
              }, ee.svg.symbol = function() {
                var Ae = Yc, Oe = Jl;
                function Be(Ue, nt) {
                  return (Ko.get(Ae.call(this, Ue, nt)) || nc)(Oe.call(this, Ue, nt));
                }
                return Be.type = function(Ue) {
                  return arguments.length ? (Ae = Ci(Ue), Be) : Ae;
                }, Be.size = function(Ue) {
                  return arguments.length ? (Oe = Ci(Ue), Be) : Oe;
                }, Be;
              };
              var Ko = ee.map({ circle: nc, cross: function(Ae) {
                var Oe = Math.sqrt(Ae / 5) / 2;
                return "M" + -3 * Oe + "," + -Oe + "H" + -Oe + "V" + -3 * Oe + "H" + Oe + "V" + -Oe + "H" + 3 * Oe + "V" + Oe + "H" + Oe + "V" + 3 * Oe + "H" + -Oe + "V" + Oe + "H" + -3 * Oe + "Z";
              }, diamond: function(Ae) {
                var Oe = Math.sqrt(Ae / (2 * qs)), Be = Oe * qs;
                return "M0," + -Oe + "L" + Be + ",0 0," + Oe + " " + -Be + ",0Z";
              }, square: function(Ae) {
                var Oe = Math.sqrt(Ae) / 2;
                return "M" + -Oe + "," + -Oe + "L" + Oe + "," + -Oe + " " + Oe + "," + Oe + " " + -Oe + "," + Oe + "Z";
              }, "triangle-down": function(Ae) {
                var Oe = Math.sqrt(Ae / ks), Be = Oe * ks / 2;
                return "M0," + Be + "L" + Oe + "," + -Be + " " + -Oe + "," + -Be + "Z";
              }, "triangle-up": function(Ae) {
                var Oe = Math.sqrt(Ae / ks), Be = Oe * ks / 2;
                return "M0," + -Be + "L" + Oe + "," + Be + " " + -Oe + "," + Be + "Z";
              } });
              ee.svg.symbolTypes = Ko.keys();
              var ks = Math.sqrt(3), qs = Math.tan(30 * Dt);
              St.transition = function(Ae) {
                for (var Oe, Be, Ue = $c || ++zs, nt = xc(Ae), ht = [], Et = lu || { time: Date.now(), ease: Nc, delay: 0, duration: 250 }, Mt = -1, qt = this.length; ++Mt < qt; ) {
                  ht.push(Oe = []);
                  for (var tn = this[Mt], xn = -1, An = tn.length; ++xn < An; ) (Be = tn[xn]) && Iu(Be, xn, nt, Ue, Et), Oe.push(Be);
                }
                return o1(ht, nt, Ue);
              }, St.interrupt = function(Ae) {
                return this.each(Ae == null ? a1 : $l(xc(Ae)));
              };
              var a1 = $l(xc());
              function $l(Ae) {
                return function() {
                  var Oe, Be, Ue;
                  (Oe = this[Ae]) && (Ue = Oe[Be = Oe.active]) && (Ue.timer.c = null, Ue.timer.t = NaN, --Oe.count ? delete Oe[Be] : delete this[Ae], Oe.active += 0.5, Ue.event && Ue.event.interrupt.call(this, this.__data__, Ue.index));
                };
              }
              function o1(Ae, Oe, Be) {
                return it(Ae, Ul), Ae.namespace = Oe, Ae.id = Be, Ae;
              }
              var $c, lu, Ul = [], zs = 0;
              function Os(Ae, Oe, Be, Ue) {
                var nt = Ae.id, ht = Ae.namespace;
                return Qt(Ae, typeof Be == "function" ? function(Et, Mt, qt) {
                  Et[ht][nt].tween.set(Oe, Ue(Be.call(Et, Et.__data__, Mt, qt)));
                } : (Be = Ue(Be), function(Et) {
                  Et[ht][nt].tween.set(Oe, Be);
                }));
              }
              function lc(Ae) {
                return Ae == null && (Ae = ""), function() {
                  this.textContent = Ae;
                };
              }
              function xc(Ae) {
                return Ae == null ? "__transition__" : "__transition_" + Ae + "__";
              }
              function Iu(Ae, Oe, Be, Ue, nt) {
                var ht, Et, Mt, qt, tn, xn = Ae[Be] || (Ae[Be] = { active: 0, count: 0 }), An = xn[Ue];
                function On(qn) {
                  var Vn = xn.active, sr = xn[Vn];
                  for (var Un in sr && (sr.timer.c = null, sr.timer.t = NaN, --xn.count, delete xn[Vn], sr.event && sr.event.interrupt.call(Ae, Ae.__data__, sr.index)), xn) if (+Un < Ue) {
                    var wr = xn[Un];
                    wr.timer.c = null, wr.timer.t = NaN, --xn.count, delete xn[Un];
                  }
                  Et.c = Bn, Dr(function() {
                    return Et.c && Bn(qn || 1) && (Et.c = null, Et.t = NaN), 1;
                  }, 0, ht), xn.active = Ue, An.event && An.event.start.call(Ae, Ae.__data__, Oe), tn = [], An.tween.forEach(function(Cr, Tr) {
                    (Tr = Tr.call(Ae, Ae.__data__, Oe)) && tn.push(Tr);
                  }), qt = An.ease, Mt = An.duration;
                }
                function Bn(qn) {
                  for (var Vn = qn / Mt, sr = qt(Vn), Un = tn.length; Un > 0; ) tn[--Un].call(Ae, sr);
                  if (Vn >= 1) return An.event && An.event.end.call(Ae, Ae.__data__, Oe), --xn.count ? delete xn[Ue] : delete Ae[Be], 1;
                }
                An || (ht = nt.time, Et = Dr(function(qn) {
                  var Vn = An.delay;
                  if (Et.t = Vn + ht, Vn <= qn) return On(qn - Vn);
                  Et.c = On;
                }, 0, ht), An = xn[Ue] = { tween: new Ee(), time: ht, timer: Et, delay: nt.delay, duration: nt.duration, ease: nt.ease, index: Oe }, nt = null, ++xn.count);
              }
              Ul.call = St.call, Ul.empty = St.empty, Ul.node = St.node, Ul.size = St.size, ee.transition = function(Ae, Oe) {
                return Ae && Ae.transition ? $c ? Ae.transition(Oe) : Ae : ee.selection().transition(Ae);
              }, ee.transition.prototype = Ul, Ul.select = function(Ae) {
                var Oe, Be, Ue, nt = this.id, ht = this.namespace, Et = [];
                Ae = Lt(Ae);
                for (var Mt = -1, qt = this.length; ++Mt < qt; ) {
                  Et.push(Oe = []);
                  for (var tn = this[Mt], xn = -1, An = tn.length; ++xn < An; ) (Ue = tn[xn]) && (Be = Ae.call(Ue, Ue.__data__, xn, Mt)) ? ("__data__" in Ue && (Be.__data__ = Ue.__data__), Iu(Be, xn, ht, nt, Ue[ht][nt]), Oe.push(Be)) : Oe.push(null);
                }
                return o1(Et, ht, nt);
              }, Ul.selectAll = function(Ae) {
                var Oe, Be, Ue, nt, ht, Et = this.id, Mt = this.namespace, qt = [];
                Ae = Rt(Ae);
                for (var tn = -1, xn = this.length; ++tn < xn; ) for (var An = this[tn], On = -1, Bn = An.length; ++On < Bn; ) if (Ue = An[On]) {
                  ht = Ue[Mt][Et], Be = Ae.call(Ue, Ue.__data__, On, tn), qt.push(Oe = []);
                  for (var qn = -1, Vn = Be.length; ++qn < Vn; ) (nt = Be[qn]) && Iu(nt, qn, Mt, Et, ht), Oe.push(nt);
                }
                return o1(qt, Mt, Et);
              }, Ul.filter = function(Ae) {
                var Oe, Be, Ue = [];
                typeof Ae != "function" && (Ae = sn(Ae));
                for (var nt = 0, ht = this.length; nt < ht; nt++) {
                  Ue.push(Oe = []);
                  for (var Et, Mt = 0, qt = (Et = this[nt]).length; Mt < qt; Mt++) (Be = Et[Mt]) && Ae.call(Be, Be.__data__, Mt, nt) && Oe.push(Be);
                }
                return o1(Ue, this.namespace, this.id);
              }, Ul.tween = function(Ae, Oe) {
                var Be = this.id, Ue = this.namespace;
                return arguments.length < 2 ? this.node()[Ue][Be].tween.get(Ae) : Qt(this, Oe == null ? function(nt) {
                  nt[Ue][Be].tween.remove(Ae);
                } : function(nt) {
                  nt[Ue][Be].tween.set(Ae, Oe);
                });
              }, Ul.attr = function(Ae, Oe) {
                if (arguments.length < 2) {
                  for (Oe in Ae) this.attr(Oe, Ae[Oe]);
                  return this;
                }
                var Be = Ae == "transform" ? D1 : Gl, Ue = ee.ns.qualify(Ae);
                function nt() {
                  this.removeAttribute(Ue);
                }
                function ht() {
                  this.removeAttributeNS(Ue.space, Ue.local);
                }
                return Os(this, "attr." + Ae, Oe, Ue.local ? function(Et) {
                  return Et == null ? ht : (Et += "", function() {
                    var Mt, qt = this.getAttributeNS(Ue.space, Ue.local);
                    return qt !== Et && (Mt = Be(qt, Et), function(tn) {
                      this.setAttributeNS(Ue.space, Ue.local, Mt(tn));
                    });
                  });
                } : function(Et) {
                  return Et == null ? nt : (Et += "", function() {
                    var Mt, qt = this.getAttribute(Ue);
                    return qt !== Et && (Mt = Be(qt, Et), function(tn) {
                      this.setAttribute(Ue, Mt(tn));
                    });
                  });
                });
              }, Ul.attrTween = function(Ae, Oe) {
                var Be = ee.ns.qualify(Ae);
                return this.tween("attr." + Ae, Be.local ? function(Ue, nt) {
                  var ht = Oe.call(this, Ue, nt, this.getAttributeNS(Be.space, Be.local));
                  return ht && function(Et) {
                    this.setAttributeNS(Be.space, Be.local, ht(Et));
                  };
                } : function(Ue, nt) {
                  var ht = Oe.call(this, Ue, nt, this.getAttribute(Be));
                  return ht && function(Et) {
                    this.setAttribute(Be, ht(Et));
                  };
                });
              }, Ul.style = function(Ae, Oe, Be) {
                var Ue = arguments.length;
                if (Ue < 3) {
                  if (typeof Ae != "string") {
                    for (Be in Ue < 2 && (Oe = ""), Ae) this.style(Be, Ae[Be], Oe);
                    return this;
                  }
                  Be = "";
                }
                function nt() {
                  this.style.removeProperty(Ae);
                }
                return Os(this, "style." + Ae, Oe, function(ht) {
                  return ht == null ? nt : (ht += "", function() {
                    var Et, Mt = le(this).getComputedStyle(this, null).getPropertyValue(Ae);
                    return Mt !== ht && (Et = Gl(Mt, ht), function(qt) {
                      this.style.setProperty(Ae, Et(qt), Be);
                    });
                  });
                });
              }, Ul.styleTween = function(Ae, Oe, Be) {
                return arguments.length < 3 && (Be = ""), this.tween("style." + Ae, function(Ue, nt) {
                  var ht = Oe.call(this, Ue, nt, le(this).getComputedStyle(this, null).getPropertyValue(Ae));
                  return ht && function(Et) {
                    this.style.setProperty(Ae, ht(Et), Be);
                  };
                });
              }, Ul.text = function(Ae) {
                return Os(this, "text", Ae, lc);
              }, Ul.remove = function() {
                var Ae = this.namespace;
                return this.each("end.transition", function() {
                  var Oe;
                  this[Ae].count < 2 && (Oe = this.parentNode) && Oe.removeChild(this);
                });
              }, Ul.ease = function(Ae) {
                var Oe = this.id, Be = this.namespace;
                return arguments.length < 1 ? this.node()[Be][Oe].ease : (typeof Ae != "function" && (Ae = ee.ease.apply(ee, arguments)), Qt(this, function(Ue) {
                  Ue[Be][Oe].ease = Ae;
                }));
              }, Ul.delay = function(Ae) {
                var Oe = this.id, Be = this.namespace;
                return arguments.length < 1 ? this.node()[Be][Oe].delay : Qt(this, typeof Ae == "function" ? function(Ue, nt, ht) {
                  Ue[Be][Oe].delay = +Ae.call(Ue, Ue.__data__, nt, ht);
                } : (Ae = +Ae, function(Ue) {
                  Ue[Be][Oe].delay = Ae;
                }));
              }, Ul.duration = function(Ae) {
                var Oe = this.id, Be = this.namespace;
                return arguments.length < 1 ? this.node()[Be][Oe].duration : Qt(this, typeof Ae == "function" ? function(Ue, nt, ht) {
                  Ue[Be][Oe].duration = Math.max(1, Ae.call(Ue, Ue.__data__, nt, ht));
                } : (Ae = Math.max(1, Ae), function(Ue) {
                  Ue[Be][Oe].duration = Ae;
                }));
              }, Ul.each = function(Ae, Oe) {
                var Be = this.id, Ue = this.namespace;
                if (arguments.length < 2) {
                  var nt = lu, ht = $c;
                  try {
                    $c = Be, Qt(this, function(Et, Mt, qt) {
                      lu = Et[Ue][Be], Ae.call(Et, Et.__data__, Mt, qt);
                    });
                  } finally {
                    lu = nt, $c = ht;
                  }
                } else Qt(this, function(Et) {
                  var Mt = Et[Ue][Be];
                  (Mt.event || (Mt.event = ee.dispatch("start", "end", "interrupt"))).on(Ae, Oe);
                });
                return this;
              }, Ul.transition = function() {
                for (var Ae, Oe, Be, Ue = this.id, nt = ++zs, ht = this.namespace, Et = [], Mt = 0, qt = this.length; Mt < qt; Mt++) {
                  Et.push(Ae = []);
                  for (var tn, xn = 0, An = (tn = this[Mt]).length; xn < An; xn++) (Oe = tn[xn]) && Iu(Oe, xn, ht, nt, { time: (Be = Oe[ht][Ue]).time, ease: Be.ease, delay: Be.delay + Be.duration, duration: Be.duration }), Ae.push(Oe);
                }
                return o1(Et, ht, nt);
              }, ee.svg.axis = function() {
                var Ae, Oe = ee.scale.linear(), Be = tu, Ue = 6, nt = 6, ht = 3, Et = [10], Mt = null;
                function qt(tn) {
                  tn.each(function() {
                    var xn, An = ee.select(this), On = this.__chart__ || Oe, Bn = this.__chart__ = Oe.copy(), qn = Mt ?? (Bn.ticks ? Bn.ticks.apply(Bn, Et) : Bn.domain()), Vn = Ae ?? (Bn.tickFormat ? Bn.tickFormat.apply(Bn, Et) : Xe), sr = An.selectAll(".tick").data(qn, Bn), Un = sr.enter().insert("g", ".domain").attr("class", "tick").style("opacity", rt), wr = ee.transition(sr.exit()).style("opacity", rt).remove(), Cr = ee.transition(sr.order()).style("opacity", 1), Tr = Math.max(Ue, 0) + ht, ii = ku(Bn), wi = An.selectAll(".domain").data([0]), Wi = (wi.enter().append("path").attr("class", "domain"), ee.transition(wi));
                    Un.append("line"), Un.append("text");
                    var Yi, ra, ha, ya, va = Un.select("line"), qa = Cr.select("line"), Aa = sr.select("text").text(Vn), Xa = Un.select("text"), Oa = Cr.select("text"), Ya = Be === "top" || Be === "left" ? -1 : 1;
                    if (Be === "bottom" || Be === "top" ? (xn = $n, Yi = "x", ha = "y", ra = "x2", ya = "y2", Aa.attr("dy", Ya < 0 ? "0em" : ".71em").style("text-anchor", "middle"), Wi.attr("d", "M" + ii[0] + "," + Ya * nt + "V0H" + ii[1] + "V" + Ya * nt)) : (xn = zn, Yi = "y", ha = "x", ra = "y2", ya = "x2", Aa.attr("dy", ".32em").style("text-anchor", Ya < 0 ? "end" : "start"), Wi.attr("d", "M" + Ya * nt + "," + ii[0] + "H0V" + ii[1] + "H" + Ya * nt)), va.attr(ya, Ya * Ue), Xa.attr(ha, Ya * Tr), qa.attr(ra, 0).attr(ya, Ya * Ue), Oa.attr(Yi, 0).attr(ha, Ya * Tr), Bn.rangeBand) {
                      var Wo = Bn, Ja = Wo.rangeBand() / 2;
                      On = Bn = function(Uo) {
                        return Wo(Uo) + Ja;
                      };
                    } else On.rangeBand ? On = Bn : wr.call(xn, Bn, On);
                    Un.call(xn, On, Bn), Cr.call(xn, Bn, Bn);
                  });
                }
                return qt.scale = function(tn) {
                  return arguments.length ? (Oe = tn, qt) : Oe;
                }, qt.orient = function(tn) {
                  return arguments.length ? (Be = tn in w1 ? tn + "" : tu, qt) : Be;
                }, qt.ticks = function() {
                  return arguments.length ? (Et = oe(arguments), qt) : Et;
                }, qt.tickValues = function(tn) {
                  return arguments.length ? (Mt = tn, qt) : Mt;
                }, qt.tickFormat = function(tn) {
                  return arguments.length ? (Ae = tn, qt) : Ae;
                }, qt.tickSize = function(tn) {
                  var xn = arguments.length;
                  return xn ? (Ue = +tn, nt = +arguments[xn - 1], qt) : Ue;
                }, qt.innerTickSize = function(tn) {
                  return arguments.length ? (Ue = +tn, qt) : Ue;
                }, qt.outerTickSize = function(tn) {
                  return arguments.length ? (nt = +tn, qt) : nt;
                }, qt.tickPadding = function(tn) {
                  return arguments.length ? (ht = +tn, qt) : ht;
                }, qt.tickSubdivide = function() {
                  return arguments.length && qt;
                }, qt;
              };
              var tu = "bottom", w1 = { top: 1, right: 1, bottom: 1, left: 1 };
              function $n(Ae, Oe, Be) {
                Ae.attr("transform", function(Ue) {
                  var nt = Oe(Ue);
                  return "translate(" + (isFinite(nt) ? nt : Be(Ue)) + ",0)";
                });
              }
              function zn(Ae, Oe, Be) {
                Ae.attr("transform", function(Ue) {
                  var nt = Oe(Ue);
                  return "translate(0," + (isFinite(nt) ? nt : Be(Ue)) + ")";
                });
              }
              ee.svg.brush = function() {
                var Ae, Oe, Be = dt(xn, "brushstart", "brush", "brushend"), Ue = null, nt = null, ht = [0, 0], Et = [0, 0], Mt = !0, qt = !0, tn = Ke[0];
                function xn(Vn) {
                  Vn.each(function() {
                    var sr = ee.select(this).style("pointer-events", "all").style("-webkit-tap-highlight-color", "rgba(0,0,0,0)").on("mousedown.brush", qn).on("touchstart.brush", qn), Un = sr.selectAll(".background").data([0]);
                    Un.enter().append("rect").attr("class", "background").style("visibility", "hidden").style("cursor", "crosshair"), sr.selectAll(".extent").data([0]).enter().append("rect").attr("class", "extent").style("cursor", "move");
                    var wr = sr.selectAll(".resize").data(tn, Xe);
                    wr.exit().remove(), wr.enter().append("g").attr("class", function(wi) {
                      return "resize " + wi;
                    }).style("cursor", function(wi) {
                      return Gn[wi];
                    }).append("rect").attr("x", function(wi) {
                      return /[ew]$/.test(wi) ? -3 : null;
                    }).attr("y", function(wi) {
                      return /^[ns]/.test(wi) ? -3 : null;
                    }).attr("width", 6).attr("height", 6).style("visibility", "hidden"), wr.style("display", xn.empty() ? "none" : null);
                    var Cr, Tr = ee.transition(sr), ii = ee.transition(Un);
                    Ue && (Cr = ku(Ue), ii.attr("x", Cr[0]).attr("width", Cr[1] - Cr[0]), On(Tr)), nt && (Cr = ku(nt), ii.attr("y", Cr[0]).attr("height", Cr[1] - Cr[0]), Bn(Tr)), An(Tr);
                  });
                }
                function An(Vn) {
                  Vn.selectAll(".resize").attr("transform", function(sr) {
                    return "translate(" + ht[+/e$/.test(sr)] + "," + Et[+/^s/.test(sr)] + ")";
                  });
                }
                function On(Vn) {
                  Vn.select(".extent").attr("x", ht[0]), Vn.selectAll(".extent,.n>rect,.s>rect").attr("width", ht[1] - ht[0]);
                }
                function Bn(Vn) {
                  Vn.select(".extent").attr("y", Et[0]), Vn.selectAll(".extent,.e>rect,.w>rect").attr("height", Et[1] - Et[0]);
                }
                function qn() {
                  var Vn, sr, Un = this, wr = ee.select(ee.event.target), Cr = Be.of(Un, arguments), Tr = ee.select(Un), ii = wr.datum(), wi = !/^(n|s)$/.test(ii) && Ue, Wi = !/^(e|w)$/.test(ii) && nt, Yi = wr.classed("extent"), ra = wn(Un), ha = ee.mouse(Un), ya = ee.select(le(Un)).on("keydown.brush", function() {
                    ee.event.keyCode == 32 && (Yi || (Vn = null, ha[0] -= ht[1], ha[1] -= Et[1], Yi = 2), ct());
                  }).on("keyup.brush", function() {
                    ee.event.keyCode == 32 && Yi == 2 && (ha[0] += ht[1], ha[1] += Et[1], Yi = 0, ct());
                  });
                  if (ee.event.changedTouches ? ya.on("touchmove.brush", Aa).on("touchend.brush", Oa) : ya.on("mousemove.brush", Aa).on("mouseup.brush", Oa), Tr.interrupt().selectAll("*").interrupt(), Yi) ha[0] = ht[0] - ha[0], ha[1] = Et[0] - ha[1];
                  else if (ii) {
                    var va = +/w$/.test(ii), qa = +/^n/.test(ii);
                    sr = [ht[1 - va] - ha[0], Et[1 - qa] - ha[1]], ha[0] = ht[va], ha[1] = Et[qa];
                  } else ee.event.altKey && (Vn = ha.slice());
                  function Aa() {
                    var Ya = ee.mouse(Un), Wo = !1;
                    sr && (Ya[0] += sr[0], Ya[1] += sr[1]), Yi || (ee.event.altKey ? (Vn || (Vn = [(ht[0] + ht[1]) / 2, (Et[0] + Et[1]) / 2]), ha[0] = ht[+(Ya[0] < Vn[0])], ha[1] = Et[+(Ya[1] < Vn[1])]) : Vn = null), wi && Xa(Ya, Ue, 0) && (On(Tr), Wo = !0), Wi && Xa(Ya, nt, 1) && (Bn(Tr), Wo = !0), Wo && (An(Tr), Cr({ type: "brush", mode: Yi ? "move" : "resize" }));
                  }
                  function Xa(Ya, Wo, Ja) {
                    var Uo, ys, il = ku(Wo), Is = il[0], ml = il[1], nl = ha[Ja], Il = Ja ? Et : ht, Fl = Il[1] - Il[0];
                    if (Yi && (Is -= nl, ml -= Fl + nl), Uo = (Ja ? qt : Mt) ? Math.max(Is, Math.min(ml, Ya[Ja])) : Ya[Ja], Yi ? ys = (Uo += nl) + Fl : (Vn && (nl = Math.max(Is, Math.min(ml, 2 * Vn[Ja] - Uo))), nl < Uo ? (ys = Uo, Uo = nl) : ys = nl), Il[0] != Uo || Il[1] != ys) return Ja ? Oe = null : Ae = null, Il[0] = Uo, Il[1] = ys, !0;
                  }
                  function Oa() {
                    Aa(), Tr.style("pointer-events", "all").selectAll(".resize").style("display", xn.empty() ? "none" : null), ee.select("body").style("cursor", null), ya.on("mousemove.brush", null).on("mouseup.brush", null).on("touchmove.brush", null).on("touchend.brush", null).on("keydown.brush", null).on("keyup.brush", null), ra(), Cr({ type: "brushend" });
                  }
                  Tr.style("pointer-events", "none").selectAll(".resize").style("display", null), ee.select("body").style("cursor", wr.style("cursor")), Cr({ type: "brushstart" }), Aa();
                }
                return xn.event = function(Vn) {
                  Vn.each(function() {
                    var sr = Be.of(this, arguments), Un = { x: ht, y: Et, i: Ae, j: Oe }, wr = this.__chart__ || Un;
                    this.__chart__ = Un, $c ? ee.select(this).transition().each("start.brush", function() {
                      Ae = wr.i, Oe = wr.j, ht = wr.x, Et = wr.y, sr({ type: "brushstart" });
                    }).tween("brush:brush", function() {
                      var Cr = Sl(ht, Un.x), Tr = Sl(Et, Un.y);
                      return Ae = Oe = null, function(ii) {
                        ht = Un.x = Cr(ii), Et = Un.y = Tr(ii), sr({ type: "brush", mode: "resize" });
                      };
                    }).each("end.brush", function() {
                      Ae = Un.i, Oe = Un.j, sr({ type: "brush", mode: "resize" }), sr({ type: "brushend" });
                    }) : (sr({ type: "brushstart" }), sr({ type: "brush", mode: "resize" }), sr({ type: "brushend" }));
                  });
                }, xn.x = function(Vn) {
                  return arguments.length ? (tn = Ke[!(Ue = Vn) << 1 | !nt], xn) : Ue;
                }, xn.y = function(Vn) {
                  return arguments.length ? (tn = Ke[!Ue << 1 | !(nt = Vn)], xn) : nt;
                }, xn.clamp = function(Vn) {
                  return arguments.length ? (Ue && nt ? (Mt = !!Vn[0], qt = !!Vn[1]) : Ue ? Mt = !!Vn : nt && (qt = !!Vn), xn) : Ue && nt ? [Mt, qt] : Ue ? Mt : nt ? qt : null;
                }, xn.extent = function(Vn) {
                  var sr, Un, wr, Cr, Tr;
                  return arguments.length ? (Ue && (sr = Vn[0], Un = Vn[1], nt && (sr = sr[0], Un = Un[0]), Ae = [sr, Un], Ue.invert && (sr = Ue(sr), Un = Ue(Un)), Un < sr && (Tr = sr, sr = Un, Un = Tr), sr == ht[0] && Un == ht[1] || (ht = [sr, Un])), nt && (wr = Vn[0], Cr = Vn[1], Ue && (wr = wr[1], Cr = Cr[1]), Oe = [wr, Cr], nt.invert && (wr = nt(wr), Cr = nt(Cr)), Cr < wr && (Tr = wr, wr = Cr, Cr = Tr), wr == Et[0] && Cr == Et[1] || (Et = [wr, Cr])), xn) : (Ue && (Ae ? (sr = Ae[0], Un = Ae[1]) : (sr = ht[0], Un = ht[1], Ue.invert && (sr = Ue.invert(sr), Un = Ue.invert(Un)), Un < sr && (Tr = sr, sr = Un, Un = Tr))), nt && (Oe ? (wr = Oe[0], Cr = Oe[1]) : (wr = Et[0], Cr = Et[1], nt.invert && (wr = nt.invert(wr), Cr = nt.invert(Cr)), Cr < wr && (Tr = wr, wr = Cr, Cr = Tr))), Ue && nt ? [[sr, wr], [Un, Cr]] : Ue ? [sr, Un] : nt && [wr, Cr]);
                }, xn.clear = function() {
                  return xn.empty() || (ht = [0, 0], Et = [0, 0], Ae = Oe = null), xn;
                }, xn.empty = function() {
                  return !!Ue && ht[0] == ht[1] || !!nt && Et[0] == Et[1];
                }, ee.rebind(xn, Be, "on");
              };
              var Gn = { n: "ns-resize", e: "ew-resize", s: "ns-resize", w: "ew-resize", nw: "nwse-resize", ne: "nesw-resize", se: "nwse-resize", sw: "nesw-resize" }, Ke = [["n", "e", "s", "w", "nw", "ne", "se", "sw"], ["e", "w"], ["n", "s"], []];
              function Fe(Ae) {
                return JSON.parse(Ae.responseText);
              }
              function Te(Ae) {
                var Oe = se.createRange();
                return Oe.selectNode(se.body), Oe.createContextualFragment(Ae.responseText);
              }
              ee.text = oa(function(Ae) {
                return Ae.responseText;
              }), ee.json = function(Ae, Oe) {
                return ka(Ae, "application/json", Fe, Oe);
              }, ee.html = function(Ae, Oe) {
                return ka(Ae, "text/html", Te, Oe);
              }, ee.xml = oa(function(Ae) {
                return Ae.responseXML;
              }), (ne = typeof (te = ee) == "function" ? te.call(J, Z, J, G) : te) === void 0 || (G.exports = ne);
            }).apply(self);
          }, 32280: function(G) {
            G.exports = function() {
              var J, Z, te;
              function ne(ee, ae) {
                if (J) if (Z) {
                  var oe = "var sharedChunk = {}; (" + J + ")(sharedChunk); (" + Z + ")(sharedChunk);", se = {};
                  J(se), te = ae(se), typeof window < "u" && (te.workerUrl = window.URL.createObjectURL(new Blob([oe], { type: "text/javascript" })));
                } else Z = ae;
                else J = ae;
              }
              return ne(0, function(ee) {
                function ae(Ne, je) {
                  return Ne(je = { exports: {} }, je.exports), je.exports;
                }
                var oe = "1.13.4", se = ce;
                function ce(Ne, je, ut, kt) {
                  this.cx = 3 * Ne, this.bx = 3 * (ut - Ne) - this.cx, this.ax = 1 - this.cx - this.bx, this.cy = 3 * je, this.by = 3 * (kt - je) - this.cy, this.ay = 1 - this.cy - this.by, this.p1x = Ne, this.p1y = kt, this.p2x = ut, this.p2y = kt;
                }
                ce.prototype.sampleCurveX = function(Ne) {
                  return ((this.ax * Ne + this.bx) * Ne + this.cx) * Ne;
                }, ce.prototype.sampleCurveY = function(Ne) {
                  return ((this.ay * Ne + this.by) * Ne + this.cy) * Ne;
                }, ce.prototype.sampleCurveDerivativeX = function(Ne) {
                  return (3 * this.ax * Ne + 2 * this.bx) * Ne + this.cx;
                }, ce.prototype.solveCurveX = function(Ne, je) {
                  var ut, kt, Jt, fn, Sn;
                  for (je === void 0 && (je = 1e-6), Jt = Ne, Sn = 0; Sn < 8; Sn++) {
                    if (fn = this.sampleCurveX(Jt) - Ne, Math.abs(fn) < je) return Jt;
                    var Hn = this.sampleCurveDerivativeX(Jt);
                    if (Math.abs(Hn) < 1e-6) break;
                    Jt -= fn / Hn;
                  }
                  if ((Jt = Ne) < (ut = 0)) return ut;
                  if (Jt > (kt = 1)) return kt;
                  for (; ut < kt; ) {
                    if (fn = this.sampleCurveX(Jt), Math.abs(fn - Ne) < je) return Jt;
                    Ne > fn ? ut = Jt : kt = Jt, Jt = 0.5 * (kt - ut) + ut;
                  }
                  return Jt;
                }, ce.prototype.solve = function(Ne, je) {
                  return this.sampleCurveY(this.solveCurveX(Ne, je));
                };
                var le = ue;
                function ue(Ne, je) {
                  this.x = Ne, this.y = je;
                }
                ue.prototype = { clone: function() {
                  return new ue(this.x, this.y);
                }, add: function(Ne) {
                  return this.clone()._add(Ne);
                }, sub: function(Ne) {
                  return this.clone()._sub(Ne);
                }, multByPoint: function(Ne) {
                  return this.clone()._multByPoint(Ne);
                }, divByPoint: function(Ne) {
                  return this.clone()._divByPoint(Ne);
                }, mult: function(Ne) {
                  return this.clone()._mult(Ne);
                }, div: function(Ne) {
                  return this.clone()._div(Ne);
                }, rotate: function(Ne) {
                  return this.clone()._rotate(Ne);
                }, rotateAround: function(Ne, je) {
                  return this.clone()._rotateAround(Ne, je);
                }, matMult: function(Ne) {
                  return this.clone()._matMult(Ne);
                }, unit: function() {
                  return this.clone()._unit();
                }, perp: function() {
                  return this.clone()._perp();
                }, round: function() {
                  return this.clone()._round();
                }, mag: function() {
                  return Math.sqrt(this.x * this.x + this.y * this.y);
                }, equals: function(Ne) {
                  return this.x === Ne.x && this.y === Ne.y;
                }, dist: function(Ne) {
                  return Math.sqrt(this.distSqr(Ne));
                }, distSqr: function(Ne) {
                  var je = Ne.x - this.x, ut = Ne.y - this.y;
                  return je * je + ut * ut;
                }, angle: function() {
                  return Math.atan2(this.y, this.x);
                }, angleTo: function(Ne) {
                  return Math.atan2(this.y - Ne.y, this.x - Ne.x);
                }, angleWith: function(Ne) {
                  return this.angleWithSep(Ne.x, Ne.y);
                }, angleWithSep: function(Ne, je) {
                  return Math.atan2(this.x * je - this.y * Ne, this.x * Ne + this.y * je);
                }, _matMult: function(Ne) {
                  var je = Ne[0] * this.x + Ne[1] * this.y, ut = Ne[2] * this.x + Ne[3] * this.y;
                  return this.x = je, this.y = ut, this;
                }, _add: function(Ne) {
                  return this.x += Ne.x, this.y += Ne.y, this;
                }, _sub: function(Ne) {
                  return this.x -= Ne.x, this.y -= Ne.y, this;
                }, _mult: function(Ne) {
                  return this.x *= Ne, this.y *= Ne, this;
                }, _div: function(Ne) {
                  return this.x /= Ne, this.y /= Ne, this;
                }, _multByPoint: function(Ne) {
                  return this.x *= Ne.x, this.y *= Ne.y, this;
                }, _divByPoint: function(Ne) {
                  return this.x /= Ne.x, this.y /= Ne.y, this;
                }, _unit: function() {
                  return this._div(this.mag()), this;
                }, _perp: function() {
                  var Ne = this.y;
                  return this.y = this.x, this.x = -Ne, this;
                }, _rotate: function(Ne) {
                  var je = Math.cos(Ne), ut = Math.sin(Ne), kt = je * this.x - ut * this.y, Jt = ut * this.x + je * this.y;
                  return this.x = kt, this.y = Jt, this;
                }, _rotateAround: function(Ne, je) {
                  var ut = Math.cos(Ne), kt = Math.sin(Ne), Jt = je.x + ut * (this.x - je.x) - kt * (this.y - je.y), fn = je.y + kt * (this.x - je.x) + ut * (this.y - je.y);
                  return this.x = Jt, this.y = fn, this;
                }, _round: function() {
                  return this.x = Math.round(this.x), this.y = Math.round(this.y), this;
                } }, ue.convert = function(Ne) {
                  return Ne instanceof ue ? Ne : Array.isArray(Ne) ? new ue(Ne[0], Ne[1]) : Ne;
                };
                var de = typeof self < "u" ? self : {}, fe = Math.pow(2, 53) - 1;
                function he(Ne, je, ut, kt) {
                  var Jt = new se(Ne, je, ut, kt);
                  return function(fn) {
                    return Jt.solve(fn);
                  };
                }
                var me = he(0.25, 0.1, 0.25, 1);
                function pe(Ne, je, ut) {
                  return Math.min(ut, Math.max(je, Ne));
                }
                function ye(Ne, je, ut) {
                  var kt = ut - je, Jt = ((Ne - je) % kt + kt) % kt + je;
                  return Jt === je ? ut : Jt;
                }
                function ge(Ne) {
                  for (var je = [], ut = arguments.length - 1; ut-- > 0; ) je[ut] = arguments[ut + 1];
                  for (var kt = 0, Jt = je; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    for (var Sn in fn) Ne[Sn] = fn[Sn];
                  }
                  return Ne;
                }
                var ve = 1;
                function be() {
                  return ve++;
                }
                function Ce() {
                  return function Ne(je) {
                    return je ? (je ^ 16 * Math.random() >> je / 4).toString(16) : ([1e7] + -[1e3] + -4e3 + -8e3 + -1e11).replace(/[018]/g, Ne);
                  }();
                }
                function Se(Ne) {
                  return !!Ne && /^[0-9a-f]{8}-[0-9a-f]{4}-[4][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i.test(Ne);
                }
                function we(Ne, je) {
                  Ne.forEach(function(ut) {
                    je[ut] && (je[ut] = je[ut].bind(je));
                  });
                }
                function Ee(Ne, je) {
                  return Ne.indexOf(je, Ne.length - je.length) !== -1;
                }
                function xe(Ne, je, ut) {
                  var kt = {};
                  for (var Jt in Ne) kt[Jt] = je.call(ut || this, Ne[Jt], Jt, Ne);
                  return kt;
                }
                function Le(Ne, je, ut) {
                  var kt = {};
                  for (var Jt in Ne) je.call(ut || this, Ne[Jt], Jt, Ne) && (kt[Jt] = Ne[Jt]);
                  return kt;
                }
                function Qe(Ne) {
                  return Array.isArray(Ne) ? Ne.map(Qe) : typeof Ne == "object" && Ne ? xe(Ne, Qe) : Ne;
                }
                var ke = {};
                function Me(Ne) {
                  ke[Ne] || (typeof console < "u" && console.warn(Ne), ke[Ne] = !0);
                }
                function Re(Ne, je, ut) {
                  return (ut.y - Ne.y) * (je.x - Ne.x) > (je.y - Ne.y) * (ut.x - Ne.x);
                }
                function De(Ne) {
                  for (var je = 0, ut = 0, kt = Ne.length, Jt = kt - 1, fn = void 0, Sn = void 0; ut < kt; Jt = ut++) fn = Ne[ut], je += ((Sn = Ne[Jt]).x - fn.x) * (fn.y + Sn.y);
                  return je;
                }
                function Pe() {
                  return typeof WorkerGlobalScope < "u" && typeof self < "u" && self instanceof WorkerGlobalScope;
                }
                function $e(Ne) {
                  var je = {};
                  if (Ne.replace(/(?:^|(?:\s*\,\s*))([^\x00-\x20\(\)<>@\,;\:\\"\/\[\]\?\=\{\}\x7F]+)(?:\=(?:([^\x00-\x20\(\)<>@\,;\:\\"\/\[\]\?\=\{\}\x7F]+)|(?:\"((?:[^"\\]|\\.)*)\")))?/g, function(kt, Jt, fn, Sn) {
                    var Hn = fn || Sn;
                    return je[Jt] = !Hn || Hn.toLowerCase(), "";
                  }), je["max-age"]) {
                    var ut = parseInt(je["max-age"], 10);
                    isNaN(ut) ? delete je["max-age"] : je["max-age"] = ut;
                  }
                  return je;
                }
                var qe = null;
                function Xe(Ne) {
                  if (qe == null) {
                    var je = Ne.navigator ? Ne.navigator.userAgent : null;
                    qe = !!Ne.safari || !(!je || !(/\b(iPad|iPhone|iPod)\b/.test(je) || je.match("Safari") && !je.match("Chrome")));
                  }
                  return qe;
                }
                function Ve(Ne) {
                  try {
                    var je = de[Ne];
                    return je.setItem("_mapbox_test_", 1), je.removeItem("_mapbox_test_"), !0;
                  } catch {
                    return !1;
                  }
                }
                var Ze, Ge, ze, We, at = de.performance && de.performance.now ? de.performance.now.bind(de.performance) : Date.now.bind(Date), ct = de.requestAnimationFrame || de.mozRequestAnimationFrame || de.webkitRequestAnimationFrame || de.msRequestAnimationFrame, et = de.cancelAnimationFrame || de.mozCancelAnimationFrame || de.webkitCancelAnimationFrame || de.msCancelAnimationFrame, dt = { now: at, frame: function(Ne) {
                  var je = ct(Ne);
                  return { cancel: function() {
                    return et(je);
                  } };
                }, getImageData: function(Ne, je) {
                  je === void 0 && (je = 0);
                  var ut = de.document.createElement("canvas"), kt = ut.getContext("2d");
                  if (!kt) throw new Error("failed to create canvas 2d context");
                  return ut.width = Ne.width, ut.height = Ne.height, kt.drawImage(Ne, 0, 0, Ne.width, Ne.height), kt.getImageData(-je, -je, Ne.width + 2 * je, Ne.height + 2 * je);
                }, resolveURL: function(Ne) {
                  return Ze || (Ze = de.document.createElement("a")), Ze.href = Ne, Ze.href;
                }, hardwareConcurrency: de.navigator && de.navigator.hardwareConcurrency || 4, get devicePixelRatio() {
                  return de.devicePixelRatio;
                }, get prefersReducedMotion() {
                  return !!de.matchMedia && (Ge == null && (Ge = de.matchMedia("(prefers-reduced-motion: reduce)")), Ge.matches);
                } }, Je = { API_URL: "https://api.mapbox.com", get EVENTS_URL() {
                  return this.API_URL ? this.API_URL.indexOf("https://api.mapbox.cn") === 0 ? "https://events.mapbox.cn/events/v2" : this.API_URL.indexOf("https://api.mapbox.com") === 0 ? "https://events.mapbox.com/events/v2" : null : null;
                }, FEEDBACK_URL: "https://apps.mapbox.com/feedback", REQUIRE_ACCESS_TOKEN: !0, ACCESS_TOKEN: null, MAX_PARALLEL_IMAGE_REQUESTS: 16 }, it = { supported: !1, testSupport: function(Ne) {
                  !st && We && (gt ? bt(Ne) : ze = Ne);
                } }, st = !1, gt = !1;
                function bt(Ne) {
                  var je = Ne.createTexture();
                  Ne.bindTexture(Ne.TEXTURE_2D, je);
                  try {
                    if (Ne.texImage2D(Ne.TEXTURE_2D, 0, Ne.RGBA, Ne.RGBA, Ne.UNSIGNED_BYTE, We), Ne.isContextLost()) return;
                    it.supported = !0;
                  } catch {
                  }
                  Ne.deleteTexture(je), st = !0;
                }
                de.document && ((We = de.document.createElement("img")).onload = function() {
                  ze && bt(ze), ze = null, gt = !0;
                }, We.onerror = function() {
                  st = !0, ze = null;
                }, We.src = "data:image/webp;base64,UklGRh4AAABXRUJQVlA4TBEAAAAvAQAAAAfQ//73v/+BiOh/AAA=");
                var yt = "01", St = function(Ne, je) {
                  this._transformRequestFn = Ne, this._customAccessToken = je, this._createSkuToken();
                };
                function Lt(Ne) {
                  return Ne.indexOf("mapbox:") === 0;
                }
                St.prototype._createSkuToken = function() {
                  var Ne = function() {
                    for (var je = "", ut = 0; ut < 10; ut++) je += "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"[Math.floor(62 * Math.random())];
                    return { token: ["1", yt, je].join(""), tokenExpiresAt: Date.now() + 432e5 };
                  }();
                  this._skuToken = Ne.token, this._skuTokenExpiresAt = Ne.tokenExpiresAt;
                }, St.prototype._isSkuTokenExpired = function() {
                  return Date.now() > this._skuTokenExpiresAt;
                }, St.prototype.transformRequest = function(Ne, je) {
                  return this._transformRequestFn && this._transformRequestFn(Ne, je) || { url: Ne };
                }, St.prototype.normalizeStyleURL = function(Ne, je) {
                  if (!Lt(Ne)) return Ne;
                  var ut = Pt(Ne);
                  return ut.path = "/styles/v1" + ut.path, this._makeAPIURL(ut, this._customAccessToken || je);
                }, St.prototype.normalizeGlyphsURL = function(Ne, je) {
                  if (!Lt(Ne)) return Ne;
                  var ut = Pt(Ne);
                  return ut.path = "/fonts/v1" + ut.path, this._makeAPIURL(ut, this._customAccessToken || je);
                }, St.prototype.normalizeSourceURL = function(Ne, je) {
                  if (!Lt(Ne)) return Ne;
                  var ut = Pt(Ne);
                  return ut.path = "/v4/" + ut.authority + ".json", ut.params.push("secure"), this._makeAPIURL(ut, this._customAccessToken || je);
                }, St.prototype.normalizeSpriteURL = function(Ne, je, ut, kt) {
                  var Jt = Pt(Ne);
                  return Lt(Ne) ? (Jt.path = "/styles/v1" + Jt.path + "/sprite" + je + ut, this._makeAPIURL(Jt, this._customAccessToken || kt)) : (Jt.path += "" + je + ut, Xt(Jt));
                }, St.prototype.normalizeTileURL = function(Ne, je) {
                  if (this._isSkuTokenExpired() && this._createSkuToken(), Ne && !Lt(Ne)) return Ne;
                  var ut = Pt(Ne), kt = dt.devicePixelRatio >= 2 || je === 512 ? "@2x" : "", Jt = it.supported ? ".webp" : "$1";
                  ut.path = ut.path.replace(/(\.(png|jpg)\d*)(?=$)/, "" + kt + Jt), ut.path = ut.path.replace(/^.+\/v4\//, "/"), ut.path = "/v4" + ut.path;
                  var fn = this._customAccessToken || function(Sn) {
                    for (var Hn = 0, nr = Sn; Hn < nr.length; Hn += 1) {
                      var kr = nr[Hn].match(/^access_token=(.*)$/);
                      if (kr) return kr[1];
                    }
                    return null;
                  }(ut.params) || Je.ACCESS_TOKEN;
                  return Je.REQUIRE_ACCESS_TOKEN && fn && this._skuToken && ut.params.push("sku=" + this._skuToken), this._makeAPIURL(ut, fn);
                }, St.prototype.canonicalizeTileURL = function(Ne, je) {
                  var ut = Pt(Ne);
                  if (!ut.path.match(/(^\/v4\/)/) || !ut.path.match(/\.[\w]+$/)) return Ne;
                  var kt = "mapbox://tiles/";
                  kt += ut.path.replace("/v4/", "");
                  var Jt = ut.params;
                  return je && (Jt = Jt.filter(function(fn) {
                    return !fn.match(/^access_token=/);
                  })), Jt.length && (kt += "?" + Jt.join("&")), kt;
                }, St.prototype.canonicalizeTileset = function(Ne, je) {
                  for (var ut = !!je && Lt(je), kt = [], Jt = 0, fn = Ne.tiles || []; Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt];
                    vt(Sn) ? kt.push(this.canonicalizeTileURL(Sn, ut)) : kt.push(Sn);
                  }
                  return kt;
                }, St.prototype._makeAPIURL = function(Ne, je) {
                  var ut = "See https://www.mapbox.com/api-documentation/#access-tokens-and-token-scopes", kt = Pt(Je.API_URL);
                  if (Ne.protocol = kt.protocol, Ne.authority = kt.authority, Ne.protocol === "http") {
                    var Jt = Ne.params.indexOf("secure");
                    Jt >= 0 && Ne.params.splice(Jt, 1);
                  }
                  if (kt.path !== "/" && (Ne.path = "" + kt.path + Ne.path), !Je.REQUIRE_ACCESS_TOKEN) return Xt(Ne);
                  if (!(je = je || Je.ACCESS_TOKEN)) throw new Error("An API access token is required to use Mapbox GL. " + ut);
                  if (je[0] === "s") throw new Error("Use a public access token (pk.*) with Mapbox GL, not a secret access token (sk.*). " + ut);
                  return Ne.params = Ne.params.filter(function(fn) {
                    return fn.indexOf("access_token") === -1;
                  }), Ne.params.push("access_token=" + je), Xt(Ne);
                };
                var Rt = /^((https?:)?\/\/)?([^\/]+\.)?mapbox\.c(n|om)(\/|\?|$)/i;
                function vt(Ne) {
                  return Rt.test(Ne);
                }
                var $t = /^(\w+):\/\/([^/?]*)(\/[^?]+)?\??(.+)?/;
                function Pt(Ne) {
                  var je = Ne.match($t);
                  if (!je) throw new Error("Unable to parse URL object");
                  return { protocol: je[1], authority: je[2], path: je[3] || "/", params: je[4] ? je[4].split("&") : [] };
                }
                function Xt(Ne) {
                  var je = Ne.params.length ? "?" + Ne.params.join("&") : "";
                  return Ne.protocol + "://" + Ne.authority + Ne.path + je;
                }
                var Nt = "mapbox.eventData";
                function Gt(Ne) {
                  if (!Ne) return null;
                  var je, ut = Ne.split(".");
                  if (!ut || ut.length !== 3) return null;
                  try {
                    return JSON.parse((je = ut[1], decodeURIComponent(de.atob(je).split("").map(function(kt) {
                      return "%" + ("00" + kt.charCodeAt(0).toString(16)).slice(-2);
                    }).join(""))));
                  } catch {
                    return null;
                  }
                }
                var Ft = function(Ne) {
                  this.type = Ne, this.anonId = null, this.eventData = {}, this.queue = [], this.pendingRequest = null;
                };
                Ft.prototype.getStorageKey = function(Ne) {
                  var je, ut, kt = Gt(Je.ACCESS_TOKEN);
                  return je = kt && kt.u ? (ut = kt.u, de.btoa(encodeURIComponent(ut).replace(/%([0-9A-F]{2})/g, function(Jt, fn) {
                    return String.fromCharCode(+("0x" + fn));
                  }))) : Je.ACCESS_TOKEN || "", Ne ? Nt + "." + Ne + ":" + je : Nt + ":" + je;
                }, Ft.prototype.fetchEventData = function() {
                  var Ne = Ve("localStorage"), je = this.getStorageKey(), ut = this.getStorageKey("uuid");
                  if (Ne) try {
                    var kt = de.localStorage.getItem(je);
                    kt && (this.eventData = JSON.parse(kt));
                    var Jt = de.localStorage.getItem(ut);
                    Jt && (this.anonId = Jt);
                  } catch {
                    Me("Unable to read from LocalStorage");
                  }
                }, Ft.prototype.saveEventData = function() {
                  var Ne = Ve("localStorage"), je = this.getStorageKey(), ut = this.getStorageKey("uuid");
                  if (Ne) try {
                    de.localStorage.setItem(ut, this.anonId), Object.keys(this.eventData).length >= 1 && de.localStorage.setItem(je, JSON.stringify(this.eventData));
                  } catch {
                    Me("Unable to write to LocalStorage");
                  }
                }, Ft.prototype.processRequests = function(Ne) {
                }, Ft.prototype.postEvent = function(Ne, je, ut, kt) {
                  var Jt = this;
                  if (Je.EVENTS_URL) {
                    var fn = Pt(Je.EVENTS_URL);
                    fn.params.push("access_token=" + (kt || Je.ACCESS_TOKEN || ""));
                    var Sn = { event: this.type, created: new Date(Ne).toISOString(), sdkIdentifier: "mapbox-gl-js", sdkVersion: oe, skuId: yt, userId: this.anonId }, Hn = je ? ge(Sn, je) : Sn, nr = { url: Xt(fn), headers: { "Content-Type": "text/plain" }, body: JSON.stringify([Hn]) };
                    this.pendingRequest = Bt(nr, function(kr) {
                      Jt.pendingRequest = null, ut(kr), Jt.saveEventData(), Jt.processRequests(kt);
                    });
                  }
                }, Ft.prototype.queueRequest = function(Ne, je) {
                  this.queue.push(Ne), this.processRequests(je);
                };
                var Tt, It, Ct = function(Ne) {
                  function je() {
                    Ne.call(this, "map.load"), this.success = {}, this.skuToken = "";
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.postMapLoadEvent = function(ut, kt, Jt, fn) {
                    this.skuToken = Jt, (Je.EVENTS_URL && fn || Je.ACCESS_TOKEN && Array.isArray(ut) && ut.some(function(Sn) {
                      return Lt(Sn) || vt(Sn);
                    })) && this.queueRequest({ id: kt, timestamp: Date.now() }, fn);
                  }, je.prototype.processRequests = function(ut) {
                    var kt = this;
                    if (!this.pendingRequest && this.queue.length !== 0) {
                      var Jt = this.queue.shift(), fn = Jt.id, Sn = Jt.timestamp;
                      fn && this.success[fn] || (this.anonId || this.fetchEventData(), Se(this.anonId) || (this.anonId = Ce()), this.postEvent(Sn, { skuToken: this.skuToken }, function(Hn) {
                        Hn || fn && (kt.success[fn] = !0);
                      }, ut));
                    }
                  }, je;
                }(Ft), Kt = function(Ne) {
                  function je(ut) {
                    Ne.call(this, "appUserTurnstile"), this._customAccessToken = ut;
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.postTurnstileEvent = function(ut, kt) {
                    Je.EVENTS_URL && Je.ACCESS_TOKEN && Array.isArray(ut) && ut.some(function(Jt) {
                      return Lt(Jt) || vt(Jt);
                    }) && this.queueRequest(Date.now(), kt);
                  }, je.prototype.processRequests = function(ut) {
                    var kt = this;
                    if (!this.pendingRequest && this.queue.length !== 0) {
                      this.anonId && this.eventData.lastSuccess && this.eventData.tokenU || this.fetchEventData();
                      var Jt = Gt(Je.ACCESS_TOKEN), fn = Jt ? Jt.u : Je.ACCESS_TOKEN, Sn = fn !== this.eventData.tokenU;
                      Se(this.anonId) || (this.anonId = Ce(), Sn = !0);
                      var Hn = this.queue.shift();
                      if (this.eventData.lastSuccess) {
                        var nr = new Date(this.eventData.lastSuccess), kr = new Date(Hn), Ir = (Hn - this.eventData.lastSuccess) / 864e5;
                        Sn = Sn || Ir >= 1 || Ir < -1 || nr.getDate() !== kr.getDate();
                      } else Sn = !0;
                      if (!Sn) return this.processRequests();
                      this.postEvent(Hn, { "enabled.telemetry": !1 }, function(qr) {
                        qr || (kt.eventData.lastSuccess = Hn, kt.eventData.tokenU = fn);
                      }, ut);
                    }
                  }, je;
                }(Ft), Vt = new Kt(), Yt = Vt.postTurnstileEvent.bind(Vt), sn = new Ct(), hn = sn.postMapLoadEvent.bind(sn), Qt = "mapbox-tiles", Ut = 500, Wt = 50, rn = 42e4;
                function vn() {
                  de.caches && !Tt && (Tt = de.caches.open(Qt));
                }
                function dn(Ne, je, ut) {
                  if (vn(), Tt) {
                    var kt = { status: je.status, statusText: je.statusText, headers: new de.Headers() };
                    je.headers.forEach(function(fn, Sn) {
                      return kt.headers.set(Sn, fn);
                    });
                    var Jt = $e(je.headers.get("Cache-Control") || "");
                    Jt["no-store"] || (Jt["max-age"] && kt.headers.set("Expires", new Date(ut + 1e3 * Jt["max-age"]).toUTCString()), new Date(kt.headers.get("Expires")).getTime() - ut < rn || function(fn, Sn) {
                      if (It === void 0) try {
                        new Response(new ReadableStream()), It = !0;
                      } catch {
                        It = !1;
                      }
                      It ? Sn(fn.body) : fn.blob().then(Sn);
                    }(je, function(fn) {
                      var Sn = new de.Response(fn, kt);
                      vn(), Tt && Tt.then(function(Hn) {
                        return Hn.put(Ht(Ne.url), Sn);
                      }).catch(function(Hn) {
                        return Me(Hn.message);
                      });
                    }));
                  }
                }
                function Ht(Ne) {
                  var je = Ne.indexOf("?");
                  return je < 0 ? Ne : Ne.slice(0, je);
                }
                function jt(Ne, je) {
                  if (vn(), !Tt) return je(null);
                  var ut = Ht(Ne.url);
                  Tt.then(function(kt) {
                    kt.match(ut).then(function(Jt) {
                      var fn = function(Sn) {
                        if (!Sn) return !1;
                        var Hn = new Date(Sn.headers.get("Expires") || 0), nr = $e(Sn.headers.get("Cache-Control") || "");
                        return Hn > Date.now() && !nr["no-cache"];
                      }(Jt);
                      kt.delete(ut), fn && kt.put(ut, Jt.clone()), je(null, Jt, fn);
                    }).catch(je);
                  }).catch(je);
                }
                var cn, wn = 1 / 0;
                function Rn() {
                  return cn == null && (cn = de.OffscreenCanvas && new de.OffscreenCanvas(1, 1).getContext("2d") && typeof de.createImageBitmap == "function"), cn;
                }
                var Pn = { Unknown: "Unknown", Style: "Style", Source: "Source", Tile: "Tile", Glyphs: "Glyphs", SpriteImage: "SpriteImage", SpriteJSON: "SpriteJSON", Image: "Image" };
                typeof Object.freeze == "function" && Object.freeze(Pn);
                var Ie = function(Ne) {
                  function je(ut, kt, Jt) {
                    kt === 401 && vt(Jt) && (ut += ": you may have provided an invalid Mapbox access token. See https://www.mapbox.com/api-documentation/#access-tokens-and-token-scopes"), Ne.call(this, ut), this.status = kt, this.url = Jt, this.name = this.constructor.name, this.message = ut;
                  }
                  return Ne && (je.__proto__ = Ne), je.prototype = Object.create(Ne && Ne.prototype), je.prototype.constructor = je, je.prototype.toString = function() {
                    return this.name + ": " + this.message + " (" + this.status + "): " + this.url;
                  }, je;
                }(Error), rt = Pe() ? function() {
                  return self.worker && self.worker.referrer;
                } : function() {
                  return (de.location.protocol === "blob:" ? de.parent : de).location.href;
                };
                function ft(Ne, je) {
                  var ut, kt = new de.AbortController(), Jt = new de.Request(Ne.url, { method: Ne.method || "GET", body: Ne.body, credentials: Ne.credentials, headers: Ne.headers, referrer: rt(), signal: kt.signal }), fn = !1, Sn = !1, Hn = (ut = Jt.url).indexOf("sku=") > 0 && vt(ut);
                  Ne.type === "json" && Jt.headers.set("Accept", "application/json");
                  var nr = function(Ir, qr, ci) {
                    if (!Sn) {
                      if (Ir && Ir.message !== "SecurityError" && Me(Ir), qr && ci) return kr(qr);
                      var ni = Date.now();
                      de.fetch(Jt).then(function(Ii) {
                        if (Ii.ok) {
                          var Gi = Hn ? Ii.clone() : null;
                          return kr(Ii, Gi, ni);
                        }
                        return je(new Ie(Ii.statusText, Ii.status, Ne.url));
                      }).catch(function(Ii) {
                        Ii.code !== 20 && je(new Error(Ii.message));
                      });
                    }
                  }, kr = function(Ir, qr, ci) {
                    (Ne.type === "arrayBuffer" ? Ir.arrayBuffer() : Ne.type === "json" ? Ir.json() : Ir.text()).then(function(ni) {
                      Sn || (qr && ci && dn(Jt, qr, ci), fn = !0, je(null, ni, Ir.headers.get("Cache-Control"), Ir.headers.get("Expires")));
                    }).catch(function(ni) {
                      Sn || je(new Error(ni.message));
                    });
                  };
                  return Hn ? jt(Jt, nr) : nr(null, null), { cancel: function() {
                    Sn = !0, fn || kt.abort();
                  } };
                }
                var pt = function(Ne, je) {
                  if (ut = Ne.url, !(/^file:/.test(ut) || /^file:/.test(rt()) && !/^\w+:/.test(ut))) {
                    if (de.fetch && de.Request && de.AbortController && de.Request.prototype.hasOwnProperty("signal")) return ft(Ne, je);
                    if (Pe() && self.worker && self.worker.actor)
                      return self.worker.actor.send("getResource", Ne, je, void 0, !0);
                  }
                  var ut;
                  return function(kt, Jt) {
                    var fn = new de.XMLHttpRequest();
                    for (var Sn in fn.open(kt.method || "GET", kt.url, !0), kt.type === "arrayBuffer" && (fn.responseType = "arraybuffer"), kt.headers) fn.setRequestHeader(Sn, kt.headers[Sn]);
                    return kt.type === "json" && (fn.responseType = "text", fn.setRequestHeader("Accept", "application/json")), fn.withCredentials = kt.credentials === "include", fn.onerror = function() {
                      Jt(new Error(fn.statusText));
                    }, fn.onload = function() {
                      if ((fn.status >= 200 && fn.status < 300 || fn.status === 0) && fn.response !== null) {
                        var Hn = fn.response;
                        if (kt.type === "json") try {
                          Hn = JSON.parse(fn.response);
                        } catch (nr) {
                          return Jt(nr);
                        }
                        Jt(null, Hn, fn.getResponseHeader("Cache-Control"), fn.getResponseHeader("Expires"));
                      } else Jt(new Ie(fn.statusText, fn.status, kt.url));
                    }, fn.send(kt.body), { cancel: function() {
                      return fn.abort();
                    } };
                  }(Ne, je);
                }, wt = function(Ne, je) {
                  return pt(ge(Ne, { type: "arrayBuffer" }), je);
                }, Bt = function(Ne, je) {
                  return pt(ge(Ne, { method: "POST" }), je);
                }, At, Dt, Ot = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAC0lEQVQYV2NgAAIAAAUAAarVyFEAAAAASUVORK5CYII=";
                At = [], Dt = 0;
                var un = function(Ne, je) {
                  if (it.supported && (Ne.headers || (Ne.headers = {}), Ne.headers.accept = "image/webp,*/*"), Dt >= Je.MAX_PARALLEL_IMAGE_REQUESTS) {
                    var ut = { requestParameters: Ne, callback: je, cancelled: !1, cancel: function() {
                      this.cancelled = !0;
                    } };
                    return At.push(ut), ut;
                  }
                  Dt++;
                  var kt = !1, Jt = function() {
                    if (!kt) for (kt = !0, Dt--; At.length && Dt < Je.MAX_PARALLEL_IMAGE_REQUESTS; ) {
                      var Sn = At.shift(), Hn = Sn.requestParameters, nr = Sn.callback;
                      Sn.cancelled || (Sn.cancel = un(Hn, nr).cancel);
                    }
                  }, fn = wt(Ne, function(Sn, Hn, nr, kr) {
                    Jt(), Sn ? je(Sn) : Hn && (Rn() ? function(Ir, qr) {
                      var ci = new de.Blob([new Uint8Array(Ir)], { type: "image/png" });
                      de.createImageBitmap(ci).then(function(ni) {
                        qr(null, ni);
                      }).catch(function(ni) {
                        qr(new Error("Could not load image because of " + ni.message + ". Please make sure to use a supported image type such as PNG or JPEG. Note that SVGs are not supported."));
                      });
                    }(Hn, je) : function(Ir, qr, ci, ni) {
                      var Ii = new de.Image(), Gi = de.URL;
                      Ii.onload = function() {
                        qr(null, Ii), Gi.revokeObjectURL(Ii.src), Ii.onload = null, de.requestAnimationFrame(function() {
                          Ii.src = Ot;
                        });
                      }, Ii.onerror = function() {
                        return qr(new Error("Could not load image. Please make sure to use a supported image type such as PNG or JPEG. Note that SVGs are not supported."));
                      };
                      var xa = new de.Blob([new Uint8Array(Ir)], { type: "image/png" });
                      Ii.cacheControl = ci, Ii.expires = ni, Ii.src = Ir.byteLength ? Gi.createObjectURL(xa) : Ot;
                    }(Hn, je, nr, kr));
                  });
                  return { cancel: function() {
                    fn.cancel(), Jt();
                  } };
                };
                function yn(Ne, je, ut) {
                  ut[Ne] && ut[Ne].indexOf(je) !== -1 || (ut[Ne] = ut[Ne] || [], ut[Ne].push(je));
                }
                function _n(Ne, je, ut) {
                  if (ut && ut[Ne]) {
                    var kt = ut[Ne].indexOf(je);
                    kt !== -1 && ut[Ne].splice(kt, 1);
                  }
                }
                var gn = function(Ne, je) {
                  je === void 0 && (je = {}), ge(this, je), this.type = Ne;
                }, bn = function(Ne) {
                  function je(ut, kt) {
                    kt === void 0 && (kt = {}), Ne.call(this, "error", ge({ error: ut }, kt));
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je;
                }(gn), an = function() {
                };
                an.prototype.on = function(Ne, je) {
                  return this._listeners = this._listeners || {}, yn(Ne, je, this._listeners), this;
                }, an.prototype.off = function(Ne, je) {
                  return _n(Ne, je, this._listeners), _n(Ne, je, this._oneTimeListeners), this;
                }, an.prototype.once = function(Ne, je) {
                  return this._oneTimeListeners = this._oneTimeListeners || {}, yn(Ne, je, this._oneTimeListeners), this;
                }, an.prototype.fire = function(Ne, je) {
                  typeof Ne == "string" && (Ne = new gn(Ne, je || {}));
                  var ut = Ne.type;
                  if (this.listens(ut)) {
                    Ne.target = this;
                    for (var kt = 0, Jt = this._listeners && this._listeners[ut] ? this._listeners[ut].slice() : []; kt < Jt.length; kt += 1) Jt[kt].call(this, Ne);
                    for (var fn = 0, Sn = this._oneTimeListeners && this._oneTimeListeners[ut] ? this._oneTimeListeners[ut].slice() : []; fn < Sn.length; fn += 1) {
                      var Hn = Sn[fn];
                      _n(ut, Hn, this._oneTimeListeners), Hn.call(this, Ne);
                    }
                    var nr = this._eventedParent;
                    nr && (ge(Ne, typeof this._eventedParentData == "function" ? this._eventedParentData() : this._eventedParentData), nr.fire(Ne));
                  } else Ne instanceof bn && console.error(Ne.error);
                  return this;
                }, an.prototype.listens = function(Ne) {
                  return this._listeners && this._listeners[Ne] && this._listeners[Ne].length > 0 || this._oneTimeListeners && this._oneTimeListeners[Ne] && this._oneTimeListeners[Ne].length > 0 || this._eventedParent && this._eventedParent.listens(Ne);
                }, an.prototype.setEventedParent = function(Ne, je) {
                  return this._eventedParent = Ne, this._eventedParentData = je, this;
                };
                var Zt = { $version: 8, $root: { version: { required: !0, type: "enum", values: [8] }, name: { type: "string" }, metadata: { type: "*" }, center: { type: "array", value: "number" }, zoom: { type: "number" }, bearing: { type: "number", default: 0, period: 360, units: "degrees" }, pitch: { type: "number", default: 0, units: "degrees" }, light: { type: "light" }, sources: { required: !0, type: "sources" }, sprite: { type: "string" }, glyphs: { type: "string" }, transition: { type: "transition" }, layers: { required: !0, type: "array", value: "layer" } }, sources: { "*": { type: "source" } }, source: ["source_vector", "source_raster", "source_raster_dem", "source_geojson", "source_video", "source_image"], source_vector: { type: { required: !0, type: "enum", values: { vector: {} } }, url: { type: "string" }, tiles: { type: "array", value: "string" }, bounds: { type: "array", value: "number", length: 4, default: [-180, -85.051129, 180, 85.051129] }, scheme: { type: "enum", values: { xyz: {}, tms: {} }, default: "xyz" }, minzoom: { type: "number", default: 0 }, maxzoom: { type: "number", default: 22 }, attribution: { type: "string" }, promoteId: { type: "promoteId" }, volatile: { type: "boolean", default: !1 }, "*": { type: "*" } }, source_raster: { type: { required: !0, type: "enum", values: { raster: {} } }, url: { type: "string" }, tiles: { type: "array", value: "string" }, bounds: { type: "array", value: "number", length: 4, default: [-180, -85.051129, 180, 85.051129] }, minzoom: { type: "number", default: 0 }, maxzoom: { type: "number", default: 22 }, tileSize: { type: "number", default: 512, units: "pixels" }, scheme: { type: "enum", values: { xyz: {}, tms: {} }, default: "xyz" }, attribution: { type: "string" }, volatile: { type: "boolean", default: !1 }, "*": { type: "*" } }, source_raster_dem: { type: { required: !0, type: "enum", values: { "raster-dem": {} } }, url: { type: "string" }, tiles: { type: "array", value: "string" }, bounds: { type: "array", value: "number", length: 4, default: [-180, -85.051129, 180, 85.051129] }, minzoom: { type: "number", default: 0 }, maxzoom: { type: "number", default: 22 }, tileSize: { type: "number", default: 512, units: "pixels" }, attribution: { type: "string" }, encoding: { type: "enum", values: { terrarium: {}, mapbox: {} }, default: "mapbox" }, volatile: { type: "boolean", default: !1 }, "*": { type: "*" } }, source_geojson: { type: { required: !0, type: "enum", values: { geojson: {} } }, data: { type: "*" }, maxzoom: { type: "number", default: 18 }, attribution: { type: "string" }, buffer: { type: "number", default: 128, maximum: 512, minimum: 0 }, filter: { type: "*" }, tolerance: { type: "number", default: 0.375 }, cluster: { type: "boolean", default: !1 }, clusterRadius: { type: "number", default: 50, minimum: 0 }, clusterMaxZoom: { type: "number" }, clusterMinPoints: { type: "number" }, clusterProperties: { type: "*" }, lineMetrics: { type: "boolean", default: !1 }, generateId: { type: "boolean", default: !1 }, promoteId: { type: "promoteId" } }, source_video: { type: { required: !0, type: "enum", values: { video: {} } }, urls: { required: !0, type: "array", value: "string" }, coordinates: { required: !0, type: "array", length: 4, value: { type: "array", length: 2, value: "number" } } }, source_image: { type: { required: !0, type: "enum", values: { image: {} } }, url: { required: !0, type: "string" }, coordinates: { required: !0, type: "array", length: 4, value: { type: "array", length: 2, value: "number" } } }, layer: { id: { type: "string", required: !0 }, type: { type: "enum", values: { fill: {}, line: {}, symbol: {}, circle: {}, heatmap: {}, "fill-extrusion": {}, raster: {}, hillshade: {}, background: {} }, required: !0 }, metadata: { type: "*" }, source: { type: "string" }, "source-layer": { type: "string" }, minzoom: { type: "number", minimum: 0, maximum: 24 }, maxzoom: { type: "number", minimum: 0, maximum: 24 }, filter: { type: "filter" }, layout: { type: "layout" }, paint: { type: "paint" } }, layout: ["layout_fill", "layout_line", "layout_circle", "layout_heatmap", "layout_fill-extrusion", "layout_symbol", "layout_raster", "layout_hillshade", "layout_background"], layout_background: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_fill: { "fill-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_circle: { "circle-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_heatmap: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, "layout_fill-extrusion": { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_line: { "line-cap": { type: "enum", values: { butt: {}, round: {}, square: {} }, default: "butt", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-join": { type: "enum", values: { bevel: {}, round: {}, miter: {} }, default: "miter", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "line-miter-limit": { type: "number", default: 2, requires: [{ "line-join": "miter" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-round-limit": { type: "number", default: 1.05, requires: [{ "line-join": "round" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_symbol: { "symbol-placement": { type: "enum", values: { point: {}, line: {}, "line-center": {} }, default: "point", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "symbol-spacing": { type: "number", default: 250, minimum: 1, units: "pixels", requires: [{ "symbol-placement": "line" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "symbol-avoid-edges": { type: "boolean", default: !1, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "symbol-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "symbol-z-order": { type: "enum", values: { auto: {}, "viewport-y": {}, source: {} }, default: "auto", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-allow-overlap": { type: "boolean", default: !1, requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-ignore-placement": { type: "boolean", default: !1, requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-optional": { type: "boolean", default: !1, requires: ["icon-image", "text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-rotation-alignment": { type: "enum", values: { map: {}, viewport: {}, auto: {} }, default: "auto", requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-size": { type: "number", default: 1, minimum: 0, units: "factor of the original icon size", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-text-fit": { type: "enum", values: { none: {}, width: {}, height: {}, both: {} }, default: "none", requires: ["icon-image", "text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-text-fit-padding": { type: "array", value: "number", length: 4, default: [0, 0, 0, 0], units: "pixels", requires: ["icon-image", "text-field", { "icon-text-fit": ["both", "width", "height"] }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-image": { type: "resolvedImage", tokens: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-rotate": { type: "number", default: 0, period: 360, units: "degrees", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-padding": { type: "number", default: 2, minimum: 0, units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-keep-upright": { type: "boolean", default: !1, requires: ["icon-image", { "icon-rotation-alignment": "map" }, { "symbol-placement": ["line", "line-center"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-offset": { type: "array", value: "number", length: 2, default: [0, 0], requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-anchor": { type: "enum", values: { center: {}, left: {}, right: {}, top: {}, bottom: {}, "top-left": {}, "top-right": {}, "bottom-left": {}, "bottom-right": {} }, default: "center", requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-pitch-alignment": { type: "enum", values: { map: {}, viewport: {}, auto: {} }, default: "auto", requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-pitch-alignment": { type: "enum", values: { map: {}, viewport: {}, auto: {} }, default: "auto", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-rotation-alignment": { type: "enum", values: { map: {}, viewport: {}, auto: {} }, default: "auto", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-field": { type: "formatted", default: "", tokens: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-font": { type: "array", value: "string", default: ["Open Sans Regular", "Arial Unicode MS Regular"], requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-size": { type: "number", default: 16, minimum: 0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-max-width": { type: "number", default: 10, minimum: 0, units: "ems", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-line-height": { type: "number", default: 1.2, units: "ems", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-letter-spacing": { type: "number", default: 0, units: "ems", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-justify": { type: "enum", values: { auto: {}, left: {}, center: {}, right: {} }, default: "center", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-radial-offset": { type: "number", units: "ems", default: 0, requires: ["text-field"], "property-type": "data-driven", expression: { interpolated: !0, parameters: ["zoom", "feature"] } }, "text-variable-anchor": { type: "array", value: "enum", values: { center: {}, left: {}, right: {}, top: {}, bottom: {}, "top-left": {}, "top-right": {}, "bottom-left": {}, "bottom-right": {} }, requires: ["text-field", { "symbol-placement": ["point"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-anchor": { type: "enum", values: { center: {}, left: {}, right: {}, top: {}, bottom: {}, "top-left": {}, "top-right": {}, "bottom-left": {}, "bottom-right": {} }, default: "center", requires: ["text-field", { "!": "text-variable-anchor" }], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-max-angle": { type: "number", default: 45, units: "degrees", requires: ["text-field", { "symbol-placement": ["line", "line-center"] }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-writing-mode": { type: "array", value: "enum", values: { horizontal: {}, vertical: {} }, requires: ["text-field", { "symbol-placement": ["point"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-rotate": { type: "number", default: 0, period: 360, units: "degrees", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-padding": { type: "number", default: 2, minimum: 0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-keep-upright": { type: "boolean", default: !0, requires: ["text-field", { "text-rotation-alignment": "map" }, { "symbol-placement": ["line", "line-center"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-transform": { type: "enum", values: { none: {}, uppercase: {}, lowercase: {} }, default: "none", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-offset": { type: "array", value: "number", units: "ems", length: 2, default: [0, 0], requires: ["text-field", { "!": "text-radial-offset" }], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-allow-overlap": { type: "boolean", default: !1, requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-ignore-placement": { type: "boolean", default: !1, requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-optional": { type: "boolean", default: !1, requires: ["text-field", "icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_raster: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_hillshade: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, filter: { type: "array", value: "*" }, filter_operator: { type: "enum", values: { "==": {}, "!=": {}, ">": {}, ">=": {}, "<": {}, "<=": {}, in: {}, "!in": {}, all: {}, any: {}, none: {}, has: {}, "!has": {}, within: {} } }, geometry_type: { type: "enum", values: { Point: {}, LineString: {}, Polygon: {} } }, function: { expression: { type: "expression" }, stops: { type: "array", value: "function_stop" }, base: { type: "number", default: 1, minimum: 0 }, property: { type: "string", default: "$zoom" }, type: { type: "enum", values: { identity: {}, exponential: {}, interval: {}, categorical: {} }, default: "exponential" }, colorSpace: { type: "enum", values: { rgb: {}, lab: {}, hcl: {} }, default: "rgb" }, default: { type: "*", required: !1 } }, function_stop: { type: "array", minimum: 0, maximum: 24, value: ["number", "color"], length: 2 }, expression: { type: "array", value: "*", minimum: 1 }, light: { anchor: { type: "enum", default: "viewport", values: { map: {}, viewport: {} }, "property-type": "data-constant", transition: !1, expression: { interpolated: !1, parameters: ["zoom"] } }, position: { type: "array", default: [1.15, 210, 30], length: 3, value: "number", "property-type": "data-constant", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] } }, color: { type: "color", "property-type": "data-constant", default: "#ffffff", expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, intensity: { type: "number", "property-type": "data-constant", default: 0.5, minimum: 0, maximum: 1, expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 } }, paint: ["paint_fill", "paint_line", "paint_circle", "paint_heatmap", "paint_fill-extrusion", "paint_symbol", "paint_raster", "paint_hillshade", "paint_background"], paint_fill: { "fill-antialias": { type: "boolean", default: !0, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "fill-pattern" }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-outline-color": { type: "color", transition: !0, requires: [{ "!": "fill-pattern" }, { "fill-antialias": !0 }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["fill-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "cross-faded-data-driven" } }, "paint_fill-extrusion": { "fill-extrusion-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-extrusion-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "fill-extrusion-pattern" }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-extrusion-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-extrusion-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["fill-extrusion-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-extrusion-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "cross-faded-data-driven" }, "fill-extrusion-height": { type: "number", default: 0, minimum: 0, units: "meters", transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-extrusion-base": { type: "number", default: 0, minimum: 0, units: "meters", transition: !0, requires: ["fill-extrusion-height"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-extrusion-vertical-gradient": { type: "boolean", default: !0, transition: !1, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_line: { "line-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "line-pattern" }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["line-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-width": { type: "number", default: 1, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-gap-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-offset": { type: "number", default: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-blur": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-dasharray": { type: "array", value: "number", minimum: 0, transition: !0, units: "line widths", requires: [{ "!": "line-pattern" }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "cross-faded" }, "line-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "cross-faded-data-driven" }, "line-gradient": { type: "color", transition: !1, requires: [{ "!": "line-dasharray" }, { "!": "line-pattern" }, { source: "geojson", has: { lineMetrics: !0 } }], expression: { interpolated: !0, parameters: ["line-progress"] }, "property-type": "color-ramp" } }, paint_circle: { "circle-radius": { type: "number", default: 5, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-blur": { type: "number", default: 0, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["circle-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-pitch-scale": { type: "enum", values: { map: {}, viewport: {} }, default: "map", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-pitch-alignment": { type: "enum", values: { map: {}, viewport: {} }, default: "viewport", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-stroke-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-stroke-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-stroke-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" } }, paint_heatmap: { "heatmap-radius": { type: "number", default: 30, minimum: 1, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "heatmap-weight": { type: "number", default: 1, minimum: 0, transition: !1, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "heatmap-intensity": { type: "number", default: 1, minimum: 0, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "heatmap-color": { type: "color", default: ["interpolate", ["linear"], ["heatmap-density"], 0, "rgba(0, 0, 255, 0)", 0.1, "royalblue", 0.3, "cyan", 0.5, "lime", 0.7, "yellow", 1, "red"], transition: !1, expression: { interpolated: !0, parameters: ["heatmap-density"] }, "property-type": "color-ramp" }, "heatmap-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_symbol: { "icon-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-color": { type: "color", default: "#000000", transition: !0, requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-halo-color": { type: "color", default: "rgba(0, 0, 0, 0)", transition: !0, requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-halo-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-halo-blur": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["icon-image", "icon-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-color": { type: "color", default: "#000000", transition: !0, overridable: !0, requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-halo-color": { type: "color", default: "rgba(0, 0, 0, 0)", transition: !0, requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-halo-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-halo-blur": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["text-field", "text-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_raster: { "raster-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-hue-rotate": { type: "number", default: 0, period: 360, transition: !0, units: "degrees", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-brightness-min": { type: "number", default: 0, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-brightness-max": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-saturation": { type: "number", default: 0, minimum: -1, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-contrast": { type: "number", default: 0, minimum: -1, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-resampling": { type: "enum", values: { linear: {}, nearest: {} }, default: "linear", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-fade-duration": { type: "number", default: 300, minimum: 0, transition: !1, units: "milliseconds", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_hillshade: { "hillshade-illumination-direction": { type: "number", default: 335, minimum: 0, maximum: 359, transition: !1, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-illumination-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "viewport", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-exaggeration": { type: "number", default: 0.5, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-shadow-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-highlight-color": { type: "color", default: "#FFFFFF", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-accent-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_background: { "background-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "background-pattern" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "background-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "cross-faded" }, "background-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, transition: { duration: { type: "number", default: 300, minimum: 0, units: "milliseconds" }, delay: { type: "number", default: 0, minimum: 0, units: "milliseconds" } }, "property-type": { "data-driven": { type: "property-type" }, "cross-faded": { type: "property-type" }, "cross-faded-data-driven": { type: "property-type" }, "color-ramp": { type: "property-type" }, "data-constant": { type: "property-type" }, constant: { type: "property-type" } }, promoteId: { "*": { type: "string" } } }, Mn = function(Ne, je, ut, kt) {
                  this.message = (Ne ? Ne + ": " : "") + ut, kt && (this.identifier = kt), je != null && je.__line__ && (this.line = je.__line__);
                };
                function Ln(Ne) {
                  var je = Ne.key, ut = Ne.value;
                  return ut ? [new Mn(je, ut, "constants have been deprecated as of v8")] : [];
                }
                function In(Ne) {
                  for (var je = [], ut = arguments.length - 1; ut-- > 0; ) je[ut] = arguments[ut + 1];
                  for (var kt = 0, Jt = je; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    for (var Sn in fn) Ne[Sn] = fn[Sn];
                  }
                  return Ne;
                }
                function kn(Ne) {
                  return Ne instanceof Number || Ne instanceof String || Ne instanceof Boolean ? Ne.valueOf() : Ne;
                }
                function er(Ne) {
                  if (Array.isArray(Ne)) return Ne.map(er);
                  if (Ne instanceof Object && !(Ne instanceof Number || Ne instanceof String || Ne instanceof Boolean)) {
                    var je = {};
                    for (var ut in Ne) je[ut] = er(Ne[ut]);
                    return je;
                  }
                  return kn(Ne);
                }
                var jn = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, kt), this.message = kt, this.key = ut;
                  }
                  return Ne && (je.__proto__ = Ne), je.prototype = Object.create(Ne && Ne.prototype), je.prototype.constructor = je, je;
                }(Error), ur = function(Ne, je) {
                  je === void 0 && (je = []), this.parent = Ne, this.bindings = {};
                  for (var ut = 0, kt = je; ut < kt.length; ut += 1) {
                    var Jt = kt[ut], fn = Jt[0], Sn = Jt[1];
                    this.bindings[fn] = Sn;
                  }
                };
                ur.prototype.concat = function(Ne) {
                  return new ur(this, Ne);
                }, ur.prototype.get = function(Ne) {
                  if (this.bindings[Ne]) return this.bindings[Ne];
                  if (this.parent) return this.parent.get(Ne);
                  throw new Error(Ne + " not found in scope.");
                }, ur.prototype.has = function(Ne) {
                  return !!this.bindings[Ne] || !!this.parent && this.parent.has(Ne);
                };
                var Kn = { kind: "null" }, Jn = { kind: "number" }, Lr = { kind: "string" }, mr = { kind: "boolean" }, Ar = { kind: "color" }, Yr = { kind: "object" }, jr = { kind: "value" }, Mr = { kind: "collator" }, Wr = { kind: "formatted" }, Bi = { kind: "resolvedImage" };
                function gi(Ne, je) {
                  return { kind: "array", itemType: Ne, N: je };
                }
                function Zr(Ne) {
                  if (Ne.kind === "array") {
                    var je = Zr(Ne.itemType);
                    return typeof Ne.N == "number" ? "array<" + je + ", " + Ne.N + ">" : Ne.itemType.kind === "value" ? "array" : "array<" + je + ">";
                  }
                  return Ne.kind;
                }
                var fi = [Kn, Jn, Lr, mr, Ar, Wr, Yr, gi(jr), Bi];
                function ri(Ne, je) {
                  if (je.kind === "error") return null;
                  if (Ne.kind === "array") {
                    if (je.kind === "array" && (je.N === 0 && je.itemType.kind === "value" || !ri(Ne.itemType, je.itemType)) && (typeof Ne.N != "number" || Ne.N === je.N)) return null;
                  } else {
                    if (Ne.kind === je.kind) return null;
                    if (Ne.kind === "value") {
                      for (var ut = 0, kt = fi; ut < kt.length; ut += 1) if (!ri(kt[ut], je)) return null;
                    }
                  }
                  return "Expected " + Zr(Ne) + " but found " + Zr(je) + " instead.";
                }
                function bi(Ne, je) {
                  return je.some(function(ut) {
                    return ut.kind === Ne.kind;
                  });
                }
                function Zi(Ne, je) {
                  return je.some(function(ut) {
                    return ut === "null" ? Ne === null : ut === "array" ? Array.isArray(Ne) : ut === "object" ? Ne && !Array.isArray(Ne) && typeof Ne == "object" : ut === typeof Ne;
                  });
                }
                var $i = ae(function(Ne, je) {
                  var ut = { transparent: [0, 0, 0, 0], aliceblue: [240, 248, 255, 1], antiquewhite: [250, 235, 215, 1], aqua: [0, 255, 255, 1], aquamarine: [127, 255, 212, 1], azure: [240, 255, 255, 1], beige: [245, 245, 220, 1], bisque: [255, 228, 196, 1], black: [0, 0, 0, 1], blanchedalmond: [255, 235, 205, 1], blue: [0, 0, 255, 1], blueviolet: [138, 43, 226, 1], brown: [165, 42, 42, 1], burlywood: [222, 184, 135, 1], cadetblue: [95, 158, 160, 1], chartreuse: [127, 255, 0, 1], chocolate: [210, 105, 30, 1], coral: [255, 127, 80, 1], cornflowerblue: [100, 149, 237, 1], cornsilk: [255, 248, 220, 1], crimson: [220, 20, 60, 1], cyan: [0, 255, 255, 1], darkblue: [0, 0, 139, 1], darkcyan: [0, 139, 139, 1], darkgoldenrod: [184, 134, 11, 1], darkgray: [169, 169, 169, 1], darkgreen: [0, 100, 0, 1], darkgrey: [169, 169, 169, 1], darkkhaki: [189, 183, 107, 1], darkmagenta: [139, 0, 139, 1], darkolivegreen: [85, 107, 47, 1], darkorange: [255, 140, 0, 1], darkorchid: [153, 50, 204, 1], darkred: [139, 0, 0, 1], darksalmon: [233, 150, 122, 1], darkseagreen: [143, 188, 143, 1], darkslateblue: [72, 61, 139, 1], darkslategray: [47, 79, 79, 1], darkslategrey: [47, 79, 79, 1], darkturquoise: [0, 206, 209, 1], darkviolet: [148, 0, 211, 1], deeppink: [255, 20, 147, 1], deepskyblue: [0, 191, 255, 1], dimgray: [105, 105, 105, 1], dimgrey: [105, 105, 105, 1], dodgerblue: [30, 144, 255, 1], firebrick: [178, 34, 34, 1], floralwhite: [255, 250, 240, 1], forestgreen: [34, 139, 34, 1], fuchsia: [255, 0, 255, 1], gainsboro: [220, 220, 220, 1], ghostwhite: [248, 248, 255, 1], gold: [255, 215, 0, 1], goldenrod: [218, 165, 32, 1], gray: [128, 128, 128, 1], green: [0, 128, 0, 1], greenyellow: [173, 255, 47, 1], grey: [128, 128, 128, 1], honeydew: [240, 255, 240, 1], hotpink: [255, 105, 180, 1], indianred: [205, 92, 92, 1], indigo: [75, 0, 130, 1], ivory: [255, 255, 240, 1], khaki: [240, 230, 140, 1], lavender: [230, 230, 250, 1], lavenderblush: [255, 240, 245, 1], lawngreen: [124, 252, 0, 1], lemonchiffon: [255, 250, 205, 1], lightblue: [173, 216, 230, 1], lightcoral: [240, 128, 128, 1], lightcyan: [224, 255, 255, 1], lightgoldenrodyellow: [250, 250, 210, 1], lightgray: [211, 211, 211, 1], lightgreen: [144, 238, 144, 1], lightgrey: [211, 211, 211, 1], lightpink: [255, 182, 193, 1], lightsalmon: [255, 160, 122, 1], lightseagreen: [32, 178, 170, 1], lightskyblue: [135, 206, 250, 1], lightslategray: [119, 136, 153, 1], lightslategrey: [119, 136, 153, 1], lightsteelblue: [176, 196, 222, 1], lightyellow: [255, 255, 224, 1], lime: [0, 255, 0, 1], limegreen: [50, 205, 50, 1], linen: [250, 240, 230, 1], magenta: [255, 0, 255, 1], maroon: [128, 0, 0, 1], mediumaquamarine: [102, 205, 170, 1], mediumblue: [0, 0, 205, 1], mediumorchid: [186, 85, 211, 1], mediumpurple: [147, 112, 219, 1], mediumseagreen: [60, 179, 113, 1], mediumslateblue: [123, 104, 238, 1], mediumspringgreen: [0, 250, 154, 1], mediumturquoise: [72, 209, 204, 1], mediumvioletred: [199, 21, 133, 1], midnightblue: [25, 25, 112, 1], mintcream: [245, 255, 250, 1], mistyrose: [255, 228, 225, 1], moccasin: [255, 228, 181, 1], navajowhite: [255, 222, 173, 1], navy: [0, 0, 128, 1], oldlace: [253, 245, 230, 1], olive: [128, 128, 0, 1], olivedrab: [107, 142, 35, 1], orange: [255, 165, 0, 1], orangered: [255, 69, 0, 1], orchid: [218, 112, 214, 1], palegoldenrod: [238, 232, 170, 1], palegreen: [152, 251, 152, 1], paleturquoise: [175, 238, 238, 1], palevioletred: [219, 112, 147, 1], papayawhip: [255, 239, 213, 1], peachpuff: [255, 218, 185, 1], peru: [205, 133, 63, 1], pink: [255, 192, 203, 1], plum: [221, 160, 221, 1], powderblue: [176, 224, 230, 1], purple: [128, 0, 128, 1], rebeccapurple: [102, 51, 153, 1], red: [255, 0, 0, 1], rosybrown: [188, 143, 143, 1], royalblue: [65, 105, 225, 1], saddlebrown: [139, 69, 19, 1], salmon: [250, 128, 114, 1], sandybrown: [244, 164, 96, 1], seagreen: [46, 139, 87, 1], seashell: [255, 245, 238, 1], sienna: [160, 82, 45, 1], silver: [192, 192, 192, 1], skyblue: [135, 206, 235, 1], slateblue: [106, 90, 205, 1], slategray: [112, 128, 144, 1], slategrey: [112, 128, 144, 1], snow: [255, 250, 250, 1], springgreen: [0, 255, 127, 1], steelblue: [70, 130, 180, 1], tan: [210, 180, 140, 1], teal: [0, 128, 128, 1], thistle: [216, 191, 216, 1], tomato: [255, 99, 71, 1], turquoise: [64, 224, 208, 1], violet: [238, 130, 238, 1], wheat: [245, 222, 179, 1], white: [255, 255, 255, 1], whitesmoke: [245, 245, 245, 1], yellow: [255, 255, 0, 1], yellowgreen: [154, 205, 50, 1] };
                  function kt(nr) {
                    return (nr = Math.round(nr)) < 0 ? 0 : nr > 255 ? 255 : nr;
                  }
                  function Jt(nr) {
                    return nr < 0 ? 0 : nr > 1 ? 1 : nr;
                  }
                  function fn(nr) {
                    return nr[nr.length - 1] === "%" ? kt(parseFloat(nr) / 100 * 255) : kt(parseInt(nr));
                  }
                  function Sn(nr) {
                    return nr[nr.length - 1] === "%" ? Jt(parseFloat(nr) / 100) : Jt(parseFloat(nr));
                  }
                  function Hn(nr, kr, Ir) {
                    return Ir < 0 ? Ir += 1 : Ir > 1 && (Ir -= 1), 6 * Ir < 1 ? nr + (kr - nr) * Ir * 6 : 2 * Ir < 1 ? kr : 3 * Ir < 2 ? nr + (kr - nr) * (2 / 3 - Ir) * 6 : nr;
                  }
                  try {
                    je.parseCSSColor = function(nr) {
                      var kr, Ir = nr.replace(/ /g, "").toLowerCase();
                      if (Ir in ut) return ut[Ir].slice();
                      if (Ir[0] === "#") return Ir.length === 4 ? (kr = parseInt(Ir.substr(1), 16)) >= 0 && kr <= 4095 ? [(3840 & kr) >> 4 | (3840 & kr) >> 8, 240 & kr | (240 & kr) >> 4, 15 & kr | (15 & kr) << 4, 1] : null : Ir.length === 7 && (kr = parseInt(Ir.substr(1), 16)) >= 0 && kr <= 16777215 ? [(16711680 & kr) >> 16, (65280 & kr) >> 8, 255 & kr, 1] : null;
                      var qr = Ir.indexOf("("), ci = Ir.indexOf(")");
                      if (qr !== -1 && ci + 1 === Ir.length) {
                        var ni = Ir.substr(0, qr), Ii = Ir.substr(qr + 1, ci - (qr + 1)).split(","), Gi = 1;
                        switch (ni) {
                          case "rgba":
                            if (Ii.length !== 4) return null;
                            Gi = Sn(Ii.pop());
                          case "rgb":
                            return Ii.length !== 3 ? null : [fn(Ii[0]), fn(Ii[1]), fn(Ii[2]), Gi];
                          case "hsla":
                            if (Ii.length !== 4) return null;
                            Gi = Sn(Ii.pop());
                          case "hsl":
                            if (Ii.length !== 3) return null;
                            var xa = (parseFloat(Ii[0]) % 360 + 360) % 360 / 360, ga = Sn(Ii[1]), $a = Sn(Ii[2]), Va = $a <= 0.5 ? $a * (ga + 1) : $a + ga - $a * ga, eo = 2 * $a - Va;
                            return [kt(255 * Hn(eo, Va, xa + 1 / 3)), kt(255 * Hn(eo, Va, xa)), kt(255 * Hn(eo, Va, xa - 1 / 3)), Gi];
                          default:
                            return null;
                        }
                      }
                      return null;
                    };
                  } catch {
                  }
                }), Br = $i.parseCSSColor, Pr = function(Ne, je, ut, kt) {
                  kt === void 0 && (kt = 1), this.r = Ne, this.g = je, this.b = ut, this.a = kt;
                };
                Pr.parse = function(Ne) {
                  if (Ne) {
                    if (Ne instanceof Pr) return Ne;
                    if (typeof Ne == "string") {
                      var je = Br(Ne);
                      if (je) return new Pr(je[0] / 255 * je[3], je[1] / 255 * je[3], je[2] / 255 * je[3], je[3]);
                    }
                  }
                }, Pr.prototype.toString = function() {
                  var Ne = this.toArray(), je = Ne[0], ut = Ne[1], kt = Ne[2], Jt = Ne[3];
                  return "rgba(" + Math.round(je) + "," + Math.round(ut) + "," + Math.round(kt) + "," + Jt + ")";
                }, Pr.prototype.toArray = function() {
                  var Ne = this, je = Ne.r, ut = Ne.g, kt = Ne.b, Jt = Ne.a;
                  return Jt === 0 ? [0, 0, 0, 0] : [255 * je / Jt, 255 * ut / Jt, 255 * kt / Jt, Jt];
                }, Pr.black = new Pr(0, 0, 0, 1), Pr.white = new Pr(1, 1, 1, 1), Pr.transparent = new Pr(0, 0, 0, 0), Pr.red = new Pr(1, 0, 0, 1);
                var ei = function(Ne, je, ut) {
                  this.sensitivity = Ne ? je ? "variant" : "case" : je ? "accent" : "base", this.locale = ut, this.collator = new Intl.Collator(this.locale ? this.locale : [], { sensitivity: this.sensitivity, usage: "search" });
                };
                ei.prototype.compare = function(Ne, je) {
                  return this.collator.compare(Ne, je);
                }, ei.prototype.resolvedLocale = function() {
                  return new Intl.Collator(this.locale ? this.locale : []).resolvedOptions().locale;
                };
                var Ci = function(Ne, je, ut, kt, Jt) {
                  this.text = Ne, this.image = je, this.scale = ut, this.fontStack = kt, this.textColor = Jt;
                }, oa = function(Ne) {
                  this.sections = Ne;
                };
                oa.fromString = function(Ne) {
                  return new oa([new Ci(Ne, null, null, null, null)]);
                }, oa.prototype.isEmpty = function() {
                  return this.sections.length === 0 || !this.sections.some(function(Ne) {
                    return Ne.text.length !== 0 || Ne.image && Ne.image.name.length !== 0;
                  });
                }, oa.factory = function(Ne) {
                  return Ne instanceof oa ? Ne : oa.fromString(Ne);
                }, oa.prototype.toString = function() {
                  return this.sections.length === 0 ? "" : this.sections.map(function(Ne) {
                    return Ne.text;
                  }).join("");
                }, oa.prototype.serialize = function() {
                  for (var Ne = ["format"], je = 0, ut = this.sections; je < ut.length; je += 1) {
                    var kt = ut[je];
                    if (kt.image) Ne.push(["image", kt.image.name]);
                    else {
                      Ne.push(kt.text);
                      var Jt = {};
                      kt.fontStack && (Jt["text-font"] = ["literal", kt.fontStack.split(",")]), kt.scale && (Jt["font-scale"] = kt.scale), kt.textColor && (Jt["text-color"] = ["rgba"].concat(kt.textColor.toArray())), Ne.push(Jt);
                    }
                  }
                  return Ne;
                };
                var ka = function(Ne) {
                  this.name = Ne.name, this.available = Ne.available;
                };
                function aa(Ne, je, ut, kt) {
                  return typeof Ne == "number" && Ne >= 0 && Ne <= 255 && typeof je == "number" && je >= 0 && je <= 255 && typeof ut == "number" && ut >= 0 && ut <= 255 ? kt === void 0 || typeof kt == "number" && kt >= 0 && kt <= 1 ? null : "Invalid rgba value [" + [Ne, je, ut, kt].join(", ") + "]: 'a' must be between 0 and 1." : "Invalid rgba value [" + (typeof kt == "number" ? [Ne, je, ut, kt] : [Ne, je, ut]).join(", ") + "]: 'r', 'g', and 'b' must be between 0 and 255.";
                }
                function hi(Ne) {
                  if (Ne === null || typeof Ne == "string" || typeof Ne == "boolean" || typeof Ne == "number" || Ne instanceof Pr || Ne instanceof ei || Ne instanceof oa || Ne instanceof ka) return !0;
                  if (Array.isArray(Ne)) {
                    for (var je = 0, ut = Ne; je < ut.length; je += 1) if (!hi(ut[je])) return !1;
                    return !0;
                  }
                  if (typeof Ne == "object") {
                    for (var kt in Ne) if (!hi(Ne[kt])) return !1;
                    return !0;
                  }
                  return !1;
                }
                function Gr(Ne) {
                  if (Ne === null) return Kn;
                  if (typeof Ne == "string") return Lr;
                  if (typeof Ne == "boolean") return mr;
                  if (typeof Ne == "number") return Jn;
                  if (Ne instanceof Pr) return Ar;
                  if (Ne instanceof ei) return Mr;
                  if (Ne instanceof oa) return Wr;
                  if (Ne instanceof ka) return Bi;
                  if (Array.isArray(Ne)) {
                    for (var je, ut = Ne.length, kt = 0, Jt = Ne; kt < Jt.length; kt += 1) {
                      var fn = Gr(Jt[kt]);
                      if (je) {
                        if (je === fn) continue;
                        je = jr;
                        break;
                      }
                      je = fn;
                    }
                    return gi(je || jr, ut);
                  }
                  return Yr;
                }
                function Ca(Ne) {
                  var je = typeof Ne;
                  return Ne === null ? "" : je === "string" || je === "number" || je === "boolean" ? String(Ne) : Ne instanceof Pr || Ne instanceof oa || Ne instanceof ka ? Ne.toString() : JSON.stringify(Ne);
                }
                ka.prototype.toString = function() {
                  return this.name;
                }, ka.fromString = function(Ne) {
                  return Ne ? new ka({ name: Ne, available: !1 }) : null;
                }, ka.prototype.serialize = function() {
                  return ["image", this.name];
                };
                var Sr = function(Ne, je) {
                  this.type = Ne, this.value = je;
                };
                Sr.parse = function(Ne, je) {
                  if (Ne.length !== 2) return je.error("'literal' expression requires exactly one argument, but found " + (Ne.length - 1) + " instead.");
                  if (!hi(Ne[1])) return je.error("invalid value");
                  var ut = Ne[1], kt = Gr(ut), Jt = je.expectedType;
                  return kt.kind !== "array" || kt.N !== 0 || !Jt || Jt.kind !== "array" || typeof Jt.N == "number" && Jt.N !== 0 || (kt = Jt), new Sr(kt, ut);
                }, Sr.prototype.evaluate = function() {
                  return this.value;
                }, Sr.prototype.eachChild = function() {
                }, Sr.prototype.outputDefined = function() {
                  return !0;
                }, Sr.prototype.serialize = function() {
                  return this.type.kind === "array" || this.type.kind === "object" ? ["literal", this.value] : this.value instanceof Pr ? ["rgba"].concat(this.value.toArray()) : this.value instanceof oa ? this.value.serialize() : this.value;
                };
                var Dr = function(Ne) {
                  this.name = "ExpressionEvaluationError", this.message = Ne;
                };
                Dr.prototype.toJSON = function() {
                  return this.message;
                };
                var si = { string: Lr, number: Jn, boolean: mr, object: Yr }, zr = function(Ne, je) {
                  this.type = Ne, this.args = je;
                };
                zr.parse = function(Ne, je) {
                  if (Ne.length < 2) return je.error("Expected at least one argument.");
                  var ut, kt = 1, Jt = Ne[0];
                  if (Jt === "array") {
                    var fn, Sn;
                    if (Ne.length > 2) {
                      var Hn = Ne[1];
                      if (typeof Hn != "string" || !(Hn in si) || Hn === "object") return je.error('The item type argument of "array" must be one of string, number, boolean', 1);
                      fn = si[Hn], kt++;
                    } else fn = jr;
                    if (Ne.length > 3) {
                      if (Ne[2] !== null && (typeof Ne[2] != "number" || Ne[2] < 0 || Ne[2] !== Math.floor(Ne[2]))) return je.error('The length argument to "array" must be a positive integer literal', 2);
                      Sn = Ne[2], kt++;
                    }
                    ut = gi(fn, Sn);
                  } else ut = si[Jt];
                  for (var nr = []; kt < Ne.length; kt++) {
                    var kr = je.parse(Ne[kt], kt, jr);
                    if (!kr) return null;
                    nr.push(kr);
                  }
                  return new zr(ut, nr);
                }, zr.prototype.evaluate = function(Ne) {
                  for (var je = 0; je < this.args.length; je++) {
                    var ut = this.args[je].evaluate(Ne);
                    if (!ri(this.type, Gr(ut))) return ut;
                    if (je === this.args.length - 1) throw new Dr("Expected value to be of type " + Zr(this.type) + ", but found " + Zr(Gr(ut)) + " instead.");
                  }
                  return null;
                }, zr.prototype.eachChild = function(Ne) {
                  this.args.forEach(Ne);
                }, zr.prototype.outputDefined = function() {
                  return this.args.every(function(Ne) {
                    return Ne.outputDefined();
                  });
                }, zr.prototype.serialize = function() {
                  var Ne = this.type, je = [Ne.kind];
                  if (Ne.kind === "array") {
                    var ut = Ne.itemType;
                    if (ut.kind === "string" || ut.kind === "number" || ut.kind === "boolean") {
                      je.push(ut.kind);
                      var kt = Ne.N;
                      (typeof kt == "number" || this.args.length > 1) && je.push(kt);
                    }
                  }
                  return je.concat(this.args.map(function(Jt) {
                    return Jt.serialize();
                  }));
                };
                var di = function(Ne) {
                  this.type = Wr, this.sections = Ne;
                };
                di.parse = function(Ne, je) {
                  if (Ne.length < 2) return je.error("Expected at least one argument.");
                  var ut = Ne[1];
                  if (!Array.isArray(ut) && typeof ut == "object") return je.error("First argument must be an image or text section.");
                  for (var kt = [], Jt = !1, fn = 1; fn <= Ne.length - 1; ++fn) {
                    var Sn = Ne[fn];
                    if (Jt && typeof Sn == "object" && !Array.isArray(Sn)) {
                      Jt = !1;
                      var Hn = null;
                      if (Sn["font-scale"] && !(Hn = je.parse(Sn["font-scale"], 1, Jn))) return null;
                      var nr = null;
                      if (Sn["text-font"] && !(nr = je.parse(Sn["text-font"], 1, gi(Lr)))) return null;
                      var kr = null;
                      if (Sn["text-color"] && !(kr = je.parse(Sn["text-color"], 1, Ar))) return null;
                      var Ir = kt[kt.length - 1];
                      Ir.scale = Hn, Ir.font = nr, Ir.textColor = kr;
                    } else {
                      var qr = je.parse(Ne[fn], 1, jr);
                      if (!qr) return null;
                      var ci = qr.type.kind;
                      if (ci !== "string" && ci !== "value" && ci !== "null" && ci !== "resolvedImage") return je.error("Formatted text type must be 'string', 'value', 'image' or 'null'.");
                      Jt = !0, kt.push({ content: qr, scale: null, font: null, textColor: null });
                    }
                  }
                  return new di(kt);
                }, di.prototype.evaluate = function(Ne) {
                  return new oa(this.sections.map(function(je) {
                    var ut = je.content.evaluate(Ne);
                    return Gr(ut) === Bi ? new Ci("", ut, null, null, null) : new Ci(Ca(ut), null, je.scale ? je.scale.evaluate(Ne) : null, je.font ? je.font.evaluate(Ne).join(",") : null, je.textColor ? je.textColor.evaluate(Ne) : null);
                  }));
                }, di.prototype.eachChild = function(Ne) {
                  for (var je = 0, ut = this.sections; je < ut.length; je += 1) {
                    var kt = ut[je];
                    Ne(kt.content), kt.scale && Ne(kt.scale), kt.font && Ne(kt.font), kt.textColor && Ne(kt.textColor);
                  }
                }, di.prototype.outputDefined = function() {
                  return !1;
                }, di.prototype.serialize = function() {
                  for (var Ne = ["format"], je = 0, ut = this.sections; je < ut.length; je += 1) {
                    var kt = ut[je];
                    Ne.push(kt.content.serialize());
                    var Jt = {};
                    kt.scale && (Jt["font-scale"] = kt.scale.serialize()), kt.font && (Jt["text-font"] = kt.font.serialize()), kt.textColor && (Jt["text-color"] = kt.textColor.serialize()), Ne.push(Jt);
                  }
                  return Ne;
                };
                var Da = function(Ne) {
                  this.type = Bi, this.input = Ne;
                };
                Da.parse = function(Ne, je) {
                  if (Ne.length !== 2) return je.error("Expected two arguments.");
                  var ut = je.parse(Ne[1], 1, Lr);
                  return ut ? new Da(ut) : je.error("No image name provided.");
                }, Da.prototype.evaluate = function(Ne) {
                  var je = this.input.evaluate(Ne), ut = ka.fromString(je);
                  return ut && Ne.availableImages && (ut.available = Ne.availableImages.indexOf(je) > -1), ut;
                }, Da.prototype.eachChild = function(Ne) {
                  Ne(this.input);
                }, Da.prototype.outputDefined = function() {
                  return !1;
                }, Da.prototype.serialize = function() {
                  return ["image", this.input.serialize()];
                };
                var la = { "to-boolean": mr, "to-color": Ar, "to-number": Jn, "to-string": Lr }, wa = function(Ne, je) {
                  this.type = Ne, this.args = je;
                };
                wa.parse = function(Ne, je) {
                  if (Ne.length < 2) return je.error("Expected at least one argument.");
                  var ut = Ne[0];
                  if ((ut === "to-boolean" || ut === "to-string") && Ne.length !== 2) return je.error("Expected one argument.");
                  for (var kt = la[ut], Jt = [], fn = 1; fn < Ne.length; fn++) {
                    var Sn = je.parse(Ne[fn], fn, jr);
                    if (!Sn) return null;
                    Jt.push(Sn);
                  }
                  return new wa(kt, Jt);
                }, wa.prototype.evaluate = function(Ne) {
                  if (this.type.kind === "boolean") return !!this.args[0].evaluate(Ne);
                  if (this.type.kind === "color") {
                    for (var je, ut, kt = 0, Jt = this.args; kt < Jt.length; kt += 1) {
                      if (ut = null, (je = Jt[kt].evaluate(Ne)) instanceof Pr) return je;
                      if (typeof je == "string") {
                        var fn = Ne.parseColor(je);
                        if (fn) return fn;
                      } else if (Array.isArray(je) && !(ut = je.length < 3 || je.length > 4 ? "Invalid rbga value " + JSON.stringify(je) + ": expected an array containing either three or four numeric values." : aa(je[0], je[1], je[2], je[3]))) return new Pr(je[0] / 255, je[1] / 255, je[2] / 255, je[3]);
                    }
                    throw new Dr(ut || "Could not parse color from value '" + (typeof je == "string" ? je : String(JSON.stringify(je))) + "'");
                  }
                  if (this.type.kind === "number") {
                    for (var Sn = null, Hn = 0, nr = this.args; Hn < nr.length; Hn += 1) {
                      if ((Sn = nr[Hn].evaluate(Ne)) === null) return 0;
                      var kr = Number(Sn);
                      if (!isNaN(kr)) return kr;
                    }
                    throw new Dr("Could not convert " + JSON.stringify(Sn) + " to number.");
                  }
                  return this.type.kind === "formatted" ? oa.fromString(Ca(this.args[0].evaluate(Ne))) : this.type.kind === "resolvedImage" ? ka.fromString(Ca(this.args[0].evaluate(Ne))) : Ca(this.args[0].evaluate(Ne));
                }, wa.prototype.eachChild = function(Ne) {
                  this.args.forEach(Ne);
                }, wa.prototype.outputDefined = function() {
                  return this.args.every(function(Ne) {
                    return Ne.outputDefined();
                  });
                }, wa.prototype.serialize = function() {
                  if (this.type.kind === "formatted") return new di([{ content: this.args[0], scale: null, font: null, textColor: null }]).serialize();
                  if (this.type.kind === "resolvedImage") return new Da(this.args[0]).serialize();
                  var Ne = ["to-" + this.type.kind];
                  return this.eachChild(function(je) {
                    Ne.push(je.serialize());
                  }), Ne;
                };
                var Wa = ["Unknown", "Point", "LineString", "Polygon"], Qr = function() {
                  this.globals = null, this.feature = null, this.featureState = null, this.formattedSection = null, this._parseColorCache = {}, this.availableImages = null, this.canonical = null;
                };
                Qr.prototype.id = function() {
                  return this.feature && "id" in this.feature ? this.feature.id : null;
                }, Qr.prototype.geometryType = function() {
                  return this.feature ? typeof this.feature.type == "number" ? Wa[this.feature.type] : this.feature.type : null;
                }, Qr.prototype.geometry = function() {
                  return this.feature && "geometry" in this.feature ? this.feature.geometry : null;
                }, Qr.prototype.canonicalID = function() {
                  return this.canonical;
                }, Qr.prototype.properties = function() {
                  return this.feature && this.feature.properties || {};
                }, Qr.prototype.parseColor = function(Ne) {
                  var je = this._parseColorCache[Ne];
                  return je || (je = this._parseColorCache[Ne] = Pr.parse(Ne)), je;
                };
                var Ur = function(Ne, je, ut, kt) {
                  this.name = Ne, this.type = je, this._evaluate = ut, this.args = kt;
                };
                Ur.prototype.evaluate = function(Ne) {
                  return this._evaluate(Ne, this.args);
                }, Ur.prototype.eachChild = function(Ne) {
                  this.args.forEach(Ne);
                }, Ur.prototype.outputDefined = function() {
                  return !1;
                }, Ur.prototype.serialize = function() {
                  return [this.name].concat(this.args.map(function(Ne) {
                    return Ne.serialize();
                  }));
                }, Ur.parse = function(Ne, je) {
                  var ut, kt = Ne[0], Jt = Ur.definitions[kt];
                  if (!Jt) return je.error('Unknown expression "' + kt + '". If you wanted a literal array, use ["literal", [...]].', 0);
                  for (var fn = Array.isArray(Jt) ? Jt[0] : Jt.type, Sn = Array.isArray(Jt) ? [[Jt[1], Jt[2]]] : Jt.overloads, Hn = Sn.filter(function(Pl) {
                    var rl = Pl[0];
                    return !Array.isArray(rl) || rl.length === Ne.length - 1;
                  }), nr = null, kr = 0, Ir = Hn; kr < Ir.length; kr += 1) {
                    var qr = Ir[kr], ci = qr[0], ni = qr[1];
                    nr = new Ki(je.registry, je.path, null, je.scope);
                    for (var Ii = [], Gi = !1, xa = 1; xa < Ne.length; xa++) {
                      var ga = Ne[xa], $a = Array.isArray(ci) ? ci[xa - 1] : ci.type, Va = nr.parse(ga, 1 + Ii.length, $a);
                      if (!Va) {
                        Gi = !0;
                        break;
                      }
                      Ii.push(Va);
                    }
                    if (!Gi) if (Array.isArray(ci) && ci.length !== Ii.length) nr.error("Expected " + ci.length + " arguments, but found " + Ii.length + " instead.");
                    else {
                      for (var eo = 0; eo < Ii.length; eo++) {
                        var co = Array.isArray(ci) ? ci[eo] : ci.type, wo = Ii[eo];
                        nr.concat(eo + 1).checkSubtype(co, wo.type);
                      }
                      if (nr.errors.length === 0) return new Ur(kt, fn, ni, Ii);
                    }
                  }
                  if (Hn.length === 1) (ut = je.errors).push.apply(ut, nr.errors);
                  else {
                    for (var ds = (Hn.length ? Hn : Sn).map(function(Pl) {
                      return rl = Pl[0], Array.isArray(rl) ? "(" + rl.map(Zr).join(", ") + ")" : "(" + Zr(rl.type) + "...)";
                      var rl;
                    }).join(" | "), Lo = [], As = 1; As < Ne.length; As++) {
                      var Fs = je.parse(Ne[As], 1 + Lo.length);
                      if (!Fs) return null;
                      Lo.push(Zr(Fs.type));
                    }
                    je.error("Expected arguments of type " + ds + ", but found (" + Lo.join(", ") + ") instead.");
                  }
                  return null;
                }, Ur.register = function(Ne, je) {
                  for (var ut in Ur.definitions = je, je) Ne[ut] = Ur;
                };
                var rr = function(Ne, je, ut) {
                  this.type = Mr, this.locale = ut, this.caseSensitive = Ne, this.diacriticSensitive = je;
                };
                rr.parse = function(Ne, je) {
                  if (Ne.length !== 2) return je.error("Expected one argument.");
                  var ut = Ne[1];
                  if (typeof ut != "object" || Array.isArray(ut)) return je.error("Collator options argument must be an object.");
                  var kt = je.parse(ut["case-sensitive"] !== void 0 && ut["case-sensitive"], 1, mr);
                  if (!kt) return null;
                  var Jt = je.parse(ut["diacritic-sensitive"] !== void 0 && ut["diacritic-sensitive"], 1, mr);
                  if (!Jt) return null;
                  var fn = null;
                  return ut.locale && !(fn = je.parse(ut.locale, 1, Lr)) ? null : new rr(kt, Jt, fn);
                }, rr.prototype.evaluate = function(Ne) {
                  return new ei(this.caseSensitive.evaluate(Ne), this.diacriticSensitive.evaluate(Ne), this.locale ? this.locale.evaluate(Ne) : null);
                }, rr.prototype.eachChild = function(Ne) {
                  Ne(this.caseSensitive), Ne(this.diacriticSensitive), this.locale && Ne(this.locale);
                }, rr.prototype.outputDefined = function() {
                  return !1;
                }, rr.prototype.serialize = function() {
                  var Ne = {};
                  return Ne["case-sensitive"] = this.caseSensitive.serialize(), Ne["diacritic-sensitive"] = this.diacriticSensitive.serialize(), this.locale && (Ne.locale = this.locale.serialize()), ["collator", Ne];
                };
                var vr = 8192;
                function Rr(Ne, je) {
                  Ne[0] = Math.min(Ne[0], je[0]), Ne[1] = Math.min(Ne[1], je[1]), Ne[2] = Math.max(Ne[2], je[0]), Ne[3] = Math.max(Ne[3], je[1]);
                }
                function Vr(Ne, je) {
                  return !(Ne[0] <= je[0] || Ne[2] >= je[2] || Ne[1] <= je[1] || Ne[3] >= je[3]);
                }
                function Xr(Ne, je) {
                  var ut, kt = (180 + Ne[0]) / 360, Jt = (ut = Ne[1], (180 - 180 / Math.PI * Math.log(Math.tan(Math.PI / 4 + ut * Math.PI / 360))) / 360), fn = Math.pow(2, je.z);
                  return [Math.round(kt * fn * vr), Math.round(Jt * fn * vr)];
                }
                function Jr(Ne, je, ut) {
                  return je[1] > Ne[1] != ut[1] > Ne[1] && Ne[0] < (ut[0] - je[0]) * (Ne[1] - je[1]) / (ut[1] - je[1]) + je[0];
                }
                function ti(Ne, je) {
                  for (var ut = !1, kt = 0, Jt = je.length; kt < Jt; kt++) for (var fn = je[kt], Sn = 0, Hn = fn.length; Sn < Hn - 1; Sn++) {
                    if (nr = Ne, kr = fn[Sn], Ir = fn[Sn + 1], qr = void 0, ci = void 0, ni = void 0, Ii = void 0, qr = nr[0] - kr[0], ci = nr[1] - kr[1], ni = nr[0] - Ir[0], Ii = nr[1] - Ir[1], qr * Ii - ni * ci == 0 && qr * ni <= 0 && ci * Ii <= 0) return !1;
                    Jr(Ne, fn[Sn], fn[Sn + 1]) && (ut = !ut);
                  }
                  var nr, kr, Ir, qr, ci, ni, Ii;
                  return ut;
                }
                function Hr(Ne, je) {
                  for (var ut = 0; ut < je.length; ut++) if (ti(Ne, je[ut])) return !0;
                  return !1;
                }
                function Er(Ne, je, ut, kt) {
                  var Jt = Ne[0] - ut[0], fn = Ne[1] - ut[1], Sn = je[0] - ut[0], Hn = je[1] - ut[1], nr = kt[0] - ut[0], kr = kt[1] - ut[1], Ir = Jt * kr - nr * fn, qr = Sn * kr - nr * Hn;
                  return Ir > 0 && qr < 0 || Ir < 0 && qr > 0;
                }
                function gr(Ne, je, ut) {
                  for (var kt = 0, Jt = ut; kt < Jt.length; kt += 1) for (var fn = Jt[kt], Sn = 0; Sn < fn.length - 1; ++Sn) if (Hn = Ne, nr = je, kr = fn[Sn], Ir = fn[Sn + 1], qr = void 0, ci = void 0, ni = void 0, ni = [nr[0] - Hn[0], nr[1] - Hn[1]], qr = [Ir[0] - kr[0], Ir[1] - kr[1]], ci = ni, qr[0] * ci[1] - qr[1] * ci[0] != 0 && Er(Hn, nr, kr, Ir) && Er(kr, Ir, Hn, nr)) return !0;
                  var Hn, nr, kr, Ir, qr, ci, ni;
                  return !1;
                }
                function $r(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ++ut) if (!ti(Ne[ut], je)) return !1;
                  for (var kt = 0; kt < Ne.length - 1; ++kt) if (gr(Ne[kt], Ne[kt + 1], je)) return !1;
                  return !0;
                }
                function li(Ne, je) {
                  for (var ut = 0; ut < je.length; ut++) if ($r(Ne, je[ut])) return !0;
                  return !1;
                }
                function Qi(Ne, je, ut) {
                  for (var kt = [], Jt = 0; Jt < Ne.length; Jt++) {
                    for (var fn = [], Sn = 0; Sn < Ne[Jt].length; Sn++) {
                      var Hn = Xr(Ne[Jt][Sn], ut);
                      Rr(je, Hn), fn.push(Hn);
                    }
                    kt.push(fn);
                  }
                  return kt;
                }
                function Ai(Ne, je, ut) {
                  for (var kt = [], Jt = 0; Jt < Ne.length; Jt++) {
                    var fn = Qi(Ne[Jt], je, ut);
                    kt.push(fn);
                  }
                  return kt;
                }
                function Li(Ne, je, ut, kt) {
                  if (Ne[0] < ut[0] || Ne[0] > ut[2]) {
                    var Jt = 0.5 * kt, fn = Ne[0] - ut[0] > Jt ? -kt : ut[0] - Ne[0] > Jt ? kt : 0;
                    fn === 0 && (fn = Ne[0] - ut[2] > Jt ? -kt : ut[2] - Ne[0] > Jt ? kt : 0), Ne[0] += fn;
                  }
                  Rr(je, Ne);
                }
                function Ni(Ne, je, ut, kt) {
                  for (var Jt = Math.pow(2, kt.z) * vr, fn = [kt.x * vr, kt.y * vr], Sn = [], Hn = 0, nr = Ne; Hn < nr.length; Hn += 1) for (var kr = 0, Ir = nr[Hn]; kr < Ir.length; kr += 1) {
                    var qr = Ir[kr], ci = [qr.x + fn[0], qr.y + fn[1]];
                    Li(ci, je, ut, Jt), Sn.push(ci);
                  }
                  return Sn;
                }
                function Pi(Ne, je, ut, kt) {
                  for (var Jt = Math.pow(2, kt.z) * vr, fn = [kt.x * vr, kt.y * vr], Sn = [], Hn = 0, nr = Ne; Hn < nr.length; Hn += 1) {
                    for (var kr = [], Ir = 0, qr = nr[Hn]; Ir < qr.length; Ir += 1) {
                      var ci = qr[Ir], ni = [ci.x + fn[0], ci.y + fn[1]];
                      Rr(je, ni), kr.push(ni);
                    }
                    Sn.push(kr);
                  }
                  if (je[2] - je[0] <= Jt / 2) {
                    ($a = je)[0] = $a[1] = 1 / 0, $a[2] = $a[3] = -1 / 0;
                    for (var Ii = 0, Gi = Sn; Ii < Gi.length; Ii += 1) for (var xa = 0, ga = Gi[Ii]; xa < ga.length; xa += 1) Li(ga[xa], je, ut, Jt);
                  }
                  var $a;
                  return Sn;
                }
                var Oi = function(Ne, je) {
                  this.type = mr, this.geojson = Ne, this.geometries = je;
                };
                function ki(Ne) {
                  if (Ne instanceof Ur && (Ne.name === "get" && Ne.args.length === 1 || Ne.name === "feature-state" || Ne.name === "has" && Ne.args.length === 1 || Ne.name === "properties" || Ne.name === "geometry-type" || Ne.name === "id" || /^filter-/.test(Ne.name)) || Ne instanceof Oi) return !1;
                  var je = !0;
                  return Ne.eachChild(function(ut) {
                    je && !ki(ut) && (je = !1);
                  }), je;
                }
                function Mi(Ne) {
                  if (Ne instanceof Ur && Ne.name === "feature-state") return !1;
                  var je = !0;
                  return Ne.eachChild(function(ut) {
                    je && !Mi(ut) && (je = !1);
                  }), je;
                }
                function Vi(Ne, je) {
                  if (Ne instanceof Ur && je.indexOf(Ne.name) >= 0) return !1;
                  var ut = !0;
                  return Ne.eachChild(function(kt) {
                    ut && !Vi(kt, je) && (ut = !1);
                  }), ut;
                }
                Oi.parse = function(Ne, je) {
                  if (Ne.length !== 2) return je.error("'within' expression requires exactly one argument, but found " + (Ne.length - 1) + " instead.");
                  if (hi(Ne[1])) {
                    var ut = Ne[1];
                    if (ut.type === "FeatureCollection") for (var kt = 0; kt < ut.features.length; ++kt) {
                      var Jt = ut.features[kt].geometry.type;
                      if (Jt === "Polygon" || Jt === "MultiPolygon") return new Oi(ut, ut.features[kt].geometry);
                    }
                    else if (ut.type === "Feature") {
                      var fn = ut.geometry.type;
                      if (fn === "Polygon" || fn === "MultiPolygon") return new Oi(ut, ut.geometry);
                    } else if (ut.type === "Polygon" || ut.type === "MultiPolygon") return new Oi(ut, ut);
                  }
                  return je.error("'within' expression requires valid geojson object that contains polygon geometry type.");
                }, Oi.prototype.evaluate = function(Ne) {
                  if (Ne.geometry() != null && Ne.canonicalID() != null) {
                    if (Ne.geometryType() === "Point") return function(je, ut) {
                      var kt = [1 / 0, 1 / 0, -1 / 0, -1 / 0], Jt = [1 / 0, 1 / 0, -1 / 0, -1 / 0], fn = je.canonicalID();
                      if (ut.type === "Polygon") {
                        var Sn = Qi(ut.coordinates, Jt, fn), Hn = Ni(je.geometry(), kt, Jt, fn);
                        if (!Vr(kt, Jt)) return !1;
                        for (var nr = 0, kr = Hn; nr < kr.length; nr += 1) if (!ti(kr[nr], Sn)) return !1;
                      }
                      if (ut.type === "MultiPolygon") {
                        var Ir = Ai(ut.coordinates, Jt, fn), qr = Ni(je.geometry(), kt, Jt, fn);
                        if (!Vr(kt, Jt)) return !1;
                        for (var ci = 0, ni = qr; ci < ni.length; ci += 1) if (!Hr(ni[ci], Ir)) return !1;
                      }
                      return !0;
                    }(Ne, this.geometries);
                    if (Ne.geometryType() === "LineString") return function(je, ut) {
                      var kt = [1 / 0, 1 / 0, -1 / 0, -1 / 0], Jt = [1 / 0, 1 / 0, -1 / 0, -1 / 0], fn = je.canonicalID();
                      if (ut.type === "Polygon") {
                        var Sn = Qi(ut.coordinates, Jt, fn), Hn = Pi(je.geometry(), kt, Jt, fn);
                        if (!Vr(kt, Jt)) return !1;
                        for (var nr = 0, kr = Hn; nr < kr.length; nr += 1) if (!$r(kr[nr], Sn)) return !1;
                      }
                      if (ut.type === "MultiPolygon") {
                        var Ir = Ai(ut.coordinates, Jt, fn), qr = Pi(je.geometry(), kt, Jt, fn);
                        if (!Vr(kt, Jt)) return !1;
                        for (var ci = 0, ni = qr; ci < ni.length; ci += 1) if (!li(ni[ci], Ir)) return !1;
                      }
                      return !0;
                    }(Ne, this.geometries);
                  }
                  return !1;
                }, Oi.prototype.eachChild = function() {
                }, Oi.prototype.outputDefined = function() {
                  return !0;
                }, Oi.prototype.serialize = function() {
                  return ["within", this.geojson];
                };
                var ua = function(Ne, je) {
                  this.type = je.type, this.name = Ne, this.boundExpression = je;
                };
                ua.parse = function(Ne, je) {
                  if (Ne.length !== 2 || typeof Ne[1] != "string") return je.error("'var' expression requires exactly one string literal argument.");
                  var ut = Ne[1];
                  return je.scope.has(ut) ? new ua(ut, je.scope.get(ut)) : je.error('Unknown variable "' + ut + '". Make sure "' + ut + '" has been bound in an enclosing "let" expression before using it.', 1);
                }, ua.prototype.evaluate = function(Ne) {
                  return this.boundExpression.evaluate(Ne);
                }, ua.prototype.eachChild = function() {
                }, ua.prototype.outputDefined = function() {
                  return !1;
                }, ua.prototype.serialize = function() {
                  return ["var", this.name];
                };
                var Ki = function(Ne, je, ut, kt, Jt) {
                  je === void 0 && (je = []), kt === void 0 && (kt = new ur()), Jt === void 0 && (Jt = []), this.registry = Ne, this.path = je, this.key = je.map(function(fn) {
                    return "[" + fn + "]";
                  }).join(""), this.scope = kt, this.errors = Jt, this.expectedType = ut;
                };
                function Fi(Ne) {
                  if (Ne instanceof ua) return Fi(Ne.boundExpression);
                  if (Ne instanceof Ur && Ne.name === "error" || Ne instanceof rr || Ne instanceof Oi) return !1;
                  var je = Ne instanceof wa || Ne instanceof zr, ut = !0;
                  return Ne.eachChild(function(kt) {
                    ut = je ? ut && Fi(kt) : ut && kt instanceof Sr;
                  }), !!ut && ki(Ne) && Vi(Ne, ["zoom", "heatmap-density", "line-progress", "accumulated", "is-supported-script"]);
                }
                function na(Ne, je) {
                  for (var ut, kt, Jt = Ne.length - 1, fn = 0, Sn = Jt, Hn = 0; fn <= Sn; ) if (ut = Ne[Hn = Math.floor((fn + Sn) / 2)], kt = Ne[Hn + 1], ut <= je) {
                    if (Hn === Jt || je < kt) return Hn;
                    fn = Hn + 1;
                  } else {
                    if (!(ut > je)) throw new Dr("Input is not a number.");
                    Sn = Hn - 1;
                  }
                  return 0;
                }
                Ki.prototype.parse = function(Ne, je, ut, kt, Jt) {
                  return Jt === void 0 && (Jt = {}), je ? this.concat(je, ut, kt)._parse(Ne, Jt) : this._parse(Ne, Jt);
                }, Ki.prototype._parse = function(Ne, je) {
                  function ut(kr, Ir, qr) {
                    return qr === "assert" ? new zr(Ir, [kr]) : qr === "coerce" ? new wa(Ir, [kr]) : kr;
                  }
                  if (Ne !== null && typeof Ne != "string" && typeof Ne != "boolean" && typeof Ne != "number" || (Ne = ["literal", Ne]), Array.isArray(Ne)) {
                    if (Ne.length === 0) return this.error('Expected an array with at least one element. If you wanted a literal array, use ["literal", []].');
                    var kt = Ne[0];
                    if (typeof kt != "string") return this.error("Expression name must be a string, but found " + typeof kt + ' instead. If you wanted a literal array, use ["literal", [...]].', 0), null;
                    var Jt = this.registry[kt];
                    if (Jt) {
                      var fn = Jt.parse(Ne, this);
                      if (!fn) return null;
                      if (this.expectedType) {
                        var Sn = this.expectedType, Hn = fn.type;
                        if (Sn.kind !== "string" && Sn.kind !== "number" && Sn.kind !== "boolean" && Sn.kind !== "object" && Sn.kind !== "array" || Hn.kind !== "value") if (Sn.kind !== "color" && Sn.kind !== "formatted" && Sn.kind !== "resolvedImage" || Hn.kind !== "value" && Hn.kind !== "string") {
                          if (this.checkSubtype(Sn, Hn)) return null;
                        } else fn = ut(fn, Sn, je.typeAnnotation || "coerce");
                        else fn = ut(fn, Sn, je.typeAnnotation || "assert");
                      }
                      if (!(fn instanceof Sr) && fn.type.kind !== "resolvedImage" && Fi(fn)) {
                        var nr = new Qr();
                        try {
                          fn = new Sr(fn.type, fn.evaluate(nr));
                        } catch (kr) {
                          return this.error(kr.message), null;
                        }
                      }
                      return fn;
                    }
                    return this.error('Unknown expression "' + kt + '". If you wanted a literal array, use ["literal", [...]].', 0);
                  }
                  return Ne === void 0 ? this.error("'undefined' value invalid. Use null instead.") : typeof Ne == "object" ? this.error('Bare objects invalid. Use ["literal", {...}] instead.') : this.error("Expected an array, but found " + typeof Ne + " instead.");
                }, Ki.prototype.concat = function(Ne, je, ut) {
                  var kt = typeof Ne == "number" ? this.path.concat(Ne) : this.path, Jt = ut ? this.scope.concat(ut) : this.scope;
                  return new Ki(this.registry, kt, je || null, Jt, this.errors);
                }, Ki.prototype.error = function(Ne) {
                  for (var je = [], ut = arguments.length - 1; ut-- > 0; ) je[ut] = arguments[ut + 1];
                  var kt = "" + this.key + je.map(function(Jt) {
                    return "[" + Jt + "]";
                  }).join("");
                  this.errors.push(new jn(kt, Ne));
                }, Ki.prototype.checkSubtype = function(Ne, je) {
                  var ut = ri(Ne, je);
                  return ut && this.error(ut), ut;
                };
                var nn = function(Ne, je, ut) {
                  this.type = Ne, this.input = je, this.labels = [], this.outputs = [];
                  for (var kt = 0, Jt = ut; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt], Sn = fn[0], Hn = fn[1];
                    this.labels.push(Sn), this.outputs.push(Hn);
                  }
                };
                function Tn(Ne, je, ut) {
                  return Ne * (1 - ut) + je * ut;
                }
                nn.parse = function(Ne, je) {
                  if (Ne.length - 1 < 4) return je.error("Expected at least 4 arguments, but found only " + (Ne.length - 1) + ".");
                  if ((Ne.length - 1) % 2 != 0) return je.error("Expected an even number of arguments.");
                  var ut = je.parse(Ne[1], 1, Jn);
                  if (!ut) return null;
                  var kt = [], Jt = null;
                  je.expectedType && je.expectedType.kind !== "value" && (Jt = je.expectedType);
                  for (var fn = 1; fn < Ne.length; fn += 2) {
                    var Sn = fn === 1 ? -1 / 0 : Ne[fn], Hn = Ne[fn + 1], nr = fn, kr = fn + 1;
                    if (typeof Sn != "number") return je.error('Input/output pairs for "step" expressions must be defined using literal numeric values (not computed expressions) for the input values.', nr);
                    if (kt.length && kt[kt.length - 1][0] >= Sn) return je.error('Input/output pairs for "step" expressions must be arranged with input values in strictly ascending order.', nr);
                    var Ir = je.parse(Hn, kr, Jt);
                    if (!Ir) return null;
                    Jt = Jt || Ir.type, kt.push([Sn, Ir]);
                  }
                  return new nn(Jt, ut, kt);
                }, nn.prototype.evaluate = function(Ne) {
                  var je = this.labels, ut = this.outputs;
                  if (je.length === 1) return ut[0].evaluate(Ne);
                  var kt = this.input.evaluate(Ne);
                  if (kt <= je[0]) return ut[0].evaluate(Ne);
                  var Jt = je.length;
                  return kt >= je[Jt - 1] ? ut[Jt - 1].evaluate(Ne) : ut[na(je, kt)].evaluate(Ne);
                }, nn.prototype.eachChild = function(Ne) {
                  Ne(this.input);
                  for (var je = 0, ut = this.outputs; je < ut.length; je += 1) Ne(ut[je]);
                }, nn.prototype.outputDefined = function() {
                  return this.outputs.every(function(Ne) {
                    return Ne.outputDefined();
                  });
                }, nn.prototype.serialize = function() {
                  for (var Ne = ["step", this.input.serialize()], je = 0; je < this.labels.length; je++) je > 0 && Ne.push(this.labels[je]), Ne.push(this.outputs[je].serialize());
                  return Ne;
                };
                var Dn = Object.freeze({ __proto__: null, number: Tn, color: function(Ne, je, ut) {
                  return new Pr(Tn(Ne.r, je.r, ut), Tn(Ne.g, je.g, ut), Tn(Ne.b, je.b, ut), Tn(Ne.a, je.a, ut));
                }, array: function(Ne, je, ut) {
                  return Ne.map(function(kt, Jt) {
                    return Tn(kt, je[Jt], ut);
                  });
                } }), Fn = 0.95047, fr = 1, dr = 1.08883, yr = 4 / 29, ai = 6 / 29, Fr = 3 * ai * ai, Ti = ai * ai * ai, Si = Math.PI / 180, Ei = 180 / Math.PI;
                function Fa(Ne) {
                  return Ne > Ti ? Math.pow(Ne, 0.3333333333333333) : Ne / Fr + yr;
                }
                function $o(Ne) {
                  return Ne > ai ? Ne * Ne * Ne : Fr * (Ne - yr);
                }
                function Gs(Ne) {
                  return 255 * (Ne <= 31308e-7 ? 12.92 * Ne : 1.055 * Math.pow(Ne, 0.4166666666666667) - 0.055);
                }
                function Al(Ne) {
                  return (Ne /= 255) <= 0.04045 ? Ne / 12.92 : Math.pow((Ne + 0.055) / 1.055, 2.4);
                }
                function ul(Ne) {
                  var je = Al(Ne.r), ut = Al(Ne.g), kt = Al(Ne.b), Jt = Fa((0.4124564 * je + 0.3575761 * ut + 0.1804375 * kt) / Fn), fn = Fa((0.2126729 * je + 0.7151522 * ut + 0.072175 * kt) / fr);
                  return { l: 116 * fn - 16, a: 500 * (Jt - fn), b: 200 * (fn - Fa((0.0193339 * je + 0.119192 * ut + 0.9503041 * kt) / dr)), alpha: Ne.a };
                }
                function gc(Ne) {
                  var je = (Ne.l + 16) / 116, ut = isNaN(Ne.a) ? je : je + Ne.a / 500, kt = isNaN(Ne.b) ? je : je - Ne.b / 200;
                  return je = fr * $o(je), ut = Fn * $o(ut), kt = dr * $o(kt), new Pr(Gs(3.2404542 * ut - 1.5371385 * je - 0.4985314 * kt), Gs(-0.969266 * ut + 1.8760108 * je + 0.041556 * kt), Gs(0.0556434 * ut - 0.2040259 * je + 1.0572252 * kt), Ne.alpha);
                }
                function Gl(Ne, je, ut) {
                  var kt = je - Ne;
                  return Ne + ut * (kt > 180 || kt < -180 ? kt - 360 * Math.round(kt / 360) : kt);
                }
                var Sl = { forward: ul, reverse: gc, interpolate: function(Ne, je, ut) {
                  return { l: Tn(Ne.l, je.l, ut), a: Tn(Ne.a, je.a, ut), b: Tn(Ne.b, je.b, ut), alpha: Tn(Ne.alpha, je.alpha, ut) };
                } }, ls = { forward: function(Ne) {
                  var je = ul(Ne), ut = je.l, kt = je.a, Jt = je.b, fn = Math.atan2(Jt, kt) * Ei;
                  return { h: fn < 0 ? fn + 360 : fn, c: Math.sqrt(kt * kt + Jt * Jt), l: ut, alpha: Ne.a };
                }, reverse: function(Ne) {
                  var je = Ne.h * Si, ut = Ne.c;
                  return gc({ l: Ne.l, a: Math.cos(je) * ut, b: Math.sin(je) * ut, alpha: Ne.alpha });
                }, interpolate: function(Ne, je, ut) {
                  return { h: Gl(Ne.h, je.h, ut), c: Tn(Ne.c, je.c, ut), l: Tn(Ne.l, je.l, ut), alpha: Tn(Ne.alpha, je.alpha, ut) };
                } }, vs = Object.freeze({ __proto__: null, lab: Sl, hcl: ls }), Vl = function(Ne, je, ut, kt, Jt) {
                  this.type = Ne, this.operator = je, this.interpolation = ut, this.input = kt, this.labels = [], this.outputs = [];
                  for (var fn = 0, Sn = Jt; fn < Sn.length; fn += 1) {
                    var Hn = Sn[fn], nr = Hn[0], kr = Hn[1];
                    this.labels.push(nr), this.outputs.push(kr);
                  }
                };
                function Ll(Ne, je, ut, kt) {
                  var Jt = kt - ut, fn = Ne - ut;
                  return Jt === 0 ? 0 : je === 1 ? fn / Jt : (Math.pow(je, fn) - 1) / (Math.pow(je, Jt) - 1);
                }
                Vl.interpolationFactor = function(Ne, je, ut, kt) {
                  var Jt = 0;
                  if (Ne.name === "exponential") Jt = Ll(je, Ne.base, ut, kt);
                  else if (Ne.name === "linear") Jt = Ll(je, 1, ut, kt);
                  else if (Ne.name === "cubic-bezier") {
                    var fn = Ne.controlPoints;
                    Jt = new se(fn[0], fn[1], fn[2], fn[3]).solve(Ll(je, 1, ut, kt));
                  }
                  return Jt;
                }, Vl.parse = function(Ne, je) {
                  var ut = Ne[0], kt = Ne[1], Jt = Ne[2], fn = Ne.slice(3);
                  if (!Array.isArray(kt) || kt.length === 0) return je.error("Expected an interpolation type expression.", 1);
                  if (kt[0] === "linear") kt = { name: "linear" };
                  else if (kt[0] === "exponential") {
                    var Sn = kt[1];
                    if (typeof Sn != "number") return je.error("Exponential interpolation requires a numeric base.", 1, 1);
                    kt = { name: "exponential", base: Sn };
                  } else {
                    if (kt[0] !== "cubic-bezier") return je.error("Unknown interpolation type " + String(kt[0]), 1, 0);
                    var Hn = kt.slice(1);
                    if (Hn.length !== 4 || Hn.some(function(xa) {
                      return typeof xa != "number" || xa < 0 || xa > 1;
                    })) return je.error("Cubic bezier interpolation requires four numeric arguments with values between 0 and 1.", 1);
                    kt = { name: "cubic-bezier", controlPoints: Hn };
                  }
                  if (Ne.length - 1 < 4) return je.error("Expected at least 4 arguments, but found only " + (Ne.length - 1) + ".");
                  if ((Ne.length - 1) % 2 != 0) return je.error("Expected an even number of arguments.");
                  if (!(Jt = je.parse(Jt, 2, Jn))) return null;
                  var nr = [], kr = null;
                  ut === "interpolate-hcl" || ut === "interpolate-lab" ? kr = Ar : je.expectedType && je.expectedType.kind !== "value" && (kr = je.expectedType);
                  for (var Ir = 0; Ir < fn.length; Ir += 2) {
                    var qr = fn[Ir], ci = fn[Ir + 1], ni = Ir + 3, Ii = Ir + 4;
                    if (typeof qr != "number") return je.error('Input/output pairs for "interpolate" expressions must be defined using literal numeric values (not computed expressions) for the input values.', ni);
                    if (nr.length && nr[nr.length - 1][0] >= qr) return je.error('Input/output pairs for "interpolate" expressions must be arranged with input values in strictly ascending order.', ni);
                    var Gi = je.parse(ci, Ii, kr);
                    if (!Gi) return null;
                    kr = kr || Gi.type, nr.push([qr, Gi]);
                  }
                  return kr.kind === "number" || kr.kind === "color" || kr.kind === "array" && kr.itemType.kind === "number" && typeof kr.N == "number" ? new Vl(kr, ut, kt, Jt, nr) : je.error("Type " + Zr(kr) + " is not interpolatable.");
                }, Vl.prototype.evaluate = function(Ne) {
                  var je = this.labels, ut = this.outputs;
                  if (je.length === 1) return ut[0].evaluate(Ne);
                  var kt = this.input.evaluate(Ne);
                  if (kt <= je[0]) return ut[0].evaluate(Ne);
                  var Jt = je.length;
                  if (kt >= je[Jt - 1]) return ut[Jt - 1].evaluate(Ne);
                  var fn = na(je, kt), Sn = je[fn], Hn = je[fn + 1], nr = Vl.interpolationFactor(this.interpolation, kt, Sn, Hn), kr = ut[fn].evaluate(Ne), Ir = ut[fn + 1].evaluate(Ne);
                  return this.operator === "interpolate" ? Dn[this.type.kind.toLowerCase()](kr, Ir, nr) : this.operator === "interpolate-hcl" ? ls.reverse(ls.interpolate(ls.forward(kr), ls.forward(Ir), nr)) : Sl.reverse(Sl.interpolate(Sl.forward(kr), Sl.forward(Ir), nr));
                }, Vl.prototype.eachChild = function(Ne) {
                  Ne(this.input);
                  for (var je = 0, ut = this.outputs; je < ut.length; je += 1) Ne(ut[je]);
                }, Vl.prototype.outputDefined = function() {
                  return this.outputs.every(function(Ne) {
                    return Ne.outputDefined();
                  });
                }, Vl.prototype.serialize = function() {
                  var Ne;
                  Ne = this.interpolation.name === "linear" ? ["linear"] : this.interpolation.name === "exponential" ? this.interpolation.base === 1 ? ["linear"] : ["exponential", this.interpolation.base] : ["cubic-bezier"].concat(this.interpolation.controlPoints);
                  for (var je = [this.operator, Ne, this.input.serialize()], ut = 0; ut < this.labels.length; ut++) je.push(this.labels[ut], this.outputs[ut].serialize());
                  return je;
                };
                var Xl = function(Ne, je) {
                  this.type = Ne, this.args = je;
                };
                Xl.parse = function(Ne, je) {
                  if (Ne.length < 2) return je.error("Expectected at least one argument.");
                  var ut = null, kt = je.expectedType;
                  kt && kt.kind !== "value" && (ut = kt);
                  for (var Jt = [], fn = 0, Sn = Ne.slice(1); fn < Sn.length; fn += 1) {
                    var Hn = Sn[fn], nr = je.parse(Hn, 1 + Jt.length, ut, void 0, { typeAnnotation: "omit" });
                    if (!nr) return null;
                    ut = ut || nr.type, Jt.push(nr);
                  }
                  var kr = kt && Jt.some(function(Ir) {
                    return ri(kt, Ir.type);
                  });
                  return new Xl(kr ? jr : ut, Jt);
                }, Xl.prototype.evaluate = function(Ne) {
                  for (var je, ut = null, kt = 0, Jt = 0, fn = this.args; Jt < fn.length && (kt++, (ut = fn[Jt].evaluate(Ne)) && ut instanceof ka && !ut.available && (je || (je = ut.name), ut = null, kt === this.args.length && (ut = je)), ut === null); Jt += 1) ;
                  return ut;
                }, Xl.prototype.eachChild = function(Ne) {
                  this.args.forEach(Ne);
                }, Xl.prototype.outputDefined = function() {
                  return this.args.every(function(Ne) {
                    return Ne.outputDefined();
                  });
                }, Xl.prototype.serialize = function() {
                  var Ne = ["coalesce"];
                  return this.eachChild(function(je) {
                    Ne.push(je.serialize());
                  }), Ne;
                };
                var yc = function(Ne, je) {
                  this.type = je.type, this.bindings = [].concat(Ne), this.result = je;
                };
                yc.prototype.evaluate = function(Ne) {
                  return this.result.evaluate(Ne);
                }, yc.prototype.eachChild = function(Ne) {
                  for (var je = 0, ut = this.bindings; je < ut.length; je += 1) Ne(ut[je][1]);
                  Ne(this.result);
                }, yc.parse = function(Ne, je) {
                  if (Ne.length < 4) return je.error("Expected at least 3 arguments, but found " + (Ne.length - 1) + " instead.");
                  for (var ut = [], kt = 1; kt < Ne.length - 1; kt += 2) {
                    var Jt = Ne[kt];
                    if (typeof Jt != "string") return je.error("Expected string, but found " + typeof Jt + " instead.", kt);
                    if (/[^a-zA-Z0-9_]/.test(Jt)) return je.error("Variable names must contain only alphanumeric characters or '_'.", kt);
                    var fn = je.parse(Ne[kt + 1], kt + 1);
                    if (!fn) return null;
                    ut.push([Jt, fn]);
                  }
                  var Sn = je.parse(Ne[Ne.length - 1], Ne.length - 1, je.expectedType, ut);
                  return Sn ? new yc(ut, Sn) : null;
                }, yc.prototype.outputDefined = function() {
                  return this.result.outputDefined();
                }, yc.prototype.serialize = function() {
                  for (var Ne = ["let"], je = 0, ut = this.bindings; je < ut.length; je += 1) {
                    var kt = ut[je], Jt = kt[0], fn = kt[1];
                    Ne.push(Jt, fn.serialize());
                  }
                  return Ne.push(this.result.serialize()), Ne;
                };
                var gu = function(Ne, je, ut) {
                  this.type = Ne, this.index = je, this.input = ut;
                };
                gu.parse = function(Ne, je) {
                  if (Ne.length !== 3) return je.error("Expected 2 arguments, but found " + (Ne.length - 1) + " instead.");
                  var ut = je.parse(Ne[1], 1, Jn), kt = je.parse(Ne[2], 2, gi(je.expectedType || jr));
                  if (!ut || !kt) return null;
                  var Jt = kt.type;
                  return new gu(Jt.itemType, ut, kt);
                }, gu.prototype.evaluate = function(Ne) {
                  var je = this.index.evaluate(Ne), ut = this.input.evaluate(Ne);
                  if (je < 0) throw new Dr("Array index out of bounds: " + je + " < 0.");
                  if (je >= ut.length) throw new Dr("Array index out of bounds: " + je + " > " + (ut.length - 1) + ".");
                  if (je !== Math.floor(je)) throw new Dr("Array index must be an integer, but found " + je + " instead.");
                  return ut[je];
                }, gu.prototype.eachChild = function(Ne) {
                  Ne(this.index), Ne(this.input);
                }, gu.prototype.outputDefined = function() {
                  return !1;
                }, gu.prototype.serialize = function() {
                  return ["at", this.index.serialize(), this.input.serialize()];
                };
                var Nc = function(Ne, je) {
                  this.type = mr, this.needle = Ne, this.haystack = je;
                };
                Nc.parse = function(Ne, je) {
                  if (Ne.length !== 3) return je.error("Expected 2 arguments, but found " + (Ne.length - 1) + " instead.");
                  var ut = je.parse(Ne[1], 1, jr), kt = je.parse(Ne[2], 2, jr);
                  return ut && kt ? bi(ut.type, [mr, Lr, Jn, Kn, jr]) ? new Nc(ut, kt) : je.error("Expected first argument to be of type boolean, string, number or null, but found " + Zr(ut.type) + " instead") : null;
                }, Nc.prototype.evaluate = function(Ne) {
                  var je = this.needle.evaluate(Ne), ut = this.haystack.evaluate(Ne);
                  if (!ut) return !1;
                  if (!Zi(je, ["boolean", "string", "number", "null"])) throw new Dr("Expected first argument to be of type boolean, string, number or null, but found " + Zr(Gr(je)) + " instead.");
                  if (!Zi(ut, ["string", "array"])) throw new Dr("Expected second argument to be of type array or string, but found " + Zr(Gr(ut)) + " instead.");
                  return ut.indexOf(je) >= 0;
                }, Nc.prototype.eachChild = function(Ne) {
                  Ne(this.needle), Ne(this.haystack);
                }, Nc.prototype.outputDefined = function() {
                  return !0;
                }, Nc.prototype.serialize = function() {
                  return ["in", this.needle.serialize(), this.haystack.serialize()];
                };
                var Cu = function(Ne, je, ut) {
                  this.type = Jn, this.needle = Ne, this.haystack = je, this.fromIndex = ut;
                };
                Cu.parse = function(Ne, je) {
                  if (Ne.length <= 2 || Ne.length >= 5) return je.error("Expected 3 or 4 arguments, but found " + (Ne.length - 1) + " instead.");
                  var ut = je.parse(Ne[1], 1, jr), kt = je.parse(Ne[2], 2, jr);
                  if (!ut || !kt) return null;
                  if (!bi(ut.type, [mr, Lr, Jn, Kn, jr])) return je.error("Expected first argument to be of type boolean, string, number or null, but found " + Zr(ut.type) + " instead");
                  if (Ne.length === 4) {
                    var Jt = je.parse(Ne[3], 3, Jn);
                    return Jt ? new Cu(ut, kt, Jt) : null;
                  }
                  return new Cu(ut, kt);
                }, Cu.prototype.evaluate = function(Ne) {
                  var je = this.needle.evaluate(Ne), ut = this.haystack.evaluate(Ne);
                  if (!Zi(je, ["boolean", "string", "number", "null"])) throw new Dr("Expected first argument to be of type boolean, string, number or null, but found " + Zr(Gr(je)) + " instead.");
                  if (!Zi(ut, ["string", "array"])) throw new Dr("Expected second argument to be of type array or string, but found " + Zr(Gr(ut)) + " instead.");
                  if (this.fromIndex) {
                    var kt = this.fromIndex.evaluate(Ne);
                    return ut.indexOf(je, kt);
                  }
                  return ut.indexOf(je);
                }, Cu.prototype.eachChild = function(Ne) {
                  Ne(this.needle), Ne(this.haystack), this.fromIndex && Ne(this.fromIndex);
                }, Cu.prototype.outputDefined = function() {
                  return !1;
                }, Cu.prototype.serialize = function() {
                  if (this.fromIndex != null && this.fromIndex !== void 0) {
                    var Ne = this.fromIndex.serialize();
                    return ["index-of", this.needle.serialize(), this.haystack.serialize(), Ne];
                  }
                  return ["index-of", this.needle.serialize(), this.haystack.serialize()];
                };
                var zc = function(Ne, je, ut, kt, Jt, fn) {
                  this.inputType = Ne, this.type = je, this.input = ut, this.cases = kt, this.outputs = Jt, this.otherwise = fn;
                };
                zc.parse = function(Ne, je) {
                  if (Ne.length < 5) return je.error("Expected at least 4 arguments, but found only " + (Ne.length - 1) + ".");
                  if (Ne.length % 2 != 1) return je.error("Expected an even number of arguments.");
                  var ut, kt;
                  je.expectedType && je.expectedType.kind !== "value" && (kt = je.expectedType);
                  for (var Jt = {}, fn = [], Sn = 2; Sn < Ne.length - 1; Sn += 2) {
                    var Hn = Ne[Sn], nr = Ne[Sn + 1];
                    Array.isArray(Hn) || (Hn = [Hn]);
                    var kr = je.concat(Sn);
                    if (Hn.length === 0) return kr.error("Expected at least one branch label.");
                    for (var Ir = 0, qr = Hn; Ir < qr.length; Ir += 1) {
                      var ci = qr[Ir];
                      if (typeof ci != "number" && typeof ci != "string") return kr.error("Branch labels must be numbers or strings.");
                      if (typeof ci == "number" && Math.abs(ci) > Number.MAX_SAFE_INTEGER) return kr.error("Branch labels must be integers no larger than " + Number.MAX_SAFE_INTEGER + ".");
                      if (typeof ci == "number" && Math.floor(ci) !== ci) return kr.error("Numeric branch labels must be integer values.");
                      if (ut) {
                        if (kr.checkSubtype(ut, Gr(ci))) return null;
                      } else ut = Gr(ci);
                      if (Jt[String(ci)] !== void 0) return kr.error("Branch labels must be unique.");
                      Jt[String(ci)] = fn.length;
                    }
                    var ni = je.parse(nr, Sn, kt);
                    if (!ni) return null;
                    kt = kt || ni.type, fn.push(ni);
                  }
                  var Ii = je.parse(Ne[1], 1, jr);
                  if (!Ii) return null;
                  var Gi = je.parse(Ne[Ne.length - 1], Ne.length - 1, kt);
                  return Gi ? Ii.type.kind !== "value" && je.concat(1).checkSubtype(ut, Ii.type) ? null : new zc(ut, kt, Ii, Jt, fn, Gi) : null;
                }, zc.prototype.evaluate = function(Ne) {
                  var je = this.input.evaluate(Ne);
                  return (Gr(je) === this.inputType && this.outputs[this.cases[je]] || this.otherwise).evaluate(Ne);
                }, zc.prototype.eachChild = function(Ne) {
                  Ne(this.input), this.outputs.forEach(Ne), Ne(this.otherwise);
                }, zc.prototype.outputDefined = function() {
                  return this.outputs.every(function(Ne) {
                    return Ne.outputDefined();
                  }) && this.otherwise.outputDefined();
                }, zc.prototype.serialize = function() {
                  for (var Ne = this, je = ["match", this.input.serialize()], ut = [], kt = {}, Jt = 0, fn = Object.keys(this.cases).sort(); Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt];
                    (qr = kt[this.cases[Sn]]) === void 0 ? (kt[this.cases[Sn]] = ut.length, ut.push([this.cases[Sn], [Sn]])) : ut[qr][1].push(Sn);
                  }
                  for (var Hn = function(ni) {
                    return Ne.inputType.kind === "number" ? Number(ni) : ni;
                  }, nr = 0, kr = ut; nr < kr.length; nr += 1) {
                    var Ir = kr[nr], qr = Ir[0], ci = Ir[1];
                    ci.length === 1 ? je.push(Hn(ci[0])) : je.push(ci.map(Hn)), je.push(this.outputs[outputIndex$1].serialize());
                  }
                  return je.push(this.otherwise.serialize()), je;
                };
                var Kc = function(Ne, je, ut) {
                  this.type = Ne, this.branches = je, this.otherwise = ut;
                };
                Kc.parse = function(Ne, je) {
                  if (Ne.length < 4) return je.error("Expected at least 3 arguments, but found only " + (Ne.length - 1) + ".");
                  if (Ne.length % 2 != 0) return je.error("Expected an odd number of arguments.");
                  var ut;
                  je.expectedType && je.expectedType.kind !== "value" && (ut = je.expectedType);
                  for (var kt = [], Jt = 1; Jt < Ne.length - 1; Jt += 2) {
                    var fn = je.parse(Ne[Jt], Jt, mr);
                    if (!fn) return null;
                    var Sn = je.parse(Ne[Jt + 1], Jt + 1, ut);
                    if (!Sn) return null;
                    kt.push([fn, Sn]), ut = ut || Sn.type;
                  }
                  var Hn = je.parse(Ne[Ne.length - 1], Ne.length - 1, ut);
                  return Hn ? new Kc(ut, kt, Hn) : null;
                }, Kc.prototype.evaluate = function(Ne) {
                  for (var je = 0, ut = this.branches; je < ut.length; je += 1) {
                    var kt = ut[je], Jt = kt[0], fn = kt[1];
                    if (Jt.evaluate(Ne)) return fn.evaluate(Ne);
                  }
                  return this.otherwise.evaluate(Ne);
                }, Kc.prototype.eachChild = function(Ne) {
                  for (var je = 0, ut = this.branches; je < ut.length; je += 1) {
                    var kt = ut[je], Jt = kt[0], fn = kt[1];
                    Ne(Jt), Ne(fn);
                  }
                  Ne(this.otherwise);
                }, Kc.prototype.outputDefined = function() {
                  return this.branches.every(function(Ne) {
                    return Ne[0], Ne[1].outputDefined();
                  }) && this.otherwise.outputDefined();
                }, Kc.prototype.serialize = function() {
                  var Ne = ["case"];
                  return this.eachChild(function(je) {
                    Ne.push(je.serialize());
                  }), Ne;
                };
                var zi = function(Ne, je, ut, kt) {
                  this.type = Ne, this.input = je, this.beginIndex = ut, this.endIndex = kt;
                };
                function Fu(Ne, je) {
                  return Ne === "==" || Ne === "!=" ? je.kind === "boolean" || je.kind === "string" || je.kind === "number" || je.kind === "null" || je.kind === "value" : je.kind === "string" || je.kind === "number" || je.kind === "value";
                }
                function d1(Ne, je, ut, kt) {
                  return kt.compare(je, ut) === 0;
                }
                function Gu(Ne, je, ut) {
                  var kt = Ne !== "==" && Ne !== "!=";
                  return function() {
                    function Jt(fn, Sn, Hn) {
                      this.type = mr, this.lhs = fn, this.rhs = Sn, this.collator = Hn, this.hasUntypedArgument = fn.type.kind === "value" || Sn.type.kind === "value";
                    }
                    return Jt.parse = function(fn, Sn) {
                      if (fn.length !== 3 && fn.length !== 4) return Sn.error("Expected two or three arguments.");
                      var Hn = fn[0], nr = Sn.parse(fn[1], 1, jr);
                      if (!nr) return null;
                      if (!Fu(Hn, nr.type)) return Sn.concat(1).error('"' + Hn + `" comparisons are not supported for type '` + Zr(nr.type) + "'.");
                      var kr = Sn.parse(fn[2], 2, jr);
                      if (!kr) return null;
                      if (!Fu(Hn, kr.type)) return Sn.concat(2).error('"' + Hn + `" comparisons are not supported for type '` + Zr(kr.type) + "'.");
                      if (nr.type.kind !== kr.type.kind && nr.type.kind !== "value" && kr.type.kind !== "value") return Sn.error("Cannot compare types '" + Zr(nr.type) + "' and '" + Zr(kr.type) + "'.");
                      kt && (nr.type.kind === "value" && kr.type.kind !== "value" ? nr = new zr(kr.type, [nr]) : nr.type.kind !== "value" && kr.type.kind === "value" && (kr = new zr(nr.type, [kr])));
                      var Ir = null;
                      if (fn.length === 4) {
                        if (nr.type.kind !== "string" && kr.type.kind !== "string" && nr.type.kind !== "value" && kr.type.kind !== "value") return Sn.error("Cannot use collator to compare non-string types.");
                        if (!(Ir = Sn.parse(fn[3], 3, Mr))) return null;
                      }
                      return new Jt(nr, kr, Ir);
                    }, Jt.prototype.evaluate = function(fn) {
                      var Sn = this.lhs.evaluate(fn), Hn = this.rhs.evaluate(fn);
                      if (kt && this.hasUntypedArgument) {
                        var nr = Gr(Sn), kr = Gr(Hn);
                        if (nr.kind !== kr.kind || nr.kind !== "string" && nr.kind !== "number") throw new Dr('Expected arguments for "' + Ne + '" to be (string, string) or (number, number), but found (' + nr.kind + ", " + kr.kind + ") instead.");
                      }
                      if (this.collator && !kt && this.hasUntypedArgument) {
                        var Ir = Gr(Sn), qr = Gr(Hn);
                        if (Ir.kind !== "string" || qr.kind !== "string") return je(fn, Sn, Hn);
                      }
                      return this.collator ? ut(fn, Sn, Hn, this.collator.evaluate(fn)) : je(fn, Sn, Hn);
                    }, Jt.prototype.eachChild = function(fn) {
                      fn(this.lhs), fn(this.rhs), this.collator && fn(this.collator);
                    }, Jt.prototype.outputDefined = function() {
                      return !0;
                    }, Jt.prototype.serialize = function() {
                      var fn = [Ne];
                      return this.eachChild(function(Sn) {
                        fn.push(Sn.serialize());
                      }), fn;
                    }, Jt;
                  }();
                }
                zi.parse = function(Ne, je) {
                  if (Ne.length <= 2 || Ne.length >= 5) return je.error("Expected 3 or 4 arguments, but found " + (Ne.length - 1) + " instead.");
                  var ut = je.parse(Ne[1], 1, jr), kt = je.parse(Ne[2], 2, Jn);
                  if (!ut || !kt) return null;
                  if (!bi(ut.type, [gi(jr), Lr, jr])) return je.error("Expected first argument to be of type array or string, but found " + Zr(ut.type) + " instead");
                  if (Ne.length === 4) {
                    var Jt = je.parse(Ne[3], 3, Jn);
                    return Jt ? new zi(ut.type, ut, kt, Jt) : null;
                  }
                  return new zi(ut.type, ut, kt);
                }, zi.prototype.evaluate = function(Ne) {
                  var je = this.input.evaluate(Ne), ut = this.beginIndex.evaluate(Ne);
                  if (!Zi(je, ["string", "array"])) throw new Dr("Expected first argument to be of type array or string, but found " + Zr(Gr(je)) + " instead.");
                  if (this.endIndex) {
                    var kt = this.endIndex.evaluate(Ne);
                    return je.slice(ut, kt);
                  }
                  return je.slice(ut);
                }, zi.prototype.eachChild = function(Ne) {
                  Ne(this.input), Ne(this.beginIndex), this.endIndex && Ne(this.endIndex);
                }, zi.prototype.outputDefined = function() {
                  return !1;
                }, zi.prototype.serialize = function() {
                  if (this.endIndex != null && this.endIndex !== void 0) {
                    var Ne = this.endIndex.serialize();
                    return ["slice", this.input.serialize(), this.beginIndex.serialize(), Ne];
                  }
                  return ["slice", this.input.serialize(), this.beginIndex.serialize()];
                };
                var Ku = Gu("==", function(Ne, je, ut) {
                  return je === ut;
                }, d1), I1 = Gu("!=", function(Ne, je, ut) {
                  return je !== ut;
                }, function(Ne, je, ut, kt) {
                  return !d1(0, je, ut, kt);
                }), f1 = Gu("<", function(Ne, je, ut) {
                  return je < ut;
                }, function(Ne, je, ut, kt) {
                  return kt.compare(je, ut) < 0;
                }), D1 = Gu(">", function(Ne, je, ut) {
                  return je > ut;
                }, function(Ne, je, ut, kt) {
                  return kt.compare(je, ut) > 0;
                }), Hl = Gu("<=", function(Ne, je, ut) {
                  return je <= ut;
                }, function(Ne, je, ut, kt) {
                  return kt.compare(je, ut) <= 0;
                }), Kl = Gu(">=", function(Ne, je, ut) {
                  return je >= ut;
                }, function(Ne, je, ut, kt) {
                  return kt.compare(je, ut) >= 0;
                }), uu = function(Ne, je, ut, kt, Jt) {
                  this.type = Lr, this.number = Ne, this.locale = je, this.currency = ut, this.minFractionDigits = kt, this.maxFractionDigits = Jt;
                };
                uu.parse = function(Ne, je) {
                  if (Ne.length !== 3) return je.error("Expected two arguments.");
                  var ut = je.parse(Ne[1], 1, Jn);
                  if (!ut) return null;
                  var kt = Ne[2];
                  if (typeof kt != "object" || Array.isArray(kt)) return je.error("NumberFormat options argument must be an object.");
                  var Jt = null;
                  if (kt.locale && !(Jt = je.parse(kt.locale, 1, Lr))) return null;
                  var fn = null;
                  if (kt.currency && !(fn = je.parse(kt.currency, 1, Lr))) return null;
                  var Sn = null;
                  if (kt["min-fraction-digits"] && !(Sn = je.parse(kt["min-fraction-digits"], 1, Jn))) return null;
                  var Hn = null;
                  return kt["max-fraction-digits"] && !(Hn = je.parse(kt["max-fraction-digits"], 1, Jn)) ? null : new uu(ut, Jt, fn, Sn, Hn);
                }, uu.prototype.evaluate = function(Ne) {
                  return new Intl.NumberFormat(this.locale ? this.locale.evaluate(Ne) : [], { style: this.currency ? "currency" : "decimal", currency: this.currency ? this.currency.evaluate(Ne) : void 0, minimumFractionDigits: this.minFractionDigits ? this.minFractionDigits.evaluate(Ne) : void 0, maximumFractionDigits: this.maxFractionDigits ? this.maxFractionDigits.evaluate(Ne) : void 0 }).format(this.number.evaluate(Ne));
                }, uu.prototype.eachChild = function(Ne) {
                  Ne(this.number), this.locale && Ne(this.locale), this.currency && Ne(this.currency), this.minFractionDigits && Ne(this.minFractionDigits), this.maxFractionDigits && Ne(this.maxFractionDigits);
                }, uu.prototype.outputDefined = function() {
                  return !1;
                }, uu.prototype.serialize = function() {
                  var Ne = {};
                  return this.locale && (Ne.locale = this.locale.serialize()), this.currency && (Ne.currency = this.currency.serialize()), this.minFractionDigits && (Ne["min-fraction-digits"] = this.minFractionDigits.serialize()), this.maxFractionDigits && (Ne["max-fraction-digits"] = this.maxFractionDigits.serialize()), ["number-format", this.number.serialize(), Ne];
                };
                var Hs = function(Ne) {
                  this.type = Jn, this.input = Ne;
                };
                Hs.parse = function(Ne, je) {
                  if (Ne.length !== 2) return je.error("Expected 1 argument, but found " + (Ne.length - 1) + " instead.");
                  var ut = je.parse(Ne[1], 1);
                  return ut ? ut.type.kind !== "array" && ut.type.kind !== "string" && ut.type.kind !== "value" ? je.error("Expected argument of type string or array, but found " + Zr(ut.type) + " instead.") : new Hs(ut) : null;
                }, Hs.prototype.evaluate = function(Ne) {
                  var je = this.input.evaluate(Ne);
                  if (typeof je == "string" || Array.isArray(je)) return je.length;
                  throw new Dr("Expected value to be of type string or array, but found " + Zr(Gr(je)) + " instead.");
                }, Hs.prototype.eachChild = function(Ne) {
                  Ne(this.input);
                }, Hs.prototype.outputDefined = function() {
                  return !1;
                }, Hs.prototype.serialize = function() {
                  var Ne = ["length"];
                  return this.eachChild(function(je) {
                    Ne.push(je.serialize());
                  }), Ne;
                };
                var b1 = { "==": Ku, "!=": I1, ">": D1, "<": f1, ">=": Kl, "<=": Hl, array: zr, at: gu, boolean: zr, case: Kc, coalesce: Xl, collator: rr, format: di, image: Da, in: Nc, "index-of": Cu, interpolate: Vl, "interpolate-hcl": Vl, "interpolate-lab": Vl, length: Hs, let: yc, literal: Sr, match: zc, number: zr, "number-format": uu, object: zr, slice: zi, step: nn, string: zr, "to-boolean": wa, "to-color": wa, "to-number": wa, "to-string": wa, var: ua, within: Oi };
                function Wl(Ne, je) {
                  var ut = je[0], kt = je[1], Jt = je[2], fn = je[3];
                  ut = ut.evaluate(Ne), kt = kt.evaluate(Ne), Jt = Jt.evaluate(Ne);
                  var Sn = fn ? fn.evaluate(Ne) : 1, Hn = aa(ut, kt, Jt, Sn);
                  if (Hn) throw new Dr(Hn);
                  return new Pr(ut / 255 * Sn, kt / 255 * Sn, Jt / 255 * Sn, Sn);
                }
                function zl(Ne, je) {
                  return Ne in je;
                }
                function os(Ne, je) {
                  var ut = je[Ne];
                  return ut === void 0 ? null : ut;
                }
                function jo(Ne) {
                  return { type: Ne };
                }
                function uo(Ne) {
                  return { result: "success", value: Ne };
                }
                function so(Ne) {
                  return { result: "error", value: Ne };
                }
                function So(Ne) {
                  return Ne["property-type"] === "data-driven" || Ne["property-type"] === "cross-faded-data-driven";
                }
                function Jc(Ne) {
                  return !!Ne.expression && Ne.expression.parameters.indexOf("zoom") > -1;
                }
                function No(Ne) {
                  return !!Ne.expression && Ne.expression.interpolated;
                }
                function ro(Ne) {
                  return Ne instanceof Number ? "number" : Ne instanceof String ? "string" : Ne instanceof Boolean ? "boolean" : Array.isArray(Ne) ? "array" : Ne === null ? "null" : typeof Ne;
                }
                function Io(Ne) {
                  return typeof Ne == "object" && Ne !== null && !Array.isArray(Ne);
                }
                function bs(Ne) {
                  return Ne;
                }
                function ws(Ne, je) {
                  var ut, kt, Jt, fn = je.type === "color", Sn = Ne.stops && typeof Ne.stops[0][0] == "object", Hn = Sn || Ne.property !== void 0, nr = Sn || !Hn, kr = Ne.type || (No(je) ? "exponential" : "interval");
                  if (fn && ((Ne = In({}, Ne)).stops && (Ne.stops = Ne.stops.map(function(Lo) {
                    return [Lo[0], Pr.parse(Lo[1])];
                  })), Ne.default ? Ne.default = Pr.parse(Ne.default) : Ne.default = Pr.parse(je.default)), Ne.colorSpace && Ne.colorSpace !== "rgb" && !vs[Ne.colorSpace]) throw new Error("Unknown color space: " + Ne.colorSpace);
                  if (kr === "exponential") ut = Ms;
                  else if (kr === "interval") ut = Ls;
                  else if (kr === "categorical") {
                    ut = Yo, kt = /* @__PURE__ */ Object.create(null);
                    for (var Ir = 0, qr = Ne.stops; Ir < qr.length; Ir += 1) {
                      var ci = qr[Ir];
                      kt[ci[0]] = ci[1];
                    }
                    Jt = typeof Ne.stops[0][0];
                  } else {
                    if (kr !== "identity") throw new Error('Unknown function type "' + kr + '"');
                    ut = vi;
                  }
                  if (Sn) {
                    for (var ni = {}, Ii = [], Gi = 0; Gi < Ne.stops.length; Gi++) {
                      var xa = Ne.stops[Gi], ga = xa[0].zoom;
                      ni[ga] === void 0 && (ni[ga] = { zoom: ga, type: Ne.type, property: Ne.property, default: Ne.default, stops: [] }, Ii.push(ga)), ni[ga].stops.push([xa[0].value, xa[1]]);
                    }
                    for (var $a = [], Va = 0, eo = Ii; Va < eo.length; Va += 1) {
                      var co = eo[Va];
                      $a.push([ni[co].zoom, ws(ni[co], je)]);
                    }
                    var wo = { name: "linear" };
                    return { kind: "composite", interpolationType: wo, interpolationFactor: Vl.interpolationFactor.bind(void 0, wo), zoomStops: $a.map(function(Lo) {
                      return Lo[0];
                    }), evaluate: function(Lo, As) {
                      var Fs = Lo.zoom;
                      return Ms({ stops: $a, base: Ne.base }, je, Fs).evaluate(Fs, As);
                    } };
                  }
                  if (nr) {
                    var ds = kr === "exponential" ? { name: "exponential", base: Ne.base !== void 0 ? Ne.base : 1 } : null;
                    return { kind: "camera", interpolationType: ds, interpolationFactor: Vl.interpolationFactor.bind(void 0, ds), zoomStops: Ne.stops.map(function(Lo) {
                      return Lo[0];
                    }), evaluate: function(Lo) {
                      var As = Lo.zoom;
                      return ut(Ne, je, As, kt, Jt);
                    } };
                  }
                  return { kind: "source", evaluate: function(Lo, As) {
                    var Fs = As && As.properties ? As.properties[Ne.property] : void 0;
                    return Fs === void 0 ? zo(Ne.default, je.default) : ut(Ne, je, Fs, kt, Jt);
                  } };
                }
                function zo(Ne, je, ut) {
                  return Ne !== void 0 ? Ne : je !== void 0 ? je : ut !== void 0 ? ut : void 0;
                }
                function Yo(Ne, je, ut, kt, Jt) {
                  return zo(typeof ut === Jt ? kt[ut] : void 0, Ne.default, je.default);
                }
                function Ls(Ne, je, ut) {
                  if (ro(ut) !== "number") return zo(Ne.default, je.default);
                  var kt = Ne.stops.length;
                  if (kt === 1 || ut <= Ne.stops[0][0]) return Ne.stops[0][1];
                  if (ut >= Ne.stops[kt - 1][0]) return Ne.stops[kt - 1][1];
                  var Jt = na(Ne.stops.map(function(fn) {
                    return fn[0];
                  }), ut);
                  return Ne.stops[Jt][1];
                }
                function Ms(Ne, je, ut) {
                  var kt = Ne.base !== void 0 ? Ne.base : 1;
                  if (ro(ut) !== "number") return zo(Ne.default, je.default);
                  var Jt = Ne.stops.length;
                  if (Jt === 1 || ut <= Ne.stops[0][0]) return Ne.stops[0][1];
                  if (ut >= Ne.stops[Jt - 1][0]) return Ne.stops[Jt - 1][1];
                  var fn = na(Ne.stops.map(function(qr) {
                    return qr[0];
                  }), ut), Sn = function(qr, ci, ni, Ii) {
                    var Gi = Ii - ni, xa = qr - ni;
                    return Gi === 0 ? 0 : ci === 1 ? xa / Gi : (Math.pow(ci, xa) - 1) / (Math.pow(ci, Gi) - 1);
                  }(ut, kt, Ne.stops[fn][0], Ne.stops[fn + 1][0]), Hn = Ne.stops[fn][1], nr = Ne.stops[fn + 1][1], kr = Dn[je.type] || bs;
                  if (Ne.colorSpace && Ne.colorSpace !== "rgb") {
                    var Ir = vs[Ne.colorSpace];
                    kr = function(qr, ci) {
                      return Ir.reverse(Ir.interpolate(Ir.forward(qr), Ir.forward(ci), Sn));
                    };
                  }
                  return typeof Hn.evaluate == "function" ? { evaluate: function() {
                    for (var qr = [], ci = arguments.length; ci--; ) qr[ci] = arguments[ci];
                    var ni = Hn.evaluate.apply(void 0, qr), Ii = nr.evaluate.apply(void 0, qr);
                    if (ni !== void 0 && Ii !== void 0) return kr(ni, Ii, Sn);
                  } } : kr(Hn, nr, Sn);
                }
                function vi(Ne, je, ut) {
                  return je.type === "color" ? ut = Pr.parse(ut) : je.type === "formatted" ? ut = oa.fromString(ut.toString()) : je.type === "resolvedImage" ? ut = ka.fromString(ut.toString()) : ro(ut) === je.type || je.type === "enum" && je.values[ut] || (ut = void 0), zo(ut, Ne.default, je.default);
                }
                Ur.register(b1, { error: [{ kind: "error" }, [Lr], function(Ne, je) {
                  var ut = je[0];
                  throw new Dr(ut.evaluate(Ne));
                }], typeof: [Lr, [jr], function(Ne, je) {
                  return Zr(Gr(je[0].evaluate(Ne)));
                }], "to-rgba": [gi(Jn, 4), [Ar], function(Ne, je) {
                  return je[0].evaluate(Ne).toArray();
                }], rgb: [Ar, [Jn, Jn, Jn], Wl], rgba: [Ar, [Jn, Jn, Jn, Jn], Wl], has: { type: mr, overloads: [[[Lr], function(Ne, je) {
                  return zl(je[0].evaluate(Ne), Ne.properties());
                }], [[Lr, Yr], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return zl(ut.evaluate(Ne), kt.evaluate(Ne));
                }]] }, get: { type: jr, overloads: [[[Lr], function(Ne, je) {
                  return os(je[0].evaluate(Ne), Ne.properties());
                }], [[Lr, Yr], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return os(ut.evaluate(Ne), kt.evaluate(Ne));
                }]] }, "feature-state": [jr, [Lr], function(Ne, je) {
                  return os(je[0].evaluate(Ne), Ne.featureState || {});
                }], properties: [Yr, [], function(Ne) {
                  return Ne.properties();
                }], "geometry-type": [Lr, [], function(Ne) {
                  return Ne.geometryType();
                }], id: [jr, [], function(Ne) {
                  return Ne.id();
                }], zoom: [Jn, [], function(Ne) {
                  return Ne.globals.zoom;
                }], "heatmap-density": [Jn, [], function(Ne) {
                  return Ne.globals.heatmapDensity || 0;
                }], "line-progress": [Jn, [], function(Ne) {
                  return Ne.globals.lineProgress || 0;
                }], accumulated: [jr, [], function(Ne) {
                  return Ne.globals.accumulated === void 0 ? null : Ne.globals.accumulated;
                }], "+": [Jn, jo(Jn), function(Ne, je) {
                  for (var ut = 0, kt = 0, Jt = je; kt < Jt.length; kt += 1) ut += Jt[kt].evaluate(Ne);
                  return ut;
                }], "*": [Jn, jo(Jn), function(Ne, je) {
                  for (var ut = 1, kt = 0, Jt = je; kt < Jt.length; kt += 1) ut *= Jt[kt].evaluate(Ne);
                  return ut;
                }], "-": { type: Jn, overloads: [[[Jn, Jn], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return ut.evaluate(Ne) - kt.evaluate(Ne);
                }], [[Jn], function(Ne, je) {
                  return -je[0].evaluate(Ne);
                }]] }, "/": [Jn, [Jn, Jn], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return ut.evaluate(Ne) / kt.evaluate(Ne);
                }], "%": [Jn, [Jn, Jn], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return ut.evaluate(Ne) % kt.evaluate(Ne);
                }], ln2: [Jn, [], function() {
                  return Math.LN2;
                }], pi: [Jn, [], function() {
                  return Math.PI;
                }], e: [Jn, [], function() {
                  return Math.E;
                }], "^": [Jn, [Jn, Jn], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return Math.pow(ut.evaluate(Ne), kt.evaluate(Ne));
                }], sqrt: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.sqrt(ut.evaluate(Ne));
                }], log10: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.log(ut.evaluate(Ne)) / Math.LN10;
                }], ln: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.log(ut.evaluate(Ne));
                }], log2: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.log(ut.evaluate(Ne)) / Math.LN2;
                }], sin: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.sin(ut.evaluate(Ne));
                }], cos: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.cos(ut.evaluate(Ne));
                }], tan: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.tan(ut.evaluate(Ne));
                }], asin: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.asin(ut.evaluate(Ne));
                }], acos: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.acos(ut.evaluate(Ne));
                }], atan: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.atan(ut.evaluate(Ne));
                }], min: [Jn, jo(Jn), function(Ne, je) {
                  return Math.min.apply(Math, je.map(function(ut) {
                    return ut.evaluate(Ne);
                  }));
                }], max: [Jn, jo(Jn), function(Ne, je) {
                  return Math.max.apply(Math, je.map(function(ut) {
                    return ut.evaluate(Ne);
                  }));
                }], abs: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.abs(ut.evaluate(Ne));
                }], round: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0].evaluate(Ne);
                  return ut < 0 ? -Math.round(-ut) : Math.round(ut);
                }], floor: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.floor(ut.evaluate(Ne));
                }], ceil: [Jn, [Jn], function(Ne, je) {
                  var ut = je[0];
                  return Math.ceil(ut.evaluate(Ne));
                }], "filter-==": [mr, [Lr, jr], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return Ne.properties()[ut.value] === kt.value;
                }], "filter-id-==": [mr, [jr], function(Ne, je) {
                  var ut = je[0];
                  return Ne.id() === ut.value;
                }], "filter-type-==": [mr, [Lr], function(Ne, je) {
                  var ut = je[0];
                  return Ne.geometryType() === ut.value;
                }], "filter-<": [mr, [Lr, jr], function(Ne, je) {
                  var ut = je[0], kt = je[1], Jt = Ne.properties()[ut.value], fn = kt.value;
                  return typeof Jt == typeof fn && Jt < fn;
                }], "filter-id-<": [mr, [jr], function(Ne, je) {
                  var ut = je[0], kt = Ne.id(), Jt = ut.value;
                  return typeof kt == typeof Jt && kt < Jt;
                }], "filter->": [mr, [Lr, jr], function(Ne, je) {
                  var ut = je[0], kt = je[1], Jt = Ne.properties()[ut.value], fn = kt.value;
                  return typeof Jt == typeof fn && Jt > fn;
                }], "filter-id->": [mr, [jr], function(Ne, je) {
                  var ut = je[0], kt = Ne.id(), Jt = ut.value;
                  return typeof kt == typeof Jt && kt > Jt;
                }], "filter-<=": [mr, [Lr, jr], function(Ne, je) {
                  var ut = je[0], kt = je[1], Jt = Ne.properties()[ut.value], fn = kt.value;
                  return typeof Jt == typeof fn && Jt <= fn;
                }], "filter-id-<=": [mr, [jr], function(Ne, je) {
                  var ut = je[0], kt = Ne.id(), Jt = ut.value;
                  return typeof kt == typeof Jt && kt <= Jt;
                }], "filter->=": [mr, [Lr, jr], function(Ne, je) {
                  var ut = je[0], kt = je[1], Jt = Ne.properties()[ut.value], fn = kt.value;
                  return typeof Jt == typeof fn && Jt >= fn;
                }], "filter-id->=": [mr, [jr], function(Ne, je) {
                  var ut = je[0], kt = Ne.id(), Jt = ut.value;
                  return typeof kt == typeof Jt && kt >= Jt;
                }], "filter-has": [mr, [jr], function(Ne, je) {
                  return je[0].value in Ne.properties();
                }], "filter-has-id": [mr, [], function(Ne) {
                  return Ne.id() !== null && Ne.id() !== void 0;
                }], "filter-type-in": [mr, [gi(Lr)], function(Ne, je) {
                  return je[0].value.indexOf(Ne.geometryType()) >= 0;
                }], "filter-id-in": [mr, [gi(jr)], function(Ne, je) {
                  return je[0].value.indexOf(Ne.id()) >= 0;
                }], "filter-in-small": [mr, [Lr, gi(jr)], function(Ne, je) {
                  var ut = je[0];
                  return je[1].value.indexOf(Ne.properties()[ut.value]) >= 0;
                }], "filter-in-large": [mr, [Lr, gi(jr)], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return function(Jt, fn, Sn, Hn) {
                    for (; Sn <= Hn; ) {
                      var nr = Sn + Hn >> 1;
                      if (fn[nr] === Jt) return !0;
                      fn[nr] > Jt ? Hn = nr - 1 : Sn = nr + 1;
                    }
                    return !1;
                  }(Ne.properties()[ut.value], kt.value, 0, kt.value.length - 1);
                }], all: { type: mr, overloads: [[[mr, mr], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return ut.evaluate(Ne) && kt.evaluate(Ne);
                }], [jo(mr), function(Ne, je) {
                  for (var ut = 0, kt = je; ut < kt.length; ut += 1) if (!kt[ut].evaluate(Ne)) return !1;
                  return !0;
                }]] }, any: { type: mr, overloads: [[[mr, mr], function(Ne, je) {
                  var ut = je[0], kt = je[1];
                  return ut.evaluate(Ne) || kt.evaluate(Ne);
                }], [jo(mr), function(Ne, je) {
                  for (var ut = 0, kt = je; ut < kt.length; ut += 1) if (kt[ut].evaluate(Ne)) return !0;
                  return !1;
                }]] }, "!": [mr, [mr], function(Ne, je) {
                  return !je[0].evaluate(Ne);
                }], "is-supported-script": [mr, [Lr], function(Ne, je) {
                  var ut = je[0], kt = Ne.globals && Ne.globals.isSupportedScript;
                  return !kt || kt(ut.evaluate(Ne));
                }], upcase: [Lr, [Lr], function(Ne, je) {
                  return je[0].evaluate(Ne).toUpperCase();
                }], downcase: [Lr, [Lr], function(Ne, je) {
                  return je[0].evaluate(Ne).toLowerCase();
                }], concat: [Lr, jo(jr), function(Ne, je) {
                  return je.map(function(ut) {
                    return Ca(ut.evaluate(Ne));
                  }).join("");
                }], "resolved-locale": [Lr, [Mr], function(Ne, je) {
                  return je[0].evaluate(Ne).resolvedLocale();
                }] });
                var Ri = function(Ne, je) {
                  this.expression = Ne, this._warningHistory = {}, this._evaluator = new Qr(), this._defaultValue = je ? function(ut) {
                    return ut.type === "color" && Io(ut.default) ? new Pr(0, 0, 0, 0) : ut.type === "color" ? Pr.parse(ut.default) || null : ut.default === void 0 ? null : ut.default;
                  }(je) : null, this._enumValues = je && je.type === "enum" ? je.values : null;
                };
                function ta(Ne) {
                  return Array.isArray(Ne) && Ne.length > 0 && typeof Ne[0] == "string" && Ne[0] in b1;
                }
                function da(Ne, je) {
                  var ut = new Ki(b1, [], je ? function(Jt) {
                    var fn = { color: Ar, string: Lr, number: Jn, enum: Lr, boolean: mr, formatted: Wr, resolvedImage: Bi };
                    return Jt.type === "array" ? gi(fn[Jt.value] || jr, Jt.length) : fn[Jt.type];
                  }(je) : void 0), kt = ut.parse(Ne, void 0, void 0, void 0, je && je.type === "string" ? { typeAnnotation: "coerce" } : void 0);
                  return kt ? uo(new Ri(kt, je)) : so(ut.errors);
                }
                Ri.prototype.evaluateWithoutErrorHandling = function(Ne, je, ut, kt, Jt, fn) {
                  return this._evaluator.globals = Ne, this._evaluator.feature = je, this._evaluator.featureState = ut, this._evaluator.canonical = kt, this._evaluator.availableImages = Jt || null, this._evaluator.formattedSection = fn, this.expression.evaluate(this._evaluator);
                }, Ri.prototype.evaluate = function(Ne, je, ut, kt, Jt, fn) {
                  this._evaluator.globals = Ne, this._evaluator.feature = je || null, this._evaluator.featureState = ut || null, this._evaluator.canonical = kt, this._evaluator.availableImages = Jt || null, this._evaluator.formattedSection = fn || null;
                  try {
                    var Sn = this.expression.evaluate(this._evaluator);
                    if (Sn == null || typeof Sn == "number" && Sn != Sn) return this._defaultValue;
                    if (this._enumValues && !(Sn in this._enumValues)) throw new Dr("Expected value to be one of " + Object.keys(this._enumValues).map(function(Hn) {
                      return JSON.stringify(Hn);
                    }).join(", ") + ", but found " + JSON.stringify(Sn) + " instead.");
                    return Sn;
                  } catch (Hn) {
                    return this._warningHistory[Hn.message] || (this._warningHistory[Hn.message] = !0, typeof console < "u" && console.warn(Hn.message)), this._defaultValue;
                  }
                };
                var Ji = function(Ne, je) {
                  this.kind = Ne, this._styleExpression = je, this.isStateDependent = Ne !== "constant" && !Mi(je.expression);
                };
                Ji.prototype.evaluateWithoutErrorHandling = function(Ne, je, ut, kt, Jt, fn) {
                  return this._styleExpression.evaluateWithoutErrorHandling(Ne, je, ut, kt, Jt, fn);
                }, Ji.prototype.evaluate = function(Ne, je, ut, kt, Jt, fn) {
                  return this._styleExpression.evaluate(Ne, je, ut, kt, Jt, fn);
                };
                var Pa = function(Ne, je, ut, kt) {
                  this.kind = Ne, this.zoomStops = ut, this._styleExpression = je, this.isStateDependent = Ne !== "camera" && !Mi(je.expression), this.interpolationType = kt;
                };
                function Ga(Ne, je) {
                  if ((Ne = da(Ne, je)).result === "error") return Ne;
                  var ut = Ne.value.expression, kt = ki(ut);
                  if (!kt && !So(je)) return so([new jn("", "data expressions not supported")]);
                  var Jt = Vi(ut, ["zoom"]);
                  if (!Jt && !Jc(je)) return so([new jn("", "zoom expressions not supported")]);
                  var fn = Ba(ut);
                  if (!fn && !Jt) return so([new jn("", '"zoom" expression may only be used as input to a top-level "step" or "interpolate" expression.')]);
                  if (fn instanceof jn) return so([fn]);
                  if (fn instanceof Vl && !No(je)) return so([new jn("", '"interpolate" expressions cannot be used with this property')]);
                  if (!fn) return uo(new Ji(kt ? "constant" : "source", Ne.value));
                  var Sn = fn instanceof Vl ? fn.interpolation : void 0;
                  return uo(new Pa(kt ? "camera" : "composite", Ne.value, fn.labels, Sn));
                }
                Pa.prototype.evaluateWithoutErrorHandling = function(Ne, je, ut, kt, Jt, fn) {
                  return this._styleExpression.evaluateWithoutErrorHandling(Ne, je, ut, kt, Jt, fn);
                }, Pa.prototype.evaluate = function(Ne, je, ut, kt, Jt, fn) {
                  return this._styleExpression.evaluate(Ne, je, ut, kt, Jt, fn);
                }, Pa.prototype.interpolationFactor = function(Ne, je, ut) {
                  return this.interpolationType ? Vl.interpolationFactor(this.interpolationType, Ne, je, ut) : 0;
                };
                var Ha = function(Ne, je) {
                  this._parameters = Ne, this._specification = je, In(this, ws(this._parameters, this._specification));
                };
                function Ba(Ne) {
                  var je = null;
                  if (Ne instanceof yc) je = Ba(Ne.result);
                  else if (Ne instanceof Xl) for (var ut = 0, kt = Ne.args; ut < kt.length; ut += 1) {
                    var Jt = kt[ut];
                    if (je = Ba(Jt)) break;
                  }
                  else (Ne instanceof nn || Ne instanceof Vl) && Ne.input instanceof Ur && Ne.input.name === "zoom" && (je = Ne);
                  return je instanceof jn || Ne.eachChild(function(fn) {
                    var Sn = Ba(fn);
                    Sn instanceof jn ? je = Sn : !je && Sn ? je = new jn("", '"zoom" expression may only be used as input to a top-level "step" or "interpolate" expression.') : je && Sn && je !== Sn && (je = new jn("", 'Only one zoom-based "step" or "interpolate" subexpression may be used in an expression.'));
                  }), je;
                }
                function ao(Ne) {
                  var je = Ne.key, ut = Ne.value, kt = Ne.valueSpec || {}, Jt = Ne.objectElementValidators || {}, fn = Ne.style, Sn = Ne.styleSpec, Hn = [], nr = ro(ut);
                  if (nr !== "object") return [new Mn(je, ut, "object expected, " + nr + " found")];
                  for (var kr in ut) {
                    var Ir = kr.split(".")[0], qr = kt[Ir] || kt["*"], ci = void 0;
                    if (Jt[Ir]) ci = Jt[Ir];
                    else if (kt[Ir]) ci = ic;
                    else if (Jt["*"]) ci = Jt["*"];
                    else {
                      if (!kt["*"]) {
                        Hn.push(new Mn(je, ut[kr], 'unknown property "' + kr + '"'));
                        continue;
                      }
                      ci = ic;
                    }
                    Hn = Hn.concat(ci({ key: (je && je + ".") + kr, value: ut[kr], valueSpec: qr, style: fn, styleSpec: Sn, object: ut, objectKey: kr }, ut));
                  }
                  for (var ni in kt) Jt[ni] || kt[ni].required && kt[ni].default === void 0 && ut[ni] === void 0 && Hn.push(new Mn(je, ut, 'missing required property "' + ni + '"'));
                  return Hn;
                }
                function io(Ne) {
                  var je = Ne.value, ut = Ne.valueSpec, kt = Ne.style, Jt = Ne.styleSpec, fn = Ne.key, Sn = Ne.arrayElementValidator || ic;
                  if (ro(je) !== "array") return [new Mn(fn, je, "array expected, " + ro(je) + " found")];
                  if (ut.length && je.length !== ut.length) return [new Mn(fn, je, "array length " + ut.length + " expected, length " + je.length + " found")];
                  if (ut["min-length"] && je.length < ut["min-length"]) return [new Mn(fn, je, "array length at least " + ut["min-length"] + " expected, length " + je.length + " found")];
                  var Hn = { type: ut.value, values: ut.values };
                  Jt.$version < 7 && (Hn.function = ut.function), ro(ut.value) === "object" && (Hn = ut.value);
                  for (var nr = [], kr = 0; kr < je.length; kr++) nr = nr.concat(Sn({ array: je, arrayIndex: kr, value: je[kr], valueSpec: Hn, style: kt, styleSpec: Jt, key: fn + "[" + kr + "]" }));
                  return nr;
                }
                function To(Ne) {
                  var je = Ne.key, ut = Ne.value, kt = Ne.valueSpec, Jt = ro(ut);
                  return Jt === "number" && ut != ut && (Jt = "NaN"), Jt !== "number" ? [new Mn(je, ut, "number expected, " + Jt + " found")] : "minimum" in kt && ut < kt.minimum ? [new Mn(je, ut, ut + " is less than the minimum value " + kt.minimum)] : "maximum" in kt && ut > kt.maximum ? [new Mn(je, ut, ut + " is greater than the maximum value " + kt.maximum)] : [];
                }
                function Mo(Ne) {
                  var je, ut, kt, Jt = Ne.valueSpec, fn = kn(Ne.value.type), Sn = {}, Hn = fn !== "categorical" && Ne.value.property === void 0, nr = !Hn, kr = ro(Ne.value.stops) === "array" && ro(Ne.value.stops[0]) === "array" && ro(Ne.value.stops[0][0]) === "object", Ir = ao({ key: Ne.key, value: Ne.value, valueSpec: Ne.styleSpec.function, style: Ne.style, styleSpec: Ne.styleSpec, objectElementValidators: { stops: function(ni) {
                    if (fn === "identity") return [new Mn(ni.key, ni.value, 'identity function may not have a "stops" property')];
                    var Ii = [], Gi = ni.value;
                    return Ii = Ii.concat(io({ key: ni.key, value: Gi, valueSpec: ni.valueSpec, style: ni.style, styleSpec: ni.styleSpec, arrayElementValidator: qr })), ro(Gi) === "array" && Gi.length === 0 && Ii.push(new Mn(ni.key, Gi, "array must have at least one stop")), Ii;
                  }, default: function(ni) {
                    return ic({ key: ni.key, value: ni.value, valueSpec: Jt, style: ni.style, styleSpec: ni.styleSpec });
                  } } });
                  return fn === "identity" && Hn && Ir.push(new Mn(Ne.key, Ne.value, 'missing required property "property"')), fn === "identity" || Ne.value.stops || Ir.push(new Mn(Ne.key, Ne.value, 'missing required property "stops"')), fn === "exponential" && Ne.valueSpec.expression && !No(Ne.valueSpec) && Ir.push(new Mn(Ne.key, Ne.value, "exponential functions not supported")), Ne.styleSpec.$version >= 8 && (nr && !So(Ne.valueSpec) ? Ir.push(new Mn(Ne.key, Ne.value, "property functions not supported")) : Hn && !Jc(Ne.valueSpec) && Ir.push(new Mn(Ne.key, Ne.value, "zoom functions not supported"))), fn !== "categorical" && !kr || Ne.value.property !== void 0 || Ir.push(new Mn(Ne.key, Ne.value, '"property" property is required')), Ir;
                  function qr(ni) {
                    var Ii = [], Gi = ni.value, xa = ni.key;
                    if (ro(Gi) !== "array") return [new Mn(xa, Gi, "array expected, " + ro(Gi) + " found")];
                    if (Gi.length !== 2) return [new Mn(xa, Gi, "array length 2 expected, length " + Gi.length + " found")];
                    if (kr) {
                      if (ro(Gi[0]) !== "object") return [new Mn(xa, Gi, "object expected, " + ro(Gi[0]) + " found")];
                      if (Gi[0].zoom === void 0) return [new Mn(xa, Gi, "object stop key must have zoom")];
                      if (Gi[0].value === void 0) return [new Mn(xa, Gi, "object stop key must have value")];
                      if (kt && kt > kn(Gi[0].zoom)) return [new Mn(xa, Gi[0].zoom, "stop zoom values must appear in ascending order")];
                      kn(Gi[0].zoom) !== kt && (kt = kn(Gi[0].zoom), ut = void 0, Sn = {}), Ii = Ii.concat(ao({ key: xa + "[0]", value: Gi[0], valueSpec: { zoom: {} }, style: ni.style, styleSpec: ni.styleSpec, objectElementValidators: { zoom: To, value: ci } }));
                    } else Ii = Ii.concat(ci({ key: xa + "[0]", value: Gi[0], valueSpec: {}, style: ni.style, styleSpec: ni.styleSpec }, Gi));
                    return ta(er(Gi[1])) ? Ii.concat([new Mn(xa + "[1]", Gi[1], "expressions are not allowed in function stops.")]) : Ii.concat(ic({ key: xa + "[1]", value: Gi[1], valueSpec: Jt, style: ni.style, styleSpec: ni.styleSpec }));
                  }
                  function ci(ni, Ii) {
                    var Gi = ro(ni.value), xa = kn(ni.value), ga = ni.value !== null ? ni.value : Ii;
                    if (je) {
                      if (Gi !== je) return [new Mn(ni.key, ga, Gi + " stop domain type must match previous stop domain type " + je)];
                    } else je = Gi;
                    if (Gi !== "number" && Gi !== "string" && Gi !== "boolean") return [new Mn(ni.key, ga, "stop domain value must be a number, string, or boolean")];
                    if (Gi !== "number" && fn !== "categorical") {
                      var $a = "number expected, " + Gi + " found";
                      return So(Jt) && fn === void 0 && ($a += '\nIf you intended to use a categorical function, specify `"type": "categorical"`.'), [new Mn(ni.key, ga, $a)];
                    }
                    return fn !== "categorical" || Gi !== "number" || isFinite(xa) && Math.floor(xa) === xa ? fn !== "categorical" && Gi === "number" && ut !== void 0 && xa < ut ? [new Mn(ni.key, ga, "stop domain values must appear in ascending order")] : (ut = xa, fn === "categorical" && xa in Sn ? [new Mn(ni.key, ga, "stop domain values must be unique")] : (Sn[xa] = !0, [])) : [new Mn(ni.key, ga, "integer expected, found " + xa)];
                  }
                }
                function po(Ne) {
                  var je = (Ne.expressionContext === "property" ? Ga : da)(er(Ne.value), Ne.valueSpec);
                  if (je.result === "error") return je.value.map(function(kt) {
                    return new Mn("" + Ne.key + kt.key, Ne.value, kt.message);
                  });
                  var ut = je.value.expression || je.value._styleExpression.expression;
                  if (Ne.expressionContext === "property" && Ne.propertyKey === "text-font" && !ut.outputDefined()) return [new Mn(Ne.key, Ne.value, 'Invalid data expression for "' + Ne.propertyKey + '". Output values must be contained as literals within the expression.')];
                  if (Ne.expressionContext === "property" && Ne.propertyType === "layout" && !Mi(ut)) return [new Mn(Ne.key, Ne.value, '"feature-state" data expressions are not supported with layout properties.')];
                  if (Ne.expressionContext === "filter" && !Mi(ut)) return [new Mn(Ne.key, Ne.value, '"feature-state" data expressions are not supported with filters.')];
                  if (Ne.expressionContext && Ne.expressionContext.indexOf("cluster") === 0) {
                    if (!Vi(ut, ["zoom", "feature-state"])) return [new Mn(Ne.key, Ne.value, '"zoom" and "feature-state" expressions are not supported with cluster properties.')];
                    if (Ne.expressionContext === "cluster-initial" && !ki(ut)) return [new Mn(Ne.key, Ne.value, "Feature data expressions are not supported with initial expression part of cluster properties.")];
                  }
                  return [];
                }
                function Vo(Ne) {
                  var je = Ne.key, ut = Ne.value, kt = Ne.valueSpec, Jt = [];
                  return Array.isArray(kt.values) ? kt.values.indexOf(kn(ut)) === -1 && Jt.push(new Mn(je, ut, "expected one of [" + kt.values.join(", ") + "], " + JSON.stringify(ut) + " found")) : Object.keys(kt.values).indexOf(kn(ut)) === -1 && Jt.push(new Mn(je, ut, "expected one of [" + Object.keys(kt.values).join(", ") + "], " + JSON.stringify(ut) + " found")), Jt;
                }
                function Xo(Ne) {
                  if (Ne === !0 || Ne === !1) return !0;
                  if (!Array.isArray(Ne) || Ne.length === 0) return !1;
                  switch (Ne[0]) {
                    case "has":
                      return Ne.length >= 2 && Ne[1] !== "$id" && Ne[1] !== "$type";
                    case "in":
                      return Ne.length >= 3 && (typeof Ne[1] != "string" || Array.isArray(Ne[2]));
                    case "!in":
                    case "!has":
                    case "none":
                      return !1;
                    case "==":
                    case "!=":
                    case ">":
                    case ">=":
                    case "<":
                    case "<=":
                      return Ne.length !== 3 || Array.isArray(Ne[1]) || Array.isArray(Ne[2]);
                    case "any":
                    case "all":
                      for (var je = 0, ut = Ne.slice(1); je < ut.length; je += 1) {
                        var kt = ut[je];
                        if (!Xo(kt) && typeof kt != "boolean") return !1;
                      }
                      return !0;
                    default:
                      return !0;
                  }
                }
                Ha.deserialize = function(Ne) {
                  return new Ha(Ne._parameters, Ne._specification);
                }, Ha.serialize = function(Ne) {
                  return { _parameters: Ne._parameters, _specification: Ne._specification };
                };
                var Go = { type: "boolean", default: !1, transition: !1, "property-type": "data-driven", expression: { interpolated: !1, parameters: ["zoom", "feature"] } };
                function es(Ne) {
                  if (Ne == null) return { filter: function() {
                    return !0;
                  }, needGeometry: !1 };
                  Xo(Ne) || (Ne = Oo(Ne));
                  var je = da(Ne, Go);
                  if (je.result === "error") throw new Error(je.value.map(function(ut) {
                    return ut.key + ": " + ut.message;
                  }).join(", "));
                  return { filter: function(ut, kt, Jt) {
                    return je.value.evaluate(ut, kt, {}, Jt);
                  }, needGeometry: Fo(Ne) };
                }
                function Ts(Ne, je) {
                  return Ne < je ? -1 : Ne > je ? 1 : 0;
                }
                function Fo(Ne) {
                  if (!Array.isArray(Ne)) return !1;
                  if (Ne[0] === "within") return !0;
                  for (var je = 1; je < Ne.length; je++) if (Fo(Ne[je])) return !0;
                  return !1;
                }
                function Oo(Ne) {
                  if (!Ne) return !0;
                  var je, ut = Ne[0];
                  return Ne.length <= 1 ? ut !== "any" : ut === "==" ? gs(Ne[1], Ne[2], "==") : ut === "!=" ? dl(gs(Ne[1], Ne[2], "==")) : ut === "<" || ut === ">" || ut === "<=" || ut === ">=" ? gs(Ne[1], Ne[2], ut) : ut === "any" ? (je = Ne.slice(1), ["any"].concat(je.map(Oo))) : ut === "all" ? ["all"].concat(Ne.slice(1).map(Oo)) : ut === "none" ? ["all"].concat(Ne.slice(1).map(Oo).map(dl)) : ut === "in" ? Bu(Ne[1], Ne.slice(2)) : ut === "!in" ? dl(Bu(Ne[1], Ne.slice(2))) : ut === "has" ? Ql(Ne[1]) : ut === "!has" ? dl(Ql(Ne[1])) : ut !== "within" || Ne;
                }
                function gs(Ne, je, ut) {
                  switch (Ne) {
                    case "$type":
                      return ["filter-type-" + ut, je];
                    case "$id":
                      return ["filter-id-" + ut, je];
                    default:
                      return ["filter-" + ut, Ne, je];
                  }
                }
                function Bu(Ne, je) {
                  if (je.length === 0) return !1;
                  switch (Ne) {
                    case "$type":
                      return ["filter-type-in", ["literal", je]];
                    case "$id":
                      return ["filter-id-in", ["literal", je]];
                    default:
                      return je.length > 200 && !je.some(function(ut) {
                        return typeof ut != typeof je[0];
                      }) ? ["filter-in-large", Ne, ["literal", je.sort(Ts)]] : ["filter-in-small", Ne, ["literal", je]];
                  }
                }
                function Ql(Ne) {
                  switch (Ne) {
                    case "$type":
                      return !0;
                    case "$id":
                      return ["filter-has-id"];
                    default:
                      return ["filter-has", Ne];
                  }
                }
                function dl(Ne) {
                  return ["!", Ne];
                }
                function h1(Ne) {
                  return Xo(er(Ne.value)) ? po(In({}, Ne, { expressionContext: "filter", valueSpec: { value: "boolean" } })) : z1(Ne);
                }
                function z1(Ne) {
                  var je = Ne.value, ut = Ne.key;
                  if (ro(je) !== "array") return [new Mn(ut, je, "array expected, " + ro(je) + " found")];
                  var kt, Jt = Ne.styleSpec, fn = [];
                  if (je.length < 1) return [new Mn(ut, je, "filter array must have at least 1 element")];
                  switch (fn = fn.concat(Vo({ key: ut + "[0]", value: je[0], valueSpec: Jt.filter_operator, style: Ne.style, styleSpec: Ne.styleSpec })), kn(je[0])) {
                    case "<":
                    case "<=":
                    case ">":
                    case ">=":
                      je.length >= 2 && kn(je[1]) === "$type" && fn.push(new Mn(ut, je, '"$type" cannot be use with operator "' + je[0] + '"'));
                    case "==":
                    case "!=":
                      je.length !== 3 && fn.push(new Mn(ut, je, 'filter array for operator "' + je[0] + '" must have 3 elements'));
                    case "in":
                    case "!in":
                      je.length >= 2 && (kt = ro(je[1])) !== "string" && fn.push(new Mn(ut + "[1]", je[1], "string expected, " + kt + " found"));
                      for (var Sn = 2; Sn < je.length; Sn++) kt = ro(je[Sn]), kn(je[1]) === "$type" ? fn = fn.concat(Vo({ key: ut + "[" + Sn + "]", value: je[Sn], valueSpec: Jt.geometry_type, style: Ne.style, styleSpec: Ne.styleSpec })) : kt !== "string" && kt !== "number" && kt !== "boolean" && fn.push(new Mn(ut + "[" + Sn + "]", je[Sn], "string, number, or boolean expected, " + kt + " found"));
                      break;
                    case "any":
                    case "all":
                    case "none":
                      for (var Hn = 1; Hn < je.length; Hn++) fn = fn.concat(z1({ key: ut + "[" + Hn + "]", value: je[Hn], style: Ne.style, styleSpec: Ne.styleSpec }));
                      break;
                    case "has":
                    case "!has":
                      kt = ro(je[1]), je.length !== 2 ? fn.push(new Mn(ut, je, 'filter array for "' + je[0] + '" operator must have 2 elements')) : kt !== "string" && fn.push(new Mn(ut + "[1]", je[1], "string expected, " + kt + " found"));
                      break;
                    case "within":
                      kt = ro(je[1]), je.length !== 2 ? fn.push(new Mn(ut, je, 'filter array for "' + je[0] + '" operator must have 2 elements')) : kt !== "object" && fn.push(new Mn(ut + "[1]", je[1], "object expected, " + kt + " found"));
                  }
                  return fn;
                }
                function J1(Ne, je) {
                  var ut = Ne.key, kt = Ne.style, Jt = Ne.styleSpec, fn = Ne.value, Sn = Ne.objectKey, Hn = Jt[je + "_" + Ne.layerType];
                  if (!Hn) return [];
                  var nr = Sn.match(/^(.*)-transition$/);
                  if (je === "paint" && nr && Hn[nr[1]] && Hn[nr[1]].transition) return ic({ key: ut, value: fn, valueSpec: Jt.transition, style: kt, styleSpec: Jt });
                  var kr, Ir = Ne.valueSpec || Hn[Sn];
                  if (!Ir) return [new Mn(ut, fn, 'unknown property "' + Sn + '"')];
                  if (ro(fn) === "string" && So(Ir) && !Ir.tokens && (kr = /^{([^}]+)}$/.exec(fn))) return [new Mn(ut, fn, '"' + Sn + '" does not support interpolation syntax\nUse an identity property function instead: `{ "type": "identity", "property": ' + JSON.stringify(kr[1]) + " }`.")];
                  var qr = [];
                  return Ne.layerType === "symbol" && (Sn === "text-field" && kt && !kt.glyphs && qr.push(new Mn(ut, fn, 'use of "text-field" requires a style "glyphs" property')), Sn === "text-font" && Io(er(fn)) && kn(fn.type) === "identity" && qr.push(new Mn(ut, fn, '"text-font" does not support identity functions'))), qr.concat(ic({ key: Ne.key, value: fn, valueSpec: Ir, style: kt, styleSpec: Jt, expressionContext: "property", propertyType: je, propertyKey: Sn }));
                }
                function Lu(Ne) {
                  return J1(Ne, "paint");
                }
                function M1(Ne) {
                  return J1(Ne, "layout");
                }
                function ku(Ne) {
                  var je = [], ut = Ne.value, kt = Ne.key, Jt = Ne.style, fn = Ne.styleSpec;
                  ut.type || ut.ref || je.push(new Mn(kt, ut, 'either "type" or "ref" is required'));
                  var Sn, Hn = kn(ut.type), nr = kn(ut.ref);
                  if (ut.id) for (var kr = kn(ut.id), Ir = 0; Ir < Ne.arrayIndex; Ir++) {
                    var qr = Jt.layers[Ir];
                    kn(qr.id) === kr && je.push(new Mn(kt, ut.id, 'duplicate layer id "' + ut.id + '", previously used at line ' + qr.id.__line__));
                  }
                  if ("ref" in ut) ["type", "source", "source-layer", "filter", "layout"].forEach(function(Ii) {
                    Ii in ut && je.push(new Mn(kt, ut[Ii], '"' + Ii + '" is prohibited for ref layers'));
                  }), Jt.layers.forEach(function(Ii) {
                    kn(Ii.id) === nr && (Sn = Ii);
                  }), Sn ? Sn.ref ? je.push(new Mn(kt, ut.ref, "ref cannot reference another ref layer")) : Hn = kn(Sn.type) : je.push(new Mn(kt, ut.ref, 'ref layer "' + nr + '" not found'));
                  else if (Hn !== "background") if (ut.source) {
                    var ci = Jt.sources && Jt.sources[ut.source], ni = ci && kn(ci.type);
                    ci ? ni === "vector" && Hn === "raster" ? je.push(new Mn(kt, ut.source, 'layer "' + ut.id + '" requires a raster source')) : ni === "raster" && Hn !== "raster" ? je.push(new Mn(kt, ut.source, 'layer "' + ut.id + '" requires a vector source')) : ni !== "vector" || ut["source-layer"] ? ni === "raster-dem" && Hn !== "hillshade" ? je.push(new Mn(kt, ut.source, "raster-dem source can only be used with layer type 'hillshade'.")) : Hn !== "line" || !ut.paint || !ut.paint["line-gradient"] || ni === "geojson" && ci.lineMetrics || je.push(new Mn(kt, ut, 'layer "' + ut.id + '" specifies a line-gradient, which requires a GeoJSON source with `lineMetrics` enabled.')) : je.push(new Mn(kt, ut, 'layer "' + ut.id + '" must specify a "source-layer"')) : je.push(new Mn(kt, ut.source, 'source "' + ut.source + '" not found'));
                  } else je.push(new Mn(kt, ut, 'missing required property "source"'));
                  return je = je.concat(ao({ key: kt, value: ut, valueSpec: fn.layer, style: Ne.style, styleSpec: Ne.styleSpec, objectElementValidators: { "*": function() {
                    return [];
                  }, type: function() {
                    return ic({ key: kt + ".type", value: ut.type, valueSpec: fn.layer.type, style: Ne.style, styleSpec: Ne.styleSpec, object: ut, objectKey: "type" });
                  }, filter: h1, layout: function(Ii) {
                    return ao({ layer: ut, key: Ii.key, value: Ii.value, style: Ii.style, styleSpec: Ii.styleSpec, objectElementValidators: { "*": function(Gi) {
                      return M1(In({ layerType: Hn }, Gi));
                    } } });
                  }, paint: function(Ii) {
                    return ao({ layer: ut, key: Ii.key, value: Ii.value, style: Ii.style, styleSpec: Ii.styleSpec, objectElementValidators: { "*": function(Gi) {
                      return Lu(In({ layerType: Hn }, Gi));
                    } } });
                  } } })), je;
                }
                function Wu(Ne) {
                  var je = Ne.value, ut = Ne.key, kt = ro(je);
                  return kt !== "string" ? [new Mn(ut, je, "string expected, " + kt + " found")] : [];
                }
                var T1 = { promoteId: function(Ne) {
                  var je = Ne.key, ut = Ne.value;
                  if (ro(ut) === "string") return Wu({ key: je, value: ut });
                  var kt = [];
                  for (var Jt in ut) kt.push.apply(kt, Wu({ key: je + "." + Jt, value: ut[Jt] }));
                  return kt;
                } };
                function kl(Ne) {
                  var je = Ne.value, ut = Ne.key, kt = Ne.styleSpec, Jt = Ne.style;
                  if (!je.type) return [new Mn(ut, je, '"type" is required')];
                  var fn, Sn = kn(je.type);
                  switch (Sn) {
                    case "vector":
                    case "raster":
                    case "raster-dem":
                      return ao({ key: ut, value: je, valueSpec: kt["source_" + Sn.replace("-", "_")], style: Ne.style, styleSpec: kt, objectElementValidators: T1 });
                    case "geojson":
                      if (fn = ao({ key: ut, value: je, valueSpec: kt.source_geojson, style: Jt, styleSpec: kt, objectElementValidators: T1 }), je.cluster) for (var Hn in je.clusterProperties) {
                        var nr = je.clusterProperties[Hn], kr = nr[0], Ir = nr[1], qr = typeof kr == "string" ? [kr, ["accumulated"], ["get", Hn]] : kr;
                        fn.push.apply(fn, po({ key: ut + "." + Hn + ".map", value: Ir, expressionContext: "cluster-map" })), fn.push.apply(fn, po({ key: ut + "." + Hn + ".reduce", value: qr, expressionContext: "cluster-reduce" }));
                      }
                      return fn;
                    case "video":
                      return ao({ key: ut, value: je, valueSpec: kt.source_video, style: Jt, styleSpec: kt });
                    case "image":
                      return ao({ key: ut, value: je, valueSpec: kt.source_image, style: Jt, styleSpec: kt });
                    case "canvas":
                      return [new Mn(ut, null, "Please use runtime APIs to add canvas sources, rather than including them in stylesheets.", "source.canvas")];
                    default:
                      return Vo({ key: ut + ".type", value: je.type, valueSpec: { values: ["vector", "raster", "raster-dem", "geojson", "video", "image"] }, style: Jt, styleSpec: kt });
                  }
                }
                function Yl(Ne) {
                  var je = Ne.value, ut = Ne.styleSpec, kt = ut.light, Jt = Ne.style, fn = [], Sn = ro(je);
                  if (je === void 0) return fn;
                  if (Sn !== "object") return fn.concat([new Mn("light", je, "object expected, " + Sn + " found")]);
                  for (var Hn in je) {
                    var nr = Hn.match(/^(.*)-transition$/);
                    fn = nr && kt[nr[1]] && kt[nr[1]].transition ? fn.concat(ic({ key: Hn, value: je[Hn], valueSpec: ut.transition, style: Jt, styleSpec: ut })) : kt[Hn] ? fn.concat(ic({ key: Hn, value: je[Hn], valueSpec: kt[Hn], style: Jt, styleSpec: ut })) : fn.concat([new Mn(Hn, je[Hn], 'unknown property "' + Hn + '"')]);
                  }
                  return fn;
                }
                var i1 = { "*": function() {
                  return [];
                }, array: io, boolean: function(Ne) {
                  var je = Ne.value, ut = Ne.key, kt = ro(je);
                  return kt !== "boolean" ? [new Mn(ut, je, "boolean expected, " + kt + " found")] : [];
                }, number: To, color: function(Ne) {
                  var je = Ne.key, ut = Ne.value, kt = ro(ut);
                  return kt !== "string" ? [new Mn(je, ut, "color expected, " + kt + " found")] : Br(ut) === null ? [new Mn(je, ut, 'color expected, "' + ut + '" found')] : [];
                }, constants: Ln, enum: Vo, filter: h1, function: Mo, layer: ku, object: ao, source: kl, light: Yl, string: Wu, formatted: function(Ne) {
                  return Wu(Ne).length === 0 ? [] : po(Ne);
                }, resolvedImage: function(Ne) {
                  return Wu(Ne).length === 0 ? [] : po(Ne);
                } };
                function ic(Ne) {
                  var je = Ne.value, ut = Ne.valueSpec, kt = Ne.styleSpec;
                  return ut.expression && Io(kn(je)) ? Mo(Ne) : ut.expression && ta(er(je)) ? po(Ne) : ut.type && i1[ut.type] ? i1[ut.type](Ne) : ao(In({}, Ne, { valueSpec: ut.type ? kt[ut.type] : ut }));
                }
                function iu(Ne) {
                  var je = Ne.value, ut = Ne.key, kt = Wu(Ne);
                  return kt.length || (je.indexOf("{fontstack}") === -1 && kt.push(new Mn(ut, je, '"glyphs" url must include a "{fontstack}" token')), je.indexOf("{range}") === -1 && kt.push(new Mn(ut, je, '"glyphs" url must include a "{range}" token'))), kt;
                }
                function xu(Ne, je) {
                  je === void 0 && (je = Zt);
                  var ut = [];
                  return ut = ut.concat(ic({ key: "", value: Ne, valueSpec: je.$root, styleSpec: je, style: Ne, objectElementValidators: { glyphs: iu, "*": function() {
                    return [];
                  } } })), Ne.constants && (ut = ut.concat(Ln({ key: "constants", value: Ne.constants, style: Ne, styleSpec: je }))), vc(ut);
                }
                function vc(Ne) {
                  return [].concat(Ne).sort(function(je, ut) {
                    return je.line - ut.line;
                  });
                }
                function Zl(Ne) {
                  return function() {
                    for (var je = [], ut = arguments.length; ut--; ) je[ut] = arguments[ut];
                    return vc(Ne.apply(this, je));
                  };
                }
                xu.source = Zl(kl), xu.light = Zl(Yl), xu.layer = Zl(ku), xu.filter = Zl(h1), xu.paintProperty = Zl(Lu), xu.layoutProperty = Zl(M1);
                var Ec = xu, wu = Ec.light, Pc = Ec.paintProperty, du = Ec.layoutProperty;
                function Ac(Ne, je) {
                  var ut = !1;
                  if (je && je.length) for (var kt = 0, Jt = je; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    Ne.fire(new bn(new Error(fn.message))), ut = !0;
                  }
                  return ut;
                }
                var yu = Qc, au = 3;
                function Qc(Ne, je, ut) {
                  var kt = this.cells = [];
                  if (Ne instanceof ArrayBuffer) {
                    this.arrayBuffer = Ne;
                    var Jt = new Int32Array(this.arrayBuffer);
                    Ne = Jt[0], je = Jt[1], ut = Jt[2], this.d = je + 2 * ut;
                    for (var fn = 0; fn < this.d * this.d; fn++) {
                      var Sn = Jt[au + fn], Hn = Jt[au + fn + 1];
                      kt.push(Sn === Hn ? null : Jt.subarray(Sn, Hn));
                    }
                    var nr = Jt[au + kt.length], kr = Jt[au + kt.length + 1];
                    this.keys = Jt.subarray(nr, kr), this.bboxes = Jt.subarray(kr), this.insert = this._insertReadonly;
                  } else {
                    this.d = je + 2 * ut;
                    for (var Ir = 0; Ir < this.d * this.d; Ir++) kt.push([]);
                    this.keys = [], this.bboxes = [];
                  }
                  this.n = je, this.extent = Ne, this.padding = ut, this.scale = je / Ne, this.uid = 0;
                  var qr = ut / je * Ne;
                  this.min = -qr, this.max = Ne + qr;
                }
                Qc.prototype.insert = function(Ne, je, ut, kt, Jt) {
                  this._forEachCell(je, ut, kt, Jt, this._insertCell, this.uid++), this.keys.push(Ne), this.bboxes.push(je), this.bboxes.push(ut), this.bboxes.push(kt), this.bboxes.push(Jt);
                }, Qc.prototype._insertReadonly = function() {
                  throw "Cannot insert into a GridIndex created from an ArrayBuffer.";
                }, Qc.prototype._insertCell = function(Ne, je, ut, kt, Jt, fn) {
                  this.cells[Jt].push(fn);
                }, Qc.prototype.query = function(Ne, je, ut, kt, Jt) {
                  var fn = this.min, Sn = this.max;
                  if (Ne <= fn && je <= fn && Sn <= ut && Sn <= kt && !Jt) return Array.prototype.slice.call(this.keys);
                  var Hn = [];
                  return this._forEachCell(Ne, je, ut, kt, this._queryCell, Hn, {}, Jt), Hn;
                }, Qc.prototype._queryCell = function(Ne, je, ut, kt, Jt, fn, Sn, Hn) {
                  var nr = this.cells[Jt];
                  if (nr !== null) for (var kr = this.keys, Ir = this.bboxes, qr = 0; qr < nr.length; qr++) {
                    var ci = nr[qr];
                    if (Sn[ci] === void 0) {
                      var ni = 4 * ci;
                      (Hn ? Hn(Ir[ni + 0], Ir[ni + 1], Ir[ni + 2], Ir[ni + 3]) : Ne <= Ir[ni + 2] && je <= Ir[ni + 3] && ut >= Ir[ni + 0] && kt >= Ir[ni + 1]) ? (Sn[ci] = !0, fn.push(kr[ci])) : Sn[ci] = !1;
                    }
                  }
                }, Qc.prototype._forEachCell = function(Ne, je, ut, kt, Jt, fn, Sn, Hn) {
                  for (var nr = this._convertToCellCoord(Ne), kr = this._convertToCellCoord(je), Ir = this._convertToCellCoord(ut), qr = this._convertToCellCoord(kt), ci = nr; ci <= Ir; ci++) for (var ni = kr; ni <= qr; ni++) {
                    var Ii = this.d * ni + ci;
                    if ((!Hn || Hn(this._convertFromCellCoord(ci), this._convertFromCellCoord(ni), this._convertFromCellCoord(ci + 1), this._convertFromCellCoord(ni + 1))) && Jt.call(this, Ne, je, ut, kt, Ii, fn, Sn, Hn)) return;
                  }
                }, Qc.prototype._convertFromCellCoord = function(Ne) {
                  return (Ne - this.padding) / this.scale;
                }, Qc.prototype._convertToCellCoord = function(Ne) {
                  return Math.max(0, Math.min(this.d - 1, Math.floor(Ne * this.scale) + this.padding));
                }, Qc.prototype.toArrayBuffer = function() {
                  if (this.arrayBuffer) return this.arrayBuffer;
                  for (var Ne = this.cells, je = au + this.cells.length + 1 + 1, ut = 0, kt = 0; kt < this.cells.length; kt++) ut += this.cells[kt].length;
                  var Jt = new Int32Array(je + ut + this.keys.length + this.bboxes.length);
                  Jt[0] = this.extent, Jt[1] = this.n, Jt[2] = this.padding;
                  for (var fn = je, Sn = 0; Sn < Ne.length; Sn++) {
                    var Hn = Ne[Sn];
                    Jt[au + Sn] = fn, Jt.set(Hn, fn), fn += Hn.length;
                  }
                  return Jt[au + Ne.length] = fn, Jt.set(this.keys, fn), fn += this.keys.length, Jt[au + Ne.length + 1] = fn, Jt.set(this.bboxes, fn), fn += this.bboxes.length, Jt.buffer;
                };
                var Cc = de.ImageData, Ol = de.ImageBitmap, uc = {};
                function cs(Ne, je, ut) {
                  ut === void 0 && (ut = {}), Object.defineProperty(je, "_classRegistryKey", { value: Ne, writeable: !1 }), uc[Ne] = { klass: je, omit: ut.omit || [], shallow: ut.shallow || [] };
                }
                for (var Mu in cs("Object", Object), yu.serialize = function(Ne, je) {
                  var ut = Ne.toArrayBuffer();
                  return je && je.push(ut), { buffer: ut };
                }, yu.deserialize = function(Ne) {
                  return new yu(Ne.buffer);
                }, cs("Grid", yu), cs("Color", Pr), cs("Error", Error), cs("ResolvedImage", ka), cs("StylePropertyFunction", Ha), cs("StyleExpression", Ri, { omit: ["_evaluator"] }), cs("ZoomDependentExpression", Pa), cs("ZoomConstantExpression", Ji), cs("CompoundExpression", Ur, { omit: ["_evaluate"] }), b1) b1[Mu]._classRegistryKey || cs("Expression_" + Mu, b1[Mu]);
                function Hu(Ne) {
                  return Ne && typeof ArrayBuffer < "u" && (Ne instanceof ArrayBuffer || Ne.constructor && Ne.constructor.name === "ArrayBuffer");
                }
                function ou(Ne) {
                  return Ol && Ne instanceof Ol;
                }
                function Gc(Ne, je) {
                  if (Ne == null || typeof Ne == "boolean" || typeof Ne == "number" || typeof Ne == "string" || Ne instanceof Boolean || Ne instanceof Number || Ne instanceof String || Ne instanceof Date || Ne instanceof RegExp) return Ne;
                  if (Hu(Ne) || ou(Ne)) return je && je.push(Ne), Ne;
                  if (ArrayBuffer.isView(Ne)) {
                    var ut = Ne;
                    return je && je.push(ut.buffer), ut;
                  }
                  if (Ne instanceof Cc) return je && je.push(Ne.data.buffer), Ne;
                  if (Array.isArray(Ne)) {
                    for (var kt = [], Jt = 0, fn = Ne; Jt < fn.length; Jt += 1) {
                      var Sn = fn[Jt];
                      kt.push(Gc(Sn, je));
                    }
                    return kt;
                  }
                  if (typeof Ne == "object") {
                    var Hn = Ne.constructor, nr = Hn._classRegistryKey;
                    if (!nr) throw new Error("can't serialize object of unregistered class");
                    var kr = Hn.serialize ? Hn.serialize(Ne, je) : {};
                    if (!Hn.serialize) {
                      for (var Ir in Ne) if (Ne.hasOwnProperty(Ir) && !(uc[nr].omit.indexOf(Ir) >= 0)) {
                        var qr = Ne[Ir];
                        kr[Ir] = uc[nr].shallow.indexOf(Ir) >= 0 ? qr : Gc(qr, je);
                      }
                      Ne instanceof Error && (kr.message = Ne.message);
                    }
                    if (kr.$name) throw new Error("$name property is reserved for worker serialization logic.");
                    return nr !== "Object" && (kr.$name = nr), kr;
                  }
                  throw new Error("can't serialize object of type " + typeof Ne);
                }
                function Su(Ne) {
                  if (Ne == null || typeof Ne == "boolean" || typeof Ne == "number" || typeof Ne == "string" || Ne instanceof Boolean || Ne instanceof Number || Ne instanceof String || Ne instanceof Date || Ne instanceof RegExp || Hu(Ne) || ou(Ne) || ArrayBuffer.isView(Ne) || Ne instanceof Cc) return Ne;
                  if (Array.isArray(Ne)) return Ne.map(Su);
                  if (typeof Ne == "object") {
                    var je = Ne.$name || "Object", ut = uc[je].klass;
                    if (!ut) throw new Error("can't deserialize unregistered class " + je);
                    if (ut.deserialize) return ut.deserialize(Ne);
                    for (var kt = Object.create(ut.prototype), Jt = 0, fn = Object.keys(Ne); Jt < fn.length; Jt += 1) {
                      var Sn = fn[Jt];
                      if (Sn !== "$name") {
                        var Hn = Ne[Sn];
                        kt[Sn] = uc[je].shallow.indexOf(Sn) >= 0 ? Hn : Su(Hn);
                      }
                    }
                    return kt;
                  }
                  throw new Error("can't deserialize object of type " + typeof Ne);
                }
                var Lc = function() {
                  this.first = !0;
                };
                Lc.prototype.update = function(Ne, je) {
                  var ut = Math.floor(Ne);
                  return this.first ? (this.first = !1, this.lastIntegerZoom = ut, this.lastIntegerZoomTime = 0, this.lastZoom = Ne, this.lastFloorZoom = ut, !0) : (this.lastFloorZoom > ut ? (this.lastIntegerZoom = ut + 1, this.lastIntegerZoomTime = je) : this.lastFloorZoom < ut && (this.lastIntegerZoom = ut, this.lastIntegerZoomTime = je), Ne !== this.lastZoom && (this.lastZoom = Ne, this.lastFloorZoom = ut, !0));
                };
                var Do = { "Latin-1 Supplement": function(Ne) {
                  return Ne >= 128 && Ne <= 255;
                }, Arabic: function(Ne) {
                  return Ne >= 1536 && Ne <= 1791;
                }, "Arabic Supplement": function(Ne) {
                  return Ne >= 1872 && Ne <= 1919;
                }, "Arabic Extended-A": function(Ne) {
                  return Ne >= 2208 && Ne <= 2303;
                }, "Hangul Jamo": function(Ne) {
                  return Ne >= 4352 && Ne <= 4607;
                }, "Unified Canadian Aboriginal Syllabics": function(Ne) {
                  return Ne >= 5120 && Ne <= 5759;
                }, Khmer: function(Ne) {
                  return Ne >= 6016 && Ne <= 6143;
                }, "Unified Canadian Aboriginal Syllabics Extended": function(Ne) {
                  return Ne >= 6320 && Ne <= 6399;
                }, "General Punctuation": function(Ne) {
                  return Ne >= 8192 && Ne <= 8303;
                }, "Letterlike Symbols": function(Ne) {
                  return Ne >= 8448 && Ne <= 8527;
                }, "Number Forms": function(Ne) {
                  return Ne >= 8528 && Ne <= 8591;
                }, "Miscellaneous Technical": function(Ne) {
                  return Ne >= 8960 && Ne <= 9215;
                }, "Control Pictures": function(Ne) {
                  return Ne >= 9216 && Ne <= 9279;
                }, "Optical Character Recognition": function(Ne) {
                  return Ne >= 9280 && Ne <= 9311;
                }, "Enclosed Alphanumerics": function(Ne) {
                  return Ne >= 9312 && Ne <= 9471;
                }, "Geometric Shapes": function(Ne) {
                  return Ne >= 9632 && Ne <= 9727;
                }, "Miscellaneous Symbols": function(Ne) {
                  return Ne >= 9728 && Ne <= 9983;
                }, "Miscellaneous Symbols and Arrows": function(Ne) {
                  return Ne >= 11008 && Ne <= 11263;
                }, "CJK Radicals Supplement": function(Ne) {
                  return Ne >= 11904 && Ne <= 12031;
                }, "Kangxi Radicals": function(Ne) {
                  return Ne >= 12032 && Ne <= 12255;
                }, "Ideographic Description Characters": function(Ne) {
                  return Ne >= 12272 && Ne <= 12287;
                }, "CJK Symbols and Punctuation": function(Ne) {
                  return Ne >= 12288 && Ne <= 12351;
                }, Hiragana: function(Ne) {
                  return Ne >= 12352 && Ne <= 12447;
                }, Katakana: function(Ne) {
                  return Ne >= 12448 && Ne <= 12543;
                }, Bopomofo: function(Ne) {
                  return Ne >= 12544 && Ne <= 12591;
                }, "Hangul Compatibility Jamo": function(Ne) {
                  return Ne >= 12592 && Ne <= 12687;
                }, Kanbun: function(Ne) {
                  return Ne >= 12688 && Ne <= 12703;
                }, "Bopomofo Extended": function(Ne) {
                  return Ne >= 12704 && Ne <= 12735;
                }, "CJK Strokes": function(Ne) {
                  return Ne >= 12736 && Ne <= 12783;
                }, "Katakana Phonetic Extensions": function(Ne) {
                  return Ne >= 12784 && Ne <= 12799;
                }, "Enclosed CJK Letters and Months": function(Ne) {
                  return Ne >= 12800 && Ne <= 13055;
                }, "CJK Compatibility": function(Ne) {
                  return Ne >= 13056 && Ne <= 13311;
                }, "CJK Unified Ideographs Extension A": function(Ne) {
                  return Ne >= 13312 && Ne <= 19903;
                }, "Yijing Hexagram Symbols": function(Ne) {
                  return Ne >= 19904 && Ne <= 19967;
                }, "CJK Unified Ideographs": function(Ne) {
                  return Ne >= 19968 && Ne <= 40959;
                }, "Yi Syllables": function(Ne) {
                  return Ne >= 40960 && Ne <= 42127;
                }, "Yi Radicals": function(Ne) {
                  return Ne >= 42128 && Ne <= 42191;
                }, "Hangul Jamo Extended-A": function(Ne) {
                  return Ne >= 43360 && Ne <= 43391;
                }, "Hangul Syllables": function(Ne) {
                  return Ne >= 44032 && Ne <= 55215;
                }, "Hangul Jamo Extended-B": function(Ne) {
                  return Ne >= 55216 && Ne <= 55295;
                }, "Private Use Area": function(Ne) {
                  return Ne >= 57344 && Ne <= 63743;
                }, "CJK Compatibility Ideographs": function(Ne) {
                  return Ne >= 63744 && Ne <= 64255;
                }, "Arabic Presentation Forms-A": function(Ne) {
                  return Ne >= 64336 && Ne <= 65023;
                }, "Vertical Forms": function(Ne) {
                  return Ne >= 65040 && Ne <= 65055;
                }, "CJK Compatibility Forms": function(Ne) {
                  return Ne >= 65072 && Ne <= 65103;
                }, "Small Form Variants": function(Ne) {
                  return Ne >= 65104 && Ne <= 65135;
                }, "Arabic Presentation Forms-B": function(Ne) {
                  return Ne >= 65136 && Ne <= 65279;
                }, "Halfwidth and Fullwidth Forms": function(Ne) {
                  return Ne >= 65280 && Ne <= 65519;
                } };
                function Ru(Ne) {
                  for (var je = 0, ut = Ne; je < ut.length; je += 1) if (dc(ut[je].charCodeAt(0))) return !0;
                  return !1;
                }
                function dc(Ne) {
                  return !(Ne !== 746 && Ne !== 747 && (Ne < 4352 || !(Do["Bopomofo Extended"](Ne) || Do.Bopomofo(Ne) || Do["CJK Compatibility Forms"](Ne) && !(Ne >= 65097 && Ne <= 65103) || Do["CJK Compatibility Ideographs"](Ne) || Do["CJK Compatibility"](Ne) || Do["CJK Radicals Supplement"](Ne) || Do["CJK Strokes"](Ne) || !(!Do["CJK Symbols and Punctuation"](Ne) || Ne >= 12296 && Ne <= 12305 || Ne >= 12308 && Ne <= 12319 || Ne === 12336) || Do["CJK Unified Ideographs Extension A"](Ne) || Do["CJK Unified Ideographs"](Ne) || Do["Enclosed CJK Letters and Months"](Ne) || Do["Hangul Compatibility Jamo"](Ne) || Do["Hangul Jamo Extended-A"](Ne) || Do["Hangul Jamo Extended-B"](Ne) || Do["Hangul Jamo"](Ne) || Do["Hangul Syllables"](Ne) || Do.Hiragana(Ne) || Do["Ideographic Description Characters"](Ne) || Do.Kanbun(Ne) || Do["Kangxi Radicals"](Ne) || Do["Katakana Phonetic Extensions"](Ne) || Do.Katakana(Ne) && Ne !== 12540 || !(!Do["Halfwidth and Fullwidth Forms"](Ne) || Ne === 65288 || Ne === 65289 || Ne === 65293 || Ne >= 65306 && Ne <= 65310 || Ne === 65339 || Ne === 65341 || Ne === 65343 || Ne >= 65371 && Ne <= 65503 || Ne === 65507 || Ne >= 65512 && Ne <= 65519) || !(!Do["Small Form Variants"](Ne) || Ne >= 65112 && Ne <= 65118 || Ne >= 65123 && Ne <= 65126) || Do["Unified Canadian Aboriginal Syllabics"](Ne) || Do["Unified Canadian Aboriginal Syllabics Extended"](Ne) || Do["Vertical Forms"](Ne) || Do["Yijing Hexagram Symbols"](Ne) || Do["Yi Syllables"](Ne) || Do["Yi Radicals"](Ne))));
                }
                function vl(Ne) {
                  return !(dc(Ne) || function(je) {
                    return !!(Do["Latin-1 Supplement"](je) && (je === 167 || je === 169 || je === 174 || je === 177 || je === 188 || je === 189 || je === 190 || je === 215 || je === 247) || Do["General Punctuation"](je) && (je === 8214 || je === 8224 || je === 8225 || je === 8240 || je === 8241 || je === 8251 || je === 8252 || je === 8258 || je === 8263 || je === 8264 || je === 8265 || je === 8273) || Do["Letterlike Symbols"](je) || Do["Number Forms"](je) || Do["Miscellaneous Technical"](je) && (je >= 8960 && je <= 8967 || je >= 8972 && je <= 8991 || je >= 8996 && je <= 9e3 || je === 9003 || je >= 9085 && je <= 9114 || je >= 9150 && je <= 9165 || je === 9167 || je >= 9169 && je <= 9179 || je >= 9186 && je <= 9215) || Do["Control Pictures"](je) && je !== 9251 || Do["Optical Character Recognition"](je) || Do["Enclosed Alphanumerics"](je) || Do["Geometric Shapes"](je) || Do["Miscellaneous Symbols"](je) && !(je >= 9754 && je <= 9759) || Do["Miscellaneous Symbols and Arrows"](je) && (je >= 11026 && je <= 11055 || je >= 11088 && je <= 11097 || je >= 11192 && je <= 11243) || Do["CJK Symbols and Punctuation"](je) || Do.Katakana(je) || Do["Private Use Area"](je) || Do["CJK Compatibility Forms"](je) || Do["Small Form Variants"](je) || Do["Halfwidth and Fullwidth Forms"](je) || je === 8734 || je === 8756 || je === 8757 || je >= 9984 && je <= 10087 || je >= 10102 && je <= 10131 || je === 65532 || je === 65533);
                  }(Ne));
                }
                function Eu(Ne) {
                  return Do.Arabic(Ne) || Do["Arabic Supplement"](Ne) || Do["Arabic Extended-A"](Ne) || Do["Arabic Presentation Forms-A"](Ne) || Do["Arabic Presentation Forms-B"](Ne);
                }
                function fu(Ne) {
                  return Ne >= 1424 && Ne <= 2303 || Do["Arabic Presentation Forms-A"](Ne) || Do["Arabic Presentation Forms-B"](Ne);
                }
                function ac(Ne, je) {
                  return !(!je && fu(Ne) || Ne >= 2304 && Ne <= 3583 || Ne >= 3840 && Ne <= 4255 || Do.Khmer(Ne));
                }
                function bc(Ne) {
                  for (var je = 0, ut = Ne; je < ut.length; je += 1) if (fu(ut[je].charCodeAt(0))) return !0;
                  return !1;
                }
                var kc = "deferred", fc = "loading", oc = "loaded", ss = "error", su = null, Tl = "unavailable", sc = null, hc = function(Ne) {
                  Ne && typeof Ne == "string" && Ne.indexOf("NetworkError") > -1 && (Tl = ss), su && su(Ne);
                };
                function Vc() {
                  Rc.fire(new gn("pluginStateChange", { pluginStatus: Tl, pluginURL: sc }));
                }
                var Rc = new an(), Zo = function() {
                  return Tl;
                }, Ou = function() {
                  if (Tl !== kc || !sc) throw new Error("rtl-text-plugin cannot be downloaded unless a pluginURL is specified");
                  Tl = fc, Vc(), sc && wt({ url: sc }, function(Ne) {
                    Ne ? hc(Ne) : (Tl = oc, Vc());
                  });
                }, Uc = { applyArabicShaping: null, processBidirectionalText: null, processStyledBidirectionalText: null, isLoaded: function() {
                  return Tl === oc || Uc.applyArabicShaping != null;
                }, isLoading: function() {
                  return Tl === fc;
                }, setState: function(Ne) {
                  Tl = Ne.pluginStatus, sc = Ne.pluginURL;
                }, isParsed: function() {
                  return Uc.applyArabicShaping != null && Uc.processBidirectionalText != null && Uc.processStyledBidirectionalText != null;
                }, getPluginURL: function() {
                  return sc;
                } }, El = function(Ne, je) {
                  this.zoom = Ne, je ? (this.now = je.now, this.fadeDuration = je.fadeDuration, this.zoomHistory = je.zoomHistory, this.transition = je.transition) : (this.now = 0, this.fadeDuration = 0, this.zoomHistory = new Lc(), this.transition = {});
                };
                El.prototype.isSupportedScript = function(Ne) {
                  return function(je, ut) {
                    for (var kt = 0, Jt = je; kt < Jt.length; kt += 1) if (!ac(Jt[kt].charCodeAt(0), ut)) return !1;
                    return !0;
                  }(Ne, Uc.isLoaded());
                }, El.prototype.crossFadingFactor = function() {
                  return this.fadeDuration === 0 ? 1 : Math.min((this.now - this.zoomHistory.lastIntegerZoomTime) / this.fadeDuration, 1);
                }, El.prototype.getCrossfadeParameters = function() {
                  var Ne = this.zoom, je = Ne - Math.floor(Ne), ut = this.crossFadingFactor();
                  return Ne > this.zoomHistory.lastIntegerZoom ? { fromScale: 2, toScale: 1, t: je + (1 - je) * ut } : { fromScale: 0.5, toScale: 1, t: 1 - (1 - ut) * je };
                };
                var eu = function(Ne, je) {
                  this.property = Ne, this.value = je, this.expression = function(ut, kt) {
                    if (Io(ut)) return new Ha(ut, kt);
                    if (ta(ut)) {
                      var Jt = Ga(ut, kt);
                      if (Jt.result === "error") throw new Error(Jt.value.map(function(Sn) {
                        return Sn.key + ": " + Sn.message;
                      }).join(", "));
                      return Jt.value;
                    }
                    var fn = ut;
                    return typeof ut == "string" && kt.type === "color" && (fn = Pr.parse(ut)), { kind: "constant", evaluate: function() {
                      return fn;
                    } };
                  }(je === void 0 ? Ne.specification.default : je, Ne.specification);
                };
                eu.prototype.isDataDriven = function() {
                  return this.expression.kind === "source" || this.expression.kind === "composite";
                }, eu.prototype.possiblyEvaluate = function(Ne, je, ut) {
                  return this.property.possiblyEvaluate(this, Ne, je, ut);
                };
                var Wc = function(Ne) {
                  this.property = Ne, this.value = new eu(Ne, void 0);
                };
                Wc.prototype.transitioned = function(Ne, je) {
                  return new gd(this.property, this.value, je, ge({}, Ne.transition, this.transition), Ne.now);
                }, Wc.prototype.untransitioned = function() {
                  return new gd(this.property, this.value, null, {}, 0);
                };
                var hu = function(Ne) {
                  this._properties = Ne, this._values = Object.create(Ne.defaultTransitionablePropertyValues);
                };
                hu.prototype.getValue = function(Ne) {
                  return Qe(this._values[Ne].value.value);
                }, hu.prototype.setValue = function(Ne, je) {
                  this._values.hasOwnProperty(Ne) || (this._values[Ne] = new Wc(this._values[Ne].property)), this._values[Ne].value = new eu(this._values[Ne].property, je === null ? void 0 : Qe(je));
                }, hu.prototype.getTransition = function(Ne) {
                  return Qe(this._values[Ne].transition);
                }, hu.prototype.setTransition = function(Ne, je) {
                  this._values.hasOwnProperty(Ne) || (this._values[Ne] = new Wc(this._values[Ne].property)), this._values[Ne].transition = Qe(je) || void 0;
                }, hu.prototype.serialize = function() {
                  for (var Ne = {}, je = 0, ut = Object.keys(this._values); je < ut.length; je += 1) {
                    var kt = ut[je], Jt = this.getValue(kt);
                    Jt !== void 0 && (Ne[kt] = Jt);
                    var fn = this.getTransition(kt);
                    fn !== void 0 && (Ne[kt + "-transition"] = fn);
                  }
                  return Ne;
                }, hu.prototype.transitioned = function(Ne, je) {
                  for (var ut = new Jl(this._properties), kt = 0, Jt = Object.keys(this._values); kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    ut._values[fn] = this._values[fn].transitioned(Ne, je._values[fn]);
                  }
                  return ut;
                }, hu.prototype.untransitioned = function() {
                  for (var Ne = new Jl(this._properties), je = 0, ut = Object.keys(this._values); je < ut.length; je += 1) {
                    var kt = ut[je];
                    Ne._values[kt] = this._values[kt].untransitioned();
                  }
                  return Ne;
                };
                var gd = function(Ne, je, ut, kt, Jt) {
                  this.property = Ne, this.value = je, this.begin = Jt + kt.delay || 0, this.end = this.begin + kt.duration || 0, Ne.specification.transition && (kt.delay || kt.duration) && (this.prior = ut);
                };
                gd.prototype.possiblyEvaluate = function(Ne, je, ut) {
                  var kt = Ne.now || 0, Jt = this.value.possiblyEvaluate(Ne, je, ut), fn = this.prior;
                  if (fn) {
                    if (kt > this.end) return this.prior = null, Jt;
                    if (this.value.isDataDriven()) return this.prior = null, Jt;
                    if (kt < this.begin) return fn.possiblyEvaluate(Ne, je, ut);
                    var Sn = (kt - this.begin) / (this.end - this.begin);
                    return this.property.interpolate(fn.possiblyEvaluate(Ne, je, ut), Jt, function(Hn) {
                      if (Hn <= 0) return 0;
                      if (Hn >= 1) return 1;
                      var nr = Hn * Hn, kr = nr * Hn;
                      return 4 * (Hn < 0.5 ? kr : 3 * (Hn - nr) + kr - 0.75);
                    }(Sn));
                  }
                  return Jt;
                };
                var Jl = function(Ne) {
                  this._properties = Ne, this._values = Object.create(Ne.defaultTransitioningPropertyValues);
                };
                Jl.prototype.possiblyEvaluate = function(Ne, je, ut) {
                  for (var kt = new Ko(this._properties), Jt = 0, fn = Object.keys(this._values); Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt];
                    kt._values[Sn] = this._values[Sn].possiblyEvaluate(Ne, je, ut);
                  }
                  return kt;
                }, Jl.prototype.hasTransition = function() {
                  for (var Ne = 0, je = Object.keys(this._values); Ne < je.length; Ne += 1) {
                    var ut = je[Ne];
                    if (this._values[ut].prior) return !0;
                  }
                  return !1;
                };
                var Yc = function(Ne) {
                  this._properties = Ne, this._values = Object.create(Ne.defaultPropertyValues);
                };
                Yc.prototype.getValue = function(Ne) {
                  return Qe(this._values[Ne].value);
                }, Yc.prototype.setValue = function(Ne, je) {
                  this._values[Ne] = new eu(this._values[Ne].property, je === null ? void 0 : Qe(je));
                }, Yc.prototype.serialize = function() {
                  for (var Ne = {}, je = 0, ut = Object.keys(this._values); je < ut.length; je += 1) {
                    var kt = ut[je], Jt = this.getValue(kt);
                    Jt !== void 0 && (Ne[kt] = Jt);
                  }
                  return Ne;
                }, Yc.prototype.possiblyEvaluate = function(Ne, je, ut) {
                  for (var kt = new Ko(this._properties), Jt = 0, fn = Object.keys(this._values); Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt];
                    kt._values[Sn] = this._values[Sn].possiblyEvaluate(Ne, je, ut);
                  }
                  return kt;
                };
                var nc = function(Ne, je, ut) {
                  this.property = Ne, this.value = je, this.parameters = ut;
                };
                nc.prototype.isConstant = function() {
                  return this.value.kind === "constant";
                }, nc.prototype.constantOr = function(Ne) {
                  return this.value.kind === "constant" ? this.value.value : Ne;
                }, nc.prototype.evaluate = function(Ne, je, ut, kt) {
                  return this.property.evaluate(this.value, this.parameters, Ne, je, ut, kt);
                };
                var Ko = function(Ne) {
                  this._properties = Ne, this._values = Object.create(Ne.defaultPossiblyEvaluatedValues);
                };
                Ko.prototype.get = function(Ne) {
                  return this._values[Ne];
                };
                var ks = function(Ne) {
                  this.specification = Ne;
                };
                ks.prototype.possiblyEvaluate = function(Ne, je) {
                  return Ne.expression.evaluate(je);
                }, ks.prototype.interpolate = function(Ne, je, ut) {
                  var kt = Dn[this.specification.type];
                  return kt ? kt(Ne, je, ut) : Ne;
                };
                var qs = function(Ne, je) {
                  this.specification = Ne, this.overrides = je;
                };
                qs.prototype.possiblyEvaluate = function(Ne, je, ut, kt) {
                  return Ne.expression.kind === "constant" || Ne.expression.kind === "camera" ? new nc(this, { kind: "constant", value: Ne.expression.evaluate(je, null, {}, ut, kt) }, je) : new nc(this, Ne.expression, je);
                }, qs.prototype.interpolate = function(Ne, je, ut) {
                  if (Ne.value.kind !== "constant" || je.value.kind !== "constant") return Ne;
                  if (Ne.value.value === void 0 || je.value.value === void 0) return new nc(this, { kind: "constant", value: void 0 }, Ne.parameters);
                  var kt = Dn[this.specification.type];
                  return kt ? new nc(this, { kind: "constant", value: kt(Ne.value.value, je.value.value, ut) }, Ne.parameters) : Ne;
                }, qs.prototype.evaluate = function(Ne, je, ut, kt, Jt, fn) {
                  return Ne.kind === "constant" ? Ne.value : Ne.evaluate(je, ut, kt, Jt, fn);
                };
                var a1 = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.possiblyEvaluate = function(ut, kt, Jt, fn) {
                    if (ut.value === void 0) return new nc(this, { kind: "constant", value: void 0 }, kt);
                    if (ut.expression.kind === "constant") {
                      var Sn = ut.expression.evaluate(kt, null, {}, Jt, fn), Hn = ut.property.specification.type === "resolvedImage" && typeof Sn != "string" ? Sn.name : Sn, nr = this._calculate(Hn, Hn, Hn, kt);
                      return new nc(this, { kind: "constant", value: nr }, kt);
                    }
                    if (ut.expression.kind === "camera") {
                      var kr = this._calculate(ut.expression.evaluate({ zoom: kt.zoom - 1 }), ut.expression.evaluate({ zoom: kt.zoom }), ut.expression.evaluate({ zoom: kt.zoom + 1 }), kt);
                      return new nc(this, { kind: "constant", value: kr }, kt);
                    }
                    return new nc(this, ut.expression, kt);
                  }, je.prototype.evaluate = function(ut, kt, Jt, fn, Sn, Hn) {
                    if (ut.kind === "source") {
                      var nr = ut.evaluate(kt, Jt, fn, Sn, Hn);
                      return this._calculate(nr, nr, nr, kt);
                    }
                    return ut.kind === "composite" ? this._calculate(ut.evaluate({ zoom: Math.floor(kt.zoom) - 1 }, Jt, fn), ut.evaluate({ zoom: Math.floor(kt.zoom) }, Jt, fn), ut.evaluate({ zoom: Math.floor(kt.zoom) + 1 }, Jt, fn), kt) : ut.value;
                  }, je.prototype._calculate = function(ut, kt, Jt, fn) {
                    return fn.zoom > fn.zoomHistory.lastIntegerZoom ? { from: ut, to: kt } : { from: Jt, to: kt };
                  }, je.prototype.interpolate = function(ut) {
                    return ut;
                  }, je;
                }(qs), $l = function(Ne) {
                  this.specification = Ne;
                };
                $l.prototype.possiblyEvaluate = function(Ne, je, ut, kt) {
                  if (Ne.value !== void 0) {
                    if (Ne.expression.kind === "constant") {
                      var Jt = Ne.expression.evaluate(je, null, {}, ut, kt);
                      return this._calculate(Jt, Jt, Jt, je);
                    }
                    return this._calculate(Ne.expression.evaluate(new El(Math.floor(je.zoom - 1), je)), Ne.expression.evaluate(new El(Math.floor(je.zoom), je)), Ne.expression.evaluate(new El(Math.floor(je.zoom + 1), je)), je);
                  }
                }, $l.prototype._calculate = function(Ne, je, ut, kt) {
                  return kt.zoom > kt.zoomHistory.lastIntegerZoom ? { from: Ne, to: je } : { from: ut, to: je };
                }, $l.prototype.interpolate = function(Ne) {
                  return Ne;
                };
                var o1 = function(Ne) {
                  this.specification = Ne;
                };
                o1.prototype.possiblyEvaluate = function(Ne, je, ut, kt) {
                  return !!Ne.expression.evaluate(je, null, {}, ut, kt);
                }, o1.prototype.interpolate = function() {
                  return !1;
                };
                var $c = function(Ne) {
                  for (var je in this.properties = Ne, this.defaultPropertyValues = {}, this.defaultTransitionablePropertyValues = {}, this.defaultTransitioningPropertyValues = {}, this.defaultPossiblyEvaluatedValues = {}, this.overridableProperties = [], Ne) {
                    var ut = Ne[je];
                    ut.specification.overridable && this.overridableProperties.push(je);
                    var kt = this.defaultPropertyValues[je] = new eu(ut, void 0), Jt = this.defaultTransitionablePropertyValues[je] = new Wc(ut);
                    this.defaultTransitioningPropertyValues[je] = Jt.untransitioned(), this.defaultPossiblyEvaluatedValues[je] = kt.possiblyEvaluate({});
                  }
                };
                cs("DataDrivenProperty", qs), cs("DataConstantProperty", ks), cs("CrossFadedDataDrivenProperty", a1), cs("CrossFadedProperty", $l), cs("ColorRampProperty", o1);
                var lu = "-transition", Ul = function(Ne) {
                  function je(ut, kt) {
                    if (Ne.call(this), this.id = ut.id, this.type = ut.type, this._featureFilter = { filter: function() {
                      return !0;
                    }, needGeometry: !1 }, ut.type !== "custom" && (this.metadata = ut.metadata, this.minzoom = ut.minzoom, this.maxzoom = ut.maxzoom, ut.type !== "background" && (this.source = ut.source, this.sourceLayer = ut["source-layer"], this.filter = ut.filter), kt.layout && (this._unevaluatedLayout = new Yc(kt.layout)), kt.paint)) {
                      for (var Jt in this._transitionablePaint = new hu(kt.paint), ut.paint) this.setPaintProperty(Jt, ut.paint[Jt], { validate: !1 });
                      for (var fn in ut.layout) this.setLayoutProperty(fn, ut.layout[fn], { validate: !1 });
                      this._transitioningPaint = this._transitionablePaint.untransitioned(), this.paint = new Ko(kt.paint);
                    }
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.getCrossfadeParameters = function() {
                    return this._crossfadeParameters;
                  }, je.prototype.getLayoutProperty = function(ut) {
                    return ut === "visibility" ? this.visibility : this._unevaluatedLayout.getValue(ut);
                  }, je.prototype.setLayoutProperty = function(ut, kt, Jt) {
                    if (Jt === void 0 && (Jt = {}), kt != null) {
                      var fn = "layers." + this.id + ".layout." + ut;
                      if (this._validate(du, fn, ut, kt, Jt)) return;
                    }
                    ut !== "visibility" ? this._unevaluatedLayout.setValue(ut, kt) : this.visibility = kt;
                  }, je.prototype.getPaintProperty = function(ut) {
                    return Ee(ut, lu) ? this._transitionablePaint.getTransition(ut.slice(0, -11)) : this._transitionablePaint.getValue(ut);
                  }, je.prototype.setPaintProperty = function(ut, kt, Jt) {
                    if (Jt === void 0 && (Jt = {}), kt != null) {
                      var fn = "layers." + this.id + ".paint." + ut;
                      if (this._validate(Pc, fn, ut, kt, Jt)) return !1;
                    }
                    if (Ee(ut, lu)) return this._transitionablePaint.setTransition(ut.slice(0, -11), kt || void 0), !1;
                    var Sn = this._transitionablePaint._values[ut], Hn = Sn.property.specification["property-type"] === "cross-faded-data-driven", nr = Sn.value.isDataDriven(), kr = Sn.value;
                    this._transitionablePaint.setValue(ut, kt), this._handleSpecialPaintPropertyUpdate(ut);
                    var Ir = this._transitionablePaint._values[ut].value;
                    return Ir.isDataDriven() || nr || Hn || this._handleOverridablePaintPropertyUpdate(ut, kr, Ir);
                  }, je.prototype._handleSpecialPaintPropertyUpdate = function(ut) {
                  }, je.prototype._handleOverridablePaintPropertyUpdate = function(ut, kt, Jt) {
                    return !1;
                  }, je.prototype.isHidden = function(ut) {
                    return !!(this.minzoom && ut < this.minzoom) || !!(this.maxzoom && ut >= this.maxzoom) || this.visibility === "none";
                  }, je.prototype.updateTransitions = function(ut) {
                    this._transitioningPaint = this._transitionablePaint.transitioned(ut, this._transitioningPaint);
                  }, je.prototype.hasTransition = function() {
                    return this._transitioningPaint.hasTransition();
                  }, je.prototype.recalculate = function(ut, kt) {
                    ut.getCrossfadeParameters && (this._crossfadeParameters = ut.getCrossfadeParameters()), this._unevaluatedLayout && (this.layout = this._unevaluatedLayout.possiblyEvaluate(ut, void 0, kt)), this.paint = this._transitioningPaint.possiblyEvaluate(ut, void 0, kt);
                  }, je.prototype.serialize = function() {
                    var ut = { id: this.id, type: this.type, source: this.source, "source-layer": this.sourceLayer, metadata: this.metadata, minzoom: this.minzoom, maxzoom: this.maxzoom, filter: this.filter, layout: this._unevaluatedLayout && this._unevaluatedLayout.serialize(), paint: this._transitionablePaint && this._transitionablePaint.serialize() };
                    return this.visibility && (ut.layout = ut.layout || {}, ut.layout.visibility = this.visibility), Le(ut, function(kt, Jt) {
                      return !(kt === void 0 || Jt === "layout" && !Object.keys(kt).length || Jt === "paint" && !Object.keys(kt).length);
                    });
                  }, je.prototype._validate = function(ut, kt, Jt, fn, Sn) {
                    return Sn === void 0 && (Sn = {}), (!Sn || Sn.validate !== !1) && Ac(this, ut.call(Ec, { key: kt, layerType: this.type, objectKey: Jt, value: fn, styleSpec: Zt, style: { glyphs: !0, sprite: !0 } }));
                  }, je.prototype.is3D = function() {
                    return !1;
                  }, je.prototype.isTileClipped = function() {
                    return !1;
                  }, je.prototype.hasOffscreenPass = function() {
                    return !1;
                  }, je.prototype.resize = function() {
                  }, je.prototype.isStateDependent = function() {
                    for (var ut in this.paint._values) {
                      var kt = this.paint.get(ut);
                      if (kt instanceof nc && So(kt.property.specification) && (kt.value.kind === "source" || kt.value.kind === "composite") && kt.value.isStateDependent) return !0;
                    }
                    return !1;
                  }, je;
                }(an), zs = { Int8: Int8Array, Uint8: Uint8Array, Int16: Int16Array, Uint16: Uint16Array, Int32: Int32Array, Uint32: Uint32Array, Float32: Float32Array }, Os = function(Ne, je) {
                  this._structArray = Ne, this._pos1 = je * this.size, this._pos2 = this._pos1 / 2, this._pos4 = this._pos1 / 4, this._pos8 = this._pos1 / 8;
                }, lc = function() {
                  this.isTransferred = !1, this.capacity = -1, this.resize(0);
                };
                function xc(Ne, je) {
                  je === void 0 && (je = 1);
                  var ut = 0, kt = 0;
                  return { members: Ne.map(function(Jt) {
                    var fn, Sn = (fn = Jt.type, zs[fn].BYTES_PER_ELEMENT), Hn = ut = Iu(ut, Math.max(je, Sn)), nr = Jt.components || 1;
                    return kt = Math.max(kt, Sn), ut += Sn * nr, { name: Jt.name, type: Jt.type, components: nr, offset: Hn };
                  }), size: Iu(ut, Math.max(kt, je)), alignment: je };
                }
                function Iu(Ne, je) {
                  return Math.ceil(Ne / je) * je;
                }
                lc.serialize = function(Ne, je) {
                  return Ne._trim(), je && (Ne.isTransferred = !0, je.push(Ne.arrayBuffer)), { length: Ne.length, arrayBuffer: Ne.arrayBuffer };
                }, lc.deserialize = function(Ne) {
                  var je = Object.create(this.prototype);
                  return je.arrayBuffer = Ne.arrayBuffer, je.length = Ne.length, je.capacity = Ne.arrayBuffer.byteLength / je.bytesPerElement, je._refreshViews(), je;
                }, lc.prototype._trim = function() {
                  this.length !== this.capacity && (this.capacity = this.length, this.arrayBuffer = this.arrayBuffer.slice(0, this.length * this.bytesPerElement), this._refreshViews());
                }, lc.prototype.clear = function() {
                  this.length = 0;
                }, lc.prototype.resize = function(Ne) {
                  this.reserve(Ne), this.length = Ne;
                }, lc.prototype.reserve = function(Ne) {
                  if (Ne > this.capacity) {
                    this.capacity = Math.max(Ne, Math.floor(5 * this.capacity), 128), this.arrayBuffer = new ArrayBuffer(this.capacity * this.bytesPerElement);
                    var je = this.uint8;
                    this._refreshViews(), je && this.uint8.set(je);
                  }
                }, lc.prototype._refreshViews = function() {
                  throw new Error("_refreshViews() must be implemented by each concrete StructArray layout");
                };
                var tu = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt) {
                    var Jt = this.length;
                    return this.resize(Jt + 1), this.emplace(Jt, ut, kt);
                  }, je.prototype.emplace = function(ut, kt, Jt) {
                    var fn = 2 * ut;
                    return this.int16[fn + 0] = kt, this.int16[fn + 1] = Jt, ut;
                  }, je;
                }(lc);
                tu.prototype.bytesPerElement = 4, cs("StructArrayLayout2i4", tu);
                var w1 = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn) {
                    var Sn = this.length;
                    return this.resize(Sn + 1), this.emplace(Sn, ut, kt, Jt, fn);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn) {
                    var Hn = 4 * ut;
                    return this.int16[Hn + 0] = kt, this.int16[Hn + 1] = Jt, this.int16[Hn + 2] = fn, this.int16[Hn + 3] = Sn, ut;
                  }, je;
                }(lc);
                w1.prototype.bytesPerElement = 8, cs("StructArrayLayout4i8", w1);
                var $n = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn) {
                    var nr = this.length;
                    return this.resize(nr + 1), this.emplace(nr, ut, kt, Jt, fn, Sn, Hn);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr) {
                    var kr = 6 * ut;
                    return this.int16[kr + 0] = kt, this.int16[kr + 1] = Jt, this.int16[kr + 2] = fn, this.int16[kr + 3] = Sn, this.int16[kr + 4] = Hn, this.int16[kr + 5] = nr, ut;
                  }, je;
                }(lc);
                $n.prototype.bytesPerElement = 12, cs("StructArrayLayout2i4i12", $n);
                var zn = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn) {
                    var nr = this.length;
                    return this.resize(nr + 1), this.emplace(nr, ut, kt, Jt, fn, Sn, Hn);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr) {
                    var kr = 4 * ut, Ir = 8 * ut;
                    return this.int16[kr + 0] = kt, this.int16[kr + 1] = Jt, this.uint8[Ir + 4] = fn, this.uint8[Ir + 5] = Sn, this.uint8[Ir + 6] = Hn, this.uint8[Ir + 7] = nr, ut;
                  }, je;
                }(lc);
                zn.prototype.bytesPerElement = 8, cs("StructArrayLayout2i4ub8", zn);
                var Gn = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt) {
                    var Jt = this.length;
                    return this.resize(Jt + 1), this.emplace(Jt, ut, kt);
                  }, je.prototype.emplace = function(ut, kt, Jt) {
                    var fn = 2 * ut;
                    return this.float32[fn + 0] = kt, this.float32[fn + 1] = Jt, ut;
                  }, je;
                }(lc);
                Gn.prototype.bytesPerElement = 8, cs("StructArrayLayout2f8", Gn);
                var Ke = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr) {
                    var ci = this.length;
                    return this.resize(ci + 1), this.emplace(ci, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci) {
                    var ni = 10 * ut;
                    return this.uint16[ni + 0] = kt, this.uint16[ni + 1] = Jt, this.uint16[ni + 2] = fn, this.uint16[ni + 3] = Sn, this.uint16[ni + 4] = Hn, this.uint16[ni + 5] = nr, this.uint16[ni + 6] = kr, this.uint16[ni + 7] = Ir, this.uint16[ni + 8] = qr, this.uint16[ni + 9] = ci, ut;
                  }, je;
                }(lc);
                Ke.prototype.bytesPerElement = 20, cs("StructArrayLayout10ui20", Ke);
                var Fe = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni) {
                    var Ii = this.length;
                    return this.resize(Ii + 1), this.emplace(Ii, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii) {
                    var Gi = 12 * ut;
                    return this.int16[Gi + 0] = kt, this.int16[Gi + 1] = Jt, this.int16[Gi + 2] = fn, this.int16[Gi + 3] = Sn, this.uint16[Gi + 4] = Hn, this.uint16[Gi + 5] = nr, this.uint16[Gi + 6] = kr, this.uint16[Gi + 7] = Ir, this.int16[Gi + 8] = qr, this.int16[Gi + 9] = ci, this.int16[Gi + 10] = ni, this.int16[Gi + 11] = Ii, ut;
                  }, je;
                }(lc);
                Fe.prototype.bytesPerElement = 24, cs("StructArrayLayout4i4ui4i24", Fe);
                var Te = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt) {
                    var fn = this.length;
                    return this.resize(fn + 1), this.emplace(fn, ut, kt, Jt);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn) {
                    var Sn = 3 * ut;
                    return this.float32[Sn + 0] = kt, this.float32[Sn + 1] = Jt, this.float32[Sn + 2] = fn, ut;
                  }, je;
                }(lc);
                Te.prototype.bytesPerElement = 12, cs("StructArrayLayout3f12", Te);
                var Ae = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut) {
                    var kt = this.length;
                    return this.resize(kt + 1), this.emplace(kt, ut);
                  }, je.prototype.emplace = function(ut, kt) {
                    var Jt = 1 * ut;
                    return this.uint32[Jt + 0] = kt, ut;
                  }, je;
                }(lc);
                Ae.prototype.bytesPerElement = 4, cs("StructArrayLayout1ul4", Ae);
                var Oe = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir) {
                    var qr = this.length;
                    return this.resize(qr + 1), this.emplace(qr, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr) {
                    var ci = 10 * ut, ni = 5 * ut;
                    return this.int16[ci + 0] = kt, this.int16[ci + 1] = Jt, this.int16[ci + 2] = fn, this.int16[ci + 3] = Sn, this.int16[ci + 4] = Hn, this.int16[ci + 5] = nr, this.uint32[ni + 3] = kr, this.uint16[ci + 8] = Ir, this.uint16[ci + 9] = qr, ut;
                  }, je;
                }(lc);
                Oe.prototype.bytesPerElement = 20, cs("StructArrayLayout6i1ul2ui20", Oe);
                var Be = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn) {
                    var nr = this.length;
                    return this.resize(nr + 1), this.emplace(nr, ut, kt, Jt, fn, Sn, Hn);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr) {
                    var kr = 6 * ut;
                    return this.int16[kr + 0] = kt, this.int16[kr + 1] = Jt, this.int16[kr + 2] = fn, this.int16[kr + 3] = Sn, this.int16[kr + 4] = Hn, this.int16[kr + 5] = nr, ut;
                  }, je;
                }(lc);
                Be.prototype.bytesPerElement = 12, cs("StructArrayLayout2i2i2i12", Be);
                var Ue = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn) {
                    var Hn = this.length;
                    return this.resize(Hn + 1), this.emplace(Hn, ut, kt, Jt, fn, Sn);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn) {
                    var nr = 4 * ut, kr = 8 * ut;
                    return this.float32[nr + 0] = kt, this.float32[nr + 1] = Jt, this.float32[nr + 2] = fn, this.int16[kr + 6] = Sn, this.int16[kr + 7] = Hn, ut;
                  }, je;
                }(lc);
                Ue.prototype.bytesPerElement = 16, cs("StructArrayLayout2f1f2i16", Ue);
                var nt = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn) {
                    var Sn = this.length;
                    return this.resize(Sn + 1), this.emplace(Sn, ut, kt, Jt, fn);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn) {
                    var Hn = 12 * ut, nr = 3 * ut;
                    return this.uint8[Hn + 0] = kt, this.uint8[Hn + 1] = Jt, this.float32[nr + 1] = fn, this.float32[nr + 2] = Sn, ut;
                  }, je;
                }(lc);
                nt.prototype.bytesPerElement = 12, cs("StructArrayLayout2ub2f12", nt);
                var ht = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt) {
                    var fn = this.length;
                    return this.resize(fn + 1), this.emplace(fn, ut, kt, Jt);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn) {
                    var Sn = 3 * ut;
                    return this.uint16[Sn + 0] = kt, this.uint16[Sn + 1] = Jt, this.uint16[Sn + 2] = fn, ut;
                  }, je;
                }(lc);
                ht.prototype.bytesPerElement = 6, cs("StructArrayLayout3ui6", ht);
                var Et = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi, xa, ga, $a) {
                    var Va = this.length;
                    return this.resize(Va + 1), this.emplace(Va, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi, xa, ga, $a);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi, xa, ga, $a, Va) {
                    var eo = 24 * ut, co = 12 * ut, wo = 48 * ut;
                    return this.int16[eo + 0] = kt, this.int16[eo + 1] = Jt, this.uint16[eo + 2] = fn, this.uint16[eo + 3] = Sn, this.uint32[co + 2] = Hn, this.uint32[co + 3] = nr, this.uint32[co + 4] = kr, this.uint16[eo + 10] = Ir, this.uint16[eo + 11] = qr, this.uint16[eo + 12] = ci, this.float32[co + 7] = ni, this.float32[co + 8] = Ii, this.uint8[wo + 36] = Gi, this.uint8[wo + 37] = xa, this.uint8[wo + 38] = ga, this.uint32[co + 10] = $a, this.int16[eo + 22] = Va, ut;
                  }, je;
                }(lc);
                Et.prototype.bytesPerElement = 48, cs("StructArrayLayout2i2ui3ul3ui2f3ub1ul1i48", Et);
                var Mt = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi, xa, ga, $a, Va, eo, co, wo, ds, Lo, As, Fs, Pl, rl, Rl) {
                    var pl = this.length;
                    return this.resize(pl + 1), this.emplace(pl, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi, xa, ga, $a, Va, eo, co, wo, ds, Lo, As, Fs, Pl, rl, Rl);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi, xa, ga, $a, Va, eo, co, wo, ds, Lo, As, Fs, Pl, rl, Rl, pl) {
                    var Zs = 34 * ut, Hc = 17 * ut;
                    return this.int16[Zs + 0] = kt, this.int16[Zs + 1] = Jt, this.int16[Zs + 2] = fn, this.int16[Zs + 3] = Sn, this.int16[Zs + 4] = Hn, this.int16[Zs + 5] = nr, this.int16[Zs + 6] = kr, this.int16[Zs + 7] = Ir, this.uint16[Zs + 8] = qr, this.uint16[Zs + 9] = ci, this.uint16[Zs + 10] = ni, this.uint16[Zs + 11] = Ii, this.uint16[Zs + 12] = Gi, this.uint16[Zs + 13] = xa, this.uint16[Zs + 14] = ga, this.uint16[Zs + 15] = $a, this.uint16[Zs + 16] = Va, this.uint16[Zs + 17] = eo, this.uint16[Zs + 18] = co, this.uint16[Zs + 19] = wo, this.uint16[Zs + 20] = ds, this.uint16[Zs + 21] = Lo, this.uint16[Zs + 22] = As, this.uint32[Hc + 12] = Fs, this.float32[Hc + 13] = Pl, this.float32[Hc + 14] = rl, this.float32[Hc + 15] = Rl, this.float32[Hc + 16] = pl, ut;
                  }, je;
                }(lc);
                Mt.prototype.bytesPerElement = 68, cs("StructArrayLayout8i15ui1ul4f68", Mt);
                var qt = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut) {
                    var kt = this.length;
                    return this.resize(kt + 1), this.emplace(kt, ut);
                  }, je.prototype.emplace = function(ut, kt) {
                    var Jt = 1 * ut;
                    return this.float32[Jt + 0] = kt, ut;
                  }, je;
                }(lc);
                qt.prototype.bytesPerElement = 4, cs("StructArrayLayout1f4", qt);
                var tn = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt) {
                    var fn = this.length;
                    return this.resize(fn + 1), this.emplace(fn, ut, kt, Jt);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn) {
                    var Sn = 3 * ut;
                    return this.int16[Sn + 0] = kt, this.int16[Sn + 1] = Jt, this.int16[Sn + 2] = fn, ut;
                  }, je;
                }(lc);
                tn.prototype.bytesPerElement = 6, cs("StructArrayLayout3i6", tn);
                var xn = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt) {
                    var fn = this.length;
                    return this.resize(fn + 1), this.emplace(fn, ut, kt, Jt);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn) {
                    var Sn = 2 * ut, Hn = 4 * ut;
                    return this.uint32[Sn + 0] = kt, this.uint16[Hn + 2] = Jt, this.uint16[Hn + 3] = fn, ut;
                  }, je;
                }(lc);
                xn.prototype.bytesPerElement = 8, cs("StructArrayLayout1ul2ui8", xn);
                var An = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt) {
                    var Jt = this.length;
                    return this.resize(Jt + 1), this.emplace(Jt, ut, kt);
                  }, je.prototype.emplace = function(ut, kt, Jt) {
                    var fn = 2 * ut;
                    return this.uint16[fn + 0] = kt, this.uint16[fn + 1] = Jt, ut;
                  }, je;
                }(lc);
                An.prototype.bytesPerElement = 4, cs("StructArrayLayout2ui4", An);
                var On = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut) {
                    var kt = this.length;
                    return this.resize(kt + 1), this.emplace(kt, ut);
                  }, je.prototype.emplace = function(ut, kt) {
                    var Jt = 1 * ut;
                    return this.uint16[Jt + 0] = kt, ut;
                  }, je;
                }(lc);
                On.prototype.bytesPerElement = 2, cs("StructArrayLayout1ui2", On);
                var Bn = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._refreshViews = function() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }, je.prototype.emplaceBack = function(ut, kt, Jt, fn) {
                    var Sn = this.length;
                    return this.resize(Sn + 1), this.emplace(Sn, ut, kt, Jt, fn);
                  }, je.prototype.emplace = function(ut, kt, Jt, fn, Sn) {
                    var Hn = 4 * ut;
                    return this.float32[Hn + 0] = kt, this.float32[Hn + 1] = Jt, this.float32[Hn + 2] = fn, this.float32[Hn + 3] = Sn, ut;
                  }, je;
                }(lc);
                Bn.prototype.bytesPerElement = 16, cs("StructArrayLayout4f16", Bn);
                var qn = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je;
                  var ut = { anchorPointX: { configurable: !0 }, anchorPointY: { configurable: !0 }, x1: { configurable: !0 }, y1: { configurable: !0 }, x2: { configurable: !0 }, y2: { configurable: !0 }, featureIndex: { configurable: !0 }, sourceLayerIndex: { configurable: !0 }, bucketIndex: { configurable: !0 }, anchorPoint: { configurable: !0 } };
                  return ut.anchorPointX.get = function() {
                    return this._structArray.int16[this._pos2 + 0];
                  }, ut.anchorPointY.get = function() {
                    return this._structArray.int16[this._pos2 + 1];
                  }, ut.x1.get = function() {
                    return this._structArray.int16[this._pos2 + 2];
                  }, ut.y1.get = function() {
                    return this._structArray.int16[this._pos2 + 3];
                  }, ut.x2.get = function() {
                    return this._structArray.int16[this._pos2 + 4];
                  }, ut.y2.get = function() {
                    return this._structArray.int16[this._pos2 + 5];
                  }, ut.featureIndex.get = function() {
                    return this._structArray.uint32[this._pos4 + 3];
                  }, ut.sourceLayerIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 8];
                  }, ut.bucketIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 9];
                  }, ut.anchorPoint.get = function() {
                    return new le(this.anchorPointX, this.anchorPointY);
                  }, Object.defineProperties(je.prototype, ut), je;
                }(Os);
                qn.prototype.size = 20;
                var Vn = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.get = function(ut) {
                    return new qn(this, ut);
                  }, je;
                }(Oe);
                cs("CollisionBoxArray", Vn);
                var sr = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je;
                  var ut = { anchorX: { configurable: !0 }, anchorY: { configurable: !0 }, glyphStartIndex: { configurable: !0 }, numGlyphs: { configurable: !0 }, vertexStartIndex: { configurable: !0 }, lineStartIndex: { configurable: !0 }, lineLength: { configurable: !0 }, segment: { configurable: !0 }, lowerSize: { configurable: !0 }, upperSize: { configurable: !0 }, lineOffsetX: { configurable: !0 }, lineOffsetY: { configurable: !0 }, writingMode: { configurable: !0 }, placedOrientation: { configurable: !0 }, hidden: { configurable: !0 }, crossTileID: { configurable: !0 }, associatedIconIndex: { configurable: !0 } };
                  return ut.anchorX.get = function() {
                    return this._structArray.int16[this._pos2 + 0];
                  }, ut.anchorY.get = function() {
                    return this._structArray.int16[this._pos2 + 1];
                  }, ut.glyphStartIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 2];
                  }, ut.numGlyphs.get = function() {
                    return this._structArray.uint16[this._pos2 + 3];
                  }, ut.vertexStartIndex.get = function() {
                    return this._structArray.uint32[this._pos4 + 2];
                  }, ut.lineStartIndex.get = function() {
                    return this._structArray.uint32[this._pos4 + 3];
                  }, ut.lineLength.get = function() {
                    return this._structArray.uint32[this._pos4 + 4];
                  }, ut.segment.get = function() {
                    return this._structArray.uint16[this._pos2 + 10];
                  }, ut.lowerSize.get = function() {
                    return this._structArray.uint16[this._pos2 + 11];
                  }, ut.upperSize.get = function() {
                    return this._structArray.uint16[this._pos2 + 12];
                  }, ut.lineOffsetX.get = function() {
                    return this._structArray.float32[this._pos4 + 7];
                  }, ut.lineOffsetY.get = function() {
                    return this._structArray.float32[this._pos4 + 8];
                  }, ut.writingMode.get = function() {
                    return this._structArray.uint8[this._pos1 + 36];
                  }, ut.placedOrientation.get = function() {
                    return this._structArray.uint8[this._pos1 + 37];
                  }, ut.placedOrientation.set = function(kt) {
                    this._structArray.uint8[this._pos1 + 37] = kt;
                  }, ut.hidden.get = function() {
                    return this._structArray.uint8[this._pos1 + 38];
                  }, ut.hidden.set = function(kt) {
                    this._structArray.uint8[this._pos1 + 38] = kt;
                  }, ut.crossTileID.get = function() {
                    return this._structArray.uint32[this._pos4 + 10];
                  }, ut.crossTileID.set = function(kt) {
                    this._structArray.uint32[this._pos4 + 10] = kt;
                  }, ut.associatedIconIndex.get = function() {
                    return this._structArray.int16[this._pos2 + 22];
                  }, Object.defineProperties(je.prototype, ut), je;
                }(Os);
                sr.prototype.size = 48;
                var Un = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.get = function(ut) {
                    return new sr(this, ut);
                  }, je;
                }(Et);
                cs("PlacedSymbolArray", Un);
                var wr = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je;
                  var ut = { anchorX: { configurable: !0 }, anchorY: { configurable: !0 }, rightJustifiedTextSymbolIndex: { configurable: !0 }, centerJustifiedTextSymbolIndex: { configurable: !0 }, leftJustifiedTextSymbolIndex: { configurable: !0 }, verticalPlacedTextSymbolIndex: { configurable: !0 }, placedIconSymbolIndex: { configurable: !0 }, verticalPlacedIconSymbolIndex: { configurable: !0 }, key: { configurable: !0 }, textBoxStartIndex: { configurable: !0 }, textBoxEndIndex: { configurable: !0 }, verticalTextBoxStartIndex: { configurable: !0 }, verticalTextBoxEndIndex: { configurable: !0 }, iconBoxStartIndex: { configurable: !0 }, iconBoxEndIndex: { configurable: !0 }, verticalIconBoxStartIndex: { configurable: !0 }, verticalIconBoxEndIndex: { configurable: !0 }, featureIndex: { configurable: !0 }, numHorizontalGlyphVertices: { configurable: !0 }, numVerticalGlyphVertices: { configurable: !0 }, numIconVertices: { configurable: !0 }, numVerticalIconVertices: { configurable: !0 }, useRuntimeCollisionCircles: { configurable: !0 }, crossTileID: { configurable: !0 }, textBoxScale: { configurable: !0 }, textOffset0: { configurable: !0 }, textOffset1: { configurable: !0 }, collisionCircleDiameter: { configurable: !0 } };
                  return ut.anchorX.get = function() {
                    return this._structArray.int16[this._pos2 + 0];
                  }, ut.anchorY.get = function() {
                    return this._structArray.int16[this._pos2 + 1];
                  }, ut.rightJustifiedTextSymbolIndex.get = function() {
                    return this._structArray.int16[this._pos2 + 2];
                  }, ut.centerJustifiedTextSymbolIndex.get = function() {
                    return this._structArray.int16[this._pos2 + 3];
                  }, ut.leftJustifiedTextSymbolIndex.get = function() {
                    return this._structArray.int16[this._pos2 + 4];
                  }, ut.verticalPlacedTextSymbolIndex.get = function() {
                    return this._structArray.int16[this._pos2 + 5];
                  }, ut.placedIconSymbolIndex.get = function() {
                    return this._structArray.int16[this._pos2 + 6];
                  }, ut.verticalPlacedIconSymbolIndex.get = function() {
                    return this._structArray.int16[this._pos2 + 7];
                  }, ut.key.get = function() {
                    return this._structArray.uint16[this._pos2 + 8];
                  }, ut.textBoxStartIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 9];
                  }, ut.textBoxEndIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 10];
                  }, ut.verticalTextBoxStartIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 11];
                  }, ut.verticalTextBoxEndIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 12];
                  }, ut.iconBoxStartIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 13];
                  }, ut.iconBoxEndIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 14];
                  }, ut.verticalIconBoxStartIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 15];
                  }, ut.verticalIconBoxEndIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 16];
                  }, ut.featureIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 17];
                  }, ut.numHorizontalGlyphVertices.get = function() {
                    return this._structArray.uint16[this._pos2 + 18];
                  }, ut.numVerticalGlyphVertices.get = function() {
                    return this._structArray.uint16[this._pos2 + 19];
                  }, ut.numIconVertices.get = function() {
                    return this._structArray.uint16[this._pos2 + 20];
                  }, ut.numVerticalIconVertices.get = function() {
                    return this._structArray.uint16[this._pos2 + 21];
                  }, ut.useRuntimeCollisionCircles.get = function() {
                    return this._structArray.uint16[this._pos2 + 22];
                  }, ut.crossTileID.get = function() {
                    return this._structArray.uint32[this._pos4 + 12];
                  }, ut.crossTileID.set = function(kt) {
                    this._structArray.uint32[this._pos4 + 12] = kt;
                  }, ut.textBoxScale.get = function() {
                    return this._structArray.float32[this._pos4 + 13];
                  }, ut.textOffset0.get = function() {
                    return this._structArray.float32[this._pos4 + 14];
                  }, ut.textOffset1.get = function() {
                    return this._structArray.float32[this._pos4 + 15];
                  }, ut.collisionCircleDiameter.get = function() {
                    return this._structArray.float32[this._pos4 + 16];
                  }, Object.defineProperties(je.prototype, ut), je;
                }(Os);
                wr.prototype.size = 68;
                var Cr = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.get = function(ut) {
                    return new wr(this, ut);
                  }, je;
                }(Mt);
                cs("SymbolInstanceArray", Cr);
                var Tr = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.getoffsetX = function(ut) {
                    return this.float32[1 * ut + 0];
                  }, je;
                }(qt);
                cs("GlyphOffsetArray", Tr);
                var ii = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.getx = function(ut) {
                    return this.int16[3 * ut + 0];
                  }, je.prototype.gety = function(ut) {
                    return this.int16[3 * ut + 1];
                  }, je.prototype.gettileUnitDistanceFromAnchor = function(ut) {
                    return this.int16[3 * ut + 2];
                  }, je;
                }(tn);
                cs("SymbolLineVertexArray", ii);
                var wi = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je;
                  var ut = { featureIndex: { configurable: !0 }, sourceLayerIndex: { configurable: !0 }, bucketIndex: { configurable: !0 } };
                  return ut.featureIndex.get = function() {
                    return this._structArray.uint32[this._pos4 + 0];
                  }, ut.sourceLayerIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 2];
                  }, ut.bucketIndex.get = function() {
                    return this._structArray.uint16[this._pos2 + 3];
                  }, Object.defineProperties(je.prototype, ut), je;
                }(Os);
                wi.prototype.size = 8;
                var Wi = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.get = function(ut) {
                    return new wi(this, ut);
                  }, je;
                }(xn);
                cs("FeatureIndexArray", Wi);
                var Yi = xc([{ name: "a_pos", components: 2, type: "Int16" }], 4).members, ra = function(Ne) {
                  Ne === void 0 && (Ne = []), this.segments = Ne;
                };
                function ha(Ne, je) {
                  return 256 * (Ne = pe(Math.floor(Ne), 0, 255)) + pe(Math.floor(je), 0, 255);
                }
                ra.prototype.prepareSegment = function(Ne, je, ut, kt) {
                  var Jt = this.segments[this.segments.length - 1];
                  return Ne > ra.MAX_VERTEX_ARRAY_LENGTH && Me("Max vertices per segment is " + ra.MAX_VERTEX_ARRAY_LENGTH + ": bucket requested " + Ne), (!Jt || Jt.vertexLength + Ne > ra.MAX_VERTEX_ARRAY_LENGTH || Jt.sortKey !== kt) && (Jt = { vertexOffset: je.length, primitiveOffset: ut.length, vertexLength: 0, primitiveLength: 0 }, kt !== void 0 && (Jt.sortKey = kt), this.segments.push(Jt)), Jt;
                }, ra.prototype.get = function() {
                  return this.segments;
                }, ra.prototype.destroy = function() {
                  for (var Ne = 0, je = this.segments; Ne < je.length; Ne += 1) {
                    var ut = je[Ne];
                    for (var kt in ut.vaos) ut.vaos[kt].destroy();
                  }
                }, ra.simpleSegment = function(Ne, je, ut, kt) {
                  return new ra([{ vertexOffset: Ne, primitiveOffset: je, vertexLength: ut, primitiveLength: kt, vaos: {}, sortKey: 0 }]);
                }, ra.MAX_VERTEX_ARRAY_LENGTH = Math.pow(2, 16) - 1, cs("SegmentVector", ra);
                var ya = xc([{ name: "a_pattern_from", components: 4, type: "Uint16" }, { name: "a_pattern_to", components: 4, type: "Uint16" }, { name: "a_pixel_ratio_from", components: 1, type: "Uint16" }, { name: "a_pixel_ratio_to", components: 1, type: "Uint16" }]), va = ae(function(Ne) {
                  Ne.exports = function(je, ut) {
                    var kt, Jt, fn, Sn, Hn, nr, kr, Ir;
                    for (kt = 3 & je.length, Jt = je.length - kt, fn = ut, Hn = 3432918353, nr = 461845907, Ir = 0; Ir < Jt; ) kr = 255 & je.charCodeAt(Ir) | (255 & je.charCodeAt(++Ir)) << 8 | (255 & je.charCodeAt(++Ir)) << 16 | (255 & je.charCodeAt(++Ir)) << 24, ++Ir, fn = 27492 + (65535 & (Sn = 5 * (65535 & (fn = (fn ^= kr = (65535 & (kr = (kr = (65535 & kr) * Hn + (((kr >>> 16) * Hn & 65535) << 16) & 4294967295) << 15 | kr >>> 17)) * nr + (((kr >>> 16) * nr & 65535) << 16) & 4294967295) << 13 | fn >>> 19)) + ((5 * (fn >>> 16) & 65535) << 16) & 4294967295)) + ((58964 + (Sn >>> 16) & 65535) << 16);
                    switch (kr = 0, kt) {
                      case 3:
                        kr ^= (255 & je.charCodeAt(Ir + 2)) << 16;
                      case 2:
                        kr ^= (255 & je.charCodeAt(Ir + 1)) << 8;
                      case 1:
                        fn ^= kr = (65535 & (kr = (kr = (65535 & (kr ^= 255 & je.charCodeAt(Ir))) * Hn + (((kr >>> 16) * Hn & 65535) << 16) & 4294967295) << 15 | kr >>> 17)) * nr + (((kr >>> 16) * nr & 65535) << 16) & 4294967295;
                    }
                    return fn ^= je.length, fn = 2246822507 * (65535 & (fn ^= fn >>> 16)) + ((2246822507 * (fn >>> 16) & 65535) << 16) & 4294967295, fn = 3266489909 * (65535 & (fn ^= fn >>> 13)) + ((3266489909 * (fn >>> 16) & 65535) << 16) & 4294967295, (fn ^= fn >>> 16) >>> 0;
                  };
                }), qa = ae(function(Ne) {
                  Ne.exports = function(je, ut) {
                    for (var kt, Jt = je.length, fn = ut ^ Jt, Sn = 0; Jt >= 4; ) kt = 1540483477 * (65535 & (kt = 255 & je.charCodeAt(Sn) | (255 & je.charCodeAt(++Sn)) << 8 | (255 & je.charCodeAt(++Sn)) << 16 | (255 & je.charCodeAt(++Sn)) << 24)) + ((1540483477 * (kt >>> 16) & 65535) << 16), fn = 1540483477 * (65535 & fn) + ((1540483477 * (fn >>> 16) & 65535) << 16) ^ (kt = 1540483477 * (65535 & (kt ^= kt >>> 24)) + ((1540483477 * (kt >>> 16) & 65535) << 16)), Jt -= 4, ++Sn;
                    switch (Jt) {
                      case 3:
                        fn ^= (255 & je.charCodeAt(Sn + 2)) << 16;
                      case 2:
                        fn ^= (255 & je.charCodeAt(Sn + 1)) << 8;
                      case 1:
                        fn = 1540483477 * (65535 & (fn ^= 255 & je.charCodeAt(Sn))) + ((1540483477 * (fn >>> 16) & 65535) << 16);
                    }
                    return fn = 1540483477 * (65535 & (fn ^= fn >>> 13)) + ((1540483477 * (fn >>> 16) & 65535) << 16), (fn ^= fn >>> 15) >>> 0;
                  };
                }), Aa = va, Xa = va, Oa = qa;
                Aa.murmur3 = Xa, Aa.murmur2 = Oa;
                var Ya = function() {
                  this.ids = [], this.positions = [], this.indexed = !1;
                };
                Ya.prototype.add = function(Ne, je, ut, kt) {
                  this.ids.push(Ja(Ne)), this.positions.push(je, ut, kt);
                }, Ya.prototype.getPositions = function(Ne) {
                  for (var je = Ja(Ne), ut = 0, kt = this.ids.length - 1; ut < kt; ) {
                    var Jt = ut + kt >> 1;
                    this.ids[Jt] >= je ? kt = Jt : ut = Jt + 1;
                  }
                  for (var fn = []; this.ids[ut] === je; ) {
                    var Sn = this.positions[3 * ut], Hn = this.positions[3 * ut + 1], nr = this.positions[3 * ut + 2];
                    fn.push({ index: Sn, start: Hn, end: nr }), ut++;
                  }
                  return fn;
                }, Ya.serialize = function(Ne, je) {
                  var ut = new Float64Array(Ne.ids), kt = new Uint32Array(Ne.positions);
                  return Uo(ut, kt, 0, ut.length - 1), je && je.push(ut.buffer, kt.buffer), { ids: ut, positions: kt };
                }, Ya.deserialize = function(Ne) {
                  var je = new Ya();
                  return je.ids = Ne.ids, je.positions = Ne.positions, je.indexed = !0, je;
                };
                var Wo = Math.pow(2, 53) - 1;
                function Ja(Ne) {
                  var je = +Ne;
                  return !isNaN(je) && je <= Wo ? je : Aa(String(Ne));
                }
                function Uo(Ne, je, ut, kt) {
                  for (; ut < kt; ) {
                    for (var Jt = Ne[ut + kt >> 1], fn = ut - 1, Sn = kt + 1; ; ) {
                      do
                        fn++;
                      while (Ne[fn] < Jt);
                      do
                        Sn--;
                      while (Ne[Sn] > Jt);
                      if (fn >= Sn) break;
                      ys(Ne, fn, Sn), ys(je, 3 * fn, 3 * Sn), ys(je, 3 * fn + 1, 3 * Sn + 1), ys(je, 3 * fn + 2, 3 * Sn + 2);
                    }
                    Sn - ut < kt - Sn ? (Uo(Ne, je, ut, Sn), ut = Sn + 1) : (Uo(Ne, je, Sn + 1, kt), kt = Sn);
                  }
                }
                function ys(Ne, je, ut) {
                  var kt = Ne[je];
                  Ne[je] = Ne[ut], Ne[ut] = kt;
                }
                cs("FeaturePositionMap", Ya);
                var il = function(Ne, je) {
                  this.gl = Ne.gl, this.location = je;
                }, Is = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, ut, kt), this.current = 0;
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.set = function(ut) {
                    this.current !== ut && (this.current = ut, this.gl.uniform1i(this.location, ut));
                  }, je;
                }(il), ml = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, ut, kt), this.current = 0;
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.set = function(ut) {
                    this.current !== ut && (this.current = ut, this.gl.uniform1f(this.location, ut));
                  }, je;
                }(il), nl = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, ut, kt), this.current = [0, 0];
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.set = function(ut) {
                    ut[0] === this.current[0] && ut[1] === this.current[1] || (this.current = ut, this.gl.uniform2f(this.location, ut[0], ut[1]));
                  }, je;
                }(il), Il = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, ut, kt), this.current = [0, 0, 0];
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.set = function(ut) {
                    ut[0] === this.current[0] && ut[1] === this.current[1] && ut[2] === this.current[2] || (this.current = ut, this.gl.uniform3f(this.location, ut[0], ut[1], ut[2]));
                  }, je;
                }(il), Fl = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, ut, kt), this.current = [0, 0, 0, 0];
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.set = function(ut) {
                    ut[0] === this.current[0] && ut[1] === this.current[1] && ut[2] === this.current[2] && ut[3] === this.current[3] || (this.current = ut, this.gl.uniform4f(this.location, ut[0], ut[1], ut[2], ut[3]));
                  }, je;
                }(il), qc = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, ut, kt), this.current = Pr.transparent;
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.set = function(ut) {
                    ut.r === this.current.r && ut.g === this.current.g && ut.b === this.current.b && ut.a === this.current.a || (this.current = ut, this.gl.uniform4f(this.location, ut.r, ut.g, ut.b, ut.a));
                  }, je;
                }(il), ql = new Float32Array(16), jc = function(Ne) {
                  function je(ut, kt) {
                    Ne.call(this, ut, kt), this.current = ql;
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.set = function(ut) {
                    if (ut[12] !== this.current[12] || ut[0] !== this.current[0]) return this.current = ut, void this.gl.uniformMatrix4fv(this.location, !1, ut);
                    for (var kt = 1; kt < 16; kt++) if (ut[kt] !== this.current[kt]) {
                      this.current = ut, this.gl.uniformMatrix4fv(this.location, !1, ut);
                      break;
                    }
                  }, je;
                }(il);
                function Dl(Ne) {
                  return [ha(255 * Ne.r, 255 * Ne.g), ha(255 * Ne.b, 255 * Ne.a)];
                }
                var Ml = function(Ne, je, ut) {
                  this.value = Ne, this.uniformNames = je.map(function(kt) {
                    return "u_" + kt;
                  }), this.type = ut;
                };
                Ml.prototype.setUniform = function(Ne, je, ut) {
                  Ne.set(ut.constantOr(this.value));
                }, Ml.prototype.getBinding = function(Ne, je, ut) {
                  return this.type === "color" ? new qc(Ne, je) : new ml(Ne, je);
                };
                var pc = function(Ne, je) {
                  this.uniformNames = je.map(function(ut) {
                    return "u_" + ut;
                  }), this.patternFrom = null, this.patternTo = null, this.pixelRatioFrom = 1, this.pixelRatioTo = 1;
                };
                pc.prototype.setConstantPatternPositions = function(Ne, je) {
                  this.pixelRatioFrom = je.pixelRatio, this.pixelRatioTo = Ne.pixelRatio, this.patternFrom = je.tlbr, this.patternTo = Ne.tlbr;
                }, pc.prototype.setUniform = function(Ne, je, ut, kt) {
                  var Jt = kt === "u_pattern_to" ? this.patternTo : kt === "u_pattern_from" ? this.patternFrom : kt === "u_pixel_ratio_to" ? this.pixelRatioTo : kt === "u_pixel_ratio_from" ? this.pixelRatioFrom : null;
                  Jt && Ne.set(Jt);
                }, pc.prototype.getBinding = function(Ne, je, ut) {
                  return ut.substr(0, 9) === "u_pattern" ? new Fl(Ne, je) : new ml(Ne, je);
                };
                var hl = function(Ne, je, ut, kt) {
                  this.expression = Ne, this.type = ut, this.maxValue = 0, this.paintVertexAttributes = je.map(function(Jt) {
                    return { name: "a_" + Jt, type: "Float32", components: ut === "color" ? 2 : 1, offset: 0 };
                  }), this.paintVertexArray = new kt();
                };
                hl.prototype.populatePaintArray = function(Ne, je, ut, kt, Jt) {
                  var fn = this.paintVertexArray.length, Sn = this.expression.evaluate(new El(0), je, {}, kt, [], Jt);
                  this.paintVertexArray.resize(Ne), this._setPaintValue(fn, Ne, Sn);
                }, hl.prototype.updatePaintArray = function(Ne, je, ut, kt) {
                  var Jt = this.expression.evaluate({ zoom: 0 }, ut, kt);
                  this._setPaintValue(Ne, je, Jt);
                }, hl.prototype._setPaintValue = function(Ne, je, ut) {
                  if (this.type === "color") for (var kt = Dl(ut), Jt = Ne; Jt < je; Jt++) this.paintVertexArray.emplace(Jt, kt[0], kt[1]);
                  else {
                    for (var fn = Ne; fn < je; fn++) this.paintVertexArray.emplace(fn, ut);
                    this.maxValue = Math.max(this.maxValue, Math.abs(ut));
                  }
                }, hl.prototype.upload = function(Ne) {
                  this.paintVertexArray && this.paintVertexArray.arrayBuffer && (this.paintVertexBuffer && this.paintVertexBuffer.buffer ? this.paintVertexBuffer.updateData(this.paintVertexArray) : this.paintVertexBuffer = Ne.createVertexBuffer(this.paintVertexArray, this.paintVertexAttributes, this.expression.isStateDependent));
                }, hl.prototype.destroy = function() {
                  this.paintVertexBuffer && this.paintVertexBuffer.destroy();
                };
                var xl = function(Ne, je, ut, kt, Jt, fn) {
                  this.expression = Ne, this.uniformNames = je.map(function(Sn) {
                    return "u_" + Sn + "_t";
                  }), this.type = ut, this.useIntegerZoom = kt, this.zoom = Jt, this.maxValue = 0, this.paintVertexAttributes = je.map(function(Sn) {
                    return { name: "a_" + Sn, type: "Float32", components: ut === "color" ? 4 : 2, offset: 0 };
                  }), this.paintVertexArray = new fn();
                };
                xl.prototype.populatePaintArray = function(Ne, je, ut, kt, Jt) {
                  var fn = this.expression.evaluate(new El(this.zoom), je, {}, kt, [], Jt), Sn = this.expression.evaluate(new El(this.zoom + 1), je, {}, kt, [], Jt), Hn = this.paintVertexArray.length;
                  this.paintVertexArray.resize(Ne), this._setPaintValue(Hn, Ne, fn, Sn);
                }, xl.prototype.updatePaintArray = function(Ne, je, ut, kt) {
                  var Jt = this.expression.evaluate({ zoom: this.zoom }, ut, kt), fn = this.expression.evaluate({ zoom: this.zoom + 1 }, ut, kt);
                  this._setPaintValue(Ne, je, Jt, fn);
                }, xl.prototype._setPaintValue = function(Ne, je, ut, kt) {
                  if (this.type === "color") for (var Jt = Dl(ut), fn = Dl(kt), Sn = Ne; Sn < je; Sn++) this.paintVertexArray.emplace(Sn, Jt[0], Jt[1], fn[0], fn[1]);
                  else {
                    for (var Hn = Ne; Hn < je; Hn++) this.paintVertexArray.emplace(Hn, ut, kt);
                    this.maxValue = Math.max(this.maxValue, Math.abs(ut), Math.abs(kt));
                  }
                }, xl.prototype.upload = function(Ne) {
                  this.paintVertexArray && this.paintVertexArray.arrayBuffer && (this.paintVertexBuffer && this.paintVertexBuffer.buffer ? this.paintVertexBuffer.updateData(this.paintVertexArray) : this.paintVertexBuffer = Ne.createVertexBuffer(this.paintVertexArray, this.paintVertexAttributes, this.expression.isStateDependent));
                }, xl.prototype.destroy = function() {
                  this.paintVertexBuffer && this.paintVertexBuffer.destroy();
                }, xl.prototype.setUniform = function(Ne, je) {
                  var ut = this.useIntegerZoom ? Math.floor(je.zoom) : je.zoom, kt = pe(this.expression.interpolationFactor(ut, this.zoom, this.zoom + 1), 0, 1);
                  Ne.set(kt);
                }, xl.prototype.getBinding = function(Ne, je, ut) {
                  return new ml(Ne, je);
                };
                var Bl = function(Ne, je, ut, kt, Jt, fn) {
                  this.expression = Ne, this.type = je, this.useIntegerZoom = ut, this.zoom = kt, this.layerId = fn, this.zoomInPaintVertexArray = new Jt(), this.zoomOutPaintVertexArray = new Jt();
                };
                Bl.prototype.populatePaintArray = function(Ne, je, ut) {
                  var kt = this.zoomInPaintVertexArray.length;
                  this.zoomInPaintVertexArray.resize(Ne), this.zoomOutPaintVertexArray.resize(Ne), this._setPaintValues(kt, Ne, je.patterns && je.patterns[this.layerId], ut);
                }, Bl.prototype.updatePaintArray = function(Ne, je, ut, kt, Jt) {
                  this._setPaintValues(Ne, je, ut.patterns && ut.patterns[this.layerId], Jt);
                }, Bl.prototype._setPaintValues = function(Ne, je, ut, kt) {
                  if (kt && ut) {
                    var Jt = ut.min, fn = ut.mid, Sn = ut.max, Hn = kt[Jt], nr = kt[fn], kr = kt[Sn];
                    if (Hn && nr && kr) for (var Ir = Ne; Ir < je; Ir++) this.zoomInPaintVertexArray.emplace(Ir, nr.tl[0], nr.tl[1], nr.br[0], nr.br[1], Hn.tl[0], Hn.tl[1], Hn.br[0], Hn.br[1], nr.pixelRatio, Hn.pixelRatio), this.zoomOutPaintVertexArray.emplace(Ir, nr.tl[0], nr.tl[1], nr.br[0], nr.br[1], kr.tl[0], kr.tl[1], kr.br[0], kr.br[1], nr.pixelRatio, kr.pixelRatio);
                  }
                }, Bl.prototype.upload = function(Ne) {
                  this.zoomInPaintVertexArray && this.zoomInPaintVertexArray.arrayBuffer && this.zoomOutPaintVertexArray && this.zoomOutPaintVertexArray.arrayBuffer && (this.zoomInPaintVertexBuffer = Ne.createVertexBuffer(this.zoomInPaintVertexArray, ya.members, this.expression.isStateDependent), this.zoomOutPaintVertexBuffer = Ne.createVertexBuffer(this.zoomOutPaintVertexArray, ya.members, this.expression.isStateDependent));
                }, Bl.prototype.destroy = function() {
                  this.zoomOutPaintVertexBuffer && this.zoomOutPaintVertexBuffer.destroy(), this.zoomInPaintVertexBuffer && this.zoomInPaintVertexBuffer.destroy();
                };
                var al = function(Ne, je, ut) {
                  this.binders = {}, this._buffers = [];
                  var kt = [];
                  for (var Jt in Ne.paint._values) if (ut(Jt)) {
                    var fn = Ne.paint.get(Jt);
                    if (fn instanceof nc && So(fn.property.specification)) {
                      var Sn = nu(Jt, Ne.type), Hn = fn.value, nr = fn.property.specification.type, kr = fn.property.useIntegerZoom, Ir = fn.property.specification["property-type"], qr = Ir === "cross-faded" || Ir === "cross-faded-data-driven";
                      if (Hn.kind === "constant") this.binders[Jt] = qr ? new pc(Hn.value, Sn) : new Ml(Hn.value, Sn, nr), kt.push("/u_" + Jt);
                      else if (Hn.kind === "source" || qr) {
                        var ci = Ic(Jt, nr, "source");
                        this.binders[Jt] = qr ? new Bl(Hn, nr, kr, je, ci, Ne.id) : new hl(Hn, Sn, nr, ci), kt.push("/a_" + Jt);
                      } else {
                        var ni = Ic(Jt, nr, "composite");
                        this.binders[Jt] = new xl(Hn, Sn, nr, kr, je, ni), kt.push("/z_" + Jt);
                      }
                    }
                  }
                  this.cacheKey = kt.sort().join("");
                };
                al.prototype.getMaxValue = function(Ne) {
                  var je = this.binders[Ne];
                  return je instanceof hl || je instanceof xl ? je.maxValue : 0;
                }, al.prototype.populatePaintArrays = function(Ne, je, ut, kt, Jt) {
                  for (var fn in this.binders) {
                    var Sn = this.binders[fn];
                    (Sn instanceof hl || Sn instanceof xl || Sn instanceof Bl) && Sn.populatePaintArray(Ne, je, ut, kt, Jt);
                  }
                }, al.prototype.setConstantPatternPositions = function(Ne, je) {
                  for (var ut in this.binders) {
                    var kt = this.binders[ut];
                    kt instanceof pc && kt.setConstantPatternPositions(Ne, je);
                  }
                }, al.prototype.updatePaintArrays = function(Ne, je, ut, kt, Jt) {
                  var fn = !1;
                  for (var Sn in Ne) for (var Hn = 0, nr = je.getPositions(Sn); Hn < nr.length; Hn += 1) {
                    var kr = nr[Hn], Ir = ut.feature(kr.index);
                    for (var qr in this.binders) {
                      var ci = this.binders[qr];
                      if ((ci instanceof hl || ci instanceof xl || ci instanceof Bl) && ci.expression.isStateDependent === !0) {
                        var ni = kt.paint.get(qr);
                        ci.expression = ni.value, ci.updatePaintArray(kr.start, kr.end, Ir, Ne[Sn], Jt), fn = !0;
                      }
                    }
                  }
                  return fn;
                }, al.prototype.defines = function() {
                  var Ne = [];
                  for (var je in this.binders) {
                    var ut = this.binders[je];
                    (ut instanceof Ml || ut instanceof pc) && Ne.push.apply(Ne, ut.uniformNames.map(function(kt) {
                      return "#define HAS_UNIFORM_" + kt;
                    }));
                  }
                  return Ne;
                }, al.prototype.getBinderAttributes = function() {
                  var Ne = [];
                  for (var je in this.binders) {
                    var ut = this.binders[je];
                    if (ut instanceof hl || ut instanceof xl) for (var kt = 0; kt < ut.paintVertexAttributes.length; kt++) Ne.push(ut.paintVertexAttributes[kt].name);
                    else if (ut instanceof Bl) for (var Jt = 0; Jt < ya.members.length; Jt++) Ne.push(ya.members[Jt].name);
                  }
                  return Ne;
                }, al.prototype.getBinderUniforms = function() {
                  var Ne = [];
                  for (var je in this.binders) {
                    var ut = this.binders[je];
                    if (ut instanceof Ml || ut instanceof pc || ut instanceof xl) for (var kt = 0, Jt = ut.uniformNames; kt < Jt.length; kt += 1) {
                      var fn = Jt[kt];
                      Ne.push(fn);
                    }
                  }
                  return Ne;
                }, al.prototype.getPaintVertexBuffers = function() {
                  return this._buffers;
                }, al.prototype.getUniforms = function(Ne, je) {
                  var ut = [];
                  for (var kt in this.binders) {
                    var Jt = this.binders[kt];
                    if (Jt instanceof Ml || Jt instanceof pc || Jt instanceof xl) for (var fn = 0, Sn = Jt.uniformNames; fn < Sn.length; fn += 1) {
                      var Hn = Sn[fn];
                      if (je[Hn]) {
                        var nr = Jt.getBinding(Ne, je[Hn], Hn);
                        ut.push({ name: Hn, property: kt, binding: nr });
                      }
                    }
                  }
                  return ut;
                }, al.prototype.setUniforms = function(Ne, je, ut, kt) {
                  for (var Jt = 0, fn = je; Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt], Hn = Sn.name, nr = Sn.property, kr = Sn.binding;
                    this.binders[nr].setUniform(kr, kt, ut.get(nr), Hn);
                  }
                }, al.prototype.updatePaintBuffers = function(Ne) {
                  for (var je in this._buffers = [], this.binders) {
                    var ut = this.binders[je];
                    if (Ne && ut instanceof Bl) {
                      var kt = Ne.fromScale === 2 ? ut.zoomInPaintVertexBuffer : ut.zoomOutPaintVertexBuffer;
                      kt && this._buffers.push(kt);
                    } else (ut instanceof hl || ut instanceof xl) && ut.paintVertexBuffer && this._buffers.push(ut.paintVertexBuffer);
                  }
                }, al.prototype.upload = function(Ne) {
                  for (var je in this.binders) {
                    var ut = this.binders[je];
                    (ut instanceof hl || ut instanceof xl || ut instanceof Bl) && ut.upload(Ne);
                  }
                  this.updatePaintBuffers();
                }, al.prototype.destroy = function() {
                  for (var Ne in this.binders) {
                    var je = this.binders[Ne];
                    (je instanceof hl || je instanceof xl || je instanceof Bl) && je.destroy();
                  }
                };
                var Oc = function(Ne, je, ut) {
                  ut === void 0 && (ut = function() {
                    return !0;
                  }), this.programConfigurations = {};
                  for (var kt = 0, Jt = Ne; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    this.programConfigurations[fn.id] = new al(fn, je, ut);
                  }
                  this.needsUpload = !1, this._featureMap = new Ya(), this._bufferOffset = 0;
                };
                function nu(Ne, je) {
                  return { "text-opacity": ["opacity"], "icon-opacity": ["opacity"], "text-color": ["fill_color"], "icon-color": ["fill_color"], "text-halo-color": ["halo_color"], "icon-halo-color": ["halo_color"], "text-halo-blur": ["halo_blur"], "icon-halo-blur": ["halo_blur"], "text-halo-width": ["halo_width"], "icon-halo-width": ["halo_width"], "line-gap-width": ["gapwidth"], "line-pattern": ["pattern_to", "pattern_from", "pixel_ratio_to", "pixel_ratio_from"], "fill-pattern": ["pattern_to", "pattern_from", "pixel_ratio_to", "pixel_ratio_from"], "fill-extrusion-pattern": ["pattern_to", "pattern_from", "pixel_ratio_to", "pixel_ratio_from"] }[Ne] || [Ne.replace(je + "-", "").replace(/-/g, "_")];
                }
                function Ic(Ne, je, ut) {
                  var kt = { color: { source: Gn, composite: Bn }, number: { source: qt, composite: Gn } }, Jt = function(fn) {
                    return { "line-pattern": { source: Ke, composite: Ke }, "fill-pattern": { source: Ke, composite: Ke }, "fill-extrusion-pattern": { source: Ke, composite: Ke } }[fn];
                  }(Ne);
                  return Jt && Jt[ut] || kt[je][ut];
                }
                Oc.prototype.populatePaintArrays = function(Ne, je, ut, kt, Jt, fn) {
                  for (var Sn in this.programConfigurations) this.programConfigurations[Sn].populatePaintArrays(Ne, je, kt, Jt, fn);
                  je.id !== void 0 && this._featureMap.add(je.id, ut, this._bufferOffset, Ne), this._bufferOffset = Ne, this.needsUpload = !0;
                }, Oc.prototype.updatePaintArrays = function(Ne, je, ut, kt) {
                  for (var Jt = 0, fn = ut; Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt];
                    this.needsUpload = this.programConfigurations[Sn.id].updatePaintArrays(Ne, this._featureMap, je, Sn, kt) || this.needsUpload;
                  }
                }, Oc.prototype.get = function(Ne) {
                  return this.programConfigurations[Ne];
                }, Oc.prototype.upload = function(Ne) {
                  if (this.needsUpload) {
                    for (var je in this.programConfigurations) this.programConfigurations[je].upload(Ne);
                    this.needsUpload = !1;
                  }
                }, Oc.prototype.destroy = function() {
                  for (var Ne in this.programConfigurations) this.programConfigurations[Ne].destroy();
                }, cs("ConstantBinder", Ml), cs("CrossFadedConstantBinder", pc), cs("SourceExpressionBinder", hl), cs("CrossFadedCompositeBinder", Bl), cs("CompositeExpressionBinder", xl), cs("ProgramConfiguration", al, { omit: ["_buffers"] }), cs("ProgramConfigurationSet", Oc);
                var $s = 8192, Xc = Math.pow(2, 14) - 1, wc = -Xc - 1;
                function zu(Ne) {
                  for (var je = $s / Ne.extent, ut = Ne.loadGeometry(), kt = 0; kt < ut.length; kt++) for (var Jt = ut[kt], fn = 0; fn < Jt.length; fn++) {
                    var Sn = Jt[fn], Hn = Math.round(Sn.x * je), nr = Math.round(Sn.y * je);
                    Sn.x = pe(Hn, wc, Xc), Sn.y = pe(nr, wc, Xc), (Hn < Sn.x || Hn > Sn.x + 1 || nr < Sn.y || nr > Sn.y + 1) && Me("Geometry exceeds allowed extent, reduce your vector tile buffer size");
                  }
                  return ut;
                }
                function Zc(Ne, je) {
                  return { type: Ne.type, id: Ne.id, properties: Ne.properties, geometry: je ? zu(Ne) : [] };
                }
                function Ju(Ne, je, ut, kt, Jt) {
                  Ne.emplaceBack(2 * je + (kt + 1) / 2, 2 * ut + (Jt + 1) / 2);
                }
                var ru = function(Ne) {
                  this.zoom = Ne.zoom, this.overscaling = Ne.overscaling, this.layers = Ne.layers, this.layerIds = this.layers.map(function(je) {
                    return je.id;
                  }), this.index = Ne.index, this.hasPattern = !1, this.layoutVertexArray = new tu(), this.indexArray = new ht(), this.segments = new ra(), this.programConfigurations = new Oc(Ne.layers, Ne.zoom), this.stateDependentLayerIds = this.layers.filter(function(je) {
                    return je.isStateDependent();
                  }).map(function(je) {
                    return je.id;
                  });
                };
                function Bd(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) if (S1(je, Ne[ut])) return !0;
                  for (var kt = 0; kt < je.length; kt++) if (S1(Ne, je[kt])) return !0;
                  return !!yd(Ne, je);
                }
                function tf(Ne, je, ut) {
                  return !!S1(Ne, je) || !!Md(je, Ne, ut);
                }
                function wd(Ne, je) {
                  if (Ne.length === 1) return Vd(je, Ne[0]);
                  for (var ut = 0; ut < je.length; ut++) for (var kt = je[ut], Jt = 0; Jt < kt.length; Jt++) if (S1(Ne, kt[Jt])) return !0;
                  for (var fn = 0; fn < Ne.length; fn++) if (Vd(je, Ne[fn])) return !0;
                  for (var Sn = 0; Sn < je.length; Sn++) if (yd(Ne, je[Sn])) return !0;
                  return !1;
                }
                function od(Ne, je, ut) {
                  if (Ne.length > 1) {
                    if (yd(Ne, je)) return !0;
                    for (var kt = 0; kt < je.length; kt++) if (Md(je[kt], Ne, ut)) return !0;
                  }
                  for (var Jt = 0; Jt < Ne.length; Jt++) if (Md(Ne[Jt], je, ut)) return !0;
                  return !1;
                }
                function yd(Ne, je) {
                  if (Ne.length === 0 || je.length === 0) return !1;
                  for (var ut = 0; ut < Ne.length - 1; ut++) for (var kt = Ne[ut], Jt = Ne[ut + 1], fn = 0; fn < je.length - 1; fn++) if (Ef(kt, Jt, je[fn], je[fn + 1])) return !0;
                  return !1;
                }
                function Ef(Ne, je, ut, kt) {
                  return Re(Ne, ut, kt) !== Re(je, ut, kt) && Re(Ne, je, ut) !== Re(Ne, je, kt);
                }
                function Md(Ne, je, ut) {
                  var kt = ut * ut;
                  if (je.length === 1) return Ne.distSqr(je[0]) < kt;
                  for (var Jt = 1; Jt < je.length; Jt++) if (nf(Ne, je[Jt - 1], je[Jt]) < kt) return !0;
                  return !1;
                }
                function nf(Ne, je, ut) {
                  var kt = je.distSqr(ut);
                  if (kt === 0) return Ne.distSqr(je);
                  var Jt = ((Ne.x - je.x) * (ut.x - je.x) + (Ne.y - je.y) * (ut.y - je.y)) / kt;
                  return Jt < 0 ? Ne.distSqr(je) : Jt > 1 ? Ne.distSqr(ut) : Ne.distSqr(ut.sub(je)._mult(Jt)._add(je));
                }
                function Vd(Ne, je) {
                  for (var ut, kt, Jt, fn = !1, Sn = 0; Sn < Ne.length; Sn++) for (var Hn = 0, nr = (ut = Ne[Sn]).length - 1; Hn < ut.length; nr = Hn++) kt = ut[Hn], Jt = ut[nr], kt.y > je.y != Jt.y > je.y && je.x < (Jt.x - kt.x) * (je.y - kt.y) / (Jt.y - kt.y) + kt.x && (fn = !fn);
                  return fn;
                }
                function S1(Ne, je) {
                  for (var ut = !1, kt = 0, Jt = Ne.length - 1; kt < Ne.length; Jt = kt++) {
                    var fn = Ne[kt], Sn = Ne[Jt];
                    fn.y > je.y != Sn.y > je.y && je.x < (Sn.x - fn.x) * (je.y - fn.y) / (Sn.y - fn.y) + fn.x && (ut = !ut);
                  }
                  return ut;
                }
                function Af(Ne, je, ut) {
                  var kt = ut[0], Jt = ut[2];
                  if (Ne.x < kt.x && je.x < kt.x || Ne.x > Jt.x && je.x > Jt.x || Ne.y < kt.y && je.y < kt.y || Ne.y > Jt.y && je.y > Jt.y) return !1;
                  var fn = Re(Ne, je, ut[0]);
                  return fn !== Re(Ne, je, ut[1]) || fn !== Re(Ne, je, ut[2]) || fn !== Re(Ne, je, ut[3]);
                }
                function Pd(Ne, je, ut) {
                  var kt = je.paint.get(Ne).value;
                  return kt.kind === "constant" ? kt.value : ut.programConfigurations.get(je.id).getMaxValue(Ne);
                }
                function Au(Ne) {
                  return Math.sqrt(Ne[0] * Ne[0] + Ne[1] * Ne[1]);
                }
                function Vu(Ne, je, ut, kt, Jt) {
                  if (!je[0] && !je[1]) return Ne;
                  var fn = le.convert(je)._mult(Jt);
                  ut === "viewport" && fn._rotate(-kt);
                  for (var Sn = [], Hn = 0; Hn < Ne.length; Hn++) {
                    var nr = Ne[Hn];
                    Sn.push(nr.sub(fn));
                  }
                  return Sn;
                }
                ru.prototype.populate = function(Ne, je, ut) {
                  var kt = this.layers[0], Jt = [], fn = null;
                  kt.type === "circle" && (fn = kt.layout.get("circle-sort-key"));
                  for (var Sn = 0, Hn = Ne; Sn < Hn.length; Sn += 1) {
                    var nr = Hn[Sn], kr = nr.feature, Ir = nr.id, qr = nr.index, ci = nr.sourceLayerIndex, ni = this.layers[0]._featureFilter.needGeometry, Ii = Zc(kr, ni);
                    if (this.layers[0]._featureFilter.filter(new El(this.zoom), Ii, ut)) {
                      var Gi = fn ? fn.evaluate(Ii, {}, ut) : void 0, xa = { id: Ir, properties: kr.properties, type: kr.type, sourceLayerIndex: ci, index: qr, geometry: ni ? Ii.geometry : zu(kr), patterns: {}, sortKey: Gi };
                      Jt.push(xa);
                    }
                  }
                  fn && Jt.sort(function(As, Fs) {
                    return As.sortKey - Fs.sortKey;
                  });
                  for (var ga = 0, $a = Jt; ga < $a.length; ga += 1) {
                    var Va = $a[ga], eo = Va, co = eo.geometry, wo = eo.index, ds = eo.sourceLayerIndex, Lo = Ne[wo].feature;
                    this.addFeature(Va, co, wo, ut), je.featureIndex.insert(Lo, co, wo, ds, this.index);
                  }
                }, ru.prototype.update = function(Ne, je, ut) {
                  this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(Ne, je, this.stateDependentLayers, ut);
                }, ru.prototype.isEmpty = function() {
                  return this.layoutVertexArray.length === 0;
                }, ru.prototype.uploadPending = function() {
                  return !this.uploaded || this.programConfigurations.needsUpload;
                }, ru.prototype.upload = function(Ne) {
                  this.uploaded || (this.layoutVertexBuffer = Ne.createVertexBuffer(this.layoutVertexArray, Yi), this.indexBuffer = Ne.createIndexBuffer(this.indexArray)), this.programConfigurations.upload(Ne), this.uploaded = !0;
                }, ru.prototype.destroy = function() {
                  this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy());
                }, ru.prototype.addFeature = function(Ne, je, ut, kt) {
                  for (var Jt = 0, fn = je; Jt < fn.length; Jt += 1) for (var Sn = 0, Hn = fn[Jt]; Sn < Hn.length; Sn += 1) {
                    var nr = Hn[Sn], kr = nr.x, Ir = nr.y;
                    if (!(kr < 0 || kr >= $s || Ir < 0 || Ir >= $s)) {
                      var qr = this.segments.prepareSegment(4, this.layoutVertexArray, this.indexArray, Ne.sortKey), ci = qr.vertexLength;
                      Ju(this.layoutVertexArray, kr, Ir, -1, -1), Ju(this.layoutVertexArray, kr, Ir, 1, -1), Ju(this.layoutVertexArray, kr, Ir, 1, 1), Ju(this.layoutVertexArray, kr, Ir, -1, 1), this.indexArray.emplaceBack(ci, ci + 1, ci + 2), this.indexArray.emplaceBack(ci, ci + 3, ci + 2), qr.vertexLength += 4, qr.primitiveLength += 2;
                    }
                  }
                  this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, Ne, ut, {}, kt);
                }, cs("CircleBucket", ru, { omit: ["layers"] });
                var Yh = new $c({ "circle-sort-key": new qs(Zt.layout_circle["circle-sort-key"]) }), Qf = { paint: new $c({ "circle-radius": new qs(Zt.paint_circle["circle-radius"]), "circle-color": new qs(Zt.paint_circle["circle-color"]), "circle-blur": new qs(Zt.paint_circle["circle-blur"]), "circle-opacity": new qs(Zt.paint_circle["circle-opacity"]), "circle-translate": new ks(Zt.paint_circle["circle-translate"]), "circle-translate-anchor": new ks(Zt.paint_circle["circle-translate-anchor"]), "circle-pitch-scale": new ks(Zt.paint_circle["circle-pitch-scale"]), "circle-pitch-alignment": new ks(Zt.paint_circle["circle-pitch-alignment"]), "circle-stroke-width": new qs(Zt.paint_circle["circle-stroke-width"]), "circle-stroke-color": new qs(Zt.paint_circle["circle-stroke-color"]), "circle-stroke-opacity": new qs(Zt.paint_circle["circle-stroke-opacity"]) }), layout: Yh }, vd = typeof Float32Array < "u" ? Float32Array : Array;
                function Ff(Ne) {
                  return Ne[0] = 1, Ne[1] = 0, Ne[2] = 0, Ne[3] = 0, Ne[4] = 0, Ne[5] = 1, Ne[6] = 0, Ne[7] = 0, Ne[8] = 0, Ne[9] = 0, Ne[10] = 1, Ne[11] = 0, Ne[12] = 0, Ne[13] = 0, Ne[14] = 0, Ne[15] = 1, Ne;
                }
                function Uu(Ne, je, ut) {
                  var kt = je[0], Jt = je[1], fn = je[2], Sn = je[3], Hn = je[4], nr = je[5], kr = je[6], Ir = je[7], qr = je[8], ci = je[9], ni = je[10], Ii = je[11], Gi = je[12], xa = je[13], ga = je[14], $a = je[15], Va = ut[0], eo = ut[1], co = ut[2], wo = ut[3];
                  return Ne[0] = Va * kt + eo * Hn + co * qr + wo * Gi, Ne[1] = Va * Jt + eo * nr + co * ci + wo * xa, Ne[2] = Va * fn + eo * kr + co * ni + wo * ga, Ne[3] = Va * Sn + eo * Ir + co * Ii + wo * $a, Va = ut[4], eo = ut[5], co = ut[6], wo = ut[7], Ne[4] = Va * kt + eo * Hn + co * qr + wo * Gi, Ne[5] = Va * Jt + eo * nr + co * ci + wo * xa, Ne[6] = Va * fn + eo * kr + co * ni + wo * ga, Ne[7] = Va * Sn + eo * Ir + co * Ii + wo * $a, Va = ut[8], eo = ut[9], co = ut[10], wo = ut[11], Ne[8] = Va * kt + eo * Hn + co * qr + wo * Gi, Ne[9] = Va * Jt + eo * nr + co * ci + wo * xa, Ne[10] = Va * fn + eo * kr + co * ni + wo * ga, Ne[11] = Va * Sn + eo * Ir + co * Ii + wo * $a, Va = ut[12], eo = ut[13], co = ut[14], wo = ut[15], Ne[12] = Va * kt + eo * Hn + co * qr + wo * Gi, Ne[13] = Va * Jt + eo * nr + co * ci + wo * xa, Ne[14] = Va * fn + eo * kr + co * ni + wo * ga, Ne[15] = Va * Sn + eo * Ir + co * Ii + wo * $a, Ne;
                }
                Math.hypot || (Math.hypot = function() {
                  for (var Ne = arguments, je = 0, ut = arguments.length; ut--; ) je += Ne[ut] * Ne[ut];
                  return Math.sqrt(je);
                });
                var rf = Uu, Ud, $d = function(Ne, je, ut) {
                  return Ne[0] = je[0] - ut[0], Ne[1] = je[1] - ut[1], Ne[2] = je[2] - ut[2], Ne;
                };
                function Sd(Ne, je, ut) {
                  var kt = je[0], Jt = je[1], fn = je[2], Sn = je[3];
                  return Ne[0] = ut[0] * kt + ut[4] * Jt + ut[8] * fn + ut[12] * Sn, Ne[1] = ut[1] * kt + ut[5] * Jt + ut[9] * fn + ut[13] * Sn, Ne[2] = ut[2] * kt + ut[6] * Jt + ut[10] * fn + ut[14] * Sn, Ne[3] = ut[3] * kt + ut[7] * Jt + ut[11] * fn + ut[15] * Sn, Ne;
                }
                Ud = new vd(3), vd != Float32Array && (Ud[0] = 0, Ud[1] = 0, Ud[2] = 0), function() {
                  var Ne = new vd(4);
                  vd != Float32Array && (Ne[0] = 0, Ne[1] = 0, Ne[2] = 0, Ne[3] = 0);
                }();
                var Yd = function(Ne) {
                  var je = Ne[0], ut = Ne[1];
                  return je * je + ut * ut;
                }, af = (function() {
                  var Ne = new vd(2);
                  vd != Float32Array && (Ne[0] = 0, Ne[1] = 0);
                }(), function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, Qf);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.createBucket = function(ut) {
                    return new ru(ut);
                  }, je.prototype.queryRadius = function(ut) {
                    var kt = ut;
                    return Pd("circle-radius", this, kt) + Pd("circle-stroke-width", this, kt) + Au(this.paint.get("circle-translate"));
                  }, je.prototype.queryIntersectsFeature = function(ut, kt, Jt, fn, Sn, Hn, nr, kr) {
                    for (var Ir = Vu(ut, this.paint.get("circle-translate"), this.paint.get("circle-translate-anchor"), Hn.angle, nr), qr = this.paint.get("circle-radius").evaluate(kt, Jt) + this.paint.get("circle-stroke-width").evaluate(kt, Jt), ci = this.paint.get("circle-pitch-alignment") === "map", ni = ci ? Ir : function(ds, Lo) {
                      return ds.map(function(As) {
                        return of(As, Lo);
                      });
                    }(Ir, kr), Ii = ci ? qr * nr : qr, Gi = 0, xa = fn; Gi < xa.length; Gi += 1) for (var ga = 0, $a = xa[Gi]; ga < $a.length; ga += 1) {
                      var Va = $a[ga], eo = ci ? Va : of(Va, kr), co = Ii, wo = Sd([], [Va.x, Va.y, 0, 1], kr);
                      if (this.paint.get("circle-pitch-scale") === "viewport" && this.paint.get("circle-pitch-alignment") === "map" ? co *= wo[3] / Hn.cameraToCenterDistance : this.paint.get("circle-pitch-scale") === "map" && this.paint.get("circle-pitch-alignment") === "viewport" && (co *= Hn.cameraToCenterDistance / wo[3]), tf(ni, eo, co)) return !0;
                    }
                    return !1;
                  }, je;
                }(Ul));
                function of(Ne, je) {
                  var ut = Sd([], [Ne.x, Ne.y, 0, 1], je);
                  return new le(ut[0] / ut[3], ut[1] / ut[3]);
                }
                var Bf = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je;
                }(ru);
                function qd(Ne, je, ut, kt) {
                  var Jt = je.width, fn = je.height;
                  if (kt) {
                    if (kt instanceof Uint8ClampedArray) kt = new Uint8Array(kt.buffer);
                    else if (kt.length !== Jt * fn * ut) throw new RangeError("mismatched image size");
                  } else kt = new Uint8Array(Jt * fn * ut);
                  return Ne.width = Jt, Ne.height = fn, Ne.data = kt, Ne;
                }
                function Cf(Ne, je, ut) {
                  var kt = je.width, Jt = je.height;
                  if (kt !== Ne.width || Jt !== Ne.height) {
                    var fn = qd({}, { width: kt, height: Jt }, ut);
                    V1(Ne, fn, { x: 0, y: 0 }, { x: 0, y: 0 }, { width: Math.min(Ne.width, kt), height: Math.min(Ne.height, Jt) }, ut), Ne.width = kt, Ne.height = Jt, Ne.data = fn.data;
                  }
                }
                function V1(Ne, je, ut, kt, Jt, fn) {
                  if (Jt.width === 0 || Jt.height === 0) return je;
                  if (Jt.width > Ne.width || Jt.height > Ne.height || ut.x > Ne.width - Jt.width || ut.y > Ne.height - Jt.height) throw new RangeError("out of range source coordinates for image copy");
                  if (Jt.width > je.width || Jt.height > je.height || kt.x > je.width - Jt.width || kt.y > je.height - Jt.height) throw new RangeError("out of range destination coordinates for image copy");
                  for (var Sn = Ne.data, Hn = je.data, nr = 0; nr < Jt.height; nr++) for (var kr = ((ut.y + nr) * Ne.width + ut.x) * fn, Ir = ((kt.y + nr) * je.width + kt.x) * fn, qr = 0; qr < Jt.width * fn; qr++) Hn[Ir + qr] = Sn[kr + qr];
                  return je;
                }
                cs("HeatmapBucket", Bf, { omit: ["layers"] });
                var jd = function(Ne, je) {
                  qd(this, Ne, 1, je);
                };
                jd.prototype.resize = function(Ne) {
                  Cf(this, Ne, 1);
                }, jd.prototype.clone = function() {
                  return new jd({ width: this.width, height: this.height }, new Uint8Array(this.data));
                }, jd.copy = function(Ne, je, ut, kt, Jt) {
                  V1(Ne, je, ut, kt, Jt, 1);
                };
                var E1 = function(Ne, je) {
                  qd(this, Ne, 4, je);
                };
                E1.prototype.resize = function(Ne) {
                  Cf(this, Ne, 4);
                }, E1.prototype.replace = function(Ne, je) {
                  je ? this.data.set(Ne) : Ne instanceof Uint8ClampedArray ? this.data = new Uint8Array(Ne.buffer) : this.data = Ne;
                }, E1.prototype.clone = function() {
                  return new E1({ width: this.width, height: this.height }, new Uint8Array(this.data));
                }, E1.copy = function(Ne, je, ut, kt, Jt) {
                  V1(Ne, je, ut, kt, Jt, 4);
                }, cs("AlphaImage", jd), cs("RGBAImage", E1);
                var sd = { paint: new $c({ "heatmap-radius": new qs(Zt.paint_heatmap["heatmap-radius"]), "heatmap-weight": new qs(Zt.paint_heatmap["heatmap-weight"]), "heatmap-intensity": new ks(Zt.paint_heatmap["heatmap-intensity"]), "heatmap-color": new o1(Zt.paint_heatmap["heatmap-color"]), "heatmap-opacity": new ks(Zt.paint_heatmap["heatmap-opacity"]) }) };
                function Xd(Ne) {
                  var je = {}, ut = Ne.resolution || 256, kt = Ne.clips ? Ne.clips.length : 1, Jt = Ne.image || new E1({ width: ut, height: kt }), fn = function(Ii, Gi, xa) {
                    je[Ne.evaluationKey] = xa;
                    var ga = Ne.expression.evaluate(je);
                    Jt.data[Ii + Gi + 0] = Math.floor(255 * ga.r / ga.a), Jt.data[Ii + Gi + 1] = Math.floor(255 * ga.g / ga.a), Jt.data[Ii + Gi + 2] = Math.floor(255 * ga.b / ga.a), Jt.data[Ii + Gi + 3] = Math.floor(255 * ga.a);
                  };
                  if (Ne.clips) for (var Sn = 0, Hn = 0; Sn < kt; ++Sn, Hn += 4 * ut) for (var nr = 0, kr = 0; nr < ut; nr++, kr += 4) {
                    var Ir = nr / (ut - 1), qr = Ne.clips[Sn];
                    fn(Hn, kr, qr.start * (1 - Ir) + qr.end * Ir);
                  }
                  else for (var ci = 0, ni = 0; ci < ut; ci++, ni += 4) fn(0, ni, ci / (ut - 1));
                  return Jt;
                }
                var ld = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, sd), this._updateColorRamp();
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.createBucket = function(ut) {
                    return new Bf(ut);
                  }, je.prototype._handleSpecialPaintPropertyUpdate = function(ut) {
                    ut === "heatmap-color" && this._updateColorRamp();
                  }, je.prototype._updateColorRamp = function() {
                    var ut = this._transitionablePaint._values["heatmap-color"].value.expression;
                    this.colorRamp = Xd({ expression: ut, evaluationKey: "heatmapDensity", image: this.colorRamp }), this.colorRampTexture = null;
                  }, je.prototype.resize = function() {
                    this.heatmapFbo && (this.heatmapFbo.destroy(), this.heatmapFbo = null);
                  }, je.prototype.queryRadius = function() {
                    return 0;
                  }, je.prototype.queryIntersectsFeature = function() {
                    return !1;
                  }, je.prototype.hasOffscreenPass = function() {
                    return this.paint.get("heatmap-opacity") !== 0 && this.visibility !== "none";
                  }, je;
                }(Ul), Ch = { paint: new $c({ "hillshade-illumination-direction": new ks(Zt.paint_hillshade["hillshade-illumination-direction"]), "hillshade-illumination-anchor": new ks(Zt.paint_hillshade["hillshade-illumination-anchor"]), "hillshade-exaggeration": new ks(Zt.paint_hillshade["hillshade-exaggeration"]), "hillshade-shadow-color": new ks(Zt.paint_hillshade["hillshade-shadow-color"]), "hillshade-highlight-color": new ks(Zt.paint_hillshade["hillshade-highlight-color"]), "hillshade-accent-color": new ks(Zt.paint_hillshade["hillshade-accent-color"]) }) }, sf = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, Ch);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.hasOffscreenPass = function() {
                    return this.paint.get("hillshade-exaggeration") !== 0 && this.visibility !== "none";
                  }, je;
                }(Ul), Zd = xc([{ name: "a_pos", components: 2, type: "Int16" }], 4).members, Lf = Xf, uh = Xf;
                function Xf(Ne, je, ut) {
                  ut = ut || 2;
                  var kt, Jt, fn, Sn, Hn, nr, kr, Ir = je && je.length, qr = Ir ? je[0] * ut : Ne.length, ci = Lh(Ne, 0, qr, ut, !0), ni = [];
                  if (!ci || ci.next === ci.prev) return ni;
                  if (Ir && (ci = function(Gi, xa, ga, $a) {
                    var Va, eo, co, wo = [];
                    for (Va = 0, eo = xa.length; Va < eo; Va++) (co = Lh(Gi, xa[Va] * $a, Va < eo - 1 ? xa[Va + 1] * $a : Gi.length, $a, !1)) === co.next && (co.steiner = !0), wo.push(Hf(co));
                    for (wo.sort(Rh), Va = 0; Va < wo.length; Va++) Zf(wo[Va], ga), ga = p1(ga, ga.next);
                    return ga;
                  }(Ne, je, ci, ut)), Ne.length > 80 * ut) {
                    kt = fn = Ne[0], Jt = Sn = Ne[1];
                    for (var Ii = ut; Ii < qr; Ii += ut) (Hn = Ne[Ii]) < kt && (kt = Hn), (nr = Ne[Ii + 1]) < Jt && (Jt = nr), Hn > fn && (fn = Hn), nr > Sn && (Sn = nr);
                    kr = (kr = Math.max(fn - kt, Sn - Jt)) !== 0 ? 1 / kr : 0;
                  }
                  return cd(ci, ni, ut, kt, Jt, kr), ni;
                }
                function Lh(Ne, je, ut, kt, Jt) {
                  var fn, Sn;
                  if (Jt === Ih(Ne, je, ut, kt) > 0) for (fn = je; fn < ut; fn += kt) Sn = gf(fn, Ne[fn], Ne[fn + 1], Sn);
                  else for (fn = ut - kt; fn >= je; fn -= kt) Sn = gf(fn, Ne[fn], Ne[fn + 1], Sn);
                  return Sn && kf(Sn, Sn.next) && (lf(Sn), Sn = Sn.next), Sn;
                }
                function p1(Ne, je) {
                  if (!Ne) return Ne;
                  je || (je = Ne);
                  var ut, kt = Ne;
                  do
                    if (ut = !1, kt.steiner || !kf(kt, kt.next) && Yu(kt.prev, kt, kt.next) !== 0) kt = kt.next;
                    else {
                      if (lf(kt), (kt = je = kt.prev) === kt.next) break;
                      ut = !0;
                    }
                  while (ut || kt !== je);
                  return je;
                }
                function cd(Ne, je, ut, kt, Jt, fn, Sn) {
                  if (Ne) {
                    !Sn && fn && function(Ir, qr, ci, ni) {
                      var Ii = Ir;
                      do
                        Ii.z === null && (Ii.z = ed(Ii.x, Ii.y, qr, ci, ni)), Ii.prevZ = Ii.prev, Ii.nextZ = Ii.next, Ii = Ii.next;
                      while (Ii !== Ir);
                      Ii.prevZ.nextZ = null, Ii.prevZ = null, function(Gi) {
                        var xa, ga, $a, Va, eo, co, wo, ds, Lo = 1;
                        do {
                          for (ga = Gi, Gi = null, eo = null, co = 0; ga; ) {
                            for (co++, $a = ga, wo = 0, xa = 0; xa < Lo && (wo++, $a = $a.nextZ); xa++) ;
                            for (ds = Lo; wo > 0 || ds > 0 && $a; ) wo !== 0 && (ds === 0 || !$a || ga.z <= $a.z) ? (Va = ga, ga = ga.nextZ, wo--) : (Va = $a, $a = $a.nextZ, ds--), eo ? eo.nextZ = Va : Gi = Va, Va.prevZ = eo, eo = Va;
                            ga = $a;
                          }
                          eo.nextZ = null, Lo *= 2;
                        } while (co > 1);
                      }(Ii);
                    }(Ne, kt, Jt, fn);
                    for (var Hn, nr, kr = Ne; Ne.prev !== Ne.next; ) if (Hn = Ne.prev, nr = Ne.next, fn ? kh(Ne, kt, Jt, fn) : dh(Ne)) je.push(Hn.i / ut), je.push(Ne.i / ut), je.push(nr.i / ut), lf(Ne), Ne = nr.next, kr = nr.next;
                    else if ((Ne = nr) === kr) {
                      Sn ? Sn === 1 ? cd(Ne = Mh(p1(Ne), je, ut), je, ut, kt, Jt, fn, 2) : Sn === 2 && h2(Ne, je, ut, kt, Jt, fn) : cd(p1(Ne), je, ut, kt, Jt, fn, 1);
                      break;
                    }
                  }
                }
                function dh(Ne) {
                  var je = Ne.prev, ut = Ne, kt = Ne.next;
                  if (Yu(je, ut, kt) >= 0) return !1;
                  for (var Jt = Ne.next.next; Jt !== Ne.prev; ) {
                    if (Od(je.x, je.y, ut.x, ut.y, kt.x, kt.y, Jt.x, Jt.y) && Yu(Jt.prev, Jt, Jt.next) >= 0) return !1;
                    Jt = Jt.next;
                  }
                  return !0;
                }
                function kh(Ne, je, ut, kt) {
                  var Jt = Ne.prev, fn = Ne, Sn = Ne.next;
                  if (Yu(Jt, fn, Sn) >= 0) return !1;
                  for (var Hn = Jt.x < fn.x ? Jt.x < Sn.x ? Jt.x : Sn.x : fn.x < Sn.x ? fn.x : Sn.x, nr = Jt.y < fn.y ? Jt.y < Sn.y ? Jt.y : Sn.y : fn.y < Sn.y ? fn.y : Sn.y, kr = Jt.x > fn.x ? Jt.x > Sn.x ? Jt.x : Sn.x : fn.x > Sn.x ? fn.x : Sn.x, Ir = Jt.y > fn.y ? Jt.y > Sn.y ? Jt.y : Sn.y : fn.y > Sn.y ? fn.y : Sn.y, qr = ed(Hn, nr, je, ut, kt), ci = ed(kr, Ir, je, ut, kt), ni = Ne.prevZ, Ii = Ne.nextZ; ni && ni.z >= qr && Ii && Ii.z <= ci; ) {
                    if (ni !== Ne.prev && ni !== Ne.next && Od(Jt.x, Jt.y, fn.x, fn.y, Sn.x, Sn.y, ni.x, ni.y) && Yu(ni.prev, ni, ni.next) >= 0 || (ni = ni.prevZ, Ii !== Ne.prev && Ii !== Ne.next && Od(Jt.x, Jt.y, fn.x, fn.y, Sn.x, Sn.y, Ii.x, Ii.y) && Yu(Ii.prev, Ii, Ii.next) >= 0)) return !1;
                    Ii = Ii.nextZ;
                  }
                  for (; ni && ni.z >= qr; ) {
                    if (ni !== Ne.prev && ni !== Ne.next && Od(Jt.x, Jt.y, fn.x, fn.y, Sn.x, Sn.y, ni.x, ni.y) && Yu(ni.prev, ni, ni.next) >= 0) return !1;
                    ni = ni.prevZ;
                  }
                  for (; Ii && Ii.z <= ci; ) {
                    if (Ii !== Ne.prev && Ii !== Ne.next && Od(Jt.x, Jt.y, fn.x, fn.y, Sn.x, Sn.y, Ii.x, Ii.y) && Yu(Ii.prev, Ii, Ii.next) >= 0) return !1;
                    Ii = Ii.nextZ;
                  }
                  return !0;
                }
                function Mh(Ne, je, ut) {
                  var kt = Ne;
                  do {
                    var Jt = kt.prev, fn = kt.next.next;
                    !kf(Jt, fn) && Mf(Jt, kt, kt.next, fn) && cu(Jt, fn) && cu(fn, Jt) && (je.push(Jt.i / ut), je.push(kt.i / ut), je.push(fn.i / ut), lf(kt), lf(kt.next), kt = Ne = fn), kt = kt.next;
                  } while (kt !== Ne);
                  return p1(kt);
                }
                function h2(Ne, je, ut, kt, Jt, fn) {
                  var Sn = Ne;
                  do {
                    for (var Hn = Sn.next.next; Hn !== Sn.prev; ) {
                      if (Sn.i !== Hn.i && Oh(Sn, Hn)) {
                        var nr = pu(Sn, Hn);
                        return Sn = p1(Sn, Sn.next), nr = p1(nr, nr.next), cd(Sn, je, ut, kt, Jt, fn), void cd(nr, je, ut, kt, Jt, fn);
                      }
                      Hn = Hn.next;
                    }
                    Sn = Sn.next;
                  } while (Sn !== Ne);
                }
                function Rh(Ne, je) {
                  return Ne.x - je.x;
                }
                function Zf(Ne, je) {
                  if (je = function(kt, Jt) {
                    var fn, Sn = Jt, Hn = kt.x, nr = kt.y, kr = -1 / 0;
                    do {
                      if (nr <= Sn.y && nr >= Sn.next.y && Sn.next.y !== Sn.y) {
                        var Ir = Sn.x + (nr - Sn.y) * (Sn.next.x - Sn.x) / (Sn.next.y - Sn.y);
                        if (Ir <= Hn && Ir > kr) {
                          if (kr = Ir, Ir === Hn) {
                            if (nr === Sn.y) return Sn;
                            if (nr === Sn.next.y) return Sn.next;
                          }
                          fn = Sn.x < Sn.next.x ? Sn : Sn.next;
                        }
                      }
                      Sn = Sn.next;
                    } while (Sn !== Jt);
                    if (!fn) return null;
                    if (Hn === kr) return fn;
                    var qr, ci = fn, ni = fn.x, Ii = fn.y, Gi = 1 / 0;
                    Sn = fn;
                    do
                      Hn >= Sn.x && Sn.x >= ni && Hn !== Sn.x && Od(nr < Ii ? Hn : kr, nr, ni, Ii, nr < Ii ? kr : Hn, nr, Sn.x, Sn.y) && (qr = Math.abs(nr - Sn.y) / (Hn - Sn.x), cu(Sn, kt) && (qr < Gi || qr === Gi && (Sn.x > fn.x || Sn.x === fn.x && Rd(fn, Sn))) && (fn = Sn, Gi = qr)), Sn = Sn.next;
                    while (Sn !== ci);
                    return fn;
                  }(Ne, je)) {
                    var ut = pu(je, Ne);
                    p1(je, je.next), p1(ut, ut.next);
                  }
                }
                function Rd(Ne, je) {
                  return Yu(Ne.prev, Ne, je.prev) < 0 && Yu(je.next, Ne, Ne.next) < 0;
                }
                function ed(Ne, je, ut, kt, Jt) {
                  return (Ne = 1431655765 & ((Ne = 858993459 & ((Ne = 252645135 & ((Ne = 16711935 & ((Ne = 32767 * (Ne - ut) * Jt) | Ne << 8)) | Ne << 4)) | Ne << 2)) | Ne << 1)) | (je = 1431655765 & ((je = 858993459 & ((je = 252645135 & ((je = 16711935 & ((je = 32767 * (je - kt) * Jt) | je << 8)) | je << 4)) | je << 2)) | je << 1)) << 1;
                }
                function Hf(Ne) {
                  var je = Ne, ut = Ne;
                  do
                    (je.x < ut.x || je.x === ut.x && je.y < ut.y) && (ut = je), je = je.next;
                  while (je !== Ne);
                  return ut;
                }
                function Od(Ne, je, ut, kt, Jt, fn, Sn, Hn) {
                  return (Jt - Sn) * (je - Hn) - (Ne - Sn) * (fn - Hn) >= 0 && (Ne - Sn) * (kt - Hn) - (ut - Sn) * (je - Hn) >= 0 && (ut - Sn) * (fn - Hn) - (Jt - Sn) * (kt - Hn) >= 0;
                }
                function Oh(Ne, je) {
                  return Ne.next.i !== je.i && Ne.prev.i !== je.i && !function(ut, kt) {
                    var Jt = ut;
                    do {
                      if (Jt.i !== ut.i && Jt.next.i !== ut.i && Jt.i !== kt.i && Jt.next.i !== kt.i && Mf(Jt, Jt.next, ut, kt)) return !0;
                      Jt = Jt.next;
                    } while (Jt !== ut);
                    return !1;
                  }(Ne, je) && (cu(Ne, je) && cu(je, Ne) && function(ut, kt) {
                    var Jt = ut, fn = !1, Sn = (ut.x + kt.x) / 2, Hn = (ut.y + kt.y) / 2;
                    do
                      Jt.y > Hn != Jt.next.y > Hn && Jt.next.y !== Jt.y && Sn < (Jt.next.x - Jt.x) * (Hn - Jt.y) / (Jt.next.y - Jt.y) + Jt.x && (fn = !fn), Jt = Jt.next;
                    while (Jt !== ut);
                    return fn;
                  }(Ne, je) && (Yu(Ne.prev, Ne, je.prev) || Yu(Ne, je.prev, je)) || kf(Ne, je) && Yu(Ne.prev, Ne, Ne.next) > 0 && Yu(je.prev, je, je.next) > 0);
                }
                function Yu(Ne, je, ut) {
                  return (je.y - Ne.y) * (ut.x - je.x) - (je.x - Ne.x) * (ut.y - je.y);
                }
                function kf(Ne, je) {
                  return Ne.x === je.x && Ne.y === je.y;
                }
                function Mf(Ne, je, ut, kt) {
                  var Jt = Id(Yu(Ne, je, ut)), fn = Id(Yu(Ne, je, kt)), Sn = Id(Yu(ut, kt, Ne)), Hn = Id(Yu(ut, kt, je));
                  return Jt !== fn && Sn !== Hn || !(Jt !== 0 || !e1(Ne, ut, je)) || !(fn !== 0 || !e1(Ne, kt, je)) || !(Sn !== 0 || !e1(ut, Ne, kt)) || !(Hn !== 0 || !e1(ut, je, kt));
                }
                function e1(Ne, je, ut) {
                  return je.x <= Math.max(Ne.x, ut.x) && je.x >= Math.min(Ne.x, ut.x) && je.y <= Math.max(Ne.y, ut.y) && je.y >= Math.min(Ne.y, ut.y);
                }
                function Id(Ne) {
                  return Ne > 0 ? 1 : Ne < 0 ? -1 : 0;
                }
                function cu(Ne, je) {
                  return Yu(Ne.prev, Ne, Ne.next) < 0 ? Yu(Ne, je, Ne.next) >= 0 && Yu(Ne, Ne.prev, je) >= 0 : Yu(Ne, je, Ne.prev) < 0 || Yu(Ne, Ne.next, je) < 0;
                }
                function pu(Ne, je) {
                  var ut = new yf(Ne.i, Ne.x, Ne.y), kt = new yf(je.i, je.x, je.y), Jt = Ne.next, fn = je.prev;
                  return Ne.next = je, je.prev = Ne, ut.next = Jt, Jt.prev = ut, kt.next = ut, ut.prev = kt, fn.next = kt, kt.prev = fn, kt;
                }
                function gf(Ne, je, ut, kt) {
                  var Jt = new yf(Ne, je, ut);
                  return kt ? (Jt.next = kt.next, Jt.prev = kt, kt.next.prev = Jt, kt.next = Jt) : (Jt.prev = Jt, Jt.next = Jt), Jt;
                }
                function lf(Ne) {
                  Ne.next.prev = Ne.prev, Ne.prev.next = Ne.next, Ne.prevZ && (Ne.prevZ.nextZ = Ne.nextZ), Ne.nextZ && (Ne.nextZ.prevZ = Ne.prevZ);
                }
                function yf(Ne, je, ut) {
                  this.i = Ne, this.x = je, this.y = ut, this.prev = null, this.next = null, this.z = null, this.prevZ = null, this.nextZ = null, this.steiner = !1;
                }
                function Ih(Ne, je, ut, kt) {
                  for (var Jt = 0, fn = je, Sn = ut - kt; fn < ut; fn += kt) Jt += (Ne[Sn] - Ne[fn]) * (Ne[fn + 1] + Ne[Sn + 1]), Sn = fn;
                  return Jt;
                }
                function Kf(Ne, je, ut, kt, Jt) {
                  zf(Ne, je, ut, kt || Ne.length - 1, Jt);
                }
                function zf(Ne, je, ut, kt, Jt) {
                  for (; kt > ut; ) {
                    if (kt - ut > 600) {
                      var fn = kt - ut + 1, Sn = je - ut + 1, Hn = Math.log(fn), nr = 0.5 * Math.exp(2 * Hn / 3), kr = 0.5 * Math.sqrt(Hn * nr * (fn - nr) / fn) * (Sn - fn / 2 < 0 ? -1 : 1);
                      zf(Ne, je, Math.max(ut, Math.floor(je - Sn * nr / fn + kr)), Math.min(kt, Math.floor(je + (fn - Sn) * nr / fn + kr)), Jt);
                    }
                    var Ir = Ne[je], qr = ut, ci = kt;
                    for (cf(Ne, ut, je), Jt(Ne[kt], Ir) > 0 && cf(Ne, ut, kt); qr < ci; ) {
                      for (cf(Ne, qr, ci), qr++, ci--; Jt(Ne[qr], Ir) < 0; ) qr++;
                      for (; Jt(Ne[ci], Ir) > 0; ) ci--;
                    }
                    Jt(Ne[ut], Ir) === 0 ? cf(Ne, ut, ci) : cf(Ne, ++ci, kt), ci <= je && (ut = ci + 1), je <= ci && (kt = ci - 1);
                  }
                }
                function cf(Ne, je, ut) {
                  var kt = Ne[je];
                  Ne[je] = Ne[ut], Ne[ut] = kt;
                }
                function Xh(Ne, je) {
                  var ut = Ne.length;
                  if (ut <= 1) return [Ne];
                  for (var kt, Jt, fn = [], Sn = 0; Sn < ut; Sn++) {
                    var Hn = De(Ne[Sn]);
                    Hn !== 0 && (Ne[Sn].area = Math.abs(Hn), Jt === void 0 && (Jt = Hn < 0), Jt === Hn < 0 ? (kt && fn.push(kt), kt = [Ne[Sn]]) : kt.push(Ne[Sn]));
                  }
                  if (kt && fn.push(kt), je > 1) for (var nr = 0; nr < fn.length; nr++) fn[nr].length <= je || (Kf(fn[nr], je, 1, fn[nr].length - 1, P2), fn[nr] = fn[nr].slice(0, je));
                  return fn;
                }
                function P2(Ne, je) {
                  return je.area - Ne.area;
                }
                function Jf(Ne, je, ut) {
                  for (var kt = ut.patternDependencies, Jt = !1, fn = 0, Sn = je; fn < Sn.length; fn += 1) {
                    var Hn = Sn[fn].paint.get(Ne + "-pattern");
                    Hn.isConstant() || (Jt = !0);
                    var nr = Hn.constantOr(null);
                    nr && (Jt = !0, kt[nr.to] = !0, kt[nr.from] = !0);
                  }
                  return Jt;
                }
                function eh(Ne, je, ut, kt, Jt) {
                  for (var fn = Jt.patternDependencies, Sn = 0, Hn = je; Sn < Hn.length; Sn += 1) {
                    var nr = Hn[Sn], kr = nr.paint.get(Ne + "-pattern").value;
                    if (kr.kind !== "constant") {
                      var Ir = kr.evaluate({ zoom: kt - 1 }, ut, {}, Jt.availableImages), qr = kr.evaluate({ zoom: kt }, ut, {}, Jt.availableImages), ci = kr.evaluate({ zoom: kt + 1 }, ut, {}, Jt.availableImages);
                      Ir = Ir && Ir.name ? Ir.name : Ir, qr = qr && qr.name ? qr.name : qr, ci = ci && ci.name ? ci.name : ci, fn[Ir] = !0, fn[qr] = !0, fn[ci] = !0, ut.patterns[nr.id] = { min: Ir, mid: qr, max: ci };
                    }
                  }
                  return ut;
                }
                Xf.deviation = function(Ne, je, ut, kt) {
                  var Jt = je && je.length, fn = Jt ? je[0] * ut : Ne.length, Sn = Math.abs(Ih(Ne, 0, fn, ut));
                  if (Jt) for (var Hn = 0, nr = je.length; Hn < nr; Hn++) {
                    var kr = je[Hn] * ut, Ir = Hn < nr - 1 ? je[Hn + 1] * ut : Ne.length;
                    Sn -= Math.abs(Ih(Ne, kr, Ir, ut));
                  }
                  var qr = 0;
                  for (Hn = 0; Hn < kt.length; Hn += 3) {
                    var ci = kt[Hn] * ut, ni = kt[Hn + 1] * ut, Ii = kt[Hn + 2] * ut;
                    qr += Math.abs((Ne[ci] - Ne[Ii]) * (Ne[ni + 1] - Ne[ci + 1]) - (Ne[ci] - Ne[ni]) * (Ne[Ii + 1] - Ne[ci + 1]));
                  }
                  return Sn === 0 && qr === 0 ? 0 : Math.abs((qr - Sn) / Sn);
                }, Xf.flatten = function(Ne) {
                  for (var je = Ne[0][0].length, ut = { vertices: [], holes: [], dimensions: je }, kt = 0, Jt = 0; Jt < Ne.length; Jt++) {
                    for (var fn = 0; fn < Ne[Jt].length; fn++) for (var Sn = 0; Sn < je; Sn++) ut.vertices.push(Ne[Jt][fn][Sn]);
                    Jt > 0 && (kt += Ne[Jt - 1].length, ut.holes.push(kt));
                  }
                  return ut;
                }, Lf.default = uh;
                var td = function(Ne) {
                  this.zoom = Ne.zoom, this.overscaling = Ne.overscaling, this.layers = Ne.layers, this.layerIds = this.layers.map(function(je) {
                    return je.id;
                  }), this.index = Ne.index, this.hasPattern = !1, this.patternFeatures = [], this.layoutVertexArray = new tu(), this.indexArray = new ht(), this.indexArray2 = new An(), this.programConfigurations = new Oc(Ne.layers, Ne.zoom), this.segments = new ra(), this.segments2 = new ra(), this.stateDependentLayerIds = this.layers.filter(function(je) {
                    return je.isStateDependent();
                  }).map(function(je) {
                    return je.id;
                  });
                };
                td.prototype.populate = function(Ne, je, ut) {
                  this.hasPattern = Jf("fill", this.layers, je);
                  for (var kt = this.layers[0].layout.get("fill-sort-key"), Jt = [], fn = 0, Sn = Ne; fn < Sn.length; fn += 1) {
                    var Hn = Sn[fn], nr = Hn.feature, kr = Hn.id, Ir = Hn.index, qr = Hn.sourceLayerIndex, ci = this.layers[0]._featureFilter.needGeometry, ni = Zc(nr, ci);
                    if (this.layers[0]._featureFilter.filter(new El(this.zoom), ni, ut)) {
                      var Ii = kt ? kt.evaluate(ni, {}, ut, je.availableImages) : void 0, Gi = { id: kr, properties: nr.properties, type: nr.type, sourceLayerIndex: qr, index: Ir, geometry: ci ? ni.geometry : zu(nr), patterns: {}, sortKey: Ii };
                      Jt.push(Gi);
                    }
                  }
                  kt && Jt.sort(function(As, Fs) {
                    return As.sortKey - Fs.sortKey;
                  });
                  for (var xa = 0, ga = Jt; xa < ga.length; xa += 1) {
                    var $a = ga[xa], Va = $a, eo = Va.geometry, co = Va.index, wo = Va.sourceLayerIndex;
                    if (this.hasPattern) {
                      var ds = eh("fill", this.layers, $a, this.zoom, je);
                      this.patternFeatures.push(ds);
                    } else this.addFeature($a, eo, co, ut, {});
                    var Lo = Ne[co].feature;
                    je.featureIndex.insert(Lo, eo, co, wo, this.index);
                  }
                }, td.prototype.update = function(Ne, je, ut) {
                  this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(Ne, je, this.stateDependentLayers, ut);
                }, td.prototype.addFeatures = function(Ne, je, ut) {
                  for (var kt = 0, Jt = this.patternFeatures; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    this.addFeature(fn, fn.geometry, fn.index, je, ut);
                  }
                }, td.prototype.isEmpty = function() {
                  return this.layoutVertexArray.length === 0;
                }, td.prototype.uploadPending = function() {
                  return !this.uploaded || this.programConfigurations.needsUpload;
                }, td.prototype.upload = function(Ne) {
                  this.uploaded || (this.layoutVertexBuffer = Ne.createVertexBuffer(this.layoutVertexArray, Zd), this.indexBuffer = Ne.createIndexBuffer(this.indexArray), this.indexBuffer2 = Ne.createIndexBuffer(this.indexArray2)), this.programConfigurations.upload(Ne), this.uploaded = !0;
                }, td.prototype.destroy = function() {
                  this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.indexBuffer2.destroy(), this.programConfigurations.destroy(), this.segments.destroy(), this.segments2.destroy());
                }, td.prototype.addFeature = function(Ne, je, ut, kt, Jt) {
                  for (var fn = 0, Sn = Xh(je, 500); fn < Sn.length; fn += 1) {
                    for (var Hn = Sn[fn], nr = 0, kr = 0, Ir = Hn; kr < Ir.length; kr += 1) nr += Ir[kr].length;
                    for (var qr = this.segments.prepareSegment(nr, this.layoutVertexArray, this.indexArray), ci = qr.vertexLength, ni = [], Ii = [], Gi = 0, xa = Hn; Gi < xa.length; Gi += 1) {
                      var ga = xa[Gi];
                      if (ga.length !== 0) {
                        ga !== Hn[0] && Ii.push(ni.length / 2);
                        var $a = this.segments2.prepareSegment(ga.length, this.layoutVertexArray, this.indexArray2), Va = $a.vertexLength;
                        this.layoutVertexArray.emplaceBack(ga[0].x, ga[0].y), this.indexArray2.emplaceBack(Va + ga.length - 1, Va), ni.push(ga[0].x), ni.push(ga[0].y);
                        for (var eo = 1; eo < ga.length; eo++) this.layoutVertexArray.emplaceBack(ga[eo].x, ga[eo].y), this.indexArray2.emplaceBack(Va + eo - 1, Va + eo), ni.push(ga[eo].x), ni.push(ga[eo].y);
                        $a.vertexLength += ga.length, $a.primitiveLength += ga.length;
                      }
                    }
                    for (var co = Lf(ni, Ii), wo = 0; wo < co.length; wo += 3) this.indexArray.emplaceBack(ci + co[wo], ci + co[wo + 1], ci + co[wo + 2]);
                    qr.vertexLength += nr, qr.primitiveLength += co.length / 3;
                  }
                  this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, Ne, ut, Jt, kt);
                }, cs("FillBucket", td, { omit: ["layers", "patternFeatures"] });
                var Rf = new $c({ "fill-sort-key": new qs(Zt.layout_fill["fill-sort-key"]) }), fh = { paint: new $c({ "fill-antialias": new ks(Zt.paint_fill["fill-antialias"]), "fill-opacity": new qs(Zt.paint_fill["fill-opacity"]), "fill-color": new qs(Zt.paint_fill["fill-color"]), "fill-outline-color": new qs(Zt.paint_fill["fill-outline-color"]), "fill-translate": new ks(Zt.paint_fill["fill-translate"]), "fill-translate-anchor": new ks(Zt.paint_fill["fill-translate-anchor"]), "fill-pattern": new a1(Zt.paint_fill["fill-pattern"]) }), layout: Rf }, $2 = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, fh);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.recalculate = function(ut, kt) {
                    Ne.prototype.recalculate.call(this, ut, kt);
                    var Jt = this.paint._values["fill-outline-color"];
                    Jt.value.kind === "constant" && Jt.value.value === void 0 && (this.paint._values["fill-outline-color"] = this.paint._values["fill-color"]);
                  }, je.prototype.createBucket = function(ut) {
                    return new td(ut);
                  }, je.prototype.queryRadius = function() {
                    return Au(this.paint.get("fill-translate"));
                  }, je.prototype.queryIntersectsFeature = function(ut, kt, Jt, fn, Sn, Hn, nr) {
                    return wd(Vu(ut, this.paint.get("fill-translate"), this.paint.get("fill-translate-anchor"), Hn.angle, nr), fn);
                  }, je.prototype.isTileClipped = function() {
                    return !0;
                  }, je;
                }(Ul), F2 = xc([{ name: "a_pos", components: 2, type: "Int16" }, { name: "a_normal_ed", components: 4, type: "Int16" }], 4).members, hh = bd;
                function bd(Ne, je, ut, kt, Jt) {
                  this.properties = {}, this.extent = ut, this.type = 0, this._pbf = Ne, this._geometry = -1, this._keys = kt, this._values = Jt, Ne.readFields(pr, this, je);
                }
                function pr(Ne, je, ut) {
                  Ne == 1 ? je.id = ut.readVarint() : Ne == 2 ? function(kt, Jt) {
                    for (var fn = kt.readVarint() + kt.pos; kt.pos < fn; ) {
                      var Sn = Jt._keys[kt.readVarint()], Hn = Jt._values[kt.readVarint()];
                      Jt.properties[Sn] = Hn;
                    }
                  }(ut, je) : Ne == 3 ? je.type = ut.readVarint() : Ne == 4 && (je._geometry = ut.pos);
                }
                function oi(Ne) {
                  for (var je, ut, kt = 0, Jt = 0, fn = Ne.length, Sn = fn - 1; Jt < fn; Sn = Jt++) je = Ne[Jt], kt += ((ut = Ne[Sn]).x - je.x) * (je.y + ut.y);
                  return kt;
                }
                bd.types = ["Unknown", "Point", "LineString", "Polygon"], bd.prototype.loadGeometry = function() {
                  var Ne = this._pbf;
                  Ne.pos = this._geometry;
                  for (var je, ut = Ne.readVarint() + Ne.pos, kt = 1, Jt = 0, fn = 0, Sn = 0, Hn = []; Ne.pos < ut; ) {
                    if (Jt <= 0) {
                      var nr = Ne.readVarint();
                      kt = 7 & nr, Jt = nr >> 3;
                    }
                    if (Jt--, kt === 1 || kt === 2) fn += Ne.readSVarint(), Sn += Ne.readSVarint(), kt === 1 && (je && Hn.push(je), je = []), je.push(new le(fn, Sn));
                    else {
                      if (kt !== 7) throw new Error("unknown command " + kt);
                      je && je.push(je[0].clone());
                    }
                  }
                  return je && Hn.push(je), Hn;
                }, bd.prototype.bbox = function() {
                  var Ne = this._pbf;
                  Ne.pos = this._geometry;
                  for (var je = Ne.readVarint() + Ne.pos, ut = 1, kt = 0, Jt = 0, fn = 0, Sn = 1 / 0, Hn = -1 / 0, nr = 1 / 0, kr = -1 / 0; Ne.pos < je; ) {
                    if (kt <= 0) {
                      var Ir = Ne.readVarint();
                      ut = 7 & Ir, kt = Ir >> 3;
                    }
                    if (kt--, ut === 1 || ut === 2) (Jt += Ne.readSVarint()) < Sn && (Sn = Jt), Jt > Hn && (Hn = Jt), (fn += Ne.readSVarint()) < nr && (nr = fn), fn > kr && (kr = fn);
                    else if (ut !== 7) throw new Error("unknown command " + ut);
                  }
                  return [Sn, nr, Hn, kr];
                }, bd.prototype.toGeoJSON = function(Ne, je, ut) {
                  var kt, Jt, fn = this.extent * Math.pow(2, ut), Sn = this.extent * Ne, Hn = this.extent * je, nr = this.loadGeometry(), kr = bd.types[this.type];
                  function Ir(ni) {
                    for (var Ii = 0; Ii < ni.length; Ii++) {
                      var Gi = ni[Ii], xa = 180 - 360 * (Gi.y + Hn) / fn;
                      ni[Ii] = [360 * (Gi.x + Sn) / fn - 180, 360 / Math.PI * Math.atan(Math.exp(xa * Math.PI / 180)) - 90];
                    }
                  }
                  switch (this.type) {
                    case 1:
                      var qr = [];
                      for (kt = 0; kt < nr.length; kt++) qr[kt] = nr[kt][0];
                      Ir(nr = qr);
                      break;
                    case 2:
                      for (kt = 0; kt < nr.length; kt++) Ir(nr[kt]);
                      break;
                    case 3:
                      for (nr = function(ni) {
                        var Ii = ni.length;
                        if (Ii <= 1) return [ni];
                        for (var Gi, xa, ga = [], $a = 0; $a < Ii; $a++) {
                          var Va = oi(ni[$a]);
                          Va !== 0 && (xa === void 0 && (xa = Va < 0), xa === Va < 0 ? (Gi && ga.push(Gi), Gi = [ni[$a]]) : Gi.push(ni[$a]));
                        }
                        return Gi && ga.push(Gi), ga;
                      }(nr), kt = 0; kt < nr.length; kt++) for (Jt = 0; Jt < nr[kt].length; Jt++) Ir(nr[kt][Jt]);
                  }
                  nr.length === 1 ? nr = nr[0] : kr = "Multi" + kr;
                  var ci = { type: "Feature", geometry: { type: kr, coordinates: nr }, properties: this.properties };
                  return "id" in this && (ci.id = this.id), ci;
                };
                var ar = Sa;
                function Sa(Ne, je) {
                  this.version = 1, this.name = null, this.extent = 4096, this.length = 0, this._pbf = Ne, this._keys = [], this._values = [], this._features = [], Ne.readFields(Qn, this, je), this.length = this._features.length;
                }
                function Qn(Ne, je, ut) {
                  Ne === 15 ? je.version = ut.readVarint() : Ne === 1 ? je.name = ut.readString() : Ne === 5 ? je.extent = ut.readVarint() : Ne === 2 ? je._features.push(ut.pos) : Ne === 3 ? je._keys.push(ut.readString()) : Ne === 4 && je._values.push(function(kt) {
                    for (var Jt = null, fn = kt.readVarint() + kt.pos; kt.pos < fn; ) {
                      var Sn = kt.readVarint() >> 3;
                      Jt = Sn === 1 ? kt.readString() : Sn === 2 ? kt.readFloat() : Sn === 3 ? kt.readDouble() : Sn === 4 ? kt.readVarint64() : Sn === 5 ? kt.readVarint() : Sn === 6 ? kt.readSVarint() : Sn === 7 ? kt.readBoolean() : null;
                    }
                    return Jt;
                  }(ut));
                }
                function za(Ne, je, ut) {
                  if (Ne === 3) {
                    var kt = new ar(ut, ut.readVarint() + ut.pos);
                    kt.length && (je[kt.name] = kt);
                  }
                }
                Sa.prototype.feature = function(Ne) {
                  if (Ne < 0 || Ne >= this._features.length) throw new Error("feature index out of bounds");
                  this._pbf.pos = this._features[Ne];
                  var je = this._pbf.readVarint() + this._pbf.pos;
                  return new hh(this._pbf, je, this.extent, this._keys, this._values);
                };
                var En = { VectorTile: function(Ne, je) {
                  this.layers = Ne.readFields(za, {}, je);
                }, VectorTileFeature: hh, VectorTileLayer: ar }, Qo = En.VectorTileFeature.types, lo = Math.pow(2, 13);
                function Eo(Ne, je, ut, kt, Jt, fn, Sn, Hn) {
                  Ne.emplaceBack(je, ut, 2 * Math.floor(kt * lo) + Sn, Jt * lo * 2, fn * lo * 2, Math.round(Hn));
                }
                var bo = function(Ne) {
                  this.zoom = Ne.zoom, this.overscaling = Ne.overscaling, this.layers = Ne.layers, this.layerIds = this.layers.map(function(je) {
                    return je.id;
                  }), this.index = Ne.index, this.hasPattern = !1, this.layoutVertexArray = new $n(), this.indexArray = new ht(), this.programConfigurations = new Oc(Ne.layers, Ne.zoom), this.segments = new ra(), this.stateDependentLayerIds = this.layers.filter(function(je) {
                    return je.isStateDependent();
                  }).map(function(je) {
                    return je.id;
                  });
                };
                function Ho(Ne, je) {
                  return Ne.x === je.x && (Ne.x < 0 || Ne.x > $s) || Ne.y === je.y && (Ne.y < 0 || Ne.y > $s);
                }
                bo.prototype.populate = function(Ne, je, ut) {
                  this.features = [], this.hasPattern = Jf("fill-extrusion", this.layers, je);
                  for (var kt = 0, Jt = Ne; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt], Sn = fn.feature, Hn = fn.id, nr = fn.index, kr = fn.sourceLayerIndex, Ir = this.layers[0]._featureFilter.needGeometry, qr = Zc(Sn, Ir);
                    if (this.layers[0]._featureFilter.filter(new El(this.zoom), qr, ut)) {
                      var ci = { id: Hn, sourceLayerIndex: kr, index: nr, geometry: Ir ? qr.geometry : zu(Sn), properties: Sn.properties, type: Sn.type, patterns: {} };
                      this.hasPattern ? this.features.push(eh("fill-extrusion", this.layers, ci, this.zoom, je)) : this.addFeature(ci, ci.geometry, nr, ut, {}), je.featureIndex.insert(Sn, ci.geometry, nr, kr, this.index, !0);
                    }
                  }
                }, bo.prototype.addFeatures = function(Ne, je, ut) {
                  for (var kt = 0, Jt = this.features; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt], Sn = fn.geometry;
                    this.addFeature(fn, Sn, fn.index, je, ut);
                  }
                }, bo.prototype.update = function(Ne, je, ut) {
                  this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(Ne, je, this.stateDependentLayers, ut);
                }, bo.prototype.isEmpty = function() {
                  return this.layoutVertexArray.length === 0;
                }, bo.prototype.uploadPending = function() {
                  return !this.uploaded || this.programConfigurations.needsUpload;
                }, bo.prototype.upload = function(Ne) {
                  this.uploaded || (this.layoutVertexBuffer = Ne.createVertexBuffer(this.layoutVertexArray, F2), this.indexBuffer = Ne.createIndexBuffer(this.indexArray)), this.programConfigurations.upload(Ne), this.uploaded = !0;
                }, bo.prototype.destroy = function() {
                  this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy());
                }, bo.prototype.addFeature = function(Ne, je, ut, kt, Jt) {
                  for (var fn = 0, Sn = Xh(je, 500); fn < Sn.length; fn += 1) {
                    for (var Hn = Sn[fn], nr = 0, kr = 0, Ir = Hn; kr < Ir.length; kr += 1) nr += Ir[kr].length;
                    for (var qr = this.segments.prepareSegment(4, this.layoutVertexArray, this.indexArray), ci = 0, ni = Hn; ci < ni.length; ci += 1) {
                      var Ii = ni[ci];
                      if (Ii.length !== 0 && !((Hc = Ii).every(function(tc) {
                        return tc.x < 0;
                      }) || Hc.every(function(tc) {
                        return tc.x > $s;
                      }) || Hc.every(function(tc) {
                        return tc.y < 0;
                      }) || Hc.every(function(tc) {
                        return tc.y > $s;
                      }))) for (var Gi = 0, xa = 0; xa < Ii.length; xa++) {
                        var ga = Ii[xa];
                        if (xa >= 1) {
                          var $a = Ii[xa - 1];
                          if (!Ho(ga, $a)) {
                            qr.vertexLength + 4 > ra.MAX_VERTEX_ARRAY_LENGTH && (qr = this.segments.prepareSegment(4, this.layoutVertexArray, this.indexArray));
                            var Va = ga.sub($a)._perp()._unit(), eo = $a.dist(ga);
                            Gi + eo > 32768 && (Gi = 0), Eo(this.layoutVertexArray, ga.x, ga.y, Va.x, Va.y, 0, 0, Gi), Eo(this.layoutVertexArray, ga.x, ga.y, Va.x, Va.y, 0, 1, Gi), Gi += eo, Eo(this.layoutVertexArray, $a.x, $a.y, Va.x, Va.y, 0, 0, Gi), Eo(this.layoutVertexArray, $a.x, $a.y, Va.x, Va.y, 0, 1, Gi);
                            var co = qr.vertexLength;
                            this.indexArray.emplaceBack(co, co + 2, co + 1), this.indexArray.emplaceBack(co + 1, co + 2, co + 3), qr.vertexLength += 4, qr.primitiveLength += 2;
                          }
                        }
                      }
                    }
                    if (qr.vertexLength + nr > ra.MAX_VERTEX_ARRAY_LENGTH && (qr = this.segments.prepareSegment(nr, this.layoutVertexArray, this.indexArray)), Qo[Ne.type] === "Polygon") {
                      for (var wo = [], ds = [], Lo = qr.vertexLength, As = 0, Fs = Hn; As < Fs.length; As += 1) {
                        var Pl = Fs[As];
                        if (Pl.length !== 0) {
                          Pl !== Hn[0] && ds.push(wo.length / 2);
                          for (var rl = 0; rl < Pl.length; rl++) {
                            var Rl = Pl[rl];
                            Eo(this.layoutVertexArray, Rl.x, Rl.y, 0, 0, 1, 1, 0), wo.push(Rl.x), wo.push(Rl.y);
                          }
                        }
                      }
                      for (var pl = Lf(wo, ds), Zs = 0; Zs < pl.length; Zs += 3) this.indexArray.emplaceBack(Lo + pl[Zs], Lo + pl[Zs + 2], Lo + pl[Zs + 1]);
                      qr.primitiveLength += pl.length / 3, qr.vertexLength += nr;
                    }
                  }
                  var Hc;
                  this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, Ne, ut, Jt, kt);
                }, cs("FillExtrusionBucket", bo, { omit: ["layers", "features"] });
                var Co = { paint: new $c({ "fill-extrusion-opacity": new ks(Zt["paint_fill-extrusion"]["fill-extrusion-opacity"]), "fill-extrusion-color": new qs(Zt["paint_fill-extrusion"]["fill-extrusion-color"]), "fill-extrusion-translate": new ks(Zt["paint_fill-extrusion"]["fill-extrusion-translate"]), "fill-extrusion-translate-anchor": new ks(Zt["paint_fill-extrusion"]["fill-extrusion-translate-anchor"]), "fill-extrusion-pattern": new a1(Zt["paint_fill-extrusion"]["fill-extrusion-pattern"]), "fill-extrusion-height": new qs(Zt["paint_fill-extrusion"]["fill-extrusion-height"]), "fill-extrusion-base": new qs(Zt["paint_fill-extrusion"]["fill-extrusion-base"]), "fill-extrusion-vertical-gradient": new ks(Zt["paint_fill-extrusion"]["fill-extrusion-vertical-gradient"]) }) }, Ao = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, Co);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.createBucket = function(ut) {
                    return new bo(ut);
                  }, je.prototype.queryRadius = function() {
                    return Au(this.paint.get("fill-extrusion-translate"));
                  }, je.prototype.is3D = function() {
                    return !0;
                  }, je.prototype.queryIntersectsFeature = function(ut, kt, Jt, fn, Sn, Hn, nr, kr) {
                    var Ir = Vu(ut, this.paint.get("fill-extrusion-translate"), this.paint.get("fill-extrusion-translate-anchor"), Hn.angle, nr), qr = this.paint.get("fill-extrusion-height").evaluate(kt, Jt), ci = this.paint.get("fill-extrusion-base").evaluate(kt, Jt), ni = function(Gi, xa, ga, $a) {
                      for (var Va = [], eo = 0, co = Gi; eo < co.length; eo += 1) {
                        var wo = co[eo], ds = [wo.x, wo.y, $a, 1];
                        Sd(ds, ds, xa), Va.push(new le(ds[0] / ds[3], ds[1] / ds[3]));
                      }
                      return Va;
                    }(Ir, kr, 0, 0), Ii = function(Gi, xa, ga, $a) {
                      for (var Va = [], eo = [], co = $a[8] * xa, wo = $a[9] * xa, ds = $a[10] * xa, Lo = $a[11] * xa, As = $a[8] * ga, Fs = $a[9] * ga, Pl = $a[10] * ga, rl = $a[11] * ga, Rl = 0, pl = Gi; Rl < pl.length; Rl += 1) {
                        for (var Zs = [], Hc = [], tc = 0, Pu = pl[Rl]; tc < Pu.length; tc += 1) {
                          var Cl = Pu[tc], vu = Cl.x, qu = Cl.y, Nu = $a[0] * vu + $a[4] * qu + $a[12], ju = $a[1] * vu + $a[5] * qu + $a[13], $u = $a[2] * vu + $a[6] * qu + $a[14], $1 = $a[3] * vu + $a[7] * qu + $a[15], Zu = $u + ds, F1 = $1 + Lo, Td = Nu + As, hd = ju + Fs, Dd = $u + Pl, Qd = $1 + rl, Ye = new le((Nu + co) / F1, (ju + wo) / F1);
                          Ye.z = Zu / F1, Zs.push(Ye);
                          var He = new le(Td / Qd, hd / Qd);
                          He.z = Dd / Qd, Hc.push(He);
                        }
                        Va.push(Zs), eo.push(Hc);
                      }
                      return [Va, eo];
                    }(fn, ci, qr, kr);
                    return function(Gi, xa, ga) {
                      var $a = 1 / 0;
                      wd(ga, xa) && ($a = Qa(ga, xa[0]));
                      for (var Va = 0; Va < xa.length; Va++) for (var eo = xa[Va], co = Gi[Va], wo = 0; wo < eo.length - 1; wo++) {
                        var ds = eo[wo], Lo = eo[wo + 1], As = co[wo], Fs = [ds, Lo, co[wo + 1], As, ds];
                        Bd(ga, Fs) && ($a = Math.min($a, Qa(ga, Fs)));
                      }
                      return $a !== 1 / 0 && $a;
                    }(Ii[0], Ii[1], ni);
                  }, je;
                }(Ul);
                function Ro(Ne, je) {
                  return Ne.x * je.x + Ne.y * je.y;
                }
                function Qa(Ne, je) {
                  if (Ne.length === 1) {
                    for (var ut, kt = 0, Jt = je[kt++]; !ut || Jt.equals(ut); ) if (!(ut = je[kt++])) return 1 / 0;
                    for (; kt < je.length; kt++) {
                      var fn = je[kt], Sn = Ne[0], Hn = ut.sub(Jt), nr = fn.sub(Jt), kr = Sn.sub(Jt), Ir = Ro(Hn, Hn), qr = Ro(Hn, nr), ci = Ro(nr, nr), ni = Ro(kr, Hn), Ii = Ro(kr, nr), Gi = Ir * ci - qr * qr, xa = (ci * ni - qr * Ii) / Gi, ga = (Ir * Ii - qr * ni) / Gi, $a = 1 - xa - ga, Va = Jt.z * $a + ut.z * xa + fn.z * ga;
                      if (isFinite(Va)) return Va;
                    }
                    return 1 / 0;
                  }
                  for (var eo = 1 / 0, co = 0, wo = je; co < wo.length; co += 1) {
                    var ds = wo[co];
                    eo = Math.min(eo, ds.z);
                  }
                  return eo;
                }
                var Ka = xc([{ name: "a_pos_normal", components: 2, type: "Int16" }, { name: "a_data", components: 4, type: "Uint8" }], 4).members, qo = xc([{ name: "a_uv_x", components: 1, type: "Float32" }, { name: "a_split_index", components: 1, type: "Float32" }]).members, Ss = En.VectorTileFeature.types, ho = Math.cos(Math.PI / 180 * 37.5), hs = Math.pow(2, 14) / 0.5, Bo = function(Ne) {
                  var je = this;
                  this.zoom = Ne.zoom, this.overscaling = Ne.overscaling, this.layers = Ne.layers, this.layerIds = this.layers.map(function(ut) {
                    return ut.id;
                  }), this.index = Ne.index, this.hasPattern = !1, this.patternFeatures = [], this.lineClipsArray = [], this.gradients = {}, this.layers.forEach(function(ut) {
                    je.gradients[ut.id] = {};
                  }), this.layoutVertexArray = new zn(), this.layoutVertexArray2 = new Gn(), this.indexArray = new ht(), this.programConfigurations = new Oc(Ne.layers, Ne.zoom), this.segments = new ra(), this.maxLineLength = 0, this.stateDependentLayerIds = this.layers.filter(function(ut) {
                    return ut.isStateDependent();
                  }).map(function(ut) {
                    return ut.id;
                  });
                };
                Bo.prototype.populate = function(Ne, je, ut) {
                  this.hasPattern = Jf("line", this.layers, je);
                  for (var kt = this.layers[0].layout.get("line-sort-key"), Jt = [], fn = 0, Sn = Ne; fn < Sn.length; fn += 1) {
                    var Hn = Sn[fn], nr = Hn.feature, kr = Hn.id, Ir = Hn.index, qr = Hn.sourceLayerIndex, ci = this.layers[0]._featureFilter.needGeometry, ni = Zc(nr, ci);
                    if (this.layers[0]._featureFilter.filter(new El(this.zoom), ni, ut)) {
                      var Ii = kt ? kt.evaluate(ni, {}, ut) : void 0, Gi = { id: kr, properties: nr.properties, type: nr.type, sourceLayerIndex: qr, index: Ir, geometry: ci ? ni.geometry : zu(nr), patterns: {}, sortKey: Ii };
                      Jt.push(Gi);
                    }
                  }
                  kt && Jt.sort(function(As, Fs) {
                    return As.sortKey - Fs.sortKey;
                  });
                  for (var xa = 0, ga = Jt; xa < ga.length; xa += 1) {
                    var $a = ga[xa], Va = $a, eo = Va.geometry, co = Va.index, wo = Va.sourceLayerIndex;
                    if (this.hasPattern) {
                      var ds = eh("line", this.layers, $a, this.zoom, je);
                      this.patternFeatures.push(ds);
                    } else this.addFeature($a, eo, co, ut, {});
                    var Lo = Ne[co].feature;
                    je.featureIndex.insert(Lo, eo, co, wo, this.index);
                  }
                }, Bo.prototype.update = function(Ne, je, ut) {
                  this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(Ne, je, this.stateDependentLayers, ut);
                }, Bo.prototype.addFeatures = function(Ne, je, ut) {
                  for (var kt = 0, Jt = this.patternFeatures; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    this.addFeature(fn, fn.geometry, fn.index, je, ut);
                  }
                }, Bo.prototype.isEmpty = function() {
                  return this.layoutVertexArray.length === 0;
                }, Bo.prototype.uploadPending = function() {
                  return !this.uploaded || this.programConfigurations.needsUpload;
                }, Bo.prototype.upload = function(Ne) {
                  this.uploaded || (this.layoutVertexArray2.length !== 0 && (this.layoutVertexBuffer2 = Ne.createVertexBuffer(this.layoutVertexArray2, qo)), this.layoutVertexBuffer = Ne.createVertexBuffer(this.layoutVertexArray, Ka), this.indexBuffer = Ne.createIndexBuffer(this.indexArray)), this.programConfigurations.upload(Ne), this.uploaded = !0;
                }, Bo.prototype.destroy = function() {
                  this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy());
                }, Bo.prototype.lineFeatureClips = function(Ne) {
                  if (Ne.properties && Ne.properties.hasOwnProperty("mapbox_clip_start") && Ne.properties.hasOwnProperty("mapbox_clip_end")) return { start: +Ne.properties.mapbox_clip_start, end: +Ne.properties.mapbox_clip_end };
                }, Bo.prototype.addFeature = function(Ne, je, ut, kt, Jt) {
                  var fn = this.layers[0].layout, Sn = fn.get("line-join").evaluate(Ne, {}), Hn = fn.get("line-cap"), nr = fn.get("line-miter-limit"), kr = fn.get("line-round-limit");
                  this.lineClips = this.lineFeatureClips(Ne);
                  for (var Ir = 0, qr = je; Ir < qr.length; Ir += 1) {
                    var ci = qr[Ir];
                    this.addLine(ci, Ne, Sn, Hn, nr, kr);
                  }
                  this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, Ne, ut, Jt, kt);
                }, Bo.prototype.addLine = function(Ne, je, ut, kt, Jt, fn) {
                  if (this.distance = 0, this.scaledDistance = 0, this.totalDistance = 0, this.lineClips) {
                    this.lineClipsArray.push(this.lineClips);
                    for (var Sn = 0; Sn < Ne.length - 1; Sn++) this.totalDistance += Ne[Sn].dist(Ne[Sn + 1]);
                    this.updateScaledDistance(), this.maxLineLength = Math.max(this.maxLineLength, this.totalDistance);
                  }
                  for (var Hn = Ss[je.type] === "Polygon", nr = Ne.length; nr >= 2 && Ne[nr - 1].equals(Ne[nr - 2]); ) nr--;
                  for (var kr = 0; kr < nr - 1 && Ne[kr].equals(Ne[kr + 1]); ) kr++;
                  if (!(nr < (Hn ? 3 : 2))) {
                    ut === "bevel" && (Jt = 1.05);
                    var Ir, qr = this.overscaling <= 16 ? 15 * $s / (512 * this.overscaling) : 0, ci = this.segments.prepareSegment(10 * nr, this.layoutVertexArray, this.indexArray), ni = void 0, Ii = void 0, Gi = void 0, xa = void 0;
                    this.e1 = this.e2 = -1, Hn && (Ir = Ne[nr - 2], xa = Ne[kr].sub(Ir)._unit()._perp());
                    for (var ga = kr; ga < nr; ga++) if (!(Ii = ga === nr - 1 ? Hn ? Ne[kr + 1] : void 0 : Ne[ga + 1]) || !Ne[ga].equals(Ii)) {
                      xa && (Gi = xa), Ir && (ni = Ir), Ir = Ne[ga], xa = Ii ? Ii.sub(Ir)._unit()._perp() : Gi;
                      var $a = (Gi = Gi || xa).add(xa);
                      $a.x === 0 && $a.y === 0 || $a._unit();
                      var Va = Gi.x * xa.x + Gi.y * xa.y, eo = $a.x * xa.x + $a.y * xa.y, co = eo !== 0 ? 1 / eo : 1 / 0, wo = 2 * Math.sqrt(2 - 2 * eo), ds = eo < ho && ni && Ii, Lo = Gi.x * xa.y - Gi.y * xa.x > 0;
                      if (ds && ga > kr) {
                        var As = Ir.dist(ni);
                        if (As > 2 * qr) {
                          var Fs = Ir.sub(Ir.sub(ni)._mult(qr / As)._round());
                          this.updateDistance(ni, Fs), this.addCurrentVertex(Fs, Gi, 0, 0, ci), ni = Fs;
                        }
                      }
                      var Pl = ni && Ii, rl = Pl ? ut : Hn ? "butt" : kt;
                      if (Pl && rl === "round" && (co < fn ? rl = "miter" : co <= 2 && (rl = "fakeround")), rl === "miter" && co > Jt && (rl = "bevel"), rl === "bevel" && (co > 2 && (rl = "flipbevel"), co < Jt && (rl = "miter")), ni && this.updateDistance(ni, Ir), rl === "miter") $a._mult(co), this.addCurrentVertex(Ir, $a, 0, 0, ci);
                      else if (rl === "flipbevel") {
                        if (co > 100) $a = xa.mult(-1);
                        else {
                          var Rl = co * Gi.add(xa).mag() / Gi.sub(xa).mag();
                          $a._perp()._mult(Rl * (Lo ? -1 : 1));
                        }
                        this.addCurrentVertex(Ir, $a, 0, 0, ci), this.addCurrentVertex(Ir, $a.mult(-1), 0, 0, ci);
                      } else if (rl === "bevel" || rl === "fakeround") {
                        var pl = -Math.sqrt(co * co - 1), Zs = Lo ? pl : 0, Hc = Lo ? 0 : pl;
                        if (ni && this.addCurrentVertex(Ir, Gi, Zs, Hc, ci), rl === "fakeround") for (var tc = Math.round(180 * wo / Math.PI / 20), Pu = 1; Pu < tc; Pu++) {
                          var Cl = Pu / tc;
                          if (Cl !== 0.5) {
                            var vu = Cl - 0.5;
                            Cl += Cl * vu * (Cl - 1) * ((1.0904 + Va * (Va * (3.55645 - 1.43519 * Va) - 3.2452)) * vu * vu + (0.848013 + Va * (0.215638 * Va - 1.06021)));
                          }
                          var qu = xa.sub(Gi)._mult(Cl)._add(Gi)._unit()._mult(Lo ? -1 : 1);
                          this.addHalfVertex(Ir, qu.x, qu.y, !1, Lo, 0, ci);
                        }
                        Ii && this.addCurrentVertex(Ir, xa, -Zs, -Hc, ci);
                      } else if (rl === "butt") this.addCurrentVertex(Ir, $a, 0, 0, ci);
                      else if (rl === "square") {
                        var Nu = ni ? 1 : -1;
                        this.addCurrentVertex(Ir, $a, Nu, Nu, ci);
                      } else rl === "round" && (ni && (this.addCurrentVertex(Ir, Gi, 0, 0, ci), this.addCurrentVertex(Ir, Gi, 1, 1, ci, !0)), Ii && (this.addCurrentVertex(Ir, xa, -1, -1, ci, !0), this.addCurrentVertex(Ir, xa, 0, 0, ci)));
                      if (ds && ga < nr - 1) {
                        var ju = Ir.dist(Ii);
                        if (ju > 2 * qr) {
                          var $u = Ir.add(Ii.sub(Ir)._mult(qr / ju)._round());
                          this.updateDistance(Ir, $u), this.addCurrentVertex($u, xa, 0, 0, ci), Ir = $u;
                        }
                      }
                    }
                  }
                }, Bo.prototype.addCurrentVertex = function(Ne, je, ut, kt, Jt, fn) {
                  fn === void 0 && (fn = !1);
                  var Sn = je.x + je.y * ut, Hn = je.y - je.x * ut, nr = -je.x + je.y * kt, kr = -je.y - je.x * kt;
                  this.addHalfVertex(Ne, Sn, Hn, fn, !1, ut, Jt), this.addHalfVertex(Ne, nr, kr, fn, !0, -kt, Jt), this.distance > hs / 2 && this.totalDistance === 0 && (this.distance = 0, this.addCurrentVertex(Ne, je, ut, kt, Jt, fn));
                }, Bo.prototype.addHalfVertex = function(Ne, je, ut, kt, Jt, fn, Sn) {
                  var Hn = Ne.x, nr = Ne.y, kr = 0.5 * (this.lineClips ? this.scaledDistance * (hs - 1) : this.scaledDistance);
                  if (this.layoutVertexArray.emplaceBack((Hn << 1) + (kt ? 1 : 0), (nr << 1) + (Jt ? 1 : 0), Math.round(63 * je) + 128, Math.round(63 * ut) + 128, 1 + (fn === 0 ? 0 : fn < 0 ? -1 : 1) | (63 & kr) << 2, kr >> 6), this.lineClips) {
                    var Ir = (this.scaledDistance - this.lineClips.start) / (this.lineClips.end - this.lineClips.start);
                    this.layoutVertexArray2.emplaceBack(Ir, this.lineClipsArray.length);
                  }
                  var qr = Sn.vertexLength++;
                  this.e1 >= 0 && this.e2 >= 0 && (this.indexArray.emplaceBack(this.e1, this.e2, qr), Sn.primitiveLength++), Jt ? this.e2 = qr : this.e1 = qr;
                }, Bo.prototype.updateScaledDistance = function() {
                  this.scaledDistance = this.lineClips ? this.lineClips.start + (this.lineClips.end - this.lineClips.start) * this.distance / this.totalDistance : this.distance;
                }, Bo.prototype.updateDistance = function(Ne, je) {
                  this.distance += Ne.dist(je), this.updateScaledDistance();
                }, cs("LineBucket", Bo, { omit: ["layers", "patternFeatures"] });
                var Vs = new $c({ "line-cap": new ks(Zt.layout_line["line-cap"]), "line-join": new qs(Zt.layout_line["line-join"]), "line-miter-limit": new ks(Zt.layout_line["line-miter-limit"]), "line-round-limit": new ks(Zt.layout_line["line-round-limit"]), "line-sort-key": new qs(Zt.layout_line["line-sort-key"]) }), vo = { paint: new $c({ "line-opacity": new qs(Zt.paint_line["line-opacity"]), "line-color": new qs(Zt.paint_line["line-color"]), "line-translate": new ks(Zt.paint_line["line-translate"]), "line-translate-anchor": new ks(Zt.paint_line["line-translate-anchor"]), "line-width": new qs(Zt.paint_line["line-width"]), "line-gap-width": new qs(Zt.paint_line["line-gap-width"]), "line-offset": new qs(Zt.paint_line["line-offset"]), "line-blur": new qs(Zt.paint_line["line-blur"]), "line-dasharray": new $l(Zt.paint_line["line-dasharray"]), "line-pattern": new a1(Zt.paint_line["line-pattern"]), "line-gradient": new o1(Zt.paint_line["line-gradient"]) }), layout: Vs }, go = function(Ne) {
                  function je() {
                    Ne.apply(this, arguments);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.possiblyEvaluate = function(ut, kt) {
                    return kt = new El(Math.floor(kt.zoom), { now: kt.now, fadeDuration: kt.fadeDuration, zoomHistory: kt.zoomHistory, transition: kt.transition }), Ne.prototype.possiblyEvaluate.call(this, ut, kt);
                  }, je.prototype.evaluate = function(ut, kt, Jt, fn) {
                    return kt = ge({}, kt, { zoom: Math.floor(kt.zoom) }), Ne.prototype.evaluate.call(this, ut, kt, Jt, fn);
                  }, je;
                }(qs), Qs = new go(vo.paint.properties["line-width"].specification);
                Qs.useIntegerZoom = !0;
                var us = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, vo), this.gradientVersion = 0;
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype._handleSpecialPaintPropertyUpdate = function(ut) {
                    if (ut === "line-gradient") {
                      var kt = this._transitionablePaint._values["line-gradient"].value.expression;
                      this.stepInterpolant = kt._styleExpression.expression instanceof nn, this.gradientVersion = (this.gradientVersion + 1) % fe;
                    }
                  }, je.prototype.gradientExpression = function() {
                    return this._transitionablePaint._values["line-gradient"].value.expression;
                  }, je.prototype.recalculate = function(ut, kt) {
                    Ne.prototype.recalculate.call(this, ut, kt), this.paint._values["line-floorwidth"] = Qs.possiblyEvaluate(this._transitioningPaint._values["line-width"].value, ut);
                  }, je.prototype.createBucket = function(ut) {
                    return new Bo(ut);
                  }, je.prototype.queryRadius = function(ut) {
                    var kt = ut, Jt = Po(Pd("line-width", this, kt), Pd("line-gap-width", this, kt)), fn = Pd("line-offset", this, kt);
                    return Jt / 2 + Math.abs(fn) + Au(this.paint.get("line-translate"));
                  }, je.prototype.queryIntersectsFeature = function(ut, kt, Jt, fn, Sn, Hn, nr) {
                    var kr = Vu(ut, this.paint.get("line-translate"), this.paint.get("line-translate-anchor"), Hn.angle, nr), Ir = nr / 2 * Po(this.paint.get("line-width").evaluate(kt, Jt), this.paint.get("line-gap-width").evaluate(kt, Jt)), qr = this.paint.get("line-offset").evaluate(kt, Jt);
                    return qr && (fn = function(ci, ni) {
                      for (var Ii = [], Gi = new le(0, 0), xa = 0; xa < ci.length; xa++) {
                        for (var ga = ci[xa], $a = [], Va = 0; Va < ga.length; Va++) {
                          var eo = ga[Va - 1], co = ga[Va], wo = ga[Va + 1], ds = Va === 0 ? Gi : co.sub(eo)._unit()._perp(), Lo = Va === ga.length - 1 ? Gi : wo.sub(co)._unit()._perp(), As = ds._add(Lo)._unit(), Fs = As.x * Lo.x + As.y * Lo.y;
                          As._mult(1 / Fs), $a.push(As._mult(ni)._add(co));
                        }
                        Ii.push($a);
                      }
                      return Ii;
                    }(fn, qr * nr)), function(ci, ni, Ii) {
                      for (var Gi = 0; Gi < ni.length; Gi++) {
                        var xa = ni[Gi];
                        if (ci.length >= 3) {
                          for (var ga = 0; ga < xa.length; ga++) if (S1(ci, xa[ga])) return !0;
                        }
                        if (od(ci, xa, Ii)) return !0;
                      }
                      return !1;
                    }(kr, fn, Ir);
                  }, je.prototype.isTileClipped = function() {
                    return !0;
                  }, je;
                }(Ul);
                function Po(Ne, je) {
                  return je > 0 ? je + 2 * Ne : Ne;
                }
                var Jo = xc([{ name: "a_pos_offset", components: 4, type: "Int16" }, { name: "a_data", components: 4, type: "Uint16" }, { name: "a_pixeloffset", components: 4, type: "Int16" }], 4), ps = xc([{ name: "a_projected_pos", components: 3, type: "Float32" }], 4), wl = (xc([{ name: "a_fade_opacity", components: 1, type: "Uint32" }], 4), xc([{ name: "a_placed", components: 2, type: "Uint8" }, { name: "a_shift", components: 2, type: "Float32" }])), ol = (xc([{ type: "Int16", name: "anchorPointX" }, { type: "Int16", name: "anchorPointY" }, { type: "Int16", name: "x1" }, { type: "Int16", name: "y1" }, { type: "Int16", name: "x2" }, { type: "Int16", name: "y2" }, { type: "Uint32", name: "featureIndex" }, { type: "Uint16", name: "sourceLayerIndex" }, { type: "Uint16", name: "bucketIndex" }]), xc([{ name: "a_pos", components: 2, type: "Int16" }, { name: "a_anchor_pos", components: 2, type: "Int16" }, { name: "a_extrude", components: 2, type: "Int16" }], 4)), xs = xc([{ name: "a_pos", components: 2, type: "Float32" }, { name: "a_radius", components: 1, type: "Float32" }, { name: "a_flags", components: 2, type: "Int16" }], 4);
                function ns(Ne, je, ut) {
                  return Ne.sections.forEach(function(kt) {
                    kt.text = function(Jt, fn, Sn) {
                      var Hn = fn.layout.get("text-transform").evaluate(Sn, {});
                      return Hn === "uppercase" ? Jt = Jt.toLocaleUpperCase() : Hn === "lowercase" && (Jt = Jt.toLocaleLowerCase()), Uc.applyArabicShaping && (Jt = Uc.applyArabicShaping(Jt)), Jt;
                    }(kt.text, je, ut);
                  }), Ne;
                }
                xc([{ name: "triangle", components: 3, type: "Uint16" }]), xc([{ type: "Int16", name: "anchorX" }, { type: "Int16", name: "anchorY" }, { type: "Uint16", name: "glyphStartIndex" }, { type: "Uint16", name: "numGlyphs" }, { type: "Uint32", name: "vertexStartIndex" }, { type: "Uint32", name: "lineStartIndex" }, { type: "Uint32", name: "lineLength" }, { type: "Uint16", name: "segment" }, { type: "Uint16", name: "lowerSize" }, { type: "Uint16", name: "upperSize" }, { type: "Float32", name: "lineOffsetX" }, { type: "Float32", name: "lineOffsetY" }, { type: "Uint8", name: "writingMode" }, { type: "Uint8", name: "placedOrientation" }, { type: "Uint8", name: "hidden" }, { type: "Uint32", name: "crossTileID" }, { type: "Int16", name: "associatedIconIndex" }]), xc([{ type: "Int16", name: "anchorX" }, { type: "Int16", name: "anchorY" }, { type: "Int16", name: "rightJustifiedTextSymbolIndex" }, { type: "Int16", name: "centerJustifiedTextSymbolIndex" }, { type: "Int16", name: "leftJustifiedTextSymbolIndex" }, { type: "Int16", name: "verticalPlacedTextSymbolIndex" }, { type: "Int16", name: "placedIconSymbolIndex" }, { type: "Int16", name: "verticalPlacedIconSymbolIndex" }, { type: "Uint16", name: "key" }, { type: "Uint16", name: "textBoxStartIndex" }, { type: "Uint16", name: "textBoxEndIndex" }, { type: "Uint16", name: "verticalTextBoxStartIndex" }, { type: "Uint16", name: "verticalTextBoxEndIndex" }, { type: "Uint16", name: "iconBoxStartIndex" }, { type: "Uint16", name: "iconBoxEndIndex" }, { type: "Uint16", name: "verticalIconBoxStartIndex" }, { type: "Uint16", name: "verticalIconBoxEndIndex" }, { type: "Uint16", name: "featureIndex" }, { type: "Uint16", name: "numHorizontalGlyphVertices" }, { type: "Uint16", name: "numVerticalGlyphVertices" }, { type: "Uint16", name: "numIconVertices" }, { type: "Uint16", name: "numVerticalIconVertices" }, { type: "Uint16", name: "useRuntimeCollisionCircles" }, { type: "Uint32", name: "crossTileID" }, { type: "Float32", name: "textBoxScale" }, { type: "Float32", components: 2, name: "textOffset" }, { type: "Float32", name: "collisionCircleDiameter" }]), xc([{ type: "Float32", name: "offsetX" }]), xc([{ type: "Int16", name: "x" }, { type: "Int16", name: "y" }, { type: "Int16", name: "tileUnitDistanceFromAnchor" }]);
                var ll = { "!": "︕", "#": "＃", $: "＄", "%": "％", "&": "＆", "(": "︵", ")": "︶", "*": "＊", "+": "＋", ",": "︐", "-": "︲", ".": "・", "/": "／", ":": "︓", ";": "︔", "<": "︿", "=": "＝", ">": "﹀", "?": "︖", "@": "＠", "[": "﹇", "\\": "＼", "]": "﹈", "^": "＾", _: "︳", "`": "｀", "{": "︷", "|": "―", "}": "︸", "~": "～", "¢": "￠", "£": "￡", "¥": "￥", "¦": "￤", "¬": "￢", "¯": "￣", "–": "︲", "—": "︱", "‘": "﹃", "’": "﹄", "“": "﹁", "”": "﹂", "…": "︙", "‧": "・", "₩": "￦", "、": "︑", "。": "︒", "〈": "︿", "〉": "﹀", "《": "︽", "》": "︾", "「": "﹁", "」": "﹂", "『": "﹃", "』": "﹄", "【": "︻", "】": "︼", "〔": "︹", "〕": "︺", "〖": "︗", "〗": "︘", "！": "︕", "（": "︵", "）": "︶", "，": "︐", "－": "︲", "．": "・", "：": "︓", "；": "︔", "＜": "︿", "＞": "﹀", "？": "︖", "［": "﹇", "］": "﹈", "＿": "︳", "｛": "︷", "｜": "―", "｝": "︸", "｟": "︵", "｠": "︶", "｡": "︒", "｢": "﹁", "｣": "﹂" }, Ws = 24, ec = function(Ne, je, ut, kt, Jt) {
                  var fn, Sn, Hn = 8 * Jt - kt - 1, nr = (1 << Hn) - 1, kr = nr >> 1, Ir = -7, qr = Jt - 1, ci = -1, ni = Ne[je + qr];
                  for (qr += ci, fn = ni & (1 << -Ir) - 1, ni >>= -Ir, Ir += Hn; Ir > 0; fn = 256 * fn + Ne[je + qr], qr += ci, Ir -= 8) ;
                  for (Sn = fn & (1 << -Ir) - 1, fn >>= -Ir, Ir += kt; Ir > 0; Sn = 256 * Sn + Ne[je + qr], qr += ci, Ir -= 8) ;
                  if (fn === 0) fn = 1 - kr;
                  else {
                    if (fn === nr) return Sn ? NaN : 1 / 0 * (ni ? -1 : 1);
                    Sn += Math.pow(2, kt), fn -= kr;
                  }
                  return (ni ? -1 : 1) * Sn * Math.pow(2, fn - kt);
                }, s1 = function(Ne, je, ut, kt, Jt, fn) {
                  var Sn, Hn, nr, kr = 8 * fn - Jt - 1, Ir = (1 << kr) - 1, qr = Ir >> 1, ci = Jt === 23 ? Math.pow(2, -24) - Math.pow(2, -77) : 0, ni = 0, Ii = 1, Gi = je < 0 || je === 0 && 1 / je < 0 ? 1 : 0;
                  for (je = Math.abs(je), isNaN(je) || je === 1 / 0 ? (Hn = isNaN(je) ? 1 : 0, Sn = Ir) : (Sn = Math.floor(Math.log(je) / Math.LN2), je * (nr = Math.pow(2, -Sn)) < 1 && (Sn--, nr *= 2), (je += Sn + qr >= 1 ? ci / nr : ci * Math.pow(2, 1 - qr)) * nr >= 2 && (Sn++, nr /= 2), Sn + qr >= Ir ? (Hn = 0, Sn = Ir) : Sn + qr >= 1 ? (Hn = (je * nr - 1) * Math.pow(2, Jt), Sn += qr) : (Hn = je * Math.pow(2, qr - 1) * Math.pow(2, Jt), Sn = 0)); Jt >= 8; Ne[ut + ni] = 255 & Hn, ni += Ii, Hn /= 256, Jt -= 8) ;
                  for (Sn = Sn << Jt | Hn, kr += Jt; kr > 0; Ne[ut + ni] = 255 & Sn, ni += Ii, Sn /= 256, kr -= 8) ;
                  Ne[ut + ni - Ii] |= 128 * Gi;
                }, n1 = Js;
                function Js(Ne) {
                  this.buf = ArrayBuffer.isView && ArrayBuffer.isView(Ne) ? Ne : new Uint8Array(Ne || 0), this.pos = 0, this.type = 0, this.length = this.buf.length;
                }
                Js.Varint = 0, Js.Fixed64 = 1, Js.Bytes = 2, Js.Fixed32 = 5;
                var N1 = 4294967296, Xu = 1 / N1, Q1 = typeof TextDecoder > "u" ? null : new TextDecoder("utf8");
                function Qu(Ne) {
                  return Ne.type === Js.Bytes ? Ne.readVarint() + Ne.pos : Ne.pos + 1;
                }
                function r1(Ne, je, ut) {
                  return ut ? 4294967296 * je + (Ne >>> 0) : 4294967296 * (je >>> 0) + (Ne >>> 0);
                }
                function ud(Ne, je, ut) {
                  var kt = je <= 16383 ? 1 : je <= 2097151 ? 2 : je <= 268435455 ? 3 : Math.floor(Math.log(je) / (7 * Math.LN2));
                  ut.realloc(kt);
                  for (var Jt = ut.pos - 1; Jt >= Ne; Jt--) ut.buf[Jt + kt] = ut.buf[Jt];
                }
                function Ed(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeVarint(Ne[ut]);
                }
                function R1(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeSVarint(Ne[ut]);
                }
                function jl(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeFloat(Ne[ut]);
                }
                function Ad(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeDouble(Ne[ut]);
                }
                function Fc(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeBoolean(Ne[ut]);
                }
                function yl(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeFixed32(Ne[ut]);
                }
                function cc(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeSFixed32(Ne[ut]);
                }
                function Bc(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeFixed64(Ne[ut]);
                }
                function U1(Ne, je) {
                  for (var ut = 0; ut < Ne.length; ut++) je.writeSFixed64(Ne[ut]);
                }
                function uf(Ne, je) {
                  return (Ne[je] | Ne[je + 1] << 8 | Ne[je + 2] << 16) + 16777216 * Ne[je + 3];
                }
                function df(Ne, je, ut) {
                  Ne[ut] = je, Ne[ut + 1] = je >>> 8, Ne[ut + 2] = je >>> 16, Ne[ut + 3] = je >>> 24;
                }
                function Xi(Ne, je) {
                  return (Ne[je] | Ne[je + 1] << 8 | Ne[je + 2] << 16) + (Ne[je + 3] << 24);
                }
                Js.prototype = { destroy: function() {
                  this.buf = null;
                }, readFields: function(Ne, je, ut) {
                  for (ut = ut || this.length; this.pos < ut; ) {
                    var kt = this.readVarint(), Jt = kt >> 3, fn = this.pos;
                    this.type = 7 & kt, Ne(Jt, je, this), this.pos === fn && this.skip(kt);
                  }
                  return je;
                }, readMessage: function(Ne, je) {
                  return this.readFields(Ne, je, this.readVarint() + this.pos);
                }, readFixed32: function() {
                  var Ne = uf(this.buf, this.pos);
                  return this.pos += 4, Ne;
                }, readSFixed32: function() {
                  var Ne = Xi(this.buf, this.pos);
                  return this.pos += 4, Ne;
                }, readFixed64: function() {
                  var Ne = uf(this.buf, this.pos) + uf(this.buf, this.pos + 4) * N1;
                  return this.pos += 8, Ne;
                }, readSFixed64: function() {
                  var Ne = uf(this.buf, this.pos) + Xi(this.buf, this.pos + 4) * N1;
                  return this.pos += 8, Ne;
                }, readFloat: function() {
                  var Ne = ec(this.buf, this.pos, !0, 23, 4);
                  return this.pos += 4, Ne;
                }, readDouble: function() {
                  var Ne = ec(this.buf, this.pos, !0, 52, 8);
                  return this.pos += 8, Ne;
                }, readVarint: function(Ne) {
                  var je, ut, kt = this.buf;
                  return je = 127 & (ut = kt[this.pos++]), ut < 128 ? je : (je |= (127 & (ut = kt[this.pos++])) << 7, ut < 128 ? je : (je |= (127 & (ut = kt[this.pos++])) << 14, ut < 128 ? je : (je |= (127 & (ut = kt[this.pos++])) << 21, ut < 128 ? je : function(Jt, fn, Sn) {
                    var Hn, nr, kr = Sn.buf;
                    if (Hn = (112 & (nr = kr[Sn.pos++])) >> 4, nr < 128 || (Hn |= (127 & (nr = kr[Sn.pos++])) << 3, nr < 128) || (Hn |= (127 & (nr = kr[Sn.pos++])) << 10, nr < 128) || (Hn |= (127 & (nr = kr[Sn.pos++])) << 17, nr < 128) || (Hn |= (127 & (nr = kr[Sn.pos++])) << 24, nr < 128) || (Hn |= (1 & (nr = kr[Sn.pos++])) << 31, nr < 128)) return r1(Jt, Hn, fn);
                    throw new Error("Expected varint not more than 10 bytes");
                  }(je |= (15 & (ut = kt[this.pos])) << 28, Ne, this))));
                }, readVarint64: function() {
                  return this.readVarint(!0);
                }, readSVarint: function() {
                  var Ne = this.readVarint();
                  return Ne % 2 == 1 ? (Ne + 1) / -2 : Ne / 2;
                }, readBoolean: function() {
                  return !!this.readVarint();
                }, readString: function() {
                  var Ne = this.readVarint() + this.pos, je = this.pos;
                  return this.pos = Ne, Ne - je >= 12 && Q1 ? function(ut, kt, Jt) {
                    return Q1.decode(ut.subarray(kt, Jt));
                  }(this.buf, je, Ne) : function(ut, kt, Jt) {
                    for (var fn = "", Sn = kt; Sn < Jt; ) {
                      var Hn, nr, kr, Ir = ut[Sn], qr = null, ci = Ir > 239 ? 4 : Ir > 223 ? 3 : Ir > 191 ? 2 : 1;
                      if (Sn + ci > Jt) break;
                      ci === 1 ? Ir < 128 && (qr = Ir) : ci === 2 ? (192 & (Hn = ut[Sn + 1])) == 128 && (qr = (31 & Ir) << 6 | 63 & Hn) <= 127 && (qr = null) : ci === 3 ? (Hn = ut[Sn + 1], nr = ut[Sn + 2], (192 & Hn) == 128 && (192 & nr) == 128 && ((qr = (15 & Ir) << 12 | (63 & Hn) << 6 | 63 & nr) <= 2047 || qr >= 55296 && qr <= 57343) && (qr = null)) : ci === 4 && (Hn = ut[Sn + 1], nr = ut[Sn + 2], kr = ut[Sn + 3], (192 & Hn) == 128 && (192 & nr) == 128 && (192 & kr) == 128 && ((qr = (15 & Ir) << 18 | (63 & Hn) << 12 | (63 & nr) << 6 | 63 & kr) <= 65535 || qr >= 1114112) && (qr = null)), qr === null ? (qr = 65533, ci = 1) : qr > 65535 && (qr -= 65536, fn += String.fromCharCode(qr >>> 10 & 1023 | 55296), qr = 56320 | 1023 & qr), fn += String.fromCharCode(qr), Sn += ci;
                    }
                    return fn;
                  }(this.buf, je, Ne);
                }, readBytes: function() {
                  var Ne = this.readVarint() + this.pos, je = this.buf.subarray(this.pos, Ne);
                  return this.pos = Ne, je;
                }, readPackedVarint: function(Ne, je) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readVarint(je));
                  var ut = Qu(this);
                  for (Ne = Ne || []; this.pos < ut; ) Ne.push(this.readVarint(je));
                  return Ne;
                }, readPackedSVarint: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readSVarint());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readSVarint());
                  return Ne;
                }, readPackedBoolean: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readBoolean());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readBoolean());
                  return Ne;
                }, readPackedFloat: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readFloat());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readFloat());
                  return Ne;
                }, readPackedDouble: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readDouble());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readDouble());
                  return Ne;
                }, readPackedFixed32: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readFixed32());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readFixed32());
                  return Ne;
                }, readPackedSFixed32: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readSFixed32());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readSFixed32());
                  return Ne;
                }, readPackedFixed64: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readFixed64());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readFixed64());
                  return Ne;
                }, readPackedSFixed64: function(Ne) {
                  if (this.type !== Js.Bytes) return Ne.push(this.readSFixed64());
                  var je = Qu(this);
                  for (Ne = Ne || []; this.pos < je; ) Ne.push(this.readSFixed64());
                  return Ne;
                }, skip: function(Ne) {
                  var je = 7 & Ne;
                  if (je === Js.Varint) for (; this.buf[this.pos++] > 127; ) ;
                  else if (je === Js.Bytes) this.pos = this.readVarint() + this.pos;
                  else if (je === Js.Fixed32) this.pos += 4;
                  else {
                    if (je !== Js.Fixed64) throw new Error("Unimplemented type: " + je);
                    this.pos += 8;
                  }
                }, writeTag: function(Ne, je) {
                  this.writeVarint(Ne << 3 | je);
                }, realloc: function(Ne) {
                  for (var je = this.length || 16; je < this.pos + Ne; ) je *= 2;
                  if (je !== this.length) {
                    var ut = new Uint8Array(je);
                    ut.set(this.buf), this.buf = ut, this.length = je;
                  }
                }, finish: function() {
                  return this.length = this.pos, this.pos = 0, this.buf.subarray(0, this.length);
                }, writeFixed32: function(Ne) {
                  this.realloc(4), df(this.buf, Ne, this.pos), this.pos += 4;
                }, writeSFixed32: function(Ne) {
                  this.realloc(4), df(this.buf, Ne, this.pos), this.pos += 4;
                }, writeFixed64: function(Ne) {
                  this.realloc(8), df(this.buf, -1 & Ne, this.pos), df(this.buf, Math.floor(Ne * Xu), this.pos + 4), this.pos += 8;
                }, writeSFixed64: function(Ne) {
                  this.realloc(8), df(this.buf, -1 & Ne, this.pos), df(this.buf, Math.floor(Ne * Xu), this.pos + 4), this.pos += 8;
                }, writeVarint: function(Ne) {
                  (Ne = +Ne || 0) > 268435455 || Ne < 0 ? function(je, ut) {
                    var kt, Jt;
                    if (je >= 0 ? (kt = je % 4294967296 | 0, Jt = je / 4294967296 | 0) : (Jt = ~(-je / 4294967296), 4294967295 ^ (kt = ~(-je % 4294967296)) ? kt = kt + 1 | 0 : (kt = 0, Jt = Jt + 1 | 0)), je >= 18446744073709552e3 || je < -18446744073709552e3) throw new Error("Given varint doesn't fit into 10 bytes");
                    ut.realloc(10), function(fn, Sn, Hn) {
                      Hn.buf[Hn.pos++] = 127 & fn | 128, fn >>>= 7, Hn.buf[Hn.pos++] = 127 & fn | 128, fn >>>= 7, Hn.buf[Hn.pos++] = 127 & fn | 128, fn >>>= 7, Hn.buf[Hn.pos++] = 127 & fn | 128, fn >>>= 7, Hn.buf[Hn.pos] = 127 & fn;
                    }(kt, 0, ut), function(fn, Sn) {
                      var Hn = (7 & fn) << 4;
                      Sn.buf[Sn.pos++] |= Hn | ((fn >>>= 3) ? 128 : 0), fn && (Sn.buf[Sn.pos++] = 127 & fn | ((fn >>>= 7) ? 128 : 0), fn && (Sn.buf[Sn.pos++] = 127 & fn | ((fn >>>= 7) ? 128 : 0), fn && (Sn.buf[Sn.pos++] = 127 & fn | ((fn >>>= 7) ? 128 : 0), fn && (Sn.buf[Sn.pos++] = 127 & fn | ((fn >>>= 7) ? 128 : 0), fn && (Sn.buf[Sn.pos++] = 127 & fn)))));
                    }(Jt, ut);
                  }(Ne, this) : (this.realloc(4), this.buf[this.pos++] = 127 & Ne | (Ne > 127 ? 128 : 0), Ne <= 127 || (this.buf[this.pos++] = 127 & (Ne >>>= 7) | (Ne > 127 ? 128 : 0), Ne <= 127 || (this.buf[this.pos++] = 127 & (Ne >>>= 7) | (Ne > 127 ? 128 : 0), Ne <= 127 || (this.buf[this.pos++] = Ne >>> 7 & 127))));
                }, writeSVarint: function(Ne) {
                  this.writeVarint(Ne < 0 ? 2 * -Ne - 1 : 2 * Ne);
                }, writeBoolean: function(Ne) {
                  this.writeVarint(!!Ne);
                }, writeString: function(Ne) {
                  Ne = String(Ne), this.realloc(4 * Ne.length), this.pos++;
                  var je = this.pos;
                  this.pos = function(kt, Jt, fn) {
                    for (var Sn, Hn, nr = 0; nr < Jt.length; nr++) {
                      if ((Sn = Jt.charCodeAt(nr)) > 55295 && Sn < 57344) {
                        if (!Hn) {
                          Sn > 56319 || nr + 1 === Jt.length ? (kt[fn++] = 239, kt[fn++] = 191, kt[fn++] = 189) : Hn = Sn;
                          continue;
                        }
                        if (Sn < 56320) {
                          kt[fn++] = 239, kt[fn++] = 191, kt[fn++] = 189, Hn = Sn;
                          continue;
                        }
                        Sn = Hn - 55296 << 10 | Sn - 56320 | 65536, Hn = null;
                      } else Hn && (kt[fn++] = 239, kt[fn++] = 191, kt[fn++] = 189, Hn = null);
                      Sn < 128 ? kt[fn++] = Sn : (Sn < 2048 ? kt[fn++] = Sn >> 6 | 192 : (Sn < 65536 ? kt[fn++] = Sn >> 12 | 224 : (kt[fn++] = Sn >> 18 | 240, kt[fn++] = Sn >> 12 & 63 | 128), kt[fn++] = Sn >> 6 & 63 | 128), kt[fn++] = 63 & Sn | 128);
                    }
                    return fn;
                  }(this.buf, Ne, this.pos);
                  var ut = this.pos - je;
                  ut >= 128 && ud(je, ut, this), this.pos = je - 1, this.writeVarint(ut), this.pos += ut;
                }, writeFloat: function(Ne) {
                  this.realloc(4), s1(this.buf, Ne, this.pos, !0, 23, 4), this.pos += 4;
                }, writeDouble: function(Ne) {
                  this.realloc(8), s1(this.buf, Ne, this.pos, !0, 52, 8), this.pos += 8;
                }, writeBytes: function(Ne) {
                  var je = Ne.length;
                  this.writeVarint(je), this.realloc(je);
                  for (var ut = 0; ut < je; ut++) this.buf[this.pos++] = Ne[ut];
                }, writeRawMessage: function(Ne, je) {
                  this.pos++;
                  var ut = this.pos;
                  Ne(je, this);
                  var kt = this.pos - ut;
                  kt >= 128 && ud(ut, kt, this), this.pos = ut - 1, this.writeVarint(kt), this.pos += kt;
                }, writeMessage: function(Ne, je, ut) {
                  this.writeTag(Ne, Js.Bytes), this.writeRawMessage(je, ut);
                }, writePackedVarint: function(Ne, je) {
                  je.length && this.writeMessage(Ne, Ed, je);
                }, writePackedSVarint: function(Ne, je) {
                  je.length && this.writeMessage(Ne, R1, je);
                }, writePackedBoolean: function(Ne, je) {
                  je.length && this.writeMessage(Ne, Fc, je);
                }, writePackedFloat: function(Ne, je) {
                  je.length && this.writeMessage(Ne, jl, je);
                }, writePackedDouble: function(Ne, je) {
                  je.length && this.writeMessage(Ne, Ad, je);
                }, writePackedFixed32: function(Ne, je) {
                  je.length && this.writeMessage(Ne, yl, je);
                }, writePackedSFixed32: function(Ne, je) {
                  je.length && this.writeMessage(Ne, cc, je);
                }, writePackedFixed64: function(Ne, je) {
                  je.length && this.writeMessage(Ne, Bc, je);
                }, writePackedSFixed64: function(Ne, je) {
                  je.length && this.writeMessage(Ne, U1, je);
                }, writeBytesField: function(Ne, je) {
                  this.writeTag(Ne, Js.Bytes), this.writeBytes(je);
                }, writeFixed32Field: function(Ne, je) {
                  this.writeTag(Ne, Js.Fixed32), this.writeFixed32(je);
                }, writeSFixed32Field: function(Ne, je) {
                  this.writeTag(Ne, Js.Fixed32), this.writeSFixed32(je);
                }, writeFixed64Field: function(Ne, je) {
                  this.writeTag(Ne, Js.Fixed64), this.writeFixed64(je);
                }, writeSFixed64Field: function(Ne, je) {
                  this.writeTag(Ne, Js.Fixed64), this.writeSFixed64(je);
                }, writeVarintField: function(Ne, je) {
                  this.writeTag(Ne, Js.Varint), this.writeVarint(je);
                }, writeSVarintField: function(Ne, je) {
                  this.writeTag(Ne, Js.Varint), this.writeSVarint(je);
                }, writeStringField: function(Ne, je) {
                  this.writeTag(Ne, Js.Bytes), this.writeString(je);
                }, writeFloatField: function(Ne, je) {
                  this.writeTag(Ne, Js.Fixed32), this.writeFloat(je);
                }, writeDoubleField: function(Ne, je) {
                  this.writeTag(Ne, Js.Fixed64), this.writeDouble(je);
                }, writeBooleanField: function(Ne, je) {
                  this.writeVarintField(Ne, !!je);
                } };
                var Za = 3;
                function ja(Ne, je, ut) {
                  Ne === 1 && ut.readMessage(Ds, je);
                }
                function Ds(Ne, je, ut) {
                  if (Ne === 3) {
                    var kt = ut.readMessage(rc, {}), Jt = kt.id, fn = kt.bitmap, Sn = kt.width, Hn = kt.height, nr = kt.left, kr = kt.top, Ir = kt.advance;
                    je.push({ id: Jt, bitmap: new jd({ width: Sn + 2 * Za, height: Hn + 2 * Za }, fn), metrics: { width: Sn, height: Hn, left: nr, top: kr, advance: Ir } });
                  }
                }
                function rc(Ne, je, ut) {
                  Ne === 1 ? je.id = ut.readVarint() : Ne === 2 ? je.bitmap = ut.readBytes() : Ne === 3 ? je.width = ut.readVarint() : Ne === 4 ? je.height = ut.readVarint() : Ne === 5 ? je.left = ut.readSVarint() : Ne === 6 ? je.top = ut.readSVarint() : Ne === 7 && (je.advance = ut.readVarint());
                }
                var Di = Za;
                function yi(Ne) {
                  for (var je = 0, ut = 0, kt = 0, Jt = Ne; kt < Jt.length; kt += 1) {
                    var fn = Jt[kt];
                    je += fn.w * fn.h, ut = Math.max(ut, fn.w);
                  }
                  Ne.sort(function(Gi, xa) {
                    return xa.h - Gi.h;
                  });
                  for (var Sn = [{ x: 0, y: 0, w: Math.max(Math.ceil(Math.sqrt(je / 0.95)), ut), h: 1 / 0 }], Hn = 0, nr = 0, kr = 0, Ir = Ne; kr < Ir.length; kr += 1) for (var qr = Ir[kr], ci = Sn.length - 1; ci >= 0; ci--) {
                    var ni = Sn[ci];
                    if (!(qr.w > ni.w || qr.h > ni.h)) {
                      if (qr.x = ni.x, qr.y = ni.y, nr = Math.max(nr, qr.y + qr.h), Hn = Math.max(Hn, qr.x + qr.w), qr.w === ni.w && qr.h === ni.h) {
                        var Ii = Sn.pop();
                        ci < Sn.length && (Sn[ci] = Ii);
                      } else qr.h === ni.h ? (ni.x += qr.w, ni.w -= qr.w) : qr.w === ni.w ? (ni.y += qr.h, ni.h -= qr.h) : (Sn.push({ x: ni.x + qr.w, y: ni.y, w: ni.w - qr.w, h: qr.h }), ni.y += qr.h, ni.h -= qr.h);
                      break;
                    }
                  }
                  return { w: Hn, h: nr, fill: je / (Hn * nr) || 0 };
                }
                var ca = 1, Ma = function(Ne, je) {
                  var ut = je.pixelRatio, kt = je.version, Jt = je.stretchX, fn = je.stretchY, Sn = je.content;
                  this.paddedRect = Ne, this.pixelRatio = ut, this.stretchX = Jt, this.stretchY = fn, this.content = Sn, this.version = kt;
                }, Ia = { tl: { configurable: !0 }, br: { configurable: !0 }, tlbr: { configurable: !0 }, displaySize: { configurable: !0 } };
                Ia.tl.get = function() {
                  return [this.paddedRect.x + ca, this.paddedRect.y + ca];
                }, Ia.br.get = function() {
                  return [this.paddedRect.x + this.paddedRect.w - ca, this.paddedRect.y + this.paddedRect.h - ca];
                }, Ia.tlbr.get = function() {
                  return this.tl.concat(this.br);
                }, Ia.displaySize.get = function() {
                  return [(this.paddedRect.w - 2 * ca) / this.pixelRatio, (this.paddedRect.h - 2 * ca) / this.pixelRatio];
                }, Object.defineProperties(Ma.prototype, Ia);
                var ba = function(Ne, je) {
                  var ut = {}, kt = {};
                  this.haveRenderCallbacks = [];
                  var Jt = [];
                  this.addImages(Ne, ut, Jt), this.addImages(je, kt, Jt);
                  var fn = yi(Jt), Sn = fn.w, Hn = fn.h, nr = new E1({ width: Sn || 1, height: Hn || 1 });
                  for (var kr in Ne) {
                    var Ir = Ne[kr], qr = ut[kr].paddedRect;
                    E1.copy(Ir.data, nr, { x: 0, y: 0 }, { x: qr.x + ca, y: qr.y + ca }, Ir.data);
                  }
                  for (var ci in je) {
                    var ni = je[ci], Ii = kt[ci].paddedRect, Gi = Ii.x + ca, xa = Ii.y + ca, ga = ni.data.width, $a = ni.data.height;
                    E1.copy(ni.data, nr, { x: 0, y: 0 }, { x: Gi, y: xa }, ni.data), E1.copy(ni.data, nr, { x: 0, y: $a - 1 }, { x: Gi, y: xa - 1 }, { width: ga, height: 1 }), E1.copy(ni.data, nr, { x: 0, y: 0 }, { x: Gi, y: xa + $a }, { width: ga, height: 1 }), E1.copy(ni.data, nr, { x: ga - 1, y: 0 }, { x: Gi - 1, y: xa }, { width: 1, height: $a }), E1.copy(ni.data, nr, { x: 0, y: 0 }, { x: Gi + ga, y: xa }, { width: 1, height: $a });
                  }
                  this.image = nr, this.iconPositions = ut, this.patternPositions = kt;
                };
                ba.prototype.addImages = function(Ne, je, ut) {
                  for (var kt in Ne) {
                    var Jt = Ne[kt], fn = { x: 0, y: 0, w: Jt.data.width + 2 * ca, h: Jt.data.height + 2 * ca };
                    ut.push(fn), je[kt] = new Ma(fn, Jt), Jt.hasRenderCallback && this.haveRenderCallbacks.push(kt);
                  }
                }, ba.prototype.patchUpdatedImages = function(Ne, je) {
                  for (var ut in Ne.dispatchRenderCallbacks(this.haveRenderCallbacks), Ne.updatedImages) this.patchUpdatedImage(this.iconPositions[ut], Ne.getImage(ut), je), this.patchUpdatedImage(this.patternPositions[ut], Ne.getImage(ut), je);
                }, ba.prototype.patchUpdatedImage = function(Ne, je, ut) {
                  if (Ne && je && Ne.version !== je.version) {
                    Ne.version = je.version;
                    var kt = Ne.tl, Jt = kt[0], fn = kt[1];
                    ut.update(je.data, void 0, { x: Jt, y: fn });
                  }
                }, cs("ImagePosition", Ma), cs("ImageAtlas", ba);
                var fa = { horizontal: 1, vertical: 2, horizontalOnly: 3 }, Ea = -17, Hi = function() {
                  this.scale = 1, this.fontStack = "", this.imageName = null;
                };
                Hi.forText = function(Ne, je) {
                  var ut = new Hi();
                  return ut.scale = Ne || 1, ut.fontStack = je, ut;
                }, Hi.forImage = function(Ne) {
                  var je = new Hi();
                  return je.imageName = Ne, je;
                };
                var ji = function() {
                  this.text = "", this.sectionIndex = [], this.sections = [], this.imageSectionID = null;
                };
                function qi(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi) {
                  var xa, ga = ji.fromFeature(Ne, Jt);
                  qr === fa.vertical && ga.verticalizePunctuation();
                  var $a = Uc.processBidirectionalText, Va = Uc.processStyledBidirectionalText;
                  if ($a && ga.sections.length === 1) {
                    xa = [];
                    for (var eo = 0, co = $a(ga.toString(), Cs(ga, kr, fn, je, kt, ni, Ii)); eo < co.length; eo += 1) {
                      var wo = co[eo], ds = new ji();
                      ds.text = wo, ds.sections = ga.sections;
                      for (var Lo = 0; Lo < wo.length; Lo++) ds.sectionIndex.push(0);
                      xa.push(ds);
                    }
                  } else if (Va) {
                    xa = [];
                    for (var As = 0, Fs = Va(ga.text, ga.sectionIndex, Cs(ga, kr, fn, je, kt, ni, Ii)); As < Fs.length; As += 1) {
                      var Pl = Fs[As], rl = new ji();
                      rl.text = Pl[0], rl.sectionIndex = Pl[1], rl.sections = ga.sections, xa.push(rl);
                    }
                  } else xa = function(Zs, Hc) {
                    for (var tc = [], Pu = Zs.text, Cl = 0, vu = 0, qu = Hc; vu < qu.length; vu += 1) {
                      var Nu = qu[vu];
                      tc.push(Zs.substring(Cl, Nu)), Cl = Nu;
                    }
                    return Cl < Pu.length && tc.push(Zs.substring(Cl, Pu.length)), tc;
                  }(ga, Cs(ga, kr, fn, je, kt, ni, Ii));
                  var Rl = [], pl = { positionedLines: Rl, text: ga.toString(), top: Ir[1], bottom: Ir[1], left: Ir[0], right: Ir[0], writingMode: qr, iconsInText: !1, verticalizable: !1 };
                  return function(Zs, Hc, tc, Pu, Cl, vu, qu, Nu, ju, $u, $1, Zu) {
                    for (var F1 = 0, Td = Ea, hd = 0, Dd = 0, Qd = Nu === "right" ? 1 : Nu === "left" ? 0 : 0.5, Ye = 0, He = 0, ot = Cl; He < ot.length; He += 1) {
                      var xt = ot[He];
                      xt.trim();
                      var zt = xt.getMaxScale(), ln = (zt - 1) * Ws, pn = { positionedGlyphs: [], lineOffset: 0 };
                      Zs.positionedLines[Ye] = pn;
                      var Cn = pn.positionedGlyphs, Nn = 0;
                      if (xt.length()) {
                        for (var Wn = 0; Wn < xt.length(); Wn++) {
                          var lr = xt.getSection(Wn), xr = xt.getSectionIndex(Wn), Nr = xt.getCharCode(Wn), Kr = 0, xi = null, ui = null, Ui = null, ia = Ws, Na = !(ju === fa.horizontal || !$1 && !dc(Nr) || $1 && (Ua[Nr] || Eu(Nr)));
                          if (lr.imageName) {
                            var Ta = Pu[lr.imageName];
                            if (!Ta) continue;
                            Ui = lr.imageName, Zs.iconsInText = Zs.iconsInText || !0, ui = Ta.paddedRect;
                            var Ra = Ta.displaySize;
                            lr.scale = lr.scale * Ws / Zu, xi = { width: Ra[0], height: Ra[1], left: ca, top: -3, advance: Na ? Ra[1] : Ra[0] }, Kr = ln + (Ws - Ra[1] * lr.scale), ia = xi.advance;
                            var oo = Na ? Ra[0] * lr.scale - Ws * zt : Ra[1] * lr.scale - Ws * zt;
                            oo > 0 && oo > Nn && (Nn = oo);
                          } else {
                            var fs = tc[lr.fontStack], Bs = fs && fs[Nr];
                            if (Bs && Bs.rect) ui = Bs.rect, xi = Bs.metrics;
                            else {
                              var fl = Hc[lr.fontStack], Ns = fl && fl[Nr];
                              if (!Ns) continue;
                              xi = Ns.metrics;
                            }
                            Kr = (zt - lr.scale) * Ws;
                          }
                          Na ? (Zs.verticalizable = !0, Cn.push({ glyph: Nr, imageName: Ui, x: F1, y: Td + Kr, vertical: Na, scale: lr.scale, fontStack: lr.fontStack, sectionIndex: xr, metrics: xi, rect: ui }), F1 += ia * lr.scale + $u) : (Cn.push({ glyph: Nr, imageName: Ui, x: F1, y: Td + Kr, vertical: Na, scale: lr.scale, fontStack: lr.fontStack, sectionIndex: xr, metrics: xi, rect: ui }), F1 += xi.advance * lr.scale + $u);
                        }
                        if (Cn.length !== 0) {
                          var Es = F1 - $u;
                          hd = Math.max(Es, hd), fo(Cn, 0, Cn.length - 1, Qd, Nn);
                        }
                        F1 = 0;
                        var cl = vu * zt + Nn;
                        pn.lineOffset = Math.max(Nn, ln), Td += cl, Dd = Math.max(cl, Dd), ++Ye;
                      } else Td += vu, ++Ye;
                    }
                    var Xs = Td - Ea, Ps = no(qu), ko = Ps.horizontalAlign, Ks = Ps.verticalAlign;
                    (function(el, Nl, bu, Tu, B1, Hd, nd, j1, Fd) {
                      var Cd = (Nl - bu) * B1, L1 = 0;
                      L1 = Hd !== nd ? -j1 * Tu - Ea : (-Tu * Fd + 0.5) * nd;
                      for (var m1 = 0, xd = el; m1 < xd.length; m1 += 1) for (var H1 = 0, rd = xd[m1].positionedGlyphs; H1 < rd.length; H1 += 1) {
                        var Ld = rd[H1];
                        Ld.x += Cd, Ld.y += L1;
                      }
                    })(Zs.positionedLines, Qd, ko, Ks, hd, Dd, vu, Xs, Cl.length), Zs.top += -Ks * Xs, Zs.bottom = Zs.top + Xs, Zs.left += -ko * hd, Zs.right = Zs.left + hd;
                  }(pl, je, ut, kt, xa, Sn, Hn, nr, qr, kr, ci, Gi), !function(Zs) {
                    for (var Hc = 0, tc = Zs; Hc < tc.length; Hc += 1) if (tc[Hc].positionedGlyphs.length !== 0) return !1;
                    return !0;
                  }(Rl) && pl;
                }
                ji.fromFeature = function(Ne, je) {
                  for (var ut = new ji(), kt = 0; kt < Ne.sections.length; kt++) {
                    var Jt = Ne.sections[kt];
                    Jt.image ? ut.addImageSection(Jt) : ut.addTextSection(Jt, je);
                  }
                  return ut;
                }, ji.prototype.length = function() {
                  return this.text.length;
                }, ji.prototype.getSection = function(Ne) {
                  return this.sections[this.sectionIndex[Ne]];
                }, ji.prototype.getSectionIndex = function(Ne) {
                  return this.sectionIndex[Ne];
                }, ji.prototype.getCharCode = function(Ne) {
                  return this.text.charCodeAt(Ne);
                }, ji.prototype.verticalizePunctuation = function() {
                  this.text = function(Ne) {
                    for (var je = "", ut = 0; ut < Ne.length; ut++) {
                      var kt = Ne.charCodeAt(ut + 1) || null, Jt = Ne.charCodeAt(ut - 1) || null;
                      kt && vl(kt) && !ll[Ne[ut + 1]] || Jt && vl(Jt) && !ll[Ne[ut - 1]] || !ll[Ne[ut]] ? je += Ne[ut] : je += ll[Ne[ut]];
                    }
                    return je;
                  }(this.text);
                }, ji.prototype.trim = function() {
                  for (var Ne = 0, je = 0; je < this.text.length && Ua[this.text.charCodeAt(je)]; je++) Ne++;
                  for (var ut = this.text.length, kt = this.text.length - 1; kt >= 0 && kt >= Ne && Ua[this.text.charCodeAt(kt)]; kt--) ut--;
                  this.text = this.text.substring(Ne, ut), this.sectionIndex = this.sectionIndex.slice(Ne, ut);
                }, ji.prototype.substring = function(Ne, je) {
                  var ut = new ji();
                  return ut.text = this.text.substring(Ne, je), ut.sectionIndex = this.sectionIndex.slice(Ne, je), ut.sections = this.sections, ut;
                }, ji.prototype.toString = function() {
                  return this.text;
                }, ji.prototype.getMaxScale = function() {
                  var Ne = this;
                  return this.sectionIndex.reduce(function(je, ut) {
                    return Math.max(je, Ne.sections[ut].scale);
                  }, 0);
                }, ji.prototype.addTextSection = function(Ne, je) {
                  this.text += Ne.text, this.sections.push(Hi.forText(Ne.scale, Ne.fontStack || je));
                  for (var ut = this.sections.length - 1, kt = 0; kt < Ne.text.length; ++kt) this.sectionIndex.push(ut);
                }, ji.prototype.addImageSection = function(Ne) {
                  var je = Ne.image ? Ne.image.name : "";
                  if (je.length !== 0) {
                    var ut = this.getNextImageSectionCharCode();
                    ut ? (this.text += String.fromCharCode(ut), this.sections.push(Hi.forImage(je)), this.sectionIndex.push(this.sections.length - 1)) : Me("Reached maximum number of images 6401");
                  } else Me("Can't add FormattedSection with an empty image.");
                }, ji.prototype.getNextImageSectionCharCode = function() {
                  return this.imageSectionID ? this.imageSectionID >= 63743 ? null : ++this.imageSectionID : (this.imageSectionID = 57344, this.imageSectionID);
                };
                var Ua = { 9: !0, 10: !0, 11: !0, 12: !0, 13: !0, 32: !0 }, La = {};
                function yo(Ne, je, ut, kt, Jt, fn) {
                  if (je.imageName) {
                    var Sn = kt[je.imageName];
                    return Sn ? Sn.displaySize[0] * je.scale * Ws / fn + Jt : 0;
                  }
                  var Hn = ut[je.fontStack], nr = Hn && Hn[Ne];
                  return nr ? nr.metrics.advance * je.scale + Jt : 0;
                }
                function xo(Ne, je, ut, kt) {
                  var Jt = Math.pow(Ne - je, 2);
                  return kt ? Ne < je ? Jt / 2 : 2 * Jt : Jt + Math.abs(ut) * ut;
                }
                function rs(Ne, je, ut) {
                  var kt = 0;
                  return Ne === 10 && (kt -= 1e4), ut && (kt += 150), Ne !== 40 && Ne !== 65288 || (kt += 50), je !== 41 && je !== 65289 || (kt += 50), kt;
                }
                function Us(Ne, je, ut, kt, Jt, fn) {
                  for (var Sn = null, Hn = xo(je, ut, Jt, fn), nr = 0, kr = kt; nr < kr.length; nr += 1) {
                    var Ir = kr[nr], qr = xo(je - Ir.x, ut, Jt, fn) + Ir.badness;
                    qr <= Hn && (Sn = Ir, Hn = qr);
                  }
                  return { index: Ne, x: je, priorBreak: Sn, badness: Hn };
                }
                function sl(Ne) {
                  return Ne ? sl(Ne.priorBreak).concat(Ne.index) : [];
                }
                function Cs(Ne, je, ut, kt, Jt, fn, Sn) {
                  if (fn !== "point") return [];
                  if (!Ne) return [];
                  for (var Hn = [], nr = function(xa, ga, $a, Va, eo, co) {
                    for (var wo = 0, ds = 0; ds < xa.length(); ds++) {
                      var Lo = xa.getSection(ds);
                      wo += yo(xa.getCharCode(ds), Lo, Va, eo, ga, co);
                    }
                    return wo / Math.max(1, Math.ceil(wo / $a));
                  }(Ne, je, ut, kt, Jt, Sn), kr = Ne.text.indexOf("​") >= 0, Ir = 0, qr = 0; qr < Ne.length(); qr++) {
                    var ci = Ne.getSection(qr), ni = Ne.getCharCode(qr);
                    if (Ua[ni] || (Ir += yo(ni, ci, kt, Jt, je, Sn)), qr < Ne.length() - 1) {
                      var Ii = !((Gi = ni) < 11904 || !(Do["Bopomofo Extended"](Gi) || Do.Bopomofo(Gi) || Do["CJK Compatibility Forms"](Gi) || Do["CJK Compatibility Ideographs"](Gi) || Do["CJK Compatibility"](Gi) || Do["CJK Radicals Supplement"](Gi) || Do["CJK Strokes"](Gi) || Do["CJK Symbols and Punctuation"](Gi) || Do["CJK Unified Ideographs Extension A"](Gi) || Do["CJK Unified Ideographs"](Gi) || Do["Enclosed CJK Letters and Months"](Gi) || Do["Halfwidth and Fullwidth Forms"](Gi) || Do.Hiragana(Gi) || Do["Ideographic Description Characters"](Gi) || Do["Kangxi Radicals"](Gi) || Do["Katakana Phonetic Extensions"](Gi) || Do.Katakana(Gi) || Do["Vertical Forms"](Gi) || Do["Yi Radicals"](Gi) || Do["Yi Syllables"](Gi)));
                      (La[ni] || Ii || ci.imageName) && Hn.push(Us(qr + 1, Ir, nr, Hn, rs(ni, Ne.getCharCode(qr + 1), Ii && kr), !1));
                    }
                  }
                  var Gi;
                  return sl(Us(Ne.length(), Ir, nr, Hn, 0, !0));
                }
                function no(Ne) {
                  var je = 0.5, ut = 0.5;
                  switch (Ne) {
                    case "right":
                    case "top-right":
                    case "bottom-right":
                      je = 1;
                      break;
                    case "left":
                    case "top-left":
                    case "bottom-left":
                      je = 0;
                  }
                  switch (Ne) {
                    case "bottom":
                    case "bottom-right":
                    case "bottom-left":
                      ut = 1;
                      break;
                    case "top":
                    case "top-right":
                    case "top-left":
                      ut = 0;
                  }
                  return { horizontalAlign: je, verticalAlign: ut };
                }
                function fo(Ne, je, ut, kt, Jt) {
                  if (kt || Jt) for (var fn = Ne[ut], Sn = fn.metrics.advance * fn.scale, Hn = (Ne[ut].x + Sn) * kt, nr = je; nr <= ut; nr++) Ne[nr].x -= Hn, Ne[nr].y += Jt;
                }
                function Ys(Ne, je, ut, kt, Jt, fn) {
                  var Sn, Hn = Ne.image;
                  if (Hn.content) {
                    var nr = Hn.content, kr = Hn.pixelRatio || 1;
                    Sn = [nr[0] / kr, nr[1] / kr, Hn.displaySize[0] - nr[2] / kr, Hn.displaySize[1] - nr[3] / kr];
                  }
                  var Ir, qr, ci, ni, Ii = je.left * fn, Gi = je.right * fn;
                  ut === "width" || ut === "both" ? (ni = Jt[0] + Ii - kt[3], qr = Jt[0] + Gi + kt[1]) : qr = (ni = Jt[0] + (Ii + Gi - Hn.displaySize[0]) / 2) + Hn.displaySize[0];
                  var xa = je.top * fn, ga = je.bottom * fn;
                  return ut === "height" || ut === "both" ? (Ir = Jt[1] + xa - kt[0], ci = Jt[1] + ga + kt[2]) : ci = (Ir = Jt[1] + (xa + ga - Hn.displaySize[1]) / 2) + Hn.displaySize[1], { image: Hn, top: Ir, right: qr, bottom: ci, left: ni, collisionPadding: Sn };
                }
                La[10] = !0, La[32] = !0, La[38] = !0, La[40] = !0, La[41] = !0, La[43] = !0, La[45] = !0, La[47] = !0, La[173] = !0, La[183] = !0, La[8203] = !0, La[8208] = !0, La[8211] = !0, La[8231] = !0;
                var Rs = function(Ne) {
                  function je(ut, kt, Jt, fn) {
                    Ne.call(this, ut, kt), this.angle = Jt, fn !== void 0 && (this.segment = fn);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.clone = function() {
                    return new je(this.x, this.y, this.angle, this.segment);
                  }, je;
                }(le);
                cs("Anchor", Rs);
                var js = 128;
                function Mc(Ne, je) {
                  var ut = je.expression;
                  if (ut.kind === "constant") return { kind: "constant", layoutSize: ut.evaluate(new El(Ne + 1)) };
                  if (ut.kind === "source") return { kind: "source" };
                  for (var kt = ut.zoomStops, Jt = ut.interpolationType, fn = 0; fn < kt.length && kt[fn] <= Ne; ) fn++;
                  for (var Sn = fn = Math.max(0, fn - 1); Sn < kt.length && kt[Sn] < Ne + 1; ) Sn++;
                  Sn = Math.min(kt.length - 1, Sn);
                  var Hn = kt[fn], nr = kt[Sn];
                  return ut.kind === "composite" ? { kind: "composite", minZoom: Hn, maxZoom: nr, interpolationType: Jt } : { kind: "camera", minZoom: Hn, maxZoom: nr, minSize: ut.evaluate(new El(Hn)), maxSize: ut.evaluate(new El(nr)), interpolationType: Jt };
                }
                function dd(Ne, je, ut) {
                  var kt = je.uSize, Jt = je.uSizeT, fn = ut.lowerSize, Sn = ut.upperSize;
                  return Ne.kind === "source" ? fn / js : Ne.kind === "composite" ? Tn(fn / js, Sn / js, Jt) : kt;
                }
                function nh(Ne, je) {
                  var ut = 0, kt = 0;
                  if (Ne.kind === "constant") kt = Ne.layoutSize;
                  else if (Ne.kind !== "source") {
                    var Jt = Ne.interpolationType, fn = Ne.minZoom, Sn = Ne.maxZoom, Hn = Jt ? pe(Vl.interpolationFactor(Jt, je, fn, Sn), 0, 1) : 0;
                    Ne.kind === "camera" ? kt = Tn(Ne.minSize, Ne.maxSize, Hn) : ut = Hn;
                  }
                  return { uSizeT: ut, uSize: kt };
                }
                var Vf = Object.freeze({ __proto__: null, getSizeData: Mc, evaluateSizeForFeature: dd, evaluateSizeForZoom: nh, SIZE_PACK_FACTOR: js });
                function vf(Ne, je, ut, kt, Jt) {
                  if (je.segment === void 0) return !0;
                  for (var fn = je, Sn = je.segment + 1, Hn = 0; Hn > -ut / 2; ) {
                    if (--Sn < 0) return !1;
                    Hn -= Ne[Sn].dist(fn), fn = Ne[Sn];
                  }
                  Hn += Ne[Sn].dist(Ne[Sn + 1]), Sn++;
                  for (var nr = [], kr = 0; Hn < ut / 2; ) {
                    var Ir = Ne[Sn - 1], qr = Ne[Sn], ci = Ne[Sn + 1];
                    if (!ci) return !1;
                    var ni = Ir.angleTo(qr) - qr.angleTo(ci);
                    for (ni = Math.abs((ni + 3 * Math.PI) % (2 * Math.PI) - Math.PI), nr.push({ distance: Hn, angleDelta: ni }), kr += ni; Hn - nr[0].distance > kt; ) kr -= nr.shift().angleDelta;
                    if (kr > Jt) return !1;
                    Sn++, Hn += qr.dist(ci);
                  }
                  return !0;
                }
                function Uf(Ne) {
                  for (var je = 0, ut = 0; ut < Ne.length - 1; ut++) je += Ne[ut].dist(Ne[ut + 1]);
                  return je;
                }
                function rh(Ne, je, ut) {
                  return Ne ? 0.6 * je * ut : 0;
                }
                function B2(Ne, je) {
                  return Math.max(Ne ? Ne.right - Ne.left : 0, je ? je.right - je.left : 0);
                }
                function Kd(Ne, je, ut, kt, Jt, fn) {
                  for (var Sn = rh(ut, Jt, fn), Hn = B2(ut, kt) * fn, nr = 0, kr = Uf(Ne) / 2, Ir = 0; Ir < Ne.length - 1; Ir++) {
                    var qr = Ne[Ir], ci = Ne[Ir + 1], ni = qr.dist(ci);
                    if (nr + ni > kr) {
                      var Ii = (kr - nr) / ni, Gi = Tn(qr.x, ci.x, Ii), xa = Tn(qr.y, ci.y, Ii), ga = new Rs(Gi, xa, ci.angleTo(qr), Ir);
                      return ga._round(), !Sn || vf(Ne, ga, Hn, Sn, je) ? ga : void 0;
                    }
                    nr += ni;
                  }
                }
                function mh(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr) {
                  var kr = rh(kt, fn, Sn), Ir = B2(kt, Jt), qr = Ir * Sn, ci = Ne[0].x === 0 || Ne[0].x === nr || Ne[0].y === 0 || Ne[0].y === nr;
                  return je - qr < je / 4 && (je = qr + je / 4), Jd(Ne, ci ? je / 2 * Hn % je : (Ir / 2 + 2 * fn) * Sn * Hn % je, je, kr, ut, qr, ci, !1, nr);
                }
                function Jd(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr) {
                  for (var kr = fn / 2, Ir = Uf(Ne), qr = 0, ci = je - ut, ni = [], Ii = 0; Ii < Ne.length - 1; Ii++) {
                    for (var Gi = Ne[Ii], xa = Ne[Ii + 1], ga = Gi.dist(xa), $a = xa.angleTo(Gi); ci + ut < qr + ga; ) {
                      var Va = ((ci += ut) - qr) / ga, eo = Tn(Gi.x, xa.x, Va), co = Tn(Gi.y, xa.y, Va);
                      if (eo >= 0 && eo < nr && co >= 0 && co < nr && ci - kr >= 0 && ci + kr <= Ir) {
                        var wo = new Rs(eo, co, $a, Ii);
                        wo._round(), kt && !vf(Ne, wo, fn, kt, Jt) || ni.push(wo);
                      }
                    }
                    qr += ga;
                  }
                  return Hn || ni.length || Sn || (ni = Jd(Ne, qr / 2, ut, kt, Jt, fn, Sn, !0, nr)), ni;
                }
                function Dh(Ne, je, ut, kt, Jt) {
                  for (var fn = [], Sn = 0; Sn < Ne.length; Sn++) for (var Hn = Ne[Sn], nr = void 0, kr = 0; kr < Hn.length - 1; kr++) {
                    var Ir = Hn[kr], qr = Hn[kr + 1];
                    Ir.x < je && qr.x < je || (Ir.x < je ? Ir = new le(je, Ir.y + (qr.y - Ir.y) * ((je - Ir.x) / (qr.x - Ir.x)))._round() : qr.x < je && (qr = new le(je, Ir.y + (qr.y - Ir.y) * ((je - Ir.x) / (qr.x - Ir.x)))._round()), Ir.y < ut && qr.y < ut || (Ir.y < ut ? Ir = new le(Ir.x + (qr.x - Ir.x) * ((ut - Ir.y) / (qr.y - Ir.y)), ut)._round() : qr.y < ut && (qr = new le(Ir.x + (qr.x - Ir.x) * ((ut - Ir.y) / (qr.y - Ir.y)), ut)._round()), Ir.x >= kt && qr.x >= kt || (Ir.x >= kt ? Ir = new le(kt, Ir.y + (qr.y - Ir.y) * ((kt - Ir.x) / (qr.x - Ir.x)))._round() : qr.x >= kt && (qr = new le(kt, Ir.y + (qr.y - Ir.y) * ((kt - Ir.x) / (qr.x - Ir.x)))._round()), Ir.y >= Jt && qr.y >= Jt || (Ir.y >= Jt ? Ir = new le(Ir.x + (qr.x - Ir.x) * ((Jt - Ir.y) / (qr.y - Ir.y)), Jt)._round() : qr.y >= Jt && (qr = new le(Ir.x + (qr.x - Ir.x) * ((Jt - Ir.y) / (qr.y - Ir.y)), Jt)._round()), nr && Ir.equals(nr[nr.length - 1]) || (nr = [Ir], fn.push(nr)), nr.push(qr)))));
                  }
                  return fn;
                }
                var P1 = ca;
                function Nh(Ne, je, ut, kt) {
                  var Jt = [], fn = Ne.image, Sn = fn.pixelRatio, Hn = fn.paddedRect.w - 2 * P1, nr = fn.paddedRect.h - 2 * P1, kr = Ne.right - Ne.left, Ir = Ne.bottom - Ne.top, qr = fn.stretchX || [[0, Hn]], ci = fn.stretchY || [[0, nr]], ni = function(vu, qu) {
                    return vu + qu[1] - qu[0];
                  }, Ii = qr.reduce(ni, 0), Gi = ci.reduce(ni, 0), xa = Hn - Ii, ga = nr - Gi, $a = 0, Va = Ii, eo = 0, co = Gi, wo = 0, ds = xa, Lo = 0, As = ga;
                  if (fn.content && kt) {
                    var Fs = fn.content;
                    $a = qf(qr, 0, Fs[0]), eo = qf(ci, 0, Fs[1]), Va = qf(qr, Fs[0], Fs[2]), co = qf(ci, Fs[1], Fs[3]), wo = Fs[0] - $a, Lo = Fs[1] - eo, ds = Fs[2] - Fs[0] - Va, As = Fs[3] - Fs[1] - co;
                  }
                  var Pl = function(vu, qu, Nu, ju) {
                    var $u = U0(vu.stretch - $a, Va, kr, Ne.left), $1 = q0(vu.fixed - wo, ds, vu.stretch, Ii), Zu = U0(qu.stretch - eo, co, Ir, Ne.top), F1 = q0(qu.fixed - Lo, As, qu.stretch, Gi), Td = U0(Nu.stretch - $a, Va, kr, Ne.left), hd = q0(Nu.fixed - wo, ds, Nu.stretch, Ii), Dd = U0(ju.stretch - eo, co, Ir, Ne.top), Qd = q0(ju.fixed - Lo, As, ju.stretch, Gi), Ye = new le($u, Zu), He = new le(Td, Zu), ot = new le(Td, Dd), xt = new le($u, Dd), zt = new le($1 / Sn, F1 / Sn), ln = new le(hd / Sn, Qd / Sn), pn = je * Math.PI / 180;
                    if (pn) {
                      var Cn = Math.sin(pn), Nn = Math.cos(pn), Wn = [Nn, -Cn, Cn, Nn];
                      Ye._matMult(Wn), He._matMult(Wn), xt._matMult(Wn), ot._matMult(Wn);
                    }
                    var lr = vu.stretch + vu.fixed, xr = Nu.stretch + Nu.fixed, Nr = qu.stretch + qu.fixed, Kr = ju.stretch + ju.fixed;
                    return { tl: Ye, tr: He, bl: xt, br: ot, tex: { x: fn.paddedRect.x + P1 + lr, y: fn.paddedRect.y + P1 + Nr, w: xr - lr, h: Kr - Nr }, writingMode: void 0, glyphOffset: [0, 0], sectionIndex: 0, pixelOffsetTL: zt, pixelOffsetBR: ln, minFontScaleX: ds / Sn / kr, minFontScaleY: As / Sn / Ir, isSDF: ut };
                  };
                  if (kt && (fn.stretchX || fn.stretchY)) for (var rl = V0(qr, xa, Ii), Rl = V0(ci, ga, Gi), pl = 0; pl < rl.length - 1; pl++) for (var Zs = rl[pl], Hc = rl[pl + 1], tc = 0; tc < Rl.length - 1; tc++) {
                    var Pu = Rl[tc], Cl = Rl[tc + 1];
                    Jt.push(Pl(Zs, Pu, Hc, Cl));
                  }
                  else Jt.push(Pl({ fixed: 0, stretch: -1 }, { fixed: 0, stretch: -1 }, { fixed: 0, stretch: Hn + 1 }, { fixed: 0, stretch: nr + 1 }));
                  return Jt;
                }
                function qf(Ne, je, ut) {
                  for (var kt = 0, Jt = 0, fn = Ne; Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt];
                    kt += Math.max(je, Math.min(ut, Sn[1])) - Math.max(je, Math.min(ut, Sn[0]));
                  }
                  return kt;
                }
                function V0(Ne, je, ut) {
                  for (var kt = [{ fixed: -1, stretch: 0 }], Jt = 0, fn = Ne; Jt < fn.length; Jt += 1) {
                    var Sn = fn[Jt], Hn = Sn[0], nr = Sn[1], kr = kt[kt.length - 1];
                    kt.push({ fixed: Hn - kr.stretch, stretch: kr.stretch }), kt.push({ fixed: Hn - kr.stretch, stretch: kr.stretch + (nr - Hn) });
                  }
                  return kt.push({ fixed: je + P1, stretch: ut }), kt;
                }
                function U0(Ne, je, ut, kt) {
                  return Ne / je * ut + kt;
                }
                function q0(Ne, je, ut, kt) {
                  return Ne - je * ut / kt;
                }
                var j0 = function(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr, kr) {
                  if (this.boxStartIndex = Ne.length, nr) {
                    var Ir = fn.top, qr = fn.bottom, ci = fn.collisionPadding;
                    ci && (Ir -= ci[1], qr += ci[3]);
                    var ni = qr - Ir;
                    ni > 0 && (ni = Math.max(10, ni), this.circleDiameter = ni);
                  } else {
                    var Ii = fn.top * Sn - Hn, Gi = fn.bottom * Sn + Hn, xa = fn.left * Sn - Hn, ga = fn.right * Sn + Hn, $a = fn.collisionPadding;
                    if ($a && (xa -= $a[0] * Sn, Ii -= $a[1] * Sn, ga += $a[2] * Sn, Gi += $a[3] * Sn), kr) {
                      var Va = new le(xa, Ii), eo = new le(ga, Ii), co = new le(xa, Gi), wo = new le(ga, Gi), ds = kr * Math.PI / 180;
                      Va._rotate(ds), eo._rotate(ds), co._rotate(ds), wo._rotate(ds), xa = Math.min(Va.x, eo.x, co.x, wo.x), ga = Math.max(Va.x, eo.x, co.x, wo.x), Ii = Math.min(Va.y, eo.y, co.y, wo.y), Gi = Math.max(Va.y, eo.y, co.y, wo.y);
                    }
                    Ne.emplaceBack(je.x, je.y, xa, Ii, ga, Gi, ut, kt, Jt);
                  }
                  this.boxEndIndex = Ne.length;
                }, p2 = function(Ne, je) {
                  if (Ne === void 0 && (Ne = []), je === void 0 && (je = C4), this.data = Ne, this.length = this.data.length, this.compare = je, this.length > 0) for (var ut = (this.length >> 1) - 1; ut >= 0; ut--) this._down(ut);
                };
                function C4(Ne, je) {
                  return Ne < je ? -1 : Ne > je ? 1 : 0;
                }
                function ef(Ne, je, ut) {
                  ut === void 0 && (ut = !1);
                  for (var kt = 1 / 0, Jt = 1 / 0, fn = -1 / 0, Sn = -1 / 0, Hn = Ne[0], nr = 0; nr < Hn.length; nr++) {
                    var kr = Hn[nr];
                    (!nr || kr.x < kt) && (kt = kr.x), (!nr || kr.y < Jt) && (Jt = kr.y), (!nr || kr.x > fn) && (fn = kr.x), (!nr || kr.y > Sn) && (Sn = kr.y);
                  }
                  var Ir = fn - kt, qr = Sn - Jt, ci = Math.min(Ir, qr), ni = ci / 2, Ii = new p2([], r3);
                  if (ci === 0) return new le(kt, Jt);
                  for (var Gi = kt; Gi < fn; Gi += ci) for (var xa = Jt; xa < Sn; xa += ci) Ii.push(new Zh(Gi + ni, xa + ni, ni, Ne));
                  for (var ga = function(eo) {
                    for (var co = 0, wo = 0, ds = 0, Lo = eo[0], As = 0, Fs = Lo.length, Pl = Fs - 1; As < Fs; Pl = As++) {
                      var rl = Lo[As], Rl = Lo[Pl], pl = rl.x * Rl.y - Rl.x * rl.y;
                      wo += (rl.x + Rl.x) * pl, ds += (rl.y + Rl.y) * pl, co += 3 * pl;
                    }
                    return new Zh(wo / co, ds / co, 0, eo);
                  }(Ne), $a = Ii.length; Ii.length; ) {
                    var Va = Ii.pop();
                    (Va.d > ga.d || !ga.d) && (ga = Va, ut && console.log("found best %d after %d probes", Math.round(1e4 * Va.d) / 1e4, $a)), Va.max - ga.d <= je || (ni = Va.h / 2, Ii.push(new Zh(Va.p.x - ni, Va.p.y - ni, ni, Ne)), Ii.push(new Zh(Va.p.x + ni, Va.p.y - ni, ni, Ne)), Ii.push(new Zh(Va.p.x - ni, Va.p.y + ni, ni, Ne)), Ii.push(new Zh(Va.p.x + ni, Va.p.y + ni, ni, Ne)), $a += 4);
                  }
                  return ut && (console.log("num probes: " + $a), console.log("best distance: " + ga.d)), ga.p;
                }
                function r3(Ne, je) {
                  return je.max - Ne.max;
                }
                function Zh(Ne, je, ut, kt) {
                  this.p = new le(Ne, je), this.h = ut, this.d = function(Jt, fn) {
                    for (var Sn = !1, Hn = 1 / 0, nr = 0; nr < fn.length; nr++) for (var kr = fn[nr], Ir = 0, qr = kr.length, ci = qr - 1; Ir < qr; ci = Ir++) {
                      var ni = kr[Ir], Ii = kr[ci];
                      ni.y > Jt.y != Ii.y > Jt.y && Jt.x < (Ii.x - ni.x) * (Jt.y - ni.y) / (Ii.y - ni.y) + ni.x && (Sn = !Sn), Hn = Math.min(Hn, nf(Jt, ni, Ii));
                    }
                    return (Sn ? 1 : -1) * Math.sqrt(Hn);
                  }(this.p, kt), this.max = this.d + this.h * Math.SQRT2;
                }
                p2.prototype.push = function(Ne) {
                  this.data.push(Ne), this.length++, this._up(this.length - 1);
                }, p2.prototype.pop = function() {
                  if (this.length !== 0) {
                    var Ne = this.data[0], je = this.data.pop();
                    return this.length--, this.length > 0 && (this.data[0] = je, this._down(0)), Ne;
                  }
                }, p2.prototype.peek = function() {
                  return this.data[0];
                }, p2.prototype._up = function(Ne) {
                  for (var je = this.data, ut = this.compare, kt = je[Ne]; Ne > 0; ) {
                    var Jt = Ne - 1 >> 1, fn = je[Jt];
                    if (ut(kt, fn) >= 0) break;
                    je[Ne] = fn, Ne = Jt;
                  }
                  je[Ne] = kt;
                }, p2.prototype._down = function(Ne) {
                  for (var je = this.data, ut = this.compare, kt = this.length >> 1, Jt = je[Ne]; Ne < kt; ) {
                    var fn = 1 + (Ne << 1), Sn = je[fn], Hn = fn + 1;
                    if (Hn < this.length && ut(je[Hn], Sn) < 0 && (fn = Hn, Sn = je[Hn]), ut(Sn, Jt) >= 0) break;
                    je[Ne] = Sn, Ne = fn;
                  }
                  je[Ne] = Jt;
                };
                var bf = 7, Kh = Number.POSITIVE_INFINITY;
                function m2(Ne, je) {
                  return je[1] !== Kh ? function(ut, kt, Jt) {
                    var fn = 0, Sn = 0;
                    switch (kt = Math.abs(kt), Jt = Math.abs(Jt), ut) {
                      case "top-right":
                      case "top-left":
                      case "top":
                        Sn = Jt - bf;
                        break;
                      case "bottom-right":
                      case "bottom-left":
                      case "bottom":
                        Sn = -Jt + bf;
                    }
                    switch (ut) {
                      case "top-right":
                      case "bottom-right":
                      case "right":
                        fn = -kt;
                        break;
                      case "top-left":
                      case "bottom-left":
                      case "left":
                        fn = kt;
                    }
                    return [fn, Sn];
                  }(Ne, je[0], je[1]) : function(ut, kt) {
                    var Jt = 0, fn = 0;
                    kt < 0 && (kt = 0);
                    var Sn = kt / Math.sqrt(2);
                    switch (ut) {
                      case "top-right":
                      case "top-left":
                        fn = Sn - bf;
                        break;
                      case "bottom-right":
                      case "bottom-left":
                        fn = -Sn + bf;
                        break;
                      case "bottom":
                        fn = -kt + bf;
                        break;
                      case "top":
                        fn = kt - bf;
                    }
                    switch (ut) {
                      case "top-right":
                      case "bottom-right":
                        Jt = -Sn;
                        break;
                      case "top-left":
                      case "bottom-left":
                        Jt = Sn;
                        break;
                      case "left":
                        Jt = kt;
                        break;
                      case "right":
                        Jt = -kt;
                    }
                    return [Jt, fn];
                  }(Ne, je[0]);
                }
                function Ph(Ne) {
                  switch (Ne) {
                    case "right":
                    case "top-right":
                    case "bottom-right":
                      return "right";
                    case "left":
                    case "top-left":
                    case "bottom-left":
                      return "left";
                  }
                  return "center";
                }
                var $h = 255, jf = $h * js;
                function L4(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii) {
                  var Gi = function(eo, co, wo, ds, Lo, As, Fs, Pl) {
                    for (var rl = ds.layout.get("text-rotate").evaluate(As, {}) * Math.PI / 180, Rl = [], pl = 0, Zs = co.positionedLines; pl < Zs.length; pl += 1) for (var Hc = Zs[pl], tc = 0, Pu = Hc.positionedGlyphs; tc < Pu.length; tc += 1) {
                      var Cl = Pu[tc];
                      if (Cl.rect) {
                        var vu = Cl.rect || {}, qu = Di + 1, Nu = !0, ju = 1, $u = 0, $1 = (Lo || Pl) && Cl.vertical, Zu = Cl.metrics.advance * Cl.scale / 2;
                        if (Pl && co.verticalizable) {
                          var F1 = (Cl.scale - 1) * Ws, Td = (Ws - Cl.metrics.width * Cl.scale) / 2;
                          $u = Hc.lineOffset / 2 - (Cl.imageName ? -Td : F1);
                        }
                        if (Cl.imageName) {
                          var hd = Fs[Cl.imageName];
                          Nu = hd.sdf, ju = hd.pixelRatio, qu = ca / ju;
                        }
                        var Dd = Lo ? [Cl.x + Zu, Cl.y] : [0, 0], Qd = Lo ? [0, 0] : [Cl.x + Zu + wo[0], Cl.y + wo[1] - $u], Ye = [0, 0];
                        $1 && (Ye = Qd, Qd = [0, 0]);
                        var He = (Cl.metrics.left - qu) * Cl.scale - Zu + Qd[0], ot = (-Cl.metrics.top - qu) * Cl.scale + Qd[1], xt = He + vu.w * Cl.scale / ju, zt = ot + vu.h * Cl.scale / ju, ln = new le(He, ot), pn = new le(xt, ot), Cn = new le(He, zt), Nn = new le(xt, zt);
                        if ($1) {
                          var Wn = new le(-Zu, Zu - Ea), lr = -Math.PI / 2, xr = Ws / 2 - Zu, Nr = Cl.imageName ? xr : 0, Kr = new le(5 - Ea - xr, -Nr), xi = new (Function.prototype.bind.apply(le, [null].concat(Ye)))();
                          ln._rotateAround(lr, Wn)._add(Kr)._add(xi), pn._rotateAround(lr, Wn)._add(Kr)._add(xi), Cn._rotateAround(lr, Wn)._add(Kr)._add(xi), Nn._rotateAround(lr, Wn)._add(Kr)._add(xi);
                        }
                        if (rl) {
                          var ui = Math.sin(rl), Ui = Math.cos(rl), ia = [Ui, -ui, ui, Ui];
                          ln._matMult(ia), pn._matMult(ia), Cn._matMult(ia), Nn._matMult(ia);
                        }
                        var Na = new le(0, 0), Ta = new le(0, 0);
                        Rl.push({ tl: ln, tr: pn, bl: Cn, br: Nn, tex: vu, writingMode: co.writingMode, glyphOffset: Dd, sectionIndex: Cl.sectionIndex, isSDF: Nu, pixelOffsetTL: Na, pixelOffsetBR: Ta, minFontScaleX: 0, minFontScaleY: 0 });
                      }
                    }
                    return Rl;
                  }(0, ut, Hn, Jt, fn, Sn, kt, Ne.allowVerticalPlacement), xa = Ne.textSizeData, ga = null;
                  xa.kind === "source" ? (ga = [js * Jt.layout.get("text-size").evaluate(Sn, {})])[0] > jf && Me(Ne.layerIds[0] + ': Value for "text-size" is >= ' + $h + '. Reduce your "text-size".') : xa.kind === "composite" && ((ga = [js * ni.compositeTextSizes[0].evaluate(Sn, {}, Ii), js * ni.compositeTextSizes[1].evaluate(Sn, {}, Ii)])[0] > jf || ga[1] > jf) && Me(Ne.layerIds[0] + ': Value for "text-size" is >= ' + $h + '. Reduce your "text-size".'), Ne.addSymbols(Ne.text, Gi, ga, Hn, fn, Sn, kr, je, nr.lineStartIndex, nr.lineLength, ci, Ii);
                  for (var $a = 0, Va = Ir; $a < Va.length; $a += 1) qr[Va[$a]] = Ne.text.placedSymbolArray.length - 1;
                  return 4 * Gi.length;
                }
                function k4(Ne) {
                  for (var je in Ne) return Ne[je];
                  return null;
                }
                function M4(Ne, je, ut, kt) {
                  var Jt = Ne.compareText;
                  if (je in Jt) {
                    for (var fn = Jt[je], Sn = fn.length - 1; Sn >= 0; Sn--) if (kt.dist(fn[Sn]) < ut) return !0;
                  } else Jt[je] = [];
                  return Jt[je].push(kt), !1;
                }
                var R4 = En.VectorTileFeature.types, J6 = [{ name: "a_fade_opacity", components: 1, type: "Uint8", offset: 0 }];
                function H2(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci) {
                  var ni = Hn ? Math.min(jf, Math.round(Hn[0])) : 0, Ii = Hn ? Math.min(jf, Math.round(Hn[1])) : 0;
                  Ne.emplaceBack(je, ut, Math.round(32 * kt), Math.round(32 * Jt), fn, Sn, (ni << 1) + (nr ? 1 : 0), Ii, 16 * kr, 16 * Ir, 256 * qr, 256 * ci);
                }
                function G0(Ne, je, ut) {
                  Ne.emplaceBack(je.x, je.y, ut), Ne.emplaceBack(je.x, je.y, ut), Ne.emplaceBack(je.x, je.y, ut), Ne.emplaceBack(je.x, je.y, ut);
                }
                function i3(Ne) {
                  for (var je = 0, ut = Ne.sections; je < ut.length; je += 1) if (bc(ut[je].text)) return !0;
                  return !1;
                }
                var Fh = function(Ne) {
                  this.layoutVertexArray = new Fe(), this.indexArray = new ht(), this.programConfigurations = Ne, this.segments = new ra(), this.dynamicLayoutVertexArray = new Te(), this.opacityVertexArray = new Ae(), this.placedSymbolArray = new Un();
                };
                Fh.prototype.isEmpty = function() {
                  return this.layoutVertexArray.length === 0 && this.indexArray.length === 0 && this.dynamicLayoutVertexArray.length === 0 && this.opacityVertexArray.length === 0;
                }, Fh.prototype.upload = function(Ne, je, ut, kt) {
                  this.isEmpty() || (ut && (this.layoutVertexBuffer = Ne.createVertexBuffer(this.layoutVertexArray, Jo.members), this.indexBuffer = Ne.createIndexBuffer(this.indexArray, je), this.dynamicLayoutVertexBuffer = Ne.createVertexBuffer(this.dynamicLayoutVertexArray, ps.members, !0), this.opacityVertexBuffer = Ne.createVertexBuffer(this.opacityVertexArray, J6, !0), this.opacityVertexBuffer.itemSize = 1), (ut || kt) && this.programConfigurations.upload(Ne));
                }, Fh.prototype.destroy = function() {
                  this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy(), this.dynamicLayoutVertexBuffer.destroy(), this.opacityVertexBuffer.destroy());
                }, cs("SymbolBuffers", Fh);
                var z2 = function(Ne, je, ut) {
                  this.layoutVertexArray = new Ne(), this.layoutAttributes = je, this.indexArray = new ut(), this.segments = new ra(), this.collisionVertexArray = new nt();
                };
                z2.prototype.upload = function(Ne) {
                  this.layoutVertexBuffer = Ne.createVertexBuffer(this.layoutVertexArray, this.layoutAttributes), this.indexBuffer = Ne.createIndexBuffer(this.indexArray), this.collisionVertexBuffer = Ne.createVertexBuffer(this.collisionVertexArray, wl.members, !0);
                }, z2.prototype.destroy = function() {
                  this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.segments.destroy(), this.collisionVertexBuffer.destroy());
                }, cs("CollisionBuffers", z2);
                var Du = function(Ne) {
                  this.collisionBoxArray = Ne.collisionBoxArray, this.zoom = Ne.zoom, this.overscaling = Ne.overscaling, this.layers = Ne.layers, this.layerIds = this.layers.map(function(Sn) {
                    return Sn.id;
                  }), this.index = Ne.index, this.pixelRatio = Ne.pixelRatio, this.sourceLayerIndex = Ne.sourceLayerIndex, this.hasPattern = !1, this.hasRTLText = !1, this.sortKeyRanges = [], this.collisionCircleArray = [], this.placementInvProjMatrix = Ff([]), this.placementViewportMatrix = Ff([]);
                  var je = this.layers[0]._unevaluatedLayout._values;
                  this.textSizeData = Mc(this.zoom, je["text-size"]), this.iconSizeData = Mc(this.zoom, je["icon-size"]);
                  var ut = this.layers[0].layout, kt = ut.get("symbol-sort-key"), Jt = ut.get("symbol-z-order");
                  this.canOverlap = ut.get("text-allow-overlap") || ut.get("icon-allow-overlap") || ut.get("text-ignore-placement") || ut.get("icon-ignore-placement"), this.sortFeaturesByKey = Jt !== "viewport-y" && kt.constantOr(1) !== void 0;
                  var fn = Jt === "viewport-y" || Jt === "auto" && !this.sortFeaturesByKey;
                  this.sortFeaturesByY = fn && this.canOverlap, ut.get("symbol-placement") === "point" && (this.writingModes = ut.get("text-writing-mode").map(function(Sn) {
                    return fa[Sn];
                  })), this.stateDependentLayerIds = this.layers.filter(function(Sn) {
                    return Sn.isStateDependent();
                  }).map(function(Sn) {
                    return Sn.id;
                  }), this.sourceID = Ne.sourceID;
                };
                Du.prototype.createArrays = function() {
                  this.text = new Fh(new Oc(this.layers, this.zoom, function(Ne) {
                    return /^text/.test(Ne);
                  })), this.icon = new Fh(new Oc(this.layers, this.zoom, function(Ne) {
                    return /^icon/.test(Ne);
                  })), this.glyphOffsetArray = new Tr(), this.lineVertexArray = new ii(), this.symbolInstances = new Cr();
                }, Du.prototype.calculateGlyphDependencies = function(Ne, je, ut, kt, Jt) {
                  for (var fn = 0; fn < Ne.length; fn++) if (je[Ne.charCodeAt(fn)] = !0, (ut || kt) && Jt) {
                    var Sn = ll[Ne.charAt(fn)];
                    Sn && (je[Sn.charCodeAt(0)] = !0);
                  }
                }, Du.prototype.populate = function(Ne, je, ut) {
                  var kt = this.layers[0], Jt = kt.layout, fn = Jt.get("text-font"), Sn = Jt.get("text-field"), Hn = Jt.get("icon-image"), nr = (Sn.value.kind !== "constant" || Sn.value.value instanceof oa && !Sn.value.value.isEmpty() || Sn.value.value.toString().length > 0) && (fn.value.kind !== "constant" || fn.value.value.length > 0), kr = Hn.value.kind !== "constant" || !!Hn.value.value || Object.keys(Hn.parameters).length > 0, Ir = Jt.get("symbol-sort-key");
                  if (this.features = [], nr || kr) {
                    for (var qr = je.iconDependencies, ci = je.glyphDependencies, ni = je.availableImages, Ii = new El(this.zoom), Gi = 0, xa = Ne; Gi < xa.length; Gi += 1) {
                      var ga = xa[Gi], $a = ga.feature, Va = ga.id, eo = ga.index, co = ga.sourceLayerIndex, wo = kt._featureFilter.needGeometry, ds = Zc($a, wo);
                      if (kt._featureFilter.filter(Ii, ds, ut)) {
                        wo || (ds.geometry = zu($a));
                        var Lo = void 0;
                        if (nr) {
                          var As = kt.getValueAndResolveTokens("text-field", ds, ut, ni), Fs = oa.factory(As);
                          i3(Fs) && (this.hasRTLText = !0), (!this.hasRTLText || Zo() === "unavailable" || this.hasRTLText && Uc.isParsed()) && (Lo = ns(Fs, kt, ds));
                        }
                        var Pl = void 0;
                        if (kr) {
                          var rl = kt.getValueAndResolveTokens("icon-image", ds, ut, ni);
                          Pl = rl instanceof ka ? rl : ka.fromString(rl);
                        }
                        if (Lo || Pl) {
                          var Rl = this.sortFeaturesByKey ? Ir.evaluate(ds, {}, ut) : void 0, pl = { id: Va, text: Lo, icon: Pl, index: eo, sourceLayerIndex: co, geometry: ds.geometry, properties: $a.properties, type: R4[$a.type], sortKey: Rl };
                          if (this.features.push(pl), Pl && (qr[Pl.name] = !0), Lo) {
                            var Zs = fn.evaluate(ds, {}, ut).join(","), Hc = Jt.get("text-rotation-alignment") === "map" && Jt.get("symbol-placement") !== "point";
                            this.allowVerticalPlacement = this.writingModes && this.writingModes.indexOf(fa.vertical) >= 0;
                            for (var tc = 0, Pu = Lo.sections; tc < Pu.length; tc += 1) {
                              var Cl = Pu[tc];
                              if (Cl.image) qr[Cl.image.name] = !0;
                              else {
                                var vu = Ru(Lo.toString()), qu = Cl.fontStack || Zs, Nu = ci[qu] = ci[qu] || {};
                                this.calculateGlyphDependencies(Cl.text, Nu, Hc, this.allowVerticalPlacement, vu);
                              }
                            }
                          }
                        }
                      }
                    }
                    Jt.get("symbol-placement") === "line" && (this.features = function(ju) {
                      var $u = {}, $1 = {}, Zu = [], F1 = 0;
                      function Td(Nn) {
                        Zu.push(ju[Nn]), F1++;
                      }
                      function hd(Nn, Wn, lr) {
                        var xr = $1[Nn];
                        return delete $1[Nn], $1[Wn] = xr, Zu[xr].geometry[0].pop(), Zu[xr].geometry[0] = Zu[xr].geometry[0].concat(lr[0]), xr;
                      }
                      function Dd(Nn, Wn, lr) {
                        var xr = $u[Wn];
                        return delete $u[Wn], $u[Nn] = xr, Zu[xr].geometry[0].shift(), Zu[xr].geometry[0] = lr[0].concat(Zu[xr].geometry[0]), xr;
                      }
                      function Qd(Nn, Wn, lr) {
                        var xr = lr ? Wn[0][Wn[0].length - 1] : Wn[0][0];
                        return Nn + ":" + xr.x + ":" + xr.y;
                      }
                      for (var Ye = 0; Ye < ju.length; Ye++) {
                        var He = ju[Ye], ot = He.geometry, xt = He.text ? He.text.toString() : null;
                        if (xt) {
                          var zt = Qd(xt, ot), ln = Qd(xt, ot, !0);
                          if (zt in $1 && ln in $u && $1[zt] !== $u[ln]) {
                            var pn = Dd(zt, ln, ot), Cn = hd(zt, ln, Zu[pn].geometry);
                            delete $u[zt], delete $1[ln], $1[Qd(xt, Zu[Cn].geometry, !0)] = Cn, Zu[pn].geometry = null;
                          } else zt in $1 ? hd(zt, ln, ot) : ln in $u ? Dd(zt, ln, ot) : (Td(Ye), $u[zt] = F1 - 1, $1[ln] = F1 - 1);
                        } else Td(Ye);
                      }
                      return Zu.filter(function(Nn) {
                        return Nn.geometry;
                      });
                    }(this.features)), this.sortFeaturesByKey && this.features.sort(function(ju, $u) {
                      return ju.sortKey - $u.sortKey;
                    });
                  }
                }, Du.prototype.update = function(Ne, je, ut) {
                  this.stateDependentLayers.length && (this.text.programConfigurations.updatePaintArrays(Ne, je, this.layers, ut), this.icon.programConfigurations.updatePaintArrays(Ne, je, this.layers, ut));
                }, Du.prototype.isEmpty = function() {
                  return this.symbolInstances.length === 0 && !this.hasRTLText;
                }, Du.prototype.uploadPending = function() {
                  return !this.uploaded || this.text.programConfigurations.needsUpload || this.icon.programConfigurations.needsUpload;
                }, Du.prototype.upload = function(Ne) {
                  !this.uploaded && this.hasDebugData() && (this.textCollisionBox.upload(Ne), this.iconCollisionBox.upload(Ne)), this.text.upload(Ne, this.sortFeaturesByY, !this.uploaded, this.text.programConfigurations.needsUpload), this.icon.upload(Ne, this.sortFeaturesByY, !this.uploaded, this.icon.programConfigurations.needsUpload), this.uploaded = !0;
                }, Du.prototype.destroyDebugData = function() {
                  this.textCollisionBox.destroy(), this.iconCollisionBox.destroy();
                }, Du.prototype.destroy = function() {
                  this.text.destroy(), this.icon.destroy(), this.hasDebugData() && this.destroyDebugData();
                }, Du.prototype.addToLineVertexArray = function(Ne, je) {
                  var ut = this.lineVertexArray.length;
                  if (Ne.segment !== void 0) {
                    for (var kt = Ne.dist(je[Ne.segment + 1]), Jt = Ne.dist(je[Ne.segment]), fn = {}, Sn = Ne.segment + 1; Sn < je.length; Sn++) fn[Sn] = { x: je[Sn].x, y: je[Sn].y, tileUnitDistanceFromAnchor: kt }, Sn < je.length - 1 && (kt += je[Sn + 1].dist(je[Sn]));
                    for (var Hn = Ne.segment || 0; Hn >= 0; Hn--) fn[Hn] = { x: je[Hn].x, y: je[Hn].y, tileUnitDistanceFromAnchor: Jt }, Hn > 0 && (Jt += je[Hn - 1].dist(je[Hn]));
                    for (var nr = 0; nr < je.length; nr++) {
                      var kr = fn[nr];
                      this.lineVertexArray.emplaceBack(kr.x, kr.y, kr.tileUnitDistanceFromAnchor);
                    }
                  }
                  return { lineStartIndex: ut, lineLength: this.lineVertexArray.length - ut };
                }, Du.prototype.addSymbols = function(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr) {
                  for (var ci = Ne.indexArray, ni = Ne.layoutVertexArray, Ii = Ne.segments.prepareSegment(4 * je.length, ni, ci, this.canOverlap ? fn.sortKey : void 0), Gi = this.glyphOffsetArray.length, xa = Ii.vertexLength, ga = this.allowVerticalPlacement && Sn === fa.vertical ? Math.PI / 2 : 0, $a = fn.text && fn.text.sections, Va = 0; Va < je.length; Va++) {
                    var eo = je[Va], co = eo.tl, wo = eo.tr, ds = eo.bl, Lo = eo.br, As = eo.tex, Fs = eo.pixelOffsetTL, Pl = eo.pixelOffsetBR, rl = eo.minFontScaleX, Rl = eo.minFontScaleY, pl = eo.glyphOffset, Zs = eo.isSDF, Hc = eo.sectionIndex, tc = Ii.vertexLength, Pu = pl[1];
                    H2(ni, Hn.x, Hn.y, co.x, Pu + co.y, As.x, As.y, ut, Zs, Fs.x, Fs.y, rl, Rl), H2(ni, Hn.x, Hn.y, wo.x, Pu + wo.y, As.x + As.w, As.y, ut, Zs, Pl.x, Fs.y, rl, Rl), H2(ni, Hn.x, Hn.y, ds.x, Pu + ds.y, As.x, As.y + As.h, ut, Zs, Fs.x, Pl.y, rl, Rl), H2(ni, Hn.x, Hn.y, Lo.x, Pu + Lo.y, As.x + As.w, As.y + As.h, ut, Zs, Pl.x, Pl.y, rl, Rl), G0(Ne.dynamicLayoutVertexArray, Hn, ga), ci.emplaceBack(tc, tc + 1, tc + 2), ci.emplaceBack(tc + 1, tc + 2, tc + 3), Ii.vertexLength += 4, Ii.primitiveLength += 2, this.glyphOffsetArray.emplaceBack(pl[0]), Va !== je.length - 1 && Hc === je[Va + 1].sectionIndex || Ne.programConfigurations.populatePaintArrays(ni.length, fn, fn.index, {}, qr, $a && $a[Hc]);
                  }
                  Ne.placedSymbolArray.emplaceBack(Hn.x, Hn.y, Gi, this.glyphOffsetArray.length - Gi, xa, nr, kr, Hn.segment, ut ? ut[0] : 0, ut ? ut[1] : 0, kt[0], kt[1], Sn, 0, !1, 0, Ir);
                }, Du.prototype._addCollisionDebugVertex = function(Ne, je, ut, kt, Jt, fn) {
                  return je.emplaceBack(0, 0), Ne.emplaceBack(ut.x, ut.y, kt, Jt, Math.round(fn.x), Math.round(fn.y));
                }, Du.prototype.addCollisionDebugVertices = function(Ne, je, ut, kt, Jt, fn, Sn) {
                  var Hn = Jt.segments.prepareSegment(4, Jt.layoutVertexArray, Jt.indexArray), nr = Hn.vertexLength, kr = Jt.layoutVertexArray, Ir = Jt.collisionVertexArray, qr = Sn.anchorX, ci = Sn.anchorY;
                  this._addCollisionDebugVertex(kr, Ir, fn, qr, ci, new le(Ne, je)), this._addCollisionDebugVertex(kr, Ir, fn, qr, ci, new le(ut, je)), this._addCollisionDebugVertex(kr, Ir, fn, qr, ci, new le(ut, kt)), this._addCollisionDebugVertex(kr, Ir, fn, qr, ci, new le(Ne, kt)), Hn.vertexLength += 4;
                  var ni = Jt.indexArray;
                  ni.emplaceBack(nr, nr + 1), ni.emplaceBack(nr + 1, nr + 2), ni.emplaceBack(nr + 2, nr + 3), ni.emplaceBack(nr + 3, nr), Hn.primitiveLength += 4;
                }, Du.prototype.addDebugCollisionBoxes = function(Ne, je, ut, kt) {
                  for (var Jt = Ne; Jt < je; Jt++) {
                    var fn = this.collisionBoxArray.get(Jt), Sn = fn.x1, Hn = fn.y1, nr = fn.x2, kr = fn.y2;
                    this.addCollisionDebugVertices(Sn, Hn, nr, kr, kt ? this.textCollisionBox : this.iconCollisionBox, fn.anchorPoint, ut);
                  }
                }, Du.prototype.generateCollisionDebugBuffers = function() {
                  this.hasDebugData() && this.destroyDebugData(), this.textCollisionBox = new z2(Be, ol.members, An), this.iconCollisionBox = new z2(Be, ol.members, An);
                  for (var Ne = 0; Ne < this.symbolInstances.length; Ne++) {
                    var je = this.symbolInstances.get(Ne);
                    this.addDebugCollisionBoxes(je.textBoxStartIndex, je.textBoxEndIndex, je, !0), this.addDebugCollisionBoxes(je.verticalTextBoxStartIndex, je.verticalTextBoxEndIndex, je, !0), this.addDebugCollisionBoxes(je.iconBoxStartIndex, je.iconBoxEndIndex, je, !1), this.addDebugCollisionBoxes(je.verticalIconBoxStartIndex, je.verticalIconBoxEndIndex, je, !1);
                  }
                }, Du.prototype._deserializeCollisionBoxesForSymbol = function(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr) {
                  for (var kr = {}, Ir = je; Ir < ut; Ir++) {
                    var qr = Ne.get(Ir);
                    kr.textBox = { x1: qr.x1, y1: qr.y1, x2: qr.x2, y2: qr.y2, anchorPointX: qr.anchorPointX, anchorPointY: qr.anchorPointY }, kr.textFeatureIndex = qr.featureIndex;
                    break;
                  }
                  for (var ci = kt; ci < Jt; ci++) {
                    var ni = Ne.get(ci);
                    kr.verticalTextBox = { x1: ni.x1, y1: ni.y1, x2: ni.x2, y2: ni.y2, anchorPointX: ni.anchorPointX, anchorPointY: ni.anchorPointY }, kr.verticalTextFeatureIndex = ni.featureIndex;
                    break;
                  }
                  for (var Ii = fn; Ii < Sn; Ii++) {
                    var Gi = Ne.get(Ii);
                    kr.iconBox = { x1: Gi.x1, y1: Gi.y1, x2: Gi.x2, y2: Gi.y2, anchorPointX: Gi.anchorPointX, anchorPointY: Gi.anchorPointY }, kr.iconFeatureIndex = Gi.featureIndex;
                    break;
                  }
                  for (var xa = Hn; xa < nr; xa++) {
                    var ga = Ne.get(xa);
                    kr.verticalIconBox = { x1: ga.x1, y1: ga.y1, x2: ga.x2, y2: ga.y2, anchorPointX: ga.anchorPointX, anchorPointY: ga.anchorPointY }, kr.verticalIconFeatureIndex = ga.featureIndex;
                    break;
                  }
                  return kr;
                }, Du.prototype.deserializeCollisionBoxes = function(Ne) {
                  this.collisionArrays = [];
                  for (var je = 0; je < this.symbolInstances.length; je++) {
                    var ut = this.symbolInstances.get(je);
                    this.collisionArrays.push(this._deserializeCollisionBoxesForSymbol(Ne, ut.textBoxStartIndex, ut.textBoxEndIndex, ut.verticalTextBoxStartIndex, ut.verticalTextBoxEndIndex, ut.iconBoxStartIndex, ut.iconBoxEndIndex, ut.verticalIconBoxStartIndex, ut.verticalIconBoxEndIndex));
                  }
                }, Du.prototype.hasTextData = function() {
                  return this.text.segments.get().length > 0;
                }, Du.prototype.hasIconData = function() {
                  return this.icon.segments.get().length > 0;
                }, Du.prototype.hasDebugData = function() {
                  return this.textCollisionBox && this.iconCollisionBox;
                }, Du.prototype.hasTextCollisionBoxData = function() {
                  return this.hasDebugData() && this.textCollisionBox.segments.get().length > 0;
                }, Du.prototype.hasIconCollisionBoxData = function() {
                  return this.hasDebugData() && this.iconCollisionBox.segments.get().length > 0;
                }, Du.prototype.addIndicesForPlacedSymbol = function(Ne, je) {
                  for (var ut = Ne.placedSymbolArray.get(je), kt = ut.vertexStartIndex + 4 * ut.numGlyphs, Jt = ut.vertexStartIndex; Jt < kt; Jt += 4) Ne.indexArray.emplaceBack(Jt, Jt + 1, Jt + 2), Ne.indexArray.emplaceBack(Jt + 1, Jt + 2, Jt + 3);
                }, Du.prototype.getSortedSymbolIndexes = function(Ne) {
                  if (this.sortedAngle === Ne && this.symbolInstanceIndexes !== void 0) return this.symbolInstanceIndexes;
                  for (var je = Math.sin(Ne), ut = Math.cos(Ne), kt = [], Jt = [], fn = [], Sn = 0; Sn < this.symbolInstances.length; ++Sn) {
                    fn.push(Sn);
                    var Hn = this.symbolInstances.get(Sn);
                    kt.push(0 | Math.round(je * Hn.anchorX + ut * Hn.anchorY)), Jt.push(Hn.featureIndex);
                  }
                  return fn.sort(function(nr, kr) {
                    return kt[nr] - kt[kr] || Jt[kr] - Jt[nr];
                  }), fn;
                }, Du.prototype.addToSortKeyRanges = function(Ne, je) {
                  var ut = this.sortKeyRanges[this.sortKeyRanges.length - 1];
                  ut && ut.sortKey === je ? ut.symbolInstanceEnd = Ne + 1 : this.sortKeyRanges.push({ sortKey: je, symbolInstanceStart: Ne, symbolInstanceEnd: Ne + 1 });
                }, Du.prototype.sortFeatures = function(Ne) {
                  var je = this;
                  if (this.sortFeaturesByY && this.sortedAngle !== Ne && !(this.text.segments.get().length > 1 || this.icon.segments.get().length > 1)) {
                    this.symbolInstanceIndexes = this.getSortedSymbolIndexes(Ne), this.sortedAngle = Ne, this.text.indexArray.clear(), this.icon.indexArray.clear(), this.featureSortOrder = [];
                    for (var ut = 0, kt = this.symbolInstanceIndexes; ut < kt.length; ut += 1) {
                      var Jt = kt[ut], fn = this.symbolInstances.get(Jt);
                      this.featureSortOrder.push(fn.featureIndex), [fn.rightJustifiedTextSymbolIndex, fn.centerJustifiedTextSymbolIndex, fn.leftJustifiedTextSymbolIndex].forEach(function(Sn, Hn, nr) {
                        Sn >= 0 && nr.indexOf(Sn) === Hn && je.addIndicesForPlacedSymbol(je.text, Sn);
                      }), fn.verticalPlacedTextSymbolIndex >= 0 && this.addIndicesForPlacedSymbol(this.text, fn.verticalPlacedTextSymbolIndex), fn.placedIconSymbolIndex >= 0 && this.addIndicesForPlacedSymbol(this.icon, fn.placedIconSymbolIndex), fn.verticalPlacedIconSymbolIndex >= 0 && this.addIndicesForPlacedSymbol(this.icon, fn.verticalPlacedIconSymbolIndex);
                    }
                    this.text.indexBuffer && this.text.indexBuffer.updateData(this.text.indexArray), this.icon.indexBuffer && this.icon.indexBuffer.updateData(this.icon.indexArray);
                  }
                }, cs("SymbolBucket", Du, { omit: ["layers", "collisionBoxArray", "features", "compareText"] }), Du.MAX_GLYPHS = 65535, Du.addDynamicAttributes = G0;
                var O4 = new $c({ "symbol-placement": new ks(Zt.layout_symbol["symbol-placement"]), "symbol-spacing": new ks(Zt.layout_symbol["symbol-spacing"]), "symbol-avoid-edges": new ks(Zt.layout_symbol["symbol-avoid-edges"]), "symbol-sort-key": new qs(Zt.layout_symbol["symbol-sort-key"]), "symbol-z-order": new ks(Zt.layout_symbol["symbol-z-order"]), "icon-allow-overlap": new ks(Zt.layout_symbol["icon-allow-overlap"]), "icon-ignore-placement": new ks(Zt.layout_symbol["icon-ignore-placement"]), "icon-optional": new ks(Zt.layout_symbol["icon-optional"]), "icon-rotation-alignment": new ks(Zt.layout_symbol["icon-rotation-alignment"]), "icon-size": new qs(Zt.layout_symbol["icon-size"]), "icon-text-fit": new ks(Zt.layout_symbol["icon-text-fit"]), "icon-text-fit-padding": new ks(Zt.layout_symbol["icon-text-fit-padding"]), "icon-image": new qs(Zt.layout_symbol["icon-image"]), "icon-rotate": new qs(Zt.layout_symbol["icon-rotate"]), "icon-padding": new ks(Zt.layout_symbol["icon-padding"]), "icon-keep-upright": new ks(Zt.layout_symbol["icon-keep-upright"]), "icon-offset": new qs(Zt.layout_symbol["icon-offset"]), "icon-anchor": new qs(Zt.layout_symbol["icon-anchor"]), "icon-pitch-alignment": new ks(Zt.layout_symbol["icon-pitch-alignment"]), "text-pitch-alignment": new ks(Zt.layout_symbol["text-pitch-alignment"]), "text-rotation-alignment": new ks(Zt.layout_symbol["text-rotation-alignment"]), "text-field": new qs(Zt.layout_symbol["text-field"]), "text-font": new qs(Zt.layout_symbol["text-font"]), "text-size": new qs(Zt.layout_symbol["text-size"]), "text-max-width": new qs(Zt.layout_symbol["text-max-width"]), "text-line-height": new ks(Zt.layout_symbol["text-line-height"]), "text-letter-spacing": new qs(Zt.layout_symbol["text-letter-spacing"]), "text-justify": new qs(Zt.layout_symbol["text-justify"]), "text-radial-offset": new qs(Zt.layout_symbol["text-radial-offset"]), "text-variable-anchor": new ks(Zt.layout_symbol["text-variable-anchor"]), "text-anchor": new qs(Zt.layout_symbol["text-anchor"]), "text-max-angle": new ks(Zt.layout_symbol["text-max-angle"]), "text-writing-mode": new ks(Zt.layout_symbol["text-writing-mode"]), "text-rotate": new qs(Zt.layout_symbol["text-rotate"]), "text-padding": new ks(Zt.layout_symbol["text-padding"]), "text-keep-upright": new ks(Zt.layout_symbol["text-keep-upright"]), "text-transform": new qs(Zt.layout_symbol["text-transform"]), "text-offset": new qs(Zt.layout_symbol["text-offset"]), "text-allow-overlap": new ks(Zt.layout_symbol["text-allow-overlap"]), "text-ignore-placement": new ks(Zt.layout_symbol["text-ignore-placement"]), "text-optional": new ks(Zt.layout_symbol["text-optional"]) }), K0 = { paint: new $c({ "icon-opacity": new qs(Zt.paint_symbol["icon-opacity"]), "icon-color": new qs(Zt.paint_symbol["icon-color"]), "icon-halo-color": new qs(Zt.paint_symbol["icon-halo-color"]), "icon-halo-width": new qs(Zt.paint_symbol["icon-halo-width"]), "icon-halo-blur": new qs(Zt.paint_symbol["icon-halo-blur"]), "icon-translate": new ks(Zt.paint_symbol["icon-translate"]), "icon-translate-anchor": new ks(Zt.paint_symbol["icon-translate-anchor"]), "text-opacity": new qs(Zt.paint_symbol["text-opacity"]), "text-color": new qs(Zt.paint_symbol["text-color"], { runtimeType: Ar, getOverride: function(Ne) {
                  return Ne.textColor;
                }, hasOverride: function(Ne) {
                  return !!Ne.textColor;
                } }), "text-halo-color": new qs(Zt.paint_symbol["text-halo-color"]), "text-halo-width": new qs(Zt.paint_symbol["text-halo-width"]), "text-halo-blur": new qs(Zt.paint_symbol["text-halo-blur"]), "text-translate": new ks(Zt.paint_symbol["text-translate"]), "text-translate-anchor": new ks(Zt.paint_symbol["text-translate-anchor"]) }), layout: O4 }, ih = function(Ne) {
                  this.type = Ne.property.overrides ? Ne.property.overrides.runtimeType : Kn, this.defaultValue = Ne;
                };
                ih.prototype.evaluate = function(Ne) {
                  if (Ne.formattedSection) {
                    var je = this.defaultValue.property.overrides;
                    if (je && je.hasOverride(Ne.formattedSection)) return je.getOverride(Ne.formattedSection);
                  }
                  return Ne.feature && Ne.featureState ? this.defaultValue.evaluate(Ne.feature, Ne.featureState) : this.defaultValue.property.specification.default;
                }, ih.prototype.eachChild = function(Ne) {
                  this.defaultValue.isConstant() || Ne(this.defaultValue.value._styleExpression.expression);
                }, ih.prototype.outputDefined = function() {
                  return !1;
                }, ih.prototype.serialize = function() {
                  return null;
                }, cs("FormatSectionOverride", ih, { omit: ["defaultValue"] });
                var ah = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, K0);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.recalculate = function(ut, kt) {
                    if (Ne.prototype.recalculate.call(this, ut, kt), this.layout.get("icon-rotation-alignment") === "auto" && (this.layout.get("symbol-placement") !== "point" ? this.layout._values["icon-rotation-alignment"] = "map" : this.layout._values["icon-rotation-alignment"] = "viewport"), this.layout.get("text-rotation-alignment") === "auto" && (this.layout.get("symbol-placement") !== "point" ? this.layout._values["text-rotation-alignment"] = "map" : this.layout._values["text-rotation-alignment"] = "viewport"), this.layout.get("text-pitch-alignment") === "auto" && (this.layout._values["text-pitch-alignment"] = this.layout.get("text-rotation-alignment")), this.layout.get("icon-pitch-alignment") === "auto" && (this.layout._values["icon-pitch-alignment"] = this.layout.get("icon-rotation-alignment")), this.layout.get("symbol-placement") === "point") {
                      var Jt = this.layout.get("text-writing-mode");
                      if (Jt) {
                        for (var fn = [], Sn = 0, Hn = Jt; Sn < Hn.length; Sn += 1) {
                          var nr = Hn[Sn];
                          fn.indexOf(nr) < 0 && fn.push(nr);
                        }
                        this.layout._values["text-writing-mode"] = fn;
                      } else this.layout._values["text-writing-mode"] = ["horizontal"];
                    }
                    this._setPaintOverrides();
                  }, je.prototype.getValueAndResolveTokens = function(ut, kt, Jt, fn) {
                    var Sn = this.layout.get(ut).evaluate(kt, {}, Jt, fn), Hn = this._unevaluatedLayout._values[ut];
                    return Hn.isDataDriven() || ta(Hn.value) || !Sn ? Sn : function(nr, kr) {
                      return kr.replace(/{([^{}]+)}/g, function(Ir, qr) {
                        return qr in nr ? String(nr[qr]) : "";
                      });
                    }(kt.properties, Sn);
                  }, je.prototype.createBucket = function(ut) {
                    return new Du(ut);
                  }, je.prototype.queryRadius = function() {
                    return 0;
                  }, je.prototype.queryIntersectsFeature = function() {
                    return !1;
                  }, je.prototype._setPaintOverrides = function() {
                    for (var ut = 0, kt = K0.paint.overridableProperties; ut < kt.length; ut += 1) {
                      var Jt = kt[ut];
                      if (je.hasPaintOverride(this.layout, Jt)) {
                        var fn, Sn = this.paint.get(Jt), Hn = new ih(Sn), nr = new Ri(Hn, Sn.property.specification);
                        fn = Sn.value.kind === "constant" || Sn.value.kind === "source" ? new Ji("source", nr) : new Pa("composite", nr, Sn.value.zoomStops, Sn.value._interpolationType), this.paint._values[Jt] = new nc(Sn.property, fn, Sn.parameters);
                      }
                    }
                  }, je.prototype._handleOverridablePaintPropertyUpdate = function(ut, kt, Jt) {
                    return !(!this.layout || kt.isDataDriven() || Jt.isDataDriven()) && je.hasPaintOverride(this.layout, ut);
                  }, je.hasPaintOverride = function(ut, kt) {
                    var Jt = ut.get("text-field"), fn = K0.paint.properties[kt], Sn = !1, Hn = function(Ir) {
                      for (var qr = 0, ci = Ir; qr < ci.length; qr += 1) {
                        var ni = ci[qr];
                        if (fn.overrides && fn.overrides.hasOverride(ni)) return void (Sn = !0);
                      }
                    };
                    if (Jt.value.kind === "constant" && Jt.value.value instanceof oa) Hn(Jt.value.value.sections);
                    else if (Jt.value.kind === "source") {
                      var nr = function(Ir) {
                        if (!Sn) if (Ir instanceof Sr && Gr(Ir.value) === Wr) {
                          var qr = Ir.value;
                          Hn(qr.sections);
                        } else Ir instanceof di ? Hn(Ir.sections) : Ir.eachChild(nr);
                      }, kr = Jt.value;
                      kr._styleExpression && nr(kr._styleExpression.expression);
                    }
                    return Sn;
                  }, je;
                }(Ul), Bh = { paint: new $c({ "background-color": new ks(Zt.paint_background["background-color"]), "background-pattern": new $l(Zt.paint_background["background-pattern"]), "background-opacity": new ks(Zt.paint_background["background-opacity"]) }) }, I4 = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, Bh);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je;
                }(Ul), a3 = { paint: new $c({ "raster-opacity": new ks(Zt.paint_raster["raster-opacity"]), "raster-hue-rotate": new ks(Zt.paint_raster["raster-hue-rotate"]), "raster-brightness-min": new ks(Zt.paint_raster["raster-brightness-min"]), "raster-brightness-max": new ks(Zt.paint_raster["raster-brightness-max"]), "raster-saturation": new ks(Zt.paint_raster["raster-saturation"]), "raster-contrast": new ks(Zt.paint_raster["raster-contrast"]), "raster-resampling": new ks(Zt.paint_raster["raster-resampling"]), "raster-fade-duration": new ks(Zt.paint_raster["raster-fade-duration"]) }) }, e_ = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, a3);
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je;
                }(Ul), t_ = function(Ne) {
                  function je(ut) {
                    Ne.call(this, ut, {}), this.implementation = ut;
                  }
                  return je.__proto__ = Ne, je.prototype = Object.create(Ne.prototype), je.prototype.constructor = je, je.prototype.is3D = function() {
                    return this.implementation.renderingMode === "3d";
                  }, je.prototype.hasOffscreenPass = function() {
                    return this.implementation.prerender !== void 0;
                  }, je.prototype.recalculate = function() {
                  }, je.prototype.updateTransitions = function() {
                  }, je.prototype.hasTransition = function() {
                  }, je.prototype.serialize = function() {
                  }, je.prototype.onAdd = function(ut) {
                    this.implementation.onAdd && this.implementation.onAdd(ut, ut.painter.context.gl);
                  }, je.prototype.onRemove = function(ut) {
                    this.implementation.onRemove && this.implementation.onRemove(ut, ut.painter.context.gl);
                  }, je;
                }(Ul), J0 = { circle: af, heatmap: ld, hillshade: sf, fill: $2, "fill-extrusion": Ao, line: us, symbol: ah, background: I4, raster: e_ }, ep = de.HTMLImageElement, D4 = de.HTMLCanvasElement, tp = de.HTMLVideoElement, rp = de.ImageData, gh = de.ImageBitmap, Hh = function(Ne, je, ut, kt) {
                  this.context = Ne, this.format = ut, this.texture = Ne.gl.createTexture(), this.update(je, kt);
                };
                Hh.prototype.update = function(Ne, je, ut) {
                  var kt = Ne.width, Jt = Ne.height, fn = !(this.size && this.size[0] === kt && this.size[1] === Jt || ut), Sn = this.context, Hn = Sn.gl;
                  if (this.useMipmap = !!(je && je.useMipmap), Hn.bindTexture(Hn.TEXTURE_2D, this.texture), Sn.pixelStoreUnpackFlipY.set(!1), Sn.pixelStoreUnpack.set(1), Sn.pixelStoreUnpackPremultiplyAlpha.set(this.format === Hn.RGBA && (!je || je.premultiply !== !1)), fn) this.size = [kt, Jt], Ne instanceof ep || Ne instanceof D4 || Ne instanceof tp || Ne instanceof rp || gh && Ne instanceof gh ? Hn.texImage2D(Hn.TEXTURE_2D, 0, this.format, this.format, Hn.UNSIGNED_BYTE, Ne) : Hn.texImage2D(Hn.TEXTURE_2D, 0, this.format, kt, Jt, 0, this.format, Hn.UNSIGNED_BYTE, Ne.data);
                  else {
                    var nr = ut || { x: 0, y: 0 }, kr = nr.x, Ir = nr.y;
                    Ne instanceof ep || Ne instanceof D4 || Ne instanceof tp || Ne instanceof rp || gh && Ne instanceof gh ? Hn.texSubImage2D(Hn.TEXTURE_2D, 0, kr, Ir, Hn.RGBA, Hn.UNSIGNED_BYTE, Ne) : Hn.texSubImage2D(Hn.TEXTURE_2D, 0, kr, Ir, kt, Jt, Hn.RGBA, Hn.UNSIGNED_BYTE, Ne.data);
                  }
                  this.useMipmap && this.isSizePowerOfTwo() && Hn.generateMipmap(Hn.TEXTURE_2D);
                }, Hh.prototype.bind = function(Ne, je, ut) {
                  var kt = this.context.gl;
                  kt.bindTexture(kt.TEXTURE_2D, this.texture), ut !== kt.LINEAR_MIPMAP_NEAREST || this.isSizePowerOfTwo() || (ut = kt.LINEAR), Ne !== this.filter && (kt.texParameteri(kt.TEXTURE_2D, kt.TEXTURE_MAG_FILTER, Ne), kt.texParameteri(kt.TEXTURE_2D, kt.TEXTURE_MIN_FILTER, ut || Ne), this.filter = Ne), je !== this.wrap && (kt.texParameteri(kt.TEXTURE_2D, kt.TEXTURE_WRAP_S, je), kt.texParameteri(kt.TEXTURE_2D, kt.TEXTURE_WRAP_T, je), this.wrap = je);
                }, Hh.prototype.isSizePowerOfTwo = function() {
                  return this.size[0] === this.size[1] && Math.log(this.size[0]) / Math.LN2 % 1 == 0;
                }, Hh.prototype.destroy = function() {
                  this.context.gl.deleteTexture(this.texture), this.texture = null;
                };
                var ip = function(Ne) {
                  var je = this;
                  this._callback = Ne, this._triggered = !1, typeof MessageChannel < "u" && (this._channel = new MessageChannel(), this._channel.port2.onmessage = function() {
                    je._triggered = !1, je._callback();
                  });
                };
                ip.prototype.trigger = function() {
                  var Ne = this;
                  this._triggered || (this._triggered = !0, this._channel ? this._channel.port1.postMessage(!0) : setTimeout(function() {
                    Ne._triggered = !1, Ne._callback();
                  }, 0));
                }, ip.prototype.remove = function() {
                  delete this._channel, this._callback = function() {
                  };
                };
                var zh = function(Ne, je, ut) {
                  this.target = Ne, this.parent = je, this.mapId = ut, this.callbacks = {}, this.tasks = {}, this.taskQueue = [], this.cancelCallbacks = {}, we(["receive", "process"], this), this.invoker = new ip(this.process), this.target.addEventListener("message", this.receive, !1), this.globalScope = Pe() ? Ne : de;
                };
                function o3(Ne, je, ut) {
                  var kt = 2 * Math.PI * 6378137 / 256 / Math.pow(2, ut);
                  return [Ne * kt - 2 * Math.PI * 6378137 / 2, je * kt - 2 * Math.PI * 6378137 / 2];
                }
                zh.prototype.send = function(Ne, je, ut, kt, Jt) {
                  var fn = this;
                  Jt === void 0 && (Jt = !1);
                  var Sn = Math.round(1e18 * Math.random()).toString(36).substring(0, 10);
                  ut && (this.callbacks[Sn] = ut);
                  var Hn = Xe(this.globalScope) ? void 0 : [];
                  return this.target.postMessage({ id: Sn, type: Ne, hasCallback: !!ut, targetMapId: kt, mustQueue: Jt, sourceMapId: this.mapId, data: Gc(je, Hn) }, Hn), { cancel: function() {
                    ut && delete fn.callbacks[Sn], fn.target.postMessage({ id: Sn, type: "<cancel>", targetMapId: kt, sourceMapId: fn.mapId });
                  } };
                }, zh.prototype.receive = function(Ne) {
                  var je = Ne.data, ut = je.id;
                  if (ut && (!je.targetMapId || this.mapId === je.targetMapId)) if (je.type === "<cancel>") {
                    delete this.tasks[ut];
                    var kt = this.cancelCallbacks[ut];
                    delete this.cancelCallbacks[ut], kt && kt();
                  } else Pe() || je.mustQueue ? (this.tasks[ut] = je, this.taskQueue.push(ut), this.invoker.trigger()) : this.processTask(ut, je);
                }, zh.prototype.process = function() {
                  if (this.taskQueue.length) {
                    var Ne = this.taskQueue.shift(), je = this.tasks[Ne];
                    delete this.tasks[Ne], this.taskQueue.length && this.invoker.trigger(), je && this.processTask(Ne, je);
                  }
                }, zh.prototype.processTask = function(Ne, je) {
                  var ut = this;
                  if (je.type === "<response>") {
                    var kt = this.callbacks[Ne];
                    delete this.callbacks[Ne], kt && (je.error ? kt(Su(je.error)) : kt(null, Su(je.data)));
                  } else {
                    var Jt = !1, fn = Xe(this.globalScope) ? void 0 : [], Sn = je.hasCallback ? function(Ir, qr) {
                      Jt = !0, delete ut.cancelCallbacks[Ne], ut.target.postMessage({ id: Ne, type: "<response>", sourceMapId: ut.mapId, error: Ir ? Gc(Ir) : null, data: Gc(qr, fn) }, fn);
                    } : function(Ir) {
                      Jt = !0;
                    }, Hn = null, nr = Su(je.data);
                    if (this.parent[je.type]) Hn = this.parent[je.type](je.sourceMapId, nr, Sn);
                    else if (this.parent.getWorkerSource) {
                      var kr = je.type.split(".");
                      Hn = this.parent.getWorkerSource(je.sourceMapId, kr[0], nr.source)[kr[1]](nr, Sn);
                    } else Sn(new Error("Could not find function " + je.type));
                    !Jt && Hn && Hn.cancel && (this.cancelCallbacks[Ne] = Hn.cancel);
                  }
                }, zh.prototype.remove = function() {
                  this.invoker.remove(), this.target.removeEventListener("message", this.receive, !1);
                };
                var O1 = function(Ne, je) {
                  Ne && (je ? this.setSouthWest(Ne).setNorthEast(je) : Ne.length === 4 ? this.setSouthWest([Ne[0], Ne[1]]).setNorthEast([Ne[2], Ne[3]]) : this.setSouthWest(Ne[0]).setNorthEast(Ne[1]));
                };
                O1.prototype.setNorthEast = function(Ne) {
                  return this._ne = Ne instanceof l1 ? new l1(Ne.lng, Ne.lat) : l1.convert(Ne), this;
                }, O1.prototype.setSouthWest = function(Ne) {
                  return this._sw = Ne instanceof l1 ? new l1(Ne.lng, Ne.lat) : l1.convert(Ne), this;
                }, O1.prototype.extend = function(Ne) {
                  var je, ut, kt = this._sw, Jt = this._ne;
                  if (Ne instanceof l1) je = Ne, ut = Ne;
                  else {
                    if (!(Ne instanceof O1)) {
                      if (Array.isArray(Ne)) {
                        if (Ne.length === 4 || Ne.every(Array.isArray)) {
                          var fn = Ne;
                          return this.extend(O1.convert(fn));
                        }
                        var Sn = Ne;
                        return this.extend(l1.convert(Sn));
                      }
                      return this;
                    }
                    if (je = Ne._sw, ut = Ne._ne, !je || !ut) return this;
                  }
                  return kt || Jt ? (kt.lng = Math.min(je.lng, kt.lng), kt.lat = Math.min(je.lat, kt.lat), Jt.lng = Math.max(ut.lng, Jt.lng), Jt.lat = Math.max(ut.lat, Jt.lat)) : (this._sw = new l1(je.lng, je.lat), this._ne = new l1(ut.lng, ut.lat)), this;
                }, O1.prototype.getCenter = function() {
                  return new l1((this._sw.lng + this._ne.lng) / 2, (this._sw.lat + this._ne.lat) / 2);
                }, O1.prototype.getSouthWest = function() {
                  return this._sw;
                }, O1.prototype.getNorthEast = function() {
                  return this._ne;
                }, O1.prototype.getNorthWest = function() {
                  return new l1(this.getWest(), this.getNorth());
                }, O1.prototype.getSouthEast = function() {
                  return new l1(this.getEast(), this.getSouth());
                }, O1.prototype.getWest = function() {
                  return this._sw.lng;
                }, O1.prototype.getSouth = function() {
                  return this._sw.lat;
                }, O1.prototype.getEast = function() {
                  return this._ne.lng;
                }, O1.prototype.getNorth = function() {
                  return this._ne.lat;
                }, O1.prototype.toArray = function() {
                  return [this._sw.toArray(), this._ne.toArray()];
                }, O1.prototype.toString = function() {
                  return "LngLatBounds(" + this._sw.toString() + ", " + this._ne.toString() + ")";
                }, O1.prototype.isEmpty = function() {
                  return !(this._sw && this._ne);
                }, O1.prototype.contains = function(Ne) {
                  var je = l1.convert(Ne), ut = je.lng, kt = je.lat, Jt = this._sw.lat <= kt && kt <= this._ne.lat, fn = this._sw.lng <= ut && ut <= this._ne.lng;
                  return this._sw.lng > this._ne.lng && (fn = this._sw.lng >= ut && ut >= this._ne.lng), Jt && fn;
                }, O1.convert = function(Ne) {
                  return !Ne || Ne instanceof O1 ? Ne : new O1(Ne);
                };
                var s3 = 63710088e-1, l1 = function(Ne, je) {
                  if (isNaN(Ne) || isNaN(je)) throw new Error("Invalid LngLat object: (" + Ne + ", " + je + ")");
                  if (this.lng = +Ne, this.lat = +je, this.lat > 90 || this.lat < -90) throw new Error("Invalid LngLat latitude value: must be between -90 and 90");
                };
                l1.prototype.wrap = function() {
                  return new l1(ye(this.lng, -180, 180), this.lat);
                }, l1.prototype.toArray = function() {
                  return [this.lng, this.lat];
                }, l1.prototype.toString = function() {
                  return "LngLat(" + this.lng + ", " + this.lat + ")";
                }, l1.prototype.distanceTo = function(Ne) {
                  var je = Math.PI / 180, ut = this.lat * je, kt = Ne.lat * je, Jt = Math.sin(ut) * Math.sin(kt) + Math.cos(ut) * Math.cos(kt) * Math.cos((Ne.lng - this.lng) * je);
                  return s3 * Math.acos(Math.min(Jt, 1));
                }, l1.prototype.toBounds = function(Ne) {
                  Ne === void 0 && (Ne = 0);
                  var je = 360 * Ne / 40075017, ut = je / Math.cos(Math.PI / 180 * this.lat);
                  return new O1(new l1(this.lng - ut, this.lat - je), new l1(this.lng + ut, this.lat + je));
                }, l1.convert = function(Ne) {
                  if (Ne instanceof l1) return Ne;
                  if (Array.isArray(Ne) && (Ne.length === 2 || Ne.length === 3)) return new l1(Number(Ne[0]), Number(Ne[1]));
                  if (!Array.isArray(Ne) && typeof Ne == "object" && Ne !== null) return new l1(Number("lng" in Ne ? Ne.lng : Ne.lon), Number(Ne.lat));
                  throw new Error("`LngLatLike` argument must be specified as a LngLat instance, an object {lng: <lng>, lat: <lat>}, an object {lon: <lng>, lat: <lat>}, or an array of [<lng>, <lat>]");
                };
                var N4 = 2 * Math.PI * s3;
                function l3(Ne) {
                  return N4 * Math.cos(Ne * Math.PI / 180);
                }
                function P4(Ne) {
                  return (180 + Ne) / 360;
                }
                function $4(Ne) {
                  return (180 - 180 / Math.PI * Math.log(Math.tan(Math.PI / 4 + Ne * Math.PI / 360))) / 360;
                }
                function F4(Ne, je) {
                  return Ne / l3(je);
                }
                function c3(Ne) {
                  var je = 180 - 360 * Ne;
                  return 360 / Math.PI * Math.atan(Math.exp(je * Math.PI / 180)) - 90;
                }
                var vh = function(Ne, je, ut) {
                  ut === void 0 && (ut = 0), this.x = +Ne, this.y = +je, this.z = +ut;
                };
                vh.fromLngLat = function(Ne, je) {
                  je === void 0 && (je = 0);
                  var ut = l1.convert(Ne);
                  return new vh(P4(ut.lng), $4(ut.lat), F4(je, ut.lat));
                }, vh.prototype.toLngLat = function() {
                  return new l1(360 * this.x - 180, c3(this.y));
                }, vh.prototype.toAltitude = function() {
                  return Ne = this.z, je = this.y, Ne * l3(c3(je));
                  var Ne, je;
                }, vh.prototype.meterInMercatorCoordinateUnits = function() {
                  return 1 / N4 * (Ne = c3(this.y), 1 / Math.cos(Ne * Math.PI / 180));
                  var Ne;
                };
                var ff = function(Ne, je, ut) {
                  this.z = Ne, this.x = je, this.y = ut, this.key = g2(0, Ne, Ne, je, ut);
                };
                ff.prototype.equals = function(Ne) {
                  return this.z === Ne.z && this.x === Ne.x && this.y === Ne.y;
                }, ff.prototype.url = function(Ne, je) {
                  var ut, kt, Jt, fn, Sn, Hn = (ut = this.x, kt = this.y, Jt = this.z, fn = o3(256 * ut, 256 * (kt = Math.pow(2, Jt) - kt - 1), Jt), Sn = o3(256 * (ut + 1), 256 * (kt + 1), Jt), fn[0] + "," + fn[1] + "," + Sn[0] + "," + Sn[1]), nr = function(kr, Ir, qr) {
                    for (var ci, ni = "", Ii = kr; Ii > 0; Ii--) ni += (Ir & (ci = 1 << Ii - 1) ? 1 : 0) + (qr & ci ? 2 : 0);
                    return ni;
                  }(this.z, this.x, this.y);
                  return Ne[(this.x + this.y) % Ne.length].replace("{prefix}", (this.x % 16).toString(16) + (this.y % 16).toString(16)).replace("{z}", String(this.z)).replace("{x}", String(this.x)).replace("{y}", String(je === "tms" ? Math.pow(2, this.z) - this.y - 1 : this.y)).replace("{quadkey}", nr).replace("{bbox-epsg-3857}", Hn);
                }, ff.prototype.getTilePoint = function(Ne) {
                  var je = Math.pow(2, this.z);
                  return new le((Ne.x * je - this.x) * $s, (Ne.y * je - this.y) * $s);
                }, ff.prototype.toString = function() {
                  return this.z + "/" + this.x + "/" + this.y;
                };
                var u3 = function(Ne, je) {
                  this.wrap = Ne, this.canonical = je, this.key = g2(Ne, je.z, je.z, je.x, je.y);
                }, q1 = function(Ne, je, ut, kt, Jt) {
                  this.overscaledZ = Ne, this.wrap = je, this.canonical = new ff(ut, +kt, +Jt), this.key = g2(je, Ne, ut, kt, Jt);
                };
                function g2(Ne, je, ut, kt, Jt) {
                  (Ne *= 2) < 0 && (Ne = -1 * Ne - 1);
                  var fn = 1 << ut;
                  return (fn * fn * Ne + fn * Jt + kt).toString(36) + ut.toString(36) + je.toString(36);
                }
                q1.prototype.equals = function(Ne) {
                  return this.overscaledZ === Ne.overscaledZ && this.wrap === Ne.wrap && this.canonical.equals(Ne.canonical);
                }, q1.prototype.scaledTo = function(Ne) {
                  var je = this.canonical.z - Ne;
                  return Ne > this.canonical.z ? new q1(Ne, this.wrap, this.canonical.z, this.canonical.x, this.canonical.y) : new q1(Ne, this.wrap, Ne, this.canonical.x >> je, this.canonical.y >> je);
                }, q1.prototype.calculateScaledKey = function(Ne, je) {
                  var ut = this.canonical.z - Ne;
                  return Ne > this.canonical.z ? g2(this.wrap * +je, Ne, this.canonical.z, this.canonical.x, this.canonical.y) : g2(this.wrap * +je, Ne, Ne, this.canonical.x >> ut, this.canonical.y >> ut);
                }, q1.prototype.isChildOf = function(Ne) {
                  if (Ne.wrap !== this.wrap) return !1;
                  var je = this.canonical.z - Ne.canonical.z;
                  return Ne.overscaledZ === 0 || Ne.overscaledZ < this.overscaledZ && Ne.canonical.x === this.canonical.x >> je && Ne.canonical.y === this.canonical.y >> je;
                }, q1.prototype.children = function(Ne) {
                  if (this.overscaledZ >= Ne) return [new q1(this.overscaledZ + 1, this.wrap, this.canonical.z, this.canonical.x, this.canonical.y)];
                  var je = this.canonical.z + 1, ut = 2 * this.canonical.x, kt = 2 * this.canonical.y;
                  return [new q1(je, this.wrap, je, ut, kt), new q1(je, this.wrap, je, ut + 1, kt), new q1(je, this.wrap, je, ut, kt + 1), new q1(je, this.wrap, je, ut + 1, kt + 1)];
                }, q1.prototype.isLessThan = function(Ne) {
                  return this.wrap < Ne.wrap || !(this.wrap > Ne.wrap) && (this.overscaledZ < Ne.overscaledZ || !(this.overscaledZ > Ne.overscaledZ) && (this.canonical.x < Ne.canonical.x || !(this.canonical.x > Ne.canonical.x) && this.canonical.y < Ne.canonical.y));
                }, q1.prototype.wrapped = function() {
                  return new q1(this.overscaledZ, 0, this.canonical.z, this.canonical.x, this.canonical.y);
                }, q1.prototype.unwrapTo = function(Ne) {
                  return new q1(this.overscaledZ, Ne, this.canonical.z, this.canonical.x, this.canonical.y);
                }, q1.prototype.overscaleFactor = function() {
                  return Math.pow(2, this.overscaledZ - this.canonical.z);
                }, q1.prototype.toUnwrapped = function() {
                  return new u3(this.wrap, this.canonical);
                }, q1.prototype.toString = function() {
                  return this.overscaledZ + "/" + this.canonical.x + "/" + this.canonical.y;
                }, q1.prototype.getTilePoint = function(Ne) {
                  return this.canonical.getTilePoint(new vh(Ne.x - this.wrap, Ne.y));
                }, cs("CanonicalTileID", ff), cs("OverscaledTileID", q1, { omit: ["posMatrix"] });
                var oh = function(Ne, je, ut) {
                  if (this.uid = Ne, je.height !== je.width) throw new RangeError("DEM tiles must be square");
                  if (ut && ut !== "mapbox" && ut !== "terrarium") return Me('"' + ut + '" is not a valid encoding type. Valid types include "mapbox" and "terrarium".');
                  this.stride = je.height;
                  var kt = this.dim = je.height - 2;
                  this.data = new Uint32Array(je.data.buffer), this.encoding = ut || "mapbox";
                  for (var Jt = 0; Jt < kt; Jt++) this.data[this._idx(-1, Jt)] = this.data[this._idx(0, Jt)], this.data[this._idx(kt, Jt)] = this.data[this._idx(kt - 1, Jt)], this.data[this._idx(Jt, -1)] = this.data[this._idx(Jt, 0)], this.data[this._idx(Jt, kt)] = this.data[this._idx(Jt, kt - 1)];
                  this.data[this._idx(-1, -1)] = this.data[this._idx(0, 0)], this.data[this._idx(kt, -1)] = this.data[this._idx(kt - 1, 0)], this.data[this._idx(-1, kt)] = this.data[this._idx(0, kt - 1)], this.data[this._idx(kt, kt)] = this.data[this._idx(kt - 1, kt - 1)];
                };
                oh.prototype.get = function(Ne, je) {
                  var ut = new Uint8Array(this.data.buffer), kt = 4 * this._idx(Ne, je);
                  return (this.encoding === "terrarium" ? this._unpackTerrarium : this._unpackMapbox)(ut[kt], ut[kt + 1], ut[kt + 2]);
                }, oh.prototype.getUnpackVector = function() {
                  return this.encoding === "terrarium" ? [256, 1, 390625e-8, 32768] : [6553.6, 25.6, 0.1, 1e4];
                }, oh.prototype._idx = function(Ne, je) {
                  if (Ne < -1 || Ne >= this.dim + 1 || je < -1 || je >= this.dim + 1) throw new RangeError("out of range source coordinates for DEM data");
                  return (je + 1) * this.stride + (Ne + 1);
                }, oh.prototype._unpackMapbox = function(Ne, je, ut) {
                  return (256 * Ne * 256 + 256 * je + ut) / 10 - 1e4;
                }, oh.prototype._unpackTerrarium = function(Ne, je, ut) {
                  return 256 * Ne + je + ut / 256 - 32768;
                }, oh.prototype.getPixels = function() {
                  return new E1({ width: this.stride, height: this.stride }, new Uint8Array(this.data.buffer));
                }, oh.prototype.backfillBorder = function(Ne, je, ut) {
                  if (this.dim !== Ne.dim) throw new Error("dem dimension mismatch");
                  var kt = je * this.dim, Jt = je * this.dim + this.dim, fn = ut * this.dim, Sn = ut * this.dim + this.dim;
                  switch (je) {
                    case -1:
                      kt = Jt - 1;
                      break;
                    case 1:
                      Jt = kt + 1;
                  }
                  switch (ut) {
                    case -1:
                      fn = Sn - 1;
                      break;
                    case 1:
                      Sn = fn + 1;
                  }
                  for (var Hn = -je * this.dim, nr = -ut * this.dim, kr = fn; kr < Sn; kr++) for (var Ir = kt; Ir < Jt; Ir++) this.data[this._idx(Ir, kr)] = Ne.data[this._idx(Ir + Hn, kr + nr)];
                }, cs("DEMData", oh);
                var V2 = function(Ne) {
                  this._stringToNumber = {}, this._numberToString = [];
                  for (var je = 0; je < Ne.length; je++) {
                    var ut = Ne[je];
                    this._stringToNumber[ut] = je, this._numberToString[je] = ut;
                  }
                };
                V2.prototype.encode = function(Ne) {
                  return this._stringToNumber[Ne];
                }, V2.prototype.decode = function(Ne) {
                  return this._numberToString[Ne];
                };
                var _2 = function(Ne, je, ut, kt, Jt) {
                  this.type = "Feature", this._vectorTileFeature = Ne, Ne._z = je, Ne._x = ut, Ne._y = kt, this.properties = Ne.properties, this.id = Jt;
                }, y2 = { geometry: { configurable: !0 } };
                y2.geometry.get = function() {
                  return this._geometry === void 0 && (this._geometry = this._vectorTileFeature.toGeoJSON(this._vectorTileFeature._x, this._vectorTileFeature._y, this._vectorTileFeature._z).geometry), this._geometry;
                }, y2.geometry.set = function(Ne) {
                  this._geometry = Ne;
                }, _2.prototype.toJSON = function() {
                  var Ne = { geometry: this.geometry };
                  for (var je in this) je !== "_geometry" && je !== "_vectorTileFeature" && (Ne[je] = this[je]);
                  return Ne;
                }, Object.defineProperties(_2.prototype, y2);
                var Jh = function() {
                  this.state = {}, this.stateChanges = {}, this.deletedStates = {};
                };
                Jh.prototype.updateState = function(Ne, je, ut) {
                  var kt = String(je);
                  if (this.stateChanges[Ne] = this.stateChanges[Ne] || {}, this.stateChanges[Ne][kt] = this.stateChanges[Ne][kt] || {}, ge(this.stateChanges[Ne][kt], ut), this.deletedStates[Ne] === null) for (var Jt in this.deletedStates[Ne] = {}, this.state[Ne]) Jt !== kt && (this.deletedStates[Ne][Jt] = null);
                  else if (this.deletedStates[Ne] && this.deletedStates[Ne][kt] === null) for (var fn in this.deletedStates[Ne][kt] = {}, this.state[Ne][kt]) ut[fn] || (this.deletedStates[Ne][kt][fn] = null);
                  else for (var Sn in ut) this.deletedStates[Ne] && this.deletedStates[Ne][kt] && this.deletedStates[Ne][kt][Sn] === null && delete this.deletedStates[Ne][kt][Sn];
                }, Jh.prototype.removeFeatureState = function(Ne, je, ut) {
                  if (this.deletedStates[Ne] !== null) {
                    var kt = String(je);
                    if (this.deletedStates[Ne] = this.deletedStates[Ne] || {}, ut && je !== void 0) this.deletedStates[Ne][kt] !== null && (this.deletedStates[Ne][kt] = this.deletedStates[Ne][kt] || {}, this.deletedStates[Ne][kt][ut] = null);
                    else if (je !== void 0) if (this.stateChanges[Ne] && this.stateChanges[Ne][kt]) for (ut in this.deletedStates[Ne][kt] = {}, this.stateChanges[Ne][kt]) this.deletedStates[Ne][kt][ut] = null;
                    else this.deletedStates[Ne][kt] = null;
                    else this.deletedStates[Ne] = null;
                  }
                }, Jh.prototype.getState = function(Ne, je) {
                  var ut = String(je), kt = this.state[Ne] || {}, Jt = this.stateChanges[Ne] || {}, fn = ge({}, kt[ut], Jt[ut]);
                  if (this.deletedStates[Ne] === null) return {};
                  if (this.deletedStates[Ne]) {
                    var Sn = this.deletedStates[Ne][je];
                    if (Sn === null) return {};
                    for (var Hn in Sn) delete fn[Hn];
                  }
                  return fn;
                }, Jh.prototype.initializeTileState = function(Ne, je) {
                  Ne.setFeatureState(this.state, je);
                }, Jh.prototype.coalesceChanges = function(Ne, je) {
                  var ut = {};
                  for (var kt in this.stateChanges) {
                    this.state[kt] = this.state[kt] || {};
                    var Jt = {};
                    for (var fn in this.stateChanges[kt]) this.state[kt][fn] || (this.state[kt][fn] = {}), ge(this.state[kt][fn], this.stateChanges[kt][fn]), Jt[fn] = this.state[kt][fn];
                    ut[kt] = Jt;
                  }
                  for (var Sn in this.deletedStates) {
                    this.state[Sn] = this.state[Sn] || {};
                    var Hn = {};
                    if (this.deletedStates[Sn] === null) for (var nr in this.state[Sn]) Hn[nr] = {}, this.state[Sn][nr] = {};
                    else for (var kr in this.deletedStates[Sn]) {
                      if (this.deletedStates[Sn][kr] === null) this.state[Sn][kr] = {};
                      else for (var Ir = 0, qr = Object.keys(this.deletedStates[Sn][kr]); Ir < qr.length; Ir += 1) {
                        var ci = qr[Ir];
                        delete this.state[Sn][kr][ci];
                      }
                      Hn[kr] = this.state[Sn][kr];
                    }
                    ut[Sn] = ut[Sn] || {}, ge(ut[Sn], Hn);
                  }
                  if (this.stateChanges = {}, this.deletedStates = {}, Object.keys(ut).length !== 0) for (var ni in Ne) Ne[ni].setFeatureState(ut, je);
                };
                var Gf = function(Ne, je) {
                  this.tileID = Ne, this.x = Ne.canonical.x, this.y = Ne.canonical.y, this.z = Ne.canonical.z, this.grid = new yu($s, 16, 0), this.grid3D = new yu($s, 16, 0), this.featureIndexArray = new Wi(), this.promoteId = je;
                };
                function d3(Ne, je, ut, kt, Jt) {
                  return xe(Ne, function(fn, Sn) {
                    var Hn = je instanceof Ko ? je.get(Sn) : null;
                    return Hn && Hn.evaluate ? Hn.evaluate(ut, kt, Jt) : Hn;
                  });
                }
                function hf(Ne) {
                  for (var je = 1 / 0, ut = 1 / 0, kt = -1 / 0, Jt = -1 / 0, fn = 0, Sn = Ne; fn < Sn.length; fn += 1) {
                    var Hn = Sn[fn];
                    je = Math.min(je, Hn.x), ut = Math.min(ut, Hn.y), kt = Math.max(kt, Hn.x), Jt = Math.max(Jt, Hn.y);
                  }
                  return { minX: je, minY: ut, maxX: kt, maxY: Jt };
                }
                function U2(Ne, je) {
                  return je - Ne;
                }
                Gf.prototype.insert = function(Ne, je, ut, kt, Jt, fn) {
                  var Sn = this.featureIndexArray.length;
                  this.featureIndexArray.emplaceBack(ut, kt, Jt);
                  for (var Hn = fn ? this.grid3D : this.grid, nr = 0; nr < je.length; nr++) {
                    for (var kr = je[nr], Ir = [1 / 0, 1 / 0, -1 / 0, -1 / 0], qr = 0; qr < kr.length; qr++) {
                      var ci = kr[qr];
                      Ir[0] = Math.min(Ir[0], ci.x), Ir[1] = Math.min(Ir[1], ci.y), Ir[2] = Math.max(Ir[2], ci.x), Ir[3] = Math.max(Ir[3], ci.y);
                    }
                    Ir[0] < $s && Ir[1] < $s && Ir[2] >= 0 && Ir[3] >= 0 && Hn.insert(Sn, Ir[0], Ir[1], Ir[2], Ir[3]);
                  }
                }, Gf.prototype.loadVTLayers = function() {
                  return this.vtLayers || (this.vtLayers = new En.VectorTile(new n1(this.rawTileData)).layers, this.sourceLayerCoder = new V2(this.vtLayers ? Object.keys(this.vtLayers).sort() : ["_geojsonTileLayer"])), this.vtLayers;
                }, Gf.prototype.query = function(Ne, je, ut, kt) {
                  var Jt = this;
                  this.loadVTLayers();
                  for (var fn = Ne.params || {}, Sn = $s / Ne.tileSize / Ne.scale, Hn = es(fn.filter), nr = Ne.queryGeometry, kr = Ne.queryPadding * Sn, Ir = hf(nr), qr = this.grid.query(Ir.minX - kr, Ir.minY - kr, Ir.maxX + kr, Ir.maxY + kr), ci = hf(Ne.cameraQueryGeometry), ni = 0, Ii = this.grid3D.query(ci.minX - kr, ci.minY - kr, ci.maxX + kr, ci.maxY + kr, function(eo, co, wo, ds) {
                    return function(Lo, As, Fs, Pl, rl) {
                      for (var Rl = 0, pl = Lo; Rl < pl.length; Rl += 1) {
                        var Zs = pl[Rl];
                        if (As <= Zs.x && Fs <= Zs.y && Pl >= Zs.x && rl >= Zs.y) return !0;
                      }
                      var Hc = [new le(As, Fs), new le(As, rl), new le(Pl, rl), new le(Pl, Fs)];
                      if (Lo.length > 2) {
                        for (var tc = 0, Pu = Hc; tc < Pu.length; tc += 1) if (S1(Lo, Pu[tc])) return !0;
                      }
                      for (var Cl = 0; Cl < Lo.length - 1; Cl++) if (Af(Lo[Cl], Lo[Cl + 1], Hc)) return !0;
                      return !1;
                    }(Ne.cameraQueryGeometry, eo - kr, co - kr, wo + kr, ds + kr);
                  }); ni < Ii.length; ni += 1) {
                    var Gi = Ii[ni];
                    qr.push(Gi);
                  }
                  qr.sort(U2);
                  for (var xa, ga = {}, $a = function(eo) {
                    var co = qr[eo];
                    if (co !== xa) {
                      xa = co;
                      var wo = Jt.featureIndexArray.get(co), ds = null;
                      Jt.loadMatchingFeature(ga, wo.bucketIndex, wo.sourceLayerIndex, wo.featureIndex, Hn, fn.layers, fn.availableImages, je, ut, kt, function(Lo, As, Fs) {
                        return ds || (ds = zu(Lo)), As.queryIntersectsFeature(nr, Lo, Fs, ds, Jt.z, Ne.transform, Sn, Ne.pixelPosMatrix);
                      });
                    }
                  }, Va = 0; Va < qr.length; Va++) $a(Va);
                  return ga;
                }, Gf.prototype.loadMatchingFeature = function(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr, kr, Ir) {
                  var qr = this.bucketLayerIDs[je];
                  if (!fn || function(Lo, As) {
                    for (var Fs = 0; Fs < Lo.length; Fs++) if (As.indexOf(Lo[Fs]) >= 0) return !0;
                    return !1;
                  }(fn, qr)) {
                    var ci = this.sourceLayerCoder.decode(ut), ni = this.vtLayers[ci].feature(kt);
                    if (Jt.needGeometry) {
                      var Ii = Zc(ni, !0);
                      if (!Jt.filter(new El(this.tileID.overscaledZ), Ii, this.tileID.canonical)) return;
                    } else if (!Jt.filter(new El(this.tileID.overscaledZ), ni)) return;
                    for (var Gi = this.getId(ni, ci), xa = 0; xa < qr.length; xa++) {
                      var ga = qr[xa];
                      if (!(fn && fn.indexOf(ga) < 0)) {
                        var $a = Hn[ga];
                        if ($a) {
                          var Va = {};
                          Gi !== void 0 && kr && (Va = kr.getState($a.sourceLayer || "_geojsonTileLayer", Gi));
                          var eo = ge({}, nr[ga]);
                          eo.paint = d3(eo.paint, $a.paint, ni, Va, Sn), eo.layout = d3(eo.layout, $a.layout, ni, Va, Sn);
                          var co = !Ir || Ir(ni, $a, Va);
                          if (co) {
                            var wo = new _2(ni, this.z, this.x, this.y, Gi);
                            wo.layer = eo;
                            var ds = Ne[ga];
                            ds === void 0 && (ds = Ne[ga] = []), ds.push({ featureIndex: kt, feature: wo, intersectionZ: co });
                          }
                        }
                      }
                    }
                  }
                }, Gf.prototype.lookupSymbolFeatures = function(Ne, je, ut, kt, Jt, fn, Sn, Hn) {
                  var nr = {};
                  this.loadVTLayers();
                  for (var kr = es(Jt), Ir = 0, qr = Ne; Ir < qr.length; Ir += 1) {
                    var ci = qr[Ir];
                    this.loadMatchingFeature(nr, ut, kt, ci, kr, fn, Sn, Hn, je);
                  }
                  return nr;
                }, Gf.prototype.hasLayer = function(Ne) {
                  for (var je = 0, ut = this.bucketLayerIDs; je < ut.length; je += 1) for (var kt = 0, Jt = ut[je]; kt < Jt.length; kt += 1) if (Ne === Jt[kt]) return !0;
                  return !1;
                }, Gf.prototype.getId = function(Ne, je) {
                  var ut = Ne.id;
                  if (this.promoteId) {
                    var kt = typeof this.promoteId == "string" ? this.promoteId : this.promoteId[je];
                    typeof (ut = Ne.properties[kt]) == "boolean" && (ut = Number(ut));
                  }
                  return ut;
                }, cs("FeatureIndex", Gf, { omit: ["rawTileData", "sourceLayerCoder"] });
                var fd = function(Ne, je) {
                  this.tileID = Ne, this.uid = be(), this.uses = 0, this.tileSize = je, this.buckets = {}, this.expirationTime = null, this.queryPadding = 0, this.hasSymbolBuckets = !1, this.hasRTLText = !1, this.dependencies = {}, this.expiredRequestCount = 0, this.state = "loading";
                };
                fd.prototype.registerFadeDuration = function(Ne) {
                  var je = Ne + this.timeAdded;
                  je < dt.now() || this.fadeEndTime && je < this.fadeEndTime || (this.fadeEndTime = je);
                }, fd.prototype.wasRequested = function() {
                  return this.state === "errored" || this.state === "loaded" || this.state === "reloading";
                }, fd.prototype.loadVectorData = function(Ne, je, ut) {
                  if (this.hasData() && this.unloadVectorData(), this.state = "loaded", Ne) {
                    for (var kt in Ne.featureIndex && (this.latestFeatureIndex = Ne.featureIndex, Ne.rawTileData ? (this.latestRawTileData = Ne.rawTileData, this.latestFeatureIndex.rawTileData = Ne.rawTileData) : this.latestRawTileData && (this.latestFeatureIndex.rawTileData = this.latestRawTileData)), this.collisionBoxArray = Ne.collisionBoxArray, this.buckets = function(kr, Ir) {
                      var qr = {};
                      if (!Ir) return qr;
                      for (var ci = function() {
                        var Gi = Ii[ni], xa = Gi.layerIds.map(function(eo) {
                          return Ir.getLayer(eo);
                        }).filter(Boolean);
                        if (xa.length !== 0) {
                          Gi.layers = xa, Gi.stateDependentLayerIds && (Gi.stateDependentLayers = Gi.stateDependentLayerIds.map(function(eo) {
                            return xa.filter(function(co) {
                              return co.id === eo;
                            })[0];
                          }));
                          for (var ga = 0, $a = xa; ga < $a.length; ga += 1) {
                            var Va = $a[ga];
                            qr[Va.id] = Gi;
                          }
                        }
                      }, ni = 0, Ii = kr; ni < Ii.length; ni += 1) ci();
                      return qr;
                    }(Ne.buckets, je.style), this.hasSymbolBuckets = !1, this.buckets) {
                      var Jt = this.buckets[kt];
                      if (Jt instanceof Du) {
                        if (this.hasSymbolBuckets = !0, !ut) break;
                        Jt.justReloaded = !0;
                      }
                    }
                    if (this.hasRTLText = !1, this.hasSymbolBuckets) for (var fn in this.buckets) {
                      var Sn = this.buckets[fn];
                      if (Sn instanceof Du && Sn.hasRTLText) {
                        this.hasRTLText = !0, Uc.isLoading() || Uc.isLoaded() || Zo() !== "deferred" || Ou();
                        break;
                      }
                    }
                    for (var Hn in this.queryPadding = 0, this.buckets) {
                      var nr = this.buckets[Hn];
                      this.queryPadding = Math.max(this.queryPadding, je.style.getLayer(Hn).queryRadius(nr));
                    }
                    Ne.imageAtlas && (this.imageAtlas = Ne.imageAtlas), Ne.glyphAtlasImage && (this.glyphAtlasImage = Ne.glyphAtlasImage);
                  } else this.collisionBoxArray = new Vn();
                }, fd.prototype.unloadVectorData = function() {
                  for (var Ne in this.buckets) this.buckets[Ne].destroy();
                  this.buckets = {}, this.imageAtlasTexture && this.imageAtlasTexture.destroy(), this.imageAtlas && (this.imageAtlas = null), this.glyphAtlasTexture && this.glyphAtlasTexture.destroy(), this.latestFeatureIndex = null, this.state = "unloaded";
                }, fd.prototype.getBucket = function(Ne) {
                  return this.buckets[Ne.id];
                }, fd.prototype.upload = function(Ne) {
                  for (var je in this.buckets) {
                    var ut = this.buckets[je];
                    ut.uploadPending() && ut.upload(Ne);
                  }
                  var kt = Ne.gl;
                  this.imageAtlas && !this.imageAtlas.uploaded && (this.imageAtlasTexture = new Hh(Ne, this.imageAtlas.image, kt.RGBA), this.imageAtlas.uploaded = !0), this.glyphAtlasImage && (this.glyphAtlasTexture = new Hh(Ne, this.glyphAtlasImage, kt.ALPHA), this.glyphAtlasImage = null);
                }, fd.prototype.prepare = function(Ne) {
                  this.imageAtlas && this.imageAtlas.patchUpdatedImages(Ne, this.imageAtlasTexture);
                }, fd.prototype.queryRenderedFeatures = function(Ne, je, ut, kt, Jt, fn, Sn, Hn, nr, kr) {
                  return this.latestFeatureIndex && this.latestFeatureIndex.rawTileData ? this.latestFeatureIndex.query({ queryGeometry: kt, cameraQueryGeometry: Jt, scale: fn, tileSize: this.tileSize, pixelPosMatrix: kr, transform: Hn, params: Sn, queryPadding: this.queryPadding * nr }, Ne, je, ut) : {};
                }, fd.prototype.querySourceFeatures = function(Ne, je) {
                  var ut = this.latestFeatureIndex;
                  if (ut && ut.rawTileData) {
                    var kt = ut.loadVTLayers(), Jt = je ? je.sourceLayer : "", fn = kt._geojsonTileLayer || kt[Jt];
                    if (fn) for (var Sn = es(je && je.filter), Hn = this.tileID.canonical, nr = Hn.z, kr = Hn.x, Ir = Hn.y, qr = { z: nr, x: kr, y: Ir }, ci = 0; ci < fn.length; ci++) {
                      var ni = fn.feature(ci);
                      if (Sn.needGeometry) {
                        var Ii = Zc(ni, !0);
                        if (!Sn.filter(new El(this.tileID.overscaledZ), Ii, this.tileID.canonical)) continue;
                      } else if (!Sn.filter(new El(this.tileID.overscaledZ), ni)) continue;
                      var Gi = ut.getId(ni, Jt), xa = new _2(ni, nr, kr, Ir, Gi);
                      xa.tile = qr, Ne.push(xa);
                    }
                  }
                }, fd.prototype.hasData = function() {
                  return this.state === "loaded" || this.state === "reloading" || this.state === "expired";
                }, fd.prototype.patternsLoaded = function() {
                  return this.imageAtlas && !!Object.keys(this.imageAtlas.patternPositions).length;
                }, fd.prototype.setExpiryData = function(Ne) {
                  var je = this.expirationTime;
                  if (Ne.cacheControl) {
                    var ut = $e(Ne.cacheControl);
                    ut["max-age"] && (this.expirationTime = Date.now() + 1e3 * ut["max-age"]);
                  } else Ne.expires && (this.expirationTime = new Date(Ne.expires).getTime());
                  if (this.expirationTime) {
                    var kt = Date.now(), Jt = !1;
                    if (this.expirationTime > kt) Jt = !1;
                    else if (je) if (this.expirationTime < je) Jt = !0;
                    else {
                      var fn = this.expirationTime - je;
                      fn ? this.expirationTime = kt + Math.max(fn, 3e4) : Jt = !0;
                    }
                    else Jt = !0;
                    Jt ? (this.expiredRequestCount++, this.state = "expired") : this.expiredRequestCount = 0;
                  }
                }, fd.prototype.getExpiryTimeout = function() {
                  if (this.expirationTime) return this.expiredRequestCount ? 1e3 * (1 << Math.min(this.expiredRequestCount - 1, 31)) : Math.min(this.expirationTime - (/* @__PURE__ */ new Date()).getTime(), Math.pow(2, 31) - 1);
                }, fd.prototype.setFeatureState = function(Ne, je) {
                  if (this.latestFeatureIndex && this.latestFeatureIndex.rawTileData && Object.keys(Ne).length !== 0) {
                    var ut = this.latestFeatureIndex.loadVTLayers();
                    for (var kt in this.buckets) if (je.style.hasLayer(kt)) {
                      var Jt = this.buckets[kt], fn = Jt.layers[0].sourceLayer || "_geojsonTileLayer", Sn = ut[fn], Hn = Ne[fn];
                      if (Sn && Hn && Object.keys(Hn).length !== 0) {
                        Jt.update(Hn, Sn, this.imageAtlas && this.imageAtlas.patternPositions || {});
                        var nr = je && je.style && je.style.getLayer(kt);
                        nr && (this.queryPadding = Math.max(this.queryPadding, nr.queryRadius(Jt)));
                      }
                    }
                  }
                }, fd.prototype.holdingForFade = function() {
                  return this.symbolFadeHoldUntil !== void 0;
                }, fd.prototype.symbolFadeFinished = function() {
                  return !this.symbolFadeHoldUntil || this.symbolFadeHoldUntil < dt.now();
                }, fd.prototype.clearFadeHold = function() {
                  this.symbolFadeHoldUntil = void 0;
                }, fd.prototype.setHoldDuration = function(Ne) {
                  this.symbolFadeHoldUntil = dt.now() + Ne;
                }, fd.prototype.setDependencies = function(Ne, je) {
                  for (var ut = {}, kt = 0, Jt = je; kt < Jt.length; kt += 1) ut[Jt[kt]] = !0;
                  this.dependencies[Ne] = ut;
                }, fd.prototype.hasDependency = function(Ne, je) {
                  for (var ut = 0, kt = Ne; ut < kt.length; ut += 1) {
                    var Jt = kt[ut], fn = this.dependencies[Jt];
                    if (fn) {
                      for (var Sn = 0, Hn = je; Sn < Hn.length; Sn += 1) if (fn[Hn[Sn]]) return !0;
                    }
                  }
                  return !1;
                };
                var bh = de.performance, f3 = function(Ne) {
                  this._marks = { start: [Ne.url, "start"].join("#"), end: [Ne.url, "end"].join("#"), measure: Ne.url.toString() }, bh.mark(this._marks.start);
                };
                f3.prototype.finish = function() {
                  bh.mark(this._marks.end);
                  var Ne = bh.getEntriesByName(this._marks.measure);
                  return Ne.length === 0 && (bh.measure(this._marks.measure, this._marks.start, this._marks.end), Ne = bh.getEntriesByName(this._marks.measure), bh.clearMarks(this._marks.start), bh.clearMarks(this._marks.end), bh.clearMeasures(this._marks.measure)), Ne;
                }, ee.Actor = zh, ee.AlphaImage = jd, ee.CanonicalTileID = ff, ee.CollisionBoxArray = Vn, ee.Color = Pr, ee.DEMData = oh, ee.DataConstantProperty = ks, ee.DictionaryCoder = V2, ee.EXTENT = $s, ee.ErrorEvent = bn, ee.EvaluationParameters = El, ee.Event = gn, ee.Evented = an, ee.FeatureIndex = Gf, ee.FillBucket = td, ee.FillExtrusionBucket = bo, ee.ImageAtlas = ba, ee.ImagePosition = Ma, ee.LineBucket = Bo, ee.LngLat = l1, ee.LngLatBounds = O1, ee.MercatorCoordinate = vh, ee.ONE_EM = Ws, ee.OverscaledTileID = q1, ee.Point = le, ee.Point$1 = le, ee.Properties = $c, ee.Protobuf = n1, ee.RGBAImage = E1, ee.RequestManager = St, ee.RequestPerformance = f3, ee.ResourceType = Pn, ee.SegmentVector = ra, ee.SourceFeatureState = Jh, ee.StructArrayLayout1ui2 = On, ee.StructArrayLayout2f1f2i16 = Ue, ee.StructArrayLayout2i4 = tu, ee.StructArrayLayout3ui6 = ht, ee.StructArrayLayout4i8 = w1, ee.SymbolBucket = Du, ee.Texture = Hh, ee.Tile = fd, ee.Transitionable = hu, ee.Uniform1f = ml, ee.Uniform1i = Is, ee.Uniform2f = nl, ee.Uniform3f = Il, ee.Uniform4f = Fl, ee.UniformColor = qc, ee.UniformMatrix4f = jc, ee.UnwrappedTileID = u3, ee.ValidationError = Mn, ee.WritingMode = fa, ee.ZoomHistory = Lc, ee.add = function(Ne, je, ut) {
                  return Ne[0] = je[0] + ut[0], Ne[1] = je[1] + ut[1], Ne[2] = je[2] + ut[2], Ne;
                }, ee.addDynamicAttributes = G0, ee.asyncAll = function(Ne, je, ut) {
                  if (!Ne.length) return ut(null, []);
                  var kt = Ne.length, Jt = new Array(Ne.length), fn = null;
                  Ne.forEach(function(Sn, Hn) {
                    je(Sn, function(nr, kr) {
                      nr && (fn = nr), Jt[Hn] = kr, --kt == 0 && ut(fn, Jt);
                    });
                  });
                }, ee.bezier = he, ee.bindAll = we, ee.browser = dt, ee.cacheEntryPossiblyAdded = function(Ne) {
                  ++wn > Wt && (Ne.getActor().send("enforceCacheSizeLimit", Ut), wn = 0);
                }, ee.clamp = pe, ee.clearTileCache = function(Ne) {
                  var je = de.caches.delete(Qt);
                  Ne && je.catch(Ne).then(function() {
                    return Ne();
                  });
                }, ee.clipLine = Dh, ee.clone = function(Ne) {
                  var je = new vd(16);
                  return je[0] = Ne[0], je[1] = Ne[1], je[2] = Ne[2], je[3] = Ne[3], je[4] = Ne[4], je[5] = Ne[5], je[6] = Ne[6], je[7] = Ne[7], je[8] = Ne[8], je[9] = Ne[9], je[10] = Ne[10], je[11] = Ne[11], je[12] = Ne[12], je[13] = Ne[13], je[14] = Ne[14], je[15] = Ne[15], je;
                }, ee.clone$1 = Qe, ee.clone$2 = function(Ne) {
                  var je = new vd(3);
                  return je[0] = Ne[0], je[1] = Ne[1], je[2] = Ne[2], je;
                }, ee.collisionCircleLayout = xs, ee.config = Je, ee.create = function() {
                  var Ne = new vd(16);
                  return vd != Float32Array && (Ne[1] = 0, Ne[2] = 0, Ne[3] = 0, Ne[4] = 0, Ne[6] = 0, Ne[7] = 0, Ne[8] = 0, Ne[9] = 0, Ne[11] = 0, Ne[12] = 0, Ne[13] = 0, Ne[14] = 0), Ne[0] = 1, Ne[5] = 1, Ne[10] = 1, Ne[15] = 1, Ne;
                }, ee.create$1 = function() {
                  var Ne = new vd(9);
                  return vd != Float32Array && (Ne[1] = 0, Ne[2] = 0, Ne[3] = 0, Ne[5] = 0, Ne[6] = 0, Ne[7] = 0), Ne[0] = 1, Ne[4] = 1, Ne[8] = 1, Ne;
                }, ee.create$2 = function() {
                  var Ne = new vd(4);
                  return vd != Float32Array && (Ne[1] = 0, Ne[2] = 0), Ne[0] = 1, Ne[3] = 1, Ne;
                }, ee.createCommonjsModule = ae, ee.createExpression = da, ee.createLayout = xc, ee.createStyleLayer = function(Ne) {
                  return Ne.type === "custom" ? new t_(Ne) : new J0[Ne.type](Ne);
                }, ee.cross = function(Ne, je, ut) {
                  var kt = je[0], Jt = je[1], fn = je[2], Sn = ut[0], Hn = ut[1], nr = ut[2];
                  return Ne[0] = Jt * nr - fn * Hn, Ne[1] = fn * Sn - kt * nr, Ne[2] = kt * Hn - Jt * Sn, Ne;
                }, ee.deepEqual = function Ne(je, ut) {
                  if (Array.isArray(je)) {
                    if (!Array.isArray(ut) || je.length !== ut.length) return !1;
                    for (var kt = 0; kt < je.length; kt++) if (!Ne(je[kt], ut[kt])) return !1;
                    return !0;
                  }
                  if (typeof je == "object" && je !== null && ut !== null) {
                    if (typeof ut != "object" || Object.keys(je).length !== Object.keys(ut).length) return !1;
                    for (var Jt in je) if (!Ne(je[Jt], ut[Jt])) return !1;
                    return !0;
                  }
                  return je === ut;
                }, ee.dot = function(Ne, je) {
                  return Ne[0] * je[0] + Ne[1] * je[1] + Ne[2] * je[2];
                }, ee.dot$1 = function(Ne, je) {
                  return Ne[0] * je[0] + Ne[1] * je[1] + Ne[2] * je[2] + Ne[3] * je[3];
                }, ee.ease = me, ee.emitValidationErrors = Ac, ee.endsWith = Ee, ee.enforceCacheSizeLimit = function(Ne) {
                  vn(), Tt && Tt.then(function(je) {
                    je.keys().then(function(ut) {
                      for (var kt = 0; kt < ut.length - Ne; kt++) je.delete(ut[kt]);
                    });
                  });
                }, ee.evaluateSizeForFeature = dd, ee.evaluateSizeForZoom = nh, ee.evaluateVariableOffset = m2, ee.evented = Rc, ee.extend = ge, ee.featureFilter = es, ee.filterObject = Le, ee.fromRotation = function(Ne, je) {
                  var ut = Math.sin(je), kt = Math.cos(je);
                  return Ne[0] = kt, Ne[1] = ut, Ne[2] = 0, Ne[3] = -ut, Ne[4] = kt, Ne[5] = 0, Ne[6] = 0, Ne[7] = 0, Ne[8] = 1, Ne;
                }, ee.getAnchorAlignment = no, ee.getAnchorJustification = Ph, ee.getArrayBuffer = wt, ee.getImage = un, ee.getJSON = function(Ne, je) {
                  return pt(ge(Ne, { type: "json" }), je);
                }, ee.getRTLTextPluginStatus = Zo, ee.getReferrer = rt, ee.getVideo = function(Ne, je) {
                  var ut, kt, Jt = de.document.createElement("video");
                  Jt.muted = !0, Jt.onloadstart = function() {
                    je(null, Jt);
                  };
                  for (var fn = 0; fn < Ne.length; fn++) {
                    var Sn = de.document.createElement("source");
                    ut = Ne[fn], kt = void 0, (kt = de.document.createElement("a")).href = ut, kt.protocol === de.document.location.protocol && kt.host === de.document.location.host || (Jt.crossOrigin = "Anonymous"), Sn.src = Ne[fn], Jt.appendChild(Sn);
                  }
                  return { cancel: function() {
                  } };
                }, ee.identity = Ff, ee.invert = function(Ne, je) {
                  var ut = je[0], kt = je[1], Jt = je[2], fn = je[3], Sn = je[4], Hn = je[5], nr = je[6], kr = je[7], Ir = je[8], qr = je[9], ci = je[10], ni = je[11], Ii = je[12], Gi = je[13], xa = je[14], ga = je[15], $a = ut * Hn - kt * Sn, Va = ut * nr - Jt * Sn, eo = ut * kr - fn * Sn, co = kt * nr - Jt * Hn, wo = kt * kr - fn * Hn, ds = Jt * kr - fn * nr, Lo = Ir * Gi - qr * Ii, As = Ir * xa - ci * Ii, Fs = Ir * ga - ni * Ii, Pl = qr * xa - ci * Gi, rl = qr * ga - ni * Gi, Rl = ci * ga - ni * xa, pl = $a * Rl - Va * rl + eo * Pl + co * Fs - wo * As + ds * Lo;
                  return pl ? (pl = 1 / pl, Ne[0] = (Hn * Rl - nr * rl + kr * Pl) * pl, Ne[1] = (Jt * rl - kt * Rl - fn * Pl) * pl, Ne[2] = (Gi * ds - xa * wo + ga * co) * pl, Ne[3] = (ci * wo - qr * ds - ni * co) * pl, Ne[4] = (nr * Fs - Sn * Rl - kr * As) * pl, Ne[5] = (ut * Rl - Jt * Fs + fn * As) * pl, Ne[6] = (xa * eo - Ii * ds - ga * Va) * pl, Ne[7] = (Ir * ds - ci * eo + ni * Va) * pl, Ne[8] = (Sn * rl - Hn * Fs + kr * Lo) * pl, Ne[9] = (kt * Fs - ut * rl - fn * Lo) * pl, Ne[10] = (Ii * wo - Gi * eo + ga * $a) * pl, Ne[11] = (qr * eo - Ir * wo - ni * $a) * pl, Ne[12] = (Hn * As - Sn * Pl - nr * Lo) * pl, Ne[13] = (ut * Pl - kt * As + Jt * Lo) * pl, Ne[14] = (Gi * Va - Ii * co - xa * $a) * pl, Ne[15] = (Ir * co - qr * Va + ci * $a) * pl, Ne) : null;
                }, ee.isChar = Do, ee.isMapboxURL = Lt, ee.keysDifference = function(Ne, je) {
                  var ut = [];
                  for (var kt in Ne) kt in je || ut.push(kt);
                  return ut;
                }, ee.makeRequest = pt, ee.mapObject = xe, ee.mercatorXfromLng = P4, ee.mercatorYfromLat = $4, ee.mercatorZfromAltitude = F4, ee.mul = rf, ee.multiply = Uu, ee.mvt = En, ee.nextPowerOfTwo = function(Ne) {
                  return Ne <= 1 ? 1 : Math.pow(2, Math.ceil(Math.log(Ne) / Math.LN2));
                }, ee.normalize = function(Ne, je) {
                  var ut = je[0], kt = je[1], Jt = je[2], fn = ut * ut + kt * kt + Jt * Jt;
                  return fn > 0 && (fn = 1 / Math.sqrt(fn)), Ne[0] = je[0] * fn, Ne[1] = je[1] * fn, Ne[2] = je[2] * fn, Ne;
                }, ee.number = Tn, ee.offscreenCanvasSupported = Rn, ee.ortho = function(Ne, je, ut, kt, Jt, fn, Sn) {
                  var Hn = 1 / (je - ut), nr = 1 / (kt - Jt), kr = 1 / (fn - Sn);
                  return Ne[0] = -2 * Hn, Ne[1] = 0, Ne[2] = 0, Ne[3] = 0, Ne[4] = 0, Ne[5] = -2 * nr, Ne[6] = 0, Ne[7] = 0, Ne[8] = 0, Ne[9] = 0, Ne[10] = 2 * kr, Ne[11] = 0, Ne[12] = (je + ut) * Hn, Ne[13] = (Jt + kt) * nr, Ne[14] = (Sn + fn) * kr, Ne[15] = 1, Ne;
                }, ee.parseGlyphPBF = function(Ne) {
                  return new n1(Ne).readFields(ja, []);
                }, ee.pbf = n1, ee.performSymbolLayout = function(Ne, je, ut, kt, Jt, fn, Sn) {
                  Ne.createArrays();
                  var Hn = 512 * Ne.overscaling;
                  Ne.tilePixelRatio = $s / Hn, Ne.compareText = {}, Ne.iconsNeedLinear = !1;
                  var nr = Ne.layers[0].layout, kr = Ne.layers[0]._unevaluatedLayout._values, Ir = {};
                  if (Ne.textSizeData.kind === "composite") {
                    var qr = Ne.textSizeData, ci = qr.minZoom, ni = qr.maxZoom;
                    Ir.compositeTextSizes = [kr["text-size"].possiblyEvaluate(new El(ci), Sn), kr["text-size"].possiblyEvaluate(new El(ni), Sn)];
                  }
                  if (Ne.iconSizeData.kind === "composite") {
                    var Ii = Ne.iconSizeData, Gi = Ii.minZoom, xa = Ii.maxZoom;
                    Ir.compositeIconSizes = [kr["icon-size"].possiblyEvaluate(new El(Gi), Sn), kr["icon-size"].possiblyEvaluate(new El(xa), Sn)];
                  }
                  Ir.layoutTextSize = kr["text-size"].possiblyEvaluate(new El(Ne.zoom + 1), Sn), Ir.layoutIconSize = kr["icon-size"].possiblyEvaluate(new El(Ne.zoom + 1), Sn), Ir.textMaxSize = kr["text-size"].possiblyEvaluate(new El(18));
                  for (var ga = nr.get("text-line-height") * Ws, $a = nr.get("text-rotation-alignment") === "map" && nr.get("symbol-placement") !== "point", Va = nr.get("text-keep-upright"), eo = nr.get("text-size"), co = function() {
                    var Lo = ds[wo], As = nr.get("text-font").evaluate(Lo, {}, Sn).join(","), Fs = eo.evaluate(Lo, {}, Sn), Pl = Ir.layoutTextSize.evaluate(Lo, {}, Sn), rl = Ir.layoutIconSize.evaluate(Lo, {}, Sn), Rl = { horizontal: {}, vertical: void 0 }, pl = Lo.text, Zs = [0, 0];
                    if (pl) {
                      var Hc = pl.toString(), tc = nr.get("text-letter-spacing").evaluate(Lo, {}, Sn) * Ws, Pu = function(zt) {
                        for (var ln = 0, pn = zt; ln < pn.length; ln += 1) if (Cn = pn[ln].charCodeAt(0), Do.Arabic(Cn) || Do["Arabic Supplement"](Cn) || Do["Arabic Extended-A"](Cn) || Do["Arabic Presentation Forms-A"](Cn) || Do["Arabic Presentation Forms-B"](Cn)) return !1;
                        var Cn;
                        return !0;
                      }(Hc) ? tc : 0, Cl = nr.get("text-anchor").evaluate(Lo, {}, Sn), vu = nr.get("text-variable-anchor");
                      if (!vu) {
                        var qu = nr.get("text-radial-offset").evaluate(Lo, {}, Sn);
                        Zs = qu ? m2(Cl, [qu * Ws, Kh]) : nr.get("text-offset").evaluate(Lo, {}, Sn).map(function(zt) {
                          return zt * Ws;
                        });
                      }
                      var Nu = $a ? "center" : nr.get("text-justify").evaluate(Lo, {}, Sn), ju = nr.get("symbol-placement"), $u = ju === "point" ? nr.get("text-max-width").evaluate(Lo, {}, Sn) * Ws : 0, $1 = function() {
                        Ne.allowVerticalPlacement && Ru(Hc) && (Rl.vertical = qi(pl, je, ut, Jt, As, $u, ga, Cl, "left", Pu, Zs, fa.vertical, !0, ju, Pl, Fs));
                      };
                      if (!$a && vu) {
                        for (var Zu = Nu === "auto" ? vu.map(function(zt) {
                          return Ph(zt);
                        }) : [Nu], F1 = !1, Td = 0; Td < Zu.length; Td++) {
                          var hd = Zu[Td];
                          if (!Rl.horizontal[hd]) if (F1) Rl.horizontal[hd] = Rl.horizontal[0];
                          else {
                            var Dd = qi(pl, je, ut, Jt, As, $u, ga, "center", hd, Pu, Zs, fa.horizontal, !1, ju, Pl, Fs);
                            Dd && (Rl.horizontal[hd] = Dd, F1 = Dd.positionedLines.length === 1);
                          }
                        }
                        $1();
                      } else {
                        Nu === "auto" && (Nu = Ph(Cl));
                        var Qd = qi(pl, je, ut, Jt, As, $u, ga, Cl, Nu, Pu, Zs, fa.horizontal, !1, ju, Pl, Fs);
                        Qd && (Rl.horizontal[Nu] = Qd), $1(), Ru(Hc) && $a && Va && (Rl.vertical = qi(pl, je, ut, Jt, As, $u, ga, Cl, Nu, Pu, Zs, fa.vertical, !1, ju, Pl, Fs));
                      }
                    }
                    var Ye = void 0, He = !1;
                    if (Lo.icon && Lo.icon.name) {
                      var ot = kt[Lo.icon.name];
                      ot && (Ye = function(zt, ln, pn) {
                        var Cn = no(pn), Nn = Cn.horizontalAlign, Wn = Cn.verticalAlign, lr = ln[0], xr = ln[1], Nr = lr - zt.displaySize[0] * Nn, Kr = Nr + zt.displaySize[0], xi = xr - zt.displaySize[1] * Wn;
                        return { image: zt, top: xi, bottom: xi + zt.displaySize[1], left: Nr, right: Kr };
                      }(Jt[Lo.icon.name], nr.get("icon-offset").evaluate(Lo, {}, Sn), nr.get("icon-anchor").evaluate(Lo, {}, Sn)), He = ot.sdf, Ne.sdfIcons === void 0 ? Ne.sdfIcons = ot.sdf : Ne.sdfIcons !== ot.sdf && Me("Style sheet warning: Cannot mix SDF and non-SDF icons in one buffer"), (ot.pixelRatio !== Ne.pixelRatio || nr.get("icon-rotate").constantOr(1) !== 0) && (Ne.iconsNeedLinear = !0));
                    }
                    var xt = k4(Rl.horizontal) || Rl.vertical;
                    Ne.iconsInText = !!xt && xt.iconsInText, (xt || Ye) && function(zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi) {
                      var ui = Wn.textMaxSize.evaluate(ln, {});
                      ui === void 0 && (ui = lr);
                      var Ui, ia = zt.layers[0].layout, Na = ia.get("icon-offset").evaluate(ln, {}, xi), Ta = k4(pn.horizontal), Ra = 24, oo = lr / Ra, fs = zt.tilePixelRatio * oo, Bs = zt.tilePixelRatio * ui / Ra, fl = zt.tilePixelRatio * xr, Ns = zt.tilePixelRatio * ia.get("symbol-spacing"), Es = ia.get("text-padding") * zt.tilePixelRatio, cl = ia.get("icon-padding") * zt.tilePixelRatio, Xs = ia.get("text-max-angle") / 180 * Math.PI, Ps = ia.get("text-rotation-alignment") === "map" && ia.get("symbol-placement") !== "point", ko = ia.get("icon-rotation-alignment") === "map" && ia.get("symbol-placement") !== "point", Ks = ia.get("symbol-placement"), el = Ns / 2, Nl = ia.get("icon-text-fit");
                      Cn && Nl !== "none" && (zt.allowVerticalPlacement && pn.vertical && (Ui = Ys(Cn, pn.vertical, Nl, ia.get("icon-text-fit-padding"), Na, oo)), Ta && (Cn = Ys(Cn, Ta, Nl, ia.get("icon-text-fit-padding"), Na, oo)));
                      var bu = function(pf, pd) {
                        pd.x < 0 || pd.x >= $s || pd.y < 0 || pd.y >= $s || function(mu, u1, wf, Of, ap, h3, q2, If, v2, Sf, j2, G2, p3, B4, W2, H4, z4, V4, U4, q4, G1, n2, m3, sh, n_) {
                          var j4, ad, kd, Df, Gd, Nf = mu.addToLineVertexArray(u1, wf), lh = 0, g3 = 0, my = 0, gy = 0, r_ = -1, i_ = -1, r2 = {}, yy = Aa(""), a_ = 0, o_ = 0;
                          if (If._unevaluatedLayout.getValue("text-radial-offset") === void 0 ? (a_ = (j4 = If.layout.get("text-offset").evaluate(G1, {}, sh).map(function(y3) {
                            return y3 * Ws;
                          }))[0], o_ = j4[1]) : (a_ = If.layout.get("text-radial-offset").evaluate(G1, {}, sh) * Ws, o_ = Kh), mu.allowVerticalPlacement && Of.vertical) {
                            var vy = If.layout.get("text-rotate").evaluate(G1, {}, sh) + 90, r7 = Of.vertical;
                            Df = new j0(v2, u1, Sf, j2, G2, r7, p3, B4, W2, vy), q2 && (Gd = new j0(v2, u1, Sf, j2, G2, q2, z4, V4, W2, vy));
                          }
                          if (ap) {
                            var s_ = If.layout.get("icon-rotate").evaluate(G1, {}), by = If.layout.get("icon-text-fit") !== "none", Ty = Nh(ap, s_, m3, by), l_ = q2 ? Nh(q2, s_, m3, by) : void 0;
                            kd = new j0(v2, u1, Sf, j2, G2, ap, z4, V4, !1, s_), lh = 4 * Ty.length;
                            var wy = mu.iconSizeData, _3 = null;
                            wy.kind === "source" ? (_3 = [js * If.layout.get("icon-size").evaluate(G1, {})])[0] > jf && Me(mu.layerIds[0] + ': Value for "icon-size" is >= ' + $h + '. Reduce your "icon-size".') : wy.kind === "composite" && ((_3 = [js * n2.compositeIconSizes[0].evaluate(G1, {}, sh), js * n2.compositeIconSizes[1].evaluate(G1, {}, sh)])[0] > jf || _3[1] > jf) && Me(mu.layerIds[0] + ': Value for "icon-size" is >= ' + $h + '. Reduce your "icon-size".'), mu.addSymbols(mu.icon, Ty, _3, q4, U4, G1, !1, u1, Nf.lineStartIndex, Nf.lineLength, -1, sh), r_ = mu.icon.placedSymbolArray.length - 1, l_ && (g3 = 4 * l_.length, mu.addSymbols(mu.icon, l_, _3, q4, U4, G1, fa.vertical, u1, Nf.lineStartIndex, Nf.lineLength, -1, sh), i_ = mu.icon.placedSymbolArray.length - 1);
                          }
                          for (var Sy in Of.horizontal) {
                            var G4 = Of.horizontal[Sy];
                            if (!ad) {
                              yy = Aa(G4.text);
                              var i7 = If.layout.get("text-rotate").evaluate(G1, {}, sh);
                              ad = new j0(v2, u1, Sf, j2, G2, G4, p3, B4, W2, i7);
                            }
                            var Ey = G4.positionedLines.length === 1;
                            if (my += L4(mu, u1, G4, h3, If, W2, G1, H4, Nf, Of.vertical ? fa.horizontal : fa.horizontalOnly, Ey ? Object.keys(Of.horizontal) : [Sy], r2, r_, n2, sh), Ey) break;
                          }
                          Of.vertical && (gy += L4(mu, u1, Of.vertical, h3, If, W2, G1, H4, Nf, fa.vertical, ["vertical"], r2, i_, n2, sh));
                          var a7 = ad ? ad.boxStartIndex : mu.collisionBoxArray.length, o7 = ad ? ad.boxEndIndex : mu.collisionBoxArray.length, s7 = Df ? Df.boxStartIndex : mu.collisionBoxArray.length, l7 = Df ? Df.boxEndIndex : mu.collisionBoxArray.length, c7 = kd ? kd.boxStartIndex : mu.collisionBoxArray.length, u7 = kd ? kd.boxEndIndex : mu.collisionBoxArray.length, d7 = Gd ? Gd.boxStartIndex : mu.collisionBoxArray.length, f7 = Gd ? Gd.boxEndIndex : mu.collisionBoxArray.length, Vh = -1, W4 = function(y3, Qy) {
                            return y3 && y3.circleDiameter ? Math.max(y3.circleDiameter, Qy) : Qy;
                          };
                          Vh = W4(ad, Vh), Vh = W4(Df, Vh), Vh = W4(kd, Vh);
                          var Ay = (Vh = W4(Gd, Vh)) > -1 ? 1 : 0;
                          Ay && (Vh *= n_ / Ws), mu.glyphOffsetArray.length >= Du.MAX_GLYPHS && Me("Too many glyphs being rendered in a tile. See https://github.com/mapbox/mapbox-gl-js/issues/2907"), G1.sortKey !== void 0 && mu.addToSortKeyRanges(mu.symbolInstances.length, G1.sortKey), mu.symbolInstances.emplaceBack(u1.x, u1.y, r2.right >= 0 ? r2.right : -1, r2.center >= 0 ? r2.center : -1, r2.left >= 0 ? r2.left : -1, r2.vertical || -1, r_, i_, yy, a7, o7, s7, l7, c7, u7, d7, f7, Sf, my, gy, lh, g3, Ay, 0, p3, a_, o_, Vh);
                        }(zt, pd, pf, pn, Cn, Nn, Ui, zt.layers[0], zt.collisionBoxArray, ln.index, ln.sourceLayerIndex, zt.index, fs, Es, Ps, Nr, fl, cl, ko, Na, ln, Wn, Kr, xi, lr);
                      };
                      if (Ks === "line") for (var Tu = 0, B1 = Dh(ln.geometry, 0, 0, $s, $s); Tu < B1.length; Tu += 1) for (var Hd = B1[Tu], nd = 0, j1 = mh(Hd, Ns, Xs, pn.vertical || Ta, Cn, Ra, Bs, zt.overscaling, $s); nd < j1.length; nd += 1) {
                        var Fd = j1[nd];
                        Ta && M4(zt, Ta.text, el, Fd) || bu(Hd, Fd);
                      }
                      else if (Ks === "line-center") for (var Cd = 0, L1 = ln.geometry; Cd < L1.length; Cd += 1) {
                        var m1 = L1[Cd];
                        if (m1.length > 1) {
                          var xd = Kd(m1, Xs, pn.vertical || Ta, Cn, Ra, Bs);
                          xd && bu(m1, xd);
                        }
                      }
                      else if (ln.type === "Polygon") for (var H1 = 0, rd = Xh(ln.geometry, 0); H1 < rd.length; H1 += 1) {
                        var Ld = rd[H1], zd = ef(Ld, 16);
                        bu(Ld[0], new Rs(zd.x, zd.y, 0));
                      }
                      else if (ln.type === "LineString") for (var Nd = 0, k1 = ln.geometry; Nd < k1.length; Nd += 1) {
                        var Tf = k1[Nd];
                        bu(Tf, new Rs(Tf[0].x, Tf[0].y, 0));
                      }
                      else if (ln.type === "Point") for (var c1 = 0, g1 = ln.geometry; c1 < g1.length; c1 += 1) for (var _1 = 0, v1 = g1[c1]; _1 < v1.length; _1 += 1) {
                        var Th = v1[_1];
                        bu([Th], new Rs(Th.x, Th.y, 0));
                      }
                    }(Ne, Lo, Rl, Ye, kt, Ir, Pl, rl, Zs, He, Sn);
                  }, wo = 0, ds = Ne.features; wo < ds.length; wo += 1) co();
                  fn && Ne.generateCollisionDebugBuffers();
                }, ee.perspective = function(Ne, je, ut, kt, Jt) {
                  var fn, Sn = 1 / Math.tan(je / 2);
                  return Ne[0] = Sn / ut, Ne[1] = 0, Ne[2] = 0, Ne[3] = 0, Ne[4] = 0, Ne[5] = Sn, Ne[6] = 0, Ne[7] = 0, Ne[8] = 0, Ne[9] = 0, Ne[11] = -1, Ne[12] = 0, Ne[13] = 0, Ne[15] = 0, Jt != null && Jt !== 1 / 0 ? (fn = 1 / (kt - Jt), Ne[10] = (Jt + kt) * fn, Ne[14] = 2 * Jt * kt * fn) : (Ne[10] = -1, Ne[14] = -2 * kt), Ne;
                }, ee.pick = function(Ne, je) {
                  for (var ut = {}, kt = 0; kt < je.length; kt++) {
                    var Jt = je[kt];
                    Jt in Ne && (ut[Jt] = Ne[Jt]);
                  }
                  return ut;
                }, ee.plugin = Uc, ee.polygonIntersectsPolygon = Bd, ee.postMapLoadEvent = hn, ee.postTurnstileEvent = Yt, ee.potpack = yi, ee.refProperties = ["type", "source", "source-layer", "minzoom", "maxzoom", "filter", "layout"], ee.register = cs, ee.registerForPluginStateChange = function(Ne) {
                  return Ne({ pluginStatus: Tl, pluginURL: sc }), Rc.on("pluginStateChange", Ne), Ne;
                }, ee.renderColorRamp = Xd, ee.rotate = function(Ne, je, ut) {
                  var kt = je[0], Jt = je[1], fn = je[2], Sn = je[3], Hn = Math.sin(ut), nr = Math.cos(ut);
                  return Ne[0] = kt * nr + fn * Hn, Ne[1] = Jt * nr + Sn * Hn, Ne[2] = kt * -Hn + fn * nr, Ne[3] = Jt * -Hn + Sn * nr, Ne;
                }, ee.rotateX = function(Ne, je, ut) {
                  var kt = Math.sin(ut), Jt = Math.cos(ut), fn = je[4], Sn = je[5], Hn = je[6], nr = je[7], kr = je[8], Ir = je[9], qr = je[10], ci = je[11];
                  return je !== Ne && (Ne[0] = je[0], Ne[1] = je[1], Ne[2] = je[2], Ne[3] = je[3], Ne[12] = je[12], Ne[13] = je[13], Ne[14] = je[14], Ne[15] = je[15]), Ne[4] = fn * Jt + kr * kt, Ne[5] = Sn * Jt + Ir * kt, Ne[6] = Hn * Jt + qr * kt, Ne[7] = nr * Jt + ci * kt, Ne[8] = kr * Jt - fn * kt, Ne[9] = Ir * Jt - Sn * kt, Ne[10] = qr * Jt - Hn * kt, Ne[11] = ci * Jt - nr * kt, Ne;
                }, ee.rotateZ = function(Ne, je, ut) {
                  var kt = Math.sin(ut), Jt = Math.cos(ut), fn = je[0], Sn = je[1], Hn = je[2], nr = je[3], kr = je[4], Ir = je[5], qr = je[6], ci = je[7];
                  return je !== Ne && (Ne[8] = je[8], Ne[9] = je[9], Ne[10] = je[10], Ne[11] = je[11], Ne[12] = je[12], Ne[13] = je[13], Ne[14] = je[14], Ne[15] = je[15]), Ne[0] = fn * Jt + kr * kt, Ne[1] = Sn * Jt + Ir * kt, Ne[2] = Hn * Jt + qr * kt, Ne[3] = nr * Jt + ci * kt, Ne[4] = kr * Jt - fn * kt, Ne[5] = Ir * Jt - Sn * kt, Ne[6] = qr * Jt - Hn * kt, Ne[7] = ci * Jt - nr * kt, Ne;
                }, ee.scale = function(Ne, je, ut) {
                  var kt = ut[0], Jt = ut[1], fn = ut[2];
                  return Ne[0] = je[0] * kt, Ne[1] = je[1] * kt, Ne[2] = je[2] * kt, Ne[3] = je[3] * kt, Ne[4] = je[4] * Jt, Ne[5] = je[5] * Jt, Ne[6] = je[6] * Jt, Ne[7] = je[7] * Jt, Ne[8] = je[8] * fn, Ne[9] = je[9] * fn, Ne[10] = je[10] * fn, Ne[11] = je[11] * fn, Ne[12] = je[12], Ne[13] = je[13], Ne[14] = je[14], Ne[15] = je[15], Ne;
                }, ee.scale$1 = function(Ne, je, ut) {
                  return Ne[0] = je[0] * ut, Ne[1] = je[1] * ut, Ne[2] = je[2] * ut, Ne[3] = je[3] * ut, Ne;
                }, ee.scale$2 = function(Ne, je, ut) {
                  return Ne[0] = je[0] * ut, Ne[1] = je[1] * ut, Ne[2] = je[2] * ut, Ne;
                }, ee.setCacheLimits = function(Ne, je) {
                  Ut = Ne, Wt = je;
                }, ee.setRTLTextPlugin = function(Ne, je, ut) {
                  if (ut === void 0 && (ut = !1), Tl === kc || Tl === fc || Tl === oc) throw new Error("setRTLTextPlugin cannot be called multiple times.");
                  sc = dt.resolveURL(Ne), Tl = kc, su = je, Vc(), ut || Ou();
                }, ee.sphericalToCartesian = function(Ne) {
                  var je = Ne[0], ut = Ne[1], kt = Ne[2];
                  return ut += 90, ut *= Math.PI / 180, kt *= Math.PI / 180, { x: je * Math.cos(ut) * Math.sin(kt), y: je * Math.sin(ut) * Math.sin(kt), z: je * Math.cos(kt) };
                }, ee.sqrLen = Yd, ee.styleSpec = Zt, ee.sub = $d, ee.symbolSize = Vf, ee.transformMat3 = function(Ne, je, ut) {
                  var kt = je[0], Jt = je[1], fn = je[2];
                  return Ne[0] = kt * ut[0] + Jt * ut[3] + fn * ut[6], Ne[1] = kt * ut[1] + Jt * ut[4] + fn * ut[7], Ne[2] = kt * ut[2] + Jt * ut[5] + fn * ut[8], Ne;
                }, ee.transformMat4 = Sd, ee.translate = function(Ne, je, ut) {
                  var kt, Jt, fn, Sn, Hn, nr, kr, Ir, qr, ci, ni, Ii, Gi = ut[0], xa = ut[1], ga = ut[2];
                  return je === Ne ? (Ne[12] = je[0] * Gi + je[4] * xa + je[8] * ga + je[12], Ne[13] = je[1] * Gi + je[5] * xa + je[9] * ga + je[13], Ne[14] = je[2] * Gi + je[6] * xa + je[10] * ga + je[14], Ne[15] = je[3] * Gi + je[7] * xa + je[11] * ga + je[15]) : (kt = je[0], Jt = je[1], fn = je[2], Sn = je[3], Hn = je[4], nr = je[5], kr = je[6], Ir = je[7], qr = je[8], ci = je[9], ni = je[10], Ii = je[11], Ne[0] = kt, Ne[1] = Jt, Ne[2] = fn, Ne[3] = Sn, Ne[4] = Hn, Ne[5] = nr, Ne[6] = kr, Ne[7] = Ir, Ne[8] = qr, Ne[9] = ci, Ne[10] = ni, Ne[11] = Ii, Ne[12] = kt * Gi + Hn * xa + qr * ga + je[12], Ne[13] = Jt * Gi + nr * xa + ci * ga + je[13], Ne[14] = fn * Gi + kr * xa + ni * ga + je[14], Ne[15] = Sn * Gi + Ir * xa + Ii * ga + je[15]), Ne;
                }, ee.triggerPluginCompletionEvent = hc, ee.uniqueId = be, ee.validateCustomStyleLayer = function(Ne) {
                  var je = [], ut = Ne.id;
                  return ut === void 0 && je.push({ message: "layers." + ut + ': missing required property "id"' }), Ne.render === void 0 && je.push({ message: "layers." + ut + ': missing required method "render"' }), Ne.renderingMode && Ne.renderingMode !== "2d" && Ne.renderingMode !== "3d" && je.push({ message: "layers." + ut + ': property "renderingMode" must be either "2d" or "3d"' }), je;
                }, ee.validateLight = wu, ee.validateStyle = Ec, ee.values = function(Ne) {
                  var je = [];
                  for (var ut in Ne) je.push(Ne[ut]);
                  return je;
                }, ee.vectorTile = En, ee.version = oe, ee.warnOnce = Me, ee.webpSupported = it, ee.window = de, ee.wrap = ye;
              }), ne(0, function(ee) {
                function ae(At) {
                  var Dt = typeof At;
                  if (Dt === "number" || Dt === "boolean" || Dt === "string" || At == null) return JSON.stringify(At);
                  if (Array.isArray(At)) {
                    for (var Ot = "[", un = 0, yn = At; un < yn.length; un += 1) Ot += ae(yn[un]) + ",";
                    return Ot + "]";
                  }
                  for (var _n = Object.keys(At).sort(), gn = "{", bn = 0; bn < _n.length; bn++) gn += JSON.stringify(_n[bn]) + ":" + ae(At[_n[bn]]) + ",";
                  return gn + "}";
                }
                function oe(At) {
                  for (var Dt = "", Ot = 0, un = ee.refProperties; Ot < un.length; Ot += 1) Dt += "/" + ae(At[un[Ot]]);
                  return Dt;
                }
                var se = function(At) {
                  this.keyCache = {}, At && this.replace(At);
                };
                se.prototype.replace = function(At) {
                  this._layerConfigs = {}, this._layers = {}, this.update(At, []);
                }, se.prototype.update = function(At, Dt) {
                  for (var Ot = this, un = 0, yn = At; un < yn.length; un += 1) {
                    var _n = yn[un];
                    this._layerConfigs[_n.id] = _n;
                    var gn = this._layers[_n.id] = ee.createStyleLayer(_n);
                    gn._featureFilter = ee.featureFilter(gn.filter), this.keyCache[_n.id] && delete this.keyCache[_n.id];
                  }
                  for (var bn = 0, an = Dt; bn < an.length; bn += 1) {
                    var Zt = an[bn];
                    delete this.keyCache[Zt], delete this._layerConfigs[Zt], delete this._layers[Zt];
                  }
                  this.familiesBySource = {};
                  for (var Mn = 0, Ln = function(Jn, Lr) {
                    for (var mr = {}, Ar = 0; Ar < Jn.length; Ar++) {
                      var Yr = Lr && Lr[Jn[Ar].id] || oe(Jn[Ar]);
                      Lr && (Lr[Jn[Ar].id] = Yr);
                      var jr = mr[Yr];
                      jr || (jr = mr[Yr] = []), jr.push(Jn[Ar]);
                    }
                    var Mr = [];
                    for (var Wr in mr) Mr.push(mr[Wr]);
                    return Mr;
                  }(ee.values(this._layerConfigs), this.keyCache); Mn < Ln.length; Mn += 1) {
                    var In = Ln[Mn].map(function(Jn) {
                      return Ot._layers[Jn.id];
                    }), kn = In[0];
                    if (kn.visibility !== "none") {
                      var er = kn.source || "", jn = this.familiesBySource[er];
                      jn || (jn = this.familiesBySource[er] = {});
                      var ur = kn.sourceLayer || "_geojsonTileLayer", Kn = jn[ur];
                      Kn || (Kn = jn[ur] = []), Kn.push(In);
                    }
                  }
                };
                var ce = function(At) {
                  var Dt = {}, Ot = [];
                  for (var un in At) {
                    var yn = At[un], _n = Dt[un] = {};
                    for (var gn in yn) {
                      var bn = yn[+gn];
                      if (bn && bn.bitmap.width !== 0 && bn.bitmap.height !== 0) {
                        var an = { x: 0, y: 0, w: bn.bitmap.width + 2, h: bn.bitmap.height + 2 };
                        Ot.push(an), _n[gn] = { rect: an, metrics: bn.metrics };
                      }
                    }
                  }
                  var Zt = ee.potpack(Ot), Mn = Zt.w, Ln = Zt.h, In = new ee.AlphaImage({ width: Mn || 1, height: Ln || 1 });
                  for (var kn in At) {
                    var er = At[kn];
                    for (var jn in er) {
                      var ur = er[+jn];
                      if (ur && ur.bitmap.width !== 0 && ur.bitmap.height !== 0) {
                        var Kn = Dt[kn][jn].rect;
                        ee.AlphaImage.copy(ur.bitmap, In, { x: 0, y: 0 }, { x: Kn.x + 1, y: Kn.y + 1 }, ur.bitmap);
                      }
                    }
                  }
                  this.image = In, this.positions = Dt;
                };
                ee.register("GlyphAtlas", ce);
                var le = function(At) {
                  this.tileID = new ee.OverscaledTileID(At.tileID.overscaledZ, At.tileID.wrap, At.tileID.canonical.z, At.tileID.canonical.x, At.tileID.canonical.y), this.uid = At.uid, this.zoom = At.zoom, this.pixelRatio = At.pixelRatio, this.tileSize = At.tileSize, this.source = At.source, this.overscaling = this.tileID.overscaleFactor(), this.showCollisionBoxes = At.showCollisionBoxes, this.collectResourceTiming = !!At.collectResourceTiming, this.returnDependencies = !!At.returnDependencies, this.promoteId = At.promoteId;
                };
                function ue(At, Dt, Ot) {
                  for (var un = new ee.EvaluationParameters(Dt), yn = 0, _n = At; yn < _n.length; yn += 1) _n[yn].recalculate(un, Ot);
                }
                function de(At, Dt) {
                  var Ot = ee.getArrayBuffer(At.request, function(un, yn, _n, gn) {
                    un ? Dt(un) : yn && Dt(null, { vectorTile: new ee.vectorTile.VectorTile(new ee.pbf(yn)), rawData: yn, cacheControl: _n, expires: gn });
                  });
                  return function() {
                    Ot.cancel(), Dt();
                  };
                }
                le.prototype.parse = function(At, Dt, Ot, un, yn) {
                  var _n = this;
                  this.status = "parsing", this.data = At, this.collisionBoxArray = new ee.CollisionBoxArray();
                  var gn = new ee.DictionaryCoder(Object.keys(At.layers).sort()), bn = new ee.FeatureIndex(this.tileID, this.promoteId);
                  bn.bucketLayerIDs = [];
                  var an, Zt, Mn, Ln, In = {}, kn = { featureIndex: bn, iconDependencies: {}, patternDependencies: {}, glyphDependencies: {}, availableImages: Ot }, er = Dt.familiesBySource[this.source];
                  for (var jn in er) {
                    var ur = At.layers[jn];
                    if (ur) {
                      ur.version === 1 && ee.warnOnce('Vector tile source "' + this.source + '" layer "' + jn + '" does not use vector tile spec v2 and therefore may have some rendering errors.');
                      for (var Kn = gn.encode(jn), Jn = [], Lr = 0; Lr < ur.length; Lr++) {
                        var mr = ur.feature(Lr), Ar = bn.getId(mr, jn);
                        Jn.push({ feature: mr, id: Ar, index: Lr, sourceLayerIndex: Kn });
                      }
                      for (var Yr = 0, jr = er[jn]; Yr < jr.length; Yr += 1) {
                        var Mr = jr[Yr], Wr = Mr[0];
                        Wr.minzoom && this.zoom < Math.floor(Wr.minzoom) || Wr.maxzoom && this.zoom >= Wr.maxzoom || Wr.visibility !== "none" && (ue(Mr, this.zoom, Ot), (In[Wr.id] = Wr.createBucket({ index: bn.bucketLayerIDs.length, layers: Mr, zoom: this.zoom, pixelRatio: this.pixelRatio, overscaling: this.overscaling, collisionBoxArray: this.collisionBoxArray, sourceLayerIndex: Kn, sourceID: this.source })).populate(Jn, kn, this.tileID.canonical), bn.bucketLayerIDs.push(Mr.map(function(ri) {
                          return ri.id;
                        })));
                      }
                    }
                  }
                  var Bi = ee.mapObject(kn.glyphDependencies, function(ri) {
                    return Object.keys(ri).map(Number);
                  });
                  Object.keys(Bi).length ? un.send("getGlyphs", { uid: this.uid, stacks: Bi }, function(ri, bi) {
                    an || (an = ri, Zt = bi, fi.call(_n));
                  }) : Zt = {};
                  var gi = Object.keys(kn.iconDependencies);
                  gi.length ? un.send("getImages", { icons: gi, source: this.source, tileID: this.tileID, type: "icons" }, function(ri, bi) {
                    an || (an = ri, Mn = bi, fi.call(_n));
                  }) : Mn = {};
                  var Zr = Object.keys(kn.patternDependencies);
                  function fi() {
                    if (an) return yn(an);
                    if (Zt && Mn && Ln) {
                      var ri = new ce(Zt), bi = new ee.ImageAtlas(Mn, Ln);
                      for (var Zi in In) {
                        var $i = In[Zi];
                        $i instanceof ee.SymbolBucket ? (ue($i.layers, this.zoom, Ot), ee.performSymbolLayout($i, Zt, ri.positions, Mn, bi.iconPositions, this.showCollisionBoxes, this.tileID.canonical)) : $i.hasPattern && ($i instanceof ee.LineBucket || $i instanceof ee.FillBucket || $i instanceof ee.FillExtrusionBucket) && (ue($i.layers, this.zoom, Ot), $i.addFeatures(kn, this.tileID.canonical, bi.patternPositions));
                      }
                      this.status = "done", yn(null, { buckets: ee.values(In).filter(function(Br) {
                        return !Br.isEmpty();
                      }), featureIndex: bn, collisionBoxArray: this.collisionBoxArray, glyphAtlasImage: ri.image, imageAtlas: bi, glyphMap: this.returnDependencies ? Zt : null, iconMap: this.returnDependencies ? Mn : null, glyphPositions: this.returnDependencies ? ri.positions : null });
                    }
                  }
                  Zr.length ? un.send("getImages", { icons: Zr, source: this.source, tileID: this.tileID, type: "patterns" }, function(ri, bi) {
                    an || (an = ri, Ln = bi, fi.call(_n));
                  }) : Ln = {}, fi.call(this);
                };
                var fe = function(At, Dt, Ot, un) {
                  this.actor = At, this.layerIndex = Dt, this.availableImages = Ot, this.loadVectorData = un || de, this.loading = {}, this.loaded = {};
                };
                fe.prototype.loadTile = function(At, Dt) {
                  var Ot = this, un = At.uid;
                  this.loading || (this.loading = {});
                  var yn = !!(At && At.request && At.request.collectResourceTiming) && new ee.RequestPerformance(At.request), _n = this.loading[un] = new le(At);
                  _n.abort = this.loadVectorData(At, function(gn, bn) {
                    if (delete Ot.loading[un], gn || !bn) return _n.status = "done", Ot.loaded[un] = _n, Dt(gn);
                    var an = bn.rawData, Zt = {};
                    bn.expires && (Zt.expires = bn.expires), bn.cacheControl && (Zt.cacheControl = bn.cacheControl);
                    var Mn = {};
                    if (yn) {
                      var Ln = yn.finish();
                      Ln && (Mn.resourceTiming = JSON.parse(JSON.stringify(Ln)));
                    }
                    _n.vectorTile = bn.vectorTile, _n.parse(bn.vectorTile, Ot.layerIndex, Ot.availableImages, Ot.actor, function(In, kn) {
                      if (In || !kn) return Dt(In);
                      Dt(null, ee.extend({ rawTileData: an.slice(0) }, kn, Zt, Mn));
                    }), Ot.loaded = Ot.loaded || {}, Ot.loaded[un] = _n;
                  });
                }, fe.prototype.reloadTile = function(At, Dt) {
                  var Ot = this, un = this.loaded, yn = At.uid, _n = this;
                  if (un && un[yn]) {
                    var gn = un[yn];
                    gn.showCollisionBoxes = At.showCollisionBoxes;
                    var bn = function(an, Zt) {
                      var Mn = gn.reloadCallback;
                      Mn && (delete gn.reloadCallback, gn.parse(gn.vectorTile, _n.layerIndex, Ot.availableImages, _n.actor, Mn)), Dt(an, Zt);
                    };
                    gn.status === "parsing" ? gn.reloadCallback = bn : gn.status === "done" && (gn.vectorTile ? gn.parse(gn.vectorTile, this.layerIndex, this.availableImages, this.actor, bn) : bn());
                  }
                }, fe.prototype.abortTile = function(At, Dt) {
                  var Ot = this.loading, un = At.uid;
                  Ot && Ot[un] && Ot[un].abort && (Ot[un].abort(), delete Ot[un]), Dt();
                }, fe.prototype.removeTile = function(At, Dt) {
                  var Ot = this.loaded, un = At.uid;
                  Ot && Ot[un] && delete Ot[un], Dt();
                };
                var he = ee.window.ImageBitmap, me = function() {
                  this.loaded = {};
                };
                me.prototype.loadTile = function(At, Dt) {
                  var Ot = At.uid, un = At.encoding, yn = At.rawImageData, _n = he && yn instanceof he ? this.getImageData(yn) : yn, gn = new ee.DEMData(Ot, _n, un);
                  this.loaded = this.loaded || {}, this.loaded[Ot] = gn, Dt(null, gn);
                }, me.prototype.getImageData = function(At) {
                  this.offscreenCanvas && this.offscreenCanvasContext || (this.offscreenCanvas = new OffscreenCanvas(At.width, At.height), this.offscreenCanvasContext = this.offscreenCanvas.getContext("2d")), this.offscreenCanvas.width = At.width, this.offscreenCanvas.height = At.height, this.offscreenCanvasContext.drawImage(At, 0, 0, At.width, At.height);
                  var Dt = this.offscreenCanvasContext.getImageData(-1, -1, At.width + 2, At.height + 2);
                  return this.offscreenCanvasContext.clearRect(0, 0, this.offscreenCanvas.width, this.offscreenCanvas.height), new ee.RGBAImage({ width: Dt.width, height: Dt.height }, Dt.data);
                }, me.prototype.removeTile = function(At) {
                  var Dt = this.loaded, Ot = At.uid;
                  Dt && Dt[Ot] && delete Dt[Ot];
                };
                var pe = function At(Dt, Ot) {
                  var un, yn = Dt && Dt.type;
                  if (yn === "FeatureCollection") for (un = 0; un < Dt.features.length; un++) At(Dt.features[un], Ot);
                  else if (yn === "GeometryCollection") for (un = 0; un < Dt.geometries.length; un++) At(Dt.geometries[un], Ot);
                  else if (yn === "Feature") At(Dt.geometry, Ot);
                  else if (yn === "Polygon") ye(Dt.coordinates, Ot);
                  else if (yn === "MultiPolygon") for (un = 0; un < Dt.coordinates.length; un++) ye(Dt.coordinates[un], Ot);
                  return Dt;
                };
                function ye(At, Dt) {
                  if (At.length !== 0) {
                    ge(At[0], Dt);
                    for (var Ot = 1; Ot < At.length; Ot++) ge(At[Ot], !1);
                  }
                }
                function ge(At, Dt) {
                  for (var Ot = 0, un = 0, yn = At.length, _n = yn - 1; un < yn; _n = un++) Ot += (At[un][0] - At[_n][0]) * (At[_n][1] + At[un][1]);
                  Ot >= 0 != !!Dt && At.reverse();
                }
                var ve = ee.vectorTile.VectorTileFeature.prototype.toGeoJSON, be = function(At) {
                  this._feature = At, this.extent = ee.EXTENT, this.type = At.type, this.properties = At.tags, "id" in At && !isNaN(At.id) && (this.id = parseInt(At.id, 10));
                };
                be.prototype.loadGeometry = function() {
                  if (this._feature.type === 1) {
                    for (var At = [], Dt = 0, Ot = this._feature.geometry; Dt < Ot.length; Dt += 1) {
                      var un = Ot[Dt];
                      At.push([new ee.Point$1(un[0], un[1])]);
                    }
                    return At;
                  }
                  for (var yn = [], _n = 0, gn = this._feature.geometry; _n < gn.length; _n += 1) {
                    for (var bn = [], an = 0, Zt = gn[_n]; an < Zt.length; an += 1) {
                      var Mn = Zt[an];
                      bn.push(new ee.Point$1(Mn[0], Mn[1]));
                    }
                    yn.push(bn);
                  }
                  return yn;
                }, be.prototype.toGeoJSON = function(At, Dt, Ot) {
                  return ve.call(this, At, Dt, Ot);
                };
                var Ce = function(At) {
                  this.layers = { _geojsonTileLayer: this }, this.name = "_geojsonTileLayer", this.extent = ee.EXTENT, this.length = At.length, this._features = At;
                };
                Ce.prototype.feature = function(At) {
                  return new be(this._features[At]);
                };
                var Se = ee.vectorTile.VectorTileFeature, we = Ee;
                function Ee(At, Dt) {
                  this.options = Dt || {}, this.features = At, this.length = At.length;
                }
                function xe(At, Dt) {
                  this.id = typeof At.id == "number" ? At.id : void 0, this.type = At.type, this.rawGeometry = At.type === 1 ? [At.geometry] : At.geometry, this.properties = At.tags, this.extent = Dt || 4096;
                }
                Ee.prototype.feature = function(At) {
                  return new xe(this.features[At], this.options.extent);
                }, xe.prototype.loadGeometry = function() {
                  var At = this.rawGeometry;
                  this.geometry = [];
                  for (var Dt = 0; Dt < At.length; Dt++) {
                    for (var Ot = At[Dt], un = [], yn = 0; yn < Ot.length; yn++) un.push(new ee.Point$1(Ot[yn][0], Ot[yn][1]));
                    this.geometry.push(un);
                  }
                  return this.geometry;
                }, xe.prototype.bbox = function() {
                  this.geometry || this.loadGeometry();
                  for (var At = this.geometry, Dt = 1 / 0, Ot = -1 / 0, un = 1 / 0, yn = -1 / 0, _n = 0; _n < At.length; _n++) for (var gn = At[_n], bn = 0; bn < gn.length; bn++) {
                    var an = gn[bn];
                    Dt = Math.min(Dt, an.x), Ot = Math.max(Ot, an.x), un = Math.min(un, an.y), yn = Math.max(yn, an.y);
                  }
                  return [Dt, un, Ot, yn];
                }, xe.prototype.toGeoJSON = Se.prototype.toGeoJSON;
                var Le = Re, Qe = Re, ke = function(At, Dt) {
                  Dt = Dt || {};
                  var Ot = {};
                  for (var un in At) Ot[un] = new we(At[un].features, Dt), Ot[un].name = un, Ot[un].version = Dt.version, Ot[un].extent = Dt.extent;
                  return Re({ layers: Ot });
                }, Me = we;
                function Re(At) {
                  var Dt = new ee.pbf();
                  return function(Ot, un) {
                    for (var yn in Ot.layers) un.writeMessage(3, De, Ot.layers[yn]);
                  }(At, Dt), Dt.finish();
                }
                function De(At, Dt) {
                  var Ot;
                  Dt.writeVarintField(15, At.version || 1), Dt.writeStringField(1, At.name || ""), Dt.writeVarintField(5, At.extent || 4096);
                  var un = { keys: [], values: [], keycache: {}, valuecache: {} };
                  for (Ot = 0; Ot < At.length; Ot++) un.feature = At.feature(Ot), Dt.writeMessage(2, Pe, un);
                  var yn = un.keys;
                  for (Ot = 0; Ot < yn.length; Ot++) Dt.writeStringField(3, yn[Ot]);
                  var _n = un.values;
                  for (Ot = 0; Ot < _n.length; Ot++) Dt.writeMessage(4, Ze, _n[Ot]);
                }
                function Pe(At, Dt) {
                  var Ot = At.feature;
                  Ot.id !== void 0 && Dt.writeVarintField(1, Ot.id), Dt.writeMessage(2, $e, At), Dt.writeVarintField(3, Ot.type), Dt.writeMessage(4, Ve, Ot);
                }
                function $e(At, Dt) {
                  var Ot = At.feature, un = At.keys, yn = At.values, _n = At.keycache, gn = At.valuecache;
                  for (var bn in Ot.properties) {
                    var an = _n[bn];
                    an === void 0 && (un.push(bn), an = un.length - 1, _n[bn] = an), Dt.writeVarint(an);
                    var Zt = Ot.properties[bn], Mn = typeof Zt;
                    Mn !== "string" && Mn !== "boolean" && Mn !== "number" && (Zt = JSON.stringify(Zt));
                    var Ln = Mn + ":" + Zt, In = gn[Ln];
                    In === void 0 && (yn.push(Zt), In = yn.length - 1, gn[Ln] = In), Dt.writeVarint(In);
                  }
                }
                function qe(At, Dt) {
                  return (Dt << 3) + (7 & At);
                }
                function Xe(At) {
                  return At << 1 ^ At >> 31;
                }
                function Ve(At, Dt) {
                  for (var Ot = At.loadGeometry(), un = At.type, yn = 0, _n = 0, gn = Ot.length, bn = 0; bn < gn; bn++) {
                    var an = Ot[bn], Zt = 1;
                    un === 1 && (Zt = an.length), Dt.writeVarint(qe(1, Zt));
                    for (var Mn = un === 3 ? an.length - 1 : an.length, Ln = 0; Ln < Mn; Ln++) {
                      Ln === 1 && un !== 1 && Dt.writeVarint(qe(2, Mn - 1));
                      var In = an[Ln].x - yn, kn = an[Ln].y - _n;
                      Dt.writeVarint(Xe(In)), Dt.writeVarint(Xe(kn)), yn += In, _n += kn;
                    }
                    un === 3 && Dt.writeVarint(qe(7, 1));
                  }
                }
                function Ze(At, Dt) {
                  var Ot = typeof At;
                  Ot === "string" ? Dt.writeStringField(1, At) : Ot === "boolean" ? Dt.writeBooleanField(7, At) : Ot === "number" && (At % 1 != 0 ? Dt.writeDoubleField(3, At) : At < 0 ? Dt.writeSVarintField(6, At) : Dt.writeVarintField(5, At));
                }
                function Ge(At, Dt, Ot, un, yn, _n) {
                  if (!(yn - un <= Ot)) {
                    var gn = un + yn >> 1;
                    ze(At, Dt, gn, un, yn, _n % 2), Ge(At, Dt, Ot, un, gn - 1, _n + 1), Ge(At, Dt, Ot, gn + 1, yn, _n + 1);
                  }
                }
                function ze(At, Dt, Ot, un, yn, _n) {
                  for (; yn > un; ) {
                    if (yn - un > 600) {
                      var gn = yn - un + 1, bn = Ot - un + 1, an = Math.log(gn), Zt = 0.5 * Math.exp(2 * an / 3), Mn = 0.5 * Math.sqrt(an * Zt * (gn - Zt) / gn) * (bn - gn / 2 < 0 ? -1 : 1);
                      ze(At, Dt, Ot, Math.max(un, Math.floor(Ot - bn * Zt / gn + Mn)), Math.min(yn, Math.floor(Ot + (gn - bn) * Zt / gn + Mn)), _n);
                    }
                    var Ln = Dt[2 * Ot + _n], In = un, kn = yn;
                    for (We(At, Dt, un, Ot), Dt[2 * yn + _n] > Ln && We(At, Dt, un, yn); In < kn; ) {
                      for (We(At, Dt, In, kn), In++, kn--; Dt[2 * In + _n] < Ln; ) In++;
                      for (; Dt[2 * kn + _n] > Ln; ) kn--;
                    }
                    Dt[2 * un + _n] === Ln ? We(At, Dt, un, kn) : We(At, Dt, ++kn, yn), kn <= Ot && (un = kn + 1), Ot <= kn && (yn = kn - 1);
                  }
                }
                function We(At, Dt, Ot, un) {
                  at(At, Ot, un), at(Dt, 2 * Ot, 2 * un), at(Dt, 2 * Ot + 1, 2 * un + 1);
                }
                function at(At, Dt, Ot) {
                  var un = At[Dt];
                  At[Dt] = At[Ot], At[Ot] = un;
                }
                function ct(At, Dt, Ot, un) {
                  var yn = At - Ot, _n = Dt - un;
                  return yn * yn + _n * _n;
                }
                Le.fromVectorTileJs = Qe, Le.fromGeojsonVt = ke, Le.GeoJSONWrapper = Me;
                var et = function(At) {
                  return At[0];
                }, dt = function(At) {
                  return At[1];
                }, Je = function(At, Dt, Ot, un, yn) {
                  Dt === void 0 && (Dt = et), Ot === void 0 && (Ot = dt), un === void 0 && (un = 64), yn === void 0 && (yn = Float64Array), this.nodeSize = un, this.points = At;
                  for (var _n = At.length < 65536 ? Uint16Array : Uint32Array, gn = this.ids = new _n(At.length), bn = this.coords = new yn(2 * At.length), an = 0; an < At.length; an++) gn[an] = an, bn[2 * an] = Dt(At[an]), bn[2 * an + 1] = Ot(At[an]);
                  Ge(gn, bn, un, 0, gn.length - 1, 0);
                };
                Je.prototype.range = function(At, Dt, Ot, un) {
                  return function(yn, _n, gn, bn, an, Zt, Mn) {
                    for (var Ln, In, kn = [0, yn.length - 1, 0], er = []; kn.length; ) {
                      var jn = kn.pop(), ur = kn.pop(), Kn = kn.pop();
                      if (ur - Kn <= Mn) for (var Jn = Kn; Jn <= ur; Jn++) Ln = _n[2 * Jn], In = _n[2 * Jn + 1], Ln >= gn && Ln <= an && In >= bn && In <= Zt && er.push(yn[Jn]);
                      else {
                        var Lr = Math.floor((Kn + ur) / 2);
                        Ln = _n[2 * Lr], In = _n[2 * Lr + 1], Ln >= gn && Ln <= an && In >= bn && In <= Zt && er.push(yn[Lr]);
                        var mr = (jn + 1) % 2;
                        (jn === 0 ? gn <= Ln : bn <= In) && (kn.push(Kn), kn.push(Lr - 1), kn.push(mr)), (jn === 0 ? an >= Ln : Zt >= In) && (kn.push(Lr + 1), kn.push(ur), kn.push(mr));
                      }
                    }
                    return er;
                  }(this.ids, this.coords, At, Dt, Ot, un, this.nodeSize);
                }, Je.prototype.within = function(At, Dt, Ot) {
                  return function(un, yn, _n, gn, bn, an) {
                    for (var Zt = [0, un.length - 1, 0], Mn = [], Ln = bn * bn; Zt.length; ) {
                      var In = Zt.pop(), kn = Zt.pop(), er = Zt.pop();
                      if (kn - er <= an) for (var jn = er; jn <= kn; jn++) ct(yn[2 * jn], yn[2 * jn + 1], _n, gn) <= Ln && Mn.push(un[jn]);
                      else {
                        var ur = Math.floor((er + kn) / 2), Kn = yn[2 * ur], Jn = yn[2 * ur + 1];
                        ct(Kn, Jn, _n, gn) <= Ln && Mn.push(un[ur]);
                        var Lr = (In + 1) % 2;
                        (In === 0 ? _n - bn <= Kn : gn - bn <= Jn) && (Zt.push(er), Zt.push(ur - 1), Zt.push(Lr)), (In === 0 ? _n + bn >= Kn : gn + bn >= Jn) && (Zt.push(ur + 1), Zt.push(kn), Zt.push(Lr));
                      }
                    }
                    return Mn;
                  }(this.ids, this.coords, At, Dt, Ot, this.nodeSize);
                };
                var it = { minZoom: 0, maxZoom: 16, minPoints: 2, radius: 40, extent: 512, nodeSize: 64, log: !1, generateId: !1, reduce: null, map: function(At) {
                  return At;
                } }, st = function(At) {
                  this.options = vt(Object.create(it), At), this.trees = new Array(this.options.maxZoom + 1);
                };
                function gt(At, Dt, Ot, un, yn) {
                  return { x: At, y: Dt, zoom: 1 / 0, id: Ot, parentId: -1, numPoints: un, properties: yn };
                }
                function bt(At, Dt) {
                  var Ot = At.geometry.coordinates, un = Ot[0], yn = Ot[1];
                  return { x: Lt(un), y: Rt(yn), zoom: 1 / 0, index: Dt, parentId: -1 };
                }
                function yt(At) {
                  return { type: "Feature", id: At.id, properties: St(At), geometry: { type: "Point", coordinates: [(un = At.x, 360 * (un - 0.5)), (Dt = At.y, Ot = (180 - 360 * Dt) * Math.PI / 180, 360 * Math.atan(Math.exp(Ot)) / Math.PI - 90)] } };
                  var Dt, Ot, un;
                }
                function St(At) {
                  var Dt = At.numPoints, Ot = Dt >= 1e4 ? Math.round(Dt / 1e3) + "k" : Dt >= 1e3 ? Math.round(Dt / 100) / 10 + "k" : Dt;
                  return vt(vt({}, At.properties), { cluster: !0, cluster_id: At.id, point_count: Dt, point_count_abbreviated: Ot });
                }
                function Lt(At) {
                  return At / 360 + 0.5;
                }
                function Rt(At) {
                  var Dt = Math.sin(At * Math.PI / 180), Ot = 0.5 - 0.25 * Math.log((1 + Dt) / (1 - Dt)) / Math.PI;
                  return Ot < 0 ? 0 : Ot > 1 ? 1 : Ot;
                }
                function vt(At, Dt) {
                  for (var Ot in Dt) At[Ot] = Dt[Ot];
                  return At;
                }
                function $t(At) {
                  return At.x;
                }
                function Pt(At) {
                  return At.y;
                }
                function Xt(At, Dt, Ot, un) {
                  for (var yn, _n = un, gn = Ot - Dt >> 1, bn = Ot - Dt, an = At[Dt], Zt = At[Dt + 1], Mn = At[Ot], Ln = At[Ot + 1], In = Dt + 3; In < Ot; In += 3) {
                    var kn = Nt(At[In], At[In + 1], an, Zt, Mn, Ln);
                    if (kn > _n) yn = In, _n = kn;
                    else if (kn === _n) {
                      var er = Math.abs(In - gn);
                      er < bn && (yn = In, bn = er);
                    }
                  }
                  _n > un && (yn - Dt > 3 && Xt(At, Dt, yn, un), At[yn + 2] = _n, Ot - yn > 3 && Xt(At, yn, Ot, un));
                }
                function Nt(At, Dt, Ot, un, yn, _n) {
                  var gn = yn - Ot, bn = _n - un;
                  if (gn !== 0 || bn !== 0) {
                    var an = ((At - Ot) * gn + (Dt - un) * bn) / (gn * gn + bn * bn);
                    an > 1 ? (Ot = yn, un = _n) : an > 0 && (Ot += gn * an, un += bn * an);
                  }
                  return (gn = At - Ot) * gn + (bn = Dt - un) * bn;
                }
                function Gt(At, Dt, Ot, un) {
                  var yn = { id: At === void 0 ? null : At, type: Dt, geometry: Ot, tags: un, minX: 1 / 0, minY: 1 / 0, maxX: -1 / 0, maxY: -1 / 0 };
                  return function(_n) {
                    var gn = _n.geometry, bn = _n.type;
                    if (bn === "Point" || bn === "MultiPoint" || bn === "LineString") Ft(_n, gn);
                    else if (bn === "Polygon" || bn === "MultiLineString") for (var an = 0; an < gn.length; an++) Ft(_n, gn[an]);
                    else if (bn === "MultiPolygon") for (an = 0; an < gn.length; an++) for (var Zt = 0; Zt < gn[an].length; Zt++) Ft(_n, gn[an][Zt]);
                  }(yn), yn;
                }
                function Ft(At, Dt) {
                  for (var Ot = 0; Ot < Dt.length; Ot += 3) At.minX = Math.min(At.minX, Dt[Ot]), At.minY = Math.min(At.minY, Dt[Ot + 1]), At.maxX = Math.max(At.maxX, Dt[Ot]), At.maxY = Math.max(At.maxY, Dt[Ot + 1]);
                }
                function Tt(At, Dt, Ot, un) {
                  if (Dt.geometry) {
                    var yn = Dt.geometry.coordinates, _n = Dt.geometry.type, gn = Math.pow(Ot.tolerance / ((1 << Ot.maxZoom) * Ot.extent), 2), bn = [], an = Dt.id;
                    if (Ot.promoteId ? an = Dt.properties[Ot.promoteId] : Ot.generateId && (an = un || 0), _n === "Point") It(yn, bn);
                    else if (_n === "MultiPoint") for (var Zt = 0; Zt < yn.length; Zt++) It(yn[Zt], bn);
                    else if (_n === "LineString") Ct(yn, bn, gn, !1);
                    else if (_n === "MultiLineString") {
                      if (Ot.lineMetrics) {
                        for (Zt = 0; Zt < yn.length; Zt++) bn = [], Ct(yn[Zt], bn, gn, !1), At.push(Gt(an, "LineString", bn, Dt.properties));
                        return;
                      }
                      Kt(yn, bn, gn, !1);
                    } else if (_n === "Polygon") Kt(yn, bn, gn, !0);
                    else {
                      if (_n !== "MultiPolygon") {
                        if (_n === "GeometryCollection") {
                          for (Zt = 0; Zt < Dt.geometry.geometries.length; Zt++) Tt(At, { id: an, geometry: Dt.geometry.geometries[Zt], properties: Dt.properties }, Ot, un);
                          return;
                        }
                        throw new Error("Input data is not a valid GeoJSON object.");
                      }
                      for (Zt = 0; Zt < yn.length; Zt++) {
                        var Mn = [];
                        Kt(yn[Zt], Mn, gn, !0), bn.push(Mn);
                      }
                    }
                    At.push(Gt(an, _n, bn, Dt.properties));
                  }
                }
                function It(At, Dt) {
                  Dt.push(Vt(At[0])), Dt.push(Yt(At[1])), Dt.push(0);
                }
                function Ct(At, Dt, Ot, un) {
                  for (var yn, _n, gn = 0, bn = 0; bn < At.length; bn++) {
                    var an = Vt(At[bn][0]), Zt = Yt(At[bn][1]);
                    Dt.push(an), Dt.push(Zt), Dt.push(0), bn > 0 && (gn += un ? (yn * Zt - an * _n) / 2 : Math.sqrt(Math.pow(an - yn, 2) + Math.pow(Zt - _n, 2))), yn = an, _n = Zt;
                  }
                  var Mn = Dt.length - 3;
                  Dt[2] = 1, Xt(Dt, 0, Mn, Ot), Dt[Mn + 2] = 1, Dt.size = Math.abs(gn), Dt.start = 0, Dt.end = Dt.size;
                }
                function Kt(At, Dt, Ot, un) {
                  for (var yn = 0; yn < At.length; yn++) {
                    var _n = [];
                    Ct(At[yn], _n, Ot, un), Dt.push(_n);
                  }
                }
                function Vt(At) {
                  return At / 360 + 0.5;
                }
                function Yt(At) {
                  var Dt = Math.sin(At * Math.PI / 180), Ot = 0.5 - 0.25 * Math.log((1 + Dt) / (1 - Dt)) / Math.PI;
                  return Ot < 0 ? 0 : Ot > 1 ? 1 : Ot;
                }
                function sn(At, Dt, Ot, un, yn, _n, gn, bn) {
                  if (un /= Dt, _n >= (Ot /= Dt) && gn < un) return At;
                  if (gn < Ot || _n >= un) return null;
                  for (var an = [], Zt = 0; Zt < At.length; Zt++) {
                    var Mn = At[Zt], Ln = Mn.geometry, In = Mn.type, kn = yn === 0 ? Mn.minX : Mn.minY, er = yn === 0 ? Mn.maxX : Mn.maxY;
                    if (kn >= Ot && er < un) an.push(Mn);
                    else if (!(er < Ot || kn >= un)) {
                      var jn = [];
                      if (In === "Point" || In === "MultiPoint") hn(Ln, jn, Ot, un, yn);
                      else if (In === "LineString") Qt(Ln, jn, Ot, un, yn, !1, bn.lineMetrics);
                      else if (In === "MultiLineString") Wt(Ln, jn, Ot, un, yn, !1);
                      else if (In === "Polygon") Wt(Ln, jn, Ot, un, yn, !0);
                      else if (In === "MultiPolygon") for (var ur = 0; ur < Ln.length; ur++) {
                        var Kn = [];
                        Wt(Ln[ur], Kn, Ot, un, yn, !0), Kn.length && jn.push(Kn);
                      }
                      if (jn.length) {
                        if (bn.lineMetrics && In === "LineString") {
                          for (ur = 0; ur < jn.length; ur++) an.push(Gt(Mn.id, In, jn[ur], Mn.tags));
                          continue;
                        }
                        In !== "LineString" && In !== "MultiLineString" || (jn.length === 1 ? (In = "LineString", jn = jn[0]) : In = "MultiLineString"), In !== "Point" && In !== "MultiPoint" || (In = jn.length === 3 ? "Point" : "MultiPoint"), an.push(Gt(Mn.id, In, jn, Mn.tags));
                      }
                    }
                  }
                  return an.length ? an : null;
                }
                function hn(At, Dt, Ot, un, yn) {
                  for (var _n = 0; _n < At.length; _n += 3) {
                    var gn = At[_n + yn];
                    gn >= Ot && gn <= un && (Dt.push(At[_n]), Dt.push(At[_n + 1]), Dt.push(At[_n + 2]));
                  }
                }
                function Qt(At, Dt, Ot, un, yn, _n, gn) {
                  for (var bn, an, Zt = Ut(At), Mn = yn === 0 ? vn : dn, Ln = At.start, In = 0; In < At.length - 3; In += 3) {
                    var kn = At[In], er = At[In + 1], jn = At[In + 2], ur = At[In + 3], Kn = At[In + 4], Jn = yn === 0 ? kn : er, Lr = yn === 0 ? ur : Kn, mr = !1;
                    gn && (bn = Math.sqrt(Math.pow(kn - ur, 2) + Math.pow(er - Kn, 2))), Jn < Ot ? Lr > Ot && (an = Mn(Zt, kn, er, ur, Kn, Ot), gn && (Zt.start = Ln + bn * an)) : Jn > un ? Lr < un && (an = Mn(Zt, kn, er, ur, Kn, un), gn && (Zt.start = Ln + bn * an)) : rn(Zt, kn, er, jn), Lr < Ot && Jn >= Ot && (an = Mn(Zt, kn, er, ur, Kn, Ot), mr = !0), Lr > un && Jn <= un && (an = Mn(Zt, kn, er, ur, Kn, un), mr = !0), !_n && mr && (gn && (Zt.end = Ln + bn * an), Dt.push(Zt), Zt = Ut(At)), gn && (Ln += bn);
                  }
                  var Ar = At.length - 3;
                  kn = At[Ar], er = At[Ar + 1], jn = At[Ar + 2], (Jn = yn === 0 ? kn : er) >= Ot && Jn <= un && rn(Zt, kn, er, jn), Ar = Zt.length - 3, _n && Ar >= 3 && (Zt[Ar] !== Zt[0] || Zt[Ar + 1] !== Zt[1]) && rn(Zt, Zt[0], Zt[1], Zt[2]), Zt.length && Dt.push(Zt);
                }
                function Ut(At) {
                  var Dt = [];
                  return Dt.size = At.size, Dt.start = At.start, Dt.end = At.end, Dt;
                }
                function Wt(At, Dt, Ot, un, yn, _n) {
                  for (var gn = 0; gn < At.length; gn++) Qt(At[gn], Dt, Ot, un, yn, _n, !1);
                }
                function rn(At, Dt, Ot, un) {
                  At.push(Dt), At.push(Ot), At.push(un);
                }
                function vn(At, Dt, Ot, un, yn, _n) {
                  var gn = (_n - Dt) / (un - Dt);
                  return At.push(_n), At.push(Ot + (yn - Ot) * gn), At.push(1), gn;
                }
                function dn(At, Dt, Ot, un, yn, _n) {
                  var gn = (_n - Ot) / (yn - Ot);
                  return At.push(Dt + (un - Dt) * gn), At.push(_n), At.push(1), gn;
                }
                function Ht(At, Dt) {
                  for (var Ot = [], un = 0; un < At.length; un++) {
                    var yn, _n = At[un], gn = _n.type;
                    if (gn === "Point" || gn === "MultiPoint" || gn === "LineString") yn = jt(_n.geometry, Dt);
                    else if (gn === "MultiLineString" || gn === "Polygon") {
                      yn = [];
                      for (var bn = 0; bn < _n.geometry.length; bn++) yn.push(jt(_n.geometry[bn], Dt));
                    } else if (gn === "MultiPolygon") for (yn = [], bn = 0; bn < _n.geometry.length; bn++) {
                      for (var an = [], Zt = 0; Zt < _n.geometry[bn].length; Zt++) an.push(jt(_n.geometry[bn][Zt], Dt));
                      yn.push(an);
                    }
                    Ot.push(Gt(_n.id, gn, yn, _n.tags));
                  }
                  return Ot;
                }
                function jt(At, Dt) {
                  var Ot = [];
                  Ot.size = At.size, At.start !== void 0 && (Ot.start = At.start, Ot.end = At.end);
                  for (var un = 0; un < At.length; un += 3) Ot.push(At[un] + Dt, At[un + 1], At[un + 2]);
                  return Ot;
                }
                function cn(At, Dt) {
                  if (At.transformed) return At;
                  var Ot, un, yn, _n = 1 << At.z, gn = At.x, bn = At.y;
                  for (Ot = 0; Ot < At.features.length; Ot++) {
                    var an = At.features[Ot], Zt = an.geometry, Mn = an.type;
                    if (an.geometry = [], Mn === 1) for (un = 0; un < Zt.length; un += 2) an.geometry.push(wn(Zt[un], Zt[un + 1], Dt, _n, gn, bn));
                    else for (un = 0; un < Zt.length; un++) {
                      var Ln = [];
                      for (yn = 0; yn < Zt[un].length; yn += 2) Ln.push(wn(Zt[un][yn], Zt[un][yn + 1], Dt, _n, gn, bn));
                      an.geometry.push(Ln);
                    }
                  }
                  return At.transformed = !0, At;
                }
                function wn(At, Dt, Ot, un, yn, _n) {
                  return [Math.round(Ot * (At * un - yn)), Math.round(Ot * (Dt * un - _n))];
                }
                function Rn(At, Dt, Ot, un, yn) {
                  for (var _n = Dt === yn.maxZoom ? 0 : yn.tolerance / ((1 << Dt) * yn.extent), gn = { features: [], numPoints: 0, numSimplified: 0, numFeatures: 0, source: null, x: Ot, y: un, z: Dt, transformed: !1, minX: 2, minY: 1, maxX: -1, maxY: 0 }, bn = 0; bn < At.length; bn++) {
                    gn.numFeatures++, Pn(gn, At[bn], _n, yn);
                    var an = At[bn].minX, Zt = At[bn].minY, Mn = At[bn].maxX, Ln = At[bn].maxY;
                    an < gn.minX && (gn.minX = an), Zt < gn.minY && (gn.minY = Zt), Mn > gn.maxX && (gn.maxX = Mn), Ln > gn.maxY && (gn.maxY = Ln);
                  }
                  return gn;
                }
                function Pn(At, Dt, Ot, un) {
                  var yn = Dt.geometry, _n = Dt.type, gn = [];
                  if (_n === "Point" || _n === "MultiPoint") for (var bn = 0; bn < yn.length; bn += 3) gn.push(yn[bn]), gn.push(yn[bn + 1]), At.numPoints++, At.numSimplified++;
                  else if (_n === "LineString") Ie(gn, yn, At, Ot, !1, !1);
                  else if (_n === "MultiLineString" || _n === "Polygon") for (bn = 0; bn < yn.length; bn++) Ie(gn, yn[bn], At, Ot, _n === "Polygon", bn === 0);
                  else if (_n === "MultiPolygon") for (var an = 0; an < yn.length; an++) {
                    var Zt = yn[an];
                    for (bn = 0; bn < Zt.length; bn++) Ie(gn, Zt[bn], At, Ot, !0, bn === 0);
                  }
                  if (gn.length) {
                    var Mn = Dt.tags || null;
                    if (_n === "LineString" && un.lineMetrics) {
                      for (var Ln in Mn = {}, Dt.tags) Mn[Ln] = Dt.tags[Ln];
                      Mn.mapbox_clip_start = yn.start / yn.size, Mn.mapbox_clip_end = yn.end / yn.size;
                    }
                    var In = { geometry: gn, type: _n === "Polygon" || _n === "MultiPolygon" ? 3 : _n === "LineString" || _n === "MultiLineString" ? 2 : 1, tags: Mn };
                    Dt.id !== null && (In.id = Dt.id), At.features.push(In);
                  }
                }
                function Ie(At, Dt, Ot, un, yn, _n) {
                  var gn = un * un;
                  if (un > 0 && Dt.size < (yn ? gn : un)) Ot.numPoints += Dt.length / 3;
                  else {
                    for (var bn = [], an = 0; an < Dt.length; an += 3) (un === 0 || Dt[an + 2] > gn) && (Ot.numSimplified++, bn.push(Dt[an]), bn.push(Dt[an + 1])), Ot.numPoints++;
                    yn && function(Zt, Mn) {
                      for (var Ln = 0, In = 0, kn = Zt.length, er = kn - 2; In < kn; er = In, In += 2) Ln += (Zt[In] - Zt[er]) * (Zt[In + 1] + Zt[er + 1]);
                      if (Ln > 0 === Mn) for (In = 0, kn = Zt.length; In < kn / 2; In += 2) {
                        var jn = Zt[In], ur = Zt[In + 1];
                        Zt[In] = Zt[kn - 2 - In], Zt[In + 1] = Zt[kn - 1 - In], Zt[kn - 2 - In] = jn, Zt[kn - 1 - In] = ur;
                      }
                    }(bn, _n), At.push(bn);
                  }
                }
                function rt(At, Dt) {
                  var Ot = (Dt = this.options = function(yn, _n) {
                    for (var gn in _n) yn[gn] = _n[gn];
                    return yn;
                  }(Object.create(this.options), Dt)).debug;
                  if (Ot && console.time("preprocess data"), Dt.maxZoom < 0 || Dt.maxZoom > 24) throw new Error("maxZoom should be in the 0-24 range");
                  if (Dt.promoteId && Dt.generateId) throw new Error("promoteId and generateId cannot be used together.");
                  var un = function(yn, _n) {
                    var gn = [];
                    if (yn.type === "FeatureCollection") for (var bn = 0; bn < yn.features.length; bn++) Tt(gn, yn.features[bn], _n, bn);
                    else yn.type === "Feature" ? Tt(gn, yn, _n) : Tt(gn, { geometry: yn }, _n);
                    return gn;
                  }(At, Dt);
                  this.tiles = {}, this.tileCoords = [], Ot && (console.timeEnd("preprocess data"), console.log("index: maxZoom: %d, maxPoints: %d", Dt.indexMaxZoom, Dt.indexMaxPoints), console.time("generate tiles"), this.stats = {}, this.total = 0), (un = function(yn, _n) {
                    var gn = _n.buffer / _n.extent, bn = yn, an = sn(yn, 1, -1 - gn, gn, 0, -1, 2, _n), Zt = sn(yn, 1, 1 - gn, 2 + gn, 0, -1, 2, _n);
                    return (an || Zt) && (bn = sn(yn, 1, -gn, 1 + gn, 0, -1, 2, _n) || [], an && (bn = Ht(an, 1).concat(bn)), Zt && (bn = bn.concat(Ht(Zt, -1)))), bn;
                  }(un, Dt)).length && this.splitTile(un, 0, 0, 0), Ot && (un.length && console.log("features: %d, points: %d", this.tiles[0].numFeatures, this.tiles[0].numPoints), console.timeEnd("generate tiles"), console.log("tiles generated:", this.total, JSON.stringify(this.stats)));
                }
                function ft(At, Dt, Ot) {
                  return 32 * ((1 << At) * Ot + Dt) + At;
                }
                function pt(At, Dt) {
                  var Ot = At.tileID.canonical;
                  if (!this._geoJSONIndex) return Dt(null, null);
                  var un = this._geoJSONIndex.getTile(Ot.z, Ot.x, Ot.y);
                  if (!un) return Dt(null, null);
                  var yn = new Ce(un.features), _n = Le(yn);
                  _n.byteOffset === 0 && _n.byteLength === _n.buffer.byteLength || (_n = new Uint8Array(_n)), Dt(null, { vectorTile: yn, rawData: _n.buffer });
                }
                st.prototype.load = function(At) {
                  var Dt = this.options, Ot = Dt.log, un = Dt.minZoom, yn = Dt.maxZoom, _n = Dt.nodeSize;
                  Ot && console.time("total time");
                  var gn = "prepare " + At.length + " points";
                  Ot && console.time(gn), this.points = At;
                  for (var bn = [], an = 0; an < At.length; an++) At[an].geometry && bn.push(bt(At[an], an));
                  this.trees[yn + 1] = new Je(bn, $t, Pt, _n, Float32Array), Ot && console.timeEnd(gn);
                  for (var Zt = yn; Zt >= un; Zt--) {
                    var Mn = +Date.now();
                    bn = this._cluster(bn, Zt), this.trees[Zt] = new Je(bn, $t, Pt, _n, Float32Array), Ot && console.log("z%d: %d clusters in %dms", Zt, bn.length, +Date.now() - Mn);
                  }
                  return Ot && console.timeEnd("total time"), this;
                }, st.prototype.getClusters = function(At, Dt) {
                  var Ot = ((At[0] + 180) % 360 + 360) % 360 - 180, un = Math.max(-90, Math.min(90, At[1])), yn = At[2] === 180 ? 180 : ((At[2] + 180) % 360 + 360) % 360 - 180, _n = Math.max(-90, Math.min(90, At[3]));
                  if (At[2] - At[0] >= 360) Ot = -180, yn = 180;
                  else if (Ot > yn) {
                    var gn = this.getClusters([Ot, un, 180, _n], Dt), bn = this.getClusters([-180, un, yn, _n], Dt);
                    return gn.concat(bn);
                  }
                  for (var an = this.trees[this._limitZoom(Dt)], Zt = [], Mn = 0, Ln = an.range(Lt(Ot), Rt(_n), Lt(yn), Rt(un)); Mn < Ln.length; Mn += 1) {
                    var In = Ln[Mn], kn = an.points[In];
                    Zt.push(kn.numPoints ? yt(kn) : this.points[kn.index]);
                  }
                  return Zt;
                }, st.prototype.getChildren = function(At) {
                  var Dt = this._getOriginId(At), Ot = this._getOriginZoom(At), un = "No cluster with the specified id.", yn = this.trees[Ot];
                  if (!yn) throw new Error(un);
                  var _n = yn.points[Dt];
                  if (!_n) throw new Error(un);
                  for (var gn = this.options.radius / (this.options.extent * Math.pow(2, Ot - 1)), bn = [], an = 0, Zt = yn.within(_n.x, _n.y, gn); an < Zt.length; an += 1) {
                    var Mn = Zt[an], Ln = yn.points[Mn];
                    Ln.parentId === At && bn.push(Ln.numPoints ? yt(Ln) : this.points[Ln.index]);
                  }
                  if (bn.length === 0) throw new Error(un);
                  return bn;
                }, st.prototype.getLeaves = function(At, Dt, Ot) {
                  Dt = Dt || 10, Ot = Ot || 0;
                  var un = [];
                  return this._appendLeaves(un, At, Dt, Ot, 0), un;
                }, st.prototype.getTile = function(At, Dt, Ot) {
                  var un = this.trees[this._limitZoom(At)], yn = Math.pow(2, At), _n = this.options, gn = _n.extent, bn = _n.radius / gn, an = (Ot - bn) / yn, Zt = (Ot + 1 + bn) / yn, Mn = { features: [] };
                  return this._addTileFeatures(un.range((Dt - bn) / yn, an, (Dt + 1 + bn) / yn, Zt), un.points, Dt, Ot, yn, Mn), Dt === 0 && this._addTileFeatures(un.range(1 - bn / yn, an, 1, Zt), un.points, yn, Ot, yn, Mn), Dt === yn - 1 && this._addTileFeatures(un.range(0, an, bn / yn, Zt), un.points, -1, Ot, yn, Mn), Mn.features.length ? Mn : null;
                }, st.prototype.getClusterExpansionZoom = function(At) {
                  for (var Dt = this._getOriginZoom(At) - 1; Dt <= this.options.maxZoom; ) {
                    var Ot = this.getChildren(At);
                    if (Dt++, Ot.length !== 1) break;
                    At = Ot[0].properties.cluster_id;
                  }
                  return Dt;
                }, st.prototype._appendLeaves = function(At, Dt, Ot, un, yn) {
                  for (var _n = 0, gn = this.getChildren(Dt); _n < gn.length; _n += 1) {
                    var bn = gn[_n], an = bn.properties;
                    if (an && an.cluster ? yn + an.point_count <= un ? yn += an.point_count : yn = this._appendLeaves(At, an.cluster_id, Ot, un, yn) : yn < un ? yn++ : At.push(bn), At.length === Ot) break;
                  }
                  return yn;
                }, st.prototype._addTileFeatures = function(At, Dt, Ot, un, yn, _n) {
                  for (var gn = 0, bn = At; gn < bn.length; gn += 1) {
                    var an = Dt[bn[gn]], Zt = an.numPoints, Mn = { type: 1, geometry: [[Math.round(this.options.extent * (an.x * yn - Ot)), Math.round(this.options.extent * (an.y * yn - un))]], tags: Zt ? St(an) : this.points[an.index].properties }, Ln = void 0;
                    Zt ? Ln = an.id : this.options.generateId ? Ln = an.index : this.points[an.index].id && (Ln = this.points[an.index].id), Ln !== void 0 && (Mn.id = Ln), _n.features.push(Mn);
                  }
                }, st.prototype._limitZoom = function(At) {
                  return Math.max(this.options.minZoom, Math.min(+At, this.options.maxZoom + 1));
                }, st.prototype._cluster = function(At, Dt) {
                  for (var Ot = [], un = this.options, yn = un.radius, _n = un.extent, gn = un.reduce, bn = un.minPoints, an = yn / (_n * Math.pow(2, Dt)), Zt = 0; Zt < At.length; Zt++) {
                    var Mn = At[Zt];
                    if (!(Mn.zoom <= Dt)) {
                      Mn.zoom = Dt;
                      for (var Ln = this.trees[Dt + 1], In = Ln.within(Mn.x, Mn.y, an), kn = Mn.numPoints || 1, er = kn, jn = 0, ur = In; jn < ur.length; jn += 1) {
                        var Kn = ur[jn], Jn = Ln.points[Kn];
                        Jn.zoom > Dt && (er += Jn.numPoints || 1);
                      }
                      if (er >= bn) {
                        for (var Lr = Mn.x * kn, mr = Mn.y * kn, Ar = gn && kn > 1 ? this._map(Mn, !0) : null, Yr = (Zt << 5) + (Dt + 1) + this.points.length, jr = 0, Mr = In; jr < Mr.length; jr += 1) {
                          var Wr = Mr[jr], Bi = Ln.points[Wr];
                          if (!(Bi.zoom <= Dt)) {
                            Bi.zoom = Dt;
                            var gi = Bi.numPoints || 1;
                            Lr += Bi.x * gi, mr += Bi.y * gi, Bi.parentId = Yr, gn && (Ar || (Ar = this._map(Mn, !0)), gn(Ar, this._map(Bi)));
                          }
                        }
                        Mn.parentId = Yr, Ot.push(gt(Lr / er, mr / er, Yr, er, Ar));
                      } else if (Ot.push(Mn), er > 1) for (var Zr = 0, fi = In; Zr < fi.length; Zr += 1) {
                        var ri = fi[Zr], bi = Ln.points[ri];
                        bi.zoom <= Dt || (bi.zoom = Dt, Ot.push(bi));
                      }
                    }
                  }
                  return Ot;
                }, st.prototype._getOriginId = function(At) {
                  return At - this.points.length >> 5;
                }, st.prototype._getOriginZoom = function(At) {
                  return (At - this.points.length) % 32;
                }, st.prototype._map = function(At, Dt) {
                  if (At.numPoints) return Dt ? vt({}, At.properties) : At.properties;
                  var Ot = this.points[At.index].properties, un = this.options.map(Ot);
                  return Dt && un === Ot ? vt({}, un) : un;
                }, rt.prototype.options = { maxZoom: 14, indexMaxZoom: 5, indexMaxPoints: 1e5, tolerance: 3, extent: 4096, buffer: 64, lineMetrics: !1, promoteId: null, generateId: !1, debug: 0 }, rt.prototype.splitTile = function(At, Dt, Ot, un, yn, _n, gn) {
                  for (var bn = [At, Dt, Ot, un], an = this.options, Zt = an.debug; bn.length; ) {
                    un = bn.pop(), Ot = bn.pop(), Dt = bn.pop(), At = bn.pop();
                    var Mn = 1 << Dt, Ln = ft(Dt, Ot, un), In = this.tiles[Ln];
                    if (!In && (Zt > 1 && console.time("creation"), In = this.tiles[Ln] = Rn(At, Dt, Ot, un, an), this.tileCoords.push({ z: Dt, x: Ot, y: un }), Zt)) {
                      Zt > 1 && (console.log("tile z%d-%d-%d (features: %d, points: %d, simplified: %d)", Dt, Ot, un, In.numFeatures, In.numPoints, In.numSimplified), console.timeEnd("creation"));
                      var kn = "z" + Dt;
                      this.stats[kn] = (this.stats[kn] || 0) + 1, this.total++;
                    }
                    if (In.source = At, yn) {
                      if (Dt === an.maxZoom || Dt === yn) continue;
                      var er = 1 << yn - Dt;
                      if (Ot !== Math.floor(_n / er) || un !== Math.floor(gn / er)) continue;
                    } else if (Dt === an.indexMaxZoom || In.numPoints <= an.indexMaxPoints) continue;
                    if (In.source = null, At.length !== 0) {
                      Zt > 1 && console.time("clipping");
                      var jn, ur, Kn, Jn, Lr, mr, Ar = 0.5 * an.buffer / an.extent, Yr = 0.5 - Ar, jr = 0.5 + Ar, Mr = 1 + Ar;
                      jn = ur = Kn = Jn = null, Lr = sn(At, Mn, Ot - Ar, Ot + jr, 0, In.minX, In.maxX, an), mr = sn(At, Mn, Ot + Yr, Ot + Mr, 0, In.minX, In.maxX, an), At = null, Lr && (jn = sn(Lr, Mn, un - Ar, un + jr, 1, In.minY, In.maxY, an), ur = sn(Lr, Mn, un + Yr, un + Mr, 1, In.minY, In.maxY, an), Lr = null), mr && (Kn = sn(mr, Mn, un - Ar, un + jr, 1, In.minY, In.maxY, an), Jn = sn(mr, Mn, un + Yr, un + Mr, 1, In.minY, In.maxY, an), mr = null), Zt > 1 && console.timeEnd("clipping"), bn.push(jn || [], Dt + 1, 2 * Ot, 2 * un), bn.push(ur || [], Dt + 1, 2 * Ot, 2 * un + 1), bn.push(Kn || [], Dt + 1, 2 * Ot + 1, 2 * un), bn.push(Jn || [], Dt + 1, 2 * Ot + 1, 2 * un + 1);
                    }
                  }
                }, rt.prototype.getTile = function(At, Dt, Ot) {
                  var un = this.options, yn = un.extent, _n = un.debug;
                  if (At < 0 || At > 24) return null;
                  var gn = 1 << At, bn = ft(At, Dt = (Dt % gn + gn) % gn, Ot);
                  if (this.tiles[bn]) return cn(this.tiles[bn], yn);
                  _n > 1 && console.log("drilling down to z%d-%d-%d", At, Dt, Ot);
                  for (var an, Zt = At, Mn = Dt, Ln = Ot; !an && Zt > 0; ) Zt--, Mn = Math.floor(Mn / 2), Ln = Math.floor(Ln / 2), an = this.tiles[ft(Zt, Mn, Ln)];
                  return an && an.source ? (_n > 1 && console.log("found parent tile z%d-%d-%d", Zt, Mn, Ln), _n > 1 && console.time("drilling down"), this.splitTile(an.source, Zt, Mn, Ln, At, Dt, Ot), _n > 1 && console.timeEnd("drilling down"), this.tiles[bn] ? cn(this.tiles[bn], yn) : null) : null;
                };
                var wt = function(At) {
                  function Dt(Ot, un, yn, _n) {
                    At.call(this, Ot, un, yn, pt), _n && (this.loadGeoJSON = _n);
                  }
                  return Dt.__proto__ = At, Dt.prototype = Object.create(At.prototype), Dt.prototype.constructor = Dt, Dt.prototype.loadData = function(Ot, un) {
                    this._pendingCallback && this._pendingCallback(null, { abandoned: !0 }), this._pendingCallback = un, this._pendingLoadDataParams = Ot, this._state && this._state !== "Idle" ? this._state = "NeedsLoadData" : (this._state = "Coalescing", this._loadData());
                  }, Dt.prototype._loadData = function() {
                    var Ot = this;
                    if (this._pendingCallback && this._pendingLoadDataParams) {
                      var un = this._pendingCallback, yn = this._pendingLoadDataParams;
                      delete this._pendingCallback, delete this._pendingLoadDataParams;
                      var _n = !!(yn && yn.request && yn.request.collectResourceTiming) && new ee.RequestPerformance(yn.request);
                      this.loadGeoJSON(yn, function(gn, bn) {
                        if (gn || !bn) return un(gn);
                        if (typeof bn != "object") return un(new Error("Input data given to '" + yn.source + "' is not a valid GeoJSON object."));
                        pe(bn, !0);
                        try {
                          if (yn.filter) {
                            var an = ee.createExpression(yn.filter, { type: "boolean", "property-type": "data-driven", overridable: !1, transition: !1 });
                            if (an.result === "error") throw new Error(an.value.map(function(In) {
                              return In.key + ": " + In.message;
                            }).join(", "));
                            var Zt = bn.features.filter(function(In) {
                              return an.value.evaluate({ zoom: 0 }, In);
                            });
                            bn = { type: "FeatureCollection", features: Zt };
                          }
                          Ot._geoJSONIndex = yn.cluster ? new st(function(In) {
                            var kn = In.superclusterOptions, er = In.clusterProperties;
                            if (!er || !kn) return kn;
                            for (var jn = {}, ur = {}, Kn = { accumulated: null, zoom: 0 }, Jn = { properties: null }, Lr = Object.keys(er), mr = 0, Ar = Lr; mr < Ar.length; mr += 1) {
                              var Yr = Ar[mr], jr = er[Yr], Mr = jr[0], Wr = jr[1], Bi = ee.createExpression(Wr), gi = ee.createExpression(typeof Mr == "string" ? [Mr, ["accumulated"], ["get", Yr]] : Mr);
                              jn[Yr] = Bi.value, ur[Yr] = gi.value;
                            }
                            return kn.map = function(Zr) {
                              Jn.properties = Zr;
                              for (var fi = {}, ri = 0, bi = Lr; ri < bi.length; ri += 1) {
                                var Zi = bi[ri];
                                fi[Zi] = jn[Zi].evaluate(Kn, Jn);
                              }
                              return fi;
                            }, kn.reduce = function(Zr, fi) {
                              Jn.properties = fi;
                              for (var ri = 0, bi = Lr; ri < bi.length; ri += 1) {
                                var Zi = bi[ri];
                                Kn.accumulated = Zr[Zi], Zr[Zi] = ur[Zi].evaluate(Kn, Jn);
                              }
                            }, kn;
                          }(yn)).load(bn.features) : function(In, kn) {
                            return new rt(In, kn);
                          }(bn, yn.geojsonVtOptions);
                        } catch (In) {
                          return un(In);
                        }
                        Ot.loaded = {};
                        var Mn = {};
                        if (_n) {
                          var Ln = _n.finish();
                          Ln && (Mn.resourceTiming = {}, Mn.resourceTiming[yn.source] = JSON.parse(JSON.stringify(Ln)));
                        }
                        un(null, Mn);
                      });
                    }
                  }, Dt.prototype.coalesce = function() {
                    this._state === "Coalescing" ? this._state = "Idle" : this._state === "NeedsLoadData" && (this._state = "Coalescing", this._loadData());
                  }, Dt.prototype.reloadTile = function(Ot, un) {
                    var yn = this.loaded, _n = Ot.uid;
                    return yn && yn[_n] ? At.prototype.reloadTile.call(this, Ot, un) : this.loadTile(Ot, un);
                  }, Dt.prototype.loadGeoJSON = function(Ot, un) {
                    if (Ot.request) ee.getJSON(Ot.request, un);
                    else {
                      if (typeof Ot.data != "string") return un(new Error("Input data given to '" + Ot.source + "' is not a valid GeoJSON object."));
                      try {
                        return un(null, JSON.parse(Ot.data));
                      } catch {
                        return un(new Error("Input data given to '" + Ot.source + "' is not a valid GeoJSON object."));
                      }
                    }
                  }, Dt.prototype.removeSource = function(Ot, un) {
                    this._pendingCallback && this._pendingCallback(null, { abandoned: !0 }), un();
                  }, Dt.prototype.getClusterExpansionZoom = function(Ot, un) {
                    try {
                      un(null, this._geoJSONIndex.getClusterExpansionZoom(Ot.clusterId));
                    } catch (yn) {
                      un(yn);
                    }
                  }, Dt.prototype.getClusterChildren = function(Ot, un) {
                    try {
                      un(null, this._geoJSONIndex.getChildren(Ot.clusterId));
                    } catch (yn) {
                      un(yn);
                    }
                  }, Dt.prototype.getClusterLeaves = function(Ot, un) {
                    try {
                      un(null, this._geoJSONIndex.getLeaves(Ot.clusterId, Ot.limit, Ot.offset));
                    } catch (yn) {
                      un(yn);
                    }
                  }, Dt;
                }(fe), Bt = function(At) {
                  var Dt = this;
                  this.self = At, this.actor = new ee.Actor(At, this), this.layerIndexes = {}, this.availableImages = {}, this.workerSourceTypes = { vector: fe, geojson: wt }, this.workerSources = {}, this.demWorkerSources = {}, this.self.registerWorkerSource = function(Ot, un) {
                    if (Dt.workerSourceTypes[Ot]) throw new Error('Worker source with name "' + Ot + '" already registered.');
                    Dt.workerSourceTypes[Ot] = un;
                  }, this.self.registerRTLTextPlugin = function(Ot) {
                    if (ee.plugin.isParsed()) throw new Error("RTL text plugin already registered.");
                    ee.plugin.applyArabicShaping = Ot.applyArabicShaping, ee.plugin.processBidirectionalText = Ot.processBidirectionalText, ee.plugin.processStyledBidirectionalText = Ot.processStyledBidirectionalText;
                  };
                };
                return Bt.prototype.setReferrer = function(At, Dt) {
                  this.referrer = Dt;
                }, Bt.prototype.setImages = function(At, Dt, Ot) {
                  for (var un in this.availableImages[At] = Dt, this.workerSources[At]) {
                    var yn = this.workerSources[At][un];
                    for (var _n in yn) yn[_n].availableImages = Dt;
                  }
                  Ot();
                }, Bt.prototype.setLayers = function(At, Dt, Ot) {
                  this.getLayerIndex(At).replace(Dt), Ot();
                }, Bt.prototype.updateLayers = function(At, Dt, Ot) {
                  this.getLayerIndex(At).update(Dt.layers, Dt.removedIds), Ot();
                }, Bt.prototype.loadTile = function(At, Dt, Ot) {
                  this.getWorkerSource(At, Dt.type, Dt.source).loadTile(Dt, Ot);
                }, Bt.prototype.loadDEMTile = function(At, Dt, Ot) {
                  this.getDEMWorkerSource(At, Dt.source).loadTile(Dt, Ot);
                }, Bt.prototype.reloadTile = function(At, Dt, Ot) {
                  this.getWorkerSource(At, Dt.type, Dt.source).reloadTile(Dt, Ot);
                }, Bt.prototype.abortTile = function(At, Dt, Ot) {
                  this.getWorkerSource(At, Dt.type, Dt.source).abortTile(Dt, Ot);
                }, Bt.prototype.removeTile = function(At, Dt, Ot) {
                  this.getWorkerSource(At, Dt.type, Dt.source).removeTile(Dt, Ot);
                }, Bt.prototype.removeDEMTile = function(At, Dt) {
                  this.getDEMWorkerSource(At, Dt.source).removeTile(Dt);
                }, Bt.prototype.removeSource = function(At, Dt, Ot) {
                  if (this.workerSources[At] && this.workerSources[At][Dt.type] && this.workerSources[At][Dt.type][Dt.source]) {
                    var un = this.workerSources[At][Dt.type][Dt.source];
                    delete this.workerSources[At][Dt.type][Dt.source], un.removeSource !== void 0 ? un.removeSource(Dt, Ot) : Ot();
                  }
                }, Bt.prototype.loadWorkerSource = function(At, Dt, Ot) {
                  try {
                    this.self.importScripts(Dt.url), Ot();
                  } catch (un) {
                    Ot(un.toString());
                  }
                }, Bt.prototype.syncRTLPluginState = function(At, Dt, Ot) {
                  try {
                    ee.plugin.setState(Dt);
                    var un = ee.plugin.getPluginURL();
                    if (ee.plugin.isLoaded() && !ee.plugin.isParsed() && un != null) {
                      this.self.importScripts(un);
                      var yn = ee.plugin.isParsed();
                      Ot(yn ? void 0 : new Error("RTL Text Plugin failed to import scripts from " + un), yn);
                    }
                  } catch (_n) {
                    Ot(_n.toString());
                  }
                }, Bt.prototype.getAvailableImages = function(At) {
                  var Dt = this.availableImages[At];
                  return Dt || (Dt = []), Dt;
                }, Bt.prototype.getLayerIndex = function(At) {
                  var Dt = this.layerIndexes[At];
                  return Dt || (Dt = this.layerIndexes[At] = new se()), Dt;
                }, Bt.prototype.getWorkerSource = function(At, Dt, Ot) {
                  var un = this;
                  if (this.workerSources[At] || (this.workerSources[At] = {}), this.workerSources[At][Dt] || (this.workerSources[At][Dt] = {}), !this.workerSources[At][Dt][Ot]) {
                    var yn = { send: function(_n, gn, bn) {
                      un.actor.send(_n, gn, bn, At);
                    } };
                    this.workerSources[At][Dt][Ot] = new this.workerSourceTypes[Dt](yn, this.getLayerIndex(At), this.getAvailableImages(At));
                  }
                  return this.workerSources[At][Dt][Ot];
                }, Bt.prototype.getDEMWorkerSource = function(At, Dt) {
                  return this.demWorkerSources[At] || (this.demWorkerSources[At] = {}), this.demWorkerSources[At][Dt] || (this.demWorkerSources[At][Dt] = new me()), this.demWorkerSources[At][Dt];
                }, Bt.prototype.enforceCacheSizeLimit = function(At, Dt) {
                  ee.enforceCacheSizeLimit(Dt);
                }, typeof WorkerGlobalScope < "u" && typeof self < "u" && self instanceof WorkerGlobalScope && (self.worker = new Bt(self)), Bt;
              }), ne(0, function(ee) {
                var ae = ee.createCommonjsModule(function(Ke) {
                  function Fe(Oe) {
                    return !Te(Oe);
                  }
                  function Te(Oe) {
                    return typeof window < "u" && typeof document < "u" ? Array.prototype && Array.prototype.every && Array.prototype.filter && Array.prototype.forEach && Array.prototype.indexOf && Array.prototype.lastIndexOf && Array.prototype.map && Array.prototype.some && Array.prototype.reduce && Array.prototype.reduceRight && Array.isArray ? Function.prototype && Function.prototype.bind ? Object.keys && Object.create && Object.getPrototypeOf && Object.getOwnPropertyNames && Object.isSealed && Object.isFrozen && Object.isExtensible && Object.getOwnPropertyDescriptor && Object.defineProperty && Object.defineProperties && Object.seal && Object.freeze && Object.preventExtensions ? "JSON" in window && "parse" in JSON && "stringify" in JSON ? function() {
                      if (!("Worker" in window && "Blob" in window && "URL" in window)) return !1;
                      var Ue, nt, ht = new Blob([""], { type: "text/javascript" }), Et = URL.createObjectURL(ht);
                      try {
                        nt = new Worker(Et), Ue = !0;
                      } catch {
                        Ue = !1;
                      }
                      return nt && nt.terminate(), URL.revokeObjectURL(Et), Ue;
                    }() ? "Uint8ClampedArray" in window ? ArrayBuffer.isView ? function() {
                      var Ue = document.createElement("canvas");
                      Ue.width = Ue.height = 1;
                      var nt = Ue.getContext("2d");
                      if (!nt) return !1;
                      var ht = nt.getImageData(0, 0, 1, 1);
                      return ht && ht.width === Ue.width;
                    }() ? (Be = Oe && Oe.failIfMajorPerformanceCaveat, Ae[Be] === void 0 && (Ae[Be] = function(Ue) {
                      var nt = function(Et) {
                        var Mt = document.createElement("canvas"), qt = Object.create(Fe.webGLContextAttributes);
                        return qt.failIfMajorPerformanceCaveat = Et, Mt.probablySupportsContext ? Mt.probablySupportsContext("webgl", qt) || Mt.probablySupportsContext("experimental-webgl", qt) : Mt.supportsContext ? Mt.supportsContext("webgl", qt) || Mt.supportsContext("experimental-webgl", qt) : Mt.getContext("webgl", qt) || Mt.getContext("experimental-webgl", qt);
                      }(Ue);
                      if (!nt) return !1;
                      var ht = nt.createShader(nt.VERTEX_SHADER);
                      return !(!ht || nt.isContextLost()) && (nt.shaderSource(ht, "void main() {}"), nt.compileShader(ht), nt.getShaderParameter(ht, nt.COMPILE_STATUS) === !0);
                    }(Be)), Ae[Be] ? void 0 : "insufficient WebGL support") : "insufficient Canvas/getImageData support" : "insufficient ArrayBuffer support" : "insufficient Uint8ClampedArray support" : "insufficient worker support" : "insufficient JSON support" : "insufficient Object support" : "insufficient Function support" : "insufficent Array support" : "not a browser";
                    var Be;
                  }
                  Ke.exports ? Ke.exports = Fe : window && (window.mapboxgl = window.mapboxgl || {}, window.mapboxgl.supported = Fe, window.mapboxgl.notSupportedReason = Te);
                  var Ae = {};
                  Fe.webGLContextAttributes = { antialias: !1, alpha: !0, stencil: !0, depth: !0 };
                }), oe = { create: function(Ke, Fe, Te) {
                  var Ae = ee.window.document.createElement(Ke);
                  return Fe !== void 0 && (Ae.className = Fe), Te && Te.appendChild(Ae), Ae;
                }, createNS: function(Ke, Fe) {
                  return ee.window.document.createElementNS(Ke, Fe);
                } }, se = ee.window.document && ee.window.document.documentElement.style;
                function ce(Ke) {
                  if (!se) return Ke[0];
                  for (var Fe = 0; Fe < Ke.length; Fe++) if (Ke[Fe] in se) return Ke[Fe];
                  return Ke[0];
                }
                var le, ue = ce(["userSelect", "MozUserSelect", "WebkitUserSelect", "msUserSelect"]);
                oe.disableDrag = function() {
                  se && ue && (le = se[ue], se[ue] = "none");
                }, oe.enableDrag = function() {
                  se && ue && (se[ue] = le);
                };
                var de = ce(["transform", "WebkitTransform"]);
                oe.setTransform = function(Ke, Fe) {
                  Ke.style[de] = Fe;
                };
                var fe = !1;
                try {
                  var he = Object.defineProperty({}, "passive", { get: function() {
                    fe = !0;
                  } });
                  ee.window.addEventListener("test", he, he), ee.window.removeEventListener("test", he, he);
                } catch {
                  fe = !1;
                }
                oe.addEventListener = function(Ke, Fe, Te, Ae) {
                  Ae === void 0 && (Ae = {}), "passive" in Ae && fe ? Ke.addEventListener(Fe, Te, Ae) : Ke.addEventListener(Fe, Te, Ae.capture);
                }, oe.removeEventListener = function(Ke, Fe, Te, Ae) {
                  Ae === void 0 && (Ae = {}), "passive" in Ae && fe ? Ke.removeEventListener(Fe, Te, Ae) : Ke.removeEventListener(Fe, Te, Ae.capture);
                };
                var me = function(Ke) {
                  Ke.preventDefault(), Ke.stopPropagation(), ee.window.removeEventListener("click", me, !0);
                };
                function pe(Ke) {
                  var Fe = Ke.userImage;
                  return !!(Fe && Fe.render && Fe.render()) && (Ke.data.replace(new Uint8Array(Fe.data.buffer)), !0);
                }
                oe.suppressClick = function() {
                  ee.window.addEventListener("click", me, !0), ee.window.setTimeout(function() {
                    ee.window.removeEventListener("click", me, !0);
                  }, 0);
                }, oe.mousePos = function(Ke, Fe) {
                  var Te = Ke.getBoundingClientRect();
                  return new ee.Point(Fe.clientX - Te.left - Ke.clientLeft, Fe.clientY - Te.top - Ke.clientTop);
                }, oe.touchPos = function(Ke, Fe) {
                  for (var Te = Ke.getBoundingClientRect(), Ae = [], Oe = 0; Oe < Fe.length; Oe++) Ae.push(new ee.Point(Fe[Oe].clientX - Te.left - Ke.clientLeft, Fe[Oe].clientY - Te.top - Ke.clientTop));
                  return Ae;
                }, oe.mouseButton = function(Ke) {
                  return ee.window.InstallTrigger !== void 0 && Ke.button === 2 && Ke.ctrlKey && ee.window.navigator.platform.toUpperCase().indexOf("MAC") >= 0 ? 0 : Ke.button;
                }, oe.remove = function(Ke) {
                  Ke.parentNode && Ke.parentNode.removeChild(Ke);
                };
                var ye = function(Ke) {
                  function Fe() {
                    Ke.call(this), this.images = {}, this.updatedImages = {}, this.callbackDispatchedThisFrame = {}, this.loaded = !1, this.requestors = [], this.patterns = {}, this.atlasImage = new ee.RGBAImage({ width: 1, height: 1 }), this.dirty = !0;
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.isLoaded = function() {
                    return this.loaded;
                  }, Fe.prototype.setLoaded = function(Te) {
                    if (this.loaded !== Te && (this.loaded = Te, Te)) {
                      for (var Ae = 0, Oe = this.requestors; Ae < Oe.length; Ae += 1) {
                        var Be = Oe[Ae], Ue = Be.ids, nt = Be.callback;
                        this._notify(Ue, nt);
                      }
                      this.requestors = [];
                    }
                  }, Fe.prototype.getImage = function(Te) {
                    return this.images[Te];
                  }, Fe.prototype.addImage = function(Te, Ae) {
                    this._validate(Te, Ae) && (this.images[Te] = Ae);
                  }, Fe.prototype._validate = function(Te, Ae) {
                    var Oe = !0;
                    return this._validateStretch(Ae.stretchX, Ae.data && Ae.data.width) || (this.fire(new ee.ErrorEvent(new Error('Image "' + Te + '" has invalid "stretchX" value'))), Oe = !1), this._validateStretch(Ae.stretchY, Ae.data && Ae.data.height) || (this.fire(new ee.ErrorEvent(new Error('Image "' + Te + '" has invalid "stretchY" value'))), Oe = !1), this._validateContent(Ae.content, Ae) || (this.fire(new ee.ErrorEvent(new Error('Image "' + Te + '" has invalid "content" value'))), Oe = !1), Oe;
                  }, Fe.prototype._validateStretch = function(Te, Ae) {
                    if (!Te) return !0;
                    for (var Oe = 0, Be = 0, Ue = Te; Be < Ue.length; Be += 1) {
                      var nt = Ue[Be];
                      if (nt[0] < Oe || nt[1] < nt[0] || Ae < nt[1]) return !1;
                      Oe = nt[1];
                    }
                    return !0;
                  }, Fe.prototype._validateContent = function(Te, Ae) {
                    return !(Te && (Te.length !== 4 || Te[0] < 0 || Ae.data.width < Te[0] || Te[1] < 0 || Ae.data.height < Te[1] || Te[2] < 0 || Ae.data.width < Te[2] || Te[3] < 0 || Ae.data.height < Te[3] || Te[2] < Te[0] || Te[3] < Te[1]));
                  }, Fe.prototype.updateImage = function(Te, Ae) {
                    var Oe = this.images[Te];
                    Ae.version = Oe.version + 1, this.images[Te] = Ae, this.updatedImages[Te] = !0;
                  }, Fe.prototype.removeImage = function(Te) {
                    var Ae = this.images[Te];
                    delete this.images[Te], delete this.patterns[Te], Ae.userImage && Ae.userImage.onRemove && Ae.userImage.onRemove();
                  }, Fe.prototype.listImages = function() {
                    return Object.keys(this.images);
                  }, Fe.prototype.getImages = function(Te, Ae) {
                    var Oe = !0;
                    if (!this.isLoaded()) for (var Be = 0, Ue = Te; Be < Ue.length; Be += 1) {
                      var nt = Ue[Be];
                      this.images[nt] || (Oe = !1);
                    }
                    this.isLoaded() || Oe ? this._notify(Te, Ae) : this.requestors.push({ ids: Te, callback: Ae });
                  }, Fe.prototype._notify = function(Te, Ae) {
                    for (var Oe = {}, Be = 0, Ue = Te; Be < Ue.length; Be += 1) {
                      var nt = Ue[Be];
                      this.images[nt] || this.fire(new ee.Event("styleimagemissing", { id: nt }));
                      var ht = this.images[nt];
                      ht ? Oe[nt] = { data: ht.data.clone(), pixelRatio: ht.pixelRatio, sdf: ht.sdf, version: ht.version, stretchX: ht.stretchX, stretchY: ht.stretchY, content: ht.content, hasRenderCallback: !!(ht.userImage && ht.userImage.render) } : ee.warnOnce('Image "' + nt + '" could not be loaded. Please make sure you have added the image with map.addImage() or a "sprite" property in your style. You can provide missing images by listening for the "styleimagemissing" map event.');
                    }
                    Ae(null, Oe);
                  }, Fe.prototype.getPixelSize = function() {
                    var Te = this.atlasImage;
                    return { width: Te.width, height: Te.height };
                  }, Fe.prototype.getPattern = function(Te) {
                    var Ae = this.patterns[Te], Oe = this.getImage(Te);
                    if (!Oe) return null;
                    if (Ae && Ae.position.version === Oe.version) return Ae.position;
                    if (Ae) Ae.position.version = Oe.version;
                    else {
                      var Be = { w: Oe.data.width + 2, h: Oe.data.height + 2, x: 0, y: 0 }, Ue = new ee.ImagePosition(Be, Oe);
                      this.patterns[Te] = { bin: Be, position: Ue };
                    }
                    return this._updatePatternAtlas(), this.patterns[Te].position;
                  }, Fe.prototype.bind = function(Te) {
                    var Ae = Te.gl;
                    this.atlasTexture ? this.dirty && (this.atlasTexture.update(this.atlasImage), this.dirty = !1) : this.atlasTexture = new ee.Texture(Te, this.atlasImage, Ae.RGBA), this.atlasTexture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE);
                  }, Fe.prototype._updatePatternAtlas = function() {
                    var Te = [];
                    for (var Ae in this.patterns) Te.push(this.patterns[Ae].bin);
                    var Oe = ee.potpack(Te), Be = Oe.w, Ue = Oe.h, nt = this.atlasImage;
                    for (var ht in nt.resize({ width: Be || 1, height: Ue || 1 }), this.patterns) {
                      var Et = this.patterns[ht].bin, Mt = Et.x + 1, qt = Et.y + 1, tn = this.images[ht].data, xn = tn.width, An = tn.height;
                      ee.RGBAImage.copy(tn, nt, { x: 0, y: 0 }, { x: Mt, y: qt }, { width: xn, height: An }), ee.RGBAImage.copy(tn, nt, { x: 0, y: An - 1 }, { x: Mt, y: qt - 1 }, { width: xn, height: 1 }), ee.RGBAImage.copy(tn, nt, { x: 0, y: 0 }, { x: Mt, y: qt + An }, { width: xn, height: 1 }), ee.RGBAImage.copy(tn, nt, { x: xn - 1, y: 0 }, { x: Mt - 1, y: qt }, { width: 1, height: An }), ee.RGBAImage.copy(tn, nt, { x: 0, y: 0 }, { x: Mt + xn, y: qt }, { width: 1, height: An });
                    }
                    this.dirty = !0;
                  }, Fe.prototype.beginFrame = function() {
                    this.callbackDispatchedThisFrame = {};
                  }, Fe.prototype.dispatchRenderCallbacks = function(Te) {
                    for (var Ae = 0, Oe = Te; Ae < Oe.length; Ae += 1) {
                      var Be = Oe[Ae];
                      if (!this.callbackDispatchedThisFrame[Be]) {
                        this.callbackDispatchedThisFrame[Be] = !0;
                        var Ue = this.images[Be];
                        pe(Ue) && this.updateImage(Be, Ue);
                      }
                    }
                  }, Fe;
                }(ee.Evented), ge = Ce, ve = Ce, be = 1e20;
                function Ce(Ke, Fe, Te, Ae, Oe, Be) {
                  this.fontSize = Ke || 24, this.buffer = Fe === void 0 ? 3 : Fe, this.cutoff = Ae || 0.25, this.fontFamily = Oe || "sans-serif", this.fontWeight = Be || "normal", this.radius = Te || 8;
                  var Ue = this.size = this.fontSize + 2 * this.buffer;
                  this.canvas = document.createElement("canvas"), this.canvas.width = this.canvas.height = Ue, this.ctx = this.canvas.getContext("2d"), this.ctx.font = this.fontWeight + " " + this.fontSize + "px " + this.fontFamily, this.ctx.textBaseline = "middle", this.ctx.fillStyle = "black", this.gridOuter = new Float64Array(Ue * Ue), this.gridInner = new Float64Array(Ue * Ue), this.f = new Float64Array(Ue), this.d = new Float64Array(Ue), this.z = new Float64Array(Ue + 1), this.v = new Int16Array(Ue), this.middle = Math.round(Ue / 2 * (navigator.userAgent.indexOf("Gecko/") >= 0 ? 1.2 : 1));
                }
                function Se(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  for (var nt = 0; nt < Fe; nt++) {
                    for (var ht = 0; ht < Te; ht++) Ae[ht] = Ke[ht * Fe + nt];
                    for (we(Ae, Oe, Be, Ue, Te), ht = 0; ht < Te; ht++) Ke[ht * Fe + nt] = Oe[ht];
                  }
                  for (ht = 0; ht < Te; ht++) {
                    for (nt = 0; nt < Fe; nt++) Ae[nt] = Ke[ht * Fe + nt];
                    for (we(Ae, Oe, Be, Ue, Fe), nt = 0; nt < Fe; nt++) Ke[ht * Fe + nt] = Math.sqrt(Oe[nt]);
                  }
                }
                function we(Ke, Fe, Te, Ae, Oe) {
                  Te[0] = 0, Ae[0] = -1e20, Ae[1] = 1e20;
                  for (var Be = 1, Ue = 0; Be < Oe; Be++) {
                    for (var nt = (Ke[Be] + Be * Be - (Ke[Te[Ue]] + Te[Ue] * Te[Ue])) / (2 * Be - 2 * Te[Ue]); nt <= Ae[Ue]; ) Ue--, nt = (Ke[Be] + Be * Be - (Ke[Te[Ue]] + Te[Ue] * Te[Ue])) / (2 * Be - 2 * Te[Ue]);
                    Te[++Ue] = Be, Ae[Ue] = nt, Ae[Ue + 1] = 1e20;
                  }
                  for (Be = 0, Ue = 0; Be < Oe; Be++) {
                    for (; Ae[Ue + 1] < Be; ) Ue++;
                    Fe[Be] = (Be - Te[Ue]) * (Be - Te[Ue]) + Ke[Te[Ue]];
                  }
                }
                Ce.prototype.draw = function(Ke) {
                  this.ctx.clearRect(0, 0, this.size, this.size), this.ctx.fillText(Ke, this.buffer, this.middle);
                  for (var Fe = this.ctx.getImageData(0, 0, this.size, this.size), Te = new Uint8ClampedArray(this.size * this.size), Ae = 0; Ae < this.size * this.size; Ae++) {
                    var Oe = Fe.data[4 * Ae + 3] / 255;
                    this.gridOuter[Ae] = Oe === 1 ? 0 : Oe === 0 ? be : Math.pow(Math.max(0, 0.5 - Oe), 2), this.gridInner[Ae] = Oe === 1 ? be : Oe === 0 ? 0 : Math.pow(Math.max(0, Oe - 0.5), 2);
                  }
                  for (Se(this.gridOuter, this.size, this.size, this.f, this.d, this.v, this.z), Se(this.gridInner, this.size, this.size, this.f, this.d, this.v, this.z), Ae = 0; Ae < this.size * this.size; Ae++) {
                    var Be = this.gridOuter[Ae] - this.gridInner[Ae];
                    Te[Ae] = Math.max(0, Math.min(255, Math.round(255 - 255 * (Be / this.radius + this.cutoff))));
                  }
                  return Te;
                }, ge.default = ve;
                var Ee = function(Ke, Fe) {
                  this.requestManager = Ke, this.localIdeographFontFamily = Fe, this.entries = {};
                };
                Ee.prototype.setURL = function(Ke) {
                  this.url = Ke;
                }, Ee.prototype.getGlyphs = function(Ke, Fe) {
                  var Te = this, Ae = [];
                  for (var Oe in Ke) for (var Be = 0, Ue = Ke[Oe]; Be < Ue.length; Be += 1) {
                    var nt = Ue[Be];
                    Ae.push({ stack: Oe, id: nt });
                  }
                  ee.asyncAll(Ae, function(ht, Et) {
                    var Mt = ht.stack, qt = ht.id, tn = Te.entries[Mt];
                    tn || (tn = Te.entries[Mt] = { glyphs: {}, requests: {}, ranges: {} });
                    var xn = tn.glyphs[qt];
                    if (xn === void 0) {
                      if (xn = Te._tinySDF(tn, Mt, qt)) return tn.glyphs[qt] = xn, void Et(null, { stack: Mt, id: qt, glyph: xn });
                      var An = Math.floor(qt / 256);
                      if (256 * An > 65535) Et(new Error("glyphs > 65535 not supported"));
                      else if (tn.ranges[An]) Et(null, { stack: Mt, id: qt, glyph: xn });
                      else {
                        var On = tn.requests[An];
                        On || (On = tn.requests[An] = [], Ee.loadGlyphRange(Mt, An, Te.url, Te.requestManager, function(Bn, qn) {
                          if (qn) {
                            for (var Vn in qn) Te._doesCharSupportLocalGlyph(+Vn) || (tn.glyphs[+Vn] = qn[+Vn]);
                            tn.ranges[An] = !0;
                          }
                          for (var sr = 0, Un = On; sr < Un.length; sr += 1) (0, Un[sr])(Bn, qn);
                          delete tn.requests[An];
                        })), On.push(function(Bn, qn) {
                          Bn ? Et(Bn) : qn && Et(null, { stack: Mt, id: qt, glyph: qn[qt] || null });
                        });
                      }
                    } else Et(null, { stack: Mt, id: qt, glyph: xn });
                  }, function(ht, Et) {
                    if (ht) Fe(ht);
                    else if (Et) {
                      for (var Mt = {}, qt = 0, tn = Et; qt < tn.length; qt += 1) {
                        var xn = tn[qt], An = xn.stack, On = xn.id, Bn = xn.glyph;
                        (Mt[An] || (Mt[An] = {}))[On] = Bn && { id: Bn.id, bitmap: Bn.bitmap.clone(), metrics: Bn.metrics };
                      }
                      Fe(null, Mt);
                    }
                  });
                }, Ee.prototype._doesCharSupportLocalGlyph = function(Ke) {
                  return !!this.localIdeographFontFamily && (ee.isChar["CJK Unified Ideographs"](Ke) || ee.isChar["Hangul Syllables"](Ke) || ee.isChar.Hiragana(Ke) || ee.isChar.Katakana(Ke));
                }, Ee.prototype._tinySDF = function(Ke, Fe, Te) {
                  var Ae = this.localIdeographFontFamily;
                  if (Ae && this._doesCharSupportLocalGlyph(Te)) {
                    var Oe = Ke.tinySDF;
                    if (!Oe) {
                      var Be = "400";
                      /bold/i.test(Fe) ? Be = "900" : /medium/i.test(Fe) ? Be = "500" : /light/i.test(Fe) && (Be = "200"), Oe = Ke.tinySDF = new Ee.TinySDF(24, 3, 8, 0.25, Ae, Be);
                    }
                    return { id: Te, bitmap: new ee.AlphaImage({ width: 30, height: 30 }, Oe.draw(String.fromCharCode(Te))), metrics: { width: 24, height: 24, left: 0, top: -8, advance: 24 } };
                  }
                }, Ee.loadGlyphRange = function(Ke, Fe, Te, Ae, Oe) {
                  var Be = 256 * Fe, Ue = Be + 255, nt = Ae.transformRequest(Ae.normalizeGlyphsURL(Te).replace("{fontstack}", Ke).replace("{range}", Be + "-" + Ue), ee.ResourceType.Glyphs);
                  ee.getArrayBuffer(nt, function(ht, Et) {
                    if (ht) Oe(ht);
                    else if (Et) {
                      for (var Mt = {}, qt = 0, tn = ee.parseGlyphPBF(Et); qt < tn.length; qt += 1) {
                        var xn = tn[qt];
                        Mt[xn.id] = xn;
                      }
                      Oe(null, Mt);
                    }
                  });
                }, Ee.TinySDF = ge;
                var xe = function() {
                  this.specification = ee.styleSpec.light.position;
                };
                xe.prototype.possiblyEvaluate = function(Ke, Fe) {
                  return ee.sphericalToCartesian(Ke.expression.evaluate(Fe));
                }, xe.prototype.interpolate = function(Ke, Fe, Te) {
                  return { x: ee.number(Ke.x, Fe.x, Te), y: ee.number(Ke.y, Fe.y, Te), z: ee.number(Ke.z, Fe.z, Te) };
                };
                var Le = new ee.Properties({ anchor: new ee.DataConstantProperty(ee.styleSpec.light.anchor), position: new xe(), color: new ee.DataConstantProperty(ee.styleSpec.light.color), intensity: new ee.DataConstantProperty(ee.styleSpec.light.intensity) }), Qe = "-transition", ke = function(Ke) {
                  function Fe(Te) {
                    Ke.call(this), this._transitionable = new ee.Transitionable(Le), this.setLight(Te), this._transitioning = this._transitionable.untransitioned();
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getLight = function() {
                    return this._transitionable.serialize();
                  }, Fe.prototype.setLight = function(Te, Ae) {
                    if (Ae === void 0 && (Ae = {}), !this._validate(ee.validateLight, Te, Ae)) for (var Oe in Te) {
                      var Be = Te[Oe];
                      ee.endsWith(Oe, Qe) ? this._transitionable.setTransition(Oe.slice(0, -11), Be) : this._transitionable.setValue(Oe, Be);
                    }
                  }, Fe.prototype.updateTransitions = function(Te) {
                    this._transitioning = this._transitionable.transitioned(Te, this._transitioning);
                  }, Fe.prototype.hasTransition = function() {
                    return this._transitioning.hasTransition();
                  }, Fe.prototype.recalculate = function(Te) {
                    this.properties = this._transitioning.possiblyEvaluate(Te);
                  }, Fe.prototype._validate = function(Te, Ae, Oe) {
                    return (!Oe || Oe.validate !== !1) && ee.emitValidationErrors(this, Te.call(ee.validateStyle, ee.extend({ value: Ae, style: { glyphs: !0, sprite: !0 }, styleSpec: ee.styleSpec })));
                  }, Fe;
                }(ee.Evented), Me = function(Ke, Fe) {
                  this.width = Ke, this.height = Fe, this.nextRow = 0, this.data = new Uint8Array(this.width * this.height), this.dashEntry = {};
                };
                Me.prototype.getDash = function(Ke, Fe) {
                  var Te = Ke.join(",") + String(Fe);
                  return this.dashEntry[Te] || (this.dashEntry[Te] = this.addDash(Ke, Fe)), this.dashEntry[Te];
                }, Me.prototype.getDashRanges = function(Ke, Fe, Te) {
                  var Ae = [], Oe = Ke.length % 2 == 1 ? -Ke[Ke.length - 1] * Te : 0, Be = Ke[0] * Te, Ue = !0;
                  Ae.push({ left: Oe, right: Be, isDash: Ue, zeroLength: Ke[0] === 0 });
                  for (var nt = Ke[0], ht = 1; ht < Ke.length; ht++) {
                    Ue = !Ue;
                    var Et = Ke[ht];
                    Oe = nt * Te, Be = (nt += Et) * Te, Ae.push({ left: Oe, right: Be, isDash: Ue, zeroLength: Et === 0 });
                  }
                  return Ae;
                }, Me.prototype.addRoundDash = function(Ke, Fe, Te) {
                  for (var Ae = Fe / 2, Oe = -Te; Oe <= Te; Oe++) for (var Be = this.nextRow + Te + Oe, Ue = this.width * Be, nt = 0, ht = Ke[nt], Et = 0; Et < this.width; Et++) {
                    Et / ht.right > 1 && (ht = Ke[++nt]);
                    var Mt = Math.abs(Et - ht.left), qt = Math.abs(Et - ht.right), tn = Math.min(Mt, qt), xn = void 0, An = Oe / Te * (Ae + 1);
                    if (ht.isDash) {
                      var On = Ae - Math.abs(An);
                      xn = Math.sqrt(tn * tn + On * On);
                    } else xn = Ae - Math.sqrt(tn * tn + An * An);
                    this.data[Ue + Et] = Math.max(0, Math.min(255, xn + 128));
                  }
                }, Me.prototype.addRegularDash = function(Ke) {
                  for (var Fe = Ke.length - 1; Fe >= 0; --Fe) {
                    var Te = Ke[Fe], Ae = Ke[Fe + 1];
                    Te.zeroLength ? Ke.splice(Fe, 1) : Ae && Ae.isDash === Te.isDash && (Ae.left = Te.left, Ke.splice(Fe, 1));
                  }
                  var Oe = Ke[0], Be = Ke[Ke.length - 1];
                  Oe.isDash === Be.isDash && (Oe.left = Be.left - this.width, Be.right = Oe.right + this.width);
                  for (var Ue = this.width * this.nextRow, nt = 0, ht = Ke[nt], Et = 0; Et < this.width; Et++) {
                    Et / ht.right > 1 && (ht = Ke[++nt]);
                    var Mt = Math.abs(Et - ht.left), qt = Math.abs(Et - ht.right), tn = Math.min(Mt, qt), xn = ht.isDash ? tn : -tn;
                    this.data[Ue + Et] = Math.max(0, Math.min(255, xn + 128));
                  }
                }, Me.prototype.addDash = function(Ke, Fe) {
                  var Te = Fe ? 7 : 0, Ae = 2 * Te + 1;
                  if (this.nextRow + Ae > this.height) return ee.warnOnce("LineAtlas out of space"), null;
                  for (var Oe = 0, Be = 0; Be < Ke.length; Be++) Oe += Ke[Be];
                  if (Oe !== 0) {
                    var Ue = this.width / Oe, nt = this.getDashRanges(Ke, this.width, Ue);
                    Fe ? this.addRoundDash(nt, Ue, Te) : this.addRegularDash(nt);
                  }
                  var ht = { y: (this.nextRow + Te + 0.5) / this.height, height: 2 * Te / this.height, width: Oe };
                  return this.nextRow += Ae, this.dirty = !0, ht;
                }, Me.prototype.bind = function(Ke) {
                  var Fe = Ke.gl;
                  this.texture ? (Fe.bindTexture(Fe.TEXTURE_2D, this.texture), this.dirty && (this.dirty = !1, Fe.texSubImage2D(Fe.TEXTURE_2D, 0, 0, 0, this.width, this.height, Fe.ALPHA, Fe.UNSIGNED_BYTE, this.data))) : (this.texture = Fe.createTexture(), Fe.bindTexture(Fe.TEXTURE_2D, this.texture), Fe.texParameteri(Fe.TEXTURE_2D, Fe.TEXTURE_WRAP_S, Fe.REPEAT), Fe.texParameteri(Fe.TEXTURE_2D, Fe.TEXTURE_WRAP_T, Fe.REPEAT), Fe.texParameteri(Fe.TEXTURE_2D, Fe.TEXTURE_MIN_FILTER, Fe.LINEAR), Fe.texParameteri(Fe.TEXTURE_2D, Fe.TEXTURE_MAG_FILTER, Fe.LINEAR), Fe.texImage2D(Fe.TEXTURE_2D, 0, Fe.ALPHA, this.width, this.height, 0, Fe.ALPHA, Fe.UNSIGNED_BYTE, this.data));
                };
                var Re = function Ke(Fe, Te) {
                  this.workerPool = Fe, this.actors = [], this.currentActor = 0, this.id = ee.uniqueId();
                  for (var Ae = this.workerPool.acquire(this.id), Oe = 0; Oe < Ae.length; Oe++) {
                    var Be = Ae[Oe], Ue = new Ke.Actor(Be, Te, this.id);
                    Ue.name = "Worker " + Oe, this.actors.push(Ue);
                  }
                };
                function De(Ke, Fe, Te) {
                  var Ae = function(Oe, Be) {
                    if (Oe) return Te(Oe);
                    if (Be) {
                      var Ue = ee.pick(ee.extend(Be, Ke), ["tiles", "minzoom", "maxzoom", "attribution", "mapbox_logo", "bounds", "scheme", "tileSize", "encoding"]);
                      Be.vector_layers && (Ue.vectorLayers = Be.vector_layers, Ue.vectorLayerIds = Ue.vectorLayers.map(function(nt) {
                        return nt.id;
                      })), Ue.tiles = Fe.canonicalizeTileset(Ue, Ke.url), Te(null, Ue);
                    }
                  };
                  return Ke.url ? ee.getJSON(Fe.transformRequest(Fe.normalizeSourceURL(Ke.url), ee.ResourceType.Source), Ae) : ee.browser.frame(function() {
                    return Ae(null, Ke);
                  });
                }
                Re.prototype.broadcast = function(Ke, Fe, Te) {
                  Te = Te || function() {
                  }, ee.asyncAll(this.actors, function(Ae, Oe) {
                    Ae.send(Ke, Fe, Oe);
                  }, Te);
                }, Re.prototype.getActor = function() {
                  return this.currentActor = (this.currentActor + 1) % this.actors.length, this.actors[this.currentActor];
                }, Re.prototype.remove = function() {
                  this.actors.forEach(function(Ke) {
                    Ke.remove();
                  }), this.actors = [], this.workerPool.release(this.id);
                }, Re.Actor = ee.Actor;
                var Pe = function(Ke, Fe, Te) {
                  this.bounds = ee.LngLatBounds.convert(this.validateBounds(Ke)), this.minzoom = Fe || 0, this.maxzoom = Te || 24;
                };
                Pe.prototype.validateBounds = function(Ke) {
                  return Array.isArray(Ke) && Ke.length === 4 ? [Math.max(-180, Ke[0]), Math.max(-90, Ke[1]), Math.min(180, Ke[2]), Math.min(90, Ke[3])] : [-180, -90, 180, 90];
                }, Pe.prototype.contains = function(Ke) {
                  var Fe = Math.pow(2, Ke.z), Te = Math.floor(ee.mercatorXfromLng(this.bounds.getWest()) * Fe), Ae = Math.floor(ee.mercatorYfromLat(this.bounds.getNorth()) * Fe), Oe = Math.ceil(ee.mercatorXfromLng(this.bounds.getEast()) * Fe), Be = Math.ceil(ee.mercatorYfromLat(this.bounds.getSouth()) * Fe);
                  return Ke.x >= Te && Ke.x < Oe && Ke.y >= Ae && Ke.y < Be;
                };
                var $e = function(Ke) {
                  function Fe(Te, Ae, Oe, Be) {
                    if (Ke.call(this), this.id = Te, this.dispatcher = Oe, this.type = "vector", this.minzoom = 0, this.maxzoom = 22, this.scheme = "xyz", this.tileSize = 512, this.reparseOverscaled = !0, this.isTileClipped = !0, this._loaded = !1, ee.extend(this, ee.pick(Ae, ["url", "scheme", "tileSize", "promoteId"])), this._options = ee.extend({ type: "vector" }, Ae), this._collectResourceTiming = Ae.collectResourceTiming, this.tileSize !== 512) throw new Error("vector tile sources must have a tileSize of 512");
                    this.setEventedParent(Be);
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.load = function() {
                    var Te = this;
                    this._loaded = !1, this.fire(new ee.Event("dataloading", { dataType: "source" })), this._tileJSONRequest = De(this._options, this.map._requestManager, function(Ae, Oe) {
                      Te._tileJSONRequest = null, Te._loaded = !0, Ae ? Te.fire(new ee.ErrorEvent(Ae)) : Oe && (ee.extend(Te, Oe), Oe.bounds && (Te.tileBounds = new Pe(Oe.bounds, Te.minzoom, Te.maxzoom)), ee.postTurnstileEvent(Oe.tiles, Te.map._requestManager._customAccessToken), ee.postMapLoadEvent(Oe.tiles, Te.map._getMapId(), Te.map._requestManager._skuToken, Te.map._requestManager._customAccessToken), Te.fire(new ee.Event("data", { dataType: "source", sourceDataType: "metadata" })), Te.fire(new ee.Event("data", { dataType: "source", sourceDataType: "content" })));
                    });
                  }, Fe.prototype.loaded = function() {
                    return this._loaded;
                  }, Fe.prototype.hasTile = function(Te) {
                    return !this.tileBounds || this.tileBounds.contains(Te.canonical);
                  }, Fe.prototype.onAdd = function(Te) {
                    this.map = Te, this.load();
                  }, Fe.prototype.setSourceProperty = function(Te) {
                    this._tileJSONRequest && this._tileJSONRequest.cancel(), Te(), this.map.style.sourceCaches[this.id].clearTiles(), this.load();
                  }, Fe.prototype.setTiles = function(Te) {
                    var Ae = this;
                    return this.setSourceProperty(function() {
                      Ae._options.tiles = Te;
                    }), this;
                  }, Fe.prototype.setUrl = function(Te) {
                    var Ae = this;
                    return this.setSourceProperty(function() {
                      Ae.url = Te, Ae._options.url = Te;
                    }), this;
                  }, Fe.prototype.onRemove = function() {
                    this._tileJSONRequest && (this._tileJSONRequest.cancel(), this._tileJSONRequest = null);
                  }, Fe.prototype.serialize = function() {
                    return ee.extend({}, this._options);
                  }, Fe.prototype.loadTile = function(Te, Ae) {
                    var Oe = this.map._requestManager.normalizeTileURL(Te.tileID.canonical.url(this.tiles, this.scheme)), Be = { request: this.map._requestManager.transformRequest(Oe, ee.ResourceType.Tile), uid: Te.uid, tileID: Te.tileID, zoom: Te.tileID.overscaledZ, tileSize: this.tileSize * Te.tileID.overscaleFactor(), type: this.type, source: this.id, pixelRatio: ee.browser.devicePixelRatio, showCollisionBoxes: this.map.showCollisionBoxes, promoteId: this.promoteId };
                    function Ue(nt, ht) {
                      return delete Te.request, Te.aborted ? Ae(null) : nt && nt.status !== 404 ? Ae(nt) : (ht && ht.resourceTiming && (Te.resourceTiming = ht.resourceTiming), this.map._refreshExpiredTiles && ht && Te.setExpiryData(ht), Te.loadVectorData(ht, this.map.painter), ee.cacheEntryPossiblyAdded(this.dispatcher), Ae(null), void (Te.reloadCallback && (this.loadTile(Te, Te.reloadCallback), Te.reloadCallback = null)));
                    }
                    Be.request.collectResourceTiming = this._collectResourceTiming, Te.actor && Te.state !== "expired" ? Te.state === "loading" ? Te.reloadCallback = Ae : Te.request = Te.actor.send("reloadTile", Be, Ue.bind(this)) : (Te.actor = this.dispatcher.getActor(), Te.request = Te.actor.send("loadTile", Be, Ue.bind(this)));
                  }, Fe.prototype.abortTile = function(Te) {
                    Te.request && (Te.request.cancel(), delete Te.request), Te.actor && Te.actor.send("abortTile", { uid: Te.uid, type: this.type, source: this.id }, void 0);
                  }, Fe.prototype.unloadTile = function(Te) {
                    Te.unloadVectorData(), Te.actor && Te.actor.send("removeTile", { uid: Te.uid, type: this.type, source: this.id }, void 0);
                  }, Fe.prototype.hasTransition = function() {
                    return !1;
                  }, Fe;
                }(ee.Evented), qe = function(Ke) {
                  function Fe(Te, Ae, Oe, Be) {
                    Ke.call(this), this.id = Te, this.dispatcher = Oe, this.setEventedParent(Be), this.type = "raster", this.minzoom = 0, this.maxzoom = 22, this.roundZoom = !0, this.scheme = "xyz", this.tileSize = 512, this._loaded = !1, this._options = ee.extend({ type: "raster" }, Ae), ee.extend(this, ee.pick(Ae, ["url", "scheme", "tileSize"]));
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.load = function() {
                    var Te = this;
                    this._loaded = !1, this.fire(new ee.Event("dataloading", { dataType: "source" })), this._tileJSONRequest = De(this._options, this.map._requestManager, function(Ae, Oe) {
                      Te._tileJSONRequest = null, Te._loaded = !0, Ae ? Te.fire(new ee.ErrorEvent(Ae)) : Oe && (ee.extend(Te, Oe), Oe.bounds && (Te.tileBounds = new Pe(Oe.bounds, Te.minzoom, Te.maxzoom)), ee.postTurnstileEvent(Oe.tiles), ee.postMapLoadEvent(Oe.tiles, Te.map._getMapId(), Te.map._requestManager._skuToken), Te.fire(new ee.Event("data", { dataType: "source", sourceDataType: "metadata" })), Te.fire(new ee.Event("data", { dataType: "source", sourceDataType: "content" })));
                    });
                  }, Fe.prototype.loaded = function() {
                    return this._loaded;
                  }, Fe.prototype.onAdd = function(Te) {
                    this.map = Te, this.load();
                  }, Fe.prototype.onRemove = function() {
                    this._tileJSONRequest && (this._tileJSONRequest.cancel(), this._tileJSONRequest = null);
                  }, Fe.prototype.serialize = function() {
                    return ee.extend({}, this._options);
                  }, Fe.prototype.hasTile = function(Te) {
                    return !this.tileBounds || this.tileBounds.contains(Te.canonical);
                  }, Fe.prototype.loadTile = function(Te, Ae) {
                    var Oe = this, Be = this.map._requestManager.normalizeTileURL(Te.tileID.canonical.url(this.tiles, this.scheme), this.tileSize);
                    Te.request = ee.getImage(this.map._requestManager.transformRequest(Be, ee.ResourceType.Tile), function(Ue, nt) {
                      if (delete Te.request, Te.aborted) Te.state = "unloaded", Ae(null);
                      else if (Ue) Te.state = "errored", Ae(Ue);
                      else if (nt) {
                        Oe.map._refreshExpiredTiles && Te.setExpiryData(nt), delete nt.cacheControl, delete nt.expires;
                        var ht = Oe.map.painter.context, Et = ht.gl;
                        Te.texture = Oe.map.painter.getTileTexture(nt.width), Te.texture ? Te.texture.update(nt, { useMipmap: !0 }) : (Te.texture = new ee.Texture(ht, nt, Et.RGBA, { useMipmap: !0 }), Te.texture.bind(Et.LINEAR, Et.CLAMP_TO_EDGE, Et.LINEAR_MIPMAP_NEAREST), ht.extTextureFilterAnisotropic && Et.texParameterf(Et.TEXTURE_2D, ht.extTextureFilterAnisotropic.TEXTURE_MAX_ANISOTROPY_EXT, ht.extTextureFilterAnisotropicMax)), Te.state = "loaded", ee.cacheEntryPossiblyAdded(Oe.dispatcher), Ae(null);
                      }
                    });
                  }, Fe.prototype.abortTile = function(Te, Ae) {
                    Te.request && (Te.request.cancel(), delete Te.request), Ae();
                  }, Fe.prototype.unloadTile = function(Te, Ae) {
                    Te.texture && this.map.painter.saveTileTexture(Te.texture), Ae();
                  }, Fe.prototype.hasTransition = function() {
                    return !1;
                  }, Fe;
                }(ee.Evented), Xe = function(Ke) {
                  function Fe(Te, Ae, Oe, Be) {
                    Ke.call(this, Te, Ae, Oe, Be), this.type = "raster-dem", this.maxzoom = 22, this._options = ee.extend({ type: "raster-dem" }, Ae), this.encoding = Ae.encoding || "mapbox";
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.serialize = function() {
                    return { type: "raster-dem", url: this.url, tileSize: this.tileSize, tiles: this.tiles, bounds: this.bounds, encoding: this.encoding };
                  }, Fe.prototype.loadTile = function(Te, Ae) {
                    var Oe = this.map._requestManager.normalizeTileURL(Te.tileID.canonical.url(this.tiles, this.scheme), this.tileSize);
                    function Be(Ue, nt) {
                      Ue && (Te.state = "errored", Ae(Ue)), nt && (Te.dem = nt, Te.needsHillshadePrepare = !0, Te.state = "loaded", Ae(null));
                    }
                    Te.request = ee.getImage(this.map._requestManager.transformRequest(Oe, ee.ResourceType.Tile), (function(Ue, nt) {
                      if (delete Te.request, Te.aborted) Te.state = "unloaded", Ae(null);
                      else if (Ue) Te.state = "errored", Ae(Ue);
                      else if (nt) {
                        this.map._refreshExpiredTiles && Te.setExpiryData(nt), delete nt.cacheControl, delete nt.expires;
                        var ht = ee.window.ImageBitmap && nt instanceof ee.window.ImageBitmap && ee.offscreenCanvasSupported() ? nt : ee.browser.getImageData(nt, 1), Et = { uid: Te.uid, coord: Te.tileID, source: this.id, rawImageData: ht, encoding: this.encoding };
                        Te.actor && Te.state !== "expired" || (Te.actor = this.dispatcher.getActor(), Te.actor.send("loadDEMTile", Et, Be.bind(this)));
                      }
                    }).bind(this)), Te.neighboringTiles = this._getNeighboringTiles(Te.tileID);
                  }, Fe.prototype._getNeighboringTiles = function(Te) {
                    var Ae = Te.canonical, Oe = Math.pow(2, Ae.z), Be = (Ae.x - 1 + Oe) % Oe, Ue = Ae.x === 0 ? Te.wrap - 1 : Te.wrap, nt = (Ae.x + 1 + Oe) % Oe, ht = Ae.x + 1 === Oe ? Te.wrap + 1 : Te.wrap, Et = {};
                    return Et[new ee.OverscaledTileID(Te.overscaledZ, Ue, Ae.z, Be, Ae.y).key] = { backfilled: !1 }, Et[new ee.OverscaledTileID(Te.overscaledZ, ht, Ae.z, nt, Ae.y).key] = { backfilled: !1 }, Ae.y > 0 && (Et[new ee.OverscaledTileID(Te.overscaledZ, Ue, Ae.z, Be, Ae.y - 1).key] = { backfilled: !1 }, Et[new ee.OverscaledTileID(Te.overscaledZ, Te.wrap, Ae.z, Ae.x, Ae.y - 1).key] = { backfilled: !1 }, Et[new ee.OverscaledTileID(Te.overscaledZ, ht, Ae.z, nt, Ae.y - 1).key] = { backfilled: !1 }), Ae.y + 1 < Oe && (Et[new ee.OverscaledTileID(Te.overscaledZ, Ue, Ae.z, Be, Ae.y + 1).key] = { backfilled: !1 }, Et[new ee.OverscaledTileID(Te.overscaledZ, Te.wrap, Ae.z, Ae.x, Ae.y + 1).key] = { backfilled: !1 }, Et[new ee.OverscaledTileID(Te.overscaledZ, ht, Ae.z, nt, Ae.y + 1).key] = { backfilled: !1 }), Et;
                  }, Fe.prototype.unloadTile = function(Te) {
                    Te.demTexture && this.map.painter.saveTileTexture(Te.demTexture), Te.fbo && (Te.fbo.destroy(), delete Te.fbo), Te.dem && delete Te.dem, delete Te.neighboringTiles, Te.state = "unloaded", Te.actor && Te.actor.send("removeDEMTile", { uid: Te.uid, source: this.id });
                  }, Fe;
                }(qe), Ve = function(Ke) {
                  function Fe(Te, Ae, Oe, Be) {
                    Ke.call(this), this.id = Te, this.type = "geojson", this.minzoom = 0, this.maxzoom = 18, this.tileSize = 512, this.isTileClipped = !0, this.reparseOverscaled = !0, this._removed = !1, this._loaded = !1, this.actor = Oe.getActor(), this.setEventedParent(Be), this._data = Ae.data, this._options = ee.extend({}, Ae), this._collectResourceTiming = Ae.collectResourceTiming, this._resourceTiming = [], Ae.maxzoom !== void 0 && (this.maxzoom = Ae.maxzoom), Ae.type && (this.type = Ae.type), Ae.attribution && (this.attribution = Ae.attribution), this.promoteId = Ae.promoteId;
                    var Ue = ee.EXTENT / this.tileSize;
                    this.workerOptions = ee.extend({ source: this.id, cluster: Ae.cluster || !1, geojsonVtOptions: { buffer: (Ae.buffer !== void 0 ? Ae.buffer : 128) * Ue, tolerance: (Ae.tolerance !== void 0 ? Ae.tolerance : 0.375) * Ue, extent: ee.EXTENT, maxZoom: this.maxzoom, lineMetrics: Ae.lineMetrics || !1, generateId: Ae.generateId || !1 }, superclusterOptions: { maxZoom: Ae.clusterMaxZoom !== void 0 ? Math.min(Ae.clusterMaxZoom, this.maxzoom - 1) : this.maxzoom - 1, minPoints: Math.max(2, Ae.clusterMinPoints || 2), extent: ee.EXTENT, radius: (Ae.clusterRadius || 50) * Ue, log: !1, generateId: Ae.generateId || !1 }, clusterProperties: Ae.clusterProperties, filter: Ae.filter }, Ae.workerOptions);
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.load = function() {
                    var Te = this;
                    this.fire(new ee.Event("dataloading", { dataType: "source" })), this._updateWorkerData(function(Ae) {
                      if (Ae) Te.fire(new ee.ErrorEvent(Ae));
                      else {
                        var Oe = { dataType: "source", sourceDataType: "metadata" };
                        Te._collectResourceTiming && Te._resourceTiming && Te._resourceTiming.length > 0 && (Oe.resourceTiming = Te._resourceTiming, Te._resourceTiming = []), Te.fire(new ee.Event("data", Oe));
                      }
                    });
                  }, Fe.prototype.onAdd = function(Te) {
                    this.map = Te, this.load();
                  }, Fe.prototype.setData = function(Te) {
                    var Ae = this;
                    return this._data = Te, this.fire(new ee.Event("dataloading", { dataType: "source" })), this._updateWorkerData(function(Oe) {
                      if (Oe) Ae.fire(new ee.ErrorEvent(Oe));
                      else {
                        var Be = { dataType: "source", sourceDataType: "content" };
                        Ae._collectResourceTiming && Ae._resourceTiming && Ae._resourceTiming.length > 0 && (Be.resourceTiming = Ae._resourceTiming, Ae._resourceTiming = []), Ae.fire(new ee.Event("data", Be));
                      }
                    }), this;
                  }, Fe.prototype.getClusterExpansionZoom = function(Te, Ae) {
                    return this.actor.send("geojson.getClusterExpansionZoom", { clusterId: Te, source: this.id }, Ae), this;
                  }, Fe.prototype.getClusterChildren = function(Te, Ae) {
                    return this.actor.send("geojson.getClusterChildren", { clusterId: Te, source: this.id }, Ae), this;
                  }, Fe.prototype.getClusterLeaves = function(Te, Ae, Oe, Be) {
                    return this.actor.send("geojson.getClusterLeaves", { source: this.id, clusterId: Te, limit: Ae, offset: Oe }, Be), this;
                  }, Fe.prototype._updateWorkerData = function(Te) {
                    var Ae = this;
                    this._loaded = !1;
                    var Oe = ee.extend({}, this.workerOptions), Be = this._data;
                    typeof Be == "string" ? (Oe.request = this.map._requestManager.transformRequest(ee.browser.resolveURL(Be), ee.ResourceType.Source), Oe.request.collectResourceTiming = this._collectResourceTiming) : Oe.data = JSON.stringify(Be), this.actor.send(this.type + ".loadData", Oe, function(Ue, nt) {
                      Ae._removed || nt && nt.abandoned || (Ae._loaded = !0, nt && nt.resourceTiming && nt.resourceTiming[Ae.id] && (Ae._resourceTiming = nt.resourceTiming[Ae.id].slice(0)), Ae.actor.send(Ae.type + ".coalesce", { source: Oe.source }, null), Te(Ue));
                    });
                  }, Fe.prototype.loaded = function() {
                    return this._loaded;
                  }, Fe.prototype.loadTile = function(Te, Ae) {
                    var Oe = this, Be = Te.actor ? "reloadTile" : "loadTile";
                    Te.actor = this.actor;
                    var Ue = { type: this.type, uid: Te.uid, tileID: Te.tileID, zoom: Te.tileID.overscaledZ, maxZoom: this.maxzoom, tileSize: this.tileSize, source: this.id, pixelRatio: ee.browser.devicePixelRatio, showCollisionBoxes: this.map.showCollisionBoxes, promoteId: this.promoteId };
                    Te.request = this.actor.send(Be, Ue, function(nt, ht) {
                      return delete Te.request, Te.unloadVectorData(), Te.aborted ? Ae(null) : nt ? Ae(nt) : (Te.loadVectorData(ht, Oe.map.painter, Be === "reloadTile"), Ae(null));
                    });
                  }, Fe.prototype.abortTile = function(Te) {
                    Te.request && (Te.request.cancel(), delete Te.request), Te.aborted = !0;
                  }, Fe.prototype.unloadTile = function(Te) {
                    Te.unloadVectorData(), this.actor.send("removeTile", { uid: Te.uid, type: this.type, source: this.id });
                  }, Fe.prototype.onRemove = function() {
                    this._removed = !0, this.actor.send("removeSource", { type: this.type, source: this.id });
                  }, Fe.prototype.serialize = function() {
                    return ee.extend({}, this._options, { type: this.type, data: this._data });
                  }, Fe.prototype.hasTransition = function() {
                    return !1;
                  }, Fe;
                }(ee.Evented), Ze = ee.createLayout([{ name: "a_pos", type: "Int16", components: 2 }, { name: "a_texture_pos", type: "Int16", components: 2 }]), Ge = function(Ke) {
                  function Fe(Te, Ae, Oe, Be) {
                    Ke.call(this), this.id = Te, this.dispatcher = Oe, this.coordinates = Ae.coordinates, this.type = "image", this.minzoom = 0, this.maxzoom = 22, this.tileSize = 512, this.tiles = {}, this._loaded = !1, this.setEventedParent(Be), this.options = Ae;
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.load = function(Te, Ae) {
                    var Oe = this;
                    this._loaded = !1, this.fire(new ee.Event("dataloading", { dataType: "source" })), this.url = this.options.url, ee.getImage(this.map._requestManager.transformRequest(this.url, ee.ResourceType.Image), function(Be, Ue) {
                      Oe._loaded = !0, Be ? Oe.fire(new ee.ErrorEvent(Be)) : Ue && (Oe.image = Ue, Te && (Oe.coordinates = Te), Ae && Ae(), Oe._finishLoading());
                    });
                  }, Fe.prototype.loaded = function() {
                    return this._loaded;
                  }, Fe.prototype.updateImage = function(Te) {
                    var Ae = this;
                    return this.image && Te.url ? (this.options.url = Te.url, this.load(Te.coordinates, function() {
                      Ae.texture = null;
                    }), this) : this;
                  }, Fe.prototype._finishLoading = function() {
                    this.map && (this.setCoordinates(this.coordinates), this.fire(new ee.Event("data", { dataType: "source", sourceDataType: "metadata" })));
                  }, Fe.prototype.onAdd = function(Te) {
                    this.map = Te, this.load();
                  }, Fe.prototype.setCoordinates = function(Te) {
                    var Ae = this;
                    this.coordinates = Te;
                    var Oe = Te.map(ee.MercatorCoordinate.fromLngLat);
                    this.tileID = function(Ue) {
                      for (var nt = 1 / 0, ht = 1 / 0, Et = -1 / 0, Mt = -1 / 0, qt = 0, tn = Ue; qt < tn.length; qt += 1) {
                        var xn = tn[qt];
                        nt = Math.min(nt, xn.x), ht = Math.min(ht, xn.y), Et = Math.max(Et, xn.x), Mt = Math.max(Mt, xn.y);
                      }
                      var An = Et - nt, On = Mt - ht, Bn = Math.max(An, On), qn = Math.max(0, Math.floor(-Math.log(Bn) / Math.LN2)), Vn = Math.pow(2, qn);
                      return new ee.CanonicalTileID(qn, Math.floor((nt + Et) / 2 * Vn), Math.floor((ht + Mt) / 2 * Vn));
                    }(Oe), this.minzoom = this.maxzoom = this.tileID.z;
                    var Be = Oe.map(function(Ue) {
                      return Ae.tileID.getTilePoint(Ue)._round();
                    });
                    return this._boundsArray = new ee.StructArrayLayout4i8(), this._boundsArray.emplaceBack(Be[0].x, Be[0].y, 0, 0), this._boundsArray.emplaceBack(Be[1].x, Be[1].y, ee.EXTENT, 0), this._boundsArray.emplaceBack(Be[3].x, Be[3].y, 0, ee.EXTENT), this._boundsArray.emplaceBack(Be[2].x, Be[2].y, ee.EXTENT, ee.EXTENT), this.boundsBuffer && (this.boundsBuffer.destroy(), delete this.boundsBuffer), this.fire(new ee.Event("data", { dataType: "source", sourceDataType: "content" })), this;
                  }, Fe.prototype.prepare = function() {
                    if (Object.keys(this.tiles).length !== 0 && this.image) {
                      var Te = this.map.painter.context, Ae = Te.gl;
                      for (var Oe in this.boundsBuffer || (this.boundsBuffer = Te.createVertexBuffer(this._boundsArray, Ze.members)), this.boundsSegments || (this.boundsSegments = ee.SegmentVector.simpleSegment(0, 0, 4, 2)), this.texture || (this.texture = new ee.Texture(Te, this.image, Ae.RGBA), this.texture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE)), this.tiles) {
                        var Be = this.tiles[Oe];
                        Be.state !== "loaded" && (Be.state = "loaded", Be.texture = this.texture);
                      }
                    }
                  }, Fe.prototype.loadTile = function(Te, Ae) {
                    this.tileID && this.tileID.equals(Te.tileID.canonical) ? (this.tiles[String(Te.tileID.wrap)] = Te, Te.buckets = {}, Ae(null)) : (Te.state = "errored", Ae(null));
                  }, Fe.prototype.serialize = function() {
                    return { type: "image", url: this.options.url, coordinates: this.coordinates };
                  }, Fe.prototype.hasTransition = function() {
                    return !1;
                  }, Fe;
                }(ee.Evented), ze = function(Ke) {
                  function Fe(Te, Ae, Oe, Be) {
                    Ke.call(this, Te, Ae, Oe, Be), this.roundZoom = !0, this.type = "video", this.options = Ae;
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.load = function() {
                    var Te = this;
                    this._loaded = !1;
                    var Ae = this.options;
                    this.urls = [];
                    for (var Oe = 0, Be = Ae.urls; Oe < Be.length; Oe += 1) {
                      var Ue = Be[Oe];
                      this.urls.push(this.map._requestManager.transformRequest(Ue, ee.ResourceType.Source).url);
                    }
                    ee.getVideo(this.urls, function(nt, ht) {
                      Te._loaded = !0, nt ? Te.fire(new ee.ErrorEvent(nt)) : ht && (Te.video = ht, Te.video.loop = !0, Te.video.addEventListener("playing", function() {
                        Te.map.triggerRepaint();
                      }), Te.map && Te.video.play(), Te._finishLoading());
                    });
                  }, Fe.prototype.pause = function() {
                    this.video && this.video.pause();
                  }, Fe.prototype.play = function() {
                    this.video && this.video.play();
                  }, Fe.prototype.seek = function(Te) {
                    if (this.video) {
                      var Ae = this.video.seekable;
                      Te < Ae.start(0) || Te > Ae.end(0) ? this.fire(new ee.ErrorEvent(new ee.ValidationError("sources." + this.id, null, "Playback for this video can be set only between the " + Ae.start(0) + " and " + Ae.end(0) + "-second mark."))) : this.video.currentTime = Te;
                    }
                  }, Fe.prototype.getVideo = function() {
                    return this.video;
                  }, Fe.prototype.onAdd = function(Te) {
                    this.map || (this.map = Te, this.load(), this.video && (this.video.play(), this.setCoordinates(this.coordinates)));
                  }, Fe.prototype.prepare = function() {
                    if (!(Object.keys(this.tiles).length === 0 || this.video.readyState < 2)) {
                      var Te = this.map.painter.context, Ae = Te.gl;
                      for (var Oe in this.boundsBuffer || (this.boundsBuffer = Te.createVertexBuffer(this._boundsArray, Ze.members)), this.boundsSegments || (this.boundsSegments = ee.SegmentVector.simpleSegment(0, 0, 4, 2)), this.texture ? this.video.paused || (this.texture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE), Ae.texSubImage2D(Ae.TEXTURE_2D, 0, 0, 0, Ae.RGBA, Ae.UNSIGNED_BYTE, this.video)) : (this.texture = new ee.Texture(Te, this.video, Ae.RGBA), this.texture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE)), this.tiles) {
                        var Be = this.tiles[Oe];
                        Be.state !== "loaded" && (Be.state = "loaded", Be.texture = this.texture);
                      }
                    }
                  }, Fe.prototype.serialize = function() {
                    return { type: "video", urls: this.urls, coordinates: this.coordinates };
                  }, Fe.prototype.hasTransition = function() {
                    return this.video && !this.video.paused;
                  }, Fe;
                }(Ge), We = function(Ke) {
                  function Fe(Te, Ae, Oe, Be) {
                    Ke.call(this, Te, Ae, Oe, Be), Ae.coordinates ? Array.isArray(Ae.coordinates) && Ae.coordinates.length === 4 && !Ae.coordinates.some(function(Ue) {
                      return !Array.isArray(Ue) || Ue.length !== 2 || Ue.some(function(nt) {
                        return typeof nt != "number";
                      });
                    }) || this.fire(new ee.ErrorEvent(new ee.ValidationError("sources." + Te, null, '"coordinates" property must be an array of 4 longitude/latitude array pairs'))) : this.fire(new ee.ErrorEvent(new ee.ValidationError("sources." + Te, null, 'missing required property "coordinates"'))), Ae.animate && typeof Ae.animate != "boolean" && this.fire(new ee.ErrorEvent(new ee.ValidationError("sources." + Te, null, 'optional "animate" property must be a boolean value'))), Ae.canvas ? typeof Ae.canvas == "string" || Ae.canvas instanceof ee.window.HTMLCanvasElement || this.fire(new ee.ErrorEvent(new ee.ValidationError("sources." + Te, null, '"canvas" must be either a string representing the ID of the canvas element from which to read, or an HTMLCanvasElement instance'))) : this.fire(new ee.ErrorEvent(new ee.ValidationError("sources." + Te, null, 'missing required property "canvas"'))), this.options = Ae, this.animate = Ae.animate === void 0 || Ae.animate;
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.load = function() {
                    this._loaded = !0, this.canvas || (this.canvas = this.options.canvas instanceof ee.window.HTMLCanvasElement ? this.options.canvas : ee.window.document.getElementById(this.options.canvas)), this.width = this.canvas.width, this.height = this.canvas.height, this._hasInvalidDimensions() ? this.fire(new ee.ErrorEvent(new Error("Canvas dimensions cannot be less than or equal to zero."))) : (this.play = function() {
                      this._playing = !0, this.map.triggerRepaint();
                    }, this.pause = function() {
                      this._playing && (this.prepare(), this._playing = !1);
                    }, this._finishLoading());
                  }, Fe.prototype.getCanvas = function() {
                    return this.canvas;
                  }, Fe.prototype.onAdd = function(Te) {
                    this.map = Te, this.load(), this.canvas && this.animate && this.play();
                  }, Fe.prototype.onRemove = function() {
                    this.pause();
                  }, Fe.prototype.prepare = function() {
                    var Te = !1;
                    if (this.canvas.width !== this.width && (this.width = this.canvas.width, Te = !0), this.canvas.height !== this.height && (this.height = this.canvas.height, Te = !0), !this._hasInvalidDimensions() && Object.keys(this.tiles).length !== 0) {
                      var Ae = this.map.painter.context, Oe = Ae.gl;
                      for (var Be in this.boundsBuffer || (this.boundsBuffer = Ae.createVertexBuffer(this._boundsArray, Ze.members)), this.boundsSegments || (this.boundsSegments = ee.SegmentVector.simpleSegment(0, 0, 4, 2)), this.texture ? (Te || this._playing) && this.texture.update(this.canvas, { premultiply: !0 }) : this.texture = new ee.Texture(Ae, this.canvas, Oe.RGBA, { premultiply: !0 }), this.tiles) {
                        var Ue = this.tiles[Be];
                        Ue.state !== "loaded" && (Ue.state = "loaded", Ue.texture = this.texture);
                      }
                    }
                  }, Fe.prototype.serialize = function() {
                    return { type: "canvas", coordinates: this.coordinates };
                  }, Fe.prototype.hasTransition = function() {
                    return this._playing;
                  }, Fe.prototype._hasInvalidDimensions = function() {
                    for (var Te = 0, Ae = [this.canvas.width, this.canvas.height]; Te < Ae.length; Te += 1) {
                      var Oe = Ae[Te];
                      if (isNaN(Oe) || Oe <= 0) return !0;
                    }
                    return !1;
                  }, Fe;
                }(Ge), at = { vector: $e, raster: qe, "raster-dem": Xe, geojson: Ve, video: ze, image: Ge, canvas: We };
                function ct(Ke, Fe) {
                  var Te = ee.identity([]);
                  return ee.translate(Te, Te, [1, 1, 0]), ee.scale(Te, Te, [0.5 * Ke.width, 0.5 * Ke.height, 1]), ee.multiply(Te, Te, Ke.calculatePosMatrix(Fe.toUnwrapped()));
                }
                function et(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = function(On, Bn, qn) {
                    if (On) for (var Vn = 0, sr = On; Vn < sr.length; Vn += 1) {
                      var Un = Bn[sr[Vn]];
                      if (Un && Un.source === qn && Un.type === "fill-extrusion") return !0;
                    }
                    else for (var wr in Bn) {
                      var Cr = Bn[wr];
                      if (Cr.source === qn && Cr.type === "fill-extrusion") return !0;
                    }
                    return !1;
                  }(Oe && Oe.layers, Fe, Ke.id), nt = Be.maxPitchScaleFactor(), ht = Ke.tilesIn(Ae, nt, Ue);
                  ht.sort(dt);
                  for (var Et = [], Mt = 0, qt = ht; Mt < qt.length; Mt += 1) {
                    var tn = qt[Mt];
                    Et.push({ wrappedTileID: tn.tileID.wrapped().key, queryResults: tn.tile.queryRenderedFeatures(Fe, Te, Ke._state, tn.queryGeometry, tn.cameraQueryGeometry, tn.scale, Oe, Be, nt, ct(Ke.transform, tn.tileID)) });
                  }
                  var xn = function(On) {
                    for (var Bn = {}, qn = {}, Vn = 0, sr = On; Vn < sr.length; Vn += 1) {
                      var Un = sr[Vn], wr = Un.queryResults, Cr = Un.wrappedTileID, Tr = qn[Cr] = qn[Cr] || {};
                      for (var ii in wr) for (var wi = wr[ii], Wi = Tr[ii] = Tr[ii] || {}, Yi = Bn[ii] = Bn[ii] || [], ra = 0, ha = wi; ra < ha.length; ra += 1) {
                        var ya = ha[ra];
                        Wi[ya.featureIndex] || (Wi[ya.featureIndex] = !0, Yi.push(ya));
                      }
                    }
                    return Bn;
                  }(Et);
                  for (var An in xn) xn[An].forEach(function(On) {
                    var Bn = On.feature, qn = Ke.getFeatureState(Bn.layer["source-layer"], Bn.id);
                    Bn.source = Bn.layer.source, Bn.layer["source-layer"] && (Bn.sourceLayer = Bn.layer["source-layer"]), Bn.state = qn;
                  });
                  return xn;
                }
                function dt(Ke, Fe) {
                  var Te = Ke.tileID, Ae = Fe.tileID;
                  return Te.overscaledZ - Ae.overscaledZ || Te.canonical.y - Ae.canonical.y || Te.wrap - Ae.wrap || Te.canonical.x - Ae.canonical.x;
                }
                var Je = function(Ke, Fe) {
                  this.max = Ke, this.onRemove = Fe, this.reset();
                };
                Je.prototype.reset = function() {
                  for (var Ke in this.data) for (var Fe = 0, Te = this.data[Ke]; Fe < Te.length; Fe += 1) {
                    var Ae = Te[Fe];
                    Ae.timeout && clearTimeout(Ae.timeout), this.onRemove(Ae.value);
                  }
                  return this.data = {}, this.order = [], this;
                }, Je.prototype.add = function(Ke, Fe, Te) {
                  var Ae = this, Oe = Ke.wrapped().key;
                  this.data[Oe] === void 0 && (this.data[Oe] = []);
                  var Be = { value: Fe, timeout: void 0 };
                  if (Te !== void 0 && (Be.timeout = setTimeout(function() {
                    Ae.remove(Ke, Be);
                  }, Te)), this.data[Oe].push(Be), this.order.push(Oe), this.order.length > this.max) {
                    var Ue = this._getAndRemoveByKey(this.order[0]);
                    Ue && this.onRemove(Ue);
                  }
                  return this;
                }, Je.prototype.has = function(Ke) {
                  return Ke.wrapped().key in this.data;
                }, Je.prototype.getAndRemove = function(Ke) {
                  return this.has(Ke) ? this._getAndRemoveByKey(Ke.wrapped().key) : null;
                }, Je.prototype._getAndRemoveByKey = function(Ke) {
                  var Fe = this.data[Ke].shift();
                  return Fe.timeout && clearTimeout(Fe.timeout), this.data[Ke].length === 0 && delete this.data[Ke], this.order.splice(this.order.indexOf(Ke), 1), Fe.value;
                }, Je.prototype.getByKey = function(Ke) {
                  var Fe = this.data[Ke];
                  return Fe ? Fe[0].value : null;
                }, Je.prototype.get = function(Ke) {
                  return this.has(Ke) ? this.data[Ke.wrapped().key][0].value : null;
                }, Je.prototype.remove = function(Ke, Fe) {
                  if (!this.has(Ke)) return this;
                  var Te = Ke.wrapped().key, Ae = Fe === void 0 ? 0 : this.data[Te].indexOf(Fe), Oe = this.data[Te][Ae];
                  return this.data[Te].splice(Ae, 1), Oe.timeout && clearTimeout(Oe.timeout), this.data[Te].length === 0 && delete this.data[Te], this.onRemove(Oe.value), this.order.splice(this.order.indexOf(Te), 1), this;
                }, Je.prototype.setMaxSize = function(Ke) {
                  for (this.max = Ke; this.order.length > this.max; ) {
                    var Fe = this._getAndRemoveByKey(this.order[0]);
                    Fe && this.onRemove(Fe);
                  }
                  return this;
                }, Je.prototype.filter = function(Ke) {
                  var Fe = [];
                  for (var Te in this.data) for (var Ae = 0, Oe = this.data[Te]; Ae < Oe.length; Ae += 1) {
                    var Be = Oe[Ae];
                    Ke(Be.value) || Fe.push(Be);
                  }
                  for (var Ue = 0, nt = Fe; Ue < nt.length; Ue += 1) {
                    var ht = nt[Ue];
                    this.remove(ht.value.tileID, ht);
                  }
                };
                var it = function(Ke, Fe, Te) {
                  this.context = Ke;
                  var Ae = Ke.gl;
                  this.buffer = Ae.createBuffer(), this.dynamicDraw = !!Te, this.context.unbindVAO(), Ke.bindElementBuffer.set(this.buffer), Ae.bufferData(Ae.ELEMENT_ARRAY_BUFFER, Fe.arrayBuffer, this.dynamicDraw ? Ae.DYNAMIC_DRAW : Ae.STATIC_DRAW), this.dynamicDraw || delete Fe.arrayBuffer;
                };
                it.prototype.bind = function() {
                  this.context.bindElementBuffer.set(this.buffer);
                }, it.prototype.updateData = function(Ke) {
                  var Fe = this.context.gl;
                  this.context.unbindVAO(), this.bind(), Fe.bufferSubData(Fe.ELEMENT_ARRAY_BUFFER, 0, Ke.arrayBuffer);
                }, it.prototype.destroy = function() {
                  var Ke = this.context.gl;
                  this.buffer && (Ke.deleteBuffer(this.buffer), delete this.buffer);
                };
                var st = { Int8: "BYTE", Uint8: "UNSIGNED_BYTE", Int16: "SHORT", Uint16: "UNSIGNED_SHORT", Int32: "INT", Uint32: "UNSIGNED_INT", Float32: "FLOAT" }, gt = function(Ke, Fe, Te, Ae) {
                  this.length = Fe.length, this.attributes = Te, this.itemSize = Fe.bytesPerElement, this.dynamicDraw = Ae, this.context = Ke;
                  var Oe = Ke.gl;
                  this.buffer = Oe.createBuffer(), Ke.bindVertexBuffer.set(this.buffer), Oe.bufferData(Oe.ARRAY_BUFFER, Fe.arrayBuffer, this.dynamicDraw ? Oe.DYNAMIC_DRAW : Oe.STATIC_DRAW), this.dynamicDraw || delete Fe.arrayBuffer;
                };
                gt.prototype.bind = function() {
                  this.context.bindVertexBuffer.set(this.buffer);
                }, gt.prototype.updateData = function(Ke) {
                  var Fe = this.context.gl;
                  this.bind(), Fe.bufferSubData(Fe.ARRAY_BUFFER, 0, Ke.arrayBuffer);
                }, gt.prototype.enableAttributes = function(Ke, Fe) {
                  for (var Te = 0; Te < this.attributes.length; Te++) {
                    var Ae = this.attributes[Te], Oe = Fe.attributes[Ae.name];
                    Oe !== void 0 && Ke.enableVertexAttribArray(Oe);
                  }
                }, gt.prototype.setVertexAttribPointers = function(Ke, Fe, Te) {
                  for (var Ae = 0; Ae < this.attributes.length; Ae++) {
                    var Oe = this.attributes[Ae], Be = Fe.attributes[Oe.name];
                    Be !== void 0 && Ke.vertexAttribPointer(Be, Oe.components, Ke[st[Oe.type]], !1, this.itemSize, Oe.offset + this.itemSize * (Te || 0));
                  }
                }, gt.prototype.destroy = function() {
                  var Ke = this.context.gl;
                  this.buffer && (Ke.deleteBuffer(this.buffer), delete this.buffer);
                };
                var bt = function(Ke) {
                  this.gl = Ke.gl, this.default = this.getDefault(), this.current = this.default, this.dirty = !1;
                };
                bt.prototype.get = function() {
                  return this.current;
                }, bt.prototype.set = function(Ke) {
                }, bt.prototype.getDefault = function() {
                  return this.default;
                }, bt.prototype.setDefault = function() {
                  this.set(this.default);
                };
                var yt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return ee.Color.transparent;
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te.r !== Ae.r || Te.g !== Ae.g || Te.b !== Ae.b || Te.a !== Ae.a || this.dirty) && (this.gl.clearColor(Te.r, Te.g, Te.b, Te.a), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), St = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return 1;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.clearDepth(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Lt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return 0;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.clearStencil(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Rt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return [!0, !0, !0, !0];
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || Te[2] !== Ae[2] || Te[3] !== Ae[3] || this.dirty) && (this.gl.colorMask(Te[0], Te[1], Te[2], Te[3]), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), vt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return !0;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.depthMask(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), $t = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return 255;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.stencilMask(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Pt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return { func: this.gl.ALWAYS, ref: 0, mask: 255 };
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te.func !== Ae.func || Te.ref !== Ae.ref || Te.mask !== Ae.mask || this.dirty) && (this.gl.stencilFunc(Te.func, Te.ref, Te.mask), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Xt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    var Te = this.gl;
                    return [Te.KEEP, Te.KEEP, Te.KEEP];
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || Te[2] !== Ae[2] || this.dirty) && (this.gl.stencilOp(Te[0], Te[1], Te[2]), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Nt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return !1;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Te ? Ae.enable(Ae.STENCIL_TEST) : Ae.disable(Ae.STENCIL_TEST), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), Gt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return [0, 1];
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || this.dirty) && (this.gl.depthRange(Te[0], Te[1]), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Ft = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return !1;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Te ? Ae.enable(Ae.DEPTH_TEST) : Ae.disable(Ae.DEPTH_TEST), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), Tt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return this.gl.LESS;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.depthFunc(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), It = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return !1;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Te ? Ae.enable(Ae.BLEND) : Ae.disable(Ae.BLEND), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), Ct = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    var Te = this.gl;
                    return [Te.ONE, Te.ZERO];
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || this.dirty) && (this.gl.blendFunc(Te[0], Te[1]), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Kt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return ee.Color.transparent;
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te.r !== Ae.r || Te.g !== Ae.g || Te.b !== Ae.b || Te.a !== Ae.a || this.dirty) && (this.gl.blendColor(Te.r, Te.g, Te.b, Te.a), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Vt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return this.gl.FUNC_ADD;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.blendEquation(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Yt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return !1;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Te ? Ae.enable(Ae.CULL_FACE) : Ae.disable(Ae.CULL_FACE), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), sn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return this.gl.BACK;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.cullFace(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), hn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return this.gl.CCW;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.frontFace(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Qt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.useProgram(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Ut = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return this.gl.TEXTURE0;
                  }, Fe.prototype.set = function(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.activeTexture(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), Wt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    var Te = this.gl;
                    return [0, 0, Te.drawingBufferWidth, Te.drawingBufferHeight];
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || Te[2] !== Ae[2] || Te[3] !== Ae[3] || this.dirty) && (this.gl.viewport(Te[0], Te[1], Te[2], Te[3]), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), rn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Ae.bindFramebuffer(Ae.FRAMEBUFFER, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), vn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Ae.bindRenderbuffer(Ae.RENDERBUFFER, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), dn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Ae.bindTexture(Ae.TEXTURE_2D, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), Ht = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Ae.bindBuffer(Ae.ARRAY_BUFFER, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), jt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe.prototype.set = function(Te) {
                    var Ae = this.gl;
                    Ae.bindBuffer(Ae.ELEMENT_ARRAY_BUFFER, Te), this.current = Te, this.dirty = !1;
                  }, Fe;
                }(bt), cn = function(Ke) {
                  function Fe(Te) {
                    Ke.call(this, Te), this.vao = Te.extVertexArrayObject;
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe.prototype.set = function(Te) {
                    this.vao && (Te !== this.current || this.dirty) && (this.vao.bindVertexArrayOES(Te), this.current = Te, this.dirty = !1);
                  }, Fe;
                }(bt), wn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return 4;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Ae.pixelStorei(Ae.UNPACK_ALIGNMENT, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), Rn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return !1;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Ae.pixelStorei(Ae.UNPACK_PREMULTIPLY_ALPHA_WEBGL, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), Pn = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return !1;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      var Ae = this.gl;
                      Ae.pixelStorei(Ae.UNPACK_FLIP_Y_WEBGL, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(bt), Ie = function(Ke) {
                  function Fe(Te, Ae) {
                    Ke.call(this, Te), this.context = Te, this.parent = Ae;
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getDefault = function() {
                    return null;
                  }, Fe;
                }(bt), rt = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.setDirty = function() {
                    this.dirty = !0;
                  }, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      this.context.bindFramebuffer.set(this.parent);
                      var Ae = this.gl;
                      Ae.framebufferTexture2D(Ae.FRAMEBUFFER, Ae.COLOR_ATTACHMENT0, Ae.TEXTURE_2D, Te, 0), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(Ie), ft = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.set = function(Te) {
                    if (Te !== this.current || this.dirty) {
                      this.context.bindFramebuffer.set(this.parent);
                      var Ae = this.gl;
                      Ae.framebufferRenderbuffer(Ae.FRAMEBUFFER, Ae.DEPTH_ATTACHMENT, Ae.RENDERBUFFER, Te), this.current = Te, this.dirty = !1;
                    }
                  }, Fe;
                }(Ie), pt = function(Ke, Fe, Te, Ae) {
                  this.context = Ke, this.width = Fe, this.height = Te;
                  var Oe = Ke.gl, Be = this.framebuffer = Oe.createFramebuffer();
                  this.colorAttachment = new rt(Ke, Be), Ae && (this.depthAttachment = new ft(Ke, Be));
                };
                pt.prototype.destroy = function() {
                  var Ke = this.context.gl, Fe = this.colorAttachment.get();
                  if (Fe && Ke.deleteTexture(Fe), this.depthAttachment) {
                    var Te = this.depthAttachment.get();
                    Te && Ke.deleteRenderbuffer(Te);
                  }
                  Ke.deleteFramebuffer(this.framebuffer);
                };
                var wt = function(Ke, Fe, Te) {
                  this.func = Ke, this.mask = Fe, this.range = Te;
                };
                wt.ReadOnly = !1, wt.ReadWrite = !0, wt.disabled = new wt(519, wt.ReadOnly, [0, 1]);
                var Bt = 7680, At = function(Ke, Fe, Te, Ae, Oe, Be) {
                  this.test = Ke, this.ref = Fe, this.mask = Te, this.fail = Ae, this.depthFail = Oe, this.pass = Be;
                };
                At.disabled = new At({ func: 519, mask: 0 }, 0, 0, Bt, Bt, Bt);
                var Dt = function(Ke, Fe, Te) {
                  this.blendFunction = Ke, this.blendColor = Fe, this.mask = Te;
                };
                Dt.Replace = [1, 0], Dt.disabled = new Dt(Dt.Replace, ee.Color.transparent, [!1, !1, !1, !1]), Dt.unblended = new Dt(Dt.Replace, ee.Color.transparent, [!0, !0, !0, !0]), Dt.alphaBlended = new Dt([1, 771], ee.Color.transparent, [!0, !0, !0, !0]);
                var Ot = function(Ke, Fe, Te) {
                  this.enable = Ke, this.mode = Fe, this.frontFace = Te;
                };
                Ot.disabled = new Ot(!1, 1029, 2305), Ot.backCCW = new Ot(!0, 1029, 2305);
                var un = function(Ke) {
                  this.gl = Ke, this.extVertexArrayObject = this.gl.getExtension("OES_vertex_array_object"), this.clearColor = new yt(this), this.clearDepth = new St(this), this.clearStencil = new Lt(this), this.colorMask = new Rt(this), this.depthMask = new vt(this), this.stencilMask = new $t(this), this.stencilFunc = new Pt(this), this.stencilOp = new Xt(this), this.stencilTest = new Nt(this), this.depthRange = new Gt(this), this.depthTest = new Ft(this), this.depthFunc = new Tt(this), this.blend = new It(this), this.blendFunc = new Ct(this), this.blendColor = new Kt(this), this.blendEquation = new Vt(this), this.cullFace = new Yt(this), this.cullFaceSide = new sn(this), this.frontFace = new hn(this), this.program = new Qt(this), this.activeTexture = new Ut(this), this.viewport = new Wt(this), this.bindFramebuffer = new rn(this), this.bindRenderbuffer = new vn(this), this.bindTexture = new dn(this), this.bindVertexBuffer = new Ht(this), this.bindElementBuffer = new jt(this), this.bindVertexArrayOES = this.extVertexArrayObject && new cn(this), this.pixelStoreUnpack = new wn(this), this.pixelStoreUnpackPremultiplyAlpha = new Rn(this), this.pixelStoreUnpackFlipY = new Pn(this), this.extTextureFilterAnisotropic = Ke.getExtension("EXT_texture_filter_anisotropic") || Ke.getExtension("MOZ_EXT_texture_filter_anisotropic") || Ke.getExtension("WEBKIT_EXT_texture_filter_anisotropic"), this.extTextureFilterAnisotropic && (this.extTextureFilterAnisotropicMax = Ke.getParameter(this.extTextureFilterAnisotropic.MAX_TEXTURE_MAX_ANISOTROPY_EXT)), this.extTextureHalfFloat = Ke.getExtension("OES_texture_half_float"), this.extTextureHalfFloat && (Ke.getExtension("OES_texture_half_float_linear"), this.extRenderToTextureHalfFloat = Ke.getExtension("EXT_color_buffer_half_float")), this.extTimerQuery = Ke.getExtension("EXT_disjoint_timer_query"), this.maxTextureSize = Ke.getParameter(Ke.MAX_TEXTURE_SIZE);
                };
                un.prototype.setDefault = function() {
                  this.unbindVAO(), this.clearColor.setDefault(), this.clearDepth.setDefault(), this.clearStencil.setDefault(), this.colorMask.setDefault(), this.depthMask.setDefault(), this.stencilMask.setDefault(), this.stencilFunc.setDefault(), this.stencilOp.setDefault(), this.stencilTest.setDefault(), this.depthRange.setDefault(), this.depthTest.setDefault(), this.depthFunc.setDefault(), this.blend.setDefault(), this.blendFunc.setDefault(), this.blendColor.setDefault(), this.blendEquation.setDefault(), this.cullFace.setDefault(), this.cullFaceSide.setDefault(), this.frontFace.setDefault(), this.program.setDefault(), this.activeTexture.setDefault(), this.bindFramebuffer.setDefault(), this.pixelStoreUnpack.setDefault(), this.pixelStoreUnpackPremultiplyAlpha.setDefault(), this.pixelStoreUnpackFlipY.setDefault();
                }, un.prototype.setDirty = function() {
                  this.clearColor.dirty = !0, this.clearDepth.dirty = !0, this.clearStencil.dirty = !0, this.colorMask.dirty = !0, this.depthMask.dirty = !0, this.stencilMask.dirty = !0, this.stencilFunc.dirty = !0, this.stencilOp.dirty = !0, this.stencilTest.dirty = !0, this.depthRange.dirty = !0, this.depthTest.dirty = !0, this.depthFunc.dirty = !0, this.blend.dirty = !0, this.blendFunc.dirty = !0, this.blendColor.dirty = !0, this.blendEquation.dirty = !0, this.cullFace.dirty = !0, this.cullFaceSide.dirty = !0, this.frontFace.dirty = !0, this.program.dirty = !0, this.activeTexture.dirty = !0, this.viewport.dirty = !0, this.bindFramebuffer.dirty = !0, this.bindRenderbuffer.dirty = !0, this.bindTexture.dirty = !0, this.bindVertexBuffer.dirty = !0, this.bindElementBuffer.dirty = !0, this.extVertexArrayObject && (this.bindVertexArrayOES.dirty = !0), this.pixelStoreUnpack.dirty = !0, this.pixelStoreUnpackPremultiplyAlpha.dirty = !0, this.pixelStoreUnpackFlipY.dirty = !0;
                }, un.prototype.createIndexBuffer = function(Ke, Fe) {
                  return new it(this, Ke, Fe);
                }, un.prototype.createVertexBuffer = function(Ke, Fe, Te) {
                  return new gt(this, Ke, Fe, Te);
                }, un.prototype.createRenderbuffer = function(Ke, Fe, Te) {
                  var Ae = this.gl, Oe = Ae.createRenderbuffer();
                  return this.bindRenderbuffer.set(Oe), Ae.renderbufferStorage(Ae.RENDERBUFFER, Ke, Fe, Te), this.bindRenderbuffer.set(null), Oe;
                }, un.prototype.createFramebuffer = function(Ke, Fe, Te) {
                  return new pt(this, Ke, Fe, Te);
                }, un.prototype.clear = function(Ke) {
                  var Fe = Ke.color, Te = Ke.depth, Ae = this.gl, Oe = 0;
                  Fe && (Oe |= Ae.COLOR_BUFFER_BIT, this.clearColor.set(Fe), this.colorMask.set([!0, !0, !0, !0])), Te !== void 0 && (Oe |= Ae.DEPTH_BUFFER_BIT, this.depthRange.set([0, 1]), this.clearDepth.set(Te), this.depthMask.set(!0)), Ae.clear(Oe);
                }, un.prototype.setCullFace = function(Ke) {
                  Ke.enable === !1 ? this.cullFace.set(!1) : (this.cullFace.set(!0), this.cullFaceSide.set(Ke.mode), this.frontFace.set(Ke.frontFace));
                }, un.prototype.setDepthMode = function(Ke) {
                  Ke.func !== this.gl.ALWAYS || Ke.mask ? (this.depthTest.set(!0), this.depthFunc.set(Ke.func), this.depthMask.set(Ke.mask), this.depthRange.set(Ke.range)) : this.depthTest.set(!1);
                }, un.prototype.setStencilMode = function(Ke) {
                  Ke.test.func !== this.gl.ALWAYS || Ke.mask ? (this.stencilTest.set(!0), this.stencilMask.set(Ke.mask), this.stencilOp.set([Ke.fail, Ke.depthFail, Ke.pass]), this.stencilFunc.set({ func: Ke.test.func, ref: Ke.ref, mask: Ke.test.mask })) : this.stencilTest.set(!1);
                }, un.prototype.setColorMode = function(Ke) {
                  ee.deepEqual(Ke.blendFunction, Dt.Replace) ? this.blend.set(!1) : (this.blend.set(!0), this.blendFunc.set(Ke.blendFunction), this.blendColor.set(Ke.blendColor)), this.colorMask.set(Ke.mask);
                }, un.prototype.unbindVAO = function() {
                  this.extVertexArrayObject && this.bindVertexArrayOES.set(null);
                };
                var yn = function(Ke) {
                  function Fe(Te, Ae, Oe) {
                    var Be = this;
                    Ke.call(this), this.id = Te, this.dispatcher = Oe, this.on("data", function(Ue) {
                      Ue.dataType === "source" && Ue.sourceDataType === "metadata" && (Be._sourceLoaded = !0), Be._sourceLoaded && !Be._paused && Ue.dataType === "source" && Ue.sourceDataType === "content" && (Be.reload(), Be.transform && Be.update(Be.transform));
                    }), this.on("error", function() {
                      Be._sourceErrored = !0;
                    }), this._source = function(Ue, nt, ht, Et) {
                      var Mt = new at[nt.type](Ue, nt, ht, Et);
                      if (Mt.id !== Ue) throw new Error("Expected Source id to be " + Ue + " instead of " + Mt.id);
                      return ee.bindAll(["load", "abort", "unload", "serialize", "prepare"], Mt), Mt;
                    }(Te, Ae, Oe, this), this._tiles = {}, this._cache = new Je(0, this._unloadTile.bind(this)), this._timers = {}, this._cacheTimers = {}, this._maxTileCacheSize = null, this._loadedParentTiles = {}, this._coveredTiles = {}, this._state = new ee.SourceFeatureState();
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.onAdd = function(Te) {
                    this.map = Te, this._maxTileCacheSize = Te ? Te._maxTileCacheSize : null, this._source && this._source.onAdd && this._source.onAdd(Te);
                  }, Fe.prototype.onRemove = function(Te) {
                    this._source && this._source.onRemove && this._source.onRemove(Te);
                  }, Fe.prototype.loaded = function() {
                    if (this._sourceErrored) return !0;
                    if (!this._sourceLoaded || !this._source.loaded()) return !1;
                    for (var Te in this._tiles) {
                      var Ae = this._tiles[Te];
                      if (Ae.state !== "loaded" && Ae.state !== "errored") return !1;
                    }
                    return !0;
                  }, Fe.prototype.getSource = function() {
                    return this._source;
                  }, Fe.prototype.pause = function() {
                    this._paused = !0;
                  }, Fe.prototype.resume = function() {
                    if (this._paused) {
                      var Te = this._shouldReloadOnResume;
                      this._paused = !1, this._shouldReloadOnResume = !1, Te && this.reload(), this.transform && this.update(this.transform);
                    }
                  }, Fe.prototype._loadTile = function(Te, Ae) {
                    return this._source.loadTile(Te, Ae);
                  }, Fe.prototype._unloadTile = function(Te) {
                    if (this._source.unloadTile) return this._source.unloadTile(Te, function() {
                    });
                  }, Fe.prototype._abortTile = function(Te) {
                    if (this._source.abortTile) return this._source.abortTile(Te, function() {
                    });
                  }, Fe.prototype.serialize = function() {
                    return this._source.serialize();
                  }, Fe.prototype.prepare = function(Te) {
                    for (var Ae in this._source.prepare && this._source.prepare(), this._state.coalesceChanges(this._tiles, this.map ? this.map.painter : null), this._tiles) {
                      var Oe = this._tiles[Ae];
                      Oe.upload(Te), Oe.prepare(this.map.style.imageManager);
                    }
                  }, Fe.prototype.getIds = function() {
                    return ee.values(this._tiles).map(function(Te) {
                      return Te.tileID;
                    }).sort(_n).map(function(Te) {
                      return Te.key;
                    });
                  }, Fe.prototype.getRenderableIds = function(Te) {
                    var Ae = this, Oe = [];
                    for (var Be in this._tiles) this._isIdRenderable(Be, Te) && Oe.push(this._tiles[Be]);
                    return Te ? Oe.sort(function(Ue, nt) {
                      var ht = Ue.tileID, Et = nt.tileID, Mt = new ee.Point(ht.canonical.x, ht.canonical.y)._rotate(Ae.transform.angle), qt = new ee.Point(Et.canonical.x, Et.canonical.y)._rotate(Ae.transform.angle);
                      return ht.overscaledZ - Et.overscaledZ || qt.y - Mt.y || qt.x - Mt.x;
                    }).map(function(Ue) {
                      return Ue.tileID.key;
                    }) : Oe.map(function(Ue) {
                      return Ue.tileID;
                    }).sort(_n).map(function(Ue) {
                      return Ue.key;
                    });
                  }, Fe.prototype.hasRenderableParent = function(Te) {
                    var Ae = this.findLoadedParent(Te, 0);
                    return !!Ae && this._isIdRenderable(Ae.tileID.key);
                  }, Fe.prototype._isIdRenderable = function(Te, Ae) {
                    return this._tiles[Te] && this._tiles[Te].hasData() && !this._coveredTiles[Te] && (Ae || !this._tiles[Te].holdingForFade());
                  }, Fe.prototype.reload = function() {
                    if (this._paused) this._shouldReloadOnResume = !0;
                    else for (var Te in this._cache.reset(), this._tiles) this._tiles[Te].state !== "errored" && this._reloadTile(Te, "reloading");
                  }, Fe.prototype._reloadTile = function(Te, Ae) {
                    var Oe = this._tiles[Te];
                    Oe && (Oe.state !== "loading" && (Oe.state = Ae), this._loadTile(Oe, this._tileLoaded.bind(this, Oe, Te, Ae)));
                  }, Fe.prototype._tileLoaded = function(Te, Ae, Oe, Be) {
                    if (Be) return Te.state = "errored", void (Be.status !== 404 ? this._source.fire(new ee.ErrorEvent(Be, { tile: Te })) : this.update(this.transform));
                    Te.timeAdded = ee.browser.now(), Oe === "expired" && (Te.refreshedUponExpiration = !0), this._setTileReloadTimer(Ae, Te), this.getSource().type === "raster-dem" && Te.dem && this._backfillDEM(Te), this._state.initializeTileState(Te, this.map ? this.map.painter : null), this._source.fire(new ee.Event("data", { dataType: "source", tile: Te, coord: Te.tileID }));
                  }, Fe.prototype._backfillDEM = function(Te) {
                    for (var Ae = this.getRenderableIds(), Oe = 0; Oe < Ae.length; Oe++) {
                      var Be = Ae[Oe];
                      if (Te.neighboringTiles && Te.neighboringTiles[Be]) {
                        var Ue = this.getTileByID(Be);
                        nt(Te, Ue), nt(Ue, Te);
                      }
                    }
                    function nt(ht, Et) {
                      ht.needsHillshadePrepare = !0;
                      var Mt = Et.tileID.canonical.x - ht.tileID.canonical.x, qt = Et.tileID.canonical.y - ht.tileID.canonical.y, tn = Math.pow(2, ht.tileID.canonical.z), xn = Et.tileID.key;
                      Mt === 0 && qt === 0 || Math.abs(qt) > 1 || (Math.abs(Mt) > 1 && (Math.abs(Mt + tn) === 1 ? Mt += tn : Math.abs(Mt - tn) === 1 && (Mt -= tn)), Et.dem && ht.dem && (ht.dem.backfillBorder(Et.dem, Mt, qt), ht.neighboringTiles && ht.neighboringTiles[xn] && (ht.neighboringTiles[xn].backfilled = !0)));
                    }
                  }, Fe.prototype.getTile = function(Te) {
                    return this.getTileByID(Te.key);
                  }, Fe.prototype.getTileByID = function(Te) {
                    return this._tiles[Te];
                  }, Fe.prototype._retainLoadedChildren = function(Te, Ae, Oe, Be) {
                    for (var Ue in this._tiles) {
                      var nt = this._tiles[Ue];
                      if (!(Be[Ue] || !nt.hasData() || nt.tileID.overscaledZ <= Ae || nt.tileID.overscaledZ > Oe)) {
                        for (var ht = nt.tileID; nt && nt.tileID.overscaledZ > Ae + 1; ) {
                          var Et = nt.tileID.scaledTo(nt.tileID.overscaledZ - 1);
                          (nt = this._tiles[Et.key]) && nt.hasData() && (ht = Et);
                        }
                        for (var Mt = ht; Mt.overscaledZ > Ae; ) if (Te[(Mt = Mt.scaledTo(Mt.overscaledZ - 1)).key]) {
                          Be[ht.key] = ht;
                          break;
                        }
                      }
                    }
                  }, Fe.prototype.findLoadedParent = function(Te, Ae) {
                    if (Te.key in this._loadedParentTiles) {
                      var Oe = this._loadedParentTiles[Te.key];
                      return Oe && Oe.tileID.overscaledZ >= Ae ? Oe : null;
                    }
                    for (var Be = Te.overscaledZ - 1; Be >= Ae; Be--) {
                      var Ue = Te.scaledTo(Be), nt = this._getLoadedTile(Ue);
                      if (nt) return nt;
                    }
                  }, Fe.prototype._getLoadedTile = function(Te) {
                    var Ae = this._tiles[Te.key];
                    return Ae && Ae.hasData() ? Ae : this._cache.getByKey(Te.wrapped().key);
                  }, Fe.prototype.updateCacheSize = function(Te) {
                    var Ae = (Math.ceil(Te.width / this._source.tileSize) + 1) * (Math.ceil(Te.height / this._source.tileSize) + 1), Oe = Math.floor(5 * Ae), Be = typeof this._maxTileCacheSize == "number" ? Math.min(this._maxTileCacheSize, Oe) : Oe;
                    this._cache.setMaxSize(Be);
                  }, Fe.prototype.handleWrapJump = function(Te) {
                    var Ae = (Te - (this._prevLng === void 0 ? Te : this._prevLng)) / 360, Oe = Math.round(Ae);
                    if (this._prevLng = Te, Oe) {
                      var Be = {};
                      for (var Ue in this._tiles) {
                        var nt = this._tiles[Ue];
                        nt.tileID = nt.tileID.unwrapTo(nt.tileID.wrap + Oe), Be[nt.tileID.key] = nt;
                      }
                      for (var ht in this._tiles = Be, this._timers) clearTimeout(this._timers[ht]), delete this._timers[ht];
                      for (var Et in this._tiles) {
                        var Mt = this._tiles[Et];
                        this._setTileReloadTimer(Et, Mt);
                      }
                    }
                  }, Fe.prototype.update = function(Te) {
                    var Ae = this;
                    if (this.transform = Te, this._sourceLoaded && !this._paused) {
                      var Oe;
                      this.updateCacheSize(Te), this.handleWrapJump(this.transform.center.lng), this._coveredTiles = {}, this.used ? this._source.tileID ? Oe = Te.getVisibleUnwrappedCoordinates(this._source.tileID).map(function(Tr) {
                        return new ee.OverscaledTileID(Tr.canonical.z, Tr.wrap, Tr.canonical.z, Tr.canonical.x, Tr.canonical.y);
                      }) : (Oe = Te.coveringTiles({ tileSize: this._source.tileSize, minzoom: this._source.minzoom, maxzoom: this._source.maxzoom, roundZoom: this._source.roundZoom, reparseOverscaled: this._source.reparseOverscaled }), this._source.hasTile && (Oe = Oe.filter(function(Tr) {
                        return Ae._source.hasTile(Tr);
                      }))) : Oe = [];
                      var Be = Te.coveringZoomLevel(this._source), Ue = Math.max(Be - Fe.maxOverzooming, this._source.minzoom), nt = Math.max(Be + Fe.maxUnderzooming, this._source.minzoom), ht = this._updateRetainedTiles(Oe, Be);
                      if (gn(this._source.type)) {
                        for (var Et = {}, Mt = {}, qt = 0, tn = Object.keys(ht); qt < tn.length; qt += 1) {
                          var xn = tn[qt], An = ht[xn], On = this._tiles[xn];
                          if (On && !(On.fadeEndTime && On.fadeEndTime <= ee.browser.now())) {
                            var Bn = this.findLoadedParent(An, Ue);
                            Bn && (this._addTile(Bn.tileID), Et[Bn.tileID.key] = Bn.tileID), Mt[xn] = An;
                          }
                        }
                        for (var qn in this._retainLoadedChildren(Mt, Be, nt, ht), Et) ht[qn] || (this._coveredTiles[qn] = !0, ht[qn] = Et[qn]);
                      }
                      for (var Vn in ht) this._tiles[Vn].clearFadeHold();
                      for (var sr = 0, Un = ee.keysDifference(this._tiles, ht); sr < Un.length; sr += 1) {
                        var wr = Un[sr], Cr = this._tiles[wr];
                        Cr.hasSymbolBuckets && !Cr.holdingForFade() ? Cr.setHoldDuration(this.map._fadeDuration) : Cr.hasSymbolBuckets && !Cr.symbolFadeFinished() || this._removeTile(wr);
                      }
                      this._updateLoadedParentTileCache();
                    }
                  }, Fe.prototype.releaseSymbolFadeTiles = function() {
                    for (var Te in this._tiles) this._tiles[Te].holdingForFade() && this._removeTile(Te);
                  }, Fe.prototype._updateRetainedTiles = function(Te, Ae) {
                    for (var Oe = {}, Be = {}, Ue = Math.max(Ae - Fe.maxOverzooming, this._source.minzoom), nt = Math.max(Ae + Fe.maxUnderzooming, this._source.minzoom), ht = {}, Et = 0, Mt = Te; Et < Mt.length; Et += 1) {
                      var qt = Mt[Et], tn = this._addTile(qt);
                      Oe[qt.key] = qt, tn.hasData() || Ae < this._source.maxzoom && (ht[qt.key] = qt);
                    }
                    this._retainLoadedChildren(ht, Ae, nt, Oe);
                    for (var xn = 0, An = Te; xn < An.length; xn += 1) {
                      var On = An[xn], Bn = this._tiles[On.key];
                      if (!Bn.hasData()) {
                        if (Ae + 1 > this._source.maxzoom) {
                          var qn = On.children(this._source.maxzoom)[0], Vn = this.getTile(qn);
                          if (Vn && Vn.hasData()) {
                            Oe[qn.key] = qn;
                            continue;
                          }
                        } else {
                          var sr = On.children(this._source.maxzoom);
                          if (Oe[sr[0].key] && Oe[sr[1].key] && Oe[sr[2].key] && Oe[sr[3].key]) continue;
                        }
                        for (var Un = Bn.wasRequested(), wr = On.overscaledZ - 1; wr >= Ue; --wr) {
                          var Cr = On.scaledTo(wr);
                          if (Be[Cr.key] || (Be[Cr.key] = !0, !(Bn = this.getTile(Cr)) && Un && (Bn = this._addTile(Cr)), Bn && (Oe[Cr.key] = Cr, Un = Bn.wasRequested(), Bn.hasData()))) break;
                        }
                      }
                    }
                    return Oe;
                  }, Fe.prototype._updateLoadedParentTileCache = function() {
                    for (var Te in this._loadedParentTiles = {}, this._tiles) {
                      for (var Ae = [], Oe = void 0, Be = this._tiles[Te].tileID; Be.overscaledZ > 0; ) {
                        if (Be.key in this._loadedParentTiles) {
                          Oe = this._loadedParentTiles[Be.key];
                          break;
                        }
                        Ae.push(Be.key);
                        var Ue = Be.scaledTo(Be.overscaledZ - 1);
                        if (Oe = this._getLoadedTile(Ue)) break;
                        Be = Ue;
                      }
                      for (var nt = 0, ht = Ae; nt < ht.length; nt += 1) {
                        var Et = ht[nt];
                        this._loadedParentTiles[Et] = Oe;
                      }
                    }
                  }, Fe.prototype._addTile = function(Te) {
                    var Ae = this._tiles[Te.key];
                    if (Ae) return Ae;
                    (Ae = this._cache.getAndRemove(Te)) && (this._setTileReloadTimer(Te.key, Ae), Ae.tileID = Te, this._state.initializeTileState(Ae, this.map ? this.map.painter : null), this._cacheTimers[Te.key] && (clearTimeout(this._cacheTimers[Te.key]), delete this._cacheTimers[Te.key], this._setTileReloadTimer(Te.key, Ae)));
                    var Oe = !!Ae;
                    return Oe || (Ae = new ee.Tile(Te, this._source.tileSize * Te.overscaleFactor()), this._loadTile(Ae, this._tileLoaded.bind(this, Ae, Te.key, Ae.state))), Ae ? (Ae.uses++, this._tiles[Te.key] = Ae, Oe || this._source.fire(new ee.Event("dataloading", { tile: Ae, coord: Ae.tileID, dataType: "source" })), Ae) : null;
                  }, Fe.prototype._setTileReloadTimer = function(Te, Ae) {
                    var Oe = this;
                    Te in this._timers && (clearTimeout(this._timers[Te]), delete this._timers[Te]);
                    var Be = Ae.getExpiryTimeout();
                    Be && (this._timers[Te] = setTimeout(function() {
                      Oe._reloadTile(Te, "expired"), delete Oe._timers[Te];
                    }, Be));
                  }, Fe.prototype._removeTile = function(Te) {
                    var Ae = this._tiles[Te];
                    Ae && (Ae.uses--, delete this._tiles[Te], this._timers[Te] && (clearTimeout(this._timers[Te]), delete this._timers[Te]), Ae.uses > 0 || (Ae.hasData() && Ae.state !== "reloading" ? this._cache.add(Ae.tileID, Ae, Ae.getExpiryTimeout()) : (Ae.aborted = !0, this._abortTile(Ae), this._unloadTile(Ae))));
                  }, Fe.prototype.clearTiles = function() {
                    for (var Te in this._shouldReloadOnResume = !1, this._paused = !1, this._tiles) this._removeTile(Te);
                    this._cache.reset();
                  }, Fe.prototype.tilesIn = function(Te, Ae, Oe) {
                    var Be = this, Ue = [], nt = this.transform;
                    if (!nt) return Ue;
                    for (var ht = Oe ? nt.getCameraQueryGeometry(Te) : Te, Et = Te.map(function(wr) {
                      return nt.pointCoordinate(wr);
                    }), Mt = ht.map(function(wr) {
                      return nt.pointCoordinate(wr);
                    }), qt = this.getIds(), tn = 1 / 0, xn = 1 / 0, An = -1 / 0, On = -1 / 0, Bn = 0, qn = Mt; Bn < qn.length; Bn += 1) {
                      var Vn = qn[Bn];
                      tn = Math.min(tn, Vn.x), xn = Math.min(xn, Vn.y), An = Math.max(An, Vn.x), On = Math.max(On, Vn.y);
                    }
                    for (var sr = function(wr) {
                      var Cr = Be._tiles[qt[wr]];
                      if (!Cr.holdingForFade()) {
                        var Tr = Cr.tileID, ii = Math.pow(2, nt.zoom - Cr.tileID.overscaledZ), wi = Ae * Cr.queryPadding * ee.EXTENT / Cr.tileSize / ii, Wi = [Tr.getTilePoint(new ee.MercatorCoordinate(tn, xn)), Tr.getTilePoint(new ee.MercatorCoordinate(An, On))];
                        if (Wi[0].x - wi < ee.EXTENT && Wi[0].y - wi < ee.EXTENT && Wi[1].x + wi >= 0 && Wi[1].y + wi >= 0) {
                          var Yi = Et.map(function(ha) {
                            return Tr.getTilePoint(ha);
                          }), ra = Mt.map(function(ha) {
                            return Tr.getTilePoint(ha);
                          });
                          Ue.push({ tile: Cr, tileID: Tr, queryGeometry: Yi, cameraQueryGeometry: ra, scale: ii });
                        }
                      }
                    }, Un = 0; Un < qt.length; Un++) sr(Un);
                    return Ue;
                  }, Fe.prototype.getVisibleCoordinates = function(Te) {
                    for (var Ae = this, Oe = this.getRenderableIds(Te).map(function(ht) {
                      return Ae._tiles[ht].tileID;
                    }), Be = 0, Ue = Oe; Be < Ue.length; Be += 1) {
                      var nt = Ue[Be];
                      nt.posMatrix = this.transform.calculatePosMatrix(nt.toUnwrapped());
                    }
                    return Oe;
                  }, Fe.prototype.hasTransition = function() {
                    if (this._source.hasTransition()) return !0;
                    if (gn(this._source.type)) for (var Te in this._tiles) {
                      var Ae = this._tiles[Te];
                      if (Ae.fadeEndTime !== void 0 && Ae.fadeEndTime >= ee.browser.now()) return !0;
                    }
                    return !1;
                  }, Fe.prototype.setFeatureState = function(Te, Ae, Oe) {
                    Te = Te || "_geojsonTileLayer", this._state.updateState(Te, Ae, Oe);
                  }, Fe.prototype.removeFeatureState = function(Te, Ae, Oe) {
                    Te = Te || "_geojsonTileLayer", this._state.removeFeatureState(Te, Ae, Oe);
                  }, Fe.prototype.getFeatureState = function(Te, Ae) {
                    return Te = Te || "_geojsonTileLayer", this._state.getState(Te, Ae);
                  }, Fe.prototype.setDependencies = function(Te, Ae, Oe) {
                    var Be = this._tiles[Te];
                    Be && Be.setDependencies(Ae, Oe);
                  }, Fe.prototype.reloadTilesForDependencies = function(Te, Ae) {
                    for (var Oe in this._tiles) this._tiles[Oe].hasDependency(Te, Ae) && this._reloadTile(Oe, "reloading");
                    this._cache.filter(function(Be) {
                      return !Be.hasDependency(Te, Ae);
                    });
                  }, Fe;
                }(ee.Evented);
                function _n(Ke, Fe) {
                  var Te = Math.abs(2 * Ke.wrap) - +(Ke.wrap < 0), Ae = Math.abs(2 * Fe.wrap) - +(Fe.wrap < 0);
                  return Ke.overscaledZ - Fe.overscaledZ || Ae - Te || Fe.canonical.y - Ke.canonical.y || Fe.canonical.x - Ke.canonical.x;
                }
                function gn(Ke) {
                  return Ke === "raster" || Ke === "image" || Ke === "video";
                }
                function bn() {
                  return new ee.window.Worker(Gn.workerUrl);
                }
                yn.maxOverzooming = 10, yn.maxUnderzooming = 3;
                var an = "mapboxgl_preloaded_worker_pool", Zt = function() {
                  this.active = {};
                };
                Zt.prototype.acquire = function(Ke) {
                  if (!this.workers) for (this.workers = []; this.workers.length < Zt.workerCount; ) this.workers.push(new bn());
                  return this.active[Ke] = !0, this.workers.slice();
                }, Zt.prototype.release = function(Ke) {
                  delete this.active[Ke], this.numActive() === 0 && (this.workers.forEach(function(Fe) {
                    Fe.terminate();
                  }), this.workers = null);
                }, Zt.prototype.isPreloaded = function() {
                  return !!this.active[an];
                }, Zt.prototype.numActive = function() {
                  return Object.keys(this.active).length;
                };
                var Mn, Ln = Math.floor(ee.browser.hardwareConcurrency / 2);
                function In() {
                  return Mn || (Mn = new Zt()), Mn;
                }
                function kn(Ke, Fe) {
                  var Te = {};
                  for (var Ae in Ke) Ae !== "ref" && (Te[Ae] = Ke[Ae]);
                  return ee.refProperties.forEach(function(Oe) {
                    Oe in Fe && (Te[Oe] = Fe[Oe]);
                  }), Te;
                }
                function er(Ke) {
                  Ke = Ke.slice();
                  for (var Fe = /* @__PURE__ */ Object.create(null), Te = 0; Te < Ke.length; Te++) Fe[Ke[Te].id] = Ke[Te];
                  for (var Ae = 0; Ae < Ke.length; Ae++) "ref" in Ke[Ae] && (Ke[Ae] = kn(Ke[Ae], Fe[Ke[Ae].ref]));
                  return Ke;
                }
                Zt.workerCount = Math.max(Math.min(Ln, 6), 1);
                var jn = { setStyle: "setStyle", addLayer: "addLayer", removeLayer: "removeLayer", setPaintProperty: "setPaintProperty", setLayoutProperty: "setLayoutProperty", setFilter: "setFilter", addSource: "addSource", removeSource: "removeSource", setGeoJSONSourceData: "setGeoJSONSourceData", setLayerZoomRange: "setLayerZoomRange", setLayerProperty: "setLayerProperty", setCenter: "setCenter", setZoom: "setZoom", setBearing: "setBearing", setPitch: "setPitch", setSprite: "setSprite", setGlyphs: "setGlyphs", setTransition: "setTransition", setLight: "setLight" };
                function ur(Ke, Fe, Te) {
                  Te.push({ command: jn.addSource, args: [Ke, Fe[Ke]] });
                }
                function Kn(Ke, Fe, Te) {
                  Fe.push({ command: jn.removeSource, args: [Ke] }), Te[Ke] = !0;
                }
                function Jn(Ke, Fe, Te, Ae) {
                  Kn(Ke, Te, Ae), ur(Ke, Fe, Te);
                }
                function Lr(Ke, Fe, Te) {
                  var Ae;
                  for (Ae in Ke[Te]) if (Ke[Te].hasOwnProperty(Ae) && Ae !== "data" && !ee.deepEqual(Ke[Te][Ae], Fe[Te][Ae])) return !1;
                  for (Ae in Fe[Te]) if (Fe[Te].hasOwnProperty(Ae) && Ae !== "data" && !ee.deepEqual(Ke[Te][Ae], Fe[Te][Ae])) return !1;
                  return !0;
                }
                function mr(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue;
                  for (Ue in Fe = Fe || {}, Ke = Ke || {}) Ke.hasOwnProperty(Ue) && (ee.deepEqual(Ke[Ue], Fe[Ue]) || Te.push({ command: Be, args: [Ae, Ue, Fe[Ue], Oe] }));
                  for (Ue in Fe) Fe.hasOwnProperty(Ue) && !Ke.hasOwnProperty(Ue) && (ee.deepEqual(Ke[Ue], Fe[Ue]) || Te.push({ command: Be, args: [Ae, Ue, Fe[Ue], Oe] }));
                }
                function Ar(Ke) {
                  return Ke.id;
                }
                function Yr(Ke, Fe) {
                  return Ke[Fe.id] = Fe, Ke;
                }
                function jr(Ke, Fe) {
                  if (!Ke) return [{ command: jn.setStyle, args: [Fe] }];
                  var Te = [];
                  try {
                    if (!ee.deepEqual(Ke.version, Fe.version)) return [{ command: jn.setStyle, args: [Fe] }];
                    ee.deepEqual(Ke.center, Fe.center) || Te.push({ command: jn.setCenter, args: [Fe.center] }), ee.deepEqual(Ke.zoom, Fe.zoom) || Te.push({ command: jn.setZoom, args: [Fe.zoom] }), ee.deepEqual(Ke.bearing, Fe.bearing) || Te.push({ command: jn.setBearing, args: [Fe.bearing] }), ee.deepEqual(Ke.pitch, Fe.pitch) || Te.push({ command: jn.setPitch, args: [Fe.pitch] }), ee.deepEqual(Ke.sprite, Fe.sprite) || Te.push({ command: jn.setSprite, args: [Fe.sprite] }), ee.deepEqual(Ke.glyphs, Fe.glyphs) || Te.push({ command: jn.setGlyphs, args: [Fe.glyphs] }), ee.deepEqual(Ke.transition, Fe.transition) || Te.push({ command: jn.setTransition, args: [Fe.transition] }), ee.deepEqual(Ke.light, Fe.light) || Te.push({ command: jn.setLight, args: [Fe.light] });
                    var Ae = {}, Oe = [];
                    (function(Ue, nt, ht, Et) {
                      var Mt;
                      for (Mt in nt = nt || {}, Ue = Ue || {}) Ue.hasOwnProperty(Mt) && (nt.hasOwnProperty(Mt) || Kn(Mt, ht, Et));
                      for (Mt in nt) nt.hasOwnProperty(Mt) && (Ue.hasOwnProperty(Mt) ? ee.deepEqual(Ue[Mt], nt[Mt]) || (Ue[Mt].type === "geojson" && nt[Mt].type === "geojson" && Lr(Ue, nt, Mt) ? ht.push({ command: jn.setGeoJSONSourceData, args: [Mt, nt[Mt].data] }) : Jn(Mt, nt, ht, Et)) : ur(Mt, nt, ht));
                    })(Ke.sources, Fe.sources, Oe, Ae);
                    var Be = [];
                    Ke.layers && Ke.layers.forEach(function(Ue) {
                      Ae[Ue.source] ? Te.push({ command: jn.removeLayer, args: [Ue.id] }) : Be.push(Ue);
                    }), Te = Te.concat(Oe), function(Ue, nt, ht) {
                      nt = nt || [];
                      var Et, Mt, qt, tn, xn, An, On, Bn = (Ue = Ue || []).map(Ar), qn = nt.map(Ar), Vn = Ue.reduce(Yr, {}), sr = nt.reduce(Yr, {}), Un = Bn.slice(), wr = /* @__PURE__ */ Object.create(null);
                      for (Et = 0, Mt = 0; Et < Bn.length; Et++) qt = Bn[Et], sr.hasOwnProperty(qt) ? Mt++ : (ht.push({ command: jn.removeLayer, args: [qt] }), Un.splice(Un.indexOf(qt, Mt), 1));
                      for (Et = 0, Mt = 0; Et < qn.length; Et++) qt = qn[qn.length - 1 - Et], Un[Un.length - 1 - Et] !== qt && (Vn.hasOwnProperty(qt) ? (ht.push({ command: jn.removeLayer, args: [qt] }), Un.splice(Un.lastIndexOf(qt, Un.length - Mt), 1)) : Mt++, An = Un[Un.length - Et], ht.push({ command: jn.addLayer, args: [sr[qt], An] }), Un.splice(Un.length - Et, 0, qt), wr[qt] = !0);
                      for (Et = 0; Et < qn.length; Et++) if (tn = Vn[qt = qn[Et]], xn = sr[qt], !wr[qt] && !ee.deepEqual(tn, xn)) if (ee.deepEqual(tn.source, xn.source) && ee.deepEqual(tn["source-layer"], xn["source-layer"]) && ee.deepEqual(tn.type, xn.type)) {
                        for (On in mr(tn.layout, xn.layout, ht, qt, null, jn.setLayoutProperty), mr(tn.paint, xn.paint, ht, qt, null, jn.setPaintProperty), ee.deepEqual(tn.filter, xn.filter) || ht.push({ command: jn.setFilter, args: [qt, xn.filter] }), ee.deepEqual(tn.minzoom, xn.minzoom) && ee.deepEqual(tn.maxzoom, xn.maxzoom) || ht.push({ command: jn.setLayerZoomRange, args: [qt, xn.minzoom, xn.maxzoom] }), tn) tn.hasOwnProperty(On) && On !== "layout" && On !== "paint" && On !== "filter" && On !== "metadata" && On !== "minzoom" && On !== "maxzoom" && (On.indexOf("paint.") === 0 ? mr(tn[On], xn[On], ht, qt, On.slice(6), jn.setPaintProperty) : ee.deepEqual(tn[On], xn[On]) || ht.push({ command: jn.setLayerProperty, args: [qt, On, xn[On]] }));
                        for (On in xn) xn.hasOwnProperty(On) && !tn.hasOwnProperty(On) && On !== "layout" && On !== "paint" && On !== "filter" && On !== "metadata" && On !== "minzoom" && On !== "maxzoom" && (On.indexOf("paint.") === 0 ? mr(tn[On], xn[On], ht, qt, On.slice(6), jn.setPaintProperty) : ee.deepEqual(tn[On], xn[On]) || ht.push({ command: jn.setLayerProperty, args: [qt, On, xn[On]] }));
                      } else ht.push({ command: jn.removeLayer, args: [qt] }), An = Un[Un.lastIndexOf(qt) + 1], ht.push({ command: jn.addLayer, args: [xn, An] });
                    }(Be, Fe.layers, Te);
                  } catch (Ue) {
                    console.warn("Unable to compute style diff:", Ue), Te = [{ command: jn.setStyle, args: [Fe] }];
                  }
                  return Te;
                }
                var Mr = function(Ke, Fe) {
                  this.reset(Ke, Fe);
                };
                Mr.prototype.reset = function(Ke, Fe) {
                  this.points = Ke || [], this._distances = [0];
                  for (var Te = 1; Te < this.points.length; Te++) this._distances[Te] = this._distances[Te - 1] + this.points[Te].dist(this.points[Te - 1]);
                  this.length = this._distances[this._distances.length - 1], this.padding = Math.min(Fe || 0, 0.5 * this.length), this.paddedLength = this.length - 2 * this.padding;
                }, Mr.prototype.lerp = function(Ke) {
                  if (this.points.length === 1) return this.points[0];
                  Ke = ee.clamp(Ke, 0, 1);
                  for (var Fe = 1, Te = this._distances[Fe], Ae = Ke * this.paddedLength + this.padding; Te < Ae && Fe < this._distances.length; ) Te = this._distances[++Fe];
                  var Oe = Fe - 1, Be = this._distances[Oe], Ue = Te - Be, nt = Ue > 0 ? (Ae - Be) / Ue : 0;
                  return this.points[Oe].mult(1 - nt).add(this.points[Fe].mult(nt));
                };
                var Wr = function(Ke, Fe, Te) {
                  var Ae = this.boxCells = [], Oe = this.circleCells = [];
                  this.xCellCount = Math.ceil(Ke / Te), this.yCellCount = Math.ceil(Fe / Te);
                  for (var Be = 0; Be < this.xCellCount * this.yCellCount; Be++) Ae.push([]), Oe.push([]);
                  this.circleKeys = [], this.boxKeys = [], this.bboxes = [], this.circles = [], this.width = Ke, this.height = Fe, this.xScale = this.xCellCount / Ke, this.yScale = this.yCellCount / Fe, this.boxUid = 0, this.circleUid = 0;
                };
                function Bi(Ke, Fe, Te, Ae, Oe) {
                  var Be = ee.create();
                  return Fe ? (ee.scale(Be, Be, [1 / Oe, 1 / Oe, 1]), Te || ee.rotateZ(Be, Be, Ae.angle)) : ee.multiply(Be, Ae.labelPlaneMatrix, Ke), Be;
                }
                function gi(Ke, Fe, Te, Ae, Oe) {
                  if (Fe) {
                    var Be = ee.clone(Ke);
                    return ee.scale(Be, Be, [Oe, Oe, 1]), Te || ee.rotateZ(Be, Be, -Ae.angle), Be;
                  }
                  return Ae.glCoordMatrix;
                }
                function Zr(Ke, Fe) {
                  var Te = [Ke.x, Ke.y, 0, 1];
                  ka(Te, Te, Fe);
                  var Ae = Te[3];
                  return { point: new ee.Point(Te[0] / Ae, Te[1] / Ae), signedDistanceFromCamera: Ae };
                }
                function fi(Ke, Fe) {
                  return 0.5 + Ke / Fe * 0.5;
                }
                function ri(Ke, Fe) {
                  var Te = Ke[0] / Ke[3], Ae = Ke[1] / Ke[3];
                  return Te >= -Fe[0] && Te <= Fe[0] && Ae >= -Fe[1] && Ae <= Fe[1];
                }
                function bi(Ke, Fe, Te, Ae, Oe, Be, Ue, nt) {
                  var ht = Ae ? Ke.textSizeData : Ke.iconSizeData, Et = ee.evaluateSizeForZoom(ht, Te.transform.zoom), Mt = [256 / Te.width * 2 + 1, 256 / Te.height * 2 + 1], qt = Ae ? Ke.text.dynamicLayoutVertexArray : Ke.icon.dynamicLayoutVertexArray;
                  qt.clear();
                  for (var tn = Ke.lineVertexArray, xn = Ae ? Ke.text.placedSymbolArray : Ke.icon.placedSymbolArray, An = Te.transform.width / Te.transform.height, On = !1, Bn = 0; Bn < xn.length; Bn++) {
                    var qn = xn.get(Bn);
                    if (qn.hidden || qn.writingMode === ee.WritingMode.vertical && !On) oa(qn.numGlyphs, qt);
                    else {
                      On = !1;
                      var Vn = [qn.anchorX, qn.anchorY, 0, 1];
                      if (ee.transformMat4(Vn, Vn, Fe), ri(Vn, Mt)) {
                        var sr = Vn[3], Un = fi(Te.transform.cameraToCenterDistance, sr), wr = ee.evaluateSizeForFeature(ht, Et, qn), Cr = Ue ? wr / Un : wr * Un, Tr = new ee.Point(qn.anchorX, qn.anchorY), ii = Zr(Tr, Oe).point, wi = {}, Wi = Br(qn, Cr, !1, nt, Fe, Oe, Be, Ke.glyphOffsetArray, tn, qt, ii, Tr, wi, An);
                        On = Wi.useVertical, (Wi.notEnoughRoom || On || Wi.needsFlipping && Br(qn, Cr, !0, nt, Fe, Oe, Be, Ke.glyphOffsetArray, tn, qt, ii, Tr, wi, An).notEnoughRoom) && oa(qn.numGlyphs, qt);
                      } else oa(qn.numGlyphs, qt);
                    }
                  }
                  Ae ? Ke.text.dynamicLayoutVertexBuffer.updateData(qt) : Ke.icon.dynamicLayoutVertexBuffer.updateData(qt);
                }
                function Zi(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt) {
                  var qt = nt.glyphStartIndex + nt.numGlyphs, tn = nt.lineStartIndex, xn = nt.lineStartIndex + nt.lineLength, An = Fe.getoffsetX(nt.glyphStartIndex), On = Fe.getoffsetX(qt - 1), Bn = ei(Ke * An, Te, Ae, Oe, Be, Ue, nt.segment, tn, xn, ht, Et, Mt);
                  if (!Bn) return null;
                  var qn = ei(Ke * On, Te, Ae, Oe, Be, Ue, nt.segment, tn, xn, ht, Et, Mt);
                  return qn ? { first: Bn, last: qn } : null;
                }
                function $i(Ke, Fe, Te, Ae) {
                  return Ke === ee.WritingMode.horizontal && Math.abs(Te.y - Fe.y) > Math.abs(Te.x - Fe.x) * Ae ? { useVertical: !0 } : (Ke === ee.WritingMode.vertical ? Fe.y < Te.y : Fe.x > Te.x) ? { needsFlipping: !0 } : null;
                }
                function Br(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn) {
                  var An, On = Fe / 24, Bn = Ke.lineOffsetX * On, qn = Ke.lineOffsetY * On;
                  if (Ke.numGlyphs > 1) {
                    var Vn = Ke.glyphStartIndex + Ke.numGlyphs, sr = Ke.lineStartIndex, Un = Ke.lineStartIndex + Ke.lineLength, wr = Zi(On, nt, Bn, qn, Te, Mt, qt, Ke, ht, Be, tn);
                    if (!wr) return { notEnoughRoom: !0 };
                    var Cr = Zr(wr.first.point, Ue).point, Tr = Zr(wr.last.point, Ue).point;
                    if (Ae && !Te) {
                      var ii = $i(Ke.writingMode, Cr, Tr, xn);
                      if (ii) return ii;
                    }
                    An = [wr.first];
                    for (var wi = Ke.glyphStartIndex + 1; wi < Vn - 1; wi++) An.push(ei(On * nt.getoffsetX(wi), Bn, qn, Te, Mt, qt, Ke.segment, sr, Un, ht, Be, tn));
                    An.push(wr.last);
                  } else {
                    if (Ae && !Te) {
                      var Wi = Zr(qt, Oe).point, Yi = Ke.lineStartIndex + Ke.segment + 1, ra = new ee.Point(ht.getx(Yi), ht.gety(Yi)), ha = Zr(ra, Oe), ya = ha.signedDistanceFromCamera > 0 ? ha.point : Pr(qt, ra, Wi, 1, Oe), va = $i(Ke.writingMode, Wi, ya, xn);
                      if (va) return va;
                    }
                    var qa = ei(On * nt.getoffsetX(Ke.glyphStartIndex), Bn, qn, Te, Mt, qt, Ke.segment, Ke.lineStartIndex, Ke.lineStartIndex + Ke.lineLength, ht, Be, tn);
                    if (!qa) return { notEnoughRoom: !0 };
                    An = [qa];
                  }
                  for (var Aa = 0, Xa = An; Aa < Xa.length; Aa += 1) {
                    var Oa = Xa[Aa];
                    ee.addDynamicAttributes(Et, Oa.point, Oa.angle);
                  }
                  return {};
                }
                function Pr(Ke, Fe, Te, Ae, Oe) {
                  var Be = Zr(Ke.add(Ke.sub(Fe)._unit()), Oe).point, Ue = Te.sub(Be);
                  return Te.add(Ue._mult(Ae / Ue.mag()));
                }
                function ei(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt) {
                  var tn = Ae ? Ke - Fe : Ke + Fe, xn = tn > 0 ? 1 : -1, An = 0;
                  Ae && (xn *= -1, An = Math.PI), xn < 0 && (An += Math.PI);
                  for (var On = xn > 0 ? nt + Ue : nt + Ue + 1, Bn = Oe, qn = Oe, Vn = 0, sr = 0, Un = Math.abs(tn), wr = []; Vn + sr <= Un; ) {
                    if ((On += xn) < nt || On >= ht) return null;
                    if (qn = Bn, wr.push(Bn), (Bn = qt[On]) === void 0) {
                      var Cr = new ee.Point(Et.getx(On), Et.gety(On)), Tr = Zr(Cr, Mt);
                      if (Tr.signedDistanceFromCamera > 0) Bn = qt[On] = Tr.point;
                      else {
                        var ii = On - xn;
                        Bn = Pr(Vn === 0 ? Be : new ee.Point(Et.getx(ii), Et.gety(ii)), Cr, qn, Un - Vn + 1, Mt);
                      }
                    }
                    Vn += sr, sr = qn.dist(Bn);
                  }
                  var wi = (Un - Vn) / sr, Wi = Bn.sub(qn), Yi = Wi.mult(wi)._add(qn);
                  Yi._add(Wi._unit()._perp()._mult(Te * xn));
                  var ra = An + Math.atan2(Bn.y - qn.y, Bn.x - qn.x);
                  return wr.push(Yi), { point: Yi, angle: ra, path: wr };
                }
                Wr.prototype.keysLength = function() {
                  return this.boxKeys.length + this.circleKeys.length;
                }, Wr.prototype.insert = function(Ke, Fe, Te, Ae, Oe) {
                  this._forEachCell(Fe, Te, Ae, Oe, this._insertBoxCell, this.boxUid++), this.boxKeys.push(Ke), this.bboxes.push(Fe), this.bboxes.push(Te), this.bboxes.push(Ae), this.bboxes.push(Oe);
                }, Wr.prototype.insertCircle = function(Ke, Fe, Te, Ae) {
                  this._forEachCell(Fe - Ae, Te - Ae, Fe + Ae, Te + Ae, this._insertCircleCell, this.circleUid++), this.circleKeys.push(Ke), this.circles.push(Fe), this.circles.push(Te), this.circles.push(Ae);
                }, Wr.prototype._insertBoxCell = function(Ke, Fe, Te, Ae, Oe, Be) {
                  this.boxCells[Oe].push(Be);
                }, Wr.prototype._insertCircleCell = function(Ke, Fe, Te, Ae, Oe, Be) {
                  this.circleCells[Oe].push(Be);
                }, Wr.prototype._query = function(Ke, Fe, Te, Ae, Oe, Be) {
                  if (Te < 0 || Ke > this.width || Ae < 0 || Fe > this.height) return !Oe && [];
                  var Ue = [];
                  if (Ke <= 0 && Fe <= 0 && this.width <= Te && this.height <= Ae) {
                    if (Oe) return !0;
                    for (var nt = 0; nt < this.boxKeys.length; nt++) Ue.push({ key: this.boxKeys[nt], x1: this.bboxes[4 * nt], y1: this.bboxes[4 * nt + 1], x2: this.bboxes[4 * nt + 2], y2: this.bboxes[4 * nt + 3] });
                    for (var ht = 0; ht < this.circleKeys.length; ht++) {
                      var Et = this.circles[3 * ht], Mt = this.circles[3 * ht + 1], qt = this.circles[3 * ht + 2];
                      Ue.push({ key: this.circleKeys[ht], x1: Et - qt, y1: Mt - qt, x2: Et + qt, y2: Mt + qt });
                    }
                    return Be ? Ue.filter(Be) : Ue;
                  }
                  var tn = { hitTest: Oe, seenUids: { box: {}, circle: {} } };
                  return this._forEachCell(Ke, Fe, Te, Ae, this._queryCell, Ue, tn, Be), Oe ? Ue.length > 0 : Ue;
                }, Wr.prototype._queryCircle = function(Ke, Fe, Te, Ae, Oe) {
                  var Be = Ke - Te, Ue = Ke + Te, nt = Fe - Te, ht = Fe + Te;
                  if (Ue < 0 || Be > this.width || ht < 0 || nt > this.height) return !Ae && [];
                  var Et = [], Mt = { hitTest: Ae, circle: { x: Ke, y: Fe, radius: Te }, seenUids: { box: {}, circle: {} } };
                  return this._forEachCell(Be, nt, Ue, ht, this._queryCellCircle, Et, Mt, Oe), Ae ? Et.length > 0 : Et;
                }, Wr.prototype.query = function(Ke, Fe, Te, Ae, Oe) {
                  return this._query(Ke, Fe, Te, Ae, !1, Oe);
                }, Wr.prototype.hitTest = function(Ke, Fe, Te, Ae, Oe) {
                  return this._query(Ke, Fe, Te, Ae, !0, Oe);
                }, Wr.prototype.hitTestCircle = function(Ke, Fe, Te, Ae) {
                  return this._queryCircle(Ke, Fe, Te, !0, Ae);
                }, Wr.prototype._queryCell = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt) {
                  var ht = Ue.seenUids, Et = this.boxCells[Oe];
                  if (Et !== null) for (var Mt = this.bboxes, qt = 0, tn = Et; qt < tn.length; qt += 1) {
                    var xn = tn[qt];
                    if (!ht.box[xn]) {
                      ht.box[xn] = !0;
                      var An = 4 * xn;
                      if (Ke <= Mt[An + 2] && Fe <= Mt[An + 3] && Te >= Mt[An + 0] && Ae >= Mt[An + 1] && (!nt || nt(this.boxKeys[xn]))) {
                        if (Ue.hitTest) return Be.push(!0), !0;
                        Be.push({ key: this.boxKeys[xn], x1: Mt[An], y1: Mt[An + 1], x2: Mt[An + 2], y2: Mt[An + 3] });
                      }
                    }
                  }
                  var On = this.circleCells[Oe];
                  if (On !== null) for (var Bn = this.circles, qn = 0, Vn = On; qn < Vn.length; qn += 1) {
                    var sr = Vn[qn];
                    if (!ht.circle[sr]) {
                      ht.circle[sr] = !0;
                      var Un = 3 * sr;
                      if (this._circleAndRectCollide(Bn[Un], Bn[Un + 1], Bn[Un + 2], Ke, Fe, Te, Ae) && (!nt || nt(this.circleKeys[sr]))) {
                        if (Ue.hitTest) return Be.push(!0), !0;
                        var wr = Bn[Un], Cr = Bn[Un + 1], Tr = Bn[Un + 2];
                        Be.push({ key: this.circleKeys[sr], x1: wr - Tr, y1: Cr - Tr, x2: wr + Tr, y2: Cr + Tr });
                      }
                    }
                  }
                }, Wr.prototype._queryCellCircle = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt) {
                  var ht = Ue.circle, Et = Ue.seenUids, Mt = this.boxCells[Oe];
                  if (Mt !== null) for (var qt = this.bboxes, tn = 0, xn = Mt; tn < xn.length; tn += 1) {
                    var An = xn[tn];
                    if (!Et.box[An]) {
                      Et.box[An] = !0;
                      var On = 4 * An;
                      if (this._circleAndRectCollide(ht.x, ht.y, ht.radius, qt[On + 0], qt[On + 1], qt[On + 2], qt[On + 3]) && (!nt || nt(this.boxKeys[An]))) return Be.push(!0), !0;
                    }
                  }
                  var Bn = this.circleCells[Oe];
                  if (Bn !== null) for (var qn = this.circles, Vn = 0, sr = Bn; Vn < sr.length; Vn += 1) {
                    var Un = sr[Vn];
                    if (!Et.circle[Un]) {
                      Et.circle[Un] = !0;
                      var wr = 3 * Un;
                      if (this._circlesCollide(qn[wr], qn[wr + 1], qn[wr + 2], ht.x, ht.y, ht.radius) && (!nt || nt(this.circleKeys[Un]))) return Be.push(!0), !0;
                    }
                  }
                }, Wr.prototype._forEachCell = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt) {
                  for (var ht = this._convertToXCellCoord(Ke), Et = this._convertToYCellCoord(Fe), Mt = this._convertToXCellCoord(Te), qt = this._convertToYCellCoord(Ae), tn = ht; tn <= Mt; tn++) for (var xn = Et; xn <= qt; xn++) {
                    var An = this.xCellCount * xn + tn;
                    if (Oe.call(this, Ke, Fe, Te, Ae, An, Be, Ue, nt)) return;
                  }
                }, Wr.prototype._convertToXCellCoord = function(Ke) {
                  return Math.max(0, Math.min(this.xCellCount - 1, Math.floor(Ke * this.xScale)));
                }, Wr.prototype._convertToYCellCoord = function(Ke) {
                  return Math.max(0, Math.min(this.yCellCount - 1, Math.floor(Ke * this.yScale)));
                }, Wr.prototype._circlesCollide = function(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = Ae - Ke, nt = Oe - Fe, ht = Te + Be;
                  return ht * ht > Ue * Ue + nt * nt;
                }, Wr.prototype._circleAndRectCollide = function(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  var nt = (Be - Ae) / 2, ht = Math.abs(Ke - (Ae + nt));
                  if (ht > nt + Te) return !1;
                  var Et = (Ue - Oe) / 2, Mt = Math.abs(Fe - (Oe + Et));
                  if (Mt > Et + Te) return !1;
                  if (ht <= nt || Mt <= Et) return !0;
                  var qt = ht - nt, tn = Mt - Et;
                  return qt * qt + tn * tn <= Te * Te;
                };
                var Ci = new Float32Array([-1 / 0, -1 / 0, 0, -1 / 0, -1 / 0, 0, -1 / 0, -1 / 0, 0, -1 / 0, -1 / 0, 0]);
                function oa(Ke, Fe) {
                  for (var Te = 0; Te < Ke; Te++) {
                    var Ae = Fe.length;
                    Fe.resize(Ae + 4), Fe.float32.set(Ci, 3 * Ae);
                  }
                }
                function ka(Ke, Fe, Te) {
                  var Ae = Fe[0], Oe = Fe[1];
                  return Ke[0] = Te[0] * Ae + Te[4] * Oe + Te[12], Ke[1] = Te[1] * Ae + Te[5] * Oe + Te[13], Ke[3] = Te[3] * Ae + Te[7] * Oe + Te[15], Ke;
                }
                var aa = 100, hi = function(Ke, Fe, Te) {
                  Fe === void 0 && (Fe = new Wr(Ke.width + 200, Ke.height + 200, 25)), Te === void 0 && (Te = new Wr(Ke.width + 200, Ke.height + 200, 25)), this.transform = Ke, this.grid = Fe, this.ignoredGrid = Te, this.pitchfactor = Math.cos(Ke._pitch) * Ke.cameraToCenterDistance, this.screenRightBoundary = Ke.width + aa, this.screenBottomBoundary = Ke.height + aa, this.gridRightBoundary = Ke.width + 200, this.gridBottomBoundary = Ke.height + 200;
                };
                function Gr(Ke, Fe, Te) {
                  return Fe * (ee.EXTENT / (Ke.tileSize * Math.pow(2, Te - Ke.tileID.overscaledZ)));
                }
                hi.prototype.placeCollisionBox = function(Ke, Fe, Te, Ae, Oe) {
                  var Be = this.projectAndGetPerspectiveRatio(Ae, Ke.anchorPointX, Ke.anchorPointY), Ue = Te * Be.perspectiveRatio, nt = Ke.x1 * Ue + Be.point.x, ht = Ke.y1 * Ue + Be.point.y, Et = Ke.x2 * Ue + Be.point.x, Mt = Ke.y2 * Ue + Be.point.y;
                  return !this.isInsideGrid(nt, ht, Et, Mt) || !Fe && this.grid.hitTest(nt, ht, Et, Mt, Oe) ? { box: [], offscreen: !1 } : { box: [nt, ht, Et, Mt], offscreen: this.isOffscreen(nt, ht, Et, Mt) };
                }, hi.prototype.placeCollisionCircles = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn) {
                  var xn = [], An = new ee.Point(Fe.anchorX, Fe.anchorY), On = Zr(An, Be), Bn = fi(this.transform.cameraToCenterDistance, On.signedDistanceFromCamera), qn = (Et ? Oe / Bn : Oe * Bn) / ee.ONE_EM, Vn = Zr(An, Ue).point, sr = Zi(qn, Ae, Fe.lineOffsetX * qn, Fe.lineOffsetY * qn, !1, Vn, An, Fe, Te, Ue, {}), Un = !1, wr = !1, Cr = !0;
                  if (sr) {
                    for (var Tr = 0.5 * qt * Bn + tn, ii = new ee.Point(-100, -100), wi = new ee.Point(this.screenRightBoundary, this.screenBottomBoundary), Wi = new Mr(), Yi = sr.first, ra = sr.last, ha = [], ya = Yi.path.length - 1; ya >= 1; ya--) ha.push(Yi.path[ya]);
                    for (var va = 1; va < ra.path.length; va++) ha.push(ra.path[va]);
                    var qa = 2.5 * Tr;
                    if (nt) {
                      var Aa = ha.map(function(Ml) {
                        return Zr(Ml, nt);
                      });
                      ha = Aa.some(function(Ml) {
                        return Ml.signedDistanceFromCamera <= 0;
                      }) ? [] : Aa.map(function(Ml) {
                        return Ml.point;
                      });
                    }
                    var Xa = [];
                    if (ha.length > 0) {
                      for (var Oa = ha[0].clone(), Ya = ha[0].clone(), Wo = 1; Wo < ha.length; Wo++) Oa.x = Math.min(Oa.x, ha[Wo].x), Oa.y = Math.min(Oa.y, ha[Wo].y), Ya.x = Math.max(Ya.x, ha[Wo].x), Ya.y = Math.max(Ya.y, ha[Wo].y);
                      Xa = Oa.x >= ii.x && Ya.x <= wi.x && Oa.y >= ii.y && Ya.y <= wi.y ? [ha] : Ya.x < ii.x || Oa.x > wi.x || Ya.y < ii.y || Oa.y > wi.y ? [] : ee.clipLine([ha], ii.x, ii.y, wi.x, wi.y);
                    }
                    for (var Ja = 0, Uo = Xa; Ja < Uo.length; Ja += 1) {
                      var ys = Uo[Ja];
                      Wi.reset(ys, 0.25 * Tr);
                      var il;
                      il = Wi.length <= 0.5 * Tr ? 1 : Math.ceil(Wi.paddedLength / qa) + 1;
                      for (var Is = 0; Is < il; Is++) {
                        var ml = Is / Math.max(il - 1, 1), nl = Wi.lerp(ml), Il = nl.x + aa, Fl = nl.y + aa;
                        xn.push(Il, Fl, Tr, 0);
                        var qc = Il - Tr, ql = Fl - Tr, jc = Il + Tr, Dl = Fl + Tr;
                        if (Cr = Cr && this.isOffscreen(qc, ql, jc, Dl), wr = wr || this.isInsideGrid(qc, ql, jc, Dl), !Ke && this.grid.hitTestCircle(Il, Fl, Tr, Mt) && (Un = !0, !ht)) return { circles: [], offscreen: !1, collisionDetected: Un };
                      }
                    }
                  }
                  return { circles: !ht && Un || !wr ? [] : xn, offscreen: Cr, collisionDetected: Un };
                }, hi.prototype.queryRenderedSymbols = function(Ke) {
                  if (Ke.length === 0 || this.grid.keysLength() === 0 && this.ignoredGrid.keysLength() === 0) return {};
                  for (var Fe = [], Te = 1 / 0, Ae = 1 / 0, Oe = -1 / 0, Be = -1 / 0, Ue = 0, nt = Ke; Ue < nt.length; Ue += 1) {
                    var ht = nt[Ue], Et = new ee.Point(ht.x + aa, ht.y + aa);
                    Te = Math.min(Te, Et.x), Ae = Math.min(Ae, Et.y), Oe = Math.max(Oe, Et.x), Be = Math.max(Be, Et.y), Fe.push(Et);
                  }
                  for (var Mt = {}, qt = {}, tn = 0, xn = this.grid.query(Te, Ae, Oe, Be).concat(this.ignoredGrid.query(Te, Ae, Oe, Be)); tn < xn.length; tn += 1) {
                    var An = xn[tn], On = An.key;
                    if (Mt[On.bucketInstanceId] === void 0 && (Mt[On.bucketInstanceId] = {}), !Mt[On.bucketInstanceId][On.featureIndex]) {
                      var Bn = [new ee.Point(An.x1, An.y1), new ee.Point(An.x2, An.y1), new ee.Point(An.x2, An.y2), new ee.Point(An.x1, An.y2)];
                      ee.polygonIntersectsPolygon(Fe, Bn) && (Mt[On.bucketInstanceId][On.featureIndex] = !0, qt[On.bucketInstanceId] === void 0 && (qt[On.bucketInstanceId] = []), qt[On.bucketInstanceId].push(On.featureIndex));
                    }
                  }
                  return qt;
                }, hi.prototype.insertCollisionBox = function(Ke, Fe, Te, Ae, Oe) {
                  var Be = { bucketInstanceId: Te, featureIndex: Ae, collisionGroupID: Oe };
                  (Fe ? this.ignoredGrid : this.grid).insert(Be, Ke[0], Ke[1], Ke[2], Ke[3]);
                }, hi.prototype.insertCollisionCircles = function(Ke, Fe, Te, Ae, Oe) {
                  for (var Be = Fe ? this.ignoredGrid : this.grid, Ue = { bucketInstanceId: Te, featureIndex: Ae, collisionGroupID: Oe }, nt = 0; nt < Ke.length; nt += 4) Be.insertCircle(Ue, Ke[nt], Ke[nt + 1], Ke[nt + 2]);
                }, hi.prototype.projectAndGetPerspectiveRatio = function(Ke, Fe, Te) {
                  var Ae = [Fe, Te, 0, 1];
                  return ka(Ae, Ae, Ke), { point: new ee.Point((Ae[0] / Ae[3] + 1) / 2 * this.transform.width + aa, (-Ae[1] / Ae[3] + 1) / 2 * this.transform.height + aa), perspectiveRatio: 0.5 + this.transform.cameraToCenterDistance / Ae[3] * 0.5 };
                }, hi.prototype.isOffscreen = function(Ke, Fe, Te, Ae) {
                  return Te < aa || Ke >= this.screenRightBoundary || Ae < aa || Fe > this.screenBottomBoundary;
                }, hi.prototype.isInsideGrid = function(Ke, Fe, Te, Ae) {
                  return Te >= 0 && Ke < this.gridRightBoundary && Ae >= 0 && Fe < this.gridBottomBoundary;
                }, hi.prototype.getViewportMatrix = function() {
                  var Ke = ee.identity([]);
                  return ee.translate(Ke, Ke, [-100, -100, 0]), Ke;
                };
                var Ca = function(Ke, Fe, Te, Ae) {
                  this.opacity = Ke ? Math.max(0, Math.min(1, Ke.opacity + (Ke.placed ? Fe : -Fe))) : Ae && Te ? 1 : 0, this.placed = Te;
                };
                Ca.prototype.isHidden = function() {
                  return this.opacity === 0 && !this.placed;
                };
                var Sr = function(Ke, Fe, Te, Ae, Oe) {
                  this.text = new Ca(Ke ? Ke.text : null, Fe, Te, Oe), this.icon = new Ca(Ke ? Ke.icon : null, Fe, Ae, Oe);
                };
                Sr.prototype.isHidden = function() {
                  return this.text.isHidden() && this.icon.isHidden();
                };
                var Dr = function(Ke, Fe, Te) {
                  this.text = Ke, this.icon = Fe, this.skipFade = Te;
                }, si = function() {
                  this.invProjMatrix = ee.create(), this.viewportMatrix = ee.create(), this.circles = [];
                }, zr = function(Ke, Fe, Te, Ae, Oe) {
                  this.bucketInstanceId = Ke, this.featureIndex = Fe, this.sourceLayerIndex = Te, this.bucketIndex = Ae, this.tileID = Oe;
                }, di = function(Ke) {
                  this.crossSourceCollisions = Ke, this.maxGroupID = 0, this.collisionGroups = {};
                };
                function Da(Ke, Fe, Te, Ae, Oe) {
                  var Be = ee.getAnchorAlignment(Ke), Ue = -(Be.horizontalAlign - 0.5) * Fe, nt = -(Be.verticalAlign - 0.5) * Te, ht = ee.evaluateVariableOffset(Ke, Ae);
                  return new ee.Point(Ue + ht[0] * Oe, nt + ht[1] * Oe);
                }
                function la(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = Ke.x1, nt = Ke.x2, ht = Ke.y1, Et = Ke.y2, Mt = Ke.anchorPointX, qt = Ke.anchorPointY, tn = new ee.Point(Fe, Te);
                  return Ae && tn._rotate(Oe ? Be : -Be), { x1: Ue + tn.x, y1: ht + tn.y, x2: nt + tn.x, y2: Et + tn.y, anchorPointX: Mt, anchorPointY: qt };
                }
                di.prototype.get = function(Ke) {
                  if (this.crossSourceCollisions) return { ID: 0, predicate: null };
                  if (!this.collisionGroups[Ke]) {
                    var Fe = ++this.maxGroupID;
                    this.collisionGroups[Ke] = { ID: Fe, predicate: function(Te) {
                      return Te.collisionGroupID === Fe;
                    } };
                  }
                  return this.collisionGroups[Ke];
                };
                var wa = function(Ke, Fe, Te, Ae) {
                  this.transform = Ke.clone(), this.collisionIndex = new hi(this.transform), this.placements = {}, this.opacities = {}, this.variableOffsets = {}, this.stale = !1, this.commitTime = 0, this.fadeDuration = Fe, this.retainedQueryData = {}, this.collisionGroups = new di(Te), this.collisionCircleArrays = {}, this.prevPlacement = Ae, Ae && (Ae.prevPlacement = void 0), this.placedOrientations = {};
                };
                function Wa(Ke, Fe, Te, Ae, Oe) {
                  Ke.emplaceBack(Fe ? 1 : 0, Te ? 1 : 0, Ae || 0, Oe || 0), Ke.emplaceBack(Fe ? 1 : 0, Te ? 1 : 0, Ae || 0, Oe || 0), Ke.emplaceBack(Fe ? 1 : 0, Te ? 1 : 0, Ae || 0, Oe || 0), Ke.emplaceBack(Fe ? 1 : 0, Te ? 1 : 0, Ae || 0, Oe || 0);
                }
                wa.prototype.getBucketParts = function(Ke, Fe, Te, Ae) {
                  var Oe = Te.getBucket(Fe), Be = Te.latestFeatureIndex;
                  if (Oe && Be && Fe.id === Oe.layerIds[0]) {
                    var Ue = Te.collisionBoxArray, nt = Oe.layers[0].layout, ht = Math.pow(2, this.transform.zoom - Te.tileID.overscaledZ), Et = Te.tileSize / ee.EXTENT, Mt = this.transform.calculatePosMatrix(Te.tileID.toUnwrapped()), qt = nt.get("text-pitch-alignment") === "map", tn = nt.get("text-rotation-alignment") === "map", xn = Gr(Te, 1, this.transform.zoom), An = Bi(Mt, qt, tn, this.transform, xn), On = null;
                    if (qt) {
                      var Bn = gi(Mt, qt, tn, this.transform, xn);
                      On = ee.multiply([], this.transform.labelPlaneMatrix, Bn);
                    }
                    this.retainedQueryData[Oe.bucketInstanceId] = new zr(Oe.bucketInstanceId, Be, Oe.sourceLayerIndex, Oe.index, Te.tileID);
                    var qn = { bucket: Oe, layout: nt, posMatrix: Mt, textLabelPlaneMatrix: An, labelToScreenMatrix: On, scale: ht, textPixelRatio: Et, holdingForFade: Te.holdingForFade(), collisionBoxArray: Ue, partiallyEvaluatedTextSize: ee.evaluateSizeForZoom(Oe.textSizeData, this.transform.zoom), collisionGroup: this.collisionGroups.get(Oe.sourceID) };
                    if (Ae) for (var Vn = 0, sr = Oe.sortKeyRanges; Vn < sr.length; Vn += 1) {
                      var Un = sr[Vn], wr = Un.sortKey, Cr = Un.symbolInstanceStart, Tr = Un.symbolInstanceEnd;
                      Ke.push({ sortKey: wr, symbolInstanceStart: Cr, symbolInstanceEnd: Tr, parameters: qn });
                    }
                    else Ke.push({ symbolInstanceStart: 0, symbolInstanceEnd: Oe.symbolInstances.length, parameters: qn });
                  }
                }, wa.prototype.attemptAnchorPlacement = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An) {
                  var On, Bn = [qt.textOffset0, qt.textOffset1], qn = Da(Ke, Te, Ae, Bn, Oe), Vn = this.collisionIndex.placeCollisionBox(la(Fe, qn.x, qn.y, Be, Ue, this.transform.angle), Mt, nt, ht, Et.predicate);
                  if (!An || this.collisionIndex.placeCollisionBox(la(An, qn.x, qn.y, Be, Ue, this.transform.angle), Mt, nt, ht, Et.predicate).box.length !== 0) return Vn.box.length > 0 ? (this.prevPlacement && this.prevPlacement.variableOffsets[qt.crossTileID] && this.prevPlacement.placements[qt.crossTileID] && this.prevPlacement.placements[qt.crossTileID].text && (On = this.prevPlacement.variableOffsets[qt.crossTileID].anchor), this.variableOffsets[qt.crossTileID] = { textOffset: Bn, width: Te, height: Ae, anchor: Ke, textBoxScale: Oe, prevAnchor: On }, this.markUsedJustification(tn, Ke, qt, xn), tn.allowVerticalPlacement && (this.markUsedOrientation(tn, xn, qt), this.placedOrientations[qt.crossTileID] = xn), { shift: qn, placedGlyphBoxes: Vn }) : void 0;
                }, wa.prototype.placeLayerBucketPart = function(Ke, Fe, Te) {
                  var Ae = this, Oe = Ke.parameters, Be = Oe.bucket, Ue = Oe.layout, nt = Oe.posMatrix, ht = Oe.textLabelPlaneMatrix, Et = Oe.labelToScreenMatrix, Mt = Oe.textPixelRatio, qt = Oe.holdingForFade, tn = Oe.collisionBoxArray, xn = Oe.partiallyEvaluatedTextSize, An = Oe.collisionGroup, On = Ue.get("text-optional"), Bn = Ue.get("icon-optional"), qn = Ue.get("text-allow-overlap"), Vn = Ue.get("icon-allow-overlap"), sr = Ue.get("text-rotation-alignment") === "map", Un = Ue.get("text-pitch-alignment") === "map", wr = Ue.get("icon-text-fit") !== "none", Cr = Ue.get("symbol-z-order") === "viewport-y", Tr = qn && (Vn || !Be.hasIconData() || Bn), ii = Vn && (qn || !Be.hasTextData() || On);
                  !Be.collisionArrays && tn && Be.deserializeCollisionBoxes(tn);
                  var wi = function(va, qa) {
                    if (!Fe[va.crossTileID]) if (qt) Ae.placements[va.crossTileID] = new Dr(!1, !1, !1);
                    else {
                      var Aa, Xa = !1, Oa = !1, Ya = !0, Wo = null, Ja = { box: null, offscreen: null }, Uo = { box: null, offscreen: null }, ys = null, il = null, Is = 0, ml = 0, nl = 0;
                      qa.textFeatureIndex ? Is = qa.textFeatureIndex : va.useRuntimeCollisionCircles && (Is = va.featureIndex), qa.verticalTextFeatureIndex && (ml = qa.verticalTextFeatureIndex);
                      var Il = qa.textBox;
                      if (Il) {
                        var Fl = function(Zc) {
                          var Ju = ee.WritingMode.horizontal;
                          if (Be.allowVerticalPlacement && !Zc && Ae.prevPlacement) {
                            var ru = Ae.prevPlacement.placedOrientations[va.crossTileID];
                            ru && (Ae.placedOrientations[va.crossTileID] = ru, Ju = ru, Ae.markUsedOrientation(Be, Ju, va));
                          }
                          return Ju;
                        }, qc = function(Zc, Ju) {
                          if (Be.allowVerticalPlacement && va.numVerticalGlyphVertices > 0 && qa.verticalTextBox) for (var ru = 0, Bd = Be.writingModes; ru < Bd.length && (Bd[ru] === ee.WritingMode.vertical ? (Ja = Ju(), Uo = Ja) : Ja = Zc(), !(Ja && Ja.box && Ja.box.length)); ru += 1) ;
                          else Ja = Zc();
                        };
                        if (Ue.get("text-variable-anchor")) {
                          var ql = Ue.get("text-variable-anchor");
                          if (Ae.prevPlacement && Ae.prevPlacement.variableOffsets[va.crossTileID]) {
                            var jc = Ae.prevPlacement.variableOffsets[va.crossTileID];
                            ql.indexOf(jc.anchor) > 0 && (ql = ql.filter(function(Zc) {
                              return Zc !== jc.anchor;
                            })).unshift(jc.anchor);
                          }
                          var Dl = function(Zc, Ju, ru) {
                            for (var Bd = Zc.x2 - Zc.x1, tf = Zc.y2 - Zc.y1, wd = va.textBoxScale, od = wr && !Vn ? Ju : null, yd = { box: [], offscreen: !1 }, Ef = qn ? 2 * ql.length : ql.length, Md = 0; Md < Ef; ++Md) {
                              var nf = ql[Md % ql.length], Vd = Md >= ql.length, S1 = Ae.attemptAnchorPlacement(nf, Zc, Bd, tf, wd, sr, Un, Mt, nt, An, Vd, va, Be, ru, od);
                              if (S1 && (yd = S1.placedGlyphBoxes) && yd.box && yd.box.length) {
                                Xa = !0, Wo = S1.shift;
                                break;
                              }
                            }
                            return yd;
                          };
                          qc(function() {
                            return Dl(Il, qa.iconBox, ee.WritingMode.horizontal);
                          }, function() {
                            var Zc = qa.verticalTextBox, Ju = Ja && Ja.box && Ja.box.length;
                            return Be.allowVerticalPlacement && !Ju && va.numVerticalGlyphVertices > 0 && Zc ? Dl(Zc, qa.verticalIconBox, ee.WritingMode.vertical) : { box: null, offscreen: null };
                          }), Ja && (Xa = Ja.box, Ya = Ja.offscreen);
                          var Ml = Fl(Ja && Ja.box);
                          if (!Xa && Ae.prevPlacement) {
                            var pc = Ae.prevPlacement.variableOffsets[va.crossTileID];
                            pc && (Ae.variableOffsets[va.crossTileID] = pc, Ae.markUsedJustification(Be, pc.anchor, va, Ml));
                          }
                        } else {
                          var hl = function(Zc, Ju) {
                            var ru = Ae.collisionIndex.placeCollisionBox(Zc, qn, Mt, nt, An.predicate);
                            return ru && ru.box && ru.box.length && (Ae.markUsedOrientation(Be, Ju, va), Ae.placedOrientations[va.crossTileID] = Ju), ru;
                          };
                          qc(function() {
                            return hl(Il, ee.WritingMode.horizontal);
                          }, function() {
                            var Zc = qa.verticalTextBox;
                            return Be.allowVerticalPlacement && va.numVerticalGlyphVertices > 0 && Zc ? hl(Zc, ee.WritingMode.vertical) : { box: null, offscreen: null };
                          }), Fl(Ja && Ja.box && Ja.box.length);
                        }
                      }
                      if (Xa = (Aa = Ja) && Aa.box && Aa.box.length > 0, Ya = Aa && Aa.offscreen, va.useRuntimeCollisionCircles) {
                        var xl = Be.text.placedSymbolArray.get(va.centerJustifiedTextSymbolIndex), Bl = ee.evaluateSizeForFeature(Be.textSizeData, xn, xl), al = Ue.get("text-padding"), Oc = va.collisionCircleDiameter;
                        ys = Ae.collisionIndex.placeCollisionCircles(qn, xl, Be.lineVertexArray, Be.glyphOffsetArray, Bl, nt, ht, Et, Te, Un, An.predicate, Oc, al), Xa = qn || ys.circles.length > 0 && !ys.collisionDetected, Ya = Ya && ys.offscreen;
                      }
                      if (qa.iconFeatureIndex && (nl = qa.iconFeatureIndex), qa.iconBox) {
                        var nu = function(Zc) {
                          var Ju = wr && Wo ? la(Zc, Wo.x, Wo.y, sr, Un, Ae.transform.angle) : Zc;
                          return Ae.collisionIndex.placeCollisionBox(Ju, Vn, Mt, nt, An.predicate);
                        };
                        Oa = Uo && Uo.box && Uo.box.length && qa.verticalIconBox ? (il = nu(qa.verticalIconBox)).box.length > 0 : (il = nu(qa.iconBox)).box.length > 0, Ya = Ya && il.offscreen;
                      }
                      var Ic = On || va.numHorizontalGlyphVertices === 0 && va.numVerticalGlyphVertices === 0, $s = Bn || va.numIconVertices === 0;
                      if (Ic || $s ? $s ? Ic || (Oa = Oa && Xa) : Xa = Oa && Xa : Oa = Xa = Oa && Xa, Xa && Aa && Aa.box && (Uo && Uo.box && ml ? Ae.collisionIndex.insertCollisionBox(Aa.box, Ue.get("text-ignore-placement"), Be.bucketInstanceId, ml, An.ID) : Ae.collisionIndex.insertCollisionBox(Aa.box, Ue.get("text-ignore-placement"), Be.bucketInstanceId, Is, An.ID)), Oa && il && Ae.collisionIndex.insertCollisionBox(il.box, Ue.get("icon-ignore-placement"), Be.bucketInstanceId, nl, An.ID), ys && (Xa && Ae.collisionIndex.insertCollisionCircles(ys.circles, Ue.get("text-ignore-placement"), Be.bucketInstanceId, Is, An.ID), Te)) {
                        var Xc = Be.bucketInstanceId, wc = Ae.collisionCircleArrays[Xc];
                        wc === void 0 && (wc = Ae.collisionCircleArrays[Xc] = new si());
                        for (var zu = 0; zu < ys.circles.length; zu += 4) wc.circles.push(ys.circles[zu + 0]), wc.circles.push(ys.circles[zu + 1]), wc.circles.push(ys.circles[zu + 2]), wc.circles.push(ys.collisionDetected ? 1 : 0);
                      }
                      Ae.placements[va.crossTileID] = new Dr(Xa || Tr, Oa || ii, Ya || Be.justReloaded), Fe[va.crossTileID] = !0;
                    }
                  };
                  if (Cr) for (var Wi = Be.getSortedSymbolIndexes(this.transform.angle), Yi = Wi.length - 1; Yi >= 0; --Yi) {
                    var ra = Wi[Yi];
                    wi(Be.symbolInstances.get(ra), Be.collisionArrays[ra]);
                  }
                  else for (var ha = Ke.symbolInstanceStart; ha < Ke.symbolInstanceEnd; ha++) wi(Be.symbolInstances.get(ha), Be.collisionArrays[ha]);
                  if (Te && Be.bucketInstanceId in this.collisionCircleArrays) {
                    var ya = this.collisionCircleArrays[Be.bucketInstanceId];
                    ee.invert(ya.invProjMatrix, nt), ya.viewportMatrix = this.collisionIndex.getViewportMatrix();
                  }
                  Be.justReloaded = !1;
                }, wa.prototype.markUsedJustification = function(Ke, Fe, Te, Ae) {
                  var Oe, Be = { left: Te.leftJustifiedTextSymbolIndex, center: Te.centerJustifiedTextSymbolIndex, right: Te.rightJustifiedTextSymbolIndex };
                  Oe = Ae === ee.WritingMode.vertical ? Te.verticalPlacedTextSymbolIndex : Be[ee.getAnchorJustification(Fe)];
                  for (var Ue = 0, nt = [Te.leftJustifiedTextSymbolIndex, Te.centerJustifiedTextSymbolIndex, Te.rightJustifiedTextSymbolIndex, Te.verticalPlacedTextSymbolIndex]; Ue < nt.length; Ue += 1) {
                    var ht = nt[Ue];
                    ht >= 0 && (Ke.text.placedSymbolArray.get(ht).crossTileID = Oe >= 0 && ht !== Oe ? 0 : Te.crossTileID);
                  }
                }, wa.prototype.markUsedOrientation = function(Ke, Fe, Te) {
                  for (var Ae = Fe === ee.WritingMode.horizontal || Fe === ee.WritingMode.horizontalOnly ? Fe : 0, Oe = Fe === ee.WritingMode.vertical ? Fe : 0, Be = 0, Ue = [Te.leftJustifiedTextSymbolIndex, Te.centerJustifiedTextSymbolIndex, Te.rightJustifiedTextSymbolIndex]; Be < Ue.length; Be += 1) {
                    var nt = Ue[Be];
                    Ke.text.placedSymbolArray.get(nt).placedOrientation = Ae;
                  }
                  Te.verticalPlacedTextSymbolIndex && (Ke.text.placedSymbolArray.get(Te.verticalPlacedTextSymbolIndex).placedOrientation = Oe);
                }, wa.prototype.commit = function(Ke) {
                  this.commitTime = Ke, this.zoomAtLastRecencyCheck = this.transform.zoom;
                  var Fe = this.prevPlacement, Te = !1;
                  this.prevZoomAdjustment = Fe ? Fe.zoomAdjustment(this.transform.zoom) : 0;
                  var Ae = Fe ? Fe.symbolFadeChange(Ke) : 1, Oe = Fe ? Fe.opacities : {}, Be = Fe ? Fe.variableOffsets : {}, Ue = Fe ? Fe.placedOrientations : {};
                  for (var nt in this.placements) {
                    var ht = this.placements[nt], Et = Oe[nt];
                    Et ? (this.opacities[nt] = new Sr(Et, Ae, ht.text, ht.icon), Te = Te || ht.text !== Et.text.placed || ht.icon !== Et.icon.placed) : (this.opacities[nt] = new Sr(null, Ae, ht.text, ht.icon, ht.skipFade), Te = Te || ht.text || ht.icon);
                  }
                  for (var Mt in Oe) {
                    var qt = Oe[Mt];
                    if (!this.opacities[Mt]) {
                      var tn = new Sr(qt, Ae, !1, !1);
                      tn.isHidden() || (this.opacities[Mt] = tn, Te = Te || qt.text.placed || qt.icon.placed);
                    }
                  }
                  for (var xn in Be) this.variableOffsets[xn] || !this.opacities[xn] || this.opacities[xn].isHidden() || (this.variableOffsets[xn] = Be[xn]);
                  for (var An in Ue) this.placedOrientations[An] || !this.opacities[An] || this.opacities[An].isHidden() || (this.placedOrientations[An] = Ue[An]);
                  Te ? this.lastPlacementChangeTime = Ke : typeof this.lastPlacementChangeTime != "number" && (this.lastPlacementChangeTime = Fe ? Fe.lastPlacementChangeTime : Ke);
                }, wa.prototype.updateLayerOpacities = function(Ke, Fe) {
                  for (var Te = {}, Ae = 0, Oe = Fe; Ae < Oe.length; Ae += 1) {
                    var Be = Oe[Ae], Ue = Be.getBucket(Ke);
                    Ue && Be.latestFeatureIndex && Ke.id === Ue.layerIds[0] && this.updateBucketOpacities(Ue, Te, Be.collisionBoxArray);
                  }
                }, wa.prototype.updateBucketOpacities = function(Ke, Fe, Te) {
                  var Ae = this;
                  Ke.hasTextData() && Ke.text.opacityVertexArray.clear(), Ke.hasIconData() && Ke.icon.opacityVertexArray.clear(), Ke.hasIconCollisionBoxData() && Ke.iconCollisionBox.collisionVertexArray.clear(), Ke.hasTextCollisionBoxData() && Ke.textCollisionBox.collisionVertexArray.clear();
                  var Oe = Ke.layers[0].layout, Be = new Sr(null, 0, !1, !1, !0), Ue = Oe.get("text-allow-overlap"), nt = Oe.get("icon-allow-overlap"), ht = Oe.get("text-variable-anchor"), Et = Oe.get("text-rotation-alignment") === "map", Mt = Oe.get("text-pitch-alignment") === "map", qt = Oe.get("icon-text-fit") !== "none", tn = new Sr(null, 0, Ue && (nt || !Ke.hasIconData() || Oe.get("icon-optional")), nt && (Ue || !Ke.hasTextData() || Oe.get("text-optional")), !0);
                  !Ke.collisionArrays && Te && (Ke.hasIconCollisionBoxData() || Ke.hasTextCollisionBoxData()) && Ke.deserializeCollisionBoxes(Te);
                  for (var xn = function(qn, Vn, sr) {
                    for (var Un = 0; Un < Vn / 4; Un++) qn.opacityVertexArray.emplaceBack(sr);
                  }, An = function(qn) {
                    var Vn = Ke.symbolInstances.get(qn), sr = Vn.numHorizontalGlyphVertices, Un = Vn.numVerticalGlyphVertices, wr = Vn.crossTileID, Cr = Fe[wr], Tr = Ae.opacities[wr];
                    Cr ? Tr = Be : Tr || (Tr = tn, Ae.opacities[wr] = Tr), Fe[wr] = !0;
                    var ii = sr > 0 || Un > 0, wi = Vn.numIconVertices > 0, Wi = Ae.placedOrientations[Vn.crossTileID], Yi = Wi === ee.WritingMode.vertical, ra = Wi === ee.WritingMode.horizontal || Wi === ee.WritingMode.horizontalOnly;
                    if (ii) {
                      var ha = Jr(Tr.text), ya = Yi ? ti : ha;
                      xn(Ke.text, sr, ya);
                      var va = ra ? ti : ha;
                      xn(Ke.text, Un, va);
                      var qa = Tr.text.isHidden();
                      [Vn.rightJustifiedTextSymbolIndex, Vn.centerJustifiedTextSymbolIndex, Vn.leftJustifiedTextSymbolIndex].forEach(function(nl) {
                        nl >= 0 && (Ke.text.placedSymbolArray.get(nl).hidden = qa || Yi ? 1 : 0);
                      }), Vn.verticalPlacedTextSymbolIndex >= 0 && (Ke.text.placedSymbolArray.get(Vn.verticalPlacedTextSymbolIndex).hidden = qa || ra ? 1 : 0);
                      var Aa = Ae.variableOffsets[Vn.crossTileID];
                      Aa && Ae.markUsedJustification(Ke, Aa.anchor, Vn, Wi);
                      var Xa = Ae.placedOrientations[Vn.crossTileID];
                      Xa && (Ae.markUsedJustification(Ke, "left", Vn, Xa), Ae.markUsedOrientation(Ke, Xa, Vn));
                    }
                    if (wi) {
                      var Oa = Jr(Tr.icon), Ya = !(qt && Vn.verticalPlacedIconSymbolIndex && Yi);
                      if (Vn.placedIconSymbolIndex >= 0) {
                        var Wo = Ya ? Oa : ti;
                        xn(Ke.icon, Vn.numIconVertices, Wo), Ke.icon.placedSymbolArray.get(Vn.placedIconSymbolIndex).hidden = Tr.icon.isHidden();
                      }
                      if (Vn.verticalPlacedIconSymbolIndex >= 0) {
                        var Ja = Ya ? ti : Oa;
                        xn(Ke.icon, Vn.numVerticalIconVertices, Ja), Ke.icon.placedSymbolArray.get(Vn.verticalPlacedIconSymbolIndex).hidden = Tr.icon.isHidden();
                      }
                    }
                    if (Ke.hasIconCollisionBoxData() || Ke.hasTextCollisionBoxData()) {
                      var Uo = Ke.collisionArrays[qn];
                      if (Uo) {
                        var ys = new ee.Point(0, 0);
                        if (Uo.textBox || Uo.verticalTextBox) {
                          var il = !0;
                          if (ht) {
                            var Is = Ae.variableOffsets[wr];
                            Is ? (ys = Da(Is.anchor, Is.width, Is.height, Is.textOffset, Is.textBoxScale), Et && ys._rotate(Mt ? Ae.transform.angle : -Ae.transform.angle)) : il = !1;
                          }
                          Uo.textBox && Wa(Ke.textCollisionBox.collisionVertexArray, Tr.text.placed, !il || Yi, ys.x, ys.y), Uo.verticalTextBox && Wa(Ke.textCollisionBox.collisionVertexArray, Tr.text.placed, !il || ra, ys.x, ys.y);
                        }
                        var ml = !!(!ra && Uo.verticalIconBox);
                        Uo.iconBox && Wa(Ke.iconCollisionBox.collisionVertexArray, Tr.icon.placed, ml, qt ? ys.x : 0, qt ? ys.y : 0), Uo.verticalIconBox && Wa(Ke.iconCollisionBox.collisionVertexArray, Tr.icon.placed, !ml, qt ? ys.x : 0, qt ? ys.y : 0);
                      }
                    }
                  }, On = 0; On < Ke.symbolInstances.length; On++) An(On);
                  if (Ke.sortFeatures(this.transform.angle), this.retainedQueryData[Ke.bucketInstanceId] && (this.retainedQueryData[Ke.bucketInstanceId].featureSortOrder = Ke.featureSortOrder), Ke.hasTextData() && Ke.text.opacityVertexBuffer && Ke.text.opacityVertexBuffer.updateData(Ke.text.opacityVertexArray), Ke.hasIconData() && Ke.icon.opacityVertexBuffer && Ke.icon.opacityVertexBuffer.updateData(Ke.icon.opacityVertexArray), Ke.hasIconCollisionBoxData() && Ke.iconCollisionBox.collisionVertexBuffer && Ke.iconCollisionBox.collisionVertexBuffer.updateData(Ke.iconCollisionBox.collisionVertexArray), Ke.hasTextCollisionBoxData() && Ke.textCollisionBox.collisionVertexBuffer && Ke.textCollisionBox.collisionVertexBuffer.updateData(Ke.textCollisionBox.collisionVertexArray), Ke.bucketInstanceId in this.collisionCircleArrays) {
                    var Bn = this.collisionCircleArrays[Ke.bucketInstanceId];
                    Ke.placementInvProjMatrix = Bn.invProjMatrix, Ke.placementViewportMatrix = Bn.viewportMatrix, Ke.collisionCircleArray = Bn.circles, delete this.collisionCircleArrays[Ke.bucketInstanceId];
                  }
                }, wa.prototype.symbolFadeChange = function(Ke) {
                  return this.fadeDuration === 0 ? 1 : (Ke - this.commitTime) / this.fadeDuration + this.prevZoomAdjustment;
                }, wa.prototype.zoomAdjustment = function(Ke) {
                  return Math.max(0, (this.transform.zoom - Ke) / 1.5);
                }, wa.prototype.hasTransitions = function(Ke) {
                  return this.stale || Ke - this.lastPlacementChangeTime < this.fadeDuration;
                }, wa.prototype.stillRecent = function(Ke, Fe) {
                  var Te = this.zoomAtLastRecencyCheck === Fe ? 1 - this.zoomAdjustment(Fe) : 1;
                  return this.zoomAtLastRecencyCheck = Fe, this.commitTime + this.fadeDuration * Te > Ke;
                }, wa.prototype.setStale = function() {
                  this.stale = !0;
                };
                var Qr = Math.pow(2, 25), Ur = Math.pow(2, 24), rr = Math.pow(2, 17), vr = Math.pow(2, 16), Rr = Math.pow(2, 9), Vr = Math.pow(2, 8), Xr = Math.pow(2, 1);
                function Jr(Ke) {
                  if (Ke.opacity === 0 && !Ke.placed) return 0;
                  if (Ke.opacity === 1 && Ke.placed) return 4294967295;
                  var Fe = Ke.placed ? 1 : 0, Te = Math.floor(127 * Ke.opacity);
                  return Te * Qr + Fe * Ur + Te * rr + Fe * vr + Te * Rr + Fe * Vr + Te * Xr + Fe;
                }
                var ti = 0, Hr = function(Ke) {
                  this._sortAcrossTiles = Ke.layout.get("symbol-z-order") !== "viewport-y" && Ke.layout.get("symbol-sort-key").constantOr(1) !== void 0, this._currentTileIndex = 0, this._currentPartIndex = 0, this._seenCrossTileIDs = {}, this._bucketParts = [];
                };
                Hr.prototype.continuePlacement = function(Ke, Fe, Te, Ae, Oe) {
                  for (var Be = this._bucketParts; this._currentTileIndex < Ke.length; ) {
                    var Ue = Ke[this._currentTileIndex];
                    if (Fe.getBucketParts(Be, Ae, Ue, this._sortAcrossTiles), this._currentTileIndex++, Oe()) return !0;
                  }
                  for (this._sortAcrossTiles && (this._sortAcrossTiles = !1, Be.sort(function(ht, Et) {
                    return ht.sortKey - Et.sortKey;
                  })); this._currentPartIndex < Be.length; ) {
                    var nt = Be[this._currentPartIndex];
                    if (Fe.placeLayerBucketPart(nt, this._seenCrossTileIDs, Te), this._currentPartIndex++, Oe()) return !0;
                  }
                  return !1;
                };
                var Er = function(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  this.placement = new wa(Ke, Oe, Be, Ue), this._currentPlacementIndex = Fe.length - 1, this._forceFullPlacement = Te, this._showCollisionBoxes = Ae, this._done = !1;
                };
                Er.prototype.isDone = function() {
                  return this._done;
                }, Er.prototype.continuePlacement = function(Ke, Fe, Te) {
                  for (var Ae = this, Oe = ee.browser.now(), Be = function() {
                    var ht = ee.browser.now() - Oe;
                    return !Ae._forceFullPlacement && ht > 2;
                  }; this._currentPlacementIndex >= 0; ) {
                    var Ue = Fe[Ke[this._currentPlacementIndex]], nt = this.placement.collisionIndex.transform.zoom;
                    if (Ue.type === "symbol" && (!Ue.minzoom || Ue.minzoom <= nt) && (!Ue.maxzoom || Ue.maxzoom > nt)) {
                      if (this._inProgressLayer || (this._inProgressLayer = new Hr(Ue)), this._inProgressLayer.continuePlacement(Te[Ue.source], this.placement, this._showCollisionBoxes, Ue, Be)) return;
                      delete this._inProgressLayer;
                    }
                    this._currentPlacementIndex--;
                  }
                  this._done = !0;
                }, Er.prototype.commit = function(Ke) {
                  return this.placement.commit(Ke), this.placement;
                };
                var gr = 512 / ee.EXTENT / 2, $r = function(Ke, Fe, Te) {
                  this.tileID = Ke, this.indexedSymbolInstances = {}, this.bucketInstanceId = Te;
                  for (var Ae = 0; Ae < Fe.length; Ae++) {
                    var Oe = Fe.get(Ae), Be = Oe.key;
                    this.indexedSymbolInstances[Be] || (this.indexedSymbolInstances[Be] = []), this.indexedSymbolInstances[Be].push({ crossTileID: Oe.crossTileID, coord: this.getScaledCoordinates(Oe, Ke) });
                  }
                };
                $r.prototype.getScaledCoordinates = function(Ke, Fe) {
                  var Te = Fe.canonical.z - this.tileID.canonical.z, Ae = gr / Math.pow(2, Te);
                  return { x: Math.floor((Fe.canonical.x * ee.EXTENT + Ke.anchorX) * Ae), y: Math.floor((Fe.canonical.y * ee.EXTENT + Ke.anchorY) * Ae) };
                }, $r.prototype.findMatches = function(Ke, Fe, Te) {
                  for (var Ae = this.tileID.canonical.z < Fe.canonical.z ? 1 : Math.pow(2, this.tileID.canonical.z - Fe.canonical.z), Oe = 0; Oe < Ke.length; Oe++) {
                    var Be = Ke.get(Oe);
                    if (!Be.crossTileID) {
                      var Ue = this.indexedSymbolInstances[Be.key];
                      if (Ue) for (var nt = this.getScaledCoordinates(Be, Fe), ht = 0, Et = Ue; ht < Et.length; ht += 1) {
                        var Mt = Et[ht];
                        if (Math.abs(Mt.coord.x - nt.x) <= Ae && Math.abs(Mt.coord.y - nt.y) <= Ae && !Te[Mt.crossTileID]) {
                          Te[Mt.crossTileID] = !0, Be.crossTileID = Mt.crossTileID;
                          break;
                        }
                      }
                    }
                  }
                };
                var li = function() {
                  this.maxCrossTileID = 0;
                };
                li.prototype.generate = function() {
                  return ++this.maxCrossTileID;
                };
                var Qi = function() {
                  this.indexes = {}, this.usedCrossTileIDs = {}, this.lng = 0;
                };
                Qi.prototype.handleWrapJump = function(Ke) {
                  var Fe = Math.round((Ke - this.lng) / 360);
                  if (Fe !== 0) for (var Te in this.indexes) {
                    var Ae = this.indexes[Te], Oe = {};
                    for (var Be in Ae) {
                      var Ue = Ae[Be];
                      Ue.tileID = Ue.tileID.unwrapTo(Ue.tileID.wrap + Fe), Oe[Ue.tileID.key] = Ue;
                    }
                    this.indexes[Te] = Oe;
                  }
                  this.lng = Ke;
                }, Qi.prototype.addBucket = function(Ke, Fe, Te) {
                  if (this.indexes[Ke.overscaledZ] && this.indexes[Ke.overscaledZ][Ke.key]) {
                    if (this.indexes[Ke.overscaledZ][Ke.key].bucketInstanceId === Fe.bucketInstanceId) return !1;
                    this.removeBucketCrossTileIDs(Ke.overscaledZ, this.indexes[Ke.overscaledZ][Ke.key]);
                  }
                  for (var Ae = 0; Ae < Fe.symbolInstances.length; Ae++) Fe.symbolInstances.get(Ae).crossTileID = 0;
                  this.usedCrossTileIDs[Ke.overscaledZ] || (this.usedCrossTileIDs[Ke.overscaledZ] = {});
                  var Oe = this.usedCrossTileIDs[Ke.overscaledZ];
                  for (var Be in this.indexes) {
                    var Ue = this.indexes[Be];
                    if (Number(Be) > Ke.overscaledZ) for (var nt in Ue) {
                      var ht = Ue[nt];
                      ht.tileID.isChildOf(Ke) && ht.findMatches(Fe.symbolInstances, Ke, Oe);
                    }
                    else {
                      var Et = Ue[Ke.scaledTo(Number(Be)).key];
                      Et && Et.findMatches(Fe.symbolInstances, Ke, Oe);
                    }
                  }
                  for (var Mt = 0; Mt < Fe.symbolInstances.length; Mt++) {
                    var qt = Fe.symbolInstances.get(Mt);
                    qt.crossTileID || (qt.crossTileID = Te.generate(), Oe[qt.crossTileID] = !0);
                  }
                  return this.indexes[Ke.overscaledZ] === void 0 && (this.indexes[Ke.overscaledZ] = {}), this.indexes[Ke.overscaledZ][Ke.key] = new $r(Ke, Fe.symbolInstances, Fe.bucketInstanceId), !0;
                }, Qi.prototype.removeBucketCrossTileIDs = function(Ke, Fe) {
                  for (var Te in Fe.indexedSymbolInstances) for (var Ae = 0, Oe = Fe.indexedSymbolInstances[Te]; Ae < Oe.length; Ae += 1) {
                    var Be = Oe[Ae];
                    delete this.usedCrossTileIDs[Ke][Be.crossTileID];
                  }
                }, Qi.prototype.removeStaleBuckets = function(Ke) {
                  var Fe = !1;
                  for (var Te in this.indexes) {
                    var Ae = this.indexes[Te];
                    for (var Oe in Ae) Ke[Ae[Oe].bucketInstanceId] || (this.removeBucketCrossTileIDs(Te, Ae[Oe]), delete Ae[Oe], Fe = !0);
                  }
                  return Fe;
                };
                var Ai = function() {
                  this.layerIndexes = {}, this.crossTileIDs = new li(), this.maxBucketInstanceId = 0, this.bucketsInCurrentPlacement = {};
                };
                Ai.prototype.addLayer = function(Ke, Fe, Te) {
                  var Ae = this.layerIndexes[Ke.id];
                  Ae === void 0 && (Ae = this.layerIndexes[Ke.id] = new Qi());
                  var Oe = !1, Be = {};
                  Ae.handleWrapJump(Te);
                  for (var Ue = 0, nt = Fe; Ue < nt.length; Ue += 1) {
                    var ht = nt[Ue], Et = ht.getBucket(Ke);
                    Et && Ke.id === Et.layerIds[0] && (Et.bucketInstanceId || (Et.bucketInstanceId = ++this.maxBucketInstanceId), Ae.addBucket(ht.tileID, Et, this.crossTileIDs) && (Oe = !0), Be[Et.bucketInstanceId] = !0);
                  }
                  return Ae.removeStaleBuckets(Be) && (Oe = !0), Oe;
                }, Ai.prototype.pruneUnusedLayers = function(Ke) {
                  var Fe = {};
                  for (var Te in Ke.forEach(function(Ae) {
                    Fe[Ae] = !0;
                  }), this.layerIndexes) Fe[Te] || delete this.layerIndexes[Te];
                };
                var Li = function(Ke, Fe) {
                  return ee.emitValidationErrors(Ke, Fe && Fe.filter(function(Te) {
                    return Te.identifier !== "source.canvas";
                  }));
                }, Ni = ee.pick(jn, ["addLayer", "removeLayer", "setPaintProperty", "setLayoutProperty", "setFilter", "addSource", "removeSource", "setLayerZoomRange", "setLight", "setTransition", "setGeoJSONSourceData"]), Pi = ee.pick(jn, ["setCenter", "setZoom", "setBearing", "setPitch"]), Oi = function() {
                  var Ke = {}, Fe = ee.styleSpec.$version;
                  for (var Te in ee.styleSpec.$root) {
                    var Ae = ee.styleSpec.$root[Te];
                    if (Ae.required) {
                      var Oe;
                      (Oe = Te === "version" ? Fe : Ae.type === "array" ? [] : {}) != null && (Ke[Te] = Oe);
                    }
                  }
                  return Ke;
                }(), ki = function(Ke) {
                  function Fe(Te, Ae) {
                    var Oe = this;
                    Ae === void 0 && (Ae = {}), Ke.call(this), this.map = Te, this.dispatcher = new Re(In(), this), this.imageManager = new ye(), this.imageManager.setEventedParent(this), this.glyphManager = new Ee(Te._requestManager, Ae.localIdeographFontFamily), this.lineAtlas = new Me(256, 512), this.crossTileSymbolIndex = new Ai(), this._layers = {}, this._serializedLayers = {}, this._order = [], this.sourceCaches = {}, this.zoomHistory = new ee.ZoomHistory(), this._loaded = !1, this._availableImages = [], this._resetUpdates(), this.dispatcher.broadcast("setReferrer", ee.getReferrer());
                    var Be = this;
                    this._rtlTextPluginCallback = Fe.registerForPluginStateChange(function(Ue) {
                      var nt = { pluginStatus: Ue.pluginStatus, pluginURL: Ue.pluginURL };
                      Be.dispatcher.broadcast("syncRTLPluginState", nt, function(ht, Et) {
                        if (ee.triggerPluginCompletionEvent(ht), Et && Et.every(function(qt) {
                          return qt;
                        })) for (var Mt in Be.sourceCaches) Be.sourceCaches[Mt].reload();
                      });
                    }), this.on("data", function(Ue) {
                      if (Ue.dataType === "source" && Ue.sourceDataType === "metadata") {
                        var nt = Oe.sourceCaches[Ue.sourceId];
                        if (nt) {
                          var ht = nt.getSource();
                          if (ht && ht.vectorLayerIds) for (var Et in Oe._layers) {
                            var Mt = Oe._layers[Et];
                            Mt.source === ht.id && Oe._validateLayer(Mt);
                          }
                        }
                      }
                    });
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.loadURL = function(Te, Ae) {
                    var Oe = this;
                    Ae === void 0 && (Ae = {}), this.fire(new ee.Event("dataloading", { dataType: "style" }));
                    var Be = typeof Ae.validate == "boolean" ? Ae.validate : !ee.isMapboxURL(Te);
                    Te = this.map._requestManager.normalizeStyleURL(Te, Ae.accessToken);
                    var Ue = this.map._requestManager.transformRequest(Te, ee.ResourceType.Style);
                    this._request = ee.getJSON(Ue, function(nt, ht) {
                      Oe._request = null, nt ? Oe.fire(new ee.ErrorEvent(nt)) : ht && Oe._load(ht, Be);
                    });
                  }, Fe.prototype.loadJSON = function(Te, Ae) {
                    var Oe = this;
                    Ae === void 0 && (Ae = {}), this.fire(new ee.Event("dataloading", { dataType: "style" })), this._request = ee.browser.frame(function() {
                      Oe._request = null, Oe._load(Te, Ae.validate !== !1);
                    });
                  }, Fe.prototype.loadEmpty = function() {
                    this.fire(new ee.Event("dataloading", { dataType: "style" })), this._load(Oi, !1);
                  }, Fe.prototype._load = function(Te, Ae) {
                    if (!Ae || !Li(this, ee.validateStyle(Te))) {
                      for (var Oe in this._loaded = !0, this.stylesheet = Te, Te.sources) this.addSource(Oe, Te.sources[Oe], { validate: !1 });
                      Te.sprite ? this._loadSprite(Te.sprite) : this.imageManager.setLoaded(!0), this.glyphManager.setURL(Te.glyphs);
                      var Be = er(this.stylesheet.layers);
                      this._order = Be.map(function(Et) {
                        return Et.id;
                      }), this._layers = {}, this._serializedLayers = {};
                      for (var Ue = 0, nt = Be; Ue < nt.length; Ue += 1) {
                        var ht = nt[Ue];
                        (ht = ee.createStyleLayer(ht)).setEventedParent(this, { layer: { id: ht.id } }), this._layers[ht.id] = ht, this._serializedLayers[ht.id] = ht.serialize();
                      }
                      this.dispatcher.broadcast("setLayers", this._serializeLayers(this._order)), this.light = new ke(this.stylesheet.light), this.fire(new ee.Event("data", { dataType: "style" })), this.fire(new ee.Event("style.load"));
                    }
                  }, Fe.prototype._loadSprite = function(Te) {
                    var Ae = this;
                    this._spriteRequest = function(Oe, Be, Ue) {
                      var nt, ht, Et, Mt = ee.browser.devicePixelRatio > 1 ? "@2x" : "", qt = ee.getJSON(Be.transformRequest(Be.normalizeSpriteURL(Oe, Mt, ".json"), ee.ResourceType.SpriteJSON), function(An, On) {
                        qt = null, Et || (Et = An, nt = On, xn());
                      }), tn = ee.getImage(Be.transformRequest(Be.normalizeSpriteURL(Oe, Mt, ".png"), ee.ResourceType.SpriteImage), function(An, On) {
                        tn = null, Et || (Et = An, ht = On, xn());
                      });
                      function xn() {
                        if (Et) Ue(Et);
                        else if (nt && ht) {
                          var An = ee.browser.getImageData(ht), On = {};
                          for (var Bn in nt) {
                            var qn = nt[Bn], Vn = qn.width, sr = qn.height, Un = qn.x, wr = qn.y, Cr = qn.sdf, Tr = qn.pixelRatio, ii = qn.stretchX, wi = qn.stretchY, Wi = qn.content, Yi = new ee.RGBAImage({ width: Vn, height: sr });
                            ee.RGBAImage.copy(An, Yi, { x: Un, y: wr }, { x: 0, y: 0 }, { width: Vn, height: sr }), On[Bn] = { data: Yi, pixelRatio: Tr, sdf: Cr, stretchX: ii, stretchY: wi, content: Wi };
                          }
                          Ue(null, On);
                        }
                      }
                      return { cancel: function() {
                        qt && (qt.cancel(), qt = null), tn && (tn.cancel(), tn = null);
                      } };
                    }(Te, this.map._requestManager, function(Oe, Be) {
                      if (Ae._spriteRequest = null, Oe) Ae.fire(new ee.ErrorEvent(Oe));
                      else if (Be) for (var Ue in Be) Ae.imageManager.addImage(Ue, Be[Ue]);
                      Ae.imageManager.setLoaded(!0), Ae._availableImages = Ae.imageManager.listImages(), Ae.dispatcher.broadcast("setImages", Ae._availableImages), Ae.fire(new ee.Event("data", { dataType: "style" }));
                    });
                  }, Fe.prototype._validateLayer = function(Te) {
                    var Ae = this.sourceCaches[Te.source];
                    if (Ae) {
                      var Oe = Te.sourceLayer;
                      if (Oe) {
                        var Be = Ae.getSource();
                        (Be.type === "geojson" || Be.vectorLayerIds && Be.vectorLayerIds.indexOf(Oe) === -1) && this.fire(new ee.ErrorEvent(new Error('Source layer "' + Oe + '" does not exist on source "' + Be.id + '" as specified by style layer "' + Te.id + '"')));
                      }
                    }
                  }, Fe.prototype.loaded = function() {
                    if (!this._loaded || Object.keys(this._updatedSources).length) return !1;
                    for (var Te in this.sourceCaches) if (!this.sourceCaches[Te].loaded()) return !1;
                    return !!this.imageManager.isLoaded();
                  }, Fe.prototype._serializeLayers = function(Te) {
                    for (var Ae = [], Oe = 0, Be = Te; Oe < Be.length; Oe += 1) {
                      var Ue = Be[Oe], nt = this._layers[Ue];
                      nt.type !== "custom" && Ae.push(nt.serialize());
                    }
                    return Ae;
                  }, Fe.prototype.hasTransitions = function() {
                    if (this.light && this.light.hasTransition()) return !0;
                    for (var Te in this.sourceCaches) if (this.sourceCaches[Te].hasTransition()) return !0;
                    for (var Ae in this._layers) if (this._layers[Ae].hasTransition()) return !0;
                    return !1;
                  }, Fe.prototype._checkLoaded = function() {
                    if (!this._loaded) throw new Error("Style is not done loading");
                  }, Fe.prototype.update = function(Te) {
                    if (this._loaded) {
                      var Ae = this._changed;
                      if (this._changed) {
                        var Oe = Object.keys(this._updatedLayers), Be = Object.keys(this._removedLayers);
                        for (var Ue in (Oe.length || Be.length) && this._updateWorkerLayers(Oe, Be), this._updatedSources) {
                          var nt = this._updatedSources[Ue];
                          nt === "reload" ? this._reloadSource(Ue) : nt === "clear" && this._clearSource(Ue);
                        }
                        for (var ht in this._updateTilesForChangedImages(), this._updatedPaintProps) this._layers[ht].updateTransitions(Te);
                        this.light.updateTransitions(Te), this._resetUpdates();
                      }
                      var Et = {};
                      for (var Mt in this.sourceCaches) {
                        var qt = this.sourceCaches[Mt];
                        Et[Mt] = qt.used, qt.used = !1;
                      }
                      for (var tn = 0, xn = this._order; tn < xn.length; tn += 1) {
                        var An = xn[tn], On = this._layers[An];
                        On.recalculate(Te, this._availableImages), !On.isHidden(Te.zoom) && On.source && (this.sourceCaches[On.source].used = !0);
                      }
                      for (var Bn in Et) {
                        var qn = this.sourceCaches[Bn];
                        Et[Bn] !== qn.used && qn.fire(new ee.Event("data", { sourceDataType: "visibility", dataType: "source", sourceId: Bn }));
                      }
                      this.light.recalculate(Te), this.z = Te.zoom, Ae && this.fire(new ee.Event("data", { dataType: "style" }));
                    }
                  }, Fe.prototype._updateTilesForChangedImages = function() {
                    var Te = Object.keys(this._changedImages);
                    if (Te.length) {
                      for (var Ae in this.sourceCaches) this.sourceCaches[Ae].reloadTilesForDependencies(["icons", "patterns"], Te);
                      this._changedImages = {};
                    }
                  }, Fe.prototype._updateWorkerLayers = function(Te, Ae) {
                    this.dispatcher.broadcast("updateLayers", { layers: this._serializeLayers(Te), removedIds: Ae });
                  }, Fe.prototype._resetUpdates = function() {
                    this._changed = !1, this._updatedLayers = {}, this._removedLayers = {}, this._updatedSources = {}, this._updatedPaintProps = {}, this._changedImages = {};
                  }, Fe.prototype.setState = function(Te) {
                    var Ae = this;
                    if (this._checkLoaded(), Li(this, ee.validateStyle(Te))) return !1;
                    (Te = ee.clone$1(Te)).layers = er(Te.layers);
                    var Oe = jr(this.serialize(), Te).filter(function(Ue) {
                      return !(Ue.command in Pi);
                    });
                    if (Oe.length === 0) return !1;
                    var Be = Oe.filter(function(Ue) {
                      return !(Ue.command in Ni);
                    });
                    if (Be.length > 0) throw new Error("Unimplemented: " + Be.map(function(Ue) {
                      return Ue.command;
                    }).join(", ") + ".");
                    return Oe.forEach(function(Ue) {
                      Ue.command !== "setTransition" && Ae[Ue.command].apply(Ae, Ue.args);
                    }), this.stylesheet = Te, !0;
                  }, Fe.prototype.addImage = function(Te, Ae) {
                    if (this.getImage(Te)) return this.fire(new ee.ErrorEvent(new Error("An image with this name already exists.")));
                    this.imageManager.addImage(Te, Ae), this._afterImageUpdated(Te);
                  }, Fe.prototype.updateImage = function(Te, Ae) {
                    this.imageManager.updateImage(Te, Ae);
                  }, Fe.prototype.getImage = function(Te) {
                    return this.imageManager.getImage(Te);
                  }, Fe.prototype.removeImage = function(Te) {
                    if (!this.getImage(Te)) return this.fire(new ee.ErrorEvent(new Error("No image with this name exists.")));
                    this.imageManager.removeImage(Te), this._afterImageUpdated(Te);
                  }, Fe.prototype._afterImageUpdated = function(Te) {
                    this._availableImages = this.imageManager.listImages(), this._changedImages[Te] = !0, this._changed = !0, this.dispatcher.broadcast("setImages", this._availableImages), this.fire(new ee.Event("data", { dataType: "style" }));
                  }, Fe.prototype.listImages = function() {
                    return this._checkLoaded(), this.imageManager.listImages();
                  }, Fe.prototype.addSource = function(Te, Ae, Oe) {
                    var Be = this;
                    if (Oe === void 0 && (Oe = {}), this._checkLoaded(), this.sourceCaches[Te] !== void 0) throw new Error("There is already a source with this ID");
                    if (!Ae.type) throw new Error("The type property must be defined, but only the following properties were given: " + Object.keys(Ae).join(", ") + ".");
                    if (!(["vector", "raster", "geojson", "video", "image"].indexOf(Ae.type) >= 0 && this._validate(ee.validateStyle.source, "sources." + Te, Ae, null, Oe))) {
                      this.map && this.map._collectResourceTiming && (Ae.collectResourceTiming = !0);
                      var Ue = this.sourceCaches[Te] = new yn(Te, Ae, this.dispatcher);
                      Ue.style = this, Ue.setEventedParent(this, function() {
                        return { isSourceLoaded: Be.loaded(), source: Ue.serialize(), sourceId: Te };
                      }), Ue.onAdd(this.map), this._changed = !0;
                    }
                  }, Fe.prototype.removeSource = function(Te) {
                    if (this._checkLoaded(), this.sourceCaches[Te] === void 0) throw new Error("There is no source with this ID");
                    for (var Ae in this._layers) if (this._layers[Ae].source === Te) return this.fire(new ee.ErrorEvent(new Error('Source "' + Te + '" cannot be removed while layer "' + Ae + '" is using it.')));
                    var Oe = this.sourceCaches[Te];
                    delete this.sourceCaches[Te], delete this._updatedSources[Te], Oe.fire(new ee.Event("data", { sourceDataType: "metadata", dataType: "source", sourceId: Te })), Oe.setEventedParent(null), Oe.clearTiles(), Oe.onRemove && Oe.onRemove(this.map), this._changed = !0;
                  }, Fe.prototype.setGeoJSONSourceData = function(Te, Ae) {
                    this._checkLoaded(), this.sourceCaches[Te].getSource().setData(Ae), this._changed = !0;
                  }, Fe.prototype.getSource = function(Te) {
                    return this.sourceCaches[Te] && this.sourceCaches[Te].getSource();
                  }, Fe.prototype.addLayer = function(Te, Ae, Oe) {
                    Oe === void 0 && (Oe = {}), this._checkLoaded();
                    var Be = Te.id;
                    if (this.getLayer(Be)) this.fire(new ee.ErrorEvent(new Error('Layer with id "' + Be + '" already exists on this map')));
                    else {
                      var Ue;
                      if (Te.type === "custom") {
                        if (Li(this, ee.validateCustomStyleLayer(Te))) return;
                        Ue = ee.createStyleLayer(Te);
                      } else {
                        if (typeof Te.source == "object" && (this.addSource(Be, Te.source), Te = ee.clone$1(Te), Te = ee.extend(Te, { source: Be })), this._validate(ee.validateStyle.layer, "layers." + Be, Te, { arrayIndex: -1 }, Oe)) return;
                        Ue = ee.createStyleLayer(Te), this._validateLayer(Ue), Ue.setEventedParent(this, { layer: { id: Be } }), this._serializedLayers[Ue.id] = Ue.serialize();
                      }
                      var nt = Ae ? this._order.indexOf(Ae) : this._order.length;
                      if (Ae && nt === -1) this.fire(new ee.ErrorEvent(new Error('Layer with id "' + Ae + '" does not exist on this map.')));
                      else {
                        if (this._order.splice(nt, 0, Be), this._layerOrderChanged = !0, this._layers[Be] = Ue, this._removedLayers[Be] && Ue.source && Ue.type !== "custom") {
                          var ht = this._removedLayers[Be];
                          delete this._removedLayers[Be], ht.type !== Ue.type ? this._updatedSources[Ue.source] = "clear" : (this._updatedSources[Ue.source] = "reload", this.sourceCaches[Ue.source].pause());
                        }
                        this._updateLayer(Ue), Ue.onAdd && Ue.onAdd(this.map);
                      }
                    }
                  }, Fe.prototype.moveLayer = function(Te, Ae) {
                    if (this._checkLoaded(), this._changed = !0, this._layers[Te]) {
                      if (Te !== Ae) {
                        var Oe = this._order.indexOf(Te);
                        this._order.splice(Oe, 1);
                        var Be = Ae ? this._order.indexOf(Ae) : this._order.length;
                        Ae && Be === -1 ? this.fire(new ee.ErrorEvent(new Error('Layer with id "' + Ae + '" does not exist on this map.'))) : (this._order.splice(Be, 0, Te), this._layerOrderChanged = !0);
                      }
                    } else this.fire(new ee.ErrorEvent(new Error("The layer '" + Te + "' does not exist in the map's style and cannot be moved.")));
                  }, Fe.prototype.removeLayer = function(Te) {
                    this._checkLoaded();
                    var Ae = this._layers[Te];
                    if (Ae) {
                      Ae.setEventedParent(null);
                      var Oe = this._order.indexOf(Te);
                      this._order.splice(Oe, 1), this._layerOrderChanged = !0, this._changed = !0, this._removedLayers[Te] = Ae, delete this._layers[Te], delete this._serializedLayers[Te], delete this._updatedLayers[Te], delete this._updatedPaintProps[Te], Ae.onRemove && Ae.onRemove(this.map);
                    } else this.fire(new ee.ErrorEvent(new Error("The layer '" + Te + "' does not exist in the map's style and cannot be removed.")));
                  }, Fe.prototype.getLayer = function(Te) {
                    return this._layers[Te];
                  }, Fe.prototype.hasLayer = function(Te) {
                    return Te in this._layers;
                  }, Fe.prototype.setLayerZoomRange = function(Te, Ae, Oe) {
                    this._checkLoaded();
                    var Be = this.getLayer(Te);
                    Be ? Be.minzoom === Ae && Be.maxzoom === Oe || (Ae != null && (Be.minzoom = Ae), Oe != null && (Be.maxzoom = Oe), this._updateLayer(Be)) : this.fire(new ee.ErrorEvent(new Error("The layer '" + Te + "' does not exist in the map's style and cannot have zoom extent.")));
                  }, Fe.prototype.setFilter = function(Te, Ae, Oe) {
                    Oe === void 0 && (Oe = {}), this._checkLoaded();
                    var Be = this.getLayer(Te);
                    if (Be) {
                      if (!ee.deepEqual(Be.filter, Ae)) return Ae == null ? (Be.filter = void 0, void this._updateLayer(Be)) : void (this._validate(ee.validateStyle.filter, "layers." + Be.id + ".filter", Ae, null, Oe) || (Be.filter = ee.clone$1(Ae), this._updateLayer(Be)));
                    } else this.fire(new ee.ErrorEvent(new Error("The layer '" + Te + "' does not exist in the map's style and cannot be filtered.")));
                  }, Fe.prototype.getFilter = function(Te) {
                    return ee.clone$1(this.getLayer(Te).filter);
                  }, Fe.prototype.setLayoutProperty = function(Te, Ae, Oe, Be) {
                    Be === void 0 && (Be = {}), this._checkLoaded();
                    var Ue = this.getLayer(Te);
                    Ue ? ee.deepEqual(Ue.getLayoutProperty(Ae), Oe) || (Ue.setLayoutProperty(Ae, Oe, Be), this._updateLayer(Ue)) : this.fire(new ee.ErrorEvent(new Error("The layer '" + Te + "' does not exist in the map's style and cannot be styled.")));
                  }, Fe.prototype.getLayoutProperty = function(Te, Ae) {
                    var Oe = this.getLayer(Te);
                    if (Oe) return Oe.getLayoutProperty(Ae);
                    this.fire(new ee.ErrorEvent(new Error("The layer '" + Te + "' does not exist in the map's style.")));
                  }, Fe.prototype.setPaintProperty = function(Te, Ae, Oe, Be) {
                    Be === void 0 && (Be = {}), this._checkLoaded();
                    var Ue = this.getLayer(Te);
                    Ue ? ee.deepEqual(Ue.getPaintProperty(Ae), Oe) || (Ue.setPaintProperty(Ae, Oe, Be) && this._updateLayer(Ue), this._changed = !0, this._updatedPaintProps[Te] = !0) : this.fire(new ee.ErrorEvent(new Error("The layer '" + Te + "' does not exist in the map's style and cannot be styled.")));
                  }, Fe.prototype.getPaintProperty = function(Te, Ae) {
                    return this.getLayer(Te).getPaintProperty(Ae);
                  }, Fe.prototype.setFeatureState = function(Te, Ae) {
                    this._checkLoaded();
                    var Oe = Te.source, Be = Te.sourceLayer, Ue = this.sourceCaches[Oe];
                    if (Ue !== void 0) {
                      var nt = Ue.getSource().type;
                      nt === "geojson" && Be ? this.fire(new ee.ErrorEvent(new Error("GeoJSON sources cannot have a sourceLayer parameter."))) : nt !== "vector" || Be ? (Te.id === void 0 && this.fire(new ee.ErrorEvent(new Error("The feature id parameter must be provided."))), Ue.setFeatureState(Be, Te.id, Ae)) : this.fire(new ee.ErrorEvent(new Error("The sourceLayer parameter must be provided for vector source types.")));
                    } else this.fire(new ee.ErrorEvent(new Error("The source '" + Oe + "' does not exist in the map's style.")));
                  }, Fe.prototype.removeFeatureState = function(Te, Ae) {
                    this._checkLoaded();
                    var Oe = Te.source, Be = this.sourceCaches[Oe];
                    if (Be !== void 0) {
                      var Ue = Be.getSource().type, nt = Ue === "vector" ? Te.sourceLayer : void 0;
                      Ue !== "vector" || nt ? Ae && typeof Te.id != "string" && typeof Te.id != "number" ? this.fire(new ee.ErrorEvent(new Error("A feature id is required to remove its specific state property."))) : Be.removeFeatureState(nt, Te.id, Ae) : this.fire(new ee.ErrorEvent(new Error("The sourceLayer parameter must be provided for vector source types.")));
                    } else this.fire(new ee.ErrorEvent(new Error("The source '" + Oe + "' does not exist in the map's style.")));
                  }, Fe.prototype.getFeatureState = function(Te) {
                    this._checkLoaded();
                    var Ae = Te.source, Oe = Te.sourceLayer, Be = this.sourceCaches[Ae];
                    if (Be !== void 0) {
                      if (Be.getSource().type !== "vector" || Oe) return Te.id === void 0 && this.fire(new ee.ErrorEvent(new Error("The feature id parameter must be provided."))), Be.getFeatureState(Oe, Te.id);
                      this.fire(new ee.ErrorEvent(new Error("The sourceLayer parameter must be provided for vector source types.")));
                    } else this.fire(new ee.ErrorEvent(new Error("The source '" + Ae + "' does not exist in the map's style.")));
                  }, Fe.prototype.getTransition = function() {
                    return ee.extend({ duration: 300, delay: 0 }, this.stylesheet && this.stylesheet.transition);
                  }, Fe.prototype.serialize = function() {
                    return ee.filterObject({ version: this.stylesheet.version, name: this.stylesheet.name, metadata: this.stylesheet.metadata, light: this.stylesheet.light, center: this.stylesheet.center, zoom: this.stylesheet.zoom, bearing: this.stylesheet.bearing, pitch: this.stylesheet.pitch, sprite: this.stylesheet.sprite, glyphs: this.stylesheet.glyphs, transition: this.stylesheet.transition, sources: ee.mapObject(this.sourceCaches, function(Te) {
                      return Te.serialize();
                    }), layers: this._serializeLayers(this._order) }, function(Te) {
                      return Te !== void 0;
                    });
                  }, Fe.prototype._updateLayer = function(Te) {
                    this._updatedLayers[Te.id] = !0, Te.source && !this._updatedSources[Te.source] && this.sourceCaches[Te.source].getSource().type !== "raster" && (this._updatedSources[Te.source] = "reload", this.sourceCaches[Te.source].pause()), this._changed = !0;
                  }, Fe.prototype._flattenAndSortRenderedFeatures = function(Te) {
                    for (var Ae = this, Oe = function(Wi) {
                      return Ae._layers[Wi].type === "fill-extrusion";
                    }, Be = {}, Ue = [], nt = this._order.length - 1; nt >= 0; nt--) {
                      var ht = this._order[nt];
                      if (Oe(ht)) {
                        Be[ht] = nt;
                        for (var Et = 0, Mt = Te; Et < Mt.length; Et += 1) {
                          var qt = Mt[Et][ht];
                          if (qt) for (var tn = 0, xn = qt; tn < xn.length; tn += 1) {
                            var An = xn[tn];
                            Ue.push(An);
                          }
                        }
                      }
                    }
                    Ue.sort(function(Wi, Yi) {
                      return Yi.intersectionZ - Wi.intersectionZ;
                    });
                    for (var On = [], Bn = this._order.length - 1; Bn >= 0; Bn--) {
                      var qn = this._order[Bn];
                      if (Oe(qn)) for (var Vn = Ue.length - 1; Vn >= 0; Vn--) {
                        var sr = Ue[Vn].feature;
                        if (Be[sr.layer.id] < Bn) break;
                        On.push(sr), Ue.pop();
                      }
                      else for (var Un = 0, wr = Te; Un < wr.length; Un += 1) {
                        var Cr = wr[Un][qn];
                        if (Cr) for (var Tr = 0, ii = Cr; Tr < ii.length; Tr += 1) {
                          var wi = ii[Tr];
                          On.push(wi.feature);
                        }
                      }
                    }
                    return On;
                  }, Fe.prototype.queryRenderedFeatures = function(Te, Ae, Oe) {
                    Ae && Ae.filter && this._validate(ee.validateStyle.filter, "queryRenderedFeatures.filter", Ae.filter, null, Ae);
                    var Be = {};
                    if (Ae && Ae.layers) {
                      if (!Array.isArray(Ae.layers)) return this.fire(new ee.ErrorEvent(new Error("parameters.layers must be an Array."))), [];
                      for (var Ue = 0, nt = Ae.layers; Ue < nt.length; Ue += 1) {
                        var ht = nt[Ue], Et = this._layers[ht];
                        if (!Et) return this.fire(new ee.ErrorEvent(new Error("The layer '" + ht + "' does not exist in the map's style and cannot be queried for features."))), [];
                        Be[Et.source] = !0;
                      }
                    }
                    var Mt = [];
                    for (var qt in Ae.availableImages = this._availableImages, this.sourceCaches) Ae.layers && !Be[qt] || Mt.push(et(this.sourceCaches[qt], this._layers, this._serializedLayers, Te, Ae, Oe));
                    return this.placement && Mt.push(function(tn, xn, An, On, Bn, qn, Vn) {
                      for (var sr = {}, Un = qn.queryRenderedSymbols(On), wr = [], Cr = 0, Tr = Object.keys(Un).map(Number); Cr < Tr.length; Cr += 1) {
                        var ii = Tr[Cr];
                        wr.push(Vn[ii]);
                      }
                      wr.sort(dt);
                      for (var wi = function() {
                        var ya = Yi[Wi], va = ya.featureIndex.lookupSymbolFeatures(Un[ya.bucketInstanceId], xn, ya.bucketIndex, ya.sourceLayerIndex, Bn.filter, Bn.layers, Bn.availableImages, tn);
                        for (var qa in va) {
                          var Aa = sr[qa] = sr[qa] || [], Xa = va[qa];
                          Xa.sort(function(Ja, Uo) {
                            var ys = ya.featureSortOrder;
                            if (ys) {
                              var il = ys.indexOf(Ja.featureIndex);
                              return ys.indexOf(Uo.featureIndex) - il;
                            }
                            return Uo.featureIndex - Ja.featureIndex;
                          });
                          for (var Oa = 0, Ya = Xa; Oa < Ya.length; Oa += 1) {
                            var Wo = Ya[Oa];
                            Aa.push(Wo);
                          }
                        }
                      }, Wi = 0, Yi = wr; Wi < Yi.length; Wi += 1) wi();
                      var ra = function(ya) {
                        sr[ya].forEach(function(va) {
                          var qa = va.feature, Aa = tn[ya], Xa = An[Aa.source].getFeatureState(qa.layer["source-layer"], qa.id);
                          qa.source = qa.layer.source, qa.layer["source-layer"] && (qa.sourceLayer = qa.layer["source-layer"]), qa.state = Xa;
                        });
                      };
                      for (var ha in sr) ra(ha);
                      return sr;
                    }(this._layers, this._serializedLayers, this.sourceCaches, Te, Ae, this.placement.collisionIndex, this.placement.retainedQueryData)), this._flattenAndSortRenderedFeatures(Mt);
                  }, Fe.prototype.querySourceFeatures = function(Te, Ae) {
                    Ae && Ae.filter && this._validate(ee.validateStyle.filter, "querySourceFeatures.filter", Ae.filter, null, Ae);
                    var Oe = this.sourceCaches[Te];
                    return Oe ? function(Be, Ue) {
                      for (var nt = Be.getRenderableIds().map(function(xn) {
                        return Be.getTileByID(xn);
                      }), ht = [], Et = {}, Mt = 0; Mt < nt.length; Mt++) {
                        var qt = nt[Mt], tn = qt.tileID.canonical.key;
                        Et[tn] || (Et[tn] = !0, qt.querySourceFeatures(ht, Ue));
                      }
                      return ht;
                    }(Oe, Ae) : [];
                  }, Fe.prototype.addSourceType = function(Te, Ae, Oe) {
                    return Fe.getSourceType(Te) ? Oe(new Error('A source type called "' + Te + '" already exists.')) : (Fe.setSourceType(Te, Ae), Ae.workerSourceURL ? void this.dispatcher.broadcast("loadWorkerSource", { name: Te, url: Ae.workerSourceURL }, Oe) : Oe(null, null));
                  }, Fe.prototype.getLight = function() {
                    return this.light.getLight();
                  }, Fe.prototype.setLight = function(Te, Ae) {
                    Ae === void 0 && (Ae = {}), this._checkLoaded();
                    var Oe = this.light.getLight(), Be = !1;
                    for (var Ue in Te) if (!ee.deepEqual(Te[Ue], Oe[Ue])) {
                      Be = !0;
                      break;
                    }
                    if (Be) {
                      var nt = { now: ee.browser.now(), transition: ee.extend({ duration: 300, delay: 0 }, this.stylesheet.transition) };
                      this.light.setLight(Te, Ae), this.light.updateTransitions(nt);
                    }
                  }, Fe.prototype._validate = function(Te, Ae, Oe, Be, Ue) {
                    return Ue === void 0 && (Ue = {}), (!Ue || Ue.validate !== !1) && Li(this, Te.call(ee.validateStyle, ee.extend({ key: Ae, style: this.serialize(), value: Oe, styleSpec: ee.styleSpec }, Be)));
                  }, Fe.prototype._remove = function() {
                    for (var Te in this._request && (this._request.cancel(), this._request = null), this._spriteRequest && (this._spriteRequest.cancel(), this._spriteRequest = null), ee.evented.off("pluginStateChange", this._rtlTextPluginCallback), this._layers) this._layers[Te].setEventedParent(null);
                    for (var Ae in this.sourceCaches) this.sourceCaches[Ae].clearTiles(), this.sourceCaches[Ae].setEventedParent(null);
                    this.imageManager.setEventedParent(null), this.setEventedParent(null), this.dispatcher.remove();
                  }, Fe.prototype._clearSource = function(Te) {
                    this.sourceCaches[Te].clearTiles();
                  }, Fe.prototype._reloadSource = function(Te) {
                    this.sourceCaches[Te].resume(), this.sourceCaches[Te].reload();
                  }, Fe.prototype._updateSources = function(Te) {
                    for (var Ae in this.sourceCaches) this.sourceCaches[Ae].update(Te);
                  }, Fe.prototype._generateCollisionBoxes = function() {
                    for (var Te in this.sourceCaches) this._reloadSource(Te);
                  }, Fe.prototype._updatePlacement = function(Te, Ae, Oe, Be, Ue) {
                    Ue === void 0 && (Ue = !1);
                    for (var nt = !1, ht = !1, Et = {}, Mt = 0, qt = this._order; Mt < qt.length; Mt += 1) {
                      var tn = qt[Mt], xn = this._layers[tn];
                      if (xn.type === "symbol") {
                        if (!Et[xn.source]) {
                          var An = this.sourceCaches[xn.source];
                          Et[xn.source] = An.getRenderableIds(!0).map(function(Un) {
                            return An.getTileByID(Un);
                          }).sort(function(Un, wr) {
                            return wr.tileID.overscaledZ - Un.tileID.overscaledZ || (Un.tileID.isLessThan(wr.tileID) ? -1 : 1);
                          });
                        }
                        var On = this.crossTileSymbolIndex.addLayer(xn, Et[xn.source], Te.center.lng);
                        nt = nt || On;
                      }
                    }
                    if (this.crossTileSymbolIndex.pruneUnusedLayers(this._order), ((Ue = Ue || this._layerOrderChanged || Oe === 0) || !this.pauseablePlacement || this.pauseablePlacement.isDone() && !this.placement.stillRecent(ee.browser.now(), Te.zoom)) && (this.pauseablePlacement = new Er(Te, this._order, Ue, Ae, Oe, Be, this.placement), this._layerOrderChanged = !1), this.pauseablePlacement.isDone() ? this.placement.setStale() : (this.pauseablePlacement.continuePlacement(this._order, this._layers, Et), this.pauseablePlacement.isDone() && (this.placement = this.pauseablePlacement.commit(ee.browser.now()), ht = !0), nt && this.pauseablePlacement.placement.setStale()), ht || nt) for (var Bn = 0, qn = this._order; Bn < qn.length; Bn += 1) {
                      var Vn = qn[Bn], sr = this._layers[Vn];
                      sr.type === "symbol" && this.placement.updateLayerOpacities(sr, Et[sr.source]);
                    }
                    return !this.pauseablePlacement.isDone() || this.placement.hasTransitions(ee.browser.now());
                  }, Fe.prototype._releaseSymbolFadeTiles = function() {
                    for (var Te in this.sourceCaches) this.sourceCaches[Te].releaseSymbolFadeTiles();
                  }, Fe.prototype.getImages = function(Te, Ae, Oe) {
                    this.imageManager.getImages(Ae.icons, Oe), this._updateTilesForChangedImages();
                    var Be = this.sourceCaches[Ae.source];
                    Be && Be.setDependencies(Ae.tileID.key, Ae.type, Ae.icons);
                  }, Fe.prototype.getGlyphs = function(Te, Ae, Oe) {
                    this.glyphManager.getGlyphs(Ae.stacks, Oe);
                  }, Fe.prototype.getResource = function(Te, Ae, Oe) {
                    return ee.makeRequest(Ae, Oe);
                  }, Fe;
                }(ee.Evented);
                ki.getSourceType = function(Ke) {
                  return at[Ke];
                }, ki.setSourceType = function(Ke, Fe) {
                  at[Ke] = Fe;
                }, ki.registerForPluginStateChange = ee.registerForPluginStateChange;
                var Mi = ee.createLayout([{ name: "a_pos", type: "Int16", components: 2 }]), Vi = vs(`#ifdef GL_ES
precision mediump float;
#else
#if !defined(lowp)
#define lowp
#endif
#if !defined(mediump)
#define mediump
#endif
#if !defined(highp)
#define highp
#endif
#endif`, `#ifdef GL_ES
precision highp float;
#else
#if !defined(lowp)
#define lowp
#endif
#if !defined(mediump)
#define mediump
#endif
#if !defined(highp)
#define highp
#endif
#endif
vec2 unpack_float(const float packedValue) {int packedIntValue=int(packedValue);int v0=packedIntValue/256;return vec2(v0,packedIntValue-v0*256);}vec2 unpack_opacity(const float packedOpacity) {int intOpacity=int(packedOpacity)/2;return vec2(float(intOpacity)/127.0,mod(packedOpacity,2.0));}vec4 decode_color(const vec2 encodedColor) {return vec4(unpack_float(encodedColor[0])/255.0,unpack_float(encodedColor[1])/255.0
);}float unpack_mix_vec2(const vec2 packedValue,const float t) {return mix(packedValue[0],packedValue[1],t);}vec4 unpack_mix_color(const vec4 packedColors,const float t) {vec4 minColor=decode_color(vec2(packedColors[0],packedColors[1]));vec4 maxColor=decode_color(vec2(packedColors[2],packedColors[3]));return mix(minColor,maxColor,t);}vec2 get_pattern_pos(const vec2 pixel_coord_upper,const vec2 pixel_coord_lower,const vec2 pattern_size,const float tile_units_to_pixels,const vec2 pos) {vec2 offset=mod(mod(mod(pixel_coord_upper,pattern_size)*256.0,pattern_size)*256.0+pixel_coord_lower,pattern_size);return (tile_units_to_pixels*pos+offset)/pattern_size;}`), ua = vs(`uniform vec4 u_color;uniform float u_opacity;void main() {gl_FragColor=u_color*u_opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "attribute vec2 a_pos;uniform mat4 u_matrix;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);}"), Ki = vs(`uniform vec2 u_pattern_tl_a;uniform vec2 u_pattern_br_a;uniform vec2 u_pattern_tl_b;uniform vec2 u_pattern_br_b;uniform vec2 u_texsize;uniform float u_mix;uniform float u_opacity;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;void main() {vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(u_pattern_tl_a/u_texsize,u_pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(u_pattern_tl_b/u_texsize,u_pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);gl_FragColor=mix(color1,color2,u_mix)*u_opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_pattern_size_a;uniform vec2 u_pattern_size_b;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform float u_scale_a;uniform float u_scale_b;uniform float u_tile_units_to_pixels;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,u_scale_a*u_pattern_size_a,u_tile_units_to_pixels,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,u_scale_b*u_pattern_size_b,u_tile_units_to_pixels,a_pos);}"), Fi = vs(`varying vec3 v_data;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define mediump float radius
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define highp vec4 stroke_color
#pragma mapbox: define mediump float stroke_width
#pragma mapbox: define lowp float stroke_opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize mediump float radius
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize highp vec4 stroke_color
#pragma mapbox: initialize mediump float stroke_width
#pragma mapbox: initialize lowp float stroke_opacity
vec2 extrude=v_data.xy;float extrude_length=length(extrude);lowp float antialiasblur=v_data.z;float antialiased_blur=-max(blur,antialiasblur);float opacity_t=smoothstep(0.0,antialiased_blur,extrude_length-1.0);float color_t=stroke_width < 0.01 ? 0.0 : smoothstep(antialiased_blur,0.0,extrude_length-radius/(radius+stroke_width));gl_FragColor=opacity_t*mix(color*opacity,stroke_color*stroke_opacity,color_t);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform bool u_scale_with_map;uniform bool u_pitch_with_map;uniform vec2 u_extrude_scale;uniform lowp float u_device_pixel_ratio;uniform highp float u_camera_to_center_distance;attribute vec2 a_pos;varying vec3 v_data;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define mediump float radius
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define highp vec4 stroke_color
#pragma mapbox: define mediump float stroke_width
#pragma mapbox: define lowp float stroke_opacity
void main(void) {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize mediump float radius
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize highp vec4 stroke_color
#pragma mapbox: initialize mediump float stroke_width
#pragma mapbox: initialize lowp float stroke_opacity
vec2 extrude=vec2(mod(a_pos,2.0)*2.0-1.0);vec2 circle_center=floor(a_pos*0.5);if (u_pitch_with_map) {vec2 corner_position=circle_center;if (u_scale_with_map) {corner_position+=extrude*(radius+stroke_width)*u_extrude_scale;} else {vec4 projected_center=u_matrix*vec4(circle_center,0,1);corner_position+=extrude*(radius+stroke_width)*u_extrude_scale*(projected_center.w/u_camera_to_center_distance);}gl_Position=u_matrix*vec4(corner_position,0,1);} else {gl_Position=u_matrix*vec4(circle_center,0,1);if (u_scale_with_map) {gl_Position.xy+=extrude*(radius+stroke_width)*u_extrude_scale*u_camera_to_center_distance;} else {gl_Position.xy+=extrude*(radius+stroke_width)*u_extrude_scale*gl_Position.w;}}lowp float antialiasblur=1.0/u_device_pixel_ratio/(radius+stroke_width);v_data=vec3(extrude.x,extrude.y,antialiasblur);}`), na = vs("void main() {gl_FragColor=vec4(1.0);}", "attribute vec2 a_pos;uniform mat4 u_matrix;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);}"), nn = vs(`uniform highp float u_intensity;varying vec2 v_extrude;
#pragma mapbox: define highp float weight
#define GAUSS_COEF 0.3989422804014327
void main() {
#pragma mapbox: initialize highp float weight
float d=-0.5*3.0*3.0*dot(v_extrude,v_extrude);float val=weight*u_intensity*GAUSS_COEF*exp(d);gl_FragColor=vec4(val,1.0,1.0,1.0);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform float u_extrude_scale;uniform float u_opacity;uniform float u_intensity;attribute vec2 a_pos;varying vec2 v_extrude;
#pragma mapbox: define highp float weight
#pragma mapbox: define mediump float radius
const highp float ZERO=1.0/255.0/16.0;
#define GAUSS_COEF 0.3989422804014327
void main(void) {
#pragma mapbox: initialize highp float weight
#pragma mapbox: initialize mediump float radius
vec2 unscaled_extrude=vec2(mod(a_pos,2.0)*2.0-1.0);float S=sqrt(-2.0*log(ZERO/weight/u_intensity/GAUSS_COEF))/3.0;v_extrude=S*unscaled_extrude;vec2 extrude=v_extrude*radius*u_extrude_scale;vec4 pos=vec4(floor(a_pos*0.5)+extrude,0,1);gl_Position=u_matrix*pos;}`), Tn = vs(`uniform sampler2D u_image;uniform sampler2D u_color_ramp;uniform float u_opacity;varying vec2 v_pos;void main() {float t=texture2D(u_image,v_pos).r;vec4 color=texture2D(u_color_ramp,vec2(t,0.5));gl_FragColor=color*u_opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(0.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_world;attribute vec2 a_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos*u_world,0,1);v_pos.x=a_pos.x;v_pos.y=1.0-a_pos.y;}"), Dn = vs("varying float v_placed;varying float v_notUsed;void main() {float alpha=0.5;gl_FragColor=vec4(1.0,0.0,0.0,1.0)*alpha;if (v_placed > 0.5) {gl_FragColor=vec4(0.0,0.0,1.0,0.5)*alpha;}if (v_notUsed > 0.5) {gl_FragColor*=.1;}}", "attribute vec2 a_pos;attribute vec2 a_anchor_pos;attribute vec2 a_extrude;attribute vec2 a_placed;attribute vec2 a_shift;uniform mat4 u_matrix;uniform vec2 u_extrude_scale;uniform float u_camera_to_center_distance;varying float v_placed;varying float v_notUsed;void main() {vec4 projectedPoint=u_matrix*vec4(a_anchor_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float collision_perspective_ratio=clamp(0.5+0.5*(u_camera_to_center_distance/camera_to_anchor_distance),0.0,4.0);gl_Position=u_matrix*vec4(a_pos,0.0,1.0);gl_Position.xy+=(a_extrude+a_shift)*u_extrude_scale*gl_Position.w*collision_perspective_ratio;v_placed=a_placed.x;v_notUsed=a_placed.y;}"), Fn = vs("varying float v_radius;varying vec2 v_extrude;varying float v_perspective_ratio;varying float v_collision;void main() {float alpha=0.5*min(v_perspective_ratio,1.0);float stroke_radius=0.9*max(v_perspective_ratio,1.0);float distance_to_center=length(v_extrude);float distance_to_edge=abs(distance_to_center-v_radius);float opacity_t=smoothstep(-stroke_radius,0.0,-distance_to_edge);vec4 color=mix(vec4(0.0,0.0,1.0,0.5),vec4(1.0,0.0,0.0,1.0),v_collision);gl_FragColor=color*alpha*opacity_t;}", "attribute vec2 a_pos;attribute float a_radius;attribute vec2 a_flags;uniform mat4 u_matrix;uniform mat4 u_inv_matrix;uniform vec2 u_viewport_size;uniform float u_camera_to_center_distance;varying float v_radius;varying vec2 v_extrude;varying float v_perspective_ratio;varying float v_collision;vec3 toTilePosition(vec2 screenPos) {vec4 rayStart=u_inv_matrix*vec4(screenPos,-1.0,1.0);vec4 rayEnd  =u_inv_matrix*vec4(screenPos, 1.0,1.0);rayStart.xyz/=rayStart.w;rayEnd.xyz  /=rayEnd.w;highp float t=(0.0-rayStart.z)/(rayEnd.z-rayStart.z);return mix(rayStart.xyz,rayEnd.xyz,t);}void main() {vec2 quadCenterPos=a_pos;float radius=a_radius;float collision=a_flags.x;float vertexIdx=a_flags.y;vec2 quadVertexOffset=vec2(mix(-1.0,1.0,float(vertexIdx >=2.0)),mix(-1.0,1.0,float(vertexIdx >=1.0 && vertexIdx <=2.0)));vec2 quadVertexExtent=quadVertexOffset*radius;vec3 tilePos=toTilePosition(quadCenterPos);vec4 clipPos=u_matrix*vec4(tilePos,1.0);highp float camera_to_anchor_distance=clipPos.w;highp float collision_perspective_ratio=clamp(0.5+0.5*(u_camera_to_center_distance/camera_to_anchor_distance),0.0,4.0);float padding_factor=1.2;v_radius=radius;v_extrude=quadVertexExtent*padding_factor;v_perspective_ratio=collision_perspective_ratio;v_collision=collision;gl_Position=vec4(clipPos.xyz/clipPos.w,1.0)+vec4(quadVertexExtent*padding_factor/u_viewport_size*2.0,0.0,0.0);}"), fr = vs("uniform highp vec4 u_color;uniform sampler2D u_overlay;varying vec2 v_uv;void main() {vec4 overlay_color=texture2D(u_overlay,v_uv);gl_FragColor=mix(u_color,overlay_color,overlay_color.a);}", "attribute vec2 a_pos;varying vec2 v_uv;uniform mat4 u_matrix;uniform float u_overlay_scale;void main() {v_uv=a_pos/8192.0;gl_Position=u_matrix*vec4(a_pos*u_overlay_scale,0,1);}"), dr = vs(`#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float opacity
gl_FragColor=color*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `attribute vec2 a_pos;uniform mat4 u_matrix;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float opacity
gl_Position=u_matrix*vec4(a_pos,0,1);}`), yr = vs(`varying vec2 v_pos;
#pragma mapbox: define highp vec4 outline_color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 outline_color
#pragma mapbox: initialize lowp float opacity
float dist=length(v_pos-gl_FragCoord.xy);float alpha=1.0-smoothstep(0.0,1.0,dist);gl_FragColor=outline_color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `attribute vec2 a_pos;uniform mat4 u_matrix;uniform vec2 u_world;varying vec2 v_pos;
#pragma mapbox: define highp vec4 outline_color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 outline_color
#pragma mapbox: initialize lowp float opacity
gl_Position=u_matrix*vec4(a_pos,0,1);v_pos=(gl_Position.xy/gl_Position.w+1.0)/2.0*u_world;}`), ai = vs(`uniform vec2 u_texsize;uniform sampler2D u_image;uniform float u_fade;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec2 v_pos;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);float dist=length(v_pos-gl_FragCoord.xy);float alpha=1.0-smoothstep(0.0,1.0,dist);gl_FragColor=mix(color1,color2,u_fade)*alpha*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec2 u_world;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform vec3 u_scale;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec2 v_pos;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;gl_Position=u_matrix*vec4(a_pos,0,1);vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileRatio,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileRatio,a_pos);v_pos=(gl_Position.xy/gl_Position.w+1.0)/2.0*u_world;}`), Fr = vs(`uniform vec2 u_texsize;uniform float u_fade;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);gl_FragColor=mix(color1,color2,u_fade)*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform vec3 u_scale;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileZoomRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;gl_Position=u_matrix*vec4(a_pos,0,1);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileZoomRatio,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileZoomRatio,a_pos);}`), Ti = vs(`varying vec4 v_color;void main() {gl_FragColor=v_color;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec3 u_lightcolor;uniform lowp vec3 u_lightpos;uniform lowp float u_lightintensity;uniform float u_vertical_gradient;uniform lowp float u_opacity;attribute vec2 a_pos;attribute vec4 a_normal_ed;varying vec4 v_color;
#pragma mapbox: define highp float base
#pragma mapbox: define highp float height
#pragma mapbox: define highp vec4 color
void main() {
#pragma mapbox: initialize highp float base
#pragma mapbox: initialize highp float height
#pragma mapbox: initialize highp vec4 color
vec3 normal=a_normal_ed.xyz;base=max(0.0,base);height=max(0.0,height);float t=mod(normal.x,2.0);gl_Position=u_matrix*vec4(a_pos,t > 0.0 ? height : base,1);float colorvalue=color.r*0.2126+color.g*0.7152+color.b*0.0722;v_color=vec4(0.0,0.0,0.0,1.0);vec4 ambientlight=vec4(0.03,0.03,0.03,1.0);color+=ambientlight;float directional=clamp(dot(normal/16384.0,u_lightpos),0.0,1.0);directional=mix((1.0-u_lightintensity),max((1.0-colorvalue+u_lightintensity),1.0),directional);if (normal.y !=0.0) {directional*=((1.0-u_vertical_gradient)+(u_vertical_gradient*clamp((t+base)*pow(height/150.0,0.5),mix(0.7,0.98,1.0-u_lightintensity),1.0)));}v_color.r+=clamp(color.r*directional*u_lightcolor.r,mix(0.0,0.3,1.0-u_lightcolor.r),1.0);v_color.g+=clamp(color.g*directional*u_lightcolor.g,mix(0.0,0.3,1.0-u_lightcolor.g),1.0);v_color.b+=clamp(color.b*directional*u_lightcolor.b,mix(0.0,0.3,1.0-u_lightcolor.b),1.0);v_color*=u_opacity;}`), Si = vs(`uniform vec2 u_texsize;uniform float u_fade;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec4 v_lighting;
#pragma mapbox: define lowp float base
#pragma mapbox: define lowp float height
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float base
#pragma mapbox: initialize lowp float height
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);vec4 mixedColor=mix(color1,color2,u_fade);gl_FragColor=mixedColor*v_lighting;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform float u_height_factor;uniform vec3 u_scale;uniform float u_vertical_gradient;uniform lowp float u_opacity;uniform vec3 u_lightcolor;uniform lowp vec3 u_lightpos;uniform lowp float u_lightintensity;attribute vec2 a_pos;attribute vec4 a_normal_ed;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec4 v_lighting;
#pragma mapbox: define lowp float base
#pragma mapbox: define lowp float height
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float base
#pragma mapbox: initialize lowp float height
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;vec3 normal=a_normal_ed.xyz;float edgedistance=a_normal_ed.w;vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;base=max(0.0,base);height=max(0.0,height);float t=mod(normal.x,2.0);float z=t > 0.0 ? height : base;gl_Position=u_matrix*vec4(a_pos,z,1);vec2 pos=normal.x==1.0 && normal.y==0.0 && normal.z==16384.0
? a_pos
: vec2(edgedistance,z*u_height_factor);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileRatio,pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileRatio,pos);v_lighting=vec4(0.0,0.0,0.0,1.0);float directional=clamp(dot(normal/16383.0,u_lightpos),0.0,1.0);directional=mix((1.0-u_lightintensity),max((0.5+u_lightintensity),1.0),directional);if (normal.y !=0.0) {directional*=((1.0-u_vertical_gradient)+(u_vertical_gradient*clamp((t+base)*pow(height/150.0,0.5),mix(0.7,0.98,1.0-u_lightintensity),1.0)));}v_lighting.rgb+=clamp(directional*u_lightcolor,mix(vec3(0.0),vec3(0.3),1.0-u_lightcolor),vec3(1.0));v_lighting*=u_opacity;}`), Ei = vs(`#ifdef GL_ES
precision highp float;
#endif
uniform sampler2D u_image;varying vec2 v_pos;uniform vec2 u_dimension;uniform float u_zoom;uniform vec4 u_unpack;float getElevation(vec2 coord,float bias) {vec4 data=texture2D(u_image,coord)*255.0;data.a=-1.0;return dot(data,u_unpack)/4.0;}void main() {vec2 epsilon=1.0/u_dimension;float a=getElevation(v_pos+vec2(-epsilon.x,-epsilon.y),0.0);float b=getElevation(v_pos+vec2(0,-epsilon.y),0.0);float c=getElevation(v_pos+vec2(epsilon.x,-epsilon.y),0.0);float d=getElevation(v_pos+vec2(-epsilon.x,0),0.0);float e=getElevation(v_pos,0.0);float f=getElevation(v_pos+vec2(epsilon.x,0),0.0);float g=getElevation(v_pos+vec2(-epsilon.x,epsilon.y),0.0);float h=getElevation(v_pos+vec2(0,epsilon.y),0.0);float i=getElevation(v_pos+vec2(epsilon.x,epsilon.y),0.0);float exaggerationFactor=u_zoom < 2.0 ? 0.4 : u_zoom < 4.5 ? 0.35 : 0.3;float exaggeration=u_zoom < 15.0 ? (u_zoom-15.0)*exaggerationFactor : 0.0;vec2 deriv=vec2((c+f+f+i)-(a+d+d+g),(g+h+h+i)-(a+b+b+c))/pow(2.0,exaggeration+(19.2562-u_zoom));gl_FragColor=clamp(vec4(deriv.x/2.0+0.5,deriv.y/2.0+0.5,1.0,1.0),0.0,1.0);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_dimension;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);highp vec2 epsilon=1.0/u_dimension;float scale=(u_dimension.x-2.0)/u_dimension.x;v_pos=(a_texture_pos/8192.0)*scale+epsilon;}"), Fa = vs(`uniform sampler2D u_image;varying vec2 v_pos;uniform vec2 u_latrange;uniform vec2 u_light;uniform vec4 u_shadow;uniform vec4 u_highlight;uniform vec4 u_accent;
#define PI 3.141592653589793
void main() {vec4 pixel=texture2D(u_image,v_pos);vec2 deriv=((pixel.rg*2.0)-1.0);float scaleFactor=cos(radians((u_latrange[0]-u_latrange[1])*(1.0-v_pos.y)+u_latrange[1]));float slope=atan(1.25*length(deriv)/scaleFactor);float aspect=deriv.x !=0.0 ? atan(deriv.y,-deriv.x) : PI/2.0*(deriv.y > 0.0 ? 1.0 :-1.0);float intensity=u_light.x;float azimuth=u_light.y+PI;float base=1.875-intensity*1.75;float maxValue=0.5*PI;float scaledSlope=intensity !=0.5 ? ((pow(base,slope)-1.0)/(pow(base,maxValue)-1.0))*maxValue : slope;float accent=cos(scaledSlope);vec4 accent_color=(1.0-accent)*u_accent*clamp(intensity*2.0,0.0,1.0);float shade=abs(mod((aspect+azimuth)/PI+0.5,2.0)-1.0);vec4 shade_color=mix(u_shadow,u_highlight,shade)*sin(scaledSlope)*clamp(intensity*2.0,0.0,1.0);gl_FragColor=accent_color*(1.0-shade_color.a)+shade_color;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos=a_texture_pos/8192.0;}"), $o = vs(`uniform lowp float u_device_pixel_ratio;varying vec2 v_width2;varying vec2 v_normal;varying float v_gamma_scale;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);gl_FragColor=color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
attribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform vec2 u_units_to_pixels;uniform lowp float u_device_pixel_ratio;varying vec2 v_normal;varying vec2 v_width2;varying float v_gamma_scale;varying highp float v_linesofar;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float width
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float width
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;v_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*2.0;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_width2=vec2(outset,inset);}`), Gs = vs(`uniform lowp float u_device_pixel_ratio;uniform sampler2D u_image;varying vec2 v_width2;varying vec2 v_normal;varying float v_gamma_scale;varying highp vec2 v_uv;
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);vec4 color=texture2D(u_image,v_uv);gl_FragColor=color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
attribute vec2 a_pos_normal;attribute vec4 a_data;attribute float a_uv_x;attribute float a_split_index;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;uniform vec2 u_units_to_pixels;uniform float u_image_height;varying vec2 v_normal;varying vec2 v_width2;varying float v_gamma_scale;varying highp vec2 v_uv;
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float width
void main() {
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float width
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;highp float texel_height=1.0/u_image_height;highp float half_texel_height=0.5*texel_height;v_uv=vec2(a_uv_x,a_split_index*texel_height-half_texel_height);vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_width2=vec2(outset,inset);}`), Al = vs(`uniform lowp float u_device_pixel_ratio;uniform vec2 u_texsize;uniform float u_fade;uniform mediump vec3 u_scale;uniform sampler2D u_image;varying vec2 v_normal;varying vec2 v_width2;varying float v_linesofar;varying float v_gamma_scale;varying float v_width;
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileZoomRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;vec2 pattern_size_a=vec2(display_size_a.x*fromScale/tileZoomRatio,display_size_a.y);vec2 pattern_size_b=vec2(display_size_b.x*toScale/tileZoomRatio,display_size_b.y);float aspect_a=display_size_a.y/v_width;float aspect_b=display_size_b.y/v_width;float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);float x_a=mod(v_linesofar/pattern_size_a.x*aspect_a,1.0);float x_b=mod(v_linesofar/pattern_size_b.x*aspect_b,1.0);float y=0.5*v_normal.y+0.5;vec2 texel_size=1.0/u_texsize;vec2 pos_a=mix(pattern_tl_a*texel_size-texel_size,pattern_br_a*texel_size+texel_size,vec2(x_a,y));vec2 pos_b=mix(pattern_tl_b*texel_size-texel_size,pattern_br_b*texel_size+texel_size,vec2(x_b,y));vec4 color=mix(texture2D(u_image,pos_a),texture2D(u_image,pos_b),u_fade);gl_FragColor=color*alpha*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
#define LINE_DISTANCE_SCALE 2.0
attribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform vec2 u_units_to_pixels;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;varying vec2 v_normal;varying vec2 v_width2;varying float v_linesofar;varying float v_gamma_scale;varying float v_width;
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define mediump float width
#pragma mapbox: define lowp float floorwidth
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize mediump float width
#pragma mapbox: initialize lowp float floorwidth
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;float a_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*LINE_DISTANCE_SCALE;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_linesofar=a_linesofar;v_width2=vec2(outset,inset);v_width=floorwidth;}`), ul = vs(`uniform lowp float u_device_pixel_ratio;uniform sampler2D u_image;uniform float u_sdfgamma;uniform float u_mix;varying vec2 v_normal;varying vec2 v_width2;varying vec2 v_tex_a;varying vec2 v_tex_b;varying float v_gamma_scale;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float width
#pragma mapbox: define lowp float floorwidth
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float width
#pragma mapbox: initialize lowp float floorwidth
float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);float sdfdist_a=texture2D(u_image,v_tex_a).a;float sdfdist_b=texture2D(u_image,v_tex_b).a;float sdfdist=mix(sdfdist_a,sdfdist_b,u_mix);alpha*=smoothstep(0.5-u_sdfgamma/floorwidth,0.5+u_sdfgamma/floorwidth,sdfdist);gl_FragColor=color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
#define LINE_DISTANCE_SCALE 2.0
attribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;uniform vec2 u_patternscale_a;uniform float u_tex_y_a;uniform vec2 u_patternscale_b;uniform float u_tex_y_b;uniform vec2 u_units_to_pixels;varying vec2 v_normal;varying vec2 v_width2;varying vec2 v_tex_a;varying vec2 v_tex_b;varying float v_gamma_scale;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float width
#pragma mapbox: define lowp float floorwidth
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float width
#pragma mapbox: initialize lowp float floorwidth
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;float a_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*LINE_DISTANCE_SCALE;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;v_tex_a=vec2(a_linesofar*u_patternscale_a.x/floorwidth,normal.y*u_patternscale_a.y+u_tex_y_a);v_tex_b=vec2(a_linesofar*u_patternscale_b.x/floorwidth,normal.y*u_patternscale_b.y+u_tex_y_b);v_width2=vec2(outset,inset);}`), gc = vs(`uniform float u_fade_t;uniform float u_opacity;uniform sampler2D u_image0;uniform sampler2D u_image1;varying vec2 v_pos0;varying vec2 v_pos1;uniform float u_brightness_low;uniform float u_brightness_high;uniform float u_saturation_factor;uniform float u_contrast_factor;uniform vec3 u_spin_weights;void main() {vec4 color0=texture2D(u_image0,v_pos0);vec4 color1=texture2D(u_image1,v_pos1);if (color0.a > 0.0) {color0.rgb=color0.rgb/color0.a;}if (color1.a > 0.0) {color1.rgb=color1.rgb/color1.a;}vec4 color=mix(color0,color1,u_fade_t);color.a*=u_opacity;vec3 rgb=color.rgb;rgb=vec3(dot(rgb,u_spin_weights.xyz),dot(rgb,u_spin_weights.zxy),dot(rgb,u_spin_weights.yzx));float average=(color.r+color.g+color.b)/3.0;rgb+=(average-rgb)*u_saturation_factor;rgb=(rgb-0.5)*u_contrast_factor+0.5;vec3 u_high_vec=vec3(u_brightness_low,u_brightness_low,u_brightness_low);vec3 u_low_vec=vec3(u_brightness_high,u_brightness_high,u_brightness_high);gl_FragColor=vec4(mix(u_high_vec,u_low_vec,rgb)*color.a,color.a);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_tl_parent;uniform float u_scale_parent;uniform float u_buffer_scale;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos0;varying vec2 v_pos1;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos0=(((a_texture_pos/8192.0)-0.5)/u_buffer_scale )+0.5;v_pos1=(v_pos0*u_scale_parent)+u_tl_parent;}"), Gl = vs(`uniform sampler2D u_texture;varying vec2 v_tex;varying float v_fade_opacity;
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize lowp float opacity
lowp float alpha=opacity*v_fade_opacity;gl_FragColor=texture2D(u_texture,v_tex)*alpha;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `const float PI=3.141592653589793;attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec4 a_pixeloffset;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform highp float u_camera_to_center_distance;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform float u_fade_change;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform vec2 u_texsize;varying vec2 v_tex;varying float v_fade_opacity;
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize lowp float opacity
vec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;float a_size_min=floor(a_size[0]*0.5);vec2 a_pxoffset=a_pixeloffset.xy;vec2 a_minFontScale=a_pixeloffset.zw/256.0;highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size_min,a_size[1],u_size_t)/128.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size_min/128.0;} else {size=u_size;}vec4 projectedPoint=u_matrix*vec4(a_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?
camera_to_anchor_distance/u_camera_to_center_distance :
u_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=u_is_text ? size/24.0 : size;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=u_matrix*vec4(a_pos+vec2(1,0),0,1);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy,0.0,1.0);gl_Position=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*max(a_minFontScale,fontScale)+a_pxoffset/16.0),0.0,1.0);v_tex=a_tex/u_texsize;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;v_fade_opacity=max(0.0,min(1.0,fade_opacity[0]+fade_change));}`), Sl = vs(`#define SDF_PX 8.0
uniform bool u_is_halo;uniform sampler2D u_texture;uniform highp float u_gamma_scale;uniform lowp float u_device_pixel_ratio;uniform bool u_is_text;varying vec2 v_data0;varying vec3 v_data1;
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
float EDGE_GAMMA=0.105/u_device_pixel_ratio;vec2 tex=v_data0.xy;float gamma_scale=v_data1.x;float size=v_data1.y;float fade_opacity=v_data1[2];float fontScale=u_is_text ? size/24.0 : size;lowp vec4 color=fill_color;highp float gamma=EDGE_GAMMA/(fontScale*u_gamma_scale);lowp float buff=(256.0-64.0)/256.0;if (u_is_halo) {color=halo_color;gamma=(halo_blur*1.19/SDF_PX+EDGE_GAMMA)/(fontScale*u_gamma_scale);buff=(6.0-halo_width/fontScale)/SDF_PX;}lowp float dist=texture2D(u_texture,tex).a;highp float gamma_scaled=gamma*gamma_scale;highp float alpha=smoothstep(buff-gamma_scaled,buff+gamma_scaled,dist);gl_FragColor=color*(alpha*opacity*fade_opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `const float PI=3.141592653589793;attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec4 a_pixeloffset;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform highp float u_camera_to_center_distance;uniform float u_fade_change;uniform vec2 u_texsize;varying vec2 v_data0;varying vec3 v_data1;
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
vec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;float a_size_min=floor(a_size[0]*0.5);vec2 a_pxoffset=a_pixeloffset.xy;highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size_min,a_size[1],u_size_t)/128.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size_min/128.0;} else {size=u_size;}vec4 projectedPoint=u_matrix*vec4(a_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?
camera_to_anchor_distance/u_camera_to_center_distance :
u_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=u_is_text ? size/24.0 : size;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=u_matrix*vec4(a_pos+vec2(1,0),0,1);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy,0.0,1.0);gl_Position=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*fontScale+a_pxoffset),0.0,1.0);float gamma_scale=gl_Position.w;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;float interpolated_fade_opacity=max(0.0,min(1.0,fade_opacity[0]+fade_change));v_data0=a_tex/u_texsize;v_data1=vec3(gamma_scale,size,interpolated_fade_opacity);}`), ls = vs(`#define SDF_PX 8.0
#define SDF 1.0
#define ICON 0.0
uniform bool u_is_halo;uniform sampler2D u_texture;uniform sampler2D u_texture_icon;uniform highp float u_gamma_scale;uniform lowp float u_device_pixel_ratio;varying vec4 v_data0;varying vec4 v_data1;
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
float fade_opacity=v_data1[2];if (v_data1.w==ICON) {vec2 tex_icon=v_data0.zw;lowp float alpha=opacity*fade_opacity;gl_FragColor=texture2D(u_texture_icon,tex_icon)*alpha;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
return;}vec2 tex=v_data0.xy;float EDGE_GAMMA=0.105/u_device_pixel_ratio;float gamma_scale=v_data1.x;float size=v_data1.y;float fontScale=size/24.0;lowp vec4 color=fill_color;highp float gamma=EDGE_GAMMA/(fontScale*u_gamma_scale);lowp float buff=(256.0-64.0)/256.0;if (u_is_halo) {color=halo_color;gamma=(halo_blur*1.19/SDF_PX+EDGE_GAMMA)/(fontScale*u_gamma_scale);buff=(6.0-halo_width/fontScale)/SDF_PX;}lowp float dist=texture2D(u_texture,tex).a;highp float gamma_scaled=gamma*gamma_scale;highp float alpha=smoothstep(buff-gamma_scaled,buff+gamma_scaled,dist);gl_FragColor=color*(alpha*opacity*fade_opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `const float PI=3.141592653589793;attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform highp float u_camera_to_center_distance;uniform float u_fade_change;uniform vec2 u_texsize;uniform vec2 u_texsize_icon;varying vec4 v_data0;varying vec4 v_data1;
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
vec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;float a_size_min=floor(a_size[0]*0.5);float is_sdf=a_size[0]-2.0*a_size_min;highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size_min,a_size[1],u_size_t)/128.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size_min/128.0;} else {size=u_size;}vec4 projectedPoint=u_matrix*vec4(a_pos,0,1);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?
camera_to_anchor_distance/u_camera_to_center_distance :
u_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=size/24.0;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=u_matrix*vec4(a_pos+vec2(1,0),0,1);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy,0.0,1.0);gl_Position=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*fontScale),0.0,1.0);float gamma_scale=gl_Position.w;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;float interpolated_fade_opacity=max(0.0,min(1.0,fade_opacity[0]+fade_change));v_data0.xy=a_tex/u_texsize;v_data0.zw=a_tex/u_texsize_icon;v_data1=vec4(gamma_scale,size,interpolated_fade_opacity,is_sdf);}`);
                function vs(Ke, Fe) {
                  var Te = /#pragma mapbox: ([\w]+) ([\w]+) ([\w]+) ([\w]+)/g, Ae = Fe.match(/attribute ([\w]+) ([\w]+)/g), Oe = Ke.match(/uniform ([\w]+) ([\w]+)([\s]*)([\w]*)/g), Be = Fe.match(/uniform ([\w]+) ([\w]+)([\s]*)([\w]*)/g), Ue = Be ? Be.concat(Oe) : Oe, nt = {};
                  return { fragmentSource: Ke = Ke.replace(Te, function(ht, Et, Mt, qt, tn) {
                    return nt[tn] = !0, Et === "define" ? `
#ifndef HAS_UNIFORM_u_` + tn + `
varying ` + Mt + " " + qt + " " + tn + `;
#else
uniform ` + Mt + " " + qt + " u_" + tn + `;
#endif
` : `
#ifdef HAS_UNIFORM_u_` + tn + `
    ` + Mt + " " + qt + " " + tn + " = u_" + tn + `;
#endif
`;
                  }), vertexSource: Fe = Fe.replace(Te, function(ht, Et, Mt, qt, tn) {
                    var xn = qt === "float" ? "vec2" : "vec4", An = tn.match(/color/) ? "color" : xn;
                    return nt[tn] ? Et === "define" ? `
#ifndef HAS_UNIFORM_u_` + tn + `
uniform lowp float u_` + tn + `_t;
attribute ` + Mt + " " + xn + " a_" + tn + `;
varying ` + Mt + " " + qt + " " + tn + `;
#else
uniform ` + Mt + " " + qt + " u_" + tn + `;
#endif
` : An === "vec4" ? `
#ifndef HAS_UNIFORM_u_` + tn + `
    ` + tn + " = a_" + tn + `;
#else
    ` + Mt + " " + qt + " " + tn + " = u_" + tn + `;
#endif
` : `
#ifndef HAS_UNIFORM_u_` + tn + `
    ` + tn + " = unpack_mix_" + An + "(a_" + tn + ", u_" + tn + `_t);
#else
    ` + Mt + " " + qt + " " + tn + " = u_" + tn + `;
#endif
` : Et === "define" ? `
#ifndef HAS_UNIFORM_u_` + tn + `
uniform lowp float u_` + tn + `_t;
attribute ` + Mt + " " + xn + " a_" + tn + `;
#else
uniform ` + Mt + " " + qt + " u_" + tn + `;
#endif
` : An === "vec4" ? `
#ifndef HAS_UNIFORM_u_` + tn + `
    ` + Mt + " " + qt + " " + tn + " = a_" + tn + `;
#else
    ` + Mt + " " + qt + " " + tn + " = u_" + tn + `;
#endif
` : `
#ifndef HAS_UNIFORM_u_` + tn + `
    ` + Mt + " " + qt + " " + tn + " = unpack_mix_" + An + "(a_" + tn + ", u_" + tn + `_t);
#else
    ` + Mt + " " + qt + " " + tn + " = u_" + tn + `;
#endif
`;
                  }), staticAttributes: Ae, staticUniforms: Ue };
                }
                var Vl = Object.freeze({ __proto__: null, prelude: Vi, background: ua, backgroundPattern: Ki, circle: Fi, clippingMask: na, heatmap: nn, heatmapTexture: Tn, collisionBox: Dn, collisionCircle: Fn, debug: fr, fill: dr, fillOutline: yr, fillOutlinePattern: ai, fillPattern: Fr, fillExtrusion: Ti, fillExtrusionPattern: Si, hillshadePrepare: Ei, hillshade: Fa, line: $o, lineGradient: Gs, linePattern: Al, lineSDF: ul, raster: gc, symbolIcon: Gl, symbolSDF: Sl, symbolTextAndIcon: ls }), Ll = function() {
                  this.boundProgram = null, this.boundLayoutVertexBuffer = null, this.boundPaintVertexBuffers = [], this.boundIndexBuffer = null, this.boundVertexOffset = null, this.boundDynamicVertexBuffer = null, this.vao = null;
                };
                function Xl(Ke) {
                  for (var Fe = [], Te = 0; Te < Ke.length; Te++) if (Ke[Te] !== null) {
                    var Ae = Ke[Te].split(" ");
                    Fe.push(Ae.pop());
                  }
                  return Fe;
                }
                Ll.prototype.bind = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt) {
                  this.context = Ke;
                  for (var ht = this.boundPaintVertexBuffers.length !== Ae.length, Et = 0; !ht && Et < Ae.length; Et++) this.boundPaintVertexBuffers[Et] !== Ae[Et] && (ht = !0);
                  var Mt = !this.vao || this.boundProgram !== Fe || this.boundLayoutVertexBuffer !== Te || ht || this.boundIndexBuffer !== Oe || this.boundVertexOffset !== Be || this.boundDynamicVertexBuffer !== Ue || this.boundDynamicVertexBuffer2 !== nt;
                  !Ke.extVertexArrayObject || Mt ? this.freshBind(Fe, Te, Ae, Oe, Be, Ue, nt) : (Ke.bindVertexArrayOES.set(this.vao), Ue && Ue.bind(), Oe && Oe.dynamicDraw && Oe.bind(), nt && nt.bind());
                }, Ll.prototype.freshBind = function(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  var nt, ht = Ke.numAttributes, Et = this.context, Mt = Et.gl;
                  if (Et.extVertexArrayObject) this.vao && this.destroy(), this.vao = Et.extVertexArrayObject.createVertexArrayOES(), Et.bindVertexArrayOES.set(this.vao), nt = 0, this.boundProgram = Ke, this.boundLayoutVertexBuffer = Fe, this.boundPaintVertexBuffers = Te, this.boundIndexBuffer = Ae, this.boundVertexOffset = Oe, this.boundDynamicVertexBuffer = Be, this.boundDynamicVertexBuffer2 = Ue;
                  else {
                    nt = Et.currentNumAttributes || 0;
                    for (var qt = ht; qt < nt; qt++) Mt.disableVertexAttribArray(qt);
                  }
                  Fe.enableAttributes(Mt, Ke);
                  for (var tn = 0, xn = Te; tn < xn.length; tn += 1) xn[tn].enableAttributes(Mt, Ke);
                  Be && Be.enableAttributes(Mt, Ke), Ue && Ue.enableAttributes(Mt, Ke), Fe.bind(), Fe.setVertexAttribPointers(Mt, Ke, Oe);
                  for (var An = 0, On = Te; An < On.length; An += 1) {
                    var Bn = On[An];
                    Bn.bind(), Bn.setVertexAttribPointers(Mt, Ke, Oe);
                  }
                  Be && (Be.bind(), Be.setVertexAttribPointers(Mt, Ke, Oe)), Ae && Ae.bind(), Ue && (Ue.bind(), Ue.setVertexAttribPointers(Mt, Ke, Oe)), Et.currentNumAttributes = ht;
                }, Ll.prototype.destroy = function() {
                  this.vao && (this.context.extVertexArrayObject.deleteVertexArrayOES(this.vao), this.vao = null);
                };
                var yc = function(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = Ke.gl;
                  this.program = Ue.createProgram();
                  for (var nt = Xl(Te.staticAttributes), ht = Ae ? Ae.getBinderAttributes() : [], Et = nt.concat(ht), Mt = Te.staticUniforms ? Xl(Te.staticUniforms) : [], qt = Ae ? Ae.getBinderUniforms() : [], tn = [], xn = 0, An = Mt.concat(qt); xn < An.length; xn += 1) {
                    var On = An[xn];
                    tn.indexOf(On) < 0 && tn.push(On);
                  }
                  var Bn = Ae ? Ae.defines() : [];
                  Be && Bn.push("#define OVERDRAW_INSPECTOR;");
                  var qn = Bn.concat(Vi.fragmentSource, Te.fragmentSource).join(`
`), Vn = Bn.concat(Vi.vertexSource, Te.vertexSource).join(`
`), sr = Ue.createShader(Ue.FRAGMENT_SHADER);
                  if (Ue.isContextLost()) this.failedToCreate = !0;
                  else {
                    Ue.shaderSource(sr, qn), Ue.compileShader(sr), Ue.attachShader(this.program, sr);
                    var Un = Ue.createShader(Ue.VERTEX_SHADER);
                    if (Ue.isContextLost()) this.failedToCreate = !0;
                    else {
                      Ue.shaderSource(Un, Vn), Ue.compileShader(Un), Ue.attachShader(this.program, Un), this.attributes = {};
                      var wr = {};
                      this.numAttributes = Et.length;
                      for (var Cr = 0; Cr < this.numAttributes; Cr++) Et[Cr] && (Ue.bindAttribLocation(this.program, Cr, Et[Cr]), this.attributes[Et[Cr]] = Cr);
                      Ue.linkProgram(this.program), Ue.deleteShader(Un), Ue.deleteShader(sr);
                      for (var Tr = 0; Tr < tn.length; Tr++) {
                        var ii = tn[Tr];
                        if (ii && !wr[ii]) {
                          var wi = Ue.getUniformLocation(this.program, ii);
                          wi && (wr[ii] = wi);
                        }
                      }
                      this.fixedUniforms = Oe(Ke, wr), this.binderUniforms = Ae ? Ae.getUniforms(Ke, wr) : [];
                    }
                  }
                };
                function gu(Ke, Fe, Te) {
                  var Ae = 1 / Gr(Te, 1, Fe.transform.tileZoom), Oe = Math.pow(2, Te.tileID.overscaledZ), Be = Te.tileSize * Math.pow(2, Fe.transform.tileZoom) / Oe, Ue = Be * (Te.tileID.canonical.x + Te.tileID.wrap * Oe), nt = Be * Te.tileID.canonical.y;
                  return { u_image: 0, u_texsize: Te.imageAtlasTexture.size, u_scale: [Ae, Ke.fromScale, Ke.toScale], u_fade: Ke.t, u_pixel_coord_upper: [Ue >> 16, nt >> 16], u_pixel_coord_lower: [65535 & Ue, 65535 & nt] };
                }
                yc.prototype.draw = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An, On) {
                  var Bn, qn = Ke.gl;
                  if (!this.failedToCreate) {
                    for (var Vn in Ke.program.set(this.program), Ke.setDepthMode(Te), Ke.setStencilMode(Ae), Ke.setColorMode(Oe), Ke.setCullFace(Be), this.fixedUniforms) this.fixedUniforms[Vn].set(Ue[Vn]);
                    xn && xn.setUniforms(Ke, this.binderUniforms, qt, { zoom: tn });
                    for (var sr = (Bn = {}, Bn[qn.LINES] = 2, Bn[qn.TRIANGLES] = 3, Bn[qn.LINE_STRIP] = 1, Bn)[Fe], Un = 0, wr = Mt.get(); Un < wr.length; Un += 1) {
                      var Cr = wr[Un], Tr = Cr.vaos || (Cr.vaos = {});
                      (Tr[nt] || (Tr[nt] = new Ll())).bind(Ke, this, ht, xn ? xn.getPaintVertexBuffers() : [], Et, Cr.vertexOffset, An, On), qn.drawElements(Fe, Cr.primitiveLength * sr, qn.UNSIGNED_SHORT, Cr.primitiveOffset * sr * 2);
                    }
                  }
                };
                var Nc = function(Ke, Fe, Te, Ae) {
                  var Oe = Fe.style.light, Be = Oe.properties.get("position"), Ue = [Be.x, Be.y, Be.z], nt = ee.create$1();
                  Oe.properties.get("anchor") === "viewport" && ee.fromRotation(nt, -Fe.transform.angle), ee.transformMat3(Ue, Ue, nt);
                  var ht = Oe.properties.get("color");
                  return { u_matrix: Ke, u_lightpos: Ue, u_lightintensity: Oe.properties.get("intensity"), u_lightcolor: [ht.r, ht.g, ht.b], u_vertical_gradient: +Te, u_opacity: Ae };
                }, Cu = function(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  return ee.extend(Nc(Ke, Fe, Te, Ae), gu(Be, Fe, Ue), { u_height_factor: -Math.pow(2, Oe.overscaledZ) / Ue.tileSize / 8 });
                }, zc = function(Ke) {
                  return { u_matrix: Ke };
                }, Kc = function(Ke, Fe, Te, Ae) {
                  return ee.extend(zc(Ke), gu(Te, Fe, Ae));
                }, zi = function(Ke, Fe) {
                  return { u_matrix: Ke, u_world: Fe };
                }, Fu = function(Ke, Fe, Te, Ae, Oe) {
                  return ee.extend(Kc(Ke, Fe, Te, Ae), { u_world: Oe });
                }, d1 = function(Ke, Fe, Te, Ae) {
                  var Oe, Be, Ue = Ke.transform;
                  if (Ae.paint.get("circle-pitch-alignment") === "map") {
                    var nt = Gr(Te, 1, Ue.zoom);
                    Oe = !0, Be = [nt, nt];
                  } else Oe = !1, Be = Ue.pixelsToGLUnits;
                  return { u_camera_to_center_distance: Ue.cameraToCenterDistance, u_scale_with_map: +(Ae.paint.get("circle-pitch-scale") === "map"), u_matrix: Ke.translatePosMatrix(Fe.posMatrix, Te, Ae.paint.get("circle-translate"), Ae.paint.get("circle-translate-anchor")), u_pitch_with_map: +Oe, u_device_pixel_ratio: ee.browser.devicePixelRatio, u_extrude_scale: Be };
                }, Gu = function(Ke, Fe, Te) {
                  var Ae = Gr(Te, 1, Fe.zoom), Oe = Math.pow(2, Fe.zoom - Te.tileID.overscaledZ), Be = Te.tileID.overscaleFactor();
                  return { u_matrix: Ke, u_camera_to_center_distance: Fe.cameraToCenterDistance, u_pixels_to_tile_units: Ae, u_extrude_scale: [Fe.pixelsToGLUnits[0] / (Ae * Oe), Fe.pixelsToGLUnits[1] / (Ae * Oe)], u_overscale_factor: Be };
                }, Ku = function(Ke, Fe, Te) {
                  return { u_matrix: Ke, u_inv_matrix: Fe, u_camera_to_center_distance: Te.cameraToCenterDistance, u_viewport_size: [Te.width, Te.height] };
                }, I1 = function(Ke, Fe, Te) {
                  return Te === void 0 && (Te = 1), { u_matrix: Ke, u_color: Fe, u_overlay: 0, u_overlay_scale: Te };
                }, f1 = function(Ke) {
                  return { u_matrix: Ke };
                }, D1 = function(Ke, Fe, Te, Ae) {
                  return { u_matrix: Ke, u_extrude_scale: Gr(Fe, 1, Te), u_intensity: Ae };
                }, Hl = function(Ke, Fe, Te, Ae) {
                  var Oe = ee.create();
                  ee.ortho(Oe, 0, Ke.width, Ke.height, 0, 0, 1);
                  var Be = Ke.context.gl;
                  return { u_matrix: Oe, u_world: [Be.drawingBufferWidth, Be.drawingBufferHeight], u_image: Te, u_color_ramp: Ae, u_opacity: Fe.paint.get("heatmap-opacity") };
                }, Kl = function(Ke, Fe, Te) {
                  var Ae = Te.paint.get("hillshade-shadow-color"), Oe = Te.paint.get("hillshade-highlight-color"), Be = Te.paint.get("hillshade-accent-color"), Ue = Te.paint.get("hillshade-illumination-direction") * (Math.PI / 180);
                  Te.paint.get("hillshade-illumination-anchor") === "viewport" && (Ue -= Ke.transform.angle);
                  var nt, ht, Et, Mt = !Ke.options.moving;
                  return { u_matrix: Ke.transform.calculatePosMatrix(Fe.tileID.toUnwrapped(), Mt), u_image: 0, u_latrange: (nt = Fe.tileID, ht = Math.pow(2, nt.canonical.z), Et = nt.canonical.y, [new ee.MercatorCoordinate(0, Et / ht).toLngLat().lat, new ee.MercatorCoordinate(0, (Et + 1) / ht).toLngLat().lat]), u_light: [Te.paint.get("hillshade-exaggeration"), Ue], u_shadow: Ae, u_highlight: Oe, u_accent: Be };
                }, uu = function(Ke, Fe) {
                  var Te = Fe.stride, Ae = ee.create();
                  return ee.ortho(Ae, 0, ee.EXTENT, -ee.EXTENT, 0, 0, 1), ee.translate(Ae, Ae, [0, -ee.EXTENT, 0]), { u_matrix: Ae, u_image: 1, u_dimension: [Te, Te], u_zoom: Ke.overscaledZ, u_unpack: Fe.getUnpackVector() };
                }, Hs = function(Ke, Fe, Te) {
                  var Ae = Ke.transform;
                  return { u_matrix: jo(Ke, Fe, Te), u_ratio: 1 / Gr(Fe, 1, Ae.zoom), u_device_pixel_ratio: ee.browser.devicePixelRatio, u_units_to_pixels: [1 / Ae.pixelsToGLUnits[0], 1 / Ae.pixelsToGLUnits[1]] };
                }, b1 = function(Ke, Fe, Te, Ae) {
                  return ee.extend(Hs(Ke, Fe, Te), { u_image: 0, u_image_height: Ae });
                }, Wl = function(Ke, Fe, Te, Ae) {
                  var Oe = Ke.transform, Be = os(Fe, Oe);
                  return { u_matrix: jo(Ke, Fe, Te), u_texsize: Fe.imageAtlasTexture.size, u_ratio: 1 / Gr(Fe, 1, Oe.zoom), u_device_pixel_ratio: ee.browser.devicePixelRatio, u_image: 0, u_scale: [Be, Ae.fromScale, Ae.toScale], u_fade: Ae.t, u_units_to_pixels: [1 / Oe.pixelsToGLUnits[0], 1 / Oe.pixelsToGLUnits[1]] };
                }, zl = function(Ke, Fe, Te, Ae, Oe) {
                  var Be = Ke.transform, Ue = Ke.lineAtlas, nt = os(Fe, Be), ht = Te.layout.get("line-cap") === "round", Et = Ue.getDash(Ae.from, ht), Mt = Ue.getDash(Ae.to, ht), qt = Et.width * Oe.fromScale, tn = Mt.width * Oe.toScale;
                  return ee.extend(Hs(Ke, Fe, Te), { u_patternscale_a: [nt / qt, -Et.height / 2], u_patternscale_b: [nt / tn, -Mt.height / 2], u_sdfgamma: Ue.width / (256 * Math.min(qt, tn) * ee.browser.devicePixelRatio) / 2, u_image: 0, u_tex_y_a: Et.y, u_tex_y_b: Mt.y, u_mix: Oe.t });
                };
                function os(Ke, Fe) {
                  return 1 / Gr(Ke, 1, Fe.tileZoom);
                }
                function jo(Ke, Fe, Te) {
                  return Ke.translatePosMatrix(Fe.tileID.posMatrix, Fe, Te.paint.get("line-translate"), Te.paint.get("line-translate-anchor"));
                }
                var uo = function(Ke, Fe, Te, Ae, Oe) {
                  return { u_matrix: Ke, u_tl_parent: Fe, u_scale_parent: Te, u_buffer_scale: 1, u_fade_t: Ae.mix, u_opacity: Ae.opacity * Oe.paint.get("raster-opacity"), u_image0: 0, u_image1: 1, u_brightness_low: Oe.paint.get("raster-brightness-min"), u_brightness_high: Oe.paint.get("raster-brightness-max"), u_saturation_factor: (Ue = Oe.paint.get("raster-saturation"), Ue > 0 ? 1 - 1 / (1.001 - Ue) : -Ue), u_contrast_factor: (Be = Oe.paint.get("raster-contrast"), Be > 0 ? 1 / (1 - Be) : 1 + Be), u_spin_weights: so(Oe.paint.get("raster-hue-rotate")) };
                  var Be, Ue;
                };
                function so(Ke) {
                  Ke *= Math.PI / 180;
                  var Fe = Math.sin(Ke), Te = Math.cos(Ke);
                  return [(2 * Te + 1) / 3, (-Math.sqrt(3) * Fe - Te + 1) / 3, (Math.sqrt(3) * Fe - Te + 1) / 3];
                }
                var So, Jc = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                  var Mt = Oe.transform;
                  return { u_is_size_zoom_constant: +(Ke === "constant" || Ke === "source"), u_is_size_feature_constant: +(Ke === "constant" || Ke === "camera"), u_size_t: Fe ? Fe.uSizeT : 0, u_size: Fe ? Fe.uSize : 0, u_camera_to_center_distance: Mt.cameraToCenterDistance, u_pitch: Mt.pitch / 360 * 2 * Math.PI, u_rotate_symbol: +Te, u_aspect_ratio: Mt.width / Mt.height, u_fade_change: Oe.options.fadeDuration ? Oe.symbolFadeChange : 1, u_matrix: Be, u_label_plane_matrix: Ue, u_coord_matrix: nt, u_is_text: +ht, u_pitch_with_map: +Ae, u_texsize: Et, u_texture: 0 };
                }, No = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt) {
                  var qt = Oe.transform;
                  return ee.extend(Jc(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et), { u_gamma_scale: Ae ? Math.cos(qt._pitch) * qt.cameraToCenterDistance : 1, u_device_pixel_ratio: ee.browser.devicePixelRatio, u_is_halo: 1 });
                }, ro = function(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                  return ee.extend(No(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, !0, ht), { u_texsize_icon: Et, u_texture_icon: 1 });
                }, Io = function(Ke, Fe, Te) {
                  return { u_matrix: Ke, u_opacity: Fe, u_color: Te };
                }, bs = function(Ke, Fe, Te, Ae, Oe, Be) {
                  return ee.extend(function(Ue, nt, ht, Et) {
                    var Mt = ht.imageManager.getPattern(Ue.from.toString()), qt = ht.imageManager.getPattern(Ue.to.toString()), tn = ht.imageManager.getPixelSize(), xn = tn.width, An = tn.height, On = Math.pow(2, Et.tileID.overscaledZ), Bn = Et.tileSize * Math.pow(2, ht.transform.tileZoom) / On, qn = Bn * (Et.tileID.canonical.x + Et.tileID.wrap * On), Vn = Bn * Et.tileID.canonical.y;
                    return { u_image: 0, u_pattern_tl_a: Mt.tl, u_pattern_br_a: Mt.br, u_pattern_tl_b: qt.tl, u_pattern_br_b: qt.br, u_texsize: [xn, An], u_mix: nt.t, u_pattern_size_a: Mt.displaySize, u_pattern_size_b: qt.displaySize, u_scale_a: nt.fromScale, u_scale_b: nt.toScale, u_tile_units_to_pixels: 1 / Gr(Et, 1, ht.transform.tileZoom), u_pixel_coord_upper: [qn >> 16, Vn >> 16], u_pixel_coord_lower: [65535 & qn, 65535 & Vn] };
                  }(Ae, Be, Te, Oe), { u_matrix: Ke, u_opacity: Fe });
                }, ws = { fillExtrusion: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_lightpos: new ee.Uniform3f(Ke, Fe.u_lightpos), u_lightintensity: new ee.Uniform1f(Ke, Fe.u_lightintensity), u_lightcolor: new ee.Uniform3f(Ke, Fe.u_lightcolor), u_vertical_gradient: new ee.Uniform1f(Ke, Fe.u_vertical_gradient), u_opacity: new ee.Uniform1f(Ke, Fe.u_opacity) };
                }, fillExtrusionPattern: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_lightpos: new ee.Uniform3f(Ke, Fe.u_lightpos), u_lightintensity: new ee.Uniform1f(Ke, Fe.u_lightintensity), u_lightcolor: new ee.Uniform3f(Ke, Fe.u_lightcolor), u_vertical_gradient: new ee.Uniform1f(Ke, Fe.u_vertical_gradient), u_height_factor: new ee.Uniform1f(Ke, Fe.u_height_factor), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_pixel_coord_upper: new ee.Uniform2f(Ke, Fe.u_pixel_coord_upper), u_pixel_coord_lower: new ee.Uniform2f(Ke, Fe.u_pixel_coord_lower), u_scale: new ee.Uniform3f(Ke, Fe.u_scale), u_fade: new ee.Uniform1f(Ke, Fe.u_fade), u_opacity: new ee.Uniform1f(Ke, Fe.u_opacity) };
                }, fill: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix) };
                }, fillPattern: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_pixel_coord_upper: new ee.Uniform2f(Ke, Fe.u_pixel_coord_upper), u_pixel_coord_lower: new ee.Uniform2f(Ke, Fe.u_pixel_coord_lower), u_scale: new ee.Uniform3f(Ke, Fe.u_scale), u_fade: new ee.Uniform1f(Ke, Fe.u_fade) };
                }, fillOutline: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_world: new ee.Uniform2f(Ke, Fe.u_world) };
                }, fillOutlinePattern: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_world: new ee.Uniform2f(Ke, Fe.u_world), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_pixel_coord_upper: new ee.Uniform2f(Ke, Fe.u_pixel_coord_upper), u_pixel_coord_lower: new ee.Uniform2f(Ke, Fe.u_pixel_coord_lower), u_scale: new ee.Uniform3f(Ke, Fe.u_scale), u_fade: new ee.Uniform1f(Ke, Fe.u_fade) };
                }, circle: function(Ke, Fe) {
                  return { u_camera_to_center_distance: new ee.Uniform1f(Ke, Fe.u_camera_to_center_distance), u_scale_with_map: new ee.Uniform1i(Ke, Fe.u_scale_with_map), u_pitch_with_map: new ee.Uniform1i(Ke, Fe.u_pitch_with_map), u_extrude_scale: new ee.Uniform2f(Ke, Fe.u_extrude_scale), u_device_pixel_ratio: new ee.Uniform1f(Ke, Fe.u_device_pixel_ratio), u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix) };
                }, collisionBox: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_camera_to_center_distance: new ee.Uniform1f(Ke, Fe.u_camera_to_center_distance), u_pixels_to_tile_units: new ee.Uniform1f(Ke, Fe.u_pixels_to_tile_units), u_extrude_scale: new ee.Uniform2f(Ke, Fe.u_extrude_scale), u_overscale_factor: new ee.Uniform1f(Ke, Fe.u_overscale_factor) };
                }, collisionCircle: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_inv_matrix: new ee.UniformMatrix4f(Ke, Fe.u_inv_matrix), u_camera_to_center_distance: new ee.Uniform1f(Ke, Fe.u_camera_to_center_distance), u_viewport_size: new ee.Uniform2f(Ke, Fe.u_viewport_size) };
                }, debug: function(Ke, Fe) {
                  return { u_color: new ee.UniformColor(Ke, Fe.u_color), u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_overlay: new ee.Uniform1i(Ke, Fe.u_overlay), u_overlay_scale: new ee.Uniform1f(Ke, Fe.u_overlay_scale) };
                }, clippingMask: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix) };
                }, heatmap: function(Ke, Fe) {
                  return { u_extrude_scale: new ee.Uniform1f(Ke, Fe.u_extrude_scale), u_intensity: new ee.Uniform1f(Ke, Fe.u_intensity), u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix) };
                }, heatmapTexture: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_world: new ee.Uniform2f(Ke, Fe.u_world), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_color_ramp: new ee.Uniform1i(Ke, Fe.u_color_ramp), u_opacity: new ee.Uniform1f(Ke, Fe.u_opacity) };
                }, hillshade: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_latrange: new ee.Uniform2f(Ke, Fe.u_latrange), u_light: new ee.Uniform2f(Ke, Fe.u_light), u_shadow: new ee.UniformColor(Ke, Fe.u_shadow), u_highlight: new ee.UniformColor(Ke, Fe.u_highlight), u_accent: new ee.UniformColor(Ke, Fe.u_accent) };
                }, hillshadePrepare: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_dimension: new ee.Uniform2f(Ke, Fe.u_dimension), u_zoom: new ee.Uniform1f(Ke, Fe.u_zoom), u_unpack: new ee.Uniform4f(Ke, Fe.u_unpack) };
                }, line: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_ratio: new ee.Uniform1f(Ke, Fe.u_ratio), u_device_pixel_ratio: new ee.Uniform1f(Ke, Fe.u_device_pixel_ratio), u_units_to_pixels: new ee.Uniform2f(Ke, Fe.u_units_to_pixels) };
                }, lineGradient: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_ratio: new ee.Uniform1f(Ke, Fe.u_ratio), u_device_pixel_ratio: new ee.Uniform1f(Ke, Fe.u_device_pixel_ratio), u_units_to_pixels: new ee.Uniform2f(Ke, Fe.u_units_to_pixels), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_image_height: new ee.Uniform1f(Ke, Fe.u_image_height) };
                }, linePattern: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_ratio: new ee.Uniform1f(Ke, Fe.u_ratio), u_device_pixel_ratio: new ee.Uniform1f(Ke, Fe.u_device_pixel_ratio), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_units_to_pixels: new ee.Uniform2f(Ke, Fe.u_units_to_pixels), u_scale: new ee.Uniform3f(Ke, Fe.u_scale), u_fade: new ee.Uniform1f(Ke, Fe.u_fade) };
                }, lineSDF: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_ratio: new ee.Uniform1f(Ke, Fe.u_ratio), u_device_pixel_ratio: new ee.Uniform1f(Ke, Fe.u_device_pixel_ratio), u_units_to_pixels: new ee.Uniform2f(Ke, Fe.u_units_to_pixels), u_patternscale_a: new ee.Uniform2f(Ke, Fe.u_patternscale_a), u_patternscale_b: new ee.Uniform2f(Ke, Fe.u_patternscale_b), u_sdfgamma: new ee.Uniform1f(Ke, Fe.u_sdfgamma), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_tex_y_a: new ee.Uniform1f(Ke, Fe.u_tex_y_a), u_tex_y_b: new ee.Uniform1f(Ke, Fe.u_tex_y_b), u_mix: new ee.Uniform1f(Ke, Fe.u_mix) };
                }, raster: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_tl_parent: new ee.Uniform2f(Ke, Fe.u_tl_parent), u_scale_parent: new ee.Uniform1f(Ke, Fe.u_scale_parent), u_buffer_scale: new ee.Uniform1f(Ke, Fe.u_buffer_scale), u_fade_t: new ee.Uniform1f(Ke, Fe.u_fade_t), u_opacity: new ee.Uniform1f(Ke, Fe.u_opacity), u_image0: new ee.Uniform1i(Ke, Fe.u_image0), u_image1: new ee.Uniform1i(Ke, Fe.u_image1), u_brightness_low: new ee.Uniform1f(Ke, Fe.u_brightness_low), u_brightness_high: new ee.Uniform1f(Ke, Fe.u_brightness_high), u_saturation_factor: new ee.Uniform1f(Ke, Fe.u_saturation_factor), u_contrast_factor: new ee.Uniform1f(Ke, Fe.u_contrast_factor), u_spin_weights: new ee.Uniform3f(Ke, Fe.u_spin_weights) };
                }, symbolIcon: function(Ke, Fe) {
                  return { u_is_size_zoom_constant: new ee.Uniform1i(Ke, Fe.u_is_size_zoom_constant), u_is_size_feature_constant: new ee.Uniform1i(Ke, Fe.u_is_size_feature_constant), u_size_t: new ee.Uniform1f(Ke, Fe.u_size_t), u_size: new ee.Uniform1f(Ke, Fe.u_size), u_camera_to_center_distance: new ee.Uniform1f(Ke, Fe.u_camera_to_center_distance), u_pitch: new ee.Uniform1f(Ke, Fe.u_pitch), u_rotate_symbol: new ee.Uniform1i(Ke, Fe.u_rotate_symbol), u_aspect_ratio: new ee.Uniform1f(Ke, Fe.u_aspect_ratio), u_fade_change: new ee.Uniform1f(Ke, Fe.u_fade_change), u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_label_plane_matrix: new ee.UniformMatrix4f(Ke, Fe.u_label_plane_matrix), u_coord_matrix: new ee.UniformMatrix4f(Ke, Fe.u_coord_matrix), u_is_text: new ee.Uniform1i(Ke, Fe.u_is_text), u_pitch_with_map: new ee.Uniform1i(Ke, Fe.u_pitch_with_map), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_texture: new ee.Uniform1i(Ke, Fe.u_texture) };
                }, symbolSDF: function(Ke, Fe) {
                  return { u_is_size_zoom_constant: new ee.Uniform1i(Ke, Fe.u_is_size_zoom_constant), u_is_size_feature_constant: new ee.Uniform1i(Ke, Fe.u_is_size_feature_constant), u_size_t: new ee.Uniform1f(Ke, Fe.u_size_t), u_size: new ee.Uniform1f(Ke, Fe.u_size), u_camera_to_center_distance: new ee.Uniform1f(Ke, Fe.u_camera_to_center_distance), u_pitch: new ee.Uniform1f(Ke, Fe.u_pitch), u_rotate_symbol: new ee.Uniform1i(Ke, Fe.u_rotate_symbol), u_aspect_ratio: new ee.Uniform1f(Ke, Fe.u_aspect_ratio), u_fade_change: new ee.Uniform1f(Ke, Fe.u_fade_change), u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_label_plane_matrix: new ee.UniformMatrix4f(Ke, Fe.u_label_plane_matrix), u_coord_matrix: new ee.UniformMatrix4f(Ke, Fe.u_coord_matrix), u_is_text: new ee.Uniform1i(Ke, Fe.u_is_text), u_pitch_with_map: new ee.Uniform1i(Ke, Fe.u_pitch_with_map), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_texture: new ee.Uniform1i(Ke, Fe.u_texture), u_gamma_scale: new ee.Uniform1f(Ke, Fe.u_gamma_scale), u_device_pixel_ratio: new ee.Uniform1f(Ke, Fe.u_device_pixel_ratio), u_is_halo: new ee.Uniform1i(Ke, Fe.u_is_halo) };
                }, symbolTextAndIcon: function(Ke, Fe) {
                  return { u_is_size_zoom_constant: new ee.Uniform1i(Ke, Fe.u_is_size_zoom_constant), u_is_size_feature_constant: new ee.Uniform1i(Ke, Fe.u_is_size_feature_constant), u_size_t: new ee.Uniform1f(Ke, Fe.u_size_t), u_size: new ee.Uniform1f(Ke, Fe.u_size), u_camera_to_center_distance: new ee.Uniform1f(Ke, Fe.u_camera_to_center_distance), u_pitch: new ee.Uniform1f(Ke, Fe.u_pitch), u_rotate_symbol: new ee.Uniform1i(Ke, Fe.u_rotate_symbol), u_aspect_ratio: new ee.Uniform1f(Ke, Fe.u_aspect_ratio), u_fade_change: new ee.Uniform1f(Ke, Fe.u_fade_change), u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_label_plane_matrix: new ee.UniformMatrix4f(Ke, Fe.u_label_plane_matrix), u_coord_matrix: new ee.UniformMatrix4f(Ke, Fe.u_coord_matrix), u_is_text: new ee.Uniform1i(Ke, Fe.u_is_text), u_pitch_with_map: new ee.Uniform1i(Ke, Fe.u_pitch_with_map), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_texsize_icon: new ee.Uniform2f(Ke, Fe.u_texsize_icon), u_texture: new ee.Uniform1i(Ke, Fe.u_texture), u_texture_icon: new ee.Uniform1i(Ke, Fe.u_texture_icon), u_gamma_scale: new ee.Uniform1f(Ke, Fe.u_gamma_scale), u_device_pixel_ratio: new ee.Uniform1f(Ke, Fe.u_device_pixel_ratio), u_is_halo: new ee.Uniform1i(Ke, Fe.u_is_halo) };
                }, background: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_opacity: new ee.Uniform1f(Ke, Fe.u_opacity), u_color: new ee.UniformColor(Ke, Fe.u_color) };
                }, backgroundPattern: function(Ke, Fe) {
                  return { u_matrix: new ee.UniformMatrix4f(Ke, Fe.u_matrix), u_opacity: new ee.Uniform1f(Ke, Fe.u_opacity), u_image: new ee.Uniform1i(Ke, Fe.u_image), u_pattern_tl_a: new ee.Uniform2f(Ke, Fe.u_pattern_tl_a), u_pattern_br_a: new ee.Uniform2f(Ke, Fe.u_pattern_br_a), u_pattern_tl_b: new ee.Uniform2f(Ke, Fe.u_pattern_tl_b), u_pattern_br_b: new ee.Uniform2f(Ke, Fe.u_pattern_br_b), u_texsize: new ee.Uniform2f(Ke, Fe.u_texsize), u_mix: new ee.Uniform1f(Ke, Fe.u_mix), u_pattern_size_a: new ee.Uniform2f(Ke, Fe.u_pattern_size_a), u_pattern_size_b: new ee.Uniform2f(Ke, Fe.u_pattern_size_b), u_scale_a: new ee.Uniform1f(Ke, Fe.u_scale_a), u_scale_b: new ee.Uniform1f(Ke, Fe.u_scale_b), u_pixel_coord_upper: new ee.Uniform2f(Ke, Fe.u_pixel_coord_upper), u_pixel_coord_lower: new ee.Uniform2f(Ke, Fe.u_pixel_coord_lower), u_tile_units_to_pixels: new ee.Uniform1f(Ke, Fe.u_tile_units_to_pixels) };
                } };
                function zo(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  for (var nt = Ke.context, ht = nt.gl, Et = Ke.useProgram("collisionBox"), Mt = [], qt = 0, tn = 0, xn = 0; xn < Ae.length; xn++) {
                    var An = Ae[xn], On = Fe.getTile(An), Bn = On.getBucket(Te);
                    if (Bn) {
                      var qn = An.posMatrix;
                      Oe[0] === 0 && Oe[1] === 0 || (qn = Ke.translatePosMatrix(An.posMatrix, On, Oe, Be));
                      var Vn = Ue ? Bn.textCollisionBox : Bn.iconCollisionBox, sr = Bn.collisionCircleArray;
                      if (sr.length > 0) {
                        var Un = ee.create(), wr = qn;
                        ee.mul(Un, Bn.placementInvProjMatrix, Ke.transform.glCoordMatrix), ee.mul(Un, Un, Bn.placementViewportMatrix), Mt.push({ circleArray: sr, circleOffset: tn, transform: wr, invTransform: Un }), tn = qt += sr.length / 4;
                      }
                      Vn && Et.draw(nt, ht.LINES, wt.disabled, At.disabled, Ke.colorModeForRenderPass(), Ot.disabled, Gu(qn, Ke.transform, On), Te.id, Vn.layoutVertexBuffer, Vn.indexBuffer, Vn.segments, null, Ke.transform.zoom, null, null, Vn.collisionVertexBuffer);
                    }
                  }
                  if (Ue && Mt.length) {
                    var Cr = Ke.useProgram("collisionCircle"), Tr = new ee.StructArrayLayout2f1f2i16();
                    Tr.resize(4 * qt), Tr._trim();
                    for (var ii = 0, wi = 0, Wi = Mt; wi < Wi.length; wi += 1) for (var Yi = Wi[wi], ra = 0; ra < Yi.circleArray.length / 4; ra++) {
                      var ha = 4 * ra, ya = Yi.circleArray[ha + 0], va = Yi.circleArray[ha + 1], qa = Yi.circleArray[ha + 2], Aa = Yi.circleArray[ha + 3];
                      Tr.emplace(ii++, ya, va, qa, Aa, 0), Tr.emplace(ii++, ya, va, qa, Aa, 1), Tr.emplace(ii++, ya, va, qa, Aa, 2), Tr.emplace(ii++, ya, va, qa, Aa, 3);
                    }
                    (!So || So.length < 2 * qt) && (So = function(ys) {
                      var il = 2 * ys, Is = new ee.StructArrayLayout3ui6();
                      Is.resize(il), Is._trim();
                      for (var ml = 0; ml < il; ml++) {
                        var nl = 6 * ml;
                        Is.uint16[nl + 0] = 4 * ml + 0, Is.uint16[nl + 1] = 4 * ml + 1, Is.uint16[nl + 2] = 4 * ml + 2, Is.uint16[nl + 3] = 4 * ml + 2, Is.uint16[nl + 4] = 4 * ml + 3, Is.uint16[nl + 5] = 4 * ml + 0;
                      }
                      return Is;
                    }(qt));
                    for (var Xa = nt.createIndexBuffer(So, !0), Oa = nt.createVertexBuffer(Tr, ee.collisionCircleLayout.members, !0), Ya = 0, Wo = Mt; Ya < Wo.length; Ya += 1) {
                      var Ja = Wo[Ya], Uo = Ku(Ja.transform, Ja.invTransform, Ke.transform);
                      Cr.draw(nt, ht.TRIANGLES, wt.disabled, At.disabled, Ke.colorModeForRenderPass(), Ot.disabled, Uo, Te.id, Oa, Xa, ee.SegmentVector.simpleSegment(0, 2 * Ja.circleOffset, Ja.circleArray.length, Ja.circleArray.length / 2), null, Ke.transform.zoom, null, null, null);
                    }
                    Oa.destroy(), Xa.destroy();
                  }
                }
                var Yo = ee.identity(new Float32Array(16));
                function Ls(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = ee.getAnchorAlignment(Ke), nt = -(Ue.horizontalAlign - 0.5) * Fe, ht = -(Ue.verticalAlign - 0.5) * Te, Et = ee.evaluateVariableOffset(Ke, Ae);
                  return new ee.Point((nt / Oe + Et[0]) * Be, (ht / Oe + Et[1]) * Be);
                }
                function Ms(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt) {
                  var qt = Ke.text.placedSymbolArray, tn = Ke.text.dynamicLayoutVertexArray, xn = Ke.icon.dynamicLayoutVertexArray, An = {};
                  tn.clear();
                  for (var On = 0; On < qt.length; On++) {
                    var Bn = qt.get(On), qn = Ke.allowVerticalPlacement && !Bn.placedOrientation, Vn = Bn.hidden || !Bn.crossTileID || qn ? null : Ae[Bn.crossTileID];
                    if (Vn) {
                      var sr = new ee.Point(Bn.anchorX, Bn.anchorY), Un = Zr(sr, Te ? nt : Ue), wr = fi(Be.cameraToCenterDistance, Un.signedDistanceFromCamera), Cr = Oe.evaluateSizeForFeature(Ke.textSizeData, Et, Bn) * wr / ee.ONE_EM;
                      Te && (Cr *= Ke.tilePixelRatio / ht);
                      for (var Tr = Vn.width, ii = Vn.height, wi = Ls(Vn.anchor, Tr, ii, Vn.textOffset, Vn.textBoxScale, Cr), Wi = Te ? Zr(sr.add(wi), Ue).point : Un.point.add(Fe ? wi.rotate(-Be.angle) : wi), Yi = Ke.allowVerticalPlacement && Bn.placedOrientation === ee.WritingMode.vertical ? Math.PI / 2 : 0, ra = 0; ra < Bn.numGlyphs; ra++) ee.addDynamicAttributes(tn, Wi, Yi);
                      Mt && Bn.associatedIconIndex >= 0 && (An[Bn.associatedIconIndex] = { shiftedAnchor: Wi, angle: Yi });
                    } else oa(Bn.numGlyphs, tn);
                  }
                  if (Mt) {
                    xn.clear();
                    for (var ha = Ke.icon.placedSymbolArray, ya = 0; ya < ha.length; ya++) {
                      var va = ha.get(ya);
                      if (va.hidden) oa(va.numGlyphs, xn);
                      else {
                        var qa = An[ya];
                        if (qa) for (var Aa = 0; Aa < va.numGlyphs; Aa++) ee.addDynamicAttributes(xn, qa.shiftedAnchor, qa.angle);
                        else oa(va.numGlyphs, xn);
                      }
                    }
                    Ke.icon.dynamicLayoutVertexBuffer.updateData(xn);
                  }
                  Ke.text.dynamicLayoutVertexBuffer.updateData(tn);
                }
                function vi(Ke, Fe, Te) {
                  return Te.iconsInText && Fe ? "symbolTextAndIcon" : Ke ? "symbolSDF" : "symbolIcon";
                }
                function Ri(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt) {
                  for (var tn = Ke.context, xn = tn.gl, An = Ke.transform, On = nt === "map", Bn = ht === "map", qn = On && Te.layout.get("symbol-placement") !== "point", Vn = On && !Bn && !qn, sr = Te.layout.get("symbol-sort-key").constantOr(1) !== void 0, Un = !1, wr = Ke.depthModeForSublayer(0, wt.ReadOnly), Cr = Te.layout.get("text-variable-anchor"), Tr = [], ii = 0, wi = Ae; ii < wi.length; ii += 1) {
                    var Wi = wi[ii], Yi = Fe.getTile(Wi), ra = Yi.getBucket(Te);
                    if (ra) {
                      var ha = Oe ? ra.text : ra.icon;
                      if (ha && ha.segments.get().length) {
                        var ya = ha.programConfigurations.get(Te.id), va = Oe || ra.sdfIcons, qa = Oe ? ra.textSizeData : ra.iconSizeData, Aa = Bn || An.pitch !== 0, Xa = Ke.useProgram(vi(va, Oe, ra), ya), Oa = ee.evaluateSizeForZoom(qa, An.zoom), Ya = void 0, Wo = [0, 0], Ja = void 0, Uo = void 0, ys = null, il = void 0;
                        if (Oe) {
                          if (Ja = Yi.glyphAtlasTexture, Uo = xn.LINEAR, Ya = Yi.glyphAtlasTexture.size, ra.iconsInText) {
                            Wo = Yi.imageAtlasTexture.size, ys = Yi.imageAtlasTexture;
                            var Is = qa.kind === "composite" || qa.kind === "camera";
                            il = Aa || Ke.options.rotating || Ke.options.zooming || Is ? xn.LINEAR : xn.NEAREST;
                          }
                        } else {
                          var ml = Te.layout.get("icon-size").constantOr(0) !== 1 || ra.iconsNeedLinear;
                          Ja = Yi.imageAtlasTexture, Uo = va || Ke.options.rotating || Ke.options.zooming || ml || Aa ? xn.LINEAR : xn.NEAREST, Ya = Yi.imageAtlasTexture.size;
                        }
                        var nl = Gr(Yi, 1, Ke.transform.zoom), Il = Bi(Wi.posMatrix, Bn, On, Ke.transform, nl), Fl = gi(Wi.posMatrix, Bn, On, Ke.transform, nl), qc = Cr && ra.hasTextData(), ql = Te.layout.get("icon-text-fit") !== "none" && qc && ra.hasIconData();
                        qn && bi(ra, Wi.posMatrix, Ke, Oe, Il, Fl, Bn, Et);
                        var jc = Ke.translatePosMatrix(Wi.posMatrix, Yi, Be, Ue), Dl = qn || Oe && Cr || ql ? Yo : Il, Ml = Ke.translatePosMatrix(Fl, Yi, Be, Ue, !0), pc = va && Te.paint.get(Oe ? "text-halo-width" : "icon-halo-width").constantOr(1) !== 0, hl = { program: Xa, buffers: ha, uniformValues: va ? ra.iconsInText ? ro(qa.kind, Oa, Vn, Bn, Ke, jc, Dl, Ml, Ya, Wo) : No(qa.kind, Oa, Vn, Bn, Ke, jc, Dl, Ml, Oe, Ya) : Jc(qa.kind, Oa, Vn, Bn, Ke, jc, Dl, Ml, Oe, Ya), atlasTexture: Ja, atlasTextureIcon: ys, atlasInterpolation: Uo, atlasInterpolationIcon: il, isSDF: va, hasHalo: pc };
                        if (sr && ra.canOverlap) {
                          Un = !0;
                          for (var xl = 0, Bl = ha.segments.get(); xl < Bl.length; xl += 1) {
                            var al = Bl[xl];
                            Tr.push({ segments: new ee.SegmentVector([al]), sortKey: al.sortKey, state: hl });
                          }
                        } else Tr.push({ segments: ha.segments, sortKey: 0, state: hl });
                      }
                    }
                  }
                  Un && Tr.sort(function(wc, zu) {
                    return wc.sortKey - zu.sortKey;
                  });
                  for (var Oc = 0, nu = Tr; Oc < nu.length; Oc += 1) {
                    var Ic = nu[Oc], $s = Ic.state;
                    if (tn.activeTexture.set(xn.TEXTURE0), $s.atlasTexture.bind($s.atlasInterpolation, xn.CLAMP_TO_EDGE), $s.atlasTextureIcon && (tn.activeTexture.set(xn.TEXTURE1), $s.atlasTextureIcon && $s.atlasTextureIcon.bind($s.atlasInterpolationIcon, xn.CLAMP_TO_EDGE)), $s.isSDF) {
                      var Xc = $s.uniformValues;
                      $s.hasHalo && (Xc.u_is_halo = 1, ta($s.buffers, Ic.segments, Te, Ke, $s.program, wr, Mt, qt, Xc)), Xc.u_is_halo = 0;
                    }
                    ta($s.buffers, Ic.segments, Te, Ke, $s.program, wr, Mt, qt, $s.uniformValues);
                  }
                }
                function ta(Ke, Fe, Te, Ae, Oe, Be, Ue, nt, ht) {
                  var Et = Ae.context, Mt = Et.gl;
                  Oe.draw(Et, Mt.TRIANGLES, Be, Ue, nt, Ot.disabled, ht, Te.id, Ke.layoutVertexBuffer, Ke.indexBuffer, Fe, Te.paint, Ae.transform.zoom, Ke.programConfigurations.get(Te.id), Ke.dynamicLayoutVertexBuffer, Ke.opacityVertexBuffer);
                }
                function da(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  var nt, ht, Et, Mt, qt, tn = Ke.context.gl, xn = Te.paint.get("fill-pattern"), An = xn && xn.constantOr(1), On = Te.getCrossfadeParameters();
                  Ue ? (ht = An && !Te.getPaintProperty("fill-outline-color") ? "fillOutlinePattern" : "fillOutline", nt = tn.LINES) : (ht = An ? "fillPattern" : "fill", nt = tn.TRIANGLES);
                  for (var Bn = 0, qn = Ae; Bn < qn.length; Bn += 1) {
                    var Vn = qn[Bn], sr = Fe.getTile(Vn);
                    if (!An || sr.patternsLoaded()) {
                      var Un = sr.getBucket(Te);
                      if (Un) {
                        var wr = Un.programConfigurations.get(Te.id), Cr = Ke.useProgram(ht, wr);
                        An && (Ke.context.activeTexture.set(tn.TEXTURE0), sr.imageAtlasTexture.bind(tn.LINEAR, tn.CLAMP_TO_EDGE), wr.updatePaintBuffers(On));
                        var Tr = xn.constantOr(null);
                        if (Tr && sr.imageAtlas) {
                          var ii = sr.imageAtlas, wi = ii.patternPositions[Tr.to.toString()], Wi = ii.patternPositions[Tr.from.toString()];
                          wi && Wi && wr.setConstantPatternPositions(wi, Wi);
                        }
                        var Yi = Ke.translatePosMatrix(Vn.posMatrix, sr, Te.paint.get("fill-translate"), Te.paint.get("fill-translate-anchor"));
                        if (Ue) {
                          Mt = Un.indexBuffer2, qt = Un.segments2;
                          var ra = [tn.drawingBufferWidth, tn.drawingBufferHeight];
                          Et = ht === "fillOutlinePattern" && An ? Fu(Yi, Ke, On, sr, ra) : zi(Yi, ra);
                        } else Mt = Un.indexBuffer, qt = Un.segments, Et = An ? Kc(Yi, Ke, On, sr) : zc(Yi);
                        Cr.draw(Ke.context, nt, Oe, Ke.stencilModeForClipping(Vn), Be, Ot.disabled, Et, Te.id, Un.layoutVertexBuffer, Mt, qt, Te.paint, Ke.transform.zoom, wr);
                      }
                    }
                  }
                }
                function Ji(Ke, Fe, Te, Ae, Oe, Be, Ue) {
                  for (var nt = Ke.context, ht = nt.gl, Et = Te.paint.get("fill-extrusion-pattern"), Mt = Et.constantOr(1), qt = Te.getCrossfadeParameters(), tn = Te.paint.get("fill-extrusion-opacity"), xn = 0, An = Ae; xn < An.length; xn += 1) {
                    var On = An[xn], Bn = Fe.getTile(On), qn = Bn.getBucket(Te);
                    if (qn) {
                      var Vn = qn.programConfigurations.get(Te.id), sr = Ke.useProgram(Mt ? "fillExtrusionPattern" : "fillExtrusion", Vn);
                      Mt && (Ke.context.activeTexture.set(ht.TEXTURE0), Bn.imageAtlasTexture.bind(ht.LINEAR, ht.CLAMP_TO_EDGE), Vn.updatePaintBuffers(qt));
                      var Un = Et.constantOr(null);
                      if (Un && Bn.imageAtlas) {
                        var wr = Bn.imageAtlas, Cr = wr.patternPositions[Un.to.toString()], Tr = wr.patternPositions[Un.from.toString()];
                        Cr && Tr && Vn.setConstantPatternPositions(Cr, Tr);
                      }
                      var ii = Ke.translatePosMatrix(On.posMatrix, Bn, Te.paint.get("fill-extrusion-translate"), Te.paint.get("fill-extrusion-translate-anchor")), wi = Te.paint.get("fill-extrusion-vertical-gradient"), Wi = Mt ? Cu(ii, Ke, wi, tn, On, qt, Bn) : Nc(ii, Ke, wi, tn);
                      sr.draw(nt, nt.gl.TRIANGLES, Oe, Be, Ue, Ot.backCCW, Wi, Te.id, qn.layoutVertexBuffer, qn.indexBuffer, qn.segments, Te.paint, Ke.transform.zoom, Vn);
                    }
                  }
                }
                function Pa(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = Ke.context, nt = Ue.gl, ht = Fe.fbo;
                  if (ht) {
                    var Et = Ke.useProgram("hillshade");
                    Ue.activeTexture.set(nt.TEXTURE0), nt.bindTexture(nt.TEXTURE_2D, ht.colorAttachment.get());
                    var Mt = Kl(Ke, Fe, Te);
                    Et.draw(Ue, nt.TRIANGLES, Ae, Oe, Be, Ot.disabled, Mt, Te.id, Ke.rasterBoundsBuffer, Ke.quadTriangleIndexBuffer, Ke.rasterBoundsSegments);
                  }
                }
                function Ga(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = Ke.context, nt = Ue.gl, ht = Fe.dem;
                  if (ht && ht.data) {
                    var Et = ht.dim, Mt = ht.stride, qt = ht.getPixels();
                    if (Ue.activeTexture.set(nt.TEXTURE1), Ue.pixelStoreUnpackPremultiplyAlpha.set(!1), Fe.demTexture = Fe.demTexture || Ke.getTileTexture(Mt), Fe.demTexture) {
                      var tn = Fe.demTexture;
                      tn.update(qt, { premultiply: !1 }), tn.bind(nt.NEAREST, nt.CLAMP_TO_EDGE);
                    } else Fe.demTexture = new ee.Texture(Ue, qt, nt.RGBA, { premultiply: !1 }), Fe.demTexture.bind(nt.NEAREST, nt.CLAMP_TO_EDGE);
                    Ue.activeTexture.set(nt.TEXTURE0);
                    var xn = Fe.fbo;
                    if (!xn) {
                      var An = new ee.Texture(Ue, { width: Et, height: Et, data: null }, nt.RGBA);
                      An.bind(nt.LINEAR, nt.CLAMP_TO_EDGE), (xn = Fe.fbo = Ue.createFramebuffer(Et, Et, !0)).colorAttachment.set(An.texture);
                    }
                    Ue.bindFramebuffer.set(xn.framebuffer), Ue.viewport.set([0, 0, Et, Et]), Ke.useProgram("hillshadePrepare").draw(Ue, nt.TRIANGLES, Ae, Oe, Be, Ot.disabled, uu(Fe.tileID, ht), Te.id, Ke.rasterBoundsBuffer, Ke.quadTriangleIndexBuffer, Ke.rasterBoundsSegments), Fe.needsHillshadePrepare = !1;
                  }
                }
                function Ha(Ke, Fe, Te, Ae, Oe) {
                  var Be = Ae.paint.get("raster-fade-duration");
                  if (Be > 0) {
                    var Ue = ee.browser.now(), nt = (Ue - Ke.timeAdded) / Be, ht = Fe ? (Ue - Fe.timeAdded) / Be : -1, Et = Te.getSource(), Mt = Oe.coveringZoomLevel({ tileSize: Et.tileSize, roundZoom: Et.roundZoom }), qt = !Fe || Math.abs(Fe.tileID.overscaledZ - Mt) > Math.abs(Ke.tileID.overscaledZ - Mt), tn = qt && Ke.refreshedUponExpiration ? 1 : ee.clamp(qt ? nt : 1 - ht, 0, 1);
                    return Ke.refreshedUponExpiration && nt >= 1 && (Ke.refreshedUponExpiration = !1), Fe ? { opacity: 1, mix: 1 - tn } : { opacity: tn, mix: 0 };
                  }
                  return { opacity: 1, mix: 0 };
                }
                var Ba = new ee.Color(1, 0, 0, 1), ao = new ee.Color(0, 1, 0, 1), io = new ee.Color(0, 0, 1, 1), To = new ee.Color(1, 0, 1, 1), Mo = new ee.Color(0, 1, 1, 1);
                function po(Ke) {
                  var Fe = Ke.transform.padding;
                  Vo(Ke, Ke.transform.height - (Fe.top || 0), 3, Ba), Vo(Ke, Fe.bottom || 0, 3, ao), Xo(Ke, Fe.left || 0, 3, io), Xo(Ke, Ke.transform.width - (Fe.right || 0), 3, To);
                  var Te = Ke.transform.centerPoint;
                  (function(Ae, Oe, Be, Ue) {
                    var nt = 20, ht = 2;
                    Go(Ae, Oe - ht / 2, Be - nt / 2, ht, nt, Ue), Go(Ae, Oe - nt / 2, Be - ht / 2, nt, ht, Ue);
                  })(Ke, Te.x, Ke.transform.height - Te.y, Mo);
                }
                function Vo(Ke, Fe, Te, Ae) {
                  Go(Ke, 0, Fe + Te / 2, Ke.transform.width, Te, Ae);
                }
                function Xo(Ke, Fe, Te, Ae) {
                  Go(Ke, Fe - Te / 2, 0, Te, Ke.transform.height, Ae);
                }
                function Go(Ke, Fe, Te, Ae, Oe, Be) {
                  var Ue = Ke.context, nt = Ue.gl;
                  nt.enable(nt.SCISSOR_TEST), nt.scissor(Fe * ee.browser.devicePixelRatio, Te * ee.browser.devicePixelRatio, Ae * ee.browser.devicePixelRatio, Oe * ee.browser.devicePixelRatio), Ue.clear({ color: Be }), nt.disable(nt.SCISSOR_TEST);
                }
                function es(Ke, Fe, Te) {
                  var Ae = Ke.context, Oe = Ae.gl, Be = Te.posMatrix, Ue = Ke.useProgram("debug"), nt = wt.disabled, ht = At.disabled, Et = Ke.colorModeForRenderPass(), Mt = "$debug";
                  Ae.activeTexture.set(Oe.TEXTURE0), Ke.emptyTexture.bind(Oe.LINEAR, Oe.CLAMP_TO_EDGE), Ue.draw(Ae, Oe.LINE_STRIP, nt, ht, Et, Ot.disabled, I1(Be, ee.Color.red), Mt, Ke.debugBuffer, Ke.tileBorderIndexBuffer, Ke.debugSegments);
                  var qt = Fe.getTileByID(Te.key).latestRawTileData, tn = qt && qt.byteLength || 0, xn = Math.floor(tn / 1024), An = Fe.getTile(Te).tileSize, On = 512 / Math.min(An, 512) * (Te.overscaledZ / Ke.transform.zoom) * 0.5, Bn = Te.canonical.toString();
                  Te.overscaledZ !== Te.canonical.z && (Bn += " => " + Te.overscaledZ), function(qn, Vn) {
                    qn.initDebugOverlayCanvas();
                    var sr = qn.debugOverlayCanvas, Un = qn.context.gl, wr = qn.debugOverlayCanvas.getContext("2d");
                    wr.clearRect(0, 0, sr.width, sr.height), wr.shadowColor = "white", wr.shadowBlur = 2, wr.lineWidth = 1.5, wr.strokeStyle = "white", wr.textBaseline = "top", wr.font = "bold 36px Open Sans, sans-serif", wr.fillText(Vn, 5, 5), wr.strokeText(Vn, 5, 5), qn.debugOverlayTexture.update(sr), qn.debugOverlayTexture.bind(Un.LINEAR, Un.CLAMP_TO_EDGE);
                  }(Ke, Bn + " " + xn + "kb"), Ue.draw(Ae, Oe.TRIANGLES, nt, ht, Dt.alphaBlended, Ot.disabled, I1(Be, ee.Color.transparent, On), Mt, Ke.debugBuffer, Ke.quadTriangleIndexBuffer, Ke.debugSegments);
                }
                var Ts = { symbol: function(Ke, Fe, Te, Ae, Oe) {
                  if (Ke.renderPass === "translucent") {
                    var Be = At.disabled, Ue = Ke.colorModeForRenderPass();
                    Te.layout.get("text-variable-anchor") && function(nt, ht, Et, Mt, qt, tn, xn) {
                      for (var An = ht.transform, On = qt === "map", Bn = tn === "map", qn = 0, Vn = nt; qn < Vn.length; qn += 1) {
                        var sr = Vn[qn], Un = Mt.getTile(sr), wr = Un.getBucket(Et);
                        if (wr && wr.text && wr.text.segments.get().length) {
                          var Cr = wr.textSizeData, Tr = ee.evaluateSizeForZoom(Cr, An.zoom), ii = Gr(Un, 1, ht.transform.zoom), wi = Bi(sr.posMatrix, Bn, On, ht.transform, ii), Wi = Et.layout.get("icon-text-fit") !== "none" && wr.hasIconData();
                          if (Tr) {
                            var Yi = Math.pow(2, An.zoom - Un.tileID.overscaledZ);
                            Ms(wr, On, Bn, xn, ee.symbolSize, An, wi, sr.posMatrix, Yi, Tr, Wi);
                          }
                        }
                      }
                    }(Ae, Ke, Te, Fe, Te.layout.get("text-rotation-alignment"), Te.layout.get("text-pitch-alignment"), Oe), Te.paint.get("icon-opacity").constantOr(1) !== 0 && Ri(Ke, Fe, Te, Ae, !1, Te.paint.get("icon-translate"), Te.paint.get("icon-translate-anchor"), Te.layout.get("icon-rotation-alignment"), Te.layout.get("icon-pitch-alignment"), Te.layout.get("icon-keep-upright"), Be, Ue), Te.paint.get("text-opacity").constantOr(1) !== 0 && Ri(Ke, Fe, Te, Ae, !0, Te.paint.get("text-translate"), Te.paint.get("text-translate-anchor"), Te.layout.get("text-rotation-alignment"), Te.layout.get("text-pitch-alignment"), Te.layout.get("text-keep-upright"), Be, Ue), Fe.map.showCollisionBoxes && (zo(Ke, Fe, Te, Ae, Te.paint.get("text-translate"), Te.paint.get("text-translate-anchor"), !0), zo(Ke, Fe, Te, Ae, Te.paint.get("icon-translate"), Te.paint.get("icon-translate-anchor"), !1));
                  }
                }, circle: function(Ke, Fe, Te, Ae) {
                  if (Ke.renderPass === "translucent") {
                    var Oe = Te.paint.get("circle-opacity"), Be = Te.paint.get("circle-stroke-width"), Ue = Te.paint.get("circle-stroke-opacity"), nt = Te.layout.get("circle-sort-key").constantOr(1) !== void 0;
                    if (Oe.constantOr(1) !== 0 || Be.constantOr(1) !== 0 && Ue.constantOr(1) !== 0) {
                      for (var ht = Ke.context, Et = ht.gl, Mt = Ke.depthModeForSublayer(0, wt.ReadOnly), qt = At.disabled, tn = Ke.colorModeForRenderPass(), xn = [], An = 0; An < Ae.length; An++) {
                        var On = Ae[An], Bn = Fe.getTile(On), qn = Bn.getBucket(Te);
                        if (qn) {
                          var Vn = qn.programConfigurations.get(Te.id), sr = { programConfiguration: Vn, program: Ke.useProgram("circle", Vn), layoutVertexBuffer: qn.layoutVertexBuffer, indexBuffer: qn.indexBuffer, uniformValues: d1(Ke, On, Bn, Te) };
                          if (nt) for (var Un = 0, wr = qn.segments.get(); Un < wr.length; Un += 1) {
                            var Cr = wr[Un];
                            xn.push({ segments: new ee.SegmentVector([Cr]), sortKey: Cr.sortKey, state: sr });
                          }
                          else xn.push({ segments: qn.segments, sortKey: 0, state: sr });
                        }
                      }
                      nt && xn.sort(function(Aa, Xa) {
                        return Aa.sortKey - Xa.sortKey;
                      });
                      for (var Tr = 0, ii = xn; Tr < ii.length; Tr += 1) {
                        var wi = ii[Tr], Wi = wi.state, Yi = Wi.programConfiguration, ra = Wi.program, ha = Wi.layoutVertexBuffer, ya = Wi.indexBuffer, va = Wi.uniformValues, qa = wi.segments;
                        ra.draw(ht, Et.TRIANGLES, Mt, qt, tn, Ot.disabled, va, Te.id, ha, ya, qa, Te.paint, Ke.transform.zoom, Yi);
                      }
                    }
                  }
                }, heatmap: function(Ke, Fe, Te, Ae) {
                  if (Te.paint.get("heatmap-opacity") !== 0) if (Ke.renderPass === "offscreen") {
                    var Oe = Ke.context, Be = Oe.gl, Ue = At.disabled, nt = new Dt([Be.ONE, Be.ONE], ee.Color.transparent, [!0, !0, !0, !0]);
                    (function(On, Bn, qn) {
                      var Vn = On.gl;
                      On.activeTexture.set(Vn.TEXTURE1), On.viewport.set([0, 0, Bn.width / 4, Bn.height / 4]);
                      var sr = qn.heatmapFbo;
                      if (sr) Vn.bindTexture(Vn.TEXTURE_2D, sr.colorAttachment.get()), On.bindFramebuffer.set(sr.framebuffer);
                      else {
                        var Un = Vn.createTexture();
                        Vn.bindTexture(Vn.TEXTURE_2D, Un), Vn.texParameteri(Vn.TEXTURE_2D, Vn.TEXTURE_WRAP_S, Vn.CLAMP_TO_EDGE), Vn.texParameteri(Vn.TEXTURE_2D, Vn.TEXTURE_WRAP_T, Vn.CLAMP_TO_EDGE), Vn.texParameteri(Vn.TEXTURE_2D, Vn.TEXTURE_MIN_FILTER, Vn.LINEAR), Vn.texParameteri(Vn.TEXTURE_2D, Vn.TEXTURE_MAG_FILTER, Vn.LINEAR), sr = qn.heatmapFbo = On.createFramebuffer(Bn.width / 4, Bn.height / 4, !1), function(wr, Cr, Tr, ii) {
                          var wi = wr.gl, Wi = wr.extRenderToTextureHalfFloat ? wr.extTextureHalfFloat.HALF_FLOAT_OES : wi.UNSIGNED_BYTE;
                          wi.texImage2D(wi.TEXTURE_2D, 0, wi.RGBA, Cr.width / 4, Cr.height / 4, 0, wi.RGBA, Wi, null), ii.colorAttachment.set(Tr);
                        }(On, Bn, Un, sr);
                      }
                    })(Oe, Ke, Te), Oe.clear({ color: ee.Color.transparent });
                    for (var ht = 0; ht < Ae.length; ht++) {
                      var Et = Ae[ht];
                      if (!Fe.hasRenderableParent(Et)) {
                        var Mt = Fe.getTile(Et), qt = Mt.getBucket(Te);
                        if (qt) {
                          var tn = qt.programConfigurations.get(Te.id), xn = Ke.useProgram("heatmap", tn), An = Ke.transform.zoom;
                          xn.draw(Oe, Be.TRIANGLES, wt.disabled, Ue, nt, Ot.disabled, D1(Et.posMatrix, Mt, An, Te.paint.get("heatmap-intensity")), Te.id, qt.layoutVertexBuffer, qt.indexBuffer, qt.segments, Te.paint, Ke.transform.zoom, tn);
                        }
                      }
                    }
                    Oe.viewport.set([0, 0, Ke.width, Ke.height]);
                  } else Ke.renderPass === "translucent" && (Ke.context.setColorMode(Ke.colorModeForRenderPass()), function(On, Bn) {
                    var qn = On.context, Vn = qn.gl, sr = Bn.heatmapFbo;
                    if (sr) {
                      qn.activeTexture.set(Vn.TEXTURE0), Vn.bindTexture(Vn.TEXTURE_2D, sr.colorAttachment.get()), qn.activeTexture.set(Vn.TEXTURE1);
                      var Un = Bn.colorRampTexture;
                      Un || (Un = Bn.colorRampTexture = new ee.Texture(qn, Bn.colorRamp, Vn.RGBA)), Un.bind(Vn.LINEAR, Vn.CLAMP_TO_EDGE), On.useProgram("heatmapTexture").draw(qn, Vn.TRIANGLES, wt.disabled, At.disabled, On.colorModeForRenderPass(), Ot.disabled, Hl(On, Bn, 0, 1), Bn.id, On.viewportBuffer, On.quadTriangleIndexBuffer, On.viewportSegments, Bn.paint, On.transform.zoom);
                    }
                  }(Ke, Te));
                }, line: function(Ke, Fe, Te, Ae) {
                  if (Ke.renderPass === "translucent") {
                    var Oe = Te.paint.get("line-opacity"), Be = Te.paint.get("line-width");
                    if (Oe.constantOr(1) !== 0 && Be.constantOr(1) !== 0) for (var Ue = Ke.depthModeForSublayer(0, wt.ReadOnly), nt = Ke.colorModeForRenderPass(), ht = Te.paint.get("line-dasharray"), Et = Te.paint.get("line-pattern"), Mt = Et.constantOr(1), qt = Te.paint.get("line-gradient"), tn = Te.getCrossfadeParameters(), xn = Mt ? "linePattern" : ht ? "lineSDF" : qt ? "lineGradient" : "line", An = Ke.context, On = An.gl, Bn = !0, qn = 0, Vn = Ae; qn < Vn.length; qn += 1) {
                      var sr = Vn[qn], Un = Fe.getTile(sr);
                      if (!Mt || Un.patternsLoaded()) {
                        var wr = Un.getBucket(Te);
                        if (wr) {
                          var Cr = wr.programConfigurations.get(Te.id), Tr = Ke.context.program.get(), ii = Ke.useProgram(xn, Cr), wi = Bn || ii.program !== Tr, Wi = Et.constantOr(null);
                          if (Wi && Un.imageAtlas) {
                            var Yi = Un.imageAtlas, ra = Yi.patternPositions[Wi.to.toString()], ha = Yi.patternPositions[Wi.from.toString()];
                            ra && ha && Cr.setConstantPatternPositions(ra, ha);
                          }
                          var ya = Mt ? Wl(Ke, Un, Te, tn) : ht ? zl(Ke, Un, Te, ht, tn) : qt ? b1(Ke, Un, Te, wr.lineClipsArray.length) : Hs(Ke, Un, Te);
                          if (Mt) An.activeTexture.set(On.TEXTURE0), Un.imageAtlasTexture.bind(On.LINEAR, On.CLAMP_TO_EDGE), Cr.updatePaintBuffers(tn);
                          else if (ht && (wi || Ke.lineAtlas.dirty)) An.activeTexture.set(On.TEXTURE0), Ke.lineAtlas.bind(An);
                          else if (qt) {
                            var va = wr.gradients[Te.id], qa = va.texture;
                            if (Te.gradientVersion !== va.version) {
                              var Aa = 256;
                              if (Te.stepInterpolant) {
                                var Xa = Fe.getSource().maxzoom, Oa = sr.canonical.z === Xa ? Math.ceil(1 << Ke.transform.maxZoom - sr.canonical.z) : 1, Ya = wr.maxLineLength / ee.EXTENT * 1024 * Oa;
                                Aa = ee.clamp(ee.nextPowerOfTwo(Ya), 256, An.maxTextureSize);
                              }
                              va.gradient = ee.renderColorRamp({ expression: Te.gradientExpression(), evaluationKey: "lineProgress", resolution: Aa, image: va.gradient || void 0, clips: wr.lineClipsArray }), va.texture ? va.texture.update(va.gradient) : va.texture = new ee.Texture(An, va.gradient, On.RGBA), va.version = Te.gradientVersion, qa = va.texture;
                            }
                            An.activeTexture.set(On.TEXTURE0), qa.bind(Te.stepInterpolant ? On.NEAREST : On.LINEAR, On.CLAMP_TO_EDGE);
                          }
                          ii.draw(An, On.TRIANGLES, Ue, Ke.stencilModeForClipping(sr), nt, Ot.disabled, ya, Te.id, wr.layoutVertexBuffer, wr.indexBuffer, wr.segments, Te.paint, Ke.transform.zoom, Cr, wr.layoutVertexBuffer2), Bn = !1;
                        }
                      }
                    }
                  }
                }, fill: function(Ke, Fe, Te, Ae) {
                  var Oe = Te.paint.get("fill-color"), Be = Te.paint.get("fill-opacity");
                  if (Be.constantOr(1) !== 0) {
                    var Ue = Ke.colorModeForRenderPass(), nt = Te.paint.get("fill-pattern"), ht = Ke.opaquePassEnabledForLayer() && !nt.constantOr(1) && Oe.constantOr(ee.Color.transparent).a === 1 && Be.constantOr(0) === 1 ? "opaque" : "translucent";
                    if (Ke.renderPass === ht) {
                      var Et = Ke.depthModeForSublayer(1, Ke.renderPass === "opaque" ? wt.ReadWrite : wt.ReadOnly);
                      da(Ke, Fe, Te, Ae, Et, Ue, !1);
                    }
                    if (Ke.renderPass === "translucent" && Te.paint.get("fill-antialias")) {
                      var Mt = Ke.depthModeForSublayer(Te.getPaintProperty("fill-outline-color") ? 2 : 0, wt.ReadOnly);
                      da(Ke, Fe, Te, Ae, Mt, Ue, !0);
                    }
                  }
                }, "fill-extrusion": function(Ke, Fe, Te, Ae) {
                  var Oe = Te.paint.get("fill-extrusion-opacity");
                  if (Oe !== 0 && Ke.renderPass === "translucent") {
                    var Be = new wt(Ke.context.gl.LEQUAL, wt.ReadWrite, Ke.depthRangeFor3D);
                    if (Oe !== 1 || Te.paint.get("fill-extrusion-pattern").constantOr(1)) Ji(Ke, Fe, Te, Ae, Be, At.disabled, Dt.disabled), Ji(Ke, Fe, Te, Ae, Be, Ke.stencilModeFor3D(), Ke.colorModeForRenderPass());
                    else {
                      var Ue = Ke.colorModeForRenderPass();
                      Ji(Ke, Fe, Te, Ae, Be, At.disabled, Ue);
                    }
                  }
                }, hillshade: function(Ke, Fe, Te, Ae) {
                  if (Ke.renderPass === "offscreen" || Ke.renderPass === "translucent") {
                    for (var Oe = Ke.context, Be = Ke.depthModeForSublayer(0, wt.ReadOnly), Ue = Ke.colorModeForRenderPass(), nt = Ke.renderPass === "translucent" ? Ke.stencilConfigForOverlap(Ae) : [{}, Ae], ht = nt[0], Et = 0, Mt = nt[1]; Et < Mt.length; Et += 1) {
                      var qt = Mt[Et], tn = Fe.getTile(qt);
                      tn.needsHillshadePrepare && Ke.renderPass === "offscreen" ? Ga(Ke, tn, Te, Be, At.disabled, Ue) : Ke.renderPass === "translucent" && Pa(Ke, tn, Te, Be, ht[qt.overscaledZ], Ue);
                    }
                    Oe.viewport.set([0, 0, Ke.width, Ke.height]);
                  }
                }, raster: function(Ke, Fe, Te, Ae) {
                  if (Ke.renderPass === "translucent" && Te.paint.get("raster-opacity") !== 0 && Ae.length) for (var Oe = Ke.context, Be = Oe.gl, Ue = Fe.getSource(), nt = Ke.useProgram("raster"), ht = Ke.colorModeForRenderPass(), Et = Ue instanceof Ge ? [{}, Ae] : Ke.stencilConfigForOverlap(Ae), Mt = Et[0], qt = Et[1], tn = qt[qt.length - 1].overscaledZ, xn = !Ke.options.moving, An = 0, On = qt; An < On.length; An += 1) {
                    var Bn = On[An], qn = Ke.depthModeForSublayer(Bn.overscaledZ - tn, Te.paint.get("raster-opacity") === 1 ? wt.ReadWrite : wt.ReadOnly, Be.LESS), Vn = Fe.getTile(Bn), sr = Ke.transform.calculatePosMatrix(Bn.toUnwrapped(), xn);
                    Vn.registerFadeDuration(Te.paint.get("raster-fade-duration"));
                    var Un = Fe.findLoadedParent(Bn, 0), wr = Ha(Vn, Un, Fe, Te, Ke.transform), Cr = void 0, Tr = void 0, ii = Te.paint.get("raster-resampling") === "nearest" ? Be.NEAREST : Be.LINEAR;
                    Oe.activeTexture.set(Be.TEXTURE0), Vn.texture.bind(ii, Be.CLAMP_TO_EDGE, Be.LINEAR_MIPMAP_NEAREST), Oe.activeTexture.set(Be.TEXTURE1), Un ? (Un.texture.bind(ii, Be.CLAMP_TO_EDGE, Be.LINEAR_MIPMAP_NEAREST), Cr = Math.pow(2, Un.tileID.overscaledZ - Vn.tileID.overscaledZ), Tr = [Vn.tileID.canonical.x * Cr % 1, Vn.tileID.canonical.y * Cr % 1]) : Vn.texture.bind(ii, Be.CLAMP_TO_EDGE, Be.LINEAR_MIPMAP_NEAREST);
                    var wi = uo(sr, Tr || [0, 0], Cr || 1, wr, Te);
                    Ue instanceof Ge ? nt.draw(Oe, Be.TRIANGLES, qn, At.disabled, ht, Ot.disabled, wi, Te.id, Ue.boundsBuffer, Ke.quadTriangleIndexBuffer, Ue.boundsSegments) : nt.draw(Oe, Be.TRIANGLES, qn, Mt[Bn.overscaledZ], ht, Ot.disabled, wi, Te.id, Ke.rasterBoundsBuffer, Ke.quadTriangleIndexBuffer, Ke.rasterBoundsSegments);
                  }
                }, background: function(Ke, Fe, Te) {
                  var Ae = Te.paint.get("background-color"), Oe = Te.paint.get("background-opacity");
                  if (Oe !== 0) {
                    var Be = Ke.context, Ue = Be.gl, nt = Ke.transform, ht = nt.tileSize, Et = Te.paint.get("background-pattern");
                    if (!Ke.isPatternMissing(Et)) {
                      var Mt = !Et && Ae.a === 1 && Oe === 1 && Ke.opaquePassEnabledForLayer() ? "opaque" : "translucent";
                      if (Ke.renderPass === Mt) {
                        var qt = At.disabled, tn = Ke.depthModeForSublayer(0, Mt === "opaque" ? wt.ReadWrite : wt.ReadOnly), xn = Ke.colorModeForRenderPass(), An = Ke.useProgram(Et ? "backgroundPattern" : "background"), On = nt.coveringTiles({ tileSize: ht });
                        Et && (Be.activeTexture.set(Ue.TEXTURE0), Ke.imageManager.bind(Ke.context));
                        for (var Bn = Te.getCrossfadeParameters(), qn = 0, Vn = On; qn < Vn.length; qn += 1) {
                          var sr = Vn[qn], Un = Ke.transform.calculatePosMatrix(sr.toUnwrapped()), wr = Et ? bs(Un, Oe, Ke, Et, { tileID: sr, tileSize: ht }, Bn) : Io(Un, Oe, Ae);
                          An.draw(Be, Ue.TRIANGLES, tn, qt, xn, Ot.disabled, wr, Te.id, Ke.tileExtentBuffer, Ke.quadTriangleIndexBuffer, Ke.tileExtentSegments);
                        }
                      }
                    }
                  }
                }, debug: function(Ke, Fe, Te) {
                  for (var Ae = 0; Ae < Te.length; Ae++) es(Ke, Fe, Te[Ae]);
                }, custom: function(Ke, Fe, Te) {
                  var Ae = Ke.context, Oe = Te.implementation;
                  if (Ke.renderPass === "offscreen") {
                    var Be = Oe.prerender;
                    Be && (Ke.setCustomLayerDefaults(), Ae.setColorMode(Ke.colorModeForRenderPass()), Be.call(Oe, Ae.gl, Ke.transform.customLayerMatrix()), Ae.setDirty(), Ke.setBaseState());
                  } else if (Ke.renderPass === "translucent") {
                    Ke.setCustomLayerDefaults(), Ae.setColorMode(Ke.colorModeForRenderPass()), Ae.setStencilMode(At.disabled);
                    var Ue = Oe.renderingMode === "3d" ? new wt(Ke.context.gl.LEQUAL, wt.ReadWrite, Ke.depthRangeFor3D) : Ke.depthModeForSublayer(0, wt.ReadOnly);
                    Ae.setDepthMode(Ue), Oe.render(Ae.gl, Ke.transform.customLayerMatrix()), Ae.setDirty(), Ke.setBaseState(), Ae.bindFramebuffer.set(null);
                  }
                } }, Fo = function(Ke, Fe) {
                  this.context = new un(Ke), this.transform = Fe, this._tileTextures = {}, this.setup(), this.numSublayers = yn.maxUnderzooming + yn.maxOverzooming + 1, this.depthEpsilon = 1 / Math.pow(2, 16), this.crossTileSymbolIndex = new Ai(), this.gpuTimers = {};
                };
                Fo.prototype.resize = function(Ke, Fe) {
                  if (this.width = Ke * ee.browser.devicePixelRatio, this.height = Fe * ee.browser.devicePixelRatio, this.context.viewport.set([0, 0, this.width, this.height]), this.style) for (var Te = 0, Ae = this.style._order; Te < Ae.length; Te += 1) {
                    var Oe = Ae[Te];
                    this.style._layers[Oe].resize();
                  }
                }, Fo.prototype.setup = function() {
                  var Ke = this.context, Fe = new ee.StructArrayLayout2i4();
                  Fe.emplaceBack(0, 0), Fe.emplaceBack(ee.EXTENT, 0), Fe.emplaceBack(0, ee.EXTENT), Fe.emplaceBack(ee.EXTENT, ee.EXTENT), this.tileExtentBuffer = Ke.createVertexBuffer(Fe, Mi.members), this.tileExtentSegments = ee.SegmentVector.simpleSegment(0, 0, 4, 2);
                  var Te = new ee.StructArrayLayout2i4();
                  Te.emplaceBack(0, 0), Te.emplaceBack(ee.EXTENT, 0), Te.emplaceBack(0, ee.EXTENT), Te.emplaceBack(ee.EXTENT, ee.EXTENT), this.debugBuffer = Ke.createVertexBuffer(Te, Mi.members), this.debugSegments = ee.SegmentVector.simpleSegment(0, 0, 4, 5);
                  var Ae = new ee.StructArrayLayout4i8();
                  Ae.emplaceBack(0, 0, 0, 0), Ae.emplaceBack(ee.EXTENT, 0, ee.EXTENT, 0), Ae.emplaceBack(0, ee.EXTENT, 0, ee.EXTENT), Ae.emplaceBack(ee.EXTENT, ee.EXTENT, ee.EXTENT, ee.EXTENT), this.rasterBoundsBuffer = Ke.createVertexBuffer(Ae, Ze.members), this.rasterBoundsSegments = ee.SegmentVector.simpleSegment(0, 0, 4, 2);
                  var Oe = new ee.StructArrayLayout2i4();
                  Oe.emplaceBack(0, 0), Oe.emplaceBack(1, 0), Oe.emplaceBack(0, 1), Oe.emplaceBack(1, 1), this.viewportBuffer = Ke.createVertexBuffer(Oe, Mi.members), this.viewportSegments = ee.SegmentVector.simpleSegment(0, 0, 4, 2);
                  var Be = new ee.StructArrayLayout1ui2();
                  Be.emplaceBack(0), Be.emplaceBack(1), Be.emplaceBack(3), Be.emplaceBack(2), Be.emplaceBack(0), this.tileBorderIndexBuffer = Ke.createIndexBuffer(Be);
                  var Ue = new ee.StructArrayLayout3ui6();
                  Ue.emplaceBack(0, 1, 2), Ue.emplaceBack(2, 1, 3), this.quadTriangleIndexBuffer = Ke.createIndexBuffer(Ue), this.emptyTexture = new ee.Texture(Ke, { width: 1, height: 1, data: new Uint8Array([0, 0, 0, 0]) }, Ke.gl.RGBA);
                  var nt = this.context.gl;
                  this.stencilClearMode = new At({ func: nt.ALWAYS, mask: 0 }, 0, 255, nt.ZERO, nt.ZERO, nt.ZERO);
                }, Fo.prototype.clearStencil = function() {
                  var Ke = this.context, Fe = Ke.gl;
                  this.nextStencilID = 1, this.currentStencilSource = void 0;
                  var Te = ee.create();
                  ee.ortho(Te, 0, this.width, this.height, 0, 0, 1), ee.scale(Te, Te, [Fe.drawingBufferWidth, Fe.drawingBufferHeight, 0]), this.useProgram("clippingMask").draw(Ke, Fe.TRIANGLES, wt.disabled, this.stencilClearMode, Dt.disabled, Ot.disabled, f1(Te), "$clipping", this.viewportBuffer, this.quadTriangleIndexBuffer, this.viewportSegments);
                }, Fo.prototype._renderTileClippingMasks = function(Ke, Fe) {
                  if (this.currentStencilSource !== Ke.source && Ke.isTileClipped() && Fe && Fe.length) {
                    this.currentStencilSource = Ke.source;
                    var Te = this.context, Ae = Te.gl;
                    this.nextStencilID + Fe.length > 256 && this.clearStencil(), Te.setColorMode(Dt.disabled), Te.setDepthMode(wt.disabled);
                    var Oe = this.useProgram("clippingMask");
                    this._tileClippingMaskIDs = {};
                    for (var Be = 0, Ue = Fe; Be < Ue.length; Be += 1) {
                      var nt = Ue[Be], ht = this._tileClippingMaskIDs[nt.key] = this.nextStencilID++;
                      Oe.draw(Te, Ae.TRIANGLES, wt.disabled, new At({ func: Ae.ALWAYS, mask: 0 }, ht, 255, Ae.KEEP, Ae.KEEP, Ae.REPLACE), Dt.disabled, Ot.disabled, f1(nt.posMatrix), "$clipping", this.tileExtentBuffer, this.quadTriangleIndexBuffer, this.tileExtentSegments);
                    }
                  }
                }, Fo.prototype.stencilModeFor3D = function() {
                  this.currentStencilSource = void 0, this.nextStencilID + 1 > 256 && this.clearStencil();
                  var Ke = this.nextStencilID++, Fe = this.context.gl;
                  return new At({ func: Fe.NOTEQUAL, mask: 255 }, Ke, 255, Fe.KEEP, Fe.KEEP, Fe.REPLACE);
                }, Fo.prototype.stencilModeForClipping = function(Ke) {
                  var Fe = this.context.gl;
                  return new At({ func: Fe.EQUAL, mask: 255 }, this._tileClippingMaskIDs[Ke.key], 0, Fe.KEEP, Fe.KEEP, Fe.REPLACE);
                }, Fo.prototype.stencilConfigForOverlap = function(Ke) {
                  var Fe, Te = this.context.gl, Ae = Ke.sort(function(ht, Et) {
                    return Et.overscaledZ - ht.overscaledZ;
                  }), Oe = Ae[Ae.length - 1].overscaledZ, Be = Ae[0].overscaledZ - Oe + 1;
                  if (Be > 1) {
                    this.currentStencilSource = void 0, this.nextStencilID + Be > 256 && this.clearStencil();
                    for (var Ue = {}, nt = 0; nt < Be; nt++) Ue[nt + Oe] = new At({ func: Te.GEQUAL, mask: 255 }, nt + this.nextStencilID, 255, Te.KEEP, Te.KEEP, Te.REPLACE);
                    return this.nextStencilID += Be, [Ue, Ae];
                  }
                  return [(Fe = {}, Fe[Oe] = At.disabled, Fe), Ae];
                }, Fo.prototype.colorModeForRenderPass = function() {
                  var Ke = this.context.gl;
                  if (this._showOverdrawInspector) {
                    var Fe = 0.125;
                    return new Dt([Ke.CONSTANT_COLOR, Ke.ONE], new ee.Color(Fe, Fe, Fe, 0), [!0, !0, !0, !0]);
                  }
                  return this.renderPass === "opaque" ? Dt.unblended : Dt.alphaBlended;
                }, Fo.prototype.depthModeForSublayer = function(Ke, Fe, Te) {
                  if (!this.opaquePassEnabledForLayer()) return wt.disabled;
                  var Ae = 1 - ((1 + this.currentLayer) * this.numSublayers + Ke) * this.depthEpsilon;
                  return new wt(Te || this.context.gl.LEQUAL, Fe, [Ae, Ae]);
                }, Fo.prototype.opaquePassEnabledForLayer = function() {
                  return this.currentLayer < this.opaquePassCutoff;
                }, Fo.prototype.render = function(Ke, Fe) {
                  var Te = this;
                  this.style = Ke, this.options = Fe, this.lineAtlas = Ke.lineAtlas, this.imageManager = Ke.imageManager, this.glyphManager = Ke.glyphManager, this.symbolFadeChange = Ke.placement.symbolFadeChange(ee.browser.now()), this.imageManager.beginFrame();
                  var Ae = this.style._order, Oe = this.style.sourceCaches;
                  for (var Be in Oe) {
                    var Ue = Oe[Be];
                    Ue.used && Ue.prepare(this.context);
                  }
                  var nt, ht, Et = {}, Mt = {}, qt = {};
                  for (var tn in Oe) {
                    var xn = Oe[tn];
                    Et[tn] = xn.getVisibleCoordinates(), Mt[tn] = Et[tn].slice().reverse(), qt[tn] = xn.getVisibleCoordinates(!0).reverse();
                  }
                  this.opaquePassCutoff = 1 / 0;
                  for (var An = 0; An < Ae.length; An++) {
                    var On = Ae[An];
                    if (this.style._layers[On].is3D()) {
                      this.opaquePassCutoff = An;
                      break;
                    }
                  }
                  this.renderPass = "offscreen";
                  for (var Bn = 0, qn = Ae; Bn < qn.length; Bn += 1) {
                    var Vn = qn[Bn], sr = this.style._layers[Vn];
                    if (sr.hasOffscreenPass() && !sr.isHidden(this.transform.zoom)) {
                      var Un = Mt[sr.source];
                      (sr.type === "custom" || Un.length) && this.renderLayer(this, Oe[sr.source], sr, Un);
                    }
                  }
                  for (this.context.bindFramebuffer.set(null), this.context.clear({ color: Fe.showOverdrawInspector ? ee.Color.black : ee.Color.transparent, depth: 1 }), this.clearStencil(), this._showOverdrawInspector = Fe.showOverdrawInspector, this.depthRangeFor3D = [0, 1 - (Ke._order.length + 2) * this.numSublayers * this.depthEpsilon], this.renderPass = "opaque", this.currentLayer = Ae.length - 1; this.currentLayer >= 0; this.currentLayer--) {
                    var wr = this.style._layers[Ae[this.currentLayer]], Cr = Oe[wr.source], Tr = Et[wr.source];
                    this._renderTileClippingMasks(wr, Tr), this.renderLayer(this, Cr, wr, Tr);
                  }
                  for (this.renderPass = "translucent", this.currentLayer = 0; this.currentLayer < Ae.length; this.currentLayer++) {
                    var ii = this.style._layers[Ae[this.currentLayer]], wi = Oe[ii.source], Wi = (ii.type === "symbol" ? qt : Mt)[ii.source];
                    this._renderTileClippingMasks(ii, Et[ii.source]), this.renderLayer(this, wi, ii, Wi);
                  }
                  this.options.showTileBoundaries && (ee.values(this.style._layers).forEach(function(Yi) {
                    Yi.source && !Yi.isHidden(Te.transform.zoom) && (Yi.source !== (ht && ht.id) && (ht = Te.style.sourceCaches[Yi.source]), (!nt || nt.getSource().maxzoom < ht.getSource().maxzoom) && (nt = ht));
                  }), nt && Ts.debug(this, nt, nt.getVisibleCoordinates())), this.options.showPadding && po(this), this.context.setDefault();
                }, Fo.prototype.renderLayer = function(Ke, Fe, Te, Ae) {
                  Te.isHidden(this.transform.zoom) || (Te.type === "background" || Te.type === "custom" || Ae.length) && (this.id = Te.id, this.gpuTimingStart(Te), Ts[Te.type](Ke, Fe, Te, Ae, this.style.placement.variableOffsets), this.gpuTimingEnd());
                }, Fo.prototype.gpuTimingStart = function(Ke) {
                  if (this.options.gpuTiming) {
                    var Fe = this.context.extTimerQuery, Te = this.gpuTimers[Ke.id];
                    Te || (Te = this.gpuTimers[Ke.id] = { calls: 0, cpuTime: 0, query: Fe.createQueryEXT() }), Te.calls++, Fe.beginQueryEXT(Fe.TIME_ELAPSED_EXT, Te.query);
                  }
                }, Fo.prototype.gpuTimingEnd = function() {
                  if (this.options.gpuTiming) {
                    var Ke = this.context.extTimerQuery;
                    Ke.endQueryEXT(Ke.TIME_ELAPSED_EXT);
                  }
                }, Fo.prototype.collectGpuTimers = function() {
                  var Ke = this.gpuTimers;
                  return this.gpuTimers = {}, Ke;
                }, Fo.prototype.queryGpuTimers = function(Ke) {
                  var Fe = {};
                  for (var Te in Ke) {
                    var Ae = Ke[Te], Oe = this.context.extTimerQuery, Be = Oe.getQueryObjectEXT(Ae.query, Oe.QUERY_RESULT_EXT) / 1e6;
                    Oe.deleteQueryEXT(Ae.query), Fe[Te] = Be;
                  }
                  return Fe;
                }, Fo.prototype.translatePosMatrix = function(Ke, Fe, Te, Ae, Oe) {
                  if (!Te[0] && !Te[1]) return Ke;
                  var Be = Oe ? Ae === "map" ? this.transform.angle : 0 : Ae === "viewport" ? -this.transform.angle : 0;
                  if (Be) {
                    var Ue = Math.sin(Be), nt = Math.cos(Be);
                    Te = [Te[0] * nt - Te[1] * Ue, Te[0] * Ue + Te[1] * nt];
                  }
                  var ht = [Oe ? Te[0] : Gr(Fe, Te[0], this.transform.zoom), Oe ? Te[1] : Gr(Fe, Te[1], this.transform.zoom), 0], Et = new Float32Array(16);
                  return ee.translate(Et, Ke, ht), Et;
                }, Fo.prototype.saveTileTexture = function(Ke) {
                  var Fe = this._tileTextures[Ke.size[0]];
                  Fe ? Fe.push(Ke) : this._tileTextures[Ke.size[0]] = [Ke];
                }, Fo.prototype.getTileTexture = function(Ke) {
                  var Fe = this._tileTextures[Ke];
                  return Fe && Fe.length > 0 ? Fe.pop() : null;
                }, Fo.prototype.isPatternMissing = function(Ke) {
                  if (!Ke) return !1;
                  if (!Ke.from || !Ke.to) return !0;
                  var Fe = this.imageManager.getPattern(Ke.from.toString()), Te = this.imageManager.getPattern(Ke.to.toString());
                  return !Fe || !Te;
                }, Fo.prototype.useProgram = function(Ke, Fe) {
                  this.cache = this.cache || {};
                  var Te = "" + Ke + (Fe ? Fe.cacheKey : "") + (this._showOverdrawInspector ? "/overdraw" : "");
                  return this.cache[Te] || (this.cache[Te] = new yc(this.context, Ke, Vl[Ke], Fe, ws[Ke], this._showOverdrawInspector)), this.cache[Te];
                }, Fo.prototype.setCustomLayerDefaults = function() {
                  this.context.unbindVAO(), this.context.cullFace.setDefault(), this.context.activeTexture.setDefault(), this.context.pixelStoreUnpack.setDefault(), this.context.pixelStoreUnpackPremultiplyAlpha.setDefault(), this.context.pixelStoreUnpackFlipY.setDefault();
                }, Fo.prototype.setBaseState = function() {
                  var Ke = this.context.gl;
                  this.context.cullFace.set(!1), this.context.viewport.set([0, 0, this.width, this.height]), this.context.blendEquation.set(Ke.FUNC_ADD);
                }, Fo.prototype.initDebugOverlayCanvas = function() {
                  if (this.debugOverlayCanvas == null) {
                    this.debugOverlayCanvas = ee.window.document.createElement("canvas"), this.debugOverlayCanvas.width = 512, this.debugOverlayCanvas.height = 512;
                    var Ke = this.context.gl;
                    this.debugOverlayTexture = new ee.Texture(this.context, this.debugOverlayCanvas, Ke.RGBA);
                  }
                }, Fo.prototype.destroy = function() {
                  this.emptyTexture.destroy(), this.debugOverlayTexture && this.debugOverlayTexture.destroy();
                };
                var Oo = function(Ke, Fe) {
                  this.points = Ke, this.planes = Fe;
                };
                Oo.fromInvProjectionMatrix = function(Ke, Fe, Te) {
                  var Ae = Math.pow(2, Te), Oe = [[-1, 1, -1, 1], [1, 1, -1, 1], [1, -1, -1, 1], [-1, -1, -1, 1], [-1, 1, 1, 1], [1, 1, 1, 1], [1, -1, 1, 1], [-1, -1, 1, 1]].map(function(Ue) {
                    return ee.transformMat4([], Ue, Ke);
                  }).map(function(Ue) {
                    return ee.scale$1([], Ue, 1 / Ue[3] / Fe * Ae);
                  }), Be = [[0, 1, 2], [6, 5, 4], [0, 3, 7], [2, 1, 5], [3, 2, 6], [0, 4, 5]].map(function(Ue) {
                    var nt = ee.sub([], Oe[Ue[0]], Oe[Ue[1]]), ht = ee.sub([], Oe[Ue[2]], Oe[Ue[1]]), Et = ee.normalize([], ee.cross([], nt, ht)), Mt = -ee.dot(Et, Oe[Ue[1]]);
                    return Et.concat(Mt);
                  });
                  return new Oo(Oe, Be);
                };
                var gs = function(Ke, Fe) {
                  this.min = Ke, this.max = Fe, this.center = ee.scale$2([], ee.add([], this.min, this.max), 0.5);
                };
                gs.prototype.quadrant = function(Ke) {
                  for (var Fe = [Ke % 2 == 0, Ke < 2], Te = ee.clone$2(this.min), Ae = ee.clone$2(this.max), Oe = 0; Oe < Fe.length; Oe++) Te[Oe] = Fe[Oe] ? this.min[Oe] : this.center[Oe], Ae[Oe] = Fe[Oe] ? this.center[Oe] : this.max[Oe];
                  return Ae[2] = this.max[2], new gs(Te, Ae);
                }, gs.prototype.distanceX = function(Ke) {
                  return Math.max(Math.min(this.max[0], Ke[0]), this.min[0]) - Ke[0];
                }, gs.prototype.distanceY = function(Ke) {
                  return Math.max(Math.min(this.max[1], Ke[1]), this.min[1]) - Ke[1];
                }, gs.prototype.intersects = function(Ke) {
                  for (var Fe = [[this.min[0], this.min[1], 0, 1], [this.max[0], this.min[1], 0, 1], [this.max[0], this.max[1], 0, 1], [this.min[0], this.max[1], 0, 1]], Te = !0, Ae = 0; Ae < Ke.planes.length; Ae++) {
                    for (var Oe = Ke.planes[Ae], Be = 0, Ue = 0; Ue < Fe.length; Ue++) Be += ee.dot$1(Oe, Fe[Ue]) >= 0;
                    if (Be === 0) return 0;
                    Be !== Fe.length && (Te = !1);
                  }
                  if (Te) return 2;
                  for (var nt = 0; nt < 3; nt++) {
                    for (var ht = Number.MAX_VALUE, Et = -Number.MAX_VALUE, Mt = 0; Mt < Ke.points.length; Mt++) {
                      var qt = Ke.points[Mt][nt] - this.min[nt];
                      ht = Math.min(ht, qt), Et = Math.max(Et, qt);
                    }
                    if (Et < 0 || ht > this.max[nt] - this.min[nt]) return 0;
                  }
                  return 1;
                };
                var Bu = function(Ke, Fe, Te, Ae) {
                  if (Ke === void 0 && (Ke = 0), Fe === void 0 && (Fe = 0), Te === void 0 && (Te = 0), Ae === void 0 && (Ae = 0), isNaN(Ke) || Ke < 0 || isNaN(Fe) || Fe < 0 || isNaN(Te) || Te < 0 || isNaN(Ae) || Ae < 0) throw new Error("Invalid value for edge-insets, top, bottom, left and right must all be numbers");
                  this.top = Ke, this.bottom = Fe, this.left = Te, this.right = Ae;
                };
                Bu.prototype.interpolate = function(Ke, Fe, Te) {
                  return Fe.top != null && Ke.top != null && (this.top = ee.number(Ke.top, Fe.top, Te)), Fe.bottom != null && Ke.bottom != null && (this.bottom = ee.number(Ke.bottom, Fe.bottom, Te)), Fe.left != null && Ke.left != null && (this.left = ee.number(Ke.left, Fe.left, Te)), Fe.right != null && Ke.right != null && (this.right = ee.number(Ke.right, Fe.right, Te)), this;
                }, Bu.prototype.getCenter = function(Ke, Fe) {
                  var Te = ee.clamp((this.left + Ke - this.right) / 2, 0, Ke), Ae = ee.clamp((this.top + Fe - this.bottom) / 2, 0, Fe);
                  return new ee.Point(Te, Ae);
                }, Bu.prototype.equals = function(Ke) {
                  return this.top === Ke.top && this.bottom === Ke.bottom && this.left === Ke.left && this.right === Ke.right;
                }, Bu.prototype.clone = function() {
                  return new Bu(this.top, this.bottom, this.left, this.right);
                }, Bu.prototype.toJSON = function() {
                  return { top: this.top, bottom: this.bottom, left: this.left, right: this.right };
                };
                var Ql = function(Ke, Fe, Te, Ae, Oe) {
                  this.tileSize = 512, this.maxValidLatitude = 85.051129, this._renderWorldCopies = Oe === void 0 || Oe, this._minZoom = Ke || 0, this._maxZoom = Fe || 22, this._minPitch = Te ?? 0, this._maxPitch = Ae ?? 60, this.setMaxBounds(), this.width = 0, this.height = 0, this._center = new ee.LngLat(0, 0), this.zoom = 0, this.angle = 0, this._fov = 0.6435011087932844, this._pitch = 0, this._unmodified = !0, this._edgeInsets = new Bu(), this._posMatrixCache = {}, this._alignedPosMatrixCache = {};
                }, dl = { minZoom: { configurable: !0 }, maxZoom: { configurable: !0 }, minPitch: { configurable: !0 }, maxPitch: { configurable: !0 }, renderWorldCopies: { configurable: !0 }, worldSize: { configurable: !0 }, centerOffset: { configurable: !0 }, size: { configurable: !0 }, bearing: { configurable: !0 }, pitch: { configurable: !0 }, fov: { configurable: !0 }, zoom: { configurable: !0 }, center: { configurable: !0 }, padding: { configurable: !0 }, centerPoint: { configurable: !0 }, unmodified: { configurable: !0 }, point: { configurable: !0 } };
                Ql.prototype.clone = function() {
                  var Ke = new Ql(this._minZoom, this._maxZoom, this._minPitch, this.maxPitch, this._renderWorldCopies);
                  return Ke.tileSize = this.tileSize, Ke.latRange = this.latRange, Ke.width = this.width, Ke.height = this.height, Ke._center = this._center, Ke.zoom = this.zoom, Ke.angle = this.angle, Ke._fov = this._fov, Ke._pitch = this._pitch, Ke._unmodified = this._unmodified, Ke._edgeInsets = this._edgeInsets.clone(), Ke._calcMatrices(), Ke;
                }, dl.minZoom.get = function() {
                  return this._minZoom;
                }, dl.minZoom.set = function(Ke) {
                  this._minZoom !== Ke && (this._minZoom = Ke, this.zoom = Math.max(this.zoom, Ke));
                }, dl.maxZoom.get = function() {
                  return this._maxZoom;
                }, dl.maxZoom.set = function(Ke) {
                  this._maxZoom !== Ke && (this._maxZoom = Ke, this.zoom = Math.min(this.zoom, Ke));
                }, dl.minPitch.get = function() {
                  return this._minPitch;
                }, dl.minPitch.set = function(Ke) {
                  this._minPitch !== Ke && (this._minPitch = Ke, this.pitch = Math.max(this.pitch, Ke));
                }, dl.maxPitch.get = function() {
                  return this._maxPitch;
                }, dl.maxPitch.set = function(Ke) {
                  this._maxPitch !== Ke && (this._maxPitch = Ke, this.pitch = Math.min(this.pitch, Ke));
                }, dl.renderWorldCopies.get = function() {
                  return this._renderWorldCopies;
                }, dl.renderWorldCopies.set = function(Ke) {
                  Ke === void 0 ? Ke = !0 : Ke === null && (Ke = !1), this._renderWorldCopies = Ke;
                }, dl.worldSize.get = function() {
                  return this.tileSize * this.scale;
                }, dl.centerOffset.get = function() {
                  return this.centerPoint._sub(this.size._div(2));
                }, dl.size.get = function() {
                  return new ee.Point(this.width, this.height);
                }, dl.bearing.get = function() {
                  return -this.angle / Math.PI * 180;
                }, dl.bearing.set = function(Ke) {
                  var Fe = -ee.wrap(Ke, -180, 180) * Math.PI / 180;
                  this.angle !== Fe && (this._unmodified = !1, this.angle = Fe, this._calcMatrices(), this.rotationMatrix = ee.create$2(), ee.rotate(this.rotationMatrix, this.rotationMatrix, this.angle));
                }, dl.pitch.get = function() {
                  return this._pitch / Math.PI * 180;
                }, dl.pitch.set = function(Ke) {
                  var Fe = ee.clamp(Ke, this.minPitch, this.maxPitch) / 180 * Math.PI;
                  this._pitch !== Fe && (this._unmodified = !1, this._pitch = Fe, this._calcMatrices());
                }, dl.fov.get = function() {
                  return this._fov / Math.PI * 180;
                }, dl.fov.set = function(Ke) {
                  Ke = Math.max(0.01, Math.min(60, Ke)), this._fov !== Ke && (this._unmodified = !1, this._fov = Ke / 180 * Math.PI, this._calcMatrices());
                }, dl.zoom.get = function() {
                  return this._zoom;
                }, dl.zoom.set = function(Ke) {
                  var Fe = Math.min(Math.max(Ke, this.minZoom), this.maxZoom);
                  this._zoom !== Fe && (this._unmodified = !1, this._zoom = Fe, this.scale = this.zoomScale(Fe), this.tileZoom = Math.floor(Fe), this.zoomFraction = Fe - this.tileZoom, this._constrain(), this._calcMatrices());
                }, dl.center.get = function() {
                  return this._center;
                }, dl.center.set = function(Ke) {
                  Ke.lat === this._center.lat && Ke.lng === this._center.lng || (this._unmodified = !1, this._center = Ke, this._constrain(), this._calcMatrices());
                }, dl.padding.get = function() {
                  return this._edgeInsets.toJSON();
                }, dl.padding.set = function(Ke) {
                  this._edgeInsets.equals(Ke) || (this._unmodified = !1, this._edgeInsets.interpolate(this._edgeInsets, Ke, 1), this._calcMatrices());
                }, dl.centerPoint.get = function() {
                  return this._edgeInsets.getCenter(this.width, this.height);
                }, Ql.prototype.isPaddingEqual = function(Ke) {
                  return this._edgeInsets.equals(Ke);
                }, Ql.prototype.interpolatePadding = function(Ke, Fe, Te) {
                  this._unmodified = !1, this._edgeInsets.interpolate(Ke, Fe, Te), this._constrain(), this._calcMatrices();
                }, Ql.prototype.coveringZoomLevel = function(Ke) {
                  var Fe = (Ke.roundZoom ? Math.round : Math.floor)(this.zoom + this.scaleZoom(this.tileSize / Ke.tileSize));
                  return Math.max(0, Fe);
                }, Ql.prototype.getVisibleUnwrappedCoordinates = function(Ke) {
                  var Fe = [new ee.UnwrappedTileID(0, Ke)];
                  if (this._renderWorldCopies) for (var Te = this.pointCoordinate(new ee.Point(0, 0)), Ae = this.pointCoordinate(new ee.Point(this.width, 0)), Oe = this.pointCoordinate(new ee.Point(this.width, this.height)), Be = this.pointCoordinate(new ee.Point(0, this.height)), Ue = Math.floor(Math.min(Te.x, Ae.x, Oe.x, Be.x)), nt = Math.floor(Math.max(Te.x, Ae.x, Oe.x, Be.x)), ht = Ue - 1; ht <= nt + 1; ht++) ht !== 0 && Fe.push(new ee.UnwrappedTileID(ht, Ke));
                  return Fe;
                }, Ql.prototype.coveringTiles = function(Ke) {
                  var Fe = this.coveringZoomLevel(Ke), Te = Fe;
                  if (Ke.minzoom !== void 0 && Fe < Ke.minzoom) return [];
                  Ke.maxzoom !== void 0 && Fe > Ke.maxzoom && (Fe = Ke.maxzoom);
                  var Ae = ee.MercatorCoordinate.fromLngLat(this.center), Oe = Math.pow(2, Fe), Be = [Oe * Ae.x, Oe * Ae.y, 0], Ue = Oo.fromInvProjectionMatrix(this.invProjMatrix, this.worldSize, Fe), nt = Ke.minzoom || 0;
                  this.pitch <= 60 && this._edgeInsets.top < 0.1 && (nt = Fe);
                  var ht = function(Wi) {
                    return { aabb: new gs([Wi * Oe, 0, 0], [(Wi + 1) * Oe, Oe, 0]), zoom: 0, x: 0, y: 0, wrap: Wi, fullyVisible: !1 };
                  }, Et = [], Mt = [], qt = Fe, tn = Ke.reparseOverscaled ? Te : Fe;
                  if (this._renderWorldCopies) for (var xn = 1; xn <= 3; xn++) Et.push(ht(-xn)), Et.push(ht(xn));
                  for (Et.push(ht(0)); Et.length > 0; ) {
                    var An = Et.pop(), On = An.x, Bn = An.y, qn = An.fullyVisible;
                    if (!qn) {
                      var Vn = An.aabb.intersects(Ue);
                      if (Vn === 0) continue;
                      qn = Vn === 2;
                    }
                    var sr = An.aabb.distanceX(Be), Un = An.aabb.distanceY(Be), wr = Math.max(Math.abs(sr), Math.abs(Un)), Cr = 3 + (1 << qt - An.zoom) - 2;
                    if (An.zoom === qt || wr > Cr && An.zoom >= nt) Mt.push({ tileID: new ee.OverscaledTileID(An.zoom === qt ? tn : An.zoom, An.wrap, An.zoom, On, Bn), distanceSq: ee.sqrLen([Be[0] - 0.5 - On, Be[1] - 0.5 - Bn]) });
                    else for (var Tr = 0; Tr < 4; Tr++) {
                      var ii = (On << 1) + Tr % 2, wi = (Bn << 1) + (Tr >> 1);
                      Et.push({ aabb: An.aabb.quadrant(Tr), zoom: An.zoom + 1, x: ii, y: wi, wrap: An.wrap, fullyVisible: qn });
                    }
                  }
                  return Mt.sort(function(Wi, Yi) {
                    return Wi.distanceSq - Yi.distanceSq;
                  }).map(function(Wi) {
                    return Wi.tileID;
                  });
                }, Ql.prototype.resize = function(Ke, Fe) {
                  this.width = Ke, this.height = Fe, this.pixelsToGLUnits = [2 / Ke, -2 / Fe], this._constrain(), this._calcMatrices();
                }, dl.unmodified.get = function() {
                  return this._unmodified;
                }, Ql.prototype.zoomScale = function(Ke) {
                  return Math.pow(2, Ke);
                }, Ql.prototype.scaleZoom = function(Ke) {
                  return Math.log(Ke) / Math.LN2;
                }, Ql.prototype.project = function(Ke) {
                  var Fe = ee.clamp(Ke.lat, -this.maxValidLatitude, this.maxValidLatitude);
                  return new ee.Point(ee.mercatorXfromLng(Ke.lng) * this.worldSize, ee.mercatorYfromLat(Fe) * this.worldSize);
                }, Ql.prototype.unproject = function(Ke) {
                  return new ee.MercatorCoordinate(Ke.x / this.worldSize, Ke.y / this.worldSize).toLngLat();
                }, dl.point.get = function() {
                  return this.project(this.center);
                }, Ql.prototype.setLocationAtPoint = function(Ke, Fe) {
                  var Te = this.pointCoordinate(Fe), Ae = this.pointCoordinate(this.centerPoint), Oe = this.locationCoordinate(Ke), Be = new ee.MercatorCoordinate(Oe.x - (Te.x - Ae.x), Oe.y - (Te.y - Ae.y));
                  this.center = this.coordinateLocation(Be), this._renderWorldCopies && (this.center = this.center.wrap());
                }, Ql.prototype.locationPoint = function(Ke) {
                  return this.coordinatePoint(this.locationCoordinate(Ke));
                }, Ql.prototype.pointLocation = function(Ke) {
                  return this.coordinateLocation(this.pointCoordinate(Ke));
                }, Ql.prototype.locationCoordinate = function(Ke) {
                  return ee.MercatorCoordinate.fromLngLat(Ke);
                }, Ql.prototype.coordinateLocation = function(Ke) {
                  return Ke.toLngLat();
                }, Ql.prototype.pointCoordinate = function(Ke) {
                  var Fe = [Ke.x, Ke.y, 0, 1], Te = [Ke.x, Ke.y, 1, 1];
                  ee.transformMat4(Fe, Fe, this.pixelMatrixInverse), ee.transformMat4(Te, Te, this.pixelMatrixInverse);
                  var Ae = Fe[3], Oe = Te[3], Be = Fe[0] / Ae, Ue = Te[0] / Oe, nt = Fe[1] / Ae, ht = Te[1] / Oe, Et = Fe[2] / Ae, Mt = Te[2] / Oe, qt = Et === Mt ? 0 : (0 - Et) / (Mt - Et);
                  return new ee.MercatorCoordinate(ee.number(Be, Ue, qt) / this.worldSize, ee.number(nt, ht, qt) / this.worldSize);
                }, Ql.prototype.coordinatePoint = function(Ke) {
                  var Fe = [Ke.x * this.worldSize, Ke.y * this.worldSize, 0, 1];
                  return ee.transformMat4(Fe, Fe, this.pixelMatrix), new ee.Point(Fe[0] / Fe[3], Fe[1] / Fe[3]);
                }, Ql.prototype.getBounds = function() {
                  return new ee.LngLatBounds().extend(this.pointLocation(new ee.Point(0, 0))).extend(this.pointLocation(new ee.Point(this.width, 0))).extend(this.pointLocation(new ee.Point(this.width, this.height))).extend(this.pointLocation(new ee.Point(0, this.height)));
                }, Ql.prototype.getMaxBounds = function() {
                  return this.latRange && this.latRange.length === 2 && this.lngRange && this.lngRange.length === 2 ? new ee.LngLatBounds([this.lngRange[0], this.latRange[0]], [this.lngRange[1], this.latRange[1]]) : null;
                }, Ql.prototype.setMaxBounds = function(Ke) {
                  Ke ? (this.lngRange = [Ke.getWest(), Ke.getEast()], this.latRange = [Ke.getSouth(), Ke.getNorth()], this._constrain()) : (this.lngRange = null, this.latRange = [-this.maxValidLatitude, this.maxValidLatitude]);
                }, Ql.prototype.calculatePosMatrix = function(Ke, Fe) {
                  Fe === void 0 && (Fe = !1);
                  var Te = Ke.key, Ae = Fe ? this._alignedPosMatrixCache : this._posMatrixCache;
                  if (Ae[Te]) return Ae[Te];
                  var Oe = Ke.canonical, Be = this.worldSize / this.zoomScale(Oe.z), Ue = Oe.x + Math.pow(2, Oe.z) * Ke.wrap, nt = ee.identity(new Float64Array(16));
                  return ee.translate(nt, nt, [Ue * Be, Oe.y * Be, 0]), ee.scale(nt, nt, [Be / ee.EXTENT, Be / ee.EXTENT, 1]), ee.multiply(nt, Fe ? this.alignedProjMatrix : this.projMatrix, nt), Ae[Te] = new Float32Array(nt), Ae[Te];
                }, Ql.prototype.customLayerMatrix = function() {
                  return this.mercatorMatrix.slice();
                }, Ql.prototype._constrain = function() {
                  if (this.center && this.width && this.height && !this._constraining) {
                    this._constraining = !0;
                    var Ke, Fe, Te, Ae, Oe = -90, Be = 90, Ue = -180, nt = 180, ht = this.size, Et = this._unmodified;
                    if (this.latRange) {
                      var Mt = this.latRange;
                      Oe = ee.mercatorYfromLat(Mt[1]) * this.worldSize, Ke = (Be = ee.mercatorYfromLat(Mt[0]) * this.worldSize) - Oe < ht.y ? ht.y / (Be - Oe) : 0;
                    }
                    if (this.lngRange) {
                      var qt = this.lngRange;
                      Ue = ee.mercatorXfromLng(qt[0]) * this.worldSize, Fe = (nt = ee.mercatorXfromLng(qt[1]) * this.worldSize) - Ue < ht.x ? ht.x / (nt - Ue) : 0;
                    }
                    var tn = this.point, xn = Math.max(Fe || 0, Ke || 0);
                    if (xn) return this.center = this.unproject(new ee.Point(Fe ? (nt + Ue) / 2 : tn.x, Ke ? (Be + Oe) / 2 : tn.y)), this.zoom += this.scaleZoom(xn), this._unmodified = Et, void (this._constraining = !1);
                    if (this.latRange) {
                      var An = tn.y, On = ht.y / 2;
                      An - On < Oe && (Ae = Oe + On), An + On > Be && (Ae = Be - On);
                    }
                    if (this.lngRange) {
                      var Bn = tn.x, qn = ht.x / 2;
                      Bn - qn < Ue && (Te = Ue + qn), Bn + qn > nt && (Te = nt - qn);
                    }
                    Te === void 0 && Ae === void 0 || (this.center = this.unproject(new ee.Point(Te !== void 0 ? Te : tn.x, Ae !== void 0 ? Ae : tn.y))), this._unmodified = Et, this._constraining = !1;
                  }
                }, Ql.prototype._calcMatrices = function() {
                  if (this.height) {
                    var Ke = this._fov / 2, Fe = this.centerOffset;
                    this.cameraToCenterDistance = 0.5 / Math.tan(Ke) * this.height;
                    var Te = Math.PI / 2 + this._pitch, Ae = this._fov * (0.5 + Fe.y / this.height), Oe = Math.sin(Ae) * this.cameraToCenterDistance / Math.sin(ee.clamp(Math.PI - Te - Ae, 0.01, Math.PI - 0.01)), Be = this.point, Ue = Be.x, nt = Be.y, ht = 1.01 * (Math.cos(Math.PI / 2 - this._pitch) * Oe + this.cameraToCenterDistance), Et = this.height / 50, Mt = new Float64Array(16);
                    ee.perspective(Mt, this._fov, this.width / this.height, Et, ht), Mt[8] = 2 * -Fe.x / this.width, Mt[9] = 2 * Fe.y / this.height, ee.scale(Mt, Mt, [1, -1, 1]), ee.translate(Mt, Mt, [0, 0, -this.cameraToCenterDistance]), ee.rotateX(Mt, Mt, this._pitch), ee.rotateZ(Mt, Mt, this.angle), ee.translate(Mt, Mt, [-Ue, -nt, 0]), this.mercatorMatrix = ee.scale([], Mt, [this.worldSize, this.worldSize, this.worldSize]), ee.scale(Mt, Mt, [1, 1, ee.mercatorZfromAltitude(1, this.center.lat) * this.worldSize, 1]), this.projMatrix = Mt, this.invProjMatrix = ee.invert([], this.projMatrix);
                    var qt = this.width % 2 / 2, tn = this.height % 2 / 2, xn = Math.cos(this.angle), An = Math.sin(this.angle), On = Ue - Math.round(Ue) + xn * qt + An * tn, Bn = nt - Math.round(nt) + xn * tn + An * qt, qn = new Float64Array(Mt);
                    if (ee.translate(qn, qn, [On > 0.5 ? On - 1 : On, Bn > 0.5 ? Bn - 1 : Bn, 0]), this.alignedProjMatrix = qn, Mt = ee.create(), ee.scale(Mt, Mt, [this.width / 2, -this.height / 2, 1]), ee.translate(Mt, Mt, [1, -1, 0]), this.labelPlaneMatrix = Mt, Mt = ee.create(), ee.scale(Mt, Mt, [1, -1, 1]), ee.translate(Mt, Mt, [-1, -1, 0]), ee.scale(Mt, Mt, [2 / this.width, 2 / this.height, 1]), this.glCoordMatrix = Mt, this.pixelMatrix = ee.multiply(new Float64Array(16), this.labelPlaneMatrix, this.projMatrix), !(Mt = ee.invert(new Float64Array(16), this.pixelMatrix))) throw new Error("failed to invert matrix");
                    this.pixelMatrixInverse = Mt, this._posMatrixCache = {}, this._alignedPosMatrixCache = {};
                  }
                }, Ql.prototype.maxPitchScaleFactor = function() {
                  if (!this.pixelMatrixInverse) return 1;
                  var Ke = this.pointCoordinate(new ee.Point(0, 0)), Fe = [Ke.x * this.worldSize, Ke.y * this.worldSize, 0, 1];
                  return ee.transformMat4(Fe, Fe, this.pixelMatrix)[3] / this.cameraToCenterDistance;
                }, Ql.prototype.getCameraPoint = function() {
                  var Ke = this._pitch, Fe = Math.tan(Ke) * (this.cameraToCenterDistance || 1);
                  return this.centerPoint.add(new ee.Point(0, Fe));
                }, Ql.prototype.getCameraQueryGeometry = function(Ke) {
                  var Fe = this.getCameraPoint();
                  if (Ke.length === 1) return [Ke[0], Fe];
                  for (var Te = Fe.x, Ae = Fe.y, Oe = Fe.x, Be = Fe.y, Ue = 0, nt = Ke; Ue < nt.length; Ue += 1) {
                    var ht = nt[Ue];
                    Te = Math.min(Te, ht.x), Ae = Math.min(Ae, ht.y), Oe = Math.max(Oe, ht.x), Be = Math.max(Be, ht.y);
                  }
                  return [new ee.Point(Te, Ae), new ee.Point(Oe, Ae), new ee.Point(Oe, Be), new ee.Point(Te, Be), new ee.Point(Te, Ae)];
                }, Object.defineProperties(Ql.prototype, dl);
                var h1 = function(Ke) {
                  var Fe, Te, Ae, Oe, Be;
                  this._hashName = Ke && encodeURIComponent(Ke), ee.bindAll(["_getCurrentHash", "_onHashChange", "_updateHash"], this), this._updateHash = (Fe = this._updateHashUnthrottled.bind(this), Te = 300, Ae = !1, Oe = null, Be = function() {
                    Oe = null, Ae && (Fe(), Oe = setTimeout(Be, Te), Ae = !1);
                  }, function() {
                    return Ae = !0, Oe || Be(), Oe;
                  });
                };
                h1.prototype.addTo = function(Ke) {
                  return this._map = Ke, ee.window.addEventListener("hashchange", this._onHashChange, !1), this._map.on("moveend", this._updateHash), this;
                }, h1.prototype.remove = function() {
                  return ee.window.removeEventListener("hashchange", this._onHashChange, !1), this._map.off("moveend", this._updateHash), clearTimeout(this._updateHash()), delete this._map, this;
                }, h1.prototype.getHashString = function(Ke) {
                  var Fe = this._map.getCenter(), Te = Math.round(100 * this._map.getZoom()) / 100, Ae = Math.ceil((Te * Math.LN2 + Math.log(512 / 360 / 0.5)) / Math.LN10), Oe = Math.pow(10, Ae), Be = Math.round(Fe.lng * Oe) / Oe, Ue = Math.round(Fe.lat * Oe) / Oe, nt = this._map.getBearing(), ht = this._map.getPitch(), Et = "";
                  if (Et += Ke ? "/" + Be + "/" + Ue + "/" + Te : Te + "/" + Ue + "/" + Be, (nt || ht) && (Et += "/" + Math.round(10 * nt) / 10), ht && (Et += "/" + Math.round(ht)), this._hashName) {
                    var Mt = this._hashName, qt = !1, tn = ee.window.location.hash.slice(1).split("&").map(function(xn) {
                      var An = xn.split("=")[0];
                      return An === Mt ? (qt = !0, An + "=" + Et) : xn;
                    }).filter(function(xn) {
                      return xn;
                    });
                    return qt || tn.push(Mt + "=" + Et), "#" + tn.join("&");
                  }
                  return "#" + Et;
                }, h1.prototype._getCurrentHash = function() {
                  var Ke, Fe = this, Te = ee.window.location.hash.replace("#", "");
                  return this._hashName ? (Te.split("&").map(function(Ae) {
                    return Ae.split("=");
                  }).forEach(function(Ae) {
                    Ae[0] === Fe._hashName && (Ke = Ae);
                  }), (Ke && Ke[1] || "").split("/")) : Te.split("/");
                }, h1.prototype._onHashChange = function() {
                  var Ke = this._getCurrentHash();
                  if (Ke.length >= 3 && !Ke.some(function(Te) {
                    return isNaN(Te);
                  })) {
                    var Fe = this._map.dragRotate.isEnabled() && this._map.touchZoomRotate.isEnabled() ? +(Ke[3] || 0) : this._map.getBearing();
                    return this._map.jumpTo({ center: [+Ke[2], +Ke[1]], zoom: +Ke[0], bearing: Fe, pitch: +(Ke[4] || 0) }), !0;
                  }
                  return !1;
                }, h1.prototype._updateHashUnthrottled = function() {
                  var Ke = ee.window.location.href.replace(/(#.+)?$/, this.getHashString());
                  try {
                    ee.window.history.replaceState(ee.window.history.state, null, Ke);
                  } catch {
                  }
                };
                var z1 = { linearity: 0.3, easing: ee.bezier(0, 0, 0.3, 1) }, J1 = ee.extend({ deceleration: 2500, maxSpeed: 1400 }, z1), Lu = ee.extend({ deceleration: 20, maxSpeed: 1400 }, z1), M1 = ee.extend({ deceleration: 1e3, maxSpeed: 360 }, z1), ku = ee.extend({ deceleration: 1e3, maxSpeed: 90 }, z1), Wu = function(Ke) {
                  this._map = Ke, this.clear();
                };
                function T1(Ke, Fe) {
                  (!Ke.duration || Ke.duration < Fe.duration) && (Ke.duration = Fe.duration, Ke.easing = Fe.easing);
                }
                function kl(Ke, Fe, Te) {
                  var Ae = Te.maxSpeed, Oe = Te.linearity, Be = Te.deceleration, Ue = ee.clamp(Ke * Oe / (Fe / 1e3), -Ae, Ae), nt = Math.abs(Ue) / (Be * Oe);
                  return { easing: Te.easing, duration: 1e3 * nt, amount: Ue * (nt / 2) };
                }
                Wu.prototype.clear = function() {
                  this._inertiaBuffer = [];
                }, Wu.prototype.record = function(Ke) {
                  this._drainInertiaBuffer(), this._inertiaBuffer.push({ time: ee.browser.now(), settings: Ke });
                }, Wu.prototype._drainInertiaBuffer = function() {
                  for (var Ke = this._inertiaBuffer, Fe = ee.browser.now(); Ke.length > 0 && Fe - Ke[0].time > 160; ) Ke.shift();
                }, Wu.prototype._onMoveEnd = function(Ke) {
                  if (this._drainInertiaBuffer(), !(this._inertiaBuffer.length < 2)) {
                    for (var Fe = { zoom: 0, bearing: 0, pitch: 0, pan: new ee.Point(0, 0), pinchAround: void 0, around: void 0 }, Te = 0, Ae = this._inertiaBuffer; Te < Ae.length; Te += 1) {
                      var Oe = Ae[Te].settings;
                      Fe.zoom += Oe.zoomDelta || 0, Fe.bearing += Oe.bearingDelta || 0, Fe.pitch += Oe.pitchDelta || 0, Oe.panDelta && Fe.pan._add(Oe.panDelta), Oe.around && (Fe.around = Oe.around), Oe.pinchAround && (Fe.pinchAround = Oe.pinchAround);
                    }
                    var Be = this._inertiaBuffer[this._inertiaBuffer.length - 1].time - this._inertiaBuffer[0].time, Ue = {};
                    if (Fe.pan.mag()) {
                      var nt = kl(Fe.pan.mag(), Be, ee.extend({}, J1, Ke || {}));
                      Ue.offset = Fe.pan.mult(nt.amount / Fe.pan.mag()), Ue.center = this._map.transform.center, T1(Ue, nt);
                    }
                    if (Fe.zoom) {
                      var ht = kl(Fe.zoom, Be, Lu);
                      Ue.zoom = this._map.transform.zoom + ht.amount, T1(Ue, ht);
                    }
                    if (Fe.bearing) {
                      var Et = kl(Fe.bearing, Be, M1);
                      Ue.bearing = this._map.transform.bearing + ee.clamp(Et.amount, -179, 179), T1(Ue, Et);
                    }
                    if (Fe.pitch) {
                      var Mt = kl(Fe.pitch, Be, ku);
                      Ue.pitch = this._map.transform.pitch + Mt.amount, T1(Ue, Mt);
                    }
                    if (Ue.zoom || Ue.bearing) {
                      var qt = Fe.pinchAround === void 0 ? Fe.around : Fe.pinchAround;
                      Ue.around = qt ? this._map.unproject(qt) : this._map.getCenter();
                    }
                    return this.clear(), ee.extend(Ue, { noMoveStart: !0 });
                  }
                };
                var Yl = function(Ke) {
                  function Fe(Ae, Oe, Be, Ue) {
                    Ue === void 0 && (Ue = {});
                    var nt = oe.mousePos(Oe.getCanvasContainer(), Be), ht = Oe.unproject(nt);
                    Ke.call(this, Ae, ee.extend({ point: nt, lngLat: ht, originalEvent: Be }, Ue)), this._defaultPrevented = !1, this.target = Oe;
                  }
                  Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe;
                  var Te = { defaultPrevented: { configurable: !0 } };
                  return Fe.prototype.preventDefault = function() {
                    this._defaultPrevented = !0;
                  }, Te.defaultPrevented.get = function() {
                    return this._defaultPrevented;
                  }, Object.defineProperties(Fe.prototype, Te), Fe;
                }(ee.Event), i1 = function(Ke) {
                  function Fe(Ae, Oe, Be) {
                    var Ue = Ae === "touchend" ? Be.changedTouches : Be.touches, nt = oe.touchPos(Oe.getCanvasContainer(), Ue), ht = nt.map(function(qt) {
                      return Oe.unproject(qt);
                    }), Et = nt.reduce(function(qt, tn, xn, An) {
                      return qt.add(tn.div(An.length));
                    }, new ee.Point(0, 0)), Mt = Oe.unproject(Et);
                    Ke.call(this, Ae, { points: nt, point: Et, lngLats: ht, lngLat: Mt, originalEvent: Be }), this._defaultPrevented = !1;
                  }
                  Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe;
                  var Te = { defaultPrevented: { configurable: !0 } };
                  return Fe.prototype.preventDefault = function() {
                    this._defaultPrevented = !0;
                  }, Te.defaultPrevented.get = function() {
                    return this._defaultPrevented;
                  }, Object.defineProperties(Fe.prototype, Te), Fe;
                }(ee.Event), ic = function(Ke) {
                  function Fe(Ae, Oe, Be) {
                    Ke.call(this, Ae, { originalEvent: Be }), this._defaultPrevented = !1;
                  }
                  Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe;
                  var Te = { defaultPrevented: { configurable: !0 } };
                  return Fe.prototype.preventDefault = function() {
                    this._defaultPrevented = !0;
                  }, Te.defaultPrevented.get = function() {
                    return this._defaultPrevented;
                  }, Object.defineProperties(Fe.prototype, Te), Fe;
                }(ee.Event), iu = function(Ke, Fe) {
                  this._map = Ke, this._clickTolerance = Fe.clickTolerance;
                };
                iu.prototype.reset = function() {
                  delete this._mousedownPos;
                }, iu.prototype.wheel = function(Ke) {
                  return this._firePreventable(new ic(Ke.type, this._map, Ke));
                }, iu.prototype.mousedown = function(Ke, Fe) {
                  return this._mousedownPos = Fe, this._firePreventable(new Yl(Ke.type, this._map, Ke));
                }, iu.prototype.mouseup = function(Ke) {
                  this._map.fire(new Yl(Ke.type, this._map, Ke));
                }, iu.prototype.click = function(Ke, Fe) {
                  this._mousedownPos && this._mousedownPos.dist(Fe) >= this._clickTolerance || this._map.fire(new Yl(Ke.type, this._map, Ke));
                }, iu.prototype.dblclick = function(Ke) {
                  return this._firePreventable(new Yl(Ke.type, this._map, Ke));
                }, iu.prototype.mouseover = function(Ke) {
                  this._map.fire(new Yl(Ke.type, this._map, Ke));
                }, iu.prototype.mouseout = function(Ke) {
                  this._map.fire(new Yl(Ke.type, this._map, Ke));
                }, iu.prototype.touchstart = function(Ke) {
                  return this._firePreventable(new i1(Ke.type, this._map, Ke));
                }, iu.prototype.touchmove = function(Ke) {
                  this._map.fire(new i1(Ke.type, this._map, Ke));
                }, iu.prototype.touchend = function(Ke) {
                  this._map.fire(new i1(Ke.type, this._map, Ke));
                }, iu.prototype.touchcancel = function(Ke) {
                  this._map.fire(new i1(Ke.type, this._map, Ke));
                }, iu.prototype._firePreventable = function(Ke) {
                  if (this._map.fire(Ke), Ke.defaultPrevented) return {};
                }, iu.prototype.isEnabled = function() {
                  return !0;
                }, iu.prototype.isActive = function() {
                  return !1;
                }, iu.prototype.enable = function() {
                }, iu.prototype.disable = function() {
                };
                var xu = function(Ke) {
                  this._map = Ke;
                };
                xu.prototype.reset = function() {
                  this._delayContextMenu = !1, delete this._contextMenuEvent;
                }, xu.prototype.mousemove = function(Ke) {
                  this._map.fire(new Yl(Ke.type, this._map, Ke));
                }, xu.prototype.mousedown = function() {
                  this._delayContextMenu = !0;
                }, xu.prototype.mouseup = function() {
                  this._delayContextMenu = !1, this._contextMenuEvent && (this._map.fire(new Yl("contextmenu", this._map, this._contextMenuEvent)), delete this._contextMenuEvent);
                }, xu.prototype.contextmenu = function(Ke) {
                  this._delayContextMenu ? this._contextMenuEvent = Ke : this._map.fire(new Yl(Ke.type, this._map, Ke)), this._map.listens("contextmenu") && Ke.preventDefault();
                }, xu.prototype.isEnabled = function() {
                  return !0;
                }, xu.prototype.isActive = function() {
                  return !1;
                }, xu.prototype.enable = function() {
                }, xu.prototype.disable = function() {
                };
                var vc = function(Ke, Fe) {
                  this._map = Ke, this._el = Ke.getCanvasContainer(), this._container = Ke.getContainer(), this._clickTolerance = Fe.clickTolerance || 1;
                };
                function Zl(Ke, Fe) {
                  for (var Te = {}, Ae = 0; Ae < Ke.length; Ae++) Te[Ke[Ae].identifier] = Fe[Ae];
                  return Te;
                }
                vc.prototype.isEnabled = function() {
                  return !!this._enabled;
                }, vc.prototype.isActive = function() {
                  return !!this._active;
                }, vc.prototype.enable = function() {
                  this.isEnabled() || (this._enabled = !0);
                }, vc.prototype.disable = function() {
                  this.isEnabled() && (this._enabled = !1);
                }, vc.prototype.mousedown = function(Ke, Fe) {
                  this.isEnabled() && Ke.shiftKey && Ke.button === 0 && (oe.disableDrag(), this._startPos = this._lastPos = Fe, this._active = !0);
                }, vc.prototype.mousemoveWindow = function(Ke, Fe) {
                  if (this._active) {
                    var Te = Fe;
                    if (!(this._lastPos.equals(Te) || !this._box && Te.dist(this._startPos) < this._clickTolerance)) {
                      var Ae = this._startPos;
                      this._lastPos = Te, this._box || (this._box = oe.create("div", "mapboxgl-boxzoom", this._container), this._container.classList.add("mapboxgl-crosshair"), this._fireEvent("boxzoomstart", Ke));
                      var Oe = Math.min(Ae.x, Te.x), Be = Math.max(Ae.x, Te.x), Ue = Math.min(Ae.y, Te.y), nt = Math.max(Ae.y, Te.y);
                      oe.setTransform(this._box, "translate(" + Oe + "px," + Ue + "px)"), this._box.style.width = Be - Oe + "px", this._box.style.height = nt - Ue + "px";
                    }
                  }
                }, vc.prototype.mouseupWindow = function(Ke, Fe) {
                  var Te = this;
                  if (this._active && Ke.button === 0) {
                    var Ae = this._startPos, Oe = Fe;
                    if (this.reset(), oe.suppressClick(), Ae.x !== Oe.x || Ae.y !== Oe.y) return this._map.fire(new ee.Event("boxzoomend", { originalEvent: Ke })), { cameraAnimation: function(Be) {
                      return Be.fitScreenCoordinates(Ae, Oe, Te._map.getBearing(), { linear: !0 });
                    } };
                    this._fireEvent("boxzoomcancel", Ke);
                  }
                }, vc.prototype.keydown = function(Ke) {
                  this._active && Ke.keyCode === 27 && (this.reset(), this._fireEvent("boxzoomcancel", Ke));
                }, vc.prototype.reset = function() {
                  this._active = !1, this._container.classList.remove("mapboxgl-crosshair"), this._box && (oe.remove(this._box), this._box = null), oe.enableDrag(), delete this._startPos, delete this._lastPos;
                }, vc.prototype._fireEvent = function(Ke, Fe) {
                  return this._map.fire(new ee.Event(Ke, { originalEvent: Fe }));
                };
                var Ec = function(Ke) {
                  this.reset(), this.numTouches = Ke.numTouches;
                };
                Ec.prototype.reset = function() {
                  delete this.centroid, delete this.startTime, delete this.touches, this.aborted = !1;
                }, Ec.prototype.touchstart = function(Ke, Fe, Te) {
                  (this.centroid || Te.length > this.numTouches) && (this.aborted = !0), this.aborted || (this.startTime === void 0 && (this.startTime = Ke.timeStamp), Te.length === this.numTouches && (this.centroid = function(Ae) {
                    for (var Oe = new ee.Point(0, 0), Be = 0, Ue = Ae; Be < Ue.length; Be += 1) {
                      var nt = Ue[Be];
                      Oe._add(nt);
                    }
                    return Oe.div(Ae.length);
                  }(Fe), this.touches = Zl(Te, Fe)));
                }, Ec.prototype.touchmove = function(Ke, Fe, Te) {
                  if (!this.aborted && this.centroid) {
                    var Ae = Zl(Te, Fe);
                    for (var Oe in this.touches) {
                      var Be = this.touches[Oe], Ue = Ae[Oe];
                      (!Ue || Ue.dist(Be) > 30) && (this.aborted = !0);
                    }
                  }
                }, Ec.prototype.touchend = function(Ke, Fe, Te) {
                  if ((!this.centroid || Ke.timeStamp - this.startTime > 500) && (this.aborted = !0), Te.length === 0) {
                    var Ae = !this.aborted && this.centroid;
                    if (this.reset(), Ae) return Ae;
                  }
                };
                var wu = function(Ke) {
                  this.singleTap = new Ec(Ke), this.numTaps = Ke.numTaps, this.reset();
                };
                wu.prototype.reset = function() {
                  this.lastTime = 1 / 0, delete this.lastTap, this.count = 0, this.singleTap.reset();
                }, wu.prototype.touchstart = function(Ke, Fe, Te) {
                  this.singleTap.touchstart(Ke, Fe, Te);
                }, wu.prototype.touchmove = function(Ke, Fe, Te) {
                  this.singleTap.touchmove(Ke, Fe, Te);
                }, wu.prototype.touchend = function(Ke, Fe, Te) {
                  var Ae = this.singleTap.touchend(Ke, Fe, Te);
                  if (Ae) {
                    var Oe = Ke.timeStamp - this.lastTime < 500, Be = !this.lastTap || this.lastTap.dist(Ae) < 30;
                    if (Oe && Be || this.reset(), this.count++, this.lastTime = Ke.timeStamp, this.lastTap = Ae, this.count === this.numTaps) return this.reset(), Ae;
                  }
                };
                var Pc = function() {
                  this._zoomIn = new wu({ numTouches: 1, numTaps: 2 }), this._zoomOut = new wu({ numTouches: 2, numTaps: 1 }), this.reset();
                };
                Pc.prototype.reset = function() {
                  this._active = !1, this._zoomIn.reset(), this._zoomOut.reset();
                }, Pc.prototype.touchstart = function(Ke, Fe, Te) {
                  this._zoomIn.touchstart(Ke, Fe, Te), this._zoomOut.touchstart(Ke, Fe, Te);
                }, Pc.prototype.touchmove = function(Ke, Fe, Te) {
                  this._zoomIn.touchmove(Ke, Fe, Te), this._zoomOut.touchmove(Ke, Fe, Te);
                }, Pc.prototype.touchend = function(Ke, Fe, Te) {
                  var Ae = this, Oe = this._zoomIn.touchend(Ke, Fe, Te), Be = this._zoomOut.touchend(Ke, Fe, Te);
                  return Oe ? (this._active = !0, Ke.preventDefault(), setTimeout(function() {
                    return Ae.reset();
                  }, 0), { cameraAnimation: function(Ue) {
                    return Ue.easeTo({ duration: 300, zoom: Ue.getZoom() + 1, around: Ue.unproject(Oe) }, { originalEvent: Ke });
                  } }) : Be ? (this._active = !0, Ke.preventDefault(), setTimeout(function() {
                    return Ae.reset();
                  }, 0), { cameraAnimation: function(Ue) {
                    return Ue.easeTo({ duration: 300, zoom: Ue.getZoom() - 1, around: Ue.unproject(Be) }, { originalEvent: Ke });
                  } }) : void 0;
                }, Pc.prototype.touchcancel = function() {
                  this.reset();
                }, Pc.prototype.enable = function() {
                  this._enabled = !0;
                }, Pc.prototype.disable = function() {
                  this._enabled = !1, this.reset();
                }, Pc.prototype.isEnabled = function() {
                  return this._enabled;
                }, Pc.prototype.isActive = function() {
                  return this._active;
                };
                var du = {};
                du[0] = 1, du[2] = 2;
                var Ac = function(Ke) {
                  this.reset(), this._clickTolerance = Ke.clickTolerance || 1;
                };
                Ac.prototype.reset = function() {
                  this._active = !1, this._moved = !1, delete this._lastPoint, delete this._eventButton;
                }, Ac.prototype._correctButton = function(Ke, Fe) {
                  return !1;
                }, Ac.prototype._move = function(Ke, Fe) {
                  return {};
                }, Ac.prototype.mousedown = function(Ke, Fe) {
                  if (!this._lastPoint) {
                    var Te = oe.mouseButton(Ke);
                    this._correctButton(Ke, Te) && (this._lastPoint = Fe, this._eventButton = Te);
                  }
                }, Ac.prototype.mousemoveWindow = function(Ke, Fe) {
                  var Te = this._lastPoint;
                  if (Te) {
                    if (Ke.preventDefault(), function(Ae, Oe) {
                      var Be = du[Oe];
                      return Ae.buttons === void 0 || (Ae.buttons & Be) !== Be;
                    }(Ke, this._eventButton)) this.reset();
                    else if (this._moved || !(Fe.dist(Te) < this._clickTolerance)) return this._moved = !0, this._lastPoint = Fe, this._move(Te, Fe);
                  }
                }, Ac.prototype.mouseupWindow = function(Ke) {
                  this._lastPoint && oe.mouseButton(Ke) === this._eventButton && (this._moved && oe.suppressClick(), this.reset());
                }, Ac.prototype.enable = function() {
                  this._enabled = !0;
                }, Ac.prototype.disable = function() {
                  this._enabled = !1, this.reset();
                }, Ac.prototype.isEnabled = function() {
                  return this._enabled;
                }, Ac.prototype.isActive = function() {
                  return this._active;
                };
                var yu = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.mousedown = function(Te, Ae) {
                    Ke.prototype.mousedown.call(this, Te, Ae), this._lastPoint && (this._active = !0);
                  }, Fe.prototype._correctButton = function(Te, Ae) {
                    return Ae === 0 && !Te.ctrlKey;
                  }, Fe.prototype._move = function(Te, Ae) {
                    return { around: Ae, panDelta: Ae.sub(Te) };
                  }, Fe;
                }(Ac), au = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype._correctButton = function(Te, Ae) {
                    return Ae === 0 && Te.ctrlKey || Ae === 2;
                  }, Fe.prototype._move = function(Te, Ae) {
                    var Oe = 0.8 * (Ae.x - Te.x);
                    if (Oe) return this._active = !0, { bearingDelta: Oe };
                  }, Fe.prototype.contextmenu = function(Te) {
                    Te.preventDefault();
                  }, Fe;
                }(Ac), Qc = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype._correctButton = function(Te, Ae) {
                    return Ae === 0 && Te.ctrlKey || Ae === 2;
                  }, Fe.prototype._move = function(Te, Ae) {
                    var Oe = -0.5 * (Ae.y - Te.y);
                    if (Oe) return this._active = !0, { pitchDelta: Oe };
                  }, Fe.prototype.contextmenu = function(Te) {
                    Te.preventDefault();
                  }, Fe;
                }(Ac), Cc = function(Ke) {
                  this._minTouches = 1, this._clickTolerance = Ke.clickTolerance || 1, this.reset();
                };
                Cc.prototype.reset = function() {
                  this._active = !1, this._touches = {}, this._sum = new ee.Point(0, 0);
                }, Cc.prototype.touchstart = function(Ke, Fe, Te) {
                  return this._calculateTransform(Ke, Fe, Te);
                }, Cc.prototype.touchmove = function(Ke, Fe, Te) {
                  if (this._active && !(Te.length < this._minTouches)) return Ke.preventDefault(), this._calculateTransform(Ke, Fe, Te);
                }, Cc.prototype.touchend = function(Ke, Fe, Te) {
                  this._calculateTransform(Ke, Fe, Te), this._active && Te.length < this._minTouches && this.reset();
                }, Cc.prototype.touchcancel = function() {
                  this.reset();
                }, Cc.prototype._calculateTransform = function(Ke, Fe, Te) {
                  Te.length > 0 && (this._active = !0);
                  var Ae = Zl(Te, Fe), Oe = new ee.Point(0, 0), Be = new ee.Point(0, 0), Ue = 0;
                  for (var nt in Ae) {
                    var ht = Ae[nt], Et = this._touches[nt];
                    Et && (Oe._add(ht), Be._add(ht.sub(Et)), Ue++, Ae[nt] = ht);
                  }
                  if (this._touches = Ae, !(Ue < this._minTouches) && Be.mag()) {
                    var Mt = Be.div(Ue);
                    if (this._sum._add(Mt), !(this._sum.mag() < this._clickTolerance)) return { around: Oe.div(Ue), panDelta: Mt };
                  }
                }, Cc.prototype.enable = function() {
                  this._enabled = !0;
                }, Cc.prototype.disable = function() {
                  this._enabled = !1, this.reset();
                }, Cc.prototype.isEnabled = function() {
                  return this._enabled;
                }, Cc.prototype.isActive = function() {
                  return this._active;
                };
                var Ol = function() {
                  this.reset();
                };
                function uc(Ke, Fe, Te) {
                  for (var Ae = 0; Ae < Ke.length; Ae++) if (Ke[Ae].identifier === Te) return Fe[Ae];
                }
                Ol.prototype.reset = function() {
                  this._active = !1, delete this._firstTwoTouches;
                }, Ol.prototype._start = function(Ke) {
                }, Ol.prototype._move = function(Ke, Fe, Te) {
                  return {};
                }, Ol.prototype.touchstart = function(Ke, Fe, Te) {
                  this._firstTwoTouches || Te.length < 2 || (this._firstTwoTouches = [Te[0].identifier, Te[1].identifier], this._start([Fe[0], Fe[1]]));
                }, Ol.prototype.touchmove = function(Ke, Fe, Te) {
                  if (this._firstTwoTouches) {
                    Ke.preventDefault();
                    var Ae = this._firstTwoTouches, Oe = Ae[0], Be = Ae[1], Ue = uc(Te, Fe, Oe), nt = uc(Te, Fe, Be);
                    if (Ue && nt) {
                      var ht = this._aroundCenter ? null : Ue.add(nt).div(2);
                      return this._move([Ue, nt], ht, Ke);
                    }
                  }
                }, Ol.prototype.touchend = function(Ke, Fe, Te) {
                  if (this._firstTwoTouches) {
                    var Ae = this._firstTwoTouches, Oe = Ae[0], Be = Ae[1], Ue = uc(Te, Fe, Oe), nt = uc(Te, Fe, Be);
                    Ue && nt || (this._active && oe.suppressClick(), this.reset());
                  }
                }, Ol.prototype.touchcancel = function() {
                  this.reset();
                }, Ol.prototype.enable = function(Ke) {
                  this._enabled = !0, this._aroundCenter = !!Ke && Ke.around === "center";
                }, Ol.prototype.disable = function() {
                  this._enabled = !1, this.reset();
                }, Ol.prototype.isEnabled = function() {
                  return this._enabled;
                }, Ol.prototype.isActive = function() {
                  return this._active;
                };
                function cs(Ke, Fe) {
                  return Math.log(Ke / Fe) / Math.LN2;
                }
                var Mu = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.reset = function() {
                    Ke.prototype.reset.call(this), delete this._distance, delete this._startDistance;
                  }, Fe.prototype._start = function(Te) {
                    this._startDistance = this._distance = Te[0].dist(Te[1]);
                  }, Fe.prototype._move = function(Te, Ae) {
                    var Oe = this._distance;
                    if (this._distance = Te[0].dist(Te[1]), this._active || !(Math.abs(cs(this._distance, this._startDistance)) < 0.1)) return this._active = !0, { zoomDelta: cs(this._distance, Oe), pinchAround: Ae };
                  }, Fe;
                }(Ol);
                function Hu(Ke, Fe) {
                  return 180 * Ke.angleWith(Fe) / Math.PI;
                }
                var ou = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.reset = function() {
                    Ke.prototype.reset.call(this), delete this._minDiameter, delete this._startVector, delete this._vector;
                  }, Fe.prototype._start = function(Te) {
                    this._startVector = this._vector = Te[0].sub(Te[1]), this._minDiameter = Te[0].dist(Te[1]);
                  }, Fe.prototype._move = function(Te, Ae) {
                    var Oe = this._vector;
                    if (this._vector = Te[0].sub(Te[1]), this._active || !this._isBelowThreshold(this._vector)) return this._active = !0, { bearingDelta: Hu(this._vector, Oe), pinchAround: Ae };
                  }, Fe.prototype._isBelowThreshold = function(Te) {
                    this._minDiameter = Math.min(this._minDiameter, Te.mag());
                    var Ae = 25 / (Math.PI * this._minDiameter) * 360, Oe = Hu(Te, this._startVector);
                    return Math.abs(Oe) < Ae;
                  }, Fe;
                }(Ol);
                function Gc(Ke) {
                  return Math.abs(Ke.y) > Math.abs(Ke.x);
                }
                var Su = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.reset = function() {
                    Ke.prototype.reset.call(this), this._valid = void 0, delete this._firstMove, delete this._lastPoints;
                  }, Fe.prototype._start = function(Te) {
                    this._lastPoints = Te, Gc(Te[0].sub(Te[1])) && (this._valid = !1);
                  }, Fe.prototype._move = function(Te, Ae, Oe) {
                    var Be = Te[0].sub(this._lastPoints[0]), Ue = Te[1].sub(this._lastPoints[1]);
                    if (this._valid = this.gestureBeginsVertically(Be, Ue, Oe.timeStamp), this._valid) return this._lastPoints = Te, this._active = !0, { pitchDelta: (Be.y + Ue.y) / 2 * -0.5 };
                  }, Fe.prototype.gestureBeginsVertically = function(Te, Ae, Oe) {
                    if (this._valid !== void 0) return this._valid;
                    var Be = Te.mag() >= 2, Ue = Ae.mag() >= 2;
                    if (Be || Ue) {
                      if (!Be || !Ue) return this._firstMove === void 0 && (this._firstMove = Oe), Oe - this._firstMove < 100 && void 0;
                      var nt = Te.y > 0 == Ae.y > 0;
                      return Gc(Te) && Gc(Ae) && nt;
                    }
                  }, Fe;
                }(Ol), Lc = { panStep: 100, bearingStep: 15, pitchStep: 10 }, Do = function() {
                  var Ke = Lc;
                  this._panStep = Ke.panStep, this._bearingStep = Ke.bearingStep, this._pitchStep = Ke.pitchStep, this._rotationDisabled = !1;
                };
                function Ru(Ke) {
                  return Ke * (2 - Ke);
                }
                Do.prototype.reset = function() {
                  this._active = !1;
                }, Do.prototype.keydown = function(Ke) {
                  var Fe = this;
                  if (!(Ke.altKey || Ke.ctrlKey || Ke.metaKey)) {
                    var Te = 0, Ae = 0, Oe = 0, Be = 0, Ue = 0;
                    switch (Ke.keyCode) {
                      case 61:
                      case 107:
                      case 171:
                      case 187:
                        Te = 1;
                        break;
                      case 189:
                      case 109:
                      case 173:
                        Te = -1;
                        break;
                      case 37:
                        Ke.shiftKey ? Ae = -1 : (Ke.preventDefault(), Be = -1);
                        break;
                      case 39:
                        Ke.shiftKey ? Ae = 1 : (Ke.preventDefault(), Be = 1);
                        break;
                      case 38:
                        Ke.shiftKey ? Oe = 1 : (Ke.preventDefault(), Ue = -1);
                        break;
                      case 40:
                        Ke.shiftKey ? Oe = -1 : (Ke.preventDefault(), Ue = 1);
                        break;
                      default:
                        return;
                    }
                    return this._rotationDisabled && (Ae = 0, Oe = 0), { cameraAnimation: function(nt) {
                      var ht = nt.getZoom();
                      nt.easeTo({ duration: 300, easeId: "keyboardHandler", easing: Ru, zoom: Te ? Math.round(ht) + Te * (Ke.shiftKey ? 2 : 1) : ht, bearing: nt.getBearing() + Ae * Fe._bearingStep, pitch: nt.getPitch() + Oe * Fe._pitchStep, offset: [-Be * Fe._panStep, -Ue * Fe._panStep], center: nt.getCenter() }, { originalEvent: Ke });
                    } };
                  }
                }, Do.prototype.enable = function() {
                  this._enabled = !0;
                }, Do.prototype.disable = function() {
                  this._enabled = !1, this.reset();
                }, Do.prototype.isEnabled = function() {
                  return this._enabled;
                }, Do.prototype.isActive = function() {
                  return this._active;
                }, Do.prototype.disableRotation = function() {
                  this._rotationDisabled = !0;
                }, Do.prototype.enableRotation = function() {
                  this._rotationDisabled = !1;
                };
                var dc = 4.000244140625, vl = function(Ke, Fe) {
                  this._map = Ke, this._el = Ke.getCanvasContainer(), this._handler = Fe, this._delta = 0, this._defaultZoomRate = 0.01, this._wheelZoomRate = 0.0022222222222222222, ee.bindAll(["_onTimeout"], this);
                };
                vl.prototype.setZoomRate = function(Ke) {
                  this._defaultZoomRate = Ke;
                }, vl.prototype.setWheelZoomRate = function(Ke) {
                  this._wheelZoomRate = Ke;
                }, vl.prototype.isEnabled = function() {
                  return !!this._enabled;
                }, vl.prototype.isActive = function() {
                  return !!this._active || this._finishTimeout !== void 0;
                }, vl.prototype.isZooming = function() {
                  return !!this._zooming;
                }, vl.prototype.enable = function(Ke) {
                  this.isEnabled() || (this._enabled = !0, this._aroundCenter = Ke && Ke.around === "center");
                }, vl.prototype.disable = function() {
                  this.isEnabled() && (this._enabled = !1);
                }, vl.prototype.wheel = function(Ke) {
                  if (this.isEnabled()) {
                    var Fe = Ke.deltaMode === ee.window.WheelEvent.DOM_DELTA_LINE ? 40 * Ke.deltaY : Ke.deltaY, Te = ee.browser.now(), Ae = Te - (this._lastWheelEventTime || 0);
                    this._lastWheelEventTime = Te, Fe !== 0 && Fe % dc == 0 ? this._type = "wheel" : Fe !== 0 && Math.abs(Fe) < 4 ? this._type = "trackpad" : Ae > 400 ? (this._type = null, this._lastValue = Fe, this._timeout = setTimeout(this._onTimeout, 40, Ke)) : this._type || (this._type = Math.abs(Ae * Fe) < 200 ? "trackpad" : "wheel", this._timeout && (clearTimeout(this._timeout), this._timeout = null, Fe += this._lastValue)), Ke.shiftKey && Fe && (Fe /= 4), this._type && (this._lastWheelEvent = Ke, this._delta -= Fe, this._active || this._start(Ke)), Ke.preventDefault();
                  }
                }, vl.prototype._onTimeout = function(Ke) {
                  this._type = "wheel", this._delta -= this._lastValue, this._active || this._start(Ke);
                }, vl.prototype._start = function(Ke) {
                  if (this._delta) {
                    this._frameId && (this._frameId = null), this._active = !0, this.isZooming() || (this._zooming = !0), this._finishTimeout && (clearTimeout(this._finishTimeout), delete this._finishTimeout);
                    var Fe = oe.mousePos(this._el, Ke);
                    this._around = ee.LngLat.convert(this._aroundCenter ? this._map.getCenter() : this._map.unproject(Fe)), this._aroundPoint = this._map.transform.locationPoint(this._around), this._frameId || (this._frameId = !0, this._handler._triggerRenderFrame());
                  }
                }, vl.prototype.renderFrame = function() {
                  var Ke = this;
                  if (this._frameId && (this._frameId = null, this.isActive())) {
                    var Fe = this._map.transform;
                    if (this._delta !== 0) {
                      var Te = this._type === "wheel" && Math.abs(this._delta) > dc ? this._wheelZoomRate : this._defaultZoomRate, Ae = 2 / (1 + Math.exp(-Math.abs(this._delta * Te)));
                      this._delta < 0 && Ae !== 0 && (Ae = 1 / Ae);
                      var Oe = typeof this._targetZoom == "number" ? Fe.zoomScale(this._targetZoom) : Fe.scale;
                      this._targetZoom = Math.min(Fe.maxZoom, Math.max(Fe.minZoom, Fe.scaleZoom(Oe * Ae))), this._type === "wheel" && (this._startZoom = Fe.zoom, this._easing = this._smoothOutEasing(200)), this._delta = 0;
                    }
                    var Be, Ue = typeof this._targetZoom == "number" ? this._targetZoom : Fe.zoom, nt = this._startZoom, ht = this._easing, Et = !1;
                    if (this._type === "wheel" && nt && ht) {
                      var Mt = Math.min((ee.browser.now() - this._lastWheelEventTime) / 200, 1), qt = ht(Mt);
                      Be = ee.number(nt, Ue, qt), Mt < 1 ? this._frameId || (this._frameId = !0) : Et = !0;
                    } else Be = Ue, Et = !0;
                    return this._active = !0, Et && (this._active = !1, this._finishTimeout = setTimeout(function() {
                      Ke._zooming = !1, Ke._handler._triggerRenderFrame(), delete Ke._targetZoom, delete Ke._finishTimeout;
                    }, 200)), { noInertia: !0, needsRenderFrame: !Et, zoomDelta: Be - Fe.zoom, around: this._aroundPoint, originalEvent: this._lastWheelEvent };
                  }
                }, vl.prototype._smoothOutEasing = function(Ke) {
                  var Fe = ee.ease;
                  if (this._prevEase) {
                    var Te = this._prevEase, Ae = (ee.browser.now() - Te.start) / Te.duration, Oe = Te.easing(Ae + 0.01) - Te.easing(Ae), Be = 0.27 / Math.sqrt(Oe * Oe + 1e-4) * 0.01, Ue = Math.sqrt(0.0729 - Be * Be);
                    Fe = ee.bezier(Be, Ue, 0.25, 1);
                  }
                  return this._prevEase = { start: ee.browser.now(), duration: Ke, easing: Fe }, Fe;
                }, vl.prototype.reset = function() {
                  this._active = !1;
                };
                var Eu = function(Ke, Fe) {
                  this._clickZoom = Ke, this._tapZoom = Fe;
                };
                Eu.prototype.enable = function() {
                  this._clickZoom.enable(), this._tapZoom.enable();
                }, Eu.prototype.disable = function() {
                  this._clickZoom.disable(), this._tapZoom.disable();
                }, Eu.prototype.isEnabled = function() {
                  return this._clickZoom.isEnabled() && this._tapZoom.isEnabled();
                }, Eu.prototype.isActive = function() {
                  return this._clickZoom.isActive() || this._tapZoom.isActive();
                };
                var fu = function() {
                  this.reset();
                };
                fu.prototype.reset = function() {
                  this._active = !1;
                }, fu.prototype.dblclick = function(Ke, Fe) {
                  return Ke.preventDefault(), { cameraAnimation: function(Te) {
                    Te.easeTo({ duration: 300, zoom: Te.getZoom() + (Ke.shiftKey ? -1 : 1), around: Te.unproject(Fe) }, { originalEvent: Ke });
                  } };
                }, fu.prototype.enable = function() {
                  this._enabled = !0;
                }, fu.prototype.disable = function() {
                  this._enabled = !1, this.reset();
                }, fu.prototype.isEnabled = function() {
                  return this._enabled;
                }, fu.prototype.isActive = function() {
                  return this._active;
                };
                var ac = function() {
                  this._tap = new wu({ numTouches: 1, numTaps: 1 }), this.reset();
                };
                ac.prototype.reset = function() {
                  this._active = !1, delete this._swipePoint, delete this._swipeTouch, delete this._tapTime, this._tap.reset();
                }, ac.prototype.touchstart = function(Ke, Fe, Te) {
                  this._swipePoint || (this._tapTime && Ke.timeStamp - this._tapTime > 500 && this.reset(), this._tapTime ? Te.length > 0 && (this._swipePoint = Fe[0], this._swipeTouch = Te[0].identifier) : this._tap.touchstart(Ke, Fe, Te));
                }, ac.prototype.touchmove = function(Ke, Fe, Te) {
                  if (this._tapTime) {
                    if (this._swipePoint) {
                      if (Te[0].identifier !== this._swipeTouch) return;
                      var Ae = Fe[0], Oe = Ae.y - this._swipePoint.y;
                      return this._swipePoint = Ae, Ke.preventDefault(), this._active = !0, { zoomDelta: Oe / 128 };
                    }
                  } else this._tap.touchmove(Ke, Fe, Te);
                }, ac.prototype.touchend = function(Ke, Fe, Te) {
                  this._tapTime ? this._swipePoint && Te.length === 0 && this.reset() : this._tap.touchend(Ke, Fe, Te) && (this._tapTime = Ke.timeStamp);
                }, ac.prototype.touchcancel = function() {
                  this.reset();
                }, ac.prototype.enable = function() {
                  this._enabled = !0;
                }, ac.prototype.disable = function() {
                  this._enabled = !1, this.reset();
                }, ac.prototype.isEnabled = function() {
                  return this._enabled;
                }, ac.prototype.isActive = function() {
                  return this._active;
                };
                var bc = function(Ke, Fe, Te) {
                  this._el = Ke, this._mousePan = Fe, this._touchPan = Te;
                };
                bc.prototype.enable = function(Ke) {
                  this._inertiaOptions = Ke || {}, this._mousePan.enable(), this._touchPan.enable(), this._el.classList.add("mapboxgl-touch-drag-pan");
                }, bc.prototype.disable = function() {
                  this._mousePan.disable(), this._touchPan.disable(), this._el.classList.remove("mapboxgl-touch-drag-pan");
                }, bc.prototype.isEnabled = function() {
                  return this._mousePan.isEnabled() && this._touchPan.isEnabled();
                }, bc.prototype.isActive = function() {
                  return this._mousePan.isActive() || this._touchPan.isActive();
                };
                var kc = function(Ke, Fe, Te) {
                  this._pitchWithRotate = Ke.pitchWithRotate, this._mouseRotate = Fe, this._mousePitch = Te;
                };
                kc.prototype.enable = function() {
                  this._mouseRotate.enable(), this._pitchWithRotate && this._mousePitch.enable();
                }, kc.prototype.disable = function() {
                  this._mouseRotate.disable(), this._mousePitch.disable();
                }, kc.prototype.isEnabled = function() {
                  return this._mouseRotate.isEnabled() && (!this._pitchWithRotate || this._mousePitch.isEnabled());
                }, kc.prototype.isActive = function() {
                  return this._mouseRotate.isActive() || this._mousePitch.isActive();
                };
                var fc = function(Ke, Fe, Te, Ae) {
                  this._el = Ke, this._touchZoom = Fe, this._touchRotate = Te, this._tapDragZoom = Ae, this._rotationDisabled = !1, this._enabled = !0;
                };
                fc.prototype.enable = function(Ke) {
                  this._touchZoom.enable(Ke), this._rotationDisabled || this._touchRotate.enable(Ke), this._tapDragZoom.enable(), this._el.classList.add("mapboxgl-touch-zoom-rotate");
                }, fc.prototype.disable = function() {
                  this._touchZoom.disable(), this._touchRotate.disable(), this._tapDragZoom.disable(), this._el.classList.remove("mapboxgl-touch-zoom-rotate");
                }, fc.prototype.isEnabled = function() {
                  return this._touchZoom.isEnabled() && (this._rotationDisabled || this._touchRotate.isEnabled()) && this._tapDragZoom.isEnabled();
                }, fc.prototype.isActive = function() {
                  return this._touchZoom.isActive() || this._touchRotate.isActive() || this._tapDragZoom.isActive();
                }, fc.prototype.disableRotation = function() {
                  this._rotationDisabled = !0, this._touchRotate.disable();
                }, fc.prototype.enableRotation = function() {
                  this._rotationDisabled = !1, this._touchZoom.isEnabled() && this._touchRotate.enable();
                };
                var oc = function(Ke) {
                  return Ke.zoom || Ke.drag || Ke.pitch || Ke.rotate;
                }, ss = function(Ke) {
                  function Fe() {
                    Ke.apply(this, arguments);
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe;
                }(ee.Event);
                function su(Ke) {
                  return Ke.panDelta && Ke.panDelta.mag() || Ke.zoomDelta || Ke.bearingDelta || Ke.pitchDelta;
                }
                var Tl = function(Ke, Fe) {
                  this._map = Ke, this._el = this._map.getCanvasContainer(), this._handlers = [], this._handlersById = {}, this._changes = [], this._inertia = new Wu(Ke), this._bearingSnap = Fe.bearingSnap, this._previousActiveHandlers = {}, this._eventsInProgress = {}, this._addDefaultHandlers(Fe), ee.bindAll(["handleEvent", "handleWindowEvent"], this);
                  var Te = this._el;
                  this._listeners = [[Te, "touchstart", { passive: !0 }], [Te, "touchmove", { passive: !1 }], [Te, "touchend", void 0], [Te, "touchcancel", void 0], [Te, "mousedown", void 0], [Te, "mousemove", void 0], [Te, "mouseup", void 0], [ee.window.document, "mousemove", { capture: !0 }], [ee.window.document, "mouseup", void 0], [Te, "mouseover", void 0], [Te, "mouseout", void 0], [Te, "dblclick", void 0], [Te, "click", void 0], [Te, "keydown", { capture: !1 }], [Te, "keyup", void 0], [Te, "wheel", { passive: !1 }], [Te, "contextmenu", void 0], [ee.window, "blur", void 0]];
                  for (var Ae = 0, Oe = this._listeners; Ae < Oe.length; Ae += 1) {
                    var Be = Oe[Ae], Ue = Be[0], nt = Be[1], ht = Be[2];
                    oe.addEventListener(Ue, nt, Ue === ee.window.document ? this.handleWindowEvent : this.handleEvent, ht);
                  }
                };
                Tl.prototype.destroy = function() {
                  for (var Ke = 0, Fe = this._listeners; Ke < Fe.length; Ke += 1) {
                    var Te = Fe[Ke], Ae = Te[0], Oe = Te[1], Be = Te[2];
                    oe.removeEventListener(Ae, Oe, Ae === ee.window.document ? this.handleWindowEvent : this.handleEvent, Be);
                  }
                }, Tl.prototype._addDefaultHandlers = function(Ke) {
                  var Fe = this._map, Te = Fe.getCanvasContainer();
                  this._add("mapEvent", new iu(Fe, Ke));
                  var Ae = Fe.boxZoom = new vc(Fe, Ke);
                  this._add("boxZoom", Ae);
                  var Oe = new Pc(), Be = new fu();
                  Fe.doubleClickZoom = new Eu(Be, Oe), this._add("tapZoom", Oe), this._add("clickZoom", Be);
                  var Ue = new ac();
                  this._add("tapDragZoom", Ue);
                  var nt = Fe.touchPitch = new Su();
                  this._add("touchPitch", nt);
                  var ht = new au(Ke), Et = new Qc(Ke);
                  Fe.dragRotate = new kc(Ke, ht, Et), this._add("mouseRotate", ht, ["mousePitch"]), this._add("mousePitch", Et, ["mouseRotate"]);
                  var Mt = new yu(Ke), qt = new Cc(Ke);
                  Fe.dragPan = new bc(Te, Mt, qt), this._add("mousePan", Mt), this._add("touchPan", qt, ["touchZoom", "touchRotate"]);
                  var tn = new ou(), xn = new Mu();
                  Fe.touchZoomRotate = new fc(Te, xn, tn, Ue), this._add("touchRotate", tn, ["touchPan", "touchZoom"]), this._add("touchZoom", xn, ["touchPan", "touchRotate"]);
                  var An = Fe.scrollZoom = new vl(Fe, this);
                  this._add("scrollZoom", An, ["mousePan"]);
                  var On = Fe.keyboard = new Do();
                  this._add("keyboard", On), this._add("blockableMapEvent", new xu(Fe));
                  for (var Bn = 0, qn = ["boxZoom", "doubleClickZoom", "tapDragZoom", "touchPitch", "dragRotate", "dragPan", "touchZoomRotate", "scrollZoom", "keyboard"]; Bn < qn.length; Bn += 1) {
                    var Vn = qn[Bn];
                    Ke.interactive && Ke[Vn] && Fe[Vn].enable(Ke[Vn]);
                  }
                }, Tl.prototype._add = function(Ke, Fe, Te) {
                  this._handlers.push({ handlerName: Ke, handler: Fe, allowed: Te }), this._handlersById[Ke] = Fe;
                }, Tl.prototype.stop = function(Ke) {
                  if (!this._updatingCamera) {
                    for (var Fe = 0, Te = this._handlers; Fe < Te.length; Fe += 1) Te[Fe].handler.reset();
                    this._inertia.clear(), this._fireEvents({}, {}, Ke), this._changes = [];
                  }
                }, Tl.prototype.isActive = function() {
                  for (var Ke = 0, Fe = this._handlers; Ke < Fe.length; Ke += 1) if (Fe[Ke].handler.isActive()) return !0;
                  return !1;
                }, Tl.prototype.isZooming = function() {
                  return !!this._eventsInProgress.zoom || this._map.scrollZoom.isZooming();
                }, Tl.prototype.isRotating = function() {
                  return !!this._eventsInProgress.rotate;
                }, Tl.prototype.isMoving = function() {
                  return !!oc(this._eventsInProgress) || this.isZooming();
                }, Tl.prototype._blockedByActive = function(Ke, Fe, Te) {
                  for (var Ae in Ke) if (Ae !== Te && (!Fe || Fe.indexOf(Ae) < 0)) return !0;
                  return !1;
                }, Tl.prototype.handleWindowEvent = function(Ke) {
                  this.handleEvent(Ke, Ke.type + "Window");
                }, Tl.prototype._getMapTouches = function(Ke) {
                  for (var Fe = [], Te = 0, Ae = Ke; Te < Ae.length; Te += 1) {
                    var Oe = Ae[Te], Be = Oe.target;
                    this._el.contains(Be) && Fe.push(Oe);
                  }
                  return Fe;
                }, Tl.prototype.handleEvent = function(Ke, Fe) {
                  if (Ke.type !== "blur") {
                    this._updatingCamera = !0;
                    for (var Te = Ke.type === "renderFrame" ? void 0 : Ke, Ae = { needsRenderFrame: !1 }, Oe = {}, Be = {}, Ue = Ke.touches ? this._getMapTouches(Ke.touches) : void 0, nt = Ue ? oe.touchPos(this._el, Ue) : oe.mousePos(this._el, Ke), ht = 0, Et = this._handlers; ht < Et.length; ht += 1) {
                      var Mt = Et[ht], qt = Mt.handlerName, tn = Mt.handler, xn = Mt.allowed;
                      if (tn.isEnabled()) {
                        var An = void 0;
                        this._blockedByActive(Be, xn, qt) ? tn.reset() : tn[Fe || Ke.type] && (An = tn[Fe || Ke.type](Ke, nt, Ue), this.mergeHandlerResult(Ae, Oe, An, qt, Te), An && An.needsRenderFrame && this._triggerRenderFrame()), (An || tn.isActive()) && (Be[qt] = tn);
                      }
                    }
                    var On = {};
                    for (var Bn in this._previousActiveHandlers) Be[Bn] || (On[Bn] = Te);
                    this._previousActiveHandlers = Be, (Object.keys(On).length || su(Ae)) && (this._changes.push([Ae, Oe, On]), this._triggerRenderFrame()), (Object.keys(Be).length || su(Ae)) && this._map._stop(!0), this._updatingCamera = !1;
                    var qn = Ae.cameraAnimation;
                    qn && (this._inertia.clear(), this._fireEvents({}, {}, !0), this._changes = [], qn(this._map));
                  } else this.stop(!0);
                }, Tl.prototype.mergeHandlerResult = function(Ke, Fe, Te, Ae, Oe) {
                  if (Te) {
                    ee.extend(Ke, Te);
                    var Be = { handlerName: Ae, originalEvent: Te.originalEvent || Oe };
                    Te.zoomDelta !== void 0 && (Fe.zoom = Be), Te.panDelta !== void 0 && (Fe.drag = Be), Te.pitchDelta !== void 0 && (Fe.pitch = Be), Te.bearingDelta !== void 0 && (Fe.rotate = Be);
                  }
                }, Tl.prototype._applyChanges = function() {
                  for (var Ke = {}, Fe = {}, Te = {}, Ae = 0, Oe = this._changes; Ae < Oe.length; Ae += 1) {
                    var Be = Oe[Ae], Ue = Be[0], nt = Be[1], ht = Be[2];
                    Ue.panDelta && (Ke.panDelta = (Ke.panDelta || new ee.Point(0, 0))._add(Ue.panDelta)), Ue.zoomDelta && (Ke.zoomDelta = (Ke.zoomDelta || 0) + Ue.zoomDelta), Ue.bearingDelta && (Ke.bearingDelta = (Ke.bearingDelta || 0) + Ue.bearingDelta), Ue.pitchDelta && (Ke.pitchDelta = (Ke.pitchDelta || 0) + Ue.pitchDelta), Ue.around !== void 0 && (Ke.around = Ue.around), Ue.pinchAround !== void 0 && (Ke.pinchAround = Ue.pinchAround), Ue.noInertia && (Ke.noInertia = Ue.noInertia), ee.extend(Fe, nt), ee.extend(Te, ht);
                  }
                  this._updateMapTransform(Ke, Fe, Te), this._changes = [];
                }, Tl.prototype._updateMapTransform = function(Ke, Fe, Te) {
                  var Ae = this._map, Oe = Ae.transform;
                  if (!su(Ke)) return this._fireEvents(Fe, Te, !0);
                  var Be = Ke.panDelta, Ue = Ke.zoomDelta, nt = Ke.bearingDelta, ht = Ke.pitchDelta, Et = Ke.around, Mt = Ke.pinchAround;
                  Mt !== void 0 && (Et = Mt), Ae._stop(!0), Et = Et || Ae.transform.centerPoint;
                  var qt = Oe.pointLocation(Be ? Et.sub(Be) : Et);
                  nt && (Oe.bearing += nt), ht && (Oe.pitch += ht), Ue && (Oe.zoom += Ue), Oe.setLocationAtPoint(qt, Et), this._map._update(), Ke.noInertia || this._inertia.record(Ke), this._fireEvents(Fe, Te, !0);
                }, Tl.prototype._fireEvents = function(Ke, Fe, Te) {
                  var Ae = this, Oe = oc(this._eventsInProgress), Be = oc(Ke), Ue = {};
                  for (var nt in Ke) {
                    var ht = Ke[nt].originalEvent;
                    this._eventsInProgress[nt] || (Ue[nt + "start"] = ht), this._eventsInProgress[nt] = Ke[nt];
                  }
                  for (var Et in !Oe && Be && this._fireEvent("movestart", Be.originalEvent), Ue) this._fireEvent(Et, Ue[Et]);
                  for (var Mt in Be && this._fireEvent("move", Be.originalEvent), Ke) {
                    var qt = Ke[Mt].originalEvent;
                    this._fireEvent(Mt, qt);
                  }
                  var tn, xn = {};
                  for (var An in this._eventsInProgress) {
                    var On = this._eventsInProgress[An], Bn = On.handlerName, qn = On.originalEvent;
                    this._handlersById[Bn].isActive() || (delete this._eventsInProgress[An], tn = Fe[Bn] || qn, xn[An + "end"] = tn);
                  }
                  for (var Vn in xn) this._fireEvent(Vn, xn[Vn]);
                  var sr = oc(this._eventsInProgress);
                  if (Te && (Oe || Be) && !sr) {
                    this._updatingCamera = !0;
                    var Un = this._inertia._onMoveEnd(this._map.dragPan._inertiaOptions), wr = function(Cr) {
                      return Cr !== 0 && -Ae._bearingSnap < Cr && Cr < Ae._bearingSnap;
                    };
                    Un ? (wr(Un.bearing || this._map.getBearing()) && (Un.bearing = 0), this._map.easeTo(Un, { originalEvent: tn })) : (this._map.fire(new ee.Event("moveend", { originalEvent: tn })), wr(this._map.getBearing()) && this._map.resetNorth()), this._updatingCamera = !1;
                  }
                }, Tl.prototype._fireEvent = function(Ke, Fe) {
                  this._map.fire(new ee.Event(Ke, Fe ? { originalEvent: Fe } : {}));
                }, Tl.prototype._requestFrame = function() {
                  var Ke = this;
                  return this._map.triggerRepaint(), this._map._renderTaskQueue.add(function(Fe) {
                    delete Ke._frameId, Ke.handleEvent(new ss("renderFrame", { timeStamp: Fe })), Ke._applyChanges();
                  });
                }, Tl.prototype._triggerRenderFrame = function() {
                  this._frameId === void 0 && (this._frameId = this._requestFrame());
                };
                var sc = function(Ke) {
                  function Fe(Te, Ae) {
                    Ke.call(this), this._moving = !1, this._zooming = !1, this.transform = Te, this._bearingSnap = Ae.bearingSnap, ee.bindAll(["_renderFrameCallback"], this);
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.getCenter = function() {
                    return new ee.LngLat(this.transform.center.lng, this.transform.center.lat);
                  }, Fe.prototype.setCenter = function(Te, Ae) {
                    return this.jumpTo({ center: Te }, Ae);
                  }, Fe.prototype.panBy = function(Te, Ae, Oe) {
                    return Te = ee.Point.convert(Te).mult(-1), this.panTo(this.transform.center, ee.extend({ offset: Te }, Ae), Oe);
                  }, Fe.prototype.panTo = function(Te, Ae, Oe) {
                    return this.easeTo(ee.extend({ center: Te }, Ae), Oe);
                  }, Fe.prototype.getZoom = function() {
                    return this.transform.zoom;
                  }, Fe.prototype.setZoom = function(Te, Ae) {
                    return this.jumpTo({ zoom: Te }, Ae), this;
                  }, Fe.prototype.zoomTo = function(Te, Ae, Oe) {
                    return this.easeTo(ee.extend({ zoom: Te }, Ae), Oe);
                  }, Fe.prototype.zoomIn = function(Te, Ae) {
                    return this.zoomTo(this.getZoom() + 1, Te, Ae), this;
                  }, Fe.prototype.zoomOut = function(Te, Ae) {
                    return this.zoomTo(this.getZoom() - 1, Te, Ae), this;
                  }, Fe.prototype.getBearing = function() {
                    return this.transform.bearing;
                  }, Fe.prototype.setBearing = function(Te, Ae) {
                    return this.jumpTo({ bearing: Te }, Ae), this;
                  }, Fe.prototype.getPadding = function() {
                    return this.transform.padding;
                  }, Fe.prototype.setPadding = function(Te, Ae) {
                    return this.jumpTo({ padding: Te }, Ae), this;
                  }, Fe.prototype.rotateTo = function(Te, Ae, Oe) {
                    return this.easeTo(ee.extend({ bearing: Te }, Ae), Oe);
                  }, Fe.prototype.resetNorth = function(Te, Ae) {
                    return this.rotateTo(0, ee.extend({ duration: 1e3 }, Te), Ae), this;
                  }, Fe.prototype.resetNorthPitch = function(Te, Ae) {
                    return this.easeTo(ee.extend({ bearing: 0, pitch: 0, duration: 1e3 }, Te), Ae), this;
                  }, Fe.prototype.snapToNorth = function(Te, Ae) {
                    return Math.abs(this.getBearing()) < this._bearingSnap ? this.resetNorth(Te, Ae) : this;
                  }, Fe.prototype.getPitch = function() {
                    return this.transform.pitch;
                  }, Fe.prototype.setPitch = function(Te, Ae) {
                    return this.jumpTo({ pitch: Te }, Ae), this;
                  }, Fe.prototype.cameraForBounds = function(Te, Ae) {
                    Te = ee.LngLatBounds.convert(Te);
                    var Oe = Ae && Ae.bearing || 0;
                    return this._cameraForBoxAndBearing(Te.getNorthWest(), Te.getSouthEast(), Oe, Ae);
                  }, Fe.prototype._cameraForBoxAndBearing = function(Te, Ae, Oe, Be) {
                    var Ue = { top: 0, bottom: 0, right: 0, left: 0 };
                    if (typeof (Be = ee.extend({ padding: Ue, offset: [0, 0], maxZoom: this.transform.maxZoom }, Be)).padding == "number") {
                      var nt = Be.padding;
                      Be.padding = { top: nt, bottom: nt, right: nt, left: nt };
                    }
                    Be.padding = ee.extend(Ue, Be.padding);
                    var ht = this.transform, Et = ht.padding, Mt = ht.project(ee.LngLat.convert(Te)), qt = ht.project(ee.LngLat.convert(Ae)), tn = Mt.rotate(-Oe * Math.PI / 180), xn = qt.rotate(-Oe * Math.PI / 180), An = new ee.Point(Math.max(tn.x, xn.x), Math.max(tn.y, xn.y)), On = new ee.Point(Math.min(tn.x, xn.x), Math.min(tn.y, xn.y)), Bn = An.sub(On), qn = (ht.width - (Et.left + Et.right + Be.padding.left + Be.padding.right)) / Bn.x, Vn = (ht.height - (Et.top + Et.bottom + Be.padding.top + Be.padding.bottom)) / Bn.y;
                    if (!(Vn < 0 || qn < 0)) {
                      var sr = Math.min(ht.scaleZoom(ht.scale * Math.min(qn, Vn)), Be.maxZoom), Un = typeof Be.offset.x == "number" ? new ee.Point(Be.offset.x, Be.offset.y) : ee.Point.convert(Be.offset), wr = (Be.padding.left - Be.padding.right) / 2, Cr = (Be.padding.top - Be.padding.bottom) / 2, Tr = new ee.Point(wr, Cr).rotate(Oe * Math.PI / 180), ii = Un.add(Tr).mult(ht.scale / ht.zoomScale(sr));
                      return { center: ht.unproject(Mt.add(qt).div(2).sub(ii)), zoom: sr, bearing: Oe };
                    }
                    ee.warnOnce("Map cannot fit within canvas with the given bounds, padding, and/or offset.");
                  }, Fe.prototype.fitBounds = function(Te, Ae, Oe) {
                    return this._fitInternal(this.cameraForBounds(Te, Ae), Ae, Oe);
                  }, Fe.prototype.fitScreenCoordinates = function(Te, Ae, Oe, Be, Ue) {
                    return this._fitInternal(this._cameraForBoxAndBearing(this.transform.pointLocation(ee.Point.convert(Te)), this.transform.pointLocation(ee.Point.convert(Ae)), Oe, Be), Be, Ue);
                  }, Fe.prototype._fitInternal = function(Te, Ae, Oe) {
                    return Te ? (delete (Ae = ee.extend(Te, Ae)).padding, Ae.linear ? this.easeTo(Ae, Oe) : this.flyTo(Ae, Oe)) : this;
                  }, Fe.prototype.jumpTo = function(Te, Ae) {
                    this.stop();
                    var Oe = this.transform, Be = !1, Ue = !1, nt = !1;
                    return "zoom" in Te && Oe.zoom !== +Te.zoom && (Be = !0, Oe.zoom = +Te.zoom), Te.center !== void 0 && (Oe.center = ee.LngLat.convert(Te.center)), "bearing" in Te && Oe.bearing !== +Te.bearing && (Ue = !0, Oe.bearing = +Te.bearing), "pitch" in Te && Oe.pitch !== +Te.pitch && (nt = !0, Oe.pitch = +Te.pitch), Te.padding == null || Oe.isPaddingEqual(Te.padding) || (Oe.padding = Te.padding), this.fire(new ee.Event("movestart", Ae)).fire(new ee.Event("move", Ae)), Be && this.fire(new ee.Event("zoomstart", Ae)).fire(new ee.Event("zoom", Ae)).fire(new ee.Event("zoomend", Ae)), Ue && this.fire(new ee.Event("rotatestart", Ae)).fire(new ee.Event("rotate", Ae)).fire(new ee.Event("rotateend", Ae)), nt && this.fire(new ee.Event("pitchstart", Ae)).fire(new ee.Event("pitch", Ae)).fire(new ee.Event("pitchend", Ae)), this.fire(new ee.Event("moveend", Ae));
                  }, Fe.prototype.easeTo = function(Te, Ae) {
                    var Oe = this;
                    this._stop(!1, Te.easeId), ((Te = ee.extend({ offset: [0, 0], duration: 500, easing: ee.ease }, Te)).animate === !1 || !Te.essential && ee.browser.prefersReducedMotion) && (Te.duration = 0);
                    var Be = this.transform, Ue = this.getZoom(), nt = this.getBearing(), ht = this.getPitch(), Et = this.getPadding(), Mt = "zoom" in Te ? +Te.zoom : Ue, qt = "bearing" in Te ? this._normalizeBearing(Te.bearing, nt) : nt, tn = "pitch" in Te ? +Te.pitch : ht, xn = "padding" in Te ? Te.padding : Be.padding, An = ee.Point.convert(Te.offset), On = Be.centerPoint.add(An), Bn = Be.pointLocation(On), qn = ee.LngLat.convert(Te.center || Bn);
                    this._normalizeCenter(qn);
                    var Vn, sr, Un = Be.project(Bn), wr = Be.project(qn).sub(Un), Cr = Be.zoomScale(Mt - Ue);
                    Te.around && (Vn = ee.LngLat.convert(Te.around), sr = Be.locationPoint(Vn));
                    var Tr = { moving: this._moving, zooming: this._zooming, rotating: this._rotating, pitching: this._pitching };
                    return this._zooming = this._zooming || Mt !== Ue, this._rotating = this._rotating || nt !== qt, this._pitching = this._pitching || tn !== ht, this._padding = !Be.isPaddingEqual(xn), this._easeId = Te.easeId, this._prepareEase(Ae, Te.noMoveStart, Tr), this._ease(function(ii) {
                      if (Oe._zooming && (Be.zoom = ee.number(Ue, Mt, ii)), Oe._rotating && (Be.bearing = ee.number(nt, qt, ii)), Oe._pitching && (Be.pitch = ee.number(ht, tn, ii)), Oe._padding && (Be.interpolatePadding(Et, xn, ii), On = Be.centerPoint.add(An)), Vn) Be.setLocationAtPoint(Vn, sr);
                      else {
                        var wi = Be.zoomScale(Be.zoom - Ue), Wi = Mt > Ue ? Math.min(2, Cr) : Math.max(0.5, Cr), Yi = Math.pow(Wi, 1 - ii), ra = Be.unproject(Un.add(wr.mult(ii * Yi)).mult(wi));
                        Be.setLocationAtPoint(Be.renderWorldCopies ? ra.wrap() : ra, On);
                      }
                      Oe._fireMoveEvents(Ae);
                    }, function(ii) {
                      Oe._afterEase(Ae, ii);
                    }, Te), this;
                  }, Fe.prototype._prepareEase = function(Te, Ae, Oe) {
                    Oe === void 0 && (Oe = {}), this._moving = !0, Ae || Oe.moving || this.fire(new ee.Event("movestart", Te)), this._zooming && !Oe.zooming && this.fire(new ee.Event("zoomstart", Te)), this._rotating && !Oe.rotating && this.fire(new ee.Event("rotatestart", Te)), this._pitching && !Oe.pitching && this.fire(new ee.Event("pitchstart", Te));
                  }, Fe.prototype._fireMoveEvents = function(Te) {
                    this.fire(new ee.Event("move", Te)), this._zooming && this.fire(new ee.Event("zoom", Te)), this._rotating && this.fire(new ee.Event("rotate", Te)), this._pitching && this.fire(new ee.Event("pitch", Te));
                  }, Fe.prototype._afterEase = function(Te, Ae) {
                    if (!this._easeId || !Ae || this._easeId !== Ae) {
                      delete this._easeId;
                      var Oe = this._zooming, Be = this._rotating, Ue = this._pitching;
                      this._moving = !1, this._zooming = !1, this._rotating = !1, this._pitching = !1, this._padding = !1, Oe && this.fire(new ee.Event("zoomend", Te)), Be && this.fire(new ee.Event("rotateend", Te)), Ue && this.fire(new ee.Event("pitchend", Te)), this.fire(new ee.Event("moveend", Te));
                    }
                  }, Fe.prototype.flyTo = function(Te, Ae) {
                    var Oe = this;
                    if (!Te.essential && ee.browser.prefersReducedMotion) {
                      var Be = ee.pick(Te, ["center", "zoom", "bearing", "pitch", "around"]);
                      return this.jumpTo(Be, Ae);
                    }
                    this.stop(), Te = ee.extend({ offset: [0, 0], speed: 1.2, curve: 1.42, easing: ee.ease }, Te);
                    var Ue = this.transform, nt = this.getZoom(), ht = this.getBearing(), Et = this.getPitch(), Mt = this.getPadding(), qt = "zoom" in Te ? ee.clamp(+Te.zoom, Ue.minZoom, Ue.maxZoom) : nt, tn = "bearing" in Te ? this._normalizeBearing(Te.bearing, ht) : ht, xn = "pitch" in Te ? +Te.pitch : Et, An = "padding" in Te ? Te.padding : Ue.padding, On = Ue.zoomScale(qt - nt), Bn = ee.Point.convert(Te.offset), qn = Ue.centerPoint.add(Bn), Vn = Ue.pointLocation(qn), sr = ee.LngLat.convert(Te.center || Vn);
                    this._normalizeCenter(sr);
                    var Un = Ue.project(Vn), wr = Ue.project(sr).sub(Un), Cr = Te.curve, Tr = Math.max(Ue.width, Ue.height), ii = Tr / On, wi = wr.mag();
                    if ("minZoom" in Te) {
                      var Wi = ee.clamp(Math.min(Te.minZoom, nt, qt), Ue.minZoom, Ue.maxZoom), Yi = Tr / Ue.zoomScale(Wi - nt);
                      Cr = Math.sqrt(Yi / wi * 2);
                    }
                    var ra = Cr * Cr;
                    function ha(Ja) {
                      var Uo = (ii * ii - Tr * Tr + (Ja ? -1 : 1) * ra * ra * wi * wi) / (2 * (Ja ? ii : Tr) * ra * wi);
                      return Math.log(Math.sqrt(Uo * Uo + 1) - Uo);
                    }
                    function ya(Ja) {
                      return (Math.exp(Ja) - Math.exp(-Ja)) / 2;
                    }
                    function va(Ja) {
                      return (Math.exp(Ja) + Math.exp(-Ja)) / 2;
                    }
                    var qa = ha(0), Aa = function(Ja) {
                      return va(qa) / va(qa + Cr * Ja);
                    }, Xa = function(Ja) {
                      return Tr * ((va(qa) * (ya(Uo = qa + Cr * Ja) / va(Uo)) - ya(qa)) / ra) / wi;
                      var Uo;
                    }, Oa = (ha(1) - qa) / Cr;
                    if (Math.abs(wi) < 1e-6 || !isFinite(Oa)) {
                      if (Math.abs(Tr - ii) < 1e-6) return this.easeTo(Te, Ae);
                      var Ya = ii < Tr ? -1 : 1;
                      Oa = Math.abs(Math.log(ii / Tr)) / Cr, Xa = function() {
                        return 0;
                      }, Aa = function(Ja) {
                        return Math.exp(Ya * Cr * Ja);
                      };
                    }
                    if ("duration" in Te) Te.duration = +Te.duration;
                    else {
                      var Wo = "screenSpeed" in Te ? +Te.screenSpeed / Cr : +Te.speed;
                      Te.duration = 1e3 * Oa / Wo;
                    }
                    return Te.maxDuration && Te.duration > Te.maxDuration && (Te.duration = 0), this._zooming = !0, this._rotating = ht !== tn, this._pitching = xn !== Et, this._padding = !Ue.isPaddingEqual(An), this._prepareEase(Ae, !1), this._ease(function(Ja) {
                      var Uo = Ja * Oa, ys = 1 / Aa(Uo);
                      Ue.zoom = Ja === 1 ? qt : nt + Ue.scaleZoom(ys), Oe._rotating && (Ue.bearing = ee.number(ht, tn, Ja)), Oe._pitching && (Ue.pitch = ee.number(Et, xn, Ja)), Oe._padding && (Ue.interpolatePadding(Mt, An, Ja), qn = Ue.centerPoint.add(Bn));
                      var il = Ja === 1 ? sr : Ue.unproject(Un.add(wr.mult(Xa(Uo))).mult(ys));
                      Ue.setLocationAtPoint(Ue.renderWorldCopies ? il.wrap() : il, qn), Oe._fireMoveEvents(Ae);
                    }, function() {
                      return Oe._afterEase(Ae);
                    }, Te), this;
                  }, Fe.prototype.isEasing = function() {
                    return !!this._easeFrameId;
                  }, Fe.prototype.stop = function() {
                    return this._stop();
                  }, Fe.prototype._stop = function(Te, Ae) {
                    if (this._easeFrameId && (this._cancelRenderFrame(this._easeFrameId), delete this._easeFrameId, delete this._onEaseFrame), this._onEaseEnd) {
                      var Oe = this._onEaseEnd;
                      delete this._onEaseEnd, Oe.call(this, Ae);
                    }
                    if (!Te) {
                      var Be = this.handlers;
                      Be && Be.stop(!1);
                    }
                    return this;
                  }, Fe.prototype._ease = function(Te, Ae, Oe) {
                    Oe.animate === !1 || Oe.duration === 0 ? (Te(1), Ae()) : (this._easeStart = ee.browser.now(), this._easeOptions = Oe, this._onEaseFrame = Te, this._onEaseEnd = Ae, this._easeFrameId = this._requestRenderFrame(this._renderFrameCallback));
                  }, Fe.prototype._renderFrameCallback = function() {
                    var Te = Math.min((ee.browser.now() - this._easeStart) / this._easeOptions.duration, 1);
                    this._onEaseFrame(this._easeOptions.easing(Te)), Te < 1 ? this._easeFrameId = this._requestRenderFrame(this._renderFrameCallback) : this.stop();
                  }, Fe.prototype._normalizeBearing = function(Te, Ae) {
                    Te = ee.wrap(Te, -180, 180);
                    var Oe = Math.abs(Te - Ae);
                    return Math.abs(Te - 360 - Ae) < Oe && (Te -= 360), Math.abs(Te + 360 - Ae) < Oe && (Te += 360), Te;
                  }, Fe.prototype._normalizeCenter = function(Te) {
                    var Ae = this.transform;
                    if (Ae.renderWorldCopies && !Ae.lngRange) {
                      var Oe = Te.lng - Ae.center.lng;
                      Te.lng += Oe > 180 ? -360 : Oe < -180 ? 360 : 0;
                    }
                  }, Fe;
                }(ee.Evented), hc = function(Ke) {
                  Ke === void 0 && (Ke = {}), this.options = Ke, ee.bindAll(["_toggleAttribution", "_updateEditLink", "_updateData", "_updateCompact"], this);
                };
                hc.prototype.getDefaultPosition = function() {
                  return "bottom-right";
                }, hc.prototype.onAdd = function(Ke) {
                  var Fe = this.options && this.options.compact;
                  return this._map = Ke, this._container = oe.create("div", "mapboxgl-ctrl mapboxgl-ctrl-attrib"), this._compactButton = oe.create("button", "mapboxgl-ctrl-attrib-button", this._container), this._compactButton.addEventListener("click", this._toggleAttribution), this._setElementTitle(this._compactButton, "ToggleAttribution"), this._innerContainer = oe.create("div", "mapboxgl-ctrl-attrib-inner", this._container), this._innerContainer.setAttribute("role", "list"), Fe && this._container.classList.add("mapboxgl-compact"), this._updateAttributions(), this._updateEditLink(), this._map.on("styledata", this._updateData), this._map.on("sourcedata", this._updateData), this._map.on("moveend", this._updateEditLink), Fe === void 0 && (this._map.on("resize", this._updateCompact), this._updateCompact()), this._container;
                }, hc.prototype.onRemove = function() {
                  oe.remove(this._container), this._map.off("styledata", this._updateData), this._map.off("sourcedata", this._updateData), this._map.off("moveend", this._updateEditLink), this._map.off("resize", this._updateCompact), this._map = void 0, this._attribHTML = void 0;
                }, hc.prototype._setElementTitle = function(Ke, Fe) {
                  var Te = this._map._getUIString("AttributionControl." + Fe);
                  Ke.title = Te, Ke.setAttribute("aria-label", Te);
                }, hc.prototype._toggleAttribution = function() {
                  this._container.classList.contains("mapboxgl-compact-show") ? (this._container.classList.remove("mapboxgl-compact-show"), this._compactButton.setAttribute("aria-pressed", "false")) : (this._container.classList.add("mapboxgl-compact-show"), this._compactButton.setAttribute("aria-pressed", "true"));
                }, hc.prototype._updateEditLink = function() {
                  var Ke = this._editLink;
                  Ke || (Ke = this._editLink = this._container.querySelector(".mapbox-improve-map"));
                  var Fe = [{ key: "owner", value: this.styleOwner }, { key: "id", value: this.styleId }, { key: "access_token", value: this._map._requestManager._customAccessToken || ee.config.ACCESS_TOKEN }];
                  if (Ke) {
                    var Te = Fe.reduce(function(Ae, Oe, Be) {
                      return Oe.value && (Ae += Oe.key + "=" + Oe.value + (Be < Fe.length - 1 ? "&" : "")), Ae;
                    }, "?");
                    Ke.href = ee.config.FEEDBACK_URL + "/" + Te + (this._map._hash ? this._map._hash.getHashString(!0) : ""), Ke.rel = "noopener nofollow", this._setElementTitle(Ke, "MapFeedback");
                  }
                }, hc.prototype._updateData = function(Ke) {
                  !Ke || Ke.sourceDataType !== "metadata" && Ke.sourceDataType !== "visibility" && Ke.dataType !== "style" || (this._updateAttributions(), this._updateEditLink());
                }, hc.prototype._updateAttributions = function() {
                  if (this._map.style) {
                    var Ke = [];
                    if (this.options.customAttribution && (Array.isArray(this.options.customAttribution) ? Ke = Ke.concat(this.options.customAttribution.map(function(nt) {
                      return typeof nt != "string" ? "" : nt;
                    })) : typeof this.options.customAttribution == "string" && Ke.push(this.options.customAttribution)), this._map.style.stylesheet) {
                      var Fe = this._map.style.stylesheet;
                      this.styleOwner = Fe.owner, this.styleId = Fe.id;
                    }
                    var Te = this._map.style.sourceCaches;
                    for (var Ae in Te) {
                      var Oe = Te[Ae];
                      if (Oe.used) {
                        var Be = Oe.getSource();
                        Be.attribution && Ke.indexOf(Be.attribution) < 0 && Ke.push(Be.attribution);
                      }
                    }
                    Ke.sort(function(nt, ht) {
                      return nt.length - ht.length;
                    });
                    var Ue = (Ke = Ke.filter(function(nt, ht) {
                      for (var Et = ht + 1; Et < Ke.length; Et++) if (Ke[Et].indexOf(nt) >= 0) return !1;
                      return !0;
                    })).join(" | ");
                    Ue !== this._attribHTML && (this._attribHTML = Ue, Ke.length ? (this._innerContainer.innerHTML = Ue, this._container.classList.remove("mapboxgl-attrib-empty")) : this._container.classList.add("mapboxgl-attrib-empty"), this._editLink = null);
                  }
                }, hc.prototype._updateCompact = function() {
                  this._map.getCanvasContainer().offsetWidth <= 640 ? this._container.classList.add("mapboxgl-compact") : this._container.classList.remove("mapboxgl-compact", "mapboxgl-compact-show");
                };
                var Vc = function() {
                  ee.bindAll(["_updateLogo"], this), ee.bindAll(["_updateCompact"], this);
                };
                Vc.prototype.onAdd = function(Ke) {
                  this._map = Ke, this._container = oe.create("div", "mapboxgl-ctrl");
                  var Fe = oe.create("a", "mapboxgl-ctrl-logo");
                  return Fe.target = "_blank", Fe.rel = "noopener nofollow", Fe.href = "https://www.mapbox.com/", Fe.setAttribute("aria-label", this._map._getUIString("LogoControl.Title")), Fe.setAttribute("rel", "noopener nofollow"), this._container.appendChild(Fe), this._container.style.display = "none", this._map.on("sourcedata", this._updateLogo), this._updateLogo(), this._map.on("resize", this._updateCompact), this._updateCompact(), this._container;
                }, Vc.prototype.onRemove = function() {
                  oe.remove(this._container), this._map.off("sourcedata", this._updateLogo), this._map.off("resize", this._updateCompact);
                }, Vc.prototype.getDefaultPosition = function() {
                  return "bottom-left";
                }, Vc.prototype._updateLogo = function(Ke) {
                  Ke && Ke.sourceDataType !== "metadata" || (this._container.style.display = this._logoRequired() ? "block" : "none");
                }, Vc.prototype._logoRequired = function() {
                  if (this._map.style) {
                    var Ke = this._map.style.sourceCaches;
                    for (var Fe in Ke) if (Ke[Fe].getSource().mapbox_logo) return !0;
                    return !1;
                  }
                }, Vc.prototype._updateCompact = function() {
                  var Ke = this._container.children;
                  if (Ke.length) {
                    var Fe = Ke[0];
                    this._map.getCanvasContainer().offsetWidth < 250 ? Fe.classList.add("mapboxgl-compact") : Fe.classList.remove("mapboxgl-compact");
                  }
                };
                var Rc = function() {
                  this._queue = [], this._id = 0, this._cleared = !1, this._currentlyRunning = !1;
                };
                Rc.prototype.add = function(Ke) {
                  var Fe = ++this._id;
                  return this._queue.push({ callback: Ke, id: Fe, cancelled: !1 }), Fe;
                }, Rc.prototype.remove = function(Ke) {
                  for (var Fe = this._currentlyRunning, Te = 0, Ae = Fe ? this._queue.concat(Fe) : this._queue; Te < Ae.length; Te += 1) {
                    var Oe = Ae[Te];
                    if (Oe.id === Ke) return void (Oe.cancelled = !0);
                  }
                }, Rc.prototype.run = function(Ke) {
                  Ke === void 0 && (Ke = 0);
                  var Fe = this._currentlyRunning = this._queue;
                  this._queue = [];
                  for (var Te = 0, Ae = Fe; Te < Ae.length; Te += 1) {
                    var Oe = Ae[Te];
                    if (!Oe.cancelled && (Oe.callback(Ke), this._cleared)) break;
                  }
                  this._cleared = !1, this._currentlyRunning = !1;
                }, Rc.prototype.clear = function() {
                  this._currentlyRunning && (this._cleared = !0), this._queue = [];
                };
                var Zo = { "AttributionControl.ToggleAttribution": "Toggle attribution", "AttributionControl.MapFeedback": "Map feedback", "FullscreenControl.Enter": "Enter fullscreen", "FullscreenControl.Exit": "Exit fullscreen", "GeolocateControl.FindMyLocation": "Find my location", "GeolocateControl.LocationNotAvailable": "Location not available", "LogoControl.Title": "Mapbox logo", "NavigationControl.ResetBearing": "Reset bearing to north", "NavigationControl.ZoomIn": "Zoom in", "NavigationControl.ZoomOut": "Zoom out", "ScaleControl.Feet": "ft", "ScaleControl.Meters": "m", "ScaleControl.Kilometers": "km", "ScaleControl.Miles": "mi", "ScaleControl.NauticalMiles": "nm" }, Ou = ee.window.HTMLImageElement, Uc = ee.window.HTMLElement, El = ee.window.ImageBitmap, eu = 60, Wc = { center: [0, 0], zoom: 0, bearing: 0, pitch: 0, minZoom: -2, maxZoom: 22, minPitch: 0, maxPitch: eu, interactive: !0, scrollZoom: !0, boxZoom: !0, dragRotate: !0, dragPan: !0, keyboard: !0, doubleClickZoom: !0, touchZoomRotate: !0, touchPitch: !0, bearingSnap: 7, clickTolerance: 3, pitchWithRotate: !0, hash: !1, attributionControl: !0, failIfMajorPerformanceCaveat: !1, preserveDrawingBuffer: !1, trackResize: !0, renderWorldCopies: !0, refreshExpiredTiles: !0, maxTileCacheSize: null, localIdeographFontFamily: "sans-serif", transformRequest: null, accessToken: null, fadeDuration: 300, crossSourceCollisions: !0 }, hu = function(Ke) {
                  function Fe(Ae) {
                    var Oe = this;
                    if ((Ae = ee.extend({}, Wc, Ae)).minZoom != null && Ae.maxZoom != null && Ae.minZoom > Ae.maxZoom) throw new Error("maxZoom must be greater than or equal to minZoom");
                    if (Ae.minPitch != null && Ae.maxPitch != null && Ae.minPitch > Ae.maxPitch) throw new Error("maxPitch must be greater than or equal to minPitch");
                    if (Ae.minPitch != null && Ae.minPitch < 0) throw new Error("minPitch must be greater than or equal to 0");
                    if (Ae.maxPitch != null && Ae.maxPitch > eu) throw new Error("maxPitch must be less than or equal to 60");
                    var Be = new Ql(Ae.minZoom, Ae.maxZoom, Ae.minPitch, Ae.maxPitch, Ae.renderWorldCopies);
                    if (Ke.call(this, Be, Ae), this._interactive = Ae.interactive, this._maxTileCacheSize = Ae.maxTileCacheSize, this._failIfMajorPerformanceCaveat = Ae.failIfMajorPerformanceCaveat, this._preserveDrawingBuffer = Ae.preserveDrawingBuffer, this._antialias = Ae.antialias, this._trackResize = Ae.trackResize, this._bearingSnap = Ae.bearingSnap, this._refreshExpiredTiles = Ae.refreshExpiredTiles, this._fadeDuration = Ae.fadeDuration, this._crossSourceCollisions = Ae.crossSourceCollisions, this._crossFadingFactor = 1, this._collectResourceTiming = Ae.collectResourceTiming, this._renderTaskQueue = new Rc(), this._controls = [], this._mapId = ee.uniqueId(), this._locale = ee.extend({}, Zo, Ae.locale), this._clickTolerance = Ae.clickTolerance, this._requestManager = new ee.RequestManager(Ae.transformRequest, Ae.accessToken), typeof Ae.container == "string") {
                      if (this._container = ee.window.document.getElementById(Ae.container), !this._container) throw new Error("Container '" + Ae.container + "' not found.");
                    } else {
                      if (!(Ae.container instanceof Uc)) throw new Error("Invalid type: 'container' must be a String or HTMLElement.");
                      this._container = Ae.container;
                    }
                    if (Ae.maxBounds && this.setMaxBounds(Ae.maxBounds), ee.bindAll(["_onWindowOnline", "_onWindowResize", "_onMapScroll", "_contextLost", "_contextRestored"], this), this._setupContainer(), this._setupPainter(), this.painter === void 0) throw new Error("Failed to initialize WebGL.");
                    this.on("move", function() {
                      return Oe._update(!1);
                    }), this.on("moveend", function() {
                      return Oe._update(!1);
                    }), this.on("zoom", function() {
                      return Oe._update(!0);
                    }), ee.window !== void 0 && (ee.window.addEventListener("online", this._onWindowOnline, !1), ee.window.addEventListener("resize", this._onWindowResize, !1), ee.window.addEventListener("orientationchange", this._onWindowResize, !1)), this.handlers = new Tl(this, Ae);
                    var Ue = typeof Ae.hash == "string" && Ae.hash || void 0;
                    this._hash = Ae.hash && new h1(Ue).addTo(this), this._hash && this._hash._onHashChange() || (this.jumpTo({ center: Ae.center, zoom: Ae.zoom, bearing: Ae.bearing, pitch: Ae.pitch }), Ae.bounds && (this.resize(), this.fitBounds(Ae.bounds, ee.extend({}, Ae.fitBoundsOptions, { duration: 0 })))), this.resize(), this._localIdeographFontFamily = Ae.localIdeographFontFamily, Ae.style && this.setStyle(Ae.style, { localIdeographFontFamily: Ae.localIdeographFontFamily }), Ae.attributionControl && this.addControl(new hc({ customAttribution: Ae.customAttribution })), this.addControl(new Vc(), Ae.logoPosition), this.on("style.load", function() {
                      Oe.transform.unmodified && Oe.jumpTo(Oe.style.stylesheet);
                    }), this.on("data", function(nt) {
                      Oe._update(nt.dataType === "style"), Oe.fire(new ee.Event(nt.dataType + "data", nt));
                    }), this.on("dataloading", function(nt) {
                      Oe.fire(new ee.Event(nt.dataType + "dataloading", nt));
                    });
                  }
                  Fe.__proto__ = Ke, Fe.prototype = Object.create(Ke.prototype), Fe.prototype.constructor = Fe;
                  var Te = { showTileBoundaries: { configurable: !0 }, showPadding: { configurable: !0 }, showCollisionBoxes: { configurable: !0 }, showOverdrawInspector: { configurable: !0 }, repaint: { configurable: !0 }, vertices: { configurable: !0 }, version: { configurable: !0 } };
                  return Fe.prototype._getMapId = function() {
                    return this._mapId;
                  }, Fe.prototype.addControl = function(Ae, Oe) {
                    if (Oe === void 0 && (Oe = Ae.getDefaultPosition ? Ae.getDefaultPosition() : "top-right"), !Ae || !Ae.onAdd) return this.fire(new ee.ErrorEvent(new Error("Invalid argument to map.addControl(). Argument must be a control with onAdd and onRemove methods.")));
                    var Be = Ae.onAdd(this);
                    this._controls.push(Ae);
                    var Ue = this._controlPositions[Oe];
                    return Oe.indexOf("bottom") !== -1 ? Ue.insertBefore(Be, Ue.firstChild) : Ue.appendChild(Be), this;
                  }, Fe.prototype.removeControl = function(Ae) {
                    if (!Ae || !Ae.onRemove) return this.fire(new ee.ErrorEvent(new Error("Invalid argument to map.removeControl(). Argument must be a control with onAdd and onRemove methods.")));
                    var Oe = this._controls.indexOf(Ae);
                    return Oe > -1 && this._controls.splice(Oe, 1), Ae.onRemove(this), this;
                  }, Fe.prototype.hasControl = function(Ae) {
                    return this._controls.indexOf(Ae) > -1;
                  }, Fe.prototype.resize = function(Ae) {
                    var Oe = this._containerDimensions(), Be = Oe[0], Ue = Oe[1];
                    this._resizeCanvas(Be, Ue), this.transform.resize(Be, Ue), this.painter.resize(Be, Ue);
                    var nt = !this._moving;
                    return nt && (this.stop(), this.fire(new ee.Event("movestart", Ae)).fire(new ee.Event("move", Ae))), this.fire(new ee.Event("resize", Ae)), nt && this.fire(new ee.Event("moveend", Ae)), this;
                  }, Fe.prototype.getBounds = function() {
                    return this.transform.getBounds();
                  }, Fe.prototype.getMaxBounds = function() {
                    return this.transform.getMaxBounds();
                  }, Fe.prototype.setMaxBounds = function(Ae) {
                    return this.transform.setMaxBounds(ee.LngLatBounds.convert(Ae)), this._update();
                  }, Fe.prototype.setMinZoom = function(Ae) {
                    if ((Ae = Ae ?? -2) >= -2 && Ae <= this.transform.maxZoom) return this.transform.minZoom = Ae, this._update(), this.getZoom() < Ae && this.setZoom(Ae), this;
                    throw new Error("minZoom must be between -2 and the current maxZoom, inclusive");
                  }, Fe.prototype.getMinZoom = function() {
                    return this.transform.minZoom;
                  }, Fe.prototype.setMaxZoom = function(Ae) {
                    if ((Ae = Ae ?? 22) >= this.transform.minZoom) return this.transform.maxZoom = Ae, this._update(), this.getZoom() > Ae && this.setZoom(Ae), this;
                    throw new Error("maxZoom must be greater than the current minZoom");
                  }, Fe.prototype.getMaxZoom = function() {
                    return this.transform.maxZoom;
                  }, Fe.prototype.setMinPitch = function(Ae) {
                    if ((Ae = Ae ?? 0) < 0) throw new Error("minPitch must be greater than or equal to 0");
                    if (Ae >= 0 && Ae <= this.transform.maxPitch) return this.transform.minPitch = Ae, this._update(), this.getPitch() < Ae && this.setPitch(Ae), this;
                    throw new Error("minPitch must be between 0 and the current maxPitch, inclusive");
                  }, Fe.prototype.getMinPitch = function() {
                    return this.transform.minPitch;
                  }, Fe.prototype.setMaxPitch = function(Ae) {
                    if ((Ae = Ae ?? eu) > eu) throw new Error("maxPitch must be less than or equal to 60");
                    if (Ae >= this.transform.minPitch) return this.transform.maxPitch = Ae, this._update(), this.getPitch() > Ae && this.setPitch(Ae), this;
                    throw new Error("maxPitch must be greater than the current minPitch");
                  }, Fe.prototype.getMaxPitch = function() {
                    return this.transform.maxPitch;
                  }, Fe.prototype.getRenderWorldCopies = function() {
                    return this.transform.renderWorldCopies;
                  }, Fe.prototype.setRenderWorldCopies = function(Ae) {
                    return this.transform.renderWorldCopies = Ae, this._update();
                  }, Fe.prototype.project = function(Ae) {
                    return this.transform.locationPoint(ee.LngLat.convert(Ae));
                  }, Fe.prototype.unproject = function(Ae) {
                    return this.transform.pointLocation(ee.Point.convert(Ae));
                  }, Fe.prototype.isMoving = function() {
                    return this._moving || this.handlers.isMoving();
                  }, Fe.prototype.isZooming = function() {
                    return this._zooming || this.handlers.isZooming();
                  }, Fe.prototype.isRotating = function() {
                    return this._rotating || this.handlers.isRotating();
                  }, Fe.prototype._createDelegatedListener = function(Ae, Oe, Be) {
                    var Ue, nt = this;
                    if (Ae === "mouseenter" || Ae === "mouseover") {
                      var ht = !1;
                      return { layer: Oe, listener: Be, delegates: { mousemove: function(Mt) {
                        var qt = nt.getLayer(Oe) ? nt.queryRenderedFeatures(Mt.point, { layers: [Oe] }) : [];
                        qt.length ? ht || (ht = !0, Be.call(nt, new Yl(Ae, nt, Mt.originalEvent, { features: qt }))) : ht = !1;
                      }, mouseout: function() {
                        ht = !1;
                      } } };
                    }
                    if (Ae === "mouseleave" || Ae === "mouseout") {
                      var Et = !1;
                      return { layer: Oe, listener: Be, delegates: { mousemove: function(Mt) {
                        (nt.getLayer(Oe) ? nt.queryRenderedFeatures(Mt.point, { layers: [Oe] }) : []).length ? Et = !0 : Et && (Et = !1, Be.call(nt, new Yl(Ae, nt, Mt.originalEvent)));
                      }, mouseout: function(Mt) {
                        Et && (Et = !1, Be.call(nt, new Yl(Ae, nt, Mt.originalEvent)));
                      } } };
                    }
                    return { layer: Oe, listener: Be, delegates: (Ue = {}, Ue[Ae] = function(Mt) {
                      var qt = nt.getLayer(Oe) ? nt.queryRenderedFeatures(Mt.point, { layers: [Oe] }) : [];
                      qt.length && (Mt.features = qt, Be.call(nt, Mt), delete Mt.features);
                    }, Ue) };
                  }, Fe.prototype.on = function(Ae, Oe, Be) {
                    if (Be === void 0) return Ke.prototype.on.call(this, Ae, Oe);
                    var Ue = this._createDelegatedListener(Ae, Oe, Be);
                    for (var nt in this._delegatedListeners = this._delegatedListeners || {}, this._delegatedListeners[Ae] = this._delegatedListeners[Ae] || [], this._delegatedListeners[Ae].push(Ue), Ue.delegates) this.on(nt, Ue.delegates[nt]);
                    return this;
                  }, Fe.prototype.once = function(Ae, Oe, Be) {
                    if (Be === void 0) return Ke.prototype.once.call(this, Ae, Oe);
                    var Ue = this._createDelegatedListener(Ae, Oe, Be);
                    for (var nt in Ue.delegates) this.once(nt, Ue.delegates[nt]);
                    return this;
                  }, Fe.prototype.off = function(Ae, Oe, Be) {
                    var Ue = this;
                    return Be === void 0 ? Ke.prototype.off.call(this, Ae, Oe) : (this._delegatedListeners && this._delegatedListeners[Ae] && function(nt) {
                      for (var ht = nt[Ae], Et = 0; Et < ht.length; Et++) {
                        var Mt = ht[Et];
                        if (Mt.layer === Oe && Mt.listener === Be) {
                          for (var qt in Mt.delegates) Ue.off(qt, Mt.delegates[qt]);
                          return ht.splice(Et, 1), Ue;
                        }
                      }
                    }(this._delegatedListeners), this);
                  }, Fe.prototype.queryRenderedFeatures = function(Ae, Oe) {
                    if (!this.style) return [];
                    var Be;
                    if (Oe !== void 0 || Ae === void 0 || Ae instanceof ee.Point || Array.isArray(Ae) || (Oe = Ae, Ae = void 0), Oe = Oe || {}, (Ae = Ae || [[0, 0], [this.transform.width, this.transform.height]]) instanceof ee.Point || typeof Ae[0] == "number") Be = [ee.Point.convert(Ae)];
                    else {
                      var Ue = ee.Point.convert(Ae[0]), nt = ee.Point.convert(Ae[1]);
                      Be = [Ue, new ee.Point(nt.x, Ue.y), nt, new ee.Point(Ue.x, nt.y), Ue];
                    }
                    return this.style.queryRenderedFeatures(Be, Oe, this.transform);
                  }, Fe.prototype.querySourceFeatures = function(Ae, Oe) {
                    return this.style.querySourceFeatures(Ae, Oe);
                  }, Fe.prototype.setStyle = function(Ae, Oe) {
                    return (Oe = ee.extend({}, { localIdeographFontFamily: this._localIdeographFontFamily }, Oe)).diff !== !1 && Oe.localIdeographFontFamily === this._localIdeographFontFamily && this.style && Ae ? (this._diffStyle(Ae, Oe), this) : (this._localIdeographFontFamily = Oe.localIdeographFontFamily, this._updateStyle(Ae, Oe));
                  }, Fe.prototype._getUIString = function(Ae) {
                    var Oe = this._locale[Ae];
                    if (Oe == null) throw new Error("Missing UI string '" + Ae + "'");
                    return Oe;
                  }, Fe.prototype._updateStyle = function(Ae, Oe) {
                    return this.style && (this.style.setEventedParent(null), this.style._remove()), Ae ? (this.style = new ki(this, Oe || {}), this.style.setEventedParent(this, { style: this.style }), typeof Ae == "string" ? this.style.loadURL(Ae) : this.style.loadJSON(Ae), this) : (delete this.style, this);
                  }, Fe.prototype._lazyInitEmptyStyle = function() {
                    this.style || (this.style = new ki(this, {}), this.style.setEventedParent(this, { style: this.style }), this.style.loadEmpty());
                  }, Fe.prototype._diffStyle = function(Ae, Oe) {
                    var Be = this;
                    if (typeof Ae == "string") {
                      var Ue = this._requestManager.normalizeStyleURL(Ae), nt = this._requestManager.transformRequest(Ue, ee.ResourceType.Style);
                      ee.getJSON(nt, function(ht, Et) {
                        ht ? Be.fire(new ee.ErrorEvent(ht)) : Et && Be._updateDiff(Et, Oe);
                      });
                    } else typeof Ae == "object" && this._updateDiff(Ae, Oe);
                  }, Fe.prototype._updateDiff = function(Ae, Oe) {
                    try {
                      this.style.setState(Ae) && this._update(!0);
                    } catch (Be) {
                      ee.warnOnce("Unable to perform style diff: " + (Be.message || Be.error || Be) + ".  Rebuilding the style from scratch."), this._updateStyle(Ae, Oe);
                    }
                  }, Fe.prototype.getStyle = function() {
                    if (this.style) return this.style.serialize();
                  }, Fe.prototype.isStyleLoaded = function() {
                    return this.style ? this.style.loaded() : ee.warnOnce("There is no style added to the map.");
                  }, Fe.prototype.addSource = function(Ae, Oe) {
                    return this._lazyInitEmptyStyle(), this.style.addSource(Ae, Oe), this._update(!0);
                  }, Fe.prototype.isSourceLoaded = function(Ae) {
                    var Oe = this.style && this.style.sourceCaches[Ae];
                    if (Oe !== void 0) return Oe.loaded();
                    this.fire(new ee.ErrorEvent(new Error("There is no source with ID '" + Ae + "'")));
                  }, Fe.prototype.areTilesLoaded = function() {
                    var Ae = this.style && this.style.sourceCaches;
                    for (var Oe in Ae) {
                      var Be = Ae[Oe]._tiles;
                      for (var Ue in Be) {
                        var nt = Be[Ue];
                        if (nt.state !== "loaded" && nt.state !== "errored") return !1;
                      }
                    }
                    return !0;
                  }, Fe.prototype.addSourceType = function(Ae, Oe, Be) {
                    return this._lazyInitEmptyStyle(), this.style.addSourceType(Ae, Oe, Be);
                  }, Fe.prototype.removeSource = function(Ae) {
                    return this.style.removeSource(Ae), this._update(!0);
                  }, Fe.prototype.getSource = function(Ae) {
                    return this.style.getSource(Ae);
                  }, Fe.prototype.addImage = function(Ae, Oe, Be) {
                    Be === void 0 && (Be = {});
                    var Ue = Be.pixelRatio;
                    Ue === void 0 && (Ue = 1);
                    var nt = Be.sdf;
                    nt === void 0 && (nt = !1);
                    var ht = Be.stretchX, Et = Be.stretchY, Mt = Be.content;
                    if (this._lazyInitEmptyStyle(), Oe instanceof Ou || El && Oe instanceof El) {
                      var qt = ee.browser.getImageData(Oe), tn = qt.width, xn = qt.height, An = qt.data;
                      this.style.addImage(Ae, { data: new ee.RGBAImage({ width: tn, height: xn }, An), pixelRatio: Ue, stretchX: ht, stretchY: Et, content: Mt, sdf: nt, version: 0 });
                    } else {
                      if (Oe.width === void 0 || Oe.height === void 0) return this.fire(new ee.ErrorEvent(new Error("Invalid arguments to map.addImage(). The second argument must be an `HTMLImageElement`, `ImageData`, `ImageBitmap`, or object with `width`, `height`, and `data` properties with the same format as `ImageData`")));
                      var On = Oe.width, Bn = Oe.height, qn = Oe.data, Vn = Oe;
                      this.style.addImage(Ae, { data: new ee.RGBAImage({ width: On, height: Bn }, new Uint8Array(qn)), pixelRatio: Ue, stretchX: ht, stretchY: Et, content: Mt, sdf: nt, version: 0, userImage: Vn }), Vn.onAdd && Vn.onAdd(this, Ae);
                    }
                  }, Fe.prototype.updateImage = function(Ae, Oe) {
                    var Be = this.style.getImage(Ae);
                    if (!Be) return this.fire(new ee.ErrorEvent(new Error("The map has no image with that id. If you are adding a new image use `map.addImage(...)` instead.")));
                    var Ue = Oe instanceof Ou || El && Oe instanceof El ? ee.browser.getImageData(Oe) : Oe, nt = Ue.width, ht = Ue.height, Et = Ue.data;
                    if (nt === void 0 || ht === void 0) return this.fire(new ee.ErrorEvent(new Error("Invalid arguments to map.updateImage(). The second argument must be an `HTMLImageElement`, `ImageData`, `ImageBitmap`, or object with `width`, `height`, and `data` properties with the same format as `ImageData`")));
                    if (nt !== Be.data.width || ht !== Be.data.height) return this.fire(new ee.ErrorEvent(new Error("The width and height of the updated image must be that same as the previous version of the image")));
                    var Mt = !(Oe instanceof Ou || El && Oe instanceof El);
                    Be.data.replace(Et, Mt), this.style.updateImage(Ae, Be);
                  }, Fe.prototype.hasImage = function(Ae) {
                    return Ae ? !!this.style.getImage(Ae) : (this.fire(new ee.ErrorEvent(new Error("Missing required image id"))), !1);
                  }, Fe.prototype.removeImage = function(Ae) {
                    this.style.removeImage(Ae);
                  }, Fe.prototype.loadImage = function(Ae, Oe) {
                    ee.getImage(this._requestManager.transformRequest(Ae, ee.ResourceType.Image), Oe);
                  }, Fe.prototype.listImages = function() {
                    return this.style.listImages();
                  }, Fe.prototype.addLayer = function(Ae, Oe) {
                    return this._lazyInitEmptyStyle(), this.style.addLayer(Ae, Oe), this._update(!0);
                  }, Fe.prototype.moveLayer = function(Ae, Oe) {
                    return this.style.moveLayer(Ae, Oe), this._update(!0);
                  }, Fe.prototype.removeLayer = function(Ae) {
                    return this.style.removeLayer(Ae), this._update(!0);
                  }, Fe.prototype.getLayer = function(Ae) {
                    return this.style.getLayer(Ae);
                  }, Fe.prototype.setLayerZoomRange = function(Ae, Oe, Be) {
                    return this.style.setLayerZoomRange(Ae, Oe, Be), this._update(!0);
                  }, Fe.prototype.setFilter = function(Ae, Oe, Be) {
                    return Be === void 0 && (Be = {}), this.style.setFilter(Ae, Oe, Be), this._update(!0);
                  }, Fe.prototype.getFilter = function(Ae) {
                    return this.style.getFilter(Ae);
                  }, Fe.prototype.setPaintProperty = function(Ae, Oe, Be, Ue) {
                    return Ue === void 0 && (Ue = {}), this.style.setPaintProperty(Ae, Oe, Be, Ue), this._update(!0);
                  }, Fe.prototype.getPaintProperty = function(Ae, Oe) {
                    return this.style.getPaintProperty(Ae, Oe);
                  }, Fe.prototype.setLayoutProperty = function(Ae, Oe, Be, Ue) {
                    return Ue === void 0 && (Ue = {}), this.style.setLayoutProperty(Ae, Oe, Be, Ue), this._update(!0);
                  }, Fe.prototype.getLayoutProperty = function(Ae, Oe) {
                    return this.style.getLayoutProperty(Ae, Oe);
                  }, Fe.prototype.setLight = function(Ae, Oe) {
                    return Oe === void 0 && (Oe = {}), this._lazyInitEmptyStyle(), this.style.setLight(Ae, Oe), this._update(!0);
                  }, Fe.prototype.getLight = function() {
                    return this.style.getLight();
                  }, Fe.prototype.setFeatureState = function(Ae, Oe) {
                    return this.style.setFeatureState(Ae, Oe), this._update();
                  }, Fe.prototype.removeFeatureState = function(Ae, Oe) {
                    return this.style.removeFeatureState(Ae, Oe), this._update();
                  }, Fe.prototype.getFeatureState = function(Ae) {
                    return this.style.getFeatureState(Ae);
                  }, Fe.prototype.getContainer = function() {
                    return this._container;
                  }, Fe.prototype.getCanvasContainer = function() {
                    return this._canvasContainer;
                  }, Fe.prototype.getCanvas = function() {
                    return this._canvas;
                  }, Fe.prototype._containerDimensions = function() {
                    var Ae = 0, Oe = 0;
                    return this._container && (Ae = this._container.clientWidth || 400, Oe = this._container.clientHeight || 300), [Ae, Oe];
                  }, Fe.prototype._detectMissingCSS = function() {
                    ee.window.getComputedStyle(this._missingCSSCanary).getPropertyValue("background-color") !== "rgb(250, 128, 114)" && ee.warnOnce("This page appears to be missing CSS declarations for Mapbox GL JS, which may cause the map to display incorrectly. Please ensure your page includes mapbox-gl.css, as described in https://www.mapbox.com/mapbox-gl-js/api/.");
                  }, Fe.prototype._setupContainer = function() {
                    var Ae = this._container;
                    Ae.classList.add("mapboxgl-map"), (this._missingCSSCanary = oe.create("div", "mapboxgl-canary", Ae)).style.visibility = "hidden", this._detectMissingCSS();
                    var Oe = this._canvasContainer = oe.create("div", "mapboxgl-canvas-container", Ae);
                    this._interactive && Oe.classList.add("mapboxgl-interactive"), this._canvas = oe.create("canvas", "mapboxgl-canvas", Oe), this._canvas.addEventListener("webglcontextlost", this._contextLost, !1), this._canvas.addEventListener("webglcontextrestored", this._contextRestored, !1), this._canvas.setAttribute("tabindex", "0"), this._canvas.setAttribute("aria-label", "Map"), this._canvas.setAttribute("role", "region");
                    var Be = this._containerDimensions();
                    this._resizeCanvas(Be[0], Be[1]);
                    var Ue = this._controlContainer = oe.create("div", "mapboxgl-control-container", Ae), nt = this._controlPositions = {};
                    ["top-left", "top-right", "bottom-left", "bottom-right"].forEach(function(ht) {
                      nt[ht] = oe.create("div", "mapboxgl-ctrl-" + ht, Ue);
                    }), this._container.addEventListener("scroll", this._onMapScroll, !1);
                  }, Fe.prototype._resizeCanvas = function(Ae, Oe) {
                    var Be = ee.browser.devicePixelRatio || 1;
                    this._canvas.width = Be * Ae, this._canvas.height = Be * Oe, this._canvas.style.width = Ae + "px", this._canvas.style.height = Oe + "px";
                  }, Fe.prototype._setupPainter = function() {
                    var Ae = ee.extend({}, ae.webGLContextAttributes, { failIfMajorPerformanceCaveat: this._failIfMajorPerformanceCaveat, preserveDrawingBuffer: this._preserveDrawingBuffer, antialias: this._antialias || !1 }), Oe = this._canvas.getContext("webgl", Ae) || this._canvas.getContext("experimental-webgl", Ae);
                    Oe ? (this.painter = new Fo(Oe, this.transform), ee.webpSupported.testSupport(Oe)) : this.fire(new ee.ErrorEvent(new Error("Failed to initialize WebGL")));
                  }, Fe.prototype._contextLost = function(Ae) {
                    Ae.preventDefault(), this._frame && (this._frame.cancel(), this._frame = null), this.fire(new ee.Event("webglcontextlost", { originalEvent: Ae }));
                  }, Fe.prototype._contextRestored = function(Ae) {
                    this._setupPainter(), this.resize(), this._update(), this.fire(new ee.Event("webglcontextrestored", { originalEvent: Ae }));
                  }, Fe.prototype._onMapScroll = function(Ae) {
                    if (Ae.target === this._container) return this._container.scrollTop = 0, this._container.scrollLeft = 0, !1;
                  }, Fe.prototype.loaded = function() {
                    return !this._styleDirty && !this._sourcesDirty && !!this.style && this.style.loaded();
                  }, Fe.prototype._update = function(Ae) {
                    return this.style ? (this._styleDirty = this._styleDirty || Ae, this._sourcesDirty = !0, this.triggerRepaint(), this) : this;
                  }, Fe.prototype._requestRenderFrame = function(Ae) {
                    return this._update(), this._renderTaskQueue.add(Ae);
                  }, Fe.prototype._cancelRenderFrame = function(Ae) {
                    this._renderTaskQueue.remove(Ae);
                  }, Fe.prototype._render = function(Ae) {
                    var Oe, Be = this, Ue = 0, nt = this.painter.context.extTimerQuery;
                    if (this.listens("gpu-timing-frame") && (Oe = nt.createQueryEXT(), nt.beginQueryEXT(nt.TIME_ELAPSED_EXT, Oe), Ue = ee.browser.now()), this.painter.context.setDirty(), this.painter.setBaseState(), this._renderTaskQueue.run(Ae), !this._removed) {
                      var ht = !1;
                      if (this.style && this._styleDirty) {
                        this._styleDirty = !1;
                        var Et = this.transform.zoom, Mt = ee.browser.now();
                        this.style.zoomHistory.update(Et, Mt);
                        var qt = new ee.EvaluationParameters(Et, { now: Mt, fadeDuration: this._fadeDuration, zoomHistory: this.style.zoomHistory, transition: this.style.getTransition() }), tn = qt.crossFadingFactor();
                        tn === 1 && tn === this._crossFadingFactor || (ht = !0, this._crossFadingFactor = tn), this.style.update(qt);
                      }
                      if (this.style && this._sourcesDirty && (this._sourcesDirty = !1, this.style._updateSources(this.transform)), this._placementDirty = this.style && this.style._updatePlacement(this.painter.transform, this.showCollisionBoxes, this._fadeDuration, this._crossSourceCollisions), this.painter.render(this.style, { showTileBoundaries: this.showTileBoundaries, showOverdrawInspector: this._showOverdrawInspector, rotating: this.isRotating(), zooming: this.isZooming(), moving: this.isMoving(), fadeDuration: this._fadeDuration, showPadding: this.showPadding, gpuTiming: !!this.listens("gpu-timing-layer") }), this.fire(new ee.Event("render")), this.loaded() && !this._loaded && (this._loaded = !0, this.fire(new ee.Event("load"))), this.style && (this.style.hasTransitions() || ht) && (this._styleDirty = !0), this.style && !this._placementDirty && this.style._releaseSymbolFadeTiles(), this.listens("gpu-timing-frame")) {
                        var xn = ee.browser.now() - Ue;
                        nt.endQueryEXT(nt.TIME_ELAPSED_EXT, Oe), setTimeout(function() {
                          var Bn = nt.getQueryObjectEXT(Oe, nt.QUERY_RESULT_EXT) / 1e6;
                          nt.deleteQueryEXT(Oe), Be.fire(new ee.Event("gpu-timing-frame", { cpuTime: xn, gpuTime: Bn }));
                        }, 50);
                      }
                      if (this.listens("gpu-timing-layer")) {
                        var An = this.painter.collectGpuTimers();
                        setTimeout(function() {
                          var Bn = Be.painter.queryGpuTimers(An);
                          Be.fire(new ee.Event("gpu-timing-layer", { layerTimes: Bn }));
                        }, 50);
                      }
                      var On = this._sourcesDirty || this._styleDirty || this._placementDirty;
                      return On || this._repaint ? this.triggerRepaint() : !this.isMoving() && this.loaded() && this.fire(new ee.Event("idle")), !this._loaded || this._fullyLoaded || On || (this._fullyLoaded = !0), this;
                    }
                  }, Fe.prototype.remove = function() {
                    this._hash && this._hash.remove();
                    for (var Ae = 0, Oe = this._controls; Ae < Oe.length; Ae += 1) Oe[Ae].onRemove(this);
                    this._controls = [], this._frame && (this._frame.cancel(), this._frame = null), this._renderTaskQueue.clear(), this.painter.destroy(), this.handlers.destroy(), delete this.handlers, this.setStyle(null), ee.window !== void 0 && (ee.window.removeEventListener("resize", this._onWindowResize, !1), ee.window.removeEventListener("orientationchange", this._onWindowResize, !1), ee.window.removeEventListener("online", this._onWindowOnline, !1));
                    var Be = this.painter.context.gl.getExtension("WEBGL_lose_context");
                    Be && Be.loseContext && Be.loseContext(), gd(this._canvasContainer), gd(this._controlContainer), gd(this._missingCSSCanary), this._container.classList.remove("mapboxgl-map"), this._removed = !0, this.fire(new ee.Event("remove"));
                  }, Fe.prototype.triggerRepaint = function() {
                    var Ae = this;
                    this.style && !this._frame && (this._frame = ee.browser.frame(function(Oe) {
                      Ae._frame = null, Ae._render(Oe);
                    }));
                  }, Fe.prototype._onWindowOnline = function() {
                    this._update();
                  }, Fe.prototype._onWindowResize = function(Ae) {
                    this._trackResize && this.resize({ originalEvent: Ae })._update();
                  }, Te.showTileBoundaries.get = function() {
                    return !!this._showTileBoundaries;
                  }, Te.showTileBoundaries.set = function(Ae) {
                    this._showTileBoundaries !== Ae && (this._showTileBoundaries = Ae, this._update());
                  }, Te.showPadding.get = function() {
                    return !!this._showPadding;
                  }, Te.showPadding.set = function(Ae) {
                    this._showPadding !== Ae && (this._showPadding = Ae, this._update());
                  }, Te.showCollisionBoxes.get = function() {
                    return !!this._showCollisionBoxes;
                  }, Te.showCollisionBoxes.set = function(Ae) {
                    this._showCollisionBoxes !== Ae && (this._showCollisionBoxes = Ae, Ae ? this.style._generateCollisionBoxes() : this._update());
                  }, Te.showOverdrawInspector.get = function() {
                    return !!this._showOverdrawInspector;
                  }, Te.showOverdrawInspector.set = function(Ae) {
                    this._showOverdrawInspector !== Ae && (this._showOverdrawInspector = Ae, this._update());
                  }, Te.repaint.get = function() {
                    return !!this._repaint;
                  }, Te.repaint.set = function(Ae) {
                    this._repaint !== Ae && (this._repaint = Ae, this.triggerRepaint());
                  }, Te.vertices.get = function() {
                    return !!this._vertices;
                  }, Te.vertices.set = function(Ae) {
                    this._vertices = Ae, this._update();
                  }, Fe.prototype._setCacheLimits = function(Ae, Oe) {
                    ee.setCacheLimits(Ae, Oe);
                  }, Te.version.get = function() {
                    return ee.version;
                  }, Object.defineProperties(Fe.prototype, Te), Fe;
                }(sc);
                function gd(Ke) {
                  Ke.parentNode && Ke.parentNode.removeChild(Ke);
                }
                var Jl = { showCompass: !0, showZoom: !0, visualizePitch: !1 }, Yc = function(Ke) {
                  var Fe = this;
                  this.options = ee.extend({}, Jl, Ke), this._container = oe.create("div", "mapboxgl-ctrl mapboxgl-ctrl-group"), this._container.addEventListener("contextmenu", function(Te) {
                    return Te.preventDefault();
                  }), this.options.showZoom && (ee.bindAll(["_setButtonTitle", "_updateZoomButtons"], this), this._zoomInButton = this._createButton("mapboxgl-ctrl-zoom-in", function(Te) {
                    return Fe._map.zoomIn({}, { originalEvent: Te });
                  }), oe.create("span", "mapboxgl-ctrl-icon", this._zoomInButton).setAttribute("aria-hidden", !0), this._zoomOutButton = this._createButton("mapboxgl-ctrl-zoom-out", function(Te) {
                    return Fe._map.zoomOut({}, { originalEvent: Te });
                  }), oe.create("span", "mapboxgl-ctrl-icon", this._zoomOutButton).setAttribute("aria-hidden", !0)), this.options.showCompass && (ee.bindAll(["_rotateCompassArrow"], this), this._compass = this._createButton("mapboxgl-ctrl-compass", function(Te) {
                    Fe.options.visualizePitch ? Fe._map.resetNorthPitch({}, { originalEvent: Te }) : Fe._map.resetNorth({}, { originalEvent: Te });
                  }), this._compassIcon = oe.create("span", "mapboxgl-ctrl-icon", this._compass), this._compassIcon.setAttribute("aria-hidden", !0));
                };
                Yc.prototype._updateZoomButtons = function() {
                  var Ke = this._map.getZoom(), Fe = Ke === this._map.getMaxZoom(), Te = Ke === this._map.getMinZoom();
                  this._zoomInButton.disabled = Fe, this._zoomOutButton.disabled = Te, this._zoomInButton.setAttribute("aria-disabled", Fe.toString()), this._zoomOutButton.setAttribute("aria-disabled", Te.toString());
                }, Yc.prototype._rotateCompassArrow = function() {
                  var Ke = this.options.visualizePitch ? "scale(" + 1 / Math.pow(Math.cos(this._map.transform.pitch * (Math.PI / 180)), 0.5) + ") rotateX(" + this._map.transform.pitch + "deg) rotateZ(" + this._map.transform.angle * (180 / Math.PI) + "deg)" : "rotate(" + this._map.transform.angle * (180 / Math.PI) + "deg)";
                  this._compassIcon.style.transform = Ke;
                }, Yc.prototype.onAdd = function(Ke) {
                  return this._map = Ke, this.options.showZoom && (this._setButtonTitle(this._zoomInButton, "ZoomIn"), this._setButtonTitle(this._zoomOutButton, "ZoomOut"), this._map.on("zoom", this._updateZoomButtons), this._updateZoomButtons()), this.options.showCompass && (this._setButtonTitle(this._compass, "ResetBearing"), this.options.visualizePitch && this._map.on("pitch", this._rotateCompassArrow), this._map.on("rotate", this._rotateCompassArrow), this._rotateCompassArrow(), this._handler = new nc(this._map, this._compass, this.options.visualizePitch)), this._container;
                }, Yc.prototype.onRemove = function() {
                  oe.remove(this._container), this.options.showZoom && this._map.off("zoom", this._updateZoomButtons), this.options.showCompass && (this.options.visualizePitch && this._map.off("pitch", this._rotateCompassArrow), this._map.off("rotate", this._rotateCompassArrow), this._handler.off(), delete this._handler), delete this._map;
                }, Yc.prototype._createButton = function(Ke, Fe) {
                  var Te = oe.create("button", Ke, this._container);
                  return Te.type = "button", Te.addEventListener("click", Fe), Te;
                }, Yc.prototype._setButtonTitle = function(Ke, Fe) {
                  var Te = this._map._getUIString("NavigationControl." + Fe);
                  Ke.title = Te, Ke.setAttribute("aria-label", Te);
                };
                var nc = function(Ke, Fe, Te) {
                  Te === void 0 && (Te = !1), this._clickTolerance = 10, this.element = Fe, this.mouseRotate = new au({ clickTolerance: Ke.dragRotate._mouseRotate._clickTolerance }), this.map = Ke, Te && (this.mousePitch = new Qc({ clickTolerance: Ke.dragRotate._mousePitch._clickTolerance })), ee.bindAll(["mousedown", "mousemove", "mouseup", "touchstart", "touchmove", "touchend", "reset"], this), oe.addEventListener(Fe, "mousedown", this.mousedown), oe.addEventListener(Fe, "touchstart", this.touchstart, { passive: !1 }), oe.addEventListener(Fe, "touchmove", this.touchmove), oe.addEventListener(Fe, "touchend", this.touchend), oe.addEventListener(Fe, "touchcancel", this.reset);
                };
                function Ko(Ke, Fe, Te) {
                  if (Ke = new ee.LngLat(Ke.lng, Ke.lat), Fe) {
                    var Ae = new ee.LngLat(Ke.lng - 360, Ke.lat), Oe = new ee.LngLat(Ke.lng + 360, Ke.lat), Be = Te.locationPoint(Ke).distSqr(Fe);
                    Te.locationPoint(Ae).distSqr(Fe) < Be ? Ke = Ae : Te.locationPoint(Oe).distSqr(Fe) < Be && (Ke = Oe);
                  }
                  for (; Math.abs(Ke.lng - Te.center.lng) > 180; ) {
                    var Ue = Te.locationPoint(Ke);
                    if (Ue.x >= 0 && Ue.y >= 0 && Ue.x <= Te.width && Ue.y <= Te.height) break;
                    Ke.lng > Te.center.lng ? Ke.lng -= 360 : Ke.lng += 360;
                  }
                  return Ke;
                }
                nc.prototype.down = function(Ke, Fe) {
                  this.mouseRotate.mousedown(Ke, Fe), this.mousePitch && this.mousePitch.mousedown(Ke, Fe), oe.disableDrag();
                }, nc.prototype.move = function(Ke, Fe) {
                  var Te = this.map, Ae = this.mouseRotate.mousemoveWindow(Ke, Fe);
                  if (Ae && Ae.bearingDelta && Te.setBearing(Te.getBearing() + Ae.bearingDelta), this.mousePitch) {
                    var Oe = this.mousePitch.mousemoveWindow(Ke, Fe);
                    Oe && Oe.pitchDelta && Te.setPitch(Te.getPitch() + Oe.pitchDelta);
                  }
                }, nc.prototype.off = function() {
                  var Ke = this.element;
                  oe.removeEventListener(Ke, "mousedown", this.mousedown), oe.removeEventListener(Ke, "touchstart", this.touchstart, { passive: !1 }), oe.removeEventListener(Ke, "touchmove", this.touchmove), oe.removeEventListener(Ke, "touchend", this.touchend), oe.removeEventListener(Ke, "touchcancel", this.reset), this.offTemp();
                }, nc.prototype.offTemp = function() {
                  oe.enableDrag(), oe.removeEventListener(ee.window, "mousemove", this.mousemove), oe.removeEventListener(ee.window, "mouseup", this.mouseup);
                }, nc.prototype.mousedown = function(Ke) {
                  this.down(ee.extend({}, Ke, { ctrlKey: !0, preventDefault: function() {
                    return Ke.preventDefault();
                  } }), oe.mousePos(this.element, Ke)), oe.addEventListener(ee.window, "mousemove", this.mousemove), oe.addEventListener(ee.window, "mouseup", this.mouseup);
                }, nc.prototype.mousemove = function(Ke) {
                  this.move(Ke, oe.mousePos(this.element, Ke));
                }, nc.prototype.mouseup = function(Ke) {
                  this.mouseRotate.mouseupWindow(Ke), this.mousePitch && this.mousePitch.mouseupWindow(Ke), this.offTemp();
                }, nc.prototype.touchstart = function(Ke) {
                  Ke.targetTouches.length !== 1 ? this.reset() : (this._startPos = this._lastPos = oe.touchPos(this.element, Ke.targetTouches)[0], this.down({ type: "mousedown", button: 0, ctrlKey: !0, preventDefault: function() {
                    return Ke.preventDefault();
                  } }, this._startPos));
                }, nc.prototype.touchmove = function(Ke) {
                  Ke.targetTouches.length !== 1 ? this.reset() : (this._lastPos = oe.touchPos(this.element, Ke.targetTouches)[0], this.move({ preventDefault: function() {
                    return Ke.preventDefault();
                  } }, this._lastPos));
                }, nc.prototype.touchend = function(Ke) {
                  Ke.targetTouches.length === 0 && this._startPos && this._lastPos && this._startPos.dist(this._lastPos) < this._clickTolerance && this.element.click(), this.reset();
                }, nc.prototype.reset = function() {
                  this.mouseRotate.reset(), this.mousePitch && this.mousePitch.reset(), delete this._startPos, delete this._lastPos, this.offTemp();
                };
                var ks = { center: "translate(-50%,-50%)", top: "translate(-50%,0)", "top-left": "translate(0,0)", "top-right": "translate(-100%,0)", bottom: "translate(-50%,-100%)", "bottom-left": "translate(0,-100%)", "bottom-right": "translate(-100%,-100%)", left: "translate(0,-50%)", right: "translate(-100%,-50%)" };
                function qs(Ke, Fe, Te) {
                  var Ae = Ke.classList;
                  for (var Oe in ks) Ae.remove("mapboxgl-" + Te + "-anchor-" + Oe);
                  Ae.add("mapboxgl-" + Te + "-anchor-" + Fe);
                }
                var a1, $l = function(Ke) {
                  function Fe(Te, Ae) {
                    if (Ke.call(this), (Te instanceof ee.window.HTMLElement || Ae) && (Te = ee.extend({ element: Te }, Ae)), ee.bindAll(["_update", "_onMove", "_onUp", "_addDragHandler", "_onMapClick", "_onKeyPress"], this), this._anchor = Te && Te.anchor || "center", this._color = Te && Te.color || "#3FB1CE", this._scale = Te && Te.scale || 1, this._draggable = Te && Te.draggable || !1, this._clickTolerance = Te && Te.clickTolerance || 0, this._isDragging = !1, this._state = "inactive", this._rotation = Te && Te.rotation || 0, this._rotationAlignment = Te && Te.rotationAlignment || "auto", this._pitchAlignment = Te && Te.pitchAlignment && Te.pitchAlignment !== "auto" ? Te.pitchAlignment : this._rotationAlignment, Te && Te.element) this._element = Te.element, this._offset = ee.Point.convert(Te && Te.offset || [0, 0]);
                    else {
                      this._defaultMarker = !0, this._element = oe.create("div"), this._element.setAttribute("aria-label", "Map marker");
                      var Oe = oe.createNS("http://www.w3.org/2000/svg", "svg");
                      Oe.setAttributeNS(null, "display", "block"), Oe.setAttributeNS(null, "height", "41px"), Oe.setAttributeNS(null, "width", "27px"), Oe.setAttributeNS(null, "viewBox", "0 0 27 41");
                      var Be = oe.createNS("http://www.w3.org/2000/svg", "g");
                      Be.setAttributeNS(null, "stroke", "none"), Be.setAttributeNS(null, "stroke-width", "1"), Be.setAttributeNS(null, "fill", "none"), Be.setAttributeNS(null, "fill-rule", "evenodd");
                      var Ue = oe.createNS("http://www.w3.org/2000/svg", "g");
                      Ue.setAttributeNS(null, "fill-rule", "nonzero");
                      var nt = oe.createNS("http://www.w3.org/2000/svg", "g");
                      nt.setAttributeNS(null, "transform", "translate(3.0, 29.0)"), nt.setAttributeNS(null, "fill", "#000000");
                      for (var ht = 0, Et = [{ rx: "10.5", ry: "5.25002273" }, { rx: "10.5", ry: "5.25002273" }, { rx: "9.5", ry: "4.77275007" }, { rx: "8.5", ry: "4.29549936" }, { rx: "7.5", ry: "3.81822308" }, { rx: "6.5", ry: "3.34094679" }, { rx: "5.5", ry: "2.86367051" }, { rx: "4.5", ry: "2.38636864" }]; ht < Et.length; ht += 1) {
                        var Mt = Et[ht], qt = oe.createNS("http://www.w3.org/2000/svg", "ellipse");
                        qt.setAttributeNS(null, "opacity", "0.04"), qt.setAttributeNS(null, "cx", "10.5"), qt.setAttributeNS(null, "cy", "5.80029008"), qt.setAttributeNS(null, "rx", Mt.rx), qt.setAttributeNS(null, "ry", Mt.ry), nt.appendChild(qt);
                      }
                      var tn = oe.createNS("http://www.w3.org/2000/svg", "g");
                      tn.setAttributeNS(null, "fill", this._color);
                      var xn = oe.createNS("http://www.w3.org/2000/svg", "path");
                      xn.setAttributeNS(null, "d", "M27,13.5 C27,19.074644 20.250001,27.000002 14.75,34.500002 C14.016665,35.500004 12.983335,35.500004 12.25,34.500002 C6.7499993,27.000002 0,19.222562 0,13.5 C0,6.0441559 6.0441559,0 13.5,0 C20.955844,0 27,6.0441559 27,13.5 Z"), tn.appendChild(xn);
                      var An = oe.createNS("http://www.w3.org/2000/svg", "g");
                      An.setAttributeNS(null, "opacity", "0.25"), An.setAttributeNS(null, "fill", "#000000");
                      var On = oe.createNS("http://www.w3.org/2000/svg", "path");
                      On.setAttributeNS(null, "d", "M13.5,0 C6.0441559,0 0,6.0441559 0,13.5 C0,19.222562 6.7499993,27 12.25,34.5 C13,35.522727 14.016664,35.500004 14.75,34.5 C20.250001,27 27,19.074644 27,13.5 C27,6.0441559 20.955844,0 13.5,0 Z M13.5,1 C20.415404,1 26,6.584596 26,13.5 C26,15.898657 24.495584,19.181431 22.220703,22.738281 C19.945823,26.295132 16.705119,30.142167 13.943359,33.908203 C13.743445,34.180814 13.612715,34.322738 13.5,34.441406 C13.387285,34.322738 13.256555,34.180814 13.056641,33.908203 C10.284481,30.127985 7.4148684,26.314159 5.015625,22.773438 C2.6163816,19.232715 1,15.953538 1,13.5 C1,6.584596 6.584596,1 13.5,1 Z"), An.appendChild(On);
                      var Bn = oe.createNS("http://www.w3.org/2000/svg", "g");
                      Bn.setAttributeNS(null, "transform", "translate(6.0, 7.0)"), Bn.setAttributeNS(null, "fill", "#FFFFFF");
                      var qn = oe.createNS("http://www.w3.org/2000/svg", "g");
                      qn.setAttributeNS(null, "transform", "translate(8.0, 8.0)");
                      var Vn = oe.createNS("http://www.w3.org/2000/svg", "circle");
                      Vn.setAttributeNS(null, "fill", "#000000"), Vn.setAttributeNS(null, "opacity", "0.25"), Vn.setAttributeNS(null, "cx", "5.5"), Vn.setAttributeNS(null, "cy", "5.5"), Vn.setAttributeNS(null, "r", "5.4999962");
                      var sr = oe.createNS("http://www.w3.org/2000/svg", "circle");
                      sr.setAttributeNS(null, "fill", "#FFFFFF"), sr.setAttributeNS(null, "cx", "5.5"), sr.setAttributeNS(null, "cy", "5.5"), sr.setAttributeNS(null, "r", "5.4999962"), qn.appendChild(Vn), qn.appendChild(sr), Ue.appendChild(nt), Ue.appendChild(tn), Ue.appendChild(An), Ue.appendChild(Bn), Ue.appendChild(qn), Oe.appendChild(Ue), Oe.setAttributeNS(null, "height", 41 * this._scale + "px"), Oe.setAttributeNS(null, "width", 27 * this._scale + "px"), this._element.appendChild(Oe), this._offset = ee.Point.convert(Te && Te.offset || [0, -14]);
                    }
                    this._element.classList.add("mapboxgl-marker"), this._element.addEventListener("dragstart", function(Un) {
                      Un.preventDefault();
                    }), this._element.addEventListener("mousedown", function(Un) {
                      Un.preventDefault();
                    }), qs(this._element, this._anchor, "marker"), this._popup = null;
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.addTo = function(Te) {
                    return this.remove(), this._map = Te, Te.getCanvasContainer().appendChild(this._element), Te.on("move", this._update), Te.on("moveend", this._update), this.setDraggable(this._draggable), this._update(), this._map.on("click", this._onMapClick), this;
                  }, Fe.prototype.remove = function() {
                    return this._map && (this._map.off("click", this._onMapClick), this._map.off("move", this._update), this._map.off("moveend", this._update), this._map.off("mousedown", this._addDragHandler), this._map.off("touchstart", this._addDragHandler), this._map.off("mouseup", this._onUp), this._map.off("touchend", this._onUp), this._map.off("mousemove", this._onMove), this._map.off("touchmove", this._onMove), delete this._map), oe.remove(this._element), this._popup && this._popup.remove(), this;
                  }, Fe.prototype.getLngLat = function() {
                    return this._lngLat;
                  }, Fe.prototype.setLngLat = function(Te) {
                    return this._lngLat = ee.LngLat.convert(Te), this._pos = null, this._popup && this._popup.setLngLat(this._lngLat), this._update(), this;
                  }, Fe.prototype.getElement = function() {
                    return this._element;
                  }, Fe.prototype.setPopup = function(Te) {
                    if (this._popup && (this._popup.remove(), this._popup = null, this._element.removeEventListener("keypress", this._onKeyPress), this._originalTabIndex || this._element.removeAttribute("tabindex")), Te) {
                      if (!("offset" in Te.options)) {
                        var Ae = 13.5, Oe = Math.sqrt(Math.pow(Ae, 2) / 2);
                        Te.options.offset = this._defaultMarker ? { top: [0, 0], "top-left": [0, 0], "top-right": [0, 0], bottom: [0, -38.1], "bottom-left": [Oe, -1 * (24.6 + Oe)], "bottom-right": [-Oe, -1 * (24.6 + Oe)], left: [Ae, -24.6], right: [-13.5, -24.6] } : this._offset;
                      }
                      this._popup = Te, this._lngLat && this._popup.setLngLat(this._lngLat), this._originalTabIndex = this._element.getAttribute("tabindex"), this._originalTabIndex || this._element.setAttribute("tabindex", "0"), this._element.addEventListener("keypress", this._onKeyPress);
                    }
                    return this;
                  }, Fe.prototype._onKeyPress = function(Te) {
                    var Ae = Te.code, Oe = Te.charCode || Te.keyCode;
                    Ae !== "Space" && Ae !== "Enter" && Oe !== 32 && Oe !== 13 || this.togglePopup();
                  }, Fe.prototype._onMapClick = function(Te) {
                    var Ae = Te.originalEvent.target, Oe = this._element;
                    this._popup && (Ae === Oe || Oe.contains(Ae)) && this.togglePopup();
                  }, Fe.prototype.getPopup = function() {
                    return this._popup;
                  }, Fe.prototype.togglePopup = function() {
                    var Te = this._popup;
                    return Te ? (Te.isOpen() ? Te.remove() : Te.addTo(this._map), this) : this;
                  }, Fe.prototype._update = function(Te) {
                    if (this._map) {
                      this._map.transform.renderWorldCopies && (this._lngLat = Ko(this._lngLat, this._pos, this._map.transform)), this._pos = this._map.project(this._lngLat)._add(this._offset);
                      var Ae = "";
                      this._rotationAlignment === "viewport" || this._rotationAlignment === "auto" ? Ae = "rotateZ(" + this._rotation + "deg)" : this._rotationAlignment === "map" && (Ae = "rotateZ(" + (this._rotation - this._map.getBearing()) + "deg)");
                      var Oe = "";
                      this._pitchAlignment === "viewport" || this._pitchAlignment === "auto" ? Oe = "rotateX(0deg)" : this._pitchAlignment === "map" && (Oe = "rotateX(" + this._map.getPitch() + "deg)"), Te && Te.type !== "moveend" || (this._pos = this._pos.round()), oe.setTransform(this._element, ks[this._anchor] + " translate(" + this._pos.x + "px, " + this._pos.y + "px) " + Oe + " " + Ae);
                    }
                  }, Fe.prototype.getOffset = function() {
                    return this._offset;
                  }, Fe.prototype.setOffset = function(Te) {
                    return this._offset = ee.Point.convert(Te), this._update(), this;
                  }, Fe.prototype._onMove = function(Te) {
                    if (!this._isDragging) {
                      var Ae = this._clickTolerance || this._map._clickTolerance;
                      this._isDragging = Te.point.dist(this._pointerdownPos) >= Ae;
                    }
                    this._isDragging && (this._pos = Te.point.sub(this._positionDelta), this._lngLat = this._map.unproject(this._pos), this.setLngLat(this._lngLat), this._element.style.pointerEvents = "none", this._state === "pending" && (this._state = "active", this.fire(new ee.Event("dragstart"))), this.fire(new ee.Event("drag")));
                  }, Fe.prototype._onUp = function() {
                    this._element.style.pointerEvents = "auto", this._positionDelta = null, this._pointerdownPos = null, this._isDragging = !1, this._map.off("mousemove", this._onMove), this._map.off("touchmove", this._onMove), this._state === "active" && this.fire(new ee.Event("dragend")), this._state = "inactive";
                  }, Fe.prototype._addDragHandler = function(Te) {
                    this._element.contains(Te.originalEvent.target) && (Te.preventDefault(), this._positionDelta = Te.point.sub(this._pos).add(this._offset), this._pointerdownPos = Te.point, this._state = "pending", this._map.on("mousemove", this._onMove), this._map.on("touchmove", this._onMove), this._map.once("mouseup", this._onUp), this._map.once("touchend", this._onUp));
                  }, Fe.prototype.setDraggable = function(Te) {
                    return this._draggable = !!Te, this._map && (Te ? (this._map.on("mousedown", this._addDragHandler), this._map.on("touchstart", this._addDragHandler)) : (this._map.off("mousedown", this._addDragHandler), this._map.off("touchstart", this._addDragHandler))), this;
                  }, Fe.prototype.isDraggable = function() {
                    return this._draggable;
                  }, Fe.prototype.setRotation = function(Te) {
                    return this._rotation = Te || 0, this._update(), this;
                  }, Fe.prototype.getRotation = function() {
                    return this._rotation;
                  }, Fe.prototype.setRotationAlignment = function(Te) {
                    return this._rotationAlignment = Te || "auto", this._update(), this;
                  }, Fe.prototype.getRotationAlignment = function() {
                    return this._rotationAlignment;
                  }, Fe.prototype.setPitchAlignment = function(Te) {
                    return this._pitchAlignment = Te && Te !== "auto" ? Te : this._rotationAlignment, this._update(), this;
                  }, Fe.prototype.getPitchAlignment = function() {
                    return this._pitchAlignment;
                  }, Fe;
                }(ee.Evented), o1 = { positionOptions: { enableHighAccuracy: !1, maximumAge: 0, timeout: 6e3 }, fitBoundsOptions: { maxZoom: 15 }, trackUserLocation: !1, showAccuracyCircle: !0, showUserLocation: !0 }, $c = 0, lu = !1, Ul = function(Ke) {
                  function Fe(Te) {
                    Ke.call(this), this.options = ee.extend({}, o1, Te), ee.bindAll(["_onSuccess", "_onError", "_onZoom", "_finish", "_setupUI", "_updateCamera", "_updateMarker"], this);
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.onAdd = function(Te) {
                    return this._map = Te, this._container = oe.create("div", "mapboxgl-ctrl mapboxgl-ctrl-group"), Ae = this._setupUI, a1 !== void 0 ? Ae(a1) : ee.window.navigator.permissions !== void 0 ? ee.window.navigator.permissions.query({ name: "geolocation" }).then(function(Oe) {
                      a1 = Oe.state !== "denied", Ae(a1);
                    }) : (a1 = !!ee.window.navigator.geolocation, Ae(a1)), this._container;
                    var Ae;
                  }, Fe.prototype.onRemove = function() {
                    this._geolocationWatchID !== void 0 && (ee.window.navigator.geolocation.clearWatch(this._geolocationWatchID), this._geolocationWatchID = void 0), this.options.showUserLocation && this._userLocationDotMarker && this._userLocationDotMarker.remove(), this.options.showAccuracyCircle && this._accuracyCircleMarker && this._accuracyCircleMarker.remove(), oe.remove(this._container), this._map.off("zoom", this._onZoom), this._map = void 0, $c = 0, lu = !1;
                  }, Fe.prototype._isOutOfMapMaxBounds = function(Te) {
                    var Ae = this._map.getMaxBounds(), Oe = Te.coords;
                    return Ae && (Oe.longitude < Ae.getWest() || Oe.longitude > Ae.getEast() || Oe.latitude < Ae.getSouth() || Oe.latitude > Ae.getNorth());
                  }, Fe.prototype._setErrorState = function() {
                    switch (this._watchState) {
                      case "WAITING_ACTIVE":
                        this._watchState = "ACTIVE_ERROR", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active-error");
                        break;
                      case "ACTIVE_LOCK":
                        this._watchState = "ACTIVE_ERROR", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active-error"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting");
                        break;
                      case "BACKGROUND":
                        this._watchState = "BACKGROUND_ERROR", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background-error"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting");
                    }
                  }, Fe.prototype._onSuccess = function(Te) {
                    if (this._map) {
                      if (this._isOutOfMapMaxBounds(Te)) return this._setErrorState(), this.fire(new ee.Event("outofmaxbounds", Te)), this._updateMarker(), void this._finish();
                      if (this.options.trackUserLocation) switch (this._lastKnownPosition = Te, this._watchState) {
                        case "WAITING_ACTIVE":
                        case "ACTIVE_LOCK":
                        case "ACTIVE_ERROR":
                          this._watchState = "ACTIVE_LOCK", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active-error"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active");
                          break;
                        case "BACKGROUND":
                        case "BACKGROUND_ERROR":
                          this._watchState = "BACKGROUND", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background-error"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background");
                      }
                      this.options.showUserLocation && this._watchState !== "OFF" && this._updateMarker(Te), this.options.trackUserLocation && this._watchState !== "ACTIVE_LOCK" || this._updateCamera(Te), this.options.showUserLocation && this._dotElement.classList.remove("mapboxgl-user-location-dot-stale"), this.fire(new ee.Event("geolocate", Te)), this._finish();
                    }
                  }, Fe.prototype._updateCamera = function(Te) {
                    var Ae = new ee.LngLat(Te.coords.longitude, Te.coords.latitude), Oe = Te.coords.accuracy, Be = this._map.getBearing(), Ue = ee.extend({ bearing: Be }, this.options.fitBoundsOptions);
                    this._map.fitBounds(Ae.toBounds(Oe), Ue, { geolocateSource: !0 });
                  }, Fe.prototype._updateMarker = function(Te) {
                    if (Te) {
                      var Ae = new ee.LngLat(Te.coords.longitude, Te.coords.latitude);
                      this._accuracyCircleMarker.setLngLat(Ae).addTo(this._map), this._userLocationDotMarker.setLngLat(Ae).addTo(this._map), this._accuracy = Te.coords.accuracy, this.options.showUserLocation && this.options.showAccuracyCircle && this._updateCircleRadius();
                    } else this._userLocationDotMarker.remove(), this._accuracyCircleMarker.remove();
                  }, Fe.prototype._updateCircleRadius = function() {
                    var Te = this._map._container.clientHeight / 2, Ae = this._map.unproject([0, Te]), Oe = this._map.unproject([1, Te]), Be = Ae.distanceTo(Oe), Ue = Math.ceil(2 * this._accuracy / Be);
                    this._circleElement.style.width = Ue + "px", this._circleElement.style.height = Ue + "px";
                  }, Fe.prototype._onZoom = function() {
                    this.options.showUserLocation && this.options.showAccuracyCircle && this._updateCircleRadius();
                  }, Fe.prototype._onError = function(Te) {
                    if (this._map) {
                      if (this.options.trackUserLocation) if (Te.code === 1) {
                        this._watchState = "OFF", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active-error"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background-error"), this._geolocateButton.disabled = !0;
                        var Ae = this._map._getUIString("GeolocateControl.LocationNotAvailable");
                        this._geolocateButton.title = Ae, this._geolocateButton.setAttribute("aria-label", Ae), this._geolocationWatchID !== void 0 && this._clearWatch();
                      } else {
                        if (Te.code === 3 && lu) return;
                        this._setErrorState();
                      }
                      this._watchState !== "OFF" && this.options.showUserLocation && this._dotElement.classList.add("mapboxgl-user-location-dot-stale"), this.fire(new ee.Event("error", Te)), this._finish();
                    }
                  }, Fe.prototype._finish = function() {
                    this._timeoutId && clearTimeout(this._timeoutId), this._timeoutId = void 0;
                  }, Fe.prototype._setupUI = function(Te) {
                    var Ae = this;
                    if (this._container.addEventListener("contextmenu", function(Ue) {
                      return Ue.preventDefault();
                    }), this._geolocateButton = oe.create("button", "mapboxgl-ctrl-geolocate", this._container), oe.create("span", "mapboxgl-ctrl-icon", this._geolocateButton).setAttribute("aria-hidden", !0), this._geolocateButton.type = "button", Te === !1) {
                      ee.warnOnce("Geolocation support is not available so the GeolocateControl will be disabled.");
                      var Oe = this._map._getUIString("GeolocateControl.LocationNotAvailable");
                      this._geolocateButton.disabled = !0, this._geolocateButton.title = Oe, this._geolocateButton.setAttribute("aria-label", Oe);
                    } else {
                      var Be = this._map._getUIString("GeolocateControl.FindMyLocation");
                      this._geolocateButton.title = Be, this._geolocateButton.setAttribute("aria-label", Be);
                    }
                    this.options.trackUserLocation && (this._geolocateButton.setAttribute("aria-pressed", "false"), this._watchState = "OFF"), this.options.showUserLocation && (this._dotElement = oe.create("div", "mapboxgl-user-location-dot"), this._userLocationDotMarker = new $l(this._dotElement), this._circleElement = oe.create("div", "mapboxgl-user-location-accuracy-circle"), this._accuracyCircleMarker = new $l({ element: this._circleElement, pitchAlignment: "map" }), this.options.trackUserLocation && (this._watchState = "OFF"), this._map.on("zoom", this._onZoom)), this._geolocateButton.addEventListener("click", this.trigger.bind(this)), this._setup = !0, this.options.trackUserLocation && this._map.on("movestart", function(Ue) {
                      var nt = Ue.originalEvent && Ue.originalEvent.type === "resize";
                      Ue.geolocateSource || Ae._watchState !== "ACTIVE_LOCK" || nt || (Ae._watchState = "BACKGROUND", Ae._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background"), Ae._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"), Ae.fire(new ee.Event("trackuserlocationend")));
                    });
                  }, Fe.prototype.trigger = function() {
                    if (!this._setup) return ee.warnOnce("Geolocate control triggered before added to a map"), !1;
                    if (this.options.trackUserLocation) {
                      switch (this._watchState) {
                        case "OFF":
                          this._watchState = "WAITING_ACTIVE", this.fire(new ee.Event("trackuserlocationstart"));
                          break;
                        case "WAITING_ACTIVE":
                        case "ACTIVE_LOCK":
                        case "ACTIVE_ERROR":
                        case "BACKGROUND_ERROR":
                          $c--, lu = !1, this._watchState = "OFF", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-active-error"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"), this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background-error"), this.fire(new ee.Event("trackuserlocationend"));
                          break;
                        case "BACKGROUND":
                          this._watchState = "ACTIVE_LOCK", this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-background"), this._lastKnownPosition && this._updateCamera(this._lastKnownPosition), this.fire(new ee.Event("trackuserlocationstart"));
                      }
                      switch (this._watchState) {
                        case "WAITING_ACTIVE":
                          this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active");
                          break;
                        case "ACTIVE_LOCK":
                          this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active");
                          break;
                        case "ACTIVE_ERROR":
                          this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-active-error");
                          break;
                        case "BACKGROUND":
                          this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background");
                          break;
                        case "BACKGROUND_ERROR":
                          this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-background-error");
                      }
                      if (this._watchState === "OFF" && this._geolocationWatchID !== void 0) this._clearWatch();
                      else if (this._geolocationWatchID === void 0) {
                        var Te;
                        this._geolocateButton.classList.add("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.setAttribute("aria-pressed", "true"), ++$c > 1 ? (Te = { maximumAge: 6e5, timeout: 0 }, lu = !0) : (Te = this.options.positionOptions, lu = !1), this._geolocationWatchID = ee.window.navigator.geolocation.watchPosition(this._onSuccess, this._onError, Te);
                      }
                    } else ee.window.navigator.geolocation.getCurrentPosition(this._onSuccess, this._onError, this.options.positionOptions), this._timeoutId = setTimeout(this._finish, 1e4);
                    return !0;
                  }, Fe.prototype._clearWatch = function() {
                    ee.window.navigator.geolocation.clearWatch(this._geolocationWatchID), this._geolocationWatchID = void 0, this._geolocateButton.classList.remove("mapboxgl-ctrl-geolocate-waiting"), this._geolocateButton.setAttribute("aria-pressed", "false"), this.options.showUserLocation && this._updateMarker(null);
                  }, Fe;
                }(ee.Evented), zs = { maxWidth: 100, unit: "metric" }, Os = function(Ke) {
                  this.options = ee.extend({}, zs, Ke), ee.bindAll(["_onMove", "setUnit"], this);
                };
                function lc(Ke, Fe, Te) {
                  var Ae = Te && Te.maxWidth || 100, Oe = Ke._container.clientHeight / 2, Be = Ke.unproject([0, Oe]), Ue = Ke.unproject([Ae, Oe]), nt = Be.distanceTo(Ue);
                  if (Te && Te.unit === "imperial") {
                    var ht = 3.2808 * nt;
                    ht > 5280 ? xc(Fe, Ae, ht / 5280, Ke._getUIString("ScaleControl.Miles")) : xc(Fe, Ae, ht, Ke._getUIString("ScaleControl.Feet"));
                  } else Te && Te.unit === "nautical" ? xc(Fe, Ae, nt / 1852, Ke._getUIString("ScaleControl.NauticalMiles")) : nt >= 1e3 ? xc(Fe, Ae, nt / 1e3, Ke._getUIString("ScaleControl.Kilometers")) : xc(Fe, Ae, nt, Ke._getUIString("ScaleControl.Meters"));
                }
                function xc(Ke, Fe, Te, Ae) {
                  var Oe, Be, Ue, nt = (Oe = Te, (Be = Math.pow(10, ("" + Math.floor(Oe)).length - 1)) * ((Ue = Oe / Be) >= 10 ? 10 : Ue >= 5 ? 5 : Ue >= 3 ? 3 : Ue >= 2 ? 2 : Ue >= 1 ? 1 : function(Et) {
                    var Mt = Math.pow(10, Math.ceil(-Math.log(Et) / Math.LN10));
                    return Math.round(Et * Mt) / Mt;
                  }(Ue))), ht = nt / Te;
                  Ke.style.width = Fe * ht + "px", Ke.innerHTML = nt + "&nbsp;" + Ae;
                }
                Os.prototype.getDefaultPosition = function() {
                  return "bottom-left";
                }, Os.prototype._onMove = function() {
                  lc(this._map, this._container, this.options);
                }, Os.prototype.onAdd = function(Ke) {
                  return this._map = Ke, this._container = oe.create("div", "mapboxgl-ctrl mapboxgl-ctrl-scale", Ke.getContainer()), this._map.on("move", this._onMove), this._onMove(), this._container;
                }, Os.prototype.onRemove = function() {
                  oe.remove(this._container), this._map.off("move", this._onMove), this._map = void 0;
                }, Os.prototype.setUnit = function(Ke) {
                  this.options.unit = Ke, lc(this._map, this._container, this.options);
                };
                var Iu = function(Ke) {
                  this._fullscreen = !1, Ke && Ke.container && (Ke.container instanceof ee.window.HTMLElement ? this._container = Ke.container : ee.warnOnce("Full screen control 'container' must be a DOM element.")), ee.bindAll(["_onClickFullscreen", "_changeIcon"], this), "onfullscreenchange" in ee.window.document ? this._fullscreenchange = "fullscreenchange" : "onmozfullscreenchange" in ee.window.document ? this._fullscreenchange = "mozfullscreenchange" : "onwebkitfullscreenchange" in ee.window.document ? this._fullscreenchange = "webkitfullscreenchange" : "onmsfullscreenchange" in ee.window.document && (this._fullscreenchange = "MSFullscreenChange");
                };
                Iu.prototype.onAdd = function(Ke) {
                  return this._map = Ke, this._container || (this._container = this._map.getContainer()), this._controlContainer = oe.create("div", "mapboxgl-ctrl mapboxgl-ctrl-group"), this._checkFullscreenSupport() ? this._setupUI() : (this._controlContainer.style.display = "none", ee.warnOnce("This device does not support fullscreen mode.")), this._controlContainer;
                }, Iu.prototype.onRemove = function() {
                  oe.remove(this._controlContainer), this._map = null, ee.window.document.removeEventListener(this._fullscreenchange, this._changeIcon);
                }, Iu.prototype._checkFullscreenSupport = function() {
                  return !!(ee.window.document.fullscreenEnabled || ee.window.document.mozFullScreenEnabled || ee.window.document.msFullscreenEnabled || ee.window.document.webkitFullscreenEnabled);
                }, Iu.prototype._setupUI = function() {
                  var Ke = this._fullscreenButton = oe.create("button", "mapboxgl-ctrl-fullscreen", this._controlContainer);
                  oe.create("span", "mapboxgl-ctrl-icon", Ke).setAttribute("aria-hidden", !0), Ke.type = "button", this._updateTitle(), this._fullscreenButton.addEventListener("click", this._onClickFullscreen), ee.window.document.addEventListener(this._fullscreenchange, this._changeIcon);
                }, Iu.prototype._updateTitle = function() {
                  var Ke = this._getTitle();
                  this._fullscreenButton.setAttribute("aria-label", Ke), this._fullscreenButton.title = Ke;
                }, Iu.prototype._getTitle = function() {
                  return this._map._getUIString(this._isFullscreen() ? "FullscreenControl.Exit" : "FullscreenControl.Enter");
                }, Iu.prototype._isFullscreen = function() {
                  return this._fullscreen;
                }, Iu.prototype._changeIcon = function() {
                  (ee.window.document.fullscreenElement || ee.window.document.mozFullScreenElement || ee.window.document.webkitFullscreenElement || ee.window.document.msFullscreenElement) === this._container !== this._fullscreen && (this._fullscreen = !this._fullscreen, this._fullscreenButton.classList.toggle("mapboxgl-ctrl-shrink"), this._fullscreenButton.classList.toggle("mapboxgl-ctrl-fullscreen"), this._updateTitle());
                }, Iu.prototype._onClickFullscreen = function() {
                  this._isFullscreen() ? ee.window.document.exitFullscreen ? ee.window.document.exitFullscreen() : ee.window.document.mozCancelFullScreen ? ee.window.document.mozCancelFullScreen() : ee.window.document.msExitFullscreen ? ee.window.document.msExitFullscreen() : ee.window.document.webkitCancelFullScreen && ee.window.document.webkitCancelFullScreen() : this._container.requestFullscreen ? this._container.requestFullscreen() : this._container.mozRequestFullScreen ? this._container.mozRequestFullScreen() : this._container.msRequestFullscreen ? this._container.msRequestFullscreen() : this._container.webkitRequestFullscreen && this._container.webkitRequestFullscreen();
                };
                var tu = { closeButton: !0, closeOnClick: !0, focusAfterOpen: !0, className: "", maxWidth: "240px" }, w1 = ["a[href]", "[tabindex]:not([tabindex='-1'])", "[contenteditable]:not([contenteditable='false'])", "button:not([disabled])", "input:not([disabled])", "select:not([disabled])", "textarea:not([disabled])"].join(", "), $n = function(Ke) {
                  function Fe(Te) {
                    Ke.call(this), this.options = ee.extend(Object.create(tu), Te), ee.bindAll(["_update", "_onClose", "remove", "_onMouseMove", "_onMouseUp", "_onDrag"], this);
                  }
                  return Ke && (Fe.__proto__ = Ke), Fe.prototype = Object.create(Ke && Ke.prototype), Fe.prototype.constructor = Fe, Fe.prototype.addTo = function(Te) {
                    return this._map && this.remove(), this._map = Te, this.options.closeOnClick && this._map.on("click", this._onClose), this.options.closeOnMove && this._map.on("move", this._onClose), this._map.on("remove", this.remove), this._update(), this._focusFirstElement(), this._trackPointer ? (this._map.on("mousemove", this._onMouseMove), this._map.on("mouseup", this._onMouseUp), this._container && this._container.classList.add("mapboxgl-popup-track-pointer"), this._map._canvasContainer.classList.add("mapboxgl-track-pointer")) : this._map.on("move", this._update), this.fire(new ee.Event("open")), this;
                  }, Fe.prototype.isOpen = function() {
                    return !!this._map;
                  }, Fe.prototype.remove = function() {
                    return this._content && oe.remove(this._content), this._container && (oe.remove(this._container), delete this._container), this._map && (this._map.off("move", this._update), this._map.off("move", this._onClose), this._map.off("click", this._onClose), this._map.off("remove", this.remove), this._map.off("mousemove", this._onMouseMove), this._map.off("mouseup", this._onMouseUp), this._map.off("drag", this._onDrag), delete this._map), this.fire(new ee.Event("close")), this;
                  }, Fe.prototype.getLngLat = function() {
                    return this._lngLat;
                  }, Fe.prototype.setLngLat = function(Te) {
                    return this._lngLat = ee.LngLat.convert(Te), this._pos = null, this._trackPointer = !1, this._update(), this._map && (this._map.on("move", this._update), this._map.off("mousemove", this._onMouseMove), this._container && this._container.classList.remove("mapboxgl-popup-track-pointer"), this._map._canvasContainer.classList.remove("mapboxgl-track-pointer")), this;
                  }, Fe.prototype.trackPointer = function() {
                    return this._trackPointer = !0, this._pos = null, this._update(), this._map && (this._map.off("move", this._update), this._map.on("mousemove", this._onMouseMove), this._map.on("drag", this._onDrag), this._container && this._container.classList.add("mapboxgl-popup-track-pointer"), this._map._canvasContainer.classList.add("mapboxgl-track-pointer")), this;
                  }, Fe.prototype.getElement = function() {
                    return this._container;
                  }, Fe.prototype.setText = function(Te) {
                    return this.setDOMContent(ee.window.document.createTextNode(Te));
                  }, Fe.prototype.setHTML = function(Te) {
                    var Ae, Oe = ee.window.document.createDocumentFragment(), Be = ee.window.document.createElement("body");
                    for (Be.innerHTML = Te; Ae = Be.firstChild; ) Oe.appendChild(Ae);
                    return this.setDOMContent(Oe);
                  }, Fe.prototype.getMaxWidth = function() {
                    return this._container && this._container.style.maxWidth;
                  }, Fe.prototype.setMaxWidth = function(Te) {
                    return this.options.maxWidth = Te, this._update(), this;
                  }, Fe.prototype.setDOMContent = function(Te) {
                    if (this._content) for (; this._content.hasChildNodes(); ) this._content.firstChild && this._content.removeChild(this._content.firstChild);
                    else this._content = oe.create("div", "mapboxgl-popup-content", this._container);
                    return this._content.appendChild(Te), this._createCloseButton(), this._update(), this._focusFirstElement(), this;
                  }, Fe.prototype.addClassName = function(Te) {
                    this._container && this._container.classList.add(Te);
                  }, Fe.prototype.removeClassName = function(Te) {
                    this._container && this._container.classList.remove(Te);
                  }, Fe.prototype.setOffset = function(Te) {
                    return this.options.offset = Te, this._update(), this;
                  }, Fe.prototype.toggleClassName = function(Te) {
                    if (this._container) return this._container.classList.toggle(Te);
                  }, Fe.prototype._createCloseButton = function() {
                    this.options.closeButton && (this._closeButton = oe.create("button", "mapboxgl-popup-close-button", this._content), this._closeButton.type = "button", this._closeButton.setAttribute("aria-label", "Close popup"), this._closeButton.innerHTML = "&#215;", this._closeButton.addEventListener("click", this._onClose));
                  }, Fe.prototype._onMouseUp = function(Te) {
                    this._update(Te.point);
                  }, Fe.prototype._onMouseMove = function(Te) {
                    this._update(Te.point);
                  }, Fe.prototype._onDrag = function(Te) {
                    this._update(Te.point);
                  }, Fe.prototype._update = function(Te) {
                    var Ae = this, Oe = this._lngLat || this._trackPointer;
                    if (this._map && Oe && this._content && (this._container || (this._container = oe.create("div", "mapboxgl-popup", this._map.getContainer()), this._tip = oe.create("div", "mapboxgl-popup-tip", this._container), this._container.appendChild(this._content), this.options.className && this.options.className.split(" ").forEach(function(tn) {
                      return Ae._container.classList.add(tn);
                    }), this._trackPointer && this._container.classList.add("mapboxgl-popup-track-pointer")), this.options.maxWidth && this._container.style.maxWidth !== this.options.maxWidth && (this._container.style.maxWidth = this.options.maxWidth), this._map.transform.renderWorldCopies && !this._trackPointer && (this._lngLat = Ko(this._lngLat, this._pos, this._map.transform)), !this._trackPointer || Te)) {
                      var Be = this._pos = this._trackPointer && Te ? Te : this._map.project(this._lngLat), Ue = this.options.anchor, nt = zn(this.options.offset);
                      if (!Ue) {
                        var ht, Et = this._container.offsetWidth, Mt = this._container.offsetHeight;
                        ht = Be.y + nt.bottom.y < Mt ? ["top"] : Be.y > this._map.transform.height - Mt ? ["bottom"] : [], Be.x < Et / 2 ? ht.push("left") : Be.x > this._map.transform.width - Et / 2 && ht.push("right"), Ue = ht.length === 0 ? "bottom" : ht.join("-");
                      }
                      var qt = Be.add(nt[Ue]).round();
                      oe.setTransform(this._container, ks[Ue] + " translate(" + qt.x + "px," + qt.y + "px)"), qs(this._container, Ue, "popup");
                    }
                  }, Fe.prototype._focusFirstElement = function() {
                    if (this.options.focusAfterOpen && this._container) {
                      var Te = this._container.querySelector(w1);
                      Te && Te.focus();
                    }
                  }, Fe.prototype._onClose = function() {
                    this.remove();
                  }, Fe;
                }(ee.Evented);
                function zn(Ke) {
                  if (Ke) {
                    if (typeof Ke == "number") {
                      var Fe = Math.round(Math.sqrt(0.5 * Math.pow(Ke, 2)));
                      return { center: new ee.Point(0, 0), top: new ee.Point(0, Ke), "top-left": new ee.Point(Fe, Fe), "top-right": new ee.Point(-Fe, Fe), bottom: new ee.Point(0, -Ke), "bottom-left": new ee.Point(Fe, -Fe), "bottom-right": new ee.Point(-Fe, -Fe), left: new ee.Point(Ke, 0), right: new ee.Point(-Ke, 0) };
                    }
                    if (Ke instanceof ee.Point || Array.isArray(Ke)) {
                      var Te = ee.Point.convert(Ke);
                      return { center: Te, top: Te, "top-left": Te, "top-right": Te, bottom: Te, "bottom-left": Te, "bottom-right": Te, left: Te, right: Te };
                    }
                    return { center: ee.Point.convert(Ke.center || [0, 0]), top: ee.Point.convert(Ke.top || [0, 0]), "top-left": ee.Point.convert(Ke["top-left"] || [0, 0]), "top-right": ee.Point.convert(Ke["top-right"] || [0, 0]), bottom: ee.Point.convert(Ke.bottom || [0, 0]), "bottom-left": ee.Point.convert(Ke["bottom-left"] || [0, 0]), "bottom-right": ee.Point.convert(Ke["bottom-right"] || [0, 0]), left: ee.Point.convert(Ke.left || [0, 0]), right: ee.Point.convert(Ke.right || [0, 0]) };
                  }
                  return zn(new ee.Point(0, 0));
                }
                var Gn = { version: ee.version, supported: ae, setRTLTextPlugin: ee.setRTLTextPlugin, getRTLTextPluginStatus: ee.getRTLTextPluginStatus, Map: hu, NavigationControl: Yc, GeolocateControl: Ul, AttributionControl: hc, ScaleControl: Os, FullscreenControl: Iu, Popup: $n, Marker: $l, Style: ki, LngLat: ee.LngLat, LngLatBounds: ee.LngLatBounds, Point: ee.Point, MercatorCoordinate: ee.MercatorCoordinate, Evented: ee.Evented, config: ee.config, prewarm: function() {
                  In().acquire(an);
                }, clearPrewarmedResources: function() {
                  var Ke = Mn;
                  Ke && (Ke.isPreloaded() && Ke.numActive() === 1 ? (Ke.release(an), Mn = null) : console.warn("Could not clear WebWorkers since there are active Map instances that still reference it. The pre-warmed WebWorker pool can only be cleared when all map instances have been removed with map.remove()"));
                }, get accessToken() {
                  return ee.config.ACCESS_TOKEN;
                }, set accessToken(Ke) {
                  ee.config.ACCESS_TOKEN = Ke;
                }, get baseApiUrl() {
                  return ee.config.API_URL;
                }, set baseApiUrl(Ke) {
                  ee.config.API_URL = Ke;
                }, get workerCount() {
                  return Zt.workerCount;
                }, set workerCount(Ke) {
                  Zt.workerCount = Ke;
                }, get maxParallelImageRequests() {
                  return ee.config.MAX_PARALLEL_IMAGE_REQUESTS;
                }, set maxParallelImageRequests(Ke) {
                  ee.config.MAX_PARALLEL_IMAGE_REQUESTS = Ke;
                }, clearStorage: function(Ke) {
                  ee.clearTileCache(Ke);
                }, workerUrl: "" };
                return Gn;
              }), te;
            }();
          }, 27549: function(G, J, Z) {
            G.exports = Z(55366);
          }, 55366: function(G, J, Z) {
            var te = Z(31625), ne = Z(75144), ee = Z(5137), ae = Z(78112), oe = Z(6807), se = Z(68650), ce = Z(83473), le = Z(60201), ue = Z(10275), de = Z(62914);
            function fe(he, me) {
              for (var pe = me[0], ye = me[1], ge = 1 / (me[2] - pe), ve = 1 / (me[3] - ye), be = new Array(he.length), Ce = 0, Se = he.length / 2; Ce < Se; Ce++) be[2 * Ce] = ne((he[2 * Ce] - pe) * ge, 0, 1), be[2 * Ce + 1] = ne((he[2 * Ce + 1] - ye) * ve, 0, 1);
              return be;
            }
            G.exports = function(he, me) {
              me || (me = {}), he = ce(he, "float64"), me = oe(me, { bounds: "range bounds dataBox databox", maxDepth: "depth maxDepth maxdepth level maxLevel maxlevel levels", dtype: "type dtype format out dst output destination" });
              var pe = se(me.maxDepth, 255), ye = se(me.bounds, ae(he, 2));
              ye[0] === ye[2] && ye[2]++, ye[1] === ye[3] && ye[3]++;
              var ge, ve = fe(he, ye), be = he.length >>> 1;
              me.dtype || (me.dtype = "array"), typeof me.dtype == "string" ? ge = new (ue(me.dtype))(be) : me.dtype && (ge = me.dtype, Array.isArray(ge) && (ge.length = be));
              for (var Ce = 0; Ce < be; ++Ce) ge[Ce] = Ce;
              var Se = [], we = [], Ee = [], xe = [];
              (function $e(qe, Xe, Ve, Ze, Ge, ze) {
                if (!Ze.length) return null;
                var We = Se[Ge] || (Se[Ge] = []), at = Ee[Ge] || (Ee[Ge] = []), ct = we[Ge] || (we[Ge] = []), et = We.length;
                if (++Ge > pe || ze > 1073741824) {
                  for (var dt = 0; dt < Ze.length; dt++) We.push(Ze[dt]), at.push(ze), ct.push(null, null, null, null);
                  return et;
                }
                if (We.push(Ze[0]), at.push(ze), Ze.length <= 1) return ct.push(null, null, null, null), et;
                for (var Je = 0.5 * Ve, it = qe + Je, st = Xe + Je, gt = [], bt = [], yt = [], St = [], Lt = 1, Rt = Ze.length; Lt < Rt; Lt++) {
                  var vt = Ze[Lt], $t = ve[2 * vt], Pt = ve[2 * vt + 1];
                  $t < it ? Pt < st ? gt.push(vt) : bt.push(vt) : Pt < st ? yt.push(vt) : St.push(vt);
                }
                return ze <<= 2, ct.push($e(qe, Xe, Je, gt, Ge, ze), $e(qe, st, Je, bt, Ge, ze + 1), $e(it, Xe, Je, yt, Ge, ze + 2), $e(it, st, Je, St, Ge, ze + 3)), et;
              })(0, 0, 1, ge, 0, 1);
              for (var Le = 0, Qe = 0; Qe < Se.length; Qe++) {
                var ke = Se[Qe];
                if (ge.set) ge.set(ke, Le);
                else for (var Me = 0, Re = ke.length; Me < Re; Me++) ge[Me + Le] = ke[Me];
                var De = Le + Se[Qe].length;
                xe[Qe] = [Le, De], Le = De;
              }
              return ge.range = function() {
                for (var $e, qe = [], Xe = arguments.length; Xe--; ) qe[Xe] = arguments[Xe];
                if (le(qe[qe.length - 1])) {
                  var Ve = qe.pop();
                  qe.length || Ve.x == null && Ve.l == null && Ve.left == null || (qe = [Ve], $e = {}), $e = oe(Ve, { level: "level maxLevel", d: "d diam diameter r radius px pxSize pixel pixelSize maxD size minSize", lod: "lod details ranges offsets" });
                } else $e = {};
                qe.length || (qe = ye);
                var Ze, Ge = ee.apply(void 0, qe), ze = [Math.min(Ge.x, Ge.x + Ge.width), Math.min(Ge.y, Ge.y + Ge.height), Math.max(Ge.x, Ge.x + Ge.width), Math.max(Ge.y, Ge.y + Ge.height)], We = ze[0], at = ze[1], ct = ze[2], et = ze[3], dt = fe([We, at, ct, et], ye), Je = dt[0], it = dt[1], st = dt[2], gt = dt[3], bt = se($e.level, Se.length);
                if ($e.d != null && (typeof $e.d == "number" ? Ze = [$e.d, $e.d] : $e.d.length && (Ze = $e.d), bt = Math.min(Math.max(Math.ceil(-de(Math.abs(Ze[0]) / (ye[2] - ye[0]))), Math.ceil(-de(Math.abs(Ze[1]) / (ye[3] - ye[1])))), bt)), bt = Math.min(bt, Se.length), $e.lod) return function(St, Lt, Rt, vt, $t) {
                  for (var Pt = [], Xt = 0; Xt < $t; Xt++) {
                    var Nt = Ee[Xt], Gt = xe[Xt][0], Ft = Pe(St, Lt, Xt), Tt = Pe(Rt, vt, Xt), It = te.ge(Nt, Ft), Ct = te.gt(Nt, Tt, It, Nt.length - 1);
                    Pt[Xt] = [It + Gt, Ct + Gt];
                  }
                  return Pt;
                }(Je, it, st, gt, bt);
                var yt = [];
                return function St(Lt, Rt, vt, $t, Pt, Xt) {
                  if (Pt !== null && Xt !== null && !(Je > Lt + vt || it > Rt + vt || st < Lt || gt < Rt || $t >= bt || Pt === Xt)) {
                    var Nt = Se[$t];
                    Xt === void 0 && (Xt = Nt.length);
                    for (var Gt = Pt; Gt < Xt; Gt++) {
                      var Ft = Nt[Gt], Tt = he[2 * Ft], It = he[2 * Ft + 1];
                      Tt >= We && Tt <= ct && It >= at && It <= et && yt.push(Ft);
                    }
                    var Ct = we[$t], Kt = Ct[4 * Pt + 0], Vt = Ct[4 * Pt + 1], Yt = Ct[4 * Pt + 2], sn = Ct[4 * Pt + 3], hn = function(Wt, rn) {
                      for (var vn = null, dn = 0; vn === null; ) if (vn = Wt[4 * rn + dn], ++dn > Wt.length) return null;
                      return vn;
                    }(Ct, Pt + 1), Qt = 0.5 * vt, Ut = $t + 1;
                    St(Lt, Rt, Qt, Ut, Kt, Vt || Yt || sn || hn), St(Lt, Rt + Qt, Qt, Ut, Vt, Yt || sn || hn), St(Lt + Qt, Rt, Qt, Ut, Yt, sn || hn), St(Lt + Qt, Rt + Qt, Qt, Ut, sn, hn);
                  }
                }(0, 0, 1, 0, 0, 1), yt;
              }, ge;
              function Pe($e, qe, Xe) {
                for (var Ve = 1, Ze = 0.5, Ge = 0.5, ze = 0.5, We = 0; We < Xe; We++) Ve <<= 2, Ve += $e < Ze ? qe < Ge ? 0 : 1 : qe < Ge ? 2 : 3, ze *= 0.5, Ze += $e < Ze ? -ze : ze, Ge += qe < Ge ? -ze : ze;
                return Ve;
              }
            };
          }, 16844: function(G) {
            G.exports = function(J) {
              var Z = 0, te = 0, ne = 0, ee = 0;
              return J.map(function(ae) {
                var oe = (ae = ae.slice())[0], se = oe.toUpperCase();
                if (oe != se) switch (ae[0] = se, oe) {
                  case "a":
                    ae[6] += ne, ae[7] += ee;
                    break;
                  case "v":
                    ae[1] += ee;
                    break;
                  case "h":
                    ae[1] += ne;
                    break;
                  default:
                    for (var ce = 1; ce < ae.length; ) ae[ce++] += ne, ae[ce++] += ee;
                }
                switch (se) {
                  case "Z":
                    ne = Z, ee = te;
                    break;
                  case "H":
                    ne = ae[1];
                    break;
                  case "V":
                    ee = ae[1];
                    break;
                  case "M":
                    ne = Z = ae[1], ee = te = ae[2];
                    break;
                  default:
                    ne = ae[ae.length - 2], ee = ae[ae.length - 1];
                }
                return ae;
              });
            };
          }, 78112: function(G) {
            G.exports = function(J, Z) {
              if (!J || J.length == null) throw Error("Argument should be an array");
              Z = Z == null ? 1 : Math.floor(Z);
              for (var te = Array(2 * Z), ne = 0; ne < Z; ne++) {
                for (var ee = -1 / 0, ae = 1 / 0, oe = ne, se = J.length; oe < se; oe += Z) J[oe] > ee && (ee = J[oe]), J[oe] < ae && (ae = J[oe]);
                te[ne] = ae, te[Z + ne] = ee;
              }
              return te;
            };
          }, 33055: function(G) {
            G.exports = function(J, Z, te) {
              if (typeof Array.prototype.findIndex == "function") return J.findIndex(Z, te);
              if (typeof Z != "function") throw new TypeError("predicate must be a function");
              var ne = Object(J), ee = ne.length;
              if (ee === 0) return -1;
              for (var ae = 0; ae < ee; ae++) if (Z.call(te, ne[ae], ae, ne)) return ae;
              return -1;
            };
          }, 90956: function(G, J, Z) {
            var te = Z(78112);
            G.exports = function(ne, ee, ae) {
              if (!ne || ne.length == null) throw Error("Argument should be an array");
              ee == null && (ee = 1), ae == null && (ae = te(ne, ee));
              for (var oe = 0; oe < ee; oe++) {
                var se = ae[ee + oe], ce = ae[oe], le = oe, ue = ne.length;
                if (se === 1 / 0 && ce === -1 / 0) for (le = oe; le < ue; le += ee) ne[le] = ne[le] === se ? 1 : ne[le] === ce ? 0 : 0.5;
                else if (se === 1 / 0) for (le = oe; le < ue; le += ee) ne[le] = ne[le] === se ? 1 : 0;
                else if (ce === -1 / 0) for (le = oe; le < ue; le += ee) ne[le] = ne[le] === ce ? 0 : 1;
                else {
                  var de = se - ce;
                  for (le = oe; le < ue; le += ee) isNaN(ne[le]) || (ne[le] = de === 0 ? 0.5 : (ne[le] - ce) / de);
                }
              }
              return ne;
            };
          }, 27902: function(G) {
            G.exports = function(J, Z) {
              var te = typeof J == "number", ne = typeof Z == "number";
              te && !ne ? (Z = J, J = 0) : te || ne || (J = 0, Z = 0);
              var ee = (Z |= 0) - (J |= 0);
              if (ee < 0) throw new Error("array length must be positive");
              for (var ae = new Array(ee), oe = 0, se = J; oe < ee; oe++, se++) ae[oe] = se;
              return ae;
            };
          }, 85672: function(G, J, Z) {
            var te = Z(33282);
            function ne(Ge) {
              return ne = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(ze) {
                return typeof ze;
              } : function(ze) {
                return ze && typeof Symbol == "function" && ze.constructor === Symbol && ze !== Symbol.prototype ? "symbol" : typeof ze;
              }, ne(Ge);
            }
            function ee(Ge, ze, We) {
              return Object.defineProperty(Ge, "prototype", { writable: !1 }), Ge;
            }
            var ae, oe, se = Z(34585).codes, ce = se.ERR_AMBIGUOUS_ARGUMENT, le = se.ERR_INVALID_ARG_TYPE, ue = se.ERR_INVALID_ARG_VALUE, de = se.ERR_INVALID_RETURN_VALUE, fe = se.ERR_MISSING_ARGS, he = Z(68586), me = Z(56557).inspect, pe = Z(56557).types, ye = pe.isPromise, ge = pe.isRegExp, ve = Z(68686)(), be = Z(9622)(), Ce = Z(63063)("RegExp.prototype.test");
            function Se() {
              var Ge = Z(23879);
              ae = Ge.isDeepEqual, oe = Ge.isDeepStrictEqual;
            }
            var we = !1, Ee = G.exports = ke, xe = {};
            function Le(Ge) {
              throw Ge.message instanceof Error ? Ge.message : new he(Ge);
            }
            function Qe(Ge, ze, We, at) {
              if (!We) {
                var ct = !1;
                if (ze === 0) ct = !0, at = "No value argument passed to `assert.ok()`";
                else if (at instanceof Error) throw at;
                var et = new he({ actual: We, expected: !0, message: at, operator: "==", stackStartFn: Ge });
                throw et.generatedMessage = ct, et;
              }
            }
            function ke() {
              for (var Ge = arguments.length, ze = new Array(Ge), We = 0; We < Ge; We++) ze[We] = arguments[We];
              Qe.apply(void 0, [ke, ze.length].concat(ze));
            }
            Ee.fail = function Ge(ze, We, at, ct, et) {
              var dt, Je = arguments.length;
              if (Je === 0 ? dt = "Failed" : Je === 1 ? (at = ze, ze = void 0) : (we === !1 && (we = !0, (te.emitWarning ? te.emitWarning : console.warn.bind(console))("assert.fail() with more than one argument is deprecated. Please use assert.strictEqual() instead or only pass a message.", "DeprecationWarning", "DEP0094")), Je === 2 && (ct = "!=")), at instanceof Error) throw at;
              var it = { actual: ze, expected: We, operator: ct === void 0 ? "fail" : ct, stackStartFn: et || Ge };
              at !== void 0 && (it.message = at);
              var st = new he(it);
              throw dt && (st.message = dt, st.generatedMessage = !0), st;
            }, Ee.AssertionError = he, Ee.ok = ke, Ee.equal = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              ze != We && Le({ actual: ze, expected: We, message: at, operator: "==", stackStartFn: Ge });
            }, Ee.notEqual = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              ze == We && Le({ actual: ze, expected: We, message: at, operator: "!=", stackStartFn: Ge });
            }, Ee.deepEqual = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              ae === void 0 && Se(), ae(ze, We) || Le({ actual: ze, expected: We, message: at, operator: "deepEqual", stackStartFn: Ge });
            }, Ee.notDeepEqual = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              ae === void 0 && Se(), ae(ze, We) && Le({ actual: ze, expected: We, message: at, operator: "notDeepEqual", stackStartFn: Ge });
            }, Ee.deepStrictEqual = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              ae === void 0 && Se(), oe(ze, We) || Le({ actual: ze, expected: We, message: at, operator: "deepStrictEqual", stackStartFn: Ge });
            }, Ee.notDeepStrictEqual = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              ae === void 0 && Se(), oe(ze, We) && Le({ actual: ze, expected: We, message: at, operator: "notDeepStrictEqual", stackStartFn: Ge });
            }, Ee.strictEqual = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              be(ze, We) || Le({ actual: ze, expected: We, message: at, operator: "strictEqual", stackStartFn: Ge });
            }, Ee.notStrictEqual = function Ge(ze, We, at) {
              if (arguments.length < 2) throw new fe("actual", "expected");
              be(ze, We) && Le({ actual: ze, expected: We, message: at, operator: "notStrictEqual", stackStartFn: Ge });
            };
            var Me = ee(function Ge(ze, We, at) {
              var ct = this;
              (function(et, dt) {
                if (!(et instanceof dt)) throw new TypeError("Cannot call a class as a function");
              })(this, Ge), We.forEach(function(et) {
                et in ze && (at !== void 0 && typeof at[et] == "string" && ge(ze[et]) && Ce(ze[et], at[et]) ? ct[et] = at[et] : ct[et] = ze[et]);
              });
            });
            function Re(Ge, ze, We, at) {
              if (typeof ze != "function") {
                if (ge(ze)) return Ce(ze, Ge);
                if (arguments.length === 2) throw new le("expected", ["Function", "RegExp"], ze);
                if (ne(Ge) !== "object" || Ge === null) {
                  var ct = new he({ actual: Ge, expected: ze, message: We, operator: "deepStrictEqual", stackStartFn: at });
                  throw ct.operator = at.name, ct;
                }
                var et = Object.keys(ze);
                if (ze instanceof Error) et.push("name", "message");
                else if (et.length === 0) throw new ue("error", ze, "may not be an empty object");
                return ae === void 0 && Se(), et.forEach(function(dt) {
                  typeof Ge[dt] == "string" && ge(ze[dt]) && Ce(ze[dt], Ge[dt]) || function(Je, it, st, gt, bt, yt) {
                    if (!(st in Je) || !oe(Je[st], it[st])) {
                      if (!gt) {
                        var St = new Me(Je, bt), Lt = new Me(it, bt, Je), Rt = new he({ actual: St, expected: Lt, operator: "deepStrictEqual", stackStartFn: yt });
                        throw Rt.actual = Je, Rt.expected = it, Rt.operator = yt.name, Rt;
                      }
                      Le({ actual: Je, expected: it, message: gt, operator: yt.name, stackStartFn: yt });
                    }
                  }(Ge, ze, dt, We, et, at);
                }), !0;
              }
              return ze.prototype !== void 0 && Ge instanceof ze || !Error.isPrototypeOf(ze) && ze.call({}, Ge) === !0;
            }
            function De(Ge) {
              if (typeof Ge != "function") throw new le("fn", "Function", Ge);
              try {
                Ge();
              } catch (ze) {
                return ze;
              }
              return xe;
            }
            function Pe(Ge) {
              return ye(Ge) || Ge !== null && ne(Ge) === "object" && typeof Ge.then == "function" && typeof Ge.catch == "function";
            }
            function $e(Ge) {
              return Promise.resolve().then(function() {
                var ze;
                if (typeof Ge == "function") {
                  if (!Pe(ze = Ge())) throw new de("instance of Promise", "promiseFn", ze);
                } else {
                  if (!Pe(Ge)) throw new le("promiseFn", ["Function", "Promise"], Ge);
                  ze = Ge;
                }
                return Promise.resolve().then(function() {
                  return ze;
                }).then(function() {
                  return xe;
                }).catch(function(We) {
                  return We;
                });
              });
            }
            function qe(Ge, ze, We, at) {
              if (typeof We == "string") {
                if (arguments.length === 4) throw new le("error", ["Object", "Error", "Function", "RegExp"], We);
                if (ne(ze) === "object" && ze !== null) {
                  if (ze.message === We) throw new ce("error/message", 'The error message "'.concat(ze.message, '" is identical to the message.'));
                } else if (ze === We) throw new ce("error/message", 'The error "'.concat(ze, '" is identical to the message.'));
                at = We, We = void 0;
              } else if (We != null && ne(We) !== "object" && typeof We != "function") throw new le("error", ["Object", "Error", "Function", "RegExp"], We);
              if (ze === xe) {
                var ct = "";
                We && We.name && (ct += " (".concat(We.name, ")")), ct += at ? ": ".concat(at) : ".";
                var et = Ge.name === "rejects" ? "rejection" : "exception";
                Le({ actual: void 0, expected: We, operator: Ge.name, message: "Missing expected ".concat(et).concat(ct), stackStartFn: Ge });
              }
              if (We && !Re(ze, We, at, Ge)) throw ze;
            }
            function Xe(Ge, ze, We, at) {
              if (ze !== xe) {
                if (typeof We == "string" && (at = We, We = void 0), !We || Re(ze, We)) {
                  var ct = at ? ": ".concat(at) : ".", et = Ge.name === "doesNotReject" ? "rejection" : "exception";
                  Le({ actual: ze, expected: We, operator: Ge.name, message: "Got unwanted ".concat(et).concat(ct, `
`) + 'Actual message: "'.concat(ze && ze.message, '"'), stackStartFn: Ge });
                }
                throw ze;
              }
            }
            function Ve(Ge, ze, We, at, ct) {
              if (!ge(ze)) throw new le("regexp", "RegExp", ze);
              var et = ct === "match";
              if (typeof Ge != "string" || Ce(ze, Ge) !== et) {
                if (We instanceof Error) throw We;
                var dt = !We;
                We = We || (typeof Ge != "string" ? 'The "string" argument must be of type string. Received type ' + "".concat(ne(Ge), " (").concat(me(Ge), ")") : (et ? "The input did not match the regular expression " : "The input was expected to not match the regular expression ") + "".concat(me(ze), `. Input:

`).concat(me(Ge), `
`));
                var Je = new he({ actual: Ge, expected: ze, message: We, operator: ct, stackStartFn: at });
                throw Je.generatedMessage = dt, Je;
              }
            }
            function Ze() {
              for (var Ge = arguments.length, ze = new Array(Ge), We = 0; We < Ge; We++) ze[We] = arguments[We];
              Qe.apply(void 0, [Ze, ze.length].concat(ze));
            }
            Ee.throws = function Ge(ze) {
              for (var We = arguments.length, at = new Array(We > 1 ? We - 1 : 0), ct = 1; ct < We; ct++) at[ct - 1] = arguments[ct];
              qe.apply(void 0, [Ge, De(ze)].concat(at));
            }, Ee.rejects = function Ge(ze) {
              for (var We = arguments.length, at = new Array(We > 1 ? We - 1 : 0), ct = 1; ct < We; ct++) at[ct - 1] = arguments[ct];
              return $e(ze).then(function(et) {
                return qe.apply(void 0, [Ge, et].concat(at));
              });
            }, Ee.doesNotThrow = function Ge(ze) {
              for (var We = arguments.length, at = new Array(We > 1 ? We - 1 : 0), ct = 1; ct < We; ct++) at[ct - 1] = arguments[ct];
              Xe.apply(void 0, [Ge, De(ze)].concat(at));
            }, Ee.doesNotReject = function Ge(ze) {
              for (var We = arguments.length, at = new Array(We > 1 ? We - 1 : 0), ct = 1; ct < We; ct++) at[ct - 1] = arguments[ct];
              return $e(ze).then(function(et) {
                return Xe.apply(void 0, [Ge, et].concat(at));
              });
            }, Ee.ifError = function Ge(ze) {
              if (ze != null) {
                var We = "ifError got unwanted exception: ";
                ne(ze) === "object" && typeof ze.message == "string" ? ze.message.length === 0 && ze.constructor ? We += ze.constructor.name : We += ze.message : We += me(ze);
                var at = new he({ actual: ze, expected: null, operator: "ifError", message: We, stackStartFn: Ge }), ct = ze.stack;
                if (typeof ct == "string") {
                  var et = ct.split(`
`);
                  et.shift();
                  for (var dt = at.stack.split(`
`), Je = 0; Je < et.length; Je++) {
                    var it = dt.indexOf(et[Je]);
                    if (it !== -1) {
                      dt = dt.slice(0, it);
                      break;
                    }
                  }
                  at.stack = "".concat(dt.join(`
`), `
`).concat(et.join(`
`));
                }
                throw at;
              }
            }, Ee.match = function Ge(ze, We, at) {
              Ve(ze, We, at, Ge, "match");
            }, Ee.doesNotMatch = function Ge(ze, We, at) {
              Ve(ze, We, at, Ge, "doesNotMatch");
            }, Ee.strict = ve(Ze, Ee, { equal: Ee.strictEqual, deepEqual: Ee.deepStrictEqual, notEqual: Ee.notStrictEqual, notDeepEqual: Ee.notDeepStrictEqual }), Ee.strict.strict = Ee.strict;
          }, 68586: function(G, J, Z) {
            var te = Z(33282);
            function ne(Qe, ke) {
              var Me = Object.keys(Qe);
              if (Object.getOwnPropertySymbols) {
                var Re = Object.getOwnPropertySymbols(Qe);
                ke && (Re = Re.filter(function(De) {
                  return Object.getOwnPropertyDescriptor(Qe, De).enumerable;
                })), Me.push.apply(Me, Re);
              }
              return Me;
            }
            function ee(Qe) {
              for (var ke = 1; ke < arguments.length; ke++) {
                var Me = arguments[ke] != null ? arguments[ke] : {};
                ke % 2 ? ne(Object(Me), !0).forEach(function(Re) {
                  var De, Pe, $e;
                  De = Qe, Pe = Re, $e = Me[Re], (Pe = oe(Pe)) in De ? Object.defineProperty(De, Pe, { value: $e, enumerable: !0, configurable: !0, writable: !0 }) : De[Pe] = $e;
                }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(Qe, Object.getOwnPropertyDescriptors(Me)) : ne(Object(Me)).forEach(function(Re) {
                  Object.defineProperty(Qe, Re, Object.getOwnPropertyDescriptor(Me, Re));
                });
              }
              return Qe;
            }
            function ae(Qe, ke) {
              for (var Me = 0; Me < ke.length; Me++) {
                var Re = ke[Me];
                Re.enumerable = Re.enumerable || !1, Re.configurable = !0, "value" in Re && (Re.writable = !0), Object.defineProperty(Qe, oe(Re.key), Re);
              }
            }
            function oe(Qe) {
              var ke = function(Me, Re) {
                if (me(Me) !== "object" || Me === null) return Me;
                var De = Me[Symbol.toPrimitive];
                if (De !== void 0) {
                  var Pe = De.call(Me, "string");
                  if (me(Pe) !== "object") return Pe;
                  throw new TypeError("@@toPrimitive must return a primitive value.");
                }
                return String(Me);
              }(Qe);
              return me(ke) === "symbol" ? ke : String(ke);
            }
            function se(Qe, ke) {
              if (ke && (me(ke) === "object" || typeof ke == "function")) return ke;
              if (ke !== void 0) throw new TypeError("Derived constructors may only return object or undefined");
              return ce(Qe);
            }
            function ce(Qe) {
              if (Qe === void 0) throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
              return Qe;
            }
            function le(Qe) {
              var ke = typeof Map == "function" ? /* @__PURE__ */ new Map() : void 0;
              return le = function(Me) {
                if (Me === null || (Re = Me, Function.toString.call(Re).indexOf("[native code]") === -1)) return Me;
                var Re;
                if (typeof Me != "function") throw new TypeError("Super expression must either be null or a function");
                if (ke !== void 0) {
                  if (ke.has(Me)) return ke.get(Me);
                  ke.set(Me, De);
                }
                function De() {
                  return ue(Me, arguments, he(this).constructor);
                }
                return De.prototype = Object.create(Me.prototype, { constructor: { value: De, enumerable: !1, writable: !0, configurable: !0 } }), fe(De, Me);
              }, le(Qe);
            }
            function ue(Qe, ke, Me) {
              return ue = de() ? Reflect.construct.bind() : function(Re, De, Pe) {
                var $e = [null];
                $e.push.apply($e, De);
                var qe = new (Function.bind.apply(Re, $e))();
                return Pe && fe(qe, Pe.prototype), qe;
              }, ue.apply(null, arguments);
            }
            function de() {
              if (typeof Reflect > "u" || !Reflect.construct || Reflect.construct.sham) return !1;
              if (typeof Proxy == "function") return !0;
              try {
                return Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {
                })), !0;
              } catch {
                return !1;
              }
            }
            function fe(Qe, ke) {
              return fe = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(Me, Re) {
                return Me.__proto__ = Re, Me;
              }, fe(Qe, ke);
            }
            function he(Qe) {
              return he = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function(ke) {
                return ke.__proto__ || Object.getPrototypeOf(ke);
              }, he(Qe);
            }
            function me(Qe) {
              return me = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(ke) {
                return typeof ke;
              } : function(ke) {
                return ke && typeof Symbol == "function" && ke.constructor === Symbol && ke !== Symbol.prototype ? "symbol" : typeof ke;
              }, me(Qe);
            }
            var pe = Z(56557).inspect, ye = Z(34585).codes.ERR_INVALID_ARG_TYPE;
            function ge(Qe, ke, Me) {
              return (Me === void 0 || Me > Qe.length) && (Me = Qe.length), Qe.substring(Me - ke.length, Me) === ke;
            }
            var ve = "", be = "", Ce = "", Se = "", we = { deepStrictEqual: "Expected values to be strictly deep-equal:", strictEqual: "Expected values to be strictly equal:", strictEqualObject: 'Expected "actual" to be reference-equal to "expected":', deepEqual: "Expected values to be loosely deep-equal:", equal: "Expected values to be loosely equal:", notDeepStrictEqual: 'Expected "actual" not to be strictly deep-equal to:', notStrictEqual: 'Expected "actual" to be strictly unequal to:', notStrictEqualObject: 'Expected "actual" not to be reference-equal to "expected":', notDeepEqual: 'Expected "actual" not to be loosely deep-equal to:', notEqual: 'Expected "actual" to be loosely unequal to:', notIdentical: "Values identical but not reference-equal:" };
            function Ee(Qe) {
              var ke = Object.keys(Qe), Me = Object.create(Object.getPrototypeOf(Qe));
              return ke.forEach(function(Re) {
                Me[Re] = Qe[Re];
              }), Object.defineProperty(Me, "message", { value: Qe.message }), Me;
            }
            function xe(Qe) {
              return pe(Qe, { compact: !1, customInspect: !1, depth: 1e3, maxArrayLength: 1 / 0, showHidden: !1, breakLength: 1 / 0, showProxy: !1, sorted: !0, getters: !0 });
            }
            var Le = function(Qe, ke) {
              (function(Xe, Ve) {
                if (typeof Ve != "function" && Ve !== null) throw new TypeError("Super expression must either be null or a function");
                Xe.prototype = Object.create(Ve && Ve.prototype, { constructor: { value: Xe, writable: !0, configurable: !0 } }), Object.defineProperty(Xe, "prototype", { writable: !1 }), Ve && fe(Xe, Ve);
              })(qe, Qe);
              var Me, Re, De, Pe, $e = (Me = qe, Re = de(), function() {
                var Xe, Ve = he(Me);
                if (Re) {
                  var Ze = he(this).constructor;
                  Xe = Reflect.construct(Ve, arguments, Ze);
                } else Xe = Ve.apply(this, arguments);
                return se(this, Xe);
              });
              function qe(Xe) {
                var Ve;
                if (function(gt, bt) {
                  if (!(gt instanceof bt)) throw new TypeError("Cannot call a class as a function");
                }(this, qe), me(Xe) !== "object" || Xe === null) throw new ye("options", "Object", Xe);
                var Ze = Xe.message, Ge = Xe.operator, ze = Xe.stackStartFn, We = Xe.actual, at = Xe.expected, ct = Error.stackTraceLimit;
                if (Error.stackTraceLimit = 0, Ze != null) Ve = $e.call(this, String(Ze));
                else if (te.stderr && te.stderr.isTTY && (te.stderr && te.stderr.getColorDepth && te.stderr.getColorDepth() !== 1 ? (ve = "\x1B[34m", be = "\x1B[32m", Se = "\x1B[39m", Ce = "\x1B[31m") : (ve = "", be = "", Se = "", Ce = "")), me(We) === "object" && We !== null && me(at) === "object" && at !== null && "stack" in We && We instanceof Error && "stack" in at && at instanceof Error && (We = Ee(We), at = Ee(at)), Ge === "deepStrictEqual" || Ge === "strictEqual") Ve = $e.call(this, function(gt, bt, yt) {
                  var St = "", Lt = "", Rt = 0, vt = "", $t = !1, Pt = xe(gt), Xt = Pt.split(`
`), Nt = xe(bt).split(`
`), Gt = 0, Ft = "";
                  if (yt === "strictEqual" && me(gt) === "object" && me(bt) === "object" && gt !== null && bt !== null && (yt = "strictEqualObject"), Xt.length === 1 && Nt.length === 1 && Xt[0] !== Nt[0]) {
                    var Tt = Xt[0].length + Nt[0].length;
                    if (Tt <= 10) {
                      if (!(me(gt) === "object" && gt !== null || me(bt) === "object" && bt !== null || gt === 0 && bt === 0)) return "".concat(we[yt], `

`) + "".concat(Xt[0], " !== ").concat(Nt[0], `
`);
                    } else if (yt !== "strictEqualObject" && Tt < (te.stderr && te.stderr.isTTY ? te.stderr.columns : 80)) {
                      for (; Xt[0][Gt] === Nt[0][Gt]; ) Gt++;
                      Gt > 2 && (Ft = `
  `.concat(function(vn, dn) {
                        if (dn = Math.floor(dn), vn.length == 0 || dn == 0) return "";
                        var Ht = vn.length * dn;
                        for (dn = Math.floor(Math.log(dn) / Math.log(2)); dn; ) vn += vn, dn--;
                        return vn + vn.substring(0, Ht - vn.length);
                      }(" ", Gt), "^"), Gt = 0);
                    }
                  }
                  for (var It = Xt[Xt.length - 1], Ct = Nt[Nt.length - 1]; It === Ct && (Gt++ < 2 ? vt = `
  `.concat(It).concat(vt) : St = It, Xt.pop(), Nt.pop(), Xt.length !== 0 && Nt.length !== 0); ) It = Xt[Xt.length - 1], Ct = Nt[Nt.length - 1];
                  var Kt = Math.max(Xt.length, Nt.length);
                  if (Kt === 0) {
                    var Vt = Pt.split(`
`);
                    if (Vt.length > 30) for (Vt[26] = "".concat(ve, "...").concat(Se); Vt.length > 27; ) Vt.pop();
                    return "".concat(we.notIdentical, `

`).concat(Vt.join(`
`), `
`);
                  }
                  Gt > 3 && (vt = `
`.concat(ve, "...").concat(Se).concat(vt), $t = !0), St !== "" && (vt = `
  `.concat(St).concat(vt), St = "");
                  var Yt = 0, sn = we[yt] + `
`.concat(be, "+ actual").concat(Se, " ").concat(Ce, "- expected").concat(Se), hn = " ".concat(ve, "...").concat(Se, " Lines skipped");
                  for (Gt = 0; Gt < Kt; Gt++) {
                    var Qt = Gt - Rt;
                    if (Xt.length < Gt + 1) Qt > 1 && Gt > 2 && (Qt > 4 ? (Lt += `
`.concat(ve, "...").concat(Se), $t = !0) : Qt > 3 && (Lt += `
  `.concat(Nt[Gt - 2]), Yt++), Lt += `
  `.concat(Nt[Gt - 1]), Yt++), Rt = Gt, St += `
`.concat(Ce, "-").concat(Se, " ").concat(Nt[Gt]), Yt++;
                    else if (Nt.length < Gt + 1) Qt > 1 && Gt > 2 && (Qt > 4 ? (Lt += `
`.concat(ve, "...").concat(Se), $t = !0) : Qt > 3 && (Lt += `
  `.concat(Xt[Gt - 2]), Yt++), Lt += `
  `.concat(Xt[Gt - 1]), Yt++), Rt = Gt, Lt += `
`.concat(be, "+").concat(Se, " ").concat(Xt[Gt]), Yt++;
                    else {
                      var Ut = Nt[Gt], Wt = Xt[Gt], rn = Wt !== Ut && (!ge(Wt, ",") || Wt.slice(0, -1) !== Ut);
                      rn && ge(Ut, ",") && Ut.slice(0, -1) === Wt && (rn = !1, Wt += ","), rn ? (Qt > 1 && Gt > 2 && (Qt > 4 ? (Lt += `
`.concat(ve, "...").concat(Se), $t = !0) : Qt > 3 && (Lt += `
  `.concat(Xt[Gt - 2]), Yt++), Lt += `
  `.concat(Xt[Gt - 1]), Yt++), Rt = Gt, Lt += `
`.concat(be, "+").concat(Se, " ").concat(Wt), St += `
`.concat(Ce, "-").concat(Se, " ").concat(Ut), Yt += 2) : (Lt += St, St = "", Qt !== 1 && Gt !== 0 || (Lt += `
  `.concat(Wt), Yt++));
                    }
                    if (Yt > 20 && Gt < Kt - 2) return "".concat(sn).concat(hn, `
`).concat(Lt, `
`).concat(ve, "...").concat(Se).concat(St, `
`) + "".concat(ve, "...").concat(Se);
                  }
                  return "".concat(sn).concat($t ? hn : "", `
`).concat(Lt).concat(St).concat(vt).concat(Ft);
                }(We, at, Ge));
                else if (Ge === "notDeepStrictEqual" || Ge === "notStrictEqual") {
                  var et = we[Ge], dt = xe(We).split(`
`);
                  if (Ge === "notStrictEqual" && me(We) === "object" && We !== null && (et = we.notStrictEqualObject), dt.length > 30) for (dt[26] = "".concat(ve, "...").concat(Se); dt.length > 27; ) dt.pop();
                  Ve = dt.length === 1 ? $e.call(this, "".concat(et, " ").concat(dt[0])) : $e.call(this, "".concat(et, `

`).concat(dt.join(`
`), `
`));
                } else {
                  var Je = xe(We), it = "", st = we[Ge];
                  Ge === "notDeepEqual" || Ge === "notEqual" ? (Je = "".concat(we[Ge], `

`).concat(Je)).length > 1024 && (Je = "".concat(Je.slice(0, 1021), "...")) : (it = "".concat(xe(at)), Je.length > 512 && (Je = "".concat(Je.slice(0, 509), "...")), it.length > 512 && (it = "".concat(it.slice(0, 509), "...")), Ge === "deepEqual" || Ge === "equal" ? Je = "".concat(st, `

`).concat(Je, `

should equal

`) : it = " ".concat(Ge, " ").concat(it)), Ve = $e.call(this, "".concat(Je).concat(it));
                }
                return Error.stackTraceLimit = ct, Ve.generatedMessage = !Ze, Object.defineProperty(ce(Ve), "name", { value: "AssertionError [ERR_ASSERTION]", enumerable: !1, writable: !0, configurable: !0 }), Ve.code = "ERR_ASSERTION", Ve.actual = We, Ve.expected = at, Ve.operator = Ge, Error.captureStackTrace && Error.captureStackTrace(ce(Ve), ze), Ve.stack, Ve.name = "AssertionError", se(Ve);
              }
              return De = qe, (Pe = [{ key: "toString", value: function() {
                return "".concat(this.name, " [").concat(this.code, "]: ").concat(this.message);
              } }, { key: ke, value: function(Xe, Ve) {
                return pe(this, ee(ee({}, Ve), {}, { customInspect: !1, depth: 0 }));
              } }]) && ae(De.prototype, Pe), Object.defineProperty(De, "prototype", { writable: !1 }), qe;
            }(le(Error), pe.custom);
            G.exports = Le;
          }, 34585: function(G, J, Z) {
            function te(ue) {
              return te = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(de) {
                return typeof de;
              } : function(de) {
                return de && typeof Symbol == "function" && de.constructor === Symbol && de !== Symbol.prototype ? "symbol" : typeof de;
              }, te(ue);
            }
            function ne(ue, de) {
              return ne = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(fe, he) {
                return fe.__proto__ = he, fe;
              }, ne(ue, de);
            }
            function ee(ue) {
              return ee = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function(de) {
                return de.__proto__ || Object.getPrototypeOf(de);
              }, ee(ue);
            }
            var ae, oe, se = {};
            function ce(ue, de, fe) {
              fe || (fe = Error);
              var he = function(me) {
                (function(Ce, Se) {
                  if (typeof Se != "function" && Se !== null) throw new TypeError("Super expression must either be null or a function");
                  Ce.prototype = Object.create(Se && Se.prototype, { constructor: { value: Ce, writable: !0, configurable: !0 } }), Object.defineProperty(Ce, "prototype", { writable: !1 }), Se && ne(Ce, Se);
                })(be, me);
                var pe, ye, ge, ve = (ye = be, ge = function() {
                  if (typeof Reflect > "u" || !Reflect.construct || Reflect.construct.sham) return !1;
                  if (typeof Proxy == "function") return !0;
                  try {
                    return Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {
                    })), !0;
                  } catch {
                    return !1;
                  }
                }(), function() {
                  var Ce, Se = ee(ye);
                  if (ge) {
                    var we = ee(this).constructor;
                    Ce = Reflect.construct(Se, arguments, we);
                  } else Ce = Se.apply(this, arguments);
                  return function(Ee, xe) {
                    if (xe && (te(xe) === "object" || typeof xe == "function")) return xe;
                    if (xe !== void 0) throw new TypeError("Derived constructors may only return object or undefined");
                    return function(Le) {
                      if (Le === void 0) throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
                      return Le;
                    }(Ee);
                  }(this, Ce);
                });
                function be(Ce, Se, we) {
                  var Ee;
                  return function(xe, Le) {
                    if (!(xe instanceof Le)) throw new TypeError("Cannot call a class as a function");
                  }(this, be), Ee = ve.call(this, function(xe, Le, Qe) {
                    return typeof de == "string" ? de : de(xe, Le, Qe);
                  }(Ce, Se, we)), Ee.code = ue, Ee;
                }
                return pe = be, Object.defineProperty(pe, "prototype", { writable: !1 }), pe;
              }(fe);
              se[ue] = he;
            }
            function le(ue, de) {
              if (Array.isArray(ue)) {
                var fe = ue.length;
                return ue = ue.map(function(he) {
                  return String(he);
                }), fe > 2 ? "one of ".concat(de, " ").concat(ue.slice(0, fe - 1).join(", "), ", or ") + ue[fe - 1] : fe === 2 ? "one of ".concat(de, " ").concat(ue[0], " or ").concat(ue[1]) : "of ".concat(de, " ").concat(ue[0]);
              }
              return "of ".concat(de, " ").concat(String(ue));
            }
            ce("ERR_AMBIGUOUS_ARGUMENT", 'The "%s" argument is ambiguous. %s', TypeError), ce("ERR_INVALID_ARG_TYPE", function(ue, de, fe) {
              var he, me, pe, ye, ge;
              if (ae === void 0 && (ae = Z(85672)), ae(typeof ue == "string", "'name' must be a string"), typeof de == "string" && (me = "not ", de.substr(0, 4) === me) ? (he = "must not be", de = de.replace(/^not /, "")) : he = "must be", function(be, Ce, Se) {
                return (Se === void 0 || Se > be.length) && (Se = be.length), be.substring(Se - 9, Se) === Ce;
              }(ue, " argument")) pe = "The ".concat(ue, " ").concat(he, " ").concat(le(de, "type"));
              else {
                var ve = (typeof ge != "number" && (ge = 0), ge + 1 > (ye = ue).length || ye.indexOf(".", ge) === -1 ? "argument" : "property");
                pe = 'The "'.concat(ue, '" ').concat(ve, " ").concat(he, " ").concat(le(de, "type"));
              }
              return pe + ". Received type ".concat(te(fe));
            }, TypeError), ce("ERR_INVALID_ARG_VALUE", function(ue, de) {
              var fe = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : "is invalid";
              oe === void 0 && (oe = Z(56557));
              var he = oe.inspect(de);
              return he.length > 128 && (he = "".concat(he.slice(0, 128), "...")), "The argument '".concat(ue, "' ").concat(fe, ". Received ").concat(he);
            }, TypeError), ce("ERR_INVALID_RETURN_VALUE", function(ue, de, fe) {
              var he;
              return he = fe && fe.constructor && fe.constructor.name ? "instance of ".concat(fe.constructor.name) : "type ".concat(te(fe)), "Expected ".concat(ue, ' to be returned from the "').concat(de, '"') + " function but got ".concat(he, ".");
            }, TypeError), ce("ERR_MISSING_ARGS", function() {
              for (var ue = arguments.length, de = new Array(ue), fe = 0; fe < ue; fe++) de[fe] = arguments[fe];
              ae === void 0 && (ae = Z(85672)), ae(de.length > 0, "At least one arg needs to be specified");
              var he = "The ", me = de.length;
              switch (de = de.map(function(pe) {
                return '"'.concat(pe, '"');
              }), me) {
                case 1:
                  he += "".concat(de[0], " argument");
                  break;
                case 2:
                  he += "".concat(de[0], " and ").concat(de[1], " arguments");
                  break;
                default:
                  he += de.slice(0, me - 1).join(", "), he += ", and ".concat(de[me - 1], " arguments");
              }
              return "".concat(he, " must be specified");
            }, TypeError), G.exports.codes = se;
          }, 23879: function(G, J, Z) {
            function te(st, gt) {
              return function(bt) {
                if (Array.isArray(bt)) return bt;
              }(st) || function(bt, yt) {
                var St = bt == null ? null : typeof Symbol < "u" && bt[Symbol.iterator] || bt["@@iterator"];
                if (St != null) {
                  var Lt, Rt, vt, $t, Pt = [], Xt = !0, Nt = !1;
                  try {
                    if (vt = (St = St.call(bt)).next, yt !== 0) for (; !(Xt = (Lt = vt.call(St)).done) && (Pt.push(Lt.value), Pt.length !== yt); Xt = !0) ;
                  } catch (Gt) {
                    Nt = !0, Rt = Gt;
                  } finally {
                    try {
                      if (!Xt && St.return != null && ($t = St.return(), Object($t) !== $t)) return;
                    } finally {
                      if (Nt) throw Rt;
                    }
                  }
                  return Pt;
                }
              }(st, gt) || function(bt, yt) {
                if (bt) {
                  if (typeof bt == "string") return ne(bt, yt);
                  var St = Object.prototype.toString.call(bt).slice(8, -1);
                  return St === "Object" && bt.constructor && (St = bt.constructor.name), St === "Map" || St === "Set" ? Array.from(bt) : St === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(St) ? ne(bt, yt) : void 0;
                }
              }(st, gt) || function() {
                throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
              }();
            }
            function ne(st, gt) {
              (gt == null || gt > st.length) && (gt = st.length);
              for (var bt = 0, yt = new Array(gt); bt < gt; bt++) yt[bt] = st[bt];
              return yt;
            }
            function ee(st) {
              return ee = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(gt) {
                return typeof gt;
              } : function(gt) {
                return gt && typeof Symbol == "function" && gt.constructor === Symbol && gt !== Symbol.prototype ? "symbol" : typeof gt;
              }, ee(st);
            }
            var ae = /a/g.flags !== void 0, oe = function(st) {
              var gt = [];
              return st.forEach(function(bt) {
                return gt.push(bt);
              }), gt;
            }, se = function(st) {
              var gt = [];
              return st.forEach(function(bt, yt) {
                return gt.push([yt, bt]);
              }), gt;
            }, ce = Object.is ? Object.is : Z(13969), le = Object.getOwnPropertySymbols ? Object.getOwnPropertySymbols : function() {
              return [];
            }, ue = Number.isNaN ? Number.isNaN : Z(63057);
            function de(st) {
              return st.call.bind(st);
            }
            var fe = de(Object.prototype.hasOwnProperty), he = de(Object.prototype.propertyIsEnumerable), me = de(Object.prototype.toString), pe = Z(56557).types, ye = pe.isAnyArrayBuffer, ge = pe.isArrayBufferView, ve = pe.isDate, be = pe.isMap, Ce = pe.isRegExp, Se = pe.isSet, we = pe.isNativeError, Ee = pe.isBoxedPrimitive, xe = pe.isNumberObject, Le = pe.isStringObject, Qe = pe.isBooleanObject, ke = pe.isBigIntObject, Me = pe.isSymbolObject, Re = pe.isFloat32Array, De = pe.isFloat64Array;
            function Pe(st) {
              if (st.length === 0 || st.length > 10) return !0;
              for (var gt = 0; gt < st.length; gt++) {
                var bt = st.charCodeAt(gt);
                if (bt < 48 || bt > 57) return !0;
              }
              return st.length === 10 && st >= Math.pow(2, 32);
            }
            function $e(st) {
              return Object.keys(st).filter(Pe).concat(le(st).filter(Object.prototype.propertyIsEnumerable.bind(st)));
            }
            function qe(st, gt) {
              if (st === gt) return 0;
              for (var bt = st.length, yt = gt.length, St = 0, Lt = Math.min(bt, yt); St < Lt; ++St) if (st[St] !== gt[St]) {
                bt = st[St], yt = gt[St];
                break;
              }
              return bt < yt ? -1 : yt < bt ? 1 : 0;
            }
            var Xe = 0, Ve = 1, Ze = 2, Ge = 3;
            function ze(st, gt, bt, yt) {
              if (st === gt) return st !== 0 || !bt || ce(st, gt);
              if (bt) {
                if (ee(st) !== "object") return typeof st == "number" && ue(st) && ue(gt);
                if (ee(gt) !== "object" || st === null || gt === null || Object.getPrototypeOf(st) !== Object.getPrototypeOf(gt)) return !1;
              } else {
                if (st === null || ee(st) !== "object") return (gt === null || ee(gt) !== "object") && st == gt;
                if (gt === null || ee(gt) !== "object") return !1;
              }
              var St, Lt, Rt, vt, $t = me(st);
              if ($t !== me(gt)) return !1;
              if (Array.isArray(st)) {
                if (st.length !== gt.length) return !1;
                var Pt = $e(st), Xt = $e(gt);
                return Pt.length === Xt.length && at(st, gt, bt, yt, Ve, Pt);
              }
              if ($t === "[object Object]" && (!be(st) && be(gt) || !Se(st) && Se(gt))) return !1;
              if (ve(st)) {
                if (!ve(gt) || Date.prototype.getTime.call(st) !== Date.prototype.getTime.call(gt)) return !1;
              } else if (Ce(st)) {
                if (!Ce(gt) || (Rt = st, vt = gt, !(ae ? Rt.source === vt.source && Rt.flags === vt.flags : RegExp.prototype.toString.call(Rt) === RegExp.prototype.toString.call(vt)))) return !1;
              } else if (we(st) || st instanceof Error) {
                if (st.message !== gt.message || st.name !== gt.name) return !1;
              } else {
                if (ge(st)) {
                  if (bt || !Re(st) && !De(st)) {
                    if (!function(Ft, Tt) {
                      return Ft.byteLength === Tt.byteLength && qe(new Uint8Array(Ft.buffer, Ft.byteOffset, Ft.byteLength), new Uint8Array(Tt.buffer, Tt.byteOffset, Tt.byteLength)) === 0;
                    }(st, gt)) return !1;
                  } else if (!function(Ft, Tt) {
                    if (Ft.byteLength !== Tt.byteLength) return !1;
                    for (var It = 0; It < Ft.byteLength; It++) if (Ft[It] !== Tt[It]) return !1;
                    return !0;
                  }(st, gt)) return !1;
                  var Nt = $e(st), Gt = $e(gt);
                  return Nt.length === Gt.length && at(st, gt, bt, yt, Xe, Nt);
                }
                if (Se(st)) return !(!Se(gt) || st.size !== gt.size) && at(st, gt, bt, yt, Ze);
                if (be(st)) return !(!be(gt) || st.size !== gt.size) && at(st, gt, bt, yt, Ge);
                if (ye(st)) {
                  if (Lt = gt, (St = st).byteLength !== Lt.byteLength || qe(new Uint8Array(St), new Uint8Array(Lt)) !== 0) return !1;
                } else if (Ee(st) && !function(Ft, Tt) {
                  return xe(Ft) ? xe(Tt) && ce(Number.prototype.valueOf.call(Ft), Number.prototype.valueOf.call(Tt)) : Le(Ft) ? Le(Tt) && String.prototype.valueOf.call(Ft) === String.prototype.valueOf.call(Tt) : Qe(Ft) ? Qe(Tt) && Boolean.prototype.valueOf.call(Ft) === Boolean.prototype.valueOf.call(Tt) : ke(Ft) ? ke(Tt) && BigInt.prototype.valueOf.call(Ft) === BigInt.prototype.valueOf.call(Tt) : Me(Tt) && Symbol.prototype.valueOf.call(Ft) === Symbol.prototype.valueOf.call(Tt);
                }(st, gt)) return !1;
              }
              return at(st, gt, bt, yt, Xe);
            }
            function We(st, gt) {
              return gt.filter(function(bt) {
                return he(st, bt);
              });
            }
            function at(st, gt, bt, yt, St, Lt) {
              if (arguments.length === 5) {
                Lt = Object.keys(st);
                var Rt = Object.keys(gt);
                if (Lt.length !== Rt.length) return !1;
              }
              for (var vt = 0; vt < Lt.length; vt++) if (!fe(gt, Lt[vt])) return !1;
              if (bt && arguments.length === 5) {
                var $t = le(st);
                if ($t.length !== 0) {
                  var Pt = 0;
                  for (vt = 0; vt < $t.length; vt++) {
                    var Xt = $t[vt];
                    if (he(st, Xt)) {
                      if (!he(gt, Xt)) return !1;
                      Lt.push(Xt), Pt++;
                    } else if (he(gt, Xt)) return !1;
                  }
                  var Nt = le(gt);
                  if ($t.length !== Nt.length && We(gt, Nt).length !== Pt) return !1;
                } else {
                  var Gt = le(gt);
                  if (Gt.length !== 0 && We(gt, Gt).length !== 0) return !1;
                }
              }
              if (Lt.length === 0 && (St === Xe || St === Ve && st.length === 0 || st.size === 0)) return !0;
              if (yt === void 0) yt = { val1: /* @__PURE__ */ new Map(), val2: /* @__PURE__ */ new Map(), position: 0 };
              else {
                var Ft = yt.val1.get(st);
                if (Ft !== void 0) {
                  var Tt = yt.val2.get(gt);
                  if (Tt !== void 0) return Ft === Tt;
                }
                yt.position++;
              }
              yt.val1.set(st, yt.position), yt.val2.set(gt, yt.position);
              var It = function(Ct, Kt, Vt, Yt, sn, hn) {
                var Qt = 0;
                if (hn === Ze) {
                  if (!function(vn, dn, Ht, jt) {
                    for (var cn = null, wn = oe(vn), Rn = 0; Rn < wn.length; Rn++) {
                      var Pn = wn[Rn];
                      if (ee(Pn) === "object" && Pn !== null) cn === null && (cn = /* @__PURE__ */ new Set()), cn.add(Pn);
                      else if (!dn.has(Pn)) {
                        if (Ht || !dt(vn, dn, Pn)) return !1;
                        cn === null && (cn = /* @__PURE__ */ new Set()), cn.add(Pn);
                      }
                    }
                    if (cn !== null) {
                      for (var Ie = oe(dn), rt = 0; rt < Ie.length; rt++) {
                        var ft = Ie[rt];
                        if (ee(ft) === "object" && ft !== null) {
                          if (!ct(cn, ft, Ht, jt)) return !1;
                        } else if (!Ht && !vn.has(ft) && !ct(cn, ft, Ht, jt)) return !1;
                      }
                      return cn.size === 0;
                    }
                    return !0;
                  }(Ct, Kt, Vt, sn)) return !1;
                } else if (hn === Ge) {
                  if (!function(vn, dn, Ht, jt) {
                    for (var cn = null, wn = se(vn), Rn = 0; Rn < wn.length; Rn++) {
                      var Pn = te(wn[Rn], 2), Ie = Pn[0], rt = Pn[1];
                      if (ee(Ie) === "object" && Ie !== null) cn === null && (cn = /* @__PURE__ */ new Set()), cn.add(Ie);
                      else {
                        var ft = dn.get(Ie);
                        if (ft === void 0 && !dn.has(Ie) || !ze(rt, ft, Ht, jt)) {
                          if (Ht || !Je(vn, dn, Ie, rt, jt)) return !1;
                          cn === null && (cn = /* @__PURE__ */ new Set()), cn.add(Ie);
                        }
                      }
                    }
                    if (cn !== null) {
                      for (var pt = se(dn), wt = 0; wt < pt.length; wt++) {
                        var Bt = te(pt[wt], 2), At = Bt[0], Dt = Bt[1];
                        if (ee(At) === "object" && At !== null) {
                          if (!it(cn, vn, At, Dt, Ht, jt)) return !1;
                        } else if (!(Ht || vn.has(At) && ze(vn.get(At), Dt, !1, jt) || it(cn, vn, At, Dt, !1, jt))) return !1;
                      }
                      return cn.size === 0;
                    }
                    return !0;
                  }(Ct, Kt, Vt, sn)) return !1;
                } else if (hn === Ve) for (; Qt < Ct.length; Qt++) {
                  if (!fe(Ct, Qt)) {
                    if (fe(Kt, Qt)) return !1;
                    for (var Ut = Object.keys(Ct); Qt < Ut.length; Qt++) {
                      var Wt = Ut[Qt];
                      if (!fe(Kt, Wt) || !ze(Ct[Wt], Kt[Wt], Vt, sn)) return !1;
                    }
                    return Ut.length === Object.keys(Kt).length;
                  }
                  if (!fe(Kt, Qt) || !ze(Ct[Qt], Kt[Qt], Vt, sn)) return !1;
                }
                for (Qt = 0; Qt < Yt.length; Qt++) {
                  var rn = Yt[Qt];
                  if (!ze(Ct[rn], Kt[rn], Vt, sn)) return !1;
                }
                return !0;
              }(st, gt, bt, Lt, yt, St);
              return yt.val1.delete(st), yt.val2.delete(gt), It;
            }
            function ct(st, gt, bt, yt) {
              for (var St = oe(st), Lt = 0; Lt < St.length; Lt++) {
                var Rt = St[Lt];
                if (ze(gt, Rt, bt, yt)) return st.delete(Rt), !0;
              }
              return !1;
            }
            function et(st) {
              switch (ee(st)) {
                case "undefined":
                  return null;
                case "object":
                  return;
                case "symbol":
                  return !1;
                case "string":
                  st = +st;
                case "number":
                  if (ue(st)) return !1;
              }
              return !0;
            }
            function dt(st, gt, bt) {
              var yt = et(bt);
              return yt ?? (gt.has(yt) && !st.has(yt));
            }
            function Je(st, gt, bt, yt, St) {
              var Lt = et(bt);
              if (Lt != null) return Lt;
              var Rt = gt.get(Lt);
              return !(Rt === void 0 && !gt.has(Lt) || !ze(yt, Rt, !1, St)) && !st.has(Lt) && ze(yt, Rt, !1, St);
            }
            function it(st, gt, bt, yt, St, Lt) {
              for (var Rt = oe(st), vt = 0; vt < Rt.length; vt++) {
                var $t = Rt[vt];
                if (ze(bt, $t, St, Lt) && ze(yt, gt.get($t), St, Lt)) return st.delete($t), !0;
              }
              return !1;
            }
            G.exports = { isDeepEqual: function(st, gt) {
              return ze(st, gt, !1);
            }, isDeepStrictEqual: function(st, gt) {
              return ze(st, gt, !0);
            } };
          }, 93229: function(G, J, Z) {
            Z.r(J), Z.d(J, { decode: function() {
              return oe;
            }, encode: function() {
              return ae;
            } });
            for (var te = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/", ne = typeof Uint8Array > "u" ? [] : new Uint8Array(256), ee = 0; ee < 64; ee++) ne[te.charCodeAt(ee)] = ee;
            var ae = function(se) {
              var ce, le = new Uint8Array(se), ue = le.length, de = "";
              for (ce = 0; ce < ue; ce += 3) de += te[le[ce] >> 2], de += te[(3 & le[ce]) << 4 | le[ce + 1] >> 4], de += te[(15 & le[ce + 1]) << 2 | le[ce + 2] >> 6], de += te[63 & le[ce + 2]];
              return ue % 3 == 2 ? de = de.substring(0, de.length - 1) + "=" : ue % 3 == 1 && (de = de.substring(0, de.length - 2) + "=="), de;
            }, oe = function(se) {
              var ce, le, ue, de, fe, he = 0.75 * se.length, me = se.length, pe = 0;
              se[se.length - 1] === "=" && (he--, se[se.length - 2] === "=" && he--);
              var ye = new ArrayBuffer(he), ge = new Uint8Array(ye);
              for (ce = 0; ce < me; ce += 4) le = ne[se.charCodeAt(ce)], ue = ne[se.charCodeAt(ce + 1)], de = ne[se.charCodeAt(ce + 2)], fe = ne[se.charCodeAt(ce + 3)], ge[pe++] = le << 2 | ue >> 4, ge[pe++] = (15 & ue) << 4 | de >> 2, ge[pe++] = (3 & de) << 6 | 63 & fe;
              return ye;
            };
          }, 76226: function(G, J) {
            J.byteLength = function(ce) {
              var le = oe(ce), ue = le[0], de = le[1];
              return 3 * (ue + de) / 4 - de;
            }, J.toByteArray = function(ce) {
              var le, ue, de = oe(ce), fe = de[0], he = de[1], me = new ne(function(ge, ve, be) {
                return 3 * (ve + be) / 4 - be;
              }(0, fe, he)), pe = 0, ye = he > 0 ? fe - 4 : fe;
              for (ue = 0; ue < ye; ue += 4) le = te[ce.charCodeAt(ue)] << 18 | te[ce.charCodeAt(ue + 1)] << 12 | te[ce.charCodeAt(ue + 2)] << 6 | te[ce.charCodeAt(ue + 3)], me[pe++] = le >> 16 & 255, me[pe++] = le >> 8 & 255, me[pe++] = 255 & le;
              return he === 2 && (le = te[ce.charCodeAt(ue)] << 2 | te[ce.charCodeAt(ue + 1)] >> 4, me[pe++] = 255 & le), he === 1 && (le = te[ce.charCodeAt(ue)] << 10 | te[ce.charCodeAt(ue + 1)] << 4 | te[ce.charCodeAt(ue + 2)] >> 2, me[pe++] = le >> 8 & 255, me[pe++] = 255 & le), me;
            }, J.fromByteArray = function(ce) {
              for (var le, ue = ce.length, de = ue % 3, fe = [], he = 16383, me = 0, pe = ue - de; me < pe; me += he) fe.push(se(ce, me, me + he > pe ? pe : me + he));
              return de === 1 ? (le = ce[ue - 1], fe.push(Z[le >> 2] + Z[le << 4 & 63] + "==")) : de === 2 && (le = (ce[ue - 2] << 8) + ce[ue - 1], fe.push(Z[le >> 10] + Z[le >> 4 & 63] + Z[le << 2 & 63] + "=")), fe.join("");
            };
            for (var Z = [], te = [], ne = typeof Uint8Array < "u" ? Uint8Array : Array, ee = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/", ae = 0; ae < 64; ++ae) Z[ae] = ee[ae], te[ee.charCodeAt(ae)] = ae;
            function oe(ce) {
              var le = ce.length;
              if (le % 4 > 0) throw new Error("Invalid string. Length must be a multiple of 4");
              var ue = ce.indexOf("=");
              return ue === -1 && (ue = le), [ue, ue === le ? 0 : 4 - ue % 4];
            }
            function se(ce, le, ue) {
              for (var de, fe, he = [], me = le; me < ue; me += 3) de = (ce[me] << 16 & 16711680) + (ce[me + 1] << 8 & 65280) + (255 & ce[me + 2]), he.push(Z[(fe = de) >> 18 & 63] + Z[fe >> 12 & 63] + Z[fe >> 6 & 63] + Z[63 & fe]);
              return he.join("");
            }
            te[45] = 62, te[95] = 63;
          }, 31625: function(G) {
            function J(oe, se, ce, le, ue) {
              for (var de = ue + 1; le <= ue; ) {
                var fe = le + ue >>> 1, he = oe[fe];
                (ce !== void 0 ? ce(he, se) : he - se) >= 0 ? (de = fe, ue = fe - 1) : le = fe + 1;
              }
              return de;
            }
            function Z(oe, se, ce, le, ue) {
              for (var de = ue + 1; le <= ue; ) {
                var fe = le + ue >>> 1, he = oe[fe];
                (ce !== void 0 ? ce(he, se) : he - se) > 0 ? (de = fe, ue = fe - 1) : le = fe + 1;
              }
              return de;
            }
            function te(oe, se, ce, le, ue) {
              for (var de = le - 1; le <= ue; ) {
                var fe = le + ue >>> 1, he = oe[fe];
                (ce !== void 0 ? ce(he, se) : he - se) < 0 ? (de = fe, le = fe + 1) : ue = fe - 1;
              }
              return de;
            }
            function ne(oe, se, ce, le, ue) {
              for (var de = le - 1; le <= ue; ) {
                var fe = le + ue >>> 1, he = oe[fe];
                (ce !== void 0 ? ce(he, se) : he - se) <= 0 ? (de = fe, le = fe + 1) : ue = fe - 1;
              }
              return de;
            }
            function ee(oe, se, ce, le, ue) {
              for (; le <= ue; ) {
                var de = le + ue >>> 1, fe = oe[de], he = ce !== void 0 ? ce(fe, se) : fe - se;
                if (he === 0) return de;
                he <= 0 ? le = de + 1 : ue = de - 1;
              }
              return -1;
            }
            function ae(oe, se, ce, le, ue, de) {
              return typeof ce == "function" ? de(oe, se, ce, le === void 0 ? 0 : 0 | le, ue === void 0 ? oe.length - 1 : 0 | ue) : de(oe, se, void 0, ce === void 0 ? 0 : 0 | ce, le === void 0 ? oe.length - 1 : 0 | le);
            }
            G.exports = { ge: function(oe, se, ce, le, ue) {
              return ae(oe, se, ce, le, ue, J);
            }, gt: function(oe, se, ce, le, ue) {
              return ae(oe, se, ce, le, ue, Z);
            }, lt: function(oe, se, ce, le, ue) {
              return ae(oe, se, ce, le, ue, te);
            }, le: function(oe, se, ce, le, ue) {
              return ae(oe, se, ce, le, ue, ne);
            }, eq: function(oe, se, ce, le, ue) {
              return ae(oe, se, ce, le, ue, ee);
            } };
          }, 54689: function(G, J) {
            function Z(ne) {
              var ee = 32;
              return (ne &= -ne) && ee--, 65535 & ne && (ee -= 16), 16711935 & ne && (ee -= 8), 252645135 & ne && (ee -= 4), 858993459 & ne && (ee -= 2), 1431655765 & ne && (ee -= 1), ee;
            }
            J.INT_BITS = 32, J.INT_MAX = 2147483647, J.INT_MIN = -2147483648, J.sign = function(ne) {
              return (ne > 0) - (ne < 0);
            }, J.abs = function(ne) {
              var ee = ne >> 31;
              return (ne ^ ee) - ee;
            }, J.min = function(ne, ee) {
              return ee ^ (ne ^ ee) & -(ne < ee);
            }, J.max = function(ne, ee) {
              return ne ^ (ne ^ ee) & -(ne < ee);
            }, J.isPow2 = function(ne) {
              return !(ne & ne - 1 || !ne);
            }, J.log2 = function(ne) {
              var ee, ae;
              return ee = (ne > 65535) << 4, ee |= ae = ((ne >>>= ee) > 255) << 3, ee |= ae = ((ne >>>= ae) > 15) << 2, (ee |= ae = ((ne >>>= ae) > 3) << 1) | (ne >>>= ae) >> 1;
            }, J.log10 = function(ne) {
              return ne >= 1e9 ? 9 : ne >= 1e8 ? 8 : ne >= 1e7 ? 7 : ne >= 1e6 ? 6 : ne >= 1e5 ? 5 : ne >= 1e4 ? 4 : ne >= 1e3 ? 3 : ne >= 100 ? 2 : ne >= 10 ? 1 : 0;
            }, J.popCount = function(ne) {
              return 16843009 * ((ne = (858993459 & (ne -= ne >>> 1 & 1431655765)) + (ne >>> 2 & 858993459)) + (ne >>> 4) & 252645135) >>> 24;
            }, J.countTrailingZeros = Z, J.nextPow2 = function(ne) {
              return ne += ne === 0, --ne, ne |= ne >>> 1, ne |= ne >>> 2, ne |= ne >>> 4, ne |= ne >>> 8, 1 + (ne |= ne >>> 16);
            }, J.prevPow2 = function(ne) {
              return ne |= ne >>> 1, ne |= ne >>> 2, ne |= ne >>> 4, ne |= ne >>> 8, (ne |= ne >>> 16) - (ne >>> 1);
            }, J.parity = function(ne) {
              return ne ^= ne >>> 16, ne ^= ne >>> 8, ne ^= ne >>> 4, 27030 >>> (ne &= 15) & 1;
            };
            var te = new Array(256);
            (function(ne) {
              for (var ee = 0; ee < 256; ++ee) {
                var ae = ee, oe = ee, se = 7;
                for (ae >>>= 1; ae; ae >>>= 1) oe <<= 1, oe |= 1 & ae, --se;
                ne[ee] = oe << se & 255;
              }
            })(te), J.reverse = function(ne) {
              return te[255 & ne] << 24 | te[ne >>> 8 & 255] << 16 | te[ne >>> 16 & 255] << 8 | te[ne >>> 24 & 255];
            }, J.interleave2 = function(ne, ee) {
              return (ne = 1431655765 & ((ne = 858993459 & ((ne = 252645135 & ((ne = 16711935 & ((ne &= 65535) | ne << 8)) | ne << 4)) | ne << 2)) | ne << 1)) | (ee = 1431655765 & ((ee = 858993459 & ((ee = 252645135 & ((ee = 16711935 & ((ee &= 65535) | ee << 8)) | ee << 4)) | ee << 2)) | ee << 1)) << 1;
            }, J.deinterleave2 = function(ne, ee) {
              return (ne = 65535 & ((ne = 16711935 & ((ne = 252645135 & ((ne = 858993459 & ((ne = ne >>> ee & 1431655765) | ne >>> 1)) | ne >>> 2)) | ne >>> 4)) | ne >>> 16)) << 16 >> 16;
            }, J.interleave3 = function(ne, ee, ae) {
              return ne = 1227133513 & ((ne = 3272356035 & ((ne = 251719695 & ((ne = 4278190335 & ((ne &= 1023) | ne << 16)) | ne << 8)) | ne << 4)) | ne << 2), (ne |= (ee = 1227133513 & ((ee = 3272356035 & ((ee = 251719695 & ((ee = 4278190335 & ((ee &= 1023) | ee << 16)) | ee << 8)) | ee << 4)) | ee << 2)) << 1) | (ae = 1227133513 & ((ae = 3272356035 & ((ae = 251719695 & ((ae = 4278190335 & ((ae &= 1023) | ae << 16)) | ae << 8)) | ae << 4)) | ae << 2)) << 2;
            }, J.deinterleave3 = function(ne, ee) {
              return (ne = 1023 & ((ne = 4278190335 & ((ne = 251719695 & ((ne = 3272356035 & ((ne = ne >>> ee & 1227133513) | ne >>> 2)) | ne >>> 4)) | ne >>> 8)) | ne >>> 16)) << 22 >> 22;
            }, J.nextCombination = function(ne) {
              var ee = ne | ne - 1;
              return ee + 1 | (~ee & -~ee) - 1 >>> Z(ne) + 1;
            };
          }, 88772: function(G, J, Z) {
            var te = Z(75144);
            G.exports = function(oe, se) {
              se || (se = {});
              var ce, le, ue, de, fe, he, me, pe, ye, ge, ve, be = se.cutoff == null ? 0.25 : se.cutoff, Ce = se.radius == null ? 8 : se.radius, Se = se.channel || 0;
              if (ArrayBuffer.isView(oe) || Array.isArray(oe)) {
                if (!se.width || !se.height) throw Error("For raw data width and height should be provided by options");
                ce = se.width, le = se.height, de = oe, he = se.stride ? se.stride : Math.floor(oe.length / ce / le);
              } else window.HTMLCanvasElement && oe instanceof window.HTMLCanvasElement ? (me = (pe = oe).getContext("2d"), ce = pe.width, le = pe.height, de = (ye = me.getImageData(0, 0, ce, le)).data, he = 4) : window.CanvasRenderingContext2D && oe instanceof window.CanvasRenderingContext2D ? (me = oe, ce = (pe = oe.canvas).width, le = pe.height, de = (ye = me.getImageData(0, 0, ce, le)).data, he = 4) : window.ImageData && oe instanceof window.ImageData && (ye = oe, ce = oe.width, le = oe.height, de = ye.data, he = 4);
              if (ue = Math.max(ce, le), window.Uint8ClampedArray && de instanceof window.Uint8ClampedArray || window.Uint8Array && de instanceof window.Uint8Array) for (fe = de, de = Array(ce * le), ge = 0, ve = fe.length; ge < ve; ge++) de[ge] = fe[ge * he + Se] / 255;
              else if (he !== 1) throw Error("Raw data can have only 1 value per pixel");
              var we = Array(ce * le), Ee = Array(ce * le), xe = Array(ue), Le = Array(ue), Qe = Array(ue + 1), ke = Array(ue);
              for (ge = 0, ve = ce * le; ge < ve; ge++) {
                var Me = de[ge];
                we[ge] = Me === 1 ? 0 : Me === 0 ? ne : Math.pow(Math.max(0, 0.5 - Me), 2), Ee[ge] = Me === 1 ? ne : Me === 0 ? 0 : Math.pow(Math.max(0, Me - 0.5), 2);
              }
              ee(we, ce, le, xe, Le, ke, Qe), ee(Ee, ce, le, xe, Le, ke, Qe);
              var Re = window.Float32Array ? new Float32Array(ce * le) : new Array(ce * le);
              for (ge = 0, ve = ce * le; ge < ve; ge++) Re[ge] = te(1 - ((we[ge] - Ee[ge]) / Ce + be), 0, 1);
              return Re;
            };
            var ne = 1e20;
            function ee(oe, se, ce, le, ue, de, fe) {
              for (var he = 0; he < se; he++) {
                for (var me = 0; me < ce; me++) le[me] = oe[me * se + he];
                for (ae(le, ue, de, fe, ce), me = 0; me < ce; me++) oe[me * se + he] = ue[me];
              }
              for (me = 0; me < ce; me++) {
                for (he = 0; he < se; he++) le[he] = oe[me * se + he];
                for (ae(le, ue, de, fe, se), he = 0; he < se; he++) oe[me * se + he] = Math.sqrt(ue[he]);
              }
            }
            function ae(oe, se, ce, le, ue) {
              ce[0] = 0, le[0] = -1e20, le[1] = 1e20;
              for (var de = 1, fe = 0; de < ue; de++) {
                for (var he = (oe[de] + de * de - (oe[ce[fe]] + ce[fe] * ce[fe])) / (2 * de - 2 * ce[fe]); he <= le[fe]; ) fe--, he = (oe[de] + de * de - (oe[ce[fe]] + ce[fe] * ce[fe])) / (2 * de - 2 * ce[fe]);
                ce[++fe] = de, le[fe] = he, le[fe + 1] = 1e20;
              }
              for (de = 0, fe = 0; de < ue; de++) {
                for (; le[fe + 1] < de; ) fe++;
                se[de] = (de - ce[fe]) * (de - ce[fe]) + oe[ce[fe]];
              }
            }
          }, 63063: function(G, J, Z) {
            var te = Z(71129), ne = Z(87227), ee = ne(te("String.prototype.indexOf"));
            G.exports = function(ae, oe) {
              var se = te(ae, !!oe);
              return typeof se == "function" && ee(ae, ".prototype.") > -1 ? ne(se) : se;
            };
          }, 87227: function(G, J, Z) {
            var te = Z(87547), ne = Z(71129), ee = Z(73285), ae = Z(48631), oe = ne("%Function.prototype.apply%"), se = ne("%Function.prototype.call%"), ce = ne("%Reflect.apply%", !0) || te.call(se, oe), le = Z(40891), ue = ne("%Math.max%");
            G.exports = function(fe) {
              if (typeof fe != "function") throw new ae("a function is required");
              var he = ce(te, se, arguments);
              return ee(he, 1 + ue(0, fe.length - (arguments.length - 1)), !0);
            };
            var de = function() {
              return ce(te, oe, arguments);
            };
            le ? le(G.exports, "apply", { value: de }) : G.exports.apply = de;
          }, 75144: function(G) {
            G.exports = function(J, Z, te) {
              return Z < te ? J < Z ? Z : J > te ? te : J : J < te ? te : J > Z ? Z : J;
            };
          }, 46762: function(G, J, Z) {
            var te = Z(75144);
            function ne(ee, ae) {
              ae == null && (ae = !0);
              var oe = ee[0], se = ee[1], ce = ee[2], le = ee[3];
              return le == null && (le = ae ? 1 : 255), ae && (oe *= 255, se *= 255, ce *= 255, le *= 255), 16777216 * (oe = 255 & te(oe, 0, 255)) + ((se = 255 & te(se, 0, 255)) << 16) + ((ce = 255 & te(ce, 0, 255)) << 8) + (255 & te(le, 0, 255));
            }
            G.exports = ne, G.exports.to = ne, G.exports.from = function(ee, ae) {
              var oe = (ee = +ee) >>> 24, se = (16711680 & ee) >>> 16, ce = (65280 & ee) >>> 8, le = 255 & ee;
              return ae === !1 ? [oe, se, ce, le] : [oe / 255, se / 255, ce / 255, le / 255];
            };
          }, 86040: function(G) {
            G.exports = { aliceblue: [240, 248, 255], antiquewhite: [250, 235, 215], aqua: [0, 255, 255], aquamarine: [127, 255, 212], azure: [240, 255, 255], beige: [245, 245, 220], bisque: [255, 228, 196], black: [0, 0, 0], blanchedalmond: [255, 235, 205], blue: [0, 0, 255], blueviolet: [138, 43, 226], brown: [165, 42, 42], burlywood: [222, 184, 135], cadetblue: [95, 158, 160], chartreuse: [127, 255, 0], chocolate: [210, 105, 30], coral: [255, 127, 80], cornflowerblue: [100, 149, 237], cornsilk: [255, 248, 220], crimson: [220, 20, 60], cyan: [0, 255, 255], darkblue: [0, 0, 139], darkcyan: [0, 139, 139], darkgoldenrod: [184, 134, 11], darkgray: [169, 169, 169], darkgreen: [0, 100, 0], darkgrey: [169, 169, 169], darkkhaki: [189, 183, 107], darkmagenta: [139, 0, 139], darkolivegreen: [85, 107, 47], darkorange: [255, 140, 0], darkorchid: [153, 50, 204], darkred: [139, 0, 0], darksalmon: [233, 150, 122], darkseagreen: [143, 188, 143], darkslateblue: [72, 61, 139], darkslategray: [47, 79, 79], darkslategrey: [47, 79, 79], darkturquoise: [0, 206, 209], darkviolet: [148, 0, 211], deeppink: [255, 20, 147], deepskyblue: [0, 191, 255], dimgray: [105, 105, 105], dimgrey: [105, 105, 105], dodgerblue: [30, 144, 255], firebrick: [178, 34, 34], floralwhite: [255, 250, 240], forestgreen: [34, 139, 34], fuchsia: [255, 0, 255], gainsboro: [220, 220, 220], ghostwhite: [248, 248, 255], gold: [255, 215, 0], goldenrod: [218, 165, 32], gray: [128, 128, 128], green: [0, 128, 0], greenyellow: [173, 255, 47], grey: [128, 128, 128], honeydew: [240, 255, 240], hotpink: [255, 105, 180], indianred: [205, 92, 92], indigo: [75, 0, 130], ivory: [255, 255, 240], khaki: [240, 230, 140], lavender: [230, 230, 250], lavenderblush: [255, 240, 245], lawngreen: [124, 252, 0], lemonchiffon: [255, 250, 205], lightblue: [173, 216, 230], lightcoral: [240, 128, 128], lightcyan: [224, 255, 255], lightgoldenrodyellow: [250, 250, 210], lightgray: [211, 211, 211], lightgreen: [144, 238, 144], lightgrey: [211, 211, 211], lightpink: [255, 182, 193], lightsalmon: [255, 160, 122], lightseagreen: [32, 178, 170], lightskyblue: [135, 206, 250], lightslategray: [119, 136, 153], lightslategrey: [119, 136, 153], lightsteelblue: [176, 196, 222], lightyellow: [255, 255, 224], lime: [0, 255, 0], limegreen: [50, 205, 50], linen: [250, 240, 230], magenta: [255, 0, 255], maroon: [128, 0, 0], mediumaquamarine: [102, 205, 170], mediumblue: [0, 0, 205], mediumorchid: [186, 85, 211], mediumpurple: [147, 112, 219], mediumseagreen: [60, 179, 113], mediumslateblue: [123, 104, 238], mediumspringgreen: [0, 250, 154], mediumturquoise: [72, 209, 204], mediumvioletred: [199, 21, 133], midnightblue: [25, 25, 112], mintcream: [245, 255, 250], mistyrose: [255, 228, 225], moccasin: [255, 228, 181], navajowhite: [255, 222, 173], navy: [0, 0, 128], oldlace: [253, 245, 230], olive: [128, 128, 0], olivedrab: [107, 142, 35], orange: [255, 165, 0], orangered: [255, 69, 0], orchid: [218, 112, 214], palegoldenrod: [238, 232, 170], palegreen: [152, 251, 152], paleturquoise: [175, 238, 238], palevioletred: [219, 112, 147], papayawhip: [255, 239, 213], peachpuff: [255, 218, 185], peru: [205, 133, 63], pink: [255, 192, 203], plum: [221, 160, 221], powderblue: [176, 224, 230], purple: [128, 0, 128], rebeccapurple: [102, 51, 153], red: [255, 0, 0], rosybrown: [188, 143, 143], royalblue: [65, 105, 225], saddlebrown: [139, 69, 19], salmon: [250, 128, 114], sandybrown: [244, 164, 96], seagreen: [46, 139, 87], seashell: [255, 245, 238], sienna: [160, 82, 45], silver: [192, 192, 192], skyblue: [135, 206, 235], slateblue: [106, 90, 205], slategray: [112, 128, 144], slategrey: [112, 128, 144], snow: [255, 250, 250], springgreen: [0, 255, 127], steelblue: [70, 130, 180], tan: [210, 180, 140], teal: [0, 128, 128], thistle: [216, 191, 216], tomato: [255, 99, 71], turquoise: [64, 224, 208], violet: [238, 130, 238], wheat: [245, 222, 179], white: [255, 255, 255], whitesmoke: [245, 245, 245], yellow: [255, 255, 0], yellowgreen: [154, 205, 50] };
          }, 162: function(G, J, Z) {
            var te = Z(16401), ne = Z(75144), ee = Z(10275);
            G.exports = function(ae, oe) {
              oe !== "float" && oe || (oe = "array"), oe === "uint" && (oe = "uint8"), oe === "uint_clamped" && (oe = "uint8_clamped");
              var se = new (ee(oe))(4), ce = oe !== "uint8" && oe !== "uint8_clamped";
              return ae.length && typeof ae != "string" || ((ae = te(ae))[0] /= 255, ae[1] /= 255, ae[2] /= 255), function(le) {
                return le instanceof Uint8Array || le instanceof Uint8ClampedArray || !!(Array.isArray(le) && (le[0] > 1 || le[0] === 0) && (le[1] > 1 || le[1] === 0) && (le[2] > 1 || le[2] === 0) && (!le[3] || le[3] > 1));
              }(ae) ? (se[0] = ae[0], se[1] = ae[1], se[2] = ae[2], se[3] = ae[3] != null ? ae[3] : 255, ce && (se[0] /= 255, se[1] /= 255, se[2] /= 255, se[3] /= 255), se) : (ce ? (se[0] = ae[0], se[1] = ae[1], se[2] = ae[2], se[3] = ae[3] != null ? ae[3] : 1) : (se[0] = ne(Math.floor(255 * ae[0]), 0, 255), se[1] = ne(Math.floor(255 * ae[1]), 0, 255), se[2] = ne(Math.floor(255 * ae[2]), 0, 255), se[3] = ae[3] == null ? 255 : ne(Math.floor(255 * ae[3]), 0, 255)), se);
            };
          }, 16401: function(G, J, Z) {
            var te = Z(10826), ne = Z(52132), ee = Z(75144);
            G.exports = function(ae) {
              var oe, se = te(ae);
              return se.space ? ((oe = Array(3))[0] = ee(se.values[0], 0, 255), oe[1] = ee(se.values[1], 0, 255), oe[2] = ee(se.values[2], 0, 255), se.space[0] === "h" && (oe = ne.rgb(oe)), oe.push(ee(se.alpha, 0, 1)), oe) : [];
            };
          }, 10826: function(G, J, Z) {
            var te = Z(86040);
            G.exports = function(ee) {
              var ae, oe, se = [], ce = 1;
              if (typeof ee == "string") if (ee = ee.toLowerCase(), te[ee]) se = te[ee].slice(), oe = "rgb";
              else if (ee === "transparent") ce = 0, oe = "rgb", se = [0, 0, 0];
              else if (/^#[A-Fa-f0-9]+$/.test(ee)) {
                var le = ee.slice(1);
                ce = 1, (fe = le.length) <= 4 ? (se = [parseInt(le[0] + le[0], 16), parseInt(le[1] + le[1], 16), parseInt(le[2] + le[2], 16)], fe === 4 && (ce = parseInt(le[3] + le[3], 16) / 255)) : (se = [parseInt(le[0] + le[1], 16), parseInt(le[2] + le[3], 16), parseInt(le[4] + le[5], 16)], fe === 8 && (ce = parseInt(le[6] + le[7], 16) / 255)), se[0] || (se[0] = 0), se[1] || (se[1] = 0), se[2] || (se[2] = 0), oe = "rgb";
              } else if (ae = /^((?:rgb|hs[lvb]|hwb|cmyk?|xy[zy]|gray|lab|lchu?v?|[ly]uv|lms)a?)\s*\(([^\)]*)\)/.exec(ee)) {
                var ue = ae[1], de = ue === "rgb";
                oe = le = ue.replace(/a$/, "");
                var fe = le === "cmyk" ? 4 : le === "gray" ? 1 : 3;
                se = ae[2].trim().split(/\s*[,\/]\s*|\s+/).map(function(he, me) {
                  if (/%$/.test(he)) return me === fe ? parseFloat(he) / 100 : le === "rgb" ? 255 * parseFloat(he) / 100 : parseFloat(he);
                  if (le[me] === "h") {
                    if (/deg$/.test(he)) return parseFloat(he);
                    if (ne[he] !== void 0) return ne[he];
                  }
                  return parseFloat(he);
                }), ue === le && se.push(1), ce = de || se[fe] === void 0 ? 1 : se[fe], se = se.slice(0, fe);
              } else ee.length > 10 && /[0-9](?:\s|\/)/.test(ee) && (se = ee.match(/([0-9]+)/g).map(function(he) {
                return parseFloat(he);
              }), oe = ee.match(/([a-z])/gi).join("").toLowerCase());
              else isNaN(ee) ? Array.isArray(ee) || ee.length ? (se = [ee[0], ee[1], ee[2]], oe = "rgb", ce = ee.length === 4 ? ee[3] : 1) : ee instanceof Object && (ee.r != null || ee.red != null || ee.R != null ? (oe = "rgb", se = [ee.r || ee.red || ee.R || 0, ee.g || ee.green || ee.G || 0, ee.b || ee.blue || ee.B || 0]) : (oe = "hsl", se = [ee.h || ee.hue || ee.H || 0, ee.s || ee.saturation || ee.S || 0, ee.l || ee.lightness || ee.L || ee.b || ee.brightness]), ce = ee.a || ee.alpha || ee.opacity || 1, ee.opacity != null && (ce /= 100)) : (oe = "rgb", se = [ee >>> 16, (65280 & ee) >>> 8, 255 & ee]);
              return { space: oe, values: se, alpha: ce };
            };
            var ne = { red: 0, orange: 60, yellow: 120, green: 180, blue: 240, purple: 300 };
          }, 52132: function(G, J, Z) {
            var te = Z(10520);
            G.exports = { name: "hsl", min: [0, 0, 0], max: [360, 100, 100], channel: ["hue", "saturation", "lightness"], alias: ["HSL"], rgb: function(ne) {
              var ee, ae, oe, se, ce, le = ne[0] / 360, ue = ne[1] / 100, de = ne[2] / 100;
              if (ue === 0) return [ce = 255 * de, ce, ce];
              ee = 2 * de - (ae = de < 0.5 ? de * (1 + ue) : de + ue - de * ue), se = [0, 0, 0];
              for (var fe = 0; fe < 3; fe++) (oe = le + 0.3333333333333333 * -(fe - 1)) < 0 ? oe++ : oe > 1 && oe--, ce = 6 * oe < 1 ? ee + 6 * (ae - ee) * oe : 2 * oe < 1 ? ae : 3 * oe < 2 ? ee + (ae - ee) * (0.6666666666666666 - oe) * 6 : ee, se[fe] = 255 * ce;
              return se;
            } }, te.hsl = function(ne) {
              var ee, ae, oe = ne[0] / 255, se = ne[1] / 255, ce = ne[2] / 255, le = Math.min(oe, se, ce), ue = Math.max(oe, se, ce), de = ue - le;
              return ue === le ? ee = 0 : oe === ue ? ee = (se - ce) / de : se === ue ? ee = 2 + (ce - oe) / de : ce === ue && (ee = 4 + (oe - se) / de), (ee = Math.min(60 * ee, 360)) < 0 && (ee += 360), ae = (le + ue) / 2, [ee, 100 * (ue === le ? 0 : ae <= 0.5 ? de / (ue + le) : de / (2 - ue - le)), 100 * ae];
            };
          }, 10520: function(G) {
            G.exports = { name: "rgb", min: [0, 0, 0], max: [255, 255, 255], channel: ["red", "green", "blue"], alias: ["RGB"] };
          }, 78171: function(G) {
            G.exports = { AFG: "afghan", ALA: "\\b\\wland", ALB: "albania", DZA: "algeria", ASM: "^(?=.*americ).*samoa", AND: "andorra", AGO: "angola", AIA: "anguill?a", ATA: "antarctica", ATG: "antigua", ARG: "argentin", ARM: "armenia", ABW: "^(?!.*bonaire).*\\baruba", AUS: "australia", AUT: "^(?!.*hungary).*austria|\\baustri.*\\bemp", AZE: "azerbaijan", BHS: "bahamas", BHR: "bahrain", BGD: "bangladesh|^(?=.*east).*paki?stan", BRB: "barbados", BLR: "belarus|byelo", BEL: "^(?!.*luxem).*belgium", BLZ: "belize|^(?=.*british).*honduras", BEN: "benin|dahome", BMU: "bermuda", BTN: "bhutan", BOL: "bolivia", BES: "^(?=.*bonaire).*eustatius|^(?=.*carib).*netherlands|\\bbes.?islands", BIH: "herzegovina|bosnia", BWA: "botswana|bechuana", BVT: "bouvet", BRA: "brazil", IOT: "british.?indian.?ocean", BRN: "brunei", BGR: "bulgaria", BFA: "burkina|\\bfaso|upper.?volta", BDI: "burundi", CPV: "verde", KHM: "cambodia|kampuchea|khmer", CMR: "cameroon", CAN: "canada", CYM: "cayman", CAF: "\\bcentral.african.republic", TCD: "\\bchad", CHL: "\\bchile", CHN: "^(?!.*\\bmac)(?!.*\\bhong)(?!.*\\btai)(?!.*\\brep).*china|^(?=.*peo)(?=.*rep).*china", CXR: "christmas", CCK: "\\bcocos|keeling", COL: "colombia", COM: "comoro", COG: "^(?!.*\\bdem)(?!.*\\bd[\\.]?r)(?!.*kinshasa)(?!.*zaire)(?!.*belg)(?!.*l.opoldville)(?!.*free).*\\bcongo", COK: "\\bcook", CRI: "costa.?rica", CIV: "ivoire|ivory", HRV: "croatia", CUB: "\\bcuba", CUW: "^(?!.*bonaire).*\\bcura(c|ç)ao", CYP: "cyprus", CSK: "czechoslovakia", CZE: "^(?=.*rep).*czech|czechia|bohemia", COD: "\\bdem.*congo|congo.*\\bdem|congo.*\\bd[\\.]?r|\\bd[\\.]?r.*congo|belgian.?congo|congo.?free.?state|kinshasa|zaire|l.opoldville|drc|droc|rdc", DNK: "denmark", DJI: "djibouti", DMA: "dominica(?!n)", DOM: "dominican.rep", ECU: "ecuador", EGY: "egypt", SLV: "el.?salvador", GNQ: "guine.*eq|eq.*guine|^(?=.*span).*guinea", ERI: "eritrea", EST: "estonia", ETH: "ethiopia|abyssinia", FLK: "falkland|malvinas", FRO: "faroe|faeroe", FJI: "fiji", FIN: "finland", FRA: "^(?!.*\\bdep)(?!.*martinique).*france|french.?republic|\\bgaul", GUF: "^(?=.*french).*guiana", PYF: "french.?polynesia|tahiti", ATF: "french.?southern", GAB: "gabon", GMB: "gambia", GEO: "^(?!.*south).*georgia", DDR: "german.?democratic.?republic|democratic.?republic.*germany|east.germany", DEU: "^(?!.*east).*germany|^(?=.*\\bfed.*\\brep).*german", GHA: "ghana|gold.?coast", GIB: "gibraltar", GRC: "greece|hellenic|hellas", GRL: "greenland", GRD: "grenada", GLP: "guadeloupe", GUM: "\\bguam", GTM: "guatemala", GGY: "guernsey", GIN: "^(?!.*eq)(?!.*span)(?!.*bissau)(?!.*portu)(?!.*new).*guinea", GNB: "bissau|^(?=.*portu).*guinea", GUY: "guyana|british.?guiana", HTI: "haiti", HMD: "heard.*mcdonald", VAT: "holy.?see|vatican|papal.?st", HND: "^(?!.*brit).*honduras", HKG: "hong.?kong", HUN: "^(?!.*austr).*hungary", ISL: "iceland", IND: "india(?!.*ocea)", IDN: "indonesia", IRN: "\\biran|persia", IRQ: "\\biraq|mesopotamia", IRL: "(^ireland)|(^republic.*ireland)", IMN: "^(?=.*isle).*\\bman", ISR: "israel", ITA: "italy", JAM: "jamaica", JPN: "japan", JEY: "jersey", JOR: "jordan", KAZ: "kazak", KEN: "kenya|british.?east.?africa|east.?africa.?prot", KIR: "kiribati", PRK: "^(?=.*democrat|people|north|d.*p.*.r).*\\bkorea|dprk|korea.*(d.*p.*r)", KWT: "kuwait", KGZ: "kyrgyz|kirghiz", LAO: "\\blaos?\\b", LVA: "latvia", LBN: "lebanon", LSO: "lesotho|basuto", LBR: "liberia", LBY: "libya", LIE: "liechtenstein", LTU: "lithuania", LUX: "^(?!.*belg).*luxem", MAC: "maca(o|u)", MDG: "madagascar|malagasy", MWI: "malawi|nyasa", MYS: "malaysia", MDV: "maldive", MLI: "\\bmali\\b", MLT: "\\bmalta", MHL: "marshall", MTQ: "martinique", MRT: "mauritania", MUS: "mauritius", MYT: "\\bmayotte", MEX: "\\bmexic", FSM: "fed.*micronesia|micronesia.*fed", MCO: "monaco", MNG: "mongolia", MNE: "^(?!.*serbia).*montenegro", MSR: "montserrat", MAR: "morocco|\\bmaroc", MOZ: "mozambique", MMR: "myanmar|burma", NAM: "namibia", NRU: "nauru", NPL: "nepal", NLD: "^(?!.*\\bant)(?!.*\\bcarib).*netherlands", ANT: "^(?=.*\\bant).*(nether|dutch)", NCL: "new.?caledonia", NZL: "new.?zealand", NIC: "nicaragua", NER: "\\bniger(?!ia)", NGA: "nigeria", NIU: "niue", NFK: "norfolk", MNP: "mariana", NOR: "norway", OMN: "\\boman|trucial", PAK: "^(?!.*east).*paki?stan", PLW: "palau", PSE: "palestin|\\bgaza|west.?bank", PAN: "panama", PNG: "papua|new.?guinea", PRY: "paraguay", PER: "peru", PHL: "philippines", PCN: "pitcairn", POL: "poland", PRT: "portugal", PRI: "puerto.?rico", QAT: "qatar", KOR: "^(?!.*d.*p.*r)(?!.*democrat)(?!.*people)(?!.*north).*\\bkorea(?!.*d.*p.*r)", MDA: "moldov|b(a|e)ssarabia", REU: "r(e|é)union", ROU: "r(o|u|ou)mania", RUS: "\\brussia|soviet.?union|u\\.?s\\.?s\\.?r|socialist.?republics", RWA: "rwanda", BLM: "barth(e|é)lemy", SHN: "helena", KNA: "kitts|\\bnevis", LCA: "\\blucia", MAF: "^(?=.*collectivity).*martin|^(?=.*france).*martin(?!ique)|^(?=.*french).*martin(?!ique)", SPM: "miquelon", VCT: "vincent", WSM: "^(?!.*amer).*samoa", SMR: "san.?marino", STP: "\\bs(a|ã)o.?tom(e|é)", SAU: "\\bsa\\w*.?arabia", SEN: "senegal", SRB: "^(?!.*monte).*serbia", SYC: "seychell", SLE: "sierra", SGP: "singapore", SXM: "^(?!.*martin)(?!.*saba).*maarten", SVK: "^(?!.*cze).*slovak", SVN: "slovenia", SLB: "solomon", SOM: "somali", ZAF: "south.africa|s\\\\..?africa", SGS: "south.?georgia|sandwich", SSD: "\\bs\\w*.?sudan", ESP: "spain", LKA: "sri.?lanka|ceylon", SDN: "^(?!.*\\bs(?!u)).*sudan", SUR: "surinam|dutch.?guiana", SJM: "svalbard", SWZ: "swaziland", SWE: "sweden", CHE: "switz|swiss", SYR: "syria", TWN: "taiwan|taipei|formosa|^(?!.*peo)(?=.*rep).*china", TJK: "tajik", THA: "thailand|\\bsiam", MKD: "macedonia|fyrom", TLS: "^(?=.*leste).*timor|^(?=.*east).*timor", TGO: "togo", TKL: "tokelau", TON: "tonga", TTO: "trinidad|tobago", TUN: "tunisia", TUR: "turkey", TKM: "turkmen", TCA: "turks", TUV: "tuvalu", UGA: "uganda", UKR: "ukrain", ARE: "emirates|^u\\.?a\\.?e\\.?$|united.?arab.?em", GBR: "united.?kingdom|britain|^u\\.?k\\.?$", TZA: "tanzania", USA: "united.?states\\b(?!.*islands)|\\bu\\.?s\\.?a\\.?\\b|^\\s*u\\.?s\\.?\\b(?!.*islands)", UMI: "minor.?outlying.?is", URY: "uruguay", UZB: "uzbek", VUT: "vanuatu|new.?hebrides", VEN: "venezuela", VNM: "^(?!.*republic).*viet.?nam|^(?=.*socialist).*viet.?nam", VGB: "^(?=.*\\bu\\.?\\s?k).*virgin|^(?=.*brit).*virgin|^(?=.*kingdom).*virgin", VIR: "^(?=.*\\bu\\.?\\s?s).*virgin|^(?=.*states).*virgin", WLF: "futuna|wallis", ESH: "western.sahara", YEM: "^(?!.*arab)(?!.*north)(?!.*sana)(?!.*peo)(?!.*dem)(?!.*south)(?!.*aden)(?!.*\\bp\\.?d\\.?r).*yemen", YMD: "^(?=.*peo).*yemen|^(?!.*rep)(?=.*dem).*yemen|^(?=.*south).*yemen|^(?=.*aden).*yemen|^(?=.*\\bp\\.?d\\.?r).*yemen", YUG: "yugoslavia", ZMB: "zambia|northern.?rhodesia", EAZ: "zanzibar", ZWE: "zimbabwe|^(?!.*northern).*rhodesia" };
          }, 59518: function(G, J, Z) {
            G.exports = { parse: Z(86029), stringify: Z(38211) };
          }, 87724: function(G, J, Z) {
            var te = Z(23648);
            G.exports = { isSize: function(ne) {
              return /^[\d\.]/.test(ne) || ne.indexOf("/") !== -1 || te.indexOf(ne) !== -1;
            } };
          }, 86029: function(G, J, Z) {
            var te = Z(80886), ne = Z(54324), ee = Z(94316), ae = Z(99803), oe = Z(87486), se = Z(2362), ce = Z(28089), le = Z(87724).isSize;
            G.exports = de;
            var ue = de.cache = {};
            function de(he) {
              if (typeof he != "string") throw new Error("Font argument must be a string.");
              if (ue[he]) return ue[he];
              if (he === "") throw new Error("Cannot parse an empty string.");
              if (ee.indexOf(he) !== -1) return ue[he] = { system: he };
              for (var me, pe = { style: "normal", variant: "normal", weight: "normal", stretch: "normal", lineHeight: "normal", size: "1rem", family: ["serif"] }, ye = ce(he, /\s+/); me = ye.shift(); ) {
                if (ne.indexOf(me) !== -1) return ["style", "variant", "weight", "stretch"].forEach(function(ve) {
                  pe[ve] = me;
                }), ue[he] = pe;
                if (oe.indexOf(me) === -1) if (me !== "normal" && me !== "small-caps") if (se.indexOf(me) === -1) {
                  if (ae.indexOf(me) === -1) {
                    if (le(me)) {
                      var ge = ce(me, "/");
                      if (pe.size = ge[0], ge[1] != null ? pe.lineHeight = fe(ge[1]) : ye[0] === "/" && (ye.shift(), pe.lineHeight = fe(ye.shift())), !ye.length) throw new Error("Missing required font-family.");
                      return pe.family = ce(ye.join(" "), /\s*,\s*/).map(te), ue[he] = pe;
                    }
                    throw new Error("Unknown or unsupported font token: " + me);
                  }
                  pe.weight = me;
                } else pe.stretch = me;
                else pe.variant = me;
                else pe.style = me;
              }
              throw new Error("Missing required font-size.");
            }
            function fe(he) {
              var me = parseFloat(he);
              return me.toString() === he ? me : he;
            }
          }, 38211: function(G, J, Z) {
            var te = Z(6807), ne = Z(87724).isSize, ee = he(Z(54324)), ae = he(Z(94316)), oe = he(Z(99803)), se = he(Z(87486)), ce = he(Z(2362)), le = { normal: 1, "small-caps": 1 }, ue = { serif: 1, "sans-serif": 1, monospace: 1, cursive: 1, fantasy: 1, "system-ui": 1 }, de = "serif";
            function fe(me, pe) {
              if (me && !pe[me] && !ee[me]) throw Error("Unknown keyword `" + me + "`");
              return me;
            }
            function he(me) {
              for (var pe = {}, ye = 0; ye < me.length; ye++) pe[me[ye]] = 1;
              return pe;
            }
            G.exports = function(me) {
              if ((me = te(me, { style: "style fontstyle fontStyle font-style slope distinction", variant: "variant font-variant fontVariant fontvariant var capitalization", weight: "weight w font-weight fontWeight fontweight", stretch: "stretch font-stretch fontStretch fontstretch width", size: "size s font-size fontSize fontsize height em emSize", lineHeight: "lh line-height lineHeight lineheight leading", family: "font family fontFamily font-family fontfamily type typeface face", system: "system reserved default global" })).system) return me.system && fe(me.system, ae), me.system;
              if (fe(me.style, se), fe(me.variant, le), fe(me.weight, oe), fe(me.stretch, ce), me.size == null && (me.size = "1rem"), typeof me.size == "number" && (me.size += "px"), !ne) throw Error("Bad size value `" + me.size + "`");
              me.family || (me.family = de), Array.isArray(me.family) && (me.family.length || (me.family = [de]), me.family = me.family.map(function(ye) {
                return ue[ye] ? ye : '"' + ye + '"';
              }).join(", "));
              var pe = [];
              return pe.push(me.style), me.variant !== me.style && pe.push(me.variant), me.weight !== me.variant && me.weight !== me.style && pe.push(me.weight), me.stretch !== me.weight && me.stretch !== me.variant && me.stretch !== me.style && pe.push(me.stretch), pe.push(me.size + (me.lineHeight == null || me.lineHeight === "normal" || me.lineHeight + "" == "1" ? "" : "/" + me.lineHeight)), pe.push(me.family), pe.filter(Boolean).join(" ");
            };
          }, 51070: function(G) {
            G.exports = function(J) {
              var Z = [];
              return Z.toString = function() {
                return this.map(function(te) {
                  var ne = "", ee = te[5] !== void 0;
                  return te[4] && (ne += "@supports (".concat(te[4], ") {")), te[2] && (ne += "@media ".concat(te[2], " {")), ee && (ne += "@layer".concat(te[5].length > 0 ? " ".concat(te[5]) : "", " {")), ne += J(te), ee && (ne += "}"), te[2] && (ne += "}"), te[4] && (ne += "}"), ne;
                }).join("");
              }, Z.i = function(te, ne, ee, ae, oe) {
                typeof te == "string" && (te = [[null, te, void 0]]);
                var se = {};
                if (ee) for (var ce = 0; ce < this.length; ce++) {
                  var le = this[ce][0];
                  le != null && (se[le] = !0);
                }
                for (var ue = 0; ue < te.length; ue++) {
                  var de = [].concat(te[ue]);
                  ee && se[de[0]] || (oe !== void 0 && (de[5] === void 0 || (de[1] = "@layer".concat(de[5].length > 0 ? " ".concat(de[5]) : "", " {").concat(de[1], "}")), de[5] = oe), ne && (de[2] && (de[1] = "@media ".concat(de[2], " {").concat(de[1], "}")), de[2] = ne), ae && (de[4] ? (de[1] = "@supports (".concat(de[4], ") {").concat(de[1], "}"), de[4] = ae) : de[4] = "".concat(ae)), Z.push(de));
                }
              }, Z;
            };
          }, 62133: function(G) {
            G.exports = function(J, Z) {
              return Z || (Z = {}), J && (J = String(J.__esModule ? J.default : J), /^['"].*['"]$/.test(J) && (J = J.slice(1, -1)), Z.hash && (J += Z.hash), /["'() \t\n]|(%20)/.test(J) || Z.needQuotes ? '"'.concat(J.replace(/"/g, '\\"').replace(/\n/g, "\\n"), '"') : J);
            };
          }, 22413: function(G) {
            G.exports = function(J) {
              return J[1];
            };
          }, 84510: function(G, J, Z) {
            var te, ne = Z(80299), ee = Z(9557), ae = Z(6887), oe = Z(86591), se = Z(76504), ce = Z(29854), le = Function.prototype.bind, ue = Object.defineProperty, de = Object.prototype.hasOwnProperty;
            te = function(fe, he, me) {
              var pe, ye = ee(he) && ae(he.value);
              return delete (pe = oe(he)).writable, delete pe.value, pe.get = function() {
                return !me.overwriteDefinition && de.call(this, fe) ? ye : (he.value = le.call(ye, me.resolveContext ? me.resolveContext(this) : this), ue(this, fe, he), this[fe]);
              }, pe;
            }, G.exports = function(fe) {
              var he = se(arguments[1]);
              return ne(he.resolveContext) && ae(he.resolveContext), ce(fe, function(me, pe) {
                return te(pe, me, he);
              });
            };
          }, 91819: function(G, J, Z) {
            var te = Z(80299), ne = Z(63461), ee = Z(1920), ae = Z(76504), oe = Z(2338), se = G.exports = function(ce, le) {
              var ue, de, fe, he, me;
              return arguments.length < 2 || typeof ce != "string" ? (he = le, le = ce, ce = null) : he = arguments[2], te(ce) ? (ue = oe.call(ce, "c"), de = oe.call(ce, "e"), fe = oe.call(ce, "w")) : (ue = fe = !0, de = !1), me = { value: le, configurable: ue, enumerable: de, writable: fe }, he ? ee(ae(he), me) : me;
            };
            se.gs = function(ce, le, ue) {
              var de, fe, he, me;
              return typeof ce != "string" ? (he = ue, ue = le, le = ce, ce = null) : he = arguments[3], te(le) ? ne(le) ? te(ue) ? ne(ue) || (he = ue, ue = void 0) : ue = void 0 : (he = le, le = ue = void 0) : le = void 0, te(ce) ? (de = oe.call(ce, "c"), fe = oe.call(ce, "e")) : (de = !0, fe = !1), me = { get: le, set: ue, configurable: de, enumerable: fe }, he ? ee(ae(he), me) : me;
            };
          }, 29725: function(G, J, Z) {
            function te(fe, he) {
              return fe < he ? -1 : fe > he ? 1 : fe >= he ? 0 : NaN;
            }
            Z.d(J, { V_: function() {
              return te;
            }, T9: function() {
              return ae;
            }, i2: function() {
              return se;
            }, Am: function() {
              return ce;
            }, jk: function() {
              return le;
            }, y1: function() {
              return ue;
            }, cz: function() {
              return de;
            } }), (ne = te).length === 1 && (ee = ne, ne = function(fe, he) {
              return te(ee(fe), he);
            });
            var ne, ee;
            function ae(fe, he) {
              var me, pe, ye = fe.length, ge = -1;
              if (he == null) {
                for (; ++ge < ye; ) if ((me = fe[ge]) != null && me >= me) for (pe = me; ++ge < ye; ) (me = fe[ge]) != null && me > pe && (pe = me);
              } else for (; ++ge < ye; ) if ((me = he(fe[ge], ge, fe)) != null && me >= me) for (pe = me; ++ge < ye; ) (me = he(fe[ge], ge, fe)) != null && me > pe && (pe = me);
              return pe;
            }
            function oe(fe) {
              return fe === null ? NaN : +fe;
            }
            function se(fe, he) {
              var me, pe = fe.length, ye = pe, ge = -1, ve = 0;
              if (he == null) for (; ++ge < pe; ) isNaN(me = oe(fe[ge])) ? --ye : ve += me;
              else for (; ++ge < pe; ) isNaN(me = oe(he(fe[ge], ge, fe))) ? --ye : ve += me;
              if (ye) return ve / ye;
            }
            function ce(fe) {
              for (var he, me, pe, ye = fe.length, ge = -1, ve = 0; ++ge < ye; ) ve += fe[ge].length;
              for (me = new Array(ve); --ye >= 0; ) for (he = (pe = fe[ye]).length; --he >= 0; ) me[--ve] = pe[he];
              return me;
            }
            function le(fe, he) {
              var me, pe, ye = fe.length, ge = -1;
              if (he == null) {
                for (; ++ge < ye; ) if ((me = fe[ge]) != null && me >= me) for (pe = me; ++ge < ye; ) (me = fe[ge]) != null && pe > me && (pe = me);
              } else for (; ++ge < ye; ) if ((me = he(fe[ge], ge, fe)) != null && me >= me) for (pe = me; ++ge < ye; ) (me = he(fe[ge], ge, fe)) != null && pe > me && (pe = me);
              return pe;
            }
            function ue(fe, he, me) {
              fe = +fe, he = +he, me = (ye = arguments.length) < 2 ? (he = fe, fe = 0, 1) : ye < 3 ? 1 : +me;
              for (var pe = -1, ye = 0 | Math.max(0, Math.ceil((he - fe) / me)), ge = new Array(ye); ++pe < ye; ) ge[pe] = fe + pe * me;
              return ge;
            }
            function de(fe, he) {
              var me, pe = fe.length, ye = -1, ge = 0;
              if (he == null) for (; ++ye < pe; ) (me = +fe[ye]) && (ge += me);
              else for (; ++ye < pe; ) (me = +he(fe[ye], ye, fe)) && (ge += me);
              return ge;
            }
          }, 4575: function(G, J, Z) {
            Z.d(J, { Tj: function() {
              return ae;
            }, $I: function() {
              return oe;
            } });
            var te = "$";
            function ne() {
            }
            function ee(he, me) {
              var pe = new ne();
              if (he instanceof ne) he.each(function(Ce, Se) {
                pe.set(Se, Ce);
              });
              else if (Array.isArray(he)) {
                var ye, ge = -1, ve = he.length;
                if (me == null) for (; ++ge < ve; ) pe.set(ge, he[ge]);
                else for (; ++ge < ve; ) pe.set(me(ye = he[ge], ge, he), ye);
              } else if (he) for (var be in he) pe.set(be, he[be]);
              return pe;
            }
            ne.prototype = ee.prototype = { constructor: ne, has: function(he) {
              return te + he in this;
            }, get: function(he) {
              return this[te + he];
            }, set: function(he, me) {
              return this[te + he] = me, this;
            }, remove: function(he) {
              var me = te + he;
              return me in this && delete this[me];
            }, clear: function() {
              for (var he in this) he[0] === te && delete this[he];
            }, keys: function() {
              var he = [];
              for (var me in this) me[0] === te && he.push(me.slice(1));
              return he;
            }, values: function() {
              var he = [];
              for (var me in this) me[0] === te && he.push(this[me]);
              return he;
            }, entries: function() {
              var he = [];
              for (var me in this) me[0] === te && he.push({ key: me.slice(1), value: this[me] });
              return he;
            }, size: function() {
              var he = 0;
              for (var me in this) me[0] === te && ++he;
              return he;
            }, empty: function() {
              for (var he in this) if (he[0] === te) return !1;
              return !0;
            }, each: function(he) {
              for (var me in this) me[0] === te && he(this[me], me.slice(1), this);
            } };
            var ae = ee;
            function oe() {
              var he, me, pe, ye = [], ge = [];
              function ve(Ce, Se, we, Ee) {
                if (Se >= ye.length) return he != null && Ce.sort(he), me != null ? me(Ce) : Ce;
                for (var xe, Le, Qe, ke = -1, Me = Ce.length, Re = ye[Se++], De = ae(), Pe = we(); ++ke < Me; ) (Qe = De.get(xe = Re(Le = Ce[ke]) + "")) ? Qe.push(Le) : De.set(xe, [Le]);
                return De.each(function($e, qe) {
                  Ee(Pe, qe, ve($e, Se, we, Ee));
                }), Pe;
              }
              function be(Ce, Se) {
                if (++Se > ye.length) return Ce;
                var we, Ee = ge[Se - 1];
                return me != null && Se >= ye.length ? we = Ce.entries() : (we = [], Ce.each(function(xe, Le) {
                  we.push({ key: Le, values: be(xe, Se) });
                })), Ee != null ? we.sort(function(xe, Le) {
                  return Ee(xe.key, Le.key);
                }) : we;
              }
              return pe = { object: function(Ce) {
                return ve(Ce, 0, se, ce);
              }, map: function(Ce) {
                return ve(Ce, 0, le, ue);
              }, entries: function(Ce) {
                return be(ve(Ce, 0, le, ue), 0);
              }, key: function(Ce) {
                return ye.push(Ce), pe;
              }, sortKeys: function(Ce) {
                return ge[ye.length - 1] = Ce, pe;
              }, sortValues: function(Ce) {
                return he = Ce, pe;
              }, rollup: function(Ce) {
                return me = Ce, pe;
              } };
            }
            function se() {
              return {};
            }
            function ce(he, me, pe) {
              he[me] = pe;
            }
            function le() {
              return ae();
            }
            function ue(he, me, pe) {
              he.set(me, pe);
            }
            function de() {
            }
            var fe = ae.prototype;
            de.prototype = { constructor: de, has: fe.has, add: function(he) {
              return this[te + (he += "")] = he, this;
            }, remove: fe.remove, clear: fe.clear, values: fe.keys, size: fe.size, empty: fe.empty, each: fe.each };
          }, 32702: function(G, J, Z) {
            function te(vt, $t) {
              var Pt;
              function Xt() {
                var Nt, Gt, Ft = Pt.length, Tt = 0, It = 0;
                for (Nt = 0; Nt < Ft; ++Nt) Tt += (Gt = Pt[Nt]).x, It += Gt.y;
                for (Tt = Tt / Ft - vt, It = It / Ft - $t, Nt = 0; Nt < Ft; ++Nt) (Gt = Pt[Nt]).x -= Tt, Gt.y -= It;
              }
              return vt == null && (vt = 0), $t == null && ($t = 0), Xt.initialize = function(Nt) {
                Pt = Nt;
              }, Xt.x = function(Nt) {
                return arguments.length ? (vt = +Nt, Xt) : vt;
              }, Xt.y = function(Nt) {
                return arguments.length ? ($t = +Nt, Xt) : $t;
              }, Xt;
            }
            function ne(vt) {
              return function() {
                return vt;
              };
            }
            function ee() {
              return 1e-6 * (Math.random() - 0.5);
            }
            function ae(vt, $t, Pt, Xt) {
              if (isNaN($t) || isNaN(Pt)) return vt;
              var Nt, Gt, Ft, Tt, It, Ct, Kt, Vt, Yt, sn = vt._root, hn = { data: Xt }, Qt = vt._x0, Ut = vt._y0, Wt = vt._x1, rn = vt._y1;
              if (!sn) return vt._root = hn, vt;
              for (; sn.length; ) if ((Ct = $t >= (Gt = (Qt + Wt) / 2)) ? Qt = Gt : Wt = Gt, (Kt = Pt >= (Ft = (Ut + rn) / 2)) ? Ut = Ft : rn = Ft, Nt = sn, !(sn = sn[Vt = Kt << 1 | Ct])) return Nt[Vt] = hn, vt;
              if (Tt = +vt._x.call(null, sn.data), It = +vt._y.call(null, sn.data), $t === Tt && Pt === It) return hn.next = sn, Nt ? Nt[Vt] = hn : vt._root = hn, vt;
              do
                Nt = Nt ? Nt[Vt] = new Array(4) : vt._root = new Array(4), (Ct = $t >= (Gt = (Qt + Wt) / 2)) ? Qt = Gt : Wt = Gt, (Kt = Pt >= (Ft = (Ut + rn) / 2)) ? Ut = Ft : rn = Ft;
              while ((Vt = Kt << 1 | Ct) == (Yt = (It >= Ft) << 1 | Tt >= Gt));
              return Nt[Yt] = sn, Nt[Vt] = hn, vt;
            }
            function oe(vt, $t, Pt, Xt, Nt) {
              this.node = vt, this.x0 = $t, this.y0 = Pt, this.x1 = Xt, this.y1 = Nt;
            }
            function se(vt) {
              return vt[0];
            }
            function ce(vt) {
              return vt[1];
            }
            function le(vt, $t, Pt) {
              var Xt = new ue($t ?? se, Pt ?? ce, NaN, NaN, NaN, NaN);
              return vt == null ? Xt : Xt.addAll(vt);
            }
            function ue(vt, $t, Pt, Xt, Nt, Gt) {
              this._x = vt, this._y = $t, this._x0 = Pt, this._y0 = Xt, this._x1 = Nt, this._y1 = Gt, this._root = void 0;
            }
            function de(vt) {
              for (var $t = { data: vt.data }, Pt = $t; vt = vt.next; ) Pt = Pt.next = { data: vt.data };
              return $t;
            }
            Z.r(J), Z.d(J, { forceCenter: function() {
              return te;
            }, forceCollide: function() {
              return pe;
            }, forceLink: function() {
              return be;
            }, forceManyBody: function() {
              return yt;
            }, forceRadial: function() {
              return St;
            }, forceSimulation: function() {
              return bt;
            }, forceX: function() {
              return Lt;
            }, forceY: function() {
              return Rt;
            } });
            var fe = le.prototype = ue.prototype;
            function he(vt) {
              return vt.x + vt.vx;
            }
            function me(vt) {
              return vt.y + vt.vy;
            }
            function pe(vt) {
              var $t, Pt, Xt = 1, Nt = 1;
              function Gt() {
                for (var It, Ct, Kt, Vt, Yt, sn, hn, Qt = $t.length, Ut = 0; Ut < Nt; ++Ut) for (Ct = le($t, he, me).visitAfter(Ft), It = 0; It < Qt; ++It) Kt = $t[It], sn = Pt[Kt.index], hn = sn * sn, Vt = Kt.x + Kt.vx, Yt = Kt.y + Kt.vy, Ct.visit(Wt);
                function Wt(rn, vn, dn, Ht, jt) {
                  var cn = rn.data, wn = rn.r, Rn = sn + wn;
                  if (!cn) return vn > Vt + Rn || Ht < Vt - Rn || dn > Yt + Rn || jt < Yt - Rn;
                  if (cn.index > Kt.index) {
                    var Pn = Vt - cn.x - cn.vx, Ie = Yt - cn.y - cn.vy, rt = Pn * Pn + Ie * Ie;
                    rt < Rn * Rn && (Pn === 0 && (rt += (Pn = ee()) * Pn), Ie === 0 && (rt += (Ie = ee()) * Ie), rt = (Rn - (rt = Math.sqrt(rt))) / rt * Xt, Kt.vx += (Pn *= rt) * (Rn = (wn *= wn) / (hn + wn)), Kt.vy += (Ie *= rt) * Rn, cn.vx -= Pn * (Rn = 1 - Rn), cn.vy -= Ie * Rn);
                  }
                }
              }
              function Ft(It) {
                if (It.data) return It.r = Pt[It.data.index];
                for (var Ct = It.r = 0; Ct < 4; ++Ct) It[Ct] && It[Ct].r > It.r && (It.r = It[Ct].r);
              }
              function Tt() {
                if ($t) {
                  var It, Ct, Kt = $t.length;
                  for (Pt = new Array(Kt), It = 0; It < Kt; ++It) Ct = $t[It], Pt[Ct.index] = +vt(Ct, It, $t);
                }
              }
              return typeof vt != "function" && (vt = ne(vt == null ? 1 : +vt)), Gt.initialize = function(It) {
                $t = It, Tt();
              }, Gt.iterations = function(It) {
                return arguments.length ? (Nt = +It, Gt) : Nt;
              }, Gt.strength = function(It) {
                return arguments.length ? (Xt = +It, Gt) : Xt;
              }, Gt.radius = function(It) {
                return arguments.length ? (vt = typeof It == "function" ? It : ne(+It), Tt(), Gt) : vt;
              }, Gt;
            }
            fe.copy = function() {
              var vt, $t, Pt = new ue(this._x, this._y, this._x0, this._y0, this._x1, this._y1), Xt = this._root;
              if (!Xt) return Pt;
              if (!Xt.length) return Pt._root = de(Xt), Pt;
              for (vt = [{ source: Xt, target: Pt._root = new Array(4) }]; Xt = vt.pop(); ) for (var Nt = 0; Nt < 4; ++Nt) ($t = Xt.source[Nt]) && ($t.length ? vt.push({ source: $t, target: Xt.target[Nt] = new Array(4) }) : Xt.target[Nt] = de($t));
              return Pt;
            }, fe.add = function(vt) {
              var $t = +this._x.call(null, vt), Pt = +this._y.call(null, vt);
              return ae(this.cover($t, Pt), $t, Pt, vt);
            }, fe.addAll = function(vt) {
              var $t, Pt, Xt, Nt, Gt = vt.length, Ft = new Array(Gt), Tt = new Array(Gt), It = 1 / 0, Ct = 1 / 0, Kt = -1 / 0, Vt = -1 / 0;
              for (Pt = 0; Pt < Gt; ++Pt) isNaN(Xt = +this._x.call(null, $t = vt[Pt])) || isNaN(Nt = +this._y.call(null, $t)) || (Ft[Pt] = Xt, Tt[Pt] = Nt, Xt < It && (It = Xt), Xt > Kt && (Kt = Xt), Nt < Ct && (Ct = Nt), Nt > Vt && (Vt = Nt));
              if (It > Kt || Ct > Vt) return this;
              for (this.cover(It, Ct).cover(Kt, Vt), Pt = 0; Pt < Gt; ++Pt) ae(this, Ft[Pt], Tt[Pt], vt[Pt]);
              return this;
            }, fe.cover = function(vt, $t) {
              if (isNaN(vt = +vt) || isNaN($t = +$t)) return this;
              var Pt = this._x0, Xt = this._y0, Nt = this._x1, Gt = this._y1;
              if (isNaN(Pt)) Nt = (Pt = Math.floor(vt)) + 1, Gt = (Xt = Math.floor($t)) + 1;
              else {
                for (var Ft, Tt, It = Nt - Pt, Ct = this._root; Pt > vt || vt >= Nt || Xt > $t || $t >= Gt; ) switch (Tt = ($t < Xt) << 1 | vt < Pt, (Ft = new Array(4))[Tt] = Ct, Ct = Ft, It *= 2, Tt) {
                  case 0:
                    Nt = Pt + It, Gt = Xt + It;
                    break;
                  case 1:
                    Pt = Nt - It, Gt = Xt + It;
                    break;
                  case 2:
                    Nt = Pt + It, Xt = Gt - It;
                    break;
                  case 3:
                    Pt = Nt - It, Xt = Gt - It;
                }
                this._root && this._root.length && (this._root = Ct);
              }
              return this._x0 = Pt, this._y0 = Xt, this._x1 = Nt, this._y1 = Gt, this;
            }, fe.data = function() {
              var vt = [];
              return this.visit(function($t) {
                if (!$t.length) do
                  vt.push($t.data);
                while ($t = $t.next);
              }), vt;
            }, fe.extent = function(vt) {
              return arguments.length ? this.cover(+vt[0][0], +vt[0][1]).cover(+vt[1][0], +vt[1][1]) : isNaN(this._x0) ? void 0 : [[this._x0, this._y0], [this._x1, this._y1]];
            }, fe.find = function(vt, $t, Pt) {
              var Xt, Nt, Gt, Ft, Tt, It, Ct, Kt = this._x0, Vt = this._y0, Yt = this._x1, sn = this._y1, hn = [], Qt = this._root;
              for (Qt && hn.push(new oe(Qt, Kt, Vt, Yt, sn)), Pt == null ? Pt = 1 / 0 : (Kt = vt - Pt, Vt = $t - Pt, Yt = vt + Pt, sn = $t + Pt, Pt *= Pt); It = hn.pop(); ) if (!(!(Qt = It.node) || (Nt = It.x0) > Yt || (Gt = It.y0) > sn || (Ft = It.x1) < Kt || (Tt = It.y1) < Vt)) if (Qt.length) {
                var Ut = (Nt + Ft) / 2, Wt = (Gt + Tt) / 2;
                hn.push(new oe(Qt[3], Ut, Wt, Ft, Tt), new oe(Qt[2], Nt, Wt, Ut, Tt), new oe(Qt[1], Ut, Gt, Ft, Wt), new oe(Qt[0], Nt, Gt, Ut, Wt)), (Ct = ($t >= Wt) << 1 | vt >= Ut) && (It = hn[hn.length - 1], hn[hn.length - 1] = hn[hn.length - 1 - Ct], hn[hn.length - 1 - Ct] = It);
              } else {
                var rn = vt - +this._x.call(null, Qt.data), vn = $t - +this._y.call(null, Qt.data), dn = rn * rn + vn * vn;
                if (dn < Pt) {
                  var Ht = Math.sqrt(Pt = dn);
                  Kt = vt - Ht, Vt = $t - Ht, Yt = vt + Ht, sn = $t + Ht, Xt = Qt.data;
                }
              }
              return Xt;
            }, fe.remove = function(vt) {
              if (isNaN(Gt = +this._x.call(null, vt)) || isNaN(Ft = +this._y.call(null, vt))) return this;
              var $t, Pt, Xt, Nt, Gt, Ft, Tt, It, Ct, Kt, Vt, Yt, sn = this._root, hn = this._x0, Qt = this._y0, Ut = this._x1, Wt = this._y1;
              if (!sn) return this;
              if (sn.length) for (; ; ) {
                if ((Ct = Gt >= (Tt = (hn + Ut) / 2)) ? hn = Tt : Ut = Tt, (Kt = Ft >= (It = (Qt + Wt) / 2)) ? Qt = It : Wt = It, $t = sn, !(sn = sn[Vt = Kt << 1 | Ct])) return this;
                if (!sn.length) break;
                ($t[Vt + 1 & 3] || $t[Vt + 2 & 3] || $t[Vt + 3 & 3]) && (Pt = $t, Yt = Vt);
              }
              for (; sn.data !== vt; ) if (Xt = sn, !(sn = sn.next)) return this;
              return (Nt = sn.next) && delete sn.next, Xt ? (Nt ? Xt.next = Nt : delete Xt.next, this) : $t ? (Nt ? $t[Vt] = Nt : delete $t[Vt], (sn = $t[0] || $t[1] || $t[2] || $t[3]) && sn === ($t[3] || $t[2] || $t[1] || $t[0]) && !sn.length && (Pt ? Pt[Yt] = sn : this._root = sn), this) : (this._root = Nt, this);
            }, fe.removeAll = function(vt) {
              for (var $t = 0, Pt = vt.length; $t < Pt; ++$t) this.remove(vt[$t]);
              return this;
            }, fe.root = function() {
              return this._root;
            }, fe.size = function() {
              var vt = 0;
              return this.visit(function($t) {
                if (!$t.length) do
                  ++vt;
                while ($t = $t.next);
              }), vt;
            }, fe.visit = function(vt) {
              var $t, Pt, Xt, Nt, Gt, Ft, Tt = [], It = this._root;
              for (It && Tt.push(new oe(It, this._x0, this._y0, this._x1, this._y1)); $t = Tt.pop(); ) if (!vt(It = $t.node, Xt = $t.x0, Nt = $t.y0, Gt = $t.x1, Ft = $t.y1) && It.length) {
                var Ct = (Xt + Gt) / 2, Kt = (Nt + Ft) / 2;
                (Pt = It[3]) && Tt.push(new oe(Pt, Ct, Kt, Gt, Ft)), (Pt = It[2]) && Tt.push(new oe(Pt, Xt, Kt, Ct, Ft)), (Pt = It[1]) && Tt.push(new oe(Pt, Ct, Nt, Gt, Kt)), (Pt = It[0]) && Tt.push(new oe(Pt, Xt, Nt, Ct, Kt));
              }
              return this;
            }, fe.visitAfter = function(vt) {
              var $t, Pt = [], Xt = [];
              for (this._root && Pt.push(new oe(this._root, this._x0, this._y0, this._x1, this._y1)); $t = Pt.pop(); ) {
                var Nt = $t.node;
                if (Nt.length) {
                  var Gt, Ft = $t.x0, Tt = $t.y0, It = $t.x1, Ct = $t.y1, Kt = (Ft + It) / 2, Vt = (Tt + Ct) / 2;
                  (Gt = Nt[0]) && Pt.push(new oe(Gt, Ft, Tt, Kt, Vt)), (Gt = Nt[1]) && Pt.push(new oe(Gt, Kt, Tt, It, Vt)), (Gt = Nt[2]) && Pt.push(new oe(Gt, Ft, Vt, Kt, Ct)), (Gt = Nt[3]) && Pt.push(new oe(Gt, Kt, Vt, It, Ct));
                }
                Xt.push($t);
              }
              for (; $t = Xt.pop(); ) vt($t.node, $t.x0, $t.y0, $t.x1, $t.y1);
              return this;
            }, fe.x = function(vt) {
              return arguments.length ? (this._x = vt, this) : this._x;
            }, fe.y = function(vt) {
              return arguments.length ? (this._y = vt, this) : this._y;
            };
            var ye = Z(4575);
            function ge(vt) {
              return vt.index;
            }
            function ve(vt, $t) {
              var Pt = vt.get($t);
              if (!Pt) throw new Error("missing: " + $t);
              return Pt;
            }
            function be(vt) {
              var $t, Pt, Xt, Nt, Gt, Ft = ge, Tt = function(hn) {
                return 1 / Math.min(Nt[hn.source.index], Nt[hn.target.index]);
              }, It = ne(30), Ct = 1;
              function Kt(hn) {
                for (var Qt = 0, Ut = vt.length; Qt < Ct; ++Qt) for (var Wt, rn, vn, dn, Ht, jt, cn, wn = 0; wn < Ut; ++wn) rn = (Wt = vt[wn]).source, dn = (vn = Wt.target).x + vn.vx - rn.x - rn.vx || ee(), Ht = vn.y + vn.vy - rn.y - rn.vy || ee(), dn *= jt = ((jt = Math.sqrt(dn * dn + Ht * Ht)) - Pt[wn]) / jt * hn * $t[wn], Ht *= jt, vn.vx -= dn * (cn = Gt[wn]), vn.vy -= Ht * cn, rn.vx += dn * (cn = 1 - cn), rn.vy += Ht * cn;
              }
              function Vt() {
                if (Xt) {
                  var hn, Qt, Ut = Xt.length, Wt = vt.length, rn = (0, ye.Tj)(Xt, Ft);
                  for (hn = 0, Nt = new Array(Ut); hn < Wt; ++hn) (Qt = vt[hn]).index = hn, typeof Qt.source != "object" && (Qt.source = ve(rn, Qt.source)), typeof Qt.target != "object" && (Qt.target = ve(rn, Qt.target)), Nt[Qt.source.index] = (Nt[Qt.source.index] || 0) + 1, Nt[Qt.target.index] = (Nt[Qt.target.index] || 0) + 1;
                  for (hn = 0, Gt = new Array(Wt); hn < Wt; ++hn) Qt = vt[hn], Gt[hn] = Nt[Qt.source.index] / (Nt[Qt.source.index] + Nt[Qt.target.index]);
                  $t = new Array(Wt), Yt(), Pt = new Array(Wt), sn();
                }
              }
              function Yt() {
                if (Xt) for (var hn = 0, Qt = vt.length; hn < Qt; ++hn) $t[hn] = +Tt(vt[hn], hn, vt);
              }
              function sn() {
                if (Xt) for (var hn = 0, Qt = vt.length; hn < Qt; ++hn) Pt[hn] = +It(vt[hn], hn, vt);
              }
              return vt == null && (vt = []), Kt.initialize = function(hn) {
                Xt = hn, Vt();
              }, Kt.links = function(hn) {
                return arguments.length ? (vt = hn, Vt(), Kt) : vt;
              }, Kt.id = function(hn) {
                return arguments.length ? (Ft = hn, Kt) : Ft;
              }, Kt.iterations = function(hn) {
                return arguments.length ? (Ct = +hn, Kt) : Ct;
              }, Kt.strength = function(hn) {
                return arguments.length ? (Tt = typeof hn == "function" ? hn : ne(+hn), Yt(), Kt) : Tt;
              }, Kt.distance = function(hn) {
                return arguments.length ? (It = typeof hn == "function" ? hn : ne(+hn), sn(), Kt) : It;
              }, Kt;
            }
            var Ce = { value: function() {
            } };
            function Se() {
              for (var vt, $t = 0, Pt = arguments.length, Xt = {}; $t < Pt; ++$t) {
                if (!(vt = arguments[$t] + "") || vt in Xt || /[\s.]/.test(vt)) throw new Error("illegal type: " + vt);
                Xt[vt] = [];
              }
              return new we(Xt);
            }
            function we(vt) {
              this._ = vt;
            }
            function Ee(vt, $t) {
              for (var Pt, Xt = 0, Nt = vt.length; Xt < Nt; ++Xt) if ((Pt = vt[Xt]).name === $t) return Pt.value;
            }
            function xe(vt, $t, Pt) {
              for (var Xt = 0, Nt = vt.length; Xt < Nt; ++Xt) if (vt[Xt].name === $t) {
                vt[Xt] = Ce, vt = vt.slice(0, Xt).concat(vt.slice(Xt + 1));
                break;
              }
              return Pt != null && vt.push({ name: $t, value: Pt }), vt;
            }
            we.prototype = Se.prototype = { constructor: we, on: function(vt, $t) {
              var Pt, Xt, Nt = this._, Gt = (Xt = Nt, (vt + "").trim().split(/^|\s+/).map(function(It) {
                var Ct = "", Kt = It.indexOf(".");
                if (Kt >= 0 && (Ct = It.slice(Kt + 1), It = It.slice(0, Kt)), It && !Xt.hasOwnProperty(It)) throw new Error("unknown type: " + It);
                return { type: It, name: Ct };
              })), Ft = -1, Tt = Gt.length;
              if (!(arguments.length < 2)) {
                if ($t != null && typeof $t != "function") throw new Error("invalid callback: " + $t);
                for (; ++Ft < Tt; ) if (Pt = (vt = Gt[Ft]).type) Nt[Pt] = xe(Nt[Pt], vt.name, $t);
                else if ($t == null) for (Pt in Nt) Nt[Pt] = xe(Nt[Pt], vt.name, null);
                return this;
              }
              for (; ++Ft < Tt; ) if ((Pt = (vt = Gt[Ft]).type) && (Pt = Ee(Nt[Pt], vt.name))) return Pt;
            }, copy: function() {
              var vt = {}, $t = this._;
              for (var Pt in $t) vt[Pt] = $t[Pt].slice();
              return new we(vt);
            }, call: function(vt, $t) {
              if ((Pt = arguments.length - 2) > 0) for (var Pt, Xt, Nt = new Array(Pt), Gt = 0; Gt < Pt; ++Gt) Nt[Gt] = arguments[Gt + 2];
              if (!this._.hasOwnProperty(vt)) throw new Error("unknown type: " + vt);
              for (Gt = 0, Pt = (Xt = this._[vt]).length; Gt < Pt; ++Gt) Xt[Gt].value.apply($t, Nt);
            }, apply: function(vt, $t, Pt) {
              if (!this._.hasOwnProperty(vt)) throw new Error("unknown type: " + vt);
              for (var Xt = this._[vt], Nt = 0, Gt = Xt.length; Nt < Gt; ++Nt) Xt[Nt].value.apply($t, Pt);
            } };
            var Le, Qe, ke = Se, Me = 0, Re = 0, De = 0, Pe = 1e3, $e = 0, qe = 0, Xe = 0, Ve = typeof performance == "object" && performance.now ? performance : Date, Ze = typeof window == "object" && window.requestAnimationFrame ? window.requestAnimationFrame.bind(window) : function(vt) {
              setTimeout(vt, 17);
            };
            function Ge() {
              return qe || (Ze(ze), qe = Ve.now() + Xe);
            }
            function ze() {
              qe = 0;
            }
            function We() {
              this._call = this._time = this._next = null;
            }
            function at(vt, $t, Pt) {
              var Xt = new We();
              return Xt.restart(vt, $t, Pt), Xt;
            }
            function ct() {
              qe = ($e = Ve.now()) + Xe, Me = Re = 0;
              try {
                (function() {
                  Ge(), ++Me;
                  for (var vt, $t = Le; $t; ) (vt = qe - $t._time) >= 0 && $t._call.call(null, vt), $t = $t._next;
                  --Me;
                })();
              } finally {
                Me = 0, function() {
                  for (var vt, $t, Pt = Le, Xt = 1 / 0; Pt; ) Pt._call ? (Xt > Pt._time && (Xt = Pt._time), vt = Pt, Pt = Pt._next) : ($t = Pt._next, Pt._next = null, Pt = vt ? vt._next = $t : Le = $t);
                  Qe = vt, dt(Xt);
                }(), qe = 0;
              }
            }
            function et() {
              var vt = Ve.now(), $t = vt - $e;
              $t > Pe && (Xe -= $t, $e = vt);
            }
            function dt(vt) {
              Me || (Re && (Re = clearTimeout(Re)), vt - qe > 24 ? (vt < 1 / 0 && (Re = setTimeout(ct, vt - Ve.now() - Xe)), De && (De = clearInterval(De))) : (De || ($e = Ve.now(), De = setInterval(et, Pe)), Me = 1, Ze(ct)));
            }
            function Je(vt) {
              return vt.x;
            }
            function it(vt) {
              return vt.y;
            }
            We.prototype = at.prototype = { constructor: We, restart: function(vt, $t, Pt) {
              if (typeof vt != "function") throw new TypeError("callback is not a function");
              Pt = (Pt == null ? Ge() : +Pt) + ($t == null ? 0 : +$t), this._next || Qe === this || (Qe ? Qe._next = this : Le = this, Qe = this), this._call = vt, this._time = Pt, dt();
            }, stop: function() {
              this._call && (this._call = null, this._time = 1 / 0, dt());
            } };
            var st = 10, gt = Math.PI * (3 - Math.sqrt(5));
            function bt(vt) {
              var $t, Pt = 1, Xt = 1e-3, Nt = 1 - Math.pow(Xt, 1 / 300), Gt = 0, Ft = 0.6, Tt = (0, ye.Tj)(), It = at(Kt), Ct = ke("tick", "end");
              function Kt() {
                Vt(), Ct.call("tick", $t), Pt < Xt && (It.stop(), Ct.call("end", $t));
              }
              function Vt(hn) {
                var Qt, Ut, Wt = vt.length;
                hn === void 0 && (hn = 1);
                for (var rn = 0; rn < hn; ++rn) for (Pt += (Gt - Pt) * Nt, Tt.each(function(vn) {
                  vn(Pt);
                }), Qt = 0; Qt < Wt; ++Qt) (Ut = vt[Qt]).fx == null ? Ut.x += Ut.vx *= Ft : (Ut.x = Ut.fx, Ut.vx = 0), Ut.fy == null ? Ut.y += Ut.vy *= Ft : (Ut.y = Ut.fy, Ut.vy = 0);
                return $t;
              }
              function Yt() {
                for (var hn, Qt = 0, Ut = vt.length; Qt < Ut; ++Qt) {
                  if ((hn = vt[Qt]).index = Qt, hn.fx != null && (hn.x = hn.fx), hn.fy != null && (hn.y = hn.fy), isNaN(hn.x) || isNaN(hn.y)) {
                    var Wt = st * Math.sqrt(Qt), rn = Qt * gt;
                    hn.x = Wt * Math.cos(rn), hn.y = Wt * Math.sin(rn);
                  }
                  (isNaN(hn.vx) || isNaN(hn.vy)) && (hn.vx = hn.vy = 0);
                }
              }
              function sn(hn) {
                return hn.initialize && hn.initialize(vt), hn;
              }
              return vt == null && (vt = []), Yt(), $t = { tick: Vt, restart: function() {
                return It.restart(Kt), $t;
              }, stop: function() {
                return It.stop(), $t;
              }, nodes: function(hn) {
                return arguments.length ? (vt = hn, Yt(), Tt.each(sn), $t) : vt;
              }, alpha: function(hn) {
                return arguments.length ? (Pt = +hn, $t) : Pt;
              }, alphaMin: function(hn) {
                return arguments.length ? (Xt = +hn, $t) : Xt;
              }, alphaDecay: function(hn) {
                return arguments.length ? (Nt = +hn, $t) : +Nt;
              }, alphaTarget: function(hn) {
                return arguments.length ? (Gt = +hn, $t) : Gt;
              }, velocityDecay: function(hn) {
                return arguments.length ? (Ft = 1 - hn, $t) : 1 - Ft;
              }, force: function(hn, Qt) {
                return arguments.length > 1 ? (Qt == null ? Tt.remove(hn) : Tt.set(hn, sn(Qt)), $t) : Tt.get(hn);
              }, find: function(hn, Qt, Ut) {
                var Wt, rn, vn, dn, Ht, jt = 0, cn = vt.length;
                for (Ut == null ? Ut = 1 / 0 : Ut *= Ut, jt = 0; jt < cn; ++jt) (vn = (Wt = hn - (dn = vt[jt]).x) * Wt + (rn = Qt - dn.y) * rn) < Ut && (Ht = dn, Ut = vn);
                return Ht;
              }, on: function(hn, Qt) {
                return arguments.length > 1 ? (Ct.on(hn, Qt), $t) : Ct.on(hn);
              } };
            }
            function yt() {
              var vt, $t, Pt, Xt, Nt = ne(-30), Gt = 1, Ft = 1 / 0, Tt = 0.81;
              function It(Yt) {
                var sn, hn = vt.length, Qt = le(vt, Je, it).visitAfter(Kt);
                for (Pt = Yt, sn = 0; sn < hn; ++sn) $t = vt[sn], Qt.visit(Vt);
              }
              function Ct() {
                if (vt) {
                  var Yt, sn, hn = vt.length;
                  for (Xt = new Array(hn), Yt = 0; Yt < hn; ++Yt) sn = vt[Yt], Xt[sn.index] = +Nt(sn, Yt, vt);
                }
              }
              function Kt(Yt) {
                var sn, hn, Qt, Ut, Wt, rn = 0, vn = 0;
                if (Yt.length) {
                  for (Qt = Ut = Wt = 0; Wt < 4; ++Wt) (sn = Yt[Wt]) && (hn = Math.abs(sn.value)) && (rn += sn.value, vn += hn, Qt += hn * sn.x, Ut += hn * sn.y);
                  Yt.x = Qt / vn, Yt.y = Ut / vn;
                } else {
                  (sn = Yt).x = sn.data.x, sn.y = sn.data.y;
                  do
                    rn += Xt[sn.data.index];
                  while (sn = sn.next);
                }
                Yt.value = rn;
              }
              function Vt(Yt, sn, hn, Qt) {
                if (!Yt.value) return !0;
                var Ut = Yt.x - $t.x, Wt = Yt.y - $t.y, rn = Qt - sn, vn = Ut * Ut + Wt * Wt;
                if (rn * rn / Tt < vn) return vn < Ft && (Ut === 0 && (vn += (Ut = ee()) * Ut), Wt === 0 && (vn += (Wt = ee()) * Wt), vn < Gt && (vn = Math.sqrt(Gt * vn)), $t.vx += Ut * Yt.value * Pt / vn, $t.vy += Wt * Yt.value * Pt / vn), !0;
                if (!(Yt.length || vn >= Ft)) {
                  (Yt.data !== $t || Yt.next) && (Ut === 0 && (vn += (Ut = ee()) * Ut), Wt === 0 && (vn += (Wt = ee()) * Wt), vn < Gt && (vn = Math.sqrt(Gt * vn)));
                  do
                    Yt.data !== $t && (rn = Xt[Yt.data.index] * Pt / vn, $t.vx += Ut * rn, $t.vy += Wt * rn);
                  while (Yt = Yt.next);
                }
              }
              return It.initialize = function(Yt) {
                vt = Yt, Ct();
              }, It.strength = function(Yt) {
                return arguments.length ? (Nt = typeof Yt == "function" ? Yt : ne(+Yt), Ct(), It) : Nt;
              }, It.distanceMin = function(Yt) {
                return arguments.length ? (Gt = Yt * Yt, It) : Math.sqrt(Gt);
              }, It.distanceMax = function(Yt) {
                return arguments.length ? (Ft = Yt * Yt, It) : Math.sqrt(Ft);
              }, It.theta = function(Yt) {
                return arguments.length ? (Tt = Yt * Yt, It) : Math.sqrt(Tt);
              }, It;
            }
            function St(vt, $t, Pt) {
              var Xt, Nt, Gt, Ft = ne(0.1);
              function Tt(Ct) {
                for (var Kt = 0, Vt = Xt.length; Kt < Vt; ++Kt) {
                  var Yt = Xt[Kt], sn = Yt.x - $t || 1e-6, hn = Yt.y - Pt || 1e-6, Qt = Math.sqrt(sn * sn + hn * hn), Ut = (Gt[Kt] - Qt) * Nt[Kt] * Ct / Qt;
                  Yt.vx += sn * Ut, Yt.vy += hn * Ut;
                }
              }
              function It() {
                if (Xt) {
                  var Ct, Kt = Xt.length;
                  for (Nt = new Array(Kt), Gt = new Array(Kt), Ct = 0; Ct < Kt; ++Ct) Gt[Ct] = +vt(Xt[Ct], Ct, Xt), Nt[Ct] = isNaN(Gt[Ct]) ? 0 : +Ft(Xt[Ct], Ct, Xt);
                }
              }
              return typeof vt != "function" && (vt = ne(+vt)), $t == null && ($t = 0), Pt == null && (Pt = 0), Tt.initialize = function(Ct) {
                Xt = Ct, It();
              }, Tt.strength = function(Ct) {
                return arguments.length ? (Ft = typeof Ct == "function" ? Ct : ne(+Ct), It(), Tt) : Ft;
              }, Tt.radius = function(Ct) {
                return arguments.length ? (vt = typeof Ct == "function" ? Ct : ne(+Ct), It(), Tt) : vt;
              }, Tt.x = function(Ct) {
                return arguments.length ? ($t = +Ct, Tt) : $t;
              }, Tt.y = function(Ct) {
                return arguments.length ? (Pt = +Ct, Tt) : Pt;
              }, Tt;
            }
            function Lt(vt) {
              var $t, Pt, Xt, Nt = ne(0.1);
              function Gt(Tt) {
                for (var It, Ct = 0, Kt = $t.length; Ct < Kt; ++Ct) (It = $t[Ct]).vx += (Xt[Ct] - It.x) * Pt[Ct] * Tt;
              }
              function Ft() {
                if ($t) {
                  var Tt, It = $t.length;
                  for (Pt = new Array(It), Xt = new Array(It), Tt = 0; Tt < It; ++Tt) Pt[Tt] = isNaN(Xt[Tt] = +vt($t[Tt], Tt, $t)) ? 0 : +Nt($t[Tt], Tt, $t);
                }
              }
              return typeof vt != "function" && (vt = ne(vt == null ? 0 : +vt)), Gt.initialize = function(Tt) {
                $t = Tt, Ft();
              }, Gt.strength = function(Tt) {
                return arguments.length ? (Nt = typeof Tt == "function" ? Tt : ne(+Tt), Ft(), Gt) : Nt;
              }, Gt.x = function(Tt) {
                return arguments.length ? (vt = typeof Tt == "function" ? Tt : ne(+Tt), Ft(), Gt) : vt;
              }, Gt;
            }
            function Rt(vt) {
              var $t, Pt, Xt, Nt = ne(0.1);
              function Gt(Tt) {
                for (var It, Ct = 0, Kt = $t.length; Ct < Kt; ++Ct) (It = $t[Ct]).vy += (Xt[Ct] - It.y) * Pt[Ct] * Tt;
              }
              function Ft() {
                if ($t) {
                  var Tt, It = $t.length;
                  for (Pt = new Array(It), Xt = new Array(It), Tt = 0; Tt < It; ++Tt) Pt[Tt] = isNaN(Xt[Tt] = +vt($t[Tt], Tt, $t)) ? 0 : +Nt($t[Tt], Tt, $t);
                }
              }
              return typeof vt != "function" && (vt = ne(vt == null ? 0 : +vt)), Gt.initialize = function(Tt) {
                $t = Tt, Ft();
              }, Gt.strength = function(Tt) {
                return arguments.length ? (Nt = typeof Tt == "function" ? Tt : ne(+Tt), Ft(), Gt) : Nt;
              }, Gt.y = function(Tt) {
                return arguments.length ? (vt = typeof Tt == "function" ? Tt : ne(+Tt), Ft(), Gt) : vt;
              }, Gt;
            }
          }, 36464: function(G, J, Z) {
            function te(pe, ye) {
              if ((ge = (pe = ye ? pe.toExponential(ye - 1) : pe.toExponential()).indexOf("e")) < 0) return null;
              var ge, ve = pe.slice(0, ge);
              return [ve.length > 1 ? ve[0] + ve.slice(2) : ve, +pe.slice(ge + 1)];
            }
            Z.d(J, { GP: function() {
              return de;
            }, OE: function() {
              return me;
            } });
            var ne, ee = /^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;
            function ae(pe) {
              if (!(ye = ee.exec(pe))) throw new Error("invalid format: " + pe);
              var ye;
              return new oe({ fill: ye[1], align: ye[2], sign: ye[3], symbol: ye[4], zero: ye[5], width: ye[6], comma: ye[7], precision: ye[8] && ye[8].slice(1), trim: ye[9], type: ye[10] });
            }
            function oe(pe) {
              this.fill = pe.fill === void 0 ? " " : pe.fill + "", this.align = pe.align === void 0 ? ">" : pe.align + "", this.sign = pe.sign === void 0 ? "-" : pe.sign + "", this.symbol = pe.symbol === void 0 ? "" : pe.symbol + "", this.zero = !!pe.zero, this.width = pe.width === void 0 ? void 0 : +pe.width, this.comma = !!pe.comma, this.precision = pe.precision === void 0 ? void 0 : +pe.precision, this.trim = !!pe.trim, this.type = pe.type === void 0 ? "" : pe.type + "";
            }
            function se(pe, ye) {
              var ge = te(pe, ye);
              if (!ge) return pe + "";
              var ve = ge[0], be = ge[1];
              return be < 0 ? "0." + new Array(-be).join("0") + ve : ve.length > be + 1 ? ve.slice(0, be + 1) + "." + ve.slice(be + 1) : ve + new Array(be - ve.length + 2).join("0");
            }
            ae.prototype = oe.prototype, oe.prototype.toString = function() {
              return this.fill + this.align + this.sign + this.symbol + (this.zero ? "0" : "") + (this.width === void 0 ? "" : Math.max(1, 0 | this.width)) + (this.comma ? "," : "") + (this.precision === void 0 ? "" : "." + Math.max(0, 0 | this.precision)) + (this.trim ? "~" : "") + this.type;
            };
            var ce = { "%": function(pe, ye) {
              return (100 * pe).toFixed(ye);
            }, b: function(pe) {
              return Math.round(pe).toString(2);
            }, c: function(pe) {
              return pe + "";
            }, d: function(pe) {
              return Math.abs(pe = Math.round(pe)) >= 1e21 ? pe.toLocaleString("en").replace(/,/g, "") : pe.toString(10);
            }, e: function(pe, ye) {
              return pe.toExponential(ye);
            }, f: function(pe, ye) {
              return pe.toFixed(ye);
            }, g: function(pe, ye) {
              return pe.toPrecision(ye);
            }, o: function(pe) {
              return Math.round(pe).toString(8);
            }, p: function(pe, ye) {
              return se(100 * pe, ye);
            }, r: se, s: function(pe, ye) {
              var ge = te(pe, ye);
              if (!ge) return pe + "";
              var ve = ge[0], be = ge[1], Ce = be - (ne = 3 * Math.max(-8, Math.min(8, Math.floor(be / 3)))) + 1, Se = ve.length;
              return Ce === Se ? ve : Ce > Se ? ve + new Array(Ce - Se + 1).join("0") : Ce > 0 ? ve.slice(0, Ce) + "." + ve.slice(Ce) : "0." + new Array(1 - Ce).join("0") + te(pe, Math.max(0, ye + Ce - 1))[0];
            }, X: function(pe) {
              return Math.round(pe).toString(16).toUpperCase();
            }, x: function(pe) {
              return Math.round(pe).toString(16);
            } };
            function le(pe) {
              return pe;
            }
            var ue, de, fe = Array.prototype.map, he = ["y", "z", "a", "f", "p", "n", "µ", "m", "", "k", "M", "G", "T", "P", "E", "Z", "Y"];
            function me(pe) {
              var ye, ge, ve = pe.grouping === void 0 || pe.thousands === void 0 ? le : (ye = fe.call(pe.grouping, Number), ge = pe.thousands + "", function(ke, Me) {
                for (var Re = ke.length, De = [], Pe = 0, $e = ye[0], qe = 0; Re > 0 && $e > 0 && (qe + $e + 1 > Me && ($e = Math.max(1, Me - qe)), De.push(ke.substring(Re -= $e, Re + $e)), !((qe += $e + 1) > Me)); ) $e = ye[Pe = (Pe + 1) % ye.length];
                return De.reverse().join(ge);
              }), be = pe.currency === void 0 ? "" : pe.currency[0] + "", Ce = pe.currency === void 0 ? "" : pe.currency[1] + "", Se = pe.decimal === void 0 ? "." : pe.decimal + "", we = pe.numerals === void 0 ? le : /* @__PURE__ */ function(ke) {
                return function(Me) {
                  return Me.replace(/[0-9]/g, function(Re) {
                    return ke[+Re];
                  });
                };
              }(fe.call(pe.numerals, String)), Ee = pe.percent === void 0 ? "%" : pe.percent + "", xe = pe.minus === void 0 ? "-" : pe.minus + "", Le = pe.nan === void 0 ? "NaN" : pe.nan + "";
              function Qe(ke) {
                var Me = (ke = ae(ke)).fill, Re = ke.align, De = ke.sign, Pe = ke.symbol, $e = ke.zero, qe = ke.width, Xe = ke.comma, Ve = ke.precision, Ze = ke.trim, Ge = ke.type;
                Ge === "n" ? (Xe = !0, Ge = "g") : ce[Ge] || (Ve === void 0 && (Ve = 12), Ze = !0, Ge = "g"), ($e || Me === "0" && Re === "=") && ($e = !0, Me = "0", Re = "=");
                var ze = Pe === "$" ? be : Pe === "#" && /[boxX]/.test(Ge) ? "0" + Ge.toLowerCase() : "", We = Pe === "$" ? Ce : /[%p]/.test(Ge) ? Ee : "", at = ce[Ge], ct = /[defgprs%]/.test(Ge);
                function et(dt) {
                  var Je, it, st, gt = ze, bt = We;
                  if (Ge === "c") bt = at(dt) + bt, dt = "";
                  else {
                    var yt = (dt = +dt) < 0 || 1 / dt < 0;
                    if (dt = isNaN(dt) ? Le : at(Math.abs(dt), Ve), Ze && (dt = function(Rt) {
                      e: for (var vt, $t = Rt.length, Pt = 1, Xt = -1; Pt < $t; ++Pt) switch (Rt[Pt]) {
                        case ".":
                          Xt = vt = Pt;
                          break;
                        case "0":
                          Xt === 0 && (Xt = Pt), vt = Pt;
                          break;
                        default:
                          if (!+Rt[Pt]) break e;
                          Xt > 0 && (Xt = 0);
                      }
                      return Xt > 0 ? Rt.slice(0, Xt) + Rt.slice(vt + 1) : Rt;
                    }(dt)), yt && +dt == 0 && De !== "+" && (yt = !1), gt = (yt ? De === "(" ? De : xe : De === "-" || De === "(" ? "" : De) + gt, bt = (Ge === "s" ? he[8 + ne / 3] : "") + bt + (yt && De === "(" ? ")" : ""), ct) {
                      for (Je = -1, it = dt.length; ++Je < it; ) if (48 > (st = dt.charCodeAt(Je)) || st > 57) {
                        bt = (st === 46 ? Se + dt.slice(Je + 1) : dt.slice(Je)) + bt, dt = dt.slice(0, Je);
                        break;
                      }
                    }
                  }
                  Xe && !$e && (dt = ve(dt, 1 / 0));
                  var St = gt.length + dt.length + bt.length, Lt = St < qe ? new Array(qe - St + 1).join(Me) : "";
                  switch (Xe && $e && (dt = ve(Lt + dt, Lt.length ? qe - bt.length : 1 / 0), Lt = ""), Re) {
                    case "<":
                      dt = gt + dt + bt + Lt;
                      break;
                    case "=":
                      dt = gt + Lt + dt + bt;
                      break;
                    case "^":
                      dt = Lt.slice(0, St = Lt.length >> 1) + gt + dt + bt + Lt.slice(St);
                      break;
                    default:
                      dt = Lt + gt + dt + bt;
                  }
                  return we(dt);
                }
                return Ve = Ve === void 0 ? 6 : /[gprs]/.test(Ge) ? Math.max(1, Math.min(21, Ve)) : Math.max(0, Math.min(20, Ve)), et.toString = function() {
                  return ke + "";
                }, et;
              }
              return { format: Qe, formatPrefix: function(ke, Me) {
                var Re, De = Qe(((ke = ae(ke)).type = "f", ke)), Pe = 3 * Math.max(-8, Math.min(8, Math.floor((Re = Me, ((Re = te(Math.abs(Re))) ? Re[1] : NaN) / 3)))), $e = Math.pow(10, -Pe), qe = he[8 + Pe / 3];
                return function(Xe) {
                  return De($e * Xe) + qe;
                };
              } };
            }
            ue = me({ decimal: ".", thousands: ",", grouping: [3], currency: ["$", ""], minus: "-" }), de = ue.format, ue.formatPrefix;
          }, 75987: function(G, J, Z) {
            Z.r(J), Z.d(J, { geoAiry: function() {
              return Xe;
            }, geoAiryRaw: function() {
              return qe;
            }, geoAitoff: function() {
              return Ze;
            }, geoAitoffRaw: function() {
              return Ve;
            }, geoArmadillo: function() {
              return ze;
            }, geoArmadilloRaw: function() {
              return Ge;
            }, geoAugust: function() {
              return at;
            }, geoAugustRaw: function() {
              return We;
            }, geoBaker: function() {
              return Je;
            }, geoBakerRaw: function() {
              return dt;
            }, geoBerghaus: function() {
              return gt;
            }, geoBerghausRaw: function() {
              return st;
            }, geoBertin1953: function() {
              return Pt;
            }, geoBertin1953Raw: function() {
              return $t;
            }, geoBoggs: function() {
              return Kt;
            }, geoBoggsRaw: function() {
              return Ct;
            }, geoBonne: function() {
              return Qt;
            }, geoBonneRaw: function() {
              return hn;
            }, geoBottomley: function() {
              return Wt;
            }, geoBottomleyRaw: function() {
              return Ut;
            }, geoBromley: function() {
              return vn;
            }, geoBromleyRaw: function() {
              return rn;
            }, geoChamberlin: function() {
              return rt;
            }, geoChamberlinAfrica: function() {
              return Ie;
            }, geoChamberlinRaw: function() {
              return Rn;
            }, geoCollignon: function() {
              return pt;
            }, geoCollignonRaw: function() {
              return ft;
            }, geoCraig: function() {
              return Bt;
            }, geoCraigRaw: function() {
              return wt;
            }, geoCraster: function() {
              return Ot;
            }, geoCrasterRaw: function() {
              return Dt;
            }, geoCylindricalEqualArea: function() {
              return yn;
            }, geoCylindricalEqualAreaRaw: function() {
              return un;
            }, geoCylindricalStereographic: function() {
              return gn;
            }, geoCylindricalStereographicRaw: function() {
              return _n;
            }, geoEckert1: function() {
              return an;
            }, geoEckert1Raw: function() {
              return bn;
            }, geoEckert2: function() {
              return Mn;
            }, geoEckert2Raw: function() {
              return Zt;
            }, geoEckert3: function() {
              return In;
            }, geoEckert3Raw: function() {
              return Ln;
            }, geoEckert4: function() {
              return er;
            }, geoEckert4Raw: function() {
              return kn;
            }, geoEckert5: function() {
              return ur;
            }, geoEckert5Raw: function() {
              return jn;
            }, geoEckert6: function() {
              return Jn;
            }, geoEckert6Raw: function() {
              return Kn;
            }, geoEisenlohr: function() {
              return Ar;
            }, geoEisenlohrRaw: function() {
              return mr;
            }, geoFahey: function() {
              return Mr;
            }, geoFaheyRaw: function() {
              return jr;
            }, geoFoucaut: function() {
              return Bi;
            }, geoFoucautRaw: function() {
              return Wr;
            }, geoFoucautSinusoidal: function() {
              return Zr;
            }, geoFoucautSinusoidalRaw: function() {
              return gi;
            }, geoGilbert: function() {
              return $i;
            }, geoGingery: function() {
              return Ci;
            }, geoGingeryRaw: function() {
              return Br;
            }, geoGinzburg4: function() {
              return aa;
            }, geoGinzburg4Raw: function() {
              return ka;
            }, geoGinzburg5: function() {
              return Gr;
            }, geoGinzburg5Raw: function() {
              return hi;
            }, geoGinzburg6: function() {
              return Sr;
            }, geoGinzburg6Raw: function() {
              return Ca;
            }, geoGinzburg8: function() {
              return si;
            }, geoGinzburg8Raw: function() {
              return Dr;
            }, geoGinzburg9: function() {
              return di;
            }, geoGinzburg9Raw: function() {
              return zr;
            }, geoGringorten: function() {
              return wa;
            }, geoGringortenQuincuncial: function() {
              return Ac;
            }, geoGringortenRaw: function() {
              return la;
            }, geoGuyou: function() {
              return rr;
            }, geoGuyouRaw: function() {
              return Ur;
            }, geoHammer: function() {
              return Lt;
            }, geoHammerRaw: function() {
              return yt;
            }, geoHammerRetroazimuthal: function() {
              return Xr;
            }, geoHammerRetroazimuthalRaw: function() {
              return Rr;
            }, geoHealpix: function() {
              return Ai;
            }, geoHealpixRaw: function() {
              return gr;
            }, geoHill: function() {
              return Ni;
            }, geoHillRaw: function() {
              return Li;
            }, geoHomolosine: function() {
              return ua;
            }, geoHomolosineRaw: function() {
              return Vi;
            }, geoHufnagel: function() {
              return Fi;
            }, geoHufnagelRaw: function() {
              return Ki;
            }, geoHyperelliptical: function() {
              return Dn;
            }, geoHyperellipticalRaw: function() {
              return Tn;
            }, geoInterrupt: function() {
              return dr;
            }, geoInterruptedBoggs: function() {
              return ai;
            }, geoInterruptedHomolosine: function() {
              return Ti;
            }, geoInterruptedMollweide: function() {
              return Ei;
            }, geoInterruptedMollweideHemispheres: function() {
              return $o;
            }, geoInterruptedQuarticAuthalic: function() {
              return Ls;
            }, geoInterruptedSinuMollweide: function() {
              return Al;
            }, geoInterruptedSinusoidal: function() {
              return gc;
            }, geoKavrayskiy7: function() {
              return Sl;
            }, geoKavrayskiy7Raw: function() {
              return Gl;
            }, geoLagrange: function() {
              return vs;
            }, geoLagrangeRaw: function() {
              return ls;
            }, geoLarrivee: function() {
              return Xl;
            }, geoLarriveeRaw: function() {
              return Ll;
            }, geoLaskowski: function() {
              return gu;
            }, geoLaskowskiRaw: function() {
              return yc;
            }, geoLittrow: function() {
              return Cu;
            }, geoLittrowRaw: function() {
              return Nc;
            }, geoLoximuthal: function() {
              return Kc;
            }, geoLoximuthalRaw: function() {
              return zc;
            }, geoMiller: function() {
              return Fu;
            }, geoMillerRaw: function() {
              return zi;
            }, geoModifiedStereographic: function() {
              return Wl;
            }, geoModifiedStereographicAlaska: function() {
              return Hl;
            }, geoModifiedStereographicGs48: function() {
              return Kl;
            }, geoModifiedStereographicGs50: function() {
              return uu;
            }, geoModifiedStereographicLee: function() {
              return b1;
            }, geoModifiedStereographicMiller: function() {
              return Hs;
            }, geoModifiedStereographicRaw: function() {
              return d1;
            }, geoMollweide: function() {
              return Ft;
            }, geoMollweideRaw: function() {
              return Gt;
            }, geoMtFlatPolarParabolic: function() {
              return uo;
            }, geoMtFlatPolarParabolicRaw: function() {
              return jo;
            }, geoMtFlatPolarQuartic: function() {
              return So;
            }, geoMtFlatPolarQuarticRaw: function() {
              return so;
            }, geoMtFlatPolarSinusoidal: function() {
              return No;
            }, geoMtFlatPolarSinusoidalRaw: function() {
              return Jc;
            }, geoNaturalEarth: function() {
              return ro.A;
            }, geoNaturalEarth2: function() {
              return bs;
            }, geoNaturalEarth2Raw: function() {
              return Io;
            }, geoNaturalEarthRaw: function() {
              return ro.P;
            }, geoNellHammer: function() {
              return zo;
            }, geoNellHammerRaw: function() {
              return ws;
            }, geoNicolosi: function() {
              return vi;
            }, geoNicolosiRaw: function() {
              return Ms;
            }, geoPatterson: function() {
              return To;
            }, geoPattersonRaw: function() {
              return io;
            }, geoPeirceQuincuncial: function() {
              return yu;
            }, geoPierceQuincuncial: function() {
              return yu;
            }, geoPolyconic: function() {
              return po;
            }, geoPolyconicRaw: function() {
              return Mo;
            }, geoPolyhedral: function() {
              return Fo;
            }, geoPolyhedralButterfly: function() {
              return z1;
            }, geoPolyhedralCollignon: function() {
              return M1;
            }, geoPolyhedralWaterman: function() {
              return ku;
            }, geoProject: function() {
              return i1;
            }, geoQuantize: function() {
              return au;
            }, geoQuincuncial: function() {
              return du;
            }, geoRectangularPolyconic: function() {
              return Cc;
            }, geoRectangularPolyconicRaw: function() {
              return Qc;
            }, geoRobinson: function() {
              return cs;
            }, geoRobinsonRaw: function() {
              return uc;
            }, geoSatellite: function() {
              return Hu;
            }, geoSatelliteRaw: function() {
              return Mu;
            }, geoSinuMollweide: function() {
              return Mi;
            }, geoSinuMollweideRaw: function() {
              return ki;
            }, geoSinusoidal: function() {
              return sn;
            }, geoSinusoidalRaw: function() {
              return Yt;
            }, geoStitch: function() {
              return sc;
            }, geoTimes: function() {
              return Vc;
            }, geoTimesRaw: function() {
              return hc;
            }, geoTwoPointAzimuthal: function() {
              return Uc;
            }, geoTwoPointAzimuthalRaw: function() {
              return Zo;
            }, geoTwoPointAzimuthalUsa: function() {
              return Ou;
            }, geoTwoPointEquidistant: function() {
              return Wc;
            }, geoTwoPointEquidistantRaw: function() {
              return El;
            }, geoTwoPointEquidistantUsa: function() {
              return eu;
            }, geoVanDerGrinten: function() {
              return gd;
            }, geoVanDerGrinten2: function() {
              return Yc;
            }, geoVanDerGrinten2Raw: function() {
              return Jl;
            }, geoVanDerGrinten3: function() {
              return Ko;
            }, geoVanDerGrinten3Raw: function() {
              return nc;
            }, geoVanDerGrinten4: function() {
              return qs;
            }, geoVanDerGrinten4Raw: function() {
              return ks;
            }, geoVanDerGrintenRaw: function() {
              return hu;
            }, geoWagner: function() {
              return $l;
            }, geoWagner4: function() {
              return zs;
            }, geoWagner4Raw: function() {
              return Ul;
            }, geoWagner6: function() {
              return lc;
            }, geoWagner6Raw: function() {
              return Os;
            }, geoWagner7: function() {
              return o1;
            }, geoWagnerRaw: function() {
              return a1;
            }, geoWiechel: function() {
              return Iu;
            }, geoWiechelRaw: function() {
              return xc;
            }, geoWinkel3: function() {
              return w1;
            }, geoWinkel3Raw: function() {
              return tu;
            } });
            var te = Z(94684), ne = Math.abs, ee = Math.atan, ae = Math.atan2, oe = Math.cos, se = Math.exp, ce = Math.floor, le = Math.log, ue = Math.max, de = Math.min, fe = Math.pow, he = Math.round, me = Math.sign || function($n) {
              return $n > 0 ? 1 : $n < 0 ? -1 : 0;
            }, pe = Math.sin, ye = Math.tan, ge = 1e-6, ve = 1e-12, be = Math.PI, Ce = be / 2, Se = be / 4, we = Math.SQRT1_2, Ee = De(2), xe = De(be), Le = 2 * be, Qe = 180 / be, ke = be / 180;
            function Me($n) {
              return $n > 1 ? Ce : $n < -1 ? -Ce : Math.asin($n);
            }
            function Re($n) {
              return $n > 1 ? 0 : $n < -1 ? be : Math.acos($n);
            }
            function De($n) {
              return $n > 0 ? Math.sqrt($n) : 0;
            }
            function Pe($n) {
              return (se($n) - se(-$n)) / 2;
            }
            function $e($n) {
              return (se($n) + se(-$n)) / 2;
            }
            function qe($n) {
              var zn = ye($n / 2), Gn = 2 * le(oe($n / 2)) / (zn * zn);
              function Ke(Fe, Te) {
                var Ae = oe(Fe), Oe = oe(Te), Be = pe(Te), Ue = Oe * Ae, nt = -((1 - Ue ? le((1 + Ue) / 2) / (1 - Ue) : -0.5) + Gn / (1 + Ue));
                return [nt * Oe * pe(Fe), nt * Be];
              }
              return Ke.invert = function(Fe, Te) {
                var Ae, Oe = De(Fe * Fe + Te * Te), Be = -$n / 2, Ue = 50;
                if (!Oe) return [0, 0];
                do {
                  var nt = Be / 2, ht = oe(nt), Et = pe(nt), Mt = Et / ht, qt = -le(ne(ht));
                  Be -= Ae = (2 / Mt * qt - Gn * Mt - Oe) / (-qt / (Et * Et) + 1 - Gn / (2 * ht * ht)) * (ht < 0 ? 0.7 : 1);
                } while (ne(Ae) > ge && --Ue > 0);
                var tn = pe(Be);
                return [ae(Fe * tn, Oe * oe(Be)), Me(Te * tn / Oe)];
              }, Ke;
            }
            function Xe() {
              var $n = Ce, zn = (0, te.U)(qe), Gn = zn($n);
              return Gn.radius = function(Ke) {
                return arguments.length ? zn($n = Ke * ke) : $n * Qe;
              }, Gn.scale(179.976).clipAngle(147);
            }
            function Ve($n, zn) {
              var Gn = oe(zn), Ke = function(Fe) {
                return Fe ? Fe / Math.sin(Fe) : 1;
              }(Re(Gn * oe($n /= 2)));
              return [2 * Gn * pe($n) * Ke, pe(zn) * Ke];
            }
            function Ze() {
              return (0, te.A)(Ve).scale(152.63);
            }
            function Ge($n) {
              var zn = pe($n), Gn = oe($n), Ke = $n >= 0 ? 1 : -1, Fe = ye(Ke * $n), Te = (1 + zn - Gn) / 2;
              function Ae(Oe, Be) {
                var Ue = oe(Be), nt = oe(Oe /= 2);
                return [(1 + Ue) * pe(Oe), (Ke * Be > -ae(nt, Fe) - 1e-3 ? 0 : 10 * -Ke) + Te + pe(Be) * Gn - (1 + Ue) * zn * nt];
              }
              return Ae.invert = function(Oe, Be) {
                var Ue = 0, nt = 0, ht = 50;
                do {
                  var Et = oe(Ue), Mt = pe(Ue), qt = oe(nt), tn = pe(nt), xn = 1 + qt, An = xn * Mt - Oe, On = Te + tn * Gn - xn * zn * Et - Be, Bn = xn * Et / 2, qn = -Mt * tn, Vn = zn * xn * Mt / 2, sr = Gn * qt + zn * Et * tn, Un = qn * Vn - sr * Bn, wr = (On * qn - An * sr) / Un / 2, Cr = (An * Vn - On * Bn) / Un;
                  ne(Cr) > 2 && (Cr /= 2), Ue -= wr, nt -= Cr;
                } while ((ne(wr) > ge || ne(Cr) > ge) && --ht > 0);
                return Ke * nt > -ae(oe(Ue), Fe) - 1e-3 ? [2 * Ue, nt] : null;
              }, Ae;
            }
            function ze() {
              var $n = 20 * ke, zn = $n >= 0 ? 1 : -1, Gn = ye(zn * $n), Ke = (0, te.U)(Ge), Fe = Ke($n), Te = Fe.stream;
              return Fe.parallel = function(Ae) {
                return arguments.length ? (Gn = ye((zn = ($n = Ae * ke) >= 0 ? 1 : -1) * $n), Ke($n)) : $n * Qe;
              }, Fe.stream = function(Ae) {
                var Oe = Fe.rotate(), Be = Te(Ae), Ue = (Fe.rotate([0, 0]), Te(Ae)), nt = Fe.precision();
                return Fe.rotate(Oe), Be.sphere = function() {
                  Ue.polygonStart(), Ue.lineStart();
                  for (var ht = -180 * zn; zn * ht < 180; ht += 90 * zn) Ue.point(ht, 90 * zn);
                  if ($n) for (; zn * (ht -= 3 * zn * nt) >= -180; ) Ue.point(ht, zn * -ae(oe(ht * ke / 2), Gn) * Qe);
                  Ue.lineEnd(), Ue.polygonEnd();
                }, Be;
              }, Fe.scale(218.695).center([0, 28.0974]);
            }
            function We($n, zn) {
              var Gn = ye(zn / 2), Ke = De(1 - Gn * Gn), Fe = 1 + Ke * oe($n /= 2), Te = pe($n) * Ke / Fe, Ae = Gn / Fe, Oe = Te * Te, Be = Ae * Ae;
              return [1.3333333333333333 * Te * (3 + Oe - 3 * Be), 1.3333333333333333 * Ae * (3 + 3 * Oe - Be)];
            }
            function at() {
              return (0, te.A)(We).scale(66.1603);
            }
            Ve.invert = function($n, zn) {
              if (!($n * $n + 4 * zn * zn > be * be + ge)) {
                var Gn = $n, Ke = zn, Fe = 25;
                do {
                  var Te, Ae = pe(Gn), Oe = pe(Gn / 2), Be = oe(Gn / 2), Ue = pe(Ke), nt = oe(Ke), ht = pe(2 * Ke), Et = Ue * Ue, Mt = nt * nt, qt = Oe * Oe, tn = 1 - Mt * Be * Be, xn = tn ? Re(nt * Be) * De(Te = 1 / tn) : Te = 0, An = 2 * xn * nt * Oe - $n, On = xn * Ue - zn, Bn = Te * (Mt * qt + xn * nt * Be * Et), qn = Te * (0.5 * Ae * ht - 2 * xn * Ue * Oe), Vn = 0.25 * Te * (ht * Oe - xn * Ue * Mt * Ae), sr = Te * (Et * Be + xn * qt * nt), Un = qn * Vn - sr * Bn;
                  if (!Un) break;
                  var wr = (On * qn - An * sr) / Un, Cr = (An * Vn - On * Bn) / Un;
                  Gn -= wr, Ke -= Cr;
                } while ((ne(wr) > ge || ne(Cr) > ge) && --Fe > 0);
                return [Gn, Ke];
              }
            }, We.invert = function($n, zn) {
              if (zn *= 0.375, !($n *= 0.375) && ne(zn) > 1) return null;
              var Gn = 1 + $n * $n + zn * zn, Ke = De((Gn - De(Gn * Gn - 4 * zn * zn)) / 2), Fe = Me(Ke) / 3, Te = Ke ? function(Ue) {
                return le(Ue + De(Ue * Ue - 1));
              }(ne(zn / Ke)) / 3 : function(Ue) {
                return le(Ue + De(Ue * Ue + 1));
              }(ne($n)) / 3, Ae = oe(Fe), Oe = $e(Te), Be = Oe * Oe - Ae * Ae;
              return [2 * me($n) * ae(Pe(Te) * Ae, 0.25 - Be), 2 * me(zn) * ae(Oe * pe(Fe), 0.25 + Be)];
            };
            var ct = De(8), et = le(1 + Ee);
            function dt($n, zn) {
              var Gn = ne(zn);
              return Gn < Se ? [$n, le(ye(Se + zn / 2))] : [$n * oe(Gn) * (2 * Ee - 1 / pe(Gn)), me(zn) * (2 * Ee * (Gn - Se) - le(ye(Gn / 2)))];
            }
            function Je() {
              return (0, te.A)(dt).scale(112.314);
            }
            dt.invert = function($n, zn) {
              if ((Ke = ne(zn)) < et) return [$n, 2 * ee(se(zn)) - Ce];
              var Gn, Ke, Fe = Se, Te = 25;
              do {
                var Ae = oe(Fe / 2), Oe = ye(Fe / 2);
                Fe -= Gn = (ct * (Fe - Se) - le(Oe) - Ke) / (ct - Ae * Ae / (2 * Oe));
              } while (ne(Gn) > ve && --Te > 0);
              return [$n / (oe(Fe) * (ct - 1 / pe(Fe))), me(zn) * Fe];
            };
            var it = Z(61957);
            function st($n) {
              var zn = 2 * be / $n;
              function Gn(Ke, Fe) {
                var Te = (0, it.j)(Ke, Fe);
                if (ne(Ke) > Ce) {
                  var Ae = ae(Te[1], Te[0]), Oe = De(Te[0] * Te[0] + Te[1] * Te[1]), Be = zn * he((Ae - Ce) / zn) + Ce, Ue = ae(pe(Ae -= Be), 2 - oe(Ae));
                  Ae = Be + Me(be / Oe * pe(Ue)) - Ue, Te[0] = Oe * oe(Ae), Te[1] = Oe * pe(Ae);
                }
                return Te;
              }
              return Gn.invert = function(Ke, Fe) {
                var Te = De(Ke * Ke + Fe * Fe);
                if (Te > Ce) {
                  var Ae = ae(Fe, Ke), Oe = zn * he((Ae - Ce) / zn) + Ce, Be = Ae > Oe ? -1 : 1, Ue = Te * oe(Oe - Ae), nt = 1 / ye(Be * Re((Ue - be) / De(be * (be - 2 * Ue) + Te * Te)));
                  Ae = Oe + 2 * ee((nt + Be * De(nt * nt - 3)) / 3), Ke = Te * oe(Ae), Fe = Te * pe(Ae);
                }
                return it.j.invert(Ke, Fe);
              }, Gn;
            }
            function gt() {
              var $n = 5, zn = (0, te.U)(st), Gn = zn($n), Ke = Gn.stream, Fe = 0.01, Te = -oe(Fe * ke), Ae = pe(Fe * ke);
              return Gn.lobes = function(Oe) {
                return arguments.length ? zn($n = +Oe) : $n;
              }, Gn.stream = function(Oe) {
                var Be = Gn.rotate(), Ue = Ke(Oe), nt = (Gn.rotate([0, 0]), Ke(Oe));
                return Gn.rotate(Be), Ue.sphere = function() {
                  nt.polygonStart(), nt.lineStart();
                  for (var ht = 0, Et = 360 / $n, Mt = 2 * be / $n, qt = 90 - 180 / $n, tn = Ce; ht < $n; ++ht, qt -= Et, tn -= Mt) nt.point(ae(Ae * oe(tn), Te) * Qe, Me(Ae * pe(tn)) * Qe), qt < -90 ? (nt.point(-90, -180 - qt - Fe), nt.point(-90, -180 - qt + Fe)) : (nt.point(90, qt + Fe), nt.point(90, qt - Fe));
                  nt.lineEnd(), nt.polygonEnd();
                }, Ue;
              }, Gn.scale(87.8076).center([0, 17.1875]).clipAngle(179.999);
            }
            var bt = Z(30729);
            function yt($n, zn) {
              if (arguments.length < 2 && (zn = $n), zn === 1) return bt.n;
              if (zn === 1 / 0) return St;
              function Gn(Ke, Fe) {
                var Te = (0, bt.n)(Ke / zn, Fe);
                return Te[0] *= $n, Te;
              }
              return Gn.invert = function(Ke, Fe) {
                var Te = bt.n.invert(Ke / $n, Fe);
                return Te[0] *= zn, Te;
              }, Gn;
            }
            function St($n, zn) {
              return [$n * oe(zn) / oe(zn /= 2), 2 * pe(zn)];
            }
            function Lt() {
              var $n = 2, zn = (0, te.U)(yt), Gn = zn($n);
              return Gn.coefficient = function(Ke) {
                return arguments.length ? zn($n = +Ke) : $n;
              }, Gn.scale(169.529);
            }
            function Rt($n, zn, Gn) {
              var Ke, Fe, Te, Ae = 100;
              Gn = Gn === void 0 ? 0 : +Gn, zn = +zn;
              do
                (Fe = $n(Gn)) === (Te = $n(Gn + ge)) && (Te = Fe + ge), Gn -= Ke = -1 * ge * (Fe - zn) / (Fe - Te);
              while (Ae-- > 0 && ne(Ke) > ge);
              return Ae < 0 ? NaN : Gn;
            }
            function vt($n, zn, Gn) {
              return zn === void 0 && (zn = 40), Gn === void 0 && (Gn = ve), function(Ke, Fe, Te, Ae) {
                var Oe, Be, Ue;
                Te = Te === void 0 ? 0 : +Te, Ae = Ae === void 0 ? 0 : +Ae;
                for (var nt = 0; nt < zn; nt++) {
                  var ht = $n(Te, Ae), Et = ht[0] - Ke, Mt = ht[1] - Fe;
                  if (ne(Et) < Gn && ne(Mt) < Gn) break;
                  var qt = Et * Et + Mt * Mt;
                  if (qt > Oe) Te -= Be /= 2, Ae -= Ue /= 2;
                  else {
                    Oe = qt;
                    var tn = (Te > 0 ? -1 : 1) * Gn, xn = (Ae > 0 ? -1 : 1) * Gn, An = $n(Te + tn, Ae), On = $n(Te, Ae + xn), Bn = (An[0] - ht[0]) / tn, qn = (An[1] - ht[1]) / tn, Vn = (On[0] - ht[0]) / xn, sr = (On[1] - ht[1]) / xn, Un = sr * Bn - qn * Vn, wr = (ne(Un) < 0.5 ? 0.5 : 1) / Un;
                    if (Te += Be = (Mt * Vn - Et * sr) * wr, Ae += Ue = (Et * qn - Mt * Bn) * wr, ne(Be) < Gn && ne(Ue) < Gn) break;
                  }
                }
                return [Te, Ae];
              };
            }
            function $t() {
              var $n = yt(1.68, 2);
              function zn(Gn, Ke) {
                if (Gn + Ke < -1.4) {
                  var Fe = (Gn - Ke + 1.6) * (Gn + Ke + 1.4) / 8;
                  Gn += Fe, Ke -= 0.8 * Fe * pe(Ke + be / 2);
                }
                var Te = $n(Gn, Ke), Ae = (1 - oe(Gn * Ke)) / 12;
                return Te[1] < 0 && (Te[0] *= 1 + Ae), Te[1] > 0 && (Te[1] *= 1 + Ae / 1.5 * Te[0] * Te[0]), Te;
              }
              return zn.invert = vt(zn), zn;
            }
            function Pt() {
              return (0, te.A)($t()).rotate([-16.5, -42]).scale(176.57).center([7.93, 0.09]);
            }
            function Xt($n, zn) {
              var Gn, Ke = $n * pe(zn), Fe = 30;
              do
                zn -= Gn = (zn + pe(zn) - Ke) / (1 + oe(zn));
              while (ne(Gn) > ge && --Fe > 0);
              return zn / 2;
            }
            function Nt($n, zn, Gn) {
              function Ke(Fe, Te) {
                return [$n * Fe * oe(Te = Xt(Gn, Te)), zn * pe(Te)];
              }
              return Ke.invert = function(Fe, Te) {
                return Te = Me(Te / zn), [Fe / ($n * oe(Te)), Me((2 * Te + pe(2 * Te)) / Gn)];
              }, Ke;
            }
            St.invert = function($n, zn) {
              var Gn = 2 * Me(zn / 2);
              return [$n * oe(Gn / 2) / oe(Gn), Gn];
            };
            var Gt = Nt(Ee / Ce, Ee, be);
            function Ft() {
              return (0, te.A)(Gt).scale(169.529);
            }
            var Tt = 2.00276, It = 1.11072;
            function Ct($n, zn) {
              var Gn = Xt(be, zn);
              return [Tt * $n / (1 / oe(zn) + It / oe(Gn)), (zn + Ee * pe(Gn)) / Tt];
            }
            function Kt() {
              return (0, te.A)(Ct).scale(160.857);
            }
            function Vt($n) {
              var zn = 0, Gn = (0, te.U)($n), Ke = Gn(zn);
              return Ke.parallel = function(Fe) {
                return arguments.length ? Gn(zn = Fe * ke) : zn * Qe;
              }, Ke;
            }
            function Yt($n, zn) {
              return [$n * oe(zn), zn];
            }
            function sn() {
              return (0, te.A)(Yt).scale(152.63);
            }
            function hn($n) {
              if (!$n) return Yt;
              var zn = 1 / ye($n);
              function Gn(Ke, Fe) {
                var Te = zn + $n - Fe, Ae = Te && Ke * oe(Fe) / Te;
                return [Te * pe(Ae), zn - Te * oe(Ae)];
              }
              return Gn.invert = function(Ke, Fe) {
                var Te = De(Ke * Ke + (Fe = zn - Fe) * Fe), Ae = zn + $n - Te;
                return [Te / oe(Ae) * ae(Ke, Fe), Ae];
              }, Gn;
            }
            function Qt() {
              return Vt(hn).scale(123.082).center([0, 26.1441]).parallel(45);
            }
            function Ut($n) {
              function zn(Gn, Ke) {
                var Fe = Ce - Ke, Te = Fe && Gn * $n * pe(Fe) / Fe;
                return [Fe * pe(Te) / $n, Ce - Fe * oe(Te)];
              }
              return zn.invert = function(Gn, Ke) {
                var Fe = Gn * $n, Te = Ce - Ke, Ae = De(Fe * Fe + Te * Te), Oe = ae(Fe, Te);
                return [(Ae ? Ae / pe(Ae) : 1) * Oe / $n, Ce - Ae];
              }, zn;
            }
            function Wt() {
              var $n = 0.5, zn = (0, te.U)(Ut), Gn = zn($n);
              return Gn.fraction = function(Ke) {
                return arguments.length ? zn($n = +Ke) : $n;
              }, Gn.scale(158.837);
            }
            Ct.invert = function($n, zn) {
              var Gn, Ke, Fe = Tt * zn, Te = zn < 0 ? -Se : Se, Ae = 25;
              do
                Ke = Fe - Ee * pe(Te), Te -= Gn = (pe(2 * Te) + 2 * Te - be * pe(Ke)) / (2 * oe(2 * Te) + 2 + be * oe(Ke) * Ee * oe(Te));
              while (ne(Gn) > ge && --Ae > 0);
              return Ke = Fe - Ee * pe(Te), [$n * (1 / oe(Ke) + It / oe(Te)) / Tt, Ke];
            }, Yt.invert = function($n, zn) {
              return [$n / oe(zn), zn];
            };
            var rn = Nt(1, 4 / be, be);
            function vn() {
              return (0, te.A)(rn).scale(152.63);
            }
            var dn = Z(30021), Ht = Z(30915);
            function jt($n, zn, Gn, Ke, Fe, Te) {
              var Ae, Oe = oe(Te);
              if (ne($n) > 1 || ne(Te) > 1) Ae = Re(Gn * Fe + zn * Ke * Oe);
              else {
                var Be = pe($n / 2), Ue = pe(Te / 2);
                Ae = 2 * Me(De(Be * Be + zn * Ke * Ue * Ue));
              }
              return ne(Ae) > ge ? [Ae, ae(Ke * pe(Te), zn * Fe - Gn * Ke * Oe)] : [0, 0];
            }
            function cn($n, zn, Gn) {
              return Re(($n * $n + zn * zn - Gn * Gn) / (2 * $n * zn));
            }
            function wn($n) {
              return $n - 2 * be * ce(($n + be) / (2 * be));
            }
            function Rn($n, zn, Gn) {
              for (var Ke, Fe = [[$n[0], $n[1], pe($n[1]), oe($n[1])], [zn[0], zn[1], pe(zn[1]), oe(zn[1])], [Gn[0], Gn[1], pe(Gn[1]), oe(Gn[1])]], Te = Fe[2], Ae = 0; Ae < 3; ++Ae, Te = Ke) Ke = Fe[Ae], Te.v = jt(Ke[1] - Te[1], Te[3], Te[2], Ke[3], Ke[2], Ke[0] - Te[0]), Te.point = [0, 0];
              var Oe = cn(Fe[0].v[0], Fe[2].v[0], Fe[1].v[0]), Be = cn(Fe[0].v[0], Fe[1].v[0], Fe[2].v[0]), Ue = be - Oe;
              Fe[2].point[1] = 0, Fe[0].point[0] = -(Fe[1].point[0] = Fe[0].v[0] / 2);
              var nt = [Fe[2].point[0] = Fe[0].point[0] + Fe[2].v[0] * oe(Oe), 2 * (Fe[0].point[1] = Fe[1].point[1] = Fe[2].v[0] * pe(Oe))];
              return function(ht, Et) {
                var Mt, qt = pe(Et), tn = oe(Et), xn = new Array(3);
                for (Mt = 0; Mt < 3; ++Mt) {
                  var An = Fe[Mt];
                  if (xn[Mt] = jt(Et - An[1], An[3], An[2], tn, qt, ht - An[0]), !xn[Mt][0]) return An.point;
                  xn[Mt][1] = wn(xn[Mt][1] - An.v[1]);
                }
                var On = nt.slice();
                for (Mt = 0; Mt < 3; ++Mt) {
                  var Bn = Mt == 2 ? 0 : Mt + 1, qn = cn(Fe[Mt].v[0], xn[Mt][0], xn[Bn][0]);
                  xn[Mt][1] < 0 && (qn = -qn), Mt ? Mt == 1 ? (qn = Be - qn, On[0] -= xn[Mt][0] * oe(qn), On[1] -= xn[Mt][0] * pe(qn)) : (qn = Ue - qn, On[0] += xn[Mt][0] * oe(qn), On[1] += xn[Mt][0] * pe(qn)) : (On[0] += xn[Mt][0] * oe(qn), On[1] -= xn[Mt][0] * pe(qn));
                }
                return On[0] /= 3, On[1] /= 3, On;
              };
            }
            function Pn($n) {
              return $n[0] *= ke, $n[1] *= ke, $n;
            }
            function Ie() {
              return rt([0, 22], [45, 22], [22.5, -22]).scale(380).center([22.5, 2]);
            }
            function rt($n, zn, Gn) {
              var Ke = (0, dn.A)({ type: "MultiPoint", coordinates: [$n, zn, Gn] }), Fe = [-Ke[0], -Ke[1]], Te = (0, Ht.A)(Fe), Ae = Rn(Pn(Te($n)), Pn(Te(zn)), Pn(Te(Gn)));
              Ae.invert = vt(Ae);
              var Oe = (0, te.A)(Ae).rotate(Fe), Be = Oe.center;
              return delete Oe.rotate, Oe.center = function(Ue) {
                return arguments.length ? Be(Te(Ue)) : Te.invert(Be());
              }, Oe.clipAngle(90);
            }
            function ft($n, zn) {
              var Gn = De(1 - pe(zn));
              return [2 / xe * $n * Gn, xe * (1 - Gn)];
            }
            function pt() {
              return (0, te.A)(ft).scale(95.6464).center([0, 30]);
            }
            function wt($n) {
              var zn = ye($n);
              function Gn(Ke, Fe) {
                return [Ke, (Ke ? Ke / pe(Ke) : 1) * (pe(Fe) * oe(Ke) - zn * oe(Fe))];
              }
              return Gn.invert = zn ? function(Ke, Fe) {
                Ke && (Fe *= pe(Ke) / Ke);
                var Te = oe(Ke);
                return [Ke, 2 * ae(De(Te * Te + zn * zn - Fe * Fe) - Te, zn - Fe)];
              } : function(Ke, Fe) {
                return [Ke, Me(Ke ? Fe * ye(Ke) / Ke : Fe)];
              }, Gn;
            }
            function Bt() {
              return Vt(wt).scale(249.828).clipAngle(90);
            }
            ft.invert = function($n, zn) {
              var Gn = (Gn = zn / xe - 1) * Gn;
              return [Gn > 0 ? $n * De(be / Gn) / 2 : 0, Me(1 - Gn)];
            };
            var At = De(3);
            function Dt($n, zn) {
              return [At * $n * (2 * oe(2 * zn / 3) - 1) / xe, At * xe * pe(zn / 3)];
            }
            function Ot() {
              return (0, te.A)(Dt).scale(156.19);
            }
            function un($n) {
              var zn = oe($n);
              function Gn(Ke, Fe) {
                return [Ke * zn, pe(Fe) / zn];
              }
              return Gn.invert = function(Ke, Fe) {
                return [Ke / zn, Me(Fe * zn)];
              }, Gn;
            }
            function yn() {
              return Vt(un).parallel(38.58).scale(195.044);
            }
            function _n($n) {
              var zn = oe($n);
              function Gn(Ke, Fe) {
                return [Ke * zn, (1 + zn) * ye(Fe / 2)];
              }
              return Gn.invert = function(Ke, Fe) {
                return [Ke / zn, 2 * ee(Fe / (1 + zn))];
              }, Gn;
            }
            function gn() {
              return Vt(_n).scale(124.75);
            }
            function bn($n, zn) {
              var Gn = De(8 / (3 * be));
              return [Gn * $n * (1 - ne(zn) / be), Gn * zn];
            }
            function an() {
              return (0, te.A)(bn).scale(165.664);
            }
            function Zt($n, zn) {
              var Gn = De(4 - 3 * pe(ne(zn)));
              return [2 / De(6 * be) * $n * Gn, me(zn) * De(2 * be / 3) * (2 - Gn)];
            }
            function Mn() {
              return (0, te.A)(Zt).scale(165.664);
            }
            function Ln($n, zn) {
              var Gn = De(be * (4 + be));
              return [2 / Gn * $n * (1 + De(1 - 4 * zn * zn / (be * be))), 4 / Gn * zn];
            }
            function In() {
              return (0, te.A)(Ln).scale(180.739);
            }
            function kn($n, zn) {
              var Gn = (2 + Ce) * pe(zn);
              zn /= 2;
              for (var Ke = 0, Fe = 1 / 0; Ke < 10 && ne(Fe) > ge; Ke++) {
                var Te = oe(zn);
                zn -= Fe = (zn + pe(zn) * (Te + 2) - Gn) / (2 * Te * (1 + Te));
              }
              return [2 / De(be * (4 + be)) * $n * (1 + oe(zn)), 2 * De(be / (4 + be)) * pe(zn)];
            }
            function er() {
              return (0, te.A)(kn).scale(180.739);
            }
            function jn($n, zn) {
              return [$n * (1 + oe(zn)) / De(2 + be), 2 * zn / De(2 + be)];
            }
            function ur() {
              return (0, te.A)(jn).scale(173.044);
            }
            function Kn($n, zn) {
              for (var Gn = (1 + Ce) * pe(zn), Ke = 0, Fe = 1 / 0; Ke < 10 && ne(Fe) > ge; Ke++) zn -= Fe = (zn + pe(zn) - Gn) / (1 + oe(zn));
              return Gn = De(2 + be), [$n * (1 + oe(zn)) / Gn, 2 * zn / Gn];
            }
            function Jn() {
              return (0, te.A)(Kn).scale(173.044);
            }
            Dt.invert = function($n, zn) {
              var Gn = 3 * Me(zn / (At * xe));
              return [xe * $n / (At * (2 * oe(2 * Gn / 3) - 1)), Gn];
            }, bn.invert = function($n, zn) {
              var Gn = De(8 / (3 * be)), Ke = zn / Gn;
              return [$n / (Gn * (1 - ne(Ke) / be)), Ke];
            }, Zt.invert = function($n, zn) {
              var Gn = 2 - ne(zn) / De(2 * be / 3);
              return [$n * De(6 * be) / (2 * Gn), me(zn) * Me((4 - Gn * Gn) / 3)];
            }, Ln.invert = function($n, zn) {
              var Gn = De(be * (4 + be)) / 2;
              return [$n * Gn / (1 + De(1 - zn * zn * (4 + be) / (4 * be))), zn * Gn / 2];
            }, kn.invert = function($n, zn) {
              var Gn = zn * De((4 + be) / be) / 2, Ke = Me(Gn), Fe = oe(Ke);
              return [$n / (2 / De(be * (4 + be)) * (1 + Fe)), Me((Ke + Gn * (Fe + 2)) / (2 + Ce))];
            }, jn.invert = function($n, zn) {
              var Gn = De(2 + be), Ke = zn * Gn / 2;
              return [Gn * $n / (1 + oe(Ke)), Ke];
            }, Kn.invert = function($n, zn) {
              var Gn = 1 + Ce, Ke = De(Gn / 2);
              return [2 * $n * Ke / (1 + oe(zn *= Ke)), Me((zn + pe(zn)) / Gn)];
            };
            var Lr = 3 + 2 * Ee;
            function mr($n, zn) {
              var Gn = pe($n /= 2), Ke = oe($n), Fe = De(oe(zn)), Te = oe(zn /= 2), Ae = pe(zn) / (Te + Ee * Ke * Fe), Oe = De(2 / (1 + Ae * Ae)), Be = De((Ee * Te + (Ke + Gn) * Fe) / (Ee * Te + (Ke - Gn) * Fe));
              return [Lr * (Oe * (Be - 1 / Be) - 2 * le(Be)), Lr * (Oe * Ae * (Be + 1 / Be) - 2 * ee(Ae))];
            }
            function Ar() {
              return (0, te.A)(mr).scale(62.5271);
            }
            mr.invert = function($n, zn) {
              if (!(Gn = We.invert($n / 1.2, 1.065 * zn))) return null;
              var Gn, Ke = Gn[0], Fe = Gn[1], Te = 20;
              $n /= Lr, zn /= Lr;
              do {
                var Ae = Ke / 2, Oe = Fe / 2, Be = pe(Ae), Ue = oe(Ae), nt = pe(Oe), ht = oe(Oe), Et = oe(Fe), Mt = De(Et), qt = nt / (ht + Ee * Ue * Mt), tn = qt * qt, xn = De(2 / (1 + tn)), An = (Ee * ht + (Ue + Be) * Mt) / (Ee * ht + (Ue - Be) * Mt), On = De(An), Bn = On - 1 / On, qn = On + 1 / On, Vn = xn * Bn - 2 * le(On) - $n, sr = xn * qt * qn - 2 * ee(qt) - zn, Un = nt && we * Mt * Be * tn / nt, wr = (Ee * Ue * ht + Mt) / (2 * (ht + Ee * Ue * Mt) * (ht + Ee * Ue * Mt) * Mt), Cr = -0.5 * qt * xn * xn * xn, Tr = Cr * Un, ii = Cr * wr, wi = (wi = 2 * ht + Ee * Mt * (Ue - Be)) * wi * On, Wi = (Ee * Ue * ht * Mt + Et) / wi, Yi = -Ee * Be * nt / (Mt * wi), ra = Bn * Tr - 2 * Wi / On + xn * (Wi + Wi / An), ha = Bn * ii - 2 * Yi / On + xn * (Yi + Yi / An), ya = qt * qn * Tr - 2 * Un / (1 + tn) + xn * qn * Un + xn * qt * (Wi - Wi / An), va = qt * qn * ii - 2 * wr / (1 + tn) + xn * qn * wr + xn * qt * (Yi - Yi / An), qa = ha * ya - va * ra;
                if (!qa) break;
                var Aa = (sr * ha - Vn * va) / qa, Xa = (Vn * ya - sr * ra) / qa;
                Ke -= Aa, Fe = ue(-Ce, de(Ce, Fe - Xa));
              } while ((ne(Aa) > ge || ne(Xa) > ge) && --Te > 0);
              return ne(ne(Fe) - Ce) < ge ? [0, Fe] : Te && [Ke, Fe];
            };
            var Yr = oe(35 * ke);
            function jr($n, zn) {
              var Gn = ye(zn / 2);
              return [$n * Yr * De(1 - Gn * Gn), (1 + Yr) * Gn];
            }
            function Mr() {
              return (0, te.A)(jr).scale(137.152);
            }
            function Wr($n, zn) {
              var Gn = zn / 2, Ke = oe(Gn);
              return [2 * $n / xe * oe(zn) * Ke * Ke, xe * ye(Gn)];
            }
            function Bi() {
              return (0, te.A)(Wr).scale(135.264);
            }
            function gi($n) {
              var zn = 1 - $n, Gn = Fe(be, 0)[0] - Fe(-be, 0)[0], Ke = De(2 * (Fe(0, Ce)[1] - Fe(0, -Ce)[1]) / Gn);
              function Fe(Oe, Be) {
                var Ue = oe(Be), nt = pe(Be);
                return [Ue / (zn + $n * Ue) * Oe, zn * Be + $n * nt];
              }
              function Te(Oe, Be) {
                var Ue = Fe(Oe, Be);
                return [Ue[0] * Ke, Ue[1] / Ke];
              }
              function Ae(Oe) {
                return Te(0, Oe)[1];
              }
              return Te.invert = function(Oe, Be) {
                var Ue = Rt(Ae, Be);
                return [Oe / Ke * ($n + zn / oe(Ue)), Ue];
              }, Te;
            }
            function Zr() {
              var $n = 0.5, zn = (0, te.U)(gi), Gn = zn($n);
              return Gn.alpha = function(Ke) {
                return arguments.length ? zn($n = +Ke) : $n;
              }, Gn.scale(168.725);
            }
            jr.invert = function($n, zn) {
              var Gn = zn / (1 + Yr);
              return [$n && $n / (Yr * De(1 - Gn * Gn)), 2 * ee(Gn)];
            }, Wr.invert = function($n, zn) {
              var Gn = ee(zn / xe), Ke = oe(Gn), Fe = 2 * Gn;
              return [$n * xe / 2 / (oe(Fe) * Ke * Ke), Fe];
            };
            var fi = Z(53253), ri = Z(18139);
            function bi($n) {
              return [$n[0] / 2, Me(ye($n[1] / 2 * ke)) * Qe];
            }
            function Zi($n) {
              return [2 * $n[0], 2 * ee(pe($n[1] * ke)) * Qe];
            }
            function $i($n) {
              $n == null && ($n = fi.A);
              var zn = $n(), Gn = (0, ri.A)().scale(Qe).precision(0).clipAngle(null).translate([0, 0]);
              function Ke(Te) {
                return zn(bi(Te));
              }
              function Fe(Te) {
                Ke[Te] = function() {
                  return arguments.length ? (zn[Te].apply(zn, arguments), Ke) : zn[Te]();
                };
              }
              return zn.invert && (Ke.invert = function(Te) {
                return Zi(zn.invert(Te));
              }), Ke.stream = function(Te) {
                var Ae = zn.stream(Te), Oe = Gn.stream({ point: function(Be, Ue) {
                  Ae.point(Be / 2, Me(ye(-Ue / 2 * ke)) * Qe);
                }, lineStart: function() {
                  Ae.lineStart();
                }, lineEnd: function() {
                  Ae.lineEnd();
                }, polygonStart: function() {
                  Ae.polygonStart();
                }, polygonEnd: function() {
                  Ae.polygonEnd();
                } });
                return Oe.sphere = Ae.sphere, Oe;
              }, Ke.rotate = function(Te) {
                return arguments.length ? (Gn.rotate(Te), Ke) : Gn.rotate();
              }, Ke.center = function(Te) {
                return arguments.length ? (zn.center(bi(Te)), Ke) : Zi(zn.center());
              }, Fe("angle"), Fe("clipAngle"), Fe("clipExtent"), Fe("fitExtent"), Fe("fitHeight"), Fe("fitSize"), Fe("fitWidth"), Fe("scale"), Fe("translate"), Fe("precision"), Ke.scale(249.5);
            }
            function Br($n, zn) {
              var Gn = 2 * be / zn, Ke = $n * $n;
              function Fe(Te, Ae) {
                var Oe = (0, it.j)(Te, Ae), Be = Oe[0], Ue = Oe[1], nt = Be * Be + Ue * Ue;
                if (nt > Ke) {
                  var ht = De(nt), Et = ae(Ue, Be), Mt = Gn * he(Et / Gn), qt = Et - Mt, tn = $n * oe(qt), xn = ($n * pe(qt) - qt * pe(tn)) / (Ce - tn), An = Pr(qt, xn), On = (be - $n) / ei(An, tn, be);
                  Be = ht;
                  var Bn, qn = 50;
                  do
                    Be -= Bn = ($n + ei(An, tn, Be) * On - ht) / (An(Be) * On);
                  while (ne(Bn) > ge && --qn > 0);
                  Ue = qt * pe(Be), Be < Ce && (Ue -= xn * (Be - Ce));
                  var Vn = pe(Mt), sr = oe(Mt);
                  Oe[0] = Be * sr - Ue * Vn, Oe[1] = Be * Vn + Ue * sr;
                }
                return Oe;
              }
              return Fe.invert = function(Te, Ae) {
                var Oe = Te * Te + Ae * Ae;
                if (Oe > Ke) {
                  var Be = De(Oe), Ue = ae(Ae, Te), nt = Gn * he(Ue / Gn), ht = Ue - nt;
                  Te = Be * oe(ht), Ae = Be * pe(ht);
                  for (var Et = Te - Ce, Mt = pe(Te), qt = Ae / Mt, tn = Te < Ce ? 1 / 0 : 0, xn = 10; ; ) {
                    var An = $n * pe(qt), On = $n * oe(qt), Bn = pe(On), qn = Ce - On, Vn = (An - qt * Bn) / qn, sr = Pr(qt, Vn);
                    if (ne(tn) < ve || !--xn) break;
                    qt -= tn = (qt * Mt - Vn * Et - Ae) / (Mt - 2 * Et * (qn * (On + qt * An * oe(On) - Bn) - An * (An - qt * Bn)) / (qn * qn));
                  }
                  Te = (Be = $n + ei(sr, On, Te) * (be - $n) / ei(sr, On, be)) * oe(Ue = nt + qt), Ae = Be * pe(Ue);
                }
                return it.j.invert(Te, Ae);
              }, Fe;
            }
            function Pr($n, zn) {
              return function(Gn) {
                var Ke = $n * oe(Gn);
                return Gn < Ce && (Ke -= zn), De(1 + Ke * Ke);
              };
            }
            function ei($n, zn, Gn) {
              for (var Ke = (Gn - zn) / 50, Fe = $n(zn) + $n(Gn), Te = 1, Ae = zn; Te < 50; ++Te) Fe += 2 * $n(Ae += Ke);
              return 0.5 * Fe * Ke;
            }
            function Ci() {
              var $n = 6, zn = 30 * ke, Gn = oe(zn), Ke = pe(zn), Fe = (0, te.U)(Br), Te = Fe(zn, $n), Ae = Te.stream, Oe = -oe(0.01 * ke), Be = pe(0.01 * ke);
              return Te.radius = function(Ue) {
                return arguments.length ? (Gn = oe(zn = Ue * ke), Ke = pe(zn), Fe(zn, $n)) : zn * Qe;
              }, Te.lobes = function(Ue) {
                return arguments.length ? Fe(zn, $n = +Ue) : $n;
              }, Te.stream = function(Ue) {
                var nt = Te.rotate(), ht = Ae(Ue), Et = (Te.rotate([0, 0]), Ae(Ue));
                return Te.rotate(nt), ht.sphere = function() {
                  Et.polygonStart(), Et.lineStart();
                  for (var Mt = 0, qt = 2 * be / $n, tn = 0; Mt < $n; ++Mt, tn -= qt) Et.point(ae(Be * oe(tn), Oe) * Qe, Me(Be * pe(tn)) * Qe), Et.point(ae(Ke * oe(tn - qt / 2), Gn) * Qe, Me(Ke * pe(tn - qt / 2)) * Qe);
                  Et.lineEnd(), Et.polygonEnd();
                }, ht;
              }, Te.rotate([90, -40]).scale(91.7095).clipAngle(179.999);
            }
            function oa($n, zn, Gn, Ke, Fe, Te, Ae, Oe) {
              function Be(Ue, nt) {
                if (!nt) return [$n * Ue / be, 0];
                var ht = nt * nt, Et = $n + ht * (zn + ht * (Gn + ht * Ke)), Mt = nt * (Fe - 1 + ht * (Te - Oe + ht * Ae)), qt = (Et * Et + Mt * Mt) / (2 * Mt), tn = Ue * Me(Et / qt) / be;
                return [qt * pe(tn), nt * (1 + ht * Oe) + qt * (1 - oe(tn))];
              }
              return arguments.length < 8 && (Oe = 0), Be.invert = function(Ue, nt) {
                var ht, Et, Mt = be * Ue / $n, qt = nt, tn = 50;
                do {
                  var xn = qt * qt, An = $n + xn * (zn + xn * (Gn + xn * Ke)), On = qt * (Fe - 1 + xn * (Te - Oe + xn * Ae)), Bn = An * An + On * On, qn = 2 * On, Vn = Bn / qn, sr = Vn * Vn, Un = Me(An / Vn) / be, wr = Mt * Un, Cr = An * An, Tr = (2 * zn + xn * (4 * Gn + 6 * xn * Ke)) * qt, ii = Fe + xn * (3 * Te + 5 * xn * Ae), wi = (2 * (An * Tr + On * (ii - 1)) * qn - Bn * (2 * (ii - 1))) / (qn * qn), Wi = oe(wr), Yi = pe(wr), ra = Vn * Wi, ha = Vn * Yi, ya = Mt / be * (1 / De(1 - Cr / sr)) * (Tr * Vn - An * wi) / sr, va = ha - Ue, qa = qt * (1 + xn * Oe) + Vn - ra - nt, Aa = wi * Yi + ra * ya, Xa = ra * Un, Oa = 1 + wi - (wi * Wi - ha * ya), Ya = ha * Un, Wo = Aa * Ya - Oa * Xa;
                  if (!Wo) break;
                  Mt -= ht = (qa * Aa - va * Oa) / Wo, qt -= Et = (va * Ya - qa * Xa) / Wo;
                } while ((ne(ht) > ge || ne(Et) > ge) && --tn > 0);
                return [Mt, qt];
              }, Be;
            }
            var ka = oa(2.8284, -1.6988, 0.75432, -0.18071, 1.76003, -0.38914, 0.042555);
            function aa() {
              return (0, te.A)(ka).scale(149.995);
            }
            var hi = oa(2.583819, -0.835827, 0.170354, -0.038094, 1.543313, -0.411435, 0.082742);
            function Gr() {
              return (0, te.A)(hi).scale(153.93);
            }
            var Ca = oa(5 / 6 * be, -0.62636, -0.0344, 0, 1.3493, -0.05524, 0, 0.045);
            function Sr() {
              return (0, te.A)(Ca).scale(130.945);
            }
            function Dr($n, zn) {
              var Gn = $n * $n, Ke = zn * zn;
              return [$n * (1 - 0.162388 * Ke) * (0.87 - 952426e-9 * Gn * Gn), zn * (1 + Ke / 12)];
            }
            function si() {
              return (0, te.A)(Dr).scale(131.747);
            }
            Dr.invert = function($n, zn) {
              var Gn, Ke = $n, Fe = zn, Te = 50;
              do {
                var Ae = Fe * Fe;
                Fe -= Gn = (Fe * (1 + Ae / 12) - zn) / (1 + Ae / 4);
              } while (ne(Gn) > ge && --Te > 0);
              Te = 50, $n /= 1 - 0.162388 * Ae;
              do {
                var Oe = (Oe = Ke * Ke) * Oe;
                Ke -= Gn = (Ke * (0.87 - 952426e-9 * Oe) - $n) / (0.87 - 476213e-8 * Oe);
              } while (ne(Gn) > ge && --Te > 0);
              return [Ke, Fe];
            };
            var zr = oa(2.6516, -0.76534, 0.19123, -0.047094, 1.36289, -0.13965, 0.031762);
            function di() {
              return (0, te.A)(zr).scale(131.087);
            }
            function Da($n) {
              var zn = $n(Ce, 0)[0] - $n(-Ce, 0)[0];
              function Gn(Ke, Fe) {
                var Te = Ke > 0 ? -0.5 : 0.5, Ae = $n(Ke + Te * be, Fe);
                return Ae[0] -= Te * zn, Ae;
              }
              return $n.invert && (Gn.invert = function(Ke, Fe) {
                var Te = Ke > 0 ? -0.5 : 0.5, Ae = $n.invert(Ke + Te * zn, Fe), Oe = Ae[0] - Te * be;
                return Oe < -be ? Oe += 2 * be : Oe > be && (Oe -= 2 * be), Ae[0] = Oe, Ae;
              }), Gn;
            }
            function la($n, zn) {
              var Gn = me($n), Ke = me(zn), Fe = oe(zn), Te = oe($n) * Fe, Ae = pe($n) * Fe, Oe = pe(Ke * zn);
              $n = ne(ae(Ae, Oe)), zn = Me(Te), ne($n - Ce) > ge && ($n %= Ce);
              var Be = function(Ue, nt) {
                if (nt === Ce) return [0, 0];
                var ht, Et, Mt = pe(nt), qt = Mt * Mt, tn = qt * qt, xn = 1 + tn, An = 1 + 3 * tn, On = 1 - tn, Bn = Me(1 / De(xn)), qn = On + qt * xn * Bn, Vn = (1 - Mt) / qn, sr = De(Vn), Un = Vn * xn, wr = De(Un), Cr = sr * On;
                if (Ue === 0) return [0, -(Cr + qt * wr)];
                var Tr, ii = oe(nt), wi = 1 / ii, Wi = 2 * Mt * ii, Yi = (-qn * ii - (1 - Mt) * ((-3 * qt + Bn * An) * Wi)) / (qn * qn), ra = -wi * Wi, ha = -wi * (qt * xn * Yi + Vn * An * Wi), ya = -2 * wi * (On * (0.5 * Yi / sr) - 2 * qt * sr * Wi), va = 4 * Ue / be;
                if (Ue > 0.222 * be || nt < be / 4 && Ue > 0.175 * be) {
                  if (ht = (Cr + qt * De(Un * (1 + tn) - Cr * Cr)) / (1 + tn), Ue > be / 4) return [ht, ht];
                  var qa = ht, Aa = 0.5 * ht;
                  ht = 0.5 * (Aa + qa), Et = 50;
                  do {
                    var Xa = ht * (ya + ra * De(Un - ht * ht)) + ha * Me(ht / wr) - va;
                    if (!Xa) break;
                    Xa < 0 ? Aa = ht : qa = ht, ht = 0.5 * (Aa + qa);
                  } while (ne(qa - Aa) > ge && --Et > 0);
                } else {
                  ht = ge, Et = 25;
                  do {
                    var Oa = ht * ht, Ya = De(Un - Oa), Wo = ya + ra * Ya, Ja = ht * Wo + ha * Me(ht / wr) - va;
                    ht -= Tr = Ya ? Ja / (Wo + (ha - ra * Oa) / Ya) : 0;
                  } while (ne(Tr) > ge && --Et > 0);
                }
                return [ht, -Cr - qt * De(Un - ht * ht)];
              }($n > be / 4 ? Ce - $n : $n, zn);
              return $n > be / 4 && (Oe = Be[0], Be[0] = -Be[1], Be[1] = -Oe), Be[0] *= Gn, Be[1] *= -Ke, Be;
            }
            function wa() {
              return (0, te.A)(Da(la)).scale(239.75);
            }
            function Wa($n, zn) {
              var Gn, Ke, Fe, Te, Ae, Oe;
              if (zn < ge) return [(Te = pe($n)) - (Gn = zn * ($n - Te * (Ke = oe($n))) / 4) * Ke, Ke + Gn * Te, 1 - zn * Te * Te / 2, $n - Gn];
              if (zn >= 1 - ge) return Gn = (1 - zn) / 4, Fe = 1 / (Ke = $e($n)), [(Te = ((Oe = se(2 * (Oe = $n))) - 1) / (Oe + 1)) + Gn * ((Ae = Ke * Pe($n)) - $n) / (Ke * Ke), Fe - Gn * Te * Fe * (Ae - $n), Fe + Gn * Te * Fe * (Ae + $n), 2 * ee(se($n)) - Ce + Gn * (Ae - $n) / Ke];
              var Be = [1, 0, 0, 0, 0, 0, 0, 0, 0], Ue = [De(zn), 0, 0, 0, 0, 0, 0, 0, 0], nt = 0;
              for (Ke = De(1 - zn), Ae = 1; ne(Ue[nt] / Be[nt]) > ge && nt < 8; ) Gn = Be[nt++], Ue[nt] = (Gn - Ke) / 2, Be[nt] = (Gn + Ke) / 2, Ke = De(Gn * Ke), Ae *= 2;
              Fe = Ae * Be[nt] * $n;
              do
                Fe = (Me(Te = Ue[nt] * pe(Ke = Fe) / Be[nt]) + Fe) / 2;
              while (--nt);
              return [pe(Fe), Te = oe(Fe), Te / oe(Fe - Ke), Fe];
            }
            function Qr($n, zn) {
              if (!zn) return $n;
              if (zn === 1) return le(ye($n / 2 + Se));
              for (var Gn = 1, Ke = De(1 - zn), Fe = De(zn), Te = 0; ne(Fe) > ge; Te++) {
                if ($n % be) {
                  var Ae = ee(Ke * ye($n) / Gn);
                  Ae < 0 && (Ae += be), $n += Ae + ~~($n / be) * be;
                } else $n += $n;
                Fe = (Gn + Ke) / 2, Ke = De(Gn * Ke), Fe = ((Gn = Fe) - Ke) / 2;
              }
              return $n / (fe(2, Te) * Gn);
            }
            function Ur($n, zn) {
              var Gn = (Ee - 1) / (Ee + 1), Ke = De(1 - Gn * Gn), Fe = Qr(Ce, Ke * Ke), Te = le(ye(be / 4 + ne(zn) / 2)), Ae = se(-1 * Te) / De(Gn), Oe = function(Ue, nt) {
                var ht = Ue * Ue, Et = nt + 1, Mt = 1 - ht - nt * nt;
                return [0.5 * ((Ue >= 0 ? Ce : -Ce) - ae(Mt, 2 * Ue)), -0.25 * le(Mt * Mt + 4 * ht) + 0.5 * le(Et * Et + ht)];
              }(Ae * oe(-1 * $n), Ae * pe(-1 * $n)), Be = function(Ue, nt, ht) {
                var Et = ne(Ue), Mt = Pe(ne(nt));
                if (Et) {
                  var qt = 1 / pe(Et), tn = 1 / (ye(Et) * ye(Et)), xn = -(tn + ht * (Mt * Mt * qt * qt) - 1 + ht), An = (-xn + De(xn * xn - (ht - 1) * tn * 4)) / 2;
                  return [Qr(ee(1 / De(An)), ht) * me(Ue), Qr(ee(De((An / tn - 1) / ht)), 1 - ht) * me(nt)];
                }
                return [0, Qr(ee(Mt), 1 - ht) * me(nt)];
              }(Oe[0], Oe[1], Ke * Ke);
              return [-Be[1], (zn >= 0 ? 1 : -1) * (0.5 * Fe - Be[0])];
            }
            function rr() {
              return (0, te.A)(Da(Ur)).scale(151.496);
            }
            la.invert = function($n, zn) {
              ne($n) > 1 && ($n = 2 * me($n) - $n), ne(zn) > 1 && (zn = 2 * me(zn) - zn);
              var Gn = me($n), Ke = me(zn), Fe = -Gn * $n, Te = -Ke * zn, Ae = Te / Fe < 1, Oe = function(ht, Et) {
                for (var Mt = 0, qt = 1, tn = 0.5, xn = 50; ; ) {
                  var An = tn * tn, On = De(tn), Bn = Me(1 / De(1 + An)), qn = 1 - An + tn * (1 + An) * Bn, Vn = (1 - On) / qn, sr = De(Vn), Un = Vn * (1 + An), wr = sr * (1 - An), Cr = De(Un - ht * ht), Tr = Et + wr + tn * Cr;
                  if (ne(qt - Mt) < ve || --xn == 0 || Tr === 0) break;
                  Tr > 0 ? Mt = tn : qt = tn, tn = 0.5 * (Mt + qt);
                }
                if (!xn) return null;
                var ii = Me(On), wi = oe(ii), Wi = 1 / wi, Yi = 2 * On * wi, ra = (-qn * wi - (-3 * tn + Bn * (1 + 3 * An)) * Yi * (1 - On)) / (qn * qn);
                return [be / 4 * (ht * (-2 * Wi * ((1 - An) * (0.5 * ra / sr) - 2 * tn * sr * Yi) + -Wi * Yi * Cr) + -Wi * (tn * (1 + An) * ra + Vn * (1 + 3 * An) * Yi) * Me(ht / De(Un))), ii];
              }(Ae ? Te : Fe, Ae ? Fe : Te), Be = Oe[0], Ue = Oe[1], nt = oe(Ue);
              return Ae && (Be = -Ce - Be), [Gn * (ae(pe(Be) * nt, -pe(Ue)) + be), Ke * Me(oe(Be) * nt)];
            }, Ur.invert = function($n, zn) {
              var Gn, Ke, Fe, Te, Ae, Oe, Be = (Ee - 1) / (Ee + 1), Ue = De(1 - Be * Be), nt = (Ke = -$n, Fe = Ue * Ue, (Gn = 0.5 * Qr(Ce, Ue * Ue) - zn) ? (Te = Wa(Gn, Fe), Ke ? (Oe = (Ae = Wa(Ke, 1 - Fe))[1] * Ae[1] + Fe * Te[0] * Te[0] * Ae[0] * Ae[0], [[Te[0] * Ae[2] / Oe, Te[1] * Te[2] * Ae[0] * Ae[1] / Oe], [Te[1] * Ae[1] / Oe, -Te[0] * Te[2] * Ae[0] * Ae[2] / Oe], [Te[2] * Ae[1] * Ae[2] / Oe, -Fe * Te[0] * Te[1] * Ae[0] / Oe]]) : [[Te[0], 0], [Te[1], 0], [Te[2], 0]]) : [[0, (Ae = Wa(Ke, 1 - Fe))[0] / Ae[1]], [1 / Ae[1], 0], [Ae[2] / Ae[1], 0]]), ht = function(Et, Mt) {
                var qt = Mt[0] * Mt[0] + Mt[1] * Mt[1];
                return [(Et[0] * Mt[0] + Et[1] * Mt[1]) / qt, (Et[1] * Mt[0] - Et[0] * Mt[1]) / qt];
              }(nt[0], nt[1]);
              return [ae(ht[1], ht[0]) / -1, 2 * ee(se(-0.5 * le(Be * ht[0] * ht[0] + Be * ht[1] * ht[1]))) - Ce];
            };
            var vr = Z(39127);
            function Rr($n) {
              var zn = pe($n), Gn = oe($n), Ke = Vr($n);
              function Fe(Te, Ae) {
                var Oe = Ke(Te, Ae);
                Te = Oe[0], Ae = Oe[1];
                var Be = pe(Ae), Ue = oe(Ae), nt = oe(Te), ht = Re(zn * Be + Gn * Ue * nt), Et = pe(ht), Mt = ne(Et) > ge ? ht / Et : 1;
                return [Mt * Gn * pe(Te), (ne(Te) > Ce ? Mt : -Mt) * (zn * Ue - Gn * Be * nt)];
              }
              return Ke.invert = Vr(-$n), Fe.invert = function(Te, Ae) {
                var Oe = De(Te * Te + Ae * Ae), Be = -pe(Oe), Ue = oe(Oe), nt = Oe * Ue, ht = -Ae * Be, Et = Oe * zn, Mt = De(nt * nt + ht * ht - Et * Et), qt = ae(nt * Et + ht * Mt, ht * Et - nt * Mt), tn = (Oe > Ce ? -1 : 1) * ae(Te * Be, Oe * oe(qt) * Ue + Ae * pe(qt) * Be);
                return Ke.invert(tn, qt);
              }, Fe;
            }
            function Vr($n) {
              var zn = pe($n), Gn = oe($n);
              return function(Ke, Fe) {
                var Te = oe(Fe), Ae = oe(Ke) * Te, Oe = pe(Ke) * Te, Be = pe(Fe);
                return [ae(Oe, Ae * Gn - Be * zn), Me(Be * Gn + Ae * zn)];
              };
            }
            function Xr() {
              var $n = 0, zn = (0, te.U)(Rr), Gn = zn($n), Ke = Gn.rotate, Fe = Gn.stream, Te = (0, vr.A)();
              return Gn.parallel = function(Ae) {
                if (!arguments.length) return $n * Qe;
                var Oe = Gn.rotate();
                return zn($n = Ae * ke).rotate(Oe);
              }, Gn.rotate = function(Ae) {
                return arguments.length ? (Ke.call(Gn, [Ae[0], Ae[1] - $n * Qe]), Te.center([-Ae[0], -Ae[1]]), Gn) : ((Ae = Ke.call(Gn))[1] += $n * Qe, Ae);
              }, Gn.stream = function(Ae) {
                return (Ae = Fe(Ae)).sphere = function() {
                  Ae.polygonStart();
                  var Oe, Be = Te.radius(89.99)().coordinates[0], Ue = Be.length - 1, nt = -1;
                  for (Ae.lineStart(); ++nt < Ue; ) Ae.point((Oe = Be[nt])[0], Oe[1]);
                  for (Ae.lineEnd(), Ue = (Be = Te.radius(90.01)().coordinates[0]).length - 1, Ae.lineStart(); --nt >= 0; ) Ae.point((Oe = Be[nt])[0], Oe[1]);
                  Ae.lineEnd(), Ae.polygonEnd();
                }, Ae;
              }, Gn.scale(79.4187).parallel(45).clipAngle(179.999);
            }
            var Jr = Z(29725), ti = Z(20465), Hr = Me(1 - 1 / 3) * Qe, Er = un(0);
            function gr($n) {
              var zn = Hr * ke, Gn = ft(be, zn)[0] - ft(-be, zn)[0], Ke = Er(0, zn)[1], Fe = ft(0, zn)[1], Te = xe - Fe, Ae = Le / $n, Oe = 4 / Le, Be = Ke + Te * Te * 4 / Le;
              function Ue(nt, ht) {
                var Et, Mt = ne(ht);
                if (Mt > zn) {
                  var qt = de($n - 1, ue(0, ce((nt + be) / Ae)));
                  (Et = ft(nt += be * ($n - 1) / $n - qt * Ae, Mt))[0] = Et[0] * Le / Gn - Le * ($n - 1) / (2 * $n) + qt * Le / $n, Et[1] = Ke + 4 * (Et[1] - Fe) * Te / Le, ht < 0 && (Et[1] = -Et[1]);
                } else Et = Er(nt, ht);
                return Et[0] *= Oe, Et[1] /= Be, Et;
              }
              return Ue.invert = function(nt, ht) {
                nt /= Oe;
                var Et = ne(ht *= Be);
                if (Et > Ke) {
                  var Mt = de($n - 1, ue(0, ce((nt + be) / Ae)));
                  nt = (nt + be * ($n - 1) / $n - Mt * Ae) * Gn / Le;
                  var qt = ft.invert(nt, 0.25 * (Et - Ke) * Le / Te + Fe);
                  return qt[0] -= be * ($n - 1) / $n - Mt * Ae, ht < 0 && (qt[1] = -qt[1]), qt;
                }
                return Er.invert(nt, ht);
              }, Ue;
            }
            function $r($n, zn) {
              return [$n, 1 & zn ? 90 - ge : Hr];
            }
            function li($n, zn) {
              return [$n, 1 & zn ? -90 + ge : -Hr];
            }
            function Qi($n) {
              return [$n[0] * (1 - ge), $n[1]];
            }
            function Ai() {
              var $n = 4, zn = (0, te.U)(gr), Gn = zn($n), Ke = Gn.stream;
              return Gn.lobes = function(Fe) {
                return arguments.length ? zn($n = +Fe) : $n;
              }, Gn.stream = function(Fe) {
                var Te = Gn.rotate(), Ae = Ke(Fe), Oe = (Gn.rotate([0, 0]), Ke(Fe));
                return Gn.rotate(Te), Ae.sphere = function() {
                  var Be, Ue;
                  (0, ti.A)((Be = 180 / $n, Ue = [].concat((0, Jr.y1)(-180, 180 + Be / 2, Be).map($r), (0, Jr.y1)(180, -180 - Be / 2, -Be).map(li)), { type: "Polygon", coordinates: [Be === 180 ? Ue.map(Qi) : Ue] }), Oe);
                }, Ae;
              }, Gn.scale(239.75);
            }
            function Li($n) {
              var zn, Gn = 1 + $n, Ke = Me(pe(1 / Gn)), Fe = 2 * De(be / (zn = be + 4 * Ke * Gn)), Te = 0.5 * Fe * (Gn + De($n * (2 + $n))), Ae = $n * $n, Oe = Gn * Gn;
              function Be(Ue, nt) {
                var ht, Et, Mt = 1 - pe(nt);
                if (Mt && Mt < 2) {
                  var qt, tn = Ce - nt, xn = 25;
                  do {
                    var An = pe(tn), On = oe(tn), Bn = Ke + ae(An, Gn - On), qn = 1 + Oe - 2 * Gn * On;
                    tn -= qt = (tn - Ae * Ke - Gn * An + qn * Bn - 0.5 * Mt * zn) / (2 * Gn * An * Bn);
                  } while (ne(qt) > ve && --xn > 0);
                  ht = Fe * De(qn), Et = Ue * Bn / be;
                } else ht = Fe * ($n + Mt), Et = Ue * Ke / be;
                return [ht * pe(Et), Te - ht * oe(Et)];
              }
              return Be.invert = function(Ue, nt) {
                var ht = Ue * Ue + (nt -= Te) * nt, Et = (1 + Oe - ht / (Fe * Fe)) / (2 * Gn), Mt = Re(Et), qt = pe(Mt), tn = Ke + ae(qt, Gn - Et);
                return [Me(Ue / De(ht)) * be / tn, Me(1 - 2 * (Mt - Ae * Ke - Gn * qt + (1 + Oe - 2 * Gn * Et) * tn) / zn)];
              }, Be;
            }
            function Ni() {
              var $n = 1, zn = (0, te.U)(Li), Gn = zn($n);
              return Gn.ratio = function(Ke) {
                return arguments.length ? zn($n = +Ke) : $n;
              }, Gn.scale(167.774).center([0, 18.67]);
            }
            var Pi = 0.7109889596207567, Oi = 0.0528035274542;
            function ki($n, zn) {
              return zn > -0.7109889596207567 ? (($n = Gt($n, zn))[1] += Oi, $n) : Yt($n, zn);
            }
            function Mi() {
              return (0, te.A)(ki).rotate([-20, -55]).scale(164.263).center([0, -5.4036]);
            }
            function Vi($n, zn) {
              return ne(zn) > Pi ? (($n = Gt($n, zn))[1] -= zn > 0 ? Oi : -0.0528035274542, $n) : Yt($n, zn);
            }
            function ua() {
              return (0, te.A)(Vi).scale(152.63);
            }
            function Ki($n, zn, Gn, Ke) {
              var Fe = De(4 * be / (2 * Gn + (1 + $n - zn / 2) * pe(2 * Gn) + ($n + zn) / 2 * pe(4 * Gn) + zn / 2 * pe(6 * Gn))), Te = De(Ke * pe(Gn) * De((1 + $n * oe(2 * Gn) + zn * oe(4 * Gn)) / (1 + $n + zn))), Ae = Gn * Be(1);
              function Oe(ht) {
                return De(1 + $n * oe(2 * ht) + zn * oe(4 * ht));
              }
              function Be(ht) {
                var Et = ht * Gn;
                return (2 * Et + (1 + $n - zn / 2) * pe(2 * Et) + ($n + zn) / 2 * pe(4 * Et) + zn / 2 * pe(6 * Et)) / Gn;
              }
              function Ue(ht) {
                return Oe(ht) * pe(ht);
              }
              var nt = function(ht, Et) {
                var Mt = Gn * Rt(Be, Ae * pe(Et) / Gn, Et / be);
                isNaN(Mt) && (Mt = Gn * me(Et));
                var qt = Fe * Oe(Mt);
                return [qt * Te * ht / be * oe(Mt), qt / Te * pe(Mt)];
              };
              return nt.invert = function(ht, Et) {
                var Mt = Rt(Ue, Et * Te / Fe);
                return [ht * be / (oe(Mt) * Fe * Te * Oe(Mt)), Me(Gn * Be(Mt / Gn) / Ae)];
              }, Gn === 0 && (Fe = De(Ke / be), (nt = function(ht, Et) {
                return [ht * Fe, pe(Et) / Fe];
              }).invert = function(ht, Et) {
                return [ht / Fe, Me(Et * Fe)];
              }), nt;
            }
            function Fi() {
              var $n = 1, zn = 0, Gn = 45 * ke, Ke = 2, Fe = (0, te.U)(Ki), Te = Fe($n, zn, Gn, Ke);
              return Te.a = function(Ae) {
                return arguments.length ? Fe($n = +Ae, zn, Gn, Ke) : $n;
              }, Te.b = function(Ae) {
                return arguments.length ? Fe($n, zn = +Ae, Gn, Ke) : zn;
              }, Te.psiMax = function(Ae) {
                return arguments.length ? Fe($n, zn, Gn = +Ae * ke, Ke) : Gn * Qe;
              }, Te.ratio = function(Ae) {
                return arguments.length ? Fe($n, zn, Gn, Ke = +Ae) : Ke;
              }, Te.scale(180.739);
            }
            function na($n, zn, Gn, Ke, Fe, Te, Ae, Oe, Be, Ue, nt) {
              if (nt.nanEncountered) return NaN;
              var ht, Et, Mt, qt, tn, xn, An, On, Bn, qn;
              if (Et = $n(zn + 0.25 * (ht = Gn - zn)), Mt = $n(Gn - 0.25 * ht), isNaN(Et)) nt.nanEncountered = !0;
              else {
                if (!isNaN(Mt)) return qn = ((xn = (qt = ht * (Ke + 4 * Et + Fe) / 12) + (tn = ht * (Fe + 4 * Mt + Te) / 12)) - Ae) / 15, Ue > Be ? (nt.maxDepthCount++, xn + qn) : Math.abs(qn) < Oe ? xn + qn : (On = na($n, zn, An = zn + 0.5 * ht, Ke, Et, Fe, qt, 0.5 * Oe, Be, Ue + 1, nt), isNaN(On) ? (nt.nanEncountered = !0, NaN) : (Bn = na($n, An, Gn, Fe, Mt, Te, tn, 0.5 * Oe, Be, Ue + 1, nt), isNaN(Bn) ? (nt.nanEncountered = !0, NaN) : On + Bn));
                nt.nanEncountered = !0;
              }
            }
            function nn($n, zn, Gn, Ke, Fe) {
              Fe === void 0 && (Fe = 20);
              var Te = $n(zn), Ae = $n(0.5 * (zn + Gn)), Oe = $n(Gn);
              return na($n, zn, Gn, Te, Ae, Oe, (Te + 4 * Ae + Oe) * (Gn - zn) / 6, Ke, Fe, 1, { maxDepthCount: 0, nanEncountered: !1 });
            }
            function Tn($n, zn, Gn) {
              function Ke(Mt) {
                return $n + (1 - $n) * fe(1 - fe(Mt, zn), 1 / zn);
              }
              function Fe(Mt) {
                return nn(Ke, 0, Mt, 1e-4);
              }
              for (var Te = 1 / Fe(1), Ae = 1e3, Oe = 1.00000001 * Te, Be = [], Ue = 0; Ue <= Ae; Ue++) Be.push(Fe(Ue / Ae) * Oe);
              function nt(Mt) {
                var qt = 0, tn = Ae, xn = 500;
                do
                  Be[xn] > Mt ? tn = xn : qt = xn, xn = qt + tn >> 1;
                while (xn > qt);
                var An = Be[xn + 1] - Be[xn];
                return An && (An = (Mt - Be[xn + 1]) / An), (xn + 1 + An) / Ae;
              }
              var ht = 2 * nt(1) / be * Te / Gn, Et = function(Mt, qt) {
                var tn = nt(ne(pe(qt))), xn = Ke(tn) * Mt;
                return tn /= ht, [xn, qt >= 0 ? tn : -tn];
              };
              return Et.invert = function(Mt, qt) {
                var tn;
                return ne(qt *= ht) < 1 && (tn = me(qt) * Me(Fe(ne(qt)) * Te)), [Mt / Ke(ne(qt)), tn];
              }, Et;
            }
            function Dn() {
              var $n = 0, zn = 2.5, Gn = 1.183136, Ke = (0, te.U)(Tn), Fe = Ke($n, zn, Gn);
              return Fe.alpha = function(Te) {
                return arguments.length ? Ke($n = +Te, zn, Gn) : $n;
              }, Fe.k = function(Te) {
                return arguments.length ? Ke($n, zn = +Te, Gn) : zn;
              }, Fe.gamma = function(Te) {
                return arguments.length ? Ke($n, zn, Gn = +Te) : Gn;
              }, Fe.scale(152.63);
            }
            function Fn($n, zn) {
              return ne($n[0] - zn[0]) < ge && ne($n[1] - zn[1]) < ge;
            }
            function fr($n, zn) {
              for (var Gn, Ke, Fe, Te = -1, Ae = $n.length, Oe = $n[0], Be = []; ++Te < Ae; ) {
                Ke = ((Gn = $n[Te])[0] - Oe[0]) / zn, Fe = (Gn[1] - Oe[1]) / zn;
                for (var Ue = 0; Ue < zn; ++Ue) Be.push([Oe[0] + Ue * Ke, Oe[1] + Ue * Fe]);
                Oe = Gn;
              }
              return Be.push(Gn), Be;
            }
            function dr($n, zn, Gn) {
              var Ke, Fe;
              function Te(Be, Ue) {
                for (var nt = Ue < 0 ? -1 : 1, ht = zn[+(Ue < 0)], Et = 0, Mt = ht.length - 1; Et < Mt && Be > ht[Et][2][0]; ++Et) ;
                var qt = $n(Be - ht[Et][1][0], Ue);
                return qt[0] += $n(ht[Et][1][0], nt * Ue > nt * ht[Et][0][1] ? ht[Et][0][1] : Ue)[0], qt;
              }
              Gn ? Te.invert = Gn(Te) : $n.invert && (Te.invert = function(Be, Ue) {
                for (var nt = Fe[+(Ue < 0)], ht = zn[+(Ue < 0)], Et = 0, Mt = nt.length; Et < Mt; ++Et) {
                  var qt = nt[Et];
                  if (qt[0][0] <= Be && Be < qt[1][0] && qt[0][1] <= Ue && Ue < qt[1][1]) {
                    var tn = $n.invert(Be - $n(ht[Et][1][0], 0)[0], Ue);
                    return tn[0] += ht[Et][1][0], Fn(Te(tn[0], tn[1]), [Be, Ue]) ? tn : null;
                  }
                }
              });
              var Ae = (0, te.A)(Te), Oe = Ae.stream;
              return Ae.stream = function(Be) {
                var Ue = Ae.rotate(), nt = Oe(Be), ht = (Ae.rotate([0, 0]), Oe(Be));
                return Ae.rotate(Ue), nt.sphere = function() {
                  (0, ti.A)(Ke, ht);
                }, nt;
              }, Ae.lobes = function(Be) {
                return arguments.length ? (Ke = function(Ue) {
                  var nt, ht, Et, Mt, qt, tn, xn, An = [], On = Ue[0].length;
                  for (xn = 0; xn < On; ++xn) ht = (nt = Ue[0][xn])[0][0], Et = nt[0][1], Mt = nt[1][1], qt = nt[2][0], tn = nt[2][1], An.push(fr([[ht + ge, Et + ge], [ht + ge, Mt - ge], [qt - ge, Mt - ge], [qt - ge, tn + ge]], 30));
                  for (xn = Ue[1].length - 1; xn >= 0; --xn) ht = (nt = Ue[1][xn])[0][0], Et = nt[0][1], Mt = nt[1][1], qt = nt[2][0], tn = nt[2][1], An.push(fr([[qt - ge, tn - ge], [qt - ge, Mt + ge], [ht + ge, Mt + ge], [ht + ge, Et - ge]], 30));
                  return { type: "Polygon", coordinates: [(0, Jr.Am)(An)] };
                }(Be), zn = Be.map(function(Ue) {
                  return Ue.map(function(nt) {
                    return [[nt[0][0] * ke, nt[0][1] * ke], [nt[1][0] * ke, nt[1][1] * ke], [nt[2][0] * ke, nt[2][1] * ke]];
                  });
                }), Fe = zn.map(function(Ue) {
                  return Ue.map(function(nt) {
                    var ht, Et = $n(nt[0][0], nt[0][1])[0], Mt = $n(nt[2][0], nt[2][1])[0], qt = $n(nt[1][0], nt[0][1])[1], tn = $n(nt[1][0], nt[1][1])[1];
                    return qt > tn && (ht = qt, qt = tn, tn = ht), [[Et, qt], [Mt, tn]];
                  });
                }), Ae) : zn.map(function(Ue) {
                  return Ue.map(function(nt) {
                    return [[nt[0][0] * Qe, nt[0][1] * Qe], [nt[1][0] * Qe, nt[1][1] * Qe], [nt[2][0] * Qe, nt[2][1] * Qe]];
                  });
                });
              }, zn != null && Ae.lobes(zn), Ae;
            }
            ki.invert = function($n, zn) {
              return zn > -0.7109889596207567 ? Gt.invert($n, zn - Oi) : Yt.invert($n, zn);
            }, Vi.invert = function($n, zn) {
              return ne(zn) > Pi ? Gt.invert($n, zn + (zn > 0 ? Oi : -0.0528035274542)) : Yt.invert($n, zn);
            };
            var yr = [[[[-180, 0], [-100, 90], [-40, 0]], [[-40, 0], [30, 90], [180, 0]]], [[[-180, 0], [-160, -90], [-100, 0]], [[-100, 0], [-60, -90], [-20, 0]], [[-20, 0], [20, -90], [80, 0]], [[80, 0], [140, -90], [180, 0]]]];
            function ai() {
              return dr(Ct, yr).scale(160.857);
            }
            var Fr = [[[[-180, 0], [-100, 90], [-40, 0]], [[-40, 0], [30, 90], [180, 0]]], [[[-180, 0], [-160, -90], [-100, 0]], [[-100, 0], [-60, -90], [-20, 0]], [[-20, 0], [20, -90], [80, 0]], [[80, 0], [140, -90], [180, 0]]]];
            function Ti() {
              return dr(Vi, Fr).scale(152.63);
            }
            var Si = [[[[-180, 0], [-100, 90], [-40, 0]], [[-40, 0], [30, 90], [180, 0]]], [[[-180, 0], [-160, -90], [-100, 0]], [[-100, 0], [-60, -90], [-20, 0]], [[-20, 0], [20, -90], [80, 0]], [[80, 0], [140, -90], [180, 0]]]];
            function Ei() {
              return dr(Gt, Si).scale(169.529);
            }
            var Fa = [[[[-180, 0], [-90, 90], [0, 0]], [[0, 0], [90, 90], [180, 0]]], [[[-180, 0], [-90, -90], [0, 0]], [[0, 0], [90, -90], [180, 0]]]];
            function $o() {
              return dr(Gt, Fa).scale(169.529).rotate([20, 0]);
            }
            var Gs = [[[[-180, 35], [-30, 90], [0, 35]], [[0, 35], [30, 90], [180, 35]]], [[[-180, -10], [-102, -90], [-65, -10]], [[-65, -10], [5, -90], [77, -10]], [[77, -10], [103, -90], [180, -10]]]];
            function Al() {
              return dr(ki, Gs, vt).rotate([-20, -55]).scale(164.263).center([0, -5.4036]);
            }
            var ul = [[[[-180, 0], [-110, 90], [-40, 0]], [[-40, 0], [0, 90], [40, 0]], [[40, 0], [110, 90], [180, 0]]], [[[-180, 0], [-110, -90], [-40, 0]], [[-40, 0], [0, -90], [40, 0]], [[40, 0], [110, -90], [180, 0]]]];
            function gc() {
              return dr(Yt, ul).scale(152.63).rotate([-20, 0]);
            }
            function Gl($n, zn) {
              return [3 / Le * $n * De(be * be / 3 - zn * zn), zn];
            }
            function Sl() {
              return (0, te.A)(Gl).scale(158.837);
            }
            function ls($n) {
              function zn(Gn, Ke) {
                if (ne(ne(Ke) - Ce) < ge) return [0, Ke < 0 ? -2 : 2];
                var Fe = pe(Ke), Te = fe((1 + Fe) / (1 - Fe), $n / 2), Ae = 0.5 * (Te + 1 / Te) + oe(Gn *= $n);
                return [2 * pe(Gn) / Ae, (Te - 1 / Te) / Ae];
              }
              return zn.invert = function(Gn, Ke) {
                var Fe = ne(Ke);
                if (ne(Fe - 2) < ge) return Gn ? null : [0, me(Ke) * Ce];
                if (Fe > 2) return null;
                var Te = (Gn /= 2) * Gn, Ae = (Ke /= 2) * Ke, Oe = 2 * Ke / (1 + Te + Ae);
                return Oe = fe((1 + Oe) / (1 - Oe), 1 / $n), [ae(2 * Gn, 1 - Te - Ae) / $n, Me((Oe - 1) / (Oe + 1))];
              }, zn;
            }
            function vs() {
              var $n = 0.5, zn = (0, te.U)(ls), Gn = zn($n);
              return Gn.spacing = function(Ke) {
                return arguments.length ? zn($n = +Ke) : $n;
              }, Gn.scale(124.75);
            }
            Gl.invert = function($n, zn) {
              return [Le / 3 * $n / De(be * be / 3 - zn * zn), zn];
            };
            var Vl = be / Ee;
            function Ll($n, zn) {
              return [$n * (1 + De(oe(zn))) / 2, zn / (oe(zn / 2) * oe($n / 6))];
            }
            function Xl() {
              return (0, te.A)(Ll).scale(97.2672);
            }
            function yc($n, zn) {
              var Gn = $n * $n, Ke = zn * zn;
              return [$n * (0.975534 + Ke * (-0.0143059 * Gn - 0.119161 + -0.0547009 * Ke)), zn * (1.00384 + Gn * (0.0802894 + -0.02855 * Ke + 199025e-9 * Gn) + Ke * (0.0998909 + -0.0491032 * Ke))];
            }
            function gu() {
              return (0, te.A)(yc).scale(139.98);
            }
            function Nc($n, zn) {
              return [pe($n) / oe(zn), ye(zn) * oe($n)];
            }
            function Cu() {
              return (0, te.A)(Nc).scale(144.049).clipAngle(89.999);
            }
            function zc($n) {
              var zn = oe($n), Gn = ye(Se + $n / 2);
              function Ke(Fe, Te) {
                var Ae = Te - $n, Oe = ne(Ae) < ge ? Fe * zn : ne(Oe = Se + Te / 2) < ge || ne(ne(Oe) - Ce) < ge ? 0 : Fe * Ae / le(ye(Oe) / Gn);
                return [Oe, Ae];
              }
              return Ke.invert = function(Fe, Te) {
                var Ae, Oe = Te + $n;
                return [ne(Te) < ge ? Fe / zn : ne(Ae = Se + Oe / 2) < ge || ne(ne(Ae) - Ce) < ge ? 0 : Fe * le(ye(Ae) / Gn) / Te, Oe];
              }, Ke;
            }
            function Kc() {
              return Vt(zc).parallel(40).scale(158.837);
            }
            function zi($n, zn) {
              return [$n, 1.25 * le(ye(Se + 0.4 * zn))];
            }
            function Fu() {
              return (0, te.A)(zi).scale(108.318);
            }
            function d1($n) {
              var zn = $n.length - 1;
              function Gn(Ke, Fe) {
                for (var Te, Ae = oe(Fe), Oe = 2 / (1 + Ae * oe(Ke)), Be = Oe * Ae * pe(Ke), Ue = Oe * pe(Fe), nt = zn, ht = $n[nt], Et = ht[0], Mt = ht[1]; --nt >= 0; ) Et = (ht = $n[nt])[0] + Be * (Te = Et) - Ue * Mt, Mt = ht[1] + Be * Mt + Ue * Te;
                return [Et = Be * (Te = Et) - Ue * Mt, Mt = Be * Mt + Ue * Te];
              }
              return Gn.invert = function(Ke, Fe) {
                var Te = 20, Ae = Ke, Oe = Fe;
                do {
                  for (var Be, Ue = zn, nt = $n[Ue], ht = nt[0], Et = nt[1], Mt = 0, qt = 0; --Ue >= 0; ) Mt = ht + Ae * (Be = Mt) - Oe * qt, qt = Et + Ae * qt + Oe * Be, ht = (nt = $n[Ue])[0] + Ae * (Be = ht) - Oe * Et, Et = nt[1] + Ae * Et + Oe * Be;
                  var tn, xn, An = (Mt = ht + Ae * (Be = Mt) - Oe * qt) * Mt + (qt = Et + Ae * qt + Oe * Be) * qt;
                  Ae -= tn = ((ht = Ae * (Be = ht) - Oe * Et - Ke) * Mt + (Et = Ae * Et + Oe * Be - Fe) * qt) / An, Oe -= xn = (Et * Mt - ht * qt) / An;
                } while (ne(tn) + ne(xn) > ge * ge && --Te > 0);
                if (Te) {
                  var On = De(Ae * Ae + Oe * Oe), Bn = 2 * ee(0.5 * On), qn = pe(Bn);
                  return [ae(Ae * qn, On * oe(Bn)), On ? Me(Oe * qn / On) : 0];
                }
              }, Gn;
            }
            Ll.invert = function($n, zn) {
              var Gn = ne($n), Ke = ne(zn), Fe = ge, Te = Ce;
              Ke < Vl ? Te *= Ke / Vl : Fe += 6 * Re(Vl / Ke);
              for (var Ae = 0; Ae < 25; Ae++) {
                var Oe = pe(Te), Be = De(oe(Te)), Ue = pe(Te / 2), nt = oe(Te / 2), ht = pe(Fe / 6), Et = oe(Fe / 6), Mt = 0.5 * Fe * (1 + Be) - Gn, qt = Te / (nt * Et) - Ke, tn = Be ? -0.25 * Fe * Oe / Be : 0, xn = 0.5 * (1 + Be), An = (1 + 0.5 * Te * Ue / nt) / (nt * Et), On = Te / nt * (ht / 6) / (Et * Et), Bn = tn * On - An * xn, qn = (Mt * On - qt * xn) / Bn, Vn = (qt * tn - Mt * An) / Bn;
                if (Te -= qn, Fe -= Vn, ne(qn) < ge && ne(Vn) < ge) break;
              }
              return [$n < 0 ? -Fe : Fe, zn < 0 ? -Te : Te];
            }, yc.invert = function($n, zn) {
              var Gn = me($n) * be, Ke = zn / 2, Fe = 50;
              do {
                var Te = Gn * Gn, Ae = Ke * Ke, Oe = Gn * Ke, Be = Gn * (0.975534 + Ae * (-0.0143059 * Te - 0.119161 + -0.0547009 * Ae)) - $n, Ue = Ke * (1.00384 + Te * (0.0802894 + -0.02855 * Ae + 199025e-9 * Te) + Ae * (0.0998909 + -0.0491032 * Ae)) - zn, nt = 0.975534 - Ae * (0.119161 + 3 * Te * 0.0143059 + 0.0547009 * Ae), ht = -Oe * (0.238322 + 0.2188036 * Ae + 0.0286118 * Te), Et = Oe * (0.1605788 + 7961e-7 * Te + -0.0571 * Ae), Mt = 1.00384 + Te * (0.0802894 + 199025e-9 * Te) + Ae * (3 * (0.0998909 - 0.02855 * Te) - 0.245516 * Ae), qt = ht * Et - Mt * nt, tn = (Ue * ht - Be * Mt) / qt, xn = (Be * Et - Ue * nt) / qt;
                Gn -= tn, Ke -= xn;
              } while ((ne(tn) > ge || ne(xn) > ge) && --Fe > 0);
              return Fe && [Gn, Ke];
            }, Nc.invert = function($n, zn) {
              var Gn = $n * $n, Ke = zn * zn + 1, Fe = Gn + Ke, Te = $n ? we * De((Fe - De(Fe * Fe - 4 * Gn)) / Gn) : 1 / De(Ke);
              return [Me($n * Te), me(zn) * Re(Te)];
            }, zi.invert = function($n, zn) {
              return [$n, 2.5 * ee(se(0.8 * zn)) - 0.625 * be];
            };
            var Gu = [[0.9972523, 0], [52513e-7, -41175e-7], [74606e-7, 48125e-7], [-0.0153783, -0.1968253], [0.0636871, -0.1408027], [0.3660976, -0.2937382]], Ku = [[0.98879, 0], [0, 0], [-0.050909, 0], [0, 0], [0.075528, 0]], I1 = [[0.984299, 0], [0.0211642, 37608e-7], [-0.1036018, -0.0575102], [-0.0329095, -0.0320119], [0.0499471, 0.1223335], [0.026046, 0.0899805], [7388e-7, -0.1435792], [75848e-7, -0.1334108], [-0.0216473, 0.0776645], [-0.0225161, 0.0853673]], f1 = [[0.9245, 0], [0, 0], [0.01943, 0]], D1 = [[0.721316, 0], [0, 0], [-881625e-8, -617325e-8]];
            function Hl() {
              return Wl(Gu, [152, -64]).scale(1400).center([-160.908, 62.4864]).clipAngle(30).angle(7.8);
            }
            function Kl() {
              return Wl(Ku, [95, -38]).scale(1e3).clipAngle(55).center([-96.5563, 38.8675]);
            }
            function uu() {
              return Wl(I1, [120, -45]).scale(359.513).clipAngle(55).center([-117.474, 53.0628]);
            }
            function Hs() {
              return Wl(f1, [-20, -18]).scale(209.091).center([20, 16.7214]).clipAngle(82);
            }
            function b1() {
              return Wl(D1, [165, 10]).scale(250).clipAngle(130).center([-165, -10]);
            }
            function Wl($n, zn) {
              var Gn = (0, te.A)(d1($n)).rotate(zn).clipAngle(90), Ke = (0, Ht.A)(zn), Fe = Gn.center;
              return delete Gn.rotate, Gn.center = function(Te) {
                return arguments.length ? Fe(Ke(Te)) : Ke.invert(Fe());
              }, Gn;
            }
            var zl = De(6), os = De(7);
            function jo($n, zn) {
              var Gn = Me(7 * pe(zn) / (3 * zl));
              return [zl * $n * (2 * oe(2 * Gn / 3) - 1) / os, 9 * pe(Gn / 3) / os];
            }
            function uo() {
              return (0, te.A)(jo).scale(164.859);
            }
            function so($n, zn) {
              for (var Gn, Ke = (1 + we) * pe(zn), Fe = zn, Te = 0; Te < 25 && (Fe -= Gn = (pe(Fe / 2) + pe(Fe) - Ke) / (0.5 * oe(Fe / 2) + oe(Fe)), !(ne(Gn) < ge)); Te++) ;
              return [$n * (1 + 2 * oe(Fe) / oe(Fe / 2)) / (3 * Ee), 2 * De(3) * pe(Fe / 2) / De(2 + Ee)];
            }
            function So() {
              return (0, te.A)(so).scale(188.209);
            }
            function Jc($n, zn) {
              for (var Gn, Ke = De(6 / (4 + be)), Fe = (1 + be / 4) * pe(zn), Te = zn / 2, Ae = 0; Ae < 25 && (Te -= Gn = (Te / 2 + pe(Te) - Fe) / (0.5 + oe(Te)), !(ne(Gn) < ge)); Ae++) ;
              return [Ke * (0.5 + oe(Te)) * $n / 1.5, Ke * Te];
            }
            function No() {
              return (0, te.A)(Jc).scale(166.518);
            }
            jo.invert = function($n, zn) {
              var Gn = 3 * Me(zn * os / 9);
              return [$n * os / (zl * (2 * oe(2 * Gn / 3) - 1)), Me(3 * pe(Gn) * zl / 7)];
            }, so.invert = function($n, zn) {
              var Gn = zn * De(2 + Ee) / (2 * De(3)), Ke = 2 * Me(Gn);
              return [3 * Ee * $n / (1 + 2 * oe(Ke) / oe(Ke / 2)), Me((Gn + pe(Ke)) / (1 + we))];
            }, Jc.invert = function($n, zn) {
              var Gn = De(6 / (4 + be)), Ke = zn / Gn;
              return ne(ne(Ke) - Ce) < ge && (Ke = Ke < 0 ? -Ce : Ce), [1.5 * $n / (Gn * (0.5 + oe(Ke))), Me((Ke / 2 + pe(Ke)) / (1 + be / 4))];
            };
            var ro = Z(57949);
            function Io($n, zn) {
              var Gn = zn * zn, Ke = Gn * Gn, Fe = Gn * Ke;
              return [$n * (0.84719 - 0.13063 * Gn + Fe * Fe * (0.05494 * Gn - 0.04515 - 0.02326 * Ke + 331e-5 * Fe)), zn * (1.01183 + Ke * Ke * (0.01926 * Gn - 0.02625 - 396e-5 * Ke))];
            }
            function bs() {
              return (0, te.A)(Io).scale(175.295);
            }
            function ws($n, zn) {
              return [$n * (1 + oe(zn)) / 2, 2 * (zn - ye(zn / 2))];
            }
            function zo() {
              return (0, te.A)(ws).scale(152.63);
            }
            Io.invert = function($n, zn) {
              var Gn, Ke, Fe, Te, Ae = zn, Oe = 25;
              do
                Ae -= Gn = (Ae * (1.01183 + (Fe = (Ke = Ae * Ae) * Ke) * Fe * (0.01926 * Ke - 0.02625 - 396e-5 * Fe)) - zn) / (1.01183 + Fe * Fe * (0.21186 * Ke - 0.23625 + -0.05148 * Fe));
              while (ne(Gn) > ve && --Oe > 0);
              return [$n / (0.84719 - 0.13063 * (Ke = Ae * Ae) + (Te = Ke * (Fe = Ke * Ke)) * Te * (0.05494 * Ke - 0.04515 - 0.02326 * Fe + 331e-5 * Te)), Ae];
            }, ws.invert = function($n, zn) {
              for (var Gn = zn / 2, Ke = 0, Fe = 1 / 0; Ke < 10 && ne(Fe) > ge; ++Ke) {
                var Te = oe(zn / 2);
                zn -= Fe = (zn - ye(zn / 2) - Gn) / (1 - 0.5 / (Te * Te));
              }
              return [2 * $n / (1 + oe(zn)), zn];
            };
            var Yo = [[[[-180, 0], [-90, 90], [0, 0]], [[0, 0], [90, 90], [180, 0]]], [[[-180, 0], [-90, -90], [0, 0]], [[0, 0], [90, -90], [180, 0]]]];
            function Ls() {
              return dr(yt(1 / 0), Yo).rotate([20, 0]).scale(152.63);
            }
            function Ms($n, zn) {
              var Gn = pe(zn), Ke = oe(zn), Fe = me($n);
              if ($n === 0 || ne(zn) === Ce) return [0, zn];
              if (zn === 0) return [$n, 0];
              if (ne($n) === Ce) return [$n * Ke, Ce * Gn];
              var Te = be / (2 * $n) - 2 * $n / be, Ae = 2 * zn / be, Oe = (1 - Ae * Ae) / (Gn - Ae), Be = Te * Te, Ue = Oe * Oe, nt = 1 + Be / Ue, ht = 1 + Ue / Be, Et = (Te * Gn / Oe - Te / 2) / nt, Mt = (Ue * Gn / Be + Oe / 2) / ht, qt = Mt * Mt - (Ue * Gn * Gn / Be + Oe * Gn - 1) / ht;
              return [Ce * (Et + De(Et * Et + Ke * Ke / nt) * Fe), Ce * (Mt + De(qt < 0 ? 0 : qt) * me(-zn * Te) * Fe)];
            }
            function vi() {
              return (0, te.A)(Ms).scale(127.267);
            }
            Ms.invert = function($n, zn) {
              var Gn = ($n /= Ce) * $n, Ke = Gn + (zn /= Ce) * zn, Fe = be * be;
              return [$n ? (Ke - 1 + De((1 - Ke) * (1 - Ke) + 4 * Gn)) / (2 * $n) * Ce : 0, Rt(function(Te) {
                return Ke * (be * pe(Te) - 2 * Te) * be + 4 * Te * Te * (zn - pe(Te)) + 2 * be * Te - Fe * zn;
              }, 0)];
            };
            var Ri = 1.0148, ta = 0.23185, da = -0.14499, Ji = 0.02406, Pa = Ri, Ga = 5 * ta, Ha = 7 * da, Ba = 9 * Ji, ao = 1.790857183;
            function io($n, zn) {
              var Gn = zn * zn;
              return [$n, zn * (Ri + Gn * Gn * (ta + Gn * (da + Ji * Gn)))];
            }
            function To() {
              return (0, te.A)(io).scale(139.319);
            }
            function Mo($n, zn) {
              if (ne(zn) < ge) return [$n, 0];
              var Gn = ye(zn), Ke = $n * pe(zn);
              return [pe(Ke) / Gn, zn + (1 - oe(Ke)) / Gn];
            }
            function po() {
              return (0, te.A)(Mo).scale(103.74);
            }
            io.invert = function($n, zn) {
              zn > ao ? zn = ao : zn < -1.790857183 && (zn = -1.790857183);
              var Gn, Ke = zn;
              do {
                var Fe = Ke * Ke;
                Ke -= Gn = (Ke * (Ri + Fe * Fe * (ta + Fe * (da + Ji * Fe))) - zn) / (Pa + Fe * Fe * (Ga + Fe * (Ha + Ba * Fe)));
              } while (ne(Gn) > ge);
              return [$n, Ke];
            }, Mo.invert = function($n, zn) {
              if (ne(zn) < ge) return [$n, 0];
              var Gn, Ke = $n * $n + zn * zn, Fe = 0.5 * zn, Te = 10;
              do {
                var Ae = ye(Fe), Oe = 1 / oe(Fe), Be = Ke - 2 * zn * Fe + Fe * Fe;
                Fe -= Gn = (Ae * Be + 2 * (Fe - zn)) / (2 + Be * Oe * Oe + 2 * (Fe - zn) * Ae);
              } while (ne(Gn) > ge && --Te > 0);
              return Ae = ye(Fe), [(ne(zn) < ne(Fe + 1 / Ae) ? Me($n * Ae) : me(zn) * me($n) * (Re(ne($n * Ae)) + Ce)) / pe(Fe), Fe];
            };
            var Vo = Z(43212), Xo = Z(81758);
            function Go($n, zn) {
              return [$n[0] * zn[0] + $n[1] * zn[3], $n[0] * zn[1] + $n[1] * zn[4], $n[0] * zn[2] + $n[1] * zn[5] + $n[2], $n[3] * zn[0] + $n[4] * zn[3], $n[3] * zn[1] + $n[4] * zn[4], $n[3] * zn[2] + $n[4] * zn[5] + $n[5]];
            }
            function es($n, zn) {
              return [$n[0] - zn[0], $n[1] - zn[1]];
            }
            function Ts($n) {
              return De($n[0] * $n[0] + $n[1] * $n[1]);
            }
            function Fo($n, zn, Gn) {
              function Ke(Oe, Be) {
                var Ue, nt = zn(Oe, Be), ht = nt.project([Oe * Qe, Be * Qe]);
                return (Ue = nt.transform) ? [Ue[0] * ht[0] + Ue[1] * ht[1] + Ue[2], -(Ue[3] * ht[0] + Ue[4] * ht[1] + Ue[5])] : (ht[1] = -ht[1], ht);
              }
              function Fe(Oe, Be) {
                var Ue = Oe.project.invert, nt = Oe.transform, ht = Be;
                if (nt && (nt = function(xn) {
                  var An = 1 / (xn[0] * xn[4] - xn[1] * xn[3]);
                  return [An * xn[4], -An * xn[1], An * (xn[1] * xn[5] - xn[2] * xn[4]), -An * xn[3], An * xn[0], An * (xn[2] * xn[3] - xn[0] * xn[5])];
                }(nt), ht = [nt[0] * ht[0] + nt[1] * ht[1] + nt[2], nt[3] * ht[0] + nt[4] * ht[1] + nt[5]]), Ue && Oe === function(xn) {
                  return zn(xn[0] * ke, xn[1] * ke);
                }(Et = Ue(ht))) return Et;
                for (var Et, Mt = Oe.children, qt = 0, tn = Mt && Mt.length; qt < tn; ++qt) if (Et = Fe(Mt[qt], Be)) return Et;
              }
              (function Oe(Be, Ue) {
                if (Be.edges = function(Vn) {
                  for (var sr = Vn.length, Un = [], wr = Vn[sr - 1], Cr = 0; Cr < sr; ++Cr) Un.push([wr, wr = Vn[Cr]]);
                  return Un;
                }(Be.face), Ue.face) {
                  var nt = Be.shared = function(Vn, sr) {
                    for (var Un, wr, Cr = Vn.length, Tr = null, ii = 0; ii < Cr; ++ii) {
                      Un = Vn[ii];
                      for (var wi = sr.length; --wi >= 0; ) if (wr = sr[wi], Un[0] === wr[0] && Un[1] === wr[1]) {
                        if (Tr) return [Tr, Un];
                        Tr = Un;
                      }
                    }
                  }(Be.face, Ue.face), ht = (tn = nt.map(Ue.project), xn = nt.map(Be.project), An = es(tn[1], tn[0]), On = es(xn[1], xn[0]), Bn = function(Vn, sr) {
                    return ae(Vn[0] * sr[1] - Vn[1] * sr[0], Vn[0] * sr[0] + Vn[1] * sr[1]);
                  }(An, On), qn = Ts(An) / Ts(On), Go([1, 0, tn[0][0], 0, 1, tn[0][1]], Go([qn, 0, 0, 0, qn, 0], Go([oe(Bn), pe(Bn), 0, -pe(Bn), oe(Bn), 0], [1, 0, -xn[0][0], 0, 1, -xn[0][1]]))));
                  Be.transform = Ue.transform ? Go(Ue.transform, ht) : ht;
                  for (var Et = Ue.edges, Mt = 0, qt = Et.length; Mt < qt; ++Mt) gs(nt[0], Et[Mt][1]) && gs(nt[1], Et[Mt][0]) && (Et[Mt] = Be), gs(nt[0], Et[Mt][0]) && gs(nt[1], Et[Mt][1]) && (Et[Mt] = Be);
                  for (Mt = 0, qt = (Et = Be.edges).length; Mt < qt; ++Mt) gs(nt[0], Et[Mt][0]) && gs(nt[1], Et[Mt][1]) && (Et[Mt] = Ue), gs(nt[0], Et[Mt][1]) && gs(nt[1], Et[Mt][0]) && (Et[Mt] = Ue);
                } else Be.transform = Ue.transform;
                var tn, xn, An, On, Bn, qn;
                return Be.children && Be.children.forEach(function(Vn) {
                  Oe(Vn, Be);
                }), Be;
              })($n, { transform: null }), Bu($n) && (Ke.invert = function(Oe, Be) {
                var Ue = Fe($n, [Oe, -Be]);
                return Ue && (Ue[0] *= ke, Ue[1] *= ke, Ue);
              });
              var Te = (0, te.A)(Ke), Ae = Te.stream;
              return Te.stream = function(Oe) {
                var Be = Te.rotate(), Ue = Ae(Oe), nt = (Te.rotate([0, 0]), Ae(Oe));
                return Te.rotate(Be), Ue.sphere = function() {
                  nt.polygonStart(), nt.lineStart(), Oo(nt, $n), nt.lineEnd(), nt.polygonEnd();
                }, Ue;
              }, Te.angle(Gn == null ? -30 : Gn * Qe);
            }
            function Oo($n, zn, Gn) {
              var Ke, Fe, Te = zn.edges, Ae = Te.length, Oe = { type: "MultiPoint", coordinates: zn.face }, Be = zn.face.filter(function(tn) {
                return ne(tn[1]) !== 90;
              }), Ue = (0, Vo.A)({ type: "MultiPoint", coordinates: Be }), nt = !1, ht = -1, Et = Ue[1][0] - Ue[0][0], Mt = Et === 180 || Et === 360 ? [(Ue[0][0] + Ue[1][0]) / 2, (Ue[0][1] + Ue[1][1]) / 2] : (0, dn.A)(Oe);
              if (Gn) for (; ++ht < Ae && Te[ht] !== Gn; ) ;
              ++ht;
              for (var qt = 0; qt < Ae; ++qt) Fe = Te[(qt + ht) % Ae], Array.isArray(Fe) ? (nt || ($n.point((Ke = (0, Xo.A)(Fe[0], Mt)(ge))[0], Ke[1]), nt = !0), $n.point((Ke = (0, Xo.A)(Fe[1], Mt)(ge))[0], Ke[1])) : (nt = !1, Fe !== Gn && Oo($n, Fe, zn));
            }
            function gs($n, zn) {
              return $n && zn && $n[0] === zn[0] && $n[1] === zn[1];
            }
            function Bu($n) {
              return $n.project.invert || $n.children && $n.children.some(Bu);
            }
            var Ql = Z(48419), dl = [[0, 90], [-90, 0], [0, 0], [90, 0], [180, 0], [0, -90]], h1 = [[0, 2, 1], [0, 3, 2], [5, 1, 2], [5, 2, 3], [0, 1, 4], [0, 4, 3], [5, 4, 1], [5, 3, 4]].map(function($n) {
              return $n.map(function(zn) {
                return dl[zn];
              });
            });
            function z1($n) {
              $n = $n || function(Gn) {
                var Ke = (0, dn.A)({ type: "MultiPoint", coordinates: Gn });
                return (0, Ql.A)().scale(1).translate([0, 0]).rotate([-Ke[0], -Ke[1]]);
              };
              var zn = h1.map(function(Gn) {
                return { face: Gn, project: $n(Gn) };
              });
              return [-1, 0, 0, 1, 0, 1, 4, 5].forEach(function(Gn, Ke) {
                var Fe = zn[Gn];
                Fe && (Fe.children || (Fe.children = [])).push(zn[Ke]);
              }), Fo(zn[0], function(Gn, Ke) {
                return zn[Gn < -be / 2 ? Ke < 0 ? 6 : 4 : Gn < 0 ? Ke < 0 ? 2 : 0 : Gn < be / 2 ? Ke < 0 ? 3 : 1 : Ke < 0 ? 7 : 5];
              }).angle(-30).scale(101.858).center([0, 45]);
            }
            var J1 = 2 / De(3);
            function Lu($n, zn) {
              var Gn = ft($n, zn);
              return [Gn[0] * J1, Gn[1]];
            }
            function M1($n) {
              $n = $n || function(Gn) {
                var Ke = (0, dn.A)({ type: "MultiPoint", coordinates: Gn });
                return (0, te.A)(Lu).translate([0, 0]).scale(1).rotate(Ke[1] > 0 ? [-Ke[0], 0] : [180 - Ke[0], 180]);
              };
              var zn = h1.map(function(Gn) {
                return { face: Gn, project: $n(Gn) };
              });
              return [-1, 0, 0, 1, 0, 1, 4, 5].forEach(function(Gn, Ke) {
                var Fe = zn[Gn];
                Fe && (Fe.children || (Fe.children = [])).push(zn[Ke]);
              }), Fo(zn[0], function(Gn, Ke) {
                return zn[Gn < -be / 2 ? Ke < 0 ? 6 : 4 : Gn < 0 ? Ke < 0 ? 2 : 0 : Gn < be / 2 ? Ke < 0 ? 3 : 1 : Ke < 0 ? 7 : 5];
              }).angle(-30).scale(121.906).center([0, 48.5904]);
            }
            function ku($n) {
              $n = $n || function(Te) {
                var Ae = Te.length === 6 ? (0, dn.A)({ type: "MultiPoint", coordinates: Te }) : Te[0];
                return (0, Ql.A)().scale(1).translate([0, 0]).rotate([-Ae[0], -Ae[1]]);
              };
              var zn = h1.map(function(Te) {
                for (var Ae, Oe = Te.map(kl), Be = Oe.length, Ue = Oe[Be - 1], nt = [], ht = 0; ht < Be; ++ht) Ae = Oe[ht], nt.push(T1([0.9486832980505138 * Ue[0] + 0.31622776601683794 * Ae[0], 0.9486832980505138 * Ue[1] + 0.31622776601683794 * Ae[1], 0.9486832980505138 * Ue[2] + 0.31622776601683794 * Ae[2]]), T1([0.9486832980505138 * Ae[0] + 0.31622776601683794 * Ue[0], 0.9486832980505138 * Ae[1] + 0.31622776601683794 * Ue[1], 0.9486832980505138 * Ae[2] + 0.31622776601683794 * Ue[2]])), Ue = Ae;
                return nt;
              }), Gn = [], Ke = [-1, 0, 0, 1, 0, 1, 4, 5];
              zn.forEach(function(Te, Ae) {
                for (var Oe, Be, Ue = h1[Ae], nt = Ue.length, ht = Gn[Ae] = [], Et = 0; Et < nt; ++Et) zn.push([Ue[Et], Te[(2 * Et + 2) % (2 * nt)], Te[(2 * Et + 1) % (2 * nt)]]), Ke.push(Ae), ht.push((Oe = kl(Te[(2 * Et + 2) % (2 * nt)]), Be = kl(Te[(2 * Et + 1) % (2 * nt)]), [Oe[1] * Be[2] - Oe[2] * Be[1], Oe[2] * Be[0] - Oe[0] * Be[2], Oe[0] * Be[1] - Oe[1] * Be[0]]));
              });
              var Fe = zn.map(function(Te) {
                return { project: $n(Te), face: Te };
              });
              return Ke.forEach(function(Te, Ae) {
                var Oe = Fe[Te];
                Oe && (Oe.children || (Oe.children = [])).push(Fe[Ae]);
              }), Fo(Fe[0], function(Te, Ae) {
                var Oe = oe(Ae), Be = [Oe * oe(Te), Oe * pe(Te), pe(Ae)], Ue = Te < -be / 2 ? Ae < 0 ? 6 : 4 : Te < 0 ? Ae < 0 ? 2 : 0 : Te < be / 2 ? Ae < 0 ? 3 : 1 : Ae < 0 ? 7 : 5, nt = Gn[Ue];
                return Fe[Wu(nt[0], Be) < 0 ? 8 + 3 * Ue : Wu(nt[1], Be) < 0 ? 8 + 3 * Ue + 1 : Wu(nt[2], Be) < 0 ? 8 + 3 * Ue + 2 : Ue];
              }).angle(-30).scale(110.625).center([0, 45]);
            }
            function Wu($n, zn) {
              for (var Gn = 0, Ke = $n.length, Fe = 0; Gn < Ke; ++Gn) Fe += $n[Gn] * zn[Gn];
              return Fe;
            }
            function T1($n) {
              return [ae($n[1], $n[0]) * Qe, Me(ue(-1, de(1, $n[2]))) * Qe];
            }
            function kl($n) {
              var zn = $n[0] * ke, Gn = $n[1] * ke, Ke = oe(Gn);
              return [Ke * oe(zn), Ke * pe(zn), pe(Gn)];
            }
            function Yl() {
            }
            function i1($n, zn) {
              var Gn, Ke = zn.stream;
              if (!Ke) throw new Error("invalid projection");
              switch ($n && $n.type) {
                case "Feature":
                  Gn = iu;
                  break;
                case "FeatureCollection":
                  Gn = ic;
                  break;
                default:
                  Gn = xu;
              }
              return Gn($n, Ke);
            }
            function ic($n, zn) {
              return { type: "FeatureCollection", features: $n.features.map(function(Gn) {
                return iu(Gn, zn);
              }) };
            }
            function iu($n, zn) {
              return { type: "Feature", id: $n.id, properties: $n.properties, geometry: xu($n.geometry, zn) };
            }
            function xu($n, zn) {
              if (!$n) return null;
              if ($n.type === "GeometryCollection") return function(Ke, Fe) {
                return { type: "GeometryCollection", geometries: Ke.geometries.map(function(Te) {
                  return xu(Te, Fe);
                }) };
              }($n, zn);
              var Gn;
              switch ($n.type) {
                case "Point":
                case "MultiPoint":
                  Gn = Ec;
                  break;
                case "LineString":
                case "MultiLineString":
                  Gn = wu;
                  break;
                case "Polygon":
                case "MultiPolygon":
                case "Sphere":
                  Gn = Pc;
                  break;
                default:
                  return null;
              }
              return (0, ti.A)($n, zn(Gn)), Gn.result();
            }
            Lu.invert = function($n, zn) {
              return ft.invert($n / J1, zn);
            };
            var vc = [], Zl = [], Ec = { point: function($n, zn) {
              vc.push([$n, zn]);
            }, result: function() {
              var $n = vc.length ? vc.length < 2 ? { type: "Point", coordinates: vc[0] } : { type: "MultiPoint", coordinates: vc } : null;
              return vc = [], $n;
            } }, wu = { lineStart: Yl, point: function($n, zn) {
              vc.push([$n, zn]);
            }, lineEnd: function() {
              vc.length && (Zl.push(vc), vc = []);
            }, result: function() {
              var $n = Zl.length ? Zl.length < 2 ? { type: "LineString", coordinates: Zl[0] } : { type: "MultiLineString", coordinates: Zl } : null;
              return Zl = [], $n;
            } }, Pc = { polygonStart: Yl, lineStart: Yl, point: function($n, zn) {
              vc.push([$n, zn]);
            }, lineEnd: function() {
              var $n = vc.length;
              if ($n) {
                do
                  vc.push(vc[0].slice());
                while (++$n < 4);
                Zl.push(vc), vc = [];
              }
            }, polygonEnd: Yl, result: function() {
              if (!Zl.length) return null;
              var $n = [], zn = [];
              return Zl.forEach(function(Gn) {
                (function(Ke) {
                  if ((Fe = Ke.length) < 4) return !1;
                  for (var Fe, Te = 0, Ae = Ke[Fe - 1][1] * Ke[0][0] - Ke[Fe - 1][0] * Ke[0][1]; ++Te < Fe; ) Ae += Ke[Te - 1][1] * Ke[Te][0] - Ke[Te - 1][0] * Ke[Te][1];
                  return Ae <= 0;
                })(Gn) ? $n.push([Gn]) : zn.push(Gn);
              }), zn.forEach(function(Gn) {
                var Ke = Gn[0];
                $n.some(function(Fe) {
                  if (function(Te, Ae) {
                    for (var Oe = Ae[0], Be = Ae[1], Ue = !1, nt = 0, ht = Te.length, Et = ht - 1; nt < ht; Et = nt++) {
                      var Mt = Te[nt], qt = Mt[0], tn = Mt[1], xn = Te[Et], An = xn[0], On = xn[1];
                      tn > Be ^ On > Be && Oe < (An - qt) * (Be - tn) / (On - tn) + qt && (Ue = !Ue);
                    }
                    return Ue;
                  }(Fe[0], Ke)) return Fe.push(Gn), !0;
                }) || $n.push([Gn]);
              }), Zl = [], $n.length ? $n.length > 1 ? { type: "MultiPolygon", coordinates: $n } : { type: "Polygon", coordinates: $n[0] } : null;
            } };
            function du($n) {
              var zn = $n(Ce, 0)[0] - $n(-Ce, 0)[0];
              function Gn(Ke, Fe) {
                var Te = ne(Ke) < Ce, Ae = $n(Te ? Ke : Ke > 0 ? Ke - be : Ke + be, Fe), Oe = (Ae[0] - Ae[1]) * we, Be = (Ae[0] + Ae[1]) * we;
                if (Te) return [Oe, Be];
                var Ue = zn * we, nt = Oe > 0 ^ Be > 0 ? -1 : 1;
                return [nt * Oe - me(Be) * Ue, nt * Be - me(Oe) * Ue];
              }
              return $n.invert && (Gn.invert = function(Ke, Fe) {
                var Te = (Ke + Fe) * we, Ae = (Fe - Ke) * we, Oe = ne(Te) < 0.5 * zn && ne(Ae) < 0.5 * zn;
                if (!Oe) {
                  var Be = zn * we, Ue = Te > 0 ^ Ae > 0 ? -1 : 1, nt = -Ue * Ke + (Ae > 0 ? 1 : -1) * Be, ht = -Ue * Fe + (Te > 0 ? 1 : -1) * Be;
                  Te = (-nt - ht) * we, Ae = (nt - ht) * we;
                }
                var Et = $n.invert(Te, Ae);
                return Oe || (Et[0] += Te > 0 ? be : -be), Et;
              }), (0, te.A)(Gn).rotate([-90, -90, 45]).clipAngle(179.999);
            }
            function Ac() {
              return du(la).scale(176.423);
            }
            function yu() {
              return du(Ur).scale(111.48);
            }
            function au($n, zn) {
              if (!(0 <= (zn = +zn) && zn <= 20)) throw new Error("invalid digits");
              function Gn(Ue) {
                var nt = Ue.length, ht = 2, Et = new Array(nt);
                for (Et[0] = +Ue[0].toFixed(zn), Et[1] = +Ue[1].toFixed(zn); ht < nt; ) Et[ht] = Ue[ht], ++ht;
                return Et;
              }
              function Ke(Ue) {
                return Ue.map(Gn);
              }
              function Fe(Ue) {
                for (var nt = Gn(Ue[0]), ht = [nt], Et = 1; Et < Ue.length; Et++) {
                  var Mt = Gn(Ue[Et]);
                  (Mt.length > 2 || Mt[0] != nt[0] || Mt[1] != nt[1]) && (ht.push(Mt), nt = Mt);
                }
                return ht.length === 1 && Ue.length > 1 && ht.push(Gn(Ue[Ue.length - 1])), ht;
              }
              function Te(Ue) {
                return Ue.map(Fe);
              }
              function Ae(Ue) {
                if (Ue == null) return Ue;
                var nt;
                switch (Ue.type) {
                  case "GeometryCollection":
                    nt = { type: "GeometryCollection", geometries: Ue.geometries.map(Ae) };
                    break;
                  case "Point":
                    nt = { type: "Point", coordinates: Gn(Ue.coordinates) };
                    break;
                  case "MultiPoint":
                    nt = { type: Ue.type, coordinates: Ke(Ue.coordinates) };
                    break;
                  case "LineString":
                    nt = { type: Ue.type, coordinates: Fe(Ue.coordinates) };
                    break;
                  case "MultiLineString":
                  case "Polygon":
                    nt = { type: Ue.type, coordinates: Te(Ue.coordinates) };
                    break;
                  case "MultiPolygon":
                    nt = { type: "MultiPolygon", coordinates: Ue.coordinates.map(Te) };
                    break;
                  default:
                    return Ue;
                }
                return Ue.bbox != null && (nt.bbox = Ue.bbox), nt;
              }
              function Oe(Ue) {
                var nt = { type: "Feature", properties: Ue.properties, geometry: Ae(Ue.geometry) };
                return Ue.id != null && (nt.id = Ue.id), Ue.bbox != null && (nt.bbox = Ue.bbox), nt;
              }
              if ($n != null) switch ($n.type) {
                case "Feature":
                  return Oe($n);
                case "FeatureCollection":
                  var Be = { type: "FeatureCollection", features: $n.features.map(Oe) };
                  return $n.bbox != null && (Be.bbox = $n.bbox), Be;
                default:
                  return Ae($n);
              }
              return $n;
            }
            function Qc($n) {
              var zn = pe($n);
              function Gn(Ke, Fe) {
                var Te = zn ? ye(Ke * zn / 2) / zn : Ke / 2;
                if (!Fe) return [2 * Te, -$n];
                var Ae = 2 * ee(Te * pe(Fe)), Oe = 1 / ye(Fe);
                return [pe(Ae) * Oe, Fe + (1 - oe(Ae)) * Oe - $n];
              }
              return Gn.invert = function(Ke, Fe) {
                if (ne(Fe += $n) < ge) return [zn ? 2 * ee(zn * Ke / 2) / zn : Ke, 0];
                var Te, Ae = Ke * Ke + Fe * Fe, Oe = 0, Be = 10;
                do {
                  var Ue = ye(Oe), nt = 1 / oe(Oe), ht = Ae - 2 * Fe * Oe + Oe * Oe;
                  Oe -= Te = (Ue * ht + 2 * (Oe - Fe)) / (2 + ht * nt * nt + 2 * (Oe - Fe) * Ue);
                } while (ne(Te) > ge && --Be > 0);
                var Et = Ke * (Ue = ye(Oe)), Mt = ye(ne(Fe) < ne(Oe + 1 / Ue) ? 0.5 * Me(Et) : 0.5 * Re(Et) + be / 4) / pe(Oe);
                return [zn ? 2 * ee(zn * Mt) / zn : 2 * Mt, Oe];
              }, Gn;
            }
            function Cc() {
              return Vt(Qc).scale(131.215);
            }
            var Ol = [[0.9986, -0.062], [1, 0], [0.9986, 0.062], [0.9954, 0.124], [0.99, 0.186], [0.9822, 0.248], [0.973, 0.31], [0.96, 0.372], [0.9427, 0.434], [0.9216, 0.4958], [0.8962, 0.5571], [0.8679, 0.6176], [0.835, 0.6769], [0.7986, 0.7346], [0.7597, 0.7903], [0.7186, 0.8435], [0.6732, 0.8936], [0.6213, 0.9394], [0.5722, 0.9761], [0.5322, 1]];
            function uc($n, zn) {
              var Gn, Ke = de(18, 36 * ne(zn) / be), Fe = ce(Ke), Te = Ke - Fe, Ae = (Gn = Ol[Fe])[0], Oe = Gn[1], Be = (Gn = Ol[++Fe])[0], Ue = Gn[1], nt = (Gn = Ol[de(19, ++Fe)])[0], ht = Gn[1];
              return [$n * (Be + Te * (nt - Ae) / 2 + Te * Te * (nt - 2 * Be + Ae) / 2), (zn > 0 ? Ce : -Ce) * (Ue + Te * (ht - Oe) / 2 + Te * Te * (ht - 2 * Ue + Oe) / 2)];
            }
            function cs() {
              return (0, te.A)(uc).scale(152.63);
            }
            function Mu($n, zn) {
              var Gn = function(Ae) {
                function Oe(Be, Ue) {
                  var nt = oe(Ue), ht = (Ae - 1) / (Ae - nt * oe(Be));
                  return [ht * nt * pe(Be), ht * pe(Ue)];
                }
                return Oe.invert = function(Be, Ue) {
                  var nt = Be * Be + Ue * Ue, ht = De(nt), Et = (Ae - De(1 - nt * (Ae + 1) / (Ae - 1))) / ((Ae - 1) / ht + ht / (Ae - 1));
                  return [ae(Be * Et, ht * De(1 - Et * Et)), ht ? Me(Ue * Et / ht) : 0];
                }, Oe;
              }($n);
              if (!zn) return Gn;
              var Ke = oe(zn), Fe = pe(zn);
              function Te(Ae, Oe) {
                var Be = Gn(Ae, Oe), Ue = Be[1], nt = Ue * Fe / ($n - 1) + Ke;
                return [Be[0] * Ke / nt, Ue / nt];
              }
              return Te.invert = function(Ae, Oe) {
                var Be = ($n - 1) / ($n - 1 - Oe * Fe);
                return Gn.invert(Be * Ae, Be * Oe * Ke);
              }, Te;
            }
            function Hu() {
              var $n = 2, zn = 0, Gn = (0, te.U)(Mu), Ke = Gn($n, zn);
              return Ke.distance = function(Fe) {
                return arguments.length ? Gn($n = +Fe, zn) : $n;
              }, Ke.tilt = function(Fe) {
                return arguments.length ? Gn($n, zn = Fe * ke) : zn * Qe;
              }, Ke.scale(432.147).clipAngle(Re(1 / $n) * Qe - 1e-6);
            }
            Ol.forEach(function($n) {
              $n[1] *= 1.0144;
            }), uc.invert = function($n, zn) {
              var Gn = zn / Ce, Ke = 90 * Gn, Fe = de(18, ne(Ke / 5)), Te = ue(0, ce(Fe));
              do {
                var Ae = Ol[Te][1], Oe = Ol[Te + 1][1], Be = Ol[de(19, Te + 2)][1], Ue = Be - Ae, nt = Be - 2 * Oe + Ae, ht = 2 * (ne(Gn) - Oe) / Ue, Et = nt / Ue, Mt = ht * (1 - Et * ht * (1 - 2 * Et * ht));
                if (Mt >= 0 || Te === 1) {
                  Ke = (zn >= 0 ? 5 : -5) * (Mt + Fe);
                  var qt, tn = 50;
                  do
                    Mt = (Fe = de(18, ne(Ke) / 5)) - (Te = ce(Fe)), Ae = Ol[Te][1], Oe = Ol[Te + 1][1], Be = Ol[de(19, Te + 2)][1], Ke -= (qt = (zn >= 0 ? Ce : -Ce) * (Oe + Mt * (Be - Ae) / 2 + Mt * Mt * (Be - 2 * Oe + Ae) / 2) - zn) * Qe;
                  while (ne(qt) > ve && --tn > 0);
                  break;
                }
              } while (--Te >= 0);
              var xn = Ol[Te][0], An = Ol[Te + 1][0], On = Ol[de(19, Te + 2)][0];
              return [$n / (An + Mt * (On - xn) / 2 + Mt * Mt * (On - 2 * An + xn) / 2), Ke * ke];
            };
            var ou = 1e-4, Gc = 1e4, Su = -180, Lc = Su + ou, Do = 180, Ru = Do - ou, dc = -90, vl = dc + ou, Eu = 90, fu = Eu - ou;
            function ac($n) {
              return $n.length > 0;
            }
            function bc($n) {
              return $n === dc || $n === Eu ? [0, $n] : [Su, (zn = $n, Math.floor(zn * Gc) / Gc)];
              var zn;
            }
            function kc($n) {
              var zn = $n[0], Gn = $n[1], Ke = !1;
              return zn <= Lc ? (zn = Su, Ke = !0) : zn >= Ru && (zn = Do, Ke = !0), Gn <= vl ? (Gn = dc, Ke = !0) : Gn >= fu && (Gn = Eu, Ke = !0), Ke ? [zn, Gn] : $n;
            }
            function fc($n) {
              return $n.map(kc);
            }
            function oc($n, zn, Gn) {
              for (var Ke = 0, Fe = $n.length; Ke < Fe; ++Ke) {
                var Te = $n[Ke].slice();
                Gn.push({ index: -1, polygon: zn, ring: Te });
                for (var Ae = 0, Oe = Te.length; Ae < Oe; ++Ae) {
                  var Be = Te[Ae], Ue = Be[0], nt = Be[1];
                  if (Ue <= Lc || Ue >= Ru || nt <= vl || nt >= fu) {
                    Te[Ae] = kc(Be);
                    for (var ht = Ae + 1; ht < Oe; ++ht) {
                      var Et = Te[ht], Mt = Et[0], qt = Et[1];
                      if (Mt > Lc && Mt < Ru && qt > vl && qt < fu) break;
                    }
                    if (ht === Ae + 1) continue;
                    if (Ae) {
                      var tn = { index: -1, polygon: zn, ring: Te.slice(0, Ae + 1) };
                      tn.ring[tn.ring.length - 1] = bc(nt), Gn[Gn.length - 1] = tn;
                    } else Gn.pop();
                    if (ht >= Oe) break;
                    Gn.push({ index: -1, polygon: zn, ring: Te = Te.slice(ht - 1) }), Te[0] = bc(Te[0][1]), Ae = -1, Oe = Te.length;
                  }
                }
              }
            }
            function ss($n) {
              var zn, Gn, Ke, Fe, Te, Ae, Oe = $n.length, Be = {}, Ue = {};
              for (zn = 0; zn < Oe; ++zn) Ke = (Gn = $n[zn]).ring[0], Te = Gn.ring[Gn.ring.length - 1], Ke[0] !== Te[0] || Ke[1] !== Te[1] ? (Gn.index = zn, Be[Ke] = Ue[Te] = Gn) : (Gn.polygon.push(Gn.ring), $n[zn] = null);
              for (zn = 0; zn < Oe; ++zn) if (Gn = $n[zn]) {
                if (Ke = Gn.ring[0], Te = Gn.ring[Gn.ring.length - 1], Fe = Ue[Ke], Ae = Be[Te], delete Be[Ke], delete Ue[Te], Ke[0] === Te[0] && Ke[1] === Te[1]) {
                  Gn.polygon.push(Gn.ring);
                  continue;
                }
                Fe ? (delete Ue[Ke], delete Be[Fe.ring[0]], Fe.ring.pop(), $n[Fe.index] = null, Gn = { index: -1, polygon: Fe.polygon, ring: Fe.ring.concat(Gn.ring) }, Fe === Ae ? Gn.polygon.push(Gn.ring) : (Gn.index = Oe++, $n.push(Be[Gn.ring[0]] = Ue[Gn.ring[Gn.ring.length - 1]] = Gn))) : Ae ? (delete Be[Te], delete Ue[Ae.ring[Ae.ring.length - 1]], Gn.ring.pop(), Gn = { index: Oe++, polygon: Ae.polygon, ring: Gn.ring.concat(Ae.ring) }, $n[Ae.index] = null, $n.push(Be[Gn.ring[0]] = Ue[Gn.ring[Gn.ring.length - 1]] = Gn)) : (Gn.ring.push(Gn.ring[0]), Gn.polygon.push(Gn.ring));
              }
            }
            function su($n) {
              var zn = { type: "Feature", geometry: Tl($n.geometry) };
              return $n.id != null && (zn.id = $n.id), $n.bbox != null && (zn.bbox = $n.bbox), $n.properties != null && (zn.properties = $n.properties), zn;
            }
            function Tl($n) {
              if ($n == null) return $n;
              var zn, Gn, Ke, Fe;
              switch ($n.type) {
                case "GeometryCollection":
                  zn = { type: "GeometryCollection", geometries: $n.geometries.map(Tl) };
                  break;
                case "Point":
                  zn = { type: "Point", coordinates: kc($n.coordinates) };
                  break;
                case "MultiPoint":
                case "LineString":
                  zn = { type: $n.type, coordinates: fc($n.coordinates) };
                  break;
                case "MultiLineString":
                  zn = { type: "MultiLineString", coordinates: $n.coordinates.map(fc) };
                  break;
                case "Polygon":
                  var Te = [];
                  oc($n.coordinates, Te, Gn = []), ss(Gn), zn = { type: "Polygon", coordinates: Te };
                  break;
                case "MultiPolygon":
                  Gn = [], Ke = -1, Fe = $n.coordinates.length;
                  for (var Ae = new Array(Fe); ++Ke < Fe; ) oc($n.coordinates[Ke], Ae[Ke] = [], Gn);
                  ss(Gn), zn = { type: "MultiPolygon", coordinates: Ae.filter(ac) };
                  break;
                default:
                  return $n;
              }
              return $n.bbox != null && (zn.bbox = $n.bbox), zn;
            }
            function sc($n) {
              if ($n == null) return $n;
              switch ($n.type) {
                case "Feature":
                  return su($n);
                case "FeatureCollection":
                  var zn = { type: "FeatureCollection", features: $n.features.map(su) };
                  return $n.bbox != null && (zn.bbox = $n.bbox), zn;
                default:
                  return Tl($n);
              }
            }
            function hc($n, zn) {
              var Gn = ye(zn / 2), Ke = pe(Se * Gn);
              return [$n * (0.74482 - 0.34588 * Ke * Ke), 1.70711 * Gn];
            }
            function Vc() {
              return (0, te.A)(hc).scale(146.153);
            }
            function Rc($n, zn, Gn) {
              var Ke = (0, Xo.A)(zn, Gn), Fe = Ke(0.5), Te = (0, Ht.A)([-Fe[0], -Fe[1]])(zn), Ae = Ke.distance / 2, Oe = -Me(pe(Te[1] * ke) / pe(Ae)), Be = [-Fe[0], -Fe[1], -(Te[0] > 0 ? be - Oe : Oe) * Qe], Ue = (0, te.A)($n(Ae)).rotate(Be), nt = (0, Ht.A)(Be), ht = Ue.center;
              return delete Ue.rotate, Ue.center = function(Et) {
                return arguments.length ? ht(nt(Et)) : nt.invert(ht());
              }, Ue.clipAngle(90);
            }
            function Zo($n) {
              var zn = oe($n);
              function Gn(Ke, Fe) {
                var Te = (0, Ql.T)(Ke, Fe);
                return Te[0] *= zn, Te;
              }
              return Gn.invert = function(Ke, Fe) {
                return Ql.T.invert(Ke / zn, Fe);
              }, Gn;
            }
            function Ou() {
              return Uc([-158, 21.5], [-77, 39]).clipAngle(60).scale(400);
            }
            function Uc($n, zn) {
              return Rc(Zo, $n, zn);
            }
            function El($n) {
              if (!($n *= 2)) return it.j;
              var zn = -$n / 2, Gn = -zn, Ke = $n * $n, Fe = ye(Gn), Te = 0.5 / pe(Gn);
              function Ae(Oe, Be) {
                var Ue = Re(oe(Be) * oe(Oe - zn)), nt = Re(oe(Be) * oe(Oe - Gn));
                return [((Ue *= Ue) - (nt *= nt)) / (2 * $n), (Be < 0 ? -1 : 1) * De(4 * Ke * nt - (Ke - Ue + nt) * (Ke - Ue + nt)) / (2 * $n)];
              }
              return Ae.invert = function(Oe, Be) {
                var Ue, nt, ht = Be * Be, Et = oe(De(ht + (Ue = Oe + zn) * Ue)), Mt = oe(De(ht + (Ue = Oe + Gn) * Ue));
                return [ae(nt = Et - Mt, Ue = (Et + Mt) * Fe), (Be < 0 ? -1 : 1) * Re(De(Ue * Ue + nt * nt) * Te)];
              }, Ae;
            }
            function eu() {
              return Wc([-158, 21.5], [-77, 39]).clipAngle(130).scale(122.571);
            }
            function Wc($n, zn) {
              return Rc(El, $n, zn);
            }
            function hu($n, zn) {
              if (ne(zn) < ge) return [$n, 0];
              var Gn = ne(zn / Ce), Ke = Me(Gn);
              if (ne($n) < ge || ne(ne(zn) - Ce) < ge) return [0, me(zn) * be * ye(Ke / 2)];
              var Fe = oe(Ke), Te = ne(be / $n - $n / be) / 2, Ae = Te * Te, Oe = Fe / (Gn + Fe - 1), Be = Oe * (2 / Gn - 1), Ue = Be * Be, nt = Ue + Ae, ht = Oe - Ue, Et = Ae + Oe;
              return [me($n) * be * (Te * ht + De(Ae * ht * ht - nt * (Oe * Oe - Ue))) / nt, me(zn) * be * (Be * Et - Te * De((Ae + 1) * nt - Et * Et)) / nt];
            }
            function gd() {
              return (0, te.A)(hu).scale(79.4183);
            }
            function Jl($n, zn) {
              if (ne(zn) < ge) return [$n, 0];
              var Gn = ne(zn / Ce), Ke = Me(Gn);
              if (ne($n) < ge || ne(ne(zn) - Ce) < ge) return [0, me(zn) * be * ye(Ke / 2)];
              var Fe = oe(Ke), Te = ne(be / $n - $n / be) / 2, Ae = Te * Te, Oe = Fe * (De(1 + Ae) - Te * Fe) / (1 + Ae * Gn * Gn);
              return [me($n) * be * Oe, me(zn) * be * De(1 - Oe * (2 * Te + Oe))];
            }
            function Yc() {
              return (0, te.A)(Jl).scale(79.4183);
            }
            function nc($n, zn) {
              if (ne(zn) < ge) return [$n, 0];
              var Gn = zn / Ce, Ke = Me(Gn);
              if (ne($n) < ge || ne(ne(zn) - Ce) < ge) return [0, be * ye(Ke / 2)];
              var Fe = (be / $n - $n / be) / 2, Te = Gn / (1 + oe(Ke));
              return [be * (me($n) * De(Fe * Fe + 1 - Te * Te) - Fe), be * Te];
            }
            function Ko() {
              return (0, te.A)(nc).scale(79.4183);
            }
            function ks($n, zn) {
              if (!zn) return [$n, 0];
              var Gn = ne(zn);
              if (!$n || Gn === Ce) return [0, zn];
              var Ke = Gn / Ce, Fe = Ke * Ke, Te = (8 * Ke - Fe * (Fe + 2) - 5) / (2 * Fe * (Ke - 1)), Ae = Te * Te, Oe = Ke * Te, Be = Fe + Ae + 2 * Oe, Ue = Ke + 3 * Te, nt = $n / Ce, ht = nt + 1 / nt, Et = me(ne($n) - Ce) * De(ht * ht - 4), Mt = Et * Et, qt = (Et * (Be + Ae - 1) + 2 * De(Be * (Fe + Ae * Mt - 1) + (1 - Fe) * (Fe * (Ue * Ue + 4 * Ae) + 12 * Oe * Ae + 4 * Ae * Ae))) / (4 * Be + Mt);
              return [me($n) * Ce * qt, me(zn) * Ce * De(1 + Et * ne(qt) - qt * qt)];
            }
            function qs() {
              return (0, te.A)(ks).scale(127.16);
            }
            function a1($n, zn, Gn, Ke) {
              var Fe = be / 3;
              $n = ue($n, ge), zn = ue(zn, ge), $n = de($n, Ce), zn = de(zn, be - ge), Gn = ue(Gn, 0), Gn = de(Gn, 100 - ge);
              var Te = (Ke = ue(Ke, ge)) / 100, Ae = Re((Gn / 100 + 1) * oe(Fe)) / Fe, Oe = pe($n) / pe(Ae * Ce), Be = zn / be, Ue = De(Te * pe($n / 2) / pe(zn / 2));
              return function(nt, ht, Et, Mt, qt) {
                function tn(xn, An) {
                  var On = Et * pe(Mt * An), Bn = De(1 - On * On), qn = De(2 / (1 + Bn * oe(xn *= qt)));
                  return [nt * Bn * qn * pe(xn), ht * On * qn];
                }
                return tn.invert = function(xn, An) {
                  var On = xn / nt, Bn = An / ht, qn = De(On * On + Bn * Bn), Vn = 2 * Me(qn / 2);
                  return [ae(xn * ye(Vn), nt * qn) / qt, qn && Me(An * pe(Vn) / (ht * Et * qn)) / Mt];
                }, tn;
              }(Ue / De(Be * Oe * Ae), 1 / (Ue * De(Be * Oe * Ae)), Oe, Ae, Be);
            }
            function $l() {
              var $n = 65 * ke, zn = 60 * ke, Gn = 20, Ke = 200, Fe = (0, te.U)(a1), Te = Fe($n, zn, Gn, Ke);
              return Te.poleline = function(Ae) {
                return arguments.length ? Fe($n = +Ae * ke, zn, Gn, Ke) : $n * Qe;
              }, Te.parallels = function(Ae) {
                return arguments.length ? Fe($n, zn = +Ae * ke, Gn, Ke) : zn * Qe;
              }, Te.inflation = function(Ae) {
                return arguments.length ? Fe($n, zn, Gn = +Ae, Ke) : Gn;
              }, Te.ratio = function(Ae) {
                return arguments.length ? Fe($n, zn, Gn, Ke = +Ae) : Ke;
              }, Te.scale(163.775);
            }
            function o1() {
              return $l().poleline(65).parallels(60).inflation(0).ratio(200).scale(172.633);
            }
            hc.invert = function($n, zn) {
              var Gn = zn / 1.70711, Ke = pe(Se * Gn);
              return [$n / (0.74482 - 0.34588 * Ke * Ke), 2 * ee(Gn)];
            }, hu.invert = function($n, zn) {
              if (ne(zn) < ge) return [$n, 0];
              if (ne($n) < ge) return [0, Ce * pe(2 * ee(zn / be))];
              var Gn = ($n /= be) * $n, Ke = (zn /= be) * zn, Fe = Gn + Ke, Te = Fe * Fe, Ae = -ne(zn) * (1 + Fe), Oe = Ae - 2 * Ke + Gn, Be = -2 * Ae + 1 + 2 * Ke + Te, Ue = Ke / Be + (2 * Oe * Oe * Oe / (Be * Be * Be) - 9 * Ae * Oe / (Be * Be)) / 27, nt = (Ae - Oe * Oe / (3 * Be)) / Be, ht = 2 * De(-nt / 3), Et = Re(3 * Ue / (nt * ht)) / 3;
              return [be * (Fe - 1 + De(1 + 2 * (Gn - Ke) + Te)) / (2 * $n), me(zn) * be * (-ht * oe(Et + be / 3) - Oe / (3 * Be))];
            }, Jl.invert = function($n, zn) {
              if (!$n) return [0, Ce * pe(2 * ee(zn / be))];
              var Gn = ne($n / be), Ke = (1 - Gn * Gn - (zn /= be) * zn) / (2 * Gn), Fe = De(Ke * Ke + 1);
              return [me($n) * be * (Fe - Ke), me(zn) * Ce * pe(2 * ae(De((1 - 2 * Ke * Gn) * (Ke + Fe) - Gn), De(Fe + Ke + Gn)))];
            }, nc.invert = function($n, zn) {
              if (!zn) return [$n, 0];
              var Gn = zn / be, Ke = (be * be * (1 - Gn * Gn) - $n * $n) / (2 * be * $n);
              return [$n ? be * (me($n) * De(Ke * Ke + 1) - Ke) : 0, Ce * pe(2 * ee(Gn))];
            }, ks.invert = function($n, zn) {
              var Gn;
              if (!$n || !zn) return [$n, zn];
              zn /= be;
              var Ke = me($n) * $n / Ce, Fe = (Ke * Ke - 1 + 4 * zn * zn) / ne(Ke), Te = Fe * Fe, Ae = 2 * zn, Oe = 50;
              do {
                var Be = Ae * Ae, Ue = (8 * Ae - Be * (Be + 2) - 5) / (2 * Be * (Ae - 1)), nt = (3 * Ae - Be * Ae - 10) / (2 * Be * Ae), ht = Ue * Ue, Et = Ae * Ue, Mt = Ae + Ue, qt = Mt * Mt, tn = Ae + 3 * Ue, xn = -2 * Mt * (4 * Et * ht + (1 - 4 * Be + 3 * Be * Be) * (1 + nt) + ht * (14 * Be - 6 - Te + (8 * Be - 8 - 2 * Te) * nt) + Et * (12 * Be - 8 + (10 * Be - 10 - Te) * nt)), An = De(qt * (Be + ht * Te - 1) + (1 - Be) * (Be * (tn * tn + 4 * ht) + ht * (12 * Et + 4 * ht)));
                Ae -= Gn = (Fe * (qt + ht - 1) + 2 * An - Ke * (4 * qt + Te)) / (Fe * (2 * Ue * nt + 2 * Mt * (1 + nt)) + xn / An - 8 * Mt * (Fe * (-1 + ht + qt) + 2 * An) * (1 + nt) / (Te + 4 * qt));
              } while (Gn > ge && --Oe > 0);
              return [me($n) * (De(Fe * Fe + 4) + Fe) * be / 4, Ce * Ae];
            };
            var $c = 4 * be + 3 * De(3), lu = 2 * De(2 * be * De(3) / $c), Ul = Nt(lu * De(3) / be, lu, $c / 6);
            function zs() {
              return (0, te.A)(Ul).scale(176.84);
            }
            function Os($n, zn) {
              return [$n * De(1 - 3 * zn * zn / (be * be)), zn];
            }
            function lc() {
              return (0, te.A)(Os).scale(152.63);
            }
            function xc($n, zn) {
              var Gn = oe(zn), Ke = oe($n) * Gn, Fe = 1 - Ke, Te = oe($n = ae(pe($n) * Gn, -pe(zn))), Ae = pe($n);
              return [Ae * (Gn = De(1 - Ke * Ke)) - Te * Fe, -Te * Gn - Ae * Fe];
            }
            function Iu() {
              return (0, te.A)(xc).rotate([0, -90, 45]).scale(124.75).clipAngle(179.999);
            }
            function tu($n, zn) {
              var Gn = Ve($n, zn);
              return [(Gn[0] + $n / Ce) / 2, (Gn[1] + zn) / 2];
            }
            function w1() {
              return (0, te.A)(tu).scale(158.837);
            }
            Os.invert = function($n, zn) {
              return [$n / De(1 - 3 * zn * zn / (be * be)), zn];
            }, xc.invert = function($n, zn) {
              var Gn = ($n * $n + zn * zn) / -2, Ke = De(-Gn * (2 + Gn)), Fe = zn * Gn + $n * Ke, Te = $n * Gn - zn * Ke, Ae = De(Te * Te + Fe * Fe);
              return [ae(Ke * Fe, Ae * (1 + Gn)), Ae ? -Me(Ke * Te / Ae) : 0];
            }, tu.invert = function($n, zn) {
              var Gn = $n, Ke = zn, Fe = 25;
              do {
                var Te, Ae = oe(Ke), Oe = pe(Ke), Be = pe(2 * Ke), Ue = Oe * Oe, nt = Ae * Ae, ht = pe(Gn), Et = oe(Gn / 2), Mt = pe(Gn / 2), qt = Mt * Mt, tn = 1 - nt * Et * Et, xn = tn ? Re(Ae * Et) * De(Te = 1 / tn) : Te = 0, An = 0.5 * (2 * xn * Ae * Mt + Gn / Ce) - $n, On = 0.5 * (xn * Oe + Ke) - zn, Bn = 0.5 * Te * (nt * qt + xn * Ae * Et * Ue) + 0.5 / Ce, qn = Te * (ht * Be / 4 - xn * Oe * Mt), Vn = 0.125 * Te * (Be * Mt - xn * Oe * nt * ht), sr = 0.5 * Te * (Ue * Et + xn * qt * Ae) + 0.5, Un = qn * Vn - sr * Bn, wr = (On * qn - An * sr) / Un, Cr = (An * Vn - On * Bn) / Un;
                Gn -= wr, Ke -= Cr;
              } while ((ne(wr) > ge || ne(Cr) > ge) && --Fe > 0);
              return [Gn, Ke];
            };
          }, 49353: function(G, J, Z) {
            function te() {
              return new ne();
            }
            function ne() {
              this.reset();
            }
            Z.d(J, { A: function() {
              return te;
            } }), ne.prototype = { constructor: ne, reset: function() {
              this.s = this.t = 0;
            }, add: function(oe) {
              ae(ee, oe, this.t), ae(this, ee.s, this.s), this.s ? this.t += ee.t : this.s = ee.t;
            }, valueOf: function() {
              return this.s;
            } };
            var ee = new ne();
            function ae(oe, se, ce) {
              var le = oe.s = se + ce, ue = le - se, de = le - ue;
              oe.t = se - de + (ce - ue);
            }
          }, 43976: function(G, J, Z) {
            Z.d(J, { Ay: function() {
              return ve;
            }, B0: function() {
              return de;
            }, Y7: function() {
              return he;
            } });
            var te, ne, ee, ae, oe, se = Z(49353), ce = Z(61323), le = Z(53341), ue = Z(20465), de = (0, se.A)(), fe = (0, se.A)(), he = { point: le.A, lineStart: le.A, lineEnd: le.A, polygonStart: function() {
              de.reset(), he.lineStart = me, he.lineEnd = pe;
            }, polygonEnd: function() {
              var be = +de;
              fe.add(be < 0 ? ce.FA + be : be), this.lineStart = this.lineEnd = this.point = le.A;
            }, sphere: function() {
              fe.add(ce.FA);
            } };
            function me() {
              he.point = ye;
            }
            function pe() {
              ge(te, ne);
            }
            function ye(be, Ce) {
              he.point = ge, te = be, ne = Ce, be *= ce.F2, Ce *= ce.F2, ee = be, ae = (0, ce.gn)(Ce = Ce / 2 + ce.gz), oe = (0, ce.F8)(Ce);
            }
            function ge(be, Ce) {
              be *= ce.F2, Ce = (Ce *= ce.F2) / 2 + ce.gz;
              var Se = be - ee, we = Se >= 0 ? 1 : -1, Ee = we * Se, xe = (0, ce.gn)(Ce), Le = (0, ce.F8)(Ce), Qe = oe * Le, ke = ae * xe + Qe * (0, ce.gn)(Ee), Me = Qe * we * (0, ce.F8)(Ee);
              de.add((0, ce.FP)(Me, ke)), ee = be, ae = xe, oe = Le;
            }
            function ve(be) {
              return fe.reset(), (0, ue.A)(be, he), 2 * fe;
            }
          }, 43212: function(G, J, Z) {
            Z.d(J, { A: function() {
              return Re;
            } });
            var te, ne, ee, ae, oe, se, ce, le, ue, de, fe = Z(49353), he = Z(43976), me = Z(20375), pe = Z(61323), ye = Z(20465), ge = (0, fe.A)(), ve = { point: be, lineStart: Se, lineEnd: we, polygonStart: function() {
              ve.point = Ee, ve.lineStart = xe, ve.lineEnd = Le, ge.reset(), he.Y7.polygonStart();
            }, polygonEnd: function() {
              he.Y7.polygonEnd(), ve.point = be, ve.lineStart = Se, ve.lineEnd = we, he.B0 < 0 ? (te = -(ee = 180), ne = -(ae = 90)) : ge > pe.Ni ? ae = 90 : ge < -pe.Ni && (ne = -90), de[0] = te, de[1] = ee;
            }, sphere: function() {
              te = -(ee = 180), ne = -(ae = 90);
            } };
            function be(De, Pe) {
              ue.push(de = [te = De, ee = De]), Pe < ne && (ne = Pe), Pe > ae && (ae = Pe);
            }
            function Ce(De, Pe) {
              var $e = (0, me.jf)([De * pe.F2, Pe * pe.F2]);
              if (le) {
                var qe = (0, me.r8)(le, $e), Xe = [qe[1], -qe[0], 0], Ve = (0, me.r8)(Xe, qe);
                (0, me.Cx)(Ve), Ve = (0, me.EV)(Ve);
                var Ze, Ge = De - oe, ze = Ge > 0 ? 1 : -1, We = Ve[0] * pe.uj * ze, at = (0, pe.tn)(Ge) > 180;
                at ^ (ze * oe < We && We < ze * De) ? (Ze = Ve[1] * pe.uj) > ae && (ae = Ze) : at ^ (ze * oe < (We = (We + 360) % 360 - 180) && We < ze * De) ? (Ze = -Ve[1] * pe.uj) < ne && (ne = Ze) : (Pe < ne && (ne = Pe), Pe > ae && (ae = Pe)), at ? De < oe ? Qe(te, De) > Qe(te, ee) && (ee = De) : Qe(De, ee) > Qe(te, ee) && (te = De) : ee >= te ? (De < te && (te = De), De > ee && (ee = De)) : De > oe ? Qe(te, De) > Qe(te, ee) && (ee = De) : Qe(De, ee) > Qe(te, ee) && (te = De);
              } else ue.push(de = [te = De, ee = De]);
              Pe < ne && (ne = Pe), Pe > ae && (ae = Pe), le = $e, oe = De;
            }
            function Se() {
              ve.point = Ce;
            }
            function we() {
              de[0] = te, de[1] = ee, ve.point = be, le = null;
            }
            function Ee(De, Pe) {
              if (le) {
                var $e = De - oe;
                ge.add((0, pe.tn)($e) > 180 ? $e + ($e > 0 ? 360 : -360) : $e);
              } else se = De, ce = Pe;
              he.Y7.point(De, Pe), Ce(De, Pe);
            }
            function xe() {
              he.Y7.lineStart();
            }
            function Le() {
              Ee(se, ce), he.Y7.lineEnd(), (0, pe.tn)(ge) > pe.Ni && (te = -(ee = 180)), de[0] = te, de[1] = ee, le = null;
            }
            function Qe(De, Pe) {
              return (Pe -= De) < 0 ? Pe + 360 : Pe;
            }
            function ke(De, Pe) {
              return De[0] - Pe[0];
            }
            function Me(De, Pe) {
              return De[0] <= De[1] ? De[0] <= Pe && Pe <= De[1] : Pe < De[0] || De[1] < Pe;
            }
            function Re(De) {
              var Pe, $e, qe, Xe, Ve, Ze, Ge;
              if (ae = ee = -(te = ne = 1 / 0), ue = [], (0, ye.A)(De, ve), $e = ue.length) {
                for (ue.sort(ke), Pe = 1, Ve = [qe = ue[0]]; Pe < $e; ++Pe) Me(qe, (Xe = ue[Pe])[0]) || Me(qe, Xe[1]) ? (Qe(qe[0], Xe[1]) > Qe(qe[0], qe[1]) && (qe[1] = Xe[1]), Qe(Xe[0], qe[1]) > Qe(qe[0], qe[1]) && (qe[0] = Xe[0])) : Ve.push(qe = Xe);
                for (Ze = -1 / 0, Pe = 0, qe = Ve[$e = Ve.length - 1]; Pe <= $e; qe = Xe, ++Pe) Xe = Ve[Pe], (Ge = Qe(qe[1], Xe[0])) > Ze && (Ze = Ge, te = Xe[0], ee = qe[1]);
              }
              return ue = de = null, te === 1 / 0 || ne === 1 / 0 ? [[NaN, NaN], [NaN, NaN]] : [[te, ne], [ee, ae]];
            }
          }, 20375: function(G, J, Z) {
            Z.d(J, { Cx: function() {
              return le;
            }, EV: function() {
              return ne;
            }, W8: function() {
              return ae;
            }, ep: function() {
              return se;
            }, jf: function() {
              return ee;
            }, ly: function() {
              return ce;
            }, r8: function() {
              return oe;
            } });
            var te = Z(61323);
            function ne(ue) {
              return [(0, te.FP)(ue[1], ue[0]), (0, te.qR)(ue[2])];
            }
            function ee(ue) {
              var de = ue[0], fe = ue[1], he = (0, te.gn)(fe);
              return [he * (0, te.gn)(de), he * (0, te.F8)(de), (0, te.F8)(fe)];
            }
            function ae(ue, de) {
              return ue[0] * de[0] + ue[1] * de[1] + ue[2] * de[2];
            }
            function oe(ue, de) {
              return [ue[1] * de[2] - ue[2] * de[1], ue[2] * de[0] - ue[0] * de[2], ue[0] * de[1] - ue[1] * de[0]];
            }
            function se(ue, de) {
              ue[0] += de[0], ue[1] += de[1], ue[2] += de[2];
            }
            function ce(ue, de) {
              return [ue[0] * de, ue[1] * de, ue[2] * de];
            }
            function le(ue) {
              var de = (0, te.RZ)(ue[0] * ue[0] + ue[1] * ue[1] + ue[2] * ue[2]);
              ue[0] /= de, ue[1] /= de, ue[2] /= de;
            }
          }, 30021: function(G, J, Z) {
            Z.d(J, { A: function() {
              return $e;
            } });
            var te, ne, ee, ae, oe, se, ce, le, ue, de, fe, he, me, pe, ye, ge, ve = Z(61323), be = Z(53341), Ce = Z(20465), Se = { sphere: be.A, point: we, lineStart: xe, lineEnd: ke, polygonStart: function() {
              Se.lineStart = Me, Se.lineEnd = Re;
            }, polygonEnd: function() {
              Se.lineStart = xe, Se.lineEnd = ke;
            } };
            function we(qe, Xe) {
              qe *= ve.F2, Xe *= ve.F2;
              var Ve = (0, ve.gn)(Xe);
              Ee(Ve * (0, ve.gn)(qe), Ve * (0, ve.F8)(qe), (0, ve.F8)(Xe));
            }
            function Ee(qe, Xe, Ve) {
              ++te, ee += (qe - ee) / te, ae += (Xe - ae) / te, oe += (Ve - oe) / te;
            }
            function xe() {
              Se.point = Le;
            }
            function Le(qe, Xe) {
              qe *= ve.F2, Xe *= ve.F2;
              var Ve = (0, ve.gn)(Xe);
              pe = Ve * (0, ve.gn)(qe), ye = Ve * (0, ve.F8)(qe), ge = (0, ve.F8)(Xe), Se.point = Qe, Ee(pe, ye, ge);
            }
            function Qe(qe, Xe) {
              qe *= ve.F2, Xe *= ve.F2;
              var Ve = (0, ve.gn)(Xe), Ze = Ve * (0, ve.gn)(qe), Ge = Ve * (0, ve.F8)(qe), ze = (0, ve.F8)(Xe), We = (0, ve.FP)((0, ve.RZ)((We = ye * ze - ge * Ge) * We + (We = ge * Ze - pe * ze) * We + (We = pe * Ge - ye * Ze) * We), pe * Ze + ye * Ge + ge * ze);
              ne += We, se += We * (pe + (pe = Ze)), ce += We * (ye + (ye = Ge)), le += We * (ge + (ge = ze)), Ee(pe, ye, ge);
            }
            function ke() {
              Se.point = we;
            }
            function Me() {
              Se.point = De;
            }
            function Re() {
              Pe(he, me), Se.point = we;
            }
            function De(qe, Xe) {
              he = qe, me = Xe, qe *= ve.F2, Xe *= ve.F2, Se.point = Pe;
              var Ve = (0, ve.gn)(Xe);
              pe = Ve * (0, ve.gn)(qe), ye = Ve * (0, ve.F8)(qe), ge = (0, ve.F8)(Xe), Ee(pe, ye, ge);
            }
            function Pe(qe, Xe) {
              qe *= ve.F2, Xe *= ve.F2;
              var Ve = (0, ve.gn)(Xe), Ze = Ve * (0, ve.gn)(qe), Ge = Ve * (0, ve.F8)(qe), ze = (0, ve.F8)(Xe), We = ye * ze - ge * Ge, at = ge * Ze - pe * ze, ct = pe * Ge - ye * Ze, et = (0, ve.RZ)(We * We + at * at + ct * ct), dt = (0, ve.qR)(et), Je = et && -dt / et;
              ue += Je * We, de += Je * at, fe += Je * ct, ne += dt, se += dt * (pe + (pe = Ze)), ce += dt * (ye + (ye = Ge)), le += dt * (ge + (ge = ze)), Ee(pe, ye, ge);
            }
            function $e(qe) {
              te = ne = ee = ae = oe = se = ce = le = ue = de = fe = 0, (0, Ce.A)(qe, Se);
              var Xe = ue, Ve = de, Ze = fe, Ge = Xe * Xe + Ve * Ve + Ze * Ze;
              return Ge < ve.$t && (Xe = se, Ve = ce, Ze = le, ne < ve.Ni && (Xe = ee, Ve = ae, Ze = oe), (Ge = Xe * Xe + Ve * Ve + Ze * Ze) < ve.$t) ? [NaN, NaN] : [(0, ve.FP)(Ve, Xe) * ve.uj, (0, ve.qR)(Ze / (0, ve.RZ)(Ge)) * ve.uj];
            }
          }, 39127: function(G, J, Z) {
            Z.d(J, { J: function() {
              return oe;
            }, A: function() {
              return ce;
            } });
            var te = Z(20375);
            function ne(le) {
              return function() {
                return le;
              };
            }
            var ee = Z(61323), ae = Z(30915);
            function oe(le, ue, de, fe, he, me) {
              if (de) {
                var pe = (0, ee.gn)(ue), ye = (0, ee.F8)(ue), ge = fe * de;
                he == null ? (he = ue + fe * ee.FA, me = ue - ge / 2) : (he = se(pe, he), me = se(pe, me), (fe > 0 ? he < me : he > me) && (he += fe * ee.FA));
                for (var ve, be = he; fe > 0 ? be > me : be < me; be -= ge) ve = (0, te.EV)([pe, -ye * (0, ee.gn)(be), -ye * (0, ee.F8)(be)]), le.point(ve[0], ve[1]);
              }
            }
            function se(le, ue) {
              (ue = (0, te.jf)(ue))[0] -= le, (0, te.Cx)(ue);
              var de = (0, ee.HQ)(-ue[1]);
              return ((-ue[2] < 0 ? -de : de) + ee.FA - ee.Ni) % ee.FA;
            }
            function ce() {
              var le, ue, de = ne([0, 0]), fe = ne(90), he = ne(6), me = { point: function(ye, ge) {
                le.push(ye = ue(ye, ge)), ye[0] *= ee.uj, ye[1] *= ee.uj;
              } };
              function pe() {
                var ye = de.apply(this, arguments), ge = fe.apply(this, arguments) * ee.F2, ve = he.apply(this, arguments) * ee.F2;
                return le = [], ue = (0, ae.y)(-ye[0] * ee.F2, -ye[1] * ee.F2, 0).invert, oe(me, ge, ve, 1), ye = { type: "Polygon", coordinates: [le] }, le = ue = null, ye;
              }
              return pe.center = function(ye) {
                return arguments.length ? (de = typeof ye == "function" ? ye : ne([+ye[0], +ye[1]]), pe) : de;
              }, pe.radius = function(ye) {
                return arguments.length ? (fe = typeof ye == "function" ? ye : ne(+ye), pe) : fe;
              }, pe.precision = function(ye) {
                return arguments.length ? (he = typeof ye == "function" ? ye : ne(+ye), pe) : he;
              }, pe;
            }
          }, 42413: function(G, J, Z) {
            var te = Z(13720), ne = Z(61323);
            J.A = (0, te.A)(function() {
              return !0;
            }, function(ee) {
              var ae, oe = NaN, se = NaN, ce = NaN;
              return { lineStart: function() {
                ee.lineStart(), ae = 1;
              }, point: function(le, ue) {
                var de = le > 0 ? ne.pi : -ne.pi, fe = (0, ne.tn)(le - oe);
                (0, ne.tn)(fe - ne.pi) < ne.Ni ? (ee.point(oe, se = (se + ue) / 2 > 0 ? ne.TW : -ne.TW), ee.point(ce, se), ee.lineEnd(), ee.lineStart(), ee.point(de, se), ee.point(le, se), ae = 0) : ce !== de && fe >= ne.pi && ((0, ne.tn)(oe - ce) < ne.Ni && (oe -= ce * ne.Ni), (0, ne.tn)(le - de) < ne.Ni && (le -= de * ne.Ni), se = function(he, me, pe, ye) {
                  var ge, ve, be = (0, ne.F8)(he - pe);
                  return (0, ne.tn)(be) > ne.Ni ? (0, ne.rY)(((0, ne.F8)(me) * (ve = (0, ne.gn)(ye)) * (0, ne.F8)(pe) - (0, ne.F8)(ye) * (ge = (0, ne.gn)(me)) * (0, ne.F8)(he)) / (ge * ve * be)) : (me + ye) / 2;
                }(oe, se, le, ue), ee.point(ce, se), ee.lineEnd(), ee.lineStart(), ee.point(de, se), ae = 0), ee.point(oe = le, se = ue), ce = de;
              }, lineEnd: function() {
                ee.lineEnd(), oe = se = NaN;
              }, clean: function() {
                return 2 - ae;
              } };
            }, function(ee, ae, oe, se) {
              var ce;
              if (ee == null) ce = oe * ne.TW, se.point(-ne.pi, ce), se.point(0, ce), se.point(ne.pi, ce), se.point(ne.pi, 0), se.point(ne.pi, -ce), se.point(0, -ce), se.point(-ne.pi, -ce), se.point(-ne.pi, 0), se.point(-ne.pi, ce);
              else if ((0, ne.tn)(ee[0] - ae[0]) > ne.Ni) {
                var le = ee[0] < ae[0] ? ne.pi : -ne.pi;
                ce = oe * le / 2, se.point(-le, ce), se.point(0, ce), se.point(le, ce);
              } else se.point(ae[0], ae[1]);
            }, [-ne.pi, -ne.TW]);
          }, 39608: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ne;
            } });
            var te = Z(53341);
            function ne() {
              var ee, ae = [];
              return { point: function(oe, se, ce) {
                ee.push([oe, se, ce]);
              }, lineStart: function() {
                ae.push(ee = []);
              }, lineEnd: te.A, rejoin: function() {
                ae.length > 1 && ae.push(ae.pop().concat(ae.shift()));
              }, result: function() {
                var oe = ae;
                return ae = [], ee = null, oe;
              } };
            }
          }, 47402: function(G, J, Z) {
            Z.d(J, { A: function() {
              return se;
            } });
            var te = Z(20375), ne = Z(39127), ee = Z(61323), ae = Z(28759), oe = Z(13720);
            function se(ce) {
              var le = (0, ee.gn)(ce), ue = 6 * ee.F2, de = le > 0, fe = (0, ee.tn)(le) > ee.Ni;
              function he(ye, ge) {
                return (0, ee.gn)(ye) * (0, ee.gn)(ge) > le;
              }
              function me(ye, ge, ve) {
                var be = (0, te.jf)(ye), Ce = (0, te.jf)(ge), Se = [1, 0, 0], we = (0, te.r8)(be, Ce), Ee = (0, te.W8)(we, we), xe = we[0], Le = Ee - xe * xe;
                if (!Le) return !ve && ye;
                var Qe = le * Ee / Le, ke = -le * xe / Le, Me = (0, te.r8)(Se, we), Re = (0, te.ly)(Se, Qe), De = (0, te.ly)(we, ke);
                (0, te.ep)(Re, De);
                var Pe = Me, $e = (0, te.W8)(Re, Pe), qe = (0, te.W8)(Pe, Pe), Xe = $e * $e - qe * ((0, te.W8)(Re, Re) - 1);
                if (!(Xe < 0)) {
                  var Ve = (0, ee.RZ)(Xe), Ze = (0, te.ly)(Pe, (-$e - Ve) / qe);
                  if ((0, te.ep)(Ze, Re), Ze = (0, te.EV)(Ze), !ve) return Ze;
                  var Ge, ze = ye[0], We = ge[0], at = ye[1], ct = ge[1];
                  We < ze && (Ge = ze, ze = We, We = Ge);
                  var et = We - ze, dt = (0, ee.tn)(et - ee.pi) < ee.Ni;
                  if (!dt && ct < at && (Ge = at, at = ct, ct = Ge), dt || et < ee.Ni ? dt ? at + ct > 0 ^ Ze[1] < ((0, ee.tn)(Ze[0] - ze) < ee.Ni ? at : ct) : at <= Ze[1] && Ze[1] <= ct : et > ee.pi ^ (ze <= Ze[0] && Ze[0] <= We)) {
                    var Je = (0, te.ly)(Pe, (-$e + Ve) / qe);
                    return (0, te.ep)(Je, Re), [Ze, (0, te.EV)(Je)];
                  }
                }
              }
              function pe(ye, ge) {
                var ve = de ? ce : ee.pi - ce, be = 0;
                return ye < -ve ? be |= 1 : ye > ve && (be |= 2), ge < -ve ? be |= 4 : ge > ve && (be |= 8), be;
              }
              return (0, oe.A)(he, function(ye) {
                var ge, ve, be, Ce, Se;
                return { lineStart: function() {
                  Ce = be = !1, Se = 1;
                }, point: function(we, Ee) {
                  var xe, Le = [we, Ee], Qe = he(we, Ee), ke = de ? Qe ? 0 : pe(we, Ee) : Qe ? pe(we + (we < 0 ? ee.pi : -ee.pi), Ee) : 0;
                  if (!ge && (Ce = be = Qe) && ye.lineStart(), Qe !== be && (!(xe = me(ge, Le)) || (0, ae.A)(ge, xe) || (0, ae.A)(Le, xe)) && (Le[2] = 1), Qe !== be) Se = 0, Qe ? (ye.lineStart(), xe = me(Le, ge), ye.point(xe[0], xe[1])) : (xe = me(ge, Le), ye.point(xe[0], xe[1], 2), ye.lineEnd()), ge = xe;
                  else if (fe && ge && de ^ Qe) {
                    var Me;
                    ke & ve || !(Me = me(Le, ge, !0)) || (Se = 0, de ? (ye.lineStart(), ye.point(Me[0][0], Me[0][1]), ye.point(Me[1][0], Me[1][1]), ye.lineEnd()) : (ye.point(Me[1][0], Me[1][1]), ye.lineEnd(), ye.lineStart(), ye.point(Me[0][0], Me[0][1], 3)));
                  }
                  !Qe || ge && (0, ae.A)(ge, Le) || ye.point(Le[0], Le[1]), ge = Le, be = Qe, ve = ke;
                }, lineEnd: function() {
                  be && ye.lineEnd(), ge = null;
                }, clean: function() {
                  return Se | (Ce && be) << 1;
                } };
              }, function(ye, ge, ve, be) {
                (0, ne.J)(be, ce, ue, ve, ye, ge);
              }, de ? [0, -ce] : [-ee.pi, ce - ee.pi]);
            }
          }, 13720: function(G, J, Z) {
            Z.d(J, { A: function() {
              return se;
            } });
            var te = Z(39608), ne = Z(19119), ee = Z(61323), ae = Z(2274), oe = Z(29725);
            function se(ue, de, fe, he) {
              return function(me) {
                var pe, ye, ge, ve = de(me), be = (0, te.A)(), Ce = de(be), Se = !1, we = { point: Ee, lineStart: Le, lineEnd: Qe, polygonStart: function() {
                  we.point = ke, we.lineStart = Me, we.lineEnd = Re, ye = [], pe = [];
                }, polygonEnd: function() {
                  we.point = Ee, we.lineStart = Le, we.lineEnd = Qe, ye = (0, oe.Am)(ye);
                  var De = (0, ae.A)(pe, he);
                  ye.length ? (Se || (me.polygonStart(), Se = !0), (0, ne.A)(ye, le, De, fe, me)) : De && (Se || (me.polygonStart(), Se = !0), me.lineStart(), fe(null, null, 1, me), me.lineEnd()), Se && (me.polygonEnd(), Se = !1), ye = pe = null;
                }, sphere: function() {
                  me.polygonStart(), me.lineStart(), fe(null, null, 1, me), me.lineEnd(), me.polygonEnd();
                } };
                function Ee(De, Pe) {
                  ue(De, Pe) && me.point(De, Pe);
                }
                function xe(De, Pe) {
                  ve.point(De, Pe);
                }
                function Le() {
                  we.point = xe, ve.lineStart();
                }
                function Qe() {
                  we.point = Ee, ve.lineEnd();
                }
                function ke(De, Pe) {
                  ge.push([De, Pe]), Ce.point(De, Pe);
                }
                function Me() {
                  Ce.lineStart(), ge = [];
                }
                function Re() {
                  ke(ge[0][0], ge[0][1]), Ce.lineEnd();
                  var De, Pe, $e, qe, Xe = Ce.clean(), Ve = be.result(), Ze = Ve.length;
                  if (ge.pop(), pe.push(ge), ge = null, Ze) if (1 & Xe) {
                    if ((Pe = ($e = Ve[0]).length - 1) > 0) {
                      for (Se || (me.polygonStart(), Se = !0), me.lineStart(), De = 0; De < Pe; ++De) me.point((qe = $e[De])[0], qe[1]);
                      me.lineEnd();
                    }
                  } else Ze > 1 && 2 & Xe && Ve.push(Ve.pop().concat(Ve.shift())), ye.push(Ve.filter(ce));
                }
                return we;
              };
            }
            function ce(ue) {
              return ue.length > 1;
            }
            function le(ue, de) {
              return ((ue = ue.x)[0] < 0 ? ue[1] - ee.TW - ee.Ni : ee.TW - ue[1]) - ((de = de.x)[0] < 0 ? de[1] - ee.TW - ee.Ni : ee.TW - de[1]);
            }
          }, 21503: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ce;
            } });
            var te = Z(61323), ne = Z(39608), ee = Z(19119), ae = Z(29725), oe = 1e9, se = -1e9;
            function ce(le, ue, de, fe) {
              function he(ve, be) {
                return le <= ve && ve <= de && ue <= be && be <= fe;
              }
              function me(ve, be, Ce, Se) {
                var we = 0, Ee = 0;
                if (ve == null || (we = pe(ve, Ce)) !== (Ee = pe(be, Ce)) || ge(ve, be) < 0 ^ Ce > 0) do
                  Se.point(we === 0 || we === 3 ? le : de, we > 1 ? fe : ue);
                while ((we = (we + Ce + 4) % 4) !== Ee);
                else Se.point(be[0], be[1]);
              }
              function pe(ve, be) {
                return (0, te.tn)(ve[0] - le) < te.Ni ? be > 0 ? 0 : 3 : (0, te.tn)(ve[0] - de) < te.Ni ? be > 0 ? 2 : 1 : (0, te.tn)(ve[1] - ue) < te.Ni ? be > 0 ? 1 : 0 : be > 0 ? 3 : 2;
              }
              function ye(ve, be) {
                return ge(ve.x, be.x);
              }
              function ge(ve, be) {
                var Ce = pe(ve, 1), Se = pe(be, 1);
                return Ce !== Se ? Ce - Se : Ce === 0 ? be[1] - ve[1] : Ce === 1 ? ve[0] - be[0] : Ce === 2 ? ve[1] - be[1] : be[0] - ve[0];
              }
              return function(ve) {
                var be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me, Re, De = ve, Pe = (0, ne.A)(), $e = { point: qe, lineStart: function() {
                  $e.point = Xe, Ce && Ce.push(Se = []), Me = !0, ke = !1, Le = Qe = NaN;
                }, lineEnd: function() {
                  be && (Xe(we, Ee), xe && ke && Pe.rejoin(), be.push(Pe.result())), $e.point = qe, ke && De.lineEnd();
                }, polygonStart: function() {
                  De = Pe, be = [], Ce = [], Re = !0;
                }, polygonEnd: function() {
                  var Ve = function() {
                    for (var ze = 0, We = 0, at = Ce.length; We < at; ++We) for (var ct, et, dt = Ce[We], Je = 1, it = dt.length, st = dt[0], gt = st[0], bt = st[1]; Je < it; ++Je) ct = gt, et = bt, gt = (st = dt[Je])[0], bt = st[1], et <= fe ? bt > fe && (gt - ct) * (fe - et) > (bt - et) * (le - ct) && ++ze : bt <= fe && (gt - ct) * (fe - et) < (bt - et) * (le - ct) && --ze;
                    return ze;
                  }(), Ze = Re && Ve, Ge = (be = (0, ae.Am)(be)).length;
                  (Ze || Ge) && (ve.polygonStart(), Ze && (ve.lineStart(), me(null, null, 1, ve), ve.lineEnd()), Ge && (0, ee.A)(be, ye, Ve, me, ve), ve.polygonEnd()), De = ve, be = Ce = Se = null;
                } };
                function qe(Ve, Ze) {
                  he(Ve, Ze) && De.point(Ve, Ze);
                }
                function Xe(Ve, Ze) {
                  var Ge = he(Ve, Ze);
                  if (Ce && Se.push([Ve, Ze]), Me) we = Ve, Ee = Ze, xe = Ge, Me = !1, Ge && (De.lineStart(), De.point(Ve, Ze));
                  else if (Ge && ke) De.point(Ve, Ze);
                  else {
                    var ze = [Le = Math.max(se, Math.min(oe, Le)), Qe = Math.max(se, Math.min(oe, Qe))], We = [Ve = Math.max(se, Math.min(oe, Ve)), Ze = Math.max(se, Math.min(oe, Ze))];
                    (function(at, ct, et, dt, Je, it) {
                      var st, gt = at[0], bt = at[1], yt = 0, St = 1, Lt = ct[0] - gt, Rt = ct[1] - bt;
                      if (st = et - gt, Lt || !(st > 0)) {
                        if (st /= Lt, Lt < 0) {
                          if (st < yt) return;
                          st < St && (St = st);
                        } else if (Lt > 0) {
                          if (st > St) return;
                          st > yt && (yt = st);
                        }
                        if (st = Je - gt, Lt || !(st < 0)) {
                          if (st /= Lt, Lt < 0) {
                            if (st > St) return;
                            st > yt && (yt = st);
                          } else if (Lt > 0) {
                            if (st < yt) return;
                            st < St && (St = st);
                          }
                          if (st = dt - bt, Rt || !(st > 0)) {
                            if (st /= Rt, Rt < 0) {
                              if (st < yt) return;
                              st < St && (St = st);
                            } else if (Rt > 0) {
                              if (st > St) return;
                              st > yt && (yt = st);
                            }
                            if (st = it - bt, Rt || !(st < 0)) {
                              if (st /= Rt, Rt < 0) {
                                if (st > St) return;
                                st > yt && (yt = st);
                              } else if (Rt > 0) {
                                if (st < yt) return;
                                st < St && (St = st);
                              }
                              return yt > 0 && (at[0] = gt + yt * Lt, at[1] = bt + yt * Rt), St < 1 && (ct[0] = gt + St * Lt, ct[1] = bt + St * Rt), !0;
                            }
                          }
                        }
                      }
                    })(ze, We, le, ue, de, fe) ? (ke || (De.lineStart(), De.point(ze[0], ze[1])), De.point(We[0], We[1]), Ge || De.lineEnd(), Re = !1) : Ge && (De.lineStart(), De.point(Ve, Ze), Re = !1);
                  }
                  Le = Ve, Qe = Ze, ke = Ge;
                }
                return $e;
              };
            }
          }, 19119: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ae;
            } });
            var te = Z(28759), ne = Z(61323);
            function ee(se, ce, le, ue) {
              this.x = se, this.z = ce, this.o = le, this.e = ue, this.v = !1, this.n = this.p = null;
            }
            function ae(se, ce, le, ue, de) {
              var fe, he, me = [], pe = [];
              if (se.forEach(function(Se) {
                if (!((we = Se.length - 1) <= 0)) {
                  var we, Ee, xe = Se[0], Le = Se[we];
                  if ((0, te.A)(xe, Le)) {
                    if (!xe[2] && !Le[2]) {
                      for (de.lineStart(), fe = 0; fe < we; ++fe) de.point((xe = Se[fe])[0], xe[1]);
                      return void de.lineEnd();
                    }
                    Le[0] += 2 * ne.Ni;
                  }
                  me.push(Ee = new ee(xe, Se, null, !0)), pe.push(Ee.o = new ee(xe, null, Ee, !1)), me.push(Ee = new ee(Le, Se, null, !1)), pe.push(Ee.o = new ee(Le, null, Ee, !0));
                }
              }), me.length) {
                for (pe.sort(ce), oe(me), oe(pe), fe = 0, he = pe.length; fe < he; ++fe) pe[fe].e = le = !le;
                for (var ye, ge, ve = me[0]; ; ) {
                  for (var be = ve, Ce = !0; be.v; ) if ((be = be.n) === ve) return;
                  ye = be.z, de.lineStart();
                  do {
                    if (be.v = be.o.v = !0, be.e) {
                      if (Ce) for (fe = 0, he = ye.length; fe < he; ++fe) de.point((ge = ye[fe])[0], ge[1]);
                      else ue(be.x, be.n.x, 1, de);
                      be = be.n;
                    } else {
                      if (Ce) for (ye = be.p.z, fe = ye.length - 1; fe >= 0; --fe) de.point((ge = ye[fe])[0], ge[1]);
                      else ue(be.x, be.p.x, -1, de);
                      be = be.p;
                    }
                    ye = (be = be.o).z, Ce = !Ce;
                  } while (!be.v);
                  de.lineEnd();
                }
              }
            }
            function oe(se) {
              if (ce = se.length) {
                for (var ce, le, ue = 0, de = se[0]; ++ue < ce; ) de.n = le = se[ue], le.p = de, de = le;
                de.n = le = se[0], le.p = de;
              }
            }
          }, 19057: function(G, J, Z) {
            function te(ne, ee) {
              function ae(oe, se) {
                return oe = ne(oe, se), ee(oe[0], oe[1]);
              }
              return ne.invert && ee.invert && (ae.invert = function(oe, se) {
                return (oe = ee.invert(oe, se)) && ne.invert(oe[0], oe[1]);
              }), ae;
            }
            Z.d(J, { A: function() {
              return te;
            } });
          }, 26827: function(G, J, Z) {
            function te(ne) {
              return ne;
            }
            Z.d(J, { A: function() {
              return te;
            } });
          }, 70884: function(G, J, Z) {
            Z.r(J), Z.d(J, { geoAlbers: function() {
              return In;
            }, geoAlbersUsa: function() {
              return er;
            }, geoArea: function() {
              return te.Ay;
            }, geoAzimuthalEqualArea: function() {
              return jn.A;
            }, geoAzimuthalEqualAreaRaw: function() {
              return jn.n;
            }, geoAzimuthalEquidistant: function() {
              return ur.A;
            }, geoAzimuthalEquidistantRaw: function() {
              return ur.j;
            }, geoBounds: function() {
              return ne.A;
            }, geoCentroid: function() {
              return ee.A;
            }, geoCircle: function() {
              return ae.A;
            }, geoClipAntimeridian: function() {
              return oe.A;
            }, geoClipCircle: function() {
              return se.A;
            }, geoClipExtent: function() {
              return le;
            }, geoClipRectangle: function() {
              return ce.A;
            }, geoConicConformal: function() {
              return jr;
            }, geoConicConformalRaw: function() {
              return Yr;
            }, geoConicEqualArea: function() {
              return Ln;
            }, geoConicEqualAreaRaw: function() {
              return Mn;
            }, geoConicEquidistant: function() {
              return Bi;
            }, geoConicEquidistantRaw: function() {
              return Wr;
            }, geoContains: function() {
              return Ve;
            }, geoDistance: function() {
              return Qe;
            }, geoEqualEarth: function() {
              return $i;
            }, geoEqualEarthRaw: function() {
              return Zi;
            }, geoEquirectangular: function() {
              return Mr.A;
            }, geoEquirectangularRaw: function() {
              return Mr.f;
            }, geoGnomonic: function() {
              return Br.A;
            }, geoGnomonicRaw: function() {
              return Br.T;
            }, geoGraticule: function() {
              return We;
            }, geoGraticule10: function() {
              return at;
            }, geoIdentity: function() {
              return ei;
            }, geoInterpolate: function() {
              return it.A;
            }, geoLength: function() {
              return Ee;
            }, geoMercator: function() {
              return Lr;
            }, geoMercatorRaw: function() {
              return Jn;
            }, geoNaturalEarth1: function() {
              return Ci.A;
            }, geoNaturalEarth1Raw: function() {
              return Ci.P;
            }, geoOrthographic: function() {
              return oa.A;
            }, geoOrthographicRaw: function() {
              return oa.x;
            }, geoPath: function() {
              return bn;
            }, geoProjection: function() {
              return an.A;
            }, geoProjectionMutator: function() {
              return an.U;
            }, geoRotation: function() {
              return Kn.A;
            }, geoStereographic: function() {
              return hi;
            }, geoStereographicRaw: function() {
              return aa;
            }, geoStream: function() {
              return ge.A;
            }, geoTransform: function() {
              return Pr.A;
            }, geoTransverseMercator: function() {
              return Ca;
            }, geoTransverseMercatorRaw: function() {
              return Gr;
            } });
            var te = Z(43976), ne = Z(43212), ee = Z(30021), ae = Z(39127), oe = Z(42413), se = Z(47402), ce = Z(21503);
            function le() {
              var Sr, Dr, si, zr = 0, di = 0, Da = 960, la = 500;
              return si = { stream: function(wa) {
                return Sr && Dr === wa ? Sr : Sr = (0, ce.A)(zr, di, Da, la)(Dr = wa);
              }, extent: function(wa) {
                return arguments.length ? (zr = +wa[0][0], di = +wa[0][1], Da = +wa[1][0], la = +wa[1][1], Sr = Dr = null, si) : [[zr, di], [Da, la]];
              } };
            }
            var ue, de, fe, he = Z(2274), me = Z(49353), pe = Z(61323), ye = Z(53341), ge = Z(20465), ve = (0, me.A)(), be = { sphere: ye.A, point: ye.A, lineStart: function() {
              be.point = Se, be.lineEnd = Ce;
            }, lineEnd: ye.A, polygonStart: ye.A, polygonEnd: ye.A };
            function Ce() {
              be.point = be.lineEnd = ye.A;
            }
            function Se(Sr, Dr) {
              Sr *= pe.F2, Dr *= pe.F2, ue = Sr, de = (0, pe.F8)(Dr), fe = (0, pe.gn)(Dr), be.point = we;
            }
            function we(Sr, Dr) {
              Sr *= pe.F2, Dr *= pe.F2;
              var si = (0, pe.F8)(Dr), zr = (0, pe.gn)(Dr), di = (0, pe.tn)(Sr - ue), Da = (0, pe.gn)(di), la = zr * (0, pe.F8)(di), wa = fe * si - de * zr * Da, Wa = de * si + fe * zr * Da;
              ve.add((0, pe.FP)((0, pe.RZ)(la * la + wa * wa), Wa)), ue = Sr, de = si, fe = zr;
            }
            function Ee(Sr) {
              return ve.reset(), (0, ge.A)(Sr, be), +ve;
            }
            var xe = [null, null], Le = { type: "LineString", coordinates: xe };
            function Qe(Sr, Dr) {
              return xe[0] = Sr, xe[1] = Dr, Ee(Le);
            }
            var ke = { Feature: function(Sr, Dr) {
              return Re(Sr.geometry, Dr);
            }, FeatureCollection: function(Sr, Dr) {
              for (var si = Sr.features, zr = -1, di = si.length; ++zr < di; ) if (Re(si[zr].geometry, Dr)) return !0;
              return !1;
            } }, Me = { Sphere: function() {
              return !0;
            }, Point: function(Sr, Dr) {
              return De(Sr.coordinates, Dr);
            }, MultiPoint: function(Sr, Dr) {
              for (var si = Sr.coordinates, zr = -1, di = si.length; ++zr < di; ) if (De(si[zr], Dr)) return !0;
              return !1;
            }, LineString: function(Sr, Dr) {
              return Pe(Sr.coordinates, Dr);
            }, MultiLineString: function(Sr, Dr) {
              for (var si = Sr.coordinates, zr = -1, di = si.length; ++zr < di; ) if (Pe(si[zr], Dr)) return !0;
              return !1;
            }, Polygon: function(Sr, Dr) {
              return $e(Sr.coordinates, Dr);
            }, MultiPolygon: function(Sr, Dr) {
              for (var si = Sr.coordinates, zr = -1, di = si.length; ++zr < di; ) if ($e(si[zr], Dr)) return !0;
              return !1;
            }, GeometryCollection: function(Sr, Dr) {
              for (var si = Sr.geometries, zr = -1, di = si.length; ++zr < di; ) if (Re(si[zr], Dr)) return !0;
              return !1;
            } };
            function Re(Sr, Dr) {
              return !(!Sr || !Me.hasOwnProperty(Sr.type)) && Me[Sr.type](Sr, Dr);
            }
            function De(Sr, Dr) {
              return Qe(Sr, Dr) === 0;
            }
            function Pe(Sr, Dr) {
              for (var si, zr, di, Da = 0, la = Sr.length; Da < la; Da++) {
                if ((zr = Qe(Sr[Da], Dr)) === 0 || Da > 0 && (di = Qe(Sr[Da], Sr[Da - 1])) > 0 && si <= di && zr <= di && (si + zr - di) * (1 - Math.pow((si - zr) / di, 2)) < pe.$t * di) return !0;
                si = zr;
              }
              return !1;
            }
            function $e(Sr, Dr) {
              return !!(0, he.A)(Sr.map(qe), Xe(Dr));
            }
            function qe(Sr) {
              return (Sr = Sr.map(Xe)).pop(), Sr;
            }
            function Xe(Sr) {
              return [Sr[0] * pe.F2, Sr[1] * pe.F2];
            }
            function Ve(Sr, Dr) {
              return (Sr && ke.hasOwnProperty(Sr.type) ? ke[Sr.type] : Re)(Sr, Dr);
            }
            var Ze = Z(29725);
            function Ge(Sr, Dr, si) {
              var zr = (0, Ze.y1)(Sr, Dr - pe.Ni, si).concat(Dr);
              return function(di) {
                return zr.map(function(Da) {
                  return [di, Da];
                });
              };
            }
            function ze(Sr, Dr, si) {
              var zr = (0, Ze.y1)(Sr, Dr - pe.Ni, si).concat(Dr);
              return function(di) {
                return zr.map(function(Da) {
                  return [Da, di];
                });
              };
            }
            function We() {
              var Sr, Dr, si, zr, di, Da, la, wa, Wa, Qr, Ur, rr, vr = 10, Rr = vr, Vr = 90, Xr = 360, Jr = 2.5;
              function ti() {
                return { type: "MultiLineString", coordinates: Hr() };
              }
              function Hr() {
                return (0, Ze.y1)((0, pe.mk)(zr / Vr) * Vr, si, Vr).map(Ur).concat((0, Ze.y1)((0, pe.mk)(wa / Xr) * Xr, la, Xr).map(rr)).concat((0, Ze.y1)((0, pe.mk)(Dr / vr) * vr, Sr, vr).filter(function(Er) {
                  return (0, pe.tn)(Er % Vr) > pe.Ni;
                }).map(Wa)).concat((0, Ze.y1)((0, pe.mk)(Da / Rr) * Rr, di, Rr).filter(function(Er) {
                  return (0, pe.tn)(Er % Xr) > pe.Ni;
                }).map(Qr));
              }
              return ti.lines = function() {
                return Hr().map(function(Er) {
                  return { type: "LineString", coordinates: Er };
                });
              }, ti.outline = function() {
                return { type: "Polygon", coordinates: [Ur(zr).concat(rr(la).slice(1), Ur(si).reverse().slice(1), rr(wa).reverse().slice(1))] };
              }, ti.extent = function(Er) {
                return arguments.length ? ti.extentMajor(Er).extentMinor(Er) : ti.extentMinor();
              }, ti.extentMajor = function(Er) {
                return arguments.length ? (zr = +Er[0][0], si = +Er[1][0], wa = +Er[0][1], la = +Er[1][1], zr > si && (Er = zr, zr = si, si = Er), wa > la && (Er = wa, wa = la, la = Er), ti.precision(Jr)) : [[zr, wa], [si, la]];
              }, ti.extentMinor = function(Er) {
                return arguments.length ? (Dr = +Er[0][0], Sr = +Er[1][0], Da = +Er[0][1], di = +Er[1][1], Dr > Sr && (Er = Dr, Dr = Sr, Sr = Er), Da > di && (Er = Da, Da = di, di = Er), ti.precision(Jr)) : [[Dr, Da], [Sr, di]];
              }, ti.step = function(Er) {
                return arguments.length ? ti.stepMajor(Er).stepMinor(Er) : ti.stepMinor();
              }, ti.stepMajor = function(Er) {
                return arguments.length ? (Vr = +Er[0], Xr = +Er[1], ti) : [Vr, Xr];
              }, ti.stepMinor = function(Er) {
                return arguments.length ? (vr = +Er[0], Rr = +Er[1], ti) : [vr, Rr];
              }, ti.precision = function(Er) {
                return arguments.length ? (Jr = +Er, Wa = Ge(Da, di, 90), Qr = ze(Dr, Sr, Jr), Ur = Ge(wa, la, 90), rr = ze(zr, si, Jr), ti) : Jr;
              }, ti.extentMajor([[-180, -90 + pe.Ni], [180, 90 - pe.Ni]]).extentMinor([[-180, -80 - pe.Ni], [180, 80 + pe.Ni]]);
            }
            function at() {
              return We()();
            }
            var ct, et, dt, Je, it = Z(81758), st = Z(26827), gt = (0, me.A)(), bt = (0, me.A)(), yt = { point: ye.A, lineStart: ye.A, lineEnd: ye.A, polygonStart: function() {
              yt.lineStart = St, yt.lineEnd = vt;
            }, polygonEnd: function() {
              yt.lineStart = yt.lineEnd = yt.point = ye.A, gt.add((0, pe.tn)(bt)), bt.reset();
            }, result: function() {
              var Sr = gt / 2;
              return gt.reset(), Sr;
            } };
            function St() {
              yt.point = Lt;
            }
            function Lt(Sr, Dr) {
              yt.point = Rt, ct = dt = Sr, et = Je = Dr;
            }
            function Rt(Sr, Dr) {
              bt.add(Je * Sr - dt * Dr), dt = Sr, Je = Dr;
            }
            function vt() {
              Rt(ct, et);
            }
            var $t, Pt, Xt, Nt, Gt = yt, Ft = Z(33028), Tt = 0, It = 0, Ct = 0, Kt = 0, Vt = 0, Yt = 0, sn = 0, hn = 0, Qt = 0, Ut = { point: Wt, lineStart: rn, lineEnd: Ht, polygonStart: function() {
              Ut.lineStart = jt, Ut.lineEnd = cn;
            }, polygonEnd: function() {
              Ut.point = Wt, Ut.lineStart = rn, Ut.lineEnd = Ht;
            }, result: function() {
              var Sr = Qt ? [sn / Qt, hn / Qt] : Yt ? [Kt / Yt, Vt / Yt] : Ct ? [Tt / Ct, It / Ct] : [NaN, NaN];
              return Tt = It = Ct = Kt = Vt = Yt = sn = hn = Qt = 0, Sr;
            } };
            function Wt(Sr, Dr) {
              Tt += Sr, It += Dr, ++Ct;
            }
            function rn() {
              Ut.point = vn;
            }
            function vn(Sr, Dr) {
              Ut.point = dn, Wt(Xt = Sr, Nt = Dr);
            }
            function dn(Sr, Dr) {
              var si = Sr - Xt, zr = Dr - Nt, di = (0, pe.RZ)(si * si + zr * zr);
              Kt += di * (Xt + Sr) / 2, Vt += di * (Nt + Dr) / 2, Yt += di, Wt(Xt = Sr, Nt = Dr);
            }
            function Ht() {
              Ut.point = Wt;
            }
            function jt() {
              Ut.point = wn;
            }
            function cn() {
              Rn($t, Pt);
            }
            function wn(Sr, Dr) {
              Ut.point = Rn, Wt($t = Xt = Sr, Pt = Nt = Dr);
            }
            function Rn(Sr, Dr) {
              var si = Sr - Xt, zr = Dr - Nt, di = (0, pe.RZ)(si * si + zr * zr);
              Kt += di * (Xt + Sr) / 2, Vt += di * (Nt + Dr) / 2, Yt += di, sn += (di = Nt * Sr - Xt * Dr) * (Xt + Sr), hn += di * (Nt + Dr), Qt += 3 * di, Wt(Xt = Sr, Nt = Dr);
            }
            var Pn = Ut;
            function Ie(Sr) {
              this._context = Sr;
            }
            Ie.prototype = { _radius: 4.5, pointRadius: function(Sr) {
              return this._radius = Sr, this;
            }, polygonStart: function() {
              this._line = 0;
            }, polygonEnd: function() {
              this._line = NaN;
            }, lineStart: function() {
              this._point = 0;
            }, lineEnd: function() {
              this._line === 0 && this._context.closePath(), this._point = NaN;
            }, point: function(Sr, Dr) {
              switch (this._point) {
                case 0:
                  this._context.moveTo(Sr, Dr), this._point = 1;
                  break;
                case 1:
                  this._context.lineTo(Sr, Dr);
                  break;
                default:
                  this._context.moveTo(Sr + this._radius, Dr), this._context.arc(Sr, Dr, this._radius, 0, pe.FA);
              }
            }, result: ye.A };
            var rt, ft, pt, wt, Bt, At = (0, me.A)(), Dt = { point: ye.A, lineStart: function() {
              Dt.point = Ot;
            }, lineEnd: function() {
              rt && un(ft, pt), Dt.point = ye.A;
            }, polygonStart: function() {
              rt = !0;
            }, polygonEnd: function() {
              rt = null;
            }, result: function() {
              var Sr = +At;
              return At.reset(), Sr;
            } };
            function Ot(Sr, Dr) {
              Dt.point = un, ft = wt = Sr, pt = Bt = Dr;
            }
            function un(Sr, Dr) {
              wt -= Sr, Bt -= Dr, At.add((0, pe.RZ)(wt * wt + Bt * Bt)), wt = Sr, Bt = Dr;
            }
            var yn = Dt;
            function _n() {
              this._string = [];
            }
            function gn(Sr) {
              return "m0," + Sr + "a" + Sr + "," + Sr + " 0 1,1 0," + -2 * Sr + "a" + Sr + "," + Sr + " 0 1,1 0," + 2 * Sr + "z";
            }
            function bn(Sr, Dr) {
              var si, zr, di = 4.5;
              function Da(la) {
                return la && (typeof di == "function" && zr.pointRadius(+di.apply(this, arguments)), (0, ge.A)(la, si(zr))), zr.result();
              }
              return Da.area = function(la) {
                return (0, ge.A)(la, si(Gt)), Gt.result();
              }, Da.measure = function(la) {
                return (0, ge.A)(la, si(yn)), yn.result();
              }, Da.bounds = function(la) {
                return (0, ge.A)(la, si(Ft.A)), Ft.A.result();
              }, Da.centroid = function(la) {
                return (0, ge.A)(la, si(Pn)), Pn.result();
              }, Da.projection = function(la) {
                return arguments.length ? (si = la == null ? (Sr = null, st.A) : (Sr = la).stream, Da) : Sr;
              }, Da.context = function(la) {
                return arguments.length ? (zr = la == null ? (Dr = null, new _n()) : new Ie(Dr = la), typeof di != "function" && zr.pointRadius(di), Da) : Dr;
              }, Da.pointRadius = function(la) {
                return arguments.length ? (di = typeof la == "function" ? la : (zr.pointRadius(+la), +la), Da) : di;
              }, Da.projection(Sr).context(Dr);
            }
            _n.prototype = { _radius: 4.5, _circle: gn(4.5), pointRadius: function(Sr) {
              return (Sr = +Sr) !== this._radius && (this._radius = Sr, this._circle = null), this;
            }, polygonStart: function() {
              this._line = 0;
            }, polygonEnd: function() {
              this._line = NaN;
            }, lineStart: function() {
              this._point = 0;
            }, lineEnd: function() {
              this._line === 0 && this._string.push("Z"), this._point = NaN;
            }, point: function(Sr, Dr) {
              switch (this._point) {
                case 0:
                  this._string.push("M", Sr, ",", Dr), this._point = 1;
                  break;
                case 1:
                  this._string.push("L", Sr, ",", Dr);
                  break;
                default:
                  this._circle == null && (this._circle = gn(this._radius)), this._string.push("M", Sr, ",", Dr, this._circle);
              }
            }, result: function() {
              if (this._string.length) {
                var Sr = this._string.join("");
                return this._string = [], Sr;
              }
              return null;
            } };
            var an = Z(94684);
            function Zt(Sr) {
              var Dr = 0, si = pe.pi / 3, zr = (0, an.U)(Sr), di = zr(Dr, si);
              return di.parallels = function(Da) {
                return arguments.length ? zr(Dr = Da[0] * pe.F2, si = Da[1] * pe.F2) : [Dr * pe.uj, si * pe.uj];
              }, di;
            }
            function Mn(Sr, Dr) {
              var si = (0, pe.F8)(Sr), zr = (si + (0, pe.F8)(Dr)) / 2;
              if ((0, pe.tn)(zr) < pe.Ni) return function(wa) {
                var Wa = (0, pe.gn)(wa);
                function Qr(Ur, rr) {
                  return [Ur * Wa, (0, pe.F8)(rr) / Wa];
                }
                return Qr.invert = function(Ur, rr) {
                  return [Ur / Wa, (0, pe.qR)(rr * Wa)];
                }, Qr;
              }(Sr);
              var di = 1 + si * (2 * zr - si), Da = (0, pe.RZ)(di) / zr;
              function la(wa, Wa) {
                var Qr = (0, pe.RZ)(di - 2 * zr * (0, pe.F8)(Wa)) / zr;
                return [Qr * (0, pe.F8)(wa *= zr), Da - Qr * (0, pe.gn)(wa)];
              }
              return la.invert = function(wa, Wa) {
                var Qr = Da - Wa, Ur = (0, pe.FP)(wa, (0, pe.tn)(Qr)) * (0, pe._S)(Qr);
                return Qr * zr < 0 && (Ur -= pe.pi * (0, pe._S)(wa) * (0, pe._S)(Qr)), [Ur / zr, (0, pe.qR)((di - (wa * wa + Qr * Qr) * zr * zr) / (2 * zr))];
              }, la;
            }
            function Ln() {
              return Zt(Mn).scale(155.424).center([0, 33.6442]);
            }
            function In() {
              return Ln().parallels([29.5, 45.5]).scale(1070).translate([480, 250]).rotate([96, 0]).center([-0.6, 38.7]);
            }
            var kn = Z(7944);
            function er() {
              var Sr, Dr, si, zr, di, Da, la = In(), wa = Ln().rotate([154, 0]).center([-2, 58.5]).parallels([55, 65]), Wa = Ln().rotate([157, 0]).center([-3, 19.9]).parallels([8, 18]), Qr = { point: function(vr, Rr) {
                Da = [vr, Rr];
              } };
              function Ur(vr) {
                var Rr = vr[0], Vr = vr[1];
                return Da = null, si.point(Rr, Vr), Da || (zr.point(Rr, Vr), Da) || (di.point(Rr, Vr), Da);
              }
              function rr() {
                return Sr = Dr = null, Ur;
              }
              return Ur.invert = function(vr) {
                var Rr = la.scale(), Vr = la.translate(), Xr = (vr[0] - Vr[0]) / Rr, Jr = (vr[1] - Vr[1]) / Rr;
                return (Jr >= 0.12 && Jr < 0.234 && Xr >= -0.425 && Xr < -0.214 ? wa : Jr >= 0.166 && Jr < 0.234 && Xr >= -0.214 && Xr < -0.115 ? Wa : la).invert(vr);
              }, Ur.stream = function(vr) {
                return Sr && Dr === vr ? Sr : (Rr = [la.stream(Dr = vr), wa.stream(vr), Wa.stream(vr)], Vr = Rr.length, Sr = { point: function(Xr, Jr) {
                  for (var ti = -1; ++ti < Vr; ) Rr[ti].point(Xr, Jr);
                }, sphere: function() {
                  for (var Xr = -1; ++Xr < Vr; ) Rr[Xr].sphere();
                }, lineStart: function() {
                  for (var Xr = -1; ++Xr < Vr; ) Rr[Xr].lineStart();
                }, lineEnd: function() {
                  for (var Xr = -1; ++Xr < Vr; ) Rr[Xr].lineEnd();
                }, polygonStart: function() {
                  for (var Xr = -1; ++Xr < Vr; ) Rr[Xr].polygonStart();
                }, polygonEnd: function() {
                  for (var Xr = -1; ++Xr < Vr; ) Rr[Xr].polygonEnd();
                } });
                var Rr, Vr;
              }, Ur.precision = function(vr) {
                return arguments.length ? (la.precision(vr), wa.precision(vr), Wa.precision(vr), rr()) : la.precision();
              }, Ur.scale = function(vr) {
                return arguments.length ? (la.scale(vr), wa.scale(0.35 * vr), Wa.scale(vr), Ur.translate(la.translate())) : la.scale();
              }, Ur.translate = function(vr) {
                if (!arguments.length) return la.translate();
                var Rr = la.scale(), Vr = +vr[0], Xr = +vr[1];
                return si = la.translate(vr).clipExtent([[Vr - 0.455 * Rr, Xr - 0.238 * Rr], [Vr + 0.455 * Rr, Xr + 0.238 * Rr]]).stream(Qr), zr = wa.translate([Vr - 0.307 * Rr, Xr + 0.201 * Rr]).clipExtent([[Vr - 0.425 * Rr + pe.Ni, Xr + 0.12 * Rr + pe.Ni], [Vr - 0.214 * Rr - pe.Ni, Xr + 0.234 * Rr - pe.Ni]]).stream(Qr), di = Wa.translate([Vr - 0.205 * Rr, Xr + 0.212 * Rr]).clipExtent([[Vr - 0.214 * Rr + pe.Ni, Xr + 0.166 * Rr + pe.Ni], [Vr - 0.115 * Rr - pe.Ni, Xr + 0.234 * Rr - pe.Ni]]).stream(Qr), rr();
              }, Ur.fitExtent = function(vr, Rr) {
                return (0, kn.sp)(Ur, vr, Rr);
              }, Ur.fitSize = function(vr, Rr) {
                return (0, kn.Hv)(Ur, vr, Rr);
              }, Ur.fitWidth = function(vr, Rr) {
                return (0, kn.G0)(Ur, vr, Rr);
              }, Ur.fitHeight = function(vr, Rr) {
                return (0, kn.FL)(Ur, vr, Rr);
              }, Ur.scale(1070);
            }
            var jn = Z(30729), ur = Z(61957), Kn = Z(30915);
            function Jn(Sr, Dr) {
              return [Sr, (0, pe.Rm)((0, pe.Ml)((pe.TW + Dr) / 2))];
            }
            function Lr() {
              return mr(Jn).scale(961 / pe.FA);
            }
            function mr(Sr) {
              var Dr, si, zr, di = (0, an.A)(Sr), Da = di.center, la = di.scale, wa = di.translate, Wa = di.clipExtent, Qr = null;
              function Ur() {
                var rr = pe.pi * la(), vr = di((0, Kn.A)(di.rotate()).invert([0, 0]));
                return Wa(Qr == null ? [[vr[0] - rr, vr[1] - rr], [vr[0] + rr, vr[1] + rr]] : Sr === Jn ? [[Math.max(vr[0] - rr, Qr), Dr], [Math.min(vr[0] + rr, si), zr]] : [[Qr, Math.max(vr[1] - rr, Dr)], [si, Math.min(vr[1] + rr, zr)]]);
              }
              return di.scale = function(rr) {
                return arguments.length ? (la(rr), Ur()) : la();
              }, di.translate = function(rr) {
                return arguments.length ? (wa(rr), Ur()) : wa();
              }, di.center = function(rr) {
                return arguments.length ? (Da(rr), Ur()) : Da();
              }, di.clipExtent = function(rr) {
                return arguments.length ? (rr == null ? Qr = Dr = si = zr = null : (Qr = +rr[0][0], Dr = +rr[0][1], si = +rr[1][0], zr = +rr[1][1]), Ur()) : Qr == null ? null : [[Qr, Dr], [si, zr]];
              }, Ur();
            }
            function Ar(Sr) {
              return (0, pe.Ml)((pe.TW + Sr) / 2);
            }
            function Yr(Sr, Dr) {
              var si = (0, pe.gn)(Sr), zr = Sr === Dr ? (0, pe.F8)(Sr) : (0, pe.Rm)(si / (0, pe.gn)(Dr)) / (0, pe.Rm)(Ar(Dr) / Ar(Sr)), di = si * (0, pe.n7)(Ar(Sr), zr) / zr;
              if (!zr) return Jn;
              function Da(la, wa) {
                di > 0 ? wa < -pe.TW + pe.Ni && (wa = -pe.TW + pe.Ni) : wa > pe.TW - pe.Ni && (wa = pe.TW - pe.Ni);
                var Wa = di / (0, pe.n7)(Ar(wa), zr);
                return [Wa * (0, pe.F8)(zr * la), di - Wa * (0, pe.gn)(zr * la)];
              }
              return Da.invert = function(la, wa) {
                var Wa = di - wa, Qr = (0, pe._S)(zr) * (0, pe.RZ)(la * la + Wa * Wa), Ur = (0, pe.FP)(la, (0, pe.tn)(Wa)) * (0, pe._S)(Wa);
                return Wa * zr < 0 && (Ur -= pe.pi * (0, pe._S)(la) * (0, pe._S)(Wa)), [Ur / zr, 2 * (0, pe.rY)((0, pe.n7)(di / Qr, 1 / zr)) - pe.TW];
              }, Da;
            }
            function jr() {
              return Zt(Yr).scale(109.5).parallels([30, 30]);
            }
            Jn.invert = function(Sr, Dr) {
              return [Sr, 2 * (0, pe.rY)((0, pe.oN)(Dr)) - pe.TW];
            };
            var Mr = Z(18139);
            function Wr(Sr, Dr) {
              var si = (0, pe.gn)(Sr), zr = Sr === Dr ? (0, pe.F8)(Sr) : (si - (0, pe.gn)(Dr)) / (Dr - Sr), di = si / zr + Sr;
              if ((0, pe.tn)(zr) < pe.Ni) return Mr.f;
              function Da(la, wa) {
                var Wa = di - wa, Qr = zr * la;
                return [Wa * (0, pe.F8)(Qr), di - Wa * (0, pe.gn)(Qr)];
              }
              return Da.invert = function(la, wa) {
                var Wa = di - wa, Qr = (0, pe.FP)(la, (0, pe.tn)(Wa)) * (0, pe._S)(Wa);
                return Wa * zr < 0 && (Qr -= pe.pi * (0, pe._S)(la) * (0, pe._S)(Wa)), [Qr / zr, di - (0, pe._S)(zr) * (0, pe.RZ)(la * la + Wa * Wa)];
              }, Da;
            }
            function Bi() {
              return Zt(Wr).scale(131.154).center([0, 13.9389]);
            }
            var gi = 1.340264, Zr = -0.081106, fi = 893e-6, ri = 3796e-6, bi = (0, pe.RZ)(3) / 2;
            function Zi(Sr, Dr) {
              var si = (0, pe.qR)(bi * (0, pe.F8)(Dr)), zr = si * si, di = zr * zr * zr;
              return [Sr * (0, pe.gn)(si) / (bi * (gi + 3 * Zr * zr + di * (7 * fi + 9 * ri * zr))), si * (gi + Zr * zr + di * (fi + ri * zr))];
            }
            function $i() {
              return (0, an.A)(Zi).scale(177.158);
            }
            Zi.invert = function(Sr, Dr) {
              for (var si, zr = Dr, di = zr * zr, Da = di * di * di, la = 0; la < 12 && (Da = (di = (zr -= si = (zr * (gi + Zr * di + Da * (fi + ri * di)) - Dr) / (gi + 3 * Zr * di + Da * (7 * fi + 9 * ri * di))) * zr) * di * di, !((0, pe.tn)(si) < pe.$t)); ++la) ;
              return [bi * Sr * (gi + 3 * Zr * di + Da * (7 * fi + 9 * ri * di)) / (0, pe.gn)(zr), (0, pe.qR)((0, pe.F8)(zr) / bi)];
            };
            var Br = Z(48419), Pr = Z(913);
            function ei() {
              var Sr, Dr, si, zr, di, Da, la, wa = 1, Wa = 0, Qr = 0, Ur = 1, rr = 1, vr = 0, Rr = null, Vr = 1, Xr = 1, Jr = (0, Pr.G)({ point: function(gr, $r) {
                var li = Er([gr, $r]);
                this.stream.point(li[0], li[1]);
              } }), ti = st.A;
              function Hr() {
                return Vr = wa * Ur, Xr = wa * rr, Da = la = null, Er;
              }
              function Er(gr) {
                var $r = gr[0] * Vr, li = gr[1] * Xr;
                if (vr) {
                  var Qi = li * Sr - $r * Dr;
                  $r = $r * Sr + li * Dr, li = Qi;
                }
                return [$r + Wa, li + Qr];
              }
              return Er.invert = function(gr) {
                var $r = gr[0] - Wa, li = gr[1] - Qr;
                if (vr) {
                  var Qi = li * Sr + $r * Dr;
                  $r = $r * Sr - li * Dr, li = Qi;
                }
                return [$r / Vr, li / Xr];
              }, Er.stream = function(gr) {
                return Da && la === gr ? Da : Da = Jr(ti(la = gr));
              }, Er.postclip = function(gr) {
                return arguments.length ? (ti = gr, Rr = si = zr = di = null, Hr()) : ti;
              }, Er.clipExtent = function(gr) {
                return arguments.length ? (ti = gr == null ? (Rr = si = zr = di = null, st.A) : (0, ce.A)(Rr = +gr[0][0], si = +gr[0][1], zr = +gr[1][0], di = +gr[1][1]), Hr()) : Rr == null ? null : [[Rr, si], [zr, di]];
              }, Er.scale = function(gr) {
                return arguments.length ? (wa = +gr, Hr()) : wa;
              }, Er.translate = function(gr) {
                return arguments.length ? (Wa = +gr[0], Qr = +gr[1], Hr()) : [Wa, Qr];
              }, Er.angle = function(gr) {
                return arguments.length ? (vr = gr % 360 * pe.F2, Dr = (0, pe.F8)(vr), Sr = (0, pe.gn)(vr), Hr()) : vr * pe.uj;
              }, Er.reflectX = function(gr) {
                return arguments.length ? (Ur = gr ? -1 : 1, Hr()) : Ur < 0;
              }, Er.reflectY = function(gr) {
                return arguments.length ? (rr = gr ? -1 : 1, Hr()) : rr < 0;
              }, Er.fitExtent = function(gr, $r) {
                return (0, kn.sp)(Er, gr, $r);
              }, Er.fitSize = function(gr, $r) {
                return (0, kn.Hv)(Er, gr, $r);
              }, Er.fitWidth = function(gr, $r) {
                return (0, kn.G0)(Er, gr, $r);
              }, Er.fitHeight = function(gr, $r) {
                return (0, kn.FL)(Er, gr, $r);
              }, Er;
            }
            var Ci = Z(57949), oa = Z(53253), ka = Z(57738);
            function aa(Sr, Dr) {
              var si = (0, pe.gn)(Dr), zr = 1 + (0, pe.gn)(Sr) * si;
              return [si * (0, pe.F8)(Sr) / zr, (0, pe.F8)(Dr) / zr];
            }
            function hi() {
              return (0, an.A)(aa).scale(250).clipAngle(142);
            }
            function Gr(Sr, Dr) {
              return [(0, pe.Rm)((0, pe.Ml)((pe.TW + Dr) / 2)), -Sr];
            }
            function Ca() {
              var Sr = mr(Gr), Dr = Sr.center, si = Sr.rotate;
              return Sr.center = function(zr) {
                return arguments.length ? Dr([-zr[1], zr[0]]) : [(zr = Dr())[1], -zr[0]];
              }, Sr.rotate = function(zr) {
                return arguments.length ? si([zr[0], zr[1], zr.length > 2 ? zr[2] + 90 : 90]) : [(zr = si())[0], zr[1], zr[2] - 90];
              }, si([0, 0, 90]).scale(159.155);
            }
            aa.invert = (0, ka.I)(function(Sr) {
              return 2 * (0, pe.rY)(Sr);
            }), Gr.invert = function(Sr, Dr) {
              return [-Dr, 2 * (0, pe.rY)((0, pe.oN)(Sr)) - pe.TW];
            };
          }, 81758: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ne;
            } });
            var te = Z(61323);
            function ne(ee, ae) {
              var oe = ee[0] * te.F2, se = ee[1] * te.F2, ce = ae[0] * te.F2, le = ae[1] * te.F2, ue = (0, te.gn)(se), de = (0, te.F8)(se), fe = (0, te.gn)(le), he = (0, te.F8)(le), me = ue * (0, te.gn)(oe), pe = ue * (0, te.F8)(oe), ye = fe * (0, te.gn)(ce), ge = fe * (0, te.F8)(ce), ve = 2 * (0, te.qR)((0, te.RZ)((0, te.bo)(le - se) + ue * fe * (0, te.bo)(ce - oe))), be = (0, te.F8)(ve), Ce = ve ? function(Se) {
                var we = (0, te.F8)(Se *= ve) / be, Ee = (0, te.F8)(ve - Se) / be, xe = Ee * me + we * ye, Le = Ee * pe + we * ge, Qe = Ee * de + we * he;
                return [(0, te.FP)(Le, xe) * te.uj, (0, te.FP)(Qe, (0, te.RZ)(xe * xe + Le * Le)) * te.uj];
              } : function() {
                return [oe * te.uj, se * te.uj];
              };
              return Ce.distance = ve, Ce;
            }
          }, 61323: function(G, J, Z) {
            Z.d(J, { $t: function() {
              return ne;
            }, F2: function() {
              return le;
            }, F8: function() {
              return ve;
            }, FA: function() {
              return se;
            }, FP: function() {
              return fe;
            }, HQ: function() {
              return we;
            }, Ml: function() {
              return Se;
            }, Ni: function() {
              return te;
            }, RZ: function() {
              return Ce;
            }, Rm: function() {
              return ye;
            }, TW: function() {
              return ae;
            }, _S: function() {
              return be;
            }, bo: function() {
              return xe;
            }, gn: function() {
              return he;
            }, gz: function() {
              return oe;
            }, mk: function() {
              return me;
            }, n7: function() {
              return ge;
            }, oN: function() {
              return pe;
            }, pi: function() {
              return ee;
            }, qR: function() {
              return Ee;
            }, rY: function() {
              return de;
            }, tn: function() {
              return ue;
            }, uj: function() {
              return ce;
            } });
            var te = 1e-6, ne = 1e-12, ee = Math.PI, ae = ee / 2, oe = ee / 4, se = 2 * ee, ce = 180 / ee, le = ee / 180, ue = Math.abs, de = Math.atan, fe = Math.atan2, he = Math.cos, me = Math.ceil, pe = Math.exp, ye = Math.log, ge = Math.pow, ve = Math.sin, be = Math.sign || function(Le) {
              return Le > 0 ? 1 : Le < 0 ? -1 : 0;
            }, Ce = Math.sqrt, Se = Math.tan;
            function we(Le) {
              return Le > 1 ? 0 : Le < -1 ? ee : Math.acos(Le);
            }
            function Ee(Le) {
              return Le > 1 ? ae : Le < -1 ? -ae : Math.asin(Le);
            }
            function xe(Le) {
              return (Le = ve(Le / 2)) * Le;
            }
          }, 53341: function(G, J, Z) {
            function te() {
            }
            Z.d(J, { A: function() {
              return te;
            } });
          }, 33028: function(G, J, Z) {
            var te = Z(53341), ne = 1 / 0, ee = ne, ae = -ne, oe = ae, se = { point: function(ce, le) {
              ce < ne && (ne = ce), ce > ae && (ae = ce), le < ee && (ee = le), le > oe && (oe = le);
            }, lineStart: te.A, lineEnd: te.A, polygonStart: te.A, polygonEnd: te.A, result: function() {
              var ce = [[ne, ee], [ae, oe]];
              return ae = oe = -(ee = ne = 1 / 0), ce;
            } };
            J.A = se;
          }, 28759: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ne;
            } });
            var te = Z(61323);
            function ne(ee, ae) {
              return (0, te.tn)(ee[0] - ae[0]) < te.Ni && (0, te.tn)(ee[1] - ae[1]) < te.Ni;
            }
          }, 2274: function(G, J, Z) {
            Z.d(J, { A: function() {
              return se;
            } });
            var te = Z(49353), ne = Z(20375), ee = Z(61323), ae = (0, te.A)();
            function oe(ce) {
              return (0, ee.tn)(ce[0]) <= ee.pi ? ce[0] : (0, ee._S)(ce[0]) * (((0, ee.tn)(ce[0]) + ee.pi) % ee.FA - ee.pi);
            }
            function se(ce, le) {
              var ue = oe(le), de = le[1], fe = (0, ee.F8)(de), he = [(0, ee.F8)(ue), -(0, ee.gn)(ue), 0], me = 0, pe = 0;
              ae.reset(), fe === 1 ? de = ee.TW + ee.Ni : fe === -1 && (de = -ee.TW - ee.Ni);
              for (var ye = 0, ge = ce.length; ye < ge; ++ye) if (be = (ve = ce[ye]).length) for (var ve, be, Ce = ve[be - 1], Se = oe(Ce), we = Ce[1] / 2 + ee.gz, Ee = (0, ee.F8)(we), xe = (0, ee.gn)(we), Le = 0; Le < be; ++Le, Se = ke, Ee = Re, xe = De, Ce = Qe) {
                var Qe = ve[Le], ke = oe(Qe), Me = Qe[1] / 2 + ee.gz, Re = (0, ee.F8)(Me), De = (0, ee.gn)(Me), Pe = ke - Se, $e = Pe >= 0 ? 1 : -1, qe = $e * Pe, Xe = qe > ee.pi, Ve = Ee * Re;
                if (ae.add((0, ee.FP)(Ve * $e * (0, ee.F8)(qe), xe * De + Ve * (0, ee.gn)(qe))), me += Xe ? Pe + $e * ee.FA : Pe, Xe ^ Se >= ue ^ ke >= ue) {
                  var Ze = (0, ne.r8)((0, ne.jf)(Ce), (0, ne.jf)(Qe));
                  (0, ne.Cx)(Ze);
                  var Ge = (0, ne.r8)(he, Ze);
                  (0, ne.Cx)(Ge);
                  var ze = (Xe ^ Pe >= 0 ? -1 : 1) * (0, ee.qR)(Ge[2]);
                  (de > ze || de === ze && (Ze[0] || Ze[1])) && (pe += Xe ^ Pe >= 0 ? 1 : -1);
                }
              }
              return (me < -ee.Ni || me < ee.Ni && ae < -ee.Ni) ^ 1 & pe;
            }
          }, 57738: function(G, J, Z) {
            Z.d(J, { I: function() {
              return ee;
            }, c: function() {
              return ne;
            } });
            var te = Z(61323);
            function ne(ae) {
              return function(oe, se) {
                var ce = (0, te.gn)(oe), le = (0, te.gn)(se), ue = ae(ce * le);
                return [ue * le * (0, te.F8)(oe), ue * (0, te.F8)(se)];
              };
            }
            function ee(ae) {
              return function(oe, se) {
                var ce = (0, te.RZ)(oe * oe + se * se), le = ae(ce), ue = (0, te.F8)(le), de = (0, te.gn)(le);
                return [(0, te.FP)(oe * ue, ce * de), (0, te.qR)(ce && se * ue / ce)];
              };
            }
          }, 30729: function(G, J, Z) {
            Z.d(J, { A: function() {
              return oe;
            }, n: function() {
              return ae;
            } });
            var te = Z(61323), ne = Z(57738), ee = Z(94684), ae = (0, ne.c)(function(se) {
              return (0, te.RZ)(2 / (1 + se));
            });
            function oe() {
              return (0, ee.A)(ae).scale(124.75).clipAngle(179.999);
            }
            ae.invert = (0, ne.I)(function(se) {
              return 2 * (0, te.qR)(se / 2);
            });
          }, 61957: function(G, J, Z) {
            Z.d(J, { A: function() {
              return oe;
            }, j: function() {
              return ae;
            } });
            var te = Z(61323), ne = Z(57738), ee = Z(94684), ae = (0, ne.c)(function(se) {
              return (se = (0, te.HQ)(se)) && se / (0, te.F8)(se);
            });
            function oe() {
              return (0, ee.A)(ae).scale(79.4188).clipAngle(179.999);
            }
            ae.invert = (0, ne.I)(function(se) {
              return se;
            });
          }, 18139: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ee;
            }, f: function() {
              return ne;
            } });
            var te = Z(94684);
            function ne(ae, oe) {
              return [ae, oe];
            }
            function ee() {
              return (0, te.A)(ne).scale(152.63);
            }
            ne.invert = ne;
          }, 7944: function(G, J, Z) {
            Z.d(J, { FL: function() {
              return ce;
            }, G0: function() {
              return se;
            }, Hv: function() {
              return oe;
            }, sp: function() {
              return ae;
            } });
            var te = Z(20465), ne = Z(33028);
            function ee(le, ue, de) {
              var fe = le.clipExtent && le.clipExtent();
              return le.scale(150).translate([0, 0]), fe != null && le.clipExtent(null), (0, te.A)(de, le.stream(ne.A)), ue(ne.A.result()), fe != null && le.clipExtent(fe), le;
            }
            function ae(le, ue, de) {
              return ee(le, function(fe) {
                var he = ue[1][0] - ue[0][0], me = ue[1][1] - ue[0][1], pe = Math.min(he / (fe[1][0] - fe[0][0]), me / (fe[1][1] - fe[0][1])), ye = +ue[0][0] + (he - pe * (fe[1][0] + fe[0][0])) / 2, ge = +ue[0][1] + (me - pe * (fe[1][1] + fe[0][1])) / 2;
                le.scale(150 * pe).translate([ye, ge]);
              }, de);
            }
            function oe(le, ue, de) {
              return ae(le, [[0, 0], ue], de);
            }
            function se(le, ue, de) {
              return ee(le, function(fe) {
                var he = +ue, me = he / (fe[1][0] - fe[0][0]), pe = (he - me * (fe[1][0] + fe[0][0])) / 2, ye = -me * fe[0][1];
                le.scale(150 * me).translate([pe, ye]);
              }, de);
            }
            function ce(le, ue, de) {
              return ee(le, function(fe) {
                var he = +ue, me = he / (fe[1][1] - fe[0][1]), pe = -me * fe[0][0], ye = (he - me * (fe[1][1] + fe[0][1])) / 2;
                le.scale(150 * me).translate([pe, ye]);
              }, de);
            }
          }, 48419: function(G, J, Z) {
            Z.d(J, { A: function() {
              return oe;
            }, T: function() {
              return ae;
            } });
            var te = Z(61323), ne = Z(57738), ee = Z(94684);
            function ae(se, ce) {
              var le = (0, te.gn)(ce), ue = (0, te.gn)(se) * le;
              return [le * (0, te.F8)(se) / ue, (0, te.F8)(ce) / ue];
            }
            function oe() {
              return (0, ee.A)(ae).scale(144.049).clipAngle(60);
            }
            ae.invert = (0, ne.I)(te.rY);
          }, 94684: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ve;
            }, U: function() {
              return be;
            } });
            var te = Z(42413), ne = Z(47402), ee = Z(21503), ae = Z(19057), oe = Z(26827), se = Z(61323), ce = Z(30915), le = Z(913), ue = Z(7944), de = Z(20375), fe = 16, he = (0, se.gn)(30 * se.F2);
            function me(Ce, Se) {
              return +Se ? /* @__PURE__ */ function(we, Ee) {
                function xe(Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze, Ge, ze) {
                  var We = Pe - Le, at = $e - Qe, ct = We * We + at * at;
                  if (ct > 4 * Ee && Ge--) {
                    var et = Me + Xe, dt = Re + Ve, Je = De + Ze, it = (0, se.RZ)(et * et + dt * dt + Je * Je), st = (0, se.qR)(Je /= it), gt = (0, se.tn)((0, se.tn)(Je) - 1) < se.Ni || (0, se.tn)(ke - qe) < se.Ni ? (ke + qe) / 2 : (0, se.FP)(dt, et), bt = we(gt, st), yt = bt[0], St = bt[1], Lt = yt - Le, Rt = St - Qe, vt = at * Lt - We * Rt;
                    (vt * vt / ct > Ee || (0, se.tn)((We * Lt + at * Rt) / ct - 0.5) > 0.3 || Me * Xe + Re * Ve + De * Ze < he) && (xe(Le, Qe, ke, Me, Re, De, yt, St, gt, et /= it, dt /= it, Je, Ge, ze), ze.point(yt, St), xe(yt, St, gt, et, dt, Je, Pe, $e, qe, Xe, Ve, Ze, Ge, ze));
                  }
                }
                return function(Le) {
                  var Qe, ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze, Ge, ze = { point: We, lineStart: at, lineEnd: et, polygonStart: function() {
                    Le.polygonStart(), ze.lineStart = dt;
                  }, polygonEnd: function() {
                    Le.polygonEnd(), ze.lineStart = at;
                  } };
                  function We(st, gt) {
                    st = we(st, gt), Le.point(st[0], st[1]);
                  }
                  function at() {
                    qe = NaN, ze.point = ct, Le.lineStart();
                  }
                  function ct(st, gt) {
                    var bt = (0, de.jf)([st, gt]), yt = we(st, gt);
                    xe(qe, Xe, $e, Ve, Ze, Ge, qe = yt[0], Xe = yt[1], $e = st, Ve = bt[0], Ze = bt[1], Ge = bt[2], fe, Le), Le.point(qe, Xe);
                  }
                  function et() {
                    ze.point = We, Le.lineEnd();
                  }
                  function dt() {
                    at(), ze.point = Je, ze.lineEnd = it;
                  }
                  function Je(st, gt) {
                    ct(Qe = st, gt), ke = qe, Me = Xe, Re = Ve, De = Ze, Pe = Ge, ze.point = ct;
                  }
                  function it() {
                    xe(qe, Xe, $e, Ve, Ze, Ge, ke, Me, Qe, Re, De, Pe, fe, Le), ze.lineEnd = et, et();
                  }
                  return ze;
                };
              }(Ce, Se) : function(we) {
                return (0, le.G)({ point: function(Ee, xe) {
                  Ee = we(Ee, xe), this.stream.point(Ee[0], Ee[1]);
                } });
              }(Ce);
            }
            var pe = (0, le.G)({ point: function(Ce, Se) {
              this.stream.point(Ce * se.F2, Se * se.F2);
            } });
            function ye(Ce, Se, we, Ee, xe) {
              function Le(Qe, ke) {
                return [Se + Ce * (Qe *= Ee), we - Ce * (ke *= xe)];
              }
              return Le.invert = function(Qe, ke) {
                return [(Qe - Se) / Ce * Ee, (we - ke) / Ce * xe];
              }, Le;
            }
            function ge(Ce, Se, we, Ee, xe, Le) {
              var Qe = (0, se.gn)(Le), ke = (0, se.F8)(Le), Me = Qe * Ce, Re = ke * Ce, De = Qe / Ce, Pe = ke / Ce, $e = (ke * we - Qe * Se) / Ce, qe = (ke * Se + Qe * we) / Ce;
              function Xe(Ve, Ze) {
                return [Me * (Ve *= Ee) - Re * (Ze *= xe) + Se, we - Re * Ve - Me * Ze];
              }
              return Xe.invert = function(Ve, Ze) {
                return [Ee * (De * Ve - Pe * Ze + $e), xe * (qe - Pe * Ve - De * Ze)];
              }, Xe;
            }
            function ve(Ce) {
              return be(function() {
                return Ce;
              })();
            }
            function be(Ce) {
              var Se, we, Ee, xe, Le, Qe, ke, Me, Re, De, Pe = 150, $e = 480, qe = 250, Xe = 0, Ve = 0, Ze = 0, Ge = 0, ze = 0, We = 0, at = 1, ct = 1, et = null, dt = te.A, Je = null, it = oe.A, st = 0.5;
              function gt(Lt) {
                return Me(Lt[0] * se.F2, Lt[1] * se.F2);
              }
              function bt(Lt) {
                return (Lt = Me.invert(Lt[0], Lt[1])) && [Lt[0] * se.uj, Lt[1] * se.uj];
              }
              function yt() {
                var Lt = ge(Pe, 0, 0, at, ct, We).apply(null, Se(Xe, Ve)), Rt = (We ? ge : ye)(Pe, $e - Lt[0], qe - Lt[1], at, ct, We);
                return we = (0, ce.y)(Ze, Ge, ze), ke = (0, ae.A)(Se, Rt), Me = (0, ae.A)(we, ke), Qe = me(ke, st), St();
              }
              function St() {
                return Re = De = null, gt;
              }
              return gt.stream = function(Lt) {
                return Re && De === Lt ? Re : Re = pe(function(Rt) {
                  return (0, le.G)({ point: function(vt, $t) {
                    var Pt = Rt(vt, $t);
                    return this.stream.point(Pt[0], Pt[1]);
                  } });
                }(we)(dt(Qe(it(De = Lt)))));
              }, gt.preclip = function(Lt) {
                return arguments.length ? (dt = Lt, et = void 0, St()) : dt;
              }, gt.postclip = function(Lt) {
                return arguments.length ? (it = Lt, Je = Ee = xe = Le = null, St()) : it;
              }, gt.clipAngle = function(Lt) {
                return arguments.length ? (dt = +Lt ? (0, ne.A)(et = Lt * se.F2) : (et = null, te.A), St()) : et * se.uj;
              }, gt.clipExtent = function(Lt) {
                return arguments.length ? (it = Lt == null ? (Je = Ee = xe = Le = null, oe.A) : (0, ee.A)(Je = +Lt[0][0], Ee = +Lt[0][1], xe = +Lt[1][0], Le = +Lt[1][1]), St()) : Je == null ? null : [[Je, Ee], [xe, Le]];
              }, gt.scale = function(Lt) {
                return arguments.length ? (Pe = +Lt, yt()) : Pe;
              }, gt.translate = function(Lt) {
                return arguments.length ? ($e = +Lt[0], qe = +Lt[1], yt()) : [$e, qe];
              }, gt.center = function(Lt) {
                return arguments.length ? (Xe = Lt[0] % 360 * se.F2, Ve = Lt[1] % 360 * se.F2, yt()) : [Xe * se.uj, Ve * se.uj];
              }, gt.rotate = function(Lt) {
                return arguments.length ? (Ze = Lt[0] % 360 * se.F2, Ge = Lt[1] % 360 * se.F2, ze = Lt.length > 2 ? Lt[2] % 360 * se.F2 : 0, yt()) : [Ze * se.uj, Ge * se.uj, ze * se.uj];
              }, gt.angle = function(Lt) {
                return arguments.length ? (We = Lt % 360 * se.F2, yt()) : We * se.uj;
              }, gt.reflectX = function(Lt) {
                return arguments.length ? (at = Lt ? -1 : 1, yt()) : at < 0;
              }, gt.reflectY = function(Lt) {
                return arguments.length ? (ct = Lt ? -1 : 1, yt()) : ct < 0;
              }, gt.precision = function(Lt) {
                return arguments.length ? (Qe = me(ke, st = Lt * Lt), St()) : (0, se.RZ)(st);
              }, gt.fitExtent = function(Lt, Rt) {
                return (0, ue.sp)(gt, Lt, Rt);
              }, gt.fitSize = function(Lt, Rt) {
                return (0, ue.Hv)(gt, Lt, Rt);
              }, gt.fitWidth = function(Lt, Rt) {
                return (0, ue.G0)(gt, Lt, Rt);
              }, gt.fitHeight = function(Lt, Rt) {
                return (0, ue.FL)(gt, Lt, Rt);
              }, function() {
                return Se = Ce.apply(this, arguments), gt.invert = Se.invert && bt, yt();
              };
            }
          }, 57949: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ae;
            }, P: function() {
              return ee;
            } });
            var te = Z(94684), ne = Z(61323);
            function ee(oe, se) {
              var ce = se * se, le = ce * ce;
              return [oe * (0.8707 - 0.131979 * ce + le * (le * (3971e-6 * ce - 1529e-6 * le) - 0.013791)), se * (1.007226 + ce * (0.015085 + le * (0.028874 * ce - 0.044475 - 5916e-6 * le)))];
            }
            function ae() {
              return (0, te.A)(ee).scale(175.295);
            }
            ee.invert = function(oe, se) {
              var ce, le = se, ue = 25;
              do {
                var de = le * le, fe = de * de;
                le -= ce = (le * (1.007226 + de * (0.015085 + fe * (0.028874 * de - 0.044475 - 5916e-6 * fe))) - se) / (1.007226 + de * (0.045255 + fe * (0.259866 * de - 0.311325 - 0.06507600000000001 * fe)));
              } while ((0, ne.tn)(ce) > ne.Ni && --ue > 0);
              return [oe / (0.8707 + (de = le * le) * (de * (de * de * de * (3971e-6 - 1529e-6 * de) - 0.013791) - 0.131979)), le];
            };
          }, 53253: function(G, J, Z) {
            Z.d(J, { A: function() {
              return oe;
            }, x: function() {
              return ae;
            } });
            var te = Z(61323), ne = Z(57738), ee = Z(94684);
            function ae(se, ce) {
              return [(0, te.gn)(ce) * (0, te.F8)(se), (0, te.F8)(ce)];
            }
            function oe() {
              return (0, ee.A)(ae).scale(249.5).clipAngle(90 + te.Ni);
            }
            ae.invert = (0, ne.I)(te.qR);
          }, 30915: function(G, J, Z) {
            Z.d(J, { A: function() {
              return le;
            }, y: function() {
              return ae;
            } });
            var te = Z(19057), ne = Z(61323);
            function ee(ue, de) {
              return [(0, ne.tn)(ue) > ne.pi ? ue + Math.round(-ue / ne.FA) * ne.FA : ue, de];
            }
            function ae(ue, de, fe) {
              return (ue %= ne.FA) ? de || fe ? (0, te.A)(se(ue), ce(de, fe)) : se(ue) : de || fe ? ce(de, fe) : ee;
            }
            function oe(ue) {
              return function(de, fe) {
                return [(de += ue) > ne.pi ? de - ne.FA : de < -ne.pi ? de + ne.FA : de, fe];
              };
            }
            function se(ue) {
              var de = oe(ue);
              return de.invert = oe(-ue), de;
            }
            function ce(ue, de) {
              var fe = (0, ne.gn)(ue), he = (0, ne.F8)(ue), me = (0, ne.gn)(de), pe = (0, ne.F8)(de);
              function ye(ge, ve) {
                var be = (0, ne.gn)(ve), Ce = (0, ne.gn)(ge) * be, Se = (0, ne.F8)(ge) * be, we = (0, ne.F8)(ve), Ee = we * fe + Ce * he;
                return [(0, ne.FP)(Se * me - Ee * pe, Ce * fe - we * he), (0, ne.qR)(Ee * me + Se * pe)];
              }
              return ye.invert = function(ge, ve) {
                var be = (0, ne.gn)(ve), Ce = (0, ne.gn)(ge) * be, Se = (0, ne.F8)(ge) * be, we = (0, ne.F8)(ve), Ee = we * me - Se * pe;
                return [(0, ne.FP)(Se * me + we * pe, Ce * fe + Ee * he), (0, ne.qR)(Ee * fe - Ce * he)];
              }, ye;
            }
            function le(ue) {
              function de(fe) {
                return (fe = ue(fe[0] * ne.F2, fe[1] * ne.F2))[0] *= ne.uj, fe[1] *= ne.uj, fe;
              }
              return ue = ae(ue[0] * ne.F2, ue[1] * ne.F2, ue.length > 2 ? ue[2] * ne.F2 : 0), de.invert = function(fe) {
                return (fe = ue.invert(fe[0] * ne.F2, fe[1] * ne.F2))[0] *= ne.uj, fe[1] *= ne.uj, fe;
              }, de;
            }
            ee.invert = ee;
          }, 20465: function(G, J, Z) {
            function te(ce, le) {
              ce && ee.hasOwnProperty(ce.type) && ee[ce.type](ce, le);
            }
            Z.d(J, { A: function() {
              return se;
            } });
            var ne = { Feature: function(ce, le) {
              te(ce.geometry, le);
            }, FeatureCollection: function(ce, le) {
              for (var ue = ce.features, de = -1, fe = ue.length; ++de < fe; ) te(ue[de].geometry, le);
            } }, ee = { Sphere: function(ce, le) {
              le.sphere();
            }, Point: function(ce, le) {
              ce = ce.coordinates, le.point(ce[0], ce[1], ce[2]);
            }, MultiPoint: function(ce, le) {
              for (var ue = ce.coordinates, de = -1, fe = ue.length; ++de < fe; ) ce = ue[de], le.point(ce[0], ce[1], ce[2]);
            }, LineString: function(ce, le) {
              ae(ce.coordinates, le, 0);
            }, MultiLineString: function(ce, le) {
              for (var ue = ce.coordinates, de = -1, fe = ue.length; ++de < fe; ) ae(ue[de], le, 0);
            }, Polygon: function(ce, le) {
              oe(ce.coordinates, le);
            }, MultiPolygon: function(ce, le) {
              for (var ue = ce.coordinates, de = -1, fe = ue.length; ++de < fe; ) oe(ue[de], le);
            }, GeometryCollection: function(ce, le) {
              for (var ue = ce.geometries, de = -1, fe = ue.length; ++de < fe; ) te(ue[de], le);
            } };
            function ae(ce, le, ue) {
              var de, fe = -1, he = ce.length - ue;
              for (le.lineStart(); ++fe < he; ) de = ce[fe], le.point(de[0], de[1], de[2]);
              le.lineEnd();
            }
            function oe(ce, le) {
              var ue = -1, de = ce.length;
              for (le.polygonStart(); ++ue < de; ) ae(ce[ue], le, 1);
              le.polygonEnd();
            }
            function se(ce, le) {
              ce && ne.hasOwnProperty(ce.type) ? ne[ce.type](ce, le) : te(ce, le);
            }
          }, 913: function(G, J, Z) {
            function te(ae) {
              return { stream: ne(ae) };
            }
            function ne(ae) {
              return function(oe) {
                var se = new ee();
                for (var ce in ae) se[ce] = ae[ce];
                return se.stream = oe, se;
              };
            }
            function ee() {
            }
            Z.d(J, { A: function() {
              return te;
            }, G: function() {
              return ne;
            } }), ee.prototype = { constructor: ee, point: function(ae, oe) {
              this.stream.point(ae, oe);
            }, sphere: function() {
              this.stream.sphere();
            }, lineStart: function() {
              this.stream.lineStart();
            }, lineEnd: function() {
              this.stream.lineEnd();
            }, polygonStart: function() {
              this.stream.polygonStart();
            }, polygonEnd: function() {
              this.stream.polygonEnd();
            } };
          }, 92264: function(G, J, Z) {
            function te(Ft, Tt) {
              return Ft.parent === Tt.parent ? 1 : 2;
            }
            function ne(Ft, Tt) {
              return Ft + Tt.x;
            }
            function ee(Ft, Tt) {
              return Math.max(Ft, Tt.y);
            }
            function ae() {
              var Ft = te, Tt = 1, It = 1, Ct = !1;
              function Kt(Vt) {
                var Yt, sn = 0;
                Vt.eachAfter(function(rn) {
                  var vn = rn.children;
                  vn ? (rn.x = function(dn) {
                    return dn.reduce(ne, 0) / dn.length;
                  }(vn), rn.y = function(dn) {
                    return 1 + dn.reduce(ee, 0);
                  }(vn)) : (rn.x = Yt ? sn += Ft(rn, Yt) : 0, rn.y = 0, Yt = rn);
                });
                var hn = function(rn) {
                  for (var vn; vn = rn.children; ) rn = vn[0];
                  return rn;
                }(Vt), Qt = function(rn) {
                  for (var vn; vn = rn.children; ) rn = vn[vn.length - 1];
                  return rn;
                }(Vt), Ut = hn.x - Ft(hn, Qt) / 2, Wt = Qt.x + Ft(Qt, hn) / 2;
                return Vt.eachAfter(Ct ? function(rn) {
                  rn.x = (rn.x - Vt.x) * Tt, rn.y = (Vt.y - rn.y) * It;
                } : function(rn) {
                  rn.x = (rn.x - Ut) / (Wt - Ut) * Tt, rn.y = (1 - (Vt.y ? rn.y / Vt.y : 1)) * It;
                });
              }
              return Kt.separation = function(Vt) {
                return arguments.length ? (Ft = Vt, Kt) : Ft;
              }, Kt.size = function(Vt) {
                return arguments.length ? (Ct = !1, Tt = +Vt[0], It = +Vt[1], Kt) : Ct ? null : [Tt, It];
              }, Kt.nodeSize = function(Vt) {
                return arguments.length ? (Ct = !0, Tt = +Vt[0], It = +Vt[1], Kt) : Ct ? [Tt, It] : null;
              }, Kt;
            }
            function oe(Ft) {
              var Tt = 0, It = Ft.children, Ct = It && It.length;
              if (Ct) for (; --Ct >= 0; ) Tt += It[Ct].value;
              else Tt = 1;
              Ft.value = Tt;
            }
            function se(Ft, Tt) {
              var It, Ct, Kt, Vt, Yt, sn = new de(Ft), hn = +Ft.value && (sn.value = Ft.value), Qt = [sn];
              for (Tt == null && (Tt = ce); It = Qt.pop(); ) if (hn && (It.value = +It.data.value), (Kt = Tt(It.data)) && (Yt = Kt.length)) for (It.children = new Array(Yt), Vt = Yt - 1; Vt >= 0; --Vt) Qt.push(Ct = It.children[Vt] = new de(Kt[Vt])), Ct.parent = It, Ct.depth = It.depth + 1;
              return sn.eachBefore(ue);
            }
            function ce(Ft) {
              return Ft.children;
            }
            function le(Ft) {
              Ft.data = Ft.data.data;
            }
            function ue(Ft) {
              var Tt = 0;
              do
                Ft.height = Tt;
              while ((Ft = Ft.parent) && Ft.height < ++Tt);
            }
            function de(Ft) {
              this.data = Ft, this.depth = this.height = 0, this.parent = null;
            }
            Z.r(J), Z.d(J, { cluster: function() {
              return ae;
            }, hierarchy: function() {
              return se;
            }, pack: function() {
              return Pe;
            }, packEnclose: function() {
              return he;
            }, packSiblings: function() {
              return Qe;
            }, partition: function() {
              return Ge;
            }, stratify: function() {
              return dt;
            }, tree: function() {
              return St;
            }, treemap: function() {
              return Pt;
            }, treemapBinary: function() {
              return Xt;
            }, treemapDice: function() {
              return Ze;
            }, treemapResquarify: function() {
              return Gt;
            }, treemapSlice: function() {
              return Lt;
            }, treemapSliceDice: function() {
              return Nt;
            }, treemapSquarify: function() {
              return $t;
            } }), de.prototype = se.prototype = { constructor: de, count: function() {
              return this.eachAfter(oe);
            }, each: function(Ft) {
              var Tt, It, Ct, Kt, Vt = this, Yt = [Vt];
              do
                for (Tt = Yt.reverse(), Yt = []; Vt = Tt.pop(); ) if (Ft(Vt), It = Vt.children) for (Ct = 0, Kt = It.length; Ct < Kt; ++Ct) Yt.push(It[Ct]);
              while (Yt.length);
              return this;
            }, eachAfter: function(Ft) {
              for (var Tt, It, Ct, Kt = this, Vt = [Kt], Yt = []; Kt = Vt.pop(); ) if (Yt.push(Kt), Tt = Kt.children) for (It = 0, Ct = Tt.length; It < Ct; ++It) Vt.push(Tt[It]);
              for (; Kt = Yt.pop(); ) Ft(Kt);
              return this;
            }, eachBefore: function(Ft) {
              for (var Tt, It, Ct = this, Kt = [Ct]; Ct = Kt.pop(); ) if (Ft(Ct), Tt = Ct.children) for (It = Tt.length - 1; It >= 0; --It) Kt.push(Tt[It]);
              return this;
            }, sum: function(Ft) {
              return this.eachAfter(function(Tt) {
                for (var It = +Ft(Tt.data) || 0, Ct = Tt.children, Kt = Ct && Ct.length; --Kt >= 0; ) It += Ct[Kt].value;
                Tt.value = It;
              });
            }, sort: function(Ft) {
              return this.eachBefore(function(Tt) {
                Tt.children && Tt.children.sort(Ft);
              });
            }, path: function(Ft) {
              for (var Tt = this, It = function(Vt, Yt) {
                if (Vt === Yt) return Vt;
                var sn = Vt.ancestors(), hn = Yt.ancestors(), Qt = null;
                for (Vt = sn.pop(), Yt = hn.pop(); Vt === Yt; ) Qt = Vt, Vt = sn.pop(), Yt = hn.pop();
                return Qt;
              }(Tt, Ft), Ct = [Tt]; Tt !== It; ) Tt = Tt.parent, Ct.push(Tt);
              for (var Kt = Ct.length; Ft !== It; ) Ct.splice(Kt, 0, Ft), Ft = Ft.parent;
              return Ct;
            }, ancestors: function() {
              for (var Ft = this, Tt = [Ft]; Ft = Ft.parent; ) Tt.push(Ft);
              return Tt;
            }, descendants: function() {
              var Ft = [];
              return this.each(function(Tt) {
                Ft.push(Tt);
              }), Ft;
            }, leaves: function() {
              var Ft = [];
              return this.eachBefore(function(Tt) {
                Tt.children || Ft.push(Tt);
              }), Ft;
            }, links: function() {
              var Ft = this, Tt = [];
              return Ft.each(function(It) {
                It !== Ft && Tt.push({ source: It.parent, target: It });
              }), Tt;
            }, copy: function() {
              return se(this).eachBefore(le);
            } };
            var fe = Array.prototype.slice;
            function he(Ft) {
              for (var Tt, It, Ct = 0, Kt = (Ft = function(Yt) {
                for (var sn, hn, Qt = Yt.length; Qt; ) hn = Math.random() * Qt-- | 0, sn = Yt[Qt], Yt[Qt] = Yt[hn], Yt[hn] = sn;
                return Yt;
              }(fe.call(Ft))).length, Vt = []; Ct < Kt; ) Tt = Ft[Ct], It && ye(It, Tt) ? ++Ct : (It = ve(Vt = me(Vt, Tt)), Ct = 0);
              return It;
            }
            function me(Ft, Tt) {
              var It, Ct;
              if (ge(Tt, Ft)) return [Tt];
              for (It = 0; It < Ft.length; ++It) if (pe(Tt, Ft[It]) && ge(be(Ft[It], Tt), Ft)) return [Ft[It], Tt];
              for (It = 0; It < Ft.length - 1; ++It) for (Ct = It + 1; Ct < Ft.length; ++Ct) if (pe(be(Ft[It], Ft[Ct]), Tt) && pe(be(Ft[It], Tt), Ft[Ct]) && pe(be(Ft[Ct], Tt), Ft[It]) && ge(Ce(Ft[It], Ft[Ct], Tt), Ft)) return [Ft[It], Ft[Ct], Tt];
              throw new Error();
            }
            function pe(Ft, Tt) {
              var It = Ft.r - Tt.r, Ct = Tt.x - Ft.x, Kt = Tt.y - Ft.y;
              return It < 0 || It * It < Ct * Ct + Kt * Kt;
            }
            function ye(Ft, Tt) {
              var It = Ft.r - Tt.r + 1e-6, Ct = Tt.x - Ft.x, Kt = Tt.y - Ft.y;
              return It > 0 && It * It > Ct * Ct + Kt * Kt;
            }
            function ge(Ft, Tt) {
              for (var It = 0; It < Tt.length; ++It) if (!ye(Ft, Tt[It])) return !1;
              return !0;
            }
            function ve(Ft) {
              switch (Ft.length) {
                case 1:
                  return { x: (Tt = Ft[0]).x, y: Tt.y, r: Tt.r };
                case 2:
                  return be(Ft[0], Ft[1]);
                case 3:
                  return Ce(Ft[0], Ft[1], Ft[2]);
              }
              var Tt;
            }
            function be(Ft, Tt) {
              var It = Ft.x, Ct = Ft.y, Kt = Ft.r, Vt = Tt.x, Yt = Tt.y, sn = Tt.r, hn = Vt - It, Qt = Yt - Ct, Ut = sn - Kt, Wt = Math.sqrt(hn * hn + Qt * Qt);
              return { x: (It + Vt + hn / Wt * Ut) / 2, y: (Ct + Yt + Qt / Wt * Ut) / 2, r: (Wt + Kt + sn) / 2 };
            }
            function Ce(Ft, Tt, It) {
              var Ct = Ft.x, Kt = Ft.y, Vt = Ft.r, Yt = Tt.x, sn = Tt.y, hn = Tt.r, Qt = It.x, Ut = It.y, Wt = It.r, rn = Ct - Yt, vn = Ct - Qt, dn = Kt - sn, Ht = Kt - Ut, jt = hn - Vt, cn = Wt - Vt, wn = Ct * Ct + Kt * Kt - Vt * Vt, Rn = wn - Yt * Yt - sn * sn + hn * hn, Pn = wn - Qt * Qt - Ut * Ut + Wt * Wt, Ie = vn * dn - rn * Ht, rt = (dn * Pn - Ht * Rn) / (2 * Ie) - Ct, ft = (Ht * jt - dn * cn) / Ie, pt = (vn * Rn - rn * Pn) / (2 * Ie) - Kt, wt = (rn * cn - vn * jt) / Ie, Bt = ft * ft + wt * wt - 1, At = 2 * (Vt + rt * ft + pt * wt), Dt = rt * rt + pt * pt - Vt * Vt, Ot = -(Bt ? (At + Math.sqrt(At * At - 4 * Bt * Dt)) / (2 * Bt) : Dt / At);
              return { x: Ct + rt + ft * Ot, y: Kt + pt + wt * Ot, r: Ot };
            }
            function Se(Ft, Tt, It) {
              var Ct, Kt, Vt, Yt, sn = Ft.x - Tt.x, hn = Ft.y - Tt.y, Qt = sn * sn + hn * hn;
              Qt ? (Kt = Tt.r + It.r, Kt *= Kt, Yt = Ft.r + It.r, Kt > (Yt *= Yt) ? (Ct = (Qt + Yt - Kt) / (2 * Qt), Vt = Math.sqrt(Math.max(0, Yt / Qt - Ct * Ct)), It.x = Ft.x - Ct * sn - Vt * hn, It.y = Ft.y - Ct * hn + Vt * sn) : (Ct = (Qt + Kt - Yt) / (2 * Qt), Vt = Math.sqrt(Math.max(0, Kt / Qt - Ct * Ct)), It.x = Tt.x + Ct * sn - Vt * hn, It.y = Tt.y + Ct * hn + Vt * sn)) : (It.x = Tt.x + It.r, It.y = Tt.y);
            }
            function we(Ft, Tt) {
              var It = Ft.r + Tt.r - 1e-6, Ct = Tt.x - Ft.x, Kt = Tt.y - Ft.y;
              return It > 0 && It * It > Ct * Ct + Kt * Kt;
            }
            function Ee(Ft) {
              var Tt = Ft._, It = Ft.next._, Ct = Tt.r + It.r, Kt = (Tt.x * It.r + It.x * Tt.r) / Ct, Vt = (Tt.y * It.r + It.y * Tt.r) / Ct;
              return Kt * Kt + Vt * Vt;
            }
            function xe(Ft) {
              this._ = Ft, this.next = null, this.previous = null;
            }
            function Le(Ft) {
              if (!(Kt = Ft.length)) return 0;
              var Tt, It, Ct, Kt, Vt, Yt, sn, hn, Qt, Ut, Wt;
              if ((Tt = Ft[0]).x = 0, Tt.y = 0, !(Kt > 1)) return Tt.r;
              if (It = Ft[1], Tt.x = -It.r, It.x = Tt.r, It.y = 0, !(Kt > 2)) return Tt.r + It.r;
              Se(It, Tt, Ct = Ft[2]), Tt = new xe(Tt), It = new xe(It), Ct = new xe(Ct), Tt.next = Ct.previous = It, It.next = Tt.previous = Ct, Ct.next = It.previous = Tt;
              e: for (sn = 3; sn < Kt; ++sn) {
                Se(Tt._, It._, Ct = Ft[sn]), Ct = new xe(Ct), hn = It.next, Qt = Tt.previous, Ut = It._.r, Wt = Tt._.r;
                do
                  if (Ut <= Wt) {
                    if (we(hn._, Ct._)) {
                      It = hn, Tt.next = It, It.previous = Tt, --sn;
                      continue e;
                    }
                    Ut += hn._.r, hn = hn.next;
                  } else {
                    if (we(Qt._, Ct._)) {
                      (Tt = Qt).next = It, It.previous = Tt, --sn;
                      continue e;
                    }
                    Wt += Qt._.r, Qt = Qt.previous;
                  }
                while (hn !== Qt.next);
                for (Ct.previous = Tt, Ct.next = It, Tt.next = It.previous = It = Ct, Vt = Ee(Tt); (Ct = Ct.next) !== It; ) (Yt = Ee(Ct)) < Vt && (Tt = Ct, Vt = Yt);
                It = Tt.next;
              }
              for (Tt = [It._], Ct = It; (Ct = Ct.next) !== It; ) Tt.push(Ct._);
              for (Ct = he(Tt), sn = 0; sn < Kt; ++sn) (Tt = Ft[sn]).x -= Ct.x, Tt.y -= Ct.y;
              return Ct.r;
            }
            function Qe(Ft) {
              return Le(Ft), Ft;
            }
            function ke(Ft) {
              if (typeof Ft != "function") throw new Error();
              return Ft;
            }
            function Me() {
              return 0;
            }
            function Re(Ft) {
              return function() {
                return Ft;
              };
            }
            function De(Ft) {
              return Math.sqrt(Ft.value);
            }
            function Pe() {
              var Ft = null, Tt = 1, It = 1, Ct = Me;
              function Kt(Vt) {
                return Vt.x = Tt / 2, Vt.y = It / 2, Ft ? Vt.eachBefore($e(Ft)).eachAfter(qe(Ct, 0.5)).eachBefore(Xe(1)) : Vt.eachBefore($e(De)).eachAfter(qe(Me, 1)).eachAfter(qe(Ct, Vt.r / Math.min(Tt, It))).eachBefore(Xe(Math.min(Tt, It) / (2 * Vt.r))), Vt;
              }
              return Kt.radius = function(Vt) {
                return arguments.length ? (Ft = (Yt = Vt) == null ? null : ke(Yt), Kt) : Ft;
                var Yt;
              }, Kt.size = function(Vt) {
                return arguments.length ? (Tt = +Vt[0], It = +Vt[1], Kt) : [Tt, It];
              }, Kt.padding = function(Vt) {
                return arguments.length ? (Ct = typeof Vt == "function" ? Vt : Re(+Vt), Kt) : Ct;
              }, Kt;
            }
            function $e(Ft) {
              return function(Tt) {
                Tt.children || (Tt.r = Math.max(0, +Ft(Tt) || 0));
              };
            }
            function qe(Ft, Tt) {
              return function(It) {
                if (Ct = It.children) {
                  var Ct, Kt, Vt, Yt = Ct.length, sn = Ft(It) * Tt || 0;
                  if (sn) for (Kt = 0; Kt < Yt; ++Kt) Ct[Kt].r += sn;
                  if (Vt = Le(Ct), sn) for (Kt = 0; Kt < Yt; ++Kt) Ct[Kt].r -= sn;
                  It.r = Vt + sn;
                }
              };
            }
            function Xe(Ft) {
              return function(Tt) {
                var It = Tt.parent;
                Tt.r *= Ft, It && (Tt.x = It.x + Ft * Tt.x, Tt.y = It.y + Ft * Tt.y);
              };
            }
            function Ve(Ft) {
              Ft.x0 = Math.round(Ft.x0), Ft.y0 = Math.round(Ft.y0), Ft.x1 = Math.round(Ft.x1), Ft.y1 = Math.round(Ft.y1);
            }
            function Ze(Ft, Tt, It, Ct, Kt) {
              for (var Vt, Yt = Ft.children, sn = -1, hn = Yt.length, Qt = Ft.value && (Ct - Tt) / Ft.value; ++sn < hn; ) (Vt = Yt[sn]).y0 = It, Vt.y1 = Kt, Vt.x0 = Tt, Vt.x1 = Tt += Vt.value * Qt;
            }
            function Ge() {
              var Ft = 1, Tt = 1, It = 0, Ct = !1;
              function Kt(Vt) {
                var Yt = Vt.height + 1;
                return Vt.x0 = Vt.y0 = It, Vt.x1 = Ft, Vt.y1 = Tt / Yt, Vt.eachBefore(/* @__PURE__ */ function(sn, hn) {
                  return function(Qt) {
                    Qt.children && Ze(Qt, Qt.x0, sn * (Qt.depth + 1) / hn, Qt.x1, sn * (Qt.depth + 2) / hn);
                    var Ut = Qt.x0, Wt = Qt.y0, rn = Qt.x1 - It, vn = Qt.y1 - It;
                    rn < Ut && (Ut = rn = (Ut + rn) / 2), vn < Wt && (Wt = vn = (Wt + vn) / 2), Qt.x0 = Ut, Qt.y0 = Wt, Qt.x1 = rn, Qt.y1 = vn;
                  };
                }(Tt, Yt)), Ct && Vt.eachBefore(Ve), Vt;
              }
              return Kt.round = function(Vt) {
                return arguments.length ? (Ct = !!Vt, Kt) : Ct;
              }, Kt.size = function(Vt) {
                return arguments.length ? (Ft = +Vt[0], Tt = +Vt[1], Kt) : [Ft, Tt];
              }, Kt.padding = function(Vt) {
                return arguments.length ? (It = +Vt, Kt) : It;
              }, Kt;
            }
            var ze = "$", We = { depth: -1 }, at = {};
            function ct(Ft) {
              return Ft.id;
            }
            function et(Ft) {
              return Ft.parentId;
            }
            function dt() {
              var Ft = ct, Tt = et;
              function It(Ct) {
                var Kt, Vt, Yt, sn, hn, Qt, Ut, Wt = Ct.length, rn = new Array(Wt), vn = {};
                for (Vt = 0; Vt < Wt; ++Vt) Kt = Ct[Vt], hn = rn[Vt] = new de(Kt), (Qt = Ft(Kt, Vt, Ct)) != null && (Qt += "") && (vn[Ut = ze + (hn.id = Qt)] = Ut in vn ? at : hn);
                for (Vt = 0; Vt < Wt; ++Vt) if (hn = rn[Vt], (Qt = Tt(Ct[Vt], Vt, Ct)) != null && (Qt += "")) {
                  if (!(sn = vn[ze + Qt])) throw new Error("missing: " + Qt);
                  if (sn === at) throw new Error("ambiguous: " + Qt);
                  sn.children ? sn.children.push(hn) : sn.children = [hn], hn.parent = sn;
                } else {
                  if (Yt) throw new Error("multiple roots");
                  Yt = hn;
                }
                if (!Yt) throw new Error("no root");
                if (Yt.parent = We, Yt.eachBefore(function(dn) {
                  dn.depth = dn.parent.depth + 1, --Wt;
                }).eachBefore(ue), Yt.parent = null, Wt > 0) throw new Error("cycle");
                return Yt;
              }
              return It.id = function(Ct) {
                return arguments.length ? (Ft = ke(Ct), It) : Ft;
              }, It.parentId = function(Ct) {
                return arguments.length ? (Tt = ke(Ct), It) : Tt;
              }, It;
            }
            function Je(Ft, Tt) {
              return Ft.parent === Tt.parent ? 1 : 2;
            }
            function it(Ft) {
              var Tt = Ft.children;
              return Tt ? Tt[0] : Ft.t;
            }
            function st(Ft) {
              var Tt = Ft.children;
              return Tt ? Tt[Tt.length - 1] : Ft.t;
            }
            function gt(Ft, Tt, It) {
              var Ct = It / (Tt.i - Ft.i);
              Tt.c -= Ct, Tt.s += It, Ft.c += Ct, Tt.z += It, Tt.m += It;
            }
            function bt(Ft, Tt, It) {
              return Ft.a.parent === Tt.parent ? Ft.a : It;
            }
            function yt(Ft, Tt) {
              this._ = Ft, this.parent = null, this.children = null, this.A = null, this.a = this, this.z = 0, this.m = 0, this.c = 0, this.s = 0, this.t = null, this.i = Tt;
            }
            function St() {
              var Ft = Je, Tt = 1, It = 1, Ct = null;
              function Kt(hn) {
                var Qt = function(cn) {
                  for (var wn, Rn, Pn, Ie, rt, ft = new yt(cn, 0), pt = [ft]; wn = pt.pop(); ) if (Pn = wn._.children) for (wn.children = new Array(rt = Pn.length), Ie = rt - 1; Ie >= 0; --Ie) pt.push(Rn = wn.children[Ie] = new yt(Pn[Ie], Ie)), Rn.parent = wn;
                  return (ft.parent = new yt(null, 0)).children = [ft], ft;
                }(hn);
                if (Qt.eachAfter(Vt), Qt.parent.m = -Qt.z, Qt.eachBefore(Yt), Ct) hn.eachBefore(sn);
                else {
                  var Ut = hn, Wt = hn, rn = hn;
                  hn.eachBefore(function(cn) {
                    cn.x < Ut.x && (Ut = cn), cn.x > Wt.x && (Wt = cn), cn.depth > rn.depth && (rn = cn);
                  });
                  var vn = Ut === Wt ? 1 : Ft(Ut, Wt) / 2, dn = vn - Ut.x, Ht = Tt / (Wt.x + vn + dn), jt = It / (rn.depth || 1);
                  hn.eachBefore(function(cn) {
                    cn.x = (cn.x + dn) * Ht, cn.y = cn.depth * jt;
                  });
                }
                return hn;
              }
              function Vt(hn) {
                var Qt = hn.children, Ut = hn.parent.children, Wt = hn.i ? Ut[hn.i - 1] : null;
                if (Qt) {
                  (function(vn) {
                    for (var dn, Ht = 0, jt = 0, cn = vn.children, wn = cn.length; --wn >= 0; ) (dn = cn[wn]).z += Ht, dn.m += Ht, Ht += dn.s + (jt += dn.c);
                  })(hn);
                  var rn = (Qt[0].z + Qt[Qt.length - 1].z) / 2;
                  Wt ? (hn.z = Wt.z + Ft(hn._, Wt._), hn.m = hn.z - rn) : hn.z = rn;
                } else Wt && (hn.z = Wt.z + Ft(hn._, Wt._));
                hn.parent.A = function(vn, dn, Ht) {
                  if (dn) {
                    for (var jt, cn = vn, wn = vn, Rn = dn, Pn = cn.parent.children[0], Ie = cn.m, rt = wn.m, ft = Rn.m, pt = Pn.m; Rn = st(Rn), cn = it(cn), Rn && cn; ) Pn = it(Pn), (wn = st(wn)).a = vn, (jt = Rn.z + ft - cn.z - Ie + Ft(Rn._, cn._)) > 0 && (gt(bt(Rn, vn, Ht), vn, jt), Ie += jt, rt += jt), ft += Rn.m, Ie += cn.m, pt += Pn.m, rt += wn.m;
                    Rn && !st(wn) && (wn.t = Rn, wn.m += ft - rt), cn && !it(Pn) && (Pn.t = cn, Pn.m += Ie - pt, Ht = vn);
                  }
                  return Ht;
                }(hn, Wt, hn.parent.A || Ut[0]);
              }
              function Yt(hn) {
                hn._.x = hn.z + hn.parent.m, hn.m += hn.parent.m;
              }
              function sn(hn) {
                hn.x *= Tt, hn.y = hn.depth * It;
              }
              return Kt.separation = function(hn) {
                return arguments.length ? (Ft = hn, Kt) : Ft;
              }, Kt.size = function(hn) {
                return arguments.length ? (Ct = !1, Tt = +hn[0], It = +hn[1], Kt) : Ct ? null : [Tt, It];
              }, Kt.nodeSize = function(hn) {
                return arguments.length ? (Ct = !0, Tt = +hn[0], It = +hn[1], Kt) : Ct ? [Tt, It] : null;
              }, Kt;
            }
            function Lt(Ft, Tt, It, Ct, Kt) {
              for (var Vt, Yt = Ft.children, sn = -1, hn = Yt.length, Qt = Ft.value && (Kt - It) / Ft.value; ++sn < hn; ) (Vt = Yt[sn]).x0 = Tt, Vt.x1 = Ct, Vt.y0 = It, Vt.y1 = It += Vt.value * Qt;
            }
            yt.prototype = Object.create(de.prototype);
            var Rt = (1 + Math.sqrt(5)) / 2;
            function vt(Ft, Tt, It, Ct, Kt, Vt) {
              for (var Yt, sn, hn, Qt, Ut, Wt, rn, vn, dn, Ht, jt, cn = [], wn = Tt.children, Rn = 0, Pn = 0, Ie = wn.length, rt = Tt.value; Rn < Ie; ) {
                hn = Kt - It, Qt = Vt - Ct;
                do
                  Ut = wn[Pn++].value;
                while (!Ut && Pn < Ie);
                for (Wt = rn = Ut, jt = Ut * Ut * (Ht = Math.max(Qt / hn, hn / Qt) / (rt * Ft)), dn = Math.max(rn / jt, jt / Wt); Pn < Ie; ++Pn) {
                  if (Ut += sn = wn[Pn].value, sn < Wt && (Wt = sn), sn > rn && (rn = sn), jt = Ut * Ut * Ht, (vn = Math.max(rn / jt, jt / Wt)) > dn) {
                    Ut -= sn;
                    break;
                  }
                  dn = vn;
                }
                cn.push(Yt = { value: Ut, dice: hn < Qt, children: wn.slice(Rn, Pn) }), Yt.dice ? Ze(Yt, It, Ct, Kt, rt ? Ct += Qt * Ut / rt : Vt) : Lt(Yt, It, Ct, rt ? It += hn * Ut / rt : Kt, Vt), rt -= Ut, Rn = Pn;
              }
              return cn;
            }
            var $t = function Ft(Tt) {
              function It(Ct, Kt, Vt, Yt, sn) {
                vt(Tt, Ct, Kt, Vt, Yt, sn);
              }
              return It.ratio = function(Ct) {
                return Ft((Ct = +Ct) > 1 ? Ct : 1);
              }, It;
            }(Rt);
            function Pt() {
              var Ft = $t, Tt = !1, It = 1, Ct = 1, Kt = [0], Vt = Me, Yt = Me, sn = Me, hn = Me, Qt = Me;
              function Ut(rn) {
                return rn.x0 = rn.y0 = 0, rn.x1 = It, rn.y1 = Ct, rn.eachBefore(Wt), Kt = [0], Tt && rn.eachBefore(Ve), rn;
              }
              function Wt(rn) {
                var vn = Kt[rn.depth], dn = rn.x0 + vn, Ht = rn.y0 + vn, jt = rn.x1 - vn, cn = rn.y1 - vn;
                jt < dn && (dn = jt = (dn + jt) / 2), cn < Ht && (Ht = cn = (Ht + cn) / 2), rn.x0 = dn, rn.y0 = Ht, rn.x1 = jt, rn.y1 = cn, rn.children && (vn = Kt[rn.depth + 1] = Vt(rn) / 2, dn += Qt(rn) - vn, Ht += Yt(rn) - vn, (jt -= sn(rn) - vn) < dn && (dn = jt = (dn + jt) / 2), (cn -= hn(rn) - vn) < Ht && (Ht = cn = (Ht + cn) / 2), Ft(rn, dn, Ht, jt, cn));
              }
              return Ut.round = function(rn) {
                return arguments.length ? (Tt = !!rn, Ut) : Tt;
              }, Ut.size = function(rn) {
                return arguments.length ? (It = +rn[0], Ct = +rn[1], Ut) : [It, Ct];
              }, Ut.tile = function(rn) {
                return arguments.length ? (Ft = ke(rn), Ut) : Ft;
              }, Ut.padding = function(rn) {
                return arguments.length ? Ut.paddingInner(rn).paddingOuter(rn) : Ut.paddingInner();
              }, Ut.paddingInner = function(rn) {
                return arguments.length ? (Vt = typeof rn == "function" ? rn : Re(+rn), Ut) : Vt;
              }, Ut.paddingOuter = function(rn) {
                return arguments.length ? Ut.paddingTop(rn).paddingRight(rn).paddingBottom(rn).paddingLeft(rn) : Ut.paddingTop();
              }, Ut.paddingTop = function(rn) {
                return arguments.length ? (Yt = typeof rn == "function" ? rn : Re(+rn), Ut) : Yt;
              }, Ut.paddingRight = function(rn) {
                return arguments.length ? (sn = typeof rn == "function" ? rn : Re(+rn), Ut) : sn;
              }, Ut.paddingBottom = function(rn) {
                return arguments.length ? (hn = typeof rn == "function" ? rn : Re(+rn), Ut) : hn;
              }, Ut.paddingLeft = function(rn) {
                return arguments.length ? (Qt = typeof rn == "function" ? rn : Re(+rn), Ut) : Qt;
              }, Ut;
            }
            function Xt(Ft, Tt, It, Ct, Kt) {
              var Vt, Yt, sn = Ft.children, hn = sn.length, Qt = new Array(hn + 1);
              for (Qt[0] = Yt = Vt = 0; Vt < hn; ++Vt) Qt[Vt + 1] = Yt += sn[Vt].value;
              (function Ut(Wt, rn, vn, dn, Ht, jt, cn) {
                if (Wt >= rn - 1) {
                  var wn = sn[Wt];
                  return wn.x0 = dn, wn.y0 = Ht, wn.x1 = jt, void (wn.y1 = cn);
                }
                for (var Rn = Qt[Wt], Pn = vn / 2 + Rn, Ie = Wt + 1, rt = rn - 1; Ie < rt; ) {
                  var ft = Ie + rt >>> 1;
                  Qt[ft] < Pn ? Ie = ft + 1 : rt = ft;
                }
                Pn - Qt[Ie - 1] < Qt[Ie] - Pn && Wt + 1 < Ie && --Ie;
                var pt = Qt[Ie] - Rn, wt = vn - pt;
                if (jt - dn > cn - Ht) {
                  var Bt = (dn * wt + jt * pt) / vn;
                  Ut(Wt, Ie, pt, dn, Ht, Bt, cn), Ut(Ie, rn, wt, Bt, Ht, jt, cn);
                } else {
                  var At = (Ht * wt + cn * pt) / vn;
                  Ut(Wt, Ie, pt, dn, Ht, jt, At), Ut(Ie, rn, wt, dn, At, jt, cn);
                }
              })(0, hn, Ft.value, Tt, It, Ct, Kt);
            }
            function Nt(Ft, Tt, It, Ct, Kt) {
              (1 & Ft.depth ? Lt : Ze)(Ft, Tt, It, Ct, Kt);
            }
            var Gt = function Ft(Tt) {
              function It(Ct, Kt, Vt, Yt, sn) {
                if ((hn = Ct._squarify) && hn.ratio === Tt) for (var hn, Qt, Ut, Wt, rn, vn = -1, dn = hn.length, Ht = Ct.value; ++vn < dn; ) {
                  for (Ut = (Qt = hn[vn]).children, Wt = Qt.value = 0, rn = Ut.length; Wt < rn; ++Wt) Qt.value += Ut[Wt].value;
                  Qt.dice ? Ze(Qt, Kt, Vt, Yt, Vt += (sn - Vt) * Qt.value / Ht) : Lt(Qt, Kt, Vt, Kt += (Yt - Kt) * Qt.value / Ht, sn), Ht -= Qt.value;
                }
                else Ct._squarify = hn = vt(Tt, Ct, Kt, Vt, Yt, sn), hn.ratio = Tt;
              }
              return It.ratio = function(Ct) {
                return Ft((Ct = +Ct) > 1 ? Ct : 1);
              }, It;
            }(Rt);
          }, 48544: function(G, J, Z) {
            Z.d(J, { pq: function() {
              return ye;
            } });
            var te = Math.PI, ne = 2 * te, ee = 1e-6, ae = ne - ee;
            function oe() {
              this._x0 = this._y0 = this._x1 = this._y1 = null, this._ = "";
            }
            function se() {
              return new oe();
            }
            oe.prototype = se.prototype = { constructor: oe, moveTo: function(ge, ve) {
              this._ += "M" + (this._x0 = this._x1 = +ge) + "," + (this._y0 = this._y1 = +ve);
            }, closePath: function() {
              this._x1 !== null && (this._x1 = this._x0, this._y1 = this._y0, this._ += "Z");
            }, lineTo: function(ge, ve) {
              this._ += "L" + (this._x1 = +ge) + "," + (this._y1 = +ve);
            }, quadraticCurveTo: function(ge, ve, be, Ce) {
              this._ += "Q" + +ge + "," + +ve + "," + (this._x1 = +be) + "," + (this._y1 = +Ce);
            }, bezierCurveTo: function(ge, ve, be, Ce, Se, we) {
              this._ += "C" + +ge + "," + +ve + "," + +be + "," + +Ce + "," + (this._x1 = +Se) + "," + (this._y1 = +we);
            }, arcTo: function(ge, ve, be, Ce, Se) {
              ge = +ge, ve = +ve, be = +be, Ce = +Ce, Se = +Se;
              var we = this._x1, Ee = this._y1, xe = be - ge, Le = Ce - ve, Qe = we - ge, ke = Ee - ve, Me = Qe * Qe + ke * ke;
              if (Se < 0) throw new Error("negative radius: " + Se);
              if (this._x1 === null) this._ += "M" + (this._x1 = ge) + "," + (this._y1 = ve);
              else if (Me > ee) if (Math.abs(ke * xe - Le * Qe) > ee && Se) {
                var Re = be - we, De = Ce - Ee, Pe = xe * xe + Le * Le, $e = Re * Re + De * De, qe = Math.sqrt(Pe), Xe = Math.sqrt(Me), Ve = Se * Math.tan((te - Math.acos((Pe + Me - $e) / (2 * qe * Xe))) / 2), Ze = Ve / Xe, Ge = Ve / qe;
                Math.abs(Ze - 1) > ee && (this._ += "L" + (ge + Ze * Qe) + "," + (ve + Ze * ke)), this._ += "A" + Se + "," + Se + ",0,0," + +(ke * Re > Qe * De) + "," + (this._x1 = ge + Ge * xe) + "," + (this._y1 = ve + Ge * Le);
              } else this._ += "L" + (this._x1 = ge) + "," + (this._y1 = ve);
            }, arc: function(ge, ve, be, Ce, Se, we) {
              ge = +ge, ve = +ve, we = !!we;
              var Ee = (be = +be) * Math.cos(Ce), xe = be * Math.sin(Ce), Le = ge + Ee, Qe = ve + xe, ke = 1 ^ we, Me = we ? Ce - Se : Se - Ce;
              if (be < 0) throw new Error("negative radius: " + be);
              this._x1 === null ? this._ += "M" + Le + "," + Qe : (Math.abs(this._x1 - Le) > ee || Math.abs(this._y1 - Qe) > ee) && (this._ += "L" + Le + "," + Qe), be && (Me < 0 && (Me = Me % ne + ne), Me > ae ? this._ += "A" + be + "," + be + ",0,1," + ke + "," + (ge - Ee) + "," + (ve - xe) + "A" + be + "," + be + ",0,1," + ke + "," + (this._x1 = Le) + "," + (this._y1 = Qe) : Me > ee && (this._ += "A" + be + "," + be + ",0," + +(Me >= te) + "," + ke + "," + (this._x1 = ge + be * Math.cos(Se)) + "," + (this._y1 = ve + be * Math.sin(Se))));
            }, rect: function(ge, ve, be, Ce) {
              this._ += "M" + (this._x0 = this._x1 = +ge) + "," + (this._y0 = this._y1 = +ve) + "h" + +be + "v" + +Ce + "h" + -be + "Z";
            }, toString: function() {
              return this._;
            } };
            var ce = se, le = Array.prototype.slice;
            function ue(ge) {
              return function() {
                return ge;
              };
            }
            function de(ge) {
              return ge[0];
            }
            function fe(ge) {
              return ge[1];
            }
            function he(ge) {
              return ge.source;
            }
            function me(ge) {
              return ge.target;
            }
            function pe(ge, ve, be, Ce, Se) {
              ge.moveTo(ve, be), ge.bezierCurveTo(ve = (ve + Ce) / 2, be, ve, Se, Ce, Se);
            }
            function ye() {
              return function(ge) {
                var ve = he, be = me, Ce = de, Se = fe, we = null;
                function Ee() {
                  var xe, Le = le.call(arguments), Qe = ve.apply(this, Le), ke = be.apply(this, Le);
                  if (we || (we = xe = ce()), ge(we, +Ce.apply(this, (Le[0] = Qe, Le)), +Se.apply(this, Le), +Ce.apply(this, (Le[0] = ke, Le)), +Se.apply(this, Le)), xe) return we = null, xe + "" || null;
                }
                return Ee.source = function(xe) {
                  return arguments.length ? (ve = xe, Ee) : ve;
                }, Ee.target = function(xe) {
                  return arguments.length ? (be = xe, Ee) : be;
                }, Ee.x = function(xe) {
                  return arguments.length ? (Ce = typeof xe == "function" ? xe : ue(+xe), Ee) : Ce;
                }, Ee.y = function(xe) {
                  return arguments.length ? (Se = typeof xe == "function" ? xe : ue(+xe), Ee) : Se;
                }, Ee.context = function(xe) {
                  return arguments.length ? (we = xe ?? null, Ee) : we;
                }, Ee;
              }(pe);
            }
          }, 42696: function(G, J, Z) {
            Z.d(J, { DC: function() {
              return he;
            }, de: function() {
              return de;
            }, aL: function() {
              return me;
            } });
            var te = Z(1681), ne = Z(72543), ee = Z(55735), ae = Z(47265), oe = Z(9830), se = Z(59764);
            function ce(Ie) {
              if (0 <= Ie.y && Ie.y < 100) {
                var rt = new Date(-1, Ie.m, Ie.d, Ie.H, Ie.M, Ie.S, Ie.L);
                return rt.setFullYear(Ie.y), rt;
              }
              return new Date(Ie.y, Ie.m, Ie.d, Ie.H, Ie.M, Ie.S, Ie.L);
            }
            function le(Ie) {
              if (0 <= Ie.y && Ie.y < 100) {
                var rt = new Date(Date.UTC(-1, Ie.m, Ie.d, Ie.H, Ie.M, Ie.S, Ie.L));
                return rt.setUTCFullYear(Ie.y), rt;
              }
              return new Date(Date.UTC(Ie.y, Ie.m, Ie.d, Ie.H, Ie.M, Ie.S, Ie.L));
            }
            function ue(Ie, rt, ft) {
              return { y: Ie, m: rt, d: ft, H: 0, M: 0, S: 0, L: 0 };
            }
            function de(Ie) {
              var rt = Ie.dateTime, ft = Ie.date, pt = Ie.time, wt = Ie.periods, Bt = Ie.days, At = Ie.shortDays, Dt = Ie.months, Ot = Ie.shortMonths, un = Se(wt), yn = we(wt), _n = Se(Bt), gn = we(Bt), bn = Se(At), an = we(At), Zt = Se(Dt), Mn = we(Dt), Ln = Se(Ot), In = we(Ot), kn = { a: function(Lr) {
                return At[Lr.getDay()];
              }, A: function(Lr) {
                return Bt[Lr.getDay()];
              }, b: function(Lr) {
                return Ot[Lr.getMonth()];
              }, B: function(Lr) {
                return Dt[Lr.getMonth()];
              }, c: null, d: dt, e: dt, f: bt, H: Je, I: it, j: st, L: gt, m: yt, M: St, p: function(Lr) {
                return wt[+(Lr.getHours() >= 12)];
              }, q: function(Lr) {
                return 1 + ~~(Lr.getMonth() / 3);
              }, Q: Rn, s: Pn, S: Lt, u: Rt, U: vt, V: $t, w: Pt, W: Xt, x: null, X: null, y: Nt, Y: Gt, Z: Ft, "%": wn }, er = { a: function(Lr) {
                return At[Lr.getUTCDay()];
              }, A: function(Lr) {
                return Bt[Lr.getUTCDay()];
              }, b: function(Lr) {
                return Ot[Lr.getUTCMonth()];
              }, B: function(Lr) {
                return Dt[Lr.getUTCMonth()];
              }, c: null, d: Tt, e: Tt, f: Yt, H: It, I: Ct, j: Kt, L: Vt, m: sn, M: hn, p: function(Lr) {
                return wt[+(Lr.getUTCHours() >= 12)];
              }, q: function(Lr) {
                return 1 + ~~(Lr.getUTCMonth() / 3);
              }, Q: Rn, s: Pn, S: Qt, u: Ut, U: Wt, V: rn, w: vn, W: dn, x: null, X: null, y: Ht, Y: jt, Z: cn, "%": wn }, jn = { a: function(Lr, mr, Ar) {
                var Yr = bn.exec(mr.slice(Ar));
                return Yr ? (Lr.w = an[Yr[0].toLowerCase()], Ar + Yr[0].length) : -1;
              }, A: function(Lr, mr, Ar) {
                var Yr = _n.exec(mr.slice(Ar));
                return Yr ? (Lr.w = gn[Yr[0].toLowerCase()], Ar + Yr[0].length) : -1;
              }, b: function(Lr, mr, Ar) {
                var Yr = Ln.exec(mr.slice(Ar));
                return Yr ? (Lr.m = In[Yr[0].toLowerCase()], Ar + Yr[0].length) : -1;
              }, B: function(Lr, mr, Ar) {
                var Yr = Zt.exec(mr.slice(Ar));
                return Yr ? (Lr.m = Mn[Yr[0].toLowerCase()], Ar + Yr[0].length) : -1;
              }, c: function(Lr, mr, Ar) {
                return Jn(Lr, rt, mr, Ar);
              }, d: qe, e: qe, f: We, H: Ve, I: Ve, j: Xe, L: ze, m: $e, M: Ze, p: function(Lr, mr, Ar) {
                var Yr = un.exec(mr.slice(Ar));
                return Yr ? (Lr.p = yn[Yr[0].toLowerCase()], Ar + Yr[0].length) : -1;
              }, q: Pe, Q: ct, s: et, S: Ge, u: xe, U: Le, V: Qe, w: Ee, W: ke, x: function(Lr, mr, Ar) {
                return Jn(Lr, ft, mr, Ar);
              }, X: function(Lr, mr, Ar) {
                return Jn(Lr, pt, mr, Ar);
              }, y: Re, Y: Me, Z: De, "%": at };
              function ur(Lr, mr) {
                return function(Ar) {
                  var Yr, jr, Mr, Wr = [], Bi = -1, gi = 0, Zr = Lr.length;
                  for (Ar instanceof Date || (Ar = /* @__PURE__ */ new Date(+Ar)); ++Bi < Zr; ) Lr.charCodeAt(Bi) === 37 && (Wr.push(Lr.slice(gi, Bi)), (jr = pe[Yr = Lr.charAt(++Bi)]) != null ? Yr = Lr.charAt(++Bi) : jr = Yr === "e" ? " " : "0", (Mr = mr[Yr]) && (Yr = Mr(Ar, jr)), Wr.push(Yr), gi = Bi + 1);
                  return Wr.push(Lr.slice(gi, Bi)), Wr.join("");
                };
              }
              function Kn(Lr, mr) {
                return function(Ar) {
                  var Yr, jr, Mr = ue(1900, void 0, 1);
                  if (Jn(Mr, Lr, Ar += "", 0) != Ar.length) return null;
                  if ("Q" in Mr) return new Date(Mr.Q);
                  if ("s" in Mr) return new Date(1e3 * Mr.s + ("L" in Mr ? Mr.L : 0));
                  if (mr && !("Z" in Mr) && (Mr.Z = 0), "p" in Mr && (Mr.H = Mr.H % 12 + 12 * Mr.p), Mr.m === void 0 && (Mr.m = "q" in Mr ? Mr.q : 0), "V" in Mr) {
                    if (Mr.V < 1 || Mr.V > 53) return null;
                    "w" in Mr || (Mr.w = 1), "Z" in Mr ? (jr = (Yr = le(ue(Mr.y, 0, 1))).getUTCDay(), Yr = jr > 4 || jr === 0 ? te.rt.ceil(Yr) : (0, te.rt)(Yr), Yr = ne.A.offset(Yr, 7 * (Mr.V - 1)), Mr.y = Yr.getUTCFullYear(), Mr.m = Yr.getUTCMonth(), Mr.d = Yr.getUTCDate() + (Mr.w + 6) % 7) : (jr = (Yr = ce(ue(Mr.y, 0, 1))).getDay(), Yr = jr > 4 || jr === 0 ? ee.By.ceil(Yr) : (0, ee.By)(Yr), Yr = ae.A.offset(Yr, 7 * (Mr.V - 1)), Mr.y = Yr.getFullYear(), Mr.m = Yr.getMonth(), Mr.d = Yr.getDate() + (Mr.w + 6) % 7);
                  } else ("W" in Mr || "U" in Mr) && ("w" in Mr || (Mr.w = "u" in Mr ? Mr.u % 7 : "W" in Mr ? 1 : 0), jr = "Z" in Mr ? le(ue(Mr.y, 0, 1)).getUTCDay() : ce(ue(Mr.y, 0, 1)).getDay(), Mr.m = 0, Mr.d = "W" in Mr ? (Mr.w + 6) % 7 + 7 * Mr.W - (jr + 5) % 7 : Mr.w + 7 * Mr.U - (jr + 6) % 7);
                  return "Z" in Mr ? (Mr.H += Mr.Z / 100 | 0, Mr.M += Mr.Z % 100, le(Mr)) : ce(Mr);
                };
              }
              function Jn(Lr, mr, Ar, Yr) {
                for (var jr, Mr, Wr = 0, Bi = mr.length, gi = Ar.length; Wr < Bi; ) {
                  if (Yr >= gi) return -1;
                  if ((jr = mr.charCodeAt(Wr++)) === 37) {
                    if (jr = mr.charAt(Wr++), !(Mr = jn[jr in pe ? mr.charAt(Wr++) : jr]) || (Yr = Mr(Lr, Ar, Yr)) < 0) return -1;
                  } else if (jr != Ar.charCodeAt(Yr++)) return -1;
                }
                return Yr;
              }
              return kn.x = ur(ft, kn), kn.X = ur(pt, kn), kn.c = ur(rt, kn), er.x = ur(ft, er), er.X = ur(pt, er), er.c = ur(rt, er), { format: function(Lr) {
                var mr = ur(Lr += "", kn);
                return mr.toString = function() {
                  return Lr;
                }, mr;
              }, parse: function(Lr) {
                var mr = Kn(Lr += "", !1);
                return mr.toString = function() {
                  return Lr;
                }, mr;
              }, utcFormat: function(Lr) {
                var mr = ur(Lr += "", er);
                return mr.toString = function() {
                  return Lr;
                }, mr;
              }, utcParse: function(Lr) {
                var mr = Kn(Lr += "", !0);
                return mr.toString = function() {
                  return Lr;
                }, mr;
              } };
            }
            var fe, he, me, pe = { "-": "", _: " ", 0: "0" }, ye = /^\s*\d+/, ge = /^%/, ve = /[\\^$*+?|[\]().{}]/g;
            function be(Ie, rt, ft) {
              var pt = Ie < 0 ? "-" : "", wt = (pt ? -Ie : Ie) + "", Bt = wt.length;
              return pt + (Bt < ft ? new Array(ft - Bt + 1).join(rt) + wt : wt);
            }
            function Ce(Ie) {
              return Ie.replace(ve, "\\$&");
            }
            function Se(Ie) {
              return new RegExp("^(?:" + Ie.map(Ce).join("|") + ")", "i");
            }
            function we(Ie) {
              for (var rt = {}, ft = -1, pt = Ie.length; ++ft < pt; ) rt[Ie[ft].toLowerCase()] = ft;
              return rt;
            }
            function Ee(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 1));
              return pt ? (Ie.w = +pt[0], ft + pt[0].length) : -1;
            }
            function xe(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 1));
              return pt ? (Ie.u = +pt[0], ft + pt[0].length) : -1;
            }
            function Le(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.U = +pt[0], ft + pt[0].length) : -1;
            }
            function Qe(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.V = +pt[0], ft + pt[0].length) : -1;
            }
            function ke(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.W = +pt[0], ft + pt[0].length) : -1;
            }
            function Me(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 4));
              return pt ? (Ie.y = +pt[0], ft + pt[0].length) : -1;
            }
            function Re(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.y = +pt[0] + (+pt[0] > 68 ? 1900 : 2e3), ft + pt[0].length) : -1;
            }
            function De(Ie, rt, ft) {
              var pt = /^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(rt.slice(ft, ft + 6));
              return pt ? (Ie.Z = pt[1] ? 0 : -(pt[2] + (pt[3] || "00")), ft + pt[0].length) : -1;
            }
            function Pe(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 1));
              return pt ? (Ie.q = 3 * pt[0] - 3, ft + pt[0].length) : -1;
            }
            function $e(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.m = pt[0] - 1, ft + pt[0].length) : -1;
            }
            function qe(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.d = +pt[0], ft + pt[0].length) : -1;
            }
            function Xe(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 3));
              return pt ? (Ie.m = 0, Ie.d = +pt[0], ft + pt[0].length) : -1;
            }
            function Ve(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.H = +pt[0], ft + pt[0].length) : -1;
            }
            function Ze(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.M = +pt[0], ft + pt[0].length) : -1;
            }
            function Ge(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 2));
              return pt ? (Ie.S = +pt[0], ft + pt[0].length) : -1;
            }
            function ze(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 3));
              return pt ? (Ie.L = +pt[0], ft + pt[0].length) : -1;
            }
            function We(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft, ft + 6));
              return pt ? (Ie.L = Math.floor(pt[0] / 1e3), ft + pt[0].length) : -1;
            }
            function at(Ie, rt, ft) {
              var pt = ge.exec(rt.slice(ft, ft + 1));
              return pt ? ft + pt[0].length : -1;
            }
            function ct(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft));
              return pt ? (Ie.Q = +pt[0], ft + pt[0].length) : -1;
            }
            function et(Ie, rt, ft) {
              var pt = ye.exec(rt.slice(ft));
              return pt ? (Ie.s = +pt[0], ft + pt[0].length) : -1;
            }
            function dt(Ie, rt) {
              return be(Ie.getDate(), rt, 2);
            }
            function Je(Ie, rt) {
              return be(Ie.getHours(), rt, 2);
            }
            function it(Ie, rt) {
              return be(Ie.getHours() % 12 || 12, rt, 2);
            }
            function st(Ie, rt) {
              return be(1 + ae.A.count((0, oe.A)(Ie), Ie), rt, 3);
            }
            function gt(Ie, rt) {
              return be(Ie.getMilliseconds(), rt, 3);
            }
            function bt(Ie, rt) {
              return gt(Ie, rt) + "000";
            }
            function yt(Ie, rt) {
              return be(Ie.getMonth() + 1, rt, 2);
            }
            function St(Ie, rt) {
              return be(Ie.getMinutes(), rt, 2);
            }
            function Lt(Ie, rt) {
              return be(Ie.getSeconds(), rt, 2);
            }
            function Rt(Ie) {
              var rt = Ie.getDay();
              return rt === 0 ? 7 : rt;
            }
            function vt(Ie, rt) {
              return be(ee.fz.count((0, oe.A)(Ie) - 1, Ie), rt, 2);
            }
            function $t(Ie, rt) {
              var ft = Ie.getDay();
              return Ie = ft >= 4 || ft === 0 ? (0, ee.dt)(Ie) : ee.dt.ceil(Ie), be(ee.dt.count((0, oe.A)(Ie), Ie) + ((0, oe.A)(Ie).getDay() === 4), rt, 2);
            }
            function Pt(Ie) {
              return Ie.getDay();
            }
            function Xt(Ie, rt) {
              return be(ee.By.count((0, oe.A)(Ie) - 1, Ie), rt, 2);
            }
            function Nt(Ie, rt) {
              return be(Ie.getFullYear() % 100, rt, 2);
            }
            function Gt(Ie, rt) {
              return be(Ie.getFullYear() % 1e4, rt, 4);
            }
            function Ft(Ie) {
              var rt = Ie.getTimezoneOffset();
              return (rt > 0 ? "-" : (rt *= -1, "+")) + be(rt / 60 | 0, "0", 2) + be(rt % 60, "0", 2);
            }
            function Tt(Ie, rt) {
              return be(Ie.getUTCDate(), rt, 2);
            }
            function It(Ie, rt) {
              return be(Ie.getUTCHours(), rt, 2);
            }
            function Ct(Ie, rt) {
              return be(Ie.getUTCHours() % 12 || 12, rt, 2);
            }
            function Kt(Ie, rt) {
              return be(1 + ne.A.count((0, se.A)(Ie), Ie), rt, 3);
            }
            function Vt(Ie, rt) {
              return be(Ie.getUTCMilliseconds(), rt, 3);
            }
            function Yt(Ie, rt) {
              return Vt(Ie, rt) + "000";
            }
            function sn(Ie, rt) {
              return be(Ie.getUTCMonth() + 1, rt, 2);
            }
            function hn(Ie, rt) {
              return be(Ie.getUTCMinutes(), rt, 2);
            }
            function Qt(Ie, rt) {
              return be(Ie.getUTCSeconds(), rt, 2);
            }
            function Ut(Ie) {
              var rt = Ie.getUTCDay();
              return rt === 0 ? 7 : rt;
            }
            function Wt(Ie, rt) {
              return be(te.Hl.count((0, se.A)(Ie) - 1, Ie), rt, 2);
            }
            function rn(Ie, rt) {
              var ft = Ie.getUTCDay();
              return Ie = ft >= 4 || ft === 0 ? (0, te.pT)(Ie) : te.pT.ceil(Ie), be(te.pT.count((0, se.A)(Ie), Ie) + ((0, se.A)(Ie).getUTCDay() === 4), rt, 2);
            }
            function vn(Ie) {
              return Ie.getUTCDay();
            }
            function dn(Ie, rt) {
              return be(te.rt.count((0, se.A)(Ie) - 1, Ie), rt, 2);
            }
            function Ht(Ie, rt) {
              return be(Ie.getUTCFullYear() % 100, rt, 2);
            }
            function jt(Ie, rt) {
              return be(Ie.getUTCFullYear() % 1e4, rt, 4);
            }
            function cn() {
              return "+0000";
            }
            function wn() {
              return "%";
            }
            function Rn(Ie) {
              return +Ie;
            }
            function Pn(Ie) {
              return Math.floor(+Ie / 1e3);
            }
            fe = de({ dateTime: "%x, %X", date: "%-m/%-d/%Y", time: "%-I:%M:%S %p", periods: ["AM", "PM"], days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], shortDays: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], months: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"], shortMonths: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"] }), he = fe.format, fe.parse, me = fe.utcFormat, fe.utcParse;
          }, 47265: function(G, J, Z) {
            Z.d(J, { _: function() {
              return ae;
            } });
            var te = Z(53398), ne = Z(66291), ee = (0, te.A)(function(oe) {
              oe.setHours(0, 0, 0, 0);
            }, function(oe, se) {
              oe.setDate(oe.getDate() + se);
            }, function(oe, se) {
              return (se - oe - (se.getTimezoneOffset() - oe.getTimezoneOffset()) * ne.rR) / ne.Nm;
            }, function(oe) {
              return oe.getDate() - 1;
            });
            J.A = ee;
            var ae = ee.range;
          }, 66291: function(G, J, Z) {
            Z.d(J, { Fq: function() {
              return oe;
            }, JJ: function() {
              return ee;
            }, Nm: function() {
              return ae;
            }, Tt: function() {
              return te;
            }, rR: function() {
              return ne;
            } });
            var te = 1e3, ne = 6e4, ee = 36e5, ae = 864e5, oe = 6048e5;
          }, 50936: function(G, J, Z) {
            Z.r(J), Z.d(J, { timeDay: function() {
              return ye.A;
            }, timeDays: function() {
              return ye._;
            }, timeFriday: function() {
              return ge.Sh;
            }, timeFridays: function() {
              return ge.tz;
            }, timeHour: function() {
              return me;
            }, timeHours: function() {
              return pe;
            }, timeInterval: function() {
              return te.A;
            }, timeMillisecond: function() {
              return ee;
            }, timeMilliseconds: function() {
              return ae;
            }, timeMinute: function() {
              return de;
            }, timeMinutes: function() {
              return fe;
            }, timeMonday: function() {
              return ge.By;
            }, timeMondays: function() {
              return ge.KP;
            }, timeMonth: function() {
              return be;
            }, timeMonths: function() {
              return Ce;
            }, timeSaturday: function() {
              return ge.kS;
            }, timeSaturdays: function() {
              return ge.t$;
            }, timeSecond: function() {
              return ce;
            }, timeSeconds: function() {
              return le;
            }, timeSunday: function() {
              return ge.fz;
            }, timeSundays: function() {
              return ge.se;
            }, timeThursday: function() {
              return ge.dt;
            }, timeThursdays: function() {
              return ge.Q$;
            }, timeTuesday: function() {
              return ge.eQ;
            }, timeTuesdays: function() {
              return ge.yW;
            }, timeWednesday: function() {
              return ge.l3;
            }, timeWednesdays: function() {
              return ge.gf;
            }, timeWeek: function() {
              return ge.fz;
            }, timeWeeks: function() {
              return ge.se;
            }, timeYear: function() {
              return Se.A;
            }, timeYears: function() {
              return Se.V;
            }, utcDay: function() {
              return Me.A;
            }, utcDays: function() {
              return Me.o;
            }, utcFriday: function() {
              return Re.a1;
            }, utcFridays: function() {
              return Re.Zn;
            }, utcHour: function() {
              return Qe;
            }, utcHours: function() {
              return ke;
            }, utcMillisecond: function() {
              return ee;
            }, utcMilliseconds: function() {
              return ae;
            }, utcMinute: function() {
              return Ee;
            }, utcMinutes: function() {
              return xe;
            }, utcMonday: function() {
              return Re.rt;
            }, utcMondays: function() {
              return Re.ON;
            }, utcMonth: function() {
              return Pe;
            }, utcMonths: function() {
              return $e;
            }, utcSaturday: function() {
              return Re.c8;
            }, utcSaturdays: function() {
              return Re.Xo;
            }, utcSecond: function() {
              return ce;
            }, utcSeconds: function() {
              return le;
            }, utcSunday: function() {
              return Re.Hl;
            }, utcSundays: function() {
              return Re.aZ;
            }, utcThursday: function() {
              return Re.pT;
            }, utcThursdays: function() {
              return Re.wr;
            }, utcTuesday: function() {
              return Re.sr;
            }, utcTuesdays: function() {
              return Re.jN;
            }, utcWednesday: function() {
              return Re.z2;
            }, utcWednesdays: function() {
              return Re.G6;
            }, utcWeek: function() {
              return Re.Hl;
            }, utcWeeks: function() {
              return Re.aZ;
            }, utcYear: function() {
              return qe.A;
            }, utcYears: function() {
              return qe.j;
            } });
            var te = Z(53398), ne = (0, te.A)(function() {
            }, function(Xe, Ve) {
              Xe.setTime(+Xe + Ve);
            }, function(Xe, Ve) {
              return Ve - Xe;
            });
            ne.every = function(Xe) {
              return Xe = Math.floor(Xe), isFinite(Xe) && Xe > 0 ? Xe > 1 ? (0, te.A)(function(Ve) {
                Ve.setTime(Math.floor(Ve / Xe) * Xe);
              }, function(Ve, Ze) {
                Ve.setTime(+Ve + Ze * Xe);
              }, function(Ve, Ze) {
                return (Ze - Ve) / Xe;
              }) : ne : null;
            };
            var ee = ne, ae = ne.range, oe = Z(66291), se = (0, te.A)(function(Xe) {
              Xe.setTime(Xe - Xe.getMilliseconds());
            }, function(Xe, Ve) {
              Xe.setTime(+Xe + Ve * oe.Tt);
            }, function(Xe, Ve) {
              return (Ve - Xe) / oe.Tt;
            }, function(Xe) {
              return Xe.getUTCSeconds();
            }), ce = se, le = se.range, ue = (0, te.A)(function(Xe) {
              Xe.setTime(Xe - Xe.getMilliseconds() - Xe.getSeconds() * oe.Tt);
            }, function(Xe, Ve) {
              Xe.setTime(+Xe + Ve * oe.rR);
            }, function(Xe, Ve) {
              return (Ve - Xe) / oe.rR;
            }, function(Xe) {
              return Xe.getMinutes();
            }), de = ue, fe = ue.range, he = (0, te.A)(function(Xe) {
              Xe.setTime(Xe - Xe.getMilliseconds() - Xe.getSeconds() * oe.Tt - Xe.getMinutes() * oe.rR);
            }, function(Xe, Ve) {
              Xe.setTime(+Xe + Ve * oe.JJ);
            }, function(Xe, Ve) {
              return (Ve - Xe) / oe.JJ;
            }, function(Xe) {
              return Xe.getHours();
            }), me = he, pe = he.range, ye = Z(47265), ge = Z(55735), ve = (0, te.A)(function(Xe) {
              Xe.setDate(1), Xe.setHours(0, 0, 0, 0);
            }, function(Xe, Ve) {
              Xe.setMonth(Xe.getMonth() + Ve);
            }, function(Xe, Ve) {
              return Ve.getMonth() - Xe.getMonth() + 12 * (Ve.getFullYear() - Xe.getFullYear());
            }, function(Xe) {
              return Xe.getMonth();
            }), be = ve, Ce = ve.range, Se = Z(9830), we = (0, te.A)(function(Xe) {
              Xe.setUTCSeconds(0, 0);
            }, function(Xe, Ve) {
              Xe.setTime(+Xe + Ve * oe.rR);
            }, function(Xe, Ve) {
              return (Ve - Xe) / oe.rR;
            }, function(Xe) {
              return Xe.getUTCMinutes();
            }), Ee = we, xe = we.range, Le = (0, te.A)(function(Xe) {
              Xe.setUTCMinutes(0, 0, 0);
            }, function(Xe, Ve) {
              Xe.setTime(+Xe + Ve * oe.JJ);
            }, function(Xe, Ve) {
              return (Ve - Xe) / oe.JJ;
            }, function(Xe) {
              return Xe.getUTCHours();
            }), Qe = Le, ke = Le.range, Me = Z(72543), Re = Z(1681), De = (0, te.A)(function(Xe) {
              Xe.setUTCDate(1), Xe.setUTCHours(0, 0, 0, 0);
            }, function(Xe, Ve) {
              Xe.setUTCMonth(Xe.getUTCMonth() + Ve);
            }, function(Xe, Ve) {
              return Ve.getUTCMonth() - Xe.getUTCMonth() + 12 * (Ve.getUTCFullYear() - Xe.getUTCFullYear());
            }, function(Xe) {
              return Xe.getUTCMonth();
            }), Pe = De, $e = De.range, qe = Z(59764);
          }, 53398: function(G, J, Z) {
            Z.d(J, { A: function() {
              return ee;
            } });
            var te = /* @__PURE__ */ new Date(), ne = /* @__PURE__ */ new Date();
            function ee(ae, oe, se, ce) {
              function le(ue) {
                return ae(ue = arguments.length === 0 ? /* @__PURE__ */ new Date() : /* @__PURE__ */ new Date(+ue)), ue;
              }
              return le.floor = function(ue) {
                return ae(ue = /* @__PURE__ */ new Date(+ue)), ue;
              }, le.ceil = function(ue) {
                return ae(ue = new Date(ue - 1)), oe(ue, 1), ae(ue), ue;
              }, le.round = function(ue) {
                var de = le(ue), fe = le.ceil(ue);
                return ue - de < fe - ue ? de : fe;
              }, le.offset = function(ue, de) {
                return oe(ue = /* @__PURE__ */ new Date(+ue), de == null ? 1 : Math.floor(de)), ue;
              }, le.range = function(ue, de, fe) {
                var he, me = [];
                if (ue = le.ceil(ue), fe = fe == null ? 1 : Math.floor(fe), !(ue < de && fe > 0)) return me;
                do
                  me.push(he = /* @__PURE__ */ new Date(+ue)), oe(ue, fe), ae(ue);
                while (he < ue && ue < de);
                return me;
              }, le.filter = function(ue) {
                return ee(function(de) {
                  if (de >= de) for (; ae(de), !ue(de); ) de.setTime(de - 1);
                }, function(de, fe) {
                  if (de >= de) if (fe < 0) for (; ++fe <= 0; ) for (; oe(de, -1), !ue(de); ) ;
                  else for (; --fe >= 0; ) for (; oe(de, 1), !ue(de); ) ;
                });
              }, se && (le.count = function(ue, de) {
                return te.setTime(+ue), ne.setTime(+de), ae(te), ae(ne), Math.floor(se(te, ne));
              }, le.every = function(ue) {
                return ue = Math.floor(ue), isFinite(ue) && ue > 0 ? ue > 1 ? le.filter(ce ? function(de) {
                  return ce(de) % ue == 0;
                } : function(de) {
                  return le.count(0, de) % ue == 0;
                }) : le : null;
              }), le;
            }
          }, 72543: function(G, J, Z) {
            Z.d(J, { o: function() {
              return ae;
            } });
            var te = Z(53398), ne = Z(66291), ee = (0, te.A)(function(oe) {
              oe.setUTCHours(0, 0, 0, 0);
            }, function(oe, se) {
              oe.setUTCDate(oe.getUTCDate() + se);
            }, function(oe, se) {
              return (se - oe) / ne.Nm;
            }, function(oe) {
              return oe.getUTCDate() - 1;
            });
            J.A = ee;
            var ae = ee.range;
          }, 1681: function(G, J, Z) {
            Z.d(J, { G6: function() {
              return pe;
            }, Hl: function() {
              return ae;
            }, ON: function() {
              return he;
            }, Xo: function() {
              return ve;
            }, Zn: function() {
              return ge;
            }, a1: function() {
              return ue;
            }, aZ: function() {
              return fe;
            }, c8: function() {
              return de;
            }, jN: function() {
              return me;
            }, pT: function() {
              return le;
            }, rt: function() {
              return oe;
            }, sr: function() {
              return se;
            }, wr: function() {
              return ye;
            }, z2: function() {
              return ce;
            } });
            var te = Z(53398), ne = Z(66291);
            function ee(be) {
              return (0, te.A)(function(Ce) {
                Ce.setUTCDate(Ce.getUTCDate() - (Ce.getUTCDay() + 7 - be) % 7), Ce.setUTCHours(0, 0, 0, 0);
              }, function(Ce, Se) {
                Ce.setUTCDate(Ce.getUTCDate() + 7 * Se);
              }, function(Ce, Se) {
                return (Se - Ce) / ne.Fq;
              });
            }
            var ae = ee(0), oe = ee(1), se = ee(2), ce = ee(3), le = ee(4), ue = ee(5), de = ee(6), fe = ae.range, he = oe.range, me = se.range, pe = ce.range, ye = le.range, ge = ue.range, ve = de.range;
          }, 59764: function(G, J, Z) {
            Z.d(J, { j: function() {
              return ee;
            } });
            var te = Z(53398), ne = (0, te.A)(function(ae) {
              ae.setUTCMonth(0, 1), ae.setUTCHours(0, 0, 0, 0);
            }, function(ae, oe) {
              ae.setUTCFullYear(ae.getUTCFullYear() + oe);
            }, function(ae, oe) {
              return oe.getUTCFullYear() - ae.getUTCFullYear();
            }, function(ae) {
              return ae.getUTCFullYear();
            });
            ne.every = function(ae) {
              return isFinite(ae = Math.floor(ae)) && ae > 0 ? (0, te.A)(function(oe) {
                oe.setUTCFullYear(Math.floor(oe.getUTCFullYear() / ae) * ae), oe.setUTCMonth(0, 1), oe.setUTCHours(0, 0, 0, 0);
              }, function(oe, se) {
                oe.setUTCFullYear(oe.getUTCFullYear() + se * ae);
              }) : null;
            }, J.A = ne;
            var ee = ne.range;
          }, 55735: function(G, J, Z) {
            Z.d(J, { By: function() {
              return oe;
            }, KP: function() {
              return he;
            }, Q$: function() {
              return ye;
            }, Sh: function() {
              return ue;
            }, dt: function() {
              return le;
            }, eQ: function() {
              return se;
            }, fz: function() {
              return ae;
            }, gf: function() {
              return pe;
            }, kS: function() {
              return de;
            }, l3: function() {
              return ce;
            }, se: function() {
              return fe;
            }, t$: function() {
              return ve;
            }, tz: function() {
              return ge;
            }, yW: function() {
              return me;
            } });
            var te = Z(53398), ne = Z(66291);
            function ee(be) {
              return (0, te.A)(function(Ce) {
                Ce.setDate(Ce.getDate() - (Ce.getDay() + 7 - be) % 7), Ce.setHours(0, 0, 0, 0);
              }, function(Ce, Se) {
                Ce.setDate(Ce.getDate() + 7 * Se);
              }, function(Ce, Se) {
                return (Se - Ce - (Se.getTimezoneOffset() - Ce.getTimezoneOffset()) * ne.rR) / ne.Fq;
              });
            }
            var ae = ee(0), oe = ee(1), se = ee(2), ce = ee(3), le = ee(4), ue = ee(5), de = ee(6), fe = ae.range, he = oe.range, me = se.range, pe = ce.range, ye = le.range, ge = ue.range, ve = de.range;
          }, 9830: function(G, J, Z) {
            Z.d(J, { V: function() {
              return ee;
            } });
            var te = Z(53398), ne = (0, te.A)(function(ae) {
              ae.setMonth(0, 1), ae.setHours(0, 0, 0, 0);
            }, function(ae, oe) {
              ae.setFullYear(ae.getFullYear() + oe);
            }, function(ae, oe) {
              return oe.getFullYear() - ae.getFullYear();
            }, function(ae) {
              return ae.getFullYear();
            });
            ne.every = function(ae) {
              return isFinite(ae = Math.floor(ae)) && ae > 0 ? (0, te.A)(function(oe) {
                oe.setFullYear(Math.floor(oe.getFullYear() / ae) * ae), oe.setMonth(0, 1), oe.setHours(0, 0, 0, 0);
              }, function(oe, se) {
                oe.setFullYear(oe.getFullYear() + se * ae);
              }) : null;
            }, J.A = ne;
            var ee = ne.range;
          }, 70973: function(G, J, Z) {
            var te = Z(40891), ne = Z(98800), ee = Z(48631), ae = Z(52991);
            G.exports = function(oe, se, ce) {
              if (!oe || typeof oe != "object" && typeof oe != "function") throw new ee("`obj` must be an object or a function`");
              if (typeof se != "string" && typeof se != "symbol") throw new ee("`property` must be a string or a symbol`");
              if (arguments.length > 3 && typeof arguments[3] != "boolean" && arguments[3] !== null) throw new ee("`nonEnumerable`, if provided, must be a boolean or null");
              if (arguments.length > 4 && typeof arguments[4] != "boolean" && arguments[4] !== null) throw new ee("`nonWritable`, if provided, must be a boolean or null");
              if (arguments.length > 5 && typeof arguments[5] != "boolean" && arguments[5] !== null) throw new ee("`nonConfigurable`, if provided, must be a boolean or null");
              if (arguments.length > 6 && typeof arguments[6] != "boolean") throw new ee("`loose`, if provided, must be a boolean");
              var le = arguments.length > 3 ? arguments[3] : null, ue = arguments.length > 4 ? arguments[4] : null, de = arguments.length > 5 ? arguments[5] : null, fe = arguments.length > 6 && arguments[6], he = !!ae && ae(oe, se);
              if (te) te(oe, se, { configurable: de === null && he ? he.configurable : !de, enumerable: le === null && he ? he.enumerable : !le, value: ce, writable: ue === null && he ? he.writable : !ue });
              else {
                if (!fe && (le || ue || de)) throw new ne("This environment does not support defining a property as non-configurable, non-writable, or non-enumerable.");
                oe[se] = ce;
              }
            };
          }, 97936: function(G, J, Z) {
            var te = Z(99433), ne = typeof Symbol == "function" && typeof Symbol("foo") == "symbol", ee = Object.prototype.toString, ae = Array.prototype.concat, oe = Object.defineProperty, se = Z(74268)(), ce = oe && se, le = function(de, fe, he, me) {
              if (fe in de) {
                if (me === !0) {
                  if (de[fe] === he) return;
                } else if (typeof (pe = me) != "function" || ee.call(pe) !== "[object Function]" || !me()) return;
              }
              var pe;
              ce ? oe(de, fe, { configurable: !0, enumerable: !1, value: he, writable: !0 }) : de[fe] = he;
            }, ue = function(de, fe) {
              var he = arguments.length > 2 ? arguments[2] : {}, me = te(fe);
              ne && (me = ae.call(me, Object.getOwnPropertySymbols(fe)));
              for (var pe = 0; pe < me.length; pe += 1) le(de, me[pe], fe[me[pe]], he[me[pe]]);
            };
            ue.supportsDescriptors = !!ce, G.exports = ue;
          }, 68650: function(G) {
            G.exports = function() {
              for (var J = 0; J < arguments.length; J++) if (arguments[J] !== void 0) return arguments[J];
            };
          }, 44431: function(G) {
            G.exports = te;
            var J = (te.canvas = document.createElement("canvas")).getContext("2d"), Z = ne([32, 126]);
            function te(ee, ae) {
              Array.isArray(ee) && (ee = ee.join(", "));
              var oe, se = {}, ce = 16, le = 0.05;
              ae && (ae.length === 2 && typeof ae[0] == "number" ? oe = ne(ae) : Array.isArray(ae) ? oe = ae : (ae.o ? oe = ne(ae.o) : ae.pairs && (oe = ae.pairs), ae.fontSize && (ce = ae.fontSize), ae.threshold != null && (le = ae.threshold))), oe || (oe = Z), J.font = ce + "px " + ee;
              for (var ue = 0; ue < oe.length; ue++) {
                var de = oe[ue], fe = J.measureText(de[0]).width + J.measureText(de[1]).width, he = J.measureText(de).width;
                if (Math.abs(fe - he) > ce * le) {
                  var me = (he - fe) / ce;
                  se[de] = 1e3 * me;
                }
              }
              return se;
            }
            function ne(ee) {
              for (var ae = [], oe = ee[0]; oe <= ee[1]; oe++) for (var se = String.fromCharCode(oe), ce = ee[0]; ce < ee[1]; ce++) {
                var le = se + String.fromCharCode(ce);
                ae.push(le);
              }
              return ae;
            }
            te.createPairs = ne, te.ascii = Z;
          }, 95620: function(G, J, Z) {
            var te = Z(16844), ne = Z(60265), ee = { M: "moveTo", C: "bezierCurveTo" };
            G.exports = function(ae, oe) {
              ae.beginPath(), ne(te(oe)).forEach(function(se) {
                var ce = se[0], le = se.slice(1);
                ae[ee[ce]].apply(ae, le);
              }), ae.closePath();
            };
          }, 10275: function(G) {
            G.exports = function(J) {
              switch (J) {
                case "int8":
                  return Int8Array;
                case "int16":
                  return Int16Array;
                case "int32":
                  return Int32Array;
                case "uint8":
                  return Uint8Array;
                case "uint16":
                  return Uint16Array;
                case "uint32":
                  return Uint32Array;
                case "float32":
                  return Float32Array;
                case "float64":
                  return Float64Array;
                case "array":
                  return Array;
                case "uint8_clamped":
                  return Uint8ClampedArray;
              }
            };
          }, 49523: function(G) {
            function J(Z, te, ne) {
              var ee = 0 | Z[ne];
              if (ee <= 0) return [];
              var ae, oe = new Array(ee);
              if (ne === Z.length - 1) for (ae = 0; ae < ee; ++ae) oe[ae] = te;
              else for (ae = 0; ae < ee; ++ae) oe[ae] = J(Z, te, ne + 1);
              return oe;
            }
            G.exports = function(Z, te) {
              switch (te === void 0 && (te = 0), typeof Z) {
                case "number":
                  if (Z > 0) return function(ne, ee) {
                    var ae, oe;
                    for (ae = new Array(ne), oe = 0; oe < ne; ++oe) ae[oe] = ee;
                    return ae;
                  }(0 | Z, te);
                  break;
                case "object":
                  if (typeof Z.length == "number") return J(Z, te, 0);
              }
              return [];
            };
          }, 25782: function(G) {
            function J(Qe, ke, Me) {
              Me = Me || 2;
              var Re, De, Pe, $e, qe, Xe, Ve, Ze = ke && ke.length, Ge = Ze ? ke[0] * Me : Qe.length, ze = Z(Qe, 0, Ge, Me, !0), We = [];
              if (!ze || ze.next === ze.prev) return We;
              if (Ze && (ze = function(ct, et, dt, Je) {
                var it, st, gt, bt = [];
                for (it = 0, st = et.length; it < st; it++) (gt = Z(ct, et[it] * Je, it < st - 1 ? et[it + 1] * Je : ct.length, Je, !1)) === gt.next && (gt.steiner = !0), bt.push(fe(gt));
                for (bt.sort(ce), it = 0; it < bt.length; it++) dt = le(bt[it], dt);
                return dt;
              }(Qe, ke, ze, Me)), Qe.length > 80 * Me) {
                Re = Pe = Qe[0], De = $e = Qe[1];
                for (var at = Me; at < Ge; at += Me) (qe = Qe[at]) < Re && (Re = qe), (Xe = Qe[at + 1]) < De && (De = Xe), qe > Pe && (Pe = qe), Xe > $e && ($e = Xe);
                Ve = (Ve = Math.max(Pe - Re, $e - De)) !== 0 ? 32767 / Ve : 0;
              }
              return ne(ze, We, Me, Re, De, Ve, 0), We;
            }
            function Z(Qe, ke, Me, Re, De) {
              var Pe, $e;
              if (De === Le(Qe, ke, Me, Re) > 0) for (Pe = ke; Pe < Me; Pe += Re) $e = we(Pe, Qe[Pe], Qe[Pe + 1], $e);
              else for (Pe = Me - Re; Pe >= ke; Pe -= Re) $e = we(Pe, Qe[Pe], Qe[Pe + 1], $e);
              return $e && ye($e, $e.next) && (Ee($e), $e = $e.next), $e;
            }
            function te(Qe, ke) {
              if (!Qe) return Qe;
              ke || (ke = Qe);
              var Me, Re = Qe;
              do
                if (Me = !1, Re.steiner || !ye(Re, Re.next) && pe(Re.prev, Re, Re.next) !== 0) Re = Re.next;
                else {
                  if (Ee(Re), (Re = ke = Re.prev) === Re.next) break;
                  Me = !0;
                }
              while (Me || Re !== ke);
              return ke;
            }
            function ne(Qe, ke, Me, Re, De, Pe, $e) {
              if (Qe) {
                !$e && Pe && function(Ze, Ge, ze, We) {
                  var at = Ze;
                  do
                    at.z === 0 && (at.z = de(at.x, at.y, Ge, ze, We)), at.prevZ = at.prev, at.nextZ = at.next, at = at.next;
                  while (at !== Ze);
                  at.prevZ.nextZ = null, at.prevZ = null, function(ct) {
                    var et, dt, Je, it, st, gt, bt, yt, St = 1;
                    do {
                      for (dt = ct, ct = null, st = null, gt = 0; dt; ) {
                        for (gt++, Je = dt, bt = 0, et = 0; et < St && (bt++, Je = Je.nextZ); et++) ;
                        for (yt = St; bt > 0 || yt > 0 && Je; ) bt !== 0 && (yt === 0 || !Je || dt.z <= Je.z) ? (it = dt, dt = dt.nextZ, bt--) : (it = Je, Je = Je.nextZ, yt--), st ? st.nextZ = it : ct = it, it.prevZ = st, st = it;
                        dt = Je;
                      }
                      st.nextZ = null, St *= 2;
                    } while (gt > 1);
                  }(at);
                }(Qe, Re, De, Pe);
                for (var qe, Xe, Ve = Qe; Qe.prev !== Qe.next; ) if (qe = Qe.prev, Xe = Qe.next, Pe ? ae(Qe, Re, De, Pe) : ee(Qe)) ke.push(qe.i / Me | 0), ke.push(Qe.i / Me | 0), ke.push(Xe.i / Me | 0), Ee(Qe), Qe = Xe.next, Ve = Xe.next;
                else if ((Qe = Xe) === Ve) {
                  $e ? $e === 1 ? ne(Qe = oe(te(Qe), ke, Me), ke, Me, Re, De, Pe, 2) : $e === 2 && se(Qe, ke, Me, Re, De, Pe) : ne(te(Qe), ke, Me, Re, De, Pe, 1);
                  break;
                }
              }
            }
            function ee(Qe) {
              var ke = Qe.prev, Me = Qe, Re = Qe.next;
              if (pe(ke, Me, Re) >= 0) return !1;
              for (var De = ke.x, Pe = Me.x, $e = Re.x, qe = ke.y, Xe = Me.y, Ve = Re.y, Ze = De < Pe ? De < $e ? De : $e : Pe < $e ? Pe : $e, Ge = qe < Xe ? qe < Ve ? qe : Ve : Xe < Ve ? Xe : Ve, ze = De > Pe ? De > $e ? De : $e : Pe > $e ? Pe : $e, We = qe > Xe ? qe > Ve ? qe : Ve : Xe > Ve ? Xe : Ve, at = Re.next; at !== ke; ) {
                if (at.x >= Ze && at.x <= ze && at.y >= Ge && at.y <= We && he(De, qe, Pe, Xe, $e, Ve, at.x, at.y) && pe(at.prev, at, at.next) >= 0) return !1;
                at = at.next;
              }
              return !0;
            }
            function ae(Qe, ke, Me, Re) {
              var De = Qe.prev, Pe = Qe, $e = Qe.next;
              if (pe(De, Pe, $e) >= 0) return !1;
              for (var qe = De.x, Xe = Pe.x, Ve = $e.x, Ze = De.y, Ge = Pe.y, ze = $e.y, We = qe < Xe ? qe < Ve ? qe : Ve : Xe < Ve ? Xe : Ve, at = Ze < Ge ? Ze < ze ? Ze : ze : Ge < ze ? Ge : ze, ct = qe > Xe ? qe > Ve ? qe : Ve : Xe > Ve ? Xe : Ve, et = Ze > Ge ? Ze > ze ? Ze : ze : Ge > ze ? Ge : ze, dt = de(We, at, ke, Me, Re), Je = de(ct, et, ke, Me, Re), it = Qe.prevZ, st = Qe.nextZ; it && it.z >= dt && st && st.z <= Je; ) {
                if (it.x >= We && it.x <= ct && it.y >= at && it.y <= et && it !== De && it !== $e && he(qe, Ze, Xe, Ge, Ve, ze, it.x, it.y) && pe(it.prev, it, it.next) >= 0 || (it = it.prevZ, st.x >= We && st.x <= ct && st.y >= at && st.y <= et && st !== De && st !== $e && he(qe, Ze, Xe, Ge, Ve, ze, st.x, st.y) && pe(st.prev, st, st.next) >= 0)) return !1;
                st = st.nextZ;
              }
              for (; it && it.z >= dt; ) {
                if (it.x >= We && it.x <= ct && it.y >= at && it.y <= et && it !== De && it !== $e && he(qe, Ze, Xe, Ge, Ve, ze, it.x, it.y) && pe(it.prev, it, it.next) >= 0) return !1;
                it = it.prevZ;
              }
              for (; st && st.z <= Je; ) {
                if (st.x >= We && st.x <= ct && st.y >= at && st.y <= et && st !== De && st !== $e && he(qe, Ze, Xe, Ge, Ve, ze, st.x, st.y) && pe(st.prev, st, st.next) >= 0) return !1;
                st = st.nextZ;
              }
              return !0;
            }
            function oe(Qe, ke, Me) {
              var Re = Qe;
              do {
                var De = Re.prev, Pe = Re.next.next;
                !ye(De, Pe) && ge(De, Re, Re.next, Pe) && Ce(De, Pe) && Ce(Pe, De) && (ke.push(De.i / Me | 0), ke.push(Re.i / Me | 0), ke.push(Pe.i / Me | 0), Ee(Re), Ee(Re.next), Re = Qe = Pe), Re = Re.next;
              } while (Re !== Qe);
              return te(Re);
            }
            function se(Qe, ke, Me, Re, De, Pe) {
              var $e = Qe;
              do {
                for (var qe = $e.next.next; qe !== $e.prev; ) {
                  if ($e.i !== qe.i && me($e, qe)) {
                    var Xe = Se($e, qe);
                    return $e = te($e, $e.next), Xe = te(Xe, Xe.next), ne($e, ke, Me, Re, De, Pe, 0), void ne(Xe, ke, Me, Re, De, Pe, 0);
                  }
                  qe = qe.next;
                }
                $e = $e.next;
              } while ($e !== Qe);
            }
            function ce(Qe, ke) {
              return Qe.x - ke.x;
            }
            function le(Qe, ke) {
              var Me = function(De, Pe) {
                var $e, qe = Pe, Xe = De.x, Ve = De.y, Ze = -1 / 0;
                do {
                  if (Ve <= qe.y && Ve >= qe.next.y && qe.next.y !== qe.y) {
                    var Ge = qe.x + (Ve - qe.y) * (qe.next.x - qe.x) / (qe.next.y - qe.y);
                    if (Ge <= Xe && Ge > Ze && (Ze = Ge, $e = qe.x < qe.next.x ? qe : qe.next, Ge === Xe)) return $e;
                  }
                  qe = qe.next;
                } while (qe !== Pe);
                if (!$e) return null;
                var ze, We = $e, at = $e.x, ct = $e.y, et = 1 / 0;
                qe = $e;
                do
                  Xe >= qe.x && qe.x >= at && Xe !== qe.x && he(Ve < ct ? Xe : Ze, Ve, at, ct, Ve < ct ? Ze : Xe, Ve, qe.x, qe.y) && (ze = Math.abs(Ve - qe.y) / (Xe - qe.x), Ce(qe, De) && (ze < et || ze === et && (qe.x > $e.x || qe.x === $e.x && ue($e, qe))) && ($e = qe, et = ze)), qe = qe.next;
                while (qe !== We);
                return $e;
              }(Qe, ke);
              if (!Me) return ke;
              var Re = Se(Me, Qe);
              return te(Re, Re.next), te(Me, Me.next);
            }
            function ue(Qe, ke) {
              return pe(Qe.prev, Qe, ke.prev) < 0 && pe(ke.next, Qe, Qe.next) < 0;
            }
            function de(Qe, ke, Me, Re, De) {
              return (Qe = 1431655765 & ((Qe = 858993459 & ((Qe = 252645135 & ((Qe = 16711935 & ((Qe = (Qe - Me) * De | 0) | Qe << 8)) | Qe << 4)) | Qe << 2)) | Qe << 1)) | (ke = 1431655765 & ((ke = 858993459 & ((ke = 252645135 & ((ke = 16711935 & ((ke = (ke - Re) * De | 0) | ke << 8)) | ke << 4)) | ke << 2)) | ke << 1)) << 1;
            }
            function fe(Qe) {
              var ke = Qe, Me = Qe;
              do
                (ke.x < Me.x || ke.x === Me.x && ke.y < Me.y) && (Me = ke), ke = ke.next;
              while (ke !== Qe);
              return Me;
            }
            function he(Qe, ke, Me, Re, De, Pe, $e, qe) {
              return (De - $e) * (ke - qe) >= (Qe - $e) * (Pe - qe) && (Qe - $e) * (Re - qe) >= (Me - $e) * (ke - qe) && (Me - $e) * (Pe - qe) >= (De - $e) * (Re - qe);
            }
            function me(Qe, ke) {
              return Qe.next.i !== ke.i && Qe.prev.i !== ke.i && !function(Me, Re) {
                var De = Me;
                do {
                  if (De.i !== Me.i && De.next.i !== Me.i && De.i !== Re.i && De.next.i !== Re.i && ge(De, De.next, Me, Re)) return !0;
                  De = De.next;
                } while (De !== Me);
                return !1;
              }(Qe, ke) && (Ce(Qe, ke) && Ce(ke, Qe) && function(Me, Re) {
                var De = Me, Pe = !1, $e = (Me.x + Re.x) / 2, qe = (Me.y + Re.y) / 2;
                do
                  De.y > qe != De.next.y > qe && De.next.y !== De.y && $e < (De.next.x - De.x) * (qe - De.y) / (De.next.y - De.y) + De.x && (Pe = !Pe), De = De.next;
                while (De !== Me);
                return Pe;
              }(Qe, ke) && (pe(Qe.prev, Qe, ke.prev) || pe(Qe, ke.prev, ke)) || ye(Qe, ke) && pe(Qe.prev, Qe, Qe.next) > 0 && pe(ke.prev, ke, ke.next) > 0);
            }
            function pe(Qe, ke, Me) {
              return (ke.y - Qe.y) * (Me.x - ke.x) - (ke.x - Qe.x) * (Me.y - ke.y);
            }
            function ye(Qe, ke) {
              return Qe.x === ke.x && Qe.y === ke.y;
            }
            function ge(Qe, ke, Me, Re) {
              var De = be(pe(Qe, ke, Me)), Pe = be(pe(Qe, ke, Re)), $e = be(pe(Me, Re, Qe)), qe = be(pe(Me, Re, ke));
              return De !== Pe && $e !== qe || !(De !== 0 || !ve(Qe, Me, ke)) || !(Pe !== 0 || !ve(Qe, Re, ke)) || !($e !== 0 || !ve(Me, Qe, Re)) || !(qe !== 0 || !ve(Me, ke, Re));
            }
            function ve(Qe, ke, Me) {
              return ke.x <= Math.max(Qe.x, Me.x) && ke.x >= Math.min(Qe.x, Me.x) && ke.y <= Math.max(Qe.y, Me.y) && ke.y >= Math.min(Qe.y, Me.y);
            }
            function be(Qe) {
              return Qe > 0 ? 1 : Qe < 0 ? -1 : 0;
            }
            function Ce(Qe, ke) {
              return pe(Qe.prev, Qe, Qe.next) < 0 ? pe(Qe, ke, Qe.next) >= 0 && pe(Qe, Qe.prev, ke) >= 0 : pe(Qe, ke, Qe.prev) < 0 || pe(Qe, Qe.next, ke) < 0;
            }
            function Se(Qe, ke) {
              var Me = new xe(Qe.i, Qe.x, Qe.y), Re = new xe(ke.i, ke.x, ke.y), De = Qe.next, Pe = ke.prev;
              return Qe.next = ke, ke.prev = Qe, Me.next = De, De.prev = Me, Re.next = Me, Me.prev = Re, Pe.next = Re, Re.prev = Pe, Re;
            }
            function we(Qe, ke, Me, Re) {
              var De = new xe(Qe, ke, Me);
              return Re ? (De.next = Re.next, De.prev = Re, Re.next.prev = De, Re.next = De) : (De.prev = De, De.next = De), De;
            }
            function Ee(Qe) {
              Qe.next.prev = Qe.prev, Qe.prev.next = Qe.next, Qe.prevZ && (Qe.prevZ.nextZ = Qe.nextZ), Qe.nextZ && (Qe.nextZ.prevZ = Qe.prevZ);
            }
            function xe(Qe, ke, Me) {
              this.i = Qe, this.x = ke, this.y = Me, this.prev = null, this.next = null, this.z = 0, this.prevZ = null, this.nextZ = null, this.steiner = !1;
            }
            function Le(Qe, ke, Me, Re) {
              for (var De = 0, Pe = ke, $e = Me - Re; Pe < Me; Pe += Re) De += (Qe[$e] - Qe[Pe]) * (Qe[Pe + 1] + Qe[$e + 1]), $e = Pe;
              return De;
            }
            G.exports = J, G.exports.default = J, J.deviation = function(Qe, ke, Me, Re) {
              var De = ke && ke.length, Pe = De ? ke[0] * Me : Qe.length, $e = Math.abs(Le(Qe, 0, Pe, Me));
              if (De) for (var qe = 0, Xe = ke.length; qe < Xe; qe++) {
                var Ve = ke[qe] * Me, Ze = qe < Xe - 1 ? ke[qe + 1] * Me : Qe.length;
                $e -= Math.abs(Le(Qe, Ve, Ze, Me));
              }
              var Ge = 0;
              for (qe = 0; qe < Re.length; qe += 3) {
                var ze = Re[qe] * Me, We = Re[qe + 1] * Me, at = Re[qe + 2] * Me;
                Ge += Math.abs((Qe[ze] - Qe[at]) * (Qe[We + 1] - Qe[ze + 1]) - (Qe[ze] - Qe[We]) * (Qe[at + 1] - Qe[ze + 1]));
              }
              return $e === 0 && Ge === 0 ? 0 : Math.abs((Ge - $e) / $e);
            }, J.flatten = function(Qe) {
              for (var ke = Qe[0][0].length, Me = { vertices: [], holes: [], dimensions: ke }, Re = 0, De = 0; De < Qe.length; De++) {
                for (var Pe = 0; Pe < Qe[De].length; Pe++) for (var $e = 0; $e < ke; $e++) Me.vertices.push(Qe[De][Pe][$e]);
                De > 0 && (Re += Qe[De - 1].length, Me.holes.push(Re));
              }
              return Me;
            };
          }, 96143: function(G, J, Z) {
            var te = Z(26381);
            G.exports = function(ne, ee) {
              var ae, oe = [], se = [], ce = [], le = {}, ue = [];
              function de(Ce) {
                ce[Ce] = !1, le.hasOwnProperty(Ce) && Object.keys(le[Ce]).forEach(function(Se) {
                  delete le[Ce][Se], ce[Se] && de(Se);
                });
              }
              function fe(Ce) {
                var Se, we, Ee = !1;
                for (se.push(Ce), ce[Ce] = !0, Se = 0; Se < ue[Ce].length; Se++) (we = ue[Ce][Se]) === ae ? (he(ae, se), Ee = !0) : ce[we] || (Ee = fe(we));
                if (Ee) de(Ce);
                else for (Se = 0; Se < ue[Ce].length; Se++) {
                  we = ue[Ce][Se];
                  var xe = le[we];
                  xe || (xe = {}, le[we] = xe), xe[we] = !0;
                }
                return se.pop(), Ee;
              }
              function he(Ce, Se) {
                var we = [].concat(Se).concat(Ce);
                ee ? ee(fe) : oe.push(we);
              }
              function me(Ce) {
                (function(Me) {
                  for (var Re = 0; Re < ne.length; Re++) Re < Me && (ne[Re] = []), ne[Re] = ne[Re].filter(function(De) {
                    return De >= Me;
                  });
                })(Ce);
                for (var Se, we = te(ne).components.filter(function(Me) {
                  return Me.length > 1;
                }), Ee = 1 / 0, xe = 0; xe < we.length; xe++) for (var Le = 0; Le < we[xe].length; Le++) we[xe][Le] < Ee && (Ee = we[xe][Le], Se = xe);
                var Qe = we[Se];
                if (!Qe) return !1;
                var ke = ne.map(function(Me, Re) {
                  return Qe.indexOf(Re) === -1 ? [] : Me.filter(function(De) {
                    return Qe.indexOf(De) !== -1;
                  });
                });
                return { leastVertex: Ee, adjList: ke };
              }
              ae = 0;
              for (var pe = ne.length; ae < pe; ) {
                var ye = me(ae);
                if (ae = ye.leastVertex, ue = ye.adjList) {
                  for (var ge = 0; ge < ue.length; ge++) for (var ve = 0; ve < ue[ge].length; ve++) {
                    var be = ue[ge][ve];
                    ce[+be] = !1, le[be] = {};
                  }
                  fe(ae), ae += 1;
                } else ae = pe;
              }
              return ee ? void 0 : oe;
            };
          }, 40891: function(G, J, Z) {
            var te = Z(71129)("%Object.defineProperty%", !0) || !1;
            if (te) try {
              te({}, "a", { value: 1 });
            } catch {
              te = !1;
            }
            G.exports = te;
          }, 35465: function(G) {
            G.exports = EvalError;
          }, 77731: function(G) {
            G.exports = Error;
          }, 30582: function(G) {
            G.exports = RangeError;
          }, 50294: function(G) {
            G.exports = ReferenceError;
          }, 98800: function(G) {
            G.exports = SyntaxError;
          }, 48631: function(G) {
            G.exports = TypeError;
          }, 33149: function(G) {
            G.exports = URIError;
          }, 91445: function(G, J, Z) {
            var te = Z(69746);
            G.exports = function() {
              return te(this).length = 0, this;
            };
          }, 82377: function(G, J, Z) {
            G.exports = Z(57712)() ? Array.from : Z(33468);
          }, 57712: function(G) {
            G.exports = function() {
              var J, Z, te = Array.from;
              return typeof te == "function" && (Z = te(J = ["raz", "dwa"]), !!(Z && Z !== J && Z[1] === "dwa"));
            };
          }, 33468: function(G, J, Z) {
            var te = Z(63008).iterator, ne = Z(82262), ee = Z(59356), ae = Z(54653), oe = Z(52359), se = Z(69746), ce = Z(1974), le = Z(48488), ue = Array.isArray, de = Function.prototype.call, fe = { configurable: !0, enumerable: !0, writable: !0, value: null }, he = Object.defineProperty;
            G.exports = function(me) {
              var pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le = arguments[1], Qe = arguments[2];
              if (me = Object(se(me)), ce(Le) && oe(Le), this && this !== Array && ee(this)) pe = this;
              else {
                if (!Le) {
                  if (ne(me)) return (be = me.length) !== 1 ? Array.apply(null, me) : ((ve = new Array(1))[0] = me[0], ve);
                  if (ue(me)) {
                    for (ve = new Array(be = me.length), ye = 0; ye < be; ++ye) ve[ye] = me[ye];
                    return ve;
                  }
                }
                ve = [];
              }
              if (!ue(me)) {
                if ((Ee = me[te]) !== void 0) {
                  for (Se = oe(Ee).call(me), pe && (ve = new pe()), we = Se.next(), ye = 0; !we.done; ) xe = Le ? de.call(Le, Qe, we.value, ye) : we.value, pe ? (fe.value = xe, he(ve, ye, fe)) : ve[ye] = xe, we = Se.next(), ++ye;
                  be = ye;
                } else if (le(me)) {
                  for (be = me.length, pe && (ve = new pe()), ye = 0, ge = 0; ye < be; ++ye) xe = me[ye], ye + 1 < be && (Ce = xe.charCodeAt(0)) >= 55296 && Ce <= 56319 && (xe += me[++ye]), xe = Le ? de.call(Le, Qe, xe, ge) : xe, pe ? (fe.value = xe, he(ve, ge, fe)) : ve[ge] = xe, ++ge;
                  be = ge;
                }
              }
              if (be === void 0) for (be = ae(me.length), pe && (ve = new pe(be)), ye = 0; ye < be; ++ye) xe = Le ? de.call(Le, Qe, me[ye], ye) : me[ye], pe ? (fe.value = xe, he(ve, ye, fe)) : ve[ye] = xe;
              return pe && (fe.value = null, ve.length = be), ve;
            };
          }, 82262: function(G) {
            var J = Object.prototype.toString, Z = J.call(/* @__PURE__ */ function() {
              return arguments;
            }());
            G.exports = function(te) {
              return J.call(te) === Z;
            };
          }, 59356: function(G) {
            var J = Object.prototype.toString, Z = RegExp.prototype.test.bind(/^[object [A-Za-z0-9]*Function]$/);
            G.exports = function(te) {
              return typeof te == "function" && Z(J.call(te));
            };
          }, 62039: function(G) {
            G.exports = function() {
            };
          }, 53579: function(G, J, Z) {
            G.exports = Z(67394)() ? Math.sign : Z(37122);
          }, 67394: function(G) {
            G.exports = function() {
              var J = Math.sign;
              return typeof J == "function" && J(10) === 1 && J(-20) === -1;
            };
          }, 37122: function(G) {
            G.exports = function(J) {
              return J = Number(J), isNaN(J) || J === 0 ? J : J > 0 ? 1 : -1;
            };
          }, 10226: function(G, J, Z) {
            var te = Z(53579), ne = Math.abs, ee = Math.floor;
            G.exports = function(ae) {
              return isNaN(ae) ? 0 : (ae = Number(ae)) !== 0 && isFinite(ae) ? te(ae) * ee(ne(ae)) : ae;
            };
          }, 54653: function(G, J, Z) {
            var te = Z(10226), ne = Math.max;
            G.exports = function(ee) {
              return ne(0, te(ee));
            };
          }, 39395: function(G, J, Z) {
            var te = Z(52359), ne = Z(69746), ee = Function.prototype.bind, ae = Function.prototype.call, oe = Object.keys, se = Object.prototype.propertyIsEnumerable;
            G.exports = function(ce, le) {
              return function(ue, de) {
                var fe, he = arguments[2], me = arguments[3];
                return ue = Object(ne(ue)), te(de), fe = oe(ue), me && fe.sort(typeof me == "function" ? ee.call(me, ue) : void 0), typeof ce != "function" && (ce = fe[ce]), ae.call(ce, fe, function(pe, ye) {
                  return se.call(ue, pe) ? ae.call(de, he, ue[pe], pe, ue, ye) : le;
                });
              };
            };
          }, 1920: function(G, J, Z) {
            G.exports = Z(41271)() ? Object.assign : Z(26399);
          }, 41271: function(G) {
            G.exports = function() {
              var J, Z = Object.assign;
              return typeof Z == "function" && (Z(J = { foo: "raz" }, { bar: "dwa" }, { trzy: "trzy" }), J.foo + J.bar + J.trzy === "razdwatrzy");
            };
          }, 26399: function(G, J, Z) {
            var te = Z(36353), ne = Z(69746), ee = Math.max;
            G.exports = function(ae, oe) {
              var se, ce, le, ue = ee(arguments.length, 2);
              for (ae = Object(ne(ae)), le = function(de) {
                try {
                  ae[de] = oe[de];
                } catch (fe) {
                  se || (se = fe);
                }
              }, ce = 1; ce < ue; ++ce) te(oe = arguments[ce]).forEach(le);
              if (se !== void 0) throw se;
              return ae;
            };
          }, 86591: function(G, J, Z) {
            var te = Z(82377), ne = Z(1920), ee = Z(69746);
            G.exports = function(ae) {
              var oe = Object(ee(ae)), se = arguments[1], ce = Object(arguments[2]);
              if (oe !== ae && !se) return oe;
              var le = {};
              return se ? te(se, function(ue) {
                (ce.ensure || ue in ae) && (le[ue] = ae[ue]);
              }) : ne(le, ae), le;
            };
          }, 57842: function(G, J, Z) {
            var te, ne, ee, ae, oe = Object.create;
            Z(90361)() || (te = Z(45765)), G.exports = te ? te.level !== 1 ? oe : (ne = {}, ee = {}, ae = { configurable: !1, enumerable: !1, writable: !0, value: void 0 }, Object.getOwnPropertyNames(Object.prototype).forEach(function(se) {
              ee[se] = se !== "__proto__" ? ae : { configurable: !0, enumerable: !1, writable: !0, value: void 0 };
            }), Object.defineProperties(ne, ee), Object.defineProperty(te, "nullPolyfill", { configurable: !1, enumerable: !1, writable: !1, value: ne }), function(se, ce) {
              return oe(se === null ? ne : se, ce);
            }) : oe;
          }, 82813: function(G, J, Z) {
            G.exports = Z(39395)("forEach");
          }, 76064: function(G, J, Z) {
            var te = Z(1974), ne = { function: !0, object: !0 };
            G.exports = function(ee) {
              return te(ee) && ne[typeof ee] || !1;
            };
          }, 1974: function(G, J, Z) {
            var te = Z(62039)();
            G.exports = function(ne) {
              return ne !== te && ne !== null;
            };
          }, 36353: function(G, J, Z) {
            G.exports = Z(83800)() ? Object.keys : Z(67044);
          }, 83800: function(G) {
            G.exports = function() {
              try {
                return Object.keys("primitive"), !0;
              } catch {
                return !1;
              }
            };
          }, 67044: function(G, J, Z) {
            var te = Z(1974), ne = Object.keys;
            G.exports = function(ee) {
              return ne(te(ee) ? Object(ee) : ee);
            };
          }, 29854: function(G, J, Z) {
            var te = Z(52359), ne = Z(82813), ee = Function.prototype.call;
            G.exports = function(ae, oe) {
              var se = {}, ce = arguments[2];
              return te(oe), ne(ae, function(le, ue, de, fe) {
                se[ue] = ee.call(oe, ce, le, ue, de, fe);
              }), se;
            };
          }, 76504: function(G, J, Z) {
            var te = Z(1974), ne = Array.prototype.forEach, ee = Object.create;
            G.exports = function(ae) {
              var oe = ee(null);
              return ne.call(arguments, function(se) {
                te(se) && function(ce, le) {
                  var ue;
                  for (ue in ce) le[ue] = ce[ue];
                }(Object(se), oe);
              }), oe;
            };
          }, 22834: function(G, J, Z) {
            G.exports = Z(90361)() ? Object.setPrototypeOf : Z(45765);
          }, 90361: function(G) {
            var J = Object.create, Z = Object.getPrototypeOf, te = {};
            G.exports = function() {
              var ne = Object.setPrototypeOf;
              return typeof ne == "function" && Z(ne((arguments[0] || J)(null), te)) === te;
            };
          }, 45765: function(G, J, Z) {
            var te, ne, ee, ae, oe = Z(76064), se = Z(69746), ce = Object.prototype.isPrototypeOf, le = Object.defineProperty, ue = { configurable: !0, enumerable: !1, writable: !0, value: void 0 };
            te = function(de, fe) {
              if (se(de), fe === null || oe(fe)) return de;
              throw new TypeError("Prototype must be null or an object");
            }, G.exports = (ne = function() {
              var de, fe = /* @__PURE__ */ Object.create(null), he = {}, me = Object.getOwnPropertyDescriptor(Object.prototype, "__proto__");
              if (me) {
                try {
                  (de = me.set).call(fe, he);
                } catch {
                }
                if (Object.getPrototypeOf(fe) === he) return { set: de, level: 2 };
              }
              return fe.__proto__ = he, Object.getPrototypeOf(fe) === he ? { level: 2 } : ((fe = {}).__proto__ = he, Object.getPrototypeOf(fe) === he && { level: 1 });
            }(), ne ? (ne.level === 2 ? ne.set ? (ae = ne.set, ee = function(de, fe) {
              return ae.call(te(de, fe), fe), de;
            }) : ee = function(de, fe) {
              return te(de, fe).__proto__ = fe, de;
            } : ee = function de(fe, he) {
              var me;
              return te(fe, he), (me = ce.call(de.nullPolyfill, fe)) && delete de.nullPolyfill.__proto__, he === null && (he = de.nullPolyfill), fe.__proto__ = he, me && le(de.nullPolyfill, "__proto__", ue), fe;
            }, Object.defineProperty(ee, "level", { configurable: !1, enumerable: !1, writable: !1, value: ne.level })) : null), Z(57842);
          }, 52359: function(G) {
            G.exports = function(J) {
              if (typeof J != "function") throw new TypeError(J + " is not a function");
              return J;
            };
          }, 11004: function(G, J, Z) {
            var te = Z(76064);
            G.exports = function(ne) {
              if (!te(ne)) throw new TypeError(ne + " is not an Object");
              return ne;
            };
          }, 69746: function(G, J, Z) {
            var te = Z(1974);
            G.exports = function(ne) {
              if (!te(ne)) throw new TypeError("Cannot use null or undefined");
              return ne;
            };
          }, 2338: function(G, J, Z) {
            G.exports = Z(65961)() ? String.prototype.contains : Z(9461);
          }, 65961: function(G) {
            var J = "razdwatrzy";
            G.exports = function() {
              return typeof J.contains == "function" && J.contains("dwa") === !0 && J.contains("foo") === !1;
            };
          }, 9461: function(G) {
            var J = String.prototype.indexOf;
            G.exports = function(Z) {
              return J.call(this, Z, arguments[1]) > -1;
            };
          }, 48488: function(G) {
            var J = Object.prototype.toString, Z = J.call("");
            G.exports = function(te) {
              return typeof te == "string" || te && typeof te == "object" && (te instanceof String || J.call(te) === Z) || !1;
            };
          }, 43497: function(G) {
            var J = /* @__PURE__ */ Object.create(null), Z = Math.random;
            G.exports = function() {
              var te;
              do
                te = Z().toString(36).slice(2);
              while (J[te]);
              return te;
            };
          }, 71343: function(G, J, Z) {
            var te, ne = Z(22834), ee = Z(2338), ae = Z(91819), oe = Z(63008), se = Z(85490), ce = Object.defineProperty;
            te = G.exports = function(le, ue) {
              if (!(this instanceof te)) throw new TypeError("Constructor requires 'new'");
              se.call(this, le), ue = ue ? ee.call(ue, "key+value") ? "key+value" : ee.call(ue, "key") ? "key" : "value" : "value", ce(this, "__kind__", ae("", ue));
            }, ne && ne(te, se), delete te.prototype.constructor, te.prototype = Object.create(se.prototype, { _resolve: ae(function(le) {
              return this.__kind__ === "value" ? this.__list__[le] : this.__kind__ === "key+value" ? [le, this.__list__[le]] : le;
            }) }), ce(te.prototype, oe.toStringTag, ae("c", "Array Iterator"));
          }, 58755: function(G, J, Z) {
            var te = Z(82262), ne = Z(52359), ee = Z(48488), ae = Z(34494), oe = Array.isArray, se = Function.prototype.call, ce = Array.prototype.some;
            G.exports = function(le, ue) {
              var de, fe, he, me, pe, ye, ge, ve, be = arguments[2];
              if (oe(le) || te(le) ? de = "array" : ee(le) ? de = "string" : le = ae(le), ne(ue), he = function() {
                me = !0;
              }, de !== "array") if (de !== "string") for (fe = le.next(); !fe.done; ) {
                if (se.call(ue, be, fe.value, he), me) return;
                fe = le.next();
              }
              else for (ye = le.length, pe = 0; pe < ye && (ge = le[pe], pe + 1 < ye && (ve = ge.charCodeAt(0)) >= 55296 && ve <= 56319 && (ge += le[++pe]), se.call(ue, be, ge, he), !me); ++pe) ;
              else ce.call(le, function(Ce) {
                return se.call(ue, be, Ce, he), me;
              });
            };
          }, 34494: function(G, J, Z) {
            var te = Z(82262), ne = Z(48488), ee = Z(71343), ae = Z(23417), oe = Z(82831), se = Z(63008).iterator;
            G.exports = function(ce) {
              return typeof oe(ce)[se] == "function" ? ce[se]() : te(ce) ? new ee(ce) : ne(ce) ? new ae(ce) : new ee(ce);
            };
          }, 85490: function(G, J, Z) {
            var te, ne = Z(91445), ee = Z(1920), ae = Z(52359), oe = Z(69746), se = Z(91819), ce = Z(84510), le = Z(63008), ue = Object.defineProperty, de = Object.defineProperties;
            G.exports = te = function(fe, he) {
              if (!(this instanceof te)) throw new TypeError("Constructor requires 'new'");
              de(this, { __list__: se("w", oe(fe)), __context__: se("w", he), __nextIndex__: se("w", 0) }), he && (ae(he.on), he.on("_add", this._onAdd), he.on("_delete", this._onDelete), he.on("_clear", this._onClear));
            }, delete te.prototype.constructor, de(te.prototype, ee({ _next: se(function() {
              var fe;
              if (this.__list__) return this.__redo__ && (fe = this.__redo__.shift()) !== void 0 ? fe : this.__nextIndex__ < this.__list__.length ? this.__nextIndex__++ : void this._unBind();
            }), next: se(function() {
              return this._createResult(this._next());
            }), _createResult: se(function(fe) {
              return fe === void 0 ? { done: !0, value: void 0 } : { done: !1, value: this._resolve(fe) };
            }), _resolve: se(function(fe) {
              return this.__list__[fe];
            }), _unBind: se(function() {
              this.__list__ = null, delete this.__redo__, this.__context__ && (this.__context__.off("_add", this._onAdd), this.__context__.off("_delete", this._onDelete), this.__context__.off("_clear", this._onClear), this.__context__ = null);
            }), toString: se(function() {
              return "[object " + (this[le.toStringTag] || "Object") + "]";
            }) }, ce({ _onAdd: se(function(fe) {
              fe >= this.__nextIndex__ || (++this.__nextIndex__, this.__redo__ ? (this.__redo__.forEach(function(he, me) {
                he >= fe && (this.__redo__[me] = ++he);
              }, this), this.__redo__.push(fe)) : ue(this, "__redo__", se("c", [fe])));
            }), _onDelete: se(function(fe) {
              var he;
              fe >= this.__nextIndex__ || (--this.__nextIndex__, this.__redo__ && ((he = this.__redo__.indexOf(fe)) !== -1 && this.__redo__.splice(he, 1), this.__redo__.forEach(function(me, pe) {
                me > fe && (this.__redo__[pe] = --me);
              }, this)));
            }), _onClear: se(function() {
              this.__redo__ && ne.call(this.__redo__), this.__nextIndex__ = 0;
            }) }))), ue(te.prototype, le.iterator, se(function() {
              return this;
            }));
          }, 50567: function(G, J, Z) {
            var te = Z(82262), ne = Z(1974), ee = Z(48488), ae = Z(63008).iterator, oe = Array.isArray;
            G.exports = function(se) {
              return !(!ne(se) || !oe(se) && !ee(se) && !te(se) && typeof se[ae] != "function");
            };
          }, 23417: function(G, J, Z) {
            var te, ne = Z(22834), ee = Z(91819), ae = Z(63008), oe = Z(85490), se = Object.defineProperty;
            te = G.exports = function(ce) {
              if (!(this instanceof te)) throw new TypeError("Constructor requires 'new'");
              ce = String(ce), oe.call(this, ce), se(this, "__length__", ee("", ce.length));
            }, ne && ne(te, oe), delete te.prototype.constructor, te.prototype = Object.create(oe.prototype, { _next: ee(function() {
              if (this.__list__) return this.__nextIndex__ < this.__length__ ? this.__nextIndex__++ : void this._unBind();
            }), _resolve: ee(function(ce) {
              var le, ue = this.__list__[ce];
              return this.__nextIndex__ === this.__length__ ? ue : (le = ue.charCodeAt(0)) >= 55296 && le <= 56319 ? ue + this.__list__[this.__nextIndex__++] : ue;
            }) }), se(te.prototype, ae.toStringTag, ee("c", "String Iterator"));
          }, 82831: function(G, J, Z) {
            var te = Z(50567);
            G.exports = function(ne) {
              if (!te(ne)) throw new TypeError(ne + " is not iterable");
              return ne;
            };
          }, 63008: function(G, J, Z) {
            G.exports = Z(25143)() ? Z(64725).Symbol : Z(81905);
          }, 25143: function(G, J, Z) {
            var te = Z(64725), ne = { object: !0, symbol: !0 };
            G.exports = function() {
              var ee, ae = te.Symbol;
              if (typeof ae != "function") return !1;
              ee = ae("test symbol");
              try {
                String(ee);
              } catch {
                return !1;
              }
              return !!ne[typeof ae.iterator] && !!ne[typeof ae.toPrimitive] && !!ne[typeof ae.toStringTag];
            };
          }, 41707: function(G) {
            G.exports = function(J) {
              return !!J && (typeof J == "symbol" || !!J.constructor && J.constructor.name === "Symbol" && J[J.constructor.toStringTag] === "Symbol");
            };
          }, 74009: function(G, J, Z) {
            var te = Z(91819), ne = Object.create, ee = Object.defineProperty, ae = Object.prototype, oe = ne(null);
            G.exports = function(se) {
              for (var ce, le, ue = 0; oe[se + (ue || "")]; ) ++ue;
              return oe[se += ue || ""] = !0, ee(ae, ce = "@@" + se, te.gs(null, function(de) {
                le || (le = !0, ee(this, ce, te(de)), le = !1);
              })), ce;
            };
          }, 40313: function(G, J, Z) {
            var te = Z(91819), ne = Z(64725).Symbol;
            G.exports = function(ee) {
              return Object.defineProperties(ee, { hasInstance: te("", ne && ne.hasInstance || ee("hasInstance")), isConcatSpreadable: te("", ne && ne.isConcatSpreadable || ee("isConcatSpreadable")), iterator: te("", ne && ne.iterator || ee("iterator")), match: te("", ne && ne.match || ee("match")), replace: te("", ne && ne.replace || ee("replace")), search: te("", ne && ne.search || ee("search")), species: te("", ne && ne.species || ee("species")), split: te("", ne && ne.split || ee("split")), toPrimitive: te("", ne && ne.toPrimitive || ee("toPrimitive")), toStringTag: te("", ne && ne.toStringTag || ee("toStringTag")), unscopables: te("", ne && ne.unscopables || ee("unscopables")) });
            };
          }, 21290: function(G, J, Z) {
            var te = Z(91819), ne = Z(91765), ee = /* @__PURE__ */ Object.create(null);
            G.exports = function(ae) {
              return Object.defineProperties(ae, { for: te(function(oe) {
                return ee[oe] ? ee[oe] : ee[oe] = ae(String(oe));
              }), keyFor: te(function(oe) {
                var se;
                for (se in ne(oe), ee) if (ee[se] === oe) return se;
              }) });
            };
          }, 81905: function(G, J, Z) {
            var te, ne, ee, ae = Z(91819), oe = Z(91765), se = Z(64725).Symbol, ce = Z(74009), le = Z(40313), ue = Z(21290), de = Object.create, fe = Object.defineProperties, he = Object.defineProperty;
            if (typeof se == "function") try {
              String(se()), ee = !0;
            } catch {
            }
            else se = null;
            ne = function(me) {
              if (this instanceof ne) throw new TypeError("Symbol is not a constructor");
              return te(me);
            }, G.exports = te = function me(pe) {
              var ye;
              if (this instanceof me) throw new TypeError("Symbol is not a constructor");
              return ee ? se(pe) : (ye = de(ne.prototype), pe = pe === void 0 ? "" : String(pe), fe(ye, { __description__: ae("", pe), __name__: ae("", ce(pe)) }));
            }, le(te), ue(te), fe(ne.prototype, { constructor: ae(te), toString: ae("", function() {
              return this.__name__;
            }) }), fe(te.prototype, { toString: ae(function() {
              return "Symbol (" + oe(this).__description__ + ")";
            }), valueOf: ae(function() {
              return oe(this);
            }) }), he(te.prototype, te.toPrimitive, ae("", function() {
              var me = oe(this);
              return typeof me == "symbol" ? me : me.toString();
            })), he(te.prototype, te.toStringTag, ae("c", "Symbol")), he(ne.prototype, te.toStringTag, ae("c", te.prototype[te.toStringTag])), he(ne.prototype, te.toPrimitive, ae("c", te.prototype[te.toPrimitive]));
          }, 91765: function(G, J, Z) {
            var te = Z(41707);
            G.exports = function(ne) {
              if (!te(ne)) throw new TypeError(ne + " is not a symbol");
              return ne;
            };
          }, 93103: function(G, J, Z) {
            G.exports = Z(22742)() ? WeakMap : Z(21780);
          }, 22742: function(G) {
            G.exports = function() {
              var J, Z;
              if (typeof WeakMap != "function") return !1;
              try {
                J = new WeakMap([[Z = {}, "one"], [{}, "two"], [{}, "three"]]);
              } catch {
                return !1;
              }
              return String(J) === "[object WeakMap]" && typeof J.set == "function" && J.set({}, 1) === J && typeof J.delete == "function" && typeof J.has == "function" && J.get(Z) === "one";
            };
          }, 81810: function(G) {
            G.exports = typeof WeakMap == "function" && Object.prototype.toString.call(/* @__PURE__ */ new WeakMap()) === "[object WeakMap]";
          }, 21780: function(G, J, Z) {
            var te, ne = Z(1974), ee = Z(22834), ae = Z(11004), oe = Z(69746), se = Z(43497), ce = Z(91819), le = Z(34494), ue = Z(58755), de = Z(63008).toStringTag, fe = Z(81810), he = Array.isArray, me = Object.defineProperty, pe = Object.prototype.hasOwnProperty, ye = Object.getPrototypeOf;
            G.exports = te = function() {
              var ge, ve = arguments[0];
              if (!(this instanceof te)) throw new TypeError("Constructor requires 'new'");
              return ge = fe && ee && WeakMap !== te ? ee(/* @__PURE__ */ new WeakMap(), ye(this)) : this, ne(ve) && (he(ve) || (ve = le(ve))), me(ge, "__weakMapData__", ce("c", "$weakMap$" + se())), ve && ue(ve, function(be) {
                oe(be), ge.set(be[0], be[1]);
              }), ge;
            }, fe && (ee && ee(te, WeakMap), te.prototype = Object.create(WeakMap.prototype, { constructor: ce(te) })), Object.defineProperties(te.prototype, { delete: ce(function(ge) {
              return !!pe.call(ae(ge), this.__weakMapData__) && (delete ge[this.__weakMapData__], !0);
            }), get: ce(function(ge) {
              if (pe.call(ae(ge), this.__weakMapData__)) return ge[this.__weakMapData__];
            }), has: ce(function(ge) {
              return pe.call(ae(ge), this.__weakMapData__);
            }), set: ce(function(ge, ve) {
              return me(ae(ge), this.__weakMapData__, ce("c", ve)), this;
            }), toString: ce(function() {
              return "[object WeakMap]";
            }) }), me(te.prototype, de, ce("c", "WeakMap"));
          }, 7683: function(G) {
            var J, Z = typeof Reflect == "object" ? Reflect : null, te = Z && typeof Z.apply == "function" ? Z.apply : function(pe, ye, ge) {
              return Function.prototype.apply.call(pe, ye, ge);
            };
            J = Z && typeof Z.ownKeys == "function" ? Z.ownKeys : Object.getOwnPropertySymbols ? function(pe) {
              return Object.getOwnPropertyNames(pe).concat(Object.getOwnPropertySymbols(pe));
            } : function(pe) {
              return Object.getOwnPropertyNames(pe);
            };
            var ne = Number.isNaN || function(pe) {
              return pe != pe;
            };
            function ee() {
              ee.init.call(this);
            }
            G.exports = ee, G.exports.once = function(pe, ye) {
              return new Promise(function(ge, ve) {
                function be(Se) {
                  pe.removeListener(ye, Ce), ve(Se);
                }
                function Ce() {
                  typeof pe.removeListener == "function" && pe.removeListener("error", be), ge([].slice.call(arguments));
                }
                me(pe, ye, Ce, { once: !0 }), ye !== "error" && function(Se, we, Ee) {
                  typeof Se.on == "function" && me(Se, "error", we, { once: !0 });
                }(pe, be);
              });
            }, ee.EventEmitter = ee, ee.prototype._events = void 0, ee.prototype._eventsCount = 0, ee.prototype._maxListeners = void 0;
            var ae = 10;
            function oe(pe) {
              if (typeof pe != "function") throw new TypeError('The "listener" argument must be of type Function. Received type ' + typeof pe);
            }
            function se(pe) {
              return pe._maxListeners === void 0 ? ee.defaultMaxListeners : pe._maxListeners;
            }
            function ce(pe, ye, ge, ve) {
              var be, Ce, Se, we;
              if (oe(ge), (Ce = pe._events) === void 0 ? (Ce = pe._events = /* @__PURE__ */ Object.create(null), pe._eventsCount = 0) : (Ce.newListener !== void 0 && (pe.emit("newListener", ye, ge.listener ? ge.listener : ge), Ce = pe._events), Se = Ce[ye]), Se === void 0) Se = Ce[ye] = ge, ++pe._eventsCount;
              else if (typeof Se == "function" ? Se = Ce[ye] = ve ? [ge, Se] : [Se, ge] : ve ? Se.unshift(ge) : Se.push(ge), (be = se(pe)) > 0 && Se.length > be && !Se.warned) {
                Se.warned = !0;
                var Ee = new Error("Possible EventEmitter memory leak detected. " + Se.length + " " + String(ye) + " listeners added. Use emitter.setMaxListeners() to increase limit");
                Ee.name = "MaxListenersExceededWarning", Ee.emitter = pe, Ee.type = ye, Ee.count = Se.length, we = Ee, console && console.warn && console.warn(we);
              }
              return pe;
            }
            function le() {
              if (!this.fired) return this.target.removeListener(this.type, this.wrapFn), this.fired = !0, arguments.length === 0 ? this.listener.call(this.target) : this.listener.apply(this.target, arguments);
            }
            function ue(pe, ye, ge) {
              var ve = { fired: !1, wrapFn: void 0, target: pe, type: ye, listener: ge }, be = le.bind(ve);
              return be.listener = ge, ve.wrapFn = be, be;
            }
            function de(pe, ye, ge) {
              var ve = pe._events;
              if (ve === void 0) return [];
              var be = ve[ye];
              return be === void 0 ? [] : typeof be == "function" ? ge ? [be.listener || be] : [be] : ge ? function(Ce) {
                for (var Se = new Array(Ce.length), we = 0; we < Se.length; ++we) Se[we] = Ce[we].listener || Ce[we];
                return Se;
              }(be) : he(be, be.length);
            }
            function fe(pe) {
              var ye = this._events;
              if (ye !== void 0) {
                var ge = ye[pe];
                if (typeof ge == "function") return 1;
                if (ge !== void 0) return ge.length;
              }
              return 0;
            }
            function he(pe, ye) {
              for (var ge = new Array(ye), ve = 0; ve < ye; ++ve) ge[ve] = pe[ve];
              return ge;
            }
            function me(pe, ye, ge, ve) {
              if (typeof pe.on == "function") ve.once ? pe.once(ye, ge) : pe.on(ye, ge);
              else {
                if (typeof pe.addEventListener != "function") throw new TypeError('The "emitter" argument must be of type EventEmitter. Received type ' + typeof pe);
                pe.addEventListener(ye, function be(Ce) {
                  ve.once && pe.removeEventListener(ye, be), ge(Ce);
                });
              }
            }
            Object.defineProperty(ee, "defaultMaxListeners", { enumerable: !0, get: function() {
              return ae;
            }, set: function(pe) {
              if (typeof pe != "number" || pe < 0 || ne(pe)) throw new RangeError('The value of "defaultMaxListeners" is out of range. It must be a non-negative number. Received ' + pe + ".");
              ae = pe;
            } }), ee.init = function() {
              this._events !== void 0 && this._events !== Object.getPrototypeOf(this)._events || (this._events = /* @__PURE__ */ Object.create(null), this._eventsCount = 0), this._maxListeners = this._maxListeners || void 0;
            }, ee.prototype.setMaxListeners = function(pe) {
              if (typeof pe != "number" || pe < 0 || ne(pe)) throw new RangeError('The value of "n" is out of range. It must be a non-negative number. Received ' + pe + ".");
              return this._maxListeners = pe, this;
            }, ee.prototype.getMaxListeners = function() {
              return se(this);
            }, ee.prototype.emit = function(pe) {
              for (var ye = [], ge = 1; ge < arguments.length; ge++) ye.push(arguments[ge]);
              var ve = pe === "error", be = this._events;
              if (be !== void 0) ve = ve && be.error === void 0;
              else if (!ve) return !1;
              if (ve) {
                var Ce;
                if (ye.length > 0 && (Ce = ye[0]), Ce instanceof Error) throw Ce;
                var Se = new Error("Unhandled error." + (Ce ? " (" + Ce.message + ")" : ""));
                throw Se.context = Ce, Se;
              }
              var we = be[pe];
              if (we === void 0) return !1;
              if (typeof we == "function") te(we, this, ye);
              else {
                var Ee = we.length, xe = he(we, Ee);
                for (ge = 0; ge < Ee; ++ge) te(xe[ge], this, ye);
              }
              return !0;
            }, ee.prototype.addListener = function(pe, ye) {
              return ce(this, pe, ye, !1);
            }, ee.prototype.on = ee.prototype.addListener, ee.prototype.prependListener = function(pe, ye) {
              return ce(this, pe, ye, !0);
            }, ee.prototype.once = function(pe, ye) {
              return oe(ye), this.on(pe, ue(this, pe, ye)), this;
            }, ee.prototype.prependOnceListener = function(pe, ye) {
              return oe(ye), this.prependListener(pe, ue(this, pe, ye)), this;
            }, ee.prototype.removeListener = function(pe, ye) {
              var ge, ve, be, Ce, Se;
              if (oe(ye), (ve = this._events) === void 0) return this;
              if ((ge = ve[pe]) === void 0) return this;
              if (ge === ye || ge.listener === ye) --this._eventsCount == 0 ? this._events = /* @__PURE__ */ Object.create(null) : (delete ve[pe], ve.removeListener && this.emit("removeListener", pe, ge.listener || ye));
              else if (typeof ge != "function") {
                for (be = -1, Ce = ge.length - 1; Ce >= 0; Ce--) if (ge[Ce] === ye || ge[Ce].listener === ye) {
                  Se = ge[Ce].listener, be = Ce;
                  break;
                }
                if (be < 0) return this;
                be === 0 ? ge.shift() : function(we, Ee) {
                  for (; Ee + 1 < we.length; Ee++) we[Ee] = we[Ee + 1];
                  we.pop();
                }(ge, be), ge.length === 1 && (ve[pe] = ge[0]), ve.removeListener !== void 0 && this.emit("removeListener", pe, Se || ye);
              }
              return this;
            }, ee.prototype.off = ee.prototype.removeListener, ee.prototype.removeAllListeners = function(pe) {
              var ye, ge, ve;
              if ((ge = this._events) === void 0) return this;
              if (ge.removeListener === void 0) return arguments.length === 0 ? (this._events = /* @__PURE__ */ Object.create(null), this._eventsCount = 0) : ge[pe] !== void 0 && (--this._eventsCount == 0 ? this._events = /* @__PURE__ */ Object.create(null) : delete ge[pe]), this;
              if (arguments.length === 0) {
                var be, Ce = Object.keys(ge);
                for (ve = 0; ve < Ce.length; ++ve) (be = Ce[ve]) !== "removeListener" && this.removeAllListeners(be);
                return this.removeAllListeners("removeListener"), this._events = /* @__PURE__ */ Object.create(null), this._eventsCount = 0, this;
              }
              if (typeof (ye = ge[pe]) == "function") this.removeListener(pe, ye);
              else if (ye !== void 0) for (ve = ye.length - 1; ve >= 0; ve--) this.removeListener(pe, ye[ve]);
              return this;
            }, ee.prototype.listeners = function(pe) {
              return de(this, pe, !0);
            }, ee.prototype.rawListeners = function(pe) {
              return de(this, pe, !1);
            }, ee.listenerCount = function(pe, ye) {
              return typeof pe.listenerCount == "function" ? pe.listenerCount(ye) : fe.call(pe, ye);
            }, ee.prototype.listenerCount = fe, ee.prototype.eventNames = function() {
              return this._eventsCount > 0 ? J(this._events) : [];
            };
          }, 77083: function(G) {
            var J = function() {
              if (typeof self == "object" && self) return self;
              if (typeof window == "object" && window) return window;
              throw new Error("Unable to resolve global `this`");
            };
            G.exports = function() {
              if (this) return this;
              try {
                Object.defineProperty(Object.prototype, "__global__", { get: function() {
                  return this;
                }, configurable: !0 });
              } catch {
                return J();
              }
              try {
                return __global__ || J();
              } finally {
                delete Object.prototype.__global__;
              }
            }();
          }, 64725: function(G, J, Z) {
            G.exports = Z(17804)() ? globalThis : Z(77083);
          }, 17804: function(G) {
            G.exports = function() {
              return typeof globalThis == "object" && !!globalThis && globalThis.Array === Array;
            };
          }, 10721: function(G, J, Z) {
            var te = Z(9914);
            G.exports = function(ne) {
              var ee = typeof ne;
              if (ee === "string") {
                var ae = ne;
                if ((ne = +ne) == 0 && te(ae)) return !1;
              } else if (ee !== "number") return !1;
              return ne - ne < 1;
            };
          }, 83473: function(G, J, Z) {
            var te = Z(10275);
            G.exports = function(ne, ee, ae) {
              if (!ne) throw new TypeError("must specify data as first parameter");
              if (ae = 0 | +(ae || 0), Array.isArray(ne) && ne[0] && typeof ne[0][0] == "number") {
                var oe, se, ce, le, ue = ne[0].length, de = ne.length * ue;
                ee && typeof ee != "string" || (ee = new (te(ee || "float32"))(de + ae));
                var fe = ee.length - ae;
                if (de !== fe) throw new Error("source length " + de + " (" + ue + "x" + ne.length + ") does not match destination length " + fe);
                for (oe = 0, ce = ae; oe < ne.length; oe++) for (se = 0; se < ue; se++) ee[ce++] = ne[oe][se] === null ? NaN : ne[oe][se];
              } else if (ee && typeof ee != "string") ee.set(ne, ae);
              else {
                var he = te(ee || "float32");
                if (Array.isArray(ne) || ee === "array") for (oe = 0, ce = ae, le = (ee = new he(ne.length + ae)).length; ce < le; ce++, oe++) ee[ce] = ne[oe] === null ? NaN : ne[oe];
                else ae === 0 ? ee = new he(ne) : (ee = new he(ne.length + ae)).set(ne, ae);
              }
              return ee;
            };
          }, 68950: function(G, J, Z) {
            var te = Z(38211), ne = [32, 126];
            G.exports = function(ee) {
              var ae = (ee = ee || {}).shape ? ee.shape : ee.canvas ? [ee.canvas.width, ee.canvas.height] : [512, 512], oe = ee.canvas || document.createElement("canvas"), se = ee.font, ce = typeof ee.step == "number" ? [ee.step, ee.step] : ee.step || [32, 32], le = ee.chars || ne;
              if (se && typeof se != "string" && (se = te(se)), Array.isArray(le)) {
                if (le.length === 2 && typeof le[0] == "number" && typeof le[1] == "number") {
                  for (var ue = [], de = le[0], fe = 0; de <= le[1]; de++) ue[fe++] = String.fromCharCode(de);
                  le = ue;
                }
              } else le = String(le).split("");
              ae = ae.slice(), oe.width = ae[0], oe.height = ae[1];
              var he = oe.getContext("2d");
              he.fillStyle = "#000", he.fillRect(0, 0, oe.width, oe.height), he.font = se, he.textAlign = "center", he.textBaseline = "middle", he.fillStyle = "#fff";
              var me = ce[0] / 2, pe = ce[1] / 2;
              for (de = 0; de < le.length; de++) he.fillText(le[de], me, pe), (me += ce[0]) > ae[0] - ce[0] / 2 && (me = ce[0] / 2, pe += ce[1]);
              return oe;
            };
          }, 12673: function(G) {
            function J(ee, ae) {
              ae || (ae = {}), (typeof ee == "string" || Array.isArray(ee)) && (ae.family = ee);
              var oe = Array.isArray(ae.family) ? ae.family.join(", ") : ae.family;
              if (!oe) throw Error("`family` must be defined");
              var se = ae.size || ae.fontSize || ae.em || 48, ce = ae.weight || ae.fontWeight || "", le = (ee = [ae.style || ae.fontStyle || "", ce, se].join(" ") + "px " + oe, ae.origin || "top");
              if (J.cache[oe] && se <= J.cache[oe].em) return Z(J.cache[oe], le);
              var ue = ae.canvas || J.canvas, de = ue.getContext("2d"), fe = { upper: ae.upper !== void 0 ? ae.upper : "H", lower: ae.lower !== void 0 ? ae.lower : "x", descent: ae.descent !== void 0 ? ae.descent : "p", ascent: ae.ascent !== void 0 ? ae.ascent : "h", tittle: ae.tittle !== void 0 ? ae.tittle : "i", overshoot: ae.overshoot !== void 0 ? ae.overshoot : "O" }, he = Math.ceil(1.5 * se);
              ue.height = he, ue.width = 0.5 * he, de.font = ee;
              var me = "H", pe = { top: 0 };
              de.clearRect(0, 0, he, he), de.textBaseline = "top", de.fillStyle = "black", de.fillText(me, 0, 0);
              var ye = te(de.getImageData(0, 0, he, he));
              de.clearRect(0, 0, he, he), de.textBaseline = "bottom", de.fillText(me, 0, he);
              var ge = te(de.getImageData(0, 0, he, he));
              pe.lineHeight = pe.bottom = he - ge + ye, de.clearRect(0, 0, he, he), de.textBaseline = "alphabetic", de.fillText(me, 0, he);
              var ve = he - te(de.getImageData(0, 0, he, he)) - 1 + ye;
              pe.baseline = pe.alphabetic = ve, de.clearRect(0, 0, he, he), de.textBaseline = "middle", de.fillText(me, 0, 0.5 * he);
              var be = te(de.getImageData(0, 0, he, he));
              pe.median = pe.middle = he - be - 1 + ye - 0.5 * he, de.clearRect(0, 0, he, he), de.textBaseline = "hanging", de.fillText(me, 0, 0.5 * he);
              var Ce = te(de.getImageData(0, 0, he, he));
              pe.hanging = he - Ce - 1 + ye - 0.5 * he, de.clearRect(0, 0, he, he), de.textBaseline = "ideographic", de.fillText(me, 0, he);
              var Se = te(de.getImageData(0, 0, he, he));
              if (pe.ideographic = he - Se - 1 + ye, fe.upper && (de.clearRect(0, 0, he, he), de.textBaseline = "top", de.fillText(fe.upper, 0, 0), pe.upper = te(de.getImageData(0, 0, he, he)), pe.capHeight = pe.baseline - pe.upper), fe.lower && (de.clearRect(0, 0, he, he), de.textBaseline = "top", de.fillText(fe.lower, 0, 0), pe.lower = te(de.getImageData(0, 0, he, he)), pe.xHeight = pe.baseline - pe.lower), fe.tittle && (de.clearRect(0, 0, he, he), de.textBaseline = "top", de.fillText(fe.tittle, 0, 0), pe.tittle = te(de.getImageData(0, 0, he, he))), fe.ascent && (de.clearRect(0, 0, he, he), de.textBaseline = "top", de.fillText(fe.ascent, 0, 0), pe.ascent = te(de.getImageData(0, 0, he, he))), fe.descent && (de.clearRect(0, 0, he, he), de.textBaseline = "top", de.fillText(fe.descent, 0, 0), pe.descent = ne(de.getImageData(0, 0, he, he))), fe.overshoot) {
                de.clearRect(0, 0, he, he), de.textBaseline = "top", de.fillText(fe.overshoot, 0, 0);
                var we = ne(de.getImageData(0, 0, he, he));
                pe.overshoot = we - ve;
              }
              for (var Ee in pe) pe[Ee] /= se;
              return pe.em = se, J.cache[oe] = pe, Z(pe, le);
            }
            function Z(ee, ae) {
              var oe = {};
              for (var se in typeof ae == "string" && (ae = ee[ae]), ee) se !== "em" && (oe[se] = ee[se] - ae);
              return oe;
            }
            function te(ee) {
              for (var ae = ee.height, oe = ee.data, se = 3; se < oe.length; se += 4) if (oe[se] !== 0) return Math.floor(0.25 * (se - 3) / ae);
            }
            function ne(ee) {
              for (var ae = ee.height, oe = ee.data, se = oe.length - 1; se > 0; se -= 4) if (oe[se] !== 0) return Math.floor(0.25 * (se - 3) / ae);
            }
            G.exports = J, J.canvas = document.createElement("canvas"), J.cache = {};
          }, 61262: function(G, J, Z) {
            var te = Z(82756), ne = Object.prototype.toString, ee = Object.prototype.hasOwnProperty;
            G.exports = function(ae, oe, se) {
              if (!te(oe)) throw new TypeError("iterator must be a function");
              var ce;
              arguments.length >= 3 && (ce = se), ne.call(ae) === "[object Array]" ? function(le, ue, de) {
                for (var fe = 0, he = le.length; fe < he; fe++) ee.call(le, fe) && (de == null ? ue(le[fe], fe, le) : ue.call(de, le[fe], fe, le));
              }(ae, oe, ce) : typeof ae == "string" ? function(le, ue, de) {
                for (var fe = 0, he = le.length; fe < he; fe++) de == null ? ue(le.charAt(fe), fe, le) : ue.call(de, le.charAt(fe), fe, le);
              }(ae, oe, ce) : function(le, ue, de) {
                for (var fe in le) ee.call(le, fe) && (de == null ? ue(le[fe], fe, le) : ue.call(de, le[fe], fe, le));
              }(ae, oe, ce);
            };
          }, 31917: function(G) {
            var J = Object.prototype.toString, Z = Math.max, te = function(ne, ee) {
              for (var ae = [], oe = 0; oe < ne.length; oe += 1) ae[oe] = ne[oe];
              for (var se = 0; se < ee.length; se += 1) ae[se + ne.length] = ee[se];
              return ae;
            };
            G.exports = function(ne) {
              var ee = this;
              if (typeof ee != "function" || J.apply(ee) !== "[object Function]") throw new TypeError("Function.prototype.bind called on incompatible " + ee);
              for (var ae, oe = function(de, fe) {
                for (var he = [], me = 1, pe = 0; me < de.length; me += 1, pe += 1) he[pe] = de[me];
                return he;
              }(arguments), se = Z(0, ee.length - oe.length), ce = [], le = 0; le < se; le++) ce[le] = "$" + le;
              if (ae = Function("binder", "return function (" + function(de, fe) {
                for (var he = "", me = 0; me < de.length; me += 1) he += de[me], me + 1 < de.length && (he += ",");
                return he;
              }(ce) + "){ return binder.apply(this,arguments); }")(function() {
                if (this instanceof ae) {
                  var de = ee.apply(this, te(oe, arguments));
                  return Object(de) === de ? de : this;
                }
                return ee.apply(ne, te(oe, arguments));
              }), ee.prototype) {
                var ue = function() {
                };
                ue.prototype = ee.prototype, ae.prototype = new ue(), ue.prototype = null;
              }
              return ae;
            };
          }, 87547: function(G, J, Z) {
            var te = Z(31917);
            G.exports = Function.prototype.bind || te;
          }, 72880: function(G) {
            G.exports = function(J, Z) {
              if (typeof J != "string") throw new TypeError("must specify type string");
              if (Z = Z || {}, typeof document > "u" && !Z.canvas) return null;
              var te = Z.canvas || document.createElement("canvas");
              typeof Z.width == "number" && (te.width = Z.width), typeof Z.height == "number" && (te.height = Z.height);
              var ne, ee = Z;
              try {
                var ae = [J];
                J.indexOf("webgl") === 0 && ae.push("experimental-" + J);
                for (var oe = 0; oe < ae.length; oe++) if (ne = te.getContext(ae[oe], ee)) return ne;
              } catch {
                ne = null;
              }
              return ne || null;
            };
          }, 71129: function(G, J, Z) {
            var te, ne = Z(77731), ee = Z(35465), ae = Z(30582), oe = Z(50294), se = Z(98800), ce = Z(48631), le = Z(33149), ue = Function, de = function(Xe) {
              try {
                return ue('"use strict"; return (' + Xe + ").constructor;")();
              } catch {
              }
            }, fe = Object.getOwnPropertyDescriptor;
            if (fe) try {
              fe({}, "");
            } catch {
              fe = null;
            }
            var he = function() {
              throw new ce();
            }, me = fe ? function() {
              try {
                return he;
              } catch {
                try {
                  return fe(arguments, "callee").get;
                } catch {
                  return he;
                }
              }
            }() : he, pe = Z(8771)(), ye = Z(58436)(), ge = Object.getPrototypeOf || (ye ? function(Xe) {
              return Xe.__proto__;
            } : null), ve = {}, be = typeof Uint8Array < "u" && ge ? ge(Uint8Array) : te, Ce = { __proto__: null, "%AggregateError%": typeof AggregateError > "u" ? te : AggregateError, "%Array%": Array, "%ArrayBuffer%": typeof ArrayBuffer > "u" ? te : ArrayBuffer, "%ArrayIteratorPrototype%": pe && ge ? ge([][Symbol.iterator]()) : te, "%AsyncFromSyncIteratorPrototype%": te, "%AsyncFunction%": ve, "%AsyncGenerator%": ve, "%AsyncGeneratorFunction%": ve, "%AsyncIteratorPrototype%": ve, "%Atomics%": typeof Atomics > "u" ? te : Atomics, "%BigInt%": typeof BigInt > "u" ? te : BigInt, "%BigInt64Array%": typeof BigInt64Array > "u" ? te : BigInt64Array, "%BigUint64Array%": typeof BigUint64Array > "u" ? te : BigUint64Array, "%Boolean%": Boolean, "%DataView%": typeof DataView > "u" ? te : DataView, "%Date%": Date, "%decodeURI%": decodeURI, "%decodeURIComponent%": decodeURIComponent, "%encodeURI%": encodeURI, "%encodeURIComponent%": encodeURIComponent, "%Error%": ne, "%eval%": eval, "%EvalError%": ee, "%Float32Array%": typeof Float32Array > "u" ? te : Float32Array, "%Float64Array%": typeof Float64Array > "u" ? te : Float64Array, "%FinalizationRegistry%": typeof FinalizationRegistry > "u" ? te : FinalizationRegistry, "%Function%": ue, "%GeneratorFunction%": ve, "%Int8Array%": typeof Int8Array > "u" ? te : Int8Array, "%Int16Array%": typeof Int16Array > "u" ? te : Int16Array, "%Int32Array%": typeof Int32Array > "u" ? te : Int32Array, "%isFinite%": isFinite, "%isNaN%": isNaN, "%IteratorPrototype%": pe && ge ? ge(ge([][Symbol.iterator]())) : te, "%JSON%": typeof JSON == "object" ? JSON : te, "%Map%": typeof Map > "u" ? te : Map, "%MapIteratorPrototype%": typeof Map < "u" && pe && ge ? ge((/* @__PURE__ */ new Map())[Symbol.iterator]()) : te, "%Math%": Math, "%Number%": Number, "%Object%": Object, "%parseFloat%": parseFloat, "%parseInt%": parseInt, "%Promise%": typeof Promise > "u" ? te : Promise, "%Proxy%": typeof Proxy > "u" ? te : Proxy, "%RangeError%": ae, "%ReferenceError%": oe, "%Reflect%": typeof Reflect > "u" ? te : Reflect, "%RegExp%": RegExp, "%Set%": typeof Set > "u" ? te : Set, "%SetIteratorPrototype%": typeof Set < "u" && pe && ge ? ge((/* @__PURE__ */ new Set())[Symbol.iterator]()) : te, "%SharedArrayBuffer%": typeof SharedArrayBuffer > "u" ? te : SharedArrayBuffer, "%String%": String, "%StringIteratorPrototype%": pe && ge ? ge(""[Symbol.iterator]()) : te, "%Symbol%": pe ? Symbol : te, "%SyntaxError%": se, "%ThrowTypeError%": me, "%TypedArray%": be, "%TypeError%": ce, "%Uint8Array%": typeof Uint8Array > "u" ? te : Uint8Array, "%Uint8ClampedArray%": typeof Uint8ClampedArray > "u" ? te : Uint8ClampedArray, "%Uint16Array%": typeof Uint16Array > "u" ? te : Uint16Array, "%Uint32Array%": typeof Uint32Array > "u" ? te : Uint32Array, "%URIError%": le, "%WeakMap%": typeof WeakMap > "u" ? te : WeakMap, "%WeakRef%": typeof WeakRef > "u" ? te : WeakRef, "%WeakSet%": typeof WeakSet > "u" ? te : WeakSet };
            if (ge) try {
              null.error;
            } catch (Xe) {
              var Se = ge(ge(Xe));
              Ce["%Error.prototype%"] = Se;
            }
            var we = function Xe(Ve) {
              var Ze;
              if (Ve === "%AsyncFunction%") Ze = de("async function () {}");
              else if (Ve === "%GeneratorFunction%") Ze = de("function* () {}");
              else if (Ve === "%AsyncGeneratorFunction%") Ze = de("async function* () {}");
              else if (Ve === "%AsyncGenerator%") {
                var Ge = Xe("%AsyncGeneratorFunction%");
                Ge && (Ze = Ge.prototype);
              } else if (Ve === "%AsyncIteratorPrototype%") {
                var ze = Xe("%AsyncGenerator%");
                ze && ge && (Ze = ge(ze.prototype));
              }
              return Ce[Ve] = Ze, Ze;
            }, Ee = { __proto__: null, "%ArrayBufferPrototype%": ["ArrayBuffer", "prototype"], "%ArrayPrototype%": ["Array", "prototype"], "%ArrayProto_entries%": ["Array", "prototype", "entries"], "%ArrayProto_forEach%": ["Array", "prototype", "forEach"], "%ArrayProto_keys%": ["Array", "prototype", "keys"], "%ArrayProto_values%": ["Array", "prototype", "values"], "%AsyncFunctionPrototype%": ["AsyncFunction", "prototype"], "%AsyncGenerator%": ["AsyncGeneratorFunction", "prototype"], "%AsyncGeneratorPrototype%": ["AsyncGeneratorFunction", "prototype", "prototype"], "%BooleanPrototype%": ["Boolean", "prototype"], "%DataViewPrototype%": ["DataView", "prototype"], "%DatePrototype%": ["Date", "prototype"], "%ErrorPrototype%": ["Error", "prototype"], "%EvalErrorPrototype%": ["EvalError", "prototype"], "%Float32ArrayPrototype%": ["Float32Array", "prototype"], "%Float64ArrayPrototype%": ["Float64Array", "prototype"], "%FunctionPrototype%": ["Function", "prototype"], "%Generator%": ["GeneratorFunction", "prototype"], "%GeneratorPrototype%": ["GeneratorFunction", "prototype", "prototype"], "%Int8ArrayPrototype%": ["Int8Array", "prototype"], "%Int16ArrayPrototype%": ["Int16Array", "prototype"], "%Int32ArrayPrototype%": ["Int32Array", "prototype"], "%JSONParse%": ["JSON", "parse"], "%JSONStringify%": ["JSON", "stringify"], "%MapPrototype%": ["Map", "prototype"], "%NumberPrototype%": ["Number", "prototype"], "%ObjectPrototype%": ["Object", "prototype"], "%ObjProto_toString%": ["Object", "prototype", "toString"], "%ObjProto_valueOf%": ["Object", "prototype", "valueOf"], "%PromisePrototype%": ["Promise", "prototype"], "%PromiseProto_then%": ["Promise", "prototype", "then"], "%Promise_all%": ["Promise", "all"], "%Promise_reject%": ["Promise", "reject"], "%Promise_resolve%": ["Promise", "resolve"], "%RangeErrorPrototype%": ["RangeError", "prototype"], "%ReferenceErrorPrototype%": ["ReferenceError", "prototype"], "%RegExpPrototype%": ["RegExp", "prototype"], "%SetPrototype%": ["Set", "prototype"], "%SharedArrayBufferPrototype%": ["SharedArrayBuffer", "prototype"], "%StringPrototype%": ["String", "prototype"], "%SymbolPrototype%": ["Symbol", "prototype"], "%SyntaxErrorPrototype%": ["SyntaxError", "prototype"], "%TypedArrayPrototype%": ["TypedArray", "prototype"], "%TypeErrorPrototype%": ["TypeError", "prototype"], "%Uint8ArrayPrototype%": ["Uint8Array", "prototype"], "%Uint8ClampedArrayPrototype%": ["Uint8ClampedArray", "prototype"], "%Uint16ArrayPrototype%": ["Uint16Array", "prototype"], "%Uint32ArrayPrototype%": ["Uint32Array", "prototype"], "%URIErrorPrototype%": ["URIError", "prototype"], "%WeakMapPrototype%": ["WeakMap", "prototype"], "%WeakSetPrototype%": ["WeakSet", "prototype"] }, xe = Z(87547), Le = Z(80753), Qe = xe.call(Function.call, Array.prototype.concat), ke = xe.call(Function.apply, Array.prototype.splice), Me = xe.call(Function.call, String.prototype.replace), Re = xe.call(Function.call, String.prototype.slice), De = xe.call(Function.call, RegExp.prototype.exec), Pe = /[^%.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|%$))/g, $e = /\\(\\)?/g, qe = function(Xe, Ve) {
              var Ze, Ge = Xe;
              if (Le(Ee, Ge) && (Ge = "%" + (Ze = Ee[Ge])[0] + "%"), Le(Ce, Ge)) {
                var ze = Ce[Ge];
                if (ze === ve && (ze = we(Ge)), ze === void 0 && !Ve) throw new ce("intrinsic " + Xe + " exists, but is not available. Please file an issue!");
                return { alias: Ze, name: Ge, value: ze };
              }
              throw new se("intrinsic " + Xe + " does not exist!");
            };
            G.exports = function(Xe, Ve) {
              if (typeof Xe != "string" || Xe.length === 0) throw new ce("intrinsic name must be a non-empty string");
              if (arguments.length > 1 && typeof Ve != "boolean") throw new ce('"allowMissing" argument must be a boolean');
              if (De(/^%?[^%]*%?$/, Xe) === null) throw new se("`%` may not be present anywhere but at the beginning and end of the intrinsic name");
              var Ze = function(yt) {
                var St = Re(yt, 0, 1), Lt = Re(yt, -1);
                if (St === "%" && Lt !== "%") throw new se("invalid intrinsic syntax, expected closing `%`");
                if (Lt === "%" && St !== "%") throw new se("invalid intrinsic syntax, expected opening `%`");
                var Rt = [];
                return Me(yt, Pe, function(vt, $t, Pt, Xt) {
                  Rt[Rt.length] = Pt ? Me(Xt, $e, "$1") : $t || vt;
                }), Rt;
              }(Xe), Ge = Ze.length > 0 ? Ze[0] : "", ze = qe("%" + Ge + "%", Ve), We = ze.name, at = ze.value, ct = !1, et = ze.alias;
              et && (Ge = et[0], ke(Ze, Qe([0, 1], et)));
              for (var dt = 1, Je = !0; dt < Ze.length; dt += 1) {
                var it = Ze[dt], st = Re(it, 0, 1), gt = Re(it, -1);
                if ((st === '"' || st === "'" || st === "`" || gt === '"' || gt === "'" || gt === "`") && st !== gt) throw new se("property names with quotes must have matching quotes");
                if (it !== "constructor" && Je || (ct = !0), Le(Ce, We = "%" + (Ge += "." + it) + "%")) at = Ce[We];
                else if (at != null) {
                  if (!(it in at)) {
                    if (!Ve) throw new ce("base intrinsic for " + Xe + " exists, but the property is not available.");
                    return;
                  }
                  if (fe && dt + 1 >= Ze.length) {
                    var bt = fe(at, it);
                    at = (Je = !!bt) && "get" in bt && !("originalValue" in bt.get) ? bt.get : at[it];
                  } else Je = Le(at, it), at = at[it];
                  Je && !ct && (Ce[We] = at);
                }
              }
              return at;
            };
          }, 84840: function(G) {
            G.exports = function(J, Z) {
              var te = Z[0], ne = Z[1], ee = Z[2], ae = Z[3], oe = Z[4], se = Z[5], ce = Z[6], le = Z[7], ue = Z[8], de = Z[9], fe = Z[10], he = Z[11], me = Z[12], pe = Z[13], ye = Z[14], ge = Z[15];
              return J[0] = se * (fe * ge - he * ye) - de * (ce * ge - le * ye) + pe * (ce * he - le * fe), J[1] = -(ne * (fe * ge - he * ye) - de * (ee * ge - ae * ye) + pe * (ee * he - ae * fe)), J[2] = ne * (ce * ge - le * ye) - se * (ee * ge - ae * ye) + pe * (ee * le - ae * ce), J[3] = -(ne * (ce * he - le * fe) - se * (ee * he - ae * fe) + de * (ee * le - ae * ce)), J[4] = -(oe * (fe * ge - he * ye) - ue * (ce * ge - le * ye) + me * (ce * he - le * fe)), J[5] = te * (fe * ge - he * ye) - ue * (ee * ge - ae * ye) + me * (ee * he - ae * fe), J[6] = -(te * (ce * ge - le * ye) - oe * (ee * ge - ae * ye) + me * (ee * le - ae * ce)), J[7] = te * (ce * he - le * fe) - oe * (ee * he - ae * fe) + ue * (ee * le - ae * ce), J[8] = oe * (de * ge - he * pe) - ue * (se * ge - le * pe) + me * (se * he - le * de), J[9] = -(te * (de * ge - he * pe) - ue * (ne * ge - ae * pe) + me * (ne * he - ae * de)), J[10] = te * (se * ge - le * pe) - oe * (ne * ge - ae * pe) + me * (ne * le - ae * se), J[11] = -(te * (se * he - le * de) - oe * (ne * he - ae * de) + ue * (ne * le - ae * se)), J[12] = -(oe * (de * ye - fe * pe) - ue * (se * ye - ce * pe) + me * (se * fe - ce * de)), J[13] = te * (de * ye - fe * pe) - ue * (ne * ye - ee * pe) + me * (ne * fe - ee * de), J[14] = -(te * (se * ye - ce * pe) - oe * (ne * ye - ee * pe) + me * (ne * ce - ee * se)), J[15] = te * (se * fe - ce * de) - oe * (ne * fe - ee * de) + ue * (ne * ce - ee * se), J;
            };
          }, 99698: function(G) {
            G.exports = function(J) {
              var Z = new Float32Array(16);
              return Z[0] = J[0], Z[1] = J[1], Z[2] = J[2], Z[3] = J[3], Z[4] = J[4], Z[5] = J[5], Z[6] = J[6], Z[7] = J[7], Z[8] = J[8], Z[9] = J[9], Z[10] = J[10], Z[11] = J[11], Z[12] = J[12], Z[13] = J[13], Z[14] = J[14], Z[15] = J[15], Z;
            };
          }, 57938: function(G) {
            G.exports = function(J, Z) {
              return J[0] = Z[0], J[1] = Z[1], J[2] = Z[2], J[3] = Z[3], J[4] = Z[4], J[5] = Z[5], J[6] = Z[6], J[7] = Z[7], J[8] = Z[8], J[9] = Z[9], J[10] = Z[10], J[11] = Z[11], J[12] = Z[12], J[13] = Z[13], J[14] = Z[14], J[15] = Z[15], J;
            };
          }, 87519: function(G) {
            G.exports = function() {
              var J = new Float32Array(16);
              return J[0] = 1, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = 1, J[6] = 0, J[7] = 0, J[8] = 0, J[9] = 0, J[10] = 1, J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J;
            };
          }, 6900: function(G) {
            G.exports = function(J) {
              var Z = J[0], te = J[1], ne = J[2], ee = J[3], ae = J[4], oe = J[5], se = J[6], ce = J[7], le = J[8], ue = J[9], de = J[10], fe = J[11], he = J[12], me = J[13], pe = J[14], ye = J[15];
              return (Z * oe - te * ae) * (de * ye - fe * pe) - (Z * se - ne * ae) * (ue * ye - fe * me) + (Z * ce - ee * ae) * (ue * pe - de * me) + (te * se - ne * oe) * (le * ye - fe * he) - (te * ce - ee * oe) * (le * pe - de * he) + (ne * ce - ee * se) * (le * me - ue * he);
            };
          }, 36472: function(G) {
            G.exports = function(J, Z) {
              var te = Z[0], ne = Z[1], ee = Z[2], ae = Z[3], oe = te + te, se = ne + ne, ce = ee + ee, le = te * oe, ue = ne * oe, de = ne * se, fe = ee * oe, he = ee * se, me = ee * ce, pe = ae * oe, ye = ae * se, ge = ae * ce;
              return J[0] = 1 - de - me, J[1] = ue + ge, J[2] = fe - ye, J[3] = 0, J[4] = ue - ge, J[5] = 1 - le - me, J[6] = he + pe, J[7] = 0, J[8] = fe + ye, J[9] = he - pe, J[10] = 1 - le - de, J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J;
            };
          }, 43061: function(G) {
            G.exports = function(J, Z, te) {
              var ne, ee, ae, oe = te[0], se = te[1], ce = te[2], le = Math.sqrt(oe * oe + se * se + ce * ce);
              return Math.abs(le) < 1e-6 ? null : (oe *= le = 1 / le, se *= le, ce *= le, ne = Math.sin(Z), ae = 1 - (ee = Math.cos(Z)), J[0] = oe * oe * ae + ee, J[1] = se * oe * ae + ce * ne, J[2] = ce * oe * ae - se * ne, J[3] = 0, J[4] = oe * se * ae - ce * ne, J[5] = se * se * ae + ee, J[6] = ce * se * ae + oe * ne, J[7] = 0, J[8] = oe * ce * ae + se * ne, J[9] = se * ce * ae - oe * ne, J[10] = ce * ce * ae + ee, J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J);
            };
          }, 33606: function(G) {
            G.exports = function(J, Z, te) {
              var ne = Z[0], ee = Z[1], ae = Z[2], oe = Z[3], se = ne + ne, ce = ee + ee, le = ae + ae, ue = ne * se, de = ne * ce, fe = ne * le, he = ee * ce, me = ee * le, pe = ae * le, ye = oe * se, ge = oe * ce, ve = oe * le;
              return J[0] = 1 - (he + pe), J[1] = de + ve, J[2] = fe - ge, J[3] = 0, J[4] = de - ve, J[5] = 1 - (ue + pe), J[6] = me + ye, J[7] = 0, J[8] = fe + ge, J[9] = me - ye, J[10] = 1 - (ue + he), J[11] = 0, J[12] = te[0], J[13] = te[1], J[14] = te[2], J[15] = 1, J;
            };
          }, 98698: function(G) {
            G.exports = function(J, Z) {
              return J[0] = Z[0], J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = Z[1], J[6] = 0, J[7] = 0, J[8] = 0, J[9] = 0, J[10] = Z[2], J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J;
            };
          }, 6924: function(G) {
            G.exports = function(J, Z) {
              return J[0] = 1, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = 1, J[6] = 0, J[7] = 0, J[8] = 0, J[9] = 0, J[10] = 1, J[11] = 0, J[12] = Z[0], J[13] = Z[1], J[14] = Z[2], J[15] = 1, J;
            };
          }, 81181: function(G) {
            G.exports = function(J, Z) {
              var te = Math.sin(Z), ne = Math.cos(Z);
              return J[0] = 1, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = ne, J[6] = te, J[7] = 0, J[8] = 0, J[9] = -te, J[10] = ne, J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J;
            };
          }, 95258: function(G) {
            G.exports = function(J, Z) {
              var te = Math.sin(Z), ne = Math.cos(Z);
              return J[0] = ne, J[1] = 0, J[2] = -te, J[3] = 0, J[4] = 0, J[5] = 1, J[6] = 0, J[7] = 0, J[8] = te, J[9] = 0, J[10] = ne, J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J;
            };
          }, 94815: function(G) {
            G.exports = function(J, Z) {
              var te = Math.sin(Z), ne = Math.cos(Z);
              return J[0] = ne, J[1] = te, J[2] = 0, J[3] = 0, J[4] = -te, J[5] = ne, J[6] = 0, J[7] = 0, J[8] = 0, J[9] = 0, J[10] = 1, J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J;
            };
          }, 87301: function(G) {
            G.exports = function(J, Z, te, ne, ee, ae, oe) {
              var se = 1 / (te - Z), ce = 1 / (ee - ne), le = 1 / (ae - oe);
              return J[0] = 2 * ae * se, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = 2 * ae * ce, J[6] = 0, J[7] = 0, J[8] = (te + Z) * se, J[9] = (ee + ne) * ce, J[10] = (oe + ae) * le, J[11] = -1, J[12] = 0, J[13] = 0, J[14] = oe * ae * 2 * le, J[15] = 0, J;
            };
          }, 87193: function(G) {
            G.exports = function(J) {
              return J[0] = 1, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = 1, J[6] = 0, J[7] = 0, J[8] = 0, J[9] = 0, J[10] = 1, J[11] = 0, J[12] = 0, J[13] = 0, J[14] = 0, J[15] = 1, J;
            };
          }, 11191: function(G, J, Z) {
            G.exports = { create: Z(87519), clone: Z(99698), copy: Z(57938), identity: Z(87193), transpose: Z(10256), invert: Z(96559), adjoint: Z(84840), determinant: Z(6900), multiply: Z(14787), translate: Z(4165), scale: Z(8697), rotate: Z(32416), rotateX: Z(81066), rotateY: Z(54201), rotateZ: Z(33920), fromRotation: Z(43061), fromRotationTranslation: Z(33606), fromScaling: Z(98698), fromTranslation: Z(6924), fromXRotation: Z(81181), fromYRotation: Z(95258), fromZRotation: Z(94815), fromQuat: Z(36472), frustum: Z(87301), perspective: Z(5313), perspectiveFromFieldOfView: Z(22253), ortho: Z(4633), lookAt: Z(26645), str: Z(66992) };
          }, 96559: function(G) {
            G.exports = function(J, Z) {
              var te = Z[0], ne = Z[1], ee = Z[2], ae = Z[3], oe = Z[4], se = Z[5], ce = Z[6], le = Z[7], ue = Z[8], de = Z[9], fe = Z[10], he = Z[11], me = Z[12], pe = Z[13], ye = Z[14], ge = Z[15], ve = te * se - ne * oe, be = te * ce - ee * oe, Ce = te * le - ae * oe, Se = ne * ce - ee * se, we = ne * le - ae * se, Ee = ee * le - ae * ce, xe = ue * pe - de * me, Le = ue * ye - fe * me, Qe = ue * ge - he * me, ke = de * ye - fe * pe, Me = de * ge - he * pe, Re = fe * ge - he * ye, De = ve * Re - be * Me + Ce * ke + Se * Qe - we * Le + Ee * xe;
              return De ? (De = 1 / De, J[0] = (se * Re - ce * Me + le * ke) * De, J[1] = (ee * Me - ne * Re - ae * ke) * De, J[2] = (pe * Ee - ye * we + ge * Se) * De, J[3] = (fe * we - de * Ee - he * Se) * De, J[4] = (ce * Qe - oe * Re - le * Le) * De, J[5] = (te * Re - ee * Qe + ae * Le) * De, J[6] = (ye * Ce - me * Ee - ge * be) * De, J[7] = (ue * Ee - fe * Ce + he * be) * De, J[8] = (oe * Me - se * Qe + le * xe) * De, J[9] = (ne * Qe - te * Me - ae * xe) * De, J[10] = (me * we - pe * Ce + ge * ve) * De, J[11] = (de * Ce - ue * we - he * ve) * De, J[12] = (se * Le - oe * ke - ce * xe) * De, J[13] = (te * ke - ne * Le + ee * xe) * De, J[14] = (pe * be - me * Se - ye * ve) * De, J[15] = (ue * Se - de * be + fe * ve) * De, J) : null;
            };
          }, 26645: function(G, J, Z) {
            var te = Z(87193);
            G.exports = function(ne, ee, ae, oe) {
              var se, ce, le, ue, de, fe, he, me, pe, ye, ge = ee[0], ve = ee[1], be = ee[2], Ce = oe[0], Se = oe[1], we = oe[2], Ee = ae[0], xe = ae[1], Le = ae[2];
              return Math.abs(ge - Ee) < 1e-6 && Math.abs(ve - xe) < 1e-6 && Math.abs(be - Le) < 1e-6 ? te(ne) : (he = ge - Ee, me = ve - xe, pe = be - Le, se = Se * (pe *= ye = 1 / Math.sqrt(he * he + me * me + pe * pe)) - we * (me *= ye), ce = we * (he *= ye) - Ce * pe, le = Ce * me - Se * he, (ye = Math.sqrt(se * se + ce * ce + le * le)) ? (se *= ye = 1 / ye, ce *= ye, le *= ye) : (se = 0, ce = 0, le = 0), ue = me * le - pe * ce, de = pe * se - he * le, fe = he * ce - me * se, (ye = Math.sqrt(ue * ue + de * de + fe * fe)) ? (ue *= ye = 1 / ye, de *= ye, fe *= ye) : (ue = 0, de = 0, fe = 0), ne[0] = se, ne[1] = ue, ne[2] = he, ne[3] = 0, ne[4] = ce, ne[5] = de, ne[6] = me, ne[7] = 0, ne[8] = le, ne[9] = fe, ne[10] = pe, ne[11] = 0, ne[12] = -(se * ge + ce * ve + le * be), ne[13] = -(ue * ge + de * ve + fe * be), ne[14] = -(he * ge + me * ve + pe * be), ne[15] = 1, ne);
            };
          }, 14787: function(G) {
            G.exports = function(J, Z, te) {
              var ne = Z[0], ee = Z[1], ae = Z[2], oe = Z[3], se = Z[4], ce = Z[5], le = Z[6], ue = Z[7], de = Z[8], fe = Z[9], he = Z[10], me = Z[11], pe = Z[12], ye = Z[13], ge = Z[14], ve = Z[15], be = te[0], Ce = te[1], Se = te[2], we = te[3];
              return J[0] = be * ne + Ce * se + Se * de + we * pe, J[1] = be * ee + Ce * ce + Se * fe + we * ye, J[2] = be * ae + Ce * le + Se * he + we * ge, J[3] = be * oe + Ce * ue + Se * me + we * ve, be = te[4], Ce = te[5], Se = te[6], we = te[7], J[4] = be * ne + Ce * se + Se * de + we * pe, J[5] = be * ee + Ce * ce + Se * fe + we * ye, J[6] = be * ae + Ce * le + Se * he + we * ge, J[7] = be * oe + Ce * ue + Se * me + we * ve, be = te[8], Ce = te[9], Se = te[10], we = te[11], J[8] = be * ne + Ce * se + Se * de + we * pe, J[9] = be * ee + Ce * ce + Se * fe + we * ye, J[10] = be * ae + Ce * le + Se * he + we * ge, J[11] = be * oe + Ce * ue + Se * me + we * ve, be = te[12], Ce = te[13], Se = te[14], we = te[15], J[12] = be * ne + Ce * se + Se * de + we * pe, J[13] = be * ee + Ce * ce + Se * fe + we * ye, J[14] = be * ae + Ce * le + Se * he + we * ge, J[15] = be * oe + Ce * ue + Se * me + we * ve, J;
            };
          }, 4633: function(G) {
            G.exports = function(J, Z, te, ne, ee, ae, oe) {
              var se = 1 / (Z - te), ce = 1 / (ne - ee), le = 1 / (ae - oe);
              return J[0] = -2 * se, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = -2 * ce, J[6] = 0, J[7] = 0, J[8] = 0, J[9] = 0, J[10] = 2 * le, J[11] = 0, J[12] = (Z + te) * se, J[13] = (ee + ne) * ce, J[14] = (oe + ae) * le, J[15] = 1, J;
            };
          }, 5313: function(G) {
            G.exports = function(J, Z, te, ne, ee) {
              var ae = 1 / Math.tan(Z / 2), oe = 1 / (ne - ee);
              return J[0] = ae / te, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = ae, J[6] = 0, J[7] = 0, J[8] = 0, J[9] = 0, J[10] = (ee + ne) * oe, J[11] = -1, J[12] = 0, J[13] = 0, J[14] = 2 * ee * ne * oe, J[15] = 0, J;
            };
          }, 22253: function(G) {
            G.exports = function(J, Z, te, ne) {
              var ee = Math.tan(Z.upDegrees * Math.PI / 180), ae = Math.tan(Z.downDegrees * Math.PI / 180), oe = Math.tan(Z.leftDegrees * Math.PI / 180), se = Math.tan(Z.rightDegrees * Math.PI / 180), ce = 2 / (oe + se), le = 2 / (ee + ae);
              return J[0] = ce, J[1] = 0, J[2] = 0, J[3] = 0, J[4] = 0, J[5] = le, J[6] = 0, J[7] = 0, J[8] = -(oe - se) * ce * 0.5, J[9] = (ee - ae) * le * 0.5, J[10] = ne / (te - ne), J[11] = -1, J[12] = 0, J[13] = 0, J[14] = ne * te / (te - ne), J[15] = 0, J;
            };
          }, 32416: function(G) {
            G.exports = function(J, Z, te, ne) {
              var ee, ae, oe, se, ce, le, ue, de, fe, he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me = ne[0], Re = ne[1], De = ne[2], Pe = Math.sqrt(Me * Me + Re * Re + De * De);
              return Math.abs(Pe) < 1e-6 ? null : (Me *= Pe = 1 / Pe, Re *= Pe, De *= Pe, ee = Math.sin(te), oe = 1 - (ae = Math.cos(te)), se = Z[0], ce = Z[1], le = Z[2], ue = Z[3], de = Z[4], fe = Z[5], he = Z[6], me = Z[7], pe = Z[8], ye = Z[9], ge = Z[10], ve = Z[11], be = Me * Me * oe + ae, Ce = Re * Me * oe + De * ee, Se = De * Me * oe - Re * ee, we = Me * Re * oe - De * ee, Ee = Re * Re * oe + ae, xe = De * Re * oe + Me * ee, Le = Me * De * oe + Re * ee, Qe = Re * De * oe - Me * ee, ke = De * De * oe + ae, J[0] = se * be + de * Ce + pe * Se, J[1] = ce * be + fe * Ce + ye * Se, J[2] = le * be + he * Ce + ge * Se, J[3] = ue * be + me * Ce + ve * Se, J[4] = se * we + de * Ee + pe * xe, J[5] = ce * we + fe * Ee + ye * xe, J[6] = le * we + he * Ee + ge * xe, J[7] = ue * we + me * Ee + ve * xe, J[8] = se * Le + de * Qe + pe * ke, J[9] = ce * Le + fe * Qe + ye * ke, J[10] = le * Le + he * Qe + ge * ke, J[11] = ue * Le + me * Qe + ve * ke, Z !== J && (J[12] = Z[12], J[13] = Z[13], J[14] = Z[14], J[15] = Z[15]), J);
            };
          }, 81066: function(G) {
            G.exports = function(J, Z, te) {
              var ne = Math.sin(te), ee = Math.cos(te), ae = Z[4], oe = Z[5], se = Z[6], ce = Z[7], le = Z[8], ue = Z[9], de = Z[10], fe = Z[11];
              return Z !== J && (J[0] = Z[0], J[1] = Z[1], J[2] = Z[2], J[3] = Z[3], J[12] = Z[12], J[13] = Z[13], J[14] = Z[14], J[15] = Z[15]), J[4] = ae * ee + le * ne, J[5] = oe * ee + ue * ne, J[6] = se * ee + de * ne, J[7] = ce * ee + fe * ne, J[8] = le * ee - ae * ne, J[9] = ue * ee - oe * ne, J[10] = de * ee - se * ne, J[11] = fe * ee - ce * ne, J;
            };
          }, 54201: function(G) {
            G.exports = function(J, Z, te) {
              var ne = Math.sin(te), ee = Math.cos(te), ae = Z[0], oe = Z[1], se = Z[2], ce = Z[3], le = Z[8], ue = Z[9], de = Z[10], fe = Z[11];
              return Z !== J && (J[4] = Z[4], J[5] = Z[5], J[6] = Z[6], J[7] = Z[7], J[12] = Z[12], J[13] = Z[13], J[14] = Z[14], J[15] = Z[15]), J[0] = ae * ee - le * ne, J[1] = oe * ee - ue * ne, J[2] = se * ee - de * ne, J[3] = ce * ee - fe * ne, J[8] = ae * ne + le * ee, J[9] = oe * ne + ue * ee, J[10] = se * ne + de * ee, J[11] = ce * ne + fe * ee, J;
            };
          }, 33920: function(G) {
            G.exports = function(J, Z, te) {
              var ne = Math.sin(te), ee = Math.cos(te), ae = Z[0], oe = Z[1], se = Z[2], ce = Z[3], le = Z[4], ue = Z[5], de = Z[6], fe = Z[7];
              return Z !== J && (J[8] = Z[8], J[9] = Z[9], J[10] = Z[10], J[11] = Z[11], J[12] = Z[12], J[13] = Z[13], J[14] = Z[14], J[15] = Z[15]), J[0] = ae * ee + le * ne, J[1] = oe * ee + ue * ne, J[2] = se * ee + de * ne, J[3] = ce * ee + fe * ne, J[4] = le * ee - ae * ne, J[5] = ue * ee - oe * ne, J[6] = de * ee - se * ne, J[7] = fe * ee - ce * ne, J;
            };
          }, 8697: function(G) {
            G.exports = function(J, Z, te) {
              var ne = te[0], ee = te[1], ae = te[2];
              return J[0] = Z[0] * ne, J[1] = Z[1] * ne, J[2] = Z[2] * ne, J[3] = Z[3] * ne, J[4] = Z[4] * ee, J[5] = Z[5] * ee, J[6] = Z[6] * ee, J[7] = Z[7] * ee, J[8] = Z[8] * ae, J[9] = Z[9] * ae, J[10] = Z[10] * ae, J[11] = Z[11] * ae, J[12] = Z[12], J[13] = Z[13], J[14] = Z[14], J[15] = Z[15], J;
            };
          }, 66992: function(G) {
            G.exports = function(J) {
              return "mat4(" + J[0] + ", " + J[1] + ", " + J[2] + ", " + J[3] + ", " + J[4] + ", " + J[5] + ", " + J[6] + ", " + J[7] + ", " + J[8] + ", " + J[9] + ", " + J[10] + ", " + J[11] + ", " + J[12] + ", " + J[13] + ", " + J[14] + ", " + J[15] + ")";
            };
          }, 4165: function(G) {
            G.exports = function(J, Z, te) {
              var ne, ee, ae, oe, se, ce, le, ue, de, fe, he, me, pe = te[0], ye = te[1], ge = te[2];
              return Z === J ? (J[12] = Z[0] * pe + Z[4] * ye + Z[8] * ge + Z[12], J[13] = Z[1] * pe + Z[5] * ye + Z[9] * ge + Z[13], J[14] = Z[2] * pe + Z[6] * ye + Z[10] * ge + Z[14], J[15] = Z[3] * pe + Z[7] * ye + Z[11] * ge + Z[15]) : (ne = Z[0], ee = Z[1], ae = Z[2], oe = Z[3], se = Z[4], ce = Z[5], le = Z[6], ue = Z[7], de = Z[8], fe = Z[9], he = Z[10], me = Z[11], J[0] = ne, J[1] = ee, J[2] = ae, J[3] = oe, J[4] = se, J[5] = ce, J[6] = le, J[7] = ue, J[8] = de, J[9] = fe, J[10] = he, J[11] = me, J[12] = ne * pe + se * ye + de * ge + Z[12], J[13] = ee * pe + ce * ye + fe * ge + Z[13], J[14] = ae * pe + le * ye + he * ge + Z[14], J[15] = oe * pe + ue * ye + me * ge + Z[15]), J;
            };
          }, 10256: function(G) {
            G.exports = function(J, Z) {
              if (J === Z) {
                var te = Z[1], ne = Z[2], ee = Z[3], ae = Z[6], oe = Z[7], se = Z[11];
                J[1] = Z[4], J[2] = Z[8], J[3] = Z[12], J[4] = te, J[6] = Z[9], J[7] = Z[13], J[8] = ne, J[9] = ae, J[11] = Z[14], J[12] = ee, J[13] = oe, J[14] = se;
              } else J[0] = Z[0], J[1] = Z[4], J[2] = Z[8], J[3] = Z[12], J[4] = Z[1], J[5] = Z[5], J[6] = Z[9], J[7] = Z[13], J[8] = Z[2], J[9] = Z[6], J[10] = Z[10], J[11] = Z[14], J[12] = Z[3], J[13] = Z[7], J[14] = Z[11], J[15] = Z[15];
              return J;
            };
          }, 74024: function(G, J, Z) {
            var te = Z(59518), ne = Z(6807), ee = Z(81330), ae = Z(38862), oe = Z(93103), se = Z(162), ce = Z(68950), le = Z(66127), ue = Z(5137), de = Z(29388), fe = Z(4957), he = Z(44626), me = Z(44431), pe = Z(27976), ye = Z(12673), ge = Z(83473), ve = Z(54689).nextPow2, be = new oe(), Ce = !1;
            if (document.body) {
              var Se = document.body.appendChild(document.createElement("div"));
              Se.style.font = "italic small-caps bold condensed 16px/2 cursive", getComputedStyle(Se).fontStretch && (Ce = !0), document.body.removeChild(Se);
            }
            var we = function(Ee) {
              (function(xe) {
                return typeof xe == "function" && xe._gl && xe.prop && xe.texture && xe.buffer;
              })(Ee) ? (Ee = { regl: Ee }, this.gl = Ee.regl._gl) : this.gl = ae(Ee), this.shader = be.get(this.gl), this.shader ? this.regl = this.shader.regl : this.regl = Ee.regl || ee({ gl: this.gl }), this.charBuffer = this.regl.buffer({ type: "uint8", usage: "stream" }), this.sizeBuffer = this.regl.buffer({ type: "float", usage: "stream" }), this.shader || (this.shader = this.createShader(), be.set(this.gl, this.shader)), this.batch = [], this.fontSize = [], this.font = [], this.fontAtlas = [], this.draw = this.shader.draw.bind(this), this.render = function() {
                this.regl._refresh(), this.draw(this.batch);
              }, this.canvas = this.gl.canvas, this.update(de(Ee) ? Ee : {});
            };
            we.prototype.createShader = function() {
              var Ee = this.regl, xe = Ee({ blend: { enable: !0, color: [0, 0, 0, 1], func: { srcRGB: "src alpha", dstRGB: "one minus src alpha", srcAlpha: "one minus dst alpha", dstAlpha: "one" } }, stencil: { enable: !1 }, depth: { enable: !1 }, count: Ee.prop("count"), offset: Ee.prop("offset"), attributes: { charOffset: { offset: 4, stride: 8, buffer: Ee.this("sizeBuffer") }, width: { offset: 0, stride: 8, buffer: Ee.this("sizeBuffer") }, char: Ee.this("charBuffer"), position: Ee.this("position") }, uniforms: { atlasSize: function(Le, Qe) {
                return [Qe.atlas.width, Qe.atlas.height];
              }, atlasDim: function(Le, Qe) {
                return [Qe.atlas.cols, Qe.atlas.rows];
              }, atlas: function(Le, Qe) {
                return Qe.atlas.texture;
              }, charStep: function(Le, Qe) {
                return Qe.atlas.step;
              }, em: function(Le, Qe) {
                return Qe.atlas.em;
              }, color: Ee.prop("color"), opacity: Ee.prop("opacity"), viewport: Ee.this("viewportArray"), scale: Ee.this("scale"), align: Ee.prop("align"), baseline: Ee.prop("baseline"), translate: Ee.this("translate"), positionOffset: Ee.prop("positionOffset") }, primitive: "points", viewport: Ee.this("viewport"), vert: `
			precision highp float;
			attribute float width, charOffset, char;
			attribute vec2 position;
			uniform float fontSize, charStep, em, align, baseline;
			uniform vec4 viewport;
			uniform vec4 color;
			uniform vec2 atlasSize, atlasDim, scale, translate, positionOffset;
			varying vec2 charCoord, charId;
			varying float charWidth;
			varying vec4 fontColor;
			void main () {
				vec2 offset = floor(em * (vec2(align + charOffset, baseline)
					+ vec2(positionOffset.x, -positionOffset.y)))
					/ (viewport.zw * scale.xy);

				vec2 position = (position + translate) * scale;
				position += offset * scale;

				charCoord = position * viewport.zw + viewport.xy;

				gl_Position = vec4(position * 2. - 1., 0, 1);

				gl_PointSize = charStep;

				charId.x = mod(char, atlasDim.x);
				charId.y = floor(char / atlasDim.x);

				charWidth = width * em;

				fontColor = color / 255.;
			}`, frag: `
			precision highp float;
			uniform float fontSize, charStep, opacity;
			uniform vec2 atlasSize;
			uniform vec4 viewport;
			uniform sampler2D atlas;
			varying vec4 fontColor;
			varying vec2 charCoord, charId;
			varying float charWidth;

			float lightness(vec4 color) {
				return color.r * 0.299 + color.g * 0.587 + color.b * 0.114;
			}

			void main () {
				vec2 uv = gl_FragCoord.xy - charCoord + charStep * .5;
				float halfCharStep = floor(charStep * .5 + .5);

				// invert y and shift by 1px (FF expecially needs that)
				uv.y = charStep - uv.y;

				// ignore points outside of character bounding box
				float halfCharWidth = ceil(charWidth * .5);
				if (floor(uv.x) > halfCharStep + halfCharWidth ||
					floor(uv.x) < halfCharStep - halfCharWidth) return;

				uv += charId * charStep;
				uv = uv / atlasSize;

				vec4 color = fontColor;
				vec4 mask = texture2D(atlas, uv);

				float maskY = lightness(mask);
				// float colorY = lightness(color);
				color.a *= maskY;
				color.a *= opacity;

				// color.a += .1;

				// antialiasing, see yiq color space y-channel formula
				// color.rgb += (1. - color.rgb) * (1. - mask.rgb);

				gl_FragColor = color;
			}` });
              return { regl: Ee, draw: xe, atlas: {} };
            }, we.prototype.update = function(Ee) {
              var xe = this;
              if (typeof Ee == "string") Ee = { text: Ee };
              else if (!Ee) return;
              (Ee = ne(Ee, { position: "position positions coord coords coordinates", font: "font fontFace fontface typeface cssFont css-font family fontFamily", fontSize: "fontSize fontsize size font-size", text: "text texts chars characters value values symbols", align: "align alignment textAlign textbaseline", baseline: "baseline textBaseline textbaseline", direction: "dir direction textDirection", color: "color colour fill fill-color fillColor textColor textcolor", kerning: "kerning kern", range: "range dataBox", viewport: "vp viewport viewBox viewbox viewPort", opacity: "opacity alpha transparency visible visibility opaque", offset: "offset positionOffset padding shift indent indentation" }, !0)).opacity != null && (Array.isArray(Ee.opacity) ? this.opacity = Ee.opacity.map(function(Tt) {
                return parseFloat(Tt);
              }) : this.opacity = parseFloat(Ee.opacity)), Ee.viewport != null && (this.viewport = ue(Ee.viewport), this.viewportArray = [this.viewport.x, this.viewport.y, this.viewport.width, this.viewport.height]), this.viewport == null && (this.viewport = { x: 0, y: 0, width: this.gl.drawingBufferWidth, height: this.gl.drawingBufferHeight }, this.viewportArray = [this.viewport.x, this.viewport.y, this.viewport.width, this.viewport.height]), Ee.kerning != null && (this.kerning = Ee.kerning), Ee.offset != null && (typeof Ee.offset == "number" && (Ee.offset = [Ee.offset, 0]), this.positionOffset = ge(Ee.offset)), Ee.direction && (this.direction = Ee.direction), Ee.range && (this.range = Ee.range, this.scale = [1 / (Ee.range[2] - Ee.range[0]), 1 / (Ee.range[3] - Ee.range[1])], this.translate = [-Ee.range[0], -Ee.range[1]]), Ee.scale && (this.scale = Ee.scale), Ee.translate && (this.translate = Ee.translate), this.scale || (this.scale = [1 / this.viewport.width, 1 / this.viewport.height]), this.translate || (this.translate = [0, 0]), this.font.length || Ee.font || (Ee.font = we.baseFontSize + "px sans-serif");
              var Le, Qe = !1, ke = !1;
              if (Ee.font && (Array.isArray(Ee.font) ? Ee.font : [Ee.font]).forEach(function(Tt, It) {
                if (typeof Tt == "string") try {
                  Tt = te.parse(Tt);
                } catch {
                  Tt = te.parse(we.baseFontSize + "px " + Tt);
                }
                else {
                  var Ct = Tt.style, Kt = Tt.weight, Vt = Tt.stretch, Yt = Tt.variant;
                  Tt = te.parse(te.stringify(Tt)), Ct && (Tt.style = Ct), Kt && (Tt.weight = Kt), Vt && (Tt.stretch = Vt), Yt && (Tt.variant = Yt);
                }
                var sn = te.stringify({ size: we.baseFontSize, family: Tt.family, stretch: Ce ? Tt.stretch : void 0, variant: Tt.variant, weight: Tt.weight, style: Tt.style }), hn = fe(Tt.size), Qt = Math.round(hn[0] * he(hn[1]));
                if (Qt !== xe.fontSize[It] && (ke = !0, xe.fontSize[It] = Qt), !(xe.font[It] && sn == xe.font[It].baseString || (Qe = !0, xe.font[It] = we.fonts[sn], xe.font[It]))) {
                  var Ut = Tt.family.join(", "), Wt = [Tt.style];
                  Tt.style != Tt.variant && Wt.push(Tt.variant), Tt.variant != Tt.weight && Wt.push(Tt.weight), Ce && Tt.weight != Tt.stretch && Wt.push(Tt.stretch), xe.font[It] = { baseString: sn, family: Ut, weight: Tt.weight, stretch: Tt.stretch, style: Tt.style, variant: Tt.variant, width: {}, kerning: {}, metrics: ye(Ut, { origin: "top", fontSize: we.baseFontSize, fontStyle: Wt.join(" ") }) }, we.fonts[sn] = xe.font[It];
                }
              }), (Qe || ke) && this.font.forEach(function(Tt, It) {
                var Ct = te.stringify({ size: xe.fontSize[It], family: Tt.family, stretch: Ce ? Tt.stretch : void 0, variant: Tt.variant, weight: Tt.weight, style: Tt.style });
                if (xe.fontAtlas[It] = xe.shader.atlas[Ct], !xe.fontAtlas[It]) {
                  var Kt = Tt.metrics;
                  xe.shader.atlas[Ct] = xe.fontAtlas[It] = { fontString: Ct, step: 2 * Math.ceil(xe.fontSize[It] * Kt.bottom * 0.5), em: xe.fontSize[It], cols: 0, rows: 0, height: 0, width: 0, chars: [], ids: {}, texture: xe.regl.texture() };
                }
                Ee.text == null && (Ee.text = xe.text);
              }), typeof Ee.text == "string" && Ee.position && Ee.position.length > 2) {
                for (var Me = Array(0.5 * Ee.position.length), Re = 0; Re < Me.length; Re++) Me[Re] = Ee.text;
                Ee.text = Me;
              }
              if (Ee.text != null || Qe) {
                if (this.textOffsets = [0], Array.isArray(Ee.text)) {
                  this.count = Ee.text[0].length, this.counts = [this.count];
                  for (var De = 1; De < Ee.text.length; De++) this.textOffsets[De] = this.textOffsets[De - 1] + Ee.text[De - 1].length, this.count += Ee.text[De].length, this.counts.push(Ee.text[De].length);
                  this.text = Ee.text.join("");
                } else this.text = Ee.text, this.count = this.text.length, this.counts = [this.count];
                Le = [], this.font.forEach(function(Tt, It) {
                  we.atlasContext.font = Tt.baseString;
                  for (var Ct = xe.fontAtlas[It], Kt = 0; Kt < xe.text.length; Kt++) {
                    var Vt = xe.text.charAt(Kt);
                    if (Ct.ids[Vt] == null && (Ct.ids[Vt] = Ct.chars.length, Ct.chars.push(Vt), Le.push(Vt)), Tt.width[Vt] == null && (Tt.width[Vt] = we.atlasContext.measureText(Vt).width / we.baseFontSize, xe.kerning)) {
                      var Yt = [];
                      for (var sn in Tt.width) Yt.push(sn + Vt, Vt + sn);
                      pe(Tt.kerning, me(Tt.family, { pairs: Yt }));
                    }
                  }
                });
              }
              if (Ee.position) if (Ee.position.length > 2) {
                for (var Pe = !Ee.position[0].length, $e = le.mallocFloat(2 * this.count), qe = 0, Xe = 0; qe < this.counts.length; qe++) {
                  var Ve = this.counts[qe];
                  if (Pe) for (var Ze = 0; Ze < Ve; Ze++) $e[Xe++] = Ee.position[2 * qe], $e[Xe++] = Ee.position[2 * qe + 1];
                  else for (var Ge = 0; Ge < Ve; Ge++) $e[Xe++] = Ee.position[qe][0], $e[Xe++] = Ee.position[qe][1];
                }
                this.position.call ? this.position({ type: "float", data: $e }) : this.position = this.regl.buffer({ type: "float", data: $e }), le.freeFloat($e);
              } else this.position.destroy && this.position.destroy(), this.position = { constant: Ee.position };
              if (Ee.text || Qe) {
                var ze = le.mallocUint8(this.count), We = le.mallocFloat(2 * this.count);
                this.textWidth = [];
                for (var at = 0, ct = 0; at < this.counts.length; at++) {
                  for (var et = this.counts[at], dt = this.font[at] || this.font[0], Je = this.fontAtlas[at] || this.fontAtlas[0], it = 0; it < et; it++) {
                    var st = this.text.charAt(ct), gt = this.text.charAt(ct - 1);
                    if (ze[ct] = Je.ids[st], We[2 * ct] = dt.width[st], it) {
                      var bt = We[2 * ct - 2], yt = We[2 * ct], St = We[2 * ct - 1] + 0.5 * bt + 0.5 * yt;
                      if (this.kerning) {
                        var Lt = dt.kerning[gt + st];
                        Lt && (St += 1e-3 * Lt);
                      }
                      We[2 * ct + 1] = St;
                    } else We[2 * ct + 1] = 0.5 * We[2 * ct];
                    ct++;
                  }
                  this.textWidth.push(We.length ? 0.5 * We[2 * ct - 2] + We[2 * ct - 1] : 0);
                }
                Ee.align || (Ee.align = this.align), this.charBuffer({ data: ze, type: "uint8", usage: "stream" }), this.sizeBuffer({ data: We, type: "float", usage: "stream" }), le.freeUint8(ze), le.freeFloat(We), Le.length && this.font.forEach(function(Tt, It) {
                  var Ct = xe.fontAtlas[It], Kt = Ct.step, Vt = Math.floor(we.maxAtlasSize / Kt), Yt = Math.min(Vt, Ct.chars.length), sn = Math.ceil(Ct.chars.length / Yt), hn = ve(Yt * Kt), Qt = ve(sn * Kt);
                  Ct.width = hn, Ct.height = Qt, Ct.rows = sn, Ct.cols = Yt, Ct.em && Ct.texture({ data: ce({ canvas: we.atlasCanvas, font: Ct.fontString, chars: Ct.chars, shape: [hn, Qt], step: [Kt, Kt] }) });
                });
              }
              if (Ee.align && (this.align = Ee.align, this.alignOffset = this.textWidth.map(function(Tt, It) {
                var Ct = Array.isArray(xe.align) ? xe.align.length > 1 ? xe.align[It] : xe.align[0] : xe.align;
                if (typeof Ct == "number") return Ct;
                switch (Ct) {
                  case "right":
                  case "end":
                    return -Tt;
                  case "center":
                  case "centre":
                  case "middle":
                    return 0.5 * -Tt;
                }
                return 0;
              })), this.baseline == null && Ee.baseline == null && (Ee.baseline = 0), Ee.baseline != null && (this.baseline = Ee.baseline, Array.isArray(this.baseline) || (this.baseline = [this.baseline]), this.baselineOffset = this.baseline.map(function(Tt, It) {
                var Ct = (xe.font[It] || xe.font[0]).metrics, Kt = 0;
                return Kt += 0.5 * Ct.bottom, -1 * (Kt += typeof Tt == "number" ? Tt - Ct.baseline : -Ct[Tt]);
              })), Ee.color != null) if (Ee.color || (Ee.color = "transparent"), typeof Ee.color != "string" && isNaN(Ee.color)) {
                var Rt;
                if (typeof Ee.color[0] == "number" && Ee.color.length > this.counts.length) {
                  var vt = Ee.color.length;
                  Rt = le.mallocUint8(vt);
                  for (var $t = (Ee.color.subarray || Ee.color.slice).bind(Ee.color), Pt = 0; Pt < vt; Pt += 4) Rt.set(se($t(Pt, Pt + 4), "uint8"), Pt);
                } else {
                  var Xt = Ee.color.length;
                  Rt = le.mallocUint8(4 * Xt);
                  for (var Nt = 0; Nt < Xt; Nt++) Rt.set(se(Ee.color[Nt] || 0, "uint8"), 4 * Nt);
                }
                this.color = Rt;
              } else this.color = se(Ee.color, "uint8");
              if (Ee.position || Ee.text || Ee.color || Ee.baseline || Ee.align || Ee.font || Ee.offset || Ee.opacity) if (this.color.length > 4 || this.baselineOffset.length > 1 || this.align && this.align.length > 1 || this.fontAtlas.length > 1 || this.positionOffset.length > 2) {
                var Gt = Math.max(0.5 * this.position.length || 0, 0.25 * this.color.length || 0, this.baselineOffset.length || 0, this.alignOffset.length || 0, this.font.length || 0, this.opacity.length || 0, 0.5 * this.positionOffset.length || 0);
                this.batch = Array(Gt);
                for (var Ft = 0; Ft < this.batch.length; Ft++) this.batch[Ft] = { count: this.counts.length > 1 ? this.counts[Ft] : this.counts[0], offset: this.textOffsets.length > 1 ? this.textOffsets[Ft] : this.textOffsets[0], color: this.color ? this.color.length <= 4 ? this.color : this.color.subarray(4 * Ft, 4 * Ft + 4) : [0, 0, 0, 255], opacity: Array.isArray(this.opacity) ? this.opacity[Ft] : this.opacity, baseline: this.baselineOffset[Ft] != null ? this.baselineOffset[Ft] : this.baselineOffset[0], align: this.align ? this.alignOffset[Ft] != null ? this.alignOffset[Ft] : this.alignOffset[0] : 0, atlas: this.fontAtlas[Ft] || this.fontAtlas[0], positionOffset: this.positionOffset.length > 2 ? this.positionOffset.subarray(2 * Ft, 2 * Ft + 2) : this.positionOffset };
              } else this.count ? this.batch = [{ count: this.count, offset: 0, color: this.color || [0, 0, 0, 255], opacity: Array.isArray(this.opacity) ? this.opacity[0] : this.opacity, baseline: this.baselineOffset[0], align: this.alignOffset ? this.alignOffset[0] : 0, atlas: this.fontAtlas[0], positionOffset: this.positionOffset }] : this.batch = [];
            }, we.prototype.destroy = function() {
            }, we.prototype.kerning = !0, we.prototype.position = { constant: new Float32Array(2) }, we.prototype.translate = null, we.prototype.scale = null, we.prototype.font = null, we.prototype.text = "", we.prototype.positionOffset = [0, 0], we.prototype.opacity = 1, we.prototype.color = new Uint8Array([0, 0, 0, 255]), we.prototype.alignOffset = [0, 0], we.maxAtlasSize = 1024, we.atlasCanvas = document.createElement("canvas"), we.atlasContext = we.atlasCanvas.getContext("2d", { alpha: !1 }), we.baseFontSize = 64, we.fonts = {}, G.exports = we;
          }, 38862: function(G, J, Z) {
            var te = Z(6807);
            function ne(oe) {
              if (oe.container) if (oe.container == document.body) document.body.style.width || (oe.canvas.width = oe.width || oe.pixelRatio * Z.g.innerWidth), document.body.style.height || (oe.canvas.height = oe.height || oe.pixelRatio * Z.g.innerHeight);
              else {
                var se = oe.container.getBoundingClientRect();
                oe.canvas.width = oe.width || se.right - se.left, oe.canvas.height = oe.height || se.bottom - se.top;
              }
            }
            function ee(oe) {
              return typeof oe.getContext == "function" && "width" in oe && "height" in oe;
            }
            function ae() {
              var oe = document.createElement("canvas");
              return oe.style.position = "absolute", oe.style.top = 0, oe.style.left = 0, oe;
            }
            G.exports = function(oe) {
              var se;
              if (oe ? typeof oe == "string" && (oe = { container: oe }) : oe = {}, (oe = ee(oe) || typeof (se = oe).nodeName == "string" && typeof se.appendChild == "function" && typeof se.getBoundingClientRect == "function" ? { container: oe } : function(le) {
                return typeof le.drawArrays == "function" || typeof le.drawElements == "function";
              }(oe) ? { gl: oe } : te(oe, { container: "container target element el canvas holder parent parentNode wrapper use ref root node", gl: "gl context webgl glContext", attrs: "attributes attrs contextAttributes", pixelRatio: "pixelRatio pxRatio px ratio pxratio pixelratio", width: "w width", height: "h height" }, !0)).pixelRatio || (oe.pixelRatio = Z.g.pixelRatio || 1), oe.gl) return oe.gl;
              if (oe.canvas && (oe.container = oe.canvas.parentNode), oe.container) {
                if (typeof oe.container == "string") {
                  var ce = document.querySelector(oe.container);
                  if (!ce) throw Error("Element " + oe.container + " is not found");
                  oe.container = ce;
                }
                ee(oe.container) ? (oe.canvas = oe.container, oe.container = oe.canvas.parentNode) : oe.canvas || (oe.canvas = ae(), oe.container.appendChild(oe.canvas), ne(oe));
              } else if (!oe.canvas) {
                if (typeof document > "u") throw Error("Not DOM environment. Use headless-gl.");
                oe.container = document.body || document.documentElement, oe.canvas = ae(), oe.container.appendChild(oe.canvas), ne(oe);
              }
              return oe.gl || ["webgl", "experimental-webgl", "webgl-experimental"].some(function(le) {
                try {
                  oe.gl = oe.canvas.getContext(le, oe.attrs);
                } catch {
                }
                return oe.gl;
              }), oe.gl;
            };
          }, 76765: function(G) {
            G.exports = function(J) {
              typeof J == "string" && (J = [J]);
              for (var Z = [].slice.call(arguments, 1), te = [], ne = 0; ne < J.length - 1; ne++) te.push(J[ne], Z[ne] || "");
              return te.push(J[ne]), te.join("");
            };
          }, 52991: function(G, J, Z) {
            var te = Z(71129)("%Object.getOwnPropertyDescriptor%", !0);
            if (te) try {
              te([], "length");
            } catch {
              te = null;
            }
            G.exports = te;
          }, 39784: function(G, J, Z) {
            var te, ne = Z(78253);
            te = typeof Z.g.matchMedia == "function" ? !Z.g.matchMedia("(hover: none)").matches : ne, G.exports = te;
          }, 74043: function(G, J, Z) {
            var te = Z(78253);
            G.exports = te && function() {
              var ne = !1;
              try {
                var ee = Object.defineProperty({}, "passive", { get: function() {
                  ne = !0;
                } });
                window.addEventListener("test", null, ee), window.removeEventListener("test", null, ee);
              } catch {
                ne = !1;
              }
              return ne;
            }();
          }, 74268: function(G, J, Z) {
            var te = Z(40891), ne = function() {
              return !!te;
            };
            ne.hasArrayLengthDefineBug = function() {
              if (!te) return null;
              try {
                return te([], "length", { value: 1 }).length !== 1;
              } catch {
                return !0;
              }
            }, G.exports = ne;
          }, 58436: function(G) {
            var J = { foo: {} }, Z = Object;
            G.exports = function() {
              return { __proto__: J }.foo === J.foo && !({ __proto__: null } instanceof Z);
            };
          }, 8771: function(G, J, Z) {
            var te = typeof Symbol < "u" && Symbol, ne = Z(59457);
            G.exports = function() {
              return typeof te == "function" && typeof Symbol == "function" && typeof te("foo") == "symbol" && typeof Symbol("bar") == "symbol" && ne();
            };
          }, 59457: function(G) {
            G.exports = function() {
              if (typeof Symbol != "function" || typeof Object.getOwnPropertySymbols != "function") return !1;
              if (typeof Symbol.iterator == "symbol") return !0;
              var J = {}, Z = Symbol("test"), te = Object(Z);
              if (typeof Z == "string" || Object.prototype.toString.call(Z) !== "[object Symbol]" || Object.prototype.toString.call(te) !== "[object Symbol]") return !1;
              for (Z in J[Z] = 42, J) return !1;
              if (typeof Object.keys == "function" && Object.keys(J).length !== 0 || typeof Object.getOwnPropertyNames == "function" && Object.getOwnPropertyNames(J).length !== 0) return !1;
              var ne = Object.getOwnPropertySymbols(J);
              if (ne.length !== 1 || ne[0] !== Z || !Object.prototype.propertyIsEnumerable.call(J, Z)) return !1;
              if (typeof Object.getOwnPropertyDescriptor == "function") {
                var ee = Object.getOwnPropertyDescriptor(J, Z);
                if (ee.value !== 42 || ee.enumerable !== !0) return !1;
              }
              return !0;
            };
          }, 36912: function(G, J, Z) {
            var te = Z(59457);
            G.exports = function() {
              return te() && !!Symbol.toStringTag;
            };
          }, 80753: function(G, J, Z) {
            var te = Function.prototype.call, ne = Object.prototype.hasOwnProperty, ee = Z(87547);
            G.exports = ee.call(te, ne);
          }, 27415: function(G, J) {
            J.read = function(Z, te, ne, ee, ae) {
              var oe, se, ce = 8 * ae - ee - 1, le = (1 << ce) - 1, ue = le >> 1, de = -7, fe = ne ? ae - 1 : 0, he = ne ? -1 : 1, me = Z[te + fe];
              for (fe += he, oe = me & (1 << -de) - 1, me >>= -de, de += ce; de > 0; oe = 256 * oe + Z[te + fe], fe += he, de -= 8) ;
              for (se = oe & (1 << -de) - 1, oe >>= -de, de += ee; de > 0; se = 256 * se + Z[te + fe], fe += he, de -= 8) ;
              if (oe === 0) oe = 1 - ue;
              else {
                if (oe === le) return se ? NaN : 1 / 0 * (me ? -1 : 1);
                se += Math.pow(2, ee), oe -= ue;
              }
              return (me ? -1 : 1) * se * Math.pow(2, oe - ee);
            }, J.write = function(Z, te, ne, ee, ae, oe) {
              var se, ce, le, ue = 8 * oe - ae - 1, de = (1 << ue) - 1, fe = de >> 1, he = ae === 23 ? Math.pow(2, -24) - Math.pow(2, -77) : 0, me = ee ? 0 : oe - 1, pe = ee ? 1 : -1, ye = te < 0 || te === 0 && 1 / te < 0 ? 1 : 0;
              for (te = Math.abs(te), isNaN(te) || te === 1 / 0 ? (ce = isNaN(te) ? 1 : 0, se = de) : (se = Math.floor(Math.log(te) / Math.LN2), te * (le = Math.pow(2, -se)) < 1 && (se--, le *= 2), (te += se + fe >= 1 ? he / le : he * Math.pow(2, 1 - fe)) * le >= 2 && (se++, le /= 2), se + fe >= de ? (ce = 0, se = de) : se + fe >= 1 ? (ce = (te * le - 1) * Math.pow(2, ae), se += fe) : (ce = te * Math.pow(2, fe - 1) * Math.pow(2, ae), se = 0)); ae >= 8; Z[ne + me] = 255 & ce, me += pe, ce /= 256, ae -= 8) ;
              for (se = se << ae | ce, ue += ae; ue > 0; Z[ne + me] = 255 & se, me += pe, se /= 256, ue -= 8) ;
              Z[ne + me - pe] |= 128 * ye;
            };
          }, 28062: function(G) {
            typeof Object.create == "function" ? G.exports = function(J, Z) {
              Z && (J.super_ = Z, J.prototype = Object.create(Z.prototype, { constructor: { value: J, enumerable: !1, writable: !0, configurable: !0 } }));
            } : G.exports = function(J, Z) {
              if (Z) {
                J.super_ = Z;
                var te = function() {
                };
                te.prototype = Z.prototype, J.prototype = new te(), J.prototype.constructor = J;
              }
            };
          }, 40280: function(G, J, Z) {
            var te = Z(36912)(), ne = Z(63063)("Object.prototype.toString"), ee = function(se) {
              return !(te && se && typeof se == "object" && Symbol.toStringTag in se) && ne(se) === "[object Arguments]";
            }, ae = function(se) {
              return !!ee(se) || se !== null && typeof se == "object" && typeof se.length == "number" && se.length >= 0 && ne(se) !== "[object Array]" && ne(se.callee) === "[object Function]";
            }, oe = function() {
              return ee(arguments);
            }();
            ee.isLegacyArguments = ae, G.exports = oe ? ee : ae;
          }, 78253: function(G) {
            G.exports = !0;
          }, 82756: function(G) {
            var J, Z, te = Function.prototype.toString, ne = typeof Reflect == "object" && Reflect !== null && Reflect.apply;
            if (typeof ne == "function" && typeof Object.defineProperty == "function") try {
              J = Object.defineProperty({}, "length", { get: function() {
                throw Z;
              } }), Z = {}, ne(function() {
                throw 42;
              }, null, J);
            } catch (fe) {
              fe !== Z && (ne = null);
            }
            else ne = null;
            var ee = /^\s*class\b/, ae = function(fe) {
              try {
                var he = te.call(fe);
                return ee.test(he);
              } catch {
                return !1;
              }
            }, oe = function(fe) {
              try {
                return !ae(fe) && (te.call(fe), !0);
              } catch {
                return !1;
              }
            }, se = Object.prototype.toString, ce = typeof Symbol == "function" && !!Symbol.toStringTag, le = !(0 in [,]), ue = function() {
              return !1;
            };
            if (typeof document == "object") {
              var de = document.all;
              se.call(de) === se.call(document.all) && (ue = function(fe) {
                if ((le || !fe) && (fe === void 0 || typeof fe == "object")) try {
                  var he = se.call(fe);
                  return (he === "[object HTMLAllCollection]" || he === "[object HTML document.all class]" || he === "[object HTMLCollection]" || he === "[object Object]") && fe("") == null;
                } catch {
                }
                return !1;
              });
            }
            G.exports = ne ? function(fe) {
              if (ue(fe)) return !0;
              if (!fe || typeof fe != "function" && typeof fe != "object") return !1;
              try {
                ne(fe, null, J);
              } catch (he) {
                if (he !== Z) return !1;
              }
              return !ae(fe) && oe(fe);
            } : function(fe) {
              if (ue(fe)) return !0;
              if (!fe || typeof fe != "function" && typeof fe != "object") return !1;
              if (ce) return oe(fe);
              if (ae(fe)) return !1;
              var he = se.call(fe);
              return !(he !== "[object Function]" && he !== "[object GeneratorFunction]" && !/^\[object HTML/.test(he)) && oe(fe);
            };
          }, 80340: function(G, J, Z) {
            var te, ne = Object.prototype.toString, ee = Function.prototype.toString, ae = /^\s*(?:function)?\*/, oe = Z(36912)(), se = Object.getPrototypeOf;
            G.exports = function(ce) {
              if (typeof ce != "function") return !1;
              if (ae.test(ee.call(ce))) return !0;
              if (!oe) return ne.call(ce) === "[object GeneratorFunction]";
              if (!se) return !1;
              if (te === void 0) {
                var le = function() {
                  if (!oe) return !1;
                  try {
                    return Function("return function*() {}")();
                  } catch {
                  }
                }();
                te = !!le && se(le);
              }
              return se(ce) === te;
            };
          }, 39488: function(G) {
            G.exports = typeof navigator < "u" && (/MSIE/.test(navigator.userAgent) || /Trident\//.test(navigator.appVersion));
          }, 73287: function(G) {
            G.exports = function(J) {
              return J != J;
            };
          }, 63057: function(G, J, Z) {
            var te = Z(87227), ne = Z(97936), ee = Z(73287), ae = Z(60758), oe = Z(85684), se = te(ae(), Number);
            ne(se, { getPolyfill: ae, implementation: ee, shim: oe }), G.exports = se;
          }, 60758: function(G, J, Z) {
            var te = Z(73287);
            G.exports = function() {
              return Number.isNaN && Number.isNaN(NaN) && !Number.isNaN("a") ? Number.isNaN : te;
            };
          }, 85684: function(G, J, Z) {
            var te = Z(97936), ne = Z(60758);
            G.exports = function() {
              var ee = ne();
              return te(Number, { isNaN: ee }, { isNaN: function() {
                return Number.isNaN !== ee;
              } }), ee;
            };
          }, 60201: function(G) {
            G.exports = function(J) {
              var Z = typeof J;
              return J !== null && (Z === "object" || Z === "function");
            };
          }, 29388: function(G) {
            var J = Object.prototype.toString;
            G.exports = function(Z) {
              var te;
              return J.call(Z) === "[object Object]" && ((te = Object.getPrototypeOf(Z)) === null || te === Object.getPrototypeOf({}));
            };
          }, 9914: function(G) {
            G.exports = function(J) {
              for (var Z, te = J.length, ne = 0; ne < te; ne++) if (((Z = J.charCodeAt(ne)) < 9 || Z > 13) && Z !== 32 && Z !== 133 && Z !== 160 && Z !== 5760 && Z !== 6158 && (Z < 8192 || Z > 8205) && Z !== 8232 && Z !== 8233 && Z !== 8239 && Z !== 8287 && Z !== 8288 && Z !== 12288 && Z !== 65279) return !1;
              return !0;
            };
          }, 13986: function(G) {
            G.exports = function(J) {
              return typeof J == "string" && (J = J.trim(), !!(/^[mzlhvcsqta]\s*[-+.0-9][^mlhvzcsqta]+/i.test(J) && /[\dz]$/i.test(J) && J.length > 4));
            };
          }, 15628: function(G, J, Z) {
            var te = Z(61262), ne = Z(70085), ee = Z(63063), ae = ee("Object.prototype.toString"), oe = Z(36912)(), se = Z(52991), ce = typeof globalThis > "u" ? Z.g : globalThis, le = ne(), ue = ee("Array.prototype.indexOf", !0) || function(me, pe) {
              for (var ye = 0; ye < me.length; ye += 1) if (me[ye] === pe) return ye;
              return -1;
            }, de = ee("String.prototype.slice"), fe = {}, he = Object.getPrototypeOf;
            oe && se && he && te(le, function(me) {
              var pe = new ce[me]();
              if (Symbol.toStringTag in pe) {
                var ye = he(pe), ge = se(ye, Symbol.toStringTag);
                if (!ge) {
                  var ve = he(ye);
                  ge = se(ve, Symbol.toStringTag);
                }
                fe[me] = ge.get;
              }
            }), G.exports = function(me) {
              if (!me || typeof me != "object") return !1;
              if (!oe || !(Symbol.toStringTag in me)) {
                var pe = de(ae(me), 8, -1);
                return ue(le, pe) > -1;
              }
              return !!se && function(ye) {
                var ge = !1;
                return te(fe, function(ve, be) {
                  if (!ge) try {
                    ge = ve.call(ye) === be;
                  } catch {
                  }
                }), ge;
              }(me);
            };
          }, 62914: function(G) {
            G.exports = Math.log2 || function(J) {
              return Math.log(J) * Math.LOG2E;
            };
          }, 99978: function(G, J, Z) {
            G.exports = function(ne, ee) {
              ee || (ee = ne, ne = window);
              var ae = 0, oe = 0, se = 0, ce = { shift: !1, alt: !1, control: !1, meta: !1 }, le = !1;
              function ue(Ce) {
                var Se = !1;
                return "altKey" in Ce && (Se = Se || Ce.altKey !== ce.alt, ce.alt = !!Ce.altKey), "shiftKey" in Ce && (Se = Se || Ce.shiftKey !== ce.shift, ce.shift = !!Ce.shiftKey), "ctrlKey" in Ce && (Se = Se || Ce.ctrlKey !== ce.control, ce.control = !!Ce.ctrlKey), "metaKey" in Ce && (Se = Se || Ce.metaKey !== ce.meta, ce.meta = !!Ce.metaKey), Se;
              }
              function de(Ce, Se) {
                var we = te.x(Se), Ee = te.y(Se);
                "buttons" in Se && (Ce = 0 | Se.buttons), (Ce !== ae || we !== oe || Ee !== se || ue(Se)) && (ae = 0 | Ce, oe = we || 0, se = Ee || 0, ee && ee(ae, oe, se, ce));
              }
              function fe(Ce) {
                de(0, Ce);
              }
              function he() {
                (ae || oe || se || ce.shift || ce.alt || ce.meta || ce.control) && (oe = se = 0, ae = 0, ce.shift = ce.alt = ce.control = ce.meta = !1, ee && ee(0, 0, 0, ce));
              }
              function me(Ce) {
                ue(Ce) && ee && ee(ae, oe, se, ce);
              }
              function pe(Ce) {
                te.buttons(Ce) === 0 ? de(0, Ce) : de(ae, Ce);
              }
              function ye(Ce) {
                de(ae | te.buttons(Ce), Ce);
              }
              function ge(Ce) {
                de(ae & ~te.buttons(Ce), Ce);
              }
              function ve() {
                le || (le = !0, ne.addEventListener("mousemove", pe), ne.addEventListener("mousedown", ye), ne.addEventListener("mouseup", ge), ne.addEventListener("mouseleave", fe), ne.addEventListener("mouseenter", fe), ne.addEventListener("mouseout", fe), ne.addEventListener("mouseover", fe), ne.addEventListener("blur", he), ne.addEventListener("keyup", me), ne.addEventListener("keydown", me), ne.addEventListener("keypress", me), ne !== window && (window.addEventListener("blur", he), window.addEventListener("keyup", me), window.addEventListener("keydown", me), window.addEventListener("keypress", me)));
              }
              ve();
              var be = { element: ne };
              return Object.defineProperties(be, { enabled: { get: function() {
                return le;
              }, set: function(Ce) {
                Ce ? ve() : le && (le = !1, ne.removeEventListener("mousemove", pe), ne.removeEventListener("mousedown", ye), ne.removeEventListener("mouseup", ge), ne.removeEventListener("mouseleave", fe), ne.removeEventListener("mouseenter", fe), ne.removeEventListener("mouseout", fe), ne.removeEventListener("mouseover", fe), ne.removeEventListener("blur", he), ne.removeEventListener("keyup", me), ne.removeEventListener("keydown", me), ne.removeEventListener("keypress", me), ne !== window && (window.removeEventListener("blur", he), window.removeEventListener("keyup", me), window.removeEventListener("keydown", me), window.removeEventListener("keypress", me)));
              }, enumerable: !0 }, buttons: { get: function() {
                return ae;
              }, enumerable: !0 }, x: { get: function() {
                return oe;
              }, enumerable: !0 }, y: { get: function() {
                return se;
              }, enumerable: !0 }, mods: { get: function() {
                return ce;
              }, enumerable: !0 } }), be;
            };
            var te = Z(41926);
          }, 44039: function(G) {
            var J = { left: 0, top: 0 };
            G.exports = function(Z, te, ne) {
              te = te || Z.currentTarget || Z.srcElement, Array.isArray(ne) || (ne = [0, 0]);
              var ee, ae = Z.clientX || 0, oe = Z.clientY || 0, se = (ee = te) === window || ee === document || ee === document.body ? J : ee.getBoundingClientRect();
              return ne[0] = ae - se.left, ne[1] = oe - se.top, ne;
            };
          }, 41926: function(G, J) {
            function Z(te) {
              return te.target || te.srcElement || window;
            }
            J.buttons = function(te) {
              if (typeof te == "object") {
                if ("buttons" in te) return te.buttons;
                if ("which" in te) {
                  if ((ne = te.which) === 2) return 4;
                  if (ne === 3) return 2;
                  if (ne > 0) return 1 << ne - 1;
                } else if ("button" in te) {
                  var ne;
                  if ((ne = te.button) === 1) return 4;
                  if (ne === 2) return 2;
                  if (ne >= 0) return 1 << ne;
                }
              }
              return 0;
            }, J.element = Z, J.x = function(te) {
              if (typeof te == "object") {
                if ("offsetX" in te) return te.offsetX;
                var ne = Z(te).getBoundingClientRect();
                return te.clientX - ne.left;
              }
              return 0;
            }, J.y = function(te) {
              if (typeof te == "object") {
                if ("offsetY" in te) return te.offsetY;
                var ne = Z(te).getBoundingClientRect();
                return te.clientY - ne.top;
              }
              return 0;
            };
          }, 20573: function(G, J, Z) {
            var te = Z(44626);
            G.exports = function(ne, ee, ae) {
              typeof ne == "function" && (ae = !!ee, ee = ne, ne = window);
              var oe = te("ex", ne), se = function(ce) {
                ae && ce.preventDefault();
                var le = ce.deltaX || 0, ue = ce.deltaY || 0, de = ce.deltaZ || 0, fe = 1;
                switch (ce.deltaMode) {
                  case 1:
                    fe = oe;
                    break;
                  case 2:
                    fe = window.innerHeight;
                }
                if (ue *= fe, de *= fe, (le *= fe) || ue || de) return ee(le, ue, de, ce);
              };
              return ne.addEventListener("wheel", se), se;
            };
          }, 71116: function(G, J, Z) {
            var te;
            (function(ne, ee, ae) {
              ee[ne] = ee[ne] || function() {
                var oe, se, ce, le = Object.prototype.toString, ue = typeof setImmediate < "u" ? function(we) {
                  return setImmediate(we);
                } : setTimeout;
                try {
                  Object.defineProperty({}, "x", {}), oe = function(we, Ee, xe, Le) {
                    return Object.defineProperty(we, Ee, { value: xe, writable: !0, configurable: Le !== !1 });
                  };
                } catch {
                  oe = function(Ee, xe, Le) {
                    return Ee[xe] = Le, Ee;
                  };
                }
                function de(we, Ee) {
                  ce.add(we, Ee), se || (se = ue(ce.drain));
                }
                function fe(we) {
                  var Ee, xe = typeof we;
                  return we == null || xe != "object" && xe != "function" || (Ee = we.then), typeof Ee == "function" && Ee;
                }
                function he() {
                  for (var we = 0; we < this.chain.length; we++) me(this, this.state === 1 ? this.chain[we].success : this.chain[we].failure, this.chain[we]);
                  this.chain.length = 0;
                }
                function me(we, Ee, xe) {
                  var Le, Qe;
                  try {
                    Ee === !1 ? xe.reject(we.msg) : (Le = Ee === !0 ? we.msg : Ee.call(void 0, we.msg)) === xe.promise ? xe.reject(TypeError("Promise-chain cycle")) : (Qe = fe(Le)) ? Qe.call(Le, xe.resolve, xe.reject) : xe.resolve(Le);
                  } catch (ke) {
                    xe.reject(ke);
                  }
                }
                function pe(we) {
                  var Ee, xe = this;
                  if (!xe.triggered) {
                    xe.triggered = !0, xe.def && (xe = xe.def);
                    try {
                      (Ee = fe(we)) ? de(function() {
                        var Le = new ve(xe);
                        try {
                          Ee.call(we, function() {
                            pe.apply(Le, arguments);
                          }, function() {
                            ye.apply(Le, arguments);
                          });
                        } catch (Qe) {
                          ye.call(Le, Qe);
                        }
                      }) : (xe.msg = we, xe.state = 1, xe.chain.length > 0 && de(he, xe));
                    } catch (Le) {
                      ye.call(new ve(xe), Le);
                    }
                  }
                }
                function ye(we) {
                  var Ee = this;
                  Ee.triggered || (Ee.triggered = !0, Ee.def && (Ee = Ee.def), Ee.msg = we, Ee.state = 2, Ee.chain.length > 0 && de(he, Ee));
                }
                function ge(we, Ee, xe, Le) {
                  for (var Qe = 0; Qe < Ee.length; Qe++) (function(ke) {
                    we.resolve(Ee[ke]).then(function(Me) {
                      xe(ke, Me);
                    }, Le);
                  })(Qe);
                }
                function ve(we) {
                  this.def = we, this.triggered = !1;
                }
                function be(we) {
                  this.promise = we, this.state = 0, this.triggered = !1, this.chain = [], this.msg = void 0;
                }
                function Ce(we) {
                  if (typeof we != "function") throw TypeError("Not a function");
                  if (this.__NPO__ !== 0) throw TypeError("Not a promise");
                  this.__NPO__ = 1;
                  var Ee = new be(this);
                  this.then = function(xe, Le) {
                    var Qe = { success: typeof xe != "function" || xe, failure: typeof Le == "function" && Le };
                    return Qe.promise = new this.constructor(function(ke, Me) {
                      if (typeof ke != "function" || typeof Me != "function") throw TypeError("Not a function");
                      Qe.resolve = ke, Qe.reject = Me;
                    }), Ee.chain.push(Qe), Ee.state !== 0 && de(he, Ee), Qe.promise;
                  }, this.catch = function(xe) {
                    return this.then(void 0, xe);
                  };
                  try {
                    we.call(void 0, function(xe) {
                      pe.call(Ee, xe);
                    }, function(xe) {
                      ye.call(Ee, xe);
                    });
                  } catch (xe) {
                    ye.call(Ee, xe);
                  }
                }
                ce = /* @__PURE__ */ function() {
                  var we, Ee, xe;
                  function Le(Qe, ke) {
                    this.fn = Qe, this.self = ke, this.next = void 0;
                  }
                  return { add: function(Qe, ke) {
                    xe = new Le(Qe, ke), Ee ? Ee.next = xe : we = xe, Ee = xe, xe = void 0;
                  }, drain: function() {
                    var Qe = we;
                    for (we = Ee = se = void 0; Qe; ) Qe.fn.call(Qe.self), Qe = Qe.next;
                  } };
                }();
                var Se = oe({}, "constructor", Ce, !1);
                return Ce.prototype = Se, oe(Se, "__NPO__", 0, !1), oe(Ce, "resolve", function(we) {
                  return we && typeof we == "object" && we.__NPO__ === 1 ? we : new this(function(Ee, xe) {
                    if (typeof Ee != "function" || typeof xe != "function") throw TypeError("Not a function");
                    Ee(we);
                  });
                }), oe(Ce, "reject", function(we) {
                  return new this(function(Ee, xe) {
                    if (typeof Ee != "function" || typeof xe != "function") throw TypeError("Not a function");
                    xe(we);
                  });
                }), oe(Ce, "all", function(we) {
                  var Ee = this;
                  return le.call(we) != "[object Array]" ? Ee.reject(TypeError("Not an array")) : we.length === 0 ? Ee.resolve([]) : new Ee(function(xe, Le) {
                    if (typeof xe != "function" || typeof Le != "function") throw TypeError("Not a function");
                    var Qe = we.length, ke = Array(Qe), Me = 0;
                    ge(Ee, we, function(Re, De) {
                      ke[Re] = De, ++Me === Qe && xe(ke);
                    }, Le);
                  });
                }), oe(Ce, "race", function(we) {
                  var Ee = this;
                  return le.call(we) != "[object Array]" ? Ee.reject(TypeError("Not an array")) : new Ee(function(xe, Le) {
                    if (typeof xe != "function" || typeof Le != "function") throw TypeError("Not a function");
                    ge(Ee, we, function(Qe, ke) {
                      xe(ke);
                    }, Le);
                  });
                }), Ce;
              }(), G.exports ? G.exports = ee[ne] : (te = (function() {
                return ee[ne];
              }).call(J, Z, J, G)) === void 0 || (G.exports = te);
            })("Promise", Z.g !== void 0 ? Z.g : this);
          }, 60265: function(G) {
            var J = Math.PI, Z = oe(120);
            function te(se, ce, le, ue) {
              return ["C", se, ce, le, ue, le, ue];
            }
            function ne(se, ce, le, ue, de, fe) {
              return ["C", se / 3 + 0.6666666666666666 * le, ce / 3 + 0.6666666666666666 * ue, de / 3 + 0.6666666666666666 * le, fe / 3 + 0.6666666666666666 * ue, de, fe];
            }
            function ee(se, ce, le, ue, de, fe, he, me, pe, ye) {
              if (ye) Qe = ye[0], ke = ye[1], xe = ye[2], Le = ye[3];
              else {
                var ge = ae(se, ce, -de);
                se = ge.x, ce = ge.y;
                var ve = (se - (me = (ge = ae(me, pe, -de)).x)) / 2, be = (ce - (pe = ge.y)) / 2, Ce = ve * ve / (le * le) + be * be / (ue * ue);
                Ce > 1 && (le *= Ce = Math.sqrt(Ce), ue *= Ce);
                var Se = le * le, we = ue * ue, Ee = (fe == he ? -1 : 1) * Math.sqrt(Math.abs((Se * we - Se * be * be - we * ve * ve) / (Se * be * be + we * ve * ve)));
                Ee == 1 / 0 && (Ee = 1);
                var xe = Ee * le * be / ue + (se + me) / 2, Le = Ee * -ue * ve / le + (ce + pe) / 2, Qe = Math.asin(((ce - Le) / ue).toFixed(9)), ke = Math.asin(((pe - Le) / ue).toFixed(9));
                (Qe = se < xe ? J - Qe : Qe) < 0 && (Qe = 2 * J + Qe), (ke = me < xe ? J - ke : ke) < 0 && (ke = 2 * J + ke), he && Qe > ke && (Qe -= 2 * J), !he && ke > Qe && (ke -= 2 * J);
              }
              if (Math.abs(ke - Qe) > Z) {
                var Me = ke, Re = me, De = pe;
                ke = Qe + Z * (he && ke > Qe ? 1 : -1);
                var Pe = ee(me = xe + le * Math.cos(ke), pe = Le + ue * Math.sin(ke), le, ue, de, 0, he, Re, De, [ke, Me, xe, Le]);
              }
              var $e = Math.tan((ke - Qe) / 4), qe = 4 / 3 * le * $e, Xe = 4 / 3 * ue * $e, Ve = [2 * se - (se + qe * Math.sin(Qe)), 2 * ce - (ce - Xe * Math.cos(Qe)), me + qe * Math.sin(ke), pe - Xe * Math.cos(ke), me, pe];
              if (ye) return Ve;
              Pe && (Ve = Ve.concat(Pe));
              for (var Ze = 0; Ze < Ve.length; ) {
                var Ge = ae(Ve[Ze], Ve[Ze + 1], de);
                Ve[Ze++] = Ge.x, Ve[Ze++] = Ge.y;
              }
              return Ve;
            }
            function ae(se, ce, le) {
              return { x: se * Math.cos(le) - ce * Math.sin(le), y: se * Math.sin(le) + ce * Math.cos(le) };
            }
            function oe(se) {
              return se * (J / 180);
            }
            G.exports = function(se) {
              for (var ce, le = [], ue = 0, de = 0, fe = 0, he = 0, me = null, pe = null, ye = 0, ge = 0, ve = 0, be = se.length; ve < be; ve++) {
                var Ce = se[ve], Se = Ce[0];
                switch (Se) {
                  case "M":
                    fe = Ce[1], he = Ce[2];
                    break;
                  case "A":
                    (Ce = ee(ye, ge, Ce[1], Ce[2], oe(Ce[3]), Ce[4], Ce[5], Ce[6], Ce[7])).unshift("C"), Ce.length > 7 && (le.push(Ce.splice(0, 7)), Ce.unshift("C"));
                    break;
                  case "S":
                    var we = ye, Ee = ge;
                    ce != "C" && ce != "S" || (we += we - ue, Ee += Ee - de), Ce = ["C", we, Ee, Ce[1], Ce[2], Ce[3], Ce[4]];
                    break;
                  case "T":
                    ce == "Q" || ce == "T" ? (me = 2 * ye - me, pe = 2 * ge - pe) : (me = ye, pe = ge), Ce = ne(ye, ge, me, pe, Ce[1], Ce[2]);
                    break;
                  case "Q":
                    me = Ce[1], pe = Ce[2], Ce = ne(ye, ge, Ce[1], Ce[2], Ce[3], Ce[4]);
                    break;
                  case "L":
                    Ce = te(ye, ge, Ce[1], Ce[2]);
                    break;
                  case "H":
                    Ce = te(ye, ge, Ce[1], ge);
                    break;
                  case "V":
                    Ce = te(ye, ge, ye, Ce[1]);
                    break;
                  case "Z":
                    Ce = te(ye, ge, fe, he);
                }
                ce = Se, ye = Ce[Ce.length - 2], ge = Ce[Ce.length - 1], Ce.length > 4 ? (ue = Ce[Ce.length - 4], de = Ce[Ce.length - 3]) : (ue = ye, de = ge), le.push(Ce);
              }
              return le;
            };
          }, 27976: function(G) {
            var J = Object.getOwnPropertySymbols, Z = Object.prototype.hasOwnProperty, te = Object.prototype.propertyIsEnumerable;
            G.exports = function() {
              try {
                if (!Object.assign) return !1;
                var ne = new String("abc");
                if (ne[5] = "de", Object.getOwnPropertyNames(ne)[0] === "5") return !1;
                for (var ee = {}, ae = 0; ae < 10; ae++) ee["_" + String.fromCharCode(ae)] = ae;
                if (Object.getOwnPropertyNames(ee).map(function(se) {
                  return ee[se];
                }).join("") !== "0123456789") return !1;
                var oe = {};
                return "abcdefghijklmnopqrst".split("").forEach(function(se) {
                  oe[se] = se;
                }), Object.keys(Object.assign({}, oe)).join("") === "abcdefghijklmnopqrst";
              } catch {
                return !1;
              }
            }() ? Object.assign : function(ne, ee) {
              for (var ae, oe, se = function(de) {
                if (de == null) throw new TypeError("Object.assign cannot be called with null or undefined");
                return Object(de);
              }(ne), ce = 1; ce < arguments.length; ce++) {
                for (var le in ae = Object(arguments[ce])) Z.call(ae, le) && (se[le] = ae[le]);
                if (J) {
                  oe = J(ae);
                  for (var ue = 0; ue < oe.length; ue++) te.call(ae, oe[ue]) && (se[oe[ue]] = ae[oe[ue]]);
                }
              }
              return se;
            };
          }, 93063: function(G) {
            var J = function(Z) {
              return Z != Z;
            };
            G.exports = function(Z, te) {
              return Z === 0 && te === 0 ? 1 / Z == 1 / te : Z === te || !(!J(Z) || !J(te));
            };
          }, 13969: function(G, J, Z) {
            var te = Z(97936), ne = Z(87227), ee = Z(93063), ae = Z(9622), oe = Z(79796), se = ne(ae(), Object);
            te(se, { getPolyfill: ae, implementation: ee, shim: oe }), G.exports = se;
          }, 9622: function(G, J, Z) {
            var te = Z(93063);
            G.exports = function() {
              return typeof Object.is == "function" ? Object.is : te;
            };
          }, 79796: function(G, J, Z) {
            var te = Z(9622), ne = Z(97936);
            G.exports = function() {
              var ee = te();
              return ne(Object, { is: ee }, { is: function() {
                return Object.is !== ee;
              } }), ee;
            };
          }, 61663: function(G, J, Z) {
            var te;
            if (!Object.keys) {
              var ne = Object.prototype.hasOwnProperty, ee = Object.prototype.toString, ae = Z(52385), oe = Object.prototype.propertyIsEnumerable, se = !oe.call({ toString: null }, "toString"), ce = oe.call(function() {
              }, "prototype"), le = ["toString", "toLocaleString", "valueOf", "hasOwnProperty", "isPrototypeOf", "propertyIsEnumerable", "constructor"], ue = function(he) {
                var me = he.constructor;
                return me && me.prototype === he;
              }, de = { $applicationCache: !0, $console: !0, $external: !0, $frame: !0, $frameElement: !0, $frames: !0, $innerHeight: !0, $innerWidth: !0, $onmozfullscreenchange: !0, $onmozfullscreenerror: !0, $outerHeight: !0, $outerWidth: !0, $pageXOffset: !0, $pageYOffset: !0, $parent: !0, $scrollLeft: !0, $scrollTop: !0, $scrollX: !0, $scrollY: !0, $self: !0, $webkitIndexedDB: !0, $webkitStorageInfo: !0, $window: !0 }, fe = function() {
                if (typeof window > "u") return !1;
                for (var he in window) try {
                  if (!de["$" + he] && ne.call(window, he) && window[he] !== null && typeof window[he] == "object") try {
                    ue(window[he]);
                  } catch {
                    return !0;
                  }
                } catch {
                  return !0;
                }
                return !1;
              }();
              te = function(he) {
                var me = he !== null && typeof he == "object", pe = ee.call(he) === "[object Function]", ye = ae(he), ge = me && ee.call(he) === "[object String]", ve = [];
                if (!me && !pe && !ye) throw new TypeError("Object.keys called on a non-object");
                var be = ce && pe;
                if (ge && he.length > 0 && !ne.call(he, 0)) for (var Ce = 0; Ce < he.length; ++Ce) ve.push(String(Ce));
                if (ye && he.length > 0) for (var Se = 0; Se < he.length; ++Se) ve.push(String(Se));
                else for (var we in he) be && we === "prototype" || !ne.call(he, we) || ve.push(String(we));
                if (se) for (var Ee = function(Le) {
                  if (typeof window > "u" || !fe) return ue(Le);
                  try {
                    return ue(Le);
                  } catch {
                    return !1;
                  }
                }(he), xe = 0; xe < le.length; ++xe) Ee && le[xe] === "constructor" || !ne.call(he, le[xe]) || ve.push(le[xe]);
                return ve;
              };
            }
            G.exports = te;
          }, 99433: function(G, J, Z) {
            var te = Array.prototype.slice, ne = Z(52385), ee = Object.keys, ae = ee ? function(se) {
              return ee(se);
            } : Z(61663), oe = Object.keys;
            ae.shim = function() {
              if (Object.keys) {
                var se = function() {
                  var ce = Object.keys(arguments);
                  return ce && ce.length === arguments.length;
                }(1, 2);
                se || (Object.keys = function(ce) {
                  return ne(ce) ? oe(te.call(ce)) : oe(ce);
                });
              } else Object.keys = ae;
              return Object.keys || ae;
            }, G.exports = ae;
          }, 52385: function(G) {
            var J = Object.prototype.toString;
            G.exports = function(Z) {
              var te = J.call(Z), ne = te === "[object Arguments]";
              return ne || (ne = te !== "[object Array]" && Z !== null && typeof Z == "object" && typeof Z.length == "number" && Z.length >= 0 && J.call(Z.callee) === "[object Function]"), ne;
            };
          }, 96927: function(G, J, Z) {
            var te = Z(99433), ne = Z(59457)(), ee = Z(63063), ae = Object, oe = ee("Array.prototype.push"), se = ee("Object.prototype.propertyIsEnumerable"), ce = ne ? Object.getOwnPropertySymbols : null;
            G.exports = function(le, ue) {
              if (le == null) throw new TypeError("target must be an object");
              var de = ae(le);
              if (arguments.length === 1) return de;
              for (var fe = 1; fe < arguments.length; ++fe) {
                var he = ae(arguments[fe]), me = te(he), pe = ne && (Object.getOwnPropertySymbols || ce);
                if (pe) for (var ye = pe(he), ge = 0; ge < ye.length; ++ge) {
                  var ve = ye[ge];
                  se(he, ve) && oe(me, ve);
                }
                for (var be = 0; be < me.length; ++be) {
                  var Ce = me[be];
                  if (se(he, Ce)) {
                    var Se = he[Ce];
                    de[Ce] = Se;
                  }
                }
              }
              return de;
            };
          }, 68686: function(G, J, Z) {
            var te = Z(96927);
            G.exports = function() {
              return Object.assign ? function() {
                if (!Object.assign) return !1;
                for (var ne = "abcdefghijklmnopqrst", ee = ne.split(""), ae = {}, oe = 0; oe < ee.length; ++oe) ae[ee[oe]] = ee[oe];
                var se = Object.assign({}, ae), ce = "";
                for (var le in se) ce += le;
                return ne !== ce;
              }() || function() {
                if (!Object.assign || !Object.preventExtensions) return !1;
                var ne = Object.preventExtensions({ 1: 2 });
                try {
                  Object.assign(ne, "xy");
                } catch {
                  return ne[1] === "y";
                }
                return !1;
              }() ? te : Object.assign : te;
            };
          }, 59811: function(G) {
            function J(ne, ee) {
              if (typeof ne != "string") return [ne];
              var ae = [ne];
              typeof ee == "string" || Array.isArray(ee) ? ee = { brackets: ee } : ee || (ee = {});
              var oe = ee.brackets ? Array.isArray(ee.brackets) ? ee.brackets : [ee.brackets] : ["{}", "[]", "()"], se = ee.escape || "___", ce = !!ee.flat;
              oe.forEach(function(ue) {
                var de = new RegExp(["\\", ue[0], "[^\\", ue[0], "\\", ue[1], "]*\\", ue[1]].join("")), fe = [];
                function he(me, pe, ye) {
                  var ge = ae.push(me.slice(ue[0].length, -ue[1].length)) - 1;
                  return fe.push(ge), se + ge + se;
                }
                ae.forEach(function(me, pe) {
                  for (var ye, ge = 0; me != ye; ) if (ye = me, me = me.replace(de, he), ge++ > 1e4) throw Error("References have circular dependency. Please, check them.");
                  ae[pe] = me;
                }), fe = fe.reverse(), ae = ae.map(function(me) {
                  return fe.forEach(function(pe) {
                    me = me.replace(new RegExp("(\\" + se + pe + "\\" + se + ")", "g"), ue[0] + "$1" + ue[1]);
                  }), me;
                });
              });
              var le = new RegExp("\\" + se + "([0-9]+)\\" + se);
              return ce ? ae : function ue(de, fe, he) {
                for (var me, pe = [], ye = 0; me = le.exec(de); ) {
                  if (ye++ > 1e4) throw Error("Circular references in parenthesis");
                  pe.push(de.slice(0, me.index)), pe.push(ue(fe[me[1]], fe)), de = de.slice(me.index + me[0].length);
                }
                return pe.push(de), pe;
              }(ae[0], ae);
            }
            function Z(ne, ee) {
              if (ee && ee.flat) {
                var ae, oe = ee && ee.escape || "___", se = ne[0];
                if (!se) return "";
                for (var ce = new RegExp("\\" + oe + "([0-9]+)\\" + oe), le = 0; se != ae; ) {
                  if (le++ > 1e4) throw Error("Circular references in " + ne);
                  ae = se, se = se.replace(ce, ue);
                }
                return se;
              }
              return ne.reduce(function de(fe, he) {
                return Array.isArray(he) && (he = he.reduce(de, "")), fe + he;
              }, "");
              function ue(de, fe) {
                if (ne[fe] == null) throw Error("Reference " + fe + "is undefined");
                return ne[fe];
              }
            }
            function te(ne, ee) {
              return Array.isArray(ne) ? Z(ne, ee) : J(ne, ee);
            }
            te.parse = J, te.stringify = Z, G.exports = te;
          }, 5137: function(G, J, Z) {
            var te = Z(6807);
            G.exports = function(ne) {
              var ee;
              return arguments.length > 1 && (ne = arguments), typeof ne == "string" ? ne = ne.split(/\s/).map(parseFloat) : typeof ne == "number" && (ne = [ne]), ne.length && typeof ne[0] == "number" ? ee = ne.length === 1 ? { width: ne[0], height: ne[0], x: 0, y: 0 } : ne.length === 2 ? { width: ne[0], height: ne[1], x: 0, y: 0 } : { x: ne[0], y: ne[1], width: ne[2] - ne[0] || 0, height: ne[3] - ne[1] || 0 } : ne && (ee = { x: (ne = te(ne, { left: "x l left Left", top: "y t top Top", width: "w width W Width", height: "h height W Width", bottom: "b bottom Bottom", right: "r right Right" })).left || 0, y: ne.top || 0 }, ne.width == null ? ne.right ? ee.width = ne.right - ee.x : ee.width = 0 : ee.width = ne.width, ne.height == null ? ne.bottom ? ee.height = ne.bottom - ee.y : ee.height = 0 : ee.height = ne.height), ee;
            };
          }, 26953: function(G) {
            G.exports = function(ne) {
              var ee = [];
              return ne.replace(Z, function(ae, oe, se) {
                var ce = oe.toLowerCase();
                for (se = function(le) {
                  var ue = le.match(te);
                  return ue ? ue.map(Number) : [];
                }(se), ce == "m" && se.length > 2 && (ee.push([oe].concat(se.splice(0, 2))), ce = "l", oe = oe == "m" ? "l" : "L"); ; ) {
                  if (se.length == J[ce]) return se.unshift(oe), ee.push(se);
                  if (se.length < J[ce]) throw new Error("malformed path data");
                  ee.push([oe].concat(se.splice(0, J[ce])));
                }
              }), ee;
            };
            var J = { a: 7, c: 6, h: 1, l: 2, m: 2, q: 4, s: 4, t: 2, v: 1, z: 0 }, Z = /([astvzqmhlc])([^astvzqmhlc]*)/gi, te = /-?[0-9]*\.?[0-9]+(?:e[-+]?\d+)?/gi;
          }, 4957: function(G) {
            G.exports = function(J, Z) {
              Z || (Z = [0, ""]), J = String(J);
              var te = parseFloat(J, 10);
              return Z[0] = te, Z[1] = J.match(/[\d.\-\+]*\s*(.*)/)[1] || "", Z;
            };
          }, 71879: function(G, J, Z) {
            var te = Z(33282);
            (function() {
              var ne, ee, ae, oe, se, ce;
              typeof performance < "u" && performance !== null && performance.now ? G.exports = function() {
                return performance.now();
              } : te != null && te.hrtime ? (G.exports = function() {
                return (ne() - se) / 1e6;
              }, ee = te.hrtime, oe = (ne = function() {
                var le;
                return 1e9 * (le = ee())[0] + le[1];
              })(), ce = 1e9 * te.uptime(), se = oe - ce) : Date.now ? (G.exports = function() {
                return Date.now() - ae;
              }, ae = Date.now()) : (G.exports = function() {
                return (/* @__PURE__ */ new Date()).getTime() - ae;
              }, ae = (/* @__PURE__ */ new Date()).getTime());
            }).call(this);
          }, 6807: function(G) {
            G.exports = function(te, ne, ee) {
              var ae, oe, se = {};
              if (typeof ne == "string" && (ne = Z(ne)), Array.isArray(ne)) {
                var ce = {};
                for (oe = 0; oe < ne.length; oe++) ce[ne[oe]] = !0;
                ne = ce;
              }
              for (ae in ne) ne[ae] = Z(ne[ae]);
              var le = {};
              for (ae in ne) {
                var ue = ne[ae];
                if (Array.isArray(ue)) for (oe = 0; oe < ue.length; oe++) {
                  var de = ue[oe];
                  if (ee && (le[de] = !0), de in te) {
                    if (se[ae] = te[de], ee) for (var fe = oe; fe < ue.length; fe++) le[ue[fe]] = !0;
                    break;
                  }
                }
                else ae in te && (ne[ae] && (se[ae] = te[ae]), ee && (le[ae] = !0));
              }
              if (ee) for (ae in te) le[ae] || (se[ae] = te[ae]);
              return se;
            };
            var J = {};
            function Z(te) {
              return J[te] ? J[te] : (typeof te == "string" && (te = J[te] = te.split(/\s*,\s*|\s+/)), te);
            }
          }, 52773: function(G) {
            G.exports = function(J, Z, te, ne) {
              var ee = J[0], ae = J[1], oe = !1;
              te === void 0 && (te = 0), ne === void 0 && (ne = Z.length);
              for (var se = ne - te, ce = 0, le = se - 1; ce < se; le = ce++) {
                var ue = Z[ce + te][0], de = Z[ce + te][1], fe = Z[le + te][0], he = Z[le + te][1];
                de > ae != he > ae && ee < (fe - ue) * (ae - de) / (he - de) + ue && (oe = !oe);
              }
              return oe;
            };
          }, 11516: function(G, J, Z) {
            var te, ne = Z(42391), ee = Z(92990), ae = Z(26202), oe = Z(22222), se = Z(17527), ce = Z(24491), le = !1, ue = ee();
            function de(fe, he, me) {
              var pe = te.segments(fe), ye = te.segments(he), ge = me(te.combine(pe, ye));
              return te.polygon(ge);
            }
            te = { buildLog: function(fe) {
              return fe === !0 ? le = ne() : fe === !1 && (le = !1), le !== !1 && le.list;
            }, epsilon: function(fe) {
              return ue.epsilon(fe);
            }, segments: function(fe) {
              var he = ae(!0, ue, le);
              return fe.regions.forEach(he.addRegion), { segments: he.calculate(fe.inverted), inverted: fe.inverted };
            }, combine: function(fe, he) {
              return { combined: ae(!1, ue, le).calculate(fe.segments, fe.inverted, he.segments, he.inverted), inverted1: fe.inverted, inverted2: he.inverted };
            }, selectUnion: function(fe) {
              return { segments: se.union(fe.combined, le), inverted: fe.inverted1 || fe.inverted2 };
            }, selectIntersect: function(fe) {
              return { segments: se.intersect(fe.combined, le), inverted: fe.inverted1 && fe.inverted2 };
            }, selectDifference: function(fe) {
              return { segments: se.difference(fe.combined, le), inverted: fe.inverted1 && !fe.inverted2 };
            }, selectDifferenceRev: function(fe) {
              return { segments: se.differenceRev(fe.combined, le), inverted: !fe.inverted1 && fe.inverted2 };
            }, selectXor: function(fe) {
              return { segments: se.xor(fe.combined, le), inverted: fe.inverted1 !== fe.inverted2 };
            }, polygon: function(fe) {
              return { regions: oe(fe.segments, ue, le), inverted: fe.inverted };
            }, polygonFromGeoJSON: function(fe) {
              return ce.toPolygon(te, fe);
            }, polygonToGeoJSON: function(fe) {
              return ce.fromPolygon(te, ue, fe);
            }, union: function(fe, he) {
              return de(fe, he, te.selectUnion);
            }, intersect: function(fe, he) {
              return de(fe, he, te.selectIntersect);
            }, difference: function(fe, he) {
              return de(fe, he, te.selectDifference);
            }, differenceRev: function(fe, he) {
              return de(fe, he, te.selectDifferenceRev);
            }, xor: function(fe, he) {
              return de(fe, he, te.selectXor);
            } }, typeof window == "object" && (window.PolyBool = te), G.exports = te;
          }, 42391: function(G) {
            G.exports = function() {
              var J, Z = 0, te = !1;
              function ne(ee, ae) {
                return J.list.push({ type: ee, data: ae ? JSON.parse(JSON.stringify(ae)) : void 0 }), J;
              }
              return J = { list: [], segmentId: function() {
                return Z++;
              }, checkIntersection: function(ee, ae) {
                return ne("check", { seg1: ee, seg2: ae });
              }, segmentChop: function(ee, ae) {
                return ne("div_seg", { seg: ee, pt: ae }), ne("chop", { seg: ee, pt: ae });
              }, statusRemove: function(ee) {
                return ne("pop_seg", { seg: ee });
              }, segmentUpdate: function(ee) {
                return ne("seg_update", { seg: ee });
              }, segmentNew: function(ee, ae) {
                return ne("new_seg", { seg: ee, primary: ae });
              }, segmentRemove: function(ee) {
                return ne("rem_seg", { seg: ee });
              }, tempStatus: function(ee, ae, oe) {
                return ne("temp_status", { seg: ee, above: ae, below: oe });
              }, rewind: function(ee) {
                return ne("rewind", { seg: ee });
              }, status: function(ee, ae, oe) {
                return ne("status", { seg: ee, above: ae, below: oe });
              }, vert: function(ee) {
                return ee === te ? J : (te = ee, ne("vert", { x: ee }));
              }, log: function(ee) {
                return typeof ee != "string" && (ee = JSON.stringify(ee, !1, "  ")), ne("log", { txt: ee });
              }, reset: function() {
                return ne("reset");
              }, selected: function(ee) {
                return ne("selected", { segs: ee });
              }, chainStart: function(ee) {
                return ne("chain_start", { seg: ee });
              }, chainRemoveHead: function(ee, ae) {
                return ne("chain_rem_head", { index: ee, pt: ae });
              }, chainRemoveTail: function(ee, ae) {
                return ne("chain_rem_tail", { index: ee, pt: ae });
              }, chainNew: function(ee, ae) {
                return ne("chain_new", { pt1: ee, pt2: ae });
              }, chainMatch: function(ee) {
                return ne("chain_match", { index: ee });
              }, chainClose: function(ee) {
                return ne("chain_close", { index: ee });
              }, chainAddHead: function(ee, ae) {
                return ne("chain_add_head", { index: ee, pt: ae });
              }, chainAddTail: function(ee, ae) {
                return ne("chain_add_tail", { index: ee, pt: ae });
              }, chainConnect: function(ee, ae) {
                return ne("chain_con", { index1: ee, index2: ae });
              }, chainReverse: function(ee) {
                return ne("chain_rev", { index: ee });
              }, chainJoin: function(ee, ae) {
                return ne("chain_join", { index1: ee, index2: ae });
              }, done: function() {
                return ne("done");
              } };
            };
          }, 92990: function(G) {
            G.exports = function(J) {
              typeof J != "number" && (J = 1e-10);
              var Z = { epsilon: function(te) {
                return typeof te == "number" && (J = te), J;
              }, pointAboveOrOnLine: function(te, ne, ee) {
                var ae = ne[0], oe = ne[1], se = ee[0], ce = ee[1], le = te[0];
                return (se - ae) * (te[1] - oe) - (ce - oe) * (le - ae) >= -J;
              }, pointBetween: function(te, ne, ee) {
                var ae = te[1] - ne[1], oe = ee[0] - ne[0], se = te[0] - ne[0], ce = ee[1] - ne[1], le = se * oe + ae * ce;
                return !(le < J || le - (oe * oe + ce * ce) > -J);
              }, pointsSameX: function(te, ne) {
                return Math.abs(te[0] - ne[0]) < J;
              }, pointsSameY: function(te, ne) {
                return Math.abs(te[1] - ne[1]) < J;
              }, pointsSame: function(te, ne) {
                return Z.pointsSameX(te, ne) && Z.pointsSameY(te, ne);
              }, pointsCompare: function(te, ne) {
                return Z.pointsSameX(te, ne) ? Z.pointsSameY(te, ne) ? 0 : te[1] < ne[1] ? -1 : 1 : te[0] < ne[0] ? -1 : 1;
              }, pointsCollinear: function(te, ne, ee) {
                var ae = te[0] - ne[0], oe = te[1] - ne[1], se = ne[0] - ee[0], ce = ne[1] - ee[1];
                return Math.abs(ae * ce - se * oe) < J;
              }, linesIntersect: function(te, ne, ee, ae) {
                var oe = ne[0] - te[0], se = ne[1] - te[1], ce = ae[0] - ee[0], le = ae[1] - ee[1], ue = oe * le - se * ce;
                if (Math.abs(ue) < J) return !1;
                var de = te[0] - ee[0], fe = te[1] - ee[1], he = (ce * fe - le * de) / ue, me = (oe * fe - se * de) / ue, pe = { alongA: 0, alongB: 0, pt: [te[0] + he * oe, te[1] + he * se] };
                return pe.alongA = he <= -J ? -2 : he < J ? -1 : he - 1 <= -J ? 0 : he - 1 < J ? 1 : 2, pe.alongB = me <= -J ? -2 : me < J ? -1 : me - 1 <= -J ? 0 : me - 1 < J ? 1 : 2, pe;
              }, pointInsideRegion: function(te, ne) {
                for (var ee = te[0], ae = te[1], oe = ne[ne.length - 1][0], se = ne[ne.length - 1][1], ce = !1, le = 0; le < ne.length; le++) {
                  var ue = ne[le][0], de = ne[le][1];
                  de - ae > J != se - ae > J && (oe - ue) * (ae - de) / (se - de) + ue - ee > J && (ce = !ce), oe = ue, se = de;
                }
                return ce;
              } };
              return Z;
            };
          }, 24491: function(G) {
            var J = { toPolygon: function(Z, te) {
              function ne(oe) {
                if (oe.length <= 0) return Z.segments({ inverted: !1, regions: [] });
                function se(ue) {
                  var de = ue.slice(0, ue.length - 1);
                  return Z.segments({ inverted: !1, regions: [de] });
                }
                for (var ce = se(oe[0]), le = 1; le < oe.length; le++) ce = Z.selectDifference(Z.combine(ce, se(oe[le])));
                return ce;
              }
              if (te.type === "Polygon") return Z.polygon(ne(te.coordinates));
              if (te.type === "MultiPolygon") {
                for (var ee = Z.segments({ inverted: !1, regions: [] }), ae = 0; ae < te.coordinates.length; ae++) ee = Z.selectUnion(Z.combine(ee, ne(te.coordinates[ae])));
                return Z.polygon(ee);
              }
              throw new Error("PolyBool: Cannot convert GeoJSON object to PolyBool polygon");
            }, fromPolygon: function(Z, te, ne) {
              function ee(me, pe) {
                return te.pointInsideRegion([0.5 * (me[0][0] + me[1][0]), 0.5 * (me[0][1] + me[1][1])], pe);
              }
              function ae(me) {
                return { region: me, children: [] };
              }
              ne = Z.polygon(Z.segments(ne));
              var oe = ae(null);
              function se(me, pe) {
                for (var ye = 0; ye < me.children.length; ye++) if (ee(pe, (ve = me.children[ye]).region)) return void se(ve, pe);
                var ge = ae(pe);
                for (ye = 0; ye < me.children.length; ye++) {
                  var ve;
                  ee((ve = me.children[ye]).region, pe) && (ge.children.push(ve), me.children.splice(ye, 1), ye--);
                }
                me.children.push(ge);
              }
              for (var ce = 0; ce < ne.regions.length; ce++) {
                var le = ne.regions[ce];
                le.length < 3 || se(oe, le);
              }
              function ue(me, pe) {
                for (var ye = 0, ge = me[me.length - 1][0], ve = me[me.length - 1][1], be = [], Ce = 0; Ce < me.length; Ce++) {
                  var Se = me[Ce][0], we = me[Ce][1];
                  be.push([Se, we]), ye += we * ge - Se * ve, ge = Se, ve = we;
                }
                return ye < 0 !== pe && be.reverse(), be.push([be[0][0], be[0][1]]), be;
              }
              var de = [];
              function fe(me) {
                var pe = [ue(me.region, !1)];
                de.push(pe);
                for (var ye = 0; ye < me.children.length; ye++) pe.push(he(me.children[ye]));
              }
              function he(me) {
                for (var pe = 0; pe < me.children.length; pe++) fe(me.children[pe]);
                return ue(me.region, !0);
              }
              for (ce = 0; ce < oe.children.length; ce++) fe(oe.children[ce]);
              return de.length <= 0 ? { type: "Polygon", coordinates: [] } : de.length == 1 ? { type: "Polygon", coordinates: de[0] } : { type: "MultiPolygon", coordinates: de };
            } };
            G.exports = J;
          }, 26202: function(G, J, Z) {
            var te = Z(48916);
            G.exports = function(ne, ee, ae) {
              function oe(fe, he, me) {
                return { id: ae ? ae.segmentId() : -1, start: fe, end: he, myFill: { above: me.myFill.above, below: me.myFill.below }, otherFill: null };
              }
              var se = te.create();
              function ce(fe, he) {
                se.insertBefore(fe, function(me) {
                  return pe = fe.isStart, ye = fe.pt, ge = he, ve = me.isStart, be = me.pt, Ce = me.other.pt, ((Se = ee.pointsCompare(ye, be)) !== 0 ? Se : ee.pointsSame(ge, Ce) ? 0 : pe !== ve ? pe ? 1 : -1 : ee.pointAboveOrOnLine(ge, ve ? be : Ce, ve ? Ce : be) ? 1 : -1) < 0;
                  var pe, ye, ge, ve, be, Ce, Se;
                });
              }
              function le(fe, he) {
                var me = function(pe, ye) {
                  var ge = te.node({ isStart: !0, pt: pe.start, seg: pe, primary: ye, other: null, status: null });
                  return ce(ge, pe.end), ge;
                }(fe, he);
                return function(pe, ye, ge) {
                  var ve = te.node({ isStart: !1, pt: ye.end, seg: ye, primary: ge, other: pe, status: null });
                  pe.other = ve, ce(ve, pe.pt);
                }(me, fe, he), me;
              }
              function ue(fe, he) {
                var me = oe(he, fe.seg.end, fe.seg);
                return function(pe, ye) {
                  ae && ae.segmentChop(pe.seg, ye), pe.other.remove(), pe.seg.end = ye, pe.other.pt = ye, ce(pe.other, pe.pt);
                }(fe, he), le(me, fe.primary);
              }
              function de(fe, he) {
                var me = te.create();
                function pe(Me) {
                  return me.findTransition(function(Re) {
                    var De, Pe, $e, qe, Xe, Ve;
                    return De = Me, Pe = Re.ev, $e = De.seg.start, qe = De.seg.end, Xe = Pe.seg.start, Ve = Pe.seg.end, (ee.pointsCollinear($e, Xe, Ve) ? ee.pointsCollinear(qe, Xe, Ve) || ee.pointAboveOrOnLine(qe, Xe, Ve) ? 1 : -1 : ee.pointAboveOrOnLine($e, Xe, Ve) ? 1 : -1) > 0;
                  });
                }
                function ye(Me, Re) {
                  var De = Me.seg, Pe = Re.seg, $e = De.start, qe = De.end, Xe = Pe.start, Ve = Pe.end;
                  ae && ae.checkIntersection(De, Pe);
                  var Ze = ee.linesIntersect($e, qe, Xe, Ve);
                  if (Ze === !1) {
                    if (!ee.pointsCollinear($e, qe, Xe) || ee.pointsSame($e, Ve) || ee.pointsSame(qe, Xe)) return !1;
                    var Ge = ee.pointsSame($e, Xe), ze = ee.pointsSame(qe, Ve);
                    if (Ge && ze) return Re;
                    var We = !Ge && ee.pointBetween($e, Xe, Ve), at = !ze && ee.pointBetween(qe, Xe, Ve);
                    if (Ge) return at ? ue(Re, qe) : ue(Me, Ve), Re;
                    We && (ze || (at ? ue(Re, qe) : ue(Me, Ve)), ue(Re, $e));
                  } else Ze.alongA === 0 && (Ze.alongB === -1 ? ue(Me, Xe) : Ze.alongB === 0 ? ue(Me, Ze.pt) : Ze.alongB === 1 && ue(Me, Ve)), Ze.alongB === 0 && (Ze.alongA === -1 ? ue(Re, $e) : Ze.alongA === 0 ? ue(Re, Ze.pt) : Ze.alongA === 1 && ue(Re, qe));
                  return !1;
                }
                for (var ge = []; !se.isEmpty(); ) {
                  var ve = se.getHead();
                  if (ae && ae.vert(ve.pt[0]), ve.isStart) {
                    let Me = function() {
                      if (Ce) {
                        var Re = ye(ve, Ce);
                        if (Re) return Re;
                      }
                      return !!Se && ye(ve, Se);
                    };
                    var ke = Me;
                    ae && ae.segmentNew(ve.seg, ve.primary);
                    var be = pe(ve), Ce = be.before ? be.before.ev : null, Se = be.after ? be.after.ev : null;
                    ae && ae.tempStatus(ve.seg, !!Ce && Ce.seg, !!Se && Se.seg);
                    var we, Ee, xe = Me();
                    if (xe && (ne ? (Ee = ve.seg.myFill.below === null || ve.seg.myFill.above !== ve.seg.myFill.below) && (xe.seg.myFill.above = !xe.seg.myFill.above) : xe.seg.otherFill = ve.seg.myFill, ae && ae.segmentUpdate(xe.seg), ve.other.remove(), ve.remove()), se.getHead() !== ve) {
                      ae && ae.rewind(ve.seg);
                      continue;
                    }
                    ne ? (Ee = ve.seg.myFill.below === null || ve.seg.myFill.above !== ve.seg.myFill.below, ve.seg.myFill.below = Se ? Se.seg.myFill.above : fe, ve.seg.myFill.above = Ee ? !ve.seg.myFill.below : ve.seg.myFill.below) : ve.seg.otherFill === null && (we = Se ? ve.primary === Se.primary ? Se.seg.otherFill.above : Se.seg.myFill.above : ve.primary ? he : fe, ve.seg.otherFill = { above: we, below: we }), ae && ae.status(ve.seg, !!Ce && Ce.seg, !!Se && Se.seg), ve.other.status = be.insert(te.node({ ev: ve }));
                  } else {
                    var Le = ve.status;
                    if (Le === null) throw new Error("PolyBool: Zero-length segment detected; your epsilon is probably too small or too large");
                    if (me.exists(Le.prev) && me.exists(Le.next) && ye(Le.prev.ev, Le.next.ev), ae && ae.statusRemove(Le.ev.seg), Le.remove(), !ve.primary) {
                      var Qe = ve.seg.myFill;
                      ve.seg.myFill = ve.seg.otherFill, ve.seg.otherFill = Qe;
                    }
                    ge.push(ve.seg);
                  }
                  se.getHead().remove();
                }
                return ae && ae.done(), ge;
              }
              return ne ? { addRegion: function(fe) {
                for (var he, me, pe, ye = fe[fe.length - 1], ge = 0; ge < fe.length; ge++) {
                  he = ye, ye = fe[ge];
                  var ve = ee.pointsCompare(he, ye);
                  ve !== 0 && le((me = ve < 0 ? he : ye, pe = ve < 0 ? ye : he, { id: ae ? ae.segmentId() : -1, start: me, end: pe, myFill: { above: null, below: null }, otherFill: null }), !0);
                }
              }, calculate: function(fe) {
                return de(fe, !1);
              } } : { calculate: function(fe, he, me, pe) {
                return fe.forEach(function(ye) {
                  le(oe(ye.start, ye.end, ye), !0);
                }), me.forEach(function(ye) {
                  le(oe(ye.start, ye.end, ye), !1);
                }), de(he, pe);
              } };
            };
          }, 48916: function(G) {
            G.exports = { create: function() {
              var J = { root: { root: !0, next: null }, exists: function(Z) {
                return Z !== null && Z !== J.root;
              }, isEmpty: function() {
                return J.root.next === null;
              }, getHead: function() {
                return J.root.next;
              }, insertBefore: function(Z, te) {
                for (var ne = J.root, ee = J.root.next; ee !== null; ) {
                  if (te(ee)) return Z.prev = ee.prev, Z.next = ee, ee.prev.next = Z, void (ee.prev = Z);
                  ne = ee, ee = ee.next;
                }
                ne.next = Z, Z.prev = ne, Z.next = null;
              }, findTransition: function(Z) {
                for (var te = J.root, ne = J.root.next; ne !== null && !Z(ne); ) te = ne, ne = ne.next;
                return { before: te === J.root ? null : te, after: ne, insert: function(ee) {
                  return ee.prev = te, ee.next = ne, te.next = ee, ne !== null && (ne.prev = ee), ee;
                } };
              } };
              return J;
            }, node: function(J) {
              return J.prev = null, J.next = null, J.remove = function() {
                J.prev.next = J.next, J.next && (J.next.prev = J.prev), J.prev = null, J.next = null;
              }, J;
            } };
          }, 22222: function(G) {
            G.exports = function(J, Z, te) {
              var ne = [], ee = [];
              return J.forEach(function(ae) {
                var oe = ae.start, se = ae.end;
                if (Z.pointsSame(oe, se)) console.warn("PolyBool: Warning: Zero-length segment detected; your epsilon is probably too small or too large");
                else {
                  te && te.chainStart(ae);
                  for (var ce = { index: 0, matches_head: !1, matches_pt1: !1 }, le = { index: 0, matches_head: !1, matches_pt1: !1 }, ue = ce, de = 0; de < ne.length; de++) {
                    var fe = (ge = ne[de])[0], he = (ge[1], ge[ge.length - 1]);
                    if (ge[ge.length - 2], Z.pointsSame(fe, oe)) {
                      if (Le(de, !0, !0)) break;
                    } else if (Z.pointsSame(fe, se)) {
                      if (Le(de, !0, !1)) break;
                    } else if (Z.pointsSame(he, oe)) {
                      if (Le(de, !1, !0)) break;
                    } else if (Z.pointsSame(he, se) && Le(de, !1, !1)) break;
                  }
                  if (ue === ce) return ne.push([oe, se]), void (te && te.chainNew(oe, se));
                  if (ue === le) {
                    te && te.chainMatch(ce.index);
                    var me = ce.index, pe = ce.matches_pt1 ? se : oe, ye = ce.matches_head, ge = ne[me], ve = ye ? ge[0] : ge[ge.length - 1], be = ye ? ge[1] : ge[ge.length - 2], Ce = ye ? ge[ge.length - 1] : ge[0], Se = ye ? ge[ge.length - 2] : ge[1];
                    return Z.pointsCollinear(be, ve, pe) && (ye ? (te && te.chainRemoveHead(ce.index, pe), ge.shift()) : (te && te.chainRemoveTail(ce.index, pe), ge.pop()), ve = be), Z.pointsSame(Ce, pe) ? (ne.splice(me, 1), Z.pointsCollinear(Se, Ce, ve) && (ye ? (te && te.chainRemoveTail(ce.index, ve), ge.pop()) : (te && te.chainRemoveHead(ce.index, ve), ge.shift())), te && te.chainClose(ce.index), void ee.push(ge)) : void (ye ? (te && te.chainAddHead(ce.index, pe), ge.unshift(pe)) : (te && te.chainAddTail(ce.index, pe), ge.push(pe)));
                  }
                  var we = ce.index, Ee = le.index;
                  te && te.chainConnect(we, Ee);
                  var xe = ne[we].length < ne[Ee].length;
                  ce.matches_head ? le.matches_head ? xe ? (Qe(we), ke(we, Ee)) : (Qe(Ee), ke(Ee, we)) : ke(Ee, we) : le.matches_head ? ke(we, Ee) : xe ? (Qe(we), ke(Ee, we)) : (Qe(Ee), ke(we, Ee));
                }
                function Le(Me, Re, De) {
                  return ue.index = Me, ue.matches_head = Re, ue.matches_pt1 = De, ue === ce ? (ue = le, !1) : (ue = null, !0);
                }
                function Qe(Me) {
                  te && te.chainReverse(Me), ne[Me].reverse();
                }
                function ke(Me, Re) {
                  var De = ne[Me], Pe = ne[Re], $e = De[De.length - 1], qe = De[De.length - 2], Xe = Pe[0], Ve = Pe[1];
                  Z.pointsCollinear(qe, $e, Xe) && (te && te.chainRemoveTail(Me, $e), De.pop(), $e = qe), Z.pointsCollinear($e, Xe, Ve) && (te && te.chainRemoveHead(Re, Xe), Pe.shift()), te && te.chainJoin(Me, Re), ne[Me] = De.concat(Pe), ne.splice(Re, 1);
                }
              }), ee;
            };
          }, 17527: function(G) {
            function J(te, ne, ee) {
              var ae = [];
              return te.forEach(function(oe) {
                var se = (oe.myFill.above ? 8 : 0) + (oe.myFill.below ? 4 : 0) + (oe.otherFill && oe.otherFill.above ? 2 : 0) + (oe.otherFill && oe.otherFill.below ? 1 : 0);
                ne[se] !== 0 && ae.push({ id: ee ? ee.segmentId() : -1, start: oe.start, end: oe.end, myFill: { above: ne[se] === 1, below: ne[se] === 2 }, otherFill: null });
              }), ee && ee.selected(ae), ae;
            }
            var Z = { union: function(te, ne) {
              return J(te, [0, 2, 1, 0, 2, 2, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0], ne);
            }, intersect: function(te, ne) {
              return J(te, [0, 0, 0, 0, 0, 2, 0, 2, 0, 0, 1, 1, 0, 2, 1, 0], ne);
            }, difference: function(te, ne) {
              return J(te, [0, 0, 0, 0, 2, 0, 2, 0, 1, 1, 0, 0, 0, 1, 2, 0], ne);
            }, differenceRev: function(te, ne) {
              return J(te, [0, 2, 1, 0, 0, 0, 1, 1, 0, 2, 0, 2, 0, 0, 0, 0], ne);
            }, xor: function(te, ne) {
              return J(te, [0, 2, 1, 0, 2, 0, 0, 1, 1, 0, 0, 2, 0, 1, 2, 0], ne);
            } };
            G.exports = Z;
          }, 3944: function(G, J, Z) {
            var te = Z(90386).Transform, ne = Z(79743);
            function ee() {
              te.call(this, { readableObjectMode: !0 });
            }
            function ae(oe, se, ce) {
              Error.call(this), Error.captureStackTrace ? Error.captureStackTrace(this, this.constructor) : this.stack = new Error().stack || "", this.name = this.constructor.name, this.message = oe, se && (this.code = se), ce && (this.statusCode = ce);
            }
            ee.prototype = Object.create(te.prototype), ee.prototype.constructor = ee, ne(ee.prototype), J.rU = function(oe, se, ce) {
              for (var le = se, ue = 0; ue < ce.length; ) if (oe[le++] !== ce[ue++]) return !1;
              return !0;
            }, J.VG = function(oe, se) {
              var ce = [], le = 0;
              if (se && se === "hex") for (; le < oe.length; ) ce.push(parseInt(oe.slice(le, le + 2), 16)), le += 2;
              else for (; le < oe.length; le++) ce.push(255 & oe.charCodeAt(le));
              return ce;
            }, J.$l = function(oe, se) {
              return oe[se] | oe[se + 1] << 8;
            }, J.bc = function(oe, se) {
              return oe[se + 1] | oe[se] << 8;
            }, J.tF = function(oe, se) {
              return oe[se] | oe[se + 1] << 8 | oe[se + 2] << 16 | 16777216 * oe[se + 3];
            }, J.bb = function(oe, se) {
              return oe[se + 3] | oe[se + 2] << 8 | oe[se + 1] << 16 | 16777216 * oe[se];
            }, ae.prototype = Object.create(Error.prototype), ae.prototype.constructor = ae;
          }, 19789: function(G) {
            function J(ne, ee) {
              var ae = new Error(ne);
              return ae.code = ee, ae;
            }
            function Z(ne) {
              try {
                return decodeURIComponent(escape(ne));
              } catch {
                return ne;
              }
            }
            function te(ne, ee, ae) {
              this.input = ne.subarray(ee, ae), this.start = ee;
              var oe = String.fromCharCode.apply(null, this.input.subarray(0, 4));
              if (oe !== "II*\0" && oe !== "MM\0*") throw J("invalid TIFF signature", "EBADDATA");
              this.big_endian = oe[0] === "M";
            }
            te.prototype.each = function(ne) {
              this.aborted = !1;
              var ee = this.read_uint32(4);
              for (this.ifds_to_read = [{ id: 0, offset: ee }]; this.ifds_to_read.length > 0 && !this.aborted; ) {
                var ae = this.ifds_to_read.shift();
                ae.offset && this.scan_ifd(ae.id, ae.offset, ne);
              }
            }, te.prototype.read_uint16 = function(ne) {
              var ee = this.input;
              if (ne + 2 > ee.length) throw J("unexpected EOF", "EBADDATA");
              return this.big_endian ? 256 * ee[ne] + ee[ne + 1] : ee[ne] + 256 * ee[ne + 1];
            }, te.prototype.read_uint32 = function(ne) {
              var ee = this.input;
              if (ne + 4 > ee.length) throw J("unexpected EOF", "EBADDATA");
              return this.big_endian ? 16777216 * ee[ne] + 65536 * ee[ne + 1] + 256 * ee[ne + 2] + ee[ne + 3] : ee[ne] + 256 * ee[ne + 1] + 65536 * ee[ne + 2] + 16777216 * ee[ne + 3];
            }, te.prototype.is_subifd_link = function(ne, ee) {
              return ne === 0 && ee === 34665 || ne === 0 && ee === 34853 || ne === 34665 && ee === 40965;
            }, te.prototype.exif_format_length = function(ne) {
              switch (ne) {
                case 1:
                case 2:
                case 6:
                case 7:
                  return 1;
                case 3:
                case 8:
                  return 2;
                case 4:
                case 9:
                case 11:
                  return 4;
                case 5:
                case 10:
                case 12:
                  return 8;
                default:
                  return 0;
              }
            }, te.prototype.exif_format_read = function(ne, ee) {
              var ae;
              switch (ne) {
                case 1:
                case 2:
                  return this.input[ee];
                case 6:
                  return (ae = this.input[ee]) | 33554430 * (128 & ae);
                case 3:
                  return this.read_uint16(ee);
                case 8:
                  return (ae = this.read_uint16(ee)) | 131070 * (32768 & ae);
                case 4:
                  return this.read_uint32(ee);
                case 9:
                  return 0 | this.read_uint32(ee);
                default:
                  return null;
              }
            }, te.prototype.scan_ifd = function(ne, ee, ae) {
              var oe = this.read_uint16(ee);
              ee += 2;
              for (var se = 0; se < oe; se++) {
                var ce = this.read_uint16(ee), le = this.read_uint16(ee + 2), ue = this.read_uint32(ee + 4), de = this.exif_format_length(le), fe = ue * de, he = fe <= 4 ? ee + 8 : this.read_uint32(ee + 8), me = !1;
                if (he + fe > this.input.length) throw J("unexpected EOF", "EBADDATA");
                for (var pe = [], ye = he, ge = 0; ge < ue; ge++, ye += de) {
                  var ve = this.exif_format_read(le, ye);
                  if (ve === null) {
                    pe = null;
                    break;
                  }
                  pe.push(ve);
                }
                if (Array.isArray(pe) && le === 2 && (pe = Z(String.fromCharCode.apply(null, pe))) && pe[pe.length - 1] === "\0" && (pe = pe.slice(0, -1)), this.is_subifd_link(ne, ce) && Array.isArray(pe) && Number.isInteger(pe[0]) && pe[0] > 0 && (this.ifds_to_read.push({ id: ce, offset: pe[0] }), me = !0), ae({ is_big_endian: this.big_endian, ifd: ne, tag: ce, format: le, count: ue, entry_offset: ee + this.start, data_length: fe, data_offset: he + this.start, value: pe, is_subifd_link: me }) === !1) return void (this.aborted = !0);
                ee += 12;
              }
              ne === 0 && this.ifds_to_read.push({ id: 1, offset: this.read_uint32(ee) });
            }, G.exports.ExifParser = te, G.exports.get_orientation = function(ne) {
              var ee = 0;
              try {
                return new te(ne, 0, ne.length).each(function(ae) {
                  if (ae.ifd === 0 && ae.tag === 274 && Array.isArray(ae.value)) return ee = ae.value[0], !1;
                }), ee;
              } catch {
                return -1;
              }
            };
          }, 20186: function(G, J, Z) {
            var te = Z(3944).bc, ne = Z(3944).bb;
            function ee(ue, de) {
              if (ue.length < 4 + de) return null;
              var fe = ne(ue, de);
              return ue.length < fe + de || fe < 8 ? null : { boxtype: String.fromCharCode.apply(null, ue.slice(de + 4, de + 8)), data: ue.slice(de + 8, de + fe), end: de + fe };
            }
            function ae(ue, de) {
              for (var fe = 0; ; ) {
                var he = ee(ue, fe);
                if (!he) break;
                switch (he.boxtype) {
                  case "ispe":
                    de.sizes.push({ width: ne(he.data, 4), height: ne(he.data, 8) });
                    break;
                  case "irot":
                    de.transforms.push({ type: "irot", value: 3 & he.data[0] });
                    break;
                  case "imir":
                    de.transforms.push({ type: "imir", value: 1 & he.data[0] });
                }
                fe = he.end;
              }
            }
            function oe(ue, de, fe) {
              for (var he = 0, me = 0; me < fe; me++) he = 256 * he + (ue[de + me] || 0);
              return he;
            }
            function se(ue, de) {
              for (var fe = ue[4] >> 4 & 15, he = 15 & ue[4], me = ue[5] >> 4 & 15, pe = te(ue, 6), ye = 8, ge = 0; ge < pe; ge++) {
                var ve = te(ue, ye), be = te(ue, ye += 2), Ce = oe(ue, ye += 2, me), Se = te(ue, ye += me);
                if (ye += 2, be === 0 && Se === 1) {
                  var we = oe(ue, ye, fe), Ee = oe(ue, ye + fe, he);
                  de.item_loc[ve] = { length: Ee, offset: we + Ce };
                }
                ye += Se * (fe + he);
              }
            }
            function ce(ue, de) {
              for (var fe = te(ue, 4), he = 6, me = 0; me < fe; me++) {
                var pe = ee(ue, he);
                if (!pe) break;
                if (pe.boxtype === "infe") {
                  for (var ye = te(pe.data, 4), ge = "", ve = 8; ve < pe.data.length && pe.data[ve]; ve++) ge += String.fromCharCode(pe.data[ve]);
                  de.item_inf[ge] = ye;
                }
                he = pe.end;
              }
            }
            function le(ue, de) {
              for (var fe = 0; ; ) {
                var he = ee(ue, fe);
                if (!he) break;
                he.boxtype === "ipco" && ae(he.data, de), fe = he.end;
              }
            }
            G.exports.unbox = ee, G.exports.readSizeFromMeta = function(ue) {
              var de = { sizes: [], transforms: [], item_inf: {}, item_loc: {} };
              if (function(ve, be) {
                for (var Ce = 4; ; ) {
                  var Se = ee(ve, Ce);
                  if (!Se) break;
                  Se.boxtype === "iprp" && le(Se.data, be), Se.boxtype === "iloc" && se(Se.data, be), Se.boxtype === "iinf" && ce(Se.data, be), Ce = Se.end;
                }
              }(ue, de), de.sizes.length) {
                var fe, he, me, pe = (he = (fe = de.sizes).reduce(function(ve, be) {
                  return ve.width > be.width || ve.width === be.width && ve.height > be.height ? ve : be;
                }), me = fe.reduce(function(ve, be) {
                  return ve.height > be.height || ve.height === be.height && ve.width > be.width ? ve : be;
                }), he.width > me.height || he.width === me.height && he.height > me.width ? he : me), ye = 1;
                de.transforms.forEach(function(ve) {
                  var be = { 1: 6, 2: 5, 3: 8, 4: 7, 5: 4, 6: 3, 7: 2, 8: 1 }, Ce = { 1: 4, 2: 3, 3: 2, 4: 1, 5: 6, 6: 5, 7: 8, 8: 7 };
                  if (ve.type === "imir" && (ye = ve.value === 0 ? Ce[ye] : be[ye = be[ye = Ce[ye]]]), ve.type === "irot") for (var Se = 0; Se < ve.value; Se++) ye = be[ye];
                });
                var ge = null;
                return de.item_inf.Exif && (ge = de.item_loc[de.item_inf.Exif]), { width: pe.width, height: pe.height, orientation: de.transforms.length ? ye : null, variants: de.sizes, exif_location: ge };
              }
            }, G.exports.getMimeType = function(ue) {
              var de = String.fromCharCode.apply(null, ue.slice(0, 4)), fe = {};
              fe[de] = !0;
              for (var he = 8; he < ue.length; he += 4) fe[String.fromCharCode.apply(null, ue.slice(he, he + 4))] = !0;
              if (fe.mif1 || fe.msf1 || fe.miaf) return de === "avif" || de === "avis" || de === "avio" ? { type: "avif", mime: "image/avif" } : de === "heic" || de === "heix" ? { type: "heic", mime: "image/heic" } : de === "hevc" || de === "hevx" ? { type: "heic", mime: "image/heic-sequence" } : fe.avif || fe.avis ? { type: "avif", mime: "image/avif" } : fe.heic || fe.heix || fe.hevc || fe.hevx || fe.heis ? fe.msf1 ? { type: "heif", mime: "image/heif-sequence" } : { type: "heif", mime: "image/heif" } : { type: "avif", mime: "image/avif" };
            };
          }, 31149: function(G, J, Z) {
            var te = Z(3944).VG, ne = Z(3944).rU, ee = Z(3944).bb, ae = Z(20186), oe = Z(19789), se = te("ftyp");
            G.exports = function(ce) {
              if (ne(ce, 4, se)) {
                var le = ae.unbox(ce, 0);
                if (le) {
                  var ue = ae.getMimeType(le.data);
                  if (ue) {
                    for (var de, fe = le.end; ; ) {
                      var he = ae.unbox(ce, fe);
                      if (!he) break;
                      if (fe = he.end, he.boxtype === "mdat") return;
                      if (he.boxtype === "meta") {
                        de = he.data;
                        break;
                      }
                    }
                    if (de) {
                      var me = ae.readSizeFromMeta(de);
                      if (me) {
                        var pe = { width: me.width, height: me.height, type: ue.type, mime: ue.mime, wUnits: "px", hUnits: "px" };
                        if (me.variants.length > 1 && (pe.variants = me.variants), me.orientation && (pe.orientation = me.orientation), me.exif_location && me.exif_location.offset + me.exif_location.length <= ce.length) {
                          var ye = ee(ce, me.exif_location.offset), ge = ce.slice(me.exif_location.offset + ye + 4, me.exif_location.offset + me.exif_location.length), ve = oe.get_orientation(ge);
                          ve > 0 && (pe.orientation = ve);
                        }
                        return pe;
                      }
                    }
                  }
                }
              }
            };
          }, 78218: function(G, J, Z) {
            var te = Z(3944).VG, ne = Z(3944).rU, ee = Z(3944).$l, ae = te("BM");
            G.exports = function(oe) {
              if (!(oe.length < 26) && ne(oe, 0, ae)) return { width: ee(oe, 18), height: ee(oe, 22), type: "bmp", mime: "image/bmp", wUnits: "px", hUnits: "px" };
            };
          }, 37495: function(G, J, Z) {
            var te = Z(3944).VG, ne = Z(3944).rU, ee = Z(3944).$l, ae = te("GIF87a"), oe = te("GIF89a");
            G.exports = function(se) {
              if (!(se.length < 10) && (ne(se, 0, ae) || ne(se, 0, oe))) return { width: ee(se, 6), height: ee(se, 8), type: "gif", mime: "image/gif", wUnits: "px", hUnits: "px" };
            };
          }, 88708: function(G, J, Z) {
            var te = Z(3944).$l;
            G.exports = function(ne) {
              var ee = te(ne, 0), ae = te(ne, 2), oe = te(ne, 4);
              if (ee === 0 && ae === 1 && oe) {
                for (var se = [], ce = { width: 0, height: 0 }, le = 0; le < oe; le++) {
                  var ue = ne[6 + 16 * le] || 256, de = ne[6 + 16 * le + 1] || 256, fe = { width: ue, height: de };
                  se.push(fe), (ue > ce.width || de > ce.height) && (ce = fe);
                }
                return { width: ce.width, height: ce.height, variants: se, type: "ico", mime: "image/x-icon", wUnits: "px", hUnits: "px" };
              }
            };
          }, 13827: function(G, J, Z) {
            var te = Z(3944).bc, ne = Z(3944).VG, ee = Z(3944).rU, ae = Z(19789), oe = ne("Exif\0\0");
            G.exports = function(se) {
              if (!(se.length < 2) && se[0] === 255 && se[1] === 216 && se[2] === 255) for (var ce = 2; ; ) {
                for (; ; ) {
                  if (se.length - ce < 2) return;
                  if (se[ce++] === 255) break;
                }
                for (var le, ue, de = se[ce++]; de === 255; ) de = se[ce++];
                if (208 <= de && de <= 217 || de === 1) le = 0;
                else {
                  if (!(192 <= de && de <= 254) || se.length - ce < 2) return;
                  le = te(se, ce) - 2, ce += 2;
                }
                if (de === 217 || de === 218) return;
                if (de === 225 && le >= 10 && ee(se, ce, oe) && (ue = ae.get_orientation(se.slice(ce + 6, ce + le))), le >= 5 && 192 <= de && de <= 207 && de !== 196 && de !== 200 && de !== 204) {
                  if (se.length - ce < le) return;
                  var fe = { width: te(se, ce + 3), height: te(se, ce + 1), type: "jpg", mime: "image/jpeg", wUnits: "px", hUnits: "px" };
                  return ue > 0 && (fe.orientation = ue), fe;
                }
                ce += le;
              }
            };
          }, 46594: function(G, J, Z) {
            var te = Z(3944).VG, ne = Z(3944).rU, ee = Z(3944).bb, ae = te(`PNG\r

`), oe = te("IHDR");
            G.exports = function(se) {
              if (!(se.length < 24) && ne(se, 0, ae) && ne(se, 12, oe)) return { width: ee(se, 16), height: ee(se, 20), type: "png", mime: "image/png", wUnits: "px", hUnits: "px" };
            };
          }, 13198: function(G, J, Z) {
            var te = Z(3944).VG, ne = Z(3944).rU, ee = Z(3944).bb, ae = te("8BPS\0");
            G.exports = function(oe) {
              if (!(oe.length < 22) && ne(oe, 0, ae)) return { width: ee(oe, 18), height: ee(oe, 14), type: "psd", mime: "image/vnd.adobe.photoshop", wUnits: "px", hUnits: "px" };
            };
          }, 94203: function(G) {
            function J(ce) {
              return typeof ce == "number" && isFinite(ce) && ce > 0;
            }
            var Z = /<[-_.:a-zA-Z0-9][^>]*>/, te = /^<([-_.:a-zA-Z0-9]+:)?svg\s/, ne = /[^-]\bwidth="([^%]+?)"|[^-]\bwidth='([^%]+?)'/, ee = /\bheight="([^%]+?)"|\bheight='([^%]+?)'/, ae = /\bview[bB]ox="(.+?)"|\bview[bB]ox='(.+?)'/, oe = /in$|mm$|cm$|pt$|pc$|px$|em$|ex$/;
            function se(ce) {
              return oe.test(ce) ? ce.match(oe)[0] : "px";
            }
            G.exports = function(ce) {
              if (function(Ce) {
                var Se, we = 0, Ee = Ce.length;
                for (Ce[0] === 239 && Ce[1] === 187 && Ce[2] === 191 && (we = 3); we < Ee && ((Se = Ce[we]) === 32 || Se === 9 || Se === 13 || Se === 10); ) we++;
                return we !== Ee && Ce[we] === 60;
              }(ce)) {
                for (var le = "", ue = 0; ue < ce.length; ue++) le += String.fromCharCode(ce[ue]);
                var de = (le.match(Z) || [""])[0];
                if (te.test(de)) {
                  var fe = function(Ce) {
                    var Se = Ce.match(ne), we = Ce.match(ee), Ee = Ce.match(ae);
                    return { width: Se && (Se[1] || Se[2]), height: we && (we[1] || we[2]), viewbox: Ee && (Ee[1] || Ee[2]) };
                  }(de), he = parseFloat(fe.width), me = parseFloat(fe.height);
                  if (fe.width && fe.height)
                    return !J(he) || !J(me) ? void 0 : { width: he, height: me, type: "svg", mime: "image/svg+xml", wUnits: se(fe.width), hUnits: se(fe.height) };
                  var pe = (fe.viewbox || "").split(" "), ye = { width: pe[2], height: pe[3] }, ge = parseFloat(ye.width), ve = parseFloat(ye.height);
                  if (J(ge) && J(ve) && se(ye.width) === se(ye.height)) {
                    var be = ge / ve;
                    return fe.width ? J(he) ? { width: he, height: he / be, type: "svg", mime: "image/svg+xml", wUnits: se(fe.width), hUnits: se(fe.width) } : void 0 : fe.height ? J(me) ? { width: me * be, height: me, type: "svg", mime: "image/svg+xml", wUnits: se(fe.height), hUnits: se(fe.height) } : void 0 : { width: ge, height: ve, type: "svg", mime: "image/svg+xml", wUnits: se(ye.width), hUnits: se(ye.height) };
                  }
                }
              }
            };
          }, 46966: function(G, J, Z) {
            var te = Z(3944).VG, ne = Z(3944).rU, ee = Z(3944).$l, ae = Z(3944).bc, oe = Z(3944).tF, se = Z(3944).bb, ce = te("II*\0"), le = te("MM\0*");
            function ue(he, me, pe) {
              return pe ? ae(he, me) : ee(he, me);
            }
            function de(he, me, pe) {
              return pe ? se(he, me) : oe(he, me);
            }
            function fe(he, me, pe) {
              var ye = ue(he, me + 2, pe);
              return de(he, me + 4, pe) !== 1 || ye !== 3 && ye !== 4 ? null : ye === 3 ? ue(he, me + 8, pe) : de(he, me + 8, pe);
            }
            G.exports = function(he) {
              if (!(he.length < 8) && (ne(he, 0, ce) || ne(he, 0, le))) {
                var me = he[0] === 77, pe = de(he, 4, me) - 8;
                if (!(pe < 0)) {
                  var ye = pe + 8;
                  if (!(he.length - ye < 2)) {
                    var ge = 12 * ue(he, ye + 0, me);
                    if (!(ge <= 0 || (ye += 2, he.length - ye < ge))) {
                      var ve, be, Ce, Se;
                      for (ve = 0; ve < ge; ve += 12) (Se = ue(he, ye + ve, me)) === 256 ? be = fe(he, ye + ve, me) : Se === 257 && (Ce = fe(he, ye + ve, me));
                      return be && Ce ? { width: be, height: Ce, type: "tiff", mime: "image/tiff", wUnits: "px", hUnits: "px" } : void 0;
                    }
                  }
                }
              }
            };
          }, 88023: function(G, J, Z) {
            var te = Z(3944).VG, ne = Z(3944).rU, ee = Z(3944).$l, ae = Z(3944).tF, oe = Z(19789), se = te("RIFF"), ce = te("WEBP");
            function le(fe, he) {
              if (fe[he + 3] === 157 && fe[he + 4] === 1 && fe[he + 5] === 42) return { width: 16383 & ee(fe, he + 6), height: 16383 & ee(fe, he + 8), type: "webp", mime: "image/webp", wUnits: "px", hUnits: "px" };
            }
            function ue(fe, he) {
              if (fe[he] === 47) {
                var me = ae(fe, he + 1);
                return { width: 1 + (16383 & me), height: 1 + (me >> 14 & 16383), type: "webp", mime: "image/webp", wUnits: "px", hUnits: "px" };
              }
            }
            function de(fe, he) {
              return { width: 1 + (fe[he + 6] << 16 | fe[he + 5] << 8 | fe[he + 4]), height: 1 + (fe[he + 9] << he | fe[he + 8] << 8 | fe[he + 7]), type: "webp", mime: "image/webp", wUnits: "px", hUnits: "px" };
            }
            G.exports = function(fe) {
              if (!(fe.length < 16) && (ne(fe, 0, se) || ne(fe, 8, ce))) {
                var he = 12, me = null, pe = 0, ye = ae(fe, 4) + 8;
                if (!(ye > fe.length)) {
                  for (; he + 8 < ye; ) if (fe[he] !== 0) {
                    var ge = String.fromCharCode.apply(null, fe.slice(he, he + 4)), ve = ae(fe, he + 4);
                    ge === "VP8 " && ve >= 10 ? me = me || le(fe, he + 8) : ge === "VP8L" && ve >= 9 ? me = me || ue(fe, he + 8) : ge === "VP8X" && ve >= 10 ? me = me || de(fe, he + 8) : ge === "EXIF" && (pe = oe.get_orientation(fe.slice(he + 8, he + 8 + ve)), he = 1 / 0), he += 8 + ve;
                  } else he++;
                  if (me) return pe > 0 && (me.orientation = pe), me;
                }
              }
            };
          }, 43751: function(G, J, Z) {
            G.exports = { avif: Z(31149), bmp: Z(78218), gif: Z(37495), ico: Z(88708), jpeg: Z(13827), png: Z(46594), psd: Z(13198), svg: Z(94203), tiff: Z(46966), webp: Z(88023) };
          }, 19490: function(G, J, Z) {
            var te = Z(43751);
            G.exports = function(ne) {
              return function(ee) {
                for (var ae = Object.keys(te), oe = 0; oe < ae.length; oe++) {
                  var se = te[ae[oe]](ee);
                  if (se) return se;
                }
                return null;
              }(ne);
            }, G.exports.parsers = te;
          }, 33282: function(G) {
            var J, Z, te = G.exports = {};
            function ne() {
              throw new Error("setTimeout has not been defined");
            }
            function ee() {
              throw new Error("clearTimeout has not been defined");
            }
            function ae(me) {
              if (J === setTimeout) return setTimeout(me, 0);
              if ((J === ne || !J) && setTimeout) return J = setTimeout, setTimeout(me, 0);
              try {
                return J(me, 0);
              } catch {
                try {
                  return J.call(null, me, 0);
                } catch {
                  return J.call(this, me, 0);
                }
              }
            }
            (function() {
              try {
                J = typeof setTimeout == "function" ? setTimeout : ne;
              } catch {
                J = ne;
              }
              try {
                Z = typeof clearTimeout == "function" ? clearTimeout : ee;
              } catch {
                Z = ee;
              }
            })();
            var oe, se = [], ce = !1, le = -1;
            function ue() {
              ce && oe && (ce = !1, oe.length ? se = oe.concat(se) : le = -1, se.length && de());
            }
            function de() {
              if (!ce) {
                var me = ae(ue);
                ce = !0;
                for (var pe = se.length; pe; ) {
                  for (oe = se, se = []; ++le < pe; ) oe && oe[le].run();
                  le = -1, pe = se.length;
                }
                oe = null, ce = !1, function(ye) {
                  if (Z === clearTimeout) return clearTimeout(ye);
                  if ((Z === ee || !Z) && clearTimeout) return Z = clearTimeout, clearTimeout(ye);
                  try {
                    return Z(ye);
                  } catch {
                    try {
                      return Z.call(null, ye);
                    } catch {
                      return Z.call(this, ye);
                    }
                  }
                }(me);
              }
            }
            function fe(me, pe) {
              this.fun = me, this.array = pe;
            }
            function he() {
            }
            te.nextTick = function(me) {
              var pe = new Array(arguments.length - 1);
              if (arguments.length > 1) for (var ye = 1; ye < arguments.length; ye++) pe[ye - 1] = arguments[ye];
              se.push(new fe(me, pe)), se.length !== 1 || ce || ae(de);
            }, fe.prototype.run = function() {
              this.fun.apply(null, this.array);
            }, te.title = "browser", te.browser = !0, te.env = {}, te.argv = [], te.version = "", te.versions = {}, te.on = he, te.addListener = he, te.once = he, te.off = he, te.removeListener = he, te.removeAllListeners = he, te.emit = he, te.prependListener = he, te.prependOnceListener = he, te.listeners = function(me) {
              return [];
            }, te.binding = function(me) {
              throw new Error("process.binding is not supported");
            }, te.cwd = function() {
              return "/";
            }, te.chdir = function(me) {
              throw new Error("process.chdir is not supported");
            }, te.umask = function() {
              return 0;
            };
          }, 16494: function(G, J, Z) {
            for (var te = Z(71879), ne = typeof window > "u" ? Z.g : window, ee = ["moz", "webkit"], ae = "AnimationFrame", oe = ne["request" + ae], se = ne["cancel" + ae] || ne["cancelRequest" + ae], ce = 0; !oe && ce < ee.length; ce++) oe = ne[ee[ce] + "Request" + ae], se = ne[ee[ce] + "Cancel" + ae] || ne[ee[ce] + "CancelRequest" + ae];
            if (!oe || !se) {
              var le = 0, ue = 0, de = [];
              oe = function(fe) {
                if (de.length === 0) {
                  var he = te(), me = Math.max(0, 16.666666666666668 - (he - le));
                  le = me + he, setTimeout(function() {
                    var pe = de.slice(0);
                    de.length = 0;
                    for (var ye = 0; ye < pe.length; ye++) if (!pe[ye].cancelled) try {
                      pe[ye].callback(le);
                    } catch (ge) {
                      setTimeout(function() {
                        throw ge;
                      }, 0);
                    }
                  }, Math.round(me));
                }
                return de.push({ handle: ++ue, callback: fe, cancelled: !1 }), ue;
              }, se = function(fe) {
                for (var he = 0; he < de.length; he++) de[he].handle === fe && (de[he].cancelled = !0);
              };
            }
            G.exports = function(fe) {
              return oe.call(ne, fe);
            }, G.exports.cancel = function() {
              se.apply(ne, arguments);
            }, G.exports.polyfill = function(fe) {
              fe || (fe = ne), fe.requestAnimationFrame = oe, fe.cancelAnimationFrame = se;
            };
          }, 29978: function(G, J, Z) {
            var te = Z(78112), ne = Z(162), ee = Z(79788), ae = Z(6807), oe = Z(27976), se = Z(83473), ce = Z(51498), le = ce.float32, ue = ce.fract32;
            G.exports = function(fe, he) {
              if (typeof fe == "function" ? (he || (he = {}), he.regl = fe) : he = fe, he.length && (he.positions = he), !(fe = he.regl).hasExtension("ANGLE_instanced_arrays")) throw Error("regl-error2d: `ANGLE_instanced_arrays` extension should be enabled");
              var me, pe, ye, ge, ve, be, Ce = fe._gl, Se = { color: "black", capSize: 5, lineWidth: 1, opacity: 1, viewport: null, range: null, offset: 0, count: 0, bounds: null, positions: [], errors: [] }, we = [];
              return ge = fe.buffer({ usage: "dynamic", type: "uint8", data: new Uint8Array(0) }), pe = fe.buffer({ usage: "dynamic", type: "float", data: new Uint8Array(0) }), ye = fe.buffer({ usage: "dynamic", type: "float", data: new Uint8Array(0) }), ve = fe.buffer({ usage: "dynamic", type: "float", data: new Uint8Array(0) }), be = fe.buffer({ usage: "static", type: "float", data: de }), Qe(he), me = fe({ vert: `
		precision highp float;

		attribute vec2 position, positionFract;
		attribute vec4 error;
		attribute vec4 color;

		attribute vec2 direction, lineOffset, capOffset;

		uniform vec4 viewport;
		uniform float lineWidth, capSize;
		uniform vec2 scale, scaleFract, translate, translateFract;

		varying vec4 fragColor;

		void main() {
			fragColor = color / 255.;

			vec2 pixelOffset = lineWidth * lineOffset + (capSize + lineWidth) * capOffset;

			vec2 dxy = -step(.5, direction.xy) * error.xz + step(direction.xy, vec2(-.5)) * error.yw;

			vec2 position = position + dxy;

			vec2 pos = (position + translate) * scale
				+ (positionFract + translateFract) * scale
				+ (position + translate) * scaleFract
				+ (positionFract + translateFract) * scaleFract;

			pos += pixelOffset / viewport.zw;

			gl_Position = vec4(pos * 2. - 1., 0, 1);
		}
		`, frag: `
		precision highp float;

		varying vec4 fragColor;

		uniform float opacity;

		void main() {
			gl_FragColor = fragColor;
			gl_FragColor.a *= opacity;
		}
		`, uniforms: { range: fe.prop("range"), lineWidth: fe.prop("lineWidth"), capSize: fe.prop("capSize"), opacity: fe.prop("opacity"), scale: fe.prop("scale"), translate: fe.prop("translate"), scaleFract: fe.prop("scaleFract"), translateFract: fe.prop("translateFract"), viewport: function(Me, Re) {
                return [Re.viewport.x, Re.viewport.y, Me.viewportWidth, Me.viewportHeight];
              } }, attributes: { color: { buffer: ge, offset: function(Me, Re) {
                return 4 * Re.offset;
              }, divisor: 1 }, position: { buffer: pe, offset: function(Me, Re) {
                return 8 * Re.offset;
              }, divisor: 1 }, positionFract: { buffer: ye, offset: function(Me, Re) {
                return 8 * Re.offset;
              }, divisor: 1 }, error: { buffer: ve, offset: function(Me, Re) {
                return 16 * Re.offset;
              }, divisor: 1 }, direction: { buffer: be, stride: 24, offset: 0 }, lineOffset: { buffer: be, stride: 24, offset: 8 }, capOffset: { buffer: be, stride: 24, offset: 16 } }, primitive: "triangles", blend: { enable: !0, color: [0, 0, 0, 0], equation: { rgb: "add", alpha: "add" }, func: { srcRGB: "src alpha", dstRGB: "one minus src alpha", srcAlpha: "one minus dst alpha", dstAlpha: "one" } }, depth: { enable: !1 }, scissor: { enable: !0, box: fe.prop("viewport") }, viewport: fe.prop("viewport"), stencil: !1, instances: fe.prop("count"), count: de.length }), oe(Ee, { update: Qe, draw: xe, destroy: ke, regl: fe, gl: Ce, canvas: Ce.canvas, groups: we }), Ee;
              function Ee(Me) {
                Me ? Qe(Me) : Me === null && ke(), xe();
              }
              function xe(Me) {
                if (typeof Me == "number") return Le(Me);
                Me && !Array.isArray(Me) && (Me = [Me]), fe._refresh(), we.forEach(function(Re, De) {
                  Re && (Me && (Me[De] ? Re.draw = !0 : Re.draw = !1), Re.draw ? Le(De) : Re.draw = !0);
                });
              }
              function Le(Me) {
                typeof Me == "number" && (Me = we[Me]), Me != null && Me && Me.count && Me.color && Me.opacity && Me.positions && Me.positions.length > 1 && (Me.scaleRatio = [Me.scale[0] * Me.viewport.width, Me.scale[1] * Me.viewport.height], me(Me), Me.after && Me.after(Me));
              }
              function Qe(Me) {
                if (Me) {
                  Me.length != null ? typeof Me[0] == "number" && (Me = [{ positions: Me }]) : Array.isArray(Me) || (Me = [Me]);
                  var Re = 0, De = 0;
                  if (Ee.groups = we = Me.map(function(Ge, ze) {
                    var We = we[ze];
                    return Ge && (typeof Ge == "function" ? Ge = { after: Ge } : typeof Ge[0] == "number" && (Ge = { positions: Ge }), Ge = ae(Ge, { color: "color colors fill", capSize: "capSize cap capsize cap-size", lineWidth: "lineWidth line-width width line thickness", opacity: "opacity alpha", range: "range dataBox", viewport: "viewport viewBox", errors: "errors error", positions: "positions position data points" }), We || (we[ze] = We = { id: ze, scale: null, translate: null, scaleFract: null, translateFract: null, draw: !0 }, Ge = oe({}, Se, Ge)), ee(We, Ge, [{ lineWidth: function(at) {
                      return 0.5 * +at;
                    }, capSize: function(at) {
                      return 0.5 * +at;
                    }, opacity: parseFloat, errors: function(at) {
                      return at = se(at), De += at.length, at;
                    }, positions: function(at, ct) {
                      return at = se(at, "float64"), ct.count = Math.floor(at.length / 2), ct.bounds = te(at, 2), ct.offset = Re, Re += ct.count, at;
                    } }, { color: function(at, ct) {
                      var et = ct.count;
                      if (at || (at = "transparent"), !Array.isArray(at) || typeof at[0] == "number") {
                        var dt = at;
                        at = Array(et);
                        for (var Je = 0; Je < et; Je++) at[Je] = dt;
                      }
                      if (at.length < et) throw Error("Not enough colors");
                      for (var it = new Uint8Array(4 * et), st = 0; st < et; st++) {
                        var gt = ne(at[st], "uint8");
                        it.set(gt, 4 * st);
                      }
                      return it;
                    }, range: function(at, ct, et) {
                      var dt = ct.bounds;
                      return at || (at = dt), ct.scale = [1 / (at[2] - at[0]), 1 / (at[3] - at[1])], ct.translate = [-at[0], -at[1]], ct.scaleFract = ue(ct.scale), ct.translateFract = ue(ct.translate), at;
                    }, viewport: function(at) {
                      var ct;
                      return Array.isArray(at) ? ct = { x: at[0], y: at[1], width: at[2] - at[0], height: at[3] - at[1] } : at ? (ct = { x: at.x || at.left || 0, y: at.y || at.top || 0 }, at.right ? ct.width = at.right - ct.x : ct.width = at.w || at.width || 0, at.bottom ? ct.height = at.bottom - ct.y : ct.height = at.h || at.height || 0) : ct = { x: 0, y: 0, width: Ce.drawingBufferWidth, height: Ce.drawingBufferHeight }, ct;
                    } }])), We;
                  }), Re || De) {
                    var Pe = we.reduce(function(Ge, ze, We) {
                      return Ge + (ze ? ze.count : 0);
                    }, 0), $e = new Float64Array(2 * Pe), qe = new Uint8Array(4 * Pe), Xe = new Float32Array(4 * Pe);
                    we.forEach(function(Ge, ze) {
                      if (Ge) {
                        var We = Ge.positions, at = Ge.count, ct = Ge.offset, et = Ge.color, dt = Ge.errors;
                        at && (qe.set(et, 4 * ct), Xe.set(dt, 4 * ct), $e.set(We, 2 * ct));
                      }
                    });
                    var Ve = le($e);
                    pe(Ve);
                    var Ze = ue($e, Ve);
                    ye(Ze), ge(qe), ve(Xe);
                  }
                }
              }
              function ke() {
                pe.destroy(), ye.destroy(), ge.destroy(), ve.destroy(), be.destroy();
              }
            };
            var de = [[1, 0, 0, 1, 0, 0], [1, 0, 0, -1, 0, 0], [-1, 0, 0, -1, 0, 0], [-1, 0, 0, -1, 0, 0], [-1, 0, 0, 1, 0, 0], [1, 0, 0, 1, 0, 0], [1, 0, -1, 0, 0, 1], [1, 0, -1, 0, 0, -1], [1, 0, 1, 0, 0, -1], [1, 0, 1, 0, 0, -1], [1, 0, 1, 0, 0, 1], [1, 0, -1, 0, 0, 1], [-1, 0, -1, 0, 0, 1], [-1, 0, -1, 0, 0, -1], [-1, 0, 1, 0, 0, -1], [-1, 0, 1, 0, 0, -1], [-1, 0, 1, 0, 0, 1], [-1, 0, -1, 0, 0, 1], [0, 1, 1, 0, 0, 0], [0, 1, -1, 0, 0, 0], [0, -1, -1, 0, 0, 0], [0, -1, -1, 0, 0, 0], [0, 1, 1, 0, 0, 0], [0, -1, 1, 0, 0, 0], [0, 1, 0, -1, 1, 0], [0, 1, 0, -1, -1, 0], [0, 1, 0, 1, -1, 0], [0, 1, 0, 1, 1, 0], [0, 1, 0, -1, 1, 0], [0, 1, 0, 1, -1, 0], [0, -1, 0, -1, 1, 0], [0, -1, 0, -1, -1, 0], [0, -1, 0, 1, -1, 0], [0, -1, 0, 1, 1, 0], [0, -1, 0, -1, 1, 0], [0, -1, 0, 1, -1, 0]];
          }, 49478: function(G, J, Z) {
            var te = Z(162), ne = Z(78112), ee = Z(27976), ae = Z(6807), oe = Z(83473), se = Z(25782), ce = Z(90956), le = Z(51498), ue = le.float32, de = le.fract32, fe = Z(93103), he = Z(5137), me = Z(33055);
            function pe(ye, ge) {
              if (!(this instanceof pe)) return new pe(ye, ge);
              if (typeof ye == "function" ? (ge || (ge = {}), ge.regl = ye) : ge = ye, ge.length && (ge.positions = ge), !(ye = ge.regl).hasExtension("ANGLE_instanced_arrays")) throw Error("regl-error2d: `ANGLE_instanced_arrays` extension should be enabled");
              this.gl = ye._gl, this.regl = ye, this.passes = [], this.shaders = pe.shaders.has(ye) ? pe.shaders.get(ye) : pe.shaders.set(ye, pe.createShaders(ye)).get(ye), this.update(ge);
            }
            G.exports = pe, pe.dashMult = 2, pe.maxPatternLength = 256, pe.precisionThreshold = 3e6, pe.maxPoints = 1e4, pe.maxLines = 2048, pe.shaders = new fe(), pe.createShaders = function(ye) {
              var ge, ve = ye.buffer({ usage: "static", type: "float", data: [0, 1, 0, 0, 1, 1, 1, 0] }), be = { primitive: "triangle strip", instances: ye.prop("count"), count: 4, offset: 0, uniforms: { miterMode: function(Se, we) {
                return we.join === "round" ? 2 : 1;
              }, miterLimit: ye.prop("miterLimit"), scale: ye.prop("scale"), scaleFract: ye.prop("scaleFract"), translateFract: ye.prop("translateFract"), translate: ye.prop("translate"), thickness: ye.prop("thickness"), dashTexture: ye.prop("dashTexture"), opacity: ye.prop("opacity"), pixelRatio: ye.context("pixelRatio"), id: ye.prop("id"), dashLength: ye.prop("dashLength"), viewport: function(Se, we) {
                return [we.viewport.x, we.viewport.y, Se.viewportWidth, Se.viewportHeight];
              }, depth: ye.prop("depth") }, blend: { enable: !0, color: [0, 0, 0, 0], equation: { rgb: "add", alpha: "add" }, func: { srcRGB: "src alpha", dstRGB: "one minus src alpha", srcAlpha: "one minus dst alpha", dstAlpha: "one" } }, depth: { enable: function(Se, we) {
                return !we.overlay;
              } }, stencil: { enable: !1 }, scissor: { enable: !0, box: ye.prop("viewport") }, viewport: ye.prop("viewport") }, Ce = ye(ee({ vert: `
precision highp float;

attribute vec2 aCoord, bCoord, aCoordFract, bCoordFract;
attribute vec4 color;
attribute float lineEnd, lineTop;

uniform vec2 scale, scaleFract, translate, translateFract;
uniform float thickness, pixelRatio, id, depth;
uniform vec4 viewport;

varying vec4 fragColor;
varying vec2 tangent;

vec2 project(vec2 position, vec2 positionFract, vec2 scale, vec2 scaleFract, vec2 translate, vec2 translateFract) {
	// the order is important
	return position * scale + translate
       + positionFract * scale + translateFract
       + position * scaleFract
       + positionFract * scaleFract;
}

void main() {
	float lineStart = 1. - lineEnd;
	float lineOffset = lineTop * 2. - 1.;

	vec2 diff = (bCoord + bCoordFract - aCoord - aCoordFract);
	tangent = normalize(diff * scale * viewport.zw);
	vec2 normal = vec2(-tangent.y, tangent.x);

	vec2 position = project(aCoord, aCoordFract, scale, scaleFract, translate, translateFract) * lineStart
		+ project(bCoord, bCoordFract, scale, scaleFract, translate, translateFract) * lineEnd

		+ thickness * normal * .5 * lineOffset / viewport.zw;

	gl_Position = vec4(position * 2.0 - 1.0, depth, 1);

	fragColor = color / 255.;
}
`, frag: `
precision highp float;

uniform float dashLength, pixelRatio, thickness, opacity, id;
uniform sampler2D dashTexture;

varying vec4 fragColor;
varying vec2 tangent;

void main() {
	float alpha = 1.;

	float t = fract(dot(tangent, gl_FragCoord.xy) / dashLength) * .5 + .25;
	float dash = texture2D(dashTexture, vec2(t, .5)).r;

	gl_FragColor = fragColor;
	gl_FragColor.a *= alpha * opacity * dash;
}
`, attributes: { lineEnd: { buffer: ve, divisor: 0, stride: 8, offset: 0 }, lineTop: { buffer: ve, divisor: 0, stride: 8, offset: 4 }, aCoord: { buffer: ye.prop("positionBuffer"), stride: 8, offset: 8, divisor: 1 }, bCoord: { buffer: ye.prop("positionBuffer"), stride: 8, offset: 16, divisor: 1 }, aCoordFract: { buffer: ye.prop("positionFractBuffer"), stride: 8, offset: 8, divisor: 1 }, bCoordFract: { buffer: ye.prop("positionFractBuffer"), stride: 8, offset: 16, divisor: 1 }, color: { buffer: ye.prop("colorBuffer"), stride: 4, offset: 0, divisor: 1 } } }, be));
              try {
                ge = ye(ee({ cull: { enable: !0, face: "back" }, vert: `
precision highp float;

attribute vec2 aCoord, bCoord, nextCoord, prevCoord;
attribute vec4 aColor, bColor;
attribute float lineEnd, lineTop;

uniform vec2 scale, translate;
uniform float thickness, pixelRatio, id, depth;
uniform vec4 viewport;
uniform float miterLimit, miterMode;

varying vec4 fragColor;
varying vec4 startCutoff, endCutoff;
varying vec2 tangent;
varying vec2 startCoord, endCoord;
varying float enableStartMiter, enableEndMiter;

const float REVERSE_THRESHOLD = -.875;
const float MIN_DIFF = 1e-6;

// TODO: possible optimizations: avoid overcalculating all for vertices and calc just one instead
// TODO: precalculate dot products, normalize things beforehead etc.
// TODO: refactor to rectangular algorithm

float distToLine(vec2 p, vec2 a, vec2 b) {
	vec2 diff = b - a;
	vec2 perp = normalize(vec2(-diff.y, diff.x));
	return dot(p - a, perp);
}

bool isNaN( float val ){
  return ( val < 0.0 || 0.0 < val || val == 0.0 ) ? false : true;
}

void main() {
	vec2 aCoord = aCoord, bCoord = bCoord, prevCoord = prevCoord, nextCoord = nextCoord;

  vec2 adjustedScale;
  adjustedScale.x = (abs(scale.x) < MIN_DIFF) ? MIN_DIFF : scale.x;
  adjustedScale.y = (abs(scale.y) < MIN_DIFF) ? MIN_DIFF : scale.y;

  vec2 scaleRatio = adjustedScale * viewport.zw;
	vec2 normalWidth = thickness / scaleRatio;

	float lineStart = 1. - lineEnd;
	float lineBot = 1. - lineTop;

	fragColor = (lineStart * aColor + lineEnd * bColor) / 255.;

	if (isNaN(aCoord.x) || isNaN(aCoord.y) || isNaN(bCoord.x) || isNaN(bCoord.y)) return;

	if (aCoord == prevCoord) prevCoord = aCoord + normalize(bCoord - aCoord);
	if (bCoord == nextCoord) nextCoord = bCoord - normalize(bCoord - aCoord);


	vec2 prevDiff = aCoord - prevCoord;
	vec2 currDiff = bCoord - aCoord;
	vec2 nextDiff = nextCoord - bCoord;

	vec2 prevTangent = normalize(prevDiff * scaleRatio);
	vec2 currTangent = normalize(currDiff * scaleRatio);
	vec2 nextTangent = normalize(nextDiff * scaleRatio);

	vec2 prevNormal = vec2(-prevTangent.y, prevTangent.x);
	vec2 currNormal = vec2(-currTangent.y, currTangent.x);
	vec2 nextNormal = vec2(-nextTangent.y, nextTangent.x);

	vec2 startJoinDirection = normalize(prevTangent - currTangent);
	vec2 endJoinDirection = normalize(currTangent - nextTangent);

	// collapsed/unidirectional segment cases
	// FIXME: there should be more elegant solution
	vec2 prevTanDiff = abs(prevTangent - currTangent);
	vec2 nextTanDiff = abs(nextTangent - currTangent);
	if (max(prevTanDiff.x, prevTanDiff.y) < MIN_DIFF) {
		startJoinDirection = currNormal;
	}
	if (max(nextTanDiff.x, nextTanDiff.y) < MIN_DIFF) {
		endJoinDirection = currNormal;
	}
	if (aCoord == bCoord) {
		endJoinDirection = startJoinDirection;
		currNormal = prevNormal;
		currTangent = prevTangent;
	}

	tangent = currTangent;

	//calculate join shifts relative to normals
	float startJoinShift = dot(currNormal, startJoinDirection);
	float endJoinShift = dot(currNormal, endJoinDirection);

	float startMiterRatio = abs(1. / startJoinShift);
	float endMiterRatio = abs(1. / endJoinShift);

	vec2 startJoin = startJoinDirection * startMiterRatio;
	vec2 endJoin = endJoinDirection * endMiterRatio;

	vec2 startTopJoin, startBotJoin, endTopJoin, endBotJoin;
	startTopJoin = sign(startJoinShift) * startJoin * .5;
	startBotJoin = -startTopJoin;

	endTopJoin = sign(endJoinShift) * endJoin * .5;
	endBotJoin = -endTopJoin;

	vec2 aTopCoord = aCoord + normalWidth * startTopJoin;
	vec2 bTopCoord = bCoord + normalWidth * endTopJoin;
	vec2 aBotCoord = aCoord + normalWidth * startBotJoin;
	vec2 bBotCoord = bCoord + normalWidth * endBotJoin;

	//miter anti-clipping
	float baClipping = distToLine(bCoord, aCoord, aBotCoord) / dot(normalize(normalWidth * endBotJoin), normalize(normalWidth.yx * vec2(-startBotJoin.y, startBotJoin.x)));
	float abClipping = distToLine(aCoord, bCoord, bTopCoord) / dot(normalize(normalWidth * startBotJoin), normalize(normalWidth.yx * vec2(-endBotJoin.y, endBotJoin.x)));

	//prevent close to reverse direction switch
	bool prevReverse = dot(currTangent, prevTangent) <= REVERSE_THRESHOLD && abs(dot(currTangent, prevNormal)) * min(length(prevDiff), length(currDiff)) <  length(normalWidth * currNormal);
	bool nextReverse = dot(currTangent, nextTangent) <= REVERSE_THRESHOLD && abs(dot(currTangent, nextNormal)) * min(length(nextDiff), length(currDiff)) <  length(normalWidth * currNormal);

	if (prevReverse) {
		//make join rectangular
		vec2 miterShift = normalWidth * startJoinDirection * miterLimit * .5;
		float normalAdjust = 1. - min(miterLimit / startMiterRatio, 1.);
		aBotCoord = aCoord + miterShift - normalAdjust * normalWidth * currNormal * .5;
		aTopCoord = aCoord + miterShift + normalAdjust * normalWidth * currNormal * .5;
	}
	else if (!nextReverse && baClipping > 0. && baClipping < length(normalWidth * endBotJoin)) {
		//handle miter clipping
		bTopCoord -= normalWidth * endTopJoin;
		bTopCoord += normalize(endTopJoin * normalWidth) * baClipping;
	}

	if (nextReverse) {
		//make join rectangular
		vec2 miterShift = normalWidth * endJoinDirection * miterLimit * .5;
		float normalAdjust = 1. - min(miterLimit / endMiterRatio, 1.);
		bBotCoord = bCoord + miterShift - normalAdjust * normalWidth * currNormal * .5;
		bTopCoord = bCoord + miterShift + normalAdjust * normalWidth * currNormal * .5;
	}
	else if (!prevReverse && abClipping > 0. && abClipping < length(normalWidth * startBotJoin)) {
		//handle miter clipping
		aBotCoord -= normalWidth * startBotJoin;
		aBotCoord += normalize(startBotJoin * normalWidth) * abClipping;
	}

	vec2 aTopPosition = (aTopCoord) * adjustedScale + translate;
	vec2 aBotPosition = (aBotCoord) * adjustedScale + translate;

	vec2 bTopPosition = (bTopCoord) * adjustedScale + translate;
	vec2 bBotPosition = (bBotCoord) * adjustedScale + translate;

	//position is normalized 0..1 coord on the screen
	vec2 position = (aTopPosition * lineTop + aBotPosition * lineBot) * lineStart + (bTopPosition * lineTop + bBotPosition * lineBot) * lineEnd;

	startCoord = aCoord * scaleRatio + translate * viewport.zw + viewport.xy;
	endCoord = bCoord * scaleRatio + translate * viewport.zw + viewport.xy;

	gl_Position = vec4(position  * 2.0 - 1.0, depth, 1);

	enableStartMiter = step(dot(currTangent, prevTangent), .5);
	enableEndMiter = step(dot(currTangent, nextTangent), .5);

	//bevel miter cutoffs
	if (miterMode == 1.) {
		if (enableStartMiter == 1.) {
			vec2 startMiterWidth = vec2(startJoinDirection) * thickness * miterLimit * .5;
			startCutoff = vec4(aCoord, aCoord);
			startCutoff.zw += vec2(-startJoinDirection.y, startJoinDirection.x) / scaleRatio;
			startCutoff = startCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;
			startCutoff += viewport.xyxy;
			startCutoff += startMiterWidth.xyxy;
		}

		if (enableEndMiter == 1.) {
			vec2 endMiterWidth = vec2(endJoinDirection) * thickness * miterLimit * .5;
			endCutoff = vec4(bCoord, bCoord);
			endCutoff.zw += vec2(-endJoinDirection.y, endJoinDirection.x)  / scaleRatio;
			endCutoff = endCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;
			endCutoff += viewport.xyxy;
			endCutoff += endMiterWidth.xyxy;
		}
	}

	//round miter cutoffs
	else if (miterMode == 2.) {
		if (enableStartMiter == 1.) {
			vec2 startMiterWidth = vec2(startJoinDirection) * thickness * abs(dot(startJoinDirection, currNormal)) * .5;
			startCutoff = vec4(aCoord, aCoord);
			startCutoff.zw += vec2(-startJoinDirection.y, startJoinDirection.x) / scaleRatio;
			startCutoff = startCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;
			startCutoff += viewport.xyxy;
			startCutoff += startMiterWidth.xyxy;
		}

		if (enableEndMiter == 1.) {
			vec2 endMiterWidth = vec2(endJoinDirection) * thickness * abs(dot(endJoinDirection, currNormal)) * .5;
			endCutoff = vec4(bCoord, bCoord);
			endCutoff.zw += vec2(-endJoinDirection.y, endJoinDirection.x)  / scaleRatio;
			endCutoff = endCutoff * scaleRatio.xyxy + translate.xyxy * viewport.zwzw;
			endCutoff += viewport.xyxy;
			endCutoff += endMiterWidth.xyxy;
		}
	}
}
`, frag: `
precision highp float;

uniform float dashLength, pixelRatio, thickness, opacity, id, miterMode;
uniform sampler2D dashTexture;

varying vec4 fragColor;
varying vec2 tangent;
varying vec4 startCutoff, endCutoff;
varying vec2 startCoord, endCoord;
varying float enableStartMiter, enableEndMiter;

float distToLine(vec2 p, vec2 a, vec2 b) {
	vec2 diff = b - a;
	vec2 perp = normalize(vec2(-diff.y, diff.x));
	return dot(p - a, perp);
}

void main() {
	float alpha = 1., distToStart, distToEnd;
	float cutoff = thickness * .5;

	//bevel miter
	if (miterMode == 1.) {
		if (enableStartMiter == 1.) {
			distToStart = distToLine(gl_FragCoord.xy, startCutoff.xy, startCutoff.zw);
			if (distToStart < -1.) {
				discard;
				return;
			}
			alpha *= min(max(distToStart + 1., 0.), 1.);
		}

		if (enableEndMiter == 1.) {
			distToEnd = distToLine(gl_FragCoord.xy, endCutoff.xy, endCutoff.zw);
			if (distToEnd < -1.) {
				discard;
				return;
			}
			alpha *= min(max(distToEnd + 1., 0.), 1.);
		}
	}

	// round miter
	else if (miterMode == 2.) {
		if (enableStartMiter == 1.) {
			distToStart = distToLine(gl_FragCoord.xy, startCutoff.xy, startCutoff.zw);
			if (distToStart < 0.) {
				float radius = length(gl_FragCoord.xy - startCoord);

				if(radius > cutoff + .5) {
					discard;
					return;
				}

				alpha -= smoothstep(cutoff - .5, cutoff + .5, radius);
			}
		}

		if (enableEndMiter == 1.) {
			distToEnd = distToLine(gl_FragCoord.xy, endCutoff.xy, endCutoff.zw);
			if (distToEnd < 0.) {
				float radius = length(gl_FragCoord.xy - endCoord);

				if(radius > cutoff + .5) {
					discard;
					return;
				}

				alpha -= smoothstep(cutoff - .5, cutoff + .5, radius);
			}
		}
	}

	float t = fract(dot(tangent, gl_FragCoord.xy) / dashLength) * .5 + .25;
	float dash = texture2D(dashTexture, vec2(t, .5)).r;

	gl_FragColor = fragColor;
	gl_FragColor.a *= alpha * opacity * dash;
}
`, attributes: { lineEnd: { buffer: ve, divisor: 0, stride: 8, offset: 0 }, lineTop: { buffer: ve, divisor: 0, stride: 8, offset: 4 }, aColor: { buffer: ye.prop("colorBuffer"), stride: 4, offset: 0, divisor: 1 }, bColor: { buffer: ye.prop("colorBuffer"), stride: 4, offset: 4, divisor: 1 }, prevCoord: { buffer: ye.prop("positionBuffer"), stride: 8, offset: 0, divisor: 1 }, aCoord: { buffer: ye.prop("positionBuffer"), stride: 8, offset: 8, divisor: 1 }, bCoord: { buffer: ye.prop("positionBuffer"), stride: 8, offset: 16, divisor: 1 }, nextCoord: { buffer: ye.prop("positionBuffer"), stride: 8, offset: 24, divisor: 1 } } }, be));
              } catch {
                ge = Ce;
              }
              return { fill: ye({ primitive: "triangle", elements: function(Se, we) {
                return we.triangles;
              }, offset: 0, vert: `
precision highp float;

attribute vec2 position, positionFract;

uniform vec4 color;
uniform vec2 scale, scaleFract, translate, translateFract;
uniform float pixelRatio, id;
uniform vec4 viewport;
uniform float opacity;

varying vec4 fragColor;

const float MAX_LINES = 256.;

void main() {
	float depth = (MAX_LINES - 4. - id) / (MAX_LINES);

	vec2 position = position * scale + translate
       + positionFract * scale + translateFract
       + position * scaleFract
       + positionFract * scaleFract;

	gl_Position = vec4(position * 2.0 - 1.0, depth, 1);

	fragColor = color / 255.;
	fragColor.a *= opacity;
}
`, frag: `
precision highp float;
varying vec4 fragColor;

void main() {
	gl_FragColor = fragColor;
}
`, uniforms: { scale: ye.prop("scale"), color: ye.prop("fill"), scaleFract: ye.prop("scaleFract"), translateFract: ye.prop("translateFract"), translate: ye.prop("translate"), opacity: ye.prop("opacity"), pixelRatio: ye.context("pixelRatio"), id: ye.prop("id"), viewport: function(Se, we) {
                return [we.viewport.x, we.viewport.y, Se.viewportWidth, Se.viewportHeight];
              } }, attributes: { position: { buffer: ye.prop("positionBuffer"), stride: 8, offset: 8 }, positionFract: { buffer: ye.prop("positionFractBuffer"), stride: 8, offset: 8 } }, blend: be.blend, depth: { enable: !1 }, scissor: be.scissor, stencil: be.stencil, viewport: be.viewport }), rect: Ce, miter: ge };
            }, pe.defaults = { dashes: null, join: "miter", miterLimit: 1, thickness: 10, cap: "square", color: "black", opacity: 1, overlay: !1, viewport: null, range: null, close: !1, fill: null }, pe.prototype.render = function() {
              for (var ye, ge = [], ve = arguments.length; ve--; ) ge[ve] = arguments[ve];
              ge.length && (ye = this).update.apply(ye, ge), this.draw();
            }, pe.prototype.draw = function() {
              for (var ye = this, ge = [], ve = arguments.length; ve--; ) ge[ve] = arguments[ve];
              return (ge.length ? ge : this.passes).forEach(function(be, Ce) {
                var Se;
                if (be && Array.isArray(be)) return (Se = ye).draw.apply(Se, be);
                typeof be == "number" && (be = ye.passes[be]), be && be.count > 1 && be.opacity && (ye.regl._refresh(), be.fill && be.triangles && be.triangles.length > 2 && ye.shaders.fill(be), be.thickness && (be.scale[0] * be.viewport.width > pe.precisionThreshold || be.scale[1] * be.viewport.height > pe.precisionThreshold || be.join === "rect" || !be.join && (be.thickness <= 2 || be.count >= pe.maxPoints) ? ye.shaders.rect(be) : ye.shaders.miter(be)));
              }), this;
            }, pe.prototype.update = function(ye) {
              var ge = this;
              if (ye) {
                ye.length != null ? typeof ye[0] == "number" && (ye = [{ positions: ye }]) : Array.isArray(ye) || (ye = [ye]);
                var ve = this.regl, be = this.gl;
                if (ye.forEach(function(xe, Le) {
                  var Qe = ge.passes[Le];
                  if (xe !== void 0) if (xe !== null) {
                    if (typeof xe[0] == "number" && (xe = { positions: xe }), xe = ae(xe, { positions: "positions points data coords", thickness: "thickness lineWidth lineWidths line-width linewidth width stroke-width strokewidth strokeWidth", join: "lineJoin linejoin join type mode", miterLimit: "miterlimit miterLimit", dashes: "dash dashes dasharray dash-array dashArray", color: "color colour stroke colors colours stroke-color strokeColor", fill: "fill fill-color fillColor", opacity: "alpha opacity", overlay: "overlay crease overlap intersect", close: "closed close closed-path closePath", range: "range dataBox", viewport: "viewport viewBox", hole: "holes hole hollow", splitNull: "splitNull" }), Qe || (ge.passes[Le] = Qe = { id: Le, scale: null, scaleFract: null, translate: null, translateFract: null, count: 0, hole: [], depth: 0, dashLength: 1, dashTexture: ve.texture({ channels: 1, data: new Uint8Array([255]), width: 1, height: 1, mag: "linear", min: "linear" }), colorBuffer: ve.buffer({ usage: "dynamic", type: "uint8", data: new Uint8Array() }), positionBuffer: ve.buffer({ usage: "dynamic", type: "float", data: new Uint8Array() }), positionFractBuffer: ve.buffer({ usage: "dynamic", type: "float", data: new Uint8Array() }) }, xe = ee({}, pe.defaults, xe)), xe.thickness != null && (Qe.thickness = parseFloat(xe.thickness)), xe.opacity != null && (Qe.opacity = parseFloat(xe.opacity)), xe.miterLimit != null && (Qe.miterLimit = parseFloat(xe.miterLimit)), xe.overlay != null && (Qe.overlay = !!xe.overlay, Le < pe.maxLines && (Qe.depth = 2 * (pe.maxLines - 1 - Le % pe.maxLines) / pe.maxLines - 1)), xe.join != null && (Qe.join = xe.join), xe.hole != null && (Qe.hole = xe.hole), xe.fill != null && (Qe.fill = xe.fill ? te(xe.fill, "uint8") : null), xe.viewport != null && (Qe.viewport = he(xe.viewport)), Qe.viewport || (Qe.viewport = he([be.drawingBufferWidth, be.drawingBufferHeight])), xe.close != null && (Qe.close = xe.close), xe.positions === null && (xe.positions = []), xe.positions) {
                      var ke, Me;
                      if (xe.positions.x && xe.positions.y) {
                        var Re = xe.positions.x, De = xe.positions.y;
                        Me = Qe.count = Math.max(Re.length, De.length), ke = new Float64Array(2 * Me);
                        for (var Pe = 0; Pe < Me; Pe++) ke[2 * Pe] = Re[Pe], ke[2 * Pe + 1] = De[Pe];
                      } else ke = oe(xe.positions, "float64"), Me = Qe.count = Math.floor(ke.length / 2);
                      var $e = Qe.bounds = ne(ke, 2);
                      if (Qe.fill) {
                        for (var qe = [], Xe = {}, Ve = 0, Ze = 0, Ge = 0, ze = Qe.count; Ze < ze; Ze++) {
                          var We = ke[2 * Ze], at = ke[2 * Ze + 1];
                          isNaN(We) || isNaN(at) || We == null || at == null ? (We = ke[2 * Ve], at = ke[2 * Ve + 1], Xe[Ze] = Ve) : Ve = Ze, qe[Ge++] = We, qe[Ge++] = at;
                        }
                        if (xe.splitNull) {
                          Qe.count - 1 in Xe || (Xe[Qe.count] = Qe.count - 1);
                          var ct = Object.keys(Xe).map(Number).sort(function(Rn, Pn) {
                            return Rn - Pn;
                          }), et = [], dt = 0, Je = Qe.hole != null ? Qe.hole[0] : null;
                          if (Je != null) {
                            var it = me(ct, function(Rn) {
                              return Rn >= Je;
                            });
                            (ct = ct.slice(0, it)).push(Je);
                          }
                          for (var st = function(Rn) {
                            var Pn = qe.slice(2 * dt, 2 * ct[Rn]).concat(Je ? qe.slice(2 * Je) : []), Ie = (Qe.hole || []).map(function(ft) {
                              return ft - Je + (ct[Rn] - dt);
                            }), rt = se(Pn, Ie);
                            rt = rt.map(function(ft) {
                              return ft + dt + (ft + dt < ct[Rn] ? 0 : Je - ct[Rn]);
                            }), et.push.apply(et, rt), dt = ct[Rn] + 1;
                          }, gt = 0; gt < ct.length; gt++) st(gt);
                          for (var bt = 0, yt = et.length; bt < yt; bt++) Xe[et[bt]] != null && (et[bt] = Xe[et[bt]]);
                          Qe.triangles = et;
                        } else {
                          for (var St = se(qe, Qe.hole || []), Lt = 0, Rt = St.length; Lt < Rt; Lt++) Xe[St[Lt]] != null && (St[Lt] = Xe[St[Lt]]);
                          Qe.triangles = St;
                        }
                      }
                      var vt = new Float64Array(ke);
                      ce(vt, 2, $e);
                      var $t = new Float64Array(2 * Me + 6);
                      Qe.close ? ke[0] === ke[2 * Me - 2] && ke[1] === ke[2 * Me - 1] ? ($t[0] = vt[2 * Me - 4], $t[1] = vt[2 * Me - 3]) : ($t[0] = vt[2 * Me - 2], $t[1] = vt[2 * Me - 1]) : ($t[0] = vt[0], $t[1] = vt[1]), $t.set(vt, 2), Qe.close ? ke[0] === ke[2 * Me - 2] && ke[1] === ke[2 * Me - 1] ? ($t[2 * Me + 2] = vt[2], $t[2 * Me + 3] = vt[3], Qe.count -= 1) : ($t[2 * Me + 2] = vt[0], $t[2 * Me + 3] = vt[1], $t[2 * Me + 4] = vt[2], $t[2 * Me + 5] = vt[3]) : ($t[2 * Me + 2] = vt[2 * Me - 2], $t[2 * Me + 3] = vt[2 * Me - 1], $t[2 * Me + 4] = vt[2 * Me - 2], $t[2 * Me + 5] = vt[2 * Me - 1]);
                      var Pt = ue($t);
                      Qe.positionBuffer(Pt);
                      var Xt = de($t, Pt);
                      Qe.positionFractBuffer(Xt);
                    }
                    if (xe.range ? Qe.range = xe.range : Qe.range || (Qe.range = Qe.bounds), (xe.range || xe.positions) && Qe.count) {
                      var Nt = Qe.bounds, Gt = Nt[2] - Nt[0], Ft = Nt[3] - Nt[1], Tt = Qe.range[2] - Qe.range[0], It = Qe.range[3] - Qe.range[1];
                      Qe.scale = [Gt / Tt, Ft / It], Qe.translate = [-Qe.range[0] / Tt + Nt[0] / Tt || 0, -Qe.range[1] / It + Nt[1] / It || 0], Qe.scaleFract = de(Qe.scale), Qe.translateFract = de(Qe.translate);
                    }
                    if (xe.dashes) {
                      var Ct, Kt = 0;
                      if (!xe.dashes || xe.dashes.length < 2) Kt = 1, Ct = new Uint8Array([255, 255, 255, 255, 255, 255, 255, 255]);
                      else {
                        Kt = 0;
                        for (var Vt = 0; Vt < xe.dashes.length; ++Vt) Kt += xe.dashes[Vt];
                        Ct = new Uint8Array(Kt * pe.dashMult);
                        for (var Yt = 0, sn = 255, hn = 0; hn < 2; hn++) for (var Qt = 0; Qt < xe.dashes.length; ++Qt) {
                          for (var Ut = 0, Wt = xe.dashes[Qt] * pe.dashMult * 0.5; Ut < Wt; ++Ut) Ct[Yt++] = sn;
                          sn ^= 255;
                        }
                      }
                      Qe.dashLength = Kt, Qe.dashTexture({ channels: 1, data: Ct, width: Ct.length, height: 1, mag: "linear", min: "linear" }, 0, 0);
                    }
                    if (xe.color) {
                      var rn = Qe.count, vn = xe.color;
                      vn || (vn = "transparent");
                      var dn = new Uint8Array(4 * rn + 4);
                      if (Array.isArray(vn) && typeof vn[0] != "number") {
                        for (var Ht = 0; Ht < rn; Ht++) {
                          var jt = te(vn[Ht], "uint8");
                          dn.set(jt, 4 * Ht);
                        }
                        dn.set(te(vn[0], "uint8"), 4 * rn);
                      } else for (var cn = te(vn, "uint8"), wn = 0; wn < rn + 1; wn++) dn.set(cn, 4 * wn);
                      Qe.colorBuffer({ usage: "dynamic", type: "uint8", data: dn });
                    }
                  } else ge.passes[Le] = null;
                }), ye.length < this.passes.length) {
                  for (var Ce = ye.length; Ce < this.passes.length; Ce++) {
                    var Se = this.passes[Ce];
                    Se && (Se.colorBuffer.destroy(), Se.positionBuffer.destroy(), Se.dashTexture.destroy());
                  }
                  this.passes.length = ye.length;
                }
                for (var we = [], Ee = 0; Ee < this.passes.length; Ee++) this.passes[Ee] !== null && we.push(this.passes[Ee]);
                return this.passes = we, this;
              }
            }, pe.prototype.destroy = function() {
              return this.passes.forEach(function(ye) {
                ye.colorBuffer.destroy(), ye.positionBuffer.destroy(), ye.dashTexture.destroy();
              }), this.passes.length = 0, this;
            };
          }, 62172: function(G, J, Z) {
            function te(Ce, Se) {
              return function(we) {
                if (Array.isArray(we)) return we;
              }(Ce) || function(we, Ee) {
                var xe = we == null ? null : typeof Symbol < "u" && we[Symbol.iterator] || we["@@iterator"];
                if (xe != null) {
                  var Le, Qe, ke, Me, Re = [], De = !0, Pe = !1;
                  try {
                    if (ke = (xe = xe.call(we)).next, Ee !== 0) for (; !(De = (Le = ke.call(xe)).done) && (Re.push(Le.value), Re.length !== Ee); De = !0) ;
                  } catch ($e) {
                    Pe = !0, Qe = $e;
                  } finally {
                    try {
                      if (!De && xe.return != null && (Me = xe.return(), Object(Me) !== Me)) return;
                    } finally {
                      if (Pe) throw Qe;
                    }
                  }
                  return Re;
                }
              }(Ce, Se) || ne(Ce, Se) || function() {
                throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
              }();
            }
            function ne(Ce, Se) {
              if (Ce) {
                if (typeof Ce == "string") return ee(Ce, Se);
                var we = Object.prototype.toString.call(Ce).slice(8, -1);
                return we === "Object" && Ce.constructor && (we = Ce.constructor.name), we === "Map" || we === "Set" ? Array.from(Ce) : we === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(we) ? ee(Ce, Se) : void 0;
              }
            }
            function ee(Ce, Se) {
              (Se == null || Se > Ce.length) && (Se = Ce.length);
              for (var we = 0, Ee = new Array(Se); we < Se; we++) Ee[we] = Ce[we];
              return Ee;
            }
            var ae = Z(162), oe = Z(78112), se = Z(46762), ce = Z(27549), le = Z(27976), ue = Z(76765), de = Z(6807), fe = Z(79788), he = Z(83473), me = Z(39488), pe = Z(51498), ye = Z(5137), ge = ve;
            function ve(Ce, Se) {
              var we = this;
              if (!(this instanceof ve)) return new ve(Ce, Se);
              typeof Ce == "function" ? (Se || (Se = {}), Se.regl = Ce) : (Se = Ce, Ce = null), Se && Se.length && (Se.positions = Se);
              var Ee, xe = (Ce = Se.regl)._gl, Le = [];
              this.tooManyColors = me, Ee = Ce.texture({ data: new Uint8Array(1020), width: 255, height: 1, type: "uint8", format: "rgba", wrapS: "clamp", wrapT: "clamp", mag: "nearest", min: "nearest" }), le(this, { regl: Ce, gl: xe, groups: [], markerCache: [null], markerTextures: [null], palette: Le, paletteIds: {}, paletteTexture: Ee, maxColors: 255, maxSize: 100, canvas: xe.canvas }), this.update(Se);
              var Qe = { uniforms: { constPointSize: !!Se.constPointSize, opacity: Ce.prop("opacity"), paletteSize: function(Re, De) {
                return [we.tooManyColors ? 0 : 255, Ee.height];
              }, pixelRatio: Ce.context("pixelRatio"), scale: Ce.prop("scale"), scaleFract: Ce.prop("scaleFract"), translate: Ce.prop("translate"), translateFract: Ce.prop("translateFract"), markerTexture: Ce.prop("markerTexture"), paletteTexture: Ee }, attributes: { x: function(Re, De) {
                return De.xAttr || { buffer: De.positionBuffer, stride: 8, offset: 0 };
              }, y: function(Re, De) {
                return De.yAttr || { buffer: De.positionBuffer, stride: 8, offset: 4 };
              }, xFract: function(Re, De) {
                return De.xAttr ? { constant: [0, 0] } : { buffer: De.positionFractBuffer, stride: 8, offset: 0 };
              }, yFract: function(Re, De) {
                return De.yAttr ? { constant: [0, 0] } : { buffer: De.positionFractBuffer, stride: 8, offset: 4 };
              }, size: function(Re, De) {
                return De.size.length ? { buffer: De.sizeBuffer, stride: 2, offset: 0 } : { constant: [Math.round(255 * De.size / we.maxSize)] };
              }, borderSize: function(Re, De) {
                return De.borderSize.length ? { buffer: De.sizeBuffer, stride: 2, offset: 1 } : { constant: [Math.round(255 * De.borderSize / we.maxSize)] };
              }, colorId: function(Re, De) {
                return De.color.length ? { buffer: De.colorBuffer, stride: we.tooManyColors ? 8 : 4, offset: 0 } : { constant: we.tooManyColors ? Le.slice(4 * De.color, 4 * De.color + 4) : [De.color] };
              }, borderColorId: function(Re, De) {
                return De.borderColor.length ? { buffer: De.colorBuffer, stride: we.tooManyColors ? 8 : 4, offset: we.tooManyColors ? 4 : 2 } : { constant: we.tooManyColors ? Le.slice(4 * De.borderColor, 4 * De.borderColor + 4) : [De.borderColor] };
              }, isActive: function(Re, De) {
                return De.activation === !0 ? { constant: [1] } : De.activation ? De.activation : { constant: [0] };
              } }, blend: { enable: !0, color: [0, 0, 0, 1], func: { srcRGB: "src alpha", dstRGB: "one minus src alpha", srcAlpha: "one minus dst alpha", dstAlpha: "one" } }, scissor: { enable: !0, box: Ce.prop("viewport") }, viewport: Ce.prop("viewport"), stencil: { enable: !1 }, depth: { enable: !1 }, elements: Ce.prop("elements"), count: Ce.prop("count"), offset: Ce.prop("offset"), primitive: "points" }, ke = le({}, Qe);
              ke.frag = ue([`precision highp float;
#define GLSLIFY 1

uniform float opacity;
uniform sampler2D markerTexture;

varying vec4 fragColor, fragBorderColor;
varying float fragWidth, fragBorderColorLevel, fragColorLevel;

float smoothStep(float x, float y) {
  return 1.0 / (1.0 + exp(50.0*(x - y)));
}

void main() {
  float dist = texture2D(markerTexture, gl_PointCoord).r, delta = fragWidth;

  // max-distance alpha
  if (dist < 0.003) discard;

  // null-border case
  if (fragBorderColorLevel == fragColorLevel || fragBorderColor.a == 0.) {
    float colorAmt = smoothstep(.5 - delta, .5 + delta, dist);
    gl_FragColor = vec4(fragColor.rgb, colorAmt * fragColor.a * opacity);
  }
  else {
    float borderColorAmt = smoothstep(fragBorderColorLevel - delta, fragBorderColorLevel + delta, dist);
    float colorAmt = smoothstep(fragColorLevel - delta, fragColorLevel + delta, dist);

    vec4 color = fragBorderColor;
    color.a *= borderColorAmt;
    color = mix(color, fragColor, colorAmt);
    color.a *= opacity;

    gl_FragColor = color;
  }

}
`]), ke.vert = ue([`precision highp float;
#define GLSLIFY 1

attribute float x, y, xFract, yFract;
attribute float size, borderSize;
attribute vec4 colorId, borderColorId;
attribute float isActive;

// \`invariant\` effectively turns off optimizations for the position.
// We need this because -fast-math on M1 Macs is re-ordering
// floating point operations in a way that causes floating point
// precision limits to put points in the wrong locations.
invariant gl_Position;

uniform bool constPointSize;
uniform float pixelRatio;
uniform vec2 scale, scaleFract, translate, translateFract, paletteSize;
uniform sampler2D paletteTexture;

const float maxSize = 100.;
const float borderLevel = .5;

varying vec4 fragColor, fragBorderColor;
varying float fragPointSize, fragBorderRadius, fragWidth, fragBorderColorLevel, fragColorLevel;

float pointSizeScale = (constPointSize) ? 2. : pixelRatio;

bool isDirect = (paletteSize.x < 1.);

vec4 getColor(vec4 id) {
  return isDirect ? id / 255. : texture2D(paletteTexture,
    vec2(
      (id.x + .5) / paletteSize.x,
      (id.y + .5) / paletteSize.y
    )
  );
}

void main() {
  // ignore inactive points
  if (isActive == 0.) return;

  vec2 position = vec2(x, y);
  vec2 positionFract = vec2(xFract, yFract);

  vec4 color = getColor(colorId);
  vec4 borderColor = getColor(borderColorId);

  float size = size * maxSize / 255.;
  float borderSize = borderSize * maxSize / 255.;

  gl_PointSize = 2. * size * pointSizeScale;
  fragPointSize = size * pixelRatio;

  vec2 pos = (position + translate) * scale
      + (positionFract + translateFract) * scale
      + (position + translate) * scaleFract
      + (positionFract + translateFract) * scaleFract;

  gl_Position = vec4(pos * 2. - 1., 0., 1.);

  fragColor = color;
  fragBorderColor = borderColor;
  fragWidth = 1. / gl_PointSize;

  fragBorderColorLevel = clamp(borderLevel - borderLevel * borderSize / size, 0., 1.);
  fragColorLevel = clamp(borderLevel + (1. - borderLevel) * borderSize / size, 0., 1.);
}
`]), this.drawMarker = Ce(ke);
              var Me = le({}, Qe);
              Me.frag = ue([`precision highp float;
#define GLSLIFY 1

varying vec4 fragColor, fragBorderColor;
varying float fragBorderRadius, fragWidth;

uniform float opacity;

float smoothStep(float edge0, float edge1, float x) {
	float t;
	t = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
	return t * t * (3.0 - 2.0 * t);
}

void main() {
	float radius, alpha = 1.0, delta = fragWidth;

	radius = length(2.0 * gl_PointCoord.xy - 1.0);

	if (radius > 1.0 + delta) {
		discard;
	}

	alpha -= smoothstep(1.0 - delta, 1.0 + delta, radius);

	float borderRadius = fragBorderRadius;
	float ratio = smoothstep(borderRadius - delta, borderRadius + delta, radius);
	vec4 color = mix(fragColor, fragBorderColor, ratio);
	color.a *= alpha * opacity;
	gl_FragColor = color;
}
`]), Me.vert = ue([`precision highp float;
#define GLSLIFY 1

attribute float x, y, xFract, yFract;
attribute float size, borderSize;
attribute vec4 colorId, borderColorId;
attribute float isActive;

// \`invariant\` effectively turns off optimizations for the position.
// We need this because -fast-math on M1 Macs is re-ordering
// floating point operations in a way that causes floating point
// precision limits to put points in the wrong locations.
invariant gl_Position;

uniform bool constPointSize;
uniform float pixelRatio;
uniform vec2 paletteSize, scale, scaleFract, translate, translateFract;
uniform sampler2D paletteTexture;

const float maxSize = 100.;

varying vec4 fragColor, fragBorderColor;
varying float fragBorderRadius, fragWidth;

float pointSizeScale = (constPointSize) ? 2. : pixelRatio;

bool isDirect = (paletteSize.x < 1.);

vec4 getColor(vec4 id) {
  return isDirect ? id / 255. : texture2D(paletteTexture,
    vec2(
      (id.x + .5) / paletteSize.x,
      (id.y + .5) / paletteSize.y
    )
  );
}

void main() {
  // ignore inactive points
  if (isActive == 0.) return;

  vec2 position = vec2(x, y);
  vec2 positionFract = vec2(xFract, yFract);

  vec4 color = getColor(colorId);
  vec4 borderColor = getColor(borderColorId);

  float size = size * maxSize / 255.;
  float borderSize = borderSize * maxSize / 255.;

  gl_PointSize = (size + borderSize) * pointSizeScale;

  vec2 pos = (position + translate) * scale
      + (positionFract + translateFract) * scale
      + (position + translate) * scaleFract
      + (positionFract + translateFract) * scaleFract;

  gl_Position = vec4(pos * 2. - 1., 0., 1.);

  fragBorderRadius = 1. - 2. * borderSize / (size + borderSize);
  fragColor = color;
  fragBorderColor = borderColor.a == 0. || borderSize == 0. ? vec4(color.rgb, 0.) : borderColor;
  fragWidth = 1. / gl_PointSize;
}
`]), me && (Me.frag = Me.frag.replace("smoothstep", "smoothStep"), ke.frag = ke.frag.replace("smoothstep", "smoothStep")), this.drawCircle = Ce(Me);
            }
            ve.defaults = { color: "black", borderColor: "transparent", borderSize: 0, size: 12, opacity: 1, marker: void 0, viewport: null, range: null, pixelSize: null, count: 0, offset: 0, bounds: null, positions: [], snap: 1e4 }, ve.prototype.render = function() {
              return arguments.length && this.update.apply(this, arguments), this.draw(), this;
            }, ve.prototype.draw = function() {
              for (var Ce = this, Se = arguments.length, we = new Array(Se), Ee = 0; Ee < Se; Ee++) we[Ee] = arguments[Ee];
              var xe = this.groups;
              if (we.length === 1 && Array.isArray(we[0]) && (we[0][0] === null || Array.isArray(we[0][0])) && (we = we[0]), this.regl._refresh(), we.length) for (var Le = 0; Le < we.length; Le++) this.drawItem(Le, we[Le]);
              else xe.forEach(function(Qe, ke) {
                Ce.drawItem(ke);
              });
              return this;
            }, ve.prototype.drawItem = function(Ce, Se) {
              var we, Ee = this.groups, xe = Ee[Ce];
              if (typeof Se == "number" && (Ce = Se, xe = Ee[Se], Se = null), xe && xe.count && xe.opacity) {
                xe.activation[0] && this.drawCircle(this.getMarkerDrawOptions(0, xe, Se));
                for (var Le = [], Qe = 1; Qe < xe.activation.length; Qe++) xe.activation[Qe] && (xe.activation[Qe] === !0 || xe.activation[Qe].data.length) && Le.push.apply(Le, function(ke) {
                  if (Array.isArray(ke)) return ee(ke);
                }(we = this.getMarkerDrawOptions(Qe, xe, Se)) || function(ke) {
                  if (typeof Symbol < "u" && ke[Symbol.iterator] != null || ke["@@iterator"] != null) return Array.from(ke);
                }(we) || ne(we) || function() {
                  throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
                }());
                Le.length && this.drawMarker(Le);
              }
            }, ve.prototype.getMarkerDrawOptions = function(Ce, Se, we) {
              var Ee = Se.range, xe = Se.tree, Le = Se.viewport, Qe = Se.activation, ke = Se.selectionBuffer, Me = Se.count;
              if (this.regl, !xe) return we ? [le({}, Se, { markerTexture: this.markerTextures[Ce], activation: Qe[Ce], count: we.length, elements: we, offset: 0 })] : [le({}, Se, { markerTexture: this.markerTextures[Ce], activation: Qe[Ce], offset: 0 })];
              var Re = [], De = xe.range(Ee, { lod: !0, px: [(Ee[2] - Ee[0]) / Le.width, (Ee[3] - Ee[1]) / Le.height] });
              if (we) {
                for (var Pe = Qe[Ce].data, $e = new Uint8Array(Me), qe = 0; qe < we.length; qe++) {
                  var Xe = we[qe];
                  $e[Xe] = Pe ? Pe[Xe] : 1;
                }
                ke.subdata($e);
              }
              for (var Ve = De.length; Ve--; ) {
                var Ze = te(De[Ve], 2), Ge = Ze[0], ze = Ze[1];
                Re.push(le({}, Se, { markerTexture: this.markerTextures[Ce], activation: we ? ke : Qe[Ce], offset: Ge, count: ze - Ge }));
              }
              return Re;
            }, ve.prototype.update = function() {
              for (var Ce = this, Se = arguments.length, we = new Array(Se), Ee = 0; Ee < Se; Ee++) we[Ee] = arguments[Ee];
              if (we.length) {
                we.length === 1 && Array.isArray(we[0]) && (we = we[0]);
                var xe = this.groups, Le = this.gl, Qe = this.regl, ke = this.maxSize, Me = this.maxColors, Re = this.palette;
                this.groups = xe = we.map(function(De, Pe) {
                  var $e = xe[Pe];
                  if (De === void 0) return $e;
                  De === null ? De = { positions: null } : typeof De == "function" ? De = { ondraw: De } : typeof De[0] == "number" && (De = { positions: De }), (De = de(De, { positions: "positions data points", snap: "snap cluster lod tree", size: "sizes size radius", borderSize: "borderSizes borderSize border-size bordersize borderWidth borderWidths border-width borderwidth stroke-width strokeWidth strokewidth outline", color: "colors color fill fill-color fillColor", borderColor: "borderColors borderColor stroke stroke-color strokeColor", marker: "markers marker shape", range: "range dataBox databox", viewport: "viewport viewPort viewBox viewbox", opacity: "opacity alpha transparency", bounds: "bound bounds boundaries limits", tooManyColors: "tooManyColors palette paletteMode optimizePalette enablePalette" })).positions === null && (De.positions = []), De.tooManyColors != null && (Ce.tooManyColors = De.tooManyColors), $e || (xe[Pe] = $e = { id: Pe, scale: null, translate: null, scaleFract: null, translateFract: null, activation: [], selectionBuffer: Qe.buffer({ data: new Uint8Array(0), usage: "stream", type: "uint8" }), sizeBuffer: Qe.buffer({ data: new Uint8Array(0), usage: "dynamic", type: "uint8" }), colorBuffer: Qe.buffer({ data: new Uint8Array(0), usage: "dynamic", type: "uint8" }), positionBuffer: Qe.buffer({ data: new Uint8Array(0), usage: "dynamic", type: "float" }), positionFractBuffer: Qe.buffer({ data: new Uint8Array(0), usage: "dynamic", type: "float" }) }, De = le({}, ve.defaults, De)), De.positions && !("marker" in De) && (De.marker = $e.marker, delete $e.marker), De.marker && !("positions" in De) && (De.positions = $e.positions, delete $e.positions);
                  var qe = 0, Xe = 0;
                  if (fe($e, De, [{ snap: !0, size: function(Rt, vt) {
                    return Rt == null && (Rt = ve.defaults.size), qe += Rt && Rt.length ? 1 : 0, Rt;
                  }, borderSize: function(Rt, vt) {
                    return Rt == null && (Rt = ve.defaults.borderSize), qe += Rt && Rt.length ? 1 : 0, Rt;
                  }, opacity: parseFloat, color: function(Rt, vt) {
                    return Rt == null && (Rt = ve.defaults.color), Rt = Ce.updateColor(Rt), Xe++, Rt;
                  }, borderColor: function(Rt, vt) {
                    return Rt == null && (Rt = ve.defaults.borderColor), Rt = Ce.updateColor(Rt), Xe++, Rt;
                  }, bounds: function(Rt, vt, $t) {
                    return "range" in $t || ($t.range = null), Rt;
                  }, positions: function(Rt, vt, $t) {
                    var Pt = vt.snap, Xt = vt.positionBuffer, Nt = vt.positionFractBuffer, Gt = vt.selectionBuffer;
                    if (Rt.x || Rt.y) return Rt.x.length ? vt.xAttr = { buffer: Qe.buffer(Rt.x), offset: 0, stride: 4, count: Rt.x.length } : vt.xAttr = { buffer: Rt.x.buffer, offset: 4 * Rt.x.offset || 0, stride: 4 * (Rt.x.stride || 1), count: Rt.x.count }, Rt.y.length ? vt.yAttr = { buffer: Qe.buffer(Rt.y), offset: 0, stride: 4, count: Rt.y.length } : vt.yAttr = { buffer: Rt.y.buffer, offset: 4 * Rt.y.offset || 0, stride: 4 * (Rt.y.stride || 1), count: Rt.y.count }, vt.count = Math.max(vt.xAttr.count, vt.yAttr.count), Rt;
                    Rt = he(Rt, "float64");
                    var Ft = vt.count = Math.floor(Rt.length / 2), Tt = vt.bounds = Ft ? oe(Rt, 2) : null;
                    if ($t.range || vt.range || (delete vt.range, $t.range = Tt), $t.marker || vt.marker || (delete vt.marker, $t.marker = null), Pt && (Pt === !0 || Ft > Pt) ? vt.tree = ce(Rt, { bounds: Tt }) : Pt && Pt.length && (vt.tree = Pt), vt.tree) {
                      var It = { primitive: "points", usage: "static", data: vt.tree, type: "uint32" };
                      vt.elements ? vt.elements(It) : vt.elements = Qe.elements(It);
                    }
                    var Ct = pe.float32(Rt);
                    return Xt({ data: Ct, usage: "dynamic" }), Nt({ data: pe.fract32(Rt, Ct), usage: "dynamic" }), Gt({ data: new Uint8Array(Ft), type: "uint8", usage: "stream" }), Rt;
                  } }, { marker: function(Rt, vt, $t) {
                    var Pt = vt.activation;
                    if (Pt.forEach(function(Ct) {
                      return Ct && Ct.destroy && Ct.destroy();
                    }), Pt.length = 0, Rt && typeof Rt[0] != "number") {
                      for (var Xt = [], Nt = 0, Gt = Math.min(Rt.length, vt.count); Nt < Gt; Nt++) {
                        var Ft = Ce.addMarker(Rt[Nt]);
                        Xt[Ft] || (Xt[Ft] = new Uint8Array(vt.count)), Xt[Ft][Nt] = 1;
                      }
                      for (var Tt = 0; Tt < Xt.length; Tt++) if (Xt[Tt]) {
                        var It = { data: Xt[Tt], type: "uint8", usage: "static" };
                        Pt[Tt] ? Pt[Tt](It) : Pt[Tt] = Qe.buffer(It), Pt[Tt].data = Xt[Tt];
                      }
                    } else Pt[Ce.addMarker(Rt)] = !0;
                    return Rt;
                  }, range: function(Rt, vt, $t) {
                    var Pt = vt.bounds;
                    if (Pt) return Rt || (Rt = Pt), vt.scale = [1 / (Rt[2] - Rt[0]), 1 / (Rt[3] - Rt[1])], vt.translate = [-Rt[0], -Rt[1]], vt.scaleFract = pe.fract(vt.scale), vt.translateFract = pe.fract(vt.translate), Rt;
                  }, viewport: function(Rt) {
                    return ye(Rt || [Le.drawingBufferWidth, Le.drawingBufferHeight]);
                  } }]), qe) {
                    var Ve = $e, Ze = Ve.count, Ge = Ve.size, ze = Ve.borderSize, We = Ve.sizeBuffer, at = new Uint8Array(2 * Ze);
                    if (Ge.length || ze.length) for (var ct = 0; ct < Ze; ct++) at[2 * ct] = Math.round(255 * (Ge[ct] == null ? Ge : Ge[ct]) / ke), at[2 * ct + 1] = Math.round(255 * (ze[ct] == null ? ze : ze[ct]) / ke);
                    We({ data: at, usage: "dynamic" });
                  }
                  if (Xe) {
                    var et, dt = $e, Je = dt.count, it = dt.color, st = dt.borderColor, gt = dt.colorBuffer;
                    if (Ce.tooManyColors) {
                      if (it.length || st.length) {
                        et = new Uint8Array(8 * Je);
                        for (var bt = 0; bt < Je; bt++) {
                          var yt = it[bt];
                          et[8 * bt] = Re[4 * yt], et[8 * bt + 1] = Re[4 * yt + 1], et[8 * bt + 2] = Re[4 * yt + 2], et[8 * bt + 3] = Re[4 * yt + 3];
                          var St = st[bt];
                          et[8 * bt + 4] = Re[4 * St], et[8 * bt + 5] = Re[4 * St + 1], et[8 * bt + 6] = Re[4 * St + 2], et[8 * bt + 7] = Re[4 * St + 3];
                        }
                      }
                    } else if (it.length || st.length) {
                      et = new Uint8Array(4 * Je + 2);
                      for (var Lt = 0; Lt < Je; Lt++) it[Lt] != null && (et[4 * Lt] = it[Lt] % Me, et[4 * Lt + 1] = Math.floor(it[Lt] / Me)), st[Lt] != null && (et[4 * Lt + 2] = st[Lt] % Me, et[4 * Lt + 3] = Math.floor(st[Lt] / Me));
                    }
                    gt({ data: et || new Uint8Array(0), type: "uint8", usage: "dynamic" });
                  }
                  return $e;
                });
              }
            }, ve.prototype.addMarker = function(Ce) {
              var Se, we = this.markerTextures, Ee = this.regl, xe = this.markerCache, Le = Ce == null ? 0 : xe.indexOf(Ce);
              if (Le >= 0) return Le;
              if (Ce instanceof Uint8Array || Ce instanceof Uint8ClampedArray) Se = Ce;
              else {
                Se = new Uint8Array(Ce.length);
                for (var Qe = 0, ke = Ce.length; Qe < ke; Qe++) Se[Qe] = 255 * Ce[Qe];
              }
              var Me = Math.floor(Math.sqrt(Se.length));
              return Le = we.length, xe.push(Ce), we.push(Ee.texture({ channels: 1, data: Se, radius: Me, mag: "linear", min: "linear" })), Le;
            }, ve.prototype.updateColor = function(Ce) {
              var Se = this.paletteIds, we = this.palette, Ee = this.maxColors;
              Array.isArray(Ce) || (Ce = [Ce]);
              var xe = [];
              if (typeof Ce[0] == "number") {
                var Le = [];
                if (Array.isArray(Ce)) for (var Qe = 0; Qe < Ce.length; Qe += 4) Le.push(Ce.slice(Qe, Qe + 4));
                else for (var ke = 0; ke < Ce.length; ke += 4) Le.push(Ce.subarray(ke, ke + 4));
                Ce = Le;
              }
              for (var Me = 0; Me < Ce.length; Me++) {
                var Re = Ce[Me];
                Re = ae(Re, "uint8");
                var De = se(Re, !1);
                if (Se[De] == null) {
                  var Pe = we.length;
                  Se[De] = Math.floor(Pe / 4), we[Pe] = Re[0], we[Pe + 1] = Re[1], we[Pe + 2] = Re[2], we[Pe + 3] = Re[3];
                }
                xe[Me] = Se[De];
              }
              return !this.tooManyColors && we.length > 4 * Ee && (this.tooManyColors = !0), this.updatePalette(we), xe.length === 1 ? xe[0] : xe;
            }, ve.prototype.updatePalette = function(Ce) {
              if (!this.tooManyColors) {
                var Se = this.maxColors, we = this.paletteTexture, Ee = Math.ceil(0.25 * Ce.length / Se);
                if (Ee > 1) for (var xe = 0.25 * (Ce = Ce.slice()).length % Se; xe < Ee * Se; xe++) Ce.push(0, 0, 0, 0);
                we.height < Ee && we.resize(Se, Ee), we.subimage({ width: Math.min(0.25 * Ce.length, Se), height: Ee, data: Ce }, 0, 0);
              }
            }, ve.prototype.destroy = function() {
              return this.groups.forEach(function(Ce) {
                Ce.sizeBuffer.destroy(), Ce.positionBuffer.destroy(), Ce.positionFractBuffer.destroy(), Ce.colorBuffer.destroy(), Ce.activation.forEach(function(Se) {
                  return Se && Se.destroy && Se.destroy();
                }), Ce.selectionBuffer.destroy(), Ce.elements && Ce.elements.destroy();
              }), this.groups.length = 0, this.paletteTexture.destroy(), this.markerTextures.forEach(function(Ce) {
                return Ce && Ce.destroy && Ce.destroy();
              }), this;
            };
            var be = Z(27976);
            G.exports = function(Ce, Se) {
              var we = new ge(Ce, Se), Ee = we.render.bind(we);
              return be(Ee, { render: Ee, update: we.update.bind(we), draw: we.draw.bind(we), destroy: we.destroy.bind(we), regl: we.regl, gl: we.gl, canvas: we.gl.canvas, groups: we.groups, markers: we.markerCache, palette: we.palette }), Ee;
            };
          }, 31239: function(G, J, Z) {
            var te = Z(62172), ne = Z(6807), ee = Z(78112), ae = Z(16494), oe = Z(27902), se = Z(5137), ce = Z(83473);
            function le(he, me) {
              if (!(this instanceof le)) return new le(he);
              this.traces = [], this.passes = {}, this.regl = he, this.scatter = te(he), this.canvas = this.scatter.canvas;
            }
            function ue(he, me, pe) {
              return (he.id != null ? he.id : he) << 16 | (255 & me) << 8 | 255 & pe;
            }
            function de(he, me, pe) {
              var ye, ge, ve, be, Ce = he[me], Se = he[pe];
              return Ce.length > 2 ? (Ce[0], Ce[2], ye = Ce[1], ge = Ce[3]) : Ce.length ? (ye = Ce[0], ge = Ce[1]) : (Ce.x, ye = Ce.y, Ce.x, Ce.width, ge = Ce.y + Ce.height), Se.length > 2 ? (ve = Se[0], be = Se[2], Se[1], Se[3]) : Se.length ? (ve = Se[0], be = Se[1]) : (ve = Se.x, Se.y, be = Se.x + Se.width, Se.y, Se.height), [ve, ye, be, ge];
            }
            function fe(he) {
              if (typeof he == "number") return [he, he, he, he];
              if (he.length === 2) return [he[0], he[1], he[0], he[1]];
              var me = se(he);
              return [me.x, me.y, me.x + me.width, me.y + me.height];
            }
            G.exports = le, le.prototype.render = function() {
              for (var he, me = this, pe = [], ye = arguments.length; ye--; ) pe[ye] = arguments[ye];
              return pe.length && (he = this).update.apply(he, pe), this.regl.attributes.preserveDrawingBuffer ? this.draw() : (this.dirty ? this.planned == null && (this.planned = ae(function() {
                me.draw(), me.dirty = !0, me.planned = null;
              })) : (this.draw(), this.dirty = !0, ae(function() {
                me.dirty = !1;
              })), this);
            }, le.prototype.update = function() {
              for (var he, me = [], pe = arguments.length; pe--; ) me[pe] = arguments[pe];
              if (me.length) {
                for (var ye = 0; ye < me.length; ye++) this.updateItem(ye, me[ye]);
                this.traces = this.traces.filter(Boolean);
                for (var ge = [], ve = 0, be = 0; be < this.traces.length; be++) {
                  for (var Ce = this.traces[be], Se = this.traces[be].passes, we = 0; we < Se.length; we++) ge.push(this.passes[Se[we]]);
                  Ce.passOffset = ve, ve += Ce.passes.length;
                }
                return (he = this.scatter).update.apply(he, ge), this;
              }
            }, le.prototype.updateItem = function(he, me) {
              var pe = this.regl;
              if (me === null) return this.traces[he] = null, this;
              if (!me) return this;
              var ye, ge = ne(me, { data: "data items columns rows values dimensions samples x", snap: "snap cluster", size: "sizes size radius", color: "colors color fill fill-color fillColor", opacity: "opacity alpha transparency opaque", borderSize: "borderSizes borderSize border-size bordersize borderWidth borderWidths border-width borderwidth stroke-width strokeWidth strokewidth outline", borderColor: "borderColors borderColor bordercolor stroke stroke-color strokeColor", marker: "markers marker shape", range: "range ranges databox dataBox", viewport: "viewport viewBox viewbox", domain: "domain domains area areas", padding: "pad padding paddings pads margin margins", transpose: "transpose transposed", diagonal: "diagonal diag showDiagonal", upper: "upper up top upperhalf upperHalf showupperhalf showUpper showUpperHalf", lower: "lower low bottom lowerhalf lowerHalf showlowerhalf showLowerHalf showLower" }), ve = this.traces[he] || (this.traces[he] = { id: he, buffer: pe.buffer({ usage: "dynamic", type: "float", data: new Uint8Array() }), color: "black", marker: null, size: 12, borderColor: "transparent", borderSize: 1, viewport: se([pe._gl.drawingBufferWidth, pe._gl.drawingBufferHeight]), padding: [0, 0, 0, 0], opacity: 1, diagonal: !0, upper: !0, lower: !0 });
              if (ge.color != null && (ve.color = ge.color), ge.size != null && (ve.size = ge.size), ge.marker != null && (ve.marker = ge.marker), ge.borderColor != null && (ve.borderColor = ge.borderColor), ge.borderSize != null && (ve.borderSize = ge.borderSize), ge.opacity != null && (ve.opacity = ge.opacity), ge.viewport && (ve.viewport = se(ge.viewport)), ge.diagonal != null && (ve.diagonal = ge.diagonal), ge.upper != null && (ve.upper = ge.upper), ge.lower != null && (ve.lower = ge.lower), ge.data) {
                ve.buffer(ce(ge.data)), ve.columns = ge.data.length, ve.count = ge.data[0].length, ve.bounds = [];
                for (var be = 0; be < ve.columns; be++) ve.bounds[be] = ee(ge.data[be], 1);
              }
              ge.range && (ve.range = ge.range, ye = ve.range && typeof ve.range[0] != "number"), ge.domain && (ve.domain = ge.domain);
              var Ce = !1;
              ge.padding != null && (Array.isArray(ge.padding) && ge.padding.length === ve.columns && typeof ge.padding[ge.padding.length - 1] == "number" ? (ve.padding = ge.padding.map(fe), Ce = !0) : ve.padding = fe(ge.padding));
              var Se = ve.columns, we = ve.count, Ee = ve.viewport.width, xe = ve.viewport.height, Le = ve.viewport.x, Qe = ve.viewport.y, ke = Ee / Se, Me = xe / Se;
              ve.passes = [];
              for (var Re = 0; Re < Se; Re++) for (var De = 0; De < Se; De++) if ((ve.diagonal || De !== Re) && (ve.upper || !(Re > De)) && (ve.lower || !(Re < De))) {
                var Pe = ue(ve.id, Re, De), $e = this.passes[Pe] || (this.passes[Pe] = {});
                if (ge.data && (ge.transpose ? $e.positions = { x: { buffer: ve.buffer, offset: De, count: we, stride: Se }, y: { buffer: ve.buffer, offset: Re, count: we, stride: Se } } : $e.positions = { x: { buffer: ve.buffer, offset: De * we, count: we }, y: { buffer: ve.buffer, offset: Re * we, count: we } }, $e.bounds = de(ve.bounds, Re, De)), ge.domain || ge.viewport || ge.data) {
                  var qe = Ce ? de(ve.padding, Re, De) : ve.padding;
                  if (ve.domain) {
                    var Xe = de(ve.domain, Re, De), Ve = Xe[0], Ze = Xe[1], Ge = Xe[2], ze = Xe[3];
                    $e.viewport = [Le + Ve * Ee + qe[0], Qe + Ze * xe + qe[1], Le + Ge * Ee - qe[2], Qe + ze * xe - qe[3]];
                  } else $e.viewport = [Le + De * ke + ke * qe[0], Qe + Re * Me + Me * qe[1], Le + (De + 1) * ke - ke * qe[2], Qe + (Re + 1) * Me - Me * qe[3]];
                }
                ge.color && ($e.color = ve.color), ge.size && ($e.size = ve.size), ge.marker && ($e.marker = ve.marker), ge.borderSize && ($e.borderSize = ve.borderSize), ge.borderColor && ($e.borderColor = ve.borderColor), ge.opacity && ($e.opacity = ve.opacity), ge.range && ($e.range = ye ? de(ve.range, Re, De) : ve.range || $e.bounds), ve.passes.push(Pe);
              }
              return this;
            }, le.prototype.draw = function() {
              for (var he, me = [], pe = arguments.length; pe--; ) me[pe] = arguments[pe];
              if (me.length) {
                for (var ye = [], ge = 0; ge < me.length; ge++) if (typeof me[ge] == "number") {
                  var ve = this.traces[me[ge]], be = ve.passes, Ce = ve.passOffset;
                  ye.push.apply(ye, oe(Ce, Ce + be.length));
                } else if (me[ge].length) {
                  var Se = me[ge], we = this.traces[ge], Ee = we.passes, xe = we.passOffset;
                  Ee = Ee.map(function(Le, Qe) {
                    ye[xe + Qe] = Se;
                  });
                }
                (he = this.scatter).draw.apply(he, ye);
              } else this.scatter.draw();
              return this;
            }, le.prototype.destroy = function() {
              return this.traces.forEach(function(he) {
                he.buffer && he.buffer.destroy && he.buffer.destroy();
              }), this.traces = null, this.passes = null, this.scatter.destroy(), this;
            };
          }, 81330: function(G) {
            G.exports = function() {
              function J(Ot, un) {
                this.id = st++, this.type = Ot, this.data = un;
              }
              function Z(Ot) {
                if (Ot.length === 0) return [];
                var un = Ot.charAt(0), yn = Ot.charAt(Ot.length - 1);
                if (1 < Ot.length && un === yn && (un === '"' || un === "'")) return ['"' + Ot.substr(1, Ot.length - 2).replace(/\\/g, "\\\\").replace(/"/g, '\\"') + '"'];
                if (un = /\[(false|true|null|\d+|'[^']*'|"[^"]*")\]/.exec(Ot)) return Z(Ot.substr(0, un.index)).concat(Z(un[1])).concat(Z(Ot.substr(un.index + un[0].length)));
                if ((un = Ot.split(".")).length === 1) return ['"' + Ot.replace(/\\/g, "\\\\").replace(/"/g, '\\"') + '"'];
                for (Ot = [], yn = 0; yn < un.length; ++yn) Ot = Ot.concat(Z(un[yn]));
                return Ot;
              }
              function te(Ot) {
                return "[" + Z(Ot).join("][") + "]";
              }
              function ne(Ot) {
                return typeof Ot == "string" ? Ot.split() : Ot;
              }
              function ee(Ot) {
                return typeof Ot == "string" ? document.querySelector(Ot) : Ot;
              }
              function ae(Ot) {
                var un, yn, _n, gn, bn = Ot || {};
                Ot = {};
                var an = [], Zt = [], Mn = typeof window > "u" ? 1 : window.devicePixelRatio, Ln = !1, In = {}, kn = function(jn) {
                }, er = function() {
                };
                if (typeof bn == "string" ? un = document.querySelector(bn) : typeof bn == "object" && (typeof bn.nodeName == "string" && typeof bn.appendChild == "function" && typeof bn.getBoundingClientRect == "function" ? un = bn : typeof bn.drawArrays == "function" || typeof bn.drawElements == "function" ? _n = (gn = bn).canvas : ("gl" in bn ? gn = bn.gl : "canvas" in bn ? _n = ee(bn.canvas) : "container" in bn && (yn = ee(bn.container)), "attributes" in bn && (Ot = bn.attributes), "extensions" in bn && (an = ne(bn.extensions)), "optionalExtensions" in bn && (Zt = ne(bn.optionalExtensions)), "onDone" in bn && (kn = bn.onDone), "profile" in bn && (Ln = !!bn.profile), "pixelRatio" in bn && (Mn = +bn.pixelRatio), "cachedCode" in bn && (In = bn.cachedCode))), un && (un.nodeName.toLowerCase() === "canvas" ? _n = un : yn = un), !gn) {
                  if (!_n) {
                    if (!(un = function(jn, ur, Kn) {
                      function Jn() {
                        var Ar = window.innerWidth, Yr = window.innerHeight;
                        jn !== document.body && (Ar = (Yr = mr.getBoundingClientRect()).right - Yr.left, Yr = Yr.bottom - Yr.top), mr.width = Kn * Ar, mr.height = Kn * Yr;
                      }
                      var Lr, mr = document.createElement("canvas");
                      return it(mr.style, { border: 0, margin: 0, padding: 0, top: 0, left: 0, width: "100%", height: "100%" }), jn.appendChild(mr), jn === document.body && (mr.style.position = "absolute", it(jn.style, { margin: 0, padding: 0 })), jn !== document.body && typeof ResizeObserver == "function" ? (Lr = new ResizeObserver(function() {
                        setTimeout(Jn);
                      })).observe(jn) : window.addEventListener("resize", Jn, !1), Jn(), { canvas: mr, onDestroy: function() {
                        Lr ? Lr.disconnect() : window.removeEventListener("resize", Jn), jn.removeChild(mr);
                      } };
                    }(yn || document.body, 0, Mn))) return null;
                    _n = un.canvas, er = un.onDestroy;
                  }
                  Ot.premultipliedAlpha === void 0 && (Ot.premultipliedAlpha = !0), gn = function(jn, ur) {
                    function Kn(Jn) {
                      try {
                        return jn.getContext(Jn, ur);
                      } catch {
                        return null;
                      }
                    }
                    return Kn("webgl") || Kn("experimental-webgl") || Kn("webgl-experimental");
                  }(_n, Ot);
                }
                return gn ? { gl: gn, canvas: _n, container: yn, extensions: an, optionalExtensions: Zt, pixelRatio: Mn, profile: Ln, cachedCode: In, onDone: kn, onDestroy: er } : (er(), kn("webgl not supported, try upgrading your browser or graphics drivers http://get.webgl.org"), null);
              }
              function oe(Ot, un) {
                for (var yn = Array(Ot), _n = 0; _n < Ot; ++_n) yn[_n] = un(_n);
                return yn;
              }
              function se(Ot) {
                var un, yn;
                return un = (65535 < Ot) << 4, un |= yn = (255 < (Ot >>>= un)) << 3, (un |= yn = (15 < (Ot >>>= yn)) << 2) | (yn = (3 < (Ot >>>= yn)) << 1) | Ot >>> yn >> 1;
              }
              function ce() {
                function Ot(_n) {
                  e: {
                    for (var gn = 16; 268435456 >= gn; gn *= 16) if (_n <= gn) {
                      _n = gn;
                      break e;
                    }
                    _n = 0;
                  }
                  return 0 < (gn = yn[se(_n) >> 2]).length ? gn.pop() : new ArrayBuffer(_n);
                }
                function un(_n) {
                  yn[se(_n.byteLength) >> 2].push(_n);
                }
                var yn = oe(8, function() {
                  return [];
                });
                return { alloc: Ot, free: un, allocType: function(_n, gn) {
                  var bn = null;
                  switch (_n) {
                    case 5120:
                      bn = new Int8Array(Ot(gn), 0, gn);
                      break;
                    case 5121:
                      bn = new Uint8Array(Ot(gn), 0, gn);
                      break;
                    case 5122:
                      bn = new Int16Array(Ot(2 * gn), 0, gn);
                      break;
                    case 5123:
                      bn = new Uint16Array(Ot(2 * gn), 0, gn);
                      break;
                    case 5124:
                      bn = new Int32Array(Ot(4 * gn), 0, gn);
                      break;
                    case 5125:
                      bn = new Uint32Array(Ot(4 * gn), 0, gn);
                      break;
                    case 5126:
                      bn = new Float32Array(Ot(4 * gn), 0, gn);
                      break;
                    default:
                      return null;
                  }
                  return bn.length !== gn ? bn.subarray(0, gn) : bn;
                }, freeType: function(_n) {
                  un(_n.buffer);
                } };
              }
              function le(Ot) {
                return !!Ot && typeof Ot == "object" && Array.isArray(Ot.shape) && Array.isArray(Ot.stride) && typeof Ot.offset == "number" && Ot.shape.length === Ot.stride.length && (Array.isArray(Ot.data) || Rt(Ot.data));
              }
              function ue(Ot, un, yn, _n, gn, bn) {
                for (var an = 0; an < un; ++an) for (var Zt = Ot[an], Mn = 0; Mn < yn; ++Mn) for (var Ln = Zt[Mn], In = 0; In < _n; ++In) gn[bn++] = Ln[In];
              }
              function de(Ot, un, yn, _n, gn) {
                for (var bn = 1, an = yn + 1; an < un.length; ++an) bn *= un[an];
                var Zt = un[yn];
                if (un.length - yn == 4) {
                  var Mn = un[yn + 1], Ln = un[yn + 2];
                  for (un = un[yn + 3], an = 0; an < Zt; ++an) ue(Ot[an], Mn, Ln, un, _n, gn), gn += bn;
                } else for (an = 0; an < Zt; ++an) de(Ot[an], un, yn + 1, _n, gn), gn += bn;
              }
              function fe(Ot) {
                return 0 | Pt[Object.prototype.toString.call(Ot)];
              }
              function he(Ot, un) {
                for (var yn = 0; yn < un.length; ++yn) Ot[yn] = un[yn];
              }
              function me(Ot, un, yn, _n, gn, bn, an) {
                for (var Zt = 0, Mn = 0; Mn < yn; ++Mn) for (var Ln = 0; Ln < _n; ++Ln) Ot[Zt++] = un[gn * Mn + bn * Ln + an];
              }
              function pe(Ot, un, yn, _n) {
                function gn(kn) {
                  this.id = Mn++, this.buffer = Ot.createBuffer(), this.type = kn, this.usage = 35044, this.byteLength = 0, this.dimension = 1, this.dtype = 5121, this.persistentData = null, yn.profile && (this.stats = { size: 0 });
                }
                function bn(kn, er, jn) {
                  kn.byteLength = er.byteLength, Ot.bufferData(kn.type, er, jn);
                }
                function an(kn, er, jn, ur, Kn, Jn) {
                  if (kn.usage = jn, Array.isArray(er)) {
                    if (kn.dtype = ur || 5126, 0 < er.length) if (Array.isArray(er[0])) {
                      Kn = Ft(er);
                      for (var Lr = ur = 1; Lr < Kn.length; ++Lr) ur *= Kn[Lr];
                      kn.dimension = ur, bn(kn, er = Gt(er, Kn, kn.dtype), jn), Jn ? kn.persistentData = er : St.freeType(er);
                    } else typeof er[0] == "number" ? (kn.dimension = Kn, he(Kn = St.allocType(kn.dtype, er.length), er), bn(kn, Kn, jn), Jn ? kn.persistentData = Kn : St.freeType(Kn)) : Rt(er[0]) && (kn.dimension = er[0].length, kn.dtype = ur || fe(er[0]) || 5126, bn(kn, er = Gt(er, [er.length, er[0].length], kn.dtype), jn), Jn ? kn.persistentData = er : St.freeType(er));
                  } else if (Rt(er)) kn.dtype = ur || fe(er), kn.dimension = Kn, bn(kn, er, jn), Jn && (kn.persistentData = new Uint8Array(new Uint8Array(er.buffer)));
                  else if (le(er)) {
                    Kn = er.shape;
                    var mr = er.stride, Ar = (Lr = er.offset, 0), Yr = 0, jr = 0, Mr = 0;
                    Kn.length === 1 ? (Ar = Kn[0], Yr = 1, jr = mr[0], Mr = 0) : Kn.length === 2 && (Ar = Kn[0], Yr = Kn[1], jr = mr[0], Mr = mr[1]), kn.dtype = ur || fe(er.data) || 5126, kn.dimension = Yr, me(Kn = St.allocType(kn.dtype, Ar * Yr), er.data, Ar, Yr, jr, Mr, Lr), bn(kn, Kn, jn), Jn ? kn.persistentData = Kn : St.freeType(Kn);
                  } else er instanceof ArrayBuffer && (kn.dtype = 5121, kn.dimension = Kn, bn(kn, er, jn), Jn && (kn.persistentData = new Uint8Array(new Uint8Array(er))));
                }
                function Zt(kn) {
                  un.bufferCount--, _n(kn), Ot.deleteBuffer(kn.buffer), kn.buffer = null, delete Ln[kn.id];
                }
                var Mn = 0, Ln = {};
                gn.prototype.bind = function() {
                  Ot.bindBuffer(this.type, this.buffer);
                }, gn.prototype.destroy = function() {
                  Zt(this);
                };
                var In = [];
                return yn.profile && (un.getTotalBufferSize = function() {
                  var kn = 0;
                  return Object.keys(Ln).forEach(function(er) {
                    kn += Ln[er].stats.size;
                  }), kn;
                }), { create: function(kn, er, jn, ur) {
                  function Kn(Lr) {
                    var mr = 35044, Ar = null, Yr = 0, jr = 0, Mr = 1;
                    return Array.isArray(Lr) || Rt(Lr) || le(Lr) || Lr instanceof ArrayBuffer ? Ar = Lr : typeof Lr == "number" ? Yr = 0 | Lr : Lr && ("data" in Lr && (Ar = Lr.data), "usage" in Lr && (mr = Nt[Lr.usage]), "type" in Lr && (jr = Xt[Lr.type]), "dimension" in Lr && (Mr = 0 | Lr.dimension), "length" in Lr && (Yr = 0 | Lr.length)), Jn.bind(), Ar ? an(Jn, Ar, mr, jr, Mr, ur) : (Yr && Ot.bufferData(Jn.type, Yr, mr), Jn.dtype = jr || 5121, Jn.usage = mr, Jn.dimension = Mr, Jn.byteLength = Yr), yn.profile && (Jn.stats.size = Jn.byteLength * Tt[Jn.dtype]), Kn;
                  }
                  un.bufferCount++;
                  var Jn = new gn(er);
                  return Ln[Jn.id] = Jn, jn || Kn(kn), Kn._reglType = "buffer", Kn._buffer = Jn, Kn.subdata = function(Lr, mr) {
                    var Ar, Yr = 0 | (mr || 0);
                    if (Jn.bind(), Rt(Lr) || Lr instanceof ArrayBuffer) Ot.bufferSubData(Jn.type, Yr, Lr);
                    else if (Array.isArray(Lr)) {
                      if (0 < Lr.length) if (typeof Lr[0] == "number") {
                        var jr = St.allocType(Jn.dtype, Lr.length);
                        he(jr, Lr), Ot.bufferSubData(Jn.type, Yr, jr), St.freeType(jr);
                      } else (Array.isArray(Lr[0]) || Rt(Lr[0])) && (Ar = Ft(Lr), jr = Gt(Lr, Ar, Jn.dtype), Ot.bufferSubData(Jn.type, Yr, jr), St.freeType(jr));
                    } else if (le(Lr)) {
                      Ar = Lr.shape;
                      var Mr = Lr.stride, Wr = jr = 0, Bi = 0, gi = 0;
                      Ar.length === 1 ? (jr = Ar[0], Wr = 1, Bi = Mr[0], gi = 0) : Ar.length === 2 && (jr = Ar[0], Wr = Ar[1], Bi = Mr[0], gi = Mr[1]), Ar = Array.isArray(Lr.data) ? Jn.dtype : fe(Lr.data), me(Ar = St.allocType(Ar, jr * Wr), Lr.data, jr, Wr, Bi, gi, Lr.offset), Ot.bufferSubData(Jn.type, Yr, Ar), St.freeType(Ar);
                    }
                    return Kn;
                  }, yn.profile && (Kn.stats = Jn.stats), Kn.destroy = function() {
                    Zt(Jn);
                  }, Kn;
                }, createStream: function(kn, er) {
                  var jn = In.pop();
                  return jn || (jn = new gn(kn)), jn.bind(), an(jn, er, 35040, 0, 1, !1), jn;
                }, destroyStream: function(kn) {
                  In.push(kn);
                }, clear: function() {
                  vt(Ln).forEach(Zt), In.forEach(Zt);
                }, getBuffer: function(kn) {
                  return kn && kn._buffer instanceof gn ? kn._buffer : null;
                }, restore: function() {
                  vt(Ln).forEach(function(kn) {
                    kn.buffer = Ot.createBuffer(), Ot.bindBuffer(kn.type, kn.buffer), Ot.bufferData(kn.type, kn.persistentData || kn.byteLength, kn.usage);
                  });
                }, _initBuffer: an };
              }
              function ye(Ot, un, yn, _n) {
                function gn(kn) {
                  this.id = Mn++, Zt[this.id] = this, this.buffer = kn, this.primType = 4, this.type = this.vertCount = 0;
                }
                function bn(kn, er, jn, ur, Kn, Jn, Lr) {
                  var mr;
                  if (kn.buffer.bind(), er ? ((mr = Lr) || Rt(er) && (!le(er) || Rt(er.data)) || (mr = un.oes_element_index_uint ? 5125 : 5123), yn._initBuffer(kn.buffer, er, jn, mr, 3)) : (Ot.bufferData(34963, Jn, jn), kn.buffer.dtype = mr || 5121, kn.buffer.usage = jn, kn.buffer.dimension = 3, kn.buffer.byteLength = Jn), mr = Lr, !Lr) {
                    switch (kn.buffer.dtype) {
                      case 5121:
                      case 5120:
                        mr = 5121;
                        break;
                      case 5123:
                      case 5122:
                        mr = 5123;
                        break;
                      case 5125:
                      case 5124:
                        mr = 5125;
                    }
                    kn.buffer.dtype = mr;
                  }
                  kn.type = mr, 0 > (er = Kn) && (er = kn.buffer.byteLength, mr === 5123 ? er >>= 1 : mr === 5125 && (er >>= 2)), kn.vertCount = er, er = ur, 0 > ur && (er = 4, (ur = kn.buffer.dimension) === 1 && (er = 0), ur === 2 && (er = 1), ur === 3 && (er = 4)), kn.primType = er;
                }
                function an(kn) {
                  _n.elementsCount--, delete Zt[kn.id], kn.buffer.destroy(), kn.buffer = null;
                }
                var Zt = {}, Mn = 0, Ln = { uint8: 5121, uint16: 5123 };
                un.oes_element_index_uint && (Ln.uint32 = 5125), gn.prototype.bind = function() {
                  this.buffer.bind();
                };
                var In = [];
                return { create: function(kn, er) {
                  function jn(Jn) {
                    if (Jn) if (typeof Jn == "number") ur(Jn), Kn.primType = 4, Kn.vertCount = 0 | Jn, Kn.type = 5121;
                    else {
                      var Lr = null, mr = 35044, Ar = -1, Yr = -1, jr = 0, Mr = 0;
                      Array.isArray(Jn) || Rt(Jn) || le(Jn) ? Lr = Jn : ("data" in Jn && (Lr = Jn.data), "usage" in Jn && (mr = Nt[Jn.usage]), "primitive" in Jn && (Ar = It[Jn.primitive]), "count" in Jn && (Yr = 0 | Jn.count), "type" in Jn && (Mr = Ln[Jn.type]), "length" in Jn ? jr = 0 | Jn.length : (jr = Yr, Mr === 5123 || Mr === 5122 ? jr *= 2 : Mr !== 5125 && Mr !== 5124 || (jr *= 4))), bn(Kn, Lr, mr, Ar, Yr, jr, Mr);
                    }
                    else ur(), Kn.primType = 4, Kn.vertCount = 0, Kn.type = 5121;
                    return jn;
                  }
                  var ur = yn.create(null, 34963, !0), Kn = new gn(ur._buffer);
                  return _n.elementsCount++, jn(kn), jn._reglType = "elements", jn._elements = Kn, jn.subdata = function(Jn, Lr) {
                    return ur.subdata(Jn, Lr), jn;
                  }, jn.destroy = function() {
                    an(Kn);
                  }, jn;
                }, createStream: function(kn) {
                  var er = In.pop();
                  return er || (er = new gn(yn.create(null, 34963, !0, !1)._buffer)), bn(er, kn, 35040, -1, -1, 0, 0), er;
                }, destroyStream: function(kn) {
                  In.push(kn);
                }, getElements: function(kn) {
                  return typeof kn == "function" && kn._elements instanceof gn ? kn._elements : null;
                }, clear: function() {
                  vt(Zt).forEach(an);
                } };
              }
              function ge(Ot) {
                for (var un = St.allocType(5123, Ot.length), yn = 0; yn < Ot.length; ++yn) if (isNaN(Ot[yn])) un[yn] = 65535;
                else if (Ot[yn] === 1 / 0) un[yn] = 31744;
                else if (Ot[yn] === -1 / 0) un[yn] = 64512;
                else {
                  Ct[0] = Ot[yn];
                  var _n = (bn = Kt[0]) >>> 31 << 15, gn = (bn << 1 >>> 24) - 127, bn = bn >> 13 & 1023;
                  un[yn] = -24 > gn ? _n : -14 > gn ? _n + (bn + 1024 >> -14 - gn) : 15 < gn ? _n + 31744 : _n + (gn + 15 << 10) + bn;
                }
                return un;
              }
              function ve(Ot) {
                return Array.isArray(Ot) || Rt(Ot);
              }
              function be(Ot) {
                return "[object " + Ot + "]";
              }
              function Ce(Ot) {
                return Array.isArray(Ot) && (Ot.length === 0 || typeof Ot[0] == "number");
              }
              function Se(Ot) {
                return !(!Array.isArray(Ot) || Ot.length === 0 || !ve(Ot[0]));
              }
              function we(Ot) {
                return Object.prototype.toString.call(Ot);
              }
              function Ee(Ot) {
                if (!Ot) return !1;
                var un = we(Ot);
                return 0 <= dn.indexOf(un) || Ce(Ot) || Se(Ot) || le(Ot);
              }
              function xe(Ot, un) {
                Ot.type === 36193 ? (Ot.data = ge(un), St.freeType(un)) : Ot.data = un;
              }
              function Le(Ot, un, yn, _n, gn, bn) {
                if (Ot = jt[Ot] !== void 0 ? jt[Ot] : sn[Ot] * Ht[un], bn && (Ot *= 6), gn) {
                  for (_n = 0; 1 <= yn; ) _n += Ot * yn * yn, yn /= 2;
                  return _n;
                }
                return Ot * yn * _n;
              }
              function Qe(Ot, un, yn, _n, gn, bn, an) {
                function Zt() {
                  this.format = this.internalformat = 6408, this.type = 5121, this.flipY = this.premultiplyAlpha = this.compressed = !1, this.unpackAlignment = 1, this.colorSpace = 37444, this.channels = this.height = this.width = 0;
                }
                function Mn(Qr, Ur) {
                  Qr.internalformat = Ur.internalformat, Qr.format = Ur.format, Qr.type = Ur.type, Qr.compressed = Ur.compressed, Qr.premultiplyAlpha = Ur.premultiplyAlpha, Qr.flipY = Ur.flipY, Qr.unpackAlignment = Ur.unpackAlignment, Qr.colorSpace = Ur.colorSpace, Qr.width = Ur.width, Qr.height = Ur.height, Qr.channels = Ur.channels;
                }
                function Ln(Qr, Ur) {
                  if (typeof Ur == "object" && Ur) {
                    "premultiplyAlpha" in Ur && (Qr.premultiplyAlpha = Ur.premultiplyAlpha), "flipY" in Ur && (Qr.flipY = Ur.flipY), "alignment" in Ur && (Qr.unpackAlignment = Ur.alignment), "colorSpace" in Ur && (Qr.colorSpace = Pr[Ur.colorSpace]), "type" in Ur && (Qr.type = ei[Ur.type]);
                    var rr = Qr.width, vr = Qr.height, Rr = Qr.channels, Vr = !1;
                    "shape" in Ur ? (rr = Ur.shape[0], vr = Ur.shape[1], Ur.shape.length === 3 && (Rr = Ur.shape[2], Vr = !0)) : ("radius" in Ur && (rr = vr = Ur.radius), "width" in Ur && (rr = Ur.width), "height" in Ur && (vr = Ur.height), "channels" in Ur && (Rr = Ur.channels, Vr = !0)), Qr.width = 0 | rr, Qr.height = 0 | vr, Qr.channels = 0 | Rr, rr = !1, "format" in Ur && (rr = Ur.format, vr = Qr.internalformat = Ci[rr], Qr.format = si[vr], rr in ei && !("type" in Ur) && (Qr.type = ei[rr]), rr in oa && (Qr.compressed = !0), rr = !0), !Vr && rr ? Qr.channels = sn[Qr.format] : Vr && !rr && Qr.channels !== Yt[Qr.format] && (Qr.format = Qr.internalformat = Yt[Qr.channels]);
                  }
                }
                function In(Qr) {
                  Ot.pixelStorei(37440, Qr.flipY), Ot.pixelStorei(37441, Qr.premultiplyAlpha), Ot.pixelStorei(37443, Qr.colorSpace), Ot.pixelStorei(3317, Qr.unpackAlignment);
                }
                function kn() {
                  Zt.call(this), this.yOffset = this.xOffset = 0, this.data = null, this.needsFree = !1, this.element = null, this.needsCopy = !1;
                }
                function er(Qr, Ur) {
                  var rr = null;
                  if (Ee(Ur) ? rr = Ur : Ur && (Ln(Qr, Ur), "x" in Ur && (Qr.xOffset = 0 | Ur.x), "y" in Ur && (Qr.yOffset = 0 | Ur.y), Ee(Ur.data) && (rr = Ur.data)), Ur.copy) {
                    var vr = gn.viewportWidth, Rr = gn.viewportHeight;
                    Qr.width = Qr.width || vr - Qr.xOffset, Qr.height = Qr.height || Rr - Qr.yOffset, Qr.needsCopy = !0;
                  } else if (rr) {
                    if (Rt(rr)) Qr.channels = Qr.channels || 4, Qr.data = rr, "type" in Ur || Qr.type !== 5121 || (Qr.type = 0 | Pt[Object.prototype.toString.call(rr)]);
                    else if (Ce(rr)) {
                      switch (Qr.channels = Qr.channels || 4, Rr = (vr = rr).length, Qr.type) {
                        case 5121:
                        case 5123:
                        case 5125:
                        case 5126:
                          (Rr = St.allocType(Qr.type, Rr)).set(vr), Qr.data = Rr;
                          break;
                        case 36193:
                          Qr.data = ge(vr);
                      }
                      Qr.alignment = 1, Qr.needsFree = !0;
                    } else if (le(rr)) {
                      vr = rr.data, Array.isArray(vr) || Qr.type !== 5121 || (Qr.type = 0 | Pt[Object.prototype.toString.call(vr)]), Rr = rr.shape;
                      var Vr, Xr, Jr, ti, Hr = rr.stride;
                      Rr.length === 3 ? (Jr = Rr[2], ti = Hr[2]) : ti = Jr = 1, Vr = Rr[0], Xr = Rr[1], Rr = Hr[0], Hr = Hr[1], Qr.alignment = 1, Qr.width = Vr, Qr.height = Xr, Qr.channels = Jr, Qr.format = Qr.internalformat = Yt[Jr], Qr.needsFree = !0, Vr = ti, rr = rr.offset, Jr = Qr.width, ti = Qr.height, Xr = Qr.channels;
                      for (var Er = St.allocType(Qr.type === 36193 ? 5126 : Qr.type, Jr * ti * Xr), gr = 0, $r = 0; $r < ti; ++$r) for (var li = 0; li < Jr; ++li) for (var Qi = 0; Qi < Xr; ++Qi) Er[gr++] = vr[Rr * li + Hr * $r + Vr * Qi + rr];
                      xe(Qr, Er);
                    } else if (we(rr) === hn || we(rr) === Qt || we(rr) === Ut) we(rr) === hn || we(rr) === Qt ? Qr.element = rr : Qr.element = rr.canvas, Qr.width = Qr.element.width, Qr.height = Qr.element.height, Qr.channels = 4;
                    else if (we(rr) === Wt) Qr.element = rr, Qr.width = rr.width, Qr.height = rr.height, Qr.channels = 4;
                    else if (we(rr) === rn) Qr.element = rr, Qr.width = rr.naturalWidth, Qr.height = rr.naturalHeight, Qr.channels = 4;
                    else if (we(rr) === vn) Qr.element = rr, Qr.width = rr.videoWidth, Qr.height = rr.videoHeight, Qr.channels = 4;
                    else if (Se(rr)) {
                      for (vr = Qr.width || rr[0].length, Rr = Qr.height || rr.length, Hr = Qr.channels, Hr = ve(rr[0][0]) ? Hr || rr[0][0].length : Hr || 1, Vr = $t.shape(rr), Jr = 1, ti = 0; ti < Vr.length; ++ti) Jr *= Vr[ti];
                      Jr = St.allocType(Qr.type === 36193 ? 5126 : Qr.type, Jr), $t.flatten(rr, Vr, "", Jr), xe(Qr, Jr), Qr.alignment = 1, Qr.width = vr, Qr.height = Rr, Qr.channels = Hr, Qr.format = Qr.internalformat = Yt[Hr], Qr.needsFree = !0;
                    }
                  } else Qr.width = Qr.width || 1, Qr.height = Qr.height || 1, Qr.channels = Qr.channels || 4;
                }
                function jn(Qr, Ur, rr, vr, Rr) {
                  var Vr = Qr.element, Xr = Qr.data, Jr = Qr.internalformat, ti = Qr.format, Hr = Qr.type, Er = Qr.width, gr = Qr.height;
                  In(Qr), Vr ? Ot.texSubImage2D(Ur, Rr, rr, vr, ti, Hr, Vr) : Qr.compressed ? Ot.compressedTexSubImage2D(Ur, Rr, rr, vr, Jr, Er, gr, Xr) : Qr.needsCopy ? (_n(), Ot.copyTexSubImage2D(Ur, Rr, rr, vr, Qr.xOffset, Qr.yOffset, Er, gr)) : Ot.texSubImage2D(Ur, Rr, rr, vr, Er, gr, ti, Hr, Xr);
                }
                function ur() {
                  return zr.pop() || new kn();
                }
                function Kn(Qr) {
                  Qr.needsFree && St.freeType(Qr.data), kn.call(Qr), zr.push(Qr);
                }
                function Jn() {
                  Zt.call(this), this.genMipmaps = !1, this.mipmapHint = 4352, this.mipmask = 0, this.images = Array(16);
                }
                function Lr(Qr, Ur, rr) {
                  var vr = Qr.images[0] = ur();
                  Qr.mipmask = 1, vr.width = Qr.width = Ur, vr.height = Qr.height = rr, vr.channels = Qr.channels = 4;
                }
                function mr(Qr, Ur) {
                  var rr = null;
                  if (Ee(Ur)) Mn(rr = Qr.images[0] = ur(), Qr), er(rr, Ur), Qr.mipmask = 1;
                  else if (Ln(Qr, Ur), Array.isArray(Ur.mipmap)) for (var vr = Ur.mipmap, Rr = 0; Rr < vr.length; ++Rr) Mn(rr = Qr.images[Rr] = ur(), Qr), rr.width >>= Rr, rr.height >>= Rr, er(rr, vr[Rr]), Qr.mipmask |= 1 << Rr;
                  else Mn(rr = Qr.images[0] = ur(), Qr), er(rr, Ur), Qr.mipmask = 1;
                  Mn(Qr, Qr.images[0]);
                }
                function Ar(Qr, Ur) {
                  for (var rr = Qr.images, vr = 0; vr < rr.length && rr[vr]; ++vr) {
                    var Rr = rr[vr], Vr = Ur, Xr = vr, Jr = Rr.element, ti = Rr.data, Hr = Rr.internalformat, Er = Rr.format, gr = Rr.type, $r = Rr.width, li = Rr.height;
                    In(Rr), Jr ? Ot.texImage2D(Vr, Xr, Er, Er, gr, Jr) : Rr.compressed ? Ot.compressedTexImage2D(Vr, Xr, Hr, $r, li, 0, ti) : Rr.needsCopy ? (_n(), Ot.copyTexImage2D(Vr, Xr, Er, Rr.xOffset, Rr.yOffset, $r, li, 0)) : Ot.texImage2D(Vr, Xr, Er, $r, li, 0, Er, gr, ti || null);
                  }
                }
                function Yr() {
                  var Qr = di.pop() || new Jn();
                  Zt.call(Qr);
                  for (var Ur = Qr.mipmask = 0; 16 > Ur; ++Ur) Qr.images[Ur] = null;
                  return Qr;
                }
                function jr(Qr) {
                  for (var Ur = Qr.images, rr = 0; rr < Ur.length; ++rr) Ur[rr] && Kn(Ur[rr]), Ur[rr] = null;
                  di.push(Qr);
                }
                function Mr() {
                  this.magFilter = this.minFilter = 9728, this.wrapT = this.wrapS = 33071, this.anisotropic = 1, this.genMipmaps = !1, this.mipmapHint = 4352;
                }
                function Wr(Qr, Ur) {
                  "min" in Ur && (Qr.minFilter = Br[Ur.min], 0 <= Vt.indexOf(Qr.minFilter) && !("faces" in Ur) && (Qr.genMipmaps = !0)), "mag" in Ur && (Qr.magFilter = $i[Ur.mag]);
                  var rr = Qr.wrapS, vr = Qr.wrapT;
                  if ("wrap" in Ur) {
                    var Rr = Ur.wrap;
                    typeof Rr == "string" ? rr = vr = Zi[Rr] : Array.isArray(Rr) && (rr = Zi[Rr[0]], vr = Zi[Rr[1]]);
                  } else "wrapS" in Ur && (rr = Zi[Ur.wrapS]), "wrapT" in Ur && (vr = Zi[Ur.wrapT]);
                  if (Qr.wrapS = rr, Qr.wrapT = vr, "anisotropic" in Ur && (Qr.anisotropic = Ur.anisotropic), "mipmap" in Ur) {
                    switch (rr = !1, typeof Ur.mipmap) {
                      case "string":
                        Qr.mipmapHint = bi[Ur.mipmap], rr = Qr.genMipmaps = !0;
                        break;
                      case "boolean":
                        rr = Qr.genMipmaps = Ur.mipmap;
                        break;
                      case "object":
                        Qr.genMipmaps = !1, rr = !0;
                    }
                    !rr || "min" in Ur || (Qr.minFilter = 9984);
                  }
                }
                function Bi(Qr, Ur) {
                  Ot.texParameteri(Ur, 10241, Qr.minFilter), Ot.texParameteri(Ur, 10240, Qr.magFilter), Ot.texParameteri(Ur, 10242, Qr.wrapS), Ot.texParameteri(Ur, 10243, Qr.wrapT), un.ext_texture_filter_anisotropic && Ot.texParameteri(Ur, 34046, Qr.anisotropic), Qr.genMipmaps && (Ot.hint(33170, Qr.mipmapHint), Ot.generateMipmap(Ur));
                }
                function gi(Qr) {
                  Zt.call(this), this.mipmask = 0, this.internalformat = 6408, this.id = Da++, this.refCount = 1, this.target = Qr, this.texture = Ot.createTexture(), this.unit = -1, this.bindCount = 0, this.texInfo = new Mr(), an.profile && (this.stats = { size: 0 });
                }
                function Zr(Qr) {
                  Ot.activeTexture(33984), Ot.bindTexture(Qr.target, Qr.texture);
                }
                function fi() {
                  var Qr = Wa[0];
                  Qr ? Ot.bindTexture(Qr.target, Qr.texture) : Ot.bindTexture(3553, null);
                }
                function ri(Qr) {
                  var Ur = Qr.texture, rr = Qr.unit, vr = Qr.target;
                  0 <= rr && (Ot.activeTexture(33984 + rr), Ot.bindTexture(vr, null), Wa[rr] = null), Ot.deleteTexture(Ur), Qr.texture = null, Qr.params = null, Qr.pixels = null, Qr.refCount = 0, delete la[Qr.id], bn.textureCount--;
                }
                var bi = { "don't care": 4352, "dont care": 4352, nice: 4354, fast: 4353 }, Zi = { repeat: 10497, clamp: 33071, mirror: 33648 }, $i = { nearest: 9728, linear: 9729 }, Br = it({ mipmap: 9987, "nearest mipmap nearest": 9984, "linear mipmap nearest": 9985, "nearest mipmap linear": 9986, "linear mipmap linear": 9987 }, $i), Pr = { none: 0, browser: 37444 }, ei = { uint8: 5121, rgba4: 32819, rgb565: 33635, "rgb5 a1": 32820 }, Ci = { alpha: 6406, luminance: 6409, "luminance alpha": 6410, rgb: 6407, rgba: 6408, rgba4: 32854, "rgb5 a1": 32855, rgb565: 36194 }, oa = {};
                un.ext_srgb && (Ci.srgb = 35904, Ci.srgba = 35906), un.oes_texture_float && (ei.float32 = ei.float = 5126), un.oes_texture_half_float && (ei.float16 = ei["half float"] = 36193), un.webgl_depth_texture && (it(Ci, { depth: 6402, "depth stencil": 34041 }), it(ei, { uint16: 5123, uint32: 5125, "depth stencil": 34042 })), un.webgl_compressed_texture_s3tc && it(oa, { "rgb s3tc dxt1": 33776, "rgba s3tc dxt1": 33777, "rgba s3tc dxt3": 33778, "rgba s3tc dxt5": 33779 }), un.webgl_compressed_texture_atc && it(oa, { "rgb atc": 35986, "rgba atc explicit alpha": 35987, "rgba atc interpolated alpha": 34798 }), un.webgl_compressed_texture_pvrtc && it(oa, { "rgb pvrtc 4bppv1": 35840, "rgb pvrtc 2bppv1": 35841, "rgba pvrtc 4bppv1": 35842, "rgba pvrtc 2bppv1": 35843 }), un.webgl_compressed_texture_etc1 && (oa["rgb etc1"] = 36196);
                var ka = Array.prototype.slice.call(Ot.getParameter(34467));
                Object.keys(oa).forEach(function(Qr) {
                  var Ur = oa[Qr];
                  0 <= ka.indexOf(Ur) && (Ci[Qr] = Ur);
                });
                var aa = Object.keys(Ci);
                yn.textureFormats = aa;
                var hi = [];
                Object.keys(Ci).forEach(function(Qr) {
                  hi[Ci[Qr]] = Qr;
                });
                var Gr = [];
                Object.keys(ei).forEach(function(Qr) {
                  Gr[ei[Qr]] = Qr;
                });
                var Ca = [];
                Object.keys($i).forEach(function(Qr) {
                  Ca[$i[Qr]] = Qr;
                });
                var Sr = [];
                Object.keys(Br).forEach(function(Qr) {
                  Sr[Br[Qr]] = Qr;
                });
                var Dr = [];
                Object.keys(Zi).forEach(function(Qr) {
                  Dr[Zi[Qr]] = Qr;
                });
                var si = aa.reduce(function(Qr, Ur) {
                  var rr = Ci[Ur];
                  return rr === 6409 || rr === 6406 || rr === 6409 || rr === 6410 || rr === 6402 || rr === 34041 || un.ext_srgb && (rr === 35904 || rr === 35906) ? Qr[rr] = rr : rr === 32855 || 0 <= Ur.indexOf("rgba") ? Qr[rr] = 6408 : Qr[rr] = 6407, Qr;
                }, {}), zr = [], di = [], Da = 0, la = {}, wa = yn.maxTextureUnits, Wa = Array(wa).map(function() {
                  return null;
                });
                return it(gi.prototype, { bind: function() {
                  this.bindCount += 1;
                  var Qr = this.unit;
                  if (0 > Qr) {
                    for (var Ur = 0; Ur < wa; ++Ur) {
                      var rr = Wa[Ur];
                      if (rr) {
                        if (0 < rr.bindCount) continue;
                        rr.unit = -1;
                      }
                      Wa[Ur] = this, Qr = Ur;
                      break;
                    }
                    an.profile && bn.maxTextureUnits < Qr + 1 && (bn.maxTextureUnits = Qr + 1), this.unit = Qr, Ot.activeTexture(33984 + Qr), Ot.bindTexture(this.target, this.texture);
                  }
                  return Qr;
                }, unbind: function() {
                  --this.bindCount;
                }, decRef: function() {
                  0 >= --this.refCount && ri(this);
                } }), an.profile && (bn.getTotalTextureSize = function() {
                  var Qr = 0;
                  return Object.keys(la).forEach(function(Ur) {
                    Qr += la[Ur].stats.size;
                  }), Qr;
                }), { create2D: function(Qr, Ur) {
                  function rr(Rr, Vr) {
                    var Xr = vr.texInfo;
                    Mr.call(Xr);
                    var Jr = Yr();
                    return typeof Rr == "number" ? Lr(Jr, 0 | Rr, typeof Vr == "number" ? 0 | Vr : 0 | Rr) : Rr ? (Wr(Xr, Rr), mr(Jr, Rr)) : Lr(Jr, 1, 1), Xr.genMipmaps && (Jr.mipmask = (Jr.width << 1) - 1), vr.mipmask = Jr.mipmask, Mn(vr, Jr), vr.internalformat = Jr.internalformat, rr.width = Jr.width, rr.height = Jr.height, Zr(vr), Ar(Jr, 3553), Bi(Xr, 3553), fi(), jr(Jr), an.profile && (vr.stats.size = Le(vr.internalformat, vr.type, Jr.width, Jr.height, Xr.genMipmaps, !1)), rr.format = hi[vr.internalformat], rr.type = Gr[vr.type], rr.mag = Ca[Xr.magFilter], rr.min = Sr[Xr.minFilter], rr.wrapS = Dr[Xr.wrapS], rr.wrapT = Dr[Xr.wrapT], rr;
                  }
                  var vr = new gi(3553);
                  return la[vr.id] = vr, bn.textureCount++, rr(Qr, Ur), rr.subimage = function(Rr, Vr, Xr, Jr) {
                    Vr |= 0, Xr |= 0, Jr |= 0;
                    var ti = ur();
                    return Mn(ti, vr), ti.width = 0, ti.height = 0, er(ti, Rr), ti.width = ti.width || (vr.width >> Jr) - Vr, ti.height = ti.height || (vr.height >> Jr) - Xr, Zr(vr), jn(ti, 3553, Vr, Xr, Jr), fi(), Kn(ti), rr;
                  }, rr.resize = function(Rr, Vr) {
                    var Xr = 0 | Rr, Jr = 0 | Vr || Xr;
                    if (Xr === vr.width && Jr === vr.height) return rr;
                    rr.width = vr.width = Xr, rr.height = vr.height = Jr, Zr(vr);
                    for (var ti = 0; vr.mipmask >> ti; ++ti) {
                      var Hr = Xr >> ti, Er = Jr >> ti;
                      if (!Hr || !Er) break;
                      Ot.texImage2D(3553, ti, vr.format, Hr, Er, 0, vr.format, vr.type, null);
                    }
                    return fi(), an.profile && (vr.stats.size = Le(vr.internalformat, vr.type, Xr, Jr, !1, !1)), rr;
                  }, rr._reglType = "texture2d", rr._texture = vr, an.profile && (rr.stats = vr.stats), rr.destroy = function() {
                    vr.decRef();
                  }, rr;
                }, createCube: function(Qr, Ur, rr, vr, Rr, Vr) {
                  function Xr(Hr, Er, gr, $r, li, Qi) {
                    var Ai, Li = Jr.texInfo;
                    for (Mr.call(Li), Ai = 0; 6 > Ai; ++Ai) ti[Ai] = Yr();
                    if (typeof Hr != "number" && Hr) {
                      if (typeof Hr == "object") if (Er) mr(ti[0], Hr), mr(ti[1], Er), mr(ti[2], gr), mr(ti[3], $r), mr(ti[4], li), mr(ti[5], Qi);
                      else if (Wr(Li, Hr), Ln(Jr, Hr), "faces" in Hr) for (Hr = Hr.faces, Ai = 0; 6 > Ai; ++Ai) Mn(ti[Ai], Jr), mr(ti[Ai], Hr[Ai]);
                      else for (Ai = 0; 6 > Ai; ++Ai) mr(ti[Ai], Hr);
                    } else for (Hr = 0 | Hr || 1, Ai = 0; 6 > Ai; ++Ai) Lr(ti[Ai], Hr, Hr);
                    for (Mn(Jr, ti[0]), Jr.mipmask = Li.genMipmaps ? (ti[0].width << 1) - 1 : ti[0].mipmask, Jr.internalformat = ti[0].internalformat, Xr.width = ti[0].width, Xr.height = ti[0].height, Zr(Jr), Ai = 0; 6 > Ai; ++Ai) Ar(ti[Ai], 34069 + Ai);
                    for (Bi(Li, 34067), fi(), an.profile && (Jr.stats.size = Le(Jr.internalformat, Jr.type, Xr.width, Xr.height, Li.genMipmaps, !0)), Xr.format = hi[Jr.internalformat], Xr.type = Gr[Jr.type], Xr.mag = Ca[Li.magFilter], Xr.min = Sr[Li.minFilter], Xr.wrapS = Dr[Li.wrapS], Xr.wrapT = Dr[Li.wrapT], Ai = 0; 6 > Ai; ++Ai) jr(ti[Ai]);
                    return Xr;
                  }
                  var Jr = new gi(34067);
                  la[Jr.id] = Jr, bn.cubeCount++;
                  var ti = Array(6);
                  return Xr(Qr, Ur, rr, vr, Rr, Vr), Xr.subimage = function(Hr, Er, gr, $r, li) {
                    gr |= 0, $r |= 0, li |= 0;
                    var Qi = ur();
                    return Mn(Qi, Jr), Qi.width = 0, Qi.height = 0, er(Qi, Er), Qi.width = Qi.width || (Jr.width >> li) - gr, Qi.height = Qi.height || (Jr.height >> li) - $r, Zr(Jr), jn(Qi, 34069 + Hr, gr, $r, li), fi(), Kn(Qi), Xr;
                  }, Xr.resize = function(Hr) {
                    if ((Hr |= 0) !== Jr.width) {
                      Xr.width = Jr.width = Hr, Xr.height = Jr.height = Hr, Zr(Jr);
                      for (var Er = 0; 6 > Er; ++Er) for (var gr = 0; Jr.mipmask >> gr; ++gr) Ot.texImage2D(34069 + Er, gr, Jr.format, Hr >> gr, Hr >> gr, 0, Jr.format, Jr.type, null);
                      return fi(), an.profile && (Jr.stats.size = Le(Jr.internalformat, Jr.type, Xr.width, Xr.height, !1, !0)), Xr;
                    }
                  }, Xr._reglType = "textureCube", Xr._texture = Jr, an.profile && (Xr.stats = Jr.stats), Xr.destroy = function() {
                    Jr.decRef();
                  }, Xr;
                }, clear: function() {
                  for (var Qr = 0; Qr < wa; ++Qr) Ot.activeTexture(33984 + Qr), Ot.bindTexture(3553, null), Wa[Qr] = null;
                  vt(la).forEach(ri), bn.cubeCount = 0, bn.textureCount = 0;
                }, getTexture: function(Qr) {
                  return null;
                }, restore: function() {
                  for (var Qr = 0; Qr < wa; ++Qr) {
                    var Ur = Wa[Qr];
                    Ur && (Ur.bindCount = 0, Ur.unit = -1, Wa[Qr] = null);
                  }
                  vt(la).forEach(function(rr) {
                    rr.texture = Ot.createTexture(), Ot.bindTexture(rr.target, rr.texture);
                    for (var vr = 0; 32 > vr; ++vr) if (rr.mipmask & 1 << vr) if (rr.target === 3553) Ot.texImage2D(3553, vr, rr.internalformat, rr.width >> vr, rr.height >> vr, 0, rr.internalformat, rr.type, null);
                    else for (var Rr = 0; 6 > Rr; ++Rr) Ot.texImage2D(34069 + Rr, vr, rr.internalformat, rr.width >> vr, rr.height >> vr, 0, rr.internalformat, rr.type, null);
                    Bi(rr.texInfo, rr.target);
                  });
                }, refresh: function() {
                  for (var Qr = 0; Qr < wa; ++Qr) {
                    var Ur = Wa[Qr];
                    Ur && (Ur.bindCount = 0, Ur.unit = -1, Wa[Qr] = null), Ot.activeTexture(33984 + Qr), Ot.bindTexture(3553, null), Ot.bindTexture(34067, null);
                  }
                } };
              }
              function ke(Ot, un, yn, _n, gn, bn) {
                function an(gi, Zr, fi) {
                  this.target = gi, this.texture = Zr, this.renderbuffer = fi;
                  var ri = gi = 0;
                  Zr ? (gi = Zr.width, ri = Zr.height) : fi && (gi = fi.width, ri = fi.height), this.width = gi, this.height = ri;
                }
                function Zt(gi) {
                  gi && (gi.texture && gi.texture._texture.decRef(), gi.renderbuffer && gi.renderbuffer._renderbuffer.decRef());
                }
                function Mn(gi, Zr, fi) {
                  gi && (gi.texture ? gi.texture._texture.refCount += 1 : gi.renderbuffer._renderbuffer.refCount += 1);
                }
                function Ln(gi, Zr) {
                  Zr && (Zr.texture ? Ot.framebufferTexture2D(36160, gi, Zr.target, Zr.texture._texture.texture, 0) : Ot.framebufferRenderbuffer(36160, gi, 36161, Zr.renderbuffer._renderbuffer.renderbuffer));
                }
                function In(gi) {
                  var Zr = 3553, fi = null, ri = null, bi = gi;
                  return typeof gi == "object" && (bi = gi.data, "target" in gi && (Zr = 0 | gi.target)), (gi = bi._reglType) === "texture2d" || gi === "textureCube" ? fi = bi : gi === "renderbuffer" && (ri = bi, Zr = 36161), new an(Zr, fi, ri);
                }
                function kn(gi, Zr, fi, ri, bi) {
                  return fi ? ((gi = _n.create2D({ width: gi, height: Zr, format: ri, type: bi }))._texture.refCount = 0, new an(3553, gi, null)) : ((gi = gn.create({ width: gi, height: Zr, format: ri }))._renderbuffer.refCount = 0, new an(36161, null, gi));
                }
                function er(gi) {
                  return gi && (gi.texture || gi.renderbuffer);
                }
                function jn(gi, Zr, fi) {
                  gi && (gi.texture ? gi.texture.resize(Zr, fi) : gi.renderbuffer && gi.renderbuffer.resize(Zr, fi), gi.width = Zr, gi.height = fi);
                }
                function ur() {
                  this.id = Wr++, Bi[this.id] = this, this.framebuffer = Ot.createFramebuffer(), this.height = this.width = 0, this.colorAttachments = [], this.depthStencilAttachment = this.stencilAttachment = this.depthAttachment = null;
                }
                function Kn(gi) {
                  gi.colorAttachments.forEach(Zt), Zt(gi.depthAttachment), Zt(gi.stencilAttachment), Zt(gi.depthStencilAttachment);
                }
                function Jn(gi) {
                  Ot.deleteFramebuffer(gi.framebuffer), gi.framebuffer = null, bn.framebufferCount--, delete Bi[gi.id];
                }
                function Lr(gi) {
                  var Zr;
                  Ot.bindFramebuffer(36160, gi.framebuffer);
                  var fi = gi.colorAttachments;
                  for (Zr = 0; Zr < fi.length; ++Zr) Ln(36064 + Zr, fi[Zr]);
                  for (Zr = fi.length; Zr < yn.maxColorAttachments; ++Zr) Ot.framebufferTexture2D(36160, 36064 + Zr, 3553, null, 0);
                  Ot.framebufferTexture2D(36160, 33306, 3553, null, 0), Ot.framebufferTexture2D(36160, 36096, 3553, null, 0), Ot.framebufferTexture2D(36160, 36128, 3553, null, 0), Ln(36096, gi.depthAttachment), Ln(36128, gi.stencilAttachment), Ln(33306, gi.depthStencilAttachment), Ot.checkFramebufferStatus(36160), Ot.isContextLost(), Ot.bindFramebuffer(36160, Ar.next ? Ar.next.framebuffer : null), Ar.cur = Ar.next, Ot.getError();
                }
                function mr(gi, Zr) {
                  function fi(bi, Zi) {
                    var $i, Br = 0, Pr = 0, ei = !0, Ci = !0;
                    $i = null;
                    var oa = !0, ka = "rgba", aa = "uint8", hi = 1, Gr = null, Ca = null, Sr = null, Dr = !1;
                    typeof bi == "number" ? (Br = 0 | bi, Pr = 0 | Zi || Br) : bi ? ("shape" in bi ? (Br = (Pr = bi.shape)[0], Pr = Pr[1]) : ("radius" in bi && (Br = Pr = bi.radius), "width" in bi && (Br = bi.width), "height" in bi && (Pr = bi.height)), ("color" in bi || "colors" in bi) && ($i = bi.color || bi.colors, Array.isArray($i)), $i || ("colorCount" in bi && (hi = 0 | bi.colorCount), "colorTexture" in bi && (oa = !!bi.colorTexture, ka = "rgba4"), "colorType" in bi && (aa = bi.colorType, !oa) && (aa === "half float" || aa === "float16" ? ka = "rgba16f" : aa !== "float" && aa !== "float32" || (ka = "rgba32f")), "colorFormat" in bi && (ka = bi.colorFormat, 0 <= Yr.indexOf(ka) ? oa = !0 : 0 <= jr.indexOf(ka) && (oa = !1))), ("depthTexture" in bi || "depthStencilTexture" in bi) && (Dr = !(!bi.depthTexture && !bi.depthStencilTexture)), "depth" in bi && (typeof bi.depth == "boolean" ? ei = bi.depth : (Gr = bi.depth, Ci = !1)), "stencil" in bi && (typeof bi.stencil == "boolean" ? Ci = bi.stencil : (Ca = bi.stencil, ei = !1)), "depthStencil" in bi && (typeof bi.depthStencil == "boolean" ? ei = Ci = bi.depthStencil : (Sr = bi.depthStencil, Ci = ei = !1))) : Br = Pr = 1;
                    var si = null, zr = null, di = null, Da = null;
                    if (Array.isArray($i)) si = $i.map(In);
                    else if ($i) si = [In($i)];
                    else for (si = Array(hi), $i = 0; $i < hi; ++$i) si[$i] = kn(Br, Pr, oa, ka, aa);
                    for (Br = Br || si[0].width, Pr = Pr || si[0].height, Gr ? zr = In(Gr) : ei && !Ci && (zr = kn(Br, Pr, Dr, "depth", "uint32")), Ca ? di = In(Ca) : Ci && !ei && (di = kn(Br, Pr, !1, "stencil", "uint8")), Sr ? Da = In(Sr) : !Gr && !Ca && Ci && ei && (Da = kn(Br, Pr, Dr, "depth stencil", "depth stencil")), ei = null, $i = 0; $i < si.length; ++$i) Mn(si[$i]), si[$i] && si[$i].texture && (Ci = Rn[si[$i].texture._texture.format] * Pn[si[$i].texture._texture.type], ei === null && (ei = Ci));
                    return Mn(zr), Mn(di), Mn(Da), Kn(ri), ri.width = Br, ri.height = Pr, ri.colorAttachments = si, ri.depthAttachment = zr, ri.stencilAttachment = di, ri.depthStencilAttachment = Da, fi.color = si.map(er), fi.depth = er(zr), fi.stencil = er(di), fi.depthStencil = er(Da), fi.width = ri.width, fi.height = ri.height, Lr(ri), fi;
                  }
                  var ri = new ur();
                  return bn.framebufferCount++, fi(gi, Zr), it(fi, { resize: function(bi, Zi) {
                    var $i = Math.max(0 | bi, 1), Br = Math.max(0 | Zi || $i, 1);
                    if ($i === ri.width && Br === ri.height) return fi;
                    for (var Pr = ri.colorAttachments, ei = 0; ei < Pr.length; ++ei) jn(Pr[ei], $i, Br);
                    return jn(ri.depthAttachment, $i, Br), jn(ri.stencilAttachment, $i, Br), jn(ri.depthStencilAttachment, $i, Br), ri.width = fi.width = $i, ri.height = fi.height = Br, Lr(ri), fi;
                  }, _reglType: "framebuffer", _framebuffer: ri, destroy: function() {
                    Jn(ri), Kn(ri);
                  }, use: function(bi) {
                    Ar.setFBO({ framebuffer: fi }, bi);
                  } });
                }
                var Ar = { cur: null, next: null, dirty: !1, setFBO: null }, Yr = ["rgba"], jr = ["rgba4", "rgb565", "rgb5 a1"];
                un.ext_srgb && jr.push("srgba"), un.ext_color_buffer_half_float && jr.push("rgba16f", "rgb16f"), un.webgl_color_buffer_float && jr.push("rgba32f");
                var Mr = ["uint8"];
                un.oes_texture_half_float && Mr.push("half float", "float16"), un.oes_texture_float && Mr.push("float", "float32");
                var Wr = 0, Bi = {};
                return it(Ar, { getFramebuffer: function(gi) {
                  return typeof gi == "function" && gi._reglType === "framebuffer" && (gi = gi._framebuffer) instanceof ur ? gi : null;
                }, create: mr, createCube: function(gi) {
                  function Zr(ri) {
                    var bi, Zi = { color: null }, $i = 0, Br = null;
                    bi = "rgba";
                    var Pr = "uint8", ei = 1;
                    if (typeof ri == "number" ? $i = 0 | ri : ri ? ("shape" in ri ? $i = ri.shape[0] : ("radius" in ri && ($i = 0 | ri.radius), "width" in ri ? $i = 0 | ri.width : "height" in ri && ($i = 0 | ri.height)), ("color" in ri || "colors" in ri) && (Br = ri.color || ri.colors, Array.isArray(Br)), Br || ("colorCount" in ri && (ei = 0 | ri.colorCount), "colorType" in ri && (Pr = ri.colorType), "colorFormat" in ri && (bi = ri.colorFormat)), "depth" in ri && (Zi.depth = ri.depth), "stencil" in ri && (Zi.stencil = ri.stencil), "depthStencil" in ri && (Zi.depthStencil = ri.depthStencil)) : $i = 1, Br) if (Array.isArray(Br)) for (ri = [], bi = 0; bi < Br.length; ++bi) ri[bi] = Br[bi];
                    else ri = [Br];
                    else for (ri = Array(ei), Br = { radius: $i, format: bi, type: Pr }, bi = 0; bi < ei; ++bi) ri[bi] = _n.createCube(Br);
                    for (Zi.color = Array(ri.length), bi = 0; bi < ri.length; ++bi) ei = ri[bi], $i = $i || ei.width, Zi.color[bi] = { target: 34069, data: ri[bi] };
                    for (bi = 0; 6 > bi; ++bi) {
                      for (ei = 0; ei < ri.length; ++ei) Zi.color[ei].target = 34069 + bi;
                      0 < bi && (Zi.depth = fi[0].depth, Zi.stencil = fi[0].stencil, Zi.depthStencil = fi[0].depthStencil), fi[bi] ? fi[bi](Zi) : fi[bi] = mr(Zi);
                    }
                    return it(Zr, { width: $i, height: $i, color: ri });
                  }
                  var fi = Array(6);
                  return Zr(gi), it(Zr, { faces: fi, resize: function(ri) {
                    var bi = 0 | ri;
                    if (bi === Zr.width) return Zr;
                    var Zi = Zr.color;
                    for (ri = 0; ri < Zi.length; ++ri) Zi[ri].resize(bi);
                    for (ri = 0; 6 > ri; ++ri) fi[ri].resize(bi);
                    return Zr.width = Zr.height = bi, Zr;
                  }, _reglType: "framebufferCube", destroy: function() {
                    fi.forEach(function(ri) {
                      ri.destroy();
                    });
                  } });
                }, clear: function() {
                  vt(Bi).forEach(Jn);
                }, restore: function() {
                  Ar.cur = null, Ar.next = null, Ar.dirty = !0, vt(Bi).forEach(function(gi) {
                    gi.framebuffer = Ot.createFramebuffer(), Lr(gi);
                  });
                } });
              }
              function Me() {
                this.w = this.z = this.y = this.x = this.state = 0, this.buffer = null, this.size = 0, this.normalized = !1, this.type = 5126, this.divisor = this.stride = this.offset = 0;
              }
              function Re(Ot, un, yn, _n, gn, bn, an) {
                function Zt() {
                  this.id = ++In, this.attributes = [], this.elements = null, this.ownsElements = !1, this.offset = this.count = 0, this.instances = -1, this.primitive = 4;
                  var jn = un.oes_vertex_array_object;
                  this.vao = jn ? jn.createVertexArrayOES() : null, kn[this.id] = this, this.buffers = [];
                }
                var Mn = yn.maxAttributes, Ln = Array(Mn);
                for (yn = 0; yn < Mn; ++yn) Ln[yn] = new Me();
                var In = 0, kn = {}, er = { Record: Me, scope: {}, state: Ln, currentVAO: null, targetVAO: null, restore: un.oes_vertex_array_object ? function() {
                  un.oes_vertex_array_object && vt(kn).forEach(function(jn) {
                    jn.refresh();
                  });
                } : function() {
                }, createVAO: function(jn) {
                  function ur(Jn) {
                    var Lr;
                    Array.isArray(Jn) ? (Lr = Jn, Kn.elements && Kn.ownsElements && Kn.elements.destroy(), Kn.elements = null, Kn.ownsElements = !1, Kn.offset = 0, Kn.count = 0, Kn.instances = -1, Kn.primitive = 4) : (Jn.elements ? (Lr = Jn.elements, Kn.ownsElements ? (typeof Lr == "function" && Lr._reglType === "elements" ? Kn.elements.destroy() : Kn.elements(Lr), Kn.ownsElements = !1) : bn.getElements(Jn.elements) ? (Kn.elements = Jn.elements, Kn.ownsElements = !1) : (Kn.elements = bn.create(Jn.elements), Kn.ownsElements = !0)) : (Kn.elements = null, Kn.ownsElements = !1), Lr = Jn.attributes, Kn.offset = 0, Kn.count = -1, Kn.instances = -1, Kn.primitive = 4, Kn.elements && (Kn.count = Kn.elements._elements.vertCount, Kn.primitive = Kn.elements._elements.primType), "offset" in Jn && (Kn.offset = 0 | Jn.offset), "count" in Jn && (Kn.count = 0 | Jn.count), "instances" in Jn && (Kn.instances = 0 | Jn.instances), "primitive" in Jn && (Kn.primitive = It[Jn.primitive])), Jn = {};
                    var mr = Kn.attributes;
                    mr.length = Lr.length;
                    for (var Ar = 0; Ar < Lr.length; ++Ar) {
                      var Yr, jr = Lr[Ar], Mr = mr[Ar] = new Me(), Wr = jr.data || jr;
                      Array.isArray(Wr) || Rt(Wr) || le(Wr) ? (Kn.buffers[Ar] && (Yr = Kn.buffers[Ar], Rt(Wr) && Yr._buffer.byteLength >= Wr.byteLength ? Yr.subdata(Wr) : (Yr.destroy(), Kn.buffers[Ar] = null)), Kn.buffers[Ar] || (Yr = Kn.buffers[Ar] = gn.create(jr, 34962, !1, !0)), Mr.buffer = gn.getBuffer(Yr), Mr.size = 0 | Mr.buffer.dimension, Mr.normalized = !1, Mr.type = Mr.buffer.dtype, Mr.offset = 0, Mr.stride = 0, Mr.divisor = 0, Mr.state = 1, Jn[Ar] = 1) : gn.getBuffer(jr) ? (Mr.buffer = gn.getBuffer(jr), Mr.size = 0 | Mr.buffer.dimension, Mr.normalized = !1, Mr.type = Mr.buffer.dtype, Mr.offset = 0, Mr.stride = 0, Mr.divisor = 0, Mr.state = 1) : gn.getBuffer(jr.buffer) ? (Mr.buffer = gn.getBuffer(jr.buffer), Mr.size = 0 | (+jr.size || Mr.buffer.dimension), Mr.normalized = !!jr.normalized || !1, Mr.type = "type" in jr ? Xt[jr.type] : Mr.buffer.dtype, Mr.offset = 0 | (jr.offset || 0), Mr.stride = 0 | (jr.stride || 0), Mr.divisor = 0 | (jr.divisor || 0), Mr.state = 1) : "x" in jr && (Mr.x = +jr.x || 0, Mr.y = +jr.y || 0, Mr.z = +jr.z || 0, Mr.w = +jr.w || 0, Mr.state = 2);
                    }
                    for (Yr = 0; Yr < Kn.buffers.length; ++Yr) !Jn[Yr] && Kn.buffers[Yr] && (Kn.buffers[Yr].destroy(), Kn.buffers[Yr] = null);
                    return Kn.refresh(), ur;
                  }
                  var Kn = new Zt();
                  return _n.vaoCount += 1, ur.destroy = function() {
                    for (var Jn = 0; Jn < Kn.buffers.length; ++Jn) Kn.buffers[Jn] && Kn.buffers[Jn].destroy();
                    Kn.buffers.length = 0, Kn.ownsElements && (Kn.elements.destroy(), Kn.elements = null, Kn.ownsElements = !1), Kn.destroy();
                  }, ur._vao = Kn, ur._reglType = "vao", ur(jn);
                }, getVAO: function(jn) {
                  return typeof jn == "function" && jn._vao ? jn._vao : null;
                }, destroyBuffer: function(jn) {
                  for (var ur = 0; ur < Ln.length; ++ur) {
                    var Kn = Ln[ur];
                    Kn.buffer === jn && (Ot.disableVertexAttribArray(ur), Kn.buffer = null);
                  }
                }, setVAO: un.oes_vertex_array_object ? function(jn) {
                  if (jn !== er.currentVAO) {
                    var ur = un.oes_vertex_array_object;
                    jn ? ur.bindVertexArrayOES(jn.vao) : ur.bindVertexArrayOES(null), er.currentVAO = jn;
                  }
                } : function(jn) {
                  if (jn !== er.currentVAO) {
                    if (jn) jn.bindAttrs();
                    else {
                      for (var ur = un.angle_instanced_arrays, Kn = 0; Kn < Ln.length; ++Kn) {
                        var Jn = Ln[Kn];
                        Jn.buffer ? (Ot.enableVertexAttribArray(Kn), Jn.buffer.bind(), Ot.vertexAttribPointer(Kn, Jn.size, Jn.type, Jn.normalized, Jn.stride, Jn.offfset), ur && Jn.divisor && ur.vertexAttribDivisorANGLE(Kn, Jn.divisor)) : (Ot.disableVertexAttribArray(Kn), Ot.vertexAttrib4f(Kn, Jn.x, Jn.y, Jn.z, Jn.w));
                      }
                      an.elements ? Ot.bindBuffer(34963, an.elements.buffer.buffer) : Ot.bindBuffer(34963, null);
                    }
                    er.currentVAO = jn;
                  }
                }, clear: un.oes_vertex_array_object ? function() {
                  vt(kn).forEach(function(jn) {
                    jn.destroy();
                  });
                } : function() {
                } };
                return Zt.prototype.bindAttrs = function() {
                  for (var jn = un.angle_instanced_arrays, ur = this.attributes, Kn = 0; Kn < ur.length; ++Kn) {
                    var Jn = ur[Kn];
                    Jn.buffer ? (Ot.enableVertexAttribArray(Kn), Ot.bindBuffer(34962, Jn.buffer.buffer), Ot.vertexAttribPointer(Kn, Jn.size, Jn.type, Jn.normalized, Jn.stride, Jn.offset), jn && Jn.divisor && jn.vertexAttribDivisorANGLE(Kn, Jn.divisor)) : (Ot.disableVertexAttribArray(Kn), Ot.vertexAttrib4f(Kn, Jn.x, Jn.y, Jn.z, Jn.w));
                  }
                  for (jn = ur.length; jn < Mn; ++jn) Ot.disableVertexAttribArray(jn);
                  (jn = bn.getElements(this.elements)) ? Ot.bindBuffer(34963, jn.buffer.buffer) : Ot.bindBuffer(34963, null);
                }, Zt.prototype.refresh = function() {
                  var jn = un.oes_vertex_array_object;
                  jn && (jn.bindVertexArrayOES(this.vao), this.bindAttrs(), er.currentVAO = null, jn.bindVertexArrayOES(null));
                }, Zt.prototype.destroy = function() {
                  if (this.vao) {
                    var jn = un.oes_vertex_array_object;
                    this === er.currentVAO && (er.currentVAO = null, jn.bindVertexArrayOES(null)), jn.deleteVertexArrayOES(this.vao), this.vao = null;
                  }
                  this.ownsElements && (this.elements.destroy(), this.elements = null, this.ownsElements = !1), kn[this.id] && (delete kn[this.id], --_n.vaoCount);
                }, er;
              }
              function De(Ot, un, yn, _n) {
                function gn(ur, Kn, Jn, Lr) {
                  this.name = ur, this.id = Kn, this.location = Jn, this.info = Lr;
                }
                function bn(ur, Kn) {
                  for (var Jn = 0; Jn < ur.length; ++Jn) if (ur[Jn].id === Kn.id) return void (ur[Jn].location = Kn.location);
                  ur.push(Kn);
                }
                function an(ur, Kn, Jn) {
                  if (!(mr = (Jn = ur === 35632 ? Ln : In)[Kn])) {
                    var Lr = un.str(Kn), mr = Ot.createShader(ur);
                    Ot.shaderSource(mr, Lr), Ot.compileShader(mr), Jn[Kn] = mr;
                  }
                  return mr;
                }
                function Zt(ur, Kn) {
                  this.id = jn++, this.fragId = ur, this.vertId = Kn, this.program = null, this.uniforms = [], this.attributes = [], this.refCount = 1, _n.profile && (this.stats = { uniformsCount: 0, attributesCount: 0 });
                }
                function Mn(ur, Kn, Jn) {
                  var Lr;
                  Lr = an(35632, ur.fragId);
                  var mr = an(35633, ur.vertId);
                  if (Kn = ur.program = Ot.createProgram(), Ot.attachShader(Kn, Lr), Ot.attachShader(Kn, mr), Jn) for (Lr = 0; Lr < Jn.length; ++Lr) mr = Jn[Lr], Ot.bindAttribLocation(Kn, mr[0], mr[1]);
                  Ot.linkProgram(Kn), mr = Ot.getProgramParameter(Kn, 35718), _n.profile && (ur.stats.uniformsCount = mr);
                  var Ar = ur.uniforms;
                  for (Lr = 0; Lr < mr; ++Lr) if (Jn = Ot.getActiveUniform(Kn, Lr)) if (1 < Jn.size) for (var Yr = 0; Yr < Jn.size; ++Yr) {
                    var jr = Jn.name.replace("[0]", "[" + Yr + "]");
                    bn(Ar, new gn(jr, un.id(jr), Ot.getUniformLocation(Kn, jr), Jn));
                  }
                  else bn(Ar, new gn(Jn.name, un.id(Jn.name), Ot.getUniformLocation(Kn, Jn.name), Jn));
                  for (mr = Ot.getProgramParameter(Kn, 35721), _n.profile && (ur.stats.attributesCount = mr), ur = ur.attributes, Lr = 0; Lr < mr; ++Lr) (Jn = Ot.getActiveAttrib(Kn, Lr)) && bn(ur, new gn(Jn.name, un.id(Jn.name), Ot.getAttribLocation(Kn, Jn.name), Jn));
                }
                var Ln = {}, In = {}, kn = {}, er = [], jn = 0;
                return _n.profile && (yn.getMaxUniformsCount = function() {
                  var ur = 0;
                  return er.forEach(function(Kn) {
                    Kn.stats.uniformsCount > ur && (ur = Kn.stats.uniformsCount);
                  }), ur;
                }, yn.getMaxAttributesCount = function() {
                  var ur = 0;
                  return er.forEach(function(Kn) {
                    Kn.stats.attributesCount > ur && (ur = Kn.stats.attributesCount);
                  }), ur;
                }), { clear: function() {
                  var ur = Ot.deleteShader.bind(Ot);
                  vt(Ln).forEach(ur), Ln = {}, vt(In).forEach(ur), In = {}, er.forEach(function(Kn) {
                    Ot.deleteProgram(Kn.program);
                  }), er.length = 0, kn = {}, yn.shaderCount = 0;
                }, program: function(ur, Kn, Jn, Lr) {
                  var mr = kn[Kn];
                  mr || (mr = kn[Kn] = {});
                  var Ar = mr[ur];
                  if (Ar && (Ar.refCount++, !Lr)) return Ar;
                  var Yr = new Zt(Kn, ur);
                  return yn.shaderCount++, Mn(Yr, Jn, Lr), Ar || (mr[ur] = Yr), er.push(Yr), it(Yr, { destroy: function() {
                    if (Yr.refCount--, 0 >= Yr.refCount) {
                      Ot.deleteProgram(Yr.program);
                      var jr = er.indexOf(Yr);
                      er.splice(jr, 1), yn.shaderCount--;
                    }
                    0 >= mr[Yr.vertId].refCount && (Ot.deleteShader(In[Yr.vertId]), delete In[Yr.vertId], delete kn[Yr.fragId][Yr.vertId]), Object.keys(kn[Yr.fragId]).length || (Ot.deleteShader(Ln[Yr.fragId]), delete Ln[Yr.fragId], delete kn[Yr.fragId]);
                  } });
                }, restore: function() {
                  Ln = {}, In = {};
                  for (var ur = 0; ur < er.length; ++ur) Mn(er[ur], null, er[ur].attributes.map(function(Kn) {
                    return [Kn.location, Kn.name];
                  }));
                }, shader: an, frag: -1, vert: -1 };
              }
              function Pe(Ot, un, yn, _n, gn, bn, an) {
                function Zt(Mn) {
                  var Ln;
                  Ln = un.next === null ? 5121 : un.next.colorAttachments[0].texture._texture.type;
                  var In = 0, kn = 0, er = _n.framebufferWidth, jn = _n.framebufferHeight, ur = null;
                  return Rt(Mn) ? ur = Mn : Mn && (In = 0 | Mn.x, kn = 0 | Mn.y, er = 0 | (Mn.width || _n.framebufferWidth - In), jn = 0 | (Mn.height || _n.framebufferHeight - kn), ur = Mn.data || null), yn(), Mn = er * jn * 4, ur || (Ln === 5121 ? ur = new Uint8Array(Mn) : Ln === 5126 && (ur = ur || new Float32Array(Mn))), Ot.pixelStorei(3333, 4), Ot.readPixels(In, kn, er, jn, 6408, Ln, ur), ur;
                }
                return function(Mn) {
                  return Mn && "framebuffer" in Mn ? function(Ln) {
                    var In;
                    return un.setFBO({ framebuffer: Ln.framebuffer }, function() {
                      In = Zt(Ln);
                    }), In;
                  }(Mn) : Zt(Mn);
                };
              }
              function $e(Ot, un) {
                return Ot >>> un | Ot << 32 - un;
              }
              function qe(Ot, un) {
                var yn = (65535 & Ot) + (65535 & un);
                return (Ot >> 16) + (un >> 16) + (yn >> 16) << 16 | 65535 & yn;
              }
              function Xe(Ot) {
                return Array.prototype.slice.call(Ot);
              }
              function Ve(Ot) {
                return Xe(Ot).join("");
              }
              function Ze(Ot) {
                function un() {
                  var In = [], kn = [];
                  return it(function() {
                    In.push.apply(In, Xe(arguments));
                  }, { def: function() {
                    var er = "v" + gn++;
                    return kn.push(er), 0 < arguments.length && (In.push(er, "="), In.push.apply(In, Xe(arguments)), In.push(";")), er;
                  }, toString: function() {
                    return Ve([0 < kn.length ? "var " + kn.join(",") + ";" : "", Ve(In)]);
                  } });
                }
                function yn() {
                  function In(Kn, Jn) {
                    er(Kn, Jn, "=", kn.def(Kn, Jn), ";");
                  }
                  var kn = un(), er = un(), jn = kn.toString, ur = er.toString;
                  return it(function() {
                    kn.apply(kn, Xe(arguments));
                  }, { def: kn.def, entry: kn, exit: er, save: In, set: function(Kn, Jn, Lr) {
                    In(Kn, Jn), kn(Kn, Jn, "=", Lr, ";");
                  }, toString: function() {
                    return jn() + ur();
                  } });
                }
                var _n = Ot.cache, gn = 0, bn = [], an = [], Zt = [], Mn = un(), Ln = {};
                return { global: Mn, link: function(In, kn) {
                  var er = kn && kn.stable;
                  if (!er) {
                    for (var jn = 0; jn < an.length; ++jn) if (an[jn] === In && !Zt[jn]) return bn[jn];
                  }
                  return jn = "g" + gn++, bn.push(jn), an.push(In), Zt.push(er), jn;
                }, block: un, proc: function(In, kn) {
                  function er() {
                    var Jn = "a" + jn.length;
                    return jn.push(Jn), Jn;
                  }
                  var jn = [];
                  kn = kn || 0;
                  for (var ur = 0; ur < kn; ++ur) er();
                  var Kn = (ur = yn()).toString;
                  return Ln[In] = it(ur, { arg: er, toString: function() {
                    return Ve(["function(", jn.join(), "){", Kn(), "}"]);
                  } });
                }, scope: yn, cond: function() {
                  var In = Ve(arguments), kn = yn(), er = yn(), jn = kn.toString, ur = er.toString;
                  return it(kn, { then: function() {
                    return kn.apply(kn, Xe(arguments)), this;
                  }, else: function() {
                    return er.apply(er, Xe(arguments)), this;
                  }, toString: function() {
                    var Kn = ur();
                    return Kn && (Kn = "else{" + Kn + "}"), Ve(["if(", In, "){", jn(), "}", Kn]);
                  } });
                }, compile: function() {
                  var In = ['"use strict";', Mn, "return {"];
                  Object.keys(Ln).forEach(function(jn) {
                    In.push('"', jn, '":', Ln[jn].toString(), ",");
                  }), In.push("}");
                  var kn, er = Ve(In).replace(/;/g, `;
`).replace(/}/g, `}
`).replace(/{/g, `{
`);
                  return _n && (kn = function(jn) {
                    for (var ur, Kn = "", Jn = 0; Jn < jn.length; Jn++) ur = jn.charCodeAt(Jn), Kn += "0123456789abcdef".charAt(ur >>> 4 & 15) + "0123456789abcdef".charAt(15 & ur);
                    return Kn;
                  }(function(jn) {
                    for (var ur = Array(jn.length >> 2), Kn = 0; Kn < ur.length; Kn++) ur[Kn] = 0;
                    for (Kn = 0; Kn < 8 * jn.length; Kn += 8) ur[Kn >> 5] |= (255 & jn.charCodeAt(Kn / 8)) << 24 - Kn % 32;
                    var Jn, Lr, mr, Ar, Yr, jr, Mr, Wr, Bi, gi, Zr, fi = 8 * jn.length;
                    for (jn = [1779033703, -1150833019, 1013904242, -1521486534, 1359893119, -1694144372, 528734635, 1541459225], Kn = Array(64), ur[fi >> 5] |= 128 << 24 - fi % 32, ur[15 + (fi + 64 >> 9 << 4)] = fi, Wr = 0; Wr < ur.length; Wr += 16) {
                      for (fi = jn[0], Jn = jn[1], Lr = jn[2], mr = jn[3], Ar = jn[4], Yr = jn[5], jr = jn[6], Mr = jn[7], Bi = 0; 64 > Bi; Bi++) {
                        var ri;
                        16 > Bi ? Kn[Bi] = ur[Bi + Wr] : (gi = Bi, Zr = qe(Zr = $e(Zr = Kn[Bi - 2], 17) ^ $e(Zr, 19) ^ Zr >>> 10, Kn[Bi - 7]), ri = $e(ri = Kn[Bi - 15], 7) ^ $e(ri, 18) ^ ri >>> 3, Kn[gi] = qe(qe(Zr, ri), Kn[Bi - 16])), gi = qe(qe(qe(qe(Mr, gi = $e(gi = Ar, 6) ^ $e(gi, 11) ^ $e(gi, 25)), Ar & Yr ^ ~Ar & jr), Ie[Bi]), Kn[Bi]), Zr = qe(Mr = $e(Mr = fi, 2) ^ $e(Mr, 13) ^ $e(Mr, 22), fi & Jn ^ fi & Lr ^ Jn & Lr), Mr = jr, jr = Yr, Yr = Ar, Ar = qe(mr, gi), mr = Lr, Lr = Jn, Jn = fi, fi = qe(gi, Zr);
                      }
                      jn[0] = qe(fi, jn[0]), jn[1] = qe(Jn, jn[1]), jn[2] = qe(Lr, jn[2]), jn[3] = qe(mr, jn[3]), jn[4] = qe(Ar, jn[4]), jn[5] = qe(Yr, jn[5]), jn[6] = qe(jr, jn[6]), jn[7] = qe(Mr, jn[7]);
                    }
                    for (ur = "", Kn = 0; Kn < 32 * jn.length; Kn += 8) ur += String.fromCharCode(jn[Kn >> 5] >>> 24 - Kn % 32 & 255);
                    return ur;
                  }(function(jn) {
                    for (var ur, Kn, Jn = "", Lr = -1; ++Lr < jn.length; ) ur = jn.charCodeAt(Lr), Kn = Lr + 1 < jn.length ? jn.charCodeAt(Lr + 1) : 0, 55296 <= ur && 56319 >= ur && 56320 <= Kn && 57343 >= Kn && (ur = 65536 + ((1023 & ur) << 10) + (1023 & Kn), Lr++), 127 >= ur ? Jn += String.fromCharCode(ur) : 2047 >= ur ? Jn += String.fromCharCode(192 | ur >>> 6 & 31, 128 | 63 & ur) : 65535 >= ur ? Jn += String.fromCharCode(224 | ur >>> 12 & 15, 128 | ur >>> 6 & 63, 128 | 63 & ur) : 2097151 >= ur && (Jn += String.fromCharCode(240 | ur >>> 18 & 7, 128 | ur >>> 12 & 63, 128 | ur >>> 6 & 63, 128 | 63 & ur));
                    return Jn;
                  }(er))), _n[kn]) ? _n[kn].apply(null, an) : (er = Function.apply(null, bn.concat(er)), _n && (_n[kn] = er), er.apply(null, an));
                } };
              }
              function Ge(Ot) {
                return Array.isArray(Ot) || Rt(Ot) || le(Ot);
              }
              function ze(Ot) {
                return Ot.sort(function(un, yn) {
                  return un === "viewport" ? -1 : yn === "viewport" ? 1 : un < yn ? -1 : 1;
                });
              }
              function We(Ot, un, yn, _n) {
                this.thisDep = Ot, this.contextDep = un, this.propDep = yn, this.append = _n;
              }
              function at(Ot) {
                return Ot && !(Ot.thisDep || Ot.contextDep || Ot.propDep);
              }
              function ct(Ot) {
                return new We(!1, !1, !1, Ot);
              }
              function et(Ot, un) {
                var yn = Ot.type;
                if (yn === 0) return new We(!0, 1 <= (yn = Ot.data.length), 2 <= yn, un);
                if (yn === 4) return new We((yn = Ot.data).thisDep, yn.contextDep, yn.propDep, un);
                if (yn === 5) return new We(!1, !1, !1, un);
                if (yn === 6) {
                  for (var _n = yn = !1, gn = !1, bn = 0; bn < Ot.data.length; ++bn) {
                    var an = Ot.data[bn];
                    an.type === 1 ? gn = !0 : an.type === 2 ? _n = !0 : an.type === 3 ? yn = !0 : an.type === 0 ? (yn = !0, 1 <= (an = an.data) && (_n = !0), 2 <= an && (gn = !0)) : an.type === 4 && (yn = yn || an.data.thisDep, _n = _n || an.data.contextDep, gn = gn || an.data.propDep);
                  }
                  return new We(yn, _n, gn, un);
                }
                return new We(yn === 3, yn === 2, yn === 1, un);
              }
              function dt(Ot, un, yn, _n, gn, bn, an, Zt, Mn, Ln, In, kn, er, jn, ur, Kn) {
                function Jn(rr) {
                  return rr.replace(".", "_");
                }
                function Lr(rr, vr, Rr) {
                  var Vr = Jn(rr);
                  Da.push(rr), di[Vr] = zr[Vr] = !!Rr, la[Vr] = vr;
                }
                function mr(rr, vr, Rr) {
                  var Vr = Jn(rr);
                  Da.push(rr), Array.isArray(Rr) ? (zr[Vr] = Rr.slice(), di[Vr] = Rr.slice()) : zr[Vr] = di[Vr] = Rr, wa[Vr] = vr;
                }
                function Ar() {
                  var rr = Ze({ cache: ur }), vr = rr.link, Rr = rr.global;
                  rr.id = Ur++, rr.batchId = "0";
                  var Vr = vr(Wa), Xr = rr.shared = { props: "a0" };
                  Object.keys(Wa).forEach(function(gr) {
                    Xr[gr] = Rr.def(Vr, ".", gr);
                  });
                  var Jr = rr.next = {}, ti = rr.current = {};
                  Object.keys(wa).forEach(function(gr) {
                    Array.isArray(zr[gr]) && (Jr[gr] = Rr.def(Xr.next, ".", gr), ti[gr] = Rr.def(Xr.current, ".", gr));
                  });
                  var Hr = rr.constants = {};
                  Object.keys(Qr).forEach(function(gr) {
                    Hr[gr] = Rr.def(JSON.stringify(Qr[gr]));
                  }), rr.invoke = function(gr, $r) {
                    switch ($r.type) {
                      case 0:
                        var li = ["this", Xr.context, Xr.props, rr.batchId];
                        return gr.def(vr($r.data), ".call(", li.slice(0, Math.max($r.data.length + 1, 4)), ")");
                      case 1:
                        return gr.def(Xr.props, $r.data);
                      case 2:
                        return gr.def(Xr.context, $r.data);
                      case 3:
                        return gr.def("this", $r.data);
                      case 4:
                        return $r.data.append(rr, gr), $r.data.ref;
                      case 5:
                        return $r.data.toString();
                      case 6:
                        return $r.data.map(function(Qi) {
                          return rr.invoke(gr, Qi);
                        });
                    }
                  }, rr.attribCache = {};
                  var Er = {};
                  return rr.scopeAttrib = function(gr) {
                    if ((gr = un.id(gr)) in Er) return Er[gr];
                    var $r = Ln.scope[gr];
                    return $r || ($r = Ln.scope[gr] = new Gr()), Er[gr] = vr($r);
                  }, rr;
                }
                function Yr(rr, vr) {
                  var Rr = rr.static, Vr = rr.dynamic;
                  if ("framebuffer" in Rr) {
                    var Xr = Rr.framebuffer;
                    return Xr ? (Xr = Zt.getFramebuffer(Xr), ct(function(ti, Hr) {
                      var Er = ti.link(Xr), gr = ti.shared;
                      return Hr.set(gr.framebuffer, ".next", Er), gr = gr.context, Hr.set(gr, ".framebufferWidth", Er + ".width"), Hr.set(gr, ".framebufferHeight", Er + ".height"), Er;
                    })) : ct(function(ti, Hr) {
                      var Er = ti.shared;
                      return Hr.set(Er.framebuffer, ".next", "null"), Er = Er.context, Hr.set(Er, ".framebufferWidth", Er + ".drawingBufferWidth"), Hr.set(Er, ".framebufferHeight", Er + ".drawingBufferHeight"), "null";
                    });
                  }
                  if ("framebuffer" in Vr) {
                    var Jr = Vr.framebuffer;
                    return et(Jr, function(ti, Hr) {
                      var Er = ti.invoke(Hr, Jr), gr = ti.shared, $r = gr.framebuffer;
                      return Er = Hr.def($r, ".getFramebuffer(", Er, ")"), Hr.set($r, ".next", Er), gr = gr.context, Hr.set(gr, ".framebufferWidth", Er + "?" + Er + ".width:" + gr + ".drawingBufferWidth"), Hr.set(gr, ".framebufferHeight", Er + "?" + Er + ".height:" + gr + ".drawingBufferHeight"), Er;
                    });
                  }
                  return null;
                }
                function jr(rr, vr, Rr) {
                  function Vr(gr) {
                    if (gr in Xr) {
                      var $r = un.id(Xr[gr]);
                      return (gr = ct(function() {
                        return $r;
                      })).id = $r, gr;
                    }
                    if (gr in Jr) {
                      var li = Jr[gr];
                      return et(li, function(Qi, Ai) {
                        var Li = Qi.invoke(Ai, li);
                        return Ai.def(Qi.shared.strings, ".id(", Li, ")");
                      });
                    }
                    return null;
                  }
                  var Xr = rr.static, Jr = rr.dynamic, ti = Vr("frag"), Hr = Vr("vert"), Er = null;
                  return at(ti) && at(Hr) ? (Er = In.program(Hr.id, ti.id, null, Rr), rr = ct(function(gr, $r) {
                    return gr.link(Er);
                  })) : rr = new We(ti && ti.thisDep || Hr && Hr.thisDep, ti && ti.contextDep || Hr && Hr.contextDep, ti && ti.propDep || Hr && Hr.propDep, function(gr, $r) {
                    var li, Qi, Ai = gr.shared.shader;
                    return li = ti ? ti.append(gr, $r) : $r.def(Ai, ".", "frag"), Qi = Hr ? Hr.append(gr, $r) : $r.def(Ai, ".", "vert"), $r.def(Ai + ".program(" + Qi + "," + li + ")");
                  }), { frag: ti, vert: Hr, progVar: rr, program: Er };
                }
                function Mr(rr, vr) {
                  function Rr(Li, Ni) {
                    if (Li in Vr) {
                      var Pi = 0 | Vr[Li];
                      return Ni ? Jr.offset = Pi : Jr.instances = Pi, ct(function(ki, Mi) {
                        return Ni && (ki.OFFSET = Pi), Pi;
                      });
                    }
                    if (Li in Xr) {
                      var Oi = Xr[Li];
                      return et(Oi, function(ki, Mi) {
                        var Vi = ki.invoke(Mi, Oi);
                        return Ni && (ki.OFFSET = Vi), Vi;
                      });
                    }
                    if (Ni) {
                      if (Er) return ct(function(ki, Mi) {
                        return ki.OFFSET = 0;
                      });
                      if (ti) return new We(Hr.thisDep, Hr.contextDep, Hr.propDep, function(ki, Mi) {
                        return Mi.def(ki.shared.vao + ".currentVAO?" + ki.shared.vao + ".currentVAO.offset:0");
                      });
                    } else if (ti) return new We(Hr.thisDep, Hr.contextDep, Hr.propDep, function(ki, Mi) {
                      return Mi.def(ki.shared.vao + ".currentVAO?" + ki.shared.vao + ".currentVAO.instances:-1");
                    });
                    return null;
                  }
                  var Vr = rr.static, Xr = rr.dynamic, Jr = {}, ti = !1, Hr = function() {
                    if ("vao" in Vr) {
                      var Li = Vr.vao;
                      return Li !== null && Ln.getVAO(Li) === null && (Li = Ln.createVAO(Li)), ti = !0, Jr.vao = Li, ct(function(Pi) {
                        var Oi = Ln.getVAO(Li);
                        return Oi ? Pi.link(Oi) : "null";
                      });
                    }
                    if ("vao" in Xr) {
                      ti = !0;
                      var Ni = Xr.vao;
                      return et(Ni, function(Pi, Oi) {
                        var ki = Pi.invoke(Oi, Ni);
                        return Oi.def(Pi.shared.vao + ".getVAO(" + ki + ")");
                      });
                    }
                    return null;
                  }(), Er = !1, gr = function() {
                    if ("elements" in Vr) {
                      var Li = Vr.elements;
                      if (Jr.elements = Li, Ge(Li)) {
                        var Ni = Jr.elements = bn.create(Li, !0);
                        Li = bn.getElements(Ni), Er = !0;
                      } else Li && (Li = bn.getElements(Li), Er = !0);
                      return Ni = ct(function(Oi, ki) {
                        if (Li) {
                          var Mi = Oi.link(Li);
                          return Oi.ELEMENTS = Mi;
                        }
                        return Oi.ELEMENTS = null;
                      }), Ni.value = Li, Ni;
                    }
                    if ("elements" in Xr) {
                      Er = !0;
                      var Pi = Xr.elements;
                      return et(Pi, function(Oi, ki) {
                        var Mi = (Vi = Oi.shared).isBufferArgs, Vi = Vi.elements, ua = Oi.invoke(ki, Pi), Ki = ki.def("null");
                        return Mi = ki.def(Mi, "(", ua, ")"), ua = Oi.cond(Mi).then(Ki, "=", Vi, ".createStream(", ua, ");").else(Ki, "=", Vi, ".getElements(", ua, ");"), ki.entry(ua), ki.exit(Oi.cond(Mi).then(Vi, ".destroyStream(", Ki, ");")), Oi.ELEMENTS = Ki;
                      });
                    }
                    return ti ? new We(Hr.thisDep, Hr.contextDep, Hr.propDep, function(Oi, ki) {
                      return ki.def(Oi.shared.vao + ".currentVAO?" + Oi.shared.elements + ".getElements(" + Oi.shared.vao + ".currentVAO.elements):null");
                    }) : null;
                  }(), $r = Rr("offset", !0), li = function() {
                    if ("primitive" in Vr) {
                      var Li = Vr.primitive;
                      return Jr.primitive = Li, ct(function(Pi, Oi) {
                        return It[Li];
                      });
                    }
                    if ("primitive" in Xr) {
                      var Ni = Xr.primitive;
                      return et(Ni, function(Pi, Oi) {
                        var ki = Pi.constants.primTypes, Mi = Pi.invoke(Oi, Ni);
                        return Oi.def(ki, "[", Mi, "]");
                      });
                    }
                    return Er ? at(gr) ? gr.value ? ct(function(Pi, Oi) {
                      return Oi.def(Pi.ELEMENTS, ".primType");
                    }) : ct(function() {
                      return 4;
                    }) : new We(gr.thisDep, gr.contextDep, gr.propDep, function(Pi, Oi) {
                      var ki = Pi.ELEMENTS;
                      return Oi.def(ki, "?", ki, ".primType:", 4);
                    }) : ti ? new We(Hr.thisDep, Hr.contextDep, Hr.propDep, function(Pi, Oi) {
                      return Oi.def(Pi.shared.vao + ".currentVAO?" + Pi.shared.vao + ".currentVAO.primitive:4");
                    }) : null;
                  }(), Qi = function() {
                    if ("count" in Vr) {
                      var Li = 0 | Vr.count;
                      return Jr.count = Li, ct(function() {
                        return Li;
                      });
                    }
                    if ("count" in Xr) {
                      var Ni = Xr.count;
                      return et(Ni, function(Pi, Oi) {
                        return Pi.invoke(Oi, Ni);
                      });
                    }
                    return Er ? at(gr) ? gr ? $r ? new We($r.thisDep, $r.contextDep, $r.propDep, function(Pi, Oi) {
                      return Oi.def(Pi.ELEMENTS, ".vertCount-", Pi.OFFSET);
                    }) : ct(function(Pi, Oi) {
                      return Oi.def(Pi.ELEMENTS, ".vertCount");
                    }) : ct(function() {
                      return -1;
                    }) : new We(gr.thisDep || $r.thisDep, gr.contextDep || $r.contextDep, gr.propDep || $r.propDep, function(Pi, Oi) {
                      var ki = Pi.ELEMENTS;
                      return Pi.OFFSET ? Oi.def(ki, "?", ki, ".vertCount-", Pi.OFFSET, ":-1") : Oi.def(ki, "?", ki, ".vertCount:-1");
                    }) : ti ? new We(Hr.thisDep, Hr.contextDep, Hr.propDep, function(Pi, Oi) {
                      return Oi.def(Pi.shared.vao, ".currentVAO?", Pi.shared.vao, ".currentVAO.count:-1");
                    }) : null;
                  }(), Ai = Rr("instances", !1);
                  return { elements: gr, primitive: li, count: Qi, instances: Ai, offset: $r, vao: Hr, vaoActive: ti, elementsActive: Er, static: Jr };
                }
                function Wr(rr, vr) {
                  var Rr = rr.static, Vr = rr.dynamic, Xr = {};
                  return Object.keys(Rr).forEach(function(Jr) {
                    var ti = Rr[Jr], Hr = un.id(Jr), Er = new Gr();
                    if (Ge(ti)) Er.state = 1, Er.buffer = gn.getBuffer(gn.create(ti, 34962, !1, !0)), Er.type = 0;
                    else if ($r = gn.getBuffer(ti)) Er.state = 1, Er.buffer = $r, Er.type = 0;
                    else if ("constant" in ti) {
                      var gr = ti.constant;
                      Er.buffer = "null", Er.state = 2, typeof gr == "number" ? Er.x = gr : rt.forEach(function(Pi, Oi) {
                        Oi < gr.length && (Er[Pi] = gr[Oi]);
                      });
                    } else {
                      var $r = Ge(ti.buffer) ? gn.getBuffer(gn.create(ti.buffer, 34962, !1, !0)) : gn.getBuffer(ti.buffer), li = 0 | ti.offset, Qi = 0 | ti.stride, Ai = 0 | ti.size, Li = !!ti.normalized, Ni = 0;
                      "type" in ti && (Ni = Xt[ti.type]), ti = 0 | ti.divisor, Er.buffer = $r, Er.state = 1, Er.size = Ai, Er.normalized = Li, Er.type = Ni || $r.dtype, Er.offset = li, Er.stride = Qi, Er.divisor = ti;
                    }
                    Xr[Jr] = ct(function(Pi, Oi) {
                      var ki = Pi.attribCache;
                      if (Hr in ki) return ki[Hr];
                      var Mi = { isStream: !1 };
                      return Object.keys(Er).forEach(function(Vi) {
                        Mi[Vi] = Er[Vi];
                      }), Er.buffer && (Mi.buffer = Pi.link(Er.buffer), Mi.type = Mi.type || Mi.buffer + ".dtype"), ki[Hr] = Mi;
                    });
                  }), Object.keys(Vr).forEach(function(Jr) {
                    var ti = Vr[Jr];
                    Xr[Jr] = et(ti, function(Hr, Er) {
                      function gr(ki) {
                        Er(Li[ki], "=", $r, ".", ki, "|0;");
                      }
                      var $r = Hr.invoke(Er, ti), li = Hr.shared, Qi = Hr.constants, Ai = li.isBufferArgs, Li = (li = li.buffer, { isStream: Er.def(!1) }), Ni = new Gr();
                      Ni.state = 1, Object.keys(Ni).forEach(function(ki) {
                        Li[ki] = Er.def("" + Ni[ki]);
                      });
                      var Pi = Li.buffer, Oi = Li.type;
                      return Er("if(", Ai, "(", $r, ")){", Li.isStream, "=true;", Pi, "=", li, ".createStream(", 34962, ",", $r, ");", Oi, "=", Pi, ".dtype;", "}else{", Pi, "=", li, ".getBuffer(", $r, ");", "if(", Pi, "){", Oi, "=", Pi, ".dtype;", '}else if("constant" in ', $r, "){", Li.state, "=", 2, ";", "if(typeof " + $r + '.constant === "number"){', Li[rt[0]], "=", $r, ".constant;", rt.slice(1).map(function(ki) {
                        return Li[ki];
                      }).join("="), "=0;", "}else{", rt.map(function(ki, Mi) {
                        return Li[ki] + "=" + $r + ".constant.length>" + Mi + "?" + $r + ".constant[" + Mi + "]:0;";
                      }).join(""), "}}else{", "if(", Ai, "(", $r, ".buffer)){", Pi, "=", li, ".createStream(", 34962, ",", $r, ".buffer);", "}else{", Pi, "=", li, ".getBuffer(", $r, ".buffer);", "}", Oi, '="type" in ', $r, "?", Qi.glTypes, "[", $r, ".type]:", Pi, ".dtype;", Li.normalized, "=!!", $r, ".normalized;"), gr("size"), gr("offset"), gr("stride"), gr("divisor"), Er("}}"), Er.exit("if(", Li.isStream, "){", li, ".destroyStream(", Pi, ");", "}"), Li;
                    });
                  }), Xr;
                }
                function Bi(rr, vr, Rr, Vr, Xr) {
                  function Jr(Ni) {
                    var Pi = Hr[Ni];
                    Pi && (gr[Ni] = Pi);
                  }
                  var ti = function(Ni, Pi) {
                    if (typeof (Oi = Ni.static).frag == "string" && typeof Oi.vert == "string") {
                      if (0 < Object.keys(Pi.dynamic).length) return null;
                      var Oi = Pi.static, ki = Object.keys(Oi);
                      if (0 < ki.length && typeof Oi[ki[0]] == "number") {
                        for (var Mi = [], Vi = 0; Vi < ki.length; ++Vi) Mi.push([0 | Oi[ki[Vi]], ki[Vi]]);
                        return Mi;
                      }
                    }
                    return null;
                  }(rr, vr), Hr = function(Ni, Pi, Oi) {
                    function ki(Ki) {
                      if (Ki in Mi) {
                        var Fi = Mi[Ki];
                        Ki = !0;
                        var na, nn, Tn = 0 | Fi.x, Dn = 0 | Fi.y;
                        return "width" in Fi ? na = 0 | Fi.width : Ki = !1, "height" in Fi ? nn = 0 | Fi.height : Ki = !1, new We(!Ki && Pi && Pi.thisDep, !Ki && Pi && Pi.contextDep, !Ki && Pi && Pi.propDep, function(fr, dr) {
                          var yr = fr.shared.context, ai = na;
                          "width" in Fi || (ai = dr.def(yr, ".", "framebufferWidth", "-", Tn));
                          var Fr = nn;
                          return "height" in Fi || (Fr = dr.def(yr, ".", "framebufferHeight", "-", Dn)), [Tn, Dn, ai, Fr];
                        });
                      }
                      if (Ki in Vi) {
                        var Fn = Vi[Ki];
                        return Ki = et(Fn, function(fr, dr) {
                          var yr = fr.invoke(dr, Fn), ai = fr.shared.context, Fr = dr.def(yr, ".x|0"), Ti = dr.def(yr, ".y|0");
                          return [Fr, Ti, dr.def('"width" in ', yr, "?", yr, ".width|0:", "(", ai, ".", "framebufferWidth", "-", Fr, ")"), yr = dr.def('"height" in ', yr, "?", yr, ".height|0:", "(", ai, ".", "framebufferHeight", "-", Ti, ")")];
                        }), Pi && (Ki.thisDep = Ki.thisDep || Pi.thisDep, Ki.contextDep = Ki.contextDep || Pi.contextDep, Ki.propDep = Ki.propDep || Pi.propDep), Ki;
                      }
                      return Pi ? new We(Pi.thisDep, Pi.contextDep, Pi.propDep, function(fr, dr) {
                        var yr = fr.shared.context;
                        return [0, 0, dr.def(yr, ".", "framebufferWidth"), dr.def(yr, ".", "framebufferHeight")];
                      }) : null;
                    }
                    var Mi = Ni.static, Vi = Ni.dynamic;
                    if (Ni = ki("viewport")) {
                      var ua = Ni;
                      Ni = new We(Ni.thisDep, Ni.contextDep, Ni.propDep, function(Ki, Fi) {
                        var na = ua.append(Ki, Fi), nn = Ki.shared.context;
                        return Fi.set(nn, ".viewportWidth", na[2]), Fi.set(nn, ".viewportHeight", na[3]), na;
                      });
                    }
                    return { viewport: Ni, scissor_box: ki("scissor.box") };
                  }(rr, li = Yr(rr)), Er = Mr(rr), gr = function(Ni, Pi) {
                    var Oi = Ni.static, ki = Ni.dynamic, Mi = {};
                    return Da.forEach(function(Vi) {
                      function ua(Fi, na) {
                        if (Vi in Oi) {
                          var nn = Fi(Oi[Vi]);
                          Mi[Ki] = ct(function() {
                            return nn;
                          });
                        } else if (Vi in ki) {
                          var Tn = ki[Vi];
                          Mi[Ki] = et(Tn, function(Dn, Fn) {
                            return na(Dn, Fn, Dn.invoke(Fn, Tn));
                          });
                        }
                      }
                      var Ki = Jn(Vi);
                      switch (Vi) {
                        case "cull.enable":
                        case "blend.enable":
                        case "dither":
                        case "stencil.enable":
                        case "depth.enable":
                        case "scissor.enable":
                        case "polygonOffset.enable":
                        case "sample.alpha":
                        case "sample.enable":
                        case "depth.mask":
                        case "lineWidth":
                          return ua(function(Fi) {
                            return Fi;
                          }, function(Fi, na, nn) {
                            return nn;
                          });
                        case "depth.func":
                          return ua(function(Fi) {
                            return wt[Fi];
                          }, function(Fi, na, nn) {
                            return na.def(Fi.constants.compareFuncs, "[", nn, "]");
                          });
                        case "depth.range":
                          return ua(function(Fi) {
                            return Fi;
                          }, function(Fi, na, nn) {
                            return [na.def("+", nn, "[0]"), na = na.def("+", nn, "[1]")];
                          });
                        case "blend.func":
                          return ua(function(Fi) {
                            return [pt["srcRGB" in Fi ? Fi.srcRGB : Fi.src], pt["dstRGB" in Fi ? Fi.dstRGB : Fi.dst], pt["srcAlpha" in Fi ? Fi.srcAlpha : Fi.src], pt["dstAlpha" in Fi ? Fi.dstAlpha : Fi.dst]];
                          }, function(Fi, na, nn) {
                            function Tn(dr, yr) {
                              return na.def('"', dr, yr, '" in ', nn, "?", nn, ".", dr, yr, ":", nn, ".", dr);
                            }
                            Fi = Fi.constants.blendFuncs;
                            var Dn = Tn("src", "RGB"), Fn = Tn("dst", "RGB"), fr = (Dn = na.def(Fi, "[", Dn, "]"), na.def(Fi, "[", Tn("src", "Alpha"), "]"));
                            return [Dn, Fn = na.def(Fi, "[", Fn, "]"), fr, Fi = na.def(Fi, "[", Tn("dst", "Alpha"), "]")];
                          });
                        case "blend.equation":
                          return ua(function(Fi) {
                            return typeof Fi == "string" ? [Ca[Fi], Ca[Fi]] : typeof Fi == "object" ? [Ca[Fi.rgb], Ca[Fi.alpha]] : void 0;
                          }, function(Fi, na, nn) {
                            var Tn = Fi.constants.blendEquations, Dn = na.def(), Fn = na.def();
                            return (Fi = Fi.cond("typeof ", nn, '==="string"')).then(Dn, "=", Fn, "=", Tn, "[", nn, "];"), Fi.else(Dn, "=", Tn, "[", nn, ".rgb];", Fn, "=", Tn, "[", nn, ".alpha];"), na(Fi), [Dn, Fn];
                          });
                        case "blend.color":
                          return ua(function(Fi) {
                            return oe(4, function(na) {
                              return +Fi[na];
                            });
                          }, function(Fi, na, nn) {
                            return oe(4, function(Tn) {
                              return na.def("+", nn, "[", Tn, "]");
                            });
                          });
                        case "stencil.mask":
                          return ua(function(Fi) {
                            return 0 | Fi;
                          }, function(Fi, na, nn) {
                            return na.def(nn, "|0");
                          });
                        case "stencil.func":
                          return ua(function(Fi) {
                            return [wt[Fi.cmp || "keep"], Fi.ref || 0, "mask" in Fi ? Fi.mask : -1];
                          }, function(Fi, na, nn) {
                            return [Fi = na.def('"cmp" in ', nn, "?", Fi.constants.compareFuncs, "[", nn, ".cmp]", ":", 7680), na.def(nn, ".ref|0"), na = na.def('"mask" in ', nn, "?", nn, ".mask|0:-1")];
                          });
                        case "stencil.opFront":
                        case "stencil.opBack":
                          return ua(function(Fi) {
                            return [Vi === "stencil.opBack" ? 1029 : 1028, Bt[Fi.fail || "keep"], Bt[Fi.zfail || "keep"], Bt[Fi.zpass || "keep"]];
                          }, function(Fi, na, nn) {
                            function Tn(Fn) {
                              return na.def('"', Fn, '" in ', nn, "?", Dn, "[", nn, ".", Fn, "]:", 7680);
                            }
                            var Dn = Fi.constants.stencilOps;
                            return [Vi === "stencil.opBack" ? 1029 : 1028, Tn("fail"), Tn("zfail"), Tn("zpass")];
                          });
                        case "polygonOffset.offset":
                          return ua(function(Fi) {
                            return [0 | Fi.factor, 0 | Fi.units];
                          }, function(Fi, na, nn) {
                            return [na.def(nn, ".factor|0"), na = na.def(nn, ".units|0")];
                          });
                        case "cull.face":
                          return ua(function(Fi) {
                            var na = 0;
                            return Fi === "front" ? na = 1028 : Fi === "back" && (na = 1029), na;
                          }, function(Fi, na, nn) {
                            return na.def(nn, '==="front"?', 1028, ":", 1029);
                          });
                        case "frontFace":
                          return ua(function(Fi) {
                            return At[Fi];
                          }, function(Fi, na, nn) {
                            return na.def(nn + '==="cw"?2304:2305');
                          });
                        case "colorMask":
                          return ua(function(Fi) {
                            return Fi.map(function(na) {
                              return !!na;
                            });
                          }, function(Fi, na, nn) {
                            return oe(4, function(Tn) {
                              return "!!" + nn + "[" + Tn + "]";
                            });
                          });
                        case "sample.coverage":
                          return ua(function(Fi) {
                            return ["value" in Fi ? Fi.value : 1, !!Fi.invert];
                          }, function(Fi, na, nn) {
                            return [na.def('"value" in ', nn, "?+", nn, ".value:1"), na = na.def("!!", nn, ".invert")];
                          });
                      }
                    }), Mi;
                  }(rr), $r = jr(rr, 0, ti);
                  Jr("viewport"), Jr(Jn("scissor.box"));
                  var li, Qi = 0 < Object.keys(gr).length;
                  if ((li = { framebuffer: li, draw: Er, shader: $r, state: gr, dirty: Qi, scopeVAO: null, drawVAO: null, useVAO: !1, attributes: {} }).profile = function(Ni) {
                    var Pi, Oi = Ni.static;
                    if (Ni = Ni.dynamic, "profile" in Oi) {
                      var ki = !!Oi.profile;
                      (Pi = ct(function(Vi, ua) {
                        return ki;
                      })).enable = ki;
                    } else if ("profile" in Ni) {
                      var Mi = Ni.profile;
                      Pi = et(Mi, function(Vi, ua) {
                        return Vi.invoke(ua, Mi);
                      });
                    }
                    return Pi;
                  }(rr), li.uniforms = function(Ni, Pi) {
                    var Oi = Ni.static, ki = Ni.dynamic, Mi = {};
                    return Object.keys(Oi).forEach(function(Vi) {
                      var ua, Ki = Oi[Vi];
                      if (typeof Ki == "number" || typeof Ki == "boolean") ua = ct(function() {
                        return Ki;
                      });
                      else if (typeof Ki == "function") {
                        var Fi = Ki._reglType;
                        Fi === "texture2d" || Fi === "textureCube" ? ua = ct(function(na) {
                          return na.link(Ki);
                        }) : Fi !== "framebuffer" && Fi !== "framebufferCube" || (ua = ct(function(na) {
                          return na.link(Ki.color[0]);
                        }));
                      } else ve(Ki) && (ua = ct(function(na) {
                        return na.global.def("[", oe(Ki.length, function(nn) {
                          return Ki[nn];
                        }), "]");
                      }));
                      ua.value = Ki, Mi[Vi] = ua;
                    }), Object.keys(ki).forEach(function(Vi) {
                      var ua = ki[Vi];
                      Mi[Vi] = et(ua, function(Ki, Fi) {
                        return Ki.invoke(Fi, ua);
                      });
                    }), Mi;
                  }(Rr), li.drawVAO = li.scopeVAO = Er.vao, !li.drawVAO && $r.program && !ti && yn.angle_instanced_arrays && Er.static.elements) {
                    var Ai = !0;
                    if (rr = $r.program.attributes.map(function(Ni) {
                      return Ni = vr.static[Ni], Ai = Ai && !!Ni, Ni;
                    }), Ai && 0 < rr.length) {
                      var Li = Ln.getVAO(Ln.createVAO({ attributes: rr, elements: Er.static.elements }));
                      li.drawVAO = new We(null, null, null, function(Ni, Pi) {
                        return Ni.link(Li);
                      }), li.useVAO = !0;
                    }
                  }
                  return ti ? li.useVAO = !0 : li.attributes = Wr(vr), li.context = function(Ni) {
                    var Pi = Ni.static, Oi = Ni.dynamic, ki = {};
                    return Object.keys(Pi).forEach(function(Mi) {
                      var Vi = Pi[Mi];
                      ki[Mi] = ct(function(ua, Ki) {
                        return typeof Vi == "number" || typeof Vi == "boolean" ? "" + Vi : ua.link(Vi);
                      });
                    }), Object.keys(Oi).forEach(function(Mi) {
                      var Vi = Oi[Mi];
                      ki[Mi] = et(Vi, function(ua, Ki) {
                        return ua.invoke(Ki, Vi);
                      });
                    }), ki;
                  }(Vr), li;
                }
                function gi(rr, vr, Rr) {
                  var Vr = rr.shared.context, Xr = rr.scope();
                  Object.keys(Rr).forEach(function(Jr) {
                    vr.save(Vr, "." + Jr);
                    var ti = Rr[Jr].append(rr, vr);
                    Array.isArray(ti) ? Xr(Vr, ".", Jr, "=[", ti.join(), "];") : Xr(Vr, ".", Jr, "=", ti, ";");
                  }), vr(Xr);
                }
                function Zr(rr, vr, Rr, Vr) {
                  var Xr, Jr = (Hr = rr.shared).gl, ti = Hr.framebuffer;
                  Dr && (Xr = vr.def(Hr.extensions, ".webgl_draw_buffers"));
                  var Hr = (Er = rr.constants).drawBuffer, Er = Er.backBuffer;
                  rr = Rr ? Rr.append(rr, vr) : vr.def(ti, ".next"), Vr || vr("if(", rr, "!==", ti, ".cur){"), vr("if(", rr, "){", Jr, ".bindFramebuffer(", 36160, ",", rr, ".framebuffer);"), Dr && vr(Xr, ".drawBuffersWEBGL(", Hr, "[", rr, ".colorAttachments.length]);"), vr("}else{", Jr, ".bindFramebuffer(", 36160, ",null);"), Dr && vr(Xr, ".drawBuffersWEBGL(", Er, ");"), vr("}", ti, ".cur=", rr, ";"), Vr || vr("}");
                }
                function fi(rr, vr, Rr) {
                  var Vr = rr.shared, Xr = Vr.gl, Jr = rr.current, ti = rr.next, Hr = Vr.current, Er = Vr.next, gr = rr.cond(Hr, ".dirty");
                  Da.forEach(function($r) {
                    var li, Qi;
                    if (!(($r = Jn($r)) in Rr.state)) if ($r in ti) {
                      li = ti[$r], Qi = Jr[$r];
                      var Ai = oe(zr[$r].length, function(Li) {
                        return gr.def(li, "[", Li, "]");
                      });
                      gr(rr.cond(Ai.map(function(Li, Ni) {
                        return Li + "!==" + Qi + "[" + Ni + "]";
                      }).join("||")).then(Xr, ".", wa[$r], "(", Ai, ");", Ai.map(function(Li, Ni) {
                        return Qi + "[" + Ni + "]=" + Li;
                      }).join(";"), ";"));
                    } else li = gr.def(Er, ".", $r), Ai = rr.cond(li, "!==", Hr, ".", $r), gr(Ai), $r in la ? Ai(rr.cond(li).then(Xr, ".enable(", la[$r], ");").else(Xr, ".disable(", la[$r], ");"), Hr, ".", $r, "=", li, ";") : Ai(Xr, ".", wa[$r], "(", li, ");", Hr, ".", $r, "=", li, ";");
                  }), Object.keys(Rr.state).length === 0 && gr(Hr, ".dirty=false;"), vr(gr);
                }
                function ri(rr, vr, Rr, Vr) {
                  var Xr, Jr = rr.shared, ti = rr.current, Hr = Jr.current, Er = Jr.gl;
                  ze(Object.keys(Rr)).forEach(function(gr) {
                    var $r = Rr[gr];
                    if (!Vr || Vr($r)) {
                      var li = $r.append(rr, vr);
                      if (la[gr]) {
                        var Qi = la[gr];
                        at($r) ? (Xr = rr.link(li, { stable: !0 }), vr(rr.cond(Xr).then(Er, ".enable(", Qi, ");").else(Er, ".disable(", Qi, ");")), vr(Hr, ".", gr, "=", Xr, ";")) : (vr(rr.cond(li).then(Er, ".enable(", Qi, ");").else(Er, ".disable(", Qi, ");")), vr(Hr, ".", gr, "=", li, ";"));
                      } else if (ve(li)) {
                        var Ai = ti[gr];
                        vr(Er, ".", wa[gr], "(", li, ");", li.map(function(Li, Ni) {
                          return Ai + "[" + Ni + "]=" + Li;
                        }).join(";"), ";");
                      } else at($r) ? (Xr = rr.link(li, { stable: !0 }), vr(Er, ".", wa[gr], "(", Xr, ");", Hr, ".", gr, "=", Xr, ";")) : vr(Er, ".", wa[gr], "(", li, ");", Hr, ".", gr, "=", li, ";");
                    }
                  });
                }
                function bi(rr, vr) {
                  Sr && (rr.instancing = vr.def(rr.shared.extensions, ".angle_instanced_arrays"));
                }
                function Zi(rr, vr, Rr, Vr, Xr) {
                  function Jr() {
                    return typeof performance > "u" ? "Date.now()" : "performance.now()";
                  }
                  function ti(Ni) {
                    Ni(gr = vr.def(), "=", Jr(), ";"), typeof Xr == "string" ? Ni(Qi, ".count+=", Xr, ";") : Ni(Qi, ".count++;"), jn && (Vr ? Ni($r = vr.def(), "=", Li, ".getNumPendingQueries();") : Ni(Li, ".beginQuery(", Qi, ");"));
                  }
                  function Hr(Ni) {
                    Ni(Qi, ".cpuTime+=", Jr(), "-", gr, ";"), jn && (Vr ? Ni(Li, ".pushScopeStats(", $r, ",", Li, ".getNumPendingQueries(),", Qi, ");") : Ni(Li, ".endQuery();"));
                  }
                  function Er(Ni) {
                    var Pi = vr.def(Ai, ".profile");
                    vr(Ai, ".profile=", Ni, ";"), vr.exit(Ai, ".profile=", Pi, ";");
                  }
                  var gr, $r, li = rr.shared, Qi = rr.stats, Ai = li.current, Li = li.timer;
                  if (Rr = Rr.profile) {
                    if (at(Rr)) return void (Rr.enable ? (ti(vr), Hr(vr.exit), Er("true")) : Er("false"));
                    Er(Rr = Rr.append(rr, vr));
                  } else Rr = vr.def(Ai, ".profile");
                  ti(li = rr.block()), vr("if(", Rr, "){", li, "}"), Hr(rr = rr.block()), vr.exit("if(", Rr, "){", rr, "}");
                }
                function $i(rr, vr, Rr, Vr, Xr) {
                  function Jr(Hr, Er, gr) {
                    function $r() {
                      vr("if(!", Li, ".buffer){", Qi, ".enableVertexAttribArray(", Ai, ");}");
                      var ki, Mi = gr.type;
                      ki = gr.size ? vr.def(gr.size, "||", Er) : Er, vr("if(", Li, ".type!==", Mi, "||", Li, ".size!==", ki, "||", Oi.map(function(Vi) {
                        return Li + "." + Vi + "!==" + gr[Vi];
                      }).join("||"), "){", Qi, ".bindBuffer(", 34962, ",", Ni, ".buffer);", Qi, ".vertexAttribPointer(", [Ai, ki, Mi, gr.normalized, gr.stride, gr.offset], ");", Li, ".type=", Mi, ";", Li, ".size=", ki, ";", Oi.map(function(Vi) {
                        return Li + "." + Vi + "=" + gr[Vi] + ";";
                      }).join(""), "}"), Sr && (Mi = gr.divisor, vr("if(", Li, ".divisor!==", Mi, "){", rr.instancing, ".vertexAttribDivisorANGLE(", [Ai, Mi], ");", Li, ".divisor=", Mi, ";}"));
                    }
                    function li() {
                      vr("if(", Li, ".buffer){", Qi, ".disableVertexAttribArray(", Ai, ");", Li, ".buffer=null;", "}if(", rt.map(function(ki, Mi) {
                        return Li + "." + ki + "!==" + Pi[Mi];
                      }).join("||"), "){", Qi, ".vertexAttrib4f(", Ai, ",", Pi, ");", rt.map(function(ki, Mi) {
                        return Li + "." + ki + "=" + Pi[Mi] + ";";
                      }).join(""), "}");
                    }
                    var Qi = ti.gl, Ai = vr.def(Hr, ".location"), Li = vr.def(ti.attributes, "[", Ai, "]");
                    Hr = gr.state;
                    var Ni = gr.buffer, Pi = [gr.x, gr.y, gr.z, gr.w], Oi = ["buffer", "normalized", "offset", "stride"];
                    Hr === 1 ? $r() : Hr === 2 ? li() : (vr("if(", Hr, "===", 1, "){"), $r(), vr("}else{"), li(), vr("}"));
                  }
                  var ti = rr.shared;
                  Vr.forEach(function(Hr) {
                    var Er, gr = Hr.name, $r = Rr.attributes[gr];
                    if ($r) {
                      if (!Xr($r)) return;
                      Er = $r.append(rr, vr);
                    } else {
                      if (!Xr(Dt)) return;
                      var li = rr.scopeAttrib(gr);
                      Er = {}, Object.keys(new Gr()).forEach(function(Qi) {
                        Er[Qi] = vr.def(li, ".", Qi);
                      });
                    }
                    Jr(rr.link(Hr), function(Qi) {
                      switch (Qi) {
                        case 35664:
                        case 35667:
                        case 35671:
                          return 2;
                        case 35665:
                        case 35668:
                        case 35672:
                          return 3;
                        case 35666:
                        case 35669:
                        case 35673:
                          return 4;
                        default:
                          return 1;
                      }
                    }(Hr.info.type), Er);
                  });
                }
                function Br(rr, vr, Rr, Vr, Xr, Jr) {
                  for (var ti, Hr = rr.shared, Er = Hr.gl, gr = 0; gr < Vr.length; ++gr) {
                    var $r, li = (Li = Vr[gr]).name, Qi = Li.info.type, Ai = Rr.uniforms[li], Li = rr.link(Li) + ".location";
                    if (Ai) {
                      if (!Xr(Ai)) continue;
                      if (at(Ai)) {
                        if (li = Ai.value, Qi === 35678 || Qi === 35680) vr(Er, ".uniform1i(", Li, ",", (Qi = rr.link(li._texture || li.color[0]._texture)) + ".bind());"), vr.exit(Qi, ".unbind();");
                        else if (Qi === 35674 || Qi === 35675 || Qi === 35676) Ai = 2, Qi === 35675 ? Ai = 3 : Qi === 35676 && (Ai = 4), vr(Er, ".uniformMatrix", Ai, "fv(", Li, ",false,", li = rr.global.def("new Float32Array([" + Array.prototype.slice.call(li) + "])"), ");");
                        else {
                          switch (Qi) {
                            case 5126:
                              ti = "1f";
                              break;
                            case 35664:
                              ti = "2f";
                              break;
                            case 35665:
                              ti = "3f";
                              break;
                            case 35666:
                              ti = "4f";
                              break;
                            case 35670:
                            case 5124:
                              ti = "1i";
                              break;
                            case 35671:
                            case 35667:
                              ti = "2i";
                              break;
                            case 35672:
                            case 35668:
                              ti = "3i";
                              break;
                            case 35673:
                            case 35669:
                              ti = "4i";
                          }
                          vr(Er, ".uniform", ti, "(", Li, ",", ve(li) ? Array.prototype.slice.call(li) : li, ");");
                        }
                        continue;
                      }
                      $r = Ai.append(rr, vr);
                    } else {
                      if (!Xr(Dt)) continue;
                      $r = vr.def(Hr.uniforms, "[", un.id(li), "]");
                    }
                    switch (Qi === 35678 ? vr("if(", $r, "&&", $r, '._reglType==="framebuffer"){', $r, "=", $r, ".color[0];", "}") : Qi === 35680 && vr("if(", $r, "&&", $r, '._reglType==="framebufferCube"){', $r, "=", $r, ".color[0];", "}"), li = 1, Qi) {
                      case 35678:
                      case 35680:
                        Qi = vr.def($r, "._texture"), vr(Er, ".uniform1i(", Li, ",", Qi, ".bind());"), vr.exit(Qi, ".unbind();");
                        continue;
                      case 5124:
                      case 35670:
                        ti = "1i";
                        break;
                      case 35667:
                      case 35671:
                        ti = "2i", li = 2;
                        break;
                      case 35668:
                      case 35672:
                        ti = "3i", li = 3;
                        break;
                      case 35669:
                      case 35673:
                        ti = "4i", li = 4;
                        break;
                      case 5126:
                        ti = "1f";
                        break;
                      case 35664:
                        ti = "2f", li = 2;
                        break;
                      case 35665:
                        ti = "3f", li = 3;
                        break;
                      case 35666:
                        ti = "4f", li = 4;
                        break;
                      case 35674:
                        ti = "Matrix2fv";
                        break;
                      case 35675:
                        ti = "Matrix3fv";
                        break;
                      case 35676:
                        ti = "Matrix4fv";
                    }
                    if (ti.charAt(0) === "M") {
                      vr(Er, ".uniform", ti, "(", Li, ","), Li = Math.pow(Qi - 35674 + 2, 2);
                      var Ni = rr.global.def("new Float32Array(", Li, ")");
                      Array.isArray($r) ? vr("false,(", oe(Li, function(Oi) {
                        return Ni + "[" + Oi + "]=" + $r[Oi];
                      }), ",", Ni, ")") : vr("false,(Array.isArray(", $r, ")||", $r, " instanceof Float32Array)?", $r, ":(", oe(Li, function(Oi) {
                        return Ni + "[" + Oi + "]=" + $r + "[" + Oi + "]";
                      }), ",", Ni, ")"), vr(");");
                    } else {
                      if (1 < li) {
                        Qi = [];
                        var Pi = [];
                        for (Ai = 0; Ai < li; ++Ai) Array.isArray($r) ? Pi.push($r[Ai]) : Pi.push(vr.def($r + "[" + Ai + "]")), Jr && Qi.push(vr.def());
                        Jr && vr("if(!", rr.batchId, "||", Qi.map(function(Oi, ki) {
                          return Oi + "!==" + Pi[ki];
                        }).join("||"), "){", Qi.map(function(Oi, ki) {
                          return Oi + "=" + Pi[ki] + ";";
                        }).join("")), vr(Er, ".uniform", ti, "(", Li, ",", Pi.join(","), ");");
                      } else Jr && (Qi = vr.def(), vr("if(!", rr.batchId, "||", Qi, "!==", $r, "){", Qi, "=", $r, ";")), vr(Er, ".uniform", ti, "(", Li, ",", $r, ");");
                      Jr && vr("}");
                    }
                  }
                }
                function Pr(rr, vr, Rr, Vr) {
                  function Xr(Mi) {
                    var Vi = Qi[Mi];
                    return Vi ? Vi.contextDep && Vr.contextDynamic || Vi.propDep ? Vi.append(rr, Rr) : Vi.append(rr, vr) : vr.def(li, ".", Mi);
                  }
                  function Jr() {
                    function Mi() {
                      Rr(Er, ".drawElementsInstancedANGLE(", [Li, Pi, Oi, Ni + "<<((" + Oi + "-5121)>>1)", Hr], ");");
                    }
                    function Vi() {
                      Rr(Er, ".drawArraysInstancedANGLE(", [Li, Ni, Pi, Hr], ");");
                    }
                    Ai && Ai !== "null" ? ki ? Mi() : (Rr("if(", Ai, "){"), Mi(), Rr("}else{"), Vi(), Rr("}")) : Vi();
                  }
                  function ti() {
                    function Mi() {
                      Rr($r + ".drawElements(" + [Li, Pi, Oi, Ni + "<<((" + Oi + "-5121)>>1)"] + ");");
                    }
                    function Vi() {
                      Rr($r + ".drawArrays(" + [Li, Ni, Pi] + ");");
                    }
                    Ai && Ai !== "null" ? ki ? Mi() : (Rr("if(", Ai, "){"), Mi(), Rr("}else{"), Vi(), Rr("}")) : Vi();
                  }
                  var Hr, Er, gr = rr.shared, $r = gr.gl, li = gr.draw, Qi = Vr.draw, Ai = function() {
                    var Mi = Qi.elements, Vi = vr;
                    return Mi ? ((Mi.contextDep && Vr.contextDynamic || Mi.propDep) && (Vi = Rr), Mi = Mi.append(rr, Vi), Qi.elementsActive && Vi("if(" + Mi + ")" + $r + ".bindBuffer(34963," + Mi + ".buffer.buffer);")) : (Mi = Vi.def(), Vi(Mi, "=", li, ".", "elements", ";", "if(", Mi, "){", $r, ".bindBuffer(", 34963, ",", Mi, ".buffer.buffer);}", "else if(", gr.vao, ".currentVAO){", Mi, "=", rr.shared.elements + ".getElements(" + gr.vao, ".currentVAO.elements);", si ? "" : "if(" + Mi + ")" + $r + ".bindBuffer(34963," + Mi + ".buffer.buffer);", "}")), Mi;
                  }(), Li = Xr("primitive"), Ni = Xr("offset"), Pi = function() {
                    var Mi = Qi.count, Vi = vr;
                    return Mi ? ((Mi.contextDep && Vr.contextDynamic || Mi.propDep) && (Vi = Rr), Mi = Mi.append(rr, Vi)) : Mi = Vi.def(li, ".", "count"), Mi;
                  }();
                  if (typeof Pi == "number") {
                    if (Pi === 0) return;
                  } else Rr("if(", Pi, "){"), Rr.exit("}");
                  Sr && (Hr = Xr("instances"), Er = rr.instancing);
                  var Oi = Ai + ".type", ki = Qi.elements && at(Qi.elements) && !Qi.vaoActive;
                  Sr && (typeof Hr != "number" || 0 <= Hr) ? typeof Hr == "string" ? (Rr("if(", Hr, ">0){"), Jr(), Rr("}else if(", Hr, "<0){"), ti(), Rr("}")) : Jr() : ti();
                }
                function ei(rr, vr, Rr, Vr, Xr) {
                  return Xr = (vr = Ar()).proc("body", Xr), Sr && (vr.instancing = Xr.def(vr.shared.extensions, ".angle_instanced_arrays")), rr(vr, Xr, Rr, Vr), vr.compile().body;
                }
                function Ci(rr, vr, Rr, Vr) {
                  bi(rr, vr), Rr.useVAO ? Rr.drawVAO ? vr(rr.shared.vao, ".setVAO(", Rr.drawVAO.append(rr, vr), ");") : vr(rr.shared.vao, ".setVAO(", rr.shared.vao, ".targetVAO);") : (vr(rr.shared.vao, ".setVAO(null);"), $i(rr, vr, Rr, Vr.attributes, function() {
                    return !0;
                  })), Br(rr, vr, Rr, Vr.uniforms, function() {
                    return !0;
                  }, !1), Pr(rr, vr, vr, Rr);
                }
                function oa(rr, vr, Rr, Vr) {
                  function Xr() {
                    return !0;
                  }
                  rr.batchId = "a1", bi(rr, vr), $i(rr, vr, Rr, Vr.attributes, Xr), Br(rr, vr, Rr, Vr.uniforms, Xr, !1), Pr(rr, vr, vr, Rr);
                }
                function ka(rr, vr, Rr, Vr) {
                  function Xr(li) {
                    return li.contextDep && ti || li.propDep;
                  }
                  function Jr(li) {
                    return !Xr(li);
                  }
                  bi(rr, vr);
                  var ti = Rr.contextDep, Hr = vr.def(), Er = vr.def();
                  rr.shared.props = Er, rr.batchId = Hr;
                  var gr = rr.scope(), $r = rr.scope();
                  vr(gr.entry, "for(", Hr, "=0;", Hr, "<", "a1", ";++", Hr, "){", Er, "=", "a0", "[", Hr, "];", $r, "}", gr.exit), Rr.needsContext && gi(rr, $r, Rr.context), Rr.needsFramebuffer && Zr(rr, $r, Rr.framebuffer), ri(rr, $r, Rr.state, Xr), Rr.profile && Xr(Rr.profile) && Zi(rr, $r, Rr, !1, !0), Vr ? (Rr.useVAO ? Rr.drawVAO ? Xr(Rr.drawVAO) ? $r(rr.shared.vao, ".setVAO(", Rr.drawVAO.append(rr, $r), ");") : gr(rr.shared.vao, ".setVAO(", Rr.drawVAO.append(rr, gr), ");") : gr(rr.shared.vao, ".setVAO(", rr.shared.vao, ".targetVAO);") : (gr(rr.shared.vao, ".setVAO(null);"), $i(rr, gr, Rr, Vr.attributes, Jr), $i(rr, $r, Rr, Vr.attributes, Xr)), Br(rr, gr, Rr, Vr.uniforms, Jr, !1), Br(rr, $r, Rr, Vr.uniforms, Xr, !0), Pr(rr, gr, $r, Rr)) : (vr = rr.global.def("{}"), Vr = Rr.shader.progVar.append(rr, $r), Er = $r.def(Vr, ".id"), gr = $r.def(vr, "[", Er, "]"), $r(rr.shared.gl, ".useProgram(", Vr, ".program);", "if(!", gr, "){", gr, "=", vr, "[", Er, "]=", rr.link(function(li) {
                    return ei(oa, rr, Rr, li, 2);
                  }), "(", Vr, ");}", gr, ".call(this,a0[", Hr, "],", Hr, ");"));
                }
                function aa(rr, vr) {
                  function Rr(Hr) {
                    var Er = vr.shader[Hr];
                    Er && (Er = Er.append(rr, Vr), isNaN(Er) ? Vr.set(Xr.shader, "." + Hr, Er) : Vr.set(Xr.shader, "." + Hr, rr.link(Er, { stable: !0 })));
                  }
                  var Vr = rr.proc("scope", 3);
                  rr.batchId = "a2";
                  var Xr = rr.shared, Jr = Xr.current;
                  if (gi(rr, Vr, vr.context), vr.framebuffer && vr.framebuffer.append(rr, Vr), ze(Object.keys(vr.state)).forEach(function(Hr) {
                    var Er = vr.state[Hr], gr = Er.append(rr, Vr);
                    ve(gr) ? gr.forEach(function($r, li) {
                      isNaN($r) ? Vr.set(rr.next[Hr], "[" + li + "]", $r) : Vr.set(rr.next[Hr], "[" + li + "]", rr.link($r, { stable: !0 }));
                    }) : at(Er) ? Vr.set(Xr.next, "." + Hr, rr.link(gr, { stable: !0 })) : Vr.set(Xr.next, "." + Hr, gr);
                  }), Zi(rr, Vr, vr, !0, !0), ["elements", "offset", "count", "instances", "primitive"].forEach(function(Hr) {
                    var Er = vr.draw[Hr];
                    Er && (Er = Er.append(rr, Vr), isNaN(Er) ? Vr.set(Xr.draw, "." + Hr, Er) : Vr.set(Xr.draw, "." + Hr, rr.link(Er), { stable: !0 }));
                  }), Object.keys(vr.uniforms).forEach(function(Hr) {
                    var Er = vr.uniforms[Hr].append(rr, Vr);
                    Array.isArray(Er) && (Er = "[" + Er.map(function(gr) {
                      return isNaN(gr) ? gr : rr.link(gr, { stable: !0 });
                    }) + "]"), Vr.set(Xr.uniforms, "[" + rr.link(un.id(Hr), { stable: !0 }) + "]", Er);
                  }), Object.keys(vr.attributes).forEach(function(Hr) {
                    var Er = vr.attributes[Hr].append(rr, Vr), gr = rr.scopeAttrib(Hr);
                    Object.keys(new Gr()).forEach(function($r) {
                      Vr.set(gr, "." + $r, Er[$r]);
                    });
                  }), vr.scopeVAO) {
                    var ti = vr.scopeVAO.append(rr, Vr);
                    isNaN(ti) ? Vr.set(Xr.vao, ".targetVAO", ti) : Vr.set(Xr.vao, ".targetVAO", rr.link(ti, { stable: !0 }));
                  }
                  Rr("vert"), Rr("frag"), 0 < Object.keys(vr.state).length && (Vr(Jr, ".dirty=true;"), Vr.exit(Jr, ".dirty=true;")), Vr("a1(", rr.shared.context, ",a0,", rr.batchId, ");");
                }
                function hi(rr, vr, Rr) {
                  var Vr = vr.static[Rr];
                  if (Vr && function($r) {
                    if (typeof $r == "object" && !ve($r)) {
                      for (var li = Object.keys($r), Qi = 0; Qi < li.length; ++Qi) if (gt.isDynamic($r[li[Qi]])) return !0;
                      return !1;
                    }
                  }(Vr)) {
                    var Xr = rr.global, Jr = Object.keys(Vr), ti = !1, Hr = !1, Er = !1, gr = rr.global.def("{}");
                    Jr.forEach(function($r) {
                      var li = Vr[$r];
                      if (gt.isDynamic(li)) typeof li == "function" && (li = Vr[$r] = gt.unbox(li)), $r = et(li, null), ti = ti || $r.thisDep, Er = Er || $r.propDep, Hr = Hr || $r.contextDep;
                      else {
                        switch (Xr(gr, ".", $r, "="), typeof li) {
                          case "number":
                            Xr(li);
                            break;
                          case "string":
                            Xr('"', li, '"');
                            break;
                          case "object":
                            Array.isArray(li) && Xr("[", li.join(), "]");
                            break;
                          default:
                            Xr(rr.link(li));
                        }
                        Xr(";");
                      }
                    }), vr.dynamic[Rr] = new gt.DynamicVariable(4, { thisDep: ti, contextDep: Hr, propDep: Er, ref: gr, append: function($r, li) {
                      Jr.forEach(function(Qi) {
                        var Ai = Vr[Qi];
                        gt.isDynamic(Ai) && (Ai = $r.invoke(li, Ai), li(gr, ".", Qi, "=", Ai, ";"));
                      });
                    } }), delete vr.static[Rr];
                  }
                }
                var Gr = Ln.Record, Ca = { add: 32774, subtract: 32778, "reverse subtract": 32779 };
                yn.ext_blend_minmax && (Ca.min = 32775, Ca.max = 32776);
                var Sr = yn.angle_instanced_arrays, Dr = yn.webgl_draw_buffers, si = yn.oes_vertex_array_object, zr = { dirty: !0, profile: Kn.profile }, di = {}, Da = [], la = {}, wa = {};
                Lr("dither", 3024), Lr("blend.enable", 3042), mr("blend.color", "blendColor", [0, 0, 0, 0]), mr("blend.equation", "blendEquationSeparate", [32774, 32774]), mr("blend.func", "blendFuncSeparate", [1, 0, 1, 0]), Lr("depth.enable", 2929, !0), mr("depth.func", "depthFunc", 513), mr("depth.range", "depthRange", [0, 1]), mr("depth.mask", "depthMask", !0), mr("colorMask", "colorMask", [!0, !0, !0, !0]), Lr("cull.enable", 2884), mr("cull.face", "cullFace", 1029), mr("frontFace", "frontFace", 2305), mr("lineWidth", "lineWidth", 1), Lr("polygonOffset.enable", 32823), mr("polygonOffset.offset", "polygonOffset", [0, 0]), Lr("sample.alpha", 32926), Lr("sample.enable", 32928), mr("sample.coverage", "sampleCoverage", [1, !1]), Lr("stencil.enable", 2960), mr("stencil.mask", "stencilMask", -1), mr("stencil.func", "stencilFunc", [519, 0, -1]), mr("stencil.opFront", "stencilOpSeparate", [1028, 7680, 7680, 7680]), mr("stencil.opBack", "stencilOpSeparate", [1029, 7680, 7680, 7680]), Lr("scissor.enable", 3089), mr("scissor.box", "scissor", [0, 0, Ot.drawingBufferWidth, Ot.drawingBufferHeight]), mr("viewport", "viewport", [0, 0, Ot.drawingBufferWidth, Ot.drawingBufferHeight]);
                var Wa = { gl: Ot, context: er, strings: un, next: di, current: zr, draw: kn, elements: bn, buffer: gn, shader: In, attributes: Ln.state, vao: Ln, uniforms: Mn, framebuffer: Zt, extensions: yn, timer: jn, isBufferArgs: Ge }, Qr = { primTypes: It, compareFuncs: wt, blendFuncs: pt, blendEquations: Ca, stencilOps: Bt, glTypes: Xt, orientationType: At };
                Dr && (Qr.backBuffer = [1029], Qr.drawBuffer = oe(_n.maxDrawbuffers, function(rr) {
                  return rr === 0 ? [0] : oe(rr, function(vr) {
                    return 36064 + vr;
                  });
                }));
                var Ur = 0;
                return { next: di, current: zr, procs: function() {
                  var rr = Ar(), vr = rr.proc("poll"), Rr = rr.proc("refresh"), Vr = rr.block();
                  vr(Vr), Rr(Vr);
                  var Xr, Jr = (Er = rr.shared).gl, ti = Er.next, Hr = Er.current;
                  Vr(Hr, ".dirty=false;"), Zr(rr, vr), Zr(rr, Rr, null, !0), Sr && (Xr = rr.link(Sr)), yn.oes_vertex_array_object && Rr(rr.link(yn.oes_vertex_array_object), ".bindVertexArrayOES(null);");
                  var Er = Rr.def(Er.attributes), gr = Rr.def(0), $r = rr.cond(gr, ".buffer");
                  $r.then(Jr, ".enableVertexAttribArray(i);", Jr, ".bindBuffer(", 34962, ",", gr, ".buffer.buffer);", Jr, ".vertexAttribPointer(i,", gr, ".size,", gr, ".type,", gr, ".normalized,", gr, ".stride,", gr, ".offset);").else(Jr, ".disableVertexAttribArray(i);", Jr, ".vertexAttrib4f(i,", gr, ".x,", gr, ".y,", gr, ".z,", gr, ".w);", gr, ".buffer=null;");
                  var li = rr.link(_n.maxAttributes, { stable: !0 });
                  return Rr("for(var i=0;i<", li, ";++i){", gr, "=", Er, "[i];", $r, "}"), Sr && Rr("for(var i=0;i<", li, ";++i){", Xr, ".vertexAttribDivisorANGLE(i,", Er, "[i].divisor);", "}"), Rr(rr.shared.vao, ".currentVAO=null;", rr.shared.vao, ".setVAO(", rr.shared.vao, ".targetVAO);"), Object.keys(la).forEach(function(Qi) {
                    var Ai = la[Qi], Li = Vr.def(ti, ".", Qi), Ni = rr.block();
                    Ni("if(", Li, "){", Jr, ".enable(", Ai, ")}else{", Jr, ".disable(", Ai, ")}", Hr, ".", Qi, "=", Li, ";"), Rr(Ni), vr("if(", Li, "!==", Hr, ".", Qi, "){", Ni, "}");
                  }), Object.keys(wa).forEach(function(Qi) {
                    var Ai, Li, Ni = wa[Qi], Pi = zr[Qi], Oi = rr.block();
                    Oi(Jr, ".", Ni, "("), ve(Pi) ? (Ni = Pi.length, Ai = rr.global.def(ti, ".", Qi), Li = rr.global.def(Hr, ".", Qi), Oi(oe(Ni, function(ki) {
                      return Ai + "[" + ki + "]";
                    }), ");", oe(Ni, function(ki) {
                      return Li + "[" + ki + "]=" + Ai + "[" + ki + "];";
                    }).join("")), vr("if(", oe(Ni, function(ki) {
                      return Ai + "[" + ki + "]!==" + Li + "[" + ki + "]";
                    }).join("||"), "){", Oi, "}")) : (Ai = Vr.def(ti, ".", Qi), Li = Vr.def(Hr, ".", Qi), Oi(Ai, ");", Hr, ".", Qi, "=", Ai, ";"), vr("if(", Ai, "!==", Li, "){", Oi, "}")), Rr(Oi);
                  }), rr.compile();
                }(), compile: function(rr, vr, Rr, Vr, Xr) {
                  var Jr = Ar();
                  Jr.stats = Jr.link(Xr), Object.keys(vr.static).forEach(function(Hr) {
                    hi(Jr, vr, Hr);
                  }), ft.forEach(function(Hr) {
                    hi(Jr, rr, Hr);
                  });
                  var ti = Bi(rr, vr, Rr, Vr);
                  return ti.shader.program && (ti.shader.program.attributes.sort(function(Hr, Er) {
                    return Hr.name < Er.name ? -1 : 1;
                  }), ti.shader.program.uniforms.sort(function(Hr, Er) {
                    return Hr.name < Er.name ? -1 : 1;
                  })), function(Hr, Er) {
                    var gr = Hr.proc("draw", 1);
                    bi(Hr, gr), gi(Hr, gr, Er.context), Zr(Hr, gr, Er.framebuffer), fi(Hr, gr, Er), ri(Hr, gr, Er.state), Zi(Hr, gr, Er, !1, !0);
                    var $r = Er.shader.progVar.append(Hr, gr);
                    if (gr(Hr.shared.gl, ".useProgram(", $r, ".program);"), Er.shader.program) Ci(Hr, gr, Er, Er.shader.program);
                    else {
                      gr(Hr.shared.vao, ".setVAO(null);");
                      var li = Hr.global.def("{}"), Qi = gr.def($r, ".id"), Ai = gr.def(li, "[", Qi, "]");
                      gr(Hr.cond(Ai).then(Ai, ".call(this,a0);").else(Ai, "=", li, "[", Qi, "]=", Hr.link(function(Li) {
                        return ei(Ci, Hr, Er, Li, 1);
                      }), "(", $r, ");", Ai, ".call(this,a0);"));
                    }
                    0 < Object.keys(Er.state).length && gr(Hr.shared.current, ".dirty=true;"), Hr.shared.vao && gr(Hr.shared.vao, ".setVAO(null);");
                  }(Jr, ti), aa(Jr, ti), function(Hr, Er) {
                    function gr(Pi) {
                      return Pi.contextDep && li || Pi.propDep;
                    }
                    var $r = Hr.proc("batch", 2);
                    Hr.batchId = "0", bi(Hr, $r);
                    var li = !1, Qi = !0;
                    Object.keys(Er.context).forEach(function(Pi) {
                      li = li || Er.context[Pi].propDep;
                    }), li || (gi(Hr, $r, Er.context), Qi = !1);
                    var Ai = !1;
                    if ((Li = Er.framebuffer) ? (Li.propDep ? li = Ai = !0 : Li.contextDep && li && (Ai = !0), Ai || Zr(Hr, $r, Li)) : Zr(Hr, $r, null), Er.state.viewport && Er.state.viewport.propDep && (li = !0), fi(Hr, $r, Er), ri(Hr, $r, Er.state, function(Pi) {
                      return !gr(Pi);
                    }), Er.profile && gr(Er.profile) || Zi(Hr, $r, Er, !1, "a1"), Er.contextDep = li, Er.needsContext = Qi, Er.needsFramebuffer = Ai, (Qi = Er.shader.progVar).contextDep && li || Qi.propDep) ka(Hr, $r, Er, null);
                    else if (Qi = Qi.append(Hr, $r), $r(Hr.shared.gl, ".useProgram(", Qi, ".program);"), Er.shader.program) ka(Hr, $r, Er, Er.shader.program);
                    else {
                      $r(Hr.shared.vao, ".setVAO(null);");
                      var Li = Hr.global.def("{}"), Ni = (Ai = $r.def(Qi, ".id"), $r.def(Li, "[", Ai, "]"));
                      $r(Hr.cond(Ni).then(Ni, ".call(this,a0,a1);").else(Ni, "=", Li, "[", Ai, "]=", Hr.link(function(Pi) {
                        return ei(ka, Hr, Er, Pi, 2);
                      }), "(", Qi, ");", Ni, ".call(this,a0,a1);"));
                    }
                    0 < Object.keys(Er.state).length && $r(Hr.shared.current, ".dirty=true;"), Hr.shared.vao && $r(Hr.shared.vao, ".setVAO(null);");
                  }(Jr, ti), it(Jr.compile(), { destroy: function() {
                    ti.shader.program.destroy();
                  } });
                } };
              }
              function Je(Ot, un) {
                for (var yn = 0; yn < Ot.length; ++yn) if (Ot[yn] === un) return yn;
                return -1;
              }
              var it = function(Ot, un) {
                for (var yn = Object.keys(un), _n = 0; _n < yn.length; ++_n) Ot[yn[_n]] = un[yn[_n]];
                return Ot;
              }, st = 0, gt = { DynamicVariable: J, define: function(Ot, un) {
                return new J(Ot, te(un + ""));
              }, isDynamic: function(Ot) {
                return typeof Ot == "function" && !Ot._reglType || Ot instanceof J;
              }, unbox: function Ot(un, yn) {
                return typeof un == "function" ? new J(0, un) : typeof un == "number" || typeof un == "boolean" ? new J(5, un) : Array.isArray(un) ? new J(6, un.map(function(_n, gn) {
                  return Ot(_n);
                })) : un instanceof J ? un : void 0;
              }, accessor: te }, bt = { next: typeof requestAnimationFrame == "function" ? function(Ot) {
                return requestAnimationFrame(Ot);
              } : function(Ot) {
                return setTimeout(Ot, 16);
              }, cancel: typeof cancelAnimationFrame == "function" ? function(Ot) {
                return cancelAnimationFrame(Ot);
              } : clearTimeout }, yt = typeof performance < "u" && performance.now ? function() {
                return performance.now();
              } : function() {
                return +/* @__PURE__ */ new Date();
              }, St = ce();
              St.zero = ce();
              var Lt = function(Ot, un) {
                var yn = 1;
                un.ext_texture_filter_anisotropic && (yn = Ot.getParameter(34047));
                var _n = 1, gn = 1;
                un.webgl_draw_buffers && (_n = Ot.getParameter(34852), gn = Ot.getParameter(36063));
                var bn = !!un.oes_texture_float;
                if (bn) {
                  bn = Ot.createTexture(), Ot.bindTexture(3553, bn), Ot.texImage2D(3553, 0, 6408, 1, 1, 0, 6408, 5126, null);
                  var an = Ot.createFramebuffer();
                  if (Ot.bindFramebuffer(36160, an), Ot.framebufferTexture2D(36160, 36064, 3553, bn, 0), Ot.bindTexture(3553, null), Ot.checkFramebufferStatus(36160) !== 36053) bn = !1;
                  else {
                    Ot.viewport(0, 0, 1, 1), Ot.clearColor(1, 0, 0, 1), Ot.clear(16384);
                    var Zt = St.allocType(5126, 4);
                    Ot.readPixels(0, 0, 1, 1, 6408, 5126, Zt), Ot.getError() ? bn = !1 : (Ot.deleteFramebuffer(an), Ot.deleteTexture(bn), bn = Zt[0] === 1), St.freeType(Zt);
                  }
                }
                return Zt = !0, typeof navigator < "u" && (/MSIE/.test(navigator.userAgent) || /Trident\//.test(navigator.appVersion) || /Edge/.test(navigator.userAgent)) || (Zt = Ot.createTexture(), an = St.allocType(5121, 36), Ot.activeTexture(33984), Ot.bindTexture(34067, Zt), Ot.texImage2D(34069, 0, 6408, 3, 3, 0, 6408, 5121, an), St.freeType(an), Ot.bindTexture(34067, null), Ot.deleteTexture(Zt), Zt = !Ot.getError()), { colorBits: [Ot.getParameter(3410), Ot.getParameter(3411), Ot.getParameter(3412), Ot.getParameter(3413)], depthBits: Ot.getParameter(3414), stencilBits: Ot.getParameter(3415), subpixelBits: Ot.getParameter(3408), extensions: Object.keys(un).filter(function(Mn) {
                  return !!un[Mn];
                }), maxAnisotropic: yn, maxDrawbuffers: _n, maxColorAttachments: gn, pointSizeDims: Ot.getParameter(33901), lineWidthDims: Ot.getParameter(33902), maxViewportDims: Ot.getParameter(3386), maxCombinedTextureUnits: Ot.getParameter(35661), maxCubeMapSize: Ot.getParameter(34076), maxRenderbufferSize: Ot.getParameter(34024), maxTextureUnits: Ot.getParameter(34930), maxTextureSize: Ot.getParameter(3379), maxAttributes: Ot.getParameter(34921), maxVertexUniforms: Ot.getParameter(36347), maxVertexTextureUnits: Ot.getParameter(35660), maxVaryingVectors: Ot.getParameter(36348), maxFragmentUniforms: Ot.getParameter(36349), glsl: Ot.getParameter(35724), renderer: Ot.getParameter(7937), vendor: Ot.getParameter(7936), version: Ot.getParameter(7938), readFloat: bn, npotTextureCube: Zt };
              }, Rt = function(Ot) {
                return Ot instanceof Uint8Array || Ot instanceof Uint16Array || Ot instanceof Uint32Array || Ot instanceof Int8Array || Ot instanceof Int16Array || Ot instanceof Int32Array || Ot instanceof Float32Array || Ot instanceof Float64Array || Ot instanceof Uint8ClampedArray;
              }, vt = function(Ot) {
                return Object.keys(Ot).map(function(un) {
                  return Ot[un];
                });
              }, $t = { shape: function(Ot) {
                for (var un = []; Ot.length; Ot = Ot[0]) un.push(Ot.length);
                return un;
              }, flatten: function(Ot, un, yn, _n) {
                var gn = 1;
                if (un.length) for (var bn = 0; bn < un.length; ++bn) gn *= un[bn];
                else gn = 0;
                switch (yn = _n || St.allocType(yn, gn), un.length) {
                  case 0:
                    break;
                  case 1:
                    for (_n = un[0], un = 0; un < _n; ++un) yn[un] = Ot[un];
                    break;
                  case 2:
                    for (_n = un[0], un = un[1], bn = gn = 0; bn < _n; ++bn) for (var an = Ot[bn], Zt = 0; Zt < un; ++Zt) yn[gn++] = an[Zt];
                    break;
                  case 3:
                    ue(Ot, un[0], un[1], un[2], yn, 0);
                    break;
                  default:
                    de(Ot, un, 0, yn, 0);
                }
                return yn;
              } }, Pt = { "[object Int8Array]": 5120, "[object Int16Array]": 5122, "[object Int32Array]": 5124, "[object Uint8Array]": 5121, "[object Uint8ClampedArray]": 5121, "[object Uint16Array]": 5123, "[object Uint32Array]": 5125, "[object Float32Array]": 5126, "[object Float64Array]": 5121, "[object ArrayBuffer]": 5121 }, Xt = { int8: 5120, int16: 5122, int32: 5124, uint8: 5121, uint16: 5123, uint32: 5125, float: 5126, float32: 5126 }, Nt = { dynamic: 35048, stream: 35040, static: 35044 }, Gt = $t.flatten, Ft = $t.shape, Tt = [];
              Tt[5120] = 1, Tt[5122] = 2, Tt[5124] = 4, Tt[5121] = 1, Tt[5123] = 2, Tt[5125] = 4, Tt[5126] = 4;
              var It = { points: 0, point: 0, lines: 1, line: 1, triangles: 4, triangle: 4, "line loop": 2, "line strip": 3, "triangle strip": 5, "triangle fan": 6 }, Ct = new Float32Array(1), Kt = new Uint32Array(Ct.buffer), Vt = [9984, 9986, 9985, 9987], Yt = [0, 6409, 6410, 6407, 6408], sn = {};
              sn[6409] = sn[6406] = sn[6402] = 1, sn[34041] = sn[6410] = 2, sn[6407] = sn[35904] = 3, sn[6408] = sn[35906] = 4;
              var hn = be("HTMLCanvasElement"), Qt = be("OffscreenCanvas"), Ut = be("CanvasRenderingContext2D"), Wt = be("ImageBitmap"), rn = be("HTMLImageElement"), vn = be("HTMLVideoElement"), dn = Object.keys(Pt).concat([hn, Qt, Ut, Wt, rn, vn]), Ht = [];
              Ht[5121] = 1, Ht[5126] = 4, Ht[36193] = 2, Ht[5123] = 2, Ht[5125] = 4;
              var jt = [];
              jt[32854] = 2, jt[32855] = 2, jt[36194] = 2, jt[34041] = 4, jt[33776] = 0.5, jt[33777] = 0.5, jt[33778] = 1, jt[33779] = 1, jt[35986] = 0.5, jt[35987] = 1, jt[34798] = 1, jt[35840] = 0.5, jt[35841] = 0.25, jt[35842] = 0.5, jt[35843] = 0.25, jt[36196] = 0.5;
              var cn = [];
              cn[32854] = 2, cn[32855] = 2, cn[36194] = 2, cn[33189] = 2, cn[36168] = 1, cn[34041] = 4, cn[35907] = 4, cn[34836] = 16, cn[34842] = 8, cn[34843] = 6;
              var wn = function(Ot, un, yn, _n, gn) {
                function bn(kn) {
                  this.id = Ln++, this.refCount = 1, this.renderbuffer = kn, this.format = 32854, this.height = this.width = 0, gn.profile && (this.stats = { size: 0 });
                }
                function an(kn) {
                  var er = kn.renderbuffer;
                  Ot.bindRenderbuffer(36161, null), Ot.deleteRenderbuffer(er), kn.renderbuffer = null, kn.refCount = 0, delete In[kn.id], _n.renderbufferCount--;
                }
                var Zt = { rgba4: 32854, rgb565: 36194, "rgb5 a1": 32855, depth: 33189, stencil: 36168, "depth stencil": 34041 };
                un.ext_srgb && (Zt.srgba = 35907), un.ext_color_buffer_half_float && (Zt.rgba16f = 34842, Zt.rgb16f = 34843), un.webgl_color_buffer_float && (Zt.rgba32f = 34836);
                var Mn = [];
                Object.keys(Zt).forEach(function(kn) {
                  Mn[Zt[kn]] = kn;
                });
                var Ln = 0, In = {};
                return bn.prototype.decRef = function() {
                  0 >= --this.refCount && an(this);
                }, gn.profile && (_n.getTotalRenderbufferSize = function() {
                  var kn = 0;
                  return Object.keys(In).forEach(function(er) {
                    kn += In[er].stats.size;
                  }), kn;
                }), { create: function(kn, er) {
                  function jn(Kn, Jn) {
                    var Lr = 0, mr = 0, Ar = 32854;
                    if (typeof Kn == "object" && Kn ? ("shape" in Kn ? (Lr = 0 | (mr = Kn.shape)[0], mr = 0 | mr[1]) : ("radius" in Kn && (Lr = mr = 0 | Kn.radius), "width" in Kn && (Lr = 0 | Kn.width), "height" in Kn && (mr = 0 | Kn.height)), "format" in Kn && (Ar = Zt[Kn.format])) : typeof Kn == "number" ? (Lr = 0 | Kn, mr = typeof Jn == "number" ? 0 | Jn : Lr) : Kn || (Lr = mr = 1), Lr !== ur.width || mr !== ur.height || Ar !== ur.format) return jn.width = ur.width = Lr, jn.height = ur.height = mr, ur.format = Ar, Ot.bindRenderbuffer(36161, ur.renderbuffer), Ot.renderbufferStorage(36161, Ar, Lr, mr), gn.profile && (ur.stats.size = cn[ur.format] * ur.width * ur.height), jn.format = Mn[ur.format], jn;
                  }
                  var ur = new bn(Ot.createRenderbuffer());
                  return In[ur.id] = ur, _n.renderbufferCount++, jn(kn, er), jn.resize = function(Kn, Jn) {
                    var Lr = 0 | Kn, mr = 0 | Jn || Lr;
                    return Lr === ur.width && mr === ur.height || (jn.width = ur.width = Lr, jn.height = ur.height = mr, Ot.bindRenderbuffer(36161, ur.renderbuffer), Ot.renderbufferStorage(36161, ur.format, Lr, mr), gn.profile && (ur.stats.size = cn[ur.format] * ur.width * ur.height)), jn;
                  }, jn._reglType = "renderbuffer", jn._renderbuffer = ur, gn.profile && (jn.stats = ur.stats), jn.destroy = function() {
                    ur.decRef();
                  }, jn;
                }, clear: function() {
                  vt(In).forEach(an);
                }, restore: function() {
                  vt(In).forEach(function(kn) {
                    kn.renderbuffer = Ot.createRenderbuffer(), Ot.bindRenderbuffer(36161, kn.renderbuffer), Ot.renderbufferStorage(36161, kn.format, kn.width, kn.height);
                  }), Ot.bindRenderbuffer(36161, null);
                } };
              }, Rn = [];
              Rn[6408] = 4, Rn[6407] = 3;
              var Pn = [];
              Pn[5121] = 1, Pn[5126] = 4, Pn[36193] = 2;
              var Ie = [1116352408, 1899447441, -1245643825, -373957723, 961987163, 1508970993, -1841331548, -1424204075, -670586216, 310598401, 607225278, 1426881987, 1925078388, -2132889090, -1680079193, -1046744716, -459576895, -272742522, 264347078, 604807628, 770255983, 1249150122, 1555081692, 1996064986, -1740746414, -1473132947, -1341970488, -1084653625, -958395405, -710438585, 113926993, 338241895, 666307205, 773529912, 1294757372, 1396182291, 1695183700, 1986661051, -2117940946, -1838011259, -1564481375, -1474664885, -1035236496, -949202525, -778901479, -694614492, -200395387, 275423344, 430227734, 506948616, 659060556, 883997877, 958139571, 1322822218, 1537002063, 1747873779, 1955562222, 2024104815, -2067236844, -1933114872, -1866530822, -1538233109, -1090935817, -965641998], rt = ["x", "y", "z", "w"], ft = "blend.func blend.equation stencil.func stencil.opFront stencil.opBack sample.coverage viewport scissor.box polygonOffset.offset".split(" "), pt = { 0: 0, 1: 1, zero: 0, one: 1, "src color": 768, "one minus src color": 769, "src alpha": 770, "one minus src alpha": 771, "dst color": 774, "one minus dst color": 775, "dst alpha": 772, "one minus dst alpha": 773, "constant color": 32769, "one minus constant color": 32770, "constant alpha": 32771, "one minus constant alpha": 32772, "src alpha saturate": 776 }, wt = { never: 512, less: 513, "<": 513, equal: 514, "=": 514, "==": 514, "===": 514, lequal: 515, "<=": 515, greater: 516, ">": 516, notequal: 517, "!=": 517, "!==": 517, gequal: 518, ">=": 518, always: 519 }, Bt = { 0: 0, zero: 0, keep: 7680, replace: 7681, increment: 7682, decrement: 7683, "increment wrap": 34055, "decrement wrap": 34056, invert: 5386 }, At = { cw: 2304, ccw: 2305 }, Dt = new We(!1, !1, !1, function() {
              });
              return function(Ot) {
                function un() {
                  if (oa.length === 0) Yr && Yr.update(), Gr = null;
                  else {
                    Gr = bt.next(un), In();
                    for (var Sr = oa.length - 1; 0 <= Sr; --Sr) {
                      var Dr = oa[Sr];
                      Dr && Dr(Bi, null, 0);
                    }
                    jn.flush(), Yr && Yr.update();
                  }
                }
                function yn() {
                  !Gr && 0 < oa.length && (Gr = bt.next(un));
                }
                function _n() {
                  Gr && (bt.cancel(un), Gr = null);
                }
                function gn(Sr) {
                  Sr.preventDefault(), _n(), ka.forEach(function(Dr) {
                    Dr();
                  });
                }
                function bn(Sr) {
                  jn.getError(), Kn.restore(), bi.restore(), Zr.restore(), Zi.restore(), $i.restore(), Br.restore(), ri.restore(), Yr && Yr.restore(), Pr.procs.refresh(), yn(), aa.forEach(function(Dr) {
                    Dr();
                  });
                }
                function an(Sr) {
                  function Dr(rr, vr) {
                    var Rr = {}, Vr = {};
                    return Object.keys(rr).forEach(function(Xr) {
                      var Jr = rr[Xr];
                      if (gt.isDynamic(Jr)) Vr[Xr] = gt.unbox(Jr, Xr);
                      else {
                        if (vr && Array.isArray(Jr)) {
                          for (var ti = 0; ti < Jr.length; ++ti) if (gt.isDynamic(Jr[ti])) return void (Vr[Xr] = gt.unbox(Jr, Xr));
                        }
                        Rr[Xr] = Jr;
                      }
                    }), { dynamic: Vr, static: Rr };
                  }
                  var si = Dr(Sr.context || {}, !0), zr = Dr(Sr.uniforms || {}, !0), di = Dr(Sr.attributes || {}, !1);
                  Sr = Dr(function(rr) {
                    function vr(Vr) {
                      if (Vr in Rr) {
                        var Xr = Rr[Vr];
                        delete Rr[Vr], Object.keys(Xr).forEach(function(Jr) {
                          Rr[Vr + "." + Jr] = Xr[Jr];
                        });
                      }
                    }
                    var Rr = it({}, rr);
                    return delete Rr.uniforms, delete Rr.attributes, delete Rr.context, delete Rr.vao, "stencil" in Rr && Rr.stencil.op && (Rr.stencil.opBack = Rr.stencil.opFront = Rr.stencil.op, delete Rr.stencil.op), vr("blend"), vr("depth"), vr("cull"), vr("stencil"), vr("polygonOffset"), vr("scissor"), vr("sample"), "vao" in rr && (Rr.vao = rr.vao), Rr;
                  }(Sr), !1);
                  var Da = { gpuTime: 0, cpuTime: 0, count: 0 }, la = Pr.compile(Sr, di, zr, si, Da), wa = la.draw, Wa = la.batch, Qr = la.scope, Ur = [];
                  return it(function(rr, vr) {
                    var Rr;
                    if (typeof rr == "function") return Qr.call(this, null, rr, 0);
                    if (typeof vr == "function") if (typeof rr == "number") for (Rr = 0; Rr < rr; ++Rr) Qr.call(this, null, vr, Rr);
                    else {
                      if (!Array.isArray(rr)) return Qr.call(this, rr, vr, 0);
                      for (Rr = 0; Rr < rr.length; ++Rr) Qr.call(this, rr[Rr], vr, Rr);
                    }
                    else if (typeof rr == "number") {
                      if (0 < rr) return Wa.call(this, function(Vr) {
                        for (; Ur.length < Vr; ) Ur.push(null);
                        return Ur;
                      }(0 | rr), 0 | rr);
                    } else {
                      if (!Array.isArray(rr)) return wa.call(this, rr);
                      if (rr.length) return Wa.call(this, rr, rr.length);
                    }
                  }, { stats: Da, destroy: function() {
                    la.destroy();
                  } });
                }
                function Zt(Sr, Dr) {
                  var si = 0;
                  Pr.procs.poll();
                  var zr = Dr.color;
                  zr && (jn.clearColor(+zr[0] || 0, +zr[1] || 0, +zr[2] || 0, +zr[3] || 0), si |= 16384), "depth" in Dr && (jn.clearDepth(+Dr.depth), si |= 256), "stencil" in Dr && (jn.clearStencil(0 | Dr.stencil), si |= 1024), jn.clear(si);
                }
                function Mn(Sr) {
                  return oa.push(Sr), yn(), { cancel: function() {
                    var Dr = Je(oa, Sr);
                    oa[Dr] = function si() {
                      var zr = Je(oa, si);
                      oa[zr] = oa[oa.length - 1], --oa.length, 0 >= oa.length && _n();
                    };
                  } };
                }
                function Ln() {
                  var Sr = ei.viewport, Dr = ei.scissor_box;
                  Sr[0] = Sr[1] = Dr[0] = Dr[1] = 0, Bi.viewportWidth = Bi.framebufferWidth = Bi.drawingBufferWidth = Sr[2] = Dr[2] = jn.drawingBufferWidth, Bi.viewportHeight = Bi.framebufferHeight = Bi.drawingBufferHeight = Sr[3] = Dr[3] = jn.drawingBufferHeight;
                }
                function In() {
                  Bi.tick += 1, Bi.time = er(), Ln(), Pr.procs.poll();
                }
                function kn() {
                  Zi.refresh(), Ln(), Pr.procs.refresh(), Yr && Yr.update();
                }
                function er() {
                  return (yt() - jr) / 1e3;
                }
                if (!(Ot = ae(Ot))) return null;
                var jn = Ot.gl, ur = jn.getContextAttributes();
                jn.isContextLost();
                var Kn = function(Sr, Dr) {
                  function si(la) {
                    var wa;
                    la = la.toLowerCase();
                    try {
                      wa = zr[la] = Sr.getExtension(la);
                    } catch {
                    }
                    return !!wa;
                  }
                  for (var zr = {}, di = 0; di < Dr.extensions.length; ++di) {
                    var Da = Dr.extensions[di];
                    if (!si(Da)) return Dr.onDestroy(), Dr.onDone('"' + Da + '" extension is not supported by the current WebGL context, try upgrading your system or a different browser'), null;
                  }
                  return Dr.optionalExtensions.forEach(si), { extensions: zr, restore: function() {
                    Object.keys(zr).forEach(function(la) {
                      if (zr[la] && !si(la)) throw Error("(regl): error restoring extension " + la);
                    });
                  } };
                }(jn, Ot);
                if (!Kn) return null;
                var Jn = /* @__PURE__ */ function() {
                  var Sr = { "": 0 }, Dr = [""];
                  return { id: function(si) {
                    var zr = Sr[si];
                    return zr || (zr = Sr[si] = Dr.length, Dr.push(si), zr);
                  }, str: function(si) {
                    return Dr[si];
                  } };
                }(), Lr = { vaoCount: 0, bufferCount: 0, elementsCount: 0, framebufferCount: 0, shaderCount: 0, textureCount: 0, cubeCount: 0, renderbufferCount: 0, maxTextureUnits: 0 }, mr = Ot.cachedCode || {}, Ar = Kn.extensions, Yr = function(Sr, Dr) {
                  function si() {
                    this.endQueryIndex = this.startQueryIndex = -1, this.sum = 0, this.stats = null;
                  }
                  function zr(Ur, rr, vr) {
                    var Rr = la.pop() || new si();
                    Rr.startQueryIndex = Ur, Rr.endQueryIndex = rr, Rr.sum = 0, Rr.stats = vr, wa.push(Rr);
                  }
                  if (!Dr.ext_disjoint_timer_query) return null;
                  var di = [], Da = [], la = [], wa = [], Wa = [], Qr = [];
                  return { beginQuery: function(Ur) {
                    var rr = di.pop() || Dr.ext_disjoint_timer_query.createQueryEXT();
                    Dr.ext_disjoint_timer_query.beginQueryEXT(35007, rr), Da.push(rr), zr(Da.length - 1, Da.length, Ur);
                  }, endQuery: function() {
                    Dr.ext_disjoint_timer_query.endQueryEXT(35007);
                  }, pushScopeStats: zr, update: function() {
                    var Ur, rr;
                    if ((Ur = Da.length) !== 0) {
                      Qr.length = Math.max(Qr.length, Ur + 1), Wa.length = Math.max(Wa.length, Ur + 1), Wa[0] = 0;
                      var vr = Qr[0] = 0;
                      for (rr = Ur = 0; rr < Da.length; ++rr) {
                        var Rr = Da[rr];
                        Dr.ext_disjoint_timer_query.getQueryObjectEXT(Rr, 34919) ? (vr += Dr.ext_disjoint_timer_query.getQueryObjectEXT(Rr, 34918), di.push(Rr)) : Da[Ur++] = Rr, Wa[rr + 1] = vr, Qr[rr + 1] = Ur;
                      }
                      for (Da.length = Ur, rr = Ur = 0; rr < wa.length; ++rr) {
                        var Vr = (vr = wa[rr]).startQueryIndex;
                        Rr = vr.endQueryIndex, vr.sum += Wa[Rr] - Wa[Vr], Vr = Qr[Vr], (Rr = Qr[Rr]) === Vr ? (vr.stats.gpuTime += vr.sum / 1e6, la.push(vr)) : (vr.startQueryIndex = Vr, vr.endQueryIndex = Rr, wa[Ur++] = vr);
                      }
                      wa.length = Ur;
                    }
                  }, getNumPendingQueries: function() {
                    return Da.length;
                  }, clear: function() {
                    di.push.apply(di, Da);
                    for (var Ur = 0; Ur < di.length; Ur++) Dr.ext_disjoint_timer_query.deleteQueryEXT(di[Ur]);
                    Da.length = 0, di.length = 0;
                  }, restore: function() {
                    Da.length = 0, di.length = 0;
                  } };
                }(0, Ar), jr = yt(), Mr = jn.drawingBufferWidth, Wr = jn.drawingBufferHeight, Bi = { tick: 0, time: 0, viewportWidth: Mr, viewportHeight: Wr, framebufferWidth: Mr, framebufferHeight: Wr, drawingBufferWidth: Mr, drawingBufferHeight: Wr, pixelRatio: Ot.pixelRatio }, gi = (Mr = { elements: null, primitive: 4, count: -1, offset: 0, instances: -1 }, Lt(jn, Ar)), Zr = pe(jn, Lr, Ot, function(Sr) {
                  return ri.destroyBuffer(Sr);
                }), fi = ye(jn, Ar, Zr, Lr), ri = Re(jn, Ar, gi, Lr, Zr, fi, Mr), bi = De(jn, Jn, Lr, Ot), Zi = Qe(jn, Ar, gi, function() {
                  Pr.procs.poll();
                }, Bi, Lr, Ot), $i = wn(jn, Ar, 0, Lr, Ot), Br = ke(jn, Ar, gi, Zi, $i, Lr), Pr = dt(jn, Jn, Ar, gi, Zr, fi, 0, Br, {}, ri, bi, Mr, Bi, Yr, mr, Ot), ei = (Jn = Pe(jn, Br, Pr.procs.poll, Bi), Pr.next), Ci = jn.canvas, oa = [], ka = [], aa = [], hi = [Ot.onDestroy], Gr = null;
                Ci && (Ci.addEventListener("webglcontextlost", gn, !1), Ci.addEventListener("webglcontextrestored", bn, !1));
                var Ca = Br.setFBO = an({ framebuffer: gt.define.call(null, 1, "framebuffer") });
                return kn(), ur = it(an, { clear: function(Sr) {
                  if ("framebuffer" in Sr) if (Sr.framebuffer && Sr.framebuffer_reglType === "framebufferCube") for (var Dr = 0; 6 > Dr; ++Dr) Ca(it({ framebuffer: Sr.framebuffer.faces[Dr] }, Sr), Zt);
                  else Ca(Sr, Zt);
                  else Zt(0, Sr);
                }, prop: gt.define.bind(null, 1), context: gt.define.bind(null, 2), this: gt.define.bind(null, 3), draw: an({}), buffer: function(Sr) {
                  return Zr.create(Sr, 34962, !1, !1);
                }, elements: function(Sr) {
                  return fi.create(Sr, !1);
                }, texture: Zi.create2D, cube: Zi.createCube, renderbuffer: $i.create, framebuffer: Br.create, framebufferCube: Br.createCube, vao: ri.createVAO, attributes: ur, frame: Mn, on: function(Sr, Dr) {
                  var si;
                  switch (Sr) {
                    case "frame":
                      return Mn(Dr);
                    case "lost":
                      si = ka;
                      break;
                    case "restore":
                      si = aa;
                      break;
                    case "destroy":
                      si = hi;
                  }
                  return si.push(Dr), { cancel: function() {
                    for (var zr = 0; zr < si.length; ++zr) if (si[zr] === Dr) {
                      si[zr] = si[si.length - 1], si.pop();
                      break;
                    }
                  } };
                }, limits: gi, hasExtension: function(Sr) {
                  return 0 <= gi.extensions.indexOf(Sr.toLowerCase());
                }, read: Jn, destroy: function() {
                  oa.length = 0, _n(), Ci && (Ci.removeEventListener("webglcontextlost", gn), Ci.removeEventListener("webglcontextrestored", bn)), bi.clear(), Br.clear(), $i.clear(), ri.clear(), Zi.clear(), fi.clear(), Zr.clear(), Yr && Yr.clear(), hi.forEach(function(Sr) {
                    Sr();
                  });
                }, _gl: jn, _refresh: kn, poll: function() {
                  In(), Yr && Yr.update();
                }, now: er, stats: Lr, getCachedCode: function() {
                  return mr;
                }, preloadCachedCode: function(Sr) {
                  Object.entries(Sr).forEach(function(Dr) {
                    mr[Dr[0]] = Dr[1];
                  });
                } }), Ot.onDone(null, ur), ur;
              };
            }();
          }, 41041: function(G, J, Z) {
            var te = Z(45708), ne = te.Buffer;
            function ee(oe, se) {
              for (var ce in oe) se[ce] = oe[ce];
            }
            function ae(oe, se, ce) {
              return ne(oe, se, ce);
            }
            ne.from && ne.alloc && ne.allocUnsafe && ne.allocUnsafeSlow ? G.exports = te : (ee(te, J), J.Buffer = ae), ae.prototype = Object.create(ne.prototype), ee(ne, ae), ae.from = function(oe, se, ce) {
              if (typeof oe == "number") throw new TypeError("Argument must not be a number");
              return ne(oe, se, ce);
            }, ae.alloc = function(oe, se, ce) {
              if (typeof oe != "number") throw new TypeError("Argument must be a number");
              var le = ne(oe);
              return se !== void 0 ? typeof ce == "string" ? le.fill(se, ce) : le.fill(se) : le.fill(0), le;
            }, ae.allocUnsafe = function(oe) {
              if (typeof oe != "number") throw new TypeError("Argument must be a number");
              return ne(oe);
            }, ae.allocUnsafeSlow = function(oe) {
              if (typeof oe != "number") throw new TypeError("Argument must be a number");
              return te.SlowBuffer(oe);
            };
          }, 73285: function(G, J, Z) {
            var te = Z(71129), ne = Z(70973), ee = Z(74268)(), ae = Z(52991), oe = Z(48631), se = te("%Math.floor%");
            G.exports = function(ce, le) {
              if (typeof ce != "function") throw new oe("`fn` is not a function");
              if (typeof le != "number" || le < 0 || le > 4294967295 || se(le) !== le) throw new oe("`length` must be a positive 32-bit integer");
              var ue = arguments.length > 2 && !!arguments[2], de = !0, fe = !0;
              if ("length" in ce && ae) {
                var he = ae(ce, "length");
                he && !he.configurable && (de = !1), he && !he.writable && (fe = !1);
              }
              return (de || fe || !ue) && (ee ? ne(ce, "length", le, !0, !0) : ne(ce, "length", le)), ce;
            };
          }, 90386: function(G, J, Z) {
            G.exports = ne;
            var te = Z(7683).EventEmitter;
            function ne() {
              te.call(this);
            }
            Z(28062)(ne, te), ne.Readable = Z(44639), ne.Writable = Z(84627), ne.Duplex = Z(71977), ne.Transform = Z(40255), ne.PassThrough = Z(28765), ne.finished = Z(37165), ne.pipeline = Z(6772), ne.Stream = ne, ne.prototype.pipe = function(ee, ae) {
              var oe = this;
              function se(me) {
                ee.writable && ee.write(me) === !1 && oe.pause && oe.pause();
              }
              function ce() {
                oe.readable && oe.resume && oe.resume();
              }
              oe.on("data", se), ee.on("drain", ce), ee._isStdio || ae && ae.end === !1 || (oe.on("end", ue), oe.on("close", de));
              var le = !1;
              function ue() {
                le || (le = !0, ee.end());
              }
              function de() {
                le || (le = !0, typeof ee.destroy == "function" && ee.destroy());
              }
              function fe(me) {
                if (he(), te.listenerCount(this, "error") === 0) throw me;
              }
              function he() {
                oe.removeListener("data", se), ee.removeListener("drain", ce), oe.removeListener("end", ue), oe.removeListener("close", de), oe.removeListener("error", fe), ee.removeListener("error", fe), oe.removeListener("end", he), oe.removeListener("close", he), ee.removeListener("close", he);
              }
              return oe.on("error", fe), ee.on("error", fe), oe.on("end", he), oe.on("close", he), ee.on("close", he), ee.emit("pipe", oe), ee;
            };
          }, 44059: function(G) {
            var J = {};
            function Z(ne, ee, ae) {
              ae || (ae = Error);
              var oe = function(se) {
                var ce, le;
                function ue(de, fe, he) {
                  return se.call(this, function(me, pe, ye) {
                    return typeof ee == "string" ? ee : ee(me, pe, ye);
                  }(de, fe, he)) || this;
                }
                return le = se, (ce = ue).prototype = Object.create(le.prototype), ce.prototype.constructor = ce, ce.__proto__ = le, ue;
              }(ae);
              oe.prototype.name = ae.name, oe.prototype.code = ne, J[ne] = oe;
            }
            function te(ne, ee) {
              if (Array.isArray(ne)) {
                var ae = ne.length;
                return ne = ne.map(function(oe) {
                  return String(oe);
                }), ae > 2 ? "one of ".concat(ee, " ").concat(ne.slice(0, ae - 1).join(", "), ", or ") + ne[ae - 1] : ae === 2 ? "one of ".concat(ee, " ").concat(ne[0], " or ").concat(ne[1]) : "of ".concat(ee, " ").concat(ne[0]);
              }
              return "of ".concat(ee, " ").concat(String(ne));
            }
            Z("ERR_INVALID_OPT_VALUE", function(ne, ee) {
              return 'The value "' + ee + '" is invalid for option "' + ne + '"';
            }, TypeError), Z("ERR_INVALID_ARG_TYPE", function(ne, ee, ae) {
              var oe, se, ce, le, ue;
              if (typeof ee == "string" && (se = "not ", ee.substr(0, 4) === se) ? (oe = "must not be", ee = ee.replace(/^not /, "")) : oe = "must be", function(fe, he, me) {
                return (me === void 0 || me > fe.length) && (me = fe.length), fe.substring(me - 9, me) === he;
              }(ne, " argument")) ce = "The ".concat(ne, " ").concat(oe, " ").concat(te(ee, "type"));
              else {
                var de = (typeof ue != "number" && (ue = 0), ue + 1 > (le = ne).length || le.indexOf(".", ue) === -1 ? "argument" : "property");
                ce = 'The "'.concat(ne, '" ').concat(de, " ").concat(oe, " ").concat(te(ee, "type"));
              }
              return ce + ". Received type ".concat(typeof ae);
            }, TypeError), Z("ERR_STREAM_PUSH_AFTER_EOF", "stream.push() after EOF"), Z("ERR_METHOD_NOT_IMPLEMENTED", function(ne) {
              return "The " + ne + " method is not implemented";
            }), Z("ERR_STREAM_PREMATURE_CLOSE", "Premature close"), Z("ERR_STREAM_DESTROYED", function(ne) {
              return "Cannot call " + ne + " after a stream was destroyed";
            }), Z("ERR_MULTIPLE_CALLBACK", "Callback called multiple times"), Z("ERR_STREAM_CANNOT_PIPE", "Cannot pipe, not readable"), Z("ERR_STREAM_WRITE_AFTER_END", "write after end"), Z("ERR_STREAM_NULL_VALUES", "May not write null values to stream", TypeError), Z("ERR_UNKNOWN_ENCODING", function(ne) {
              return "Unknown encoding: " + ne;
            }, TypeError), Z("ERR_STREAM_UNSHIFT_AFTER_END_EVENT", "stream.unshift() after end event"), G.exports.F = J;
          }, 71977: function(G, J, Z) {
            var te = Z(33282), ne = Object.keys || function(fe) {
              var he = [];
              for (var me in fe) he.push(me);
              return he;
            };
            G.exports = le;
            var ee = Z(44639), ae = Z(84627);
            Z(28062)(le, ee);
            for (var oe = ne(ae.prototype), se = 0; se < oe.length; se++) {
              var ce = oe[se];
              le.prototype[ce] || (le.prototype[ce] = ae.prototype[ce]);
            }
            function le(fe) {
              if (!(this instanceof le)) return new le(fe);
              ee.call(this, fe), ae.call(this, fe), this.allowHalfOpen = !0, fe && (fe.readable === !1 && (this.readable = !1), fe.writable === !1 && (this.writable = !1), fe.allowHalfOpen === !1 && (this.allowHalfOpen = !1, this.once("end", ue)));
            }
            function ue() {
              this._writableState.ended || te.nextTick(de, this);
            }
            function de(fe) {
              fe.end();
            }
            Object.defineProperty(le.prototype, "writableHighWaterMark", { enumerable: !1, get: function() {
              return this._writableState.highWaterMark;
            } }), Object.defineProperty(le.prototype, "writableBuffer", { enumerable: !1, get: function() {
              return this._writableState && this._writableState.getBuffer();
            } }), Object.defineProperty(le.prototype, "writableLength", { enumerable: !1, get: function() {
              return this._writableState.length;
            } }), Object.defineProperty(le.prototype, "destroyed", { enumerable: !1, get: function() {
              return this._readableState !== void 0 && this._writableState !== void 0 && this._readableState.destroyed && this._writableState.destroyed;
            }, set: function(fe) {
              this._readableState !== void 0 && this._writableState !== void 0 && (this._readableState.destroyed = fe, this._writableState.destroyed = fe);
            } });
          }, 28765: function(G, J, Z) {
            G.exports = ne;
            var te = Z(40255);
            function ne(ee) {
              if (!(this instanceof ne)) return new ne(ee);
              te.call(this, ee);
            }
            Z(28062)(ne, te), ne.prototype._transform = function(ee, ae, oe) {
              oe(null, ee);
            };
          }, 44639: function(G, J, Z) {
            var te, ne = Z(33282);
            G.exports = xe, xe.ReadableState = Ee, Z(7683).EventEmitter;
            var ee, ae = function(ct, et) {
              return ct.listeners(et).length;
            }, oe = Z(60032), se = Z(45708).Buffer, ce = Z.g.Uint8Array || function() {
            }, le = Z(77199);
            ee = le && le.debuglog ? le.debuglog("stream") : function() {
            };
            var ue, de, fe, he = Z(29930), me = Z(52023), pe = Z(31976).getHighWaterMark, ye = Z(44059).F, ge = ye.ERR_INVALID_ARG_TYPE, ve = ye.ERR_STREAM_PUSH_AFTER_EOF, be = ye.ERR_METHOD_NOT_IMPLEMENTED, Ce = ye.ERR_STREAM_UNSHIFT_AFTER_END_EVENT;
            Z(28062)(xe, oe);
            var Se = me.errorOrDestroy, we = ["error", "close", "destroy", "pause", "resume"];
            function Ee(ct, et, dt) {
              te = te || Z(71977), ct = ct || {}, typeof dt != "boolean" && (dt = et instanceof te), this.objectMode = !!ct.objectMode, dt && (this.objectMode = this.objectMode || !!ct.readableObjectMode), this.highWaterMark = pe(this, ct, "readableHighWaterMark", dt), this.buffer = new he(), this.length = 0, this.pipes = null, this.pipesCount = 0, this.flowing = null, this.ended = !1, this.endEmitted = !1, this.reading = !1, this.sync = !0, this.needReadable = !1, this.emittedReadable = !1, this.readableListening = !1, this.resumeScheduled = !1, this.paused = !0, this.emitClose = ct.emitClose !== !1, this.autoDestroy = !!ct.autoDestroy, this.destroyed = !1, this.defaultEncoding = ct.defaultEncoding || "utf8", this.awaitDrain = 0, this.readingMore = !1, this.decoder = null, this.encoding = null, ct.encoding && (ue || (ue = Z(54304).I), this.decoder = new ue(ct.encoding), this.encoding = ct.encoding);
            }
            function xe(ct) {
              if (te = te || Z(71977), !(this instanceof xe)) return new xe(ct);
              var et = this instanceof te;
              this._readableState = new Ee(ct, this, et), this.readable = !0, ct && (typeof ct.read == "function" && (this._read = ct.read), typeof ct.destroy == "function" && (this._destroy = ct.destroy)), oe.call(this);
            }
            function Le(ct, et, dt, Je, it) {
              ee("readableAddChunk", et);
              var st, gt = ct._readableState;
              if (et === null) gt.reading = !1, function(bt, yt) {
                if (ee("onEofChunk"), !yt.ended) {
                  if (yt.decoder) {
                    var St = yt.decoder.end();
                    St && St.length && (yt.buffer.push(St), yt.length += yt.objectMode ? 1 : St.length);
                  }
                  yt.ended = !0, yt.sync ? Re(bt) : (yt.needReadable = !1, yt.emittedReadable || (yt.emittedReadable = !0, De(bt)));
                }
              }(ct, gt);
              else if (it || (st = function(bt, yt) {
                var St, Lt;
                return Lt = yt, se.isBuffer(Lt) || Lt instanceof ce || typeof yt == "string" || yt === void 0 || bt.objectMode || (St = new ge("chunk", ["string", "Buffer", "Uint8Array"], yt)), St;
              }(gt, et)), st) Se(ct, st);
              else if (gt.objectMode || et && et.length > 0) if (typeof et == "string" || gt.objectMode || Object.getPrototypeOf(et) === se.prototype || (et = function(bt) {
                return se.from(bt);
              }(et)), Je) gt.endEmitted ? Se(ct, new Ce()) : Qe(ct, gt, et, !0);
              else if (gt.ended) Se(ct, new ve());
              else {
                if (gt.destroyed) return !1;
                gt.reading = !1, gt.decoder && !dt ? (et = gt.decoder.write(et), gt.objectMode || et.length !== 0 ? Qe(ct, gt, et, !1) : Pe(ct, gt)) : Qe(ct, gt, et, !1);
              }
              else Je || (gt.reading = !1, Pe(ct, gt));
              return !gt.ended && (gt.length < gt.highWaterMark || gt.length === 0);
            }
            function Qe(ct, et, dt, Je) {
              et.flowing && et.length === 0 && !et.sync ? (et.awaitDrain = 0, ct.emit("data", dt)) : (et.length += et.objectMode ? 1 : dt.length, Je ? et.buffer.unshift(dt) : et.buffer.push(dt), et.needReadable && Re(ct)), Pe(ct, et);
            }
            Object.defineProperty(xe.prototype, "destroyed", { enumerable: !1, get: function() {
              return this._readableState !== void 0 && this._readableState.destroyed;
            }, set: function(ct) {
              this._readableState && (this._readableState.destroyed = ct);
            } }), xe.prototype.destroy = me.destroy, xe.prototype._undestroy = me.undestroy, xe.prototype._destroy = function(ct, et) {
              et(ct);
            }, xe.prototype.push = function(ct, et) {
              var dt, Je = this._readableState;
              return Je.objectMode ? dt = !0 : typeof ct == "string" && ((et = et || Je.defaultEncoding) !== Je.encoding && (ct = se.from(ct, et), et = ""), dt = !0), Le(this, ct, et, !1, dt);
            }, xe.prototype.unshift = function(ct) {
              return Le(this, ct, null, !0, !1);
            }, xe.prototype.isPaused = function() {
              return this._readableState.flowing === !1;
            }, xe.prototype.setEncoding = function(ct) {
              ue || (ue = Z(54304).I);
              var et = new ue(ct);
              this._readableState.decoder = et, this._readableState.encoding = this._readableState.decoder.encoding;
              for (var dt = this._readableState.buffer.head, Je = ""; dt !== null; ) Je += et.write(dt.data), dt = dt.next;
              return this._readableState.buffer.clear(), Je !== "" && this._readableState.buffer.push(Je), this._readableState.length = Je.length, this;
            };
            var ke = 1073741824;
            function Me(ct, et) {
              return ct <= 0 || et.length === 0 && et.ended ? 0 : et.objectMode ? 1 : ct != ct ? et.flowing && et.length ? et.buffer.head.data.length : et.length : (ct > et.highWaterMark && (et.highWaterMark = function(dt) {
                return dt >= ke ? dt = ke : (dt--, dt |= dt >>> 1, dt |= dt >>> 2, dt |= dt >>> 4, dt |= dt >>> 8, dt |= dt >>> 16, dt++), dt;
              }(ct)), ct <= et.length ? ct : et.ended ? et.length : (et.needReadable = !0, 0));
            }
            function Re(ct) {
              var et = ct._readableState;
              ee("emitReadable", et.needReadable, et.emittedReadable), et.needReadable = !1, et.emittedReadable || (ee("emitReadable", et.flowing), et.emittedReadable = !0, ne.nextTick(De, ct));
            }
            function De(ct) {
              var et = ct._readableState;
              ee("emitReadable_", et.destroyed, et.length, et.ended), et.destroyed || !et.length && !et.ended || (ct.emit("readable"), et.emittedReadable = !1), et.needReadable = !et.flowing && !et.ended && et.length <= et.highWaterMark, Ze(ct);
            }
            function Pe(ct, et) {
              et.readingMore || (et.readingMore = !0, ne.nextTick($e, ct, et));
            }
            function $e(ct, et) {
              for (; !et.reading && !et.ended && (et.length < et.highWaterMark || et.flowing && et.length === 0); ) {
                var dt = et.length;
                if (ee("maybeReadMore read 0"), ct.read(0), dt === et.length) break;
              }
              et.readingMore = !1;
            }
            function qe(ct) {
              var et = ct._readableState;
              et.readableListening = ct.listenerCount("readable") > 0, et.resumeScheduled && !et.paused ? et.flowing = !0 : ct.listenerCount("data") > 0 && ct.resume();
            }
            function Xe(ct) {
              ee("readable nexttick read 0"), ct.read(0);
            }
            function Ve(ct, et) {
              ee("resume", et.reading), et.reading || ct.read(0), et.resumeScheduled = !1, ct.emit("resume"), Ze(ct), et.flowing && !et.reading && ct.read(0);
            }
            function Ze(ct) {
              var et = ct._readableState;
              for (ee("flow", et.flowing); et.flowing && ct.read() !== null; ) ;
            }
            function Ge(ct, et) {
              return et.length === 0 ? null : (et.objectMode ? dt = et.buffer.shift() : !ct || ct >= et.length ? (dt = et.decoder ? et.buffer.join("") : et.buffer.length === 1 ? et.buffer.first() : et.buffer.concat(et.length), et.buffer.clear()) : dt = et.buffer.consume(ct, et.decoder), dt);
              var dt;
            }
            function ze(ct) {
              var et = ct._readableState;
              ee("endReadable", et.endEmitted), et.endEmitted || (et.ended = !0, ne.nextTick(We, et, ct));
            }
            function We(ct, et) {
              if (ee("endReadableNT", ct.endEmitted, ct.length), !ct.endEmitted && ct.length === 0 && (ct.endEmitted = !0, et.readable = !1, et.emit("end"), ct.autoDestroy)) {
                var dt = et._writableState;
                (!dt || dt.autoDestroy && dt.finished) && et.destroy();
              }
            }
            function at(ct, et) {
              for (var dt = 0, Je = ct.length; dt < Je; dt++) if (ct[dt] === et) return dt;
              return -1;
            }
            xe.prototype.read = function(ct) {
              ee("read", ct), ct = parseInt(ct, 10);
              var et = this._readableState, dt = ct;
              if (ct !== 0 && (et.emittedReadable = !1), ct === 0 && et.needReadable && ((et.highWaterMark !== 0 ? et.length >= et.highWaterMark : et.length > 0) || et.ended)) return ee("read: emitReadable", et.length, et.ended), et.length === 0 && et.ended ? ze(this) : Re(this), null;
              if ((ct = Me(ct, et)) === 0 && et.ended) return et.length === 0 && ze(this), null;
              var Je, it = et.needReadable;
              return ee("need readable", it), (et.length === 0 || et.length - ct < et.highWaterMark) && ee("length less than watermark", it = !0), et.ended || et.reading ? ee("reading or ended", it = !1) : it && (ee("do read"), et.reading = !0, et.sync = !0, et.length === 0 && (et.needReadable = !0), this._read(et.highWaterMark), et.sync = !1, et.reading || (ct = Me(dt, et))), (Je = ct > 0 ? Ge(ct, et) : null) === null ? (et.needReadable = et.length <= et.highWaterMark, ct = 0) : (et.length -= ct, et.awaitDrain = 0), et.length === 0 && (et.ended || (et.needReadable = !0), dt !== ct && et.ended && ze(this)), Je !== null && this.emit("data", Je), Je;
            }, xe.prototype._read = function(ct) {
              Se(this, new be("_read()"));
            }, xe.prototype.pipe = function(ct, et) {
              var dt = this, Je = this._readableState;
              switch (Je.pipesCount) {
                case 0:
                  Je.pipes = ct;
                  break;
                case 1:
                  Je.pipes = [Je.pipes, ct];
                  break;
                default:
                  Je.pipes.push(ct);
              }
              Je.pipesCount += 1, ee("pipe count=%d opts=%j", Je.pipesCount, et);
              var it = et && et.end === !1 || ct === ne.stdout || ct === ne.stderr ? vt : st;
              function st() {
                ee("onend"), ct.end();
              }
              Je.endEmitted ? ne.nextTick(it) : dt.once("end", it), ct.on("unpipe", function $t(Pt, Xt) {
                ee("onunpipe"), Pt === dt && Xt && Xt.hasUnpiped === !1 && (Xt.hasUnpiped = !0, ee("cleanup"), ct.removeListener("close", Lt), ct.removeListener("finish", Rt), ct.removeListener("drain", gt), ct.removeListener("error", St), ct.removeListener("unpipe", $t), dt.removeListener("end", st), dt.removeListener("end", vt), dt.removeListener("data", yt), bt = !0, !Je.awaitDrain || ct._writableState && !ct._writableState.needDrain || gt());
              });
              var gt = /* @__PURE__ */ function($t) {
                return function() {
                  var Pt = $t._readableState;
                  ee("pipeOnDrain", Pt.awaitDrain), Pt.awaitDrain && Pt.awaitDrain--, Pt.awaitDrain === 0 && ae($t, "data") && (Pt.flowing = !0, Ze($t));
                };
              }(dt);
              ct.on("drain", gt);
              var bt = !1;
              function yt($t) {
                ee("ondata");
                var Pt = ct.write($t);
                ee("dest.write", Pt), Pt === !1 && ((Je.pipesCount === 1 && Je.pipes === ct || Je.pipesCount > 1 && at(Je.pipes, ct) !== -1) && !bt && (ee("false write response, pause", Je.awaitDrain), Je.awaitDrain++), dt.pause());
              }
              function St($t) {
                ee("onerror", $t), vt(), ct.removeListener("error", St), ae(ct, "error") === 0 && Se(ct, $t);
              }
              function Lt() {
                ct.removeListener("finish", Rt), vt();
              }
              function Rt() {
                ee("onfinish"), ct.removeListener("close", Lt), vt();
              }
              function vt() {
                ee("unpipe"), dt.unpipe(ct);
              }
              return dt.on("data", yt), function($t, Pt, Xt) {
                if (typeof $t.prependListener == "function") return $t.prependListener(Pt, Xt);
                $t._events && $t._events[Pt] ? Array.isArray($t._events[Pt]) ? $t._events[Pt].unshift(Xt) : $t._events[Pt] = [Xt, $t._events[Pt]] : $t.on(Pt, Xt);
              }(ct, "error", St), ct.once("close", Lt), ct.once("finish", Rt), ct.emit("pipe", dt), Je.flowing || (ee("pipe resume"), dt.resume()), ct;
            }, xe.prototype.unpipe = function(ct) {
              var et = this._readableState, dt = { hasUnpiped: !1 };
              if (et.pipesCount === 0) return this;
              if (et.pipesCount === 1) return ct && ct !== et.pipes || (ct || (ct = et.pipes), et.pipes = null, et.pipesCount = 0, et.flowing = !1, ct && ct.emit("unpipe", this, dt)), this;
              if (!ct) {
                var Je = et.pipes, it = et.pipesCount;
                et.pipes = null, et.pipesCount = 0, et.flowing = !1;
                for (var st = 0; st < it; st++) Je[st].emit("unpipe", this, { hasUnpiped: !1 });
                return this;
              }
              var gt = at(et.pipes, ct);
              return gt === -1 || (et.pipes.splice(gt, 1), et.pipesCount -= 1, et.pipesCount === 1 && (et.pipes = et.pipes[0]), ct.emit("unpipe", this, dt)), this;
            }, xe.prototype.on = function(ct, et) {
              var dt = oe.prototype.on.call(this, ct, et), Je = this._readableState;
              return ct === "data" ? (Je.readableListening = this.listenerCount("readable") > 0, Je.flowing !== !1 && this.resume()) : ct === "readable" && (Je.endEmitted || Je.readableListening || (Je.readableListening = Je.needReadable = !0, Je.flowing = !1, Je.emittedReadable = !1, ee("on readable", Je.length, Je.reading), Je.length ? Re(this) : Je.reading || ne.nextTick(Xe, this))), dt;
            }, xe.prototype.addListener = xe.prototype.on, xe.prototype.removeListener = function(ct, et) {
              var dt = oe.prototype.removeListener.call(this, ct, et);
              return ct === "readable" && ne.nextTick(qe, this), dt;
            }, xe.prototype.removeAllListeners = function(ct) {
              var et = oe.prototype.removeAllListeners.apply(this, arguments);
              return ct !== "readable" && ct !== void 0 || ne.nextTick(qe, this), et;
            }, xe.prototype.resume = function() {
              var ct = this._readableState;
              return ct.flowing || (ee("resume"), ct.flowing = !ct.readableListening, function(et, dt) {
                dt.resumeScheduled || (dt.resumeScheduled = !0, ne.nextTick(Ve, et, dt));
              }(this, ct)), ct.paused = !1, this;
            }, xe.prototype.pause = function() {
              return ee("call pause flowing=%j", this._readableState.flowing), this._readableState.flowing !== !1 && (ee("pause"), this._readableState.flowing = !1, this.emit("pause")), this._readableState.paused = !0, this;
            }, xe.prototype.wrap = function(ct) {
              var et = this, dt = this._readableState, Je = !1;
              for (var it in ct.on("end", function() {
                if (ee("wrapped end"), dt.decoder && !dt.ended) {
                  var gt = dt.decoder.end();
                  gt && gt.length && et.push(gt);
                }
                et.push(null);
              }), ct.on("data", function(gt) {
                ee("wrapped data"), dt.decoder && (gt = dt.decoder.write(gt)), dt.objectMode && gt == null || (dt.objectMode || gt && gt.length) && (et.push(gt) || (Je = !0, ct.pause()));
              }), ct) this[it] === void 0 && typeof ct[it] == "function" && (this[it] = /* @__PURE__ */ function(gt) {
                return function() {
                  return ct[gt].apply(ct, arguments);
                };
              }(it));
              for (var st = 0; st < we.length; st++) ct.on(we[st], this.emit.bind(this, we[st]));
              return this._read = function(gt) {
                ee("wrapped _read", gt), Je && (Je = !1, ct.resume());
              }, this;
            }, typeof Symbol == "function" && (xe.prototype[Symbol.asyncIterator] = function() {
              return de === void 0 && (de = Z(73726)), de(this);
            }), Object.defineProperty(xe.prototype, "readableHighWaterMark", { enumerable: !1, get: function() {
              return this._readableState.highWaterMark;
            } }), Object.defineProperty(xe.prototype, "readableBuffer", { enumerable: !1, get: function() {
              return this._readableState && this._readableState.buffer;
            } }), Object.defineProperty(xe.prototype, "readableFlowing", { enumerable: !1, get: function() {
              return this._readableState.flowing;
            }, set: function(ct) {
              this._readableState && (this._readableState.flowing = ct);
            } }), xe._fromList = Ge, Object.defineProperty(xe.prototype, "readableLength", { enumerable: !1, get: function() {
              return this._readableState.length;
            } }), typeof Symbol == "function" && (xe.from = function(ct, et) {
              return fe === void 0 && (fe = Z(37108)), fe(xe, ct, et);
            });
          }, 40255: function(G, J, Z) {
            G.exports = le;
            var te = Z(44059).F, ne = te.ERR_METHOD_NOT_IMPLEMENTED, ee = te.ERR_MULTIPLE_CALLBACK, ae = te.ERR_TRANSFORM_ALREADY_TRANSFORMING, oe = te.ERR_TRANSFORM_WITH_LENGTH_0, se = Z(71977);
            function ce(fe, he) {
              var me = this._transformState;
              me.transforming = !1;
              var pe = me.writecb;
              if (pe === null) return this.emit("error", new ee());
              me.writechunk = null, me.writecb = null, he != null && this.push(he), pe(fe);
              var ye = this._readableState;
              ye.reading = !1, (ye.needReadable || ye.length < ye.highWaterMark) && this._read(ye.highWaterMark);
            }
            function le(fe) {
              if (!(this instanceof le)) return new le(fe);
              se.call(this, fe), this._transformState = { afterTransform: ce.bind(this), needTransform: !1, transforming: !1, writecb: null, writechunk: null, writeencoding: null }, this._readableState.needReadable = !0, this._readableState.sync = !1, fe && (typeof fe.transform == "function" && (this._transform = fe.transform), typeof fe.flush == "function" && (this._flush = fe.flush)), this.on("prefinish", ue);
            }
            function ue() {
              var fe = this;
              typeof this._flush != "function" || this._readableState.destroyed ? de(this, null, null) : this._flush(function(he, me) {
                de(fe, he, me);
              });
            }
            function de(fe, he, me) {
              if (he) return fe.emit("error", he);
              if (me != null && fe.push(me), fe._writableState.length) throw new oe();
              if (fe._transformState.transforming) throw new ae();
              return fe.push(null);
            }
            Z(28062)(le, se), le.prototype.push = function(fe, he) {
              return this._transformState.needTransform = !1, se.prototype.push.call(this, fe, he);
            }, le.prototype._transform = function(fe, he, me) {
              me(new ne("_transform()"));
            }, le.prototype._write = function(fe, he, me) {
              var pe = this._transformState;
              if (pe.writecb = me, pe.writechunk = fe, pe.writeencoding = he, !pe.transforming) {
                var ye = this._readableState;
                (pe.needTransform || ye.needReadable || ye.length < ye.highWaterMark) && this._read(ye.highWaterMark);
              }
            }, le.prototype._read = function(fe) {
              var he = this._transformState;
              he.writechunk === null || he.transforming ? he.needTransform = !0 : (he.transforming = !0, this._transform(he.writechunk, he.writeencoding, he.afterTransform));
            }, le.prototype._destroy = function(fe, he) {
              se.prototype._destroy.call(this, fe, function(me) {
                he(me);
              });
            };
          }, 84627: function(G, J, Z) {
            var te, ne = Z(33282);
            function ee(Pe) {
              var $e = this;
              this.next = null, this.entry = null, this.finish = function() {
                (function(qe, Xe, Ve) {
                  var Ze = qe.entry;
                  for (qe.entry = null; Ze; ) {
                    var Ge = Ze.callback;
                    Xe.pendingcb--, Ge(void 0), Ze = Ze.next;
                  }
                  Xe.corkedRequestsFree.next = qe;
                })($e, Pe);
              };
            }
            G.exports = xe, xe.WritableState = Ee;
            var ae, oe = { deprecate: Z(71103) }, se = Z(60032), ce = Z(45708).Buffer, le = Z.g.Uint8Array || function() {
            }, ue = Z(52023), de = Z(31976).getHighWaterMark, fe = Z(44059).F, he = fe.ERR_INVALID_ARG_TYPE, me = fe.ERR_METHOD_NOT_IMPLEMENTED, pe = fe.ERR_MULTIPLE_CALLBACK, ye = fe.ERR_STREAM_CANNOT_PIPE, ge = fe.ERR_STREAM_DESTROYED, ve = fe.ERR_STREAM_NULL_VALUES, be = fe.ERR_STREAM_WRITE_AFTER_END, Ce = fe.ERR_UNKNOWN_ENCODING, Se = ue.errorOrDestroy;
            function we() {
            }
            function Ee(Pe, $e, qe) {
              te = te || Z(71977), Pe = Pe || {}, typeof qe != "boolean" && (qe = $e instanceof te), this.objectMode = !!Pe.objectMode, qe && (this.objectMode = this.objectMode || !!Pe.writableObjectMode), this.highWaterMark = de(this, Pe, "writableHighWaterMark", qe), this.finalCalled = !1, this.needDrain = !1, this.ending = !1, this.ended = !1, this.finished = !1, this.destroyed = !1;
              var Xe = Pe.decodeStrings === !1;
              this.decodeStrings = !Xe, this.defaultEncoding = Pe.defaultEncoding || "utf8", this.length = 0, this.writing = !1, this.corked = 0, this.sync = !0, this.bufferProcessing = !1, this.onwrite = function(Ve) {
                (function(Ze, Ge) {
                  var ze = Ze._writableState, We = ze.sync, at = ze.writecb;
                  if (typeof at != "function") throw new pe();
                  if (function(et) {
                    et.writing = !1, et.writecb = null, et.length -= et.writelen, et.writelen = 0;
                  }(ze), Ge) (function(et, dt, Je, it, st) {
                    --dt.pendingcb, Je ? (ne.nextTick(st, it), ne.nextTick(De, et, dt), et._writableState.errorEmitted = !0, Se(et, it)) : (st(it), et._writableState.errorEmitted = !0, Se(et, it), De(et, dt));
                  })(Ze, ze, We, Ge, at);
                  else {
                    var ct = Me(ze) || Ze.destroyed;
                    ct || ze.corked || ze.bufferProcessing || !ze.bufferedRequest || ke(Ze, ze), We ? ne.nextTick(Qe, Ze, ze, ct, at) : Qe(Ze, ze, ct, at);
                  }
                })($e, Ve);
              }, this.writecb = null, this.writelen = 0, this.bufferedRequest = null, this.lastBufferedRequest = null, this.pendingcb = 0, this.prefinished = !1, this.errorEmitted = !1, this.emitClose = Pe.emitClose !== !1, this.autoDestroy = !!Pe.autoDestroy, this.bufferedRequestCount = 0, this.corkedRequestsFree = new ee(this);
            }
            function xe(Pe) {
              var $e = this instanceof (te = te || Z(71977));
              if (!$e && !ae.call(xe, this)) return new xe(Pe);
              this._writableState = new Ee(Pe, this, $e), this.writable = !0, Pe && (typeof Pe.write == "function" && (this._write = Pe.write), typeof Pe.writev == "function" && (this._writev = Pe.writev), typeof Pe.destroy == "function" && (this._destroy = Pe.destroy), typeof Pe.final == "function" && (this._final = Pe.final)), se.call(this);
            }
            function Le(Pe, $e, qe, Xe, Ve, Ze, Ge) {
              $e.writelen = Xe, $e.writecb = Ge, $e.writing = !0, $e.sync = !0, $e.destroyed ? $e.onwrite(new ge("write")) : qe ? Pe._writev(Ve, $e.onwrite) : Pe._write(Ve, Ze, $e.onwrite), $e.sync = !1;
            }
            function Qe(Pe, $e, qe, Xe) {
              qe || function(Ve, Ze) {
                Ze.length === 0 && Ze.needDrain && (Ze.needDrain = !1, Ve.emit("drain"));
              }(Pe, $e), $e.pendingcb--, Xe(), De(Pe, $e);
            }
            function ke(Pe, $e) {
              $e.bufferProcessing = !0;
              var qe = $e.bufferedRequest;
              if (Pe._writev && qe && qe.next) {
                var Xe = $e.bufferedRequestCount, Ve = new Array(Xe), Ze = $e.corkedRequestsFree;
                Ze.entry = qe;
                for (var Ge = 0, ze = !0; qe; ) Ve[Ge] = qe, qe.isBuf || (ze = !1), qe = qe.next, Ge += 1;
                Ve.allBuffers = ze, Le(Pe, $e, !0, $e.length, Ve, "", Ze.finish), $e.pendingcb++, $e.lastBufferedRequest = null, Ze.next ? ($e.corkedRequestsFree = Ze.next, Ze.next = null) : $e.corkedRequestsFree = new ee($e), $e.bufferedRequestCount = 0;
              } else {
                for (; qe; ) {
                  var We = qe.chunk, at = qe.encoding, ct = qe.callback;
                  if (Le(Pe, $e, !1, $e.objectMode ? 1 : We.length, We, at, ct), qe = qe.next, $e.bufferedRequestCount--, $e.writing) break;
                }
                qe === null && ($e.lastBufferedRequest = null);
              }
              $e.bufferedRequest = qe, $e.bufferProcessing = !1;
            }
            function Me(Pe) {
              return Pe.ending && Pe.length === 0 && Pe.bufferedRequest === null && !Pe.finished && !Pe.writing;
            }
            function Re(Pe, $e) {
              Pe._final(function(qe) {
                $e.pendingcb--, qe && Se(Pe, qe), $e.prefinished = !0, Pe.emit("prefinish"), De(Pe, $e);
              });
            }
            function De(Pe, $e) {
              var qe = Me($e);
              if (qe && (function(Ve, Ze) {
                Ze.prefinished || Ze.finalCalled || (typeof Ve._final != "function" || Ze.destroyed ? (Ze.prefinished = !0, Ve.emit("prefinish")) : (Ze.pendingcb++, Ze.finalCalled = !0, ne.nextTick(Re, Ve, Ze)));
              }(Pe, $e), $e.pendingcb === 0 && ($e.finished = !0, Pe.emit("finish"), $e.autoDestroy))) {
                var Xe = Pe._readableState;
                (!Xe || Xe.autoDestroy && Xe.endEmitted) && Pe.destroy();
              }
              return qe;
            }
            Z(28062)(xe, se), Ee.prototype.getBuffer = function() {
              for (var Pe = this.bufferedRequest, $e = []; Pe; ) $e.push(Pe), Pe = Pe.next;
              return $e;
            }, function() {
              try {
                Object.defineProperty(Ee.prototype, "buffer", { get: oe.deprecate(function() {
                  return this.getBuffer();
                }, "_writableState.buffer is deprecated. Use _writableState.getBuffer instead.", "DEP0003") });
              } catch {
              }
            }(), typeof Symbol == "function" && Symbol.hasInstance && typeof Function.prototype[Symbol.hasInstance] == "function" ? (ae = Function.prototype[Symbol.hasInstance], Object.defineProperty(xe, Symbol.hasInstance, { value: function(Pe) {
              return !!ae.call(this, Pe) || this === xe && Pe && Pe._writableState instanceof Ee;
            } })) : ae = function(Pe) {
              return Pe instanceof this;
            }, xe.prototype.pipe = function() {
              Se(this, new ye());
            }, xe.prototype.write = function(Pe, $e, qe) {
              var Xe, Ve = this._writableState, Ze = !1, Ge = !Ve.objectMode && (Xe = Pe, ce.isBuffer(Xe) || Xe instanceof le);
              return Ge && !ce.isBuffer(Pe) && (Pe = function(ze) {
                return ce.from(ze);
              }(Pe)), typeof $e == "function" && (qe = $e, $e = null), Ge ? $e = "buffer" : $e || ($e = Ve.defaultEncoding), typeof qe != "function" && (qe = we), Ve.ending ? function(ze, We) {
                var at = new be();
                Se(ze, at), ne.nextTick(We, at);
              }(this, qe) : (Ge || function(ze, We, at, ct) {
                var et;
                return at === null ? et = new ve() : typeof at == "string" || We.objectMode || (et = new he("chunk", ["string", "Buffer"], at)), !et || (Se(ze, et), ne.nextTick(ct, et), !1);
              }(this, Ve, Pe, qe)) && (Ve.pendingcb++, Ze = function(ze, We, at, ct, et, dt) {
                if (!at) {
                  var Je = function(bt, yt, St) {
                    return bt.objectMode || bt.decodeStrings === !1 || typeof yt != "string" || (yt = ce.from(yt, St)), yt;
                  }(We, ct, et);
                  ct !== Je && (at = !0, et = "buffer", ct = Je);
                }
                var it = We.objectMode ? 1 : ct.length;
                We.length += it;
                var st = We.length < We.highWaterMark;
                if (st || (We.needDrain = !0), We.writing || We.corked) {
                  var gt = We.lastBufferedRequest;
                  We.lastBufferedRequest = { chunk: ct, encoding: et, isBuf: at, callback: dt, next: null }, gt ? gt.next = We.lastBufferedRequest : We.bufferedRequest = We.lastBufferedRequest, We.bufferedRequestCount += 1;
                } else Le(ze, We, !1, it, ct, et, dt);
                return st;
              }(this, Ve, Ge, Pe, $e, qe)), Ze;
            }, xe.prototype.cork = function() {
              this._writableState.corked++;
            }, xe.prototype.uncork = function() {
              var Pe = this._writableState;
              Pe.corked && (Pe.corked--, Pe.writing || Pe.corked || Pe.bufferProcessing || !Pe.bufferedRequest || ke(this, Pe));
            }, xe.prototype.setDefaultEncoding = function(Pe) {
              if (typeof Pe == "string" && (Pe = Pe.toLowerCase()), !(["hex", "utf8", "utf-8", "ascii", "binary", "base64", "ucs2", "ucs-2", "utf16le", "utf-16le", "raw"].indexOf((Pe + "").toLowerCase()) > -1)) throw new Ce(Pe);
              return this._writableState.defaultEncoding = Pe, this;
            }, Object.defineProperty(xe.prototype, "writableBuffer", { enumerable: !1, get: function() {
              return this._writableState && this._writableState.getBuffer();
            } }), Object.defineProperty(xe.prototype, "writableHighWaterMark", { enumerable: !1, get: function() {
              return this._writableState.highWaterMark;
            } }), xe.prototype._write = function(Pe, $e, qe) {
              qe(new me("_write()"));
            }, xe.prototype._writev = null, xe.prototype.end = function(Pe, $e, qe) {
              var Xe = this._writableState;
              return typeof Pe == "function" ? (qe = Pe, Pe = null, $e = null) : typeof $e == "function" && (qe = $e, $e = null), Pe != null && this.write(Pe, $e), Xe.corked && (Xe.corked = 1, this.uncork()), Xe.ending || function(Ve, Ze, Ge) {
                Ze.ending = !0, De(Ve, Ze), Ge && (Ze.finished ? ne.nextTick(Ge) : Ve.once("finish", Ge)), Ze.ended = !0, Ve.writable = !1;
              }(this, Xe, qe), this;
            }, Object.defineProperty(xe.prototype, "writableLength", { enumerable: !1, get: function() {
              return this._writableState.length;
            } }), Object.defineProperty(xe.prototype, "destroyed", { enumerable: !1, get: function() {
              return this._writableState !== void 0 && this._writableState.destroyed;
            }, set: function(Pe) {
              this._writableState && (this._writableState.destroyed = Pe);
            } }), xe.prototype.destroy = ue.destroy, xe.prototype._undestroy = ue.undestroy, xe.prototype._destroy = function(Pe, $e) {
              $e(Pe);
            };
          }, 73726: function(G, J, Z) {
            var te, ne = Z(33282);
            function ee(ve, be, Ce) {
              return be in ve ? Object.defineProperty(ve, be, { value: Ce, enumerable: !0, configurable: !0, writable: !0 }) : ve[be] = Ce, ve;
            }
            var ae = Z(37165), oe = Symbol("lastResolve"), se = Symbol("lastReject"), ce = Symbol("error"), le = Symbol("ended"), ue = Symbol("lastPromise"), de = Symbol("handlePromise"), fe = Symbol("stream");
            function he(ve, be) {
              return { value: ve, done: be };
            }
            function me(ve) {
              var be = ve[oe];
              if (be !== null) {
                var Ce = ve[fe].read();
                Ce !== null && (ve[ue] = null, ve[oe] = null, ve[se] = null, be(he(Ce, !1)));
              }
            }
            function pe(ve) {
              ne.nextTick(me, ve);
            }
            var ye = Object.getPrototypeOf(function() {
            }), ge = Object.setPrototypeOf((ee(te = { get stream() {
              return this[fe];
            }, next: function() {
              var ve = this, be = this[ce];
              if (be !== null) return Promise.reject(be);
              if (this[le]) return Promise.resolve(he(void 0, !0));
              if (this[fe].destroyed) return new Promise(function(Ee, xe) {
                ne.nextTick(function() {
                  ve[ce] ? xe(ve[ce]) : Ee(he(void 0, !0));
                });
              });
              var Ce, Se = this[ue];
              if (Se) Ce = new Promise(/* @__PURE__ */ function(Ee, xe) {
                return function(Le, Qe) {
                  Ee.then(function() {
                    xe[le] ? Le(he(void 0, !0)) : xe[de](Le, Qe);
                  }, Qe);
                };
              }(Se, this));
              else {
                var we = this[fe].read();
                if (we !== null) return Promise.resolve(he(we, !1));
                Ce = new Promise(this[de]);
              }
              return this[ue] = Ce, Ce;
            } }, Symbol.asyncIterator, function() {
              return this;
            }), ee(te, "return", function() {
              var ve = this;
              return new Promise(function(be, Ce) {
                ve[fe].destroy(null, function(Se) {
                  Se ? Ce(Se) : be(he(void 0, !0));
                });
              });
            }), te), ye);
            G.exports = function(ve) {
              var be, Ce = Object.create(ge, (ee(be = {}, fe, { value: ve, writable: !0 }), ee(be, oe, { value: null, writable: !0 }), ee(be, se, { value: null, writable: !0 }), ee(be, ce, { value: null, writable: !0 }), ee(be, le, { value: ve._readableState.endEmitted, writable: !0 }), ee(be, de, { value: function(Se, we) {
                var Ee = Ce[fe].read();
                Ee ? (Ce[ue] = null, Ce[oe] = null, Ce[se] = null, Se(he(Ee, !1))) : (Ce[oe] = Se, Ce[se] = we);
              }, writable: !0 }), be));
              return Ce[ue] = null, ae(ve, function(Se) {
                if (Se && Se.code !== "ERR_STREAM_PREMATURE_CLOSE") {
                  var we = Ce[se];
                  return we !== null && (Ce[ue] = null, Ce[oe] = null, Ce[se] = null, we(Se)), void (Ce[ce] = Se);
                }
                var Ee = Ce[oe];
                Ee !== null && (Ce[ue] = null, Ce[oe] = null, Ce[se] = null, Ee(he(void 0, !0))), Ce[le] = !0;
              }), ve.on("readable", pe.bind(null, Ce)), Ce;
            };
          }, 29930: function(G, J, Z) {
            function te(ce, le) {
              var ue = Object.keys(ce);
              if (Object.getOwnPropertySymbols) {
                var de = Object.getOwnPropertySymbols(ce);
                le && (de = de.filter(function(fe) {
                  return Object.getOwnPropertyDescriptor(ce, fe).enumerable;
                })), ue.push.apply(ue, de);
              }
              return ue;
            }
            function ne(ce, le, ue) {
              return le in ce ? Object.defineProperty(ce, le, { value: ue, enumerable: !0, configurable: !0, writable: !0 }) : ce[le] = ue, ce;
            }
            function ee(ce, le) {
              for (var ue = 0; ue < le.length; ue++) {
                var de = le[ue];
                de.enumerable = de.enumerable || !1, de.configurable = !0, "value" in de && (de.writable = !0), Object.defineProperty(ce, de.key, de);
              }
            }
            var ae = Z(45708).Buffer, oe = Z(63779).inspect, se = oe && oe.custom || "inspect";
            G.exports = function() {
              function ce() {
                (function(de, fe) {
                  if (!(de instanceof fe)) throw new TypeError("Cannot call a class as a function");
                })(this, ce), this.head = null, this.tail = null, this.length = 0;
              }
              var le, ue;
              return le = ce, ue = [{ key: "push", value: function(de) {
                var fe = { data: de, next: null };
                this.length > 0 ? this.tail.next = fe : this.head = fe, this.tail = fe, ++this.length;
              } }, { key: "unshift", value: function(de) {
                var fe = { data: de, next: this.head };
                this.length === 0 && (this.tail = fe), this.head = fe, ++this.length;
              } }, { key: "shift", value: function() {
                if (this.length !== 0) {
                  var de = this.head.data;
                  return this.length === 1 ? this.head = this.tail = null : this.head = this.head.next, --this.length, de;
                }
              } }, { key: "clear", value: function() {
                this.head = this.tail = null, this.length = 0;
              } }, { key: "join", value: function(de) {
                if (this.length === 0) return "";
                for (var fe = this.head, he = "" + fe.data; fe = fe.next; ) he += de + fe.data;
                return he;
              } }, { key: "concat", value: function(de) {
                if (this.length === 0) return ae.alloc(0);
                for (var fe, he, me, pe = ae.allocUnsafe(de >>> 0), ye = this.head, ge = 0; ye; ) fe = ye.data, he = pe, me = ge, ae.prototype.copy.call(fe, he, me), ge += ye.data.length, ye = ye.next;
                return pe;
              } }, { key: "consume", value: function(de, fe) {
                var he;
                return de < this.head.data.length ? (he = this.head.data.slice(0, de), this.head.data = this.head.data.slice(de)) : he = de === this.head.data.length ? this.shift() : fe ? this._getString(de) : this._getBuffer(de), he;
              } }, { key: "first", value: function() {
                return this.head.data;
              } }, { key: "_getString", value: function(de) {
                var fe = this.head, he = 1, me = fe.data;
                for (de -= me.length; fe = fe.next; ) {
                  var pe = fe.data, ye = de > pe.length ? pe.length : de;
                  if (ye === pe.length ? me += pe : me += pe.slice(0, de), (de -= ye) == 0) {
                    ye === pe.length ? (++he, fe.next ? this.head = fe.next : this.head = this.tail = null) : (this.head = fe, fe.data = pe.slice(ye));
                    break;
                  }
                  ++he;
                }
                return this.length -= he, me;
              } }, { key: "_getBuffer", value: function(de) {
                var fe = ae.allocUnsafe(de), he = this.head, me = 1;
                for (he.data.copy(fe), de -= he.data.length; he = he.next; ) {
                  var pe = he.data, ye = de > pe.length ? pe.length : de;
                  if (pe.copy(fe, fe.length - de, 0, ye), (de -= ye) == 0) {
                    ye === pe.length ? (++me, he.next ? this.head = he.next : this.head = this.tail = null) : (this.head = he, he.data = pe.slice(ye));
                    break;
                  }
                  ++me;
                }
                return this.length -= me, fe;
              } }, { key: se, value: function(de, fe) {
                return oe(this, function(he) {
                  for (var me = 1; me < arguments.length; me++) {
                    var pe = arguments[me] != null ? arguments[me] : {};
                    me % 2 ? te(Object(pe), !0).forEach(function(ye) {
                      ne(he, ye, pe[ye]);
                    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(he, Object.getOwnPropertyDescriptors(pe)) : te(Object(pe)).forEach(function(ye) {
                      Object.defineProperty(he, ye, Object.getOwnPropertyDescriptor(pe, ye));
                    });
                  }
                  return he;
                }({}, fe, { depth: 0, customInspect: !1 }));
              } }], ue && ee(le.prototype, ue), ce;
            }();
          }, 52023: function(G, J, Z) {
            var te = Z(33282);
            function ne(oe, se) {
              ae(oe, se), ee(oe);
            }
            function ee(oe) {
              oe._writableState && !oe._writableState.emitClose || oe._readableState && !oe._readableState.emitClose || oe.emit("close");
            }
            function ae(oe, se) {
              oe.emit("error", se);
            }
            G.exports = { destroy: function(oe, se) {
              var ce = this, le = this._readableState && this._readableState.destroyed, ue = this._writableState && this._writableState.destroyed;
              return le || ue ? (se ? se(oe) : oe && (this._writableState ? this._writableState.errorEmitted || (this._writableState.errorEmitted = !0, te.nextTick(ae, this, oe)) : te.nextTick(ae, this, oe)), this) : (this._readableState && (this._readableState.destroyed = !0), this._writableState && (this._writableState.destroyed = !0), this._destroy(oe || null, function(de) {
                !se && de ? ce._writableState ? ce._writableState.errorEmitted ? te.nextTick(ee, ce) : (ce._writableState.errorEmitted = !0, te.nextTick(ne, ce, de)) : te.nextTick(ne, ce, de) : se ? (te.nextTick(ee, ce), se(de)) : te.nextTick(ee, ce);
              }), this);
            }, undestroy: function() {
              this._readableState && (this._readableState.destroyed = !1, this._readableState.reading = !1, this._readableState.ended = !1, this._readableState.endEmitted = !1), this._writableState && (this._writableState.destroyed = !1, this._writableState.ended = !1, this._writableState.ending = !1, this._writableState.finalCalled = !1, this._writableState.prefinished = !1, this._writableState.finished = !1, this._writableState.errorEmitted = !1);
            }, errorOrDestroy: function(oe, se) {
              var ce = oe._readableState, le = oe._writableState;
              ce && ce.autoDestroy || le && le.autoDestroy ? oe.destroy(se) : oe.emit("error", se);
            } };
          }, 37165: function(G, J, Z) {
            var te = Z(44059).F.ERR_STREAM_PREMATURE_CLOSE;
            function ne() {
            }
            G.exports = function ee(ae, oe, se) {
              if (typeof oe == "function") return ee(ae, null, oe);
              oe || (oe = {}), se = /* @__PURE__ */ function(ve) {
                var be = !1;
                return function() {
                  if (!be) {
                    be = !0;
                    for (var Ce = arguments.length, Se = new Array(Ce), we = 0; we < Ce; we++) Se[we] = arguments[we];
                    ve.apply(this, Se);
                  }
                };
              }(se || ne);
              var ce = oe.readable || oe.readable !== !1 && ae.readable, le = oe.writable || oe.writable !== !1 && ae.writable, ue = function() {
                ae.writable || fe();
              }, de = ae._writableState && ae._writableState.finished, fe = function() {
                le = !1, de = !0, ce || se.call(ae);
              }, he = ae._readableState && ae._readableState.endEmitted, me = function() {
                ce = !1, he = !0, le || se.call(ae);
              }, pe = function(ve) {
                se.call(ae, ve);
              }, ye = function() {
                var ve;
                return ce && !he ? (ae._readableState && ae._readableState.ended || (ve = new te()), se.call(ae, ve)) : le && !de ? (ae._writableState && ae._writableState.ended || (ve = new te()), se.call(ae, ve)) : void 0;
              }, ge = function() {
                ae.req.on("finish", fe);
              };
              return function(ve) {
                return ve.setHeader && typeof ve.abort == "function";
              }(ae) ? (ae.on("complete", fe), ae.on("abort", ye), ae.req ? ge() : ae.on("request", ge)) : le && !ae._writableState && (ae.on("end", ue), ae.on("close", ue)), ae.on("end", me), ae.on("finish", fe), oe.error !== !1 && ae.on("error", pe), ae.on("close", ye), function() {
                ae.removeListener("complete", fe), ae.removeListener("abort", ye), ae.removeListener("request", ge), ae.req && ae.req.removeListener("finish", fe), ae.removeListener("end", ue), ae.removeListener("close", ue), ae.removeListener("finish", fe), ae.removeListener("end", me), ae.removeListener("error", pe), ae.removeListener("close", ye);
              };
            };
          }, 37108: function(G) {
            G.exports = function() {
              throw new Error("Readable.from is not available in the browser");
            };
          }, 6772: function(G, J, Z) {
            var te, ne = Z(44059).F, ee = ne.ERR_MISSING_ARGS, ae = ne.ERR_STREAM_DESTROYED;
            function oe(le) {
              if (le) throw le;
            }
            function se(le) {
              le();
            }
            function ce(le, ue) {
              return le.pipe(ue);
            }
            G.exports = function() {
              for (var le = arguments.length, ue = new Array(le), de = 0; de < le; de++) ue[de] = arguments[de];
              var fe, he = function(pe) {
                return pe.length ? typeof pe[pe.length - 1] != "function" ? oe : pe.pop() : oe;
              }(ue);
              if (Array.isArray(ue[0]) && (ue = ue[0]), ue.length < 2) throw new ee("streams");
              var me = ue.map(function(pe, ye) {
                var ge = ye < ue.length - 1;
                return function(ve, be, Ce, Se) {
                  Se = /* @__PURE__ */ function(xe) {
                    var Le = !1;
                    return function() {
                      Le || (Le = !0, xe.apply(void 0, arguments));
                    };
                  }(Se);
                  var we = !1;
                  ve.on("close", function() {
                    we = !0;
                  }), te === void 0 && (te = Z(37165)), te(ve, { readable: be, writable: Ce }, function(xe) {
                    if (xe) return Se(xe);
                    we = !0, Se();
                  });
                  var Ee = !1;
                  return function(xe) {
                    if (!we && !Ee) return Ee = !0, function(Le) {
                      return Le.setHeader && typeof Le.abort == "function";
                    }(ve) ? ve.abort() : typeof ve.destroy == "function" ? ve.destroy() : void Se(xe || new ae("pipe"));
                  };
                }(pe, ge, ye > 0, function(ve) {
                  fe || (fe = ve), ve && me.forEach(se), ge || (me.forEach(se), he(fe));
                });
              });
              return ue.reduce(ce);
            };
          }, 31976: function(G, J, Z) {
            var te = Z(44059).F.ERR_INVALID_OPT_VALUE;
            G.exports = { getHighWaterMark: function(ne, ee, ae, oe) {
              var se = function(ce, le, ue) {
                return ce.highWaterMark != null ? ce.highWaterMark : le ? ce[ue] : null;
              }(ee, oe, ae);
              if (se != null) {
                if (!isFinite(se) || Math.floor(se) !== se || se < 0) throw new te(oe ? ae : "highWaterMark", se);
                return Math.floor(se);
              }
              return ne.objectMode ? 16 : 16384;
            } };
          }, 60032: function(G, J, Z) {
            G.exports = Z(7683).EventEmitter;
          }, 54304: function(G, J, Z) {
            var te = Z(41041).Buffer, ne = te.isEncoding || function(he) {
              switch ((he = "" + he) && he.toLowerCase()) {
                case "hex":
                case "utf8":
                case "utf-8":
                case "ascii":
                case "binary":
                case "base64":
                case "ucs2":
                case "ucs-2":
                case "utf16le":
                case "utf-16le":
                case "raw":
                  return !0;
                default:
                  return !1;
              }
            };
            function ee(he) {
              var me;
              switch (this.encoding = function(pe) {
                var ye = function(ge) {
                  if (!ge) return "utf8";
                  for (var ve; ; ) switch (ge) {
                    case "utf8":
                    case "utf-8":
                      return "utf8";
                    case "ucs2":
                    case "ucs-2":
                    case "utf16le":
                    case "utf-16le":
                      return "utf16le";
                    case "latin1":
                    case "binary":
                      return "latin1";
                    case "base64":
                    case "ascii":
                    case "hex":
                      return ge;
                    default:
                      if (ve) return;
                      ge = ("" + ge).toLowerCase(), ve = !0;
                  }
                }(pe);
                if (typeof ye != "string" && (te.isEncoding === ne || !ne(pe))) throw new Error("Unknown encoding: " + pe);
                return ye || pe;
              }(he), this.encoding) {
                case "utf16le":
                  this.text = se, this.end = ce, me = 4;
                  break;
                case "utf8":
                  this.fillLast = oe, me = 4;
                  break;
                case "base64":
                  this.text = le, this.end = ue, me = 3;
                  break;
                default:
                  return this.write = de, void (this.end = fe);
              }
              this.lastNeed = 0, this.lastTotal = 0, this.lastChar = te.allocUnsafe(me);
            }
            function ae(he) {
              return he <= 127 ? 0 : he >> 5 == 6 ? 2 : he >> 4 == 14 ? 3 : he >> 3 == 30 ? 4 : he >> 6 == 2 ? -1 : -2;
            }
            function oe(he) {
              var me = this.lastTotal - this.lastNeed, pe = function(ye, ge, ve) {
                if ((192 & ge[0]) != 128) return ye.lastNeed = 0, "�";
                if (ye.lastNeed > 1 && ge.length > 1) {
                  if ((192 & ge[1]) != 128) return ye.lastNeed = 1, "�";
                  if (ye.lastNeed > 2 && ge.length > 2 && (192 & ge[2]) != 128) return ye.lastNeed = 2, "�";
                }
              }(this, he);
              return pe !== void 0 ? pe : this.lastNeed <= he.length ? (he.copy(this.lastChar, me, 0, this.lastNeed), this.lastChar.toString(this.encoding, 0, this.lastTotal)) : (he.copy(this.lastChar, me, 0, he.length), void (this.lastNeed -= he.length));
            }
            function se(he, me) {
              if ((he.length - me) % 2 == 0) {
                var pe = he.toString("utf16le", me);
                if (pe) {
                  var ye = pe.charCodeAt(pe.length - 1);
                  if (ye >= 55296 && ye <= 56319) return this.lastNeed = 2, this.lastTotal = 4, this.lastChar[0] = he[he.length - 2], this.lastChar[1] = he[he.length - 1], pe.slice(0, -1);
                }
                return pe;
              }
              return this.lastNeed = 1, this.lastTotal = 2, this.lastChar[0] = he[he.length - 1], he.toString("utf16le", me, he.length - 1);
            }
            function ce(he) {
              var me = he && he.length ? this.write(he) : "";
              if (this.lastNeed) {
                var pe = this.lastTotal - this.lastNeed;
                return me + this.lastChar.toString("utf16le", 0, pe);
              }
              return me;
            }
            function le(he, me) {
              var pe = (he.length - me) % 3;
              return pe === 0 ? he.toString("base64", me) : (this.lastNeed = 3 - pe, this.lastTotal = 3, pe === 1 ? this.lastChar[0] = he[he.length - 1] : (this.lastChar[0] = he[he.length - 2], this.lastChar[1] = he[he.length - 1]), he.toString("base64", me, he.length - pe));
            }
            function ue(he) {
              var me = he && he.length ? this.write(he) : "";
              return this.lastNeed ? me + this.lastChar.toString("base64", 0, 3 - this.lastNeed) : me;
            }
            function de(he) {
              return he.toString(this.encoding);
            }
            function fe(he) {
              return he && he.length ? this.write(he) : "";
            }
            J.I = ee, ee.prototype.write = function(he) {
              if (he.length === 0) return "";
              var me, pe;
              if (this.lastNeed) {
                if ((me = this.fillLast(he)) === void 0) return "";
                pe = this.lastNeed, this.lastNeed = 0;
              } else pe = 0;
              return pe < he.length ? me ? me + this.text(he, pe) : this.text(he, pe) : me || "";
            }, ee.prototype.end = function(he) {
              var me = he && he.length ? this.write(he) : "";
              return this.lastNeed ? me + "�" : me;
            }, ee.prototype.text = function(he, me) {
              var pe = function(ge, ve, be) {
                var Ce = ve.length - 1;
                if (Ce < be) return 0;
                var Se = ae(ve[Ce]);
                return Se >= 0 ? (Se > 0 && (ge.lastNeed = Se - 1), Se) : --Ce < be || Se === -2 ? 0 : (Se = ae(ve[Ce])) >= 0 ? (Se > 0 && (ge.lastNeed = Se - 2), Se) : --Ce < be || Se === -2 ? 0 : (Se = ae(ve[Ce])) >= 0 ? (Se > 0 && (Se === 2 ? Se = 0 : ge.lastNeed = Se - 3), Se) : 0;
              }(this, he, me);
              if (!this.lastNeed) return he.toString("utf8", me);
              this.lastTotal = pe;
              var ye = he.length - (pe - this.lastNeed);
              return he.copy(this.lastChar, 0, ye), he.toString("utf8", me, ye);
            }, ee.prototype.fillLast = function(he) {
              if (this.lastNeed <= he.length) return he.copy(this.lastChar, this.lastTotal - this.lastNeed, 0, this.lastNeed), this.lastChar.toString(this.encoding, 0, this.lastTotal);
              he.copy(this.lastChar, this.lastTotal - this.lastNeed, 0, he.length), this.lastNeed -= he.length;
            };
          }, 79743: function(G, J, Z) {
            var te = Z(45708).Buffer, ne = Z(85672), ee = Z(79399)("stream-parser");
            G.exports = function(ve) {
              var be = ve && typeof ve._transform == "function", Ce = ve && typeof ve._write == "function";
              if (!be && !Ce) throw new Error("must pass a Writable or Transform stream in");
              ee("extending Parser into stream"), ve._bytes = ue, ve._skipBytes = de, be && (ve._passthrough = fe), be ? ve._transform = me : ve._write = he;
            };
            var ae = -1, oe = 0, se = 1, ce = 2;
            function le(ve) {
              ee("initializing parser stream"), ve._parserBytesLeft = 0, ve._parserBuffers = [], ve._parserBuffered = 0, ve._parserState = ae, ve._parserCallback = null, typeof ve.push == "function" && (ve._parserOutput = ve.push.bind(ve)), ve._parserInit = !0;
            }
            function ue(ve, be) {
              ne(!this._parserCallback, 'there is already a "callback" set!'), ne(isFinite(ve) && ve > 0, 'can only buffer a finite number of bytes > 0, got "' + ve + '"'), this._parserInit || le(this), ee("buffering %o bytes", ve), this._parserBytesLeft = ve, this._parserCallback = be, this._parserState = oe;
            }
            function de(ve, be) {
              ne(!this._parserCallback, 'there is already a "callback" set!'), ne(ve > 0, 'can only skip > 0 bytes, got "' + ve + '"'), this._parserInit || le(this), ee("skipping %o bytes", ve), this._parserBytesLeft = ve, this._parserCallback = be, this._parserState = se;
            }
            function fe(ve, be) {
              ne(!this._parserCallback, 'There is already a "callback" set!'), ne(ve > 0, 'can only pass through > 0 bytes, got "' + ve + '"'), this._parserInit || le(this), ee("passing through %o bytes", ve), this._parserBytesLeft = ve, this._parserCallback = be, this._parserState = ce;
            }
            function he(ve, be, Ce) {
              this._parserInit || le(this), ee("write(%o bytes)", ve.length), typeof be == "function" && (Ce = be), ye(this, ve, null, Ce);
            }
            function me(ve, be, Ce) {
              this._parserInit || le(this), ee("transform(%o bytes)", ve.length), typeof be != "function" && (be = this._parserOutput), ye(this, ve, be, Ce);
            }
            function pe(ve, be, Ce, Se) {
              if (ve._parserBytesLeft -= be.length, ee("%o bytes left for stream piece", ve._parserBytesLeft), ve._parserState === oe ? (ve._parserBuffers.push(be), ve._parserBuffered += be.length) : ve._parserState === ce && Ce(be), ve._parserBytesLeft !== 0) return Se;
              var we = ve._parserCallback;
              if (we && ve._parserState === oe && ve._parserBuffers.length > 1 && (be = te.concat(ve._parserBuffers, ve._parserBuffered)), ve._parserState !== oe && (be = null), ve._parserCallback = null, ve._parserBuffered = 0, ve._parserState = ae, ve._parserBuffers.splice(0), we) {
                var Ee = [];
                be && Ee.push(be), Ce && Ee.push(Ce);
                var xe = we.length > Ee.length;
                xe && Ee.push(ge(Se));
                var Le = we.apply(ve, Ee);
                if (!xe || Se === Le) return Se;
              }
            }
            var ye = ge(function ve(be, Ce, Se, we) {
              return be._parserBytesLeft <= 0 ? we(new Error("got data but not currently parsing anything")) : Ce.length <= be._parserBytesLeft ? function() {
                return pe(be, Ce, Se, we);
              } : function() {
                var Ee = Ce.slice(0, be._parserBytesLeft);
                return pe(be, Ee, Se, function(xe) {
                  return xe ? we(xe) : Ce.length > Ee.length ? function() {
                    return ve(be, Ce.slice(Ee.length), Se, we);
                  } : void 0;
                });
              };
            });
            function ge(ve) {
              return function() {
                for (var be = ve.apply(this, arguments); typeof be == "function"; ) be = be();
                return be;
              };
            }
          }, 79399: function(G, J, Z) {
            var te = Z(33282);
            function ne() {
              var ee;
              try {
                ee = J.storage.debug;
              } catch {
              }
              return !ee && te !== void 0 && "env" in te && (ee = te.env.DEBUG), ee;
            }
            (J = G.exports = Z(43228)).log = function() {
              return typeof console == "object" && console.log && Function.prototype.apply.call(console.log, console, arguments);
            }, J.formatArgs = function(ee) {
              var ae = this.useColors;
              if (ee[0] = (ae ? "%c" : "") + this.namespace + (ae ? " %c" : " ") + ee[0] + (ae ? "%c " : " ") + "+" + J.humanize(this.diff), ae) {
                var oe = "color: " + this.color;
                ee.splice(1, 0, oe, "color: inherit");
                var se = 0, ce = 0;
                ee[0].replace(/%[a-zA-Z%]/g, function(le) {
                  le !== "%%" && (se++, le === "%c" && (ce = se));
                }), ee.splice(ce, 0, oe);
              }
            }, J.save = function(ee) {
              try {
                ee == null ? J.storage.removeItem("debug") : J.storage.debug = ee;
              } catch {
              }
            }, J.load = ne, J.useColors = function() {
              return !(typeof window > "u" || !window.process || window.process.type !== "renderer") || typeof document < "u" && document.documentElement && document.documentElement.style && document.documentElement.style.WebkitAppearance || typeof window < "u" && window.console && (window.console.firebug || window.console.exception && window.console.table) || typeof navigator < "u" && navigator.userAgent && navigator.userAgent.toLowerCase().match(/firefox\/(\d+)/) && parseInt(RegExp.$1, 10) >= 31 || typeof navigator < "u" && navigator.userAgent && navigator.userAgent.toLowerCase().match(/applewebkit\/(\d+)/);
            }, J.storage = typeof chrome < "u" && chrome.storage !== void 0 ? chrome.storage.local : function() {
              try {
                return window.localStorage;
              } catch {
              }
            }(), J.colors = ["lightseagreen", "forestgreen", "goldenrod", "dodgerblue", "darkorchid", "crimson"], J.formatters.j = function(ee) {
              try {
                return JSON.stringify(ee);
              } catch (ae) {
                return "[UnexpectedJSONParseError]: " + ae.message;
              }
            }, J.enable(ne());
          }, 43228: function(G, J, Z) {
            var te;
            function ne(ee) {
              function ae() {
                if (ae.enabled) {
                  var oe = ae, se = +/* @__PURE__ */ new Date(), ce = se - (te || se);
                  oe.diff = ce, oe.prev = te, oe.curr = se, te = se;
                  for (var le = new Array(arguments.length), ue = 0; ue < le.length; ue++) le[ue] = arguments[ue];
                  le[0] = J.coerce(le[0]), typeof le[0] != "string" && le.unshift("%O");
                  var de = 0;
                  le[0] = le[0].replace(/%([a-zA-Z%])/g, function(fe, he) {
                    if (fe === "%%") return fe;
                    de++;
                    var me = J.formatters[he];
                    if (typeof me == "function") {
                      var pe = le[de];
                      fe = me.call(oe, pe), le.splice(de, 1), de--;
                    }
                    return fe;
                  }), J.formatArgs.call(oe, le), (ae.log || J.log || console.log.bind(console)).apply(oe, le);
                }
              }
              return ae.namespace = ee, ae.enabled = J.enabled(ee), ae.useColors = J.useColors(), ae.color = function(oe) {
                var se, ce = 0;
                for (se in oe) ce = (ce << 5) - ce + oe.charCodeAt(se), ce |= 0;
                return J.colors[Math.abs(ce) % J.colors.length];
              }(ee), typeof J.init == "function" && J.init(ae), ae;
            }
            (J = G.exports = ne.debug = ne.default = ne).coerce = function(ee) {
              return ee instanceof Error ? ee.stack || ee.message : ee;
            }, J.disable = function() {
              J.enable("");
            }, J.enable = function(ee) {
              J.save(ee), J.names = [], J.skips = [];
              for (var ae = (typeof ee == "string" ? ee : "").split(/[\s,]+/), oe = ae.length, se = 0; se < oe; se++) ae[se] && ((ee = ae[se].replace(/\*/g, ".*?"))[0] === "-" ? J.skips.push(new RegExp("^" + ee.substr(1) + "$")) : J.names.push(new RegExp("^" + ee + "$")));
            }, J.enabled = function(ee) {
              var ae, oe;
              for (ae = 0, oe = J.skips.length; ae < oe; ae++) if (J.skips[ae].test(ee)) return !1;
              for (ae = 0, oe = J.names.length; ae < oe; ae++) if (J.names[ae].test(ee)) return !0;
              return !1;
            }, J.humanize = Z(13883), J.names = [], J.skips = [], J.formatters = {};
          }, 13883: function(G) {
            var J = 1e3, Z = 60 * J, te = 60 * Z, ne = 24 * te;
            function ee(ae, oe, se) {
              if (!(ae < oe)) return ae < 1.5 * oe ? Math.floor(ae / oe) + " " + se : Math.ceil(ae / oe) + " " + se + "s";
            }
            G.exports = function(ae, oe) {
              oe = oe || {};
              var se, ce = typeof ae;
              if (ce === "string" && ae.length > 0) return function(le) {
                if (!((le = String(le)).length > 100)) {
                  var ue = /^((?:\d+)?\.?\d+) *(milliseconds?|msecs?|ms|seconds?|secs?|s|minutes?|mins?|m|hours?|hrs?|h|days?|d|years?|yrs?|y)?$/i.exec(le);
                  if (ue) {
                    var de = parseFloat(ue[1]);
                    switch ((ue[2] || "ms").toLowerCase()) {
                      case "years":
                      case "year":
                      case "yrs":
                      case "yr":
                      case "y":
                        return 315576e5 * de;
                      case "days":
                      case "day":
                      case "d":
                        return de * ne;
                      case "hours":
                      case "hour":
                      case "hrs":
                      case "hr":
                      case "h":
                        return de * te;
                      case "minutes":
                      case "minute":
                      case "mins":
                      case "min":
                      case "m":
                        return de * Z;
                      case "seconds":
                      case "second":
                      case "secs":
                      case "sec":
                      case "s":
                        return de * J;
                      case "milliseconds":
                      case "millisecond":
                      case "msecs":
                      case "msec":
                      case "ms":
                        return de;
                      default:
                        return;
                    }
                  }
                }
              }(ae);
              if (ce === "number" && isNaN(ae) === !1) return oe.long ? ee(se = ae, ne, "day") || ee(se, te, "hour") || ee(se, Z, "minute") || ee(se, J, "second") || se + " ms" : function(le) {
                return le >= ne ? Math.round(le / ne) + "d" : le >= te ? Math.round(le / te) + "h" : le >= Z ? Math.round(le / Z) + "m" : le >= J ? Math.round(le / J) + "s" : le + "ms";
              }(ae);
              throw new Error("val is not a non-empty string or a valid number. val=" + JSON.stringify(ae));
            };
          }, 28089: function(G, J, Z) {
            var te = Z(59811);
            G.exports = function(ne, ee, ae) {
              if (ne == null) throw Error("First argument should be a string");
              if (ee == null) throw Error("Separator should be a string or a RegExp");
              ae ? (typeof ae == "string" || Array.isArray(ae)) && (ae = { ignore: ae }) : ae = {}, ae.escape == null && (ae.escape = !0), ae.ignore == null ? ae.ignore = ["[]", "()", "{}", "<>", '""', "''", "``", "“”", "«»"] : (typeof ae.ignore == "string" && (ae.ignore = [ae.ignore]), ae.ignore = ae.ignore.map(function(fe) {
                return fe.length === 1 && (fe += fe), fe;
              }));
              var oe = te.parse(ne, { flat: !0, brackets: ae.ignore }), se = oe[0].split(ee);
              if (ae.escape) {
                for (var ce = [], le = 0; le < se.length; le++) {
                  var ue = se[le], de = se[le + 1];
                  ue[ue.length - 1] === "\\" && ue[ue.length - 2] !== "\\" ? (ce.push(ue + ee + de), le++) : ce.push(ue);
                }
                se = ce;
              }
              for (le = 0; le < se.length; le++) oe[0] = se[le], se[le] = te.stringify(oe, { flat: !0 });
              return se;
            };
          }, 26381: function(G) {
            G.exports = function(J) {
              for (var Z = J.length, te = new Array(Z), ne = new Array(Z), ee = new Array(Z), ae = new Array(Z), oe = new Array(Z), se = new Array(Z), ce = 0; ce < Z; ++ce) te[ce] = -1, ne[ce] = 0, ee[ce] = !1, ae[ce] = 0, oe[ce] = -1, se[ce] = [];
              var le, ue = 0, de = [], fe = [];
              function he(ye) {
                var ge = [ye], ve = [ye];
                for (te[ye] = ne[ye] = ue, ee[ye] = !0, ue += 1; ve.length > 0; ) {
                  ye = ve[ve.length - 1];
                  var be = J[ye];
                  if (ae[ye] < be.length) {
                    for (var Ce = ae[ye]; Ce < be.length; ++Ce) {
                      var Se = be[Ce];
                      if (te[Se] < 0) {
                        te[Se] = ne[Se] = ue, ee[Se] = !0, ue += 1, ge.push(Se), ve.push(Se);
                        break;
                      }
                      ee[Se] && (ne[ye] = 0 | Math.min(ne[ye], ne[Se])), oe[Se] >= 0 && se[ye].push(oe[Se]);
                    }
                    ae[ye] = Ce;
                  } else {
                    if (ne[ye] === te[ye]) {
                      var we = [], Ee = [], xe = 0;
                      for (Ce = ge.length - 1; Ce >= 0; --Ce) {
                        var Le = ge[Ce];
                        if (ee[Le] = !1, we.push(Le), Ee.push(se[Le]), xe += se[Le].length, oe[Le] = de.length, Le === ye) {
                          ge.length = Ce;
                          break;
                        }
                      }
                      de.push(we);
                      var Qe = new Array(xe);
                      for (Ce = 0; Ce < Ee.length; Ce++) for (var ke = 0; ke < Ee[Ce].length; ke++) Qe[--xe] = Ee[Ce][ke];
                      fe.push(Qe);
                    }
                    ve.pop();
                  }
                }
              }
              for (ce = 0; ce < Z; ++ce) te[ce] < 0 && he(ce);
              for (ce = 0; ce < fe.length; ce++) {
                var me = fe[ce];
                if (me.length !== 0) {
                  me.sort(function(ye, ge) {
                    return ye - ge;
                  }), le = [me[0]];
                  for (var pe = 1; pe < me.length; pe++) me[pe] !== me[pe - 1] && le.push(me[pe]);
                  fe[ce] = le;
                }
              }
              return { components: de, adjacencyList: fe };
            };
          }, 13193: function(G, J, Z) {
            Z.r(J);
            var te = 2 * Math.PI, ne = function(oe, se, ce, le, ue, de, fe) {
              var he = oe.x, me = oe.y;
              return { x: le * (he *= se) - ue * (me *= ce) + de, y: ue * he + le * me + fe };
            }, ee = function(oe, se) {
              var ce = se === 1.5707963267948966 ? 0.551915024494 : se === -1.5707963267948966 ? -0.551915024494 : 1.3333333333333333 * Math.tan(se / 4), le = Math.cos(oe), ue = Math.sin(oe), de = Math.cos(oe + se), fe = Math.sin(oe + se);
              return [{ x: le - ue * ce, y: ue + le * ce }, { x: de + fe * ce, y: fe - de * ce }, { x: de, y: fe }];
            }, ae = function(oe, se, ce, le) {
              var ue = oe * ce + se * le;
              return ue > 1 && (ue = 1), ue < -1 && (ue = -1), (oe * le - se * ce < 0 ? -1 : 1) * Math.acos(ue);
            };
            J.default = function(oe) {
              var se = oe.px, ce = oe.py, le = oe.cx, ue = oe.cy, de = oe.rx, fe = oe.ry, he = oe.xAxisRotation, me = he === void 0 ? 0 : he, pe = oe.largeArcFlag, ye = pe === void 0 ? 0 : pe, ge = oe.sweepFlag, ve = ge === void 0 ? 0 : ge, be = [];
              if (de === 0 || fe === 0) return [];
              var Ce = Math.sin(me * te / 360), Se = Math.cos(me * te / 360), we = Se * (se - le) / 2 + Ce * (ce - ue) / 2, Ee = -Ce * (se - le) / 2 + Se * (ce - ue) / 2;
              if (we === 0 && Ee === 0) return [];
              de = Math.abs(de), fe = Math.abs(fe);
              var xe = Math.pow(we, 2) / Math.pow(de, 2) + Math.pow(Ee, 2) / Math.pow(fe, 2);
              xe > 1 && (de *= Math.sqrt(xe), fe *= Math.sqrt(xe));
              var Le = function(Xe, Ve, Ze, Ge, ze, We, at, ct, et, dt, Je, it) {
                var st = Math.pow(ze, 2), gt = Math.pow(We, 2), bt = Math.pow(Je, 2), yt = Math.pow(it, 2), St = st * gt - st * yt - gt * bt;
                St < 0 && (St = 0), St /= st * yt + gt * bt;
                var Lt = (St = Math.sqrt(St) * (at === ct ? -1 : 1)) * ze / We * it, Rt = St * -We / ze * Je, vt = dt * Lt - et * Rt + (Xe + Ze) / 2, $t = et * Lt + dt * Rt + (Ve + Ge) / 2, Pt = (Je - Lt) / ze, Xt = (it - Rt) / We, Nt = (-Je - Lt) / ze, Gt = (-it - Rt) / We, Ft = ae(1, 0, Pt, Xt), Tt = ae(Pt, Xt, Nt, Gt);
                return ct === 0 && Tt > 0 && (Tt -= te), ct === 1 && Tt < 0 && (Tt += te), [vt, $t, Ft, Tt];
              }(se, ce, le, ue, de, fe, ye, ve, Ce, Se, we, Ee), Qe = function(Xe, Ve) {
                if (Array.isArray(Xe)) return Xe;
                if (Symbol.iterator in Object(Xe)) return function(Ze, Ge) {
                  var ze = [], We = !0, at = !1, ct = void 0;
                  try {
                    for (var et, dt = Ze[Symbol.iterator](); !(We = (et = dt.next()).done) && (ze.push(et.value), !Ge || ze.length !== Ge); We = !0) ;
                  } catch (Je) {
                    at = !0, ct = Je;
                  } finally {
                    try {
                      !We && dt.return && dt.return();
                    } finally {
                      if (at) throw ct;
                    }
                  }
                  return ze;
                }(Xe, Ve);
                throw new TypeError("Invalid attempt to destructure non-iterable instance");
              }(Le, 4), ke = Qe[0], Me = Qe[1], Re = Qe[2], De = Qe[3], Pe = Math.abs(De) / (te / 4);
              Math.abs(1 - Pe) < 1e-7 && (Pe = 1);
              var $e = Math.max(Math.ceil(Pe), 1);
              De /= $e;
              for (var qe = 0; qe < $e; qe++) be.push(ee(Re, De)), Re += De;
              return be.map(function(Xe) {
                var Ve = ne(Xe[0], de, fe, Se, Ce, ke, Me), Ze = Ve.x, Ge = Ve.y, ze = ne(Xe[1], de, fe, Se, Ce, ke, Me), We = ze.x, at = ze.y, ct = ne(Xe[2], de, fe, Se, Ce, ke, Me);
                return { x1: Ze, y1: Ge, x2: We, y2: at, x: ct.x, y: ct.y };
              });
            };
          }, 97251: function(G, J, Z) {
            var te = Z(26953), ne = Z(16844), ee = Z(41883), ae = Z(13986), oe = Z(85672);
            G.exports = function(se) {
              if (Array.isArray(se) && se.length === 1 && typeof se[0] == "string" && (se = se[0]), typeof se == "string" && (oe(ae(se), "String is not an SVG path."), se = te(se)), oe(Array.isArray(se), "Argument should be a string or an array of path segments."), se = ne(se), !(se = ee(se)).length) return [0, 0, 0, 0];
              for (var ce = [1 / 0, 1 / 0, -1 / 0, -1 / 0], le = 0, ue = se.length; le < ue; le++) for (var de = se[le].slice(1), fe = 0; fe < de.length; fe += 2) de[fe + 0] < ce[0] && (ce[0] = de[fe + 0]), de[fe + 1] < ce[1] && (ce[1] = de[fe + 1]), de[fe + 0] > ce[2] && (ce[2] = de[fe + 0]), de[fe + 1] > ce[3] && (ce[3] = de[fe + 1]);
              return ce;
            };
          }, 41883: function(G, J, Z) {
            G.exports = function(ae) {
              for (var oe, se = [], ce = 0, le = 0, ue = 0, de = 0, fe = null, he = null, me = 0, pe = 0, ye = 0, ge = ae.length; ye < ge; ye++) {
                var ve = ae[ye], be = ve[0];
                switch (be) {
                  case "M":
                    ue = ve[1], de = ve[2];
                    break;
                  case "A":
                    var Ce = te({ px: me, py: pe, cx: ve[6], cy: ve[7], rx: ve[1], ry: ve[2], xAxisRotation: ve[3], largeArcFlag: ve[4], sweepFlag: ve[5] });
                    if (!Ce.length) continue;
                    for (var Se, we = 0; we < Ce.length; we++) ve = ["C", (Se = Ce[we]).x1, Se.y1, Se.x2, Se.y2, Se.x, Se.y], we < Ce.length - 1 && se.push(ve);
                    break;
                  case "S":
                    var Ee = me, xe = pe;
                    oe != "C" && oe != "S" || (Ee += Ee - ce, xe += xe - le), ve = ["C", Ee, xe, ve[1], ve[2], ve[3], ve[4]];
                    break;
                  case "T":
                    oe == "Q" || oe == "T" ? (fe = 2 * me - fe, he = 2 * pe - he) : (fe = me, he = pe), ve = ee(me, pe, fe, he, ve[1], ve[2]);
                    break;
                  case "Q":
                    fe = ve[1], he = ve[2], ve = ee(me, pe, ve[1], ve[2], ve[3], ve[4]);
                    break;
                  case "L":
                    ve = ne(me, pe, ve[1], ve[2]);
                    break;
                  case "H":
                    ve = ne(me, pe, ve[1], pe);
                    break;
                  case "V":
                    ve = ne(me, pe, me, ve[1]);
                    break;
                  case "Z":
                    ve = ne(me, pe, ue, de);
                }
                oe = be, me = ve[ve.length - 2], pe = ve[ve.length - 1], ve.length > 4 ? (ce = ve[ve.length - 4], le = ve[ve.length - 3]) : (ce = me, le = pe), se.push(ve);
              }
              return se;
            };
            var te = Z(13193);
            function ne(ae, oe, se, ce) {
              return ["C", ae, oe, se, ce, se, ce];
            }
            function ee(ae, oe, se, ce, le, ue) {
              return ["C", ae / 3 + 0.6666666666666666 * se, oe / 3 + 0.6666666666666666 * ce, le / 3 + 0.6666666666666666 * se, ue / 3 + 0.6666666666666666 * ce, le, ue];
            }
          }, 96021: function(G, J, Z) {
            var te, ne = Z(97251), ee = Z(26953), ae = Z(95620), oe = Z(13986), se = Z(88772), ce = document.createElement("canvas"), le = ce.getContext("2d");
            G.exports = function(ue, de) {
              if (!oe(ue)) throw Error("Argument should be valid svg path string");
              var fe, he;
              de || (de = {}), de.shape ? (fe = de.shape[0], he = de.shape[1]) : (fe = ce.width = de.w || de.width || 200, he = ce.height = de.h || de.height || 200);
              var me = Math.min(fe, he), pe = de.stroke || 0, ye = de.viewbox || de.viewBox || ne(ue), ge = [fe / (ye[2] - ye[0]), he / (ye[3] - ye[1])], ve = Math.min(ge[0] || 0, ge[1] || 0) / 2;
              if (le.fillStyle = "black", le.fillRect(0, 0, fe, he), le.fillStyle = "white", pe && (typeof pe != "number" && (pe = 1), le.strokeStyle = pe > 0 ? "white" : "black", le.lineWidth = Math.abs(pe)), le.translate(0.5 * fe, 0.5 * he), le.scale(ve, ve), function() {
                if (te != null) return te;
                var Se = document.createElement("canvas").getContext("2d");
                if (Se.canvas.width = Se.canvas.height = 1, !window.Path2D) return te = !1;
                var we = new Path2D("M0,0h1v1h-1v-1Z");
                Se.fillStyle = "black", Se.fill(we);
                var Ee = Se.getImageData(0, 0, 1, 1);
                return te = Ee && Ee.data && Ee.data[3] === 255;
              }()) {
                var be = new Path2D(ue);
                le.fill(be), pe && le.stroke(be);
              } else {
                var Ce = ee(ue);
                ae(le, Ce), le.fill(), pe && le.stroke();
              }
              return le.setTransform(1, 0, 0, 1, 0, 0), se(le, { cutoff: de.cutoff != null ? de.cutoff : 0.5, radius: de.radius != null ? de.radius : 0.5 * me });
            };
          }, 65657: function(G, J, Z) {
            var te;
            (function(ne) {
              var ee = /^\s+/, ae = /\s+$/, oe = 0, se = ne.round, ce = ne.min, le = ne.max, ue = ne.random;
              function de(Je, it) {
                if (it = it || {}, (Je = Je || "") instanceof de) return Je;
                if (!(this instanceof de)) return new de(Je, it);
                var st = function(gt) {
                  var bt, yt, St, Lt = { r: 0, g: 0, b: 0 }, Rt = 1, vt = null, $t = null, Pt = null, Xt = !1, Nt = !1;
                  return typeof gt == "string" && (gt = function(Gt) {
                    Gt = Gt.replace(ee, "").replace(ae, "").toLowerCase();
                    var Ft, Tt = !1;
                    if (Re[Gt]) Gt = Re[Gt], Tt = !0;
                    else if (Gt == "transparent") return { r: 0, g: 0, b: 0, a: 0, format: "name" };
                    return (Ft = et.rgb.exec(Gt)) ? { r: Ft[1], g: Ft[2], b: Ft[3] } : (Ft = et.rgba.exec(Gt)) ? { r: Ft[1], g: Ft[2], b: Ft[3], a: Ft[4] } : (Ft = et.hsl.exec(Gt)) ? { h: Ft[1], s: Ft[2], l: Ft[3] } : (Ft = et.hsla.exec(Gt)) ? { h: Ft[1], s: Ft[2], l: Ft[3], a: Ft[4] } : (Ft = et.hsv.exec(Gt)) ? { h: Ft[1], s: Ft[2], v: Ft[3] } : (Ft = et.hsva.exec(Gt)) ? { h: Ft[1], s: Ft[2], v: Ft[3], a: Ft[4] } : (Ft = et.hex8.exec(Gt)) ? { r: Xe(Ft[1]), g: Xe(Ft[2]), b: Xe(Ft[3]), a: ze(Ft[4]), format: Tt ? "name" : "hex8" } : (Ft = et.hex6.exec(Gt)) ? { r: Xe(Ft[1]), g: Xe(Ft[2]), b: Xe(Ft[3]), format: Tt ? "name" : "hex" } : (Ft = et.hex4.exec(Gt)) ? { r: Xe(Ft[1] + "" + Ft[1]), g: Xe(Ft[2] + "" + Ft[2]), b: Xe(Ft[3] + "" + Ft[3]), a: ze(Ft[4] + "" + Ft[4]), format: Tt ? "name" : "hex8" } : !!(Ft = et.hex3.exec(Gt)) && { r: Xe(Ft[1] + "" + Ft[1]), g: Xe(Ft[2] + "" + Ft[2]), b: Xe(Ft[3] + "" + Ft[3]), format: Tt ? "name" : "hex" };
                  }(gt)), typeof gt == "object" && (dt(gt.r) && dt(gt.g) && dt(gt.b) ? (bt = gt.r, yt = gt.g, St = gt.b, Lt = { r: 255 * $e(bt, 255), g: 255 * $e(yt, 255), b: 255 * $e(St, 255) }, Xt = !0, Nt = String(gt.r).substr(-1) === "%" ? "prgb" : "rgb") : dt(gt.h) && dt(gt.s) && dt(gt.v) ? (vt = Ze(gt.s), $t = Ze(gt.v), Lt = function(Gt, Ft, Tt) {
                    Gt = 6 * $e(Gt, 360), Ft = $e(Ft, 100), Tt = $e(Tt, 100);
                    var It = ne.floor(Gt), Ct = Gt - It, Kt = Tt * (1 - Ft), Vt = Tt * (1 - Ct * Ft), Yt = Tt * (1 - (1 - Ct) * Ft), sn = It % 6;
                    return { r: 255 * [Tt, Vt, Kt, Kt, Yt, Tt][sn], g: 255 * [Yt, Tt, Tt, Vt, Kt, Kt][sn], b: 255 * [Kt, Kt, Yt, Tt, Tt, Vt][sn] };
                  }(gt.h, vt, $t), Xt = !0, Nt = "hsv") : dt(gt.h) && dt(gt.s) && dt(gt.l) && (vt = Ze(gt.s), Pt = Ze(gt.l), Lt = function(Gt, Ft, Tt) {
                    var It, Ct, Kt;
                    function Vt(hn, Qt, Ut) {
                      return Ut < 0 && (Ut += 1), Ut > 1 && (Ut -= 1), Ut < 1 / 6 ? hn + 6 * (Qt - hn) * Ut : Ut < 0.5 ? Qt : Ut < 2 / 3 ? hn + (Qt - hn) * (2 / 3 - Ut) * 6 : hn;
                    }
                    if (Gt = $e(Gt, 360), Ft = $e(Ft, 100), Tt = $e(Tt, 100), Ft === 0) It = Ct = Kt = Tt;
                    else {
                      var Yt = Tt < 0.5 ? Tt * (1 + Ft) : Tt + Ft - Tt * Ft, sn = 2 * Tt - Yt;
                      It = Vt(sn, Yt, Gt + 1 / 3), Ct = Vt(sn, Yt, Gt), Kt = Vt(sn, Yt, Gt - 1 / 3);
                    }
                    return { r: 255 * It, g: 255 * Ct, b: 255 * Kt };
                  }(gt.h, vt, Pt), Xt = !0, Nt = "hsl"), gt.hasOwnProperty("a") && (Rt = gt.a)), Rt = Pe(Rt), { ok: Xt, format: gt.format || Nt, r: ce(255, le(Lt.r, 0)), g: ce(255, le(Lt.g, 0)), b: ce(255, le(Lt.b, 0)), a: Rt };
                }(Je);
                this._originalInput = Je, this._r = st.r, this._g = st.g, this._b = st.b, this._a = st.a, this._roundA = se(100 * this._a) / 100, this._format = it.format || st.format, this._gradientType = it.gradientType, this._r < 1 && (this._r = se(this._r)), this._g < 1 && (this._g = se(this._g)), this._b < 1 && (this._b = se(this._b)), this._ok = st.ok, this._tc_id = oe++;
              }
              function fe(Je, it, st) {
                Je = $e(Je, 255), it = $e(it, 255), st = $e(st, 255);
                var gt, bt, yt = le(Je, it, st), St = ce(Je, it, st), Lt = (yt + St) / 2;
                if (yt == St) gt = bt = 0;
                else {
                  var Rt = yt - St;
                  switch (bt = Lt > 0.5 ? Rt / (2 - yt - St) : Rt / (yt + St), yt) {
                    case Je:
                      gt = (it - st) / Rt + (it < st ? 6 : 0);
                      break;
                    case it:
                      gt = (st - Je) / Rt + 2;
                      break;
                    case st:
                      gt = (Je - it) / Rt + 4;
                  }
                  gt /= 6;
                }
                return { h: gt, s: bt, l: Lt };
              }
              function he(Je, it, st) {
                Je = $e(Je, 255), it = $e(it, 255), st = $e(st, 255);
                var gt, bt, yt = le(Je, it, st), St = ce(Je, it, st), Lt = yt, Rt = yt - St;
                if (bt = yt === 0 ? 0 : Rt / yt, yt == St) gt = 0;
                else {
                  switch (yt) {
                    case Je:
                      gt = (it - st) / Rt + (it < st ? 6 : 0);
                      break;
                    case it:
                      gt = (st - Je) / Rt + 2;
                      break;
                    case st:
                      gt = (Je - it) / Rt + 4;
                  }
                  gt /= 6;
                }
                return { h: gt, s: bt, v: Lt };
              }
              function me(Je, it, st, gt) {
                var bt = [Ve(se(Je).toString(16)), Ve(se(it).toString(16)), Ve(se(st).toString(16))];
                return gt && bt[0].charAt(0) == bt[0].charAt(1) && bt[1].charAt(0) == bt[1].charAt(1) && bt[2].charAt(0) == bt[2].charAt(1) ? bt[0].charAt(0) + bt[1].charAt(0) + bt[2].charAt(0) : bt.join("");
              }
              function pe(Je, it, st, gt) {
                return [Ve(Ge(gt)), Ve(se(Je).toString(16)), Ve(se(it).toString(16)), Ve(se(st).toString(16))].join("");
              }
              function ye(Je, it) {
                it = it === 0 ? 0 : it || 10;
                var st = de(Je).toHsl();
                return st.s -= it / 100, st.s = qe(st.s), de(st);
              }
              function ge(Je, it) {
                it = it === 0 ? 0 : it || 10;
                var st = de(Je).toHsl();
                return st.s += it / 100, st.s = qe(st.s), de(st);
              }
              function ve(Je) {
                return de(Je).desaturate(100);
              }
              function be(Je, it) {
                it = it === 0 ? 0 : it || 10;
                var st = de(Je).toHsl();
                return st.l += it / 100, st.l = qe(st.l), de(st);
              }
              function Ce(Je, it) {
                it = it === 0 ? 0 : it || 10;
                var st = de(Je).toRgb();
                return st.r = le(0, ce(255, st.r - se(-it / 100 * 255))), st.g = le(0, ce(255, st.g - se(-it / 100 * 255))), st.b = le(0, ce(255, st.b - se(-it / 100 * 255))), de(st);
              }
              function Se(Je, it) {
                it = it === 0 ? 0 : it || 10;
                var st = de(Je).toHsl();
                return st.l -= it / 100, st.l = qe(st.l), de(st);
              }
              function we(Je, it) {
                var st = de(Je).toHsl(), gt = (st.h + it) % 360;
                return st.h = gt < 0 ? 360 + gt : gt, de(st);
              }
              function Ee(Je) {
                var it = de(Je).toHsl();
                return it.h = (it.h + 180) % 360, de(it);
              }
              function xe(Je) {
                var it = de(Je).toHsl(), st = it.h;
                return [de(Je), de({ h: (st + 120) % 360, s: it.s, l: it.l }), de({ h: (st + 240) % 360, s: it.s, l: it.l })];
              }
              function Le(Je) {
                var it = de(Je).toHsl(), st = it.h;
                return [de(Je), de({ h: (st + 90) % 360, s: it.s, l: it.l }), de({ h: (st + 180) % 360, s: it.s, l: it.l }), de({ h: (st + 270) % 360, s: it.s, l: it.l })];
              }
              function Qe(Je) {
                var it = de(Je).toHsl(), st = it.h;
                return [de(Je), de({ h: (st + 72) % 360, s: it.s, l: it.l }), de({ h: (st + 216) % 360, s: it.s, l: it.l })];
              }
              function ke(Je, it, st) {
                it = it || 6, st = st || 30;
                var gt = de(Je).toHsl(), bt = 360 / st, yt = [de(Je)];
                for (gt.h = (gt.h - (bt * it >> 1) + 720) % 360; --it; ) gt.h = (gt.h + bt) % 360, yt.push(de(gt));
                return yt;
              }
              function Me(Je, it) {
                it = it || 6;
                for (var st = de(Je).toHsv(), gt = st.h, bt = st.s, yt = st.v, St = [], Lt = 1 / it; it--; ) St.push(de({ h: gt, s: bt, v: yt })), yt = (yt + Lt) % 1;
                return St;
              }
              de.prototype = { isDark: function() {
                return this.getBrightness() < 128;
              }, isLight: function() {
                return !this.isDark();
              }, isValid: function() {
                return this._ok;
              }, getOriginalInput: function() {
                return this._originalInput;
              }, getFormat: function() {
                return this._format;
              }, getAlpha: function() {
                return this._a;
              }, getBrightness: function() {
                var Je = this.toRgb();
                return (299 * Je.r + 587 * Je.g + 114 * Je.b) / 1e3;
              }, getLuminance: function() {
                var Je, it, st, gt = this.toRgb();
                return Je = gt.r / 255, it = gt.g / 255, st = gt.b / 255, 0.2126 * (Je <= 0.03928 ? Je / 12.92 : ne.pow((Je + 0.055) / 1.055, 2.4)) + 0.7152 * (it <= 0.03928 ? it / 12.92 : ne.pow((it + 0.055) / 1.055, 2.4)) + 0.0722 * (st <= 0.03928 ? st / 12.92 : ne.pow((st + 0.055) / 1.055, 2.4));
              }, setAlpha: function(Je) {
                return this._a = Pe(Je), this._roundA = se(100 * this._a) / 100, this;
              }, toHsv: function() {
                var Je = he(this._r, this._g, this._b);
                return { h: 360 * Je.h, s: Je.s, v: Je.v, a: this._a };
              }, toHsvString: function() {
                var Je = he(this._r, this._g, this._b), it = se(360 * Je.h), st = se(100 * Je.s), gt = se(100 * Je.v);
                return this._a == 1 ? "hsv(" + it + ", " + st + "%, " + gt + "%)" : "hsva(" + it + ", " + st + "%, " + gt + "%, " + this._roundA + ")";
              }, toHsl: function() {
                var Je = fe(this._r, this._g, this._b);
                return { h: 360 * Je.h, s: Je.s, l: Je.l, a: this._a };
              }, toHslString: function() {
                var Je = fe(this._r, this._g, this._b), it = se(360 * Je.h), st = se(100 * Je.s), gt = se(100 * Je.l);
                return this._a == 1 ? "hsl(" + it + ", " + st + "%, " + gt + "%)" : "hsla(" + it + ", " + st + "%, " + gt + "%, " + this._roundA + ")";
              }, toHex: function(Je) {
                return me(this._r, this._g, this._b, Je);
              }, toHexString: function(Je) {
                return "#" + this.toHex(Je);
              }, toHex8: function(Je) {
                return function(it, st, gt, bt, yt) {
                  var St = [Ve(se(it).toString(16)), Ve(se(st).toString(16)), Ve(se(gt).toString(16)), Ve(Ge(bt))];
                  return yt && St[0].charAt(0) == St[0].charAt(1) && St[1].charAt(0) == St[1].charAt(1) && St[2].charAt(0) == St[2].charAt(1) && St[3].charAt(0) == St[3].charAt(1) ? St[0].charAt(0) + St[1].charAt(0) + St[2].charAt(0) + St[3].charAt(0) : St.join("");
                }(this._r, this._g, this._b, this._a, Je);
              }, toHex8String: function(Je) {
                return "#" + this.toHex8(Je);
              }, toRgb: function() {
                return { r: se(this._r), g: se(this._g), b: se(this._b), a: this._a };
              }, toRgbString: function() {
                return this._a == 1 ? "rgb(" + se(this._r) + ", " + se(this._g) + ", " + se(this._b) + ")" : "rgba(" + se(this._r) + ", " + se(this._g) + ", " + se(this._b) + ", " + this._roundA + ")";
              }, toPercentageRgb: function() {
                return { r: se(100 * $e(this._r, 255)) + "%", g: se(100 * $e(this._g, 255)) + "%", b: se(100 * $e(this._b, 255)) + "%", a: this._a };
              }, toPercentageRgbString: function() {
                return this._a == 1 ? "rgb(" + se(100 * $e(this._r, 255)) + "%, " + se(100 * $e(this._g, 255)) + "%, " + se(100 * $e(this._b, 255)) + "%)" : "rgba(" + se(100 * $e(this._r, 255)) + "%, " + se(100 * $e(this._g, 255)) + "%, " + se(100 * $e(this._b, 255)) + "%, " + this._roundA + ")";
              }, toName: function() {
                return this._a === 0 ? "transparent" : !(this._a < 1) && (De[me(this._r, this._g, this._b, !0)] || !1);
              }, toFilter: function(Je) {
                var it = "#" + pe(this._r, this._g, this._b, this._a), st = it, gt = this._gradientType ? "GradientType = 1, " : "";
                if (Je) {
                  var bt = de(Je);
                  st = "#" + pe(bt._r, bt._g, bt._b, bt._a);
                }
                return "progid:DXImageTransform.Microsoft.gradient(" + gt + "startColorstr=" + it + ",endColorstr=" + st + ")";
              }, toString: function(Je) {
                var it = !!Je;
                Je = Je || this._format;
                var st = !1, gt = this._a < 1 && this._a >= 0;
                return it || !gt || Je !== "hex" && Je !== "hex6" && Je !== "hex3" && Je !== "hex4" && Je !== "hex8" && Je !== "name" ? (Je === "rgb" && (st = this.toRgbString()), Je === "prgb" && (st = this.toPercentageRgbString()), Je !== "hex" && Je !== "hex6" || (st = this.toHexString()), Je === "hex3" && (st = this.toHexString(!0)), Je === "hex4" && (st = this.toHex8String(!0)), Je === "hex8" && (st = this.toHex8String()), Je === "name" && (st = this.toName()), Je === "hsl" && (st = this.toHslString()), Je === "hsv" && (st = this.toHsvString()), st || this.toHexString()) : Je === "name" && this._a === 0 ? this.toName() : this.toRgbString();
              }, clone: function() {
                return de(this.toString());
              }, _applyModification: function(Je, it) {
                var st = Je.apply(null, [this].concat([].slice.call(it)));
                return this._r = st._r, this._g = st._g, this._b = st._b, this.setAlpha(st._a), this;
              }, lighten: function() {
                return this._applyModification(be, arguments);
              }, brighten: function() {
                return this._applyModification(Ce, arguments);
              }, darken: function() {
                return this._applyModification(Se, arguments);
              }, desaturate: function() {
                return this._applyModification(ye, arguments);
              }, saturate: function() {
                return this._applyModification(ge, arguments);
              }, greyscale: function() {
                return this._applyModification(ve, arguments);
              }, spin: function() {
                return this._applyModification(we, arguments);
              }, _applyCombination: function(Je, it) {
                return Je.apply(null, [this].concat([].slice.call(it)));
              }, analogous: function() {
                return this._applyCombination(ke, arguments);
              }, complement: function() {
                return this._applyCombination(Ee, arguments);
              }, monochromatic: function() {
                return this._applyCombination(Me, arguments);
              }, splitcomplement: function() {
                return this._applyCombination(Qe, arguments);
              }, triad: function() {
                return this._applyCombination(xe, arguments);
              }, tetrad: function() {
                return this._applyCombination(Le, arguments);
              } }, de.fromRatio = function(Je, it) {
                if (typeof Je == "object") {
                  var st = {};
                  for (var gt in Je) Je.hasOwnProperty(gt) && (st[gt] = gt === "a" ? Je[gt] : Ze(Je[gt]));
                  Je = st;
                }
                return de(Je, it);
              }, de.equals = function(Je, it) {
                return !(!Je || !it) && de(Je).toRgbString() == de(it).toRgbString();
              }, de.random = function() {
                return de.fromRatio({ r: ue(), g: ue(), b: ue() });
              }, de.mix = function(Je, it, st) {
                st = st === 0 ? 0 : st || 50;
                var gt = de(Je).toRgb(), bt = de(it).toRgb(), yt = st / 100;
                return de({ r: (bt.r - gt.r) * yt + gt.r, g: (bt.g - gt.g) * yt + gt.g, b: (bt.b - gt.b) * yt + gt.b, a: (bt.a - gt.a) * yt + gt.a });
              }, de.readability = function(Je, it) {
                var st = de(Je), gt = de(it);
                return (ne.max(st.getLuminance(), gt.getLuminance()) + 0.05) / (ne.min(st.getLuminance(), gt.getLuminance()) + 0.05);
              }, de.isReadable = function(Je, it, st) {
                var gt, bt, yt, St, Lt, Rt = de.readability(Je, it);
                switch (bt = !1, (yt = st, (St = ((yt = yt || { level: "AA", size: "small" }).level || "AA").toUpperCase()) !== "AA" && St !== "AAA" && (St = "AA"), (Lt = (yt.size || "small").toLowerCase()) !== "small" && Lt !== "large" && (Lt = "small"), gt = { level: St, size: Lt }).level + gt.size) {
                  case "AAsmall":
                  case "AAAlarge":
                    bt = Rt >= 4.5;
                    break;
                  case "AAlarge":
                    bt = Rt >= 3;
                    break;
                  case "AAAsmall":
                    bt = Rt >= 7;
                }
                return bt;
              }, de.mostReadable = function(Je, it, st) {
                var gt, bt, yt, St, Lt = null, Rt = 0;
                bt = (st = st || {}).includeFallbackColors, yt = st.level, St = st.size;
                for (var vt = 0; vt < it.length; vt++) (gt = de.readability(Je, it[vt])) > Rt && (Rt = gt, Lt = de(it[vt]));
                return de.isReadable(Je, Lt, { level: yt, size: St }) || !bt ? Lt : (st.includeFallbackColors = !1, de.mostReadable(Je, ["#fff", "#000"], st));
              };
              var Re = de.names = { aliceblue: "f0f8ff", antiquewhite: "faebd7", aqua: "0ff", aquamarine: "7fffd4", azure: "f0ffff", beige: "f5f5dc", bisque: "ffe4c4", black: "000", blanchedalmond: "ffebcd", blue: "00f", blueviolet: "8a2be2", brown: "a52a2a", burlywood: "deb887", burntsienna: "ea7e5d", cadetblue: "5f9ea0", chartreuse: "7fff00", chocolate: "d2691e", coral: "ff7f50", cornflowerblue: "6495ed", cornsilk: "fff8dc", crimson: "dc143c", cyan: "0ff", darkblue: "00008b", darkcyan: "008b8b", darkgoldenrod: "b8860b", darkgray: "a9a9a9", darkgreen: "006400", darkgrey: "a9a9a9", darkkhaki: "bdb76b", darkmagenta: "8b008b", darkolivegreen: "556b2f", darkorange: "ff8c00", darkorchid: "9932cc", darkred: "8b0000", darksalmon: "e9967a", darkseagreen: "8fbc8f", darkslateblue: "483d8b", darkslategray: "2f4f4f", darkslategrey: "2f4f4f", darkturquoise: "00ced1", darkviolet: "9400d3", deeppink: "ff1493", deepskyblue: "00bfff", dimgray: "696969", dimgrey: "696969", dodgerblue: "1e90ff", firebrick: "b22222", floralwhite: "fffaf0", forestgreen: "228b22", fuchsia: "f0f", gainsboro: "dcdcdc", ghostwhite: "f8f8ff", gold: "ffd700", goldenrod: "daa520", gray: "808080", green: "008000", greenyellow: "adff2f", grey: "808080", honeydew: "f0fff0", hotpink: "ff69b4", indianred: "cd5c5c", indigo: "4b0082", ivory: "fffff0", khaki: "f0e68c", lavender: "e6e6fa", lavenderblush: "fff0f5", lawngreen: "7cfc00", lemonchiffon: "fffacd", lightblue: "add8e6", lightcoral: "f08080", lightcyan: "e0ffff", lightgoldenrodyellow: "fafad2", lightgray: "d3d3d3", lightgreen: "90ee90", lightgrey: "d3d3d3", lightpink: "ffb6c1", lightsalmon: "ffa07a", lightseagreen: "20b2aa", lightskyblue: "87cefa", lightslategray: "789", lightslategrey: "789", lightsteelblue: "b0c4de", lightyellow: "ffffe0", lime: "0f0", limegreen: "32cd32", linen: "faf0e6", magenta: "f0f", maroon: "800000", mediumaquamarine: "66cdaa", mediumblue: "0000cd", mediumorchid: "ba55d3", mediumpurple: "9370db", mediumseagreen: "3cb371", mediumslateblue: "7b68ee", mediumspringgreen: "00fa9a", mediumturquoise: "48d1cc", mediumvioletred: "c71585", midnightblue: "191970", mintcream: "f5fffa", mistyrose: "ffe4e1", moccasin: "ffe4b5", navajowhite: "ffdead", navy: "000080", oldlace: "fdf5e6", olive: "808000", olivedrab: "6b8e23", orange: "ffa500", orangered: "ff4500", orchid: "da70d6", palegoldenrod: "eee8aa", palegreen: "98fb98", paleturquoise: "afeeee", palevioletred: "db7093", papayawhip: "ffefd5", peachpuff: "ffdab9", peru: "cd853f", pink: "ffc0cb", plum: "dda0dd", powderblue: "b0e0e6", purple: "800080", rebeccapurple: "663399", red: "f00", rosybrown: "bc8f8f", royalblue: "4169e1", saddlebrown: "8b4513", salmon: "fa8072", sandybrown: "f4a460", seagreen: "2e8b57", seashell: "fff5ee", sienna: "a0522d", silver: "c0c0c0", skyblue: "87ceeb", slateblue: "6a5acd", slategray: "708090", slategrey: "708090", snow: "fffafa", springgreen: "00ff7f", steelblue: "4682b4", tan: "d2b48c", teal: "008080", thistle: "d8bfd8", tomato: "ff6347", turquoise: "40e0d0", violet: "ee82ee", wheat: "f5deb3", white: "fff", whitesmoke: "f5f5f5", yellow: "ff0", yellowgreen: "9acd32" }, De = de.hexNames = function(Je) {
                var it = {};
                for (var st in Je) Je.hasOwnProperty(st) && (it[Je[st]] = st);
                return it;
              }(Re);
              function Pe(Je) {
                return Je = parseFloat(Je), (isNaN(Je) || Je < 0 || Je > 1) && (Je = 1), Je;
              }
              function $e(Je, it) {
                (function(gt) {
                  return typeof gt == "string" && gt.indexOf(".") != -1 && parseFloat(gt) === 1;
                })(Je) && (Je = "100%");
                var st = function(gt) {
                  return typeof gt == "string" && gt.indexOf("%") != -1;
                }(Je);
                return Je = ce(it, le(0, parseFloat(Je))), st && (Je = parseInt(Je * it, 10) / 100), ne.abs(Je - it) < 1e-6 ? 1 : Je % it / parseFloat(it);
              }
              function qe(Je) {
                return ce(1, le(0, Je));
              }
              function Xe(Je) {
                return parseInt(Je, 16);
              }
              function Ve(Je) {
                return Je.length == 1 ? "0" + Je : "" + Je;
              }
              function Ze(Je) {
                return Je <= 1 && (Je = 100 * Je + "%"), Je;
              }
              function Ge(Je) {
                return ne.round(255 * parseFloat(Je)).toString(16);
              }
              function ze(Je) {
                return Xe(Je) / 255;
              }
              var We, at, ct, et = (at = "[\\s|\\(]+(" + (We = "(?:[-\\+]?\\d*\\.\\d+%?)|(?:[-\\+]?\\d+%?)") + ")[,|\\s]+(" + We + ")[,|\\s]+(" + We + ")\\s*\\)?", ct = "[\\s|\\(]+(" + We + ")[,|\\s]+(" + We + ")[,|\\s]+(" + We + ")[,|\\s]+(" + We + ")\\s*\\)?", { CSS_UNIT: new RegExp(We), rgb: new RegExp("rgb" + at), rgba: new RegExp("rgba" + ct), hsl: new RegExp("hsl" + at), hsla: new RegExp("hsla" + ct), hsv: new RegExp("hsv" + at), hsva: new RegExp("hsva" + ct), hex3: /^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/, hex6: /^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/, hex4: /^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/, hex8: /^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/ });
              function dt(Je) {
                return !!et.CSS_UNIT.exec(Je);
              }
              G.exports ? G.exports = de : (te = (function() {
                return de;
              }).call(J, Z, J, G)) === void 0 || (G.exports = te);
            })(Math);
          }, 51498: function(G) {
            G.exports = Z, G.exports.float32 = G.exports.float = Z, G.exports.fract32 = G.exports.fract = function(te, ne) {
              if (te.length) {
                if (te instanceof Float32Array) return new Float32Array(te.length);
                ne instanceof Float32Array || (ne = Z(te));
                for (var ee = 0, ae = ne.length; ee < ae; ee++) ne[ee] = te[ee] - ne[ee];
                return ne;
              }
              return Z(te - Z(te));
            };
            var J = new Float32Array(1);
            function Z(te) {
              return te.length ? te instanceof Float32Array ? te : new Float32Array(te) : (J[0] = te, J[0]);
            }
          }, 44626: function(G, J, Z) {
            var te = Z(4957);
            G.exports = ae;
            var ne = 96;
            function ee(oe, se) {
              var ce = te(getComputedStyle(oe).getPropertyValue(se));
              return ce[0] * ae(ce[1], oe);
            }
            function ae(oe, se) {
              switch (se = se || document.body, oe = (oe || "px").trim().toLowerCase(), se !== window && se !== document || (se = document.body), oe) {
                case "%":
                  return se.clientHeight / 100;
                case "ch":
                case "ex":
                  return function(ce, le) {
                    var ue = document.createElement("div");
                    ue.style["font-size"] = "128" + ce, le.appendChild(ue);
                    var de = ee(ue, "font-size") / 128;
                    return le.removeChild(ue), de;
                  }(oe, se);
                case "em":
                  return ee(se, "font-size");
                case "rem":
                  return ee(document.body, "font-size");
                case "vw":
                  return window.innerWidth / 100;
                case "vh":
                  return window.innerHeight / 100;
                case "vmin":
                  return Math.min(window.innerWidth, window.innerHeight) / 100;
                case "vmax":
                  return Math.max(window.innerWidth, window.innerHeight) / 100;
                case "in":
                  return ne;
                case "cm":
                  return ne / 2.54;
                case "mm":
                  return ne / 25.4;
                case "pt":
                  return ne / 72;
                case "pc":
                  return ne / 6;
              }
              return 1;
            }
          }, 48640: function(G, J, Z) {
            function te(ae) {
              return ae;
            }
            function ne(ae, oe) {
              return typeof oe == "string" && (oe = ae.objects[oe]), oe.type === "GeometryCollection" ? { type: "FeatureCollection", features: oe.geometries.map(function(se) {
                return ee(ae, se);
              }) } : ee(ae, oe);
            }
            function ee(ae, oe) {
              var se = oe.id, ce = oe.bbox, le = oe.properties == null ? {} : oe.properties, ue = function(de, fe) {
                var he = function(Ce) {
                  if (Ce == null) return te;
                  var Se, we, Ee = Ce.scale[0], xe = Ce.scale[1], Le = Ce.translate[0], Qe = Ce.translate[1];
                  return function(ke, Me) {
                    Me || (Se = we = 0);
                    var Re = 2, De = ke.length, Pe = new Array(De);
                    for (Pe[0] = (Se += ke[0]) * Ee + Le, Pe[1] = (we += ke[1]) * xe + Qe; Re < De; ) Pe[Re] = ke[Re], ++Re;
                    return Pe;
                  };
                }(de.transform), me = de.arcs;
                function pe(Ce, Se) {
                  Se.length && Se.pop();
                  for (var we = me[Ce < 0 ? ~Ce : Ce], Ee = 0, xe = we.length; Ee < xe; ++Ee) Se.push(he(we[Ee], Ee));
                  Ce < 0 && function(Le, Qe) {
                    for (var ke, Me = Le.length, Re = Me - Qe; Re < --Me; ) ke = Le[Re], Le[Re++] = Le[Me], Le[Me] = ke;
                  }(Se, xe);
                }
                function ye(Ce) {
                  return he(Ce);
                }
                function ge(Ce) {
                  for (var Se = [], we = 0, Ee = Ce.length; we < Ee; ++we) pe(Ce[we], Se);
                  return Se.length < 2 && Se.push(Se[0]), Se;
                }
                function ve(Ce) {
                  for (var Se = ge(Ce); Se.length < 4; ) Se.push(Se[0]);
                  return Se;
                }
                function be(Ce) {
                  return Ce.map(ve);
                }
                return function Ce(Se) {
                  var we, Ee = Se.type;
                  switch (Ee) {
                    case "GeometryCollection":
                      return { type: Ee, geometries: Se.geometries.map(Ce) };
                    case "Point":
                      we = ye(Se.coordinates);
                      break;
                    case "MultiPoint":
                      we = Se.coordinates.map(ye);
                      break;
                    case "LineString":
                      we = ge(Se.arcs);
                      break;
                    case "MultiLineString":
                      we = Se.arcs.map(ge);
                      break;
                    case "Polygon":
                      we = be(Se.arcs);
                      break;
                    case "MultiPolygon":
                      we = Se.arcs.map(be);
                      break;
                    default:
                      return null;
                  }
                  return { type: Ee, coordinates: we };
                }(fe);
              }(ae, oe);
              return se == null && ce == null ? { type: "Feature", properties: le, geometry: ue } : ce == null ? { type: "Feature", id: se, properties: le, geometry: ue } : { type: "Feature", id: se, bbox: ce, properties: le, geometry: ue };
            }
            Z.d(J, { N4: function() {
              return ne;
            } });
          }, 64276: function(G, J, Z) {
            var te = Z(31350);
            G.exports = function(ne) {
              if (typeof ne != "function" || !hasOwnProperty.call(ne, "length")) return !1;
              try {
                if (typeof ne.length != "number" || typeof ne.call != "function" || typeof ne.apply != "function") return !1;
              } catch {
                return !1;
              }
              return !te(ne);
            };
          }, 99497: function(G, J, Z) {
            var te = Z(80299), ne = Z(76481), ee = Z(58698), ae = Z(60461), oe = function(se, ce) {
              return se.replace("%v", ae(ce));
            };
            G.exports = function(se, ce, le) {
              if (!ne(le)) throw new TypeError(oe(ce, se));
              if (!te(se)) {
                if ("default" in le) return le.default;
                if (le.isOptional) return null;
              }
              var ue = ee(le.errorMessage);
              throw te(ue) || (ue = ce), new TypeError(oe(ue, se));
            };
          }, 78696: function(G) {
            G.exports = function(J) {
              try {
                return J.toString();
              } catch {
                try {
                  return String(J);
                } catch {
                  return null;
                }
              }
            };
          }, 60461: function(G, J, Z) {
            var te = Z(78696), ne = /[\n\r\u2028\u2029]/g;
            G.exports = function(ee) {
              var ae = te(ee);
              return ae === null ? "<Non-coercible to string value>" : (ae.length > 100 && (ae = ae.slice(0, 99) + "…"), ae = ae.replace(ne, function(oe) {
                switch (oe) {
                  case `
`:
                    return "\\n";
                  case "\r":
                    return "\\r";
                  case "\u2028":
                    return "\\u2028";
                  case "\u2029":
                    return "\\u2029";
                  default:
                    throw new Error("Unexpected character");
                }
              }));
            };
          }, 76481: function(G, J, Z) {
            var te = Z(80299), ne = { object: !0, function: !0, undefined: !0 };
            G.exports = function(ee) {
              return !!te(ee) && hasOwnProperty.call(ne, typeof ee);
            };
          }, 6887: function(G, J, Z) {
            var te = Z(99497), ne = Z(63461);
            G.exports = function(ee) {
              return ne(ee) ? ee : te(ee, "%v is not a plain function", arguments[1]);
            };
          }, 63461: function(G, J, Z) {
            var te = Z(64276), ne = /^\s*class[\s{/}]/, ee = Function.prototype.toString;
            G.exports = function(ae) {
              return !!te(ae) && !ne.test(ee.call(ae));
            };
          }, 31350: function(G, J, Z) {
            var te = Z(76481);
            G.exports = function(ne) {
              if (!te(ne)) return !1;
              try {
                return !!ne.constructor && ne.constructor.prototype === ne;
              } catch {
                return !1;
              }
            };
          }, 58698: function(G, J, Z) {
            var te = Z(80299), ne = Z(76481), ee = Object.prototype.toString;
            G.exports = function(ae) {
              if (!te(ae)) return null;
              if (ne(ae)) {
                var oe = ae.toString;
                if (typeof oe != "function" || oe === ee) return null;
              }
              try {
                return "" + ae;
              } catch {
                return null;
              }
            };
          }, 9557: function(G, J, Z) {
            var te = Z(99497), ne = Z(80299);
            G.exports = function(ee) {
              return ne(ee) ? ee : te(ee, "Cannot use %v", arguments[1]);
            };
          }, 80299: function(G) {
            G.exports = function(J) {
              return J != null;
            };
          }, 66127: function(G, J, Z) {
            var te = Z(54689), ne = Z(49523), ee = Z(45708).Buffer;
            Z.g.__TYPEDARRAY_POOL || (Z.g.__TYPEDARRAY_POOL = { UINT8: ne([32, 0]), UINT16: ne([32, 0]), UINT32: ne([32, 0]), BIGUINT64: ne([32, 0]), INT8: ne([32, 0]), INT16: ne([32, 0]), INT32: ne([32, 0]), BIGINT64: ne([32, 0]), FLOAT: ne([32, 0]), DOUBLE: ne([32, 0]), DATA: ne([32, 0]), UINT8C: ne([32, 0]), BUFFER: ne([32, 0]) });
            var ae = typeof Uint8ClampedArray < "u", oe = typeof BigUint64Array < "u", se = typeof BigInt64Array < "u", ce = Z.g.__TYPEDARRAY_POOL;
            ce.UINT8C || (ce.UINT8C = ne([32, 0])), ce.BIGUINT64 || (ce.BIGUINT64 = ne([32, 0])), ce.BIGINT64 || (ce.BIGINT64 = ne([32, 0])), ce.BUFFER || (ce.BUFFER = ne([32, 0]));
            var le = ce.DATA, ue = ce.BUFFER;
            function de(Qe) {
              if (Qe) {
                var ke = Qe.length || Qe.byteLength, Me = te.log2(ke);
                le[Me].push(Qe);
              }
            }
            function fe(Qe) {
              Qe = te.nextPow2(Qe);
              var ke = te.log2(Qe), Me = le[ke];
              return Me.length > 0 ? Me.pop() : new ArrayBuffer(Qe);
            }
            function he(Qe) {
              return new Uint8Array(fe(Qe), 0, Qe);
            }
            function me(Qe) {
              return new Uint16Array(fe(2 * Qe), 0, Qe);
            }
            function pe(Qe) {
              return new Uint32Array(fe(4 * Qe), 0, Qe);
            }
            function ye(Qe) {
              return new Int8Array(fe(Qe), 0, Qe);
            }
            function ge(Qe) {
              return new Int16Array(fe(2 * Qe), 0, Qe);
            }
            function ve(Qe) {
              return new Int32Array(fe(4 * Qe), 0, Qe);
            }
            function be(Qe) {
              return new Float32Array(fe(4 * Qe), 0, Qe);
            }
            function Ce(Qe) {
              return new Float64Array(fe(8 * Qe), 0, Qe);
            }
            function Se(Qe) {
              return ae ? new Uint8ClampedArray(fe(Qe), 0, Qe) : he(Qe);
            }
            function we(Qe) {
              return oe ? new BigUint64Array(fe(8 * Qe), 0, Qe) : null;
            }
            function Ee(Qe) {
              return se ? new BigInt64Array(fe(8 * Qe), 0, Qe) : null;
            }
            function xe(Qe) {
              return new DataView(fe(Qe), 0, Qe);
            }
            function Le(Qe) {
              Qe = te.nextPow2(Qe);
              var ke = te.log2(Qe), Me = ue[ke];
              return Me.length > 0 ? Me.pop() : new ee(Qe);
            }
            J.free = function(Qe) {
              if (ee.isBuffer(Qe)) ue[te.log2(Qe.length)].push(Qe);
              else {
                if (Object.prototype.toString.call(Qe) !== "[object ArrayBuffer]" && (Qe = Qe.buffer), !Qe) return;
                var ke = Qe.length || Qe.byteLength, Me = 0 | te.log2(ke);
                le[Me].push(Qe);
              }
            }, J.freeUint8 = J.freeUint16 = J.freeUint32 = J.freeBigUint64 = J.freeInt8 = J.freeInt16 = J.freeInt32 = J.freeBigInt64 = J.freeFloat32 = J.freeFloat = J.freeFloat64 = J.freeDouble = J.freeUint8Clamped = J.freeDataView = function(Qe) {
              de(Qe.buffer);
            }, J.freeArrayBuffer = de, J.freeBuffer = function(Qe) {
              ue[te.log2(Qe.length)].push(Qe);
            }, J.malloc = function(Qe, ke) {
              if (ke === void 0 || ke === "arraybuffer") return fe(Qe);
              switch (ke) {
                case "uint8":
                  return he(Qe);
                case "uint16":
                  return me(Qe);
                case "uint32":
                  return pe(Qe);
                case "int8":
                  return ye(Qe);
                case "int16":
                  return ge(Qe);
                case "int32":
                  return ve(Qe);
                case "float":
                case "float32":
                  return be(Qe);
                case "double":
                case "float64":
                  return Ce(Qe);
                case "uint8_clamped":
                  return Se(Qe);
                case "bigint64":
                  return Ee(Qe);
                case "biguint64":
                  return we(Qe);
                case "buffer":
                  return Le(Qe);
                case "data":
                case "dataview":
                  return xe(Qe);
                default:
                  return null;
              }
              return null;
            }, J.mallocArrayBuffer = fe, J.mallocUint8 = he, J.mallocUint16 = me, J.mallocUint32 = pe, J.mallocInt8 = ye, J.mallocInt16 = ge, J.mallocInt32 = ve, J.mallocFloat32 = J.mallocFloat = be, J.mallocFloat64 = J.mallocDouble = Ce, J.mallocUint8Clamped = Se, J.mallocBigUint64 = we, J.mallocBigInt64 = Ee, J.mallocDataView = xe, J.mallocBuffer = Le, J.clearCache = function() {
              for (var Qe = 0; Qe < 32; ++Qe) ce.UINT8[Qe].length = 0, ce.UINT16[Qe].length = 0, ce.UINT32[Qe].length = 0, ce.INT8[Qe].length = 0, ce.INT16[Qe].length = 0, ce.INT32[Qe].length = 0, ce.FLOAT[Qe].length = 0, ce.DOUBLE[Qe].length = 0, ce.BIGUINT64[Qe].length = 0, ce.BIGINT64[Qe].length = 0, ce.UINT8C[Qe].length = 0, le[Qe].length = 0, ue[Qe].length = 0;
            };
          }, 80886: function(G) {
            var J = /[\'\"]/;
            G.exports = function(Z) {
              return Z ? (J.test(Z.charAt(0)) && (Z = Z.substr(1)), J.test(Z.charAt(Z.length - 1)) && (Z = Z.substr(0, Z.length - 1)), Z) : "";
            };
          }, 79788: function(G) {
            G.exports = function(J, Z, te) {
              Array.isArray(te) || (te = [].slice.call(arguments, 2));
              for (var ne = 0, ee = te.length; ne < ee; ne++) {
                var ae = te[ne];
                for (var oe in ae) if ((Z[oe] === void 0 || Array.isArray(Z[oe]) || J[oe] !== Z[oe]) && oe in Z) {
                  var se;
                  if (ae[oe] === !0) se = Z[oe];
                  else if (ae[oe] === !1 || typeof ae[oe] == "function" && (se = ae[oe](Z[oe], J, Z)) === void 0) continue;
                  J[oe] = se;
                }
              }
              return J;
            };
          }, 71103: function(G, J, Z) {
            function te(ne) {
              try {
                if (!Z.g.localStorage) return !1;
              } catch {
                return !1;
              }
              var ee = Z.g.localStorage[ne];
              return ee != null && String(ee).toLowerCase() === "true";
            }
            G.exports = function(ne, ee) {
              if (te("noDeprecation")) return ne;
              var ae = !1;
              return function() {
                if (!ae) {
                  if (te("throwDeprecation")) throw new Error(ee);
                  te("traceDeprecation") ? console.trace(ee) : console.warn(ee), ae = !0;
                }
                return ne.apply(this, arguments);
              };
            };
          }, 44123: function(G) {
            G.exports = function(J) {
              return J && typeof J == "object" && typeof J.copy == "function" && typeof J.fill == "function" && typeof J.readUInt8 == "function";
            };
          }, 15724: function(G, J, Z) {
            var te = Z(40280), ne = Z(80340), ee = Z(96835), ae = Z(15628);
            function oe($e) {
              return $e.call.bind($e);
            }
            var se = typeof BigInt < "u", ce = typeof Symbol < "u", le = oe(Object.prototype.toString), ue = oe(Number.prototype.valueOf), de = oe(String.prototype.valueOf), fe = oe(Boolean.prototype.valueOf);
            if (se) var he = oe(BigInt.prototype.valueOf);
            if (ce) var me = oe(Symbol.prototype.valueOf);
            function pe($e, qe) {
              if (typeof $e != "object") return !1;
              try {
                return qe($e), !0;
              } catch {
                return !1;
              }
            }
            function ye($e) {
              return le($e) === "[object Map]";
            }
            function ge($e) {
              return le($e) === "[object Set]";
            }
            function ve($e) {
              return le($e) === "[object WeakMap]";
            }
            function be($e) {
              return le($e) === "[object WeakSet]";
            }
            function Ce($e) {
              return le($e) === "[object ArrayBuffer]";
            }
            function Se($e) {
              return typeof ArrayBuffer < "u" && (Ce.working ? Ce($e) : $e instanceof ArrayBuffer);
            }
            function we($e) {
              return le($e) === "[object DataView]";
            }
            function Ee($e) {
              return typeof DataView < "u" && (we.working ? we($e) : $e instanceof DataView);
            }
            J.isArgumentsObject = te, J.isGeneratorFunction = ne, J.isTypedArray = ae, J.isPromise = function($e) {
              return typeof Promise < "u" && $e instanceof Promise || $e !== null && typeof $e == "object" && typeof $e.then == "function" && typeof $e.catch == "function";
            }, J.isArrayBufferView = function($e) {
              return typeof ArrayBuffer < "u" && ArrayBuffer.isView ? ArrayBuffer.isView($e) : ae($e) || Ee($e);
            }, J.isUint8Array = function($e) {
              return ee($e) === "Uint8Array";
            }, J.isUint8ClampedArray = function($e) {
              return ee($e) === "Uint8ClampedArray";
            }, J.isUint16Array = function($e) {
              return ee($e) === "Uint16Array";
            }, J.isUint32Array = function($e) {
              return ee($e) === "Uint32Array";
            }, J.isInt8Array = function($e) {
              return ee($e) === "Int8Array";
            }, J.isInt16Array = function($e) {
              return ee($e) === "Int16Array";
            }, J.isInt32Array = function($e) {
              return ee($e) === "Int32Array";
            }, J.isFloat32Array = function($e) {
              return ee($e) === "Float32Array";
            }, J.isFloat64Array = function($e) {
              return ee($e) === "Float64Array";
            }, J.isBigInt64Array = function($e) {
              return ee($e) === "BigInt64Array";
            }, J.isBigUint64Array = function($e) {
              return ee($e) === "BigUint64Array";
            }, ye.working = typeof Map < "u" && ye(/* @__PURE__ */ new Map()), J.isMap = function($e) {
              return typeof Map < "u" && (ye.working ? ye($e) : $e instanceof Map);
            }, ge.working = typeof Set < "u" && ge(/* @__PURE__ */ new Set()), J.isSet = function($e) {
              return typeof Set < "u" && (ge.working ? ge($e) : $e instanceof Set);
            }, ve.working = typeof WeakMap < "u" && ve(/* @__PURE__ */ new WeakMap()), J.isWeakMap = function($e) {
              return typeof WeakMap < "u" && (ve.working ? ve($e) : $e instanceof WeakMap);
            }, be.working = typeof WeakSet < "u" && be(/* @__PURE__ */ new WeakSet()), J.isWeakSet = function($e) {
              return be($e);
            }, Ce.working = typeof ArrayBuffer < "u" && Ce(new ArrayBuffer()), J.isArrayBuffer = Se, we.working = typeof ArrayBuffer < "u" && typeof DataView < "u" && we(new DataView(new ArrayBuffer(1), 0, 1)), J.isDataView = Ee;
            var xe = typeof SharedArrayBuffer < "u" ? SharedArrayBuffer : void 0;
            function Le($e) {
              return le($e) === "[object SharedArrayBuffer]";
            }
            function Qe($e) {
              return xe !== void 0 && (Le.working === void 0 && (Le.working = Le(new xe())), Le.working ? Le($e) : $e instanceof xe);
            }
            function ke($e) {
              return pe($e, ue);
            }
            function Me($e) {
              return pe($e, de);
            }
            function Re($e) {
              return pe($e, fe);
            }
            function De($e) {
              return se && pe($e, he);
            }
            function Pe($e) {
              return ce && pe($e, me);
            }
            J.isSharedArrayBuffer = Qe, J.isAsyncFunction = function($e) {
              return le($e) === "[object AsyncFunction]";
            }, J.isMapIterator = function($e) {
              return le($e) === "[object Map Iterator]";
            }, J.isSetIterator = function($e) {
              return le($e) === "[object Set Iterator]";
            }, J.isGeneratorObject = function($e) {
              return le($e) === "[object Generator]";
            }, J.isWebAssemblyCompiledModule = function($e) {
              return le($e) === "[object WebAssembly.Module]";
            }, J.isNumberObject = ke, J.isStringObject = Me, J.isBooleanObject = Re, J.isBigIntObject = De, J.isSymbolObject = Pe, J.isBoxedPrimitive = function($e) {
              return ke($e) || Me($e) || Re($e) || De($e) || Pe($e);
            }, J.isAnyArrayBuffer = function($e) {
              return typeof Uint8Array < "u" && (Se($e) || Qe($e));
            }, ["isProxy", "isExternal", "isModuleNamespaceObject"].forEach(function($e) {
              Object.defineProperty(J, $e, { enumerable: !1, value: function() {
                throw new Error($e + " is not supported in userland");
              } });
            });
          }, 56557: function(G, J, Z) {
            var te = Z(33282), ne = Object.getOwnPropertyDescriptors || function(Pe) {
              for (var $e = Object.keys(Pe), qe = {}, Xe = 0; Xe < $e.length; Xe++) qe[$e[Xe]] = Object.getOwnPropertyDescriptor(Pe, $e[Xe]);
              return qe;
            }, ee = /%[sdj%]/g;
            J.format = function(Pe) {
              if (!ve(Pe)) {
                for (var $e = [], qe = 0; qe < arguments.length; qe++) $e.push(ce(arguments[qe]));
                return $e.join(" ");
              }
              qe = 1;
              for (var Xe = arguments, Ve = Xe.length, Ze = String(Pe).replace(ee, function(ze) {
                if (ze === "%%") return "%";
                if (qe >= Ve) return ze;
                switch (ze) {
                  case "%s":
                    return String(Xe[qe++]);
                  case "%d":
                    return Number(Xe[qe++]);
                  case "%j":
                    try {
                      return JSON.stringify(Xe[qe++]);
                    } catch {
                      return "[Circular]";
                    }
                  default:
                    return ze;
                }
              }), Ge = Xe[qe]; qe < Ve; Ge = Xe[++qe]) ye(Ge) || !Se(Ge) ? Ze += " " + Ge : Ze += " " + ce(Ge);
              return Ze;
            }, J.deprecate = function(Pe, $e) {
              if (te !== void 0 && te.noDeprecation === !0) return Pe;
              if (te === void 0) return function() {
                return J.deprecate(Pe, $e).apply(this, arguments);
              };
              var qe = !1;
              return function() {
                if (!qe) {
                  if (te.throwDeprecation) throw new Error($e);
                  te.traceDeprecation ? console.trace($e) : console.error($e), qe = !0;
                }
                return Pe.apply(this, arguments);
              };
            };
            var ae = {}, oe = /^$/;
            if (te.env.NODE_DEBUG) {
              var se = te.env.NODE_DEBUG;
              se = se.replace(/[|\\{}()[\]^$+?.]/g, "\\$&").replace(/\*/g, ".*").replace(/,/g, "$|^").toUpperCase(), oe = new RegExp("^" + se + "$", "i");
            }
            function ce(Pe, $e) {
              var qe = { seen: [], stylize: ue };
              return arguments.length >= 3 && (qe.depth = arguments[2]), arguments.length >= 4 && (qe.colors = arguments[3]), pe($e) ? qe.showHidden = $e : $e && J._extend(qe, $e), be(qe.showHidden) && (qe.showHidden = !1), be(qe.depth) && (qe.depth = 2), be(qe.colors) && (qe.colors = !1), be(qe.customInspect) && (qe.customInspect = !0), qe.colors && (qe.stylize = le), de(qe, Pe, qe.depth);
            }
            function le(Pe, $e) {
              var qe = ce.styles[$e];
              return qe ? "\x1B[" + ce.colors[qe][0] + "m" + Pe + "\x1B[" + ce.colors[qe][1] + "m" : Pe;
            }
            function ue(Pe, $e) {
              return Pe;
            }
            function de(Pe, $e, qe) {
              if (Pe.customInspect && $e && xe($e.inspect) && $e.inspect !== J.inspect && (!$e.constructor || $e.constructor.prototype !== $e)) {
                var Xe = $e.inspect(qe, Pe);
                return ve(Xe) || (Xe = de(Pe, Xe, qe)), Xe;
              }
              var Ve = function(dt, Je) {
                if (be(Je)) return dt.stylize("undefined", "undefined");
                if (ve(Je)) {
                  var it = "'" + JSON.stringify(Je).replace(/^"|"$/g, "").replace(/'/g, "\\'").replace(/\\"/g, '"') + "'";
                  return dt.stylize(it, "string");
                }
                return ge(Je) ? dt.stylize("" + Je, "number") : pe(Je) ? dt.stylize("" + Je, "boolean") : ye(Je) ? dt.stylize("null", "null") : void 0;
              }(Pe, $e);
              if (Ve) return Ve;
              var Ze = Object.keys($e), Ge = function(dt) {
                var Je = {};
                return dt.forEach(function(it, st) {
                  Je[it] = !0;
                }), Je;
              }(Ze);
              if (Pe.showHidden && (Ze = Object.getOwnPropertyNames($e)), Ee($e) && (Ze.indexOf("message") >= 0 || Ze.indexOf("description") >= 0)) return fe($e);
              if (Ze.length === 0) {
                if (xe($e)) {
                  var ze = $e.name ? ": " + $e.name : "";
                  return Pe.stylize("[Function" + ze + "]", "special");
                }
                if (Ce($e)) return Pe.stylize(RegExp.prototype.toString.call($e), "regexp");
                if (we($e)) return Pe.stylize(Date.prototype.toString.call($e), "date");
                if (Ee($e)) return fe($e);
              }
              var We, at = "", ct = !1, et = ["{", "}"];
              return me($e) && (ct = !0, et = ["[", "]"]), xe($e) && (at = " [Function" + ($e.name ? ": " + $e.name : "") + "]"), Ce($e) && (at = " " + RegExp.prototype.toString.call($e)), we($e) && (at = " " + Date.prototype.toUTCString.call($e)), Ee($e) && (at = " " + fe($e)), Ze.length !== 0 || ct && $e.length != 0 ? qe < 0 ? Ce($e) ? Pe.stylize(RegExp.prototype.toString.call($e), "regexp") : Pe.stylize("[Object]", "special") : (Pe.seen.push($e), We = ct ? function(dt, Je, it, st, gt) {
                for (var bt = [], yt = 0, St = Je.length; yt < St; ++yt) Me(Je, String(yt)) ? bt.push(he(dt, Je, it, st, String(yt), !0)) : bt.push("");
                return gt.forEach(function(Lt) {
                  Lt.match(/^\d+$/) || bt.push(he(dt, Je, it, st, Lt, !0));
                }), bt;
              }(Pe, $e, qe, Ge, Ze) : Ze.map(function(dt) {
                return he(Pe, $e, qe, Ge, dt, ct);
              }), Pe.seen.pop(), function(dt, Je, it) {
                return dt.reduce(function(st, gt) {
                  return gt.indexOf(`
`), st + gt.replace(/\u001b\[\d\d?m/g, "").length + 1;
                }, 0) > 60 ? it[0] + (Je === "" ? "" : Je + `
 `) + " " + dt.join(`,
  `) + " " + it[1] : it[0] + Je + " " + dt.join(", ") + " " + it[1];
              }(We, at, et)) : et[0] + at + et[1];
            }
            function fe(Pe) {
              return "[" + Error.prototype.toString.call(Pe) + "]";
            }
            function he(Pe, $e, qe, Xe, Ve, Ze) {
              var Ge, ze, We;
              if ((We = Object.getOwnPropertyDescriptor($e, Ve) || { value: $e[Ve] }).get ? ze = We.set ? Pe.stylize("[Getter/Setter]", "special") : Pe.stylize("[Getter]", "special") : We.set && (ze = Pe.stylize("[Setter]", "special")), Me(Xe, Ve) || (Ge = "[" + Ve + "]"), ze || (Pe.seen.indexOf(We.value) < 0 ? (ze = ye(qe) ? de(Pe, We.value, null) : de(Pe, We.value, qe - 1)).indexOf(`
`) > -1 && (ze = Ze ? ze.split(`
`).map(function(at) {
                return "  " + at;
              }).join(`
`).slice(2) : `
` + ze.split(`
`).map(function(at) {
                return "   " + at;
              }).join(`
`)) : ze = Pe.stylize("[Circular]", "special")), be(Ge)) {
                if (Ze && Ve.match(/^\d+$/)) return ze;
                (Ge = JSON.stringify("" + Ve)).match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/) ? (Ge = Ge.slice(1, -1), Ge = Pe.stylize(Ge, "name")) : (Ge = Ge.replace(/'/g, "\\'").replace(/\\"/g, '"').replace(/(^"|"$)/g, "'"), Ge = Pe.stylize(Ge, "string"));
              }
              return Ge + ": " + ze;
            }
            function me(Pe) {
              return Array.isArray(Pe);
            }
            function pe(Pe) {
              return typeof Pe == "boolean";
            }
            function ye(Pe) {
              return Pe === null;
            }
            function ge(Pe) {
              return typeof Pe == "number";
            }
            function ve(Pe) {
              return typeof Pe == "string";
            }
            function be(Pe) {
              return Pe === void 0;
            }
            function Ce(Pe) {
              return Se(Pe) && Le(Pe) === "[object RegExp]";
            }
            function Se(Pe) {
              return typeof Pe == "object" && Pe !== null;
            }
            function we(Pe) {
              return Se(Pe) && Le(Pe) === "[object Date]";
            }
            function Ee(Pe) {
              return Se(Pe) && (Le(Pe) === "[object Error]" || Pe instanceof Error);
            }
            function xe(Pe) {
              return typeof Pe == "function";
            }
            function Le(Pe) {
              return Object.prototype.toString.call(Pe);
            }
            function Qe(Pe) {
              return Pe < 10 ? "0" + Pe.toString(10) : Pe.toString(10);
            }
            J.debuglog = function(Pe) {
              if (Pe = Pe.toUpperCase(), !ae[Pe]) if (oe.test(Pe)) {
                var $e = te.pid;
                ae[Pe] = function() {
                  var qe = J.format.apply(J, arguments);
                  console.error("%s %d: %s", Pe, $e, qe);
                };
              } else ae[Pe] = function() {
              };
              return ae[Pe];
            }, J.inspect = ce, ce.colors = { bold: [1, 22], italic: [3, 23], underline: [4, 24], inverse: [7, 27], white: [37, 39], grey: [90, 39], black: [30, 39], blue: [34, 39], cyan: [36, 39], green: [32, 39], magenta: [35, 39], red: [31, 39], yellow: [33, 39] }, ce.styles = { special: "cyan", number: "yellow", boolean: "yellow", undefined: "grey", null: "bold", string: "green", date: "magenta", regexp: "red" }, J.types = Z(15724), J.isArray = me, J.isBoolean = pe, J.isNull = ye, J.isNullOrUndefined = function(Pe) {
              return Pe == null;
            }, J.isNumber = ge, J.isString = ve, J.isSymbol = function(Pe) {
              return typeof Pe == "symbol";
            }, J.isUndefined = be, J.isRegExp = Ce, J.types.isRegExp = Ce, J.isObject = Se, J.isDate = we, J.types.isDate = we, J.isError = Ee, J.types.isNativeError = Ee, J.isFunction = xe, J.isPrimitive = function(Pe) {
              return Pe === null || typeof Pe == "boolean" || typeof Pe == "number" || typeof Pe == "string" || typeof Pe == "symbol" || Pe === void 0;
            }, J.isBuffer = Z(44123);
            var ke = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
            function Me(Pe, $e) {
              return Object.prototype.hasOwnProperty.call(Pe, $e);
            }
            J.log = function() {
              var Pe, $e;
              console.log("%s - %s", ($e = [Qe((Pe = /* @__PURE__ */ new Date()).getHours()), Qe(Pe.getMinutes()), Qe(Pe.getSeconds())].join(":"), [Pe.getDate(), ke[Pe.getMonth()], $e].join(" ")), J.format.apply(J, arguments));
            }, J.inherits = Z(28062), J._extend = function(Pe, $e) {
              if (!$e || !Se($e)) return Pe;
              for (var qe = Object.keys($e), Xe = qe.length; Xe--; ) Pe[qe[Xe]] = $e[qe[Xe]];
              return Pe;
            };
            var Re = typeof Symbol < "u" ? Symbol("util.promisify.custom") : void 0;
            function De(Pe, $e) {
              if (!Pe) {
                var qe = new Error("Promise was rejected with a falsy value");
                qe.reason = Pe, Pe = qe;
              }
              return $e(Pe);
            }
            J.promisify = function(Pe) {
              if (typeof Pe != "function") throw new TypeError('The "original" argument must be of type Function');
              if (Re && Pe[Re]) {
                var $e;
                if (typeof ($e = Pe[Re]) != "function") throw new TypeError('The "util.promisify.custom" argument must be of type Function');
                return Object.defineProperty($e, Re, { value: $e, enumerable: !1, writable: !1, configurable: !0 }), $e;
              }
              function $e() {
                for (var qe, Xe, Ve = new Promise(function(ze, We) {
                  qe = ze, Xe = We;
                }), Ze = [], Ge = 0; Ge < arguments.length; Ge++) Ze.push(arguments[Ge]);
                Ze.push(function(ze, We) {
                  ze ? Xe(ze) : qe(We);
                });
                try {
                  Pe.apply(this, Ze);
                } catch (ze) {
                  Xe(ze);
                }
                return Ve;
              }
              return Object.setPrototypeOf($e, Object.getPrototypeOf(Pe)), Re && Object.defineProperty($e, Re, { value: $e, enumerable: !1, writable: !1, configurable: !0 }), Object.defineProperties($e, ne(Pe));
            }, J.promisify.custom = Re, J.callbackify = function(Pe) {
              if (typeof Pe != "function") throw new TypeError('The "original" argument must be of type Function');
              function $e() {
                for (var qe = [], Xe = 0; Xe < arguments.length; Xe++) qe.push(arguments[Xe]);
                var Ve = qe.pop();
                if (typeof Ve != "function") throw new TypeError("The last argument must be of type Function");
                var Ze = this, Ge = function() {
                  return Ve.apply(Ze, arguments);
                };
                Pe.apply(this, qe).then(function(ze) {
                  te.nextTick(Ge.bind(null, null, ze));
                }, function(ze) {
                  te.nextTick(De.bind(null, ze, Ge));
                });
              }
              return Object.setPrototypeOf($e, Object.getPrototypeOf(Pe)), Object.defineProperties($e, ne(Pe)), $e;
            };
          }, 22248: function(G, J, Z) {
            var te = Z(72880);
            G.exports = function(ne) {
              return te("webgl", ne);
            };
          }, 96835: function(G, J, Z) {
            var te = Z(61262), ne = Z(70085), ee = Z(87227), ae = Z(63063), oe = Z(52991), se = ae("Object.prototype.toString"), ce = Z(36912)(), le = typeof globalThis > "u" ? Z.g : globalThis, ue = ne(), de = ae("String.prototype.slice"), fe = Object.getPrototypeOf, he = ae("Array.prototype.indexOf", !0) || function(pe, ye) {
              for (var ge = 0; ge < pe.length; ge += 1) if (pe[ge] === ye) return ge;
              return -1;
            }, me = { __proto__: null };
            te(ue, ce && oe && fe ? function(pe) {
              var ye = new le[pe]();
              if (Symbol.toStringTag in ye) {
                var ge = fe(ye), ve = oe(ge, Symbol.toStringTag);
                if (!ve) {
                  var be = fe(ge);
                  ve = oe(be, Symbol.toStringTag);
                }
                me["$" + pe] = ee(ve.get);
              }
            } : function(pe) {
              var ye = new le[pe](), ge = ye.slice || ye.set;
              ge && (me["$" + pe] = ee(ge));
            }), G.exports = function(pe) {
              if (!pe || typeof pe != "object") return !1;
              if (!ce) {
                var ye = de(se(pe), 8, -1);
                return he(ue, ye) > -1 ? ye : ye === "Object" && function(ge) {
                  var ve = !1;
                  return te(me, function(be, Ce) {
                    if (!ve) try {
                      be(ge), ve = de(Ce, 1);
                    } catch {
                    }
                  }), ve;
                }(pe);
              }
              return oe ? function(ge) {
                var ve = !1;
                return te(me, function(be, Ce) {
                  if (!ve) try {
                    "$" + be(ge) === Ce && (ve = de(Ce, 1));
                  } catch {
                  }
                }), ve;
              }(pe) : null;
            };
          }, 1401: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976), ee = te.instance();
            function ae(fe) {
              this.local = this.regionalOptions[fe || ""] || this.regionalOptions[""];
            }
            ae.prototype = new te.baseCalendar(), ne(ae.prototype, { name: "Chinese", jdEpoch: 17214255e-1, hasYearZero: !1, minMonth: 0, firstMonth: 0, minDay: 1, regionalOptions: { "": { name: "Chinese", epochs: ["BEC", "EC"], monthNumbers: function(fe, he) {
              if (typeof fe == "string") {
                var me = fe.match(se);
                return me ? me[0] : "";
              }
              var pe = this._validateYear(fe), ye = fe.month(), ge = "" + this.toChineseMonth(pe, ye);
              return he && ge.length < 2 && (ge = "0" + ge), this.isIntercalaryMonth(pe, ye) && (ge += "i"), ge;
            }, monthNames: function(fe) {
              if (typeof fe == "string") {
                var he = fe.match(ce);
                return he ? he[0] : "";
              }
              var me = this._validateYear(fe), pe = fe.month(), ye = ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"][this.toChineseMonth(me, pe) - 1];
              return this.isIntercalaryMonth(me, pe) && (ye = "闰" + ye), ye;
            }, monthNamesShort: function(fe) {
              if (typeof fe == "string") {
                var he = fe.match(le);
                return he ? he[0] : "";
              }
              var me = this._validateYear(fe), pe = fe.month(), ye = ["一", "二", "三", "四", "五", "六", "七", "八", "九", "十", "十一", "十二"][this.toChineseMonth(me, pe) - 1];
              return this.isIntercalaryMonth(me, pe) && (ye = "闰" + ye), ye;
            }, parseMonth: function(fe, he) {
              fe = this._validateYear(fe);
              var me, pe = parseInt(he);
              if (isNaN(pe)) he[0] === "闰" && (me = !0, he = he.substring(1)), he[he.length - 1] === "月" && (he = he.substring(0, he.length - 1)), pe = 1 + ["一", "二", "三", "四", "五", "六", "七", "八", "九", "十", "十一", "十二"].indexOf(he);
              else {
                var ye = he[he.length - 1];
                me = ye === "i" || ye === "I";
              }
              return this.toMonthIndex(fe, pe, me);
            }, dayNames: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], dayNamesShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], dayNamesMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"], digits: null, dateFormat: "yyyy/mm/dd", firstDay: 1, isRTL: !1 } }, _validateYear: function(fe, he) {
              if (fe.year && (fe = fe.year()), typeof fe != "number" || fe < 1888 || fe > 2111) throw he.replace(/\{0\}/, this.local.name);
              return fe;
            }, toMonthIndex: function(fe, he, me) {
              var pe = this.intercalaryMonth(fe);
              if (me && he !== pe || he < 1 || he > 12) throw te.local.invalidMonth.replace(/\{0\}/, this.local.name);
              return pe ? !me && he <= pe ? he - 1 : he : he - 1;
            }, toChineseMonth: function(fe, he) {
              fe.year && (he = (fe = fe.year()).month());
              var me = this.intercalaryMonth(fe);
              if (he < 0 || he > (me ? 12 : 11)) throw te.local.invalidMonth.replace(/\{0\}/, this.local.name);
              return me ? he < me ? he + 1 : he : he + 1;
            }, intercalaryMonth: function(fe) {
              return fe = this._validateYear(fe), ue[fe - ue[0]] >> 13;
            }, isIntercalaryMonth: function(fe, he) {
              fe.year && (he = (fe = fe.year()).month());
              var me = this.intercalaryMonth(fe);
              return !!me && me === he;
            }, leapYear: function(fe) {
              return this.intercalaryMonth(fe) !== 0;
            }, weekOfYear: function(fe, he, me) {
              var pe, ye = this._validateYear(fe, te.local.invalidyear), ge = de[ye - de[0]], ve = ge >> 9 & 4095, be = ge >> 5 & 15, Ce = 31 & ge;
              (pe = ee.newDate(ve, be, Ce)).add(4 - (pe.dayOfWeek() || 7), "d");
              var Se = this.toJD(fe, he, me) - pe.toJD();
              return 1 + Math.floor(Se / 7);
            }, monthsInYear: function(fe) {
              return this.leapYear(fe) ? 13 : 12;
            }, daysInMonth: function(fe, he) {
              fe.year && (he = fe.month(), fe = fe.year()), fe = this._validateYear(fe);
              var me = ue[fe - ue[0]];
              if (he > (me >> 13 ? 12 : 11)) throw te.local.invalidMonth.replace(/\{0\}/, this.local.name);
              return me & 1 << 12 - he ? 30 : 29;
            }, weekDay: function(fe, he, me) {
              return (this.dayOfWeek(fe, he, me) || 7) < 6;
            }, toJD: function(fe, he, me) {
              var pe = this._validate(fe, ge, me, te.local.invalidDate);
              fe = this._validateYear(pe.year()), he = pe.month(), me = pe.day();
              var ye = this.isIntercalaryMonth(fe, he), ge = this.toChineseMonth(fe, he), ve = function(be, Ce, Se, we, Ee) {
                var xe, Le, Qe;
                if (typeof be == "object") Le = be, xe = Ce || {};
                else {
                  var ke;
                  if (!(typeof be == "number" && be >= 1888 && be <= 2111)) throw new Error("Lunar year outside range 1888-2111");
                  if (!(typeof Ce == "number" && Ce >= 1 && Ce <= 12)) throw new Error("Lunar month outside range 1 - 12");
                  if (!(typeof Se == "number" && Se >= 1 && Se <= 30)) throw new Error("Lunar day outside range 1 - 30");
                  typeof we == "object" ? (ke = !1, xe = we) : (ke = !!we, xe = {}), Le = { year: be, month: Ce, day: Se, isIntercalary: ke };
                }
                Qe = Le.day - 1;
                var Me, Re = ue[Le.year - ue[0]], De = Re >> 13;
                Me = De && (Le.month > De || Le.isIntercalary) ? Le.month : Le.month - 1;
                for (var Pe = 0; Pe < Me; Pe++) Qe += Re & 1 << 12 - Pe ? 30 : 29;
                var $e = de[Le.year - de[0]], qe = new Date($e >> 9 & 4095, ($e >> 5 & 15) - 1, (31 & $e) + Qe);
                return xe.year = qe.getFullYear(), xe.month = 1 + qe.getMonth(), xe.day = qe.getDate(), xe;
              }(fe, ge, me, ye);
              return ee.toJD(ve.year, ve.month, ve.day);
            }, fromJD: function(fe) {
              var he = ee.fromJD(fe), me = function(ye, ge, ve, be) {
                var Ce, Se;
                if (typeof ye == "object") Ce = ye, Se = ge || {};
                else {
                  if (!(typeof ye == "number" && ye >= 1888 && ye <= 2111)) throw new Error("Solar year outside range 1888-2111");
                  if (!(typeof ge == "number" && ge >= 1 && ge <= 12)) throw new Error("Solar month outside range 1 - 12");
                  if (!(typeof ve == "number" && ve >= 1 && ve <= 31)) throw new Error("Solar day outside range 1 - 31");
                  Ce = { year: ye, month: ge, day: ve }, Se = {};
                }
                var we = de[Ce.year - de[0]], Ee = Ce.year << 9 | Ce.month << 5 | Ce.day;
                Se.year = Ee >= we ? Ce.year : Ce.year - 1, we = de[Se.year - de[0]];
                var xe, Le = new Date(we >> 9 & 4095, (we >> 5 & 15) - 1, 31 & we), Qe = new Date(Ce.year, Ce.month - 1, Ce.day);
                xe = Math.round((Qe - Le) / 864e5);
                var ke, Me = ue[Se.year - ue[0]];
                for (ke = 0; ke < 13; ke++) {
                  var Re = Me & 1 << 12 - ke ? 30 : 29;
                  if (xe < Re) break;
                  xe -= Re;
                }
                var De = Me >> 13;
                return !De || ke < De ? (Se.isIntercalary = !1, Se.month = 1 + ke) : ke === De ? (Se.isIntercalary = !0, Se.month = ke) : (Se.isIntercalary = !1, Se.month = ke), Se.day = 1 + xe, Se;
              }(he.year(), he.month(), he.day()), pe = this.toMonthIndex(me.year, me.month, me.isIntercalary);
              return this.newDate(me.year, pe, me.day);
            }, fromString: function(fe) {
              var he = fe.match(oe), me = this._validateYear(+he[1]), pe = +he[2], ye = !!he[3], ge = this.toMonthIndex(me, pe, ye), ve = +he[4];
              return this.newDate(me, ge, ve);
            }, add: function(fe, he, me) {
              var pe = fe.year(), ye = fe.month(), ge = this.isIntercalaryMonth(pe, ye), ve = this.toChineseMonth(pe, ye), be = Object.getPrototypeOf(ae.prototype).add.call(this, fe, he, me);
              if (me === "y") {
                var Ce = be.year(), Se = be.month(), we = this.isIntercalaryMonth(Ce, ve), Ee = ge && we ? this.toMonthIndex(Ce, ve, !0) : this.toMonthIndex(Ce, ve, !1);
                Ee !== Se && be.month(Ee);
              }
              return be;
            } });
            var oe = /^\s*(-?\d\d\d\d|\d\d)[-/](\d?\d)([iI]?)[-/](\d?\d)/m, se = /^\d?\d[iI]?/m, ce = /^闰?十?[一二三四五六七八九]?月/m, le = /^闰?十?[一二三四五六七八九]?/m;
            te.calendars.chinese = ae;
            var ue = [1887, 5780, 5802, 19157, 2742, 50359, 1198, 2646, 46378, 7466, 3412, 30122, 5482, 67949, 2396, 5294, 43597, 6732, 6954, 36181, 2772, 4954, 18781, 2396, 54427, 5274, 6730, 47781, 5800, 6868, 21210, 4790, 59703, 2350, 5270, 46667, 3402, 3496, 38325, 1388, 4782, 18735, 2350, 52374, 6804, 7498, 44457, 2906, 1388, 29294, 4700, 63789, 6442, 6804, 56138, 5802, 2772, 38235, 1210, 4698, 22827, 5418, 63125, 3476, 5802, 43701, 2484, 5302, 27223, 2646, 70954, 7466, 3412, 54698, 5482, 2412, 38062, 5294, 2636, 32038, 6954, 60245, 2772, 4826, 43357, 2394, 5274, 39501, 6730, 72357, 5800, 5844, 53978, 4790, 2358, 38039, 5270, 87627, 3402, 3496, 54708, 5484, 4782, 43311, 2350, 3222, 27978, 7498, 68965, 2904, 5484, 45677, 4700, 6444, 39573, 6804, 6986, 19285, 2772, 62811, 1210, 4698, 47403, 5418, 5780, 38570, 5546, 76469, 2420, 5302, 51799, 2646, 5414, 36501, 3412, 5546, 18869, 2412, 54446, 5276, 6732, 48422, 6822, 2900, 28010, 4826, 92509, 2394, 5274, 55883, 6730, 6820, 47956, 5812, 2778, 18779, 2358, 62615, 5270, 5450, 46757, 3492, 5556, 27318, 4718, 67887, 2350, 3222, 52554, 7498, 3428, 38252, 5468, 4700, 31022, 6444, 64149, 6804, 6986, 43861, 2772, 5338, 35421, 2650, 70955, 5418, 5780, 54954, 5546, 2740, 38074, 5302, 2646, 29991, 3366, 61011, 3412, 5546, 43445, 2412, 5294, 35406, 6732, 72998, 6820, 6996, 52586, 2778, 2396, 38045, 5274, 6698, 23333, 6820, 64338, 5812, 2746, 43355, 2358, 5270, 39499, 5450, 79525, 3492, 5548], de = [1887, 966732, 967231, 967733, 968265, 968766, 969297, 969798, 970298, 970829, 971330, 971830, 972362, 972863, 973395, 973896, 974397, 974928, 975428, 975929, 976461, 976962, 977462, 977994, 978494, 979026, 979526, 980026, 980558, 981059, 981559, 982091, 982593, 983124, 983624, 984124, 984656, 985157, 985656, 986189, 986690, 987191, 987722, 988222, 988753, 989254, 989754, 990286, 990788, 991288, 991819, 992319, 992851, 993352, 993851, 994383, 994885, 995385, 995917, 996418, 996918, 997450, 997949, 998481, 998982, 999483, 1000014, 1000515, 1001016, 1001548, 1002047, 1002578, 1003080, 1003580, 1004111, 1004613, 1005113, 1005645, 1006146, 1006645, 1007177, 1007678, 1008209, 1008710, 1009211, 1009743, 1010243, 1010743, 1011275, 1011775, 1012306, 1012807, 1013308, 1013840, 1014341, 1014841, 1015373, 1015874, 1016404, 1016905, 1017405, 1017937, 1018438, 1018939, 1019471, 1019972, 1020471, 1021002, 1021503, 1022035, 1022535, 1023036, 1023568, 1024069, 1024568, 1025100, 1025601, 1026102, 1026633, 1027133, 1027666, 1028167, 1028666, 1029198, 1029699, 1030199, 1030730, 1031231, 1031763, 1032264, 1032764, 1033296, 1033797, 1034297, 1034828, 1035329, 1035830, 1036362, 1036861, 1037393, 1037894, 1038394, 1038925, 1039427, 1039927, 1040459, 1040959, 1041491, 1041992, 1042492, 1043023, 1043524, 1044024, 1044556, 1045057, 1045558, 1046090, 1046590, 1047121, 1047622, 1048122, 1048654, 1049154, 1049655, 1050187, 1050689, 1051219, 1051720, 1052220, 1052751, 1053252, 1053752, 1054284, 1054786, 1055285, 1055817, 1056317, 1056849, 1057349, 1057850, 1058382, 1058883, 1059383, 1059915, 1060415, 1060947, 1061447, 1061947, 1062479, 1062981, 1063480, 1064012, 1064514, 1065014, 1065545, 1066045, 1066577, 1067078, 1067578, 1068110, 1068611, 1069112, 1069642, 1070142, 1070674, 1071175, 1071675, 1072207, 1072709, 1073209, 1073740, 1074241, 1074741, 1075273, 1075773, 1076305, 1076807, 1077308, 1077839, 1078340, 1078840, 1079372, 1079871, 1080403, 1080904];
          }, 72210: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(ae) {
              this.local = this.regionalOptions[ae || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Coptic", jdEpoch: 18250295e-1, daysPerMonth: [30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 5], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Coptic", epochs: ["BAM", "AM"], monthNames: ["Thout", "Paopi", "Hathor", "Koiak", "Tobi", "Meshir", "Paremhat", "Paremoude", "Pashons", "Paoni", "Epip", "Mesori", "Pi Kogi Enavot"], monthNamesShort: ["Tho", "Pao", "Hath", "Koi", "Tob", "Mesh", "Pat", "Pad", "Pash", "Pao", "Epi", "Meso", "PiK"], dayNames: ["Tkyriaka", "Pesnau", "Pshoment", "Peftoou", "Ptiou", "Psoou", "Psabbaton"], dayNamesShort: ["Tky", "Pes", "Psh", "Pef", "Pti", "Pso", "Psa"], dayNamesMin: ["Tk", "Pes", "Psh", "Pef", "Pt", "Pso", "Psa"], digits: null, dateFormat: "dd/mm/yyyy", firstDay: 0, isRTL: !1 } }, leapYear: function(ae) {
              var oe = this._validate(ae, this.minMonth, this.minDay, te.local.invalidYear);
              return (ae = oe.year() + (oe.year() < 0 ? 1 : 0)) % 4 == 3 || ae % 4 == -1;
            }, monthsInYear: function(ae) {
              return this._validate(ae, this.minMonth, this.minDay, te.local.invalidYear || te.regionalOptions[""].invalidYear), 13;
            }, weekOfYear: function(ae, oe, se) {
              var ce = this.newDate(ae, oe, se);
              return ce.add(-ce.dayOfWeek(), "d"), Math.floor((ce.dayOfYear() - 1) / 7) + 1;
            }, daysInMonth: function(ae, oe) {
              var se = this._validate(ae, oe, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[se.month() - 1] + (se.month() === 13 && this.leapYear(se.year()) ? 1 : 0);
            }, weekDay: function(ae, oe, se) {
              return (this.dayOfWeek(ae, oe, se) || 7) < 6;
            }, toJD: function(ae, oe, se) {
              var ce = this._validate(ae, oe, se, te.local.invalidDate);
              return (ae = ce.year()) < 0 && ae++, ce.day() + 30 * (ce.month() - 1) + 365 * (ae - 1) + Math.floor(ae / 4) + this.jdEpoch - 1;
            }, fromJD: function(ae) {
              var oe = Math.floor(ae) + 0.5 - this.jdEpoch, se = Math.floor((oe - Math.floor((oe + 366) / 1461)) / 365) + 1;
              se <= 0 && se--, oe = Math.floor(ae) + 0.5 - this.newDate(se, 1, 1).toJD();
              var ce = Math.floor(oe / 30) + 1, le = oe - 30 * (ce - 1) + 1;
              return this.newDate(se, ce, le);
            } }), te.calendars.coptic = ee;
          }, 28569: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(oe) {
              this.local = this.regionalOptions[oe || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Discworld", jdEpoch: 17214255e-1, daysPerMonth: [16, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Discworld", epochs: ["BUC", "UC"], monthNames: ["Ick", "Offle", "February", "March", "April", "May", "June", "Grune", "August", "Spune", "Sektober", "Ember", "December"], monthNamesShort: ["Ick", "Off", "Feb", "Mar", "Apr", "May", "Jun", "Gru", "Aug", "Spu", "Sek", "Emb", "Dec"], dayNames: ["Sunday", "Octeday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], dayNamesShort: ["Sun", "Oct", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], dayNamesMin: ["Su", "Oc", "Mo", "Tu", "We", "Th", "Fr", "Sa"], digits: null, dateFormat: "yyyy/mm/dd", firstDay: 2, isRTL: !1 } }, leapYear: function(oe) {
              return this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear), !1;
            }, monthsInYear: function(oe) {
              return this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear), 13;
            }, daysInYear: function(oe) {
              return this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear), 400;
            }, weekOfYear: function(oe, se, ce) {
              var le = this.newDate(oe, se, ce);
              return le.add(-le.dayOfWeek(), "d"), Math.floor((le.dayOfYear() - 1) / 8) + 1;
            }, daysInMonth: function(oe, se) {
              var ce = this._validate(oe, se, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[ce.month() - 1];
            }, daysInWeek: function() {
              return 8;
            }, dayOfWeek: function(oe, se, ce) {
              return (this._validate(oe, se, ce, te.local.invalidDate).day() + 1) % 8;
            }, weekDay: function(oe, se, ce) {
              var le = this.dayOfWeek(oe, se, ce);
              return le >= 2 && le <= 6;
            }, extraInfo: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate);
              return { century: ae[Math.floor((le.year() - 1) / 100) + 1] || "" };
            }, toJD: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate);
              return oe = le.year() + (le.year() < 0 ? 1 : 0), se = le.month(), (ce = le.day()) + (se > 1 ? 16 : 0) + (se > 2 ? 32 * (se - 2) : 0) + 400 * (oe - 1) + this.jdEpoch - 1;
            }, fromJD: function(oe) {
              oe = Math.floor(oe + 0.5) - Math.floor(this.jdEpoch) - 1;
              var se = Math.floor(oe / 400) + 1;
              oe -= 400 * (se - 1), oe += oe > 15 ? 16 : 0;
              var ce = Math.floor(oe / 32) + 1, le = oe - 32 * (ce - 1) + 1;
              return this.newDate(se <= 0 ? se - 1 : se, ce, le);
            } });
            var ae = { 20: "Fruitbat", 21: "Anchovy" };
            te.calendars.discworld = ee;
          }, 81133: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(ae) {
              this.local = this.regionalOptions[ae || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Ethiopian", jdEpoch: 17242205e-1, daysPerMonth: [30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 5], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Ethiopian", epochs: ["BEE", "EE"], monthNames: ["Meskerem", "Tikemet", "Hidar", "Tahesas", "Tir", "Yekatit", "Megabit", "Miazia", "Genbot", "Sene", "Hamle", "Nehase", "Pagume"], monthNamesShort: ["Mes", "Tik", "Hid", "Tah", "Tir", "Yek", "Meg", "Mia", "Gen", "Sen", "Ham", "Neh", "Pag"], dayNames: ["Ehud", "Segno", "Maksegno", "Irob", "Hamus", "Arb", "Kidame"], dayNamesShort: ["Ehu", "Seg", "Mak", "Iro", "Ham", "Arb", "Kid"], dayNamesMin: ["Eh", "Se", "Ma", "Ir", "Ha", "Ar", "Ki"], digits: null, dateFormat: "dd/mm/yyyy", firstDay: 0, isRTL: !1 } }, leapYear: function(ae) {
              var oe = this._validate(ae, this.minMonth, this.minDay, te.local.invalidYear);
              return (ae = oe.year() + (oe.year() < 0 ? 1 : 0)) % 4 == 3 || ae % 4 == -1;
            }, monthsInYear: function(ae) {
              return this._validate(ae, this.minMonth, this.minDay, te.local.invalidYear || te.regionalOptions[""].invalidYear), 13;
            }, weekOfYear: function(ae, oe, se) {
              var ce = this.newDate(ae, oe, se);
              return ce.add(-ce.dayOfWeek(), "d"), Math.floor((ce.dayOfYear() - 1) / 7) + 1;
            }, daysInMonth: function(ae, oe) {
              var se = this._validate(ae, oe, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[se.month() - 1] + (se.month() === 13 && this.leapYear(se.year()) ? 1 : 0);
            }, weekDay: function(ae, oe, se) {
              return (this.dayOfWeek(ae, oe, se) || 7) < 6;
            }, toJD: function(ae, oe, se) {
              var ce = this._validate(ae, oe, se, te.local.invalidDate);
              return (ae = ce.year()) < 0 && ae++, ce.day() + 30 * (ce.month() - 1) + 365 * (ae - 1) + Math.floor(ae / 4) + this.jdEpoch - 1;
            }, fromJD: function(ae) {
              var oe = Math.floor(ae) + 0.5 - this.jdEpoch, se = Math.floor((oe - Math.floor((oe + 366) / 1461)) / 365) + 1;
              se <= 0 && se--, oe = Math.floor(ae) + 0.5 - this.newDate(se, 1, 1).toJD();
              var ce = Math.floor(oe / 30) + 1, le = oe - 30 * (ce - 1) + 1;
              return this.newDate(se, ce, le);
            } }), te.calendars.ethiopian = ee;
          }, 78295: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(oe) {
              this.local = this.regionalOptions[oe || ""] || this.regionalOptions[""];
            }
            function ae(oe, se) {
              return oe - se * Math.floor(oe / se);
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Hebrew", jdEpoch: 347995.5, daysPerMonth: [30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 29], hasYearZero: !1, minMonth: 1, firstMonth: 7, minDay: 1, regionalOptions: { "": { name: "Hebrew", epochs: ["BAM", "AM"], monthNames: ["Nisan", "Iyar", "Sivan", "Tammuz", "Av", "Elul", "Tishrei", "Cheshvan", "Kislev", "Tevet", "Shevat", "Adar", "Adar II"], monthNamesShort: ["Nis", "Iya", "Siv", "Tam", "Av", "Elu", "Tis", "Che", "Kis", "Tev", "She", "Ada", "Ad2"], dayNames: ["Yom Rishon", "Yom Sheni", "Yom Shlishi", "Yom Revi'i", "Yom Chamishi", "Yom Shishi", "Yom Shabbat"], dayNamesShort: ["Ris", "She", "Shl", "Rev", "Cha", "Shi", "Sha"], dayNamesMin: ["Ri", "She", "Shl", "Re", "Ch", "Shi", "Sha"], digits: null, dateFormat: "dd/mm/yyyy", firstDay: 0, isRTL: !1 } }, leapYear: function(oe) {
              var se = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear);
              return this._leapYear(se.year());
            }, _leapYear: function(oe) {
              return ae(7 * (oe = oe < 0 ? oe + 1 : oe) + 1, 19) < 7;
            }, monthsInYear: function(oe) {
              return this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear), this._leapYear(oe.year ? oe.year() : oe) ? 13 : 12;
            }, weekOfYear: function(oe, se, ce) {
              var le = this.newDate(oe, se, ce);
              return le.add(-le.dayOfWeek(), "d"), Math.floor((le.dayOfYear() - 1) / 7) + 1;
            }, daysInYear: function(oe) {
              return oe = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear).year(), this.toJD(oe === -1 ? 1 : oe + 1, 7, 1) - this.toJD(oe, 7, 1);
            }, daysInMonth: function(oe, se) {
              return oe.year && (se = oe.month(), oe = oe.year()), this._validate(oe, se, this.minDay, te.local.invalidMonth), se === 12 && this.leapYear(oe) || se === 8 && ae(this.daysInYear(oe), 10) === 5 ? 30 : se === 9 && ae(this.daysInYear(oe), 10) === 3 ? 29 : this.daysPerMonth[se - 1];
            }, weekDay: function(oe, se, ce) {
              return this.dayOfWeek(oe, se, ce) !== 6;
            }, extraInfo: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate);
              return { yearType: (this.leapYear(le) ? "embolismic" : "common") + " " + ["deficient", "regular", "complete"][this.daysInYear(le) % 10 - 3] };
            }, toJD: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate);
              oe = le.year(), se = le.month(), ce = le.day();
              var ue = oe <= 0 ? oe + 1 : oe, de = this.jdEpoch + this._delay1(ue) + this._delay2(ue) + ce + 1;
              if (se < 7) {
                for (var fe = 7; fe <= this.monthsInYear(oe); fe++) de += this.daysInMonth(oe, fe);
                for (fe = 1; fe < se; fe++) de += this.daysInMonth(oe, fe);
              } else for (fe = 7; fe < se; fe++) de += this.daysInMonth(oe, fe);
              return de;
            }, _delay1: function(oe) {
              var se = Math.floor((235 * oe - 234) / 19), ce = 12084 + 13753 * se, le = 29 * se + Math.floor(ce / 25920);
              return ae(3 * (le + 1), 7) < 3 && le++, le;
            }, _delay2: function(oe) {
              var se = this._delay1(oe - 1), ce = this._delay1(oe);
              return this._delay1(oe + 1) - ce == 356 ? 2 : ce - se == 382 ? 1 : 0;
            }, fromJD: function(oe) {
              oe = Math.floor(oe) + 0.5;
              for (var se = Math.floor(98496 * (oe - this.jdEpoch) / 35975351) - 1; oe >= this.toJD(se === -1 ? 1 : se + 1, 7, 1); ) se++;
              for (var ce = oe < this.toJD(se, 1, 1) ? 7 : 1; oe > this.toJD(se, ce, this.daysInMonth(se, ce)); ) ce++;
              var le = oe - this.toJD(se, ce, 1) + 1;
              return this.newDate(se, ce, le);
            } }), te.calendars.hebrew = ee;
          }, 25512: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(ae) {
              this.local = this.regionalOptions[ae || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Islamic", jdEpoch: 19484395e-1, daysPerMonth: [30, 29, 30, 29, 30, 29, 30, 29, 30, 29, 30, 29], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Islamic", epochs: ["BH", "AH"], monthNames: ["Muharram", "Safar", "Rabi' al-awwal", "Rabi' al-thani", "Jumada al-awwal", "Jumada al-thani", "Rajab", "Sha'aban", "Ramadan", "Shawwal", "Dhu al-Qi'dah", "Dhu al-Hijjah"], monthNamesShort: ["Muh", "Saf", "Rab1", "Rab2", "Jum1", "Jum2", "Raj", "Sha'", "Ram", "Shaw", "DhuQ", "DhuH"], dayNames: ["Yawm al-ahad", "Yawm al-ithnayn", "Yawm ath-thulaathaa'", "Yawm al-arbi'aa'", "Yawm al-khamīs", "Yawm al-jum'a", "Yawm as-sabt"], dayNamesShort: ["Aha", "Ith", "Thu", "Arb", "Kha", "Jum", "Sab"], dayNamesMin: ["Ah", "It", "Th", "Ar", "Kh", "Ju", "Sa"], digits: null, dateFormat: "yyyy/mm/dd", firstDay: 6, isRTL: !1 } }, leapYear: function(ae) {
              return (11 * this._validate(ae, this.minMonth, this.minDay, te.local.invalidYear).year() + 14) % 30 < 11;
            }, weekOfYear: function(ae, oe, se) {
              var ce = this.newDate(ae, oe, se);
              return ce.add(-ce.dayOfWeek(), "d"), Math.floor((ce.dayOfYear() - 1) / 7) + 1;
            }, daysInYear: function(ae) {
              return this.leapYear(ae) ? 355 : 354;
            }, daysInMonth: function(ae, oe) {
              var se = this._validate(ae, oe, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[se.month() - 1] + (se.month() === 12 && this.leapYear(se.year()) ? 1 : 0);
            }, weekDay: function(ae, oe, se) {
              return this.dayOfWeek(ae, oe, se) !== 5;
            }, toJD: function(ae, oe, se) {
              var ce = this._validate(ae, oe, se, te.local.invalidDate);
              return ae = ce.year(), oe = ce.month(), ae = ae <= 0 ? ae + 1 : ae, (se = ce.day()) + Math.ceil(29.5 * (oe - 1)) + 354 * (ae - 1) + Math.floor((3 + 11 * ae) / 30) + this.jdEpoch - 1;
            }, fromJD: function(ae) {
              ae = Math.floor(ae) + 0.5;
              var oe = Math.floor((30 * (ae - this.jdEpoch) + 10646) / 10631);
              oe = oe <= 0 ? oe - 1 : oe;
              var se = Math.min(12, Math.ceil((ae - 29 - this.toJD(oe, 1, 1)) / 29.5) + 1), ce = ae - this.toJD(oe, se, 1) + 1;
              return this.newDate(oe, se, ce);
            } }), te.calendars.islamic = ee;
          }, 42645: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(ae) {
              this.local = this.regionalOptions[ae || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Julian", jdEpoch: 17214235e-1, daysPerMonth: [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Julian", epochs: ["BC", "AD"], monthNames: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"], monthNamesShort: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"], dayNames: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], dayNamesShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], dayNamesMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"], digits: null, dateFormat: "mm/dd/yyyy", firstDay: 0, isRTL: !1 } }, leapYear: function(ae) {
              var oe = this._validate(ae, this.minMonth, this.minDay, te.local.invalidYear);
              return (ae = oe.year() < 0 ? oe.year() + 1 : oe.year()) % 4 == 0;
            }, weekOfYear: function(ae, oe, se) {
              var ce = this.newDate(ae, oe, se);
              return ce.add(4 - (ce.dayOfWeek() || 7), "d"), Math.floor((ce.dayOfYear() - 1) / 7) + 1;
            }, daysInMonth: function(ae, oe) {
              var se = this._validate(ae, oe, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[se.month() - 1] + (se.month() === 2 && this.leapYear(se.year()) ? 1 : 0);
            }, weekDay: function(ae, oe, se) {
              return (this.dayOfWeek(ae, oe, se) || 7) < 6;
            }, toJD: function(ae, oe, se) {
              var ce = this._validate(ae, oe, se, te.local.invalidDate);
              return ae = ce.year(), oe = ce.month(), se = ce.day(), ae < 0 && ae++, oe <= 2 && (ae--, oe += 12), Math.floor(365.25 * (ae + 4716)) + Math.floor(30.6001 * (oe + 1)) + se - 1524.5;
            }, fromJD: function(ae) {
              var oe = Math.floor(ae + 0.5) + 1524, se = Math.floor((oe - 122.1) / 365.25), ce = Math.floor(365.25 * se), le = Math.floor((oe - ce) / 30.6001), ue = le - Math.floor(le < 14 ? 1 : 13), de = se - Math.floor(ue > 2 ? 4716 : 4715), fe = oe - ce - Math.floor(30.6001 * le);
              return de <= 0 && de--, this.newDate(de, ue, fe);
            } }), te.calendars.julian = ee;
          }, 62324: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(se) {
              this.local = this.regionalOptions[se || ""] || this.regionalOptions[""];
            }
            function ae(se, ce) {
              return se - ce * Math.floor(se / ce);
            }
            function oe(se, ce) {
              return ae(se - 1, ce) + 1;
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Mayan", jdEpoch: 584282.5, hasYearZero: !0, minMonth: 0, firstMonth: 0, minDay: 0, regionalOptions: { "": { name: "Mayan", epochs: ["", ""], monthNames: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17"], monthNamesShort: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17"], dayNames: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19"], dayNamesShort: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19"], dayNamesMin: ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19"], digits: null, dateFormat: "YYYY.m.d", firstDay: 0, isRTL: !1, haabMonths: ["Pop", "Uo", "Zip", "Zotz", "Tzec", "Xul", "Yaxkin", "Mol", "Chen", "Yax", "Zac", "Ceh", "Mac", "Kankin", "Muan", "Pax", "Kayab", "Cumku", "Uayeb"], tzolkinMonths: ["Imix", "Ik", "Akbal", "Kan", "Chicchan", "Cimi", "Manik", "Lamat", "Muluc", "Oc", "Chuen", "Eb", "Ben", "Ix", "Men", "Cib", "Caban", "Etznab", "Cauac", "Ahau"] } }, leapYear: function(se) {
              return this._validate(se, this.minMonth, this.minDay, te.local.invalidYear), !1;
            }, formatYear: function(se) {
              se = this._validate(se, this.minMonth, this.minDay, te.local.invalidYear).year();
              var ce = Math.floor(se / 400);
              return se %= 400, se += se < 0 ? 400 : 0, ce + "." + Math.floor(se / 20) + "." + se % 20;
            }, forYear: function(se) {
              if ((se = se.split(".")).length < 3) throw "Invalid Mayan year";
              for (var ce = 0, le = 0; le < se.length; le++) {
                var ue = parseInt(se[le], 10);
                if (Math.abs(ue) > 19 || le > 0 && ue < 0) throw "Invalid Mayan year";
                ce = 20 * ce + ue;
              }
              return ce;
            }, monthsInYear: function(se) {
              return this._validate(se, this.minMonth, this.minDay, te.local.invalidYear), 18;
            }, weekOfYear: function(se, ce, le) {
              return this._validate(se, ce, le, te.local.invalidDate), 0;
            }, daysInYear: function(se) {
              return this._validate(se, this.minMonth, this.minDay, te.local.invalidYear), 360;
            }, daysInMonth: function(se, ce) {
              return this._validate(se, ce, this.minDay, te.local.invalidMonth), 20;
            }, daysInWeek: function() {
              return 5;
            }, dayOfWeek: function(se, ce, le) {
              return this._validate(se, ce, le, te.local.invalidDate).day();
            }, weekDay: function(se, ce, le) {
              return this._validate(se, ce, le, te.local.invalidDate), !0;
            }, extraInfo: function(se, ce, le) {
              var ue = this._validate(se, ce, le, te.local.invalidDate).toJD(), de = this._toHaab(ue), fe = this._toTzolkin(ue);
              return { haabMonthName: this.local.haabMonths[de[0] - 1], haabMonth: de[0], haabDay: de[1], tzolkinDayName: this.local.tzolkinMonths[fe[0] - 1], tzolkinDay: fe[0], tzolkinTrecena: fe[1] };
            }, _toHaab: function(se) {
              var ce = ae(8 + (se -= this.jdEpoch) + 340, 365);
              return [Math.floor(ce / 20) + 1, ae(ce, 20)];
            }, _toTzolkin: function(se) {
              return [oe(20 + (se -= this.jdEpoch), 20), oe(se + 4, 13)];
            }, toJD: function(se, ce, le) {
              var ue = this._validate(se, ce, le, te.local.invalidDate);
              return ue.day() + 20 * ue.month() + 360 * ue.year() + this.jdEpoch;
            }, fromJD: function(se) {
              se = Math.floor(se) + 0.5 - this.jdEpoch;
              var ce = Math.floor(se / 360);
              se %= 360, se += se < 0 ? 360 : 0;
              var le = Math.floor(se / 20), ue = se % 20;
              return this.newDate(ce, le, ue);
            } }), te.calendars.mayan = ee;
          }, 91662: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(oe) {
              this.local = this.regionalOptions[oe || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar();
            var ae = te.instance("gregorian");
            ne(ee.prototype, { name: "Nanakshahi", jdEpoch: 22576735e-1, daysPerMonth: [31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 30, 30], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Nanakshahi", epochs: ["BN", "AN"], monthNames: ["Chet", "Vaisakh", "Jeth", "Harh", "Sawan", "Bhadon", "Assu", "Katak", "Maghar", "Poh", "Magh", "Phagun"], monthNamesShort: ["Che", "Vai", "Jet", "Har", "Saw", "Bha", "Ass", "Kat", "Mgr", "Poh", "Mgh", "Pha"], dayNames: ["Somvaar", "Mangalvar", "Budhvaar", "Veervaar", "Shukarvaar", "Sanicharvaar", "Etvaar"], dayNamesShort: ["Som", "Mangal", "Budh", "Veer", "Shukar", "Sanichar", "Et"], dayNamesMin: ["So", "Ma", "Bu", "Ve", "Sh", "Sa", "Et"], digits: null, dateFormat: "dd-mm-yyyy", firstDay: 0, isRTL: !1 } }, leapYear: function(oe) {
              var se = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear || te.regionalOptions[""].invalidYear);
              return ae.leapYear(se.year() + (se.year() < 1 ? 1 : 0) + 1469);
            }, weekOfYear: function(oe, se, ce) {
              var le = this.newDate(oe, se, ce);
              return le.add(1 - (le.dayOfWeek() || 7), "d"), Math.floor((le.dayOfYear() - 1) / 7) + 1;
            }, daysInMonth: function(oe, se) {
              var ce = this._validate(oe, se, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[ce.month() - 1] + (ce.month() === 12 && this.leapYear(ce.year()) ? 1 : 0);
            }, weekDay: function(oe, se, ce) {
              return (this.dayOfWeek(oe, se, ce) || 7) < 6;
            }, toJD: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidMonth);
              (oe = le.year()) < 0 && oe++;
              for (var ue = le.day(), de = 1; de < le.month(); de++) ue += this.daysPerMonth[de - 1];
              return ue + ae.toJD(oe + 1468, 3, 13);
            }, fromJD: function(oe) {
              oe = Math.floor(oe + 0.5);
              for (var se = Math.floor((oe - (this.jdEpoch - 1)) / 366); oe >= this.toJD(se + 1, 1, 1); ) se++;
              for (var ce = oe - Math.floor(this.toJD(se, 1, 1) + 0.5) + 1, le = 1; ce > this.daysInMonth(se, le); ) ce -= this.daysInMonth(se, le), le++;
              return this.newDate(se, le, ce);
            } }), te.calendars.nanakshahi = ee;
          }, 66445: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(ae) {
              this.local = this.regionalOptions[ae || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Nepali", jdEpoch: 17007095e-1, daysPerMonth: [31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, daysPerYear: 365, regionalOptions: { "": { name: "Nepali", epochs: ["BBS", "ABS"], monthNames: ["Baisakh", "Jestha", "Ashadh", "Shrawan", "Bhadra", "Ashwin", "Kartik", "Mangsir", "Paush", "Mangh", "Falgun", "Chaitra"], monthNamesShort: ["Bai", "Je", "As", "Shra", "Bha", "Ash", "Kar", "Mang", "Pau", "Ma", "Fal", "Chai"], dayNames: ["Aaitabaar", "Sombaar", "Manglbaar", "Budhabaar", "Bihibaar", "Shukrabaar", "Shanibaar"], dayNamesShort: ["Aaita", "Som", "Mangl", "Budha", "Bihi", "Shukra", "Shani"], dayNamesMin: ["Aai", "So", "Man", "Bu", "Bi", "Shu", "Sha"], digits: null, dateFormat: "dd/mm/yyyy", firstDay: 1, isRTL: !1 } }, leapYear: function(ae) {
              return this.daysInYear(ae) !== this.daysPerYear;
            }, weekOfYear: function(ae, oe, se) {
              var ce = this.newDate(ae, oe, se);
              return ce.add(-ce.dayOfWeek(), "d"), Math.floor((ce.dayOfYear() - 1) / 7) + 1;
            }, daysInYear: function(ae) {
              if (ae = this._validate(ae, this.minMonth, this.minDay, te.local.invalidYear).year(), this.NEPALI_CALENDAR_DATA[ae] === void 0) return this.daysPerYear;
              for (var oe = 0, se = this.minMonth; se <= 12; se++) oe += this.NEPALI_CALENDAR_DATA[ae][se];
              return oe;
            }, daysInMonth: function(ae, oe) {
              return ae.year && (oe = ae.month(), ae = ae.year()), this._validate(ae, oe, this.minDay, te.local.invalidMonth), this.NEPALI_CALENDAR_DATA[ae] === void 0 ? this.daysPerMonth[oe - 1] : this.NEPALI_CALENDAR_DATA[ae][oe];
            }, weekDay: function(ae, oe, se) {
              return this.dayOfWeek(ae, oe, se) !== 6;
            }, toJD: function(ae, oe, se) {
              var ce = this._validate(ae, oe, se, te.local.invalidDate);
              ae = ce.year(), oe = ce.month(), se = ce.day();
              var le = te.instance(), ue = 0, de = oe, fe = ae;
              this._createMissingCalendarData(ae);
              var he = ae - (de > 9 || de === 9 && se >= this.NEPALI_CALENDAR_DATA[fe][0] ? 56 : 57);
              for (oe !== 9 && (ue = se, de--); de !== 9; ) de <= 0 && (de = 12, fe--), ue += this.NEPALI_CALENDAR_DATA[fe][de], de--;
              return oe === 9 ? (ue += se - this.NEPALI_CALENDAR_DATA[fe][0]) < 0 && (ue += le.daysInYear(he)) : ue += this.NEPALI_CALENDAR_DATA[fe][9] - this.NEPALI_CALENDAR_DATA[fe][0], le.newDate(he, 1, 1).add(ue, "d").toJD();
            }, fromJD: function(ae) {
              var oe = te.instance().fromJD(ae), se = oe.year(), ce = oe.dayOfYear(), le = se + 56;
              this._createMissingCalendarData(le);
              for (var ue = 9, de = this.NEPALI_CALENDAR_DATA[le][0], fe = this.NEPALI_CALENDAR_DATA[le][ue] - de + 1; ce > fe; ) ++ue > 12 && (ue = 1, le++), fe += this.NEPALI_CALENDAR_DATA[le][ue];
              var he = this.NEPALI_CALENDAR_DATA[le][ue] - (fe - ce);
              return this.newDate(le, ue, he);
            }, _createMissingCalendarData: function(ae) {
              var oe = this.daysPerMonth.slice(0);
              oe.unshift(17);
              for (var se = ae - 1; se < ae + 2; se++) this.NEPALI_CALENDAR_DATA[se] === void 0 && (this.NEPALI_CALENDAR_DATA[se] = oe);
            }, NEPALI_CALENDAR_DATA: { 1970: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1971: [18, 31, 31, 32, 31, 32, 30, 30, 29, 30, 29, 30, 30], 1972: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 30], 1973: [19, 30, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 1974: [19, 31, 31, 32, 30, 31, 31, 30, 29, 30, 29, 30, 30], 1975: [18, 31, 31, 32, 32, 30, 31, 30, 29, 30, 29, 30, 30], 1976: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 1977: [18, 31, 32, 31, 32, 31, 31, 29, 30, 29, 30, 29, 31], 1978: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1979: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 1980: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 1981: [18, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 30, 30], 1982: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1983: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 1984: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 1985: [18, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 30, 30], 1986: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1987: [18, 31, 32, 31, 32, 31, 30, 30, 29, 30, 29, 30, 30], 1988: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 1989: [18, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 1990: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1991: [18, 31, 32, 31, 32, 31, 30, 30, 29, 30, 29, 30, 30], 1992: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 1993: [18, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 1994: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1995: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 30], 1996: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 1997: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1998: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 1999: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2e3: [17, 30, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2001: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2002: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2003: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2004: [17, 30, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2005: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2006: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2007: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2008: [17, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 29, 31], 2009: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2010: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2011: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2012: [17, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 30, 30], 2013: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2014: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2015: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2016: [17, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 30, 30], 2017: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2018: [18, 31, 32, 31, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2019: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2020: [17, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 2021: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2022: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 30], 2023: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2024: [17, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 2025: [18, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2026: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2027: [17, 30, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2028: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2029: [18, 31, 31, 32, 31, 32, 30, 30, 29, 30, 29, 30, 30], 2030: [17, 31, 32, 31, 32, 31, 30, 30, 30, 30, 30, 30, 31], 2031: [17, 31, 32, 31, 32, 31, 31, 31, 31, 31, 31, 31, 31], 2032: [17, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32], 2033: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2034: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2035: [17, 30, 32, 31, 32, 31, 31, 29, 30, 30, 29, 29, 31], 2036: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2037: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2038: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2039: [17, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 30, 30], 2040: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2041: [18, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2042: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2043: [17, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 30, 30], 2044: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2045: [18, 31, 32, 31, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2046: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2047: [17, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 2048: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2049: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 30], 2050: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2051: [17, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 2052: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2053: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 30], 2054: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2055: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 30, 29, 30], 2056: [17, 31, 31, 32, 31, 32, 30, 30, 29, 30, 29, 30, 30], 2057: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2058: [17, 30, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2059: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2060: [17, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2061: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2062: [17, 30, 32, 31, 32, 31, 31, 29, 30, 29, 30, 29, 31], 2063: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2064: [17, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2065: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2066: [17, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 29, 31], 2067: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2068: [17, 31, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2069: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2070: [17, 31, 31, 31, 32, 31, 31, 29, 30, 30, 29, 30, 30], 2071: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2072: [17, 31, 32, 31, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2073: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 31], 2074: [17, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 2075: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2076: [16, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 30], 2077: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 29, 31], 2078: [17, 31, 31, 31, 32, 31, 31, 30, 29, 30, 29, 30, 30], 2079: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 29, 30, 30], 2080: [16, 31, 32, 31, 32, 31, 30, 30, 30, 29, 29, 30, 30], 2081: [17, 31, 31, 32, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2082: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2083: [17, 31, 31, 32, 31, 31, 30, 30, 30, 29, 30, 30, 30], 2084: [17, 31, 31, 32, 31, 31, 30, 30, 30, 29, 30, 30, 30], 2085: [17, 31, 32, 31, 32, 31, 31, 30, 30, 29, 30, 30, 30], 2086: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2087: [16, 31, 31, 32, 31, 31, 31, 30, 30, 29, 30, 30, 30], 2088: [16, 30, 31, 32, 32, 30, 31, 30, 30, 29, 30, 30, 30], 2089: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2090: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2091: [16, 31, 31, 32, 31, 31, 31, 30, 30, 29, 30, 30, 30], 2092: [16, 31, 31, 32, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2093: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2094: [17, 31, 31, 32, 31, 31, 30, 30, 30, 29, 30, 30, 30], 2095: [17, 31, 31, 32, 31, 31, 31, 30, 29, 30, 30, 30, 30], 2096: [17, 30, 31, 32, 32, 31, 30, 30, 29, 30, 29, 30, 30], 2097: [17, 31, 32, 31, 32, 31, 30, 30, 30, 29, 30, 30, 30], 2098: [17, 31, 31, 32, 31, 31, 31, 29, 30, 29, 30, 30, 31], 2099: [17, 31, 31, 32, 31, 31, 31, 30, 29, 29, 30, 30, 30], 2100: [17, 31, 32, 31, 32, 30, 31, 30, 29, 30, 29, 30, 30] } }), te.calendars.nepali = ee;
          }, 50506: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(oe) {
              this.local = this.regionalOptions[oe || ""] || this.regionalOptions[""];
            }
            function ae(oe, se) {
              return oe - se * Math.floor(oe / se);
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "Persian", jdEpoch: 19483205e-1, daysPerMonth: [31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Persian", epochs: ["BP", "AP"], monthNames: ["Farvardin", "Ordibehesht", "Khordad", "Tir", "Mordad", "Shahrivar", "Mehr", "Aban", "Azar", "Day", "Bahman", "Esfand"], monthNamesShort: ["Far", "Ord", "Kho", "Tir", "Mor", "Sha", "Meh", "Aba", "Aza", "Day", "Bah", "Esf"], dayNames: ["Yekshambe", "Doshambe", "Seshambe", "Chæharshambe", "Panjshambe", "Jom'e", "Shambe"], dayNamesShort: ["Yek", "Do", "Se", "Chæ", "Panj", "Jom", "Sha"], dayNamesMin: ["Ye", "Do", "Se", "Ch", "Pa", "Jo", "Sh"], digits: null, dateFormat: "yyyy/mm/dd", firstDay: 6, isRTL: !1 } }, leapYear: function(oe) {
              var se = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear);
              return 682 * ((se.year() - (se.year() > 0 ? 474 : 473)) % 2820 + 474 + 38) % 2816 < 682;
            }, weekOfYear: function(oe, se, ce) {
              var le = this.newDate(oe, se, ce);
              return le.add(-(le.dayOfWeek() + 1) % 7, "d"), Math.floor((le.dayOfYear() - 1) / 7) + 1;
            }, daysInMonth: function(oe, se) {
              var ce = this._validate(oe, se, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[ce.month() - 1] + (ce.month() === 12 && this.leapYear(ce.year()) ? 1 : 0);
            }, weekDay: function(oe, se, ce) {
              return this.dayOfWeek(oe, se, ce) !== 5;
            }, toJD: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate);
              oe = le.year(), se = le.month(), ce = le.day();
              var ue = oe - (oe >= 0 ? 474 : 473), de = 474 + ae(ue, 2820);
              return ce + (se <= 7 ? 31 * (se - 1) : 30 * (se - 1) + 6) + Math.floor((682 * de - 110) / 2816) + 365 * (de - 1) + 1029983 * Math.floor(ue / 2820) + this.jdEpoch - 1;
            }, fromJD: function(oe) {
              var se = (oe = Math.floor(oe) + 0.5) - this.toJD(475, 1, 1), ce = Math.floor(se / 1029983), le = ae(se, 1029983), ue = 2820;
              if (le !== 1029982) {
                var de = Math.floor(le / 366), fe = ae(le, 366);
                ue = Math.floor((2134 * de + 2816 * fe + 2815) / 1028522) + de + 1;
              }
              var he = ue + 2820 * ce + 474;
              he = he <= 0 ? he - 1 : he;
              var me = oe - this.toJD(he, 1, 1) + 1, pe = me <= 186 ? Math.ceil(me / 31) : Math.ceil((me - 6) / 30), ye = oe - this.toJD(he, pe, 1) + 1;
              return this.newDate(he, pe, ye);
            } }), te.calendars.persian = ee, te.calendars.jalali = ee;
          }, 84756: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976), ee = te.instance();
            function ae(oe) {
              this.local = this.regionalOptions[oe || ""] || this.regionalOptions[""];
            }
            ae.prototype = new te.baseCalendar(), ne(ae.prototype, { name: "Taiwan", jdEpoch: 24194025e-1, yearsOffset: 1911, daysPerMonth: [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Taiwan", epochs: ["BROC", "ROC"], monthNames: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"], monthNamesShort: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"], dayNames: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], dayNamesShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], dayNamesMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"], digits: null, dateFormat: "yyyy/mm/dd", firstDay: 1, isRTL: !1 } }, leapYear: function(oe) {
              var se = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear);
              return oe = this._t2gYear(se.year()), ee.leapYear(oe);
            }, weekOfYear: function(oe, se, ce) {
              var le = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear);
              return oe = this._t2gYear(le.year()), ee.weekOfYear(oe, le.month(), le.day());
            }, daysInMonth: function(oe, se) {
              var ce = this._validate(oe, se, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[ce.month() - 1] + (ce.month() === 2 && this.leapYear(ce.year()) ? 1 : 0);
            }, weekDay: function(oe, se, ce) {
              return (this.dayOfWeek(oe, se, ce) || 7) < 6;
            }, toJD: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate);
              return oe = this._t2gYear(le.year()), ee.toJD(oe, le.month(), le.day());
            }, fromJD: function(oe) {
              var se = ee.fromJD(oe), ce = this._g2tYear(se.year());
              return this.newDate(ce, se.month(), se.day());
            }, _t2gYear: function(oe) {
              return oe + this.yearsOffset + (oe >= -this.yearsOffset && oe <= -1 ? 1 : 0);
            }, _g2tYear: function(oe) {
              return oe - this.yearsOffset - (oe >= 1 && oe <= this.yearsOffset ? 1 : 0);
            } }), te.calendars.taiwan = ae;
          }, 41858: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976), ee = te.instance();
            function ae(oe) {
              this.local = this.regionalOptions[oe || ""] || this.regionalOptions[""];
            }
            ae.prototype = new te.baseCalendar(), ne(ae.prototype, { name: "Thai", jdEpoch: 15230985e-1, yearsOffset: 543, daysPerMonth: [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Thai", epochs: ["BBE", "BE"], monthNames: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"], monthNamesShort: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"], dayNames: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], dayNamesShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], dayNamesMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"], digits: null, dateFormat: "dd/mm/yyyy", firstDay: 0, isRTL: !1 } }, leapYear: function(oe) {
              var se = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear);
              return oe = this._t2gYear(se.year()), ee.leapYear(oe);
            }, weekOfYear: function(oe, se, ce) {
              var le = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear);
              return oe = this._t2gYear(le.year()), ee.weekOfYear(oe, le.month(), le.day());
            }, daysInMonth: function(oe, se) {
              var ce = this._validate(oe, se, this.minDay, te.local.invalidMonth);
              return this.daysPerMonth[ce.month() - 1] + (ce.month() === 2 && this.leapYear(ce.year()) ? 1 : 0);
            }, weekDay: function(oe, se, ce) {
              return (this.dayOfWeek(oe, se, ce) || 7) < 6;
            }, toJD: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate);
              return oe = this._t2gYear(le.year()), ee.toJD(oe, le.month(), le.day());
            }, fromJD: function(oe) {
              var se = ee.fromJD(oe), ce = this._g2tYear(se.year());
              return this.newDate(ce, se.month(), se.day());
            }, _t2gYear: function(oe) {
              return oe - this.yearsOffset - (oe >= 1 && oe <= this.yearsOffset ? 1 : 0);
            }, _g2tYear: function(oe) {
              return oe + this.yearsOffset + (oe >= -this.yearsOffset && oe <= -1 ? 1 : 0);
            } }), te.calendars.thai = ae;
          }, 57985: function(G, J, Z) {
            var te = Z(24453), ne = Z(27976);
            function ee(oe) {
              this.local = this.regionalOptions[oe || ""] || this.regionalOptions[""];
            }
            ee.prototype = new te.baseCalendar(), ne(ee.prototype, { name: "UmmAlQura", hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Umm al-Qura", epochs: ["BH", "AH"], monthNames: ["Al-Muharram", "Safar", "Rabi' al-awwal", "Rabi' Al-Thani", "Jumada Al-Awwal", "Jumada Al-Thani", "Rajab", "Sha'aban", "Ramadan", "Shawwal", "Dhu al-Qi'dah", "Dhu al-Hijjah"], monthNamesShort: ["Muh", "Saf", "Rab1", "Rab2", "Jum1", "Jum2", "Raj", "Sha'", "Ram", "Shaw", "DhuQ", "DhuH"], dayNames: ["Yawm al-Ahad", "Yawm al-Ithnain", "Yawm al-Thalāthā’", "Yawm al-Arba‘ā’", "Yawm al-Khamīs", "Yawm al-Jum‘a", "Yawm al-Sabt"], dayNamesMin: ["Ah", "Ith", "Th", "Ar", "Kh", "Ju", "Sa"], digits: null, dateFormat: "yyyy/mm/dd", firstDay: 6, isRTL: !0 } }, leapYear: function(oe) {
              var se = this._validate(oe, this.minMonth, this.minDay, te.local.invalidYear);
              return this.daysInYear(se.year()) === 355;
            }, weekOfYear: function(oe, se, ce) {
              var le = this.newDate(oe, se, ce);
              return le.add(-le.dayOfWeek(), "d"), Math.floor((le.dayOfYear() - 1) / 7) + 1;
            }, daysInYear: function(oe) {
              for (var se = 0, ce = 1; ce <= 12; ce++) se += this.daysInMonth(oe, ce);
              return se;
            }, daysInMonth: function(oe, se) {
              for (var ce = this._validate(oe, se, this.minDay, te.local.invalidMonth).toJD() - 24e5 + 0.5, le = 0, ue = 0; ue < ae.length; ue++) {
                if (ae[ue] > ce) return ae[le] - ae[le - 1];
                le++;
              }
              return 30;
            }, weekDay: function(oe, se, ce) {
              return this.dayOfWeek(oe, se, ce) !== 5;
            }, toJD: function(oe, se, ce) {
              var le = this._validate(oe, se, ce, te.local.invalidDate), ue = 12 * (le.year() - 1) + le.month() - 15292;
              return le.day() + ae[ue - 1] - 1 + 24e5 - 0.5;
            }, fromJD: function(oe) {
              for (var se = oe - 24e5 + 0.5, ce = 0, le = 0; le < ae.length && !(ae[le] > se); le++) ce++;
              var ue = ce + 15292, de = Math.floor((ue - 1) / 12), fe = de + 1, he = ue - 12 * de, me = se - ae[ce - 1] + 1;
              return this.newDate(fe, he, me);
            }, isValid: function(oe, se, ce) {
              var le = te.baseCalendar.prototype.isValid.apply(this, arguments);
              return le && (le = (oe = oe.year != null ? oe.year : oe) >= 1276 && oe <= 1500), le;
            }, _validate: function(oe, se, ce, le) {
              var ue = te.baseCalendar.prototype._validate.apply(this, arguments);
              if (ue.year < 1276 || ue.year > 1500) throw le.replace(/\{0\}/, this.local.name);
              return ue;
            } }), te.calendars.ummalqura = ee;
            var ae = [20, 50, 79, 109, 138, 168, 197, 227, 256, 286, 315, 345, 374, 404, 433, 463, 492, 522, 551, 581, 611, 641, 670, 700, 729, 759, 788, 818, 847, 877, 906, 936, 965, 995, 1024, 1054, 1083, 1113, 1142, 1172, 1201, 1231, 1260, 1290, 1320, 1350, 1379, 1409, 1438, 1468, 1497, 1527, 1556, 1586, 1615, 1645, 1674, 1704, 1733, 1763, 1792, 1822, 1851, 1881, 1910, 1940, 1969, 1999, 2028, 2058, 2087, 2117, 2146, 2176, 2205, 2235, 2264, 2294, 2323, 2353, 2383, 2413, 2442, 2472, 2501, 2531, 2560, 2590, 2619, 2649, 2678, 2708, 2737, 2767, 2796, 2826, 2855, 2885, 2914, 2944, 2973, 3003, 3032, 3062, 3091, 3121, 3150, 3180, 3209, 3239, 3268, 3298, 3327, 3357, 3386, 3416, 3446, 3476, 3505, 3535, 3564, 3594, 3623, 3653, 3682, 3712, 3741, 3771, 3800, 3830, 3859, 3889, 3918, 3948, 3977, 4007, 4036, 4066, 4095, 4125, 4155, 4185, 4214, 4244, 4273, 4303, 4332, 4362, 4391, 4421, 4450, 4480, 4509, 4539, 4568, 4598, 4627, 4657, 4686, 4716, 4745, 4775, 4804, 4834, 4863, 4893, 4922, 4952, 4981, 5011, 5040, 5070, 5099, 5129, 5158, 5188, 5218, 5248, 5277, 5307, 5336, 5366, 5395, 5425, 5454, 5484, 5513, 5543, 5572, 5602, 5631, 5661, 5690, 5720, 5749, 5779, 5808, 5838, 5867, 5897, 5926, 5956, 5985, 6015, 6044, 6074, 6103, 6133, 6162, 6192, 6221, 6251, 6281, 6311, 6340, 6370, 6399, 6429, 6458, 6488, 6517, 6547, 6576, 6606, 6635, 6665, 6694, 6724, 6753, 6783, 6812, 6842, 6871, 6901, 6930, 6960, 6989, 7019, 7048, 7078, 7107, 7137, 7166, 7196, 7225, 7255, 7284, 7314, 7344, 7374, 7403, 7433, 7462, 7492, 7521, 7551, 7580, 7610, 7639, 7669, 7698, 7728, 7757, 7787, 7816, 7846, 7875, 7905, 7934, 7964, 7993, 8023, 8053, 8083, 8112, 8142, 8171, 8201, 8230, 8260, 8289, 8319, 8348, 8378, 8407, 8437, 8466, 8496, 8525, 8555, 8584, 8614, 8643, 8673, 8702, 8732, 8761, 8791, 8821, 8850, 8880, 8909, 8938, 8968, 8997, 9027, 9056, 9086, 9115, 9145, 9175, 9205, 9234, 9264, 9293, 9322, 9352, 9381, 9410, 9440, 9470, 9499, 9529, 9559, 9589, 9618, 9648, 9677, 9706, 9736, 9765, 9794, 9824, 9853, 9883, 9913, 9943, 9972, 10002, 10032, 10061, 10090, 10120, 10149, 10178, 10208, 10237, 10267, 10297, 10326, 10356, 10386, 10415, 10445, 10474, 10504, 10533, 10562, 10592, 10621, 10651, 10680, 10710, 10740, 10770, 10799, 10829, 10858, 10888, 10917, 10947, 10976, 11005, 11035, 11064, 11094, 11124, 11153, 11183, 11213, 11242, 11272, 11301, 11331, 11360, 11389, 11419, 11448, 11478, 11507, 11537, 11567, 11596, 11626, 11655, 11685, 11715, 11744, 11774, 11803, 11832, 11862, 11891, 11921, 11950, 11980, 12010, 12039, 12069, 12099, 12128, 12158, 12187, 12216, 12246, 12275, 12304, 12334, 12364, 12393, 12423, 12453, 12483, 12512, 12542, 12571, 12600, 12630, 12659, 12688, 12718, 12747, 12777, 12807, 12837, 12866, 12896, 12926, 12955, 12984, 13014, 13043, 13072, 13102, 13131, 13161, 13191, 13220, 13250, 13280, 13310, 13339, 13368, 13398, 13427, 13456, 13486, 13515, 13545, 13574, 13604, 13634, 13664, 13693, 13723, 13752, 13782, 13811, 13840, 13870, 13899, 13929, 13958, 13988, 14018, 14047, 14077, 14107, 14136, 14166, 14195, 14224, 14254, 14283, 14313, 14342, 14372, 14401, 14431, 14461, 14490, 14520, 14550, 14579, 14609, 14638, 14667, 14697, 14726, 14756, 14785, 14815, 14844, 14874, 14904, 14933, 14963, 14993, 15021, 15051, 15081, 15110, 15140, 15169, 15199, 15228, 15258, 15287, 15317, 15347, 15377, 15406, 15436, 15465, 15494, 15524, 15553, 15582, 15612, 15641, 15671, 15701, 15731, 15760, 15790, 15820, 15849, 15878, 15908, 15937, 15966, 15996, 16025, 16055, 16085, 16114, 16144, 16174, 16204, 16233, 16262, 16292, 16321, 16350, 16380, 16409, 16439, 16468, 16498, 16528, 16558, 16587, 16617, 16646, 16676, 16705, 16734, 16764, 16793, 16823, 16852, 16882, 16912, 16941, 16971, 17001, 17030, 17060, 17089, 17118, 17148, 17177, 17207, 17236, 17266, 17295, 17325, 17355, 17384, 17414, 17444, 17473, 17502, 17532, 17561, 17591, 17620, 17650, 17679, 17709, 17738, 17768, 17798, 17827, 17857, 17886, 17916, 17945, 17975, 18004, 18034, 18063, 18093, 18122, 18152, 18181, 18211, 18241, 18270, 18300, 18330, 18359, 18388, 18418, 18447, 18476, 18506, 18535, 18565, 18595, 18625, 18654, 18684, 18714, 18743, 18772, 18802, 18831, 18860, 18890, 18919, 18949, 18979, 19008, 19038, 19068, 19098, 19127, 19156, 19186, 19215, 19244, 19274, 19303, 19333, 19362, 19392, 19422, 19452, 19481, 19511, 19540, 19570, 19599, 19628, 19658, 19687, 19717, 19746, 19776, 19806, 19836, 19865, 19895, 19924, 19954, 19983, 20012, 20042, 20071, 20101, 20130, 20160, 20190, 20219, 20249, 20279, 20308, 20338, 20367, 20396, 20426, 20455, 20485, 20514, 20544, 20573, 20603, 20633, 20662, 20692, 20721, 20751, 20780, 20810, 20839, 20869, 20898, 20928, 20957, 20987, 21016, 21046, 21076, 21105, 21135, 21164, 21194, 21223, 21253, 21282, 21312, 21341, 21371, 21400, 21430, 21459, 21489, 21519, 21548, 21578, 21607, 21637, 21666, 21696, 21725, 21754, 21784, 21813, 21843, 21873, 21902, 21932, 21962, 21991, 22021, 22050, 22080, 22109, 22138, 22168, 22197, 22227, 22256, 22286, 22316, 22346, 22375, 22405, 22434, 22464, 22493, 22522, 22552, 22581, 22611, 22640, 22670, 22700, 22730, 22759, 22789, 22818, 22848, 22877, 22906, 22936, 22965, 22994, 23024, 23054, 23083, 23113, 23143, 23173, 23202, 23232, 23261, 23290, 23320, 23349, 23379, 23408, 23438, 23467, 23497, 23527, 23556, 23586, 23616, 23645, 23674, 23704, 23733, 23763, 23792, 23822, 23851, 23881, 23910, 23940, 23970, 23999, 24029, 24058, 24088, 24117, 24147, 24176, 24206, 24235, 24265, 24294, 24324, 24353, 24383, 24413, 24442, 24472, 24501, 24531, 24560, 24590, 24619, 24648, 24678, 24707, 24737, 24767, 24796, 24826, 24856, 24885, 24915, 24944, 24974, 25003, 25032, 25062, 25091, 25121, 25150, 25180, 25210, 25240, 25269, 25299, 25328, 25358, 25387, 25416, 25446, 25475, 25505, 25534, 25564, 25594, 25624, 25653, 25683, 25712, 25742, 25771, 25800, 25830, 25859, 25888, 25918, 25948, 25977, 26007, 26037, 26067, 26096, 26126, 26155, 26184, 26214, 26243, 26272, 26302, 26332, 26361, 26391, 26421, 26451, 26480, 26510, 26539, 26568, 26598, 26627, 26656, 26686, 26715, 26745, 26775, 26805, 26834, 26864, 26893, 26923, 26952, 26982, 27011, 27041, 27070, 27099, 27129, 27159, 27188, 27218, 27248, 27277, 27307, 27336, 27366, 27395, 27425, 27454, 27484, 27513, 27542, 27572, 27602, 27631, 27661, 27691, 27720, 27750, 27779, 27809, 27838, 27868, 27897, 27926, 27956, 27985, 28015, 28045, 28074, 28104, 28134, 28163, 28193, 28222, 28252, 28281, 28310, 28340, 28369, 28399, 28428, 28458, 28488, 28517, 28547, 28577, 28607, 28636, 28665, 28695, 28724, 28754, 28783, 28813, 28843, 28872, 28901, 28931, 28960, 28990, 29019, 29049, 29078, 29108, 29137, 29167, 29196, 29226, 29255, 29285, 29315, 29345, 29375, 29404, 29434, 29463, 29492, 29522, 29551, 29580, 29610, 29640, 29669, 29699, 29729, 29759, 29788, 29818, 29847, 29876, 29906, 29935, 29964, 29994, 30023, 30053, 30082, 30112, 30141, 30171, 30200, 30230, 30259, 30289, 30318, 30348, 30378, 30408, 30437, 30467, 30496, 30526, 30555, 30585, 30614, 30644, 30673, 30703, 30732, 30762, 30791, 30821, 30850, 30880, 30909, 30939, 30968, 30998, 31027, 31057, 31086, 31116, 31145, 31175, 31204, 31234, 31263, 31293, 31322, 31352, 31381, 31411, 31441, 31471, 31500, 31530, 31559, 31589, 31618, 31648, 31676, 31706, 31736, 31766, 31795, 31825, 31854, 31884, 31913, 31943, 31972, 32002, 32031, 32061, 32090, 32120, 32150, 32180, 32209, 32239, 32268, 32298, 32327, 32357, 32386, 32416, 32445, 32475, 32504, 32534, 32563, 32593, 32622, 32652, 32681, 32711, 32740, 32770, 32799, 32829, 32858, 32888, 32917, 32947, 32976, 33006, 33035, 33065, 33094, 33124, 33153, 33183, 33213, 33243, 33272, 33302, 33331, 33361, 33390, 33420, 33450, 33479, 33509, 33539, 33568, 33598, 33627, 33657, 33686, 33716, 33745, 33775, 33804, 33834, 33863, 33893, 33922, 33952, 33981, 34011, 34040, 34069, 34099, 34128, 34158, 34187, 34217, 34247, 34277, 34306, 34336, 34365, 34395, 34424, 34454, 34483, 34512, 34542, 34571, 34601, 34631, 34660, 34690, 34719, 34749, 34778, 34808, 34837, 34867, 34896, 34926, 34955, 34985, 35015, 35044, 35074, 35103, 35133, 35162, 35192, 35222, 35251, 35280, 35310, 35340, 35370, 35399, 35429, 35458, 35488, 35517, 35547, 35576, 35605, 35635, 35665, 35694, 35723, 35753, 35782, 35811, 35841, 35871, 35901, 35930, 35960, 35989, 36019, 36048, 36078, 36107, 36136, 36166, 36195, 36225, 36254, 36284, 36314, 36343, 36373, 36403, 36433, 36462, 36492, 36521, 36551, 36580, 36610, 36639, 36669, 36698, 36728, 36757, 36786, 36816, 36845, 36875, 36904, 36934, 36963, 36993, 37022, 37052, 37081, 37111, 37141, 37170, 37200, 37229, 37259, 37288, 37318, 37347, 37377, 37406, 37436, 37465, 37495, 37524, 37554, 37584, 37613, 37643, 37672, 37701, 37731, 37760, 37790, 37819, 37849, 37878, 37908, 37938, 37967, 37997, 38027, 38056, 38085, 38115, 38144, 38174, 38203, 38233, 38262, 38292, 38322, 38351, 38381, 38410, 38440, 38469, 38499, 38528, 38558, 38587, 38617, 38646, 38676, 38705, 38735, 38764, 38794, 38823, 38853, 38882, 38912, 38941, 38971, 39001, 39030, 39059, 39089, 39118, 39148, 39178, 39208, 39237, 39267, 39297, 39326, 39355, 39385, 39414, 39444, 39473, 39503, 39532, 39562, 39592, 39621, 39650, 39680, 39709, 39739, 39768, 39798, 39827, 39857, 39886, 39916, 39946, 39975, 40005, 40035, 40064, 40094, 40123, 40153, 40182, 40212, 40241, 40271, 40300, 40330, 40359, 40389, 40418, 40448, 40477, 40507, 40536, 40566, 40595, 40625, 40655, 40685, 40714, 40744, 40773, 40803, 40832, 40862, 40892, 40921, 40951, 40980, 41009, 41039, 41068, 41098, 41127, 41157, 41186, 41216, 41245, 41275, 41304, 41334, 41364, 41393, 41422, 41452, 41481, 41511, 41540, 41570, 41599, 41629, 41658, 41688, 41718, 41748, 41777, 41807, 41836, 41865, 41894, 41924, 41953, 41983, 42012, 42042, 42072, 42102, 42131, 42161, 42190, 42220, 42249, 42279, 42308, 42337, 42367, 42397, 42426, 42456, 42485, 42515, 42545, 42574, 42604, 42633, 42662, 42692, 42721, 42751, 42780, 42810, 42839, 42869, 42899, 42929, 42958, 42988, 43017, 43046, 43076, 43105, 43135, 43164, 43194, 43223, 43253, 43283, 43312, 43342, 43371, 43401, 43430, 43460, 43489, 43519, 43548, 43578, 43607, 43637, 43666, 43696, 43726, 43755, 43785, 43814, 43844, 43873, 43903, 43932, 43962, 43991, 44021, 44050, 44080, 44109, 44139, 44169, 44198, 44228, 44258, 44287, 44317, 44346, 44375, 44405, 44434, 44464, 44493, 44523, 44553, 44582, 44612, 44641, 44671, 44700, 44730, 44759, 44788, 44818, 44847, 44877, 44906, 44936, 44966, 44996, 45025, 45055, 45084, 45114, 45143, 45172, 45202, 45231, 45261, 45290, 45320, 45350, 45380, 45409, 45439, 45468, 45498, 45527, 45556, 45586, 45615, 45644, 45674, 45704, 45733, 45763, 45793, 45823, 45852, 45882, 45911, 45940, 45970, 45999, 46028, 46058, 46088, 46117, 46147, 46177, 46206, 46236, 46265, 46295, 46324, 46354, 46383, 46413, 46442, 46472, 46501, 46531, 46560, 46590, 46620, 46649, 46679, 46708, 46738, 46767, 46797, 46826, 46856, 46885, 46915, 46944, 46974, 47003, 47033, 47063, 47092, 47122, 47151, 47181, 47210, 47240, 47269, 47298, 47328, 47357, 47387, 47417, 47446, 47476, 47506, 47535, 47565, 47594, 47624, 47653, 47682, 47712, 47741, 47771, 47800, 47830, 47860, 47890, 47919, 47949, 47978, 48008, 48037, 48066, 48096, 48125, 48155, 48184, 48214, 48244, 48273, 48303, 48333, 48362, 48392, 48421, 48450, 48480, 48509, 48538, 48568, 48598, 48627, 48657, 48687, 48717, 48746, 48776, 48805, 48834, 48864, 48893, 48922, 48952, 48982, 49011, 49041, 49071, 49100, 49130, 49160, 49189, 49218, 49248, 49277, 49306, 49336, 49365, 49395, 49425, 49455, 49484, 49514, 49543, 49573, 49602, 49632, 49661, 49690, 49720, 49749, 49779, 49809, 49838, 49868, 49898, 49927, 49957, 49986, 50016, 50045, 50075, 50104, 50133, 50163, 50192, 50222, 50252, 50281, 50311, 50340, 50370, 50400, 50429, 50459, 50488, 50518, 50547, 50576, 50606, 50635, 50665, 50694, 50724, 50754, 50784, 50813, 50843, 50872, 50902, 50931, 50960, 50990, 51019, 51049, 51078, 51108, 51138, 51167, 51197, 51227, 51256, 51286, 51315, 51345, 51374, 51403, 51433, 51462, 51492, 51522, 51552, 51582, 51611, 51641, 51670, 51699, 51729, 51758, 51787, 51816, 51846, 51876, 51906, 51936, 51965, 51995, 52025, 52054, 52083, 52113, 52142, 52171, 52200, 52230, 52260, 52290, 52319, 52349, 52379, 52408, 52438, 52467, 52497, 52526, 52555, 52585, 52614, 52644, 52673, 52703, 52733, 52762, 52792, 52822, 52851, 52881, 52910, 52939, 52969, 52998, 53028, 53057, 53087, 53116, 53146, 53176, 53205, 53235, 53264, 53294, 53324, 53353, 53383, 53412, 53441, 53471, 53500, 53530, 53559, 53589, 53619, 53648, 53678, 53708, 53737, 53767, 53796, 53825, 53855, 53884, 53913, 53943, 53973, 54003, 54032, 54062, 54092, 54121, 54151, 54180, 54209, 54239, 54268, 54297, 54327, 54357, 54387, 54416, 54446, 54476, 54505, 54535, 54564, 54593, 54623, 54652, 54681, 54711, 54741, 54770, 54800, 54830, 54859, 54889, 54919, 54948, 54977, 55007, 55036, 55066, 55095, 55125, 55154, 55184, 55213, 55243, 55273, 55302, 55332, 55361, 55391, 55420, 55450, 55479, 55508, 55538, 55567, 55597, 55627, 55657, 55686, 55716, 55745, 55775, 55804, 55834, 55863, 55892, 55922, 55951, 55981, 56011, 56040, 56070, 56100, 56129, 56159, 56188, 56218, 56247, 56276, 56306, 56335, 56365, 56394, 56424, 56454, 56483, 56513, 56543, 56572, 56601, 56631, 56660, 56690, 56719, 56749, 56778, 56808, 56837, 56867, 56897, 56926, 56956, 56985, 57015, 57044, 57074, 57103, 57133, 57162, 57192, 57221, 57251, 57280, 57310, 57340, 57369, 57399, 57429, 57458, 57487, 57517, 57546, 57576, 57605, 57634, 57664, 57694, 57723, 57753, 57783, 57813, 57842, 57871, 57901, 57930, 57959, 57989, 58018, 58048, 58077, 58107, 58137, 58167, 58196, 58226, 58255, 58285, 58314, 58343, 58373, 58402, 58432, 58461, 58491, 58521, 58551, 58580, 58610, 58639, 58669, 58698, 58727, 58757, 58786, 58816, 58845, 58875, 58905, 58934, 58964, 58994, 59023, 59053, 59082, 59111, 59141, 59170, 59200, 59229, 59259, 59288, 59318, 59348, 59377, 59407, 59436, 59466, 59495, 59525, 59554, 59584, 59613, 59643, 59672, 59702, 59731, 59761, 59791, 59820, 59850, 59879, 59909, 59939, 59968, 59997, 60027, 60056, 60086, 60115, 60145, 60174, 60204, 60234, 60264, 60293, 60323, 60352, 60381, 60411, 60440, 60469, 60499, 60528, 60558, 60588, 60618, 60648, 60677, 60707, 60736, 60765, 60795, 60824, 60853, 60883, 60912, 60942, 60972, 61002, 61031, 61061, 61090, 61120, 61149, 61179, 61208, 61237, 61267, 61296, 61326, 61356, 61385, 61415, 61445, 61474, 61504, 61533, 61563, 61592, 61621, 61651, 61680, 61710, 61739, 61769, 61799, 61828, 61858, 61888, 61917, 61947, 61976, 62006, 62035, 62064, 62094, 62123, 62153, 62182, 62212, 62242, 62271, 62301, 62331, 62360, 62390, 62419, 62448, 62478, 62507, 62537, 62566, 62596, 62625, 62655, 62685, 62715, 62744, 62774, 62803, 62832, 62862, 62891, 62921, 62950, 62980, 63009, 63039, 63069, 63099, 63128, 63157, 63187, 63216, 63246, 63275, 63305, 63334, 63363, 63393, 63423, 63453, 63482, 63512, 63541, 63571, 63600, 63630, 63659, 63689, 63718, 63747, 63777, 63807, 63836, 63866, 63895, 63925, 63955, 63984, 64014, 64043, 64073, 64102, 64131, 64161, 64190, 64220, 64249, 64279, 64309, 64339, 64368, 64398, 64427, 64457, 64486, 64515, 64545, 64574, 64603, 64633, 64663, 64692, 64722, 64752, 64782, 64811, 64841, 64870, 64899, 64929, 64958, 64987, 65017, 65047, 65076, 65106, 65136, 65166, 65195, 65225, 65254, 65283, 65313, 65342, 65371, 65401, 65431, 65460, 65490, 65520, 65549, 65579, 65608, 65638, 65667, 65697, 65726, 65755, 65785, 65815, 65844, 65874, 65903, 65933, 65963, 65992, 66022, 66051, 66081, 66110, 66140, 66169, 66199, 66228, 66258, 66287, 66317, 66346, 66376, 66405, 66435, 66465, 66494, 66524, 66553, 66583, 66612, 66641, 66671, 66700, 66730, 66760, 66789, 66819, 66849, 66878, 66908, 66937, 66967, 66996, 67025, 67055, 67084, 67114, 67143, 67173, 67203, 67233, 67262, 67292, 67321, 67351, 67380, 67409, 67439, 67468, 67497, 67527, 67557, 67587, 67617, 67646, 67676, 67705, 67735, 67764, 67793, 67823, 67852, 67882, 67911, 67941, 67971, 68e3, 68030, 68060, 68089, 68119, 68148, 68177, 68207, 68236, 68266, 68295, 68325, 68354, 68384, 68414, 68443, 68473, 68502, 68532, 68561, 68591, 68620, 68650, 68679, 68708, 68738, 68768, 68797, 68827, 68857, 68886, 68916, 68946, 68975, 69004, 69034, 69063, 69092, 69122, 69152, 69181, 69211, 69240, 69270, 69300, 69330, 69359, 69388, 69418, 69447, 69476, 69506, 69535, 69565, 69595, 69624, 69654, 69684, 69713, 69743, 69772, 69802, 69831, 69861, 69890, 69919, 69949, 69978, 70008, 70038, 70067, 70097, 70126, 70156, 70186, 70215, 70245, 70274, 70303, 70333, 70362, 70392, 70421, 70451, 70481, 70510, 70540, 70570, 70599, 70629, 70658, 70687, 70717, 70746, 70776, 70805, 70835, 70864, 70894, 70924, 70954, 70983, 71013, 71042, 71071, 71101, 71130, 71159, 71189, 71218, 71248, 71278, 71308, 71337, 71367, 71397, 71426, 71455, 71485, 71514, 71543, 71573, 71602, 71632, 71662, 71691, 71721, 71751, 71781, 71810, 71839, 71869, 71898, 71927, 71957, 71986, 72016, 72046, 72075, 72105, 72135, 72164, 72194, 72223, 72253, 72282, 72311, 72341, 72370, 72400, 72429, 72459, 72489, 72518, 72548, 72577, 72607, 72637, 72666, 72695, 72725, 72754, 72784, 72813, 72843, 72872, 72902, 72931, 72961, 72991, 73020, 73050, 73080, 73109, 73139, 73168, 73197, 73227, 73256, 73286, 73315, 73345, 73375, 73404, 73434, 73464, 73493, 73523, 73552, 73581, 73611, 73640, 73669, 73699, 73729, 73758, 73788, 73818, 73848, 73877, 73907, 73936, 73965, 73995, 74024, 74053, 74083, 74113, 74142, 74172, 74202, 74231, 74261, 74291, 74320, 74349, 74379, 74408, 74437, 74467, 74497, 74526, 74556, 74586, 74615, 74645, 74675, 74704, 74733, 74763, 74792, 74822, 74851, 74881, 74910, 74940, 74969, 74999, 75029, 75058, 75088, 75117, 75147, 75176, 75206, 75235, 75264, 75294, 75323, 75353, 75383, 75412, 75442, 75472, 75501, 75531, 75560, 75590, 75619, 75648, 75678, 75707, 75737, 75766, 75796, 75826, 75856, 75885, 75915, 75944, 75974, 76003, 76032, 76062, 76091, 76121, 76150, 76180, 76210, 76239, 76269, 76299, 76328, 76358, 76387, 76416, 76446, 76475, 76505, 76534, 76564, 76593, 76623, 76653, 76682, 76712, 76741, 76771, 76801, 76830, 76859, 76889, 76918, 76948, 76977, 77007, 77036, 77066, 77096, 77125, 77155, 77185, 77214, 77243, 77273, 77302, 77332, 77361, 77390, 77420, 77450, 77479, 77509, 77539, 77569, 77598, 77627, 77657, 77686, 77715, 77745, 77774, 77804, 77833, 77863, 77893, 77923, 77952, 77982, 78011, 78041, 78070, 78099, 78129, 78158, 78188, 78217, 78247, 78277, 78307, 78336, 78366, 78395, 78425, 78454, 78483, 78513, 78542, 78572, 78601, 78631, 78661, 78690, 78720, 78750, 78779, 78808, 78838, 78867, 78897, 78926, 78956, 78985, 79015, 79044, 79074, 79104, 79133, 79163, 79192, 79222, 79251, 79281, 79310, 79340, 79369, 79399, 79428, 79458, 79487, 79517, 79546, 79576, 79606, 79635, 79665, 79695, 79724, 79753, 79783, 79812, 79841, 79871, 79900, 79930, 79960, 79990];
          }, 24453: function(G, J, Z) {
            var te = Z(27976);
            function ne() {
              this.regionalOptions = [], this.regionalOptions[""] = { invalidCalendar: "Calendar {0} not found", invalidDate: "Invalid {0} date", invalidMonth: "Invalid {0} month", invalidYear: "Invalid {0} year", differentCalendars: "Cannot mix {0} and {1} dates" }, this.local = this.regionalOptions[""], this.calendars = {}, this._localCals = {};
            }
            function ee(le, ue, de, fe) {
              if (this._calendar = le, this._year = ue, this._month = de, this._day = fe, this._calendar._validateLevel === 0 && !this._calendar.isValid(this._year, this._month, this._day)) throw (ce.local.invalidDate || ce.regionalOptions[""].invalidDate).replace(/\{0\}/, this._calendar.local.name);
            }
            function ae(le, ue) {
              return "000000".substring(0, ue - (le = "" + le).length) + le;
            }
            function oe() {
              this.shortYearCutoff = "+10";
            }
            function se(le) {
              this.local = this.regionalOptions[le] || this.regionalOptions[""];
            }
            te(ne.prototype, { instance: function(le, ue) {
              le = (le || "gregorian").toLowerCase(), ue = ue || "";
              var de = this._localCals[le + "-" + ue];
              if (!de && this.calendars[le] && (de = new this.calendars[le](ue), this._localCals[le + "-" + ue] = de), !de) throw (this.local.invalidCalendar || this.regionalOptions[""].invalidCalendar).replace(/\{0\}/, le);
              return de;
            }, newDate: function(le, ue, de, fe, he) {
              return (fe = (le != null && le.year ? le.calendar() : typeof fe == "string" ? this.instance(fe, he) : fe) || this.instance()).newDate(le, ue, de);
            }, substituteDigits: function(le) {
              return function(ue) {
                return (ue + "").replace(/[0-9]/g, function(de) {
                  return le[de];
                });
              };
            }, substituteChineseDigits: function(le, ue) {
              return function(de) {
                for (var fe = "", he = 0; de > 0; ) {
                  var me = de % 10;
                  fe = (me === 0 ? "" : le[me] + ue[he]) + fe, he++, de = Math.floor(de / 10);
                }
                return fe.indexOf(le[1] + ue[1]) === 0 && (fe = fe.substr(1)), fe || le[0];
              };
            } }), te(ee.prototype, { newDate: function(le, ue, de) {
              return this._calendar.newDate(le ?? this, ue, de);
            }, year: function(le) {
              return arguments.length === 0 ? this._year : this.set(le, "y");
            }, month: function(le) {
              return arguments.length === 0 ? this._month : this.set(le, "m");
            }, day: function(le) {
              return arguments.length === 0 ? this._day : this.set(le, "d");
            }, date: function(le, ue, de) {
              if (!this._calendar.isValid(le, ue, de)) throw (ce.local.invalidDate || ce.regionalOptions[""].invalidDate).replace(/\{0\}/, this._calendar.local.name);
              return this._year = le, this._month = ue, this._day = de, this;
            }, leapYear: function() {
              return this._calendar.leapYear(this);
            }, epoch: function() {
              return this._calendar.epoch(this);
            }, formatYear: function() {
              return this._calendar.formatYear(this);
            }, monthOfYear: function() {
              return this._calendar.monthOfYear(this);
            }, weekOfYear: function() {
              return this._calendar.weekOfYear(this);
            }, daysInYear: function() {
              return this._calendar.daysInYear(this);
            }, dayOfYear: function() {
              return this._calendar.dayOfYear(this);
            }, daysInMonth: function() {
              return this._calendar.daysInMonth(this);
            }, dayOfWeek: function() {
              return this._calendar.dayOfWeek(this);
            }, weekDay: function() {
              return this._calendar.weekDay(this);
            }, extraInfo: function() {
              return this._calendar.extraInfo(this);
            }, add: function(le, ue) {
              return this._calendar.add(this, le, ue);
            }, set: function(le, ue) {
              return this._calendar.set(this, le, ue);
            }, compareTo: function(le) {
              if (this._calendar.name !== le._calendar.name) throw (ce.local.differentCalendars || ce.regionalOptions[""].differentCalendars).replace(/\{0\}/, this._calendar.local.name).replace(/\{1\}/, le._calendar.local.name);
              var ue = this._year !== le._year ? this._year - le._year : this._month !== le._month ? this.monthOfYear() - le.monthOfYear() : this._day - le._day;
              return ue === 0 ? 0 : ue < 0 ? -1 : 1;
            }, calendar: function() {
              return this._calendar;
            }, toJD: function() {
              return this._calendar.toJD(this);
            }, fromJD: function(le) {
              return this._calendar.fromJD(le);
            }, toJSDate: function() {
              return this._calendar.toJSDate(this);
            }, fromJSDate: function(le) {
              return this._calendar.fromJSDate(le);
            }, toString: function() {
              return (this.year() < 0 ? "-" : "") + ae(Math.abs(this.year()), 4) + "-" + ae(this.month(), 2) + "-" + ae(this.day(), 2);
            } }), te(oe.prototype, { _validateLevel: 0, newDate: function(le, ue, de) {
              return le == null ? this.today() : (le.year && (this._validate(le, ue, de, ce.local.invalidDate || ce.regionalOptions[""].invalidDate), de = le.day(), ue = le.month(), le = le.year()), new ee(this, le, ue, de));
            }, today: function() {
              return this.fromJSDate(/* @__PURE__ */ new Date());
            }, epoch: function(le) {
              return this._validate(le, this.minMonth, this.minDay, ce.local.invalidYear || ce.regionalOptions[""].invalidYear).year() < 0 ? this.local.epochs[0] : this.local.epochs[1];
            }, formatYear: function(le) {
              var ue = this._validate(le, this.minMonth, this.minDay, ce.local.invalidYear || ce.regionalOptions[""].invalidYear);
              return (ue.year() < 0 ? "-" : "") + ae(Math.abs(ue.year()), 4);
            }, monthsInYear: function(le) {
              return this._validate(le, this.minMonth, this.minDay, ce.local.invalidYear || ce.regionalOptions[""].invalidYear), 12;
            }, monthOfYear: function(le, ue) {
              var de = this._validate(le, ue, this.minDay, ce.local.invalidMonth || ce.regionalOptions[""].invalidMonth);
              return (de.month() + this.monthsInYear(de) - this.firstMonth) % this.monthsInYear(de) + this.minMonth;
            }, fromMonthOfYear: function(le, ue) {
              var de = (ue + this.firstMonth - 2 * this.minMonth) % this.monthsInYear(le) + this.minMonth;
              return this._validate(le, de, this.minDay, ce.local.invalidMonth || ce.regionalOptions[""].invalidMonth), de;
            }, daysInYear: function(le) {
              var ue = this._validate(le, this.minMonth, this.minDay, ce.local.invalidYear || ce.regionalOptions[""].invalidYear);
              return this.leapYear(ue) ? 366 : 365;
            }, dayOfYear: function(le, ue, de) {
              var fe = this._validate(le, ue, de, ce.local.invalidDate || ce.regionalOptions[""].invalidDate);
              return fe.toJD() - this.newDate(fe.year(), this.fromMonthOfYear(fe.year(), this.minMonth), this.minDay).toJD() + 1;
            }, daysInWeek: function() {
              return 7;
            }, dayOfWeek: function(le, ue, de) {
              var fe = this._validate(le, ue, de, ce.local.invalidDate || ce.regionalOptions[""].invalidDate);
              return (Math.floor(this.toJD(fe)) + 2) % this.daysInWeek();
            }, extraInfo: function(le, ue, de) {
              return this._validate(le, ue, de, ce.local.invalidDate || ce.regionalOptions[""].invalidDate), {};
            }, add: function(le, ue, de) {
              return this._validate(le, this.minMonth, this.minDay, ce.local.invalidDate || ce.regionalOptions[""].invalidDate), this._correctAdd(le, this._add(le, ue, de), ue, de);
            }, _add: function(le, ue, de) {
              if (this._validateLevel++, de === "d" || de === "w") {
                var fe = le.toJD() + ue * (de === "w" ? this.daysInWeek() : 1), he = le.calendar().fromJD(fe);
                return this._validateLevel--, [he.year(), he.month(), he.day()];
              }
              try {
                var me = le.year() + (de === "y" ? ue : 0), pe = le.monthOfYear() + (de === "m" ? ue : 0);
                he = le.day(), de === "y" ? (le.month() !== this.fromMonthOfYear(me, pe) && (pe = this.newDate(me, le.month(), this.minDay).monthOfYear()), pe = Math.min(pe, this.monthsInYear(me)), he = Math.min(he, this.daysInMonth(me, this.fromMonthOfYear(me, pe)))) : de === "m" && (function(ge) {
                  for (; pe < ge.minMonth; ) me--, pe += ge.monthsInYear(me);
                  for (var ve = ge.monthsInYear(me); pe > ve - 1 + ge.minMonth; ) me++, pe -= ve, ve = ge.monthsInYear(me);
                }(this), he = Math.min(he, this.daysInMonth(me, this.fromMonthOfYear(me, pe))));
                var ye = [me, this.fromMonthOfYear(me, pe), he];
                return this._validateLevel--, ye;
              } catch (ge) {
                throw this._validateLevel--, ge;
              }
            }, _correctAdd: function(le, ue, de, fe) {
              if (!(this.hasYearZero || fe !== "y" && fe !== "m" || ue[0] !== 0 && le.year() > 0 == ue[0] > 0)) {
                var he = { y: [1, 1, "y"], m: [1, this.monthsInYear(-1), "m"], w: [this.daysInWeek(), this.daysInYear(-1), "d"], d: [1, this.daysInYear(-1), "d"] }[fe], me = de < 0 ? -1 : 1;
                ue = this._add(le, de * he[0] + me * he[1], he[2]);
              }
              return le.date(ue[0], ue[1], ue[2]);
            }, set: function(le, ue, de) {
              this._validate(le, this.minMonth, this.minDay, ce.local.invalidDate || ce.regionalOptions[""].invalidDate);
              var fe = de === "y" ? ue : le.year(), he = de === "m" ? ue : le.month(), me = de === "d" ? ue : le.day();
              return de !== "y" && de !== "m" || (me = Math.min(me, this.daysInMonth(fe, he))), le.date(fe, he, me);
            }, isValid: function(le, ue, de) {
              this._validateLevel++;
              var fe = this.hasYearZero || le !== 0;
              if (fe) {
                var he = this.newDate(le, ue, this.minDay);
                fe = ue >= this.minMonth && ue - this.minMonth < this.monthsInYear(he) && de >= this.minDay && de - this.minDay < this.daysInMonth(he);
              }
              return this._validateLevel--, fe;
            }, toJSDate: function(le, ue, de) {
              var fe = this._validate(le, ue, de, ce.local.invalidDate || ce.regionalOptions[""].invalidDate);
              return ce.instance().fromJD(this.toJD(fe)).toJSDate();
            }, fromJSDate: function(le) {
              return this.fromJD(ce.instance().fromJSDate(le).toJD());
            }, _validate: function(le, ue, de, fe) {
              if (le.year) {
                if (this._validateLevel === 0 && this.name !== le.calendar().name) throw (ce.local.differentCalendars || ce.regionalOptions[""].differentCalendars).replace(/\{0\}/, this.local.name).replace(/\{1\}/, le.calendar().local.name);
                return le;
              }
              try {
                if (this._validateLevel++, this._validateLevel === 1 && !this.isValid(le, ue, de)) throw fe.replace(/\{0\}/, this.local.name);
                var he = this.newDate(le, ue, de);
                return this._validateLevel--, he;
              } catch (me) {
                throw this._validateLevel--, me;
              }
            } }), se.prototype = new oe(), te(se.prototype, { name: "Gregorian", jdEpoch: 17214255e-1, daysPerMonth: [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31], hasYearZero: !1, minMonth: 1, firstMonth: 1, minDay: 1, regionalOptions: { "": { name: "Gregorian", epochs: ["BCE", "CE"], monthNames: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"], monthNamesShort: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"], dayNames: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"], dayNamesShort: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"], dayNamesMin: ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"], digits: null, dateFormat: "mm/dd/yyyy", firstDay: 0, isRTL: !1 } }, leapYear: function(le) {
              var ue = this._validate(le, this.minMonth, this.minDay, ce.local.invalidYear || ce.regionalOptions[""].invalidYear);
              return (le = ue.year() + (ue.year() < 0 ? 1 : 0)) % 4 == 0 && (le % 100 != 0 || le % 400 == 0);
            }, weekOfYear: function(le, ue, de) {
              var fe = this.newDate(le, ue, de);
              return fe.add(4 - (fe.dayOfWeek() || 7), "d"), Math.floor((fe.dayOfYear() - 1) / 7) + 1;
            }, daysInMonth: function(le, ue) {
              var de = this._validate(le, ue, this.minDay, ce.local.invalidMonth || ce.regionalOptions[""].invalidMonth);
              return this.daysPerMonth[de.month() - 1] + (de.month() === 2 && this.leapYear(de.year()) ? 1 : 0);
            }, weekDay: function(le, ue, de) {
              return (this.dayOfWeek(le, ue, de) || 7) < 6;
            }, toJD: function(le, ue, de) {
              var fe = this._validate(le, ue, de, ce.local.invalidDate || ce.regionalOptions[""].invalidDate);
              le = fe.year(), ue = fe.month(), de = fe.day(), le < 0 && le++, ue < 3 && (ue += 12, le--);
              var he = Math.floor(le / 100), me = 2 - he + Math.floor(he / 4);
              return Math.floor(365.25 * (le + 4716)) + Math.floor(30.6001 * (ue + 1)) + de + me - 1524.5;
            }, fromJD: function(le) {
              var ue = Math.floor(le + 0.5), de = Math.floor((ue - 186721625e-2) / 36524.25), fe = 1524 + (de = ue + 1 + de - Math.floor(de / 4)), he = Math.floor((fe - 122.1) / 365.25), me = Math.floor(365.25 * he), pe = Math.floor((fe - me) / 30.6001), ye = fe - me - Math.floor(30.6001 * pe), ge = pe - (pe > 13.5 ? 13 : 1), ve = he - (ge > 2.5 ? 4716 : 4715);
              return ve <= 0 && ve--, this.newDate(ve, ge, ye);
            }, toJSDate: function(le, ue, de) {
              var fe = this._validate(le, ue, de, ce.local.invalidDate || ce.regionalOptions[""].invalidDate), he = new Date(fe.year(), fe.month() - 1, fe.day());
              return he.setHours(0), he.setMinutes(0), he.setSeconds(0), he.setMilliseconds(0), he.setHours(he.getHours() > 12 ? he.getHours() + 2 : 0), he;
            }, fromJSDate: function(le) {
              return this.newDate(le.getFullYear(), le.getMonth() + 1, le.getDate());
            } });
            var ce = G.exports = new ne();
            ce.cdate = ee, ce.baseCalendar = oe, ce.calendars.gregorian = se;
          }, 23428: function(G, J, Z) {
            var te = Z(27976), ne = Z(24453);
            te(ne.regionalOptions[""], { invalidArguments: "Invalid arguments", invalidFormat: "Cannot format a date from another calendar", missingNumberAt: "Missing number at position {0}", unknownNameAt: "Unknown name at position {0}", unexpectedLiteralAt: "Unexpected literal at position {0}", unexpectedText: "Additional text found at end" }), ne.local = ne.regionalOptions[""], te(ne.cdate.prototype, { formatDate: function(ee, ae) {
              return typeof ee != "string" && (ae = ee, ee = ""), this._calendar.formatDate(ee || "", this, ae);
            } }), te(ne.baseCalendar.prototype, { UNIX_EPOCH: ne.instance().newDate(1970, 1, 1).toJD(), SECS_PER_DAY: 86400, TICKS_EPOCH: ne.instance().jdEpoch, TICKS_PER_DAY: 864e9, ATOM: "yyyy-mm-dd", COOKIE: "D, dd M yyyy", FULL: "DD, MM d, yyyy", ISO_8601: "yyyy-mm-dd", JULIAN: "J", RFC_822: "D, d M yy", RFC_850: "DD, dd-M-yy", RFC_1036: "D, d M yy", RFC_1123: "D, d M yyyy", RFC_2822: "D, d M yyyy", RSS: "D, d M yy", TICKS: "!", TIMESTAMP: "@", W3C: "yyyy-mm-dd", formatDate: function(ee, ae, oe) {
              if (typeof ee != "string" && (oe = ae, ae = ee, ee = ""), !ae) return "";
              if (ae.calendar() !== this) throw ne.local.invalidFormat || ne.regionalOptions[""].invalidFormat;
              ee = ee || this.local.dateFormat;
              for (var se, ce, le, ue = (oe = oe || {}).dayNamesShort || this.local.dayNamesShort, de = oe.dayNames || this.local.dayNames, fe = oe.monthNumbers || this.local.monthNumbers, he = oe.monthNamesShort || this.local.monthNamesShort, me = oe.monthNames || this.local.monthNames, pe = (oe.calculateWeek || this.local.calculateWeek, function(Le, Qe) {
                for (var ke = 1; xe + ke < ee.length && ee.charAt(xe + ke) === Le; ) ke++;
                return xe += ke - 1, Math.floor(ke / (Qe || 1)) > 1;
              }), ye = function(Le, Qe, ke, Me) {
                var Re = "" + Qe;
                if (pe(Le, Me)) for (; Re.length < ke; ) Re = "0" + Re;
                return Re;
              }, ge = this, ve = function(Le) {
                return typeof fe == "function" ? fe.call(ge, Le, pe("m")) : Se(ye("m", Le.month(), 2));
              }, be = function(Le, Qe) {
                return Qe ? typeof me == "function" ? me.call(ge, Le) : me[Le.month() - ge.minMonth] : typeof he == "function" ? he.call(ge, Le) : he[Le.month() - ge.minMonth];
              }, Ce = this.local.digits, Se = function(Le) {
                return oe.localNumbers && Ce ? Ce(Le) : Le;
              }, we = "", Ee = !1, xe = 0; xe < ee.length; xe++) if (Ee) ee.charAt(xe) !== "'" || pe("'") ? we += ee.charAt(xe) : Ee = !1;
              else switch (ee.charAt(xe)) {
                case "d":
                  we += Se(ye("d", ae.day(), 2));
                  break;
                case "D":
                  we += (se = ae.dayOfWeek(), ce = ue, le = de, pe("D") ? le[se] : ce[se]);
                  break;
                case "o":
                  we += ye("o", ae.dayOfYear(), 3);
                  break;
                case "w":
                  we += ye("w", ae.weekOfYear(), 2);
                  break;
                case "m":
                  we += ve(ae);
                  break;
                case "M":
                  we += be(ae, pe("M"));
                  break;
                case "y":
                  we += pe("y", 2) ? ae.year() : (ae.year() % 100 < 10 ? "0" : "") + ae.year() % 100;
                  break;
                case "Y":
                  pe("Y", 2), we += ae.formatYear();
                  break;
                case "J":
                  we += ae.toJD();
                  break;
                case "@":
                  we += (ae.toJD() - this.UNIX_EPOCH) * this.SECS_PER_DAY;
                  break;
                case "!":
                  we += (ae.toJD() - this.TICKS_EPOCH) * this.TICKS_PER_DAY;
                  break;
                case "'":
                  pe("'") ? we += "'" : Ee = !0;
                  break;
                default:
                  we += ee.charAt(xe);
              }
              return we;
            }, parseDate: function(ee, ae, oe) {
              if (ae == null) throw ne.local.invalidArguments || ne.regionalOptions[""].invalidArguments;
              if ((ae = typeof ae == "object" ? ae.toString() : ae + "") === "") return null;
              ee = ee || this.local.dateFormat;
              var se = (oe = oe || {}).shortYearCutoff || this.shortYearCutoff;
              se = typeof se != "string" ? se : this.today().year() % 100 + parseInt(se, 10);
              for (var ce = oe.dayNamesShort || this.local.dayNamesShort, le = oe.dayNames || this.local.dayNames, ue = oe.parseMonth || this.local.parseMonth, de = oe.monthNumbers || this.local.monthNumbers, fe = oe.monthNamesShort || this.local.monthNamesShort, he = oe.monthNames || this.local.monthNames, me = -1, pe = -1, ye = -1, ge = -1, ve = -1, be = !1, Ce = !1, Se = function($e, qe) {
                for (var Xe = 1; Re + Xe < ee.length && ee.charAt(Re + Xe) === $e; ) Xe++;
                return Re += Xe - 1, Math.floor(Xe / (qe || 1)) > 1;
              }, we = function($e, qe) {
                var Xe = Se($e, qe), Ve = [2, 3, Xe ? 4 : 2, Xe ? 4 : 2, 10, 11, 20]["oyYJ@!".indexOf($e) + 1], Ze = new RegExp("^-?\\d{1," + Ve + "}"), Ge = ae.substring(Me).match(Ze);
                if (!Ge) throw (ne.local.missingNumberAt || ne.regionalOptions[""].missingNumberAt).replace(/\{0\}/, Me);
                return Me += Ge[0].length, parseInt(Ge[0], 10);
              }, Ee = this, xe = function() {
                if (typeof de == "function") {
                  Se("m");
                  var $e = de.call(Ee, ae.substring(Me));
                  return Me += $e.length, $e;
                }
                return we("m");
              }, Le = function($e, qe, Xe, Ve) {
                for (var Ze = Se($e, Ve) ? Xe : qe, Ge = 0; Ge < Ze.length; Ge++) if (ae.substr(Me, Ze[Ge].length).toLowerCase() === Ze[Ge].toLowerCase()) return Me += Ze[Ge].length, Ge + Ee.minMonth;
                throw (ne.local.unknownNameAt || ne.regionalOptions[""].unknownNameAt).replace(/\{0\}/, Me);
              }, Qe = function() {
                if (typeof he == "function") {
                  var $e = Se("M") ? he.call(Ee, ae.substring(Me)) : fe.call(Ee, ae.substring(Me));
                  return Me += $e.length, $e;
                }
                return Le("M", fe, he);
              }, ke = function() {
                if (ae.charAt(Me) !== ee.charAt(Re)) throw (ne.local.unexpectedLiteralAt || ne.regionalOptions[""].unexpectedLiteralAt).replace(/\{0\}/, Me);
                Me++;
              }, Me = 0, Re = 0; Re < ee.length; Re++) if (Ce) ee.charAt(Re) !== "'" || Se("'") ? ke() : Ce = !1;
              else switch (ee.charAt(Re)) {
                case "d":
                  ge = we("d");
                  break;
                case "D":
                  Le("D", ce, le);
                  break;
                case "o":
                  ve = we("o");
                  break;
                case "w":
                  we("w");
                  break;
                case "m":
                  ye = xe();
                  break;
                case "M":
                  ye = Qe();
                  break;
                case "y":
                  var De = Re;
                  be = !Se("y", 2), Re = De, pe = we("y", 2);
                  break;
                case "Y":
                  pe = we("Y", 2);
                  break;
                case "J":
                  me = we("J") + 0.5, ae.charAt(Me) === "." && (Me++, we("J"));
                  break;
                case "@":
                  me = we("@") / this.SECS_PER_DAY + this.UNIX_EPOCH;
                  break;
                case "!":
                  me = we("!") / this.TICKS_PER_DAY + this.TICKS_EPOCH;
                  break;
                case "*":
                  Me = ae.length;
                  break;
                case "'":
                  Se("'") ? ke() : Ce = !0;
                  break;
                default:
                  ke();
              }
              if (Me < ae.length) throw ne.local.unexpectedText || ne.regionalOptions[""].unexpectedText;
              if (pe === -1 ? pe = this.today().year() : pe < 100 && be && (pe += se === -1 ? 1900 : this.today().year() - this.today().year() % 100 - (pe <= se ? 0 : 100)), typeof ye == "string" && (ye = ue.call(this, pe, ye)), ve > -1) {
                ye = 1, ge = ve;
                for (var Pe = this.daysInMonth(pe, ye); ge > Pe; Pe = this.daysInMonth(pe, ye)) ye++, ge -= Pe;
              }
              return me > -1 ? this.fromJD(me) : this.newDate(pe, ye, ge);
            }, determineDate: function(ee, ae, oe, se, ce) {
              oe && typeof oe != "object" && (ce = se, se = oe, oe = null), typeof se != "string" && (ce = se, se = "");
              var le = this;
              return ae = ae ? ae.newDate() : null, ee == null ? ae : typeof ee == "string" ? function(ue) {
                try {
                  return le.parseDate(se, ue, ce);
                } catch {
                }
                for (var de = ((ue = ue.toLowerCase()).match(/^c/) && oe ? oe.newDate() : null) || le.today(), fe = /([+-]?[0-9]+)\s*(d|w|m|y)?/g, he = fe.exec(ue); he; ) de.add(parseInt(he[1], 10), he[2] || "d"), he = fe.exec(ue);
                return de;
              }(ee) : typeof ee == "number" ? isNaN(ee) || ee === 1 / 0 || ee === -1 / 0 ? ae : le.today().add(ee, "d") : le.newDate(ee);
            } });
          }, 96144: function(G, J, Z) {
            Z.r(J);
            var te = Z(85072), ne = Z.n(te), ee = Z(97825), ae = Z.n(ee), oe = Z(77659), se = Z.n(oe), ce = Z(55056), le = Z.n(ce), ue = Z(10540), de = Z.n(ue), fe = Z(41113), he = Z.n(fe), me = Z(5955), pe = {};
            pe.styleTagTransform = he(), pe.setAttributes = le(), pe.insert = se().bind(null, "head"), pe.domAPI = ae(), pe.insertStyleElement = de(), ne()(me.A, pe), J.default = me.A && me.A.locals ? me.A.locals : void 0;
          }, 85072: function(G) {
            var J = [];
            function Z(ee) {
              for (var ae = -1, oe = 0; oe < J.length; oe++) if (J[oe].identifier === ee) {
                ae = oe;
                break;
              }
              return ae;
            }
            function te(ee, ae) {
              for (var oe = {}, se = [], ce = 0; ce < ee.length; ce++) {
                var le = ee[ce], ue = ae.base ? le[0] + ae.base : le[0], de = oe[ue] || 0, fe = "".concat(ue, " ").concat(de);
                oe[ue] = de + 1;
                var he = Z(fe), me = { css: le[1], media: le[2], sourceMap: le[3], supports: le[4], layer: le[5] };
                if (he !== -1) J[he].references++, J[he].updater(me);
                else {
                  var pe = ne(me, ae);
                  ae.byIndex = ce, J.splice(ce, 0, { identifier: fe, updater: pe, references: 1 });
                }
                se.push(fe);
              }
              return se;
            }
            function ne(ee, ae) {
              var oe = ae.domAPI(ae);
              return oe.update(ee), function(se) {
                if (se) {
                  if (se.css === ee.css && se.media === ee.media && se.sourceMap === ee.sourceMap && se.supports === ee.supports && se.layer === ee.layer) return;
                  oe.update(ee = se);
                } else oe.remove();
              };
            }
            G.exports = function(ee, ae) {
              var oe = te(ee = ee || [], ae = ae || {});
              return function(se) {
                se = se || [];
                for (var ce = 0; ce < oe.length; ce++) {
                  var le = Z(oe[ce]);
                  J[le].references--;
                }
                for (var ue = te(se, ae), de = 0; de < oe.length; de++) {
                  var fe = Z(oe[de]);
                  J[fe].references === 0 && (J[fe].updater(), J.splice(fe, 1));
                }
                oe = ue;
              };
            };
          }, 77659: function(G) {
            var J = {};
            G.exports = function(Z, te) {
              var ne = function(ee) {
                if (J[ee] === void 0) {
                  var ae = document.querySelector(ee);
                  if (window.HTMLIFrameElement && ae instanceof window.HTMLIFrameElement) try {
                    ae = ae.contentDocument.head;
                  } catch {
                    ae = null;
                  }
                  J[ee] = ae;
                }
                return J[ee];
              }(Z);
              if (!ne) throw new Error("Couldn't find a style target. This probably means that the value for the 'insert' parameter is invalid.");
              ne.appendChild(te);
            };
          }, 10540: function(G) {
            G.exports = function(J) {
              var Z = document.createElement("style");
              return J.setAttributes(Z, J.attributes), J.insert(Z, J.options), Z;
            };
          }, 55056: function(G, J, Z) {
            G.exports = function(te) {
              var ne = Z.nc;
              ne && te.setAttribute("nonce", ne);
            };
          }, 97825: function(G) {
            G.exports = function(J) {
              if (typeof document > "u") return { update: function() {
              }, remove: function() {
              } };
              var Z = J.insertStyleElement(J);
              return { update: function(te) {
                (function(ne, ee, ae) {
                  var oe = "";
                  ae.supports && (oe += "@supports (".concat(ae.supports, ") {")), ae.media && (oe += "@media ".concat(ae.media, " {"));
                  var se = ae.layer !== void 0;
                  se && (oe += "@layer".concat(ae.layer.length > 0 ? " ".concat(ae.layer) : "", " {")), oe += ae.css, se && (oe += "}"), ae.media && (oe += "}"), ae.supports && (oe += "}");
                  var ce = ae.sourceMap;
                  ce && typeof btoa < "u" && (oe += `
/*# sourceMappingURL=data:application/json;base64,`.concat(btoa(unescape(encodeURIComponent(JSON.stringify(ce)))), " */")), ee.styleTagTransform(oe, ne, ee.options);
                })(Z, J, te);
              }, remove: function() {
                (function(te) {
                  if (te.parentNode === null) return !1;
                  te.parentNode.removeChild(te);
                })(Z);
              } };
            };
          }, 41113: function(G) {
            G.exports = function(J, Z) {
              if (Z.styleSheet) Z.styleSheet.cssText = J;
              else {
                for (; Z.firstChild; ) Z.removeChild(Z.firstChild);
                Z.appendChild(document.createTextNode(J));
              }
            };
          }, 25446: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2722%27 height=%2722%27 fill=%27%23333%27 viewBox=%270 0 22 22%27%3E%3Cpath d=%27m1.754 13.406 4.453-4.851 3.09 3.09 3.281 3.277.969-.969-3.309-3.312 3.844-4.121 6.148 6.886h1.082v-.855l-7.207-8.07-4.84 5.187L6.169 6.57l-5.48 5.965v.871ZM.688 16.844h20.625v1.375H.688Zm0 0%27/%3E%3C/svg%3E";
          }, 56694: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2722%27 height=%2722%27 fill=%27%2333b5e5%27 viewBox=%270 0 22 22%27%3E%3Cpath d=%27m1.754 13.406 4.453-4.851 3.09 3.09 3.281 3.277.969-.969-3.309-3.312 3.844-4.121 6.148 6.886h1.082v-.855l-7.207-8.07-4.84 5.187L6.169 6.57l-5.48 5.965v.871ZM.688 16.844h20.625v1.375H.688Zm0 0%27/%3E%3C/svg%3E";
          }, 26117: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2724%27 height=%2724%27 fill-rule=%27evenodd%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M4 10a6 6 0 1 0 12 0 6 6 0 1 0-12 0m5-3a1 1 0 1 0 2 0 1 1 0 1 0-2 0m0 3a1 1 0 1 1 2 0v3a1 1 0 1 1-2 0%27/%3E%3C/svg%3E";
          }, 66311: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2724%27 height=%2724%27 fill=%27%23fff%27 fill-rule=%27evenodd%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M4 10a6 6 0 1 0 12 0 6 6 0 1 0-12 0m5-3a1 1 0 1 0 2 0 1 1 0 1 0-2 0m0 3a1 1 0 1 1 2 0v3a1 1 0 1 1-2 0%27/%3E%3C/svg%3E";
          }, 24420: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23333%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3C/svg%3E";
          }, 77035: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23333%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M10 13c-.75 0-1.5.75-1.5 1.5S9.25 16 10 16h9c.75 0 1.5-.75 1.5-1.5S19.75 13 19 13z%27/%3E%3C/svg%3E";
          }, 43470: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23333%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M14.5 8.5c-.75 0-1.5.75-1.5 1.5v3h-3c-.75 0-1.5.75-1.5 1.5S9.25 16 10 16h3v3c0 .75.75 1.5 1.5 1.5S16 19.75 16 19v-3h3c.75 0 1.5-.75 1.5-1.5S19.75 13 19 13h-3v-3c0-.75-.75-1.5-1.5-1.5%27/%3E%3C/svg%3E";
          }, 13490: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23333%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M24 16v5.5c0 1.75-.75 2.5-2.5 2.5H16v-1l3-1.5-4-5.5 1-1 5.5 4 1.5-3zM6 16l1.5 3 5.5-4 1 1-4 5.5 3 1.5v1H7.5C5.75 24 5 23.25 5 21.5V16zm7-11v1l-3 1.5 4 5.5-1 1-5.5-4L6 13H5V7.5C5 5.75 5.75 5 7.5 5zm11 2.5c0-1.75-.75-2.5-2.5-2.5H16v1l3 1.5-4 5.5 1 1 5.5-4 1.5 3h1z%27/%3E%3C/svg%3E";
          }, 80216: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23333%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27m10.5 14 4-8 4 8z%27/%3E%3Cpath fill=%27%23ccc%27 d=%27m10.5 16 4 8 4-8z%27/%3E%3C/svg%3E";
          }, 47695: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%2333b5e5%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3C/svg%3E";
          }, 92228: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%2333b5e5%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3C/svg%3E";
          }, 43737: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23666%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3Cpath fill=%27red%27 d=%27m14 5 1 1-9 9-1-1z%27/%3E%3C/svg%3E";
          }, 48460: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23999%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3Cpath fill=%27red%27 d=%27m14 5 1 1-9 9-1-1z%27/%3E%3C/svg%3E";
          }, 75796: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23aaa%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3Cpath fill=%27red%27 d=%27m14 5 1 1-9 9-1-1z%27/%3E%3C/svg%3E";
          }, 28869: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23e54e33%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3C/svg%3E";
          }, 9819: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23e58978%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3C/svg%3E";
          }, 30557: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23fff%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3C/svg%3E";
          }, 68164: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23fff%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M10 13c-.75 0-1.5.75-1.5 1.5S9.25 16 10 16h9c.75 0 1.5-.75 1.5-1.5S19.75 13 19 13z%27/%3E%3C/svg%3E";
          }, 64665: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23fff%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M14.5 8.5c-.75 0-1.5.75-1.5 1.5v3h-3c-.75 0-1.5.75-1.5 1.5S9.25 16 10 16h3v3c0 .75.75 1.5 1.5 1.5S16 19.75 16 19v-3h3c.75 0 1.5-.75 1.5-1.5S19.75 13 19 13h-3v-3c0-.75-.75-1.5-1.5-1.5%27/%3E%3C/svg%3E";
          }, 91413: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23fff%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M18.5 16c-1.75 0-2.5.75-2.5 2.5V24h1l1.5-3 5.5 4 1-1-4-5.5 3-1.5v-1zM13 18.5c0-1.75-.75-2.5-2.5-2.5H5v1l3 1.5L4 24l1 1 5.5-4 1.5 3h1zm3-8c0 1.75.75 2.5 2.5 2.5H24v-1l-3-1.5L25 5l-1-1-5.5 4L17 5h-1zM10.5 13c1.75 0 2.5-.75 2.5-2.5V5h-1l-1.5 3L5 4 4 5l4 5.5L5 12v1z%27/%3E%3C/svg%3E";
          }, 13913: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23fff%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M24 16v5.5c0 1.75-.75 2.5-2.5 2.5H16v-1l3-1.5-4-5.5 1-1 5.5 4 1.5-3zM6 16l1.5 3 5.5-4 1 1-4 5.5 3 1.5v1H7.5C5.75 24 5 23.25 5 21.5V16zm7-11v1l-3 1.5 4 5.5-1 1-5.5-4L6 13H5V7.5C5 5.75 5.75 5 7.5 5zm11 2.5c0-1.75-.75-2.5-2.5-2.5H16v1l3 1.5-4 5.5 1 1 5.5-4 1.5 3h1z%27/%3E%3C/svg%3E";
          }, 61907: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 fill=%27%23fff%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27m10.5 14 4-8 4 8z%27/%3E%3Cpath fill=%27%23ccc%27 d=%27m10.5 16 4 8 4-8z%27/%3E%3C/svg%3E";
          }, 56539: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 viewBox=%270 0 20 20%27%3E%3Cpath d=%27M10 4C9 4 9 5 9 5v.1A5 5 0 0 0 5.1 9H5s-1 0-1 1 1 1 1 1h.1A5 5 0 0 0 9 14.9v.1s0 1 1 1 1-1 1-1v-.1a5 5 0 0 0 3.9-3.9h.1s1 0 1-1-1-1-1-1h-.1A5 5 0 0 0 11 5.1V5s0-1-1-1m0 2.5a3.5 3.5 0 1 1 0 7 3.5 3.5 0 1 1 0-7%27/%3E%3Ccircle cx=%2710%27 cy=%2710%27 r=%272%27/%3E%3C/svg%3E";
          }, 4890: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M10 13c-.75 0-1.5.75-1.5 1.5S9.25 16 10 16h9c.75 0 1.5-.75 1.5-1.5S19.75 13 19 13z%27/%3E%3C/svg%3E";
          }, 13363: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M14.5 8.5c-.75 0-1.5.75-1.5 1.5v3h-3c-.75 0-1.5.75-1.5 1.5S9.25 16 10 16h3v3c0 .75.75 1.5 1.5 1.5S16 19.75 16 19v-3h3c.75 0 1.5-.75 1.5-1.5S19.75 13 19 13h-3v-3c0-.75-.75-1.5-1.5-1.5%27/%3E%3C/svg%3E";
          }, 47603: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M18.5 16c-1.75 0-2.5.75-2.5 2.5V24h1l1.5-3 5.5 4 1-1-4-5.5 3-1.5v-1zM13 18.5c0-1.75-.75-2.5-2.5-2.5H5v1l3 1.5L4 24l1 1 5.5-4 1.5 3h1zm3-8c0 1.75.75 2.5 2.5 2.5H24v-1l-3-1.5L25 5l-1-1-5.5 4L17 5h-1zM10.5 13c1.75 0 2.5-.75 2.5-2.5V5h-1l-1.5 3L5 4 4 5l4 5.5L5 12v1z%27/%3E%3C/svg%3E";
          }, 64643: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27M24 16v5.5c0 1.75-.75 2.5-2.5 2.5H16v-1l3-1.5-4-5.5 1-1 5.5 4 1.5-3zM6 16l1.5 3 5.5-4 1 1-4 5.5 3 1.5v1H7.5C5.75 24 5 23.25 5 21.5V16zm7-11v1l-3 1.5 4 5.5-1 1-5.5-4L6 13H5V7.5C5 5.75 5.75 5 7.5 5zm11 2.5c0-1.75-.75-2.5-2.5-2.5H16v1l3 1.5-4 5.5 1 1 5.5-4 1.5 3h1z%27/%3E%3C/svg%3E";
          }, 68605: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2729%27 height=%2729%27 viewBox=%270 0 29 29%27%3E%3Cpath d=%27m10.5 14 4-8 4 8z%27/%3E%3Cpath fill=%27%23ccc%27 d=%27m10.5 16 4 8 4-8z%27/%3E%3C/svg%3E";
          }, 47914: function(G) {
            G.exports = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2788%27 height=%2723%27 fill=%27none%27%3E%3Cpath fill=%27%23000%27 fill-opacity=%27.4%27 fill-rule=%27evenodd%27 d=%27M17.408 16.796h-1.827l2.501-12.095h.198l3.324 6.533.988 2.19.988-2.19 3.258-6.533h.181l2.6 12.095h-1.81l-1.218-5.644-.362-1.71-.658 1.71-2.929 5.644h-.098l-2.914-5.644-.757-1.71-.345 1.71zm1.958-3.42-.726 3.663a1.255 1.255 0 0 1-1.232 1.011h-1.827a1.255 1.255 0 0 1-1.229-1.509l2.501-12.095a1.255 1.255 0 0 1 1.23-1.001h.197a1.25 1.25 0 0 1 1.12.685l3.19 6.273 3.125-6.263a1.25 1.25 0 0 1 1.123-.695h.181a1.255 1.255 0 0 1 1.227.991l1.443 6.71a5 5 0 0 1 .314-.787l.009-.016a4.6 4.6 0 0 1 1.777-1.887c.782-.46 1.668-.667 2.611-.667a4.6 4.6 0 0 1 1.7.32l.306.134c.21-.16.474-.256.759-.256h1.694a1.255 1.255 0 0 1 1.212.925 1.255 1.255 0 0 1 1.212-.925h1.711c.284 0 .545.094.755.252.613-.3 1.312-.45 2.075-.45 1.356 0 2.557.445 3.482 1.4q.47.48.763 1.064V4.701a1.255 1.255 0 0 1 1.255-1.255h1.86A1.255 1.255 0 0 1 54.44 4.7v9.194h2.217c.19 0 .37.043.532.118v-4.77c0-.356.147-.678.385-.906a2.42 2.42 0 0 1-.682-1.71c0-.665.267-1.253.735-1.7a2.45 2.45 0 0 1 1.722-.674 2.43 2.43 0 0 1 1.705.675q.318.302.504.683V4.7a1.255 1.255 0 0 1 1.255-1.255h1.744A1.255 1.255 0 0 1 65.812 4.7v3.335a4.8 4.8 0 0 1 1.526-.246c.938 0 1.817.214 2.59.69a4.47 4.47 0 0 1 1.67 1.743v-.98a1.255 1.255 0 0 1 1.256-1.256h1.777c.233 0 .451.064.639.174a3.4 3.4 0 0 1 1.567-.372c.346 0 .861.02 1.285.232a1.25 1.25 0 0 1 .689 1.004 4.7 4.7 0 0 1 .853-.588c.795-.44 1.675-.647 2.61-.647 1.385 0 2.65.39 3.525 1.396.836.938 1.168 2.173 1.168 3.528q-.001.515-.056 1.051a1.255 1.255 0 0 1-.947 1.09l.408.952a1.255 1.255 0 0 1-.477 1.552c-.418.268-.92.463-1.458.612-.613.171-1.304.244-2.049.244-1.06 0-2.043-.207-2.886-.698l-.015-.008c-.798-.48-1.419-1.135-1.818-1.963l-.004-.008a5.8 5.8 0 0 1-.548-2.512q0-.429.053-.843a1.3 1.3 0 0 1-.333-.086l-.166-.004c-.223 0-.426.062-.643.228-.03.024-.142.139-.142.59v3.883a1.255 1.255 0 0 1-1.256 1.256h-1.777a1.255 1.255 0 0 1-1.256-1.256V15.69l-.032.057a4.8 4.8 0 0 1-1.86 1.833 5.04 5.04 0 0 1-2.484.634 4.5 4.5 0 0 1-1.935-.424 1.25 1.25 0 0 1-.764.258h-1.71a1.255 1.255 0 0 1-1.256-1.255V7.687a2.4 2.4 0 0 1-.428.625c.253.23.412.561.412.93v7.553a1.255 1.255 0 0 1-1.256 1.255h-1.843a1.25 1.25 0 0 1-.894-.373c-.228.23-.544.373-.894.373H51.32a1.255 1.255 0 0 1-1.256-1.255v-1.251l-.061.117a4.7 4.7 0 0 1-1.782 1.884 4.77 4.77 0 0 1-2.485.67 5.6 5.6 0 0 1-1.485-.188l.009 2.764a1.255 1.255 0 0 1-1.255 1.259h-1.729a1.255 1.255 0 0 1-1.255-1.255v-3.537a1.255 1.255 0 0 1-1.167.793h-1.679a1.25 1.25 0 0 1-.77-.263 4.5 4.5 0 0 1-1.945.429c-.885 0-1.724-.21-2.495-.632l-.017-.01a5 5 0 0 1-1.081-.836 1.255 1.255 0 0 1-1.254 1.312h-1.81a1.255 1.255 0 0 1-1.228-.99l-.782-3.625-2.044 3.939a1.25 1.25 0 0 1-1.115.676h-.098a1.25 1.25 0 0 1-1.116-.68l-2.061-3.994zM35.92 16.63l.207-.114.223-.15q.493-.356.735-.785l.061-.118.033 1.332h1.678V9.242h-1.694l-.033 1.267q-.133-.329-.526-.658l-.032-.028a3.2 3.2 0 0 0-.668-.428l-.27-.12a3.3 3.3 0 0 0-1.235-.23q-1.136-.001-1.974.493a3.36 3.36 0 0 0-1.3 1.382q-.445.89-.444 2.074 0 1.2.51 2.107a3.8 3.8 0 0 0 1.382 1.381 3.9 3.9 0 0 0 1.893.477q.795 0 1.455-.33zm-2.789-5.38q-.576.675-.575 1.762 0 1.102.559 1.794.576.675 1.645.675a2.25 2.25 0 0 0 .934-.19 2.2 2.2 0 0 0 .468-.29l.178-.161a2.2 2.2 0 0 0 .397-.561q.244-.5.244-1.15v-.115q0-.708-.296-1.267l-.043-.077a2.2 2.2 0 0 0-.633-.709l-.13-.086-.047-.028a2.1 2.1 0 0 0-1.073-.285q-1.052 0-1.629.692zm2.316 2.706c.163-.17.28-.407.28-.83v-.114c0-.292-.06-.508-.15-.68a.96.96 0 0 0-.353-.389.85.85 0 0 0-.464-.127c-.4 0-.56.114-.664.239l-.01.012c-.148.174-.275.45-.275.945 0 .506.122.801.27.99.097.11.266.224.68.224.303 0 .504-.09.687-.269zm7.545 1.705a2.6 2.6 0 0 0 .331.423q.319.33.755.548l.173.074q.65.255 1.49.255 1.02 0 1.844-.493a3.45 3.45 0 0 0 1.316-1.4q.493-.904.493-2.089 0-1.909-.988-2.913-.988-1.02-2.584-1.02-.898 0-1.575.347a3 3 0 0 0-.415.262l-.199.166a3.4 3.4 0 0 0-.64.82V9.242h-1.712v11.553h1.729l-.017-5.134zm.53-1.138q.206.29.48.5l.155.11.053.034q.51.296 1.119.297 1.07 0 1.645-.675.577-.69.576-1.762 0-1.119-.576-1.777-.558-.675-1.645-.675-.435 0-.835.16a2 2 0 0 0-.284.136 2 2 0 0 0-.363.254 2.2 2.2 0 0 0-.46.569l-.082.162a2.6 2.6 0 0 0-.213 1.072v.115q0 .707.296 1.267l.135.211zm.964-.818a1.1 1.1 0 0 0 .367.385.94.94 0 0 0 .476.118c.423 0 .59-.117.687-.23.159-.194.28-.478.28-.95 0-.53-.133-.8-.266-.952l-.021-.025c-.078-.094-.231-.221-.68-.221a1 1 0 0 0-.503.135l-.012.007a.86.86 0 0 0-.335.343c-.073.133-.132.324-.132.614v.115a1.4 1.4 0 0 0 .14.66zm15.7-6.222q.347-.346.346-.856a1.05 1.05 0 0 0-.345-.79 1.18 1.18 0 0 0-.84-.329q-.51 0-.855.33a1.05 1.05 0 0 0-.346.79q0 .51.346.855.345.346.856.346.51 0 .839-.346zm4.337 9.314.033-1.332q.191.403.59.747l.098.081a4 4 0 0 0 .316.224l.223.122a3.2 3.2 0 0 0 1.44.322 3.8 3.8 0 0 0 1.875-.477 3.5 3.5 0 0 0 1.382-1.366q.527-.89.526-2.09 0-1.184-.444-2.073a3.24 3.24 0 0 0-1.283-1.399q-.823-.51-1.942-.51a3.5 3.5 0 0 0-1.527.344l-.086.043-.165.09a3 3 0 0 0-.33.214q-.432.315-.656.707a2 2 0 0 0-.099.198l.082-1.283V4.701h-1.744v12.095zm.473-2.509a2.5 2.5 0 0 0 .566.7q.117.098.245.18l.144.08a2.1 2.1 0 0 0 .975.232q1.07 0 1.645-.675.576-.69.576-1.778 0-1.102-.576-1.777-.56-.691-1.645-.692a2.2 2.2 0 0 0-1.015.235q-.22.113-.415.282l-.15.142a2.1 2.1 0 0 0-.42.594q-.223.479-.223 1.1v.115q0 .705.293 1.26zm2.616-.293c.157-.191.28-.479.28-.967 0-.51-.13-.79-.276-.961l-.021-.026c-.082-.1-.232-.225-.67-.225a.87.87 0 0 0-.681.279l-.012.011c-.154.155-.274.38-.274.807v.115c0 .285.057.499.144.669a1.1 1.1 0 0 0 .367.405c.137.082.28.123.455.123.423 0 .59-.118.686-.23zm8.266-3.013q.345-.13.724-.14l.069-.002q.493 0 .642.099l.247-1.794q-.196-.099-.717-.099a2.3 2.3 0 0 0-.545.063 2 2 0 0 0-.411.148 2.2 2.2 0 0 0-.4.249 2.5 2.5 0 0 0-.485.499 2.7 2.7 0 0 0-.32.581l-.05.137v-1.48h-1.778v7.553h1.777v-3.884q0-.546.159-.943a1.5 1.5 0 0 1 .466-.636 2.5 2.5 0 0 1 .399-.253 2 2 0 0 1 .224-.099zm9.784 2.656.05-.922q0-1.743-.856-2.698-.838-.97-2.584-.97-1.119-.001-2.007.493a3.46 3.46 0 0 0-1.4 1.382q-.493.906-.493 2.106 0 1.07.428 1.975.428.89 1.332 1.432.906.526 2.255.526.973 0 1.668-.185l.044-.012.135-.04q.613-.184.984-.421l-.542-1.267q-.3.162-.642.274l-.297.087q-.51.131-1.3.131-.954 0-1.497-.444a1.6 1.6 0 0 1-.192-.193q-.366-.44-.512-1.234l-.004-.021zm-5.427-1.256-.003.022h3.752v-.138q-.011-.727-.288-1.118a1 1 0 0 0-.156-.176q-.46-.428-1.316-.428-.986 0-1.494.604-.379.45-.494 1.234zm-27.053 2.77V4.7h-1.86v12.095h5.333V15.15zm7.103-5.908v7.553h-1.843V9.242h1.843z%27/%3E%3Cpath fill=%27%23fff%27 d=%27m19.63 11.151-.757-1.71-.345 1.71-1.12 5.644h-1.827L18.083 4.7h.197l3.325 6.533.988 2.19.988-2.19L26.839 4.7h.181l2.6 12.095h-1.81l-1.218-5.644-.362-1.71-.658 1.71-2.93 5.644h-.098l-2.913-5.644zm14.836 5.81q-1.02 0-1.893-.478a3.8 3.8 0 0 1-1.381-1.382q-.51-.906-.51-2.106 0-1.185.444-2.074a3.36 3.36 0 0 1 1.3-1.382q.839-.494 1.974-.494a3.3 3.3 0 0 1 1.234.231 3.3 3.3 0 0 1 .97.575q.396.33.527.659l.033-1.267h1.694v7.553H37.18l-.033-1.332q-.279.593-1.02 1.053a3.17 3.17 0 0 1-1.662.444zm.296-1.482q.938 0 1.58-.642.642-.66.642-1.711v-.115q0-.708-.296-1.267a2.2 2.2 0 0 0-.807-.872 2.1 2.1 0 0 0-1.119-.313q-1.053 0-1.629.692-.575.675-.575 1.76 0 1.103.559 1.795.577.675 1.645.675zm6.521-6.237h1.711v1.4q.906-1.597 2.83-1.597 1.596 0 2.584 1.02.988 1.005.988 2.914 0 1.185-.493 2.09a3.46 3.46 0 0 1-1.316 1.399 3.5 3.5 0 0 1-1.844.493q-.954 0-1.662-.329a2.67 2.67 0 0 1-1.086-.97l.017 5.134h-1.728zm4.048 6.22q1.07 0 1.645-.674.577-.69.576-1.762 0-1.119-.576-1.777-.558-.675-1.645-.675-.592 0-1.12.296-.51.28-.822.823-.296.527-.296 1.234v.115q0 .708.296 1.267.313.543.823.855.51.296 1.119.297z%27/%3E%3Cpath fill=%27%23e1e3e9%27 d=%27M51.325 4.7h1.86v10.45h3.473v1.646h-5.333zm7.12 4.542h1.843v7.553h-1.843zm.905-1.415a1.16 1.16 0 0 1-.856-.346 1.17 1.17 0 0 1-.346-.856 1.05 1.05 0 0 1 .346-.79q.346-.329.856-.329.494 0 .839.33a1.05 1.05 0 0 1 .345.79 1.16 1.16 0 0 1-.345.855q-.33.346-.84.346zm7.875 9.133a3.17 3.17 0 0 1-1.662-.444q-.723-.46-1.004-1.053l-.033 1.332h-1.71V4.701h1.743v4.657l-.082 1.283q.279-.658 1.086-1.119a3.5 3.5 0 0 1 1.778-.477q1.119 0 1.942.51a3.24 3.24 0 0 1 1.283 1.4q.445.888.444 2.072 0 1.201-.526 2.09a3.5 3.5 0 0 1-1.382 1.366 3.8 3.8 0 0 1-1.876.477zm-.296-1.481q1.069 0 1.645-.675.577-.69.577-1.778 0-1.102-.577-1.776-.56-.691-1.645-.692a2.12 2.12 0 0 0-1.58.659q-.642.641-.642 1.694v.115q0 .71.296 1.267a2.4 2.4 0 0 0 .807.872 2.1 2.1 0 0 0 1.119.313zm5.927-6.237h1.777v1.481q.263-.757.856-1.217a2.14 2.14 0 0 1 1.349-.46q.527 0 .724.098l-.247 1.794q-.149-.099-.642-.099-.774 0-1.416.494-.626.493-.626 1.58v3.883h-1.777V9.242zm9.534 7.718q-1.35 0-2.255-.526-.904-.543-1.332-1.432a4.6 4.6 0 0 1-.428-1.975q0-1.2.493-2.106a3.46 3.46 0 0 1 1.4-1.382q.889-.495 2.007-.494 1.744 0 2.584.97.855.956.856 2.7 0 .444-.05.92h-5.43q.18 1.005.708 1.45.542.443 1.497.443.79 0 1.3-.131a4 4 0 0 0 .938-.362l.542 1.267q-.411.263-1.119.46-.708.198-1.711.197zm1.596-4.558q.016-1.02-.444-1.432-.46-.428-1.316-.428-1.728 0-1.991 1.86z%27/%3E%3Cpath d=%27M5.074 15.948a.484.657 0 0 0-.486.659v1.84a.484.657 0 0 0 .486.659h4.101a.484.657 0 0 0 .486-.659v-1.84a.484.657 0 0 0-.486-.659zm3.56 1.16H5.617v.838h3.017z%27 style=%27fill:%23fff;fill-rule:evenodd;stroke-width:1.03600001%27/%3E%3Cg style=%27stroke-width:1.12603545%27%3E%3Cpath d=%27M-9.408-1.416c-3.833-.025-7.056 2.912-7.08 6.615-.02 3.08 1.653 4.832 3.107 6.268.903.892 1.721 1.74 2.32 2.902l-.525-.004c-.543-.003-.992.304-1.24.639a1.87 1.87 0 0 0-.362 1.121l-.011 1.877c-.003.402.104.787.347 1.125.244.338.688.653 1.23.656l4.142.028c.542.003.99-.306 1.238-.641a1.87 1.87 0 0 0 .363-1.121l.012-1.875a1.87 1.87 0 0 0-.348-1.127c-.243-.338-.688-.653-1.23-.656l-.518-.004c.597-1.145 1.425-1.983 2.348-2.87 1.473-1.414 3.18-3.149 3.2-6.226-.016-3.59-2.923-6.684-6.993-6.707m-.006 1.1v.002c3.274.02 5.92 2.532 5.9 5.6-.017 2.706-1.39 4.026-2.863 5.44-1.034.994-2.118 2.033-2.814 3.633-.018.041-.052.055-.075.065q-.013.004-.02.01a.34.34 0 0 1-.226.084.34.34 0 0 1-.224-.086l-.092-.077c-.699-1.615-1.768-2.669-2.781-3.67-1.454-1.435-2.797-2.762-2.78-5.478.02-3.067 2.7-5.545 5.975-5.523m-.02 2.826c-1.62-.01-2.944 1.315-2.955 2.96-.01 1.646 1.295 2.988 2.916 2.999h.002c1.621.01 2.943-1.316 2.953-2.961.011-1.646-1.294-2.988-2.916-2.998m-.005 1.1c1.017.006 1.829.83 1.822 1.89s-.83 1.874-1.848 1.867c-1.018-.006-1.829-.83-1.822-1.89s.83-1.874 1.848-1.868m-2.155 11.857 4.14.025c.271.002.49.305.487.676l-.013 1.875c-.003.37-.224.67-.495.668l-4.14-.025c-.27-.002-.487-.306-.485-.676l.012-1.875c.003-.37.224-.67.494-.668%27 style=%27color:%23000;font-style:normal;font-variant:normal;font-weight:400;font-stretch:normal;font-size:medium;line-height:normal;font-family:sans-serif;font-variant-ligatures:normal;font-variant-position:normal;font-variant-caps:normal;font-variant-numeric:normal;font-variant-alternates:normal;font-feature-settings:normal;text-indent:0;text-align:start;text-decoration:none;text-decoration-line:none;text-decoration-style:solid;text-decoration-color:%23000;letter-spacing:normal;word-spacing:normal;text-transform:none;writing-mode:lr-tb;direction:ltr;text-orientation:mixed;dominant-baseline:auto;baseline-shift:baseline;text-anchor:start;white-space:normal;shape-padding:0;clip-rule:evenodd;display:inline;overflow:visible;visibility:visible;opacity:1;isolation:auto;mix-blend-mode:normal;color-interpolation:sRGB;color-interpolation-filters:linearRGB;solid-color:%23000;solid-opacity:1;vector-effect:none;fill:%23000;fill-opacity:.4;fill-rule:evenodd;stroke:none;stroke-width:2.47727823;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;color-rendering:auto;image-rendering:auto;shape-rendering:auto;text-rendering:auto%27 transform=%27translate%2815.553 2.85%29scale%28.88807%29%27/%3E%3Cpath d=%27M-9.415-.316C-12.69-.338-15.37 2.14-15.39 5.207c-.017 2.716 1.326 4.041 2.78 5.477 1.013 1 2.081 2.055 2.78 3.67l.092.076a.34.34 0 0 0 .225.086.34.34 0 0 0 .227-.083l.019-.01c.022-.009.057-.024.074-.064.697-1.6 1.78-2.64 2.814-3.634 1.473-1.414 2.847-2.733 2.864-5.44.02-3.067-2.627-5.58-5.901-5.601m-.057 8.784c1.621.011 2.944-1.315 2.955-2.96.01-1.646-1.295-2.988-2.916-2.999-1.622-.01-2.945 1.315-2.955 2.96s1.295 2.989 2.916 3%27 style=%27clip-rule:evenodd;fill:%23e1e3e9;fill-opacity:1;fill-rule:evenodd;stroke:none;stroke-width:2.47727823;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:.4%27 transform=%27translate%2815.553 2.85%29scale%28.88807%29%27/%3E%3Cpath d=%27M-11.594 15.465c-.27-.002-.492.297-.494.668l-.012 1.876c-.003.371.214.673.485.675l4.14.027c.271.002.492-.298.495-.668l.012-1.877c.003-.37-.215-.672-.485-.674z%27 style=%27clip-rule:evenodd;fill:%23fff;fill-opacity:1;fill-rule:evenodd;stroke:none;stroke-width:2.47727823;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:.4%27 transform=%27translate%2815.553 2.85%29scale%28.88807%29%27/%3E%3C/g%3E%3C/svg%3E";
          }, 63779: function() {
          }, 77199: function() {
          }, 61990: function(G, J, Z) {
            Object.defineProperty(J, "__esModule", { value: !0 });
            var te = Z(85846), ne = Z(66030);
            function ee(ue) {
              return ne.geomReduce.call(void 0, ue, (de, fe) => de + function(he) {
                let me, pe = 0;
                switch (he.type) {
                  case "Polygon":
                    return ae(he.coordinates);
                  case "MultiPolygon":
                    for (me = 0; me < he.coordinates.length; me++) pe += ae(he.coordinates[me]);
                    return pe;
                  case "Point":
                  case "MultiPoint":
                  case "LineString":
                  case "MultiLineString":
                    return 0;
                }
                return 0;
              }(fe), 0);
            }
            function ae(ue) {
              let de = 0;
              if (ue && ue.length > 0) {
                de += Math.abs(ce(ue[0]));
                for (let fe = 1; fe < ue.length; fe++) de -= Math.abs(ce(ue[fe]));
              }
              return de;
            }
            var oe = te.earthRadius * te.earthRadius / 2, se = Math.PI / 180;
            function ce(ue) {
              const de = ue.length - 1;
              if (de <= 2) return 0;
              let fe = 0, he = 0;
              for (; he < de; ) {
                const me = ue[he], pe = ue[he + 1 === de ? 0 : he + 1], ye = ue[he + 2 >= de ? (he + 2) % de : he + 2], ge = me[0] * se, ve = pe[1] * se;
                fe += (ye[0] * se - ge) * Math.sin(ve), he++;
              }
              return fe * oe;
            }
            var le = ee;
            J.area = ee, J.default = le;
          }, 25368: function(G, J, Z) {
            Object.defineProperty(J, "__esModule", { value: !0 });
            var te = Z(66030);
            function ne(ae, oe = {}) {
              if (ae.bbox != null && oe.recompute !== !0) return ae.bbox;
              const se = [1 / 0, 1 / 0, -1 / 0, -1 / 0];
              return te.coordEach.call(void 0, ae, (ce) => {
                se[0] > ce[0] && (se[0] = ce[0]), se[1] > ce[1] && (se[1] = ce[1]), se[2] < ce[0] && (se[2] = ce[0]), se[3] < ce[1] && (se[3] = ce[1]);
              }), se;
            }
            var ee = ne;
            J.bbox = ne, J.default = ee;
          }, 30035: function(G, J, Z) {
            Object.defineProperty(J, "__esModule", { value: !0 });
            var te = Z(85846), ne = Z(66030);
            function ee(oe, se = {}) {
              let ce = 0, le = 0, ue = 0;
              return ne.coordEach.call(void 0, oe, function(de) {
                ce += de[0], le += de[1], ue++;
              }, !0), te.point.call(void 0, [ce / ue, le / ue], se.properties);
            }
            var ae = ee;
            J.centroid = ee, J.default = ae;
          }, 85846: function(G, J) {
            Object.defineProperty(J, "__esModule", { value: !0 });
            var Z = 63710088e-1, te = { centimeters: 637100880, centimetres: 637100880, degrees: 360 / (2 * Math.PI), feet: 20902260511392e-6, inches: 39.37 * Z, kilometers: 6371.0088, kilometres: 6371.0088, meters: Z, metres: Z, miles: 3958.761333810546, millimeters: 6371008800, millimetres: 6371008800, nauticalmiles: Z / 1852, radians: 1, yards: 6967335223679999e-9 }, ne = { acres: 247105e-9, centimeters: 1e4, centimetres: 1e4, feet: 10.763910417, hectares: 1e-4, inches: 1550.003100006, kilometers: 1e-6, kilometres: 1e-6, meters: 1, metres: 1, miles: 386e-9, nauticalmiles: 29155334959812285e-23, millimeters: 1e6, millimetres: 1e6, yards: 1.195990046 };
            function ee(ye, ge, ve = {}) {
              const be = { type: "Feature" };
              return (ve.id === 0 || ve.id) && (be.id = ve.id), ve.bbox && (be.bbox = ve.bbox), be.properties = ge || {}, be.geometry = ye, be;
            }
            function ae(ye, ge, ve = {}) {
              if (!ye) throw new Error("coordinates is required");
              if (!Array.isArray(ye)) throw new Error("coordinates must be an Array");
              if (ye.length < 2) throw new Error("coordinates must be at least 2 numbers long");
              if (!pe(ye[0]) || !pe(ye[1])) throw new Error("coordinates must contain numbers");
              return ee({ type: "Point", coordinates: ye }, ge, ve);
            }
            function oe(ye, ge, ve = {}) {
              for (const be of ye) {
                if (be.length < 4) throw new Error("Each LinearRing of a Polygon must have 4 or more Positions.");
                if (be[be.length - 1].length !== be[0].length) throw new Error("First and last Position are not equivalent.");
                for (let Ce = 0; Ce < be[be.length - 1].length; Ce++) if (be[be.length - 1][Ce] !== be[0][Ce]) throw new Error("First and last Position are not equivalent.");
              }
              return ee({ type: "Polygon", coordinates: ye }, ge, ve);
            }
            function se(ye, ge, ve = {}) {
              if (ye.length < 2) throw new Error("coordinates must be an array of two or more positions");
              return ee({ type: "LineString", coordinates: ye }, ge, ve);
            }
            function ce(ye, ge = {}) {
              const ve = { type: "FeatureCollection" };
              return ge.id && (ve.id = ge.id), ge.bbox && (ve.bbox = ge.bbox), ve.features = ye, ve;
            }
            function le(ye, ge, ve = {}) {
              return ee({ type: "MultiLineString", coordinates: ye }, ge, ve);
            }
            function ue(ye, ge, ve = {}) {
              return ee({ type: "MultiPoint", coordinates: ye }, ge, ve);
            }
            function de(ye, ge, ve = {}) {
              return ee({ type: "MultiPolygon", coordinates: ye }, ge, ve);
            }
            function fe(ye, ge = "kilometers") {
              const ve = te[ge];
              if (!ve) throw new Error(ge + " units is invalid");
              return ye * ve;
            }
            function he(ye, ge = "kilometers") {
              const ve = te[ge];
              if (!ve) throw new Error(ge + " units is invalid");
              return ye / ve;
            }
            function me(ye) {
              return ye % (2 * Math.PI) * 180 / Math.PI;
            }
            function pe(ye) {
              return !isNaN(ye) && ye !== null && !Array.isArray(ye);
            }
            J.areaFactors = ne, J.azimuthToBearing = function(ye) {
              return (ye %= 360) > 0 ? ye > 180 ? ye - 360 : ye : ye < -180 ? ye + 360 : ye;
            }, J.bearingToAzimuth = function(ye) {
              let ge = ye % 360;
              return ge < 0 && (ge += 360), ge;
            }, J.convertArea = function(ye, ge = "meters", ve = "kilometers") {
              if (!(ye >= 0)) throw new Error("area must be a positive number");
              const be = ne[ge];
              if (!be) throw new Error("invalid original units");
              const Ce = ne[ve];
              if (!Ce) throw new Error("invalid final units");
              return ye / be * Ce;
            }, J.convertLength = function(ye, ge = "kilometers", ve = "kilometers") {
              if (!(ye >= 0)) throw new Error("length must be a positive number");
              return fe(he(ye, ge), ve);
            }, J.degreesToRadians = function(ye) {
              return ye % 360 * Math.PI / 180;
            }, J.earthRadius = Z, J.factors = te, J.feature = ee, J.featureCollection = ce, J.geometry = function(ye, ge, ve = {}) {
              switch (ye) {
                case "Point":
                  return ae(ge).geometry;
                case "LineString":
                  return se(ge).geometry;
                case "Polygon":
                  return oe(ge).geometry;
                case "MultiPoint":
                  return ue(ge).geometry;
                case "MultiLineString":
                  return le(ge).geometry;
                case "MultiPolygon":
                  return de(ge).geometry;
                default:
                  throw new Error(ye + " is invalid");
              }
            }, J.geometryCollection = function(ye, ge, ve = {}) {
              return ee({ type: "GeometryCollection", geometries: ye }, ge, ve);
            }, J.isNumber = pe, J.isObject = function(ye) {
              return ye !== null && typeof ye == "object" && !Array.isArray(ye);
            }, J.lengthToDegrees = function(ye, ge) {
              return me(he(ye, ge));
            }, J.lengthToRadians = he, J.lineString = se, J.lineStrings = function(ye, ge, ve = {}) {
              return ce(ye.map((be) => se(be, ge)), ve);
            }, J.multiLineString = le, J.multiPoint = ue, J.multiPolygon = de, J.point = ae, J.points = function(ye, ge, ve = {}) {
              return ce(ye.map((be) => ae(be, ge)), ve);
            }, J.polygon = oe, J.polygons = function(ye, ge, ve = {}) {
              return ce(ye.map((be) => oe(be, ge)), ve);
            }, J.radiansToDegrees = me, J.radiansToLength = fe, J.round = function(ye, ge = 0) {
              if (ge && !(ge >= 0)) throw new Error("precision must be a positive number");
              const ve = Math.pow(10, ge || 0);
              return Math.round(ye * ve) / ve;
            }, J.validateBBox = function(ye) {
              if (!ye) throw new Error("bbox is required");
              if (!Array.isArray(ye)) throw new Error("bbox must be an Array");
              if (ye.length !== 4 && ye.length !== 6) throw new Error("bbox must be an Array of 4 or 6 numbers");
              ye.forEach((ge) => {
                if (!pe(ge)) throw new Error("bbox must only contain numbers");
              });
            }, J.validateId = function(ye) {
              if (!ye) throw new Error("id is required");
              if (["string", "number"].indexOf(typeof ye) === -1) throw new Error("id must be a number or a string");
            };
          }, 66030: function(G, J, Z) {
            Object.defineProperty(J, "__esModule", { value: !0 });
            var te = Z(85846);
            function ne(ue, de, fe) {
              if (ue !== null) for (var he, me, pe, ye, ge, ve, be, Ce, Se = 0, we = 0, Ee = ue.type, xe = Ee === "FeatureCollection", Le = Ee === "Feature", Qe = xe ? ue.features.length : 1, ke = 0; ke < Qe; ke++) {
                ge = (Ce = !!(be = xe ? ue.features[ke].geometry : Le ? ue.geometry : ue) && be.type === "GeometryCollection") ? be.geometries.length : 1;
                for (var Me = 0; Me < ge; Me++) {
                  var Re = 0, De = 0;
                  if ((ye = Ce ? be.geometries[Me] : be) !== null) {
                    ve = ye.coordinates;
                    var Pe = ye.type;
                    switch (Se = !fe || Pe !== "Polygon" && Pe !== "MultiPolygon" ? 0 : 1, Pe) {
                      case null:
                        break;
                      case "Point":
                        if (de(ve, we, ke, Re, De) === !1) return !1;
                        we++, Re++;
                        break;
                      case "LineString":
                      case "MultiPoint":
                        for (he = 0; he < ve.length; he++) {
                          if (de(ve[he], we, ke, Re, De) === !1) return !1;
                          we++, Pe === "MultiPoint" && Re++;
                        }
                        Pe === "LineString" && Re++;
                        break;
                      case "Polygon":
                      case "MultiLineString":
                        for (he = 0; he < ve.length; he++) {
                          for (me = 0; me < ve[he].length - Se; me++) {
                            if (de(ve[he][me], we, ke, Re, De) === !1) return !1;
                            we++;
                          }
                          Pe === "MultiLineString" && Re++, Pe === "Polygon" && De++;
                        }
                        Pe === "Polygon" && Re++;
                        break;
                      case "MultiPolygon":
                        for (he = 0; he < ve.length; he++) {
                          for (De = 0, me = 0; me < ve[he].length; me++) {
                            for (pe = 0; pe < ve[he][me].length - Se; pe++) {
                              if (de(ve[he][me][pe], we, ke, Re, De) === !1) return !1;
                              we++;
                            }
                            De++;
                          }
                          Re++;
                        }
                        break;
                      case "GeometryCollection":
                        for (he = 0; he < ye.geometries.length; he++) if (ne(ye.geometries[he], de, fe) === !1) return !1;
                        break;
                      default:
                        throw new Error("Unknown Geometry Type");
                    }
                  }
                }
              }
            }
            function ee(ue, de) {
              var fe;
              switch (ue.type) {
                case "FeatureCollection":
                  for (fe = 0; fe < ue.features.length && de(ue.features[fe].properties, fe) !== !1; fe++) ;
                  break;
                case "Feature":
                  de(ue.properties, 0);
              }
            }
            function ae(ue, de) {
              if (ue.type === "Feature") de(ue, 0);
              else if (ue.type === "FeatureCollection") for (var fe = 0; fe < ue.features.length && de(ue.features[fe], fe) !== !1; fe++) ;
            }
            function oe(ue, de) {
              var fe, he, me, pe, ye, ge, ve, be, Ce, Se, we = 0, Ee = ue.type === "FeatureCollection", xe = ue.type === "Feature", Le = Ee ? ue.features.length : 1;
              for (fe = 0; fe < Le; fe++) {
                for (ge = Ee ? ue.features[fe].geometry : xe ? ue.geometry : ue, be = Ee ? ue.features[fe].properties : xe ? ue.properties : {}, Ce = Ee ? ue.features[fe].bbox : xe ? ue.bbox : void 0, Se = Ee ? ue.features[fe].id : xe ? ue.id : void 0, ye = (ve = !!ge && ge.type === "GeometryCollection") ? ge.geometries.length : 1, me = 0; me < ye; me++) if ((pe = ve ? ge.geometries[me] : ge) !== null) switch (pe.type) {
                  case "Point":
                  case "LineString":
                  case "MultiPoint":
                  case "Polygon":
                  case "MultiLineString":
                  case "MultiPolygon":
                    if (de(pe, we, be, Ce, Se) === !1) return !1;
                    break;
                  case "GeometryCollection":
                    for (he = 0; he < pe.geometries.length; he++) if (de(pe.geometries[he], we, be, Ce, Se) === !1) return !1;
                    break;
                  default:
                    throw new Error("Unknown Geometry Type");
                }
                else if (de(null, we, be, Ce, Se) === !1) return !1;
                we++;
              }
            }
            function se(ue, de) {
              oe(ue, function(fe, he, me, pe, ye) {
                var ge, ve = fe === null ? null : fe.type;
                switch (ve) {
                  case null:
                  case "Point":
                  case "LineString":
                  case "Polygon":
                    return de(te.feature.call(void 0, fe, me, { bbox: pe, id: ye }), he, 0) !== !1 && void 0;
                }
                switch (ve) {
                  case "MultiPoint":
                    ge = "Point";
                    break;
                  case "MultiLineString":
                    ge = "LineString";
                    break;
                  case "MultiPolygon":
                    ge = "Polygon";
                }
                for (var be = 0; be < fe.coordinates.length; be++) {
                  var Ce = { type: ge, coordinates: fe.coordinates[be] };
                  if (de(te.feature.call(void 0, Ce, me), he, be) === !1) return !1;
                }
              });
            }
            function ce(ue, de) {
              se(ue, function(fe, he, me) {
                var pe = 0;
                if (fe.geometry) {
                  var ye = fe.geometry.type;
                  if (ye !== "Point" && ye !== "MultiPoint") {
                    var ge, ve = 0, be = 0, Ce = 0;
                    return ne(fe, function(Se, we, Ee, xe, Le) {
                      if (ge === void 0 || he > ve || xe > be || Le > Ce) return ge = Se, ve = he, be = xe, Ce = Le, void (pe = 0);
                      var Qe = te.lineString.call(void 0, [ge, Se], fe.properties);
                      if (de(Qe, he, me, Le, pe) === !1) return !1;
                      pe++, ge = Se;
                    }) !== !1 && void 0;
                  }
                }
              });
            }
            function le(ue, de) {
              if (!ue) throw new Error("geojson is required");
              se(ue, function(fe, he, me) {
                if (fe.geometry !== null) {
                  var pe = fe.geometry.type, ye = fe.geometry.coordinates;
                  switch (pe) {
                    case "LineString":
                      if (de(fe, he, me, 0, 0) === !1) return !1;
                      break;
                    case "Polygon":
                      for (var ge = 0; ge < ye.length; ge++) if (de(te.lineString.call(void 0, ye[ge], fe.properties), he, me, ge) === !1) return !1;
                  }
                }
              });
            }
            J.coordAll = function(ue) {
              var de = [];
              return ne(ue, function(fe) {
                de.push(fe);
              }), de;
            }, J.coordEach = ne, J.coordReduce = function(ue, de, fe, he) {
              var me = fe;
              return ne(ue, function(pe, ye, ge, ve, be) {
                me = ye === 0 && fe === void 0 ? pe : de(me, pe, ye, ge, ve, be);
              }, he), me;
            }, J.featureEach = ae, J.featureReduce = function(ue, de, fe) {
              var he = fe;
              return ae(ue, function(me, pe) {
                he = pe === 0 && fe === void 0 ? me : de(he, me, pe);
              }), he;
            }, J.findPoint = function(ue, de) {
              if (de = de || {}, !te.isObject.call(void 0, de)) throw new Error("options is invalid");
              var fe, he = de.featureIndex || 0, me = de.multiFeatureIndex || 0, pe = de.geometryIndex || 0, ye = de.coordIndex || 0, ge = de.properties;
              switch (ue.type) {
                case "FeatureCollection":
                  he < 0 && (he = ue.features.length + he), ge = ge || ue.features[he].properties, fe = ue.features[he].geometry;
                  break;
                case "Feature":
                  ge = ge || ue.properties, fe = ue.geometry;
                  break;
                case "Point":
                case "MultiPoint":
                  return null;
                case "LineString":
                case "Polygon":
                case "MultiLineString":
                case "MultiPolygon":
                  fe = ue;
                  break;
                default:
                  throw new Error("geojson is invalid");
              }
              if (fe === null) return null;
              var ve = fe.coordinates;
              switch (fe.type) {
                case "Point":
                  return te.point.call(void 0, ve, ge, de);
                case "MultiPoint":
                  return me < 0 && (me = ve.length + me), te.point.call(void 0, ve[me], ge, de);
                case "LineString":
                  return ye < 0 && (ye = ve.length + ye), te.point.call(void 0, ve[ye], ge, de);
                case "Polygon":
                  return pe < 0 && (pe = ve.length + pe), ye < 0 && (ye = ve[pe].length + ye), te.point.call(void 0, ve[pe][ye], ge, de);
                case "MultiLineString":
                  return me < 0 && (me = ve.length + me), ye < 0 && (ye = ve[me].length + ye), te.point.call(void 0, ve[me][ye], ge, de);
                case "MultiPolygon":
                  return me < 0 && (me = ve.length + me), pe < 0 && (pe = ve[me].length + pe), ye < 0 && (ye = ve[me][pe].length - ye), te.point.call(void 0, ve[me][pe][ye], ge, de);
              }
              throw new Error("geojson is invalid");
            }, J.findSegment = function(ue, de) {
              if (de = de || {}, !te.isObject.call(void 0, de)) throw new Error("options is invalid");
              var fe, he = de.featureIndex || 0, me = de.multiFeatureIndex || 0, pe = de.geometryIndex || 0, ye = de.segmentIndex || 0, ge = de.properties;
              switch (ue.type) {
                case "FeatureCollection":
                  he < 0 && (he = ue.features.length + he), ge = ge || ue.features[he].properties, fe = ue.features[he].geometry;
                  break;
                case "Feature":
                  ge = ge || ue.properties, fe = ue.geometry;
                  break;
                case "Point":
                case "MultiPoint":
                  return null;
                case "LineString":
                case "Polygon":
                case "MultiLineString":
                case "MultiPolygon":
                  fe = ue;
                  break;
                default:
                  throw new Error("geojson is invalid");
              }
              if (fe === null) return null;
              var ve = fe.coordinates;
              switch (fe.type) {
                case "Point":
                case "MultiPoint":
                  return null;
                case "LineString":
                  return ye < 0 && (ye = ve.length + ye - 1), te.lineString.call(void 0, [ve[ye], ve[ye + 1]], ge, de);
                case "Polygon":
                  return pe < 0 && (pe = ve.length + pe), ye < 0 && (ye = ve[pe].length + ye - 1), te.lineString.call(void 0, [ve[pe][ye], ve[pe][ye + 1]], ge, de);
                case "MultiLineString":
                  return me < 0 && (me = ve.length + me), ye < 0 && (ye = ve[me].length + ye - 1), te.lineString.call(void 0, [ve[me][ye], ve[me][ye + 1]], ge, de);
                case "MultiPolygon":
                  return me < 0 && (me = ve.length + me), pe < 0 && (pe = ve[me].length + pe), ye < 0 && (ye = ve[me][pe].length - ye - 1), te.lineString.call(void 0, [ve[me][pe][ye], ve[me][pe][ye + 1]], ge, de);
              }
              throw new Error("geojson is invalid");
            }, J.flattenEach = se, J.flattenReduce = function(ue, de, fe) {
              var he = fe;
              return se(ue, function(me, pe, ye) {
                he = pe === 0 && ye === 0 && fe === void 0 ? me : de(he, me, pe, ye);
              }), he;
            }, J.geomEach = oe, J.geomReduce = function(ue, de, fe) {
              var he = fe;
              return oe(ue, function(me, pe, ye, ge, ve) {
                he = pe === 0 && fe === void 0 ? me : de(he, me, pe, ye, ge, ve);
              }), he;
            }, J.lineEach = le, J.lineReduce = function(ue, de, fe) {
              var he = fe;
              return le(ue, function(me, pe, ye, ge) {
                he = pe === 0 && fe === void 0 ? me : de(he, me, pe, ye, ge);
              }), he;
            }, J.propEach = ee, J.propReduce = function(ue, de, fe) {
              var he = fe;
              return ee(ue, function(me, pe) {
                he = pe === 0 && fe === void 0 ? me : de(he, me, pe);
              }), he;
            }, J.segmentEach = ce, J.segmentReduce = function(ue, de, fe) {
              var he = fe, me = !1;
              return ce(ue, function(pe, ye, ge, ve, be) {
                he = me === !1 && fe === void 0 ? pe : de(he, pe, ye, ge, ve, be), me = !0;
              }), he;
            };
          }, 70085: function(G, J, Z) {
            var te = ["BigInt64Array", "BigUint64Array", "Float32Array", "Float64Array", "Int16Array", "Int32Array", "Int8Array", "Uint16Array", "Uint32Array", "Uint8Array", "Uint8ClampedArray"], ne = typeof globalThis > "u" ? Z.g : globalThis;
            G.exports = function() {
              for (var ee = [], ae = 0; ae < te.length; ae++) typeof ne[te[ae]] == "function" && (ee[ee.length] = te[ae]);
              return ee;
            };
          }, 89380: function(G) {
            G.exports = function() {
              var J = {}, Z = {};
              function te(ne, ee, ae) {
                if (Z[ne] = ae, ne === "index") {
                  var oe = "var sharedModule = {}; (" + Z.shared + ")(sharedModule); (" + Z.worker + ")(sharedModule);", se = {};
                  return Z.shared(se), Z.index(J, se), typeof window < "u" && J.setWorkerUrl(window.URL.createObjectURL(new Blob([oe], { type: "text/javascript" }))), J;
                }
              }
              return te("shared", 0, function(ne) {
                function ee(Ye, He, ot, xt) {
                  return new (ot || (ot = Promise))(function(zt, ln) {
                    function pn(Wn) {
                      try {
                        Nn(xt.next(Wn));
                      } catch (lr) {
                        ln(lr);
                      }
                    }
                    function Cn(Wn) {
                      try {
                        Nn(xt.throw(Wn));
                      } catch (lr) {
                        ln(lr);
                      }
                    }
                    function Nn(Wn) {
                      var lr;
                      Wn.done ? zt(Wn.value) : (lr = Wn.value, lr instanceof ot ? lr : new ot(function(xr) {
                        xr(lr);
                      })).then(pn, Cn);
                    }
                    Nn((xt = xt.apply(Ye, He || [])).next());
                  });
                }
                function ae(Ye) {
                  return Ye && Ye.__esModule && Object.prototype.hasOwnProperty.call(Ye, "default") ? Ye.default : Ye;
                }
                typeof SuppressedError == "function" && SuppressedError;
                var oe = se;
                function se(Ye, He) {
                  this.x = Ye, this.y = He;
                }
                se.prototype = { clone: function() {
                  return new se(this.x, this.y);
                }, add: function(Ye) {
                  return this.clone()._add(Ye);
                }, sub: function(Ye) {
                  return this.clone()._sub(Ye);
                }, multByPoint: function(Ye) {
                  return this.clone()._multByPoint(Ye);
                }, divByPoint: function(Ye) {
                  return this.clone()._divByPoint(Ye);
                }, mult: function(Ye) {
                  return this.clone()._mult(Ye);
                }, div: function(Ye) {
                  return this.clone()._div(Ye);
                }, rotate: function(Ye) {
                  return this.clone()._rotate(Ye);
                }, rotateAround: function(Ye, He) {
                  return this.clone()._rotateAround(Ye, He);
                }, matMult: function(Ye) {
                  return this.clone()._matMult(Ye);
                }, unit: function() {
                  return this.clone()._unit();
                }, perp: function() {
                  return this.clone()._perp();
                }, round: function() {
                  return this.clone()._round();
                }, mag: function() {
                  return Math.sqrt(this.x * this.x + this.y * this.y);
                }, equals: function(Ye) {
                  return this.x === Ye.x && this.y === Ye.y;
                }, dist: function(Ye) {
                  return Math.sqrt(this.distSqr(Ye));
                }, distSqr: function(Ye) {
                  var He = Ye.x - this.x, ot = Ye.y - this.y;
                  return He * He + ot * ot;
                }, angle: function() {
                  return Math.atan2(this.y, this.x);
                }, angleTo: function(Ye) {
                  return Math.atan2(this.y - Ye.y, this.x - Ye.x);
                }, angleWith: function(Ye) {
                  return this.angleWithSep(Ye.x, Ye.y);
                }, angleWithSep: function(Ye, He) {
                  return Math.atan2(this.x * He - this.y * Ye, this.x * Ye + this.y * He);
                }, _matMult: function(Ye) {
                  var He = Ye[0] * this.x + Ye[1] * this.y, ot = Ye[2] * this.x + Ye[3] * this.y;
                  return this.x = He, this.y = ot, this;
                }, _add: function(Ye) {
                  return this.x += Ye.x, this.y += Ye.y, this;
                }, _sub: function(Ye) {
                  return this.x -= Ye.x, this.y -= Ye.y, this;
                }, _mult: function(Ye) {
                  return this.x *= Ye, this.y *= Ye, this;
                }, _div: function(Ye) {
                  return this.x /= Ye, this.y /= Ye, this;
                }, _multByPoint: function(Ye) {
                  return this.x *= Ye.x, this.y *= Ye.y, this;
                }, _divByPoint: function(Ye) {
                  return this.x /= Ye.x, this.y /= Ye.y, this;
                }, _unit: function() {
                  return this._div(this.mag()), this;
                }, _perp: function() {
                  var Ye = this.y;
                  return this.y = this.x, this.x = -Ye, this;
                }, _rotate: function(Ye) {
                  var He = Math.cos(Ye), ot = Math.sin(Ye), xt = He * this.x - ot * this.y, zt = ot * this.x + He * this.y;
                  return this.x = xt, this.y = zt, this;
                }, _rotateAround: function(Ye, He) {
                  var ot = Math.cos(Ye), xt = Math.sin(Ye), zt = He.x + ot * (this.x - He.x) - xt * (this.y - He.y), ln = He.y + xt * (this.x - He.x) + ot * (this.y - He.y);
                  return this.x = zt, this.y = ln, this;
                }, _round: function() {
                  return this.x = Math.round(this.x), this.y = Math.round(this.y), this;
                } }, se.convert = function(Ye) {
                  return Ye instanceof se ? Ye : Array.isArray(Ye) ? new se(Ye[0], Ye[1]) : Ye;
                };
                var ce = ae(oe), le = ue;
                function ue(Ye, He, ot, xt) {
                  this.cx = 3 * Ye, this.bx = 3 * (ot - Ye) - this.cx, this.ax = 1 - this.cx - this.bx, this.cy = 3 * He, this.by = 3 * (xt - He) - this.cy, this.ay = 1 - this.cy - this.by, this.p1x = Ye, this.p1y = He, this.p2x = ot, this.p2y = xt;
                }
                ue.prototype = { sampleCurveX: function(Ye) {
                  return ((this.ax * Ye + this.bx) * Ye + this.cx) * Ye;
                }, sampleCurveY: function(Ye) {
                  return ((this.ay * Ye + this.by) * Ye + this.cy) * Ye;
                }, sampleCurveDerivativeX: function(Ye) {
                  return (3 * this.ax * Ye + 2 * this.bx) * Ye + this.cx;
                }, solveCurveX: function(Ye, He) {
                  if (He === void 0 && (He = 1e-6), Ye < 0) return 0;
                  if (Ye > 1) return 1;
                  for (var ot = Ye, xt = 0; xt < 8; xt++) {
                    var zt = this.sampleCurveX(ot) - Ye;
                    if (Math.abs(zt) < He) return ot;
                    var ln = this.sampleCurveDerivativeX(ot);
                    if (Math.abs(ln) < 1e-6) break;
                    ot -= zt / ln;
                  }
                  var pn = 0, Cn = 1;
                  for (ot = Ye, xt = 0; xt < 20 && (zt = this.sampleCurveX(ot), !(Math.abs(zt - Ye) < He)); xt++) Ye > zt ? pn = ot : Cn = ot, ot = 0.5 * (Cn - pn) + pn;
                  return ot;
                }, solve: function(Ye, He) {
                  return this.sampleCurveY(this.solveCurveX(Ye, He));
                } };
                var de = ae(le);
                let fe, he;
                function me() {
                  return fe == null && (fe = typeof OffscreenCanvas < "u" && new OffscreenCanvas(1, 1).getContext("2d") && typeof createImageBitmap == "function"), fe;
                }
                function pe() {
                  if (he == null && (he = !1, me())) {
                    const He = new OffscreenCanvas(5, 5).getContext("2d", { willReadFrequently: !0 });
                    if (He) {
                      for (let xt = 0; xt < 25; xt++) {
                        const zt = 4 * xt;
                        He.fillStyle = `rgb(${zt},${zt + 1},${zt + 2})`, He.fillRect(xt % 5, Math.floor(xt / 5), 1, 1);
                      }
                      const ot = He.getImageData(0, 0, 5, 5).data;
                      for (let xt = 0; xt < 100; xt++) if (xt % 4 != 3 && ot[xt] !== xt) {
                        he = !0;
                        break;
                      }
                    }
                  }
                  return he || !1;
                }
                function ye(Ye, He, ot, xt) {
                  const zt = new de(Ye, He, ot, xt);
                  return (ln) => zt.solve(ln);
                }
                const ge = ye(0.25, 0.1, 0.25, 1);
                function ve(Ye, He, ot) {
                  return Math.min(ot, Math.max(He, Ye));
                }
                function be(Ye, He, ot) {
                  const xt = ot - He, zt = ((Ye - He) % xt + xt) % xt + He;
                  return zt === He ? ot : zt;
                }
                function Ce(Ye, ...He) {
                  for (const ot of He) for (const xt in ot) Ye[xt] = ot[xt];
                  return Ye;
                }
                let Se = 1;
                function we(Ye, He, ot) {
                  const xt = {};
                  for (const zt in Ye) xt[zt] = He.call(this, Ye[zt], zt, Ye);
                  return xt;
                }
                function Ee(Ye, He, ot) {
                  const xt = {};
                  for (const zt in Ye) He.call(this, Ye[zt], zt, Ye) && (xt[zt] = Ye[zt]);
                  return xt;
                }
                function xe(Ye) {
                  return Array.isArray(Ye) ? Ye.map(xe) : typeof Ye == "object" && Ye ? we(Ye, xe) : Ye;
                }
                const Le = {};
                function Qe(Ye) {
                  Le[Ye] || (typeof console < "u" && console.warn(Ye), Le[Ye] = !0);
                }
                function ke(Ye, He, ot) {
                  return (ot.y - Ye.y) * (He.x - Ye.x) > (He.y - Ye.y) * (ot.x - Ye.x);
                }
                function Me(Ye) {
                  return typeof WorkerGlobalScope < "u" && Ye !== void 0 && Ye instanceof WorkerGlobalScope;
                }
                let Re = null;
                function De(Ye) {
                  return typeof ImageBitmap < "u" && Ye instanceof ImageBitmap;
                }
                const Pe = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAC0lEQVQYV2NgAAIAAAUAAarVyFEAAAAASUVORK5CYII=";
                function $e(Ye, He, ot, xt, zt) {
                  return ee(this, void 0, void 0, function* () {
                    if (typeof VideoFrame > "u") throw new Error("VideoFrame not supported");
                    const ln = new VideoFrame(Ye, { timestamp: 0 });
                    try {
                      const pn = ln == null ? void 0 : ln.format;
                      if (!pn || !pn.startsWith("BGR") && !pn.startsWith("RGB")) throw new Error(`Unrecognized format ${pn}`);
                      const Cn = pn.startsWith("BGR"), Nn = new Uint8ClampedArray(xt * zt * 4);
                      if (yield ln.copyTo(Nn, function(Wn, lr, xr, Nr, Kr) {
                        const xi = 4 * Math.max(-lr, 0), ui = (Math.max(0, xr) - xr) * Nr * 4 + xi, Ui = 4 * Nr, ia = Math.max(0, lr), Na = Math.max(0, xr);
                        return { rect: { x: ia, y: Na, width: Math.min(Wn.width, lr + Nr) - ia, height: Math.min(Wn.height, xr + Kr) - Na }, layout: [{ offset: ui, stride: Ui }] };
                      }(Ye, He, ot, xt, zt)), Cn) for (let Wn = 0; Wn < Nn.length; Wn += 4) {
                        const lr = Nn[Wn];
                        Nn[Wn] = Nn[Wn + 2], Nn[Wn + 2] = lr;
                      }
                      return Nn;
                    } finally {
                      ln.close();
                    }
                  });
                }
                let qe, Xe;
                const Ve = "AbortError";
                function Ze() {
                  return new Error(Ve);
                }
                const Ge = { MAX_PARALLEL_IMAGE_REQUESTS: 16, MAX_PARALLEL_IMAGE_REQUESTS_PER_FRAME: 8, MAX_TILE_CACHE_ZOOM_LEVELS: 5, REGISTERED_PROTOCOLS: {}, WORKER_URL: "" };
                function ze(Ye) {
                  return Ge.REGISTERED_PROTOCOLS[Ye.substring(0, Ye.indexOf("://"))];
                }
                const We = "global-dispatcher";
                class at extends Error {
                  constructor(He, ot, xt, zt) {
                    super(`AJAXError: ${ot} (${He}): ${xt}`), this.status = He, this.statusText = ot, this.url = xt, this.body = zt;
                  }
                }
                const ct = () => Me(self) ? self.worker && self.worker.referrer : (window.location.protocol === "blob:" ? window.parent : window).location.href, et = function(Ye, He) {
                  if (/:\/\//.test(Ye.url) && !/^https?:|^file:/.test(Ye.url)) {
                    const xt = ze(Ye.url);
                    if (xt) return xt(Ye, He);
                    if (Me(self) && self.worker && self.worker.actor) return self.worker.actor.sendAsync({ type: "GR", data: Ye, targetMapId: We }, He);
                  }
                  if (ot = Ye.url, !(/^file:/.test(ot) || /^file:/.test(ct()) && !/^\w+:/.test(ot))) {
                    if (fetch && Request && AbortController && Object.prototype.hasOwnProperty.call(Request.prototype, "signal")) return function(xt, zt) {
                      return ee(this, void 0, void 0, function* () {
                        const ln = new Request(xt.url, { method: xt.method || "GET", body: xt.body, credentials: xt.credentials, headers: xt.headers, cache: xt.cache, referrer: ct(), signal: zt.signal });
                        xt.type !== "json" || ln.headers.has("Accept") || ln.headers.set("Accept", "application/json");
                        const pn = yield fetch(ln);
                        if (!pn.ok) {
                          const Wn = yield pn.blob();
                          throw new at(pn.status, pn.statusText, xt.url, Wn);
                        }
                        let Cn;
                        Cn = xt.type === "arrayBuffer" || xt.type === "image" ? pn.arrayBuffer() : xt.type === "json" ? pn.json() : pn.text();
                        const Nn = yield Cn;
                        if (zt.signal.aborted) throw Ze();
                        return { data: Nn, cacheControl: pn.headers.get("Cache-Control"), expires: pn.headers.get("Expires") };
                      });
                    }(Ye, He);
                    if (Me(self) && self.worker && self.worker.actor) return self.worker.actor.sendAsync({ type: "GR", data: Ye, mustQueue: !0, targetMapId: We }, He);
                  }
                  var ot;
                  return function(xt, zt) {
                    return new Promise((ln, pn) => {
                      var Cn;
                      const Nn = new XMLHttpRequest();
                      Nn.open(xt.method || "GET", xt.url, !0), xt.type !== "arrayBuffer" && xt.type !== "image" || (Nn.responseType = "arraybuffer");
                      for (const Wn in xt.headers) Nn.setRequestHeader(Wn, xt.headers[Wn]);
                      xt.type === "json" && (Nn.responseType = "text", !((Cn = xt.headers) === null || Cn === void 0) && Cn.Accept || Nn.setRequestHeader("Accept", "application/json")), Nn.withCredentials = xt.credentials === "include", Nn.onerror = () => {
                        pn(new Error(Nn.statusText));
                      }, Nn.onload = () => {
                        if (!zt.signal.aborted) if ((Nn.status >= 200 && Nn.status < 300 || Nn.status === 0) && Nn.response !== null) {
                          let Wn = Nn.response;
                          if (xt.type === "json") try {
                            Wn = JSON.parse(Nn.response);
                          } catch (lr) {
                            return void pn(lr);
                          }
                          ln({ data: Wn, cacheControl: Nn.getResponseHeader("Cache-Control"), expires: Nn.getResponseHeader("Expires") });
                        } else {
                          const Wn = new Blob([Nn.response], { type: Nn.getResponseHeader("Content-Type") });
                          pn(new at(Nn.status, Nn.statusText, xt.url, Wn));
                        }
                      }, zt.signal.addEventListener("abort", () => {
                        Nn.abort(), pn(Ze());
                      }), Nn.send(xt.body);
                    });
                  }(Ye, He);
                };
                function dt(Ye) {
                  if (!Ye || Ye.indexOf("://") <= 0 || Ye.indexOf("data:image/") === 0 || Ye.indexOf("blob:") === 0) return !0;
                  const He = new URL(Ye), ot = window.location;
                  return He.protocol === ot.protocol && He.host === ot.host;
                }
                function Je(Ye, He, ot) {
                  ot[Ye] && ot[Ye].indexOf(He) !== -1 || (ot[Ye] = ot[Ye] || [], ot[Ye].push(He));
                }
                function it(Ye, He, ot) {
                  if (ot && ot[Ye]) {
                    const xt = ot[Ye].indexOf(He);
                    xt !== -1 && ot[Ye].splice(xt, 1);
                  }
                }
                class st {
                  constructor(He, ot = {}) {
                    Ce(this, ot), this.type = He;
                  }
                }
                class gt extends st {
                  constructor(He, ot = {}) {
                    super("error", Ce({ error: He }, ot));
                  }
                }
                class bt {
                  on(He, ot) {
                    return this._listeners = this._listeners || {}, Je(He, ot, this._listeners), this;
                  }
                  off(He, ot) {
                    return it(He, ot, this._listeners), it(He, ot, this._oneTimeListeners), this;
                  }
                  once(He, ot) {
                    return ot ? (this._oneTimeListeners = this._oneTimeListeners || {}, Je(He, ot, this._oneTimeListeners), this) : new Promise((xt) => this.once(He, xt));
                  }
                  fire(He, ot) {
                    typeof He == "string" && (He = new st(He, ot || {}));
                    const xt = He.type;
                    if (this.listens(xt)) {
                      He.target = this;
                      const zt = this._listeners && this._listeners[xt] ? this._listeners[xt].slice() : [];
                      for (const Cn of zt) Cn.call(this, He);
                      const ln = this._oneTimeListeners && this._oneTimeListeners[xt] ? this._oneTimeListeners[xt].slice() : [];
                      for (const Cn of ln) it(xt, Cn, this._oneTimeListeners), Cn.call(this, He);
                      const pn = this._eventedParent;
                      pn && (Ce(He, typeof this._eventedParentData == "function" ? this._eventedParentData() : this._eventedParentData), pn.fire(He));
                    } else He instanceof gt && console.error(He.error);
                    return this;
                  }
                  listens(He) {
                    return this._listeners && this._listeners[He] && this._listeners[He].length > 0 || this._oneTimeListeners && this._oneTimeListeners[He] && this._oneTimeListeners[He].length > 0 || this._eventedParent && this._eventedParent.listens(He);
                  }
                  setEventedParent(He, ot) {
                    return this._eventedParent = He, this._eventedParentData = ot, this;
                  }
                }
                var yt = { $version: 8, $root: { version: { required: !0, type: "enum", values: [8] }, name: { type: "string" }, metadata: { type: "*" }, center: { type: "array", value: "number" }, zoom: { type: "number" }, bearing: { type: "number", default: 0, period: 360, units: "degrees" }, pitch: { type: "number", default: 0, units: "degrees" }, light: { type: "light" }, sky: { type: "sky" }, projection: { type: "projection" }, terrain: { type: "terrain" }, sources: { required: !0, type: "sources" }, sprite: { type: "sprite" }, glyphs: { type: "string" }, transition: { type: "transition" }, layers: { required: !0, type: "array", value: "layer" } }, sources: { "*": { type: "source" } }, source: ["source_vector", "source_raster", "source_raster_dem", "source_geojson", "source_video", "source_image"], source_vector: { type: { required: !0, type: "enum", values: { vector: {} } }, url: { type: "string" }, tiles: { type: "array", value: "string" }, bounds: { type: "array", value: "number", length: 4, default: [-180, -85.051129, 180, 85.051129] }, scheme: { type: "enum", values: { xyz: {}, tms: {} }, default: "xyz" }, minzoom: { type: "number", default: 0 }, maxzoom: { type: "number", default: 22 }, attribution: { type: "string" }, promoteId: { type: "promoteId" }, volatile: { type: "boolean", default: !1 }, "*": { type: "*" } }, source_raster: { type: { required: !0, type: "enum", values: { raster: {} } }, url: { type: "string" }, tiles: { type: "array", value: "string" }, bounds: { type: "array", value: "number", length: 4, default: [-180, -85.051129, 180, 85.051129] }, minzoom: { type: "number", default: 0 }, maxzoom: { type: "number", default: 22 }, tileSize: { type: "number", default: 512, units: "pixels" }, scheme: { type: "enum", values: { xyz: {}, tms: {} }, default: "xyz" }, attribution: { type: "string" }, volatile: { type: "boolean", default: !1 }, "*": { type: "*" } }, source_raster_dem: { type: { required: !0, type: "enum", values: { "raster-dem": {} } }, url: { type: "string" }, tiles: { type: "array", value: "string" }, bounds: { type: "array", value: "number", length: 4, default: [-180, -85.051129, 180, 85.051129] }, minzoom: { type: "number", default: 0 }, maxzoom: { type: "number", default: 22 }, tileSize: { type: "number", default: 512, units: "pixels" }, attribution: { type: "string" }, encoding: { type: "enum", values: { terrarium: {}, mapbox: {}, custom: {} }, default: "mapbox" }, redFactor: { type: "number", default: 1 }, blueFactor: { type: "number", default: 1 }, greenFactor: { type: "number", default: 1 }, baseShift: { type: "number", default: 0 }, volatile: { type: "boolean", default: !1 }, "*": { type: "*" } }, source_geojson: { type: { required: !0, type: "enum", values: { geojson: {} } }, data: { required: !0, type: "*" }, maxzoom: { type: "number", default: 18 }, attribution: { type: "string" }, buffer: { type: "number", default: 128, maximum: 512, minimum: 0 }, filter: { type: "*" }, tolerance: { type: "number", default: 0.375 }, cluster: { type: "boolean", default: !1 }, clusterRadius: { type: "number", default: 50, minimum: 0 }, clusterMaxZoom: { type: "number" }, clusterMinPoints: { type: "number" }, clusterProperties: { type: "*" }, lineMetrics: { type: "boolean", default: !1 }, generateId: { type: "boolean", default: !1 }, promoteId: { type: "promoteId" } }, source_video: { type: { required: !0, type: "enum", values: { video: {} } }, urls: { required: !0, type: "array", value: "string" }, coordinates: { required: !0, type: "array", length: 4, value: { type: "array", length: 2, value: "number" } } }, source_image: { type: { required: !0, type: "enum", values: { image: {} } }, url: { required: !0, type: "string" }, coordinates: { required: !0, type: "array", length: 4, value: { type: "array", length: 2, value: "number" } } }, layer: { id: { type: "string", required: !0 }, type: { type: "enum", values: { fill: {}, line: {}, symbol: {}, circle: {}, heatmap: {}, "fill-extrusion": {}, raster: {}, hillshade: {}, background: {} }, required: !0 }, metadata: { type: "*" }, source: { type: "string" }, "source-layer": { type: "string" }, minzoom: { type: "number", minimum: 0, maximum: 24 }, maxzoom: { type: "number", minimum: 0, maximum: 24 }, filter: { type: "filter" }, layout: { type: "layout" }, paint: { type: "paint" } }, layout: ["layout_fill", "layout_line", "layout_circle", "layout_heatmap", "layout_fill-extrusion", "layout_symbol", "layout_raster", "layout_hillshade", "layout_background"], layout_background: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_fill: { "fill-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_circle: { "circle-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_heatmap: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, "layout_fill-extrusion": { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_line: { "line-cap": { type: "enum", values: { butt: {}, round: {}, square: {} }, default: "butt", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-join": { type: "enum", values: { bevel: {}, round: {}, miter: {} }, default: "miter", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "line-miter-limit": { type: "number", default: 2, requires: [{ "line-join": "miter" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-round-limit": { type: "number", default: 1.05, requires: [{ "line-join": "round" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_symbol: { "symbol-placement": { type: "enum", values: { point: {}, line: {}, "line-center": {} }, default: "point", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "symbol-spacing": { type: "number", default: 250, minimum: 1, units: "pixels", requires: [{ "symbol-placement": "line" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "symbol-avoid-edges": { type: "boolean", default: !1, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "symbol-sort-key": { type: "number", expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "symbol-z-order": { type: "enum", values: { auto: {}, "viewport-y": {}, source: {} }, default: "auto", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-allow-overlap": { type: "boolean", default: !1, requires: ["icon-image", { "!": "icon-overlap" }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-overlap": { type: "enum", values: { never: {}, always: {}, cooperative: {} }, requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-ignore-placement": { type: "boolean", default: !1, requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-optional": { type: "boolean", default: !1, requires: ["icon-image", "text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-rotation-alignment": { type: "enum", values: { map: {}, viewport: {}, auto: {} }, default: "auto", requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-size": { type: "number", default: 1, minimum: 0, units: "factor of the original icon size", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-text-fit": { type: "enum", values: { none: {}, width: {}, height: {}, both: {} }, default: "none", requires: ["icon-image", "text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-text-fit-padding": { type: "array", value: "number", length: 4, default: [0, 0, 0, 0], units: "pixels", requires: ["icon-image", "text-field", { "icon-text-fit": ["both", "width", "height"] }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-image": { type: "resolvedImage", tokens: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-rotate": { type: "number", default: 0, period: 360, units: "degrees", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-padding": { type: "padding", default: [2], units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-keep-upright": { type: "boolean", default: !1, requires: ["icon-image", { "icon-rotation-alignment": "map" }, { "symbol-placement": ["line", "line-center"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-offset": { type: "array", value: "number", length: 2, default: [0, 0], requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-anchor": { type: "enum", values: { center: {}, left: {}, right: {}, top: {}, bottom: {}, "top-left": {}, "top-right": {}, "bottom-left": {}, "bottom-right": {} }, default: "center", requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "icon-pitch-alignment": { type: "enum", values: { map: {}, viewport: {}, auto: {} }, default: "auto", requires: ["icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-pitch-alignment": { type: "enum", values: { map: {}, viewport: {}, auto: {} }, default: "auto", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-rotation-alignment": { type: "enum", values: { map: {}, viewport: {}, "viewport-glyph": {}, auto: {} }, default: "auto", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-field": { type: "formatted", default: "", tokens: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-font": { type: "array", value: "string", default: ["Open Sans Regular", "Arial Unicode MS Regular"], requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-size": { type: "number", default: 16, minimum: 0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-max-width": { type: "number", default: 10, minimum: 0, units: "ems", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-line-height": { type: "number", default: 1.2, units: "ems", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-letter-spacing": { type: "number", default: 0, units: "ems", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-justify": { type: "enum", values: { auto: {}, left: {}, center: {}, right: {} }, default: "center", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-radial-offset": { type: "number", units: "ems", default: 0, requires: ["text-field"], "property-type": "data-driven", expression: { interpolated: !0, parameters: ["zoom", "feature"] } }, "text-variable-anchor": { type: "array", value: "enum", values: { center: {}, left: {}, right: {}, top: {}, bottom: {}, "top-left": {}, "top-right": {}, "bottom-left": {}, "bottom-right": {} }, requires: ["text-field", { "symbol-placement": ["point"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-variable-anchor-offset": { type: "variableAnchorOffsetCollection", requires: ["text-field", { "symbol-placement": ["point"] }], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-anchor": { type: "enum", values: { center: {}, left: {}, right: {}, top: {}, bottom: {}, "top-left": {}, "top-right": {}, "bottom-left": {}, "bottom-right": {} }, default: "center", requires: ["text-field", { "!": "text-variable-anchor" }], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-max-angle": { type: "number", default: 45, units: "degrees", requires: ["text-field", { "symbol-placement": ["line", "line-center"] }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-writing-mode": { type: "array", value: "enum", values: { horizontal: {}, vertical: {} }, requires: ["text-field", { "symbol-placement": ["point"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-rotate": { type: "number", default: 0, period: 360, units: "degrees", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-padding": { type: "number", default: 2, minimum: 0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-keep-upright": { type: "boolean", default: !0, requires: ["text-field", { "text-rotation-alignment": "map" }, { "symbol-placement": ["line", "line-center"] }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-transform": { type: "enum", values: { none: {}, uppercase: {}, lowercase: {} }, default: "none", requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-offset": { type: "array", value: "number", units: "ems", length: 2, default: [0, 0], requires: ["text-field", { "!": "text-radial-offset" }], expression: { interpolated: !0, parameters: ["zoom", "feature"] }, "property-type": "data-driven" }, "text-allow-overlap": { type: "boolean", default: !1, requires: ["text-field", { "!": "text-overlap" }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-overlap": { type: "enum", values: { never: {}, always: {}, cooperative: {} }, requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-ignore-placement": { type: "boolean", default: !1, requires: ["text-field"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-optional": { type: "boolean", default: !1, requires: ["text-field", "icon-image"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_raster: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, layout_hillshade: { visibility: { type: "enum", values: { visible: {}, none: {} }, default: "visible", "property-type": "constant" } }, filter: { type: "array", value: "*" }, filter_operator: { type: "enum", values: { "==": {}, "!=": {}, ">": {}, ">=": {}, "<": {}, "<=": {}, in: {}, "!in": {}, all: {}, any: {}, none: {}, has: {}, "!has": {} } }, geometry_type: { type: "enum", values: { Point: {}, LineString: {}, Polygon: {} } }, function: { expression: { type: "expression" }, stops: { type: "array", value: "function_stop" }, base: { type: "number", default: 1, minimum: 0 }, property: { type: "string", default: "$zoom" }, type: { type: "enum", values: { identity: {}, exponential: {}, interval: {}, categorical: {} }, default: "exponential" }, colorSpace: { type: "enum", values: { rgb: {}, lab: {}, hcl: {} }, default: "rgb" }, default: { type: "*", required: !1 } }, function_stop: { type: "array", minimum: 0, maximum: 24, value: ["number", "color"], length: 2 }, expression: { type: "array", value: "*", minimum: 1 }, light: { anchor: { type: "enum", default: "viewport", values: { map: {}, viewport: {} }, "property-type": "data-constant", transition: !1, expression: { interpolated: !1, parameters: ["zoom"] } }, position: { type: "array", default: [1.15, 210, 30], length: 3, value: "number", "property-type": "data-constant", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] } }, color: { type: "color", "property-type": "data-constant", default: "#ffffff", expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, intensity: { type: "number", "property-type": "data-constant", default: 0.5, minimum: 0, maximum: 1, expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 } }, sky: { "sky-color": { type: "color", "property-type": "data-constant", default: "#88C6FC", expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, "horizon-color": { type: "color", "property-type": "data-constant", default: "#ffffff", expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, "fog-color": { type: "color", "property-type": "data-constant", default: "#ffffff", expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, "fog-ground-blend": { type: "number", "property-type": "data-constant", default: 0.5, minimum: 0, maximum: 1, expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, "horizon-fog-blend": { type: "number", "property-type": "data-constant", default: 0.8, minimum: 0, maximum: 1, expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, "sky-horizon-blend": { type: "number", "property-type": "data-constant", default: 0.8, minimum: 0, maximum: 1, expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 }, "atmosphere-blend": { type: "number", "property-type": "data-constant", default: 0.8, minimum: 0, maximum: 1, expression: { interpolated: !0, parameters: ["zoom"] }, transition: !0 } }, terrain: { source: { type: "string", required: !0 }, exaggeration: { type: "number", minimum: 0, default: 1 } }, projection: { type: { type: "enum", default: "mercator", values: { mercator: {}, globe: {} } } }, paint: ["paint_fill", "paint_line", "paint_circle", "paint_heatmap", "paint_fill-extrusion", "paint_symbol", "paint_raster", "paint_hillshade", "paint_background"], paint_fill: { "fill-antialias": { type: "boolean", default: !0, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "fill-pattern" }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-outline-color": { type: "color", transition: !0, requires: [{ "!": "fill-pattern" }, { "fill-antialias": !0 }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["fill-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "cross-faded-data-driven" } }, "paint_fill-extrusion": { "fill-extrusion-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-extrusion-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "fill-extrusion-pattern" }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-extrusion-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-extrusion-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["fill-extrusion-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "fill-extrusion-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "cross-faded-data-driven" }, "fill-extrusion-height": { type: "number", default: 0, minimum: 0, units: "meters", transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-extrusion-base": { type: "number", default: 0, minimum: 0, units: "meters", transition: !0, requires: ["fill-extrusion-height"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "fill-extrusion-vertical-gradient": { type: "boolean", default: !0, transition: !1, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_line: { "line-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "line-pattern" }], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["line-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "line-width": { type: "number", default: 1, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-gap-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-offset": { type: "number", default: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-blur": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "line-dasharray": { type: "array", value: "number", minimum: 0, transition: !0, units: "line widths", requires: [{ "!": "line-pattern" }], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "cross-faded" }, "line-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom", "feature"] }, "property-type": "cross-faded-data-driven" }, "line-gradient": { type: "color", transition: !1, requires: [{ "!": "line-dasharray" }, { "!": "line-pattern" }, { source: "geojson", has: { lineMetrics: !0 } }], expression: { interpolated: !0, parameters: ["line-progress"] }, "property-type": "color-ramp" } }, paint_circle: { "circle-radius": { type: "number", default: 5, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-blur": { type: "number", default: 0, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["circle-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-pitch-scale": { type: "enum", values: { map: {}, viewport: {} }, default: "map", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-pitch-alignment": { type: "enum", values: { map: {}, viewport: {} }, default: "viewport", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "circle-stroke-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-stroke-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "circle-stroke-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" } }, paint_heatmap: { "heatmap-radius": { type: "number", default: 30, minimum: 1, transition: !0, units: "pixels", expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "heatmap-weight": { type: "number", default: 1, minimum: 0, transition: !1, expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "heatmap-intensity": { type: "number", default: 1, minimum: 0, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "heatmap-color": { type: "color", default: ["interpolate", ["linear"], ["heatmap-density"], 0, "rgba(0, 0, 255, 0)", 0.1, "royalblue", 0.3, "cyan", 0.5, "lime", 0.7, "yellow", 1, "red"], transition: !1, expression: { interpolated: !0, parameters: ["heatmap-density"] }, "property-type": "color-ramp" }, "heatmap-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_symbol: { "icon-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-color": { type: "color", default: "#000000", transition: !0, requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-halo-color": { type: "color", default: "rgba(0, 0, 0, 0)", transition: !0, requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-halo-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-halo-blur": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "icon-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", requires: ["icon-image"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "icon-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["icon-image", "icon-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-color": { type: "color", default: "#000000", transition: !0, overridable: !0, requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-halo-color": { type: "color", default: "rgba(0, 0, 0, 0)", transition: !0, requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-halo-width": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-halo-blur": { type: "number", default: 0, minimum: 0, transition: !0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom", "feature", "feature-state"] }, "property-type": "data-driven" }, "text-translate": { type: "array", value: "number", length: 2, default: [0, 0], transition: !0, units: "pixels", requires: ["text-field"], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "text-translate-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "map", requires: ["text-field", "text-translate"], expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_raster: { "raster-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-hue-rotate": { type: "number", default: 0, period: 360, transition: !0, units: "degrees", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-brightness-min": { type: "number", default: 0, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-brightness-max": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-saturation": { type: "number", default: 0, minimum: -1, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-contrast": { type: "number", default: 0, minimum: -1, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-resampling": { type: "enum", values: { linear: {}, nearest: {} }, default: "linear", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "raster-fade-duration": { type: "number", default: 300, minimum: 0, transition: !1, units: "milliseconds", expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_hillshade: { "hillshade-illumination-direction": { type: "number", default: 335, minimum: 0, maximum: 359, transition: !1, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-illumination-anchor": { type: "enum", values: { map: {}, viewport: {} }, default: "viewport", expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-exaggeration": { type: "number", default: 0.5, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-shadow-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-highlight-color": { type: "color", default: "#FFFFFF", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "hillshade-accent-color": { type: "color", default: "#000000", transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, paint_background: { "background-color": { type: "color", default: "#000000", transition: !0, requires: [{ "!": "background-pattern" }], expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" }, "background-pattern": { type: "resolvedImage", transition: !0, expression: { interpolated: !1, parameters: ["zoom"] }, "property-type": "cross-faded" }, "background-opacity": { type: "number", default: 1, minimum: 0, maximum: 1, transition: !0, expression: { interpolated: !0, parameters: ["zoom"] }, "property-type": "data-constant" } }, transition: { duration: { type: "number", default: 300, minimum: 0, units: "milliseconds" }, delay: { type: "number", default: 0, minimum: 0, units: "milliseconds" } }, "property-type": { "data-driven": { type: "property-type" }, "cross-faded": { type: "property-type" }, "cross-faded-data-driven": { type: "property-type" }, "color-ramp": { type: "property-type" }, "data-constant": { type: "property-type" }, constant: { type: "property-type" } }, promoteId: { "*": { type: "string" } } };
                const St = ["type", "source", "source-layer", "minzoom", "maxzoom", "filter", "layout"];
                function Lt(Ye, He) {
                  const ot = {};
                  for (const xt in Ye) xt !== "ref" && (ot[xt] = Ye[xt]);
                  return St.forEach((xt) => {
                    xt in He && (ot[xt] = He[xt]);
                  }), ot;
                }
                function Rt(Ye, He) {
                  if (Array.isArray(Ye)) {
                    if (!Array.isArray(He) || Ye.length !== He.length) return !1;
                    for (let ot = 0; ot < Ye.length; ot++) if (!Rt(Ye[ot], He[ot])) return !1;
                    return !0;
                  }
                  if (typeof Ye == "object" && Ye !== null && He !== null) {
                    if (typeof He != "object" || Object.keys(Ye).length !== Object.keys(He).length) return !1;
                    for (const ot in Ye) if (!Rt(Ye[ot], He[ot])) return !1;
                    return !0;
                  }
                  return Ye === He;
                }
                function vt(Ye, He) {
                  Ye.push(He);
                }
                function $t(Ye, He, ot) {
                  vt(ot, { command: "addSource", args: [Ye, He[Ye]] });
                }
                function Pt(Ye, He, ot) {
                  vt(He, { command: "removeSource", args: [Ye] }), ot[Ye] = !0;
                }
                function Xt(Ye, He, ot, xt) {
                  Pt(Ye, ot, xt), $t(Ye, He, ot);
                }
                function Nt(Ye, He, ot) {
                  let xt;
                  for (xt in Ye[ot]) if (Object.prototype.hasOwnProperty.call(Ye[ot], xt) && xt !== "data" && !Rt(Ye[ot][xt], He[ot][xt])) return !1;
                  for (xt in He[ot]) if (Object.prototype.hasOwnProperty.call(He[ot], xt) && xt !== "data" && !Rt(Ye[ot][xt], He[ot][xt])) return !1;
                  return !0;
                }
                function Gt(Ye, He, ot, xt, zt, ln) {
                  Ye = Ye || {}, He = He || {};
                  for (const pn in Ye) Object.prototype.hasOwnProperty.call(Ye, pn) && (Rt(Ye[pn], He[pn]) || ot.push({ command: ln, args: [xt, pn, He[pn], zt] }));
                  for (const pn in He) Object.prototype.hasOwnProperty.call(He, pn) && !Object.prototype.hasOwnProperty.call(Ye, pn) && (Rt(Ye[pn], He[pn]) || ot.push({ command: ln, args: [xt, pn, He[pn], zt] }));
                }
                function Ft(Ye) {
                  return Ye.id;
                }
                function Tt(Ye, He) {
                  return Ye[He.id] = He, Ye;
                }
                class It {
                  constructor(He, ot, xt, zt) {
                    this.message = (He ? `${He}: ` : "") + xt, zt && (this.identifier = zt), ot != null && ot.__line__ && (this.line = ot.__line__);
                  }
                }
                function Ct(Ye, ...He) {
                  for (const ot of He) for (const xt in ot) Ye[xt] = ot[xt];
                  return Ye;
                }
                class Kt extends Error {
                  constructor(He, ot) {
                    super(ot), this.message = ot, this.key = He;
                  }
                }
                class Vt {
                  constructor(He, ot = []) {
                    this.parent = He, this.bindings = {};
                    for (const [xt, zt] of ot) this.bindings[xt] = zt;
                  }
                  concat(He) {
                    return new Vt(this, He);
                  }
                  get(He) {
                    if (this.bindings[He]) return this.bindings[He];
                    if (this.parent) return this.parent.get(He);
                    throw new Error(`${He} not found in scope.`);
                  }
                  has(He) {
                    return !!this.bindings[He] || !!this.parent && this.parent.has(He);
                  }
                }
                const Yt = { kind: "null" }, sn = { kind: "number" }, hn = { kind: "string" }, Qt = { kind: "boolean" }, Ut = { kind: "color" }, Wt = { kind: "object" }, rn = { kind: "value" }, vn = { kind: "collator" }, dn = { kind: "formatted" }, Ht = { kind: "padding" }, jt = { kind: "resolvedImage" }, cn = { kind: "variableAnchorOffsetCollection" };
                function wn(Ye, He) {
                  return { kind: "array", itemType: Ye, N: He };
                }
                function Rn(Ye) {
                  if (Ye.kind === "array") {
                    const He = Rn(Ye.itemType);
                    return typeof Ye.N == "number" ? `array<${He}, ${Ye.N}>` : Ye.itemType.kind === "value" ? "array" : `array<${He}>`;
                  }
                  return Ye.kind;
                }
                const Pn = [Yt, sn, hn, Qt, Ut, dn, Wt, wn(rn), Ht, jt, cn];
                function Ie(Ye, He) {
                  if (He.kind === "error") return null;
                  if (Ye.kind === "array") {
                    if (He.kind === "array" && (He.N === 0 && He.itemType.kind === "value" || !Ie(Ye.itemType, He.itemType)) && (typeof Ye.N != "number" || Ye.N === He.N)) return null;
                  } else {
                    if (Ye.kind === He.kind) return null;
                    if (Ye.kind === "value") {
                      for (const ot of Pn) if (!Ie(ot, He)) return null;
                    }
                  }
                  return `Expected ${Rn(Ye)} but found ${Rn(He)} instead.`;
                }
                function rt(Ye, He) {
                  return He.some((ot) => ot.kind === Ye.kind);
                }
                function ft(Ye, He) {
                  return He.some((ot) => ot === "null" ? Ye === null : ot === "array" ? Array.isArray(Ye) : ot === "object" ? Ye && !Array.isArray(Ye) && typeof Ye == "object" : ot === typeof Ye);
                }
                function pt(Ye, He) {
                  return Ye.kind === "array" && He.kind === "array" ? Ye.itemType.kind === He.itemType.kind && typeof Ye.N == "number" : Ye.kind === He.kind;
                }
                const wt = 0.96422, Bt = 1, At = 0.82521, Dt = 4 / 29, Ot = 6 / 29, un = 3 * Ot * Ot, yn = Ot * Ot * Ot, _n = Math.PI / 180, gn = 180 / Math.PI;
                function bn(Ye) {
                  return (Ye %= 360) < 0 && (Ye += 360), Ye;
                }
                function an([Ye, He, ot, xt]) {
                  let zt, ln;
                  const pn = Mn((0.2225045 * (Ye = Zt(Ye)) + 0.7168786 * (He = Zt(He)) + 0.0606169 * (ot = Zt(ot))) / Bt);
                  Ye === He && He === ot ? zt = ln = pn : (zt = Mn((0.4360747 * Ye + 0.3850649 * He + 0.1430804 * ot) / wt), ln = Mn((0.0139322 * Ye + 0.0971045 * He + 0.7141733 * ot) / At));
                  const Cn = 116 * pn - 16;
                  return [Cn < 0 ? 0 : Cn, 500 * (zt - pn), 200 * (pn - ln), xt];
                }
                function Zt(Ye) {
                  return Ye <= 0.04045 ? Ye / 12.92 : Math.pow((Ye + 0.055) / 1.055, 2.4);
                }
                function Mn(Ye) {
                  return Ye > yn ? Math.pow(Ye, 0.3333333333333333) : Ye / un + Dt;
                }
                function Ln([Ye, He, ot, xt]) {
                  let zt = (Ye + 16) / 116, ln = isNaN(He) ? zt : zt + He / 500, pn = isNaN(ot) ? zt : zt - ot / 200;
                  return zt = Bt * kn(zt), ln = wt * kn(ln), pn = At * kn(pn), [In(3.1338561 * ln - 1.6168667 * zt - 0.4906146 * pn), In(-0.9787684 * ln + 1.9161415 * zt + 0.033454 * pn), In(0.0719453 * ln - 0.2289914 * zt + 1.4052427 * pn), xt];
                }
                function In(Ye) {
                  return (Ye = Ye <= 304e-5 ? 12.92 * Ye : 1.055 * Math.pow(Ye, 0.4166666666666667) - 0.055) < 0 ? 0 : Ye > 1 ? 1 : Ye;
                }
                function kn(Ye) {
                  return Ye > Ot ? Ye * Ye * Ye : un * (Ye - Dt);
                }
                function er(Ye) {
                  if ((Ye = Ye.toLowerCase().trim()) === "transparent") return [0, 0, 0, 0];
                  const He = Lr[Ye];
                  if (He) {
                    const [xt, zt, ln] = He;
                    return [xt / 255, zt / 255, ln / 255, 1];
                  }
                  if (Ye.startsWith("#") && /^#(?:[0-9a-f]{3,4}|[0-9a-f]{6}|[0-9a-f]{8})$/.test(Ye)) {
                    const xt = Ye.length < 6 ? 1 : 2;
                    let zt = 1;
                    return [jn(Ye.slice(zt, zt += xt)), jn(Ye.slice(zt, zt += xt)), jn(Ye.slice(zt, zt += xt)), jn(Ye.slice(zt, zt + xt) || "ff")];
                  }
                  if (Ye.startsWith("rgb")) {
                    const xt = /^rgba?\(\s*([\de.+-]+)(%)?(?:\s+|\s*(,)\s*)([\de.+-]+)(%)?(?:\s+|\s*(,)\s*)([\de.+-]+)(%)?(?:\s*([,\/])\s*([\de.+-]+)(%)?)?\s*\)$/, zt = Ye.match(xt);
                    if (zt) {
                      const [ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui, Ui] = zt, ia = [Nn || " ", xr || " ", xi].join("");
                      if (ia === "  " || ia === "  /" || ia === ",," || ia === ",,,") {
                        const Na = [Cn, lr, Kr].join(""), Ta = Na === "%%%" ? 100 : Na === "" ? 255 : 0;
                        if (Ta) {
                          const Ra = [Kn(+pn / Ta, 0, 1), Kn(+Wn / Ta, 0, 1), Kn(+Nr / Ta, 0, 1), ui ? ur(+ui, Ui) : 1];
                          if (Jn(Ra)) return Ra;
                        }
                      }
                      return;
                    }
                  }
                  const ot = Ye.match(/^hsla?\(\s*([\de.+-]+)(?:deg)?(?:\s+|\s*(,)\s*)([\de.+-]+)%(?:\s+|\s*(,)\s*)([\de.+-]+)%(?:\s*([,\/])\s*([\de.+-]+)(%)?)?\s*\)$/);
                  if (ot) {
                    const [xt, zt, ln, pn, Cn, Nn, Wn, lr, xr] = ot, Nr = [ln || " ", Cn || " ", Wn].join("");
                    if (Nr === "  " || Nr === "  /" || Nr === ",," || Nr === ",,,") {
                      const Kr = [+zt, Kn(+pn, 0, 100), Kn(+Nn, 0, 100), lr ? ur(+lr, xr) : 1];
                      if (Jn(Kr)) return function([xi, ui, Ui, ia]) {
                        function Na(Ta) {
                          const Ra = (Ta + xi / 30) % 12, oo = ui * Math.min(Ui, 1 - Ui);
                          return Ui - oo * Math.max(-1, Math.min(Ra - 3, 9 - Ra, 1));
                        }
                        return xi = bn(xi), ui /= 100, Ui /= 100, [Na(0), Na(8), Na(4), ia];
                      }(Kr);
                    }
                  }
                }
                function jn(Ye) {
                  return parseInt(Ye.padEnd(2, Ye), 16) / 255;
                }
                function ur(Ye, He) {
                  return Kn(He ? Ye / 100 : Ye, 0, 1);
                }
                function Kn(Ye, He, ot) {
                  return Math.min(Math.max(He, Ye), ot);
                }
                function Jn(Ye) {
                  return !Ye.some(Number.isNaN);
                }
                const Lr = { aliceblue: [240, 248, 255], antiquewhite: [250, 235, 215], aqua: [0, 255, 255], aquamarine: [127, 255, 212], azure: [240, 255, 255], beige: [245, 245, 220], bisque: [255, 228, 196], black: [0, 0, 0], blanchedalmond: [255, 235, 205], blue: [0, 0, 255], blueviolet: [138, 43, 226], brown: [165, 42, 42], burlywood: [222, 184, 135], cadetblue: [95, 158, 160], chartreuse: [127, 255, 0], chocolate: [210, 105, 30], coral: [255, 127, 80], cornflowerblue: [100, 149, 237], cornsilk: [255, 248, 220], crimson: [220, 20, 60], cyan: [0, 255, 255], darkblue: [0, 0, 139], darkcyan: [0, 139, 139], darkgoldenrod: [184, 134, 11], darkgray: [169, 169, 169], darkgreen: [0, 100, 0], darkgrey: [169, 169, 169], darkkhaki: [189, 183, 107], darkmagenta: [139, 0, 139], darkolivegreen: [85, 107, 47], darkorange: [255, 140, 0], darkorchid: [153, 50, 204], darkred: [139, 0, 0], darksalmon: [233, 150, 122], darkseagreen: [143, 188, 143], darkslateblue: [72, 61, 139], darkslategray: [47, 79, 79], darkslategrey: [47, 79, 79], darkturquoise: [0, 206, 209], darkviolet: [148, 0, 211], deeppink: [255, 20, 147], deepskyblue: [0, 191, 255], dimgray: [105, 105, 105], dimgrey: [105, 105, 105], dodgerblue: [30, 144, 255], firebrick: [178, 34, 34], floralwhite: [255, 250, 240], forestgreen: [34, 139, 34], fuchsia: [255, 0, 255], gainsboro: [220, 220, 220], ghostwhite: [248, 248, 255], gold: [255, 215, 0], goldenrod: [218, 165, 32], gray: [128, 128, 128], green: [0, 128, 0], greenyellow: [173, 255, 47], grey: [128, 128, 128], honeydew: [240, 255, 240], hotpink: [255, 105, 180], indianred: [205, 92, 92], indigo: [75, 0, 130], ivory: [255, 255, 240], khaki: [240, 230, 140], lavender: [230, 230, 250], lavenderblush: [255, 240, 245], lawngreen: [124, 252, 0], lemonchiffon: [255, 250, 205], lightblue: [173, 216, 230], lightcoral: [240, 128, 128], lightcyan: [224, 255, 255], lightgoldenrodyellow: [250, 250, 210], lightgray: [211, 211, 211], lightgreen: [144, 238, 144], lightgrey: [211, 211, 211], lightpink: [255, 182, 193], lightsalmon: [255, 160, 122], lightseagreen: [32, 178, 170], lightskyblue: [135, 206, 250], lightslategray: [119, 136, 153], lightslategrey: [119, 136, 153], lightsteelblue: [176, 196, 222], lightyellow: [255, 255, 224], lime: [0, 255, 0], limegreen: [50, 205, 50], linen: [250, 240, 230], magenta: [255, 0, 255], maroon: [128, 0, 0], mediumaquamarine: [102, 205, 170], mediumblue: [0, 0, 205], mediumorchid: [186, 85, 211], mediumpurple: [147, 112, 219], mediumseagreen: [60, 179, 113], mediumslateblue: [123, 104, 238], mediumspringgreen: [0, 250, 154], mediumturquoise: [72, 209, 204], mediumvioletred: [199, 21, 133], midnightblue: [25, 25, 112], mintcream: [245, 255, 250], mistyrose: [255, 228, 225], moccasin: [255, 228, 181], navajowhite: [255, 222, 173], navy: [0, 0, 128], oldlace: [253, 245, 230], olive: [128, 128, 0], olivedrab: [107, 142, 35], orange: [255, 165, 0], orangered: [255, 69, 0], orchid: [218, 112, 214], palegoldenrod: [238, 232, 170], palegreen: [152, 251, 152], paleturquoise: [175, 238, 238], palevioletred: [219, 112, 147], papayawhip: [255, 239, 213], peachpuff: [255, 218, 185], peru: [205, 133, 63], pink: [255, 192, 203], plum: [221, 160, 221], powderblue: [176, 224, 230], purple: [128, 0, 128], rebeccapurple: [102, 51, 153], red: [255, 0, 0], rosybrown: [188, 143, 143], royalblue: [65, 105, 225], saddlebrown: [139, 69, 19], salmon: [250, 128, 114], sandybrown: [244, 164, 96], seagreen: [46, 139, 87], seashell: [255, 245, 238], sienna: [160, 82, 45], silver: [192, 192, 192], skyblue: [135, 206, 235], slateblue: [106, 90, 205], slategray: [112, 128, 144], slategrey: [112, 128, 144], snow: [255, 250, 250], springgreen: [0, 255, 127], steelblue: [70, 130, 180], tan: [210, 180, 140], teal: [0, 128, 128], thistle: [216, 191, 216], tomato: [255, 99, 71], turquoise: [64, 224, 208], violet: [238, 130, 238], wheat: [245, 222, 179], white: [255, 255, 255], whitesmoke: [245, 245, 245], yellow: [255, 255, 0], yellowgreen: [154, 205, 50] };
                class mr {
                  constructor(He, ot, xt, zt = 1, ln = !0) {
                    this.r = He, this.g = ot, this.b = xt, this.a = zt, ln || (this.r *= zt, this.g *= zt, this.b *= zt, zt || this.overwriteGetter("rgb", [He, ot, xt, zt]));
                  }
                  static parse(He) {
                    if (He instanceof mr) return He;
                    if (typeof He != "string") return;
                    const ot = er(He);
                    return ot ? new mr(...ot, !1) : void 0;
                  }
                  get rgb() {
                    const { r: He, g: ot, b: xt, a: zt } = this, ln = zt || 1 / 0;
                    return this.overwriteGetter("rgb", [He / ln, ot / ln, xt / ln, zt]);
                  }
                  get hcl() {
                    return this.overwriteGetter("hcl", function(He) {
                      const [ot, xt, zt, ln] = an(He), pn = Math.sqrt(xt * xt + zt * zt);
                      return [Math.round(1e4 * pn) ? bn(Math.atan2(zt, xt) * gn) : NaN, pn, ot, ln];
                    }(this.rgb));
                  }
                  get lab() {
                    return this.overwriteGetter("lab", an(this.rgb));
                  }
                  overwriteGetter(He, ot) {
                    return Object.defineProperty(this, He, { value: ot }), ot;
                  }
                  toString() {
                    const [He, ot, xt, zt] = this.rgb;
                    return `rgba(${[He, ot, xt].map((ln) => Math.round(255 * ln)).join(",")},${zt})`;
                  }
                }
                mr.black = new mr(0, 0, 0, 1), mr.white = new mr(1, 1, 1, 1), mr.transparent = new mr(0, 0, 0, 0), mr.red = new mr(1, 0, 0, 1);
                class Ar {
                  constructor(He, ot, xt) {
                    this.sensitivity = He ? ot ? "variant" : "case" : ot ? "accent" : "base", this.locale = xt, this.collator = new Intl.Collator(this.locale ? this.locale : [], { sensitivity: this.sensitivity, usage: "search" });
                  }
                  compare(He, ot) {
                    return this.collator.compare(He, ot);
                  }
                  resolvedLocale() {
                    return new Intl.Collator(this.locale ? this.locale : []).resolvedOptions().locale;
                  }
                }
                class Yr {
                  constructor(He, ot, xt, zt, ln) {
                    this.text = He, this.image = ot, this.scale = xt, this.fontStack = zt, this.textColor = ln;
                  }
                }
                class jr {
                  constructor(He) {
                    this.sections = He;
                  }
                  static fromString(He) {
                    return new jr([new Yr(He, null, null, null, null)]);
                  }
                  isEmpty() {
                    return this.sections.length === 0 || !this.sections.some((He) => He.text.length !== 0 || He.image && He.image.name.length !== 0);
                  }
                  static factory(He) {
                    return He instanceof jr ? He : jr.fromString(He);
                  }
                  toString() {
                    return this.sections.length === 0 ? "" : this.sections.map((He) => He.text).join("");
                  }
                }
                class Mr {
                  constructor(He) {
                    this.values = He.slice();
                  }
                  static parse(He) {
                    if (He instanceof Mr) return He;
                    if (typeof He == "number") return new Mr([He, He, He, He]);
                    if (Array.isArray(He) && !(He.length < 1 || He.length > 4)) {
                      for (const ot of He) if (typeof ot != "number") return;
                      switch (He.length) {
                        case 1:
                          He = [He[0], He[0], He[0], He[0]];
                          break;
                        case 2:
                          He = [He[0], He[1], He[0], He[1]];
                          break;
                        case 3:
                          He = [He[0], He[1], He[2], He[1]];
                      }
                      return new Mr(He);
                    }
                  }
                  toString() {
                    return JSON.stringify(this.values);
                  }
                }
                const Wr = /* @__PURE__ */ new Set(["center", "left", "right", "top", "bottom", "top-left", "top-right", "bottom-left", "bottom-right"]);
                class Bi {
                  constructor(He) {
                    this.values = He.slice();
                  }
                  static parse(He) {
                    if (He instanceof Bi) return He;
                    if (Array.isArray(He) && !(He.length < 1) && He.length % 2 == 0) {
                      for (let ot = 0; ot < He.length; ot += 2) {
                        const xt = He[ot], zt = He[ot + 1];
                        if (typeof xt != "string" || !Wr.has(xt) || !Array.isArray(zt) || zt.length !== 2 || typeof zt[0] != "number" || typeof zt[1] != "number") return;
                      }
                      return new Bi(He);
                    }
                  }
                  toString() {
                    return JSON.stringify(this.values);
                  }
                }
                class gi {
                  constructor(He) {
                    this.name = He.name, this.available = He.available;
                  }
                  toString() {
                    return this.name;
                  }
                  static fromString(He) {
                    return He ? new gi({ name: He, available: !1 }) : null;
                  }
                }
                function Zr(Ye, He, ot, xt) {
                  return typeof Ye == "number" && Ye >= 0 && Ye <= 255 && typeof He == "number" && He >= 0 && He <= 255 && typeof ot == "number" && ot >= 0 && ot <= 255 ? xt === void 0 || typeof xt == "number" && xt >= 0 && xt <= 1 ? null : `Invalid rgba value [${[Ye, He, ot, xt].join(", ")}]: 'a' must be between 0 and 1.` : `Invalid rgba value [${(typeof xt == "number" ? [Ye, He, ot, xt] : [Ye, He, ot]).join(", ")}]: 'r', 'g', and 'b' must be between 0 and 255.`;
                }
                function fi(Ye) {
                  if (Ye === null || typeof Ye == "string" || typeof Ye == "boolean" || typeof Ye == "number" || Ye instanceof mr || Ye instanceof Ar || Ye instanceof jr || Ye instanceof Mr || Ye instanceof Bi || Ye instanceof gi) return !0;
                  if (Array.isArray(Ye)) {
                    for (const He of Ye) if (!fi(He)) return !1;
                    return !0;
                  }
                  if (typeof Ye == "object") {
                    for (const He in Ye) if (!fi(Ye[He])) return !1;
                    return !0;
                  }
                  return !1;
                }
                function ri(Ye) {
                  if (Ye === null) return Yt;
                  if (typeof Ye == "string") return hn;
                  if (typeof Ye == "boolean") return Qt;
                  if (typeof Ye == "number") return sn;
                  if (Ye instanceof mr) return Ut;
                  if (Ye instanceof Ar) return vn;
                  if (Ye instanceof jr) return dn;
                  if (Ye instanceof Mr) return Ht;
                  if (Ye instanceof Bi) return cn;
                  if (Ye instanceof gi) return jt;
                  if (Array.isArray(Ye)) {
                    const He = Ye.length;
                    let ot;
                    for (const xt of Ye) {
                      const zt = ri(xt);
                      if (ot) {
                        if (ot === zt) continue;
                        ot = rn;
                        break;
                      }
                      ot = zt;
                    }
                    return wn(ot || rn, He);
                  }
                  return Wt;
                }
                function bi(Ye) {
                  const He = typeof Ye;
                  return Ye === null ? "" : He === "string" || He === "number" || He === "boolean" ? String(Ye) : Ye instanceof mr || Ye instanceof jr || Ye instanceof Mr || Ye instanceof Bi || Ye instanceof gi ? Ye.toString() : JSON.stringify(Ye);
                }
                class Zi {
                  constructor(He, ot) {
                    this.type = He, this.value = ot;
                  }
                  static parse(He, ot) {
                    if (He.length !== 2) return ot.error(`'literal' expression requires exactly one argument, but found ${He.length - 1} instead.`);
                    if (!fi(He[1])) return ot.error("invalid value");
                    const xt = He[1];
                    let zt = ri(xt);
                    const ln = ot.expectedType;
                    return zt.kind !== "array" || zt.N !== 0 || !ln || ln.kind !== "array" || typeof ln.N == "number" && ln.N !== 0 || (zt = ln), new Zi(zt, xt);
                  }
                  evaluate() {
                    return this.value;
                  }
                  eachChild() {
                  }
                  outputDefined() {
                    return !0;
                  }
                }
                class $i {
                  constructor(He) {
                    this.name = "ExpressionEvaluationError", this.message = He;
                  }
                  toJSON() {
                    return this.message;
                  }
                }
                const Br = { string: hn, number: sn, boolean: Qt, object: Wt };
                class Pr {
                  constructor(He, ot) {
                    this.type = He, this.args = ot;
                  }
                  static parse(He, ot) {
                    if (He.length < 2) return ot.error("Expected at least one argument.");
                    let xt, zt = 1;
                    const ln = He[0];
                    if (ln === "array") {
                      let Cn, Nn;
                      if (He.length > 2) {
                        const Wn = He[1];
                        if (typeof Wn != "string" || !(Wn in Br) || Wn === "object") return ot.error('The item type argument of "array" must be one of string, number, boolean', 1);
                        Cn = Br[Wn], zt++;
                      } else Cn = rn;
                      if (He.length > 3) {
                        if (He[2] !== null && (typeof He[2] != "number" || He[2] < 0 || He[2] !== Math.floor(He[2]))) return ot.error('The length argument to "array" must be a positive integer literal', 2);
                        Nn = He[2], zt++;
                      }
                      xt = wn(Cn, Nn);
                    } else {
                      if (!Br[ln]) throw new Error(`Types doesn't contain name = ${ln}`);
                      xt = Br[ln];
                    }
                    const pn = [];
                    for (; zt < He.length; zt++) {
                      const Cn = ot.parse(He[zt], zt, rn);
                      if (!Cn) return null;
                      pn.push(Cn);
                    }
                    return new Pr(xt, pn);
                  }
                  evaluate(He) {
                    for (let ot = 0; ot < this.args.length; ot++) {
                      const xt = this.args[ot].evaluate(He);
                      if (!Ie(this.type, ri(xt))) return xt;
                      if (ot === this.args.length - 1) throw new $i(`Expected value to be of type ${Rn(this.type)}, but found ${Rn(ri(xt))} instead.`);
                    }
                    throw new Error();
                  }
                  eachChild(He) {
                    this.args.forEach(He);
                  }
                  outputDefined() {
                    return this.args.every((He) => He.outputDefined());
                  }
                }
                const ei = { "to-boolean": Qt, "to-color": Ut, "to-number": sn, "to-string": hn };
                class Ci {
                  constructor(He, ot) {
                    this.type = He, this.args = ot;
                  }
                  static parse(He, ot) {
                    if (He.length < 2) return ot.error("Expected at least one argument.");
                    const xt = He[0];
                    if (!ei[xt]) throw new Error(`Can't parse ${xt} as it is not part of the known types`);
                    if ((xt === "to-boolean" || xt === "to-string") && He.length !== 2) return ot.error("Expected one argument.");
                    const zt = ei[xt], ln = [];
                    for (let pn = 1; pn < He.length; pn++) {
                      const Cn = ot.parse(He[pn], pn, rn);
                      if (!Cn) return null;
                      ln.push(Cn);
                    }
                    return new Ci(zt, ln);
                  }
                  evaluate(He) {
                    switch (this.type.kind) {
                      case "boolean":
                        return !!this.args[0].evaluate(He);
                      case "color": {
                        let ot, xt;
                        for (const zt of this.args) {
                          if (ot = zt.evaluate(He), xt = null, ot instanceof mr) return ot;
                          if (typeof ot == "string") {
                            const ln = He.parseColor(ot);
                            if (ln) return ln;
                          } else if (Array.isArray(ot) && (xt = ot.length < 3 || ot.length > 4 ? `Invalid rbga value ${JSON.stringify(ot)}: expected an array containing either three or four numeric values.` : Zr(ot[0], ot[1], ot[2], ot[3]), !xt)) return new mr(ot[0] / 255, ot[1] / 255, ot[2] / 255, ot[3]);
                        }
                        throw new $i(xt || `Could not parse color from value '${typeof ot == "string" ? ot : JSON.stringify(ot)}'`);
                      }
                      case "padding": {
                        let ot;
                        for (const xt of this.args) {
                          ot = xt.evaluate(He);
                          const zt = Mr.parse(ot);
                          if (zt) return zt;
                        }
                        throw new $i(`Could not parse padding from value '${typeof ot == "string" ? ot : JSON.stringify(ot)}'`);
                      }
                      case "variableAnchorOffsetCollection": {
                        let ot;
                        for (const xt of this.args) {
                          ot = xt.evaluate(He);
                          const zt = Bi.parse(ot);
                          if (zt) return zt;
                        }
                        throw new $i(`Could not parse variableAnchorOffsetCollection from value '${typeof ot == "string" ? ot : JSON.stringify(ot)}'`);
                      }
                      case "number": {
                        let ot = null;
                        for (const xt of this.args) {
                          if (ot = xt.evaluate(He), ot === null) return 0;
                          const zt = Number(ot);
                          if (!isNaN(zt)) return zt;
                        }
                        throw new $i(`Could not convert ${JSON.stringify(ot)} to number.`);
                      }
                      case "formatted":
                        return jr.fromString(bi(this.args[0].evaluate(He)));
                      case "resolvedImage":
                        return gi.fromString(bi(this.args[0].evaluate(He)));
                      default:
                        return bi(this.args[0].evaluate(He));
                    }
                  }
                  eachChild(He) {
                    this.args.forEach(He);
                  }
                  outputDefined() {
                    return this.args.every((He) => He.outputDefined());
                  }
                }
                const oa = ["Unknown", "Point", "LineString", "Polygon"];
                class ka {
                  constructor() {
                    this.globals = null, this.feature = null, this.featureState = null, this.formattedSection = null, this._parseColorCache = {}, this.availableImages = null, this.canonical = null;
                  }
                  id() {
                    return this.feature && "id" in this.feature ? this.feature.id : null;
                  }
                  geometryType() {
                    return this.feature ? typeof this.feature.type == "number" ? oa[this.feature.type] : this.feature.type : null;
                  }
                  geometry() {
                    return this.feature && "geometry" in this.feature ? this.feature.geometry : null;
                  }
                  canonicalID() {
                    return this.canonical;
                  }
                  properties() {
                    return this.feature && this.feature.properties || {};
                  }
                  parseColor(He) {
                    let ot = this._parseColorCache[He];
                    return ot || (ot = this._parseColorCache[He] = mr.parse(He)), ot;
                  }
                }
                class aa {
                  constructor(He, ot, xt = [], zt, ln = new Vt(), pn = []) {
                    this.registry = He, this.path = xt, this.key = xt.map((Cn) => `[${Cn}]`).join(""), this.scope = ln, this.errors = pn, this.expectedType = zt, this._isConstant = ot;
                  }
                  parse(He, ot, xt, zt, ln = {}) {
                    return ot ? this.concat(ot, xt, zt)._parse(He, ln) : this._parse(He, ln);
                  }
                  _parse(He, ot) {
                    function xt(zt, ln, pn) {
                      return pn === "assert" ? new Pr(ln, [zt]) : pn === "coerce" ? new Ci(ln, [zt]) : zt;
                    }
                    if (He !== null && typeof He != "string" && typeof He != "boolean" && typeof He != "number" || (He = ["literal", He]), Array.isArray(He)) {
                      if (He.length === 0) return this.error('Expected an array with at least one element. If you wanted a literal array, use ["literal", []].');
                      const zt = He[0];
                      if (typeof zt != "string") return this.error(`Expression name must be a string, but found ${typeof zt} instead. If you wanted a literal array, use ["literal", [...]].`, 0), null;
                      const ln = this.registry[zt];
                      if (ln) {
                        let pn = ln.parse(He, this);
                        if (!pn) return null;
                        if (this.expectedType) {
                          const Cn = this.expectedType, Nn = pn.type;
                          if (Cn.kind !== "string" && Cn.kind !== "number" && Cn.kind !== "boolean" && Cn.kind !== "object" && Cn.kind !== "array" || Nn.kind !== "value") if (Cn.kind !== "color" && Cn.kind !== "formatted" && Cn.kind !== "resolvedImage" || Nn.kind !== "value" && Nn.kind !== "string") if (Cn.kind !== "padding" || Nn.kind !== "value" && Nn.kind !== "number" && Nn.kind !== "array") if (Cn.kind !== "variableAnchorOffsetCollection" || Nn.kind !== "value" && Nn.kind !== "array") {
                            if (this.checkSubtype(Cn, Nn)) return null;
                          } else pn = xt(pn, Cn, ot.typeAnnotation || "coerce");
                          else pn = xt(pn, Cn, ot.typeAnnotation || "coerce");
                          else pn = xt(pn, Cn, ot.typeAnnotation || "coerce");
                          else pn = xt(pn, Cn, ot.typeAnnotation || "assert");
                        }
                        if (!(pn instanceof Zi) && pn.type.kind !== "resolvedImage" && this._isConstant(pn)) {
                          const Cn = new ka();
                          try {
                            pn = new Zi(pn.type, pn.evaluate(Cn));
                          } catch (Nn) {
                            return this.error(Nn.message), null;
                          }
                        }
                        return pn;
                      }
                      return this.error(`Unknown expression "${zt}". If you wanted a literal array, use ["literal", [...]].`, 0);
                    }
                    return He === void 0 ? this.error("'undefined' value invalid. Use null instead.") : typeof He == "object" ? this.error('Bare objects invalid. Use ["literal", {...}] instead.') : this.error(`Expected an array, but found ${typeof He} instead.`);
                  }
                  concat(He, ot, xt) {
                    const zt = typeof He == "number" ? this.path.concat(He) : this.path, ln = xt ? this.scope.concat(xt) : this.scope;
                    return new aa(this.registry, this._isConstant, zt, ot || null, ln, this.errors);
                  }
                  error(He, ...ot) {
                    const xt = `${this.key}${ot.map((zt) => `[${zt}]`).join("")}`;
                    this.errors.push(new Kt(xt, He));
                  }
                  checkSubtype(He, ot) {
                    const xt = Ie(He, ot);
                    return xt && this.error(xt), xt;
                  }
                }
                class hi {
                  constructor(He, ot) {
                    this.type = ot.type, this.bindings = [].concat(He), this.result = ot;
                  }
                  evaluate(He) {
                    return this.result.evaluate(He);
                  }
                  eachChild(He) {
                    for (const ot of this.bindings) He(ot[1]);
                    He(this.result);
                  }
                  static parse(He, ot) {
                    if (He.length < 4) return ot.error(`Expected at least 3 arguments, but found ${He.length - 1} instead.`);
                    const xt = [];
                    for (let ln = 1; ln < He.length - 1; ln += 2) {
                      const pn = He[ln];
                      if (typeof pn != "string") return ot.error(`Expected string, but found ${typeof pn} instead.`, ln);
                      if (/[^a-zA-Z0-9_]/.test(pn)) return ot.error("Variable names must contain only alphanumeric characters or '_'.", ln);
                      const Cn = ot.parse(He[ln + 1], ln + 1);
                      if (!Cn) return null;
                      xt.push([pn, Cn]);
                    }
                    const zt = ot.parse(He[He.length - 1], He.length - 1, ot.expectedType, xt);
                    return zt ? new hi(xt, zt) : null;
                  }
                  outputDefined() {
                    return this.result.outputDefined();
                  }
                }
                class Gr {
                  constructor(He, ot) {
                    this.type = ot.type, this.name = He, this.boundExpression = ot;
                  }
                  static parse(He, ot) {
                    if (He.length !== 2 || typeof He[1] != "string") return ot.error("'var' expression requires exactly one string literal argument.");
                    const xt = He[1];
                    return ot.scope.has(xt) ? new Gr(xt, ot.scope.get(xt)) : ot.error(`Unknown variable "${xt}". Make sure "${xt}" has been bound in an enclosing "let" expression before using it.`, 1);
                  }
                  evaluate(He) {
                    return this.boundExpression.evaluate(He);
                  }
                  eachChild() {
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                class Ca {
                  constructor(He, ot, xt) {
                    this.type = He, this.index = ot, this.input = xt;
                  }
                  static parse(He, ot) {
                    if (He.length !== 3) return ot.error(`Expected 2 arguments, but found ${He.length - 1} instead.`);
                    const xt = ot.parse(He[1], 1, sn), zt = ot.parse(He[2], 2, wn(ot.expectedType || rn));
                    if (!xt || !zt) return null;
                    const ln = zt.type;
                    return new Ca(ln.itemType, xt, zt);
                  }
                  evaluate(He) {
                    const ot = this.index.evaluate(He), xt = this.input.evaluate(He);
                    if (ot < 0) throw new $i(`Array index out of bounds: ${ot} < 0.`);
                    if (ot >= xt.length) throw new $i(`Array index out of bounds: ${ot} > ${xt.length - 1}.`);
                    if (ot !== Math.floor(ot)) throw new $i(`Array index must be an integer, but found ${ot} instead.`);
                    return xt[ot];
                  }
                  eachChild(He) {
                    He(this.index), He(this.input);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                class Sr {
                  constructor(He, ot) {
                    this.type = Qt, this.needle = He, this.haystack = ot;
                  }
                  static parse(He, ot) {
                    if (He.length !== 3) return ot.error(`Expected 2 arguments, but found ${He.length - 1} instead.`);
                    const xt = ot.parse(He[1], 1, rn), zt = ot.parse(He[2], 2, rn);
                    return xt && zt ? rt(xt.type, [Qt, hn, sn, Yt, rn]) ? new Sr(xt, zt) : ot.error(`Expected first argument to be of type boolean, string, number or null, but found ${Rn(xt.type)} instead`) : null;
                  }
                  evaluate(He) {
                    const ot = this.needle.evaluate(He), xt = this.haystack.evaluate(He);
                    if (!xt) return !1;
                    if (!ft(ot, ["boolean", "string", "number", "null"])) throw new $i(`Expected first argument to be of type boolean, string, number or null, but found ${Rn(ri(ot))} instead.`);
                    if (!ft(xt, ["string", "array"])) throw new $i(`Expected second argument to be of type array or string, but found ${Rn(ri(xt))} instead.`);
                    return xt.indexOf(ot) >= 0;
                  }
                  eachChild(He) {
                    He(this.needle), He(this.haystack);
                  }
                  outputDefined() {
                    return !0;
                  }
                }
                class Dr {
                  constructor(He, ot, xt) {
                    this.type = sn, this.needle = He, this.haystack = ot, this.fromIndex = xt;
                  }
                  static parse(He, ot) {
                    if (He.length <= 2 || He.length >= 5) return ot.error(`Expected 3 or 4 arguments, but found ${He.length - 1} instead.`);
                    const xt = ot.parse(He[1], 1, rn), zt = ot.parse(He[2], 2, rn);
                    if (!xt || !zt) return null;
                    if (!rt(xt.type, [Qt, hn, sn, Yt, rn])) return ot.error(`Expected first argument to be of type boolean, string, number or null, but found ${Rn(xt.type)} instead`);
                    if (He.length === 4) {
                      const ln = ot.parse(He[3], 3, sn);
                      return ln ? new Dr(xt, zt, ln) : null;
                    }
                    return new Dr(xt, zt);
                  }
                  evaluate(He) {
                    const ot = this.needle.evaluate(He), xt = this.haystack.evaluate(He);
                    if (!ft(ot, ["boolean", "string", "number", "null"])) throw new $i(`Expected first argument to be of type boolean, string, number or null, but found ${Rn(ri(ot))} instead.`);
                    if (!ft(xt, ["string", "array"])) throw new $i(`Expected second argument to be of type array or string, but found ${Rn(ri(xt))} instead.`);
                    if (this.fromIndex) {
                      const zt = this.fromIndex.evaluate(He);
                      return xt.indexOf(ot, zt);
                    }
                    return xt.indexOf(ot);
                  }
                  eachChild(He) {
                    He(this.needle), He(this.haystack), this.fromIndex && He(this.fromIndex);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                class si {
                  constructor(He, ot, xt, zt, ln, pn) {
                    this.inputType = He, this.type = ot, this.input = xt, this.cases = zt, this.outputs = ln, this.otherwise = pn;
                  }
                  static parse(He, ot) {
                    if (He.length < 5) return ot.error(`Expected at least 4 arguments, but found only ${He.length - 1}.`);
                    if (He.length % 2 != 1) return ot.error("Expected an even number of arguments.");
                    let xt, zt;
                    ot.expectedType && ot.expectedType.kind !== "value" && (zt = ot.expectedType);
                    const ln = {}, pn = [];
                    for (let Wn = 2; Wn < He.length - 1; Wn += 2) {
                      let lr = He[Wn];
                      const xr = He[Wn + 1];
                      Array.isArray(lr) || (lr = [lr]);
                      const Nr = ot.concat(Wn);
                      if (lr.length === 0) return Nr.error("Expected at least one branch label.");
                      for (const xi of lr) {
                        if (typeof xi != "number" && typeof xi != "string") return Nr.error("Branch labels must be numbers or strings.");
                        if (typeof xi == "number" && Math.abs(xi) > Number.MAX_SAFE_INTEGER) return Nr.error(`Branch labels must be integers no larger than ${Number.MAX_SAFE_INTEGER}.`);
                        if (typeof xi == "number" && Math.floor(xi) !== xi) return Nr.error("Numeric branch labels must be integer values.");
                        if (xt) {
                          if (Nr.checkSubtype(xt, ri(xi))) return null;
                        } else xt = ri(xi);
                        if (ln[String(xi)] !== void 0) return Nr.error("Branch labels must be unique.");
                        ln[String(xi)] = pn.length;
                      }
                      const Kr = ot.parse(xr, Wn, zt);
                      if (!Kr) return null;
                      zt = zt || Kr.type, pn.push(Kr);
                    }
                    const Cn = ot.parse(He[1], 1, rn);
                    if (!Cn) return null;
                    const Nn = ot.parse(He[He.length - 1], He.length - 1, zt);
                    return Nn ? Cn.type.kind !== "value" && ot.concat(1).checkSubtype(xt, Cn.type) ? null : new si(xt, zt, Cn, ln, pn, Nn) : null;
                  }
                  evaluate(He) {
                    const ot = this.input.evaluate(He);
                    return (ri(ot) === this.inputType && this.outputs[this.cases[ot]] || this.otherwise).evaluate(He);
                  }
                  eachChild(He) {
                    He(this.input), this.outputs.forEach(He), He(this.otherwise);
                  }
                  outputDefined() {
                    return this.outputs.every((He) => He.outputDefined()) && this.otherwise.outputDefined();
                  }
                }
                class zr {
                  constructor(He, ot, xt) {
                    this.type = He, this.branches = ot, this.otherwise = xt;
                  }
                  static parse(He, ot) {
                    if (He.length < 4) return ot.error(`Expected at least 3 arguments, but found only ${He.length - 1}.`);
                    if (He.length % 2 != 0) return ot.error("Expected an odd number of arguments.");
                    let xt;
                    ot.expectedType && ot.expectedType.kind !== "value" && (xt = ot.expectedType);
                    const zt = [];
                    for (let pn = 1; pn < He.length - 1; pn += 2) {
                      const Cn = ot.parse(He[pn], pn, Qt);
                      if (!Cn) return null;
                      const Nn = ot.parse(He[pn + 1], pn + 1, xt);
                      if (!Nn) return null;
                      zt.push([Cn, Nn]), xt = xt || Nn.type;
                    }
                    const ln = ot.parse(He[He.length - 1], He.length - 1, xt);
                    if (!ln) return null;
                    if (!xt) throw new Error("Can't infer output type");
                    return new zr(xt, zt, ln);
                  }
                  evaluate(He) {
                    for (const [ot, xt] of this.branches) if (ot.evaluate(He)) return xt.evaluate(He);
                    return this.otherwise.evaluate(He);
                  }
                  eachChild(He) {
                    for (const [ot, xt] of this.branches) He(ot), He(xt);
                    He(this.otherwise);
                  }
                  outputDefined() {
                    return this.branches.every(([He, ot]) => ot.outputDefined()) && this.otherwise.outputDefined();
                  }
                }
                class di {
                  constructor(He, ot, xt, zt) {
                    this.type = He, this.input = ot, this.beginIndex = xt, this.endIndex = zt;
                  }
                  static parse(He, ot) {
                    if (He.length <= 2 || He.length >= 5) return ot.error(`Expected 3 or 4 arguments, but found ${He.length - 1} instead.`);
                    const xt = ot.parse(He[1], 1, rn), zt = ot.parse(He[2], 2, sn);
                    if (!xt || !zt) return null;
                    if (!rt(xt.type, [wn(rn), hn, rn])) return ot.error(`Expected first argument to be of type array or string, but found ${Rn(xt.type)} instead`);
                    if (He.length === 4) {
                      const ln = ot.parse(He[3], 3, sn);
                      return ln ? new di(xt.type, xt, zt, ln) : null;
                    }
                    return new di(xt.type, xt, zt);
                  }
                  evaluate(He) {
                    const ot = this.input.evaluate(He), xt = this.beginIndex.evaluate(He);
                    if (!ft(ot, ["string", "array"])) throw new $i(`Expected first argument to be of type array or string, but found ${Rn(ri(ot))} instead.`);
                    if (this.endIndex) {
                      const zt = this.endIndex.evaluate(He);
                      return ot.slice(xt, zt);
                    }
                    return ot.slice(xt);
                  }
                  eachChild(He) {
                    He(this.input), He(this.beginIndex), this.endIndex && He(this.endIndex);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                function Da(Ye, He) {
                  const ot = Ye.length - 1;
                  let xt, zt, ln = 0, pn = ot, Cn = 0;
                  for (; ln <= pn; ) if (Cn = Math.floor((ln + pn) / 2), xt = Ye[Cn], zt = Ye[Cn + 1], xt <= He) {
                    if (Cn === ot || He < zt) return Cn;
                    ln = Cn + 1;
                  } else {
                    if (!(xt > He)) throw new $i("Input is not a number.");
                    pn = Cn - 1;
                  }
                  return 0;
                }
                class la {
                  constructor(He, ot, xt) {
                    this.type = He, this.input = ot, this.labels = [], this.outputs = [];
                    for (const [zt, ln] of xt) this.labels.push(zt), this.outputs.push(ln);
                  }
                  static parse(He, ot) {
                    if (He.length - 1 < 4) return ot.error(`Expected at least 4 arguments, but found only ${He.length - 1}.`);
                    if ((He.length - 1) % 2 != 0) return ot.error("Expected an even number of arguments.");
                    const xt = ot.parse(He[1], 1, sn);
                    if (!xt) return null;
                    const zt = [];
                    let ln = null;
                    ot.expectedType && ot.expectedType.kind !== "value" && (ln = ot.expectedType);
                    for (let pn = 1; pn < He.length; pn += 2) {
                      const Cn = pn === 1 ? -1 / 0 : He[pn], Nn = He[pn + 1], Wn = pn, lr = pn + 1;
                      if (typeof Cn != "number") return ot.error('Input/output pairs for "step" expressions must be defined using literal numeric values (not computed expressions) for the input values.', Wn);
                      if (zt.length && zt[zt.length - 1][0] >= Cn) return ot.error('Input/output pairs for "step" expressions must be arranged with input values in strictly ascending order.', Wn);
                      const xr = ot.parse(Nn, lr, ln);
                      if (!xr) return null;
                      ln = ln || xr.type, zt.push([Cn, xr]);
                    }
                    return new la(ln, xt, zt);
                  }
                  evaluate(He) {
                    const ot = this.labels, xt = this.outputs;
                    if (ot.length === 1) return xt[0].evaluate(He);
                    const zt = this.input.evaluate(He);
                    if (zt <= ot[0]) return xt[0].evaluate(He);
                    const ln = ot.length;
                    return zt >= ot[ln - 1] ? xt[ln - 1].evaluate(He) : xt[Da(ot, zt)].evaluate(He);
                  }
                  eachChild(He) {
                    He(this.input);
                    for (const ot of this.outputs) He(ot);
                  }
                  outputDefined() {
                    return this.outputs.every((He) => He.outputDefined());
                  }
                }
                function wa(Ye) {
                  return Ye.__esModule && Object.prototype.hasOwnProperty.call(Ye, "default") ? Ye.default : Ye;
                }
                var Wa = Qr;
                function Qr(Ye, He, ot, xt) {
                  this.cx = 3 * Ye, this.bx = 3 * (ot - Ye) - this.cx, this.ax = 1 - this.cx - this.bx, this.cy = 3 * He, this.by = 3 * (xt - He) - this.cy, this.ay = 1 - this.cy - this.by, this.p1x = Ye, this.p1y = He, this.p2x = ot, this.p2y = xt;
                }
                Qr.prototype = { sampleCurveX: function(Ye) {
                  return ((this.ax * Ye + this.bx) * Ye + this.cx) * Ye;
                }, sampleCurveY: function(Ye) {
                  return ((this.ay * Ye + this.by) * Ye + this.cy) * Ye;
                }, sampleCurveDerivativeX: function(Ye) {
                  return (3 * this.ax * Ye + 2 * this.bx) * Ye + this.cx;
                }, solveCurveX: function(Ye, He) {
                  if (He === void 0 && (He = 1e-6), Ye < 0) return 0;
                  if (Ye > 1) return 1;
                  for (var ot = Ye, xt = 0; xt < 8; xt++) {
                    var zt = this.sampleCurveX(ot) - Ye;
                    if (Math.abs(zt) < He) return ot;
                    var ln = this.sampleCurveDerivativeX(ot);
                    if (Math.abs(ln) < 1e-6) break;
                    ot -= zt / ln;
                  }
                  var pn = 0, Cn = 1;
                  for (ot = Ye, xt = 0; xt < 20 && (zt = this.sampleCurveX(ot), !(Math.abs(zt - Ye) < He)); xt++) Ye > zt ? pn = ot : Cn = ot, ot = 0.5 * (Cn - pn) + pn;
                  return ot;
                }, solve: function(Ye, He) {
                  return this.sampleCurveY(this.solveCurveX(Ye, He));
                } };
                var Ur = wa(Wa);
                function rr(Ye, He, ot) {
                  return Ye + ot * (He - Ye);
                }
                function vr(Ye, He, ot) {
                  return Ye.map((xt, zt) => rr(xt, He[zt], ot));
                }
                const Rr = { number: rr, color: function(Ye, He, ot, xt = "rgb") {
                  switch (xt) {
                    case "rgb": {
                      const [zt, ln, pn, Cn] = vr(Ye.rgb, He.rgb, ot);
                      return new mr(zt, ln, pn, Cn, !1);
                    }
                    case "hcl": {
                      const [zt, ln, pn, Cn] = Ye.hcl, [Nn, Wn, lr, xr] = He.hcl;
                      let Nr, Kr;
                      if (isNaN(zt) || isNaN(Nn)) isNaN(zt) ? isNaN(Nn) ? Nr = NaN : (Nr = Nn, pn !== 1 && pn !== 0 || (Kr = Wn)) : (Nr = zt, lr !== 1 && lr !== 0 || (Kr = ln));
                      else {
                        let Na = Nn - zt;
                        Nn > zt && Na > 180 ? Na -= 360 : Nn < zt && zt - Nn > 180 && (Na += 360), Nr = zt + ot * Na;
                      }
                      const [xi, ui, Ui, ia] = function([Na, Ta, Ra, oo]) {
                        return Na = isNaN(Na) ? 0 : Na * _n, Ln([Ra, Math.cos(Na) * Ta, Math.sin(Na) * Ta, oo]);
                      }([Nr, Kr ?? rr(ln, Wn, ot), rr(pn, lr, ot), rr(Cn, xr, ot)]);
                      return new mr(xi, ui, Ui, ia, !1);
                    }
                    case "lab": {
                      const [zt, ln, pn, Cn] = Ln(vr(Ye.lab, He.lab, ot));
                      return new mr(zt, ln, pn, Cn, !1);
                    }
                  }
                }, array: vr, padding: function(Ye, He, ot) {
                  return new Mr(vr(Ye.values, He.values, ot));
                }, variableAnchorOffsetCollection: function(Ye, He, ot) {
                  const xt = Ye.values, zt = He.values;
                  if (xt.length !== zt.length) throw new $i(`Cannot interpolate values of different length. from: ${Ye.toString()}, to: ${He.toString()}`);
                  const ln = [];
                  for (let pn = 0; pn < xt.length; pn += 2) {
                    if (xt[pn] !== zt[pn]) throw new $i(`Cannot interpolate values containing mismatched anchors. from[${pn}]: ${xt[pn]}, to[${pn}]: ${zt[pn]}`);
                    ln.push(xt[pn]);
                    const [Cn, Nn] = xt[pn + 1], [Wn, lr] = zt[pn + 1];
                    ln.push([rr(Cn, Wn, ot), rr(Nn, lr, ot)]);
                  }
                  return new Bi(ln);
                } };
                class Vr {
                  constructor(He, ot, xt, zt, ln) {
                    this.type = He, this.operator = ot, this.interpolation = xt, this.input = zt, this.labels = [], this.outputs = [];
                    for (const [pn, Cn] of ln) this.labels.push(pn), this.outputs.push(Cn);
                  }
                  static interpolationFactor(He, ot, xt, zt) {
                    let ln = 0;
                    if (He.name === "exponential") ln = Xr(ot, He.base, xt, zt);
                    else if (He.name === "linear") ln = Xr(ot, 1, xt, zt);
                    else if (He.name === "cubic-bezier") {
                      const pn = He.controlPoints;
                      ln = new Ur(pn[0], pn[1], pn[2], pn[3]).solve(Xr(ot, 1, xt, zt));
                    }
                    return ln;
                  }
                  static parse(He, ot) {
                    let [xt, zt, ln, ...pn] = He;
                    if (!Array.isArray(zt) || zt.length === 0) return ot.error("Expected an interpolation type expression.", 1);
                    if (zt[0] === "linear") zt = { name: "linear" };
                    else if (zt[0] === "exponential") {
                      const Wn = zt[1];
                      if (typeof Wn != "number") return ot.error("Exponential interpolation requires a numeric base.", 1, 1);
                      zt = { name: "exponential", base: Wn };
                    } else {
                      if (zt[0] !== "cubic-bezier") return ot.error(`Unknown interpolation type ${String(zt[0])}`, 1, 0);
                      {
                        const Wn = zt.slice(1);
                        if (Wn.length !== 4 || Wn.some((lr) => typeof lr != "number" || lr < 0 || lr > 1)) return ot.error("Cubic bezier interpolation requires four numeric arguments with values between 0 and 1.", 1);
                        zt = { name: "cubic-bezier", controlPoints: Wn };
                      }
                    }
                    if (He.length - 1 < 4) return ot.error(`Expected at least 4 arguments, but found only ${He.length - 1}.`);
                    if ((He.length - 1) % 2 != 0) return ot.error("Expected an even number of arguments.");
                    if (ln = ot.parse(ln, 2, sn), !ln) return null;
                    const Cn = [];
                    let Nn = null;
                    xt === "interpolate-hcl" || xt === "interpolate-lab" ? Nn = Ut : ot.expectedType && ot.expectedType.kind !== "value" && (Nn = ot.expectedType);
                    for (let Wn = 0; Wn < pn.length; Wn += 2) {
                      const lr = pn[Wn], xr = pn[Wn + 1], Nr = Wn + 3, Kr = Wn + 4;
                      if (typeof lr != "number") return ot.error('Input/output pairs for "interpolate" expressions must be defined using literal numeric values (not computed expressions) for the input values.', Nr);
                      if (Cn.length && Cn[Cn.length - 1][0] >= lr) return ot.error('Input/output pairs for "interpolate" expressions must be arranged with input values in strictly ascending order.', Nr);
                      const xi = ot.parse(xr, Kr, Nn);
                      if (!xi) return null;
                      Nn = Nn || xi.type, Cn.push([lr, xi]);
                    }
                    return pt(Nn, sn) || pt(Nn, Ut) || pt(Nn, Ht) || pt(Nn, cn) || pt(Nn, wn(sn)) ? new Vr(Nn, xt, zt, ln, Cn) : ot.error(`Type ${Rn(Nn)} is not interpolatable.`);
                  }
                  evaluate(He) {
                    const ot = this.labels, xt = this.outputs;
                    if (ot.length === 1) return xt[0].evaluate(He);
                    const zt = this.input.evaluate(He);
                    if (zt <= ot[0]) return xt[0].evaluate(He);
                    const ln = ot.length;
                    if (zt >= ot[ln - 1]) return xt[ln - 1].evaluate(He);
                    const pn = Da(ot, zt), Cn = ot[pn], Nn = ot[pn + 1], Wn = Vr.interpolationFactor(this.interpolation, zt, Cn, Nn), lr = xt[pn].evaluate(He), xr = xt[pn + 1].evaluate(He);
                    switch (this.operator) {
                      case "interpolate":
                        return Rr[this.type.kind](lr, xr, Wn);
                      case "interpolate-hcl":
                        return Rr.color(lr, xr, Wn, "hcl");
                      case "interpolate-lab":
                        return Rr.color(lr, xr, Wn, "lab");
                    }
                  }
                  eachChild(He) {
                    He(this.input);
                    for (const ot of this.outputs) He(ot);
                  }
                  outputDefined() {
                    return this.outputs.every((He) => He.outputDefined());
                  }
                }
                function Xr(Ye, He, ot, xt) {
                  const zt = xt - ot, ln = Ye - ot;
                  return zt === 0 ? 0 : He === 1 ? ln / zt : (Math.pow(He, ln) - 1) / (Math.pow(He, zt) - 1);
                }
                class Jr {
                  constructor(He, ot) {
                    this.type = He, this.args = ot;
                  }
                  static parse(He, ot) {
                    if (He.length < 2) return ot.error("Expectected at least one argument.");
                    let xt = null;
                    const zt = ot.expectedType;
                    zt && zt.kind !== "value" && (xt = zt);
                    const ln = [];
                    for (const Cn of He.slice(1)) {
                      const Nn = ot.parse(Cn, 1 + ln.length, xt, void 0, { typeAnnotation: "omit" });
                      if (!Nn) return null;
                      xt = xt || Nn.type, ln.push(Nn);
                    }
                    if (!xt) throw new Error("No output type");
                    const pn = zt && ln.some((Cn) => Ie(zt, Cn.type));
                    return new Jr(pn ? rn : xt, ln);
                  }
                  evaluate(He) {
                    let ot, xt = null, zt = 0;
                    for (const ln of this.args) if (zt++, xt = ln.evaluate(He), xt && xt instanceof gi && !xt.available && (ot || (ot = xt.name), xt = null, zt === this.args.length && (xt = ot)), xt !== null) break;
                    return xt;
                  }
                  eachChild(He) {
                    this.args.forEach(He);
                  }
                  outputDefined() {
                    return this.args.every((He) => He.outputDefined());
                  }
                }
                function ti(Ye, He) {
                  return Ye === "==" || Ye === "!=" ? He.kind === "boolean" || He.kind === "string" || He.kind === "number" || He.kind === "null" || He.kind === "value" : He.kind === "string" || He.kind === "number" || He.kind === "value";
                }
                function Hr(Ye, He, ot, xt) {
                  return xt.compare(He, ot) === 0;
                }
                function Er(Ye, He, ot) {
                  const xt = Ye !== "==" && Ye !== "!=";
                  return class Jy {
                    constructor(ln, pn, Cn) {
                      this.type = Qt, this.lhs = ln, this.rhs = pn, this.collator = Cn, this.hasUntypedArgument = ln.type.kind === "value" || pn.type.kind === "value";
                    }
                    static parse(ln, pn) {
                      if (ln.length !== 3 && ln.length !== 4) return pn.error("Expected two or three arguments.");
                      const Cn = ln[0];
                      let Nn = pn.parse(ln[1], 1, rn);
                      if (!Nn) return null;
                      if (!ti(Cn, Nn.type)) return pn.concat(1).error(`"${Cn}" comparisons are not supported for type '${Rn(Nn.type)}'.`);
                      let Wn = pn.parse(ln[2], 2, rn);
                      if (!Wn) return null;
                      if (!ti(Cn, Wn.type)) return pn.concat(2).error(`"${Cn}" comparisons are not supported for type '${Rn(Wn.type)}'.`);
                      if (Nn.type.kind !== Wn.type.kind && Nn.type.kind !== "value" && Wn.type.kind !== "value") return pn.error(`Cannot compare types '${Rn(Nn.type)}' and '${Rn(Wn.type)}'.`);
                      xt && (Nn.type.kind === "value" && Wn.type.kind !== "value" ? Nn = new Pr(Wn.type, [Nn]) : Nn.type.kind !== "value" && Wn.type.kind === "value" && (Wn = new Pr(Nn.type, [Wn])));
                      let lr = null;
                      if (ln.length === 4) {
                        if (Nn.type.kind !== "string" && Wn.type.kind !== "string" && Nn.type.kind !== "value" && Wn.type.kind !== "value") return pn.error("Cannot use collator to compare non-string types.");
                        if (lr = pn.parse(ln[3], 3, vn), !lr) return null;
                      }
                      return new Jy(Nn, Wn, lr);
                    }
                    evaluate(ln) {
                      const pn = this.lhs.evaluate(ln), Cn = this.rhs.evaluate(ln);
                      if (xt && this.hasUntypedArgument) {
                        const Nn = ri(pn), Wn = ri(Cn);
                        if (Nn.kind !== Wn.kind || Nn.kind !== "string" && Nn.kind !== "number") throw new $i(`Expected arguments for "${Ye}" to be (string, string) or (number, number), but found (${Nn.kind}, ${Wn.kind}) instead.`);
                      }
                      if (this.collator && !xt && this.hasUntypedArgument) {
                        const Nn = ri(pn), Wn = ri(Cn);
                        if (Nn.kind !== "string" || Wn.kind !== "string") return He(ln, pn, Cn);
                      }
                      return this.collator ? ot(ln, pn, Cn, this.collator.evaluate(ln)) : He(ln, pn, Cn);
                    }
                    eachChild(ln) {
                      ln(this.lhs), ln(this.rhs), this.collator && ln(this.collator);
                    }
                    outputDefined() {
                      return !0;
                    }
                  };
                }
                const gr = Er("==", function(Ye, He, ot) {
                  return He === ot;
                }, Hr), $r = Er("!=", function(Ye, He, ot) {
                  return He !== ot;
                }, function(Ye, He, ot, xt) {
                  return !Hr(0, He, ot, xt);
                }), li = Er("<", function(Ye, He, ot) {
                  return He < ot;
                }, function(Ye, He, ot, xt) {
                  return xt.compare(He, ot) < 0;
                }), Qi = Er(">", function(Ye, He, ot) {
                  return He > ot;
                }, function(Ye, He, ot, xt) {
                  return xt.compare(He, ot) > 0;
                }), Ai = Er("<=", function(Ye, He, ot) {
                  return He <= ot;
                }, function(Ye, He, ot, xt) {
                  return xt.compare(He, ot) <= 0;
                }), Li = Er(">=", function(Ye, He, ot) {
                  return He >= ot;
                }, function(Ye, He, ot, xt) {
                  return xt.compare(He, ot) >= 0;
                });
                class Ni {
                  constructor(He, ot, xt) {
                    this.type = vn, this.locale = xt, this.caseSensitive = He, this.diacriticSensitive = ot;
                  }
                  static parse(He, ot) {
                    if (He.length !== 2) return ot.error("Expected one argument.");
                    const xt = He[1];
                    if (typeof xt != "object" || Array.isArray(xt)) return ot.error("Collator options argument must be an object.");
                    const zt = ot.parse(xt["case-sensitive"] !== void 0 && xt["case-sensitive"], 1, Qt);
                    if (!zt) return null;
                    const ln = ot.parse(xt["diacritic-sensitive"] !== void 0 && xt["diacritic-sensitive"], 1, Qt);
                    if (!ln) return null;
                    let pn = null;
                    return xt.locale && (pn = ot.parse(xt.locale, 1, hn), !pn) ? null : new Ni(zt, ln, pn);
                  }
                  evaluate(He) {
                    return new Ar(this.caseSensitive.evaluate(He), this.diacriticSensitive.evaluate(He), this.locale ? this.locale.evaluate(He) : null);
                  }
                  eachChild(He) {
                    He(this.caseSensitive), He(this.diacriticSensitive), this.locale && He(this.locale);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                class Pi {
                  constructor(He, ot, xt, zt, ln) {
                    this.type = hn, this.number = He, this.locale = ot, this.currency = xt, this.minFractionDigits = zt, this.maxFractionDigits = ln;
                  }
                  static parse(He, ot) {
                    if (He.length !== 3) return ot.error("Expected two arguments.");
                    const xt = ot.parse(He[1], 1, sn);
                    if (!xt) return null;
                    const zt = He[2];
                    if (typeof zt != "object" || Array.isArray(zt)) return ot.error("NumberFormat options argument must be an object.");
                    let ln = null;
                    if (zt.locale && (ln = ot.parse(zt.locale, 1, hn), !ln)) return null;
                    let pn = null;
                    if (zt.currency && (pn = ot.parse(zt.currency, 1, hn), !pn)) return null;
                    let Cn = null;
                    if (zt["min-fraction-digits"] && (Cn = ot.parse(zt["min-fraction-digits"], 1, sn), !Cn)) return null;
                    let Nn = null;
                    return zt["max-fraction-digits"] && (Nn = ot.parse(zt["max-fraction-digits"], 1, sn), !Nn) ? null : new Pi(xt, ln, pn, Cn, Nn);
                  }
                  evaluate(He) {
                    return new Intl.NumberFormat(this.locale ? this.locale.evaluate(He) : [], { style: this.currency ? "currency" : "decimal", currency: this.currency ? this.currency.evaluate(He) : void 0, minimumFractionDigits: this.minFractionDigits ? this.minFractionDigits.evaluate(He) : void 0, maximumFractionDigits: this.maxFractionDigits ? this.maxFractionDigits.evaluate(He) : void 0 }).format(this.number.evaluate(He));
                  }
                  eachChild(He) {
                    He(this.number), this.locale && He(this.locale), this.currency && He(this.currency), this.minFractionDigits && He(this.minFractionDigits), this.maxFractionDigits && He(this.maxFractionDigits);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                class Oi {
                  constructor(He) {
                    this.type = dn, this.sections = He;
                  }
                  static parse(He, ot) {
                    if (He.length < 2) return ot.error("Expected at least one argument.");
                    const xt = He[1];
                    if (!Array.isArray(xt) && typeof xt == "object") return ot.error("First argument must be an image or text section.");
                    const zt = [];
                    let ln = !1;
                    for (let pn = 1; pn <= He.length - 1; ++pn) {
                      const Cn = He[pn];
                      if (ln && typeof Cn == "object" && !Array.isArray(Cn)) {
                        ln = !1;
                        let Nn = null;
                        if (Cn["font-scale"] && (Nn = ot.parse(Cn["font-scale"], 1, sn), !Nn)) return null;
                        let Wn = null;
                        if (Cn["text-font"] && (Wn = ot.parse(Cn["text-font"], 1, wn(hn)), !Wn)) return null;
                        let lr = null;
                        if (Cn["text-color"] && (lr = ot.parse(Cn["text-color"], 1, Ut), !lr)) return null;
                        const xr = zt[zt.length - 1];
                        xr.scale = Nn, xr.font = Wn, xr.textColor = lr;
                      } else {
                        const Nn = ot.parse(He[pn], 1, rn);
                        if (!Nn) return null;
                        const Wn = Nn.type.kind;
                        if (Wn !== "string" && Wn !== "value" && Wn !== "null" && Wn !== "resolvedImage") return ot.error("Formatted text type must be 'string', 'value', 'image' or 'null'.");
                        ln = !0, zt.push({ content: Nn, scale: null, font: null, textColor: null });
                      }
                    }
                    return new Oi(zt);
                  }
                  evaluate(He) {
                    return new jr(this.sections.map((ot) => {
                      const xt = ot.content.evaluate(He);
                      return ri(xt) === jt ? new Yr("", xt, null, null, null) : new Yr(bi(xt), null, ot.scale ? ot.scale.evaluate(He) : null, ot.font ? ot.font.evaluate(He).join(",") : null, ot.textColor ? ot.textColor.evaluate(He) : null);
                    }));
                  }
                  eachChild(He) {
                    for (const ot of this.sections) He(ot.content), ot.scale && He(ot.scale), ot.font && He(ot.font), ot.textColor && He(ot.textColor);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                class ki {
                  constructor(He) {
                    this.type = jt, this.input = He;
                  }
                  static parse(He, ot) {
                    if (He.length !== 2) return ot.error("Expected two arguments.");
                    const xt = ot.parse(He[1], 1, hn);
                    return xt ? new ki(xt) : ot.error("No image name provided.");
                  }
                  evaluate(He) {
                    const ot = this.input.evaluate(He), xt = gi.fromString(ot);
                    return xt && He.availableImages && (xt.available = He.availableImages.indexOf(ot) > -1), xt;
                  }
                  eachChild(He) {
                    He(this.input);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                class Mi {
                  constructor(He) {
                    this.type = sn, this.input = He;
                  }
                  static parse(He, ot) {
                    if (He.length !== 2) return ot.error(`Expected 1 argument, but found ${He.length - 1} instead.`);
                    const xt = ot.parse(He[1], 1);
                    return xt ? xt.type.kind !== "array" && xt.type.kind !== "string" && xt.type.kind !== "value" ? ot.error(`Expected argument of type string or array, but found ${Rn(xt.type)} instead.`) : new Mi(xt) : null;
                  }
                  evaluate(He) {
                    const ot = this.input.evaluate(He);
                    if (typeof ot == "string" || Array.isArray(ot)) return ot.length;
                    throw new $i(`Expected value to be of type string or array, but found ${Rn(ri(ot))} instead.`);
                  }
                  eachChild(He) {
                    He(this.input);
                  }
                  outputDefined() {
                    return !1;
                  }
                }
                const Vi = 8192;
                function ua(Ye, He) {
                  const ot = (180 + Ye[0]) / 360, xt = (ln = Ye[1], (180 - 180 / Math.PI * Math.log(Math.tan(Math.PI / 4 + ln * Math.PI / 360))) / 360), zt = Math.pow(2, He.z);
                  var ln;
                  return [Math.round(ot * zt * Vi), Math.round(xt * zt * Vi)];
                }
                function Ki(Ye, He) {
                  const ot = Math.pow(2, He.z), xt = (Ye[0] / Vi + He.x) / ot, zt = (Ye[1] / Vi + He.y) / ot;
                  return [(pn = xt, 360 * pn - 180), (ln = zt, 360 / Math.PI * Math.atan(Math.exp((180 - 360 * ln) * Math.PI / 180)) - 90)];
                  var ln, pn;
                }
                function Fi(Ye, He) {
                  Ye[0] = Math.min(Ye[0], He[0]), Ye[1] = Math.min(Ye[1], He[1]), Ye[2] = Math.max(Ye[2], He[0]), Ye[3] = Math.max(Ye[3], He[1]);
                }
                function na(Ye, He) {
                  return !(Ye[0] <= He[0] || Ye[2] >= He[2] || Ye[1] <= He[1] || Ye[3] >= He[3]);
                }
                function nn(Ye, He, ot) {
                  const xt = Ye[0] - He[0], zt = Ye[1] - He[1], ln = Ye[0] - ot[0], pn = Ye[1] - ot[1];
                  return xt * pn - ln * zt == 0 && xt * ln <= 0 && zt * pn <= 0;
                }
                function Tn(Ye, He, ot, xt) {
                  const zt = [He[0] - Ye[0], He[1] - Ye[1]];
                  return ln = [xt[0] - ot[0], xt[1] - ot[1]], pn = zt, ln[0] * pn[1] - ln[1] * pn[0] != 0 && !(!ai(Ye, He, ot, xt) || !ai(ot, xt, Ye, He));
                  var ln, pn;
                }
                function Dn(Ye, He, ot) {
                  for (const xt of ot) for (let zt = 0; zt < xt.length - 1; ++zt) if (Tn(Ye, He, xt[zt], xt[zt + 1])) return !0;
                  return !1;
                }
                function Fn(Ye, He, ot = !1) {
                  let xt = !1;
                  for (const Cn of He) for (let Nn = 0; Nn < Cn.length - 1; Nn++) {
                    if (nn(Ye, Cn[Nn], Cn[Nn + 1])) return ot;
                    zt = Ye, ln = Cn[Nn], pn = Cn[Nn + 1], ln[1] > zt[1] != pn[1] > zt[1] && zt[0] < (pn[0] - ln[0]) * (zt[1] - ln[1]) / (pn[1] - ln[1]) + ln[0] && (xt = !xt);
                  }
                  var zt, ln, pn;
                  return xt;
                }
                function fr(Ye, He) {
                  for (const ot of He) if (Fn(Ye, ot)) return !0;
                  return !1;
                }
                function dr(Ye, He) {
                  for (const ot of Ye) if (!Fn(ot, He)) return !1;
                  for (let ot = 0; ot < Ye.length - 1; ++ot) if (Dn(Ye[ot], Ye[ot + 1], He)) return !1;
                  return !0;
                }
                function yr(Ye, He) {
                  for (const ot of He) if (dr(Ye, ot)) return !0;
                  return !1;
                }
                function ai(Ye, He, ot, xt) {
                  const zt = Ye[0] - ot[0], ln = Ye[1] - ot[1], pn = He[0] - ot[0], Cn = He[1] - ot[1], Nn = xt[0] - ot[0], Wn = xt[1] - ot[1], lr = zt * Wn - Nn * ln, xr = pn * Wn - Nn * Cn;
                  return lr > 0 && xr < 0 || lr < 0 && xr > 0;
                }
                function Fr(Ye, He, ot) {
                  const xt = [];
                  for (let zt = 0; zt < Ye.length; zt++) {
                    const ln = [];
                    for (let pn = 0; pn < Ye[zt].length; pn++) {
                      const Cn = ua(Ye[zt][pn], ot);
                      Fi(He, Cn), ln.push(Cn);
                    }
                    xt.push(ln);
                  }
                  return xt;
                }
                function Ti(Ye, He, ot) {
                  const xt = [];
                  for (let zt = 0; zt < Ye.length; zt++) {
                    const ln = Fr(Ye[zt], He, ot);
                    xt.push(ln);
                  }
                  return xt;
                }
                function Si(Ye, He, ot, xt) {
                  if (Ye[0] < ot[0] || Ye[0] > ot[2]) {
                    const zt = 0.5 * xt;
                    let ln = Ye[0] - ot[0] > zt ? -xt : ot[0] - Ye[0] > zt ? xt : 0;
                    ln === 0 && (ln = Ye[0] - ot[2] > zt ? -xt : ot[2] - Ye[0] > zt ? xt : 0), Ye[0] += ln;
                  }
                  Fi(He, Ye);
                }
                function Ei(Ye, He, ot, xt) {
                  const zt = Math.pow(2, xt.z) * Vi, ln = [xt.x * Vi, xt.y * Vi], pn = [];
                  for (const Cn of Ye) for (const Nn of Cn) {
                    const Wn = [Nn.x + ln[0], Nn.y + ln[1]];
                    Si(Wn, He, ot, zt), pn.push(Wn);
                  }
                  return pn;
                }
                function Fa(Ye, He, ot, xt) {
                  const zt = Math.pow(2, xt.z) * Vi, ln = [xt.x * Vi, xt.y * Vi], pn = [];
                  for (const Nn of Ye) {
                    const Wn = [];
                    for (const lr of Nn) {
                      const xr = [lr.x + ln[0], lr.y + ln[1]];
                      Fi(He, xr), Wn.push(xr);
                    }
                    pn.push(Wn);
                  }
                  if (He[2] - He[0] <= zt / 2) {
                    (Cn = He)[0] = Cn[1] = 1 / 0, Cn[2] = Cn[3] = -1 / 0;
                    for (const Nn of pn) for (const Wn of Nn) Si(Wn, He, ot, zt);
                  }
                  var Cn;
                  return pn;
                }
                class $o {
                  constructor(He, ot) {
                    this.type = Qt, this.geojson = He, this.geometries = ot;
                  }
                  static parse(He, ot) {
                    if (He.length !== 2) return ot.error(`'within' expression requires exactly one argument, but found ${He.length - 1} instead.`);
                    if (fi(He[1])) {
                      const xt = He[1];
                      if (xt.type === "FeatureCollection") {
                        const zt = [];
                        for (const ln of xt.features) {
                          const { type: pn, coordinates: Cn } = ln.geometry;
                          pn === "Polygon" && zt.push(Cn), pn === "MultiPolygon" && zt.push(...Cn);
                        }
                        if (zt.length) return new $o(xt, { type: "MultiPolygon", coordinates: zt });
                      } else if (xt.type === "Feature") {
                        const zt = xt.geometry.type;
                        if (zt === "Polygon" || zt === "MultiPolygon") return new $o(xt, xt.geometry);
                      } else if (xt.type === "Polygon" || xt.type === "MultiPolygon") return new $o(xt, xt);
                    }
                    return ot.error("'within' expression requires valid geojson object that contains polygon geometry type.");
                  }
                  evaluate(He) {
                    if (He.geometry() != null && He.canonicalID() != null) {
                      if (He.geometryType() === "Point") return function(ot, xt) {
                        const zt = [1 / 0, 1 / 0, -1 / 0, -1 / 0], ln = [1 / 0, 1 / 0, -1 / 0, -1 / 0], pn = ot.canonicalID();
                        if (xt.type === "Polygon") {
                          const Cn = Fr(xt.coordinates, ln, pn), Nn = Ei(ot.geometry(), zt, ln, pn);
                          if (!na(zt, ln)) return !1;
                          for (const Wn of Nn) if (!Fn(Wn, Cn)) return !1;
                        }
                        if (xt.type === "MultiPolygon") {
                          const Cn = Ti(xt.coordinates, ln, pn), Nn = Ei(ot.geometry(), zt, ln, pn);
                          if (!na(zt, ln)) return !1;
                          for (const Wn of Nn) if (!fr(Wn, Cn)) return !1;
                        }
                        return !0;
                      }(He, this.geometries);
                      if (He.geometryType() === "LineString") return function(ot, xt) {
                        const zt = [1 / 0, 1 / 0, -1 / 0, -1 / 0], ln = [1 / 0, 1 / 0, -1 / 0, -1 / 0], pn = ot.canonicalID();
                        if (xt.type === "Polygon") {
                          const Cn = Fr(xt.coordinates, ln, pn), Nn = Fa(ot.geometry(), zt, ln, pn);
                          if (!na(zt, ln)) return !1;
                          for (const Wn of Nn) if (!dr(Wn, Cn)) return !1;
                        }
                        if (xt.type === "MultiPolygon") {
                          const Cn = Ti(xt.coordinates, ln, pn), Nn = Fa(ot.geometry(), zt, ln, pn);
                          if (!na(zt, ln)) return !1;
                          for (const Wn of Nn) if (!yr(Wn, Cn)) return !1;
                        }
                        return !0;
                      }(He, this.geometries);
                    }
                    return !1;
                  }
                  eachChild() {
                  }
                  outputDefined() {
                    return !0;
                  }
                }
                let Gs = class {
                  constructor(Ye = [], He = Al) {
                    if (this.data = Ye, this.length = this.data.length, this.compare = He, this.length > 0) for (let ot = (this.length >> 1) - 1; ot >= 0; ot--) this._down(ot);
                  }
                  push(Ye) {
                    this.data.push(Ye), this.length++, this._up(this.length - 1);
                  }
                  pop() {
                    if (this.length === 0) return;
                    const Ye = this.data[0], He = this.data.pop();
                    return this.length--, this.length > 0 && (this.data[0] = He, this._down(0)), Ye;
                  }
                  peek() {
                    return this.data[0];
                  }
                  _up(Ye) {
                    const { data: He, compare: ot } = this, xt = He[Ye];
                    for (; Ye > 0; ) {
                      const zt = Ye - 1 >> 1, ln = He[zt];
                      if (ot(xt, ln) >= 0) break;
                      He[Ye] = ln, Ye = zt;
                    }
                    He[Ye] = xt;
                  }
                  _down(Ye) {
                    const { data: He, compare: ot } = this, xt = this.length >> 1, zt = He[Ye];
                    for (; Ye < xt; ) {
                      let ln = 1 + (Ye << 1), pn = He[ln];
                      const Cn = ln + 1;
                      if (Cn < this.length && ot(He[Cn], pn) < 0 && (ln = Cn, pn = He[Cn]), ot(pn, zt) >= 0) break;
                      He[Ye] = pn, Ye = ln;
                    }
                    He[Ye] = zt;
                  }
                };
                function Al(Ye, He) {
                  return Ye < He ? -1 : Ye > He ? 1 : 0;
                }
                function ul(Ye, He, ot, xt, zt) {
                  gc(Ye, He, ot, xt || Ye.length - 1, zt);
                }
                function gc(Ye, He, ot, xt, zt) {
                  for (; xt > ot; ) {
                    if (xt - ot > 600) {
                      var ln = xt - ot + 1, pn = He - ot + 1, Cn = Math.log(ln), Nn = 0.5 * Math.exp(2 * Cn / 3), Wn = 0.5 * Math.sqrt(Cn * Nn * (ln - Nn) / ln) * (pn - ln / 2 < 0 ? -1 : 1);
                      gc(Ye, He, Math.max(ot, Math.floor(He - pn * Nn / ln + Wn)), Math.min(xt, Math.floor(He + (ln - pn) * Nn / ln + Wn)), zt);
                    }
                    var lr = Ye[He], xr = ot, Nr = xt;
                    for (Gl(Ye, ot, He), zt(Ye[xt], lr) > 0 && Gl(Ye, ot, xt); xr < Nr; ) {
                      for (Gl(Ye, xr, Nr), xr++, Nr--; zt(Ye[xr], lr) < 0; ) xr++;
                      for (; zt(Ye[Nr], lr) > 0; ) Nr--;
                    }
                    zt(Ye[ot], lr) === 0 ? Gl(Ye, ot, Nr) : Gl(Ye, ++Nr, xt), Nr <= He && (ot = Nr + 1), He <= Nr && (xt = Nr - 1);
                  }
                }
                function Gl(Ye, He, ot) {
                  var xt = Ye[He];
                  Ye[He] = Ye[ot], Ye[ot] = xt;
                }
                function Sl(Ye, He) {
                  if (Ye.length <= 1) return [Ye];
                  const ot = [];
                  let xt, zt;
                  for (const ln of Ye) {
                    const pn = vs(ln);
                    pn !== 0 && (ln.area = Math.abs(pn), zt === void 0 && (zt = pn < 0), zt === pn < 0 ? (xt && ot.push(xt), xt = [ln]) : xt.push(ln));
                  }
                  if (xt && ot.push(xt), He > 1) for (let ln = 0; ln < ot.length; ln++) ot[ln].length <= He || (ul(ot[ln], He, 1, ot[ln].length - 1, ls), ot[ln] = ot[ln].slice(0, He));
                  return ot;
                }
                function ls(Ye, He) {
                  return He.area - Ye.area;
                }
                function vs(Ye) {
                  let He = 0;
                  for (let ot, xt, zt = 0, ln = Ye.length, pn = ln - 1; zt < ln; pn = zt++) ot = Ye[zt], xt = Ye[pn], He += (xt.x - ot.x) * (ot.y + xt.y);
                  return He;
                }
                const Vl = 1 / 298.257223563, Ll = Vl * (2 - Vl), Xl = Math.PI / 180;
                class yc {
                  constructor(He) {
                    const ot = 6378.137 * Xl * 1e3, xt = Math.cos(He * Xl), zt = 1 / (1 - Ll * (1 - xt * xt)), ln = Math.sqrt(zt);
                    this.kx = ot * ln * xt, this.ky = ot * ln * zt * (1 - Ll);
                  }
                  distance(He, ot) {
                    const xt = this.wrap(He[0] - ot[0]) * this.kx, zt = (He[1] - ot[1]) * this.ky;
                    return Math.sqrt(xt * xt + zt * zt);
                  }
                  pointOnLine(He, ot) {
                    let xt, zt, ln, pn, Cn = 1 / 0;
                    for (let Nn = 0; Nn < He.length - 1; Nn++) {
                      let Wn = He[Nn][0], lr = He[Nn][1], xr = this.wrap(He[Nn + 1][0] - Wn) * this.kx, Nr = (He[Nn + 1][1] - lr) * this.ky, Kr = 0;
                      xr === 0 && Nr === 0 || (Kr = (this.wrap(ot[0] - Wn) * this.kx * xr + (ot[1] - lr) * this.ky * Nr) / (xr * xr + Nr * Nr), Kr > 1 ? (Wn = He[Nn + 1][0], lr = He[Nn + 1][1]) : Kr > 0 && (Wn += xr / this.kx * Kr, lr += Nr / this.ky * Kr)), xr = this.wrap(ot[0] - Wn) * this.kx, Nr = (ot[1] - lr) * this.ky;
                      const xi = xr * xr + Nr * Nr;
                      xi < Cn && (Cn = xi, xt = Wn, zt = lr, ln = Nn, pn = Kr);
                    }
                    return { point: [xt, zt], index: ln, t: Math.max(0, Math.min(1, pn)) };
                  }
                  wrap(He) {
                    for (; He < -180; ) He += 360;
                    for (; He > 180; ) He -= 360;
                    return He;
                  }
                }
                const gu = 100, Nc = 50;
                function Cu(Ye, He) {
                  return He[0] - Ye[0];
                }
                function zc(Ye) {
                  return Ye[1] - Ye[0] + 1;
                }
                function Kc(Ye, He) {
                  return Ye[1] >= Ye[0] && Ye[1] < He;
                }
                function zi(Ye, He) {
                  if (Ye[0] > Ye[1]) return [null, null];
                  const ot = zc(Ye);
                  if (He) {
                    if (ot === 2) return [Ye, null];
                    const zt = Math.floor(ot / 2);
                    return [[Ye[0], Ye[0] + zt], [Ye[0] + zt, Ye[1]]];
                  }
                  if (ot === 1) return [Ye, null];
                  const xt = Math.floor(ot / 2) - 1;
                  return [[Ye[0], Ye[0] + xt], [Ye[0] + xt + 1, Ye[1]]];
                }
                function Fu(Ye, He) {
                  if (!Kc(He, Ye.length)) return [1 / 0, 1 / 0, -1 / 0, -1 / 0];
                  const ot = [1 / 0, 1 / 0, -1 / 0, -1 / 0];
                  for (let xt = He[0]; xt <= He[1]; ++xt) Fi(ot, Ye[xt]);
                  return ot;
                }
                function d1(Ye) {
                  const He = [1 / 0, 1 / 0, -1 / 0, -1 / 0];
                  for (const ot of Ye) for (const xt of ot) Fi(He, xt);
                  return He;
                }
                function Gu(Ye) {
                  return Ye[0] !== -1 / 0 && Ye[1] !== -1 / 0 && Ye[2] !== 1 / 0 && Ye[3] !== 1 / 0;
                }
                function Ku(Ye, He, ot) {
                  if (!Gu(Ye) || !Gu(He)) return NaN;
                  let xt = 0, zt = 0;
                  return Ye[2] < He[0] && (xt = He[0] - Ye[2]), Ye[0] > He[2] && (xt = Ye[0] - He[2]), Ye[1] > He[3] && (zt = Ye[1] - He[3]), Ye[3] < He[1] && (zt = He[1] - Ye[3]), ot.distance([0, 0], [xt, zt]);
                }
                function I1(Ye, He, ot) {
                  const xt = ot.pointOnLine(He, Ye);
                  return ot.distance(Ye, xt.point);
                }
                function f1(Ye, He, ot, xt, zt) {
                  const ln = Math.min(I1(Ye, [ot, xt], zt), I1(He, [ot, xt], zt)), pn = Math.min(I1(ot, [Ye, He], zt), I1(xt, [Ye, He], zt));
                  return Math.min(ln, pn);
                }
                function D1(Ye, He, ot, xt, zt) {
                  if (!Kc(He, Ye.length) || !Kc(xt, ot.length)) return 1 / 0;
                  let ln = 1 / 0;
                  for (let pn = He[0]; pn < He[1]; ++pn) {
                    const Cn = Ye[pn], Nn = Ye[pn + 1];
                    for (let Wn = xt[0]; Wn < xt[1]; ++Wn) {
                      const lr = ot[Wn], xr = ot[Wn + 1];
                      if (Tn(Cn, Nn, lr, xr)) return 0;
                      ln = Math.min(ln, f1(Cn, Nn, lr, xr, zt));
                    }
                  }
                  return ln;
                }
                function Hl(Ye, He, ot, xt, zt) {
                  if (!Kc(He, Ye.length) || !Kc(xt, ot.length)) return NaN;
                  let ln = 1 / 0;
                  for (let pn = He[0]; pn <= He[1]; ++pn) for (let Cn = xt[0]; Cn <= xt[1]; ++Cn) if (ln = Math.min(ln, zt.distance(Ye[pn], ot[Cn])), ln === 0) return ln;
                  return ln;
                }
                function Kl(Ye, He, ot) {
                  if (Fn(Ye, He, !0)) return 0;
                  let xt = 1 / 0;
                  for (const zt of He) {
                    const ln = zt[0], pn = zt[zt.length - 1];
                    if (ln !== pn && (xt = Math.min(xt, I1(Ye, [pn, ln], ot)), xt === 0)) return xt;
                    const Cn = ot.pointOnLine(zt, Ye);
                    if (xt = Math.min(xt, ot.distance(Ye, Cn.point)), xt === 0) return xt;
                  }
                  return xt;
                }
                function uu(Ye, He, ot, xt) {
                  if (!Kc(He, Ye.length)) return NaN;
                  for (let ln = He[0]; ln <= He[1]; ++ln) if (Fn(Ye[ln], ot, !0)) return 0;
                  let zt = 1 / 0;
                  for (let ln = He[0]; ln < He[1]; ++ln) {
                    const pn = Ye[ln], Cn = Ye[ln + 1];
                    for (const Nn of ot) for (let Wn = 0, lr = Nn.length, xr = lr - 1; Wn < lr; xr = Wn++) {
                      const Nr = Nn[xr], Kr = Nn[Wn];
                      if (Tn(pn, Cn, Nr, Kr)) return 0;
                      zt = Math.min(zt, f1(pn, Cn, Nr, Kr, xt));
                    }
                  }
                  return zt;
                }
                function Hs(Ye, He) {
                  for (const ot of Ye) for (const xt of ot) if (Fn(xt, He, !0)) return !0;
                  return !1;
                }
                function b1(Ye, He, ot, xt = 1 / 0) {
                  const zt = d1(Ye), ln = d1(He);
                  if (xt !== 1 / 0 && Ku(zt, ln, ot) >= xt) return xt;
                  if (na(zt, ln)) {
                    if (Hs(Ye, He)) return 0;
                  } else if (Hs(He, Ye)) return 0;
                  let pn = 1 / 0;
                  for (const Cn of Ye) for (let Nn = 0, Wn = Cn.length, lr = Wn - 1; Nn < Wn; lr = Nn++) {
                    const xr = Cn[lr], Nr = Cn[Nn];
                    for (const Kr of He) for (let xi = 0, ui = Kr.length, Ui = ui - 1; xi < ui; Ui = xi++) {
                      const ia = Kr[Ui], Na = Kr[xi];
                      if (Tn(xr, Nr, ia, Na)) return 0;
                      pn = Math.min(pn, f1(xr, Nr, ia, Na, ot));
                    }
                  }
                  return pn;
                }
                function Wl(Ye, He, ot, xt, zt, ln) {
                  if (!ln) return;
                  const pn = Ku(Fu(xt, ln), zt, ot);
                  pn < He && Ye.push([pn, ln, [0, 0]]);
                }
                function zl(Ye, He, ot, xt, zt, ln, pn) {
                  if (!ln || !pn) return;
                  const Cn = Ku(Fu(xt, ln), Fu(zt, pn), ot);
                  Cn < He && Ye.push([Cn, ln, pn]);
                }
                function os(Ye, He, ot, xt, zt = 1 / 0) {
                  let ln = Math.min(xt.distance(Ye[0], ot[0][0]), zt);
                  if (ln === 0) return ln;
                  const pn = new Gs([[0, [0, Ye.length - 1], [0, 0]]], Cu), Cn = d1(ot);
                  for (; pn.length > 0; ) {
                    const Nn = pn.pop();
                    if (Nn[0] >= ln) continue;
                    const Wn = Nn[1], lr = He ? Nc : gu;
                    if (zc(Wn) <= lr) {
                      if (!Kc(Wn, Ye.length)) return NaN;
                      if (He) {
                        const xr = uu(Ye, Wn, ot, xt);
                        if (isNaN(xr) || xr === 0) return xr;
                        ln = Math.min(ln, xr);
                      } else for (let xr = Wn[0]; xr <= Wn[1]; ++xr) {
                        const Nr = Kl(Ye[xr], ot, xt);
                        if (ln = Math.min(ln, Nr), ln === 0) return 0;
                      }
                    } else {
                      const xr = zi(Wn, He);
                      Wl(pn, ln, xt, Ye, Cn, xr[0]), Wl(pn, ln, xt, Ye, Cn, xr[1]);
                    }
                  }
                  return ln;
                }
                function jo(Ye, He, ot, xt, zt, ln = 1 / 0) {
                  let pn = Math.min(ln, zt.distance(Ye[0], ot[0]));
                  if (pn === 0) return pn;
                  const Cn = new Gs([[0, [0, Ye.length - 1], [0, ot.length - 1]]], Cu);
                  for (; Cn.length > 0; ) {
                    const Nn = Cn.pop();
                    if (Nn[0] >= pn) continue;
                    const Wn = Nn[1], lr = Nn[2], xr = He ? Nc : gu, Nr = xt ? Nc : gu;
                    if (zc(Wn) <= xr && zc(lr) <= Nr) {
                      if (!Kc(Wn, Ye.length) && Kc(lr, ot.length)) return NaN;
                      let Kr;
                      if (He && xt) Kr = D1(Ye, Wn, ot, lr, zt), pn = Math.min(pn, Kr);
                      else if (He && !xt) {
                        const xi = Ye.slice(Wn[0], Wn[1] + 1);
                        for (let ui = lr[0]; ui <= lr[1]; ++ui) if (Kr = I1(ot[ui], xi, zt), pn = Math.min(pn, Kr), pn === 0) return pn;
                      } else if (!He && xt) {
                        const xi = ot.slice(lr[0], lr[1] + 1);
                        for (let ui = Wn[0]; ui <= Wn[1]; ++ui) if (Kr = I1(Ye[ui], xi, zt), pn = Math.min(pn, Kr), pn === 0) return pn;
                      } else Kr = Hl(Ye, Wn, ot, lr, zt), pn = Math.min(pn, Kr);
                    } else {
                      const Kr = zi(Wn, He), xi = zi(lr, xt);
                      zl(Cn, pn, zt, Ye, ot, Kr[0], xi[0]), zl(Cn, pn, zt, Ye, ot, Kr[0], xi[1]), zl(Cn, pn, zt, Ye, ot, Kr[1], xi[0]), zl(Cn, pn, zt, Ye, ot, Kr[1], xi[1]);
                    }
                  }
                  return pn;
                }
                function uo(Ye) {
                  return Ye.type === "MultiPolygon" ? Ye.coordinates.map((He) => ({ type: "Polygon", coordinates: He })) : Ye.type === "MultiLineString" ? Ye.coordinates.map((He) => ({ type: "LineString", coordinates: He })) : Ye.type === "MultiPoint" ? Ye.coordinates.map((He) => ({ type: "Point", coordinates: He })) : [Ye];
                }
                class so {
                  constructor(He, ot) {
                    this.type = sn, this.geojson = He, this.geometries = ot;
                  }
                  static parse(He, ot) {
                    if (He.length !== 2) return ot.error(`'distance' expression requires exactly one argument, but found ${He.length - 1} instead.`);
                    if (fi(He[1])) {
                      const xt = He[1];
                      if (xt.type === "FeatureCollection") return new so(xt, xt.features.map((zt) => uo(zt.geometry)).flat());
                      if (xt.type === "Feature") return new so(xt, uo(xt.geometry));
                      if ("type" in xt && "coordinates" in xt) return new so(xt, uo(xt));
                    }
                    return ot.error("'distance' expression requires valid geojson object that contains polygon geometry type.");
                  }
                  evaluate(He) {
                    if (He.geometry() != null && He.canonicalID() != null) {
                      if (He.geometryType() === "Point") return function(ot, xt) {
                        const zt = ot.geometry(), ln = zt.flat().map((Nn) => Ki([Nn.x, Nn.y], ot.canonical));
                        if (zt.length === 0) return NaN;
                        const pn = new yc(ln[0][1]);
                        let Cn = 1 / 0;
                        for (const Nn of xt) {
                          switch (Nn.type) {
                            case "Point":
                              Cn = Math.min(Cn, jo(ln, !1, [Nn.coordinates], !1, pn, Cn));
                              break;
                            case "LineString":
                              Cn = Math.min(Cn, jo(ln, !1, Nn.coordinates, !0, pn, Cn));
                              break;
                            case "Polygon":
                              Cn = Math.min(Cn, os(ln, !1, Nn.coordinates, pn, Cn));
                          }
                          if (Cn === 0) return Cn;
                        }
                        return Cn;
                      }(He, this.geometries);
                      if (He.geometryType() === "LineString") return function(ot, xt) {
                        const zt = ot.geometry(), ln = zt.flat().map((Nn) => Ki([Nn.x, Nn.y], ot.canonical));
                        if (zt.length === 0) return NaN;
                        const pn = new yc(ln[0][1]);
                        let Cn = 1 / 0;
                        for (const Nn of xt) {
                          switch (Nn.type) {
                            case "Point":
                              Cn = Math.min(Cn, jo(ln, !0, [Nn.coordinates], !1, pn, Cn));
                              break;
                            case "LineString":
                              Cn = Math.min(Cn, jo(ln, !0, Nn.coordinates, !0, pn, Cn));
                              break;
                            case "Polygon":
                              Cn = Math.min(Cn, os(ln, !0, Nn.coordinates, pn, Cn));
                          }
                          if (Cn === 0) return Cn;
                        }
                        return Cn;
                      }(He, this.geometries);
                      if (He.geometryType() === "Polygon") return function(ot, xt) {
                        const zt = ot.geometry();
                        if (zt.length === 0 || zt[0].length === 0) return NaN;
                        const ln = Sl(zt, 0).map((Nn) => Nn.map((Wn) => Wn.map((lr) => Ki([lr.x, lr.y], ot.canonical)))), pn = new yc(ln[0][0][0][1]);
                        let Cn = 1 / 0;
                        for (const Nn of xt) for (const Wn of ln) {
                          switch (Nn.type) {
                            case "Point":
                              Cn = Math.min(Cn, os([Nn.coordinates], !1, Wn, pn, Cn));
                              break;
                            case "LineString":
                              Cn = Math.min(Cn, os(Nn.coordinates, !0, Wn, pn, Cn));
                              break;
                            case "Polygon":
                              Cn = Math.min(Cn, b1(Wn, Nn.coordinates, pn, Cn));
                          }
                          if (Cn === 0) return Cn;
                        }
                        return Cn;
                      }(He, this.geometries);
                    }
                    return NaN;
                  }
                  eachChild() {
                  }
                  outputDefined() {
                    return !0;
                  }
                }
                const So = { "==": gr, "!=": $r, ">": Qi, "<": li, ">=": Li, "<=": Ai, array: Pr, at: Ca, boolean: Pr, case: zr, coalesce: Jr, collator: Ni, format: Oi, image: ki, in: Sr, "index-of": Dr, interpolate: Vr, "interpolate-hcl": Vr, "interpolate-lab": Vr, length: Mi, let: hi, literal: Zi, match: si, number: Pr, "number-format": Pi, object: Pr, slice: di, step: la, string: Pr, "to-boolean": Ci, "to-color": Ci, "to-number": Ci, "to-string": Ci, var: Gr, within: $o, distance: so };
                class Jc {
                  constructor(He, ot, xt, zt) {
                    this.name = He, this.type = ot, this._evaluate = xt, this.args = zt;
                  }
                  evaluate(He) {
                    return this._evaluate(He, this.args);
                  }
                  eachChild(He) {
                    this.args.forEach(He);
                  }
                  outputDefined() {
                    return !1;
                  }
                  static parse(He, ot) {
                    const xt = He[0], zt = Jc.definitions[xt];
                    if (!zt) return ot.error(`Unknown expression "${xt}". If you wanted a literal array, use ["literal", [...]].`, 0);
                    const ln = Array.isArray(zt) ? zt[0] : zt.type, pn = Array.isArray(zt) ? [[zt[1], zt[2]]] : zt.overloads, Cn = pn.filter(([Wn]) => !Array.isArray(Wn) || Wn.length === He.length - 1);
                    let Nn = null;
                    for (const [Wn, lr] of Cn) {
                      Nn = new aa(ot.registry, ws, ot.path, null, ot.scope);
                      const xr = [];
                      let Nr = !1;
                      for (let Kr = 1; Kr < He.length; Kr++) {
                        const xi = He[Kr], ui = Array.isArray(Wn) ? Wn[Kr - 1] : Wn.type, Ui = Nn.parse(xi, 1 + xr.length, ui);
                        if (!Ui) {
                          Nr = !0;
                          break;
                        }
                        xr.push(Ui);
                      }
                      if (!Nr) if (Array.isArray(Wn) && Wn.length !== xr.length) Nn.error(`Expected ${Wn.length} arguments, but found ${xr.length} instead.`);
                      else {
                        for (let Kr = 0; Kr < xr.length; Kr++) {
                          const xi = Array.isArray(Wn) ? Wn[Kr] : Wn.type, ui = xr[Kr];
                          Nn.concat(Kr + 1).checkSubtype(xi, ui.type);
                        }
                        if (Nn.errors.length === 0) return new Jc(xt, ln, lr, xr);
                      }
                    }
                    if (Cn.length === 1) ot.errors.push(...Nn.errors);
                    else {
                      const Wn = (Cn.length ? Cn : pn).map(([xr]) => {
                        return Nr = xr, Array.isArray(Nr) ? `(${Nr.map(Rn).join(", ")})` : `(${Rn(Nr.type)}...)`;
                        var Nr;
                      }).join(" | "), lr = [];
                      for (let xr = 1; xr < He.length; xr++) {
                        const Nr = ot.parse(He[xr], 1 + lr.length);
                        if (!Nr) return null;
                        lr.push(Rn(Nr.type));
                      }
                      ot.error(`Expected arguments of type ${Wn}, but found (${lr.join(", ")}) instead.`);
                    }
                    return null;
                  }
                  static register(He, ot) {
                    Jc.definitions = ot;
                    for (const xt in ot) He[xt] = Jc;
                  }
                }
                function No(Ye, [He, ot, xt, zt]) {
                  He = He.evaluate(Ye), ot = ot.evaluate(Ye), xt = xt.evaluate(Ye);
                  const ln = zt ? zt.evaluate(Ye) : 1, pn = Zr(He, ot, xt, ln);
                  if (pn) throw new $i(pn);
                  return new mr(He / 255, ot / 255, xt / 255, ln, !1);
                }
                function ro(Ye, He) {
                  return Ye in He;
                }
                function Io(Ye, He) {
                  const ot = He[Ye];
                  return ot === void 0 ? null : ot;
                }
                function bs(Ye) {
                  return { type: Ye };
                }
                function ws(Ye) {
                  if (Ye instanceof Gr) return ws(Ye.boundExpression);
                  if (Ye instanceof Jc && Ye.name === "error" || Ye instanceof Ni || Ye instanceof $o || Ye instanceof so) return !1;
                  const He = Ye instanceof Ci || Ye instanceof Pr;
                  let ot = !0;
                  return Ye.eachChild((xt) => {
                    ot = He ? ot && ws(xt) : ot && xt instanceof Zi;
                  }), !!ot && zo(Ye) && Ls(Ye, ["zoom", "heatmap-density", "line-progress", "accumulated", "is-supported-script"]);
                }
                function zo(Ye) {
                  if (Ye instanceof Jc && (Ye.name === "get" && Ye.args.length === 1 || Ye.name === "feature-state" || Ye.name === "has" && Ye.args.length === 1 || Ye.name === "properties" || Ye.name === "geometry-type" || Ye.name === "id" || /^filter-/.test(Ye.name)) || Ye instanceof $o || Ye instanceof so) return !1;
                  let He = !0;
                  return Ye.eachChild((ot) => {
                    He && !zo(ot) && (He = !1);
                  }), He;
                }
                function Yo(Ye) {
                  if (Ye instanceof Jc && Ye.name === "feature-state") return !1;
                  let He = !0;
                  return Ye.eachChild((ot) => {
                    He && !Yo(ot) && (He = !1);
                  }), He;
                }
                function Ls(Ye, He) {
                  if (Ye instanceof Jc && He.indexOf(Ye.name) >= 0) return !1;
                  let ot = !0;
                  return Ye.eachChild((xt) => {
                    ot && !Ls(xt, He) && (ot = !1);
                  }), ot;
                }
                function Ms(Ye) {
                  return { result: "success", value: Ye };
                }
                function vi(Ye) {
                  return { result: "error", value: Ye };
                }
                function Ri(Ye) {
                  return Ye["property-type"] === "data-driven" || Ye["property-type"] === "cross-faded-data-driven";
                }
                function ta(Ye) {
                  return !!Ye.expression && Ye.expression.parameters.indexOf("zoom") > -1;
                }
                function da(Ye) {
                  return !!Ye.expression && Ye.expression.interpolated;
                }
                function Ji(Ye) {
                  return Ye instanceof Number ? "number" : Ye instanceof String ? "string" : Ye instanceof Boolean ? "boolean" : Array.isArray(Ye) ? "array" : Ye === null ? "null" : typeof Ye;
                }
                function Pa(Ye) {
                  return typeof Ye == "object" && Ye !== null && !Array.isArray(Ye);
                }
                function Ga(Ye) {
                  return Ye;
                }
                function Ha(Ye, He) {
                  const ot = He.type === "color", xt = Ye.stops && typeof Ye.stops[0][0] == "object", zt = xt || Ye.property !== void 0, ln = xt || !zt, pn = Ye.type || (da(He) ? "exponential" : "interval");
                  if (ot || He.type === "padding") {
                    const xr = ot ? mr.parse : Mr.parse;
                    (Ye = Ct({}, Ye)).stops && (Ye.stops = Ye.stops.map((Nr) => [Nr[0], xr(Nr[1])])), Ye.default ? Ye.default = xr(Ye.default) : Ye.default = xr(He.default);
                  }
                  if (Ye.colorSpace && (Cn = Ye.colorSpace) !== "rgb" && Cn !== "hcl" && Cn !== "lab") throw new Error(`Unknown color space: "${Ye.colorSpace}"`);
                  var Cn;
                  let Nn, Wn, lr;
                  if (pn === "exponential") Nn = To;
                  else if (pn === "interval") Nn = io;
                  else if (pn === "categorical") {
                    Nn = ao, Wn = /* @__PURE__ */ Object.create(null);
                    for (const xr of Ye.stops) Wn[xr[0]] = xr[1];
                    lr = typeof Ye.stops[0][0];
                  } else {
                    if (pn !== "identity") throw new Error(`Unknown function type "${pn}"`);
                    Nn = Mo;
                  }
                  if (xt) {
                    const xr = {}, Nr = [];
                    for (let ui = 0; ui < Ye.stops.length; ui++) {
                      const Ui = Ye.stops[ui], ia = Ui[0].zoom;
                      xr[ia] === void 0 && (xr[ia] = { zoom: ia, type: Ye.type, property: Ye.property, default: Ye.default, stops: [] }, Nr.push(ia)), xr[ia].stops.push([Ui[0].value, Ui[1]]);
                    }
                    const Kr = [];
                    for (const ui of Nr) Kr.push([xr[ui].zoom, Ha(xr[ui], He)]);
                    const xi = { name: "linear" };
                    return { kind: "composite", interpolationType: xi, interpolationFactor: Vr.interpolationFactor.bind(void 0, xi), zoomStops: Kr.map((ui) => ui[0]), evaluate({ zoom: ui }, Ui) {
                      return To({ stops: Kr, base: Ye.base }, He, ui).evaluate(ui, Ui);
                    } };
                  }
                  if (ln) {
                    const xr = pn === "exponential" ? { name: "exponential", base: Ye.base !== void 0 ? Ye.base : 1 } : null;
                    return { kind: "camera", interpolationType: xr, interpolationFactor: Vr.interpolationFactor.bind(void 0, xr), zoomStops: Ye.stops.map((Nr) => Nr[0]), evaluate: ({ zoom: Nr }) => Nn(Ye, He, Nr, Wn, lr) };
                  }
                  return { kind: "source", evaluate(xr, Nr) {
                    const Kr = Nr && Nr.properties ? Nr.properties[Ye.property] : void 0;
                    return Kr === void 0 ? Ba(Ye.default, He.default) : Nn(Ye, He, Kr, Wn, lr);
                  } };
                }
                function Ba(Ye, He, ot) {
                  return Ye !== void 0 ? Ye : He !== void 0 ? He : ot !== void 0 ? ot : void 0;
                }
                function ao(Ye, He, ot, xt, zt) {
                  return Ba(typeof ot === zt ? xt[ot] : void 0, Ye.default, He.default);
                }
                function io(Ye, He, ot) {
                  if (Ji(ot) !== "number") return Ba(Ye.default, He.default);
                  const xt = Ye.stops.length;
                  if (xt === 1 || ot <= Ye.stops[0][0]) return Ye.stops[0][1];
                  if (ot >= Ye.stops[xt - 1][0]) return Ye.stops[xt - 1][1];
                  const zt = Da(Ye.stops.map((ln) => ln[0]), ot);
                  return Ye.stops[zt][1];
                }
                function To(Ye, He, ot) {
                  const xt = Ye.base !== void 0 ? Ye.base : 1;
                  if (Ji(ot) !== "number") return Ba(Ye.default, He.default);
                  const zt = Ye.stops.length;
                  if (zt === 1 || ot <= Ye.stops[0][0]) return Ye.stops[0][1];
                  if (ot >= Ye.stops[zt - 1][0]) return Ye.stops[zt - 1][1];
                  const ln = Da(Ye.stops.map((lr) => lr[0]), ot), pn = function(lr, xr, Nr, Kr) {
                    const xi = Kr - Nr, ui = lr - Nr;
                    return xi === 0 ? 0 : xr === 1 ? ui / xi : (Math.pow(xr, ui) - 1) / (Math.pow(xr, xi) - 1);
                  }(ot, xt, Ye.stops[ln][0], Ye.stops[ln + 1][0]), Cn = Ye.stops[ln][1], Nn = Ye.stops[ln + 1][1], Wn = Rr[He.type] || Ga;
                  return typeof Cn.evaluate == "function" ? { evaluate(...lr) {
                    const xr = Cn.evaluate.apply(void 0, lr), Nr = Nn.evaluate.apply(void 0, lr);
                    if (xr !== void 0 && Nr !== void 0) return Wn(xr, Nr, pn, Ye.colorSpace);
                  } } : Wn(Cn, Nn, pn, Ye.colorSpace);
                }
                function Mo(Ye, He, ot) {
                  switch (He.type) {
                    case "color":
                      ot = mr.parse(ot);
                      break;
                    case "formatted":
                      ot = jr.fromString(ot.toString());
                      break;
                    case "resolvedImage":
                      ot = gi.fromString(ot.toString());
                      break;
                    case "padding":
                      ot = Mr.parse(ot);
                      break;
                    default:
                      Ji(ot) === He.type || He.type === "enum" && He.values[ot] || (ot = void 0);
                  }
                  return Ba(ot, Ye.default, He.default);
                }
                Jc.register(So, { error: [{ kind: "error" }, [hn], (Ye, [He]) => {
                  throw new $i(He.evaluate(Ye));
                }], typeof: [hn, [rn], (Ye, [He]) => Rn(ri(He.evaluate(Ye)))], "to-rgba": [wn(sn, 4), [Ut], (Ye, [He]) => {
                  const [ot, xt, zt, ln] = He.evaluate(Ye).rgb;
                  return [255 * ot, 255 * xt, 255 * zt, ln];
                }], rgb: [Ut, [sn, sn, sn], No], rgba: [Ut, [sn, sn, sn, sn], No], has: { type: Qt, overloads: [[[hn], (Ye, [He]) => ro(He.evaluate(Ye), Ye.properties())], [[hn, Wt], (Ye, [He, ot]) => ro(He.evaluate(Ye), ot.evaluate(Ye))]] }, get: { type: rn, overloads: [[[hn], (Ye, [He]) => Io(He.evaluate(Ye), Ye.properties())], [[hn, Wt], (Ye, [He, ot]) => Io(He.evaluate(Ye), ot.evaluate(Ye))]] }, "feature-state": [rn, [hn], (Ye, [He]) => Io(He.evaluate(Ye), Ye.featureState || {})], properties: [Wt, [], (Ye) => Ye.properties()], "geometry-type": [hn, [], (Ye) => Ye.geometryType()], id: [rn, [], (Ye) => Ye.id()], zoom: [sn, [], (Ye) => Ye.globals.zoom], "heatmap-density": [sn, [], (Ye) => Ye.globals.heatmapDensity || 0], "line-progress": [sn, [], (Ye) => Ye.globals.lineProgress || 0], accumulated: [rn, [], (Ye) => Ye.globals.accumulated === void 0 ? null : Ye.globals.accumulated], "+": [sn, bs(sn), (Ye, He) => {
                  let ot = 0;
                  for (const xt of He) ot += xt.evaluate(Ye);
                  return ot;
                }], "*": [sn, bs(sn), (Ye, He) => {
                  let ot = 1;
                  for (const xt of He) ot *= xt.evaluate(Ye);
                  return ot;
                }], "-": { type: sn, overloads: [[[sn, sn], (Ye, [He, ot]) => He.evaluate(Ye) - ot.evaluate(Ye)], [[sn], (Ye, [He]) => -He.evaluate(Ye)]] }, "/": [sn, [sn, sn], (Ye, [He, ot]) => He.evaluate(Ye) / ot.evaluate(Ye)], "%": [sn, [sn, sn], (Ye, [He, ot]) => He.evaluate(Ye) % ot.evaluate(Ye)], ln2: [sn, [], () => Math.LN2], pi: [sn, [], () => Math.PI], e: [sn, [], () => Math.E], "^": [sn, [sn, sn], (Ye, [He, ot]) => Math.pow(He.evaluate(Ye), ot.evaluate(Ye))], sqrt: [sn, [sn], (Ye, [He]) => Math.sqrt(He.evaluate(Ye))], log10: [sn, [sn], (Ye, [He]) => Math.log(He.evaluate(Ye)) / Math.LN10], ln: [sn, [sn], (Ye, [He]) => Math.log(He.evaluate(Ye))], log2: [sn, [sn], (Ye, [He]) => Math.log(He.evaluate(Ye)) / Math.LN2], sin: [sn, [sn], (Ye, [He]) => Math.sin(He.evaluate(Ye))], cos: [sn, [sn], (Ye, [He]) => Math.cos(He.evaluate(Ye))], tan: [sn, [sn], (Ye, [He]) => Math.tan(He.evaluate(Ye))], asin: [sn, [sn], (Ye, [He]) => Math.asin(He.evaluate(Ye))], acos: [sn, [sn], (Ye, [He]) => Math.acos(He.evaluate(Ye))], atan: [sn, [sn], (Ye, [He]) => Math.atan(He.evaluate(Ye))], min: [sn, bs(sn), (Ye, He) => Math.min(...He.map((ot) => ot.evaluate(Ye)))], max: [sn, bs(sn), (Ye, He) => Math.max(...He.map((ot) => ot.evaluate(Ye)))], abs: [sn, [sn], (Ye, [He]) => Math.abs(He.evaluate(Ye))], round: [sn, [sn], (Ye, [He]) => {
                  const ot = He.evaluate(Ye);
                  return ot < 0 ? -Math.round(-ot) : Math.round(ot);
                }], floor: [sn, [sn], (Ye, [He]) => Math.floor(He.evaluate(Ye))], ceil: [sn, [sn], (Ye, [He]) => Math.ceil(He.evaluate(Ye))], "filter-==": [Qt, [hn, rn], (Ye, [He, ot]) => Ye.properties()[He.value] === ot.value], "filter-id-==": [Qt, [rn], (Ye, [He]) => Ye.id() === He.value], "filter-type-==": [Qt, [hn], (Ye, [He]) => Ye.geometryType() === He.value], "filter-<": [Qt, [hn, rn], (Ye, [He, ot]) => {
                  const xt = Ye.properties()[He.value], zt = ot.value;
                  return typeof xt == typeof zt && xt < zt;
                }], "filter-id-<": [Qt, [rn], (Ye, [He]) => {
                  const ot = Ye.id(), xt = He.value;
                  return typeof ot == typeof xt && ot < xt;
                }], "filter->": [Qt, [hn, rn], (Ye, [He, ot]) => {
                  const xt = Ye.properties()[He.value], zt = ot.value;
                  return typeof xt == typeof zt && xt > zt;
                }], "filter-id->": [Qt, [rn], (Ye, [He]) => {
                  const ot = Ye.id(), xt = He.value;
                  return typeof ot == typeof xt && ot > xt;
                }], "filter-<=": [Qt, [hn, rn], (Ye, [He, ot]) => {
                  const xt = Ye.properties()[He.value], zt = ot.value;
                  return typeof xt == typeof zt && xt <= zt;
                }], "filter-id-<=": [Qt, [rn], (Ye, [He]) => {
                  const ot = Ye.id(), xt = He.value;
                  return typeof ot == typeof xt && ot <= xt;
                }], "filter->=": [Qt, [hn, rn], (Ye, [He, ot]) => {
                  const xt = Ye.properties()[He.value], zt = ot.value;
                  return typeof xt == typeof zt && xt >= zt;
                }], "filter-id->=": [Qt, [rn], (Ye, [He]) => {
                  const ot = Ye.id(), xt = He.value;
                  return typeof ot == typeof xt && ot >= xt;
                }], "filter-has": [Qt, [rn], (Ye, [He]) => He.value in Ye.properties()], "filter-has-id": [Qt, [], (Ye) => Ye.id() !== null && Ye.id() !== void 0], "filter-type-in": [Qt, [wn(hn)], (Ye, [He]) => He.value.indexOf(Ye.geometryType()) >= 0], "filter-id-in": [Qt, [wn(rn)], (Ye, [He]) => He.value.indexOf(Ye.id()) >= 0], "filter-in-small": [Qt, [hn, wn(rn)], (Ye, [He, ot]) => ot.value.indexOf(Ye.properties()[He.value]) >= 0], "filter-in-large": [Qt, [hn, wn(rn)], (Ye, [He, ot]) => function(xt, zt, ln, pn) {
                  for (; ln <= pn; ) {
                    const Cn = ln + pn >> 1;
                    if (zt[Cn] === xt) return !0;
                    zt[Cn] > xt ? pn = Cn - 1 : ln = Cn + 1;
                  }
                  return !1;
                }(Ye.properties()[He.value], ot.value, 0, ot.value.length - 1)], all: { type: Qt, overloads: [[[Qt, Qt], (Ye, [He, ot]) => He.evaluate(Ye) && ot.evaluate(Ye)], [bs(Qt), (Ye, He) => {
                  for (const ot of He) if (!ot.evaluate(Ye)) return !1;
                  return !0;
                }]] }, any: { type: Qt, overloads: [[[Qt, Qt], (Ye, [He, ot]) => He.evaluate(Ye) || ot.evaluate(Ye)], [bs(Qt), (Ye, He) => {
                  for (const ot of He) if (ot.evaluate(Ye)) return !0;
                  return !1;
                }]] }, "!": [Qt, [Qt], (Ye, [He]) => !He.evaluate(Ye)], "is-supported-script": [Qt, [hn], (Ye, [He]) => {
                  const ot = Ye.globals && Ye.globals.isSupportedScript;
                  return !ot || ot(He.evaluate(Ye));
                }], upcase: [hn, [hn], (Ye, [He]) => He.evaluate(Ye).toUpperCase()], downcase: [hn, [hn], (Ye, [He]) => He.evaluate(Ye).toLowerCase()], concat: [hn, bs(rn), (Ye, He) => He.map((ot) => bi(ot.evaluate(Ye))).join("")], "resolved-locale": [hn, [vn], (Ye, [He]) => He.evaluate(Ye).resolvedLocale()] });
                class po {
                  constructor(He, ot) {
                    var xt;
                    this.expression = He, this._warningHistory = {}, this._evaluator = new ka(), this._defaultValue = ot ? (xt = ot).type === "color" && Pa(xt.default) ? new mr(0, 0, 0, 0) : xt.type === "color" ? mr.parse(xt.default) || null : xt.type === "padding" ? Mr.parse(xt.default) || null : xt.type === "variableAnchorOffsetCollection" ? Bi.parse(xt.default) || null : xt.default === void 0 ? null : xt.default : null, this._enumValues = ot && ot.type === "enum" ? ot.values : null;
                  }
                  evaluateWithoutErrorHandling(He, ot, xt, zt, ln, pn) {
                    return this._evaluator.globals = He, this._evaluator.feature = ot, this._evaluator.featureState = xt, this._evaluator.canonical = zt, this._evaluator.availableImages = ln || null, this._evaluator.formattedSection = pn, this.expression.evaluate(this._evaluator);
                  }
                  evaluate(He, ot, xt, zt, ln, pn) {
                    this._evaluator.globals = He, this._evaluator.feature = ot || null, this._evaluator.featureState = xt || null, this._evaluator.canonical = zt, this._evaluator.availableImages = ln || null, this._evaluator.formattedSection = pn || null;
                    try {
                      const Cn = this.expression.evaluate(this._evaluator);
                      if (Cn == null || typeof Cn == "number" && Cn != Cn) return this._defaultValue;
                      if (this._enumValues && !(Cn in this._enumValues)) throw new $i(`Expected value to be one of ${Object.keys(this._enumValues).map((Nn) => JSON.stringify(Nn)).join(", ")}, but found ${JSON.stringify(Cn)} instead.`);
                      return Cn;
                    } catch (Cn) {
                      return this._warningHistory[Cn.message] || (this._warningHistory[Cn.message] = !0, typeof console < "u" && console.warn(Cn.message)), this._defaultValue;
                    }
                  }
                }
                function Vo(Ye) {
                  return Array.isArray(Ye) && Ye.length > 0 && typeof Ye[0] == "string" && Ye[0] in So;
                }
                function Xo(Ye, He) {
                  const ot = new aa(So, ws, [], He ? function(zt) {
                    const ln = { color: Ut, string: hn, number: sn, enum: hn, boolean: Qt, formatted: dn, padding: Ht, resolvedImage: jt, variableAnchorOffsetCollection: cn };
                    return zt.type === "array" ? wn(ln[zt.value] || rn, zt.length) : ln[zt.type];
                  }(He) : void 0), xt = ot.parse(Ye, void 0, void 0, void 0, He && He.type === "string" ? { typeAnnotation: "coerce" } : void 0);
                  return xt ? Ms(new po(xt, He)) : vi(ot.errors);
                }
                class Go {
                  constructor(He, ot) {
                    this.kind = He, this._styleExpression = ot, this.isStateDependent = He !== "constant" && !Yo(ot.expression);
                  }
                  evaluateWithoutErrorHandling(He, ot, xt, zt, ln, pn) {
                    return this._styleExpression.evaluateWithoutErrorHandling(He, ot, xt, zt, ln, pn);
                  }
                  evaluate(He, ot, xt, zt, ln, pn) {
                    return this._styleExpression.evaluate(He, ot, xt, zt, ln, pn);
                  }
                }
                class es {
                  constructor(He, ot, xt, zt) {
                    this.kind = He, this.zoomStops = xt, this._styleExpression = ot, this.isStateDependent = He !== "camera" && !Yo(ot.expression), this.interpolationType = zt;
                  }
                  evaluateWithoutErrorHandling(He, ot, xt, zt, ln, pn) {
                    return this._styleExpression.evaluateWithoutErrorHandling(He, ot, xt, zt, ln, pn);
                  }
                  evaluate(He, ot, xt, zt, ln, pn) {
                    return this._styleExpression.evaluate(He, ot, xt, zt, ln, pn);
                  }
                  interpolationFactor(He, ot, xt) {
                    return this.interpolationType ? Vr.interpolationFactor(this.interpolationType, He, ot, xt) : 0;
                  }
                }
                function Ts(Ye, He) {
                  const ot = Xo(Ye, He);
                  if (ot.result === "error") return ot;
                  const xt = ot.value.expression, zt = zo(xt);
                  if (!zt && !Ri(He)) return vi([new Kt("", "data expressions not supported")]);
                  const ln = Ls(xt, ["zoom"]);
                  if (!ln && !ta(He)) return vi([new Kt("", "zoom expressions not supported")]);
                  const pn = Oo(xt);
                  if (!pn && !ln) return vi([new Kt("", '"zoom" expression may only be used as input to a top-level "step" or "interpolate" expression.')]);
                  if (pn instanceof Kt) return vi([pn]);
                  if (pn instanceof Vr && !da(He)) return vi([new Kt("", '"interpolate" expressions cannot be used with this property')]);
                  if (!pn) return Ms(new Go(zt ? "constant" : "source", ot.value));
                  const Cn = pn instanceof Vr ? pn.interpolation : void 0;
                  return Ms(new es(zt ? "camera" : "composite", ot.value, pn.labels, Cn));
                }
                class Fo {
                  constructor(He, ot) {
                    this._parameters = He, this._specification = ot, Ct(this, Ha(this._parameters, this._specification));
                  }
                  static deserialize(He) {
                    return new Fo(He._parameters, He._specification);
                  }
                  static serialize(He) {
                    return { _parameters: He._parameters, _specification: He._specification };
                  }
                }
                function Oo(Ye) {
                  let He = null;
                  if (Ye instanceof hi) He = Oo(Ye.result);
                  else if (Ye instanceof Jr) {
                    for (const ot of Ye.args) if (He = Oo(ot), He) break;
                  } else (Ye instanceof la || Ye instanceof Vr) && Ye.input instanceof Jc && Ye.input.name === "zoom" && (He = Ye);
                  return He instanceof Kt || Ye.eachChild((ot) => {
                    const xt = Oo(ot);
                    xt instanceof Kt ? He = xt : !He && xt ? He = new Kt("", '"zoom" expression may only be used as input to a top-level "step" or "interpolate" expression.') : He && xt && He !== xt && (He = new Kt("", 'Only one zoom-based "step" or "interpolate" subexpression may be used in an expression.'));
                  }), He;
                }
                function gs(Ye) {
                  if (Ye === !0 || Ye === !1) return !0;
                  if (!Array.isArray(Ye) || Ye.length === 0) return !1;
                  switch (Ye[0]) {
                    case "has":
                      return Ye.length >= 2 && Ye[1] !== "$id" && Ye[1] !== "$type";
                    case "in":
                      return Ye.length >= 3 && (typeof Ye[1] != "string" || Array.isArray(Ye[2]));
                    case "!in":
                    case "!has":
                    case "none":
                      return !1;
                    case "==":
                    case "!=":
                    case ">":
                    case ">=":
                    case "<":
                    case "<=":
                      return Ye.length !== 3 || Array.isArray(Ye[1]) || Array.isArray(Ye[2]);
                    case "any":
                    case "all":
                      for (const He of Ye.slice(1)) if (!gs(He) && typeof He != "boolean") return !1;
                      return !0;
                    default:
                      return !0;
                  }
                }
                const Bu = { type: "boolean", default: !1, transition: !1, "property-type": "data-driven", expression: { interpolated: !1, parameters: ["zoom", "feature"] } };
                function Ql(Ye) {
                  if (Ye == null) return { filter: () => !0, needGeometry: !1 };
                  gs(Ye) || (Ye = z1(Ye));
                  const He = Xo(Ye, Bu);
                  if (He.result === "error") throw new Error(He.value.map((ot) => `${ot.key}: ${ot.message}`).join(", "));
                  return { filter: (ot, xt, zt) => He.value.evaluate(ot, xt, {}, zt), needGeometry: h1(Ye) };
                }
                function dl(Ye, He) {
                  return Ye < He ? -1 : Ye > He ? 1 : 0;
                }
                function h1(Ye) {
                  if (!Array.isArray(Ye)) return !1;
                  if (Ye[0] === "within" || Ye[0] === "distance") return !0;
                  for (let He = 1; He < Ye.length; He++) if (h1(Ye[He])) return !0;
                  return !1;
                }
                function z1(Ye) {
                  if (!Ye) return !0;
                  const He = Ye[0];
                  return Ye.length <= 1 ? He !== "any" : He === "==" ? J1(Ye[1], Ye[2], "==") : He === "!=" ? ku(J1(Ye[1], Ye[2], "==")) : He === "<" || He === ">" || He === "<=" || He === ">=" ? J1(Ye[1], Ye[2], He) : He === "any" ? (ot = Ye.slice(1), ["any"].concat(ot.map(z1))) : He === "all" ? ["all"].concat(Ye.slice(1).map(z1)) : He === "none" ? ["all"].concat(Ye.slice(1).map(z1).map(ku)) : He === "in" ? Lu(Ye[1], Ye.slice(2)) : He === "!in" ? ku(Lu(Ye[1], Ye.slice(2))) : He === "has" ? M1(Ye[1]) : He !== "!has" || ku(M1(Ye[1]));
                  var ot;
                }
                function J1(Ye, He, ot) {
                  switch (Ye) {
                    case "$type":
                      return [`filter-type-${ot}`, He];
                    case "$id":
                      return [`filter-id-${ot}`, He];
                    default:
                      return [`filter-${ot}`, Ye, He];
                  }
                }
                function Lu(Ye, He) {
                  if (He.length === 0) return !1;
                  switch (Ye) {
                    case "$type":
                      return ["filter-type-in", ["literal", He]];
                    case "$id":
                      return ["filter-id-in", ["literal", He]];
                    default:
                      return He.length > 200 && !He.some((ot) => typeof ot != typeof He[0]) ? ["filter-in-large", Ye, ["literal", He.sort(dl)]] : ["filter-in-small", Ye, ["literal", He]];
                  }
                }
                function M1(Ye) {
                  switch (Ye) {
                    case "$type":
                      return !0;
                    case "$id":
                      return ["filter-has-id"];
                    default:
                      return ["filter-has", Ye];
                  }
                }
                function ku(Ye) {
                  return ["!", Ye];
                }
                function Wu(Ye) {
                  const He = typeof Ye;
                  if (He === "number" || He === "boolean" || He === "string" || Ye == null) return JSON.stringify(Ye);
                  if (Array.isArray(Ye)) {
                    let zt = "[";
                    for (const ln of Ye) zt += `${Wu(ln)},`;
                    return `${zt}]`;
                  }
                  const ot = Object.keys(Ye).sort();
                  let xt = "{";
                  for (let zt = 0; zt < ot.length; zt++) xt += `${JSON.stringify(ot[zt])}:${Wu(Ye[ot[zt]])},`;
                  return `${xt}}`;
                }
                function T1(Ye) {
                  let He = "";
                  for (const ot of St) He += `/${Wu(Ye[ot])}`;
                  return He;
                }
                function kl(Ye) {
                  const He = Ye.key, ot = Ye.value;
                  return ot ? [new It(He, ot, "constants have been deprecated as of v8")] : [];
                }
                function Yl(Ye) {
                  return Ye instanceof Number || Ye instanceof String || Ye instanceof Boolean ? Ye.valueOf() : Ye;
                }
                function i1(Ye) {
                  if (Array.isArray(Ye)) return Ye.map(i1);
                  if (Ye instanceof Object && !(Ye instanceof Number || Ye instanceof String || Ye instanceof Boolean)) {
                    const He = {};
                    for (const ot in Ye) He[ot] = i1(Ye[ot]);
                    return He;
                  }
                  return Yl(Ye);
                }
                function ic(Ye) {
                  const He = Ye.key, ot = Ye.value, xt = Ye.valueSpec || {}, zt = Ye.objectElementValidators || {}, ln = Ye.style, pn = Ye.styleSpec, Cn = Ye.validateSpec;
                  let Nn = [];
                  const Wn = Ji(ot);
                  if (Wn !== "object") return [new It(He, ot, `object expected, ${Wn} found`)];
                  for (const lr in ot) {
                    const xr = lr.split(".")[0], Nr = xt[xr] || xt["*"];
                    let Kr;
                    if (zt[xr]) Kr = zt[xr];
                    else if (xt[xr]) Kr = Cn;
                    else if (zt["*"]) Kr = zt["*"];
                    else {
                      if (!xt["*"]) {
                        Nn.push(new It(He, ot[lr], `unknown property "${lr}"`));
                        continue;
                      }
                      Kr = Cn;
                    }
                    Nn = Nn.concat(Kr({ key: (He && `${He}.`) + lr, value: ot[lr], valueSpec: Nr, style: ln, styleSpec: pn, object: ot, objectKey: lr, validateSpec: Cn }, ot));
                  }
                  for (const lr in xt) zt[lr] || xt[lr].required && xt[lr].default === void 0 && ot[lr] === void 0 && Nn.push(new It(He, ot, `missing required property "${lr}"`));
                  return Nn;
                }
                function iu(Ye) {
                  const He = Ye.value, ot = Ye.valueSpec, xt = Ye.validateSpec, zt = Ye.style, ln = Ye.styleSpec, pn = Ye.key, Cn = Ye.arrayElementValidator || xt;
                  if (Ji(He) !== "array") return [new It(pn, He, `array expected, ${Ji(He)} found`)];
                  if (ot.length && He.length !== ot.length) return [new It(pn, He, `array length ${ot.length} expected, length ${He.length} found`)];
                  if (ot["min-length"] && He.length < ot["min-length"]) return [new It(pn, He, `array length at least ${ot["min-length"]} expected, length ${He.length} found`)];
                  let Nn = { type: ot.value, values: ot.values };
                  ln.$version < 7 && (Nn.function = ot.function), Ji(ot.value) === "object" && (Nn = ot.value);
                  let Wn = [];
                  for (let lr = 0; lr < He.length; lr++) Wn = Wn.concat(Cn({ array: He, arrayIndex: lr, value: He[lr], valueSpec: Nn, validateSpec: Ye.validateSpec, style: zt, styleSpec: ln, key: `${pn}[${lr}]` }));
                  return Wn;
                }
                function xu(Ye) {
                  const He = Ye.key, ot = Ye.value, xt = Ye.valueSpec;
                  let zt = Ji(ot);
                  return zt === "number" && ot != ot && (zt = "NaN"), zt !== "number" ? [new It(He, ot, `number expected, ${zt} found`)] : "minimum" in xt && ot < xt.minimum ? [new It(He, ot, `${ot} is less than the minimum value ${xt.minimum}`)] : "maximum" in xt && ot > xt.maximum ? [new It(He, ot, `${ot} is greater than the maximum value ${xt.maximum}`)] : [];
                }
                function vc(Ye) {
                  const He = Ye.valueSpec, ot = Yl(Ye.value.type);
                  let xt, zt, ln, pn = {};
                  const Cn = ot !== "categorical" && Ye.value.property === void 0, Nn = !Cn, Wn = Ji(Ye.value.stops) === "array" && Ji(Ye.value.stops[0]) === "array" && Ji(Ye.value.stops[0][0]) === "object", lr = ic({ key: Ye.key, value: Ye.value, valueSpec: Ye.styleSpec.function, validateSpec: Ye.validateSpec, style: Ye.style, styleSpec: Ye.styleSpec, objectElementValidators: { stops: function(Kr) {
                    if (ot === "identity") return [new It(Kr.key, Kr.value, 'identity function may not have a "stops" property')];
                    let xi = [];
                    const ui = Kr.value;
                    return xi = xi.concat(iu({ key: Kr.key, value: ui, valueSpec: Kr.valueSpec, validateSpec: Kr.validateSpec, style: Kr.style, styleSpec: Kr.styleSpec, arrayElementValidator: xr })), Ji(ui) === "array" && ui.length === 0 && xi.push(new It(Kr.key, ui, "array must have at least one stop")), xi;
                  }, default: function(Kr) {
                    return Kr.validateSpec({ key: Kr.key, value: Kr.value, valueSpec: He, validateSpec: Kr.validateSpec, style: Kr.style, styleSpec: Kr.styleSpec });
                  } } });
                  return ot === "identity" && Cn && lr.push(new It(Ye.key, Ye.value, 'missing required property "property"')), ot === "identity" || Ye.value.stops || lr.push(new It(Ye.key, Ye.value, 'missing required property "stops"')), ot === "exponential" && Ye.valueSpec.expression && !da(Ye.valueSpec) && lr.push(new It(Ye.key, Ye.value, "exponential functions not supported")), Ye.styleSpec.$version >= 8 && (Nn && !Ri(Ye.valueSpec) ? lr.push(new It(Ye.key, Ye.value, "property functions not supported")) : Cn && !ta(Ye.valueSpec) && lr.push(new It(Ye.key, Ye.value, "zoom functions not supported"))), ot !== "categorical" && !Wn || Ye.value.property !== void 0 || lr.push(new It(Ye.key, Ye.value, '"property" property is required')), lr;
                  function xr(Kr) {
                    let xi = [];
                    const ui = Kr.value, Ui = Kr.key;
                    if (Ji(ui) !== "array") return [new It(Ui, ui, `array expected, ${Ji(ui)} found`)];
                    if (ui.length !== 2) return [new It(Ui, ui, `array length 2 expected, length ${ui.length} found`)];
                    if (Wn) {
                      if (Ji(ui[0]) !== "object") return [new It(Ui, ui, `object expected, ${Ji(ui[0])} found`)];
                      if (ui[0].zoom === void 0) return [new It(Ui, ui, "object stop key must have zoom")];
                      if (ui[0].value === void 0) return [new It(Ui, ui, "object stop key must have value")];
                      if (ln && ln > Yl(ui[0].zoom)) return [new It(Ui, ui[0].zoom, "stop zoom values must appear in ascending order")];
                      Yl(ui[0].zoom) !== ln && (ln = Yl(ui[0].zoom), zt = void 0, pn = {}), xi = xi.concat(ic({ key: `${Ui}[0]`, value: ui[0], valueSpec: { zoom: {} }, validateSpec: Kr.validateSpec, style: Kr.style, styleSpec: Kr.styleSpec, objectElementValidators: { zoom: xu, value: Nr } }));
                    } else xi = xi.concat(Nr({ key: `${Ui}[0]`, value: ui[0], valueSpec: {}, validateSpec: Kr.validateSpec, style: Kr.style, styleSpec: Kr.styleSpec }, ui));
                    return Vo(i1(ui[1])) ? xi.concat([new It(`${Ui}[1]`, ui[1], "expressions are not allowed in function stops.")]) : xi.concat(Kr.validateSpec({ key: `${Ui}[1]`, value: ui[1], valueSpec: He, validateSpec: Kr.validateSpec, style: Kr.style, styleSpec: Kr.styleSpec }));
                  }
                  function Nr(Kr, xi) {
                    const ui = Ji(Kr.value), Ui = Yl(Kr.value), ia = Kr.value !== null ? Kr.value : xi;
                    if (xt) {
                      if (ui !== xt) return [new It(Kr.key, ia, `${ui} stop domain type must match previous stop domain type ${xt}`)];
                    } else xt = ui;
                    if (ui !== "number" && ui !== "string" && ui !== "boolean") return [new It(Kr.key, ia, "stop domain value must be a number, string, or boolean")];
                    if (ui !== "number" && ot !== "categorical") {
                      let Na = `number expected, ${ui} found`;
                      return Ri(He) && ot === void 0 && (Na += '\nIf you intended to use a categorical function, specify `"type": "categorical"`.'), [new It(Kr.key, ia, Na)];
                    }
                    return ot !== "categorical" || ui !== "number" || isFinite(Ui) && Math.floor(Ui) === Ui ? ot !== "categorical" && ui === "number" && zt !== void 0 && Ui < zt ? [new It(Kr.key, ia, "stop domain values must appear in ascending order")] : (zt = Ui, ot === "categorical" && Ui in pn ? [new It(Kr.key, ia, "stop domain values must be unique")] : (pn[Ui] = !0, [])) : [new It(Kr.key, ia, `integer expected, found ${Ui}`)];
                  }
                }
                function Zl(Ye) {
                  const He = (Ye.expressionContext === "property" ? Ts : Xo)(i1(Ye.value), Ye.valueSpec);
                  if (He.result === "error") return He.value.map((xt) => new It(`${Ye.key}${xt.key}`, Ye.value, xt.message));
                  const ot = He.value.expression || He.value._styleExpression.expression;
                  if (Ye.expressionContext === "property" && Ye.propertyKey === "text-font" && !ot.outputDefined()) return [new It(Ye.key, Ye.value, `Invalid data expression for "${Ye.propertyKey}". Output values must be contained as literals within the expression.`)];
                  if (Ye.expressionContext === "property" && Ye.propertyType === "layout" && !Yo(ot)) return [new It(Ye.key, Ye.value, '"feature-state" data expressions are not supported with layout properties.')];
                  if (Ye.expressionContext === "filter" && !Yo(ot)) return [new It(Ye.key, Ye.value, '"feature-state" data expressions are not supported with filters.')];
                  if (Ye.expressionContext && Ye.expressionContext.indexOf("cluster") === 0) {
                    if (!Ls(ot, ["zoom", "feature-state"])) return [new It(Ye.key, Ye.value, '"zoom" and "feature-state" expressions are not supported with cluster properties.')];
                    if (Ye.expressionContext === "cluster-initial" && !zo(ot)) return [new It(Ye.key, Ye.value, "Feature data expressions are not supported with initial expression part of cluster properties.")];
                  }
                  return [];
                }
                function Ec(Ye) {
                  const He = Ye.key, ot = Ye.value, xt = Ye.valueSpec, zt = [];
                  return Array.isArray(xt.values) ? xt.values.indexOf(Yl(ot)) === -1 && zt.push(new It(He, ot, `expected one of [${xt.values.join(", ")}], ${JSON.stringify(ot)} found`)) : Object.keys(xt.values).indexOf(Yl(ot)) === -1 && zt.push(new It(He, ot, `expected one of [${Object.keys(xt.values).join(", ")}], ${JSON.stringify(ot)} found`)), zt;
                }
                function wu(Ye) {
                  return gs(i1(Ye.value)) ? Zl(Ct({}, Ye, { expressionContext: "filter", valueSpec: { value: "boolean" } })) : Pc(Ye);
                }
                function Pc(Ye) {
                  const He = Ye.value, ot = Ye.key;
                  if (Ji(He) !== "array") return [new It(ot, He, `array expected, ${Ji(He)} found`)];
                  const xt = Ye.styleSpec;
                  let zt, ln = [];
                  if (He.length < 1) return [new It(ot, He, "filter array must have at least 1 element")];
                  switch (ln = ln.concat(Ec({ key: `${ot}[0]`, value: He[0], valueSpec: xt.filter_operator, style: Ye.style, styleSpec: Ye.styleSpec })), Yl(He[0])) {
                    case "<":
                    case "<=":
                    case ">":
                    case ">=":
                      He.length >= 2 && Yl(He[1]) === "$type" && ln.push(new It(ot, He, `"$type" cannot be use with operator "${He[0]}"`));
                    case "==":
                    case "!=":
                      He.length !== 3 && ln.push(new It(ot, He, `filter array for operator "${He[0]}" must have 3 elements`));
                    case "in":
                    case "!in":
                      He.length >= 2 && (zt = Ji(He[1]), zt !== "string" && ln.push(new It(`${ot}[1]`, He[1], `string expected, ${zt} found`)));
                      for (let pn = 2; pn < He.length; pn++) zt = Ji(He[pn]), Yl(He[1]) === "$type" ? ln = ln.concat(Ec({ key: `${ot}[${pn}]`, value: He[pn], valueSpec: xt.geometry_type, style: Ye.style, styleSpec: Ye.styleSpec })) : zt !== "string" && zt !== "number" && zt !== "boolean" && ln.push(new It(`${ot}[${pn}]`, He[pn], `string, number, or boolean expected, ${zt} found`));
                      break;
                    case "any":
                    case "all":
                    case "none":
                      for (let pn = 1; pn < He.length; pn++) ln = ln.concat(Pc({ key: `${ot}[${pn}]`, value: He[pn], style: Ye.style, styleSpec: Ye.styleSpec }));
                      break;
                    case "has":
                    case "!has":
                      zt = Ji(He[1]), He.length !== 2 ? ln.push(new It(ot, He, `filter array for "${He[0]}" operator must have 2 elements`)) : zt !== "string" && ln.push(new It(`${ot}[1]`, He[1], `string expected, ${zt} found`));
                  }
                  return ln;
                }
                function du(Ye, He) {
                  const ot = Ye.key, xt = Ye.validateSpec, zt = Ye.style, ln = Ye.styleSpec, pn = Ye.value, Cn = Ye.objectKey, Nn = ln[`${He}_${Ye.layerType}`];
                  if (!Nn) return [];
                  const Wn = Cn.match(/^(.*)-transition$/);
                  if (He === "paint" && Wn && Nn[Wn[1]] && Nn[Wn[1]].transition) return xt({ key: ot, value: pn, valueSpec: ln.transition, style: zt, styleSpec: ln });
                  const lr = Ye.valueSpec || Nn[Cn];
                  if (!lr) return [new It(ot, pn, `unknown property "${Cn}"`)];
                  let xr;
                  if (Ji(pn) === "string" && Ri(lr) && !lr.tokens && (xr = /^{([^}]+)}$/.exec(pn))) return [new It(ot, pn, `"${Cn}" does not support interpolation syntax
Use an identity property function instead: \`{ "type": "identity", "property": ${JSON.stringify(xr[1])} }\`.`)];
                  const Nr = [];
                  return Ye.layerType === "symbol" && (Cn === "text-field" && zt && !zt.glyphs && Nr.push(new It(ot, pn, 'use of "text-field" requires a style "glyphs" property')), Cn === "text-font" && Pa(i1(pn)) && Yl(pn.type) === "identity" && Nr.push(new It(ot, pn, '"text-font" does not support identity functions'))), Nr.concat(xt({ key: Ye.key, value: pn, valueSpec: lr, style: zt, styleSpec: ln, expressionContext: "property", propertyType: He, propertyKey: Cn }));
                }
                function Ac(Ye) {
                  return du(Ye, "paint");
                }
                function yu(Ye) {
                  return du(Ye, "layout");
                }
                function au(Ye) {
                  let He = [];
                  const ot = Ye.value, xt = Ye.key, zt = Ye.style, ln = Ye.styleSpec;
                  ot.type || ot.ref || He.push(new It(xt, ot, 'either "type" or "ref" is required'));
                  let pn = Yl(ot.type);
                  const Cn = Yl(ot.ref);
                  if (ot.id) {
                    const Nn = Yl(ot.id);
                    for (let Wn = 0; Wn < Ye.arrayIndex; Wn++) {
                      const lr = zt.layers[Wn];
                      Yl(lr.id) === Nn && He.push(new It(xt, ot.id, `duplicate layer id "${ot.id}", previously used at line ${lr.id.__line__}`));
                    }
                  }
                  if ("ref" in ot) {
                    let Nn;
                    ["type", "source", "source-layer", "filter", "layout"].forEach((Wn) => {
                      Wn in ot && He.push(new It(xt, ot[Wn], `"${Wn}" is prohibited for ref layers`));
                    }), zt.layers.forEach((Wn) => {
                      Yl(Wn.id) === Cn && (Nn = Wn);
                    }), Nn ? Nn.ref ? He.push(new It(xt, ot.ref, "ref cannot reference another ref layer")) : pn = Yl(Nn.type) : He.push(new It(xt, ot.ref, `ref layer "${Cn}" not found`));
                  } else if (pn !== "background") if (ot.source) {
                    const Nn = zt.sources && zt.sources[ot.source], Wn = Nn && Yl(Nn.type);
                    Nn ? Wn === "vector" && pn === "raster" ? He.push(new It(xt, ot.source, `layer "${ot.id}" requires a raster source`)) : Wn !== "raster-dem" && pn === "hillshade" ? He.push(new It(xt, ot.source, `layer "${ot.id}" requires a raster-dem source`)) : Wn === "raster" && pn !== "raster" ? He.push(new It(xt, ot.source, `layer "${ot.id}" requires a vector source`)) : Wn !== "vector" || ot["source-layer"] ? Wn === "raster-dem" && pn !== "hillshade" ? He.push(new It(xt, ot.source, "raster-dem source can only be used with layer type 'hillshade'.")) : pn !== "line" || !ot.paint || !ot.paint["line-gradient"] || Wn === "geojson" && Nn.lineMetrics || He.push(new It(xt, ot, `layer "${ot.id}" specifies a line-gradient, which requires a GeoJSON source with \`lineMetrics\` enabled.`)) : He.push(new It(xt, ot, `layer "${ot.id}" must specify a "source-layer"`)) : He.push(new It(xt, ot.source, `source "${ot.source}" not found`));
                  } else He.push(new It(xt, ot, 'missing required property "source"'));
                  return He = He.concat(ic({ key: xt, value: ot, valueSpec: ln.layer, style: Ye.style, styleSpec: Ye.styleSpec, validateSpec: Ye.validateSpec, objectElementValidators: { "*"() {
                    return [];
                  }, type() {
                    return Ye.validateSpec({ key: `${xt}.type`, value: ot.type, valueSpec: ln.layer.type, style: Ye.style, styleSpec: Ye.styleSpec, validateSpec: Ye.validateSpec, object: ot, objectKey: "type" });
                  }, filter: wu, layout(Nn) {
                    return ic({ layer: ot, key: Nn.key, value: Nn.value, style: Nn.style, styleSpec: Nn.styleSpec, validateSpec: Nn.validateSpec, objectElementValidators: { "*"(Wn) {
                      return yu(Ct({ layerType: pn }, Wn));
                    } } });
                  }, paint(Nn) {
                    return ic({ layer: ot, key: Nn.key, value: Nn.value, style: Nn.style, styleSpec: Nn.styleSpec, validateSpec: Nn.validateSpec, objectElementValidators: { "*"(Wn) {
                      return Ac(Ct({ layerType: pn }, Wn));
                    } } });
                  } } })), He;
                }
                function Qc(Ye) {
                  const He = Ye.value, ot = Ye.key, xt = Ji(He);
                  return xt !== "string" ? [new It(ot, He, `string expected, ${xt} found`)] : [];
                }
                const Cc = { promoteId: function({ key: Ye, value: He }) {
                  if (Ji(He) === "string") return Qc({ key: Ye, value: He });
                  {
                    const ot = [];
                    for (const xt in He) ot.push(...Qc({ key: `${Ye}.${xt}`, value: He[xt] }));
                    return ot;
                  }
                } };
                function Ol(Ye) {
                  const He = Ye.value, ot = Ye.key, xt = Ye.styleSpec, zt = Ye.style, ln = Ye.validateSpec;
                  if (!He.type) return [new It(ot, He, '"type" is required')];
                  const pn = Yl(He.type);
                  let Cn;
                  switch (pn) {
                    case "vector":
                    case "raster":
                      return Cn = ic({ key: ot, value: He, valueSpec: xt[`source_${pn.replace("-", "_")}`], style: Ye.style, styleSpec: xt, objectElementValidators: Cc, validateSpec: ln }), Cn;
                    case "raster-dem":
                      return Cn = function(Nn) {
                        var Wn;
                        const lr = (Wn = Nn.sourceName) !== null && Wn !== void 0 ? Wn : "", xr = Nn.value, Nr = Nn.styleSpec, Kr = Nr.source_raster_dem, xi = Nn.style;
                        let ui = [];
                        const Ui = Ji(xr);
                        if (xr === void 0) return ui;
                        if (Ui !== "object") return ui.push(new It("source_raster_dem", xr, `object expected, ${Ui} found`)), ui;
                        const ia = Yl(xr.encoding) === "custom", Na = ["redFactor", "greenFactor", "blueFactor", "baseShift"], Ta = Nn.value.encoding ? `"${Nn.value.encoding}"` : "Default";
                        for (const Ra in xr) !ia && Na.includes(Ra) ? ui.push(new It(Ra, xr[Ra], `In "${lr}": "${Ra}" is only valid when "encoding" is set to "custom". ${Ta} encoding found`)) : Kr[Ra] ? ui = ui.concat(Nn.validateSpec({ key: Ra, value: xr[Ra], valueSpec: Kr[Ra], validateSpec: Nn.validateSpec, style: xi, styleSpec: Nr })) : ui.push(new It(Ra, xr[Ra], `unknown property "${Ra}"`));
                        return ui;
                      }({ sourceName: ot, value: He, style: Ye.style, styleSpec: xt, validateSpec: ln }), Cn;
                    case "geojson":
                      if (Cn = ic({ key: ot, value: He, valueSpec: xt.source_geojson, style: zt, styleSpec: xt, validateSpec: ln, objectElementValidators: Cc }), He.cluster) for (const Nn in He.clusterProperties) {
                        const [Wn, lr] = He.clusterProperties[Nn], xr = typeof Wn == "string" ? [Wn, ["accumulated"], ["get", Nn]] : Wn;
                        Cn.push(...Zl({ key: `${ot}.${Nn}.map`, value: lr, validateSpec: ln, expressionContext: "cluster-map" })), Cn.push(...Zl({ key: `${ot}.${Nn}.reduce`, value: xr, validateSpec: ln, expressionContext: "cluster-reduce" }));
                      }
                      return Cn;
                    case "video":
                      return ic({ key: ot, value: He, valueSpec: xt.source_video, style: zt, validateSpec: ln, styleSpec: xt });
                    case "image":
                      return ic({ key: ot, value: He, valueSpec: xt.source_image, style: zt, validateSpec: ln, styleSpec: xt });
                    case "canvas":
                      return [new It(ot, null, "Please use runtime APIs to add canvas sources, rather than including them in stylesheets.", "source.canvas")];
                    default:
                      return Ec({ key: `${ot}.type`, value: He.type, valueSpec: { values: ["vector", "raster", "raster-dem", "geojson", "video", "image"] }, style: zt, validateSpec: ln, styleSpec: xt });
                  }
                }
                function uc(Ye) {
                  const He = Ye.value, ot = Ye.styleSpec, xt = ot.light, zt = Ye.style;
                  let ln = [];
                  const pn = Ji(He);
                  if (He === void 0) return ln;
                  if (pn !== "object") return ln = ln.concat([new It("light", He, `object expected, ${pn} found`)]), ln;
                  for (const Cn in He) {
                    const Nn = Cn.match(/^(.*)-transition$/);
                    ln = Nn && xt[Nn[1]] && xt[Nn[1]].transition ? ln.concat(Ye.validateSpec({ key: Cn, value: He[Cn], valueSpec: ot.transition, validateSpec: Ye.validateSpec, style: zt, styleSpec: ot })) : xt[Cn] ? ln.concat(Ye.validateSpec({ key: Cn, value: He[Cn], valueSpec: xt[Cn], validateSpec: Ye.validateSpec, style: zt, styleSpec: ot })) : ln.concat([new It(Cn, He[Cn], `unknown property "${Cn}"`)]);
                  }
                  return ln;
                }
                function cs(Ye) {
                  const He = Ye.value, ot = Ye.styleSpec, xt = ot.sky, zt = Ye.style, ln = Ji(He);
                  if (He === void 0) return [];
                  if (ln !== "object") return [new It("sky", He, `object expected, ${ln} found`)];
                  let pn = [];
                  for (const Cn in He) pn = xt[Cn] ? pn.concat(Ye.validateSpec({ key: Cn, value: He[Cn], valueSpec: xt[Cn], style: zt, styleSpec: ot })) : pn.concat([new It(Cn, He[Cn], `unknown property "${Cn}"`)]);
                  return pn;
                }
                function Mu(Ye) {
                  const He = Ye.value, ot = Ye.styleSpec, xt = ot.terrain, zt = Ye.style;
                  let ln = [];
                  const pn = Ji(He);
                  if (He === void 0) return ln;
                  if (pn !== "object") return ln = ln.concat([new It("terrain", He, `object expected, ${pn} found`)]), ln;
                  for (const Cn in He) ln = xt[Cn] ? ln.concat(Ye.validateSpec({ key: Cn, value: He[Cn], valueSpec: xt[Cn], validateSpec: Ye.validateSpec, style: zt, styleSpec: ot })) : ln.concat([new It(Cn, He[Cn], `unknown property "${Cn}"`)]);
                  return ln;
                }
                function Hu(Ye) {
                  let He = [];
                  const ot = Ye.value, xt = Ye.key;
                  if (Array.isArray(ot)) {
                    const zt = [], ln = [];
                    for (const pn in ot) {
                      ot[pn].id && zt.includes(ot[pn].id) && He.push(new It(xt, ot, `all the sprites' ids must be unique, but ${ot[pn].id} is duplicated`)), zt.push(ot[pn].id), ot[pn].url && ln.includes(ot[pn].url) && He.push(new It(xt, ot, `all the sprites' URLs must be unique, but ${ot[pn].url} is duplicated`)), ln.push(ot[pn].url);
                      const Cn = { id: { type: "string", required: !0 }, url: { type: "string", required: !0 } };
                      He = He.concat(ic({ key: `${xt}[${pn}]`, value: ot[pn], valueSpec: Cn, validateSpec: Ye.validateSpec }));
                    }
                    return He;
                  }
                  return Qc({ key: xt, value: ot });
                }
                const ou = { "*"() {
                  return [];
                }, array: iu, boolean: function(Ye) {
                  const He = Ye.value, ot = Ye.key, xt = Ji(He);
                  return xt !== "boolean" ? [new It(ot, He, `boolean expected, ${xt} found`)] : [];
                }, number: xu, color: function(Ye) {
                  const He = Ye.key, ot = Ye.value, xt = Ji(ot);
                  return xt !== "string" ? [new It(He, ot, `color expected, ${xt} found`)] : mr.parse(String(ot)) ? [] : [new It(He, ot, `color expected, "${ot}" found`)];
                }, constants: kl, enum: Ec, filter: wu, function: vc, layer: au, object: ic, source: Ol, light: uc, sky: cs, terrain: Mu, projection: function(Ye) {
                  const He = Ye.value, ot = Ye.styleSpec, xt = ot.projection, zt = Ye.style, ln = Ji(He);
                  if (He === void 0) return [];
                  if (ln !== "object") return [new It("projection", He, `object expected, ${ln} found`)];
                  let pn = [];
                  for (const Cn in He) pn = xt[Cn] ? pn.concat(Ye.validateSpec({ key: Cn, value: He[Cn], valueSpec: xt[Cn], style: zt, styleSpec: ot })) : pn.concat([new It(Cn, He[Cn], `unknown property "${Cn}"`)]);
                  return pn;
                }, string: Qc, formatted: function(Ye) {
                  return Qc(Ye).length === 0 ? [] : Zl(Ye);
                }, resolvedImage: function(Ye) {
                  return Qc(Ye).length === 0 ? [] : Zl(Ye);
                }, padding: function(Ye) {
                  const He = Ye.key, ot = Ye.value;
                  if (Ji(ot) === "array") {
                    if (ot.length < 1 || ot.length > 4) return [new It(He, ot, `padding requires 1 to 4 values; ${ot.length} values found`)];
                    const xt = { type: "number" };
                    let zt = [];
                    for (let ln = 0; ln < ot.length; ln++) zt = zt.concat(Ye.validateSpec({ key: `${He}[${ln}]`, value: ot[ln], validateSpec: Ye.validateSpec, valueSpec: xt }));
                    return zt;
                  }
                  return xu({ key: He, value: ot, valueSpec: {} });
                }, variableAnchorOffsetCollection: function(Ye) {
                  const He = Ye.key, ot = Ye.value, xt = Ji(ot), zt = Ye.styleSpec;
                  if (xt !== "array" || ot.length < 1 || ot.length % 2 != 0) return [new It(He, ot, "variableAnchorOffsetCollection requires a non-empty array of even length")];
                  let ln = [];
                  for (let pn = 0; pn < ot.length; pn += 2) ln = ln.concat(Ec({ key: `${He}[${pn}]`, value: ot[pn], valueSpec: zt.layout_symbol["text-anchor"] })), ln = ln.concat(iu({ key: `${He}[${pn + 1}]`, value: ot[pn + 1], valueSpec: { length: 2, value: "number" }, validateSpec: Ye.validateSpec, style: Ye.style, styleSpec: zt }));
                  return ln;
                }, sprite: Hu };
                function Gc(Ye) {
                  const He = Ye.value, ot = Ye.valueSpec, xt = Ye.styleSpec;
                  return Ye.validateSpec = Gc, ot.expression && Pa(Yl(He)) ? vc(Ye) : ot.expression && Vo(i1(He)) ? Zl(Ye) : ot.type && ou[ot.type] ? ou[ot.type](Ye) : ic(Ct({}, Ye, { valueSpec: ot.type ? xt[ot.type] : ot }));
                }
                function Su(Ye) {
                  const He = Ye.value, ot = Ye.key, xt = Qc(Ye);
                  return xt.length || (He.indexOf("{fontstack}") === -1 && xt.push(new It(ot, He, '"glyphs" url must include a "{fontstack}" token')), He.indexOf("{range}") === -1 && xt.push(new It(ot, He, '"glyphs" url must include a "{range}" token'))), xt;
                }
                function Lc(Ye, He = yt) {
                  let ot = [];
                  return ot = ot.concat(Gc({ key: "", value: Ye, valueSpec: He.$root, styleSpec: He, style: Ye, validateSpec: Gc, objectElementValidators: { glyphs: Su, "*"() {
                    return [];
                  } } })), Ye.constants && (ot = ot.concat(kl({ key: "constants", value: Ye.constants, style: Ye, styleSpec: He, validateSpec: Gc }))), Ru(ot);
                }
                function Do(Ye) {
                  return function(He) {
                    return Ye({ ...He, validateSpec: Gc });
                  };
                }
                function Ru(Ye) {
                  return [].concat(Ye).sort((He, ot) => He.line - ot.line);
                }
                function dc(Ye) {
                  return function(...He) {
                    return Ru(Ye.apply(this, He));
                  };
                }
                Lc.source = dc(Do(Ol)), Lc.sprite = dc(Do(Hu)), Lc.glyphs = dc(Do(Su)), Lc.light = dc(Do(uc)), Lc.sky = dc(Do(cs)), Lc.terrain = dc(Do(Mu)), Lc.layer = dc(Do(au)), Lc.filter = dc(Do(wu)), Lc.paintProperty = dc(Do(Ac)), Lc.layoutProperty = dc(Do(yu));
                const vl = Lc;
                vl.source;
                const Eu = vl.light, fu = vl.sky;
                vl.terrain, vl.filter;
                const ac = vl.paintProperty, bc = vl.layoutProperty;
                function kc(Ye, He) {
                  let ot = !1;
                  if (He && He.length) for (const xt of He) Ye.fire(new gt(new Error(xt.message))), ot = !0;
                  return ot;
                }
                class fc {
                  constructor(He, ot, xt) {
                    const zt = this.cells = [];
                    if (He instanceof ArrayBuffer) {
                      this.arrayBuffer = He;
                      const pn = new Int32Array(this.arrayBuffer);
                      He = pn[0], ot = pn[1], xt = pn[2], this.d = ot + 2 * xt;
                      for (let Wn = 0; Wn < this.d * this.d; Wn++) {
                        const lr = pn[3 + Wn], xr = pn[3 + Wn + 1];
                        zt.push(lr === xr ? null : pn.subarray(lr, xr));
                      }
                      const Cn = pn[3 + zt.length], Nn = pn[3 + zt.length + 1];
                      this.keys = pn.subarray(Cn, Nn), this.bboxes = pn.subarray(Nn), this.insert = this._insertReadonly;
                    } else {
                      this.d = ot + 2 * xt;
                      for (let pn = 0; pn < this.d * this.d; pn++) zt.push([]);
                      this.keys = [], this.bboxes = [];
                    }
                    this.n = ot, this.extent = He, this.padding = xt, this.scale = ot / He, this.uid = 0;
                    const ln = xt / ot * He;
                    this.min = -ln, this.max = He + ln;
                  }
                  insert(He, ot, xt, zt, ln) {
                    this._forEachCell(ot, xt, zt, ln, this._insertCell, this.uid++, void 0, void 0), this.keys.push(He), this.bboxes.push(ot), this.bboxes.push(xt), this.bboxes.push(zt), this.bboxes.push(ln);
                  }
                  _insertReadonly() {
                    throw new Error("Cannot insert into a GridIndex created from an ArrayBuffer.");
                  }
                  _insertCell(He, ot, xt, zt, ln, pn) {
                    this.cells[ln].push(pn);
                  }
                  query(He, ot, xt, zt, ln) {
                    const pn = this.min, Cn = this.max;
                    if (He <= pn && ot <= pn && Cn <= xt && Cn <= zt && !ln) return Array.prototype.slice.call(this.keys);
                    {
                      const Nn = [], Wn = {};
                      return this._forEachCell(He, ot, xt, zt, this._queryCell, Nn, Wn, ln), Nn;
                    }
                  }
                  _queryCell(He, ot, xt, zt, ln, pn, Cn, Nn) {
                    const Wn = this.cells[ln];
                    if (Wn !== null) {
                      const lr = this.keys, xr = this.bboxes;
                      for (let Nr = 0; Nr < Wn.length; Nr++) {
                        const Kr = Wn[Nr];
                        if (Cn[Kr] === void 0) {
                          const xi = 4 * Kr;
                          (Nn ? Nn(xr[xi + 0], xr[xi + 1], xr[xi + 2], xr[xi + 3]) : He <= xr[xi + 2] && ot <= xr[xi + 3] && xt >= xr[xi + 0] && zt >= xr[xi + 1]) ? (Cn[Kr] = !0, pn.push(lr[Kr])) : Cn[Kr] = !1;
                        }
                      }
                    }
                  }
                  _forEachCell(He, ot, xt, zt, ln, pn, Cn, Nn) {
                    const Wn = this._convertToCellCoord(He), lr = this._convertToCellCoord(ot), xr = this._convertToCellCoord(xt), Nr = this._convertToCellCoord(zt);
                    for (let Kr = Wn; Kr <= xr; Kr++) for (let xi = lr; xi <= Nr; xi++) {
                      const ui = this.d * xi + Kr;
                      if ((!Nn || Nn(this._convertFromCellCoord(Kr), this._convertFromCellCoord(xi), this._convertFromCellCoord(Kr + 1), this._convertFromCellCoord(xi + 1))) && ln.call(this, He, ot, xt, zt, ui, pn, Cn, Nn)) return;
                    }
                  }
                  _convertFromCellCoord(He) {
                    return (He - this.padding) / this.scale;
                  }
                  _convertToCellCoord(He) {
                    return Math.max(0, Math.min(this.d - 1, Math.floor(He * this.scale) + this.padding));
                  }
                  toArrayBuffer() {
                    if (this.arrayBuffer) return this.arrayBuffer;
                    const He = this.cells, ot = 3 + this.cells.length + 1 + 1;
                    let xt = 0;
                    for (let pn = 0; pn < this.cells.length; pn++) xt += this.cells[pn].length;
                    const zt = new Int32Array(ot + xt + this.keys.length + this.bboxes.length);
                    zt[0] = this.extent, zt[1] = this.n, zt[2] = this.padding;
                    let ln = ot;
                    for (let pn = 0; pn < He.length; pn++) {
                      const Cn = He[pn];
                      zt[3 + pn] = ln, zt.set(Cn, ln), ln += Cn.length;
                    }
                    return zt[3 + He.length] = ln, zt.set(this.keys, ln), ln += this.keys.length, zt[3 + He.length + 1] = ln, zt.set(this.bboxes, ln), ln += this.bboxes.length, zt.buffer;
                  }
                  static serialize(He, ot) {
                    const xt = He.toArrayBuffer();
                    return ot && ot.push(xt), { buffer: xt };
                  }
                  static deserialize(He) {
                    return new fc(He.buffer);
                  }
                }
                const oc = {};
                function ss(Ye, He, ot = {}) {
                  if (oc[Ye]) throw new Error(`${Ye} is already registered.`);
                  Object.defineProperty(He, "_classRegistryKey", { value: Ye, writeable: !1 }), oc[Ye] = { klass: He, omit: ot.omit || [], shallow: ot.shallow || [] };
                }
                ss("Object", Object), ss("TransferableGridIndex", fc), ss("Color", mr), ss("Error", Error), ss("AJAXError", at), ss("ResolvedImage", gi), ss("StylePropertyFunction", Fo), ss("StyleExpression", po, { omit: ["_evaluator"] }), ss("ZoomDependentExpression", es), ss("ZoomConstantExpression", Go), ss("CompoundExpression", Jc, { omit: ["_evaluate"] });
                for (const Ye in So) So[Ye]._classRegistryKey || ss(`Expression_${Ye}`, So[Ye]);
                function su(Ye) {
                  return Ye && typeof ArrayBuffer < "u" && (Ye instanceof ArrayBuffer || Ye.constructor && Ye.constructor.name === "ArrayBuffer");
                }
                function Tl(Ye) {
                  const He = Ye.constructor;
                  return Ye.$name || He._classRegistryKey;
                }
                function sc(Ye) {
                  return !function(He) {
                    if (He === null || typeof He != "object") return !1;
                    const ot = Tl(He);
                    return !(!ot || ot === "Object");
                  }(Ye) && (Ye == null || typeof Ye == "boolean" || typeof Ye == "number" || typeof Ye == "string" || Ye instanceof Boolean || Ye instanceof Number || Ye instanceof String || Ye instanceof Date || Ye instanceof RegExp || Ye instanceof Blob || Ye instanceof Error || su(Ye) || De(Ye) || ArrayBuffer.isView(Ye) || Ye instanceof ImageData);
                }
                function hc(Ye, He) {
                  if (sc(Ye)) {
                    if ((su(Ye) || De(Ye)) && He && He.push(Ye), ArrayBuffer.isView(Ye)) {
                      const ln = Ye;
                      He && He.push(ln.buffer);
                    }
                    return Ye instanceof ImageData && He && He.push(Ye.data.buffer), Ye;
                  }
                  if (Array.isArray(Ye)) {
                    const ln = [];
                    for (const pn of Ye) ln.push(hc(pn, He));
                    return ln;
                  }
                  if (typeof Ye != "object") throw new Error("can't serialize object of type " + typeof Ye);
                  const ot = Tl(Ye);
                  if (!ot) throw new Error(`can't serialize object of unregistered class ${Ye.constructor.name}`);
                  if (!oc[ot]) throw new Error(`${ot} is not registered.`);
                  const { klass: xt } = oc[ot], zt = xt.serialize ? xt.serialize(Ye, He) : {};
                  if (xt.serialize) {
                    if (He && zt === He[He.length - 1]) throw new Error("statically serialized object won't survive transfer of $name property");
                  } else {
                    for (const ln in Ye) {
                      if (!Ye.hasOwnProperty(ln) || oc[ot].omit.indexOf(ln) >= 0) continue;
                      const pn = Ye[ln];
                      zt[ln] = oc[ot].shallow.indexOf(ln) >= 0 ? pn : hc(pn, He);
                    }
                    Ye instanceof Error && (zt.message = Ye.message);
                  }
                  if (zt.$name) throw new Error("$name property is reserved for worker serialization logic.");
                  return ot !== "Object" && (zt.$name = ot), zt;
                }
                function Vc(Ye) {
                  if (sc(Ye)) return Ye;
                  if (Array.isArray(Ye)) return Ye.map(Vc);
                  if (typeof Ye != "object") throw new Error("can't deserialize object of type " + typeof Ye);
                  const He = Tl(Ye) || "Object";
                  if (!oc[He]) throw new Error(`can't deserialize unregistered class ${He}`);
                  const { klass: ot } = oc[He];
                  if (!ot) throw new Error(`can't deserialize unregistered class ${He}`);
                  if (ot.deserialize) return ot.deserialize(Ye);
                  const xt = Object.create(ot.prototype);
                  for (const zt of Object.keys(Ye)) {
                    if (zt === "$name") continue;
                    const ln = Ye[zt];
                    xt[zt] = oc[He].shallow.indexOf(zt) >= 0 ? ln : Vc(ln);
                  }
                  return xt;
                }
                class Rc {
                  constructor() {
                    this.first = !0;
                  }
                  update(He, ot) {
                    const xt = Math.floor(He);
                    return this.first ? (this.first = !1, this.lastIntegerZoom = xt, this.lastIntegerZoomTime = 0, this.lastZoom = He, this.lastFloorZoom = xt, !0) : (this.lastFloorZoom > xt ? (this.lastIntegerZoom = xt + 1, this.lastIntegerZoomTime = ot) : this.lastFloorZoom < xt && (this.lastIntegerZoom = xt, this.lastIntegerZoomTime = ot), He !== this.lastZoom && (this.lastZoom = He, this.lastFloorZoom = xt, !0));
                  }
                }
                const Zo = { "Latin-1 Supplement": (Ye) => Ye >= 128 && Ye <= 255, Arabic: (Ye) => Ye >= 1536 && Ye <= 1791, "Arabic Supplement": (Ye) => Ye >= 1872 && Ye <= 1919, "Arabic Extended-A": (Ye) => Ye >= 2208 && Ye <= 2303, "Hangul Jamo": (Ye) => Ye >= 4352 && Ye <= 4607, "Unified Canadian Aboriginal Syllabics": (Ye) => Ye >= 5120 && Ye <= 5759, Khmer: (Ye) => Ye >= 6016 && Ye <= 6143, "Unified Canadian Aboriginal Syllabics Extended": (Ye) => Ye >= 6320 && Ye <= 6399, "General Punctuation": (Ye) => Ye >= 8192 && Ye <= 8303, "Letterlike Symbols": (Ye) => Ye >= 8448 && Ye <= 8527, "Number Forms": (Ye) => Ye >= 8528 && Ye <= 8591, "Miscellaneous Technical": (Ye) => Ye >= 8960 && Ye <= 9215, "Control Pictures": (Ye) => Ye >= 9216 && Ye <= 9279, "Optical Character Recognition": (Ye) => Ye >= 9280 && Ye <= 9311, "Enclosed Alphanumerics": (Ye) => Ye >= 9312 && Ye <= 9471, "Geometric Shapes": (Ye) => Ye >= 9632 && Ye <= 9727, "Miscellaneous Symbols": (Ye) => Ye >= 9728 && Ye <= 9983, "Miscellaneous Symbols and Arrows": (Ye) => Ye >= 11008 && Ye <= 11263, "CJK Radicals Supplement": (Ye) => Ye >= 11904 && Ye <= 12031, "Kangxi Radicals": (Ye) => Ye >= 12032 && Ye <= 12255, "Ideographic Description Characters": (Ye) => Ye >= 12272 && Ye <= 12287, "CJK Symbols and Punctuation": (Ye) => Ye >= 12288 && Ye <= 12351, Hiragana: (Ye) => Ye >= 12352 && Ye <= 12447, Katakana: (Ye) => Ye >= 12448 && Ye <= 12543, Bopomofo: (Ye) => Ye >= 12544 && Ye <= 12591, "Hangul Compatibility Jamo": (Ye) => Ye >= 12592 && Ye <= 12687, Kanbun: (Ye) => Ye >= 12688 && Ye <= 12703, "Bopomofo Extended": (Ye) => Ye >= 12704 && Ye <= 12735, "CJK Strokes": (Ye) => Ye >= 12736 && Ye <= 12783, "Katakana Phonetic Extensions": (Ye) => Ye >= 12784 && Ye <= 12799, "Enclosed CJK Letters and Months": (Ye) => Ye >= 12800 && Ye <= 13055, "CJK Compatibility": (Ye) => Ye >= 13056 && Ye <= 13311, "CJK Unified Ideographs Extension A": (Ye) => Ye >= 13312 && Ye <= 19903, "Yijing Hexagram Symbols": (Ye) => Ye >= 19904 && Ye <= 19967, "CJK Unified Ideographs": (Ye) => Ye >= 19968 && Ye <= 40959, "Yi Syllables": (Ye) => Ye >= 40960 && Ye <= 42127, "Yi Radicals": (Ye) => Ye >= 42128 && Ye <= 42191, "Hangul Jamo Extended-A": (Ye) => Ye >= 43360 && Ye <= 43391, "Hangul Syllables": (Ye) => Ye >= 44032 && Ye <= 55215, "Hangul Jamo Extended-B": (Ye) => Ye >= 55216 && Ye <= 55295, "Private Use Area": (Ye) => Ye >= 57344 && Ye <= 63743, "CJK Compatibility Ideographs": (Ye) => Ye >= 63744 && Ye <= 64255, "Arabic Presentation Forms-A": (Ye) => Ye >= 64336 && Ye <= 65023, "Vertical Forms": (Ye) => Ye >= 65040 && Ye <= 65055, "CJK Compatibility Forms": (Ye) => Ye >= 65072 && Ye <= 65103, "Small Form Variants": (Ye) => Ye >= 65104 && Ye <= 65135, "Arabic Presentation Forms-B": (Ye) => Ye >= 65136 && Ye <= 65279, "Halfwidth and Fullwidth Forms": (Ye) => Ye >= 65280 && Ye <= 65519 };
                function Ou(Ye) {
                  for (const He of Ye) if (eu(He.charCodeAt(0))) return !0;
                  return !1;
                }
                function Uc(Ye) {
                  for (const He of Ye) if (!El(He.charCodeAt(0))) return !1;
                  return !0;
                }
                function El(Ye) {
                  return !(Zo.Arabic(Ye) || Zo["Arabic Supplement"](Ye) || Zo["Arabic Extended-A"](Ye) || Zo["Arabic Presentation Forms-A"](Ye) || Zo["Arabic Presentation Forms-B"](Ye));
                }
                function eu(Ye) {
                  return !(Ye !== 746 && Ye !== 747 && (Ye < 4352 || !(Zo["Bopomofo Extended"](Ye) || Zo.Bopomofo(Ye) || Zo["CJK Compatibility Forms"](Ye) && !(Ye >= 65097 && Ye <= 65103) || Zo["CJK Compatibility Ideographs"](Ye) || Zo["CJK Compatibility"](Ye) || Zo["CJK Radicals Supplement"](Ye) || Zo["CJK Strokes"](Ye) || !(!Zo["CJK Symbols and Punctuation"](Ye) || Ye >= 12296 && Ye <= 12305 || Ye >= 12308 && Ye <= 12319 || Ye === 12336) || Zo["CJK Unified Ideographs Extension A"](Ye) || Zo["CJK Unified Ideographs"](Ye) || Zo["Enclosed CJK Letters and Months"](Ye) || Zo["Hangul Compatibility Jamo"](Ye) || Zo["Hangul Jamo Extended-A"](Ye) || Zo["Hangul Jamo Extended-B"](Ye) || Zo["Hangul Jamo"](Ye) || Zo["Hangul Syllables"](Ye) || Zo.Hiragana(Ye) || Zo["Ideographic Description Characters"](Ye) || Zo.Kanbun(Ye) || Zo["Kangxi Radicals"](Ye) || Zo["Katakana Phonetic Extensions"](Ye) || Zo.Katakana(Ye) && Ye !== 12540 || !(!Zo["Halfwidth and Fullwidth Forms"](Ye) || Ye === 65288 || Ye === 65289 || Ye === 65293 || Ye >= 65306 && Ye <= 65310 || Ye === 65339 || Ye === 65341 || Ye === 65343 || Ye >= 65371 && Ye <= 65503 || Ye === 65507 || Ye >= 65512 && Ye <= 65519) || !(!Zo["Small Form Variants"](Ye) || Ye >= 65112 && Ye <= 65118 || Ye >= 65123 && Ye <= 65126) || Zo["Unified Canadian Aboriginal Syllabics"](Ye) || Zo["Unified Canadian Aboriginal Syllabics Extended"](Ye) || Zo["Vertical Forms"](Ye) || Zo["Yijing Hexagram Symbols"](Ye) || Zo["Yi Syllables"](Ye) || Zo["Yi Radicals"](Ye))));
                }
                function Wc(Ye) {
                  return !(eu(Ye) || function(He) {
                    return !!(Zo["Latin-1 Supplement"](He) && (He === 167 || He === 169 || He === 174 || He === 177 || He === 188 || He === 189 || He === 190 || He === 215 || He === 247) || Zo["General Punctuation"](He) && (He === 8214 || He === 8224 || He === 8225 || He === 8240 || He === 8241 || He === 8251 || He === 8252 || He === 8258 || He === 8263 || He === 8264 || He === 8265 || He === 8273) || Zo["Letterlike Symbols"](He) || Zo["Number Forms"](He) || Zo["Miscellaneous Technical"](He) && (He >= 8960 && He <= 8967 || He >= 8972 && He <= 8991 || He >= 8996 && He <= 9e3 || He === 9003 || He >= 9085 && He <= 9114 || He >= 9150 && He <= 9165 || He === 9167 || He >= 9169 && He <= 9179 || He >= 9186 && He <= 9215) || Zo["Control Pictures"](He) && He !== 9251 || Zo["Optical Character Recognition"](He) || Zo["Enclosed Alphanumerics"](He) || Zo["Geometric Shapes"](He) || Zo["Miscellaneous Symbols"](He) && !(He >= 9754 && He <= 9759) || Zo["Miscellaneous Symbols and Arrows"](He) && (He >= 11026 && He <= 11055 || He >= 11088 && He <= 11097 || He >= 11192 && He <= 11243) || Zo["CJK Symbols and Punctuation"](He) || Zo.Katakana(He) || Zo["Private Use Area"](He) || Zo["CJK Compatibility Forms"](He) || Zo["Small Form Variants"](He) || Zo["Halfwidth and Fullwidth Forms"](He) || He === 8734 || He === 8756 || He === 8757 || He >= 9984 && He <= 10087 || He >= 10102 && He <= 10131 || He === 65532 || He === 65533);
                  }(Ye));
                }
                function hu(Ye) {
                  return Zo.Arabic(Ye) || Zo["Arabic Supplement"](Ye) || Zo["Arabic Extended-A"](Ye) || Zo["Arabic Presentation Forms-A"](Ye) || Zo["Arabic Presentation Forms-B"](Ye);
                }
                function gd(Ye) {
                  return Ye >= 1424 && Ye <= 2303 || Zo["Arabic Presentation Forms-A"](Ye) || Zo["Arabic Presentation Forms-B"](Ye);
                }
                function Jl(Ye, He) {
                  return !(!He && gd(Ye) || Ye >= 2304 && Ye <= 3583 || Ye >= 3840 && Ye <= 4255 || Zo.Khmer(Ye));
                }
                function Yc(Ye) {
                  for (const He of Ye) if (gd(He.charCodeAt(0))) return !0;
                  return !1;
                }
                const nc = new class {
                  constructor() {
                    this.applyArabicShaping = null, this.processBidirectionalText = null, this.processStyledBidirectionalText = null, this.pluginStatus = "unavailable", this.pluginURL = null;
                  }
                  setState(Ye) {
                    this.pluginStatus = Ye.pluginStatus, this.pluginURL = Ye.pluginURL;
                  }
                  getState() {
                    return { pluginStatus: this.pluginStatus, pluginURL: this.pluginURL };
                  }
                  setMethods(Ye) {
                    this.applyArabicShaping = Ye.applyArabicShaping, this.processBidirectionalText = Ye.processBidirectionalText, this.processStyledBidirectionalText = Ye.processStyledBidirectionalText;
                  }
                  isParsed() {
                    return this.applyArabicShaping != null && this.processBidirectionalText != null && this.processStyledBidirectionalText != null;
                  }
                  getPluginURL() {
                    return this.pluginURL;
                  }
                  getRTLTextPluginStatus() {
                    return this.pluginStatus;
                  }
                }();
                class Ko {
                  constructor(He, ot) {
                    this.zoom = He, ot ? (this.now = ot.now, this.fadeDuration = ot.fadeDuration, this.zoomHistory = ot.zoomHistory, this.transition = ot.transition) : (this.now = 0, this.fadeDuration = 0, this.zoomHistory = new Rc(), this.transition = {});
                  }
                  isSupportedScript(He) {
                    return function(ot, xt) {
                      for (const zt of ot) if (!Jl(zt.charCodeAt(0), xt)) return !1;
                      return !0;
                    }(He, nc.getRTLTextPluginStatus() === "loaded");
                  }
                  crossFadingFactor() {
                    return this.fadeDuration === 0 ? 1 : Math.min((this.now - this.zoomHistory.lastIntegerZoomTime) / this.fadeDuration, 1);
                  }
                  getCrossfadeParameters() {
                    const He = this.zoom, ot = He - Math.floor(He), xt = this.crossFadingFactor();
                    return He > this.zoomHistory.lastIntegerZoom ? { fromScale: 2, toScale: 1, t: ot + (1 - ot) * xt } : { fromScale: 0.5, toScale: 1, t: 1 - (1 - xt) * ot };
                  }
                }
                class ks {
                  constructor(He, ot) {
                    this.property = He, this.value = ot, this.expression = function(xt, zt) {
                      if (Pa(xt)) return new Fo(xt, zt);
                      if (Vo(xt)) {
                        const ln = Ts(xt, zt);
                        if (ln.result === "error") throw new Error(ln.value.map((pn) => `${pn.key}: ${pn.message}`).join(", "));
                        return ln.value;
                      }
                      {
                        let ln = xt;
                        return zt.type === "color" && typeof xt == "string" ? ln = mr.parse(xt) : zt.type !== "padding" || typeof xt != "number" && !Array.isArray(xt) ? zt.type === "variableAnchorOffsetCollection" && Array.isArray(xt) && (ln = Bi.parse(xt)) : ln = Mr.parse(xt), { kind: "constant", evaluate: () => ln };
                      }
                    }(ot === void 0 ? He.specification.default : ot, He.specification);
                  }
                  isDataDriven() {
                    return this.expression.kind === "source" || this.expression.kind === "composite";
                  }
                  possiblyEvaluate(He, ot, xt) {
                    return this.property.possiblyEvaluate(this, He, ot, xt);
                  }
                }
                class qs {
                  constructor(He) {
                    this.property = He, this.value = new ks(He, void 0);
                  }
                  transitioned(He, ot) {
                    return new $l(this.property, this.value, ot, Ce({}, He.transition, this.transition), He.now);
                  }
                  untransitioned() {
                    return new $l(this.property, this.value, null, {}, 0);
                  }
                }
                class a1 {
                  constructor(He) {
                    this._properties = He, this._values = Object.create(He.defaultTransitionablePropertyValues);
                  }
                  getValue(He) {
                    return xe(this._values[He].value.value);
                  }
                  setValue(He, ot) {
                    Object.prototype.hasOwnProperty.call(this._values, He) || (this._values[He] = new qs(this._values[He].property)), this._values[He].value = new ks(this._values[He].property, ot === null ? void 0 : xe(ot));
                  }
                  getTransition(He) {
                    return xe(this._values[He].transition);
                  }
                  setTransition(He, ot) {
                    Object.prototype.hasOwnProperty.call(this._values, He) || (this._values[He] = new qs(this._values[He].property)), this._values[He].transition = xe(ot) || void 0;
                  }
                  serialize() {
                    const He = {};
                    for (const ot of Object.keys(this._values)) {
                      const xt = this.getValue(ot);
                      xt !== void 0 && (He[ot] = xt);
                      const zt = this.getTransition(ot);
                      zt !== void 0 && (He[`${ot}-transition`] = zt);
                    }
                    return He;
                  }
                  transitioned(He, ot) {
                    const xt = new o1(this._properties);
                    for (const zt of Object.keys(this._values)) xt._values[zt] = this._values[zt].transitioned(He, ot._values[zt]);
                    return xt;
                  }
                  untransitioned() {
                    const He = new o1(this._properties);
                    for (const ot of Object.keys(this._values)) He._values[ot] = this._values[ot].untransitioned();
                    return He;
                  }
                }
                class $l {
                  constructor(He, ot, xt, zt, ln) {
                    this.property = He, this.value = ot, this.begin = ln + zt.delay || 0, this.end = this.begin + zt.duration || 0, He.specification.transition && (zt.delay || zt.duration) && (this.prior = xt);
                  }
                  possiblyEvaluate(He, ot, xt) {
                    const zt = He.now || 0, ln = this.value.possiblyEvaluate(He, ot, xt), pn = this.prior;
                    if (pn) {
                      if (zt > this.end) return this.prior = null, ln;
                      if (this.value.isDataDriven()) return this.prior = null, ln;
                      if (zt < this.begin) return pn.possiblyEvaluate(He, ot, xt);
                      {
                        const Cn = (zt - this.begin) / (this.end - this.begin);
                        return this.property.interpolate(pn.possiblyEvaluate(He, ot, xt), ln, function(Nn) {
                          if (Nn <= 0) return 0;
                          if (Nn >= 1) return 1;
                          const Wn = Nn * Nn, lr = Wn * Nn;
                          return 4 * (Nn < 0.5 ? lr : 3 * (Nn - Wn) + lr - 0.75);
                        }(Cn));
                      }
                    }
                    return ln;
                  }
                }
                class o1 {
                  constructor(He) {
                    this._properties = He, this._values = Object.create(He.defaultTransitioningPropertyValues);
                  }
                  possiblyEvaluate(He, ot, xt) {
                    const zt = new Ul(this._properties);
                    for (const ln of Object.keys(this._values)) zt._values[ln] = this._values[ln].possiblyEvaluate(He, ot, xt);
                    return zt;
                  }
                  hasTransition() {
                    for (const He of Object.keys(this._values)) if (this._values[He].prior) return !0;
                    return !1;
                  }
                }
                class $c {
                  constructor(He) {
                    this._properties = He, this._values = Object.create(He.defaultPropertyValues);
                  }
                  hasValue(He) {
                    return this._values[He].value !== void 0;
                  }
                  getValue(He) {
                    return xe(this._values[He].value);
                  }
                  setValue(He, ot) {
                    this._values[He] = new ks(this._values[He].property, ot === null ? void 0 : xe(ot));
                  }
                  serialize() {
                    const He = {};
                    for (const ot of Object.keys(this._values)) {
                      const xt = this.getValue(ot);
                      xt !== void 0 && (He[ot] = xt);
                    }
                    return He;
                  }
                  possiblyEvaluate(He, ot, xt) {
                    const zt = new Ul(this._properties);
                    for (const ln of Object.keys(this._values)) zt._values[ln] = this._values[ln].possiblyEvaluate(He, ot, xt);
                    return zt;
                  }
                }
                class lu {
                  constructor(He, ot, xt) {
                    this.property = He, this.value = ot, this.parameters = xt;
                  }
                  isConstant() {
                    return this.value.kind === "constant";
                  }
                  constantOr(He) {
                    return this.value.kind === "constant" ? this.value.value : He;
                  }
                  evaluate(He, ot, xt, zt) {
                    return this.property.evaluate(this.value, this.parameters, He, ot, xt, zt);
                  }
                }
                class Ul {
                  constructor(He) {
                    this._properties = He, this._values = Object.create(He.defaultPossiblyEvaluatedValues);
                  }
                  get(He) {
                    return this._values[He];
                  }
                }
                class zs {
                  constructor(He) {
                    this.specification = He;
                  }
                  possiblyEvaluate(He, ot) {
                    if (He.isDataDriven()) throw new Error("Value should not be data driven");
                    return He.expression.evaluate(ot);
                  }
                  interpolate(He, ot, xt) {
                    const zt = this.specification.type, ln = Rr[zt];
                    return ln ? ln(He, ot, xt) : He;
                  }
                }
                class Os {
                  constructor(He, ot) {
                    this.specification = He, this.overrides = ot;
                  }
                  possiblyEvaluate(He, ot, xt, zt) {
                    return He.expression.kind === "constant" || He.expression.kind === "camera" ? new lu(this, { kind: "constant", value: He.expression.evaluate(ot, null, {}, xt, zt) }, ot) : new lu(this, He.expression, ot);
                  }
                  interpolate(He, ot, xt) {
                    if (He.value.kind !== "constant" || ot.value.kind !== "constant") return He;
                    if (He.value.value === void 0 || ot.value.value === void 0) return new lu(this, { kind: "constant", value: void 0 }, He.parameters);
                    const zt = this.specification.type, ln = Rr[zt];
                    if (ln) {
                      const pn = ln(He.value.value, ot.value.value, xt);
                      return new lu(this, { kind: "constant", value: pn }, He.parameters);
                    }
                    return He;
                  }
                  evaluate(He, ot, xt, zt, ln, pn) {
                    return He.kind === "constant" ? He.value : He.evaluate(ot, xt, zt, ln, pn);
                  }
                }
                class lc extends Os {
                  possiblyEvaluate(He, ot, xt, zt) {
                    if (He.value === void 0) return new lu(this, { kind: "constant", value: void 0 }, ot);
                    if (He.expression.kind === "constant") {
                      const ln = He.expression.evaluate(ot, null, {}, xt, zt), pn = He.property.specification.type === "resolvedImage" && typeof ln != "string" ? ln.name : ln, Cn = this._calculate(pn, pn, pn, ot);
                      return new lu(this, { kind: "constant", value: Cn }, ot);
                    }
                    if (He.expression.kind === "camera") {
                      const ln = this._calculate(He.expression.evaluate({ zoom: ot.zoom - 1 }), He.expression.evaluate({ zoom: ot.zoom }), He.expression.evaluate({ zoom: ot.zoom + 1 }), ot);
                      return new lu(this, { kind: "constant", value: ln }, ot);
                    }
                    return new lu(this, He.expression, ot);
                  }
                  evaluate(He, ot, xt, zt, ln, pn) {
                    if (He.kind === "source") {
                      const Cn = He.evaluate(ot, xt, zt, ln, pn);
                      return this._calculate(Cn, Cn, Cn, ot);
                    }
                    return He.kind === "composite" ? this._calculate(He.evaluate({ zoom: Math.floor(ot.zoom) - 1 }, xt, zt), He.evaluate({ zoom: Math.floor(ot.zoom) }, xt, zt), He.evaluate({ zoom: Math.floor(ot.zoom) + 1 }, xt, zt), ot) : He.value;
                  }
                  _calculate(He, ot, xt, zt) {
                    return zt.zoom > zt.zoomHistory.lastIntegerZoom ? { from: He, to: ot } : { from: xt, to: ot };
                  }
                  interpolate(He) {
                    return He;
                  }
                }
                class xc {
                  constructor(He) {
                    this.specification = He;
                  }
                  possiblyEvaluate(He, ot, xt, zt) {
                    if (He.value !== void 0) {
                      if (He.expression.kind === "constant") {
                        const ln = He.expression.evaluate(ot, null, {}, xt, zt);
                        return this._calculate(ln, ln, ln, ot);
                      }
                      return this._calculate(He.expression.evaluate(new Ko(Math.floor(ot.zoom - 1), ot)), He.expression.evaluate(new Ko(Math.floor(ot.zoom), ot)), He.expression.evaluate(new Ko(Math.floor(ot.zoom + 1), ot)), ot);
                    }
                  }
                  _calculate(He, ot, xt, zt) {
                    return zt.zoom > zt.zoomHistory.lastIntegerZoom ? { from: He, to: ot } : { from: xt, to: ot };
                  }
                  interpolate(He) {
                    return He;
                  }
                }
                class Iu {
                  constructor(He) {
                    this.specification = He;
                  }
                  possiblyEvaluate(He, ot, xt, zt) {
                    return !!He.expression.evaluate(ot, null, {}, xt, zt);
                  }
                  interpolate() {
                    return !1;
                  }
                }
                class tu {
                  constructor(He) {
                    this.properties = He, this.defaultPropertyValues = {}, this.defaultTransitionablePropertyValues = {}, this.defaultTransitioningPropertyValues = {}, this.defaultPossiblyEvaluatedValues = {}, this.overridableProperties = [];
                    for (const ot in He) {
                      const xt = He[ot];
                      xt.specification.overridable && this.overridableProperties.push(ot);
                      const zt = this.defaultPropertyValues[ot] = new ks(xt, void 0), ln = this.defaultTransitionablePropertyValues[ot] = new qs(xt);
                      this.defaultTransitioningPropertyValues[ot] = ln.untransitioned(), this.defaultPossiblyEvaluatedValues[ot] = zt.possiblyEvaluate({});
                    }
                  }
                }
                ss("DataDrivenProperty", Os), ss("DataConstantProperty", zs), ss("CrossFadedDataDrivenProperty", lc), ss("CrossFadedProperty", xc), ss("ColorRampProperty", Iu);
                const w1 = "-transition";
                class $n extends bt {
                  constructor(He, ot) {
                    if (super(), this.id = He.id, this.type = He.type, this._featureFilter = { filter: () => !0, needGeometry: !1 }, He.type !== "custom" && (this.metadata = He.metadata, this.minzoom = He.minzoom, this.maxzoom = He.maxzoom, He.type !== "background" && (this.source = He.source, this.sourceLayer = He["source-layer"], this.filter = He.filter), ot.layout && (this._unevaluatedLayout = new $c(ot.layout)), ot.paint)) {
                      this._transitionablePaint = new a1(ot.paint);
                      for (const xt in He.paint) this.setPaintProperty(xt, He.paint[xt], { validate: !1 });
                      for (const xt in He.layout) this.setLayoutProperty(xt, He.layout[xt], { validate: !1 });
                      this._transitioningPaint = this._transitionablePaint.untransitioned(), this.paint = new Ul(ot.paint);
                    }
                  }
                  getCrossfadeParameters() {
                    return this._crossfadeParameters;
                  }
                  getLayoutProperty(He) {
                    return He === "visibility" ? this.visibility : this._unevaluatedLayout.getValue(He);
                  }
                  setLayoutProperty(He, ot, xt = {}) {
                    if (ot != null) {
                      const zt = `layers.${this.id}.layout.${He}`;
                      if (this._validate(bc, zt, He, ot, xt)) return;
                    }
                    He !== "visibility" ? this._unevaluatedLayout.setValue(He, ot) : this.visibility = ot;
                  }
                  getPaintProperty(He) {
                    return He.endsWith(w1) ? this._transitionablePaint.getTransition(He.slice(0, -11)) : this._transitionablePaint.getValue(He);
                  }
                  setPaintProperty(He, ot, xt = {}) {
                    if (ot != null) {
                      const zt = `layers.${this.id}.paint.${He}`;
                      if (this._validate(ac, zt, He, ot, xt)) return !1;
                    }
                    if (He.endsWith(w1)) return this._transitionablePaint.setTransition(He.slice(0, -11), ot || void 0), !1;
                    {
                      const zt = this._transitionablePaint._values[He], ln = zt.property.specification["property-type"] === "cross-faded-data-driven", pn = zt.value.isDataDriven(), Cn = zt.value;
                      this._transitionablePaint.setValue(He, ot), this._handleSpecialPaintPropertyUpdate(He);
                      const Nn = this._transitionablePaint._values[He].value;
                      return Nn.isDataDriven() || pn || ln || this._handleOverridablePaintPropertyUpdate(He, Cn, Nn);
                    }
                  }
                  _handleSpecialPaintPropertyUpdate(He) {
                  }
                  _handleOverridablePaintPropertyUpdate(He, ot, xt) {
                    return !1;
                  }
                  isHidden(He) {
                    return !!(this.minzoom && He < this.minzoom) || !!(this.maxzoom && He >= this.maxzoom) || this.visibility === "none";
                  }
                  updateTransitions(He) {
                    this._transitioningPaint = this._transitionablePaint.transitioned(He, this._transitioningPaint);
                  }
                  hasTransition() {
                    return this._transitioningPaint.hasTransition();
                  }
                  recalculate(He, ot) {
                    He.getCrossfadeParameters && (this._crossfadeParameters = He.getCrossfadeParameters()), this._unevaluatedLayout && (this.layout = this._unevaluatedLayout.possiblyEvaluate(He, void 0, ot)), this.paint = this._transitioningPaint.possiblyEvaluate(He, void 0, ot);
                  }
                  serialize() {
                    const He = { id: this.id, type: this.type, source: this.source, "source-layer": this.sourceLayer, metadata: this.metadata, minzoom: this.minzoom, maxzoom: this.maxzoom, filter: this.filter, layout: this._unevaluatedLayout && this._unevaluatedLayout.serialize(), paint: this._transitionablePaint && this._transitionablePaint.serialize() };
                    return this.visibility && (He.layout = He.layout || {}, He.layout.visibility = this.visibility), Ee(He, (ot, xt) => !(ot === void 0 || xt === "layout" && !Object.keys(ot).length || xt === "paint" && !Object.keys(ot).length));
                  }
                  _validate(He, ot, xt, zt, ln = {}) {
                    return (!ln || ln.validate !== !1) && kc(this, He.call(vl, { key: ot, layerType: this.type, objectKey: xt, value: zt, styleSpec: yt, style: { glyphs: !0, sprite: !0 } }));
                  }
                  is3D() {
                    return !1;
                  }
                  isTileClipped() {
                    return !1;
                  }
                  hasOffscreenPass() {
                    return !1;
                  }
                  resize() {
                  }
                  isStateDependent() {
                    for (const He in this.paint._values) {
                      const ot = this.paint.get(He);
                      if (ot instanceof lu && Ri(ot.property.specification) && (ot.value.kind === "source" || ot.value.kind === "composite") && ot.value.isStateDependent) return !0;
                    }
                    return !1;
                  }
                }
                const zn = { Int8: Int8Array, Uint8: Uint8Array, Int16: Int16Array, Uint16: Uint16Array, Int32: Int32Array, Uint32: Uint32Array, Float32: Float32Array };
                class Gn {
                  constructor(He, ot) {
                    this._structArray = He, this._pos1 = ot * this.size, this._pos2 = this._pos1 / 2, this._pos4 = this._pos1 / 4, this._pos8 = this._pos1 / 8;
                  }
                }
                class Ke {
                  constructor() {
                    this.isTransferred = !1, this.capacity = -1, this.resize(0);
                  }
                  static serialize(He, ot) {
                    return He._trim(), ot && (He.isTransferred = !0, ot.push(He.arrayBuffer)), { length: He.length, arrayBuffer: He.arrayBuffer };
                  }
                  static deserialize(He) {
                    const ot = Object.create(this.prototype);
                    return ot.arrayBuffer = He.arrayBuffer, ot.length = He.length, ot.capacity = He.arrayBuffer.byteLength / ot.bytesPerElement, ot._refreshViews(), ot;
                  }
                  _trim() {
                    this.length !== this.capacity && (this.capacity = this.length, this.arrayBuffer = this.arrayBuffer.slice(0, this.length * this.bytesPerElement), this._refreshViews());
                  }
                  clear() {
                    this.length = 0;
                  }
                  resize(He) {
                    this.reserve(He), this.length = He;
                  }
                  reserve(He) {
                    if (He > this.capacity) {
                      this.capacity = Math.max(He, Math.floor(5 * this.capacity), 128), this.arrayBuffer = new ArrayBuffer(this.capacity * this.bytesPerElement);
                      const ot = this.uint8;
                      this._refreshViews(), ot && this.uint8.set(ot);
                    }
                  }
                  _refreshViews() {
                    throw new Error("_refreshViews() must be implemented by each concrete StructArray layout");
                  }
                }
                function Fe(Ye, He = 1) {
                  let ot = 0, xt = 0;
                  return { members: Ye.map((zt) => {
                    const ln = (Nn = zt.type, zn[Nn].BYTES_PER_ELEMENT), pn = ot = Te(ot, Math.max(He, ln)), Cn = zt.components || 1;
                    var Nn;
                    return xt = Math.max(xt, ln), ot += ln * Cn, { name: zt.name, type: zt.type, components: Cn, offset: pn };
                  }), size: Te(ot, Math.max(xt, He)), alignment: He };
                }
                function Te(Ye, He) {
                  return Math.ceil(Ye / He) * He;
                }
                class Ae extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot) {
                    const xt = this.length;
                    return this.resize(xt + 1), this.emplace(xt, He, ot);
                  }
                  emplace(He, ot, xt) {
                    const zt = 2 * He;
                    return this.int16[zt + 0] = ot, this.int16[zt + 1] = xt, He;
                  }
                }
                Ae.prototype.bytesPerElement = 4, ss("StructArrayLayout2i4", Ae);
                class Oe extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt) {
                    const zt = this.length;
                    return this.resize(zt + 1), this.emplace(zt, He, ot, xt);
                  }
                  emplace(He, ot, xt, zt) {
                    const ln = 3 * He;
                    return this.int16[ln + 0] = ot, this.int16[ln + 1] = xt, this.int16[ln + 2] = zt, He;
                  }
                }
                Oe.prototype.bytesPerElement = 6, ss("StructArrayLayout3i6", Oe);
                class Be extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt) {
                    const ln = this.length;
                    return this.resize(ln + 1), this.emplace(ln, He, ot, xt, zt);
                  }
                  emplace(He, ot, xt, zt, ln) {
                    const pn = 4 * He;
                    return this.int16[pn + 0] = ot, this.int16[pn + 1] = xt, this.int16[pn + 2] = zt, this.int16[pn + 3] = ln, He;
                  }
                }
                Be.prototype.bytesPerElement = 8, ss("StructArrayLayout4i8", Be);
                class Ue extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn) {
                    const Cn = this.length;
                    return this.resize(Cn + 1), this.emplace(Cn, He, ot, xt, zt, ln, pn);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn) {
                    const Nn = 6 * He;
                    return this.int16[Nn + 0] = ot, this.int16[Nn + 1] = xt, this.int16[Nn + 2] = zt, this.int16[Nn + 3] = ln, this.int16[Nn + 4] = pn, this.int16[Nn + 5] = Cn, He;
                  }
                }
                Ue.prototype.bytesPerElement = 12, ss("StructArrayLayout2i4i12", Ue);
                class nt extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn) {
                    const Cn = this.length;
                    return this.resize(Cn + 1), this.emplace(Cn, He, ot, xt, zt, ln, pn);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn) {
                    const Nn = 4 * He, Wn = 8 * He;
                    return this.int16[Nn + 0] = ot, this.int16[Nn + 1] = xt, this.uint8[Wn + 4] = zt, this.uint8[Wn + 5] = ln, this.uint8[Wn + 6] = pn, this.uint8[Wn + 7] = Cn, He;
                  }
                }
                nt.prototype.bytesPerElement = 8, ss("StructArrayLayout2i4ub8", nt);
                class ht extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot) {
                    const xt = this.length;
                    return this.resize(xt + 1), this.emplace(xt, He, ot);
                  }
                  emplace(He, ot, xt) {
                    const zt = 2 * He;
                    return this.float32[zt + 0] = ot, this.float32[zt + 1] = xt, He;
                  }
                }
                ht.prototype.bytesPerElement = 8, ss("StructArrayLayout2f8", ht);
                class Et extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr) {
                    const xr = this.length;
                    return this.resize(xr + 1), this.emplace(xr, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr) {
                    const Nr = 10 * He;
                    return this.uint16[Nr + 0] = ot, this.uint16[Nr + 1] = xt, this.uint16[Nr + 2] = zt, this.uint16[Nr + 3] = ln, this.uint16[Nr + 4] = pn, this.uint16[Nr + 5] = Cn, this.uint16[Nr + 6] = Nn, this.uint16[Nr + 7] = Wn, this.uint16[Nr + 8] = lr, this.uint16[Nr + 9] = xr, He;
                  }
                }
                Et.prototype.bytesPerElement = 20, ss("StructArrayLayout10ui20", Et);
                class Mt extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr) {
                    const Kr = this.length;
                    return this.resize(Kr + 1), this.emplace(Kr, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr) {
                    const xi = 12 * He;
                    return this.int16[xi + 0] = ot, this.int16[xi + 1] = xt, this.int16[xi + 2] = zt, this.int16[xi + 3] = ln, this.uint16[xi + 4] = pn, this.uint16[xi + 5] = Cn, this.uint16[xi + 6] = Nn, this.uint16[xi + 7] = Wn, this.int16[xi + 8] = lr, this.int16[xi + 9] = xr, this.int16[xi + 10] = Nr, this.int16[xi + 11] = Kr, He;
                  }
                }
                Mt.prototype.bytesPerElement = 24, ss("StructArrayLayout4i4ui4i24", Mt);
                class qt extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt) {
                    const zt = this.length;
                    return this.resize(zt + 1), this.emplace(zt, He, ot, xt);
                  }
                  emplace(He, ot, xt, zt) {
                    const ln = 3 * He;
                    return this.float32[ln + 0] = ot, this.float32[ln + 1] = xt, this.float32[ln + 2] = zt, He;
                  }
                }
                qt.prototype.bytesPerElement = 12, ss("StructArrayLayout3f12", qt);
                class tn extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer);
                  }
                  emplaceBack(He) {
                    const ot = this.length;
                    return this.resize(ot + 1), this.emplace(ot, He);
                  }
                  emplace(He, ot) {
                    const xt = 1 * He;
                    return this.uint32[xt + 0] = ot, He;
                  }
                }
                tn.prototype.bytesPerElement = 4, ss("StructArrayLayout1ul4", tn);
                class xn extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn, Cn, Nn, Wn) {
                    const lr = this.length;
                    return this.resize(lr + 1), this.emplace(lr, He, ot, xt, zt, ln, pn, Cn, Nn, Wn);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr) {
                    const xr = 10 * He, Nr = 5 * He;
                    return this.int16[xr + 0] = ot, this.int16[xr + 1] = xt, this.int16[xr + 2] = zt, this.int16[xr + 3] = ln, this.int16[xr + 4] = pn, this.int16[xr + 5] = Cn, this.uint32[Nr + 3] = Nn, this.uint16[xr + 8] = Wn, this.uint16[xr + 9] = lr, He;
                  }
                }
                xn.prototype.bytesPerElement = 20, ss("StructArrayLayout6i1ul2ui20", xn);
                class An extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn) {
                    const Cn = this.length;
                    return this.resize(Cn + 1), this.emplace(Cn, He, ot, xt, zt, ln, pn);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn) {
                    const Nn = 6 * He;
                    return this.int16[Nn + 0] = ot, this.int16[Nn + 1] = xt, this.int16[Nn + 2] = zt, this.int16[Nn + 3] = ln, this.int16[Nn + 4] = pn, this.int16[Nn + 5] = Cn, He;
                  }
                }
                An.prototype.bytesPerElement = 12, ss("StructArrayLayout2i2i2i12", An);
                class On extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln) {
                    const pn = this.length;
                    return this.resize(pn + 1), this.emplace(pn, He, ot, xt, zt, ln);
                  }
                  emplace(He, ot, xt, zt, ln, pn) {
                    const Cn = 4 * He, Nn = 8 * He;
                    return this.float32[Cn + 0] = ot, this.float32[Cn + 1] = xt, this.float32[Cn + 2] = zt, this.int16[Nn + 6] = ln, this.int16[Nn + 7] = pn, He;
                  }
                }
                On.prototype.bytesPerElement = 16, ss("StructArrayLayout2f1f2i16", On);
                class Bn extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn) {
                    const Cn = this.length;
                    return this.resize(Cn + 1), this.emplace(Cn, He, ot, xt, zt, ln, pn);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn) {
                    const Nn = 16 * He, Wn = 4 * He, lr = 8 * He;
                    return this.uint8[Nn + 0] = ot, this.uint8[Nn + 1] = xt, this.float32[Wn + 1] = zt, this.float32[Wn + 2] = ln, this.int16[lr + 6] = pn, this.int16[lr + 7] = Cn, He;
                  }
                }
                Bn.prototype.bytesPerElement = 16, ss("StructArrayLayout2ub2f2i16", Bn);
                class qn extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt) {
                    const zt = this.length;
                    return this.resize(zt + 1), this.emplace(zt, He, ot, xt);
                  }
                  emplace(He, ot, xt, zt) {
                    const ln = 3 * He;
                    return this.uint16[ln + 0] = ot, this.uint16[ln + 1] = xt, this.uint16[ln + 2] = zt, He;
                  }
                }
                qn.prototype.bytesPerElement = 6, ss("StructArrayLayout3ui6", qn);
                class Vn extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui, Ui, ia) {
                    const Na = this.length;
                    return this.resize(Na + 1), this.emplace(Na, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui, Ui, ia);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui, Ui, ia, Na) {
                    const Ta = 24 * He, Ra = 12 * He, oo = 48 * He;
                    return this.int16[Ta + 0] = ot, this.int16[Ta + 1] = xt, this.uint16[Ta + 2] = zt, this.uint16[Ta + 3] = ln, this.uint32[Ra + 2] = pn, this.uint32[Ra + 3] = Cn, this.uint32[Ra + 4] = Nn, this.uint16[Ta + 10] = Wn, this.uint16[Ta + 11] = lr, this.uint16[Ta + 12] = xr, this.float32[Ra + 7] = Nr, this.float32[Ra + 8] = Kr, this.uint8[oo + 36] = xi, this.uint8[oo + 37] = ui, this.uint8[oo + 38] = Ui, this.uint32[Ra + 10] = ia, this.int16[Ta + 22] = Na, He;
                  }
                }
                Vn.prototype.bytesPerElement = 48, ss("StructArrayLayout2i2ui3ul3ui2f3ub1ul1i48", Vn);
                class sr extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.int16 = new Int16Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui, Ui, ia, Na, Ta, Ra, oo, fs, Bs, fl, Ns, Es, cl, Xs) {
                    const Ps = this.length;
                    return this.resize(Ps + 1), this.emplace(Ps, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui, Ui, ia, Na, Ta, Ra, oo, fs, Bs, fl, Ns, Es, cl, Xs);
                  }
                  emplace(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui, Ui, ia, Na, Ta, Ra, oo, fs, Bs, fl, Ns, Es, cl, Xs, Ps) {
                    const ko = 32 * He, Ks = 16 * He;
                    return this.int16[ko + 0] = ot, this.int16[ko + 1] = xt, this.int16[ko + 2] = zt, this.int16[ko + 3] = ln, this.int16[ko + 4] = pn, this.int16[ko + 5] = Cn, this.int16[ko + 6] = Nn, this.int16[ko + 7] = Wn, this.uint16[ko + 8] = lr, this.uint16[ko + 9] = xr, this.uint16[ko + 10] = Nr, this.uint16[ko + 11] = Kr, this.uint16[ko + 12] = xi, this.uint16[ko + 13] = ui, this.uint16[ko + 14] = Ui, this.uint16[ko + 15] = ia, this.uint16[ko + 16] = Na, this.uint16[ko + 17] = Ta, this.uint16[ko + 18] = Ra, this.uint16[ko + 19] = oo, this.uint16[ko + 20] = fs, this.uint16[ko + 21] = Bs, this.uint16[ko + 22] = fl, this.uint32[Ks + 12] = Ns, this.float32[Ks + 13] = Es, this.float32[Ks + 14] = cl, this.uint16[ko + 30] = Xs, this.uint16[ko + 31] = Ps, He;
                  }
                }
                sr.prototype.bytesPerElement = 64, ss("StructArrayLayout8i15ui1ul2f2ui64", sr);
                class Un extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }
                  emplaceBack(He) {
                    const ot = this.length;
                    return this.resize(ot + 1), this.emplace(ot, He);
                  }
                  emplace(He, ot) {
                    const xt = 1 * He;
                    return this.float32[xt + 0] = ot, He;
                  }
                }
                Un.prototype.bytesPerElement = 4, ss("StructArrayLayout1f4", Un);
                class wr extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt) {
                    const zt = this.length;
                    return this.resize(zt + 1), this.emplace(zt, He, ot, xt);
                  }
                  emplace(He, ot, xt, zt) {
                    const ln = 6 * He, pn = 3 * He;
                    return this.uint16[ln + 0] = ot, this.float32[pn + 1] = xt, this.float32[pn + 2] = zt, He;
                  }
                }
                wr.prototype.bytesPerElement = 12, ss("StructArrayLayout1ui2f12", wr);
                class Cr extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint32 = new Uint32Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt) {
                    const zt = this.length;
                    return this.resize(zt + 1), this.emplace(zt, He, ot, xt);
                  }
                  emplace(He, ot, xt, zt) {
                    const ln = 2 * He, pn = 4 * He;
                    return this.uint32[ln + 0] = ot, this.uint16[pn + 2] = xt, this.uint16[pn + 3] = zt, He;
                  }
                }
                Cr.prototype.bytesPerElement = 8, ss("StructArrayLayout1ul2ui8", Cr);
                class Tr extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot) {
                    const xt = this.length;
                    return this.resize(xt + 1), this.emplace(xt, He, ot);
                  }
                  emplace(He, ot, xt) {
                    const zt = 2 * He;
                    return this.uint16[zt + 0] = ot, this.uint16[zt + 1] = xt, He;
                  }
                }
                Tr.prototype.bytesPerElement = 4, ss("StructArrayLayout2ui4", Tr);
                class ii extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.uint16 = new Uint16Array(this.arrayBuffer);
                  }
                  emplaceBack(He) {
                    const ot = this.length;
                    return this.resize(ot + 1), this.emplace(ot, He);
                  }
                  emplace(He, ot) {
                    const xt = 1 * He;
                    return this.uint16[xt + 0] = ot, He;
                  }
                }
                ii.prototype.bytesPerElement = 2, ss("StructArrayLayout1ui2", ii);
                class wi extends Ke {
                  _refreshViews() {
                    this.uint8 = new Uint8Array(this.arrayBuffer), this.float32 = new Float32Array(this.arrayBuffer);
                  }
                  emplaceBack(He, ot, xt, zt) {
                    const ln = this.length;
                    return this.resize(ln + 1), this.emplace(ln, He, ot, xt, zt);
                  }
                  emplace(He, ot, xt, zt, ln) {
                    const pn = 4 * He;
                    return this.float32[pn + 0] = ot, this.float32[pn + 1] = xt, this.float32[pn + 2] = zt, this.float32[pn + 3] = ln, He;
                  }
                }
                wi.prototype.bytesPerElement = 16, ss("StructArrayLayout4f16", wi);
                class Wi extends Gn {
                  get anchorPointX() {
                    return this._structArray.int16[this._pos2 + 0];
                  }
                  get anchorPointY() {
                    return this._structArray.int16[this._pos2 + 1];
                  }
                  get x1() {
                    return this._structArray.int16[this._pos2 + 2];
                  }
                  get y1() {
                    return this._structArray.int16[this._pos2 + 3];
                  }
                  get x2() {
                    return this._structArray.int16[this._pos2 + 4];
                  }
                  get y2() {
                    return this._structArray.int16[this._pos2 + 5];
                  }
                  get featureIndex() {
                    return this._structArray.uint32[this._pos4 + 3];
                  }
                  get sourceLayerIndex() {
                    return this._structArray.uint16[this._pos2 + 8];
                  }
                  get bucketIndex() {
                    return this._structArray.uint16[this._pos2 + 9];
                  }
                  get anchorPoint() {
                    return new ce(this.anchorPointX, this.anchorPointY);
                  }
                }
                Wi.prototype.size = 20;
                class Yi extends xn {
                  get(He) {
                    return new Wi(this, He);
                  }
                }
                ss("CollisionBoxArray", Yi);
                class ra extends Gn {
                  get anchorX() {
                    return this._structArray.int16[this._pos2 + 0];
                  }
                  get anchorY() {
                    return this._structArray.int16[this._pos2 + 1];
                  }
                  get glyphStartIndex() {
                    return this._structArray.uint16[this._pos2 + 2];
                  }
                  get numGlyphs() {
                    return this._structArray.uint16[this._pos2 + 3];
                  }
                  get vertexStartIndex() {
                    return this._structArray.uint32[this._pos4 + 2];
                  }
                  get lineStartIndex() {
                    return this._structArray.uint32[this._pos4 + 3];
                  }
                  get lineLength() {
                    return this._structArray.uint32[this._pos4 + 4];
                  }
                  get segment() {
                    return this._structArray.uint16[this._pos2 + 10];
                  }
                  get lowerSize() {
                    return this._structArray.uint16[this._pos2 + 11];
                  }
                  get upperSize() {
                    return this._structArray.uint16[this._pos2 + 12];
                  }
                  get lineOffsetX() {
                    return this._structArray.float32[this._pos4 + 7];
                  }
                  get lineOffsetY() {
                    return this._structArray.float32[this._pos4 + 8];
                  }
                  get writingMode() {
                    return this._structArray.uint8[this._pos1 + 36];
                  }
                  get placedOrientation() {
                    return this._structArray.uint8[this._pos1 + 37];
                  }
                  set placedOrientation(He) {
                    this._structArray.uint8[this._pos1 + 37] = He;
                  }
                  get hidden() {
                    return this._structArray.uint8[this._pos1 + 38];
                  }
                  set hidden(He) {
                    this._structArray.uint8[this._pos1 + 38] = He;
                  }
                  get crossTileID() {
                    return this._structArray.uint32[this._pos4 + 10];
                  }
                  set crossTileID(He) {
                    this._structArray.uint32[this._pos4 + 10] = He;
                  }
                  get associatedIconIndex() {
                    return this._structArray.int16[this._pos2 + 22];
                  }
                }
                ra.prototype.size = 48;
                class ha extends Vn {
                  get(He) {
                    return new ra(this, He);
                  }
                }
                ss("PlacedSymbolArray", ha);
                class ya extends Gn {
                  get anchorX() {
                    return this._structArray.int16[this._pos2 + 0];
                  }
                  get anchorY() {
                    return this._structArray.int16[this._pos2 + 1];
                  }
                  get rightJustifiedTextSymbolIndex() {
                    return this._structArray.int16[this._pos2 + 2];
                  }
                  get centerJustifiedTextSymbolIndex() {
                    return this._structArray.int16[this._pos2 + 3];
                  }
                  get leftJustifiedTextSymbolIndex() {
                    return this._structArray.int16[this._pos2 + 4];
                  }
                  get verticalPlacedTextSymbolIndex() {
                    return this._structArray.int16[this._pos2 + 5];
                  }
                  get placedIconSymbolIndex() {
                    return this._structArray.int16[this._pos2 + 6];
                  }
                  get verticalPlacedIconSymbolIndex() {
                    return this._structArray.int16[this._pos2 + 7];
                  }
                  get key() {
                    return this._structArray.uint16[this._pos2 + 8];
                  }
                  get textBoxStartIndex() {
                    return this._structArray.uint16[this._pos2 + 9];
                  }
                  get textBoxEndIndex() {
                    return this._structArray.uint16[this._pos2 + 10];
                  }
                  get verticalTextBoxStartIndex() {
                    return this._structArray.uint16[this._pos2 + 11];
                  }
                  get verticalTextBoxEndIndex() {
                    return this._structArray.uint16[this._pos2 + 12];
                  }
                  get iconBoxStartIndex() {
                    return this._structArray.uint16[this._pos2 + 13];
                  }
                  get iconBoxEndIndex() {
                    return this._structArray.uint16[this._pos2 + 14];
                  }
                  get verticalIconBoxStartIndex() {
                    return this._structArray.uint16[this._pos2 + 15];
                  }
                  get verticalIconBoxEndIndex() {
                    return this._structArray.uint16[this._pos2 + 16];
                  }
                  get featureIndex() {
                    return this._structArray.uint16[this._pos2 + 17];
                  }
                  get numHorizontalGlyphVertices() {
                    return this._structArray.uint16[this._pos2 + 18];
                  }
                  get numVerticalGlyphVertices() {
                    return this._structArray.uint16[this._pos2 + 19];
                  }
                  get numIconVertices() {
                    return this._structArray.uint16[this._pos2 + 20];
                  }
                  get numVerticalIconVertices() {
                    return this._structArray.uint16[this._pos2 + 21];
                  }
                  get useRuntimeCollisionCircles() {
                    return this._structArray.uint16[this._pos2 + 22];
                  }
                  get crossTileID() {
                    return this._structArray.uint32[this._pos4 + 12];
                  }
                  set crossTileID(He) {
                    this._structArray.uint32[this._pos4 + 12] = He;
                  }
                  get textBoxScale() {
                    return this._structArray.float32[this._pos4 + 13];
                  }
                  get collisionCircleDiameter() {
                    return this._structArray.float32[this._pos4 + 14];
                  }
                  get textAnchorOffsetStartIndex() {
                    return this._structArray.uint16[this._pos2 + 30];
                  }
                  get textAnchorOffsetEndIndex() {
                    return this._structArray.uint16[this._pos2 + 31];
                  }
                }
                ya.prototype.size = 64;
                class va extends sr {
                  get(He) {
                    return new ya(this, He);
                  }
                }
                ss("SymbolInstanceArray", va);
                class qa extends Un {
                  getoffsetX(He) {
                    return this.float32[1 * He + 0];
                  }
                }
                ss("GlyphOffsetArray", qa);
                class Aa extends Oe {
                  getx(He) {
                    return this.int16[3 * He + 0];
                  }
                  gety(He) {
                    return this.int16[3 * He + 1];
                  }
                  gettileUnitDistanceFromAnchor(He) {
                    return this.int16[3 * He + 2];
                  }
                }
                ss("SymbolLineVertexArray", Aa);
                class Xa extends Gn {
                  get textAnchor() {
                    return this._structArray.uint16[this._pos2 + 0];
                  }
                  get textOffset0() {
                    return this._structArray.float32[this._pos4 + 1];
                  }
                  get textOffset1() {
                    return this._structArray.float32[this._pos4 + 2];
                  }
                }
                Xa.prototype.size = 12;
                class Oa extends wr {
                  get(He) {
                    return new Xa(this, He);
                  }
                }
                ss("TextAnchorOffsetArray", Oa);
                class Ya extends Gn {
                  get featureIndex() {
                    return this._structArray.uint32[this._pos4 + 0];
                  }
                  get sourceLayerIndex() {
                    return this._structArray.uint16[this._pos2 + 2];
                  }
                  get bucketIndex() {
                    return this._structArray.uint16[this._pos2 + 3];
                  }
                }
                Ya.prototype.size = 8;
                class Wo extends Cr {
                  get(He) {
                    return new Ya(this, He);
                  }
                }
                ss("FeatureIndexArray", Wo);
                class Ja extends Ae {
                }
                class Uo extends Ae {
                }
                class ys extends Ae {
                }
                class il extends Ue {
                }
                class Is extends nt {
                }
                class ml extends ht {
                }
                class nl extends Et {
                }
                class Il extends Mt {
                }
                class Fl extends qt {
                }
                class qc extends tn {
                }
                class ql extends An {
                }
                class jc extends Bn {
                }
                class Dl extends qn {
                }
                class Ml extends Tr {
                }
                const pc = Fe([{ name: "a_pos", components: 2, type: "Int16" }], 4), { members: hl, size: xl, alignment: Bl } = pc;
                class al {
                  constructor(He = []) {
                    this.segments = He;
                  }
                  prepareSegment(He, ot, xt, zt) {
                    let ln = this.segments[this.segments.length - 1];
                    return He > al.MAX_VERTEX_ARRAY_LENGTH && Qe(`Max vertices per segment is ${al.MAX_VERTEX_ARRAY_LENGTH}: bucket requested ${He}`), (!ln || ln.vertexLength + He > al.MAX_VERTEX_ARRAY_LENGTH || ln.sortKey !== zt) && (ln = { vertexOffset: ot.length, primitiveOffset: xt.length, vertexLength: 0, primitiveLength: 0 }, zt !== void 0 && (ln.sortKey = zt), this.segments.push(ln)), ln;
                  }
                  get() {
                    return this.segments;
                  }
                  destroy() {
                    for (const He of this.segments) for (const ot in He.vaos) He.vaos[ot].destroy();
                  }
                  static simpleSegment(He, ot, xt, zt) {
                    return new al([{ vertexOffset: He, primitiveOffset: ot, vertexLength: xt, primitiveLength: zt, vaos: {}, sortKey: 0 }]);
                  }
                }
                function Oc(Ye, He) {
                  return 256 * (Ye = ve(Math.floor(Ye), 0, 255)) + ve(Math.floor(He), 0, 255);
                }
                al.MAX_VERTEX_ARRAY_LENGTH = Math.pow(2, 16) - 1, ss("SegmentVector", al);
                const nu = Fe([{ name: "a_pattern_from", components: 4, type: "Uint16" }, { name: "a_pattern_to", components: 4, type: "Uint16" }, { name: "a_pixel_ratio_from", components: 1, type: "Uint16" }, { name: "a_pixel_ratio_to", components: 1, type: "Uint16" }]);
                var Ic = { exports: {} }, $s = { exports: {} };
                (function(Ye) {
                  Ye.exports = function(He, ot) {
                    var xt, zt, ln, pn, Cn, Nn, Wn, lr;
                    for (xt = 3 & He.length, zt = He.length - xt, ln = ot, Cn = 3432918353, Nn = 461845907, lr = 0; lr < zt; ) Wn = 255 & He.charCodeAt(lr) | (255 & He.charCodeAt(++lr)) << 8 | (255 & He.charCodeAt(++lr)) << 16 | (255 & He.charCodeAt(++lr)) << 24, ++lr, ln = 27492 + (65535 & (pn = 5 * (65535 & (ln = (ln ^= Wn = (65535 & (Wn = (Wn = (65535 & Wn) * Cn + (((Wn >>> 16) * Cn & 65535) << 16) & 4294967295) << 15 | Wn >>> 17)) * Nn + (((Wn >>> 16) * Nn & 65535) << 16) & 4294967295) << 13 | ln >>> 19)) + ((5 * (ln >>> 16) & 65535) << 16) & 4294967295)) + ((58964 + (pn >>> 16) & 65535) << 16);
                    switch (Wn = 0, xt) {
                      case 3:
                        Wn ^= (255 & He.charCodeAt(lr + 2)) << 16;
                      case 2:
                        Wn ^= (255 & He.charCodeAt(lr + 1)) << 8;
                      case 1:
                        ln ^= Wn = (65535 & (Wn = (Wn = (65535 & (Wn ^= 255 & He.charCodeAt(lr))) * Cn + (((Wn >>> 16) * Cn & 65535) << 16) & 4294967295) << 15 | Wn >>> 17)) * Nn + (((Wn >>> 16) * Nn & 65535) << 16) & 4294967295;
                    }
                    return ln ^= He.length, ln = 2246822507 * (65535 & (ln ^= ln >>> 16)) + ((2246822507 * (ln >>> 16) & 65535) << 16) & 4294967295, ln = 3266489909 * (65535 & (ln ^= ln >>> 13)) + ((3266489909 * (ln >>> 16) & 65535) << 16) & 4294967295, (ln ^= ln >>> 16) >>> 0;
                  };
                })($s);
                var Xc = $s.exports, wc = { exports: {} };
                (function(Ye) {
                  Ye.exports = function(He, ot) {
                    for (var xt, zt = He.length, ln = ot ^ zt, pn = 0; zt >= 4; ) xt = 1540483477 * (65535 & (xt = 255 & He.charCodeAt(pn) | (255 & He.charCodeAt(++pn)) << 8 | (255 & He.charCodeAt(++pn)) << 16 | (255 & He.charCodeAt(++pn)) << 24)) + ((1540483477 * (xt >>> 16) & 65535) << 16), ln = 1540483477 * (65535 & ln) + ((1540483477 * (ln >>> 16) & 65535) << 16) ^ (xt = 1540483477 * (65535 & (xt ^= xt >>> 24)) + ((1540483477 * (xt >>> 16) & 65535) << 16)), zt -= 4, ++pn;
                    switch (zt) {
                      case 3:
                        ln ^= (255 & He.charCodeAt(pn + 2)) << 16;
                      case 2:
                        ln ^= (255 & He.charCodeAt(pn + 1)) << 8;
                      case 1:
                        ln = 1540483477 * (65535 & (ln ^= 255 & He.charCodeAt(pn))) + ((1540483477 * (ln >>> 16) & 65535) << 16);
                    }
                    return ln = 1540483477 * (65535 & (ln ^= ln >>> 13)) + ((1540483477 * (ln >>> 16) & 65535) << 16), (ln ^= ln >>> 15) >>> 0;
                  };
                })(wc);
                var zu = Xc, Zc = wc.exports;
                Ic.exports = zu, Ic.exports.murmur3 = zu, Ic.exports.murmur2 = Zc;
                var Ju = ae(Ic.exports);
                class ru {
                  constructor() {
                    this.ids = [], this.positions = [], this.indexed = !1;
                  }
                  add(He, ot, xt, zt) {
                    this.ids.push(Bd(He)), this.positions.push(ot, xt, zt);
                  }
                  getPositions(He) {
                    if (!this.indexed) throw new Error("Trying to get index, but feature positions are not indexed");
                    const ot = Bd(He);
                    let xt = 0, zt = this.ids.length - 1;
                    for (; xt < zt; ) {
                      const pn = xt + zt >> 1;
                      this.ids[pn] >= ot ? zt = pn : xt = pn + 1;
                    }
                    const ln = [];
                    for (; this.ids[xt] === ot; ) {
                      const pn = this.positions[3 * xt], Cn = this.positions[3 * xt + 1], Nn = this.positions[3 * xt + 2];
                      ln.push({ index: pn, start: Cn, end: Nn }), xt++;
                    }
                    return ln;
                  }
                  static serialize(He, ot) {
                    const xt = new Float64Array(He.ids), zt = new Uint32Array(He.positions);
                    return tf(xt, zt, 0, xt.length - 1), ot && ot.push(xt.buffer, zt.buffer), { ids: xt, positions: zt };
                  }
                  static deserialize(He) {
                    const ot = new ru();
                    return ot.ids = He.ids, ot.positions = He.positions, ot.indexed = !0, ot;
                  }
                }
                function Bd(Ye) {
                  const He = +Ye;
                  return !isNaN(He) && He <= Number.MAX_SAFE_INTEGER ? He : Ju(String(Ye));
                }
                function tf(Ye, He, ot, xt) {
                  for (; ot < xt; ) {
                    const zt = Ye[ot + xt >> 1];
                    let ln = ot - 1, pn = xt + 1;
                    for (; ; ) {
                      do
                        ln++;
                      while (Ye[ln] < zt);
                      do
                        pn--;
                      while (Ye[pn] > zt);
                      if (ln >= pn) break;
                      wd(Ye, ln, pn), wd(He, 3 * ln, 3 * pn), wd(He, 3 * ln + 1, 3 * pn + 1), wd(He, 3 * ln + 2, 3 * pn + 2);
                    }
                    pn - ot < xt - pn ? (tf(Ye, He, ot, pn), ot = pn + 1) : (tf(Ye, He, pn + 1, xt), xt = pn);
                  }
                }
                function wd(Ye, He, ot) {
                  const xt = Ye[He];
                  Ye[He] = Ye[ot], Ye[ot] = xt;
                }
                ss("FeaturePositionMap", ru);
                class od {
                  constructor(He, ot) {
                    this.gl = He.gl, this.location = ot;
                  }
                }
                class yd extends od {
                  constructor(He, ot) {
                    super(He, ot), this.current = 0;
                  }
                  set(He) {
                    this.current !== He && (this.current = He, this.gl.uniform1f(this.location, He));
                  }
                }
                class Ef extends od {
                  constructor(He, ot) {
                    super(He, ot), this.current = [0, 0, 0, 0];
                  }
                  set(He) {
                    He[0] === this.current[0] && He[1] === this.current[1] && He[2] === this.current[2] && He[3] === this.current[3] || (this.current = He, this.gl.uniform4f(this.location, He[0], He[1], He[2], He[3]));
                  }
                }
                class Md extends od {
                  constructor(He, ot) {
                    super(He, ot), this.current = mr.transparent;
                  }
                  set(He) {
                    He.r === this.current.r && He.g === this.current.g && He.b === this.current.b && He.a === this.current.a || (this.current = He, this.gl.uniform4f(this.location, He.r, He.g, He.b, He.a));
                  }
                }
                const nf = new Float32Array(16);
                function Vd(Ye) {
                  return [Oc(255 * Ye.r, 255 * Ye.g), Oc(255 * Ye.b, 255 * Ye.a)];
                }
                class S1 {
                  constructor(He, ot, xt) {
                    this.value = He, this.uniformNames = ot.map((zt) => `u_${zt}`), this.type = xt;
                  }
                  setUniform(He, ot, xt) {
                    He.set(xt.constantOr(this.value));
                  }
                  getBinding(He, ot, xt) {
                    return this.type === "color" ? new Md(He, ot) : new yd(He, ot);
                  }
                }
                class Af {
                  constructor(He, ot) {
                    this.uniformNames = ot.map((xt) => `u_${xt}`), this.patternFrom = null, this.patternTo = null, this.pixelRatioFrom = 1, this.pixelRatioTo = 1;
                  }
                  setConstantPatternPositions(He, ot) {
                    this.pixelRatioFrom = ot.pixelRatio, this.pixelRatioTo = He.pixelRatio, this.patternFrom = ot.tlbr, this.patternTo = He.tlbr;
                  }
                  setUniform(He, ot, xt, zt) {
                    const ln = zt === "u_pattern_to" ? this.patternTo : zt === "u_pattern_from" ? this.patternFrom : zt === "u_pixel_ratio_to" ? this.pixelRatioTo : zt === "u_pixel_ratio_from" ? this.pixelRatioFrom : null;
                    ln && He.set(ln);
                  }
                  getBinding(He, ot, xt) {
                    return xt.substr(0, 9) === "u_pattern" ? new Ef(He, ot) : new yd(He, ot);
                  }
                }
                class Pd {
                  constructor(He, ot, xt, zt) {
                    this.expression = He, this.type = xt, this.maxValue = 0, this.paintVertexAttributes = ot.map((ln) => ({ name: `a_${ln}`, type: "Float32", components: xt === "color" ? 2 : 1, offset: 0 })), this.paintVertexArray = new zt();
                  }
                  populatePaintArray(He, ot, xt, zt, ln) {
                    const pn = this.paintVertexArray.length, Cn = this.expression.evaluate(new Ko(0), ot, {}, zt, [], ln);
                    this.paintVertexArray.resize(He), this._setPaintValue(pn, He, Cn);
                  }
                  updatePaintArray(He, ot, xt, zt) {
                    const ln = this.expression.evaluate({ zoom: 0 }, xt, zt);
                    this._setPaintValue(He, ot, ln);
                  }
                  _setPaintValue(He, ot, xt) {
                    if (this.type === "color") {
                      const zt = Vd(xt);
                      for (let ln = He; ln < ot; ln++) this.paintVertexArray.emplace(ln, zt[0], zt[1]);
                    } else {
                      for (let zt = He; zt < ot; zt++) this.paintVertexArray.emplace(zt, xt);
                      this.maxValue = Math.max(this.maxValue, Math.abs(xt));
                    }
                  }
                  upload(He) {
                    this.paintVertexArray && this.paintVertexArray.arrayBuffer && (this.paintVertexBuffer && this.paintVertexBuffer.buffer ? this.paintVertexBuffer.updateData(this.paintVertexArray) : this.paintVertexBuffer = He.createVertexBuffer(this.paintVertexArray, this.paintVertexAttributes, this.expression.isStateDependent));
                  }
                  destroy() {
                    this.paintVertexBuffer && this.paintVertexBuffer.destroy();
                  }
                }
                class Au {
                  constructor(He, ot, xt, zt, ln, pn) {
                    this.expression = He, this.uniformNames = ot.map((Cn) => `u_${Cn}_t`), this.type = xt, this.useIntegerZoom = zt, this.zoom = ln, this.maxValue = 0, this.paintVertexAttributes = ot.map((Cn) => ({ name: `a_${Cn}`, type: "Float32", components: xt === "color" ? 4 : 2, offset: 0 })), this.paintVertexArray = new pn();
                  }
                  populatePaintArray(He, ot, xt, zt, ln) {
                    const pn = this.expression.evaluate(new Ko(this.zoom), ot, {}, zt, [], ln), Cn = this.expression.evaluate(new Ko(this.zoom + 1), ot, {}, zt, [], ln), Nn = this.paintVertexArray.length;
                    this.paintVertexArray.resize(He), this._setPaintValue(Nn, He, pn, Cn);
                  }
                  updatePaintArray(He, ot, xt, zt) {
                    const ln = this.expression.evaluate({ zoom: this.zoom }, xt, zt), pn = this.expression.evaluate({ zoom: this.zoom + 1 }, xt, zt);
                    this._setPaintValue(He, ot, ln, pn);
                  }
                  _setPaintValue(He, ot, xt, zt) {
                    if (this.type === "color") {
                      const ln = Vd(xt), pn = Vd(zt);
                      for (let Cn = He; Cn < ot; Cn++) this.paintVertexArray.emplace(Cn, ln[0], ln[1], pn[0], pn[1]);
                    } else {
                      for (let ln = He; ln < ot; ln++) this.paintVertexArray.emplace(ln, xt, zt);
                      this.maxValue = Math.max(this.maxValue, Math.abs(xt), Math.abs(zt));
                    }
                  }
                  upload(He) {
                    this.paintVertexArray && this.paintVertexArray.arrayBuffer && (this.paintVertexBuffer && this.paintVertexBuffer.buffer ? this.paintVertexBuffer.updateData(this.paintVertexArray) : this.paintVertexBuffer = He.createVertexBuffer(this.paintVertexArray, this.paintVertexAttributes, this.expression.isStateDependent));
                  }
                  destroy() {
                    this.paintVertexBuffer && this.paintVertexBuffer.destroy();
                  }
                  setUniform(He, ot) {
                    const xt = this.useIntegerZoom ? Math.floor(ot.zoom) : ot.zoom, zt = ve(this.expression.interpolationFactor(xt, this.zoom, this.zoom + 1), 0, 1);
                    He.set(zt);
                  }
                  getBinding(He, ot, xt) {
                    return new yd(He, ot);
                  }
                }
                class Vu {
                  constructor(He, ot, xt, zt, ln, pn) {
                    this.expression = He, this.type = ot, this.useIntegerZoom = xt, this.zoom = zt, this.layerId = pn, this.zoomInPaintVertexArray = new ln(), this.zoomOutPaintVertexArray = new ln();
                  }
                  populatePaintArray(He, ot, xt) {
                    const zt = this.zoomInPaintVertexArray.length;
                    this.zoomInPaintVertexArray.resize(He), this.zoomOutPaintVertexArray.resize(He), this._setPaintValues(zt, He, ot.patterns && ot.patterns[this.layerId], xt);
                  }
                  updatePaintArray(He, ot, xt, zt, ln) {
                    this._setPaintValues(He, ot, xt.patterns && xt.patterns[this.layerId], ln);
                  }
                  _setPaintValues(He, ot, xt, zt) {
                    if (!zt || !xt) return;
                    const { min: ln, mid: pn, max: Cn } = xt, Nn = zt[ln], Wn = zt[pn], lr = zt[Cn];
                    if (Nn && Wn && lr) for (let xr = He; xr < ot; xr++) this.zoomInPaintVertexArray.emplace(xr, Wn.tl[0], Wn.tl[1], Wn.br[0], Wn.br[1], Nn.tl[0], Nn.tl[1], Nn.br[0], Nn.br[1], Wn.pixelRatio, Nn.pixelRatio), this.zoomOutPaintVertexArray.emplace(xr, Wn.tl[0], Wn.tl[1], Wn.br[0], Wn.br[1], lr.tl[0], lr.tl[1], lr.br[0], lr.br[1], Wn.pixelRatio, lr.pixelRatio);
                  }
                  upload(He) {
                    this.zoomInPaintVertexArray && this.zoomInPaintVertexArray.arrayBuffer && this.zoomOutPaintVertexArray && this.zoomOutPaintVertexArray.arrayBuffer && (this.zoomInPaintVertexBuffer = He.createVertexBuffer(this.zoomInPaintVertexArray, nu.members, this.expression.isStateDependent), this.zoomOutPaintVertexBuffer = He.createVertexBuffer(this.zoomOutPaintVertexArray, nu.members, this.expression.isStateDependent));
                  }
                  destroy() {
                    this.zoomOutPaintVertexBuffer && this.zoomOutPaintVertexBuffer.destroy(), this.zoomInPaintVertexBuffer && this.zoomInPaintVertexBuffer.destroy();
                  }
                }
                class Yh {
                  constructor(He, ot, xt) {
                    this.binders = {}, this._buffers = [];
                    const zt = [];
                    for (const ln in He.paint._values) {
                      if (!xt(ln)) continue;
                      const pn = He.paint.get(ln);
                      if (!(pn instanceof lu && Ri(pn.property.specification))) continue;
                      const Cn = vd(ln, He.type), Nn = pn.value, Wn = pn.property.specification.type, lr = pn.property.useIntegerZoom, xr = pn.property.specification["property-type"], Nr = xr === "cross-faded" || xr === "cross-faded-data-driven";
                      if (Nn.kind === "constant") this.binders[ln] = Nr ? new Af(Nn.value, Cn) : new S1(Nn.value, Cn, Wn), zt.push(`/u_${ln}`);
                      else if (Nn.kind === "source" || Nr) {
                        const Kr = Ff(ln, Wn, "source");
                        this.binders[ln] = Nr ? new Vu(Nn, Wn, lr, ot, Kr, He.id) : new Pd(Nn, Cn, Wn, Kr), zt.push(`/a_${ln}`);
                      } else {
                        const Kr = Ff(ln, Wn, "composite");
                        this.binders[ln] = new Au(Nn, Cn, Wn, lr, ot, Kr), zt.push(`/z_${ln}`);
                      }
                    }
                    this.cacheKey = zt.sort().join("");
                  }
                  getMaxValue(He) {
                    const ot = this.binders[He];
                    return ot instanceof Pd || ot instanceof Au ? ot.maxValue : 0;
                  }
                  populatePaintArrays(He, ot, xt, zt, ln) {
                    for (const pn in this.binders) {
                      const Cn = this.binders[pn];
                      (Cn instanceof Pd || Cn instanceof Au || Cn instanceof Vu) && Cn.populatePaintArray(He, ot, xt, zt, ln);
                    }
                  }
                  setConstantPatternPositions(He, ot) {
                    for (const xt in this.binders) {
                      const zt = this.binders[xt];
                      zt instanceof Af && zt.setConstantPatternPositions(He, ot);
                    }
                  }
                  updatePaintArrays(He, ot, xt, zt, ln) {
                    let pn = !1;
                    for (const Cn in He) {
                      const Nn = ot.getPositions(Cn);
                      for (const Wn of Nn) {
                        const lr = xt.feature(Wn.index);
                        for (const xr in this.binders) {
                          const Nr = this.binders[xr];
                          if ((Nr instanceof Pd || Nr instanceof Au || Nr instanceof Vu) && Nr.expression.isStateDependent === !0) {
                            const Kr = zt.paint.get(xr);
                            Nr.expression = Kr.value, Nr.updatePaintArray(Wn.start, Wn.end, lr, He[Cn], ln), pn = !0;
                          }
                        }
                      }
                    }
                    return pn;
                  }
                  defines() {
                    const He = [];
                    for (const ot in this.binders) {
                      const xt = this.binders[ot];
                      (xt instanceof S1 || xt instanceof Af) && He.push(...xt.uniformNames.map((zt) => `#define HAS_UNIFORM_${zt}`));
                    }
                    return He;
                  }
                  getBinderAttributes() {
                    const He = [];
                    for (const ot in this.binders) {
                      const xt = this.binders[ot];
                      if (xt instanceof Pd || xt instanceof Au) for (let zt = 0; zt < xt.paintVertexAttributes.length; zt++) He.push(xt.paintVertexAttributes[zt].name);
                      else if (xt instanceof Vu) for (let zt = 0; zt < nu.members.length; zt++) He.push(nu.members[zt].name);
                    }
                    return He;
                  }
                  getBinderUniforms() {
                    const He = [];
                    for (const ot in this.binders) {
                      const xt = this.binders[ot];
                      if (xt instanceof S1 || xt instanceof Af || xt instanceof Au) for (const zt of xt.uniformNames) He.push(zt);
                    }
                    return He;
                  }
                  getPaintVertexBuffers() {
                    return this._buffers;
                  }
                  getUniforms(He, ot) {
                    const xt = [];
                    for (const zt in this.binders) {
                      const ln = this.binders[zt];
                      if (ln instanceof S1 || ln instanceof Af || ln instanceof Au) {
                        for (const pn of ln.uniformNames) if (ot[pn]) {
                          const Cn = ln.getBinding(He, ot[pn], pn);
                          xt.push({ name: pn, property: zt, binding: Cn });
                        }
                      }
                    }
                    return xt;
                  }
                  setUniforms(He, ot, xt, zt) {
                    for (const { name: ln, property: pn, binding: Cn } of ot) this.binders[pn].setUniform(Cn, zt, xt.get(pn), ln);
                  }
                  updatePaintBuffers(He) {
                    this._buffers = [];
                    for (const ot in this.binders) {
                      const xt = this.binders[ot];
                      if (He && xt instanceof Vu) {
                        const zt = He.fromScale === 2 ? xt.zoomInPaintVertexBuffer : xt.zoomOutPaintVertexBuffer;
                        zt && this._buffers.push(zt);
                      } else (xt instanceof Pd || xt instanceof Au) && xt.paintVertexBuffer && this._buffers.push(xt.paintVertexBuffer);
                    }
                  }
                  upload(He) {
                    for (const ot in this.binders) {
                      const xt = this.binders[ot];
                      (xt instanceof Pd || xt instanceof Au || xt instanceof Vu) && xt.upload(He);
                    }
                    this.updatePaintBuffers();
                  }
                  destroy() {
                    for (const He in this.binders) {
                      const ot = this.binders[He];
                      (ot instanceof Pd || ot instanceof Au || ot instanceof Vu) && ot.destroy();
                    }
                  }
                }
                class Qf {
                  constructor(He, ot, xt = () => !0) {
                    this.programConfigurations = {};
                    for (const zt of He) this.programConfigurations[zt.id] = new Yh(zt, ot, xt);
                    this.needsUpload = !1, this._featureMap = new ru(), this._bufferOffset = 0;
                  }
                  populatePaintArrays(He, ot, xt, zt, ln, pn) {
                    for (const Cn in this.programConfigurations) this.programConfigurations[Cn].populatePaintArrays(He, ot, zt, ln, pn);
                    ot.id !== void 0 && this._featureMap.add(ot.id, xt, this._bufferOffset, He), this._bufferOffset = He, this.needsUpload = !0;
                  }
                  updatePaintArrays(He, ot, xt, zt) {
                    for (const ln of xt) this.needsUpload = this.programConfigurations[ln.id].updatePaintArrays(He, this._featureMap, ot, ln, zt) || this.needsUpload;
                  }
                  get(He) {
                    return this.programConfigurations[He];
                  }
                  upload(He) {
                    if (this.needsUpload) {
                      for (const ot in this.programConfigurations) this.programConfigurations[ot].upload(He);
                      this.needsUpload = !1;
                    }
                  }
                  destroy() {
                    for (const He in this.programConfigurations) this.programConfigurations[He].destroy();
                  }
                }
                function vd(Ye, He) {
                  return { "text-opacity": ["opacity"], "icon-opacity": ["opacity"], "text-color": ["fill_color"], "icon-color": ["fill_color"], "text-halo-color": ["halo_color"], "icon-halo-color": ["halo_color"], "text-halo-blur": ["halo_blur"], "icon-halo-blur": ["halo_blur"], "text-halo-width": ["halo_width"], "icon-halo-width": ["halo_width"], "line-gap-width": ["gapwidth"], "line-pattern": ["pattern_to", "pattern_from", "pixel_ratio_to", "pixel_ratio_from"], "fill-pattern": ["pattern_to", "pattern_from", "pixel_ratio_to", "pixel_ratio_from"], "fill-extrusion-pattern": ["pattern_to", "pattern_from", "pixel_ratio_to", "pixel_ratio_from"] }[Ye] || [Ye.replace(`${He}-`, "").replace(/-/g, "_")];
                }
                function Ff(Ye, He, ot) {
                  const xt = { color: { source: ht, composite: wi }, number: { source: Un, composite: ht } }, zt = function(ln) {
                    return { "line-pattern": { source: nl, composite: nl }, "fill-pattern": { source: nl, composite: nl }, "fill-extrusion-pattern": { source: nl, composite: nl } }[ln];
                  }(Ye);
                  return zt && zt[ot] || xt[He][ot];
                }
                ss("ConstantBinder", S1), ss("CrossFadedConstantBinder", Af), ss("SourceExpressionBinder", Pd), ss("CrossFadedCompositeBinder", Vu), ss("CompositeExpressionBinder", Au), ss("ProgramConfiguration", Yh, { omit: ["_buffers"] }), ss("ProgramConfigurationSet", Qf);
                const Uu = 8192, rf = Math.pow(2, 14) - 1, Ud = -rf - 1;
                function $d(Ye) {
                  const He = Uu / Ye.extent, ot = Ye.loadGeometry();
                  for (let xt = 0; xt < ot.length; xt++) {
                    const zt = ot[xt];
                    for (let ln = 0; ln < zt.length; ln++) {
                      const pn = zt[ln], Cn = Math.round(pn.x * He), Nn = Math.round(pn.y * He);
                      pn.x = ve(Cn, Ud, rf), pn.y = ve(Nn, Ud, rf), (Cn < pn.x || Cn > pn.x + 1 || Nn < pn.y || Nn > pn.y + 1) && Qe("Geometry exceeds allowed extent, reduce your vector tile buffer size");
                    }
                  }
                  return ot;
                }
                function Sd(Ye, He) {
                  return { type: Ye.type, id: Ye.id, properties: Ye.properties, geometry: He ? $d(Ye) : [] };
                }
                function Yd(Ye, He, ot, xt, zt) {
                  Ye.emplaceBack(2 * He + (xt + 1) / 2, 2 * ot + (zt + 1) / 2);
                }
                class af {
                  constructor(He) {
                    this.zoom = He.zoom, this.overscaling = He.overscaling, this.layers = He.layers, this.layerIds = this.layers.map((ot) => ot.id), this.index = He.index, this.hasPattern = !1, this.layoutVertexArray = new Uo(), this.indexArray = new Dl(), this.segments = new al(), this.programConfigurations = new Qf(He.layers, He.zoom), this.stateDependentLayerIds = this.layers.filter((ot) => ot.isStateDependent()).map((ot) => ot.id);
                  }
                  populate(He, ot, xt) {
                    const zt = this.layers[0], ln = [];
                    let pn = null, Cn = !1;
                    zt.type === "circle" && (pn = zt.layout.get("circle-sort-key"), Cn = !pn.isConstant());
                    for (const { feature: Nn, id: Wn, index: lr, sourceLayerIndex: xr } of He) {
                      const Nr = this.layers[0]._featureFilter.needGeometry, Kr = Sd(Nn, Nr);
                      if (!this.layers[0]._featureFilter.filter(new Ko(this.zoom), Kr, xt)) continue;
                      const xi = Cn ? pn.evaluate(Kr, {}, xt) : void 0, ui = { id: Wn, properties: Nn.properties, type: Nn.type, sourceLayerIndex: xr, index: lr, geometry: Nr ? Kr.geometry : $d(Nn), patterns: {}, sortKey: xi };
                      ln.push(ui);
                    }
                    Cn && ln.sort((Nn, Wn) => Nn.sortKey - Wn.sortKey);
                    for (const Nn of ln) {
                      const { geometry: Wn, index: lr, sourceLayerIndex: xr } = Nn, Nr = He[lr].feature;
                      this.addFeature(Nn, Wn, lr, xt), ot.featureIndex.insert(Nr, Wn, lr, xr, this.index);
                    }
                  }
                  update(He, ot, xt) {
                    this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(He, ot, this.stateDependentLayers, xt);
                  }
                  isEmpty() {
                    return this.layoutVertexArray.length === 0;
                  }
                  uploadPending() {
                    return !this.uploaded || this.programConfigurations.needsUpload;
                  }
                  upload(He) {
                    this.uploaded || (this.layoutVertexBuffer = He.createVertexBuffer(this.layoutVertexArray, hl), this.indexBuffer = He.createIndexBuffer(this.indexArray)), this.programConfigurations.upload(He), this.uploaded = !0;
                  }
                  destroy() {
                    this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy());
                  }
                  addFeature(He, ot, xt, zt) {
                    for (const ln of ot) for (const pn of ln) {
                      const Cn = pn.x, Nn = pn.y;
                      if (Cn < 0 || Cn >= Uu || Nn < 0 || Nn >= Uu) continue;
                      const Wn = this.segments.prepareSegment(4, this.layoutVertexArray, this.indexArray, He.sortKey), lr = Wn.vertexLength;
                      Yd(this.layoutVertexArray, Cn, Nn, -1, -1), Yd(this.layoutVertexArray, Cn, Nn, 1, -1), Yd(this.layoutVertexArray, Cn, Nn, 1, 1), Yd(this.layoutVertexArray, Cn, Nn, -1, 1), this.indexArray.emplaceBack(lr, lr + 1, lr + 2), this.indexArray.emplaceBack(lr, lr + 3, lr + 2), Wn.vertexLength += 4, Wn.primitiveLength += 2;
                    }
                    this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, He, xt, {}, zt);
                  }
                }
                function of(Ye, He) {
                  for (let ot = 0; ot < Ye.length; ot++) if (ld(He, Ye[ot])) return !0;
                  for (let ot = 0; ot < He.length; ot++) if (ld(Ye, He[ot])) return !0;
                  return !!V1(Ye, He);
                }
                function Bf(Ye, He, ot) {
                  return !!ld(Ye, He) || !!E1(He, Ye, ot);
                }
                function qd(Ye, He) {
                  if (Ye.length === 1) return Xd(He, Ye[0]);
                  for (let ot = 0; ot < He.length; ot++) {
                    const xt = He[ot];
                    for (let zt = 0; zt < xt.length; zt++) if (ld(Ye, xt[zt])) return !0;
                  }
                  for (let ot = 0; ot < Ye.length; ot++) if (Xd(He, Ye[ot])) return !0;
                  for (let ot = 0; ot < He.length; ot++) if (V1(Ye, He[ot])) return !0;
                  return !1;
                }
                function Cf(Ye, He, ot) {
                  if (Ye.length > 1) {
                    if (V1(Ye, He)) return !0;
                    for (let xt = 0; xt < He.length; xt++) if (E1(He[xt], Ye, ot)) return !0;
                  }
                  for (let xt = 0; xt < Ye.length; xt++) if (E1(Ye[xt], He, ot)) return !0;
                  return !1;
                }
                function V1(Ye, He) {
                  if (Ye.length === 0 || He.length === 0) return !1;
                  for (let ot = 0; ot < Ye.length - 1; ot++) {
                    const xt = Ye[ot], zt = Ye[ot + 1];
                    for (let ln = 0; ln < He.length - 1; ln++) if (jd(xt, zt, He[ln], He[ln + 1])) return !0;
                  }
                  return !1;
                }
                function jd(Ye, He, ot, xt) {
                  return ke(Ye, ot, xt) !== ke(He, ot, xt) && ke(Ye, He, ot) !== ke(Ye, He, xt);
                }
                function E1(Ye, He, ot) {
                  const xt = ot * ot;
                  if (He.length === 1) return Ye.distSqr(He[0]) < xt;
                  for (let zt = 1; zt < He.length; zt++) if (sd(Ye, He[zt - 1], He[zt]) < xt) return !0;
                  return !1;
                }
                function sd(Ye, He, ot) {
                  const xt = He.distSqr(ot);
                  if (xt === 0) return Ye.distSqr(He);
                  const zt = ((Ye.x - He.x) * (ot.x - He.x) + (Ye.y - He.y) * (ot.y - He.y)) / xt;
                  return zt < 0 ? Ye.distSqr(He) : zt > 1 ? Ye.distSqr(ot) : Ye.distSqr(ot.sub(He)._mult(zt)._add(He));
                }
                function Xd(Ye, He) {
                  let ot, xt, zt, ln = !1;
                  for (let pn = 0; pn < Ye.length; pn++) {
                    ot = Ye[pn];
                    for (let Cn = 0, Nn = ot.length - 1; Cn < ot.length; Nn = Cn++) xt = ot[Cn], zt = ot[Nn], xt.y > He.y != zt.y > He.y && He.x < (zt.x - xt.x) * (He.y - xt.y) / (zt.y - xt.y) + xt.x && (ln = !ln);
                  }
                  return ln;
                }
                function ld(Ye, He) {
                  let ot = !1;
                  for (let xt = 0, zt = Ye.length - 1; xt < Ye.length; zt = xt++) {
                    const ln = Ye[xt], pn = Ye[zt];
                    ln.y > He.y != pn.y > He.y && He.x < (pn.x - ln.x) * (He.y - ln.y) / (pn.y - ln.y) + ln.x && (ot = !ot);
                  }
                  return ot;
                }
                function Ch(Ye, He, ot) {
                  const xt = ot[0], zt = ot[2];
                  if (Ye.x < xt.x && He.x < xt.x || Ye.x > zt.x && He.x > zt.x || Ye.y < xt.y && He.y < xt.y || Ye.y > zt.y && He.y > zt.y) return !1;
                  const ln = ke(Ye, He, ot[0]);
                  return ln !== ke(Ye, He, ot[1]) || ln !== ke(Ye, He, ot[2]) || ln !== ke(Ye, He, ot[3]);
                }
                function sf(Ye, He, ot) {
                  const xt = He.paint.get(Ye).value;
                  return xt.kind === "constant" ? xt.value : ot.programConfigurations.get(He.id).getMaxValue(Ye);
                }
                function Zd(Ye) {
                  return Math.sqrt(Ye[0] * Ye[0] + Ye[1] * Ye[1]);
                }
                function Lf(Ye, He, ot, xt, zt) {
                  if (!He[0] && !He[1]) return Ye;
                  const ln = ce.convert(He)._mult(zt);
                  ot === "viewport" && ln._rotate(-xt);
                  const pn = [];
                  for (let Cn = 0; Cn < Ye.length; Cn++) {
                    const Nn = Ye[Cn];
                    pn.push(Nn.sub(ln));
                  }
                  return pn;
                }
                let uh;
                ss("CircleBucket", af, { omit: ["layers"] });
                let Xf;
                var Lh = { get paint() {
                  return Xf = Xf || new tu({ "circle-radius": new Os(yt.paint_circle["circle-radius"]), "circle-color": new Os(yt.paint_circle["circle-color"]), "circle-blur": new Os(yt.paint_circle["circle-blur"]), "circle-opacity": new Os(yt.paint_circle["circle-opacity"]), "circle-translate": new zs(yt.paint_circle["circle-translate"]), "circle-translate-anchor": new zs(yt.paint_circle["circle-translate-anchor"]), "circle-pitch-scale": new zs(yt.paint_circle["circle-pitch-scale"]), "circle-pitch-alignment": new zs(yt.paint_circle["circle-pitch-alignment"]), "circle-stroke-width": new Os(yt.paint_circle["circle-stroke-width"]), "circle-stroke-color": new Os(yt.paint_circle["circle-stroke-color"]), "circle-stroke-opacity": new Os(yt.paint_circle["circle-stroke-opacity"]) });
                }, get layout() {
                  return uh = uh || new tu({ "circle-sort-key": new Os(yt.layout_circle["circle-sort-key"]) });
                } }, p1 = 1e-6, cd = typeof Float32Array < "u" ? Float32Array : Array;
                function dh(Ye) {
                  return Ye[0] = 1, Ye[1] = 0, Ye[2] = 0, Ye[3] = 0, Ye[4] = 0, Ye[5] = 1, Ye[6] = 0, Ye[7] = 0, Ye[8] = 0, Ye[9] = 0, Ye[10] = 1, Ye[11] = 0, Ye[12] = 0, Ye[13] = 0, Ye[14] = 0, Ye[15] = 1, Ye;
                }
                function kh(Ye, He, ot) {
                  var xt = He[0], zt = He[1], ln = He[2], pn = He[3], Cn = He[4], Nn = He[5], Wn = He[6], lr = He[7], xr = He[8], Nr = He[9], Kr = He[10], xi = He[11], ui = He[12], Ui = He[13], ia = He[14], Na = He[15], Ta = ot[0], Ra = ot[1], oo = ot[2], fs = ot[3];
                  return Ye[0] = Ta * xt + Ra * Cn + oo * xr + fs * ui, Ye[1] = Ta * zt + Ra * Nn + oo * Nr + fs * Ui, Ye[2] = Ta * ln + Ra * Wn + oo * Kr + fs * ia, Ye[3] = Ta * pn + Ra * lr + oo * xi + fs * Na, Ta = ot[4], Ra = ot[5], oo = ot[6], fs = ot[7], Ye[4] = Ta * xt + Ra * Cn + oo * xr + fs * ui, Ye[5] = Ta * zt + Ra * Nn + oo * Nr + fs * Ui, Ye[6] = Ta * ln + Ra * Wn + oo * Kr + fs * ia, Ye[7] = Ta * pn + Ra * lr + oo * xi + fs * Na, Ta = ot[8], Ra = ot[9], oo = ot[10], fs = ot[11], Ye[8] = Ta * xt + Ra * Cn + oo * xr + fs * ui, Ye[9] = Ta * zt + Ra * Nn + oo * Nr + fs * Ui, Ye[10] = Ta * ln + Ra * Wn + oo * Kr + fs * ia, Ye[11] = Ta * pn + Ra * lr + oo * xi + fs * Na, Ta = ot[12], Ra = ot[13], oo = ot[14], fs = ot[15], Ye[12] = Ta * xt + Ra * Cn + oo * xr + fs * ui, Ye[13] = Ta * zt + Ra * Nn + oo * Nr + fs * Ui, Ye[14] = Ta * ln + Ra * Wn + oo * Kr + fs * ia, Ye[15] = Ta * pn + Ra * lr + oo * xi + fs * Na, Ye;
                }
                Math.hypot || (Math.hypot = function() {
                  for (var Ye = 0, He = arguments.length; He--; ) Ye += arguments[He] * arguments[He];
                  return Math.sqrt(Ye);
                });
                var Mh = function(Ye, He, ot, xt, zt) {
                  var ln, pn = 1 / Math.tan(He / 2);
                  return Ye[0] = pn / ot, Ye[1] = 0, Ye[2] = 0, Ye[3] = 0, Ye[4] = 0, Ye[5] = pn, Ye[6] = 0, Ye[7] = 0, Ye[8] = 0, Ye[9] = 0, Ye[11] = -1, Ye[12] = 0, Ye[13] = 0, Ye[15] = 0, zt != null && zt !== 1 / 0 ? (ln = 1 / (xt - zt), Ye[10] = (zt + xt) * ln, Ye[14] = 2 * zt * xt * ln) : (Ye[10] = -1, Ye[14] = -2 * xt), Ye;
                }, h2 = function(Ye, He, ot, xt, zt, ln, pn) {
                  var Cn = 1 / (He - ot), Nn = 1 / (xt - zt), Wn = 1 / (ln - pn);
                  return Ye[0] = -2 * Cn, Ye[1] = 0, Ye[2] = 0, Ye[3] = 0, Ye[4] = 0, Ye[5] = -2 * Nn, Ye[6] = 0, Ye[7] = 0, Ye[8] = 0, Ye[9] = 0, Ye[10] = 2 * Wn, Ye[11] = 0, Ye[12] = (He + ot) * Cn, Ye[13] = (zt + xt) * Nn, Ye[14] = (pn + ln) * Wn, Ye[15] = 1, Ye;
                }, Rh = kh;
                function Zf(Ye, He, ot) {
                  var xt = He[0], zt = He[1], ln = He[2], pn = He[3];
                  return Ye[0] = ot[0] * xt + ot[4] * zt + ot[8] * ln + ot[12] * pn, Ye[1] = ot[1] * xt + ot[5] * zt + ot[9] * ln + ot[13] * pn, Ye[2] = ot[2] * xt + ot[6] * zt + ot[10] * ln + ot[14] * pn, Ye[3] = ot[3] * xt + ot[7] * zt + ot[11] * ln + ot[15] * pn, Ye;
                }
                var Rd, ed = function(Ye, He, ot) {
                  return Ye[0] = He[0] * ot[0], Ye[1] = He[1] * ot[1], Ye[2] = He[2] * ot[2], Ye[3] = He[3] * ot[3], Ye;
                };
                Rd = new cd(4), cd != Float32Array && (Rd[0] = 0, Rd[1] = 0, Rd[2] = 0, Rd[3] = 0);
                class Hf extends $n {
                  constructor(He) {
                    super(He, Lh);
                  }
                  createBucket(He) {
                    return new af(He);
                  }
                  queryRadius(He) {
                    const ot = He;
                    return sf("circle-radius", this, ot) + sf("circle-stroke-width", this, ot) + Zd(this.paint.get("circle-translate"));
                  }
                  queryIntersectsFeature(He, ot, xt, zt, ln, pn, Cn, Nn) {
                    const Wn = Lf(He, this.paint.get("circle-translate"), this.paint.get("circle-translate-anchor"), pn.angle, Cn), lr = this.paint.get("circle-radius").evaluate(ot, xt) + this.paint.get("circle-stroke-width").evaluate(ot, xt), xr = this.paint.get("circle-pitch-alignment") === "map", Nr = xr ? Wn : function(xi, ui) {
                      return xi.map((Ui) => Od(Ui, ui));
                    }(Wn, Nn), Kr = xr ? lr * Cn : lr;
                    for (const xi of zt) for (const ui of xi) {
                      const Ui = xr ? ui : Od(ui, Nn);
                      let ia = Kr;
                      const Na = Zf([], [ui.x, ui.y, 0, 1], Nn);
                      if (this.paint.get("circle-pitch-scale") === "viewport" && this.paint.get("circle-pitch-alignment") === "map" ? ia *= Na[3] / pn.cameraToCenterDistance : this.paint.get("circle-pitch-scale") === "map" && this.paint.get("circle-pitch-alignment") === "viewport" && (ia *= pn.cameraToCenterDistance / Na[3]), Bf(Nr, Ui, ia)) return !0;
                    }
                    return !1;
                  }
                }
                function Od(Ye, He) {
                  const ot = Zf([], [Ye.x, Ye.y, 0, 1], He);
                  return new ce(ot[0] / ot[3], ot[1] / ot[3]);
                }
                class Oh extends af {
                }
                let Yu;
                ss("HeatmapBucket", Oh, { omit: ["layers"] });
                var kf = { get paint() {
                  return Yu = Yu || new tu({ "heatmap-radius": new Os(yt.paint_heatmap["heatmap-radius"]), "heatmap-weight": new Os(yt.paint_heatmap["heatmap-weight"]), "heatmap-intensity": new zs(yt.paint_heatmap["heatmap-intensity"]), "heatmap-color": new Iu(yt.paint_heatmap["heatmap-color"]), "heatmap-opacity": new zs(yt.paint_heatmap["heatmap-opacity"]) });
                } };
                function Mf(Ye, { width: He, height: ot }, xt, zt) {
                  if (zt) {
                    if (zt instanceof Uint8ClampedArray) zt = new Uint8Array(zt.buffer);
                    else if (zt.length !== He * ot * xt) throw new RangeError(`mismatched image size. expected: ${zt.length} but got: ${He * ot * xt}`);
                  } else zt = new Uint8Array(He * ot * xt);
                  return Ye.width = He, Ye.height = ot, Ye.data = zt, Ye;
                }
                function e1(Ye, { width: He, height: ot }, xt) {
                  if (He === Ye.width && ot === Ye.height) return;
                  const zt = Mf({}, { width: He, height: ot }, xt);
                  Id(Ye, zt, { x: 0, y: 0 }, { x: 0, y: 0 }, { width: Math.min(Ye.width, He), height: Math.min(Ye.height, ot) }, xt), Ye.width = He, Ye.height = ot, Ye.data = zt.data;
                }
                function Id(Ye, He, ot, xt, zt, ln) {
                  if (zt.width === 0 || zt.height === 0) return He;
                  if (zt.width > Ye.width || zt.height > Ye.height || ot.x > Ye.width - zt.width || ot.y > Ye.height - zt.height) throw new RangeError("out of range source coordinates for image copy");
                  if (zt.width > He.width || zt.height > He.height || xt.x > He.width - zt.width || xt.y > He.height - zt.height) throw new RangeError("out of range destination coordinates for image copy");
                  const pn = Ye.data, Cn = He.data;
                  if (pn === Cn) throw new Error("srcData equals dstData, so image is already copied");
                  for (let Nn = 0; Nn < zt.height; Nn++) {
                    const Wn = ((ot.y + Nn) * Ye.width + ot.x) * ln, lr = ((xt.y + Nn) * He.width + xt.x) * ln;
                    for (let xr = 0; xr < zt.width * ln; xr++) Cn[lr + xr] = pn[Wn + xr];
                  }
                  return He;
                }
                class cu {
                  constructor(He, ot) {
                    Mf(this, He, 1, ot);
                  }
                  resize(He) {
                    e1(this, He, 1);
                  }
                  clone() {
                    return new cu({ width: this.width, height: this.height }, new Uint8Array(this.data));
                  }
                  static copy(He, ot, xt, zt, ln) {
                    Id(He, ot, xt, zt, ln, 1);
                  }
                }
                class pu {
                  constructor(He, ot) {
                    Mf(this, He, 4, ot);
                  }
                  resize(He) {
                    e1(this, He, 4);
                  }
                  replace(He, ot) {
                    ot ? this.data.set(He) : He instanceof Uint8ClampedArray ? this.data = new Uint8Array(He.buffer) : this.data = He;
                  }
                  clone() {
                    return new pu({ width: this.width, height: this.height }, new Uint8Array(this.data));
                  }
                  static copy(He, ot, xt, zt, ln) {
                    Id(He, ot, xt, zt, ln, 4);
                  }
                }
                function gf(Ye) {
                  const He = {}, ot = Ye.resolution || 256, xt = Ye.clips ? Ye.clips.length : 1, zt = Ye.image || new pu({ width: ot, height: xt });
                  if (ln = ot, Math.log(ln) / Math.LN2 % 1 != 0) throw new Error(`width is not a power of 2 - ${ot}`);
                  var ln;
                  const pn = (Cn, Nn, Wn) => {
                    He[Ye.evaluationKey] = Wn;
                    const lr = Ye.expression.evaluate(He);
                    zt.data[Cn + Nn + 0] = Math.floor(255 * lr.r / lr.a), zt.data[Cn + Nn + 1] = Math.floor(255 * lr.g / lr.a), zt.data[Cn + Nn + 2] = Math.floor(255 * lr.b / lr.a), zt.data[Cn + Nn + 3] = Math.floor(255 * lr.a);
                  };
                  if (Ye.clips) for (let Cn = 0, Nn = 0; Cn < xt; ++Cn, Nn += 4 * ot) for (let Wn = 0, lr = 0; Wn < ot; Wn++, lr += 4) {
                    const xr = Wn / (ot - 1), { start: Nr, end: Kr } = Ye.clips[Cn];
                    pn(Nn, lr, Nr * (1 - xr) + Kr * xr);
                  }
                  else for (let Cn = 0, Nn = 0; Cn < ot; Cn++, Nn += 4) pn(0, Nn, Cn / (ot - 1));
                  return zt;
                }
                ss("AlphaImage", cu), ss("RGBAImage", pu);
                class lf extends $n {
                  createBucket(He) {
                    return new Oh(He);
                  }
                  constructor(He) {
                    super(He, kf), this._updateColorRamp();
                  }
                  _handleSpecialPaintPropertyUpdate(He) {
                    He === "heatmap-color" && this._updateColorRamp();
                  }
                  _updateColorRamp() {
                    const He = this._transitionablePaint._values["heatmap-color"].value.expression;
                    this.colorRamp = gf({ expression: He, evaluationKey: "heatmapDensity", image: this.colorRamp }), this.colorRampTexture = null;
                  }
                  resize() {
                    this.heatmapFbo && (this.heatmapFbo.destroy(), this.heatmapFbo = null);
                  }
                  queryRadius() {
                    return 0;
                  }
                  queryIntersectsFeature() {
                    return !1;
                  }
                  hasOffscreenPass() {
                    return this.paint.get("heatmap-opacity") !== 0 && this.visibility !== "none";
                  }
                }
                let yf;
                var Ih = { get paint() {
                  return yf = yf || new tu({ "hillshade-illumination-direction": new zs(yt.paint_hillshade["hillshade-illumination-direction"]), "hillshade-illumination-anchor": new zs(yt.paint_hillshade["hillshade-illumination-anchor"]), "hillshade-exaggeration": new zs(yt.paint_hillshade["hillshade-exaggeration"]), "hillshade-shadow-color": new zs(yt.paint_hillshade["hillshade-shadow-color"]), "hillshade-highlight-color": new zs(yt.paint_hillshade["hillshade-highlight-color"]), "hillshade-accent-color": new zs(yt.paint_hillshade["hillshade-accent-color"]) });
                } };
                class Kf extends $n {
                  constructor(He) {
                    super(He, Ih);
                  }
                  hasOffscreenPass() {
                    return this.paint.get("hillshade-exaggeration") !== 0 && this.visibility !== "none";
                  }
                }
                const zf = Fe([{ name: "a_pos", components: 2, type: "Int16" }], 4), { members: cf, size: Xh, alignment: P2 } = zf;
                function Jf(Ye, He, ot = 2) {
                  const xt = He && He.length, zt = xt ? He[0] * ot : Ye.length;
                  let ln = eh(Ye, 0, zt, ot, !0);
                  const pn = [];
                  if (!ln || ln.next === ln.prev) return pn;
                  let Cn, Nn, Wn;
                  if (xt && (ln = function(lr, xr, Nr, Kr) {
                    const xi = [];
                    for (let ui = 0, Ui = xr.length; ui < Ui; ui++) {
                      const ia = eh(lr, xr[ui] * Kr, ui < Ui - 1 ? xr[ui + 1] * Kr : lr.length, Kr, !1);
                      ia === ia.next && (ia.steiner = !0), xi.push(Sa(ia));
                    }
                    xi.sort(bd);
                    for (let ui = 0; ui < xi.length; ui++) Nr = pr(xi[ui], Nr);
                    return Nr;
                  }(Ye, He, ln, ot)), Ye.length > 80 * ot) {
                    Cn = 1 / 0, Nn = 1 / 0;
                    let lr = -1 / 0, xr = -1 / 0;
                    for (let Nr = ot; Nr < zt; Nr += ot) {
                      const Kr = Ye[Nr], xi = Ye[Nr + 1];
                      Kr < Cn && (Cn = Kr), xi < Nn && (Nn = xi), Kr > lr && (lr = Kr), xi > xr && (xr = xi);
                    }
                    Wn = Math.max(lr - Cn, xr - Nn), Wn = Wn !== 0 ? 32767 / Wn : 0;
                  }
                  return Rf(ln, pn, ot, Cn, Nn, Wn, 0), pn;
                }
                function eh(Ye, He, ot, xt, zt) {
                  let ln;
                  if (zt === function(pn, Cn, Nn, Wn) {
                    let lr = 0;
                    for (let xr = Cn, Nr = Nn - Wn; xr < Nn; xr += Wn) lr += (pn[Nr] - pn[xr]) * (pn[xr + 1] + pn[Nr + 1]), Nr = xr;
                    return lr;
                  }(Ye, He, ot, xt) > 0) for (let pn = He; pn < ot; pn += xt) ln = Ao(pn / xt | 0, Ye[pn], Ye[pn + 1], ln);
                  else for (let pn = ot - xt; pn >= He; pn -= xt) ln = Ao(pn / xt | 0, Ye[pn], Ye[pn + 1], ln);
                  return ln && Qo(ln, ln.next) && (Ro(ln), ln = ln.next), ln;
                }
                function td(Ye, He) {
                  if (!Ye) return Ye;
                  He || (He = Ye);
                  let ot, xt = Ye;
                  do
                    if (ot = !1, xt.steiner || !Qo(xt, xt.next) && En(xt.prev, xt, xt.next) !== 0) xt = xt.next;
                    else {
                      if (Ro(xt), xt = He = xt.prev, xt === xt.next) break;
                      ot = !0;
                    }
                  while (ot || xt !== He);
                  return He;
                }
                function Rf(Ye, He, ot, xt, zt, ln, pn) {
                  if (!Ye) return;
                  !pn && ln && function(Nn, Wn, lr, xr) {
                    let Nr = Nn;
                    do
                      Nr.z === 0 && (Nr.z = ar(Nr.x, Nr.y, Wn, lr, xr)), Nr.prevZ = Nr.prev, Nr.nextZ = Nr.next, Nr = Nr.next;
                    while (Nr !== Nn);
                    Nr.prevZ.nextZ = null, Nr.prevZ = null, function(Kr) {
                      let xi, ui = 1;
                      do {
                        let Ui, ia = Kr;
                        Kr = null;
                        let Na = null;
                        for (xi = 0; ia; ) {
                          xi++;
                          let Ta = ia, Ra = 0;
                          for (let fs = 0; fs < ui && (Ra++, Ta = Ta.nextZ, Ta); fs++) ;
                          let oo = ui;
                          for (; Ra > 0 || oo > 0 && Ta; ) Ra !== 0 && (oo === 0 || !Ta || ia.z <= Ta.z) ? (Ui = ia, ia = ia.nextZ, Ra--) : (Ui = Ta, Ta = Ta.nextZ, oo--), Na ? Na.nextZ = Ui : Kr = Ui, Ui.prevZ = Na, Na = Ui;
                          ia = Ta;
                        }
                        Na.nextZ = null, ui *= 2;
                      } while (xi > 1);
                    }(Nr);
                  }(Ye, xt, zt, ln);
                  let Cn = Ye;
                  for (; Ye.prev !== Ye.next; ) {
                    const Nn = Ye.prev, Wn = Ye.next;
                    if (ln ? $2(Ye, xt, zt, ln) : fh(Ye)) He.push(Nn.i, Ye.i, Wn.i), Ro(Ye), Ye = Wn.next, Cn = Wn.next;
                    else if ((Ye = Wn) === Cn) {
                      pn ? pn === 1 ? Rf(Ye = F2(td(Ye), He), He, ot, xt, zt, ln, 2) : pn === 2 && hh(Ye, He, ot, xt, zt, ln) : Rf(td(Ye), He, ot, xt, zt, ln, 1);
                      break;
                    }
                  }
                }
                function fh(Ye) {
                  const He = Ye.prev, ot = Ye, xt = Ye.next;
                  if (En(He, ot, xt) >= 0) return !1;
                  const zt = He.x, ln = ot.x, pn = xt.x, Cn = He.y, Nn = ot.y, Wn = xt.y, lr = zt < ln ? zt < pn ? zt : pn : ln < pn ? ln : pn, xr = Cn < Nn ? Cn < Wn ? Cn : Wn : Nn < Wn ? Nn : Wn, Nr = zt > ln ? zt > pn ? zt : pn : ln > pn ? ln : pn, Kr = Cn > Nn ? Cn > Wn ? Cn : Wn : Nn > Wn ? Nn : Wn;
                  let xi = xt.next;
                  for (; xi !== He; ) {
                    if (xi.x >= lr && xi.x <= Nr && xi.y >= xr && xi.y <= Kr && Qn(zt, Cn, ln, Nn, pn, Wn, xi.x, xi.y) && En(xi.prev, xi, xi.next) >= 0) return !1;
                    xi = xi.next;
                  }
                  return !0;
                }
                function $2(Ye, He, ot, xt) {
                  const zt = Ye.prev, ln = Ye, pn = Ye.next;
                  if (En(zt, ln, pn) >= 0) return !1;
                  const Cn = zt.x, Nn = ln.x, Wn = pn.x, lr = zt.y, xr = ln.y, Nr = pn.y, Kr = Cn < Nn ? Cn < Wn ? Cn : Wn : Nn < Wn ? Nn : Wn, xi = lr < xr ? lr < Nr ? lr : Nr : xr < Nr ? xr : Nr, ui = Cn > Nn ? Cn > Wn ? Cn : Wn : Nn > Wn ? Nn : Wn, Ui = lr > xr ? lr > Nr ? lr : Nr : xr > Nr ? xr : Nr, ia = ar(Kr, xi, He, ot, xt), Na = ar(ui, Ui, He, ot, xt);
                  let Ta = Ye.prevZ, Ra = Ye.nextZ;
                  for (; Ta && Ta.z >= ia && Ra && Ra.z <= Na; ) {
                    if (Ta.x >= Kr && Ta.x <= ui && Ta.y >= xi && Ta.y <= Ui && Ta !== zt && Ta !== pn && Qn(Cn, lr, Nn, xr, Wn, Nr, Ta.x, Ta.y) && En(Ta.prev, Ta, Ta.next) >= 0 || (Ta = Ta.prevZ, Ra.x >= Kr && Ra.x <= ui && Ra.y >= xi && Ra.y <= Ui && Ra !== zt && Ra !== pn && Qn(Cn, lr, Nn, xr, Wn, Nr, Ra.x, Ra.y) && En(Ra.prev, Ra, Ra.next) >= 0)) return !1;
                    Ra = Ra.nextZ;
                  }
                  for (; Ta && Ta.z >= ia; ) {
                    if (Ta.x >= Kr && Ta.x <= ui && Ta.y >= xi && Ta.y <= Ui && Ta !== zt && Ta !== pn && Qn(Cn, lr, Nn, xr, Wn, Nr, Ta.x, Ta.y) && En(Ta.prev, Ta, Ta.next) >= 0) return !1;
                    Ta = Ta.prevZ;
                  }
                  for (; Ra && Ra.z <= Na; ) {
                    if (Ra.x >= Kr && Ra.x <= ui && Ra.y >= xi && Ra.y <= Ui && Ra !== zt && Ra !== pn && Qn(Cn, lr, Nn, xr, Wn, Nr, Ra.x, Ra.y) && En(Ra.prev, Ra, Ra.next) >= 0) return !1;
                    Ra = Ra.nextZ;
                  }
                  return !0;
                }
                function F2(Ye, He) {
                  let ot = Ye;
                  do {
                    const xt = ot.prev, zt = ot.next.next;
                    !Qo(xt, zt) && lo(xt, ot, ot.next, zt) && Ho(xt, zt) && Ho(zt, xt) && (He.push(xt.i, ot.i, zt.i), Ro(ot), Ro(ot.next), ot = Ye = zt), ot = ot.next;
                  } while (ot !== Ye);
                  return td(ot);
                }
                function hh(Ye, He, ot, xt, zt, ln) {
                  let pn = Ye;
                  do {
                    let Cn = pn.next.next;
                    for (; Cn !== pn.prev; ) {
                      if (pn.i !== Cn.i && za(pn, Cn)) {
                        let Nn = Co(pn, Cn);
                        return pn = td(pn, pn.next), Nn = td(Nn, Nn.next), Rf(pn, He, ot, xt, zt, ln, 0), void Rf(Nn, He, ot, xt, zt, ln, 0);
                      }
                      Cn = Cn.next;
                    }
                    pn = pn.next;
                  } while (pn !== Ye);
                }
                function bd(Ye, He) {
                  return Ye.x - He.x;
                }
                function pr(Ye, He) {
                  const ot = function(zt, ln) {
                    let pn = ln;
                    const Cn = zt.x, Nn = zt.y;
                    let Wn, lr = -1 / 0;
                    do {
                      if (Nn <= pn.y && Nn >= pn.next.y && pn.next.y !== pn.y) {
                        const ui = pn.x + (Nn - pn.y) * (pn.next.x - pn.x) / (pn.next.y - pn.y);
                        if (ui <= Cn && ui > lr && (lr = ui, Wn = pn.x < pn.next.x ? pn : pn.next, ui === Cn)) return Wn;
                      }
                      pn = pn.next;
                    } while (pn !== ln);
                    if (!Wn) return null;
                    const xr = Wn, Nr = Wn.x, Kr = Wn.y;
                    let xi = 1 / 0;
                    pn = Wn;
                    do {
                      if (Cn >= pn.x && pn.x >= Nr && Cn !== pn.x && Qn(Nn < Kr ? Cn : lr, Nn, Nr, Kr, Nn < Kr ? lr : Cn, Nn, pn.x, pn.y)) {
                        const ui = Math.abs(Nn - pn.y) / (Cn - pn.x);
                        Ho(pn, zt) && (ui < xi || ui === xi && (pn.x > Wn.x || pn.x === Wn.x && oi(Wn, pn))) && (Wn = pn, xi = ui);
                      }
                      pn = pn.next;
                    } while (pn !== xr);
                    return Wn;
                  }(Ye, He);
                  if (!ot) return He;
                  const xt = Co(ot, Ye);
                  return td(xt, xt.next), td(ot, ot.next);
                }
                function oi(Ye, He) {
                  return En(Ye.prev, Ye, He.prev) < 0 && En(He.next, Ye, Ye.next) < 0;
                }
                function ar(Ye, He, ot, xt, zt) {
                  return (Ye = 1431655765 & ((Ye = 858993459 & ((Ye = 252645135 & ((Ye = 16711935 & ((Ye = (Ye - ot) * zt | 0) | Ye << 8)) | Ye << 4)) | Ye << 2)) | Ye << 1)) | (He = 1431655765 & ((He = 858993459 & ((He = 252645135 & ((He = 16711935 & ((He = (He - xt) * zt | 0) | He << 8)) | He << 4)) | He << 2)) | He << 1)) << 1;
                }
                function Sa(Ye) {
                  let He = Ye, ot = Ye;
                  do
                    (He.x < ot.x || He.x === ot.x && He.y < ot.y) && (ot = He), He = He.next;
                  while (He !== Ye);
                  return ot;
                }
                function Qn(Ye, He, ot, xt, zt, ln, pn, Cn) {
                  return (zt - pn) * (He - Cn) >= (Ye - pn) * (ln - Cn) && (Ye - pn) * (xt - Cn) >= (ot - pn) * (He - Cn) && (ot - pn) * (ln - Cn) >= (zt - pn) * (xt - Cn);
                }
                function za(Ye, He) {
                  return Ye.next.i !== He.i && Ye.prev.i !== He.i && !function(ot, xt) {
                    let zt = ot;
                    do {
                      if (zt.i !== ot.i && zt.next.i !== ot.i && zt.i !== xt.i && zt.next.i !== xt.i && lo(zt, zt.next, ot, xt)) return !0;
                      zt = zt.next;
                    } while (zt !== ot);
                    return !1;
                  }(Ye, He) && (Ho(Ye, He) && Ho(He, Ye) && function(ot, xt) {
                    let zt = ot, ln = !1;
                    const pn = (ot.x + xt.x) / 2, Cn = (ot.y + xt.y) / 2;
                    do
                      zt.y > Cn != zt.next.y > Cn && zt.next.y !== zt.y && pn < (zt.next.x - zt.x) * (Cn - zt.y) / (zt.next.y - zt.y) + zt.x && (ln = !ln), zt = zt.next;
                    while (zt !== ot);
                    return ln;
                  }(Ye, He) && (En(Ye.prev, Ye, He.prev) || En(Ye, He.prev, He)) || Qo(Ye, He) && En(Ye.prev, Ye, Ye.next) > 0 && En(He.prev, He, He.next) > 0);
                }
                function En(Ye, He, ot) {
                  return (He.y - Ye.y) * (ot.x - He.x) - (He.x - Ye.x) * (ot.y - He.y);
                }
                function Qo(Ye, He) {
                  return Ye.x === He.x && Ye.y === He.y;
                }
                function lo(Ye, He, ot, xt) {
                  const zt = bo(En(Ye, He, ot)), ln = bo(En(Ye, He, xt)), pn = bo(En(ot, xt, Ye)), Cn = bo(En(ot, xt, He));
                  return zt !== ln && pn !== Cn || !(zt !== 0 || !Eo(Ye, ot, He)) || !(ln !== 0 || !Eo(Ye, xt, He)) || !(pn !== 0 || !Eo(ot, Ye, xt)) || !(Cn !== 0 || !Eo(ot, He, xt));
                }
                function Eo(Ye, He, ot) {
                  return He.x <= Math.max(Ye.x, ot.x) && He.x >= Math.min(Ye.x, ot.x) && He.y <= Math.max(Ye.y, ot.y) && He.y >= Math.min(Ye.y, ot.y);
                }
                function bo(Ye) {
                  return Ye > 0 ? 1 : Ye < 0 ? -1 : 0;
                }
                function Ho(Ye, He) {
                  return En(Ye.prev, Ye, Ye.next) < 0 ? En(Ye, He, Ye.next) >= 0 && En(Ye, Ye.prev, He) >= 0 : En(Ye, He, Ye.prev) < 0 || En(Ye, Ye.next, He) < 0;
                }
                function Co(Ye, He) {
                  const ot = Qa(Ye.i, Ye.x, Ye.y), xt = Qa(He.i, He.x, He.y), zt = Ye.next, ln = He.prev;
                  return Ye.next = He, He.prev = Ye, ot.next = zt, zt.prev = ot, xt.next = ot, ot.prev = xt, ln.next = xt, xt.prev = ln, xt;
                }
                function Ao(Ye, He, ot, xt) {
                  const zt = Qa(Ye, He, ot);
                  return xt ? (zt.next = xt.next, zt.prev = xt, xt.next.prev = zt, xt.next = zt) : (zt.prev = zt, zt.next = zt), zt;
                }
                function Ro(Ye) {
                  Ye.next.prev = Ye.prev, Ye.prev.next = Ye.next, Ye.prevZ && (Ye.prevZ.nextZ = Ye.nextZ), Ye.nextZ && (Ye.nextZ.prevZ = Ye.prevZ);
                }
                function Qa(Ye, He, ot) {
                  return { i: Ye, x: He, y: ot, prev: null, next: null, z: 0, prevZ: null, nextZ: null, steiner: !1 };
                }
                function Ka(Ye, He, ot) {
                  const xt = ot.patternDependencies;
                  let zt = !1;
                  for (const ln of He) {
                    const pn = ln.paint.get(`${Ye}-pattern`);
                    pn.isConstant() || (zt = !0);
                    const Cn = pn.constantOr(null);
                    Cn && (zt = !0, xt[Cn.to] = !0, xt[Cn.from] = !0);
                  }
                  return zt;
                }
                function qo(Ye, He, ot, xt, zt) {
                  const ln = zt.patternDependencies;
                  for (const pn of He) {
                    const Cn = pn.paint.get(`${Ye}-pattern`).value;
                    if (Cn.kind !== "constant") {
                      let Nn = Cn.evaluate({ zoom: xt - 1 }, ot, {}, zt.availableImages), Wn = Cn.evaluate({ zoom: xt }, ot, {}, zt.availableImages), lr = Cn.evaluate({ zoom: xt + 1 }, ot, {}, zt.availableImages);
                      Nn = Nn && Nn.name ? Nn.name : Nn, Wn = Wn && Wn.name ? Wn.name : Wn, lr = lr && lr.name ? lr.name : lr, ln[Nn] = !0, ln[Wn] = !0, ln[lr] = !0, ot.patterns[pn.id] = { min: Nn, mid: Wn, max: lr };
                    }
                  }
                  return ot;
                }
                class Ss {
                  constructor(He) {
                    this.zoom = He.zoom, this.overscaling = He.overscaling, this.layers = He.layers, this.layerIds = this.layers.map((ot) => ot.id), this.index = He.index, this.hasPattern = !1, this.patternFeatures = [], this.layoutVertexArray = new ys(), this.indexArray = new Dl(), this.indexArray2 = new Ml(), this.programConfigurations = new Qf(He.layers, He.zoom), this.segments = new al(), this.segments2 = new al(), this.stateDependentLayerIds = this.layers.filter((ot) => ot.isStateDependent()).map((ot) => ot.id);
                  }
                  populate(He, ot, xt) {
                    this.hasPattern = Ka("fill", this.layers, ot);
                    const zt = this.layers[0].layout.get("fill-sort-key"), ln = !zt.isConstant(), pn = [];
                    for (const { feature: Cn, id: Nn, index: Wn, sourceLayerIndex: lr } of He) {
                      const xr = this.layers[0]._featureFilter.needGeometry, Nr = Sd(Cn, xr);
                      if (!this.layers[0]._featureFilter.filter(new Ko(this.zoom), Nr, xt)) continue;
                      const Kr = ln ? zt.evaluate(Nr, {}, xt, ot.availableImages) : void 0, xi = { id: Nn, properties: Cn.properties, type: Cn.type, sourceLayerIndex: lr, index: Wn, geometry: xr ? Nr.geometry : $d(Cn), patterns: {}, sortKey: Kr };
                      pn.push(xi);
                    }
                    ln && pn.sort((Cn, Nn) => Cn.sortKey - Nn.sortKey);
                    for (const Cn of pn) {
                      const { geometry: Nn, index: Wn, sourceLayerIndex: lr } = Cn;
                      if (this.hasPattern) {
                        const Nr = qo("fill", this.layers, Cn, this.zoom, ot);
                        this.patternFeatures.push(Nr);
                      } else this.addFeature(Cn, Nn, Wn, xt, {});
                      const xr = He[Wn].feature;
                      ot.featureIndex.insert(xr, Nn, Wn, lr, this.index);
                    }
                  }
                  update(He, ot, xt) {
                    this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(He, ot, this.stateDependentLayers, xt);
                  }
                  addFeatures(He, ot, xt) {
                    for (const zt of this.patternFeatures) this.addFeature(zt, zt.geometry, zt.index, ot, xt);
                  }
                  isEmpty() {
                    return this.layoutVertexArray.length === 0;
                  }
                  uploadPending() {
                    return !this.uploaded || this.programConfigurations.needsUpload;
                  }
                  upload(He) {
                    this.uploaded || (this.layoutVertexBuffer = He.createVertexBuffer(this.layoutVertexArray, cf), this.indexBuffer = He.createIndexBuffer(this.indexArray), this.indexBuffer2 = He.createIndexBuffer(this.indexArray2)), this.programConfigurations.upload(He), this.uploaded = !0;
                  }
                  destroy() {
                    this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.indexBuffer2.destroy(), this.programConfigurations.destroy(), this.segments.destroy(), this.segments2.destroy());
                  }
                  addFeature(He, ot, xt, zt, ln) {
                    for (const pn of Sl(ot, 500)) {
                      let Cn = 0;
                      for (const Kr of pn) Cn += Kr.length;
                      const Nn = this.segments.prepareSegment(Cn, this.layoutVertexArray, this.indexArray), Wn = Nn.vertexLength, lr = [], xr = [];
                      for (const Kr of pn) {
                        if (Kr.length === 0) continue;
                        Kr !== pn[0] && xr.push(lr.length / 2);
                        const xi = this.segments2.prepareSegment(Kr.length, this.layoutVertexArray, this.indexArray2), ui = xi.vertexLength;
                        this.layoutVertexArray.emplaceBack(Kr[0].x, Kr[0].y), this.indexArray2.emplaceBack(ui + Kr.length - 1, ui), lr.push(Kr[0].x), lr.push(Kr[0].y);
                        for (let Ui = 1; Ui < Kr.length; Ui++) this.layoutVertexArray.emplaceBack(Kr[Ui].x, Kr[Ui].y), this.indexArray2.emplaceBack(ui + Ui - 1, ui + Ui), lr.push(Kr[Ui].x), lr.push(Kr[Ui].y);
                        xi.vertexLength += Kr.length, xi.primitiveLength += Kr.length;
                      }
                      const Nr = Jf(lr, xr);
                      for (let Kr = 0; Kr < Nr.length; Kr += 3) this.indexArray.emplaceBack(Wn + Nr[Kr], Wn + Nr[Kr + 1], Wn + Nr[Kr + 2]);
                      Nn.vertexLength += Cn, Nn.primitiveLength += Nr.length / 3;
                    }
                    this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, He, xt, ln, zt);
                  }
                }
                let ho;
                ss("FillBucket", Ss, { omit: ["layers", "patternFeatures"] });
                let hs;
                var Bo = { get paint() {
                  return hs = hs || new tu({ "fill-antialias": new zs(yt.paint_fill["fill-antialias"]), "fill-opacity": new Os(yt.paint_fill["fill-opacity"]), "fill-color": new Os(yt.paint_fill["fill-color"]), "fill-outline-color": new Os(yt.paint_fill["fill-outline-color"]), "fill-translate": new zs(yt.paint_fill["fill-translate"]), "fill-translate-anchor": new zs(yt.paint_fill["fill-translate-anchor"]), "fill-pattern": new lc(yt.paint_fill["fill-pattern"]) });
                }, get layout() {
                  return ho = ho || new tu({ "fill-sort-key": new Os(yt.layout_fill["fill-sort-key"]) });
                } };
                class Vs extends $n {
                  constructor(He) {
                    super(He, Bo);
                  }
                  recalculate(He, ot) {
                    super.recalculate(He, ot);
                    const xt = this.paint._values["fill-outline-color"];
                    xt.value.kind === "constant" && xt.value.value === void 0 && (this.paint._values["fill-outline-color"] = this.paint._values["fill-color"]);
                  }
                  createBucket(He) {
                    return new Ss(He);
                  }
                  queryRadius() {
                    return Zd(this.paint.get("fill-translate"));
                  }
                  queryIntersectsFeature(He, ot, xt, zt, ln, pn, Cn) {
                    return qd(Lf(He, this.paint.get("fill-translate"), this.paint.get("fill-translate-anchor"), pn.angle, Cn), zt);
                  }
                  isTileClipped() {
                    return !0;
                  }
                }
                const vo = Fe([{ name: "a_pos", components: 2, type: "Int16" }, { name: "a_normal_ed", components: 4, type: "Int16" }], 4), go = Fe([{ name: "a_centroid", components: 2, type: "Int16" }], 4), { members: Qs, size: us, alignment: Po } = vo;
                var Jo = {}, ps = oe, wl = ol;
                function ol(Ye, He, ot, xt, zt) {
                  this.properties = {}, this.extent = ot, this.type = 0, this._pbf = Ye, this._geometry = -1, this._keys = xt, this._values = zt, Ye.readFields(xs, this, He);
                }
                function xs(Ye, He, ot) {
                  Ye == 1 ? He.id = ot.readVarint() : Ye == 2 ? function(xt, zt) {
                    for (var ln = xt.readVarint() + xt.pos; xt.pos < ln; ) {
                      var pn = zt._keys[xt.readVarint()], Cn = zt._values[xt.readVarint()];
                      zt.properties[pn] = Cn;
                    }
                  }(ot, He) : Ye == 3 ? He.type = ot.readVarint() : Ye == 4 && (He._geometry = ot.pos);
                }
                function ns(Ye) {
                  for (var He, ot, xt = 0, zt = 0, ln = Ye.length, pn = ln - 1; zt < ln; pn = zt++) He = Ye[zt], xt += ((ot = Ye[pn]).x - He.x) * (He.y + ot.y);
                  return xt;
                }
                ol.types = ["Unknown", "Point", "LineString", "Polygon"], ol.prototype.loadGeometry = function() {
                  var Ye = this._pbf;
                  Ye.pos = this._geometry;
                  for (var He, ot = Ye.readVarint() + Ye.pos, xt = 1, zt = 0, ln = 0, pn = 0, Cn = []; Ye.pos < ot; ) {
                    if (zt <= 0) {
                      var Nn = Ye.readVarint();
                      xt = 7 & Nn, zt = Nn >> 3;
                    }
                    if (zt--, xt === 1 || xt === 2) ln += Ye.readSVarint(), pn += Ye.readSVarint(), xt === 1 && (He && Cn.push(He), He = []), He.push(new ps(ln, pn));
                    else {
                      if (xt !== 7) throw new Error("unknown command " + xt);
                      He && He.push(He[0].clone());
                    }
                  }
                  return He && Cn.push(He), Cn;
                }, ol.prototype.bbox = function() {
                  var Ye = this._pbf;
                  Ye.pos = this._geometry;
                  for (var He = Ye.readVarint() + Ye.pos, ot = 1, xt = 0, zt = 0, ln = 0, pn = 1 / 0, Cn = -1 / 0, Nn = 1 / 0, Wn = -1 / 0; Ye.pos < He; ) {
                    if (xt <= 0) {
                      var lr = Ye.readVarint();
                      ot = 7 & lr, xt = lr >> 3;
                    }
                    if (xt--, ot === 1 || ot === 2) (zt += Ye.readSVarint()) < pn && (pn = zt), zt > Cn && (Cn = zt), (ln += Ye.readSVarint()) < Nn && (Nn = ln), ln > Wn && (Wn = ln);
                    else if (ot !== 7) throw new Error("unknown command " + ot);
                  }
                  return [pn, Nn, Cn, Wn];
                }, ol.prototype.toGeoJSON = function(Ye, He, ot) {
                  var xt, zt, ln = this.extent * Math.pow(2, ot), pn = this.extent * Ye, Cn = this.extent * He, Nn = this.loadGeometry(), Wn = ol.types[this.type];
                  function lr(Kr) {
                    for (var xi = 0; xi < Kr.length; xi++) {
                      var ui = Kr[xi], Ui = 180 - 360 * (ui.y + Cn) / ln;
                      Kr[xi] = [360 * (ui.x + pn) / ln - 180, 360 / Math.PI * Math.atan(Math.exp(Ui * Math.PI / 180)) - 90];
                    }
                  }
                  switch (this.type) {
                    case 1:
                      var xr = [];
                      for (xt = 0; xt < Nn.length; xt++) xr[xt] = Nn[xt][0];
                      lr(Nn = xr);
                      break;
                    case 2:
                      for (xt = 0; xt < Nn.length; xt++) lr(Nn[xt]);
                      break;
                    case 3:
                      for (Nn = function(Kr) {
                        var xi = Kr.length;
                        if (xi <= 1) return [Kr];
                        for (var ui, Ui, ia = [], Na = 0; Na < xi; Na++) {
                          var Ta = ns(Kr[Na]);
                          Ta !== 0 && (Ui === void 0 && (Ui = Ta < 0), Ui === Ta < 0 ? (ui && ia.push(ui), ui = [Kr[Na]]) : ui.push(Kr[Na]));
                        }
                        return ui && ia.push(ui), ia;
                      }(Nn), xt = 0; xt < Nn.length; xt++) for (zt = 0; zt < Nn[xt].length; zt++) lr(Nn[xt][zt]);
                  }
                  Nn.length === 1 ? Nn = Nn[0] : Wn = "Multi" + Wn;
                  var Nr = { type: "Feature", geometry: { type: Wn, coordinates: Nn }, properties: this.properties };
                  return "id" in this && (Nr.id = this.id), Nr;
                };
                var ll = wl, Ws = ec;
                function ec(Ye, He) {
                  this.version = 1, this.name = null, this.extent = 4096, this.length = 0, this._pbf = Ye, this._keys = [], this._values = [], this._features = [], Ye.readFields(s1, this, He), this.length = this._features.length;
                }
                function s1(Ye, He, ot) {
                  Ye === 15 ? He.version = ot.readVarint() : Ye === 1 ? He.name = ot.readString() : Ye === 5 ? He.extent = ot.readVarint() : Ye === 2 ? He._features.push(ot.pos) : Ye === 3 ? He._keys.push(ot.readString()) : Ye === 4 && He._values.push(function(xt) {
                    for (var zt = null, ln = xt.readVarint() + xt.pos; xt.pos < ln; ) {
                      var pn = xt.readVarint() >> 3;
                      zt = pn === 1 ? xt.readString() : pn === 2 ? xt.readFloat() : pn === 3 ? xt.readDouble() : pn === 4 ? xt.readVarint64() : pn === 5 ? xt.readVarint() : pn === 6 ? xt.readSVarint() : pn === 7 ? xt.readBoolean() : null;
                    }
                    return zt;
                  }(ot));
                }
                ec.prototype.feature = function(Ye) {
                  if (Ye < 0 || Ye >= this._features.length) throw new Error("feature index out of bounds");
                  this._pbf.pos = this._features[Ye];
                  var He = this._pbf.readVarint() + this._pbf.pos;
                  return new ll(this._pbf, He, this.extent, this._keys, this._values);
                };
                var n1 = Ws, Js = function(Ye, He) {
                  this.layers = Ye.readFields(N1, {}, He);
                };
                function N1(Ye, He, ot) {
                  if (Ye === 3) {
                    var xt = new n1(ot, ot.readVarint() + ot.pos);
                    xt.length && (He[xt.name] = xt);
                  }
                }
                Jo.VectorTile = Js, Jo.VectorTileFeature = wl, Jo.VectorTileLayer = Ws;
                const Xu = Jo.VectorTileFeature.types, Q1 = Math.pow(2, 13);
                function Qu(Ye, He, ot, xt, zt, ln, pn, Cn) {
                  Ye.emplaceBack(He, ot, 2 * Math.floor(xt * Q1) + pn, zt * Q1 * 2, ln * Q1 * 2, Math.round(Cn));
                }
                class r1 {
                  constructor(He) {
                    this.zoom = He.zoom, this.overscaling = He.overscaling, this.layers = He.layers, this.layerIds = this.layers.map((ot) => ot.id), this.index = He.index, this.hasPattern = !1, this.layoutVertexArray = new il(), this.centroidVertexArray = new Ja(), this.indexArray = new Dl(), this.programConfigurations = new Qf(He.layers, He.zoom), this.segments = new al(), this.stateDependentLayerIds = this.layers.filter((ot) => ot.isStateDependent()).map((ot) => ot.id);
                  }
                  populate(He, ot, xt) {
                    this.features = [], this.hasPattern = Ka("fill-extrusion", this.layers, ot);
                    for (const { feature: zt, id: ln, index: pn, sourceLayerIndex: Cn } of He) {
                      const Nn = this.layers[0]._featureFilter.needGeometry, Wn = Sd(zt, Nn);
                      if (!this.layers[0]._featureFilter.filter(new Ko(this.zoom), Wn, xt)) continue;
                      const lr = { id: ln, sourceLayerIndex: Cn, index: pn, geometry: Nn ? Wn.geometry : $d(zt), properties: zt.properties, type: zt.type, patterns: {} };
                      this.hasPattern ? this.features.push(qo("fill-extrusion", this.layers, lr, this.zoom, ot)) : this.addFeature(lr, lr.geometry, pn, xt, {}), ot.featureIndex.insert(zt, lr.geometry, pn, Cn, this.index, !0);
                    }
                  }
                  addFeatures(He, ot, xt) {
                    for (const zt of this.features) {
                      const { geometry: ln } = zt;
                      this.addFeature(zt, ln, zt.index, ot, xt);
                    }
                  }
                  update(He, ot, xt) {
                    this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(He, ot, this.stateDependentLayers, xt);
                  }
                  isEmpty() {
                    return this.layoutVertexArray.length === 0 && this.centroidVertexArray.length === 0;
                  }
                  uploadPending() {
                    return !this.uploaded || this.programConfigurations.needsUpload;
                  }
                  upload(He) {
                    this.uploaded || (this.layoutVertexBuffer = He.createVertexBuffer(this.layoutVertexArray, Qs), this.centroidVertexBuffer = He.createVertexBuffer(this.centroidVertexArray, go.members, !0), this.indexBuffer = He.createIndexBuffer(this.indexArray)), this.programConfigurations.upload(He), this.uploaded = !0;
                  }
                  destroy() {
                    this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy(), this.centroidVertexBuffer.destroy());
                  }
                  addFeature(He, ot, xt, zt, ln) {
                    for (const pn of Sl(ot, 500)) {
                      const Cn = { x: 0, y: 0, vertexCount: 0 };
                      let Nn = 0;
                      for (const xi of pn) Nn += xi.length;
                      let Wn = this.segments.prepareSegment(4, this.layoutVertexArray, this.indexArray);
                      for (const xi of pn) {
                        if (xi.length === 0 || Ed(xi)) continue;
                        let ui = 0;
                        for (let Ui = 0; Ui < xi.length; Ui++) {
                          const ia = xi[Ui];
                          if (Ui >= 1) {
                            const Na = xi[Ui - 1];
                            if (!ud(ia, Na)) {
                              Wn.vertexLength + 4 > al.MAX_VERTEX_ARRAY_LENGTH && (Wn = this.segments.prepareSegment(4, this.layoutVertexArray, this.indexArray));
                              const Ta = ia.sub(Na)._perp()._unit(), Ra = Na.dist(ia);
                              ui + Ra > 32768 && (ui = 0), Qu(this.layoutVertexArray, ia.x, ia.y, Ta.x, Ta.y, 0, 0, ui), Qu(this.layoutVertexArray, ia.x, ia.y, Ta.x, Ta.y, 0, 1, ui), Cn.x += 2 * ia.x, Cn.y += 2 * ia.y, Cn.vertexCount += 2, ui += Ra, Qu(this.layoutVertexArray, Na.x, Na.y, Ta.x, Ta.y, 0, 0, ui), Qu(this.layoutVertexArray, Na.x, Na.y, Ta.x, Ta.y, 0, 1, ui), Cn.x += 2 * Na.x, Cn.y += 2 * Na.y, Cn.vertexCount += 2;
                              const oo = Wn.vertexLength;
                              this.indexArray.emplaceBack(oo, oo + 2, oo + 1), this.indexArray.emplaceBack(oo + 1, oo + 2, oo + 3), Wn.vertexLength += 4, Wn.primitiveLength += 2;
                            }
                          }
                        }
                      }
                      if (Wn.vertexLength + Nn > al.MAX_VERTEX_ARRAY_LENGTH && (Wn = this.segments.prepareSegment(Nn, this.layoutVertexArray, this.indexArray)), Xu[He.type] !== "Polygon") continue;
                      const lr = [], xr = [], Nr = Wn.vertexLength;
                      for (const xi of pn) if (xi.length !== 0) {
                        xi !== pn[0] && xr.push(lr.length / 2);
                        for (let ui = 0; ui < xi.length; ui++) {
                          const Ui = xi[ui];
                          Qu(this.layoutVertexArray, Ui.x, Ui.y, 0, 0, 1, 1, 0), Cn.x += Ui.x, Cn.y += Ui.y, Cn.vertexCount += 1, lr.push(Ui.x), lr.push(Ui.y);
                        }
                      }
                      const Kr = Jf(lr, xr);
                      for (let xi = 0; xi < Kr.length; xi += 3) this.indexArray.emplaceBack(Nr + Kr[xi], Nr + Kr[xi + 2], Nr + Kr[xi + 1]);
                      Wn.primitiveLength += Kr.length / 3, Wn.vertexLength += Nn;
                      for (let xi = 0; xi < Cn.vertexCount; xi++) {
                        const ui = Math.floor(Cn.x / Cn.vertexCount), Ui = Math.floor(Cn.y / Cn.vertexCount);
                        this.centroidVertexArray.emplaceBack(ui, Ui);
                      }
                    }
                    this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, He, xt, ln, zt);
                  }
                }
                function ud(Ye, He) {
                  return Ye.x === He.x && (Ye.x < 0 || Ye.x > Uu) || Ye.y === He.y && (Ye.y < 0 || Ye.y > Uu);
                }
                function Ed(Ye) {
                  return Ye.every((He) => He.x < 0) || Ye.every((He) => He.x > Uu) || Ye.every((He) => He.y < 0) || Ye.every((He) => He.y > Uu);
                }
                let R1;
                ss("FillExtrusionBucket", r1, { omit: ["layers", "features"] });
                var jl = { get paint() {
                  return R1 = R1 || new tu({ "fill-extrusion-opacity": new zs(yt["paint_fill-extrusion"]["fill-extrusion-opacity"]), "fill-extrusion-color": new Os(yt["paint_fill-extrusion"]["fill-extrusion-color"]), "fill-extrusion-translate": new zs(yt["paint_fill-extrusion"]["fill-extrusion-translate"]), "fill-extrusion-translate-anchor": new zs(yt["paint_fill-extrusion"]["fill-extrusion-translate-anchor"]), "fill-extrusion-pattern": new lc(yt["paint_fill-extrusion"]["fill-extrusion-pattern"]), "fill-extrusion-height": new Os(yt["paint_fill-extrusion"]["fill-extrusion-height"]), "fill-extrusion-base": new Os(yt["paint_fill-extrusion"]["fill-extrusion-base"]), "fill-extrusion-vertical-gradient": new zs(yt["paint_fill-extrusion"]["fill-extrusion-vertical-gradient"]) });
                } };
                class Ad extends $n {
                  constructor(He) {
                    super(He, jl);
                  }
                  createBucket(He) {
                    return new r1(He);
                  }
                  queryRadius() {
                    return Zd(this.paint.get("fill-extrusion-translate"));
                  }
                  is3D() {
                    return !0;
                  }
                  queryIntersectsFeature(He, ot, xt, zt, ln, pn, Cn, Nn) {
                    const Wn = Lf(He, this.paint.get("fill-extrusion-translate"), this.paint.get("fill-extrusion-translate-anchor"), pn.angle, Cn), lr = this.paint.get("fill-extrusion-height").evaluate(ot, xt), xr = this.paint.get("fill-extrusion-base").evaluate(ot, xt), Nr = function(xi, ui, Ui, ia) {
                      const Na = [];
                      for (const Ta of xi) {
                        const Ra = [Ta.x, Ta.y, ia, 1];
                        Zf(Ra, Ra, ui), Na.push(new ce(Ra[0] / Ra[3], Ra[1] / Ra[3]));
                      }
                      return Na;
                    }(Wn, Nn, 0, 0), Kr = function(xi, ui, Ui, ia) {
                      const Na = [], Ta = [], Ra = ia[8] * ui, oo = ia[9] * ui, fs = ia[10] * ui, Bs = ia[11] * ui, fl = ia[8] * Ui, Ns = ia[9] * Ui, Es = ia[10] * Ui, cl = ia[11] * Ui;
                      for (const Xs of xi) {
                        const Ps = [], ko = [];
                        for (const Ks of Xs) {
                          const el = Ks.x, Nl = Ks.y, bu = ia[0] * el + ia[4] * Nl + ia[12], Tu = ia[1] * el + ia[5] * Nl + ia[13], B1 = ia[2] * el + ia[6] * Nl + ia[14], Hd = ia[3] * el + ia[7] * Nl + ia[15], nd = B1 + fs, j1 = Hd + Bs, Fd = bu + fl, Cd = Tu + Ns, L1 = B1 + Es, m1 = Hd + cl, xd = new ce((bu + Ra) / j1, (Tu + oo) / j1);
                          xd.z = nd / j1, Ps.push(xd);
                          const H1 = new ce(Fd / m1, Cd / m1);
                          H1.z = L1 / m1, ko.push(H1);
                        }
                        Na.push(Ps), Ta.push(ko);
                      }
                      return [Na, Ta];
                    }(zt, xr, lr, Nn);
                    return function(xi, ui, Ui) {
                      let ia = 1 / 0;
                      qd(Ui, ui) && (ia = yl(Ui, ui[0]));
                      for (let Na = 0; Na < ui.length; Na++) {
                        const Ta = ui[Na], Ra = xi[Na];
                        for (let oo = 0; oo < Ta.length - 1; oo++) {
                          const fs = Ta[oo], Bs = Ta[oo + 1], fl = Ra[oo], Ns = [fs, Bs, Ra[oo + 1], fl, fs];
                          of(Ui, Ns) && (ia = Math.min(ia, yl(Ui, Ns)));
                        }
                      }
                      return ia !== 1 / 0 && ia;
                    }(Kr[0], Kr[1], Nr);
                  }
                }
                function Fc(Ye, He) {
                  return Ye.x * He.x + Ye.y * He.y;
                }
                function yl(Ye, He) {
                  if (Ye.length === 1) {
                    let ot = 0;
                    const xt = He[ot++];
                    let zt;
                    for (; !zt || xt.equals(zt); ) if (zt = He[ot++], !zt) return 1 / 0;
                    for (; ot < He.length; ot++) {
                      const ln = He[ot], pn = Ye[0], Cn = zt.sub(xt), Nn = ln.sub(xt), Wn = pn.sub(xt), lr = Fc(Cn, Cn), xr = Fc(Cn, Nn), Nr = Fc(Nn, Nn), Kr = Fc(Wn, Cn), xi = Fc(Wn, Nn), ui = lr * Nr - xr * xr, Ui = (Nr * Kr - xr * xi) / ui, ia = (lr * xi - xr * Kr) / ui, Na = 1 - Ui - ia, Ta = xt.z * Na + zt.z * Ui + ln.z * ia;
                      if (isFinite(Ta)) return Ta;
                    }
                    return 1 / 0;
                  }
                  {
                    let ot = 1 / 0;
                    for (const xt of He) ot = Math.min(ot, xt.z);
                    return ot;
                  }
                }
                const cc = Fe([{ name: "a_pos_normal", components: 2, type: "Int16" }, { name: "a_data", components: 4, type: "Uint8" }], 4), { members: Bc, size: U1, alignment: uf } = cc, df = Fe([{ name: "a_uv_x", components: 1, type: "Float32" }, { name: "a_split_index", components: 1, type: "Float32" }]), { members: Xi, size: Za, alignment: ja } = df, Ds = Jo.VectorTileFeature.types, rc = Math.cos(Math.PI / 180 * 37.5), Di = Math.pow(2, 14) / 0.5;
                class yi {
                  constructor(He) {
                    this.zoom = He.zoom, this.overscaling = He.overscaling, this.layers = He.layers, this.layerIds = this.layers.map((ot) => ot.id), this.index = He.index, this.hasPattern = !1, this.patternFeatures = [], this.lineClipsArray = [], this.gradients = {}, this.layers.forEach((ot) => {
                      this.gradients[ot.id] = {};
                    }), this.layoutVertexArray = new Is(), this.layoutVertexArray2 = new ml(), this.indexArray = new Dl(), this.programConfigurations = new Qf(He.layers, He.zoom), this.segments = new al(), this.maxLineLength = 0, this.stateDependentLayerIds = this.layers.filter((ot) => ot.isStateDependent()).map((ot) => ot.id);
                  }
                  populate(He, ot, xt) {
                    this.hasPattern = Ka("line", this.layers, ot);
                    const zt = this.layers[0].layout.get("line-sort-key"), ln = !zt.isConstant(), pn = [];
                    for (const { feature: Cn, id: Nn, index: Wn, sourceLayerIndex: lr } of He) {
                      const xr = this.layers[0]._featureFilter.needGeometry, Nr = Sd(Cn, xr);
                      if (!this.layers[0]._featureFilter.filter(new Ko(this.zoom), Nr, xt)) continue;
                      const Kr = ln ? zt.evaluate(Nr, {}, xt) : void 0, xi = { id: Nn, properties: Cn.properties, type: Cn.type, sourceLayerIndex: lr, index: Wn, geometry: xr ? Nr.geometry : $d(Cn), patterns: {}, sortKey: Kr };
                      pn.push(xi);
                    }
                    ln && pn.sort((Cn, Nn) => Cn.sortKey - Nn.sortKey);
                    for (const Cn of pn) {
                      const { geometry: Nn, index: Wn, sourceLayerIndex: lr } = Cn;
                      if (this.hasPattern) {
                        const Nr = qo("line", this.layers, Cn, this.zoom, ot);
                        this.patternFeatures.push(Nr);
                      } else this.addFeature(Cn, Nn, Wn, xt, {});
                      const xr = He[Wn].feature;
                      ot.featureIndex.insert(xr, Nn, Wn, lr, this.index);
                    }
                  }
                  update(He, ot, xt) {
                    this.stateDependentLayers.length && this.programConfigurations.updatePaintArrays(He, ot, this.stateDependentLayers, xt);
                  }
                  addFeatures(He, ot, xt) {
                    for (const zt of this.patternFeatures) this.addFeature(zt, zt.geometry, zt.index, ot, xt);
                  }
                  isEmpty() {
                    return this.layoutVertexArray.length === 0;
                  }
                  uploadPending() {
                    return !this.uploaded || this.programConfigurations.needsUpload;
                  }
                  upload(He) {
                    this.uploaded || (this.layoutVertexArray2.length !== 0 && (this.layoutVertexBuffer2 = He.createVertexBuffer(this.layoutVertexArray2, Xi)), this.layoutVertexBuffer = He.createVertexBuffer(this.layoutVertexArray, Bc), this.indexBuffer = He.createIndexBuffer(this.indexArray)), this.programConfigurations.upload(He), this.uploaded = !0;
                  }
                  destroy() {
                    this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy());
                  }
                  lineFeatureClips(He) {
                    if (He.properties && Object.prototype.hasOwnProperty.call(He.properties, "mapbox_clip_start") && Object.prototype.hasOwnProperty.call(He.properties, "mapbox_clip_end")) return { start: +He.properties.mapbox_clip_start, end: +He.properties.mapbox_clip_end };
                  }
                  addFeature(He, ot, xt, zt, ln) {
                    const pn = this.layers[0].layout, Cn = pn.get("line-join").evaluate(He, {}), Nn = pn.get("line-cap"), Wn = pn.get("line-miter-limit"), lr = pn.get("line-round-limit");
                    this.lineClips = this.lineFeatureClips(He);
                    for (const xr of ot) this.addLine(xr, He, Cn, Nn, Wn, lr);
                    this.programConfigurations.populatePaintArrays(this.layoutVertexArray.length, He, xt, ln, zt);
                  }
                  addLine(He, ot, xt, zt, ln, pn) {
                    if (this.distance = 0, this.scaledDistance = 0, this.totalDistance = 0, this.lineClips) {
                      this.lineClipsArray.push(this.lineClips);
                      for (let ia = 0; ia < He.length - 1; ia++) this.totalDistance += He[ia].dist(He[ia + 1]);
                      this.updateScaledDistance(), this.maxLineLength = Math.max(this.maxLineLength, this.totalDistance);
                    }
                    const Cn = Ds[ot.type] === "Polygon";
                    let Nn = He.length;
                    for (; Nn >= 2 && He[Nn - 1].equals(He[Nn - 2]); ) Nn--;
                    let Wn = 0;
                    for (; Wn < Nn - 1 && He[Wn].equals(He[Wn + 1]); ) Wn++;
                    if (Nn < (Cn ? 3 : 2)) return;
                    xt === "bevel" && (ln = 1.05);
                    const lr = this.overscaling <= 16 ? 15 * Uu / (512 * this.overscaling) : 0, xr = this.segments.prepareSegment(10 * Nn, this.layoutVertexArray, this.indexArray);
                    let Nr, Kr, xi, ui, Ui;
                    this.e1 = this.e2 = -1, Cn && (Nr = He[Nn - 2], Ui = He[Wn].sub(Nr)._unit()._perp());
                    for (let ia = Wn; ia < Nn; ia++) {
                      if (xi = ia === Nn - 1 ? Cn ? He[Wn + 1] : void 0 : He[ia + 1], xi && He[ia].equals(xi)) continue;
                      Ui && (ui = Ui), Nr && (Kr = Nr), Nr = He[ia], Ui = xi ? xi.sub(Nr)._unit()._perp() : ui, ui = ui || Ui;
                      let Na = ui.add(Ui);
                      Na.x === 0 && Na.y === 0 || Na._unit();
                      const Ta = ui.x * Ui.x + ui.y * Ui.y, Ra = Na.x * Ui.x + Na.y * Ui.y, oo = Ra !== 0 ? 1 / Ra : 1 / 0, fs = 2 * Math.sqrt(2 - 2 * Ra), Bs = Ra < rc && Kr && xi, fl = ui.x * Ui.y - ui.y * Ui.x > 0;
                      if (Bs && ia > Wn) {
                        const cl = Nr.dist(Kr);
                        if (cl > 2 * lr) {
                          const Xs = Nr.sub(Nr.sub(Kr)._mult(lr / cl)._round());
                          this.updateDistance(Kr, Xs), this.addCurrentVertex(Xs, ui, 0, 0, xr), Kr = Xs;
                        }
                      }
                      const Ns = Kr && xi;
                      let Es = Ns ? xt : Cn ? "butt" : zt;
                      if (Ns && Es === "round" && (oo < pn ? Es = "miter" : oo <= 2 && (Es = "fakeround")), Es === "miter" && oo > ln && (Es = "bevel"), Es === "bevel" && (oo > 2 && (Es = "flipbevel"), oo < ln && (Es = "miter")), Kr && this.updateDistance(Kr, Nr), Es === "miter") Na._mult(oo), this.addCurrentVertex(Nr, Na, 0, 0, xr);
                      else if (Es === "flipbevel") {
                        if (oo > 100) Na = Ui.mult(-1);
                        else {
                          const cl = oo * ui.add(Ui).mag() / ui.sub(Ui).mag();
                          Na._perp()._mult(cl * (fl ? -1 : 1));
                        }
                        this.addCurrentVertex(Nr, Na, 0, 0, xr), this.addCurrentVertex(Nr, Na.mult(-1), 0, 0, xr);
                      } else if (Es === "bevel" || Es === "fakeround") {
                        const cl = -Math.sqrt(oo * oo - 1), Xs = fl ? cl : 0, Ps = fl ? 0 : cl;
                        if (Kr && this.addCurrentVertex(Nr, ui, Xs, Ps, xr), Es === "fakeround") {
                          const ko = Math.round(180 * fs / Math.PI / 20);
                          for (let Ks = 1; Ks < ko; Ks++) {
                            let el = Ks / ko;
                            if (el !== 0.5) {
                              const bu = el - 0.5;
                              el += el * bu * (el - 1) * ((1.0904 + Ta * (Ta * (3.55645 - 1.43519 * Ta) - 3.2452)) * bu * bu + (0.848013 + Ta * (0.215638 * Ta - 1.06021)));
                            }
                            const Nl = Ui.sub(ui)._mult(el)._add(ui)._unit()._mult(fl ? -1 : 1);
                            this.addHalfVertex(Nr, Nl.x, Nl.y, !1, fl, 0, xr);
                          }
                        }
                        xi && this.addCurrentVertex(Nr, Ui, -Xs, -Ps, xr);
                      } else if (Es === "butt") this.addCurrentVertex(Nr, Na, 0, 0, xr);
                      else if (Es === "square") {
                        const cl = Kr ? 1 : -1;
                        this.addCurrentVertex(Nr, Na, cl, cl, xr);
                      } else Es === "round" && (Kr && (this.addCurrentVertex(Nr, ui, 0, 0, xr), this.addCurrentVertex(Nr, ui, 1, 1, xr, !0)), xi && (this.addCurrentVertex(Nr, Ui, -1, -1, xr, !0), this.addCurrentVertex(Nr, Ui, 0, 0, xr)));
                      if (Bs && ia < Nn - 1) {
                        const cl = Nr.dist(xi);
                        if (cl > 2 * lr) {
                          const Xs = Nr.add(xi.sub(Nr)._mult(lr / cl)._round());
                          this.updateDistance(Nr, Xs), this.addCurrentVertex(Xs, Ui, 0, 0, xr), Nr = Xs;
                        }
                      }
                    }
                  }
                  addCurrentVertex(He, ot, xt, zt, ln, pn = !1) {
                    const Cn = ot.x + ot.y * xt, Nn = ot.y - ot.x * xt, Wn = -ot.x + ot.y * zt, lr = -ot.y - ot.x * zt;
                    this.addHalfVertex(He, Cn, Nn, pn, !1, xt, ln), this.addHalfVertex(He, Wn, lr, pn, !0, -zt, ln), this.distance > Di / 2 && this.totalDistance === 0 && (this.distance = 0, this.updateScaledDistance(), this.addCurrentVertex(He, ot, xt, zt, ln, pn));
                  }
                  addHalfVertex({ x: He, y: ot }, xt, zt, ln, pn, Cn, Nn) {
                    const Wn = 0.5 * (this.lineClips ? this.scaledDistance * (Di - 1) : this.scaledDistance);
                    if (this.layoutVertexArray.emplaceBack((He << 1) + (ln ? 1 : 0), (ot << 1) + (pn ? 1 : 0), Math.round(63 * xt) + 128, Math.round(63 * zt) + 128, 1 + (Cn === 0 ? 0 : Cn < 0 ? -1 : 1) | (63 & Wn) << 2, Wn >> 6), this.lineClips) {
                      const xr = (this.scaledDistance - this.lineClips.start) / (this.lineClips.end - this.lineClips.start);
                      this.layoutVertexArray2.emplaceBack(xr, this.lineClipsArray.length);
                    }
                    const lr = Nn.vertexLength++;
                    this.e1 >= 0 && this.e2 >= 0 && (this.indexArray.emplaceBack(this.e1, this.e2, lr), Nn.primitiveLength++), pn ? this.e2 = lr : this.e1 = lr;
                  }
                  updateScaledDistance() {
                    this.scaledDistance = this.lineClips ? this.lineClips.start + (this.lineClips.end - this.lineClips.start) * this.distance / this.totalDistance : this.distance;
                  }
                  updateDistance(He, ot) {
                    this.distance += He.dist(ot), this.updateScaledDistance();
                  }
                }
                let ca;
                ss("LineBucket", yi, { omit: ["layers", "patternFeatures"] });
                let Ma;
                var Ia = { get paint() {
                  return Ma = Ma || new tu({ "line-opacity": new Os(yt.paint_line["line-opacity"]), "line-color": new Os(yt.paint_line["line-color"]), "line-translate": new zs(yt.paint_line["line-translate"]), "line-translate-anchor": new zs(yt.paint_line["line-translate-anchor"]), "line-width": new Os(yt.paint_line["line-width"]), "line-gap-width": new Os(yt.paint_line["line-gap-width"]), "line-offset": new Os(yt.paint_line["line-offset"]), "line-blur": new Os(yt.paint_line["line-blur"]), "line-dasharray": new xc(yt.paint_line["line-dasharray"]), "line-pattern": new lc(yt.paint_line["line-pattern"]), "line-gradient": new Iu(yt.paint_line["line-gradient"]) });
                }, get layout() {
                  return ca = ca || new tu({ "line-cap": new zs(yt.layout_line["line-cap"]), "line-join": new Os(yt.layout_line["line-join"]), "line-miter-limit": new zs(yt.layout_line["line-miter-limit"]), "line-round-limit": new zs(yt.layout_line["line-round-limit"]), "line-sort-key": new Os(yt.layout_line["line-sort-key"]) });
                } };
                class ba extends Os {
                  possiblyEvaluate(He, ot) {
                    return ot = new Ko(Math.floor(ot.zoom), { now: ot.now, fadeDuration: ot.fadeDuration, zoomHistory: ot.zoomHistory, transition: ot.transition }), super.possiblyEvaluate(He, ot);
                  }
                  evaluate(He, ot, xt, zt) {
                    return ot = Ce({}, ot, { zoom: Math.floor(ot.zoom) }), super.evaluate(He, ot, xt, zt);
                  }
                }
                let fa;
                class Ea extends $n {
                  constructor(He) {
                    super(He, Ia), this.gradientVersion = 0, fa || (fa = new ba(Ia.paint.properties["line-width"].specification), fa.useIntegerZoom = !0);
                  }
                  _handleSpecialPaintPropertyUpdate(He) {
                    if (He === "line-gradient") {
                      const ot = this.gradientExpression();
                      (function(xt) {
                        return xt._styleExpression !== void 0;
                      })(ot) ? this.stepInterpolant = ot._styleExpression.expression instanceof la : this.stepInterpolant = !1, this.gradientVersion = (this.gradientVersion + 1) % Number.MAX_SAFE_INTEGER;
                    }
                  }
                  gradientExpression() {
                    return this._transitionablePaint._values["line-gradient"].value.expression;
                  }
                  recalculate(He, ot) {
                    super.recalculate(He, ot), this.paint._values["line-floorwidth"] = fa.possiblyEvaluate(this._transitioningPaint._values["line-width"].value, He);
                  }
                  createBucket(He) {
                    return new yi(He);
                  }
                  queryRadius(He) {
                    const ot = He, xt = Hi(sf("line-width", this, ot), sf("line-gap-width", this, ot)), zt = sf("line-offset", this, ot);
                    return xt / 2 + Math.abs(zt) + Zd(this.paint.get("line-translate"));
                  }
                  queryIntersectsFeature(He, ot, xt, zt, ln, pn, Cn) {
                    const Nn = Lf(He, this.paint.get("line-translate"), this.paint.get("line-translate-anchor"), pn.angle, Cn), Wn = Cn / 2 * Hi(this.paint.get("line-width").evaluate(ot, xt), this.paint.get("line-gap-width").evaluate(ot, xt)), lr = this.paint.get("line-offset").evaluate(ot, xt);
                    return lr && (zt = function(xr, Nr) {
                      const Kr = [];
                      for (let xi = 0; xi < xr.length; xi++) {
                        const ui = xr[xi], Ui = [];
                        for (let ia = 0; ia < ui.length; ia++) {
                          const Na = ui[ia - 1], Ta = ui[ia], Ra = ui[ia + 1], oo = ia === 0 ? new ce(0, 0) : Ta.sub(Na)._unit()._perp(), fs = ia === ui.length - 1 ? new ce(0, 0) : Ra.sub(Ta)._unit()._perp(), Bs = oo._add(fs)._unit(), fl = Bs.x * fs.x + Bs.y * fs.y;
                          fl !== 0 && Bs._mult(1 / fl), Ui.push(Bs._mult(Nr)._add(Ta));
                        }
                        Kr.push(Ui);
                      }
                      return Kr;
                    }(zt, lr * Cn)), function(xr, Nr, Kr) {
                      for (let xi = 0; xi < Nr.length; xi++) {
                        const ui = Nr[xi];
                        if (xr.length >= 3) {
                          for (let Ui = 0; Ui < ui.length; Ui++) if (ld(xr, ui[Ui])) return !0;
                        }
                        if (Cf(xr, ui, Kr)) return !0;
                      }
                      return !1;
                    }(Nn, zt, Wn);
                  }
                  isTileClipped() {
                    return !0;
                  }
                }
                function Hi(Ye, He) {
                  return He > 0 ? He + 2 * Ye : Ye;
                }
                const ji = Fe([{ name: "a_pos_offset", components: 4, type: "Int16" }, { name: "a_data", components: 4, type: "Uint16" }, { name: "a_pixeloffset", components: 4, type: "Int16" }], 4), qi = Fe([{ name: "a_projected_pos", components: 3, type: "Float32" }], 4);
                Fe([{ name: "a_fade_opacity", components: 1, type: "Uint32" }], 4);
                const Ua = Fe([{ name: "a_placed", components: 2, type: "Uint8" }, { name: "a_shift", components: 2, type: "Float32" }, { name: "a_box_real", components: 2, type: "Int16" }]);
                Fe([{ type: "Int16", name: "anchorPointX" }, { type: "Int16", name: "anchorPointY" }, { type: "Int16", name: "x1" }, { type: "Int16", name: "y1" }, { type: "Int16", name: "x2" }, { type: "Int16", name: "y2" }, { type: "Uint32", name: "featureIndex" }, { type: "Uint16", name: "sourceLayerIndex" }, { type: "Uint16", name: "bucketIndex" }]);
                const La = Fe([{ name: "a_pos", components: 2, type: "Int16" }, { name: "a_anchor_pos", components: 2, type: "Int16" }, { name: "a_extrude", components: 2, type: "Int16" }], 4), yo = Fe([{ name: "a_pos", components: 2, type: "Float32" }, { name: "a_radius", components: 1, type: "Float32" }, { name: "a_flags", components: 2, type: "Int16" }], 4);
                function xo(Ye, He, ot) {
                  return Ye.sections.forEach((xt) => {
                    xt.text = function(zt, ln, pn) {
                      const Cn = ln.layout.get("text-transform").evaluate(pn, {});
                      return Cn === "uppercase" ? zt = zt.toLocaleUpperCase() : Cn === "lowercase" && (zt = zt.toLocaleLowerCase()), nc.applyArabicShaping && (zt = nc.applyArabicShaping(zt)), zt;
                    }(xt.text, He, ot);
                  }), Ye;
                }
                Fe([{ name: "triangle", components: 3, type: "Uint16" }]), Fe([{ type: "Int16", name: "anchorX" }, { type: "Int16", name: "anchorY" }, { type: "Uint16", name: "glyphStartIndex" }, { type: "Uint16", name: "numGlyphs" }, { type: "Uint32", name: "vertexStartIndex" }, { type: "Uint32", name: "lineStartIndex" }, { type: "Uint32", name: "lineLength" }, { type: "Uint16", name: "segment" }, { type: "Uint16", name: "lowerSize" }, { type: "Uint16", name: "upperSize" }, { type: "Float32", name: "lineOffsetX" }, { type: "Float32", name: "lineOffsetY" }, { type: "Uint8", name: "writingMode" }, { type: "Uint8", name: "placedOrientation" }, { type: "Uint8", name: "hidden" }, { type: "Uint32", name: "crossTileID" }, { type: "Int16", name: "associatedIconIndex" }]), Fe([{ type: "Int16", name: "anchorX" }, { type: "Int16", name: "anchorY" }, { type: "Int16", name: "rightJustifiedTextSymbolIndex" }, { type: "Int16", name: "centerJustifiedTextSymbolIndex" }, { type: "Int16", name: "leftJustifiedTextSymbolIndex" }, { type: "Int16", name: "verticalPlacedTextSymbolIndex" }, { type: "Int16", name: "placedIconSymbolIndex" }, { type: "Int16", name: "verticalPlacedIconSymbolIndex" }, { type: "Uint16", name: "key" }, { type: "Uint16", name: "textBoxStartIndex" }, { type: "Uint16", name: "textBoxEndIndex" }, { type: "Uint16", name: "verticalTextBoxStartIndex" }, { type: "Uint16", name: "verticalTextBoxEndIndex" }, { type: "Uint16", name: "iconBoxStartIndex" }, { type: "Uint16", name: "iconBoxEndIndex" }, { type: "Uint16", name: "verticalIconBoxStartIndex" }, { type: "Uint16", name: "verticalIconBoxEndIndex" }, { type: "Uint16", name: "featureIndex" }, { type: "Uint16", name: "numHorizontalGlyphVertices" }, { type: "Uint16", name: "numVerticalGlyphVertices" }, { type: "Uint16", name: "numIconVertices" }, { type: "Uint16", name: "numVerticalIconVertices" }, { type: "Uint16", name: "useRuntimeCollisionCircles" }, { type: "Uint32", name: "crossTileID" }, { type: "Float32", name: "textBoxScale" }, { type: "Float32", name: "collisionCircleDiameter" }, { type: "Uint16", name: "textAnchorOffsetStartIndex" }, { type: "Uint16", name: "textAnchorOffsetEndIndex" }]), Fe([{ type: "Float32", name: "offsetX" }]), Fe([{ type: "Int16", name: "x" }, { type: "Int16", name: "y" }, { type: "Int16", name: "tileUnitDistanceFromAnchor" }]), Fe([{ type: "Uint16", name: "textAnchor" }, { type: "Float32", components: 2, name: "textOffset" }]);
                const rs = { "!": "︕", "#": "＃", $: "＄", "%": "％", "&": "＆", "(": "︵", ")": "︶", "*": "＊", "+": "＋", ",": "︐", "-": "︲", ".": "・", "/": "／", ":": "︓", ";": "︔", "<": "︿", "=": "＝", ">": "﹀", "?": "︖", "@": "＠", "[": "﹇", "\\": "＼", "]": "﹈", "^": "＾", _: "︳", "`": "｀", "{": "︷", "|": "―", "}": "︸", "~": "～", "¢": "￠", "£": "￡", "¥": "￥", "¦": "￤", "¬": "￢", "¯": "￣", "–": "︲", "—": "︱", "‘": "﹃", "’": "﹄", "“": "﹁", "”": "﹂", "…": "︙", "‧": "・", "₩": "￦", "、": "︑", "。": "︒", "〈": "︿", "〉": "﹀", "《": "︽", "》": "︾", "「": "﹁", "」": "﹂", "『": "﹃", "』": "﹄", "【": "︻", "】": "︼", "〔": "︹", "〕": "︺", "〖": "︗", "〗": "︘", "！": "︕", "（": "︵", "）": "︶", "，": "︐", "－": "︲", "．": "・", "：": "︓", "；": "︔", "＜": "︿", "＞": "﹀", "？": "︖", "［": "﹇", "］": "﹈", "＿": "︳", "｛": "︷", "｜": "―", "｝": "︸", "｟": "︵", "｠": "︶", "｡": "︒", "｢": "﹁", "｣": "﹂" };
                var Us = 24, sl = no, Cs = { read: function(Ye, He, ot, xt, zt) {
                  var ln, pn, Cn = 8 * zt - xt - 1, Nn = (1 << Cn) - 1, Wn = Nn >> 1, lr = -7, xr = ot ? zt - 1 : 0, Nr = ot ? -1 : 1, Kr = Ye[He + xr];
                  for (xr += Nr, ln = Kr & (1 << -lr) - 1, Kr >>= -lr, lr += Cn; lr > 0; ln = 256 * ln + Ye[He + xr], xr += Nr, lr -= 8) ;
                  for (pn = ln & (1 << -lr) - 1, ln >>= -lr, lr += xt; lr > 0; pn = 256 * pn + Ye[He + xr], xr += Nr, lr -= 8) ;
                  if (ln === 0) ln = 1 - Wn;
                  else {
                    if (ln === Nn) return pn ? NaN : 1 / 0 * (Kr ? -1 : 1);
                    pn += Math.pow(2, xt), ln -= Wn;
                  }
                  return (Kr ? -1 : 1) * pn * Math.pow(2, ln - xt);
                }, write: function(Ye, He, ot, xt, zt, ln) {
                  var pn, Cn, Nn, Wn = 8 * ln - zt - 1, lr = (1 << Wn) - 1, xr = lr >> 1, Nr = zt === 23 ? Math.pow(2, -24) - Math.pow(2, -77) : 0, Kr = xt ? 0 : ln - 1, xi = xt ? 1 : -1, ui = He < 0 || He === 0 && 1 / He < 0 ? 1 : 0;
                  for (He = Math.abs(He), isNaN(He) || He === 1 / 0 ? (Cn = isNaN(He) ? 1 : 0, pn = lr) : (pn = Math.floor(Math.log(He) / Math.LN2), He * (Nn = Math.pow(2, -pn)) < 1 && (pn--, Nn *= 2), (He += pn + xr >= 1 ? Nr / Nn : Nr * Math.pow(2, 1 - xr)) * Nn >= 2 && (pn++, Nn /= 2), pn + xr >= lr ? (Cn = 0, pn = lr) : pn + xr >= 1 ? (Cn = (He * Nn - 1) * Math.pow(2, zt), pn += xr) : (Cn = He * Math.pow(2, xr - 1) * Math.pow(2, zt), pn = 0)); zt >= 8; Ye[ot + Kr] = 255 & Cn, Kr += xi, Cn /= 256, zt -= 8) ;
                  for (pn = pn << zt | Cn, Wn += zt; Wn > 0; Ye[ot + Kr] = 255 & pn, Kr += xi, pn /= 256, Wn -= 8) ;
                  Ye[ot + Kr - xi] |= 128 * ui;
                } };
                function no(Ye) {
                  this.buf = ArrayBuffer.isView && ArrayBuffer.isView(Ye) ? Ye : new Uint8Array(Ye || 0), this.pos = 0, this.type = 0, this.length = this.buf.length;
                }
                no.Varint = 0, no.Fixed64 = 1, no.Bytes = 2, no.Fixed32 = 5;
                var fo = 4294967296, Ys = 1 / fo, Rs = typeof TextDecoder > "u" ? null : new TextDecoder("utf-8");
                function js(Ye) {
                  return Ye.type === no.Bytes ? Ye.readVarint() + Ye.pos : Ye.pos + 1;
                }
                function Mc(Ye, He, ot) {
                  return ot ? 4294967296 * He + (Ye >>> 0) : 4294967296 * (He >>> 0) + (Ye >>> 0);
                }
                function dd(Ye, He, ot) {
                  var xt = He <= 16383 ? 1 : He <= 2097151 ? 2 : He <= 268435455 ? 3 : Math.floor(Math.log(He) / (7 * Math.LN2));
                  ot.realloc(xt);
                  for (var zt = ot.pos - 1; zt >= Ye; zt--) ot.buf[zt + xt] = ot.buf[zt];
                }
                function nh(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeVarint(Ye[ot]);
                }
                function Vf(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeSVarint(Ye[ot]);
                }
                function vf(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeFloat(Ye[ot]);
                }
                function Uf(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeDouble(Ye[ot]);
                }
                function rh(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeBoolean(Ye[ot]);
                }
                function B2(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeFixed32(Ye[ot]);
                }
                function Kd(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeSFixed32(Ye[ot]);
                }
                function mh(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeFixed64(Ye[ot]);
                }
                function Jd(Ye, He) {
                  for (var ot = 0; ot < Ye.length; ot++) He.writeSFixed64(Ye[ot]);
                }
                function Dh(Ye, He) {
                  return (Ye[He] | Ye[He + 1] << 8 | Ye[He + 2] << 16) + 16777216 * Ye[He + 3];
                }
                function P1(Ye, He, ot) {
                  Ye[ot] = He, Ye[ot + 1] = He >>> 8, Ye[ot + 2] = He >>> 16, Ye[ot + 3] = He >>> 24;
                }
                function Nh(Ye, He) {
                  return (Ye[He] | Ye[He + 1] << 8 | Ye[He + 2] << 16) + (Ye[He + 3] << 24);
                }
                no.prototype = { destroy: function() {
                  this.buf = null;
                }, readFields: function(Ye, He, ot) {
                  for (ot = ot || this.length; this.pos < ot; ) {
                    var xt = this.readVarint(), zt = xt >> 3, ln = this.pos;
                    this.type = 7 & xt, Ye(zt, He, this), this.pos === ln && this.skip(xt);
                  }
                  return He;
                }, readMessage: function(Ye, He) {
                  return this.readFields(Ye, He, this.readVarint() + this.pos);
                }, readFixed32: function() {
                  var Ye = Dh(this.buf, this.pos);
                  return this.pos += 4, Ye;
                }, readSFixed32: function() {
                  var Ye = Nh(this.buf, this.pos);
                  return this.pos += 4, Ye;
                }, readFixed64: function() {
                  var Ye = Dh(this.buf, this.pos) + Dh(this.buf, this.pos + 4) * fo;
                  return this.pos += 8, Ye;
                }, readSFixed64: function() {
                  var Ye = Dh(this.buf, this.pos) + Nh(this.buf, this.pos + 4) * fo;
                  return this.pos += 8, Ye;
                }, readFloat: function() {
                  var Ye = Cs.read(this.buf, this.pos, !0, 23, 4);
                  return this.pos += 4, Ye;
                }, readDouble: function() {
                  var Ye = Cs.read(this.buf, this.pos, !0, 52, 8);
                  return this.pos += 8, Ye;
                }, readVarint: function(Ye) {
                  var He, ot, xt = this.buf;
                  return He = 127 & (ot = xt[this.pos++]), ot < 128 ? He : (He |= (127 & (ot = xt[this.pos++])) << 7, ot < 128 ? He : (He |= (127 & (ot = xt[this.pos++])) << 14, ot < 128 ? He : (He |= (127 & (ot = xt[this.pos++])) << 21, ot < 128 ? He : function(zt, ln, pn) {
                    var Cn, Nn, Wn = pn.buf;
                    if (Cn = (112 & (Nn = Wn[pn.pos++])) >> 4, Nn < 128 || (Cn |= (127 & (Nn = Wn[pn.pos++])) << 3, Nn < 128) || (Cn |= (127 & (Nn = Wn[pn.pos++])) << 10, Nn < 128) || (Cn |= (127 & (Nn = Wn[pn.pos++])) << 17, Nn < 128) || (Cn |= (127 & (Nn = Wn[pn.pos++])) << 24, Nn < 128) || (Cn |= (1 & (Nn = Wn[pn.pos++])) << 31, Nn < 128)) return Mc(zt, Cn, ln);
                    throw new Error("Expected varint not more than 10 bytes");
                  }(He |= (15 & (ot = xt[this.pos])) << 28, Ye, this))));
                }, readVarint64: function() {
                  return this.readVarint(!0);
                }, readSVarint: function() {
                  var Ye = this.readVarint();
                  return Ye % 2 == 1 ? (Ye + 1) / -2 : Ye / 2;
                }, readBoolean: function() {
                  return !!this.readVarint();
                }, readString: function() {
                  var Ye = this.readVarint() + this.pos, He = this.pos;
                  return this.pos = Ye, Ye - He >= 12 && Rs ? function(ot, xt, zt) {
                    return Rs.decode(ot.subarray(xt, zt));
                  }(this.buf, He, Ye) : function(ot, xt, zt) {
                    for (var ln = "", pn = xt; pn < zt; ) {
                      var Cn, Nn, Wn, lr = ot[pn], xr = null, Nr = lr > 239 ? 4 : lr > 223 ? 3 : lr > 191 ? 2 : 1;
                      if (pn + Nr > zt) break;
                      Nr === 1 ? lr < 128 && (xr = lr) : Nr === 2 ? (192 & (Cn = ot[pn + 1])) == 128 && (xr = (31 & lr) << 6 | 63 & Cn) <= 127 && (xr = null) : Nr === 3 ? (Cn = ot[pn + 1], Nn = ot[pn + 2], (192 & Cn) == 128 && (192 & Nn) == 128 && ((xr = (15 & lr) << 12 | (63 & Cn) << 6 | 63 & Nn) <= 2047 || xr >= 55296 && xr <= 57343) && (xr = null)) : Nr === 4 && (Cn = ot[pn + 1], Nn = ot[pn + 2], Wn = ot[pn + 3], (192 & Cn) == 128 && (192 & Nn) == 128 && (192 & Wn) == 128 && ((xr = (15 & lr) << 18 | (63 & Cn) << 12 | (63 & Nn) << 6 | 63 & Wn) <= 65535 || xr >= 1114112) && (xr = null)), xr === null ? (xr = 65533, Nr = 1) : xr > 65535 && (xr -= 65536, ln += String.fromCharCode(xr >>> 10 & 1023 | 55296), xr = 56320 | 1023 & xr), ln += String.fromCharCode(xr), pn += Nr;
                    }
                    return ln;
                  }(this.buf, He, Ye);
                }, readBytes: function() {
                  var Ye = this.readVarint() + this.pos, He = this.buf.subarray(this.pos, Ye);
                  return this.pos = Ye, He;
                }, readPackedVarint: function(Ye, He) {
                  if (this.type !== no.Bytes) return Ye.push(this.readVarint(He));
                  var ot = js(this);
                  for (Ye = Ye || []; this.pos < ot; ) Ye.push(this.readVarint(He));
                  return Ye;
                }, readPackedSVarint: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readSVarint());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readSVarint());
                  return Ye;
                }, readPackedBoolean: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readBoolean());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readBoolean());
                  return Ye;
                }, readPackedFloat: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readFloat());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readFloat());
                  return Ye;
                }, readPackedDouble: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readDouble());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readDouble());
                  return Ye;
                }, readPackedFixed32: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readFixed32());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readFixed32());
                  return Ye;
                }, readPackedSFixed32: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readSFixed32());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readSFixed32());
                  return Ye;
                }, readPackedFixed64: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readFixed64());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readFixed64());
                  return Ye;
                }, readPackedSFixed64: function(Ye) {
                  if (this.type !== no.Bytes) return Ye.push(this.readSFixed64());
                  var He = js(this);
                  for (Ye = Ye || []; this.pos < He; ) Ye.push(this.readSFixed64());
                  return Ye;
                }, skip: function(Ye) {
                  var He = 7 & Ye;
                  if (He === no.Varint) for (; this.buf[this.pos++] > 127; ) ;
                  else if (He === no.Bytes) this.pos = this.readVarint() + this.pos;
                  else if (He === no.Fixed32) this.pos += 4;
                  else {
                    if (He !== no.Fixed64) throw new Error("Unimplemented type: " + He);
                    this.pos += 8;
                  }
                }, writeTag: function(Ye, He) {
                  this.writeVarint(Ye << 3 | He);
                }, realloc: function(Ye) {
                  for (var He = this.length || 16; He < this.pos + Ye; ) He *= 2;
                  if (He !== this.length) {
                    var ot = new Uint8Array(He);
                    ot.set(this.buf), this.buf = ot, this.length = He;
                  }
                }, finish: function() {
                  return this.length = this.pos, this.pos = 0, this.buf.subarray(0, this.length);
                }, writeFixed32: function(Ye) {
                  this.realloc(4), P1(this.buf, Ye, this.pos), this.pos += 4;
                }, writeSFixed32: function(Ye) {
                  this.realloc(4), P1(this.buf, Ye, this.pos), this.pos += 4;
                }, writeFixed64: function(Ye) {
                  this.realloc(8), P1(this.buf, -1 & Ye, this.pos), P1(this.buf, Math.floor(Ye * Ys), this.pos + 4), this.pos += 8;
                }, writeSFixed64: function(Ye) {
                  this.realloc(8), P1(this.buf, -1 & Ye, this.pos), P1(this.buf, Math.floor(Ye * Ys), this.pos + 4), this.pos += 8;
                }, writeVarint: function(Ye) {
                  (Ye = +Ye || 0) > 268435455 || Ye < 0 ? function(He, ot) {
                    var xt, zt;
                    if (He >= 0 ? (xt = He % 4294967296 | 0, zt = He / 4294967296 | 0) : (zt = ~(-He / 4294967296), 4294967295 ^ (xt = ~(-He % 4294967296)) ? xt = xt + 1 | 0 : (xt = 0, zt = zt + 1 | 0)), He >= 18446744073709552e3 || He < -18446744073709552e3) throw new Error("Given varint doesn't fit into 10 bytes");
                    ot.realloc(10), function(ln, pn, Cn) {
                      Cn.buf[Cn.pos++] = 127 & ln | 128, ln >>>= 7, Cn.buf[Cn.pos++] = 127 & ln | 128, ln >>>= 7, Cn.buf[Cn.pos++] = 127 & ln | 128, ln >>>= 7, Cn.buf[Cn.pos++] = 127 & ln | 128, ln >>>= 7, Cn.buf[Cn.pos] = 127 & ln;
                    }(xt, 0, ot), function(ln, pn) {
                      var Cn = (7 & ln) << 4;
                      pn.buf[pn.pos++] |= Cn | ((ln >>>= 3) ? 128 : 0), ln && (pn.buf[pn.pos++] = 127 & ln | ((ln >>>= 7) ? 128 : 0), ln && (pn.buf[pn.pos++] = 127 & ln | ((ln >>>= 7) ? 128 : 0), ln && (pn.buf[pn.pos++] = 127 & ln | ((ln >>>= 7) ? 128 : 0), ln && (pn.buf[pn.pos++] = 127 & ln | ((ln >>>= 7) ? 128 : 0), ln && (pn.buf[pn.pos++] = 127 & ln)))));
                    }(zt, ot);
                  }(Ye, this) : (this.realloc(4), this.buf[this.pos++] = 127 & Ye | (Ye > 127 ? 128 : 0), Ye <= 127 || (this.buf[this.pos++] = 127 & (Ye >>>= 7) | (Ye > 127 ? 128 : 0), Ye <= 127 || (this.buf[this.pos++] = 127 & (Ye >>>= 7) | (Ye > 127 ? 128 : 0), Ye <= 127 || (this.buf[this.pos++] = Ye >>> 7 & 127))));
                }, writeSVarint: function(Ye) {
                  this.writeVarint(Ye < 0 ? 2 * -Ye - 1 : 2 * Ye);
                }, writeBoolean: function(Ye) {
                  this.writeVarint(!!Ye);
                }, writeString: function(Ye) {
                  Ye = String(Ye), this.realloc(4 * Ye.length), this.pos++;
                  var He = this.pos;
                  this.pos = function(xt, zt, ln) {
                    for (var pn, Cn, Nn = 0; Nn < zt.length; Nn++) {
                      if ((pn = zt.charCodeAt(Nn)) > 55295 && pn < 57344) {
                        if (!Cn) {
                          pn > 56319 || Nn + 1 === zt.length ? (xt[ln++] = 239, xt[ln++] = 191, xt[ln++] = 189) : Cn = pn;
                          continue;
                        }
                        if (pn < 56320) {
                          xt[ln++] = 239, xt[ln++] = 191, xt[ln++] = 189, Cn = pn;
                          continue;
                        }
                        pn = Cn - 55296 << 10 | pn - 56320 | 65536, Cn = null;
                      } else Cn && (xt[ln++] = 239, xt[ln++] = 191, xt[ln++] = 189, Cn = null);
                      pn < 128 ? xt[ln++] = pn : (pn < 2048 ? xt[ln++] = pn >> 6 | 192 : (pn < 65536 ? xt[ln++] = pn >> 12 | 224 : (xt[ln++] = pn >> 18 | 240, xt[ln++] = pn >> 12 & 63 | 128), xt[ln++] = pn >> 6 & 63 | 128), xt[ln++] = 63 & pn | 128);
                    }
                    return ln;
                  }(this.buf, Ye, this.pos);
                  var ot = this.pos - He;
                  ot >= 128 && dd(He, ot, this), this.pos = He - 1, this.writeVarint(ot), this.pos += ot;
                }, writeFloat: function(Ye) {
                  this.realloc(4), Cs.write(this.buf, Ye, this.pos, !0, 23, 4), this.pos += 4;
                }, writeDouble: function(Ye) {
                  this.realloc(8), Cs.write(this.buf, Ye, this.pos, !0, 52, 8), this.pos += 8;
                }, writeBytes: function(Ye) {
                  var He = Ye.length;
                  this.writeVarint(He), this.realloc(He);
                  for (var ot = 0; ot < He; ot++) this.buf[this.pos++] = Ye[ot];
                }, writeRawMessage: function(Ye, He) {
                  this.pos++;
                  var ot = this.pos;
                  Ye(He, this);
                  var xt = this.pos - ot;
                  xt >= 128 && dd(ot, xt, this), this.pos = ot - 1, this.writeVarint(xt), this.pos += xt;
                }, writeMessage: function(Ye, He, ot) {
                  this.writeTag(Ye, no.Bytes), this.writeRawMessage(He, ot);
                }, writePackedVarint: function(Ye, He) {
                  He.length && this.writeMessage(Ye, nh, He);
                }, writePackedSVarint: function(Ye, He) {
                  He.length && this.writeMessage(Ye, Vf, He);
                }, writePackedBoolean: function(Ye, He) {
                  He.length && this.writeMessage(Ye, rh, He);
                }, writePackedFloat: function(Ye, He) {
                  He.length && this.writeMessage(Ye, vf, He);
                }, writePackedDouble: function(Ye, He) {
                  He.length && this.writeMessage(Ye, Uf, He);
                }, writePackedFixed32: function(Ye, He) {
                  He.length && this.writeMessage(Ye, B2, He);
                }, writePackedSFixed32: function(Ye, He) {
                  He.length && this.writeMessage(Ye, Kd, He);
                }, writePackedFixed64: function(Ye, He) {
                  He.length && this.writeMessage(Ye, mh, He);
                }, writePackedSFixed64: function(Ye, He) {
                  He.length && this.writeMessage(Ye, Jd, He);
                }, writeBytesField: function(Ye, He) {
                  this.writeTag(Ye, no.Bytes), this.writeBytes(He);
                }, writeFixed32Field: function(Ye, He) {
                  this.writeTag(Ye, no.Fixed32), this.writeFixed32(He);
                }, writeSFixed32Field: function(Ye, He) {
                  this.writeTag(Ye, no.Fixed32), this.writeSFixed32(He);
                }, writeFixed64Field: function(Ye, He) {
                  this.writeTag(Ye, no.Fixed64), this.writeFixed64(He);
                }, writeSFixed64Field: function(Ye, He) {
                  this.writeTag(Ye, no.Fixed64), this.writeSFixed64(He);
                }, writeVarintField: function(Ye, He) {
                  this.writeTag(Ye, no.Varint), this.writeVarint(He);
                }, writeSVarintField: function(Ye, He) {
                  this.writeTag(Ye, no.Varint), this.writeSVarint(He);
                }, writeStringField: function(Ye, He) {
                  this.writeTag(Ye, no.Bytes), this.writeString(He);
                }, writeFloatField: function(Ye, He) {
                  this.writeTag(Ye, no.Fixed32), this.writeFloat(He);
                }, writeDoubleField: function(Ye, He) {
                  this.writeTag(Ye, no.Fixed64), this.writeDouble(He);
                }, writeBooleanField: function(Ye, He) {
                  this.writeVarintField(Ye, !!He);
                } };
                var qf = ae(sl);
                const V0 = 3;
                function U0(Ye, He, ot) {
                  Ye === 1 && ot.readMessage(q0, He);
                }
                function q0(Ye, He, ot) {
                  if (Ye === 3) {
                    const { id: xt, bitmap: zt, width: ln, height: pn, left: Cn, top: Nn, advance: Wn } = ot.readMessage(j0, {});
                    He.push({ id: xt, bitmap: new cu({ width: ln + 2 * V0, height: pn + 2 * V0 }, zt), metrics: { width: ln, height: pn, left: Cn, top: Nn, advance: Wn } });
                  }
                }
                function j0(Ye, He, ot) {
                  Ye === 1 ? He.id = ot.readVarint() : Ye === 2 ? He.bitmap = ot.readBytes() : Ye === 3 ? He.width = ot.readVarint() : Ye === 4 ? He.height = ot.readVarint() : Ye === 5 ? He.left = ot.readSVarint() : Ye === 6 ? He.top = ot.readSVarint() : Ye === 7 && (He.advance = ot.readVarint());
                }
                const p2 = V0;
                function C4(Ye) {
                  let He = 0, ot = 0;
                  for (const pn of Ye) He += pn.w * pn.h, ot = Math.max(ot, pn.w);
                  Ye.sort((pn, Cn) => Cn.h - pn.h);
                  const xt = [{ x: 0, y: 0, w: Math.max(Math.ceil(Math.sqrt(He / 0.95)), ot), h: 1 / 0 }];
                  let zt = 0, ln = 0;
                  for (const pn of Ye) for (let Cn = xt.length - 1; Cn >= 0; Cn--) {
                    const Nn = xt[Cn];
                    if (!(pn.w > Nn.w || pn.h > Nn.h)) {
                      if (pn.x = Nn.x, pn.y = Nn.y, ln = Math.max(ln, pn.y + pn.h), zt = Math.max(zt, pn.x + pn.w), pn.w === Nn.w && pn.h === Nn.h) {
                        const Wn = xt.pop();
                        Cn < xt.length && (xt[Cn] = Wn);
                      } else pn.h === Nn.h ? (Nn.x += pn.w, Nn.w -= pn.w) : pn.w === Nn.w ? (Nn.y += pn.h, Nn.h -= pn.h) : (xt.push({ x: Nn.x + pn.w, y: Nn.y, w: Nn.w - pn.w, h: pn.h }), Nn.y += pn.h, Nn.h -= pn.h);
                      break;
                    }
                  }
                  return { w: zt, h: ln, fill: He / (zt * ln) || 0 };
                }
                const ef = 1;
                class r3 {
                  constructor(He, { pixelRatio: ot, version: xt, stretchX: zt, stretchY: ln, content: pn, textFitWidth: Cn, textFitHeight: Nn }) {
                    this.paddedRect = He, this.pixelRatio = ot, this.stretchX = zt, this.stretchY = ln, this.content = pn, this.version = xt, this.textFitWidth = Cn, this.textFitHeight = Nn;
                  }
                  get tl() {
                    return [this.paddedRect.x + ef, this.paddedRect.y + ef];
                  }
                  get br() {
                    return [this.paddedRect.x + this.paddedRect.w - ef, this.paddedRect.y + this.paddedRect.h - ef];
                  }
                  get tlbr() {
                    return this.tl.concat(this.br);
                  }
                  get displaySize() {
                    return [(this.paddedRect.w - 2 * ef) / this.pixelRatio, (this.paddedRect.h - 2 * ef) / this.pixelRatio];
                  }
                }
                class Zh {
                  constructor(He, ot) {
                    const xt = {}, zt = {};
                    this.haveRenderCallbacks = [];
                    const ln = [];
                    this.addImages(He, xt, ln), this.addImages(ot, zt, ln);
                    const { w: pn, h: Cn } = C4(ln), Nn = new pu({ width: pn || 1, height: Cn || 1 });
                    for (const Wn in He) {
                      const lr = He[Wn], xr = xt[Wn].paddedRect;
                      pu.copy(lr.data, Nn, { x: 0, y: 0 }, { x: xr.x + ef, y: xr.y + ef }, lr.data);
                    }
                    for (const Wn in ot) {
                      const lr = ot[Wn], xr = zt[Wn].paddedRect, Nr = xr.x + ef, Kr = xr.y + ef, xi = lr.data.width, ui = lr.data.height;
                      pu.copy(lr.data, Nn, { x: 0, y: 0 }, { x: Nr, y: Kr }, lr.data), pu.copy(lr.data, Nn, { x: 0, y: ui - 1 }, { x: Nr, y: Kr - 1 }, { width: xi, height: 1 }), pu.copy(lr.data, Nn, { x: 0, y: 0 }, { x: Nr, y: Kr + ui }, { width: xi, height: 1 }), pu.copy(lr.data, Nn, { x: xi - 1, y: 0 }, { x: Nr - 1, y: Kr }, { width: 1, height: ui }), pu.copy(lr.data, Nn, { x: 0, y: 0 }, { x: Nr + xi, y: Kr }, { width: 1, height: ui });
                    }
                    this.image = Nn, this.iconPositions = xt, this.patternPositions = zt;
                  }
                  addImages(He, ot, xt) {
                    for (const zt in He) {
                      const ln = He[zt], pn = { x: 0, y: 0, w: ln.data.width + 2 * ef, h: ln.data.height + 2 * ef };
                      xt.push(pn), ot[zt] = new r3(pn, ln), ln.hasRenderCallback && this.haveRenderCallbacks.push(zt);
                    }
                  }
                  patchUpdatedImages(He, ot) {
                    He.dispatchRenderCallbacks(this.haveRenderCallbacks);
                    for (const xt in He.updatedImages) this.patchUpdatedImage(this.iconPositions[xt], He.getImage(xt), ot), this.patchUpdatedImage(this.patternPositions[xt], He.getImage(xt), ot);
                  }
                  patchUpdatedImage(He, ot, xt) {
                    if (!He || !ot || He.version === ot.version) return;
                    He.version = ot.version;
                    const [zt, ln] = He.tl;
                    xt.update(ot.data, void 0, { x: zt, y: ln });
                  }
                }
                var bf;
                ss("ImagePosition", r3), ss("ImageAtlas", Zh), ne.ai = void 0, (bf = ne.ai || (ne.ai = {}))[bf.none = 0] = "none", bf[bf.horizontal = 1] = "horizontal", bf[bf.vertical = 2] = "vertical", bf[bf.horizontalOnly = 3] = "horizontalOnly";
                const Kh = -17;
                class m2 {
                  constructor() {
                    this.scale = 1, this.fontStack = "", this.imageName = null;
                  }
                  static forText(He, ot) {
                    const xt = new m2();
                    return xt.scale = He || 1, xt.fontStack = ot, xt;
                  }
                  static forImage(He) {
                    const ot = new m2();
                    return ot.imageName = He, ot;
                  }
                }
                class Ph {
                  constructor() {
                    this.text = "", this.sectionIndex = [], this.sections = [], this.imageSectionID = null;
                  }
                  static fromFeature(He, ot) {
                    const xt = new Ph();
                    for (let zt = 0; zt < He.sections.length; zt++) {
                      const ln = He.sections[zt];
                      ln.image ? xt.addImageSection(ln) : xt.addTextSection(ln, ot);
                    }
                    return xt;
                  }
                  length() {
                    return this.text.length;
                  }
                  getSection(He) {
                    return this.sections[this.sectionIndex[He]];
                  }
                  getSectionIndex(He) {
                    return this.sectionIndex[He];
                  }
                  getCharCode(He) {
                    return this.text.charCodeAt(He);
                  }
                  verticalizePunctuation() {
                    this.text = function(He) {
                      let ot = "";
                      for (let xt = 0; xt < He.length; xt++) {
                        const zt = He.charCodeAt(xt + 1) || null, ln = He.charCodeAt(xt - 1) || null;
                        zt && Wc(zt) && !rs[He[xt + 1]] || ln && Wc(ln) && !rs[He[xt - 1]] || !rs[He[xt]] ? ot += He[xt] : ot += rs[He[xt]];
                      }
                      return ot;
                    }(this.text);
                  }
                  trim() {
                    let He = 0;
                    for (let xt = 0; xt < this.text.length && jf[this.text.charCodeAt(xt)]; xt++) He++;
                    let ot = this.text.length;
                    for (let xt = this.text.length - 1; xt >= 0 && xt >= He && jf[this.text.charCodeAt(xt)]; xt--) ot--;
                    this.text = this.text.substring(He, ot), this.sectionIndex = this.sectionIndex.slice(He, ot);
                  }
                  substring(He, ot) {
                    const xt = new Ph();
                    return xt.text = this.text.substring(He, ot), xt.sectionIndex = this.sectionIndex.slice(He, ot), xt.sections = this.sections, xt;
                  }
                  toString() {
                    return this.text;
                  }
                  getMaxScale() {
                    return this.sectionIndex.reduce((He, ot) => Math.max(He, this.sections[ot].scale), 0);
                  }
                  addTextSection(He, ot) {
                    this.text += He.text, this.sections.push(m2.forText(He.scale, He.fontStack || ot));
                    const xt = this.sections.length - 1;
                    for (let zt = 0; zt < He.text.length; ++zt) this.sectionIndex.push(xt);
                  }
                  addImageSection(He) {
                    const ot = He.image ? He.image.name : "";
                    if (ot.length === 0) return void Qe("Can't add FormattedSection with an empty image.");
                    const xt = this.getNextImageSectionCharCode();
                    xt ? (this.text += String.fromCharCode(xt), this.sections.push(m2.forImage(ot)), this.sectionIndex.push(this.sections.length - 1)) : Qe("Reached maximum number of images 6401");
                  }
                  getNextImageSectionCharCode() {
                    return this.imageSectionID ? this.imageSectionID >= 63743 ? null : ++this.imageSectionID : (this.imageSectionID = 57344, this.imageSectionID);
                  }
                }
                function $h(Ye, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi) {
                  const ui = Ph.fromFeature(Ye, zt);
                  let Ui;
                  xr === ne.ai.vertical && ui.verticalizePunctuation();
                  const { processBidirectionalText: ia, processStyledBidirectionalText: Na } = nc;
                  if (ia && ui.sections.length === 1) {
                    Ui = [];
                    const oo = ia(ui.toString(), i3(ui, Wn, ln, He, xt, Kr));
                    for (const fs of oo) {
                      const Bs = new Ph();
                      Bs.text = fs, Bs.sections = ui.sections;
                      for (let fl = 0; fl < fs.length; fl++) Bs.sectionIndex.push(0);
                      Ui.push(Bs);
                    }
                  } else if (Na) {
                    Ui = [];
                    const oo = Na(ui.text, ui.sectionIndex, i3(ui, Wn, ln, He, xt, Kr));
                    for (const fs of oo) {
                      const Bs = new Ph();
                      Bs.text = fs[0], Bs.sectionIndex = fs[1], Bs.sections = ui.sections, Ui.push(Bs);
                    }
                  } else Ui = function(oo, fs) {
                    const Bs = [], fl = oo.text;
                    let Ns = 0;
                    for (const Es of fs) Bs.push(oo.substring(Ns, Es)), Ns = Es;
                    return Ns < fl.length && Bs.push(oo.substring(Ns, fl.length)), Bs;
                  }(ui, i3(ui, Wn, ln, He, xt, Kr));
                  const Ta = [], Ra = { positionedLines: Ta, text: ui.toString(), top: lr[1], bottom: lr[1], left: lr[0], right: lr[0], writingMode: xr, iconsInText: !1, verticalizable: !1 };
                  return function(oo, fs, Bs, fl, Ns, Es, cl, Xs, Ps, ko, Ks, el) {
                    let Nl = 0, bu = Kh, Tu = 0, B1 = 0;
                    const Hd = Xs === "right" ? 1 : Xs === "left" ? 0 : 0.5;
                    let nd = 0;
                    for (const L1 of Ns) {
                      L1.trim();
                      const m1 = L1.getMaxScale(), xd = (m1 - 1) * Us, H1 = { positionedGlyphs: [], lineOffset: 0 };
                      oo.positionedLines[nd] = H1;
                      const rd = H1.positionedGlyphs;
                      let Ld = 0;
                      if (!L1.length()) {
                        bu += Es, ++nd;
                        continue;
                      }
                      for (let Nd = 0; Nd < L1.length(); Nd++) {
                        const k1 = L1.getSection(Nd), Tf = L1.getSectionIndex(Nd), c1 = L1.getCharCode(Nd);
                        let g1 = 0, _1 = null, v1 = null, Th = null, pf = Us;
                        const pd = !(Ps === ne.ai.horizontal || !Ks && !eu(c1) || Ks && (jf[c1] || hu(c1)));
                        if (k1.imageName) {
                          const mu = fl[k1.imageName];
                          if (!mu) continue;
                          Th = k1.imageName, oo.iconsInText = oo.iconsInText || !0, v1 = mu.paddedRect;
                          const u1 = mu.displaySize;
                          k1.scale = k1.scale * Us / el, _1 = { width: u1[0], height: u1[1], left: ef, top: -3, advance: pd ? u1[1] : u1[0] }, g1 = xd + (Us - u1[1] * k1.scale), pf = _1.advance;
                          const wf = pd ? u1[0] * k1.scale - Us * m1 : u1[1] * k1.scale - Us * m1;
                          wf > 0 && wf > Ld && (Ld = wf);
                        } else {
                          const mu = Bs[k1.fontStack], u1 = mu && mu[c1];
                          if (u1 && u1.rect) v1 = u1.rect, _1 = u1.metrics;
                          else {
                            const wf = fs[k1.fontStack], Of = wf && wf[c1];
                            if (!Of) continue;
                            _1 = Of.metrics;
                          }
                          g1 = (m1 - k1.scale) * Us;
                        }
                        pd ? (oo.verticalizable = !0, rd.push({ glyph: c1, imageName: Th, x: Nl, y: bu + g1, vertical: pd, scale: k1.scale, fontStack: k1.fontStack, sectionIndex: Tf, metrics: _1, rect: v1 }), Nl += pf * k1.scale + ko) : (rd.push({ glyph: c1, imageName: Th, x: Nl, y: bu + g1, vertical: pd, scale: k1.scale, fontStack: k1.fontStack, sectionIndex: Tf, metrics: _1, rect: v1 }), Nl += _1.advance * k1.scale + ko);
                      }
                      if (rd.length !== 0) {
                        const Nd = Nl - ko;
                        Tu = Math.max(Nd, Tu), z2(rd, 0, rd.length - 1, Hd, Ld);
                      }
                      Nl = 0;
                      const zd = Es * m1 + Ld;
                      H1.lineOffset = Math.max(Ld, xd), bu += zd, B1 = Math.max(zd, B1), ++nd;
                    }
                    const j1 = bu - Kh, { horizontalAlign: Fd, verticalAlign: Cd } = Fh(cl);
                    (function(L1, m1, xd, H1, rd, Ld, zd, Nd, k1) {
                      const Tf = (m1 - xd) * rd;
                      let c1 = 0;
                      c1 = Ld !== zd ? -Nd * H1 - Kh : (-H1 * k1 + 0.5) * zd;
                      for (const g1 of L1) for (const _1 of g1.positionedGlyphs) _1.x += Tf, _1.y += c1;
                    })(oo.positionedLines, Hd, Fd, Cd, Tu, B1, Es, j1, Ns.length), oo.top += -Cd * j1, oo.bottom = oo.top + j1, oo.left += -Fd * Tu, oo.right = oo.left + Tu;
                  }(Ra, He, ot, xt, Ui, pn, Cn, Nn, xr, Wn, Nr, xi), !function(oo) {
                    for (const fs of oo) if (fs.positionedGlyphs.length !== 0) return !1;
                    return !0;
                  }(Ta) && Ra;
                }
                const jf = { 9: !0, 10: !0, 11: !0, 12: !0, 13: !0, 32: !0 }, L4 = { 10: !0, 32: !0, 38: !0, 41: !0, 43: !0, 45: !0, 47: !0, 173: !0, 183: !0, 8203: !0, 8208: !0, 8211: !0, 8231: !0 }, k4 = { 40: !0 };
                function M4(Ye, He, ot, xt, zt, ln) {
                  if (He.imageName) {
                    const pn = xt[He.imageName];
                    return pn ? pn.displaySize[0] * He.scale * Us / ln + zt : 0;
                  }
                  {
                    const pn = ot[He.fontStack], Cn = pn && pn[Ye];
                    return Cn ? Cn.metrics.advance * He.scale + zt : 0;
                  }
                }
                function R4(Ye, He, ot, xt) {
                  const zt = Math.pow(Ye - He, 2);
                  return xt ? Ye < He ? zt / 2 : 2 * zt : zt + Math.abs(ot) * ot;
                }
                function J6(Ye, He, ot) {
                  let xt = 0;
                  return Ye === 10 && (xt -= 1e4), ot && (xt += 150), Ye !== 40 && Ye !== 65288 || (xt += 50), He !== 41 && He !== 65289 || (xt += 50), xt;
                }
                function H2(Ye, He, ot, xt, zt, ln) {
                  let pn = null, Cn = R4(He, ot, zt, ln);
                  for (const Nn of xt) {
                    const Wn = R4(He - Nn.x, ot, zt, ln) + Nn.badness;
                    Wn <= Cn && (pn = Nn, Cn = Wn);
                  }
                  return { index: Ye, x: He, priorBreak: pn, badness: Cn };
                }
                function G0(Ye) {
                  return Ye ? G0(Ye.priorBreak).concat(Ye.index) : [];
                }
                function i3(Ye, He, ot, xt, zt, ln) {
                  if (!Ye) return [];
                  const pn = [], Cn = function(xr, Nr, Kr, xi, ui, Ui) {
                    let ia = 0;
                    for (let Na = 0; Na < xr.length(); Na++) {
                      const Ta = xr.getSection(Na);
                      ia += M4(xr.getCharCode(Na), Ta, xi, ui, Nr, Ui);
                    }
                    return ia / Math.max(1, Math.ceil(ia / Kr));
                  }(Ye, He, ot, xt, zt, ln), Nn = Ye.text.indexOf("​") >= 0;
                  let Wn = 0;
                  for (let xr = 0; xr < Ye.length(); xr++) {
                    const Nr = Ye.getSection(xr), Kr = Ye.getCharCode(xr);
                    if (jf[Kr] || (Wn += M4(Kr, Nr, xt, zt, He, ln)), xr < Ye.length() - 1) {
                      const xi = !((lr = Kr) < 11904 || !(Zo["Bopomofo Extended"](lr) || Zo.Bopomofo(lr) || Zo["CJK Compatibility Forms"](lr) || Zo["CJK Compatibility Ideographs"](lr) || Zo["CJK Compatibility"](lr) || Zo["CJK Radicals Supplement"](lr) || Zo["CJK Strokes"](lr) || Zo["CJK Symbols and Punctuation"](lr) || Zo["CJK Unified Ideographs Extension A"](lr) || Zo["CJK Unified Ideographs"](lr) || Zo["Enclosed CJK Letters and Months"](lr) || Zo["Halfwidth and Fullwidth Forms"](lr) || Zo.Hiragana(lr) || Zo["Ideographic Description Characters"](lr) || Zo["Kangxi Radicals"](lr) || Zo["Katakana Phonetic Extensions"](lr) || Zo.Katakana(lr) || Zo["Vertical Forms"](lr) || Zo["Yi Radicals"](lr) || Zo["Yi Syllables"](lr)));
                      (L4[Kr] || xi || Nr.imageName || xr !== Ye.length() - 2 && k4[Ye.getCharCode(xr + 1)]) && pn.push(H2(xr + 1, Wn, Cn, pn, J6(Kr, Ye.getCharCode(xr + 1), xi && Nn), !1));
                    }
                  }
                  var lr;
                  return G0(H2(Ye.length(), Wn, Cn, pn, 0, !0));
                }
                function Fh(Ye) {
                  let He = 0.5, ot = 0.5;
                  switch (Ye) {
                    case "right":
                    case "top-right":
                    case "bottom-right":
                      He = 1;
                      break;
                    case "left":
                    case "top-left":
                    case "bottom-left":
                      He = 0;
                  }
                  switch (Ye) {
                    case "bottom":
                    case "bottom-right":
                    case "bottom-left":
                      ot = 1;
                      break;
                    case "top":
                    case "top-right":
                    case "top-left":
                      ot = 0;
                  }
                  return { horizontalAlign: He, verticalAlign: ot };
                }
                function z2(Ye, He, ot, xt, zt) {
                  if (!xt && !zt) return;
                  const ln = Ye[ot], pn = ln.metrics.advance * ln.scale, Cn = (Ye[ot].x + pn) * xt;
                  for (let Nn = He; Nn <= ot; Nn++) Ye[Nn].x -= Cn, Ye[Nn].y += zt;
                }
                function Du(Ye, He, ot) {
                  const { horizontalAlign: xt, verticalAlign: zt } = Fh(ot), ln = He[0], pn = He[1], Cn = ln - Ye.displaySize[0] * xt, Nn = Cn + Ye.displaySize[0], Wn = pn - Ye.displaySize[1] * zt;
                  return { image: Ye, top: Wn, bottom: Wn + Ye.displaySize[1], left: Cn, right: Nn };
                }
                function O4(Ye) {
                  var He, ot;
                  let xt = Ye.left, zt = Ye.top, ln = Ye.right - xt, pn = Ye.bottom - zt;
                  const Cn = Ye.image.content[2] - Ye.image.content[0], Nn = Ye.image.content[3] - Ye.image.content[1], Wn = (He = Ye.image.textFitWidth) !== null && He !== void 0 ? He : "stretchOrShrink", lr = (ot = Ye.image.textFitHeight) !== null && ot !== void 0 ? ot : "stretchOrShrink", xr = Cn / Nn;
                  if (lr === "proportional") {
                    if (Wn === "stretchOnly" && ln / pn < xr || Wn === "proportional") {
                      const Nr = Math.ceil(pn * xr);
                      xt *= Nr / ln, ln = Nr;
                    }
                  } else if (Wn === "proportional" && lr === "stretchOnly" && xr !== 0 && ln / pn > xr) {
                    const Nr = Math.ceil(ln / xr);
                    zt *= Nr / pn, pn = Nr;
                  }
                  return { x1: xt, y1: zt, x2: xt + ln, y2: zt + pn };
                }
                function K0(Ye, He, ot, xt, zt, ln) {
                  const pn = Ye.image;
                  let Cn;
                  if (pn.content) {
                    const Ui = pn.content, ia = pn.pixelRatio || 1;
                    Cn = [Ui[0] / ia, Ui[1] / ia, pn.displaySize[0] - Ui[2] / ia, pn.displaySize[1] - Ui[3] / ia];
                  }
                  const Nn = He.left * ln, Wn = He.right * ln;
                  let lr, xr, Nr, Kr;
                  ot === "width" || ot === "both" ? (Kr = zt[0] + Nn - xt[3], xr = zt[0] + Wn + xt[1]) : (Kr = zt[0] + (Nn + Wn - pn.displaySize[0]) / 2, xr = Kr + pn.displaySize[0]);
                  const xi = He.top * ln, ui = He.bottom * ln;
                  return ot === "height" || ot === "both" ? (lr = zt[1] + xi - xt[0], Nr = zt[1] + ui + xt[2]) : (lr = zt[1] + (xi + ui - pn.displaySize[1]) / 2, Nr = lr + pn.displaySize[1]), { image: pn, top: lr, right: xr, bottom: Nr, left: Kr, collisionPadding: Cn };
                }
                const ih = 255, ah = 128, Bh = ih * ah;
                function I4(Ye, He) {
                  const { expression: ot } = He;
                  if (ot.kind === "constant") return { kind: "constant", layoutSize: ot.evaluate(new Ko(Ye + 1)) };
                  if (ot.kind === "source") return { kind: "source" };
                  {
                    const { zoomStops: xt, interpolationType: zt } = ot;
                    let ln = 0;
                    for (; ln < xt.length && xt[ln] <= Ye; ) ln++;
                    ln = Math.max(0, ln - 1);
                    let pn = ln;
                    for (; pn < xt.length && xt[pn] < Ye + 1; ) pn++;
                    pn = Math.min(xt.length - 1, pn);
                    const Cn = xt[ln], Nn = xt[pn];
                    return ot.kind === "composite" ? { kind: "composite", minZoom: Cn, maxZoom: Nn, interpolationType: zt } : { kind: "camera", minZoom: Cn, maxZoom: Nn, minSize: ot.evaluate(new Ko(Cn)), maxSize: ot.evaluate(new Ko(Nn)), interpolationType: zt };
                  }
                }
                function a3(Ye, He, ot) {
                  let xt = "never";
                  const zt = Ye.get(He);
                  return zt ? xt = zt : Ye.get(ot) && (xt = "always"), xt;
                }
                const e_ = Jo.VectorTileFeature.types, t_ = [{ name: "a_fade_opacity", components: 1, type: "Uint8", offset: 0 }];
                function J0(Ye, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr) {
                  const Kr = Cn ? Math.min(Bh, Math.round(Cn[0])) : 0, xi = Cn ? Math.min(Bh, Math.round(Cn[1])) : 0;
                  Ye.emplaceBack(He, ot, Math.round(32 * xt), Math.round(32 * zt), ln, pn, (Kr << 1) + (Nn ? 1 : 0), xi, 16 * Wn, 16 * lr, 256 * xr, 256 * Nr);
                }
                function ep(Ye, He, ot) {
                  Ye.emplaceBack(He.x, He.y, ot), Ye.emplaceBack(He.x, He.y, ot), Ye.emplaceBack(He.x, He.y, ot), Ye.emplaceBack(He.x, He.y, ot);
                }
                function D4(Ye) {
                  for (const He of Ye.sections) if (Yc(He.text)) return !0;
                  return !1;
                }
                class tp {
                  constructor(He) {
                    this.layoutVertexArray = new Il(), this.indexArray = new Dl(), this.programConfigurations = He, this.segments = new al(), this.dynamicLayoutVertexArray = new Fl(), this.opacityVertexArray = new qc(), this.hasVisibleVertices = !1, this.placedSymbolArray = new ha();
                  }
                  isEmpty() {
                    return this.layoutVertexArray.length === 0 && this.indexArray.length === 0 && this.dynamicLayoutVertexArray.length === 0 && this.opacityVertexArray.length === 0;
                  }
                  upload(He, ot, xt, zt) {
                    this.isEmpty() || (xt && (this.layoutVertexBuffer = He.createVertexBuffer(this.layoutVertexArray, ji.members), this.indexBuffer = He.createIndexBuffer(this.indexArray, ot), this.dynamicLayoutVertexBuffer = He.createVertexBuffer(this.dynamicLayoutVertexArray, qi.members, !0), this.opacityVertexBuffer = He.createVertexBuffer(this.opacityVertexArray, t_, !0), this.opacityVertexBuffer.itemSize = 1), (xt || zt) && this.programConfigurations.upload(He));
                  }
                  destroy() {
                    this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.programConfigurations.destroy(), this.segments.destroy(), this.dynamicLayoutVertexBuffer.destroy(), this.opacityVertexBuffer.destroy());
                  }
                }
                ss("SymbolBuffers", tp);
                class rp {
                  constructor(He, ot, xt) {
                    this.layoutVertexArray = new He(), this.layoutAttributes = ot, this.indexArray = new xt(), this.segments = new al(), this.collisionVertexArray = new jc();
                  }
                  upload(He) {
                    this.layoutVertexBuffer = He.createVertexBuffer(this.layoutVertexArray, this.layoutAttributes), this.indexBuffer = He.createIndexBuffer(this.indexArray), this.collisionVertexBuffer = He.createVertexBuffer(this.collisionVertexArray, Ua.members, !0);
                  }
                  destroy() {
                    this.layoutVertexBuffer && (this.layoutVertexBuffer.destroy(), this.indexBuffer.destroy(), this.segments.destroy(), this.collisionVertexBuffer.destroy());
                  }
                }
                ss("CollisionBuffers", rp);
                class gh {
                  constructor(He) {
                    this.collisionBoxArray = He.collisionBoxArray, this.zoom = He.zoom, this.overscaling = He.overscaling, this.layers = He.layers, this.layerIds = this.layers.map((Cn) => Cn.id), this.index = He.index, this.pixelRatio = He.pixelRatio, this.sourceLayerIndex = He.sourceLayerIndex, this.hasPattern = !1, this.hasRTLText = !1, this.sortKeyRanges = [], this.collisionCircleArray = [], this.placementInvProjMatrix = dh([]), this.placementViewportMatrix = dh([]);
                    const ot = this.layers[0]._unevaluatedLayout._values;
                    this.textSizeData = I4(this.zoom, ot["text-size"]), this.iconSizeData = I4(this.zoom, ot["icon-size"]);
                    const xt = this.layers[0].layout, zt = xt.get("symbol-sort-key"), ln = xt.get("symbol-z-order");
                    this.canOverlap = a3(xt, "text-overlap", "text-allow-overlap") !== "never" || a3(xt, "icon-overlap", "icon-allow-overlap") !== "never" || xt.get("text-ignore-placement") || xt.get("icon-ignore-placement"), this.sortFeaturesByKey = ln !== "viewport-y" && !zt.isConstant();
                    const pn = ln === "viewport-y" || ln === "auto" && !this.sortFeaturesByKey;
                    this.sortFeaturesByY = pn && this.canOverlap, xt.get("symbol-placement") === "point" && (this.writingModes = xt.get("text-writing-mode").map((Cn) => ne.ai[Cn])), this.stateDependentLayerIds = this.layers.filter((Cn) => Cn.isStateDependent()).map((Cn) => Cn.id), this.sourceID = He.sourceID;
                  }
                  createArrays() {
                    this.text = new tp(new Qf(this.layers, this.zoom, (He) => /^text/.test(He))), this.icon = new tp(new Qf(this.layers, this.zoom, (He) => /^icon/.test(He))), this.glyphOffsetArray = new qa(), this.lineVertexArray = new Aa(), this.symbolInstances = new va(), this.textAnchorOffsets = new Oa();
                  }
                  calculateGlyphDependencies(He, ot, xt, zt, ln) {
                    for (let pn = 0; pn < He.length; pn++) if (ot[He.charCodeAt(pn)] = !0, (xt || zt) && ln) {
                      const Cn = rs[He.charAt(pn)];
                      Cn && (ot[Cn.charCodeAt(0)] = !0);
                    }
                  }
                  populate(He, ot, xt) {
                    const zt = this.layers[0], ln = zt.layout, pn = ln.get("text-font"), Cn = ln.get("text-field"), Nn = ln.get("icon-image"), Wn = (Cn.value.kind !== "constant" || Cn.value.value instanceof jr && !Cn.value.value.isEmpty() || Cn.value.value.toString().length > 0) && (pn.value.kind !== "constant" || pn.value.value.length > 0), lr = Nn.value.kind !== "constant" || !!Nn.value.value || Object.keys(Nn.parameters).length > 0, xr = ln.get("symbol-sort-key");
                    if (this.features = [], !Wn && !lr) return;
                    const Nr = ot.iconDependencies, Kr = ot.glyphDependencies, xi = ot.availableImages, ui = new Ko(this.zoom);
                    for (const { feature: Ui, id: ia, index: Na, sourceLayerIndex: Ta } of He) {
                      const Ra = zt._featureFilter.needGeometry, oo = Sd(Ui, Ra);
                      if (!zt._featureFilter.filter(ui, oo, xt)) continue;
                      let fs, Bs;
                      if (Ra || (oo.geometry = $d(Ui)), Wn) {
                        const Es = zt.getValueAndResolveTokens("text-field", oo, xt, xi), cl = jr.factory(Es), Xs = this.hasRTLText = this.hasRTLText || D4(cl);
                        (!Xs || nc.getRTLTextPluginStatus() === "unavailable" || Xs && nc.isParsed()) && (fs = xo(cl, zt, oo));
                      }
                      if (lr) {
                        const Es = zt.getValueAndResolveTokens("icon-image", oo, xt, xi);
                        Bs = Es instanceof gi ? Es : gi.fromString(Es);
                      }
                      if (!fs && !Bs) continue;
                      const fl = this.sortFeaturesByKey ? xr.evaluate(oo, {}, xt) : void 0, Ns = { id: ia, text: fs, icon: Bs, index: Na, sourceLayerIndex: Ta, geometry: oo.geometry, properties: Ui.properties, type: e_[Ui.type], sortKey: fl };
                      if (this.features.push(Ns), Bs && (Nr[Bs.name] = !0), fs) {
                        const Es = pn.evaluate(oo, {}, xt).join(","), cl = ln.get("text-rotation-alignment") !== "viewport" && ln.get("symbol-placement") !== "point";
                        this.allowVerticalPlacement = this.writingModes && this.writingModes.indexOf(ne.ai.vertical) >= 0;
                        for (const Xs of fs.sections) if (Xs.image) Nr[Xs.image.name] = !0;
                        else {
                          const Ps = Ou(fs.toString()), ko = Xs.fontStack || Es, Ks = Kr[ko] = Kr[ko] || {};
                          this.calculateGlyphDependencies(Xs.text, Ks, cl, this.allowVerticalPlacement, Ps);
                        }
                      }
                    }
                    ln.get("symbol-placement") === "line" && (this.features = function(Ui) {
                      const ia = {}, Na = {}, Ta = [];
                      let Ra = 0;
                      function oo(Ns) {
                        Ta.push(Ui[Ns]), Ra++;
                      }
                      function fs(Ns, Es, cl) {
                        const Xs = Na[Ns];
                        return delete Na[Ns], Na[Es] = Xs, Ta[Xs].geometry[0].pop(), Ta[Xs].geometry[0] = Ta[Xs].geometry[0].concat(cl[0]), Xs;
                      }
                      function Bs(Ns, Es, cl) {
                        const Xs = ia[Es];
                        return delete ia[Es], ia[Ns] = Xs, Ta[Xs].geometry[0].shift(), Ta[Xs].geometry[0] = cl[0].concat(Ta[Xs].geometry[0]), Xs;
                      }
                      function fl(Ns, Es, cl) {
                        const Xs = cl ? Es[0][Es[0].length - 1] : Es[0][0];
                        return `${Ns}:${Xs.x}:${Xs.y}`;
                      }
                      for (let Ns = 0; Ns < Ui.length; Ns++) {
                        const Es = Ui[Ns], cl = Es.geometry, Xs = Es.text ? Es.text.toString() : null;
                        if (!Xs) {
                          oo(Ns);
                          continue;
                        }
                        const Ps = fl(Xs, cl), ko = fl(Xs, cl, !0);
                        if (Ps in Na && ko in ia && Na[Ps] !== ia[ko]) {
                          const Ks = Bs(Ps, ko, cl), el = fs(Ps, ko, Ta[Ks].geometry);
                          delete ia[Ps], delete Na[ko], Na[fl(Xs, Ta[el].geometry, !0)] = el, Ta[Ks].geometry = null;
                        } else Ps in Na ? fs(Ps, ko, cl) : ko in ia ? Bs(Ps, ko, cl) : (oo(Ns), ia[Ps] = Ra - 1, Na[ko] = Ra - 1);
                      }
                      return Ta.filter((Ns) => Ns.geometry);
                    }(this.features)), this.sortFeaturesByKey && this.features.sort((Ui, ia) => Ui.sortKey - ia.sortKey);
                  }
                  update(He, ot, xt) {
                    this.stateDependentLayers.length && (this.text.programConfigurations.updatePaintArrays(He, ot, this.layers, xt), this.icon.programConfigurations.updatePaintArrays(He, ot, this.layers, xt));
                  }
                  isEmpty() {
                    return this.symbolInstances.length === 0 && !this.hasRTLText;
                  }
                  uploadPending() {
                    return !this.uploaded || this.text.programConfigurations.needsUpload || this.icon.programConfigurations.needsUpload;
                  }
                  upload(He) {
                    !this.uploaded && this.hasDebugData() && (this.textCollisionBox.upload(He), this.iconCollisionBox.upload(He)), this.text.upload(He, this.sortFeaturesByY, !this.uploaded, this.text.programConfigurations.needsUpload), this.icon.upload(He, this.sortFeaturesByY, !this.uploaded, this.icon.programConfigurations.needsUpload), this.uploaded = !0;
                  }
                  destroyDebugData() {
                    this.textCollisionBox.destroy(), this.iconCollisionBox.destroy();
                  }
                  destroy() {
                    this.text.destroy(), this.icon.destroy(), this.hasDebugData() && this.destroyDebugData();
                  }
                  addToLineVertexArray(He, ot) {
                    const xt = this.lineVertexArray.length;
                    if (He.segment !== void 0) {
                      let zt = He.dist(ot[He.segment + 1]), ln = He.dist(ot[He.segment]);
                      const pn = {};
                      for (let Cn = He.segment + 1; Cn < ot.length; Cn++) pn[Cn] = { x: ot[Cn].x, y: ot[Cn].y, tileUnitDistanceFromAnchor: zt }, Cn < ot.length - 1 && (zt += ot[Cn + 1].dist(ot[Cn]));
                      for (let Cn = He.segment || 0; Cn >= 0; Cn--) pn[Cn] = { x: ot[Cn].x, y: ot[Cn].y, tileUnitDistanceFromAnchor: ln }, Cn > 0 && (ln += ot[Cn - 1].dist(ot[Cn]));
                      for (let Cn = 0; Cn < ot.length; Cn++) {
                        const Nn = pn[Cn];
                        this.lineVertexArray.emplaceBack(Nn.x, Nn.y, Nn.tileUnitDistanceFromAnchor);
                      }
                    }
                    return { lineStartIndex: xt, lineLength: this.lineVertexArray.length - xt };
                  }
                  addSymbols(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr) {
                    const Kr = He.indexArray, xi = He.layoutVertexArray, ui = He.segments.prepareSegment(4 * ot.length, xi, Kr, this.canOverlap ? pn.sortKey : void 0), Ui = this.glyphOffsetArray.length, ia = ui.vertexLength, Na = this.allowVerticalPlacement && Cn === ne.ai.vertical ? Math.PI / 2 : 0, Ta = pn.text && pn.text.sections;
                    for (let Ra = 0; Ra < ot.length; Ra++) {
                      const { tl: oo, tr: fs, bl: Bs, br: fl, tex: Ns, pixelOffsetTL: Es, pixelOffsetBR: cl, minFontScaleX: Xs, minFontScaleY: Ps, glyphOffset: ko, isSDF: Ks, sectionIndex: el } = ot[Ra], Nl = ui.vertexLength, bu = ko[1];
                      J0(xi, Nn.x, Nn.y, oo.x, bu + oo.y, Ns.x, Ns.y, xt, Ks, Es.x, Es.y, Xs, Ps), J0(xi, Nn.x, Nn.y, fs.x, bu + fs.y, Ns.x + Ns.w, Ns.y, xt, Ks, cl.x, Es.y, Xs, Ps), J0(xi, Nn.x, Nn.y, Bs.x, bu + Bs.y, Ns.x, Ns.y + Ns.h, xt, Ks, Es.x, cl.y, Xs, Ps), J0(xi, Nn.x, Nn.y, fl.x, bu + fl.y, Ns.x + Ns.w, Ns.y + Ns.h, xt, Ks, cl.x, cl.y, Xs, Ps), ep(He.dynamicLayoutVertexArray, Nn, Na), Kr.emplaceBack(Nl, Nl + 1, Nl + 2), Kr.emplaceBack(Nl + 1, Nl + 2, Nl + 3), ui.vertexLength += 4, ui.primitiveLength += 2, this.glyphOffsetArray.emplaceBack(ko[0]), Ra !== ot.length - 1 && el === ot[Ra + 1].sectionIndex || He.programConfigurations.populatePaintArrays(xi.length, pn, pn.index, {}, Nr, Ta && Ta[el]);
                    }
                    He.placedSymbolArray.emplaceBack(Nn.x, Nn.y, Ui, this.glyphOffsetArray.length - Ui, ia, Wn, lr, Nn.segment, xt ? xt[0] : 0, xt ? xt[1] : 0, zt[0], zt[1], Cn, 0, !1, 0, xr);
                  }
                  _addCollisionDebugVertex(He, ot, xt, zt, ln, pn) {
                    return ot.emplaceBack(0, 0), He.emplaceBack(xt.x, xt.y, zt, ln, Math.round(pn.x), Math.round(pn.y));
                  }
                  addCollisionDebugVertices(He, ot, xt, zt, ln, pn, Cn) {
                    const Nn = ln.segments.prepareSegment(4, ln.layoutVertexArray, ln.indexArray), Wn = Nn.vertexLength, lr = ln.layoutVertexArray, xr = ln.collisionVertexArray, Nr = Cn.anchorX, Kr = Cn.anchorY;
                    this._addCollisionDebugVertex(lr, xr, pn, Nr, Kr, new ce(He, ot)), this._addCollisionDebugVertex(lr, xr, pn, Nr, Kr, new ce(xt, ot)), this._addCollisionDebugVertex(lr, xr, pn, Nr, Kr, new ce(xt, zt)), this._addCollisionDebugVertex(lr, xr, pn, Nr, Kr, new ce(He, zt)), Nn.vertexLength += 4;
                    const xi = ln.indexArray;
                    xi.emplaceBack(Wn, Wn + 1), xi.emplaceBack(Wn + 1, Wn + 2), xi.emplaceBack(Wn + 2, Wn + 3), xi.emplaceBack(Wn + 3, Wn), Nn.primitiveLength += 4;
                  }
                  addDebugCollisionBoxes(He, ot, xt, zt) {
                    for (let ln = He; ln < ot; ln++) {
                      const pn = this.collisionBoxArray.get(ln), Cn = pn.x1, Nn = pn.y1, Wn = pn.x2, lr = pn.y2;
                      this.addCollisionDebugVertices(Cn, Nn, Wn, lr, zt ? this.textCollisionBox : this.iconCollisionBox, pn.anchorPoint, xt);
                    }
                  }
                  generateCollisionDebugBuffers() {
                    this.hasDebugData() && this.destroyDebugData(), this.textCollisionBox = new rp(ql, La.members, Ml), this.iconCollisionBox = new rp(ql, La.members, Ml);
                    for (let He = 0; He < this.symbolInstances.length; He++) {
                      const ot = this.symbolInstances.get(He);
                      this.addDebugCollisionBoxes(ot.textBoxStartIndex, ot.textBoxEndIndex, ot, !0), this.addDebugCollisionBoxes(ot.verticalTextBoxStartIndex, ot.verticalTextBoxEndIndex, ot, !0), this.addDebugCollisionBoxes(ot.iconBoxStartIndex, ot.iconBoxEndIndex, ot, !1), this.addDebugCollisionBoxes(ot.verticalIconBoxStartIndex, ot.verticalIconBoxEndIndex, ot, !1);
                    }
                  }
                  _deserializeCollisionBoxesForSymbol(He, ot, xt, zt, ln, pn, Cn, Nn, Wn) {
                    const lr = {};
                    for (let xr = ot; xr < xt; xr++) {
                      const Nr = He.get(xr);
                      lr.textBox = { x1: Nr.x1, y1: Nr.y1, x2: Nr.x2, y2: Nr.y2, anchorPointX: Nr.anchorPointX, anchorPointY: Nr.anchorPointY }, lr.textFeatureIndex = Nr.featureIndex;
                      break;
                    }
                    for (let xr = zt; xr < ln; xr++) {
                      const Nr = He.get(xr);
                      lr.verticalTextBox = { x1: Nr.x1, y1: Nr.y1, x2: Nr.x2, y2: Nr.y2, anchorPointX: Nr.anchorPointX, anchorPointY: Nr.anchorPointY }, lr.verticalTextFeatureIndex = Nr.featureIndex;
                      break;
                    }
                    for (let xr = pn; xr < Cn; xr++) {
                      const Nr = He.get(xr);
                      lr.iconBox = { x1: Nr.x1, y1: Nr.y1, x2: Nr.x2, y2: Nr.y2, anchorPointX: Nr.anchorPointX, anchorPointY: Nr.anchorPointY }, lr.iconFeatureIndex = Nr.featureIndex;
                      break;
                    }
                    for (let xr = Nn; xr < Wn; xr++) {
                      const Nr = He.get(xr);
                      lr.verticalIconBox = { x1: Nr.x1, y1: Nr.y1, x2: Nr.x2, y2: Nr.y2, anchorPointX: Nr.anchorPointX, anchorPointY: Nr.anchorPointY }, lr.verticalIconFeatureIndex = Nr.featureIndex;
                      break;
                    }
                    return lr;
                  }
                  deserializeCollisionBoxes(He) {
                    this.collisionArrays = [];
                    for (let ot = 0; ot < this.symbolInstances.length; ot++) {
                      const xt = this.symbolInstances.get(ot);
                      this.collisionArrays.push(this._deserializeCollisionBoxesForSymbol(He, xt.textBoxStartIndex, xt.textBoxEndIndex, xt.verticalTextBoxStartIndex, xt.verticalTextBoxEndIndex, xt.iconBoxStartIndex, xt.iconBoxEndIndex, xt.verticalIconBoxStartIndex, xt.verticalIconBoxEndIndex));
                    }
                  }
                  hasTextData() {
                    return this.text.segments.get().length > 0;
                  }
                  hasIconData() {
                    return this.icon.segments.get().length > 0;
                  }
                  hasDebugData() {
                    return this.textCollisionBox && this.iconCollisionBox;
                  }
                  hasTextCollisionBoxData() {
                    return this.hasDebugData() && this.textCollisionBox.segments.get().length > 0;
                  }
                  hasIconCollisionBoxData() {
                    return this.hasDebugData() && this.iconCollisionBox.segments.get().length > 0;
                  }
                  addIndicesForPlacedSymbol(He, ot) {
                    const xt = He.placedSymbolArray.get(ot), zt = xt.vertexStartIndex + 4 * xt.numGlyphs;
                    for (let ln = xt.vertexStartIndex; ln < zt; ln += 4) He.indexArray.emplaceBack(ln, ln + 1, ln + 2), He.indexArray.emplaceBack(ln + 1, ln + 2, ln + 3);
                  }
                  getSortedSymbolIndexes(He) {
                    if (this.sortedAngle === He && this.symbolInstanceIndexes !== void 0) return this.symbolInstanceIndexes;
                    const ot = Math.sin(He), xt = Math.cos(He), zt = [], ln = [], pn = [];
                    for (let Cn = 0; Cn < this.symbolInstances.length; ++Cn) {
                      pn.push(Cn);
                      const Nn = this.symbolInstances.get(Cn);
                      zt.push(0 | Math.round(ot * Nn.anchorX + xt * Nn.anchorY)), ln.push(Nn.featureIndex);
                    }
                    return pn.sort((Cn, Nn) => zt[Cn] - zt[Nn] || ln[Nn] - ln[Cn]), pn;
                  }
                  addToSortKeyRanges(He, ot) {
                    const xt = this.sortKeyRanges[this.sortKeyRanges.length - 1];
                    xt && xt.sortKey === ot ? xt.symbolInstanceEnd = He + 1 : this.sortKeyRanges.push({ sortKey: ot, symbolInstanceStart: He, symbolInstanceEnd: He + 1 });
                  }
                  sortFeatures(He) {
                    if (this.sortFeaturesByY && this.sortedAngle !== He && !(this.text.segments.get().length > 1 || this.icon.segments.get().length > 1)) {
                      this.symbolInstanceIndexes = this.getSortedSymbolIndexes(He), this.sortedAngle = He, this.text.indexArray.clear(), this.icon.indexArray.clear(), this.featureSortOrder = [];
                      for (const ot of this.symbolInstanceIndexes) {
                        const xt = this.symbolInstances.get(ot);
                        this.featureSortOrder.push(xt.featureIndex), [xt.rightJustifiedTextSymbolIndex, xt.centerJustifiedTextSymbolIndex, xt.leftJustifiedTextSymbolIndex].forEach((zt, ln, pn) => {
                          zt >= 0 && pn.indexOf(zt) === ln && this.addIndicesForPlacedSymbol(this.text, zt);
                        }), xt.verticalPlacedTextSymbolIndex >= 0 && this.addIndicesForPlacedSymbol(this.text, xt.verticalPlacedTextSymbolIndex), xt.placedIconSymbolIndex >= 0 && this.addIndicesForPlacedSymbol(this.icon, xt.placedIconSymbolIndex), xt.verticalPlacedIconSymbolIndex >= 0 && this.addIndicesForPlacedSymbol(this.icon, xt.verticalPlacedIconSymbolIndex);
                      }
                      this.text.indexBuffer && this.text.indexBuffer.updateData(this.text.indexArray), this.icon.indexBuffer && this.icon.indexBuffer.updateData(this.icon.indexArray);
                    }
                  }
                }
                let Hh;
                ss("SymbolBucket", gh, { omit: ["layers", "collisionBoxArray", "features", "compareText"] }), gh.MAX_GLYPHS = 65535, gh.addDynamicAttributes = ep;
                let ip;
                var zh = { get paint() {
                  return ip = ip || new tu({ "icon-opacity": new Os(yt.paint_symbol["icon-opacity"]), "icon-color": new Os(yt.paint_symbol["icon-color"]), "icon-halo-color": new Os(yt.paint_symbol["icon-halo-color"]), "icon-halo-width": new Os(yt.paint_symbol["icon-halo-width"]), "icon-halo-blur": new Os(yt.paint_symbol["icon-halo-blur"]), "icon-translate": new zs(yt.paint_symbol["icon-translate"]), "icon-translate-anchor": new zs(yt.paint_symbol["icon-translate-anchor"]), "text-opacity": new Os(yt.paint_symbol["text-opacity"]), "text-color": new Os(yt.paint_symbol["text-color"], { runtimeType: Ut, getOverride: (Ye) => Ye.textColor, hasOverride: (Ye) => !!Ye.textColor }), "text-halo-color": new Os(yt.paint_symbol["text-halo-color"]), "text-halo-width": new Os(yt.paint_symbol["text-halo-width"]), "text-halo-blur": new Os(yt.paint_symbol["text-halo-blur"]), "text-translate": new zs(yt.paint_symbol["text-translate"]), "text-translate-anchor": new zs(yt.paint_symbol["text-translate-anchor"]) });
                }, get layout() {
                  return Hh = Hh || new tu({ "symbol-placement": new zs(yt.layout_symbol["symbol-placement"]), "symbol-spacing": new zs(yt.layout_symbol["symbol-spacing"]), "symbol-avoid-edges": new zs(yt.layout_symbol["symbol-avoid-edges"]), "symbol-sort-key": new Os(yt.layout_symbol["symbol-sort-key"]), "symbol-z-order": new zs(yt.layout_symbol["symbol-z-order"]), "icon-allow-overlap": new zs(yt.layout_symbol["icon-allow-overlap"]), "icon-overlap": new zs(yt.layout_symbol["icon-overlap"]), "icon-ignore-placement": new zs(yt.layout_symbol["icon-ignore-placement"]), "icon-optional": new zs(yt.layout_symbol["icon-optional"]), "icon-rotation-alignment": new zs(yt.layout_symbol["icon-rotation-alignment"]), "icon-size": new Os(yt.layout_symbol["icon-size"]), "icon-text-fit": new zs(yt.layout_symbol["icon-text-fit"]), "icon-text-fit-padding": new zs(yt.layout_symbol["icon-text-fit-padding"]), "icon-image": new Os(yt.layout_symbol["icon-image"]), "icon-rotate": new Os(yt.layout_symbol["icon-rotate"]), "icon-padding": new Os(yt.layout_symbol["icon-padding"]), "icon-keep-upright": new zs(yt.layout_symbol["icon-keep-upright"]), "icon-offset": new Os(yt.layout_symbol["icon-offset"]), "icon-anchor": new Os(yt.layout_symbol["icon-anchor"]), "icon-pitch-alignment": new zs(yt.layout_symbol["icon-pitch-alignment"]), "text-pitch-alignment": new zs(yt.layout_symbol["text-pitch-alignment"]), "text-rotation-alignment": new zs(yt.layout_symbol["text-rotation-alignment"]), "text-field": new Os(yt.layout_symbol["text-field"]), "text-font": new Os(yt.layout_symbol["text-font"]), "text-size": new Os(yt.layout_symbol["text-size"]), "text-max-width": new Os(yt.layout_symbol["text-max-width"]), "text-line-height": new zs(yt.layout_symbol["text-line-height"]), "text-letter-spacing": new Os(yt.layout_symbol["text-letter-spacing"]), "text-justify": new Os(yt.layout_symbol["text-justify"]), "text-radial-offset": new Os(yt.layout_symbol["text-radial-offset"]), "text-variable-anchor": new zs(yt.layout_symbol["text-variable-anchor"]), "text-variable-anchor-offset": new Os(yt.layout_symbol["text-variable-anchor-offset"]), "text-anchor": new Os(yt.layout_symbol["text-anchor"]), "text-max-angle": new zs(yt.layout_symbol["text-max-angle"]), "text-writing-mode": new zs(yt.layout_symbol["text-writing-mode"]), "text-rotate": new Os(yt.layout_symbol["text-rotate"]), "text-padding": new zs(yt.layout_symbol["text-padding"]), "text-keep-upright": new zs(yt.layout_symbol["text-keep-upright"]), "text-transform": new Os(yt.layout_symbol["text-transform"]), "text-offset": new Os(yt.layout_symbol["text-offset"]), "text-allow-overlap": new zs(yt.layout_symbol["text-allow-overlap"]), "text-overlap": new zs(yt.layout_symbol["text-overlap"]), "text-ignore-placement": new zs(yt.layout_symbol["text-ignore-placement"]), "text-optional": new zs(yt.layout_symbol["text-optional"]) });
                } };
                class o3 {
                  constructor(He) {
                    if (He.property.overrides === void 0) throw new Error("overrides must be provided to instantiate FormatSectionOverride class");
                    this.type = He.property.overrides ? He.property.overrides.runtimeType : Yt, this.defaultValue = He;
                  }
                  evaluate(He) {
                    if (He.formattedSection) {
                      const ot = this.defaultValue.property.overrides;
                      if (ot && ot.hasOverride(He.formattedSection)) return ot.getOverride(He.formattedSection);
                    }
                    return He.feature && He.featureState ? this.defaultValue.evaluate(He.feature, He.featureState) : this.defaultValue.property.specification.default;
                  }
                  eachChild(He) {
                    this.defaultValue.isConstant() || He(this.defaultValue.value._styleExpression.expression);
                  }
                  outputDefined() {
                    return !1;
                  }
                  serialize() {
                    return null;
                  }
                }
                ss("FormatSectionOverride", o3, { omit: ["defaultValue"] });
                class O1 extends $n {
                  constructor(He) {
                    super(He, zh);
                  }
                  recalculate(He, ot) {
                    if (super.recalculate(He, ot), this.layout.get("icon-rotation-alignment") === "auto" && (this.layout.get("symbol-placement") !== "point" ? this.layout._values["icon-rotation-alignment"] = "map" : this.layout._values["icon-rotation-alignment"] = "viewport"), this.layout.get("text-rotation-alignment") === "auto" && (this.layout.get("symbol-placement") !== "point" ? this.layout._values["text-rotation-alignment"] = "map" : this.layout._values["text-rotation-alignment"] = "viewport"), this.layout.get("text-pitch-alignment") === "auto" && (this.layout._values["text-pitch-alignment"] = this.layout.get("text-rotation-alignment") === "map" ? "map" : "viewport"), this.layout.get("icon-pitch-alignment") === "auto" && (this.layout._values["icon-pitch-alignment"] = this.layout.get("icon-rotation-alignment")), this.layout.get("symbol-placement") === "point") {
                      const xt = this.layout.get("text-writing-mode");
                      if (xt) {
                        const zt = [];
                        for (const ln of xt) zt.indexOf(ln) < 0 && zt.push(ln);
                        this.layout._values["text-writing-mode"] = zt;
                      } else this.layout._values["text-writing-mode"] = ["horizontal"];
                    }
                    this._setPaintOverrides();
                  }
                  getValueAndResolveTokens(He, ot, xt, zt) {
                    const ln = this.layout.get(He).evaluate(ot, {}, xt, zt), pn = this._unevaluatedLayout._values[He];
                    return pn.isDataDriven() || Vo(pn.value) || !ln ? ln : function(Cn, Nn) {
                      return Nn.replace(/{([^{}]+)}/g, (Wn, lr) => Cn && lr in Cn ? String(Cn[lr]) : "");
                    }(ot.properties, ln);
                  }
                  createBucket(He) {
                    return new gh(He);
                  }
                  queryRadius() {
                    return 0;
                  }
                  queryIntersectsFeature() {
                    throw new Error("Should take a different path in FeatureIndex");
                  }
                  _setPaintOverrides() {
                    for (const He of zh.paint.overridableProperties) {
                      if (!O1.hasPaintOverride(this.layout, He)) continue;
                      const ot = this.paint.get(He), xt = new o3(ot), zt = new po(xt, ot.property.specification);
                      let ln = null;
                      ln = ot.value.kind === "constant" || ot.value.kind === "source" ? new Go("source", zt) : new es("composite", zt, ot.value.zoomStops), this.paint._values[He] = new lu(ot.property, ln, ot.parameters);
                    }
                  }
                  _handleOverridablePaintPropertyUpdate(He, ot, xt) {
                    return !(!this.layout || ot.isDataDriven() || xt.isDataDriven()) && O1.hasPaintOverride(this.layout, He);
                  }
                  static hasPaintOverride(He, ot) {
                    const xt = He.get("text-field"), zt = zh.paint.properties[ot];
                    let ln = !1;
                    const pn = (Cn) => {
                      for (const Nn of Cn) if (zt.overrides && zt.overrides.hasOverride(Nn)) return void (ln = !0);
                    };
                    if (xt.value.kind === "constant" && xt.value.value instanceof jr) pn(xt.value.value.sections);
                    else if (xt.value.kind === "source") {
                      const Cn = (Wn) => {
                        if (!ln) if (Wn instanceof Zi && ri(Wn.value) === dn) {
                          const lr = Wn.value;
                          pn(lr.sections);
                        } else Wn instanceof Oi ? pn(Wn.sections) : Wn.eachChild(Cn);
                      }, Nn = xt.value;
                      Nn._styleExpression && Cn(Nn._styleExpression.expression);
                    }
                    return ln;
                  }
                }
                let s3;
                var l1 = { get paint() {
                  return s3 = s3 || new tu({ "background-color": new zs(yt.paint_background["background-color"]), "background-pattern": new xc(yt.paint_background["background-pattern"]), "background-opacity": new zs(yt.paint_background["background-opacity"]) });
                } };
                class N4 extends $n {
                  constructor(He) {
                    super(He, l1);
                  }
                }
                let l3;
                var P4 = { get paint() {
                  return l3 = l3 || new tu({ "raster-opacity": new zs(yt.paint_raster["raster-opacity"]), "raster-hue-rotate": new zs(yt.paint_raster["raster-hue-rotate"]), "raster-brightness-min": new zs(yt.paint_raster["raster-brightness-min"]), "raster-brightness-max": new zs(yt.paint_raster["raster-brightness-max"]), "raster-saturation": new zs(yt.paint_raster["raster-saturation"]), "raster-contrast": new zs(yt.paint_raster["raster-contrast"]), "raster-resampling": new zs(yt.paint_raster["raster-resampling"]), "raster-fade-duration": new zs(yt.paint_raster["raster-fade-duration"]) });
                } };
                class $4 extends $n {
                  constructor(He) {
                    super(He, P4);
                  }
                }
                class F4 extends $n {
                  constructor(He) {
                    super(He, {}), this.onAdd = (ot) => {
                      this.implementation.onAdd && this.implementation.onAdd(ot, ot.painter.context.gl);
                    }, this.onRemove = (ot) => {
                      this.implementation.onRemove && this.implementation.onRemove(ot, ot.painter.context.gl);
                    }, this.implementation = He;
                  }
                  is3D() {
                    return this.implementation.renderingMode === "3d";
                  }
                  hasOffscreenPass() {
                    return this.implementation.prerender !== void 0;
                  }
                  recalculate() {
                  }
                  updateTransitions() {
                  }
                  hasTransition() {
                    return !1;
                  }
                  serialize() {
                    throw new Error("Custom layers cannot be serialized");
                  }
                }
                class c3 {
                  constructor(He) {
                    this._methodToThrottle = He, this._triggered = !1, typeof MessageChannel < "u" && (this._channel = new MessageChannel(), this._channel.port2.onmessage = () => {
                      this._triggered = !1, this._methodToThrottle();
                    });
                  }
                  trigger() {
                    this._triggered || (this._triggered = !0, this._channel ? this._channel.port1.postMessage(!0) : setTimeout(() => {
                      this._triggered = !1, this._methodToThrottle();
                    }, 0));
                  }
                  remove() {
                    delete this._channel, this._methodToThrottle = () => {
                    };
                  }
                }
                const vh = 63710088e-1;
                class ff {
                  constructor(He, ot) {
                    if (isNaN(He) || isNaN(ot)) throw new Error(`Invalid LngLat object: (${He}, ${ot})`);
                    if (this.lng = +He, this.lat = +ot, this.lat > 90 || this.lat < -90) throw new Error("Invalid LngLat latitude value: must be between -90 and 90");
                  }
                  wrap() {
                    return new ff(be(this.lng, -180, 180), this.lat);
                  }
                  toArray() {
                    return [this.lng, this.lat];
                  }
                  toString() {
                    return `LngLat(${this.lng}, ${this.lat})`;
                  }
                  distanceTo(He) {
                    const ot = Math.PI / 180, xt = this.lat * ot, zt = He.lat * ot, ln = Math.sin(xt) * Math.sin(zt) + Math.cos(xt) * Math.cos(zt) * Math.cos((He.lng - this.lng) * ot);
                    return vh * Math.acos(Math.min(ln, 1));
                  }
                  static convert(He) {
                    if (He instanceof ff) return He;
                    if (Array.isArray(He) && (He.length === 2 || He.length === 3)) return new ff(Number(He[0]), Number(He[1]));
                    if (!Array.isArray(He) && typeof He == "object" && He !== null) return new ff(Number("lng" in He ? He.lng : He.lon), Number(He.lat));
                    throw new Error("`LngLatLike` argument must be specified as a LngLat instance, an object {lng: <lng>, lat: <lat>}, an object {lon: <lng>, lat: <lat>}, or an array of [<lng>, <lat>]");
                  }
                }
                const u3 = 2 * Math.PI * vh;
                function q1(Ye) {
                  return u3 * Math.cos(Ye * Math.PI / 180);
                }
                function g2(Ye) {
                  return (180 + Ye) / 360;
                }
                function oh(Ye) {
                  return (180 - 180 / Math.PI * Math.log(Math.tan(Math.PI / 4 + Ye * Math.PI / 360))) / 360;
                }
                function V2(Ye, He) {
                  return Ye / q1(He);
                }
                function _2(Ye) {
                  const He = 180 - 360 * Ye;
                  return 360 / Math.PI * Math.atan(Math.exp(He * Math.PI / 180)) - 90;
                }
                class y2 {
                  constructor(He, ot, xt = 0) {
                    this.x = +He, this.y = +ot, this.z = +xt;
                  }
                  static fromLngLat(He, ot = 0) {
                    const xt = ff.convert(He);
                    return new y2(g2(xt.lng), oh(xt.lat), V2(ot, xt.lat));
                  }
                  toLngLat() {
                    return new ff(360 * this.x - 180, _2(this.y));
                  }
                  toAltitude() {
                    return He = this.z, ot = this.y, He * q1(_2(ot));
                    var He, ot;
                  }
                  meterInMercatorCoordinateUnits() {
                    return 1 / u3 * (He = _2(this.y), 1 / Math.cos(He * Math.PI / 180));
                    var He;
                  }
                }
                function Jh(Ye, He, ot) {
                  var xt = 2 * Math.PI * 6378137 / 256 / Math.pow(2, ot);
                  return [Ye * xt - 2 * Math.PI * 6378137 / 2, He * xt - 2 * Math.PI * 6378137 / 2];
                }
                class Gf {
                  constructor(He, ot, xt) {
                    if (He < 0 || He > 25 || xt < 0 || xt >= Math.pow(2, He) || ot < 0 || ot >= Math.pow(2, He)) throw new Error(`x=${ot}, y=${xt}, z=${He} outside of bounds. 0<=x<${Math.pow(2, He)}, 0<=y<${Math.pow(2, He)} 0<=z<=25 `);
                    this.z = He, this.x = ot, this.y = xt, this.key = U2(0, He, He, ot, xt);
                  }
                  equals(He) {
                    return this.z === He.z && this.x === He.x && this.y === He.y;
                  }
                  url(He, ot, xt) {
                    const zt = (pn = this.x, Cn = this.y, Nn = this.z, Wn = Jh(256 * pn, 256 * (Cn = Math.pow(2, Nn) - Cn - 1), Nn), lr = Jh(256 * (pn + 1), 256 * (Cn + 1), Nn), Wn[0] + "," + Wn[1] + "," + lr[0] + "," + lr[1]), ln = function(xr, Nr, Kr) {
                      let xi, ui = "";
                      for (let Ui = xr; Ui > 0; Ui--) xi = 1 << Ui - 1, ui += (Nr & xi ? 1 : 0) + (Kr & xi ? 2 : 0);
                      return ui;
                    }(this.z, this.x, this.y);
                    var pn, Cn, Nn, Wn, lr;
                    return He[(this.x + this.y) % He.length].replace(/{prefix}/g, (this.x % 16).toString(16) + (this.y % 16).toString(16)).replace(/{z}/g, String(this.z)).replace(/{x}/g, String(this.x)).replace(/{y}/g, String(xt === "tms" ? Math.pow(2, this.z) - this.y - 1 : this.y)).replace(/{ratio}/g, ot > 1 ? "@2x" : "").replace(/{quadkey}/g, ln).replace(/{bbox-epsg-3857}/g, zt);
                  }
                  isChildOf(He) {
                    const ot = this.z - He.z;
                    return ot > 0 && He.x === this.x >> ot && He.y === this.y >> ot;
                  }
                  getTilePoint(He) {
                    const ot = Math.pow(2, this.z);
                    return new ce((He.x * ot - this.x) * Uu, (He.y * ot - this.y) * Uu);
                  }
                  toString() {
                    return `${this.z}/${this.x}/${this.y}`;
                  }
                }
                class d3 {
                  constructor(He, ot) {
                    this.wrap = He, this.canonical = ot, this.key = U2(He, ot.z, ot.z, ot.x, ot.y);
                  }
                }
                class hf {
                  constructor(He, ot, xt, zt, ln) {
                    if (He < xt) throw new Error(`overscaledZ should be >= z; overscaledZ = ${He}; z = ${xt}`);
                    this.overscaledZ = He, this.wrap = ot, this.canonical = new Gf(xt, +zt, +ln), this.key = U2(ot, He, xt, zt, ln);
                  }
                  clone() {
                    return new hf(this.overscaledZ, this.wrap, this.canonical.z, this.canonical.x, this.canonical.y);
                  }
                  equals(He) {
                    return this.overscaledZ === He.overscaledZ && this.wrap === He.wrap && this.canonical.equals(He.canonical);
                  }
                  scaledTo(He) {
                    if (He > this.overscaledZ) throw new Error(`targetZ > this.overscaledZ; targetZ = ${He}; overscaledZ = ${this.overscaledZ}`);
                    const ot = this.canonical.z - He;
                    return He > this.canonical.z ? new hf(He, this.wrap, this.canonical.z, this.canonical.x, this.canonical.y) : new hf(He, this.wrap, He, this.canonical.x >> ot, this.canonical.y >> ot);
                  }
                  calculateScaledKey(He, ot) {
                    if (He > this.overscaledZ) throw new Error(`targetZ > this.overscaledZ; targetZ = ${He}; overscaledZ = ${this.overscaledZ}`);
                    const xt = this.canonical.z - He;
                    return He > this.canonical.z ? U2(this.wrap * +ot, He, this.canonical.z, this.canonical.x, this.canonical.y) : U2(this.wrap * +ot, He, He, this.canonical.x >> xt, this.canonical.y >> xt);
                  }
                  isChildOf(He) {
                    if (He.wrap !== this.wrap) return !1;
                    const ot = this.canonical.z - He.canonical.z;
                    return He.overscaledZ === 0 || He.overscaledZ < this.overscaledZ && He.canonical.x === this.canonical.x >> ot && He.canonical.y === this.canonical.y >> ot;
                  }
                  children(He) {
                    if (this.overscaledZ >= He) return [new hf(this.overscaledZ + 1, this.wrap, this.canonical.z, this.canonical.x, this.canonical.y)];
                    const ot = this.canonical.z + 1, xt = 2 * this.canonical.x, zt = 2 * this.canonical.y;
                    return [new hf(ot, this.wrap, ot, xt, zt), new hf(ot, this.wrap, ot, xt + 1, zt), new hf(ot, this.wrap, ot, xt, zt + 1), new hf(ot, this.wrap, ot, xt + 1, zt + 1)];
                  }
                  isLessThan(He) {
                    return this.wrap < He.wrap || !(this.wrap > He.wrap) && (this.overscaledZ < He.overscaledZ || !(this.overscaledZ > He.overscaledZ) && (this.canonical.x < He.canonical.x || !(this.canonical.x > He.canonical.x) && this.canonical.y < He.canonical.y));
                  }
                  wrapped() {
                    return new hf(this.overscaledZ, 0, this.canonical.z, this.canonical.x, this.canonical.y);
                  }
                  unwrapTo(He) {
                    return new hf(this.overscaledZ, He, this.canonical.z, this.canonical.x, this.canonical.y);
                  }
                  overscaleFactor() {
                    return Math.pow(2, this.overscaledZ - this.canonical.z);
                  }
                  toUnwrapped() {
                    return new d3(this.wrap, this.canonical);
                  }
                  toString() {
                    return `${this.overscaledZ}/${this.canonical.x}/${this.canonical.y}`;
                  }
                  getTilePoint(He) {
                    return this.canonical.getTilePoint(new y2(He.x - this.wrap, He.y));
                  }
                }
                function U2(Ye, He, ot, xt, zt) {
                  (Ye *= 2) < 0 && (Ye = -1 * Ye - 1);
                  const ln = 1 << ot;
                  return (ln * ln * Ye + ln * zt + xt).toString(36) + ot.toString(36) + He.toString(36);
                }
                ss("CanonicalTileID", Gf), ss("OverscaledTileID", hf, { omit: ["posMatrix"] });
                class fd {
                  constructor(He, ot, xt, zt = 1, ln = 1, pn = 1, Cn = 0) {
                    if (this.uid = He, ot.height !== ot.width) throw new RangeError("DEM tiles must be square");
                    if (xt && !["mapbox", "terrarium", "custom"].includes(xt)) return void Qe(`"${xt}" is not a valid encoding type. Valid types include "mapbox", "terrarium" and "custom".`);
                    this.stride = ot.height;
                    const Nn = this.dim = ot.height - 2;
                    switch (this.data = new Uint32Array(ot.data.buffer), xt) {
                      case "terrarium":
                        this.redFactor = 256, this.greenFactor = 1, this.blueFactor = 390625e-8, this.baseShift = 32768;
                        break;
                      case "custom":
                        this.redFactor = zt, this.greenFactor = ln, this.blueFactor = pn, this.baseShift = Cn;
                        break;
                      default:
                        this.redFactor = 6553.6, this.greenFactor = 25.6, this.blueFactor = 0.1, this.baseShift = 1e4;
                    }
                    for (let Wn = 0; Wn < Nn; Wn++) this.data[this._idx(-1, Wn)] = this.data[this._idx(0, Wn)], this.data[this._idx(Nn, Wn)] = this.data[this._idx(Nn - 1, Wn)], this.data[this._idx(Wn, -1)] = this.data[this._idx(Wn, 0)], this.data[this._idx(Wn, Nn)] = this.data[this._idx(Wn, Nn - 1)];
                    this.data[this._idx(-1, -1)] = this.data[this._idx(0, 0)], this.data[this._idx(Nn, -1)] = this.data[this._idx(Nn - 1, 0)], this.data[this._idx(-1, Nn)] = this.data[this._idx(0, Nn - 1)], this.data[this._idx(Nn, Nn)] = this.data[this._idx(Nn - 1, Nn - 1)], this.min = Number.MAX_SAFE_INTEGER, this.max = Number.MIN_SAFE_INTEGER;
                    for (let Wn = 0; Wn < Nn; Wn++) for (let lr = 0; lr < Nn; lr++) {
                      const xr = this.get(Wn, lr);
                      xr > this.max && (this.max = xr), xr < this.min && (this.min = xr);
                    }
                  }
                  get(He, ot) {
                    const xt = new Uint8Array(this.data.buffer), zt = 4 * this._idx(He, ot);
                    return this.unpack(xt[zt], xt[zt + 1], xt[zt + 2]);
                  }
                  getUnpackVector() {
                    return [this.redFactor, this.greenFactor, this.blueFactor, this.baseShift];
                  }
                  _idx(He, ot) {
                    if (He < -1 || He >= this.dim + 1 || ot < -1 || ot >= this.dim + 1) throw new RangeError("out of range source coordinates for DEM data");
                    return (ot + 1) * this.stride + (He + 1);
                  }
                  unpack(He, ot, xt) {
                    return He * this.redFactor + ot * this.greenFactor + xt * this.blueFactor - this.baseShift;
                  }
                  getPixels() {
                    return new pu({ width: this.stride, height: this.stride }, new Uint8Array(this.data.buffer));
                  }
                  backfillBorder(He, ot, xt) {
                    if (this.dim !== He.dim) throw new Error("dem dimension mismatch");
                    let zt = ot * this.dim, ln = ot * this.dim + this.dim, pn = xt * this.dim, Cn = xt * this.dim + this.dim;
                    switch (ot) {
                      case -1:
                        zt = ln - 1;
                        break;
                      case 1:
                        ln = zt + 1;
                    }
                    switch (xt) {
                      case -1:
                        pn = Cn - 1;
                        break;
                      case 1:
                        Cn = pn + 1;
                    }
                    const Nn = -ot * this.dim, Wn = -xt * this.dim;
                    for (let lr = pn; lr < Cn; lr++) for (let xr = zt; xr < ln; xr++) this.data[this._idx(xr, lr)] = He.data[this._idx(xr + Nn, lr + Wn)];
                  }
                }
                ss("DEMData", fd);
                class bh {
                  constructor(He) {
                    this._stringToNumber = {}, this._numberToString = [];
                    for (let ot = 0; ot < He.length; ot++) {
                      const xt = He[ot];
                      this._stringToNumber[xt] = ot, this._numberToString[ot] = xt;
                    }
                  }
                  encode(He) {
                    return this._stringToNumber[He];
                  }
                  decode(He) {
                    if (He >= this._numberToString.length) throw new Error(`Out of bounds. Index requested n=${He} can't be >= this._numberToString.length ${this._numberToString.length}`);
                    return this._numberToString[He];
                  }
                }
                class f3 {
                  constructor(He, ot, xt, zt, ln) {
                    this.type = "Feature", this._vectorTileFeature = He, He._z = ot, He._x = xt, He._y = zt, this.properties = He.properties, this.id = ln;
                  }
                  get geometry() {
                    return this._geometry === void 0 && (this._geometry = this._vectorTileFeature.toGeoJSON(this._vectorTileFeature._x, this._vectorTileFeature._y, this._vectorTileFeature._z).geometry), this._geometry;
                  }
                  set geometry(He) {
                    this._geometry = He;
                  }
                  toJSON() {
                    const He = { geometry: this.geometry };
                    for (const ot in this) ot !== "_geometry" && ot !== "_vectorTileFeature" && (He[ot] = this[ot]);
                    return He;
                  }
                }
                class Ne {
                  constructor(He, ot) {
                    this.tileID = He, this.x = He.canonical.x, this.y = He.canonical.y, this.z = He.canonical.z, this.grid = new fc(Uu, 16, 0), this.grid3D = new fc(Uu, 16, 0), this.featureIndexArray = new Wo(), this.promoteId = ot;
                  }
                  insert(He, ot, xt, zt, ln, pn) {
                    const Cn = this.featureIndexArray.length;
                    this.featureIndexArray.emplaceBack(xt, zt, ln);
                    const Nn = pn ? this.grid3D : this.grid;
                    for (let Wn = 0; Wn < ot.length; Wn++) {
                      const lr = ot[Wn], xr = [1 / 0, 1 / 0, -1 / 0, -1 / 0];
                      for (let Nr = 0; Nr < lr.length; Nr++) {
                        const Kr = lr[Nr];
                        xr[0] = Math.min(xr[0], Kr.x), xr[1] = Math.min(xr[1], Kr.y), xr[2] = Math.max(xr[2], Kr.x), xr[3] = Math.max(xr[3], Kr.y);
                      }
                      xr[0] < Uu && xr[1] < Uu && xr[2] >= 0 && xr[3] >= 0 && Nn.insert(Cn, xr[0], xr[1], xr[2], xr[3]);
                    }
                  }
                  loadVTLayers() {
                    return this.vtLayers || (this.vtLayers = new Jo.VectorTile(new qf(this.rawTileData)).layers, this.sourceLayerCoder = new bh(this.vtLayers ? Object.keys(this.vtLayers).sort() : ["_geojsonTileLayer"])), this.vtLayers;
                  }
                  query(He, ot, xt, zt) {
                    this.loadVTLayers();
                    const ln = He.params || {}, pn = Uu / He.tileSize / He.scale, Cn = Ql(ln.filter), Nn = He.queryGeometry, Wn = He.queryPadding * pn, lr = ut(Nn), xr = this.grid.query(lr.minX - Wn, lr.minY - Wn, lr.maxX + Wn, lr.maxY + Wn), Nr = ut(He.cameraQueryGeometry), Kr = this.grid3D.query(Nr.minX - Wn, Nr.minY - Wn, Nr.maxX + Wn, Nr.maxY + Wn, (Ui, ia, Na, Ta) => function(Ra, oo, fs, Bs, fl) {
                      for (const Es of Ra) if (oo <= Es.x && fs <= Es.y && Bs >= Es.x && fl >= Es.y) return !0;
                      const Ns = [new ce(oo, fs), new ce(oo, fl), new ce(Bs, fl), new ce(Bs, fs)];
                      if (Ra.length > 2) {
                        for (const Es of Ns) if (ld(Ra, Es)) return !0;
                      }
                      for (let Es = 0; Es < Ra.length - 1; Es++) if (Ch(Ra[Es], Ra[Es + 1], Ns)) return !0;
                      return !1;
                    }(He.cameraQueryGeometry, Ui - Wn, ia - Wn, Na + Wn, Ta + Wn));
                    for (const Ui of Kr) xr.push(Ui);
                    xr.sort(kt);
                    const xi = {};
                    let ui;
                    for (let Ui = 0; Ui < xr.length; Ui++) {
                      const ia = xr[Ui];
                      if (ia === ui) continue;
                      ui = ia;
                      const Na = this.featureIndexArray.get(ia);
                      let Ta = null;
                      this.loadMatchingFeature(xi, Na.bucketIndex, Na.sourceLayerIndex, Na.featureIndex, Cn, ln.layers, ln.availableImages, ot, xt, zt, (Ra, oo, fs) => (Ta || (Ta = $d(Ra)), oo.queryIntersectsFeature(Nn, Ra, fs, Ta, this.z, He.transform, pn, He.pixelPosMatrix)));
                    }
                    return xi;
                  }
                  loadMatchingFeature(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr) {
                    const Nr = this.bucketLayerIDs[ot];
                    if (pn && !function(Ui, ia) {
                      for (let Na = 0; Na < Ui.length; Na++) if (ia.indexOf(Ui[Na]) >= 0) return !0;
                      return !1;
                    }(pn, Nr)) return;
                    const Kr = this.sourceLayerCoder.decode(xt), xi = this.vtLayers[Kr].feature(zt);
                    if (ln.needGeometry) {
                      const Ui = Sd(xi, !0);
                      if (!ln.filter(new Ko(this.tileID.overscaledZ), Ui, this.tileID.canonical)) return;
                    } else if (!ln.filter(new Ko(this.tileID.overscaledZ), xi)) return;
                    const ui = this.getId(xi, Kr);
                    for (let Ui = 0; Ui < Nr.length; Ui++) {
                      const ia = Nr[Ui];
                      if (pn && pn.indexOf(ia) < 0) continue;
                      const Na = Nn[ia];
                      if (!Na) continue;
                      let Ta = {};
                      ui && lr && (Ta = lr.getState(Na.sourceLayer || "_geojsonTileLayer", ui));
                      const Ra = Ce({}, Wn[ia]);
                      Ra.paint = je(Ra.paint, Na.paint, xi, Ta, Cn), Ra.layout = je(Ra.layout, Na.layout, xi, Ta, Cn);
                      const oo = !xr || xr(xi, Na, Ta);
                      if (!oo) continue;
                      const fs = new f3(xi, this.z, this.x, this.y, ui);
                      fs.layer = Ra;
                      let Bs = He[ia];
                      Bs === void 0 && (Bs = He[ia] = []), Bs.push({ featureIndex: zt, feature: fs, intersectionZ: oo });
                    }
                  }
                  lookupSymbolFeatures(He, ot, xt, zt, ln, pn, Cn, Nn) {
                    const Wn = {};
                    this.loadVTLayers();
                    const lr = Ql(ln);
                    for (const xr of He) this.loadMatchingFeature(Wn, xt, zt, xr, lr, pn, Cn, Nn, ot);
                    return Wn;
                  }
                  hasLayer(He) {
                    for (const ot of this.bucketLayerIDs) for (const xt of ot) if (He === xt) return !0;
                    return !1;
                  }
                  getId(He, ot) {
                    let xt = He.id;
                    if (this.promoteId) {
                      const zt = typeof this.promoteId == "string" ? this.promoteId : this.promoteId[ot];
                      xt = He.properties[zt], typeof xt == "boolean" && (xt = Number(xt));
                    }
                    return xt;
                  }
                }
                function je(Ye, He, ot, xt, zt) {
                  return we(Ye, (ln, pn) => {
                    const Cn = He instanceof Ul ? He.get(pn) : null;
                    return Cn && Cn.evaluate ? Cn.evaluate(ot, xt, zt) : Cn;
                  });
                }
                function ut(Ye) {
                  let He = 1 / 0, ot = 1 / 0, xt = -1 / 0, zt = -1 / 0;
                  for (const ln of Ye) He = Math.min(He, ln.x), ot = Math.min(ot, ln.y), xt = Math.max(xt, ln.x), zt = Math.max(zt, ln.y);
                  return { minX: He, minY: ot, maxX: xt, maxY: zt };
                }
                function kt(Ye, He) {
                  return He - Ye;
                }
                function Jt(Ye, He, ot, xt, zt) {
                  const ln = [];
                  for (let pn = 0; pn < Ye.length; pn++) {
                    const Cn = Ye[pn];
                    let Nn;
                    for (let Wn = 0; Wn < Cn.length - 1; Wn++) {
                      let lr = Cn[Wn], xr = Cn[Wn + 1];
                      lr.x < He && xr.x < He || (lr.x < He ? lr = new ce(He, lr.y + (xr.y - lr.y) * ((He - lr.x) / (xr.x - lr.x)))._round() : xr.x < He && (xr = new ce(He, lr.y + (xr.y - lr.y) * ((He - lr.x) / (xr.x - lr.x)))._round()), lr.y < ot && xr.y < ot || (lr.y < ot ? lr = new ce(lr.x + (xr.x - lr.x) * ((ot - lr.y) / (xr.y - lr.y)), ot)._round() : xr.y < ot && (xr = new ce(lr.x + (xr.x - lr.x) * ((ot - lr.y) / (xr.y - lr.y)), ot)._round()), lr.x >= xt && xr.x >= xt || (lr.x >= xt ? lr = new ce(xt, lr.y + (xr.y - lr.y) * ((xt - lr.x) / (xr.x - lr.x)))._round() : xr.x >= xt && (xr = new ce(xt, lr.y + (xr.y - lr.y) * ((xt - lr.x) / (xr.x - lr.x)))._round()), lr.y >= zt && xr.y >= zt || (lr.y >= zt ? lr = new ce(lr.x + (xr.x - lr.x) * ((zt - lr.y) / (xr.y - lr.y)), zt)._round() : xr.y >= zt && (xr = new ce(lr.x + (xr.x - lr.x) * ((zt - lr.y) / (xr.y - lr.y)), zt)._round()), Nn && lr.equals(Nn[Nn.length - 1]) || (Nn = [lr], ln.push(Nn)), Nn.push(xr)))));
                    }
                  }
                  return ln;
                }
                ss("FeatureIndex", Ne, { omit: ["rawTileData", "sourceLayerCoder"] });
                class fn extends ce {
                  constructor(He, ot, xt, zt) {
                    super(He, ot), this.angle = xt, zt !== void 0 && (this.segment = zt);
                  }
                  clone() {
                    return new fn(this.x, this.y, this.angle, this.segment);
                  }
                }
                function Sn(Ye, He, ot, xt, zt) {
                  if (He.segment === void 0 || ot === 0) return !0;
                  let ln = He, pn = He.segment + 1, Cn = 0;
                  for (; Cn > -ot / 2; ) {
                    if (pn--, pn < 0) return !1;
                    Cn -= Ye[pn].dist(ln), ln = Ye[pn];
                  }
                  Cn += Ye[pn].dist(Ye[pn + 1]), pn++;
                  const Nn = [];
                  let Wn = 0;
                  for (; Cn < ot / 2; ) {
                    const lr = Ye[pn - 1], xr = Ye[pn], Nr = Ye[pn + 1];
                    if (!Nr) return !1;
                    let Kr = lr.angleTo(xr) - xr.angleTo(Nr);
                    for (Kr = Math.abs((Kr + 3 * Math.PI) % (2 * Math.PI) - Math.PI), Nn.push({ distance: Cn, angleDelta: Kr }), Wn += Kr; Cn - Nn[0].distance > xt; ) Wn -= Nn.shift().angleDelta;
                    if (Wn > zt) return !1;
                    pn++, Cn += xr.dist(Nr);
                  }
                  return !0;
                }
                function Hn(Ye) {
                  let He = 0;
                  for (let ot = 0; ot < Ye.length - 1; ot++) He += Ye[ot].dist(Ye[ot + 1]);
                  return He;
                }
                function nr(Ye, He, ot) {
                  return Ye ? 0.6 * He * ot : 0;
                }
                function kr(Ye, He) {
                  return Math.max(Ye ? Ye.right - Ye.left : 0, He ? He.right - He.left : 0);
                }
                function Ir(Ye, He, ot, xt, zt, ln) {
                  const pn = nr(ot, zt, ln), Cn = kr(ot, xt) * ln;
                  let Nn = 0;
                  const Wn = Hn(Ye) / 2;
                  for (let lr = 0; lr < Ye.length - 1; lr++) {
                    const xr = Ye[lr], Nr = Ye[lr + 1], Kr = xr.dist(Nr);
                    if (Nn + Kr > Wn) {
                      const xi = (Wn - Nn) / Kr, ui = Rr.number(xr.x, Nr.x, xi), Ui = Rr.number(xr.y, Nr.y, xi), ia = new fn(ui, Ui, Nr.angleTo(xr), lr);
                      return ia._round(), !pn || Sn(Ye, ia, Cn, pn, He) ? ia : void 0;
                    }
                    Nn += Kr;
                  }
                }
                function qr(Ye, He, ot, xt, zt, ln, pn, Cn, Nn) {
                  const Wn = nr(xt, ln, pn), lr = kr(xt, zt), xr = lr * pn, Nr = Ye[0].x === 0 || Ye[0].x === Nn || Ye[0].y === 0 || Ye[0].y === Nn;
                  return He - xr < He / 4 && (He = xr + He / 4), ci(Ye, Nr ? He / 2 * Cn % He : (lr / 2 + 2 * ln) * pn * Cn % He, He, Wn, ot, xr, Nr, !1, Nn);
                }
                function ci(Ye, He, ot, xt, zt, ln, pn, Cn, Nn) {
                  const Wn = ln / 2, lr = Hn(Ye);
                  let xr = 0, Nr = He - ot, Kr = [];
                  for (let xi = 0; xi < Ye.length - 1; xi++) {
                    const ui = Ye[xi], Ui = Ye[xi + 1], ia = ui.dist(Ui), Na = Ui.angleTo(ui);
                    for (; Nr + ot < xr + ia; ) {
                      Nr += ot;
                      const Ta = (Nr - xr) / ia, Ra = Rr.number(ui.x, Ui.x, Ta), oo = Rr.number(ui.y, Ui.y, Ta);
                      if (Ra >= 0 && Ra < Nn && oo >= 0 && oo < Nn && Nr - Wn >= 0 && Nr + Wn <= lr) {
                        const fs = new fn(Ra, oo, Na, xi);
                        fs._round(), xt && !Sn(Ye, fs, ln, xt, zt) || Kr.push(fs);
                      }
                    }
                    xr += ia;
                  }
                  return Cn || Kr.length || pn || (Kr = ci(Ye, xr / 2, ot, xt, zt, ln, pn, !0, Nn)), Kr;
                }
                ss("Anchor", fn);
                const ni = ef;
                function Ii(Ye, He, ot, xt) {
                  const zt = [], ln = Ye.image, pn = ln.pixelRatio, Cn = ln.paddedRect.w - 2 * ni, Nn = ln.paddedRect.h - 2 * ni;
                  let Wn = { x1: Ye.left, y1: Ye.top, x2: Ye.right, y2: Ye.bottom };
                  const lr = ln.stretchX || [[0, Cn]], xr = ln.stretchY || [[0, Nn]], Nr = (ko, Ks) => ko + Ks[1] - Ks[0], Kr = lr.reduce(Nr, 0), xi = xr.reduce(Nr, 0), ui = Cn - Kr, Ui = Nn - xi;
                  let ia = 0, Na = Kr, Ta = 0, Ra = xi, oo = 0, fs = ui, Bs = 0, fl = Ui;
                  if (ln.content && xt) {
                    const ko = ln.content, Ks = ko[2] - ko[0], el = ko[3] - ko[1];
                    (ln.textFitWidth || ln.textFitHeight) && (Wn = O4(Ye)), ia = Gi(lr, 0, ko[0]), Ta = Gi(xr, 0, ko[1]), Na = Gi(lr, ko[0], ko[2]), Ra = Gi(xr, ko[1], ko[3]), oo = ko[0] - ia, Bs = ko[1] - Ta, fs = Ks - Na, fl = el - Ra;
                  }
                  const Ns = Wn.x1, Es = Wn.y1, cl = Wn.x2 - Ns, Xs = Wn.y2 - Es, Ps = (ko, Ks, el, Nl) => {
                    const bu = ga(ko.stretch - ia, Na, cl, Ns), Tu = $a(ko.fixed - oo, fs, ko.stretch, Kr), B1 = ga(Ks.stretch - Ta, Ra, Xs, Es), Hd = $a(Ks.fixed - Bs, fl, Ks.stretch, xi), nd = ga(el.stretch - ia, Na, cl, Ns), j1 = $a(el.fixed - oo, fs, el.stretch, Kr), Fd = ga(Nl.stretch - Ta, Ra, Xs, Es), Cd = $a(Nl.fixed - Bs, fl, Nl.stretch, xi), L1 = new ce(bu, B1), m1 = new ce(nd, B1), xd = new ce(nd, Fd), H1 = new ce(bu, Fd), rd = new ce(Tu / pn, Hd / pn), Ld = new ce(j1 / pn, Cd / pn), zd = He * Math.PI / 180;
                    if (zd) {
                      const g1 = Math.sin(zd), _1 = Math.cos(zd), v1 = [_1, -g1, g1, _1];
                      L1._matMult(v1), m1._matMult(v1), H1._matMult(v1), xd._matMult(v1);
                    }
                    const Nd = ko.stretch + ko.fixed, k1 = el.stretch + el.fixed, Tf = Ks.stretch + Ks.fixed, c1 = Nl.stretch + Nl.fixed;
                    return { tl: L1, tr: m1, bl: H1, br: xd, tex: { x: ln.paddedRect.x + ni + Nd, y: ln.paddedRect.y + ni + Tf, w: k1 - Nd, h: c1 - Tf }, writingMode: void 0, glyphOffset: [0, 0], sectionIndex: 0, pixelOffsetTL: rd, pixelOffsetBR: Ld, minFontScaleX: fs / pn / cl, minFontScaleY: fl / pn / Xs, isSDF: ot };
                  };
                  if (xt && (ln.stretchX || ln.stretchY)) {
                    const ko = xa(lr, ui, Kr), Ks = xa(xr, Ui, xi);
                    for (let el = 0; el < ko.length - 1; el++) {
                      const Nl = ko[el], bu = ko[el + 1];
                      for (let Tu = 0; Tu < Ks.length - 1; Tu++) {
                        const B1 = Ks[Tu], Hd = Ks[Tu + 1];
                        zt.push(Ps(Nl, B1, bu, Hd));
                      }
                    }
                  } else zt.push(Ps({ fixed: 0, stretch: -1 }, { fixed: 0, stretch: -1 }, { fixed: 0, stretch: Cn + 1 }, { fixed: 0, stretch: Nn + 1 }));
                  return zt;
                }
                function Gi(Ye, He, ot) {
                  let xt = 0;
                  for (const zt of Ye) xt += Math.max(He, Math.min(ot, zt[1])) - Math.max(He, Math.min(ot, zt[0]));
                  return xt;
                }
                function xa(Ye, He, ot) {
                  const xt = [{ fixed: -1, stretch: 0 }];
                  for (const [zt, ln] of Ye) {
                    const pn = xt[xt.length - 1];
                    xt.push({ fixed: zt - pn.stretch, stretch: pn.stretch }), xt.push({ fixed: zt - pn.stretch, stretch: pn.stretch + (ln - zt) });
                  }
                  return xt.push({ fixed: He + ni, stretch: ot }), xt;
                }
                function ga(Ye, He, ot, xt) {
                  return Ye / He * ot + xt;
                }
                function $a(Ye, He, ot, xt) {
                  return Ye - He * ot / xt;
                }
                class Va {
                  constructor(He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr) {
                    var xr;
                    if (this.boxStartIndex = He.length, Wn) {
                      let Nr = pn.top, Kr = pn.bottom;
                      const xi = pn.collisionPadding;
                      xi && (Nr -= xi[1], Kr += xi[3]);
                      let ui = Kr - Nr;
                      ui > 0 && (ui = Math.max(10, ui), this.circleDiameter = ui);
                    } else {
                      const Nr = !((xr = pn.image) === null || xr === void 0) && xr.content && (pn.image.textFitWidth || pn.image.textFitHeight) ? O4(pn) : { x1: pn.left, y1: pn.top, x2: pn.right, y2: pn.bottom };
                      Nr.y1 = Nr.y1 * Cn - Nn[0], Nr.y2 = Nr.y2 * Cn + Nn[2], Nr.x1 = Nr.x1 * Cn - Nn[3], Nr.x2 = Nr.x2 * Cn + Nn[1];
                      const Kr = pn.collisionPadding;
                      if (Kr && (Nr.x1 -= Kr[0] * Cn, Nr.y1 -= Kr[1] * Cn, Nr.x2 += Kr[2] * Cn, Nr.y2 += Kr[3] * Cn), lr) {
                        const xi = new ce(Nr.x1, Nr.y1), ui = new ce(Nr.x2, Nr.y1), Ui = new ce(Nr.x1, Nr.y2), ia = new ce(Nr.x2, Nr.y2), Na = lr * Math.PI / 180;
                        xi._rotate(Na), ui._rotate(Na), Ui._rotate(Na), ia._rotate(Na), Nr.x1 = Math.min(xi.x, ui.x, Ui.x, ia.x), Nr.x2 = Math.max(xi.x, ui.x, Ui.x, ia.x), Nr.y1 = Math.min(xi.y, ui.y, Ui.y, ia.y), Nr.y2 = Math.max(xi.y, ui.y, Ui.y, ia.y);
                      }
                      He.emplaceBack(ot.x, ot.y, Nr.x1, Nr.y1, Nr.x2, Nr.y2, xt, zt, ln);
                    }
                    this.boxEndIndex = He.length;
                  }
                }
                class eo {
                  constructor(He = [], ot = (xt, zt) => xt < zt ? -1 : xt > zt ? 1 : 0) {
                    if (this.data = He, this.length = this.data.length, this.compare = ot, this.length > 0) for (let xt = (this.length >> 1) - 1; xt >= 0; xt--) this._down(xt);
                  }
                  push(He) {
                    this.data.push(He), this._up(this.length++);
                  }
                  pop() {
                    if (this.length === 0) return;
                    const He = this.data[0], ot = this.data.pop();
                    return --this.length > 0 && (this.data[0] = ot, this._down(0)), He;
                  }
                  peek() {
                    return this.data[0];
                  }
                  _up(He) {
                    const { data: ot, compare: xt } = this, zt = ot[He];
                    for (; He > 0; ) {
                      const ln = He - 1 >> 1, pn = ot[ln];
                      if (xt(zt, pn) >= 0) break;
                      ot[He] = pn, He = ln;
                    }
                    ot[He] = zt;
                  }
                  _down(He) {
                    const { data: ot, compare: xt } = this, zt = this.length >> 1, ln = ot[He];
                    for (; He < zt; ) {
                      let pn = 1 + (He << 1);
                      const Cn = pn + 1;
                      if (Cn < this.length && xt(ot[Cn], ot[pn]) < 0 && (pn = Cn), xt(ot[pn], ln) >= 0) break;
                      ot[He] = ot[pn], He = pn;
                    }
                    ot[He] = ln;
                  }
                }
                function co(Ye, He = 1, ot = !1) {
                  let xt = 1 / 0, zt = 1 / 0, ln = -1 / 0, pn = -1 / 0;
                  const Cn = Ye[0];
                  for (let ui = 0; ui < Cn.length; ui++) {
                    const Ui = Cn[ui];
                    (!ui || Ui.x < xt) && (xt = Ui.x), (!ui || Ui.y < zt) && (zt = Ui.y), (!ui || Ui.x > ln) && (ln = Ui.x), (!ui || Ui.y > pn) && (pn = Ui.y);
                  }
                  const Nn = ln - xt, Wn = pn - zt, lr = Math.min(Nn, Wn);
                  let xr = lr / 2;
                  const Nr = new eo([], wo);
                  if (lr === 0) return new ce(xt, zt);
                  for (let ui = xt; ui < ln; ui += lr) for (let Ui = zt; Ui < pn; Ui += lr) Nr.push(new ds(ui + xr, Ui + xr, xr, Ye));
                  let Kr = function(ui) {
                    let Ui = 0, ia = 0, Na = 0;
                    const Ta = ui[0];
                    for (let Ra = 0, oo = Ta.length, fs = oo - 1; Ra < oo; fs = Ra++) {
                      const Bs = Ta[Ra], fl = Ta[fs], Ns = Bs.x * fl.y - fl.x * Bs.y;
                      ia += (Bs.x + fl.x) * Ns, Na += (Bs.y + fl.y) * Ns, Ui += 3 * Ns;
                    }
                    return new ds(ia / Ui, Na / Ui, 0, ui);
                  }(Ye), xi = Nr.length;
                  for (; Nr.length; ) {
                    const ui = Nr.pop();
                    (ui.d > Kr.d || !Kr.d) && (Kr = ui, ot && console.log("found best %d after %d probes", Math.round(1e4 * ui.d) / 1e4, xi)), ui.max - Kr.d <= He || (xr = ui.h / 2, Nr.push(new ds(ui.p.x - xr, ui.p.y - xr, xr, Ye)), Nr.push(new ds(ui.p.x + xr, ui.p.y - xr, xr, Ye)), Nr.push(new ds(ui.p.x - xr, ui.p.y + xr, xr, Ye)), Nr.push(new ds(ui.p.x + xr, ui.p.y + xr, xr, Ye)), xi += 4);
                  }
                  return ot && (console.log(`num probes: ${xi}`), console.log(`best distance: ${Kr.d}`)), Kr.p;
                }
                function wo(Ye, He) {
                  return He.max - Ye.max;
                }
                function ds(Ye, He, ot, xt) {
                  this.p = new ce(Ye, He), this.h = ot, this.d = function(zt, ln) {
                    let pn = !1, Cn = 1 / 0;
                    for (let Nn = 0; Nn < ln.length; Nn++) {
                      const Wn = ln[Nn];
                      for (let lr = 0, xr = Wn.length, Nr = xr - 1; lr < xr; Nr = lr++) {
                        const Kr = Wn[lr], xi = Wn[Nr];
                        Kr.y > zt.y != xi.y > zt.y && zt.x < (xi.x - Kr.x) * (zt.y - Kr.y) / (xi.y - Kr.y) + Kr.x && (pn = !pn), Cn = Math.min(Cn, sd(zt, Kr, xi));
                      }
                    }
                    return (pn ? 1 : -1) * Math.sqrt(Cn);
                  }(this.p, xt), this.max = this.d + this.h * Math.SQRT2;
                }
                var Lo;
                ne.ar = void 0, (Lo = ne.ar || (ne.ar = {}))[Lo.center = 1] = "center", Lo[Lo.left = 2] = "left", Lo[Lo.right = 3] = "right", Lo[Lo.top = 4] = "top", Lo[Lo.bottom = 5] = "bottom", Lo[Lo["top-left"] = 6] = "top-left", Lo[Lo["top-right"] = 7] = "top-right", Lo[Lo["bottom-left"] = 8] = "bottom-left", Lo[Lo["bottom-right"] = 9] = "bottom-right";
                const As = 7, Fs = Number.POSITIVE_INFINITY;
                function Pl(Ye, He) {
                  return He[1] !== Fs ? function(ot, xt, zt) {
                    let ln = 0, pn = 0;
                    switch (xt = Math.abs(xt), zt = Math.abs(zt), ot) {
                      case "top-right":
                      case "top-left":
                      case "top":
                        pn = zt - As;
                        break;
                      case "bottom-right":
                      case "bottom-left":
                      case "bottom":
                        pn = -zt + As;
                    }
                    switch (ot) {
                      case "top-right":
                      case "bottom-right":
                      case "right":
                        ln = -xt;
                        break;
                      case "top-left":
                      case "bottom-left":
                      case "left":
                        ln = xt;
                    }
                    return [ln, pn];
                  }(Ye, He[0], He[1]) : function(ot, xt) {
                    let zt = 0, ln = 0;
                    xt < 0 && (xt = 0);
                    const pn = xt / Math.SQRT2;
                    switch (ot) {
                      case "top-right":
                      case "top-left":
                        ln = pn - As;
                        break;
                      case "bottom-right":
                      case "bottom-left":
                        ln = -pn + As;
                        break;
                      case "bottom":
                        ln = -xt + As;
                        break;
                      case "top":
                        ln = xt - As;
                    }
                    switch (ot) {
                      case "top-right":
                      case "bottom-right":
                        zt = -pn;
                        break;
                      case "top-left":
                      case "bottom-left":
                        zt = pn;
                        break;
                      case "left":
                        zt = xt;
                        break;
                      case "right":
                        zt = -xt;
                    }
                    return [zt, ln];
                  }(Ye, He[0]);
                }
                function rl(Ye, He, ot) {
                  var xt;
                  const zt = Ye.layout, ln = (xt = zt.get("text-variable-anchor-offset")) === null || xt === void 0 ? void 0 : xt.evaluate(He, {}, ot);
                  if (ln) {
                    const Cn = ln.values, Nn = [];
                    for (let Wn = 0; Wn < Cn.length; Wn += 2) {
                      const lr = Nn[Wn] = Cn[Wn], xr = Cn[Wn + 1].map((Nr) => Nr * Us);
                      lr.startsWith("top") ? xr[1] -= As : lr.startsWith("bottom") && (xr[1] += As), Nn[Wn + 1] = xr;
                    }
                    return new Bi(Nn);
                  }
                  const pn = zt.get("text-variable-anchor");
                  if (pn) {
                    let Cn;
                    Cn = Ye._unevaluatedLayout.getValue("text-radial-offset") !== void 0 ? [zt.get("text-radial-offset").evaluate(He, {}, ot) * Us, Fs] : zt.get("text-offset").evaluate(He, {}, ot).map((Wn) => Wn * Us);
                    const Nn = [];
                    for (const Wn of pn) Nn.push(Wn, Pl(Wn, Cn));
                    return new Bi(Nn);
                  }
                  return null;
                }
                function Rl(Ye) {
                  switch (Ye) {
                    case "right":
                    case "top-right":
                    case "bottom-right":
                      return "right";
                    case "left":
                    case "top-left":
                    case "bottom-left":
                      return "left";
                  }
                  return "center";
                }
                function pl(Ye, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr) {
                  let xr = ln.textMaxSize.evaluate(He, {});
                  xr === void 0 && (xr = pn);
                  const Nr = Ye.layers[0].layout, Kr = Nr.get("icon-offset").evaluate(He, {}, lr), xi = Hc(ot.horizontal), ui = pn / 24, Ui = Ye.tilePixelRatio * ui, ia = Ye.tilePixelRatio * xr / 24, Na = Ye.tilePixelRatio * Cn, Ta = Ye.tilePixelRatio * Nr.get("symbol-spacing"), Ra = Nr.get("text-padding") * Ye.tilePixelRatio, oo = function(ko, Ks, el, Nl = 1) {
                    const bu = ko.get("icon-padding").evaluate(Ks, {}, el), Tu = bu && bu.values;
                    return [Tu[0] * Nl, Tu[1] * Nl, Tu[2] * Nl, Tu[3] * Nl];
                  }(Nr, He, lr, Ye.tilePixelRatio), fs = Nr.get("text-max-angle") / 180 * Math.PI, Bs = Nr.get("text-rotation-alignment") !== "viewport" && Nr.get("symbol-placement") !== "point", fl = Nr.get("icon-rotation-alignment") === "map" && Nr.get("symbol-placement") !== "point", Ns = Nr.get("symbol-placement"), Es = Ta / 2, cl = Nr.get("icon-text-fit");
                  let Xs;
                  xt && cl !== "none" && (Ye.allowVerticalPlacement && ot.vertical && (Xs = K0(xt, ot.vertical, cl, Nr.get("icon-text-fit-padding"), Kr, ui)), xi && (xt = K0(xt, xi, cl, Nr.get("icon-text-fit-padding"), Kr, ui)));
                  const Ps = (ko, Ks) => {
                    Ks.x < 0 || Ks.x >= Uu || Ks.y < 0 || Ks.y >= Uu || function(el, Nl, bu, Tu, B1, Hd, nd, j1, Fd, Cd, L1, m1, xd, H1, rd, Ld, zd, Nd, k1, Tf, c1, g1, _1, v1, Th) {
                      const pf = el.addToLineVertexArray(Nl, bu);
                      let pd, mu, u1, wf, Of = 0, ap = 0, h3 = 0, q2 = 0, If = -1, v2 = -1;
                      const Sf = {};
                      let j2 = Ju("");
                      if (el.allowVerticalPlacement && Tu.vertical) {
                        const ad = j1.layout.get("text-rotate").evaluate(c1, {}, v1) + 90, kd = Tu.vertical;
                        u1 = new Va(Fd, Nl, Cd, L1, m1, kd, xd, H1, rd, ad), nd && (wf = new Va(Fd, Nl, Cd, L1, m1, nd, zd, Nd, rd, ad));
                      }
                      if (B1) {
                        const ad = j1.layout.get("icon-rotate").evaluate(c1, {}), kd = j1.layout.get("icon-text-fit") !== "none", Df = Ii(B1, ad, _1, kd), Gd = nd ? Ii(nd, ad, _1, kd) : void 0;
                        mu = new Va(Fd, Nl, Cd, L1, m1, B1, zd, Nd, !1, ad), Of = 4 * Df.length;
                        const Nf = el.iconSizeData;
                        let lh = null;
                        Nf.kind === "source" ? (lh = [ah * j1.layout.get("icon-size").evaluate(c1, {})], lh[0] > Bh && Qe(`${el.layerIds[0]}: Value for "icon-size" is >= ${ih}. Reduce your "icon-size".`)) : Nf.kind === "composite" && (lh = [ah * g1.compositeIconSizes[0].evaluate(c1, {}, v1), ah * g1.compositeIconSizes[1].evaluate(c1, {}, v1)], (lh[0] > Bh || lh[1] > Bh) && Qe(`${el.layerIds[0]}: Value for "icon-size" is >= ${ih}. Reduce your "icon-size".`)), el.addSymbols(el.icon, Df, lh, Tf, k1, c1, ne.ai.none, Nl, pf.lineStartIndex, pf.lineLength, -1, v1), If = el.icon.placedSymbolArray.length - 1, Gd && (ap = 4 * Gd.length, el.addSymbols(el.icon, Gd, lh, Tf, k1, c1, ne.ai.vertical, Nl, pf.lineStartIndex, pf.lineLength, -1, v1), v2 = el.icon.placedSymbolArray.length - 1);
                      }
                      const G2 = Object.keys(Tu.horizontal);
                      for (const ad of G2) {
                        const kd = Tu.horizontal[ad];
                        if (!pd) {
                          j2 = Ju(kd.text);
                          const Gd = j1.layout.get("text-rotate").evaluate(c1, {}, v1);
                          pd = new Va(Fd, Nl, Cd, L1, m1, kd, xd, H1, rd, Gd);
                        }
                        const Df = kd.positionedLines.length === 1;
                        if (h3 += Zs(el, Nl, kd, Hd, j1, rd, c1, Ld, pf, Tu.vertical ? ne.ai.horizontal : ne.ai.horizontalOnly, Df ? G2 : [ad], Sf, If, g1, v1), Df) break;
                      }
                      Tu.vertical && (q2 += Zs(el, Nl, Tu.vertical, Hd, j1, rd, c1, Ld, pf, ne.ai.vertical, ["vertical"], Sf, v2, g1, v1));
                      const p3 = pd ? pd.boxStartIndex : el.collisionBoxArray.length, B4 = pd ? pd.boxEndIndex : el.collisionBoxArray.length, W2 = u1 ? u1.boxStartIndex : el.collisionBoxArray.length, H4 = u1 ? u1.boxEndIndex : el.collisionBoxArray.length, z4 = mu ? mu.boxStartIndex : el.collisionBoxArray.length, V4 = mu ? mu.boxEndIndex : el.collisionBoxArray.length, U4 = wf ? wf.boxStartIndex : el.collisionBoxArray.length, q4 = wf ? wf.boxEndIndex : el.collisionBoxArray.length;
                      let G1 = -1;
                      const n2 = (ad, kd) => ad && ad.circleDiameter ? Math.max(ad.circleDiameter, kd) : kd;
                      G1 = n2(pd, G1), G1 = n2(u1, G1), G1 = n2(mu, G1), G1 = n2(wf, G1);
                      const m3 = G1 > -1 ? 1 : 0;
                      m3 && (G1 *= Th / Us), el.glyphOffsetArray.length >= gh.MAX_GLYPHS && Qe("Too many glyphs being rendered in a tile. See https://github.com/mapbox/mapbox-gl-js/issues/2907"), c1.sortKey !== void 0 && el.addToSortKeyRanges(el.symbolInstances.length, c1.sortKey);
                      const sh = rl(j1, c1, v1), [n_, j4] = function(ad, kd) {
                        const Df = ad.length, Gd = kd == null ? void 0 : kd.values;
                        if ((Gd == null ? void 0 : Gd.length) > 0) for (let Nf = 0; Nf < Gd.length; Nf += 2) {
                          const lh = ne.ar[Gd[Nf]], g3 = Gd[Nf + 1];
                          ad.emplaceBack(lh, g3[0], g3[1]);
                        }
                        return [Df, ad.length];
                      }(el.textAnchorOffsets, sh);
                      el.symbolInstances.emplaceBack(Nl.x, Nl.y, Sf.right >= 0 ? Sf.right : -1, Sf.center >= 0 ? Sf.center : -1, Sf.left >= 0 ? Sf.left : -1, Sf.vertical || -1, If, v2, j2, p3, B4, W2, H4, z4, V4, U4, q4, Cd, h3, q2, Of, ap, m3, 0, xd, G1, n_, j4);
                    }(Ye, Ks, ko, ot, xt, zt, Xs, Ye.layers[0], Ye.collisionBoxArray, He.index, He.sourceLayerIndex, Ye.index, Ui, [Ra, Ra, Ra, Ra], Bs, Nn, Na, oo, fl, Kr, He, ln, Wn, lr, pn);
                  };
                  if (Ns === "line") for (const ko of Jt(He.geometry, 0, 0, Uu, Uu)) {
                    const Ks = qr(ko, Ta, fs, ot.vertical || xi, xt, 24, ia, Ye.overscaling, Uu);
                    for (const el of Ks) xi && tc(Ye, xi.text, Es, el) || Ps(ko, el);
                  }
                  else if (Ns === "line-center") {
                    for (const ko of He.geometry) if (ko.length > 1) {
                      const Ks = Ir(ko, fs, ot.vertical || xi, xt, 24, ia);
                      Ks && Ps(ko, Ks);
                    }
                  } else if (He.type === "Polygon") for (const ko of Sl(He.geometry, 0)) {
                    const Ks = co(ko, 16);
                    Ps(ko[0], new fn(Ks.x, Ks.y, 0));
                  }
                  else if (He.type === "LineString") for (const ko of He.geometry) Ps(ko, new fn(ko[0].x, ko[0].y, 0));
                  else if (He.type === "Point") for (const ko of He.geometry) for (const Ks of ko) Ps([Ks], new fn(Ks.x, Ks.y, 0));
                }
                function Zs(Ye, He, ot, xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi) {
                  const ui = function(Na, Ta, Ra, oo, fs, Bs, fl, Ns) {
                    const Es = oo.layout.get("text-rotate").evaluate(Bs, {}) * Math.PI / 180, cl = [];
                    for (const Xs of Ta.positionedLines) for (const Ps of Xs.positionedGlyphs) {
                      if (!Ps.rect) continue;
                      const ko = Ps.rect || {};
                      let Ks = p2 + 1, el = !0, Nl = 1, bu = 0;
                      const Tu = (fs || Ns) && Ps.vertical, B1 = Ps.metrics.advance * Ps.scale / 2;
                      if (Ns && Ta.verticalizable) {
                        const g1 = (Ps.scale - 1) * Us, _1 = (Us - Ps.metrics.width * Ps.scale) / 2;
                        bu = Xs.lineOffset / 2 - (Ps.imageName ? -_1 : g1);
                      }
                      if (Ps.imageName) {
                        const g1 = fl[Ps.imageName];
                        el = g1.sdf, Nl = g1.pixelRatio, Ks = ef / Nl;
                      }
                      const Hd = fs ? [Ps.x + B1, Ps.y] : [0, 0];
                      let nd = fs ? [0, 0] : [Ps.x + B1 + Ra[0], Ps.y + Ra[1] - bu], j1 = [0, 0];
                      Tu && (j1 = nd, nd = [0, 0]);
                      const Fd = Ps.metrics.isDoubleResolution ? 2 : 1, Cd = (Ps.metrics.left - Ks) * Ps.scale - B1 + nd[0], L1 = (-Ps.metrics.top - Ks) * Ps.scale + nd[1], m1 = Cd + ko.w / Fd * Ps.scale / Nl, xd = L1 + ko.h / Fd * Ps.scale / Nl, H1 = new ce(Cd, L1), rd = new ce(m1, L1), Ld = new ce(Cd, xd), zd = new ce(m1, xd);
                      if (Tu) {
                        const g1 = new ce(-B1, B1 - Kh), _1 = -Math.PI / 2, v1 = Us / 2 - B1, Th = Ps.imageName ? v1 : 0, pf = new ce(5 - Kh - v1, -Th), pd = new ce(...j1);
                        H1._rotateAround(_1, g1)._add(pf)._add(pd), rd._rotateAround(_1, g1)._add(pf)._add(pd), Ld._rotateAround(_1, g1)._add(pf)._add(pd), zd._rotateAround(_1, g1)._add(pf)._add(pd);
                      }
                      if (Es) {
                        const g1 = Math.sin(Es), _1 = Math.cos(Es), v1 = [_1, -g1, g1, _1];
                        H1._matMult(v1), rd._matMult(v1), Ld._matMult(v1), zd._matMult(v1);
                      }
                      const Nd = new ce(0, 0), k1 = new ce(0, 0);
                      cl.push({ tl: H1, tr: rd, bl: Ld, br: zd, tex: ko, writingMode: Ta.writingMode, glyphOffset: Hd, sectionIndex: Ps.sectionIndex, isSDF: el, pixelOffsetTL: Nd, pixelOffsetBR: k1, minFontScaleX: 0, minFontScaleY: 0 });
                    }
                    return cl;
                  }(0, ot, Cn, zt, ln, pn, xt, Ye.allowVerticalPlacement), Ui = Ye.textSizeData;
                  let ia = null;
                  Ui.kind === "source" ? (ia = [ah * zt.layout.get("text-size").evaluate(pn, {})], ia[0] > Bh && Qe(`${Ye.layerIds[0]}: Value for "text-size" is >= ${ih}. Reduce your "text-size".`)) : Ui.kind === "composite" && (ia = [ah * Kr.compositeTextSizes[0].evaluate(pn, {}, xi), ah * Kr.compositeTextSizes[1].evaluate(pn, {}, xi)], (ia[0] > Bh || ia[1] > Bh) && Qe(`${Ye.layerIds[0]}: Value for "text-size" is >= ${ih}. Reduce your "text-size".`)), Ye.addSymbols(Ye.text, ui, ia, Cn, ln, pn, Wn, He, Nn.lineStartIndex, Nn.lineLength, Nr, xi);
                  for (const Na of lr) xr[Na] = Ye.text.placedSymbolArray.length - 1;
                  return 4 * ui.length;
                }
                function Hc(Ye) {
                  for (const He in Ye) return Ye[He];
                  return null;
                }
                function tc(Ye, He, ot, xt) {
                  const zt = Ye.compareText;
                  if (He in zt) {
                    const ln = zt[He];
                    for (let pn = ln.length - 1; pn >= 0; pn--) if (xt.dist(ln[pn]) < ot) return !0;
                  } else zt[He] = [];
                  return zt[He].push(xt), !1;
                }
                const Pu = [Int8Array, Uint8Array, Uint8ClampedArray, Int16Array, Uint16Array, Int32Array, Uint32Array, Float32Array, Float64Array];
                class Cl {
                  static from(He) {
                    if (!(He instanceof ArrayBuffer)) throw new Error("Data must be an instance of ArrayBuffer.");
                    const [ot, xt] = new Uint8Array(He, 0, 2);
                    if (ot !== 219) throw new Error("Data does not appear to be in a KDBush format.");
                    const zt = xt >> 4;
                    if (zt !== 1) throw new Error(`Got v${zt} data when expected v1.`);
                    const ln = Pu[15 & xt];
                    if (!ln) throw new Error("Unrecognized array type.");
                    const [pn] = new Uint16Array(He, 2, 1), [Cn] = new Uint32Array(He, 4, 1);
                    return new Cl(Cn, pn, ln, He);
                  }
                  constructor(He, ot = 64, xt = Float64Array, zt) {
                    if (isNaN(He) || He < 0) throw new Error(`Unpexpected numItems value: ${He}.`);
                    this.numItems = +He, this.nodeSize = Math.min(Math.max(+ot, 2), 65535), this.ArrayType = xt, this.IndexArrayType = He < 65536 ? Uint16Array : Uint32Array;
                    const ln = Pu.indexOf(this.ArrayType), pn = 2 * He * this.ArrayType.BYTES_PER_ELEMENT, Cn = He * this.IndexArrayType.BYTES_PER_ELEMENT, Nn = (8 - Cn % 8) % 8;
                    if (ln < 0) throw new Error(`Unexpected typed array class: ${xt}.`);
                    zt && zt instanceof ArrayBuffer ? (this.data = zt, this.ids = new this.IndexArrayType(this.data, 8, He), this.coords = new this.ArrayType(this.data, 8 + Cn + Nn, 2 * He), this._pos = 2 * He, this._finished = !0) : (this.data = new ArrayBuffer(8 + pn + Cn + Nn), this.ids = new this.IndexArrayType(this.data, 8, He), this.coords = new this.ArrayType(this.data, 8 + Cn + Nn, 2 * He), this._pos = 0, this._finished = !1, new Uint8Array(this.data, 0, 2).set([219, 16 + ln]), new Uint16Array(this.data, 2, 1)[0] = ot, new Uint32Array(this.data, 4, 1)[0] = He);
                  }
                  add(He, ot) {
                    const xt = this._pos >> 1;
                    return this.ids[xt] = xt, this.coords[this._pos++] = He, this.coords[this._pos++] = ot, xt;
                  }
                  finish() {
                    const He = this._pos >> 1;
                    if (He !== this.numItems) throw new Error(`Added ${He} items when expected ${this.numItems}.`);
                    return vu(this.ids, this.coords, this.nodeSize, 0, this.numItems - 1, 0), this._finished = !0, this;
                  }
                  range(He, ot, xt, zt) {
                    if (!this._finished) throw new Error("Data not yet indexed - call index.finish().");
                    const { ids: ln, coords: pn, nodeSize: Cn } = this, Nn = [0, ln.length - 1, 0], Wn = [];
                    for (; Nn.length; ) {
                      const lr = Nn.pop() || 0, xr = Nn.pop() || 0, Nr = Nn.pop() || 0;
                      if (xr - Nr <= Cn) {
                        for (let Ui = Nr; Ui <= xr; Ui++) {
                          const ia = pn[2 * Ui], Na = pn[2 * Ui + 1];
                          ia >= He && ia <= xt && Na >= ot && Na <= zt && Wn.push(ln[Ui]);
                        }
                        continue;
                      }
                      const Kr = Nr + xr >> 1, xi = pn[2 * Kr], ui = pn[2 * Kr + 1];
                      xi >= He && xi <= xt && ui >= ot && ui <= zt && Wn.push(ln[Kr]), (lr === 0 ? He <= xi : ot <= ui) && (Nn.push(Nr), Nn.push(Kr - 1), Nn.push(1 - lr)), (lr === 0 ? xt >= xi : zt >= ui) && (Nn.push(Kr + 1), Nn.push(xr), Nn.push(1 - lr));
                    }
                    return Wn;
                  }
                  within(He, ot, xt) {
                    if (!this._finished) throw new Error("Data not yet indexed - call index.finish().");
                    const { ids: zt, coords: ln, nodeSize: pn } = this, Cn = [0, zt.length - 1, 0], Nn = [], Wn = xt * xt;
                    for (; Cn.length; ) {
                      const lr = Cn.pop() || 0, xr = Cn.pop() || 0, Nr = Cn.pop() || 0;
                      if (xr - Nr <= pn) {
                        for (let Ui = Nr; Ui <= xr; Ui++) $u(ln[2 * Ui], ln[2 * Ui + 1], He, ot) <= Wn && Nn.push(zt[Ui]);
                        continue;
                      }
                      const Kr = Nr + xr >> 1, xi = ln[2 * Kr], ui = ln[2 * Kr + 1];
                      $u(xi, ui, He, ot) <= Wn && Nn.push(zt[Kr]), (lr === 0 ? He - xt <= xi : ot - xt <= ui) && (Cn.push(Nr), Cn.push(Kr - 1), Cn.push(1 - lr)), (lr === 0 ? He + xt >= xi : ot + xt >= ui) && (Cn.push(Kr + 1), Cn.push(xr), Cn.push(1 - lr));
                    }
                    return Nn;
                  }
                }
                function vu(Ye, He, ot, xt, zt, ln) {
                  if (zt - xt <= ot) return;
                  const pn = xt + zt >> 1;
                  qu(Ye, He, pn, xt, zt, ln), vu(Ye, He, ot, xt, pn - 1, 1 - ln), vu(Ye, He, ot, pn + 1, zt, 1 - ln);
                }
                function qu(Ye, He, ot, xt, zt, ln) {
                  for (; zt > xt; ) {
                    if (zt - xt > 600) {
                      const Wn = zt - xt + 1, lr = ot - xt + 1, xr = Math.log(Wn), Nr = 0.5 * Math.exp(2 * xr / 3), Kr = 0.5 * Math.sqrt(xr * Nr * (Wn - Nr) / Wn) * (lr - Wn / 2 < 0 ? -1 : 1);
                      qu(Ye, He, ot, Math.max(xt, Math.floor(ot - lr * Nr / Wn + Kr)), Math.min(zt, Math.floor(ot + (Wn - lr) * Nr / Wn + Kr)), ln);
                    }
                    const pn = He[2 * ot + ln];
                    let Cn = xt, Nn = zt;
                    for (Nu(Ye, He, xt, ot), He[2 * zt + ln] > pn && Nu(Ye, He, xt, zt); Cn < Nn; ) {
                      for (Nu(Ye, He, Cn, Nn), Cn++, Nn--; He[2 * Cn + ln] < pn; ) Cn++;
                      for (; He[2 * Nn + ln] > pn; ) Nn--;
                    }
                    He[2 * xt + ln] === pn ? Nu(Ye, He, xt, Nn) : (Nn++, Nu(Ye, He, Nn, zt)), Nn <= ot && (xt = Nn + 1), ot <= Nn && (zt = Nn - 1);
                  }
                }
                function Nu(Ye, He, ot, xt) {
                  ju(Ye, ot, xt), ju(He, 2 * ot, 2 * xt), ju(He, 2 * ot + 1, 2 * xt + 1);
                }
                function ju(Ye, He, ot) {
                  const xt = Ye[He];
                  Ye[He] = Ye[ot], Ye[ot] = xt;
                }
                function $u(Ye, He, ot, xt) {
                  const zt = Ye - ot, ln = He - xt;
                  return zt * zt + ln * ln;
                }
                var $1;
                ne.bf = void 0, ($1 = ne.bf || (ne.bf = {})).create = "create", $1.load = "load", $1.fullLoad = "fullLoad";
                let Zu = null, F1 = [];
                const Td = 1e3 / 60, hd = "loadTime", Dd = "fullLoadTime", Qd = { mark(Ye) {
                  performance.mark(Ye);
                }, frame(Ye) {
                  const He = Ye;
                  if (Zu != null) {
                    const ot = He - Zu;
                    F1.push(ot);
                  }
                  Zu = He;
                }, clearMetrics() {
                  Zu = null, F1 = [], performance.clearMeasures(hd), performance.clearMeasures(Dd);
                  for (const Ye in ne.bf) performance.clearMarks(ne.bf[Ye]);
                }, getPerformanceMetrics() {
                  performance.measure(hd, ne.bf.create, ne.bf.load), performance.measure(Dd, ne.bf.create, ne.bf.fullLoad);
                  const Ye = performance.getEntriesByName(hd)[0].duration, He = performance.getEntriesByName(Dd)[0].duration, ot = F1.length, xt = 1 / (F1.reduce((ln, pn) => ln + pn, 0) / ot / 1e3), zt = F1.filter((ln) => ln > Td).reduce((ln, pn) => ln + (pn - Td) / Td, 0);
                  return { loadTime: Ye, fullLoadTime: He, fps: xt, percentDroppedFrames: zt / (ot + zt) * 100, totalFrames: ot };
                } };
                ne.$ = class extends Be {
                }, ne.A = cd, ne.B = fu, ne.C = function(Ye) {
                  if (Re == null) {
                    const He = Ye.navigator ? Ye.navigator.userAgent : null;
                    Re = !!Ye.safari || !(!He || !(/\b(iPad|iPhone|iPod)\b/.test(He) || He.match("Safari") && !He.match("Chrome")));
                  }
                  return Re;
                }, ne.D = zs, ne.E = bt, ne.F = class {
                  constructor(Ye, He) {
                    this.target = Ye, this.mapId = He, this.resolveRejects = {}, this.tasks = {}, this.taskQueue = [], this.abortControllers = {}, this.messageHandlers = {}, this.invoker = new c3(() => this.process()), this.subscription = function(ot, xt, zt, ln) {
                      return ot.addEventListener(xt, zt, ln), { unsubscribe: () => {
                        ot.removeEventListener(xt, zt, ln);
                      } };
                    }(this.target, "message", (ot) => this.receive(ot), !1), this.globalScope = Me(self) ? Ye : window;
                  }
                  registerMessageHandler(Ye, He) {
                    this.messageHandlers[Ye] = He;
                  }
                  sendAsync(Ye, He) {
                    return new Promise((ot, xt) => {
                      const zt = Math.round(1e18 * Math.random()).toString(36).substring(0, 10);
                      this.resolveRejects[zt] = { resolve: ot, reject: xt }, He && He.signal.addEventListener("abort", () => {
                        delete this.resolveRejects[zt];
                        const Cn = { id: zt, type: "<cancel>", origin: location.origin, targetMapId: Ye.targetMapId, sourceMapId: this.mapId };
                        this.target.postMessage(Cn);
                      }, { once: !0 });
                      const ln = [], pn = Object.assign(Object.assign({}, Ye), { id: zt, sourceMapId: this.mapId, origin: location.origin, data: hc(Ye.data, ln) });
                      this.target.postMessage(pn, { transfer: ln });
                    });
                  }
                  receive(Ye) {
                    const He = Ye.data, ot = He.id;
                    if (!(He.origin !== "file://" && location.origin !== "file://" && He.origin !== "resource://android" && location.origin !== "resource://android" && He.origin !== location.origin || He.targetMapId && this.mapId !== He.targetMapId)) {
                      if (He.type === "<cancel>") {
                        delete this.tasks[ot];
                        const xt = this.abortControllers[ot];
                        return delete this.abortControllers[ot], void (xt && xt.abort());
                      }
                      if (Me(self) || He.mustQueue) return this.tasks[ot] = He, this.taskQueue.push(ot), void this.invoker.trigger();
                      this.processTask(ot, He);
                    }
                  }
                  process() {
                    if (this.taskQueue.length === 0) return;
                    const Ye = this.taskQueue.shift(), He = this.tasks[Ye];
                    delete this.tasks[Ye], this.taskQueue.length > 0 && this.invoker.trigger(), He && this.processTask(Ye, He);
                  }
                  processTask(Ye, He) {
                    return ee(this, void 0, void 0, function* () {
                      if (He.type === "<response>") {
                        const zt = this.resolveRejects[Ye];
                        return delete this.resolveRejects[Ye], zt ? void (He.error ? zt.reject(Vc(He.error)) : zt.resolve(Vc(He.data))) : void 0;
                      }
                      if (!this.messageHandlers[He.type]) return void this.completeTask(Ye, new Error(`Could not find a registered handler for ${He.type}, map ID: ${this.mapId}, available handlers: ${Object.keys(this.messageHandlers).join(", ")}`));
                      const ot = Vc(He.data), xt = new AbortController();
                      this.abortControllers[Ye] = xt;
                      try {
                        const zt = yield this.messageHandlers[He.type](He.sourceMapId, ot, xt);
                        this.completeTask(Ye, null, zt);
                      } catch (zt) {
                        this.completeTask(Ye, zt);
                      }
                    });
                  }
                  completeTask(Ye, He, ot) {
                    const xt = [];
                    delete this.abortControllers[Ye];
                    const zt = { id: Ye, type: "<response>", sourceMapId: this.mapId, origin: location.origin, error: He ? hc(He) : null, data: hc(ot, xt) };
                    this.target.postMessage(zt, { transfer: xt });
                  }
                  remove() {
                    this.invoker.remove(), this.subscription.unsubscribe();
                  }
                }, ne.G = We, ne.H = function() {
                  var Ye = new cd(16);
                  return cd != Float32Array && (Ye[1] = 0, Ye[2] = 0, Ye[3] = 0, Ye[4] = 0, Ye[6] = 0, Ye[7] = 0, Ye[8] = 0, Ye[9] = 0, Ye[11] = 0, Ye[12] = 0, Ye[13] = 0, Ye[14] = 0), Ye[0] = 1, Ye[5] = 1, Ye[10] = 1, Ye[15] = 1, Ye;
                }, ne.I = r3, ne.J = function(Ye, He, ot) {
                  var xt, zt, ln, pn, Cn, Nn, Wn, lr, xr, Nr, Kr, xi, ui = ot[0], Ui = ot[1], ia = ot[2];
                  return He === Ye ? (Ye[12] = He[0] * ui + He[4] * Ui + He[8] * ia + He[12], Ye[13] = He[1] * ui + He[5] * Ui + He[9] * ia + He[13], Ye[14] = He[2] * ui + He[6] * Ui + He[10] * ia + He[14], Ye[15] = He[3] * ui + He[7] * Ui + He[11] * ia + He[15]) : (xt = He[0], zt = He[1], ln = He[2], pn = He[3], Cn = He[4], Nn = He[5], Wn = He[6], lr = He[7], xr = He[8], Nr = He[9], Kr = He[10], xi = He[11], Ye[0] = xt, Ye[1] = zt, Ye[2] = ln, Ye[3] = pn, Ye[4] = Cn, Ye[5] = Nn, Ye[6] = Wn, Ye[7] = lr, Ye[8] = xr, Ye[9] = Nr, Ye[10] = Kr, Ye[11] = xi, Ye[12] = xt * ui + Cn * Ui + xr * ia + He[12], Ye[13] = zt * ui + Nn * Ui + Nr * ia + He[13], Ye[14] = ln * ui + Wn * Ui + Kr * ia + He[14], Ye[15] = pn * ui + lr * Ui + xi * ia + He[15]), Ye;
                }, ne.K = function(Ye, He, ot) {
                  var xt = ot[0], zt = ot[1], ln = ot[2];
                  return Ye[0] = He[0] * xt, Ye[1] = He[1] * xt, Ye[2] = He[2] * xt, Ye[3] = He[3] * xt, Ye[4] = He[4] * zt, Ye[5] = He[5] * zt, Ye[6] = He[6] * zt, Ye[7] = He[7] * zt, Ye[8] = He[8] * ln, Ye[9] = He[9] * ln, Ye[10] = He[10] * ln, Ye[11] = He[11] * ln, Ye[12] = He[12], Ye[13] = He[13], Ye[14] = He[14], Ye[15] = He[15], Ye;
                }, ne.L = kh, ne.M = function(Ye, He) {
                  const ot = {};
                  for (let xt = 0; xt < He.length; xt++) {
                    const zt = He[xt];
                    zt in Ye && (ot[zt] = Ye[zt]);
                  }
                  return ot;
                }, ne.N = ff, ne.O = g2, ne.P = ce, ne.Q = oh, ne.R = pu, ne.S = hf, ne.T = a1, ne.U = me, ne.V = pe, ne.W = $e, ne.X = Uu, ne.Y = Fe, ne.Z = y2, ne._ = ee, ne.a = Ge, ne.a$ = function(Ye, He) {
                  var ot = Ye[0], xt = Ye[1], zt = Ye[2], ln = Ye[3], pn = Ye[4], Cn = Ye[5], Nn = Ye[6], Wn = Ye[7], lr = Ye[8], xr = Ye[9], Nr = Ye[10], Kr = Ye[11], xi = Ye[12], ui = Ye[13], Ui = Ye[14], ia = Ye[15], Na = He[0], Ta = He[1], Ra = He[2], oo = He[3], fs = He[4], Bs = He[5], fl = He[6], Ns = He[7], Es = He[8], cl = He[9], Xs = He[10], Ps = He[11], ko = He[12], Ks = He[13], el = He[14], Nl = He[15];
                  return Math.abs(ot - Na) <= p1 * Math.max(1, Math.abs(ot), Math.abs(Na)) && Math.abs(xt - Ta) <= p1 * Math.max(1, Math.abs(xt), Math.abs(Ta)) && Math.abs(zt - Ra) <= p1 * Math.max(1, Math.abs(zt), Math.abs(Ra)) && Math.abs(ln - oo) <= p1 * Math.max(1, Math.abs(ln), Math.abs(oo)) && Math.abs(pn - fs) <= p1 * Math.max(1, Math.abs(pn), Math.abs(fs)) && Math.abs(Cn - Bs) <= p1 * Math.max(1, Math.abs(Cn), Math.abs(Bs)) && Math.abs(Nn - fl) <= p1 * Math.max(1, Math.abs(Nn), Math.abs(fl)) && Math.abs(Wn - Ns) <= p1 * Math.max(1, Math.abs(Wn), Math.abs(Ns)) && Math.abs(lr - Es) <= p1 * Math.max(1, Math.abs(lr), Math.abs(Es)) && Math.abs(xr - cl) <= p1 * Math.max(1, Math.abs(xr), Math.abs(cl)) && Math.abs(Nr - Xs) <= p1 * Math.max(1, Math.abs(Nr), Math.abs(Xs)) && Math.abs(Kr - Ps) <= p1 * Math.max(1, Math.abs(Kr), Math.abs(Ps)) && Math.abs(xi - ko) <= p1 * Math.max(1, Math.abs(xi), Math.abs(ko)) && Math.abs(ui - Ks) <= p1 * Math.max(1, Math.abs(ui), Math.abs(Ks)) && Math.abs(Ui - el) <= p1 * Math.max(1, Math.abs(Ui), Math.abs(el)) && Math.abs(ia - Nl) <= p1 * Math.max(1, Math.abs(ia), Math.abs(Nl));
                }, ne.a0 = al, ne.a1 = Gf, ne.a2 = It, ne.a3 = (Ye) => {
                  const He = window.document.createElement("video");
                  return He.muted = !0, new Promise((ot) => {
                    He.onloadstart = () => {
                      ot(He);
                    };
                    for (const xt of Ye) {
                      const zt = window.document.createElement("source");
                      dt(xt) || (He.crossOrigin = "Anonymous"), zt.src = xt, He.appendChild(zt);
                    }
                  });
                }, ne.a4 = function() {
                  return Se++;
                }, ne.a5 = Yi, ne.a6 = gh, ne.a7 = Ql, ne.a8 = Sd, ne.a9 = Ko, ne.aA = function(Ye) {
                  Ye = Ye.slice();
                  const He = /* @__PURE__ */ Object.create(null);
                  for (let ot = 0; ot < Ye.length; ot++) He[Ye[ot].id] = Ye[ot];
                  for (let ot = 0; ot < Ye.length; ot++) "ref" in Ye[ot] && (Ye[ot] = Lt(Ye[ot], He[Ye[ot].ref]));
                  return Ye;
                }, ne.aB = function(Ye) {
                  if (Ye.type === "custom") return new F4(Ye);
                  switch (Ye.type) {
                    case "background":
                      return new N4(Ye);
                    case "circle":
                      return new Hf(Ye);
                    case "fill":
                      return new Vs(Ye);
                    case "fill-extrusion":
                      return new Ad(Ye);
                    case "heatmap":
                      return new lf(Ye);
                    case "hillshade":
                      return new Kf(Ye);
                    case "line":
                      return new Ea(Ye);
                    case "raster":
                      return new $4(Ye);
                    case "symbol":
                      return new O1(Ye);
                  }
                }, ne.aC = xe, ne.aD = function(Ye, He) {
                  if (!Ye) return [{ command: "setStyle", args: [He] }];
                  let ot = [];
                  try {
                    if (!Rt(Ye.version, He.version)) return [{ command: "setStyle", args: [He] }];
                    Rt(Ye.center, He.center) || ot.push({ command: "setCenter", args: [He.center] }), Rt(Ye.zoom, He.zoom) || ot.push({ command: "setZoom", args: [He.zoom] }), Rt(Ye.bearing, He.bearing) || ot.push({ command: "setBearing", args: [He.bearing] }), Rt(Ye.pitch, He.pitch) || ot.push({ command: "setPitch", args: [He.pitch] }), Rt(Ye.sprite, He.sprite) || ot.push({ command: "setSprite", args: [He.sprite] }), Rt(Ye.glyphs, He.glyphs) || ot.push({ command: "setGlyphs", args: [He.glyphs] }), Rt(Ye.transition, He.transition) || ot.push({ command: "setTransition", args: [He.transition] }), Rt(Ye.light, He.light) || ot.push({ command: "setLight", args: [He.light] }), Rt(Ye.terrain, He.terrain) || ot.push({ command: "setTerrain", args: [He.terrain] }), Rt(Ye.sky, He.sky) || ot.push({ command: "setSky", args: [He.sky] }), Rt(Ye.projection, He.projection) || ot.push({ command: "setProjection", args: [He.projection] });
                    const xt = {}, zt = [];
                    (function(pn, Cn, Nn, Wn) {
                      let lr;
                      for (lr in Cn = Cn || {}, pn = pn || {}) Object.prototype.hasOwnProperty.call(pn, lr) && (Object.prototype.hasOwnProperty.call(Cn, lr) || Pt(lr, Nn, Wn));
                      for (lr in Cn) Object.prototype.hasOwnProperty.call(Cn, lr) && (Object.prototype.hasOwnProperty.call(pn, lr) ? Rt(pn[lr], Cn[lr]) || (pn[lr].type === "geojson" && Cn[lr].type === "geojson" && Nt(pn, Cn, lr) ? vt(Nn, { command: "setGeoJSONSourceData", args: [lr, Cn[lr].data] }) : Xt(lr, Cn, Nn, Wn)) : $t(lr, Cn, Nn));
                    })(Ye.sources, He.sources, zt, xt);
                    const ln = [];
                    Ye.layers && Ye.layers.forEach((pn) => {
                      "source" in pn && xt[pn.source] ? ot.push({ command: "removeLayer", args: [pn.id] }) : ln.push(pn);
                    }), ot = ot.concat(zt), function(pn, Cn, Nn) {
                      Cn = Cn || [];
                      const Wn = (pn = pn || []).map(Ft), lr = Cn.map(Ft), xr = pn.reduce(Tt, {}), Nr = Cn.reduce(Tt, {}), Kr = Wn.slice(), xi = /* @__PURE__ */ Object.create(null);
                      let ui, Ui, ia, Na, Ta;
                      for (let Ra = 0, oo = 0; Ra < Wn.length; Ra++) ui = Wn[Ra], Object.prototype.hasOwnProperty.call(Nr, ui) ? oo++ : (vt(Nn, { command: "removeLayer", args: [ui] }), Kr.splice(Kr.indexOf(ui, oo), 1));
                      for (let Ra = 0, oo = 0; Ra < lr.length; Ra++) ui = lr[lr.length - 1 - Ra], Kr[Kr.length - 1 - Ra] !== ui && (Object.prototype.hasOwnProperty.call(xr, ui) ? (vt(Nn, { command: "removeLayer", args: [ui] }), Kr.splice(Kr.lastIndexOf(ui, Kr.length - oo), 1)) : oo++, Na = Kr[Kr.length - Ra], vt(Nn, { command: "addLayer", args: [Nr[ui], Na] }), Kr.splice(Kr.length - Ra, 0, ui), xi[ui] = !0);
                      for (let Ra = 0; Ra < lr.length; Ra++) if (ui = lr[Ra], Ui = xr[ui], ia = Nr[ui], !xi[ui] && !Rt(Ui, ia)) if (Rt(Ui.source, ia.source) && Rt(Ui["source-layer"], ia["source-layer"]) && Rt(Ui.type, ia.type)) {
                        for (Ta in Gt(Ui.layout, ia.layout, Nn, ui, null, "setLayoutProperty"), Gt(Ui.paint, ia.paint, Nn, ui, null, "setPaintProperty"), Rt(Ui.filter, ia.filter) || vt(Nn, { command: "setFilter", args: [ui, ia.filter] }), Rt(Ui.minzoom, ia.minzoom) && Rt(Ui.maxzoom, ia.maxzoom) || vt(Nn, { command: "setLayerZoomRange", args: [ui, ia.minzoom, ia.maxzoom] }), Ui) Object.prototype.hasOwnProperty.call(Ui, Ta) && Ta !== "layout" && Ta !== "paint" && Ta !== "filter" && Ta !== "metadata" && Ta !== "minzoom" && Ta !== "maxzoom" && (Ta.indexOf("paint.") === 0 ? Gt(Ui[Ta], ia[Ta], Nn, ui, Ta.slice(6), "setPaintProperty") : Rt(Ui[Ta], ia[Ta]) || vt(Nn, { command: "setLayerProperty", args: [ui, Ta, ia[Ta]] }));
                        for (Ta in ia) Object.prototype.hasOwnProperty.call(ia, Ta) && !Object.prototype.hasOwnProperty.call(Ui, Ta) && Ta !== "layout" && Ta !== "paint" && Ta !== "filter" && Ta !== "metadata" && Ta !== "minzoom" && Ta !== "maxzoom" && (Ta.indexOf("paint.") === 0 ? Gt(Ui[Ta], ia[Ta], Nn, ui, Ta.slice(6), "setPaintProperty") : Rt(Ui[Ta], ia[Ta]) || vt(Nn, { command: "setLayerProperty", args: [ui, Ta, ia[Ta]] }));
                      } else vt(Nn, { command: "removeLayer", args: [ui] }), Na = Kr[Kr.lastIndexOf(ui) + 1], vt(Nn, { command: "addLayer", args: [ia, Na] });
                    }(ln, He.layers, ot);
                  } catch (xt) {
                    console.warn("Unable to compute style diff:", xt), ot = [{ command: "setStyle", args: [He] }];
                  }
                  return ot;
                }, ne.aE = function(Ye) {
                  const He = [], ot = Ye.id;
                  return ot === void 0 && He.push({ message: `layers.${ot}: missing required property "id"` }), Ye.render === void 0 && He.push({ message: `layers.${ot}: missing required method "render"` }), Ye.renderingMode && Ye.renderingMode !== "2d" && Ye.renderingMode !== "3d" && He.push({ message: `layers.${ot}: property "renderingMode" must be either "2d" or "3d"` }), He;
                }, ne.aF = function Ye(He, ot) {
                  if (Array.isArray(He)) {
                    if (!Array.isArray(ot) || He.length !== ot.length) return !1;
                    for (let xt = 0; xt < He.length; xt++) if (!Ye(He[xt], ot[xt])) return !1;
                    return !0;
                  }
                  if (typeof He == "object" && He !== null && ot !== null) {
                    if (typeof ot != "object" || Object.keys(He).length !== Object.keys(ot).length) return !1;
                    for (const xt in He) if (!Ye(He[xt], ot[xt])) return !1;
                    return !0;
                  }
                  return He === ot;
                }, ne.aG = we, ne.aH = Ee, ne.aI = class extends od {
                  constructor(Ye, He) {
                    super(Ye, He), this.current = 0;
                  }
                  set(Ye) {
                    this.current !== Ye && (this.current = Ye, this.gl.uniform1i(this.location, Ye));
                  }
                }, ne.aJ = yd, ne.aK = class extends od {
                  constructor(Ye, He) {
                    super(Ye, He), this.current = nf;
                  }
                  set(Ye) {
                    if (Ye[12] !== this.current[12] || Ye[0] !== this.current[0]) return this.current = Ye, void this.gl.uniformMatrix4fv(this.location, !1, Ye);
                    for (let He = 1; He < 16; He++) if (Ye[He] !== this.current[He]) {
                      this.current = Ye, this.gl.uniformMatrix4fv(this.location, !1, Ye);
                      break;
                    }
                  }
                }, ne.aL = Ef, ne.aM = Md, ne.aN = mr, ne.aO = class extends od {
                  constructor(Ye, He) {
                    super(Ye, He), this.current = [0, 0, 0];
                  }
                  set(Ye) {
                    Ye[0] === this.current[0] && Ye[1] === this.current[1] && Ye[2] === this.current[2] || (this.current = Ye, this.gl.uniform3f(this.location, Ye[0], Ye[1], Ye[2]));
                  }
                }, ne.aP = class extends od {
                  constructor(Ye, He) {
                    super(Ye, He), this.current = [0, 0];
                  }
                  set(Ye) {
                    Ye[0] === this.current[0] && Ye[1] === this.current[1] || (this.current = Ye, this.gl.uniform2f(this.location, Ye[0], Ye[1]));
                  }
                }, ne.aQ = h2, ne.aR = Rh, ne.aS = class extends On {
                }, ne.aT = yo, ne.aU = class extends qn {
                }, ne.aV = function(Ye) {
                  return Ye <= 1 ? 1 : Math.pow(2, Math.ceil(Math.log(Ye) / Math.LN2));
                }, ne.aW = gf, ne.aX = Ja, ne.aY = Dl, ne.aZ = class extends ii {
                }, ne.a_ = function(Ye, He) {
                  return Ye[0] === He[0] && Ye[1] === He[1] && Ye[2] === He[2] && Ye[3] === He[3] && Ye[4] === He[4] && Ye[5] === He[5] && Ye[6] === He[6] && Ye[7] === He[7] && Ye[8] === He[8] && Ye[9] === He[9] && Ye[10] === He[10] && Ye[11] === He[11] && Ye[12] === He[12] && Ye[13] === He[13] && Ye[14] === He[14] && Ye[15] === He[15];
                }, ne.aa = f3, ne.ab = function(Ye) {
                  const He = {};
                  if (Ye.replace(/(?:^|(?:\s*\,\s*))([^\x00-\x20\(\)<>@\,;\:\\"\/\[\]\?\=\{\}\x7F]+)(?:\=(?:([^\x00-\x20\(\)<>@\,;\:\\"\/\[\]\?\=\{\}\x7F]+)|(?:\"((?:[^"\\]|\\.)*)\")))?/g, (ot, xt, zt, ln) => {
                    const pn = zt || ln;
                    return He[xt] = !pn || pn.toLowerCase(), "";
                  }), He["max-age"]) {
                    const ot = parseInt(He["max-age"], 10);
                    isNaN(ot) ? delete He["max-age"] : He["max-age"] = ot;
                  }
                  return He;
                }, ne.ac = function(Ye, He) {
                  const ot = [];
                  for (const xt in Ye) xt in He || ot.push(xt);
                  return ot;
                }, ne.ad = ve, ne.ae = function(Ye, He, ot) {
                  var xt = Math.sin(ot), zt = Math.cos(ot), ln = He[0], pn = He[1], Cn = He[2], Nn = He[3], Wn = He[4], lr = He[5], xr = He[6], Nr = He[7];
                  return He !== Ye && (Ye[8] = He[8], Ye[9] = He[9], Ye[10] = He[10], Ye[11] = He[11], Ye[12] = He[12], Ye[13] = He[13], Ye[14] = He[14], Ye[15] = He[15]), Ye[0] = ln * zt + Wn * xt, Ye[1] = pn * zt + lr * xt, Ye[2] = Cn * zt + xr * xt, Ye[3] = Nn * zt + Nr * xt, Ye[4] = Wn * zt - ln * xt, Ye[5] = lr * zt - pn * xt, Ye[6] = xr * zt - Cn * xt, Ye[7] = Nr * zt - Nn * xt, Ye;
                }, ne.af = function(Ye) {
                  var He = new cd(16);
                  return He[0] = Ye[0], He[1] = Ye[1], He[2] = Ye[2], He[3] = Ye[3], He[4] = Ye[4], He[5] = Ye[5], He[6] = Ye[6], He[7] = Ye[7], He[8] = Ye[8], He[9] = Ye[9], He[10] = Ye[10], He[11] = Ye[11], He[12] = Ye[12], He[13] = Ye[13], He[14] = Ye[14], He[15] = Ye[15], He;
                }, ne.ag = Zf, ne.ah = function(Ye, He) {
                  let ot = 0, xt = 0;
                  if (Ye.kind === "constant") xt = Ye.layoutSize;
                  else if (Ye.kind !== "source") {
                    const { interpolationType: zt, minZoom: ln, maxZoom: pn } = Ye, Cn = zt ? ve(Vr.interpolationFactor(zt, He, ln, pn), 0, 1) : 0;
                    Ye.kind === "camera" ? xt = Rr.number(Ye.minSize, Ye.maxSize, Cn) : ot = Cn;
                  }
                  return { uSizeT: ot, uSize: xt };
                }, ne.aj = function(Ye, { uSize: He, uSizeT: ot }, { lowerSize: xt, upperSize: zt }) {
                  return Ye.kind === "source" ? xt / ah : Ye.kind === "composite" ? Rr.number(xt / ah, zt / ah, ot) : He;
                }, ne.ak = ep, ne.al = function(Ye, He, ot, xt) {
                  const zt = He.y - Ye.y, ln = He.x - Ye.x, pn = xt.y - ot.y, Cn = xt.x - ot.x, Nn = pn * ln - Cn * zt;
                  if (Nn === 0) return null;
                  const Wn = (Cn * (Ye.y - ot.y) - pn * (Ye.x - ot.x)) / Nn;
                  return new ce(Ye.x + Wn * ln, Ye.y + Wn * zt);
                }, ne.am = Jt, ne.an = of, ne.ao = dh, ne.ap = function(Ye) {
                  let He = 1 / 0, ot = 1 / 0, xt = -1 / 0, zt = -1 / 0;
                  for (const ln of Ye) He = Math.min(He, ln.x), ot = Math.min(ot, ln.y), xt = Math.max(xt, ln.x), zt = Math.max(zt, ln.y);
                  return [He, ot, xt, zt];
                }, ne.aq = Us, ne.as = a3, ne.at = function(Ye, He) {
                  var ot = He[0], xt = He[1], zt = He[2], ln = He[3], pn = He[4], Cn = He[5], Nn = He[6], Wn = He[7], lr = He[8], xr = He[9], Nr = He[10], Kr = He[11], xi = He[12], ui = He[13], Ui = He[14], ia = He[15], Na = ot * Cn - xt * pn, Ta = ot * Nn - zt * pn, Ra = ot * Wn - ln * pn, oo = xt * Nn - zt * Cn, fs = xt * Wn - ln * Cn, Bs = zt * Wn - ln * Nn, fl = lr * ui - xr * xi, Ns = lr * Ui - Nr * xi, Es = lr * ia - Kr * xi, cl = xr * Ui - Nr * ui, Xs = xr * ia - Kr * ui, Ps = Nr * ia - Kr * Ui, ko = Na * Ps - Ta * Xs + Ra * cl + oo * Es - fs * Ns + Bs * fl;
                  return ko ? (ko = 1 / ko, Ye[0] = (Cn * Ps - Nn * Xs + Wn * cl) * ko, Ye[1] = (zt * Xs - xt * Ps - ln * cl) * ko, Ye[2] = (ui * Bs - Ui * fs + ia * oo) * ko, Ye[3] = (Nr * fs - xr * Bs - Kr * oo) * ko, Ye[4] = (Nn * Es - pn * Ps - Wn * Ns) * ko, Ye[5] = (ot * Ps - zt * Es + ln * Ns) * ko, Ye[6] = (Ui * Ra - xi * Bs - ia * Ta) * ko, Ye[7] = (lr * Bs - Nr * Ra + Kr * Ta) * ko, Ye[8] = (pn * Xs - Cn * Es + Wn * fl) * ko, Ye[9] = (xt * Es - ot * Xs - ln * fl) * ko, Ye[10] = (xi * fs - ui * Ra + ia * Na) * ko, Ye[11] = (xr * Ra - lr * fs - Kr * Na) * ko, Ye[12] = (Cn * Ns - pn * cl - Nn * fl) * ko, Ye[13] = (ot * cl - xt * Ns + zt * fl) * ko, Ye[14] = (ui * Ta - xi * oo - Ui * Na) * ko, Ye[15] = (lr * oo - xr * Ta + Nr * Na) * ko, Ye) : null;
                }, ne.au = Rl, ne.av = Fh, ne.aw = Cl, ne.ax = function() {
                  const Ye = {}, He = yt.$version;
                  for (const ot in yt.$root) {
                    const xt = yt.$root[ot];
                    if (xt.required) {
                      let zt = null;
                      zt = ot === "version" ? He : xt.type === "array" ? [] : {}, zt != null && (Ye[ot] = zt);
                    }
                  }
                  return Ye;
                }, ne.ay = Rc, ne.az = ct, ne.b = De, ne.b0 = function(Ye, He) {
                  return Ye[0] = He[0], Ye[1] = He[1], Ye[2] = He[2], Ye[3] = He[3], Ye[4] = He[4], Ye[5] = He[5], Ye[6] = He[6], Ye[7] = He[7], Ye[8] = He[8], Ye[9] = He[9], Ye[10] = He[10], Ye[11] = He[11], Ye[12] = He[12], Ye[13] = He[13], Ye[14] = He[14], Ye[15] = He[15], Ye;
                }, ne.b1 = ed, ne.b2 = function(Ye, He) {
                  return Ye[0] * He[0] + Ye[1] * He[1] + Ye[2] * He[2] + Ye[3] * He[3];
                }, ne.b3 = be, ne.b4 = d3, ne.b5 = V2, ne.b6 = Mh, ne.b7 = function(Ye, He, ot) {
                  var xt = Math.sin(ot), zt = Math.cos(ot), ln = He[4], pn = He[5], Cn = He[6], Nn = He[7], Wn = He[8], lr = He[9], xr = He[10], Nr = He[11];
                  return He !== Ye && (Ye[0] = He[0], Ye[1] = He[1], Ye[2] = He[2], Ye[3] = He[3], Ye[12] = He[12], Ye[13] = He[13], Ye[14] = He[14], Ye[15] = He[15]), Ye[4] = ln * zt + Wn * xt, Ye[5] = pn * zt + lr * xt, Ye[6] = Cn * zt + xr * xt, Ye[7] = Nn * zt + Nr * xt, Ye[8] = Wn * zt - ln * xt, Ye[9] = lr * zt - pn * xt, Ye[10] = xr * zt - Cn * xt, Ye[11] = Nr * zt - Nn * xt, Ye;
                }, ne.b8 = ye, ne.b9 = ge, ne.bA = function(Ye) {
                  return Ye.message === Ve;
                }, ne.bB = Xo, ne.bC = nc, ne.ba = function(Ye) {
                  return Ye * Math.PI / 180;
                }, ne.bb = function(Ye, He) {
                  return Ye[0] = He[0], Ye[1] = 0, Ye[2] = 0, Ye[3] = 0, Ye[4] = 0, Ye[5] = He[1], Ye[6] = 0, Ye[7] = 0, Ye[8] = 0, Ye[9] = 0, Ye[10] = He[2], Ye[11] = 0, Ye[12] = 0, Ye[13] = 0, Ye[14] = 0, Ye[15] = 1, Ye;
                }, ne.bc = class extends Oe {
                }, ne.bd = vh, ne.be = Qd, ne.bg = at, ne.bh = function(Ye, He) {
                  Ge.REGISTERED_PROTOCOLS[Ye] = He;
                }, ne.bi = function(Ye) {
                  delete Ge.REGISTERED_PROTOCOLS[Ye];
                }, ne.bj = function(Ye, He) {
                  const ot = {};
                  for (let zt = 0; zt < Ye.length; zt++) {
                    const ln = He && He[Ye[zt].id] || T1(Ye[zt]);
                    He && (He[Ye[zt].id] = ln);
                    let pn = ot[ln];
                    pn || (pn = ot[ln] = []), pn.push(Ye[zt]);
                  }
                  const xt = [];
                  for (const zt in ot) xt.push(ot[zt]);
                  return xt;
                }, ne.bk = ss, ne.bl = bh, ne.bm = Ne, ne.bn = Zh, ne.bo = function(Ye) {
                  Ye.bucket.createArrays();
                  const He = 512 * Ye.bucket.overscaling;
                  Ye.bucket.tilePixelRatio = Uu / He, Ye.bucket.compareText = {}, Ye.bucket.iconsNeedLinear = !1;
                  const ot = Ye.bucket.layers[0], xt = ot.layout, zt = ot._unevaluatedLayout._values, ln = { layoutIconSize: zt["icon-size"].possiblyEvaluate(new Ko(Ye.bucket.zoom + 1), Ye.canonical), layoutTextSize: zt["text-size"].possiblyEvaluate(new Ko(Ye.bucket.zoom + 1), Ye.canonical), textMaxSize: zt["text-size"].possiblyEvaluate(new Ko(18)) };
                  if (Ye.bucket.textSizeData.kind === "composite") {
                    const { minZoom: lr, maxZoom: xr } = Ye.bucket.textSizeData;
                    ln.compositeTextSizes = [zt["text-size"].possiblyEvaluate(new Ko(lr), Ye.canonical), zt["text-size"].possiblyEvaluate(new Ko(xr), Ye.canonical)];
                  }
                  if (Ye.bucket.iconSizeData.kind === "composite") {
                    const { minZoom: lr, maxZoom: xr } = Ye.bucket.iconSizeData;
                    ln.compositeIconSizes = [zt["icon-size"].possiblyEvaluate(new Ko(lr), Ye.canonical), zt["icon-size"].possiblyEvaluate(new Ko(xr), Ye.canonical)];
                  }
                  const pn = xt.get("text-line-height") * Us, Cn = xt.get("text-rotation-alignment") !== "viewport" && xt.get("symbol-placement") !== "point", Nn = xt.get("text-keep-upright"), Wn = xt.get("text-size");
                  for (const lr of Ye.bucket.features) {
                    const xr = xt.get("text-font").evaluate(lr, {}, Ye.canonical).join(","), Nr = Wn.evaluate(lr, {}, Ye.canonical), Kr = ln.layoutTextSize.evaluate(lr, {}, Ye.canonical), xi = ln.layoutIconSize.evaluate(lr, {}, Ye.canonical), ui = { horizontal: {}, vertical: void 0 }, Ui = lr.text;
                    let ia, Na = [0, 0];
                    if (Ui) {
                      const oo = Ui.toString(), fs = xt.get("text-letter-spacing").evaluate(lr, {}, Ye.canonical) * Us, Bs = Uc(oo) ? fs : 0, fl = xt.get("text-anchor").evaluate(lr, {}, Ye.canonical), Ns = rl(ot, lr, Ye.canonical);
                      if (!Ns) {
                        const Ps = xt.get("text-radial-offset").evaluate(lr, {}, Ye.canonical);
                        Na = Ps ? Pl(fl, [Ps * Us, Fs]) : xt.get("text-offset").evaluate(lr, {}, Ye.canonical).map((ko) => ko * Us);
                      }
                      let Es = Cn ? "center" : xt.get("text-justify").evaluate(lr, {}, Ye.canonical);
                      const cl = xt.get("symbol-placement") === "point" ? xt.get("text-max-width").evaluate(lr, {}, Ye.canonical) * Us : 1 / 0, Xs = () => {
                        Ye.bucket.allowVerticalPlacement && Ou(oo) && (ui.vertical = $h(Ui, Ye.glyphMap, Ye.glyphPositions, Ye.imagePositions, xr, cl, pn, fl, "left", Bs, Na, ne.ai.vertical, !0, Kr, Nr));
                      };
                      if (!Cn && Ns) {
                        const Ps = /* @__PURE__ */ new Set();
                        if (Es === "auto") for (let Ks = 0; Ks < Ns.values.length; Ks += 2) Ps.add(Rl(Ns.values[Ks]));
                        else Ps.add(Es);
                        let ko = !1;
                        for (const Ks of Ps) if (!ui.horizontal[Ks]) if (ko) ui.horizontal[Ks] = ui.horizontal[0];
                        else {
                          const el = $h(Ui, Ye.glyphMap, Ye.glyphPositions, Ye.imagePositions, xr, cl, pn, "center", Ks, Bs, Na, ne.ai.horizontal, !1, Kr, Nr);
                          el && (ui.horizontal[Ks] = el, ko = el.positionedLines.length === 1);
                        }
                        Xs();
                      } else {
                        Es === "auto" && (Es = Rl(fl));
                        const Ps = $h(Ui, Ye.glyphMap, Ye.glyphPositions, Ye.imagePositions, xr, cl, pn, fl, Es, Bs, Na, ne.ai.horizontal, !1, Kr, Nr);
                        Ps && (ui.horizontal[Es] = Ps), Xs(), Ou(oo) && Cn && Nn && (ui.vertical = $h(Ui, Ye.glyphMap, Ye.glyphPositions, Ye.imagePositions, xr, cl, pn, fl, Es, Bs, Na, ne.ai.vertical, !1, Kr, Nr));
                      }
                    }
                    let Ta = !1;
                    if (lr.icon && lr.icon.name) {
                      const oo = Ye.imageMap[lr.icon.name];
                      oo && (ia = Du(Ye.imagePositions[lr.icon.name], xt.get("icon-offset").evaluate(lr, {}, Ye.canonical), xt.get("icon-anchor").evaluate(lr, {}, Ye.canonical)), Ta = !!oo.sdf, Ye.bucket.sdfIcons === void 0 ? Ye.bucket.sdfIcons = Ta : Ye.bucket.sdfIcons !== Ta && Qe("Style sheet warning: Cannot mix SDF and non-SDF icons in one buffer"), (oo.pixelRatio !== Ye.bucket.pixelRatio || xt.get("icon-rotate").constantOr(1) !== 0) && (Ye.bucket.iconsNeedLinear = !0));
                    }
                    const Ra = Hc(ui.horizontal) || ui.vertical;
                    Ye.bucket.iconsInText = !!Ra && Ra.iconsInText, (Ra || ia) && pl(Ye.bucket, lr, ui, ia, Ye.imageMap, ln, Kr, xi, Na, Ta, Ye.canonical);
                  }
                  Ye.showCollisionBoxes && Ye.bucket.generateCollisionDebugBuffers();
                }, ne.bp = yi, ne.bq = Ss, ne.br = r1, ne.bs = Jo, ne.bt = qf, ne.bu = class {
                  constructor(Ye) {
                    this._marks = { start: [Ye.url, "start"].join("#"), end: [Ye.url, "end"].join("#"), measure: Ye.url.toString() }, performance.mark(this._marks.start);
                  }
                  finish() {
                    performance.mark(this._marks.end);
                    let Ye = performance.getEntriesByName(this._marks.measure);
                    return Ye.length === 0 && (performance.measure(this._marks.measure, this._marks.start, this._marks.end), Ye = performance.getEntriesByName(this._marks.measure), performance.clearMarks(this._marks.start), performance.clearMarks(this._marks.end), performance.clearMeasures(this._marks.measure)), Ye;
                  }
                }, ne.bv = function(Ye, He, ot, xt, zt) {
                  return ee(this, void 0, void 0, function* () {
                    if (pe()) try {
                      return yield $e(Ye, He, ot, xt, zt);
                    } catch {
                    }
                    return function(ln, pn, Cn, Nn, Wn) {
                      const lr = ln.width, xr = ln.height;
                      qe && Xe || (qe = new OffscreenCanvas(lr, xr), Xe = qe.getContext("2d", { willReadFrequently: !0 })), qe.width = lr, qe.height = xr, Xe.drawImage(ln, 0, 0, lr, xr);
                      const Nr = Xe.getImageData(pn, Cn, Nn, Wn);
                      return Xe.clearRect(0, 0, lr, xr), Nr.data;
                    }(Ye, He, ot, xt, zt);
                  });
                }, ne.bw = fd, ne.bx = ae, ne.by = oe, ne.bz = sl, ne.c = Ze, ne.d = (Ye) => ee(void 0, void 0, void 0, function* () {
                  if (Ye.byteLength === 0) return createImageBitmap(new ImageData(1, 1));
                  const He = new Blob([new Uint8Array(Ye)], { type: "image/png" });
                  try {
                    return createImageBitmap(He);
                  } catch (ot) {
                    throw new Error(`Could not load image because of ${ot.message}. Please make sure to use a supported image type such as PNG or JPEG. Note that SVGs are not supported.`);
                  }
                }), ne.e = Ce, ne.f = (Ye) => new Promise((He, ot) => {
                  const xt = new Image();
                  xt.onload = () => {
                    He(xt), URL.revokeObjectURL(xt.src), xt.onload = null, window.requestAnimationFrame(() => {
                      xt.src = Pe;
                    });
                  }, xt.onerror = () => ot(new Error("Could not load image. Please make sure to use a supported image type such as PNG or JPEG. Note that SVGs are not supported."));
                  const zt = new Blob([new Uint8Array(Ye)], { type: "image/png" });
                  xt.src = Ye.byteLength ? URL.createObjectURL(zt) : Pe;
                }), ne.g = ze, ne.h = (Ye, He) => et(Ce(Ye, { type: "json" }), He), ne.i = Me, ne.j = gt, ne.k = st, ne.l = (Ye, He) => et(Ce(Ye, { type: "arrayBuffer" }), He), ne.m = et, ne.n = function(Ye) {
                  return new qf(Ye).readFields(U0, []);
                }, ne.o = cu, ne.p = C4, ne.q = tu, ne.r = Eu, ne.s = dt, ne.t = kc, ne.u = Zo, ne.v = yt, ne.w = Qe, ne.x = vl, ne.y = function([Ye, He, ot]) {
                  return He += 90, He *= Math.PI / 180, ot *= Math.PI / 180, { x: Ye * Math.cos(He) * Math.sin(ot), y: Ye * Math.sin(He) * Math.sin(ot), z: Ye * Math.cos(ot) };
                }, ne.z = Rr;
              }), te("worker", 0, function(ne) {
                class ee {
                  constructor(rt) {
                    this.keyCache = {}, rt && this.replace(rt);
                  }
                  replace(rt) {
                    this._layerConfigs = {}, this._layers = {}, this.update(rt, []);
                  }
                  update(rt, ft) {
                    for (const wt of rt) {
                      this._layerConfigs[wt.id] = wt;
                      const Bt = this._layers[wt.id] = ne.aB(wt);
                      Bt._featureFilter = ne.a7(Bt.filter), this.keyCache[wt.id] && delete this.keyCache[wt.id];
                    }
                    for (const wt of ft) delete this.keyCache[wt], delete this._layerConfigs[wt], delete this._layers[wt];
                    this.familiesBySource = {};
                    const pt = ne.bj(Object.values(this._layerConfigs), this.keyCache);
                    for (const wt of pt) {
                      const Bt = wt.map((_n) => this._layers[_n.id]), At = Bt[0];
                      if (At.visibility === "none") continue;
                      const Dt = At.source || "";
                      let Ot = this.familiesBySource[Dt];
                      Ot || (Ot = this.familiesBySource[Dt] = {});
                      const un = At.sourceLayer || "_geojsonTileLayer";
                      let yn = Ot[un];
                      yn || (yn = Ot[un] = []), yn.push(Bt);
                    }
                  }
                }
                class ae {
                  constructor(rt) {
                    const ft = {}, pt = [];
                    for (const Dt in rt) {
                      const Ot = rt[Dt], un = ft[Dt] = {};
                      for (const yn in Ot) {
                        const _n = Ot[+yn];
                        if (!_n || _n.bitmap.width === 0 || _n.bitmap.height === 0) continue;
                        const gn = { x: 0, y: 0, w: _n.bitmap.width + 2, h: _n.bitmap.height + 2 };
                        pt.push(gn), un[yn] = { rect: gn, metrics: _n.metrics };
                      }
                    }
                    const { w: wt, h: Bt } = ne.p(pt), At = new ne.o({ width: wt || 1, height: Bt || 1 });
                    for (const Dt in rt) {
                      const Ot = rt[Dt];
                      for (const un in Ot) {
                        const yn = Ot[+un];
                        if (!yn || yn.bitmap.width === 0 || yn.bitmap.height === 0) continue;
                        const _n = ft[Dt][un].rect;
                        ne.o.copy(yn.bitmap, At, { x: 0, y: 0 }, { x: _n.x + 1, y: _n.y + 1 }, yn.bitmap);
                      }
                    }
                    this.image = At, this.positions = ft;
                  }
                }
                ne.bk("GlyphAtlas", ae);
                class oe {
                  constructor(rt) {
                    this.tileID = new ne.S(rt.tileID.overscaledZ, rt.tileID.wrap, rt.tileID.canonical.z, rt.tileID.canonical.x, rt.tileID.canonical.y), this.uid = rt.uid, this.zoom = rt.zoom, this.pixelRatio = rt.pixelRatio, this.tileSize = rt.tileSize, this.source = rt.source, this.overscaling = this.tileID.overscaleFactor(), this.showCollisionBoxes = rt.showCollisionBoxes, this.collectResourceTiming = !!rt.collectResourceTiming, this.returnDependencies = !!rt.returnDependencies, this.promoteId = rt.promoteId, this.inFlightDependencies = [];
                  }
                  parse(rt, ft, pt, wt) {
                    return ne._(this, void 0, void 0, function* () {
                      this.status = "parsing", this.data = rt, this.collisionBoxArray = new ne.a5();
                      const Bt = new ne.bl(Object.keys(rt.layers).sort()), At = new ne.bm(this.tileID, this.promoteId);
                      At.bucketLayerIDs = [];
                      const Dt = {}, Ot = { featureIndex: At, iconDependencies: {}, patternDependencies: {}, glyphDependencies: {}, availableImages: pt }, un = ft.familiesBySource[this.source];
                      for (const jn in un) {
                        const ur = rt.layers[jn];
                        if (!ur) continue;
                        ur.version === 1 && ne.w(`Vector tile source "${this.source}" layer "${jn}" does not use vector tile spec v2 and therefore may have some rendering errors.`);
                        const Kn = Bt.encode(jn), Jn = [];
                        for (let Lr = 0; Lr < ur.length; Lr++) {
                          const mr = ur.feature(Lr), Ar = At.getId(mr, jn);
                          Jn.push({ feature: mr, id: Ar, index: Lr, sourceLayerIndex: Kn });
                        }
                        for (const Lr of un[jn]) {
                          const mr = Lr[0];
                          mr.source !== this.source && ne.w(`layer.source = ${mr.source} does not equal this.source = ${this.source}`), mr.minzoom && this.zoom < Math.floor(mr.minzoom) || mr.maxzoom && this.zoom >= mr.maxzoom || mr.visibility !== "none" && (se(Lr, this.zoom, pt), (Dt[mr.id] = mr.createBucket({ index: At.bucketLayerIDs.length, layers: Lr, zoom: this.zoom, pixelRatio: this.pixelRatio, overscaling: this.overscaling, collisionBoxArray: this.collisionBoxArray, sourceLayerIndex: Kn, sourceID: this.source })).populate(Jn, Ot, this.tileID.canonical), At.bucketLayerIDs.push(Lr.map((Ar) => Ar.id)));
                        }
                      }
                      const yn = ne.aG(Ot.glyphDependencies, (jn) => Object.keys(jn).map(Number));
                      this.inFlightDependencies.forEach((jn) => jn == null ? void 0 : jn.abort()), this.inFlightDependencies = [];
                      let _n = Promise.resolve({});
                      if (Object.keys(yn).length) {
                        const jn = new AbortController();
                        this.inFlightDependencies.push(jn), _n = wt.sendAsync({ type: "GG", data: { stacks: yn, source: this.source, tileID: this.tileID, type: "glyphs" } }, jn);
                      }
                      const gn = Object.keys(Ot.iconDependencies);
                      let bn = Promise.resolve({});
                      if (gn.length) {
                        const jn = new AbortController();
                        this.inFlightDependencies.push(jn), bn = wt.sendAsync({ type: "GI", data: { icons: gn, source: this.source, tileID: this.tileID, type: "icons" } }, jn);
                      }
                      const an = Object.keys(Ot.patternDependencies);
                      let Zt = Promise.resolve({});
                      if (an.length) {
                        const jn = new AbortController();
                        this.inFlightDependencies.push(jn), Zt = wt.sendAsync({ type: "GI", data: { icons: an, source: this.source, tileID: this.tileID, type: "patterns" } }, jn);
                      }
                      const [Mn, Ln, In] = yield Promise.all([_n, bn, Zt]), kn = new ae(Mn), er = new ne.bn(Ln, In);
                      for (const jn in Dt) {
                        const ur = Dt[jn];
                        ur instanceof ne.a6 ? (se(ur.layers, this.zoom, pt), ne.bo({ bucket: ur, glyphMap: Mn, glyphPositions: kn.positions, imageMap: Ln, imagePositions: er.iconPositions, showCollisionBoxes: this.showCollisionBoxes, canonical: this.tileID.canonical })) : ur.hasPattern && (ur instanceof ne.bp || ur instanceof ne.bq || ur instanceof ne.br) && (se(ur.layers, this.zoom, pt), ur.addFeatures(Ot, this.tileID.canonical, er.patternPositions));
                      }
                      return this.status = "done", { buckets: Object.values(Dt).filter((jn) => !jn.isEmpty()), featureIndex: At, collisionBoxArray: this.collisionBoxArray, glyphAtlasImage: kn.image, imageAtlas: er, glyphMap: this.returnDependencies ? Mn : null, iconMap: this.returnDependencies ? Ln : null, glyphPositions: this.returnDependencies ? kn.positions : null };
                    });
                  }
                }
                function se(Ie, rt, ft) {
                  const pt = new ne.a9(rt);
                  for (const wt of Ie) wt.recalculate(pt, ft);
                }
                class ce {
                  constructor(rt, ft, pt) {
                    this.actor = rt, this.layerIndex = ft, this.availableImages = pt, this.fetching = {}, this.loading = {}, this.loaded = {};
                  }
                  loadVectorTile(rt, ft) {
                    return ne._(this, void 0, void 0, function* () {
                      const pt = yield ne.l(rt.request, ft);
                      try {
                        return { vectorTile: new ne.bs.VectorTile(new ne.bt(pt.data)), rawData: pt.data, cacheControl: pt.cacheControl, expires: pt.expires };
                      } catch (wt) {
                        const Bt = new Uint8Array(pt.data), At = Bt[0] === 31 && Bt[1] === 139;
                        let Dt = `Unable to parse the tile at ${rt.request.url}, `;
                        throw Dt += At ? "please make sure the data is not gzipped and that you have configured the relevant header in the server" : `got error: ${wt.message}`, new Error(Dt);
                      }
                    });
                  }
                  loadTile(rt) {
                    return ne._(this, void 0, void 0, function* () {
                      const ft = rt.uid, pt = !!(rt && rt.request && rt.request.collectResourceTiming) && new ne.bu(rt.request), wt = new oe(rt);
                      this.loading[ft] = wt;
                      const Bt = new AbortController();
                      wt.abort = Bt;
                      try {
                        const At = yield this.loadVectorTile(rt, Bt);
                        if (delete this.loading[ft], !At) return null;
                        const Dt = At.rawData, Ot = {};
                        At.expires && (Ot.expires = At.expires), At.cacheControl && (Ot.cacheControl = At.cacheControl);
                        const un = {};
                        if (pt) {
                          const _n = pt.finish();
                          _n && (un.resourceTiming = JSON.parse(JSON.stringify(_n)));
                        }
                        wt.vectorTile = At.vectorTile;
                        const yn = wt.parse(At.vectorTile, this.layerIndex, this.availableImages, this.actor);
                        this.loaded[ft] = wt, this.fetching[ft] = { rawTileData: Dt, cacheControl: Ot, resourceTiming: un };
                        try {
                          const _n = yield yn;
                          return ne.e({ rawTileData: Dt.slice(0) }, _n, Ot, un);
                        } finally {
                          delete this.fetching[ft];
                        }
                      } catch (At) {
                        throw delete this.loading[ft], wt.status = "done", this.loaded[ft] = wt, At;
                      }
                    });
                  }
                  reloadTile(rt) {
                    return ne._(this, void 0, void 0, function* () {
                      const ft = rt.uid;
                      if (!this.loaded || !this.loaded[ft]) throw new Error("Should not be trying to reload a tile that was never loaded or has been removed");
                      const pt = this.loaded[ft];
                      if (pt.showCollisionBoxes = rt.showCollisionBoxes, pt.status === "parsing") {
                        const wt = yield pt.parse(pt.vectorTile, this.layerIndex, this.availableImages, this.actor);
                        let Bt;
                        if (this.fetching[ft]) {
                          const { rawTileData: At, cacheControl: Dt, resourceTiming: Ot } = this.fetching[ft];
                          delete this.fetching[ft], Bt = ne.e({ rawTileData: At.slice(0) }, wt, Dt, Ot);
                        } else Bt = wt;
                        return Bt;
                      }
                      if (pt.status === "done" && pt.vectorTile) return pt.parse(pt.vectorTile, this.layerIndex, this.availableImages, this.actor);
                    });
                  }
                  abortTile(rt) {
                    return ne._(this, void 0, void 0, function* () {
                      const ft = this.loading, pt = rt.uid;
                      ft && ft[pt] && ft[pt].abort && (ft[pt].abort.abort(), delete ft[pt]);
                    });
                  }
                  removeTile(rt) {
                    return ne._(this, void 0, void 0, function* () {
                      this.loaded && this.loaded[rt.uid] && delete this.loaded[rt.uid];
                    });
                  }
                }
                class le {
                  constructor() {
                    this.loaded = {};
                  }
                  loadTile(rt) {
                    return ne._(this, void 0, void 0, function* () {
                      const { uid: ft, encoding: pt, rawImageData: wt, redFactor: Bt, greenFactor: At, blueFactor: Dt, baseShift: Ot } = rt, un = wt.width + 2, yn = wt.height + 2, _n = ne.b(wt) ? new ne.R({ width: un, height: yn }, yield ne.bv(wt, -1, -1, un, yn)) : wt, gn = new ne.bw(ft, _n, pt, Bt, At, Dt, Ot);
                      return this.loaded = this.loaded || {}, this.loaded[ft] = gn, gn;
                    });
                  }
                  removeTile(rt) {
                    const ft = this.loaded, pt = rt.uid;
                    ft && ft[pt] && delete ft[pt];
                  }
                }
                var ue = function Ie(rt, ft) {
                  var pt, wt = rt && rt.type;
                  if (wt === "FeatureCollection") for (pt = 0; pt < rt.features.length; pt++) Ie(rt.features[pt], ft);
                  else if (wt === "GeometryCollection") for (pt = 0; pt < rt.geometries.length; pt++) Ie(rt.geometries[pt], ft);
                  else if (wt === "Feature") Ie(rt.geometry, ft);
                  else if (wt === "Polygon") de(rt.coordinates, ft);
                  else if (wt === "MultiPolygon") for (pt = 0; pt < rt.coordinates.length; pt++) de(rt.coordinates[pt], ft);
                  return rt;
                };
                function de(Ie, rt) {
                  if (Ie.length !== 0) {
                    fe(Ie[0], rt);
                    for (var ft = 1; ft < Ie.length; ft++) fe(Ie[ft], !rt);
                  }
                }
                function fe(Ie, rt) {
                  for (var ft = 0, pt = 0, wt = 0, Bt = Ie.length, At = Bt - 1; wt < Bt; At = wt++) {
                    var Dt = (Ie[wt][0] - Ie[At][0]) * (Ie[At][1] + Ie[wt][1]), Ot = ft + Dt;
                    pt += Math.abs(ft) >= Math.abs(Dt) ? ft - Ot + Dt : Dt - Ot + ft, ft = Ot;
                  }
                  ft + pt >= 0 != !!rt && Ie.reverse();
                }
                var he = ne.bx(ue);
                const me = ne.bs.VectorTileFeature.prototype.toGeoJSON;
                let pe = class {
                  constructor(Ie) {
                    this._feature = Ie, this.extent = ne.X, this.type = Ie.type, this.properties = Ie.tags, "id" in Ie && !isNaN(Ie.id) && (this.id = parseInt(Ie.id, 10));
                  }
                  loadGeometry() {
                    if (this._feature.type === 1) {
                      const Ie = [];
                      for (const rt of this._feature.geometry) Ie.push([new ne.P(rt[0], rt[1])]);
                      return Ie;
                    }
                    {
                      const Ie = [];
                      for (const rt of this._feature.geometry) {
                        const ft = [];
                        for (const pt of rt) ft.push(new ne.P(pt[0], pt[1]));
                        Ie.push(ft);
                      }
                      return Ie;
                    }
                  }
                  toGeoJSON(Ie, rt, ft) {
                    return me.call(this, Ie, rt, ft);
                  }
                }, ye = class {
                  constructor(Ie) {
                    this.layers = { _geojsonTileLayer: this }, this.name = "_geojsonTileLayer", this.extent = ne.X, this.length = Ie.length, this._features = Ie;
                  }
                  feature(Ie) {
                    return new pe(this._features[Ie]);
                  }
                };
                var ge = { exports: {} }, ve = ne.by, be = ne.bs.VectorTileFeature, Ce = Se;
                function Se(Ie, rt) {
                  this.options = rt || {}, this.features = Ie, this.length = Ie.length;
                }
                function we(Ie, rt) {
                  this.id = typeof Ie.id == "number" ? Ie.id : void 0, this.type = Ie.type, this.rawGeometry = Ie.type === 1 ? [Ie.geometry] : Ie.geometry, this.properties = Ie.tags, this.extent = rt || 4096;
                }
                Se.prototype.feature = function(Ie) {
                  return new we(this.features[Ie], this.options.extent);
                }, we.prototype.loadGeometry = function() {
                  var Ie = this.rawGeometry;
                  this.geometry = [];
                  for (var rt = 0; rt < Ie.length; rt++) {
                    for (var ft = Ie[rt], pt = [], wt = 0; wt < ft.length; wt++) pt.push(new ve(ft[wt][0], ft[wt][1]));
                    this.geometry.push(pt);
                  }
                  return this.geometry;
                }, we.prototype.bbox = function() {
                  this.geometry || this.loadGeometry();
                  for (var Ie = this.geometry, rt = 1 / 0, ft = -1 / 0, pt = 1 / 0, wt = -1 / 0, Bt = 0; Bt < Ie.length; Bt++) for (var At = Ie[Bt], Dt = 0; Dt < At.length; Dt++) {
                    var Ot = At[Dt];
                    rt = Math.min(rt, Ot.x), ft = Math.max(ft, Ot.x), pt = Math.min(pt, Ot.y), wt = Math.max(wt, Ot.y);
                  }
                  return [rt, pt, ft, wt];
                }, we.prototype.toGeoJSON = be.prototype.toGeoJSON;
                var Ee = ne.bz, xe = Ce;
                function Le(Ie) {
                  var rt = new Ee();
                  return function(ft, pt) {
                    for (var wt in ft.layers) pt.writeMessage(3, Qe, ft.layers[wt]);
                  }(Ie, rt), rt.finish();
                }
                function Qe(Ie, rt) {
                  var ft;
                  rt.writeVarintField(15, Ie.version || 1), rt.writeStringField(1, Ie.name || ""), rt.writeVarintField(5, Ie.extent || 4096);
                  var pt = { keys: [], values: [], keycache: {}, valuecache: {} };
                  for (ft = 0; ft < Ie.length; ft++) pt.feature = Ie.feature(ft), rt.writeMessage(2, ke, pt);
                  var wt = pt.keys;
                  for (ft = 0; ft < wt.length; ft++) rt.writeStringField(3, wt[ft]);
                  var Bt = pt.values;
                  for (ft = 0; ft < Bt.length; ft++) rt.writeMessage(4, $e, Bt[ft]);
                }
                function ke(Ie, rt) {
                  var ft = Ie.feature;
                  ft.id !== void 0 && rt.writeVarintField(1, ft.id), rt.writeMessage(2, Me, Ie), rt.writeVarintField(3, ft.type), rt.writeMessage(4, Pe, ft);
                }
                function Me(Ie, rt) {
                  var ft = Ie.feature, pt = Ie.keys, wt = Ie.values, Bt = Ie.keycache, At = Ie.valuecache;
                  for (var Dt in ft.properties) {
                    var Ot = ft.properties[Dt], un = Bt[Dt];
                    if (Ot !== null) {
                      un === void 0 && (pt.push(Dt), un = pt.length - 1, Bt[Dt] = un), rt.writeVarint(un);
                      var yn = typeof Ot;
                      yn !== "string" && yn !== "boolean" && yn !== "number" && (Ot = JSON.stringify(Ot));
                      var _n = yn + ":" + Ot, gn = At[_n];
                      gn === void 0 && (wt.push(Ot), gn = wt.length - 1, At[_n] = gn), rt.writeVarint(gn);
                    }
                  }
                }
                function Re(Ie, rt) {
                  return (rt << 3) + (7 & Ie);
                }
                function De(Ie) {
                  return Ie << 1 ^ Ie >> 31;
                }
                function Pe(Ie, rt) {
                  for (var ft = Ie.loadGeometry(), pt = Ie.type, wt = 0, Bt = 0, At = ft.length, Dt = 0; Dt < At; Dt++) {
                    var Ot = ft[Dt], un = 1;
                    pt === 1 && (un = Ot.length), rt.writeVarint(Re(1, un));
                    for (var yn = pt === 3 ? Ot.length - 1 : Ot.length, _n = 0; _n < yn; _n++) {
                      _n === 1 && pt !== 1 && rt.writeVarint(Re(2, yn - 1));
                      var gn = Ot[_n].x - wt, bn = Ot[_n].y - Bt;
                      rt.writeVarint(De(gn)), rt.writeVarint(De(bn)), wt += gn, Bt += bn;
                    }
                    pt === 3 && rt.writeVarint(Re(7, 1));
                  }
                }
                function $e(Ie, rt) {
                  var ft = typeof Ie;
                  ft === "string" ? rt.writeStringField(1, Ie) : ft === "boolean" ? rt.writeBooleanField(7, Ie) : ft === "number" && (Ie % 1 != 0 ? rt.writeDoubleField(3, Ie) : Ie < 0 ? rt.writeSVarintField(6, Ie) : rt.writeVarintField(5, Ie));
                }
                ge.exports = Le, ge.exports.fromVectorTileJs = Le, ge.exports.fromGeojsonVt = function(Ie, rt) {
                  rt = rt || {};
                  var ft = {};
                  for (var pt in Ie) ft[pt] = new xe(Ie[pt].features, rt), ft[pt].name = pt, ft[pt].version = rt.version, ft[pt].extent = rt.extent;
                  return Le({ layers: ft });
                }, ge.exports.GeoJSONWrapper = xe;
                var qe = ge.exports, Xe = ne.bx(qe);
                const Ve = { minZoom: 0, maxZoom: 16, minPoints: 2, radius: 40, extent: 512, nodeSize: 64, log: !1, generateId: !1, reduce: null, map: (Ie) => Ie }, Ze = Math.fround || (Ge = new Float32Array(1), (Ie) => (Ge[0] = +Ie, Ge[0]));
                var Ge;
                const ze = 3, We = 5, at = 6;
                class ct {
                  constructor(rt) {
                    this.options = Object.assign(Object.create(Ve), rt), this.trees = new Array(this.options.maxZoom + 1), this.stride = this.options.reduce ? 7 : 6, this.clusterProps = [];
                  }
                  load(rt) {
                    const { log: ft, minZoom: pt, maxZoom: wt } = this.options;
                    ft && console.time("total time");
                    const Bt = `prepare ${rt.length} points`;
                    ft && console.time(Bt), this.points = rt;
                    const At = [];
                    for (let Ot = 0; Ot < rt.length; Ot++) {
                      const un = rt[Ot];
                      if (!un.geometry) continue;
                      const [yn, _n] = un.geometry.coordinates, gn = Ze(Je(yn)), bn = Ze(it(_n));
                      At.push(gn, bn, 1 / 0, Ot, -1, 1), this.options.reduce && At.push(0);
                    }
                    let Dt = this.trees[wt + 1] = this._createTree(At);
                    ft && console.timeEnd(Bt);
                    for (let Ot = wt; Ot >= pt; Ot--) {
                      const un = +Date.now();
                      Dt = this.trees[Ot] = this._createTree(this._cluster(Dt, Ot)), ft && console.log("z%d: %d clusters in %dms", Ot, Dt.numItems, +Date.now() - un);
                    }
                    return ft && console.timeEnd("total time"), this;
                  }
                  getClusters(rt, ft) {
                    let pt = ((rt[0] + 180) % 360 + 360) % 360 - 180;
                    const wt = Math.max(-90, Math.min(90, rt[1]));
                    let Bt = rt[2] === 180 ? 180 : ((rt[2] + 180) % 360 + 360) % 360 - 180;
                    const At = Math.max(-90, Math.min(90, rt[3]));
                    if (rt[2] - rt[0] >= 360) pt = -180, Bt = 180;
                    else if (pt > Bt) {
                      const _n = this.getClusters([pt, wt, 180, At], ft), gn = this.getClusters([-180, wt, Bt, At], ft);
                      return _n.concat(gn);
                    }
                    const Dt = this.trees[this._limitZoom(ft)], Ot = Dt.range(Je(pt), it(At), Je(Bt), it(wt)), un = Dt.data, yn = [];
                    for (const _n of Ot) {
                      const gn = this.stride * _n;
                      yn.push(un[gn + We] > 1 ? et(un, gn, this.clusterProps) : this.points[un[gn + ze]]);
                    }
                    return yn;
                  }
                  getChildren(rt) {
                    const ft = this._getOriginId(rt), pt = this._getOriginZoom(rt), wt = "No cluster with the specified id.", Bt = this.trees[pt];
                    if (!Bt) throw new Error(wt);
                    const At = Bt.data;
                    if (ft * this.stride >= At.length) throw new Error(wt);
                    const Dt = this.options.radius / (this.options.extent * Math.pow(2, pt - 1)), Ot = At[ft * this.stride], un = At[ft * this.stride + 1], yn = Bt.within(Ot, un, Dt), _n = [];
                    for (const gn of yn) {
                      const bn = gn * this.stride;
                      At[bn + 4] === rt && _n.push(At[bn + We] > 1 ? et(At, bn, this.clusterProps) : this.points[At[bn + ze]]);
                    }
                    if (_n.length === 0) throw new Error(wt);
                    return _n;
                  }
                  getLeaves(rt, ft, pt) {
                    ft = ft || 10, pt = pt || 0;
                    const wt = [];
                    return this._appendLeaves(wt, rt, ft, pt, 0), wt;
                  }
                  getTile(rt, ft, pt) {
                    const wt = this.trees[this._limitZoom(rt)], Bt = Math.pow(2, rt), { extent: At, radius: Dt } = this.options, Ot = Dt / At, un = (pt - Ot) / Bt, yn = (pt + 1 + Ot) / Bt, _n = { features: [] };
                    return this._addTileFeatures(wt.range((ft - Ot) / Bt, un, (ft + 1 + Ot) / Bt, yn), wt.data, ft, pt, Bt, _n), ft === 0 && this._addTileFeatures(wt.range(1 - Ot / Bt, un, 1, yn), wt.data, Bt, pt, Bt, _n), ft === Bt - 1 && this._addTileFeatures(wt.range(0, un, Ot / Bt, yn), wt.data, -1, pt, Bt, _n), _n.features.length ? _n : null;
                  }
                  getClusterExpansionZoom(rt) {
                    let ft = this._getOriginZoom(rt) - 1;
                    for (; ft <= this.options.maxZoom; ) {
                      const pt = this.getChildren(rt);
                      if (ft++, pt.length !== 1) break;
                      rt = pt[0].properties.cluster_id;
                    }
                    return ft;
                  }
                  _appendLeaves(rt, ft, pt, wt, Bt) {
                    const At = this.getChildren(ft);
                    for (const Dt of At) {
                      const Ot = Dt.properties;
                      if (Ot && Ot.cluster ? Bt + Ot.point_count <= wt ? Bt += Ot.point_count : Bt = this._appendLeaves(rt, Ot.cluster_id, pt, wt, Bt) : Bt < wt ? Bt++ : rt.push(Dt), rt.length === pt) break;
                    }
                    return Bt;
                  }
                  _createTree(rt) {
                    const ft = new ne.aw(rt.length / this.stride | 0, this.options.nodeSize, Float32Array);
                    for (let pt = 0; pt < rt.length; pt += this.stride) ft.add(rt[pt], rt[pt + 1]);
                    return ft.finish(), ft.data = rt, ft;
                  }
                  _addTileFeatures(rt, ft, pt, wt, Bt, At) {
                    for (const Dt of rt) {
                      const Ot = Dt * this.stride, un = ft[Ot + We] > 1;
                      let yn, _n, gn;
                      if (un) yn = dt(ft, Ot, this.clusterProps), _n = ft[Ot], gn = ft[Ot + 1];
                      else {
                        const Zt = this.points[ft[Ot + ze]];
                        yn = Zt.properties;
                        const [Mn, Ln] = Zt.geometry.coordinates;
                        _n = Je(Mn), gn = it(Ln);
                      }
                      const bn = { type: 1, geometry: [[Math.round(this.options.extent * (_n * Bt - pt)), Math.round(this.options.extent * (gn * Bt - wt))]], tags: yn };
                      let an;
                      an = un || this.options.generateId ? ft[Ot + ze] : this.points[ft[Ot + ze]].id, an !== void 0 && (bn.id = an), At.features.push(bn);
                    }
                  }
                  _limitZoom(rt) {
                    return Math.max(this.options.minZoom, Math.min(Math.floor(+rt), this.options.maxZoom + 1));
                  }
                  _cluster(rt, ft) {
                    const { radius: pt, extent: wt, reduce: Bt, minPoints: At } = this.options, Dt = pt / (wt * Math.pow(2, ft)), Ot = rt.data, un = [], yn = this.stride;
                    for (let _n = 0; _n < Ot.length; _n += yn) {
                      if (Ot[_n + 2] <= ft) continue;
                      Ot[_n + 2] = ft;
                      const gn = Ot[_n], bn = Ot[_n + 1], an = rt.within(Ot[_n], Ot[_n + 1], Dt), Zt = Ot[_n + We];
                      let Mn = Zt;
                      for (const Ln of an) {
                        const In = Ln * yn;
                        Ot[In + 2] > ft && (Mn += Ot[In + We]);
                      }
                      if (Mn > Zt && Mn >= At) {
                        let Ln, In = gn * Zt, kn = bn * Zt, er = -1;
                        const jn = ((_n / yn | 0) << 5) + (ft + 1) + this.points.length;
                        for (const ur of an) {
                          const Kn = ur * yn;
                          if (Ot[Kn + 2] <= ft) continue;
                          Ot[Kn + 2] = ft;
                          const Jn = Ot[Kn + We];
                          In += Ot[Kn] * Jn, kn += Ot[Kn + 1] * Jn, Ot[Kn + 4] = jn, Bt && (Ln || (Ln = this._map(Ot, _n, !0), er = this.clusterProps.length, this.clusterProps.push(Ln)), Bt(Ln, this._map(Ot, Kn)));
                        }
                        Ot[_n + 4] = jn, un.push(In / Mn, kn / Mn, 1 / 0, jn, -1, Mn), Bt && un.push(er);
                      } else {
                        for (let Ln = 0; Ln < yn; Ln++) un.push(Ot[_n + Ln]);
                        if (Mn > 1) for (const Ln of an) {
                          const In = Ln * yn;
                          if (!(Ot[In + 2] <= ft)) {
                            Ot[In + 2] = ft;
                            for (let kn = 0; kn < yn; kn++) un.push(Ot[In + kn]);
                          }
                        }
                      }
                    }
                    return un;
                  }
                  _getOriginId(rt) {
                    return rt - this.points.length >> 5;
                  }
                  _getOriginZoom(rt) {
                    return (rt - this.points.length) % 32;
                  }
                  _map(rt, ft, pt) {
                    if (rt[ft + We] > 1) {
                      const At = this.clusterProps[rt[ft + at]];
                      return pt ? Object.assign({}, At) : At;
                    }
                    const wt = this.points[rt[ft + ze]].properties, Bt = this.options.map(wt);
                    return pt && Bt === wt ? Object.assign({}, Bt) : Bt;
                  }
                }
                function et(Ie, rt, ft) {
                  return { type: "Feature", id: Ie[rt + ze], properties: dt(Ie, rt, ft), geometry: { type: "Point", coordinates: [(pt = Ie[rt], 360 * (pt - 0.5)), st(Ie[rt + 1])] } };
                  var pt;
                }
                function dt(Ie, rt, ft) {
                  const pt = Ie[rt + We], wt = pt >= 1e4 ? `${Math.round(pt / 1e3)}k` : pt >= 1e3 ? Math.round(pt / 100) / 10 + "k" : pt, Bt = Ie[rt + at], At = Bt === -1 ? {} : Object.assign({}, ft[Bt]);
                  return Object.assign(At, { cluster: !0, cluster_id: Ie[rt + ze], point_count: pt, point_count_abbreviated: wt });
                }
                function Je(Ie) {
                  return Ie / 360 + 0.5;
                }
                function it(Ie) {
                  const rt = Math.sin(Ie * Math.PI / 180), ft = 0.5 - 0.25 * Math.log((1 + rt) / (1 - rt)) / Math.PI;
                  return ft < 0 ? 0 : ft > 1 ? 1 : ft;
                }
                function st(Ie) {
                  const rt = (180 - 360 * Ie) * Math.PI / 180;
                  return 360 * Math.atan(Math.exp(rt)) / Math.PI - 90;
                }
                function gt(Ie, rt, ft, pt) {
                  let wt = pt;
                  const Bt = rt + (ft - rt >> 1);
                  let At, Dt = ft - rt;
                  const Ot = Ie[rt], un = Ie[rt + 1], yn = Ie[ft], _n = Ie[ft + 1];
                  for (let gn = rt + 3; gn < ft; gn += 3) {
                    const bn = bt(Ie[gn], Ie[gn + 1], Ot, un, yn, _n);
                    if (bn > wt) At = gn, wt = bn;
                    else if (bn === wt) {
                      const an = Math.abs(gn - Bt);
                      an < Dt && (At = gn, Dt = an);
                    }
                  }
                  wt > pt && (At - rt > 3 && gt(Ie, rt, At, pt), Ie[At + 2] = wt, ft - At > 3 && gt(Ie, At, ft, pt));
                }
                function bt(Ie, rt, ft, pt, wt, Bt) {
                  let At = wt - ft, Dt = Bt - pt;
                  if (At !== 0 || Dt !== 0) {
                    const Ot = ((Ie - ft) * At + (rt - pt) * Dt) / (At * At + Dt * Dt);
                    Ot > 1 ? (ft = wt, pt = Bt) : Ot > 0 && (ft += At * Ot, pt += Dt * Ot);
                  }
                  return At = Ie - ft, Dt = rt - pt, At * At + Dt * Dt;
                }
                function yt(Ie, rt, ft, pt) {
                  const wt = { id: Ie ?? null, type: rt, geometry: ft, tags: pt, minX: 1 / 0, minY: 1 / 0, maxX: -1 / 0, maxY: -1 / 0 };
                  if (rt === "Point" || rt === "MultiPoint" || rt === "LineString") St(wt, ft);
                  else if (rt === "Polygon") St(wt, ft[0]);
                  else if (rt === "MultiLineString") for (const Bt of ft) St(wt, Bt);
                  else if (rt === "MultiPolygon") for (const Bt of ft) St(wt, Bt[0]);
                  return wt;
                }
                function St(Ie, rt) {
                  for (let ft = 0; ft < rt.length; ft += 3) Ie.minX = Math.min(Ie.minX, rt[ft]), Ie.minY = Math.min(Ie.minY, rt[ft + 1]), Ie.maxX = Math.max(Ie.maxX, rt[ft]), Ie.maxY = Math.max(Ie.maxY, rt[ft + 1]);
                }
                function Lt(Ie, rt, ft, pt) {
                  if (!rt.geometry) return;
                  const wt = rt.geometry.coordinates;
                  if (wt && wt.length === 0) return;
                  const Bt = rt.geometry.type, At = Math.pow(ft.tolerance / ((1 << ft.maxZoom) * ft.extent), 2);
                  let Dt = [], Ot = rt.id;
                  if (ft.promoteId ? Ot = rt.properties[ft.promoteId] : ft.generateId && (Ot = pt || 0), Bt === "Point") Rt(wt, Dt);
                  else if (Bt === "MultiPoint") for (const un of wt) Rt(un, Dt);
                  else if (Bt === "LineString") vt(wt, Dt, At, !1);
                  else if (Bt === "MultiLineString") {
                    if (ft.lineMetrics) {
                      for (const un of wt) Dt = [], vt(un, Dt, At, !1), Ie.push(yt(Ot, "LineString", Dt, rt.properties));
                      return;
                    }
                    $t(wt, Dt, At, !1);
                  } else if (Bt === "Polygon") $t(wt, Dt, At, !0);
                  else {
                    if (Bt !== "MultiPolygon") {
                      if (Bt === "GeometryCollection") {
                        for (const un of rt.geometry.geometries) Lt(Ie, { id: Ot, geometry: un, properties: rt.properties }, ft, pt);
                        return;
                      }
                      throw new Error("Input data is not a valid GeoJSON object.");
                    }
                    for (const un of wt) {
                      const yn = [];
                      $t(un, yn, At, !0), Dt.push(yn);
                    }
                  }
                  Ie.push(yt(Ot, Bt, Dt, rt.properties));
                }
                function Rt(Ie, rt) {
                  rt.push(Pt(Ie[0]), Xt(Ie[1]), 0);
                }
                function vt(Ie, rt, ft, pt) {
                  let wt, Bt, At = 0;
                  for (let Ot = 0; Ot < Ie.length; Ot++) {
                    const un = Pt(Ie[Ot][0]), yn = Xt(Ie[Ot][1]);
                    rt.push(un, yn, 0), Ot > 0 && (At += pt ? (wt * yn - un * Bt) / 2 : Math.sqrt(Math.pow(un - wt, 2) + Math.pow(yn - Bt, 2))), wt = un, Bt = yn;
                  }
                  const Dt = rt.length - 3;
                  rt[2] = 1, gt(rt, 0, Dt, ft), rt[Dt + 2] = 1, rt.size = Math.abs(At), rt.start = 0, rt.end = rt.size;
                }
                function $t(Ie, rt, ft, pt) {
                  for (let wt = 0; wt < Ie.length; wt++) {
                    const Bt = [];
                    vt(Ie[wt], Bt, ft, pt), rt.push(Bt);
                  }
                }
                function Pt(Ie) {
                  return Ie / 360 + 0.5;
                }
                function Xt(Ie) {
                  const rt = Math.sin(Ie * Math.PI / 180), ft = 0.5 - 0.25 * Math.log((1 + rt) / (1 - rt)) / Math.PI;
                  return ft < 0 ? 0 : ft > 1 ? 1 : ft;
                }
                function Nt(Ie, rt, ft, pt, wt, Bt, At, Dt) {
                  if (pt /= rt, Bt >= (ft /= rt) && At < pt) return Ie;
                  if (At < ft || Bt >= pt) return null;
                  const Ot = [];
                  for (const un of Ie) {
                    const yn = un.geometry;
                    let _n = un.type;
                    const gn = wt === 0 ? un.minX : un.minY, bn = wt === 0 ? un.maxX : un.maxY;
                    if (gn >= ft && bn < pt) {
                      Ot.push(un);
                      continue;
                    }
                    if (bn < ft || gn >= pt) continue;
                    let an = [];
                    if (_n === "Point" || _n === "MultiPoint") Gt(yn, an, ft, pt, wt);
                    else if (_n === "LineString") Ft(yn, an, ft, pt, wt, !1, Dt.lineMetrics);
                    else if (_n === "MultiLineString") It(yn, an, ft, pt, wt, !1);
                    else if (_n === "Polygon") It(yn, an, ft, pt, wt, !0);
                    else if (_n === "MultiPolygon") for (const Zt of yn) {
                      const Mn = [];
                      It(Zt, Mn, ft, pt, wt, !0), Mn.length && an.push(Mn);
                    }
                    if (an.length) {
                      if (Dt.lineMetrics && _n === "LineString") {
                        for (const Zt of an) Ot.push(yt(un.id, _n, Zt, un.tags));
                        continue;
                      }
                      _n !== "LineString" && _n !== "MultiLineString" || (an.length === 1 ? (_n = "LineString", an = an[0]) : _n = "MultiLineString"), _n !== "Point" && _n !== "MultiPoint" || (_n = an.length === 3 ? "Point" : "MultiPoint"), Ot.push(yt(un.id, _n, an, un.tags));
                    }
                  }
                  return Ot.length ? Ot : null;
                }
                function Gt(Ie, rt, ft, pt, wt) {
                  for (let Bt = 0; Bt < Ie.length; Bt += 3) {
                    const At = Ie[Bt + wt];
                    At >= ft && At <= pt && Ct(rt, Ie[Bt], Ie[Bt + 1], Ie[Bt + 2]);
                  }
                }
                function Ft(Ie, rt, ft, pt, wt, Bt, At) {
                  let Dt = Tt(Ie);
                  const Ot = wt === 0 ? Kt : Vt;
                  let un, yn, _n = Ie.start;
                  for (let Ln = 0; Ln < Ie.length - 3; Ln += 3) {
                    const In = Ie[Ln], kn = Ie[Ln + 1], er = Ie[Ln + 2], jn = Ie[Ln + 3], ur = Ie[Ln + 4], Kn = wt === 0 ? In : kn, Jn = wt === 0 ? jn : ur;
                    let Lr = !1;
                    At && (un = Math.sqrt(Math.pow(In - jn, 2) + Math.pow(kn - ur, 2))), Kn < ft ? Jn > ft && (yn = Ot(Dt, In, kn, jn, ur, ft), At && (Dt.start = _n + un * yn)) : Kn > pt ? Jn < pt && (yn = Ot(Dt, In, kn, jn, ur, pt), At && (Dt.start = _n + un * yn)) : Ct(Dt, In, kn, er), Jn < ft && Kn >= ft && (yn = Ot(Dt, In, kn, jn, ur, ft), Lr = !0), Jn > pt && Kn <= pt && (yn = Ot(Dt, In, kn, jn, ur, pt), Lr = !0), !Bt && Lr && (At && (Dt.end = _n + un * yn), rt.push(Dt), Dt = Tt(Ie)), At && (_n += un);
                  }
                  let gn = Ie.length - 3;
                  const bn = Ie[gn], an = Ie[gn + 1], Zt = Ie[gn + 2], Mn = wt === 0 ? bn : an;
                  Mn >= ft && Mn <= pt && Ct(Dt, bn, an, Zt), gn = Dt.length - 3, Bt && gn >= 3 && (Dt[gn] !== Dt[0] || Dt[gn + 1] !== Dt[1]) && Ct(Dt, Dt[0], Dt[1], Dt[2]), Dt.length && rt.push(Dt);
                }
                function Tt(Ie) {
                  const rt = [];
                  return rt.size = Ie.size, rt.start = Ie.start, rt.end = Ie.end, rt;
                }
                function It(Ie, rt, ft, pt, wt, Bt) {
                  for (const At of Ie) Ft(At, rt, ft, pt, wt, Bt, !1);
                }
                function Ct(Ie, rt, ft, pt) {
                  Ie.push(rt, ft, pt);
                }
                function Kt(Ie, rt, ft, pt, wt, Bt) {
                  const At = (Bt - rt) / (pt - rt);
                  return Ct(Ie, Bt, ft + (wt - ft) * At, 1), At;
                }
                function Vt(Ie, rt, ft, pt, wt, Bt) {
                  const At = (Bt - ft) / (wt - ft);
                  return Ct(Ie, rt + (pt - rt) * At, Bt, 1), At;
                }
                function Yt(Ie, rt) {
                  const ft = [];
                  for (let pt = 0; pt < Ie.length; pt++) {
                    const wt = Ie[pt], Bt = wt.type;
                    let At;
                    if (Bt === "Point" || Bt === "MultiPoint" || Bt === "LineString") At = sn(wt.geometry, rt);
                    else if (Bt === "MultiLineString" || Bt === "Polygon") {
                      At = [];
                      for (const Dt of wt.geometry) At.push(sn(Dt, rt));
                    } else if (Bt === "MultiPolygon") {
                      At = [];
                      for (const Dt of wt.geometry) {
                        const Ot = [];
                        for (const un of Dt) Ot.push(sn(un, rt));
                        At.push(Ot);
                      }
                    }
                    ft.push(yt(wt.id, Bt, At, wt.tags));
                  }
                  return ft;
                }
                function sn(Ie, rt) {
                  const ft = [];
                  ft.size = Ie.size, Ie.start !== void 0 && (ft.start = Ie.start, ft.end = Ie.end);
                  for (let pt = 0; pt < Ie.length; pt += 3) ft.push(Ie[pt] + rt, Ie[pt + 1], Ie[pt + 2]);
                  return ft;
                }
                function hn(Ie, rt) {
                  if (Ie.transformed) return Ie;
                  const ft = 1 << Ie.z, pt = Ie.x, wt = Ie.y;
                  for (const Bt of Ie.features) {
                    const At = Bt.geometry, Dt = Bt.type;
                    if (Bt.geometry = [], Dt === 1) for (let Ot = 0; Ot < At.length; Ot += 2) Bt.geometry.push(Qt(At[Ot], At[Ot + 1], rt, ft, pt, wt));
                    else for (let Ot = 0; Ot < At.length; Ot++) {
                      const un = [];
                      for (let yn = 0; yn < At[Ot].length; yn += 2) un.push(Qt(At[Ot][yn], At[Ot][yn + 1], rt, ft, pt, wt));
                      Bt.geometry.push(un);
                    }
                  }
                  return Ie.transformed = !0, Ie;
                }
                function Qt(Ie, rt, ft, pt, wt, Bt) {
                  return [Math.round(ft * (Ie * pt - wt)), Math.round(ft * (rt * pt - Bt))];
                }
                function Ut(Ie, rt, ft, pt, wt) {
                  const Bt = rt === wt.maxZoom ? 0 : wt.tolerance / ((1 << rt) * wt.extent), At = { features: [], numPoints: 0, numSimplified: 0, numFeatures: Ie.length, source: null, x: ft, y: pt, z: rt, transformed: !1, minX: 2, minY: 1, maxX: -1, maxY: 0 };
                  for (const Dt of Ie) Wt(At, Dt, Bt, wt);
                  return At;
                }
                function Wt(Ie, rt, ft, pt) {
                  const wt = rt.geometry, Bt = rt.type, At = [];
                  if (Ie.minX = Math.min(Ie.minX, rt.minX), Ie.minY = Math.min(Ie.minY, rt.minY), Ie.maxX = Math.max(Ie.maxX, rt.maxX), Ie.maxY = Math.max(Ie.maxY, rt.maxY), Bt === "Point" || Bt === "MultiPoint") for (let Dt = 0; Dt < wt.length; Dt += 3) At.push(wt[Dt], wt[Dt + 1]), Ie.numPoints++, Ie.numSimplified++;
                  else if (Bt === "LineString") rn(At, wt, Ie, ft, !1, !1);
                  else if (Bt === "MultiLineString" || Bt === "Polygon") for (let Dt = 0; Dt < wt.length; Dt++) rn(At, wt[Dt], Ie, ft, Bt === "Polygon", Dt === 0);
                  else if (Bt === "MultiPolygon") for (let Dt = 0; Dt < wt.length; Dt++) {
                    const Ot = wt[Dt];
                    for (let un = 0; un < Ot.length; un++) rn(At, Ot[un], Ie, ft, !0, un === 0);
                  }
                  if (At.length) {
                    let Dt = rt.tags || null;
                    if (Bt === "LineString" && pt.lineMetrics) {
                      Dt = {};
                      for (const un in rt.tags) Dt[un] = rt.tags[un];
                      Dt.mapbox_clip_start = wt.start / wt.size, Dt.mapbox_clip_end = wt.end / wt.size;
                    }
                    const Ot = { geometry: At, type: Bt === "Polygon" || Bt === "MultiPolygon" ? 3 : Bt === "LineString" || Bt === "MultiLineString" ? 2 : 1, tags: Dt };
                    rt.id !== null && (Ot.id = rt.id), Ie.features.push(Ot);
                  }
                }
                function rn(Ie, rt, ft, pt, wt, Bt) {
                  const At = pt * pt;
                  if (pt > 0 && rt.size < (wt ? At : pt)) return void (ft.numPoints += rt.length / 3);
                  const Dt = [];
                  for (let Ot = 0; Ot < rt.length; Ot += 3) (pt === 0 || rt[Ot + 2] > At) && (ft.numSimplified++, Dt.push(rt[Ot], rt[Ot + 1])), ft.numPoints++;
                  wt && function(Ot, un) {
                    let yn = 0;
                    for (let _n = 0, gn = Ot.length, bn = gn - 2; _n < gn; bn = _n, _n += 2) yn += (Ot[_n] - Ot[bn]) * (Ot[_n + 1] + Ot[bn + 1]);
                    if (yn > 0 === un) for (let _n = 0, gn = Ot.length; _n < gn / 2; _n += 2) {
                      const bn = Ot[_n], an = Ot[_n + 1];
                      Ot[_n] = Ot[gn - 2 - _n], Ot[_n + 1] = Ot[gn - 1 - _n], Ot[gn - 2 - _n] = bn, Ot[gn - 1 - _n] = an;
                    }
                  }(Dt, Bt), Ie.push(Dt);
                }
                const vn = { maxZoom: 14, indexMaxZoom: 5, indexMaxPoints: 1e5, tolerance: 3, extent: 4096, buffer: 64, lineMetrics: !1, promoteId: null, generateId: !1, debug: 0 };
                class dn {
                  constructor(rt, ft) {
                    const pt = (ft = this.options = function(Bt, At) {
                      for (const Dt in At) Bt[Dt] = At[Dt];
                      return Bt;
                    }(Object.create(vn), ft)).debug;
                    if (pt && console.time("preprocess data"), ft.maxZoom < 0 || ft.maxZoom > 24) throw new Error("maxZoom should be in the 0-24 range");
                    if (ft.promoteId && ft.generateId) throw new Error("promoteId and generateId cannot be used together.");
                    let wt = function(Bt, At) {
                      const Dt = [];
                      if (Bt.type === "FeatureCollection") for (let Ot = 0; Ot < Bt.features.length; Ot++) Lt(Dt, Bt.features[Ot], At, Ot);
                      else Bt.type === "Feature" ? Lt(Dt, Bt, At) : Lt(Dt, { geometry: Bt }, At);
                      return Dt;
                    }(rt, ft);
                    this.tiles = {}, this.tileCoords = [], pt && (console.timeEnd("preprocess data"), console.log("index: maxZoom: %d, maxPoints: %d", ft.indexMaxZoom, ft.indexMaxPoints), console.time("generate tiles"), this.stats = {}, this.total = 0), wt = function(Bt, At) {
                      const Dt = At.buffer / At.extent;
                      let Ot = Bt;
                      const un = Nt(Bt, 1, -1 - Dt, Dt, 0, -1, 2, At), yn = Nt(Bt, 1, 1 - Dt, 2 + Dt, 0, -1, 2, At);
                      return (un || yn) && (Ot = Nt(Bt, 1, -Dt, 1 + Dt, 0, -1, 2, At) || [], un && (Ot = Yt(un, 1).concat(Ot)), yn && (Ot = Ot.concat(Yt(yn, -1)))), Ot;
                    }(wt, ft), wt.length && this.splitTile(wt, 0, 0, 0), pt && (wt.length && console.log("features: %d, points: %d", this.tiles[0].numFeatures, this.tiles[0].numPoints), console.timeEnd("generate tiles"), console.log("tiles generated:", this.total, JSON.stringify(this.stats)));
                  }
                  splitTile(rt, ft, pt, wt, Bt, At, Dt) {
                    const Ot = [rt, ft, pt, wt], un = this.options, yn = un.debug;
                    for (; Ot.length; ) {
                      wt = Ot.pop(), pt = Ot.pop(), ft = Ot.pop(), rt = Ot.pop();
                      const _n = 1 << ft, gn = Ht(ft, pt, wt);
                      let bn = this.tiles[gn];
                      if (!bn && (yn > 1 && console.time("creation"), bn = this.tiles[gn] = Ut(rt, ft, pt, wt, un), this.tileCoords.push({ z: ft, x: pt, y: wt }), yn)) {
                        yn > 1 && (console.log("tile z%d-%d-%d (features: %d, points: %d, simplified: %d)", ft, pt, wt, bn.numFeatures, bn.numPoints, bn.numSimplified), console.timeEnd("creation"));
                        const Jn = `z${ft}`;
                        this.stats[Jn] = (this.stats[Jn] || 0) + 1, this.total++;
                      }
                      if (bn.source = rt, Bt == null) {
                        if (ft === un.indexMaxZoom || bn.numPoints <= un.indexMaxPoints) continue;
                      } else {
                        if (ft === un.maxZoom || ft === Bt) continue;
                        if (Bt != null) {
                          const Jn = Bt - ft;
                          if (pt !== At >> Jn || wt !== Dt >> Jn) continue;
                        }
                      }
                      if (bn.source = null, rt.length === 0) continue;
                      yn > 1 && console.time("clipping");
                      const an = 0.5 * un.buffer / un.extent, Zt = 0.5 - an, Mn = 0.5 + an, Ln = 1 + an;
                      let In = null, kn = null, er = null, jn = null, ur = Nt(rt, _n, pt - an, pt + Mn, 0, bn.minX, bn.maxX, un), Kn = Nt(rt, _n, pt + Zt, pt + Ln, 0, bn.minX, bn.maxX, un);
                      rt = null, ur && (In = Nt(ur, _n, wt - an, wt + Mn, 1, bn.minY, bn.maxY, un), kn = Nt(ur, _n, wt + Zt, wt + Ln, 1, bn.minY, bn.maxY, un), ur = null), Kn && (er = Nt(Kn, _n, wt - an, wt + Mn, 1, bn.minY, bn.maxY, un), jn = Nt(Kn, _n, wt + Zt, wt + Ln, 1, bn.minY, bn.maxY, un), Kn = null), yn > 1 && console.timeEnd("clipping"), Ot.push(In || [], ft + 1, 2 * pt, 2 * wt), Ot.push(kn || [], ft + 1, 2 * pt, 2 * wt + 1), Ot.push(er || [], ft + 1, 2 * pt + 1, 2 * wt), Ot.push(jn || [], ft + 1, 2 * pt + 1, 2 * wt + 1);
                    }
                  }
                  getTile(rt, ft, pt) {
                    rt = +rt, ft = +ft, pt = +pt;
                    const wt = this.options, { extent: Bt, debug: At } = wt;
                    if (rt < 0 || rt > 24) return null;
                    const Dt = 1 << rt, Ot = Ht(rt, ft = ft + Dt & Dt - 1, pt);
                    if (this.tiles[Ot]) return hn(this.tiles[Ot], Bt);
                    At > 1 && console.log("drilling down to z%d-%d-%d", rt, ft, pt);
                    let un, yn = rt, _n = ft, gn = pt;
                    for (; !un && yn > 0; ) yn--, _n >>= 1, gn >>= 1, un = this.tiles[Ht(yn, _n, gn)];
                    return un && un.source ? (At > 1 && (console.log("found parent tile z%d-%d-%d", yn, _n, gn), console.time("drilling down")), this.splitTile(un.source, yn, _n, gn, rt, ft, pt), At > 1 && console.timeEnd("drilling down"), this.tiles[Ot] ? hn(this.tiles[Ot], Bt) : null) : null;
                  }
                }
                function Ht(Ie, rt, ft) {
                  return 32 * ((1 << Ie) * ft + rt) + Ie;
                }
                function jt(Ie, rt) {
                  return rt ? Ie.properties[rt] : Ie.id;
                }
                function cn(Ie, rt) {
                  if (Ie == null) return !0;
                  if (Ie.type === "Feature") return jt(Ie, rt) != null;
                  if (Ie.type === "FeatureCollection") {
                    const ft = /* @__PURE__ */ new Set();
                    for (const pt of Ie.features) {
                      const wt = jt(pt, rt);
                      if (wt == null || ft.has(wt)) return !1;
                      ft.add(wt);
                    }
                    return !0;
                  }
                  return !1;
                }
                function wn(Ie, rt) {
                  const ft = /* @__PURE__ */ new Map();
                  if (Ie != null) if (Ie.type === "Feature") ft.set(jt(Ie, rt), Ie);
                  else for (const pt of Ie.features) ft.set(jt(pt, rt), pt);
                  return ft;
                }
                class Rn extends ce {
                  constructor() {
                    super(...arguments), this._dataUpdateable = /* @__PURE__ */ new Map();
                  }
                  loadVectorTile(rt, ft) {
                    return ne._(this, void 0, void 0, function* () {
                      const pt = rt.tileID.canonical;
                      if (!this._geoJSONIndex) throw new Error("Unable to parse the data into a cluster or geojson");
                      const wt = this._geoJSONIndex.getTile(pt.z, pt.x, pt.y);
                      if (!wt) return null;
                      const Bt = new ye(wt.features);
                      let At = Xe(Bt);
                      return At.byteOffset === 0 && At.byteLength === At.buffer.byteLength || (At = new Uint8Array(At)), { vectorTile: Bt, rawData: At.buffer };
                    });
                  }
                  loadData(rt) {
                    return ne._(this, void 0, void 0, function* () {
                      var ft;
                      (ft = this._pendingRequest) === null || ft === void 0 || ft.abort();
                      const pt = !!(rt && rt.request && rt.request.collectResourceTiming) && new ne.bu(rt.request);
                      this._pendingRequest = new AbortController();
                      try {
                        this._pendingData = this.loadAndProcessGeoJSON(rt, this._pendingRequest), this._geoJSONIndex = rt.cluster ? new ct(function({ superclusterOptions: Dt, clusterProperties: Ot }) {
                          if (!Ot || !Dt) return Dt;
                          const un = {}, yn = {}, _n = { accumulated: null, zoom: 0 }, gn = { properties: null }, bn = Object.keys(Ot);
                          for (const an of bn) {
                            const [Zt, Mn] = Ot[an], Ln = ne.bB(Mn), In = ne.bB(typeof Zt == "string" ? [Zt, ["accumulated"], ["get", an]] : Zt);
                            un[an] = Ln.value, yn[an] = In.value;
                          }
                          return Dt.map = (an) => {
                            gn.properties = an;
                            const Zt = {};
                            for (const Mn of bn) Zt[Mn] = un[Mn].evaluate(_n, gn);
                            return Zt;
                          }, Dt.reduce = (an, Zt) => {
                            gn.properties = Zt;
                            for (const Mn of bn) _n.accumulated = an[Mn], an[Mn] = yn[Mn].evaluate(_n, gn);
                          }, Dt;
                        }(rt)).load((yield this._pendingData).features) : (wt = yield this._pendingData, Bt = rt.geojsonVtOptions, new dn(wt, Bt)), this.loaded = {};
                        const At = {};
                        if (pt) {
                          const Dt = pt.finish();
                          Dt && (At.resourceTiming = {}, At.resourceTiming[rt.source] = JSON.parse(JSON.stringify(Dt)));
                        }
                        return At;
                      } catch (At) {
                        if (delete this._pendingRequest, ne.bA(At)) return { abandoned: !0 };
                        throw At;
                      }
                      var wt, Bt;
                    });
                  }
                  getData() {
                    return ne._(this, void 0, void 0, function* () {
                      return this._pendingData;
                    });
                  }
                  reloadTile(rt) {
                    const ft = this.loaded, pt = rt.uid;
                    return ft && ft[pt] ? super.reloadTile(rt) : this.loadTile(rt);
                  }
                  loadAndProcessGeoJSON(rt, ft) {
                    return ne._(this, void 0, void 0, function* () {
                      let pt = yield this.loadGeoJSON(rt, ft);
                      if (delete this._pendingRequest, typeof pt != "object") throw new Error(`Input data given to '${rt.source}' is not a valid GeoJSON object.`);
                      if (he(pt, !0), rt.filter) {
                        const wt = ne.bB(rt.filter, { type: "boolean", "property-type": "data-driven", overridable: !1, transition: !1 });
                        if (wt.result === "error") throw new Error(wt.value.map((At) => `${At.key}: ${At.message}`).join(", "));
                        pt = { type: "FeatureCollection", features: pt.features.filter((At) => wt.value.evaluate({ zoom: 0 }, At)) };
                      }
                      return pt;
                    });
                  }
                  loadGeoJSON(rt, ft) {
                    return ne._(this, void 0, void 0, function* () {
                      const { promoteId: pt } = rt;
                      if (rt.request) {
                        const wt = yield ne.h(rt.request, ft);
                        return this._dataUpdateable = cn(wt.data, pt) ? wn(wt.data, pt) : void 0, wt.data;
                      }
                      if (typeof rt.data == "string") try {
                        const wt = JSON.parse(rt.data);
                        return this._dataUpdateable = cn(wt, pt) ? wn(wt, pt) : void 0, wt;
                      } catch {
                        throw new Error(`Input data given to '${rt.source}' is not a valid GeoJSON object.`);
                      }
                      if (!rt.dataDiff) throw new Error(`Input data given to '${rt.source}' is not a valid GeoJSON object.`);
                      if (!this._dataUpdateable) throw new Error(`Cannot update existing geojson data in ${rt.source}`);
                      return function(wt, Bt, At) {
                        var Dt, Ot, un, yn;
                        if (Bt.removeAll && wt.clear(), Bt.remove) for (const _n of Bt.remove) wt.delete(_n);
                        if (Bt.add) for (const _n of Bt.add) {
                          const gn = jt(_n, At);
                          gn != null && wt.set(gn, _n);
                        }
                        if (Bt.update) for (const _n of Bt.update) {
                          let gn = wt.get(_n.id);
                          if (gn == null) continue;
                          const bn = _n.newGeometry || _n.removeAllProperties, an = !_n.removeAllProperties && (((Dt = _n.removeProperties) === null || Dt === void 0 ? void 0 : Dt.length) > 0 || ((Ot = _n.addOrUpdateProperties) === null || Ot === void 0 ? void 0 : Ot.length) > 0);
                          if ((bn || an) && (gn = Object.assign({}, gn), wt.set(_n.id, gn), an && (gn.properties = Object.assign({}, gn.properties))), _n.newGeometry && (gn.geometry = _n.newGeometry), _n.removeAllProperties) gn.properties = {};
                          else if (((un = _n.removeProperties) === null || un === void 0 ? void 0 : un.length) > 0) for (const Zt of _n.removeProperties) Object.prototype.hasOwnProperty.call(gn.properties, Zt) && delete gn.properties[Zt];
                          if (((yn = _n.addOrUpdateProperties) === null || yn === void 0 ? void 0 : yn.length) > 0) for (const { key: Zt, value: Mn } of _n.addOrUpdateProperties) gn.properties[Zt] = Mn;
                        }
                      }(this._dataUpdateable, rt.dataDiff, pt), { type: "FeatureCollection", features: Array.from(this._dataUpdateable.values()) };
                    });
                  }
                  removeSource(rt) {
                    return ne._(this, void 0, void 0, function* () {
                      this._pendingRequest && this._pendingRequest.abort();
                    });
                  }
                  getClusterExpansionZoom(rt) {
                    return this._geoJSONIndex.getClusterExpansionZoom(rt.clusterId);
                  }
                  getClusterChildren(rt) {
                    return this._geoJSONIndex.getChildren(rt.clusterId);
                  }
                  getClusterLeaves(rt) {
                    return this._geoJSONIndex.getLeaves(rt.clusterId, rt.limit, rt.offset);
                  }
                }
                class Pn {
                  constructor(rt) {
                    this.self = rt, this.actor = new ne.F(rt), this.layerIndexes = {}, this.availableImages = {}, this.workerSources = {}, this.demWorkerSources = {}, this.externalWorkerSourceTypes = {}, this.self.registerWorkerSource = (ft, pt) => {
                      if (this.externalWorkerSourceTypes[ft]) throw new Error(`Worker source with name "${ft}" already registered.`);
                      this.externalWorkerSourceTypes[ft] = pt;
                    }, this.self.addProtocol = ne.bh, this.self.removeProtocol = ne.bi, this.self.registerRTLTextPlugin = (ft) => {
                      if (ne.bC.isParsed()) throw new Error("RTL text plugin already registered.");
                      ne.bC.setMethods(ft);
                    }, this.actor.registerMessageHandler("LDT", (ft, pt) => this._getDEMWorkerSource(ft, pt.source).loadTile(pt)), this.actor.registerMessageHandler("RDT", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      this._getDEMWorkerSource(ft, pt.source).removeTile(pt);
                    })), this.actor.registerMessageHandler("GCEZ", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      return this._getWorkerSource(ft, pt.type, pt.source).getClusterExpansionZoom(pt);
                    })), this.actor.registerMessageHandler("GCC", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      return this._getWorkerSource(ft, pt.type, pt.source).getClusterChildren(pt);
                    })), this.actor.registerMessageHandler("GCL", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      return this._getWorkerSource(ft, pt.type, pt.source).getClusterLeaves(pt);
                    })), this.actor.registerMessageHandler("LD", (ft, pt) => this._getWorkerSource(ft, pt.type, pt.source).loadData(pt)), this.actor.registerMessageHandler("GD", (ft, pt) => this._getWorkerSource(ft, pt.type, pt.source).getData()), this.actor.registerMessageHandler("LT", (ft, pt) => this._getWorkerSource(ft, pt.type, pt.source).loadTile(pt)), this.actor.registerMessageHandler("RT", (ft, pt) => this._getWorkerSource(ft, pt.type, pt.source).reloadTile(pt)), this.actor.registerMessageHandler("AT", (ft, pt) => this._getWorkerSource(ft, pt.type, pt.source).abortTile(pt)), this.actor.registerMessageHandler("RMT", (ft, pt) => this._getWorkerSource(ft, pt.type, pt.source).removeTile(pt)), this.actor.registerMessageHandler("RS", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      if (!this.workerSources[ft] || !this.workerSources[ft][pt.type] || !this.workerSources[ft][pt.type][pt.source]) return;
                      const wt = this.workerSources[ft][pt.type][pt.source];
                      delete this.workerSources[ft][pt.type][pt.source], wt.removeSource !== void 0 && wt.removeSource(pt);
                    })), this.actor.registerMessageHandler("RM", (ft) => ne._(this, void 0, void 0, function* () {
                      delete this.layerIndexes[ft], delete this.availableImages[ft], delete this.workerSources[ft], delete this.demWorkerSources[ft];
                    })), this.actor.registerMessageHandler("SR", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      this.referrer = pt;
                    })), this.actor.registerMessageHandler("SRPS", (ft, pt) => this._syncRTLPluginState(ft, pt)), this.actor.registerMessageHandler("IS", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      this.self.importScripts(pt);
                    })), this.actor.registerMessageHandler("SI", (ft, pt) => this._setImages(ft, pt)), this.actor.registerMessageHandler("UL", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      this._getLayerIndex(ft).update(pt.layers, pt.removedIds);
                    })), this.actor.registerMessageHandler("SL", (ft, pt) => ne._(this, void 0, void 0, function* () {
                      this._getLayerIndex(ft).replace(pt);
                    }));
                  }
                  _setImages(rt, ft) {
                    return ne._(this, void 0, void 0, function* () {
                      this.availableImages[rt] = ft;
                      for (const pt in this.workerSources[rt]) {
                        const wt = this.workerSources[rt][pt];
                        for (const Bt in wt) wt[Bt].availableImages = ft;
                      }
                    });
                  }
                  _syncRTLPluginState(rt, ft) {
                    return ne._(this, void 0, void 0, function* () {
                      if (ne.bC.isParsed()) return ne.bC.getState();
                      if (ft.pluginStatus !== "loading") return ne.bC.setState(ft), ft;
                      const pt = ft.pluginURL;
                      if (this.self.importScripts(pt), ne.bC.isParsed()) {
                        const wt = { pluginStatus: "loaded", pluginURL: pt };
                        return ne.bC.setState(wt), wt;
                      }
                      throw ne.bC.setState({ pluginStatus: "error", pluginURL: "" }), new Error(`RTL Text Plugin failed to import scripts from ${pt}`);
                    });
                  }
                  _getAvailableImages(rt) {
                    let ft = this.availableImages[rt];
                    return ft || (ft = []), ft;
                  }
                  _getLayerIndex(rt) {
                    let ft = this.layerIndexes[rt];
                    return ft || (ft = this.layerIndexes[rt] = new ee()), ft;
                  }
                  _getWorkerSource(rt, ft, pt) {
                    if (this.workerSources[rt] || (this.workerSources[rt] = {}), this.workerSources[rt][ft] || (this.workerSources[rt][ft] = {}), !this.workerSources[rt][ft][pt]) {
                      const wt = { sendAsync: (Bt, At) => (Bt.targetMapId = rt, this.actor.sendAsync(Bt, At)) };
                      switch (ft) {
                        case "vector":
                          this.workerSources[rt][ft][pt] = new ce(wt, this._getLayerIndex(rt), this._getAvailableImages(rt));
                          break;
                        case "geojson":
                          this.workerSources[rt][ft][pt] = new Rn(wt, this._getLayerIndex(rt), this._getAvailableImages(rt));
                          break;
                        default:
                          this.workerSources[rt][ft][pt] = new this.externalWorkerSourceTypes[ft](wt, this._getLayerIndex(rt), this._getAvailableImages(rt));
                      }
                    }
                    return this.workerSources[rt][ft][pt];
                  }
                  _getDEMWorkerSource(rt, ft) {
                    return this.demWorkerSources[rt] || (this.demWorkerSources[rt] = {}), this.demWorkerSources[rt][ft] || (this.demWorkerSources[rt][ft] = new le()), this.demWorkerSources[rt][ft];
                  }
                }
                return ne.i(self) && (self.worker = new Pn(self)), Pn;
              }), te("index", 0, function(ne, ee) {
                var ae = "4.5.2";
                let oe, se;
                const ce = { now: typeof performance < "u" && performance && performance.now ? performance.now.bind(performance) : Date.now.bind(Date), frameAsync(Fe) {
                  return new Promise((Te, Ae) => {
                    const Oe = requestAnimationFrame(Te);
                    Fe.signal.addEventListener("abort", () => {
                      cancelAnimationFrame(Oe), Ae(ee.c());
                    });
                  });
                }, getImageData(Fe, Te = 0) {
                  return this.getImageCanvasContext(Fe).getImageData(-Te, -Te, Fe.width + 2 * Te, Fe.height + 2 * Te);
                }, getImageCanvasContext(Fe) {
                  const Te = window.document.createElement("canvas"), Ae = Te.getContext("2d", { willReadFrequently: !0 });
                  if (!Ae) throw new Error("failed to create canvas 2d context");
                  return Te.width = Fe.width, Te.height = Fe.height, Ae.drawImage(Fe, 0, 0, Fe.width, Fe.height), Ae;
                }, resolveURL(Fe) {
                  return oe || (oe = document.createElement("a")), oe.href = Fe, oe.href;
                }, hardwareConcurrency: typeof navigator < "u" && navigator.hardwareConcurrency || 4, get prefersReducedMotion() {
                  return !!matchMedia && (se == null && (se = matchMedia("(prefers-reduced-motion: reduce)")), se.matches);
                } };
                class le {
                  static testProp(Te) {
                    if (!le.docStyle) return Te[0];
                    for (let Ae = 0; Ae < Te.length; Ae++) if (Te[Ae] in le.docStyle) return Te[Ae];
                    return Te[0];
                  }
                  static create(Te, Ae, Oe) {
                    const Be = window.document.createElement(Te);
                    return Ae !== void 0 && (Be.className = Ae), Oe && Oe.appendChild(Be), Be;
                  }
                  static createNS(Te, Ae) {
                    return window.document.createElementNS(Te, Ae);
                  }
                  static disableDrag() {
                    le.docStyle && le.selectProp && (le.userSelect = le.docStyle[le.selectProp], le.docStyle[le.selectProp] = "none");
                  }
                  static enableDrag() {
                    le.docStyle && le.selectProp && (le.docStyle[le.selectProp] = le.userSelect);
                  }
                  static setTransform(Te, Ae) {
                    Te.style[le.transformProp] = Ae;
                  }
                  static addEventListener(Te, Ae, Oe, Be = {}) {
                    "passive" in Be ? Te.addEventListener(Ae, Oe, Be) : Te.addEventListener(Ae, Oe, Be.capture);
                  }
                  static removeEventListener(Te, Ae, Oe, Be = {}) {
                    "passive" in Be ? Te.removeEventListener(Ae, Oe, Be) : Te.removeEventListener(Ae, Oe, Be.capture);
                  }
                  static suppressClickInternal(Te) {
                    Te.preventDefault(), Te.stopPropagation(), window.removeEventListener("click", le.suppressClickInternal, !0);
                  }
                  static suppressClick() {
                    window.addEventListener("click", le.suppressClickInternal, !0), window.setTimeout(() => {
                      window.removeEventListener("click", le.suppressClickInternal, !0);
                    }, 0);
                  }
                  static getScale(Te) {
                    const Ae = Te.getBoundingClientRect();
                    return { x: Ae.width / Te.offsetWidth || 1, y: Ae.height / Te.offsetHeight || 1, boundingClientRect: Ae };
                  }
                  static getPoint(Te, Ae, Oe) {
                    const Be = Ae.boundingClientRect;
                    return new ee.P((Oe.clientX - Be.left) / Ae.x - Te.clientLeft, (Oe.clientY - Be.top) / Ae.y - Te.clientTop);
                  }
                  static mousePos(Te, Ae) {
                    const Oe = le.getScale(Te);
                    return le.getPoint(Te, Oe, Ae);
                  }
                  static touchPos(Te, Ae) {
                    const Oe = [], Be = le.getScale(Te);
                    for (let Ue = 0; Ue < Ae.length; Ue++) Oe.push(le.getPoint(Te, Be, Ae[Ue]));
                    return Oe;
                  }
                  static mouseButton(Te) {
                    return Te.button;
                  }
                  static remove(Te) {
                    Te.parentNode && Te.parentNode.removeChild(Te);
                  }
                }
                le.docStyle = typeof window < "u" && window.document && window.document.documentElement.style, le.selectProp = le.testProp(["userSelect", "MozUserSelect", "WebkitUserSelect", "msUserSelect"]), le.transformProp = le.testProp(["transform", "WebkitTransform"]);
                const ue = { supported: !1, testSupport: function(Fe) {
                  !he && fe && (me ? pe(Fe) : de = Fe);
                } };
                let de, fe, he = !1, me = !1;
                function pe(Fe) {
                  const Te = Fe.createTexture();
                  Fe.bindTexture(Fe.TEXTURE_2D, Te);
                  try {
                    if (Fe.texImage2D(Fe.TEXTURE_2D, 0, Fe.RGBA, Fe.RGBA, Fe.UNSIGNED_BYTE, fe), Fe.isContextLost()) return;
                    ue.supported = !0;
                  } catch {
                  }
                  Fe.deleteTexture(Te), he = !0;
                }
                var ye;
                typeof document < "u" && (fe = document.createElement("img"), fe.onload = () => {
                  de && pe(de), de = null, me = !0;
                }, fe.onerror = () => {
                  he = !0, de = null;
                }, fe.src = "data:image/webp;base64,UklGRh4AAABXRUJQVlA4TBEAAAAvAQAAAAfQ//73v/+BiOh/AAA="), function(Fe) {
                  let Te, Ae, Oe, Be;
                  Fe.resetRequestQueue = () => {
                    Te = [], Ae = 0, Oe = 0, Be = {};
                  }, Fe.addThrottleControl = (Et) => {
                    const Mt = Oe++;
                    return Be[Mt] = Et, Mt;
                  }, Fe.removeThrottleControl = (Et) => {
                    delete Be[Et], nt();
                  }, Fe.getImage = (Et, Mt, qt = !0) => new Promise((tn, xn) => {
                    ue.supported && (Et.headers || (Et.headers = {}), Et.headers.accept = "image/webp,*/*"), ee.e(Et, { type: "image" });
                    const An = { abortController: Mt, requestParameters: Et, supportImageRefresh: qt, state: "queued", onError: (On) => {
                      xn(On);
                    }, onSuccess: (On) => {
                      tn(On);
                    } };
                    Te.push(An), nt();
                  });
                  const Ue = (Et) => ee._(this, void 0, void 0, function* () {
                    Et.state = "running";
                    const { requestParameters: Mt, supportImageRefresh: qt, onError: tn, onSuccess: xn, abortController: An } = Et, On = qt === !1 && !ee.i(self) && !ee.g(Mt.url) && (!Mt.headers || Object.keys(Mt.headers).reduce((Vn, sr) => Vn && sr === "accept", !0));
                    Ae++;
                    const Bn = On ? ht(Mt, An) : ee.m(Mt, An);
                    try {
                      const Vn = yield Bn;
                      delete Et.abortController, Et.state = "completed", Vn.data instanceof HTMLImageElement || ee.b(Vn.data) ? xn(Vn) : Vn.data && xn({ data: yield (qn = Vn.data, typeof createImageBitmap == "function" ? ee.d(qn) : ee.f(qn)), cacheControl: Vn.cacheControl, expires: Vn.expires });
                    } catch (Vn) {
                      delete Et.abortController, tn(Vn);
                    } finally {
                      Ae--, nt();
                    }
                    var qn;
                  }), nt = () => {
                    const Et = (() => {
                      for (const Mt of Object.keys(Be)) if (Be[Mt]()) return !0;
                      return !1;
                    })() ? ee.a.MAX_PARALLEL_IMAGE_REQUESTS_PER_FRAME : ee.a.MAX_PARALLEL_IMAGE_REQUESTS;
                    for (let Mt = Ae; Mt < Et && Te.length > 0; Mt++) {
                      const qt = Te.shift();
                      qt.abortController.signal.aborted ? Mt-- : Ue(qt);
                    }
                  }, ht = (Et, Mt) => new Promise((qt, tn) => {
                    const xn = new Image(), An = Et.url, On = Et.credentials;
                    On && On === "include" ? xn.crossOrigin = "use-credentials" : (On && On === "same-origin" || !ee.s(An)) && (xn.crossOrigin = "anonymous"), Mt.signal.addEventListener("abort", () => {
                      xn.src = "", tn(ee.c());
                    }), xn.fetchPriority = "high", xn.onload = () => {
                      xn.onerror = xn.onload = null, qt({ data: xn });
                    }, xn.onerror = () => {
                      xn.onerror = xn.onload = null, Mt.signal.aborted || tn(new Error("Could not load image. Please make sure to use a supported image type such as PNG or JPEG. Note that SVGs are not supported."));
                    }, xn.src = An;
                  });
                }(ye || (ye = {})), ye.resetRequestQueue();
                class ge {
                  constructor(Te) {
                    this._transformRequestFn = Te;
                  }
                  transformRequest(Te, Ae) {
                    return this._transformRequestFn && this._transformRequestFn(Te, Ae) || { url: Te };
                  }
                  setTransformRequest(Te) {
                    this._transformRequestFn = Te;
                  }
                }
                function ve(Fe) {
                  var Te = new ee.A(3);
                  return Te[0] = Fe[0], Te[1] = Fe[1], Te[2] = Fe[2], Te;
                }
                var be, Ce = function(Fe, Te, Ae) {
                  return Fe[0] = Te[0] - Ae[0], Fe[1] = Te[1] - Ae[1], Fe[2] = Te[2] - Ae[2], Fe;
                };
                be = new ee.A(3), ee.A != Float32Array && (be[0] = 0, be[1] = 0, be[2] = 0);
                var Se = function(Fe) {
                  var Te = Fe[0], Ae = Fe[1];
                  return Te * Te + Ae * Ae;
                };
                function we(Fe) {
                  const Te = [];
                  if (typeof Fe == "string") Te.push({ id: "default", url: Fe });
                  else if (Fe && Fe.length > 0) {
                    const Ae = [];
                    for (const { id: Oe, url: Be } of Fe) {
                      const Ue = `${Oe}${Be}`;
                      Ae.indexOf(Ue) === -1 && (Ae.push(Ue), Te.push({ id: Oe, url: Be }));
                    }
                  }
                  return Te;
                }
                function Ee(Fe, Te, Ae) {
                  const Oe = Fe.split("?");
                  return Oe[0] += `${Te}${Ae}`, Oe.join("?");
                }
                function xe(Fe, Te, Ae, Oe) {
                  return ee._(this, void 0, void 0, function* () {
                    const Be = we(Fe), Ue = Ae > 1 ? "@2x" : "", nt = {}, ht = {};
                    for (const { id: Et, url: Mt } of Be) {
                      const qt = Te.transformRequest(Ee(Mt, Ue, ".json"), "SpriteJSON");
                      nt[Et] = ee.h(qt, Oe);
                      const tn = Te.transformRequest(Ee(Mt, Ue, ".png"), "SpriteImage");
                      ht[Et] = ye.getImage(tn, Oe);
                    }
                    return yield Promise.all([...Object.values(nt), ...Object.values(ht)]), function(Et, Mt) {
                      return ee._(this, void 0, void 0, function* () {
                        const qt = {};
                        for (const tn in Et) {
                          qt[tn] = {};
                          const xn = ce.getImageCanvasContext((yield Mt[tn]).data), An = (yield Et[tn]).data;
                          for (const On in An) {
                            const { width: Bn, height: qn, x: Vn, y: sr, sdf: Un, pixelRatio: wr, stretchX: Cr, stretchY: Tr, content: ii, textFitWidth: wi, textFitHeight: Wi } = An[On], Yi = { width: Bn, height: qn, x: Vn, y: sr, context: xn };
                            qt[tn][On] = { data: null, pixelRatio: wr, sdf: Un, stretchX: Cr, stretchY: Tr, content: ii, textFitWidth: wi, textFitHeight: Wi, spriteData: Yi };
                          }
                        }
                        return qt;
                      });
                    }(nt, ht);
                  });
                }
                (function() {
                  var Fe = new ee.A(2);
                  ee.A != Float32Array && (Fe[0] = 0, Fe[1] = 0);
                })();
                class Le {
                  constructor(Te, Ae, Oe, Be) {
                    this.context = Te, this.format = Oe, this.texture = Te.gl.createTexture(), this.update(Ae, Be);
                  }
                  update(Te, Ae, Oe) {
                    const { width: Be, height: Ue } = Te, nt = !(this.size && this.size[0] === Be && this.size[1] === Ue || Oe), { context: ht } = this, { gl: Et } = ht;
                    if (this.useMipmap = !!(Ae && Ae.useMipmap), Et.bindTexture(Et.TEXTURE_2D, this.texture), ht.pixelStoreUnpackFlipY.set(!1), ht.pixelStoreUnpack.set(1), ht.pixelStoreUnpackPremultiplyAlpha.set(this.format === Et.RGBA && (!Ae || Ae.premultiply !== !1)), nt) this.size = [Be, Ue], Te instanceof HTMLImageElement || Te instanceof HTMLCanvasElement || Te instanceof HTMLVideoElement || Te instanceof ImageData || ee.b(Te) ? Et.texImage2D(Et.TEXTURE_2D, 0, this.format, this.format, Et.UNSIGNED_BYTE, Te) : Et.texImage2D(Et.TEXTURE_2D, 0, this.format, Be, Ue, 0, this.format, Et.UNSIGNED_BYTE, Te.data);
                    else {
                      const { x: Mt, y: qt } = Oe || { x: 0, y: 0 };
                      Te instanceof HTMLImageElement || Te instanceof HTMLCanvasElement || Te instanceof HTMLVideoElement || Te instanceof ImageData || ee.b(Te) ? Et.texSubImage2D(Et.TEXTURE_2D, 0, Mt, qt, Et.RGBA, Et.UNSIGNED_BYTE, Te) : Et.texSubImage2D(Et.TEXTURE_2D, 0, Mt, qt, Be, Ue, Et.RGBA, Et.UNSIGNED_BYTE, Te.data);
                    }
                    this.useMipmap && this.isSizePowerOfTwo() && Et.generateMipmap(Et.TEXTURE_2D);
                  }
                  bind(Te, Ae, Oe) {
                    const { context: Be } = this, { gl: Ue } = Be;
                    Ue.bindTexture(Ue.TEXTURE_2D, this.texture), Oe !== Ue.LINEAR_MIPMAP_NEAREST || this.isSizePowerOfTwo() || (Oe = Ue.LINEAR), Te !== this.filter && (Ue.texParameteri(Ue.TEXTURE_2D, Ue.TEXTURE_MAG_FILTER, Te), Ue.texParameteri(Ue.TEXTURE_2D, Ue.TEXTURE_MIN_FILTER, Oe || Te), this.filter = Te), Ae !== this.wrap && (Ue.texParameteri(Ue.TEXTURE_2D, Ue.TEXTURE_WRAP_S, Ae), Ue.texParameteri(Ue.TEXTURE_2D, Ue.TEXTURE_WRAP_T, Ae), this.wrap = Ae);
                  }
                  isSizePowerOfTwo() {
                    return this.size[0] === this.size[1] && Math.log(this.size[0]) / Math.LN2 % 1 == 0;
                  }
                  destroy() {
                    const { gl: Te } = this.context;
                    Te.deleteTexture(this.texture), this.texture = null;
                  }
                }
                function Qe(Fe) {
                  const { userImage: Te } = Fe;
                  return !!(Te && Te.render && Te.render()) && (Fe.data.replace(new Uint8Array(Te.data.buffer)), !0);
                }
                class ke extends ee.E {
                  constructor() {
                    super(), this.images = {}, this.updatedImages = {}, this.callbackDispatchedThisFrame = {}, this.loaded = !1, this.requestors = [], this.patterns = {}, this.atlasImage = new ee.R({ width: 1, height: 1 }), this.dirty = !0;
                  }
                  isLoaded() {
                    return this.loaded;
                  }
                  setLoaded(Te) {
                    if (this.loaded !== Te && (this.loaded = Te, Te)) {
                      for (const { ids: Ae, promiseResolve: Oe } of this.requestors) Oe(this._getImagesForIds(Ae));
                      this.requestors = [];
                    }
                  }
                  getImage(Te) {
                    const Ae = this.images[Te];
                    if (Ae && !Ae.data && Ae.spriteData) {
                      const Oe = Ae.spriteData;
                      Ae.data = new ee.R({ width: Oe.width, height: Oe.height }, Oe.context.getImageData(Oe.x, Oe.y, Oe.width, Oe.height).data), Ae.spriteData = null;
                    }
                    return Ae;
                  }
                  addImage(Te, Ae) {
                    if (this.images[Te]) throw new Error(`Image id ${Te} already exist, use updateImage instead`);
                    this._validate(Te, Ae) && (this.images[Te] = Ae);
                  }
                  _validate(Te, Ae) {
                    let Oe = !0;
                    const Be = Ae.data || Ae.spriteData;
                    return this._validateStretch(Ae.stretchX, Be && Be.width) || (this.fire(new ee.j(new Error(`Image "${Te}" has invalid "stretchX" value`))), Oe = !1), this._validateStretch(Ae.stretchY, Be && Be.height) || (this.fire(new ee.j(new Error(`Image "${Te}" has invalid "stretchY" value`))), Oe = !1), this._validateContent(Ae.content, Ae) || (this.fire(new ee.j(new Error(`Image "${Te}" has invalid "content" value`))), Oe = !1), Oe;
                  }
                  _validateStretch(Te, Ae) {
                    if (!Te) return !0;
                    let Oe = 0;
                    for (const Be of Te) {
                      if (Be[0] < Oe || Be[1] < Be[0] || Ae < Be[1]) return !1;
                      Oe = Be[1];
                    }
                    return !0;
                  }
                  _validateContent(Te, Ae) {
                    if (!Te) return !0;
                    if (Te.length !== 4) return !1;
                    const Oe = Ae.spriteData, Be = Oe && Oe.width || Ae.data.width, Ue = Oe && Oe.height || Ae.data.height;
                    return !(Te[0] < 0 || Be < Te[0] || Te[1] < 0 || Ue < Te[1] || Te[2] < 0 || Be < Te[2] || Te[3] < 0 || Ue < Te[3] || Te[2] < Te[0] || Te[3] < Te[1]);
                  }
                  updateImage(Te, Ae, Oe = !0) {
                    const Be = this.getImage(Te);
                    if (Oe && (Be.data.width !== Ae.data.width || Be.data.height !== Ae.data.height)) throw new Error(`size mismatch between old image (${Be.data.width}x${Be.data.height}) and new image (${Ae.data.width}x${Ae.data.height}).`);
                    Ae.version = Be.version + 1, this.images[Te] = Ae, this.updatedImages[Te] = !0;
                  }
                  removeImage(Te) {
                    const Ae = this.images[Te];
                    delete this.images[Te], delete this.patterns[Te], Ae.userImage && Ae.userImage.onRemove && Ae.userImage.onRemove();
                  }
                  listImages() {
                    return Object.keys(this.images);
                  }
                  getImages(Te) {
                    return new Promise((Ae, Oe) => {
                      let Be = !0;
                      if (!this.isLoaded()) for (const Ue of Te) this.images[Ue] || (Be = !1);
                      this.isLoaded() || Be ? Ae(this._getImagesForIds(Te)) : this.requestors.push({ ids: Te, promiseResolve: Ae });
                    });
                  }
                  _getImagesForIds(Te) {
                    const Ae = {};
                    for (const Oe of Te) {
                      let Be = this.getImage(Oe);
                      Be || (this.fire(new ee.k("styleimagemissing", { id: Oe })), Be = this.getImage(Oe)), Be ? Ae[Oe] = { data: Be.data.clone(), pixelRatio: Be.pixelRatio, sdf: Be.sdf, version: Be.version, stretchX: Be.stretchX, stretchY: Be.stretchY, content: Be.content, textFitWidth: Be.textFitWidth, textFitHeight: Be.textFitHeight, hasRenderCallback: !!(Be.userImage && Be.userImage.render) } : ee.w(`Image "${Oe}" could not be loaded. Please make sure you have added the image with map.addImage() or a "sprite" property in your style. You can provide missing images by listening for the "styleimagemissing" map event.`);
                    }
                    return Ae;
                  }
                  getPixelSize() {
                    const { width: Te, height: Ae } = this.atlasImage;
                    return { width: Te, height: Ae };
                  }
                  getPattern(Te) {
                    const Ae = this.patterns[Te], Oe = this.getImage(Te);
                    if (!Oe) return null;
                    if (Ae && Ae.position.version === Oe.version) return Ae.position;
                    if (Ae) Ae.position.version = Oe.version;
                    else {
                      const Be = { w: Oe.data.width + 2, h: Oe.data.height + 2, x: 0, y: 0 }, Ue = new ee.I(Be, Oe);
                      this.patterns[Te] = { bin: Be, position: Ue };
                    }
                    return this._updatePatternAtlas(), this.patterns[Te].position;
                  }
                  bind(Te) {
                    const Ae = Te.gl;
                    this.atlasTexture ? this.dirty && (this.atlasTexture.update(this.atlasImage), this.dirty = !1) : this.atlasTexture = new Le(Te, this.atlasImage, Ae.RGBA), this.atlasTexture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE);
                  }
                  _updatePatternAtlas() {
                    const Te = [];
                    for (const Ue in this.patterns) Te.push(this.patterns[Ue].bin);
                    const { w: Ae, h: Oe } = ee.p(Te), Be = this.atlasImage;
                    Be.resize({ width: Ae || 1, height: Oe || 1 });
                    for (const Ue in this.patterns) {
                      const { bin: nt } = this.patterns[Ue], ht = nt.x + 1, Et = nt.y + 1, Mt = this.getImage(Ue).data, qt = Mt.width, tn = Mt.height;
                      ee.R.copy(Mt, Be, { x: 0, y: 0 }, { x: ht, y: Et }, { width: qt, height: tn }), ee.R.copy(Mt, Be, { x: 0, y: tn - 1 }, { x: ht, y: Et - 1 }, { width: qt, height: 1 }), ee.R.copy(Mt, Be, { x: 0, y: 0 }, { x: ht, y: Et + tn }, { width: qt, height: 1 }), ee.R.copy(Mt, Be, { x: qt - 1, y: 0 }, { x: ht - 1, y: Et }, { width: 1, height: tn }), ee.R.copy(Mt, Be, { x: 0, y: 0 }, { x: ht + qt, y: Et }, { width: 1, height: tn });
                    }
                    this.dirty = !0;
                  }
                  beginFrame() {
                    this.callbackDispatchedThisFrame = {};
                  }
                  dispatchRenderCallbacks(Te) {
                    for (const Ae of Te) {
                      if (this.callbackDispatchedThisFrame[Ae]) continue;
                      this.callbackDispatchedThisFrame[Ae] = !0;
                      const Oe = this.getImage(Ae);
                      Oe || ee.w(`Image with ID: "${Ae}" was not found`), Qe(Oe) && this.updateImage(Ae, Oe);
                    }
                  }
                }
                const Me = 1e20;
                function Re(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                  for (let Mt = Te; Mt < Te + Oe; Mt++) De(Fe, Ae * Ue + Mt, Ue, Be, nt, ht, Et);
                  for (let Mt = Ae; Mt < Ae + Be; Mt++) De(Fe, Mt * Ue + Te, 1, Oe, nt, ht, Et);
                }
                function De(Fe, Te, Ae, Oe, Be, Ue, nt) {
                  Ue[0] = 0, nt[0] = -1e20, nt[1] = Me, Be[0] = Fe[Te];
                  for (let ht = 1, Et = 0, Mt = 0; ht < Oe; ht++) {
                    Be[ht] = Fe[Te + ht * Ae];
                    const qt = ht * ht;
                    do {
                      const tn = Ue[Et];
                      Mt = (Be[ht] - Be[tn] + qt - tn * tn) / (ht - tn) / 2;
                    } while (Mt <= nt[Et] && --Et > -1);
                    Et++, Ue[Et] = ht, nt[Et] = Mt, nt[Et + 1] = Me;
                  }
                  for (let ht = 0, Et = 0; ht < Oe; ht++) {
                    for (; nt[Et + 1] < ht; ) Et++;
                    const Mt = Ue[Et], qt = ht - Mt;
                    Fe[Te + ht * Ae] = Be[Mt] + qt * qt;
                  }
                }
                class Pe {
                  constructor(Te, Ae) {
                    this.requestManager = Te, this.localIdeographFontFamily = Ae, this.entries = {};
                  }
                  setURL(Te) {
                    this.url = Te;
                  }
                  getGlyphs(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      const Ae = [];
                      for (const Ue in Te) for (const nt of Te[Ue]) Ae.push(this._getAndCacheGlyphsPromise(Ue, nt));
                      const Oe = yield Promise.all(Ae), Be = {};
                      for (const { stack: Ue, id: nt, glyph: ht } of Oe) Be[Ue] || (Be[Ue] = {}), Be[Ue][nt] = ht && { id: ht.id, bitmap: ht.bitmap.clone(), metrics: ht.metrics };
                      return Be;
                    });
                  }
                  _getAndCacheGlyphsPromise(Te, Ae) {
                    return ee._(this, void 0, void 0, function* () {
                      let Oe = this.entries[Te];
                      Oe || (Oe = this.entries[Te] = { glyphs: {}, requests: {}, ranges: {} });
                      let Be = Oe.glyphs[Ae];
                      if (Be !== void 0) return { stack: Te, id: Ae, glyph: Be };
                      if (Be = this._tinySDF(Oe, Te, Ae), Be) return Oe.glyphs[Ae] = Be, { stack: Te, id: Ae, glyph: Be };
                      const Ue = Math.floor(Ae / 256);
                      if (256 * Ue > 65535) throw new Error("glyphs > 65535 not supported");
                      if (Oe.ranges[Ue]) return { stack: Te, id: Ae, glyph: Be };
                      if (!this.url) throw new Error("glyphsUrl is not set");
                      if (!Oe.requests[Ue]) {
                        const ht = Pe.loadGlyphRange(Te, Ue, this.url, this.requestManager);
                        Oe.requests[Ue] = ht;
                      }
                      const nt = yield Oe.requests[Ue];
                      for (const ht in nt) this._doesCharSupportLocalGlyph(+ht) || (Oe.glyphs[+ht] = nt[+ht]);
                      return Oe.ranges[Ue] = !0, { stack: Te, id: Ae, glyph: nt[Ae] || null };
                    });
                  }
                  _doesCharSupportLocalGlyph(Te) {
                    return !!this.localIdeographFontFamily && (ee.u["CJK Unified Ideographs"](Te) || ee.u["Hangul Syllables"](Te) || ee.u.Hiragana(Te) || ee.u.Katakana(Te));
                  }
                  _tinySDF(Te, Ae, Oe) {
                    const Be = this.localIdeographFontFamily;
                    if (!Be || !this._doesCharSupportLocalGlyph(Oe)) return;
                    let Ue = Te.tinySDF;
                    if (!Ue) {
                      let ht = "400";
                      /bold/i.test(Ae) ? ht = "900" : /medium/i.test(Ae) ? ht = "500" : /light/i.test(Ae) && (ht = "200"), Ue = Te.tinySDF = new Pe.TinySDF({ fontSize: 48, buffer: 6, radius: 16, cutoff: 0.25, fontFamily: Be, fontWeight: ht });
                    }
                    const nt = Ue.draw(String.fromCharCode(Oe));
                    return { id: Oe, bitmap: new ee.o({ width: nt.width || 60, height: nt.height || 60 }, nt.data), metrics: { width: nt.glyphWidth / 2 || 24, height: nt.glyphHeight / 2 || 24, left: nt.glyphLeft / 2 + 0.5 || 0, top: nt.glyphTop / 2 - 27.5 || -8, advance: nt.glyphAdvance / 2 || 24, isDoubleResolution: !0 } };
                  }
                }
                Pe.loadGlyphRange = function(Fe, Te, Ae, Oe) {
                  return ee._(this, void 0, void 0, function* () {
                    const Be = 256 * Te, Ue = Be + 255, nt = Oe.transformRequest(Ae.replace("{fontstack}", Fe).replace("{range}", `${Be}-${Ue}`), "Glyphs"), ht = yield ee.l(nt, new AbortController());
                    if (!ht || !ht.data) throw new Error(`Could not load glyph range. range: ${Te}, ${Be}-${Ue}`);
                    const Et = {};
                    for (const Mt of ee.n(ht.data)) Et[Mt.id] = Mt;
                    return Et;
                  });
                }, Pe.TinySDF = class {
                  constructor({ fontSize: Fe = 24, buffer: Te = 3, radius: Ae = 8, cutoff: Oe = 0.25, fontFamily: Be = "sans-serif", fontWeight: Ue = "normal", fontStyle: nt = "normal" } = {}) {
                    this.buffer = Te, this.cutoff = Oe, this.radius = Ae;
                    const ht = this.size = Fe + 4 * Te, Et = this._createCanvas(ht), Mt = this.ctx = Et.getContext("2d", { willReadFrequently: !0 });
                    Mt.font = `${nt} ${Ue} ${Fe}px ${Be}`, Mt.textBaseline = "alphabetic", Mt.textAlign = "left", Mt.fillStyle = "black", this.gridOuter = new Float64Array(ht * ht), this.gridInner = new Float64Array(ht * ht), this.f = new Float64Array(ht), this.z = new Float64Array(ht + 1), this.v = new Uint16Array(ht);
                  }
                  _createCanvas(Fe) {
                    const Te = document.createElement("canvas");
                    return Te.width = Te.height = Fe, Te;
                  }
                  draw(Fe) {
                    const { width: Te, actualBoundingBoxAscent: Ae, actualBoundingBoxDescent: Oe, actualBoundingBoxLeft: Be, actualBoundingBoxRight: Ue } = this.ctx.measureText(Fe), nt = Math.ceil(Ae), ht = Math.max(0, Math.min(this.size - this.buffer, Math.ceil(Ue - Be))), Et = Math.min(this.size - this.buffer, nt + Math.ceil(Oe)), Mt = ht + 2 * this.buffer, qt = Et + 2 * this.buffer, tn = Math.max(Mt * qt, 0), xn = new Uint8ClampedArray(tn), An = { data: xn, width: Mt, height: qt, glyphWidth: ht, glyphHeight: Et, glyphTop: nt, glyphLeft: 0, glyphAdvance: Te };
                    if (ht === 0 || Et === 0) return An;
                    const { ctx: On, buffer: Bn, gridInner: qn, gridOuter: Vn } = this;
                    On.clearRect(Bn, Bn, ht, Et), On.fillText(Fe, Bn, Bn + nt);
                    const sr = On.getImageData(Bn, Bn, ht, Et);
                    Vn.fill(Me, 0, tn), qn.fill(0, 0, tn);
                    for (let Un = 0; Un < Et; Un++) for (let wr = 0; wr < ht; wr++) {
                      const Cr = sr.data[4 * (Un * ht + wr) + 3] / 255;
                      if (Cr === 0) continue;
                      const Tr = (Un + Bn) * Mt + wr + Bn;
                      if (Cr === 1) Vn[Tr] = 0, qn[Tr] = Me;
                      else {
                        const ii = 0.5 - Cr;
                        Vn[Tr] = ii > 0 ? ii * ii : 0, qn[Tr] = ii < 0 ? ii * ii : 0;
                      }
                    }
                    Re(Vn, 0, 0, Mt, qt, Mt, this.f, this.v, this.z), Re(qn, Bn, Bn, ht, Et, Mt, this.f, this.v, this.z);
                    for (let Un = 0; Un < tn; Un++) {
                      const wr = Math.sqrt(Vn[Un]) - Math.sqrt(qn[Un]);
                      xn[Un] = Math.round(255 - 255 * (wr / this.radius + this.cutoff));
                    }
                    return An;
                  }
                };
                class $e {
                  constructor() {
                    this.specification = ee.v.light.position;
                  }
                  possiblyEvaluate(Te, Ae) {
                    return ee.y(Te.expression.evaluate(Ae));
                  }
                  interpolate(Te, Ae, Oe) {
                    return { x: ee.z.number(Te.x, Ae.x, Oe), y: ee.z.number(Te.y, Ae.y, Oe), z: ee.z.number(Te.z, Ae.z, Oe) };
                  }
                }
                const qe = "-transition";
                let Xe;
                class Ve extends ee.E {
                  constructor(Te) {
                    super(), Xe = Xe || new ee.q({ anchor: new ee.D(ee.v.light.anchor), position: new $e(), color: new ee.D(ee.v.light.color), intensity: new ee.D(ee.v.light.intensity) }), this._transitionable = new ee.T(Xe), this.setLight(Te), this._transitioning = this._transitionable.untransitioned();
                  }
                  getLight() {
                    return this._transitionable.serialize();
                  }
                  setLight(Te, Ae = {}) {
                    if (!this._validate(ee.r, Te, Ae)) for (const Oe in Te) {
                      const Be = Te[Oe];
                      Oe.endsWith(qe) ? this._transitionable.setTransition(Oe.slice(0, -11), Be) : this._transitionable.setValue(Oe, Be);
                    }
                  }
                  updateTransitions(Te) {
                    this._transitioning = this._transitionable.transitioned(Te, this._transitioning);
                  }
                  hasTransition() {
                    return this._transitioning.hasTransition();
                  }
                  recalculate(Te) {
                    this.properties = this._transitioning.possiblyEvaluate(Te);
                  }
                  _validate(Te, Ae, Oe) {
                    return (!Oe || Oe.validate !== !1) && ee.t(this, Te.call(ee.x, { value: Ae, style: { glyphs: !0, sprite: !0 }, styleSpec: ee.v }));
                  }
                }
                const Ze = new ee.q({ "sky-color": new ee.D(ee.v.sky["sky-color"]), "horizon-color": new ee.D(ee.v.sky["horizon-color"]), "fog-color": new ee.D(ee.v.sky["fog-color"]), "fog-ground-blend": new ee.D(ee.v.sky["fog-ground-blend"]), "horizon-fog-blend": new ee.D(ee.v.sky["horizon-fog-blend"]), "sky-horizon-blend": new ee.D(ee.v.sky["sky-horizon-blend"]), "atmosphere-blend": new ee.D(ee.v.sky["atmosphere-blend"]) }), Ge = "-transition";
                class ze extends ee.E {
                  constructor(Te) {
                    super(), this._transitionable = new ee.T(Ze), this.setSky(Te), this._transitioning = this._transitionable.untransitioned();
                  }
                  setSky(Te, Ae = {}) {
                    if (!this._validate(ee.B, Te, Ae)) for (const Oe in Te) {
                      const Be = Te[Oe];
                      Oe.endsWith(Ge) ? this._transitionable.setTransition(Oe.slice(0, -11), Be) : this._transitionable.setValue(Oe, Be);
                    }
                  }
                  getSky() {
                    return this._transitionable.serialize();
                  }
                  updateTransitions(Te) {
                    this._transitioning = this._transitionable.transitioned(Te, this._transitioning);
                  }
                  hasTransition() {
                    return this._transitioning.hasTransition();
                  }
                  recalculate(Te) {
                    this.properties = this._transitioning.possiblyEvaluate(Te);
                  }
                  _validate(Te, Ae, Oe = {}) {
                    return (Oe == null ? void 0 : Oe.validate) !== !1 && ee.t(this, Te.call(ee.x, ee.e({ value: Ae, style: { glyphs: !0, sprite: !0 }, styleSpec: ee.v })));
                  }
                  calculateFogBlendOpacity(Te) {
                    return Te < 60 ? 0 : Te < 70 ? (Te - 60) / 10 : 1;
                  }
                }
                class We {
                  constructor(Te, Ae) {
                    this.width = Te, this.height = Ae, this.nextRow = 0, this.data = new Uint8Array(this.width * this.height), this.dashEntry = {};
                  }
                  getDash(Te, Ae) {
                    const Oe = Te.join(",") + String(Ae);
                    return this.dashEntry[Oe] || (this.dashEntry[Oe] = this.addDash(Te, Ae)), this.dashEntry[Oe];
                  }
                  getDashRanges(Te, Ae, Oe) {
                    const Be = [];
                    let Ue = Te.length % 2 == 1 ? -Te[Te.length - 1] * Oe : 0, nt = Te[0] * Oe, ht = !0;
                    Be.push({ left: Ue, right: nt, isDash: ht, zeroLength: Te[0] === 0 });
                    let Et = Te[0];
                    for (let Mt = 1; Mt < Te.length; Mt++) {
                      ht = !ht;
                      const qt = Te[Mt];
                      Ue = Et * Oe, Et += qt, nt = Et * Oe, Be.push({ left: Ue, right: nt, isDash: ht, zeroLength: qt === 0 });
                    }
                    return Be;
                  }
                  addRoundDash(Te, Ae, Oe) {
                    const Be = Ae / 2;
                    for (let Ue = -Oe; Ue <= Oe; Ue++) {
                      const nt = this.nextRow + Oe + Ue, ht = this.width * nt;
                      let Et = 0, Mt = Te[Et];
                      for (let qt = 0; qt < this.width; qt++) {
                        qt / Mt.right > 1 && (Mt = Te[++Et]);
                        const tn = Math.abs(qt - Mt.left), xn = Math.abs(qt - Mt.right), An = Math.min(tn, xn);
                        let On;
                        const Bn = Ue / Oe * (Be + 1);
                        if (Mt.isDash) {
                          const qn = Be - Math.abs(Bn);
                          On = Math.sqrt(An * An + qn * qn);
                        } else On = Be - Math.sqrt(An * An + Bn * Bn);
                        this.data[ht + qt] = Math.max(0, Math.min(255, On + 128));
                      }
                    }
                  }
                  addRegularDash(Te) {
                    for (let ht = Te.length - 1; ht >= 0; --ht) {
                      const Et = Te[ht], Mt = Te[ht + 1];
                      Et.zeroLength ? Te.splice(ht, 1) : Mt && Mt.isDash === Et.isDash && (Mt.left = Et.left, Te.splice(ht, 1));
                    }
                    const Ae = Te[0], Oe = Te[Te.length - 1];
                    Ae.isDash === Oe.isDash && (Ae.left = Oe.left - this.width, Oe.right = Ae.right + this.width);
                    const Be = this.width * this.nextRow;
                    let Ue = 0, nt = Te[Ue];
                    for (let ht = 0; ht < this.width; ht++) {
                      ht / nt.right > 1 && (nt = Te[++Ue]);
                      const Et = Math.abs(ht - nt.left), Mt = Math.abs(ht - nt.right), qt = Math.min(Et, Mt), tn = nt.isDash ? qt : -qt;
                      this.data[Be + ht] = Math.max(0, Math.min(255, tn + 128));
                    }
                  }
                  addDash(Te, Ae) {
                    const Oe = Ae ? 7 : 0, Be = 2 * Oe + 1;
                    if (this.nextRow + Be > this.height) return ee.w("LineAtlas out of space"), null;
                    let Ue = 0;
                    for (let ht = 0; ht < Te.length; ht++) Ue += Te[ht];
                    if (Ue !== 0) {
                      const ht = this.width / Ue, Et = this.getDashRanges(Te, this.width, ht);
                      Ae ? this.addRoundDash(Et, ht, Oe) : this.addRegularDash(Et);
                    }
                    const nt = { y: (this.nextRow + Oe + 0.5) / this.height, height: 2 * Oe / this.height, width: Ue };
                    return this.nextRow += Be, this.dirty = !0, nt;
                  }
                  bind(Te) {
                    const Ae = Te.gl;
                    this.texture ? (Ae.bindTexture(Ae.TEXTURE_2D, this.texture), this.dirty && (this.dirty = !1, Ae.texSubImage2D(Ae.TEXTURE_2D, 0, 0, 0, this.width, this.height, Ae.ALPHA, Ae.UNSIGNED_BYTE, this.data))) : (this.texture = Ae.createTexture(), Ae.bindTexture(Ae.TEXTURE_2D, this.texture), Ae.texParameteri(Ae.TEXTURE_2D, Ae.TEXTURE_WRAP_S, Ae.REPEAT), Ae.texParameteri(Ae.TEXTURE_2D, Ae.TEXTURE_WRAP_T, Ae.REPEAT), Ae.texParameteri(Ae.TEXTURE_2D, Ae.TEXTURE_MIN_FILTER, Ae.LINEAR), Ae.texParameteri(Ae.TEXTURE_2D, Ae.TEXTURE_MAG_FILTER, Ae.LINEAR), Ae.texImage2D(Ae.TEXTURE_2D, 0, Ae.ALPHA, this.width, this.height, 0, Ae.ALPHA, Ae.UNSIGNED_BYTE, this.data));
                  }
                }
                const at = "maplibre_preloaded_worker_pool";
                class ct {
                  constructor() {
                    this.active = {};
                  }
                  acquire(Te) {
                    if (!this.workers) for (this.workers = []; this.workers.length < ct.workerCount; ) this.workers.push(new Worker(ee.a.WORKER_URL));
                    return this.active[Te] = !0, this.workers.slice();
                  }
                  release(Te) {
                    delete this.active[Te], this.numActive() === 0 && (this.workers.forEach((Ae) => {
                      Ae.terminate();
                    }), this.workers = null);
                  }
                  isPreloaded() {
                    return !!this.active[at];
                  }
                  numActive() {
                    return Object.keys(this.active).length;
                  }
                }
                const et = Math.floor(ce.hardwareConcurrency / 2);
                let dt, Je;
                function it() {
                  return dt || (dt = new ct()), dt;
                }
                ct.workerCount = ee.C(globalThis) ? Math.max(Math.min(et, 3), 1) : 1;
                class st {
                  constructor(Te, Ae) {
                    this.workerPool = Te, this.actors = [], this.currentActor = 0, this.id = Ae;
                    const Oe = this.workerPool.acquire(Ae);
                    for (let Be = 0; Be < Oe.length; Be++) {
                      const Ue = Oe[Be], nt = new ee.F(Ue, Ae);
                      nt.name = `Worker ${Be}`, this.actors.push(nt);
                    }
                    if (!this.actors.length) throw new Error("No actors found");
                  }
                  broadcast(Te, Ae) {
                    const Oe = [];
                    for (const Be of this.actors) Oe.push(Be.sendAsync({ type: Te, data: Ae }));
                    return Promise.all(Oe);
                  }
                  getActor() {
                    return this.currentActor = (this.currentActor + 1) % this.actors.length, this.actors[this.currentActor];
                  }
                  remove(Te = !0) {
                    this.actors.forEach((Ae) => {
                      Ae.remove();
                    }), this.actors = [], Te && this.workerPool.release(this.id);
                  }
                  registerMessageHandler(Te, Ae) {
                    for (const Oe of this.actors) Oe.registerMessageHandler(Te, Ae);
                  }
                }
                function gt() {
                  return Je || (Je = new st(it(), ee.G), Je.registerMessageHandler("GR", (Fe, Te, Ae) => ee.m(Te, Ae))), Je;
                }
                function bt(Fe, Te) {
                  const Ae = ee.H();
                  return ee.J(Ae, Ae, [1, 1, 0]), ee.K(Ae, Ae, [0.5 * Fe.width, 0.5 * Fe.height, 1]), ee.L(Ae, Ae, Fe.calculatePosMatrix(Te.toUnwrapped()));
                }
                function yt(Fe, Te, Ae, Oe, Be, Ue) {
                  const nt = function(tn, xn, An) {
                    if (tn) for (const On of tn) {
                      const Bn = xn[On];
                      if (Bn && Bn.source === An && Bn.type === "fill-extrusion") return !0;
                    }
                    else for (const On in xn) {
                      const Bn = xn[On];
                      if (Bn.source === An && Bn.type === "fill-extrusion") return !0;
                    }
                    return !1;
                  }(Be && Be.layers, Te, Fe.id), ht = Ue.maxPitchScaleFactor(), Et = Fe.tilesIn(Oe, ht, nt);
                  Et.sort(St);
                  const Mt = [];
                  for (const tn of Et) Mt.push({ wrappedTileID: tn.tileID.wrapped().key, queryResults: tn.tile.queryRenderedFeatures(Te, Ae, Fe._state, tn.queryGeometry, tn.cameraQueryGeometry, tn.scale, Be, Ue, ht, bt(Fe.transform, tn.tileID)) });
                  const qt = function(tn) {
                    const xn = {}, An = {};
                    for (const On of tn) {
                      const Bn = On.queryResults, qn = On.wrappedTileID, Vn = An[qn] = An[qn] || {};
                      for (const sr in Bn) {
                        const Un = Bn[sr], wr = Vn[sr] = Vn[sr] || {}, Cr = xn[sr] = xn[sr] || [];
                        for (const Tr of Un) wr[Tr.featureIndex] || (wr[Tr.featureIndex] = !0, Cr.push(Tr));
                      }
                    }
                    return xn;
                  }(Mt);
                  for (const tn in qt) qt[tn].forEach((xn) => {
                    const An = xn.feature, On = Fe.getFeatureState(An.layer["source-layer"], An.id);
                    An.source = An.layer.source, An.layer["source-layer"] && (An.sourceLayer = An.layer["source-layer"]), An.state = On;
                  });
                  return qt;
                }
                function St(Fe, Te) {
                  const Ae = Fe.tileID, Oe = Te.tileID;
                  return Ae.overscaledZ - Oe.overscaledZ || Ae.canonical.y - Oe.canonical.y || Ae.wrap - Oe.wrap || Ae.canonical.x - Oe.canonical.x;
                }
                function Lt(Fe, Te, Ae) {
                  return ee._(this, void 0, void 0, function* () {
                    let Oe = Fe;
                    if (Fe.url ? Oe = (yield ee.h(Te.transformRequest(Fe.url, "Source"), Ae)).data : yield ce.frameAsync(Ae), !Oe) return null;
                    const Be = ee.M(ee.e(Oe, Fe), ["tiles", "minzoom", "maxzoom", "attribution", "bounds", "scheme", "tileSize", "encoding"]);
                    return "vector_layers" in Oe && Oe.vector_layers && (Be.vectorLayerIds = Oe.vector_layers.map((Ue) => Ue.id)), Be;
                  });
                }
                class Rt {
                  constructor(Te, Ae) {
                    Te && (Ae ? this.setSouthWest(Te).setNorthEast(Ae) : Array.isArray(Te) && (Te.length === 4 ? this.setSouthWest([Te[0], Te[1]]).setNorthEast([Te[2], Te[3]]) : this.setSouthWest(Te[0]).setNorthEast(Te[1])));
                  }
                  setNorthEast(Te) {
                    return this._ne = Te instanceof ee.N ? new ee.N(Te.lng, Te.lat) : ee.N.convert(Te), this;
                  }
                  setSouthWest(Te) {
                    return this._sw = Te instanceof ee.N ? new ee.N(Te.lng, Te.lat) : ee.N.convert(Te), this;
                  }
                  extend(Te) {
                    const Ae = this._sw, Oe = this._ne;
                    let Be, Ue;
                    if (Te instanceof ee.N) Be = Te, Ue = Te;
                    else {
                      if (!(Te instanceof Rt)) {
                        if (Array.isArray(Te)) {
                          if (Te.length === 4 || Te.every(Array.isArray)) {
                            const nt = Te;
                            return this.extend(Rt.convert(nt));
                          }
                          {
                            const nt = Te;
                            return this.extend(ee.N.convert(nt));
                          }
                        }
                        return Te && ("lng" in Te || "lon" in Te) && "lat" in Te ? this.extend(ee.N.convert(Te)) : this;
                      }
                      if (Be = Te._sw, Ue = Te._ne, !Be || !Ue) return this;
                    }
                    return Ae || Oe ? (Ae.lng = Math.min(Be.lng, Ae.lng), Ae.lat = Math.min(Be.lat, Ae.lat), Oe.lng = Math.max(Ue.lng, Oe.lng), Oe.lat = Math.max(Ue.lat, Oe.lat)) : (this._sw = new ee.N(Be.lng, Be.lat), this._ne = new ee.N(Ue.lng, Ue.lat)), this;
                  }
                  getCenter() {
                    return new ee.N((this._sw.lng + this._ne.lng) / 2, (this._sw.lat + this._ne.lat) / 2);
                  }
                  getSouthWest() {
                    return this._sw;
                  }
                  getNorthEast() {
                    return this._ne;
                  }
                  getNorthWest() {
                    return new ee.N(this.getWest(), this.getNorth());
                  }
                  getSouthEast() {
                    return new ee.N(this.getEast(), this.getSouth());
                  }
                  getWest() {
                    return this._sw.lng;
                  }
                  getSouth() {
                    return this._sw.lat;
                  }
                  getEast() {
                    return this._ne.lng;
                  }
                  getNorth() {
                    return this._ne.lat;
                  }
                  toArray() {
                    return [this._sw.toArray(), this._ne.toArray()];
                  }
                  toString() {
                    return `LngLatBounds(${this._sw.toString()}, ${this._ne.toString()})`;
                  }
                  isEmpty() {
                    return !(this._sw && this._ne);
                  }
                  contains(Te) {
                    const { lng: Ae, lat: Oe } = ee.N.convert(Te), Be = this._sw.lat <= Oe && Oe <= this._ne.lat;
                    let Ue = this._sw.lng <= Ae && Ae <= this._ne.lng;
                    return this._sw.lng > this._ne.lng && (Ue = this._sw.lng >= Ae && Ae >= this._ne.lng), Be && Ue;
                  }
                  static convert(Te) {
                    return Te instanceof Rt ? Te : Te && new Rt(Te);
                  }
                  static fromLngLat(Te, Ae = 0) {
                    const Oe = 360 * Ae / 40075017, Be = Oe / Math.cos(Math.PI / 180 * Te.lat);
                    return new Rt(new ee.N(Te.lng - Be, Te.lat - Oe), new ee.N(Te.lng + Be, Te.lat + Oe));
                  }
                }
                class vt {
                  constructor(Te, Ae, Oe) {
                    this.bounds = Rt.convert(this.validateBounds(Te)), this.minzoom = Ae || 0, this.maxzoom = Oe || 24;
                  }
                  validateBounds(Te) {
                    return Array.isArray(Te) && Te.length === 4 ? [Math.max(-180, Te[0]), Math.max(-90, Te[1]), Math.min(180, Te[2]), Math.min(90, Te[3])] : [-180, -90, 180, 90];
                  }
                  contains(Te) {
                    const Ae = Math.pow(2, Te.z), Oe = Math.floor(ee.O(this.bounds.getWest()) * Ae), Be = Math.floor(ee.Q(this.bounds.getNorth()) * Ae), Ue = Math.ceil(ee.O(this.bounds.getEast()) * Ae), nt = Math.ceil(ee.Q(this.bounds.getSouth()) * Ae);
                    return Te.x >= Oe && Te.x < Ue && Te.y >= Be && Te.y < nt;
                  }
                }
                class $t extends ee.E {
                  constructor(Te, Ae, Oe, Be) {
                    if (super(), this.id = Te, this.dispatcher = Oe, this.type = "vector", this.minzoom = 0, this.maxzoom = 22, this.scheme = "xyz", this.tileSize = 512, this.reparseOverscaled = !0, this.isTileClipped = !0, this._loaded = !1, ee.e(this, ee.M(Ae, ["url", "scheme", "tileSize", "promoteId"])), this._options = ee.e({ type: "vector" }, Ae), this._collectResourceTiming = Ae.collectResourceTiming, this.tileSize !== 512) throw new Error("vector tile sources must have a tileSize of 512");
                    this.setEventedParent(Be);
                  }
                  load() {
                    return ee._(this, void 0, void 0, function* () {
                      this._loaded = !1, this.fire(new ee.k("dataloading", { dataType: "source" })), this._tileJSONRequest = new AbortController();
                      try {
                        const Te = yield Lt(this._options, this.map._requestManager, this._tileJSONRequest);
                        this._tileJSONRequest = null, this._loaded = !0, this.map.style.sourceCaches[this.id].clearTiles(), Te && (ee.e(this, Te), Te.bounds && (this.tileBounds = new vt(Te.bounds, this.minzoom, this.maxzoom)), this.fire(new ee.k("data", { dataType: "source", sourceDataType: "metadata" })), this.fire(new ee.k("data", { dataType: "source", sourceDataType: "content" })));
                      } catch (Te) {
                        this._tileJSONRequest = null, this.fire(new ee.j(Te));
                      }
                    });
                  }
                  loaded() {
                    return this._loaded;
                  }
                  hasTile(Te) {
                    return !this.tileBounds || this.tileBounds.contains(Te.canonical);
                  }
                  onAdd(Te) {
                    this.map = Te, this.load();
                  }
                  setSourceProperty(Te) {
                    this._tileJSONRequest && this._tileJSONRequest.abort(), Te(), this.load();
                  }
                  setTiles(Te) {
                    return this.setSourceProperty(() => {
                      this._options.tiles = Te;
                    }), this;
                  }
                  setUrl(Te) {
                    return this.setSourceProperty(() => {
                      this.url = Te, this._options.url = Te;
                    }), this;
                  }
                  onRemove() {
                    this._tileJSONRequest && (this._tileJSONRequest.abort(), this._tileJSONRequest = null);
                  }
                  serialize() {
                    return ee.e({}, this._options);
                  }
                  loadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      const Ae = Te.tileID.canonical.url(this.tiles, this.map.getPixelRatio(), this.scheme), Oe = { request: this.map._requestManager.transformRequest(Ae, "Tile"), uid: Te.uid, tileID: Te.tileID, zoom: Te.tileID.overscaledZ, tileSize: this.tileSize * Te.tileID.overscaleFactor(), type: this.type, source: this.id, pixelRatio: this.map.getPixelRatio(), showCollisionBoxes: this.map.showCollisionBoxes, promoteId: this.promoteId };
                      Oe.request.collectResourceTiming = this._collectResourceTiming;
                      let Be = "RT";
                      if (Te.actor && Te.state !== "expired") {
                        if (Te.state === "loading") return new Promise((Ue, nt) => {
                          Te.reloadPromise = { resolve: Ue, reject: nt };
                        });
                      } else Te.actor = this.dispatcher.getActor(), Be = "LT";
                      Te.abortController = new AbortController();
                      try {
                        const Ue = yield Te.actor.sendAsync({ type: Be, data: Oe }, Te.abortController);
                        if (delete Te.abortController, Te.aborted) return;
                        this._afterTileLoadWorkerResponse(Te, Ue);
                      } catch (Ue) {
                        if (delete Te.abortController, Te.aborted) return;
                        if (Ue && Ue.status !== 404) throw Ue;
                        this._afterTileLoadWorkerResponse(Te, null);
                      }
                    });
                  }
                  _afterTileLoadWorkerResponse(Te, Ae) {
                    if (Ae && Ae.resourceTiming && (Te.resourceTiming = Ae.resourceTiming), Ae && this.map._refreshExpiredTiles && Te.setExpiryData(Ae), Te.loadVectorData(Ae, this.map.painter), Te.reloadPromise) {
                      const Oe = Te.reloadPromise;
                      Te.reloadPromise = null, this.loadTile(Te).then(Oe.resolve).catch(Oe.reject);
                    }
                  }
                  abortTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      Te.abortController && (Te.abortController.abort(), delete Te.abortController), Te.actor && (yield Te.actor.sendAsync({ type: "AT", data: { uid: Te.uid, type: this.type, source: this.id } }));
                    });
                  }
                  unloadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      Te.unloadVectorData(), Te.actor && (yield Te.actor.sendAsync({ type: "RMT", data: { uid: Te.uid, type: this.type, source: this.id } }));
                    });
                  }
                  hasTransition() {
                    return !1;
                  }
                }
                class Pt extends ee.E {
                  constructor(Te, Ae, Oe, Be) {
                    super(), this.id = Te, this.dispatcher = Oe, this.setEventedParent(Be), this.type = "raster", this.minzoom = 0, this.maxzoom = 22, this.roundZoom = !0, this.scheme = "xyz", this.tileSize = 512, this._loaded = !1, this._options = ee.e({ type: "raster" }, Ae), ee.e(this, ee.M(Ae, ["url", "scheme", "tileSize"]));
                  }
                  load() {
                    return ee._(this, void 0, void 0, function* () {
                      this._loaded = !1, this.fire(new ee.k("dataloading", { dataType: "source" })), this._tileJSONRequest = new AbortController();
                      try {
                        const Te = yield Lt(this._options, this.map._requestManager, this._tileJSONRequest);
                        this._tileJSONRequest = null, this._loaded = !0, Te && (ee.e(this, Te), Te.bounds && (this.tileBounds = new vt(Te.bounds, this.minzoom, this.maxzoom)), this.fire(new ee.k("data", { dataType: "source", sourceDataType: "metadata" })), this.fire(new ee.k("data", { dataType: "source", sourceDataType: "content" })));
                      } catch (Te) {
                        this._tileJSONRequest = null, this.fire(new ee.j(Te));
                      }
                    });
                  }
                  loaded() {
                    return this._loaded;
                  }
                  onAdd(Te) {
                    this.map = Te, this.load();
                  }
                  onRemove() {
                    this._tileJSONRequest && (this._tileJSONRequest.abort(), this._tileJSONRequest = null);
                  }
                  setSourceProperty(Te) {
                    this._tileJSONRequest && (this._tileJSONRequest.abort(), this._tileJSONRequest = null), Te(), this.load();
                  }
                  setTiles(Te) {
                    return this.setSourceProperty(() => {
                      this._options.tiles = Te;
                    }), this;
                  }
                  setUrl(Te) {
                    return this.setSourceProperty(() => {
                      this.url = Te, this._options.url = Te;
                    }), this;
                  }
                  serialize() {
                    return ee.e({}, this._options);
                  }
                  hasTile(Te) {
                    return !this.tileBounds || this.tileBounds.contains(Te.canonical);
                  }
                  loadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      const Ae = Te.tileID.canonical.url(this.tiles, this.map.getPixelRatio(), this.scheme);
                      Te.abortController = new AbortController();
                      try {
                        const Oe = yield ye.getImage(this.map._requestManager.transformRequest(Ae, "Tile"), Te.abortController, this.map._refreshExpiredTiles);
                        if (delete Te.abortController, Te.aborted) return void (Te.state = "unloaded");
                        if (Oe && Oe.data) {
                          this.map._refreshExpiredTiles && Oe.cacheControl && Oe.expires && Te.setExpiryData({ cacheControl: Oe.cacheControl, expires: Oe.expires });
                          const Be = this.map.painter.context, Ue = Be.gl, nt = Oe.data;
                          Te.texture = this.map.painter.getTileTexture(nt.width), Te.texture ? Te.texture.update(nt, { useMipmap: !0 }) : (Te.texture = new Le(Be, nt, Ue.RGBA, { useMipmap: !0 }), Te.texture.bind(Ue.LINEAR, Ue.CLAMP_TO_EDGE, Ue.LINEAR_MIPMAP_NEAREST)), Te.state = "loaded";
                        }
                      } catch (Oe) {
                        if (delete Te.abortController, Te.aborted) Te.state = "unloaded";
                        else if (Oe) throw Te.state = "errored", Oe;
                      }
                    });
                  }
                  abortTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      Te.abortController && (Te.abortController.abort(), delete Te.abortController);
                    });
                  }
                  unloadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      Te.texture && this.map.painter.saveTileTexture(Te.texture);
                    });
                  }
                  hasTransition() {
                    return !1;
                  }
                }
                class Xt extends Pt {
                  constructor(Te, Ae, Oe, Be) {
                    super(Te, Ae, Oe, Be), this.type = "raster-dem", this.maxzoom = 22, this._options = ee.e({ type: "raster-dem" }, Ae), this.encoding = Ae.encoding || "mapbox", this.redFactor = Ae.redFactor, this.greenFactor = Ae.greenFactor, this.blueFactor = Ae.blueFactor, this.baseShift = Ae.baseShift;
                  }
                  loadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      const Ae = Te.tileID.canonical.url(this.tiles, this.map.getPixelRatio(), this.scheme), Oe = this.map._requestManager.transformRequest(Ae, "Tile");
                      Te.neighboringTiles = this._getNeighboringTiles(Te.tileID), Te.abortController = new AbortController();
                      try {
                        const Be = yield ye.getImage(Oe, Te.abortController, this.map._refreshExpiredTiles);
                        if (delete Te.abortController, Te.aborted) return void (Te.state = "unloaded");
                        if (Be && Be.data) {
                          const Ue = Be.data;
                          this.map._refreshExpiredTiles && Be.cacheControl && Be.expires && Te.setExpiryData({ cacheControl: Be.cacheControl, expires: Be.expires });
                          const nt = ee.b(Ue) && ee.U() ? Ue : yield this.readImageNow(Ue), ht = { type: this.type, uid: Te.uid, source: this.id, rawImageData: nt, encoding: this.encoding, redFactor: this.redFactor, greenFactor: this.greenFactor, blueFactor: this.blueFactor, baseShift: this.baseShift };
                          if (!Te.actor || Te.state === "expired") {
                            Te.actor = this.dispatcher.getActor();
                            const Et = yield Te.actor.sendAsync({ type: "LDT", data: ht });
                            Te.dem = Et, Te.needsHillshadePrepare = !0, Te.needsTerrainPrepare = !0, Te.state = "loaded";
                          }
                        }
                      } catch (Be) {
                        if (delete Te.abortController, Te.aborted) Te.state = "unloaded";
                        else if (Be) throw Te.state = "errored", Be;
                      }
                    });
                  }
                  readImageNow(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      if (typeof VideoFrame < "u" && ee.V()) {
                        const Ae = Te.width + 2, Oe = Te.height + 2;
                        try {
                          return new ee.R({ width: Ae, height: Oe }, yield ee.W(Te, -1, -1, Ae, Oe));
                        } catch {
                        }
                      }
                      return ce.getImageData(Te, 1);
                    });
                  }
                  _getNeighboringTiles(Te) {
                    const Ae = Te.canonical, Oe = Math.pow(2, Ae.z), Be = (Ae.x - 1 + Oe) % Oe, Ue = Ae.x === 0 ? Te.wrap - 1 : Te.wrap, nt = (Ae.x + 1 + Oe) % Oe, ht = Ae.x + 1 === Oe ? Te.wrap + 1 : Te.wrap, Et = {};
                    return Et[new ee.S(Te.overscaledZ, Ue, Ae.z, Be, Ae.y).key] = { backfilled: !1 }, Et[new ee.S(Te.overscaledZ, ht, Ae.z, nt, Ae.y).key] = { backfilled: !1 }, Ae.y > 0 && (Et[new ee.S(Te.overscaledZ, Ue, Ae.z, Be, Ae.y - 1).key] = { backfilled: !1 }, Et[new ee.S(Te.overscaledZ, Te.wrap, Ae.z, Ae.x, Ae.y - 1).key] = { backfilled: !1 }, Et[new ee.S(Te.overscaledZ, ht, Ae.z, nt, Ae.y - 1).key] = { backfilled: !1 }), Ae.y + 1 < Oe && (Et[new ee.S(Te.overscaledZ, Ue, Ae.z, Be, Ae.y + 1).key] = { backfilled: !1 }, Et[new ee.S(Te.overscaledZ, Te.wrap, Ae.z, Ae.x, Ae.y + 1).key] = { backfilled: !1 }, Et[new ee.S(Te.overscaledZ, ht, Ae.z, nt, Ae.y + 1).key] = { backfilled: !1 }), Et;
                  }
                  unloadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      Te.demTexture && this.map.painter.saveTileTexture(Te.demTexture), Te.fbo && (Te.fbo.destroy(), delete Te.fbo), Te.dem && delete Te.dem, delete Te.neighboringTiles, Te.state = "unloaded", Te.actor && (yield Te.actor.sendAsync({ type: "RDT", data: { type: this.type, uid: Te.uid, source: this.id } }));
                    });
                  }
                }
                class Nt extends ee.E {
                  constructor(Te, Ae, Oe, Be) {
                    super(), this.id = Te, this.type = "geojson", this.minzoom = 0, this.maxzoom = 18, this.tileSize = 512, this.isTileClipped = !0, this.reparseOverscaled = !0, this._removed = !1, this._pendingLoads = 0, this.actor = Oe.getActor(), this.setEventedParent(Be), this._data = Ae.data, this._options = ee.e({}, Ae), this._collectResourceTiming = Ae.collectResourceTiming, Ae.maxzoom !== void 0 && (this.maxzoom = Ae.maxzoom), Ae.type && (this.type = Ae.type), Ae.attribution && (this.attribution = Ae.attribution), this.promoteId = Ae.promoteId;
                    const Ue = ee.X / this.tileSize;
                    this.workerOptions = ee.e({ source: this.id, cluster: Ae.cluster || !1, geojsonVtOptions: { buffer: (Ae.buffer !== void 0 ? Ae.buffer : 128) * Ue, tolerance: (Ae.tolerance !== void 0 ? Ae.tolerance : 0.375) * Ue, extent: ee.X, maxZoom: this.maxzoom, lineMetrics: Ae.lineMetrics || !1, generateId: Ae.generateId || !1 }, superclusterOptions: { maxZoom: Ae.clusterMaxZoom !== void 0 ? Ae.clusterMaxZoom : this.maxzoom - 1, minPoints: Math.max(2, Ae.clusterMinPoints || 2), extent: ee.X, radius: (Ae.clusterRadius || 50) * Ue, log: !1, generateId: Ae.generateId || !1 }, clusterProperties: Ae.clusterProperties, filter: Ae.filter }, Ae.workerOptions), typeof this.promoteId == "string" && (this.workerOptions.promoteId = this.promoteId);
                  }
                  load() {
                    return ee._(this, void 0, void 0, function* () {
                      yield this._updateWorkerData();
                    });
                  }
                  onAdd(Te) {
                    this.map = Te, this.load();
                  }
                  setData(Te) {
                    return this._data = Te, this._updateWorkerData(), this;
                  }
                  updateData(Te) {
                    return this._updateWorkerData(Te), this;
                  }
                  getData() {
                    return ee._(this, void 0, void 0, function* () {
                      const Te = ee.e({ type: this.type }, this.workerOptions);
                      return this.actor.sendAsync({ type: "GD", data: Te });
                    });
                  }
                  setClusterOptions(Te) {
                    return this.workerOptions.cluster = Te.cluster, Te && (Te.clusterRadius !== void 0 && (this.workerOptions.superclusterOptions.radius = Te.clusterRadius), Te.clusterMaxZoom !== void 0 && (this.workerOptions.superclusterOptions.maxZoom = Te.clusterMaxZoom)), this._updateWorkerData(), this;
                  }
                  getClusterExpansionZoom(Te) {
                    return this.actor.sendAsync({ type: "GCEZ", data: { type: this.type, clusterId: Te, source: this.id } });
                  }
                  getClusterChildren(Te) {
                    return this.actor.sendAsync({ type: "GCC", data: { type: this.type, clusterId: Te, source: this.id } });
                  }
                  getClusterLeaves(Te, Ae, Oe) {
                    return this.actor.sendAsync({ type: "GCL", data: { type: this.type, source: this.id, clusterId: Te, limit: Ae, offset: Oe } });
                  }
                  _updateWorkerData(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      const Ae = ee.e({ type: this.type }, this.workerOptions);
                      Te ? Ae.dataDiff = Te : typeof this._data == "string" ? (Ae.request = this.map._requestManager.transformRequest(ce.resolveURL(this._data), "Source"), Ae.request.collectResourceTiming = this._collectResourceTiming) : Ae.data = JSON.stringify(this._data), this._pendingLoads++, this.fire(new ee.k("dataloading", { dataType: "source" }));
                      try {
                        const Oe = yield this.actor.sendAsync({ type: "LD", data: Ae });
                        if (this._pendingLoads--, this._removed || Oe.abandoned) return void this.fire(new ee.k("dataabort", { dataType: "source" }));
                        let Be = null;
                        Oe.resourceTiming && Oe.resourceTiming[this.id] && (Be = Oe.resourceTiming[this.id].slice(0));
                        const Ue = { dataType: "source" };
                        this._collectResourceTiming && Be && Be.length > 0 && ee.e(Ue, { resourceTiming: Be }), this.fire(new ee.k("data", Object.assign(Object.assign({}, Ue), { sourceDataType: "metadata" }))), this.fire(new ee.k("data", Object.assign(Object.assign({}, Ue), { sourceDataType: "content" })));
                      } catch (Oe) {
                        if (this._pendingLoads--, this._removed) return void this.fire(new ee.k("dataabort", { dataType: "source" }));
                        this.fire(new ee.j(Oe));
                      }
                    });
                  }
                  loaded() {
                    return this._pendingLoads === 0;
                  }
                  loadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      const Ae = Te.actor ? "RT" : "LT";
                      Te.actor = this.actor;
                      const Oe = { type: this.type, uid: Te.uid, tileID: Te.tileID, zoom: Te.tileID.overscaledZ, maxZoom: this.maxzoom, tileSize: this.tileSize, source: this.id, pixelRatio: this.map.getPixelRatio(), showCollisionBoxes: this.map.showCollisionBoxes, promoteId: this.promoteId };
                      Te.abortController = new AbortController();
                      const Be = yield this.actor.sendAsync({ type: Ae, data: Oe }, Te.abortController);
                      delete Te.abortController, Te.unloadVectorData(), Te.aborted || Te.loadVectorData(Be, this.map.painter, Ae === "RT");
                    });
                  }
                  abortTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      Te.abortController && (Te.abortController.abort(), delete Te.abortController), Te.aborted = !0;
                    });
                  }
                  unloadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      Te.unloadVectorData(), yield this.actor.sendAsync({ type: "RMT", data: { uid: Te.uid, type: this.type, source: this.id } });
                    });
                  }
                  onRemove() {
                    this._removed = !0, this.actor.sendAsync({ type: "RS", data: { type: this.type, source: this.id } });
                  }
                  serialize() {
                    return ee.e({}, this._options, { type: this.type, data: this._data });
                  }
                  hasTransition() {
                    return !1;
                  }
                }
                var Gt = ee.Y([{ name: "a_pos", type: "Int16", components: 2 }, { name: "a_texture_pos", type: "Int16", components: 2 }]);
                class Ft extends ee.E {
                  constructor(Te, Ae, Oe, Be) {
                    super(), this.id = Te, this.dispatcher = Oe, this.coordinates = Ae.coordinates, this.type = "image", this.minzoom = 0, this.maxzoom = 22, this.tileSize = 512, this.tiles = {}, this._loaded = !1, this.setEventedParent(Be), this.options = Ae;
                  }
                  load(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      this._loaded = !1, this.fire(new ee.k("dataloading", { dataType: "source" })), this.url = this.options.url, this._request = new AbortController();
                      try {
                        const Ae = yield ye.getImage(this.map._requestManager.transformRequest(this.url, "Image"), this._request);
                        this._request = null, this._loaded = !0, Ae && Ae.data && (this.image = Ae.data, Te && (this.coordinates = Te), this._finishLoading());
                      } catch (Ae) {
                        this._request = null, this._loaded = !0, this.fire(new ee.j(Ae));
                      }
                    });
                  }
                  loaded() {
                    return this._loaded;
                  }
                  updateImage(Te) {
                    return Te.url ? (this._request && (this._request.abort(), this._request = null), this.options.url = Te.url, this.load(Te.coordinates).finally(() => {
                      this.texture = null;
                    }), this) : this;
                  }
                  _finishLoading() {
                    this.map && (this.setCoordinates(this.coordinates), this.fire(new ee.k("data", { dataType: "source", sourceDataType: "metadata" })));
                  }
                  onAdd(Te) {
                    this.map = Te, this.load();
                  }
                  onRemove() {
                    this._request && (this._request.abort(), this._request = null);
                  }
                  setCoordinates(Te) {
                    this.coordinates = Te;
                    const Ae = Te.map(ee.Z.fromLngLat);
                    this.tileID = function(Be) {
                      let Ue = 1 / 0, nt = 1 / 0, ht = -1 / 0, Et = -1 / 0;
                      for (const On of Be) Ue = Math.min(Ue, On.x), nt = Math.min(nt, On.y), ht = Math.max(ht, On.x), Et = Math.max(Et, On.y);
                      const Mt = ht - Ue, qt = Et - nt, tn = Math.max(Mt, qt), xn = Math.max(0, Math.floor(-Math.log(tn) / Math.LN2)), An = Math.pow(2, xn);
                      return new ee.a1(xn, Math.floor((Ue + ht) / 2 * An), Math.floor((nt + Et) / 2 * An));
                    }(Ae), this.minzoom = this.maxzoom = this.tileID.z;
                    const Oe = Ae.map((Be) => this.tileID.getTilePoint(Be)._round());
                    return this._boundsArray = new ee.$(), this._boundsArray.emplaceBack(Oe[0].x, Oe[0].y, 0, 0), this._boundsArray.emplaceBack(Oe[1].x, Oe[1].y, ee.X, 0), this._boundsArray.emplaceBack(Oe[3].x, Oe[3].y, 0, ee.X), this._boundsArray.emplaceBack(Oe[2].x, Oe[2].y, ee.X, ee.X), this.boundsBuffer && (this.boundsBuffer.destroy(), delete this.boundsBuffer), this.fire(new ee.k("data", { dataType: "source", sourceDataType: "content" })), this;
                  }
                  prepare() {
                    if (Object.keys(this.tiles).length === 0 || !this.image) return;
                    const Te = this.map.painter.context, Ae = Te.gl;
                    this.boundsBuffer || (this.boundsBuffer = Te.createVertexBuffer(this._boundsArray, Gt.members)), this.boundsSegments || (this.boundsSegments = ee.a0.simpleSegment(0, 0, 4, 2)), this.texture || (this.texture = new Le(Te, this.image, Ae.RGBA), this.texture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE));
                    let Oe = !1;
                    for (const Be in this.tiles) {
                      const Ue = this.tiles[Be];
                      Ue.state !== "loaded" && (Ue.state = "loaded", Ue.texture = this.texture, Oe = !0);
                    }
                    Oe && this.fire(new ee.k("data", { dataType: "source", sourceDataType: "idle", sourceId: this.id }));
                  }
                  loadTile(Te) {
                    return ee._(this, void 0, void 0, function* () {
                      this.tileID && this.tileID.equals(Te.tileID.canonical) ? (this.tiles[String(Te.tileID.wrap)] = Te, Te.buckets = {}) : Te.state = "errored";
                    });
                  }
                  serialize() {
                    return { type: "image", url: this.options.url, coordinates: this.coordinates };
                  }
                  hasTransition() {
                    return !1;
                  }
                }
                class Tt extends Ft {
                  constructor(Te, Ae, Oe, Be) {
                    super(Te, Ae, Oe, Be), this.roundZoom = !0, this.type = "video", this.options = Ae;
                  }
                  load() {
                    return ee._(this, void 0, void 0, function* () {
                      this._loaded = !1;
                      const Te = this.options;
                      this.urls = [];
                      for (const Ae of Te.urls) this.urls.push(this.map._requestManager.transformRequest(Ae, "Source").url);
                      try {
                        const Ae = yield ee.a3(this.urls);
                        if (this._loaded = !0, !Ae) return;
                        this.video = Ae, this.video.loop = !0, this.video.addEventListener("playing", () => {
                          this.map.triggerRepaint();
                        }), this.map && this.video.play(), this._finishLoading();
                      } catch (Ae) {
                        this.fire(new ee.j(Ae));
                      }
                    });
                  }
                  pause() {
                    this.video && this.video.pause();
                  }
                  play() {
                    this.video && this.video.play();
                  }
                  seek(Te) {
                    if (this.video) {
                      const Ae = this.video.seekable;
                      Te < Ae.start(0) || Te > Ae.end(0) ? this.fire(new ee.j(new ee.a2(`sources.${this.id}`, null, `Playback for this video can be set only between the ${Ae.start(0)} and ${Ae.end(0)}-second mark.`))) : this.video.currentTime = Te;
                    }
                  }
                  getVideo() {
                    return this.video;
                  }
                  onAdd(Te) {
                    this.map || (this.map = Te, this.load(), this.video && (this.video.play(), this.setCoordinates(this.coordinates)));
                  }
                  prepare() {
                    if (Object.keys(this.tiles).length === 0 || this.video.readyState < 2) return;
                    const Te = this.map.painter.context, Ae = Te.gl;
                    this.boundsBuffer || (this.boundsBuffer = Te.createVertexBuffer(this._boundsArray, Gt.members)), this.boundsSegments || (this.boundsSegments = ee.a0.simpleSegment(0, 0, 4, 2)), this.texture ? this.video.paused || (this.texture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE), Ae.texSubImage2D(Ae.TEXTURE_2D, 0, 0, 0, Ae.RGBA, Ae.UNSIGNED_BYTE, this.video)) : (this.texture = new Le(Te, this.video, Ae.RGBA), this.texture.bind(Ae.LINEAR, Ae.CLAMP_TO_EDGE));
                    let Oe = !1;
                    for (const Be in this.tiles) {
                      const Ue = this.tiles[Be];
                      Ue.state !== "loaded" && (Ue.state = "loaded", Ue.texture = this.texture, Oe = !0);
                    }
                    Oe && this.fire(new ee.k("data", { dataType: "source", sourceDataType: "idle", sourceId: this.id }));
                  }
                  serialize() {
                    return { type: "video", urls: this.urls, coordinates: this.coordinates };
                  }
                  hasTransition() {
                    return this.video && !this.video.paused;
                  }
                }
                class It extends Ft {
                  constructor(Te, Ae, Oe, Be) {
                    super(Te, Ae, Oe, Be), Ae.coordinates ? Array.isArray(Ae.coordinates) && Ae.coordinates.length === 4 && !Ae.coordinates.some((Ue) => !Array.isArray(Ue) || Ue.length !== 2 || Ue.some((nt) => typeof nt != "number")) || this.fire(new ee.j(new ee.a2(`sources.${Te}`, null, '"coordinates" property must be an array of 4 longitude/latitude array pairs'))) : this.fire(new ee.j(new ee.a2(`sources.${Te}`, null, 'missing required property "coordinates"'))), Ae.animate && typeof Ae.animate != "boolean" && this.fire(new ee.j(new ee.a2(`sources.${Te}`, null, 'optional "animate" property must be a boolean value'))), Ae.canvas ? typeof Ae.canvas == "string" || Ae.canvas instanceof HTMLCanvasElement || this.fire(new ee.j(new ee.a2(`sources.${Te}`, null, '"canvas" must be either a string representing the ID of the canvas element from which to read, or an HTMLCanvasElement instance'))) : this.fire(new ee.j(new ee.a2(`sources.${Te}`, null, 'missing required property "canvas"'))), this.options = Ae, this.animate = Ae.animate === void 0 || Ae.animate;
                  }
                  load() {
                    return ee._(this, void 0, void 0, function* () {
                      this._loaded = !0, this.canvas || (this.canvas = this.options.canvas instanceof HTMLCanvasElement ? this.options.canvas : document.getElementById(this.options.canvas)), this.width = this.canvas.width, this.height = this.canvas.height, this._hasInvalidDimensions() ? this.fire(new ee.j(new Error("Canvas dimensions cannot be less than or equal to zero."))) : (this.play = function() {
                        this._playing = !0, this.map.triggerRepaint();
                      }, this.pause = function() {
                        this._playing && (this.prepare(), this._playing = !1);
                      }, this._finishLoading());
                    });
                  }
                  getCanvas() {
                    return this.canvas;
                  }
                  onAdd(Te) {
                    this.map = Te, this.load(), this.canvas && this.animate && this.play();
                  }
                  onRemove() {
                    this.pause();
                  }
                  prepare() {
                    let Te = !1;
                    if (this.canvas.width !== this.width && (this.width = this.canvas.width, Te = !0), this.canvas.height !== this.height && (this.height = this.canvas.height, Te = !0), this._hasInvalidDimensions() || Object.keys(this.tiles).length === 0) return;
                    const Ae = this.map.painter.context, Oe = Ae.gl;
                    this.boundsBuffer || (this.boundsBuffer = Ae.createVertexBuffer(this._boundsArray, Gt.members)), this.boundsSegments || (this.boundsSegments = ee.a0.simpleSegment(0, 0, 4, 2)), this.texture ? (Te || this._playing) && this.texture.update(this.canvas, { premultiply: !0 }) : this.texture = new Le(Ae, this.canvas, Oe.RGBA, { premultiply: !0 });
                    let Be = !1;
                    for (const Ue in this.tiles) {
                      const nt = this.tiles[Ue];
                      nt.state !== "loaded" && (nt.state = "loaded", nt.texture = this.texture, Be = !0);
                    }
                    Be && this.fire(new ee.k("data", { dataType: "source", sourceDataType: "idle", sourceId: this.id }));
                  }
                  serialize() {
                    return { type: "canvas", coordinates: this.coordinates };
                  }
                  hasTransition() {
                    return this._playing;
                  }
                  _hasInvalidDimensions() {
                    for (const Te of [this.canvas.width, this.canvas.height]) if (isNaN(Te) || Te <= 0) return !0;
                    return !1;
                  }
                }
                const Ct = {}, Kt = (Fe) => {
                  switch (Fe) {
                    case "geojson":
                      return Nt;
                    case "image":
                      return Ft;
                    case "raster":
                      return Pt;
                    case "raster-dem":
                      return Xt;
                    case "vector":
                      return $t;
                    case "video":
                      return Tt;
                    case "canvas":
                      return It;
                  }
                  return Ct[Fe];
                }, Vt = "RTLPluginLoaded";
                class Yt extends ee.E {
                  constructor() {
                    super(...arguments), this.status = "unavailable", this.url = null, this.dispatcher = gt();
                  }
                  _syncState(Te) {
                    return this.status = Te, this.dispatcher.broadcast("SRPS", { pluginStatus: Te, pluginURL: this.url }).catch((Ae) => {
                      throw this.status = "error", Ae;
                    });
                  }
                  getRTLTextPluginStatus() {
                    return this.status;
                  }
                  clearRTLTextPlugin() {
                    this.status = "unavailable", this.url = null;
                  }
                  setRTLTextPlugin(Te) {
                    return ee._(this, arguments, void 0, function* (Ae, Oe = !1) {
                      if (this.url) throw new Error("setRTLTextPlugin cannot be called multiple times.");
                      if (this.url = ce.resolveURL(Ae), !this.url) throw new Error(`requested url ${Ae} is invalid`);
                      if (this.status === "unavailable") {
                        if (!Oe) return this._requestImport();
                        this.status = "deferred", this._syncState(this.status);
                      } else if (this.status === "requested") return this._requestImport();
                    });
                  }
                  _requestImport() {
                    return ee._(this, void 0, void 0, function* () {
                      yield this._syncState("loading"), this.status = "loaded", this.fire(new ee.k(Vt));
                    });
                  }
                  lazyLoad() {
                    this.status === "unavailable" ? this.status = "requested" : this.status === "deferred" && this._requestImport();
                  }
                }
                let sn = null;
                function hn() {
                  return sn || (sn = new Yt()), sn;
                }
                class Qt {
                  constructor(Te, Ae) {
                    this.timeAdded = 0, this.fadeEndTime = 0, this.tileID = Te, this.uid = ee.a4(), this.uses = 0, this.tileSize = Ae, this.buckets = {}, this.expirationTime = null, this.queryPadding = 0, this.hasSymbolBuckets = !1, this.hasRTLText = !1, this.dependencies = {}, this.rtt = [], this.rttCoords = {}, this.expiredRequestCount = 0, this.state = "loading";
                  }
                  registerFadeDuration(Te) {
                    const Ae = Te + this.timeAdded;
                    Ae < this.fadeEndTime || (this.fadeEndTime = Ae);
                  }
                  wasRequested() {
                    return this.state === "errored" || this.state === "loaded" || this.state === "reloading";
                  }
                  clearTextures(Te) {
                    this.demTexture && Te.saveTileTexture(this.demTexture), this.demTexture = null;
                  }
                  loadVectorData(Te, Ae, Oe) {
                    if (this.hasData() && this.unloadVectorData(), this.state = "loaded", Te) {
                      Te.featureIndex && (this.latestFeatureIndex = Te.featureIndex, Te.rawTileData ? (this.latestRawTileData = Te.rawTileData, this.latestFeatureIndex.rawTileData = Te.rawTileData) : this.latestRawTileData && (this.latestFeatureIndex.rawTileData = this.latestRawTileData)), this.collisionBoxArray = Te.collisionBoxArray, this.buckets = function(Be, Ue) {
                        const nt = {};
                        if (!Ue) return nt;
                        for (const ht of Be) {
                          const Et = ht.layerIds.map((Mt) => Ue.getLayer(Mt)).filter(Boolean);
                          if (Et.length !== 0) {
                            ht.layers = Et, ht.stateDependentLayerIds && (ht.stateDependentLayers = ht.stateDependentLayerIds.map((Mt) => Et.filter((qt) => qt.id === Mt)[0]));
                            for (const Mt of Et) nt[Mt.id] = ht;
                          }
                        }
                        return nt;
                      }(Te.buckets, Ae.style), this.hasSymbolBuckets = !1;
                      for (const Be in this.buckets) {
                        const Ue = this.buckets[Be];
                        if (Ue instanceof ee.a6) {
                          if (this.hasSymbolBuckets = !0, !Oe) break;
                          Ue.justReloaded = !0;
                        }
                      }
                      if (this.hasRTLText = !1, this.hasSymbolBuckets) for (const Be in this.buckets) {
                        const Ue = this.buckets[Be];
                        if (Ue instanceof ee.a6 && Ue.hasRTLText) {
                          this.hasRTLText = !0, hn().lazyLoad();
                          break;
                        }
                      }
                      this.queryPadding = 0;
                      for (const Be in this.buckets) {
                        const Ue = this.buckets[Be];
                        this.queryPadding = Math.max(this.queryPadding, Ae.style.getLayer(Be).queryRadius(Ue));
                      }
                      Te.imageAtlas && (this.imageAtlas = Te.imageAtlas), Te.glyphAtlasImage && (this.glyphAtlasImage = Te.glyphAtlasImage);
                    } else this.collisionBoxArray = new ee.a5();
                  }
                  unloadVectorData() {
                    for (const Te in this.buckets) this.buckets[Te].destroy();
                    this.buckets = {}, this.imageAtlasTexture && this.imageAtlasTexture.destroy(), this.imageAtlas && (this.imageAtlas = null), this.glyphAtlasTexture && this.glyphAtlasTexture.destroy(), this.latestFeatureIndex = null, this.state = "unloaded";
                  }
                  getBucket(Te) {
                    return this.buckets[Te.id];
                  }
                  upload(Te) {
                    for (const Oe in this.buckets) {
                      const Be = this.buckets[Oe];
                      Be.uploadPending() && Be.upload(Te);
                    }
                    const Ae = Te.gl;
                    this.imageAtlas && !this.imageAtlas.uploaded && (this.imageAtlasTexture = new Le(Te, this.imageAtlas.image, Ae.RGBA), this.imageAtlas.uploaded = !0), this.glyphAtlasImage && (this.glyphAtlasTexture = new Le(Te, this.glyphAtlasImage, Ae.ALPHA), this.glyphAtlasImage = null);
                  }
                  prepare(Te) {
                    this.imageAtlas && this.imageAtlas.patchUpdatedImages(Te, this.imageAtlasTexture);
                  }
                  queryRenderedFeatures(Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt) {
                    return this.latestFeatureIndex && this.latestFeatureIndex.rawTileData ? this.latestFeatureIndex.query({ queryGeometry: Be, cameraQueryGeometry: Ue, scale: nt, tileSize: this.tileSize, pixelPosMatrix: qt, transform: Et, params: ht, queryPadding: this.queryPadding * Mt }, Te, Ae, Oe) : {};
                  }
                  querySourceFeatures(Te, Ae) {
                    const Oe = this.latestFeatureIndex;
                    if (!Oe || !Oe.rawTileData) return;
                    const Be = Oe.loadVTLayers(), Ue = Ae && Ae.sourceLayer ? Ae.sourceLayer : "", nt = Be._geojsonTileLayer || Be[Ue];
                    if (!nt) return;
                    const ht = ee.a7(Ae && Ae.filter), { z: Et, x: Mt, y: qt } = this.tileID.canonical, tn = { z: Et, x: Mt, y: qt };
                    for (let xn = 0; xn < nt.length; xn++) {
                      const An = nt.feature(xn);
                      if (ht.needGeometry) {
                        const qn = ee.a8(An, !0);
                        if (!ht.filter(new ee.a9(this.tileID.overscaledZ), qn, this.tileID.canonical)) continue;
                      } else if (!ht.filter(new ee.a9(this.tileID.overscaledZ), An)) continue;
                      const On = Oe.getId(An, Ue), Bn = new ee.aa(An, Et, Mt, qt, On);
                      Bn.tile = tn, Te.push(Bn);
                    }
                  }
                  hasData() {
                    return this.state === "loaded" || this.state === "reloading" || this.state === "expired";
                  }
                  patternsLoaded() {
                    return this.imageAtlas && !!Object.keys(this.imageAtlas.patternPositions).length;
                  }
                  setExpiryData(Te) {
                    const Ae = this.expirationTime;
                    if (Te.cacheControl) {
                      const Oe = ee.ab(Te.cacheControl);
                      Oe["max-age"] && (this.expirationTime = Date.now() + 1e3 * Oe["max-age"]);
                    } else Te.expires && (this.expirationTime = new Date(Te.expires).getTime());
                    if (this.expirationTime) {
                      const Oe = Date.now();
                      let Be = !1;
                      if (this.expirationTime > Oe) Be = !1;
                      else if (Ae) if (this.expirationTime < Ae) Be = !0;
                      else {
                        const Ue = this.expirationTime - Ae;
                        Ue ? this.expirationTime = Oe + Math.max(Ue, 3e4) : Be = !0;
                      }
                      else Be = !0;
                      Be ? (this.expiredRequestCount++, this.state = "expired") : this.expiredRequestCount = 0;
                    }
                  }
                  getExpiryTimeout() {
                    if (this.expirationTime) return this.expiredRequestCount ? 1e3 * (1 << Math.min(this.expiredRequestCount - 1, 31)) : Math.min(this.expirationTime - (/* @__PURE__ */ new Date()).getTime(), Math.pow(2, 31) - 1);
                  }
                  setFeatureState(Te, Ae) {
                    if (!this.latestFeatureIndex || !this.latestFeatureIndex.rawTileData || Object.keys(Te).length === 0) return;
                    const Oe = this.latestFeatureIndex.loadVTLayers();
                    for (const Be in this.buckets) {
                      if (!Ae.style.hasLayer(Be)) continue;
                      const Ue = this.buckets[Be], nt = Ue.layers[0].sourceLayer || "_geojsonTileLayer", ht = Oe[nt], Et = Te[nt];
                      if (!ht || !Et || Object.keys(Et).length === 0) continue;
                      Ue.update(Et, ht, this.imageAtlas && this.imageAtlas.patternPositions || {});
                      const Mt = Ae && Ae.style && Ae.style.getLayer(Be);
                      Mt && (this.queryPadding = Math.max(this.queryPadding, Mt.queryRadius(Ue)));
                    }
                  }
                  holdingForFade() {
                    return this.symbolFadeHoldUntil !== void 0;
                  }
                  symbolFadeFinished() {
                    return !this.symbolFadeHoldUntil || this.symbolFadeHoldUntil < ce.now();
                  }
                  clearFadeHold() {
                    this.symbolFadeHoldUntil = void 0;
                  }
                  setHoldDuration(Te) {
                    this.symbolFadeHoldUntil = ce.now() + Te;
                  }
                  setDependencies(Te, Ae) {
                    const Oe = {};
                    for (const Be of Ae) Oe[Be] = !0;
                    this.dependencies[Te] = Oe;
                  }
                  hasDependency(Te, Ae) {
                    for (const Oe of Te) {
                      const Be = this.dependencies[Oe];
                      if (Be) {
                        for (const Ue of Ae) if (Be[Ue]) return !0;
                      }
                    }
                    return !1;
                  }
                }
                class Ut {
                  constructor(Te, Ae) {
                    this.max = Te, this.onRemove = Ae, this.reset();
                  }
                  reset() {
                    for (const Te in this.data) for (const Ae of this.data[Te]) Ae.timeout && clearTimeout(Ae.timeout), this.onRemove(Ae.value);
                    return this.data = {}, this.order = [], this;
                  }
                  add(Te, Ae, Oe) {
                    const Be = Te.wrapped().key;
                    this.data[Be] === void 0 && (this.data[Be] = []);
                    const Ue = { value: Ae, timeout: void 0 };
                    if (Oe !== void 0 && (Ue.timeout = setTimeout(() => {
                      this.remove(Te, Ue);
                    }, Oe)), this.data[Be].push(Ue), this.order.push(Be), this.order.length > this.max) {
                      const nt = this._getAndRemoveByKey(this.order[0]);
                      nt && this.onRemove(nt);
                    }
                    return this;
                  }
                  has(Te) {
                    return Te.wrapped().key in this.data;
                  }
                  getAndRemove(Te) {
                    return this.has(Te) ? this._getAndRemoveByKey(Te.wrapped().key) : null;
                  }
                  _getAndRemoveByKey(Te) {
                    const Ae = this.data[Te].shift();
                    return Ae.timeout && clearTimeout(Ae.timeout), this.data[Te].length === 0 && delete this.data[Te], this.order.splice(this.order.indexOf(Te), 1), Ae.value;
                  }
                  getByKey(Te) {
                    const Ae = this.data[Te];
                    return Ae ? Ae[0].value : null;
                  }
                  get(Te) {
                    return this.has(Te) ? this.data[Te.wrapped().key][0].value : null;
                  }
                  remove(Te, Ae) {
                    if (!this.has(Te)) return this;
                    const Oe = Te.wrapped().key, Be = Ae === void 0 ? 0 : this.data[Oe].indexOf(Ae), Ue = this.data[Oe][Be];
                    return this.data[Oe].splice(Be, 1), Ue.timeout && clearTimeout(Ue.timeout), this.data[Oe].length === 0 && delete this.data[Oe], this.onRemove(Ue.value), this.order.splice(this.order.indexOf(Oe), 1), this;
                  }
                  setMaxSize(Te) {
                    for (this.max = Te; this.order.length > this.max; ) {
                      const Ae = this._getAndRemoveByKey(this.order[0]);
                      Ae && this.onRemove(Ae);
                    }
                    return this;
                  }
                  filter(Te) {
                    const Ae = [];
                    for (const Oe in this.data) for (const Be of this.data[Oe]) Te(Be.value) || Ae.push(Be);
                    for (const Oe of Ae) this.remove(Oe.value.tileID, Oe);
                  }
                }
                class Wt {
                  constructor() {
                    this.state = {}, this.stateChanges = {}, this.deletedStates = {};
                  }
                  updateState(Te, Ae, Oe) {
                    const Be = String(Ae);
                    if (this.stateChanges[Te] = this.stateChanges[Te] || {}, this.stateChanges[Te][Be] = this.stateChanges[Te][Be] || {}, ee.e(this.stateChanges[Te][Be], Oe), this.deletedStates[Te] === null) {
                      this.deletedStates[Te] = {};
                      for (const Ue in this.state[Te]) Ue !== Be && (this.deletedStates[Te][Ue] = null);
                    } else if (this.deletedStates[Te] && this.deletedStates[Te][Be] === null) {
                      this.deletedStates[Te][Be] = {};
                      for (const Ue in this.state[Te][Be]) Oe[Ue] || (this.deletedStates[Te][Be][Ue] = null);
                    } else for (const Ue in Oe) this.deletedStates[Te] && this.deletedStates[Te][Be] && this.deletedStates[Te][Be][Ue] === null && delete this.deletedStates[Te][Be][Ue];
                  }
                  removeFeatureState(Te, Ae, Oe) {
                    if (this.deletedStates[Te] === null) return;
                    const Be = String(Ae);
                    if (this.deletedStates[Te] = this.deletedStates[Te] || {}, Oe && Ae !== void 0) this.deletedStates[Te][Be] !== null && (this.deletedStates[Te][Be] = this.deletedStates[Te][Be] || {}, this.deletedStates[Te][Be][Oe] = null);
                    else if (Ae !== void 0) if (this.stateChanges[Te] && this.stateChanges[Te][Be]) for (Oe in this.deletedStates[Te][Be] = {}, this.stateChanges[Te][Be]) this.deletedStates[Te][Be][Oe] = null;
                    else this.deletedStates[Te][Be] = null;
                    else this.deletedStates[Te] = null;
                  }
                  getState(Te, Ae) {
                    const Oe = String(Ae), Be = this.state[Te] || {}, Ue = this.stateChanges[Te] || {}, nt = ee.e({}, Be[Oe], Ue[Oe]);
                    if (this.deletedStates[Te] === null) return {};
                    if (this.deletedStates[Te]) {
                      const ht = this.deletedStates[Te][Ae];
                      if (ht === null) return {};
                      for (const Et in ht) delete nt[Et];
                    }
                    return nt;
                  }
                  initializeTileState(Te, Ae) {
                    Te.setFeatureState(this.state, Ae);
                  }
                  coalesceChanges(Te, Ae) {
                    const Oe = {};
                    for (const Be in this.stateChanges) {
                      this.state[Be] = this.state[Be] || {};
                      const Ue = {};
                      for (const nt in this.stateChanges[Be]) this.state[Be][nt] || (this.state[Be][nt] = {}), ee.e(this.state[Be][nt], this.stateChanges[Be][nt]), Ue[nt] = this.state[Be][nt];
                      Oe[Be] = Ue;
                    }
                    for (const Be in this.deletedStates) {
                      this.state[Be] = this.state[Be] || {};
                      const Ue = {};
                      if (this.deletedStates[Be] === null) for (const nt in this.state[Be]) Ue[nt] = {}, this.state[Be][nt] = {};
                      else for (const nt in this.deletedStates[Be]) {
                        if (this.deletedStates[Be][nt] === null) this.state[Be][nt] = {};
                        else for (const ht of Object.keys(this.deletedStates[Be][nt])) delete this.state[Be][nt][ht];
                        Ue[nt] = this.state[Be][nt];
                      }
                      Oe[Be] = Oe[Be] || {}, ee.e(Oe[Be], Ue);
                    }
                    if (this.stateChanges = {}, this.deletedStates = {}, Object.keys(Oe).length !== 0) for (const Be in Te) Te[Be].setFeatureState(Oe, Ae);
                  }
                }
                class rn extends ee.E {
                  constructor(Te, Ae, Oe) {
                    super(), this.id = Te, this.dispatcher = Oe, this.on("data", (Be) => this._dataHandler(Be)), this.on("dataloading", () => {
                      this._sourceErrored = !1;
                    }), this.on("error", () => {
                      this._sourceErrored = this._source.loaded();
                    }), this._source = ((Be, Ue, nt, ht) => {
                      const Et = new (Kt(Ue.type))(Be, Ue, nt, ht);
                      if (Et.id !== Be) throw new Error(`Expected Source id to be ${Be} instead of ${Et.id}`);
                      return Et;
                    })(Te, Ae, Oe, this), this._tiles = {}, this._cache = new Ut(0, (Be) => this._unloadTile(Be)), this._timers = {}, this._cacheTimers = {}, this._maxTileCacheSize = null, this._maxTileCacheZoomLevels = null, this._loadedParentTiles = {}, this._coveredTiles = {}, this._state = new Wt(), this._didEmitContent = !1, this._updated = !1;
                  }
                  onAdd(Te) {
                    this.map = Te, this._maxTileCacheSize = Te ? Te._maxTileCacheSize : null, this._maxTileCacheZoomLevels = Te ? Te._maxTileCacheZoomLevels : null, this._source && this._source.onAdd && this._source.onAdd(Te);
                  }
                  onRemove(Te) {
                    this.clearTiles(), this._source && this._source.onRemove && this._source.onRemove(Te);
                  }
                  loaded() {
                    if (this._sourceErrored) return !0;
                    if (!this._sourceLoaded || !this._source.loaded()) return !1;
                    if (!(this.used === void 0 && this.usedForTerrain === void 0 || this.used || this.usedForTerrain)) return !0;
                    if (!this._updated) return !1;
                    for (const Te in this._tiles) {
                      const Ae = this._tiles[Te];
                      if (Ae.state !== "loaded" && Ae.state !== "errored") return !1;
                    }
                    return !0;
                  }
                  getSource() {
                    return this._source;
                  }
                  pause() {
                    this._paused = !0;
                  }
                  resume() {
                    if (!this._paused) return;
                    const Te = this._shouldReloadOnResume;
                    this._paused = !1, this._shouldReloadOnResume = !1, Te && this.reload(), this.transform && this.update(this.transform, this.terrain);
                  }
                  _loadTile(Te, Ae, Oe) {
                    return ee._(this, void 0, void 0, function* () {
                      try {
                        yield this._source.loadTile(Te), this._tileLoaded(Te, Ae, Oe);
                      } catch (Be) {
                        Te.state = "errored", Be.status !== 404 ? this._source.fire(new ee.j(Be, { tile: Te })) : this.update(this.transform, this.terrain);
                      }
                    });
                  }
                  _unloadTile(Te) {
                    this._source.unloadTile && this._source.unloadTile(Te);
                  }
                  _abortTile(Te) {
                    this._source.abortTile && this._source.abortTile(Te), this._source.fire(new ee.k("dataabort", { tile: Te, coord: Te.tileID, dataType: "source" }));
                  }
                  serialize() {
                    return this._source.serialize();
                  }
                  prepare(Te) {
                    this._source.prepare && this._source.prepare(), this._state.coalesceChanges(this._tiles, this.map ? this.map.painter : null);
                    for (const Ae in this._tiles) {
                      const Oe = this._tiles[Ae];
                      Oe.upload(Te), Oe.prepare(this.map.style.imageManager);
                    }
                  }
                  getIds() {
                    return Object.values(this._tiles).map((Te) => Te.tileID).sort(vn).map((Te) => Te.key);
                  }
                  getRenderableIds(Te) {
                    const Ae = [];
                    for (const Oe in this._tiles) this._isIdRenderable(Oe, Te) && Ae.push(this._tiles[Oe]);
                    return Te ? Ae.sort((Oe, Be) => {
                      const Ue = Oe.tileID, nt = Be.tileID, ht = new ee.P(Ue.canonical.x, Ue.canonical.y)._rotate(this.transform.angle), Et = new ee.P(nt.canonical.x, nt.canonical.y)._rotate(this.transform.angle);
                      return Ue.overscaledZ - nt.overscaledZ || Et.y - ht.y || Et.x - ht.x;
                    }).map((Oe) => Oe.tileID.key) : Ae.map((Oe) => Oe.tileID).sort(vn).map((Oe) => Oe.key);
                  }
                  hasRenderableParent(Te) {
                    const Ae = this.findLoadedParent(Te, 0);
                    return !!Ae && this._isIdRenderable(Ae.tileID.key);
                  }
                  _isIdRenderable(Te, Ae) {
                    return this._tiles[Te] && this._tiles[Te].hasData() && !this._coveredTiles[Te] && (Ae || !this._tiles[Te].holdingForFade());
                  }
                  reload() {
                    if (this._paused) this._shouldReloadOnResume = !0;
                    else {
                      this._cache.reset();
                      for (const Te in this._tiles) this._tiles[Te].state !== "errored" && this._reloadTile(Te, "reloading");
                    }
                  }
                  _reloadTile(Te, Ae) {
                    return ee._(this, void 0, void 0, function* () {
                      const Oe = this._tiles[Te];
                      Oe && (Oe.state !== "loading" && (Oe.state = Ae), yield this._loadTile(Oe, Te, Ae));
                    });
                  }
                  _tileLoaded(Te, Ae, Oe) {
                    Te.timeAdded = ce.now(), Oe === "expired" && (Te.refreshedUponExpiration = !0), this._setTileReloadTimer(Ae, Te), this.getSource().type === "raster-dem" && Te.dem && this._backfillDEM(Te), this._state.initializeTileState(Te, this.map ? this.map.painter : null), Te.aborted || this._source.fire(new ee.k("data", { dataType: "source", tile: Te, coord: Te.tileID }));
                  }
                  _backfillDEM(Te) {
                    const Ae = this.getRenderableIds();
                    for (let Be = 0; Be < Ae.length; Be++) {
                      const Ue = Ae[Be];
                      if (Te.neighboringTiles && Te.neighboringTiles[Ue]) {
                        const nt = this.getTileByID(Ue);
                        Oe(Te, nt), Oe(nt, Te);
                      }
                    }
                    function Oe(Be, Ue) {
                      Be.needsHillshadePrepare = !0, Be.needsTerrainPrepare = !0;
                      let nt = Ue.tileID.canonical.x - Be.tileID.canonical.x;
                      const ht = Ue.tileID.canonical.y - Be.tileID.canonical.y, Et = Math.pow(2, Be.tileID.canonical.z), Mt = Ue.tileID.key;
                      nt === 0 && ht === 0 || Math.abs(ht) > 1 || (Math.abs(nt) > 1 && (Math.abs(nt + Et) === 1 ? nt += Et : Math.abs(nt - Et) === 1 && (nt -= Et)), Ue.dem && Be.dem && (Be.dem.backfillBorder(Ue.dem, nt, ht), Be.neighboringTiles && Be.neighboringTiles[Mt] && (Be.neighboringTiles[Mt].backfilled = !0)));
                    }
                  }
                  getTile(Te) {
                    return this.getTileByID(Te.key);
                  }
                  getTileByID(Te) {
                    return this._tiles[Te];
                  }
                  _retainLoadedChildren(Te, Ae, Oe, Be) {
                    for (const Ue in this._tiles) {
                      let nt = this._tiles[Ue];
                      if (Be[Ue] || !nt.hasData() || nt.tileID.overscaledZ <= Ae || nt.tileID.overscaledZ > Oe) continue;
                      let ht = nt.tileID;
                      for (; nt && nt.tileID.overscaledZ > Ae + 1; ) {
                        const Mt = nt.tileID.scaledTo(nt.tileID.overscaledZ - 1);
                        nt = this._tiles[Mt.key], nt && nt.hasData() && (ht = Mt);
                      }
                      let Et = ht;
                      for (; Et.overscaledZ > Ae; ) if (Et = Et.scaledTo(Et.overscaledZ - 1), Te[Et.key]) {
                        Be[ht.key] = ht;
                        break;
                      }
                    }
                  }
                  findLoadedParent(Te, Ae) {
                    if (Te.key in this._loadedParentTiles) {
                      const Oe = this._loadedParentTiles[Te.key];
                      return Oe && Oe.tileID.overscaledZ >= Ae ? Oe : null;
                    }
                    for (let Oe = Te.overscaledZ - 1; Oe >= Ae; Oe--) {
                      const Be = Te.scaledTo(Oe), Ue = this._getLoadedTile(Be);
                      if (Ue) return Ue;
                    }
                  }
                  findLoadedSibling(Te) {
                    return this._getLoadedTile(Te);
                  }
                  _getLoadedTile(Te) {
                    const Ae = this._tiles[Te.key];
                    return Ae && Ae.hasData() ? Ae : this._cache.getByKey(Te.wrapped().key);
                  }
                  updateCacheSize(Te) {
                    const Ae = (Math.ceil(Te.width / this._source.tileSize) + 1) * (Math.ceil(Te.height / this._source.tileSize) + 1), Oe = this._maxTileCacheZoomLevels === null ? ee.a.MAX_TILE_CACHE_ZOOM_LEVELS : this._maxTileCacheZoomLevels, Be = Math.floor(Ae * Oe), Ue = typeof this._maxTileCacheSize == "number" ? Math.min(this._maxTileCacheSize, Be) : Be;
                    this._cache.setMaxSize(Ue);
                  }
                  handleWrapJump(Te) {
                    const Ae = (Te - (this._prevLng === void 0 ? Te : this._prevLng)) / 360, Oe = Math.round(Ae);
                    if (this._prevLng = Te, Oe) {
                      const Be = {};
                      for (const Ue in this._tiles) {
                        const nt = this._tiles[Ue];
                        nt.tileID = nt.tileID.unwrapTo(nt.tileID.wrap + Oe), Be[nt.tileID.key] = nt;
                      }
                      this._tiles = Be;
                      for (const Ue in this._timers) clearTimeout(this._timers[Ue]), delete this._timers[Ue];
                      for (const Ue in this._tiles) {
                        const nt = this._tiles[Ue];
                        this._setTileReloadTimer(Ue, nt);
                      }
                    }
                  }
                  _updateCoveredAndRetainedTiles(Te, Ae, Oe, Be, Ue, nt) {
                    const ht = {}, Et = {}, Mt = Object.keys(Te), qt = ce.now();
                    for (const tn of Mt) {
                      const xn = Te[tn], An = this._tiles[tn];
                      if (!An || An.fadeEndTime !== 0 && An.fadeEndTime <= qt) continue;
                      const On = this.findLoadedParent(xn, Ae), Bn = this.findLoadedSibling(xn), qn = On || Bn || null;
                      qn && (this._addTile(qn.tileID), ht[qn.tileID.key] = qn.tileID), Et[tn] = xn;
                    }
                    this._retainLoadedChildren(Et, Be, Oe, Te);
                    for (const tn in ht) Te[tn] || (this._coveredTiles[tn] = !0, Te[tn] = ht[tn]);
                    if (nt) {
                      const tn = {}, xn = {};
                      for (const An of Ue) this._tiles[An.key].hasData() ? tn[An.key] = An : xn[An.key] = An;
                      for (const An in xn) {
                        const On = xn[An].children(this._source.maxzoom);
                        this._tiles[On[0].key] && this._tiles[On[1].key] && this._tiles[On[2].key] && this._tiles[On[3].key] && (tn[On[0].key] = Te[On[0].key] = On[0], tn[On[1].key] = Te[On[1].key] = On[1], tn[On[2].key] = Te[On[2].key] = On[2], tn[On[3].key] = Te[On[3].key] = On[3], delete xn[An]);
                      }
                      for (const An in xn) {
                        const On = xn[An], Bn = this.findLoadedParent(On, this._source.minzoom), qn = this.findLoadedSibling(On), Vn = Bn || qn || null;
                        if (Vn) {
                          tn[Vn.tileID.key] = Te[Vn.tileID.key] = Vn.tileID;
                          for (const sr in tn) tn[sr].isChildOf(Vn.tileID) && delete tn[sr];
                        }
                      }
                      for (const An in this._tiles) tn[An] || (this._coveredTiles[An] = !0);
                    }
                  }
                  update(Te, Ae) {
                    if (!this._sourceLoaded || this._paused) return;
                    let Oe;
                    this.transform = Te, this.terrain = Ae, this.updateCacheSize(Te), this.handleWrapJump(this.transform.center.lng), this._coveredTiles = {}, this.used || this.usedForTerrain ? this._source.tileID ? Oe = Te.getVisibleUnwrappedCoordinates(this._source.tileID).map((qt) => new ee.S(qt.canonical.z, qt.wrap, qt.canonical.z, qt.canonical.x, qt.canonical.y)) : (Oe = Te.coveringTiles({ tileSize: this.usedForTerrain ? this.tileSize : this._source.tileSize, minzoom: this._source.minzoom, maxzoom: this._source.maxzoom, roundZoom: !this.usedForTerrain && this._source.roundZoom, reparseOverscaled: this._source.reparseOverscaled, terrain: Ae }), this._source.hasTile && (Oe = Oe.filter((qt) => this._source.hasTile(qt)))) : Oe = [];
                    const Be = Te.coveringZoomLevel(this._source), Ue = Math.max(Be - rn.maxOverzooming, this._source.minzoom), nt = Math.max(Be + rn.maxUnderzooming, this._source.minzoom);
                    if (this.usedForTerrain) {
                      const qt = {};
                      for (const tn of Oe) if (tn.canonical.z > this._source.minzoom) {
                        const xn = tn.scaledTo(tn.canonical.z - 1);
                        qt[xn.key] = xn;
                        const An = tn.scaledTo(Math.max(this._source.minzoom, Math.min(tn.canonical.z, 5)));
                        qt[An.key] = An;
                      }
                      Oe = Oe.concat(Object.values(qt));
                    }
                    const ht = Oe.length === 0 && !this._updated && this._didEmitContent;
                    this._updated = !0, ht && this.fire(new ee.k("data", { sourceDataType: "idle", dataType: "source", sourceId: this.id }));
                    const Et = this._updateRetainedTiles(Oe, Be);
                    dn(this._source.type) && this._updateCoveredAndRetainedTiles(Et, Ue, nt, Be, Oe, Ae);
                    for (const qt in Et) this._tiles[qt].clearFadeHold();
                    const Mt = ee.ac(this._tiles, Et);
                    for (const qt of Mt) {
                      const tn = this._tiles[qt];
                      tn.hasSymbolBuckets && !tn.holdingForFade() ? tn.setHoldDuration(this.map._fadeDuration) : tn.hasSymbolBuckets && !tn.symbolFadeFinished() || this._removeTile(qt);
                    }
                    this._updateLoadedParentTileCache(), this._updateLoadedSiblingTileCache();
                  }
                  releaseSymbolFadeTiles() {
                    for (const Te in this._tiles) this._tiles[Te].holdingForFade() && this._removeTile(Te);
                  }
                  _updateRetainedTiles(Te, Ae) {
                    var Oe;
                    const Be = {}, Ue = {}, nt = Math.max(Ae - rn.maxOverzooming, this._source.minzoom), ht = Math.max(Ae + rn.maxUnderzooming, this._source.minzoom), Et = {};
                    for (const Mt of Te) {
                      const qt = this._addTile(Mt);
                      Be[Mt.key] = Mt, qt.hasData() || Ae < this._source.maxzoom && (Et[Mt.key] = Mt);
                    }
                    this._retainLoadedChildren(Et, Ae, ht, Be);
                    for (const Mt of Te) {
                      let qt = this._tiles[Mt.key];
                      if (qt.hasData()) continue;
                      if (Ae + 1 > this._source.maxzoom) {
                        const xn = Mt.children(this._source.maxzoom)[0], An = this.getTile(xn);
                        if (An && An.hasData()) {
                          Be[xn.key] = xn;
                          continue;
                        }
                      } else {
                        const xn = Mt.children(this._source.maxzoom);
                        if (Be[xn[0].key] && Be[xn[1].key] && Be[xn[2].key] && Be[xn[3].key]) continue;
                      }
                      let tn = qt.wasRequested();
                      for (let xn = Mt.overscaledZ - 1; xn >= nt; --xn) {
                        const An = Mt.scaledTo(xn);
                        if (Ue[An.key]) break;
                        if (Ue[An.key] = !0, qt = this.getTile(An), !qt && tn && (qt = this._addTile(An)), qt) {
                          const On = qt.hasData();
                          if ((On || !(!((Oe = this.map) === null || Oe === void 0) && Oe.cancelPendingTileRequestsWhileZooming) || tn) && (Be[An.key] = An), tn = qt.wasRequested(), On) break;
                        }
                      }
                    }
                    return Be;
                  }
                  _updateLoadedParentTileCache() {
                    this._loadedParentTiles = {};
                    for (const Te in this._tiles) {
                      const Ae = [];
                      let Oe, Be = this._tiles[Te].tileID;
                      for (; Be.overscaledZ > 0; ) {
                        if (Be.key in this._loadedParentTiles) {
                          Oe = this._loadedParentTiles[Be.key];
                          break;
                        }
                        Ae.push(Be.key);
                        const Ue = Be.scaledTo(Be.overscaledZ - 1);
                        if (Oe = this._getLoadedTile(Ue), Oe) break;
                        Be = Ue;
                      }
                      for (const Ue of Ae) this._loadedParentTiles[Ue] = Oe;
                    }
                  }
                  _updateLoadedSiblingTileCache() {
                    this._loadedSiblingTiles = {};
                    for (const Te in this._tiles) {
                      const Ae = this._tiles[Te].tileID, Oe = this._getLoadedTile(Ae);
                      this._loadedSiblingTiles[Ae.key] = Oe;
                    }
                  }
                  _addTile(Te) {
                    let Ae = this._tiles[Te.key];
                    if (Ae) return Ae;
                    Ae = this._cache.getAndRemove(Te), Ae && (this._setTileReloadTimer(Te.key, Ae), Ae.tileID = Te, this._state.initializeTileState(Ae, this.map ? this.map.painter : null), this._cacheTimers[Te.key] && (clearTimeout(this._cacheTimers[Te.key]), delete this._cacheTimers[Te.key], this._setTileReloadTimer(Te.key, Ae)));
                    const Oe = Ae;
                    return Ae || (Ae = new Qt(Te, this._source.tileSize * Te.overscaleFactor()), this._loadTile(Ae, Te.key, Ae.state)), Ae.uses++, this._tiles[Te.key] = Ae, Oe || this._source.fire(new ee.k("dataloading", { tile: Ae, coord: Ae.tileID, dataType: "source" })), Ae;
                  }
                  _setTileReloadTimer(Te, Ae) {
                    Te in this._timers && (clearTimeout(this._timers[Te]), delete this._timers[Te]);
                    const Oe = Ae.getExpiryTimeout();
                    Oe && (this._timers[Te] = setTimeout(() => {
                      this._reloadTile(Te, "expired"), delete this._timers[Te];
                    }, Oe));
                  }
                  _removeTile(Te) {
                    const Ae = this._tiles[Te];
                    Ae && (Ae.uses--, delete this._tiles[Te], this._timers[Te] && (clearTimeout(this._timers[Te]), delete this._timers[Te]), Ae.uses > 0 || (Ae.hasData() && Ae.state !== "reloading" ? this._cache.add(Ae.tileID, Ae, Ae.getExpiryTimeout()) : (Ae.aborted = !0, this._abortTile(Ae), this._unloadTile(Ae))));
                  }
                  _dataHandler(Te) {
                    const Ae = Te.sourceDataType;
                    Te.dataType === "source" && Ae === "metadata" && (this._sourceLoaded = !0), this._sourceLoaded && !this._paused && Te.dataType === "source" && Ae === "content" && (this.reload(), this.transform && this.update(this.transform, this.terrain), this._didEmitContent = !0);
                  }
                  clearTiles() {
                    this._shouldReloadOnResume = !1, this._paused = !1;
                    for (const Te in this._tiles) this._removeTile(Te);
                    this._cache.reset();
                  }
                  tilesIn(Te, Ae, Oe) {
                    const Be = [], Ue = this.transform;
                    if (!Ue) return Be;
                    const nt = Oe ? Ue.getCameraQueryGeometry(Te) : Te, ht = Te.map((On) => Ue.pointCoordinate(On, this.terrain)), Et = nt.map((On) => Ue.pointCoordinate(On, this.terrain)), Mt = this.getIds();
                    let qt = 1 / 0, tn = 1 / 0, xn = -1 / 0, An = -1 / 0;
                    for (const On of Et) qt = Math.min(qt, On.x), tn = Math.min(tn, On.y), xn = Math.max(xn, On.x), An = Math.max(An, On.y);
                    for (let On = 0; On < Mt.length; On++) {
                      const Bn = this._tiles[Mt[On]];
                      if (Bn.holdingForFade()) continue;
                      const qn = Bn.tileID, Vn = Math.pow(2, Ue.zoom - Bn.tileID.overscaledZ), sr = Ae * Bn.queryPadding * ee.X / Bn.tileSize / Vn, Un = [qn.getTilePoint(new ee.Z(qt, tn)), qn.getTilePoint(new ee.Z(xn, An))];
                      if (Un[0].x - sr < ee.X && Un[0].y - sr < ee.X && Un[1].x + sr >= 0 && Un[1].y + sr >= 0) {
                        const wr = ht.map((Tr) => qn.getTilePoint(Tr)), Cr = Et.map((Tr) => qn.getTilePoint(Tr));
                        Be.push({ tile: Bn, tileID: qn, queryGeometry: wr, cameraQueryGeometry: Cr, scale: Vn });
                      }
                    }
                    return Be;
                  }
                  getVisibleCoordinates(Te) {
                    const Ae = this.getRenderableIds(Te).map((Oe) => this._tiles[Oe].tileID);
                    for (const Oe of Ae) Oe.posMatrix = this.transform.calculatePosMatrix(Oe.toUnwrapped());
                    return Ae;
                  }
                  hasTransition() {
                    if (this._source.hasTransition()) return !0;
                    if (dn(this._source.type)) {
                      const Te = ce.now();
                      for (const Ae in this._tiles) if (this._tiles[Ae].fadeEndTime >= Te) return !0;
                    }
                    return !1;
                  }
                  setFeatureState(Te, Ae, Oe) {
                    Te = Te || "_geojsonTileLayer", this._state.updateState(Te, Ae, Oe);
                  }
                  removeFeatureState(Te, Ae, Oe) {
                    Te = Te || "_geojsonTileLayer", this._state.removeFeatureState(Te, Ae, Oe);
                  }
                  getFeatureState(Te, Ae) {
                    return Te = Te || "_geojsonTileLayer", this._state.getState(Te, Ae);
                  }
                  setDependencies(Te, Ae, Oe) {
                    const Be = this._tiles[Te];
                    Be && Be.setDependencies(Ae, Oe);
                  }
                  reloadTilesForDependencies(Te, Ae) {
                    for (const Oe in this._tiles) this._tiles[Oe].hasDependency(Te, Ae) && this._reloadTile(Oe, "reloading");
                    this._cache.filter((Oe) => !Oe.hasDependency(Te, Ae));
                  }
                }
                function vn(Fe, Te) {
                  const Ae = Math.abs(2 * Fe.wrap) - +(Fe.wrap < 0), Oe = Math.abs(2 * Te.wrap) - +(Te.wrap < 0);
                  return Fe.overscaledZ - Te.overscaledZ || Oe - Ae || Te.canonical.y - Fe.canonical.y || Te.canonical.x - Fe.canonical.x;
                }
                function dn(Fe) {
                  return Fe === "raster" || Fe === "image" || Fe === "video";
                }
                rn.maxOverzooming = 10, rn.maxUnderzooming = 3;
                class Ht {
                  constructor(Te, Ae) {
                    this.reset(Te, Ae);
                  }
                  reset(Te, Ae) {
                    this.points = Te || [], this._distances = [0];
                    for (let Oe = 1; Oe < this.points.length; Oe++) this._distances[Oe] = this._distances[Oe - 1] + this.points[Oe].dist(this.points[Oe - 1]);
                    this.length = this._distances[this._distances.length - 1], this.padding = Math.min(Ae || 0, 0.5 * this.length), this.paddedLength = this.length - 2 * this.padding;
                  }
                  lerp(Te) {
                    if (this.points.length === 1) return this.points[0];
                    Te = ee.ad(Te, 0, 1);
                    let Ae = 1, Oe = this._distances[Ae];
                    const Be = Te * this.paddedLength + this.padding;
                    for (; Oe < Be && Ae < this._distances.length; ) Oe = this._distances[++Ae];
                    const Ue = Ae - 1, nt = this._distances[Ue], ht = Oe - nt, Et = ht > 0 ? (Be - nt) / ht : 0;
                    return this.points[Ue].mult(1 - Et).add(this.points[Ae].mult(Et));
                  }
                }
                function jt(Fe, Te) {
                  let Ae = !0;
                  return Fe === "always" || Fe !== "never" && Te !== "never" || (Ae = !1), Ae;
                }
                class cn {
                  constructor(Te, Ae, Oe) {
                    const Be = this.boxCells = [], Ue = this.circleCells = [];
                    this.xCellCount = Math.ceil(Te / Oe), this.yCellCount = Math.ceil(Ae / Oe);
                    for (let nt = 0; nt < this.xCellCount * this.yCellCount; nt++) Be.push([]), Ue.push([]);
                    this.circleKeys = [], this.boxKeys = [], this.bboxes = [], this.circles = [], this.width = Te, this.height = Ae, this.xScale = this.xCellCount / Te, this.yScale = this.yCellCount / Ae, this.boxUid = 0, this.circleUid = 0;
                  }
                  keysLength() {
                    return this.boxKeys.length + this.circleKeys.length;
                  }
                  insert(Te, Ae, Oe, Be, Ue) {
                    this._forEachCell(Ae, Oe, Be, Ue, this._insertBoxCell, this.boxUid++), this.boxKeys.push(Te), this.bboxes.push(Ae), this.bboxes.push(Oe), this.bboxes.push(Be), this.bboxes.push(Ue);
                  }
                  insertCircle(Te, Ae, Oe, Be) {
                    this._forEachCell(Ae - Be, Oe - Be, Ae + Be, Oe + Be, this._insertCircleCell, this.circleUid++), this.circleKeys.push(Te), this.circles.push(Ae), this.circles.push(Oe), this.circles.push(Be);
                  }
                  _insertBoxCell(Te, Ae, Oe, Be, Ue, nt) {
                    this.boxCells[Ue].push(nt);
                  }
                  _insertCircleCell(Te, Ae, Oe, Be, Ue, nt) {
                    this.circleCells[Ue].push(nt);
                  }
                  _query(Te, Ae, Oe, Be, Ue, nt, ht) {
                    if (Oe < 0 || Te > this.width || Be < 0 || Ae > this.height) return [];
                    const Et = [];
                    if (Te <= 0 && Ae <= 0 && this.width <= Oe && this.height <= Be) {
                      if (Ue) return [{ key: null, x1: Te, y1: Ae, x2: Oe, y2: Be }];
                      for (let Mt = 0; Mt < this.boxKeys.length; Mt++) Et.push({ key: this.boxKeys[Mt], x1: this.bboxes[4 * Mt], y1: this.bboxes[4 * Mt + 1], x2: this.bboxes[4 * Mt + 2], y2: this.bboxes[4 * Mt + 3] });
                      for (let Mt = 0; Mt < this.circleKeys.length; Mt++) {
                        const qt = this.circles[3 * Mt], tn = this.circles[3 * Mt + 1], xn = this.circles[3 * Mt + 2];
                        Et.push({ key: this.circleKeys[Mt], x1: qt - xn, y1: tn - xn, x2: qt + xn, y2: tn + xn });
                      }
                    } else {
                      const Mt = { hitTest: Ue, overlapMode: nt, seenUids: { box: {}, circle: {} } };
                      this._forEachCell(Te, Ae, Oe, Be, this._queryCell, Et, Mt, ht);
                    }
                    return Et;
                  }
                  query(Te, Ae, Oe, Be) {
                    return this._query(Te, Ae, Oe, Be, !1, null);
                  }
                  hitTest(Te, Ae, Oe, Be, Ue, nt) {
                    return this._query(Te, Ae, Oe, Be, !0, Ue, nt).length > 0;
                  }
                  hitTestCircle(Te, Ae, Oe, Be, Ue) {
                    const nt = Te - Oe, ht = Te + Oe, Et = Ae - Oe, Mt = Ae + Oe;
                    if (ht < 0 || nt > this.width || Mt < 0 || Et > this.height) return !1;
                    const qt = [], tn = { hitTest: !0, overlapMode: Be, circle: { x: Te, y: Ae, radius: Oe }, seenUids: { box: {}, circle: {} } };
                    return this._forEachCell(nt, Et, ht, Mt, this._queryCellCircle, qt, tn, Ue), qt.length > 0;
                  }
                  _queryCell(Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                    const { seenUids: Mt, hitTest: qt, overlapMode: tn } = ht, xn = this.boxCells[Ue];
                    if (xn !== null) {
                      const On = this.bboxes;
                      for (const Bn of xn) if (!Mt.box[Bn]) {
                        Mt.box[Bn] = !0;
                        const qn = 4 * Bn, Vn = this.boxKeys[Bn];
                        if (Te <= On[qn + 2] && Ae <= On[qn + 3] && Oe >= On[qn + 0] && Be >= On[qn + 1] && (!Et || Et(Vn)) && (!qt || !jt(tn, Vn.overlapMode)) && (nt.push({ key: Vn, x1: On[qn], y1: On[qn + 1], x2: On[qn + 2], y2: On[qn + 3] }), qt)) return !0;
                      }
                    }
                    const An = this.circleCells[Ue];
                    if (An !== null) {
                      const On = this.circles;
                      for (const Bn of An) if (!Mt.circle[Bn]) {
                        Mt.circle[Bn] = !0;
                        const qn = 3 * Bn, Vn = this.circleKeys[Bn];
                        if (this._circleAndRectCollide(On[qn], On[qn + 1], On[qn + 2], Te, Ae, Oe, Be) && (!Et || Et(Vn)) && (!qt || !jt(tn, Vn.overlapMode))) {
                          const sr = On[qn], Un = On[qn + 1], wr = On[qn + 2];
                          if (nt.push({ key: Vn, x1: sr - wr, y1: Un - wr, x2: sr + wr, y2: Un + wr }), qt) return !0;
                        }
                      }
                    }
                    return !1;
                  }
                  _queryCellCircle(Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                    const { circle: Mt, seenUids: qt, overlapMode: tn } = ht, xn = this.boxCells[Ue];
                    if (xn !== null) {
                      const On = this.bboxes;
                      for (const Bn of xn) if (!qt.box[Bn]) {
                        qt.box[Bn] = !0;
                        const qn = 4 * Bn, Vn = this.boxKeys[Bn];
                        if (this._circleAndRectCollide(Mt.x, Mt.y, Mt.radius, On[qn + 0], On[qn + 1], On[qn + 2], On[qn + 3]) && (!Et || Et(Vn)) && !jt(tn, Vn.overlapMode)) return nt.push(!0), !0;
                      }
                    }
                    const An = this.circleCells[Ue];
                    if (An !== null) {
                      const On = this.circles;
                      for (const Bn of An) if (!qt.circle[Bn]) {
                        qt.circle[Bn] = !0;
                        const qn = 3 * Bn, Vn = this.circleKeys[Bn];
                        if (this._circlesCollide(On[qn], On[qn + 1], On[qn + 2], Mt.x, Mt.y, Mt.radius) && (!Et || Et(Vn)) && !jt(tn, Vn.overlapMode)) return nt.push(!0), !0;
                      }
                    }
                  }
                  _forEachCell(Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                    const Mt = this._convertToXCellCoord(Te), qt = this._convertToYCellCoord(Ae), tn = this._convertToXCellCoord(Oe), xn = this._convertToYCellCoord(Be);
                    for (let An = Mt; An <= tn; An++) for (let On = qt; On <= xn; On++) {
                      const Bn = this.xCellCount * On + An;
                      if (Ue.call(this, Te, Ae, Oe, Be, Bn, nt, ht, Et)) return;
                    }
                  }
                  _convertToXCellCoord(Te) {
                    return Math.max(0, Math.min(this.xCellCount - 1, Math.floor(Te * this.xScale)));
                  }
                  _convertToYCellCoord(Te) {
                    return Math.max(0, Math.min(this.yCellCount - 1, Math.floor(Te * this.yScale)));
                  }
                  _circlesCollide(Te, Ae, Oe, Be, Ue, nt) {
                    const ht = Be - Te, Et = Ue - Ae, Mt = Oe + nt;
                    return Mt * Mt > ht * ht + Et * Et;
                  }
                  _circleAndRectCollide(Te, Ae, Oe, Be, Ue, nt, ht) {
                    const Et = (nt - Be) / 2, Mt = Math.abs(Te - (Be + Et));
                    if (Mt > Et + Oe) return !1;
                    const qt = (ht - Ue) / 2, tn = Math.abs(Ae - (Ue + qt));
                    if (tn > qt + Oe) return !1;
                    if (Mt <= Et || tn <= qt) return !0;
                    const xn = Mt - Et, An = tn - qt;
                    return xn * xn + An * An <= Oe * Oe;
                  }
                }
                function wn(Fe, Te, Ae, Oe, Be) {
                  const Ue = ee.H();
                  return Te ? (ee.K(Ue, Ue, [1 / Be, 1 / Be, 1]), Ae || ee.ae(Ue, Ue, Oe.angle)) : ee.L(Ue, Oe.labelPlaneMatrix, Fe), Ue;
                }
                function Rn(Fe, Te, Ae, Oe, Be) {
                  if (Te) {
                    const Ue = ee.af(Fe);
                    return ee.K(Ue, Ue, [Be, Be, 1]), Ae || ee.ae(Ue, Ue, -Oe.angle), Ue;
                  }
                  return Oe.glCoordMatrix;
                }
                function Pn(Fe, Te, Ae) {
                  let Oe;
                  Ae ? (Oe = [Fe.x, Fe.y, Ae(Fe.x, Fe.y), 1], ee.ag(Oe, Oe, Te)) : (Oe = [Fe.x, Fe.y, 0, 1], function(Ue, nt, ht) {
                    const Et = nt[0], Mt = nt[1];
                    Ue[0] = ht[0] * Et + ht[4] * Mt + ht[12], Ue[1] = ht[1] * Et + ht[5] * Mt + ht[13], Ue[3] = ht[3] * Et + ht[7] * Mt + ht[15];
                  }(Oe, Oe, Te));
                  const Be = Oe[3];
                  return { point: new ee.P(Oe[0] / Be, Oe[1] / Be), signedDistanceFromCamera: Be, isOccluded: !1 };
                }
                function Ie(Fe, Te) {
                  return 0.5 + Fe / Te * 0.5;
                }
                function rt(Fe, Te) {
                  return Fe.x >= -Te[0] && Fe.x <= Te[0] && Fe.y >= -Te[1] && Fe.y <= Te[1];
                }
                function ft(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An, On) {
                  const Bn = Oe ? Fe.textSizeData : Fe.iconSizeData, qn = ee.ah(Bn, Ae.transform.zoom), Vn = [256 / Ae.width * 2 + 1, 256 / Ae.height * 2 + 1], sr = Oe ? Fe.text.dynamicLayoutVertexArray : Fe.icon.dynamicLayoutVertexArray;
                  sr.clear();
                  const Un = Fe.lineVertexArray, wr = Oe ? Fe.text.placedSymbolArray : Fe.icon.placedSymbolArray, Cr = Ae.transform.width / Ae.transform.height;
                  let Tr = !1;
                  for (let ii = 0; ii < wr.length; ii++) {
                    const wi = wr.get(ii);
                    if (wi.hidden || wi.writingMode === ee.ai.vertical && !Tr) {
                      bn(wi.numGlyphs, sr);
                      continue;
                    }
                    Tr = !1;
                    const Wi = Pn(new ee.P(wi.anchorX, wi.anchorY), Te, On);
                    if (!rt(Wi.point, Vn)) {
                      bn(wi.numGlyphs, sr);
                      continue;
                    }
                    const Yi = Wi.signedDistanceFromCamera, ra = Ie(Ae.transform.cameraToCenterDistance, Yi), ha = ee.aj(Bn, qn, wi), ya = nt ? ha / ra : ha * ra, va = { getElevation: On, labelPlaneMatrix: Be, lineVertexArray: Un, pitchWithMap: nt, projectionCache: { projections: {}, offsets: {}, cachedAnchorPoint: void 0, anyProjectionOccluded: !1 }, projection: Mt, tileAnchorPoint: new ee.P(wi.anchorX, wi.anchorY), unwrappedTileID: qt, width: tn, height: xn, translation: An }, qa = Bt(va, wi, ya, !1, ht, Te, Ue, Fe.glyphOffsetArray, sr, Cr, Et);
                    Tr = qa.useVertical, (qa.notEnoughRoom || Tr || qa.needsFlipping && Bt(va, wi, ya, !0, ht, Te, Ue, Fe.glyphOffsetArray, sr, Cr, Et).notEnoughRoom) && bn(wi.numGlyphs, sr);
                  }
                  Oe ? Fe.text.dynamicLayoutVertexBuffer.updateData(sr) : Fe.icon.dynamicLayoutVertexBuffer.updateData(sr);
                }
                function pt(Fe, Te, Ae, Oe, Be, Ue, nt, ht) {
                  const Et = Ue.glyphStartIndex + Ue.numGlyphs, Mt = Ue.lineStartIndex, qt = Ue.lineStartIndex + Ue.lineLength, tn = Te.getoffsetX(Ue.glyphStartIndex), xn = Te.getoffsetX(Et - 1), An = _n(Fe * tn, Ae, Oe, Be, Ue.segment, Mt, qt, ht, nt);
                  if (!An) return null;
                  const On = _n(Fe * xn, Ae, Oe, Be, Ue.segment, Mt, qt, ht, nt);
                  return On ? ht.projectionCache.anyProjectionOccluded ? null : { first: An, last: On } : null;
                }
                function wt(Fe, Te, Ae, Oe) {
                  return Fe === ee.ai.horizontal && Math.abs(Ae.y - Te.y) > Math.abs(Ae.x - Te.x) * Oe ? { useVertical: !0 } : (Fe === ee.ai.vertical ? Te.y < Ae.y : Te.x > Ae.x) ? { needsFlipping: !0 } : null;
                }
                function Bt(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt) {
                  const tn = Ae / 24, xn = Te.lineOffsetX * tn, An = Te.lineOffsetY * tn;
                  let On;
                  if (Te.numGlyphs > 1) {
                    const Bn = Te.glyphStartIndex + Te.numGlyphs, qn = Te.lineStartIndex, Vn = Te.lineStartIndex + Te.lineLength, sr = pt(tn, ht, xn, An, Oe, Te, qt, Fe);
                    if (!sr) return { notEnoughRoom: !0 };
                    const Un = Pn(sr.first.point, nt, Fe.getElevation).point, wr = Pn(sr.last.point, nt, Fe.getElevation).point;
                    if (Be && !Oe) {
                      const Cr = wt(Te.writingMode, Un, wr, Mt);
                      if (Cr) return Cr;
                    }
                    On = [sr.first];
                    for (let Cr = Te.glyphStartIndex + 1; Cr < Bn - 1; Cr++) On.push(_n(tn * ht.getoffsetX(Cr), xn, An, Oe, Te.segment, qn, Vn, Fe, qt));
                    On.push(sr.last);
                  } else {
                    if (Be && !Oe) {
                      const qn = Pn(Fe.tileAnchorPoint, Ue, Fe.getElevation).point, Vn = Te.lineStartIndex + Te.segment + 1, sr = new ee.P(Fe.lineVertexArray.getx(Vn), Fe.lineVertexArray.gety(Vn)), Un = Pn(sr, Ue, Fe.getElevation), wr = Un.signedDistanceFromCamera > 0 ? Un.point : function(Tr, ii, wi, Wi, Yi, ra) {
                        return At(Tr, ii, wi, Wi, Yi, ra);
                      }(Fe.tileAnchorPoint, sr, qn, 1, Ue, Fe), Cr = wt(Te.writingMode, qn, wr, Mt);
                      if (Cr) return Cr;
                    }
                    const Bn = _n(tn * ht.getoffsetX(Te.glyphStartIndex), xn, An, Oe, Te.segment, Te.lineStartIndex, Te.lineStartIndex + Te.lineLength, Fe, qt);
                    if (!Bn || Fe.projectionCache.anyProjectionOccluded) return { notEnoughRoom: !0 };
                    On = [Bn];
                  }
                  for (const Bn of On) ee.ak(Et, Bn.point, Bn.angle);
                  return {};
                }
                function At(Fe, Te, Ae, Oe, Be, Ue) {
                  const nt = Fe.add(Fe.sub(Te)._unit()), ht = Be !== void 0 ? Pn(nt, Be, Ue.getElevation).point : Ot(nt.x, nt.y, Ue).point, Et = Ae.sub(ht);
                  return Ae.add(Et._mult(Oe / Et.mag()));
                }
                function Dt(Fe, Te, Ae) {
                  const Oe = Te.projectionCache;
                  if (Oe.projections[Fe]) return Oe.projections[Fe];
                  const Be = new ee.P(Te.lineVertexArray.getx(Fe), Te.lineVertexArray.gety(Fe)), Ue = Ot(Be.x, Be.y, Te);
                  if (Ue.signedDistanceFromCamera > 0) return Oe.projections[Fe] = Ue.point, Oe.anyProjectionOccluded = Oe.anyProjectionOccluded || Ue.isOccluded, Ue.point;
                  const nt = Fe - Ae.direction, ht = Ae.distanceFromAnchor === 0 ? Te.tileAnchorPoint : new ee.P(Te.lineVertexArray.getx(nt), Te.lineVertexArray.gety(nt)), Et = Ae.absOffsetX - Ae.distanceFromAnchor + 1;
                  return function(Mt, qt, tn, xn, An) {
                    return At(Mt, qt, tn, xn, void 0, An);
                  }(ht, Be, Ae.previousVertex, Et, Te);
                }
                function Ot(Fe, Te, Ae) {
                  const Oe = Fe + Ae.translation[0], Be = Te + Ae.translation[1];
                  let Ue;
                  return !Ae.pitchWithMap && Ae.projection.useSpecialProjectionForSymbols ? (Ue = Ae.projection.projectTileCoordinates(Oe, Be, Ae.unwrappedTileID, Ae.getElevation), Ue.point.x = (0.5 * Ue.point.x + 0.5) * Ae.width, Ue.point.y = (0.5 * -Ue.point.y + 0.5) * Ae.height) : (Ue = Pn(new ee.P(Oe, Be), Ae.labelPlaneMatrix, Ae.getElevation), Ue.isOccluded = !1), Ue;
                }
                function un(Fe, Te, Ae) {
                  return Fe._unit()._perp()._mult(Te * Ae);
                }
                function yn(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                  if (ht.projectionCache.offsets[Fe]) return ht.projectionCache.offsets[Fe];
                  const Mt = Ae.add(Te);
                  if (Fe + Et.direction < Oe || Fe + Et.direction >= Be) return ht.projectionCache.offsets[Fe] = Mt, Mt;
                  const qt = Dt(Fe + Et.direction, ht, Et), tn = un(qt.sub(Ae), nt, Et.direction), xn = Ae.add(tn), An = qt.add(tn);
                  return ht.projectionCache.offsets[Fe] = ee.al(Ue, Mt, xn, An) || Mt, ht.projectionCache.offsets[Fe];
                }
                function _n(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                  const Mt = Oe ? Fe - Te : Fe + Te;
                  let qt = Mt > 0 ? 1 : -1, tn = 0;
                  Oe && (qt *= -1, tn = Math.PI), qt < 0 && (tn += Math.PI);
                  let xn, An = qt > 0 ? Ue + Be : Ue + Be + 1;
                  ht.projectionCache.cachedAnchorPoint ? xn = ht.projectionCache.cachedAnchorPoint : (xn = Ot(ht.tileAnchorPoint.x, ht.tileAnchorPoint.y, ht).point, ht.projectionCache.cachedAnchorPoint = xn);
                  let On, Bn, qn = xn, Vn = xn, sr = 0, Un = 0;
                  const wr = Math.abs(Mt), Cr = [];
                  let Tr;
                  for (; sr + Un <= wr; ) {
                    if (An += qt, An < Ue || An >= nt) return null;
                    sr += Un, Vn = qn, Bn = On;
                    const Yi = { absOffsetX: wr, direction: qt, distanceFromAnchor: sr, previousVertex: Vn };
                    if (qn = Dt(An, ht, Yi), Ae === 0) Cr.push(Vn), Tr = qn.sub(Vn);
                    else {
                      let ra;
                      const ha = qn.sub(Vn);
                      ra = ha.mag() === 0 ? un(Dt(An + qt, ht, Yi).sub(qn), Ae, qt) : un(ha, Ae, qt), Bn || (Bn = Vn.add(ra)), On = yn(An, ra, qn, Ue, nt, Bn, Ae, ht, Yi), Cr.push(Bn), Tr = On.sub(Bn);
                    }
                    Un = Tr.mag();
                  }
                  const ii = (wr - sr) / Un, wi = Tr._mult(ii)._add(Bn || Vn), Wi = tn + Math.atan2(qn.y - Vn.y, qn.x - Vn.x);
                  return Cr.push(wi), { point: wi, angle: Et ? Wi : 0, path: Cr };
                }
                const gn = new Float32Array([-1 / 0, -1 / 0, 0, -1 / 0, -1 / 0, 0, -1 / 0, -1 / 0, 0, -1 / 0, -1 / 0, 0]);
                function bn(Fe, Te) {
                  for (let Ae = 0; Ae < Fe; Ae++) {
                    const Oe = Te.length;
                    Te.resize(Oe + 4), Te.float32.set(gn, 3 * Oe);
                  }
                }
                const an = 100;
                class Zt {
                  constructor(Te, Ae, Oe = new cn(Te.width + 200, Te.height + 200, 25), Be = new cn(Te.width + 200, Te.height + 200, 25)) {
                    this.transform = Te, this.mapProjection = Ae, this.grid = Oe, this.ignoredGrid = Be, this.pitchFactor = Math.cos(Te._pitch) * Te.cameraToCenterDistance, this.screenRightBoundary = Te.width + an, this.screenBottomBoundary = Te.height + an, this.gridRightBoundary = Te.width + 200, this.gridBottomBoundary = Te.height + 200, this.perspectiveRatioCutoff = 0.6;
                  }
                  placeCollisionBox(Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn) {
                    const xn = Te.anchorPointX + Et[0], An = Te.anchorPointY + Et[1], On = this.projectAndGetPerspectiveRatio(Be, xn, An, Ue, qt), Bn = this._projectCollisionBox(Te, Oe, Be, Ue, nt, ht, Et, On, qt, tn), [qn, Vn, sr, Un] = Bn.box;
                    return this.mapProjection.useSpecialProjectionForSymbols && (nt ? Bn.allPointsOccluded : this.mapProjection.isOccluded(xn, An, Ue)) || On.perspectiveRatio < this.perspectiveRatioCutoff || !this.isInsideGrid(qn, Vn, sr, Un) || Ae !== "always" && this.grid.hitTest(qn, Vn, sr, Un, Ae, Mt) ? { box: [qn, Vn, sr, Un], placeable: !1, offscreen: !1 } : { box: [qn, Vn, sr, Un], placeable: !0, offscreen: this.isOffscreen(qn, Vn, sr, Un) };
                  }
                  placeCollisionCircles(Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An, On, Bn, qn) {
                    const Vn = [], sr = new ee.P(Ae.anchorX, Ae.anchorY), Un = this.getPerspectiveRatio(nt, sr.x, sr.y, ht, qn), wr = (tn ? Ue / Un : Ue * Un) / ee.aq, Cr = Ae.lineOffsetX * wr, Tr = Ae.lineOffsetY * wr, ii = { getElevation: qn, labelPlaneMatrix: Et, lineVertexArray: Oe, pitchWithMap: tn, projectionCache: { projections: {}, offsets: {}, cachedAnchorPoint: void 0, anyProjectionOccluded: !1 }, projection: this.mapProjection, tileAnchorPoint: sr, unwrappedTileID: ht, width: this.transform.width, height: this.transform.height, translation: Bn }, wi = pt(wr, Be, Cr, Tr, !1, Ae, !1, ii);
                    let Wi = !1, Yi = !1, ra = !0;
                    if (wi) {
                      const ha = 0.5 * An * Un + On, ya = new ee.P(-100, -100), va = new ee.P(this.screenRightBoundary, this.screenBottomBoundary), qa = new Ht(), Aa = wi.first, Xa = wi.last;
                      let Oa = [];
                      for (let Ja = Aa.path.length - 1; Ja >= 1; Ja--) Oa.push(Aa.path[Ja]);
                      for (let Ja = 1; Ja < Xa.path.length; Ja++) Oa.push(Xa.path[Ja]);
                      const Ya = 2.5 * ha;
                      if (Mt) {
                        const Ja = this.projectPathToScreenSpace(Oa, ii, Mt);
                        Oa = Ja.some((Uo) => Uo.signedDistanceFromCamera <= 0) ? [] : Ja.map((Uo) => Uo.point);
                      }
                      let Wo = [];
                      if (Oa.length > 0) {
                        const Ja = Oa[0].clone(), Uo = Oa[0].clone();
                        for (let ys = 1; ys < Oa.length; ys++) Ja.x = Math.min(Ja.x, Oa[ys].x), Ja.y = Math.min(Ja.y, Oa[ys].y), Uo.x = Math.max(Uo.x, Oa[ys].x), Uo.y = Math.max(Uo.y, Oa[ys].y);
                        Wo = Ja.x >= ya.x && Uo.x <= va.x && Ja.y >= ya.y && Uo.y <= va.y ? [Oa] : Uo.x < ya.x || Ja.x > va.x || Uo.y < ya.y || Ja.y > va.y ? [] : ee.am([Oa], ya.x, ya.y, va.x, va.y);
                      }
                      for (const Ja of Wo) {
                        qa.reset(Ja, 0.25 * ha);
                        let Uo = 0;
                        Uo = qa.length <= 0.5 * ha ? 1 : Math.ceil(qa.paddedLength / Ya) + 1;
                        for (let ys = 0; ys < Uo; ys++) {
                          const il = ys / Math.max(Uo - 1, 1), Is = qa.lerp(il), ml = Is.x + an, nl = Is.y + an;
                          Vn.push(ml, nl, ha, 0);
                          const Il = ml - ha, Fl = nl - ha, qc = ml + ha, ql = nl + ha;
                          if (ra = ra && this.isOffscreen(Il, Fl, qc, ql), Yi = Yi || this.isInsideGrid(Il, Fl, qc, ql), Te !== "always" && this.grid.hitTestCircle(ml, nl, ha, Te, xn) && (Wi = !0, !qt)) return { circles: [], offscreen: !1, collisionDetected: Wi };
                        }
                      }
                    }
                    return { circles: !qt && Wi || !Yi || Un < this.perspectiveRatioCutoff ? [] : Vn, offscreen: ra, collisionDetected: Wi };
                  }
                  projectPathToScreenSpace(Te, Ae, Oe) {
                    return Te.map((Be) => Pn(Be, Oe, Ae.getElevation));
                  }
                  queryRenderedSymbols(Te) {
                    if (Te.length === 0 || this.grid.keysLength() === 0 && this.ignoredGrid.keysLength() === 0) return {};
                    const Ae = [];
                    let Oe = 1 / 0, Be = 1 / 0, Ue = -1 / 0, nt = -1 / 0;
                    for (const qt of Te) {
                      const tn = new ee.P(qt.x + an, qt.y + an);
                      Oe = Math.min(Oe, tn.x), Be = Math.min(Be, tn.y), Ue = Math.max(Ue, tn.x), nt = Math.max(nt, tn.y), Ae.push(tn);
                    }
                    const ht = this.grid.query(Oe, Be, Ue, nt).concat(this.ignoredGrid.query(Oe, Be, Ue, nt)), Et = {}, Mt = {};
                    for (const qt of ht) {
                      const tn = qt.key;
                      if (Et[tn.bucketInstanceId] === void 0 && (Et[tn.bucketInstanceId] = {}), Et[tn.bucketInstanceId][tn.featureIndex]) continue;
                      const xn = [new ee.P(qt.x1, qt.y1), new ee.P(qt.x2, qt.y1), new ee.P(qt.x2, qt.y2), new ee.P(qt.x1, qt.y2)];
                      ee.an(Ae, xn) && (Et[tn.bucketInstanceId][tn.featureIndex] = !0, Mt[tn.bucketInstanceId] === void 0 && (Mt[tn.bucketInstanceId] = []), Mt[tn.bucketInstanceId].push(tn.featureIndex));
                    }
                    return Mt;
                  }
                  insertCollisionBox(Te, Ae, Oe, Be, Ue, nt) {
                    const ht = { bucketInstanceId: Be, featureIndex: Ue, collisionGroupID: nt, overlapMode: Ae };
                    (Oe ? this.ignoredGrid : this.grid).insert(ht, Te[0], Te[1], Te[2], Te[3]);
                  }
                  insertCollisionCircles(Te, Ae, Oe, Be, Ue, nt) {
                    const ht = Oe ? this.ignoredGrid : this.grid, Et = { bucketInstanceId: Be, featureIndex: Ue, collisionGroupID: nt, overlapMode: Ae };
                    for (let Mt = 0; Mt < Te.length; Mt += 4) ht.insertCircle(Et, Te[Mt], Te[Mt + 1], Te[Mt + 2]);
                  }
                  projectAndGetPerspectiveRatio(Te, Ae, Oe, Be, Ue) {
                    const nt = this.mapProjection.useSpecialProjectionForSymbols ? this.mapProjection.projectTileCoordinates(Ae, Oe, Be, Ue) : Pn(new ee.P(Ae, Oe), Te, Ue);
                    return { point: new ee.P((nt.point.x + 1) / 2 * this.transform.width + an, (1 - nt.point.y) / 2 * this.transform.height + an), perspectiveRatio: 0.5 + this.transform.cameraToCenterDistance / nt.signedDistanceFromCamera * 0.5, isOccluded: nt.isOccluded, signedDistanceFromCamera: nt.signedDistanceFromCamera };
                  }
                  getPerspectiveRatio(Te, Ae, Oe, Be, Ue) {
                    const nt = this.mapProjection.useSpecialProjectionForSymbols ? this.mapProjection.projectTileCoordinates(Ae, Oe, Be, Ue) : Pn(new ee.P(Ae, Oe), Te, Ue);
                    return 0.5 + this.transform.cameraToCenterDistance / nt.signedDistanceFromCamera * 0.5;
                  }
                  isOffscreen(Te, Ae, Oe, Be) {
                    return Oe < an || Te >= this.screenRightBoundary || Be < an || Ae > this.screenBottomBoundary;
                  }
                  isInsideGrid(Te, Ae, Oe, Be) {
                    return Oe >= 0 && Te < this.gridRightBoundary && Be >= 0 && Ae < this.gridBottomBoundary;
                  }
                  getViewportMatrix() {
                    const Te = ee.ao([]);
                    return ee.J(Te, Te, [-100, -100, 0]), Te;
                  }
                  _projectCollisionBox(Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt) {
                    const tn = Ae * Et.perspectiveRatio;
                    let xn = new ee.P(1, 0), An = new ee.P(0, 1);
                    const On = new ee.P(Te.anchorPointX + ht[0], Te.anchorPointY + ht[1]);
                    if (nt && !Ue) {
                      const Yi = this.projectAndGetPerspectiveRatio(Oe, On.x + 1, On.y, Be, Mt).point.sub(Et.point).unit(), ra = Math.atan(Yi.y / Yi.x) + (Yi.x < 0 ? Math.PI : 0), ha = Math.sin(ra), ya = Math.cos(ra);
                      xn = new ee.P(ya, ha), An = new ee.P(-ha, ya);
                    } else if (!nt && Ue) {
                      const Yi = -this.transform.angle, ra = Math.sin(Yi), ha = Math.cos(Yi);
                      xn = new ee.P(ha, ra), An = new ee.P(-ra, ha);
                    }
                    let Bn = Et.point, qn = tn;
                    if (Ue) {
                      Bn = On;
                      const Yi = this.transform.zoom - Math.floor(this.transform.zoom);
                      if (qn = Math.pow(2, -Yi), qn *= this.mapProjection.getPitchedTextCorrection(this.transform, On, Be), !qt) {
                        const ra = Et.signedDistanceFromCamera / this.transform.cameraToCenterDistance;
                        qn *= ee.ad(0.5 + 0.5 * ra, 0, 4);
                      }
                    }
                    qt && (Bn = Bn.add(xn.mult(qt.x * qn)).add(An.mult(qt.y * qn)));
                    const Vn = Te.x1 * qn, sr = Te.x2 * qn, Un = (Vn + sr) / 2, wr = Te.y1 * qn, Cr = Te.y2 * qn, Tr = (wr + Cr) / 2, ii = [{ offsetX: Vn, offsetY: wr }, { offsetX: Un, offsetY: wr }, { offsetX: sr, offsetY: wr }, { offsetX: sr, offsetY: Tr }, { offsetX: sr, offsetY: Cr }, { offsetX: Un, offsetY: Cr }, { offsetX: Vn, offsetY: Cr }, { offsetX: Vn, offsetY: Tr }];
                    let wi = [];
                    for (const { offsetX: Yi, offsetY: ra } of ii) wi.push(new ee.P(Bn.x + xn.x * Yi + An.x * ra, Bn.y + xn.y * Yi + An.y * ra));
                    let Wi = !1;
                    if (Ue) {
                      const Yi = wi.map((ra) => this.projectAndGetPerspectiveRatio(Oe, ra.x, ra.y, Be, Mt));
                      Wi = Yi.some((ra) => !ra.isOccluded), wi = Yi.map((ra) => ra.point);
                    } else Wi = !0;
                    return { box: ee.ap(wi), allPointsOccluded: !Wi };
                  }
                }
                function Mn(Fe, Te, Ae) {
                  return Te * (ee.X / (Fe.tileSize * Math.pow(2, Ae - Fe.tileID.overscaledZ)));
                }
                class Ln {
                  constructor(Te, Ae, Oe, Be) {
                    this.opacity = Te ? Math.max(0, Math.min(1, Te.opacity + (Te.placed ? Ae : -Ae))) : Be && Oe ? 1 : 0, this.placed = Oe;
                  }
                  isHidden() {
                    return this.opacity === 0 && !this.placed;
                  }
                }
                class In {
                  constructor(Te, Ae, Oe, Be, Ue) {
                    this.text = new Ln(Te ? Te.text : null, Ae, Oe, Ue), this.icon = new Ln(Te ? Te.icon : null, Ae, Be, Ue);
                  }
                  isHidden() {
                    return this.text.isHidden() && this.icon.isHidden();
                  }
                }
                class kn {
                  constructor(Te, Ae, Oe) {
                    this.text = Te, this.icon = Ae, this.skipFade = Oe;
                  }
                }
                class er {
                  constructor() {
                    this.invProjMatrix = ee.H(), this.viewportMatrix = ee.H(), this.circles = [];
                  }
                }
                class jn {
                  constructor(Te, Ae, Oe, Be, Ue) {
                    this.bucketInstanceId = Te, this.featureIndex = Ae, this.sourceLayerIndex = Oe, this.bucketIndex = Be, this.tileID = Ue;
                  }
                }
                class ur {
                  constructor(Te) {
                    this.crossSourceCollisions = Te, this.maxGroupID = 0, this.collisionGroups = {};
                  }
                  get(Te) {
                    if (this.crossSourceCollisions) return { ID: 0, predicate: null };
                    if (!this.collisionGroups[Te]) {
                      const Ae = ++this.maxGroupID;
                      this.collisionGroups[Te] = { ID: Ae, predicate: (Oe) => Oe.collisionGroupID === Ae };
                    }
                    return this.collisionGroups[Te];
                  }
                }
                function Kn(Fe, Te, Ae, Oe, Be) {
                  const { horizontalAlign: Ue, verticalAlign: nt } = ee.av(Fe), ht = -(Ue - 0.5) * Te, Et = -(nt - 0.5) * Ae;
                  return new ee.P(ht + Oe[0] * Be, Et + Oe[1] * Be);
                }
                class Jn {
                  constructor(Te, Ae, Oe, Be, Ue, nt) {
                    this.transform = Te.clone(), this.terrain = Oe, this.collisionIndex = new Zt(this.transform, Ae), this.placements = {}, this.opacities = {}, this.variableOffsets = {}, this.stale = !1, this.commitTime = 0, this.fadeDuration = Be, this.retainedQueryData = {}, this.collisionGroups = new ur(Ue), this.collisionCircleArrays = {}, this.collisionBoxArrays = /* @__PURE__ */ new Map(), this.prevPlacement = nt, nt && (nt.prevPlacement = void 0), this.placedOrientations = {};
                  }
                  _getTerrainElevationFunc(Te) {
                    const Ae = this.terrain;
                    return Ae ? (Oe, Be) => Ae.getElevation(Te, Oe, Be) : null;
                  }
                  getBucketParts(Te, Ae, Oe, Be) {
                    const Ue = Oe.getBucket(Ae), nt = Oe.latestFeatureIndex;
                    if (!Ue || !nt || Ae.id !== Ue.layerIds[0]) return;
                    const ht = Oe.collisionBoxArray, Et = Ue.layers[0].layout, Mt = Ue.layers[0].paint, qt = Math.pow(2, this.transform.zoom - Oe.tileID.overscaledZ), tn = Oe.tileSize / ee.X, xn = Oe.tileID.toUnwrapped(), An = this.transform.calculatePosMatrix(xn), On = Et.get("text-pitch-alignment") === "map", Bn = Et.get("text-rotation-alignment") === "map", qn = Mn(Oe, 1, this.transform.zoom), Vn = this.collisionIndex.mapProjection.translatePosition(this.transform, Oe, Mt.get("text-translate"), Mt.get("text-translate-anchor")), sr = this.collisionIndex.mapProjection.translatePosition(this.transform, Oe, Mt.get("icon-translate"), Mt.get("icon-translate-anchor")), Un = wn(An, On, Bn, this.transform, qn);
                    let wr = null;
                    if (On) {
                      const Tr = Rn(An, On, Bn, this.transform, qn);
                      wr = ee.L([], this.transform.labelPlaneMatrix, Tr);
                    }
                    this.retainedQueryData[Ue.bucketInstanceId] = new jn(Ue.bucketInstanceId, nt, Ue.sourceLayerIndex, Ue.index, Oe.tileID);
                    const Cr = { bucket: Ue, layout: Et, translationText: Vn, translationIcon: sr, posMatrix: An, unwrappedTileID: xn, textLabelPlaneMatrix: Un, labelToScreenMatrix: wr, scale: qt, textPixelRatio: tn, holdingForFade: Oe.holdingForFade(), collisionBoxArray: ht, partiallyEvaluatedTextSize: ee.ah(Ue.textSizeData, this.transform.zoom), collisionGroup: this.collisionGroups.get(Ue.sourceID) };
                    if (Be) for (const Tr of Ue.sortKeyRanges) {
                      const { sortKey: ii, symbolInstanceStart: wi, symbolInstanceEnd: Wi } = Tr;
                      Te.push({ sortKey: ii, symbolInstanceStart: wi, symbolInstanceEnd: Wi, parameters: Cr });
                    }
                    else Te.push({ symbolInstanceStart: 0, symbolInstanceEnd: Ue.symbolInstances.length, parameters: Cr });
                  }
                  attemptAnchorPlacement(Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An, On, Bn, qn, Vn, sr, Un) {
                    const wr = ee.ar[Te.textAnchor], Cr = [Te.textOffset0, Te.textOffset1], Tr = Kn(wr, Oe, Be, Cr, Ue), ii = this.collisionIndex.placeCollisionBox(Ae, xn, Et, Mt, qt, ht, nt, qn, tn.predicate, Un, Tr);
                    if ((!sr || this.collisionIndex.placeCollisionBox(sr, xn, Et, Mt, qt, ht, nt, Vn, tn.predicate, Un, Tr).placeable) && ii.placeable) {
                      let wi;
                      if (this.prevPlacement && this.prevPlacement.variableOffsets[An.crossTileID] && this.prevPlacement.placements[An.crossTileID] && this.prevPlacement.placements[An.crossTileID].text && (wi = this.prevPlacement.variableOffsets[An.crossTileID].anchor), An.crossTileID === 0) throw new Error("symbolInstance.crossTileID can't be 0");
                      return this.variableOffsets[An.crossTileID] = { textOffset: Cr, width: Oe, height: Be, anchor: wr, textBoxScale: Ue, prevAnchor: wi }, this.markUsedJustification(On, wr, An, Bn), On.allowVerticalPlacement && (this.markUsedOrientation(On, Bn, An), this.placedOrientations[An.crossTileID] = Bn), { shift: Tr, placedGlyphBoxes: ii };
                    }
                  }
                  placeLayerBucketPart(Te, Ae, Oe) {
                    const { bucket: Be, layout: Ue, translationText: nt, translationIcon: ht, posMatrix: Et, unwrappedTileID: Mt, textLabelPlaneMatrix: qt, labelToScreenMatrix: tn, textPixelRatio: xn, holdingForFade: An, collisionBoxArray: On, partiallyEvaluatedTextSize: Bn, collisionGroup: qn } = Te.parameters, Vn = Ue.get("text-optional"), sr = Ue.get("icon-optional"), Un = ee.as(Ue, "text-overlap", "text-allow-overlap"), wr = Un === "always", Cr = ee.as(Ue, "icon-overlap", "icon-allow-overlap"), Tr = Cr === "always", ii = Ue.get("text-rotation-alignment") === "map", wi = Ue.get("text-pitch-alignment") === "map", Wi = Ue.get("icon-text-fit") !== "none", Yi = Ue.get("symbol-z-order") === "viewport-y", ra = wr && (Tr || !Be.hasIconData() || sr), ha = Tr && (wr || !Be.hasTextData() || Vn);
                    !Be.collisionArrays && On && Be.deserializeCollisionBoxes(On);
                    const ya = this.retainedQueryData[Be.bucketInstanceId].tileID, va = this._getTerrainElevationFunc(ya), qa = (Aa, Xa, Oa) => {
                      var Ya, Wo;
                      if (Ae[Aa.crossTileID]) return;
                      if (An) return void (this.placements[Aa.crossTileID] = new kn(!1, !1, !1));
                      let Ja = !1, Uo = !1, ys = !0, il = null, Is = { box: null, placeable: !1, offscreen: null }, ml = { box: null, placeable: !1, offscreen: null }, nl = null, Il = null, Fl = null, qc = 0, ql = 0, jc = 0;
                      Xa.textFeatureIndex ? qc = Xa.textFeatureIndex : Aa.useRuntimeCollisionCircles && (qc = Aa.featureIndex), Xa.verticalTextFeatureIndex && (ql = Xa.verticalTextFeatureIndex);
                      const Dl = Xa.textBox;
                      if (Dl) {
                        const Bl = (Ic) => {
                          let $s = ee.ai.horizontal;
                          if (Be.allowVerticalPlacement && !Ic && this.prevPlacement) {
                            const Xc = this.prevPlacement.placedOrientations[Aa.crossTileID];
                            Xc && (this.placedOrientations[Aa.crossTileID] = Xc, $s = Xc, this.markUsedOrientation(Be, $s, Aa));
                          }
                          return $s;
                        }, al = (Ic, $s) => {
                          if (Be.allowVerticalPlacement && Aa.numVerticalGlyphVertices > 0 && Xa.verticalTextBox) {
                            for (const Xc of Be.writingModes) if (Xc === ee.ai.vertical ? (Is = $s(), ml = Is) : Is = Ic(), Is && Is.placeable) break;
                          } else Is = Ic();
                        }, Oc = Aa.textAnchorOffsetStartIndex, nu = Aa.textAnchorOffsetEndIndex;
                        if (nu === Oc) {
                          const Ic = ($s, Xc) => {
                            const wc = this.collisionIndex.placeCollisionBox($s, Un, xn, Et, Mt, wi, ii, nt, qn.predicate, va);
                            return wc && wc.placeable && (this.markUsedOrientation(Be, Xc, Aa), this.placedOrientations[Aa.crossTileID] = Xc), wc;
                          };
                          al(() => Ic(Dl, ee.ai.horizontal), () => {
                            const $s = Xa.verticalTextBox;
                            return Be.allowVerticalPlacement && Aa.numVerticalGlyphVertices > 0 && $s ? Ic($s, ee.ai.vertical) : { box: null, offscreen: null };
                          }), Bl(Is && Is.placeable);
                        } else {
                          let Ic = ee.ar[(Wo = (Ya = this.prevPlacement) === null || Ya === void 0 ? void 0 : Ya.variableOffsets[Aa.crossTileID]) === null || Wo === void 0 ? void 0 : Wo.anchor];
                          const $s = (wc, zu, Zc) => {
                            const Ju = wc.x2 - wc.x1, ru = wc.y2 - wc.y1, Bd = Aa.textBoxScale, tf = Wi && Cr === "never" ? zu : null;
                            let wd = null, od = Un === "never" ? 1 : 2, yd = "never";
                            Ic && od++;
                            for (let Ef = 0; Ef < od; Ef++) {
                              for (let Md = Oc; Md < nu; Md++) {
                                const nf = Be.textAnchorOffsets.get(Md);
                                if (Ic && nf.textAnchor !== Ic) continue;
                                const Vd = this.attemptAnchorPlacement(nf, wc, Ju, ru, Bd, ii, wi, xn, Et, Mt, qn, yd, Aa, Be, Zc, nt, ht, tf, va);
                                if (Vd && (wd = Vd.placedGlyphBoxes, wd && wd.placeable)) return Ja = !0, il = Vd.shift, wd;
                              }
                              Ic ? Ic = null : yd = Un;
                            }
                            return Oe && !wd && (wd = { box: this.collisionIndex.placeCollisionBox(Dl, "always", xn, Et, Mt, wi, ii, nt, qn.predicate, va, new ee.P(0, 0)).box, offscreen: !1, placeable: !1 }), wd;
                          };
                          al(() => $s(Dl, Xa.iconBox, ee.ai.horizontal), () => {
                            const wc = Xa.verticalTextBox, zu = Is && Is.placeable;
                            return Be.allowVerticalPlacement && !zu && Aa.numVerticalGlyphVertices > 0 && wc ? $s(wc, Xa.verticalIconBox, ee.ai.vertical) : { box: null, occluded: !0, offscreen: null };
                          }), Is && (Ja = Is.placeable, ys = Is.offscreen);
                          const Xc = Bl(Is && Is.placeable);
                          if (!Ja && this.prevPlacement) {
                            const wc = this.prevPlacement.variableOffsets[Aa.crossTileID];
                            wc && (this.variableOffsets[Aa.crossTileID] = wc, this.markUsedJustification(Be, wc.anchor, Aa, Xc));
                          }
                        }
                      }
                      if (nl = Is, Ja = nl && nl.placeable, ys = nl && nl.offscreen, Aa.useRuntimeCollisionCircles) {
                        const Bl = Be.text.placedSymbolArray.get(Aa.centerJustifiedTextSymbolIndex), al = ee.aj(Be.textSizeData, Bn, Bl), Oc = Ue.get("text-padding"), nu = Aa.collisionCircleDiameter;
                        Il = this.collisionIndex.placeCollisionCircles(Un, Bl, Be.lineVertexArray, Be.glyphOffsetArray, al, Et, Mt, qt, tn, Oe, wi, qn.predicate, nu, Oc, nt, va), Il.circles.length && Il.collisionDetected && !Oe && ee.w("Collisions detected, but collision boxes are not shown"), Ja = wr || Il.circles.length > 0 && !Il.collisionDetected, ys = ys && Il.offscreen;
                      }
                      if (Xa.iconFeatureIndex && (jc = Xa.iconFeatureIndex), Xa.iconBox) {
                        const Bl = (al) => this.collisionIndex.placeCollisionBox(al, Cr, xn, Et, Mt, wi, ii, ht, qn.predicate, va, Wi && il ? il : void 0);
                        ml && ml.placeable && Xa.verticalIconBox ? (Fl = Bl(Xa.verticalIconBox), Uo = Fl.placeable) : (Fl = Bl(Xa.iconBox), Uo = Fl.placeable), ys = ys && Fl.offscreen;
                      }
                      const Ml = Vn || Aa.numHorizontalGlyphVertices === 0 && Aa.numVerticalGlyphVertices === 0, pc = sr || Aa.numIconVertices === 0;
                      Ml || pc ? pc ? Ml || (Uo = Uo && Ja) : Ja = Uo && Ja : Uo = Ja = Uo && Ja;
                      const hl = Ja && nl.placeable, xl = Uo && Fl.placeable;
                      if (hl && (ml && ml.placeable && ql ? this.collisionIndex.insertCollisionBox(nl.box, Un, Ue.get("text-ignore-placement"), Be.bucketInstanceId, ql, qn.ID) : this.collisionIndex.insertCollisionBox(nl.box, Un, Ue.get("text-ignore-placement"), Be.bucketInstanceId, qc, qn.ID)), xl && this.collisionIndex.insertCollisionBox(Fl.box, Cr, Ue.get("icon-ignore-placement"), Be.bucketInstanceId, jc, qn.ID), Il && Ja && this.collisionIndex.insertCollisionCircles(Il.circles, Un, Ue.get("text-ignore-placement"), Be.bucketInstanceId, qc, qn.ID), Oe && this.storeCollisionData(Be.bucketInstanceId, Oa, Xa, nl, Fl, Il), Aa.crossTileID === 0) throw new Error("symbolInstance.crossTileID can't be 0");
                      if (Be.bucketInstanceId === 0) throw new Error("bucket.bucketInstanceId can't be 0");
                      this.placements[Aa.crossTileID] = new kn(Ja || ra, Uo || ha, ys || Be.justReloaded), Ae[Aa.crossTileID] = !0;
                    };
                    if (Yi) {
                      if (Te.symbolInstanceStart !== 0) throw new Error("bucket.bucketInstanceId should be 0");
                      const Aa = Be.getSortedSymbolIndexes(this.transform.angle);
                      for (let Xa = Aa.length - 1; Xa >= 0; --Xa) {
                        const Oa = Aa[Xa];
                        qa(Be.symbolInstances.get(Oa), Be.collisionArrays[Oa], Oa);
                      }
                    } else for (let Aa = Te.symbolInstanceStart; Aa < Te.symbolInstanceEnd; Aa++) qa(Be.symbolInstances.get(Aa), Be.collisionArrays[Aa], Aa);
                    if (Oe && Be.bucketInstanceId in this.collisionCircleArrays) {
                      const Aa = this.collisionCircleArrays[Be.bucketInstanceId];
                      ee.at(Aa.invProjMatrix, Et), Aa.viewportMatrix = this.collisionIndex.getViewportMatrix();
                    }
                    Be.justReloaded = !1;
                  }
                  storeCollisionData(Te, Ae, Oe, Be, Ue, nt) {
                    if (Oe.textBox || Oe.iconBox) {
                      let ht, Et;
                      this.collisionBoxArrays.has(Te) ? ht = this.collisionBoxArrays.get(Te) : (ht = /* @__PURE__ */ new Map(), this.collisionBoxArrays.set(Te, ht)), ht.has(Ae) ? Et = ht.get(Ae) : (Et = { text: null, icon: null }, ht.set(Ae, Et)), Oe.textBox && (Et.text = Be.box), Oe.iconBox && (Et.icon = Ue.box);
                    }
                    if (nt) {
                      let ht = this.collisionCircleArrays[Te];
                      ht === void 0 && (ht = this.collisionCircleArrays[Te] = new er());
                      for (let Et = 0; Et < nt.circles.length; Et += 4) ht.circles.push(nt.circles[Et + 0]), ht.circles.push(nt.circles[Et + 1]), ht.circles.push(nt.circles[Et + 2]), ht.circles.push(nt.collisionDetected ? 1 : 0);
                    }
                  }
                  markUsedJustification(Te, Ae, Oe, Be) {
                    const Ue = { left: Oe.leftJustifiedTextSymbolIndex, center: Oe.centerJustifiedTextSymbolIndex, right: Oe.rightJustifiedTextSymbolIndex };
                    let nt;
                    nt = Be === ee.ai.vertical ? Oe.verticalPlacedTextSymbolIndex : Ue[ee.au(Ae)];
                    const ht = [Oe.leftJustifiedTextSymbolIndex, Oe.centerJustifiedTextSymbolIndex, Oe.rightJustifiedTextSymbolIndex, Oe.verticalPlacedTextSymbolIndex];
                    for (const Et of ht) Et >= 0 && (Te.text.placedSymbolArray.get(Et).crossTileID = nt >= 0 && Et !== nt ? 0 : Oe.crossTileID);
                  }
                  markUsedOrientation(Te, Ae, Oe) {
                    const Be = Ae === ee.ai.horizontal || Ae === ee.ai.horizontalOnly ? Ae : 0, Ue = Ae === ee.ai.vertical ? Ae : 0, nt = [Oe.leftJustifiedTextSymbolIndex, Oe.centerJustifiedTextSymbolIndex, Oe.rightJustifiedTextSymbolIndex];
                    for (const ht of nt) Te.text.placedSymbolArray.get(ht).placedOrientation = Be;
                    Oe.verticalPlacedTextSymbolIndex && (Te.text.placedSymbolArray.get(Oe.verticalPlacedTextSymbolIndex).placedOrientation = Ue);
                  }
                  commit(Te) {
                    this.commitTime = Te, this.zoomAtLastRecencyCheck = this.transform.zoom;
                    const Ae = this.prevPlacement;
                    let Oe = !1;
                    this.prevZoomAdjustment = Ae ? Ae.zoomAdjustment(this.transform.zoom) : 0;
                    const Be = Ae ? Ae.symbolFadeChange(Te) : 1, Ue = Ae ? Ae.opacities : {}, nt = Ae ? Ae.variableOffsets : {}, ht = Ae ? Ae.placedOrientations : {};
                    for (const Et in this.placements) {
                      const Mt = this.placements[Et], qt = Ue[Et];
                      qt ? (this.opacities[Et] = new In(qt, Be, Mt.text, Mt.icon), Oe = Oe || Mt.text !== qt.text.placed || Mt.icon !== qt.icon.placed) : (this.opacities[Et] = new In(null, Be, Mt.text, Mt.icon, Mt.skipFade), Oe = Oe || Mt.text || Mt.icon);
                    }
                    for (const Et in Ue) {
                      const Mt = Ue[Et];
                      if (!this.opacities[Et]) {
                        const qt = new In(Mt, Be, !1, !1);
                        qt.isHidden() || (this.opacities[Et] = qt, Oe = Oe || Mt.text.placed || Mt.icon.placed);
                      }
                    }
                    for (const Et in nt) this.variableOffsets[Et] || !this.opacities[Et] || this.opacities[Et].isHidden() || (this.variableOffsets[Et] = nt[Et]);
                    for (const Et in ht) this.placedOrientations[Et] || !this.opacities[Et] || this.opacities[Et].isHidden() || (this.placedOrientations[Et] = ht[Et]);
                    if (Ae && Ae.lastPlacementChangeTime === void 0) throw new Error("Last placement time for previous placement is not defined");
                    Oe ? this.lastPlacementChangeTime = Te : typeof this.lastPlacementChangeTime != "number" && (this.lastPlacementChangeTime = Ae ? Ae.lastPlacementChangeTime : Te);
                  }
                  updateLayerOpacities(Te, Ae) {
                    const Oe = {};
                    for (const Be of Ae) {
                      const Ue = Be.getBucket(Te);
                      Ue && Be.latestFeatureIndex && Te.id === Ue.layerIds[0] && this.updateBucketOpacities(Ue, Be.tileID, Oe, Be.collisionBoxArray);
                    }
                  }
                  updateBucketOpacities(Te, Ae, Oe, Be) {
                    Te.hasTextData() && (Te.text.opacityVertexArray.clear(), Te.text.hasVisibleVertices = !1), Te.hasIconData() && (Te.icon.opacityVertexArray.clear(), Te.icon.hasVisibleVertices = !1), Te.hasIconCollisionBoxData() && Te.iconCollisionBox.collisionVertexArray.clear(), Te.hasTextCollisionBoxData() && Te.textCollisionBox.collisionVertexArray.clear();
                    const Ue = Te.layers[0], nt = Ue.layout, ht = new In(null, 0, !1, !1, !0), Et = nt.get("text-allow-overlap"), Mt = nt.get("icon-allow-overlap"), qt = Ue._unevaluatedLayout.hasValue("text-variable-anchor") || Ue._unevaluatedLayout.hasValue("text-variable-anchor-offset"), tn = nt.get("text-rotation-alignment") === "map", xn = nt.get("text-pitch-alignment") === "map", An = nt.get("icon-text-fit") !== "none", On = new In(null, 0, Et && (Mt || !Te.hasIconData() || nt.get("icon-optional")), Mt && (Et || !Te.hasTextData() || nt.get("text-optional")), !0);
                    !Te.collisionArrays && Be && (Te.hasIconCollisionBoxData() || Te.hasTextCollisionBoxData()) && Te.deserializeCollisionBoxes(Be);
                    const Bn = (Vn, sr, Un) => {
                      for (let wr = 0; wr < sr / 4; wr++) Vn.opacityVertexArray.emplaceBack(Un);
                      Vn.hasVisibleVertices = Vn.hasVisibleVertices || Un !== Zr;
                    }, qn = this.collisionBoxArrays.get(Te.bucketInstanceId);
                    for (let Vn = 0; Vn < Te.symbolInstances.length; Vn++) {
                      const sr = Te.symbolInstances.get(Vn), { numHorizontalGlyphVertices: Un, numVerticalGlyphVertices: wr, crossTileID: Cr } = sr, Tr = Oe[Cr];
                      let ii = this.opacities[Cr];
                      Tr ? ii = ht : ii || (ii = On, this.opacities[Cr] = ii), Oe[Cr] = !0;
                      const wi = Un > 0 || wr > 0, Wi = sr.numIconVertices > 0, Yi = this.placedOrientations[sr.crossTileID], ra = Yi === ee.ai.vertical, ha = Yi === ee.ai.horizontal || Yi === ee.ai.horizontalOnly;
                      if (wi) {
                        const va = gi(ii.text), qa = ra ? Zr : va;
                        Bn(Te.text, Un, qa);
                        const Aa = ha ? Zr : va;
                        Bn(Te.text, wr, Aa);
                        const Xa = ii.text.isHidden();
                        [sr.rightJustifiedTextSymbolIndex, sr.centerJustifiedTextSymbolIndex, sr.leftJustifiedTextSymbolIndex].forEach((Wo) => {
                          Wo >= 0 && (Te.text.placedSymbolArray.get(Wo).hidden = Xa || ra ? 1 : 0);
                        }), sr.verticalPlacedTextSymbolIndex >= 0 && (Te.text.placedSymbolArray.get(sr.verticalPlacedTextSymbolIndex).hidden = Xa || ha ? 1 : 0);
                        const Oa = this.variableOffsets[sr.crossTileID];
                        Oa && this.markUsedJustification(Te, Oa.anchor, sr, Yi);
                        const Ya = this.placedOrientations[sr.crossTileID];
                        Ya && (this.markUsedJustification(Te, "left", sr, Ya), this.markUsedOrientation(Te, Ya, sr));
                      }
                      if (Wi) {
                        const va = gi(ii.icon), qa = !(An && sr.verticalPlacedIconSymbolIndex && ra);
                        if (sr.placedIconSymbolIndex >= 0) {
                          const Aa = qa ? va : Zr;
                          Bn(Te.icon, sr.numIconVertices, Aa), Te.icon.placedSymbolArray.get(sr.placedIconSymbolIndex).hidden = ii.icon.isHidden();
                        }
                        if (sr.verticalPlacedIconSymbolIndex >= 0) {
                          const Aa = qa ? Zr : va;
                          Bn(Te.icon, sr.numVerticalIconVertices, Aa), Te.icon.placedSymbolArray.get(sr.verticalPlacedIconSymbolIndex).hidden = ii.icon.isHidden();
                        }
                      }
                      const ya = qn && qn.has(Vn) ? qn.get(Vn) : { text: null, icon: null };
                      if (Te.hasIconCollisionBoxData() || Te.hasTextCollisionBoxData()) {
                        const va = Te.collisionArrays[Vn];
                        if (va) {
                          let qa = new ee.P(0, 0);
                          if (va.textBox || va.verticalTextBox) {
                            let Aa = !0;
                            if (qt) {
                              const Xa = this.variableOffsets[Cr];
                              Xa ? (qa = Kn(Xa.anchor, Xa.width, Xa.height, Xa.textOffset, Xa.textBoxScale), tn && qa._rotate(xn ? this.transform.angle : -this.transform.angle)) : Aa = !1;
                            }
                            if (va.textBox || va.verticalTextBox) {
                              let Xa;
                              va.textBox && (Xa = ra), va.verticalTextBox && (Xa = ha), Lr(Te.textCollisionBox.collisionVertexArray, ii.text.placed, !Aa || Xa, ya.text, qa.x, qa.y);
                            }
                          }
                          if (va.iconBox || va.verticalIconBox) {
                            const Aa = !!(!ha && va.verticalIconBox);
                            let Xa;
                            va.iconBox && (Xa = Aa), va.verticalIconBox && (Xa = !Aa), Lr(Te.iconCollisionBox.collisionVertexArray, ii.icon.placed, Xa, ya.icon, An ? qa.x : 0, An ? qa.y : 0);
                          }
                        }
                      }
                    }
                    if (Te.sortFeatures(this.transform.angle), this.retainedQueryData[Te.bucketInstanceId] && (this.retainedQueryData[Te.bucketInstanceId].featureSortOrder = Te.featureSortOrder), Te.hasTextData() && Te.text.opacityVertexBuffer && Te.text.opacityVertexBuffer.updateData(Te.text.opacityVertexArray), Te.hasIconData() && Te.icon.opacityVertexBuffer && Te.icon.opacityVertexBuffer.updateData(Te.icon.opacityVertexArray), Te.hasIconCollisionBoxData() && Te.iconCollisionBox.collisionVertexBuffer && Te.iconCollisionBox.collisionVertexBuffer.updateData(Te.iconCollisionBox.collisionVertexArray), Te.hasTextCollisionBoxData() && Te.textCollisionBox.collisionVertexBuffer && Te.textCollisionBox.collisionVertexBuffer.updateData(Te.textCollisionBox.collisionVertexArray), Te.text.opacityVertexArray.length !== Te.text.layoutVertexArray.length / 4) throw new Error(`bucket.text.opacityVertexArray.length (= ${Te.text.opacityVertexArray.length}) !== bucket.text.layoutVertexArray.length (= ${Te.text.layoutVertexArray.length}) / 4`);
                    if (Te.icon.opacityVertexArray.length !== Te.icon.layoutVertexArray.length / 4) throw new Error(`bucket.icon.opacityVertexArray.length (= ${Te.icon.opacityVertexArray.length}) !== bucket.icon.layoutVertexArray.length (= ${Te.icon.layoutVertexArray.length}) / 4`);
                    if (Te.bucketInstanceId in this.collisionCircleArrays) {
                      const Vn = this.collisionCircleArrays[Te.bucketInstanceId];
                      Te.placementInvProjMatrix = Vn.invProjMatrix, Te.placementViewportMatrix = Vn.viewportMatrix, Te.collisionCircleArray = Vn.circles, delete this.collisionCircleArrays[Te.bucketInstanceId];
                    }
                  }
                  symbolFadeChange(Te) {
                    return this.fadeDuration === 0 ? 1 : (Te - this.commitTime) / this.fadeDuration + this.prevZoomAdjustment;
                  }
                  zoomAdjustment(Te) {
                    return Math.max(0, (this.transform.zoom - Te) / 1.5);
                  }
                  hasTransitions(Te) {
                    return this.stale || Te - this.lastPlacementChangeTime < this.fadeDuration;
                  }
                  stillRecent(Te, Ae) {
                    const Oe = this.zoomAtLastRecencyCheck === Ae ? 1 - this.zoomAdjustment(Ae) : 1;
                    return this.zoomAtLastRecencyCheck = Ae, this.commitTime + this.fadeDuration * Oe > Te;
                  }
                  setStale() {
                    this.stale = !0;
                  }
                }
                function Lr(Fe, Te, Ae, Oe, Be, Ue) {
                  Oe && Oe.length !== 0 || (Oe = [0, 0, 0, 0]);
                  const nt = Oe[0] - an, ht = Oe[1] - an, Et = Oe[2] - an, Mt = Oe[3] - an;
                  Fe.emplaceBack(Te ? 1 : 0, Ae ? 1 : 0, Be || 0, Ue || 0, nt, ht), Fe.emplaceBack(Te ? 1 : 0, Ae ? 1 : 0, Be || 0, Ue || 0, Et, ht), Fe.emplaceBack(Te ? 1 : 0, Ae ? 1 : 0, Be || 0, Ue || 0, Et, Mt), Fe.emplaceBack(Te ? 1 : 0, Ae ? 1 : 0, Be || 0, Ue || 0, nt, Mt);
                }
                const mr = Math.pow(2, 25), Ar = Math.pow(2, 24), Yr = Math.pow(2, 17), jr = Math.pow(2, 16), Mr = Math.pow(2, 9), Wr = Math.pow(2, 8), Bi = Math.pow(2, 1);
                function gi(Fe) {
                  if (Fe.opacity === 0 && !Fe.placed) return 0;
                  if (Fe.opacity === 1 && Fe.placed) return 4294967295;
                  const Te = Fe.placed ? 1 : 0, Ae = Math.floor(127 * Fe.opacity);
                  return Ae * mr + Te * Ar + Ae * Yr + Te * jr + Ae * Mr + Te * Wr + Ae * Bi + Te;
                }
                const Zr = 0;
                function fi() {
                  return { isOccluded(Fe, Te, Ae) {
                    return !1;
                  }, getPitchedTextCorrection(Fe, Te, Ae) {
                    return 1;
                  }, get useSpecialProjectionForSymbols() {
                    return !1;
                  }, projectTileCoordinates(Fe, Te, Ae, Oe) {
                    throw new Error("Not implemented.");
                  }, translatePosition(Fe, Te, Ae, Oe) {
                    return function(Be, Ue, nt, ht, Et = !1) {
                      if (!nt[0] && !nt[1]) return [0, 0];
                      const Mt = Et ? ht === "map" ? Be.angle : 0 : ht === "viewport" ? -Be.angle : 0;
                      if (Mt) {
                        const qt = Math.sin(Mt), tn = Math.cos(Mt);
                        nt = [nt[0] * tn - nt[1] * qt, nt[0] * qt + nt[1] * tn];
                      }
                      return [Et ? nt[0] : Mn(Ue, nt[0], Be.zoom), Et ? nt[1] : Mn(Ue, nt[1], Be.zoom)];
                    }(Fe, Te, Ae, Oe);
                  }, getCircleRadiusCorrection(Fe) {
                    return 1;
                  } };
                }
                class ri {
                  constructor(Te) {
                    this._sortAcrossTiles = Te.layout.get("symbol-z-order") !== "viewport-y" && !Te.layout.get("symbol-sort-key").isConstant(), this._currentTileIndex = 0, this._currentPartIndex = 0, this._seenCrossTileIDs = {}, this._bucketParts = [];
                  }
                  continuePlacement(Te, Ae, Oe, Be, Ue) {
                    const nt = this._bucketParts;
                    for (; this._currentTileIndex < Te.length; ) {
                      const ht = Te[this._currentTileIndex];
                      if (Ae.getBucketParts(nt, Be, ht, this._sortAcrossTiles), this._currentTileIndex++, Ue()) return !0;
                    }
                    for (this._sortAcrossTiles && (this._sortAcrossTiles = !1, nt.sort((ht, Et) => ht.sortKey - Et.sortKey)); this._currentPartIndex < nt.length; ) {
                      const ht = nt[this._currentPartIndex];
                      if (Ae.placeLayerBucketPart(ht, this._seenCrossTileIDs, Oe), this._currentPartIndex++, Ue()) return !0;
                    }
                    return !1;
                  }
                }
                class bi {
                  constructor(Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                    this.placement = new Jn(Te, fi(), Ae, nt, ht, Et), this._currentPlacementIndex = Oe.length - 1, this._forceFullPlacement = Be, this._showCollisionBoxes = Ue, this._done = !1;
                  }
                  isDone() {
                    return this._done;
                  }
                  continuePlacement(Te, Ae, Oe) {
                    const Be = ce.now(), Ue = () => !this._forceFullPlacement && ce.now() - Be > 2;
                    for (; this._currentPlacementIndex >= 0; ) {
                      const nt = Ae[Te[this._currentPlacementIndex]], ht = this.placement.collisionIndex.transform.zoom;
                      if (nt.type === "symbol" && (!nt.minzoom || nt.minzoom <= ht) && (!nt.maxzoom || nt.maxzoom > ht)) {
                        if (this._inProgressLayer || (this._inProgressLayer = new ri(nt)), this._inProgressLayer.continuePlacement(Oe[nt.source], this.placement, this._showCollisionBoxes, nt, Ue)) return;
                        delete this._inProgressLayer;
                      }
                      this._currentPlacementIndex--;
                    }
                    this._done = !0;
                  }
                  commit(Te) {
                    return this.placement.commit(Te), this.placement;
                  }
                }
                const Zi = 512 / ee.X / 2;
                class $i {
                  constructor(Te, Ae, Oe) {
                    this.tileID = Te, this.bucketInstanceId = Oe, this._symbolsByKey = {};
                    const Be = /* @__PURE__ */ new Map();
                    for (let Ue = 0; Ue < Ae.length; Ue++) {
                      const nt = Ae.get(Ue), ht = nt.key, Et = Be.get(ht);
                      Et ? Et.push(nt) : Be.set(ht, [nt]);
                    }
                    for (const [Ue, nt] of Be) {
                      const ht = { positions: nt.map((Et) => ({ x: Math.floor(Et.anchorX * Zi), y: Math.floor(Et.anchorY * Zi) })), crossTileIDs: nt.map((Et) => Et.crossTileID) };
                      if (ht.positions.length > 128) {
                        const Et = new ee.aw(ht.positions.length, 16, Uint16Array);
                        for (const { x: Mt, y: qt } of ht.positions) Et.add(Mt, qt);
                        Et.finish(), delete ht.positions, ht.index = Et;
                      }
                      this._symbolsByKey[Ue] = ht;
                    }
                  }
                  getScaledCoordinates(Te, Ae) {
                    const { x: Oe, y: Be, z: Ue } = this.tileID.canonical, { x: nt, y: ht, z: Et } = Ae.canonical, Mt = Et - Ue, qt = Zi / Math.pow(2, Mt), tn = (nt * ee.X + Te.anchorX) * qt, xn = (ht * ee.X + Te.anchorY) * qt, An = Oe * ee.X * Zi, On = Be * ee.X * Zi;
                    return { x: Math.floor(tn - An), y: Math.floor(xn - On) };
                  }
                  findMatches(Te, Ae, Oe) {
                    const Be = this.tileID.canonical.z < Ae.canonical.z ? 1 : Math.pow(2, this.tileID.canonical.z - Ae.canonical.z);
                    for (let Ue = 0; Ue < Te.length; Ue++) {
                      const nt = Te.get(Ue);
                      if (nt.crossTileID) continue;
                      const ht = this._symbolsByKey[nt.key];
                      if (!ht) continue;
                      const Et = this.getScaledCoordinates(nt, Ae);
                      if (ht.index) {
                        const Mt = ht.index.range(Et.x - Be, Et.y - Be, Et.x + Be, Et.y + Be).sort();
                        for (const qt of Mt) {
                          const tn = ht.crossTileIDs[qt];
                          if (!Oe[tn]) {
                            Oe[tn] = !0, nt.crossTileID = tn;
                            break;
                          }
                        }
                      } else if (ht.positions) for (let Mt = 0; Mt < ht.positions.length; Mt++) {
                        const qt = ht.positions[Mt], tn = ht.crossTileIDs[Mt];
                        if (Math.abs(qt.x - Et.x) <= Be && Math.abs(qt.y - Et.y) <= Be && !Oe[tn]) {
                          Oe[tn] = !0, nt.crossTileID = tn;
                          break;
                        }
                      }
                    }
                  }
                  getCrossTileIDsLists() {
                    return Object.values(this._symbolsByKey).map(({ crossTileIDs: Te }) => Te);
                  }
                }
                class Br {
                  constructor() {
                    this.maxCrossTileID = 0;
                  }
                  generate() {
                    return ++this.maxCrossTileID;
                  }
                }
                class Pr {
                  constructor() {
                    this.indexes = {}, this.usedCrossTileIDs = {}, this.lng = 0;
                  }
                  handleWrapJump(Te) {
                    const Ae = Math.round((Te - this.lng) / 360);
                    if (Ae !== 0) for (const Oe in this.indexes) {
                      const Be = this.indexes[Oe], Ue = {};
                      for (const nt in Be) {
                        const ht = Be[nt];
                        ht.tileID = ht.tileID.unwrapTo(ht.tileID.wrap + Ae), Ue[ht.tileID.key] = ht;
                      }
                      this.indexes[Oe] = Ue;
                    }
                    this.lng = Te;
                  }
                  addBucket(Te, Ae, Oe) {
                    if (this.indexes[Te.overscaledZ] && this.indexes[Te.overscaledZ][Te.key]) {
                      if (this.indexes[Te.overscaledZ][Te.key].bucketInstanceId === Ae.bucketInstanceId) return !1;
                      this.removeBucketCrossTileIDs(Te.overscaledZ, this.indexes[Te.overscaledZ][Te.key]);
                    }
                    for (let Ue = 0; Ue < Ae.symbolInstances.length; Ue++) Ae.symbolInstances.get(Ue).crossTileID = 0;
                    this.usedCrossTileIDs[Te.overscaledZ] || (this.usedCrossTileIDs[Te.overscaledZ] = {});
                    const Be = this.usedCrossTileIDs[Te.overscaledZ];
                    for (const Ue in this.indexes) {
                      const nt = this.indexes[Ue];
                      if (Number(Ue) > Te.overscaledZ) for (const ht in nt) {
                        const Et = nt[ht];
                        Et.tileID.isChildOf(Te) && Et.findMatches(Ae.symbolInstances, Te, Be);
                      }
                      else {
                        const ht = nt[Te.scaledTo(Number(Ue)).key];
                        ht && ht.findMatches(Ae.symbolInstances, Te, Be);
                      }
                    }
                    for (let Ue = 0; Ue < Ae.symbolInstances.length; Ue++) {
                      const nt = Ae.symbolInstances.get(Ue);
                      nt.crossTileID || (nt.crossTileID = Oe.generate(), Be[nt.crossTileID] = !0);
                    }
                    return this.indexes[Te.overscaledZ] === void 0 && (this.indexes[Te.overscaledZ] = {}), this.indexes[Te.overscaledZ][Te.key] = new $i(Te, Ae.symbolInstances, Ae.bucketInstanceId), !0;
                  }
                  removeBucketCrossTileIDs(Te, Ae) {
                    for (const Oe of Ae.getCrossTileIDsLists()) for (const Be of Oe) delete this.usedCrossTileIDs[Te][Be];
                  }
                  removeStaleBuckets(Te) {
                    let Ae = !1;
                    for (const Oe in this.indexes) {
                      const Be = this.indexes[Oe];
                      for (const Ue in Be) Te[Be[Ue].bucketInstanceId] || (this.removeBucketCrossTileIDs(Oe, Be[Ue]), delete Be[Ue], Ae = !0);
                    }
                    return Ae;
                  }
                }
                class ei {
                  constructor() {
                    this.layerIndexes = {}, this.crossTileIDs = new Br(), this.maxBucketInstanceId = 0, this.bucketsInCurrentPlacement = {};
                  }
                  addLayer(Te, Ae, Oe) {
                    let Be = this.layerIndexes[Te.id];
                    Be === void 0 && (Be = this.layerIndexes[Te.id] = new Pr());
                    let Ue = !1;
                    const nt = {};
                    Be.handleWrapJump(Oe);
                    for (const ht of Ae) {
                      const Et = ht.getBucket(Te);
                      Et && Te.id === Et.layerIds[0] && (Et.bucketInstanceId || (Et.bucketInstanceId = ++this.maxBucketInstanceId), Be.addBucket(ht.tileID, Et, this.crossTileIDs) && (Ue = !0), nt[Et.bucketInstanceId] = !0);
                    }
                    return Be.removeStaleBuckets(nt) && (Ue = !0), Ue;
                  }
                  pruneUnusedLayers(Te) {
                    const Ae = {};
                    Te.forEach((Oe) => {
                      Ae[Oe] = !0;
                    });
                    for (const Oe in this.layerIndexes) Ae[Oe] || delete this.layerIndexes[Oe];
                  }
                }
                const Ci = (Fe, Te) => ee.t(Fe, Te && Te.filter((Ae) => Ae.identifier !== "source.canvas")), oa = ee.ax();
                class ka extends ee.E {
                  constructor(Te, Ae = {}) {
                    super(), this._rtlPluginLoaded = () => {
                      for (const Oe in this.sourceCaches) {
                        const Be = this.sourceCaches[Oe].getSource().type;
                        Be !== "vector" && Be !== "geojson" || this.sourceCaches[Oe].reload();
                      }
                    }, this.map = Te, this.dispatcher = new st(it(), Te._getMapId()), this.dispatcher.registerMessageHandler("GG", (Oe, Be) => this.getGlyphs(Oe, Be)), this.dispatcher.registerMessageHandler("GI", (Oe, Be) => this.getImages(Oe, Be)), this.imageManager = new ke(), this.imageManager.setEventedParent(this), this.glyphManager = new Pe(Te._requestManager, Ae.localIdeographFontFamily), this.lineAtlas = new We(256, 512), this.crossTileSymbolIndex = new ei(), this._spritesImagesIds = {}, this._layers = {}, this._order = [], this.sourceCaches = {}, this.zoomHistory = new ee.ay(), this._loaded = !1, this._availableImages = [], this._resetUpdates(), this.dispatcher.broadcast("SR", ee.az()), hn().on(Vt, this._rtlPluginLoaded), this.on("data", (Oe) => {
                      if (Oe.dataType !== "source" || Oe.sourceDataType !== "metadata") return;
                      const Be = this.sourceCaches[Oe.sourceId];
                      if (!Be) return;
                      const Ue = Be.getSource();
                      if (Ue && Ue.vectorLayerIds) for (const nt in this._layers) {
                        const ht = this._layers[nt];
                        ht.source === Ue.id && this._validateLayer(ht);
                      }
                    });
                  }
                  loadURL(Te, Ae = {}, Oe) {
                    this.fire(new ee.k("dataloading", { dataType: "style" })), Ae.validate = typeof Ae.validate != "boolean" || Ae.validate;
                    const Be = this.map._requestManager.transformRequest(Te, "Style");
                    this._loadStyleRequest = new AbortController();
                    const Ue = this._loadStyleRequest;
                    ee.h(Be, this._loadStyleRequest).then((nt) => {
                      this._loadStyleRequest = null, this._load(nt.data, Ae, Oe);
                    }).catch((nt) => {
                      this._loadStyleRequest = null, nt && !Ue.signal.aborted && this.fire(new ee.j(nt));
                    });
                  }
                  loadJSON(Te, Ae = {}, Oe) {
                    this.fire(new ee.k("dataloading", { dataType: "style" })), this._frameRequest = new AbortController(), ce.frameAsync(this._frameRequest).then(() => {
                      this._frameRequest = null, Ae.validate = Ae.validate !== !1, this._load(Te, Ae, Oe);
                    }).catch(() => {
                    });
                  }
                  loadEmpty() {
                    this.fire(new ee.k("dataloading", { dataType: "style" })), this._load(oa, { validate: !1 });
                  }
                  _load(Te, Ae, Oe) {
                    var Be;
                    const Ue = Ae.transformStyle ? Ae.transformStyle(Oe, Te) : Te;
                    if (!Ae.validate || !Ci(this, ee.x(Ue))) {
                      this._loaded = !0, this.stylesheet = Ue;
                      for (const nt in Ue.sources) this.addSource(nt, Ue.sources[nt], { validate: !1 });
                      Ue.sprite ? this._loadSprite(Ue.sprite) : this.imageManager.setLoaded(!0), this.glyphManager.setURL(Ue.glyphs), this._createLayers(), this.light = new Ve(this.stylesheet.light), this.sky = new ze(this.stylesheet.sky), this.map.setTerrain((Be = this.stylesheet.terrain) !== null && Be !== void 0 ? Be : null), this.fire(new ee.k("data", { dataType: "style" })), this.fire(new ee.k("style.load"));
                    }
                  }
                  _createLayers() {
                    const Te = ee.aA(this.stylesheet.layers);
                    this.dispatcher.broadcast("SL", Te), this._order = Te.map((Ae) => Ae.id), this._layers = {}, this._serializedLayers = null;
                    for (const Ae of Te) {
                      const Oe = ee.aB(Ae);
                      Oe.setEventedParent(this, { layer: { id: Ae.id } }), this._layers[Ae.id] = Oe;
                    }
                  }
                  _loadSprite(Te, Ae = !1, Oe = void 0) {
                    let Be;
                    this.imageManager.setLoaded(!1), this._spriteRequest = new AbortController(), xe(Te, this.map._requestManager, this.map.getPixelRatio(), this._spriteRequest).then((Ue) => {
                      if (this._spriteRequest = null, Ue) for (const nt in Ue) {
                        this._spritesImagesIds[nt] = [];
                        const ht = this._spritesImagesIds[nt] ? this._spritesImagesIds[nt].filter((Et) => !(Et in Ue)) : [];
                        for (const Et of ht) this.imageManager.removeImage(Et), this._changedImages[Et] = !0;
                        for (const Et in Ue[nt]) {
                          const Mt = nt === "default" ? Et : `${nt}:${Et}`;
                          this._spritesImagesIds[nt].push(Mt), Mt in this.imageManager.images ? this.imageManager.updateImage(Mt, Ue[nt][Et], !1) : this.imageManager.addImage(Mt, Ue[nt][Et]), Ae && (this._changedImages[Mt] = !0);
                        }
                      }
                    }).catch((Ue) => {
                      this._spriteRequest = null, Be = Ue, this.fire(new ee.j(Be));
                    }).finally(() => {
                      this.imageManager.setLoaded(!0), this._availableImages = this.imageManager.listImages(), Ae && (this._changed = !0), this.dispatcher.broadcast("SI", this._availableImages), this.fire(new ee.k("data", { dataType: "style" })), Oe && Oe(Be);
                    });
                  }
                  _unloadSprite() {
                    for (const Te of Object.values(this._spritesImagesIds).flat()) this.imageManager.removeImage(Te), this._changedImages[Te] = !0;
                    this._spritesImagesIds = {}, this._availableImages = this.imageManager.listImages(), this._changed = !0, this.dispatcher.broadcast("SI", this._availableImages), this.fire(new ee.k("data", { dataType: "style" }));
                  }
                  _validateLayer(Te) {
                    const Ae = this.sourceCaches[Te.source];
                    if (!Ae) return;
                    const Oe = Te.sourceLayer;
                    if (!Oe) return;
                    const Be = Ae.getSource();
                    (Be.type === "geojson" || Be.vectorLayerIds && Be.vectorLayerIds.indexOf(Oe) === -1) && this.fire(new ee.j(new Error(`Source layer "${Oe}" does not exist on source "${Be.id}" as specified by style layer "${Te.id}".`)));
                  }
                  loaded() {
                    if (!this._loaded || Object.keys(this._updatedSources).length) return !1;
                    for (const Te in this.sourceCaches) if (!this.sourceCaches[Te].loaded()) return !1;
                    return !!this.imageManager.isLoaded();
                  }
                  _serializeByIds(Te) {
                    const Ae = this._serializedAllLayers();
                    if (!Te || Te.length === 0) return Object.values(Ae);
                    const Oe = [];
                    for (const Be of Te) Ae[Be] && Oe.push(Ae[Be]);
                    return Oe;
                  }
                  _serializedAllLayers() {
                    let Te = this._serializedLayers;
                    if (Te) return Te;
                    Te = this._serializedLayers = {};
                    const Ae = Object.keys(this._layers);
                    for (const Oe of Ae) {
                      const Be = this._layers[Oe];
                      Be.type !== "custom" && (Te[Oe] = Be.serialize());
                    }
                    return Te;
                  }
                  hasTransitions() {
                    if (this.light && this.light.hasTransition() || this.sky && this.sky.hasTransition()) return !0;
                    for (const Te in this.sourceCaches) if (this.sourceCaches[Te].hasTransition()) return !0;
                    for (const Te in this._layers) if (this._layers[Te].hasTransition()) return !0;
                    return !1;
                  }
                  _checkLoaded() {
                    if (!this._loaded) throw new Error("Style is not done loading.");
                  }
                  update(Te) {
                    if (!this._loaded) return;
                    const Ae = this._changed;
                    if (Ae) {
                      const Be = Object.keys(this._updatedLayers), Ue = Object.keys(this._removedLayers);
                      (Be.length || Ue.length) && this._updateWorkerLayers(Be, Ue);
                      for (const nt in this._updatedSources) {
                        const ht = this._updatedSources[nt];
                        if (ht === "reload") this._reloadSource(nt);
                        else {
                          if (ht !== "clear") throw new Error(`Invalid action ${ht}`);
                          this._clearSource(nt);
                        }
                      }
                      this._updateTilesForChangedImages(), this._updateTilesForChangedGlyphs();
                      for (const nt in this._updatedPaintProps) this._layers[nt].updateTransitions(Te);
                      this.light.updateTransitions(Te), this.sky.updateTransitions(Te), this._resetUpdates();
                    }
                    const Oe = {};
                    for (const Be in this.sourceCaches) {
                      const Ue = this.sourceCaches[Be];
                      Oe[Be] = Ue.used, Ue.used = !1;
                    }
                    for (const Be of this._order) {
                      const Ue = this._layers[Be];
                      Ue.recalculate(Te, this._availableImages), !Ue.isHidden(Te.zoom) && Ue.source && (this.sourceCaches[Ue.source].used = !0);
                    }
                    for (const Be in Oe) {
                      const Ue = this.sourceCaches[Be];
                      !!Oe[Be] != !!Ue.used && Ue.fire(new ee.k("data", { sourceDataType: "visibility", dataType: "source", sourceId: Be }));
                    }
                    this.light.recalculate(Te), this.sky.recalculate(Te), this.z = Te.zoom, Ae && this.fire(new ee.k("data", { dataType: "style" }));
                  }
                  _updateTilesForChangedImages() {
                    const Te = Object.keys(this._changedImages);
                    if (Te.length) {
                      for (const Ae in this.sourceCaches) this.sourceCaches[Ae].reloadTilesForDependencies(["icons", "patterns"], Te);
                      this._changedImages = {};
                    }
                  }
                  _updateTilesForChangedGlyphs() {
                    if (this._glyphsDidChange) {
                      for (const Te in this.sourceCaches) this.sourceCaches[Te].reloadTilesForDependencies(["glyphs"], [""]);
                      this._glyphsDidChange = !1;
                    }
                  }
                  _updateWorkerLayers(Te, Ae) {
                    this.dispatcher.broadcast("UL", { layers: this._serializeByIds(Te), removedIds: Ae });
                  }
                  _resetUpdates() {
                    this._changed = !1, this._updatedLayers = {}, this._removedLayers = {}, this._updatedSources = {}, this._updatedPaintProps = {}, this._changedImages = {}, this._glyphsDidChange = !1;
                  }
                  setState(Te, Ae = {}) {
                    var Oe;
                    this._checkLoaded();
                    const Be = this.serialize();
                    if (Te = Ae.transformStyle ? Ae.transformStyle(Be, Te) : Te, ((Oe = Ae.validate) === null || Oe === void 0 || Oe) && Ci(this, ee.x(Te))) return !1;
                    (Te = ee.aC(Te)).layers = ee.aA(Te.layers);
                    const Ue = ee.aD(Be, Te), nt = this._getOperationsToPerform(Ue);
                    if (nt.unimplemented.length > 0) throw new Error(`Unimplemented: ${nt.unimplemented.join(", ")}.`);
                    if (nt.operations.length === 0) return !1;
                    for (const ht of nt.operations) ht();
                    return this.stylesheet = Te, this._serializedLayers = null, !0;
                  }
                  _getOperationsToPerform(Te) {
                    const Ae = [], Oe = [];
                    for (const Be of Te) switch (Be.command) {
                      case "setCenter":
                      case "setZoom":
                      case "setBearing":
                      case "setPitch":
                        continue;
                      case "addLayer":
                        Ae.push(() => this.addLayer.apply(this, Be.args));
                        break;
                      case "removeLayer":
                        Ae.push(() => this.removeLayer.apply(this, Be.args));
                        break;
                      case "setPaintProperty":
                        Ae.push(() => this.setPaintProperty.apply(this, Be.args));
                        break;
                      case "setLayoutProperty":
                        Ae.push(() => this.setLayoutProperty.apply(this, Be.args));
                        break;
                      case "setFilter":
                        Ae.push(() => this.setFilter.apply(this, Be.args));
                        break;
                      case "addSource":
                        Ae.push(() => this.addSource.apply(this, Be.args));
                        break;
                      case "removeSource":
                        Ae.push(() => this.removeSource.apply(this, Be.args));
                        break;
                      case "setLayerZoomRange":
                        Ae.push(() => this.setLayerZoomRange.apply(this, Be.args));
                        break;
                      case "setLight":
                        Ae.push(() => this.setLight.apply(this, Be.args));
                        break;
                      case "setGeoJSONSourceData":
                        Ae.push(() => this.setGeoJSONSourceData.apply(this, Be.args));
                        break;
                      case "setGlyphs":
                        Ae.push(() => this.setGlyphs.apply(this, Be.args));
                        break;
                      case "setSprite":
                        Ae.push(() => this.setSprite.apply(this, Be.args));
                        break;
                      case "setSky":
                        Ae.push(() => this.setSky.apply(this, Be.args));
                        break;
                      case "setTerrain":
                        Ae.push(() => this.map.setTerrain.apply(this, Be.args));
                        break;
                      case "setTransition":
                        Ae.push(() => {
                        });
                        break;
                      default:
                        Oe.push(Be.command);
                    }
                    return { operations: Ae, unimplemented: Oe };
                  }
                  addImage(Te, Ae) {
                    if (this.getImage(Te)) return this.fire(new ee.j(new Error(`An image named "${Te}" already exists.`)));
                    this.imageManager.addImage(Te, Ae), this._afterImageUpdated(Te);
                  }
                  updateImage(Te, Ae) {
                    this.imageManager.updateImage(Te, Ae);
                  }
                  getImage(Te) {
                    return this.imageManager.getImage(Te);
                  }
                  removeImage(Te) {
                    if (!this.getImage(Te)) return this.fire(new ee.j(new Error(`An image named "${Te}" does not exist.`)));
                    this.imageManager.removeImage(Te), this._afterImageUpdated(Te);
                  }
                  _afterImageUpdated(Te) {
                    this._availableImages = this.imageManager.listImages(), this._changedImages[Te] = !0, this._changed = !0, this.dispatcher.broadcast("SI", this._availableImages), this.fire(new ee.k("data", { dataType: "style" }));
                  }
                  listImages() {
                    return this._checkLoaded(), this.imageManager.listImages();
                  }
                  addSource(Te, Ae, Oe = {}) {
                    if (this._checkLoaded(), this.sourceCaches[Te] !== void 0) throw new Error(`Source "${Te}" already exists.`);
                    if (!Ae.type) throw new Error(`The type property must be defined, but only the following properties were given: ${Object.keys(Ae).join(", ")}.`);
                    if (["vector", "raster", "geojson", "video", "image"].indexOf(Ae.type) >= 0 && this._validate(ee.x.source, `sources.${Te}`, Ae, null, Oe)) return;
                    this.map && this.map._collectResourceTiming && (Ae.collectResourceTiming = !0);
                    const Be = this.sourceCaches[Te] = new rn(Te, Ae, this.dispatcher);
                    Be.style = this, Be.setEventedParent(this, () => ({ isSourceLoaded: Be.loaded(), source: Be.serialize(), sourceId: Te })), Be.onAdd(this.map), this._changed = !0;
                  }
                  removeSource(Te) {
                    if (this._checkLoaded(), this.sourceCaches[Te] === void 0) throw new Error("There is no source with this ID");
                    for (const Oe in this._layers) if (this._layers[Oe].source === Te) return this.fire(new ee.j(new Error(`Source "${Te}" cannot be removed while layer "${Oe}" is using it.`)));
                    const Ae = this.sourceCaches[Te];
                    delete this.sourceCaches[Te], delete this._updatedSources[Te], Ae.fire(new ee.k("data", { sourceDataType: "metadata", dataType: "source", sourceId: Te })), Ae.setEventedParent(null), Ae.onRemove(this.map), this._changed = !0;
                  }
                  setGeoJSONSourceData(Te, Ae) {
                    if (this._checkLoaded(), this.sourceCaches[Te] === void 0) throw new Error(`There is no source with this ID=${Te}`);
                    const Oe = this.sourceCaches[Te].getSource();
                    if (Oe.type !== "geojson") throw new Error(`geojsonSource.type is ${Oe.type}, which is !== 'geojson`);
                    Oe.setData(Ae), this._changed = !0;
                  }
                  getSource(Te) {
                    return this.sourceCaches[Te] && this.sourceCaches[Te].getSource();
                  }
                  addLayer(Te, Ae, Oe = {}) {
                    this._checkLoaded();
                    const Be = Te.id;
                    if (this.getLayer(Be)) return void this.fire(new ee.j(new Error(`Layer "${Be}" already exists on this map.`)));
                    let Ue;
                    if (Te.type === "custom") {
                      if (Ci(this, ee.aE(Te))) return;
                      Ue = ee.aB(Te);
                    } else {
                      if ("source" in Te && typeof Te.source == "object" && (this.addSource(Be, Te.source), Te = ee.aC(Te), Te = ee.e(Te, { source: Be })), this._validate(ee.x.layer, `layers.${Be}`, Te, { arrayIndex: -1 }, Oe)) return;
                      Ue = ee.aB(Te), this._validateLayer(Ue), Ue.setEventedParent(this, { layer: { id: Be } });
                    }
                    const nt = Ae ? this._order.indexOf(Ae) : this._order.length;
                    if (Ae && nt === -1) this.fire(new ee.j(new Error(`Cannot add layer "${Be}" before non-existing layer "${Ae}".`)));
                    else {
                      if (this._order.splice(nt, 0, Be), this._layerOrderChanged = !0, this._layers[Be] = Ue, this._removedLayers[Be] && Ue.source && Ue.type !== "custom") {
                        const ht = this._removedLayers[Be];
                        delete this._removedLayers[Be], ht.type !== Ue.type ? this._updatedSources[Ue.source] = "clear" : (this._updatedSources[Ue.source] = "reload", this.sourceCaches[Ue.source].pause());
                      }
                      this._updateLayer(Ue), Ue.onAdd && Ue.onAdd(this.map);
                    }
                  }
                  moveLayer(Te, Ae) {
                    if (this._checkLoaded(), this._changed = !0, !this._layers[Te]) return void this.fire(new ee.j(new Error(`The layer '${Te}' does not exist in the map's style and cannot be moved.`)));
                    if (Te === Ae) return;
                    const Oe = this._order.indexOf(Te);
                    this._order.splice(Oe, 1);
                    const Be = Ae ? this._order.indexOf(Ae) : this._order.length;
                    Ae && Be === -1 ? this.fire(new ee.j(new Error(`Cannot move layer "${Te}" before non-existing layer "${Ae}".`))) : (this._order.splice(Be, 0, Te), this._layerOrderChanged = !0);
                  }
                  removeLayer(Te) {
                    this._checkLoaded();
                    const Ae = this._layers[Te];
                    if (!Ae) return void this.fire(new ee.j(new Error(`Cannot remove non-existing layer "${Te}".`)));
                    Ae.setEventedParent(null);
                    const Oe = this._order.indexOf(Te);
                    this._order.splice(Oe, 1), this._layerOrderChanged = !0, this._changed = !0, this._removedLayers[Te] = Ae, delete this._layers[Te], this._serializedLayers && delete this._serializedLayers[Te], delete this._updatedLayers[Te], delete this._updatedPaintProps[Te], Ae.onRemove && Ae.onRemove(this.map);
                  }
                  getLayer(Te) {
                    return this._layers[Te];
                  }
                  getLayersOrder() {
                    return [...this._order];
                  }
                  hasLayer(Te) {
                    return Te in this._layers;
                  }
                  setLayerZoomRange(Te, Ae, Oe) {
                    this._checkLoaded();
                    const Be = this.getLayer(Te);
                    Be ? Be.minzoom === Ae && Be.maxzoom === Oe || (Ae != null && (Be.minzoom = Ae), Oe != null && (Be.maxzoom = Oe), this._updateLayer(Be)) : this.fire(new ee.j(new Error(`Cannot set the zoom range of non-existing layer "${Te}".`)));
                  }
                  setFilter(Te, Ae, Oe = {}) {
                    this._checkLoaded();
                    const Be = this.getLayer(Te);
                    if (Be) {
                      if (!ee.aF(Be.filter, Ae)) return Ae == null ? (Be.filter = void 0, void this._updateLayer(Be)) : void (this._validate(ee.x.filter, `layers.${Be.id}.filter`, Ae, null, Oe) || (Be.filter = ee.aC(Ae), this._updateLayer(Be)));
                    } else this.fire(new ee.j(new Error(`Cannot filter non-existing layer "${Te}".`)));
                  }
                  getFilter(Te) {
                    return ee.aC(this.getLayer(Te).filter);
                  }
                  setLayoutProperty(Te, Ae, Oe, Be = {}) {
                    this._checkLoaded();
                    const Ue = this.getLayer(Te);
                    Ue ? ee.aF(Ue.getLayoutProperty(Ae), Oe) || (Ue.setLayoutProperty(Ae, Oe, Be), this._updateLayer(Ue)) : this.fire(new ee.j(new Error(`Cannot style non-existing layer "${Te}".`)));
                  }
                  getLayoutProperty(Te, Ae) {
                    const Oe = this.getLayer(Te);
                    if (Oe) return Oe.getLayoutProperty(Ae);
                    this.fire(new ee.j(new Error(`Cannot get style of non-existing layer "${Te}".`)));
                  }
                  setPaintProperty(Te, Ae, Oe, Be = {}) {
                    this._checkLoaded();
                    const Ue = this.getLayer(Te);
                    Ue ? ee.aF(Ue.getPaintProperty(Ae), Oe) || (Ue.setPaintProperty(Ae, Oe, Be) && this._updateLayer(Ue), this._changed = !0, this._updatedPaintProps[Te] = !0, this._serializedLayers = null) : this.fire(new ee.j(new Error(`Cannot style non-existing layer "${Te}".`)));
                  }
                  getPaintProperty(Te, Ae) {
                    return this.getLayer(Te).getPaintProperty(Ae);
                  }
                  setFeatureState(Te, Ae) {
                    this._checkLoaded();
                    const Oe = Te.source, Be = Te.sourceLayer, Ue = this.sourceCaches[Oe];
                    if (Ue === void 0) return void this.fire(new ee.j(new Error(`The source '${Oe}' does not exist in the map's style.`)));
                    const nt = Ue.getSource().type;
                    nt === "geojson" && Be ? this.fire(new ee.j(new Error("GeoJSON sources cannot have a sourceLayer parameter."))) : nt !== "vector" || Be ? (Te.id === void 0 && this.fire(new ee.j(new Error("The feature id parameter must be provided."))), Ue.setFeatureState(Be, Te.id, Ae)) : this.fire(new ee.j(new Error("The sourceLayer parameter must be provided for vector source types.")));
                  }
                  removeFeatureState(Te, Ae) {
                    this._checkLoaded();
                    const Oe = Te.source, Be = this.sourceCaches[Oe];
                    if (Be === void 0) return void this.fire(new ee.j(new Error(`The source '${Oe}' does not exist in the map's style.`)));
                    const Ue = Be.getSource().type, nt = Ue === "vector" ? Te.sourceLayer : void 0;
                    Ue !== "vector" || nt ? Ae && typeof Te.id != "string" && typeof Te.id != "number" ? this.fire(new ee.j(new Error("A feature id is required to remove its specific state property."))) : Be.removeFeatureState(nt, Te.id, Ae) : this.fire(new ee.j(new Error("The sourceLayer parameter must be provided for vector source types.")));
                  }
                  getFeatureState(Te) {
                    this._checkLoaded();
                    const Ae = Te.source, Oe = Te.sourceLayer, Be = this.sourceCaches[Ae];
                    if (Be !== void 0) return Be.getSource().type !== "vector" || Oe ? (Te.id === void 0 && this.fire(new ee.j(new Error("The feature id parameter must be provided."))), Be.getFeatureState(Oe, Te.id)) : void this.fire(new ee.j(new Error("The sourceLayer parameter must be provided for vector source types.")));
                    this.fire(new ee.j(new Error(`The source '${Ae}' does not exist in the map's style.`)));
                  }
                  getTransition() {
                    return ee.e({ duration: 300, delay: 0 }, this.stylesheet && this.stylesheet.transition);
                  }
                  serialize() {
                    if (!this._loaded) return;
                    const Te = ee.aG(this.sourceCaches, (Ue) => Ue.serialize()), Ae = this._serializeByIds(this._order), Oe = this.map.getTerrain() || void 0, Be = this.stylesheet;
                    return ee.aH({ version: Be.version, name: Be.name, metadata: Be.metadata, light: Be.light, sky: Be.sky, center: Be.center, zoom: Be.zoom, bearing: Be.bearing, pitch: Be.pitch, sprite: Be.sprite, glyphs: Be.glyphs, transition: Be.transition, sources: Te, layers: Ae, terrain: Oe }, (Ue) => Ue !== void 0);
                  }
                  _updateLayer(Te) {
                    this._updatedLayers[Te.id] = !0, Te.source && !this._updatedSources[Te.source] && this.sourceCaches[Te.source].getSource().type !== "raster" && (this._updatedSources[Te.source] = "reload", this.sourceCaches[Te.source].pause()), this._serializedLayers = null, this._changed = !0;
                  }
                  _flattenAndSortRenderedFeatures(Te) {
                    const Ae = (nt) => this._layers[nt].type === "fill-extrusion", Oe = {}, Be = [];
                    for (let nt = this._order.length - 1; nt >= 0; nt--) {
                      const ht = this._order[nt];
                      if (Ae(ht)) {
                        Oe[ht] = nt;
                        for (const Et of Te) {
                          const Mt = Et[ht];
                          if (Mt) for (const qt of Mt) Be.push(qt);
                        }
                      }
                    }
                    Be.sort((nt, ht) => ht.intersectionZ - nt.intersectionZ);
                    const Ue = [];
                    for (let nt = this._order.length - 1; nt >= 0; nt--) {
                      const ht = this._order[nt];
                      if (Ae(ht)) for (let Et = Be.length - 1; Et >= 0; Et--) {
                        const Mt = Be[Et].feature;
                        if (Oe[Mt.layer.id] < nt) break;
                        Ue.push(Mt), Be.pop();
                      }
                      else for (const Et of Te) {
                        const Mt = Et[ht];
                        if (Mt) for (const qt of Mt) Ue.push(qt.feature);
                      }
                    }
                    return Ue;
                  }
                  queryRenderedFeatures(Te, Ae, Oe) {
                    Ae && Ae.filter && this._validate(ee.x.filter, "queryRenderedFeatures.filter", Ae.filter, null, Ae);
                    const Be = {};
                    if (Ae && Ae.layers) {
                      if (!Array.isArray(Ae.layers)) return this.fire(new ee.j(new Error("parameters.layers must be an Array."))), [];
                      for (const ht of Ae.layers) {
                        const Et = this._layers[ht];
                        if (!Et) return this.fire(new ee.j(new Error(`The layer '${ht}' does not exist in the map's style and cannot be queried for features.`))), [];
                        Be[Et.source] = !0;
                      }
                    }
                    const Ue = [];
                    Ae.availableImages = this._availableImages;
                    const nt = this._serializedAllLayers();
                    for (const ht in this.sourceCaches) Ae.layers && !Be[ht] || Ue.push(yt(this.sourceCaches[ht], this._layers, nt, Te, Ae, Oe));
                    return this.placement && Ue.push(function(ht, Et, Mt, qt, tn, xn, An) {
                      const On = {}, Bn = xn.queryRenderedSymbols(qt), qn = [];
                      for (const Vn of Object.keys(Bn).map(Number)) qn.push(An[Vn]);
                      qn.sort(St);
                      for (const Vn of qn) {
                        const sr = Vn.featureIndex.lookupSymbolFeatures(Bn[Vn.bucketInstanceId], Et, Vn.bucketIndex, Vn.sourceLayerIndex, tn.filter, tn.layers, tn.availableImages, ht);
                        for (const Un in sr) {
                          const wr = On[Un] = On[Un] || [], Cr = sr[Un];
                          Cr.sort((Tr, ii) => {
                            const wi = Vn.featureSortOrder;
                            if (wi) {
                              const Wi = wi.indexOf(Tr.featureIndex);
                              return wi.indexOf(ii.featureIndex) - Wi;
                            }
                            return ii.featureIndex - Tr.featureIndex;
                          });
                          for (const Tr of Cr) wr.push(Tr);
                        }
                      }
                      for (const Vn in On) On[Vn].forEach((sr) => {
                        const Un = sr.feature, wr = ht[Vn], Cr = Mt[wr.source].getFeatureState(Un.layer["source-layer"], Un.id);
                        Un.source = Un.layer.source, Un.layer["source-layer"] && (Un.sourceLayer = Un.layer["source-layer"]), Un.state = Cr;
                      });
                      return On;
                    }(this._layers, nt, this.sourceCaches, Te, Ae, this.placement.collisionIndex, this.placement.retainedQueryData)), this._flattenAndSortRenderedFeatures(Ue);
                  }
                  querySourceFeatures(Te, Ae) {
                    Ae && Ae.filter && this._validate(ee.x.filter, "querySourceFeatures.filter", Ae.filter, null, Ae);
                    const Oe = this.sourceCaches[Te];
                    return Oe ? function(Be, Ue) {
                      const nt = Be.getRenderableIds().map((Mt) => Be.getTileByID(Mt)), ht = [], Et = {};
                      for (let Mt = 0; Mt < nt.length; Mt++) {
                        const qt = nt[Mt], tn = qt.tileID.canonical.key;
                        Et[tn] || (Et[tn] = !0, qt.querySourceFeatures(ht, Ue));
                      }
                      return ht;
                    }(Oe, Ae) : [];
                  }
                  getLight() {
                    return this.light.getLight();
                  }
                  setLight(Te, Ae = {}) {
                    this._checkLoaded();
                    const Oe = this.light.getLight();
                    let Be = !1;
                    for (const nt in Te) if (!ee.aF(Te[nt], Oe[nt])) {
                      Be = !0;
                      break;
                    }
                    if (!Be) return;
                    const Ue = { now: ce.now(), transition: ee.e({ duration: 300, delay: 0 }, this.stylesheet.transition) };
                    this.light.setLight(Te, Ae), this.light.updateTransitions(Ue);
                  }
                  getSky() {
                    var Te;
                    return (Te = this.stylesheet) === null || Te === void 0 ? void 0 : Te.sky;
                  }
                  setSky(Te, Ae = {}) {
                    const Oe = this.sky.getSky();
                    let Be = !1;
                    Te || Oe && (Be = !0);
                    for (const nt in Te) if (!ee.aF(Te[nt], Oe[nt])) {
                      Be = !0;
                      break;
                    }
                    if (!Be) return;
                    const Ue = { now: ce.now(), transition: ee.e({ duration: 300, delay: 0 }, this.stylesheet.transition) };
                    this.stylesheet.sky = Te, this.sky.setSky(Te, Ae), this.sky.updateTransitions(Ue);
                  }
                  _validate(Te, Ae, Oe, Be, Ue = {}) {
                    return (!Ue || Ue.validate !== !1) && Ci(this, Te.call(ee.x, ee.e({ key: Ae, style: this.serialize(), value: Oe, styleSpec: ee.v }, Be)));
                  }
                  _remove(Te = !0) {
                    this._frameRequest && (this._frameRequest.abort(), this._frameRequest = null), this._loadStyleRequest && (this._loadStyleRequest.abort(), this._loadStyleRequest = null), this._spriteRequest && (this._spriteRequest.abort(), this._spriteRequest = null), hn().off(Vt, this._rtlPluginLoaded);
                    for (const Ae in this._layers) this._layers[Ae].setEventedParent(null);
                    for (const Ae in this.sourceCaches) {
                      const Oe = this.sourceCaches[Ae];
                      Oe.setEventedParent(null), Oe.onRemove(this.map);
                    }
                    this.imageManager.setEventedParent(null), this.setEventedParent(null), Te && this.dispatcher.broadcast("RM", void 0), this.dispatcher.remove(Te);
                  }
                  _clearSource(Te) {
                    this.sourceCaches[Te].clearTiles();
                  }
                  _reloadSource(Te) {
                    this.sourceCaches[Te].resume(), this.sourceCaches[Te].reload();
                  }
                  _updateSources(Te) {
                    for (const Ae in this.sourceCaches) this.sourceCaches[Ae].update(Te, this.map.terrain);
                  }
                  _generateCollisionBoxes() {
                    for (const Te in this.sourceCaches) this._reloadSource(Te);
                  }
                  _updatePlacement(Te, Ae, Oe, Be, Ue = !1) {
                    let nt = !1, ht = !1;
                    const Et = {};
                    for (const Mt of this._order) {
                      const qt = this._layers[Mt];
                      if (qt.type !== "symbol") continue;
                      if (!Et[qt.source]) {
                        const xn = this.sourceCaches[qt.source];
                        Et[qt.source] = xn.getRenderableIds(!0).map((An) => xn.getTileByID(An)).sort((An, On) => On.tileID.overscaledZ - An.tileID.overscaledZ || (An.tileID.isLessThan(On.tileID) ? -1 : 1));
                      }
                      const tn = this.crossTileSymbolIndex.addLayer(qt, Et[qt.source], Te.center.lng);
                      nt = nt || tn;
                    }
                    if (this.crossTileSymbolIndex.pruneUnusedLayers(this._order), ((Ue = Ue || this._layerOrderChanged || Oe === 0) || !this.pauseablePlacement || this.pauseablePlacement.isDone() && !this.placement.stillRecent(ce.now(), Te.zoom)) && (this.pauseablePlacement = new bi(Te, this.map.terrain, this._order, Ue, Ae, Oe, Be, this.placement), this._layerOrderChanged = !1), this.pauseablePlacement.isDone() ? this.placement.setStale() : (this.pauseablePlacement.continuePlacement(this._order, this._layers, Et), this.pauseablePlacement.isDone() && (this.placement = this.pauseablePlacement.commit(ce.now()), ht = !0), nt && this.pauseablePlacement.placement.setStale()), ht || nt) for (const Mt of this._order) {
                      const qt = this._layers[Mt];
                      qt.type === "symbol" && this.placement.updateLayerOpacities(qt, Et[qt.source]);
                    }
                    return !this.pauseablePlacement.isDone() || this.placement.hasTransitions(ce.now());
                  }
                  _releaseSymbolFadeTiles() {
                    for (const Te in this.sourceCaches) this.sourceCaches[Te].releaseSymbolFadeTiles();
                  }
                  getImages(Te, Ae) {
                    return ee._(this, void 0, void 0, function* () {
                      const Oe = yield this.imageManager.getImages(Ae.icons);
                      this._updateTilesForChangedImages();
                      const Be = this.sourceCaches[Ae.source];
                      return Be && Be.setDependencies(Ae.tileID.key, Ae.type, Ae.icons), Oe;
                    });
                  }
                  getGlyphs(Te, Ae) {
                    return ee._(this, void 0, void 0, function* () {
                      const Oe = yield this.glyphManager.getGlyphs(Ae.stacks), Be = this.sourceCaches[Ae.source];
                      return Be && Be.setDependencies(Ae.tileID.key, Ae.type, [""]), Oe;
                    });
                  }
                  getGlyphsUrl() {
                    return this.stylesheet.glyphs || null;
                  }
                  setGlyphs(Te, Ae = {}) {
                    this._checkLoaded(), Te && this._validate(ee.x.glyphs, "glyphs", Te, null, Ae) || (this._glyphsDidChange = !0, this.stylesheet.glyphs = Te, this.glyphManager.entries = {}, this.glyphManager.setURL(Te));
                  }
                  addSprite(Te, Ae, Oe = {}, Be) {
                    this._checkLoaded();
                    const Ue = [{ id: Te, url: Ae }], nt = [...we(this.stylesheet.sprite), ...Ue];
                    this._validate(ee.x.sprite, "sprite", nt, null, Oe) || (this.stylesheet.sprite = nt, this._loadSprite(Ue, !0, Be));
                  }
                  removeSprite(Te) {
                    this._checkLoaded();
                    const Ae = we(this.stylesheet.sprite);
                    if (Ae.find((Oe) => Oe.id === Te)) {
                      if (this._spritesImagesIds[Te]) for (const Oe of this._spritesImagesIds[Te]) this.imageManager.removeImage(Oe), this._changedImages[Oe] = !0;
                      Ae.splice(Ae.findIndex((Oe) => Oe.id === Te), 1), this.stylesheet.sprite = Ae.length > 0 ? Ae : void 0, delete this._spritesImagesIds[Te], this._availableImages = this.imageManager.listImages(), this._changed = !0, this.dispatcher.broadcast("SI", this._availableImages), this.fire(new ee.k("data", { dataType: "style" }));
                    } else this.fire(new ee.j(new Error(`Sprite "${Te}" doesn't exists on this map.`)));
                  }
                  getSprite() {
                    return we(this.stylesheet.sprite);
                  }
                  setSprite(Te, Ae = {}, Oe) {
                    this._checkLoaded(), Te && this._validate(ee.x.sprite, "sprite", Te, null, Ae) || (this.stylesheet.sprite = Te, Te ? this._loadSprite(Te, !0, Oe) : (this._unloadSprite(), Oe && Oe(null)));
                  }
                }
                var aa = ee.Y([{ name: "a_pos", type: "Int16", components: 2 }]);
                const hi = { prelude: Gr(`#ifdef GL_ES
precision mediump float;
#else
#if !defined(lowp)
#define lowp
#endif
#if !defined(mediump)
#define mediump
#endif
#if !defined(highp)
#define highp
#endif
#endif
`, `#ifdef GL_ES
precision highp float;
#else
#if !defined(lowp)
#define lowp
#endif
#if !defined(mediump)
#define mediump
#endif
#if !defined(highp)
#define highp
#endif
#endif
vec2 unpack_float(const float packedValue) {int packedIntValue=int(packedValue);int v0=packedIntValue/256;return vec2(v0,packedIntValue-v0*256);}vec2 unpack_opacity(const float packedOpacity) {int intOpacity=int(packedOpacity)/2;return vec2(float(intOpacity)/127.0,mod(packedOpacity,2.0));}vec4 decode_color(const vec2 encodedColor) {return vec4(unpack_float(encodedColor[0])/255.0,unpack_float(encodedColor[1])/255.0
);}float unpack_mix_vec2(const vec2 packedValue,const float t) {return mix(packedValue[0],packedValue[1],t);}vec4 unpack_mix_color(const vec4 packedColors,const float t) {vec4 minColor=decode_color(vec2(packedColors[0],packedColors[1]));vec4 maxColor=decode_color(vec2(packedColors[2],packedColors[3]));return mix(minColor,maxColor,t);}vec2 get_pattern_pos(const vec2 pixel_coord_upper,const vec2 pixel_coord_lower,const vec2 pattern_size,const float tile_units_to_pixels,const vec2 pos) {vec2 offset=mod(mod(mod(pixel_coord_upper,pattern_size)*256.0,pattern_size)*256.0+pixel_coord_lower,pattern_size);return (tile_units_to_pixels*pos+offset)/pattern_size;}
#ifdef TERRAIN3D
uniform sampler2D u_terrain;uniform float u_terrain_dim;uniform mat4 u_terrain_matrix;uniform vec4 u_terrain_unpack;uniform float u_terrain_exaggeration;uniform highp sampler2D u_depth;
#endif
const highp vec4 bitSh=vec4(256.*256.*256.,256.*256.,256.,1.);const highp vec4 bitShifts=vec4(1.)/bitSh;highp float unpack(highp vec4 color) {return dot(color,bitShifts);}highp float depthOpacity(vec3 frag) {
#ifdef TERRAIN3D
highp float d=unpack(texture2D(u_depth,frag.xy*0.5+0.5))+0.0001-frag.z;return 1.0-max(0.0,min(1.0,-d*500.0));
#else
return 1.0;
#endif
}float calculate_visibility(vec4 pos) {
#ifdef TERRAIN3D
vec3 frag=pos.xyz/pos.w;highp float d=depthOpacity(frag);if (d > 0.95) return 1.0;return (d+depthOpacity(frag+vec3(0.0,0.01,0.0)))/2.0;
#else
return 1.0;
#endif
}float ele(vec2 pos) {
#ifdef TERRAIN3D
vec4 rgb=(texture2D(u_terrain,pos)*255.0)*u_terrain_unpack;return rgb.r+rgb.g+rgb.b-u_terrain_unpack.a;
#else
return 0.0;
#endif
}float get_elevation(vec2 pos) {
#ifdef TERRAIN3D
vec2 coord=(u_terrain_matrix*vec4(pos,0.0,1.0)).xy*u_terrain_dim+1.0;vec2 f=fract(coord);vec2 c=(floor(coord)+0.5)/(u_terrain_dim+2.0);float d=1.0/(u_terrain_dim+2.0);float tl=ele(c);float tr=ele(c+vec2(d,0.0));float bl=ele(c+vec2(0.0,d));float br=ele(c+vec2(d,d));float elevation=mix(mix(tl,tr,f.x),mix(bl,br,f.x),f.y);return elevation*u_terrain_exaggeration;
#else
return 0.0;
#endif
}`), background: Gr(`uniform vec4 u_color;uniform float u_opacity;void main() {gl_FragColor=u_color*u_opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "attribute vec2 a_pos;uniform mat4 u_matrix;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);}"), backgroundPattern: Gr(`uniform vec2 u_pattern_tl_a;uniform vec2 u_pattern_br_a;uniform vec2 u_pattern_tl_b;uniform vec2 u_pattern_br_b;uniform vec2 u_texsize;uniform float u_mix;uniform float u_opacity;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;void main() {vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(u_pattern_tl_a/u_texsize,u_pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(u_pattern_tl_b/u_texsize,u_pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);gl_FragColor=mix(color1,color2,u_mix)*u_opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_pattern_size_a;uniform vec2 u_pattern_size_b;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform float u_scale_a;uniform float u_scale_b;uniform float u_tile_units_to_pixels;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,u_scale_a*u_pattern_size_a,u_tile_units_to_pixels,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,u_scale_b*u_pattern_size_b,u_tile_units_to_pixels,a_pos);}"), circle: Gr(`varying vec3 v_data;varying float v_visibility;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define mediump float radius
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define highp vec4 stroke_color
#pragma mapbox: define mediump float stroke_width
#pragma mapbox: define lowp float stroke_opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize mediump float radius
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize highp vec4 stroke_color
#pragma mapbox: initialize mediump float stroke_width
#pragma mapbox: initialize lowp float stroke_opacity
vec2 extrude=v_data.xy;float extrude_length=length(extrude);lowp float antialiasblur=v_data.z;float antialiased_blur=-max(blur,antialiasblur);float opacity_t=smoothstep(0.0,antialiased_blur,extrude_length-1.0);float color_t=stroke_width < 0.01 ? 0.0 : smoothstep(antialiased_blur,0.0,extrude_length-radius/(radius+stroke_width));gl_FragColor=v_visibility*opacity_t*mix(color*opacity,stroke_color*stroke_opacity,color_t);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform bool u_scale_with_map;uniform bool u_pitch_with_map;uniform vec2 u_extrude_scale;uniform lowp float u_device_pixel_ratio;uniform highp float u_camera_to_center_distance;attribute vec2 a_pos;varying vec3 v_data;varying float v_visibility;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define mediump float radius
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define highp vec4 stroke_color
#pragma mapbox: define mediump float stroke_width
#pragma mapbox: define lowp float stroke_opacity
void main(void) {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize mediump float radius
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize highp vec4 stroke_color
#pragma mapbox: initialize mediump float stroke_width
#pragma mapbox: initialize lowp float stroke_opacity
vec2 extrude=vec2(mod(a_pos,2.0)*2.0-1.0);vec2 circle_center=floor(a_pos*0.5);float ele=get_elevation(circle_center);v_visibility=calculate_visibility(u_matrix*vec4(circle_center,ele,1.0));if (u_pitch_with_map) {vec2 corner_position=circle_center;if (u_scale_with_map) {corner_position+=extrude*(radius+stroke_width)*u_extrude_scale;} else {vec4 projected_center=u_matrix*vec4(circle_center,0,1);corner_position+=extrude*(radius+stroke_width)*u_extrude_scale*(projected_center.w/u_camera_to_center_distance);}gl_Position=u_matrix*vec4(corner_position,ele,1);} else {gl_Position=u_matrix*vec4(circle_center,ele,1);if (u_scale_with_map) {gl_Position.xy+=extrude*(radius+stroke_width)*u_extrude_scale*u_camera_to_center_distance;} else {gl_Position.xy+=extrude*(radius+stroke_width)*u_extrude_scale*gl_Position.w;}}lowp float antialiasblur=1.0/u_device_pixel_ratio/(radius+stroke_width);v_data=vec3(extrude.x,extrude.y,antialiasblur);}`), clippingMask: Gr("void main() {gl_FragColor=vec4(1.0);}", "attribute vec2 a_pos;uniform mat4 u_matrix;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);}"), heatmap: Gr(`uniform highp float u_intensity;varying vec2 v_extrude;
#pragma mapbox: define highp float weight
#define GAUSS_COEF 0.3989422804014327
void main() {
#pragma mapbox: initialize highp float weight
float d=-0.5*3.0*3.0*dot(v_extrude,v_extrude);float val=weight*u_intensity*GAUSS_COEF*exp(d);gl_FragColor=vec4(val,1.0,1.0,1.0);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform float u_extrude_scale;uniform float u_opacity;uniform float u_intensity;attribute vec2 a_pos;varying vec2 v_extrude;
#pragma mapbox: define highp float weight
#pragma mapbox: define mediump float radius
const highp float ZERO=1.0/255.0/16.0;
#define GAUSS_COEF 0.3989422804014327
void main(void) {
#pragma mapbox: initialize highp float weight
#pragma mapbox: initialize mediump float radius
vec2 unscaled_extrude=vec2(mod(a_pos,2.0)*2.0-1.0);float S=sqrt(-2.0*log(ZERO/weight/u_intensity/GAUSS_COEF))/3.0;v_extrude=S*unscaled_extrude;vec2 extrude=v_extrude*radius*u_extrude_scale;vec4 pos=vec4(floor(a_pos*0.5)+extrude,0,1);gl_Position=u_matrix*pos;}`), heatmapTexture: Gr(`uniform sampler2D u_image;uniform sampler2D u_color_ramp;uniform float u_opacity;varying vec2 v_pos;void main() {float t=texture2D(u_image,v_pos).r;vec4 color=texture2D(u_color_ramp,vec2(t,0.5));gl_FragColor=color*u_opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(0.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_world;attribute vec2 a_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos*u_world,0,1);v_pos.x=a_pos.x;v_pos.y=1.0-a_pos.y;}"), collisionBox: Gr("varying float v_placed;varying float v_notUsed;void main() {float alpha=0.5;gl_FragColor=vec4(1.0,0.0,0.0,1.0)*alpha;if (v_placed > 0.5) {gl_FragColor=vec4(0.0,0.0,1.0,0.5)*alpha;}if (v_notUsed > 0.5) {gl_FragColor*=.1;}}", "attribute vec2 a_anchor_pos;attribute vec2 a_placed;attribute vec2 a_box_real;uniform mat4 u_matrix;uniform vec2 u_pixel_extrude_scale;varying float v_placed;varying float v_notUsed;vec4 projectTileWithElevation(vec2 posInTile,float elevation) {return u_matrix*vec4(posInTile,elevation,1.0);}void main() {gl_Position=projectTileWithElevation(a_anchor_pos,get_elevation(a_anchor_pos));gl_Position.xy=((a_box_real+0.5)*u_pixel_extrude_scale*2.0-1.0)*vec2(1.0,-1.0)*gl_Position.w;if (gl_Position.z/gl_Position.w < 1.1) {gl_Position.z=0.5;}v_placed=a_placed.x;v_notUsed=a_placed.y;}"), collisionCircle: Gr("varying float v_radius;varying vec2 v_extrude;varying float v_perspective_ratio;varying float v_collision;void main() {float alpha=0.5*min(v_perspective_ratio,1.0);float stroke_radius=0.9*max(v_perspective_ratio,1.0);float distance_to_center=length(v_extrude);float distance_to_edge=abs(distance_to_center-v_radius);float opacity_t=smoothstep(-stroke_radius,0.0,-distance_to_edge);vec4 color=mix(vec4(0.0,0.0,1.0,0.5),vec4(1.0,0.0,0.0,1.0),v_collision);gl_FragColor=color*alpha*opacity_t;}", "attribute vec2 a_pos;attribute float a_radius;attribute vec2 a_flags;uniform mat4 u_matrix;uniform mat4 u_inv_matrix;uniform vec2 u_viewport_size;uniform float u_camera_to_center_distance;varying float v_radius;varying vec2 v_extrude;varying float v_perspective_ratio;varying float v_collision;vec3 toTilePosition(vec2 screenPos) {vec4 rayStart=u_inv_matrix*vec4(screenPos,-1.0,1.0);vec4 rayEnd  =u_inv_matrix*vec4(screenPos, 1.0,1.0);rayStart.xyz/=rayStart.w;rayEnd.xyz  /=rayEnd.w;highp float t=(0.0-rayStart.z)/(rayEnd.z-rayStart.z);return mix(rayStart.xyz,rayEnd.xyz,t);}void main() {vec2 quadCenterPos=a_pos;float radius=a_radius;float collision=a_flags.x;float vertexIdx=a_flags.y;vec2 quadVertexOffset=vec2(mix(-1.0,1.0,float(vertexIdx >=2.0)),mix(-1.0,1.0,float(vertexIdx >=1.0 && vertexIdx <=2.0)));vec2 quadVertexExtent=quadVertexOffset*radius;vec3 tilePos=toTilePosition(quadCenterPos);vec4 clipPos=u_matrix*vec4(tilePos,1.0);highp float camera_to_anchor_distance=clipPos.w;highp float collision_perspective_ratio=clamp(0.5+0.5*(u_camera_to_center_distance/camera_to_anchor_distance),0.0,4.0);float padding_factor=1.2;v_radius=radius;v_extrude=quadVertexExtent*padding_factor;v_perspective_ratio=collision_perspective_ratio;v_collision=collision;gl_Position=vec4(clipPos.xyz/clipPos.w,1.0)+vec4(quadVertexExtent*padding_factor/u_viewport_size*2.0,0.0,0.0);}"), debug: Gr("uniform highp vec4 u_color;uniform sampler2D u_overlay;varying vec2 v_uv;void main() {vec4 overlay_color=texture2D(u_overlay,v_uv);gl_FragColor=mix(u_color,overlay_color,overlay_color.a);}", "attribute vec2 a_pos;varying vec2 v_uv;uniform mat4 u_matrix;uniform float u_overlay_scale;void main() {v_uv=a_pos/8192.0;gl_Position=u_matrix*vec4(a_pos*u_overlay_scale,get_elevation(a_pos),1);}"), fill: Gr(`#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float opacity
gl_FragColor=color*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `attribute vec2 a_pos;uniform mat4 u_matrix;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float opacity
gl_Position=u_matrix*vec4(a_pos,0,1);}`), fillOutline: Gr(`varying vec2 v_pos;
#pragma mapbox: define highp vec4 outline_color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 outline_color
#pragma mapbox: initialize lowp float opacity
float dist=length(v_pos-gl_FragCoord.xy);float alpha=1.0-smoothstep(0.0,1.0,dist);gl_FragColor=outline_color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `attribute vec2 a_pos;uniform mat4 u_matrix;uniform vec2 u_world;varying vec2 v_pos;
#pragma mapbox: define highp vec4 outline_color
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 outline_color
#pragma mapbox: initialize lowp float opacity
gl_Position=u_matrix*vec4(a_pos,0,1);v_pos=(gl_Position.xy/gl_Position.w+1.0)/2.0*u_world;}`), fillOutlinePattern: Gr(`uniform vec2 u_texsize;uniform sampler2D u_image;uniform float u_fade;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec2 v_pos;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);float dist=length(v_pos-gl_FragCoord.xy);float alpha=1.0-smoothstep(0.0,1.0,dist);gl_FragColor=mix(color1,color2,u_fade)*alpha*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec2 u_world;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform vec3 u_scale;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec2 v_pos;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;gl_Position=u_matrix*vec4(a_pos,0,1);vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileRatio,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileRatio,a_pos);v_pos=(gl_Position.xy/gl_Position.w+1.0)/2.0*u_world;}`), fillPattern: Gr(`#ifdef GL_ES
precision highp float;
#endif
uniform vec2 u_texsize;uniform float u_fade;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);gl_FragColor=mix(color1,color2,u_fade)*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform vec3 u_scale;attribute vec2 a_pos;varying vec2 v_pos_a;varying vec2 v_pos_b;
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileZoomRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;gl_Position=u_matrix*vec4(a_pos,0,1);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileZoomRatio,a_pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileZoomRatio,a_pos);}`), fillExtrusion: Gr(`varying vec4 v_color;void main() {gl_FragColor=v_color;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec3 u_lightcolor;uniform lowp vec3 u_lightpos;uniform lowp float u_lightintensity;uniform float u_vertical_gradient;uniform lowp float u_opacity;attribute vec2 a_pos;attribute vec4 a_normal_ed;
#ifdef TERRAIN3D
attribute vec2 a_centroid;
#endif
varying vec4 v_color;
#pragma mapbox: define highp float base
#pragma mapbox: define highp float height
#pragma mapbox: define highp vec4 color
void main() {
#pragma mapbox: initialize highp float base
#pragma mapbox: initialize highp float height
#pragma mapbox: initialize highp vec4 color
vec3 normal=a_normal_ed.xyz;
#ifdef TERRAIN3D
float height_terrain3d_offset=get_elevation(a_centroid);float base_terrain3d_offset=height_terrain3d_offset-(base > 0.0 ? 0.0 : 10.0);
#else
float height_terrain3d_offset=0.0;float base_terrain3d_offset=0.0;
#endif
base=max(0.0,base)+base_terrain3d_offset;height=max(0.0,height)+height_terrain3d_offset;float t=mod(normal.x,2.0);gl_Position=u_matrix*vec4(a_pos,t > 0.0 ? height : base,1);float colorvalue=color.r*0.2126+color.g*0.7152+color.b*0.0722;v_color=vec4(0.0,0.0,0.0,1.0);vec4 ambientlight=vec4(0.03,0.03,0.03,1.0);color+=ambientlight;float directional=clamp(dot(normal/16384.0,u_lightpos),0.0,1.0);directional=mix((1.0-u_lightintensity),max((1.0-colorvalue+u_lightintensity),1.0),directional);if (normal.y !=0.0) {directional*=((1.0-u_vertical_gradient)+(u_vertical_gradient*clamp((t+base)*pow(height/150.0,0.5),mix(0.7,0.98,1.0-u_lightintensity),1.0)));}v_color.r+=clamp(color.r*directional*u_lightcolor.r,mix(0.0,0.3,1.0-u_lightcolor.r),1.0);v_color.g+=clamp(color.g*directional*u_lightcolor.g,mix(0.0,0.3,1.0-u_lightcolor.g),1.0);v_color.b+=clamp(color.b*directional*u_lightcolor.b,mix(0.0,0.3,1.0-u_lightcolor.b),1.0);v_color*=u_opacity;}`), fillExtrusionPattern: Gr(`uniform vec2 u_texsize;uniform float u_fade;uniform sampler2D u_image;varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec4 v_lighting;
#pragma mapbox: define lowp float base
#pragma mapbox: define lowp float height
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float base
#pragma mapbox: initialize lowp float height
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;vec2 imagecoord=mod(v_pos_a,1.0);vec2 pos=mix(pattern_tl_a/u_texsize,pattern_br_a/u_texsize,imagecoord);vec4 color1=texture2D(u_image,pos);vec2 imagecoord_b=mod(v_pos_b,1.0);vec2 pos2=mix(pattern_tl_b/u_texsize,pattern_br_b/u_texsize,imagecoord_b);vec4 color2=texture2D(u_image,pos2);vec4 mixedColor=mix(color1,color2,u_fade);gl_FragColor=mixedColor*v_lighting;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `uniform mat4 u_matrix;uniform vec2 u_pixel_coord_upper;uniform vec2 u_pixel_coord_lower;uniform float u_height_factor;uniform vec3 u_scale;uniform float u_vertical_gradient;uniform lowp float u_opacity;uniform vec3 u_lightcolor;uniform lowp vec3 u_lightpos;uniform lowp float u_lightintensity;attribute vec2 a_pos;attribute vec4 a_normal_ed;
#ifdef TERRAIN3D
attribute vec2 a_centroid;
#endif
varying vec2 v_pos_a;varying vec2 v_pos_b;varying vec4 v_lighting;
#pragma mapbox: define lowp float base
#pragma mapbox: define lowp float height
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float base
#pragma mapbox: initialize lowp float height
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;vec3 normal=a_normal_ed.xyz;float edgedistance=a_normal_ed.w;vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;
#ifdef TERRAIN3D
float height_terrain3d_offset=get_elevation(a_centroid);float base_terrain3d_offset=height_terrain3d_offset-(base > 0.0 ? 0.0 : 10.0);
#else
float height_terrain3d_offset=0.0;float base_terrain3d_offset=0.0;
#endif
base=max(0.0,base)+base_terrain3d_offset;height=max(0.0,height)+height_terrain3d_offset;float t=mod(normal.x,2.0);float z=t > 0.0 ? height : base;gl_Position=u_matrix*vec4(a_pos,z,1);vec2 pos=normal.x==1.0 && normal.y==0.0 && normal.z==16384.0
? a_pos
: vec2(edgedistance,z*u_height_factor);v_pos_a=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,fromScale*display_size_a,tileRatio,pos);v_pos_b=get_pattern_pos(u_pixel_coord_upper,u_pixel_coord_lower,toScale*display_size_b,tileRatio,pos);v_lighting=vec4(0.0,0.0,0.0,1.0);float directional=clamp(dot(normal/16383.0,u_lightpos),0.0,1.0);directional=mix((1.0-u_lightintensity),max((0.5+u_lightintensity),1.0),directional);if (normal.y !=0.0) {directional*=((1.0-u_vertical_gradient)+(u_vertical_gradient*clamp((t+base)*pow(height/150.0,0.5),mix(0.7,0.98,1.0-u_lightintensity),1.0)));}v_lighting.rgb+=clamp(directional*u_lightcolor,mix(vec3(0.0),vec3(0.3),1.0-u_lightcolor),vec3(1.0));v_lighting*=u_opacity;}`), hillshadePrepare: Gr(`#ifdef GL_ES
precision highp float;
#endif
uniform sampler2D u_image;varying vec2 v_pos;uniform vec2 u_dimension;uniform float u_zoom;uniform vec4 u_unpack;float getElevation(vec2 coord,float bias) {vec4 data=texture2D(u_image,coord)*255.0;data.a=-1.0;return dot(data,u_unpack)/4.0;}void main() {vec2 epsilon=1.0/u_dimension;float a=getElevation(v_pos+vec2(-epsilon.x,-epsilon.y),0.0);float b=getElevation(v_pos+vec2(0,-epsilon.y),0.0);float c=getElevation(v_pos+vec2(epsilon.x,-epsilon.y),0.0);float d=getElevation(v_pos+vec2(-epsilon.x,0),0.0);float e=getElevation(v_pos,0.0);float f=getElevation(v_pos+vec2(epsilon.x,0),0.0);float g=getElevation(v_pos+vec2(-epsilon.x,epsilon.y),0.0);float h=getElevation(v_pos+vec2(0,epsilon.y),0.0);float i=getElevation(v_pos+vec2(epsilon.x,epsilon.y),0.0);float exaggerationFactor=u_zoom < 2.0 ? 0.4 : u_zoom < 4.5 ? 0.35 : 0.3;float exaggeration=u_zoom < 15.0 ? (u_zoom-15.0)*exaggerationFactor : 0.0;vec2 deriv=vec2((c+f+f+i)-(a+d+d+g),(g+h+h+i)-(a+b+b+c))/pow(2.0,exaggeration+(19.2562-u_zoom));gl_FragColor=clamp(vec4(deriv.x/2.0+0.5,deriv.y/2.0+0.5,1.0,1.0),0.0,1.0);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_dimension;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);highp vec2 epsilon=1.0/u_dimension;float scale=(u_dimension.x-2.0)/u_dimension.x;v_pos=(a_texture_pos/8192.0)*scale+epsilon;}"), hillshade: Gr(`uniform sampler2D u_image;varying vec2 v_pos;uniform vec2 u_latrange;uniform vec2 u_light;uniform vec4 u_shadow;uniform vec4 u_highlight;uniform vec4 u_accent;
#define PI 3.141592653589793
void main() {vec4 pixel=texture2D(u_image,v_pos);vec2 deriv=((pixel.rg*2.0)-1.0);float scaleFactor=cos(radians((u_latrange[0]-u_latrange[1])*(1.0-v_pos.y)+u_latrange[1]));float slope=atan(1.25*length(deriv)/scaleFactor);float aspect=deriv.x !=0.0 ? atan(deriv.y,-deriv.x) : PI/2.0*(deriv.y > 0.0 ? 1.0 :-1.0);float intensity=u_light.x;float azimuth=u_light.y+PI;float base=1.875-intensity*1.75;float maxValue=0.5*PI;float scaledSlope=intensity !=0.5 ? ((pow(base,slope)-1.0)/(pow(base,maxValue)-1.0))*maxValue : slope;float accent=cos(scaledSlope);vec4 accent_color=(1.0-accent)*u_accent*clamp(intensity*2.0,0.0,1.0);float shade=abs(mod((aspect+azimuth)/PI+0.5,2.0)-1.0);vec4 shade_color=mix(u_shadow,u_highlight,shade)*sin(scaledSlope)*clamp(intensity*2.0,0.0,1.0);gl_FragColor=accent_color*(1.0-shade_color.a)+shade_color;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos=a_texture_pos/8192.0;}"), line: Gr(`uniform lowp float u_device_pixel_ratio;varying vec2 v_width2;varying vec2 v_normal;varying float v_gamma_scale;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);gl_FragColor=color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
attribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform vec2 u_units_to_pixels;uniform lowp float u_device_pixel_ratio;varying vec2 v_normal;varying vec2 v_width2;varying float v_gamma_scale;varying highp float v_linesofar;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float width
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float width
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;v_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*2.0;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;
#ifdef TERRAIN3D
v_gamma_scale=1.0;
#else
float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;
#endif
v_width2=vec2(outset,inset);}`), lineGradient: Gr(`uniform lowp float u_device_pixel_ratio;uniform sampler2D u_image;varying vec2 v_width2;varying vec2 v_normal;varying float v_gamma_scale;varying highp vec2 v_uv;
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);vec4 color=texture2D(u_image,v_uv);gl_FragColor=color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
attribute vec2 a_pos_normal;attribute vec4 a_data;attribute float a_uv_x;attribute float a_split_index;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;uniform vec2 u_units_to_pixels;uniform float u_image_height;varying vec2 v_normal;varying vec2 v_width2;varying float v_gamma_scale;varying highp vec2 v_uv;
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float width
void main() {
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float width
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;highp float texel_height=1.0/u_image_height;highp float half_texel_height=0.5*texel_height;v_uv=vec2(a_uv_x,a_split_index*texel_height-half_texel_height);vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;
#ifdef TERRAIN3D
v_gamma_scale=1.0;
#else
float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;
#endif
v_width2=vec2(outset,inset);}`), linePattern: Gr(`#ifdef GL_ES
precision highp float;
#endif
uniform lowp float u_device_pixel_ratio;uniform vec2 u_texsize;uniform float u_fade;uniform mediump vec3 u_scale;uniform sampler2D u_image;varying vec2 v_normal;varying vec2 v_width2;varying float v_linesofar;varying float v_gamma_scale;varying float v_width;
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
vec2 pattern_tl_a=pattern_from.xy;vec2 pattern_br_a=pattern_from.zw;vec2 pattern_tl_b=pattern_to.xy;vec2 pattern_br_b=pattern_to.zw;float tileZoomRatio=u_scale.x;float fromScale=u_scale.y;float toScale=u_scale.z;vec2 display_size_a=(pattern_br_a-pattern_tl_a)/pixel_ratio_from;vec2 display_size_b=(pattern_br_b-pattern_tl_b)/pixel_ratio_to;vec2 pattern_size_a=vec2(display_size_a.x*fromScale/tileZoomRatio,display_size_a.y);vec2 pattern_size_b=vec2(display_size_b.x*toScale/tileZoomRatio,display_size_b.y);float aspect_a=display_size_a.y/v_width;float aspect_b=display_size_b.y/v_width;float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);float x_a=mod(v_linesofar/pattern_size_a.x*aspect_a,1.0);float x_b=mod(v_linesofar/pattern_size_b.x*aspect_b,1.0);float y=0.5*v_normal.y+0.5;vec2 texel_size=1.0/u_texsize;vec2 pos_a=mix(pattern_tl_a*texel_size-texel_size,pattern_br_a*texel_size+texel_size,vec2(x_a,y));vec2 pos_b=mix(pattern_tl_b*texel_size-texel_size,pattern_br_b*texel_size+texel_size,vec2(x_b,y));vec4 color=mix(texture2D(u_image,pos_a),texture2D(u_image,pos_b),u_fade);gl_FragColor=color*alpha*opacity;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
#define LINE_DISTANCE_SCALE 2.0
attribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform vec2 u_units_to_pixels;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;varying vec2 v_normal;varying vec2 v_width2;varying float v_linesofar;varying float v_gamma_scale;varying float v_width;
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define mediump float width
#pragma mapbox: define lowp float floorwidth
#pragma mapbox: define lowp vec4 pattern_from
#pragma mapbox: define lowp vec4 pattern_to
#pragma mapbox: define lowp float pixel_ratio_from
#pragma mapbox: define lowp float pixel_ratio_to
void main() {
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize mediump float width
#pragma mapbox: initialize lowp float floorwidth
#pragma mapbox: initialize mediump vec4 pattern_from
#pragma mapbox: initialize mediump vec4 pattern_to
#pragma mapbox: initialize lowp float pixel_ratio_from
#pragma mapbox: initialize lowp float pixel_ratio_to
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;float a_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*LINE_DISTANCE_SCALE;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;
#ifdef TERRAIN3D
v_gamma_scale=1.0;
#else
float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;
#endif
v_linesofar=a_linesofar;v_width2=vec2(outset,inset);v_width=floorwidth;}`), lineSDF: Gr(`uniform lowp float u_device_pixel_ratio;uniform sampler2D u_image;uniform float u_sdfgamma;uniform float u_mix;varying vec2 v_normal;varying vec2 v_width2;varying vec2 v_tex_a;varying vec2 v_tex_b;varying float v_gamma_scale;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float width
#pragma mapbox: define lowp float floorwidth
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float width
#pragma mapbox: initialize lowp float floorwidth
float dist=length(v_normal)*v_width2.s;float blur2=(blur+1.0/u_device_pixel_ratio)*v_gamma_scale;float alpha=clamp(min(dist-(v_width2.t-blur2),v_width2.s-dist)/blur2,0.0,1.0);float sdfdist_a=texture2D(u_image,v_tex_a).a;float sdfdist_b=texture2D(u_image,v_tex_b).a;float sdfdist=mix(sdfdist_a,sdfdist_b,u_mix);alpha*=smoothstep(0.5-u_sdfgamma/floorwidth,0.5+u_sdfgamma/floorwidth,sdfdist);gl_FragColor=color*(alpha*opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `
#define scale 0.015873016
#define LINE_DISTANCE_SCALE 2.0
attribute vec2 a_pos_normal;attribute vec4 a_data;uniform mat4 u_matrix;uniform mediump float u_ratio;uniform lowp float u_device_pixel_ratio;uniform vec2 u_patternscale_a;uniform float u_tex_y_a;uniform vec2 u_patternscale_b;uniform float u_tex_y_b;uniform vec2 u_units_to_pixels;varying vec2 v_normal;varying vec2 v_width2;varying vec2 v_tex_a;varying vec2 v_tex_b;varying float v_gamma_scale;
#pragma mapbox: define highp vec4 color
#pragma mapbox: define lowp float blur
#pragma mapbox: define lowp float opacity
#pragma mapbox: define mediump float gapwidth
#pragma mapbox: define lowp float offset
#pragma mapbox: define mediump float width
#pragma mapbox: define lowp float floorwidth
void main() {
#pragma mapbox: initialize highp vec4 color
#pragma mapbox: initialize lowp float blur
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize mediump float gapwidth
#pragma mapbox: initialize lowp float offset
#pragma mapbox: initialize mediump float width
#pragma mapbox: initialize lowp float floorwidth
float ANTIALIASING=1.0/u_device_pixel_ratio/2.0;vec2 a_extrude=a_data.xy-128.0;float a_direction=mod(a_data.z,4.0)-1.0;float a_linesofar=(floor(a_data.z/4.0)+a_data.w*64.0)*LINE_DISTANCE_SCALE;vec2 pos=floor(a_pos_normal*0.5);mediump vec2 normal=a_pos_normal-2.0*pos;normal.y=normal.y*2.0-1.0;v_normal=normal;gapwidth=gapwidth/2.0;float halfwidth=width/2.0;offset=-1.0*offset;float inset=gapwidth+(gapwidth > 0.0 ? ANTIALIASING : 0.0);float outset=gapwidth+halfwidth*(gapwidth > 0.0 ? 2.0 : 1.0)+(halfwidth==0.0 ? 0.0 : ANTIALIASING);mediump vec2 dist=outset*a_extrude*scale;mediump float u=0.5*a_direction;mediump float t=1.0-abs(u);mediump vec2 offset2=offset*a_extrude*scale*normal.y*mat2(t,-u,u,t);vec4 projected_extrude=u_matrix*vec4(dist/u_ratio,0.0,0.0);gl_Position=u_matrix*vec4(pos+offset2/u_ratio,0.0,1.0)+projected_extrude;
#ifdef TERRAIN3D
v_gamma_scale=1.0;
#else
float extrude_length_without_perspective=length(dist);float extrude_length_with_perspective=length(projected_extrude.xy/gl_Position.w*u_units_to_pixels);v_gamma_scale=extrude_length_without_perspective/extrude_length_with_perspective;
#endif
v_tex_a=vec2(a_linesofar*u_patternscale_a.x/floorwidth,normal.y*u_patternscale_a.y+u_tex_y_a);v_tex_b=vec2(a_linesofar*u_patternscale_b.x/floorwidth,normal.y*u_patternscale_b.y+u_tex_y_b);v_width2=vec2(outset,inset);}`), raster: Gr(`uniform float u_fade_t;uniform float u_opacity;uniform sampler2D u_image0;uniform sampler2D u_image1;varying vec2 v_pos0;varying vec2 v_pos1;uniform float u_brightness_low;uniform float u_brightness_high;uniform float u_saturation_factor;uniform float u_contrast_factor;uniform vec3 u_spin_weights;void main() {vec4 color0=texture2D(u_image0,v_pos0);vec4 color1=texture2D(u_image1,v_pos1);if (color0.a > 0.0) {color0.rgb=color0.rgb/color0.a;}if (color1.a > 0.0) {color1.rgb=color1.rgb/color1.a;}vec4 color=mix(color0,color1,u_fade_t);color.a*=u_opacity;vec3 rgb=color.rgb;rgb=vec3(dot(rgb,u_spin_weights.xyz),dot(rgb,u_spin_weights.zxy),dot(rgb,u_spin_weights.yzx));float average=(color.r+color.g+color.b)/3.0;rgb+=(average-rgb)*u_saturation_factor;rgb=(rgb-0.5)*u_contrast_factor+0.5;vec3 u_high_vec=vec3(u_brightness_low,u_brightness_low,u_brightness_low);vec3 u_low_vec=vec3(u_brightness_high,u_brightness_high,u_brightness_high);gl_FragColor=vec4(mix(u_high_vec,u_low_vec,rgb)*color.a,color.a);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, "uniform mat4 u_matrix;uniform vec2 u_tl_parent;uniform float u_scale_parent;uniform float u_buffer_scale;attribute vec2 a_pos;attribute vec2 a_texture_pos;varying vec2 v_pos0;varying vec2 v_pos1;void main() {gl_Position=u_matrix*vec4(a_pos,0,1);v_pos0=(((a_texture_pos/8192.0)-0.5)/u_buffer_scale )+0.5;v_pos1=(v_pos0*u_scale_parent)+u_tl_parent;}"), symbolIcon: Gr(`uniform sampler2D u_texture;varying vec2 v_tex;varying float v_fade_opacity;
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize lowp float opacity
lowp float alpha=opacity*v_fade_opacity;gl_FragColor=texture2D(u_texture,v_tex)*alpha;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec4 a_pixeloffset;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform highp float u_camera_to_center_distance;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform float u_fade_change;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform vec2 u_texsize;uniform bool u_is_along_line;uniform bool u_is_variable_anchor;uniform vec2 u_translation;uniform float u_pitched_scale;varying vec2 v_tex;varying float v_fade_opacity;vec4 projectTileWithElevation(vec2 posInTile,float elevation) {return u_matrix*vec4(posInTile,elevation,1.0);}
#pragma mapbox: define lowp float opacity
void main() {
#pragma mapbox: initialize lowp float opacity
vec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;float a_size_min=floor(a_size[0]*0.5);vec2 a_pxoffset=a_pixeloffset.xy;vec2 a_minFontScale=a_pixeloffset.zw/256.0;float ele=get_elevation(a_pos);highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size_min,a_size[1],u_size_t)/128.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size_min/128.0;} else {size=u_size;}vec2 translated_a_pos=a_pos+u_translation;vec4 projectedPoint=projectTileWithElevation(translated_a_pos,ele);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?
camera_to_anchor_distance/u_camera_to_center_distance :
u_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=u_is_text ? size/24.0 : size;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=projectTileWithElevation(translated_a_pos+vec2(1,0),ele);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos;if (u_is_along_line || u_is_variable_anchor) {projected_pos=vec4(a_projected_pos.xy,ele,1.0);} else if (u_pitch_with_map) {projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy+u_translation,ele,1.0);} else {projected_pos=u_label_plane_matrix*projectTileWithElevation(a_projected_pos.xy+u_translation,ele);}float z=float(u_pitch_with_map)*projected_pos.z/projected_pos.w;float projectionScaling=1.0;vec4 finalPos=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*max(a_minFontScale,fontScale)+a_pxoffset/16.0)*projectionScaling,z,1.0);if(u_pitch_with_map) {finalPos=projectTileWithElevation(finalPos.xy,finalPos.z);}gl_Position=finalPos;v_tex=a_tex/u_texsize;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;float visibility=calculate_visibility(projectedPoint);v_fade_opacity=max(0.0,min(visibility,fade_opacity[0]+fade_change));}`), symbolSDF: Gr(`#define SDF_PX 8.0
uniform bool u_is_halo;uniform sampler2D u_texture;uniform highp float u_gamma_scale;uniform lowp float u_device_pixel_ratio;uniform bool u_is_text;varying vec2 v_data0;varying vec3 v_data1;
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
float EDGE_GAMMA=0.105/u_device_pixel_ratio;vec2 tex=v_data0.xy;float gamma_scale=v_data1.x;float size=v_data1.y;float fade_opacity=v_data1[2];float fontScale=u_is_text ? size/24.0 : size;lowp vec4 color=fill_color;highp float gamma=EDGE_GAMMA/(fontScale*u_gamma_scale);lowp float inner_edge=(256.0-64.0)/256.0;if (u_is_halo) {color=halo_color;gamma=(halo_blur*1.19/SDF_PX+EDGE_GAMMA)/(fontScale*u_gamma_scale);inner_edge=inner_edge+gamma*gamma_scale;}lowp float dist=texture2D(u_texture,tex).a;highp float gamma_scaled=gamma*gamma_scale;highp float alpha=smoothstep(inner_edge-gamma_scaled,inner_edge+gamma_scaled,dist);if (u_is_halo) {lowp float halo_edge=(6.0-halo_width/fontScale)/SDF_PX;alpha=min(smoothstep(halo_edge-gamma_scaled,halo_edge+gamma_scaled,dist),1.0-alpha);}gl_FragColor=color*(alpha*opacity*fade_opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec4 a_pixeloffset;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform bool u_is_along_line;uniform bool u_is_variable_anchor;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform highp float u_camera_to_center_distance;uniform float u_fade_change;uniform vec2 u_texsize;uniform vec2 u_translation;uniform float u_pitched_scale;varying vec2 v_data0;varying vec3 v_data1;vec4 projectTileWithElevation(vec2 posInTile,float elevation) {return u_matrix*vec4(posInTile,elevation,1.0);}
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
vec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;float a_size_min=floor(a_size[0]*0.5);vec2 a_pxoffset=a_pixeloffset.xy;float ele=get_elevation(a_pos);highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size_min,a_size[1],u_size_t)/128.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size_min/128.0;} else {size=u_size;}vec2 translated_a_pos=a_pos+u_translation;vec4 projectedPoint=projectTileWithElevation(translated_a_pos,ele);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?
camera_to_anchor_distance/u_camera_to_center_distance :
u_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=u_is_text ? size/24.0 : size;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=projectTileWithElevation(translated_a_pos+vec2(1,0),ele);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos;if (u_is_along_line || u_is_variable_anchor) {projected_pos=vec4(a_projected_pos.xy,ele,1.0);} else if (u_pitch_with_map) {projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy+u_translation,ele,1.0);} else {projected_pos=u_label_plane_matrix*projectTileWithElevation(a_projected_pos.xy+u_translation,ele);}float z=float(u_pitch_with_map)*projected_pos.z/projected_pos.w;float projectionScaling=1.0;vec4 finalPos=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*fontScale+a_pxoffset)*projectionScaling,z,1.0);if(u_pitch_with_map) {finalPos=projectTileWithElevation(finalPos.xy,finalPos.z);}float gamma_scale=finalPos.w;gl_Position=finalPos;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float visibility=calculate_visibility(projectedPoint);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;float interpolated_fade_opacity=max(0.0,min(visibility,fade_opacity[0]+fade_change));v_data0=a_tex/u_texsize;v_data1=vec3(gamma_scale,size,interpolated_fade_opacity);}`), symbolTextAndIcon: Gr(`#define SDF_PX 8.0
#define SDF 1.0
#define ICON 0.0
uniform bool u_is_halo;uniform sampler2D u_texture;uniform sampler2D u_texture_icon;uniform highp float u_gamma_scale;uniform lowp float u_device_pixel_ratio;varying vec4 v_data0;varying vec4 v_data1;
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
float fade_opacity=v_data1[2];if (v_data1.w==ICON) {vec2 tex_icon=v_data0.zw;lowp float alpha=opacity*fade_opacity;gl_FragColor=texture2D(u_texture_icon,tex_icon)*alpha;
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
return;}vec2 tex=v_data0.xy;float EDGE_GAMMA=0.105/u_device_pixel_ratio;float gamma_scale=v_data1.x;float size=v_data1.y;float fontScale=size/24.0;lowp vec4 color=fill_color;highp float gamma=EDGE_GAMMA/(fontScale*u_gamma_scale);lowp float buff=(256.0-64.0)/256.0;if (u_is_halo) {color=halo_color;gamma=(halo_blur*1.19/SDF_PX+EDGE_GAMMA)/(fontScale*u_gamma_scale);buff=(6.0-halo_width/fontScale)/SDF_PX;}lowp float dist=texture2D(u_texture,tex).a;highp float gamma_scaled=gamma*gamma_scale;highp float alpha=smoothstep(buff-gamma_scaled,buff+gamma_scaled,dist);gl_FragColor=color*(alpha*opacity*fade_opacity);
#ifdef OVERDRAW_INSPECTOR
gl_FragColor=vec4(1.0);
#endif
}`, `attribute vec4 a_pos_offset;attribute vec4 a_data;attribute vec3 a_projected_pos;attribute float a_fade_opacity;uniform bool u_is_size_zoom_constant;uniform bool u_is_size_feature_constant;uniform highp float u_size_t;uniform highp float u_size;uniform mat4 u_matrix;uniform mat4 u_label_plane_matrix;uniform mat4 u_coord_matrix;uniform bool u_is_text;uniform bool u_pitch_with_map;uniform highp float u_pitch;uniform bool u_rotate_symbol;uniform highp float u_aspect_ratio;uniform highp float u_camera_to_center_distance;uniform float u_fade_change;uniform vec2 u_texsize;uniform vec2 u_texsize_icon;uniform bool u_is_along_line;uniform bool u_is_variable_anchor;uniform vec2 u_translation;uniform float u_pitched_scale;varying vec4 v_data0;varying vec4 v_data1;vec4 projectTileWithElevation(vec2 posInTile,float elevation) {return u_matrix*vec4(posInTile,elevation,1.0);}
#pragma mapbox: define highp vec4 fill_color
#pragma mapbox: define highp vec4 halo_color
#pragma mapbox: define lowp float opacity
#pragma mapbox: define lowp float halo_width
#pragma mapbox: define lowp float halo_blur
void main() {
#pragma mapbox: initialize highp vec4 fill_color
#pragma mapbox: initialize highp vec4 halo_color
#pragma mapbox: initialize lowp float opacity
#pragma mapbox: initialize lowp float halo_width
#pragma mapbox: initialize lowp float halo_blur
vec2 a_pos=a_pos_offset.xy;vec2 a_offset=a_pos_offset.zw;vec2 a_tex=a_data.xy;vec2 a_size=a_data.zw;float a_size_min=floor(a_size[0]*0.5);float is_sdf=a_size[0]-2.0*a_size_min;float ele=get_elevation(a_pos);highp float segment_angle=-a_projected_pos[2];float size;if (!u_is_size_zoom_constant && !u_is_size_feature_constant) {size=mix(a_size_min,a_size[1],u_size_t)/128.0;} else if (u_is_size_zoom_constant && !u_is_size_feature_constant) {size=a_size_min/128.0;} else {size=u_size;}vec2 translated_a_pos=a_pos+u_translation;vec4 projectedPoint=projectTileWithElevation(translated_a_pos,ele);highp float camera_to_anchor_distance=projectedPoint.w;highp float distance_ratio=u_pitch_with_map ?
camera_to_anchor_distance/u_camera_to_center_distance :
u_camera_to_center_distance/camera_to_anchor_distance;highp float perspective_ratio=clamp(0.5+0.5*distance_ratio,0.0,4.0);size*=perspective_ratio;float fontScale=size/24.0;highp float symbol_rotation=0.0;if (u_rotate_symbol) {vec4 offsetProjectedPoint=projectTileWithElevation(translated_a_pos+vec2(1,0),ele);vec2 a=projectedPoint.xy/projectedPoint.w;vec2 b=offsetProjectedPoint.xy/offsetProjectedPoint.w;symbol_rotation=atan((b.y-a.y)/u_aspect_ratio,b.x-a.x);}highp float angle_sin=sin(segment_angle+symbol_rotation);highp float angle_cos=cos(segment_angle+symbol_rotation);mat2 rotation_matrix=mat2(angle_cos,-1.0*angle_sin,angle_sin,angle_cos);vec4 projected_pos;if (u_is_along_line || u_is_variable_anchor) {projected_pos=vec4(a_projected_pos.xy,ele,1.0);} else if (u_pitch_with_map) {projected_pos=u_label_plane_matrix*vec4(a_projected_pos.xy+u_translation,ele,1.0);} else {projected_pos=u_label_plane_matrix*projectTileWithElevation(a_projected_pos.xy+u_translation,ele);}float z=float(u_pitch_with_map)*projected_pos.z/projected_pos.w;float projectionScaling=1.0;vec4 finalPos=u_coord_matrix*vec4(projected_pos.xy/projected_pos.w+rotation_matrix*(a_offset/32.0*fontScale)*projectionScaling,z,1.0);if(u_pitch_with_map) {finalPos=projectTileWithElevation(finalPos.xy,finalPos.z);}float gamma_scale=finalPos.w;gl_Position=finalPos;vec2 fade_opacity=unpack_opacity(a_fade_opacity);float visibility=calculate_visibility(projectedPoint);float fade_change=fade_opacity[1] > 0.5 ? u_fade_change :-u_fade_change;float interpolated_fade_opacity=max(0.0,min(visibility,fade_opacity[0]+fade_change));v_data0.xy=a_tex/u_texsize;v_data0.zw=a_tex/u_texsize_icon;v_data1=vec4(gamma_scale,size,interpolated_fade_opacity,is_sdf);}`), terrain: Gr("uniform sampler2D u_texture;uniform vec4 u_fog_color;uniform vec4 u_horizon_color;uniform float u_fog_ground_blend;uniform float u_fog_ground_blend_opacity;uniform float u_horizon_fog_blend;varying vec2 v_texture_pos;varying float v_fog_depth;const float gamma=2.2;vec4 gammaToLinear(vec4 color) {return pow(color,vec4(gamma));}vec4 linearToGamma(vec4 color) {return pow(color,vec4(1.0/gamma));}void main() {vec4 surface_color=texture2D(u_texture,v_texture_pos);if (v_fog_depth > u_fog_ground_blend) {vec4 surface_color_linear=gammaToLinear(surface_color);float blend_color=smoothstep(0.0,1.0,max((v_fog_depth-u_horizon_fog_blend)/(1.0-u_horizon_fog_blend),0.0));vec4 fog_horizon_color_linear=mix(gammaToLinear(u_fog_color),gammaToLinear(u_horizon_color),blend_color);float factor_fog=max(v_fog_depth-u_fog_ground_blend,0.0)/(1.0-u_fog_ground_blend);gl_FragColor=linearToGamma(mix(surface_color_linear,fog_horizon_color_linear,pow(factor_fog,2.0)*u_fog_ground_blend_opacity));} else {gl_FragColor=surface_color;}}", "attribute vec3 a_pos3d;uniform mat4 u_matrix;uniform mat4 u_fog_matrix;uniform float u_ele_delta;varying vec2 v_texture_pos;varying float v_fog_depth;void main() {float ele=get_elevation(a_pos3d.xy);float ele_delta=a_pos3d.z==1.0 ? u_ele_delta : 0.0;v_texture_pos=a_pos3d.xy/8192.0;gl_Position=u_matrix*vec4(a_pos3d.xy,ele-ele_delta,1.0);vec4 pos=u_fog_matrix*vec4(a_pos3d.xy,ele,1.0);v_fog_depth=pos.z/pos.w*0.5+0.5;}"), terrainDepth: Gr("varying float v_depth;const highp vec4 bitSh=vec4(256.*256.*256.,256.*256.,256.,1.);const highp vec4 bitMsk=vec4(0.,vec3(1./256.0));highp vec4 pack(highp float value) {highp vec4 comp=fract(value*bitSh);comp-=comp.xxyz*bitMsk;return comp;}void main() {gl_FragColor=pack(v_depth);}", "attribute vec3 a_pos3d;uniform mat4 u_matrix;uniform float u_ele_delta;varying float v_depth;void main() {float ele=get_elevation(a_pos3d.xy);float ele_delta=a_pos3d.z==1.0 ? u_ele_delta : 0.0;gl_Position=u_matrix*vec4(a_pos3d.xy,ele-ele_delta,1.0);v_depth=gl_Position.z/gl_Position.w;}"), terrainCoords: Gr("precision mediump float;uniform sampler2D u_texture;uniform float u_terrain_coords_id;varying vec2 v_texture_pos;void main() {vec4 rgba=texture2D(u_texture,v_texture_pos);gl_FragColor=vec4(rgba.r,rgba.g,rgba.b,u_terrain_coords_id);}", "attribute vec3 a_pos3d;uniform mat4 u_matrix;uniform float u_ele_delta;varying vec2 v_texture_pos;void main() {float ele=get_elevation(a_pos3d.xy);float ele_delta=a_pos3d.z==1.0 ? u_ele_delta : 0.0;v_texture_pos=a_pos3d.xy/8192.0;gl_Position=u_matrix*vec4(a_pos3d.xy,ele-ele_delta,1.0);}"), sky: Gr("uniform vec4 u_sky_color;uniform vec4 u_horizon_color;uniform float u_horizon;uniform float u_sky_horizon_blend;void main() {float y=gl_FragCoord.y;if (y > u_horizon) {float blend=y-u_horizon;if (blend < u_sky_horizon_blend) {gl_FragColor=mix(u_sky_color,u_horizon_color,pow(1.0-blend/u_sky_horizon_blend,2.0));} else {gl_FragColor=u_sky_color;}}}", "attribute vec2 a_pos;void main() {gl_Position=vec4(a_pos,1.0,1.0);}") };
                function Gr(Fe, Te) {
                  const Ae = /#pragma mapbox: ([\w]+) ([\w]+) ([\w]+) ([\w]+)/g, Oe = Te.match(/attribute ([\w]+) ([\w]+)/g), Be = Fe.match(/uniform ([\w]+) ([\w]+)([\s]*)([\w]*)/g), Ue = Te.match(/uniform ([\w]+) ([\w]+)([\s]*)([\w]*)/g), nt = Ue ? Ue.concat(Be) : Be, ht = {};
                  return { fragmentSource: Fe = Fe.replace(Ae, (Et, Mt, qt, tn, xn) => (ht[xn] = !0, Mt === "define" ? `
#ifndef HAS_UNIFORM_u_${xn}
varying ${qt} ${tn} ${xn};
#else
uniform ${qt} ${tn} u_${xn};
#endif
` : `
#ifdef HAS_UNIFORM_u_${xn}
    ${qt} ${tn} ${xn} = u_${xn};
#endif
`)), vertexSource: Te = Te.replace(Ae, (Et, Mt, qt, tn, xn) => {
                    const An = tn === "float" ? "vec2" : "vec4", On = xn.match(/color/) ? "color" : An;
                    return ht[xn] ? Mt === "define" ? `
#ifndef HAS_UNIFORM_u_${xn}
uniform lowp float u_${xn}_t;
attribute ${qt} ${An} a_${xn};
varying ${qt} ${tn} ${xn};
#else
uniform ${qt} ${tn} u_${xn};
#endif
` : On === "vec4" ? `
#ifndef HAS_UNIFORM_u_${xn}
    ${xn} = a_${xn};
#else
    ${qt} ${tn} ${xn} = u_${xn};
#endif
` : `
#ifndef HAS_UNIFORM_u_${xn}
    ${xn} = unpack_mix_${On}(a_${xn}, u_${xn}_t);
#else
    ${qt} ${tn} ${xn} = u_${xn};
#endif
` : Mt === "define" ? `
#ifndef HAS_UNIFORM_u_${xn}
uniform lowp float u_${xn}_t;
attribute ${qt} ${An} a_${xn};
#else
uniform ${qt} ${tn} u_${xn};
#endif
` : On === "vec4" ? `
#ifndef HAS_UNIFORM_u_${xn}
    ${qt} ${tn} ${xn} = a_${xn};
#else
    ${qt} ${tn} ${xn} = u_${xn};
#endif
` : `
#ifndef HAS_UNIFORM_u_${xn}
    ${qt} ${tn} ${xn} = unpack_mix_${On}(a_${xn}, u_${xn}_t);
#else
    ${qt} ${tn} ${xn} = u_${xn};
#endif
`;
                  }), staticAttributes: Oe, staticUniforms: nt };
                }
                class Ca {
                  constructor() {
                    this.boundProgram = null, this.boundLayoutVertexBuffer = null, this.boundPaintVertexBuffers = [], this.boundIndexBuffer = null, this.boundVertexOffset = null, this.boundDynamicVertexBuffer = null, this.vao = null;
                  }
                  bind(Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt) {
                    this.context = Te;
                    let qt = this.boundPaintVertexBuffers.length !== Be.length;
                    for (let tn = 0; !qt && tn < Be.length; tn++) this.boundPaintVertexBuffers[tn] !== Be[tn] && (qt = !0);
                    !this.vao || this.boundProgram !== Ae || this.boundLayoutVertexBuffer !== Oe || qt || this.boundIndexBuffer !== Ue || this.boundVertexOffset !== nt || this.boundDynamicVertexBuffer !== ht || this.boundDynamicVertexBuffer2 !== Et || this.boundDynamicVertexBuffer3 !== Mt ? this.freshBind(Ae, Oe, Be, Ue, nt, ht, Et, Mt) : (Te.bindVertexArray.set(this.vao), ht && ht.bind(), Ue && Ue.dynamicDraw && Ue.bind(), Et && Et.bind(), Mt && Mt.bind());
                  }
                  freshBind(Te, Ae, Oe, Be, Ue, nt, ht, Et) {
                    const Mt = Te.numAttributes, qt = this.context, tn = qt.gl;
                    this.vao && this.destroy(), this.vao = qt.createVertexArray(), qt.bindVertexArray.set(this.vao), this.boundProgram = Te, this.boundLayoutVertexBuffer = Ae, this.boundPaintVertexBuffers = Oe, this.boundIndexBuffer = Be, this.boundVertexOffset = Ue, this.boundDynamicVertexBuffer = nt, this.boundDynamicVertexBuffer2 = ht, this.boundDynamicVertexBuffer3 = Et, Ae.enableAttributes(tn, Te);
                    for (const xn of Oe) xn.enableAttributes(tn, Te);
                    nt && nt.enableAttributes(tn, Te), ht && ht.enableAttributes(tn, Te), Et && Et.enableAttributes(tn, Te), Ae.bind(), Ae.setVertexAttribPointers(tn, Te, Ue);
                    for (const xn of Oe) xn.bind(), xn.setVertexAttribPointers(tn, Te, Ue);
                    nt && (nt.bind(), nt.setVertexAttribPointers(tn, Te, Ue)), Be && Be.bind(), ht && (ht.bind(), ht.setVertexAttribPointers(tn, Te, Ue)), Et && (Et.bind(), Et.setVertexAttribPointers(tn, Te, Ue)), qt.currentNumAttributes = Mt;
                  }
                  destroy() {
                    this.vao && (this.context.deleteVertexArray(this.vao), this.vao = null);
                  }
                }
                const Sr = (Fe, Te, Ae, Oe, Be) => ({ u_matrix: Fe, u_texture: 0, u_ele_delta: Te, u_fog_matrix: Ae, u_fog_color: Oe ? Oe.properties.get("fog-color") : ee.aN.white, u_fog_ground_blend: Oe ? Oe.properties.get("fog-ground-blend") : 1, u_fog_ground_blend_opacity: Oe ? Oe.calculateFogBlendOpacity(Be) : 0, u_horizon_color: Oe ? Oe.properties.get("horizon-color") : ee.aN.white, u_horizon_fog_blend: Oe ? Oe.properties.get("horizon-fog-blend") : 1 });
                function Dr(Fe) {
                  const Te = [];
                  for (let Ae = 0; Ae < Fe.length; Ae++) {
                    if (Fe[Ae] === null) continue;
                    const Oe = Fe[Ae].split(" ");
                    Te.push(Oe.pop());
                  }
                  return Te;
                }
                class si {
                  constructor(Te, Ae, Oe, Be, Ue, nt) {
                    const ht = Te.gl;
                    this.program = ht.createProgram();
                    const Et = Dr(Ae.staticAttributes), Mt = Oe ? Oe.getBinderAttributes() : [], qt = Et.concat(Mt), tn = hi.prelude.staticUniforms ? Dr(hi.prelude.staticUniforms) : [], xn = Ae.staticUniforms ? Dr(Ae.staticUniforms) : [], An = Oe ? Oe.getBinderUniforms() : [], On = tn.concat(xn).concat(An), Bn = [];
                    for (const Tr of On) Bn.indexOf(Tr) < 0 && Bn.push(Tr);
                    const qn = Oe ? Oe.defines() : [];
                    Ue && qn.push("#define OVERDRAW_INSPECTOR;"), nt && qn.push("#define TERRAIN3D;");
                    const Vn = qn.concat(hi.prelude.fragmentSource, Ae.fragmentSource).join(`
`), sr = qn.concat(hi.prelude.vertexSource, Ae.vertexSource).join(`
`), Un = ht.createShader(ht.FRAGMENT_SHADER);
                    if (ht.isContextLost()) return void (this.failedToCreate = !0);
                    if (ht.shaderSource(Un, Vn), ht.compileShader(Un), !ht.getShaderParameter(Un, ht.COMPILE_STATUS)) throw new Error(`Could not compile fragment shader: ${ht.getShaderInfoLog(Un)}`);
                    ht.attachShader(this.program, Un);
                    const wr = ht.createShader(ht.VERTEX_SHADER);
                    if (ht.isContextLost()) return void (this.failedToCreate = !0);
                    if (ht.shaderSource(wr, sr), ht.compileShader(wr), !ht.getShaderParameter(wr, ht.COMPILE_STATUS)) throw new Error(`Could not compile vertex shader: ${ht.getShaderInfoLog(wr)}`);
                    ht.attachShader(this.program, wr), this.attributes = {};
                    const Cr = {};
                    this.numAttributes = qt.length;
                    for (let Tr = 0; Tr < this.numAttributes; Tr++) qt[Tr] && (ht.bindAttribLocation(this.program, Tr, qt[Tr]), this.attributes[qt[Tr]] = Tr);
                    if (ht.linkProgram(this.program), !ht.getProgramParameter(this.program, ht.LINK_STATUS)) throw new Error(`Program failed to link: ${ht.getProgramInfoLog(this.program)}`);
                    ht.deleteShader(wr), ht.deleteShader(Un);
                    for (let Tr = 0; Tr < Bn.length; Tr++) {
                      const ii = Bn[Tr];
                      if (ii && !Cr[ii]) {
                        const wi = ht.getUniformLocation(this.program, ii);
                        wi && (Cr[ii] = wi);
                      }
                    }
                    this.fixedUniforms = Be(Te, Cr), this.terrainUniforms = ((Tr, ii) => ({ u_depth: new ee.aI(Tr, ii.u_depth), u_terrain: new ee.aI(Tr, ii.u_terrain), u_terrain_dim: new ee.aJ(Tr, ii.u_terrain_dim), u_terrain_matrix: new ee.aK(Tr, ii.u_terrain_matrix), u_terrain_unpack: new ee.aL(Tr, ii.u_terrain_unpack), u_terrain_exaggeration: new ee.aJ(Tr, ii.u_terrain_exaggeration) }))(Te, Cr), this.binderUniforms = Oe ? Oe.getUniforms(Te, Cr) : [];
                  }
                  draw(Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An, On, Bn, qn, Vn, sr) {
                    const Un = Te.gl;
                    if (this.failedToCreate) return;
                    if (Te.program.set(this.program), Te.setDepthMode(Oe), Te.setStencilMode(Be), Te.setColorMode(Ue), Te.setCullFace(nt), Et) {
                      Te.activeTexture.set(Un.TEXTURE2), Un.bindTexture(Un.TEXTURE_2D, Et.depthTexture), Te.activeTexture.set(Un.TEXTURE3), Un.bindTexture(Un.TEXTURE_2D, Et.texture);
                      for (const Cr in this.terrainUniforms) this.terrainUniforms[Cr].set(Et[Cr]);
                    }
                    for (const Cr in this.fixedUniforms) this.fixedUniforms[Cr].set(ht[Cr]);
                    Bn && Bn.setUniforms(Te, this.binderUniforms, An, { zoom: On });
                    let wr = 0;
                    switch (Ae) {
                      case Un.LINES:
                        wr = 2;
                        break;
                      case Un.TRIANGLES:
                        wr = 3;
                        break;
                      case Un.LINE_STRIP:
                        wr = 1;
                    }
                    for (const Cr of xn.get()) {
                      const Tr = Cr.vaos || (Cr.vaos = {});
                      (Tr[Mt] || (Tr[Mt] = new Ca())).bind(Te, this, qt, Bn ? Bn.getPaintVertexBuffers() : [], tn, Cr.vertexOffset, qn, Vn, sr), Un.drawElements(Ae, Cr.primitiveLength * wr, Un.UNSIGNED_SHORT, Cr.primitiveOffset * wr * 2);
                    }
                  }
                }
                function zr(Fe, Te, Ae) {
                  const Oe = 1 / Mn(Ae, 1, Te.transform.tileZoom), Be = Math.pow(2, Ae.tileID.overscaledZ), Ue = Ae.tileSize * Math.pow(2, Te.transform.tileZoom) / Be, nt = Ue * (Ae.tileID.canonical.x + Ae.tileID.wrap * Be), ht = Ue * Ae.tileID.canonical.y;
                  return { u_image: 0, u_texsize: Ae.imageAtlasTexture.size, u_scale: [Oe, Fe.fromScale, Fe.toScale], u_fade: Fe.t, u_pixel_coord_upper: [nt >> 16, ht >> 16], u_pixel_coord_lower: [65535 & nt, 65535 & ht] };
                }
                const di = (Fe, Te, Ae, Oe) => {
                  const Be = Te.style.light, Ue = Be.properties.get("position"), nt = [Ue.x, Ue.y, Ue.z], ht = function() {
                    var Mt = new ee.A(9);
                    return ee.A != Float32Array && (Mt[1] = 0, Mt[2] = 0, Mt[3] = 0, Mt[5] = 0, Mt[6] = 0, Mt[7] = 0), Mt[0] = 1, Mt[4] = 1, Mt[8] = 1, Mt;
                  }();
                  Be.properties.get("anchor") === "viewport" && function(Mt, qt) {
                    var tn = Math.sin(qt), xn = Math.cos(qt);
                    Mt[0] = xn, Mt[1] = tn, Mt[2] = 0, Mt[3] = -tn, Mt[4] = xn, Mt[5] = 0, Mt[6] = 0, Mt[7] = 0, Mt[8] = 1;
                  }(ht, -Te.transform.angle), function(Mt, qt, tn) {
                    var xn = qt[0], An = qt[1], On = qt[2];
                    Mt[0] = xn * tn[0] + An * tn[3] + On * tn[6], Mt[1] = xn * tn[1] + An * tn[4] + On * tn[7], Mt[2] = xn * tn[2] + An * tn[5] + On * tn[8];
                  }(nt, nt, ht);
                  const Et = Be.properties.get("color");
                  return { u_matrix: Fe, u_lightpos: nt, u_lightintensity: Be.properties.get("intensity"), u_lightcolor: [Et.r, Et.g, Et.b], u_vertical_gradient: +Ae, u_opacity: Oe };
                }, Da = (Fe, Te, Ae, Oe, Be, Ue, nt) => ee.e(di(Fe, Te, Ae, Oe), zr(Ue, Te, nt), { u_height_factor: -Math.pow(2, Be.overscaledZ) / nt.tileSize / 8 }), la = (Fe) => ({ u_matrix: Fe }), wa = (Fe, Te, Ae, Oe) => ee.e(la(Fe), zr(Ae, Te, Oe)), Wa = (Fe, Te) => ({ u_matrix: Fe, u_world: Te }), Qr = (Fe, Te, Ae, Oe, Be) => ee.e(wa(Fe, Te, Ae, Oe), { u_world: Be }), Ur = (Fe, Te, Ae, Oe) => {
                  const Be = Fe.transform;
                  let Ue, nt;
                  if (Oe.paint.get("circle-pitch-alignment") === "map") {
                    const ht = Mn(Ae, 1, Be.zoom);
                    Ue = !0, nt = [ht, ht];
                  } else Ue = !1, nt = Be.pixelsToGLUnits;
                  return { u_camera_to_center_distance: Be.cameraToCenterDistance, u_scale_with_map: +(Oe.paint.get("circle-pitch-scale") === "map"), u_matrix: Fe.translatePosMatrix(Te.posMatrix, Ae, Oe.paint.get("circle-translate"), Oe.paint.get("circle-translate-anchor")), u_pitch_with_map: +Ue, u_device_pixel_ratio: Fe.pixelRatio, u_extrude_scale: nt };
                }, rr = (Fe, Te) => ({ u_matrix: Te, u_pixel_extrude_scale: [1 / Fe.width, 1 / Fe.height] }), vr = (Fe, Te, Ae) => ({ u_matrix: Fe, u_inv_matrix: Te, u_camera_to_center_distance: Ae.cameraToCenterDistance, u_viewport_size: [Ae.width, Ae.height] }), Rr = (Fe, Te, Ae = 1) => ({ u_matrix: Fe, u_color: Te, u_overlay: 0, u_overlay_scale: Ae }), Vr = (Fe) => ({ u_matrix: Fe }), Xr = (Fe, Te, Ae, Oe) => ({ u_matrix: Fe, u_extrude_scale: Mn(Te, 1, Ae), u_intensity: Oe }), Jr = (Fe, Te, Ae, Oe) => {
                  const Be = ee.H();
                  ee.aQ(Be, 0, Fe.width, Fe.height, 0, 0, 1);
                  const Ue = Fe.context.gl;
                  return { u_matrix: Be, u_world: [Ue.drawingBufferWidth, Ue.drawingBufferHeight], u_image: Ae, u_color_ramp: Oe, u_opacity: Te.paint.get("heatmap-opacity") };
                }, ti = (Fe, Te, Ae, Oe) => {
                  const Be = Ae.paint.get("hillshade-shadow-color"), Ue = Ae.paint.get("hillshade-highlight-color"), nt = Ae.paint.get("hillshade-accent-color");
                  let ht = Ae.paint.get("hillshade-illumination-direction") * (Math.PI / 180);
                  Ae.paint.get("hillshade-illumination-anchor") === "viewport" && (ht -= Fe.transform.angle);
                  const Et = !Fe.options.moving;
                  return { u_matrix: Oe ? Oe.posMatrix : Fe.transform.calculatePosMatrix(Te.tileID.toUnwrapped(), Et), u_image: 0, u_latrange: Er(0, Te.tileID), u_light: [Ae.paint.get("hillshade-exaggeration"), ht], u_shadow: Be, u_highlight: Ue, u_accent: nt };
                }, Hr = (Fe, Te) => {
                  const Ae = Te.stride, Oe = ee.H();
                  return ee.aQ(Oe, 0, ee.X, -ee.X, 0, 0, 1), ee.J(Oe, Oe, [0, -ee.X, 0]), { u_matrix: Oe, u_image: 1, u_dimension: [Ae, Ae], u_zoom: Fe.overscaledZ, u_unpack: Te.getUnpackVector() };
                };
                function Er(Fe, Te) {
                  const Ae = Math.pow(2, Te.canonical.z), Oe = Te.canonical.y;
                  return [new ee.Z(0, Oe / Ae).toLngLat().lat, new ee.Z(0, (Oe + 1) / Ae).toLngLat().lat];
                }
                const gr = (Fe, Te, Ae, Oe) => {
                  const Be = Fe.transform;
                  return { u_matrix: Li(Fe, Te, Ae, Oe), u_ratio: 1 / Mn(Te, 1, Be.zoom), u_device_pixel_ratio: Fe.pixelRatio, u_units_to_pixels: [1 / Be.pixelsToGLUnits[0], 1 / Be.pixelsToGLUnits[1]] };
                }, $r = (Fe, Te, Ae, Oe, Be) => ee.e(gr(Fe, Te, Ae, Be), { u_image: 0, u_image_height: Oe }), li = (Fe, Te, Ae, Oe, Be) => {
                  const Ue = Fe.transform, nt = Ai(Te, Ue);
                  return { u_matrix: Li(Fe, Te, Ae, Be), u_texsize: Te.imageAtlasTexture.size, u_ratio: 1 / Mn(Te, 1, Ue.zoom), u_device_pixel_ratio: Fe.pixelRatio, u_image: 0, u_scale: [nt, Oe.fromScale, Oe.toScale], u_fade: Oe.t, u_units_to_pixels: [1 / Ue.pixelsToGLUnits[0], 1 / Ue.pixelsToGLUnits[1]] };
                }, Qi = (Fe, Te, Ae, Oe, Be, Ue) => {
                  const nt = Fe.transform, ht = Fe.lineAtlas, Et = Ai(Te, nt), Mt = Ae.layout.get("line-cap") === "round", qt = ht.getDash(Oe.from, Mt), tn = ht.getDash(Oe.to, Mt), xn = qt.width * Be.fromScale, An = tn.width * Be.toScale;
                  return ee.e(gr(Fe, Te, Ae, Ue), { u_patternscale_a: [Et / xn, -qt.height / 2], u_patternscale_b: [Et / An, -tn.height / 2], u_sdfgamma: ht.width / (256 * Math.min(xn, An) * Fe.pixelRatio) / 2, u_image: 0, u_tex_y_a: qt.y, u_tex_y_b: tn.y, u_mix: Be.t });
                };
                function Ai(Fe, Te) {
                  return 1 / Mn(Fe, 1, Te.tileZoom);
                }
                function Li(Fe, Te, Ae, Oe) {
                  return Fe.translatePosMatrix(Oe ? Oe.posMatrix : Te.tileID.posMatrix, Te, Ae.paint.get("line-translate"), Ae.paint.get("line-translate-anchor"));
                }
                const Ni = (Fe, Te, Ae, Oe, Be) => {
                  return { u_matrix: Fe, u_tl_parent: Te, u_scale_parent: Ae, u_buffer_scale: 1, u_fade_t: Oe.mix, u_opacity: Oe.opacity * Be.paint.get("raster-opacity"), u_image0: 0, u_image1: 1, u_brightness_low: Be.paint.get("raster-brightness-min"), u_brightness_high: Be.paint.get("raster-brightness-max"), u_saturation_factor: (nt = Be.paint.get("raster-saturation"), nt > 0 ? 1 - 1 / (1.001 - nt) : -nt), u_contrast_factor: (Ue = Be.paint.get("raster-contrast"), Ue > 0 ? 1 / (1 - Ue) : 1 + Ue), u_spin_weights: Pi(Be.paint.get("raster-hue-rotate")) };
                  var Ue, nt;
                };
                function Pi(Fe) {
                  Fe *= Math.PI / 180;
                  const Te = Math.sin(Fe), Ae = Math.cos(Fe);
                  return [(2 * Ae + 1) / 3, (-Math.sqrt(3) * Te - Ae + 1) / 3, (Math.sqrt(3) * Te - Ae + 1) / 3];
                }
                const Oi = (Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An) => {
                  const On = nt.transform;
                  return { u_is_size_zoom_constant: +(Fe === "constant" || Fe === "source"), u_is_size_feature_constant: +(Fe === "constant" || Fe === "camera"), u_size_t: Te ? Te.uSizeT : 0, u_size: Te ? Te.uSize : 0, u_camera_to_center_distance: On.cameraToCenterDistance, u_pitch: On.pitch / 360 * 2 * Math.PI, u_rotate_symbol: +Ae, u_aspect_ratio: On.width / On.height, u_fade_change: nt.options.fadeDuration ? nt.symbolFadeChange : 1, u_matrix: ht, u_label_plane_matrix: Et, u_coord_matrix: Mt, u_is_text: +tn, u_pitch_with_map: +Oe, u_is_along_line: Be, u_is_variable_anchor: Ue, u_texsize: xn, u_texture: 0, u_translation: qt, u_pitched_scale: An };
                }, ki = (Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An, On) => {
                  const Bn = nt.transform;
                  return ee.e(Oi(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, On), { u_gamma_scale: Oe ? Math.cos(Bn._pitch) * Bn.cameraToCenterDistance : 1, u_device_pixel_ratio: nt.pixelRatio, u_is_halo: 1 });
                }, Mi = (Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An) => ee.e(ki(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, !0, tn, !0, An), { u_texsize_icon: xn, u_texture_icon: 1 }), Vi = (Fe, Te, Ae) => ({ u_matrix: Fe, u_opacity: Te, u_color: Ae }), ua = (Fe, Te, Ae, Oe, Be, Ue) => ee.e(function(nt, ht, Et, Mt) {
                  const qt = Et.imageManager.getPattern(nt.from.toString()), tn = Et.imageManager.getPattern(nt.to.toString()), { width: xn, height: An } = Et.imageManager.getPixelSize(), On = Math.pow(2, Mt.tileID.overscaledZ), Bn = Mt.tileSize * Math.pow(2, Et.transform.tileZoom) / On, qn = Bn * (Mt.tileID.canonical.x + Mt.tileID.wrap * On), Vn = Bn * Mt.tileID.canonical.y;
                  return { u_image: 0, u_pattern_tl_a: qt.tl, u_pattern_br_a: qt.br, u_pattern_tl_b: tn.tl, u_pattern_br_b: tn.br, u_texsize: [xn, An], u_mix: ht.t, u_pattern_size_a: qt.displaySize, u_pattern_size_b: tn.displaySize, u_scale_a: ht.fromScale, u_scale_b: ht.toScale, u_tile_units_to_pixels: 1 / Mn(Mt, 1, Et.transform.tileZoom), u_pixel_coord_upper: [qn >> 16, Vn >> 16], u_pixel_coord_lower: [65535 & qn, 65535 & Vn] };
                }(Oe, Ue, Ae, Be), { u_matrix: Fe, u_opacity: Te }), Ki = { fillExtrusion: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_lightpos: new ee.aO(Fe, Te.u_lightpos), u_lightintensity: new ee.aJ(Fe, Te.u_lightintensity), u_lightcolor: new ee.aO(Fe, Te.u_lightcolor), u_vertical_gradient: new ee.aJ(Fe, Te.u_vertical_gradient), u_opacity: new ee.aJ(Fe, Te.u_opacity) }), fillExtrusionPattern: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_lightpos: new ee.aO(Fe, Te.u_lightpos), u_lightintensity: new ee.aJ(Fe, Te.u_lightintensity), u_lightcolor: new ee.aO(Fe, Te.u_lightcolor), u_vertical_gradient: new ee.aJ(Fe, Te.u_vertical_gradient), u_height_factor: new ee.aJ(Fe, Te.u_height_factor), u_image: new ee.aI(Fe, Te.u_image), u_texsize: new ee.aP(Fe, Te.u_texsize), u_pixel_coord_upper: new ee.aP(Fe, Te.u_pixel_coord_upper), u_pixel_coord_lower: new ee.aP(Fe, Te.u_pixel_coord_lower), u_scale: new ee.aO(Fe, Te.u_scale), u_fade: new ee.aJ(Fe, Te.u_fade), u_opacity: new ee.aJ(Fe, Te.u_opacity) }), fill: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix) }), fillPattern: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_image: new ee.aI(Fe, Te.u_image), u_texsize: new ee.aP(Fe, Te.u_texsize), u_pixel_coord_upper: new ee.aP(Fe, Te.u_pixel_coord_upper), u_pixel_coord_lower: new ee.aP(Fe, Te.u_pixel_coord_lower), u_scale: new ee.aO(Fe, Te.u_scale), u_fade: new ee.aJ(Fe, Te.u_fade) }), fillOutline: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_world: new ee.aP(Fe, Te.u_world) }), fillOutlinePattern: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_world: new ee.aP(Fe, Te.u_world), u_image: new ee.aI(Fe, Te.u_image), u_texsize: new ee.aP(Fe, Te.u_texsize), u_pixel_coord_upper: new ee.aP(Fe, Te.u_pixel_coord_upper), u_pixel_coord_lower: new ee.aP(Fe, Te.u_pixel_coord_lower), u_scale: new ee.aO(Fe, Te.u_scale), u_fade: new ee.aJ(Fe, Te.u_fade) }), circle: (Fe, Te) => ({ u_camera_to_center_distance: new ee.aJ(Fe, Te.u_camera_to_center_distance), u_scale_with_map: new ee.aI(Fe, Te.u_scale_with_map), u_pitch_with_map: new ee.aI(Fe, Te.u_pitch_with_map), u_extrude_scale: new ee.aP(Fe, Te.u_extrude_scale), u_device_pixel_ratio: new ee.aJ(Fe, Te.u_device_pixel_ratio), u_matrix: new ee.aK(Fe, Te.u_matrix) }), collisionBox: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_pixel_extrude_scale: new ee.aP(Fe, Te.u_pixel_extrude_scale) }), collisionCircle: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_inv_matrix: new ee.aK(Fe, Te.u_inv_matrix), u_camera_to_center_distance: new ee.aJ(Fe, Te.u_camera_to_center_distance), u_viewport_size: new ee.aP(Fe, Te.u_viewport_size) }), debug: (Fe, Te) => ({ u_color: new ee.aM(Fe, Te.u_color), u_matrix: new ee.aK(Fe, Te.u_matrix), u_overlay: new ee.aI(Fe, Te.u_overlay), u_overlay_scale: new ee.aJ(Fe, Te.u_overlay_scale) }), clippingMask: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix) }), heatmap: (Fe, Te) => ({ u_extrude_scale: new ee.aJ(Fe, Te.u_extrude_scale), u_intensity: new ee.aJ(Fe, Te.u_intensity), u_matrix: new ee.aK(Fe, Te.u_matrix) }), heatmapTexture: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_world: new ee.aP(Fe, Te.u_world), u_image: new ee.aI(Fe, Te.u_image), u_color_ramp: new ee.aI(Fe, Te.u_color_ramp), u_opacity: new ee.aJ(Fe, Te.u_opacity) }), hillshade: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_image: new ee.aI(Fe, Te.u_image), u_latrange: new ee.aP(Fe, Te.u_latrange), u_light: new ee.aP(Fe, Te.u_light), u_shadow: new ee.aM(Fe, Te.u_shadow), u_highlight: new ee.aM(Fe, Te.u_highlight), u_accent: new ee.aM(Fe, Te.u_accent) }), hillshadePrepare: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_image: new ee.aI(Fe, Te.u_image), u_dimension: new ee.aP(Fe, Te.u_dimension), u_zoom: new ee.aJ(Fe, Te.u_zoom), u_unpack: new ee.aL(Fe, Te.u_unpack) }), line: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_ratio: new ee.aJ(Fe, Te.u_ratio), u_device_pixel_ratio: new ee.aJ(Fe, Te.u_device_pixel_ratio), u_units_to_pixels: new ee.aP(Fe, Te.u_units_to_pixels) }), lineGradient: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_ratio: new ee.aJ(Fe, Te.u_ratio), u_device_pixel_ratio: new ee.aJ(Fe, Te.u_device_pixel_ratio), u_units_to_pixels: new ee.aP(Fe, Te.u_units_to_pixels), u_image: new ee.aI(Fe, Te.u_image), u_image_height: new ee.aJ(Fe, Te.u_image_height) }), linePattern: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_texsize: new ee.aP(Fe, Te.u_texsize), u_ratio: new ee.aJ(Fe, Te.u_ratio), u_device_pixel_ratio: new ee.aJ(Fe, Te.u_device_pixel_ratio), u_image: new ee.aI(Fe, Te.u_image), u_units_to_pixels: new ee.aP(Fe, Te.u_units_to_pixels), u_scale: new ee.aO(Fe, Te.u_scale), u_fade: new ee.aJ(Fe, Te.u_fade) }), lineSDF: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_ratio: new ee.aJ(Fe, Te.u_ratio), u_device_pixel_ratio: new ee.aJ(Fe, Te.u_device_pixel_ratio), u_units_to_pixels: new ee.aP(Fe, Te.u_units_to_pixels), u_patternscale_a: new ee.aP(Fe, Te.u_patternscale_a), u_patternscale_b: new ee.aP(Fe, Te.u_patternscale_b), u_sdfgamma: new ee.aJ(Fe, Te.u_sdfgamma), u_image: new ee.aI(Fe, Te.u_image), u_tex_y_a: new ee.aJ(Fe, Te.u_tex_y_a), u_tex_y_b: new ee.aJ(Fe, Te.u_tex_y_b), u_mix: new ee.aJ(Fe, Te.u_mix) }), raster: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_tl_parent: new ee.aP(Fe, Te.u_tl_parent), u_scale_parent: new ee.aJ(Fe, Te.u_scale_parent), u_buffer_scale: new ee.aJ(Fe, Te.u_buffer_scale), u_fade_t: new ee.aJ(Fe, Te.u_fade_t), u_opacity: new ee.aJ(Fe, Te.u_opacity), u_image0: new ee.aI(Fe, Te.u_image0), u_image1: new ee.aI(Fe, Te.u_image1), u_brightness_low: new ee.aJ(Fe, Te.u_brightness_low), u_brightness_high: new ee.aJ(Fe, Te.u_brightness_high), u_saturation_factor: new ee.aJ(Fe, Te.u_saturation_factor), u_contrast_factor: new ee.aJ(Fe, Te.u_contrast_factor), u_spin_weights: new ee.aO(Fe, Te.u_spin_weights) }), symbolIcon: (Fe, Te) => ({ u_is_size_zoom_constant: new ee.aI(Fe, Te.u_is_size_zoom_constant), u_is_size_feature_constant: new ee.aI(Fe, Te.u_is_size_feature_constant), u_size_t: new ee.aJ(Fe, Te.u_size_t), u_size: new ee.aJ(Fe, Te.u_size), u_camera_to_center_distance: new ee.aJ(Fe, Te.u_camera_to_center_distance), u_pitch: new ee.aJ(Fe, Te.u_pitch), u_rotate_symbol: new ee.aI(Fe, Te.u_rotate_symbol), u_aspect_ratio: new ee.aJ(Fe, Te.u_aspect_ratio), u_fade_change: new ee.aJ(Fe, Te.u_fade_change), u_matrix: new ee.aK(Fe, Te.u_matrix), u_label_plane_matrix: new ee.aK(Fe, Te.u_label_plane_matrix), u_coord_matrix: new ee.aK(Fe, Te.u_coord_matrix), u_is_text: new ee.aI(Fe, Te.u_is_text), u_pitch_with_map: new ee.aI(Fe, Te.u_pitch_with_map), u_is_along_line: new ee.aI(Fe, Te.u_is_along_line), u_is_variable_anchor: new ee.aI(Fe, Te.u_is_variable_anchor), u_texsize: new ee.aP(Fe, Te.u_texsize), u_texture: new ee.aI(Fe, Te.u_texture), u_translation: new ee.aP(Fe, Te.u_translation), u_pitched_scale: new ee.aJ(Fe, Te.u_pitched_scale) }), symbolSDF: (Fe, Te) => ({ u_is_size_zoom_constant: new ee.aI(Fe, Te.u_is_size_zoom_constant), u_is_size_feature_constant: new ee.aI(Fe, Te.u_is_size_feature_constant), u_size_t: new ee.aJ(Fe, Te.u_size_t), u_size: new ee.aJ(Fe, Te.u_size), u_camera_to_center_distance: new ee.aJ(Fe, Te.u_camera_to_center_distance), u_pitch: new ee.aJ(Fe, Te.u_pitch), u_rotate_symbol: new ee.aI(Fe, Te.u_rotate_symbol), u_aspect_ratio: new ee.aJ(Fe, Te.u_aspect_ratio), u_fade_change: new ee.aJ(Fe, Te.u_fade_change), u_matrix: new ee.aK(Fe, Te.u_matrix), u_label_plane_matrix: new ee.aK(Fe, Te.u_label_plane_matrix), u_coord_matrix: new ee.aK(Fe, Te.u_coord_matrix), u_is_text: new ee.aI(Fe, Te.u_is_text), u_pitch_with_map: new ee.aI(Fe, Te.u_pitch_with_map), u_is_along_line: new ee.aI(Fe, Te.u_is_along_line), u_is_variable_anchor: new ee.aI(Fe, Te.u_is_variable_anchor), u_texsize: new ee.aP(Fe, Te.u_texsize), u_texture: new ee.aI(Fe, Te.u_texture), u_gamma_scale: new ee.aJ(Fe, Te.u_gamma_scale), u_device_pixel_ratio: new ee.aJ(Fe, Te.u_device_pixel_ratio), u_is_halo: new ee.aI(Fe, Te.u_is_halo), u_translation: new ee.aP(Fe, Te.u_translation), u_pitched_scale: new ee.aJ(Fe, Te.u_pitched_scale) }), symbolTextAndIcon: (Fe, Te) => ({ u_is_size_zoom_constant: new ee.aI(Fe, Te.u_is_size_zoom_constant), u_is_size_feature_constant: new ee.aI(Fe, Te.u_is_size_feature_constant), u_size_t: new ee.aJ(Fe, Te.u_size_t), u_size: new ee.aJ(Fe, Te.u_size), u_camera_to_center_distance: new ee.aJ(Fe, Te.u_camera_to_center_distance), u_pitch: new ee.aJ(Fe, Te.u_pitch), u_rotate_symbol: new ee.aI(Fe, Te.u_rotate_symbol), u_aspect_ratio: new ee.aJ(Fe, Te.u_aspect_ratio), u_fade_change: new ee.aJ(Fe, Te.u_fade_change), u_matrix: new ee.aK(Fe, Te.u_matrix), u_label_plane_matrix: new ee.aK(Fe, Te.u_label_plane_matrix), u_coord_matrix: new ee.aK(Fe, Te.u_coord_matrix), u_is_text: new ee.aI(Fe, Te.u_is_text), u_pitch_with_map: new ee.aI(Fe, Te.u_pitch_with_map), u_is_along_line: new ee.aI(Fe, Te.u_is_along_line), u_is_variable_anchor: new ee.aI(Fe, Te.u_is_variable_anchor), u_texsize: new ee.aP(Fe, Te.u_texsize), u_texsize_icon: new ee.aP(Fe, Te.u_texsize_icon), u_texture: new ee.aI(Fe, Te.u_texture), u_texture_icon: new ee.aI(Fe, Te.u_texture_icon), u_gamma_scale: new ee.aJ(Fe, Te.u_gamma_scale), u_device_pixel_ratio: new ee.aJ(Fe, Te.u_device_pixel_ratio), u_is_halo: new ee.aI(Fe, Te.u_is_halo), u_translation: new ee.aP(Fe, Te.u_translation), u_pitched_scale: new ee.aJ(Fe, Te.u_pitched_scale) }), background: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_opacity: new ee.aJ(Fe, Te.u_opacity), u_color: new ee.aM(Fe, Te.u_color) }), backgroundPattern: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_opacity: new ee.aJ(Fe, Te.u_opacity), u_image: new ee.aI(Fe, Te.u_image), u_pattern_tl_a: new ee.aP(Fe, Te.u_pattern_tl_a), u_pattern_br_a: new ee.aP(Fe, Te.u_pattern_br_a), u_pattern_tl_b: new ee.aP(Fe, Te.u_pattern_tl_b), u_pattern_br_b: new ee.aP(Fe, Te.u_pattern_br_b), u_texsize: new ee.aP(Fe, Te.u_texsize), u_mix: new ee.aJ(Fe, Te.u_mix), u_pattern_size_a: new ee.aP(Fe, Te.u_pattern_size_a), u_pattern_size_b: new ee.aP(Fe, Te.u_pattern_size_b), u_scale_a: new ee.aJ(Fe, Te.u_scale_a), u_scale_b: new ee.aJ(Fe, Te.u_scale_b), u_pixel_coord_upper: new ee.aP(Fe, Te.u_pixel_coord_upper), u_pixel_coord_lower: new ee.aP(Fe, Te.u_pixel_coord_lower), u_tile_units_to_pixels: new ee.aJ(Fe, Te.u_tile_units_to_pixels) }), terrain: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_texture: new ee.aI(Fe, Te.u_texture), u_ele_delta: new ee.aJ(Fe, Te.u_ele_delta), u_fog_matrix: new ee.aK(Fe, Te.u_fog_matrix), u_fog_color: new ee.aM(Fe, Te.u_fog_color), u_fog_ground_blend: new ee.aJ(Fe, Te.u_fog_ground_blend), u_fog_ground_blend_opacity: new ee.aJ(Fe, Te.u_fog_ground_blend_opacity), u_horizon_color: new ee.aM(Fe, Te.u_horizon_color), u_horizon_fog_blend: new ee.aJ(Fe, Te.u_horizon_fog_blend) }), terrainDepth: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_ele_delta: new ee.aJ(Fe, Te.u_ele_delta) }), terrainCoords: (Fe, Te) => ({ u_matrix: new ee.aK(Fe, Te.u_matrix), u_texture: new ee.aI(Fe, Te.u_texture), u_terrain_coords_id: new ee.aJ(Fe, Te.u_terrain_coords_id), u_ele_delta: new ee.aJ(Fe, Te.u_ele_delta) }), sky: (Fe, Te) => ({ u_sky_color: new ee.aM(Fe, Te.u_sky_color), u_horizon_color: new ee.aM(Fe, Te.u_horizon_color), u_horizon: new ee.aJ(Fe, Te.u_horizon), u_sky_horizon_blend: new ee.aJ(Fe, Te.u_sky_horizon_blend) }) };
                class Fi {
                  constructor(Te, Ae, Oe) {
                    this.context = Te;
                    const Be = Te.gl;
                    this.buffer = Be.createBuffer(), this.dynamicDraw = !!Oe, this.context.unbindVAO(), Te.bindElementBuffer.set(this.buffer), Be.bufferData(Be.ELEMENT_ARRAY_BUFFER, Ae.arrayBuffer, this.dynamicDraw ? Be.DYNAMIC_DRAW : Be.STATIC_DRAW), this.dynamicDraw || delete Ae.arrayBuffer;
                  }
                  bind() {
                    this.context.bindElementBuffer.set(this.buffer);
                  }
                  updateData(Te) {
                    const Ae = this.context.gl;
                    if (!this.dynamicDraw) throw new Error("Attempted to update data while not in dynamic mode.");
                    this.context.unbindVAO(), this.bind(), Ae.bufferSubData(Ae.ELEMENT_ARRAY_BUFFER, 0, Te.arrayBuffer);
                  }
                  destroy() {
                    const Te = this.context.gl;
                    this.buffer && (Te.deleteBuffer(this.buffer), delete this.buffer);
                  }
                }
                const na = { Int8: "BYTE", Uint8: "UNSIGNED_BYTE", Int16: "SHORT", Uint16: "UNSIGNED_SHORT", Int32: "INT", Uint32: "UNSIGNED_INT", Float32: "FLOAT" };
                class nn {
                  constructor(Te, Ae, Oe, Be) {
                    this.length = Ae.length, this.attributes = Oe, this.itemSize = Ae.bytesPerElement, this.dynamicDraw = Be, this.context = Te;
                    const Ue = Te.gl;
                    this.buffer = Ue.createBuffer(), Te.bindVertexBuffer.set(this.buffer), Ue.bufferData(Ue.ARRAY_BUFFER, Ae.arrayBuffer, this.dynamicDraw ? Ue.DYNAMIC_DRAW : Ue.STATIC_DRAW), this.dynamicDraw || delete Ae.arrayBuffer;
                  }
                  bind() {
                    this.context.bindVertexBuffer.set(this.buffer);
                  }
                  updateData(Te) {
                    if (Te.length !== this.length) throw new Error(`Length of new data is ${Te.length}, which doesn't match current length of ${this.length}`);
                    const Ae = this.context.gl;
                    this.bind(), Ae.bufferSubData(Ae.ARRAY_BUFFER, 0, Te.arrayBuffer);
                  }
                  enableAttributes(Te, Ae) {
                    for (let Oe = 0; Oe < this.attributes.length; Oe++) {
                      const Be = this.attributes[Oe], Ue = Ae.attributes[Be.name];
                      Ue !== void 0 && Te.enableVertexAttribArray(Ue);
                    }
                  }
                  setVertexAttribPointers(Te, Ae, Oe) {
                    for (let Be = 0; Be < this.attributes.length; Be++) {
                      const Ue = this.attributes[Be], nt = Ae.attributes[Ue.name];
                      nt !== void 0 && Te.vertexAttribPointer(nt, Ue.components, Te[na[Ue.type]], !1, this.itemSize, Ue.offset + this.itemSize * (Oe || 0));
                    }
                  }
                  destroy() {
                    const Te = this.context.gl;
                    this.buffer && (Te.deleteBuffer(this.buffer), delete this.buffer);
                  }
                }
                const Tn = /* @__PURE__ */ new WeakMap();
                function Dn(Fe) {
                  var Te;
                  if (Tn.has(Fe)) return Tn.get(Fe);
                  {
                    const Ae = (Te = Fe.getParameter(Fe.VERSION)) === null || Te === void 0 ? void 0 : Te.startsWith("WebGL 2.0");
                    return Tn.set(Fe, Ae), Ae;
                  }
                }
                class Fn {
                  constructor(Te) {
                    this.gl = Te.gl, this.default = this.getDefault(), this.current = this.default, this.dirty = !1;
                  }
                  get() {
                    return this.current;
                  }
                  set(Te) {
                  }
                  getDefault() {
                    return this.default;
                  }
                  setDefault() {
                    this.set(this.default);
                  }
                }
                class fr extends Fn {
                  getDefault() {
                    return ee.aN.transparent;
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te.r !== Ae.r || Te.g !== Ae.g || Te.b !== Ae.b || Te.a !== Ae.a || this.dirty) && (this.gl.clearColor(Te.r, Te.g, Te.b, Te.a), this.current = Te, this.dirty = !1);
                  }
                }
                class dr extends Fn {
                  getDefault() {
                    return 1;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.clearDepth(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class yr extends Fn {
                  getDefault() {
                    return 0;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.clearStencil(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class ai extends Fn {
                  getDefault() {
                    return [!0, !0, !0, !0];
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || Te[2] !== Ae[2] || Te[3] !== Ae[3] || this.dirty) && (this.gl.colorMask(Te[0], Te[1], Te[2], Te[3]), this.current = Te, this.dirty = !1);
                  }
                }
                class Fr extends Fn {
                  getDefault() {
                    return !0;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.depthMask(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class Ti extends Fn {
                  getDefault() {
                    return 255;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.stencilMask(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class Si extends Fn {
                  getDefault() {
                    return { func: this.gl.ALWAYS, ref: 0, mask: 255 };
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te.func !== Ae.func || Te.ref !== Ae.ref || Te.mask !== Ae.mask || this.dirty) && (this.gl.stencilFunc(Te.func, Te.ref, Te.mask), this.current = Te, this.dirty = !1);
                  }
                }
                class Ei extends Fn {
                  getDefault() {
                    const Te = this.gl;
                    return [Te.KEEP, Te.KEEP, Te.KEEP];
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || Te[2] !== Ae[2] || this.dirty) && (this.gl.stencilOp(Te[0], Te[1], Te[2]), this.current = Te, this.dirty = !1);
                  }
                }
                class Fa extends Fn {
                  getDefault() {
                    return !1;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Te ? Ae.enable(Ae.STENCIL_TEST) : Ae.disable(Ae.STENCIL_TEST), this.current = Te, this.dirty = !1;
                  }
                }
                class $o extends Fn {
                  getDefault() {
                    return [0, 1];
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || this.dirty) && (this.gl.depthRange(Te[0], Te[1]), this.current = Te, this.dirty = !1);
                  }
                }
                class Gs extends Fn {
                  getDefault() {
                    return !1;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Te ? Ae.enable(Ae.DEPTH_TEST) : Ae.disable(Ae.DEPTH_TEST), this.current = Te, this.dirty = !1;
                  }
                }
                class Al extends Fn {
                  getDefault() {
                    return this.gl.LESS;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.depthFunc(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class ul extends Fn {
                  getDefault() {
                    return !1;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Te ? Ae.enable(Ae.BLEND) : Ae.disable(Ae.BLEND), this.current = Te, this.dirty = !1;
                  }
                }
                class gc extends Fn {
                  getDefault() {
                    const Te = this.gl;
                    return [Te.ONE, Te.ZERO];
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || this.dirty) && (this.gl.blendFunc(Te[0], Te[1]), this.current = Te, this.dirty = !1);
                  }
                }
                class Gl extends Fn {
                  getDefault() {
                    return ee.aN.transparent;
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te.r !== Ae.r || Te.g !== Ae.g || Te.b !== Ae.b || Te.a !== Ae.a || this.dirty) && (this.gl.blendColor(Te.r, Te.g, Te.b, Te.a), this.current = Te, this.dirty = !1);
                  }
                }
                class Sl extends Fn {
                  getDefault() {
                    return this.gl.FUNC_ADD;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.blendEquation(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class ls extends Fn {
                  getDefault() {
                    return !1;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Te ? Ae.enable(Ae.CULL_FACE) : Ae.disable(Ae.CULL_FACE), this.current = Te, this.dirty = !1;
                  }
                }
                class vs extends Fn {
                  getDefault() {
                    return this.gl.BACK;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.cullFace(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class Vl extends Fn {
                  getDefault() {
                    return this.gl.CCW;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.frontFace(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class Ll extends Fn {
                  getDefault() {
                    return null;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.useProgram(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class Xl extends Fn {
                  getDefault() {
                    return this.gl.TEXTURE0;
                  }
                  set(Te) {
                    (Te !== this.current || this.dirty) && (this.gl.activeTexture(Te), this.current = Te, this.dirty = !1);
                  }
                }
                class yc extends Fn {
                  getDefault() {
                    const Te = this.gl;
                    return [0, 0, Te.drawingBufferWidth, Te.drawingBufferHeight];
                  }
                  set(Te) {
                    const Ae = this.current;
                    (Te[0] !== Ae[0] || Te[1] !== Ae[1] || Te[2] !== Ae[2] || Te[3] !== Ae[3] || this.dirty) && (this.gl.viewport(Te[0], Te[1], Te[2], Te[3]), this.current = Te, this.dirty = !1);
                  }
                }
                class gu extends Fn {
                  getDefault() {
                    return null;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Ae.bindFramebuffer(Ae.FRAMEBUFFER, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class Nc extends Fn {
                  getDefault() {
                    return null;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Ae.bindRenderbuffer(Ae.RENDERBUFFER, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class Cu extends Fn {
                  getDefault() {
                    return null;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Ae.bindTexture(Ae.TEXTURE_2D, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class zc extends Fn {
                  getDefault() {
                    return null;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Ae.bindBuffer(Ae.ARRAY_BUFFER, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class Kc extends Fn {
                  getDefault() {
                    return null;
                  }
                  set(Te) {
                    const Ae = this.gl;
                    Ae.bindBuffer(Ae.ELEMENT_ARRAY_BUFFER, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class zi extends Fn {
                  getDefault() {
                    return null;
                  }
                  set(Te) {
                    var Ae;
                    if (Te === this.current && !this.dirty) return;
                    const Oe = this.gl;
                    Dn(Oe) ? Oe.bindVertexArray(Te) : (Ae = Oe.getExtension("OES_vertex_array_object")) === null || Ae === void 0 || Ae.bindVertexArrayOES(Te), this.current = Te, this.dirty = !1;
                  }
                }
                class Fu extends Fn {
                  getDefault() {
                    return 4;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Ae.pixelStorei(Ae.UNPACK_ALIGNMENT, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class d1 extends Fn {
                  getDefault() {
                    return !1;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Ae.pixelStorei(Ae.UNPACK_PREMULTIPLY_ALPHA_WEBGL, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class Gu extends Fn {
                  getDefault() {
                    return !1;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    const Ae = this.gl;
                    Ae.pixelStorei(Ae.UNPACK_FLIP_Y_WEBGL, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class Ku extends Fn {
                  constructor(Te, Ae) {
                    super(Te), this.context = Te, this.parent = Ae;
                  }
                  getDefault() {
                    return null;
                  }
                }
                class I1 extends Ku {
                  setDirty() {
                    this.dirty = !0;
                  }
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    this.context.bindFramebuffer.set(this.parent);
                    const Ae = this.gl;
                    Ae.framebufferTexture2D(Ae.FRAMEBUFFER, Ae.COLOR_ATTACHMENT0, Ae.TEXTURE_2D, Te, 0), this.current = Te, this.dirty = !1;
                  }
                }
                class f1 extends Ku {
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    this.context.bindFramebuffer.set(this.parent);
                    const Ae = this.gl;
                    Ae.framebufferRenderbuffer(Ae.FRAMEBUFFER, Ae.DEPTH_ATTACHMENT, Ae.RENDERBUFFER, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class D1 extends Ku {
                  set(Te) {
                    if (Te === this.current && !this.dirty) return;
                    this.context.bindFramebuffer.set(this.parent);
                    const Ae = this.gl;
                    Ae.framebufferRenderbuffer(Ae.FRAMEBUFFER, Ae.DEPTH_STENCIL_ATTACHMENT, Ae.RENDERBUFFER, Te), this.current = Te, this.dirty = !1;
                  }
                }
                class Hl {
                  constructor(Te, Ae, Oe, Be, Ue) {
                    this.context = Te, this.width = Ae, this.height = Oe;
                    const nt = Te.gl, ht = this.framebuffer = nt.createFramebuffer();
                    if (this.colorAttachment = new I1(Te, ht), Be) this.depthAttachment = Ue ? new D1(Te, ht) : new f1(Te, ht);
                    else if (Ue) throw new Error("Stencil cannot be set without depth");
                    if (nt.checkFramebufferStatus(nt.FRAMEBUFFER) !== nt.FRAMEBUFFER_COMPLETE) throw new Error("Framebuffer is not complete");
                  }
                  destroy() {
                    const Te = this.context.gl, Ae = this.colorAttachment.get();
                    if (Ae && Te.deleteTexture(Ae), this.depthAttachment) {
                      const Oe = this.depthAttachment.get();
                      Oe && Te.deleteRenderbuffer(Oe);
                    }
                    Te.deleteFramebuffer(this.framebuffer);
                  }
                }
                class Kl {
                  constructor(Te, Ae, Oe) {
                    this.blendFunction = Te, this.blendColor = Ae, this.mask = Oe;
                  }
                }
                Kl.Replace = [1, 0], Kl.disabled = new Kl(Kl.Replace, ee.aN.transparent, [!1, !1, !1, !1]), Kl.unblended = new Kl(Kl.Replace, ee.aN.transparent, [!0, !0, !0, !0]), Kl.alphaBlended = new Kl([1, 771], ee.aN.transparent, [!0, !0, !0, !0]);
                class uu {
                  constructor(Te) {
                    var Ae, Oe;
                    if (this.gl = Te, this.clearColor = new fr(this), this.clearDepth = new dr(this), this.clearStencil = new yr(this), this.colorMask = new ai(this), this.depthMask = new Fr(this), this.stencilMask = new Ti(this), this.stencilFunc = new Si(this), this.stencilOp = new Ei(this), this.stencilTest = new Fa(this), this.depthRange = new $o(this), this.depthTest = new Gs(this), this.depthFunc = new Al(this), this.blend = new ul(this), this.blendFunc = new gc(this), this.blendColor = new Gl(this), this.blendEquation = new Sl(this), this.cullFace = new ls(this), this.cullFaceSide = new vs(this), this.frontFace = new Vl(this), this.program = new Ll(this), this.activeTexture = new Xl(this), this.viewport = new yc(this), this.bindFramebuffer = new gu(this), this.bindRenderbuffer = new Nc(this), this.bindTexture = new Cu(this), this.bindVertexBuffer = new zc(this), this.bindElementBuffer = new Kc(this), this.bindVertexArray = new zi(this), this.pixelStoreUnpack = new Fu(this), this.pixelStoreUnpackPremultiplyAlpha = new d1(this), this.pixelStoreUnpackFlipY = new Gu(this), this.extTextureFilterAnisotropic = Te.getExtension("EXT_texture_filter_anisotropic") || Te.getExtension("MOZ_EXT_texture_filter_anisotropic") || Te.getExtension("WEBKIT_EXT_texture_filter_anisotropic"), this.extTextureFilterAnisotropic && (this.extTextureFilterAnisotropicMax = Te.getParameter(this.extTextureFilterAnisotropic.MAX_TEXTURE_MAX_ANISOTROPY_EXT)), this.maxTextureSize = Te.getParameter(Te.MAX_TEXTURE_SIZE), Dn(Te)) {
                      this.HALF_FLOAT = Te.HALF_FLOAT;
                      const Be = Te.getExtension("EXT_color_buffer_half_float");
                      this.RGBA16F = (Ae = Te.RGBA16F) !== null && Ae !== void 0 ? Ae : Be == null ? void 0 : Be.RGBA16F_EXT, this.RGB16F = (Oe = Te.RGB16F) !== null && Oe !== void 0 ? Oe : Be == null ? void 0 : Be.RGB16F_EXT, Te.getExtension("EXT_color_buffer_float");
                    } else {
                      Te.getExtension("EXT_color_buffer_half_float"), Te.getExtension("OES_texture_half_float_linear");
                      const Be = Te.getExtension("OES_texture_half_float");
                      this.HALF_FLOAT = Be == null ? void 0 : Be.HALF_FLOAT_OES;
                    }
                  }
                  setDefault() {
                    this.unbindVAO(), this.clearColor.setDefault(), this.clearDepth.setDefault(), this.clearStencil.setDefault(), this.colorMask.setDefault(), this.depthMask.setDefault(), this.stencilMask.setDefault(), this.stencilFunc.setDefault(), this.stencilOp.setDefault(), this.stencilTest.setDefault(), this.depthRange.setDefault(), this.depthTest.setDefault(), this.depthFunc.setDefault(), this.blend.setDefault(), this.blendFunc.setDefault(), this.blendColor.setDefault(), this.blendEquation.setDefault(), this.cullFace.setDefault(), this.cullFaceSide.setDefault(), this.frontFace.setDefault(), this.program.setDefault(), this.activeTexture.setDefault(), this.bindFramebuffer.setDefault(), this.pixelStoreUnpack.setDefault(), this.pixelStoreUnpackPremultiplyAlpha.setDefault(), this.pixelStoreUnpackFlipY.setDefault();
                  }
                  setDirty() {
                    this.clearColor.dirty = !0, this.clearDepth.dirty = !0, this.clearStencil.dirty = !0, this.colorMask.dirty = !0, this.depthMask.dirty = !0, this.stencilMask.dirty = !0, this.stencilFunc.dirty = !0, this.stencilOp.dirty = !0, this.stencilTest.dirty = !0, this.depthRange.dirty = !0, this.depthTest.dirty = !0, this.depthFunc.dirty = !0, this.blend.dirty = !0, this.blendFunc.dirty = !0, this.blendColor.dirty = !0, this.blendEquation.dirty = !0, this.cullFace.dirty = !0, this.cullFaceSide.dirty = !0, this.frontFace.dirty = !0, this.program.dirty = !0, this.activeTexture.dirty = !0, this.viewport.dirty = !0, this.bindFramebuffer.dirty = !0, this.bindRenderbuffer.dirty = !0, this.bindTexture.dirty = !0, this.bindVertexBuffer.dirty = !0, this.bindElementBuffer.dirty = !0, this.bindVertexArray.dirty = !0, this.pixelStoreUnpack.dirty = !0, this.pixelStoreUnpackPremultiplyAlpha.dirty = !0, this.pixelStoreUnpackFlipY.dirty = !0;
                  }
                  createIndexBuffer(Te, Ae) {
                    return new Fi(this, Te, Ae);
                  }
                  createVertexBuffer(Te, Ae, Oe) {
                    return new nn(this, Te, Ae, Oe);
                  }
                  createRenderbuffer(Te, Ae, Oe) {
                    const Be = this.gl, Ue = Be.createRenderbuffer();
                    return this.bindRenderbuffer.set(Ue), Be.renderbufferStorage(Be.RENDERBUFFER, Te, Ae, Oe), this.bindRenderbuffer.set(null), Ue;
                  }
                  createFramebuffer(Te, Ae, Oe, Be) {
                    return new Hl(this, Te, Ae, Oe, Be);
                  }
                  clear({ color: Te, depth: Ae, stencil: Oe }) {
                    const Be = this.gl;
                    let Ue = 0;
                    Te && (Ue |= Be.COLOR_BUFFER_BIT, this.clearColor.set(Te), this.colorMask.set([!0, !0, !0, !0])), Ae !== void 0 && (Ue |= Be.DEPTH_BUFFER_BIT, this.depthRange.set([0, 1]), this.clearDepth.set(Ae), this.depthMask.set(!0)), Oe !== void 0 && (Ue |= Be.STENCIL_BUFFER_BIT, this.clearStencil.set(Oe), this.stencilMask.set(255)), Be.clear(Ue);
                  }
                  setCullFace(Te) {
                    Te.enable === !1 ? this.cullFace.set(!1) : (this.cullFace.set(!0), this.cullFaceSide.set(Te.mode), this.frontFace.set(Te.frontFace));
                  }
                  setDepthMode(Te) {
                    Te.func !== this.gl.ALWAYS || Te.mask ? (this.depthTest.set(!0), this.depthFunc.set(Te.func), this.depthMask.set(Te.mask), this.depthRange.set(Te.range)) : this.depthTest.set(!1);
                  }
                  setStencilMode(Te) {
                    Te.test.func !== this.gl.ALWAYS || Te.mask ? (this.stencilTest.set(!0), this.stencilMask.set(Te.mask), this.stencilOp.set([Te.fail, Te.depthFail, Te.pass]), this.stencilFunc.set({ func: Te.test.func, ref: Te.ref, mask: Te.test.mask })) : this.stencilTest.set(!1);
                  }
                  setColorMode(Te) {
                    ee.aF(Te.blendFunction, Kl.Replace) ? this.blend.set(!1) : (this.blend.set(!0), this.blendFunc.set(Te.blendFunction), this.blendColor.set(Te.blendColor)), this.colorMask.set(Te.mask);
                  }
                  createVertexArray() {
                    var Te;
                    return Dn(this.gl) ? this.gl.createVertexArray() : (Te = this.gl.getExtension("OES_vertex_array_object")) === null || Te === void 0 ? void 0 : Te.createVertexArrayOES();
                  }
                  deleteVertexArray(Te) {
                    var Ae;
                    return Dn(this.gl) ? this.gl.deleteVertexArray(Te) : (Ae = this.gl.getExtension("OES_vertex_array_object")) === null || Ae === void 0 ? void 0 : Ae.deleteVertexArrayOES(Te);
                  }
                  unbindVAO() {
                    this.bindVertexArray.set(null);
                  }
                }
                class Hs {
                  constructor(Te, Ae, Oe) {
                    this.func = Te, this.mask = Ae, this.range = Oe;
                  }
                }
                Hs.ReadOnly = !1, Hs.ReadWrite = !0, Hs.disabled = new Hs(519, Hs.ReadOnly, [0, 1]);
                const b1 = 7680;
                class Wl {
                  constructor(Te, Ae, Oe, Be, Ue, nt) {
                    this.test = Te, this.ref = Ae, this.mask = Oe, this.fail = Be, this.depthFail = Ue, this.pass = nt;
                  }
                }
                Wl.disabled = new Wl({ func: 519, mask: 0 }, 0, 0, b1, b1, b1);
                class zl {
                  constructor(Te, Ae, Oe) {
                    this.enable = Te, this.mode = Ae, this.frontFace = Oe;
                  }
                }
                let os;
                function jo(Fe, Te, Ae, Oe, Be) {
                  const Ue = Fe.context, nt = Ue.gl, ht = Fe.useProgram("collisionBox"), Et = [];
                  let Mt = 0, qt = 0;
                  for (let qn = 0; qn < Oe.length; qn++) {
                    const Vn = Oe[qn], sr = Te.getTile(Vn).getBucket(Ae);
                    if (!sr) continue;
                    const Un = Be ? sr.textCollisionBox : sr.iconCollisionBox, wr = sr.collisionCircleArray;
                    if (wr.length > 0) {
                      const Cr = ee.H();
                      ee.aR(Cr, sr.placementInvProjMatrix, Fe.transform.glCoordMatrix), ee.aR(Cr, Cr, sr.placementViewportMatrix), Et.push({ circleArray: wr, circleOffset: qt, transform: Vn.posMatrix, invTransform: Cr, coord: Vn }), Mt += wr.length / 4, qt = Mt;
                    }
                    Un && ht.draw(Ue, nt.LINES, Hs.disabled, Wl.disabled, Fe.colorModeForRenderPass(), zl.disabled, rr(Fe.transform, Vn.posMatrix), Fe.style.map.terrain && Fe.style.map.terrain.getTerrainData(Vn), Ae.id, Un.layoutVertexBuffer, Un.indexBuffer, Un.segments, null, Fe.transform.zoom, null, null, Un.collisionVertexBuffer);
                  }
                  if (!Be || !Et.length) return;
                  const tn = Fe.useProgram("collisionCircle"), xn = new ee.aS();
                  xn.resize(4 * Mt), xn._trim();
                  let An = 0;
                  for (const qn of Et) for (let Vn = 0; Vn < qn.circleArray.length / 4; Vn++) {
                    const sr = 4 * Vn, Un = qn.circleArray[sr + 0], wr = qn.circleArray[sr + 1], Cr = qn.circleArray[sr + 2], Tr = qn.circleArray[sr + 3];
                    xn.emplace(An++, Un, wr, Cr, Tr, 0), xn.emplace(An++, Un, wr, Cr, Tr, 1), xn.emplace(An++, Un, wr, Cr, Tr, 2), xn.emplace(An++, Un, wr, Cr, Tr, 3);
                  }
                  (!os || os.length < 2 * Mt) && (os = function(qn) {
                    const Vn = 2 * qn, sr = new ee.aU();
                    sr.resize(Vn), sr._trim();
                    for (let Un = 0; Un < Vn; Un++) {
                      const wr = 6 * Un;
                      sr.uint16[wr + 0] = 4 * Un + 0, sr.uint16[wr + 1] = 4 * Un + 1, sr.uint16[wr + 2] = 4 * Un + 2, sr.uint16[wr + 3] = 4 * Un + 2, sr.uint16[wr + 4] = 4 * Un + 3, sr.uint16[wr + 5] = 4 * Un + 0;
                    }
                    return sr;
                  }(Mt));
                  const On = Ue.createIndexBuffer(os, !0), Bn = Ue.createVertexBuffer(xn, ee.aT.members, !0);
                  for (const qn of Et) {
                    const Vn = vr(qn.transform, qn.invTransform, Fe.transform);
                    tn.draw(Ue, nt.TRIANGLES, Hs.disabled, Wl.disabled, Fe.colorModeForRenderPass(), zl.disabled, Vn, Fe.style.map.terrain && Fe.style.map.terrain.getTerrainData(qn.coord), Ae.id, Bn, On, ee.a0.simpleSegment(0, 2 * qn.circleOffset, qn.circleArray.length, qn.circleArray.length / 2), null, Fe.transform.zoom, null, null, null);
                  }
                  Bn.destroy(), On.destroy();
                }
                zl.disabled = new zl(!1, 1029, 2305), zl.backCCW = new zl(!0, 1029, 2305);
                const uo = ee.ao(new Float32Array(16));
                function so(Fe, Te, Ae, Oe, Be) {
                  if (Fe.renderPass !== "translucent") return;
                  const Ue = Wl.disabled, nt = Fe.colorModeForRenderPass();
                  (Ae._unevaluatedLayout.hasValue("text-variable-anchor") || Ae._unevaluatedLayout.hasValue("text-variable-anchor-offset")) && function(ht, Et, Mt, qt, tn, xn, An, On, Bn) {
                    const qn = Et.transform, Vn = fi(), sr = tn === "map", Un = xn === "map";
                    for (const wr of ht) {
                      const Cr = qt.getTile(wr), Tr = Cr.getBucket(Mt);
                      if (!Tr || !Tr.text || !Tr.text.segments.get().length) continue;
                      const ii = Tr.textSizeData, wi = ee.ah(ii, qn.zoom), Wi = Mn(Cr, 1, Et.transform.zoom), Yi = wn(wr.posMatrix, Un, sr, Et.transform, Wi), ra = Mt.layout.get("icon-text-fit") !== "none" && Tr.hasIconData();
                      if (wi) {
                        const ha = Math.pow(2, qn.zoom - Cr.tileID.overscaledZ), ya = Et.style.map.terrain ? (qa, Aa) => Et.style.map.terrain.getElevation(wr, qa, Aa) : null, va = Vn.translatePosition(qn, Cr, An, On);
                        No(Tr, sr, Un, Bn, qn, Yi, wr.posMatrix, ha, wi, ra, Vn, va, wr.toUnwrapped(), ya);
                      }
                    }
                  }(Oe, Fe, Ae, Te, Ae.layout.get("text-rotation-alignment"), Ae.layout.get("text-pitch-alignment"), Ae.paint.get("text-translate"), Ae.paint.get("text-translate-anchor"), Be), Ae.paint.get("icon-opacity").constantOr(1) !== 0 && Io(Fe, Te, Ae, Oe, !1, Ae.paint.get("icon-translate"), Ae.paint.get("icon-translate-anchor"), Ae.layout.get("icon-rotation-alignment"), Ae.layout.get("icon-pitch-alignment"), Ae.layout.get("icon-keep-upright"), Ue, nt), Ae.paint.get("text-opacity").constantOr(1) !== 0 && Io(Fe, Te, Ae, Oe, !0, Ae.paint.get("text-translate"), Ae.paint.get("text-translate-anchor"), Ae.layout.get("text-rotation-alignment"), Ae.layout.get("text-pitch-alignment"), Ae.layout.get("text-keep-upright"), Ue, nt), Te.map.showCollisionBoxes && (jo(Fe, Te, Ae, Oe, !0), jo(Fe, Te, Ae, Oe, !1));
                }
                function So(Fe, Te, Ae, Oe, Be, Ue) {
                  const { horizontalAlign: nt, verticalAlign: ht } = ee.av(Fe), Et = -(nt - 0.5) * Te, Mt = -(ht - 0.5) * Ae;
                  return new ee.P((Et / Be + Oe[0]) * Ue, (Mt / Be + Oe[1]) * Ue);
                }
                function Jc(Fe, Te, Ae, Oe, Be, Ue) {
                  const nt = Te.tileAnchorPoint.add(new ee.P(Te.translation[0], Te.translation[1]));
                  if (Te.pitchWithMap) {
                    let ht = Oe.mult(Ue);
                    return Ae || (ht = ht.rotate(-Be)), Pn(nt.add(ht), Te.labelPlaneMatrix, Te.getElevation).point;
                  }
                  if (Ae) {
                    const ht = Ot(Te.tileAnchorPoint.x + 1, Te.tileAnchorPoint.y, Te).point.sub(Fe), Et = Math.atan(ht.y / ht.x) + (ht.x < 0 ? Math.PI : 0);
                    return Fe.add(Oe.rotate(Et));
                  }
                  return Fe.add(Oe);
                }
                function No(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn, xn, An) {
                  const On = Fe.text.placedSymbolArray, Bn = Fe.text.dynamicLayoutVertexArray, qn = Fe.icon.dynamicLayoutVertexArray, Vn = {};
                  Bn.clear();
                  for (let sr = 0; sr < On.length; sr++) {
                    const Un = On.get(sr), wr = Fe.allowVerticalPlacement && !Un.placedOrientation, Cr = Un.hidden || !Un.crossTileID || wr ? null : Oe[Un.crossTileID];
                    if (Cr) {
                      const Tr = new ee.P(Un.anchorX, Un.anchorY), ii = { getElevation: An, width: Be.width, height: Be.height, labelPlaneMatrix: Ue, lineVertexArray: null, pitchWithMap: Ae, projection: qt, projectionCache: null, tileAnchorPoint: Tr, translation: tn, unwrappedTileID: xn }, wi = Ae ? Pn(Tr, nt, An) : Ot(Tr.x, Tr.y, ii), Wi = Ie(Be.cameraToCenterDistance, wi.signedDistanceFromCamera);
                      let Yi = ee.aj(Fe.textSizeData, Et, Un) * Wi / ee.aq;
                      Ae && (Yi *= Fe.tilePixelRatio / ht);
                      const { width: ra, height: ha, anchor: ya, textOffset: va, textBoxScale: qa } = Cr, Aa = So(ya, ra, ha, va, qa, Yi), Xa = qt.getPitchedTextCorrection(Be, Tr.add(new ee.P(tn[0], tn[1])), xn), Oa = Jc(wi.point, ii, Te, Aa, Be.angle, Xa), Ya = Fe.allowVerticalPlacement && Un.placedOrientation === ee.ai.vertical ? Math.PI / 2 : 0;
                      for (let Wo = 0; Wo < Un.numGlyphs; Wo++) ee.ak(Bn, Oa, Ya);
                      Mt && Un.associatedIconIndex >= 0 && (Vn[Un.associatedIconIndex] = { shiftedAnchor: Oa, angle: Ya });
                    } else bn(Un.numGlyphs, Bn);
                  }
                  if (Mt) {
                    qn.clear();
                    const sr = Fe.icon.placedSymbolArray;
                    for (let Un = 0; Un < sr.length; Un++) {
                      const wr = sr.get(Un);
                      if (wr.hidden) bn(wr.numGlyphs, qn);
                      else {
                        const Cr = Vn[Un];
                        if (Cr) for (let Tr = 0; Tr < wr.numGlyphs; Tr++) ee.ak(qn, Cr.shiftedAnchor, Cr.angle);
                        else bn(wr.numGlyphs, qn);
                      }
                    }
                    Fe.icon.dynamicLayoutVertexBuffer.updateData(qn);
                  }
                  Fe.text.dynamicLayoutVertexBuffer.updateData(Bn);
                }
                function ro(Fe, Te, Ae) {
                  return Ae.iconsInText && Te ? "symbolTextAndIcon" : Fe ? "symbolSDF" : "symbolIcon";
                }
                function Io(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt, qt, tn) {
                  const xn = Fe.context, An = xn.gl, On = Fe.transform, Bn = fi(), qn = ht === "map", Vn = Et === "map", sr = ht !== "viewport" && Ae.layout.get("symbol-placement") !== "point", Un = qn && !Vn && !sr, wr = !Vn && sr, Cr = !Ae.layout.get("symbol-sort-key").isConstant();
                  let Tr = !1;
                  const ii = Fe.depthModeForSublayer(0, Hs.ReadOnly), wi = Ae._unevaluatedLayout.hasValue("text-variable-anchor") || Ae._unevaluatedLayout.hasValue("text-variable-anchor-offset"), Wi = [], Yi = Bn.getCircleRadiusCorrection(On);
                  for (const ra of Oe) {
                    const ha = Te.getTile(ra), ya = ha.getBucket(Ae);
                    if (!ya) continue;
                    const va = Be ? ya.text : ya.icon;
                    if (!va || !va.segments.get().length || !va.hasVisibleVertices) continue;
                    const qa = va.programConfigurations.get(Ae.id), Aa = Be || ya.sdfIcons, Xa = Be ? ya.textSizeData : ya.iconSizeData, Oa = Vn || On.pitch !== 0, Ya = Fe.useProgram(ro(Aa, Be, ya), qa), Wo = ee.ah(Xa, On.zoom), Ja = Fe.style.map.terrain && Fe.style.map.terrain.getTerrainData(ra);
                    let Uo, ys, il, Is, ml = [0, 0], nl = null;
                    if (Be) {
                      if (ys = ha.glyphAtlasTexture, il = An.LINEAR, Uo = ha.glyphAtlasTexture.size, ya.iconsInText) {
                        ml = ha.imageAtlasTexture.size, nl = ha.imageAtlasTexture;
                        const $s = Xa.kind === "composite" || Xa.kind === "camera";
                        Is = Oa || Fe.options.rotating || Fe.options.zooming || $s ? An.LINEAR : An.NEAREST;
                      }
                    } else {
                      const $s = Ae.layout.get("icon-size").constantOr(0) !== 1 || ya.iconsNeedLinear;
                      ys = ha.imageAtlasTexture, il = Aa || Fe.options.rotating || Fe.options.zooming || $s || Oa ? An.LINEAR : An.NEAREST, Uo = ha.imageAtlasTexture.size;
                    }
                    const Il = Mn(ha, 1, Fe.transform.zoom), Fl = wr ? ra.posMatrix : uo, qc = wn(Fl, Vn, qn, Fe.transform, Il), ql = Rn(Fl, Vn, qn, Fe.transform, Il), jc = Rn(ra.posMatrix, Vn, qn, Fe.transform, Il), Dl = Bn.translatePosition(Fe.transform, ha, Ue, nt), Ml = wi && ya.hasTextData(), pc = Ae.layout.get("icon-text-fit") !== "none" && Ml && ya.hasIconData();
                    if (sr) {
                      const $s = Fe.style.map.terrain ? (wc, zu) => Fe.style.map.terrain.getElevation(ra, wc, zu) : null, Xc = Ae.layout.get("text-rotation-alignment") === "map";
                      ft(ya, ra.posMatrix, Fe, Be, qc, jc, Vn, Mt, Xc, Bn, ra.toUnwrapped(), On.width, On.height, Dl, $s);
                    }
                    const hl = ra.posMatrix, xl = Be && wi || pc, Bl = sr || xl ? uo : qc, al = ql, Oc = Aa && Ae.paint.get(Be ? "text-halo-width" : "icon-halo-width").constantOr(1) !== 0;
                    let nu;
                    nu = Aa ? ya.iconsInText ? Mi(Xa.kind, Wo, Un, Vn, sr, xl, Fe, hl, Bl, al, Dl, Uo, ml, Yi) : ki(Xa.kind, Wo, Un, Vn, sr, xl, Fe, hl, Bl, al, Dl, Be, Uo, !0, Yi) : Oi(Xa.kind, Wo, Un, Vn, sr, xl, Fe, hl, Bl, al, Dl, Be, Uo, Yi);
                    const Ic = { program: Ya, buffers: va, uniformValues: nu, atlasTexture: ys, atlasTextureIcon: nl, atlasInterpolation: il, atlasInterpolationIcon: Is, isSDF: Aa, hasHalo: Oc };
                    if (Cr && ya.canOverlap) {
                      Tr = !0;
                      const $s = va.segments.get();
                      for (const Xc of $s) Wi.push({ segments: new ee.a0([Xc]), sortKey: Xc.sortKey, state: Ic, terrainData: Ja });
                    } else Wi.push({ segments: va.segments, sortKey: 0, state: Ic, terrainData: Ja });
                  }
                  Tr && Wi.sort((ra, ha) => ra.sortKey - ha.sortKey);
                  for (const ra of Wi) {
                    const ha = ra.state;
                    if (xn.activeTexture.set(An.TEXTURE0), ha.atlasTexture.bind(ha.atlasInterpolation, An.CLAMP_TO_EDGE), ha.atlasTextureIcon && (xn.activeTexture.set(An.TEXTURE1), ha.atlasTextureIcon && ha.atlasTextureIcon.bind(ha.atlasInterpolationIcon, An.CLAMP_TO_EDGE)), ha.isSDF) {
                      const ya = ha.uniformValues;
                      ha.hasHalo && (ya.u_is_halo = 1, bs(ha.buffers, ra.segments, Ae, Fe, ha.program, ii, qt, tn, ya, ra.terrainData)), ya.u_is_halo = 0;
                    }
                    bs(ha.buffers, ra.segments, Ae, Fe, ha.program, ii, qt, tn, ha.uniformValues, ra.terrainData);
                  }
                }
                function bs(Fe, Te, Ae, Oe, Be, Ue, nt, ht, Et, Mt) {
                  const qt = Oe.context, tn = qt.gl;
                  Be.draw(qt, tn.TRIANGLES, Ue, nt, ht, zl.disabled, Et, Mt, Ae.id, Fe.layoutVertexBuffer, Fe.indexBuffer, Te, Ae.paint, Oe.transform.zoom, Fe.programConfigurations.get(Ae.id), Fe.dynamicLayoutVertexBuffer, Fe.opacityVertexBuffer);
                }
                function ws(Fe, Te, Ae, Oe) {
                  if (Ae.paint.get("heatmap-opacity") !== 0) if (Fe.renderPass === "offscreen") {
                    const Be = Fe.context, Ue = Be.gl, nt = Wl.disabled, ht = new Kl([Ue.ONE, Ue.ONE], ee.aN.transparent, [!0, !0, !0, !0]);
                    (function(Et, Mt, qt) {
                      const tn = Et.gl;
                      Et.activeTexture.set(tn.TEXTURE1), Et.viewport.set([0, 0, Mt.width / 4, Mt.height / 4]);
                      let xn = qt.heatmapFbo;
                      if (xn) tn.bindTexture(tn.TEXTURE_2D, xn.colorAttachment.get()), Et.bindFramebuffer.set(xn.framebuffer);
                      else {
                        const An = tn.createTexture();
                        tn.bindTexture(tn.TEXTURE_2D, An), tn.texParameteri(tn.TEXTURE_2D, tn.TEXTURE_WRAP_S, tn.CLAMP_TO_EDGE), tn.texParameteri(tn.TEXTURE_2D, tn.TEXTURE_WRAP_T, tn.CLAMP_TO_EDGE), tn.texParameteri(tn.TEXTURE_2D, tn.TEXTURE_MIN_FILTER, tn.LINEAR), tn.texParameteri(tn.TEXTURE_2D, tn.TEXTURE_MAG_FILTER, tn.LINEAR), xn = qt.heatmapFbo = Et.createFramebuffer(Mt.width / 4, Mt.height / 4, !1, !1), function(On, Bn, qn, Vn) {
                          var sr, Un;
                          const wr = On.gl, Cr = (sr = On.HALF_FLOAT) !== null && sr !== void 0 ? sr : wr.UNSIGNED_BYTE, Tr = (Un = On.RGBA16F) !== null && Un !== void 0 ? Un : wr.RGBA;
                          wr.texImage2D(wr.TEXTURE_2D, 0, Tr, Bn.width / 4, Bn.height / 4, 0, wr.RGBA, Cr, null), Vn.colorAttachment.set(qn);
                        }(Et, Mt, An, xn);
                      }
                    })(Be, Fe, Ae), Be.clear({ color: ee.aN.transparent });
                    for (let Et = 0; Et < Oe.length; Et++) {
                      const Mt = Oe[Et];
                      if (Te.hasRenderableParent(Mt)) continue;
                      const qt = Te.getTile(Mt), tn = qt.getBucket(Ae);
                      if (!tn) continue;
                      const xn = tn.programConfigurations.get(Ae.id), An = Fe.useProgram("heatmap", xn), { zoom: On } = Fe.transform;
                      An.draw(Be, Ue.TRIANGLES, Hs.disabled, nt, ht, zl.disabled, Xr(Mt.posMatrix, qt, On, Ae.paint.get("heatmap-intensity")), null, Ae.id, tn.layoutVertexBuffer, tn.indexBuffer, tn.segments, Ae.paint, Fe.transform.zoom, xn);
                    }
                    Be.viewport.set([0, 0, Fe.width, Fe.height]);
                  } else Fe.renderPass === "translucent" && (Fe.context.setColorMode(Fe.colorModeForRenderPass()), function(Be, Ue) {
                    const nt = Be.context, ht = nt.gl, Et = Ue.heatmapFbo;
                    if (!Et) return;
                    nt.activeTexture.set(ht.TEXTURE0), ht.bindTexture(ht.TEXTURE_2D, Et.colorAttachment.get()), nt.activeTexture.set(ht.TEXTURE1);
                    let Mt = Ue.colorRampTexture;
                    Mt || (Mt = Ue.colorRampTexture = new Le(nt, Ue.colorRamp, ht.RGBA)), Mt.bind(ht.LINEAR, ht.CLAMP_TO_EDGE), Be.useProgram("heatmapTexture").draw(nt, ht.TRIANGLES, Hs.disabled, Wl.disabled, Be.colorModeForRenderPass(), zl.disabled, Jr(Be, Ue, 0, 1), null, Ue.id, Be.viewportBuffer, Be.quadTriangleIndexBuffer, Be.viewportSegments, Ue.paint, Be.transform.zoom);
                  }(Fe, Ae));
                }
                function zo(Fe, Te, Ae, Oe, Be) {
                  if (!Ae || !Oe || !Oe.imageAtlas) return;
                  const Ue = Oe.imageAtlas.patternPositions;
                  let nt = Ue[Ae.to.toString()], ht = Ue[Ae.from.toString()];
                  if (!nt && ht && (nt = ht), !ht && nt && (ht = nt), !nt || !ht) {
                    const Et = Be.getPaintProperty(Te);
                    nt = Ue[Et], ht = Ue[Et];
                  }
                  nt && ht && Fe.setConstantPatternPositions(nt, ht);
                }
                function Yo(Fe, Te, Ae, Oe, Be, Ue, nt) {
                  const ht = Fe.context.gl, Et = "fill-pattern", Mt = Ae.paint.get(Et), qt = Mt && Mt.constantOr(1), tn = Ae.getCrossfadeParameters();
                  let xn, An, On, Bn, qn;
                  nt ? (An = qt && !Ae.getPaintProperty("fill-outline-color") ? "fillOutlinePattern" : "fillOutline", xn = ht.LINES) : (An = qt ? "fillPattern" : "fill", xn = ht.TRIANGLES);
                  const Vn = Mt.constantOr(null);
                  for (const sr of Oe) {
                    const Un = Te.getTile(sr);
                    if (qt && !Un.patternsLoaded()) continue;
                    const wr = Un.getBucket(Ae);
                    if (!wr) continue;
                    const Cr = wr.programConfigurations.get(Ae.id), Tr = Fe.useProgram(An, Cr), ii = Fe.style.map.terrain && Fe.style.map.terrain.getTerrainData(sr);
                    qt && (Fe.context.activeTexture.set(ht.TEXTURE0), Un.imageAtlasTexture.bind(ht.LINEAR, ht.CLAMP_TO_EDGE), Cr.updatePaintBuffers(tn)), zo(Cr, Et, Vn, Un, Ae);
                    const wi = ii ? sr : null, Wi = wi ? wi.posMatrix : sr.posMatrix, Yi = Fe.translatePosMatrix(Wi, Un, Ae.paint.get("fill-translate"), Ae.paint.get("fill-translate-anchor"));
                    if (nt) {
                      Bn = wr.indexBuffer2, qn = wr.segments2;
                      const ra = [ht.drawingBufferWidth, ht.drawingBufferHeight];
                      On = An === "fillOutlinePattern" && qt ? Qr(Yi, Fe, tn, Un, ra) : Wa(Yi, ra);
                    } else Bn = wr.indexBuffer, qn = wr.segments, On = qt ? wa(Yi, Fe, tn, Un) : la(Yi);
                    Tr.draw(Fe.context, xn, Be, Fe.stencilModeForClipping(sr), Ue, zl.disabled, On, ii, Ae.id, wr.layoutVertexBuffer, Bn, qn, Ae.paint, Fe.transform.zoom, Cr);
                  }
                }
                function Ls(Fe, Te, Ae, Oe, Be, Ue, nt) {
                  const ht = Fe.context, Et = ht.gl, Mt = "fill-extrusion-pattern", qt = Ae.paint.get(Mt), tn = qt.constantOr(1), xn = Ae.getCrossfadeParameters(), An = Ae.paint.get("fill-extrusion-opacity"), On = qt.constantOr(null);
                  for (const Bn of Oe) {
                    const qn = Te.getTile(Bn), Vn = qn.getBucket(Ae);
                    if (!Vn) continue;
                    const sr = Fe.style.map.terrain && Fe.style.map.terrain.getTerrainData(Bn), Un = Vn.programConfigurations.get(Ae.id), wr = Fe.useProgram(tn ? "fillExtrusionPattern" : "fillExtrusion", Un);
                    tn && (Fe.context.activeTexture.set(Et.TEXTURE0), qn.imageAtlasTexture.bind(Et.LINEAR, Et.CLAMP_TO_EDGE), Un.updatePaintBuffers(xn)), zo(Un, Mt, On, qn, Ae);
                    const Cr = Fe.translatePosMatrix(Bn.posMatrix, qn, Ae.paint.get("fill-extrusion-translate"), Ae.paint.get("fill-extrusion-translate-anchor")), Tr = Ae.paint.get("fill-extrusion-vertical-gradient"), ii = tn ? Da(Cr, Fe, Tr, An, Bn, xn, qn) : di(Cr, Fe, Tr, An);
                    wr.draw(ht, ht.gl.TRIANGLES, Be, Ue, nt, zl.backCCW, ii, sr, Ae.id, Vn.layoutVertexBuffer, Vn.indexBuffer, Vn.segments, Ae.paint, Fe.transform.zoom, Un, Fe.style.map.terrain && Vn.centroidVertexBuffer);
                  }
                }
                function Ms(Fe, Te, Ae, Oe, Be, Ue, nt) {
                  const ht = Fe.context, Et = ht.gl, Mt = Ae.fbo;
                  if (!Mt) return;
                  const qt = Fe.useProgram("hillshade"), tn = Fe.style.map.terrain && Fe.style.map.terrain.getTerrainData(Te);
                  ht.activeTexture.set(Et.TEXTURE0), Et.bindTexture(Et.TEXTURE_2D, Mt.colorAttachment.get());
                  const xn = tn ? Te : null;
                  qt.draw(ht, Et.TRIANGLES, Be, Ue, nt, zl.disabled, ti(Fe, Ae, Oe, xn), tn, Oe.id, Fe.rasterBoundsBuffer, Fe.quadTriangleIndexBuffer, Fe.rasterBoundsSegments);
                }
                function vi(Fe, Te, Ae, Oe, Be, Ue) {
                  const nt = Fe.context, ht = nt.gl, Et = Te.dem;
                  if (Et && Et.data) {
                    const Mt = Et.dim, qt = Et.stride, tn = Et.getPixels();
                    if (nt.activeTexture.set(ht.TEXTURE1), nt.pixelStoreUnpackPremultiplyAlpha.set(!1), Te.demTexture = Te.demTexture || Fe.getTileTexture(qt), Te.demTexture) {
                      const An = Te.demTexture;
                      An.update(tn, { premultiply: !1 }), An.bind(ht.NEAREST, ht.CLAMP_TO_EDGE);
                    } else Te.demTexture = new Le(nt, tn, ht.RGBA, { premultiply: !1 }), Te.demTexture.bind(ht.NEAREST, ht.CLAMP_TO_EDGE);
                    nt.activeTexture.set(ht.TEXTURE0);
                    let xn = Te.fbo;
                    if (!xn) {
                      const An = new Le(nt, { width: Mt, height: Mt, data: null }, ht.RGBA);
                      An.bind(ht.LINEAR, ht.CLAMP_TO_EDGE), xn = Te.fbo = nt.createFramebuffer(Mt, Mt, !0, !1), xn.colorAttachment.set(An.texture);
                    }
                    nt.bindFramebuffer.set(xn.framebuffer), nt.viewport.set([0, 0, Mt, Mt]), Fe.useProgram("hillshadePrepare").draw(nt, ht.TRIANGLES, Oe, Be, Ue, zl.disabled, Hr(Te.tileID, Et), null, Ae.id, Fe.rasterBoundsBuffer, Fe.quadTriangleIndexBuffer, Fe.rasterBoundsSegments), Te.needsHillshadePrepare = !1;
                  }
                }
                function Ri(Fe, Te, Ae, Oe, Be, Ue) {
                  const nt = Oe.paint.get("raster-fade-duration");
                  if (!Ue && nt > 0) {
                    const ht = ce.now(), Et = (ht - Fe.timeAdded) / nt, Mt = Te ? (ht - Te.timeAdded) / nt : -1, qt = Ae.getSource(), tn = Be.coveringZoomLevel({ tileSize: qt.tileSize, roundZoom: qt.roundZoom }), xn = !Te || Math.abs(Te.tileID.overscaledZ - tn) > Math.abs(Fe.tileID.overscaledZ - tn), An = xn && Fe.refreshedUponExpiration ? 1 : ee.ad(xn ? Et : 1 - Mt, 0, 1);
                    return Fe.refreshedUponExpiration && Et >= 1 && (Fe.refreshedUponExpiration = !1), Te ? { opacity: 1, mix: 1 - An } : { opacity: An, mix: 0 };
                  }
                  return { opacity: 1, mix: 0 };
                }
                const ta = new ee.aN(1, 0, 0, 1), da = new ee.aN(0, 1, 0, 1), Ji = new ee.aN(0, 0, 1, 1), Pa = new ee.aN(1, 0, 1, 1), Ga = new ee.aN(0, 1, 1, 1);
                function Ha(Fe) {
                  const Te = Fe.transform.padding;
                  Ba(Fe, Fe.transform.height - (Te.top || 0), 3, ta), Ba(Fe, Te.bottom || 0, 3, da), ao(Fe, Te.left || 0, 3, Ji), ao(Fe, Fe.transform.width - (Te.right || 0), 3, Pa);
                  const Ae = Fe.transform.centerPoint;
                  (function(Oe, Be, Ue, nt) {
                    io(Oe, Be - 1, Ue - 10, 2, 20, nt), io(Oe, Be - 10, Ue - 1, 20, 2, nt);
                  })(Fe, Ae.x, Fe.transform.height - Ae.y, Ga);
                }
                function Ba(Fe, Te, Ae, Oe) {
                  io(Fe, 0, Te + Ae / 2, Fe.transform.width, Ae, Oe);
                }
                function ao(Fe, Te, Ae, Oe) {
                  io(Fe, Te - Ae / 2, 0, Ae, Fe.transform.height, Oe);
                }
                function io(Fe, Te, Ae, Oe, Be, Ue) {
                  const nt = Fe.context, ht = nt.gl;
                  ht.enable(ht.SCISSOR_TEST), ht.scissor(Te * Fe.pixelRatio, Ae * Fe.pixelRatio, Oe * Fe.pixelRatio, Be * Fe.pixelRatio), nt.clear({ color: Ue }), ht.disable(ht.SCISSOR_TEST);
                }
                function To(Fe, Te, Ae) {
                  const Oe = Fe.context, Be = Oe.gl, Ue = Ae.posMatrix, nt = Fe.useProgram("debug"), ht = Hs.disabled, Et = Wl.disabled, Mt = Fe.colorModeForRenderPass(), qt = "$debug", tn = Fe.style.map.terrain && Fe.style.map.terrain.getTerrainData(Ae);
                  Oe.activeTexture.set(Be.TEXTURE0);
                  const xn = Te.getTileByID(Ae.key).latestRawTileData, An = xn && xn.byteLength || 0, On = Math.floor(An / 1024), Bn = Te.getTile(Ae).tileSize, qn = 512 / Math.min(Bn, 512) * (Ae.overscaledZ / Fe.transform.zoom) * 0.5;
                  let Vn = Ae.canonical.toString();
                  Ae.overscaledZ !== Ae.canonical.z && (Vn += ` => ${Ae.overscaledZ}`), function(sr, Un) {
                    sr.initDebugOverlayCanvas();
                    const wr = sr.debugOverlayCanvas, Cr = sr.context.gl, Tr = sr.debugOverlayCanvas.getContext("2d");
                    Tr.clearRect(0, 0, wr.width, wr.height), Tr.shadowColor = "white", Tr.shadowBlur = 2, Tr.lineWidth = 1.5, Tr.strokeStyle = "white", Tr.textBaseline = "top", Tr.font = "bold 36px Open Sans, sans-serif", Tr.fillText(Un, 5, 5), Tr.strokeText(Un, 5, 5), sr.debugOverlayTexture.update(wr), sr.debugOverlayTexture.bind(Cr.LINEAR, Cr.CLAMP_TO_EDGE);
                  }(Fe, `${Vn} ${On}kB`), nt.draw(Oe, Be.TRIANGLES, ht, Et, Kl.alphaBlended, zl.disabled, Rr(Ue, ee.aN.transparent, qn), null, qt, Fe.debugBuffer, Fe.quadTriangleIndexBuffer, Fe.debugSegments), nt.draw(Oe, Be.LINE_STRIP, ht, Et, Mt, zl.disabled, Rr(Ue, ee.aN.red), tn, qt, Fe.debugBuffer, Fe.tileBorderIndexBuffer, Fe.debugSegments);
                }
                function Mo(Fe, Te, Ae) {
                  const Oe = Fe.context, Be = Oe.gl, Ue = Fe.colorModeForRenderPass(), nt = new Hs(Be.LEQUAL, Hs.ReadWrite, Fe.depthRangeFor3D), ht = Fe.useProgram("terrain"), Et = Te.getTerrainMesh();
                  Oe.bindFramebuffer.set(null), Oe.viewport.set([0, 0, Fe.width, Fe.height]);
                  for (const Mt of Ae) {
                    const qt = Fe.renderToTexture.getTexture(Mt), tn = Te.getTerrainData(Mt.tileID);
                    Oe.activeTexture.set(Be.TEXTURE0), Be.bindTexture(Be.TEXTURE_2D, qt.texture);
                    const xn = Fe.transform.calculatePosMatrix(Mt.tileID.toUnwrapped()), An = Te.getMeshFrameDelta(Fe.transform.zoom), On = Fe.transform.calculateFogMatrix(Mt.tileID.toUnwrapped()), Bn = Sr(xn, An, On, Fe.style.sky, Fe.transform.pitch);
                    ht.draw(Oe, Be.TRIANGLES, nt, Wl.disabled, Ue, zl.backCCW, Bn, tn, "terrain", Et.vertexBuffer, Et.indexBuffer, Et.segments);
                  }
                }
                class po {
                  constructor(Te, Ae, Oe) {
                    this.vertexBuffer = Te, this.indexBuffer = Ae, this.segments = Oe;
                  }
                  destroy() {
                    this.vertexBuffer.destroy(), this.indexBuffer.destroy(), this.segments.destroy(), this.vertexBuffer = null, this.indexBuffer = null, this.segments = null;
                  }
                }
                class Vo {
                  constructor(Te, Ae) {
                    this.context = new uu(Te), this.transform = Ae, this._tileTextures = {}, this.terrainFacilitator = { dirty: !0, matrix: ee.ao(new Float64Array(16)), renderTime: 0 }, this.setup(), this.numSublayers = rn.maxUnderzooming + rn.maxOverzooming + 1, this.depthEpsilon = 1 / Math.pow(2, 16), this.crossTileSymbolIndex = new ei();
                  }
                  resize(Te, Ae, Oe) {
                    if (this.width = Math.floor(Te * Oe), this.height = Math.floor(Ae * Oe), this.pixelRatio = Oe, this.context.viewport.set([0, 0, this.width, this.height]), this.style) for (const Be of this.style._order) this.style._layers[Be].resize();
                  }
                  setup() {
                    const Te = this.context, Ae = new ee.aX();
                    Ae.emplaceBack(0, 0), Ae.emplaceBack(ee.X, 0), Ae.emplaceBack(0, ee.X), Ae.emplaceBack(ee.X, ee.X), this.tileExtentBuffer = Te.createVertexBuffer(Ae, aa.members), this.tileExtentSegments = ee.a0.simpleSegment(0, 0, 4, 2);
                    const Oe = new ee.aX();
                    Oe.emplaceBack(0, 0), Oe.emplaceBack(ee.X, 0), Oe.emplaceBack(0, ee.X), Oe.emplaceBack(ee.X, ee.X), this.debugBuffer = Te.createVertexBuffer(Oe, aa.members), this.debugSegments = ee.a0.simpleSegment(0, 0, 4, 5);
                    const Be = new ee.$();
                    Be.emplaceBack(0, 0, 0, 0), Be.emplaceBack(ee.X, 0, ee.X, 0), Be.emplaceBack(0, ee.X, 0, ee.X), Be.emplaceBack(ee.X, ee.X, ee.X, ee.X), this.rasterBoundsBuffer = Te.createVertexBuffer(Be, Gt.members), this.rasterBoundsSegments = ee.a0.simpleSegment(0, 0, 4, 2);
                    const Ue = new ee.aX();
                    Ue.emplaceBack(0, 0), Ue.emplaceBack(1, 0), Ue.emplaceBack(0, 1), Ue.emplaceBack(1, 1), this.viewportBuffer = Te.createVertexBuffer(Ue, aa.members), this.viewportSegments = ee.a0.simpleSegment(0, 0, 4, 2);
                    const nt = new ee.aZ();
                    nt.emplaceBack(0), nt.emplaceBack(1), nt.emplaceBack(3), nt.emplaceBack(2), nt.emplaceBack(0), this.tileBorderIndexBuffer = Te.createIndexBuffer(nt);
                    const ht = new ee.aY();
                    ht.emplaceBack(0, 1, 2), ht.emplaceBack(2, 1, 3), this.quadTriangleIndexBuffer = Te.createIndexBuffer(ht);
                    const Et = this.context.gl;
                    this.stencilClearMode = new Wl({ func: Et.ALWAYS, mask: 0 }, 0, 255, Et.ZERO, Et.ZERO, Et.ZERO);
                  }
                  clearStencil() {
                    const Te = this.context, Ae = Te.gl;
                    this.nextStencilID = 1, this.currentStencilSource = void 0;
                    const Oe = ee.H();
                    ee.aQ(Oe, 0, this.width, this.height, 0, 0, 1), ee.K(Oe, Oe, [Ae.drawingBufferWidth, Ae.drawingBufferHeight, 0]), this.useProgram("clippingMask").draw(Te, Ae.TRIANGLES, Hs.disabled, this.stencilClearMode, Kl.disabled, zl.disabled, Vr(Oe), null, "$clipping", this.viewportBuffer, this.quadTriangleIndexBuffer, this.viewportSegments);
                  }
                  _renderTileClippingMasks(Te, Ae) {
                    if (this.currentStencilSource === Te.source || !Te.isTileClipped() || !Ae || !Ae.length) return;
                    this.currentStencilSource = Te.source;
                    const Oe = this.context, Be = Oe.gl;
                    this.nextStencilID + Ae.length > 256 && this.clearStencil(), Oe.setColorMode(Kl.disabled), Oe.setDepthMode(Hs.disabled);
                    const Ue = this.useProgram("clippingMask");
                    this._tileClippingMaskIDs = {};
                    for (const nt of Ae) {
                      const ht = this._tileClippingMaskIDs[nt.key] = this.nextStencilID++, Et = this.style.map.terrain && this.style.map.terrain.getTerrainData(nt);
                      Ue.draw(Oe, Be.TRIANGLES, Hs.disabled, new Wl({ func: Be.ALWAYS, mask: 0 }, ht, 255, Be.KEEP, Be.KEEP, Be.REPLACE), Kl.disabled, zl.disabled, Vr(nt.posMatrix), Et, "$clipping", this.tileExtentBuffer, this.quadTriangleIndexBuffer, this.tileExtentSegments);
                    }
                  }
                  stencilModeFor3D() {
                    this.currentStencilSource = void 0, this.nextStencilID + 1 > 256 && this.clearStencil();
                    const Te = this.nextStencilID++, Ae = this.context.gl;
                    return new Wl({ func: Ae.NOTEQUAL, mask: 255 }, Te, 255, Ae.KEEP, Ae.KEEP, Ae.REPLACE);
                  }
                  stencilModeForClipping(Te) {
                    const Ae = this.context.gl;
                    return new Wl({ func: Ae.EQUAL, mask: 255 }, this._tileClippingMaskIDs[Te.key], 0, Ae.KEEP, Ae.KEEP, Ae.REPLACE);
                  }
                  stencilConfigForOverlap(Te) {
                    const Ae = this.context.gl, Oe = Te.sort((nt, ht) => ht.overscaledZ - nt.overscaledZ), Be = Oe[Oe.length - 1].overscaledZ, Ue = Oe[0].overscaledZ - Be + 1;
                    if (Ue > 1) {
                      this.currentStencilSource = void 0, this.nextStencilID + Ue > 256 && this.clearStencil();
                      const nt = {};
                      for (let ht = 0; ht < Ue; ht++) nt[ht + Be] = new Wl({ func: Ae.GEQUAL, mask: 255 }, ht + this.nextStencilID, 255, Ae.KEEP, Ae.KEEP, Ae.REPLACE);
                      return this.nextStencilID += Ue, [nt, Oe];
                    }
                    return [{ [Be]: Wl.disabled }, Oe];
                  }
                  colorModeForRenderPass() {
                    const Te = this.context.gl;
                    return this._showOverdrawInspector ? new Kl([Te.CONSTANT_COLOR, Te.ONE], new ee.aN(0.125, 0.125, 0.125, 0), [!0, !0, !0, !0]) : this.renderPass === "opaque" ? Kl.unblended : Kl.alphaBlended;
                  }
                  depthModeForSublayer(Te, Ae, Oe) {
                    if (!this.opaquePassEnabledForLayer()) return Hs.disabled;
                    const Be = 1 - ((1 + this.currentLayer) * this.numSublayers + Te) * this.depthEpsilon;
                    return new Hs(Oe || this.context.gl.LEQUAL, Ae, [Be, Be]);
                  }
                  opaquePassEnabledForLayer() {
                    return this.currentLayer < this.opaquePassCutoff;
                  }
                  render(Te, Ae) {
                    var Oe;
                    this.style = Te, this.options = Ae, this.lineAtlas = Te.lineAtlas, this.imageManager = Te.imageManager, this.glyphManager = Te.glyphManager, this.symbolFadeChange = Te.placement.symbolFadeChange(ce.now()), this.imageManager.beginFrame();
                    const Be = this.style._order, Ue = this.style.sourceCaches, nt = {}, ht = {}, Et = {};
                    for (const Mt in Ue) {
                      const qt = Ue[Mt];
                      qt.used && qt.prepare(this.context), nt[Mt] = qt.getVisibleCoordinates(), ht[Mt] = nt[Mt].slice().reverse(), Et[Mt] = qt.getVisibleCoordinates(!0).reverse();
                    }
                    this.opaquePassCutoff = 1 / 0;
                    for (let Mt = 0; Mt < Be.length; Mt++) {
                      const qt = Be[Mt];
                      if (this.style._layers[qt].is3D()) {
                        this.opaquePassCutoff = Mt;
                        break;
                      }
                    }
                    this.maybeDrawDepthAndCoords(!1), this.renderToTexture && (this.renderToTexture.prepareForRender(this.style, this.transform.zoom), this.opaquePassCutoff = 0), this.renderPass = "offscreen";
                    for (const Mt of Be) {
                      const qt = this.style._layers[Mt];
                      if (!qt.hasOffscreenPass() || qt.isHidden(this.transform.zoom)) continue;
                      const tn = ht[qt.source];
                      (qt.type === "custom" || tn.length) && this.renderLayer(this, Ue[qt.source], qt, tn);
                    }
                    if (this.context.bindFramebuffer.set(null), this.context.clear({ color: Ae.showOverdrawInspector ? ee.aN.black : ee.aN.transparent, depth: 1 }), this.clearStencil(), !((Oe = this.style.stylesheet) === null || Oe === void 0) && Oe.sky && function(Mt, qt) {
                      const tn = Mt.context, xn = tn.gl, An = ((sr, Un, wr) => ({ u_sky_color: sr.properties.get("sky-color"), u_horizon_color: sr.properties.get("horizon-color"), u_horizon: (Un.height / 2 + Un.getHorizon()) * wr, u_sky_horizon_blend: sr.properties.get("sky-horizon-blend") * Un.height / 2 * wr }))(qt, Mt.style.map.transform, Mt.pixelRatio), On = new Hs(xn.LEQUAL, Hs.ReadWrite, [0, 1]), Bn = Wl.disabled, qn = Mt.colorModeForRenderPass(), Vn = Mt.useProgram("sky");
                      if (!qt.mesh) {
                        const sr = new ee.aX();
                        sr.emplaceBack(-1, -1), sr.emplaceBack(1, -1), sr.emplaceBack(1, 1), sr.emplaceBack(-1, 1);
                        const Un = new ee.aY();
                        Un.emplaceBack(0, 1, 2), Un.emplaceBack(0, 2, 3), qt.mesh = new po(tn.createVertexBuffer(sr, aa.members), tn.createIndexBuffer(Un), ee.a0.simpleSegment(0, 0, sr.length, Un.length));
                      }
                      Vn.draw(tn, xn.TRIANGLES, On, Bn, qn, zl.disabled, An, void 0, "sky", qt.mesh.vertexBuffer, qt.mesh.indexBuffer, qt.mesh.segments);
                    }(this, this.style.sky), this._showOverdrawInspector = Ae.showOverdrawInspector, this.depthRangeFor3D = [0, 1 - (Te._order.length + 2) * this.numSublayers * this.depthEpsilon], !this.renderToTexture) for (this.renderPass = "opaque", this.currentLayer = Be.length - 1; this.currentLayer >= 0; this.currentLayer--) {
                      const Mt = this.style._layers[Be[this.currentLayer]], qt = Ue[Mt.source], tn = nt[Mt.source];
                      this._renderTileClippingMasks(Mt, tn), this.renderLayer(this, qt, Mt, tn);
                    }
                    for (this.renderPass = "translucent", this.currentLayer = 0; this.currentLayer < Be.length; this.currentLayer++) {
                      const Mt = this.style._layers[Be[this.currentLayer]], qt = Ue[Mt.source];
                      if (this.renderToTexture && this.renderToTexture.renderLayer(Mt)) continue;
                      const tn = (Mt.type === "symbol" ? Et : ht)[Mt.source];
                      this._renderTileClippingMasks(Mt, nt[Mt.source]), this.renderLayer(this, qt, Mt, tn);
                    }
                    if (this.options.showTileBoundaries) {
                      const Mt = function(qt, tn) {
                        let xn = null;
                        const An = Object.values(qt._layers).flatMap((Vn) => Vn.source && !Vn.isHidden(tn) ? [qt.sourceCaches[Vn.source]] : []), On = An.filter((Vn) => Vn.getSource().type === "vector"), Bn = An.filter((Vn) => Vn.getSource().type !== "vector"), qn = (Vn) => {
                          (!xn || xn.getSource().maxzoom < Vn.getSource().maxzoom) && (xn = Vn);
                        };
                        return On.forEach((Vn) => qn(Vn)), xn || Bn.forEach((Vn) => qn(Vn)), xn;
                      }(this.style, this.transform.zoom);
                      Mt && function(qt, tn, xn) {
                        for (let An = 0; An < xn.length; An++) To(qt, tn, xn[An]);
                      }(this, Mt, Mt.getVisibleCoordinates());
                    }
                    this.options.showPadding && Ha(this), this.context.setDefault();
                  }
                  maybeDrawDepthAndCoords(Te) {
                    if (!this.style || !this.style.map || !this.style.map.terrain) return;
                    const Ae = this.terrainFacilitator.matrix, Oe = this.transform.modelViewProjectionMatrix;
                    let Be = this.terrainFacilitator.dirty;
                    Be || (Be = Te ? !ee.a_(Ae, Oe) : !ee.a$(Ae, Oe)), Be || (Be = this.style.map.terrain.sourceCache.tilesAfterTime(this.terrainFacilitator.renderTime).length > 0), Be && (ee.b0(Ae, Oe), this.terrainFacilitator.renderTime = Date.now(), this.terrainFacilitator.dirty = !1, function(Ue, nt) {
                      const ht = Ue.context, Et = ht.gl, Mt = Kl.unblended, qt = new Hs(Et.LEQUAL, Hs.ReadWrite, [0, 1]), tn = nt.getTerrainMesh(), xn = nt.sourceCache.getRenderableTiles(), An = Ue.useProgram("terrainDepth");
                      ht.bindFramebuffer.set(nt.getFramebuffer("depth").framebuffer), ht.viewport.set([0, 0, Ue.width / devicePixelRatio, Ue.height / devicePixelRatio]), ht.clear({ color: ee.aN.transparent, depth: 1 });
                      for (const On of xn) {
                        const Bn = nt.getTerrainData(On.tileID), qn = { u_matrix: Ue.transform.calculatePosMatrix(On.tileID.toUnwrapped()), u_ele_delta: nt.getMeshFrameDelta(Ue.transform.zoom) };
                        An.draw(ht, Et.TRIANGLES, qt, Wl.disabled, Mt, zl.backCCW, qn, Bn, "terrain", tn.vertexBuffer, tn.indexBuffer, tn.segments);
                      }
                      ht.bindFramebuffer.set(null), ht.viewport.set([0, 0, Ue.width, Ue.height]);
                    }(this, this.style.map.terrain), function(Ue, nt) {
                      const ht = Ue.context, Et = ht.gl, Mt = Kl.unblended, qt = new Hs(Et.LEQUAL, Hs.ReadWrite, [0, 1]), tn = nt.getTerrainMesh(), xn = nt.getCoordsTexture(), An = nt.sourceCache.getRenderableTiles(), On = Ue.useProgram("terrainCoords");
                      ht.bindFramebuffer.set(nt.getFramebuffer("coords").framebuffer), ht.viewport.set([0, 0, Ue.width / devicePixelRatio, Ue.height / devicePixelRatio]), ht.clear({ color: ee.aN.transparent, depth: 1 }), nt.coordsIndex = [];
                      for (const Bn of An) {
                        const qn = nt.getTerrainData(Bn.tileID);
                        ht.activeTexture.set(Et.TEXTURE0), Et.bindTexture(Et.TEXTURE_2D, xn.texture);
                        const Vn = { u_matrix: Ue.transform.calculatePosMatrix(Bn.tileID.toUnwrapped()), u_terrain_coords_id: (255 - nt.coordsIndex.length) / 255, u_texture: 0, u_ele_delta: nt.getMeshFrameDelta(Ue.transform.zoom) };
                        On.draw(ht, Et.TRIANGLES, qt, Wl.disabled, Mt, zl.backCCW, Vn, qn, "terrain", tn.vertexBuffer, tn.indexBuffer, tn.segments), nt.coordsIndex.push(Bn.tileID.key);
                      }
                      ht.bindFramebuffer.set(null), ht.viewport.set([0, 0, Ue.width, Ue.height]);
                    }(this, this.style.map.terrain));
                  }
                  renderLayer(Te, Ae, Oe, Be) {
                    if (!Oe.isHidden(this.transform.zoom) && (Oe.type === "background" || Oe.type === "custom" || (Be || []).length)) switch (this.id = Oe.id, Oe.type) {
                      case "symbol":
                        so(Te, Ae, Oe, Be, this.style.placement.variableOffsets);
                        break;
                      case "circle":
                        (function(Ue, nt, ht, Et) {
                          if (Ue.renderPass !== "translucent") return;
                          const Mt = ht.paint.get("circle-opacity"), qt = ht.paint.get("circle-stroke-width"), tn = ht.paint.get("circle-stroke-opacity"), xn = !ht.layout.get("circle-sort-key").isConstant();
                          if (Mt.constantOr(1) === 0 && (qt.constantOr(1) === 0 || tn.constantOr(1) === 0)) return;
                          const An = Ue.context, On = An.gl, Bn = Ue.depthModeForSublayer(0, Hs.ReadOnly), qn = Wl.disabled, Vn = Ue.colorModeForRenderPass(), sr = [];
                          for (let Un = 0; Un < Et.length; Un++) {
                            const wr = Et[Un], Cr = nt.getTile(wr), Tr = Cr.getBucket(ht);
                            if (!Tr) continue;
                            const ii = Tr.programConfigurations.get(ht.id), wi = Ue.useProgram("circle", ii), Wi = Tr.layoutVertexBuffer, Yi = Tr.indexBuffer, ra = Ue.style.map.terrain && Ue.style.map.terrain.getTerrainData(wr), ha = { programConfiguration: ii, program: wi, layoutVertexBuffer: Wi, indexBuffer: Yi, uniformValues: Ur(Ue, wr, Cr, ht), terrainData: ra };
                            if (xn) {
                              const ya = Tr.segments.get();
                              for (const va of ya) sr.push({ segments: new ee.a0([va]), sortKey: va.sortKey, state: ha });
                            } else sr.push({ segments: Tr.segments, sortKey: 0, state: ha });
                          }
                          xn && sr.sort((Un, wr) => Un.sortKey - wr.sortKey);
                          for (const Un of sr) {
                            const { programConfiguration: wr, program: Cr, layoutVertexBuffer: Tr, indexBuffer: ii, uniformValues: wi, terrainData: Wi } = Un.state, Yi = Un.segments;
                            Cr.draw(An, On.TRIANGLES, Bn, qn, Vn, zl.disabled, wi, Wi, ht.id, Tr, ii, Yi, ht.paint, Ue.transform.zoom, wr);
                          }
                        })(Te, Ae, Oe, Be);
                        break;
                      case "heatmap":
                        ws(Te, Ae, Oe, Be);
                        break;
                      case "line":
                        (function(Ue, nt, ht, Et) {
                          if (Ue.renderPass !== "translucent") return;
                          const Mt = ht.paint.get("line-opacity"), qt = ht.paint.get("line-width");
                          if (Mt.constantOr(1) === 0 || qt.constantOr(1) === 0) return;
                          const tn = Ue.depthModeForSublayer(0, Hs.ReadOnly), xn = Ue.colorModeForRenderPass(), An = ht.paint.get("line-dasharray"), On = ht.paint.get("line-pattern"), Bn = On.constantOr(1), qn = ht.paint.get("line-gradient"), Vn = ht.getCrossfadeParameters(), sr = Bn ? "linePattern" : An ? "lineSDF" : qn ? "lineGradient" : "line", Un = Ue.context, wr = Un.gl;
                          let Cr = !0;
                          for (const Tr of Et) {
                            const ii = nt.getTile(Tr);
                            if (Bn && !ii.patternsLoaded()) continue;
                            const wi = ii.getBucket(ht);
                            if (!wi) continue;
                            const Wi = wi.programConfigurations.get(ht.id), Yi = Ue.context.program.get(), ra = Ue.useProgram(sr, Wi), ha = Cr || ra.program !== Yi, ya = Ue.style.map.terrain && Ue.style.map.terrain.getTerrainData(Tr), va = On.constantOr(null);
                            if (va && ii.imageAtlas) {
                              const Xa = ii.imageAtlas, Oa = Xa.patternPositions[va.to.toString()], Ya = Xa.patternPositions[va.from.toString()];
                              Oa && Ya && Wi.setConstantPatternPositions(Oa, Ya);
                            }
                            const qa = ya ? Tr : null, Aa = Bn ? li(Ue, ii, ht, Vn, qa) : An ? Qi(Ue, ii, ht, An, Vn, qa) : qn ? $r(Ue, ii, ht, wi.lineClipsArray.length, qa) : gr(Ue, ii, ht, qa);
                            if (Bn) Un.activeTexture.set(wr.TEXTURE0), ii.imageAtlasTexture.bind(wr.LINEAR, wr.CLAMP_TO_EDGE), Wi.updatePaintBuffers(Vn);
                            else if (An && (ha || Ue.lineAtlas.dirty)) Un.activeTexture.set(wr.TEXTURE0), Ue.lineAtlas.bind(Un);
                            else if (qn) {
                              const Xa = wi.gradients[ht.id];
                              let Oa = Xa.texture;
                              if (ht.gradientVersion !== Xa.version) {
                                let Ya = 256;
                                if (ht.stepInterpolant) {
                                  const Wo = nt.getSource().maxzoom, Ja = Tr.canonical.z === Wo ? Math.ceil(1 << Ue.transform.maxZoom - Tr.canonical.z) : 1, Uo = wi.maxLineLength / ee.X * 1024 * Ja;
                                  Ya = ee.ad(ee.aV(Uo), 256, Un.maxTextureSize);
                                }
                                Xa.gradient = ee.aW({ expression: ht.gradientExpression(), evaluationKey: "lineProgress", resolution: Ya, image: Xa.gradient || void 0, clips: wi.lineClipsArray }), Xa.texture ? Xa.texture.update(Xa.gradient) : Xa.texture = new Le(Un, Xa.gradient, wr.RGBA), Xa.version = ht.gradientVersion, Oa = Xa.texture;
                              }
                              Un.activeTexture.set(wr.TEXTURE0), Oa.bind(ht.stepInterpolant ? wr.NEAREST : wr.LINEAR, wr.CLAMP_TO_EDGE);
                            }
                            ra.draw(Un, wr.TRIANGLES, tn, Ue.stencilModeForClipping(Tr), xn, zl.disabled, Aa, ya, ht.id, wi.layoutVertexBuffer, wi.indexBuffer, wi.segments, ht.paint, Ue.transform.zoom, Wi, wi.layoutVertexBuffer2), Cr = !1;
                          }
                        })(Te, Ae, Oe, Be);
                        break;
                      case "fill":
                        (function(Ue, nt, ht, Et) {
                          const Mt = ht.paint.get("fill-color"), qt = ht.paint.get("fill-opacity");
                          if (qt.constantOr(1) === 0) return;
                          const tn = Ue.colorModeForRenderPass(), xn = ht.paint.get("fill-pattern"), An = Ue.opaquePassEnabledForLayer() && !xn.constantOr(1) && Mt.constantOr(ee.aN.transparent).a === 1 && qt.constantOr(0) === 1 ? "opaque" : "translucent";
                          if (Ue.renderPass === An) {
                            const On = Ue.depthModeForSublayer(1, Ue.renderPass === "opaque" ? Hs.ReadWrite : Hs.ReadOnly);
                            Yo(Ue, nt, ht, Et, On, tn, !1);
                          }
                          if (Ue.renderPass === "translucent" && ht.paint.get("fill-antialias")) {
                            const On = Ue.depthModeForSublayer(ht.getPaintProperty("fill-outline-color") ? 2 : 0, Hs.ReadOnly);
                            Yo(Ue, nt, ht, Et, On, tn, !0);
                          }
                        })(Te, Ae, Oe, Be);
                        break;
                      case "fill-extrusion":
                        (function(Ue, nt, ht, Et) {
                          const Mt = ht.paint.get("fill-extrusion-opacity");
                          if (Mt !== 0 && Ue.renderPass === "translucent") {
                            const qt = new Hs(Ue.context.gl.LEQUAL, Hs.ReadWrite, Ue.depthRangeFor3D);
                            if (Mt !== 1 || ht.paint.get("fill-extrusion-pattern").constantOr(1)) Ls(Ue, nt, ht, Et, qt, Wl.disabled, Kl.disabled), Ls(Ue, nt, ht, Et, qt, Ue.stencilModeFor3D(), Ue.colorModeForRenderPass());
                            else {
                              const tn = Ue.colorModeForRenderPass();
                              Ls(Ue, nt, ht, Et, qt, Wl.disabled, tn);
                            }
                          }
                        })(Te, Ae, Oe, Be);
                        break;
                      case "hillshade":
                        (function(Ue, nt, ht, Et) {
                          if (Ue.renderPass !== "offscreen" && Ue.renderPass !== "translucent") return;
                          const Mt = Ue.context, qt = Ue.depthModeForSublayer(0, Hs.ReadOnly), tn = Ue.colorModeForRenderPass(), [xn, An] = Ue.renderPass === "translucent" ? Ue.stencilConfigForOverlap(Et) : [{}, Et];
                          for (const On of An) {
                            const Bn = nt.getTile(On);
                            Bn.needsHillshadePrepare !== void 0 && Bn.needsHillshadePrepare && Ue.renderPass === "offscreen" ? vi(Ue, Bn, ht, qt, Wl.disabled, tn) : Ue.renderPass === "translucent" && Ms(Ue, On, Bn, ht, qt, xn[On.overscaledZ], tn);
                          }
                          Mt.viewport.set([0, 0, Ue.width, Ue.height]);
                        })(Te, Ae, Oe, Be);
                        break;
                      case "raster":
                        (function(Ue, nt, ht, Et) {
                          if (Ue.renderPass !== "translucent" || ht.paint.get("raster-opacity") === 0 || !Et.length) return;
                          const Mt = Ue.context, qt = Mt.gl, tn = nt.getSource(), xn = Ue.useProgram("raster"), An = Ue.colorModeForRenderPass(), [On, Bn] = tn instanceof Ft ? [{}, Et] : Ue.stencilConfigForOverlap(Et), qn = Bn[Bn.length - 1].overscaledZ, Vn = !Ue.options.moving;
                          for (const sr of Bn) {
                            const Un = Ue.depthModeForSublayer(sr.overscaledZ - qn, ht.paint.get("raster-opacity") === 1 ? Hs.ReadWrite : Hs.ReadOnly, qt.LESS), wr = nt.getTile(sr);
                            wr.registerFadeDuration(ht.paint.get("raster-fade-duration"));
                            const Cr = nt.findLoadedParent(sr, 0), Tr = nt.findLoadedSibling(sr), ii = Ri(wr, Cr || Tr || null, nt, ht, Ue.transform, Ue.style.map.terrain);
                            let wi, Wi;
                            const Yi = ht.paint.get("raster-resampling") === "nearest" ? qt.NEAREST : qt.LINEAR;
                            Mt.activeTexture.set(qt.TEXTURE0), wr.texture.bind(Yi, qt.CLAMP_TO_EDGE, qt.LINEAR_MIPMAP_NEAREST), Mt.activeTexture.set(qt.TEXTURE1), Cr ? (Cr.texture.bind(Yi, qt.CLAMP_TO_EDGE, qt.LINEAR_MIPMAP_NEAREST), wi = Math.pow(2, Cr.tileID.overscaledZ - wr.tileID.overscaledZ), Wi = [wr.tileID.canonical.x * wi % 1, wr.tileID.canonical.y * wi % 1]) : wr.texture.bind(Yi, qt.CLAMP_TO_EDGE, qt.LINEAR_MIPMAP_NEAREST), wr.texture.useMipmap && Mt.extTextureFilterAnisotropic && Ue.transform.pitch > 20 && qt.texParameterf(qt.TEXTURE_2D, Mt.extTextureFilterAnisotropic.TEXTURE_MAX_ANISOTROPY_EXT, Mt.extTextureFilterAnisotropicMax);
                            const ra = Ue.style.map.terrain && Ue.style.map.terrain.getTerrainData(sr), ha = ra ? sr : null, ya = ha ? ha.posMatrix : Ue.transform.calculatePosMatrix(sr.toUnwrapped(), Vn), va = Ni(ya, Wi || [0, 0], wi || 1, ii, ht);
                            tn instanceof Ft ? xn.draw(Mt, qt.TRIANGLES, Un, Wl.disabled, An, zl.disabled, va, ra, ht.id, tn.boundsBuffer, Ue.quadTriangleIndexBuffer, tn.boundsSegments) : xn.draw(Mt, qt.TRIANGLES, Un, On[sr.overscaledZ], An, zl.disabled, va, ra, ht.id, Ue.rasterBoundsBuffer, Ue.quadTriangleIndexBuffer, Ue.rasterBoundsSegments);
                          }
                        })(Te, Ae, Oe, Be);
                        break;
                      case "background":
                        (function(Ue, nt, ht, Et) {
                          const Mt = ht.paint.get("background-color"), qt = ht.paint.get("background-opacity");
                          if (qt === 0) return;
                          const tn = Ue.context, xn = tn.gl, An = Ue.transform, On = An.tileSize, Bn = ht.paint.get("background-pattern");
                          if (Ue.isPatternMissing(Bn)) return;
                          const qn = !Bn && Mt.a === 1 && qt === 1 && Ue.opaquePassEnabledForLayer() ? "opaque" : "translucent";
                          if (Ue.renderPass !== qn) return;
                          const Vn = Wl.disabled, sr = Ue.depthModeForSublayer(0, qn === "opaque" ? Hs.ReadWrite : Hs.ReadOnly), Un = Ue.colorModeForRenderPass(), wr = Ue.useProgram(Bn ? "backgroundPattern" : "background"), Cr = Et || An.coveringTiles({ tileSize: On, terrain: Ue.style.map.terrain });
                          Bn && (tn.activeTexture.set(xn.TEXTURE0), Ue.imageManager.bind(Ue.context));
                          const Tr = ht.getCrossfadeParameters();
                          for (const ii of Cr) {
                            const wi = Et ? ii.posMatrix : Ue.transform.calculatePosMatrix(ii.toUnwrapped()), Wi = Bn ? ua(wi, qt, Ue, Bn, { tileID: ii, tileSize: On }, Tr) : Vi(wi, qt, Mt), Yi = Ue.style.map.terrain && Ue.style.map.terrain.getTerrainData(ii);
                            wr.draw(tn, xn.TRIANGLES, sr, Vn, Un, zl.disabled, Wi, Yi, ht.id, Ue.tileExtentBuffer, Ue.quadTriangleIndexBuffer, Ue.tileExtentSegments);
                          }
                        })(Te, 0, Oe, Be);
                        break;
                      case "custom":
                        (function(Ue, nt, ht) {
                          const Et = Ue.context, Mt = ht.implementation;
                          if (Ue.renderPass === "offscreen") {
                            const qt = Mt.prerender;
                            qt && (Ue.setCustomLayerDefaults(), Et.setColorMode(Ue.colorModeForRenderPass()), qt.call(Mt, Et.gl, Ue.transform.customLayerMatrix()), Et.setDirty(), Ue.setBaseState());
                          } else if (Ue.renderPass === "translucent") {
                            Ue.setCustomLayerDefaults(), Et.setColorMode(Ue.colorModeForRenderPass()), Et.setStencilMode(Wl.disabled);
                            const qt = Mt.renderingMode === "3d" ? new Hs(Ue.context.gl.LEQUAL, Hs.ReadWrite, Ue.depthRangeFor3D) : Ue.depthModeForSublayer(0, Hs.ReadOnly);
                            Et.setDepthMode(qt), Mt.render(Et.gl, Ue.transform.customLayerMatrix(), { farZ: Ue.transform.farZ, nearZ: Ue.transform.nearZ, fov: Ue.transform._fov, modelViewProjectionMatrix: Ue.transform.modelViewProjectionMatrix, projectionMatrix: Ue.transform.projectionMatrix }), Et.setDirty(), Ue.setBaseState(), Et.bindFramebuffer.set(null);
                          }
                        })(Te, 0, Oe);
                    }
                  }
                  translatePosMatrix(Te, Ae, Oe, Be, Ue) {
                    if (!Oe[0] && !Oe[1]) return Te;
                    const nt = Ue ? Be === "map" ? this.transform.angle : 0 : Be === "viewport" ? -this.transform.angle : 0;
                    if (nt) {
                      const Mt = Math.sin(nt), qt = Math.cos(nt);
                      Oe = [Oe[0] * qt - Oe[1] * Mt, Oe[0] * Mt + Oe[1] * qt];
                    }
                    const ht = [Ue ? Oe[0] : Mn(Ae, Oe[0], this.transform.zoom), Ue ? Oe[1] : Mn(Ae, Oe[1], this.transform.zoom), 0], Et = new Float32Array(16);
                    return ee.J(Et, Te, ht), Et;
                  }
                  saveTileTexture(Te) {
                    const Ae = this._tileTextures[Te.size[0]];
                    Ae ? Ae.push(Te) : this._tileTextures[Te.size[0]] = [Te];
                  }
                  getTileTexture(Te) {
                    const Ae = this._tileTextures[Te];
                    return Ae && Ae.length > 0 ? Ae.pop() : null;
                  }
                  isPatternMissing(Te) {
                    if (!Te) return !1;
                    if (!Te.from || !Te.to) return !0;
                    const Ae = this.imageManager.getPattern(Te.from.toString()), Oe = this.imageManager.getPattern(Te.to.toString());
                    return !Ae || !Oe;
                  }
                  useProgram(Te, Ae) {
                    this.cache = this.cache || {};
                    const Oe = Te + (Ae ? Ae.cacheKey : "") + (this._showOverdrawInspector ? "/overdraw" : "") + (this.style.map.terrain ? "/terrain" : "");
                    return this.cache[Oe] || (this.cache[Oe] = new si(this.context, hi[Te], Ae, Ki[Te], this._showOverdrawInspector, this.style.map.terrain)), this.cache[Oe];
                  }
                  setCustomLayerDefaults() {
                    this.context.unbindVAO(), this.context.cullFace.setDefault(), this.context.activeTexture.setDefault(), this.context.pixelStoreUnpack.setDefault(), this.context.pixelStoreUnpackPremultiplyAlpha.setDefault(), this.context.pixelStoreUnpackFlipY.setDefault();
                  }
                  setBaseState() {
                    const Te = this.context.gl;
                    this.context.cullFace.set(!1), this.context.viewport.set([0, 0, this.width, this.height]), this.context.blendEquation.set(Te.FUNC_ADD);
                  }
                  initDebugOverlayCanvas() {
                    if (this.debugOverlayCanvas == null) {
                      this.debugOverlayCanvas = document.createElement("canvas"), this.debugOverlayCanvas.width = 512, this.debugOverlayCanvas.height = 512;
                      const Te = this.context.gl;
                      this.debugOverlayTexture = new Le(this.context, this.debugOverlayCanvas, Te.RGBA);
                    }
                  }
                  destroy() {
                    this.debugOverlayTexture && this.debugOverlayTexture.destroy();
                  }
                  overLimit() {
                    const { drawingBufferWidth: Te, drawingBufferHeight: Ae } = this.context.gl;
                    return this.width !== Te || this.height !== Ae;
                  }
                }
                class Xo {
                  constructor(Te, Ae) {
                    this.points = Te, this.planes = Ae;
                  }
                  static fromInvProjectionMatrix(Te, Ae, Oe) {
                    const Be = Math.pow(2, Oe), Ue = [[-1, 1, -1, 1], [1, 1, -1, 1], [1, -1, -1, 1], [-1, -1, -1, 1], [-1, 1, 1, 1], [1, 1, 1, 1], [1, -1, 1, 1], [-1, -1, 1, 1]].map((ht) => {
                      const Et = 1 / (ht = ee.ag([], ht, Te))[3] / Ae * Be;
                      return ee.b1(ht, ht, [Et, Et, 1 / ht[3], Et]);
                    }), nt = [[0, 1, 2], [6, 5, 4], [0, 3, 7], [2, 1, 5], [3, 2, 6], [0, 4, 5]].map((ht) => {
                      const Et = function(xn, An) {
                        var On = An[0], Bn = An[1], qn = An[2], Vn = On * On + Bn * Bn + qn * qn;
                        return Vn > 0 && (Vn = 1 / Math.sqrt(Vn)), xn[0] = An[0] * Vn, xn[1] = An[1] * Vn, xn[2] = An[2] * Vn, xn;
                      }([], function(xn, An, On) {
                        var Bn = An[0], qn = An[1], Vn = An[2], sr = On[0], Un = On[1], wr = On[2];
                        return xn[0] = qn * wr - Vn * Un, xn[1] = Vn * sr - Bn * wr, xn[2] = Bn * Un - qn * sr, xn;
                      }([], Ce([], Ue[ht[0]], Ue[ht[1]]), Ce([], Ue[ht[2]], Ue[ht[1]]))), Mt = (qt = Et, tn = Ue[ht[1]], -(qt[0] * tn[0] + qt[1] * tn[1] + qt[2] * tn[2]));
                      var qt, tn;
                      return Et.concat(Mt);
                    });
                    return new Xo(Ue, nt);
                  }
                }
                class Go {
                  constructor(Te, Ae) {
                    this.min = Te, this.max = Ae, this.center = function(Oe, Be, Ue) {
                      return Oe[0] = Be[0] * Ue, Oe[1] = Be[1] * Ue, Oe[2] = Be[2] * Ue, Oe;
                    }([], function(Oe, Be, Ue) {
                      return Oe[0] = Be[0] + Ue[0], Oe[1] = Be[1] + Ue[1], Oe[2] = Be[2] + Ue[2], Oe;
                    }([], this.min, this.max), 0.5);
                  }
                  quadrant(Te) {
                    const Ae = [Te % 2 == 0, Te < 2], Oe = ve(this.min), Be = ve(this.max);
                    for (let Ue = 0; Ue < Ae.length; Ue++) Oe[Ue] = Ae[Ue] ? this.min[Ue] : this.center[Ue], Be[Ue] = Ae[Ue] ? this.center[Ue] : this.max[Ue];
                    return Be[2] = this.max[2], new Go(Oe, Be);
                  }
                  distanceX(Te) {
                    return Math.max(Math.min(this.max[0], Te[0]), this.min[0]) - Te[0];
                  }
                  distanceY(Te) {
                    return Math.max(Math.min(this.max[1], Te[1]), this.min[1]) - Te[1];
                  }
                  intersects(Te) {
                    const Ae = [[this.min[0], this.min[1], this.min[2], 1], [this.max[0], this.min[1], this.min[2], 1], [this.max[0], this.max[1], this.min[2], 1], [this.min[0], this.max[1], this.min[2], 1], [this.min[0], this.min[1], this.max[2], 1], [this.max[0], this.min[1], this.max[2], 1], [this.max[0], this.max[1], this.max[2], 1], [this.min[0], this.max[1], this.max[2], 1]];
                    let Oe = !0;
                    for (let Be = 0; Be < Te.planes.length; Be++) {
                      const Ue = Te.planes[Be];
                      let nt = 0;
                      for (let ht = 0; ht < Ae.length; ht++) ee.b2(Ue, Ae[ht]) >= 0 && nt++;
                      if (nt === 0) return 0;
                      nt !== Ae.length && (Oe = !1);
                    }
                    if (Oe) return 2;
                    for (let Be = 0; Be < 3; Be++) {
                      let Ue = Number.MAX_VALUE, nt = -Number.MAX_VALUE;
                      for (let ht = 0; ht < Te.points.length; ht++) {
                        const Et = Te.points[ht][Be] - this.min[Be];
                        Ue = Math.min(Ue, Et), nt = Math.max(nt, Et);
                      }
                      if (nt < 0 || Ue > this.max[Be] - this.min[Be]) return 0;
                    }
                    return 1;
                  }
                }
                class es {
                  constructor(Te = 0, Ae = 0, Oe = 0, Be = 0) {
                    if (isNaN(Te) || Te < 0 || isNaN(Ae) || Ae < 0 || isNaN(Oe) || Oe < 0 || isNaN(Be) || Be < 0) throw new Error("Invalid value for edge-insets, top, bottom, left and right must all be numbers");
                    this.top = Te, this.bottom = Ae, this.left = Oe, this.right = Be;
                  }
                  interpolate(Te, Ae, Oe) {
                    return Ae.top != null && Te.top != null && (this.top = ee.z.number(Te.top, Ae.top, Oe)), Ae.bottom != null && Te.bottom != null && (this.bottom = ee.z.number(Te.bottom, Ae.bottom, Oe)), Ae.left != null && Te.left != null && (this.left = ee.z.number(Te.left, Ae.left, Oe)), Ae.right != null && Te.right != null && (this.right = ee.z.number(Te.right, Ae.right, Oe)), this;
                  }
                  getCenter(Te, Ae) {
                    const Oe = ee.ad((this.left + Te - this.right) / 2, 0, Te), Be = ee.ad((this.top + Ae - this.bottom) / 2, 0, Ae);
                    return new ee.P(Oe, Be);
                  }
                  equals(Te) {
                    return this.top === Te.top && this.bottom === Te.bottom && this.left === Te.left && this.right === Te.right;
                  }
                  clone() {
                    return new es(this.top, this.bottom, this.left, this.right);
                  }
                  toJSON() {
                    return { top: this.top, bottom: this.bottom, left: this.left, right: this.right };
                  }
                }
                const Ts = 85.051129;
                class Fo {
                  constructor(Te, Ae, Oe, Be, Ue) {
                    this.tileSize = 512, this._renderWorldCopies = Ue === void 0 || !!Ue, this._minZoom = Te || 0, this._maxZoom = Ae || 22, this._minPitch = Oe ?? 0, this._maxPitch = Be ?? 60, this.setMaxBounds(), this.width = 0, this.height = 0, this._center = new ee.N(0, 0), this._elevation = 0, this.zoom = 0, this.angle = 0, this._fov = 0.6435011087932844, this._pitch = 0, this._unmodified = !0, this._edgeInsets = new es(), this._posMatrixCache = {}, this._alignedPosMatrixCache = {}, this._fogMatrixCache = {}, this.minElevationForCurrentTile = 0;
                  }
                  clone() {
                    const Te = new Fo(this._minZoom, this._maxZoom, this._minPitch, this.maxPitch, this._renderWorldCopies);
                    return Te.apply(this), Te;
                  }
                  apply(Te) {
                    this.tileSize = Te.tileSize, this.latRange = Te.latRange, this.width = Te.width, this.height = Te.height, this._center = Te._center, this._elevation = Te._elevation, this.minElevationForCurrentTile = Te.minElevationForCurrentTile, this.zoom = Te.zoom, this.angle = Te.angle, this._fov = Te._fov, this._pitch = Te._pitch, this._unmodified = Te._unmodified, this._edgeInsets = Te._edgeInsets.clone(), this._calcMatrices();
                  }
                  get minZoom() {
                    return this._minZoom;
                  }
                  set minZoom(Te) {
                    this._minZoom !== Te && (this._minZoom = Te, this.zoom = Math.max(this.zoom, Te));
                  }
                  get maxZoom() {
                    return this._maxZoom;
                  }
                  set maxZoom(Te) {
                    this._maxZoom !== Te && (this._maxZoom = Te, this.zoom = Math.min(this.zoom, Te));
                  }
                  get minPitch() {
                    return this._minPitch;
                  }
                  set minPitch(Te) {
                    this._minPitch !== Te && (this._minPitch = Te, this.pitch = Math.max(this.pitch, Te));
                  }
                  get maxPitch() {
                    return this._maxPitch;
                  }
                  set maxPitch(Te) {
                    this._maxPitch !== Te && (this._maxPitch = Te, this.pitch = Math.min(this.pitch, Te));
                  }
                  get renderWorldCopies() {
                    return this._renderWorldCopies;
                  }
                  set renderWorldCopies(Te) {
                    Te === void 0 ? Te = !0 : Te === null && (Te = !1), this._renderWorldCopies = Te;
                  }
                  get worldSize() {
                    return this.tileSize * this.scale;
                  }
                  get centerOffset() {
                    return this.centerPoint._sub(this.size._div(2));
                  }
                  get size() {
                    return new ee.P(this.width, this.height);
                  }
                  get bearing() {
                    return -this.angle / Math.PI * 180;
                  }
                  set bearing(Te) {
                    const Ae = -ee.b3(Te, -180, 180) * Math.PI / 180;
                    this.angle !== Ae && (this._unmodified = !1, this.angle = Ae, this._calcMatrices(), this.rotationMatrix = function() {
                      var Oe = new ee.A(4);
                      return ee.A != Float32Array && (Oe[1] = 0, Oe[2] = 0), Oe[0] = 1, Oe[3] = 1, Oe;
                    }(), function(Oe, Be, Ue) {
                      var nt = Be[0], ht = Be[1], Et = Be[2], Mt = Be[3], qt = Math.sin(Ue), tn = Math.cos(Ue);
                      Oe[0] = nt * tn + Et * qt, Oe[1] = ht * tn + Mt * qt, Oe[2] = nt * -qt + Et * tn, Oe[3] = ht * -qt + Mt * tn;
                    }(this.rotationMatrix, this.rotationMatrix, this.angle));
                  }
                  get pitch() {
                    return this._pitch / Math.PI * 180;
                  }
                  set pitch(Te) {
                    const Ae = ee.ad(Te, this.minPitch, this.maxPitch) / 180 * Math.PI;
                    this._pitch !== Ae && (this._unmodified = !1, this._pitch = Ae, this._calcMatrices());
                  }
                  get fov() {
                    return this._fov / Math.PI * 180;
                  }
                  set fov(Te) {
                    Te = Math.max(0.01, Math.min(60, Te)), this._fov !== Te && (this._unmodified = !1, this._fov = Te / 180 * Math.PI, this._calcMatrices());
                  }
                  get zoom() {
                    return this._zoom;
                  }
                  set zoom(Te) {
                    const Ae = Math.min(Math.max(Te, this.minZoom), this.maxZoom);
                    this._zoom !== Ae && (this._unmodified = !1, this._zoom = Ae, this.tileZoom = Math.max(0, Math.floor(Ae)), this.scale = this.zoomScale(Ae), this._constrain(), this._calcMatrices());
                  }
                  get center() {
                    return this._center;
                  }
                  set center(Te) {
                    Te.lat === this._center.lat && Te.lng === this._center.lng || (this._unmodified = !1, this._center = Te, this._constrain(), this._calcMatrices());
                  }
                  get elevation() {
                    return this._elevation;
                  }
                  set elevation(Te) {
                    Te !== this._elevation && (this._elevation = Te, this._constrain(), this._calcMatrices());
                  }
                  get padding() {
                    return this._edgeInsets.toJSON();
                  }
                  set padding(Te) {
                    this._edgeInsets.equals(Te) || (this._unmodified = !1, this._edgeInsets.interpolate(this._edgeInsets, Te, 1), this._calcMatrices());
                  }
                  get centerPoint() {
                    return this._edgeInsets.getCenter(this.width, this.height);
                  }
                  isPaddingEqual(Te) {
                    return this._edgeInsets.equals(Te);
                  }
                  interpolatePadding(Te, Ae, Oe) {
                    this._unmodified = !1, this._edgeInsets.interpolate(Te, Ae, Oe), this._constrain(), this._calcMatrices();
                  }
                  coveringZoomLevel(Te) {
                    const Ae = (Te.roundZoom ? Math.round : Math.floor)(this.zoom + this.scaleZoom(this.tileSize / Te.tileSize));
                    return Math.max(0, Ae);
                  }
                  getVisibleUnwrappedCoordinates(Te) {
                    const Ae = [new ee.b4(0, Te)];
                    if (this._renderWorldCopies) {
                      const Oe = this.pointCoordinate(new ee.P(0, 0)), Be = this.pointCoordinate(new ee.P(this.width, 0)), Ue = this.pointCoordinate(new ee.P(this.width, this.height)), nt = this.pointCoordinate(new ee.P(0, this.height)), ht = Math.floor(Math.min(Oe.x, Be.x, Ue.x, nt.x)), Et = Math.floor(Math.max(Oe.x, Be.x, Ue.x, nt.x)), Mt = 1;
                      for (let qt = ht - Mt; qt <= Et + Mt; qt++) qt !== 0 && Ae.push(new ee.b4(qt, Te));
                    }
                    return Ae;
                  }
                  coveringTiles(Te) {
                    var Ae, Oe;
                    let Be = this.coveringZoomLevel(Te);
                    const Ue = Be;
                    if (Te.minzoom !== void 0 && Be < Te.minzoom) return [];
                    Te.maxzoom !== void 0 && Be > Te.maxzoom && (Be = Te.maxzoom);
                    const nt = this.pointCoordinate(this.getCameraPoint()), ht = ee.Z.fromLngLat(this.center), Et = Math.pow(2, Be), Mt = [Et * nt.x, Et * nt.y, 0], qt = [Et * ht.x, Et * ht.y, 0], tn = Xo.fromInvProjectionMatrix(this.invModelViewProjectionMatrix, this.worldSize, Be);
                    let xn = Te.minzoom || 0;
                    !Te.terrain && this.pitch <= 60 && this._edgeInsets.top < 0.1 && (xn = Be);
                    const An = Te.terrain ? 2 / Math.min(this.tileSize, Te.tileSize) * this.tileSize : 3, On = (Un) => ({ aabb: new Go([Un * Et, 0, 0], [(Un + 1) * Et, Et, 0]), zoom: 0, x: 0, y: 0, wrap: Un, fullyVisible: !1 }), Bn = [], qn = [], Vn = Be, sr = Te.reparseOverscaled ? Ue : Be;
                    if (this._renderWorldCopies) for (let Un = 1; Un <= 3; Un++) Bn.push(On(-Un)), Bn.push(On(Un));
                    for (Bn.push(On(0)); Bn.length > 0; ) {
                      const Un = Bn.pop(), wr = Un.x, Cr = Un.y;
                      let Tr = Un.fullyVisible;
                      if (!Tr) {
                        const ha = Un.aabb.intersects(tn);
                        if (ha === 0) continue;
                        Tr = ha === 2;
                      }
                      const ii = Te.terrain ? Mt : qt, wi = Un.aabb.distanceX(ii), Wi = Un.aabb.distanceY(ii), Yi = Math.max(Math.abs(wi), Math.abs(Wi)), ra = An + (1 << Vn - Un.zoom) - 2;
                      if (Un.zoom === Vn || Yi > ra && Un.zoom >= xn) {
                        const ha = Vn - Un.zoom, ya = Mt[0] - 0.5 - (wr << ha), va = Mt[1] - 0.5 - (Cr << ha);
                        qn.push({ tileID: new ee.S(Un.zoom === Vn ? sr : Un.zoom, Un.wrap, Un.zoom, wr, Cr), distanceSq: Se([qt[0] - 0.5 - wr, qt[1] - 0.5 - Cr]), tileDistanceToCamera: Math.sqrt(ya * ya + va * va) });
                      } else for (let ha = 0; ha < 4; ha++) {
                        const ya = (wr << 1) + ha % 2, va = (Cr << 1) + (ha >> 1), qa = Un.zoom + 1;
                        let Aa = Un.aabb.quadrant(ha);
                        if (Te.terrain) {
                          const Xa = new ee.S(qa, Un.wrap, qa, ya, va), Oa = Te.terrain.getMinMaxElevation(Xa), Ya = (Ae = Oa.minElevation) !== null && Ae !== void 0 ? Ae : this.elevation, Wo = (Oe = Oa.maxElevation) !== null && Oe !== void 0 ? Oe : this.elevation;
                          Aa = new Go([Aa.min[0], Aa.min[1], Ya], [Aa.max[0], Aa.max[1], Wo]);
                        }
                        Bn.push({ aabb: Aa, zoom: qa, x: ya, y: va, wrap: Un.wrap, fullyVisible: Tr });
                      }
                    }
                    return qn.sort((Un, wr) => Un.distanceSq - wr.distanceSq).map((Un) => Un.tileID);
                  }
                  resize(Te, Ae) {
                    this.width = Te, this.height = Ae, this.pixelsToGLUnits = [2 / Te, -2 / Ae], this._constrain(), this._calcMatrices();
                  }
                  get unmodified() {
                    return this._unmodified;
                  }
                  zoomScale(Te) {
                    return Math.pow(2, Te);
                  }
                  scaleZoom(Te) {
                    return Math.log(Te) / Math.LN2;
                  }
                  project(Te) {
                    const Ae = ee.ad(Te.lat, -85.051129, Ts);
                    return new ee.P(ee.O(Te.lng) * this.worldSize, ee.Q(Ae) * this.worldSize);
                  }
                  unproject(Te) {
                    return new ee.Z(Te.x / this.worldSize, Te.y / this.worldSize).toLngLat();
                  }
                  get point() {
                    return this.project(this.center);
                  }
                  getCameraPosition() {
                    return { lngLat: this.pointLocation(this.getCameraPoint()), altitude: Math.cos(this._pitch) * this.cameraToCenterDistance / this._pixelPerMeter + this.elevation };
                  }
                  recalculateZoom(Te) {
                    const Ae = this.elevation, Oe = Math.cos(this._pitch) * this.cameraToCenterDistance / this._pixelPerMeter, Be = this.pointLocation(this.centerPoint, Te), Ue = Te.getElevationForLngLatZoom(Be, this.tileZoom);
                    if (!(this.elevation - Ue)) return;
                    const nt = Oe + Ae - Ue, ht = Math.cos(this._pitch) * this.cameraToCenterDistance / nt / ee.b5(1, Be.lat) / this.tileSize, Et = this.scaleZoom(ht);
                    this._elevation = Ue, this._center = Be, this.zoom = Et;
                  }
                  setLocationAtPoint(Te, Ae) {
                    const Oe = this.pointCoordinate(Ae), Be = this.pointCoordinate(this.centerPoint), Ue = this.locationCoordinate(Te), nt = new ee.Z(Ue.x - (Oe.x - Be.x), Ue.y - (Oe.y - Be.y));
                    this.center = this.coordinateLocation(nt), this._renderWorldCopies && (this.center = this.center.wrap());
                  }
                  locationPoint(Te, Ae) {
                    return Ae ? this.coordinatePoint(this.locationCoordinate(Te), Ae.getElevationForLngLatZoom(Te, this.tileZoom), this.pixelMatrix3D) : this.coordinatePoint(this.locationCoordinate(Te));
                  }
                  pointLocation(Te, Ae) {
                    return this.coordinateLocation(this.pointCoordinate(Te, Ae));
                  }
                  locationCoordinate(Te) {
                    return ee.Z.fromLngLat(Te);
                  }
                  coordinateLocation(Te) {
                    return Te && Te.toLngLat();
                  }
                  pointCoordinate(Te, Ae) {
                    if (Ae) {
                      const On = Ae.pointCoordinate(Te);
                      if (On != null) return On;
                    }
                    const Oe = [Te.x, Te.y, 0, 1], Be = [Te.x, Te.y, 1, 1];
                    ee.ag(Oe, Oe, this.pixelMatrixInverse), ee.ag(Be, Be, this.pixelMatrixInverse);
                    const Ue = Oe[3], nt = Be[3], ht = Oe[0] / Ue, Et = Be[0] / nt, Mt = Oe[1] / Ue, qt = Be[1] / nt, tn = Oe[2] / Ue, xn = Be[2] / nt, An = tn === xn ? 0 : (0 - tn) / (xn - tn);
                    return new ee.Z(ee.z.number(ht, Et, An) / this.worldSize, ee.z.number(Mt, qt, An) / this.worldSize);
                  }
                  coordinatePoint(Te, Ae = 0, Oe = this.pixelMatrix) {
                    const Be = [Te.x * this.worldSize, Te.y * this.worldSize, Ae, 1];
                    return ee.ag(Be, Be, Oe), new ee.P(Be[0] / Be[3], Be[1] / Be[3]);
                  }
                  getBounds() {
                    const Te = Math.max(0, this.height / 2 - this.getHorizon());
                    return new Rt().extend(this.pointLocation(new ee.P(0, Te))).extend(this.pointLocation(new ee.P(this.width, Te))).extend(this.pointLocation(new ee.P(this.width, this.height))).extend(this.pointLocation(new ee.P(0, this.height)));
                  }
                  getMaxBounds() {
                    return this.latRange && this.latRange.length === 2 && this.lngRange && this.lngRange.length === 2 ? new Rt([this.lngRange[0], this.latRange[0]], [this.lngRange[1], this.latRange[1]]) : null;
                  }
                  getHorizon() {
                    return Math.tan(Math.PI / 2 - this._pitch) * this.cameraToCenterDistance * 0.85;
                  }
                  setMaxBounds(Te) {
                    Te ? (this.lngRange = [Te.getWest(), Te.getEast()], this.latRange = [Te.getSouth(), Te.getNorth()], this._constrain()) : (this.lngRange = null, this.latRange = [-85.051129, Ts]);
                  }
                  calculateTileMatrix(Te) {
                    const Ae = Te.canonical, Oe = this.worldSize / this.zoomScale(Ae.z), Be = Ae.x + Math.pow(2, Ae.z) * Te.wrap, Ue = ee.ao(new Float64Array(16));
                    return ee.J(Ue, Ue, [Be * Oe, Ae.y * Oe, 0]), ee.K(Ue, Ue, [Oe / ee.X, Oe / ee.X, 1]), Ue;
                  }
                  calculatePosMatrix(Te, Ae = !1) {
                    const Oe = Te.key, Be = Ae ? this._alignedPosMatrixCache : this._posMatrixCache;
                    if (Be[Oe]) return Be[Oe];
                    const Ue = this.calculateTileMatrix(Te);
                    return ee.L(Ue, Ae ? this.alignedModelViewProjectionMatrix : this.modelViewProjectionMatrix, Ue), Be[Oe] = new Float32Array(Ue), Be[Oe];
                  }
                  calculateFogMatrix(Te) {
                    const Ae = Te.key, Oe = this._fogMatrixCache;
                    if (Oe[Ae]) return Oe[Ae];
                    const Be = this.calculateTileMatrix(Te);
                    return ee.L(Be, this.fogMatrix, Be), Oe[Ae] = new Float32Array(Be), Oe[Ae];
                  }
                  customLayerMatrix() {
                    return this.mercatorMatrix.slice();
                  }
                  getConstrained(Te, Ae) {
                    Ae = ee.ad(+Ae, this.minZoom, this.maxZoom);
                    const Oe = { center: new ee.N(Te.lng, Te.lat), zoom: Ae };
                    let Be = this.lngRange;
                    !this._renderWorldCopies && Be === null && (Be = [-179.9999999999, 179.9999999999]);
                    const Ue = this.tileSize * this.zoomScale(Oe.zoom);
                    let nt = 0, ht = Ue, Et = 0, Mt = Ue, qt = 0, tn = 0;
                    const { x: xn, y: An } = this.size;
                    if (this.latRange) {
                      const Un = this.latRange;
                      nt = ee.Q(Un[1]) * Ue, ht = ee.Q(Un[0]) * Ue, ht - nt < An && (qt = An / (ht - nt));
                    }
                    Be && (Et = ee.b3(ee.O(Be[0]) * Ue, 0, Ue), Mt = ee.b3(ee.O(Be[1]) * Ue, 0, Ue), Mt < Et && (Mt += Ue), Mt - Et < xn && (tn = xn / (Mt - Et)));
                    const { x: On, y: Bn } = this.project.call({ worldSize: Ue }, Te);
                    let qn, Vn;
                    const sr = Math.max(tn || 0, qt || 0);
                    if (sr) {
                      const Un = new ee.P(tn ? (Mt + Et) / 2 : On, qt ? (ht + nt) / 2 : Bn);
                      return Oe.center = this.unproject.call({ worldSize: Ue }, Un).wrap(), Oe.zoom += this.scaleZoom(sr), Oe;
                    }
                    if (this.latRange) {
                      const Un = An / 2;
                      Bn - Un < nt && (Vn = nt + Un), Bn + Un > ht && (Vn = ht - Un);
                    }
                    if (Be) {
                      const Un = (Et + Mt) / 2;
                      let wr = On;
                      this._renderWorldCopies && (wr = ee.b3(On, Un - Ue / 2, Un + Ue / 2));
                      const Cr = xn / 2;
                      wr - Cr < Et && (qn = Et + Cr), wr + Cr > Mt && (qn = Mt - Cr);
                    }
                    if (qn !== void 0 || Vn !== void 0) {
                      const Un = new ee.P(qn ?? On, Vn ?? Bn);
                      Oe.center = this.unproject.call({ worldSize: Ue }, Un).wrap();
                    }
                    return Oe;
                  }
                  _constrain() {
                    if (!this.center || !this.width || !this.height || this._constraining) return;
                    this._constraining = !0;
                    const Te = this._unmodified, { center: Ae, zoom: Oe } = this.getConstrained(this.center, this.zoom);
                    this.center = Ae, this.zoom = Oe, this._unmodified = Te, this._constraining = !1;
                  }
                  _calcMatrices() {
                    if (!this.height) return;
                    const Te = this._fov / 2, Ae = this.centerOffset, Oe = this.point.x, Be = this.point.y;
                    this.cameraToCenterDistance = 0.5 / Math.tan(Te) * this.height, this._pixelPerMeter = ee.b5(1, this.center.lat) * this.worldSize;
                    let Ue = ee.ao(new Float64Array(16));
                    ee.K(Ue, Ue, [this.width / 2, -this.height / 2, 1]), ee.J(Ue, Ue, [1, -1, 0]), this.labelPlaneMatrix = Ue, Ue = ee.ao(new Float64Array(16)), ee.K(Ue, Ue, [1, -1, 1]), ee.J(Ue, Ue, [-1, -1, 0]), ee.K(Ue, Ue, [2 / this.width, 2 / this.height, 1]), this.glCoordMatrix = Ue;
                    const nt = this.cameraToCenterDistance + this._elevation * this._pixelPerMeter / Math.cos(this._pitch), ht = Math.min(this.elevation, this.minElevationForCurrentTile), Et = nt - ht * this._pixelPerMeter / Math.cos(this._pitch), Mt = ht < 0 ? Et : nt, qt = Math.PI / 2 + this._pitch, tn = this._fov * (0.5 + Ae.y / this.height), xn = Math.sin(tn) * Mt / Math.sin(ee.ad(Math.PI - qt - tn, 0.01, Math.PI - 0.01)), An = this.getHorizon(), On = 2 * Math.atan(An / this.cameraToCenterDistance) * (0.5 + Ae.y / (2 * An)), Bn = Math.sin(On) * Mt / Math.sin(ee.ad(Math.PI - qt - On, 0.01, Math.PI - 0.01)), qn = Math.min(xn, Bn);
                    this.farZ = 1.01 * (Math.cos(Math.PI / 2 - this._pitch) * qn + Mt), this.nearZ = this.height / 50, Ue = new Float64Array(16), ee.b6(Ue, this._fov, this.width / this.height, this.nearZ, this.farZ), Ue[8] = 2 * -Ae.x / this.width, Ue[9] = 2 * Ae.y / this.height, this.projectionMatrix = ee.af(Ue), ee.K(Ue, Ue, [1, -1, 1]), ee.J(Ue, Ue, [0, 0, -this.cameraToCenterDistance]), ee.b7(Ue, Ue, this._pitch), ee.ae(Ue, Ue, this.angle), ee.J(Ue, Ue, [-Oe, -Be, 0]), this.mercatorMatrix = ee.K([], Ue, [this.worldSize, this.worldSize, this.worldSize]), ee.K(Ue, Ue, [1, 1, this._pixelPerMeter]), this.pixelMatrix = ee.L(new Float64Array(16), this.labelPlaneMatrix, Ue), ee.J(Ue, Ue, [0, 0, -this.elevation]), this.modelViewProjectionMatrix = Ue, this.invModelViewProjectionMatrix = ee.at([], Ue), this.fogMatrix = new Float64Array(16), ee.b6(this.fogMatrix, this._fov, this.width / this.height, nt, this.farZ), this.fogMatrix[8] = 2 * -Ae.x / this.width, this.fogMatrix[9] = 2 * Ae.y / this.height, ee.K(this.fogMatrix, this.fogMatrix, [1, -1, 1]), ee.J(this.fogMatrix, this.fogMatrix, [0, 0, -this.cameraToCenterDistance]), ee.b7(this.fogMatrix, this.fogMatrix, this._pitch), ee.ae(this.fogMatrix, this.fogMatrix, this.angle), ee.J(this.fogMatrix, this.fogMatrix, [-Oe, -Be, 0]), ee.K(this.fogMatrix, this.fogMatrix, [1, 1, this._pixelPerMeter]), ee.J(this.fogMatrix, this.fogMatrix, [0, 0, -this.elevation]), this.pixelMatrix3D = ee.L(new Float64Array(16), this.labelPlaneMatrix, Ue);
                    const Vn = this.width % 2 / 2, sr = this.height % 2 / 2, Un = Math.cos(this.angle), wr = Math.sin(this.angle), Cr = Oe - Math.round(Oe) + Un * Vn + wr * sr, Tr = Be - Math.round(Be) + Un * sr + wr * Vn, ii = new Float64Array(Ue);
                    if (ee.J(ii, ii, [Cr > 0.5 ? Cr - 1 : Cr, Tr > 0.5 ? Tr - 1 : Tr, 0]), this.alignedModelViewProjectionMatrix = ii, Ue = ee.at(new Float64Array(16), this.pixelMatrix), !Ue) throw new Error("failed to invert matrix");
                    this.pixelMatrixInverse = Ue, this._posMatrixCache = {}, this._alignedPosMatrixCache = {}, this._fogMatrixCache = {};
                  }
                  maxPitchScaleFactor() {
                    if (!this.pixelMatrixInverse) return 1;
                    const Te = this.pointCoordinate(new ee.P(0, 0)), Ae = [Te.x * this.worldSize, Te.y * this.worldSize, 0, 1];
                    return ee.ag(Ae, Ae, this.pixelMatrix)[3] / this.cameraToCenterDistance;
                  }
                  getCameraPoint() {
                    const Te = this._pitch, Ae = Math.tan(Te) * (this.cameraToCenterDistance || 1);
                    return this.centerPoint.add(new ee.P(0, Ae));
                  }
                  getCameraQueryGeometry(Te) {
                    const Ae = this.getCameraPoint();
                    if (Te.length === 1) return [Te[0], Ae];
                    {
                      let Oe = Ae.x, Be = Ae.y, Ue = Ae.x, nt = Ae.y;
                      for (const ht of Te) Oe = Math.min(Oe, ht.x), Be = Math.min(Be, ht.y), Ue = Math.max(Ue, ht.x), nt = Math.max(nt, ht.y);
                      return [new ee.P(Oe, Be), new ee.P(Ue, Be), new ee.P(Ue, nt), new ee.P(Oe, nt), new ee.P(Oe, Be)];
                    }
                  }
                  lngLatToCameraDepth(Te, Ae) {
                    const Oe = this.locationCoordinate(Te), Be = [Oe.x * this.worldSize, Oe.y * this.worldSize, Ae, 1];
                    return ee.ag(Be, Be, this.modelViewProjectionMatrix), Be[2] / Be[3];
                  }
                }
                function Oo(Fe, Te) {
                  let Ae, Oe = !1, Be = null, Ue = null;
                  const nt = () => {
                    Be = null, Oe && (Fe.apply(Ue, Ae), Be = setTimeout(nt, Te), Oe = !1);
                  };
                  return (...ht) => (Oe = !0, Ue = this, Ae = ht, Be || nt(), Be);
                }
                class gs {
                  constructor(Te) {
                    this._getCurrentHash = () => {
                      const Ae = window.location.hash.replace("#", "");
                      if (this._hashName) {
                        let Oe;
                        return Ae.split("&").map((Be) => Be.split("=")).forEach((Be) => {
                          Be[0] === this._hashName && (Oe = Be);
                        }), (Oe && Oe[1] || "").split("/");
                      }
                      return Ae.split("/");
                    }, this._onHashChange = () => {
                      const Ae = this._getCurrentHash();
                      if (Ae.length >= 3 && !Ae.some((Oe) => isNaN(Oe))) {
                        const Oe = this._map.dragRotate.isEnabled() && this._map.touchZoomRotate.isEnabled() ? +(Ae[3] || 0) : this._map.getBearing();
                        return this._map.jumpTo({ center: [+Ae[2], +Ae[1]], zoom: +Ae[0], bearing: Oe, pitch: +(Ae[4] || 0) }), !0;
                      }
                      return !1;
                    }, this._updateHashUnthrottled = () => {
                      const Ae = window.location.href.replace(/(#.+)?$/, this.getHashString());
                      window.history.replaceState(window.history.state, null, Ae);
                    }, this._removeHash = () => {
                      const Ae = this._getCurrentHash();
                      if (Ae.length === 0) return;
                      const Oe = Ae.join("/");
                      let Be = Oe;
                      Be.split("&").length > 0 && (Be = Be.split("&")[0]), this._hashName && (Be = `${this._hashName}=${Oe}`);
                      let Ue = window.location.hash.replace(Be, "");
                      Ue.startsWith("#&") ? Ue = Ue.slice(0, 1) + Ue.slice(2) : Ue === "#" && (Ue = "");
                      let nt = window.location.href.replace(/(#.+)?$/, Ue);
                      nt = nt.replace("&&", "&"), window.history.replaceState(window.history.state, null, nt);
                    }, this._updateHash = Oo(this._updateHashUnthrottled, 300), this._hashName = Te && encodeURIComponent(Te);
                  }
                  addTo(Te) {
                    return this._map = Te, addEventListener("hashchange", this._onHashChange, !1), this._map.on("moveend", this._updateHash), this;
                  }
                  remove() {
                    return removeEventListener("hashchange", this._onHashChange, !1), this._map.off("moveend", this._updateHash), clearTimeout(this._updateHash()), this._removeHash(), delete this._map, this;
                  }
                  getHashString(Te) {
                    const Ae = this._map.getCenter(), Oe = Math.round(100 * this._map.getZoom()) / 100, Be = Math.ceil((Oe * Math.LN2 + Math.log(512 / 360 / 0.5)) / Math.LN10), Ue = Math.pow(10, Be), nt = Math.round(Ae.lng * Ue) / Ue, ht = Math.round(Ae.lat * Ue) / Ue, Et = this._map.getBearing(), Mt = this._map.getPitch();
                    let qt = "";
                    if (qt += Te ? `/${nt}/${ht}/${Oe}` : `${Oe}/${ht}/${nt}`, (Et || Mt) && (qt += "/" + Math.round(10 * Et) / 10), Mt && (qt += `/${Math.round(Mt)}`), this._hashName) {
                      const tn = this._hashName;
                      let xn = !1;
                      const An = window.location.hash.slice(1).split("&").map((On) => {
                        const Bn = On.split("=")[0];
                        return Bn === tn ? (xn = !0, `${Bn}=${qt}`) : On;
                      }).filter((On) => On);
                      return xn || An.push(`${tn}=${qt}`), `#${An.join("&")}`;
                    }
                    return `#${qt}`;
                  }
                }
                const Bu = { linearity: 0.3, easing: ee.b8(0, 0, 0.3, 1) }, Ql = ee.e({ deceleration: 2500, maxSpeed: 1400 }, Bu), dl = ee.e({ deceleration: 20, maxSpeed: 1400 }, Bu), h1 = ee.e({ deceleration: 1e3, maxSpeed: 360 }, Bu), z1 = ee.e({ deceleration: 1e3, maxSpeed: 90 }, Bu);
                class J1 {
                  constructor(Te) {
                    this._map = Te, this.clear();
                  }
                  clear() {
                    this._inertiaBuffer = [];
                  }
                  record(Te) {
                    this._drainInertiaBuffer(), this._inertiaBuffer.push({ time: ce.now(), settings: Te });
                  }
                  _drainInertiaBuffer() {
                    const Te = this._inertiaBuffer, Ae = ce.now();
                    for (; Te.length > 0 && Ae - Te[0].time > 160; ) Te.shift();
                  }
                  _onMoveEnd(Te) {
                    if (this._drainInertiaBuffer(), this._inertiaBuffer.length < 2) return;
                    const Ae = { zoom: 0, bearing: 0, pitch: 0, pan: new ee.P(0, 0), pinchAround: void 0, around: void 0 };
                    for (const { settings: Ue } of this._inertiaBuffer) Ae.zoom += Ue.zoomDelta || 0, Ae.bearing += Ue.bearingDelta || 0, Ae.pitch += Ue.pitchDelta || 0, Ue.panDelta && Ae.pan._add(Ue.panDelta), Ue.around && (Ae.around = Ue.around), Ue.pinchAround && (Ae.pinchAround = Ue.pinchAround);
                    const Oe = this._inertiaBuffer[this._inertiaBuffer.length - 1].time - this._inertiaBuffer[0].time, Be = {};
                    if (Ae.pan.mag()) {
                      const Ue = M1(Ae.pan.mag(), Oe, ee.e({}, Ql, Te || {}));
                      Be.offset = Ae.pan.mult(Ue.amount / Ae.pan.mag()), Be.center = this._map.transform.center, Lu(Be, Ue);
                    }
                    if (Ae.zoom) {
                      const Ue = M1(Ae.zoom, Oe, dl);
                      Be.zoom = this._map.transform.zoom + Ue.amount, Lu(Be, Ue);
                    }
                    if (Ae.bearing) {
                      const Ue = M1(Ae.bearing, Oe, h1);
                      Be.bearing = this._map.transform.bearing + ee.ad(Ue.amount, -179, 179), Lu(Be, Ue);
                    }
                    if (Ae.pitch) {
                      const Ue = M1(Ae.pitch, Oe, z1);
                      Be.pitch = this._map.transform.pitch + Ue.amount, Lu(Be, Ue);
                    }
                    if (Be.zoom || Be.bearing) {
                      const Ue = Ae.pinchAround === void 0 ? Ae.around : Ae.pinchAround;
                      Be.around = Ue ? this._map.unproject(Ue) : this._map.getCenter();
                    }
                    return this.clear(), ee.e(Be, { noMoveStart: !0 });
                  }
                }
                function Lu(Fe, Te) {
                  (!Fe.duration || Fe.duration < Te.duration) && (Fe.duration = Te.duration, Fe.easing = Te.easing);
                }
                function M1(Fe, Te, Ae) {
                  const { maxSpeed: Oe, linearity: Be, deceleration: Ue } = Ae, nt = ee.ad(Fe * Be / (Te / 1e3), -Oe, Oe), ht = Math.abs(nt) / (Ue * Be);
                  return { easing: Ae.easing, duration: 1e3 * ht, amount: nt * (ht / 2) };
                }
                class ku extends ee.k {
                  preventDefault() {
                    this._defaultPrevented = !0;
                  }
                  get defaultPrevented() {
                    return this._defaultPrevented;
                  }
                  constructor(Te, Ae, Oe, Be = {}) {
                    const Ue = le.mousePos(Ae.getCanvas(), Oe), nt = Ae.unproject(Ue);
                    super(Te, ee.e({ point: Ue, lngLat: nt, originalEvent: Oe }, Be)), this._defaultPrevented = !1, this.target = Ae;
                  }
                }
                class Wu extends ee.k {
                  preventDefault() {
                    this._defaultPrevented = !0;
                  }
                  get defaultPrevented() {
                    return this._defaultPrevented;
                  }
                  constructor(Te, Ae, Oe) {
                    const Be = Te === "touchend" ? Oe.changedTouches : Oe.touches, Ue = le.touchPos(Ae.getCanvasContainer(), Be), nt = Ue.map((Et) => Ae.unproject(Et)), ht = Ue.reduce((Et, Mt, qt, tn) => Et.add(Mt.div(tn.length)), new ee.P(0, 0));
                    super(Te, { points: Ue, point: ht, lngLats: nt, lngLat: Ae.unproject(ht), originalEvent: Oe }), this._defaultPrevented = !1;
                  }
                }
                class T1 extends ee.k {
                  preventDefault() {
                    this._defaultPrevented = !0;
                  }
                  get defaultPrevented() {
                    return this._defaultPrevented;
                  }
                  constructor(Te, Ae, Oe) {
                    super(Te, { originalEvent: Oe }), this._defaultPrevented = !1;
                  }
                }
                class kl {
                  constructor(Te, Ae) {
                    this._map = Te, this._clickTolerance = Ae.clickTolerance;
                  }
                  reset() {
                    delete this._mousedownPos;
                  }
                  wheel(Te) {
                    return this._firePreventable(new T1(Te.type, this._map, Te));
                  }
                  mousedown(Te, Ae) {
                    return this._mousedownPos = Ae, this._firePreventable(new ku(Te.type, this._map, Te));
                  }
                  mouseup(Te) {
                    this._map.fire(new ku(Te.type, this._map, Te));
                  }
                  click(Te, Ae) {
                    this._mousedownPos && this._mousedownPos.dist(Ae) >= this._clickTolerance || this._map.fire(new ku(Te.type, this._map, Te));
                  }
                  dblclick(Te) {
                    return this._firePreventable(new ku(Te.type, this._map, Te));
                  }
                  mouseover(Te) {
                    this._map.fire(new ku(Te.type, this._map, Te));
                  }
                  mouseout(Te) {
                    this._map.fire(new ku(Te.type, this._map, Te));
                  }
                  touchstart(Te) {
                    return this._firePreventable(new Wu(Te.type, this._map, Te));
                  }
                  touchmove(Te) {
                    this._map.fire(new Wu(Te.type, this._map, Te));
                  }
                  touchend(Te) {
                    this._map.fire(new Wu(Te.type, this._map, Te));
                  }
                  touchcancel(Te) {
                    this._map.fire(new Wu(Te.type, this._map, Te));
                  }
                  _firePreventable(Te) {
                    if (this._map.fire(Te), Te.defaultPrevented) return {};
                  }
                  isEnabled() {
                    return !0;
                  }
                  isActive() {
                    return !1;
                  }
                  enable() {
                  }
                  disable() {
                  }
                }
                class Yl {
                  constructor(Te) {
                    this._map = Te;
                  }
                  reset() {
                    this._delayContextMenu = !1, this._ignoreContextMenu = !0, delete this._contextMenuEvent;
                  }
                  mousemove(Te) {
                    this._map.fire(new ku(Te.type, this._map, Te));
                  }
                  mousedown() {
                    this._delayContextMenu = !0, this._ignoreContextMenu = !1;
                  }
                  mouseup() {
                    this._delayContextMenu = !1, this._contextMenuEvent && (this._map.fire(new ku("contextmenu", this._map, this._contextMenuEvent)), delete this._contextMenuEvent);
                  }
                  contextmenu(Te) {
                    this._delayContextMenu ? this._contextMenuEvent = Te : this._ignoreContextMenu || this._map.fire(new ku(Te.type, this._map, Te)), this._map.listens("contextmenu") && Te.preventDefault();
                  }
                  isEnabled() {
                    return !0;
                  }
                  isActive() {
                    return !1;
                  }
                  enable() {
                  }
                  disable() {
                  }
                }
                class i1 {
                  constructor(Te) {
                    this._map = Te;
                  }
                  get transform() {
                    return this._map._requestedCameraState || this._map.transform;
                  }
                  get center() {
                    return { lng: this.transform.center.lng, lat: this.transform.center.lat };
                  }
                  get zoom() {
                    return this.transform.zoom;
                  }
                  get pitch() {
                    return this.transform.pitch;
                  }
                  get bearing() {
                    return this.transform.bearing;
                  }
                  unproject(Te) {
                    return this.transform.pointLocation(ee.P.convert(Te), this._map.terrain);
                  }
                }
                class ic {
                  constructor(Te, Ae) {
                    this._map = Te, this._tr = new i1(Te), this._el = Te.getCanvasContainer(), this._container = Te.getContainer(), this._clickTolerance = Ae.clickTolerance || 1;
                  }
                  isEnabled() {
                    return !!this._enabled;
                  }
                  isActive() {
                    return !!this._active;
                  }
                  enable() {
                    this.isEnabled() || (this._enabled = !0);
                  }
                  disable() {
                    this.isEnabled() && (this._enabled = !1);
                  }
                  mousedown(Te, Ae) {
                    this.isEnabled() && Te.shiftKey && Te.button === 0 && (le.disableDrag(), this._startPos = this._lastPos = Ae, this._active = !0);
                  }
                  mousemoveWindow(Te, Ae) {
                    if (!this._active) return;
                    const Oe = Ae;
                    if (this._lastPos.equals(Oe) || !this._box && Oe.dist(this._startPos) < this._clickTolerance) return;
                    const Be = this._startPos;
                    this._lastPos = Oe, this._box || (this._box = le.create("div", "maplibregl-boxzoom", this._container), this._container.classList.add("maplibregl-crosshair"), this._fireEvent("boxzoomstart", Te));
                    const Ue = Math.min(Be.x, Oe.x), nt = Math.max(Be.x, Oe.x), ht = Math.min(Be.y, Oe.y), Et = Math.max(Be.y, Oe.y);
                    le.setTransform(this._box, `translate(${Ue}px,${ht}px)`), this._box.style.width = nt - Ue + "px", this._box.style.height = Et - ht + "px";
                  }
                  mouseupWindow(Te, Ae) {
                    if (!this._active || Te.button !== 0) return;
                    const Oe = this._startPos, Be = Ae;
                    if (this.reset(), le.suppressClick(), Oe.x !== Be.x || Oe.y !== Be.y) return this._map.fire(new ee.k("boxzoomend", { originalEvent: Te })), { cameraAnimation: (Ue) => Ue.fitScreenCoordinates(Oe, Be, this._tr.bearing, { linear: !0 }) };
                    this._fireEvent("boxzoomcancel", Te);
                  }
                  keydown(Te) {
                    this._active && Te.keyCode === 27 && (this.reset(), this._fireEvent("boxzoomcancel", Te));
                  }
                  reset() {
                    this._active = !1, this._container.classList.remove("maplibregl-crosshair"), this._box && (le.remove(this._box), this._box = null), le.enableDrag(), delete this._startPos, delete this._lastPos;
                  }
                  _fireEvent(Te, Ae) {
                    return this._map.fire(new ee.k(Te, { originalEvent: Ae }));
                  }
                }
                function iu(Fe, Te) {
                  if (Fe.length !== Te.length) throw new Error(`The number of touches and points are not equal - touches ${Fe.length}, points ${Te.length}`);
                  const Ae = {};
                  for (let Oe = 0; Oe < Fe.length; Oe++) Ae[Fe[Oe].identifier] = Te[Oe];
                  return Ae;
                }
                class xu {
                  constructor(Te) {
                    this.reset(), this.numTouches = Te.numTouches;
                  }
                  reset() {
                    delete this.centroid, delete this.startTime, delete this.touches, this.aborted = !1;
                  }
                  touchstart(Te, Ae, Oe) {
                    (this.centroid || Oe.length > this.numTouches) && (this.aborted = !0), this.aborted || (this.startTime === void 0 && (this.startTime = Te.timeStamp), Oe.length === this.numTouches && (this.centroid = function(Be) {
                      const Ue = new ee.P(0, 0);
                      for (const nt of Be) Ue._add(nt);
                      return Ue.div(Be.length);
                    }(Ae), this.touches = iu(Oe, Ae)));
                  }
                  touchmove(Te, Ae, Oe) {
                    if (this.aborted || !this.centroid) return;
                    const Be = iu(Oe, Ae);
                    for (const Ue in this.touches) {
                      const nt = this.touches[Ue], ht = Be[Ue];
                      (!ht || ht.dist(nt) > 30) && (this.aborted = !0);
                    }
                  }
                  touchend(Te, Ae, Oe) {
                    if ((!this.centroid || Te.timeStamp - this.startTime > 500) && (this.aborted = !0), Oe.length === 0) {
                      const Be = !this.aborted && this.centroid;
                      if (this.reset(), Be) return Be;
                    }
                  }
                }
                class vc {
                  constructor(Te) {
                    this.singleTap = new xu(Te), this.numTaps = Te.numTaps, this.reset();
                  }
                  reset() {
                    this.lastTime = 1 / 0, delete this.lastTap, this.count = 0, this.singleTap.reset();
                  }
                  touchstart(Te, Ae, Oe) {
                    this.singleTap.touchstart(Te, Ae, Oe);
                  }
                  touchmove(Te, Ae, Oe) {
                    this.singleTap.touchmove(Te, Ae, Oe);
                  }
                  touchend(Te, Ae, Oe) {
                    const Be = this.singleTap.touchend(Te, Ae, Oe);
                    if (Be) {
                      const Ue = Te.timeStamp - this.lastTime < 500, nt = !this.lastTap || this.lastTap.dist(Be) < 30;
                      if (Ue && nt || this.reset(), this.count++, this.lastTime = Te.timeStamp, this.lastTap = Be, this.count === this.numTaps) return this.reset(), Be;
                    }
                  }
                }
                class Zl {
                  constructor(Te) {
                    this._tr = new i1(Te), this._zoomIn = new vc({ numTouches: 1, numTaps: 2 }), this._zoomOut = new vc({ numTouches: 2, numTaps: 1 }), this.reset();
                  }
                  reset() {
                    this._active = !1, this._zoomIn.reset(), this._zoomOut.reset();
                  }
                  touchstart(Te, Ae, Oe) {
                    this._zoomIn.touchstart(Te, Ae, Oe), this._zoomOut.touchstart(Te, Ae, Oe);
                  }
                  touchmove(Te, Ae, Oe) {
                    this._zoomIn.touchmove(Te, Ae, Oe), this._zoomOut.touchmove(Te, Ae, Oe);
                  }
                  touchend(Te, Ae, Oe) {
                    const Be = this._zoomIn.touchend(Te, Ae, Oe), Ue = this._zoomOut.touchend(Te, Ae, Oe), nt = this._tr;
                    return Be ? (this._active = !0, Te.preventDefault(), setTimeout(() => this.reset(), 0), { cameraAnimation: (ht) => ht.easeTo({ duration: 300, zoom: nt.zoom + 1, around: nt.unproject(Be) }, { originalEvent: Te }) }) : Ue ? (this._active = !0, Te.preventDefault(), setTimeout(() => this.reset(), 0), { cameraAnimation: (ht) => ht.easeTo({ duration: 300, zoom: nt.zoom - 1, around: nt.unproject(Ue) }, { originalEvent: Te }) }) : void 0;
                  }
                  touchcancel() {
                    this.reset();
                  }
                  enable() {
                    this._enabled = !0;
                  }
                  disable() {
                    this._enabled = !1, this.reset();
                  }
                  isEnabled() {
                    return this._enabled;
                  }
                  isActive() {
                    return this._active;
                  }
                }
                class Ec {
                  constructor(Te) {
                    this._enabled = !!Te.enable, this._moveStateManager = Te.moveStateManager, this._clickTolerance = Te.clickTolerance || 1, this._moveFunction = Te.move, this._activateOnStart = !!Te.activateOnStart, Te.assignEvents(this), this.reset();
                  }
                  reset(Te) {
                    this._active = !1, this._moved = !1, delete this._lastPoint, this._moveStateManager.endMove(Te);
                  }
                  _move(...Te) {
                    const Ae = this._moveFunction(...Te);
                    if (Ae.bearingDelta || Ae.pitchDelta || Ae.around || Ae.panDelta) return this._active = !0, Ae;
                  }
                  dragStart(Te, Ae) {
                    this.isEnabled() && !this._lastPoint && this._moveStateManager.isValidStartEvent(Te) && (this._moveStateManager.startMove(Te), this._lastPoint = Ae.length ? Ae[0] : Ae, this._activateOnStart && this._lastPoint && (this._active = !0));
                  }
                  dragMove(Te, Ae) {
                    if (!this.isEnabled()) return;
                    const Oe = this._lastPoint;
                    if (!Oe) return;
                    if (Te.preventDefault(), !this._moveStateManager.isValidMoveEvent(Te)) return void this.reset(Te);
                    const Be = Ae.length ? Ae[0] : Ae;
                    return !this._moved && Be.dist(Oe) < this._clickTolerance ? void 0 : (this._moved = !0, this._lastPoint = Be, this._move(Oe, Be));
                  }
                  dragEnd(Te) {
                    this.isEnabled() && this._lastPoint && this._moveStateManager.isValidEndEvent(Te) && (this._moved && le.suppressClick(), this.reset(Te));
                  }
                  enable() {
                    this._enabled = !0;
                  }
                  disable() {
                    this._enabled = !1, this.reset();
                  }
                  isEnabled() {
                    return this._enabled;
                  }
                  isActive() {
                    return this._active;
                  }
                  getClickTolerance() {
                    return this._clickTolerance;
                  }
                }
                const wu = { 0: 1, 2: 2 };
                class Pc {
                  constructor(Te) {
                    this._correctEvent = Te.checkCorrectEvent;
                  }
                  startMove(Te) {
                    const Ae = le.mouseButton(Te);
                    this._eventButton = Ae;
                  }
                  endMove(Te) {
                    delete this._eventButton;
                  }
                  isValidStartEvent(Te) {
                    return this._correctEvent(Te);
                  }
                  isValidMoveEvent(Te) {
                    return !function(Ae, Oe) {
                      const Be = wu[Oe];
                      return Ae.buttons === void 0 || (Ae.buttons & Be) !== Be;
                    }(Te, this._eventButton);
                  }
                  isValidEndEvent(Te) {
                    return le.mouseButton(Te) === this._eventButton;
                  }
                }
                class du {
                  constructor() {
                    this._firstTouch = void 0;
                  }
                  _isOneFingerTouch(Te) {
                    return Te.targetTouches.length === 1;
                  }
                  _isSameTouchEvent(Te) {
                    return Te.targetTouches[0].identifier === this._firstTouch;
                  }
                  startMove(Te) {
                    const Ae = Te.targetTouches[0].identifier;
                    this._firstTouch = Ae;
                  }
                  endMove(Te) {
                    delete this._firstTouch;
                  }
                  isValidStartEvent(Te) {
                    return this._isOneFingerTouch(Te);
                  }
                  isValidMoveEvent(Te) {
                    return this._isOneFingerTouch(Te) && this._isSameTouchEvent(Te);
                  }
                  isValidEndEvent(Te) {
                    return this._isOneFingerTouch(Te) && this._isSameTouchEvent(Te);
                  }
                }
                const Ac = (Fe) => {
                  Fe.mousedown = Fe.dragStart, Fe.mousemoveWindow = Fe.dragMove, Fe.mouseup = Fe.dragEnd, Fe.contextmenu = (Te) => {
                    Te.preventDefault();
                  };
                }, yu = ({ enable: Fe, clickTolerance: Te, bearingDegreesPerPixelMoved: Ae = 0.8 }) => {
                  const Oe = new Pc({ checkCorrectEvent: (Be) => le.mouseButton(Be) === 0 && Be.ctrlKey || le.mouseButton(Be) === 2 });
                  return new Ec({ clickTolerance: Te, move: (Be, Ue) => ({ bearingDelta: (Ue.x - Be.x) * Ae }), moveStateManager: Oe, enable: Fe, assignEvents: Ac });
                }, au = ({ enable: Fe, clickTolerance: Te, pitchDegreesPerPixelMoved: Ae = -0.5 }) => {
                  const Oe = new Pc({ checkCorrectEvent: (Be) => le.mouseButton(Be) === 0 && Be.ctrlKey || le.mouseButton(Be) === 2 });
                  return new Ec({ clickTolerance: Te, move: (Be, Ue) => ({ pitchDelta: (Ue.y - Be.y) * Ae }), moveStateManager: Oe, enable: Fe, assignEvents: Ac });
                };
                class Qc {
                  constructor(Te, Ae) {
                    this._clickTolerance = Te.clickTolerance || 1, this._map = Ae, this.reset();
                  }
                  reset() {
                    this._active = !1, this._touches = {}, this._sum = new ee.P(0, 0);
                  }
                  _shouldBePrevented(Te) {
                    return Te < (this._map.cooperativeGestures.isEnabled() ? 2 : 1);
                  }
                  touchstart(Te, Ae, Oe) {
                    return this._calculateTransform(Te, Ae, Oe);
                  }
                  touchmove(Te, Ae, Oe) {
                    if (this._active) {
                      if (!this._shouldBePrevented(Oe.length)) return Te.preventDefault(), this._calculateTransform(Te, Ae, Oe);
                      this._map.cooperativeGestures.notifyGestureBlocked("touch_pan", Te);
                    }
                  }
                  touchend(Te, Ae, Oe) {
                    this._calculateTransform(Te, Ae, Oe), this._active && this._shouldBePrevented(Oe.length) && this.reset();
                  }
                  touchcancel() {
                    this.reset();
                  }
                  _calculateTransform(Te, Ae, Oe) {
                    Oe.length > 0 && (this._active = !0);
                    const Be = iu(Oe, Ae), Ue = new ee.P(0, 0), nt = new ee.P(0, 0);
                    let ht = 0;
                    for (const Mt in Be) {
                      const qt = Be[Mt], tn = this._touches[Mt];
                      tn && (Ue._add(qt), nt._add(qt.sub(tn)), ht++, Be[Mt] = qt);
                    }
                    if (this._touches = Be, this._shouldBePrevented(ht) || !nt.mag()) return;
                    const Et = nt.div(ht);
                    return this._sum._add(Et), this._sum.mag() < this._clickTolerance ? void 0 : { around: Ue.div(ht), panDelta: Et };
                  }
                  enable() {
                    this._enabled = !0;
                  }
                  disable() {
                    this._enabled = !1, this.reset();
                  }
                  isEnabled() {
                    return this._enabled;
                  }
                  isActive() {
                    return this._active;
                  }
                }
                class Cc {
                  constructor() {
                    this.reset();
                  }
                  reset() {
                    this._active = !1, delete this._firstTwoTouches;
                  }
                  touchstart(Te, Ae, Oe) {
                    this._firstTwoTouches || Oe.length < 2 || (this._firstTwoTouches = [Oe[0].identifier, Oe[1].identifier], this._start([Ae[0], Ae[1]]));
                  }
                  touchmove(Te, Ae, Oe) {
                    if (!this._firstTwoTouches) return;
                    Te.preventDefault();
                    const [Be, Ue] = this._firstTwoTouches, nt = Ol(Oe, Ae, Be), ht = Ol(Oe, Ae, Ue);
                    if (!nt || !ht) return;
                    const Et = this._aroundCenter ? null : nt.add(ht).div(2);
                    return this._move([nt, ht], Et, Te);
                  }
                  touchend(Te, Ae, Oe) {
                    if (!this._firstTwoTouches) return;
                    const [Be, Ue] = this._firstTwoTouches, nt = Ol(Oe, Ae, Be), ht = Ol(Oe, Ae, Ue);
                    nt && ht || (this._active && le.suppressClick(), this.reset());
                  }
                  touchcancel() {
                    this.reset();
                  }
                  enable(Te) {
                    this._enabled = !0, this._aroundCenter = !!Te && Te.around === "center";
                  }
                  disable() {
                    this._enabled = !1, this.reset();
                  }
                  isEnabled() {
                    return !!this._enabled;
                  }
                  isActive() {
                    return !!this._active;
                  }
                }
                function Ol(Fe, Te, Ae) {
                  for (let Oe = 0; Oe < Fe.length; Oe++) if (Fe[Oe].identifier === Ae) return Te[Oe];
                }
                function uc(Fe, Te) {
                  return Math.log(Fe / Te) / Math.LN2;
                }
                class cs extends Cc {
                  reset() {
                    super.reset(), delete this._distance, delete this._startDistance;
                  }
                  _start(Te) {
                    this._startDistance = this._distance = Te[0].dist(Te[1]);
                  }
                  _move(Te, Ae) {
                    const Oe = this._distance;
                    if (this._distance = Te[0].dist(Te[1]), this._active || !(Math.abs(uc(this._distance, this._startDistance)) < 0.1)) return this._active = !0, { zoomDelta: uc(this._distance, Oe), pinchAround: Ae };
                  }
                }
                function Mu(Fe, Te) {
                  return 180 * Fe.angleWith(Te) / Math.PI;
                }
                class Hu extends Cc {
                  reset() {
                    super.reset(), delete this._minDiameter, delete this._startVector, delete this._vector;
                  }
                  _start(Te) {
                    this._startVector = this._vector = Te[0].sub(Te[1]), this._minDiameter = Te[0].dist(Te[1]);
                  }
                  _move(Te, Ae, Oe) {
                    const Be = this._vector;
                    if (this._vector = Te[0].sub(Te[1]), this._active || !this._isBelowThreshold(this._vector)) return this._active = !0, { bearingDelta: Mu(this._vector, Be), pinchAround: Ae };
                  }
                  _isBelowThreshold(Te) {
                    this._minDiameter = Math.min(this._minDiameter, Te.mag());
                    const Ae = 25 / (Math.PI * this._minDiameter) * 360, Oe = Mu(Te, this._startVector);
                    return Math.abs(Oe) < Ae;
                  }
                }
                function ou(Fe) {
                  return Math.abs(Fe.y) > Math.abs(Fe.x);
                }
                class Gc extends Cc {
                  constructor(Te) {
                    super(), this._currentTouchCount = 0, this._map = Te;
                  }
                  reset() {
                    super.reset(), this._valid = void 0, delete this._firstMove, delete this._lastPoints;
                  }
                  touchstart(Te, Ae, Oe) {
                    super.touchstart(Te, Ae, Oe), this._currentTouchCount = Oe.length;
                  }
                  _start(Te) {
                    this._lastPoints = Te, ou(Te[0].sub(Te[1])) && (this._valid = !1);
                  }
                  _move(Te, Ae, Oe) {
                    if (this._map.cooperativeGestures.isEnabled() && this._currentTouchCount < 3) return;
                    const Be = Te[0].sub(this._lastPoints[0]), Ue = Te[1].sub(this._lastPoints[1]);
                    return this._valid = this.gestureBeginsVertically(Be, Ue, Oe.timeStamp), this._valid ? (this._lastPoints = Te, this._active = !0, { pitchDelta: (Be.y + Ue.y) / 2 * -0.5 }) : void 0;
                  }
                  gestureBeginsVertically(Te, Ae, Oe) {
                    if (this._valid !== void 0) return this._valid;
                    const Be = Te.mag() >= 2, Ue = Ae.mag() >= 2;
                    if (!Be && !Ue) return;
                    if (!Be || !Ue) return this._firstMove === void 0 && (this._firstMove = Oe), Oe - this._firstMove < 100 && void 0;
                    const nt = Te.y > 0 == Ae.y > 0;
                    return ou(Te) && ou(Ae) && nt;
                  }
                }
                const Su = { panStep: 100, bearingStep: 15, pitchStep: 10 };
                class Lc {
                  constructor(Te) {
                    this._tr = new i1(Te);
                    const Ae = Su;
                    this._panStep = Ae.panStep, this._bearingStep = Ae.bearingStep, this._pitchStep = Ae.pitchStep, this._rotationDisabled = !1;
                  }
                  reset() {
                    this._active = !1;
                  }
                  keydown(Te) {
                    if (Te.altKey || Te.ctrlKey || Te.metaKey) return;
                    let Ae = 0, Oe = 0, Be = 0, Ue = 0, nt = 0;
                    switch (Te.keyCode) {
                      case 61:
                      case 107:
                      case 171:
                      case 187:
                        Ae = 1;
                        break;
                      case 189:
                      case 109:
                      case 173:
                        Ae = -1;
                        break;
                      case 37:
                        Te.shiftKey ? Oe = -1 : (Te.preventDefault(), Ue = -1);
                        break;
                      case 39:
                        Te.shiftKey ? Oe = 1 : (Te.preventDefault(), Ue = 1);
                        break;
                      case 38:
                        Te.shiftKey ? Be = 1 : (Te.preventDefault(), nt = -1);
                        break;
                      case 40:
                        Te.shiftKey ? Be = -1 : (Te.preventDefault(), nt = 1);
                        break;
                      default:
                        return;
                    }
                    return this._rotationDisabled && (Oe = 0, Be = 0), { cameraAnimation: (ht) => {
                      const Et = this._tr;
                      ht.easeTo({ duration: 300, easeId: "keyboardHandler", easing: Do, zoom: Ae ? Math.round(Et.zoom) + Ae * (Te.shiftKey ? 2 : 1) : Et.zoom, bearing: Et.bearing + Oe * this._bearingStep, pitch: Et.pitch + Be * this._pitchStep, offset: [-Ue * this._panStep, -nt * this._panStep], center: Et.center }, { originalEvent: Te });
                    } };
                  }
                  enable() {
                    this._enabled = !0;
                  }
                  disable() {
                    this._enabled = !1, this.reset();
                  }
                  isEnabled() {
                    return this._enabled;
                  }
                  isActive() {
                    return this._active;
                  }
                  disableRotation() {
                    this._rotationDisabled = !0;
                  }
                  enableRotation() {
                    this._rotationDisabled = !1;
                  }
                }
                function Do(Fe) {
                  return Fe * (2 - Fe);
                }
                const Ru = 4.000244140625;
                class dc {
                  constructor(Te, Ae) {
                    this._onTimeout = (Oe) => {
                      this._type = "wheel", this._delta -= this._lastValue, this._active || this._start(Oe);
                    }, this._map = Te, this._tr = new i1(Te), this._triggerRenderFrame = Ae, this._delta = 0, this._defaultZoomRate = 0.01, this._wheelZoomRate = 0.0022222222222222222;
                  }
                  setZoomRate(Te) {
                    this._defaultZoomRate = Te;
                  }
                  setWheelZoomRate(Te) {
                    this._wheelZoomRate = Te;
                  }
                  isEnabled() {
                    return !!this._enabled;
                  }
                  isActive() {
                    return !!this._active || this._finishTimeout !== void 0;
                  }
                  isZooming() {
                    return !!this._zooming;
                  }
                  enable(Te) {
                    this.isEnabled() || (this._enabled = !0, this._aroundCenter = !!Te && Te.around === "center");
                  }
                  disable() {
                    this.isEnabled() && (this._enabled = !1);
                  }
                  _shouldBePrevented(Te) {
                    return !!this._map.cooperativeGestures.isEnabled() && !(Te.ctrlKey || this._map.cooperativeGestures.isBypassed(Te));
                  }
                  wheel(Te) {
                    if (!this.isEnabled()) return;
                    if (this._shouldBePrevented(Te)) return void this._map.cooperativeGestures.notifyGestureBlocked("wheel_zoom", Te);
                    let Ae = Te.deltaMode === WheelEvent.DOM_DELTA_LINE ? 40 * Te.deltaY : Te.deltaY;
                    const Oe = ce.now(), Be = Oe - (this._lastWheelEventTime || 0);
                    this._lastWheelEventTime = Oe, Ae !== 0 && Ae % Ru == 0 ? this._type = "wheel" : Ae !== 0 && Math.abs(Ae) < 4 ? this._type = "trackpad" : Be > 400 ? (this._type = null, this._lastValue = Ae, this._timeout = setTimeout(this._onTimeout, 40, Te)) : this._type || (this._type = Math.abs(Be * Ae) < 200 ? "trackpad" : "wheel", this._timeout && (clearTimeout(this._timeout), this._timeout = null, Ae += this._lastValue)), Te.shiftKey && Ae && (Ae /= 4), this._type && (this._lastWheelEvent = Te, this._delta -= Ae, this._active || this._start(Te)), Te.preventDefault();
                  }
                  _start(Te) {
                    if (!this._delta) return;
                    this._frameId && (this._frameId = null), this._active = !0, this.isZooming() || (this._zooming = !0), this._finishTimeout && (clearTimeout(this._finishTimeout), delete this._finishTimeout);
                    const Ae = le.mousePos(this._map.getCanvas(), Te), Oe = this._tr;
                    Ae.y > Oe.transform.height / 2 - Oe.transform.getHorizon() ? this._around = ee.N.convert(this._aroundCenter ? Oe.center : Oe.unproject(Ae)) : this._around = ee.N.convert(Oe.center), this._aroundPoint = Oe.transform.locationPoint(this._around), this._frameId || (this._frameId = !0, this._triggerRenderFrame());
                  }
                  renderFrame() {
                    if (!this._frameId || (this._frameId = null, !this.isActive())) return;
                    const Te = this._tr.transform;
                    if (this._delta !== 0) {
                      const Et = this._type === "wheel" && Math.abs(this._delta) > Ru ? this._wheelZoomRate : this._defaultZoomRate;
                      let Mt = 2 / (1 + Math.exp(-Math.abs(this._delta * Et)));
                      this._delta < 0 && Mt !== 0 && (Mt = 1 / Mt);
                      const qt = typeof this._targetZoom == "number" ? Te.zoomScale(this._targetZoom) : Te.scale;
                      this._targetZoom = Math.min(Te.maxZoom, Math.max(Te.minZoom, Te.scaleZoom(qt * Mt))), this._type === "wheel" && (this._startZoom = Te.zoom, this._easing = this._smoothOutEasing(200)), this._delta = 0;
                    }
                    const Ae = typeof this._targetZoom == "number" ? this._targetZoom : Te.zoom, Oe = this._startZoom, Be = this._easing;
                    let Ue, nt = !1;
                    const ht = ce.now() - this._lastWheelEventTime;
                    if (this._type === "wheel" && Oe && Be && ht) {
                      const Et = Math.min(ht / 200, 1), Mt = Be(Et);
                      Ue = ee.z.number(Oe, Ae, Mt), Et < 1 ? this._frameId || (this._frameId = !0) : nt = !0;
                    } else Ue = Ae, nt = !0;
                    return this._active = !0, nt && (this._active = !1, this._finishTimeout = setTimeout(() => {
                      this._zooming = !1, this._triggerRenderFrame(), delete this._targetZoom, delete this._finishTimeout;
                    }, 200)), { noInertia: !0, needsRenderFrame: !nt, zoomDelta: Ue - Te.zoom, around: this._aroundPoint, originalEvent: this._lastWheelEvent };
                  }
                  _smoothOutEasing(Te) {
                    let Ae = ee.b9;
                    if (this._prevEase) {
                      const Oe = this._prevEase, Be = (ce.now() - Oe.start) / Oe.duration, Ue = Oe.easing(Be + 0.01) - Oe.easing(Be), nt = 0.27 / Math.sqrt(Ue * Ue + 1e-4) * 0.01, ht = Math.sqrt(0.0729 - nt * nt);
                      Ae = ee.b8(nt, ht, 0.25, 1);
                    }
                    return this._prevEase = { start: ce.now(), duration: Te, easing: Ae }, Ae;
                  }
                  reset() {
                    this._active = !1, this._zooming = !1, delete this._targetZoom, this._finishTimeout && (clearTimeout(this._finishTimeout), delete this._finishTimeout);
                  }
                }
                class vl {
                  constructor(Te, Ae) {
                    this._clickZoom = Te, this._tapZoom = Ae;
                  }
                  enable() {
                    this._clickZoom.enable(), this._tapZoom.enable();
                  }
                  disable() {
                    this._clickZoom.disable(), this._tapZoom.disable();
                  }
                  isEnabled() {
                    return this._clickZoom.isEnabled() && this._tapZoom.isEnabled();
                  }
                  isActive() {
                    return this._clickZoom.isActive() || this._tapZoom.isActive();
                  }
                }
                class Eu {
                  constructor(Te) {
                    this._tr = new i1(Te), this.reset();
                  }
                  reset() {
                    this._active = !1;
                  }
                  dblclick(Te, Ae) {
                    return Te.preventDefault(), { cameraAnimation: (Oe) => {
                      Oe.easeTo({ duration: 300, zoom: this._tr.zoom + (Te.shiftKey ? -1 : 1), around: this._tr.unproject(Ae) }, { originalEvent: Te });
                    } };
                  }
                  enable() {
                    this._enabled = !0;
                  }
                  disable() {
                    this._enabled = !1, this.reset();
                  }
                  isEnabled() {
                    return this._enabled;
                  }
                  isActive() {
                    return this._active;
                  }
                }
                class fu {
                  constructor() {
                    this._tap = new vc({ numTouches: 1, numTaps: 1 }), this.reset();
                  }
                  reset() {
                    this._active = !1, delete this._swipePoint, delete this._swipeTouch, delete this._tapTime, delete this._tapPoint, this._tap.reset();
                  }
                  touchstart(Te, Ae, Oe) {
                    if (!this._swipePoint) if (this._tapTime) {
                      const Be = Ae[0], Ue = Te.timeStamp - this._tapTime < 500, nt = this._tapPoint.dist(Be) < 30;
                      Ue && nt ? Oe.length > 0 && (this._swipePoint = Be, this._swipeTouch = Oe[0].identifier) : this.reset();
                    } else this._tap.touchstart(Te, Ae, Oe);
                  }
                  touchmove(Te, Ae, Oe) {
                    if (this._tapTime) {
                      if (this._swipePoint) {
                        if (Oe[0].identifier !== this._swipeTouch) return;
                        const Be = Ae[0], Ue = Be.y - this._swipePoint.y;
                        return this._swipePoint = Be, Te.preventDefault(), this._active = !0, { zoomDelta: Ue / 128 };
                      }
                    } else this._tap.touchmove(Te, Ae, Oe);
                  }
                  touchend(Te, Ae, Oe) {
                    if (this._tapTime) this._swipePoint && Oe.length === 0 && this.reset();
                    else {
                      const Be = this._tap.touchend(Te, Ae, Oe);
                      Be && (this._tapTime = Te.timeStamp, this._tapPoint = Be);
                    }
                  }
                  touchcancel() {
                    this.reset();
                  }
                  enable() {
                    this._enabled = !0;
                  }
                  disable() {
                    this._enabled = !1, this.reset();
                  }
                  isEnabled() {
                    return this._enabled;
                  }
                  isActive() {
                    return this._active;
                  }
                }
                class ac {
                  constructor(Te, Ae, Oe) {
                    this._el = Te, this._mousePan = Ae, this._touchPan = Oe;
                  }
                  enable(Te) {
                    this._inertiaOptions = Te || {}, this._mousePan.enable(), this._touchPan.enable(), this._el.classList.add("maplibregl-touch-drag-pan");
                  }
                  disable() {
                    this._mousePan.disable(), this._touchPan.disable(), this._el.classList.remove("maplibregl-touch-drag-pan");
                  }
                  isEnabled() {
                    return this._mousePan.isEnabled() && this._touchPan.isEnabled();
                  }
                  isActive() {
                    return this._mousePan.isActive() || this._touchPan.isActive();
                  }
                }
                class bc {
                  constructor(Te, Ae, Oe) {
                    this._pitchWithRotate = Te.pitchWithRotate, this._mouseRotate = Ae, this._mousePitch = Oe;
                  }
                  enable() {
                    this._mouseRotate.enable(), this._pitchWithRotate && this._mousePitch.enable();
                  }
                  disable() {
                    this._mouseRotate.disable(), this._mousePitch.disable();
                  }
                  isEnabled() {
                    return this._mouseRotate.isEnabled() && (!this._pitchWithRotate || this._mousePitch.isEnabled());
                  }
                  isActive() {
                    return this._mouseRotate.isActive() || this._mousePitch.isActive();
                  }
                }
                class kc {
                  constructor(Te, Ae, Oe, Be) {
                    this._el = Te, this._touchZoom = Ae, this._touchRotate = Oe, this._tapDragZoom = Be, this._rotationDisabled = !1, this._enabled = !0;
                  }
                  enable(Te) {
                    this._touchZoom.enable(Te), this._rotationDisabled || this._touchRotate.enable(Te), this._tapDragZoom.enable(), this._el.classList.add("maplibregl-touch-zoom-rotate");
                  }
                  disable() {
                    this._touchZoom.disable(), this._touchRotate.disable(), this._tapDragZoom.disable(), this._el.classList.remove("maplibregl-touch-zoom-rotate");
                  }
                  isEnabled() {
                    return this._touchZoom.isEnabled() && (this._rotationDisabled || this._touchRotate.isEnabled()) && this._tapDragZoom.isEnabled();
                  }
                  isActive() {
                    return this._touchZoom.isActive() || this._touchRotate.isActive() || this._tapDragZoom.isActive();
                  }
                  disableRotation() {
                    this._rotationDisabled = !0, this._touchRotate.disable();
                  }
                  enableRotation() {
                    this._rotationDisabled = !1, this._touchZoom.isEnabled() && this._touchRotate.enable();
                  }
                }
                class fc {
                  constructor(Te, Ae) {
                    this._bypassKey = navigator.userAgent.indexOf("Mac") !== -1 ? "metaKey" : "ctrlKey", this._map = Te, this._options = Ae, this._enabled = !1;
                  }
                  isActive() {
                    return !1;
                  }
                  reset() {
                  }
                  _setupUI() {
                    if (this._container) return;
                    const Te = this._map.getCanvasContainer();
                    Te.classList.add("maplibregl-cooperative-gestures"), this._container = le.create("div", "maplibregl-cooperative-gesture-screen", Te);
                    let Ae = this._map._getUIString("CooperativeGesturesHandler.WindowsHelpText");
                    this._bypassKey === "metaKey" && (Ae = this._map._getUIString("CooperativeGesturesHandler.MacHelpText"));
                    const Oe = this._map._getUIString("CooperativeGesturesHandler.MobileHelpText"), Be = document.createElement("div");
                    Be.className = "maplibregl-desktop-message", Be.textContent = Ae, this._container.appendChild(Be);
                    const Ue = document.createElement("div");
                    Ue.className = "maplibregl-mobile-message", Ue.textContent = Oe, this._container.appendChild(Ue), this._container.setAttribute("aria-hidden", "true");
                  }
                  _destroyUI() {
                    this._container && (le.remove(this._container), this._map.getCanvasContainer().classList.remove("maplibregl-cooperative-gestures")), delete this._container;
                  }
                  enable() {
                    this._setupUI(), this._enabled = !0;
                  }
                  disable() {
                    this._enabled = !1, this._destroyUI();
                  }
                  isEnabled() {
                    return this._enabled;
                  }
                  isBypassed(Te) {
                    return Te[this._bypassKey];
                  }
                  notifyGestureBlocked(Te, Ae) {
                    this._enabled && (this._map.fire(new ee.k("cooperativegestureprevented", { gestureType: Te, originalEvent: Ae })), this._container.classList.add("maplibregl-show"), setTimeout(() => {
                      this._container.classList.remove("maplibregl-show");
                    }, 100));
                  }
                }
                const oc = (Fe) => Fe.zoom || Fe.drag || Fe.pitch || Fe.rotate;
                class ss extends ee.k {
                }
                function su(Fe) {
                  return Fe.panDelta && Fe.panDelta.mag() || Fe.zoomDelta || Fe.bearingDelta || Fe.pitchDelta;
                }
                class Tl {
                  constructor(Te, Ae) {
                    this.handleWindowEvent = (Be) => {
                      this.handleEvent(Be, `${Be.type}Window`);
                    }, this.handleEvent = (Be, Ue) => {
                      if (Be.type === "blur") return void this.stop(!0);
                      this._updatingCamera = !0;
                      const nt = Be.type === "renderFrame" ? void 0 : Be, ht = { needsRenderFrame: !1 }, Et = {}, Mt = {}, qt = Be.touches, tn = qt ? this._getMapTouches(qt) : void 0, xn = tn ? le.touchPos(this._map.getCanvas(), tn) : le.mousePos(this._map.getCanvas(), Be);
                      for (const { handlerName: Bn, handler: qn, allowed: Vn } of this._handlers) {
                        if (!qn.isEnabled()) continue;
                        let sr;
                        this._blockedByActive(Mt, Vn, Bn) ? qn.reset() : qn[Ue || Be.type] && (sr = qn[Ue || Be.type](Be, xn, tn), this.mergeHandlerResult(ht, Et, sr, Bn, nt), sr && sr.needsRenderFrame && this._triggerRenderFrame()), (sr || qn.isActive()) && (Mt[Bn] = qn);
                      }
                      const An = {};
                      for (const Bn in this._previousActiveHandlers) Mt[Bn] || (An[Bn] = nt);
                      this._previousActiveHandlers = Mt, (Object.keys(An).length || su(ht)) && (this._changes.push([ht, Et, An]), this._triggerRenderFrame()), (Object.keys(Mt).length || su(ht)) && this._map._stop(!0), this._updatingCamera = !1;
                      const { cameraAnimation: On } = ht;
                      On && (this._inertia.clear(), this._fireEvents({}, {}, !0), this._changes = [], On(this._map));
                    }, this._map = Te, this._el = this._map.getCanvasContainer(), this._handlers = [], this._handlersById = {}, this._changes = [], this._inertia = new J1(Te), this._bearingSnap = Ae.bearingSnap, this._previousActiveHandlers = {}, this._eventsInProgress = {}, this._addDefaultHandlers(Ae);
                    const Oe = this._el;
                    this._listeners = [[Oe, "touchstart", { passive: !0 }], [Oe, "touchmove", { passive: !1 }], [Oe, "touchend", void 0], [Oe, "touchcancel", void 0], [Oe, "mousedown", void 0], [Oe, "mousemove", void 0], [Oe, "mouseup", void 0], [document, "mousemove", { capture: !0 }], [document, "mouseup", void 0], [Oe, "mouseover", void 0], [Oe, "mouseout", void 0], [Oe, "dblclick", void 0], [Oe, "click", void 0], [Oe, "keydown", { capture: !1 }], [Oe, "keyup", void 0], [Oe, "wheel", { passive: !1 }], [Oe, "contextmenu", void 0], [window, "blur", void 0]];
                    for (const [Be, Ue, nt] of this._listeners) le.addEventListener(Be, Ue, Be === document ? this.handleWindowEvent : this.handleEvent, nt);
                  }
                  destroy() {
                    for (const [Te, Ae, Oe] of this._listeners) le.removeEventListener(Te, Ae, Te === document ? this.handleWindowEvent : this.handleEvent, Oe);
                  }
                  _addDefaultHandlers(Te) {
                    const Ae = this._map, Oe = Ae.getCanvasContainer();
                    this._add("mapEvent", new kl(Ae, Te));
                    const Be = Ae.boxZoom = new ic(Ae, Te);
                    this._add("boxZoom", Be), Te.interactive && Te.boxZoom && Be.enable();
                    const Ue = Ae.cooperativeGestures = new fc(Ae, Te.cooperativeGestures);
                    this._add("cooperativeGestures", Ue), Te.cooperativeGestures && Ue.enable();
                    const nt = new Zl(Ae), ht = new Eu(Ae);
                    Ae.doubleClickZoom = new vl(ht, nt), this._add("tapZoom", nt), this._add("clickZoom", ht), Te.interactive && Te.doubleClickZoom && Ae.doubleClickZoom.enable();
                    const Et = new fu();
                    this._add("tapDragZoom", Et);
                    const Mt = Ae.touchPitch = new Gc(Ae);
                    this._add("touchPitch", Mt), Te.interactive && Te.touchPitch && Ae.touchPitch.enable(Te.touchPitch);
                    const qt = yu(Te), tn = au(Te);
                    Ae.dragRotate = new bc(Te, qt, tn), this._add("mouseRotate", qt, ["mousePitch"]), this._add("mousePitch", tn, ["mouseRotate"]), Te.interactive && Te.dragRotate && Ae.dragRotate.enable();
                    const xn = (({ enable: sr, clickTolerance: Un }) => {
                      const wr = new Pc({ checkCorrectEvent: (Cr) => le.mouseButton(Cr) === 0 && !Cr.ctrlKey });
                      return new Ec({ clickTolerance: Un, move: (Cr, Tr) => ({ around: Tr, panDelta: Tr.sub(Cr) }), activateOnStart: !0, moveStateManager: wr, enable: sr, assignEvents: Ac });
                    })(Te), An = new Qc(Te, Ae);
                    Ae.dragPan = new ac(Oe, xn, An), this._add("mousePan", xn), this._add("touchPan", An, ["touchZoom", "touchRotate"]), Te.interactive && Te.dragPan && Ae.dragPan.enable(Te.dragPan);
                    const On = new Hu(), Bn = new cs();
                    Ae.touchZoomRotate = new kc(Oe, Bn, On, Et), this._add("touchRotate", On, ["touchPan", "touchZoom"]), this._add("touchZoom", Bn, ["touchPan", "touchRotate"]), Te.interactive && Te.touchZoomRotate && Ae.touchZoomRotate.enable(Te.touchZoomRotate);
                    const qn = Ae.scrollZoom = new dc(Ae, () => this._triggerRenderFrame());
                    this._add("scrollZoom", qn, ["mousePan"]), Te.interactive && Te.scrollZoom && Ae.scrollZoom.enable(Te.scrollZoom);
                    const Vn = Ae.keyboard = new Lc(Ae);
                    this._add("keyboard", Vn), Te.interactive && Te.keyboard && Ae.keyboard.enable(), this._add("blockableMapEvent", new Yl(Ae));
                  }
                  _add(Te, Ae, Oe) {
                    this._handlers.push({ handlerName: Te, handler: Ae, allowed: Oe }), this._handlersById[Te] = Ae;
                  }
                  stop(Te) {
                    if (!this._updatingCamera) {
                      for (const { handler: Ae } of this._handlers) Ae.reset();
                      this._inertia.clear(), this._fireEvents({}, {}, Te), this._changes = [];
                    }
                  }
                  isActive() {
                    for (const { handler: Te } of this._handlers) if (Te.isActive()) return !0;
                    return !1;
                  }
                  isZooming() {
                    return !!this._eventsInProgress.zoom || this._map.scrollZoom.isZooming();
                  }
                  isRotating() {
                    return !!this._eventsInProgress.rotate;
                  }
                  isMoving() {
                    return !!oc(this._eventsInProgress) || this.isZooming();
                  }
                  _blockedByActive(Te, Ae, Oe) {
                    for (const Be in Te) if (Be !== Oe && (!Ae || Ae.indexOf(Be) < 0)) return !0;
                    return !1;
                  }
                  _getMapTouches(Te) {
                    const Ae = [];
                    for (const Oe of Te) {
                      const Be = Oe.target;
                      this._el.contains(Be) && Ae.push(Oe);
                    }
                    return Ae;
                  }
                  mergeHandlerResult(Te, Ae, Oe, Be, Ue) {
                    if (!Oe) return;
                    ee.e(Te, Oe);
                    const nt = { handlerName: Be, originalEvent: Oe.originalEvent || Ue };
                    Oe.zoomDelta !== void 0 && (Ae.zoom = nt), Oe.panDelta !== void 0 && (Ae.drag = nt), Oe.pitchDelta !== void 0 && (Ae.pitch = nt), Oe.bearingDelta !== void 0 && (Ae.rotate = nt);
                  }
                  _applyChanges() {
                    const Te = {}, Ae = {}, Oe = {};
                    for (const [Be, Ue, nt] of this._changes) Be.panDelta && (Te.panDelta = (Te.panDelta || new ee.P(0, 0))._add(Be.panDelta)), Be.zoomDelta && (Te.zoomDelta = (Te.zoomDelta || 0) + Be.zoomDelta), Be.bearingDelta && (Te.bearingDelta = (Te.bearingDelta || 0) + Be.bearingDelta), Be.pitchDelta && (Te.pitchDelta = (Te.pitchDelta || 0) + Be.pitchDelta), Be.around !== void 0 && (Te.around = Be.around), Be.pinchAround !== void 0 && (Te.pinchAround = Be.pinchAround), Be.noInertia && (Te.noInertia = Be.noInertia), ee.e(Ae, Ue), ee.e(Oe, nt);
                    this._updateMapTransform(Te, Ae, Oe), this._changes = [];
                  }
                  _updateMapTransform(Te, Ae, Oe) {
                    const Be = this._map, Ue = Be._getTransformForUpdate(), nt = Be.terrain;
                    if (!(su(Te) || nt && this._terrainMovement)) return this._fireEvents(Ae, Oe, !0);
                    let { panDelta: ht, zoomDelta: Et, bearingDelta: Mt, pitchDelta: qt, around: tn, pinchAround: xn } = Te;
                    xn !== void 0 && (tn = xn), Be._stop(!0), tn = tn || Be.transform.centerPoint;
                    const An = Ue.pointLocation(ht ? tn.sub(ht) : tn);
                    Mt && (Ue.bearing += Mt), qt && (Ue.pitch += qt), Et && (Ue.zoom += Et), nt ? this._terrainMovement || !Ae.drag && !Ae.zoom ? Ae.drag && this._terrainMovement ? Ue.center = Ue.pointLocation(Ue.centerPoint.sub(ht)) : Ue.setLocationAtPoint(An, tn) : (this._terrainMovement = !0, this._map._elevationFreeze = !0, Ue.setLocationAtPoint(An, tn)) : Ue.setLocationAtPoint(An, tn), Be._applyUpdatedTransform(Ue), this._map._update(), Te.noInertia || this._inertia.record(Te), this._fireEvents(Ae, Oe, !0);
                  }
                  _fireEvents(Te, Ae, Oe) {
                    const Be = oc(this._eventsInProgress), Ue = oc(Te), nt = {};
                    for (const tn in Te) {
                      const { originalEvent: xn } = Te[tn];
                      this._eventsInProgress[tn] || (nt[`${tn}start`] = xn), this._eventsInProgress[tn] = Te[tn];
                    }
                    !Be && Ue && this._fireEvent("movestart", Ue.originalEvent);
                    for (const tn in nt) this._fireEvent(tn, nt[tn]);
                    Ue && this._fireEvent("move", Ue.originalEvent);
                    for (const tn in Te) {
                      const { originalEvent: xn } = Te[tn];
                      this._fireEvent(tn, xn);
                    }
                    const ht = {};
                    let Et;
                    for (const tn in this._eventsInProgress) {
                      const { handlerName: xn, originalEvent: An } = this._eventsInProgress[tn];
                      this._handlersById[xn].isActive() || (delete this._eventsInProgress[tn], Et = Ae[xn] || An, ht[`${tn}end`] = Et);
                    }
                    for (const tn in ht) this._fireEvent(tn, ht[tn]);
                    const Mt = oc(this._eventsInProgress), qt = (Be || Ue) && !Mt;
                    if (qt && this._terrainMovement) {
                      this._map._elevationFreeze = !1, this._terrainMovement = !1;
                      const tn = this._map._getTransformForUpdate();
                      tn.recalculateZoom(this._map.terrain), this._map._applyUpdatedTransform(tn);
                    }
                    if (Oe && qt) {
                      this._updatingCamera = !0;
                      const tn = this._inertia._onMoveEnd(this._map.dragPan._inertiaOptions), xn = (An) => An !== 0 && -this._bearingSnap < An && An < this._bearingSnap;
                      !tn || !tn.essential && ce.prefersReducedMotion ? (this._map.fire(new ee.k("moveend", { originalEvent: Et })), xn(this._map.getBearing()) && this._map.resetNorth()) : (xn(tn.bearing || this._map.getBearing()) && (tn.bearing = 0), tn.freezeElevation = !0, this._map.easeTo(tn, { originalEvent: Et })), this._updatingCamera = !1;
                    }
                  }
                  _fireEvent(Te, Ae) {
                    this._map.fire(new ee.k(Te, Ae ? { originalEvent: Ae } : {}));
                  }
                  _requestFrame() {
                    return this._map.triggerRepaint(), this._map._renderTaskQueue.add((Te) => {
                      delete this._frameId, this.handleEvent(new ss("renderFrame", { timeStamp: Te })), this._applyChanges();
                    });
                  }
                  _triggerRenderFrame() {
                    this._frameId === void 0 && (this._frameId = this._requestFrame());
                  }
                }
                class sc extends ee.E {
                  constructor(Te, Ae) {
                    super(), this._renderFrameCallback = () => {
                      const Oe = Math.min((ce.now() - this._easeStart) / this._easeOptions.duration, 1);
                      this._onEaseFrame(this._easeOptions.easing(Oe)), Oe < 1 && this._easeFrameId ? this._easeFrameId = this._requestRenderFrame(this._renderFrameCallback) : this.stop();
                    }, this._moving = !1, this._zooming = !1, this.transform = Te, this._bearingSnap = Ae.bearingSnap, this.on("moveend", () => {
                      delete this._requestedCameraState;
                    });
                  }
                  getCenter() {
                    return new ee.N(this.transform.center.lng, this.transform.center.lat);
                  }
                  setCenter(Te, Ae) {
                    return this.jumpTo({ center: Te }, Ae);
                  }
                  panBy(Te, Ae, Oe) {
                    return Te = ee.P.convert(Te).mult(-1), this.panTo(this.transform.center, ee.e({ offset: Te }, Ae), Oe);
                  }
                  panTo(Te, Ae, Oe) {
                    return this.easeTo(ee.e({ center: Te }, Ae), Oe);
                  }
                  getZoom() {
                    return this.transform.zoom;
                  }
                  setZoom(Te, Ae) {
                    return this.jumpTo({ zoom: Te }, Ae), this;
                  }
                  zoomTo(Te, Ae, Oe) {
                    return this.easeTo(ee.e({ zoom: Te }, Ae), Oe);
                  }
                  zoomIn(Te, Ae) {
                    return this.zoomTo(this.getZoom() + 1, Te, Ae), this;
                  }
                  zoomOut(Te, Ae) {
                    return this.zoomTo(this.getZoom() - 1, Te, Ae), this;
                  }
                  getBearing() {
                    return this.transform.bearing;
                  }
                  setBearing(Te, Ae) {
                    return this.jumpTo({ bearing: Te }, Ae), this;
                  }
                  getPadding() {
                    return this.transform.padding;
                  }
                  setPadding(Te, Ae) {
                    return this.jumpTo({ padding: Te }, Ae), this;
                  }
                  rotateTo(Te, Ae, Oe) {
                    return this.easeTo(ee.e({ bearing: Te }, Ae), Oe);
                  }
                  resetNorth(Te, Ae) {
                    return this.rotateTo(0, ee.e({ duration: 1e3 }, Te), Ae), this;
                  }
                  resetNorthPitch(Te, Ae) {
                    return this.easeTo(ee.e({ bearing: 0, pitch: 0, duration: 1e3 }, Te), Ae), this;
                  }
                  snapToNorth(Te, Ae) {
                    return Math.abs(this.getBearing()) < this._bearingSnap ? this.resetNorth(Te, Ae) : this;
                  }
                  getPitch() {
                    return this.transform.pitch;
                  }
                  setPitch(Te, Ae) {
                    return this.jumpTo({ pitch: Te }, Ae), this;
                  }
                  cameraForBounds(Te, Ae) {
                    Te = Rt.convert(Te);
                    const Oe = Ae && Ae.bearing || 0;
                    return this._cameraForBoxAndBearing(Te.getNorthWest(), Te.getSouthEast(), Oe, Ae);
                  }
                  _cameraForBoxAndBearing(Te, Ae, Oe, Be) {
                    const Ue = { top: 0, bottom: 0, right: 0, left: 0 };
                    if (typeof (Be = ee.e({ padding: Ue, offset: [0, 0], maxZoom: this.transform.maxZoom }, Be)).padding == "number") {
                      const ya = Be.padding;
                      Be.padding = { top: ya, bottom: ya, right: ya, left: ya };
                    }
                    Be.padding = ee.e(Ue, Be.padding);
                    const nt = this.transform, ht = nt.padding, Et = new Rt(Te, Ae), Mt = nt.project(Et.getNorthWest()), qt = nt.project(Et.getNorthEast()), tn = nt.project(Et.getSouthEast()), xn = nt.project(Et.getSouthWest()), An = ee.ba(-Oe), On = Mt.rotate(An), Bn = qt.rotate(An), qn = tn.rotate(An), Vn = xn.rotate(An), sr = new ee.P(Math.max(On.x, Bn.x, Vn.x, qn.x), Math.max(On.y, Bn.y, Vn.y, qn.y)), Un = new ee.P(Math.min(On.x, Bn.x, Vn.x, qn.x), Math.min(On.y, Bn.y, Vn.y, qn.y)), wr = sr.sub(Un), Cr = (nt.width - (ht.left + ht.right + Be.padding.left + Be.padding.right)) / wr.x, Tr = (nt.height - (ht.top + ht.bottom + Be.padding.top + Be.padding.bottom)) / wr.y;
                    if (Tr < 0 || Cr < 0) return void ee.w("Map cannot fit within canvas with the given bounds, padding, and/or offset.");
                    const ii = Math.min(nt.scaleZoom(nt.scale * Math.min(Cr, Tr)), Be.maxZoom), wi = ee.P.convert(Be.offset), Wi = (Be.padding.left - Be.padding.right) / 2, Yi = (Be.padding.top - Be.padding.bottom) / 2, ra = new ee.P(Wi, Yi).rotate(ee.ba(Oe)), ha = wi.add(ra).mult(nt.scale / nt.zoomScale(ii));
                    return { center: nt.unproject(Mt.add(tn).div(2).sub(ha)), zoom: ii, bearing: Oe };
                  }
                  fitBounds(Te, Ae, Oe) {
                    return this._fitInternal(this.cameraForBounds(Te, Ae), Ae, Oe);
                  }
                  fitScreenCoordinates(Te, Ae, Oe, Be, Ue) {
                    return this._fitInternal(this._cameraForBoxAndBearing(this.transform.pointLocation(ee.P.convert(Te)), this.transform.pointLocation(ee.P.convert(Ae)), Oe, Be), Be, Ue);
                  }
                  _fitInternal(Te, Ae, Oe) {
                    return Te ? (delete (Ae = ee.e(Te, Ae)).padding, Ae.linear ? this.easeTo(Ae, Oe) : this.flyTo(Ae, Oe)) : this;
                  }
                  jumpTo(Te, Ae) {
                    this.stop();
                    const Oe = this._getTransformForUpdate();
                    let Be = !1, Ue = !1, nt = !1;
                    return "zoom" in Te && Oe.zoom !== +Te.zoom && (Be = !0, Oe.zoom = +Te.zoom), Te.center !== void 0 && (Oe.center = ee.N.convert(Te.center)), "bearing" in Te && Oe.bearing !== +Te.bearing && (Ue = !0, Oe.bearing = +Te.bearing), "pitch" in Te && Oe.pitch !== +Te.pitch && (nt = !0, Oe.pitch = +Te.pitch), Te.padding == null || Oe.isPaddingEqual(Te.padding) || (Oe.padding = Te.padding), this._applyUpdatedTransform(Oe), this.fire(new ee.k("movestart", Ae)).fire(new ee.k("move", Ae)), Be && this.fire(new ee.k("zoomstart", Ae)).fire(new ee.k("zoom", Ae)).fire(new ee.k("zoomend", Ae)), Ue && this.fire(new ee.k("rotatestart", Ae)).fire(new ee.k("rotate", Ae)).fire(new ee.k("rotateend", Ae)), nt && this.fire(new ee.k("pitchstart", Ae)).fire(new ee.k("pitch", Ae)).fire(new ee.k("pitchend", Ae)), this.fire(new ee.k("moveend", Ae));
                  }
                  calculateCameraOptionsFromTo(Te, Ae, Oe, Be = 0) {
                    const Ue = ee.Z.fromLngLat(Te, Ae), nt = ee.Z.fromLngLat(Oe, Be), ht = nt.x - Ue.x, Et = nt.y - Ue.y, Mt = nt.z - Ue.z, qt = Math.hypot(ht, Et, Mt);
                    if (qt === 0) throw new Error("Can't calculate camera options with same From and To");
                    const tn = Math.hypot(ht, Et), xn = this.transform.scaleZoom(this.transform.cameraToCenterDistance / qt / this.transform.tileSize), An = 180 * Math.atan2(ht, -Et) / Math.PI;
                    let On = 180 * Math.acos(tn / qt) / Math.PI;
                    return On = Mt < 0 ? 90 - On : 90 + On, { center: nt.toLngLat(), zoom: xn, pitch: On, bearing: An };
                  }
                  easeTo(Te, Ae) {
                    var Oe;
                    this._stop(!1, Te.easeId), ((Te = ee.e({ offset: [0, 0], duration: 500, easing: ee.b9 }, Te)).animate === !1 || !Te.essential && ce.prefersReducedMotion) && (Te.duration = 0);
                    const Be = this._getTransformForUpdate(), Ue = Be.zoom, nt = Be.bearing, ht = Be.pitch, Et = Be.padding, Mt = "bearing" in Te ? this._normalizeBearing(Te.bearing, nt) : nt, qt = "pitch" in Te ? +Te.pitch : ht, tn = "padding" in Te ? Te.padding : Be.padding, xn = ee.P.convert(Te.offset);
                    let An = Be.centerPoint.add(xn);
                    const On = Be.pointLocation(An), { center: Bn, zoom: qn } = Be.getConstrained(ee.N.convert(Te.center || On), (Oe = Te.zoom) !== null && Oe !== void 0 ? Oe : Ue);
                    this._normalizeCenter(Bn, Be);
                    const Vn = Be.project(On), sr = Be.project(Bn).sub(Vn), Un = Be.zoomScale(qn - Ue);
                    let wr, Cr;
                    Te.around && (wr = ee.N.convert(Te.around), Cr = Be.locationPoint(wr));
                    const Tr = { moving: this._moving, zooming: this._zooming, rotating: this._rotating, pitching: this._pitching };
                    return this._zooming = this._zooming || qn !== Ue, this._rotating = this._rotating || nt !== Mt, this._pitching = this._pitching || qt !== ht, this._padding = !Be.isPaddingEqual(tn), this._easeId = Te.easeId, this._prepareEase(Ae, Te.noMoveStart, Tr), this.terrain && this._prepareElevation(Bn), this._ease((ii) => {
                      if (this._zooming && (Be.zoom = ee.z.number(Ue, qn, ii)), this._rotating && (Be.bearing = ee.z.number(nt, Mt, ii)), this._pitching && (Be.pitch = ee.z.number(ht, qt, ii)), this._padding && (Be.interpolatePadding(Et, tn, ii), An = Be.centerPoint.add(xn)), this.terrain && !Te.freezeElevation && this._updateElevation(ii), wr) Be.setLocationAtPoint(wr, Cr);
                      else {
                        const wi = Be.zoomScale(Be.zoom - Ue), Wi = qn > Ue ? Math.min(2, Un) : Math.max(0.5, Un), Yi = Math.pow(Wi, 1 - ii), ra = Be.unproject(Vn.add(sr.mult(ii * Yi)).mult(wi));
                        Be.setLocationAtPoint(Be.renderWorldCopies ? ra.wrap() : ra, An);
                      }
                      this._applyUpdatedTransform(Be), this._fireMoveEvents(Ae);
                    }, (ii) => {
                      this.terrain && Te.freezeElevation && this._finalizeElevation(), this._afterEase(Ae, ii);
                    }, Te), this;
                  }
                  _prepareEase(Te, Ae, Oe = {}) {
                    this._moving = !0, Ae || Oe.moving || this.fire(new ee.k("movestart", Te)), this._zooming && !Oe.zooming && this.fire(new ee.k("zoomstart", Te)), this._rotating && !Oe.rotating && this.fire(new ee.k("rotatestart", Te)), this._pitching && !Oe.pitching && this.fire(new ee.k("pitchstart", Te));
                  }
                  _prepareElevation(Te) {
                    this._elevationCenter = Te, this._elevationStart = this.transform.elevation, this._elevationTarget = this.terrain.getElevationForLngLatZoom(Te, this.transform.tileZoom), this._elevationFreeze = !0;
                  }
                  _updateElevation(Te) {
                    this.transform.minElevationForCurrentTile = this.terrain.getMinTileElevationForLngLatZoom(this._elevationCenter, this.transform.tileZoom);
                    const Ae = this.terrain.getElevationForLngLatZoom(this._elevationCenter, this.transform.tileZoom);
                    if (Te < 1 && Ae !== this._elevationTarget) {
                      const Oe = this._elevationTarget - this._elevationStart, Be = (Ae - (Oe * Te + this._elevationStart)) / (1 - Te);
                      this._elevationStart += Te * (Oe - Be), this._elevationTarget = Ae;
                    }
                    this.transform.elevation = ee.z.number(this._elevationStart, this._elevationTarget, Te);
                  }
                  _finalizeElevation() {
                    this._elevationFreeze = !1, this.transform.recalculateZoom(this.terrain);
                  }
                  _getTransformForUpdate() {
                    return this.transformCameraUpdate || this.terrain ? (this._requestedCameraState || (this._requestedCameraState = this.transform.clone()), this._requestedCameraState) : this.transform;
                  }
                  _elevateCameraIfInsideTerrain(Te) {
                    const Ae = Te.getCameraPosition(), Oe = this.terrain.getElevationForLngLatZoom(Ae.lngLat, Te.zoom);
                    if (Ae.altitude < Oe) {
                      const Be = this.calculateCameraOptionsFromTo(Ae.lngLat, Oe, Te.center, Te.elevation);
                      return { pitch: Be.pitch, zoom: Be.zoom };
                    }
                    return {};
                  }
                  _applyUpdatedTransform(Te) {
                    const Ae = [];
                    if (this.terrain && Ae.push((Be) => this._elevateCameraIfInsideTerrain(Be)), this.transformCameraUpdate && Ae.push((Be) => this.transformCameraUpdate(Be)), !Ae.length) return;
                    const Oe = Te.clone();
                    for (const Be of Ae) {
                      const Ue = Oe.clone(), { center: nt, zoom: ht, pitch: Et, bearing: Mt, elevation: qt } = Be(Ue);
                      nt && (Ue.center = nt), ht !== void 0 && (Ue.zoom = ht), Et !== void 0 && (Ue.pitch = Et), Mt !== void 0 && (Ue.bearing = Mt), qt !== void 0 && (Ue.elevation = qt), Oe.apply(Ue);
                    }
                    this.transform.apply(Oe);
                  }
                  _fireMoveEvents(Te) {
                    this.fire(new ee.k("move", Te)), this._zooming && this.fire(new ee.k("zoom", Te)), this._rotating && this.fire(new ee.k("rotate", Te)), this._pitching && this.fire(new ee.k("pitch", Te));
                  }
                  _afterEase(Te, Ae) {
                    if (this._easeId && Ae && this._easeId === Ae) return;
                    delete this._easeId;
                    const Oe = this._zooming, Be = this._rotating, Ue = this._pitching;
                    this._moving = !1, this._zooming = !1, this._rotating = !1, this._pitching = !1, this._padding = !1, Oe && this.fire(new ee.k("zoomend", Te)), Be && this.fire(new ee.k("rotateend", Te)), Ue && this.fire(new ee.k("pitchend", Te)), this.fire(new ee.k("moveend", Te));
                  }
                  flyTo(Te, Ae) {
                    var Oe;
                    if (!Te.essential && ce.prefersReducedMotion) {
                      const Aa = ee.M(Te, ["center", "zoom", "bearing", "pitch", "around"]);
                      return this.jumpTo(Aa, Ae);
                    }
                    this.stop(), Te = ee.e({ offset: [0, 0], speed: 1.2, curve: 1.42, easing: ee.b9 }, Te);
                    const Be = this._getTransformForUpdate(), Ue = Be.zoom, nt = Be.bearing, ht = Be.pitch, Et = Be.padding, Mt = "bearing" in Te ? this._normalizeBearing(Te.bearing, nt) : nt, qt = "pitch" in Te ? +Te.pitch : ht, tn = "padding" in Te ? Te.padding : Be.padding, xn = ee.P.convert(Te.offset);
                    let An = Be.centerPoint.add(xn);
                    const On = Be.pointLocation(An), { center: Bn, zoom: qn } = Be.getConstrained(ee.N.convert(Te.center || On), (Oe = Te.zoom) !== null && Oe !== void 0 ? Oe : Ue);
                    this._normalizeCenter(Bn, Be);
                    const Vn = Be.zoomScale(qn - Ue), sr = Be.project(On), Un = Be.project(Bn).sub(sr);
                    let wr = Te.curve;
                    const Cr = Math.max(Be.width, Be.height), Tr = Cr / Vn, ii = Un.mag();
                    if ("minZoom" in Te) {
                      const Aa = ee.ad(Math.min(Te.minZoom, Ue, qn), Be.minZoom, Be.maxZoom), Xa = Cr / Be.zoomScale(Aa - Ue);
                      wr = Math.sqrt(Xa / ii * 2);
                    }
                    const wi = wr * wr;
                    function Wi(Aa) {
                      const Xa = (Tr * Tr - Cr * Cr + (Aa ? -1 : 1) * wi * wi * ii * ii) / (2 * (Aa ? Tr : Cr) * wi * ii);
                      return Math.log(Math.sqrt(Xa * Xa + 1) - Xa);
                    }
                    function Yi(Aa) {
                      return (Math.exp(Aa) - Math.exp(-Aa)) / 2;
                    }
                    function ra(Aa) {
                      return (Math.exp(Aa) + Math.exp(-Aa)) / 2;
                    }
                    const ha = Wi(!1);
                    let ya = function(Aa) {
                      return ra(ha) / ra(ha + wr * Aa);
                    }, va = function(Aa) {
                      return Cr * ((ra(ha) * (Yi(Xa = ha + wr * Aa) / ra(Xa)) - Yi(ha)) / wi) / ii;
                      var Xa;
                    }, qa = (Wi(!0) - ha) / wr;
                    if (Math.abs(ii) < 1e-6 || !isFinite(qa)) {
                      if (Math.abs(Cr - Tr) < 1e-6) return this.easeTo(Te, Ae);
                      const Aa = Tr < Cr ? -1 : 1;
                      qa = Math.abs(Math.log(Tr / Cr)) / wr, va = () => 0, ya = (Xa) => Math.exp(Aa * wr * Xa);
                    }
                    if ("duration" in Te) Te.duration = +Te.duration;
                    else {
                      const Aa = "screenSpeed" in Te ? +Te.screenSpeed / wr : +Te.speed;
                      Te.duration = 1e3 * qa / Aa;
                    }
                    return Te.maxDuration && Te.duration > Te.maxDuration && (Te.duration = 0), this._zooming = !0, this._rotating = nt !== Mt, this._pitching = qt !== ht, this._padding = !Be.isPaddingEqual(tn), this._prepareEase(Ae, !1), this.terrain && this._prepareElevation(Bn), this._ease((Aa) => {
                      const Xa = Aa * qa, Oa = 1 / ya(Xa);
                      Be.zoom = Aa === 1 ? qn : Ue + Be.scaleZoom(Oa), this._rotating && (Be.bearing = ee.z.number(nt, Mt, Aa)), this._pitching && (Be.pitch = ee.z.number(ht, qt, Aa)), this._padding && (Be.interpolatePadding(Et, tn, Aa), An = Be.centerPoint.add(xn)), this.terrain && !Te.freezeElevation && this._updateElevation(Aa);
                      const Ya = Aa === 1 ? Bn : Be.unproject(sr.add(Un.mult(va(Xa))).mult(Oa));
                      Be.setLocationAtPoint(Be.renderWorldCopies ? Ya.wrap() : Ya, An), this._applyUpdatedTransform(Be), this._fireMoveEvents(Ae);
                    }, () => {
                      this.terrain && Te.freezeElevation && this._finalizeElevation(), this._afterEase(Ae);
                    }, Te), this;
                  }
                  isEasing() {
                    return !!this._easeFrameId;
                  }
                  stop() {
                    return this._stop();
                  }
                  _stop(Te, Ae) {
                    var Oe;
                    if (this._easeFrameId && (this._cancelRenderFrame(this._easeFrameId), delete this._easeFrameId, delete this._onEaseFrame), this._onEaseEnd) {
                      const Be = this._onEaseEnd;
                      delete this._onEaseEnd, Be.call(this, Ae);
                    }
                    return Te || (Oe = this.handlers) === null || Oe === void 0 || Oe.stop(!1), this;
                  }
                  _ease(Te, Ae, Oe) {
                    Oe.animate === !1 || Oe.duration === 0 ? (Te(1), Ae()) : (this._easeStart = ce.now(), this._easeOptions = Oe, this._onEaseFrame = Te, this._onEaseEnd = Ae, this._easeFrameId = this._requestRenderFrame(this._renderFrameCallback));
                  }
                  _normalizeBearing(Te, Ae) {
                    Te = ee.b3(Te, -180, 180);
                    const Oe = Math.abs(Te - Ae);
                    return Math.abs(Te - 360 - Ae) < Oe && (Te -= 360), Math.abs(Te + 360 - Ae) < Oe && (Te += 360), Te;
                  }
                  _normalizeCenter(Te, Ae) {
                    if (!Ae.renderWorldCopies || Ae.lngRange) return;
                    const Oe = Te.lng - Ae.center.lng;
                    Te.lng += Oe > 180 ? -360 : Oe < -180 ? 360 : 0;
                  }
                  queryTerrainElevation(Te) {
                    return this.terrain ? this.terrain.getElevationForLngLatZoom(ee.N.convert(Te), this.transform.tileZoom) - this.transform.elevation : null;
                  }
                }
                const hc = { compact: !0, customAttribution: '<a href="https://maplibre.org/" target="_blank">MapLibre</a>' };
                class Vc {
                  constructor(Te = hc) {
                    this._toggleAttribution = () => {
                      this._container.classList.contains("maplibregl-compact") && (this._container.classList.contains("maplibregl-compact-show") ? (this._container.setAttribute("open", ""), this._container.classList.remove("maplibregl-compact-show")) : (this._container.classList.add("maplibregl-compact-show"), this._container.removeAttribute("open")));
                    }, this._updateData = (Ae) => {
                      !Ae || Ae.sourceDataType !== "metadata" && Ae.sourceDataType !== "visibility" && Ae.dataType !== "style" && Ae.type !== "terrain" || this._updateAttributions();
                    }, this._updateCompact = () => {
                      this._map.getCanvasContainer().offsetWidth <= 640 || this._compact ? this._compact === !1 ? this._container.setAttribute("open", "") : this._container.classList.contains("maplibregl-compact") || this._container.classList.contains("maplibregl-attrib-empty") || (this._container.setAttribute("open", ""), this._container.classList.add("maplibregl-compact", "maplibregl-compact-show")) : (this._container.setAttribute("open", ""), this._container.classList.contains("maplibregl-compact") && this._container.classList.remove("maplibregl-compact", "maplibregl-compact-show"));
                    }, this._updateCompactMinimize = () => {
                      this._container.classList.contains("maplibregl-compact") && this._container.classList.contains("maplibregl-compact-show") && this._container.classList.remove("maplibregl-compact-show");
                    }, this.options = Te;
                  }
                  getDefaultPosition() {
                    return "bottom-right";
                  }
                  onAdd(Te) {
                    return this._map = Te, this._compact = this.options.compact, this._container = le.create("details", "maplibregl-ctrl maplibregl-ctrl-attrib"), this._compactButton = le.create("summary", "maplibregl-ctrl-attrib-button", this._container), this._compactButton.addEventListener("click", this._toggleAttribution), this._setElementTitle(this._compactButton, "ToggleAttribution"), this._innerContainer = le.create("div", "maplibregl-ctrl-attrib-inner", this._container), this._updateAttributions(), this._updateCompact(), this._map.on("styledata", this._updateData), this._map.on("sourcedata", this._updateData), this._map.on("terrain", this._updateData), this._map.on("resize", this._updateCompact), this._map.on("drag", this._updateCompactMinimize), this._container;
                  }
                  onRemove() {
                    le.remove(this._container), this._map.off("styledata", this._updateData), this._map.off("sourcedata", this._updateData), this._map.off("terrain", this._updateData), this._map.off("resize", this._updateCompact), this._map.off("drag", this._updateCompactMinimize), this._map = void 0, this._compact = void 0, this._attribHTML = void 0;
                  }
                  _setElementTitle(Te, Ae) {
                    const Oe = this._map._getUIString(`AttributionControl.${Ae}`);
                    Te.title = Oe, Te.setAttribute("aria-label", Oe);
                  }
                  _updateAttributions() {
                    if (!this._map.style) return;
                    let Te = [];
                    if (this.options.customAttribution && (Array.isArray(this.options.customAttribution) ? Te = Te.concat(this.options.customAttribution.map((Be) => typeof Be != "string" ? "" : Be)) : typeof this.options.customAttribution == "string" && Te.push(this.options.customAttribution)), this._map.style.stylesheet) {
                      const Be = this._map.style.stylesheet;
                      this.styleOwner = Be.owner, this.styleId = Be.id;
                    }
                    const Ae = this._map.style.sourceCaches;
                    for (const Be in Ae) {
                      const Ue = Ae[Be];
                      if (Ue.used || Ue.usedForTerrain) {
                        const nt = Ue.getSource();
                        nt.attribution && Te.indexOf(nt.attribution) < 0 && Te.push(nt.attribution);
                      }
                    }
                    Te = Te.filter((Be) => String(Be).trim()), Te.sort((Be, Ue) => Be.length - Ue.length), Te = Te.filter((Be, Ue) => {
                      for (let nt = Ue + 1; nt < Te.length; nt++) if (Te[nt].indexOf(Be) >= 0) return !1;
                      return !0;
                    });
                    const Oe = Te.join(" | ");
                    Oe !== this._attribHTML && (this._attribHTML = Oe, Te.length ? (this._innerContainer.innerHTML = Oe, this._container.classList.remove("maplibregl-attrib-empty")) : this._container.classList.add("maplibregl-attrib-empty"), this._updateCompact(), this._editLink = null);
                  }
                }
                class Rc {
                  constructor(Te = {}) {
                    this._updateCompact = () => {
                      const Ae = this._container.children;
                      if (Ae.length) {
                        const Oe = Ae[0];
                        this._map.getCanvasContainer().offsetWidth <= 640 || this._compact ? this._compact !== !1 && Oe.classList.add("maplibregl-compact") : Oe.classList.remove("maplibregl-compact");
                      }
                    }, this.options = Te;
                  }
                  getDefaultPosition() {
                    return "bottom-left";
                  }
                  onAdd(Te) {
                    this._map = Te, this._compact = this.options && this.options.compact, this._container = le.create("div", "maplibregl-ctrl");
                    const Ae = le.create("a", "maplibregl-ctrl-logo");
                    return Ae.target = "_blank", Ae.rel = "noopener nofollow", Ae.href = "https://maplibre.org/", Ae.setAttribute("aria-label", this._map._getUIString("LogoControl.Title")), Ae.setAttribute("rel", "noopener nofollow"), this._container.appendChild(Ae), this._container.style.display = "block", this._map.on("resize", this._updateCompact), this._updateCompact(), this._container;
                  }
                  onRemove() {
                    le.remove(this._container), this._map.off("resize", this._updateCompact), this._map = void 0, this._compact = void 0;
                  }
                }
                class Zo {
                  constructor() {
                    this._queue = [], this._id = 0, this._cleared = !1, this._currentlyRunning = !1;
                  }
                  add(Te) {
                    const Ae = ++this._id;
                    return this._queue.push({ callback: Te, id: Ae, cancelled: !1 }), Ae;
                  }
                  remove(Te) {
                    const Ae = this._currentlyRunning, Oe = Ae ? this._queue.concat(Ae) : this._queue;
                    for (const Be of Oe) if (Be.id === Te) return void (Be.cancelled = !0);
                  }
                  run(Te = 0) {
                    if (this._currentlyRunning) throw new Error("Attempting to run(), but is already running.");
                    const Ae = this._currentlyRunning = this._queue;
                    this._queue = [];
                    for (const Oe of Ae) if (!Oe.cancelled && (Oe.callback(Te), this._cleared)) break;
                    this._cleared = !1, this._currentlyRunning = !1;
                  }
                  clear() {
                    this._currentlyRunning && (this._cleared = !0), this._queue = [];
                  }
                }
                var Ou = ee.Y([{ name: "a_pos3d", type: "Int16", components: 3 }]);
                class Uc extends ee.E {
                  constructor(Te) {
                    super(), this.sourceCache = Te, this._tiles = {}, this._renderableTilesKeys = [], this._sourceTileCache = {}, this.minzoom = 0, this.maxzoom = 22, this.tileSize = 512, this.deltaZoom = 1, Te.usedForTerrain = !0, Te.tileSize = this.tileSize * 2 ** this.deltaZoom;
                  }
                  destruct() {
                    this.sourceCache.usedForTerrain = !1, this.sourceCache.tileSize = null;
                  }
                  update(Te, Ae) {
                    this.sourceCache.update(Te, Ae), this._renderableTilesKeys = [];
                    const Oe = {};
                    for (const Be of Te.coveringTiles({ tileSize: this.tileSize, minzoom: this.minzoom, maxzoom: this.maxzoom, reparseOverscaled: !1, terrain: Ae })) Oe[Be.key] = !0, this._renderableTilesKeys.push(Be.key), this._tiles[Be.key] || (Be.posMatrix = new Float64Array(16), ee.aQ(Be.posMatrix, 0, ee.X, 0, ee.X, 0, 1), this._tiles[Be.key] = new Qt(Be, this.tileSize));
                    for (const Be in this._tiles) Oe[Be] || delete this._tiles[Be];
                  }
                  freeRtt(Te) {
                    for (const Ae in this._tiles) {
                      const Oe = this._tiles[Ae];
                      (!Te || Oe.tileID.equals(Te) || Oe.tileID.isChildOf(Te) || Te.isChildOf(Oe.tileID)) && (Oe.rtt = []);
                    }
                  }
                  getRenderableTiles() {
                    return this._renderableTilesKeys.map((Te) => this.getTileByID(Te));
                  }
                  getTileByID(Te) {
                    return this._tiles[Te];
                  }
                  getTerrainCoords(Te) {
                    const Ae = {};
                    for (const Oe of this._renderableTilesKeys) {
                      const Be = this._tiles[Oe].tileID;
                      if (Be.canonical.equals(Te.canonical)) {
                        const Ue = Te.clone();
                        Ue.posMatrix = new Float64Array(16), ee.aQ(Ue.posMatrix, 0, ee.X, 0, ee.X, 0, 1), Ae[Oe] = Ue;
                      } else if (Be.canonical.isChildOf(Te.canonical)) {
                        const Ue = Te.clone();
                        Ue.posMatrix = new Float64Array(16);
                        const nt = Be.canonical.z - Te.canonical.z, ht = Be.canonical.x - (Be.canonical.x >> nt << nt), Et = Be.canonical.y - (Be.canonical.y >> nt << nt), Mt = ee.X >> nt;
                        ee.aQ(Ue.posMatrix, 0, Mt, 0, Mt, 0, 1), ee.J(Ue.posMatrix, Ue.posMatrix, [-ht * Mt, -Et * Mt, 0]), Ae[Oe] = Ue;
                      } else if (Te.canonical.isChildOf(Be.canonical)) {
                        const Ue = Te.clone();
                        Ue.posMatrix = new Float64Array(16);
                        const nt = Te.canonical.z - Be.canonical.z, ht = Te.canonical.x - (Te.canonical.x >> nt << nt), Et = Te.canonical.y - (Te.canonical.y >> nt << nt), Mt = ee.X >> nt;
                        ee.aQ(Ue.posMatrix, 0, ee.X, 0, ee.X, 0, 1), ee.J(Ue.posMatrix, Ue.posMatrix, [ht * Mt, Et * Mt, 0]), ee.K(Ue.posMatrix, Ue.posMatrix, [1 / 2 ** nt, 1 / 2 ** nt, 0]), Ae[Oe] = Ue;
                      }
                    }
                    return Ae;
                  }
                  getSourceTile(Te, Ae) {
                    const Oe = this.sourceCache._source;
                    let Be = Te.overscaledZ - this.deltaZoom;
                    if (Be > Oe.maxzoom && (Be = Oe.maxzoom), Be < Oe.minzoom) return null;
                    this._sourceTileCache[Te.key] || (this._sourceTileCache[Te.key] = Te.scaledTo(Be).key);
                    let Ue = this.sourceCache.getTileByID(this._sourceTileCache[Te.key]);
                    if ((!Ue || !Ue.dem) && Ae) for (; Be >= Oe.minzoom && (!Ue || !Ue.dem); ) Ue = this.sourceCache.getTileByID(Te.scaledTo(Be--).key);
                    return Ue;
                  }
                  tilesAfterTime(Te = Date.now()) {
                    return Object.values(this._tiles).filter((Ae) => Ae.timeAdded >= Te);
                  }
                }
                class El {
                  constructor(Te, Ae, Oe) {
                    this.painter = Te, this.sourceCache = new Uc(Ae), this.options = Oe, this.exaggeration = typeof Oe.exaggeration == "number" ? Oe.exaggeration : 1, this.qualityFactor = 2, this.meshSize = 128, this._demMatrixCache = {}, this.coordsIndex = [], this._coordsTextureSize = 1024;
                  }
                  getDEMElevation(Te, Ae, Oe, Be = ee.X) {
                    var Ue;
                    if (!(Ae >= 0 && Ae < Be && Oe >= 0 && Oe < Be)) return 0;
                    const nt = this.getTerrainData(Te), ht = (Ue = nt.tile) === null || Ue === void 0 ? void 0 : Ue.dem;
                    if (!ht) return 0;
                    const Et = function(On, Bn, qn) {
                      var Vn = Bn[0], sr = Bn[1];
                      return On[0] = qn[0] * Vn + qn[4] * sr + qn[12], On[1] = qn[1] * Vn + qn[5] * sr + qn[13], On;
                    }([], [Ae / Be * ee.X, Oe / Be * ee.X], nt.u_terrain_matrix), Mt = [Et[0] * ht.dim, Et[1] * ht.dim], qt = Math.floor(Mt[0]), tn = Math.floor(Mt[1]), xn = Mt[0] - qt, An = Mt[1] - tn;
                    return ht.get(qt, tn) * (1 - xn) * (1 - An) + ht.get(qt + 1, tn) * xn * (1 - An) + ht.get(qt, tn + 1) * (1 - xn) * An + ht.get(qt + 1, tn + 1) * xn * An;
                  }
                  getElevationForLngLatZoom(Te, Ae) {
                    const { tileID: Oe, mercatorX: Be, mercatorY: Ue } = this._getOverscaledTileIDFromLngLatZoom(Te, Ae);
                    return this.getElevation(Oe, Be % ee.X, Ue % ee.X, ee.X);
                  }
                  getElevation(Te, Ae, Oe, Be = ee.X) {
                    return this.getDEMElevation(Te, Ae, Oe, Be) * this.exaggeration;
                  }
                  getTerrainData(Te) {
                    if (!this._emptyDemTexture) {
                      const Be = this.painter.context, Ue = new ee.R({ width: 1, height: 1 }, new Uint8Array(4));
                      this._emptyDepthTexture = new Le(Be, Ue, Be.gl.RGBA, { premultiply: !1 }), this._emptyDemUnpack = [0, 0, 0, 0], this._emptyDemTexture = new Le(Be, new ee.R({ width: 1, height: 1 }), Be.gl.RGBA, { premultiply: !1 }), this._emptyDemTexture.bind(Be.gl.NEAREST, Be.gl.CLAMP_TO_EDGE), this._emptyDemMatrix = ee.ao([]);
                    }
                    const Ae = this.sourceCache.getSourceTile(Te, !0);
                    if (Ae && Ae.dem && (!Ae.demTexture || Ae.needsTerrainPrepare)) {
                      const Be = this.painter.context;
                      Ae.demTexture = this.painter.getTileTexture(Ae.dem.stride), Ae.demTexture ? Ae.demTexture.update(Ae.dem.getPixels(), { premultiply: !1 }) : Ae.demTexture = new Le(Be, Ae.dem.getPixels(), Be.gl.RGBA, { premultiply: !1 }), Ae.demTexture.bind(Be.gl.NEAREST, Be.gl.CLAMP_TO_EDGE), Ae.needsTerrainPrepare = !1;
                    }
                    const Oe = Ae && Ae + Ae.tileID.key + Te.key;
                    if (Oe && !this._demMatrixCache[Oe]) {
                      const Be = this.sourceCache.sourceCache._source.maxzoom;
                      let Ue = Te.canonical.z - Ae.tileID.canonical.z;
                      Te.overscaledZ > Te.canonical.z && (Te.canonical.z >= Be ? Ue = Te.canonical.z - Be : ee.w("cannot calculate elevation if elevation maxzoom > source.maxzoom"));
                      const nt = Te.canonical.x - (Te.canonical.x >> Ue << Ue), ht = Te.canonical.y - (Te.canonical.y >> Ue << Ue), Et = ee.bb(new Float64Array(16), [1 / (ee.X << Ue), 1 / (ee.X << Ue), 0]);
                      ee.J(Et, Et, [nt * ee.X, ht * ee.X, 0]), this._demMatrixCache[Te.key] = { matrix: Et, coord: Te };
                    }
                    return { u_depth: 2, u_terrain: 3, u_terrain_dim: Ae && Ae.dem && Ae.dem.dim || 1, u_terrain_matrix: Oe ? this._demMatrixCache[Te.key].matrix : this._emptyDemMatrix, u_terrain_unpack: Ae && Ae.dem && Ae.dem.getUnpackVector() || this._emptyDemUnpack, u_terrain_exaggeration: this.exaggeration, texture: (Ae && Ae.demTexture || this._emptyDemTexture).texture, depthTexture: (this._fboDepthTexture || this._emptyDepthTexture).texture, tile: Ae };
                  }
                  getFramebuffer(Te) {
                    const Ae = this.painter, Oe = Ae.width / devicePixelRatio, Be = Ae.height / devicePixelRatio;
                    return !this._fbo || this._fbo.width === Oe && this._fbo.height === Be || (this._fbo.destroy(), this._fboCoordsTexture.destroy(), this._fboDepthTexture.destroy(), delete this._fbo, delete this._fboDepthTexture, delete this._fboCoordsTexture), this._fboCoordsTexture || (this._fboCoordsTexture = new Le(Ae.context, { width: Oe, height: Be, data: null }, Ae.context.gl.RGBA, { premultiply: !1 }), this._fboCoordsTexture.bind(Ae.context.gl.NEAREST, Ae.context.gl.CLAMP_TO_EDGE)), this._fboDepthTexture || (this._fboDepthTexture = new Le(Ae.context, { width: Oe, height: Be, data: null }, Ae.context.gl.RGBA, { premultiply: !1 }), this._fboDepthTexture.bind(Ae.context.gl.NEAREST, Ae.context.gl.CLAMP_TO_EDGE)), this._fbo || (this._fbo = Ae.context.createFramebuffer(Oe, Be, !0, !1), this._fbo.depthAttachment.set(Ae.context.createRenderbuffer(Ae.context.gl.DEPTH_COMPONENT16, Oe, Be))), this._fbo.colorAttachment.set(Te === "coords" ? this._fboCoordsTexture.texture : this._fboDepthTexture.texture), this._fbo;
                  }
                  getCoordsTexture() {
                    const Te = this.painter.context;
                    if (this._coordsTexture) return this._coordsTexture;
                    const Ae = new Uint8Array(this._coordsTextureSize * this._coordsTextureSize * 4);
                    for (let Ue = 0, nt = 0; Ue < this._coordsTextureSize; Ue++) for (let ht = 0; ht < this._coordsTextureSize; ht++, nt += 4) Ae[nt + 0] = 255 & ht, Ae[nt + 1] = 255 & Ue, Ae[nt + 2] = ht >> 8 << 4 | Ue >> 8, Ae[nt + 3] = 0;
                    const Oe = new ee.R({ width: this._coordsTextureSize, height: this._coordsTextureSize }, new Uint8Array(Ae.buffer)), Be = new Le(Te, Oe, Te.gl.RGBA, { premultiply: !1 });
                    return Be.bind(Te.gl.NEAREST, Te.gl.CLAMP_TO_EDGE), this._coordsTexture = Be, Be;
                  }
                  pointCoordinate(Te) {
                    this.painter.maybeDrawDepthAndCoords(!0);
                    const Ae = new Uint8Array(4), Oe = this.painter.context, Be = Oe.gl, Ue = Math.round(Te.x * this.painter.pixelRatio / devicePixelRatio), nt = Math.round(Te.y * this.painter.pixelRatio / devicePixelRatio), ht = Math.round(this.painter.height / devicePixelRatio);
                    Oe.bindFramebuffer.set(this.getFramebuffer("coords").framebuffer), Be.readPixels(Ue, ht - nt - 1, 1, 1, Be.RGBA, Be.UNSIGNED_BYTE, Ae), Oe.bindFramebuffer.set(null);
                    const Et = Ae[0] + (Ae[2] >> 4 << 8), Mt = Ae[1] + ((15 & Ae[2]) << 8), qt = this.coordsIndex[255 - Ae[3]], tn = qt && this.sourceCache.getTileByID(qt);
                    if (!tn) return null;
                    const xn = this._coordsTextureSize, An = (1 << tn.tileID.canonical.z) * xn;
                    return new ee.Z((tn.tileID.canonical.x * xn + Et) / An + tn.tileID.wrap, (tn.tileID.canonical.y * xn + Mt) / An, this.getElevation(tn.tileID, Et, Mt, xn));
                  }
                  depthAtPoint(Te) {
                    const Ae = new Uint8Array(4), Oe = this.painter.context, Be = Oe.gl;
                    return Oe.bindFramebuffer.set(this.getFramebuffer("depth").framebuffer), Be.readPixels(Te.x, this.painter.height / devicePixelRatio - Te.y - 1, 1, 1, Be.RGBA, Be.UNSIGNED_BYTE, Ae), Oe.bindFramebuffer.set(null), (Ae[0] / 16777216 + Ae[1] / 65536 + Ae[2] / 256 + Ae[3]) / 256;
                  }
                  getTerrainMesh() {
                    if (this._mesh) return this._mesh;
                    const Te = this.painter.context, Ae = new ee.bc(), Oe = new ee.aY(), Be = this.meshSize, Ue = ee.X / Be, nt = Be * Be;
                    for (let tn = 0; tn <= Be; tn++) for (let xn = 0; xn <= Be; xn++) Ae.emplaceBack(xn * Ue, tn * Ue, 0);
                    for (let tn = 0; tn < nt; tn += Be + 1) for (let xn = 0; xn < Be; xn++) Oe.emplaceBack(xn + tn, Be + xn + tn + 1, Be + xn + tn + 2), Oe.emplaceBack(xn + tn, Be + xn + tn + 2, xn + tn + 1);
                    const ht = Ae.length, Et = ht + 2 * (Be + 1);
                    for (const tn of [0, 1]) for (let xn = 0; xn <= Be; xn++) for (const An of [0, 1]) Ae.emplaceBack(xn * Ue, tn * ee.X, An);
                    for (let tn = 0; tn < 2 * Be; tn += 2) Oe.emplaceBack(Et + tn, Et + tn + 1, Et + tn + 3), Oe.emplaceBack(Et + tn, Et + tn + 3, Et + tn + 2), Oe.emplaceBack(ht + tn, ht + tn + 3, ht + tn + 1), Oe.emplaceBack(ht + tn, ht + tn + 2, ht + tn + 3);
                    const Mt = Ae.length, qt = Mt + 2 * (Be + 1);
                    for (const tn of [0, 1]) for (let xn = 0; xn <= Be; xn++) for (const An of [0, 1]) Ae.emplaceBack(tn * ee.X, xn * Ue, An);
                    for (let tn = 0; tn < 2 * Be; tn += 2) Oe.emplaceBack(Mt + tn, Mt + tn + 1, Mt + tn + 3), Oe.emplaceBack(Mt + tn, Mt + tn + 3, Mt + tn + 2), Oe.emplaceBack(qt + tn, qt + tn + 3, qt + tn + 1), Oe.emplaceBack(qt + tn, qt + tn + 2, qt + tn + 3);
                    return this._mesh = new po(Te.createVertexBuffer(Ae, Ou.members), Te.createIndexBuffer(Oe), ee.a0.simpleSegment(0, 0, Ae.length, Oe.length)), this._mesh;
                  }
                  getMeshFrameDelta(Te) {
                    return 2 * Math.PI * ee.bd / Math.pow(2, Te) / 5;
                  }
                  getMinTileElevationForLngLatZoom(Te, Ae) {
                    var Oe;
                    const { tileID: Be } = this._getOverscaledTileIDFromLngLatZoom(Te, Ae);
                    return (Oe = this.getMinMaxElevation(Be).minElevation) !== null && Oe !== void 0 ? Oe : 0;
                  }
                  getMinMaxElevation(Te) {
                    const Ae = this.getTerrainData(Te).tile, Oe = { minElevation: null, maxElevation: null };
                    return Ae && Ae.dem && (Oe.minElevation = Ae.dem.min * this.exaggeration, Oe.maxElevation = Ae.dem.max * this.exaggeration), Oe;
                  }
                  _getOverscaledTileIDFromLngLatZoom(Te, Ae) {
                    const Oe = ee.Z.fromLngLat(Te.wrap()), Be = (1 << Ae) * ee.X, Ue = Oe.x * Be, nt = Oe.y * Be, ht = Math.floor(Ue / ee.X), Et = Math.floor(nt / ee.X);
                    return { tileID: new ee.S(Ae, 0, Ae, ht, Et), mercatorX: Ue, mercatorY: nt };
                  }
                }
                class eu {
                  constructor(Te, Ae, Oe) {
                    this._context = Te, this._size = Ae, this._tileSize = Oe, this._objects = [], this._recentlyUsed = [], this._stamp = 0;
                  }
                  destruct() {
                    for (const Te of this._objects) Te.texture.destroy(), Te.fbo.destroy();
                  }
                  _createObject(Te) {
                    const Ae = this._context.createFramebuffer(this._tileSize, this._tileSize, !0, !0), Oe = new Le(this._context, { width: this._tileSize, height: this._tileSize, data: null }, this._context.gl.RGBA);
                    return Oe.bind(this._context.gl.LINEAR, this._context.gl.CLAMP_TO_EDGE), Ae.depthAttachment.set(this._context.createRenderbuffer(this._context.gl.DEPTH_STENCIL, this._tileSize, this._tileSize)), Ae.colorAttachment.set(Oe.texture), { id: Te, fbo: Ae, texture: Oe, stamp: -1, inUse: !1 };
                  }
                  getObjectForId(Te) {
                    return this._objects[Te];
                  }
                  useObject(Te) {
                    Te.inUse = !0, this._recentlyUsed = this._recentlyUsed.filter((Ae) => Te.id !== Ae), this._recentlyUsed.push(Te.id);
                  }
                  stampObject(Te) {
                    Te.stamp = ++this._stamp;
                  }
                  getOrCreateFreeObject() {
                    for (const Ae of this._recentlyUsed) if (!this._objects[Ae].inUse) return this._objects[Ae];
                    if (this._objects.length >= this._size) throw new Error("No free RenderPool available, call freeAllObjects() required!");
                    const Te = this._createObject(this._objects.length);
                    return this._objects.push(Te), Te;
                  }
                  freeObject(Te) {
                    Te.inUse = !1;
                  }
                  freeAllObjects() {
                    for (const Te of this._objects) this.freeObject(Te);
                  }
                  isFull() {
                    return !(this._objects.length < this._size) && this._objects.some((Te) => !Te.inUse) === !1;
                  }
                }
                const Wc = { background: !0, fill: !0, line: !0, raster: !0, hillshade: !0 };
                class hu {
                  constructor(Te, Ae) {
                    this.painter = Te, this.terrain = Ae, this.pool = new eu(Te.context, 30, Ae.sourceCache.tileSize * Ae.qualityFactor);
                  }
                  destruct() {
                    this.pool.destruct();
                  }
                  getTexture(Te) {
                    return this.pool.getObjectForId(Te.rtt[this._stacks.length - 1].id).texture;
                  }
                  prepareForRender(Te, Ae) {
                    this._stacks = [], this._prevType = null, this._rttTiles = [], this._renderableTiles = this.terrain.sourceCache.getRenderableTiles(), this._renderableLayerIds = Te._order.filter((Oe) => !Te._layers[Oe].isHidden(Ae)), this._coordsDescendingInv = {};
                    for (const Oe in Te.sourceCaches) {
                      this._coordsDescendingInv[Oe] = {};
                      const Be = Te.sourceCaches[Oe].getVisibleCoordinates();
                      for (const Ue of Be) {
                        const nt = this.terrain.sourceCache.getTerrainCoords(Ue);
                        for (const ht in nt) this._coordsDescendingInv[Oe][ht] || (this._coordsDescendingInv[Oe][ht] = []), this._coordsDescendingInv[Oe][ht].push(nt[ht]);
                      }
                    }
                    this._coordsDescendingInvStr = {};
                    for (const Oe of Te._order) {
                      const Be = Te._layers[Oe], Ue = Be.source;
                      if (Wc[Be.type] && !this._coordsDescendingInvStr[Ue]) {
                        this._coordsDescendingInvStr[Ue] = {};
                        for (const nt in this._coordsDescendingInv[Ue]) this._coordsDescendingInvStr[Ue][nt] = this._coordsDescendingInv[Ue][nt].map((ht) => ht.key).sort().join();
                      }
                    }
                    for (const Oe of this._renderableTiles) for (const Be in this._coordsDescendingInvStr) {
                      const Ue = this._coordsDescendingInvStr[Be][Oe.tileID.key];
                      Ue && Ue !== Oe.rttCoords[Be] && (Oe.rtt = []);
                    }
                  }
                  renderLayer(Te) {
                    if (Te.isHidden(this.painter.transform.zoom)) return !1;
                    const Ae = Te.type, Oe = this.painter, Be = this._renderableLayerIds[this._renderableLayerIds.length - 1] === Te.id;
                    if (Wc[Ae] && (this._prevType && Wc[this._prevType] || this._stacks.push([]), this._prevType = Ae, this._stacks[this._stacks.length - 1].push(Te.id), !Be)) return !0;
                    if (Wc[this._prevType] || Wc[Ae] && Be) {
                      this._prevType = Ae;
                      const Ue = this._stacks.length - 1, nt = this._stacks[Ue] || [];
                      for (const ht of this._renderableTiles) {
                        if (this.pool.isFull() && (Mo(this.painter, this.terrain, this._rttTiles), this._rttTiles = [], this.pool.freeAllObjects()), this._rttTiles.push(ht), ht.rtt[Ue]) {
                          const Mt = this.pool.getObjectForId(ht.rtt[Ue].id);
                          if (Mt.stamp === ht.rtt[Ue].stamp) {
                            this.pool.useObject(Mt);
                            continue;
                          }
                        }
                        const Et = this.pool.getOrCreateFreeObject();
                        this.pool.useObject(Et), this.pool.stampObject(Et), ht.rtt[Ue] = { id: Et.id, stamp: Et.stamp }, Oe.context.bindFramebuffer.set(Et.fbo.framebuffer), Oe.context.clear({ color: ee.aN.transparent, stencil: 0 }), Oe.currentStencilSource = void 0;
                        for (let Mt = 0; Mt < nt.length; Mt++) {
                          const qt = Oe.style._layers[nt[Mt]], tn = qt.source ? this._coordsDescendingInv[qt.source][ht.tileID.key] : [ht.tileID];
                          Oe.context.viewport.set([0, 0, Et.fbo.width, Et.fbo.height]), Oe._renderTileClippingMasks(qt, tn), Oe.renderLayer(Oe, Oe.style.sourceCaches[qt.source], qt, tn), qt.source && (ht.rttCoords[qt.source] = this._coordsDescendingInvStr[qt.source][ht.tileID.key]);
                        }
                      }
                      return Mo(this.painter, this.terrain, this._rttTiles), this._rttTiles = [], this.pool.freeAllObjects(), Wc[Ae];
                    }
                    return !1;
                  }
                }
                const gd = { "AttributionControl.ToggleAttribution": "Toggle attribution", "AttributionControl.MapFeedback": "Map feedback", "FullscreenControl.Enter": "Enter fullscreen", "FullscreenControl.Exit": "Exit fullscreen", "GeolocateControl.FindMyLocation": "Find my location", "GeolocateControl.LocationNotAvailable": "Location not available", "LogoControl.Title": "MapLibre logo", "Map.Title": "Map", "Marker.Title": "Map marker", "NavigationControl.ResetBearing": "Reset bearing to north", "NavigationControl.ZoomIn": "Zoom in", "NavigationControl.ZoomOut": "Zoom out", "Popup.Close": "Close popup", "ScaleControl.Feet": "ft", "ScaleControl.Meters": "m", "ScaleControl.Kilometers": "km", "ScaleControl.Miles": "mi", "ScaleControl.NauticalMiles": "nm", "TerrainControl.Enable": "Enable terrain", "TerrainControl.Disable": "Disable terrain", "CooperativeGesturesHandler.WindowsHelpText": "Use Ctrl + scroll to zoom the map", "CooperativeGesturesHandler.MacHelpText": "Use ⌘ + scroll to zoom the map", "CooperativeGesturesHandler.MobileHelpText": "Use two fingers to move the map" }, Jl = ae, Yc = { hash: !1, interactive: !0, bearingSnap: 7, attributionControl: hc, maplibreLogo: !1, failIfMajorPerformanceCaveat: !1, preserveDrawingBuffer: !1, refreshExpiredTiles: !0, scrollZoom: !0, minZoom: -2, maxZoom: 22, minPitch: 0, maxPitch: 60, boxZoom: !0, dragRotate: !0, dragPan: !0, keyboard: !0, doubleClickZoom: !0, touchZoomRotate: !0, touchPitch: !0, cooperativeGestures: !1, trackResize: !0, center: [0, 0], zoom: 0, bearing: 0, pitch: 0, renderWorldCopies: !0, maxTileCacheSize: null, maxTileCacheZoomLevels: ee.a.MAX_TILE_CACHE_ZOOM_LEVELS, transformRequest: null, transformCameraUpdate: null, fadeDuration: 300, crossSourceCollisions: !0, clickTolerance: 3, localIdeographFontFamily: "sans-serif", pitchWithRotate: !0, validateStyle: !0, maxCanvasSize: [4096, 4096], cancelPendingTileRequestsWhileZooming: !0 }, nc = (Fe) => {
                  Fe.touchstart = Fe.dragStart, Fe.touchmoveWindow = Fe.dragMove, Fe.touchend = Fe.dragEnd;
                }, Ko = { showCompass: !0, showZoom: !0, visualizePitch: !1 };
                class ks {
                  constructor(Te, Ae, Oe = !1) {
                    this.mousedown = (nt) => {
                      this.startMouse(ee.e({}, nt, { ctrlKey: !0, preventDefault: () => nt.preventDefault() }), le.mousePos(this.element, nt)), le.addEventListener(window, "mousemove", this.mousemove), le.addEventListener(window, "mouseup", this.mouseup);
                    }, this.mousemove = (nt) => {
                      this.moveMouse(nt, le.mousePos(this.element, nt));
                    }, this.mouseup = (nt) => {
                      this.mouseRotate.dragEnd(nt), this.mousePitch && this.mousePitch.dragEnd(nt), this.offTemp();
                    }, this.touchstart = (nt) => {
                      nt.targetTouches.length !== 1 ? this.reset() : (this._startPos = this._lastPos = le.touchPos(this.element, nt.targetTouches)[0], this.startTouch(nt, this._startPos), le.addEventListener(window, "touchmove", this.touchmove, { passive: !1 }), le.addEventListener(window, "touchend", this.touchend));
                    }, this.touchmove = (nt) => {
                      nt.targetTouches.length !== 1 ? this.reset() : (this._lastPos = le.touchPos(this.element, nt.targetTouches)[0], this.moveTouch(nt, this._lastPos));
                    }, this.touchend = (nt) => {
                      nt.targetTouches.length === 0 && this._startPos && this._lastPos && this._startPos.dist(this._lastPos) < this._clickTolerance && this.element.click(), delete this._startPos, delete this._lastPos, this.offTemp();
                    }, this.reset = () => {
                      this.mouseRotate.reset(), this.mousePitch && this.mousePitch.reset(), this.touchRotate.reset(), this.touchPitch && this.touchPitch.reset(), delete this._startPos, delete this._lastPos, this.offTemp();
                    }, this._clickTolerance = 10;
                    const Be = Te.dragRotate._mouseRotate.getClickTolerance(), Ue = Te.dragRotate._mousePitch.getClickTolerance();
                    this.element = Ae, this.mouseRotate = yu({ clickTolerance: Be, enable: !0 }), this.touchRotate = (({ enable: nt, clickTolerance: ht, bearingDegreesPerPixelMoved: Et = 0.8 }) => {
                      const Mt = new du();
                      return new Ec({ clickTolerance: ht, move: (qt, tn) => ({ bearingDelta: (tn.x - qt.x) * Et }), moveStateManager: Mt, enable: nt, assignEvents: nc });
                    })({ clickTolerance: Be, enable: !0 }), this.map = Te, Oe && (this.mousePitch = au({ clickTolerance: Ue, enable: !0 }), this.touchPitch = (({ enable: nt, clickTolerance: ht, pitchDegreesPerPixelMoved: Et = -0.5 }) => {
                      const Mt = new du();
                      return new Ec({ clickTolerance: ht, move: (qt, tn) => ({ pitchDelta: (tn.y - qt.y) * Et }), moveStateManager: Mt, enable: nt, assignEvents: nc });
                    })({ clickTolerance: Ue, enable: !0 })), le.addEventListener(Ae, "mousedown", this.mousedown), le.addEventListener(Ae, "touchstart", this.touchstart, { passive: !1 }), le.addEventListener(Ae, "touchcancel", this.reset);
                  }
                  startMouse(Te, Ae) {
                    this.mouseRotate.dragStart(Te, Ae), this.mousePitch && this.mousePitch.dragStart(Te, Ae), le.disableDrag();
                  }
                  startTouch(Te, Ae) {
                    this.touchRotate.dragStart(Te, Ae), this.touchPitch && this.touchPitch.dragStart(Te, Ae), le.disableDrag();
                  }
                  moveMouse(Te, Ae) {
                    const Oe = this.map, { bearingDelta: Be } = this.mouseRotate.dragMove(Te, Ae) || {};
                    if (Be && Oe.setBearing(Oe.getBearing() + Be), this.mousePitch) {
                      const { pitchDelta: Ue } = this.mousePitch.dragMove(Te, Ae) || {};
                      Ue && Oe.setPitch(Oe.getPitch() + Ue);
                    }
                  }
                  moveTouch(Te, Ae) {
                    const Oe = this.map, { bearingDelta: Be } = this.touchRotate.dragMove(Te, Ae) || {};
                    if (Be && Oe.setBearing(Oe.getBearing() + Be), this.touchPitch) {
                      const { pitchDelta: Ue } = this.touchPitch.dragMove(Te, Ae) || {};
                      Ue && Oe.setPitch(Oe.getPitch() + Ue);
                    }
                  }
                  off() {
                    const Te = this.element;
                    le.removeEventListener(Te, "mousedown", this.mousedown), le.removeEventListener(Te, "touchstart", this.touchstart, { passive: !1 }), le.removeEventListener(window, "touchmove", this.touchmove, { passive: !1 }), le.removeEventListener(window, "touchend", this.touchend), le.removeEventListener(Te, "touchcancel", this.reset), this.offTemp();
                  }
                  offTemp() {
                    le.enableDrag(), le.removeEventListener(window, "mousemove", this.mousemove), le.removeEventListener(window, "mouseup", this.mouseup), le.removeEventListener(window, "touchmove", this.touchmove, { passive: !1 }), le.removeEventListener(window, "touchend", this.touchend);
                  }
                }
                let qs;
                function a1(Fe, Te, Ae) {
                  const Oe = new ee.N(Fe.lng, Fe.lat);
                  if (Fe = new ee.N(Fe.lng, Fe.lat), Te) {
                    const Be = new ee.N(Fe.lng - 360, Fe.lat), Ue = new ee.N(Fe.lng + 360, Fe.lat), nt = Ae.locationPoint(Fe).distSqr(Te);
                    Ae.locationPoint(Be).distSqr(Te) < nt ? Fe = Be : Ae.locationPoint(Ue).distSqr(Te) < nt && (Fe = Ue);
                  }
                  for (; Math.abs(Fe.lng - Ae.center.lng) > 180; ) {
                    const Be = Ae.locationPoint(Fe);
                    if (Be.x >= 0 && Be.y >= 0 && Be.x <= Ae.width && Be.y <= Ae.height) break;
                    Fe.lng > Ae.center.lng ? Fe.lng -= 360 : Fe.lng += 360;
                  }
                  return Fe.lng !== Oe.lng && Ae.locationPoint(Fe).y > Ae.height / 2 - Ae.getHorizon() ? Fe : Oe;
                }
                const $l = { center: "translate(-50%,-50%)", top: "translate(-50%,0)", "top-left": "translate(0,0)", "top-right": "translate(-100%,0)", bottom: "translate(-50%,-100%)", "bottom-left": "translate(0,-100%)", "bottom-right": "translate(-100%,-100%)", left: "translate(0,-50%)", right: "translate(-100%,-50%)" };
                function o1(Fe, Te, Ae) {
                  const Oe = Fe.classList;
                  for (const Be in $l) Oe.remove(`maplibregl-${Ae}-anchor-${Be}`);
                  Oe.add(`maplibregl-${Ae}-anchor-${Te}`);
                }
                class $c extends ee.E {
                  constructor(Te) {
                    if (super(), this._onKeyPress = (Ae) => {
                      const Oe = Ae.code, Be = Ae.charCode || Ae.keyCode;
                      Oe !== "Space" && Oe !== "Enter" && Be !== 32 && Be !== 13 || this.togglePopup();
                    }, this._onMapClick = (Ae) => {
                      const Oe = Ae.originalEvent.target, Be = this._element;
                      this._popup && (Oe === Be || Be.contains(Oe)) && this.togglePopup();
                    }, this._update = (Ae) => {
                      var Oe;
                      if (!this._map) return;
                      const Be = this._map.loaded() && !this._map.isMoving();
                      ((Ae == null ? void 0 : Ae.type) === "terrain" || (Ae == null ? void 0 : Ae.type) === "render" && !Be) && this._map.once("render", this._update), this._map.transform.renderWorldCopies ? this._lngLat = a1(this._lngLat, this._flatPos, this._map.transform) : this._lngLat = (Oe = this._lngLat) === null || Oe === void 0 ? void 0 : Oe.wrap(), this._flatPos = this._pos = this._map.project(this._lngLat)._add(this._offset), this._map.terrain && (this._flatPos = this._map.transform.locationPoint(this._lngLat)._add(this._offset));
                      let Ue = "";
                      this._rotationAlignment === "viewport" || this._rotationAlignment === "auto" ? Ue = `rotateZ(${this._rotation}deg)` : this._rotationAlignment === "map" && (Ue = `rotateZ(${this._rotation - this._map.getBearing()}deg)`);
                      let nt = "";
                      this._pitchAlignment === "viewport" || this._pitchAlignment === "auto" ? nt = "rotateX(0deg)" : this._pitchAlignment === "map" && (nt = `rotateX(${this._map.getPitch()}deg)`), this._subpixelPositioning || Ae && Ae.type !== "moveend" || (this._pos = this._pos.round()), le.setTransform(this._element, `${$l[this._anchor]} translate(${this._pos.x}px, ${this._pos.y}px) ${nt} ${Ue}`), ce.frameAsync(new AbortController()).then(() => {
                        this._updateOpacity(Ae && Ae.type === "moveend");
                      }).catch(() => {
                      });
                    }, this._onMove = (Ae) => {
                      if (!this._isDragging) {
                        const Oe = this._clickTolerance || this._map._clickTolerance;
                        this._isDragging = Ae.point.dist(this._pointerdownPos) >= Oe;
                      }
                      this._isDragging && (this._pos = Ae.point.sub(this._positionDelta), this._lngLat = this._map.unproject(this._pos), this.setLngLat(this._lngLat), this._element.style.pointerEvents = "none", this._state === "pending" && (this._state = "active", this.fire(new ee.k("dragstart"))), this.fire(new ee.k("drag")));
                    }, this._onUp = () => {
                      this._element.style.pointerEvents = "auto", this._positionDelta = null, this._pointerdownPos = null, this._isDragging = !1, this._map.off("mousemove", this._onMove), this._map.off("touchmove", this._onMove), this._state === "active" && this.fire(new ee.k("dragend")), this._state = "inactive";
                    }, this._addDragHandler = (Ae) => {
                      this._element.contains(Ae.originalEvent.target) && (Ae.preventDefault(), this._positionDelta = Ae.point.sub(this._pos).add(this._offset), this._pointerdownPos = Ae.point, this._state = "pending", this._map.on("mousemove", this._onMove), this._map.on("touchmove", this._onMove), this._map.once("mouseup", this._onUp), this._map.once("touchend", this._onUp));
                    }, this._anchor = Te && Te.anchor || "center", this._color = Te && Te.color || "#3FB1CE", this._scale = Te && Te.scale || 1, this._draggable = Te && Te.draggable || !1, this._clickTolerance = Te && Te.clickTolerance || 0, this._subpixelPositioning = Te && Te.subpixelPositioning || !1, this._isDragging = !1, this._state = "inactive", this._rotation = Te && Te.rotation || 0, this._rotationAlignment = Te && Te.rotationAlignment || "auto", this._pitchAlignment = Te && Te.pitchAlignment && Te.pitchAlignment !== "auto" ? Te.pitchAlignment : this._rotationAlignment, this.setOpacity(), this.setOpacity(Te == null ? void 0 : Te.opacity, Te == null ? void 0 : Te.opacityWhenCovered), Te && Te.element) this._element = Te.element, this._offset = ee.P.convert(Te && Te.offset || [0, 0]);
                    else {
                      this._defaultMarker = !0, this._element = le.create("div");
                      const Ae = le.createNS("http://www.w3.org/2000/svg", "svg"), Oe = 41, Be = 27;
                      Ae.setAttributeNS(null, "display", "block"), Ae.setAttributeNS(null, "height", `${Oe}px`), Ae.setAttributeNS(null, "width", `${Be}px`), Ae.setAttributeNS(null, "viewBox", `0 0 ${Be} ${Oe}`);
                      const Ue = le.createNS("http://www.w3.org/2000/svg", "g");
                      Ue.setAttributeNS(null, "stroke", "none"), Ue.setAttributeNS(null, "stroke-width", "1"), Ue.setAttributeNS(null, "fill", "none"), Ue.setAttributeNS(null, "fill-rule", "evenodd");
                      const nt = le.createNS("http://www.w3.org/2000/svg", "g");
                      nt.setAttributeNS(null, "fill-rule", "nonzero");
                      const ht = le.createNS("http://www.w3.org/2000/svg", "g");
                      ht.setAttributeNS(null, "transform", "translate(3.0, 29.0)"), ht.setAttributeNS(null, "fill", "#000000");
                      const Et = [{ rx: "10.5", ry: "5.25002273" }, { rx: "10.5", ry: "5.25002273" }, { rx: "9.5", ry: "4.77275007" }, { rx: "8.5", ry: "4.29549936" }, { rx: "7.5", ry: "3.81822308" }, { rx: "6.5", ry: "3.34094679" }, { rx: "5.5", ry: "2.86367051" }, { rx: "4.5", ry: "2.38636864" }];
                      for (const Vn of Et) {
                        const sr = le.createNS("http://www.w3.org/2000/svg", "ellipse");
                        sr.setAttributeNS(null, "opacity", "0.04"), sr.setAttributeNS(null, "cx", "10.5"), sr.setAttributeNS(null, "cy", "5.80029008"), sr.setAttributeNS(null, "rx", Vn.rx), sr.setAttributeNS(null, "ry", Vn.ry), ht.appendChild(sr);
                      }
                      const Mt = le.createNS("http://www.w3.org/2000/svg", "g");
                      Mt.setAttributeNS(null, "fill", this._color);
                      const qt = le.createNS("http://www.w3.org/2000/svg", "path");
                      qt.setAttributeNS(null, "d", "M27,13.5 C27,19.074644 20.250001,27.000002 14.75,34.500002 C14.016665,35.500004 12.983335,35.500004 12.25,34.500002 C6.7499993,27.000002 0,19.222562 0,13.5 C0,6.0441559 6.0441559,0 13.5,0 C20.955844,0 27,6.0441559 27,13.5 Z"), Mt.appendChild(qt);
                      const tn = le.createNS("http://www.w3.org/2000/svg", "g");
                      tn.setAttributeNS(null, "opacity", "0.25"), tn.setAttributeNS(null, "fill", "#000000");
                      const xn = le.createNS("http://www.w3.org/2000/svg", "path");
                      xn.setAttributeNS(null, "d", "M13.5,0 C6.0441559,0 0,6.0441559 0,13.5 C0,19.222562 6.7499993,27 12.25,34.5 C13,35.522727 14.016664,35.500004 14.75,34.5 C20.250001,27 27,19.074644 27,13.5 C27,6.0441559 20.955844,0 13.5,0 Z M13.5,1 C20.415404,1 26,6.584596 26,13.5 C26,15.898657 24.495584,19.181431 22.220703,22.738281 C19.945823,26.295132 16.705119,30.142167 13.943359,33.908203 C13.743445,34.180814 13.612715,34.322738 13.5,34.441406 C13.387285,34.322738 13.256555,34.180814 13.056641,33.908203 C10.284481,30.127985 7.4148684,26.314159 5.015625,22.773438 C2.6163816,19.232715 1,15.953538 1,13.5 C1,6.584596 6.584596,1 13.5,1 Z"), tn.appendChild(xn);
                      const An = le.createNS("http://www.w3.org/2000/svg", "g");
                      An.setAttributeNS(null, "transform", "translate(6.0, 7.0)"), An.setAttributeNS(null, "fill", "#FFFFFF");
                      const On = le.createNS("http://www.w3.org/2000/svg", "g");
                      On.setAttributeNS(null, "transform", "translate(8.0, 8.0)");
                      const Bn = le.createNS("http://www.w3.org/2000/svg", "circle");
                      Bn.setAttributeNS(null, "fill", "#000000"), Bn.setAttributeNS(null, "opacity", "0.25"), Bn.setAttributeNS(null, "cx", "5.5"), Bn.setAttributeNS(null, "cy", "5.5"), Bn.setAttributeNS(null, "r", "5.4999962");
                      const qn = le.createNS("http://www.w3.org/2000/svg", "circle");
                      qn.setAttributeNS(null, "fill", "#FFFFFF"), qn.setAttributeNS(null, "cx", "5.5"), qn.setAttributeNS(null, "cy", "5.5"), qn.setAttributeNS(null, "r", "5.4999962"), On.appendChild(Bn), On.appendChild(qn), nt.appendChild(ht), nt.appendChild(Mt), nt.appendChild(tn), nt.appendChild(An), nt.appendChild(On), Ae.appendChild(nt), Ae.setAttributeNS(null, "height", Oe * this._scale + "px"), Ae.setAttributeNS(null, "width", Be * this._scale + "px"), this._element.appendChild(Ae), this._offset = ee.P.convert(Te && Te.offset || [0, -14]);
                    }
                    if (this._element.classList.add("maplibregl-marker"), this._element.addEventListener("dragstart", (Ae) => {
                      Ae.preventDefault();
                    }), this._element.addEventListener("mousedown", (Ae) => {
                      Ae.preventDefault();
                    }), o1(this._element, this._anchor, "marker"), Te && Te.className) for (const Ae of Te.className.split(" ")) this._element.classList.add(Ae);
                    this._popup = null;
                  }
                  addTo(Te) {
                    return this.remove(), this._map = Te, this._element.setAttribute("aria-label", Te._getUIString("Marker.Title")), Te.getCanvasContainer().appendChild(this._element), Te.on("move", this._update), Te.on("moveend", this._update), Te.on("terrain", this._update), this.setDraggable(this._draggable), this._update(), this._map.on("click", this._onMapClick), this;
                  }
                  remove() {
                    return this._opacityTimeout && (clearTimeout(this._opacityTimeout), delete this._opacityTimeout), this._map && (this._map.off("click", this._onMapClick), this._map.off("move", this._update), this._map.off("moveend", this._update), this._map.off("terrain", this._update), this._map.off("mousedown", this._addDragHandler), this._map.off("touchstart", this._addDragHandler), this._map.off("mouseup", this._onUp), this._map.off("touchend", this._onUp), this._map.off("mousemove", this._onMove), this._map.off("touchmove", this._onMove), delete this._map), le.remove(this._element), this._popup && this._popup.remove(), this;
                  }
                  getLngLat() {
                    return this._lngLat;
                  }
                  setLngLat(Te) {
                    return this._lngLat = ee.N.convert(Te), this._pos = null, this._popup && this._popup.setLngLat(this._lngLat), this._update(), this;
                  }
                  getElement() {
                    return this._element;
                  }
                  setPopup(Te) {
                    if (this._popup && (this._popup.remove(), this._popup = null, this._element.removeEventListener("keypress", this._onKeyPress), this._originalTabIndex || this._element.removeAttribute("tabindex")), Te) {
                      if (!("offset" in Te.options)) {
                        const Be = Math.abs(13.5) / Math.SQRT2;
                        Te.options.offset = this._defaultMarker ? { top: [0, 0], "top-left": [0, 0], "top-right": [0, 0], bottom: [0, -38.1], "bottom-left": [Be, -1 * (24.6 + Be)], "bottom-right": [-Be, -1 * (24.6 + Be)], left: [13.5, -24.6], right: [-13.5, -24.6] } : this._offset;
                      }
                      this._popup = Te, this._originalTabIndex = this._element.getAttribute("tabindex"), this._originalTabIndex || this._element.setAttribute("tabindex", "0"), this._element.addEventListener("keypress", this._onKeyPress);
                    }
                    return this;
                  }
                  setSubpixelPositioning(Te) {
                    return this._subpixelPositioning = Te, this;
                  }
                  getPopup() {
                    return this._popup;
                  }
                  togglePopup() {
                    const Te = this._popup;
                    return this._element.style.opacity === this._opacityWhenCovered ? this : Te ? (Te.isOpen() ? Te.remove() : (Te.setLngLat(this._lngLat), Te.addTo(this._map)), this) : this;
                  }
                  _updateOpacity(Te = !1) {
                    var Ae, Oe;
                    if (!(!((Ae = this._map) === null || Ae === void 0) && Ae.terrain)) return void (this._element.style.opacity !== this._opacity && (this._element.style.opacity = this._opacity));
                    if (Te) this._opacityTimeout = null;
                    else {
                      if (this._opacityTimeout) return;
                      this._opacityTimeout = setTimeout(() => {
                        this._opacityTimeout = null;
                      }, 100);
                    }
                    const Be = this._map, Ue = Be.terrain.depthAtPoint(this._pos), nt = Be.terrain.getElevationForLngLatZoom(this._lngLat, Be.transform.tileZoom);
                    if (Be.transform.lngLatToCameraDepth(this._lngLat, nt) - Ue < 6e-3) return void (this._element.style.opacity = this._opacity);
                    const ht = -this._offset.y / Be.transform._pixelPerMeter, Et = Math.sin(Be.getPitch() * Math.PI / 180) * ht, Mt = Be.terrain.depthAtPoint(new ee.P(this._pos.x, this._pos.y - this._offset.y)), qt = Be.transform.lngLatToCameraDepth(this._lngLat, nt + Et) - Mt > 6e-3;
                    !((Oe = this._popup) === null || Oe === void 0) && Oe.isOpen() && qt && this._popup.remove(), this._element.style.opacity = qt ? this._opacityWhenCovered : this._opacity;
                  }
                  getOffset() {
                    return this._offset;
                  }
                  setOffset(Te) {
                    return this._offset = ee.P.convert(Te), this._update(), this;
                  }
                  addClassName(Te) {
                    this._element.classList.add(Te);
                  }
                  removeClassName(Te) {
                    this._element.classList.remove(Te);
                  }
                  toggleClassName(Te) {
                    return this._element.classList.toggle(Te);
                  }
                  setDraggable(Te) {
                    return this._draggable = !!Te, this._map && (Te ? (this._map.on("mousedown", this._addDragHandler), this._map.on("touchstart", this._addDragHandler)) : (this._map.off("mousedown", this._addDragHandler), this._map.off("touchstart", this._addDragHandler))), this;
                  }
                  isDraggable() {
                    return this._draggable;
                  }
                  setRotation(Te) {
                    return this._rotation = Te || 0, this._update(), this;
                  }
                  getRotation() {
                    return this._rotation;
                  }
                  setRotationAlignment(Te) {
                    return this._rotationAlignment = Te || "auto", this._update(), this;
                  }
                  getRotationAlignment() {
                    return this._rotationAlignment;
                  }
                  setPitchAlignment(Te) {
                    return this._pitchAlignment = Te && Te !== "auto" ? Te : this._rotationAlignment, this._update(), this;
                  }
                  getPitchAlignment() {
                    return this._pitchAlignment;
                  }
                  setOpacity(Te, Ae) {
                    return Te === void 0 && Ae === void 0 && (this._opacity = "1", this._opacityWhenCovered = "0.2"), Te !== void 0 && (this._opacity = Te), Ae !== void 0 && (this._opacityWhenCovered = Ae), this._map && this._updateOpacity(!0), this;
                  }
                }
                const lu = { positionOptions: { enableHighAccuracy: !1, maximumAge: 0, timeout: 6e3 }, fitBoundsOptions: { maxZoom: 15 }, trackUserLocation: !1, showAccuracyCircle: !0, showUserLocation: !0 };
                let Ul = 0, zs = !1;
                class Os extends ee.E {
                  constructor(Te) {
                    super(), this._onSuccess = (Ae) => {
                      if (this._map) {
                        if (this._isOutOfMapMaxBounds(Ae)) return this._setErrorState(), this.fire(new ee.k("outofmaxbounds", Ae)), this._updateMarker(), void this._finish();
                        if (this.options.trackUserLocation) switch (this._lastKnownPosition = Ae, this._watchState) {
                          case "WAITING_ACTIVE":
                          case "ACTIVE_LOCK":
                          case "ACTIVE_ERROR":
                            this._watchState = "ACTIVE_LOCK", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active-error"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-active");
                            break;
                          case "BACKGROUND":
                          case "BACKGROUND_ERROR":
                            this._watchState = "BACKGROUND", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-background-error"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-background");
                            break;
                          default:
                            throw new Error(`Unexpected watchState ${this._watchState}`);
                        }
                        this.options.showUserLocation && this._watchState !== "OFF" && this._updateMarker(Ae), this.options.trackUserLocation && this._watchState !== "ACTIVE_LOCK" || this._updateCamera(Ae), this.options.showUserLocation && this._dotElement.classList.remove("maplibregl-user-location-dot-stale"), this.fire(new ee.k("geolocate", Ae)), this._finish();
                      }
                    }, this._updateCamera = (Ae) => {
                      const Oe = new ee.N(Ae.coords.longitude, Ae.coords.latitude), Be = Ae.coords.accuracy, Ue = this._map.getBearing(), nt = ee.e({ bearing: Ue }, this.options.fitBoundsOptions), ht = Rt.fromLngLat(Oe, Be);
                      this._map.fitBounds(ht, nt, { geolocateSource: !0 });
                    }, this._updateMarker = (Ae) => {
                      if (Ae) {
                        const Oe = new ee.N(Ae.coords.longitude, Ae.coords.latitude);
                        this._accuracyCircleMarker.setLngLat(Oe).addTo(this._map), this._userLocationDotMarker.setLngLat(Oe).addTo(this._map), this._accuracy = Ae.coords.accuracy, this.options.showUserLocation && this.options.showAccuracyCircle && this._updateCircleRadius();
                      } else this._userLocationDotMarker.remove(), this._accuracyCircleMarker.remove();
                    }, this._onZoom = () => {
                      this.options.showUserLocation && this.options.showAccuracyCircle && this._updateCircleRadius();
                    }, this._onError = (Ae) => {
                      if (this._map) {
                        if (this.options.trackUserLocation) if (Ae.code === 1) {
                          this._watchState = "OFF", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active-error"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-background"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-background-error"), this._geolocateButton.disabled = !0;
                          const Oe = this._map._getUIString("GeolocateControl.LocationNotAvailable");
                          this._geolocateButton.title = Oe, this._geolocateButton.setAttribute("aria-label", Oe), this._geolocationWatchID !== void 0 && this._clearWatch();
                        } else {
                          if (Ae.code === 3 && zs) return;
                          this._setErrorState();
                        }
                        this._watchState !== "OFF" && this.options.showUserLocation && this._dotElement.classList.add("maplibregl-user-location-dot-stale"), this.fire(new ee.k("error", Ae)), this._finish();
                      }
                    }, this._finish = () => {
                      this._timeoutId && clearTimeout(this._timeoutId), this._timeoutId = void 0;
                    }, this._setupUI = () => {
                      this._map && (this._container.addEventListener("contextmenu", (Ae) => Ae.preventDefault()), this._geolocateButton = le.create("button", "maplibregl-ctrl-geolocate", this._container), le.create("span", "maplibregl-ctrl-icon", this._geolocateButton).setAttribute("aria-hidden", "true"), this._geolocateButton.type = "button", this._geolocateButton.disabled = !0);
                    }, this._finishSetupUI = (Ae) => {
                      if (this._map) {
                        if (Ae === !1) {
                          ee.w("Geolocation support is not available so the GeolocateControl will be disabled.");
                          const Oe = this._map._getUIString("GeolocateControl.LocationNotAvailable");
                          this._geolocateButton.disabled = !0, this._geolocateButton.title = Oe, this._geolocateButton.setAttribute("aria-label", Oe);
                        } else {
                          const Oe = this._map._getUIString("GeolocateControl.FindMyLocation");
                          this._geolocateButton.disabled = !1, this._geolocateButton.title = Oe, this._geolocateButton.setAttribute("aria-label", Oe);
                        }
                        this.options.trackUserLocation && (this._geolocateButton.setAttribute("aria-pressed", "false"), this._watchState = "OFF"), this.options.showUserLocation && (this._dotElement = le.create("div", "maplibregl-user-location-dot"), this._userLocationDotMarker = new $c({ element: this._dotElement }), this._circleElement = le.create("div", "maplibregl-user-location-accuracy-circle"), this._accuracyCircleMarker = new $c({ element: this._circleElement, pitchAlignment: "map" }), this.options.trackUserLocation && (this._watchState = "OFF"), this._map.on("zoom", this._onZoom)), this._geolocateButton.addEventListener("click", () => this.trigger()), this._setup = !0, this.options.trackUserLocation && this._map.on("movestart", (Oe) => {
                          const Be = Oe.originalEvent && Oe.originalEvent.type === "resize";
                          Oe.geolocateSource || this._watchState !== "ACTIVE_LOCK" || Be || (this._watchState = "BACKGROUND", this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-background"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active"), this.fire(new ee.k("trackuserlocationend")), this.fire(new ee.k("userlocationlostfocus")));
                        });
                      }
                    }, this.options = ee.e({}, lu, Te);
                  }
                  onAdd(Te) {
                    return this._map = Te, this._container = le.create("div", "maplibregl-ctrl maplibregl-ctrl-group"), this._setupUI(), function() {
                      return ee._(this, arguments, void 0, function* (Ae = !1) {
                        if (qs !== void 0 && !Ae) return qs;
                        if (window.navigator.permissions === void 0) return qs = !!window.navigator.geolocation, qs;
                        try {
                          qs = (yield window.navigator.permissions.query({ name: "geolocation" })).state !== "denied";
                        } catch {
                          qs = !!window.navigator.geolocation;
                        }
                        return qs;
                      });
                    }().then((Ae) => this._finishSetupUI(Ae)), this._container;
                  }
                  onRemove() {
                    this._geolocationWatchID !== void 0 && (window.navigator.geolocation.clearWatch(this._geolocationWatchID), this._geolocationWatchID = void 0), this.options.showUserLocation && this._userLocationDotMarker && this._userLocationDotMarker.remove(), this.options.showAccuracyCircle && this._accuracyCircleMarker && this._accuracyCircleMarker.remove(), le.remove(this._container), this._map.off("zoom", this._onZoom), this._map = void 0, Ul = 0, zs = !1;
                  }
                  _isOutOfMapMaxBounds(Te) {
                    const Ae = this._map.getMaxBounds(), Oe = Te.coords;
                    return Ae && (Oe.longitude < Ae.getWest() || Oe.longitude > Ae.getEast() || Oe.latitude < Ae.getSouth() || Oe.latitude > Ae.getNorth());
                  }
                  _setErrorState() {
                    switch (this._watchState) {
                      case "WAITING_ACTIVE":
                        this._watchState = "ACTIVE_ERROR", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-active-error");
                        break;
                      case "ACTIVE_LOCK":
                        this._watchState = "ACTIVE_ERROR", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-active-error"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-waiting");
                        break;
                      case "BACKGROUND":
                        this._watchState = "BACKGROUND_ERROR", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-background"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-background-error"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-waiting");
                        break;
                      case "ACTIVE_ERROR":
                        break;
                      default:
                        throw new Error(`Unexpected watchState ${this._watchState}`);
                    }
                  }
                  _updateCircleRadius() {
                    const Te = this._map.getBounds(), Ae = Te.getSouthEast(), Oe = Te.getNorthEast(), Be = Ae.distanceTo(Oe), Ue = this._map._container.clientHeight, nt = Math.ceil(this._accuracy / (Be / Ue) * 2);
                    this._circleElement.style.width = `${nt}px`, this._circleElement.style.height = `${nt}px`;
                  }
                  trigger() {
                    if (!this._setup) return ee.w("Geolocate control triggered before added to a map"), !1;
                    if (this.options.trackUserLocation) {
                      switch (this._watchState) {
                        case "OFF":
                          this._watchState = "WAITING_ACTIVE", this.fire(new ee.k("trackuserlocationstart"));
                          break;
                        case "WAITING_ACTIVE":
                        case "ACTIVE_LOCK":
                        case "ACTIVE_ERROR":
                        case "BACKGROUND_ERROR":
                          Ul--, zs = !1, this._watchState = "OFF", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-waiting"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-active-error"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-background"), this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-background-error"), this.fire(new ee.k("trackuserlocationend"));
                          break;
                        case "BACKGROUND":
                          this._watchState = "ACTIVE_LOCK", this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-background"), this._lastKnownPosition && this._updateCamera(this._lastKnownPosition), this.fire(new ee.k("trackuserlocationstart")), this.fire(new ee.k("userlocationfocus"));
                          break;
                        default:
                          throw new Error(`Unexpected watchState ${this._watchState}`);
                      }
                      switch (this._watchState) {
                        case "WAITING_ACTIVE":
                          this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-waiting"), this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-active");
                          break;
                        case "ACTIVE_LOCK":
                          this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-active");
                          break;
                        case "OFF":
                          break;
                        default:
                          throw new Error(`Unexpected watchState ${this._watchState}`);
                      }
                      if (this._watchState === "OFF" && this._geolocationWatchID !== void 0) this._clearWatch();
                      else if (this._geolocationWatchID === void 0) {
                        let Te;
                        this._geolocateButton.classList.add("maplibregl-ctrl-geolocate-waiting"), this._geolocateButton.setAttribute("aria-pressed", "true"), Ul++, Ul > 1 ? (Te = { maximumAge: 6e5, timeout: 0 }, zs = !0) : (Te = this.options.positionOptions, zs = !1), this._geolocationWatchID = window.navigator.geolocation.watchPosition(this._onSuccess, this._onError, Te);
                      }
                    } else window.navigator.geolocation.getCurrentPosition(this._onSuccess, this._onError, this.options.positionOptions), this._timeoutId = setTimeout(this._finish, 1e4);
                    return !0;
                  }
                  _clearWatch() {
                    window.navigator.geolocation.clearWatch(this._geolocationWatchID), this._geolocationWatchID = void 0, this._geolocateButton.classList.remove("maplibregl-ctrl-geolocate-waiting"), this._geolocateButton.setAttribute("aria-pressed", "false"), this.options.showUserLocation && this._updateMarker(null);
                  }
                }
                const lc = { maxWidth: 100, unit: "metric" };
                function xc(Fe, Te, Ae) {
                  const Oe = Ae && Ae.maxWidth || 100, Be = Fe._container.clientHeight / 2, Ue = Fe.unproject([0, Be]), nt = Fe.unproject([Oe, Be]), ht = Ue.distanceTo(nt);
                  if (Ae && Ae.unit === "imperial") {
                    const Et = 3.2808 * ht;
                    Et > 5280 ? Iu(Te, Oe, Et / 5280, Fe._getUIString("ScaleControl.Miles")) : Iu(Te, Oe, Et, Fe._getUIString("ScaleControl.Feet"));
                  } else Ae && Ae.unit === "nautical" ? Iu(Te, Oe, ht / 1852, Fe._getUIString("ScaleControl.NauticalMiles")) : ht >= 1e3 ? Iu(Te, Oe, ht / 1e3, Fe._getUIString("ScaleControl.Kilometers")) : Iu(Te, Oe, ht, Fe._getUIString("ScaleControl.Meters"));
                }
                function Iu(Fe, Te, Ae, Oe) {
                  const Be = function(nt) {
                    const ht = Math.pow(10, `${Math.floor(nt)}`.length - 1);
                    let Et = nt / ht;
                    return Et = Et >= 10 ? 10 : Et >= 5 ? 5 : Et >= 3 ? 3 : Et >= 2 ? 2 : Et >= 1 ? 1 : function(Mt) {
                      const qt = Math.pow(10, Math.ceil(-Math.log(Mt) / Math.LN10));
                      return Math.round(Mt * qt) / qt;
                    }(Et), ht * Et;
                  }(Ae), Ue = Be / Ae;
                  Fe.style.width = Te * Ue + "px", Fe.innerHTML = `${Be}&nbsp;${Oe}`;
                }
                class tu extends ee.E {
                  constructor(Te = {}) {
                    super(), this._onFullscreenChange = () => {
                      var Ae;
                      let Oe = window.document.fullscreenElement || window.document.mozFullScreenElement || window.document.webkitFullscreenElement || window.document.msFullscreenElement;
                      for (; !((Ae = Oe == null ? void 0 : Oe.shadowRoot) === null || Ae === void 0) && Ae.fullscreenElement; ) Oe = Oe.shadowRoot.fullscreenElement;
                      Oe === this._container !== this._fullscreen && this._handleFullscreenChange();
                    }, this._onClickFullscreen = () => {
                      this._isFullscreen() ? this._exitFullscreen() : this._requestFullscreen();
                    }, this._fullscreen = !1, Te && Te.container && (Te.container instanceof HTMLElement ? this._container = Te.container : ee.w("Full screen control 'container' must be a DOM element.")), "onfullscreenchange" in document ? this._fullscreenchange = "fullscreenchange" : "onmozfullscreenchange" in document ? this._fullscreenchange = "mozfullscreenchange" : "onwebkitfullscreenchange" in document ? this._fullscreenchange = "webkitfullscreenchange" : "onmsfullscreenchange" in document && (this._fullscreenchange = "MSFullscreenChange");
                  }
                  onAdd(Te) {
                    return this._map = Te, this._container || (this._container = this._map.getContainer()), this._controlContainer = le.create("div", "maplibregl-ctrl maplibregl-ctrl-group"), this._setupUI(), this._controlContainer;
                  }
                  onRemove() {
                    le.remove(this._controlContainer), this._map = null, window.document.removeEventListener(this._fullscreenchange, this._onFullscreenChange);
                  }
                  _setupUI() {
                    const Te = this._fullscreenButton = le.create("button", "maplibregl-ctrl-fullscreen", this._controlContainer);
                    le.create("span", "maplibregl-ctrl-icon", Te).setAttribute("aria-hidden", "true"), Te.type = "button", this._updateTitle(), this._fullscreenButton.addEventListener("click", this._onClickFullscreen), window.document.addEventListener(this._fullscreenchange, this._onFullscreenChange);
                  }
                  _updateTitle() {
                    const Te = this._getTitle();
                    this._fullscreenButton.setAttribute("aria-label", Te), this._fullscreenButton.title = Te;
                  }
                  _getTitle() {
                    return this._map._getUIString(this._isFullscreen() ? "FullscreenControl.Exit" : "FullscreenControl.Enter");
                  }
                  _isFullscreen() {
                    return this._fullscreen;
                  }
                  _handleFullscreenChange() {
                    this._fullscreen = !this._fullscreen, this._fullscreenButton.classList.toggle("maplibregl-ctrl-shrink"), this._fullscreenButton.classList.toggle("maplibregl-ctrl-fullscreen"), this._updateTitle(), this._fullscreen ? (this.fire(new ee.k("fullscreenstart")), this._prevCooperativeGesturesEnabled = this._map.cooperativeGestures.isEnabled(), this._map.cooperativeGestures.disable()) : (this.fire(new ee.k("fullscreenend")), this._prevCooperativeGesturesEnabled && this._map.cooperativeGestures.enable());
                  }
                  _exitFullscreen() {
                    window.document.exitFullscreen ? window.document.exitFullscreen() : window.document.mozCancelFullScreen ? window.document.mozCancelFullScreen() : window.document.msExitFullscreen ? window.document.msExitFullscreen() : window.document.webkitCancelFullScreen ? window.document.webkitCancelFullScreen() : this._togglePseudoFullScreen();
                  }
                  _requestFullscreen() {
                    this._container.requestFullscreen ? this._container.requestFullscreen() : this._container.mozRequestFullScreen ? this._container.mozRequestFullScreen() : this._container.msRequestFullscreen ? this._container.msRequestFullscreen() : this._container.webkitRequestFullscreen ? this._container.webkitRequestFullscreen() : this._togglePseudoFullScreen();
                  }
                  _togglePseudoFullScreen() {
                    this._container.classList.toggle("maplibregl-pseudo-fullscreen"), this._handleFullscreenChange(), this._map.resize();
                  }
                }
                const w1 = { closeButton: !0, closeOnClick: !0, focusAfterOpen: !0, className: "", maxWidth: "240px", subpixelPositioning: !1 }, $n = ["a[href]", "[tabindex]:not([tabindex='-1'])", "[contenteditable]:not([contenteditable='false'])", "button:not([disabled])", "input:not([disabled])", "select:not([disabled])", "textarea:not([disabled])"].join(", ");
                class zn extends ee.E {
                  constructor(Te) {
                    super(), this.remove = () => (this._content && le.remove(this._content), this._container && (le.remove(this._container), delete this._container), this._map && (this._map.off("move", this._update), this._map.off("move", this._onClose), this._map.off("click", this._onClose), this._map.off("remove", this.remove), this._map.off("mousemove", this._onMouseMove), this._map.off("mouseup", this._onMouseUp), this._map.off("drag", this._onDrag), this._map._canvasContainer.classList.remove("maplibregl-track-pointer"), delete this._map, this.fire(new ee.k("close"))), this), this._onMouseUp = (Ae) => {
                      this._update(Ae.point);
                    }, this._onMouseMove = (Ae) => {
                      this._update(Ae.point);
                    }, this._onDrag = (Ae) => {
                      this._update(Ae.point);
                    }, this._update = (Ae) => {
                      var Oe;
                      const Be = this._lngLat || this._trackPointer;
                      if (!this._map || !Be || !this._content) return;
                      if (!this._container) {
                        if (this._container = le.create("div", "maplibregl-popup", this._map.getContainer()), this._tip = le.create("div", "maplibregl-popup-tip", this._container), this._container.appendChild(this._content), this.options.className) for (const Mt of this.options.className.split(" ")) this._container.classList.add(Mt);
                        this._closeButton && this._closeButton.setAttribute("aria-label", this._map._getUIString("Popup.Close")), this._trackPointer && this._container.classList.add("maplibregl-popup-track-pointer");
                      }
                      if (this.options.maxWidth && this._container.style.maxWidth !== this.options.maxWidth && (this._container.style.maxWidth = this.options.maxWidth), this._map.transform.renderWorldCopies && !this._trackPointer ? this._lngLat = a1(this._lngLat, this._flatPos, this._map.transform) : this._lngLat = (Oe = this._lngLat) === null || Oe === void 0 ? void 0 : Oe.wrap(), this._trackPointer && !Ae) return;
                      const Ue = this._flatPos = this._pos = this._trackPointer && Ae ? Ae : this._map.project(this._lngLat);
                      this._map.terrain && (this._flatPos = this._trackPointer && Ae ? Ae : this._map.transform.locationPoint(this._lngLat));
                      let nt = this.options.anchor;
                      const ht = Gn(this.options.offset);
                      if (!nt) {
                        const Mt = this._container.offsetWidth, qt = this._container.offsetHeight;
                        let tn;
                        tn = Ue.y + ht.bottom.y < qt ? ["top"] : Ue.y > this._map.transform.height - qt ? ["bottom"] : [], Ue.x < Mt / 2 ? tn.push("left") : Ue.x > this._map.transform.width - Mt / 2 && tn.push("right"), nt = tn.length === 0 ? "bottom" : tn.join("-");
                      }
                      let Et = Ue.add(ht[nt]);
                      this.options.subpixelPositioning || (Et = Et.round()), le.setTransform(this._container, `${$l[nt]} translate(${Et.x}px,${Et.y}px)`), o1(this._container, nt, "popup");
                    }, this._onClose = () => {
                      this.remove();
                    }, this.options = ee.e(Object.create(w1), Te);
                  }
                  addTo(Te) {
                    return this._map && this.remove(), this._map = Te, this.options.closeOnClick && this._map.on("click", this._onClose), this.options.closeOnMove && this._map.on("move", this._onClose), this._map.on("remove", this.remove), this._update(), this._focusFirstElement(), this._trackPointer ? (this._map.on("mousemove", this._onMouseMove), this._map.on("mouseup", this._onMouseUp), this._container && this._container.classList.add("maplibregl-popup-track-pointer"), this._map._canvasContainer.classList.add("maplibregl-track-pointer")) : this._map.on("move", this._update), this.fire(new ee.k("open")), this;
                  }
                  isOpen() {
                    return !!this._map;
                  }
                  getLngLat() {
                    return this._lngLat;
                  }
                  setLngLat(Te) {
                    return this._lngLat = ee.N.convert(Te), this._pos = null, this._flatPos = null, this._trackPointer = !1, this._update(), this._map && (this._map.on("move", this._update), this._map.off("mousemove", this._onMouseMove), this._container && this._container.classList.remove("maplibregl-popup-track-pointer"), this._map._canvasContainer.classList.remove("maplibregl-track-pointer")), this;
                  }
                  trackPointer() {
                    return this._trackPointer = !0, this._pos = null, this._flatPos = null, this._update(), this._map && (this._map.off("move", this._update), this._map.on("mousemove", this._onMouseMove), this._map.on("drag", this._onDrag), this._container && this._container.classList.add("maplibregl-popup-track-pointer"), this._map._canvasContainer.classList.add("maplibregl-track-pointer")), this;
                  }
                  getElement() {
                    return this._container;
                  }
                  setText(Te) {
                    return this.setDOMContent(document.createTextNode(Te));
                  }
                  setHTML(Te) {
                    const Ae = document.createDocumentFragment(), Oe = document.createElement("body");
                    let Be;
                    for (Oe.innerHTML = Te; Be = Oe.firstChild, Be; ) Ae.appendChild(Be);
                    return this.setDOMContent(Ae);
                  }
                  getMaxWidth() {
                    var Te;
                    return (Te = this._container) === null || Te === void 0 ? void 0 : Te.style.maxWidth;
                  }
                  setMaxWidth(Te) {
                    return this.options.maxWidth = Te, this._update(), this;
                  }
                  setDOMContent(Te) {
                    if (this._content) for (; this._content.hasChildNodes(); ) this._content.firstChild && this._content.removeChild(this._content.firstChild);
                    else this._content = le.create("div", "maplibregl-popup-content", this._container);
                    return this._content.appendChild(Te), this._createCloseButton(), this._update(), this._focusFirstElement(), this;
                  }
                  addClassName(Te) {
                    return this._container && this._container.classList.add(Te), this;
                  }
                  removeClassName(Te) {
                    return this._container && this._container.classList.remove(Te), this;
                  }
                  setOffset(Te) {
                    return this.options.offset = Te, this._update(), this;
                  }
                  toggleClassName(Te) {
                    if (this._container) return this._container.classList.toggle(Te);
                  }
                  setSubpixelPositioning(Te) {
                    this.options.subpixelPositioning = Te;
                  }
                  _createCloseButton() {
                    this.options.closeButton && (this._closeButton = le.create("button", "maplibregl-popup-close-button", this._content), this._closeButton.type = "button", this._closeButton.innerHTML = "&#215;", this._closeButton.addEventListener("click", this._onClose));
                  }
                  _focusFirstElement() {
                    if (!this.options.focusAfterOpen || !this._container) return;
                    const Te = this._container.querySelector($n);
                    Te && Te.focus();
                  }
                }
                function Gn(Fe) {
                  if (Fe) {
                    if (typeof Fe == "number") {
                      const Te = Math.round(Math.abs(Fe) / Math.SQRT2);
                      return { center: new ee.P(0, 0), top: new ee.P(0, Fe), "top-left": new ee.P(Te, Te), "top-right": new ee.P(-Te, Te), bottom: new ee.P(0, -Fe), "bottom-left": new ee.P(Te, -Te), "bottom-right": new ee.P(-Te, -Te), left: new ee.P(Fe, 0), right: new ee.P(-Fe, 0) };
                    }
                    if (Fe instanceof ee.P || Array.isArray(Fe)) {
                      const Te = ee.P.convert(Fe);
                      return { center: Te, top: Te, "top-left": Te, "top-right": Te, bottom: Te, "bottom-left": Te, "bottom-right": Te, left: Te, right: Te };
                    }
                    return { center: ee.P.convert(Fe.center || [0, 0]), top: ee.P.convert(Fe.top || [0, 0]), "top-left": ee.P.convert(Fe["top-left"] || [0, 0]), "top-right": ee.P.convert(Fe["top-right"] || [0, 0]), bottom: ee.P.convert(Fe.bottom || [0, 0]), "bottom-left": ee.P.convert(Fe["bottom-left"] || [0, 0]), "bottom-right": ee.P.convert(Fe["bottom-right"] || [0, 0]), left: ee.P.convert(Fe.left || [0, 0]), right: ee.P.convert(Fe.right || [0, 0]) };
                  }
                  return Gn(new ee.P(0, 0));
                }
                const Ke = ae;
                ne.AJAXError = ee.bg, ne.Evented = ee.E, ne.LngLat = ee.N, ne.MercatorCoordinate = ee.Z, ne.Point = ee.P, ne.addProtocol = ee.bh, ne.config = ee.a, ne.removeProtocol = ee.bi, ne.AttributionControl = Vc, ne.BoxZoomHandler = ic, ne.CanvasSource = It, ne.CooperativeGesturesHandler = fc, ne.DoubleClickZoomHandler = vl, ne.DragPanHandler = ac, ne.DragRotateHandler = bc, ne.EdgeInsets = es, ne.FullscreenControl = tu, ne.GeoJSONSource = Nt, ne.GeolocateControl = Os, ne.Hash = gs, ne.ImageSource = Ft, ne.KeyboardHandler = Lc, ne.LngLatBounds = Rt, ne.LogoControl = Rc, ne.Map = class extends sc {
                  constructor(Fe) {
                    ee.be.mark(ee.bf.create);
                    const Te = Object.assign(Object.assign({}, Yc), Fe);
                    if (Te.minZoom != null && Te.maxZoom != null && Te.minZoom > Te.maxZoom) throw new Error("maxZoom must be greater than or equal to minZoom");
                    if (Te.minPitch != null && Te.maxPitch != null && Te.minPitch > Te.maxPitch) throw new Error("maxPitch must be greater than or equal to minPitch");
                    if (Te.minPitch != null && Te.minPitch < 0) throw new Error("minPitch must be greater than or equal to 0");
                    if (Te.maxPitch != null && Te.maxPitch > 85) throw new Error("maxPitch must be less than or equal to 85");
                    if (super(new Fo(Te.minZoom, Te.maxZoom, Te.minPitch, Te.maxPitch, Te.renderWorldCopies), { bearingSnap: Te.bearingSnap }), this._idleTriggered = !1, this._crossFadingFactor = 1, this._renderTaskQueue = new Zo(), this._controls = [], this._mapId = ee.a4(), this._contextLost = (Oe) => {
                      Oe.preventDefault(), this._frameRequest && (this._frameRequest.abort(), this._frameRequest = null), this.fire(new ee.k("webglcontextlost", { originalEvent: Oe }));
                    }, this._contextRestored = (Oe) => {
                      this._setupPainter(), this.resize(), this._update(), this.fire(new ee.k("webglcontextrestored", { originalEvent: Oe }));
                    }, this._onMapScroll = (Oe) => {
                      if (Oe.target === this._container) return this._container.scrollTop = 0, this._container.scrollLeft = 0, !1;
                    }, this._onWindowOnline = () => {
                      this._update();
                    }, this._interactive = Te.interactive, this._maxTileCacheSize = Te.maxTileCacheSize, this._maxTileCacheZoomLevels = Te.maxTileCacheZoomLevels, this._failIfMajorPerformanceCaveat = Te.failIfMajorPerformanceCaveat === !0, this._preserveDrawingBuffer = Te.preserveDrawingBuffer === !0, this._antialias = Te.antialias === !0, this._trackResize = Te.trackResize === !0, this._bearingSnap = Te.bearingSnap, this._refreshExpiredTiles = Te.refreshExpiredTiles === !0, this._fadeDuration = Te.fadeDuration, this._crossSourceCollisions = Te.crossSourceCollisions === !0, this._collectResourceTiming = Te.collectResourceTiming === !0, this._locale = Object.assign(Object.assign({}, gd), Te.locale), this._clickTolerance = Te.clickTolerance, this._overridePixelRatio = Te.pixelRatio, this._maxCanvasSize = Te.maxCanvasSize, this.transformCameraUpdate = Te.transformCameraUpdate, this.cancelPendingTileRequestsWhileZooming = Te.cancelPendingTileRequestsWhileZooming === !0, this._imageQueueHandle = ye.addThrottleControl(() => this.isMoving()), this._requestManager = new ge(Te.transformRequest), typeof Te.container == "string") {
                      if (this._container = document.getElementById(Te.container), !this._container) throw new Error(`Container '${Te.container}' not found.`);
                    } else {
                      if (!(Te.container instanceof HTMLElement)) throw new Error("Invalid type: 'container' must be a String or HTMLElement.");
                      this._container = Te.container;
                    }
                    if (Te.maxBounds && this.setMaxBounds(Te.maxBounds), this._setupContainer(), this._setupPainter(), this.on("move", () => this._update(!1)).on("moveend", () => this._update(!1)).on("zoom", () => this._update(!0)).on("terrain", () => {
                      this.painter.terrainFacilitator.dirty = !0, this._update(!0);
                    }).once("idle", () => {
                      this._idleTriggered = !0;
                    }), typeof window < "u") {
                      addEventListener("online", this._onWindowOnline, !1);
                      let Oe = !1;
                      const Be = Oo((Ue) => {
                        this._trackResize && !this._removed && this.resize(Ue)._update();
                      }, 50);
                      this._resizeObserver = new ResizeObserver((Ue) => {
                        Oe ? Be(Ue) : Oe = !0;
                      }), this._resizeObserver.observe(this._container);
                    }
                    this.handlers = new Tl(this, Te);
                    const Ae = typeof Te.hash == "string" && Te.hash || void 0;
                    this._hash = Te.hash && new gs(Ae).addTo(this), this._hash && this._hash._onHashChange() || (this.jumpTo({ center: Te.center, zoom: Te.zoom, bearing: Te.bearing, pitch: Te.pitch }), Te.bounds && (this.resize(), this.fitBounds(Te.bounds, ee.e({}, Te.fitBoundsOptions, { duration: 0 })))), this.resize(), this._localIdeographFontFamily = Te.localIdeographFontFamily, this._validateStyle = Te.validateStyle, Te.style && this.setStyle(Te.style, { localIdeographFontFamily: Te.localIdeographFontFamily }), Te.attributionControl && this.addControl(new Vc(typeof Te.attributionControl == "boolean" ? void 0 : Te.attributionControl)), Te.maplibreLogo && this.addControl(new Rc(), Te.logoPosition), this.on("style.load", () => {
                      this.transform.unmodified && this.jumpTo(this.style.stylesheet);
                    }), this.on("data", (Oe) => {
                      this._update(Oe.dataType === "style"), this.fire(new ee.k(`${Oe.dataType}data`, Oe));
                    }), this.on("dataloading", (Oe) => {
                      this.fire(new ee.k(`${Oe.dataType}dataloading`, Oe));
                    }), this.on("dataabort", (Oe) => {
                      this.fire(new ee.k("sourcedataabort", Oe));
                    });
                  }
                  _getMapId() {
                    return this._mapId;
                  }
                  addControl(Fe, Te) {
                    if (Te === void 0 && (Te = Fe.getDefaultPosition ? Fe.getDefaultPosition() : "top-right"), !Fe || !Fe.onAdd) return this.fire(new ee.j(new Error("Invalid argument to map.addControl(). Argument must be a control with onAdd and onRemove methods.")));
                    const Ae = Fe.onAdd(this);
                    this._controls.push(Fe);
                    const Oe = this._controlPositions[Te];
                    return Te.indexOf("bottom") !== -1 ? Oe.insertBefore(Ae, Oe.firstChild) : Oe.appendChild(Ae), this;
                  }
                  removeControl(Fe) {
                    if (!Fe || !Fe.onRemove) return this.fire(new ee.j(new Error("Invalid argument to map.removeControl(). Argument must be a control with onAdd and onRemove methods.")));
                    const Te = this._controls.indexOf(Fe);
                    return Te > -1 && this._controls.splice(Te, 1), Fe.onRemove(this), this;
                  }
                  hasControl(Fe) {
                    return this._controls.indexOf(Fe) > -1;
                  }
                  calculateCameraOptionsFromTo(Fe, Te, Ae, Oe) {
                    return Oe == null && this.terrain && (Oe = this.terrain.getElevationForLngLatZoom(Ae, this.transform.tileZoom)), super.calculateCameraOptionsFromTo(Fe, Te, Ae, Oe);
                  }
                  resize(Fe) {
                    var Te;
                    const Ae = this._containerDimensions(), Oe = Ae[0], Be = Ae[1], Ue = this._getClampedPixelRatio(Oe, Be);
                    if (this._resizeCanvas(Oe, Be, Ue), this.painter.resize(Oe, Be, Ue), this.painter.overLimit()) {
                      const ht = this.painter.context.gl;
                      this._maxCanvasSize = [ht.drawingBufferWidth, ht.drawingBufferHeight];
                      const Et = this._getClampedPixelRatio(Oe, Be);
                      this._resizeCanvas(Oe, Be, Et), this.painter.resize(Oe, Be, Et);
                    }
                    this.transform.resize(Oe, Be), (Te = this._requestedCameraState) === null || Te === void 0 || Te.resize(Oe, Be);
                    const nt = !this._moving;
                    return nt && (this.stop(), this.fire(new ee.k("movestart", Fe)).fire(new ee.k("move", Fe))), this.fire(new ee.k("resize", Fe)), nt && this.fire(new ee.k("moveend", Fe)), this;
                  }
                  _getClampedPixelRatio(Fe, Te) {
                    const { 0: Ae, 1: Oe } = this._maxCanvasSize, Be = this.getPixelRatio(), Ue = Fe * Be, nt = Te * Be, ht = Ue > Ae ? Ae / Ue : 1, Et = nt > Oe ? Oe / nt : 1;
                    return Math.min(ht, Et) * Be;
                  }
                  getPixelRatio() {
                    var Fe;
                    return (Fe = this._overridePixelRatio) !== null && Fe !== void 0 ? Fe : devicePixelRatio;
                  }
                  setPixelRatio(Fe) {
                    this._overridePixelRatio = Fe, this.resize();
                  }
                  getBounds() {
                    return this.transform.getBounds();
                  }
                  getMaxBounds() {
                    return this.transform.getMaxBounds();
                  }
                  setMaxBounds(Fe) {
                    return this.transform.setMaxBounds(Rt.convert(Fe)), this._update();
                  }
                  setMinZoom(Fe) {
                    if ((Fe = Fe ?? -2) >= -2 && Fe <= this.transform.maxZoom) return this.transform.minZoom = Fe, this._update(), this.getZoom() < Fe && this.setZoom(Fe), this;
                    throw new Error("minZoom must be between -2 and the current maxZoom, inclusive");
                  }
                  getMinZoom() {
                    return this.transform.minZoom;
                  }
                  setMaxZoom(Fe) {
                    if ((Fe = Fe ?? 22) >= this.transform.minZoom) return this.transform.maxZoom = Fe, this._update(), this.getZoom() > Fe && this.setZoom(Fe), this;
                    throw new Error("maxZoom must be greater than the current minZoom");
                  }
                  getMaxZoom() {
                    return this.transform.maxZoom;
                  }
                  setMinPitch(Fe) {
                    if ((Fe = Fe ?? 0) < 0) throw new Error("minPitch must be greater than or equal to 0");
                    if (Fe >= 0 && Fe <= this.transform.maxPitch) return this.transform.minPitch = Fe, this._update(), this.getPitch() < Fe && this.setPitch(Fe), this;
                    throw new Error("minPitch must be between 0 and the current maxPitch, inclusive");
                  }
                  getMinPitch() {
                    return this.transform.minPitch;
                  }
                  setMaxPitch(Fe) {
                    if ((Fe = Fe ?? 60) > 85) throw new Error("maxPitch must be less than or equal to 85");
                    if (Fe >= this.transform.minPitch) return this.transform.maxPitch = Fe, this._update(), this.getPitch() > Fe && this.setPitch(Fe), this;
                    throw new Error("maxPitch must be greater than the current minPitch");
                  }
                  getMaxPitch() {
                    return this.transform.maxPitch;
                  }
                  getRenderWorldCopies() {
                    return this.transform.renderWorldCopies;
                  }
                  setRenderWorldCopies(Fe) {
                    return this.transform.renderWorldCopies = Fe, this._update();
                  }
                  project(Fe) {
                    return this.transform.locationPoint(ee.N.convert(Fe), this.style && this.terrain);
                  }
                  unproject(Fe) {
                    return this.transform.pointLocation(ee.P.convert(Fe), this.terrain);
                  }
                  isMoving() {
                    var Fe;
                    return this._moving || ((Fe = this.handlers) === null || Fe === void 0 ? void 0 : Fe.isMoving());
                  }
                  isZooming() {
                    var Fe;
                    return this._zooming || ((Fe = this.handlers) === null || Fe === void 0 ? void 0 : Fe.isZooming());
                  }
                  isRotating() {
                    var Fe;
                    return this._rotating || ((Fe = this.handlers) === null || Fe === void 0 ? void 0 : Fe.isRotating());
                  }
                  _createDelegatedListener(Fe, Te, Ae) {
                    if (Fe === "mouseenter" || Fe === "mouseover") {
                      let Oe = !1;
                      return { layer: Te, listener: Ae, delegates: { mousemove: (Ue) => {
                        const nt = this.getLayer(Te) ? this.queryRenderedFeatures(Ue.point, { layers: [Te] }) : [];
                        nt.length ? Oe || (Oe = !0, Ae.call(this, new ku(Fe, this, Ue.originalEvent, { features: nt }))) : Oe = !1;
                      }, mouseout: () => {
                        Oe = !1;
                      } } };
                    }
                    if (Fe === "mouseleave" || Fe === "mouseout") {
                      let Oe = !1;
                      return { layer: Te, listener: Ae, delegates: { mousemove: (nt) => {
                        (this.getLayer(Te) ? this.queryRenderedFeatures(nt.point, { layers: [Te] }) : []).length ? Oe = !0 : Oe && (Oe = !1, Ae.call(this, new ku(Fe, this, nt.originalEvent)));
                      }, mouseout: (nt) => {
                        Oe && (Oe = !1, Ae.call(this, new ku(Fe, this, nt.originalEvent)));
                      } } };
                    }
                    {
                      const Oe = (Be) => {
                        const Ue = this.getLayer(Te) ? this.queryRenderedFeatures(Be.point, { layers: [Te] }) : [];
                        Ue.length && (Be.features = Ue, Ae.call(this, Be), delete Be.features);
                      };
                      return { layer: Te, listener: Ae, delegates: { [Fe]: Oe } };
                    }
                  }
                  on(Fe, Te, Ae) {
                    if (Ae === void 0) return super.on(Fe, Te);
                    const Oe = this._createDelegatedListener(Fe, Te, Ae);
                    this._delegatedListeners = this._delegatedListeners || {}, this._delegatedListeners[Fe] = this._delegatedListeners[Fe] || [], this._delegatedListeners[Fe].push(Oe);
                    for (const Be in Oe.delegates) this.on(Be, Oe.delegates[Be]);
                    return this;
                  }
                  once(Fe, Te, Ae) {
                    if (Ae === void 0) return super.once(Fe, Te);
                    const Oe = this._createDelegatedListener(Fe, Te, Ae);
                    for (const Be in Oe.delegates) this.once(Be, Oe.delegates[Be]);
                    return this;
                  }
                  off(Fe, Te, Ae) {
                    return Ae === void 0 ? super.off(Fe, Te) : (this._delegatedListeners && this._delegatedListeners[Fe] && ((Oe) => {
                      const Be = Oe[Fe];
                      for (let Ue = 0; Ue < Be.length; Ue++) {
                        const nt = Be[Ue];
                        if (nt.layer === Te && nt.listener === Ae) {
                          for (const ht in nt.delegates) this.off(ht, nt.delegates[ht]);
                          return Be.splice(Ue, 1), this;
                        }
                      }
                    })(this._delegatedListeners), this);
                  }
                  queryRenderedFeatures(Fe, Te) {
                    if (!this.style) return [];
                    let Ae;
                    const Oe = Fe instanceof ee.P || Array.isArray(Fe), Be = Oe ? Fe : [[0, 0], [this.transform.width, this.transform.height]];
                    if (Te = Te || (Oe ? {} : Fe) || {}, Be instanceof ee.P || typeof Be[0] == "number") Ae = [ee.P.convert(Be)];
                    else {
                      const Ue = ee.P.convert(Be[0]), nt = ee.P.convert(Be[1]);
                      Ae = [Ue, new ee.P(nt.x, Ue.y), nt, new ee.P(Ue.x, nt.y), Ue];
                    }
                    return this.style.queryRenderedFeatures(Ae, Te, this.transform);
                  }
                  querySourceFeatures(Fe, Te) {
                    return this.style.querySourceFeatures(Fe, Te);
                  }
                  setStyle(Fe, Te) {
                    return (Te = ee.e({}, { localIdeographFontFamily: this._localIdeographFontFamily, validate: this._validateStyle }, Te)).diff !== !1 && Te.localIdeographFontFamily === this._localIdeographFontFamily && this.style && Fe ? (this._diffStyle(Fe, Te), this) : (this._localIdeographFontFamily = Te.localIdeographFontFamily, this._updateStyle(Fe, Te));
                  }
                  setTransformRequest(Fe) {
                    return this._requestManager.setTransformRequest(Fe), this;
                  }
                  _getUIString(Fe) {
                    const Te = this._locale[Fe];
                    if (Te == null) throw new Error(`Missing UI string '${Fe}'`);
                    return Te;
                  }
                  _updateStyle(Fe, Te) {
                    if (Te.transformStyle && this.style && !this.style._loaded) return void this.style.once("style.load", () => this._updateStyle(Fe, Te));
                    const Ae = this.style && Te.transformStyle ? this.style.serialize() : void 0;
                    return this.style && (this.style.setEventedParent(null), this.style._remove(!Fe)), Fe ? (this.style = new ka(this, Te || {}), this.style.setEventedParent(this, { style: this.style }), typeof Fe == "string" ? this.style.loadURL(Fe, Te, Ae) : this.style.loadJSON(Fe, Te, Ae), this) : (delete this.style, this);
                  }
                  _lazyInitEmptyStyle() {
                    this.style || (this.style = new ka(this, {}), this.style.setEventedParent(this, { style: this.style }), this.style.loadEmpty());
                  }
                  _diffStyle(Fe, Te) {
                    if (typeof Fe == "string") {
                      const Ae = Fe, Oe = this._requestManager.transformRequest(Ae, "Style");
                      ee.h(Oe, new AbortController()).then((Be) => {
                        this._updateDiff(Be.data, Te);
                      }).catch((Be) => {
                        Be && this.fire(new ee.j(Be));
                      });
                    } else typeof Fe == "object" && this._updateDiff(Fe, Te);
                  }
                  _updateDiff(Fe, Te) {
                    try {
                      this.style.setState(Fe, Te) && this._update(!0);
                    } catch (Ae) {
                      ee.w(`Unable to perform style diff: ${Ae.message || Ae.error || Ae}.  Rebuilding the style from scratch.`), this._updateStyle(Fe, Te);
                    }
                  }
                  getStyle() {
                    if (this.style) return this.style.serialize();
                  }
                  isStyleLoaded() {
                    return this.style ? this.style.loaded() : ee.w("There is no style added to the map.");
                  }
                  addSource(Fe, Te) {
                    return this._lazyInitEmptyStyle(), this.style.addSource(Fe, Te), this._update(!0);
                  }
                  isSourceLoaded(Fe) {
                    const Te = this.style && this.style.sourceCaches[Fe];
                    if (Te !== void 0) return Te.loaded();
                    this.fire(new ee.j(new Error(`There is no source with ID '${Fe}'`)));
                  }
                  setTerrain(Fe) {
                    if (this.style._checkLoaded(), this._terrainDataCallback && this.style.off("data", this._terrainDataCallback), Fe) {
                      const Te = this.style.sourceCaches[Fe.source];
                      if (!Te) throw new Error(`cannot load terrain, because there exists no source with ID: ${Fe.source}`);
                      this.terrain === null && Te.reload();
                      for (const Ae in this.style._layers) {
                        const Oe = this.style._layers[Ae];
                        Oe.type === "hillshade" && Oe.source === Fe.source && ee.w("You are using the same source for a hillshade layer and for 3D terrain. Please consider using two separate sources to improve rendering quality.");
                      }
                      this.terrain = new El(this.painter, Te, Fe), this.painter.renderToTexture = new hu(this.painter, this.terrain), this.transform.minElevationForCurrentTile = this.terrain.getMinTileElevationForLngLatZoom(this.transform.center, this.transform.tileZoom), this.transform.elevation = this.terrain.getElevationForLngLatZoom(this.transform.center, this.transform.tileZoom), this._terrainDataCallback = (Ae) => {
                        Ae.dataType === "style" ? this.terrain.sourceCache.freeRtt() : Ae.dataType === "source" && Ae.tile && (Ae.sourceId !== Fe.source || this._elevationFreeze || (this.transform.minElevationForCurrentTile = this.terrain.getMinTileElevationForLngLatZoom(this.transform.center, this.transform.tileZoom), this.transform.elevation = this.terrain.getElevationForLngLatZoom(this.transform.center, this.transform.tileZoom)), this.terrain.sourceCache.freeRtt(Ae.tile.tileID));
                      }, this.style.on("data", this._terrainDataCallback);
                    } else this.terrain && this.terrain.sourceCache.destruct(), this.terrain = null, this.painter.renderToTexture && this.painter.renderToTexture.destruct(), this.painter.renderToTexture = null, this.transform.minElevationForCurrentTile = 0, this.transform.elevation = 0;
                    return this.fire(new ee.k("terrain", { terrain: Fe })), this;
                  }
                  getTerrain() {
                    var Fe, Te;
                    return (Te = (Fe = this.terrain) === null || Fe === void 0 ? void 0 : Fe.options) !== null && Te !== void 0 ? Te : null;
                  }
                  areTilesLoaded() {
                    const Fe = this.style && this.style.sourceCaches;
                    for (const Te in Fe) {
                      const Ae = Fe[Te]._tiles;
                      for (const Oe in Ae) {
                        const Be = Ae[Oe];
                        if (Be.state !== "loaded" && Be.state !== "errored") return !1;
                      }
                    }
                    return !0;
                  }
                  removeSource(Fe) {
                    return this.style.removeSource(Fe), this._update(!0);
                  }
                  getSource(Fe) {
                    return this.style.getSource(Fe);
                  }
                  addImage(Fe, Te, Ae = {}) {
                    const { pixelRatio: Oe = 1, sdf: Be = !1, stretchX: Ue, stretchY: nt, content: ht, textFitWidth: Et, textFitHeight: Mt } = Ae;
                    if (this._lazyInitEmptyStyle(), !(Te instanceof HTMLImageElement || ee.b(Te))) {
                      if (Te.width === void 0 || Te.height === void 0) return this.fire(new ee.j(new Error("Invalid arguments to map.addImage(). The second argument must be an `HTMLImageElement`, `ImageData`, `ImageBitmap`, or object with `width`, `height`, and `data` properties with the same format as `ImageData`")));
                      {
                        const { width: qt, height: tn, data: xn } = Te, An = Te;
                        return this.style.addImage(Fe, { data: new ee.R({ width: qt, height: tn }, new Uint8Array(xn)), pixelRatio: Oe, stretchX: Ue, stretchY: nt, content: ht, textFitWidth: Et, textFitHeight: Mt, sdf: Be, version: 0, userImage: An }), An.onAdd && An.onAdd(this, Fe), this;
                      }
                    }
                    {
                      const { width: qt, height: tn, data: xn } = ce.getImageData(Te);
                      this.style.addImage(Fe, { data: new ee.R({ width: qt, height: tn }, xn), pixelRatio: Oe, stretchX: Ue, stretchY: nt, content: ht, textFitWidth: Et, textFitHeight: Mt, sdf: Be, version: 0 });
                    }
                  }
                  updateImage(Fe, Te) {
                    const Ae = this.style.getImage(Fe);
                    if (!Ae) return this.fire(new ee.j(new Error("The map has no image with that id. If you are adding a new image use `map.addImage(...)` instead.")));
                    const Oe = Te instanceof HTMLImageElement || ee.b(Te) ? ce.getImageData(Te) : Te, { width: Be, height: Ue, data: nt } = Oe;
                    if (Be === void 0 || Ue === void 0) return this.fire(new ee.j(new Error("Invalid arguments to map.updateImage(). The second argument must be an `HTMLImageElement`, `ImageData`, `ImageBitmap`, or object with `width`, `height`, and `data` properties with the same format as `ImageData`")));
                    if (Be !== Ae.data.width || Ue !== Ae.data.height) return this.fire(new ee.j(new Error("The width and height of the updated image must be that same as the previous version of the image")));
                    const ht = !(Te instanceof HTMLImageElement || ee.b(Te));
                    return Ae.data.replace(nt, ht), this.style.updateImage(Fe, Ae), this;
                  }
                  getImage(Fe) {
                    return this.style.getImage(Fe);
                  }
                  hasImage(Fe) {
                    return Fe ? !!this.style.getImage(Fe) : (this.fire(new ee.j(new Error("Missing required image id"))), !1);
                  }
                  removeImage(Fe) {
                    this.style.removeImage(Fe);
                  }
                  loadImage(Fe) {
                    return ye.getImage(this._requestManager.transformRequest(Fe, "Image"), new AbortController());
                  }
                  listImages() {
                    return this.style.listImages();
                  }
                  addLayer(Fe, Te) {
                    return this._lazyInitEmptyStyle(), this.style.addLayer(Fe, Te), this._update(!0);
                  }
                  moveLayer(Fe, Te) {
                    return this.style.moveLayer(Fe, Te), this._update(!0);
                  }
                  removeLayer(Fe) {
                    return this.style.removeLayer(Fe), this._update(!0);
                  }
                  getLayer(Fe) {
                    return this.style.getLayer(Fe);
                  }
                  getLayersOrder() {
                    return this.style.getLayersOrder();
                  }
                  setLayerZoomRange(Fe, Te, Ae) {
                    return this.style.setLayerZoomRange(Fe, Te, Ae), this._update(!0);
                  }
                  setFilter(Fe, Te, Ae = {}) {
                    return this.style.setFilter(Fe, Te, Ae), this._update(!0);
                  }
                  getFilter(Fe) {
                    return this.style.getFilter(Fe);
                  }
                  setPaintProperty(Fe, Te, Ae, Oe = {}) {
                    return this.style.setPaintProperty(Fe, Te, Ae, Oe), this._update(!0);
                  }
                  getPaintProperty(Fe, Te) {
                    return this.style.getPaintProperty(Fe, Te);
                  }
                  setLayoutProperty(Fe, Te, Ae, Oe = {}) {
                    return this.style.setLayoutProperty(Fe, Te, Ae, Oe), this._update(!0);
                  }
                  getLayoutProperty(Fe, Te) {
                    return this.style.getLayoutProperty(Fe, Te);
                  }
                  setGlyphs(Fe, Te = {}) {
                    return this._lazyInitEmptyStyle(), this.style.setGlyphs(Fe, Te), this._update(!0);
                  }
                  getGlyphs() {
                    return this.style.getGlyphsUrl();
                  }
                  addSprite(Fe, Te, Ae = {}) {
                    return this._lazyInitEmptyStyle(), this.style.addSprite(Fe, Te, Ae, (Oe) => {
                      Oe || this._update(!0);
                    }), this;
                  }
                  removeSprite(Fe) {
                    return this._lazyInitEmptyStyle(), this.style.removeSprite(Fe), this._update(!0);
                  }
                  getSprite() {
                    return this.style.getSprite();
                  }
                  setSprite(Fe, Te = {}) {
                    return this._lazyInitEmptyStyle(), this.style.setSprite(Fe, Te, (Ae) => {
                      Ae || this._update(!0);
                    }), this;
                  }
                  setLight(Fe, Te = {}) {
                    return this._lazyInitEmptyStyle(), this.style.setLight(Fe, Te), this._update(!0);
                  }
                  getLight() {
                    return this.style.getLight();
                  }
                  setSky(Fe) {
                    return this._lazyInitEmptyStyle(), this.style.setSky(Fe), this._update(!0);
                  }
                  getSky() {
                    return this.style.getSky();
                  }
                  setFeatureState(Fe, Te) {
                    return this.style.setFeatureState(Fe, Te), this._update();
                  }
                  removeFeatureState(Fe, Te) {
                    return this.style.removeFeatureState(Fe, Te), this._update();
                  }
                  getFeatureState(Fe) {
                    return this.style.getFeatureState(Fe);
                  }
                  getContainer() {
                    return this._container;
                  }
                  getCanvasContainer() {
                    return this._canvasContainer;
                  }
                  getCanvas() {
                    return this._canvas;
                  }
                  _containerDimensions() {
                    let Fe = 0, Te = 0;
                    return this._container && (Fe = this._container.clientWidth || 400, Te = this._container.clientHeight || 300), [Fe, Te];
                  }
                  _setupContainer() {
                    const Fe = this._container;
                    Fe.classList.add("maplibregl-map");
                    const Te = this._canvasContainer = le.create("div", "maplibregl-canvas-container", Fe);
                    this._interactive && Te.classList.add("maplibregl-interactive"), this._canvas = le.create("canvas", "maplibregl-canvas", Te), this._canvas.addEventListener("webglcontextlost", this._contextLost, !1), this._canvas.addEventListener("webglcontextrestored", this._contextRestored, !1), this._canvas.setAttribute("tabindex", this._interactive ? "0" : "-1"), this._canvas.setAttribute("aria-label", this._getUIString("Map.Title")), this._canvas.setAttribute("role", "region");
                    const Ae = this._containerDimensions(), Oe = this._getClampedPixelRatio(Ae[0], Ae[1]);
                    this._resizeCanvas(Ae[0], Ae[1], Oe);
                    const Be = this._controlContainer = le.create("div", "maplibregl-control-container", Fe), Ue = this._controlPositions = {};
                    ["top-left", "top-right", "bottom-left", "bottom-right"].forEach((nt) => {
                      Ue[nt] = le.create("div", `maplibregl-ctrl-${nt} `, Be);
                    }), this._container.addEventListener("scroll", this._onMapScroll, !1);
                  }
                  _resizeCanvas(Fe, Te, Ae) {
                    this._canvas.width = Math.floor(Ae * Fe), this._canvas.height = Math.floor(Ae * Te), this._canvas.style.width = `${Fe}px`, this._canvas.style.height = `${Te}px`;
                  }
                  _setupPainter() {
                    const Fe = { alpha: !0, stencil: !0, depth: !0, failIfMajorPerformanceCaveat: this._failIfMajorPerformanceCaveat, preserveDrawingBuffer: this._preserveDrawingBuffer, antialias: this._antialias || !1 };
                    let Te = null;
                    this._canvas.addEventListener("webglcontextcreationerror", (Oe) => {
                      Te = { requestedAttributes: Fe }, Oe && (Te.statusMessage = Oe.statusMessage, Te.type = Oe.type);
                    }, { once: !0 });
                    const Ae = this._canvas.getContext("webgl2", Fe) || this._canvas.getContext("webgl", Fe);
                    if (!Ae) {
                      const Oe = "Failed to initialize WebGL";
                      throw Te ? (Te.message = Oe, new Error(JSON.stringify(Te))) : new Error(Oe);
                    }
                    this.painter = new Vo(Ae, this.transform), ue.testSupport(Ae);
                  }
                  loaded() {
                    return !this._styleDirty && !this._sourcesDirty && !!this.style && this.style.loaded();
                  }
                  _update(Fe) {
                    return this.style && this.style._loaded ? (this._styleDirty = this._styleDirty || Fe, this._sourcesDirty = !0, this.triggerRepaint(), this) : this;
                  }
                  _requestRenderFrame(Fe) {
                    return this._update(), this._renderTaskQueue.add(Fe);
                  }
                  _cancelRenderFrame(Fe) {
                    this._renderTaskQueue.remove(Fe);
                  }
                  _render(Fe) {
                    const Te = this._idleTriggered ? this._fadeDuration : 0;
                    if (this.painter.context.setDirty(), this.painter.setBaseState(), this._renderTaskQueue.run(Fe), this._removed) return;
                    let Ae = !1;
                    if (this.style && this._styleDirty) {
                      this._styleDirty = !1;
                      const Be = this.transform.zoom, Ue = ce.now();
                      this.style.zoomHistory.update(Be, Ue);
                      const nt = new ee.a9(Be, { now: Ue, fadeDuration: Te, zoomHistory: this.style.zoomHistory, transition: this.style.getTransition() }), ht = nt.crossFadingFactor();
                      ht === 1 && ht === this._crossFadingFactor || (Ae = !0, this._crossFadingFactor = ht), this.style.update(nt);
                    }
                    this.style && this._sourcesDirty && (this._sourcesDirty = !1, this.style._updateSources(this.transform)), this.terrain ? (this.terrain.sourceCache.update(this.transform, this.terrain), this.transform.minElevationForCurrentTile = this.terrain.getMinTileElevationForLngLatZoom(this.transform.center, this.transform.tileZoom), this._elevationFreeze || (this.transform.elevation = this.terrain.getElevationForLngLatZoom(this.transform.center, this.transform.tileZoom))) : (this.transform.minElevationForCurrentTile = 0, this.transform.elevation = 0), this._placementDirty = this.style && this.style._updatePlacement(this.painter.transform, this.showCollisionBoxes, Te, this._crossSourceCollisions), this.painter.render(this.style, { showTileBoundaries: this.showTileBoundaries, showOverdrawInspector: this._showOverdrawInspector, rotating: this.isRotating(), zooming: this.isZooming(), moving: this.isMoving(), fadeDuration: Te, showPadding: this.showPadding }), this.fire(new ee.k("render")), this.loaded() && !this._loaded && (this._loaded = !0, ee.be.mark(ee.bf.load), this.fire(new ee.k("load"))), this.style && (this.style.hasTransitions() || Ae) && (this._styleDirty = !0), this.style && !this._placementDirty && this.style._releaseSymbolFadeTiles();
                    const Oe = this._sourcesDirty || this._styleDirty || this._placementDirty;
                    return Oe || this._repaint ? this.triggerRepaint() : !this.isMoving() && this.loaded() && this.fire(new ee.k("idle")), !this._loaded || this._fullyLoaded || Oe || (this._fullyLoaded = !0, ee.be.mark(ee.bf.fullLoad)), this;
                  }
                  redraw() {
                    return this.style && (this._frameRequest && (this._frameRequest.abort(), this._frameRequest = null), this._render(0)), this;
                  }
                  remove() {
                    var Fe;
                    this._hash && this._hash.remove();
                    for (const Ae of this._controls) Ae.onRemove(this);
                    this._controls = [], this._frameRequest && (this._frameRequest.abort(), this._frameRequest = null), this._renderTaskQueue.clear(), this.painter.destroy(), this.handlers.destroy(), delete this.handlers, this.setStyle(null), typeof window < "u" && removeEventListener("online", this._onWindowOnline, !1), ye.removeThrottleControl(this._imageQueueHandle), (Fe = this._resizeObserver) === null || Fe === void 0 || Fe.disconnect();
                    const Te = this.painter.context.gl.getExtension("WEBGL_lose_context");
                    Te != null && Te.loseContext && Te.loseContext(), this._canvas.removeEventListener("webglcontextrestored", this._contextRestored, !1), this._canvas.removeEventListener("webglcontextlost", this._contextLost, !1), le.remove(this._canvasContainer), le.remove(this._controlContainer), this._container.classList.remove("maplibregl-map"), ee.be.clearMetrics(), this._removed = !0, this.fire(new ee.k("remove"));
                  }
                  triggerRepaint() {
                    this.style && !this._frameRequest && (this._frameRequest = new AbortController(), ce.frameAsync(this._frameRequest).then((Fe) => {
                      ee.be.frame(Fe), this._frameRequest = null, this._render(Fe);
                    }).catch(() => {
                    }));
                  }
                  get showTileBoundaries() {
                    return !!this._showTileBoundaries;
                  }
                  set showTileBoundaries(Fe) {
                    this._showTileBoundaries !== Fe && (this._showTileBoundaries = Fe, this._update());
                  }
                  get showPadding() {
                    return !!this._showPadding;
                  }
                  set showPadding(Fe) {
                    this._showPadding !== Fe && (this._showPadding = Fe, this._update());
                  }
                  get showCollisionBoxes() {
                    return !!this._showCollisionBoxes;
                  }
                  set showCollisionBoxes(Fe) {
                    this._showCollisionBoxes !== Fe && (this._showCollisionBoxes = Fe, Fe ? this.style._generateCollisionBoxes() : this._update());
                  }
                  get showOverdrawInspector() {
                    return !!this._showOverdrawInspector;
                  }
                  set showOverdrawInspector(Fe) {
                    this._showOverdrawInspector !== Fe && (this._showOverdrawInspector = Fe, this._update());
                  }
                  get repaint() {
                    return !!this._repaint;
                  }
                  set repaint(Fe) {
                    this._repaint !== Fe && (this._repaint = Fe, this.triggerRepaint());
                  }
                  get vertices() {
                    return !!this._vertices;
                  }
                  set vertices(Fe) {
                    this._vertices = Fe, this._update();
                  }
                  get version() {
                    return Jl;
                  }
                  getCameraTargetElevation() {
                    return this.transform.elevation;
                  }
                }, ne.MapMouseEvent = ku, ne.MapTouchEvent = Wu, ne.MapWheelEvent = T1, ne.Marker = $c, ne.NavigationControl = class {
                  constructor(Fe) {
                    this._updateZoomButtons = () => {
                      const Te = this._map.getZoom(), Ae = Te === this._map.getMaxZoom(), Oe = Te === this._map.getMinZoom();
                      this._zoomInButton.disabled = Ae, this._zoomOutButton.disabled = Oe, this._zoomInButton.setAttribute("aria-disabled", Ae.toString()), this._zoomOutButton.setAttribute("aria-disabled", Oe.toString());
                    }, this._rotateCompassArrow = () => {
                      const Te = this.options.visualizePitch ? `scale(${1 / Math.pow(Math.cos(this._map.transform.pitch * (Math.PI / 180)), 0.5)}) rotateX(${this._map.transform.pitch}deg) rotateZ(${this._map.transform.angle * (180 / Math.PI)}deg)` : `rotate(${this._map.transform.angle * (180 / Math.PI)}deg)`;
                      this._compassIcon.style.transform = Te;
                    }, this._setButtonTitle = (Te, Ae) => {
                      const Oe = this._map._getUIString(`NavigationControl.${Ae}`);
                      Te.title = Oe, Te.setAttribute("aria-label", Oe);
                    }, this.options = ee.e({}, Ko, Fe), this._container = le.create("div", "maplibregl-ctrl maplibregl-ctrl-group"), this._container.addEventListener("contextmenu", (Te) => Te.preventDefault()), this.options.showZoom && (this._zoomInButton = this._createButton("maplibregl-ctrl-zoom-in", (Te) => this._map.zoomIn({}, { originalEvent: Te })), le.create("span", "maplibregl-ctrl-icon", this._zoomInButton).setAttribute("aria-hidden", "true"), this._zoomOutButton = this._createButton("maplibregl-ctrl-zoom-out", (Te) => this._map.zoomOut({}, { originalEvent: Te })), le.create("span", "maplibregl-ctrl-icon", this._zoomOutButton).setAttribute("aria-hidden", "true")), this.options.showCompass && (this._compass = this._createButton("maplibregl-ctrl-compass", (Te) => {
                      this.options.visualizePitch ? this._map.resetNorthPitch({}, { originalEvent: Te }) : this._map.resetNorth({}, { originalEvent: Te });
                    }), this._compassIcon = le.create("span", "maplibregl-ctrl-icon", this._compass), this._compassIcon.setAttribute("aria-hidden", "true"));
                  }
                  onAdd(Fe) {
                    return this._map = Fe, this.options.showZoom && (this._setButtonTitle(this._zoomInButton, "ZoomIn"), this._setButtonTitle(this._zoomOutButton, "ZoomOut"), this._map.on("zoom", this._updateZoomButtons), this._updateZoomButtons()), this.options.showCompass && (this._setButtonTitle(this._compass, "ResetBearing"), this.options.visualizePitch && this._map.on("pitch", this._rotateCompassArrow), this._map.on("rotate", this._rotateCompassArrow), this._rotateCompassArrow(), this._handler = new ks(this._map, this._compass, this.options.visualizePitch)), this._container;
                  }
                  onRemove() {
                    le.remove(this._container), this.options.showZoom && this._map.off("zoom", this._updateZoomButtons), this.options.showCompass && (this.options.visualizePitch && this._map.off("pitch", this._rotateCompassArrow), this._map.off("rotate", this._rotateCompassArrow), this._handler.off(), delete this._handler), delete this._map;
                  }
                  _createButton(Fe, Te) {
                    const Ae = le.create("button", Fe, this._container);
                    return Ae.type = "button", Ae.addEventListener("click", Te), Ae;
                  }
                }, ne.Popup = zn, ne.RasterDEMTileSource = Xt, ne.RasterTileSource = Pt, ne.ScaleControl = class {
                  constructor(Fe) {
                    this._onMove = () => {
                      xc(this._map, this._container, this.options);
                    }, this.setUnit = (Te) => {
                      this.options.unit = Te, xc(this._map, this._container, this.options);
                    }, this.options = Object.assign(Object.assign({}, lc), Fe);
                  }
                  getDefaultPosition() {
                    return "bottom-left";
                  }
                  onAdd(Fe) {
                    return this._map = Fe, this._container = le.create("div", "maplibregl-ctrl maplibregl-ctrl-scale", Fe.getContainer()), this._map.on("move", this._onMove), this._onMove(), this._container;
                  }
                  onRemove() {
                    le.remove(this._container), this._map.off("move", this._onMove), this._map = void 0;
                  }
                }, ne.ScrollZoomHandler = dc, ne.Style = ka, ne.TerrainControl = class {
                  constructor(Fe) {
                    this._toggleTerrain = () => {
                      this._map.getTerrain() ? this._map.setTerrain(null) : this._map.setTerrain(this.options), this._updateTerrainIcon();
                    }, this._updateTerrainIcon = () => {
                      this._terrainButton.classList.remove("maplibregl-ctrl-terrain"), this._terrainButton.classList.remove("maplibregl-ctrl-terrain-enabled"), this._map.terrain ? (this._terrainButton.classList.add("maplibregl-ctrl-terrain-enabled"), this._terrainButton.title = this._map._getUIString("TerrainControl.Disable")) : (this._terrainButton.classList.add("maplibregl-ctrl-terrain"), this._terrainButton.title = this._map._getUIString("TerrainControl.Enable"));
                    }, this.options = Fe;
                  }
                  onAdd(Fe) {
                    return this._map = Fe, this._container = le.create("div", "maplibregl-ctrl maplibregl-ctrl-group"), this._terrainButton = le.create("button", "maplibregl-ctrl-terrain", this._container), le.create("span", "maplibregl-ctrl-icon", this._terrainButton).setAttribute("aria-hidden", "true"), this._terrainButton.type = "button", this._terrainButton.addEventListener("click", this._toggleTerrain), this._updateTerrainIcon(), this._map.on("terrain", this._updateTerrainIcon), this._container;
                  }
                  onRemove() {
                    le.remove(this._container), this._map.off("terrain", this._updateTerrainIcon), this._map = void 0;
                  }
                }, ne.TwoFingersTouchPitchHandler = Gc, ne.TwoFingersTouchRotateHandler = Hu, ne.TwoFingersTouchZoomHandler = cs, ne.TwoFingersTouchZoomRotateHandler = kc, ne.VectorTileSource = $t, ne.VideoSource = Tt, ne.addSourceType = (Fe, Te) => ee._(void 0, void 0, void 0, function* () {
                  if (Kt(Fe)) throw new Error(`A source type called "${Fe}" already exists.`);
                  ((Ae, Oe) => {
                    Ct[Ae] = Oe;
                  })(Fe, Te);
                }), ne.clearPrewarmedResources = function() {
                  const Fe = dt;
                  Fe && (Fe.isPreloaded() && Fe.numActive() === 1 ? (Fe.release(at), dt = null) : console.warn("Could not clear WebWorkers since there are active Map instances that still reference it. The pre-warmed WebWorker pool can only be cleared when all map instances have been removed with map.remove()"));
                }, ne.getMaxParallelImageRequests = function() {
                  return ee.a.MAX_PARALLEL_IMAGE_REQUESTS;
                }, ne.getRTLTextPluginStatus = function() {
                  return hn().getRTLTextPluginStatus();
                }, ne.getVersion = function() {
                  return Ke;
                }, ne.getWorkerCount = function() {
                  return ct.workerCount;
                }, ne.getWorkerUrl = function() {
                  return ee.a.WORKER_URL;
                }, ne.importScriptInWorkers = function(Fe) {
                  return gt().broadcast("IS", Fe);
                }, ne.prewarm = function() {
                  it().acquire(at);
                }, ne.setMaxParallelImageRequests = function(Fe) {
                  ee.a.MAX_PARALLEL_IMAGE_REQUESTS = Fe;
                }, ne.setRTLTextPlugin = function(Fe, Te) {
                  return hn().setRTLTextPlugin(Fe, Te);
                }, ne.setWorkerCount = function(Fe) {
                  ct.workerCount = Fe;
                }, ne.setWorkerUrl = function(Fe) {
                  ee.a.WORKER_URL = Fe;
                };
              }), J;
            }();
          }, 88640: function(G, J, Z) {
            function te(St, Lt, Rt) {
              St.prototype = Lt.prototype = Rt, Rt.constructor = St;
            }
            function ne(St, Lt) {
              var Rt = Object.create(St.prototype);
              for (var vt in Lt) Rt[vt] = Lt[vt];
              return Rt;
            }
            function ee() {
            }
            Z.d(J, { GW: function() {
              return yt;
            }, Dj: function() {
              return ct;
            } });
            var ae = 0.7, oe = 1 / ae, se = "\\s*([+-]?\\d+)\\s*", ce = "\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)\\s*", le = "\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)%\\s*", ue = /^#([0-9a-f]{3,8})$/, de = new RegExp("^rgb\\(".concat(se, ",").concat(se, ",").concat(se, "\\)$")), fe = new RegExp("^rgb\\(".concat(le, ",").concat(le, ",").concat(le, "\\)$")), he = new RegExp("^rgba\\(".concat(se, ",").concat(se, ",").concat(se, ",").concat(ce, "\\)$")), me = new RegExp("^rgba\\(".concat(le, ",").concat(le, ",").concat(le, ",").concat(ce, "\\)$")), pe = new RegExp("^hsl\\(".concat(ce, ",").concat(le, ",").concat(le, "\\)$")), ye = new RegExp("^hsla\\(".concat(ce, ",").concat(le, ",").concat(le, ",").concat(ce, "\\)$")), ge = { aliceblue: 15792383, antiquewhite: 16444375, aqua: 65535, aquamarine: 8388564, azure: 15794175, beige: 16119260, bisque: 16770244, black: 0, blanchedalmond: 16772045, blue: 255, blueviolet: 9055202, brown: 10824234, burlywood: 14596231, cadetblue: 6266528, chartreuse: 8388352, chocolate: 13789470, coral: 16744272, cornflowerblue: 6591981, cornsilk: 16775388, crimson: 14423100, cyan: 65535, darkblue: 139, darkcyan: 35723, darkgoldenrod: 12092939, darkgray: 11119017, darkgreen: 25600, darkgrey: 11119017, darkkhaki: 12433259, darkmagenta: 9109643, darkolivegreen: 5597999, darkorange: 16747520, darkorchid: 10040012, darkred: 9109504, darksalmon: 15308410, darkseagreen: 9419919, darkslateblue: 4734347, darkslategray: 3100495, darkslategrey: 3100495, darkturquoise: 52945, darkviolet: 9699539, deeppink: 16716947, deepskyblue: 49151, dimgray: 6908265, dimgrey: 6908265, dodgerblue: 2003199, firebrick: 11674146, floralwhite: 16775920, forestgreen: 2263842, fuchsia: 16711935, gainsboro: 14474460, ghostwhite: 16316671, gold: 16766720, goldenrod: 14329120, gray: 8421504, green: 32768, greenyellow: 11403055, grey: 8421504, honeydew: 15794160, hotpink: 16738740, indianred: 13458524, indigo: 4915330, ivory: 16777200, khaki: 15787660, lavender: 15132410, lavenderblush: 16773365, lawngreen: 8190976, lemonchiffon: 16775885, lightblue: 11393254, lightcoral: 15761536, lightcyan: 14745599, lightgoldenrodyellow: 16448210, lightgray: 13882323, lightgreen: 9498256, lightgrey: 13882323, lightpink: 16758465, lightsalmon: 16752762, lightseagreen: 2142890, lightskyblue: 8900346, lightslategray: 7833753, lightslategrey: 7833753, lightsteelblue: 11584734, lightyellow: 16777184, lime: 65280, limegreen: 3329330, linen: 16445670, magenta: 16711935, maroon: 8388608, mediumaquamarine: 6737322, mediumblue: 205, mediumorchid: 12211667, mediumpurple: 9662683, mediumseagreen: 3978097, mediumslateblue: 8087790, mediumspringgreen: 64154, mediumturquoise: 4772300, mediumvioletred: 13047173, midnightblue: 1644912, mintcream: 16121850, mistyrose: 16770273, moccasin: 16770229, navajowhite: 16768685, navy: 128, oldlace: 16643558, olive: 8421376, olivedrab: 7048739, orange: 16753920, orangered: 16729344, orchid: 14315734, palegoldenrod: 15657130, palegreen: 10025880, paleturquoise: 11529966, palevioletred: 14381203, papayawhip: 16773077, peachpuff: 16767673, peru: 13468991, pink: 16761035, plum: 14524637, powderblue: 11591910, purple: 8388736, rebeccapurple: 6697881, red: 16711680, rosybrown: 12357519, royalblue: 4286945, saddlebrown: 9127187, salmon: 16416882, sandybrown: 16032864, seagreen: 3050327, seashell: 16774638, sienna: 10506797, silver: 12632256, skyblue: 8900331, slateblue: 6970061, slategray: 7372944, slategrey: 7372944, snow: 16775930, springgreen: 65407, steelblue: 4620980, tan: 13808780, teal: 32896, thistle: 14204888, tomato: 16737095, turquoise: 4251856, violet: 15631086, wheat: 16113331, white: 16777215, whitesmoke: 16119285, yellow: 16776960, yellowgreen: 10145074 };
            function ve() {
              return this.rgb().formatHex();
            }
            function be() {
              return this.rgb().formatRgb();
            }
            function Ce(St) {
              var Lt, Rt;
              return St = (St + "").trim().toLowerCase(), (Lt = ue.exec(St)) ? (Rt = Lt[1].length, Lt = parseInt(Lt[1], 16), Rt === 6 ? Se(Lt) : Rt === 3 ? new xe(Lt >> 8 & 15 | Lt >> 4 & 240, Lt >> 4 & 15 | 240 & Lt, (15 & Lt) << 4 | 15 & Lt, 1) : Rt === 8 ? we(Lt >> 24 & 255, Lt >> 16 & 255, Lt >> 8 & 255, (255 & Lt) / 255) : Rt === 4 ? we(Lt >> 12 & 15 | Lt >> 8 & 240, Lt >> 8 & 15 | Lt >> 4 & 240, Lt >> 4 & 15 | 240 & Lt, ((15 & Lt) << 4 | 15 & Lt) / 255) : null) : (Lt = de.exec(St)) ? new xe(Lt[1], Lt[2], Lt[3], 1) : (Lt = fe.exec(St)) ? new xe(255 * Lt[1] / 100, 255 * Lt[2] / 100, 255 * Lt[3] / 100, 1) : (Lt = he.exec(St)) ? we(Lt[1], Lt[2], Lt[3], Lt[4]) : (Lt = me.exec(St)) ? we(255 * Lt[1] / 100, 255 * Lt[2] / 100, 255 * Lt[3] / 100, Lt[4]) : (Lt = pe.exec(St)) ? De(Lt[1], Lt[2] / 100, Lt[3] / 100, 1) : (Lt = ye.exec(St)) ? De(Lt[1], Lt[2] / 100, Lt[3] / 100, Lt[4]) : ge.hasOwnProperty(St) ? Se(ge[St]) : St === "transparent" ? new xe(NaN, NaN, NaN, 0) : null;
            }
            function Se(St) {
              return new xe(St >> 16 & 255, St >> 8 & 255, 255 & St, 1);
            }
            function we(St, Lt, Rt, vt) {
              return vt <= 0 && (St = Lt = Rt = NaN), new xe(St, Lt, Rt, vt);
            }
            function Ee(St, Lt, Rt, vt) {
              return arguments.length === 1 ? (($t = St) instanceof ee || ($t = Ce($t)), $t ? new xe(($t = $t.rgb()).r, $t.g, $t.b, $t.opacity) : new xe()) : new xe(St, Lt, Rt, vt ?? 1);
              var $t;
            }
            function xe(St, Lt, Rt, vt) {
              this.r = +St, this.g = +Lt, this.b = +Rt, this.opacity = +vt;
            }
            function Le() {
              return "#".concat(Re(this.r)).concat(Re(this.g)).concat(Re(this.b));
            }
            function Qe() {
              var St = ke(this.opacity);
              return "".concat(St === 1 ? "rgb(" : "rgba(").concat(Me(this.r), ", ").concat(Me(this.g), ", ").concat(Me(this.b)).concat(St === 1 ? ")" : ", ".concat(St, ")"));
            }
            function ke(St) {
              return isNaN(St) ? 1 : Math.max(0, Math.min(1, St));
            }
            function Me(St) {
              return Math.max(0, Math.min(255, Math.round(St) || 0));
            }
            function Re(St) {
              return ((St = Me(St)) < 16 ? "0" : "") + St.toString(16);
            }
            function De(St, Lt, Rt, vt) {
              return vt <= 0 ? St = Lt = Rt = NaN : Rt <= 0 || Rt >= 1 ? St = Lt = NaN : Lt <= 0 && (St = NaN), new $e(St, Lt, Rt, vt);
            }
            function Pe(St) {
              if (St instanceof $e) return new $e(St.h, St.s, St.l, St.opacity);
              if (St instanceof ee || (St = Ce(St)), !St) return new $e();
              if (St instanceof $e) return St;
              var Lt = (St = St.rgb()).r / 255, Rt = St.g / 255, vt = St.b / 255, $t = Math.min(Lt, Rt, vt), Pt = Math.max(Lt, Rt, vt), Xt = NaN, Nt = Pt - $t, Gt = (Pt + $t) / 2;
              return Nt ? (Xt = Lt === Pt ? (Rt - vt) / Nt + 6 * (Rt < vt) : Rt === Pt ? (vt - Lt) / Nt + 2 : (Lt - Rt) / Nt + 4, Nt /= Gt < 0.5 ? Pt + $t : 2 - Pt - $t, Xt *= 60) : Nt = Gt > 0 && Gt < 1 ? 0 : Xt, new $e(Xt, Nt, Gt, St.opacity);
            }
            function $e(St, Lt, Rt, vt) {
              this.h = +St, this.s = +Lt, this.l = +Rt, this.opacity = +vt;
            }
            function qe(St) {
              return (St = (St || 0) % 360) < 0 ? St + 360 : St;
            }
            function Xe(St) {
              return Math.max(0, Math.min(1, St || 0));
            }
            function Ve(St, Lt, Rt) {
              return 255 * (St < 60 ? Lt + (Rt - Lt) * St / 60 : St < 180 ? Rt : St < 240 ? Lt + (Rt - Lt) * (240 - St) / 60 : Lt);
            }
            te(ee, Ce, { copy: function(St) {
              return Object.assign(new this.constructor(), this, St);
            }, displayable: function() {
              return this.rgb().displayable();
            }, hex: ve, formatHex: ve, formatHex8: function() {
              return this.rgb().formatHex8();
            }, formatHsl: function() {
              return Pe(this).formatHsl();
            }, formatRgb: be, toString: be }), te(xe, Ee, ne(ee, { brighter: function(St) {
              return St = St == null ? oe : Math.pow(oe, St), new xe(this.r * St, this.g * St, this.b * St, this.opacity);
            }, darker: function(St) {
              return St = St == null ? ae : Math.pow(ae, St), new xe(this.r * St, this.g * St, this.b * St, this.opacity);
            }, rgb: function() {
              return this;
            }, clamp: function() {
              return new xe(Me(this.r), Me(this.g), Me(this.b), ke(this.opacity));
            }, displayable: function() {
              return -0.5 <= this.r && this.r < 255.5 && -0.5 <= this.g && this.g < 255.5 && -0.5 <= this.b && this.b < 255.5 && 0 <= this.opacity && this.opacity <= 1;
            }, hex: Le, formatHex: Le, formatHex8: function() {
              return "#".concat(Re(this.r)).concat(Re(this.g)).concat(Re(this.b)).concat(Re(255 * (isNaN(this.opacity) ? 1 : this.opacity)));
            }, formatRgb: Qe, toString: Qe })), te($e, function(St, Lt, Rt, vt) {
              return arguments.length === 1 ? Pe(St) : new $e(St, Lt, Rt, vt ?? 1);
            }, ne(ee, { brighter: function(St) {
              return St = St == null ? oe : Math.pow(oe, St), new $e(this.h, this.s, this.l * St, this.opacity);
            }, darker: function(St) {
              return St = St == null ? ae : Math.pow(ae, St), new $e(this.h, this.s, this.l * St, this.opacity);
            }, rgb: function() {
              var St = this.h % 360 + 360 * (this.h < 0), Lt = isNaN(St) || isNaN(this.s) ? 0 : this.s, Rt = this.l, vt = Rt + (Rt < 0.5 ? Rt : 1 - Rt) * Lt, $t = 2 * Rt - vt;
              return new xe(Ve(St >= 240 ? St - 240 : St + 120, $t, vt), Ve(St, $t, vt), Ve(St < 120 ? St + 240 : St - 120, $t, vt), this.opacity);
            }, clamp: function() {
              return new $e(qe(this.h), Xe(this.s), Xe(this.l), ke(this.opacity));
            }, displayable: function() {
              return (0 <= this.s && this.s <= 1 || isNaN(this.s)) && 0 <= this.l && this.l <= 1 && 0 <= this.opacity && this.opacity <= 1;
            }, formatHsl: function() {
              var St = ke(this.opacity);
              return "".concat(St === 1 ? "hsl(" : "hsla(").concat(qe(this.h), ", ").concat(100 * Xe(this.s), "%, ").concat(100 * Xe(this.l), "%").concat(St === 1 ? ")" : ", ".concat(St, ")"));
            } }));
            var Ze = function(St) {
              return function() {
                return St;
              };
            };
            function Ge(St, Lt) {
              var Rt = Lt - St;
              return Rt ? /* @__PURE__ */ function(vt, $t) {
                return function(Pt) {
                  return vt + Pt * $t;
                };
              }(St, Rt) : Ze(isNaN(St) ? Lt : St);
            }
            var ze = function St(Lt) {
              var Rt = function($t) {
                return ($t = +$t) == 1 ? Ge : function(Pt, Xt) {
                  return Xt - Pt ? function(Nt, Gt, Ft) {
                    return Nt = Math.pow(Nt, Ft), Gt = Math.pow(Gt, Ft) - Nt, Ft = 1 / Ft, function(Tt) {
                      return Math.pow(Nt + Tt * Gt, Ft);
                    };
                  }(Pt, Xt, $t) : Ze(isNaN(Pt) ? Xt : Pt);
                };
              }(Lt);
              function vt($t, Pt) {
                var Xt = Rt(($t = Ee($t)).r, (Pt = Ee(Pt)).r), Nt = Rt($t.g, Pt.g), Gt = Rt($t.b, Pt.b), Ft = Ge($t.opacity, Pt.opacity);
                return function(Tt) {
                  return $t.r = Xt(Tt), $t.g = Nt(Tt), $t.b = Gt(Tt), $t.opacity = Ft(Tt), $t + "";
                };
              }
              return vt.gamma = St, vt;
            }(1);
            function We(St, Lt) {
              var Rt, vt = Lt ? Lt.length : 0, $t = St ? Math.min(vt, St.length) : 0, Pt = new Array($t), Xt = new Array(vt);
              for (Rt = 0; Rt < $t; ++Rt) Pt[Rt] = yt(St[Rt], Lt[Rt]);
              for (; Rt < vt; ++Rt) Xt[Rt] = Lt[Rt];
              return function(Nt) {
                for (Rt = 0; Rt < $t; ++Rt) Xt[Rt] = Pt[Rt](Nt);
                return Xt;
              };
            }
            function at(St, Lt) {
              var Rt = /* @__PURE__ */ new Date();
              return St = +St, Lt = +Lt, function(vt) {
                return Rt.setTime(St * (1 - vt) + Lt * vt), Rt;
              };
            }
            function ct(St, Lt) {
              return St = +St, Lt = +Lt, function(Rt) {
                return St * (1 - Rt) + Lt * Rt;
              };
            }
            function et(St) {
              return et = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(Lt) {
                return typeof Lt;
              } : function(Lt) {
                return Lt && typeof Symbol == "function" && Lt.constructor === Symbol && Lt !== Symbol.prototype ? "symbol" : typeof Lt;
              }, et(St);
            }
            function dt(St, Lt) {
              var Rt, vt = {}, $t = {};
              for (Rt in St !== null && et(St) === "object" || (St = {}), Lt !== null && et(Lt) === "object" || (Lt = {}), Lt) Rt in St ? vt[Rt] = yt(St[Rt], Lt[Rt]) : $t[Rt] = Lt[Rt];
              return function(Pt) {
                for (Rt in vt) $t[Rt] = vt[Rt](Pt);
                return $t;
              };
            }
            var Je = /[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g, it = new RegExp(Je.source, "g");
            function st(St, Lt) {
              var Rt, vt, $t, Pt = Je.lastIndex = it.lastIndex = 0, Xt = -1, Nt = [], Gt = [];
              for (St += "", Lt += ""; (Rt = Je.exec(St)) && (vt = it.exec(Lt)); ) ($t = vt.index) > Pt && ($t = Lt.slice(Pt, $t), Nt[Xt] ? Nt[Xt] += $t : Nt[++Xt] = $t), (Rt = Rt[0]) === (vt = vt[0]) ? Nt[Xt] ? Nt[Xt] += vt : Nt[++Xt] = vt : (Nt[++Xt] = null, Gt.push({ i: Xt, x: ct(Rt, vt) })), Pt = it.lastIndex;
              return Pt < Lt.length && ($t = Lt.slice(Pt), Nt[Xt] ? Nt[Xt] += $t : Nt[++Xt] = $t), Nt.length < 2 ? Gt[0] ? /* @__PURE__ */ function(Ft) {
                return function(Tt) {
                  return Ft(Tt) + "";
                };
              }(Gt[0].x) : /* @__PURE__ */ function(Ft) {
                return function() {
                  return Ft;
                };
              }(Lt) : (Lt = Gt.length, function(Ft) {
                for (var Tt, It = 0; It < Lt; ++It) Nt[(Tt = Gt[It]).i] = Tt.x(Ft);
                return Nt.join("");
              });
            }
            function gt(St, Lt) {
              Lt || (Lt = []);
              var Rt, vt = St ? Math.min(Lt.length, St.length) : 0, $t = Lt.slice();
              return function(Pt) {
                for (Rt = 0; Rt < vt; ++Rt) $t[Rt] = St[Rt] * (1 - Pt) + Lt[Rt] * Pt;
                return $t;
              };
            }
            function bt(St) {
              return bt = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(Lt) {
                return typeof Lt;
              } : function(Lt) {
                return Lt && typeof Symbol == "function" && Lt.constructor === Symbol && Lt !== Symbol.prototype ? "symbol" : typeof Lt;
              }, bt(St);
            }
            function yt(St, Lt) {
              var Rt, vt, $t = bt(Lt);
              return Lt == null || $t === "boolean" ? Ze(Lt) : ($t === "number" ? ct : $t === "string" ? (Rt = Ce(Lt)) ? (Lt = Rt, ze) : st : Lt instanceof Ce ? ze : Lt instanceof Date ? at : (vt = Lt, !ArrayBuffer.isView(vt) || vt instanceof DataView ? Array.isArray(Lt) ? We : typeof Lt.valueOf != "function" && typeof Lt.toString != "function" || isNaN(Lt) ? dt : ct : gt))(St, Lt);
            }
          }, 23648: function(G) {
            G.exports = JSON.parse('["xx-small","x-small","small","medium","large","x-large","xx-large","larger","smaller"]');
          }, 2362: function(G) {
            G.exports = JSON.parse('["normal","condensed","semi-condensed","extra-condensed","ultra-condensed","expanded","semi-expanded","extra-expanded","ultra-expanded"]');
          }, 87486: function(G) {
            G.exports = JSON.parse('["normal","italic","oblique"]');
          }, 99803: function(G) {
            G.exports = JSON.parse('["normal","bold","bolder","lighter","100","200","300","400","500","600","700","800","900"]');
          }, 54324: function(G) {
            G.exports = JSON.parse('["inherit","initial","unset"]');
          }, 94316: function(G) {
            G.exports = JSON.parse('["caption","icon","menu","message-box","small-caption","status-bar"]');
          }, 37071: function(G) {
            G.exports = JSON.parse(`{"version":8,"name":"orto","metadata":{"maputnik:renderer":"mlgljs"},"center":[1.537786,41.837539],"zoom":12,"bearing":0,"pitch":0,"light":{"anchor":"viewport","color":"white","intensity":0.4,"position":[1.15,45,30]},"sources":{"ortoEsri":{"type":"raster","tiles":["https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}"],"tileSize":256,"maxzoom":18,"attribution":"ESRI &copy; <a href='http://www.esri.com'>ESRI</a>"},"ortoInstaMaps":{"type":"raster","tiles":["https://tilemaps.icgc.cat/mapfactory/wmts/orto_8_12/CAT3857/{z}/{x}/{y}.png"],"tileSize":256,"maxzoom":13},"ortoICGC":{"type":"raster","tiles":["https://geoserveis.icgc.cat/icc_mapesmultibase/noutm/wmts/orto/GRID3857/{z}/{x}/{y}.jpeg"],"tileSize":256,"minzoom":13.1,"maxzoom":20},"openmaptiles":{"type":"vector","url":"https://geoserveis.icgc.cat/contextmaps/basemap.json"}},"sprite":"https://geoserveis.icgc.cat/contextmaps/sprites/sprite@1","glyphs":"https://geoserveis.icgc.cat/contextmaps/glyphs/{fontstack}/{range}.pbf","layers":[{"id":"background","type":"background","paint":{"background-color":"#F4F9F4"}},{"id":"ortoEsri","type":"raster","source":"ortoEsri","maxzoom":16,"layout":{"visibility":"visible"}},{"id":"ortoICGC","type":"raster","source":"ortoICGC","minzoom":13.1,"maxzoom":19,"layout":{"visibility":"visible"}},{"id":"ortoInstaMaps","type":"raster","source":"ortoInstaMaps","maxzoom":13,"layout":{"visibility":"visible"}},{"id":"waterway_tunnel","type":"line","source":"openmaptiles","source-layer":"waterway","minzoom":14,"filter":["all",["in","class","river","stream","canal"],["==","brunnel","tunnel"]],"layout":{"line-cap":"round"},"paint":{"line-color":"#a0c8f0","line-width":{"base":1.3,"stops":[[13,0.5],[20,6]]},"line-dasharray":[2,4]}},{"id":"waterway-other","type":"line","metadata":{"mapbox:group":"1444849382550.77"},"source":"openmaptiles","source-layer":"waterway","filter":["!in","class","canal","river","stream"],"layout":{"line-cap":"round"},"paint":{"line-color":"#a0c8f0","line-width":{"base":1.3,"stops":[[13,0.5],[20,2]]}}},{"id":"waterway-stream-canal","type":"line","metadata":{"mapbox:group":"1444849382550.77"},"source":"openmaptiles","source-layer":"waterway","filter":["all",["in","class","canal","stream"],["!=","brunnel","tunnel"]],"layout":{"line-cap":"round"},"paint":{"line-color":"#a0c8f0","line-width":{"base":1.3,"stops":[[13,0.5],[20,6]]}}},{"id":"waterway-river","type":"line","metadata":{"mapbox:group":"1444849382550.77"},"source":"openmaptiles","source-layer":"waterway","filter":["all",["==","class","river"],["!=","brunnel","tunnel"]],"layout":{"line-cap":"round"},"paint":{"line-color":"#a0c8f0","line-width":{"base":1.2,"stops":[[10,0.8],[20,4]]},"line-opacity":0.5}},{"id":"water-offset","type":"fill","metadata":{"mapbox:group":"1444849382550.77"},"source":"openmaptiles","source-layer":"water","maxzoom":8,"filter":["==","$type","Polygon"],"layout":{"visibility":"visible"},"paint":{"fill-opacity":0,"fill-color":"#a0c8f0","fill-translate":{"base":1,"stops":[[6,[2,0]],[8,[0,0]]]}}},{"id":"water","type":"fill","metadata":{"mapbox:group":"1444849382550.77"},"source":"openmaptiles","source-layer":"water","layout":{"visibility":"visible"},"paint":{"fill-color":"hsl(210, 67%, 85%)","fill-opacity":0}},{"id":"water-pattern","type":"fill","metadata":{"mapbox:group":"1444849382550.77"},"source":"openmaptiles","source-layer":"water","layout":{"visibility":"visible"},"paint":{"fill-translate":[0,2.5],"fill-pattern":"wave","fill-opacity":1}},{"id":"landcover-ice-shelf","type":"fill","metadata":{"mapbox:group":"1444849382550.77"},"source":"openmaptiles","source-layer":"landcover","filter":["==","subclass","ice_shelf"],"layout":{"visibility":"visible"},"paint":{"fill-color":"#fff","fill-opacity":{"base":1,"stops":[[0,0.9],[10,0.3]]}}},{"id":"tunnel-service-track-casing","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["in","class","service","track"]],"layout":{"line-join":"round"},"paint":{"line-color":"#cfcdca","line-dasharray":[0.5,0.25],"line-width":{"base":1.2,"stops":[[15,1],[16,4],[20,11]]}}},{"id":"tunnel-minor-casing","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["==","class","minor"]],"layout":{"line-join":"round"},"paint":{"line-color":"#cfcdca","line-opacity":{"stops":[[12,0],[12.5,1]]},"line-width":{"base":1.2,"stops":[[12,0.5],[13,1],[14,4],[20,15]]}}},{"id":"tunnel-secondary-tertiary-casing","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["in","class","secondary","tertiary"]],"layout":{"line-join":"round"},"paint":{"line-color":"#e9ac77","line-opacity":1,"line-width":{"base":1.2,"stops":[[8,1.5],[20,17]]}}},{"id":"tunnel-trunk-primary-casing","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["in","class","primary","trunk"]],"layout":{"line-join":"round"},"paint":{"line-color":"#e9ac77","line-width":{"base":1.2,"stops":[[5,0.4],[6,0.6],[7,1.5],[20,22]]},"line-opacity":0.7}},{"id":"tunnel-motorway-casing","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["==","class","motorway"]],"layout":{"line-join":"round","visibility":"visible"},"paint":{"line-color":"#e9ac77","line-dasharray":[0.5,0.25],"line-width":{"base":1.2,"stops":[[5,0.4],[6,0.6],[7,1.5],[20,22]]},"line-opacity":0.5}},{"id":"tunnel-path","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["==","brunnel","tunnel"],["==","class","path"]]],"paint":{"line-color":"#cba","line-dasharray":[1.5,0.75],"line-width":{"base":1.2,"stops":[[15,1.2],[20,4]]}}},{"id":"tunnel-service-track","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["in","class","service","track"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fff","line-width":{"base":1.2,"stops":[[15.5,0],[16,2],[20,7.5]]}}},{"id":"tunnel-minor","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["==","class","minor_road"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fff","line-opacity":1,"line-width":{"base":1.2,"stops":[[13.5,0],[14,2.5],[20,11.5]]}}},{"id":"tunnel-secondary-tertiary","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["in","class","secondary","tertiary"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fff4c6","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,10]]}}},{"id":"tunnel-trunk-primary","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["in","class","primary","trunk"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fff4c6","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,18]]},"line-opacity":0.5}},{"id":"tunnel-motorway","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["==","class","motorway"]],"layout":{"line-join":"round","visibility":"visible"},"paint":{"line-color":"#ffdaa6","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,18]]},"line-opacity":0.5}},{"id":"tunnel-railway","type":"line","metadata":{"mapbox:group":"1444849354174.1904"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","tunnel"],["==","class","rail"]],"paint":{"line-color":"#bbb","line-width":{"base":1.4,"stops":[[14,0.4],[15,0.75],[20,2]]},"line-dasharray":[2,2]}},{"id":"ferry","type":"line","source":"openmaptiles","source-layer":"transportation","filter":["all",["in","class","ferry"]],"layout":{"line-join":"round","visibility":"visible"},"paint":{"line-color":"rgba(108, 159, 182, 1)","line-width":1.1,"line-dasharray":[2,2]}},{"id":"aeroway-taxiway-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"aeroway","minzoom":12,"filter":["all",["in","class","taxiway"]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"rgba(153, 153, 153, 1)","line-width":{"base":1.5,"stops":[[11,2],[17,12]]},"line-opacity":1}},{"id":"aeroway-runway-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"aeroway","minzoom":12,"filter":["all",["in","class","runway"]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"rgba(153, 153, 153, 1)","line-width":{"base":1.5,"stops":[[11,5],[17,55]]},"line-opacity":1}},{"id":"aeroway-taxiway","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"aeroway","minzoom":4,"filter":["all",["in","class","taxiway"],["==","$type","LineString"]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"rgba(255, 255, 255, 1)","line-width":{"base":1.5,"stops":[[11,1],[17,10]]},"line-opacity":{"base":1,"stops":[[11,0],[12,1]]}}},{"id":"aeroway-runway","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"aeroway","minzoom":4,"filter":["all",["in","class","runway"],["==","$type","LineString"]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"rgba(255, 255, 255, 1)","line-width":{"base":1.5,"stops":[[11,4],[17,50]]},"line-opacity":{"base":1,"stops":[[11,0],[12,1]]}}},{"id":"highway-motorway-link-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":12,"filter":["all",["!in","brunnel","bridge","tunnel"],["==","class","motorway_link"]],"layout":{"line-cap":"round","line-join":"round"},"paint":{"line-color":"#e9ac77","line-opacity":1,"line-width":{"base":1.2,"stops":[[12,1],[13,3],[14,4],[20,15]]}}},{"id":"highway-link-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":13,"filter":["all",["!in","brunnel","bridge","tunnel"],["in","class","primary_link","secondary_link","tertiary_link","trunk_link"]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"#e9ac77","line-opacity":1,"line-width":{"base":1.2,"stops":[[12,1],[13,3],[14,4],[20,15]]}}},{"id":"highway-minor-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["!=","brunnel","tunnel"],["in","class","minor","service","track"]]],"layout":{"line-cap":"round","line-join":"round"},"paint":{"line-color":"#cfcdca","line-opacity":{"stops":[[12,0],[12.5,0]]},"line-width":{"base":1.2,"stops":[[12,0.5],[13,1],[14,4],[20,15]]}}},{"id":"highway-secondary-tertiary-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["!in","brunnel","bridge","tunnel"],["in","class","secondary","tertiary"]],"layout":{"line-cap":"butt","line-join":"round","visibility":"visible"},"paint":{"line-color":"#e9ac77","line-opacity":0.5,"line-width":{"base":1.2,"stops":[[8,1.5],[20,17]]}}},{"id":"highway-primary-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":5,"filter":["all",["!in","brunnel","bridge","tunnel"],["in","class","primary"]],"layout":{"line-cap":"butt","line-join":"round","visibility":"visible"},"paint":{"line-color":"#e9ac77","line-opacity":{"stops":[[7,0],[8,0.6]]},"line-width":{"base":1.2,"stops":[[7,0],[8,0.6],[9,1.5],[20,22]]}}},{"id":"highway-trunk-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":5,"filter":["all",["!in","brunnel","bridge","tunnel"],["in","class","trunk"]],"layout":{"line-cap":"butt","line-join":"round","visibility":"visible"},"paint":{"line-color":"#e9ac77","line-opacity":{"stops":[[5,0],[6,0.5]]},"line-width":{"base":1.2,"stops":[[5,0],[6,0.6],[7,1.5],[20,22]]}}},{"id":"highway-motorway-casing","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":4,"filter":["all",["!in","brunnel","bridge","tunnel"],["==","class","motorway"]],"layout":{"line-cap":"butt","line-join":"round","visibility":"visible"},"paint":{"line-color":"#e9ac77","line-width":{"base":1.2,"stops":[[4,0],[5,0.4],[6,0.6],[7,1.5],[20,22]]},"line-opacity":{"stops":[[4,0],[5,0.5]]}}},{"id":"highway-path","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["!in","brunnel","bridge","tunnel"],["==","class","path"]]],"paint":{"line-color":"#cba","line-dasharray":[1.5,0.75],"line-width":{"base":1.2,"stops":[[15,1.2],[20,4]]}}},{"id":"highway-motorway-link","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":12,"filter":["all",["!in","brunnel","bridge","tunnel"],["==","class","motorway_link"]],"layout":{"line-cap":"round","line-join":"round"},"paint":{"line-color":"#fc8","line-width":{"base":1.2,"stops":[[12.5,0],[13,1.5],[14,2.5],[20,11.5]]}}},{"id":"highway-link","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":13,"filter":["all",["!in","brunnel","bridge","tunnel"],["in","class","primary_link","secondary_link","tertiary_link","trunk_link"]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"#fea","line-width":{"base":1.2,"stops":[[12.5,0],[13,1.5],[14,2.5],[20,11.5]]}}},{"id":"highway-minor","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["!=","brunnel","tunnel"],["in","class","minor","service","track"]]],"layout":{"line-cap":"round","line-join":"round"},"paint":{"line-color":"#fff","line-opacity":0.5,"line-width":{"base":1.2,"stops":[[13.5,0],[14,2.5],[20,11.5]]}}},{"id":"highway-secondary-tertiary","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["!in","brunnel","bridge","tunnel"],["in","class","secondary","tertiary"]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"#fea","line-width":{"base":1.2,"stops":[[6.5,0],[8,0.5],[20,13]]},"line-opacity":0.5}},{"id":"highway-primary","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["!in","brunnel","bridge","tunnel"],["in","class","primary"]]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"#fea","line-width":{"base":1.2,"stops":[[8.5,0],[9,0.5],[20,18]]},"line-opacity":0}},{"id":"highway-trunk","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["!in","brunnel","bridge","tunnel"],["in","class","trunk"]]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"#fea","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,18]]},"line-opacity":0.5}},{"id":"highway-motorway","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","minzoom":5,"filter":["all",["==","$type","LineString"],["all",["!in","brunnel","bridge","tunnel"],["==","class","motorway"]]],"layout":{"line-cap":"round","line-join":"round","visibility":"visible"},"paint":{"line-color":"#fc8","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,18]]},"line-opacity":0.5}},{"id":"railway-transit","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["==","class","transit"],["!in","brunnel","tunnel"]]],"layout":{"visibility":"visible"},"paint":{"line-color":"hsla(0, 0%, 73%, 0.77)","line-width":{"base":1.4,"stops":[[14,0.4],[20,1]]}}},{"id":"railway-transit-hatching","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["==","class","transit"],["!in","brunnel","tunnel"]]],"layout":{"visibility":"visible"},"paint":{"line-color":"hsla(0, 0%, 73%, 0.68)","line-dasharray":[0.2,8],"line-width":{"base":1.4,"stops":[[14.5,0],[15,2],[20,6]]}}},{"id":"railway-service","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["==","class","rail"],["has","service"]]],"paint":{"line-color":"hsla(0, 0%, 73%, 0.77)","line-width":{"base":1.4,"stops":[[14,0.4],[20,1]]}}},{"id":"railway-service-hatching","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["==","class","rail"],["has","service"]]],"layout":{"visibility":"visible"},"paint":{"line-color":"hsla(0, 0%, 73%, 0.68)","line-dasharray":[0.2,8],"line-width":{"base":1.4,"stops":[[14.5,0],[15,2],[20,6]]}}},{"id":"railway","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["!has","service"],["!in","brunnel","bridge","tunnel"],["==","class","rail"]]],"paint":{"line-color":"#bbb","line-width":{"base":1.4,"stops":[[14,0.4],[15,0.75],[20,2]]}}},{"id":"railway-hatching","type":"line","metadata":{"mapbox:group":"1444849345966.4436"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["!has","service"],["!in","brunnel","bridge","tunnel"],["==","class","rail"]]],"paint":{"line-color":"#bbb","line-dasharray":[0.2,8],"line-width":{"base":1.4,"stops":[[14.5,0],[15,3],[20,8]]}}},{"id":"bridge-motorway-link-casing","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["==","class","motorway_link"]],"layout":{"line-join":"round"},"paint":{"line-color":"#e9ac77","line-opacity":1,"line-width":{"base":1.2,"stops":[[12,1],[13,3],[14,4],[20,15]]}}},{"id":"bridge-link-casing","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["in","class","primary_link","secondary_link","tertiary_link","trunk_link"]],"layout":{"line-join":"round"},"paint":{"line-color":"#e9ac77","line-opacity":1,"line-width":{"base":1.2,"stops":[[12,1],[13,3],[14,4],[20,15]]}}},{"id":"bridge-secondary-tertiary-casing","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["in","class","secondary","tertiary"]],"layout":{"line-join":"round"},"paint":{"line-color":"#e9ac77","line-opacity":1,"line-width":{"base":1.2,"stops":[[8,1.5],[20,28]]}}},{"id":"bridge-trunk-primary-casing","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["in","class","primary","trunk"]],"layout":{"line-join":"round"},"paint":{"line-color":"hsl(28, 76%, 67%)","line-width":{"base":1.2,"stops":[[5,0.4],[6,0.6],[7,1.5],[20,26]]}}},{"id":"bridge-motorway-casing","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["==","class","motorway"]],"layout":{"line-join":"round"},"paint":{"line-color":"#e9ac77","line-width":{"base":1.2,"stops":[[5,0.4],[6,0.6],[7,1.5],[20,22]]},"line-opacity":0.5}},{"id":"bridge-path-casing","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["==","brunnel","bridge"],["==","class","path"]]],"paint":{"line-color":"#f8f4f0","line-width":{"base":1.2,"stops":[[15,1.2],[20,18]]}}},{"id":"bridge-path","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","$type","LineString"],["all",["==","brunnel","bridge"],["==","class","path"]]],"paint":{"line-color":"#cba","line-width":{"base":1.2,"stops":[[15,1.2],[20,4]]},"line-dasharray":[1.5,0.75]}},{"id":"bridge-motorway-link","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["==","class","motorway_link"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fc8","line-width":{"base":1.2,"stops":[[12.5,0],[13,1.5],[14,2.5],[20,11.5]]}}},{"id":"bridge-link","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["in","class","primary_link","secondary_link","tertiary_link","trunk_link"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fea","line-width":{"base":1.2,"stops":[[12.5,0],[13,1.5],[14,2.5],[20,11.5]]}}},{"id":"bridge-secondary-tertiary","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["in","class","secondary","tertiary"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fea","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,20]]}}},{"id":"bridge-trunk-primary","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["in","class","primary","trunk"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fea","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,18]]}}},{"id":"bridge-motorway","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["==","class","motorway"]],"layout":{"line-join":"round"},"paint":{"line-color":"#fc8","line-width":{"base":1.2,"stops":[[6.5,0],[7,0.5],[20,18]]},"line-opacity":0.5}},{"id":"bridge-railway","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["==","class","rail"]],"paint":{"line-color":"#bbb","line-width":{"base":1.4,"stops":[[14,0.4],[15,0.75],[20,2]]}}},{"id":"bridge-railway-hatching","type":"line","metadata":{"mapbox:group":"1444849334699.1902"},"source":"openmaptiles","source-layer":"transportation","filter":["all",["==","brunnel","bridge"],["==","class","rail"]],"paint":{"line-color":"#bbb","line-dasharray":[0.2,8],"line-width":{"base":1.4,"stops":[[14.5,0],[15,3],[20,8]]}}},{"id":"cablecar","type":"line","source":"openmaptiles","source-layer":"transportation","minzoom":13,"filter":["==","class","cable_car"],"layout":{"visibility":"visible","line-cap":"round"},"paint":{"line-color":"hsl(0, 0%, 70%)","line-width":{"base":1,"stops":[[11,1],[19,2.5]]}}},{"id":"cablecar-dash","type":"line","source":"openmaptiles","source-layer":"transportation","minzoom":13,"filter":["==","class","cable_car"],"layout":{"visibility":"visible","line-cap":"round"},"paint":{"line-color":"hsl(0, 0%, 70%)","line-width":{"base":1,"stops":[[11,3],[19,5.5]]},"line-dasharray":[2,3]}},{"id":"boundary-land-level-4","type":"line","source":"openmaptiles","source-layer":"boundary","filter":["all",[">=","admin_level",4],["<=","admin_level",8],["!=","maritime",1]],"layout":{"line-join":"round"},"paint":{"line-color":"#9e9cab","line-dasharray":[3,1,1,1],"line-width":{"base":1.4,"stops":[[4,0.4],[5,1],[12,3]]},"line-opacity":0.6}},{"id":"boundary-land-level-2","type":"line","source":"openmaptiles","source-layer":"boundary","filter":["all",["==","admin_level",2],["!=","maritime",1],["!=","disputed",1]],"layout":{"line-cap":"round","line-join":"round"},"paint":{"line-color":"hsl(248, 7%, 66%)","line-width":{"base":1,"stops":[[0,0.6],[4,1.4],[5,2],[12,2]]}}},{"id":"boundary-land-disputed","type":"line","source":"openmaptiles","source-layer":"boundary","filter":["all",["!=","maritime",1],["==","disputed",1]],"layout":{"line-cap":"round","line-join":"round"},"paint":{"line-color":"hsl(248, 7%, 70%)","line-dasharray":[1,3],"line-width":{"base":1,"stops":[[0,0.6],[4,1.4],[5,2],[12,8]]}}},{"id":"boundary-water","type":"line","source":"openmaptiles","source-layer":"boundary","filter":["all",["in","admin_level",2,4],["==","maritime",1]],"layout":{"line-cap":"round","line-join":"round"},"paint":{"line-color":"rgba(154, 189, 214, 1)","line-width":{"base":1,"stops":[[0,0.6],[4,1],[5,1],[12,1]]},"line-opacity":{"stops":[[6,0],[10,0]]}}},{"id":"waterway-name","type":"symbol","source":"openmaptiles","source-layer":"waterway","minzoom":13,"filter":["all",["==","$type","LineString"],["has","name"]],"layout":{"text-font":["Noto Sans Italic"],"text-size":14,"text-field":"{name:latin} {name:nonlatin}","text-max-width":5,"text-rotation-alignment":"map","symbol-placement":"line","text-letter-spacing":0.2,"symbol-spacing":350},"paint":{"text-color":"#74aee9","text-halo-width":1.5,"text-halo-color":"rgba(255,255,255,0.7)"}},{"id":"water-name-lakeline","type":"symbol","source":"openmaptiles","source-layer":"water_name","filter":["==","$type","LineString"],"layout":{"text-font":["Noto Sans Italic"],"text-size":14,"text-field":"{name:latin}\\n{name:nonlatin}","text-max-width":5,"text-rotation-alignment":"map","symbol-placement":"line","symbol-spacing":350,"text-letter-spacing":0.2},"paint":{"text-color":"#74aee9","text-halo-width":1.5,"text-halo-color":"rgba(255,255,255,0.7)"}},{"id":"water-name-ocean","type":"symbol","source":"openmaptiles","source-layer":"water_name","filter":["all",["==","$type","Point"],["==","class","ocean"]],"layout":{"text-font":["Noto Sans Italic"],"text-size":14,"text-field":"{name:latin}","text-max-width":5,"text-rotation-alignment":"map","symbol-placement":"point","symbol-spacing":350,"text-letter-spacing":0.2},"paint":{"text-color":"#74aee9","text-halo-width":1.5,"text-halo-color":"rgba(255,255,255,0.7)"}},{"id":"water-name-other","type":"symbol","source":"openmaptiles","source-layer":"water_name","filter":["all",["==","$type","Point"],["!in","class","ocean"]],"layout":{"text-font":["Noto Sans Italic"],"text-size":{"stops":[[0,10],[6,14]]},"text-field":"{name:latin}\\n{name:nonlatin}","text-max-width":5,"text-rotation-alignment":"map","symbol-placement":"point","symbol-spacing":350,"text-letter-spacing":0.2,"visibility":"visible"},"paint":{"text-color":"#74aee9","text-halo-width":1.5,"text-halo-color":"rgba(255,255,255,0.7)"}},{"id":"poi-level-3","type":"symbol","source":"openmaptiles","source-layer":"poi","minzoom":16,"filter":["all",["==","$type","Point"],[">=","rank",25]],"layout":{"text-padding":2,"text-font":["Noto Sans Regular"],"text-anchor":"top","icon-image":"{class}_11","text-field":"{name:latin}\\n{name:nonlatin}","text-offset":[0,0.6],"text-size":12,"text-max-width":9},"paint":{"text-halo-blur":0.5,"text-color":"#666","text-halo-width":1,"text-halo-color":"#ffffff"}},{"id":"poi-level-2","type":"symbol","source":"openmaptiles","source-layer":"poi","minzoom":15,"filter":["all",["==","$type","Point"],["<=","rank",24],[">=","rank",15]],"layout":{"text-padding":2,"text-font":["Noto Sans Regular"],"text-anchor":"top","icon-image":"{class}_11","text-field":"{name:latin}\\n{name:nonlatin}","text-offset":[0,0.6],"text-size":12,"text-max-width":9},"paint":{"text-halo-blur":0.5,"text-color":"#666","text-halo-width":1,"text-halo-color":"#ffffff"}},{"id":"poi-level-1","type":"symbol","source":"openmaptiles","source-layer":"poi","minzoom":14,"filter":["all",["==","$type","Point"],["<=","rank",14],["has","name"]],"layout":{"text-padding":2,"text-font":["Noto Sans Regular"],"text-anchor":"top","icon-image":"{class}_11","text-field":"{name:latin}\\n{name:nonlatin}","text-offset":[0,0.6],"text-size":11,"text-max-width":9},"paint":{"text-halo-blur":0.5,"text-color":"rgba(191, 228, 172, 1)","text-halo-width":1,"text-halo-color":"rgba(30, 29, 29, 1)"}},{"id":"poi-railway","type":"symbol","source":"openmaptiles","source-layer":"poi","minzoom":13,"filter":["all",["==","$type","Point"],["has","name"],["==","class","railway"],["==","subclass","station"]],"layout":{"text-padding":2,"text-font":["Noto Sans Regular"],"text-anchor":"top","icon-image":"{class}_11","text-field":"{name:latin}\\n{name:nonlatin}","text-offset":[0,0.6],"text-size":12,"text-max-width":9,"icon-optional":false,"icon-ignore-placement":false,"icon-allow-overlap":false,"text-ignore-placement":false,"text-allow-overlap":false,"text-optional":true},"paint":{"text-halo-blur":0.5,"text-color":"#666","text-halo-width":1,"text-halo-color":"#ffffff"}},{"id":"road_oneway","type":"symbol","source":"openmaptiles","source-layer":"transportation","minzoom":15,"filter":["all",["==","oneway",1],["in","class","motorway","trunk","primary","secondary","tertiary","minor","service"]],"layout":{"symbol-placement":"line","icon-image":"oneway","symbol-spacing":75,"icon-padding":2,"icon-rotation-alignment":"map","icon-rotate":90,"icon-size":{"stops":[[15,0.5],[19,1]]}},"paint":{"icon-opacity":0.5}},{"id":"road_oneway_opposite","type":"symbol","source":"openmaptiles","source-layer":"transportation","minzoom":15,"filter":["all",["==","oneway",-1],["in","class","motorway","trunk","primary","secondary","tertiary","minor","service"]],"layout":{"symbol-placement":"line","icon-image":"oneway","symbol-spacing":75,"icon-padding":2,"icon-rotation-alignment":"map","icon-rotate":-90,"icon-size":{"stops":[[15,0.5],[19,1]]}},"paint":{"icon-opacity":0.5}},{"id":"highway-name-path","type":"symbol","source":"openmaptiles","source-layer":"transportation_name","minzoom":15.5,"filter":["==","class","path"],"layout":{"text-size":{"base":1,"stops":[[13,12],[14,13]]},"text-font":["Noto Sans Regular"],"text-field":"{name:latin} {name:nonlatin}","symbol-placement":"line","text-rotation-alignment":"map"},"paint":{"text-halo-color":"#f8f4f0","text-color":"hsl(30, 23%, 62%)","text-halo-width":0.5}},{"id":"highway-name-minor","type":"symbol","source":"openmaptiles","source-layer":"transportation_name","minzoom":15,"filter":["all",["==","$type","LineString"],["in","class","minor","service","track"]],"layout":{"text-size":{"base":1,"stops":[[13,12],[14,13]]},"text-font":["Noto Sans Regular"],"text-field":"{name:latin} {name:nonlatin}","symbol-placement":"line","text-rotation-alignment":"map"},"paint":{"text-halo-blur":0.5,"text-color":"#765","text-halo-width":1}},{"id":"highway-name-major","type":"symbol","source":"openmaptiles","source-layer":"transportation_name","minzoom":12.2,"filter":["in","class","primary","secondary","tertiary","trunk"],"layout":{"text-size":{"base":1,"stops":[[13,12],[14,13]]},"text-font":["Noto Sans Regular"],"text-field":"{name:latin} {name:nonlatin}","symbol-placement":"line","text-rotation-alignment":"map"},"paint":{"text-halo-blur":0.5,"text-color":"#765","text-halo-width":1}},{"id":"highway-shield","type":"symbol","source":"openmaptiles","source-layer":"transportation_name","minzoom":8,"filter":["all",["<=","ref_length",6],["==","$type","LineString"],["!in","network","us-interstate","us-highway","us-state"]],"layout":{"text-size":10,"icon-image":"road_{ref_length}","icon-rotation-alignment":"viewport","symbol-spacing":200,"text-font":["Noto Sans Regular"],"symbol-placement":{"base":1,"stops":[[10,"point"],[11,"line"]]},"text-rotation-alignment":"viewport","icon-size":1,"text-field":"{ref}"},"paint":{"text-opacity":1,"text-color":"rgba(20, 19, 19, 1)","text-halo-color":"rgba(230, 221, 221, 0)","text-halo-width":2,"icon-color":"rgba(183, 18, 18, 1)","icon-opacity":0.3,"icon-halo-color":"rgba(183, 55, 55, 0)"}},{"id":"highway-shield-us-interstate","type":"symbol","source":"openmaptiles","source-layer":"transportation_name","minzoom":7,"filter":["all",["<=","ref_length",6],["==","$type","LineString"],["in","network","us-interstate"]],"layout":{"text-size":10,"icon-image":"{network}_{ref_length}","icon-rotation-alignment":"viewport","symbol-spacing":200,"text-font":["Noto Sans Regular"],"symbol-placement":{"base":1,"stops":[[7,"point"],[7,"line"],[8,"line"]]},"text-rotation-alignment":"viewport","icon-size":1,"text-field":"{ref}"},"paint":{"text-color":"rgba(0, 0, 0, 1)"}},{"id":"highway-shield-us-other","type":"symbol","source":"openmaptiles","source-layer":"transportation_name","minzoom":9,"filter":["all",["<=","ref_length",6],["==","$type","LineString"],["in","network","us-highway","us-state"]],"layout":{"text-size":10,"icon-image":"{network}_{ref_length}","icon-rotation-alignment":"viewport","symbol-spacing":200,"text-font":["Noto Sans Regular"],"symbol-placement":{"base":1,"stops":[[10,"point"],[11,"line"]]},"text-rotation-alignment":"viewport","icon-size":1,"text-field":"{ref}"},"paint":{"text-color":"rgba(0, 0, 0, 1)"}},{"id":"place-other","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","minzoom":12,"filter":["!in","class","city","town","village","country","continent"],"layout":{"text-letter-spacing":0.1,"text-size":{"base":1.2,"stops":[[12,10],[15,14]]},"text-font":["Noto Sans Bold"],"text-field":"{name:latin}\\n{name:nonlatin}","text-transform":"uppercase","text-max-width":9,"visibility":"visible"},"paint":{"text-color":"rgba(255,255,255,1)","text-halo-width":1.2,"text-halo-color":"rgba(57, 28, 28, 1)"}},{"id":"place-village","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","minzoom":10,"filter":["==","class","village"],"layout":{"text-font":["Noto Sans Regular"],"text-size":{"base":1.2,"stops":[[10,12],[15,16]]},"text-field":"{name:latin}\\n{name:nonlatin}","text-max-width":8,"visibility":"visible"},"paint":{"text-color":"rgba(255, 255, 255, 1)","text-halo-width":1.2,"text-halo-color":"rgba(10, 9, 9, 0.8)"}},{"id":"place-town","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","filter":["==","class","town"],"layout":{"text-font":["Noto Sans Regular"],"text-size":{"base":1.2,"stops":[[10,14],[15,24]]},"text-field":"{name:latin}\\n{name:nonlatin}","text-max-width":8,"visibility":"visible"},"paint":{"text-color":"rgba(255, 255, 255, 1)","text-halo-width":1.2,"text-halo-color":"rgba(22, 22, 22, 0.8)"}},{"id":"place-city","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","filter":["all",["!=","capital",2],["==","class","city"]],"layout":{"text-font":["Noto Sans Regular"],"text-size":{"base":1.2,"stops":[[7,14],[11,24]]},"text-field":"{name:latin}\\n{name:nonlatin}","text-max-width":8,"visibility":"visible"},"paint":{"text-color":"rgba(0, 0, 0, 1)","text-halo-width":1.2,"text-halo-color":"rgba(255,255,255,0.8)"}},{"id":"place-city-capital","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","filter":["all",["==","capital",2],["==","class","city"]],"layout":{"text-font":["Noto Sans Regular"],"text-size":{"base":1.2,"stops":[[7,14],[11,24]]},"text-field":"{name:latin}\\n{name:nonlatin}","text-max-width":8,"icon-image":"star_11","text-offset":[0.4,0],"icon-size":0.8,"text-anchor":"left","visibility":"visible"},"paint":{"text-color":"#333","text-halo-width":1.2,"text-halo-color":"rgba(255,255,255,0.8)"}},{"id":"place-country-other","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","filter":["all",["==","class","country"],[">=","rank",3],["!has","iso_a2"]],"layout":{"text-font":["Noto Sans Italic"],"text-field":"{name:latin}","text-size":{"stops":[[3,11],[7,17]]},"text-transform":"uppercase","text-max-width":6.25,"visibility":"visible"},"paint":{"text-halo-blur":1,"text-color":"#334","text-halo-width":2,"text-halo-color":"rgba(255,255,255,0.8)"}},{"id":"place-country-3","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","filter":["all",["==","class","country"],[">=","rank",3],["has","iso_a2"]],"layout":{"text-font":["Noto Sans Bold"],"text-field":"{name:latin}","text-size":{"stops":[[3,11],[7,17]]},"text-transform":"uppercase","text-max-width":6.25,"visibility":"visible"},"paint":{"text-halo-blur":1,"text-color":"#334","text-halo-width":2,"text-halo-color":"rgba(255,255,255,0.8)"}},{"id":"place-country-2","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","filter":["all",["==","class","country"],["==","rank",2],["has","iso_a2"]],"layout":{"text-font":["Noto Sans Bold"],"text-field":"{name:latin}","text-size":{"stops":[[2,11],[5,17]]},"text-transform":"uppercase","text-max-width":6.25,"visibility":"visible"},"paint":{"text-halo-blur":1,"text-color":"#334","text-halo-width":2,"text-halo-color":"rgba(255,255,255,0.8)"}},{"id":"place-country-1","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","filter":["all",["==","class","country"],["==","rank",1],["has","iso_a2"]],"layout":{"text-font":["Noto Sans Bold"],"text-field":"{name:latin}","text-size":{"stops":[[1,11],[4,17]]},"text-transform":"uppercase","text-max-width":6.25,"visibility":"visible"},"paint":{"text-halo-blur":1,"text-color":"#334","text-halo-width":2,"text-halo-color":"rgba(255,255,255,0.8)"}},{"id":"place-continent","type":"symbol","metadata":{"mapbox:group":"1444849242106.713"},"source":"openmaptiles","source-layer":"place","maxzoom":1,"filter":["==","class","continent"],"layout":{"text-font":["Noto Sans Bold"],"text-field":"{name:latin}","text-size":14,"text-max-width":6.25,"text-transform":"uppercase","visibility":"visible"},"paint":{"text-halo-blur":1,"text-color":"#334","text-halo-width":2,"text-halo-color":"rgba(255,255,255,0.8)"}}],"id":"qebnlkra6"}`);
          }, 51962: function(G) {
            G.exports = JSON.parse(`{"version":8,"name":"orto","metadata":{},"center":[1.537786,41.837539],"zoom":12,"bearing":0,"pitch":0,"light":{"anchor":"viewport","color":"white","intensity":0.4,"position":[1.15,45,30]},"sources":{"ortoEsri":{"type":"raster","tiles":["https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}"],"tileSize":256,"maxzoom":18,"attribution":"ESRI &copy; <a href='http://www.esri.com'>ESRI</a>"},"ortoInstaMaps":{"type":"raster","tiles":["https://tilemaps.icgc.cat/mapfactory/wmts/orto_8_12/CAT3857/{z}/{x}/{y}.png"],"tileSize":256,"maxzoom":13},"ortoICGC":{"type":"raster","tiles":["https://geoserveis.icgc.cat/icc_mapesmultibase/noutm/wmts/orto/GRID3857/{z}/{x}/{y}.jpeg"],"tileSize":256,"minzoom":13.1,"maxzoom":20},"openmaptiles":{"type":"vector","url":"https://geoserveis.icgc.cat/contextmaps/basemap.json"}},"sprite":"https://geoserveis.icgc.cat/contextmaps/sprites/sprite@1","glyphs":"https://geoserveis.icgc.cat/contextmaps/glyphs/{fontstack}/{range}.pbf","layers":[{"id":"background","type":"background","paint":{"background-color":"#F4F9F4"}},{"id":"ortoEsri","type":"raster","source":"ortoEsri","maxzoom":16,"layout":{"visibility":"visible"}},{"id":"ortoICGC","type":"raster","source":"ortoICGC","minzoom":13.1,"maxzoom":19,"layout":{"visibility":"visible"}},{"id":"ortoInstaMaps","type":"raster","source":"ortoInstaMaps","maxzoom":13,"layout":{"visibility":"visible"}}]}`);
          } }, V = {};
          function q(G) {
            var J = V[G];
            if (J !== void 0) return J.exports;
            var Z = V[G] = { id: G, exports: {} };
            return H[G].call(Z.exports, Z, Z.exports, q), Z.exports;
          }
          return q.m = H, q.n = function(G) {
            var J = G && G.__esModule ? function() {
              return G.default;
            } : function() {
              return G;
            };
            return q.d(J, { a: J }), J;
          }, q.d = function(G, J) {
            for (var Z in J) q.o(J, Z) && !q.o(G, Z) && Object.defineProperty(G, Z, { enumerable: !0, get: J[Z] });
          }, q.g = function() {
            if (typeof globalThis == "object") return globalThis;
            try {
              return this || new Function("return this")();
            } catch {
              if (typeof window == "object") return window;
            }
          }(), q.o = function(G, J) {
            return Object.prototype.hasOwnProperty.call(G, J);
          }, q.r = function(G) {
            typeof Symbol < "u" && Symbol.toStringTag && Object.defineProperty(G, Symbol.toStringTag, { value: "Module" }), Object.defineProperty(G, "__esModule", { value: !0 });
          }, q.b = document.baseURI || self.location.href, q.nc = void 0, q(20260);
        }();
      });
    }(plotly_min)), plotly_min.exports;
  }
  var plotly_minExports = requirePlotly_min();
  const Plotly$1 = /* @__PURE__ */ getDefaultExportFromCjs$1(plotly_minExports), PLOTLY_CONFIG = {
    displaylogo: !1,
    responsive: !0
  }, PLOTLY_LAYOUT_DEFAULTS = {
    modebar: {
      orientation: "v"
    }
  }, _sfc_main$6 = /* @__PURE__ */ defineComponent({
    __name: "Plotly",
    props: {
      plotJson: {},
      responsive: { type: Boolean },
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = `plotly_${v4()}`, V = (G) => {
        const { layout: J } = G;
        J && (delete J.autosize, delete J.width, _.singleBlockEmbed && delete J.height);
      }, q = async () => {
        _.responsive && V(_.plotJson), Plotly$1.newPlot(H, {
          data: _.plotJson.data,
          layout: {
            ...PLOTLY_LAYOUT_DEFAULTS,
            ..._.plotJson.layout,
            autosize: _.responsive
          },
          config: PLOTLY_CONFIG,
          frames: _.plotJson.frames || void 0
        });
      };
      return onMounted(() => {
        q();
      }), (G, J) => (openBlock(), createElementBlock("div", {
        "data-cy": "block-plotly",
        class: normalizeClass([
          "bg-white flex justify-center",
          { "w-full": G.responsive, "h-iframe": G.singleBlockEmbed }
        ])
      }, [
        createBaseVNode("div", {
          id: H,
          class: "w-full"
        })
      ], 2));
    }
  }), Plotly = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    default: _sfc_main$6
  }, Symbol.toStringTag, { value: "Module" })), stringOrChar = /("(?:[^\\"]|\\.)*")|[:,]/g;
  function stringify$2(O, _ = {}) {
    const H = JSON.stringify(
      [1],
      void 0,
      _.indent === void 0 ? 2 : _.indent
    ).slice(2, -3), V = H === "" ? 1 / 0 : _.maxLength === void 0 ? 80 : _.maxLength;
    let { replacer: q } = _;
    return function G(J, Z, te) {
      J && typeof J.toJSON == "function" && (J = J.toJSON());
      const ne = JSON.stringify(J, q);
      if (ne === void 0)
        return ne;
      const ee = V - Z.length - te;
      if (ne.length <= ee) {
        const ae = ne.replace(
          stringOrChar,
          (oe, se) => se || `${oe} `
        );
        if (ae.length <= ee)
          return ae;
      }
      if (q != null && (J = JSON.parse(ne), q = void 0), typeof J == "object" && J !== null) {
        const ae = Z + H, oe = [];
        let se = 0, ce, le;
        if (Array.isArray(J)) {
          ce = "[", le = "]";
          const { length: ue } = J;
          for (; se < ue; se++)
            oe.push(
              G(J[se], ae, se === ue - 1 ? 0 : 1) || "null"
            );
        } else {
          ce = "{", le = "}";
          const ue = Object.keys(J), { length: de } = ue;
          for (; se < de; se++) {
            const fe = ue[se], he = `${JSON.stringify(fe)}: `, me = G(
              J[fe],
              ae,
              he.length + (se === de - 1 ? 0 : 1)
            );
            me !== void 0 && oe.push(he + me);
          }
        }
        if (oe.length > 0)
          return [ce, H + oe.join(`,
${ae}`), le].join(
            `
${Z}`
          );
      }
      return ne;
    }(O, "", 0);
  }
  function accessor(O, _, H) {
    return O.fields = _ || [], O.fname = H, O;
  }
  function accessorName(O) {
    return O == null ? null : O.fname;
  }
  function accessorFields(O) {
    return O == null ? null : O.fields;
  }
  function getter$1(O) {
    return O.length === 1 ? get1(O[0]) : getN(O);
  }
  const get1 = (O) => function(_) {
    return _[O];
  }, getN = (O) => {
    const _ = O.length;
    return function(H) {
      for (let V = 0; V < _; ++V)
        H = H[O[V]];
      return H;
    };
  };
  function error$1(O) {
    throw Error(O);
  }
  function splitAccessPath(O) {
    const _ = [], H = O.length;
    let V = null, q = 0, G = "", J, Z, te;
    O = O + "";
    function ne() {
      _.push(G + O.substring(J, Z)), G = "", J = Z + 1;
    }
    for (J = Z = 0; Z < H; ++Z)
      if (te = O[Z], te === "\\")
        G += O.substring(J, Z++), J = Z;
      else if (te === V)
        ne(), V = null, q = -1;
      else {
        if (V)
          continue;
        J === q && te === '"' || J === q && te === "'" ? (J = Z + 1, V = te) : te === "." && !q ? Z > J ? ne() : J = Z + 1 : te === "[" ? (Z > J && ne(), q = J = Z + 1) : te === "]" && (q || error$1("Access path missing open bracket: " + O), q > 0 && ne(), q = 0, J = Z + 1);
      }
    return q && error$1("Access path missing closing bracket: " + O), V && error$1("Access path missing closing quote: " + O), Z > J && (Z++, ne()), _;
  }
  function field$1(O, _, H) {
    const V = splitAccessPath(O);
    return O = V.length === 1 ? V[0] : O, accessor((H && H.get || getter$1)(V), [O], _ || O);
  }
  const id = field$1("id"), identity$7 = accessor((O) => O, [], "identity"), zero$4 = accessor(() => 0, [], "zero"), one$2 = accessor(() => 1, [], "one"), truthy = accessor(() => !0, [], "true"), falsy = accessor(() => !1, [], "false");
  function log$1$1(O, _, H) {
    const V = [_].concat([].slice.call(H));
    console[O].apply(console, V);
  }
  const None$2 = 0, Error$1 = 1, Warn = 2, Info = 3, Debug = 4;
  function logger(O, _) {
    let H = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : log$1$1, V = O || None$2;
    return {
      level(q) {
        return arguments.length ? (V = +q, this) : V;
      },
      error() {
        return V >= Error$1 && H(_ || "error", "ERROR", arguments), this;
      },
      warn() {
        return V >= Warn && H(_ || "warn", "WARN", arguments), this;
      },
      info() {
        return V >= Info && H(_ || "log", "INFO", arguments), this;
      },
      debug() {
        return V >= Debug && H(_ || "log", "DEBUG", arguments), this;
      }
    };
  }
  var isArray$2 = Array.isArray;
  function isObject$2(O) {
    return O === Object(O);
  }
  const isLegalKey = (O) => O !== "__proto__";
  function mergeConfig() {
    for (var O = arguments.length, _ = new Array(O), H = 0; H < O; H++)
      _[H] = arguments[H];
    return _.reduce((V, q) => {
      for (const G in q)
        if (G === "signals")
          V.signals = mergeNamed(V.signals, q.signals);
        else {
          const J = G === "legend" ? {
            layout: 1
          } : G === "style" ? !0 : null;
          writeConfig(V, G, q[G], J);
        }
      return V;
    }, {});
  }
  function writeConfig(O, _, H, V) {
    if (!isLegalKey(_)) return;
    let q, G;
    if (isObject$2(H) && !isArray$2(H)) {
      G = isObject$2(O[_]) ? O[_] : O[_] = {};
      for (q in H)
        V && (V === !0 || V[q]) ? writeConfig(G, q, H[q]) : isLegalKey(q) && (G[q] = H[q]);
    } else
      O[_] = H;
  }
  function mergeNamed(O, _) {
    if (O == null) return _;
    const H = {}, V = [];
    function q(G) {
      H[G.name] || (H[G.name] = 1, V.push(G));
    }
    return _.forEach(q), O.forEach(q), V;
  }
  function peek$1(O) {
    return O[O.length - 1];
  }
  function toNumber$1(O) {
    return O == null || O === "" ? null : +O;
  }
  const exp$2 = (O) => (_) => O * Math.exp(_), log$4 = (O) => (_) => Math.log(O * _), symlog$1 = (O) => (_) => Math.sign(_) * Math.log1p(Math.abs(_ / O)), symexp = (O) => (_) => Math.sign(_) * Math.expm1(Math.abs(_)) * O, pow$4 = (O) => (_) => _ < 0 ? -Math.pow(-_, O) : Math.pow(_, O);
  function pan(O, _, H, V) {
    const q = H(O[0]), G = H(peek$1(O)), J = (G - q) * _;
    return [V(q - J), V(G - J)];
  }
  function panLinear(O, _) {
    return pan(O, _, toNumber$1, identity$7);
  }
  function panLog(O, _) {
    var H = Math.sign(O[0]);
    return pan(O, _, log$4(H), exp$2(H));
  }
  function panPow(O, _, H) {
    return pan(O, _, pow$4(H), pow$4(1 / H));
  }
  function panSymlog(O, _, H) {
    return pan(O, _, symlog$1(H), symexp(H));
  }
  function zoom$2(O, _, H, V, q) {
    const G = V(O[0]), J = V(peek$1(O)), Z = _ != null ? V(_) : (G + J) / 2;
    return [q(Z + (G - Z) * H), q(Z + (J - Z) * H)];
  }
  function zoomLinear(O, _, H) {
    return zoom$2(O, _, H, toNumber$1, identity$7);
  }
  function zoomLog(O, _, H) {
    const V = Math.sign(O[0]);
    return zoom$2(O, _, H, log$4(V), exp$2(V));
  }
  function zoomPow(O, _, H, V) {
    return zoom$2(O, _, H, pow$4(V), pow$4(1 / V));
  }
  function zoomSymlog(O, _, H, V) {
    return zoom$2(O, _, H, symlog$1(V), symexp(V));
  }
  function quarter(O) {
    return 1 + ~~(new Date(O).getMonth() / 3);
  }
  function utcquarter(O) {
    return 1 + ~~(new Date(O).getUTCMonth() / 3);
  }
  function array$5(O) {
    return O != null ? isArray$2(O) ? O : [O] : [];
  }
  function clampRange$1(O, _, H) {
    let V = O[0], q = O[1], G;
    return q < V && (G = q, q = V, V = G), G = q - V, G >= H - _ ? [_, H] : [V = Math.min(Math.max(V, _), H - G), V + G];
  }
  function isFunction$2(O) {
    return typeof O == "function";
  }
  const DESCENDING = "descending";
  function compare$3(O, _, H) {
    H = H || {}, _ = array$5(_) || [];
    const V = [], q = [], G = {}, J = H.comparator || comparator$1;
    return array$5(O).forEach((Z, te) => {
      Z != null && (V.push(_[te] === DESCENDING ? -1 : 1), q.push(Z = isFunction$2(Z) ? Z : field$1(Z, null, H)), (accessorFields(Z) || []).forEach((ne) => G[ne] = 1));
    }), q.length === 0 ? null : accessor(J(q, V), Object.keys(G));
  }
  const ascending$2 = (O, _) => (O < _ || O == null) && _ != null ? -1 : (O > _ || _ == null) && O != null ? 1 : (_ = _ instanceof Date ? +_ : _, (O = O instanceof Date ? +O : O) !== O && _ === _ ? -1 : _ !== _ && O === O ? 1 : 0), comparator$1 = (O, _) => O.length === 1 ? compare1(O[0], _[0]) : compareN(O, _, O.length), compare1 = (O, _) => function(H, V) {
    return ascending$2(O(H), O(V)) * _;
  }, compareN = (O, _, H) => (_.push(0), function(V, q) {
    let G, J = 0, Z = -1;
    for (; J === 0 && ++Z < H; )
      G = O[Z], J = ascending$2(G(V), G(q));
    return J * _[Z];
  });
  function constant$5(O) {
    return isFunction$2(O) ? O : () => O;
  }
  function debounce$2(O, _) {
    let H;
    return (V) => {
      H && clearTimeout(H), H = setTimeout(() => (_(V), H = null), O);
    };
  }
  function extend$2(O) {
    for (let _, H, V = 1, q = arguments.length; V < q; ++V) {
      _ = arguments[V];
      for (H in _)
        O[H] = _[H];
    }
    return O;
  }
  function extent(O, _) {
    let H = 0, V, q, G, J;
    if (O && (V = O.length))
      if (_ == null) {
        for (q = O[H]; H < V && (q == null || q !== q); q = O[++H]) ;
        for (G = J = q; H < V; ++H)
          q = O[H], q != null && (q < G && (G = q), q > J && (J = q));
      } else {
        for (q = _(O[H]); H < V && (q == null || q !== q); q = _(O[++H])) ;
        for (G = J = q; H < V; ++H)
          q = _(O[H]), q != null && (q < G && (G = q), q > J && (J = q));
      }
    return [G, J];
  }
  function extentIndex(O, _) {
    const H = O.length;
    let V = -1, q, G, J, Z, te;
    if (_ == null) {
      for (; ++V < H; )
        if (G = O[V], G != null && G >= G) {
          q = J = G;
          break;
        }
      if (V === H) return [-1, -1];
      for (Z = te = V; ++V < H; )
        G = O[V], G != null && (q > G && (q = G, Z = V), J < G && (J = G, te = V));
    } else {
      for (; ++V < H; )
        if (G = _(O[V], V, O), G != null && G >= G) {
          q = J = G;
          break;
        }
      if (V === H) return [-1, -1];
      for (Z = te = V; ++V < H; )
        G = _(O[V], V, O), G != null && (q > G && (q = G, Z = V), J < G && (J = G, te = V));
    }
    return [Z, te];
  }
  const hop = Object.prototype.hasOwnProperty;
  function has$1(O, _) {
    return hop.call(O, _);
  }
  const NULL = {};
  function fastmap(O) {
    let _ = {}, H;
    function V(G) {
      return has$1(_, G) && _[G] !== NULL;
    }
    const q = {
      size: 0,
      empty: 0,
      object: _,
      has: V,
      get(G) {
        return V(G) ? _[G] : void 0;
      },
      set(G, J) {
        return V(G) || (++q.size, _[G] === NULL && --q.empty), _[G] = J, this;
      },
      delete(G) {
        return V(G) && (--q.size, ++q.empty, _[G] = NULL), this;
      },
      clear() {
        q.size = q.empty = 0, q.object = _ = {};
      },
      test(G) {
        return arguments.length ? (H = G, q) : H;
      },
      clean() {
        const G = {};
        let J = 0;
        for (const Z in _) {
          const te = _[Z];
          te !== NULL && (!H || !H(te)) && (G[Z] = te, ++J);
        }
        q.size = J, q.empty = 0, q.object = _ = G;
      }
    };
    return O && Object.keys(O).forEach((G) => {
      q.set(G, O[G]);
    }), q;
  }
  function flush$1(O, _, H, V, q, G) {
    if (!H && H !== 0) return G;
    const J = +H;
    let Z = O[0], te = peek$1(O), ne;
    te < Z && (ne = Z, Z = te, te = ne), ne = Math.abs(_ - Z);
    const ee = Math.abs(te - _);
    return ne < ee && ne <= J ? V : ee <= J ? q : G;
  }
  function inherits(O, _, H) {
    const V = O.prototype = Object.create(_.prototype);
    return Object.defineProperty(V, "constructor", {
      value: O,
      writable: !0,
      enumerable: !0,
      configurable: !0
    }), extend$2(V, H);
  }
  function inrange(O, _, H, V) {
    let q = _[0], G = _[_.length - 1], J;
    return q > G && (J = q, q = G, G = J), H = H === void 0 || H, V = V === void 0 || V, (H ? q <= O : q < O) && (V ? O <= G : O < G);
  }
  function isBoolean$3(O) {
    return typeof O == "boolean";
  }
  function isDate$1(O) {
    return Object.prototype.toString.call(O) === "[object Date]";
  }
  function isIterable$1(O) {
    return O && isFunction$2(O[Symbol.iterator]);
  }
  function isNumber$2(O) {
    return typeof O == "number";
  }
  function isRegExp(O) {
    return Object.prototype.toString.call(O) === "[object RegExp]";
  }
  function isString$1(O) {
    return typeof O == "string";
  }
  function key(O, _, H) {
    O && (O = _ ? array$5(O).map((Z) => Z.replace(/\\(.)/g, "$1")) : array$5(O));
    const V = O && O.length, q = H && H.get || getter$1, G = (Z) => q(_ ? [Z] : splitAccessPath(Z));
    let J;
    if (!V)
      J = function() {
        return "";
      };
    else if (V === 1) {
      const Z = G(O[0]);
      J = function(te) {
        return "" + Z(te);
      };
    } else {
      const Z = O.map(G);
      J = function(te) {
        let ne = "" + Z[0](te), ee = 0;
        for (; ++ee < V; ) ne += "|" + Z[ee](te);
        return ne;
      };
    }
    return accessor(J, O, "key");
  }
  function lerp(O, _) {
    const H = O[0], V = peek$1(O), q = +_;
    return q ? q === 1 ? V : H + q * (V - H) : H;
  }
  const DEFAULT_MAX_SIZE = 1e4;
  function lruCache(O) {
    O = +O || DEFAULT_MAX_SIZE;
    let _, H, V;
    const q = () => {
      _ = {}, H = {}, V = 0;
    }, G = (J, Z) => (++V > O && (H = _, _ = {}, V = 1), _[J] = Z);
    return q(), {
      clear: q,
      has: (J) => has$1(_, J) || has$1(H, J),
      get: (J) => has$1(_, J) ? _[J] : has$1(H, J) ? G(J, H[J]) : void 0,
      set: (J, Z) => has$1(_, J) ? _[J] = Z : G(J, Z)
    };
  }
  function merge$3(O, _, H, V) {
    const q = _.length, G = H.length;
    if (!G) return _;
    if (!q) return H;
    const J = V || new _.constructor(q + G);
    let Z = 0, te = 0, ne = 0;
    for (; Z < q && te < G; ++ne)
      J[ne] = O(_[Z], H[te]) > 0 ? H[te++] : _[Z++];
    for (; Z < q; ++Z, ++ne)
      J[ne] = _[Z];
    for (; te < G; ++te, ++ne)
      J[ne] = H[te];
    return J;
  }
  function repeat(O, _) {
    let H = "";
    for (; --_ >= 0; ) H += O;
    return H;
  }
  function pad$2(O, _, H, V) {
    const q = H || " ", G = O + "", J = _ - G.length;
    return J <= 0 ? G : V === "left" ? repeat(q, J) + G : V === "center" ? repeat(q, ~~(J / 2)) + G + repeat(q, Math.ceil(J / 2)) : G + repeat(q, J);
  }
  function span(O) {
    return O && peek$1(O) - O[0] || 0;
  }
  function $$2(O) {
    return isArray$2(O) ? "[" + O.map($$2) + "]" : isObject$2(O) || isString$1(O) ? (
      // Output valid JSON and JS source strings.
      // See http://timelessrepo.com/json-isnt-a-javascript-subset
      JSON.stringify(O).replace("\u2028", "\\u2028").replace("\u2029", "\\u2029")
    ) : O;
  }
  function toBoolean(O) {
    return O == null || O === "" ? null : !O || O === "false" || O === "0" ? !1 : !!O;
  }
  const defaultParser = (O) => isNumber$2(O) || isDate$1(O) ? O : Date.parse(O);
  function toDate(O, _) {
    return _ = _ || defaultParser, O == null || O === "" ? null : _(O);
  }
  function toString$1(O) {
    return O == null || O === "" ? null : O + "";
  }
  function toSet$1(O) {
    const _ = {}, H = O.length;
    for (let V = 0; V < H; ++V) _[O[V]] = !0;
    return _;
  }
  function truncate$1(O, _, H, V) {
    const q = V ?? "…", G = O + "", J = G.length, Z = Math.max(0, _ - q.length);
    return J <= _ ? G : H === "left" ? q + G.slice(J - Z) : H === "center" ? G.slice(0, Math.ceil(Z / 2)) + q + G.slice(J - ~~(Z / 2)) : G.slice(0, Z) + q;
  }
  function visitArray(O, _, H) {
    if (O)
      if (_) {
        const V = O.length;
        for (let q = 0; q < V; ++q) {
          const G = _(O[q]);
          G && H(G, q, O);
        }
      } else
        O.forEach(H);
  }
  var EOL = {}, EOF = {}, QUOTE = 34, NEWLINE = 10, RETURN = 13;
  function objectConverter(O) {
    return new Function("d", "return {" + O.map(function(_, H) {
      return JSON.stringify(_) + ": d[" + H + '] || ""';
    }).join(",") + "}");
  }
  function customConverter(O, _) {
    var H = objectConverter(O);
    return function(V, q) {
      return _(H(V), q, O);
    };
  }
  function inferColumns(O) {
    var _ = /* @__PURE__ */ Object.create(null), H = [];
    return O.forEach(function(V) {
      for (var q in V)
        q in _ || H.push(_[q] = q);
    }), H;
  }
  function pad$1(O, _) {
    var H = O + "", V = H.length;
    return V < _ ? new Array(_ - V + 1).join(0) + H : H;
  }
  function formatYear$1(O) {
    return O < 0 ? "-" + pad$1(-O, 6) : O > 9999 ? "+" + pad$1(O, 6) : pad$1(O, 4);
  }
  function formatDate(O) {
    var _ = O.getUTCHours(), H = O.getUTCMinutes(), V = O.getUTCSeconds(), q = O.getUTCMilliseconds();
    return isNaN(O) ? "Invalid Date" : formatYear$1(O.getUTCFullYear()) + "-" + pad$1(O.getUTCMonth() + 1, 2) + "-" + pad$1(O.getUTCDate(), 2) + (q ? "T" + pad$1(_, 2) + ":" + pad$1(H, 2) + ":" + pad$1(V, 2) + "." + pad$1(q, 3) + "Z" : V ? "T" + pad$1(_, 2) + ":" + pad$1(H, 2) + ":" + pad$1(V, 2) + "Z" : H || _ ? "T" + pad$1(_, 2) + ":" + pad$1(H, 2) + "Z" : "");
  }
  function dsvFormat(O) {
    var _ = new RegExp('["' + O + `
\r]`), H = O.charCodeAt(0);
    function V(ae, oe) {
      var se, ce, le = q(ae, function(ue, de) {
        if (se) return se(ue, de - 1);
        ce = ue, se = oe ? customConverter(ue, oe) : objectConverter(ue);
      });
      return le.columns = ce || [], le;
    }
    function q(ae, oe) {
      var se = [], ce = ae.length, le = 0, ue = 0, de, fe = ce <= 0, he = !1;
      ae.charCodeAt(ce - 1) === NEWLINE && --ce, ae.charCodeAt(ce - 1) === RETURN && --ce;
      function me() {
        if (fe) return EOF;
        if (he) return he = !1, EOL;
        var ye, ge = le, ve;
        if (ae.charCodeAt(ge) === QUOTE) {
          for (; le++ < ce && ae.charCodeAt(le) !== QUOTE || ae.charCodeAt(++le) === QUOTE; ) ;
          return (ye = le) >= ce ? fe = !0 : (ve = ae.charCodeAt(le++)) === NEWLINE ? he = !0 : ve === RETURN && (he = !0, ae.charCodeAt(le) === NEWLINE && ++le), ae.slice(ge + 1, ye - 1).replace(/""/g, '"');
        }
        for (; le < ce; ) {
          if ((ve = ae.charCodeAt(ye = le++)) === NEWLINE) he = !0;
          else if (ve === RETURN)
            he = !0, ae.charCodeAt(le) === NEWLINE && ++le;
          else if (ve !== H) continue;
          return ae.slice(ge, ye);
        }
        return fe = !0, ae.slice(ge, ce);
      }
      for (; (de = me()) !== EOF; ) {
        for (var pe = []; de !== EOL && de !== EOF; ) pe.push(de), de = me();
        oe && (pe = oe(pe, ue++)) == null || se.push(pe);
      }
      return se;
    }
    function G(ae, oe) {
      return ae.map(function(se) {
        return oe.map(function(ce) {
          return ee(se[ce]);
        }).join(O);
      });
    }
    function J(ae, oe) {
      return oe == null && (oe = inferColumns(ae)), [oe.map(ee).join(O)].concat(G(ae, oe)).join(`
`);
    }
    function Z(ae, oe) {
      return oe == null && (oe = inferColumns(ae)), G(ae, oe).join(`
`);
    }
    function te(ae) {
      return ae.map(ne).join(`
`);
    }
    function ne(ae) {
      return ae.map(ee).join(O);
    }
    function ee(ae) {
      return ae == null ? "" : ae instanceof Date ? formatDate(ae) : _.test(ae += "") ? '"' + ae.replace(/"/g, '""') + '"' : ae;
    }
    return {
      parse: V,
      parseRows: q,
      format: J,
      formatBody: Z,
      formatRows: te,
      formatRow: ne,
      formatValue: ee
    };
  }
  function identity$6(O) {
    return O;
  }
  function transform$3(O) {
    if (O == null) return identity$6;
    var _, H, V = O.scale[0], q = O.scale[1], G = O.translate[0], J = O.translate[1];
    return function(Z, te) {
      te || (_ = H = 0);
      var ne = 2, ee = Z.length, ae = new Array(ee);
      for (ae[0] = (_ += Z[0]) * V + G, ae[1] = (H += Z[1]) * q + J; ne < ee; ) ae[ne] = Z[ne], ++ne;
      return ae;
    };
  }
  function reverse$3(O, _) {
    for (var H, V = O.length, q = V - _; q < --V; ) H = O[q], O[q++] = O[V], O[V] = H;
  }
  function feature(O, _) {
    return typeof _ == "string" && (_ = O.objects[_]), _.type === "GeometryCollection" ? { type: "FeatureCollection", features: _.geometries.map(function(H) {
      return feature$1(O, H);
    }) } : feature$1(O, _);
  }
  function feature$1(O, _) {
    var H = _.id, V = _.bbox, q = _.properties == null ? {} : _.properties, G = object$1(O, _);
    return H == null && V == null ? { type: "Feature", properties: q, geometry: G } : V == null ? { type: "Feature", id: H, properties: q, geometry: G } : { type: "Feature", id: H, bbox: V, properties: q, geometry: G };
  }
  function object$1(O, _) {
    var H = transform$3(O.transform), V = O.arcs;
    function q(ee, ae) {
      ae.length && ae.pop();
      for (var oe = V[ee < 0 ? ~ee : ee], se = 0, ce = oe.length; se < ce; ++se)
        ae.push(H(oe[se], se));
      ee < 0 && reverse$3(ae, ce);
    }
    function G(ee) {
      return H(ee);
    }
    function J(ee) {
      for (var ae = [], oe = 0, se = ee.length; oe < se; ++oe) q(ee[oe], ae);
      return ae.length < 2 && ae.push(ae[0]), ae;
    }
    function Z(ee) {
      for (var ae = J(ee); ae.length < 4; ) ae.push(ae[0]);
      return ae;
    }
    function te(ee) {
      return ee.map(Z);
    }
    function ne(ee) {
      var ae = ee.type, oe;
      switch (ae) {
        case "GeometryCollection":
          return { type: ae, geometries: ee.geometries.map(ne) };
        case "Point":
          oe = G(ee.coordinates);
          break;
        case "MultiPoint":
          oe = ee.coordinates.map(G);
          break;
        case "LineString":
          oe = J(ee.arcs);
          break;
        case "MultiLineString":
          oe = ee.arcs.map(J);
          break;
        case "Polygon":
          oe = te(ee.arcs);
          break;
        case "MultiPolygon":
          oe = ee.arcs.map(te);
          break;
        default:
          return null;
      }
      return { type: ae, coordinates: oe };
    }
    return ne(_);
  }
  function stitch(O, _) {
    var H = {}, V = {}, q = {}, G = [], J = -1;
    _.forEach(function(ne, ee) {
      var ae = O.arcs[ne < 0 ? ~ne : ne], oe;
      ae.length < 3 && !ae[1][0] && !ae[1][1] && (oe = _[++J], _[J] = ne, _[ee] = oe);
    }), _.forEach(function(ne) {
      var ee = Z(ne), ae = ee[0], oe = ee[1], se, ce;
      if (se = q[ae])
        if (delete q[se.end], se.push(ne), se.end = oe, ce = V[oe]) {
          delete V[ce.start];
          var le = ce === se ? se : se.concat(ce);
          V[le.start = se.start] = q[le.end = ce.end] = le;
        } else
          V[se.start] = q[se.end] = se;
      else if (se = V[oe])
        if (delete V[se.start], se.unshift(ne), se.start = ae, ce = q[ae]) {
          delete q[ce.end];
          var ue = ce === se ? se : ce.concat(se);
          V[ue.start = ce.start] = q[ue.end = se.end] = ue;
        } else
          V[se.start] = q[se.end] = se;
      else
        se = [ne], V[se.start = ae] = q[se.end = oe] = se;
    });
    function Z(ne) {
      var ee = O.arcs[ne < 0 ? ~ne : ne], ae = ee[0], oe;
      return O.transform ? (oe = [0, 0], ee.forEach(function(se) {
        oe[0] += se[0], oe[1] += se[1];
      })) : oe = ee[ee.length - 1], ne < 0 ? [oe, ae] : [ae, oe];
    }
    function te(ne, ee) {
      for (var ae in ne) {
        var oe = ne[ae];
        delete ee[oe.start], delete oe.start, delete oe.end, oe.forEach(function(se) {
          H[se < 0 ? ~se : se] = 1;
        }), G.push(oe);
      }
    }
    return te(q, V), te(V, q), _.forEach(function(ne) {
      H[ne < 0 ? ~ne : ne] || G.push([ne]);
    }), G;
  }
  function mesh(O) {
    return object$1(O, meshArcs.apply(this, arguments));
  }
  function meshArcs(O, _, H) {
    var V, q, G;
    if (arguments.length > 1) V = extractArcs(O, _, H);
    else for (q = 0, V = new Array(G = O.arcs.length); q < G; ++q) V[q] = q;
    return { type: "MultiLineString", arcs: stitch(O, V) };
  }
  function extractArcs(O, _, H) {
    var V = [], q = [], G;
    function J(ae) {
      var oe = ae < 0 ? ~ae : ae;
      (q[oe] || (q[oe] = [])).push({ i: ae, g: G });
    }
    function Z(ae) {
      ae.forEach(J);
    }
    function te(ae) {
      ae.forEach(Z);
    }
    function ne(ae) {
      ae.forEach(te);
    }
    function ee(ae) {
      switch (G = ae, ae.type) {
        case "GeometryCollection":
          ae.geometries.forEach(ee);
          break;
        case "LineString":
          Z(ae.arcs);
          break;
        case "MultiLineString":
        case "Polygon":
          te(ae.arcs);
          break;
        case "MultiPolygon":
          ne(ae.arcs);
          break;
      }
    }
    return ee(_), q.forEach(H == null ? function(ae) {
      V.push(ae[0].i);
    } : function(ae) {
      H(ae[0].g, ae[ae.length - 1].g) && V.push(ae[0].i);
    }), V;
  }
  function ascending$1(O, _) {
    return O == null || _ == null ? NaN : O < _ ? -1 : O > _ ? 1 : O >= _ ? 0 : NaN;
  }
  function descending$1(O, _) {
    return O == null || _ == null ? NaN : _ < O ? -1 : _ > O ? 1 : _ >= O ? 0 : NaN;
  }
  function bisector(O) {
    let _, H, V;
    O.length !== 2 ? (_ = ascending$1, H = (Z, te) => ascending$1(O(Z), te), V = (Z, te) => O(Z) - te) : (_ = O === ascending$1 || O === descending$1 ? O : zero$3, H = O, V = O);
    function q(Z, te, ne = 0, ee = Z.length) {
      if (ne < ee) {
        if (_(te, te) !== 0) return ee;
        do {
          const ae = ne + ee >>> 1;
          H(Z[ae], te) < 0 ? ne = ae + 1 : ee = ae;
        } while (ne < ee);
      }
      return ne;
    }
    function G(Z, te, ne = 0, ee = Z.length) {
      if (ne < ee) {
        if (_(te, te) !== 0) return ee;
        do {
          const ae = ne + ee >>> 1;
          H(Z[ae], te) <= 0 ? ne = ae + 1 : ee = ae;
        } while (ne < ee);
      }
      return ne;
    }
    function J(Z, te, ne = 0, ee = Z.length) {
      const ae = q(Z, te, ne, ee - 1);
      return ae > ne && V(Z[ae - 1], te) > -V(Z[ae], te) ? ae - 1 : ae;
    }
    return { left: q, center: J, right: G };
  }
  function zero$3() {
    return 0;
  }
  function number$7(O) {
    return O === null ? NaN : +O;
  }
  function* numbers$2(O, _) {
    if (_ === void 0)
      for (let H of O)
        H != null && (H = +H) >= H && (yield H);
    else {
      let H = -1;
      for (let V of O)
        (V = _(V, ++H, O)) != null && (V = +V) >= V && (yield V);
    }
  }
  const ascendingBisect = bisector(ascending$1), bisectRight$1 = ascendingBisect.right, bisectLeft$1 = ascendingBisect.left;
  bisector(number$7).center;
  function variance(O, _) {
    let H = 0, V, q = 0, G = 0;
    if (_ === void 0)
      for (let J of O)
        J != null && (J = +J) >= J && (V = J - q, q += V / ++H, G += V * (J - q));
    else {
      let J = -1;
      for (let Z of O)
        (Z = _(Z, ++J, O)) != null && (Z = +Z) >= Z && (V = Z - q, q += V / ++H, G += V * (Z - q));
    }
    if (H > 1) return G / (H - 1);
  }
  function deviation(O, _) {
    const H = variance(O, _);
    return H && Math.sqrt(H);
  }
  class Adder {
    constructor() {
      this._partials = new Float64Array(32), this._n = 0;
    }
    add(_) {
      const H = this._partials;
      let V = 0;
      for (let q = 0; q < this._n && q < 32; q++) {
        const G = H[q], J = _ + G, Z = Math.abs(_) < Math.abs(G) ? _ - (J - G) : G - (J - _);
        Z && (H[V++] = Z), _ = J;
      }
      return H[V] = _, this._n = V + 1, this;
    }
    valueOf() {
      const _ = this._partials;
      let H = this._n, V, q, G, J = 0;
      if (H > 0) {
        for (J = _[--H]; H > 0 && (V = J, q = _[--H], J = V + q, G = q - (J - V), !G); )
          ;
        H > 0 && (G < 0 && _[H - 1] < 0 || G > 0 && _[H - 1] > 0) && (q = G * 2, V = J + q, q == V - J && (J = V));
      }
      return J;
    }
  }
  class InternMap extends Map {
    constructor(_, H = keyof) {
      if (super(), Object.defineProperties(this, { _intern: { value: /* @__PURE__ */ new Map() }, _key: { value: H } }), _ != null) for (const [V, q] of _) this.set(V, q);
    }
    get(_) {
      return super.get(intern_get(this, _));
    }
    has(_) {
      return super.has(intern_get(this, _));
    }
    set(_, H) {
      return super.set(intern_set(this, _), H);
    }
    delete(_) {
      return super.delete(intern_delete(this, _));
    }
  }
  class InternSet extends Set {
    constructor(_, H = keyof) {
      if (super(), Object.defineProperties(this, { _intern: { value: /* @__PURE__ */ new Map() }, _key: { value: H } }), _ != null) for (const V of _) this.add(V);
    }
    has(_) {
      return super.has(intern_get(this, _));
    }
    add(_) {
      return super.add(intern_set(this, _));
    }
    delete(_) {
      return super.delete(intern_delete(this, _));
    }
  }
  function intern_get({ _intern: O, _key: _ }, H) {
    const V = _(H);
    return O.has(V) ? O.get(V) : H;
  }
  function intern_set({ _intern: O, _key: _ }, H) {
    const V = _(H);
    return O.has(V) ? O.get(V) : (O.set(V, H), H);
  }
  function intern_delete({ _intern: O, _key: _ }, H) {
    const V = _(H);
    return O.has(V) && (H = O.get(V), O.delete(V)), H;
  }
  function keyof(O) {
    return O !== null && typeof O == "object" ? O.valueOf() : O;
  }
  function permute(O, _) {
    return Array.from(_, (H) => O[H]);
  }
  function compareDefined(O = ascending$1) {
    if (O === ascending$1) return ascendingDefined;
    if (typeof O != "function") throw new TypeError("compare is not a function");
    return (_, H) => {
      const V = O(_, H);
      return V || V === 0 ? V : (O(H, H) === 0) - (O(_, _) === 0);
    };
  }
  function ascendingDefined(O, _) {
    return (O == null || !(O >= O)) - (_ == null || !(_ >= _)) || (O < _ ? -1 : O > _ ? 1 : 0);
  }
  const e10 = Math.sqrt(50), e5 = Math.sqrt(10), e2 = Math.sqrt(2);
  function tickSpec(O, _, H) {
    const V = (_ - O) / Math.max(0, H), q = Math.floor(Math.log10(V)), G = V / Math.pow(10, q), J = G >= e10 ? 10 : G >= e5 ? 5 : G >= e2 ? 2 : 1;
    let Z, te, ne;
    return q < 0 ? (ne = Math.pow(10, -q) / J, Z = Math.round(O * ne), te = Math.round(_ * ne), Z / ne < O && ++Z, te / ne > _ && --te, ne = -ne) : (ne = Math.pow(10, q) * J, Z = Math.round(O / ne), te = Math.round(_ / ne), Z * ne < O && ++Z, te * ne > _ && --te), te < Z && 0.5 <= H && H < 2 ? tickSpec(O, _, H * 2) : [Z, te, ne];
  }
  function ticks(O, _, H) {
    if (_ = +_, O = +O, H = +H, !(H > 0)) return [];
    if (O === _) return [O];
    const V = _ < O, [q, G, J] = V ? tickSpec(_, O, H) : tickSpec(O, _, H);
    if (!(G >= q)) return [];
    const Z = G - q + 1, te = new Array(Z);
    if (V)
      if (J < 0) for (let ne = 0; ne < Z; ++ne) te[ne] = (G - ne) / -J;
      else for (let ne = 0; ne < Z; ++ne) te[ne] = (G - ne) * J;
    else if (J < 0) for (let ne = 0; ne < Z; ++ne) te[ne] = (q + ne) / -J;
    else for (let ne = 0; ne < Z; ++ne) te[ne] = (q + ne) * J;
    return te;
  }
  function tickIncrement(O, _, H) {
    return _ = +_, O = +O, H = +H, tickSpec(O, _, H)[2];
  }
  function tickStep(O, _, H) {
    _ = +_, O = +O, H = +H;
    const V = _ < O, q = V ? tickIncrement(_, O, H) : tickIncrement(O, _, H);
    return (V ? -1 : 1) * (q < 0 ? 1 / -q : q);
  }
  function max$2(O, _) {
    let H;
    if (_ === void 0)
      for (const V of O)
        V != null && (H < V || H === void 0 && V >= V) && (H = V);
    else {
      let V = -1;
      for (let q of O)
        (q = _(q, ++V, O)) != null && (H < q || H === void 0 && q >= q) && (H = q);
    }
    return H;
  }
  function min$2(O, _) {
    let H;
    if (_ === void 0)
      for (const V of O)
        V != null && (H > V || H === void 0 && V >= V) && (H = V);
    else {
      let V = -1;
      for (let q of O)
        (q = _(q, ++V, O)) != null && (H > q || H === void 0 && q >= q) && (H = q);
    }
    return H;
  }
  function quickselect(O, _, H = 0, V = 1 / 0, q) {
    if (_ = Math.floor(_), H = Math.floor(Math.max(0, H)), V = Math.floor(Math.min(O.length - 1, V)), !(H <= _ && _ <= V)) return O;
    for (q = q === void 0 ? ascendingDefined : compareDefined(q); V > H; ) {
      if (V - H > 600) {
        const te = V - H + 1, ne = _ - H + 1, ee = Math.log(te), ae = 0.5 * Math.exp(2 * ee / 3), oe = 0.5 * Math.sqrt(ee * ae * (te - ae) / te) * (ne - te / 2 < 0 ? -1 : 1), se = Math.max(H, Math.floor(_ - ne * ae / te + oe)), ce = Math.min(V, Math.floor(_ + (te - ne) * ae / te + oe));
        quickselect(O, _, se, ce, q);
      }
      const G = O[_];
      let J = H, Z = V;
      for (swap$1(O, H, _), q(O[V], G) > 0 && swap$1(O, H, V); J < Z; ) {
        for (swap$1(O, J, Z), ++J, --Z; q(O[J], G) < 0; ) ++J;
        for (; q(O[Z], G) > 0; ) --Z;
      }
      q(O[H], G) === 0 ? swap$1(O, H, Z) : (++Z, swap$1(O, Z, V)), Z <= _ && (H = Z + 1), _ <= Z && (V = Z - 1);
    }
    return O;
  }
  function swap$1(O, _, H) {
    const V = O[_];
    O[_] = O[H], O[H] = V;
  }
  function quantile$1(O, _, H) {
    if (O = Float64Array.from(numbers$2(O, H)), !(!(V = O.length) || isNaN(_ = +_))) {
      if (_ <= 0 || V < 2) return min$2(O);
      if (_ >= 1) return max$2(O);
      var V, q = (V - 1) * _, G = Math.floor(q), J = max$2(quickselect(O, G).subarray(0, G + 1)), Z = min$2(O.subarray(G + 1));
      return J + (Z - J) * (q - G);
    }
  }
  function quantileSorted(O, _, H = number$7) {
    if (!(!(V = O.length) || isNaN(_ = +_))) {
      if (_ <= 0 || V < 2) return +H(O[0], 0, O);
      if (_ >= 1) return +H(O[V - 1], V - 1, O);
      var V, q = (V - 1) * _, G = Math.floor(q), J = +H(O[G], G, O), Z = +H(O[G + 1], G + 1, O);
      return J + (Z - J) * (q - G);
    }
  }
  function mean(O, _) {
    let H = 0, V = 0;
    if (_ === void 0)
      for (let q of O)
        q != null && (q = +q) >= q && (++H, V += q);
    else {
      let q = -1;
      for (let G of O)
        (G = _(G, ++q, O)) != null && (G = +G) >= G && (++H, V += G);
    }
    if (H) return V / H;
  }
  function median(O, _) {
    return quantile$1(O, 0.5, _);
  }
  function* flatten$1(O) {
    for (const _ of O)
      yield* _;
  }
  function merge$2(O) {
    return Array.from(flatten$1(O));
  }
  function range$7(O, _, H) {
    O = +O, _ = +_, H = (q = arguments.length) < 2 ? (_ = O, O = 0, 1) : q < 3 ? 1 : +H;
    for (var V = -1, q = Math.max(0, Math.ceil((_ - O) / H)) | 0, G = new Array(q); ++V < q; )
      G[V] = O + V * H;
    return G;
  }
  function sum$1(O, _) {
    let H = 0;
    for (let V of O)
      (V = +V) && (H += V);
    return H;
  }
  function intersection(O, ..._) {
    O = new InternSet(O), _ = _.map(set$6);
    e: for (const H of O)
      for (const V of _)
        if (!V.has(H)) {
          O.delete(H);
          continue e;
        }
    return O;
  }
  function set$6(O) {
    return O instanceof InternSet ? O : new InternSet(O);
  }
  function union(...O) {
    const _ = new InternSet();
    for (const H of O)
      for (const V of H)
        _.add(V);
    return _;
  }
  function formatDecimal(O) {
    return Math.abs(O = Math.round(O)) >= 1e21 ? O.toLocaleString("en").replace(/,/g, "") : O.toString(10);
  }
  function formatDecimalParts(O, _) {
    if ((H = (O = _ ? O.toExponential(_ - 1) : O.toExponential()).indexOf("e")) < 0) return null;
    var H, V = O.slice(0, H);
    return [
      V.length > 1 ? V[0] + V.slice(2) : V,
      +O.slice(H + 1)
    ];
  }
  function exponent(O) {
    return O = formatDecimalParts(Math.abs(O)), O ? O[1] : NaN;
  }
  function formatGroup(O, _) {
    return function(H, V) {
      for (var q = H.length, G = [], J = 0, Z = O[0], te = 0; q > 0 && Z > 0 && (te + Z + 1 > V && (Z = Math.max(1, V - te)), G.push(H.substring(q -= Z, q + Z)), !((te += Z + 1) > V)); )
        Z = O[J = (J + 1) % O.length];
      return G.reverse().join(_);
    };
  }
  function formatNumerals(O) {
    return function(_) {
      return _.replace(/[0-9]/g, function(H) {
        return O[+H];
      });
    };
  }
  var re$1 = /^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i;
  function formatSpecifier(O) {
    if (!(_ = re$1.exec(O))) throw new Error("invalid format: " + O);
    var _;
    return new FormatSpecifier({
      fill: _[1],
      align: _[2],
      sign: _[3],
      symbol: _[4],
      zero: _[5],
      width: _[6],
      comma: _[7],
      precision: _[8] && _[8].slice(1),
      trim: _[9],
      type: _[10]
    });
  }
  formatSpecifier.prototype = FormatSpecifier.prototype;
  function FormatSpecifier(O) {
    this.fill = O.fill === void 0 ? " " : O.fill + "", this.align = O.align === void 0 ? ">" : O.align + "", this.sign = O.sign === void 0 ? "-" : O.sign + "", this.symbol = O.symbol === void 0 ? "" : O.symbol + "", this.zero = !!O.zero, this.width = O.width === void 0 ? void 0 : +O.width, this.comma = !!O.comma, this.precision = O.precision === void 0 ? void 0 : +O.precision, this.trim = !!O.trim, this.type = O.type === void 0 ? "" : O.type + "";
  }
  FormatSpecifier.prototype.toString = function() {
    return this.fill + this.align + this.sign + this.symbol + (this.zero ? "0" : "") + (this.width === void 0 ? "" : Math.max(1, this.width | 0)) + (this.comma ? "," : "") + (this.precision === void 0 ? "" : "." + Math.max(0, this.precision | 0)) + (this.trim ? "~" : "") + this.type;
  };
  function formatTrim(O) {
    e: for (var _ = O.length, H = 1, V = -1, q; H < _; ++H)
      switch (O[H]) {
        case ".":
          V = q = H;
          break;
        case "0":
          V === 0 && (V = H), q = H;
          break;
        default:
          if (!+O[H]) break e;
          V > 0 && (V = 0);
          break;
      }
    return V > 0 ? O.slice(0, V) + O.slice(q + 1) : O;
  }
  var prefixExponent;
  function formatPrefixAuto(O, _) {
    var H = formatDecimalParts(O, _);
    if (!H) return O + "";
    var V = H[0], q = H[1], G = q - (prefixExponent = Math.max(-8, Math.min(8, Math.floor(q / 3))) * 3) + 1, J = V.length;
    return G === J ? V : G > J ? V + new Array(G - J + 1).join("0") : G > 0 ? V.slice(0, G) + "." + V.slice(G) : "0." + new Array(1 - G).join("0") + formatDecimalParts(O, Math.max(0, _ + G - 1))[0];
  }
  function formatRounded(O, _) {
    var H = formatDecimalParts(O, _);
    if (!H) return O + "";
    var V = H[0], q = H[1];
    return q < 0 ? "0." + new Array(-q).join("0") + V : V.length > q + 1 ? V.slice(0, q + 1) + "." + V.slice(q + 1) : V + new Array(q - V.length + 2).join("0");
  }
  const formatTypes = {
    "%": (O, _) => (O * 100).toFixed(_),
    b: (O) => Math.round(O).toString(2),
    c: (O) => O + "",
    d: formatDecimal,
    e: (O, _) => O.toExponential(_),
    f: (O, _) => O.toFixed(_),
    g: (O, _) => O.toPrecision(_),
    o: (O) => Math.round(O).toString(8),
    p: (O, _) => formatRounded(O * 100, _),
    r: formatRounded,
    s: formatPrefixAuto,
    X: (O) => Math.round(O).toString(16).toUpperCase(),
    x: (O) => Math.round(O).toString(16)
  };
  function identity$5(O) {
    return O;
  }
  var map$2 = Array.prototype.map, prefixes = ["y", "z", "a", "f", "p", "n", "µ", "m", "", "k", "M", "G", "T", "P", "E", "Z", "Y"];
  function formatLocale$1(O) {
    var _ = O.grouping === void 0 || O.thousands === void 0 ? identity$5 : formatGroup(map$2.call(O.grouping, Number), O.thousands + ""), H = O.currency === void 0 ? "" : O.currency[0] + "", V = O.currency === void 0 ? "" : O.currency[1] + "", q = O.decimal === void 0 ? "." : O.decimal + "", G = O.numerals === void 0 ? identity$5 : formatNumerals(map$2.call(O.numerals, String)), J = O.percent === void 0 ? "%" : O.percent + "", Z = O.minus === void 0 ? "−" : O.minus + "", te = O.nan === void 0 ? "NaN" : O.nan + "";
    function ne(ae) {
      ae = formatSpecifier(ae);
      var oe = ae.fill, se = ae.align, ce = ae.sign, le = ae.symbol, ue = ae.zero, de = ae.width, fe = ae.comma, he = ae.precision, me = ae.trim, pe = ae.type;
      pe === "n" ? (fe = !0, pe = "g") : formatTypes[pe] || (he === void 0 && (he = 12), me = !0, pe = "g"), (ue || oe === "0" && se === "=") && (ue = !0, oe = "0", se = "=");
      var ye = le === "$" ? H : le === "#" && /[boxX]/.test(pe) ? "0" + pe.toLowerCase() : "", ge = le === "$" ? V : /[%p]/.test(pe) ? J : "", ve = formatTypes[pe], be = /[defgprs%]/.test(pe);
      he = he === void 0 ? 6 : /[gprs]/.test(pe) ? Math.max(1, Math.min(21, he)) : Math.max(0, Math.min(20, he));
      function Ce(Se) {
        var we = ye, Ee = ge, xe, Le, Qe;
        if (pe === "c")
          Ee = ve(Se) + Ee, Se = "";
        else {
          Se = +Se;
          var ke = Se < 0 || 1 / Se < 0;
          if (Se = isNaN(Se) ? te : ve(Math.abs(Se), he), me && (Se = formatTrim(Se)), ke && +Se == 0 && ce !== "+" && (ke = !1), we = (ke ? ce === "(" ? ce : Z : ce === "-" || ce === "(" ? "" : ce) + we, Ee = (pe === "s" ? prefixes[8 + prefixExponent / 3] : "") + Ee + (ke && ce === "(" ? ")" : ""), be) {
            for (xe = -1, Le = Se.length; ++xe < Le; )
              if (Qe = Se.charCodeAt(xe), 48 > Qe || Qe > 57) {
                Ee = (Qe === 46 ? q + Se.slice(xe + 1) : Se.slice(xe)) + Ee, Se = Se.slice(0, xe);
                break;
              }
          }
        }
        fe && !ue && (Se = _(Se, 1 / 0));
        var Me = we.length + Se.length + Ee.length, Re = Me < de ? new Array(de - Me + 1).join(oe) : "";
        switch (fe && ue && (Se = _(Re + Se, Re.length ? de - Ee.length : 1 / 0), Re = ""), se) {
          case "<":
            Se = we + Se + Ee + Re;
            break;
          case "=":
            Se = we + Re + Se + Ee;
            break;
          case "^":
            Se = Re.slice(0, Me = Re.length >> 1) + we + Se + Ee + Re.slice(Me);
            break;
          default:
            Se = Re + we + Se + Ee;
            break;
        }
        return G(Se);
      }
      return Ce.toString = function() {
        return ae + "";
      }, Ce;
    }
    function ee(ae, oe) {
      var se = ne((ae = formatSpecifier(ae), ae.type = "f", ae)), ce = Math.max(-8, Math.min(8, Math.floor(exponent(oe) / 3))) * 3, le = Math.pow(10, -ce), ue = prefixes[8 + ce / 3];
      return function(de) {
        return se(le * de) + ue;
      };
    }
    return {
      format: ne,
      formatPrefix: ee
    };
  }
  var locale$2, format$3, formatPrefix;
  defaultLocale$2({
    thousands: ",",
    grouping: [3],
    currency: ["$", ""]
  });
  function defaultLocale$2(O) {
    return locale$2 = formatLocale$1(O), format$3 = locale$2.format, formatPrefix = locale$2.formatPrefix, locale$2;
  }
  function precisionFixed(O) {
    return Math.max(0, -exponent(Math.abs(O)));
  }
  function precisionPrefix(O, _) {
    return Math.max(0, Math.max(-8, Math.min(8, Math.floor(exponent(_) / 3))) * 3 - exponent(Math.abs(O)));
  }
  function precisionRound(O, _) {
    return O = Math.abs(O), _ = Math.abs(_) - O, Math.max(0, exponent(_) - exponent(O)) + 1;
  }
  const t0$2 = /* @__PURE__ */ new Date(), t1$1 = /* @__PURE__ */ new Date();
  function timeInterval$1(O, _, H, V) {
    function q(G) {
      return O(G = arguments.length === 0 ? /* @__PURE__ */ new Date() : /* @__PURE__ */ new Date(+G)), G;
    }
    return q.floor = (G) => (O(G = /* @__PURE__ */ new Date(+G)), G), q.ceil = (G) => (O(G = new Date(G - 1)), _(G, 1), O(G), G), q.round = (G) => {
      const J = q(G), Z = q.ceil(G);
      return G - J < Z - G ? J : Z;
    }, q.offset = (G, J) => (_(G = /* @__PURE__ */ new Date(+G), J == null ? 1 : Math.floor(J)), G), q.range = (G, J, Z) => {
      const te = [];
      if (G = q.ceil(G), Z = Z == null ? 1 : Math.floor(Z), !(G < J) || !(Z > 0)) return te;
      let ne;
      do
        te.push(ne = /* @__PURE__ */ new Date(+G)), _(G, Z), O(G);
      while (ne < G && G < J);
      return te;
    }, q.filter = (G) => timeInterval$1((J) => {
      if (J >= J) for (; O(J), !G(J); ) J.setTime(J - 1);
    }, (J, Z) => {
      if (J >= J)
        if (Z < 0) for (; ++Z <= 0; )
          for (; _(J, -1), !G(J); )
            ;
        else for (; --Z >= 0; )
          for (; _(J, 1), !G(J); )
            ;
    }), H && (q.count = (G, J) => (t0$2.setTime(+G), t1$1.setTime(+J), O(t0$2), O(t1$1), Math.floor(H(t0$2, t1$1))), q.every = (G) => (G = Math.floor(G), !isFinite(G) || !(G > 0) ? null : G > 1 ? q.filter(V ? (J) => V(J) % G === 0 : (J) => q.count(0, J) % G === 0) : q)), q;
  }
  const millisecond = timeInterval$1(() => {
  }, (O, _) => {
    O.setTime(+O + _);
  }, (O, _) => _ - O);
  millisecond.every = (O) => (O = Math.floor(O), !isFinite(O) || !(O > 0) ? null : O > 1 ? timeInterval$1((_) => {
    _.setTime(Math.floor(_ / O) * O);
  }, (_, H) => {
    _.setTime(+_ + H * O);
  }, (_, H) => (H - _) / O) : millisecond);
  millisecond.range;
  const durationSecond$1 = 1e3, durationMinute$1 = durationSecond$1 * 60, durationHour$1 = durationMinute$1 * 60, durationDay$1 = durationHour$1 * 24, durationWeek$1 = durationDay$1 * 7, durationMonth$1 = durationDay$1 * 30, durationYear$1 = durationDay$1 * 365, second = timeInterval$1((O) => {
    O.setTime(O - O.getMilliseconds());
  }, (O, _) => {
    O.setTime(+O + _ * durationSecond$1);
  }, (O, _) => (_ - O) / durationSecond$1, (O) => O.getUTCSeconds());
  second.range;
  const timeMinute = timeInterval$1((O) => {
    O.setTime(O - O.getMilliseconds() - O.getSeconds() * durationSecond$1);
  }, (O, _) => {
    O.setTime(+O + _ * durationMinute$1);
  }, (O, _) => (_ - O) / durationMinute$1, (O) => O.getMinutes());
  timeMinute.range;
  const utcMinute = timeInterval$1((O) => {
    O.setUTCSeconds(0, 0);
  }, (O, _) => {
    O.setTime(+O + _ * durationMinute$1);
  }, (O, _) => (_ - O) / durationMinute$1, (O) => O.getUTCMinutes());
  utcMinute.range;
  const timeHour = timeInterval$1((O) => {
    O.setTime(O - O.getMilliseconds() - O.getSeconds() * durationSecond$1 - O.getMinutes() * durationMinute$1);
  }, (O, _) => {
    O.setTime(+O + _ * durationHour$1);
  }, (O, _) => (_ - O) / durationHour$1, (O) => O.getHours());
  timeHour.range;
  const utcHour = timeInterval$1((O) => {
    O.setUTCMinutes(0, 0, 0);
  }, (O, _) => {
    O.setTime(+O + _ * durationHour$1);
  }, (O, _) => (_ - O) / durationHour$1, (O) => O.getUTCHours());
  utcHour.range;
  const timeDay = timeInterval$1(
    (O) => O.setHours(0, 0, 0, 0),
    (O, _) => O.setDate(O.getDate() + _),
    (O, _) => (_ - O - (_.getTimezoneOffset() - O.getTimezoneOffset()) * durationMinute$1) / durationDay$1,
    (O) => O.getDate() - 1
  );
  timeDay.range;
  const utcDay = timeInterval$1((O) => {
    O.setUTCHours(0, 0, 0, 0);
  }, (O, _) => {
    O.setUTCDate(O.getUTCDate() + _);
  }, (O, _) => (_ - O) / durationDay$1, (O) => O.getUTCDate() - 1);
  utcDay.range;
  const unixDay = timeInterval$1((O) => {
    O.setUTCHours(0, 0, 0, 0);
  }, (O, _) => {
    O.setUTCDate(O.getUTCDate() + _);
  }, (O, _) => (_ - O) / durationDay$1, (O) => Math.floor(O / durationDay$1));
  unixDay.range;
  function timeWeekday(O) {
    return timeInterval$1((_) => {
      _.setDate(_.getDate() - (_.getDay() + 7 - O) % 7), _.setHours(0, 0, 0, 0);
    }, (_, H) => {
      _.setDate(_.getDate() + H * 7);
    }, (_, H) => (H - _ - (H.getTimezoneOffset() - _.getTimezoneOffset()) * durationMinute$1) / durationWeek$1);
  }
  const timeSunday = timeWeekday(0), timeMonday = timeWeekday(1), timeTuesday = timeWeekday(2), timeWednesday = timeWeekday(3), timeThursday = timeWeekday(4), timeFriday = timeWeekday(5), timeSaturday = timeWeekday(6);
  timeSunday.range;
  timeMonday.range;
  timeTuesday.range;
  timeWednesday.range;
  timeThursday.range;
  timeFriday.range;
  timeSaturday.range;
  function utcWeekday(O) {
    return timeInterval$1((_) => {
      _.setUTCDate(_.getUTCDate() - (_.getUTCDay() + 7 - O) % 7), _.setUTCHours(0, 0, 0, 0);
    }, (_, H) => {
      _.setUTCDate(_.getUTCDate() + H * 7);
    }, (_, H) => (H - _) / durationWeek$1);
  }
  const utcSunday = utcWeekday(0), utcMonday = utcWeekday(1), utcTuesday = utcWeekday(2), utcWednesday = utcWeekday(3), utcThursday = utcWeekday(4), utcFriday = utcWeekday(5), utcSaturday = utcWeekday(6);
  utcSunday.range;
  utcMonday.range;
  utcTuesday.range;
  utcWednesday.range;
  utcThursday.range;
  utcFriday.range;
  utcSaturday.range;
  const timeMonth = timeInterval$1((O) => {
    O.setDate(1), O.setHours(0, 0, 0, 0);
  }, (O, _) => {
    O.setMonth(O.getMonth() + _);
  }, (O, _) => _.getMonth() - O.getMonth() + (_.getFullYear() - O.getFullYear()) * 12, (O) => O.getMonth());
  timeMonth.range;
  const utcMonth = timeInterval$1((O) => {
    O.setUTCDate(1), O.setUTCHours(0, 0, 0, 0);
  }, (O, _) => {
    O.setUTCMonth(O.getUTCMonth() + _);
  }, (O, _) => _.getUTCMonth() - O.getUTCMonth() + (_.getUTCFullYear() - O.getUTCFullYear()) * 12, (O) => O.getUTCMonth());
  utcMonth.range;
  const timeYear = timeInterval$1((O) => {
    O.setMonth(0, 1), O.setHours(0, 0, 0, 0);
  }, (O, _) => {
    O.setFullYear(O.getFullYear() + _);
  }, (O, _) => _.getFullYear() - O.getFullYear(), (O) => O.getFullYear());
  timeYear.every = (O) => !isFinite(O = Math.floor(O)) || !(O > 0) ? null : timeInterval$1((_) => {
    _.setFullYear(Math.floor(_.getFullYear() / O) * O), _.setMonth(0, 1), _.setHours(0, 0, 0, 0);
  }, (_, H) => {
    _.setFullYear(_.getFullYear() + H * O);
  });
  timeYear.range;
  const utcYear = timeInterval$1((O) => {
    O.setUTCMonth(0, 1), O.setUTCHours(0, 0, 0, 0);
  }, (O, _) => {
    O.setUTCFullYear(O.getUTCFullYear() + _);
  }, (O, _) => _.getUTCFullYear() - O.getUTCFullYear(), (O) => O.getUTCFullYear());
  utcYear.every = (O) => !isFinite(O = Math.floor(O)) || !(O > 0) ? null : timeInterval$1((_) => {
    _.setUTCFullYear(Math.floor(_.getUTCFullYear() / O) * O), _.setUTCMonth(0, 1), _.setUTCHours(0, 0, 0, 0);
  }, (_, H) => {
    _.setUTCFullYear(_.getUTCFullYear() + H * O);
  });
  utcYear.range;
  function ticker(O, _, H, V, q, G) {
    const J = [
      [second, 1, durationSecond$1],
      [second, 5, 5 * durationSecond$1],
      [second, 15, 15 * durationSecond$1],
      [second, 30, 30 * durationSecond$1],
      [G, 1, durationMinute$1],
      [G, 5, 5 * durationMinute$1],
      [G, 15, 15 * durationMinute$1],
      [G, 30, 30 * durationMinute$1],
      [q, 1, durationHour$1],
      [q, 3, 3 * durationHour$1],
      [q, 6, 6 * durationHour$1],
      [q, 12, 12 * durationHour$1],
      [V, 1, durationDay$1],
      [V, 2, 2 * durationDay$1],
      [H, 1, durationWeek$1],
      [_, 1, durationMonth$1],
      [_, 3, 3 * durationMonth$1],
      [O, 1, durationYear$1]
    ];
    function Z(ne, ee, ae) {
      const oe = ee < ne;
      oe && ([ne, ee] = [ee, ne]);
      const se = ae && typeof ae.range == "function" ? ae : te(ne, ee, ae), ce = se ? se.range(ne, +ee + 1) : [];
      return oe ? ce.reverse() : ce;
    }
    function te(ne, ee, ae) {
      const oe = Math.abs(ee - ne) / ae, se = bisector(([, , ue]) => ue).right(J, oe);
      if (se === J.length) return O.every(tickStep(ne / durationYear$1, ee / durationYear$1, ae));
      if (se === 0) return millisecond.every(Math.max(tickStep(ne, ee, ae), 1));
      const [ce, le] = J[oe / J[se - 1][2] < J[se][2] / oe ? se - 1 : se];
      return ce.every(le);
    }
    return [Z, te];
  }
  const [utcTicks, utcTickInterval] = ticker(utcYear, utcMonth, utcSunday, unixDay, utcHour, utcMinute), [timeTicks, timeTickInterval] = ticker(timeYear, timeMonth, timeSunday, timeDay, timeHour, timeMinute), YEAR = "year", QUARTER = "quarter", MONTH = "month", WEEK = "week", DATE = "date", DAY = "day", DAYOFYEAR = "dayofyear", HOURS = "hours", MINUTES = "minutes", SECONDS = "seconds", MILLISECONDS = "milliseconds", TIME_UNITS = [YEAR, QUARTER, MONTH, WEEK, DATE, DAY, DAYOFYEAR, HOURS, MINUTES, SECONDS, MILLISECONDS], UNITS = TIME_UNITS.reduce((O, _, H) => (O[_] = 1 + H, O), {});
  function timeUnits(O) {
    const _ = array$5(O).slice(), H = {};
    return _.length || error$1("Missing time unit."), _.forEach((q) => {
      has$1(UNITS, q) ? H[q] = 1 : error$1(`Invalid time unit: ${q}.`);
    }), (H[WEEK] || H[DAY] ? 1 : 0) + (H[QUARTER] || H[MONTH] || H[DATE] ? 1 : 0) + (H[DAYOFYEAR] ? 1 : 0) > 1 && error$1(`Incompatible time units: ${O}`), _.sort((q, G) => UNITS[q] - UNITS[G]), _;
  }
  const defaultSpecifiers = {
    [YEAR]: "%Y ",
    [QUARTER]: "Q%q ",
    [MONTH]: "%b ",
    [DATE]: "%d ",
    [WEEK]: "W%U ",
    [DAY]: "%a ",
    [DAYOFYEAR]: "%j ",
    [HOURS]: "%H:00",
    [MINUTES]: "00:%M",
    [SECONDS]: ":%S",
    [MILLISECONDS]: ".%L",
    [`${YEAR}-${MONTH}`]: "%Y-%m ",
    [`${YEAR}-${MONTH}-${DATE}`]: "%Y-%m-%d ",
    [`${HOURS}-${MINUTES}`]: "%H:%M"
  };
  function timeUnitSpecifier(O, _) {
    const H = extend$2({}, defaultSpecifiers, _), V = timeUnits(O), q = V.length;
    let G = "", J = 0, Z, te;
    for (J = 0; J < q; )
      for (Z = V.length; Z > J; --Z)
        if (te = V.slice(J, Z).join("-"), H[te] != null) {
          G += H[te], J = Z;
          break;
        }
    return G.trim();
  }
  const t0$1 = /* @__PURE__ */ new Date();
  function localYear(O) {
    return t0$1.setFullYear(O), t0$1.setMonth(0), t0$1.setDate(1), t0$1.setHours(0, 0, 0, 0), t0$1;
  }
  function dayofyear(O) {
    return localDayOfYear(new Date(O));
  }
  function week(O) {
    return localWeekNum(new Date(O));
  }
  function localDayOfYear(O) {
    return timeDay.count(localYear(O.getFullYear()) - 1, O);
  }
  function localWeekNum(O) {
    return timeSunday.count(localYear(O.getFullYear()) - 1, O);
  }
  function localFirst(O) {
    return localYear(O).getDay();
  }
  function localDate$1(O, _, H, V, q, G, J) {
    if (0 <= O && O < 100) {
      const Z = new Date(-1, _, H, V, q, G, J);
      return Z.setFullYear(O), Z;
    }
    return new Date(O, _, H, V, q, G, J);
  }
  function utcdayofyear(O) {
    return utcDayOfYear(new Date(O));
  }
  function utcweek(O) {
    return utcWeekNum(new Date(O));
  }
  function utcDayOfYear(O) {
    const _ = Date.UTC(O.getUTCFullYear(), 0, 1);
    return utcDay.count(_ - 1, O);
  }
  function utcWeekNum(O) {
    const _ = Date.UTC(O.getUTCFullYear(), 0, 1);
    return utcSunday.count(_ - 1, O);
  }
  function utcFirst(O) {
    return t0$1.setTime(Date.UTC(O, 0, 1)), t0$1.getUTCDay();
  }
  function utcDate$1(O, _, H, V, q, G, J) {
    if (0 <= O && O < 100) {
      const Z = new Date(Date.UTC(-1, _, H, V, q, G, J));
      return Z.setUTCFullYear(H.y), Z;
    }
    return new Date(Date.UTC(O, _, H, V, q, G, J));
  }
  function floor(O, _, H, V, q) {
    const G = _ || 1, J = peek$1(O), Z = (de, fe, he) => (he = he || de, getUnit(H[he], V[he], de === J && G, fe)), te = /* @__PURE__ */ new Date(), ne = toSet$1(O), ee = ne[YEAR] ? Z(YEAR) : constant$5(2012), ae = ne[MONTH] ? Z(MONTH) : ne[QUARTER] ? Z(QUARTER) : zero$4, oe = ne[WEEK] && ne[DAY] ? Z(DAY, 1, WEEK + DAY) : ne[WEEK] ? Z(WEEK, 1) : ne[DAY] ? Z(DAY, 1) : ne[DATE] ? Z(DATE, 1) : ne[DAYOFYEAR] ? Z(DAYOFYEAR, 1) : one$2, se = ne[HOURS] ? Z(HOURS) : zero$4, ce = ne[MINUTES] ? Z(MINUTES) : zero$4, le = ne[SECONDS] ? Z(SECONDS) : zero$4, ue = ne[MILLISECONDS] ? Z(MILLISECONDS) : zero$4;
    return function(de) {
      te.setTime(+de);
      const fe = ee(te);
      return q(fe, ae(te), oe(te, fe), se(te), ce(te), le(te), ue(te));
    };
  }
  function getUnit(O, _, H, V) {
    const q = H <= 1 ? O : V ? (G, J) => V + H * Math.floor((O(G, J) - V) / H) : (G, J) => H * Math.floor(O(G, J) / H);
    return _ ? (G, J) => _(q(G, J), J) : q;
  }
  function weekday(O, _, H) {
    return _ + O * 7 - (H + 6) % 7;
  }
  const localGet = {
    [YEAR]: (O) => O.getFullYear(),
    [QUARTER]: (O) => Math.floor(O.getMonth() / 3),
    [MONTH]: (O) => O.getMonth(),
    [DATE]: (O) => O.getDate(),
    [HOURS]: (O) => O.getHours(),
    [MINUTES]: (O) => O.getMinutes(),
    [SECONDS]: (O) => O.getSeconds(),
    [MILLISECONDS]: (O) => O.getMilliseconds(),
    [DAYOFYEAR]: (O) => localDayOfYear(O),
    [WEEK]: (O) => localWeekNum(O),
    [WEEK + DAY]: (O, _) => weekday(localWeekNum(O), O.getDay(), localFirst(_)),
    [DAY]: (O, _) => weekday(1, O.getDay(), localFirst(_))
  }, localInv = {
    [QUARTER]: (O) => 3 * O,
    [WEEK]: (O, _) => weekday(O, 0, localFirst(_))
  };
  function timeFloor(O, _) {
    return floor(O, _ || 1, localGet, localInv, localDate$1);
  }
  const utcGet = {
    [YEAR]: (O) => O.getUTCFullYear(),
    [QUARTER]: (O) => Math.floor(O.getUTCMonth() / 3),
    [MONTH]: (O) => O.getUTCMonth(),
    [DATE]: (O) => O.getUTCDate(),
    [HOURS]: (O) => O.getUTCHours(),
    [MINUTES]: (O) => O.getUTCMinutes(),
    [SECONDS]: (O) => O.getUTCSeconds(),
    [MILLISECONDS]: (O) => O.getUTCMilliseconds(),
    [DAYOFYEAR]: (O) => utcDayOfYear(O),
    [WEEK]: (O) => utcWeekNum(O),
    [DAY]: (O, _) => weekday(1, O.getUTCDay(), utcFirst(_)),
    [WEEK + DAY]: (O, _) => weekday(utcWeekNum(O), O.getUTCDay(), utcFirst(_))
  }, utcInv = {
    [QUARTER]: (O) => 3 * O,
    [WEEK]: (O, _) => weekday(O, 0, utcFirst(_))
  };
  function utcFloor(O, _) {
    return floor(O, _ || 1, utcGet, utcInv, utcDate$1);
  }
  const timeIntervals = {
    [YEAR]: timeYear,
    [QUARTER]: timeMonth.every(3),
    [MONTH]: timeMonth,
    [WEEK]: timeSunday,
    [DATE]: timeDay,
    [DAY]: timeDay,
    [DAYOFYEAR]: timeDay,
    [HOURS]: timeHour,
    [MINUTES]: timeMinute,
    [SECONDS]: second,
    [MILLISECONDS]: millisecond
  }, utcIntervals = {
    [YEAR]: utcYear,
    [QUARTER]: utcMonth.every(3),
    [MONTH]: utcMonth,
    [WEEK]: utcSunday,
    [DATE]: utcDay,
    [DAY]: utcDay,
    [DAYOFYEAR]: utcDay,
    [HOURS]: utcHour,
    [MINUTES]: utcMinute,
    [SECONDS]: second,
    [MILLISECONDS]: millisecond
  };
  function timeInterval(O) {
    return timeIntervals[O];
  }
  function utcInterval(O) {
    return utcIntervals[O];
  }
  function offset$3(O, _, H) {
    return O ? O.offset(_, H) : void 0;
  }
  function timeOffset(O, _, H) {
    return offset$3(timeInterval(O), _, H);
  }
  function utcOffset(O, _, H) {
    return offset$3(utcInterval(O), _, H);
  }
  function sequence$1(O, _, H, V) {
    return O ? O.range(_, H, V) : void 0;
  }
  function timeSequence(O, _, H, V) {
    return sequence$1(timeInterval(O), _, H, V);
  }
  function utcSequence(O, _, H, V) {
    return sequence$1(utcInterval(O), _, H, V);
  }
  const durationSecond = 1e3, durationMinute = durationSecond * 60, durationHour = durationMinute * 60, durationDay = durationHour * 24, durationWeek = durationDay * 7, durationMonth = durationDay * 30, durationYear = durationDay * 365, Milli = [YEAR, MONTH, DATE, HOURS, MINUTES, SECONDS, MILLISECONDS], Seconds = Milli.slice(0, -1), Minutes = Seconds.slice(0, -1), Hours = Minutes.slice(0, -1), Day = Hours.slice(0, -1), Week = [YEAR, WEEK], Month = [YEAR, MONTH], Year = [YEAR], intervals = [[Seconds, 1, durationSecond], [Seconds, 5, 5 * durationSecond], [Seconds, 15, 15 * durationSecond], [Seconds, 30, 30 * durationSecond], [Minutes, 1, durationMinute], [Minutes, 5, 5 * durationMinute], [Minutes, 15, 15 * durationMinute], [Minutes, 30, 30 * durationMinute], [Hours, 1, durationHour], [Hours, 3, 3 * durationHour], [Hours, 6, 6 * durationHour], [Hours, 12, 12 * durationHour], [Day, 1, durationDay], [Week, 1, durationWeek], [Month, 1, durationMonth], [Month, 3, 3 * durationMonth], [Year, 1, durationYear]];
  function bin$2(O) {
    const _ = O.extent, H = O.maxbins || 40, V = Math.abs(span(_)) / H;
    let q = bisector((Z) => Z[2]).right(intervals, V), G, J;
    return q === intervals.length ? (G = Year, J = tickStep(_[0] / durationYear, _[1] / durationYear, H)) : q ? (q = intervals[V / intervals[q - 1][2] < intervals[q][2] / V ? q - 1 : q], G = q[0], J = q[1]) : (G = Milli, J = Math.max(tickStep(_[0], _[1], H), 1)), {
      units: G,
      step: J
    };
  }
  function localDate(O) {
    if (0 <= O.y && O.y < 100) {
      var _ = new Date(-1, O.m, O.d, O.H, O.M, O.S, O.L);
      return _.setFullYear(O.y), _;
    }
    return new Date(O.y, O.m, O.d, O.H, O.M, O.S, O.L);
  }
  function utcDate(O) {
    if (0 <= O.y && O.y < 100) {
      var _ = new Date(Date.UTC(-1, O.m, O.d, O.H, O.M, O.S, O.L));
      return _.setUTCFullYear(O.y), _;
    }
    return new Date(Date.UTC(O.y, O.m, O.d, O.H, O.M, O.S, O.L));
  }
  function newDate(O, _, H) {
    return { y: O, m: _, d: H, H: 0, M: 0, S: 0, L: 0 };
  }
  function formatLocale(O) {
    var _ = O.dateTime, H = O.date, V = O.time, q = O.periods, G = O.days, J = O.shortDays, Z = O.months, te = O.shortMonths, ne = formatRe(q), ee = formatLookup(q), ae = formatRe(G), oe = formatLookup(G), se = formatRe(J), ce = formatLookup(J), le = formatRe(Z), ue = formatLookup(Z), de = formatRe(te), fe = formatLookup(te), he = {
      a: ke,
      A: Me,
      b: Re,
      B: De,
      c: null,
      d: formatDayOfMonth,
      e: formatDayOfMonth,
      f: formatMicroseconds,
      g: formatYearISO,
      G: formatFullYearISO,
      H: formatHour24,
      I: formatHour12,
      j: formatDayOfYear,
      L: formatMilliseconds,
      m: formatMonthNumber,
      M: formatMinutes,
      p: Pe,
      q: $e,
      Q: formatUnixTimestamp,
      s: formatUnixTimestampSeconds,
      S: formatSeconds,
      u: formatWeekdayNumberMonday,
      U: formatWeekNumberSunday,
      V: formatWeekNumberISO,
      w: formatWeekdayNumberSunday,
      W: formatWeekNumberMonday,
      x: null,
      X: null,
      y: formatYear,
      Y: formatFullYear,
      Z: formatZone,
      "%": formatLiteralPercent
    }, me = {
      a: qe,
      A: Xe,
      b: Ve,
      B: Ze,
      c: null,
      d: formatUTCDayOfMonth,
      e: formatUTCDayOfMonth,
      f: formatUTCMicroseconds,
      g: formatUTCYearISO,
      G: formatUTCFullYearISO,
      H: formatUTCHour24,
      I: formatUTCHour12,
      j: formatUTCDayOfYear,
      L: formatUTCMilliseconds,
      m: formatUTCMonthNumber,
      M: formatUTCMinutes,
      p: Ge,
      q: ze,
      Q: formatUnixTimestamp,
      s: formatUnixTimestampSeconds,
      S: formatUTCSeconds,
      u: formatUTCWeekdayNumberMonday,
      U: formatUTCWeekNumberSunday,
      V: formatUTCWeekNumberISO,
      w: formatUTCWeekdayNumberSunday,
      W: formatUTCWeekNumberMonday,
      x: null,
      X: null,
      y: formatUTCYear,
      Y: formatUTCFullYear,
      Z: formatUTCZone,
      "%": formatLiteralPercent
    }, pe = {
      a: Ce,
      A: Se,
      b: we,
      B: Ee,
      c: xe,
      d: parseDayOfMonth,
      e: parseDayOfMonth,
      f: parseMicroseconds,
      g: parseYear,
      G: parseFullYear,
      H: parseHour24,
      I: parseHour24,
      j: parseDayOfYear,
      L: parseMilliseconds,
      m: parseMonthNumber,
      M: parseMinutes,
      p: be,
      q: parseQuarter,
      Q: parseUnixTimestamp,
      s: parseUnixTimestampSeconds,
      S: parseSeconds,
      u: parseWeekdayNumberMonday,
      U: parseWeekNumberSunday,
      V: parseWeekNumberISO,
      w: parseWeekdayNumberSunday,
      W: parseWeekNumberMonday,
      x: Le,
      X: Qe,
      y: parseYear,
      Y: parseFullYear,
      Z: parseZone,
      "%": parseLiteralPercent
    };
    he.x = ye(H, he), he.X = ye(V, he), he.c = ye(_, he), me.x = ye(H, me), me.X = ye(V, me), me.c = ye(_, me);
    function ye(We, at) {
      return function(ct) {
        var et = [], dt = -1, Je = 0, it = We.length, st, gt, bt;
        for (ct instanceof Date || (ct = /* @__PURE__ */ new Date(+ct)); ++dt < it; )
          We.charCodeAt(dt) === 37 && (et.push(We.slice(Je, dt)), (gt = pads[st = We.charAt(++dt)]) != null ? st = We.charAt(++dt) : gt = st === "e" ? " " : "0", (bt = at[st]) && (st = bt(ct, gt)), et.push(st), Je = dt + 1);
        return et.push(We.slice(Je, dt)), et.join("");
      };
    }
    function ge(We, at) {
      return function(ct) {
        var et = newDate(1900, void 0, 1), dt = ve(et, We, ct += "", 0), Je, it;
        if (dt != ct.length) return null;
        if ("Q" in et) return new Date(et.Q);
        if ("s" in et) return new Date(et.s * 1e3 + ("L" in et ? et.L : 0));
        if (at && !("Z" in et) && (et.Z = 0), "p" in et && (et.H = et.H % 12 + et.p * 12), et.m === void 0 && (et.m = "q" in et ? et.q : 0), "V" in et) {
          if (et.V < 1 || et.V > 53) return null;
          "w" in et || (et.w = 1), "Z" in et ? (Je = utcDate(newDate(et.y, 0, 1)), it = Je.getUTCDay(), Je = it > 4 || it === 0 ? utcMonday.ceil(Je) : utcMonday(Je), Je = utcDay.offset(Je, (et.V - 1) * 7), et.y = Je.getUTCFullYear(), et.m = Je.getUTCMonth(), et.d = Je.getUTCDate() + (et.w + 6) % 7) : (Je = localDate(newDate(et.y, 0, 1)), it = Je.getDay(), Je = it > 4 || it === 0 ? timeMonday.ceil(Je) : timeMonday(Je), Je = timeDay.offset(Je, (et.V - 1) * 7), et.y = Je.getFullYear(), et.m = Je.getMonth(), et.d = Je.getDate() + (et.w + 6) % 7);
        } else ("W" in et || "U" in et) && ("w" in et || (et.w = "u" in et ? et.u % 7 : "W" in et ? 1 : 0), it = "Z" in et ? utcDate(newDate(et.y, 0, 1)).getUTCDay() : localDate(newDate(et.y, 0, 1)).getDay(), et.m = 0, et.d = "W" in et ? (et.w + 6) % 7 + et.W * 7 - (it + 5) % 7 : et.w + et.U * 7 - (it + 6) % 7);
        return "Z" in et ? (et.H += et.Z / 100 | 0, et.M += et.Z % 100, utcDate(et)) : localDate(et);
      };
    }
    function ve(We, at, ct, et) {
      for (var dt = 0, Je = at.length, it = ct.length, st, gt; dt < Je; ) {
        if (et >= it) return -1;
        if (st = at.charCodeAt(dt++), st === 37) {
          if (st = at.charAt(dt++), gt = pe[st in pads ? at.charAt(dt++) : st], !gt || (et = gt(We, ct, et)) < 0) return -1;
        } else if (st != ct.charCodeAt(et++))
          return -1;
      }
      return et;
    }
    function be(We, at, ct) {
      var et = ne.exec(at.slice(ct));
      return et ? (We.p = ee.get(et[0].toLowerCase()), ct + et[0].length) : -1;
    }
    function Ce(We, at, ct) {
      var et = se.exec(at.slice(ct));
      return et ? (We.w = ce.get(et[0].toLowerCase()), ct + et[0].length) : -1;
    }
    function Se(We, at, ct) {
      var et = ae.exec(at.slice(ct));
      return et ? (We.w = oe.get(et[0].toLowerCase()), ct + et[0].length) : -1;
    }
    function we(We, at, ct) {
      var et = de.exec(at.slice(ct));
      return et ? (We.m = fe.get(et[0].toLowerCase()), ct + et[0].length) : -1;
    }
    function Ee(We, at, ct) {
      var et = le.exec(at.slice(ct));
      return et ? (We.m = ue.get(et[0].toLowerCase()), ct + et[0].length) : -1;
    }
    function xe(We, at, ct) {
      return ve(We, _, at, ct);
    }
    function Le(We, at, ct) {
      return ve(We, H, at, ct);
    }
    function Qe(We, at, ct) {
      return ve(We, V, at, ct);
    }
    function ke(We) {
      return J[We.getDay()];
    }
    function Me(We) {
      return G[We.getDay()];
    }
    function Re(We) {
      return te[We.getMonth()];
    }
    function De(We) {
      return Z[We.getMonth()];
    }
    function Pe(We) {
      return q[+(We.getHours() >= 12)];
    }
    function $e(We) {
      return 1 + ~~(We.getMonth() / 3);
    }
    function qe(We) {
      return J[We.getUTCDay()];
    }
    function Xe(We) {
      return G[We.getUTCDay()];
    }
    function Ve(We) {
      return te[We.getUTCMonth()];
    }
    function Ze(We) {
      return Z[We.getUTCMonth()];
    }
    function Ge(We) {
      return q[+(We.getUTCHours() >= 12)];
    }
    function ze(We) {
      return 1 + ~~(We.getUTCMonth() / 3);
    }
    return {
      format: function(We) {
        var at = ye(We += "", he);
        return at.toString = function() {
          return We;
        }, at;
      },
      parse: function(We) {
        var at = ge(We += "", !1);
        return at.toString = function() {
          return We;
        }, at;
      },
      utcFormat: function(We) {
        var at = ye(We += "", me);
        return at.toString = function() {
          return We;
        }, at;
      },
      utcParse: function(We) {
        var at = ge(We += "", !0);
        return at.toString = function() {
          return We;
        }, at;
      }
    };
  }
  var pads = { "-": "", _: " ", 0: "0" }, numberRe = /^\s*\d+/, percentRe = /^%/, requoteRe = /[\\^$*+?|[\]().{}]/g;
  function pad(O, _, H) {
    var V = O < 0 ? "-" : "", q = (V ? -O : O) + "", G = q.length;
    return V + (G < H ? new Array(H - G + 1).join(_) + q : q);
  }
  function requote(O) {
    return O.replace(requoteRe, "\\$&");
  }
  function formatRe(O) {
    return new RegExp("^(?:" + O.map(requote).join("|") + ")", "i");
  }
  function formatLookup(O) {
    return new Map(O.map((_, H) => [_.toLowerCase(), H]));
  }
  function parseWeekdayNumberSunday(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 1));
    return V ? (O.w = +V[0], H + V[0].length) : -1;
  }
  function parseWeekdayNumberMonday(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 1));
    return V ? (O.u = +V[0], H + V[0].length) : -1;
  }
  function parseWeekNumberSunday(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.U = +V[0], H + V[0].length) : -1;
  }
  function parseWeekNumberISO(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.V = +V[0], H + V[0].length) : -1;
  }
  function parseWeekNumberMonday(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.W = +V[0], H + V[0].length) : -1;
  }
  function parseFullYear(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 4));
    return V ? (O.y = +V[0], H + V[0].length) : -1;
  }
  function parseYear(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.y = +V[0] + (+V[0] > 68 ? 1900 : 2e3), H + V[0].length) : -1;
  }
  function parseZone(O, _, H) {
    var V = /^(Z)|([+-]\d\d)(?::?(\d\d))?/.exec(_.slice(H, H + 6));
    return V ? (O.Z = V[1] ? 0 : -(V[2] + (V[3] || "00")), H + V[0].length) : -1;
  }
  function parseQuarter(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 1));
    return V ? (O.q = V[0] * 3 - 3, H + V[0].length) : -1;
  }
  function parseMonthNumber(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.m = V[0] - 1, H + V[0].length) : -1;
  }
  function parseDayOfMonth(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.d = +V[0], H + V[0].length) : -1;
  }
  function parseDayOfYear(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 3));
    return V ? (O.m = 0, O.d = +V[0], H + V[0].length) : -1;
  }
  function parseHour24(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.H = +V[0], H + V[0].length) : -1;
  }
  function parseMinutes(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.M = +V[0], H + V[0].length) : -1;
  }
  function parseSeconds(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 2));
    return V ? (O.S = +V[0], H + V[0].length) : -1;
  }
  function parseMilliseconds(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 3));
    return V ? (O.L = +V[0], H + V[0].length) : -1;
  }
  function parseMicroseconds(O, _, H) {
    var V = numberRe.exec(_.slice(H, H + 6));
    return V ? (O.L = Math.floor(V[0] / 1e3), H + V[0].length) : -1;
  }
  function parseLiteralPercent(O, _, H) {
    var V = percentRe.exec(_.slice(H, H + 1));
    return V ? H + V[0].length : -1;
  }
  function parseUnixTimestamp(O, _, H) {
    var V = numberRe.exec(_.slice(H));
    return V ? (O.Q = +V[0], H + V[0].length) : -1;
  }
  function parseUnixTimestampSeconds(O, _, H) {
    var V = numberRe.exec(_.slice(H));
    return V ? (O.s = +V[0], H + V[0].length) : -1;
  }
  function formatDayOfMonth(O, _) {
    return pad(O.getDate(), _, 2);
  }
  function formatHour24(O, _) {
    return pad(O.getHours(), _, 2);
  }
  function formatHour12(O, _) {
    return pad(O.getHours() % 12 || 12, _, 2);
  }
  function formatDayOfYear(O, _) {
    return pad(1 + timeDay.count(timeYear(O), O), _, 3);
  }
  function formatMilliseconds(O, _) {
    return pad(O.getMilliseconds(), _, 3);
  }
  function formatMicroseconds(O, _) {
    return formatMilliseconds(O, _) + "000";
  }
  function formatMonthNumber(O, _) {
    return pad(O.getMonth() + 1, _, 2);
  }
  function formatMinutes(O, _) {
    return pad(O.getMinutes(), _, 2);
  }
  function formatSeconds(O, _) {
    return pad(O.getSeconds(), _, 2);
  }
  function formatWeekdayNumberMonday(O) {
    var _ = O.getDay();
    return _ === 0 ? 7 : _;
  }
  function formatWeekNumberSunday(O, _) {
    return pad(timeSunday.count(timeYear(O) - 1, O), _, 2);
  }
  function dISO(O) {
    var _ = O.getDay();
    return _ >= 4 || _ === 0 ? timeThursday(O) : timeThursday.ceil(O);
  }
  function formatWeekNumberISO(O, _) {
    return O = dISO(O), pad(timeThursday.count(timeYear(O), O) + (timeYear(O).getDay() === 4), _, 2);
  }
  function formatWeekdayNumberSunday(O) {
    return O.getDay();
  }
  function formatWeekNumberMonday(O, _) {
    return pad(timeMonday.count(timeYear(O) - 1, O), _, 2);
  }
  function formatYear(O, _) {
    return pad(O.getFullYear() % 100, _, 2);
  }
  function formatYearISO(O, _) {
    return O = dISO(O), pad(O.getFullYear() % 100, _, 2);
  }
  function formatFullYear(O, _) {
    return pad(O.getFullYear() % 1e4, _, 4);
  }
  function formatFullYearISO(O, _) {
    var H = O.getDay();
    return O = H >= 4 || H === 0 ? timeThursday(O) : timeThursday.ceil(O), pad(O.getFullYear() % 1e4, _, 4);
  }
  function formatZone(O) {
    var _ = O.getTimezoneOffset();
    return (_ > 0 ? "-" : (_ *= -1, "+")) + pad(_ / 60 | 0, "0", 2) + pad(_ % 60, "0", 2);
  }
  function formatUTCDayOfMonth(O, _) {
    return pad(O.getUTCDate(), _, 2);
  }
  function formatUTCHour24(O, _) {
    return pad(O.getUTCHours(), _, 2);
  }
  function formatUTCHour12(O, _) {
    return pad(O.getUTCHours() % 12 || 12, _, 2);
  }
  function formatUTCDayOfYear(O, _) {
    return pad(1 + utcDay.count(utcYear(O), O), _, 3);
  }
  function formatUTCMilliseconds(O, _) {
    return pad(O.getUTCMilliseconds(), _, 3);
  }
  function formatUTCMicroseconds(O, _) {
    return formatUTCMilliseconds(O, _) + "000";
  }
  function formatUTCMonthNumber(O, _) {
    return pad(O.getUTCMonth() + 1, _, 2);
  }
  function formatUTCMinutes(O, _) {
    return pad(O.getUTCMinutes(), _, 2);
  }
  function formatUTCSeconds(O, _) {
    return pad(O.getUTCSeconds(), _, 2);
  }
  function formatUTCWeekdayNumberMonday(O) {
    var _ = O.getUTCDay();
    return _ === 0 ? 7 : _;
  }
  function formatUTCWeekNumberSunday(O, _) {
    return pad(utcSunday.count(utcYear(O) - 1, O), _, 2);
  }
  function UTCdISO(O) {
    var _ = O.getUTCDay();
    return _ >= 4 || _ === 0 ? utcThursday(O) : utcThursday.ceil(O);
  }
  function formatUTCWeekNumberISO(O, _) {
    return O = UTCdISO(O), pad(utcThursday.count(utcYear(O), O) + (utcYear(O).getUTCDay() === 4), _, 2);
  }
  function formatUTCWeekdayNumberSunday(O) {
    return O.getUTCDay();
  }
  function formatUTCWeekNumberMonday(O, _) {
    return pad(utcMonday.count(utcYear(O) - 1, O), _, 2);
  }
  function formatUTCYear(O, _) {
    return pad(O.getUTCFullYear() % 100, _, 2);
  }
  function formatUTCYearISO(O, _) {
    return O = UTCdISO(O), pad(O.getUTCFullYear() % 100, _, 2);
  }
  function formatUTCFullYear(O, _) {
    return pad(O.getUTCFullYear() % 1e4, _, 4);
  }
  function formatUTCFullYearISO(O, _) {
    var H = O.getUTCDay();
    return O = H >= 4 || H === 0 ? utcThursday(O) : utcThursday.ceil(O), pad(O.getUTCFullYear() % 1e4, _, 4);
  }
  function formatUTCZone() {
    return "+0000";
  }
  function formatLiteralPercent() {
    return "%";
  }
  function formatUnixTimestamp(O) {
    return +O;
  }
  function formatUnixTimestampSeconds(O) {
    return Math.floor(+O / 1e3);
  }
  var locale$1, timeFormat$2, timeParse$1, utcFormat$1, utcParse$1;
  defaultLocale$1({
    dateTime: "%x, %X",
    date: "%-m/%-d/%Y",
    time: "%-I:%M:%S %p",
    periods: ["AM", "PM"],
    days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
    shortDays: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
    months: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
    shortMonths: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
  });
  function defaultLocale$1(O) {
    return locale$1 = formatLocale(O), timeFormat$2 = locale$1.format, timeParse$1 = locale$1.parse, utcFormat$1 = locale$1.utcFormat, utcParse$1 = locale$1.utcParse, locale$1;
  }
  function memoize$1(O) {
    const _ = {};
    return (H) => _[H] || (_[H] = O(H));
  }
  function trimZeroes(O, _) {
    return (H) => {
      const V = O(H), q = V.indexOf(_);
      if (q < 0) return V;
      let G = rightmostDigit(V, q);
      const J = G < V.length ? V.slice(G) : "";
      for (; --G > q; ) if (V[G] !== "0") {
        ++G;
        break;
      }
      return V.slice(0, G) + J;
    };
  }
  function rightmostDigit(O, _) {
    let H = O.lastIndexOf("e"), V;
    if (H > 0) return H;
    for (H = O.length; --H > _; )
      if (V = O.charCodeAt(H), V >= 48 && V <= 57) return H + 1;
  }
  function numberLocale(O) {
    const _ = memoize$1(O.format), H = O.formatPrefix;
    return {
      format: _,
      formatPrefix: H,
      formatFloat(V) {
        const q = formatSpecifier(V || ",");
        if (q.precision == null) {
          switch (q.precision = 12, q.type) {
            case "%":
              q.precision -= 2;
              break;
            case "e":
              q.precision -= 1;
              break;
          }
          return trimZeroes(
            _(q),
            // number format
            _(".1f")(1)[1]
            // decimal point character
          );
        } else
          return _(q);
      },
      formatSpan(V, q, G, J) {
        J = formatSpecifier(J ?? ",f");
        const Z = tickStep(V, q, G), te = Math.max(Math.abs(V), Math.abs(q));
        let ne;
        if (J.precision == null)
          switch (J.type) {
            case "s":
              return isNaN(ne = precisionPrefix(Z, te)) || (J.precision = ne), H(J, te);
            case "":
            case "e":
            case "g":
            case "p":
            case "r": {
              isNaN(ne = precisionRound(Z, te)) || (J.precision = ne - (J.type === "e"));
              break;
            }
            case "f":
            case "%": {
              isNaN(ne = precisionFixed(Z)) || (J.precision = ne - (J.type === "%") * 2);
              break;
            }
          }
        return _(J);
      }
    };
  }
  let defaultNumberLocale;
  resetNumberFormatDefaultLocale();
  function resetNumberFormatDefaultLocale() {
    return defaultNumberLocale = numberLocale({
      format: format$3,
      formatPrefix
    });
  }
  function numberFormatLocale(O) {
    return numberLocale(formatLocale$1(O));
  }
  function numberFormatDefaultLocale(O) {
    return arguments.length ? defaultNumberLocale = numberFormatLocale(O) : defaultNumberLocale;
  }
  function timeMultiFormat(O, _, H) {
    H = H || {}, isObject$2(H) || error$1(`Invalid time multi-format specifier: ${H}`);
    const V = _(SECONDS), q = _(MINUTES), G = _(HOURS), J = _(DATE), Z = _(WEEK), te = _(MONTH), ne = _(QUARTER), ee = _(YEAR), ae = O(H[MILLISECONDS] || ".%L"), oe = O(H[SECONDS] || ":%S"), se = O(H[MINUTES] || "%I:%M"), ce = O(H[HOURS] || "%I %p"), le = O(H[DATE] || H[DAY] || "%a %d"), ue = O(H[WEEK] || "%b %d"), de = O(H[MONTH] || "%B"), fe = O(H[QUARTER] || "%B"), he = O(H[YEAR] || "%Y");
    return (me) => (V(me) < me ? ae : q(me) < me ? oe : G(me) < me ? se : J(me) < me ? ce : te(me) < me ? Z(me) < me ? le : ue : ee(me) < me ? ne(me) < me ? de : fe : he)(me);
  }
  function timeLocale(O) {
    const _ = memoize$1(O.format), H = memoize$1(O.utcFormat);
    return {
      timeFormat: (V) => isString$1(V) ? _(V) : timeMultiFormat(_, timeInterval, V),
      utcFormat: (V) => isString$1(V) ? H(V) : timeMultiFormat(H, utcInterval, V),
      timeParse: memoize$1(O.parse),
      utcParse: memoize$1(O.utcParse)
    };
  }
  let defaultTimeLocale;
  resetTimeFormatDefaultLocale();
  function resetTimeFormatDefaultLocale() {
    return defaultTimeLocale = timeLocale({
      format: timeFormat$2,
      parse: timeParse$1,
      utcFormat: utcFormat$1,
      utcParse: utcParse$1
    });
  }
  function timeFormatLocale(O) {
    return timeLocale(formatLocale(O));
  }
  function timeFormatDefaultLocale(O) {
    return arguments.length ? defaultTimeLocale = timeFormatLocale(O) : defaultTimeLocale;
  }
  const createLocale = (O, _) => extend$2({}, O, _);
  function locale(O, _) {
    const H = O ? numberFormatLocale(O) : numberFormatDefaultLocale(), V = _ ? timeFormatLocale(_) : timeFormatDefaultLocale();
    return createLocale(H, V);
  }
  function defaultLocale(O, _) {
    const H = arguments.length;
    return H && H !== 2 && error$1("defaultLocale expects either zero or two arguments."), H ? createLocale(numberFormatDefaultLocale(O), timeFormatDefaultLocale(_)) : createLocale(numberFormatDefaultLocale(), timeFormatDefaultLocale());
  }
  function resetDefaultLocale() {
    return resetNumberFormatDefaultLocale(), resetTimeFormatDefaultLocale(), defaultLocale();
  }
  const protocol_re = /^(data:|([A-Za-z]+:)?\/\/)/, allowed_re = /^(?:(?:(?:f|ht)tps?|mailto|tel|callto|cid|xmpp|file|data):|[^a-z]|[a-z+.\-]+(?:[^a-z+.\-:]|$))/i, whitespace_re = /[\u0000-\u0020\u00A0\u1680\u180E\u2000-\u2029\u205f\u3000]/g, fileProtocol = "file://";
  function loaderFactory(O, _) {
    return (H) => ({
      options: H || {},
      sanitize,
      load: load$1,
      fileAccess: !1,
      file: fileLoader(_),
      http: httpLoader(O)
    });
  }
  async function load$1(O, _) {
    const H = await this.sanitize(O, _), V = H.href;
    return H.localFile ? this.file(V) : this.http(V, _);
  }
  async function sanitize(O, _) {
    _ = extend$2({}, this.options, _);
    const H = this.fileAccess, V = {
      href: null
    };
    let q, G, J;
    const Z = allowed_re.test(O.replace(whitespace_re, ""));
    (O == null || typeof O != "string" || !Z) && error$1("Sanitize failure, invalid URI: " + $$2(O));
    const te = protocol_re.test(O);
    return (J = _.baseURL) && !te && (!O.startsWith("/") && !J.endsWith("/") && (O = "/" + O), O = J + O), G = (q = O.startsWith(fileProtocol)) || _.mode === "file" || _.mode !== "http" && !te && H, q ? O = O.slice(fileProtocol.length) : O.startsWith("//") && (_.defaultProtocol === "file" ? (O = O.slice(2), G = !0) : O = (_.defaultProtocol || "http") + ":" + O), Object.defineProperty(V, "localFile", {
      value: !!G
    }), V.href = O, _.target && (V.target = _.target + ""), _.rel && (V.rel = _.rel + ""), _.context === "image" && _.crossOrigin && (V.crossOrigin = _.crossOrigin + ""), V;
  }
  function fileLoader(O) {
    return O ? (_) => new Promise((H, V) => {
      O.readFile(_, (q, G) => {
        q ? V(q) : H(G);
      });
    }) : fileReject;
  }
  async function fileReject() {
    error$1("No file system access.");
  }
  function httpLoader(O) {
    return O ? async function(_, H) {
      const V = extend$2({}, this.options.http, H), q = H && H.response, G = await O(_, V);
      return G.ok ? isFunction$2(G[q]) ? G[q]() : G.text() : error$1(G.status + "" + G.statusText);
    } : httpReject;
  }
  async function httpReject() {
    error$1("No HTTP fetch method available.");
  }
  const isValid = (O) => O != null && O === O, isBoolean$2 = (O) => O === "true" || O === "false" || O === !0 || O === !1, isDate = (O) => !Number.isNaN(Date.parse(O)), isNumber$1 = (O) => !Number.isNaN(+O) && !(O instanceof Date), isInteger$1 = (O) => isNumber$1(O) && Number.isInteger(+O), typeParsers = {
    boolean: toBoolean,
    integer: toNumber$1,
    number: toNumber$1,
    date: toDate,
    string: toString$1,
    unknown: identity$7
  }, typeTests = [isBoolean$2, isInteger$1, isNumber$1, isDate], typeList = ["boolean", "integer", "number", "date"];
  function inferType(O, _) {
    if (!O || !O.length) return "unknown";
    const H = O.length, V = typeTests.length, q = typeTests.map((G, J) => J + 1);
    for (let G = 0, J = 0, Z, te; G < H; ++G)
      for (te = _ ? O[G][_] : O[G], Z = 0; Z < V; ++Z)
        if (q[Z] && isValid(te) && !typeTests[Z](te) && (q[Z] = 0, ++J, J === typeTests.length))
          return "string";
    return typeList[q.reduce((G, J) => G === 0 ? J : G, 0) - 1];
  }
  function inferTypes(O, _) {
    return _.reduce((H, V) => (H[V] = inferType(O, V), H), {});
  }
  function delimitedFormat(O) {
    const _ = function(H, V) {
      const q = {
        delimiter: O
      };
      return dsv(H, V ? extend$2(V, q) : q);
    };
    return _.responseType = "text", _;
  }
  function dsv(O, _) {
    return _.header && (O = _.header.map($$2).join(_.delimiter) + `
` + O), dsvFormat(_.delimiter).parse(O + "");
  }
  dsv.responseType = "text";
  function isBuffer$2(O) {
    return typeof Buffer == "function" && isFunction$2(Buffer.isBuffer) ? Buffer.isBuffer(O) : !1;
  }
  function json(O, _) {
    const H = _ && _.property ? field$1(_.property) : identity$7;
    return isObject$2(O) && !isBuffer$2(O) ? parseJSON(H(O), _) : H(JSON.parse(O));
  }
  json.responseType = "json";
  function parseJSON(O, _) {
    return !isArray$2(O) && isIterable$1(O) && (O = [...O]), _ && _.copy ? JSON.parse(JSON.stringify(O)) : O;
  }
  const filters = {
    interior: (O, _) => O !== _,
    exterior: (O, _) => O === _
  };
  function topojson(O, _) {
    let H, V, q, G;
    return O = json(O, _), _ && _.feature ? (H = feature, q = _.feature) : _ && _.mesh ? (H = mesh, q = _.mesh, G = filters[_.filter]) : error$1("Missing TopoJSON feature or mesh parameter."), V = (V = O.objects[q]) ? H(O, V, G) : error$1("Invalid TopoJSON object: " + q), V && V.features || [V];
  }
  topojson.responseType = "json";
  const format$2 = {
    dsv,
    csv: delimitedFormat(","),
    tsv: delimitedFormat("	"),
    json,
    topojson
  };
  function formats$1(O, _) {
    return arguments.length > 1 ? (format$2[O] = _, this) : has$1(format$2, O) ? format$2[O] : null;
  }
  function responseType(O) {
    const _ = formats$1(O);
    return _ && _.responseType || "text";
  }
  function read(O, _, H, V) {
    _ = _ || {};
    const q = formats$1(_.type || "json");
    return q || error$1("Unknown data format type: " + _.type), O = q(O, _), _.parse && parse$6(O, _.parse, H, V), has$1(O, "columns") && delete O.columns, O;
  }
  function parse$6(O, _, H, V) {
    if (!O.length) return;
    const q = timeFormatDefaultLocale();
    H = H || q.timeParse, V = V || q.utcParse;
    let G = O.columns || Object.keys(O[0]), J, Z, te, ne, ee, ae;
    _ === "auto" && (_ = inferTypes(O, G)), G = Object.keys(_);
    const oe = G.map((se) => {
      const ce = _[se];
      let le, ue;
      if (ce && (ce.startsWith("date:") || ce.startsWith("utc:")))
        return le = ce.split(/:(.+)?/, 2), ue = le[1], (ue[0] === "'" && ue[ue.length - 1] === "'" || ue[0] === '"' && ue[ue.length - 1] === '"') && (ue = ue.slice(1, -1)), (le[0] === "utc" ? V : H)(ue);
      if (!typeParsers[ce])
        throw Error("Illegal format pattern: " + se + ":" + ce);
      return typeParsers[ce];
    });
    for (te = 0, ee = O.length, ae = G.length; te < ee; ++te)
      for (J = O[te], ne = 0; ne < ae; ++ne)
        Z = G[ne], J[Z] = oe[ne](J[Z]);
  }
  const loader = loaderFactory(
    typeof fetch < "u" && fetch,
    // use built-in fetch API
    null
    // no file system access
  );
  function UniqueList(O) {
    const _ = O || identity$7, H = [], V = {};
    return H.add = (q) => {
      const G = _(q);
      return V[G] || (V[G] = 1, H.push(q)), H;
    }, H.remove = (q) => {
      const G = _(q);
      if (V[G]) {
        V[G] = 0;
        const J = H.indexOf(q);
        J >= 0 && H.splice(J, 1);
      }
      return H;
    }, H;
  }
  async function asyncCallback(O, _) {
    try {
      await _(O);
    } catch (H) {
      O.error(H);
    }
  }
  const TUPLE_ID_KEY = Symbol("vega_id");
  let TUPLE_ID = 1;
  function isTuple(O) {
    return !!(O && tupleid(O));
  }
  function tupleid(O) {
    return O[TUPLE_ID_KEY];
  }
  function setid(O, _) {
    return O[TUPLE_ID_KEY] = _, O;
  }
  function ingest$1(O) {
    const _ = O === Object(O) ? O : {
      data: O
    };
    return tupleid(_) ? _ : setid(_, TUPLE_ID++);
  }
  function derive(O) {
    return rederive(O, ingest$1({}));
  }
  function rederive(O, _) {
    for (const H in O) _[H] = O[H];
    return _;
  }
  function replace$2(O, _) {
    return setid(_, tupleid(O));
  }
  function stableCompare(O, _) {
    return O ? _ ? (H, V) => O(H, V) || tupleid(_(H)) - tupleid(_(V)) : (H, V) => O(H, V) || tupleid(H) - tupleid(V) : null;
  }
  function isChangeSet(O) {
    return O && O.constructor === changeset;
  }
  function changeset() {
    const O = [], _ = [], H = [], V = [], q = [];
    let G = null, J = !1;
    return {
      constructor: changeset,
      insert(Z) {
        const te = array$5(Z), ne = te.length;
        for (let ee = 0; ee < ne; ++ee) O.push(te[ee]);
        return this;
      },
      remove(Z) {
        const te = isFunction$2(Z) ? V : _, ne = array$5(Z), ee = ne.length;
        for (let ae = 0; ae < ee; ++ae) te.push(ne[ae]);
        return this;
      },
      modify(Z, te, ne) {
        const ee = {
          field: te,
          value: constant$5(ne)
        };
        return isFunction$2(Z) ? (ee.filter = Z, q.push(ee)) : (ee.tuple = Z, H.push(ee)), this;
      },
      encode(Z, te) {
        return isFunction$2(Z) ? q.push({
          filter: Z,
          field: te
        }) : H.push({
          tuple: Z,
          field: te
        }), this;
      },
      clean(Z) {
        return G = Z, this;
      },
      reflow() {
        return J = !0, this;
      },
      pulse(Z, te) {
        const ne = {}, ee = {};
        let ae, oe, se, ce, le, ue;
        for (ae = 0, oe = te.length; ae < oe; ++ae)
          ne[tupleid(te[ae])] = 1;
        for (ae = 0, oe = _.length; ae < oe; ++ae)
          le = _[ae], ne[tupleid(le)] = -1;
        for (ae = 0, oe = V.length; ae < oe; ++ae)
          ce = V[ae], te.forEach((fe) => {
            ce(fe) && (ne[tupleid(fe)] = -1);
          });
        for (ae = 0, oe = O.length; ae < oe; ++ae)
          le = O[ae], ue = tupleid(le), ne[ue] ? ne[ue] = 1 : Z.add.push(ingest$1(O[ae]));
        for (ae = 0, oe = te.length; ae < oe; ++ae)
          le = te[ae], ne[tupleid(le)] < 0 && Z.rem.push(le);
        function de(fe, he, me) {
          me ? fe[he] = me(fe) : Z.encode = he, J || (ee[tupleid(fe)] = fe);
        }
        for (ae = 0, oe = H.length; ae < oe; ++ae)
          se = H[ae], le = se.tuple, ce = se.field, ue = ne[tupleid(le)], ue > 0 && (de(le, ce, se.value), Z.modifies(ce));
        for (ae = 0, oe = q.length; ae < oe; ++ae)
          se = q[ae], ce = se.filter, te.forEach((fe) => {
            ce(fe) && ne[tupleid(fe)] > 0 && de(fe, se.field, se.value);
          }), Z.modifies(se.field);
        if (J)
          Z.mod = _.length || V.length ? te.filter((fe) => ne[tupleid(fe)] > 0) : te.slice();
        else
          for (ue in ee) Z.mod.push(ee[ue]);
        return (G || G == null && (_.length || V.length)) && Z.clean(!0), Z;
      }
    };
  }
  const CACHE = "_:mod:_";
  function Parameters() {
    Object.defineProperty(this, CACHE, {
      writable: !0,
      value: {}
    });
  }
  Parameters.prototype = {
    /**
     * Set a parameter value. If the parameter value changes, the parameter
     * will be recorded as modified.
     * @param {string} name - The parameter name.
     * @param {number} index - The index into an array-value parameter. Ignored if
     *   the argument is undefined, null or less than zero.
     * @param {*} value - The parameter value to set.
     * @param {boolean} [force=false] - If true, records the parameter as modified
     *   even if the value is unchanged.
     * @return {Parameters} - This parameter object.
     */
    set(O, _, H, V) {
      const q = this, G = q[O], J = q[CACHE];
      return _ != null && _ >= 0 ? (G[_] !== H || V) && (G[_] = H, J[_ + ":" + O] = -1, J[O] = -1) : (G !== H || V) && (q[O] = H, J[O] = isArray$2(H) ? 1 + H.length : -1), q;
    },
    /**
     * Tests if one or more parameters has been modified. If invoked with no
     * arguments, returns true if any parameter value has changed. If the first
     * argument is array, returns trues if any parameter name in the array has
     * changed. Otherwise, tests if the given name and optional array index has
     * changed.
     * @param {string} name - The parameter name to test.
     * @param {number} [index=undefined] - The parameter array index to test.
     * @return {boolean} - Returns true if a queried parameter was modified.
     */
    modified(O, _) {
      const H = this[CACHE];
      if (arguments.length) {
        if (isArray$2(O)) {
          for (let V = 0; V < O.length; ++V)
            if (H[O[V]]) return !0;
          return !1;
        }
      } else {
        for (const V in H)
          if (H[V]) return !0;
        return !1;
      }
      return _ != null && _ >= 0 ? _ + 1 < H[O] || !!H[_ + ":" + O] : !!H[O];
    },
    /**
     * Clears the modification records. After calling this method,
     * all parameters are considered unmodified.
     */
    clear() {
      return this[CACHE] = {}, this;
    }
  };
  let OP_ID = 0;
  const PULSE = "pulse", NO_PARAMS = new Parameters(), SKIP$1$1 = 1, MODIFIED = 2;
  function Operator$1(O, _, H, V) {
    this.id = ++OP_ID, this.value = O, this.stamp = -1, this.rank = -1, this.qrank = -1, this.flags = 0, _ && (this._update = _), H && this.parameters(H, V);
  }
  function flag(O) {
    return function(_) {
      const H = this.flags;
      return arguments.length === 0 ? !!(H & O) : (this.flags = _ ? H | O : H & ~O, this);
    };
  }
  Operator$1.prototype = {
    /**
     * Returns a list of target operators dependent on this operator.
     * If this list does not exist, it is created and then returned.
     * @return {UniqueList}
     */
    targets() {
      return this._targets || (this._targets = UniqueList(id));
    },
    /**
     * Sets the value of this operator.
     * @param {*} value - the value to set.
     * @return {Number} Returns 1 if the operator value has changed
     *   according to strict equality, returns 0 otherwise.
     */
    set(O) {
      return this.value !== O ? (this.value = O, 1) : 0;
    },
    /**
     * Indicates that operator evaluation should be skipped on the next pulse.
     * This operator will still propagate incoming pulses, but its update function
     * will not be invoked. The skip flag is reset after every pulse, so calling
     * this method will affect processing of the next pulse only.
     */
    skip: flag(SKIP$1$1),
    /**
     * Indicates that this operator's value has been modified on its most recent
     * pulse. Normally modification is checked via strict equality; however, in
     * some cases it is more efficient to update the internal state of an object.
     * In those cases, the modified flag can be used to trigger propagation. Once
     * set, the modification flag persists across pulses until unset. The flag can
     * be used with the last timestamp to test if a modification is recent.
     */
    modified: flag(MODIFIED),
    /**
     * Sets the parameters for this operator. The parameter values are analyzed for
     * operator instances. If found, this operator will be added as a dependency
     * of the parameterizing operator. Operator values are dynamically marshalled
     * from each operator parameter prior to evaluation. If a parameter value is
     * an array, the array will also be searched for Operator instances. However,
     * the search does not recurse into sub-arrays or object properties.
     * @param {object} params - A hash of operator parameters.
     * @param {boolean} [react=true] - A flag indicating if this operator should
     *   automatically update (react) when parameter values change. In other words,
     *   this flag determines if the operator registers itself as a listener on
     *   any upstream operators included in the parameters.
     * @param {boolean} [initonly=false] - A flag indicating if this operator
     *   should calculate an update only upon its initial evaluation, then
     *   deregister dependencies and suppress all future update invocations.
     * @return {Operator[]} - An array of upstream dependencies.
     */
    parameters(O, _, H) {
      _ = _ !== !1;
      const V = this._argval = this._argval || new Parameters(), q = this._argops = this._argops || [], G = [];
      let J, Z, te, ne;
      const ee = (ae, oe, se) => {
        se instanceof Operator$1 ? (se !== this && (_ && se.targets().add(this), G.push(se)), q.push({
          op: se,
          name: ae,
          index: oe
        })) : V.set(ae, oe, se);
      };
      for (J in O)
        if (Z = O[J], J === PULSE)
          array$5(Z).forEach((ae) => {
            ae instanceof Operator$1 ? ae !== this && (ae.targets().add(this), G.push(ae)) : error$1("Pulse parameters must be operator instances.");
          }), this.source = Z;
        else if (isArray$2(Z))
          for (V.set(J, -1, Array(te = Z.length)), ne = 0; ne < te; ++ne) ee(J, ne, Z[ne]);
        else
          ee(J, -1, Z);
      return this.marshall().clear(), H && (q.initonly = !0), G;
    },
    /**
     * Internal method for marshalling parameter values.
     * Visits each operator dependency to pull the latest value.
     * @return {Parameters} A Parameters object to pass to the update function.
     */
    marshall(O) {
      const _ = this._argval || NO_PARAMS, H = this._argops;
      let V, q, G, J;
      if (H) {
        const Z = H.length;
        for (q = 0; q < Z; ++q)
          V = H[q], G = V.op, J = G.modified() && G.stamp === O, _.set(V.name, V.index, G.value, J);
        if (H.initonly) {
          for (q = 0; q < Z; ++q)
            V = H[q], V.op.targets().remove(this);
          this._argops = null, this._update = null;
        }
      }
      return _;
    },
    /**
     * Detach this operator from the dataflow.
     * Unregisters listeners on upstream dependencies.
     */
    detach() {
      const O = this._argops;
      let _, H, V, q;
      if (O)
        for (_ = 0, H = O.length; _ < H; ++_)
          V = O[_], q = V.op, q._targets && q._targets.remove(this);
      this.pulse = null, this.source = null;
    },
    /**
     * Delegate method to perform operator processing.
     * Subclasses can override this method to perform custom processing.
     * By default, it marshalls parameters and calls the update function
     * if that function is defined. If the update function does not
     * change the operator value then StopPropagation is returned.
     * If no update function is defined, this method does nothing.
     * @param {Pulse} pulse - the current dataflow pulse.
     * @return The output pulse or StopPropagation. A falsy return value
     *   (including undefined) will let the input pulse pass through.
     */
    evaluate(O) {
      const _ = this._update;
      if (_) {
        const H = this.marshall(O.stamp), V = _.call(this, H, O);
        if (H.clear(), V !== this.value)
          this.value = V;
        else if (!this.modified())
          return O.StopPropagation;
      }
    },
    /**
     * Run this operator for the current pulse. If this operator has already
     * been run at (or after) the pulse timestamp, returns StopPropagation.
     * Internally, this method calls {@link evaluate} to perform processing.
     * If {@link evaluate} returns a falsy value, the input pulse is returned.
     * This method should NOT be overridden, instead overrride {@link evaluate}.
     * @param {Pulse} pulse - the current dataflow pulse.
     * @return the output pulse for this operator (or StopPropagation)
     */
    run(O) {
      if (O.stamp < this.stamp) return O.StopPropagation;
      let _;
      return this.skip() ? (this.skip(!1), _ = 0) : _ = this.evaluate(O), this.pulse = _ || O;
    }
  };
  function add$5(O, _, H, V) {
    let q = 1, G;
    return O instanceof Operator$1 ? G = O : O && O.prototype instanceof Operator$1 ? G = new O() : isFunction$2(O) ? G = new Operator$1(null, O) : (q = 0, G = new Operator$1(O, _)), this.rank(G), q && (V = H, H = _), H && this.connect(G, G.parameters(H, V)), this.touch(G), G;
  }
  function connect(O, _) {
    const H = O.rank, V = _.length;
    for (let q = 0; q < V; ++q)
      if (H < _[q].rank) {
        this.rerank(O);
        return;
      }
  }
  let STREAM_ID = 0;
  function EventStream(O, _, H) {
    this.id = ++STREAM_ID, this.value = null, H && (this.receive = H), O && (this._filter = O), _ && (this._apply = _);
  }
  function stream(O, _, H) {
    return new EventStream(O, _, H);
  }
  EventStream.prototype = {
    _filter: truthy,
    _apply: identity$7,
    targets() {
      return this._targets || (this._targets = UniqueList(id));
    },
    consume(O) {
      return arguments.length ? (this._consume = !!O, this) : !!this._consume;
    },
    receive(O) {
      if (this._filter(O)) {
        const _ = this.value = this._apply(O), H = this._targets, V = H ? H.length : 0;
        for (let q = 0; q < V; ++q) H[q].receive(_);
        this._consume && (O.preventDefault(), O.stopPropagation());
      }
    },
    filter(O) {
      const _ = stream(O);
      return this.targets().add(_), _;
    },
    apply(O) {
      const _ = stream(null, O);
      return this.targets().add(_), _;
    },
    merge() {
      const O = stream();
      this.targets().add(O);
      for (let _ = 0, H = arguments.length; _ < H; ++_)
        arguments[_].targets().add(O);
      return O;
    },
    throttle(O) {
      let _ = -1;
      return this.filter(() => {
        const H = Date.now();
        return H - _ > O ? (_ = H, 1) : 0;
      });
    },
    debounce(O) {
      const _ = stream();
      return this.targets().add(stream(null, null, debounce$2(O, (H) => {
        const V = H.dataflow;
        _.receive(H), V && V.run && V.run();
      }))), _;
    },
    between(O, _) {
      let H = !1;
      return O.targets().add(stream(null, null, () => H = !0)), _.targets().add(stream(null, null, () => H = !1)), this.filter(() => H);
    },
    detach() {
      this._filter = truthy, this._targets = null;
    }
  };
  function events$2(O, _, H, V) {
    const q = this, G = stream(H, V), J = function(ne) {
      ne.dataflow = q;
      try {
        G.receive(ne);
      } catch (ee) {
        q.error(ee);
      } finally {
        q.run();
      }
    };
    let Z;
    typeof O == "string" && typeof document < "u" ? Z = document.querySelectorAll(O) : Z = array$5(O);
    const te = Z.length;
    for (let ne = 0; ne < te; ++ne)
      Z[ne].addEventListener(_, J);
    return G;
  }
  function parse$5(O, _) {
    const H = this.locale();
    return read(O, _, H.timeParse, H.utcParse);
  }
  function ingest(O, _, H) {
    return _ = this.parse(_, H), this.pulse(O, this.changeset().insert(_));
  }
  async function request(O, _) {
    const H = this;
    let V = 0, q;
    try {
      q = await H.loader().load(O, {
        context: "dataflow",
        response: responseType(_ && _.type)
      });
      try {
        q = H.parse(q, _);
      } catch (G) {
        V = -2, H.warn("Data ingestion failed", O, G);
      }
    } catch (G) {
      V = -1, H.warn("Loading failed", O, G);
    }
    return {
      data: q,
      status: V
    };
  }
  async function preload(O, _, H) {
    const V = this, q = V._pending || loadPending(V);
    q.requests += 1;
    const G = await V.request(_, H);
    return V.pulse(O, V.changeset().remove(truthy).insert(G.data || [])), q.done(), G;
  }
  function loadPending(O) {
    let _;
    const H = new Promise((V) => _ = V);
    return H.requests = 0, H.done = () => {
      --H.requests === 0 && (O._pending = null, _(O));
    }, O._pending = H;
  }
  const SKIP$2 = {
    skip: !0
  };
  function on(O, _, H, V, q) {
    return (O instanceof Operator$1 ? onOperator : onStream)(this, O, _, H, V, q), this;
  }
  function onStream(O, _, H, V, q, G) {
    const J = extend$2({}, G, SKIP$2);
    let Z, te;
    isFunction$2(H) || (H = constant$5(H)), V === void 0 ? Z = (ne) => O.touch(H(ne)) : isFunction$2(V) ? (te = new Operator$1(null, V, q, !1), Z = (ne) => {
      te.evaluate(ne);
      const ee = H(ne), ae = te.value;
      isChangeSet(ae) ? O.pulse(ee, ae, G) : O.update(ee, ae, J);
    }) : Z = (ne) => O.update(H(ne), V, J), _.apply(Z);
  }
  function onOperator(O, _, H, V, q, G) {
    if (V === void 0)
      _.targets().add(H);
    else {
      const J = G || {}, Z = new Operator$1(null, updater(H, V), q, !1);
      Z.modified(J.force), Z.rank = _.rank, _.targets().add(Z), H && (Z.skip(!0), Z.value = H.value, Z.targets().add(H), O.connect(H, [Z]));
    }
  }
  function updater(O, _) {
    return _ = isFunction$2(_) ? _ : constant$5(_), O ? function(H, V) {
      const q = _(H, V);
      return O.skip() || (O.skip(q !== this.value).value = q), q;
    } : _;
  }
  function rank(O) {
    O.rank = ++this._rank;
  }
  function rerank(O) {
    const _ = [O];
    let H, V, q;
    for (; _.length; )
      if (this.rank(H = _.pop()), V = H._targets)
        for (q = V.length; --q >= 0; )
          _.push(H = V[q]), H === O && error$1("Cycle detected in dataflow graph.");
  }
  const StopPropagation = {}, ADD = 1, REM = 2, MOD$1 = 4, ADD_REM = ADD | REM, ADD_MOD = ADD | MOD$1, ALL = ADD | REM | MOD$1, REFLOW = 8, SOURCE = 16, NO_SOURCE = 32, NO_FIELDS = 64;
  function Pulse(O, _, H) {
    this.dataflow = O, this.stamp = _ ?? -1, this.add = [], this.rem = [], this.mod = [], this.fields = null, this.encode = H || null;
  }
  function materialize(O, _) {
    const H = [];
    return visitArray(O, _, (V) => H.push(V)), H;
  }
  function filter$1(O, _) {
    const H = {};
    return O.visit(_, (V) => {
      H[tupleid(V)] = 1;
    }), (V) => H[tupleid(V)] ? null : V;
  }
  function addFilter(O, _) {
    return O ? (H, V) => O(H, V) && _(H, V) : _;
  }
  Pulse.prototype = {
    /**
     * Sentinel value indicating pulse propagation should stop.
     */
    StopPropagation,
    /**
     * Boolean flag indicating ADD (added) tuples.
     */
    ADD,
    /**
     * Boolean flag indicating REM (removed) tuples.
     */
    REM,
    /**
     * Boolean flag indicating MOD (modified) tuples.
     */
    MOD: MOD$1,
    /**
     * Boolean flag indicating ADD (added) and REM (removed) tuples.
     */
    ADD_REM,
    /**
     * Boolean flag indicating ADD (added) and MOD (modified) tuples.
     */
    ADD_MOD,
    /**
     * Boolean flag indicating ADD, REM and MOD tuples.
     */
    ALL,
    /**
     * Boolean flag indicating all tuples in a data source
     * except for the ADD, REM and MOD tuples.
     */
    REFLOW,
    /**
     * Boolean flag indicating a 'pass-through' to a
     * backing data source, ignoring ADD, REM and MOD tuples.
     */
    SOURCE,
    /**
     * Boolean flag indicating that source data should be
     * suppressed when creating a forked pulse.
     */
    NO_SOURCE,
    /**
     * Boolean flag indicating that field modifications should be
     * suppressed when creating a forked pulse.
     */
    NO_FIELDS,
    /**
     * Creates a new pulse based on the values of this pulse.
     * The dataflow, time stamp and field modification values are copied over.
     * By default, new empty ADD, REM and MOD arrays are created.
     * @param {number} flags - Integer of boolean flags indicating which (if any)
     *   tuple arrays should be copied to the new pulse. The supported flag values
     *   are ADD, REM and MOD. Array references are copied directly: new array
     *   instances are not created.
     * @return {Pulse} - The forked pulse instance.
     * @see init
     */
    fork(O) {
      return new Pulse(this.dataflow).init(this, O);
    },
    /**
     * Creates a copy of this pulse with new materialized array
     * instances for the ADD, REM, MOD, and SOURCE arrays.
     * The dataflow, time stamp and field modification values are copied over.
     * @return {Pulse} - The cloned pulse instance.
     * @see init
     */
    clone() {
      const O = this.fork(ALL);
      return O.add = O.add.slice(), O.rem = O.rem.slice(), O.mod = O.mod.slice(), O.source && (O.source = O.source.slice()), O.materialize(ALL | SOURCE);
    },
    /**
     * Returns a pulse that adds all tuples from a backing source. This is
     * useful for cases where operators are added to a dataflow after an
     * upstream data pipeline has already been processed, ensuring that
     * new operators can observe all tuples within a stream.
     * @return {Pulse} - A pulse instance with all source tuples included
     *   in the add array. If the current pulse already has all source
     *   tuples in its add array, it is returned directly. If the current
     *   pulse does not have a backing source, it is returned directly.
     */
    addAll() {
      let O = this;
      return !O.source || O.add === O.rem || !O.rem.length && O.source.length === O.add.length || (O = new Pulse(this.dataflow).init(this), O.add = O.source, O.rem = []), O;
    },
    /**
     * Initialize this pulse based on the values of another pulse. This method
     * is used internally by {@link fork} to initialize a new forked tuple.
     * The dataflow, time stamp and field modification values are copied over.
     * By default, new empty ADD, REM and MOD arrays are created.
     * @param {Pulse} src - The source pulse to copy from.
     * @param {number} flags - Integer of boolean flags indicating which (if any)
     *   tuple arrays should be copied to the new pulse. The supported flag values
     *   are ADD, REM and MOD. Array references are copied directly: new array
     *   instances are not created. By default, source data arrays are copied
     *   to the new pulse. Use the NO_SOURCE flag to enforce a null source.
     * @return {Pulse} - Returns this Pulse instance.
     */
    init(O, _) {
      const H = this;
      return H.stamp = O.stamp, H.encode = O.encode, O.fields && !(_ & NO_FIELDS) && (H.fields = O.fields), _ & ADD ? (H.addF = O.addF, H.add = O.add) : (H.addF = null, H.add = []), _ & REM ? (H.remF = O.remF, H.rem = O.rem) : (H.remF = null, H.rem = []), _ & MOD$1 ? (H.modF = O.modF, H.mod = O.mod) : (H.modF = null, H.mod = []), _ & NO_SOURCE ? (H.srcF = null, H.source = null) : (H.srcF = O.srcF, H.source = O.source, O.cleans && (H.cleans = O.cleans)), H;
    },
    /**
     * Schedules a function to run after pulse propagation completes.
     * @param {function} func - The function to run.
     */
    runAfter(O) {
      this.dataflow.runAfter(O);
    },
    /**
     * Indicates if tuples have been added, removed or modified.
     * @param {number} [flags] - The tuple types (ADD, REM or MOD) to query.
     *   Defaults to ALL, returning true if any tuple type has changed.
     * @return {boolean} - Returns true if one or more queried tuple types have
     *   changed, false otherwise.
     */
    changed(O) {
      const _ = O || ALL;
      return _ & ADD && this.add.length || _ & REM && this.rem.length || _ & MOD$1 && this.mod.length;
    },
    /**
     * Forces a "reflow" of tuple values, such that all tuples in the backing
     * source are added to the MOD set, unless already present in the ADD set.
     * @param {boolean} [fork=false] - If true, returns a forked copy of this
     *   pulse, and invokes reflow on that derived pulse.
     * @return {Pulse} - The reflowed pulse instance.
     */
    reflow(O) {
      if (O) return this.fork(ALL).reflow();
      const _ = this.add.length, H = this.source && this.source.length;
      return H && H !== _ && (this.mod = this.source, _ && this.filter(MOD$1, filter$1(this, ADD))), this;
    },
    /**
     * Get/set metadata to pulse requesting garbage collection
     * to reclaim currently unused resources.
     */
    clean(O) {
      return arguments.length ? (this.cleans = !!O, this) : this.cleans;
    },
    /**
     * Marks one or more data field names as modified to assist dependency
     * tracking and incremental processing by transform operators.
     * @param {string|Array<string>} _ - The field(s) to mark as modified.
     * @return {Pulse} - This pulse instance.
     */
    modifies(O) {
      const _ = this.fields || (this.fields = {});
      return isArray$2(O) ? O.forEach((H) => _[H] = !0) : _[O] = !0, this;
    },
    /**
     * Checks if one or more data fields have been modified during this pulse
     * propagation timestamp.
     * @param {string|Array<string>} _ - The field(s) to check for modified.
     * @param {boolean} nomod - If true, will check the modified flag even if
     *   no mod tuples exist. If false (default), mod tuples must be present.
     * @return {boolean} - Returns true if any of the provided fields has been
     *   marked as modified, false otherwise.
     */
    modified(O, _) {
      const H = this.fields;
      return (_ || this.mod.length) && H ? arguments.length ? isArray$2(O) ? O.some((V) => H[V]) : H[O] : !!H : !1;
    },
    /**
     * Adds a filter function to one more tuple sets. Filters are applied to
     * backing tuple arrays, to determine the actual set of tuples considered
     * added, removed or modified. They can be used to delay materialization of
     * a tuple set in order to avoid expensive array copies. In addition, the
     * filter functions can serve as value transformers: unlike standard predicate
     * function (which return boolean values), Pulse filters should return the
     * actual tuple value to process. If a tuple set is already filtered, the
     * new filter function will be appended into a conjuntive ('and') query.
     * @param {number} flags - Flags indicating the tuple set(s) to filter.
     * @param {function(*):object} filter - Filter function that will be applied
     *   to the tuple set array, and should return a data tuple if the value
     *   should be included in the tuple set, and falsy (or null) otherwise.
     * @return {Pulse} - Returns this pulse instance.
     */
    filter(O, _) {
      const H = this;
      return O & ADD && (H.addF = addFilter(H.addF, _)), O & REM && (H.remF = addFilter(H.remF, _)), O & MOD$1 && (H.modF = addFilter(H.modF, _)), O & SOURCE && (H.srcF = addFilter(H.srcF, _)), H;
    },
    /**
     * Materialize one or more tuple sets in this pulse. If the tuple set(s) have
     * a registered filter function, it will be applied and the tuple set(s) will
     * be replaced with materialized tuple arrays.
     * @param {number} flags - Flags indicating the tuple set(s) to materialize.
     * @return {Pulse} - Returns this pulse instance.
     */
    materialize(O) {
      O = O || ALL;
      const _ = this;
      return O & ADD && _.addF && (_.add = materialize(_.add, _.addF), _.addF = null), O & REM && _.remF && (_.rem = materialize(_.rem, _.remF), _.remF = null), O & MOD$1 && _.modF && (_.mod = materialize(_.mod, _.modF), _.modF = null), O & SOURCE && _.srcF && (_.source = _.source.filter(_.srcF), _.srcF = null), _;
    },
    /**
     * Visit one or more tuple sets in this pulse.
     * @param {number} flags - Flags indicating the tuple set(s) to visit.
     *   Legal values are ADD, REM, MOD and SOURCE (if a backing data source
     *   has been set).
     * @param {function(object):*} - Visitor function invoked per-tuple.
     * @return {Pulse} - Returns this pulse instance.
     */
    visit(O, _) {
      const H = this, V = _;
      if (O & SOURCE)
        return visitArray(H.source, H.srcF, V), H;
      O & ADD && visitArray(H.add, H.addF, V), O & REM && visitArray(H.rem, H.remF, V), O & MOD$1 && visitArray(H.mod, H.modF, V);
      const q = H.source;
      if (O & REFLOW && q) {
        const G = H.add.length + H.mod.length;
        G === q.length || (G ? visitArray(q, filter$1(H, ADD_MOD), V) : visitArray(q, H.srcF, V));
      }
      return H;
    }
  };
  function MultiPulse(O, _, H, V) {
    const q = this;
    let G = 0;
    this.dataflow = O, this.stamp = _, this.fields = null, this.encode = V || null, this.pulses = H;
    for (const J of H)
      if (J.stamp === _) {
        if (J.fields) {
          const Z = q.fields || (q.fields = {});
          for (const te in J.fields)
            Z[te] = 1;
        }
        J.changed(q.ADD) && (G |= q.ADD), J.changed(q.REM) && (G |= q.REM), J.changed(q.MOD) && (G |= q.MOD);
      }
    this.changes = G;
  }
  inherits(MultiPulse, Pulse, {
    /**
     * Creates a new pulse based on the values of this pulse.
     * The dataflow, time stamp and field modification values are copied over.
     * @return {Pulse}
     */
    fork(O) {
      const _ = new Pulse(this.dataflow).init(this, O & this.NO_FIELDS);
      return O !== void 0 && (O & _.ADD && this.visit(_.ADD, (H) => _.add.push(H)), O & _.REM && this.visit(_.REM, (H) => _.rem.push(H)), O & _.MOD && this.visit(_.MOD, (H) => _.mod.push(H))), _;
    },
    changed(O) {
      return this.changes & O;
    },
    modified(O) {
      const _ = this, H = _.fields;
      return H && _.changes & _.MOD ? isArray$2(O) ? O.some((V) => H[V]) : H[O] : 0;
    },
    filter() {
      error$1("MultiPulse does not support filtering.");
    },
    materialize() {
      error$1("MultiPulse does not support materialization.");
    },
    visit(O, _) {
      const H = this, V = H.pulses, q = V.length;
      let G = 0;
      if (O & H.SOURCE)
        for (; G < q; ++G)
          V[G].visit(O, _);
      else
        for (; G < q; ++G)
          V[G].stamp === H.stamp && V[G].visit(O, _);
      return H;
    }
  });
  async function evaluate(O, _, H) {
    const V = this, q = [];
    if (V._pulse) return reentrant(V);
    if (V._pending && await V._pending, _ && await asyncCallback(V, _), !V._touched.length)
      return V.debug("Dataflow invoked, but nothing to do."), V;
    const G = ++V._clock;
    V._pulse = new Pulse(V, G, O), V._touched.forEach((ee) => V._enqueue(ee, !0)), V._touched = UniqueList(id);
    let J = 0, Z, te, ne;
    try {
      for (; V._heap.size() > 0; ) {
        if (Z = V._heap.pop(), Z.rank !== Z.qrank) {
          V._enqueue(Z, !0);
          continue;
        }
        te = Z.run(V._getPulse(Z, O)), te.then ? te = await te : te.async && (q.push(te.async), te = StopPropagation), te !== StopPropagation && Z._targets && Z._targets.forEach((ee) => V._enqueue(ee)), ++J;
      }
    } catch (ee) {
      V._heap.clear(), ne = ee;
    }
    if (V._input = {}, V._pulse = null, V.debug(`Pulse ${G}: ${J} operators`), ne && (V._postrun = [], V.error(ne)), V._postrun.length) {
      const ee = V._postrun.sort((ae, oe) => oe.priority - ae.priority);
      V._postrun = [];
      for (let ae = 0; ae < ee.length; ++ae)
        await asyncCallback(V, ee[ae].callback);
    }
    return H && await asyncCallback(V, H), q.length && Promise.all(q).then((ee) => V.runAsync(null, () => {
      ee.forEach((ae) => {
        try {
          ae(V);
        } catch (oe) {
          V.error(oe);
        }
      });
    })), V;
  }
  async function runAsync(O, _, H) {
    for (; this._running; ) await this._running;
    const V = () => this._running = null;
    return (this._running = this.evaluate(O, _, H)).then(V, V), this._running;
  }
  function run(O, _, H) {
    return this._pulse ? reentrant(this) : (this.evaluate(O, _, H), this);
  }
  function runAfter(O, _, H) {
    if (this._pulse || _)
      this._postrun.push({
        priority: H || 0,
        callback: O
      });
    else
      try {
        O(this);
      } catch (V) {
        this.error(V);
      }
  }
  function reentrant(O) {
    return O.error("Dataflow already running. Use runAsync() to chain invocations."), O;
  }
  function enqueue$1(O, _) {
    const H = O.stamp < this._clock;
    H && (O.stamp = this._clock), (H || _) && (O.qrank = O.rank, this._heap.push(O));
  }
  function getPulse(O, _) {
    const H = O.source, V = this._clock;
    return H && isArray$2(H) ? new MultiPulse(this, V, H.map((q) => q.pulse), _) : this._input[O.id] || singlePulse(this._pulse, H && H.pulse);
  }
  function singlePulse(O, _) {
    return _ && _.stamp === O.stamp ? _ : (O = O.fork(), _ && _ !== StopPropagation && (O.source = _.source), O);
  }
  const NO_OPT = {
    skip: !1,
    force: !1
  };
  function touch(O, _) {
    const H = _ || NO_OPT;
    return this._pulse ? this._enqueue(O) : this._touched.add(O), H.skip && O.skip(!0), this;
  }
  function update$6(O, _, H) {
    const V = H || NO_OPT;
    return (O.set(_) || V.force) && this.touch(O, V), this;
  }
  function pulse(O, _, H) {
    this.touch(O, H || NO_OPT);
    const V = new Pulse(this, this._clock + (this._pulse ? 0 : 1)), q = O.pulse && O.pulse.source || [];
    return V.target = O, this._input[O.id] = _.pulse(V, q), this;
  }
  function Heap(O) {
    let _ = [];
    return {
      clear: () => _ = [],
      size: () => _.length,
      peek: () => _[0],
      push: (H) => (_.push(H), siftdown(_, 0, _.length - 1, O)),
      pop: () => {
        const H = _.pop();
        let V;
        return _.length ? (V = _[0], _[0] = H, siftup(_, 0, O)) : V = H, V;
      }
    };
  }
  function siftdown(O, _, H, V) {
    let q, G;
    const J = O[H];
    for (; H > _; ) {
      if (G = H - 1 >> 1, q = O[G], V(J, q) < 0) {
        O[H] = q, H = G;
        continue;
      }
      break;
    }
    return O[H] = J;
  }
  function siftup(O, _, H) {
    const V = _, q = O.length, G = O[_];
    let J = (_ << 1) + 1, Z;
    for (; J < q; )
      Z = J + 1, Z < q && H(O[J], O[Z]) >= 0 && (J = Z), O[_] = O[J], _ = J, J = (_ << 1) + 1;
    return O[_] = G, siftdown(O, V, _, H);
  }
  function Dataflow() {
    this.logger(logger()), this.logLevel(Error$1), this._clock = 0, this._rank = 0, this._locale = defaultLocale();
    try {
      this._loader = loader();
    } catch {
    }
    this._touched = UniqueList(id), this._input = {}, this._pulse = null, this._heap = Heap((O, _) => O.qrank - _.qrank), this._postrun = [];
  }
  function logMethod(O) {
    return function() {
      return this._log[O].apply(this, arguments);
    };
  }
  Dataflow.prototype = {
    /**
     * The current timestamp of this dataflow. This value reflects the
     * timestamp of the previous dataflow run. The dataflow is initialized
     * with a stamp value of 0. The initial run of the dataflow will have
     * a timestap of 1, and so on. This value will match the
     * {@link Pulse.stamp} property.
     * @return {number} - The current timestamp value.
     */
    stamp() {
      return this._clock;
    },
    /**
     * Gets or sets the loader instance to use for data file loading. A
     * loader object must provide a "load" method for loading files and a
     * "sanitize" method for checking URL/filename validity. Both methods
     * should accept a URI and options hash as arguments, and return a Promise
     * that resolves to the loaded file contents (load) or a hash containing
     * sanitized URI data with the sanitized url assigned to the "href" property
     * (sanitize).
     * @param {object} _ - The loader instance to use.
     * @return {object|Dataflow} - If no arguments are provided, returns
     *   the current loader instance. Otherwise returns this Dataflow instance.
     */
    loader(O) {
      return arguments.length ? (this._loader = O, this) : this._loader;
    },
    /**
     * Gets or sets the locale instance to use for formatting and parsing
     * string values. The locale object should be provided by the
     * vega-format library, and include methods such as format, timeFormat,
     * utcFormat, timeParse, and utcParse.
     * @param {object} _ - The locale instance to use.
     * @return {object|Dataflow} - If no arguments are provided, returns
     *   the current locale instance. Otherwise returns this Dataflow instance.
     */
    locale(O) {
      return arguments.length ? (this._locale = O, this) : this._locale;
    },
    /**
     * Get or set the logger instance used to log messages. If no arguments are
     * provided, returns the current logger instance. Otherwise, sets the logger
     * and return this Dataflow instance. Provided loggers must support the full
     * API of logger objects generated by the vega-util logger method. Note that
     * by default the log level of the new logger will be used; use the logLevel
     * method to adjust the log level as needed.
     */
    logger(O) {
      return arguments.length ? (this._log = O, this) : this._log;
    },
    /**
     * Logs an error message. By default, logged messages are written to console
     * output. The message will only be logged if the current log level is high
     * enough to permit error messages.
     */
    error: logMethod("error"),
    /**
     * Logs a warning message. By default, logged messages are written to console
     * output. The message will only be logged if the current log level is high
     * enough to permit warning messages.
     */
    warn: logMethod("warn"),
    /**
     * Logs a information message. By default, logged messages are written to
     * console output. The message will only be logged if the current log level is
     * high enough to permit information messages.
     */
    info: logMethod("info"),
    /**
     * Logs a debug message. By default, logged messages are written to console
     * output. The message will only be logged if the current log level is high
     * enough to permit debug messages.
     */
    debug: logMethod("debug"),
    /**
     * Get or set the current log level. If an argument is provided, it
     * will be used as the new log level.
     * @param {number} [level] - Should be one of None, Warn, Info
     * @return {number} - The current log level.
     */
    logLevel: logMethod("level"),
    /**
     * Empty entry threshold for garbage cleaning. Map data structures will
     * perform cleaning once the number of empty entries exceeds this value.
     */
    cleanThreshold: 1e4,
    // OPERATOR REGISTRATION
    add: add$5,
    connect,
    rank,
    rerank,
    // OPERATOR UPDATES
    pulse,
    touch,
    update: update$6,
    changeset,
    // DATA LOADING
    ingest,
    parse: parse$5,
    preload,
    request,
    // EVENT HANDLING
    events: events$2,
    on,
    // PULSE PROPAGATION
    evaluate,
    run,
    runAsync,
    runAfter,
    _enqueue: enqueue$1,
    _getPulse: getPulse
  };
  function Transform(O, _) {
    Operator$1.call(this, O, null, _);
  }
  inherits(Transform, Operator$1, {
    /**
     * Overrides {@link Operator.evaluate} for transform operators.
     * Internally, this method calls {@link evaluate} to perform processing.
     * If {@link evaluate} returns a falsy value, the input pulse is returned.
     * This method should NOT be overridden, instead overrride {@link evaluate}.
     * @param {Pulse} pulse - the current dataflow pulse.
     * @return the output pulse for this operator (or StopPropagation)
     */
    run(O) {
      if (O.stamp < this.stamp) return O.StopPropagation;
      let _;
      return this.skip() ? this.skip(!1) : _ = this.evaluate(O), _ = _ || O, _.then ? _ = _.then((H) => this.pulse = H) : _ !== O.StopPropagation && (this.pulse = _), _;
    },
    /**
     * Overrides {@link Operator.evaluate} for transform operators.
     * Marshalls parameter values and then invokes {@link transform}.
     * @param {Pulse} pulse - the current dataflow pulse.
     * @return {Pulse} The output pulse (or StopPropagation). A falsy return
         value (including undefined) will let the input pulse pass through.
    */
    evaluate(O) {
      const _ = this.marshall(O.stamp), H = this.transform(_, O);
      return _.clear(), H;
    },
    /**
     * Process incoming pulses.
     * Subclasses should override this method to implement transforms.
     * @param {Parameters} _ - The operator parameter values.
     * @param {Pulse} pulse - The current dataflow pulse.
     * @return {Pulse} The output pulse (or StopPropagation). A falsy return
     *   value (including undefined) will let the input pulse pass through.
     */
    transform() {
    }
  });
  const transforms = {};
  function definition$1(O) {
    const _ = transform$2(O);
    return _ && _.Definition || null;
  }
  function transform$2(O) {
    return O = O && O.toLowerCase(), has$1(transforms, O) ? transforms[O] : null;
  }
  function* numbers$1(O, _) {
    if (_ == null)
      for (let H of O)
        H != null && H !== "" && (H = +H) >= H && (yield H);
    else {
      let H = -1;
      for (let V of O)
        V = _(V, ++H, O), V != null && V !== "" && (V = +V) >= V && (yield V);
    }
  }
  function quantiles(O, _, H) {
    const V = Float64Array.from(numbers$1(O, H));
    return V.sort(ascending$1), _.map((q) => quantileSorted(V, q));
  }
  function quartiles(O, _) {
    return quantiles(O, [0.25, 0.5, 0.75], _);
  }
  function estimateBandwidth(O, _) {
    const H = O.length, V = deviation(O, _), q = quartiles(O, _), G = (q[2] - q[0]) / 1.34;
    return 1.06 * (Math.min(V, G) || V || Math.abs(q[0]) || 1) * Math.pow(H, -0.2);
  }
  function bin$1(O) {
    const _ = O.maxbins || 20, H = O.base || 10, V = Math.log(H), q = O.divide || [5, 2];
    let G = O.extent[0], J = O.extent[1], Z, te, ne, ee, ae, oe;
    const se = O.span || J - G || Math.abs(G) || 1;
    if (O.step)
      Z = O.step;
    else if (O.steps) {
      for (ee = se / _, ae = 0, oe = O.steps.length; ae < oe && O.steps[ae] < ee; ++ae) ;
      Z = O.steps[Math.max(0, ae - 1)];
    } else {
      for (te = Math.ceil(Math.log(_) / V), ne = O.minstep || 0, Z = Math.max(ne, Math.pow(H, Math.round(Math.log(se) / V) - te)); Math.ceil(se / Z) > _; )
        Z *= H;
      for (ae = 0, oe = q.length; ae < oe; ++ae)
        ee = Z / q[ae], ee >= ne && se / ee <= _ && (Z = ee);
    }
    ee = Math.log(Z);
    const ce = ee >= 0 ? 0 : ~~(-ee / V) + 1, le = Math.pow(H, -ce - 1);
    return (O.nice || O.nice === void 0) && (ee = Math.floor(G / Z + le) * Z, G = G < ee ? ee - Z : ee, J = Math.ceil(J / Z) * Z), {
      start: G,
      stop: J === G ? G + Z : J,
      step: Z
    };
  }
  var random = Math.random;
  function setRandom(O) {
    random = O;
  }
  function bootstrapCI(O, _, H, V) {
    if (!O.length) return [void 0, void 0];
    const q = Float64Array.from(numbers$1(O, V)), G = q.length, J = _;
    let Z, te, ne, ee;
    for (ne = 0, ee = Array(J); ne < J; ++ne) {
      for (Z = 0, te = 0; te < G; ++te)
        Z += q[~~(random() * G)];
      ee[ne] = Z / G;
    }
    return ee.sort(ascending$1), [quantile$1(ee, H / 2), quantile$1(ee, 1 - H / 2)];
  }
  function dotbin(O, _, H, V) {
    V = V || ((oe) => oe);
    const q = O.length, G = new Float64Array(q);
    let J = 0, Z = 1, te = V(O[0]), ne = te, ee = te + _, ae;
    for (; Z < q; ++Z) {
      if (ae = V(O[Z]), ae >= ee) {
        for (ne = (te + ne) / 2; J < Z; ++J) G[J] = ne;
        ee = ae + _, te = ae;
      }
      ne = ae;
    }
    for (ne = (te + ne) / 2; J < Z; ++J) G[J] = ne;
    return H ? smoothing(G, _ + _ / 4) : G;
  }
  function smoothing(O, _) {
    const H = O.length;
    let V = 0, q = 1, G, J;
    for (; O[V] === O[q]; ) ++q;
    for (; q < H; ) {
      for (G = q + 1; O[q] === O[G]; ) ++G;
      if (O[q] - O[q - 1] < _) {
        for (J = q + (V + G - q - q >> 1); J < q; ) O[J++] = O[q];
        for (; J > q; ) O[J--] = O[V];
      }
      V = q, q = G;
    }
    return O;
  }
  function lcg$2(O) {
    return function() {
      return O = (1103515245 * O + 12345) % 2147483647, O / 2147483647;
    };
  }
  function integer(O, _) {
    _ == null && (_ = O, O = 0);
    let H, V, q;
    const G = {
      min(J) {
        return arguments.length ? (H = J || 0, q = V - H, G) : H;
      },
      max(J) {
        return arguments.length ? (V = J || 0, q = V - H, G) : V;
      },
      sample() {
        return H + Math.floor(q * random());
      },
      pdf(J) {
        return J === Math.floor(J) && J >= H && J < V ? 1 / q : 0;
      },
      cdf(J) {
        const Z = Math.floor(J);
        return Z < H ? 0 : Z >= V ? 1 : (Z - H + 1) / q;
      },
      icdf(J) {
        return J >= 0 && J <= 1 ? H - 1 + Math.floor(J * q) : NaN;
      }
    };
    return G.min(O).max(_);
  }
  const SQRT2PI = Math.sqrt(2 * Math.PI), SQRT2 = Math.SQRT2;
  let nextSample = NaN;
  function sampleNormal(O, _) {
    O = O || 0, _ = _ ?? 1;
    let H = 0, V = 0, q, G;
    if (nextSample === nextSample)
      H = nextSample, nextSample = NaN;
    else {
      do
        H = random() * 2 - 1, V = random() * 2 - 1, q = H * H + V * V;
      while (q === 0 || q > 1);
      G = Math.sqrt(-2 * Math.log(q) / q), H *= G, nextSample = V * G;
    }
    return O + H * _;
  }
  function densityNormal(O, _, H) {
    H = H ?? 1;
    const V = (O - (_ || 0)) / H;
    return Math.exp(-0.5 * V * V) / (H * SQRT2PI);
  }
  function cumulativeNormal(O, _, H) {
    _ = _ || 0, H = H ?? 1;
    const V = (O - _) / H, q = Math.abs(V);
    let G;
    if (q > 37)
      G = 0;
    else {
      const J = Math.exp(-q * q / 2);
      let Z;
      q < 7.07106781186547 ? (Z = 0.0352624965998911 * q + 0.700383064443688, Z = Z * q + 6.37396220353165, Z = Z * q + 33.912866078383, Z = Z * q + 112.079291497871, Z = Z * q + 221.213596169931, Z = Z * q + 220.206867912376, G = J * Z, Z = 0.0883883476483184 * q + 1.75566716318264, Z = Z * q + 16.064177579207, Z = Z * q + 86.7807322029461, Z = Z * q + 296.564248779674, Z = Z * q + 637.333633378831, Z = Z * q + 793.826512519948, Z = Z * q + 440.413735824752, G = G / Z) : (Z = q + 0.65, Z = q + 4 / Z, Z = q + 3 / Z, Z = q + 2 / Z, Z = q + 1 / Z, G = J / Z / 2.506628274631);
    }
    return V > 0 ? 1 - G : G;
  }
  function quantileNormal(O, _, H) {
    return O < 0 || O > 1 ? NaN : (_ || 0) + (H ?? 1) * SQRT2 * erfinv(2 * O - 1);
  }
  function erfinv(O) {
    let _ = -Math.log((1 - O) * (1 + O)), H;
    return _ < 6.25 ? (_ -= 3.125, H = -364441206401782e-35, H = -16850591381820166e-35 + H * _, H = 128584807152564e-32 + H * _, H = 11157877678025181e-33 + H * _, H = -1333171662854621e-31 + H * _, H = 20972767875968562e-33 + H * _, H = 6637638134358324e-30 + H * _, H = -4054566272975207e-29 + H * _, H = -8151934197605472e-29 + H * _, H = 26335093153082323e-28 + H * _, H = -12975133253453532e-27 + H * _, H = -5415412054294628e-26 + H * _, H = 10512122733215323e-25 + H * _, H = -4112633980346984e-24 + H * _, H = -29070369957882005e-24 + H * _, H = 42347877827932404e-23 + H * _, H = -13654692000834679e-22 + H * _, H = -13882523362786469e-21 + H * _, H = 18673420803405714e-20 + H * _, H = -740702534166267e-18 + H * _, H = -0.006033670871430149 + H * _, H = 0.24015818242558962 + H * _, H = 1.6536545626831027 + H * _) : _ < 16 ? (_ = Math.sqrt(_) - 3.25, H = 22137376921775787e-25, H = 9075656193888539e-23 + H * _, H = -27517406297064545e-23 + H * _, H = 18239629214389228e-24 + H * _, H = 15027403968909828e-22 + H * _, H = -4013867526981546e-21 + H * _, H = 29234449089955446e-22 + H * _, H = 12475304481671779e-21 + H * _, H = -47318229009055734e-21 + H * _, H = 6828485145957318e-20 + H * _, H = 24031110387097894e-21 + H * _, H = -3550375203628475e-19 + H * _, H = 9532893797373805e-19 + H * _, H = -0.0016882755560235047 + H * _, H = 0.002491442096107851 + H * _, H = -0.003751208507569241 + H * _, H = 0.005370914553590064 + H * _, H = 1.0052589676941592 + H * _, H = 3.0838856104922208 + H * _) : Number.isFinite(_) ? (_ = Math.sqrt(_) - 5, H = -27109920616438573e-27, H = -2555641816996525e-25 + H * _, H = 15076572693500548e-25 + H * _, H = -3789465440126737e-24 + H * _, H = 761570120807834e-23 + H * _, H = -1496002662714924e-23 + H * _, H = 2914795345090108e-23 + H * _, H = -6771199775845234e-23 + H * _, H = 22900482228026655e-23 + H * _, H = -99298272942317e-20 + H * _, H = 4526062597223154e-21 + H * _, H = -1968177810553167e-20 + H * _, H = 7599527703001776e-20 + H * _, H = -21503011930044477e-20 + H * _, H = -13871931833623122e-20 + H * _, H = 1.0103004648645344 + H * _, H = 4.849906401408584 + H * _) : H = 1 / 0, H * O;
  }
  function gaussian(O, _) {
    let H, V;
    const q = {
      mean(G) {
        return arguments.length ? (H = G || 0, q) : H;
      },
      stdev(G) {
        return arguments.length ? (V = G ?? 1, q) : V;
      },
      sample: () => sampleNormal(H, V),
      pdf: (G) => densityNormal(G, H, V),
      cdf: (G) => cumulativeNormal(G, H, V),
      icdf: (G) => quantileNormal(G, H, V)
    };
    return q.mean(O).stdev(_);
  }
  function kde(O, _) {
    const H = gaussian();
    let V = 0;
    const q = {
      data(G) {
        return arguments.length ? (O = G, V = G ? G.length : 0, q.bandwidth(_)) : O;
      },
      bandwidth(G) {
        return arguments.length ? (_ = G, !_ && O && (_ = estimateBandwidth(O)), q) : _;
      },
      sample() {
        return O[~~(random() * V)] + _ * H.sample();
      },
      pdf(G) {
        let J = 0, Z = 0;
        for (; Z < V; ++Z)
          J += H.pdf((G - O[Z]) / _);
        return J / _ / V;
      },
      cdf(G) {
        let J = 0, Z = 0;
        for (; Z < V; ++Z)
          J += H.cdf((G - O[Z]) / _);
        return J / V;
      },
      icdf() {
        throw Error("KDE icdf not supported.");
      }
    };
    return q.data(O);
  }
  function sampleLogNormal(O, _) {
    return O = O || 0, _ = _ ?? 1, Math.exp(O + sampleNormal() * _);
  }
  function densityLogNormal(O, _, H) {
    if (O <= 0) return 0;
    _ = _ || 0, H = H ?? 1;
    const V = (Math.log(O) - _) / H;
    return Math.exp(-0.5 * V * V) / (H * SQRT2PI * O);
  }
  function cumulativeLogNormal(O, _, H) {
    return cumulativeNormal(Math.log(O), _, H);
  }
  function quantileLogNormal(O, _, H) {
    return Math.exp(quantileNormal(O, _, H));
  }
  function lognormal(O, _) {
    let H, V;
    const q = {
      mean(G) {
        return arguments.length ? (H = G || 0, q) : H;
      },
      stdev(G) {
        return arguments.length ? (V = G ?? 1, q) : V;
      },
      sample: () => sampleLogNormal(H, V),
      pdf: (G) => densityLogNormal(G, H, V),
      cdf: (G) => cumulativeLogNormal(G, H, V),
      icdf: (G) => quantileLogNormal(G, H, V)
    };
    return q.mean(O).stdev(_);
  }
  function mixture$1(O, _) {
    let H = 0, V;
    function q(J) {
      const Z = [];
      let te = 0, ne;
      for (ne = 0; ne < H; ++ne)
        te += Z[ne] = J[ne] == null ? 1 : +J[ne];
      for (ne = 0; ne < H; ++ne)
        Z[ne] /= te;
      return Z;
    }
    const G = {
      weights(J) {
        return arguments.length ? (V = q(_ = J || []), G) : _;
      },
      distributions(J) {
        return arguments.length ? (J ? (H = J.length, O = J) : (H = 0, O = []), G.weights(_)) : O;
      },
      sample() {
        const J = random();
        let Z = O[H - 1], te = V[0], ne = 0;
        for (; ne < H - 1; te += V[++ne])
          if (J < te) {
            Z = O[ne];
            break;
          }
        return Z.sample();
      },
      pdf(J) {
        let Z = 0, te = 0;
        for (; te < H; ++te)
          Z += V[te] * O[te].pdf(J);
        return Z;
      },
      cdf(J) {
        let Z = 0, te = 0;
        for (; te < H; ++te)
          Z += V[te] * O[te].cdf(J);
        return Z;
      },
      icdf() {
        throw Error("Mixture icdf not supported.");
      }
    };
    return G.distributions(O).weights(_);
  }
  function sampleUniform(O, _) {
    return _ == null && (_ = O ?? 1, O = 0), O + (_ - O) * random();
  }
  function densityUniform(O, _, H) {
    return H == null && (H = _ ?? 1, _ = 0), O >= _ && O <= H ? 1 / (H - _) : 0;
  }
  function cumulativeUniform(O, _, H) {
    return H == null && (H = _ ?? 1, _ = 0), O < _ ? 0 : O > H ? 1 : (O - _) / (H - _);
  }
  function quantileUniform(O, _, H) {
    return H == null && (H = _ ?? 1, _ = 0), O >= 0 && O <= 1 ? _ + O * (H - _) : NaN;
  }
  function uniform(O, _) {
    let H, V;
    const q = {
      min(G) {
        return arguments.length ? (H = G || 0, q) : H;
      },
      max(G) {
        return arguments.length ? (V = G ?? 1, q) : V;
      },
      sample: () => sampleUniform(H, V),
      pdf: (G) => densityUniform(G, H, V),
      cdf: (G) => cumulativeUniform(G, H, V),
      icdf: (G) => quantileUniform(G, H, V)
    };
    return _ == null && (_ = O ?? 1, O = 0), q.min(O).max(_);
  }
  function constant$4(O, _, H) {
    let V = 0, q = 0;
    for (const G of O) {
      const J = H(G);
      _(G) == null || J == null || isNaN(J) || (V += (J - V) / ++q);
    }
    return {
      coef: [V],
      predict: () => V,
      rSquared: 0
    };
  }
  function ols(O, _, H, V) {
    const q = V - O * O, G = Math.abs(q) < 1e-24 ? 0 : (H - O * _) / q;
    return [_ - G * O, G];
  }
  function points(O, _, H, V) {
    O = O.filter((se) => {
      let ce = _(se), le = H(se);
      return ce != null && (ce = +ce) >= ce && le != null && (le = +le) >= le;
    }), V && O.sort((se, ce) => _(se) - _(ce));
    const q = O.length, G = new Float64Array(q), J = new Float64Array(q);
    let Z = 0, te = 0, ne = 0, ee, ae, oe;
    for (oe of O)
      G[Z] = ee = +_(oe), J[Z] = ae = +H(oe), ++Z, te += (ee - te) / Z, ne += (ae - ne) / Z;
    for (Z = 0; Z < q; ++Z)
      G[Z] -= te, J[Z] -= ne;
    return [G, J, te, ne];
  }
  function visitPoints(O, _, H, V) {
    let q = -1, G, J;
    for (const Z of O)
      G = _(Z), J = H(Z), G != null && (G = +G) >= G && J != null && (J = +J) >= J && V(G, J, ++q);
  }
  function rSquared(O, _, H, V, q) {
    let G = 0, J = 0;
    return visitPoints(O, _, H, (Z, te) => {
      const ne = te - q(Z), ee = te - V;
      G += ne * ne, J += ee * ee;
    }), 1 - G / J;
  }
  function linear$2(O, _, H) {
    let V = 0, q = 0, G = 0, J = 0, Z = 0;
    visitPoints(O, _, H, (ee, ae) => {
      ++Z, V += (ee - V) / Z, q += (ae - q) / Z, G += (ee * ae - G) / Z, J += (ee * ee - J) / Z;
    });
    const te = ols(V, q, G, J), ne = (ee) => te[0] + te[1] * ee;
    return {
      coef: te,
      predict: ne,
      rSquared: rSquared(O, _, H, q, ne)
    };
  }
  function log$3(O, _, H) {
    let V = 0, q = 0, G = 0, J = 0, Z = 0;
    visitPoints(O, _, H, (ee, ae) => {
      ++Z, ee = Math.log(ee), V += (ee - V) / Z, q += (ae - q) / Z, G += (ee * ae - G) / Z, J += (ee * ee - J) / Z;
    });
    const te = ols(V, q, G, J), ne = (ee) => te[0] + te[1] * Math.log(ee);
    return {
      coef: te,
      predict: ne,
      rSquared: rSquared(O, _, H, q, ne)
    };
  }
  function exp$1(O, _, H) {
    const [V, q, G, J] = points(O, _, H);
    let Z = 0, te = 0, ne = 0, ee = 0, ae = 0, oe, se, ce;
    visitPoints(O, _, H, (fe, he) => {
      oe = V[ae++], se = Math.log(he), ce = oe * he, Z += (he * se - Z) / ae, te += (ce - te) / ae, ne += (ce * se - ne) / ae, ee += (oe * ce - ee) / ae;
    });
    const [le, ue] = ols(te / J, Z / J, ne / J, ee / J), de = (fe) => Math.exp(le + ue * (fe - G));
    return {
      coef: [Math.exp(le - ue * G), ue],
      predict: de,
      rSquared: rSquared(O, _, H, J, de)
    };
  }
  function pow$3(O, _, H) {
    let V = 0, q = 0, G = 0, J = 0, Z = 0, te = 0;
    visitPoints(O, _, H, (ae, oe) => {
      const se = Math.log(ae), ce = Math.log(oe);
      ++te, V += (se - V) / te, q += (ce - q) / te, G += (se * ce - G) / te, J += (se * se - J) / te, Z += (oe - Z) / te;
    });
    const ne = ols(V, q, G, J), ee = (ae) => ne[0] * Math.pow(ae, ne[1]);
    return ne[0] = Math.exp(ne[0]), {
      coef: ne,
      predict: ee,
      rSquared: rSquared(O, _, H, Z, ee)
    };
  }
  function quad(O, _, H) {
    const [V, q, G, J] = points(O, _, H), Z = V.length;
    let te = 0, ne = 0, ee = 0, ae = 0, oe = 0, se, ce, le, ue;
    for (se = 0; se < Z; )
      ce = V[se], le = q[se++], ue = ce * ce, te += (ue - te) / se, ne += (ue * ce - ne) / se, ee += (ue * ue - ee) / se, ae += (ce * le - ae) / se, oe += (ue * le - oe) / se;
    const de = ee - te * te, fe = te * de - ne * ne, he = (oe * te - ae * ne) / fe, me = (ae * de - oe * ne) / fe, pe = -he * te, ye = (ge) => (ge = ge - G, he * ge * ge + me * ge + pe + J);
    return {
      coef: [pe - me * G + he * G * G + J, me - 2 * he * G, he],
      predict: ye,
      rSquared: rSquared(O, _, H, J, ye)
    };
  }
  function poly(O, _, H, V) {
    if (V === 0) return constant$4(O, _, H);
    if (V === 1) return linear$2(O, _, H);
    if (V === 2) return quad(O, _, H);
    const [q, G, J, Z] = points(O, _, H), te = q.length, ne = [], ee = [], ae = V + 1;
    let oe, se, ce, le, ue;
    for (oe = 0; oe < ae; ++oe) {
      for (ce = 0, le = 0; ce < te; ++ce)
        le += Math.pow(q[ce], oe) * G[ce];
      for (ne.push(le), ue = new Float64Array(ae), se = 0; se < ae; ++se) {
        for (ce = 0, le = 0; ce < te; ++ce)
          le += Math.pow(q[ce], oe + se);
        ue[se] = le;
      }
      ee.push(ue);
    }
    ee.push(ne);
    const de = gaussianElimination(ee), fe = (he) => {
      he -= J;
      let me = Z + de[0] + de[1] * he + de[2] * he * he;
      for (oe = 3; oe < ae; ++oe) me += de[oe] * Math.pow(he, oe);
      return me;
    };
    return {
      coef: uncenter(ae, de, -J, Z),
      predict: fe,
      rSquared: rSquared(O, _, H, Z, fe)
    };
  }
  function uncenter(O, _, H, V) {
    const q = Array(O);
    let G, J, Z, te;
    for (G = 0; G < O; ++G) q[G] = 0;
    for (G = O - 1; G >= 0; --G)
      for (Z = _[G], te = 1, q[G] += Z, J = 1; J <= G; ++J)
        te *= (G + 1 - J) / J, q[G - J] += Z * Math.pow(H, J) * te;
    return q[0] += V, q;
  }
  function gaussianElimination(O) {
    const _ = O.length - 1, H = [];
    let V, q, G, J, Z;
    for (V = 0; V < _; ++V) {
      for (J = V, q = V + 1; q < _; ++q)
        Math.abs(O[V][q]) > Math.abs(O[V][J]) && (J = q);
      for (G = V; G < _ + 1; ++G)
        Z = O[G][V], O[G][V] = O[G][J], O[G][J] = Z;
      for (q = V + 1; q < _; ++q)
        for (G = _; G >= V; G--)
          O[G][q] -= O[G][V] * O[V][q] / O[V][V];
    }
    for (q = _ - 1; q >= 0; --q) {
      for (Z = 0, G = q + 1; G < _; ++G)
        Z += O[G][q] * H[G];
      H[q] = (O[_][q] - Z) / O[q][q];
    }
    return H;
  }
  const maxiters = 2, epsilon$6 = 1e-12;
  function loess(O, _, H, V) {
    const [q, G, J, Z] = points(O, _, H, !0), te = q.length, ne = Math.max(2, ~~(V * te)), ee = new Float64Array(te), ae = new Float64Array(te), oe = new Float64Array(te).fill(1);
    for (let se = -1; ++se <= maxiters; ) {
      const ce = [0, ne - 1];
      for (let ue = 0; ue < te; ++ue) {
        const de = q[ue], fe = ce[0], he = ce[1], me = de - q[fe] > q[he] - de ? fe : he;
        let pe = 0, ye = 0, ge = 0, ve = 0, be = 0;
        const Ce = 1 / Math.abs(q[me] - de || 1);
        for (let Ee = fe; Ee <= he; ++Ee) {
          const xe = q[Ee], Le = G[Ee], Qe = tricube(Math.abs(de - xe) * Ce) * oe[Ee], ke = xe * Qe;
          pe += Qe, ye += ke, ge += Le * Qe, ve += Le * ke, be += xe * ke;
        }
        const [Se, we] = ols(ye / pe, ge / pe, ve / pe, be / pe);
        ee[ue] = Se + we * de, ae[ue] = Math.abs(G[ue] - ee[ue]), updateInterval(q, ue + 1, ce);
      }
      if (se === maxiters)
        break;
      const le = median(ae);
      if (Math.abs(le) < epsilon$6) break;
      for (let ue = 0, de, fe; ue < te; ++ue)
        de = ae[ue] / (6 * le), oe[ue] = de >= 1 ? epsilon$6 : (fe = 1 - de * de) * fe;
    }
    return output$1(q, ee, J, Z);
  }
  function tricube(O) {
    return (O = 1 - O * O * O) * O * O;
  }
  function updateInterval(O, _, H) {
    const V = O[_];
    let q = H[0], G = H[1] + 1;
    if (!(G >= O.length))
      for (; _ > q && O[G] - V <= V - O[q]; )
        H[0] = ++q, H[1] = G, ++G;
  }
  function output$1(O, _, H, V) {
    const q = O.length, G = [];
    let J = 0, Z = 0, te = [], ne;
    for (; J < q; ++J)
      ne = O[J] + H, te[0] === ne ? te[1] += (_[J] - te[1]) / ++Z : (Z = 0, te[1] += V, te = [ne, _[J]], G.push(te));
    return te[1] += V, G;
  }
  const MIN_RADIANS = 0.5 * Math.PI / 180;
  function sampleCurve(O, _, H, V) {
    H = H || 25, V = Math.max(H, V || 200);
    const q = (le) => [le, O(le)], G = _[0], J = _[1], Z = J - G, te = Z / V, ne = [q(G)], ee = [];
    if (H === V) {
      for (let le = 1; le < V; ++le)
        ne.push(q(G + le / H * Z));
      return ne.push(q(J)), ne;
    } else {
      ee.push(q(J));
      for (let le = H; --le > 0; )
        ee.push(q(G + le / H * Z));
    }
    let ae = ne[0], oe = ee[ee.length - 1];
    const se = 1 / Z, ce = scaleY(ae[1], ee);
    for (; oe; ) {
      const le = q((ae[0] + oe[0]) / 2);
      le[0] - ae[0] >= te && angleDelta(ae, le, oe, se, ce) > MIN_RADIANS ? ee.push(le) : (ae = oe, ne.push(oe), ee.pop()), oe = ee[ee.length - 1];
    }
    return ne;
  }
  function scaleY(O, _) {
    let H = O, V = O;
    const q = _.length;
    for (let G = 0; G < q; ++G) {
      const J = _[G][1];
      J < H && (H = J), J > V && (V = J);
    }
    return 1 / (V - H);
  }
  function angleDelta(O, _, H, V, q) {
    const G = Math.atan2(q * (H[1] - O[1]), V * (H[0] - O[0])), J = Math.atan2(q * (_[1] - O[1]), V * (_[0] - O[0]));
    return Math.abs(G - J);
  }
  function multikey(O) {
    return (_) => {
      const H = O.length;
      let V = 1, q = String(O[0](_));
      for (; V < H; ++V)
        q += "|" + O[V](_);
      return q;
    };
  }
  function groupkey(O) {
    return !O || !O.length ? function() {
      return "";
    } : O.length === 1 ? O[0] : multikey(O);
  }
  function measureName(O, _, H) {
    return H || O + (_ ? "_" + _ : "");
  }
  const noop$4 = () => {
  }, base_op = {
    init: noop$4,
    add: noop$4,
    rem: noop$4,
    idx: 0
  }, AggregateOps = {
    values: {
      init: (O) => O.cell.store = !0,
      value: (O) => O.cell.data.values(),
      idx: -1
    },
    count: {
      value: (O) => O.cell.num
    },
    __count__: {
      value: (O) => O.missing + O.valid
    },
    missing: {
      value: (O) => O.missing
    },
    valid: {
      value: (O) => O.valid
    },
    sum: {
      init: (O) => O.sum = 0,
      value: (O) => O.valid ? O.sum : void 0,
      add: (O, _) => O.sum += +_,
      rem: (O, _) => O.sum -= _
    },
    product: {
      init: (O) => O.product = 1,
      value: (O) => O.valid ? O.product : void 0,
      add: (O, _) => O.product *= _,
      rem: (O, _) => O.product /= _
    },
    mean: {
      init: (O) => O.mean = 0,
      value: (O) => O.valid ? O.mean : void 0,
      add: (O, _) => (O.mean_d = _ - O.mean, O.mean += O.mean_d / O.valid),
      rem: (O, _) => (O.mean_d = _ - O.mean, O.mean -= O.valid ? O.mean_d / O.valid : O.mean)
    },
    average: {
      value: (O) => O.valid ? O.mean : void 0,
      req: ["mean"],
      idx: 1
    },
    variance: {
      init: (O) => O.dev = 0,
      value: (O) => O.valid > 1 ? O.dev / (O.valid - 1) : void 0,
      add: (O, _) => O.dev += O.mean_d * (_ - O.mean),
      rem: (O, _) => O.dev -= O.mean_d * (_ - O.mean),
      req: ["mean"],
      idx: 1
    },
    variancep: {
      value: (O) => O.valid > 1 ? O.dev / O.valid : void 0,
      req: ["variance"],
      idx: 2
    },
    stdev: {
      value: (O) => O.valid > 1 ? Math.sqrt(O.dev / (O.valid - 1)) : void 0,
      req: ["variance"],
      idx: 2
    },
    stdevp: {
      value: (O) => O.valid > 1 ? Math.sqrt(O.dev / O.valid) : void 0,
      req: ["variance"],
      idx: 2
    },
    stderr: {
      value: (O) => O.valid > 1 ? Math.sqrt(O.dev / (O.valid * (O.valid - 1))) : void 0,
      req: ["variance"],
      idx: 2
    },
    distinct: {
      value: (O) => O.cell.data.distinct(O.get),
      req: ["values"],
      idx: 3
    },
    ci0: {
      value: (O) => O.cell.data.ci0(O.get),
      req: ["values"],
      idx: 3
    },
    ci1: {
      value: (O) => O.cell.data.ci1(O.get),
      req: ["values"],
      idx: 3
    },
    median: {
      value: (O) => O.cell.data.q2(O.get),
      req: ["values"],
      idx: 3
    },
    q1: {
      value: (O) => O.cell.data.q1(O.get),
      req: ["values"],
      idx: 3
    },
    q3: {
      value: (O) => O.cell.data.q3(O.get),
      req: ["values"],
      idx: 3
    },
    min: {
      init: (O) => O.min = void 0,
      value: (O) => O.min = Number.isNaN(O.min) ? O.cell.data.min(O.get) : O.min,
      add: (O, _) => {
        (_ < O.min || O.min === void 0) && (O.min = _);
      },
      rem: (O, _) => {
        _ <= O.min && (O.min = NaN);
      },
      req: ["values"],
      idx: 4
    },
    max: {
      init: (O) => O.max = void 0,
      value: (O) => O.max = Number.isNaN(O.max) ? O.cell.data.max(O.get) : O.max,
      add: (O, _) => {
        (_ > O.max || O.max === void 0) && (O.max = _);
      },
      rem: (O, _) => {
        _ >= O.max && (O.max = NaN);
      },
      req: ["values"],
      idx: 4
    },
    argmin: {
      init: (O) => O.argmin = void 0,
      value: (O) => O.argmin || O.cell.data.argmin(O.get),
      add: (O, _, H) => {
        _ < O.min && (O.argmin = H);
      },
      rem: (O, _) => {
        _ <= O.min && (O.argmin = void 0);
      },
      req: ["min", "values"],
      idx: 3
    },
    argmax: {
      init: (O) => O.argmax = void 0,
      value: (O) => O.argmax || O.cell.data.argmax(O.get),
      add: (O, _, H) => {
        _ > O.max && (O.argmax = H);
      },
      rem: (O, _) => {
        _ >= O.max && (O.argmax = void 0);
      },
      req: ["max", "values"],
      idx: 3
    },
    exponential: {
      init: (O, _) => {
        O.exp = 0, O.exp_r = _;
      },
      value: (O) => O.valid ? O.exp * (1 - O.exp_r) / (1 - O.exp_r ** O.valid) : void 0,
      add: (O, _) => O.exp = O.exp_r * O.exp + _,
      rem: (O, _) => O.exp = (O.exp - _ / O.exp_r ** (O.valid - 1)) / O.exp_r
    },
    exponentialb: {
      value: (O) => O.valid ? O.exp * (1 - O.exp_r) : void 0,
      req: ["exponential"],
      idx: 1
    }
  }, ValidAggregateOps = Object.keys(AggregateOps).filter((O) => O !== "__count__");
  function measure(O, _) {
    return (H, V) => extend$2({
      name: O,
      aggregate_param: V,
      out: H || O
    }, base_op, _);
  }
  [...ValidAggregateOps, "__count__"].forEach((O) => {
    AggregateOps[O] = measure(O, AggregateOps[O]);
  });
  function createMeasure(O, _, H) {
    return AggregateOps[O](H, _);
  }
  function compareIndex(O, _) {
    return O.idx - _.idx;
  }
  function resolve(O) {
    const _ = {};
    O.forEach((V) => _[V.name] = V);
    const H = (V) => {
      V.req && V.req.forEach((q) => {
        _[q] || H(_[q] = AggregateOps[q]());
      });
    };
    return O.forEach(H), Object.values(_).sort(compareIndex);
  }
  function init() {
    this.valid = 0, this.missing = 0, this._ops.forEach((O) => O.aggregate_param == null ? O.init(this) : O.init(this, O.aggregate_param));
  }
  function add$4(O, _) {
    if (O == null || O === "") {
      ++this.missing;
      return;
    }
    O === O && (++this.valid, this._ops.forEach((H) => H.add(this, O, _)));
  }
  function rem(O, _) {
    if (O == null || O === "") {
      --this.missing;
      return;
    }
    O === O && (--this.valid, this._ops.forEach((H) => H.rem(this, O, _)));
  }
  function set$5(O) {
    return this._out.forEach((_) => O[_.out] = _.value(this)), O;
  }
  function compileMeasures(O, _) {
    const H = _ || identity$7, V = resolve(O), q = O.slice().sort(compareIndex);
    function G(J) {
      this._ops = V, this._out = q, this.cell = J, this.init();
    }
    return G.prototype.init = init, G.prototype.add = add$4, G.prototype.rem = rem, G.prototype.set = set$5, G.prototype.get = H, G.fields = O.map((J) => J.out), G;
  }
  function TupleStore(O) {
    this._key = O ? field$1(O) : tupleid, this.reset();
  }
  const prototype$1 = TupleStore.prototype;
  prototype$1.reset = function() {
    this._add = [], this._rem = [], this._ext = null, this._get = null, this._q = null;
  };
  prototype$1.add = function(O) {
    this._add.push(O);
  };
  prototype$1.rem = function(O) {
    this._rem.push(O);
  };
  prototype$1.values = function() {
    if (this._get = null, this._rem.length === 0) return this._add;
    const O = this._add, _ = this._rem, H = this._key, V = O.length, q = _.length, G = Array(V - q), J = {};
    let Z, te, ne;
    for (Z = 0; Z < q; ++Z)
      J[H(_[Z])] = 1;
    for (Z = 0, te = 0; Z < V; ++Z)
      J[H(ne = O[Z])] ? J[H(ne)] = 0 : G[te++] = ne;
    return this._rem = [], this._add = G;
  };
  prototype$1.distinct = function(O) {
    const _ = this.values(), H = {};
    let V = _.length, q = 0, G;
    for (; --V >= 0; )
      G = O(_[V]) + "", has$1(H, G) || (H[G] = 1, ++q);
    return q;
  };
  prototype$1.extent = function(O) {
    if (this._get !== O || !this._ext) {
      const _ = this.values(), H = extentIndex(_, O);
      this._ext = [_[H[0]], _[H[1]]], this._get = O;
    }
    return this._ext;
  };
  prototype$1.argmin = function(O) {
    return this.extent(O)[0] || {};
  };
  prototype$1.argmax = function(O) {
    return this.extent(O)[1] || {};
  };
  prototype$1.min = function(O) {
    const _ = this.extent(O)[0];
    return _ != null ? O(_) : void 0;
  };
  prototype$1.max = function(O) {
    const _ = this.extent(O)[1];
    return _ != null ? O(_) : void 0;
  };
  prototype$1.quartile = function(O) {
    return (this._get !== O || !this._q) && (this._q = quartiles(this.values(), O), this._get = O), this._q;
  };
  prototype$1.q1 = function(O) {
    return this.quartile(O)[0];
  };
  prototype$1.q2 = function(O) {
    return this.quartile(O)[1];
  };
  prototype$1.q3 = function(O) {
    return this.quartile(O)[2];
  };
  prototype$1.ci = function(O) {
    return (this._get !== O || !this._ci) && (this._ci = bootstrapCI(this.values(), 1e3, 0.05, O), this._get = O), this._ci;
  };
  prototype$1.ci0 = function(O) {
    return this.ci(O)[0];
  };
  prototype$1.ci1 = function(O) {
    return this.ci(O)[1];
  };
  function Aggregate$1(O) {
    Transform.call(this, null, O), this._adds = [], this._mods = [], this._alen = 0, this._mlen = 0, this._drop = !0, this._cross = !1, this._dims = [], this._dnames = [], this._measures = [], this._countOnly = !1, this._counts = null, this._prev = null, this._inputs = null, this._outputs = null;
  }
  Aggregate$1.Definition = {
    type: "Aggregate",
    metadata: {
      generates: !0,
      changes: !0
    },
    params: [{
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "ops",
      type: "enum",
      array: !0,
      values: ValidAggregateOps
    }, {
      name: "aggregate_params",
      type: "number",
      null: !0,
      array: !0
    }, {
      name: "fields",
      type: "field",
      null: !0,
      array: !0
    }, {
      name: "as",
      type: "string",
      null: !0,
      array: !0
    }, {
      name: "drop",
      type: "boolean",
      default: !0
    }, {
      name: "cross",
      type: "boolean",
      default: !1
    }, {
      name: "key",
      type: "field"
    }]
  };
  inherits(Aggregate$1, Transform, {
    transform(O, _) {
      const H = this, V = _.fork(_.NO_SOURCE | _.NO_FIELDS), q = O.modified();
      return H.stamp = V.stamp, H.value && (q || _.modified(H._inputs, !0)) ? (H._prev = H.value, H.value = q ? H.init(O) : /* @__PURE__ */ Object.create(null), _.visit(_.SOURCE, (G) => H.add(G))) : (H.value = H.value || H.init(O), _.visit(_.REM, (G) => H.rem(G)), _.visit(_.ADD, (G) => H.add(G))), V.modifies(H._outputs), H._drop = O.drop !== !1, O.cross && H._dims.length > 1 && (H._drop = !1, H.cross()), _.clean() && H._drop && V.clean(!0).runAfter(() => this.clean()), H.changes(V);
    },
    cross() {
      const O = this, _ = O.value, H = O._dnames, V = H.map(() => ({})), q = H.length;
      function G(Z) {
        let te, ne, ee, ae;
        for (te in Z)
          for (ee = Z[te].tuple, ne = 0; ne < q; ++ne)
            V[ne][ae = ee[H[ne]]] = ae;
      }
      G(O._prev), G(_);
      function J(Z, te, ne) {
        const ee = H[ne], ae = V[ne++];
        for (const oe in ae) {
          const se = Z ? Z + "|" + oe : oe;
          te[ee] = ae[oe], ne < q ? J(se, te, ne) : _[se] || O.cell(se, te);
        }
      }
      J("", {}, 0);
    },
    init(O) {
      const _ = this._inputs = [], H = this._outputs = [], V = {};
      function q(fe) {
        const he = array$5(accessorFields(fe)), me = he.length;
        let pe = 0, ye;
        for (; pe < me; ++pe)
          V[ye = he[pe]] || (V[ye] = 1, _.push(ye));
      }
      this._dims = array$5(O.groupby), this._dnames = this._dims.map((fe) => {
        const he = accessorName(fe);
        return q(fe), H.push(he), he;
      }), this.cellkey = O.key ? O.key : groupkey(this._dims), this._countOnly = !0, this._counts = [], this._measures = [];
      const G = O.fields || [null], J = O.ops || ["count"], Z = O.aggregate_params || [null], te = O.as || [], ne = G.length, ee = {};
      let ae, oe, se, ce, le, ue, de;
      for (ne !== J.length && error$1("Unmatched number of fields and aggregate ops."), de = 0; de < ne; ++de) {
        if (ae = G[de], oe = J[de], se = Z[de] || null, ae == null && oe !== "count" && error$1("Null aggregate field specified."), le = accessorName(ae), ue = measureName(oe, le, te[de]), H.push(ue), oe === "count") {
          this._counts.push(ue);
          continue;
        }
        ce = ee[le], ce || (q(ae), ce = ee[le] = [], ce.field = ae, this._measures.push(ce)), oe !== "count" && (this._countOnly = !1), ce.push(createMeasure(oe, se, ue));
      }
      return this._measures = this._measures.map((fe) => compileMeasures(fe, fe.field)), /* @__PURE__ */ Object.create(null);
    },
    // -- Cell Management -----
    cellkey: groupkey(),
    cell(O, _) {
      let H = this.value[O];
      return H ? H.num === 0 && this._drop && H.stamp < this.stamp ? (H.stamp = this.stamp, this._adds[this._alen++] = H) : H.stamp < this.stamp && (H.stamp = this.stamp, this._mods[this._mlen++] = H) : (H = this.value[O] = this.newcell(O, _), this._adds[this._alen++] = H), H;
    },
    newcell(O, _) {
      const H = {
        key: O,
        num: 0,
        agg: null,
        tuple: this.newtuple(_, this._prev && this._prev[O]),
        stamp: this.stamp,
        store: !1
      };
      if (!this._countOnly) {
        const V = this._measures, q = V.length;
        H.agg = Array(q);
        for (let G = 0; G < q; ++G)
          H.agg[G] = new V[G](H);
      }
      return H.store && (H.data = new TupleStore()), H;
    },
    newtuple(O, _) {
      const H = this._dnames, V = this._dims, q = V.length, G = {};
      for (let J = 0; J < q; ++J)
        G[H[J]] = V[J](O);
      return _ ? replace$2(_.tuple, G) : ingest$1(G);
    },
    clean() {
      const O = this.value;
      for (const _ in O)
        O[_].num === 0 && delete O[_];
    },
    // -- Process Tuples -----
    add(O) {
      const _ = this.cellkey(O), H = this.cell(_, O);
      if (H.num += 1, this._countOnly) return;
      H.store && H.data.add(O);
      const V = H.agg;
      for (let q = 0, G = V.length; q < G; ++q)
        V[q].add(V[q].get(O), O);
    },
    rem(O) {
      const _ = this.cellkey(O), H = this.cell(_, O);
      if (H.num -= 1, this._countOnly) return;
      H.store && H.data.rem(O);
      const V = H.agg;
      for (let q = 0, G = V.length; q < G; ++q)
        V[q].rem(V[q].get(O), O);
    },
    celltuple(O) {
      const _ = O.tuple, H = this._counts;
      O.store && O.data.values();
      for (let V = 0, q = H.length; V < q; ++V)
        _[H[V]] = O.num;
      if (!this._countOnly) {
        const V = O.agg;
        for (let q = 0, G = V.length; q < G; ++q)
          V[q].set(_);
      }
      return _;
    },
    changes(O) {
      const _ = this._adds, H = this._mods, V = this._prev, q = this._drop, G = O.add, J = O.rem, Z = O.mod;
      let te, ne, ee, ae;
      if (V) for (ne in V)
        te = V[ne], (!q || te.num) && J.push(te.tuple);
      for (ee = 0, ae = this._alen; ee < ae; ++ee)
        G.push(this.celltuple(_[ee])), _[ee] = null;
      for (ee = 0, ae = this._mlen; ee < ae; ++ee)
        te = H[ee], (te.num === 0 && q ? J : Z).push(this.celltuple(te)), H[ee] = null;
      return this._alen = this._mlen = 0, this._prev = null, O;
    }
  });
  const EPSILON$1 = 1e-14;
  function Bin(O) {
    Transform.call(this, null, O);
  }
  Bin.Definition = {
    type: "Bin",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "interval",
      type: "boolean",
      default: !0
    }, {
      name: "anchor",
      type: "number"
    }, {
      name: "maxbins",
      type: "number",
      default: 20
    }, {
      name: "base",
      type: "number",
      default: 10
    }, {
      name: "divide",
      type: "number",
      array: !0,
      default: [5, 2]
    }, {
      name: "extent",
      type: "number",
      array: !0,
      length: 2,
      required: !0
    }, {
      name: "span",
      type: "number"
    }, {
      name: "step",
      type: "number"
    }, {
      name: "steps",
      type: "number",
      array: !0
    }, {
      name: "minstep",
      type: "number",
      default: 0
    }, {
      name: "nice",
      type: "boolean",
      default: !0
    }, {
      name: "name",
      type: "string"
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: ["bin0", "bin1"]
    }]
  };
  inherits(Bin, Transform, {
    transform(O, _) {
      const H = O.interval !== !1, V = this._bins(O), q = V.start, G = V.step, J = O.as || ["bin0", "bin1"], Z = J[0], te = J[1];
      let ne;
      return O.modified() ? (_ = _.reflow(!0), ne = _.SOURCE) : ne = _.modified(accessorFields(O.field)) ? _.ADD_MOD : _.ADD, _.visit(ne, H ? (ee) => {
        const ae = V(ee);
        ee[Z] = ae, ee[te] = ae == null ? null : q + G * (1 + (ae - q) / G);
      } : (ee) => ee[Z] = V(ee)), _.modifies(H ? J : Z);
    },
    _bins(O) {
      if (this.value && !O.modified())
        return this.value;
      const _ = O.field, H = bin$1(O), V = H.step;
      let q = H.start, G = q + Math.ceil((H.stop - q) / V) * V, J, Z;
      (J = O.anchor) != null && (Z = J - (q + V * Math.floor((J - q) / V)), q += Z, G += Z);
      const te = function(ne) {
        let ee = toNumber$1(_(ne));
        return ee == null ? null : ee < q ? -1 / 0 : ee > G ? 1 / 0 : (ee = Math.max(q, Math.min(ee, G - V)), q + V * Math.floor(EPSILON$1 + (ee - q) / V));
      };
      return te.start = q, te.stop = H.stop, te.step = V, this.value = accessor(te, accessorFields(_), O.name || "bin_" + accessorName(_));
    }
  });
  function SortedList(O, _, H) {
    const V = O;
    let q = _ || [], G = H || [], J = {}, Z = 0;
    return {
      add: (te) => G.push(te),
      remove: (te) => J[V(te)] = ++Z,
      size: () => q.length,
      data: (te, ne) => (Z && (q = q.filter((ee) => !J[V(ee)]), J = {}, Z = 0), ne && te && q.sort(te), G.length && (q = te ? merge$3(te, q, G.sort(te)) : q.concat(G), G = []), q)
    };
  }
  function Collect$1(O) {
    Transform.call(this, [], O);
  }
  Collect$1.Definition = {
    type: "Collect",
    metadata: {
      source: !0
    },
    params: [{
      name: "sort",
      type: "compare"
    }]
  };
  inherits(Collect$1, Transform, {
    transform(O, _) {
      const H = _.fork(_.ALL), V = SortedList(tupleid, this.value, H.materialize(H.ADD).add), q = O.sort, G = _.changed() || q && (O.modified("sort") || _.modified(q.fields));
      return H.visit(H.REM, V.remove), this.modified(G), this.value = H.source = V.data(stableCompare(q), G), _.source && _.source.root && (this.value.root = _.source.root), H;
    }
  });
  function Compare$1(O) {
    Operator$1.call(this, null, update$5, O);
  }
  inherits(Compare$1, Operator$1);
  function update$5(O) {
    return this.value && !O.modified() ? this.value : compare$3(O.fields, O.orders);
  }
  function CountPattern(O) {
    Transform.call(this, null, O);
  }
  CountPattern.Definition = {
    type: "CountPattern",
    metadata: {
      generates: !0,
      changes: !0
    },
    params: [{
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "case",
      type: "enum",
      values: ["upper", "lower", "mixed"],
      default: "mixed"
    }, {
      name: "pattern",
      type: "string",
      default: '[\\w"]+'
    }, {
      name: "stopwords",
      type: "string",
      default: ""
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: ["text", "count"]
    }]
  };
  function tokenize(O, _, H) {
    switch (_) {
      case "upper":
        O = O.toUpperCase();
        break;
      case "lower":
        O = O.toLowerCase();
        break;
    }
    return O.match(H);
  }
  inherits(CountPattern, Transform, {
    transform(O, _) {
      const H = (ae) => (oe) => {
        for (var se = tokenize(Z(oe), O.case, G) || [], ce, le = 0, ue = se.length; le < ue; ++le)
          J.test(ce = se[le]) || ae(ce);
      }, V = this._parameterCheck(O, _), q = this._counts, G = this._match, J = this._stop, Z = O.field, te = O.as || ["text", "count"], ne = H((ae) => q[ae] = 1 + (q[ae] || 0)), ee = H((ae) => q[ae] -= 1);
      return V ? _.visit(_.SOURCE, ne) : (_.visit(_.ADD, ne), _.visit(_.REM, ee)), this._finish(_, te);
    },
    _parameterCheck(O, _) {
      let H = !1;
      return (O.modified("stopwords") || !this._stop) && (this._stop = new RegExp("^" + (O.stopwords || "") + "$", "i"), H = !0), (O.modified("pattern") || !this._match) && (this._match = new RegExp(O.pattern || "[\\w']+", "g"), H = !0), (O.modified("field") || _.modified(O.field.fields)) && (H = !0), H && (this._counts = {}), H;
    },
    _finish(O, _) {
      const H = this._counts, V = this._tuples || (this._tuples = {}), q = _[0], G = _[1], J = O.fork(O.NO_SOURCE | O.NO_FIELDS);
      let Z, te, ne;
      for (Z in H)
        te = V[Z], ne = H[Z] || 0, !te && ne ? (V[Z] = te = ingest$1({}), te[q] = Z, te[G] = ne, J.add.push(te)) : ne === 0 ? (te && J.rem.push(te), H[Z] = null, V[Z] = null) : te[G] !== ne && (te[G] = ne, J.mod.push(te));
      return J.modifies(_);
    }
  });
  function Cross(O) {
    Transform.call(this, null, O);
  }
  Cross.Definition = {
    type: "Cross",
    metadata: {
      generates: !0
    },
    params: [{
      name: "filter",
      type: "expr"
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: ["a", "b"]
    }]
  };
  inherits(Cross, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE), V = O.as || ["a", "b"], q = V[0], G = V[1], J = !this.value || _.changed(_.ADD_REM) || O.modified("as") || O.modified("filter");
      let Z = this.value;
      return J ? (Z && (H.rem = Z), Z = _.materialize(_.SOURCE).source, H.add = this.value = cross(Z, q, G, O.filter || truthy)) : H.mod = Z, H.source = this.value, H.modifies(V);
    }
  });
  function cross(O, _, H, V) {
    for (var q = [], G = {}, J = O.length, Z = 0, te, ne; Z < J; ++Z)
      for (G[_] = ne = O[Z], te = 0; te < J; ++te)
        G[H] = O[te], V(G) && (q.push(ingest$1(G)), G = {}, G[_] = ne);
    return q;
  }
  const Distributions = {
    kde,
    mixture: mixture$1,
    normal: gaussian,
    lognormal,
    uniform
  }, DISTRIBUTIONS = "distributions", FUNCTION = "function", FIELD = "field";
  function parse$4(O, _) {
    const H = O[FUNCTION];
    has$1(Distributions, H) || error$1("Unknown distribution function: " + H);
    const V = Distributions[H]();
    for (const q in O)
      q === FIELD ? V.data((O.from || _()).map(O[q])) : q === DISTRIBUTIONS ? V[q](O[q].map((G) => parse$4(G, _))) : typeof V[q] === FUNCTION && V[q](O[q]);
    return V;
  }
  function Density(O) {
    Transform.call(this, null, O);
  }
  const distributions = [{
    key: {
      function: "normal"
    },
    params: [{
      name: "mean",
      type: "number",
      default: 0
    }, {
      name: "stdev",
      type: "number",
      default: 1
    }]
  }, {
    key: {
      function: "lognormal"
    },
    params: [{
      name: "mean",
      type: "number",
      default: 0
    }, {
      name: "stdev",
      type: "number",
      default: 1
    }]
  }, {
    key: {
      function: "uniform"
    },
    params: [{
      name: "min",
      type: "number",
      default: 0
    }, {
      name: "max",
      type: "number",
      default: 1
    }]
  }, {
    key: {
      function: "kde"
    },
    params: [{
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "from",
      type: "data"
    }, {
      name: "bandwidth",
      type: "number",
      default: 0
    }]
  }], mixture = {
    key: {
      function: "mixture"
    },
    params: [{
      name: "distributions",
      type: "param",
      array: !0,
      params: distributions
    }, {
      name: "weights",
      type: "number",
      array: !0
    }]
  };
  Density.Definition = {
    type: "Density",
    metadata: {
      generates: !0
    },
    params: [{
      name: "extent",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "steps",
      type: "number"
    }, {
      name: "minsteps",
      type: "number",
      default: 25
    }, {
      name: "maxsteps",
      type: "number",
      default: 200
    }, {
      name: "method",
      type: "string",
      default: "pdf",
      values: ["pdf", "cdf"]
    }, {
      name: "distribution",
      type: "param",
      params: distributions.concat(mixture)
    }, {
      name: "as",
      type: "string",
      array: !0,
      default: ["value", "density"]
    }]
  };
  inherits(Density, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE | _.NO_FIELDS);
      if (!this.value || _.changed() || O.modified()) {
        const V = parse$4(O.distribution, source$1(_)), q = O.steps || O.minsteps || 25, G = O.steps || O.maxsteps || 200;
        let J = O.method || "pdf";
        J !== "pdf" && J !== "cdf" && error$1("Invalid density method: " + J), !O.extent && !V.data && error$1("Missing density extent parameter."), J = V[J];
        const Z = O.as || ["value", "density"], te = O.extent || extent(V.data()), ne = sampleCurve(J, te, q, G).map((ee) => {
          const ae = {};
          return ae[Z[0]] = ee[0], ae[Z[1]] = ee[1], ingest$1(ae);
        });
        this.value && (H.rem = this.value), this.value = H.add = H.source = ne;
      }
      return H;
    }
  });
  function source$1(O) {
    return () => O.materialize(O.SOURCE).source;
  }
  function fieldNames(O, _) {
    return O ? O.map((H, V) => _[V] || accessorName(H)) : null;
  }
  function partition$1$1(O, _, H) {
    const V = [], q = (ae) => ae(te);
    let G, J, Z, te, ne, ee;
    if (_ == null)
      V.push(O.map(H));
    else
      for (G = {}, J = 0, Z = O.length; J < Z; ++J)
        te = O[J], ne = _.map(q), ee = G[ne], ee || (G[ne] = ee = [], ee.dims = ne, V.push(ee)), ee.push(H(te));
    return V;
  }
  const Output$5 = "bin";
  function DotBin(O) {
    Transform.call(this, null, O);
  }
  DotBin.Definition = {
    type: "DotBin",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "step",
      type: "number"
    }, {
      name: "smooth",
      type: "boolean",
      default: !1
    }, {
      name: "as",
      type: "string",
      default: Output$5
    }]
  };
  const autostep = (O, _) => span(extent(O, _)) / 30;
  inherits(DotBin, Transform, {
    transform(O, _) {
      if (this.value && !(O.modified() || _.changed()))
        return _;
      const H = _.materialize(_.SOURCE).source, V = partition$1$1(_.source, O.groupby, identity$7), q = O.smooth || !1, G = O.field, J = O.step || autostep(H, G), Z = stableCompare((ce, le) => G(ce) - G(le)), te = O.as || Output$5, ne = V.length;
      let ee = 1 / 0, ae = -1 / 0, oe = 0, se;
      for (; oe < ne; ++oe) {
        const ce = V[oe].sort(Z);
        se = -1;
        for (const le of dotbin(ce, J, q, G))
          le < ee && (ee = le), le > ae && (ae = le), ce[++se][te] = le;
      }
      return this.value = {
        start: ee,
        stop: ae,
        step: J
      }, _.reflow(!0).modifies(te);
    }
  });
  function Expression$1(O) {
    Operator$1.call(this, null, update$4, O), this.modified(!0);
  }
  inherits(Expression$1, Operator$1);
  function update$4(O) {
    const _ = O.expr;
    return this.value && !O.modified("expr") ? this.value : accessor((H) => _(H, O), accessorFields(_), accessorName(_));
  }
  function Extent(O) {
    Transform.call(this, [void 0, void 0], O);
  }
  Extent.Definition = {
    type: "Extent",
    metadata: {},
    params: [{
      name: "field",
      type: "field",
      required: !0
    }]
  };
  inherits(Extent, Transform, {
    transform(O, _) {
      const H = this.value, V = O.field, q = _.changed() || _.modified(V.fields) || O.modified("field");
      let G = H[0], J = H[1];
      if ((q || G == null) && (G = 1 / 0, J = -1 / 0), _.visit(q ? _.SOURCE : _.ADD, (Z) => {
        const te = toNumber$1(V(Z));
        te != null && (te < G && (G = te), te > J && (J = te));
      }), !Number.isFinite(G) || !Number.isFinite(J)) {
        let Z = accessorName(V);
        Z && (Z = ` for field "${Z}"`), _.dataflow.warn(`Infinite extent${Z}: [${G}, ${J}]`), G = J = void 0;
      }
      this.value = [G, J];
    }
  });
  function Subflow(O, _) {
    Operator$1.call(this, O), this.parent = _, this.count = 0;
  }
  inherits(Subflow, Operator$1, {
    /**
     * Routes pulses from this subflow to a target transform.
     * @param {Transform} target - A transform that receives the subflow of tuples.
     */
    connect(O) {
      return this.detachSubflow = O.detachSubflow, this.targets().add(O), O.source = this;
    },
    /**
     * Add an 'add' tuple to the subflow pulse.
     * @param {Tuple} t - The tuple being added.
     */
    add(O) {
      this.count += 1, this.value.add.push(O);
    },
    /**
     * Add a 'rem' tuple to the subflow pulse.
     * @param {Tuple} t - The tuple being removed.
     */
    rem(O) {
      this.count -= 1, this.value.rem.push(O);
    },
    /**
     * Add a 'mod' tuple to the subflow pulse.
     * @param {Tuple} t - The tuple being modified.
     */
    mod(O) {
      this.value.mod.push(O);
    },
    /**
     * Re-initialize this operator's pulse value.
     * @param {Pulse} pulse - The pulse to copy from.
     * @see Pulse.init
     */
    init(O) {
      this.value.init(O, O.NO_SOURCE);
    },
    /**
     * Evaluate this operator. This method overrides the
     * default behavior to simply return the contained pulse value.
     * @return {Pulse}
     */
    evaluate() {
      return this.value;
    }
  });
  function Facet$2(O) {
    Transform.call(this, {}, O), this._keys = fastmap();
    const _ = this._targets = [];
    _.active = 0, _.forEach = (H) => {
      for (let V = 0, q = _.active; V < q; ++V)
        H(_[V], V, _);
    };
  }
  inherits(Facet$2, Transform, {
    activate(O) {
      this._targets[this._targets.active++] = O;
    },
    // parent argument provided by PreFacet subclass
    subflow(O, _, H, V) {
      const q = this.value;
      let G = has$1(q, O) && q[O], J, Z;
      return G ? G.value.stamp < H.stamp && (G.init(H), this.activate(G)) : (Z = V || (Z = this._group[O]) && Z.tuple, J = H.dataflow, G = new Subflow(H.fork(H.NO_SOURCE), this), J.add(G).connect(_(J, O, Z)), q[O] = G, this.activate(G)), G;
    },
    clean() {
      const O = this.value;
      let _ = 0;
      for (const H in O)
        if (O[H].count === 0) {
          const V = O[H].detachSubflow;
          V && V(), delete O[H], ++_;
        }
      if (_) {
        const H = this._targets.filter((V) => V && V.count > 0);
        this.initTargets(H);
      }
    },
    initTargets(O) {
      const _ = this._targets, H = _.length, V = O ? O.length : 0;
      let q = 0;
      for (; q < V; ++q)
        _[q] = O[q];
      for (; q < H && _[q] != null; ++q)
        _[q] = null;
      _.active = V;
    },
    transform(O, _) {
      const H = _.dataflow, V = O.key, q = O.subflow, G = this._keys, J = O.modified("key"), Z = (te) => this.subflow(te, q, _);
      return this._group = O.group || {}, this.initTargets(), _.visit(_.REM, (te) => {
        const ne = tupleid(te), ee = G.get(ne);
        ee !== void 0 && (G.delete(ne), Z(ee).rem(te));
      }), _.visit(_.ADD, (te) => {
        const ne = V(te);
        G.set(tupleid(te), ne), Z(ne).add(te);
      }), J || _.modified(V.fields) ? _.visit(_.MOD, (te) => {
        const ne = tupleid(te), ee = G.get(ne), ae = V(te);
        ee === ae ? Z(ae).mod(te) : (G.set(ne, ae), Z(ee).rem(te), Z(ae).add(te));
      }) : _.changed(_.MOD) && _.visit(_.MOD, (te) => {
        Z(G.get(tupleid(te))).mod(te);
      }), J && _.visit(_.REFLOW, (te) => {
        const ne = tupleid(te), ee = G.get(ne), ae = V(te);
        ee !== ae && (G.set(ne, ae), Z(ee).rem(te), Z(ae).add(te));
      }), _.clean() ? H.runAfter(() => {
        this.clean(), G.clean();
      }) : G.empty > H.cleanThreshold && H.runAfter(G.clean), _;
    }
  });
  function Field$3(O) {
    Operator$1.call(this, null, update$3, O);
  }
  inherits(Field$3, Operator$1);
  function update$3(O) {
    return this.value && !O.modified() ? this.value : isArray$2(O.name) ? array$5(O.name).map((_) => field$1(_)) : field$1(O.name, O.as);
  }
  function Filter(O) {
    Transform.call(this, fastmap(), O);
  }
  Filter.Definition = {
    type: "Filter",
    metadata: {
      changes: !0
    },
    params: [{
      name: "expr",
      type: "expr",
      required: !0
    }]
  };
  inherits(Filter, Transform, {
    transform(O, _) {
      const H = _.dataflow, V = this.value, q = _.fork(), G = q.add, J = q.rem, Z = q.mod, te = O.expr;
      let ne = !0;
      _.visit(_.REM, (ae) => {
        const oe = tupleid(ae);
        V.has(oe) ? V.delete(oe) : J.push(ae);
      }), _.visit(_.ADD, (ae) => {
        te(ae, O) ? G.push(ae) : V.set(tupleid(ae), 1);
      });
      function ee(ae) {
        const oe = tupleid(ae), se = te(ae, O), ce = V.get(oe);
        se && ce ? (V.delete(oe), G.push(ae)) : !se && !ce ? (V.set(oe, 1), J.push(ae)) : ne && se && !ce && Z.push(ae);
      }
      return _.visit(_.MOD, ee), O.modified() && (ne = !1, _.visit(_.REFLOW, ee)), V.empty > H.cleanThreshold && H.runAfter(V.clean), q;
    }
  });
  function Flatten(O) {
    Transform.call(this, [], O);
  }
  Flatten.Definition = {
    type: "Flatten",
    metadata: {
      generates: !0
    },
    params: [{
      name: "fields",
      type: "field",
      array: !0,
      required: !0
    }, {
      name: "index",
      type: "string"
    }, {
      name: "as",
      type: "string",
      array: !0
    }]
  };
  inherits(Flatten, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE), V = O.fields, q = fieldNames(V, O.as || []), G = O.index || null, J = q.length;
      return H.rem = this.value, _.visit(_.SOURCE, (Z) => {
        const te = V.map((ce) => ce(Z)), ne = te.reduce((ce, le) => Math.max(ce, le.length), 0);
        let ee = 0, ae, oe, se;
        for (; ee < ne; ++ee) {
          for (oe = derive(Z), ae = 0; ae < J; ++ae)
            oe[q[ae]] = (se = te[ae][ee]) == null ? null : se;
          G && (oe[G] = ee), H.add.push(oe);
        }
      }), this.value = H.source = H.add, G && H.modifies(G), H.modifies(q);
    }
  });
  function Fold(O) {
    Transform.call(this, [], O);
  }
  Fold.Definition = {
    type: "Fold",
    metadata: {
      generates: !0
    },
    params: [{
      name: "fields",
      type: "field",
      array: !0,
      required: !0
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: ["key", "value"]
    }]
  };
  inherits(Fold, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE), V = O.fields, q = V.map(accessorName), G = O.as || ["key", "value"], J = G[0], Z = G[1], te = V.length;
      return H.rem = this.value, _.visit(_.SOURCE, (ne) => {
        for (let ee = 0, ae; ee < te; ++ee)
          ae = derive(ne), ae[J] = q[ee], ae[Z] = V[ee](ne), H.add.push(ae);
      }), this.value = H.source = H.add, H.modifies(G);
    }
  });
  function Formula(O) {
    Transform.call(this, null, O);
  }
  Formula.Definition = {
    type: "Formula",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "expr",
      type: "expr",
      required: !0
    }, {
      name: "as",
      type: "string",
      required: !0
    }, {
      name: "initonly",
      type: "boolean"
    }]
  };
  inherits(Formula, Transform, {
    transform(O, _) {
      const H = O.expr, V = O.as, q = O.modified(), G = O.initonly ? _.ADD : q ? _.SOURCE : _.modified(H.fields) || _.modified(V) ? _.ADD_MOD : _.ADD;
      return q && (_ = _.materialize().reflow(!0)), O.initonly || _.modifies(V), _.visit(G, (J) => J[V] = H(J, O));
    }
  });
  function Generate(O) {
    Transform.call(this, [], O);
  }
  inherits(Generate, Transform, {
    transform(O, _) {
      const H = _.fork(_.ALL), V = O.generator;
      let q = this.value, G = O.size - q.length, J, Z, te;
      if (G > 0) {
        for (J = []; --G >= 0; )
          J.push(te = ingest$1(V(O))), q.push(te);
        H.add = H.add.length ? H.materialize(H.ADD).add.concat(J) : J;
      } else
        Z = q.slice(0, -G), H.rem = H.rem.length ? H.materialize(H.REM).rem.concat(Z) : Z, q = q.slice(-G);
      return H.source = this.value = q, H;
    }
  });
  const Methods$1 = {
    value: "value",
    median,
    mean,
    min: min$2,
    max: max$2
  }, Empty$1 = [];
  function Impute(O) {
    Transform.call(this, [], O);
  }
  Impute.Definition = {
    type: "Impute",
    metadata: {
      changes: !0
    },
    params: [{
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "key",
      type: "field",
      required: !0
    }, {
      name: "keyvals",
      array: !0
    }, {
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "method",
      type: "enum",
      default: "value",
      values: ["value", "mean", "median", "max", "min"]
    }, {
      name: "value",
      default: 0
    }]
  };
  function getValue$2(O) {
    var _ = O.method || Methods$1.value, H;
    if (Methods$1[_] == null)
      error$1("Unrecognized imputation method: " + _);
    else return _ === Methods$1.value ? (H = O.value !== void 0 ? O.value : 0, () => H) : Methods$1[_];
  }
  function getField$1(O) {
    const _ = O.field;
    return (H) => H ? _(H) : NaN;
  }
  inherits(Impute, Transform, {
    transform(O, _) {
      var H = _.fork(_.ALL), V = getValue$2(O), q = getField$1(O), G = accessorName(O.field), J = accessorName(O.key), Z = (O.groupby || []).map(accessorName), te = partition$4(_.source, O.groupby, O.key, O.keyvals), ne = [], ee = this.value, ae = te.domain.length, oe, se, ce, le, ue, de, fe, he, me, pe;
      for (ue = 0, he = te.length; ue < he; ++ue)
        for (oe = te[ue], ce = oe.values, se = NaN, fe = 0; fe < ae; ++fe)
          if (oe[fe] == null) {
            for (le = te.domain[fe], pe = {
              _impute: !0
            }, de = 0, me = ce.length; de < me; ++de) pe[Z[de]] = ce[de];
            pe[J] = le, pe[G] = Number.isNaN(se) ? se = V(oe, q) : se, ne.push(ingest$1(pe));
          }
      return ne.length && (H.add = H.materialize(H.ADD).add.concat(ne)), ee.length && (H.rem = H.materialize(H.REM).rem.concat(ee)), this.value = ne, H;
    }
  });
  function partition$4(O, _, H, V) {
    var q = (de) => de(ue), G = [], J = V ? V.slice() : [], Z = {}, te = {}, ne, ee, ae, oe, se, ce, le, ue;
    for (J.forEach((de, fe) => Z[de] = fe + 1), oe = 0, le = O.length; oe < le; ++oe)
      ue = O[oe], ce = H(ue), se = Z[ce] || (Z[ce] = J.push(ce)), ee = (ne = _ ? _.map(q) : Empty$1) + "", (ae = te[ee]) || (ae = te[ee] = [], G.push(ae), ae.values = ne), ae[se - 1] = ue;
    return G.domain = J, G;
  }
  function JoinAggregate(O) {
    Aggregate$1.call(this, O);
  }
  JoinAggregate.Definition = {
    type: "JoinAggregate",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "fields",
      type: "field",
      null: !0,
      array: !0
    }, {
      name: "ops",
      type: "enum",
      array: !0,
      values: ValidAggregateOps
    }, {
      name: "as",
      type: "string",
      null: !0,
      array: !0
    }, {
      name: "key",
      type: "field"
    }]
  };
  inherits(JoinAggregate, Aggregate$1, {
    transform(O, _) {
      const H = this, V = O.modified();
      let q;
      return H.value && (V || _.modified(H._inputs, !0)) ? (q = H.value = V ? H.init(O) : {}, _.visit(_.SOURCE, (G) => H.add(G))) : (q = H.value = H.value || this.init(O), _.visit(_.REM, (G) => H.rem(G)), _.visit(_.ADD, (G) => H.add(G))), H.changes(), _.visit(_.SOURCE, (G) => {
        extend$2(G, q[H.cellkey(G)].tuple);
      }), _.reflow(V).modifies(this._outputs);
    },
    changes() {
      const O = this._adds, _ = this._mods;
      let H, V;
      for (H = 0, V = this._alen; H < V; ++H)
        this.celltuple(O[H]), O[H] = null;
      for (H = 0, V = this._mlen; H < V; ++H)
        this.celltuple(_[H]), _[H] = null;
      this._alen = this._mlen = 0;
    }
  });
  function KDE(O) {
    Transform.call(this, null, O);
  }
  KDE.Definition = {
    type: "KDE",
    metadata: {
      generates: !0
    },
    params: [{
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "cumulative",
      type: "boolean",
      default: !1
    }, {
      name: "counts",
      type: "boolean",
      default: !1
    }, {
      name: "bandwidth",
      type: "number",
      default: 0
    }, {
      name: "extent",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "resolve",
      type: "enum",
      values: ["shared", "independent"],
      default: "independent"
    }, {
      name: "steps",
      type: "number"
    }, {
      name: "minsteps",
      type: "number",
      default: 25
    }, {
      name: "maxsteps",
      type: "number",
      default: 200
    }, {
      name: "as",
      type: "string",
      array: !0,
      default: ["value", "density"]
    }]
  };
  inherits(KDE, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE | _.NO_FIELDS);
      if (!this.value || _.changed() || O.modified()) {
        const V = _.materialize(_.SOURCE).source, q = partition$1$1(V, O.groupby, O.field), G = (O.groupby || []).map(accessorName), J = O.bandwidth, Z = O.cumulative ? "cdf" : "pdf", te = O.as || ["value", "density"], ne = [];
        let ee = O.extent, ae = O.steps || O.minsteps || 25, oe = O.steps || O.maxsteps || 200;
        Z !== "pdf" && Z !== "cdf" && error$1("Invalid density method: " + Z), O.resolve === "shared" && (ee || (ee = extent(V, O.field)), ae = oe = O.steps || oe), q.forEach((se) => {
          const ce = kde(se, J)[Z], le = O.counts ? se.length : 1, ue = ee || extent(se);
          sampleCurve(ce, ue, ae, oe).forEach((de) => {
            const fe = {};
            for (let he = 0; he < G.length; ++he)
              fe[G[he]] = se.dims[he];
            fe[te[0]] = de[0], fe[te[1]] = de[1] * le, ne.push(ingest$1(fe));
          });
        }), this.value && (H.rem = this.value), this.value = H.add = H.source = ne;
      }
      return H;
    }
  });
  function Key$1(O) {
    Operator$1.call(this, null, update$2, O);
  }
  inherits(Key$1, Operator$1);
  function update$2(O) {
    return this.value && !O.modified() ? this.value : key(O.fields, O.flat);
  }
  function Load$1(O) {
    Transform.call(this, [], O), this._pending = null;
  }
  inherits(Load$1, Transform, {
    transform(O, _) {
      const H = _.dataflow;
      return this._pending ? output(this, _, this._pending) : stop(O) ? _.StopPropagation : O.values ? output(this, _, H.parse(O.values, O.format)) : O.async ? {
        async: H.request(O.url, O.format).then((q) => (this._pending = array$5(q.data), (G) => G.touch(this)))
      } : H.request(O.url, O.format).then((V) => output(this, _, array$5(V.data)));
    }
  });
  function stop(O) {
    return O.modified("async") && !(O.modified("values") || O.modified("url") || O.modified("format"));
  }
  function output(O, _, H) {
    H.forEach(ingest$1);
    const V = _.fork(_.NO_FIELDS & _.NO_SOURCE);
    return V.rem = O.value, O.value = V.source = V.add = H, O._pending = null, V.rem.length && V.clean(!0), V;
  }
  function Lookup(O) {
    Transform.call(this, {}, O);
  }
  Lookup.Definition = {
    type: "Lookup",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "index",
      type: "index",
      params: [{
        name: "from",
        type: "data",
        required: !0
      }, {
        name: "key",
        type: "field",
        required: !0
      }]
    }, {
      name: "values",
      type: "field",
      array: !0
    }, {
      name: "fields",
      type: "field",
      array: !0,
      required: !0
    }, {
      name: "as",
      type: "string",
      array: !0
    }, {
      name: "default",
      default: null
    }]
  };
  inherits(Lookup, Transform, {
    transform(O, _) {
      const H = O.fields, V = O.index, q = O.values, G = O.default == null ? null : O.default, J = O.modified(), Z = H.length;
      let te = J ? _.SOURCE : _.ADD, ne = _, ee = O.as, ae, oe, se;
      return q ? (oe = q.length, Z > 1 && !ee && error$1('Multi-field lookup requires explicit "as" parameter.'), ee && ee.length !== Z * oe && error$1('The "as" parameter has too few output field names.'), ee = ee || q.map(accessorName), ae = function(ce) {
        for (var le = 0, ue = 0, de, fe; le < Z; ++le)
          if (fe = V.get(H[le](ce)), fe == null) for (de = 0; de < oe; ++de, ++ue) ce[ee[ue]] = G;
          else for (de = 0; de < oe; ++de, ++ue) ce[ee[ue]] = q[de](fe);
      }) : (ee || error$1("Missing output field names."), ae = function(ce) {
        for (var le = 0, ue; le < Z; ++le)
          ue = V.get(H[le](ce)), ce[ee[le]] = ue ?? G;
      }), J ? ne = _.reflow(!0) : (se = H.some((ce) => _.modified(ce.fields)), te |= se ? _.MOD : 0), _.visit(te, ae), ne.modifies(ee);
    }
  });
  function MultiExtent$1(O) {
    Operator$1.call(this, null, update$1, O);
  }
  inherits(MultiExtent$1, Operator$1);
  function update$1(O) {
    if (this.value && !O.modified())
      return this.value;
    const _ = O.extents, H = _.length;
    let V = 1 / 0, q = -1 / 0, G, J;
    for (G = 0; G < H; ++G)
      J = _[G], J[0] < V && (V = J[0]), J[1] > q && (q = J[1]);
    return [V, q];
  }
  function MultiValues$1(O) {
    Operator$1.call(this, null, update, O);
  }
  inherits(MultiValues$1, Operator$1);
  function update(O) {
    return this.value && !O.modified() ? this.value : O.values.reduce((_, H) => _.concat(H), []);
  }
  function Params$2(O) {
    Transform.call(this, null, O);
  }
  inherits(Params$2, Transform, {
    transform(O, _) {
      return this.modified(O.modified()), this.value = O, _.fork(_.NO_SOURCE | _.NO_FIELDS);
    }
  });
  function Pivot(O) {
    Aggregate$1.call(this, O);
  }
  Pivot.Definition = {
    type: "Pivot",
    metadata: {
      generates: !0,
      changes: !0
    },
    params: [{
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "value",
      type: "field",
      required: !0
    }, {
      name: "op",
      type: "enum",
      values: ValidAggregateOps,
      default: "sum"
    }, {
      name: "limit",
      type: "number",
      default: 0
    }, {
      name: "key",
      type: "field"
    }]
  };
  inherits(Pivot, Aggregate$1, {
    _transform: Aggregate$1.prototype.transform,
    transform(O, _) {
      return this._transform(aggregateParams(O, _), _);
    }
  });
  function aggregateParams(O, _) {
    const H = O.field, V = O.value, q = (O.op === "count" ? "__count__" : O.op) || "sum", G = accessorFields(H).concat(accessorFields(V)), J = pivotKeys(H, O.limit || 0, _);
    return _.changed() && O.set("__pivot__", null, null, !0), {
      key: O.key,
      groupby: O.groupby,
      ops: J.map(() => q),
      fields: J.map((Z) => get$5(Z, H, V, G)),
      as: J.map((Z) => Z + ""),
      modified: O.modified.bind(O)
    };
  }
  function get$5(O, _, H, V) {
    return accessor((q) => _(q) === O ? H(q) : NaN, V, O + "");
  }
  function pivotKeys(O, _, H) {
    const V = {}, q = [];
    return H.visit(H.SOURCE, (G) => {
      const J = O(G);
      V[J] || (V[J] = 1, q.push(J));
    }), q.sort(ascending$2), _ ? q.slice(0, _) : q;
  }
  function PreFacet$1(O) {
    Facet$2.call(this, O);
  }
  inherits(PreFacet$1, Facet$2, {
    transform(O, _) {
      const H = O.subflow, V = O.field, q = (G) => this.subflow(tupleid(G), H, _, G);
      return (O.modified("field") || V && _.modified(accessorFields(V))) && error$1("PreFacet does not support field modification."), this.initTargets(), V ? (_.visit(_.MOD, (G) => {
        const J = q(G);
        V(G).forEach((Z) => J.mod(Z));
      }), _.visit(_.ADD, (G) => {
        const J = q(G);
        V(G).forEach((Z) => J.add(ingest$1(Z)));
      }), _.visit(_.REM, (G) => {
        const J = q(G);
        V(G).forEach((Z) => J.rem(Z));
      })) : (_.visit(_.MOD, (G) => q(G).mod(G)), _.visit(_.ADD, (G) => q(G).add(G)), _.visit(_.REM, (G) => q(G).rem(G))), _.clean() && _.runAfter(() => this.clean()), _;
    }
  });
  function Project(O) {
    Transform.call(this, null, O);
  }
  Project.Definition = {
    type: "Project",
    metadata: {
      generates: !0,
      changes: !0
    },
    params: [{
      name: "fields",
      type: "field",
      array: !0
    }, {
      name: "as",
      type: "string",
      null: !0,
      array: !0
    }]
  };
  inherits(Project, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE), V = O.fields, q = fieldNames(O.fields, O.as || []), G = V ? (Z, te) => project$1(Z, te, V, q) : rederive;
      let J;
      return this.value ? J = this.value : (_ = _.addAll(), J = this.value = {}), _.visit(_.REM, (Z) => {
        const te = tupleid(Z);
        H.rem.push(J[te]), J[te] = null;
      }), _.visit(_.ADD, (Z) => {
        const te = G(Z, ingest$1({}));
        J[tupleid(Z)] = te, H.add.push(te);
      }), _.visit(_.MOD, (Z) => {
        H.mod.push(G(Z, J[tupleid(Z)]));
      }), H;
    }
  });
  function project$1(O, _, H, V) {
    for (let q = 0, G = H.length; q < G; ++q)
      _[V[q]] = H[q](O);
    return _;
  }
  function Proxy$2(O) {
    Transform.call(this, null, O);
  }
  inherits(Proxy$2, Transform, {
    transform(O, _) {
      return this.value = O.value, O.modified("value") ? _.fork(_.NO_SOURCE | _.NO_FIELDS) : _.StopPropagation;
    }
  });
  function Quantile$1(O) {
    Transform.call(this, null, O);
  }
  Quantile$1.Definition = {
    type: "Quantile",
    metadata: {
      generates: !0,
      changes: !0
    },
    params: [{
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "probs",
      type: "number",
      array: !0
    }, {
      name: "step",
      type: "number",
      default: 0.01
    }, {
      name: "as",
      type: "string",
      array: !0,
      default: ["prob", "value"]
    }]
  };
  const EPSILON$2 = 1e-14;
  inherits(Quantile$1, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE | _.NO_FIELDS), V = O.as || ["prob", "value"];
      if (this.value && !O.modified() && !_.changed())
        return H.source = this.value, H;
      const q = _.materialize(_.SOURCE).source, G = partition$1$1(q, O.groupby, O.field), J = (O.groupby || []).map(accessorName), Z = [], te = O.step || 0.01, ne = O.probs || range$7(te / 2, 1 - EPSILON$2, te), ee = ne.length;
      return G.forEach((ae) => {
        const oe = quantiles(ae, ne);
        for (let se = 0; se < ee; ++se) {
          const ce = {};
          for (let le = 0; le < J.length; ++le)
            ce[J[le]] = ae.dims[le];
          ce[V[0]] = ne[se], ce[V[1]] = oe[se], Z.push(ingest$1(ce));
        }
      }), this.value && (H.rem = this.value), this.value = H.add = H.source = Z, H;
    }
  });
  function Relay$1(O) {
    Transform.call(this, null, O);
  }
  inherits(Relay$1, Transform, {
    transform(O, _) {
      let H, V;
      return this.value ? V = this.value : (H = _ = _.addAll(), V = this.value = {}), O.derive && (H = _.fork(_.NO_SOURCE), _.visit(_.REM, (q) => {
        const G = tupleid(q);
        H.rem.push(V[G]), V[G] = null;
      }), _.visit(_.ADD, (q) => {
        const G = derive(q);
        V[tupleid(q)] = G, H.add.push(G);
      }), _.visit(_.MOD, (q) => {
        const G = V[tupleid(q)];
        for (const J in q)
          G[J] = q[J], H.modifies(J);
        H.mod.push(G);
      })), H;
    }
  });
  function Sample(O) {
    Transform.call(this, [], O), this.count = 0;
  }
  Sample.Definition = {
    type: "Sample",
    metadata: {},
    params: [{
      name: "size",
      type: "number",
      default: 1e3
    }]
  };
  inherits(Sample, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE), V = O.modified("size"), q = O.size, G = this.value.reduce((ee, ae) => (ee[tupleid(ae)] = 1, ee), {});
      let J = this.value, Z = this.count, te = 0;
      function ne(ee) {
        let ae, oe;
        J.length < q ? J.push(ee) : (oe = ~~((Z + 1) * random()), oe < J.length && oe >= te && (ae = J[oe], G[tupleid(ae)] && H.rem.push(ae), J[oe] = ee)), ++Z;
      }
      if (_.rem.length && (_.visit(_.REM, (ee) => {
        const ae = tupleid(ee);
        G[ae] && (G[ae] = -1, H.rem.push(ee)), --Z;
      }), J = J.filter((ee) => G[tupleid(ee)] !== -1)), (_.rem.length || V) && J.length < q && _.source && (te = Z = J.length, _.visit(_.SOURCE, (ee) => {
        G[tupleid(ee)] || ne(ee);
      }), te = -1), V && J.length > q) {
        const ee = J.length - q;
        for (let ae = 0; ae < ee; ++ae)
          G[tupleid(J[ae])] = -1, H.rem.push(J[ae]);
        J = J.slice(ee);
      }
      return _.mod.length && _.visit(_.MOD, (ee) => {
        G[tupleid(ee)] && H.mod.push(ee);
      }), _.add.length && _.visit(_.ADD, ne), (_.add.length || te < 0) && (H.add = J.filter((ee) => !G[tupleid(ee)])), this.count = Z, this.value = H.source = J, H;
    }
  });
  function Sequence(O) {
    Transform.call(this, null, O);
  }
  Sequence.Definition = {
    type: "Sequence",
    metadata: {
      generates: !0,
      changes: !0
    },
    params: [{
      name: "start",
      type: "number",
      required: !0
    }, {
      name: "stop",
      type: "number",
      required: !0
    }, {
      name: "step",
      type: "number",
      default: 1
    }, {
      name: "as",
      type: "string",
      default: "data"
    }]
  };
  inherits(Sequence, Transform, {
    transform(O, _) {
      if (this.value && !O.modified()) return;
      const H = _.materialize().fork(_.MOD), V = O.as || "data";
      return H.rem = this.value ? _.rem.concat(this.value) : _.rem, this.value = range$7(O.start, O.stop, O.step || 1).map((q) => {
        const G = {};
        return G[V] = q, ingest$1(G);
      }), H.add = _.add.concat(this.value), H;
    }
  });
  function Sieve$1(O) {
    Transform.call(this, null, O), this.modified(!0);
  }
  inherits(Sieve$1, Transform, {
    transform(O, _) {
      return this.value = _.source, _.changed() ? _.fork(_.NO_SOURCE | _.NO_FIELDS) : _.StopPropagation;
    }
  });
  function TimeUnit$1(O) {
    Transform.call(this, null, O);
  }
  const OUTPUT = ["unit0", "unit1"];
  TimeUnit$1.Definition = {
    type: "TimeUnit",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field",
      required: !0
    }, {
      name: "interval",
      type: "boolean",
      default: !0
    }, {
      name: "units",
      type: "enum",
      values: TIME_UNITS,
      array: !0
    }, {
      name: "step",
      type: "number",
      default: 1
    }, {
      name: "maxbins",
      type: "number",
      default: 40
    }, {
      name: "extent",
      type: "date",
      array: !0
    }, {
      name: "timezone",
      type: "enum",
      default: "local",
      values: ["local", "utc"]
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: OUTPUT
    }]
  };
  inherits(TimeUnit$1, Transform, {
    transform(O, _) {
      const H = O.field, V = O.interval !== !1, q = O.timezone === "utc", G = this._floor(O, _), J = (q ? utcInterval : timeInterval)(G.unit).offset, Z = O.as || OUTPUT, te = Z[0], ne = Z[1], ee = G.step;
      let ae = G.start || 1 / 0, oe = G.stop || -1 / 0, se = _.ADD;
      return (O.modified() || _.changed(_.REM) || _.modified(accessorFields(H))) && (_ = _.reflow(!0), se = _.SOURCE, ae = 1 / 0, oe = -1 / 0), _.visit(se, (ce) => {
        const le = H(ce);
        let ue, de;
        le == null ? (ce[te] = null, V && (ce[ne] = null)) : (ce[te] = ue = de = G(le), V && (ce[ne] = de = J(ue, ee)), ue < ae && (ae = ue), de > oe && (oe = de));
      }), G.start = ae, G.stop = oe, _.modifies(V ? Z : te);
    },
    _floor(O, _) {
      const H = O.timezone === "utc", {
        units: V,
        step: q
      } = O.units ? {
        units: O.units,
        step: O.step || 1
      } : bin$2({
        extent: O.extent || extent(_.materialize(_.SOURCE).source, O.field),
        maxbins: O.maxbins
      }), G = timeUnits(V), J = this.value || {}, Z = (H ? utcFloor : timeFloor)(G, q);
      return Z.unit = peek$1(G), Z.units = G, Z.step = q, Z.start = J.start, Z.stop = J.stop, this.value = Z;
    }
  });
  function TupleIndex(O) {
    Transform.call(this, fastmap(), O);
  }
  inherits(TupleIndex, Transform, {
    transform(O, _) {
      const H = _.dataflow, V = O.field, q = this.value, G = (Z) => q.set(V(Z), Z);
      let J = !0;
      return O.modified("field") || _.modified(V.fields) ? (q.clear(), _.visit(_.SOURCE, G)) : _.changed() ? (_.visit(_.REM, (Z) => q.delete(V(Z))), _.visit(_.ADD, G)) : J = !1, this.modified(J), q.empty > H.cleanThreshold && H.runAfter(q.clean), _.fork();
    }
  });
  function Values$1(O) {
    Transform.call(this, null, O);
  }
  inherits(Values$1, Transform, {
    transform(O, _) {
      (!this.value || O.modified("field") || O.modified("sort") || _.changed() || O.sort && _.modified(O.sort.fields)) && (this.value = (O.sort ? _.source.slice().sort(stableCompare(O.sort)) : _.source).map(O.field));
    }
  });
  function WindowOp(O, _, H, V) {
    const q = WindowOps[O](_, H);
    return {
      init: q.init || zero$4,
      update: function(G, J) {
        J[V] = q.next(G);
      }
    };
  }
  const WindowOps = {
    row_number: function() {
      return {
        next: (O) => O.index + 1
      };
    },
    rank: function() {
      let O;
      return {
        init: () => O = 1,
        next: (_) => {
          const H = _.index, V = _.data;
          return H && _.compare(V[H - 1], V[H]) ? O = H + 1 : O;
        }
      };
    },
    dense_rank: function() {
      let O;
      return {
        init: () => O = 1,
        next: (_) => {
          const H = _.index, V = _.data;
          return H && _.compare(V[H - 1], V[H]) ? ++O : O;
        }
      };
    },
    percent_rank: function() {
      const O = WindowOps.rank(), _ = O.next;
      return {
        init: O.init,
        next: (H) => (_(H) - 1) / (H.data.length - 1)
      };
    },
    cume_dist: function() {
      let O;
      return {
        init: () => O = 0,
        next: (_) => {
          const H = _.data, V = _.compare;
          let q = _.index;
          if (O < q) {
            for (; q + 1 < H.length && !V(H[q], H[q + 1]); ) ++q;
            O = q;
          }
          return (1 + O) / H.length;
        }
      };
    },
    ntile: function(O, _) {
      _ = +_, _ > 0 || error$1("ntile num must be greater than zero.");
      const H = WindowOps.cume_dist(), V = H.next;
      return {
        init: H.init,
        next: (q) => Math.ceil(_ * V(q))
      };
    },
    lag: function(O, _) {
      return _ = +_ || 1, {
        next: (H) => {
          const V = H.index - _;
          return V >= 0 ? O(H.data[V]) : null;
        }
      };
    },
    lead: function(O, _) {
      return _ = +_ || 1, {
        next: (H) => {
          const V = H.index + _, q = H.data;
          return V < q.length ? O(q[V]) : null;
        }
      };
    },
    first_value: function(O) {
      return {
        next: (_) => O(_.data[_.i0])
      };
    },
    last_value: function(O) {
      return {
        next: (_) => O(_.data[_.i1 - 1])
      };
    },
    nth_value: function(O, _) {
      return _ = +_, _ > 0 || error$1("nth_value nth must be greater than zero."), {
        next: (H) => {
          const V = H.i0 + (_ - 1);
          return V < H.i1 ? O(H.data[V]) : null;
        }
      };
    },
    prev_value: function(O) {
      let _;
      return {
        init: () => _ = null,
        next: (H) => {
          const V = O(H.data[H.index]);
          return V != null ? _ = V : _;
        }
      };
    },
    next_value: function(O) {
      let _, H;
      return {
        init: () => (_ = null, H = -1),
        next: (V) => {
          const q = V.data;
          return V.index <= H ? _ : (H = find$3(O, q, V.index)) < 0 ? (H = q.length, _ = null) : _ = O(q[H]);
        }
      };
    }
  };
  function find$3(O, _, H) {
    for (let V = _.length; H < V; ++H)
      if (O(_[H]) != null) return H;
    return -1;
  }
  const ValidWindowOps = Object.keys(WindowOps);
  function WindowState(O) {
    const _ = array$5(O.ops), H = array$5(O.fields), V = array$5(O.params), q = array$5(O.aggregate_params), G = array$5(O.as), J = this.outputs = [], Z = this.windows = [], te = {}, ne = {}, ee = [], ae = [];
    let oe = !0;
    function se(ce) {
      array$5(accessorFields(ce)).forEach((le) => te[le] = 1);
    }
    se(O.sort), _.forEach((ce, le) => {
      const ue = H[le], de = V[le], fe = q[le] || null, he = accessorName(ue), me = measureName(ce, he, G[le]);
      if (se(ue), J.push(me), has$1(WindowOps, ce))
        Z.push(WindowOp(ce, ue, de, me));
      else {
        if (ue == null && ce !== "count" && error$1("Null aggregate field specified."), ce === "count") {
          ee.push(me);
          return;
        }
        oe = !1;
        let pe = ne[he];
        pe || (pe = ne[he] = [], pe.field = ue, ae.push(pe)), pe.push(createMeasure(ce, fe, me));
      }
    }), (ee.length || ae.length) && (this.cell = cell(ae, ee, oe)), this.inputs = Object.keys(te);
  }
  const prototype = WindowState.prototype;
  prototype.init = function() {
    this.windows.forEach((O) => O.init()), this.cell && this.cell.init();
  };
  prototype.update = function(O, _) {
    const H = this.cell, V = this.windows, q = O.data, G = V && V.length;
    let J;
    if (H) {
      for (J = O.p0; J < O.i0; ++J) H.rem(q[J]);
      for (J = O.p1; J < O.i1; ++J) H.add(q[J]);
      H.set(_);
    }
    for (J = 0; J < G; ++J) V[J].update(O, _);
  };
  function cell(O, _, H) {
    O = O.map((te) => compileMeasures(te, te.field));
    const V = {
      num: 0,
      agg: null,
      store: !1,
      count: _
    };
    if (!H)
      for (var q = O.length, G = V.agg = Array(q), J = 0; J < q; ++J) G[J] = new O[J](V);
    if (V.store)
      var Z = V.data = new TupleStore();
    return V.add = function(te) {
      if (V.num += 1, !H) {
        Z && Z.add(te);
        for (let ne = 0; ne < q; ++ne)
          G[ne].add(G[ne].get(te), te);
      }
    }, V.rem = function(te) {
      if (V.num -= 1, !H) {
        Z && Z.rem(te);
        for (let ne = 0; ne < q; ++ne)
          G[ne].rem(G[ne].get(te), te);
      }
    }, V.set = function(te) {
      let ne, ee;
      for (Z && Z.values(), ne = 0, ee = _.length; ne < ee; ++ne) te[_[ne]] = V.num;
      if (!H) for (ne = 0, ee = G.length; ne < ee; ++ne) G[ne].set(te);
    }, V.init = function() {
      V.num = 0, Z && Z.reset();
      for (let te = 0; te < q; ++te) G[te].init();
    }, V;
  }
  function Window(O) {
    Transform.call(this, {}, O), this._mlen = 0, this._mods = [];
  }
  Window.Definition = {
    type: "Window",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "sort",
      type: "compare"
    }, {
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "ops",
      type: "enum",
      array: !0,
      values: ValidWindowOps.concat(ValidAggregateOps)
    }, {
      name: "params",
      type: "number",
      null: !0,
      array: !0
    }, {
      name: "aggregate_params",
      type: "number",
      null: !0,
      array: !0
    }, {
      name: "fields",
      type: "field",
      null: !0,
      array: !0
    }, {
      name: "as",
      type: "string",
      null: !0,
      array: !0
    }, {
      name: "frame",
      type: "number",
      null: !0,
      array: !0,
      length: 2,
      default: [null, 0]
    }, {
      name: "ignorePeers",
      type: "boolean",
      default: !1
    }]
  };
  inherits(Window, Transform, {
    transform(O, _) {
      this.stamp = _.stamp;
      const H = O.modified(), V = stableCompare(O.sort), q = groupkey(O.groupby), G = (Z) => this.group(q(Z));
      let J = this.state;
      (!J || H) && (J = this.state = new WindowState(O)), H || _.modified(J.inputs) ? (this.value = {}, _.visit(_.SOURCE, (Z) => G(Z).add(Z))) : (_.visit(_.REM, (Z) => G(Z).remove(Z)), _.visit(_.ADD, (Z) => G(Z).add(Z)));
      for (let Z = 0, te = this._mlen; Z < te; ++Z)
        processPartition(this._mods[Z], J, V, O);
      return this._mlen = 0, this._mods = [], _.reflow(H).modifies(J.outputs);
    },
    group(O) {
      let _ = this.value[O];
      return _ || (_ = this.value[O] = SortedList(tupleid), _.stamp = -1), _.stamp < this.stamp && (_.stamp = this.stamp, this._mods[this._mlen++] = _), _;
    }
  });
  function processPartition(O, _, H, V) {
    const q = V.sort, G = q && !V.ignorePeers, J = V.frame || [null, 0], Z = O.data(H), te = Z.length, ne = G ? bisector(q) : null, ee = {
      i0: 0,
      i1: 0,
      p0: 0,
      p1: 0,
      index: 0,
      data: Z,
      compare: q || constant$5(-1)
    };
    _.init();
    for (let ae = 0; ae < te; ++ae)
      setWindow(ee, J, ae, te), G && adjustRange(ee, ne), _.update(ee, Z[ae]);
  }
  function setWindow(O, _, H, V) {
    O.p0 = O.i0, O.p1 = O.i1, O.i0 = _[0] == null ? 0 : Math.max(0, H - Math.abs(_[0])), O.i1 = _[1] == null ? V : Math.min(V, H + Math.abs(_[1]) + 1), O.index = H;
  }
  function adjustRange(O, _) {
    const H = O.i0, V = O.i1 - 1, q = O.compare, G = O.data, J = G.length - 1;
    H > 0 && !q(G[H], G[H - 1]) && (O.i0 = _.left(G, G[H])), V < J && !q(G[V], G[V + 1]) && (O.i1 = _.right(G, G[V]));
  }
  const tx = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    aggregate: Aggregate$1,
    bin: Bin,
    collect: Collect$1,
    compare: Compare$1,
    countpattern: CountPattern,
    cross: Cross,
    density: Density,
    dotbin: DotBin,
    expression: Expression$1,
    extent: Extent,
    facet: Facet$2,
    field: Field$3,
    filter: Filter,
    flatten: Flatten,
    fold: Fold,
    formula: Formula,
    generate: Generate,
    impute: Impute,
    joinaggregate: JoinAggregate,
    kde: KDE,
    key: Key$1,
    load: Load$1,
    lookup: Lookup,
    multiextent: MultiExtent$1,
    multivalues: MultiValues$1,
    params: Params$2,
    pivot: Pivot,
    prefacet: PreFacet$1,
    project: Project,
    proxy: Proxy$2,
    quantile: Quantile$1,
    relay: Relay$1,
    sample: Sample,
    sequence: Sequence,
    sieve: Sieve$1,
    subflow: Subflow,
    timeunit: TimeUnit$1,
    tupleindex: TupleIndex,
    values: Values$1,
    window: Window
  }, Symbol.toStringTag, { value: "Module" }));
  function constant$3(O) {
    return function() {
      return O;
    };
  }
  const abs$2 = Math.abs, atan2$1 = Math.atan2, cos$2 = Math.cos, max$1 = Math.max, min$1 = Math.min, sin$2 = Math.sin, sqrt$3 = Math.sqrt, epsilon$5 = 1e-12, pi$3 = Math.PI, halfPi$2 = pi$3 / 2, tau$3 = 2 * pi$3;
  function acos$1(O) {
    return O > 1 ? 0 : O < -1 ? pi$3 : Math.acos(O);
  }
  function asin$2(O) {
    return O >= 1 ? halfPi$2 : O <= -1 ? -halfPi$2 : Math.asin(O);
  }
  const pi$2 = Math.PI, tau$2 = 2 * pi$2, epsilon$4 = 1e-6, tauEpsilon = tau$2 - epsilon$4;
  function append$1(O) {
    this._ += O[0];
    for (let _ = 1, H = O.length; _ < H; ++_)
      this._ += arguments[_] + O[_];
  }
  function appendRound$1(O) {
    let _ = Math.floor(O);
    if (!(_ >= 0)) throw new Error(`invalid digits: ${O}`);
    if (_ > 15) return append$1;
    const H = 10 ** _;
    return function(V) {
      this._ += V[0];
      for (let q = 1, G = V.length; q < G; ++q)
        this._ += Math.round(arguments[q] * H) / H + V[q];
    };
  }
  let Path$1 = class {
    constructor(_) {
      this._x0 = this._y0 = // start of current subpath
      this._x1 = this._y1 = null, this._ = "", this._append = _ == null ? append$1 : appendRound$1(_);
    }
    moveTo(_, H) {
      this._append`M${this._x0 = this._x1 = +_},${this._y0 = this._y1 = +H}`;
    }
    closePath() {
      this._x1 !== null && (this._x1 = this._x0, this._y1 = this._y0, this._append`Z`);
    }
    lineTo(_, H) {
      this._append`L${this._x1 = +_},${this._y1 = +H}`;
    }
    quadraticCurveTo(_, H, V, q) {
      this._append`Q${+_},${+H},${this._x1 = +V},${this._y1 = +q}`;
    }
    bezierCurveTo(_, H, V, q, G, J) {
      this._append`C${+_},${+H},${+V},${+q},${this._x1 = +G},${this._y1 = +J}`;
    }
    arcTo(_, H, V, q, G) {
      if (_ = +_, H = +H, V = +V, q = +q, G = +G, G < 0) throw new Error(`negative radius: ${G}`);
      let J = this._x1, Z = this._y1, te = V - _, ne = q - H, ee = J - _, ae = Z - H, oe = ee * ee + ae * ae;
      if (this._x1 === null)
        this._append`M${this._x1 = _},${this._y1 = H}`;
      else if (oe > epsilon$4) if (!(Math.abs(ae * te - ne * ee) > epsilon$4) || !G)
        this._append`L${this._x1 = _},${this._y1 = H}`;
      else {
        let se = V - J, ce = q - Z, le = te * te + ne * ne, ue = se * se + ce * ce, de = Math.sqrt(le), fe = Math.sqrt(oe), he = G * Math.tan((pi$2 - Math.acos((le + oe - ue) / (2 * de * fe))) / 2), me = he / fe, pe = he / de;
        Math.abs(me - 1) > epsilon$4 && this._append`L${_ + me * ee},${H + me * ae}`, this._append`A${G},${G},0,0,${+(ae * se > ee * ce)},${this._x1 = _ + pe * te},${this._y1 = H + pe * ne}`;
      }
    }
    arc(_, H, V, q, G, J) {
      if (_ = +_, H = +H, V = +V, J = !!J, V < 0) throw new Error(`negative radius: ${V}`);
      let Z = V * Math.cos(q), te = V * Math.sin(q), ne = _ + Z, ee = H + te, ae = 1 ^ J, oe = J ? q - G : G - q;
      this._x1 === null ? this._append`M${ne},${ee}` : (Math.abs(this._x1 - ne) > epsilon$4 || Math.abs(this._y1 - ee) > epsilon$4) && this._append`L${ne},${ee}`, V && (oe < 0 && (oe = oe % tau$2 + tau$2), oe > tauEpsilon ? this._append`A${V},${V},0,1,${ae},${_ - Z},${H - te}A${V},${V},0,1,${ae},${this._x1 = ne},${this._y1 = ee}` : oe > epsilon$4 && this._append`A${V},${V},0,${+(oe >= pi$2)},${ae},${this._x1 = _ + V * Math.cos(G)},${this._y1 = H + V * Math.sin(G)}`);
    }
    rect(_, H, V, q) {
      this._append`M${this._x0 = this._x1 = +_},${this._y0 = this._y1 = +H}h${V = +V}v${+q}h${-V}Z`;
    }
    toString() {
      return this._;
    }
  };
  function path$3() {
    return new Path$1();
  }
  path$3.prototype = Path$1.prototype;
  function withPath(O) {
    let _ = 3;
    return O.digits = function(H) {
      if (!arguments.length) return _;
      if (H == null)
        _ = null;
      else {
        const V = Math.floor(H);
        if (!(V >= 0)) throw new RangeError(`invalid digits: ${H}`);
        _ = V;
      }
      return O;
    }, () => new Path$1(_);
  }
  function arcInnerRadius(O) {
    return O.innerRadius;
  }
  function arcOuterRadius(O) {
    return O.outerRadius;
  }
  function arcStartAngle(O) {
    return O.startAngle;
  }
  function arcEndAngle(O) {
    return O.endAngle;
  }
  function arcPadAngle(O) {
    return O && O.padAngle;
  }
  function intersect$3(O, _, H, V, q, G, J, Z) {
    var te = H - O, ne = V - _, ee = J - q, ae = Z - G, oe = ae * te - ee * ne;
    if (!(oe * oe < epsilon$5))
      return oe = (ee * (_ - G) - ae * (O - q)) / oe, [O + oe * te, _ + oe * ne];
  }
  function cornerTangents(O, _, H, V, q, G, J) {
    var Z = O - H, te = _ - V, ne = (J ? G : -G) / sqrt$3(Z * Z + te * te), ee = ne * te, ae = -ne * Z, oe = O + ee, se = _ + ae, ce = H + ee, le = V + ae, ue = (oe + ce) / 2, de = (se + le) / 2, fe = ce - oe, he = le - se, me = fe * fe + he * he, pe = q - G, ye = oe * le - ce * se, ge = (he < 0 ? -1 : 1) * sqrt$3(max$1(0, pe * pe * me - ye * ye)), ve = (ye * he - fe * ge) / me, be = (-ye * fe - he * ge) / me, Ce = (ye * he + fe * ge) / me, Se = (-ye * fe + he * ge) / me, we = ve - ue, Ee = be - de, xe = Ce - ue, Le = Se - de;
    return we * we + Ee * Ee > xe * xe + Le * Le && (ve = Ce, be = Se), {
      cx: ve,
      cy: be,
      x01: -ee,
      y01: -ae,
      x11: ve * (q / pe - 1),
      y11: be * (q / pe - 1)
    };
  }
  function arc$2$1() {
    var O = arcInnerRadius, _ = arcOuterRadius, H = constant$3(0), V = null, q = arcStartAngle, G = arcEndAngle, J = arcPadAngle, Z = null, te = withPath(ne);
    function ne() {
      var ee, ae, oe = +O.apply(this, arguments), se = +_.apply(this, arguments), ce = q.apply(this, arguments) - halfPi$2, le = G.apply(this, arguments) - halfPi$2, ue = abs$2(le - ce), de = le > ce;
      if (Z || (Z = ee = te()), se < oe && (ae = se, se = oe, oe = ae), !(se > epsilon$5)) Z.moveTo(0, 0);
      else if (ue > tau$3 - epsilon$5)
        Z.moveTo(se * cos$2(ce), se * sin$2(ce)), Z.arc(0, 0, se, ce, le, !de), oe > epsilon$5 && (Z.moveTo(oe * cos$2(le), oe * sin$2(le)), Z.arc(0, 0, oe, le, ce, de));
      else {
        var fe = ce, he = le, me = ce, pe = le, ye = ue, ge = ue, ve = J.apply(this, arguments) / 2, be = ve > epsilon$5 && (V ? +V.apply(this, arguments) : sqrt$3(oe * oe + se * se)), Ce = min$1(abs$2(se - oe) / 2, +H.apply(this, arguments)), Se = Ce, we = Ce, Ee, xe;
        if (be > epsilon$5) {
          var Le = asin$2(be / oe * sin$2(ve)), Qe = asin$2(be / se * sin$2(ve));
          (ye -= Le * 2) > epsilon$5 ? (Le *= de ? 1 : -1, me += Le, pe -= Le) : (ye = 0, me = pe = (ce + le) / 2), (ge -= Qe * 2) > epsilon$5 ? (Qe *= de ? 1 : -1, fe += Qe, he -= Qe) : (ge = 0, fe = he = (ce + le) / 2);
        }
        var ke = se * cos$2(fe), Me = se * sin$2(fe), Re = oe * cos$2(pe), De = oe * sin$2(pe);
        if (Ce > epsilon$5) {
          var Pe = se * cos$2(he), $e = se * sin$2(he), qe = oe * cos$2(me), Xe = oe * sin$2(me), Ve;
          if (ue < pi$3)
            if (Ve = intersect$3(ke, Me, qe, Xe, Pe, $e, Re, De)) {
              var Ze = ke - Ve[0], Ge = Me - Ve[1], ze = Pe - Ve[0], We = $e - Ve[1], at = 1 / sin$2(acos$1((Ze * ze + Ge * We) / (sqrt$3(Ze * Ze + Ge * Ge) * sqrt$3(ze * ze + We * We))) / 2), ct = sqrt$3(Ve[0] * Ve[0] + Ve[1] * Ve[1]);
              Se = min$1(Ce, (oe - ct) / (at - 1)), we = min$1(Ce, (se - ct) / (at + 1));
            } else
              Se = we = 0;
        }
        ge > epsilon$5 ? we > epsilon$5 ? (Ee = cornerTangents(qe, Xe, ke, Me, se, we, de), xe = cornerTangents(Pe, $e, Re, De, se, we, de), Z.moveTo(Ee.cx + Ee.x01, Ee.cy + Ee.y01), we < Ce ? Z.arc(Ee.cx, Ee.cy, we, atan2$1(Ee.y01, Ee.x01), atan2$1(xe.y01, xe.x01), !de) : (Z.arc(Ee.cx, Ee.cy, we, atan2$1(Ee.y01, Ee.x01), atan2$1(Ee.y11, Ee.x11), !de), Z.arc(0, 0, se, atan2$1(Ee.cy + Ee.y11, Ee.cx + Ee.x11), atan2$1(xe.cy + xe.y11, xe.cx + xe.x11), !de), Z.arc(xe.cx, xe.cy, we, atan2$1(xe.y11, xe.x11), atan2$1(xe.y01, xe.x01), !de))) : (Z.moveTo(ke, Me), Z.arc(0, 0, se, fe, he, !de)) : Z.moveTo(ke, Me), !(oe > epsilon$5) || !(ye > epsilon$5) ? Z.lineTo(Re, De) : Se > epsilon$5 ? (Ee = cornerTangents(Re, De, Pe, $e, oe, -Se, de), xe = cornerTangents(ke, Me, qe, Xe, oe, -Se, de), Z.lineTo(Ee.cx + Ee.x01, Ee.cy + Ee.y01), Se < Ce ? Z.arc(Ee.cx, Ee.cy, Se, atan2$1(Ee.y01, Ee.x01), atan2$1(xe.y01, xe.x01), !de) : (Z.arc(Ee.cx, Ee.cy, Se, atan2$1(Ee.y01, Ee.x01), atan2$1(Ee.y11, Ee.x11), !de), Z.arc(0, 0, oe, atan2$1(Ee.cy + Ee.y11, Ee.cx + Ee.x11), atan2$1(xe.cy + xe.y11, xe.cx + xe.x11), de), Z.arc(xe.cx, xe.cy, Se, atan2$1(xe.y11, xe.x11), atan2$1(xe.y01, xe.x01), !de))) : Z.arc(0, 0, oe, pe, me, de);
      }
      if (Z.closePath(), ee) return Z = null, ee + "" || null;
    }
    return ne.centroid = function() {
      var ee = (+O.apply(this, arguments) + +_.apply(this, arguments)) / 2, ae = (+q.apply(this, arguments) + +G.apply(this, arguments)) / 2 - pi$3 / 2;
      return [cos$2(ae) * ee, sin$2(ae) * ee];
    }, ne.innerRadius = function(ee) {
      return arguments.length ? (O = typeof ee == "function" ? ee : constant$3(+ee), ne) : O;
    }, ne.outerRadius = function(ee) {
      return arguments.length ? (_ = typeof ee == "function" ? ee : constant$3(+ee), ne) : _;
    }, ne.cornerRadius = function(ee) {
      return arguments.length ? (H = typeof ee == "function" ? ee : constant$3(+ee), ne) : H;
    }, ne.padRadius = function(ee) {
      return arguments.length ? (V = ee == null ? null : typeof ee == "function" ? ee : constant$3(+ee), ne) : V;
    }, ne.startAngle = function(ee) {
      return arguments.length ? (q = typeof ee == "function" ? ee : constant$3(+ee), ne) : q;
    }, ne.endAngle = function(ee) {
      return arguments.length ? (G = typeof ee == "function" ? ee : constant$3(+ee), ne) : G;
    }, ne.padAngle = function(ee) {
      return arguments.length ? (J = typeof ee == "function" ? ee : constant$3(+ee), ne) : J;
    }, ne.context = function(ee) {
      return arguments.length ? (Z = ee ?? null, ne) : Z;
    }, ne;
  }
  function array$4(O) {
    return typeof O == "object" && "length" in O ? O : Array.from(O);
  }
  function Linear$1(O) {
    this._context = O;
  }
  Linear$1.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._point = 0;
    },
    lineEnd: function() {
      (this._line || this._line !== 0 && this._point === 1) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1, this._line ? this._context.lineTo(O, _) : this._context.moveTo(O, _);
          break;
        case 1:
          this._point = 2;
        // falls through
        default:
          this._context.lineTo(O, _);
          break;
      }
    }
  };
  function curveLinear(O) {
    return new Linear$1(O);
  }
  function x$4(O) {
    return O[0];
  }
  function y$3(O) {
    return O[1];
  }
  function line$2$1(O, _) {
    var H = constant$3(!0), V = null, q = curveLinear, G = null, J = withPath(Z);
    O = typeof O == "function" ? O : O === void 0 ? x$4 : constant$3(O), _ = typeof _ == "function" ? _ : _ === void 0 ? y$3 : constant$3(_);
    function Z(te) {
      var ne, ee = (te = array$4(te)).length, ae, oe = !1, se;
      for (V == null && (G = q(se = J())), ne = 0; ne <= ee; ++ne)
        !(ne < ee && H(ae = te[ne], ne, te)) === oe && ((oe = !oe) ? G.lineStart() : G.lineEnd()), oe && G.point(+O(ae, ne, te), +_(ae, ne, te));
      if (se) return G = null, se + "" || null;
    }
    return Z.x = function(te) {
      return arguments.length ? (O = typeof te == "function" ? te : constant$3(+te), Z) : O;
    }, Z.y = function(te) {
      return arguments.length ? (_ = typeof te == "function" ? te : constant$3(+te), Z) : _;
    }, Z.defined = function(te) {
      return arguments.length ? (H = typeof te == "function" ? te : constant$3(!!te), Z) : H;
    }, Z.curve = function(te) {
      return arguments.length ? (q = te, V != null && (G = q(V)), Z) : q;
    }, Z.context = function(te) {
      return arguments.length ? (te == null ? V = G = null : G = q(V = te), Z) : V;
    }, Z;
  }
  function area$2$1(O, _, H) {
    var V = null, q = constant$3(!0), G = null, J = curveLinear, Z = null, te = withPath(ne);
    O = typeof O == "function" ? O : O === void 0 ? x$4 : constant$3(+O), _ = typeof _ == "function" ? _ : constant$3(_ === void 0 ? 0 : +_), H = typeof H == "function" ? H : H === void 0 ? y$3 : constant$3(+H);
    function ne(ae) {
      var oe, se, ce, le = (ae = array$4(ae)).length, ue, de = !1, fe, he = new Array(le), me = new Array(le);
      for (G == null && (Z = J(fe = te())), oe = 0; oe <= le; ++oe) {
        if (!(oe < le && q(ue = ae[oe], oe, ae)) === de)
          if (de = !de)
            se = oe, Z.areaStart(), Z.lineStart();
          else {
            for (Z.lineEnd(), Z.lineStart(), ce = oe - 1; ce >= se; --ce)
              Z.point(he[ce], me[ce]);
            Z.lineEnd(), Z.areaEnd();
          }
        de && (he[oe] = +O(ue, oe, ae), me[oe] = +_(ue, oe, ae), Z.point(V ? +V(ue, oe, ae) : he[oe], H ? +H(ue, oe, ae) : me[oe]));
      }
      if (fe) return Z = null, fe + "" || null;
    }
    function ee() {
      return line$2$1().defined(q).curve(J).context(G);
    }
    return ne.x = function(ae) {
      return arguments.length ? (O = typeof ae == "function" ? ae : constant$3(+ae), V = null, ne) : O;
    }, ne.x0 = function(ae) {
      return arguments.length ? (O = typeof ae == "function" ? ae : constant$3(+ae), ne) : O;
    }, ne.x1 = function(ae) {
      return arguments.length ? (V = ae == null ? null : typeof ae == "function" ? ae : constant$3(+ae), ne) : V;
    }, ne.y = function(ae) {
      return arguments.length ? (_ = typeof ae == "function" ? ae : constant$3(+ae), H = null, ne) : _;
    }, ne.y0 = function(ae) {
      return arguments.length ? (_ = typeof ae == "function" ? ae : constant$3(+ae), ne) : _;
    }, ne.y1 = function(ae) {
      return arguments.length ? (H = ae == null ? null : typeof ae == "function" ? ae : constant$3(+ae), ne) : H;
    }, ne.lineX0 = ne.lineY0 = function() {
      return ee().x(O).y(_);
    }, ne.lineY1 = function() {
      return ee().x(O).y(H);
    }, ne.lineX1 = function() {
      return ee().x(V).y(_);
    }, ne.defined = function(ae) {
      return arguments.length ? (q = typeof ae == "function" ? ae : constant$3(!!ae), ne) : q;
    }, ne.curve = function(ae) {
      return arguments.length ? (J = ae, G != null && (Z = J(G)), ne) : J;
    }, ne.context = function(ae) {
      return arguments.length ? (ae == null ? G = Z = null : Z = J(G = ae), ne) : G;
    }, ne;
  }
  const circle$1 = {
    draw(O, _) {
      const H = sqrt$3(_ / pi$3);
      O.moveTo(H, 0), O.arc(0, 0, H, 0, tau$3);
    }
  };
  function Symbol$2(O, _) {
    let H = null, V = withPath(q);
    O = typeof O == "function" ? O : constant$3(O || circle$1), _ = typeof _ == "function" ? _ : constant$3(_ === void 0 ? 64 : +_);
    function q() {
      let G;
      if (H || (H = G = V()), O.apply(this, arguments).draw(H, +_.apply(this, arguments)), G) return H = null, G + "" || null;
    }
    return q.type = function(G) {
      return arguments.length ? (O = typeof G == "function" ? G : constant$3(G), q) : O;
    }, q.size = function(G) {
      return arguments.length ? (_ = typeof G == "function" ? G : constant$3(+G), q) : _;
    }, q.context = function(G) {
      return arguments.length ? (H = G ?? null, q) : H;
    }, q;
  }
  function noop$3() {
  }
  function point$7(O, _, H) {
    O._context.bezierCurveTo(
      (2 * O._x0 + O._x1) / 3,
      (2 * O._y0 + O._y1) / 3,
      (O._x0 + 2 * O._x1) / 3,
      (O._y0 + 2 * O._y1) / 3,
      (O._x0 + 4 * O._x1 + _) / 6,
      (O._y0 + 4 * O._y1 + H) / 6
    );
  }
  function Basis(O) {
    this._context = O;
  }
  Basis.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x0 = this._x1 = this._y0 = this._y1 = NaN, this._point = 0;
    },
    lineEnd: function() {
      switch (this._point) {
        case 3:
          point$7(this, this._x1, this._y1);
        // falls through
        case 2:
          this._context.lineTo(this._x1, this._y1);
          break;
      }
      (this._line || this._line !== 0 && this._point === 1) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1, this._line ? this._context.lineTo(O, _) : this._context.moveTo(O, _);
          break;
        case 1:
          this._point = 2;
          break;
        case 2:
          this._point = 3, this._context.lineTo((5 * this._x0 + this._x1) / 6, (5 * this._y0 + this._y1) / 6);
        // falls through
        default:
          point$7(this, O, _);
          break;
      }
      this._x0 = this._x1, this._x1 = O, this._y0 = this._y1, this._y1 = _;
    }
  };
  function curveBasis(O) {
    return new Basis(O);
  }
  function BasisClosed(O) {
    this._context = O;
  }
  BasisClosed.prototype = {
    areaStart: noop$3,
    areaEnd: noop$3,
    lineStart: function() {
      this._x0 = this._x1 = this._x2 = this._x3 = this._x4 = this._y0 = this._y1 = this._y2 = this._y3 = this._y4 = NaN, this._point = 0;
    },
    lineEnd: function() {
      switch (this._point) {
        case 1: {
          this._context.moveTo(this._x2, this._y2), this._context.closePath();
          break;
        }
        case 2: {
          this._context.moveTo((this._x2 + 2 * this._x3) / 3, (this._y2 + 2 * this._y3) / 3), this._context.lineTo((this._x3 + 2 * this._x2) / 3, (this._y3 + 2 * this._y2) / 3), this._context.closePath();
          break;
        }
        case 3: {
          this.point(this._x2, this._y2), this.point(this._x3, this._y3), this.point(this._x4, this._y4);
          break;
        }
      }
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1, this._x2 = O, this._y2 = _;
          break;
        case 1:
          this._point = 2, this._x3 = O, this._y3 = _;
          break;
        case 2:
          this._point = 3, this._x4 = O, this._y4 = _, this._context.moveTo((this._x0 + 4 * this._x1 + O) / 6, (this._y0 + 4 * this._y1 + _) / 6);
          break;
        default:
          point$7(this, O, _);
          break;
      }
      this._x0 = this._x1, this._x1 = O, this._y0 = this._y1, this._y1 = _;
    }
  };
  function curveBasisClosed(O) {
    return new BasisClosed(O);
  }
  function BasisOpen(O) {
    this._context = O;
  }
  BasisOpen.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x0 = this._x1 = this._y0 = this._y1 = NaN, this._point = 0;
    },
    lineEnd: function() {
      (this._line || this._line !== 0 && this._point === 3) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1;
          break;
        case 1:
          this._point = 2;
          break;
        case 2:
          this._point = 3;
          var H = (this._x0 + 4 * this._x1 + O) / 6, V = (this._y0 + 4 * this._y1 + _) / 6;
          this._line ? this._context.lineTo(H, V) : this._context.moveTo(H, V);
          break;
        case 3:
          this._point = 4;
        // falls through
        default:
          point$7(this, O, _);
          break;
      }
      this._x0 = this._x1, this._x1 = O, this._y0 = this._y1, this._y1 = _;
    }
  };
  function curveBasisOpen(O) {
    return new BasisOpen(O);
  }
  function Bundle(O, _) {
    this._basis = new Basis(O), this._beta = _;
  }
  Bundle.prototype = {
    lineStart: function() {
      this._x = [], this._y = [], this._basis.lineStart();
    },
    lineEnd: function() {
      var O = this._x, _ = this._y, H = O.length - 1;
      if (H > 0)
        for (var V = O[0], q = _[0], G = O[H] - V, J = _[H] - q, Z = -1, te; ++Z <= H; )
          te = Z / H, this._basis.point(
            this._beta * O[Z] + (1 - this._beta) * (V + te * G),
            this._beta * _[Z] + (1 - this._beta) * (q + te * J)
          );
      this._x = this._y = null, this._basis.lineEnd();
    },
    point: function(O, _) {
      this._x.push(+O), this._y.push(+_);
    }
  };
  const curveBundle = function O(_) {
    function H(V) {
      return _ === 1 ? new Basis(V) : new Bundle(V, _);
    }
    return H.beta = function(V) {
      return O(+V);
    }, H;
  }(0.85);
  function point$6(O, _, H) {
    O._context.bezierCurveTo(
      O._x1 + O._k * (O._x2 - O._x0),
      O._y1 + O._k * (O._y2 - O._y0),
      O._x2 + O._k * (O._x1 - _),
      O._y2 + O._k * (O._y1 - H),
      O._x2,
      O._y2
    );
  }
  function Cardinal(O, _) {
    this._context = O, this._k = (1 - _) / 6;
  }
  Cardinal.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x0 = this._x1 = this._x2 = this._y0 = this._y1 = this._y2 = NaN, this._point = 0;
    },
    lineEnd: function() {
      switch (this._point) {
        case 2:
          this._context.lineTo(this._x2, this._y2);
          break;
        case 3:
          point$6(this, this._x1, this._y1);
          break;
      }
      (this._line || this._line !== 0 && this._point === 1) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1, this._line ? this._context.lineTo(O, _) : this._context.moveTo(O, _);
          break;
        case 1:
          this._point = 2, this._x1 = O, this._y1 = _;
          break;
        case 2:
          this._point = 3;
        // falls through
        default:
          point$6(this, O, _);
          break;
      }
      this._x0 = this._x1, this._x1 = this._x2, this._x2 = O, this._y0 = this._y1, this._y1 = this._y2, this._y2 = _;
    }
  };
  const curveCardinal = function O(_) {
    function H(V) {
      return new Cardinal(V, _);
    }
    return H.tension = function(V) {
      return O(+V);
    }, H;
  }(0);
  function CardinalClosed(O, _) {
    this._context = O, this._k = (1 - _) / 6;
  }
  CardinalClosed.prototype = {
    areaStart: noop$3,
    areaEnd: noop$3,
    lineStart: function() {
      this._x0 = this._x1 = this._x2 = this._x3 = this._x4 = this._x5 = this._y0 = this._y1 = this._y2 = this._y3 = this._y4 = this._y5 = NaN, this._point = 0;
    },
    lineEnd: function() {
      switch (this._point) {
        case 1: {
          this._context.moveTo(this._x3, this._y3), this._context.closePath();
          break;
        }
        case 2: {
          this._context.lineTo(this._x3, this._y3), this._context.closePath();
          break;
        }
        case 3: {
          this.point(this._x3, this._y3), this.point(this._x4, this._y4), this.point(this._x5, this._y5);
          break;
        }
      }
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1, this._x3 = O, this._y3 = _;
          break;
        case 1:
          this._point = 2, this._context.moveTo(this._x4 = O, this._y4 = _);
          break;
        case 2:
          this._point = 3, this._x5 = O, this._y5 = _;
          break;
        default:
          point$6(this, O, _);
          break;
      }
      this._x0 = this._x1, this._x1 = this._x2, this._x2 = O, this._y0 = this._y1, this._y1 = this._y2, this._y2 = _;
    }
  };
  const curveCardinalClosed = function O(_) {
    function H(V) {
      return new CardinalClosed(V, _);
    }
    return H.tension = function(V) {
      return O(+V);
    }, H;
  }(0);
  function CardinalOpen(O, _) {
    this._context = O, this._k = (1 - _) / 6;
  }
  CardinalOpen.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x0 = this._x1 = this._x2 = this._y0 = this._y1 = this._y2 = NaN, this._point = 0;
    },
    lineEnd: function() {
      (this._line || this._line !== 0 && this._point === 3) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1;
          break;
        case 1:
          this._point = 2;
          break;
        case 2:
          this._point = 3, this._line ? this._context.lineTo(this._x2, this._y2) : this._context.moveTo(this._x2, this._y2);
          break;
        case 3:
          this._point = 4;
        // falls through
        default:
          point$6(this, O, _);
          break;
      }
      this._x0 = this._x1, this._x1 = this._x2, this._x2 = O, this._y0 = this._y1, this._y1 = this._y2, this._y2 = _;
    }
  };
  const curveCardinalOpen = function O(_) {
    function H(V) {
      return new CardinalOpen(V, _);
    }
    return H.tension = function(V) {
      return O(+V);
    }, H;
  }(0);
  function point$5(O, _, H) {
    var V = O._x1, q = O._y1, G = O._x2, J = O._y2;
    if (O._l01_a > epsilon$5) {
      var Z = 2 * O._l01_2a + 3 * O._l01_a * O._l12_a + O._l12_2a, te = 3 * O._l01_a * (O._l01_a + O._l12_a);
      V = (V * Z - O._x0 * O._l12_2a + O._x2 * O._l01_2a) / te, q = (q * Z - O._y0 * O._l12_2a + O._y2 * O._l01_2a) / te;
    }
    if (O._l23_a > epsilon$5) {
      var ne = 2 * O._l23_2a + 3 * O._l23_a * O._l12_a + O._l12_2a, ee = 3 * O._l23_a * (O._l23_a + O._l12_a);
      G = (G * ne + O._x1 * O._l23_2a - _ * O._l12_2a) / ee, J = (J * ne + O._y1 * O._l23_2a - H * O._l12_2a) / ee;
    }
    O._context.bezierCurveTo(V, q, G, J, O._x2, O._y2);
  }
  function CatmullRom(O, _) {
    this._context = O, this._alpha = _;
  }
  CatmullRom.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x0 = this._x1 = this._x2 = this._y0 = this._y1 = this._y2 = NaN, this._l01_a = this._l12_a = this._l23_a = this._l01_2a = this._l12_2a = this._l23_2a = this._point = 0;
    },
    lineEnd: function() {
      switch (this._point) {
        case 2:
          this._context.lineTo(this._x2, this._y2);
          break;
        case 3:
          this.point(this._x2, this._y2);
          break;
      }
      (this._line || this._line !== 0 && this._point === 1) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      if (O = +O, _ = +_, this._point) {
        var H = this._x2 - O, V = this._y2 - _;
        this._l23_a = Math.sqrt(this._l23_2a = Math.pow(H * H + V * V, this._alpha));
      }
      switch (this._point) {
        case 0:
          this._point = 1, this._line ? this._context.lineTo(O, _) : this._context.moveTo(O, _);
          break;
        case 1:
          this._point = 2;
          break;
        case 2:
          this._point = 3;
        // falls through
        default:
          point$5(this, O, _);
          break;
      }
      this._l01_a = this._l12_a, this._l12_a = this._l23_a, this._l01_2a = this._l12_2a, this._l12_2a = this._l23_2a, this._x0 = this._x1, this._x1 = this._x2, this._x2 = O, this._y0 = this._y1, this._y1 = this._y2, this._y2 = _;
    }
  };
  const curveCatmullRom = function O(_) {
    function H(V) {
      return _ ? new CatmullRom(V, _) : new Cardinal(V, 0);
    }
    return H.alpha = function(V) {
      return O(+V);
    }, H;
  }(0.5);
  function CatmullRomClosed(O, _) {
    this._context = O, this._alpha = _;
  }
  CatmullRomClosed.prototype = {
    areaStart: noop$3,
    areaEnd: noop$3,
    lineStart: function() {
      this._x0 = this._x1 = this._x2 = this._x3 = this._x4 = this._x5 = this._y0 = this._y1 = this._y2 = this._y3 = this._y4 = this._y5 = NaN, this._l01_a = this._l12_a = this._l23_a = this._l01_2a = this._l12_2a = this._l23_2a = this._point = 0;
    },
    lineEnd: function() {
      switch (this._point) {
        case 1: {
          this._context.moveTo(this._x3, this._y3), this._context.closePath();
          break;
        }
        case 2: {
          this._context.lineTo(this._x3, this._y3), this._context.closePath();
          break;
        }
        case 3: {
          this.point(this._x3, this._y3), this.point(this._x4, this._y4), this.point(this._x5, this._y5);
          break;
        }
      }
    },
    point: function(O, _) {
      if (O = +O, _ = +_, this._point) {
        var H = this._x2 - O, V = this._y2 - _;
        this._l23_a = Math.sqrt(this._l23_2a = Math.pow(H * H + V * V, this._alpha));
      }
      switch (this._point) {
        case 0:
          this._point = 1, this._x3 = O, this._y3 = _;
          break;
        case 1:
          this._point = 2, this._context.moveTo(this._x4 = O, this._y4 = _);
          break;
        case 2:
          this._point = 3, this._x5 = O, this._y5 = _;
          break;
        default:
          point$5(this, O, _);
          break;
      }
      this._l01_a = this._l12_a, this._l12_a = this._l23_a, this._l01_2a = this._l12_2a, this._l12_2a = this._l23_2a, this._x0 = this._x1, this._x1 = this._x2, this._x2 = O, this._y0 = this._y1, this._y1 = this._y2, this._y2 = _;
    }
  };
  const curveCatmullRomClosed = function O(_) {
    function H(V) {
      return _ ? new CatmullRomClosed(V, _) : new CardinalClosed(V, 0);
    }
    return H.alpha = function(V) {
      return O(+V);
    }, H;
  }(0.5);
  function CatmullRomOpen(O, _) {
    this._context = O, this._alpha = _;
  }
  CatmullRomOpen.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x0 = this._x1 = this._x2 = this._y0 = this._y1 = this._y2 = NaN, this._l01_a = this._l12_a = this._l23_a = this._l01_2a = this._l12_2a = this._l23_2a = this._point = 0;
    },
    lineEnd: function() {
      (this._line || this._line !== 0 && this._point === 3) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      if (O = +O, _ = +_, this._point) {
        var H = this._x2 - O, V = this._y2 - _;
        this._l23_a = Math.sqrt(this._l23_2a = Math.pow(H * H + V * V, this._alpha));
      }
      switch (this._point) {
        case 0:
          this._point = 1;
          break;
        case 1:
          this._point = 2;
          break;
        case 2:
          this._point = 3, this._line ? this._context.lineTo(this._x2, this._y2) : this._context.moveTo(this._x2, this._y2);
          break;
        case 3:
          this._point = 4;
        // falls through
        default:
          point$5(this, O, _);
          break;
      }
      this._l01_a = this._l12_a, this._l12_a = this._l23_a, this._l01_2a = this._l12_2a, this._l12_2a = this._l23_2a, this._x0 = this._x1, this._x1 = this._x2, this._x2 = O, this._y0 = this._y1, this._y1 = this._y2, this._y2 = _;
    }
  };
  const curveCatmullRomOpen = function O(_) {
    function H(V) {
      return _ ? new CatmullRomOpen(V, _) : new CardinalOpen(V, 0);
    }
    return H.alpha = function(V) {
      return O(+V);
    }, H;
  }(0.5);
  function LinearClosed(O) {
    this._context = O;
  }
  LinearClosed.prototype = {
    areaStart: noop$3,
    areaEnd: noop$3,
    lineStart: function() {
      this._point = 0;
    },
    lineEnd: function() {
      this._point && this._context.closePath();
    },
    point: function(O, _) {
      O = +O, _ = +_, this._point ? this._context.lineTo(O, _) : (this._point = 1, this._context.moveTo(O, _));
    }
  };
  function curveLinearClosed(O) {
    return new LinearClosed(O);
  }
  function sign$1(O) {
    return O < 0 ? -1 : 1;
  }
  function slope3(O, _, H) {
    var V = O._x1 - O._x0, q = _ - O._x1, G = (O._y1 - O._y0) / (V || q < 0 && -0), J = (H - O._y1) / (q || V < 0 && -0), Z = (G * q + J * V) / (V + q);
    return (sign$1(G) + sign$1(J)) * Math.min(Math.abs(G), Math.abs(J), 0.5 * Math.abs(Z)) || 0;
  }
  function slope2(O, _) {
    var H = O._x1 - O._x0;
    return H ? (3 * (O._y1 - O._y0) / H - _) / 2 : _;
  }
  function point$4(O, _, H) {
    var V = O._x0, q = O._y0, G = O._x1, J = O._y1, Z = (G - V) / 3;
    O._context.bezierCurveTo(V + Z, q + Z * _, G - Z, J - Z * H, G, J);
  }
  function MonotoneX(O) {
    this._context = O;
  }
  MonotoneX.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x0 = this._x1 = this._y0 = this._y1 = this._t0 = NaN, this._point = 0;
    },
    lineEnd: function() {
      switch (this._point) {
        case 2:
          this._context.lineTo(this._x1, this._y1);
          break;
        case 3:
          point$4(this, this._t0, slope2(this, this._t0));
          break;
      }
      (this._line || this._line !== 0 && this._point === 1) && this._context.closePath(), this._line = 1 - this._line;
    },
    point: function(O, _) {
      var H = NaN;
      if (O = +O, _ = +_, !(O === this._x1 && _ === this._y1)) {
        switch (this._point) {
          case 0:
            this._point = 1, this._line ? this._context.lineTo(O, _) : this._context.moveTo(O, _);
            break;
          case 1:
            this._point = 2;
            break;
          case 2:
            this._point = 3, point$4(this, slope2(this, H = slope3(this, O, _)), H);
            break;
          default:
            point$4(this, this._t0, H = slope3(this, O, _));
            break;
        }
        this._x0 = this._x1, this._x1 = O, this._y0 = this._y1, this._y1 = _, this._t0 = H;
      }
    }
  };
  function MonotoneY(O) {
    this._context = new ReflectContext(O);
  }
  (MonotoneY.prototype = Object.create(MonotoneX.prototype)).point = function(O, _) {
    MonotoneX.prototype.point.call(this, _, O);
  };
  function ReflectContext(O) {
    this._context = O;
  }
  ReflectContext.prototype = {
    moveTo: function(O, _) {
      this._context.moveTo(_, O);
    },
    closePath: function() {
      this._context.closePath();
    },
    lineTo: function(O, _) {
      this._context.lineTo(_, O);
    },
    bezierCurveTo: function(O, _, H, V, q, G) {
      this._context.bezierCurveTo(_, O, V, H, G, q);
    }
  };
  function monotoneX(O) {
    return new MonotoneX(O);
  }
  function monotoneY(O) {
    return new MonotoneY(O);
  }
  function Natural(O) {
    this._context = O;
  }
  Natural.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x = [], this._y = [];
    },
    lineEnd: function() {
      var O = this._x, _ = this._y, H = O.length;
      if (H)
        if (this._line ? this._context.lineTo(O[0], _[0]) : this._context.moveTo(O[0], _[0]), H === 2)
          this._context.lineTo(O[1], _[1]);
        else
          for (var V = controlPoints(O), q = controlPoints(_), G = 0, J = 1; J < H; ++G, ++J)
            this._context.bezierCurveTo(V[0][G], q[0][G], V[1][G], q[1][G], O[J], _[J]);
      (this._line || this._line !== 0 && H === 1) && this._context.closePath(), this._line = 1 - this._line, this._x = this._y = null;
    },
    point: function(O, _) {
      this._x.push(+O), this._y.push(+_);
    }
  };
  function controlPoints(O) {
    var _, H = O.length - 1, V, q = new Array(H), G = new Array(H), J = new Array(H);
    for (q[0] = 0, G[0] = 2, J[0] = O[0] + 2 * O[1], _ = 1; _ < H - 1; ++_) q[_] = 1, G[_] = 4, J[_] = 4 * O[_] + 2 * O[_ + 1];
    for (q[H - 1] = 2, G[H - 1] = 7, J[H - 1] = 8 * O[H - 1] + O[H], _ = 1; _ < H; ++_) V = q[_] / G[_ - 1], G[_] -= V, J[_] -= V * J[_ - 1];
    for (q[H - 1] = J[H - 1] / G[H - 1], _ = H - 2; _ >= 0; --_) q[_] = (J[_] - q[_ + 1]) / G[_];
    for (G[H - 1] = (O[H] + q[H - 1]) / 2, _ = 0; _ < H - 1; ++_) G[_] = 2 * O[_ + 1] - q[_ + 1];
    return [q, G];
  }
  function curveNatural(O) {
    return new Natural(O);
  }
  function Step(O, _) {
    this._context = O, this._t = _;
  }
  Step.prototype = {
    areaStart: function() {
      this._line = 0;
    },
    areaEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._x = this._y = NaN, this._point = 0;
    },
    lineEnd: function() {
      0 < this._t && this._t < 1 && this._point === 2 && this._context.lineTo(this._x, this._y), (this._line || this._line !== 0 && this._point === 1) && this._context.closePath(), this._line >= 0 && (this._t = 1 - this._t, this._line = 1 - this._line);
    },
    point: function(O, _) {
      switch (O = +O, _ = +_, this._point) {
        case 0:
          this._point = 1, this._line ? this._context.lineTo(O, _) : this._context.moveTo(O, _);
          break;
        case 1:
          this._point = 2;
        // falls through
        default: {
          if (this._t <= 0)
            this._context.lineTo(this._x, _), this._context.lineTo(O, _);
          else {
            var H = this._x * (1 - this._t) + O * this._t;
            this._context.lineTo(H, this._y), this._context.lineTo(H, _);
          }
          break;
        }
      }
      this._x = O, this._y = _;
    }
  };
  function curveStep(O) {
    return new Step(O, 0.5);
  }
  function stepBefore(O) {
    return new Step(O, 0);
  }
  function stepAfter(O) {
    return new Step(O, 1);
  }
  function domCanvas(O, _) {
    if (typeof document < "u" && document.createElement) {
      const H = document.createElement("canvas");
      if (H && H.getContext)
        return H.width = O, H.height = _, H;
    }
    return null;
  }
  const domImage = () => typeof Image < "u" ? Image : null;
  function initRange(O, _) {
    switch (arguments.length) {
      case 0:
        break;
      case 1:
        this.range(O);
        break;
      default:
        this.range(_).domain(O);
        break;
    }
    return this;
  }
  function initInterpolator(O, _) {
    switch (arguments.length) {
      case 0:
        break;
      case 1: {
        typeof O == "function" ? this.interpolator(O) : this.range(O);
        break;
      }
      default: {
        this.domain(O), typeof _ == "function" ? this.interpolator(_) : this.range(_);
        break;
      }
    }
    return this;
  }
  const implicit = Symbol("implicit");
  function ordinal() {
    var O = new InternMap(), _ = [], H = [], V = implicit;
    function q(G) {
      let J = O.get(G);
      if (J === void 0) {
        if (V !== implicit) return V;
        O.set(G, J = _.push(G) - 1);
      }
      return H[J % H.length];
    }
    return q.domain = function(G) {
      if (!arguments.length) return _.slice();
      _ = [], O = new InternMap();
      for (const J of G)
        O.has(J) || O.set(J, _.push(J) - 1);
      return q;
    }, q.range = function(G) {
      return arguments.length ? (H = Array.from(G), q) : H.slice();
    }, q.unknown = function(G) {
      return arguments.length ? (V = G, q) : V;
    }, q.copy = function() {
      return ordinal(_, H).unknown(V);
    }, initRange.apply(q, arguments), q;
  }
  function define(O, _, H) {
    O.prototype = _.prototype = H, H.constructor = O;
  }
  function extend$1(O, _) {
    var H = Object.create(O.prototype);
    for (var V in _) H[V] = _[V];
    return H;
  }
  function Color() {
  }
  var darker = 0.7, brighter = 1 / darker, reI = "\\s*([+-]?\\d+)\\s*", reN = "\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)\\s*", reP = "\\s*([+-]?(?:\\d*\\.)?\\d+(?:[eE][+-]?\\d+)?)%\\s*", reHex = /^#([0-9a-f]{3,8})$/, reRgbInteger = new RegExp(`^rgb\\(${reI},${reI},${reI}\\)$`), reRgbPercent = new RegExp(`^rgb\\(${reP},${reP},${reP}\\)$`), reRgbaInteger = new RegExp(`^rgba\\(${reI},${reI},${reI},${reN}\\)$`), reRgbaPercent = new RegExp(`^rgba\\(${reP},${reP},${reP},${reN}\\)$`), reHslPercent = new RegExp(`^hsl\\(${reN},${reP},${reP}\\)$`), reHslaPercent = new RegExp(`^hsla\\(${reN},${reP},${reP},${reN}\\)$`), named = {
    aliceblue: 15792383,
    antiquewhite: 16444375,
    aqua: 65535,
    aquamarine: 8388564,
    azure: 15794175,
    beige: 16119260,
    bisque: 16770244,
    black: 0,
    blanchedalmond: 16772045,
    blue: 255,
    blueviolet: 9055202,
    brown: 10824234,
    burlywood: 14596231,
    cadetblue: 6266528,
    chartreuse: 8388352,
    chocolate: 13789470,
    coral: 16744272,
    cornflowerblue: 6591981,
    cornsilk: 16775388,
    crimson: 14423100,
    cyan: 65535,
    darkblue: 139,
    darkcyan: 35723,
    darkgoldenrod: 12092939,
    darkgray: 11119017,
    darkgreen: 25600,
    darkgrey: 11119017,
    darkkhaki: 12433259,
    darkmagenta: 9109643,
    darkolivegreen: 5597999,
    darkorange: 16747520,
    darkorchid: 10040012,
    darkred: 9109504,
    darksalmon: 15308410,
    darkseagreen: 9419919,
    darkslateblue: 4734347,
    darkslategray: 3100495,
    darkslategrey: 3100495,
    darkturquoise: 52945,
    darkviolet: 9699539,
    deeppink: 16716947,
    deepskyblue: 49151,
    dimgray: 6908265,
    dimgrey: 6908265,
    dodgerblue: 2003199,
    firebrick: 11674146,
    floralwhite: 16775920,
    forestgreen: 2263842,
    fuchsia: 16711935,
    gainsboro: 14474460,
    ghostwhite: 16316671,
    gold: 16766720,
    goldenrod: 14329120,
    gray: 8421504,
    green: 32768,
    greenyellow: 11403055,
    grey: 8421504,
    honeydew: 15794160,
    hotpink: 16738740,
    indianred: 13458524,
    indigo: 4915330,
    ivory: 16777200,
    khaki: 15787660,
    lavender: 15132410,
    lavenderblush: 16773365,
    lawngreen: 8190976,
    lemonchiffon: 16775885,
    lightblue: 11393254,
    lightcoral: 15761536,
    lightcyan: 14745599,
    lightgoldenrodyellow: 16448210,
    lightgray: 13882323,
    lightgreen: 9498256,
    lightgrey: 13882323,
    lightpink: 16758465,
    lightsalmon: 16752762,
    lightseagreen: 2142890,
    lightskyblue: 8900346,
    lightslategray: 7833753,
    lightslategrey: 7833753,
    lightsteelblue: 11584734,
    lightyellow: 16777184,
    lime: 65280,
    limegreen: 3329330,
    linen: 16445670,
    magenta: 16711935,
    maroon: 8388608,
    mediumaquamarine: 6737322,
    mediumblue: 205,
    mediumorchid: 12211667,
    mediumpurple: 9662683,
    mediumseagreen: 3978097,
    mediumslateblue: 8087790,
    mediumspringgreen: 64154,
    mediumturquoise: 4772300,
    mediumvioletred: 13047173,
    midnightblue: 1644912,
    mintcream: 16121850,
    mistyrose: 16770273,
    moccasin: 16770229,
    navajowhite: 16768685,
    navy: 128,
    oldlace: 16643558,
    olive: 8421376,
    olivedrab: 7048739,
    orange: 16753920,
    orangered: 16729344,
    orchid: 14315734,
    palegoldenrod: 15657130,
    palegreen: 10025880,
    paleturquoise: 11529966,
    palevioletred: 14381203,
    papayawhip: 16773077,
    peachpuff: 16767673,
    peru: 13468991,
    pink: 16761035,
    plum: 14524637,
    powderblue: 11591910,
    purple: 8388736,
    rebeccapurple: 6697881,
    red: 16711680,
    rosybrown: 12357519,
    royalblue: 4286945,
    saddlebrown: 9127187,
    salmon: 16416882,
    sandybrown: 16032864,
    seagreen: 3050327,
    seashell: 16774638,
    sienna: 10506797,
    silver: 12632256,
    skyblue: 8900331,
    slateblue: 6970061,
    slategray: 7372944,
    slategrey: 7372944,
    snow: 16775930,
    springgreen: 65407,
    steelblue: 4620980,
    tan: 13808780,
    teal: 32896,
    thistle: 14204888,
    tomato: 16737095,
    turquoise: 4251856,
    violet: 15631086,
    wheat: 16113331,
    white: 16777215,
    whitesmoke: 16119285,
    yellow: 16776960,
    yellowgreen: 10145074
  };
  define(Color, color$3, {
    copy(O) {
      return Object.assign(new this.constructor(), this, O);
    },
    displayable() {
      return this.rgb().displayable();
    },
    hex: color_formatHex,
    // Deprecated! Use color.formatHex.
    formatHex: color_formatHex,
    formatHex8: color_formatHex8,
    formatHsl: color_formatHsl,
    formatRgb: color_formatRgb,
    toString: color_formatRgb
  });
  function color_formatHex() {
    return this.rgb().formatHex();
  }
  function color_formatHex8() {
    return this.rgb().formatHex8();
  }
  function color_formatHsl() {
    return hslConvert(this).formatHsl();
  }
  function color_formatRgb() {
    return this.rgb().formatRgb();
  }
  function color$3(O) {
    var _, H;
    return O = (O + "").trim().toLowerCase(), (_ = reHex.exec(O)) ? (H = _[1].length, _ = parseInt(_[1], 16), H === 6 ? rgbn(_) : H === 3 ? new Rgb(_ >> 8 & 15 | _ >> 4 & 240, _ >> 4 & 15 | _ & 240, (_ & 15) << 4 | _ & 15, 1) : H === 8 ? rgba(_ >> 24 & 255, _ >> 16 & 255, _ >> 8 & 255, (_ & 255) / 255) : H === 4 ? rgba(_ >> 12 & 15 | _ >> 8 & 240, _ >> 8 & 15 | _ >> 4 & 240, _ >> 4 & 15 | _ & 240, ((_ & 15) << 4 | _ & 15) / 255) : null) : (_ = reRgbInteger.exec(O)) ? new Rgb(_[1], _[2], _[3], 1) : (_ = reRgbPercent.exec(O)) ? new Rgb(_[1] * 255 / 100, _[2] * 255 / 100, _[3] * 255 / 100, 1) : (_ = reRgbaInteger.exec(O)) ? rgba(_[1], _[2], _[3], _[4]) : (_ = reRgbaPercent.exec(O)) ? rgba(_[1] * 255 / 100, _[2] * 255 / 100, _[3] * 255 / 100, _[4]) : (_ = reHslPercent.exec(O)) ? hsla(_[1], _[2] / 100, _[3] / 100, 1) : (_ = reHslaPercent.exec(O)) ? hsla(_[1], _[2] / 100, _[3] / 100, _[4]) : named.hasOwnProperty(O) ? rgbn(named[O]) : O === "transparent" ? new Rgb(NaN, NaN, NaN, 0) : null;
  }
  function rgbn(O) {
    return new Rgb(O >> 16 & 255, O >> 8 & 255, O & 255, 1);
  }
  function rgba(O, _, H, V) {
    return V <= 0 && (O = _ = H = NaN), new Rgb(O, _, H, V);
  }
  function rgbConvert(O) {
    return O instanceof Color || (O = color$3(O)), O ? (O = O.rgb(), new Rgb(O.r, O.g, O.b, O.opacity)) : new Rgb();
  }
  function rgb$1(O, _, H, V) {
    return arguments.length === 1 ? rgbConvert(O) : new Rgb(O, _, H, V ?? 1);
  }
  function Rgb(O, _, H, V) {
    this.r = +O, this.g = +_, this.b = +H, this.opacity = +V;
  }
  define(Rgb, rgb$1, extend$1(Color, {
    brighter(O) {
      return O = O == null ? brighter : Math.pow(brighter, O), new Rgb(this.r * O, this.g * O, this.b * O, this.opacity);
    },
    darker(O) {
      return O = O == null ? darker : Math.pow(darker, O), new Rgb(this.r * O, this.g * O, this.b * O, this.opacity);
    },
    rgb() {
      return this;
    },
    clamp() {
      return new Rgb(clampi(this.r), clampi(this.g), clampi(this.b), clampa(this.opacity));
    },
    displayable() {
      return -0.5 <= this.r && this.r < 255.5 && -0.5 <= this.g && this.g < 255.5 && -0.5 <= this.b && this.b < 255.5 && 0 <= this.opacity && this.opacity <= 1;
    },
    hex: rgb_formatHex,
    // Deprecated! Use color.formatHex.
    formatHex: rgb_formatHex,
    formatHex8: rgb_formatHex8,
    formatRgb: rgb_formatRgb,
    toString: rgb_formatRgb
  }));
  function rgb_formatHex() {
    return `#${hex(this.r)}${hex(this.g)}${hex(this.b)}`;
  }
  function rgb_formatHex8() {
    return `#${hex(this.r)}${hex(this.g)}${hex(this.b)}${hex((isNaN(this.opacity) ? 1 : this.opacity) * 255)}`;
  }
  function rgb_formatRgb() {
    const O = clampa(this.opacity);
    return `${O === 1 ? "rgb(" : "rgba("}${clampi(this.r)}, ${clampi(this.g)}, ${clampi(this.b)}${O === 1 ? ")" : `, ${O})`}`;
  }
  function clampa(O) {
    return isNaN(O) ? 1 : Math.max(0, Math.min(1, O));
  }
  function clampi(O) {
    return Math.max(0, Math.min(255, Math.round(O) || 0));
  }
  function hex(O) {
    return O = clampi(O), (O < 16 ? "0" : "") + O.toString(16);
  }
  function hsla(O, _, H, V) {
    return V <= 0 ? O = _ = H = NaN : H <= 0 || H >= 1 ? O = _ = NaN : _ <= 0 && (O = NaN), new Hsl(O, _, H, V);
  }
  function hslConvert(O) {
    if (O instanceof Hsl) return new Hsl(O.h, O.s, O.l, O.opacity);
    if (O instanceof Color || (O = color$3(O)), !O) return new Hsl();
    if (O instanceof Hsl) return O;
    O = O.rgb();
    var _ = O.r / 255, H = O.g / 255, V = O.b / 255, q = Math.min(_, H, V), G = Math.max(_, H, V), J = NaN, Z = G - q, te = (G + q) / 2;
    return Z ? (_ === G ? J = (H - V) / Z + (H < V) * 6 : H === G ? J = (V - _) / Z + 2 : J = (_ - H) / Z + 4, Z /= te < 0.5 ? G + q : 2 - G - q, J *= 60) : Z = te > 0 && te < 1 ? 0 : J, new Hsl(J, Z, te, O.opacity);
  }
  function hsl$2(O, _, H, V) {
    return arguments.length === 1 ? hslConvert(O) : new Hsl(O, _, H, V ?? 1);
  }
  function Hsl(O, _, H, V) {
    this.h = +O, this.s = +_, this.l = +H, this.opacity = +V;
  }
  define(Hsl, hsl$2, extend$1(Color, {
    brighter(O) {
      return O = O == null ? brighter : Math.pow(brighter, O), new Hsl(this.h, this.s, this.l * O, this.opacity);
    },
    darker(O) {
      return O = O == null ? darker : Math.pow(darker, O), new Hsl(this.h, this.s, this.l * O, this.opacity);
    },
    rgb() {
      var O = this.h % 360 + (this.h < 0) * 360, _ = isNaN(O) || isNaN(this.s) ? 0 : this.s, H = this.l, V = H + (H < 0.5 ? H : 1 - H) * _, q = 2 * H - V;
      return new Rgb(
        hsl2rgb(O >= 240 ? O - 240 : O + 120, q, V),
        hsl2rgb(O, q, V),
        hsl2rgb(O < 120 ? O + 240 : O - 120, q, V),
        this.opacity
      );
    },
    clamp() {
      return new Hsl(clamph(this.h), clampt(this.s), clampt(this.l), clampa(this.opacity));
    },
    displayable() {
      return (0 <= this.s && this.s <= 1 || isNaN(this.s)) && 0 <= this.l && this.l <= 1 && 0 <= this.opacity && this.opacity <= 1;
    },
    formatHsl() {
      const O = clampa(this.opacity);
      return `${O === 1 ? "hsl(" : "hsla("}${clamph(this.h)}, ${clampt(this.s) * 100}%, ${clampt(this.l) * 100}%${O === 1 ? ")" : `, ${O})`}`;
    }
  }));
  function clamph(O) {
    return O = (O || 0) % 360, O < 0 ? O + 360 : O;
  }
  function clampt(O) {
    return Math.max(0, Math.min(1, O || 0));
  }
  function hsl2rgb(O, _, H) {
    return (O < 60 ? _ + (H - _) * O / 60 : O < 180 ? H : O < 240 ? _ + (H - _) * (240 - O) / 60 : _) * 255;
  }
  const radians$1 = Math.PI / 180, degrees$2 = 180 / Math.PI, K = 18, Xn = 0.96422, Yn = 1, Zn = 0.82521, t0 = 4 / 29, t1 = 6 / 29, t2 = 3 * t1 * t1, t3 = t1 * t1 * t1;
  function labConvert(O) {
    if (O instanceof Lab) return new Lab(O.l, O.a, O.b, O.opacity);
    if (O instanceof Hcl) return hcl2lab(O);
    O instanceof Rgb || (O = rgbConvert(O));
    var _ = rgb2lrgb(O.r), H = rgb2lrgb(O.g), V = rgb2lrgb(O.b), q = xyz2lab((0.2225045 * _ + 0.7168786 * H + 0.0606169 * V) / Yn), G, J;
    return _ === H && H === V ? G = J = q : (G = xyz2lab((0.4360747 * _ + 0.3850649 * H + 0.1430804 * V) / Xn), J = xyz2lab((0.0139322 * _ + 0.0971045 * H + 0.7141733 * V) / Zn)), new Lab(116 * q - 16, 500 * (G - q), 200 * (q - J), O.opacity);
  }
  function lab$1(O, _, H, V) {
    return arguments.length === 1 ? labConvert(O) : new Lab(O, _, H, V ?? 1);
  }
  function Lab(O, _, H, V) {
    this.l = +O, this.a = +_, this.b = +H, this.opacity = +V;
  }
  define(Lab, lab$1, extend$1(Color, {
    brighter(O) {
      return new Lab(this.l + K * (O ?? 1), this.a, this.b, this.opacity);
    },
    darker(O) {
      return new Lab(this.l - K * (O ?? 1), this.a, this.b, this.opacity);
    },
    rgb() {
      var O = (this.l + 16) / 116, _ = isNaN(this.a) ? O : O + this.a / 500, H = isNaN(this.b) ? O : O - this.b / 200;
      return _ = Xn * lab2xyz(_), O = Yn * lab2xyz(O), H = Zn * lab2xyz(H), new Rgb(
        lrgb2rgb(3.1338561 * _ - 1.6168667 * O - 0.4906146 * H),
        lrgb2rgb(-0.9787684 * _ + 1.9161415 * O + 0.033454 * H),
        lrgb2rgb(0.0719453 * _ - 0.2289914 * O + 1.4052427 * H),
        this.opacity
      );
    }
  }));
  function xyz2lab(O) {
    return O > t3 ? Math.pow(O, 1 / 3) : O / t2 + t0;
  }
  function lab2xyz(O) {
    return O > t1 ? O * O * O : t2 * (O - t0);
  }
  function lrgb2rgb(O) {
    return 255 * (O <= 31308e-7 ? 12.92 * O : 1.055 * Math.pow(O, 1 / 2.4) - 0.055);
  }
  function rgb2lrgb(O) {
    return (O /= 255) <= 0.04045 ? O / 12.92 : Math.pow((O + 0.055) / 1.055, 2.4);
  }
  function hclConvert(O) {
    if (O instanceof Hcl) return new Hcl(O.h, O.c, O.l, O.opacity);
    if (O instanceof Lab || (O = labConvert(O)), O.a === 0 && O.b === 0) return new Hcl(NaN, 0 < O.l && O.l < 100 ? 0 : NaN, O.l, O.opacity);
    var _ = Math.atan2(O.b, O.a) * degrees$2;
    return new Hcl(_ < 0 ? _ + 360 : _, Math.sqrt(O.a * O.a + O.b * O.b), O.l, O.opacity);
  }
  function hcl$2(O, _, H, V) {
    return arguments.length === 1 ? hclConvert(O) : new Hcl(O, _, H, V ?? 1);
  }
  function Hcl(O, _, H, V) {
    this.h = +O, this.c = +_, this.l = +H, this.opacity = +V;
  }
  function hcl2lab(O) {
    if (isNaN(O.h)) return new Lab(O.l, 0, 0, O.opacity);
    var _ = O.h * radians$1;
    return new Lab(O.l, Math.cos(_) * O.c, Math.sin(_) * O.c, O.opacity);
  }
  define(Hcl, hcl$2, extend$1(Color, {
    brighter(O) {
      return new Hcl(this.h, this.c, this.l + K * (O ?? 1), this.opacity);
    },
    darker(O) {
      return new Hcl(this.h, this.c, this.l - K * (O ?? 1), this.opacity);
    },
    rgb() {
      return hcl2lab(this).rgb();
    }
  }));
  var A$1 = -0.14861, B$1 = 1.78277, C$2 = -0.29227, D$1 = -0.90649, E$1 = 1.97294, ED = E$1 * D$1, EB = E$1 * B$1, BC_DA = B$1 * C$2 - D$1 * A$1;
  function cubehelixConvert(O) {
    if (O instanceof Cubehelix) return new Cubehelix(O.h, O.s, O.l, O.opacity);
    O instanceof Rgb || (O = rgbConvert(O));
    var _ = O.r / 255, H = O.g / 255, V = O.b / 255, q = (BC_DA * V + ED * _ - EB * H) / (BC_DA + ED - EB), G = V - q, J = (E$1 * (H - q) - C$2 * G) / D$1, Z = Math.sqrt(J * J + G * G) / (E$1 * q * (1 - q)), te = Z ? Math.atan2(J, G) * degrees$2 - 120 : NaN;
    return new Cubehelix(te < 0 ? te + 360 : te, Z, q, O.opacity);
  }
  function cubehelix$2(O, _, H, V) {
    return arguments.length === 1 ? cubehelixConvert(O) : new Cubehelix(O, _, H, V ?? 1);
  }
  function Cubehelix(O, _, H, V) {
    this.h = +O, this.s = +_, this.l = +H, this.opacity = +V;
  }
  define(Cubehelix, cubehelix$2, extend$1(Color, {
    brighter(O) {
      return O = O == null ? brighter : Math.pow(brighter, O), new Cubehelix(this.h, this.s, this.l * O, this.opacity);
    },
    darker(O) {
      return O = O == null ? darker : Math.pow(darker, O), new Cubehelix(this.h, this.s, this.l * O, this.opacity);
    },
    rgb() {
      var O = isNaN(this.h) ? 0 : (this.h + 120) * radians$1, _ = +this.l, H = isNaN(this.s) ? 0 : this.s * _ * (1 - _), V = Math.cos(O), q = Math.sin(O);
      return new Rgb(
        255 * (_ + H * (A$1 * V + B$1 * q)),
        255 * (_ + H * (C$2 * V + D$1 * q)),
        255 * (_ + H * (E$1 * V)),
        this.opacity
      );
    }
  }));
  function basis(O, _, H, V, q) {
    var G = O * O, J = G * O;
    return ((1 - 3 * O + 3 * G - J) * _ + (4 - 6 * G + 3 * J) * H + (1 + 3 * O + 3 * G - 3 * J) * V + J * q) / 6;
  }
  function basis$1(O) {
    var _ = O.length - 1;
    return function(H) {
      var V = H <= 0 ? H = 0 : H >= 1 ? (H = 1, _ - 1) : Math.floor(H * _), q = O[V], G = O[V + 1], J = V > 0 ? O[V - 1] : 2 * q - G, Z = V < _ - 1 ? O[V + 2] : 2 * G - q;
      return basis((H - V / _) * _, J, q, G, Z);
    };
  }
  function basisClosed(O) {
    var _ = O.length;
    return function(H) {
      var V = Math.floor(((H %= 1) < 0 ? ++H : H) * _), q = O[(V + _ - 1) % _], G = O[V % _], J = O[(V + 1) % _], Z = O[(V + 2) % _];
      return basis((H - V / _) * _, q, G, J, Z);
    };
  }
  const constant$2 = (O) => () => O;
  function linear$1(O, _) {
    return function(H) {
      return O + H * _;
    };
  }
  function exponential(O, _, H) {
    return O = Math.pow(O, H), _ = Math.pow(_, H) - O, H = 1 / H, function(V) {
      return Math.pow(O + V * _, H);
    };
  }
  function hue$1(O, _) {
    var H = _ - O;
    return H ? linear$1(O, H > 180 || H < -180 ? H - 360 * Math.round(H / 360) : H) : constant$2(isNaN(O) ? _ : O);
  }
  function gamma(O) {
    return (O = +O) == 1 ? nogamma : function(_, H) {
      return H - _ ? exponential(_, H, O) : constant$2(isNaN(_) ? H : _);
    };
  }
  function nogamma(O, _) {
    var H = _ - O;
    return H ? linear$1(O, H) : constant$2(isNaN(O) ? _ : O);
  }
  const rgb = function O(_) {
    var H = gamma(_);
    function V(q, G) {
      var J = H((q = rgb$1(q)).r, (G = rgb$1(G)).r), Z = H(q.g, G.g), te = H(q.b, G.b), ne = nogamma(q.opacity, G.opacity);
      return function(ee) {
        return q.r = J(ee), q.g = Z(ee), q.b = te(ee), q.opacity = ne(ee), q + "";
      };
    }
    return V.gamma = O, V;
  }(1);
  function rgbSpline(O) {
    return function(_) {
      var H = _.length, V = new Array(H), q = new Array(H), G = new Array(H), J, Z;
      for (J = 0; J < H; ++J)
        Z = rgb$1(_[J]), V[J] = Z.r || 0, q[J] = Z.g || 0, G[J] = Z.b || 0;
      return V = O(V), q = O(q), G = O(G), Z.opacity = 1, function(te) {
        return Z.r = V(te), Z.g = q(te), Z.b = G(te), Z + "";
      };
    };
  }
  var rgbBasis = rgbSpline(basis$1), rgbBasisClosed = rgbSpline(basisClosed);
  function numberArray(O, _) {
    _ || (_ = []);
    var H = O ? Math.min(_.length, O.length) : 0, V = _.slice(), q;
    return function(G) {
      for (q = 0; q < H; ++q) V[q] = O[q] * (1 - G) + _[q] * G;
      return V;
    };
  }
  function isNumberArray(O) {
    return ArrayBuffer.isView(O) && !(O instanceof DataView);
  }
  function array$3(O, _) {
    return (isNumberArray(_) ? numberArray : genericArray)(O, _);
  }
  function genericArray(O, _) {
    var H = _ ? _.length : 0, V = O ? Math.min(H, O.length) : 0, q = new Array(V), G = new Array(H), J;
    for (J = 0; J < V; ++J) q[J] = interpolate$1$1(O[J], _[J]);
    for (; J < H; ++J) G[J] = _[J];
    return function(Z) {
      for (J = 0; J < V; ++J) G[J] = q[J](Z);
      return G;
    };
  }
  function date$1(O, _) {
    var H = /* @__PURE__ */ new Date();
    return O = +O, _ = +_, function(V) {
      return H.setTime(O * (1 - V) + _ * V), H;
    };
  }
  function interpolateNumber(O, _) {
    return O = +O, _ = +_, function(H) {
      return O * (1 - H) + _ * H;
    };
  }
  function object(O, _) {
    var H = {}, V = {}, q;
    (O === null || typeof O != "object") && (O = {}), (_ === null || typeof _ != "object") && (_ = {});
    for (q in _)
      q in O ? H[q] = interpolate$1$1(O[q], _[q]) : V[q] = _[q];
    return function(G) {
      for (q in H) V[q] = H[q](G);
      return V;
    };
  }
  var reA = /[-+]?(?:\d+\.?\d*|\.?\d+)(?:[eE][-+]?\d+)?/g, reB = new RegExp(reA.source, "g");
  function zero$2(O) {
    return function() {
      return O;
    };
  }
  function one$1(O) {
    return function(_) {
      return O(_) + "";
    };
  }
  function string$1(O, _) {
    var H = reA.lastIndex = reB.lastIndex = 0, V, q, G, J = -1, Z = [], te = [];
    for (O = O + "", _ = _ + ""; (V = reA.exec(O)) && (q = reB.exec(_)); )
      (G = q.index) > H && (G = _.slice(H, G), Z[J] ? Z[J] += G : Z[++J] = G), (V = V[0]) === (q = q[0]) ? Z[J] ? Z[J] += q : Z[++J] = q : (Z[++J] = null, te.push({ i: J, x: interpolateNumber(V, q) })), H = reB.lastIndex;
    return H < _.length && (G = _.slice(H), Z[J] ? Z[J] += G : Z[++J] = G), Z.length < 2 ? te[0] ? one$1(te[0].x) : zero$2(_) : (_ = te.length, function(ne) {
      for (var ee = 0, ae; ee < _; ++ee) Z[(ae = te[ee]).i] = ae.x(ne);
      return Z.join("");
    });
  }
  function interpolate$1$1(O, _) {
    var H = typeof _, V;
    return _ == null || H === "boolean" ? constant$2(_) : (H === "number" ? interpolateNumber : H === "string" ? (V = color$3(_)) ? (_ = V, rgb) : string$1 : _ instanceof color$3 ? rgb : _ instanceof Date ? date$1 : isNumberArray(_) ? numberArray : Array.isArray(_) ? genericArray : typeof _.valueOf != "function" && typeof _.toString != "function" || isNaN(_) ? object : interpolateNumber)(O, _);
  }
  function discrete$1(O) {
    var _ = O.length;
    return function(H) {
      return O[Math.max(0, Math.min(_ - 1, Math.floor(H * _)))];
    };
  }
  function hue(O, _) {
    var H = hue$1(+O, +_);
    return function(V) {
      var q = H(V);
      return q - 360 * Math.floor(q / 360);
    };
  }
  function interpolateRound(O, _) {
    return O = +O, _ = +_, function(H) {
      return Math.round(O * (1 - H) + _ * H);
    };
  }
  var degrees$1 = 180 / Math.PI, identity$4 = {
    translateX: 0,
    translateY: 0,
    rotate: 0,
    skewX: 0,
    scaleX: 1,
    scaleY: 1
  };
  function decompose(O, _, H, V, q, G) {
    var J, Z, te;
    return (J = Math.sqrt(O * O + _ * _)) && (O /= J, _ /= J), (te = O * H + _ * V) && (H -= O * te, V -= _ * te), (Z = Math.sqrt(H * H + V * V)) && (H /= Z, V /= Z, te /= Z), O * V < _ * H && (O = -O, _ = -_, te = -te, J = -J), {
      translateX: q,
      translateY: G,
      rotate: Math.atan2(_, O) * degrees$1,
      skewX: Math.atan(te) * degrees$1,
      scaleX: J,
      scaleY: Z
    };
  }
  var svgNode;
  function parseCss(O) {
    const _ = new (typeof DOMMatrix == "function" ? DOMMatrix : WebKitCSSMatrix)(O + "");
    return _.isIdentity ? identity$4 : decompose(_.a, _.b, _.c, _.d, _.e, _.f);
  }
  function parseSvg(O) {
    return O == null || (svgNode || (svgNode = document.createElementNS("http://www.w3.org/2000/svg", "g")), svgNode.setAttribute("transform", O), !(O = svgNode.transform.baseVal.consolidate())) ? identity$4 : (O = O.matrix, decompose(O.a, O.b, O.c, O.d, O.e, O.f));
  }
  function interpolateTransform(O, _, H, V) {
    function q(ne) {
      return ne.length ? ne.pop() + " " : "";
    }
    function G(ne, ee, ae, oe, se, ce) {
      if (ne !== ae || ee !== oe) {
        var le = se.push("translate(", null, _, null, H);
        ce.push({ i: le - 4, x: interpolateNumber(ne, ae) }, { i: le - 2, x: interpolateNumber(ee, oe) });
      } else (ae || oe) && se.push("translate(" + ae + _ + oe + H);
    }
    function J(ne, ee, ae, oe) {
      ne !== ee ? (ne - ee > 180 ? ee += 360 : ee - ne > 180 && (ne += 360), oe.push({ i: ae.push(q(ae) + "rotate(", null, V) - 2, x: interpolateNumber(ne, ee) })) : ee && ae.push(q(ae) + "rotate(" + ee + V);
    }
    function Z(ne, ee, ae, oe) {
      ne !== ee ? oe.push({ i: ae.push(q(ae) + "skewX(", null, V) - 2, x: interpolateNumber(ne, ee) }) : ee && ae.push(q(ae) + "skewX(" + ee + V);
    }
    function te(ne, ee, ae, oe, se, ce) {
      if (ne !== ae || ee !== oe) {
        var le = se.push(q(se) + "scale(", null, ",", null, ")");
        ce.push({ i: le - 4, x: interpolateNumber(ne, ae) }, { i: le - 2, x: interpolateNumber(ee, oe) });
      } else (ae !== 1 || oe !== 1) && se.push(q(se) + "scale(" + ae + "," + oe + ")");
    }
    return function(ne, ee) {
      var ae = [], oe = [];
      return ne = O(ne), ee = O(ee), G(ne.translateX, ne.translateY, ee.translateX, ee.translateY, ae, oe), J(ne.rotate, ee.rotate, ae, oe), Z(ne.skewX, ee.skewX, ae, oe), te(ne.scaleX, ne.scaleY, ee.scaleX, ee.scaleY, ae, oe), ne = ee = null, function(se) {
        for (var ce = -1, le = oe.length, ue; ++ce < le; ) ae[(ue = oe[ce]).i] = ue.x(se);
        return ae.join("");
      };
    };
  }
  var interpolateTransformCss = interpolateTransform(parseCss, "px, ", "px)", "deg)"), interpolateTransformSvg = interpolateTransform(parseSvg, ", ", ")", ")"), epsilon2$1 = 1e-12;
  function cosh(O) {
    return ((O = Math.exp(O)) + 1 / O) / 2;
  }
  function sinh(O) {
    return ((O = Math.exp(O)) - 1 / O) / 2;
  }
  function tanh(O) {
    return ((O = Math.exp(2 * O)) - 1) / (O + 1);
  }
  const zoom$1 = function O(_, H, V) {
    function q(G, J) {
      var Z = G[0], te = G[1], ne = G[2], ee = J[0], ae = J[1], oe = J[2], se = ee - Z, ce = ae - te, le = se * se + ce * ce, ue, de;
      if (le < epsilon2$1)
        de = Math.log(oe / ne) / _, ue = function(ge) {
          return [
            Z + ge * se,
            te + ge * ce,
            ne * Math.exp(_ * ge * de)
          ];
        };
      else {
        var fe = Math.sqrt(le), he = (oe * oe - ne * ne + V * le) / (2 * ne * H * fe), me = (oe * oe - ne * ne - V * le) / (2 * oe * H * fe), pe = Math.log(Math.sqrt(he * he + 1) - he), ye = Math.log(Math.sqrt(me * me + 1) - me);
        de = (ye - pe) / _, ue = function(ge) {
          var ve = ge * de, be = cosh(pe), Ce = ne / (H * fe) * (be * tanh(_ * ve + pe) - sinh(pe));
          return [
            Z + Ce * se,
            te + Ce * ce,
            ne * be / cosh(_ * ve + pe)
          ];
        };
      }
      return ue.duration = de * 1e3 * _ / Math.SQRT2, ue;
    }
    return q.rho = function(G) {
      var J = Math.max(1e-3, +G), Z = J * J, te = Z * Z;
      return O(J, Z, te);
    }, q;
  }(Math.SQRT2, 2, 4);
  function hsl(O) {
    return function(_, H) {
      var V = O((_ = hsl$2(_)).h, (H = hsl$2(H)).h), q = nogamma(_.s, H.s), G = nogamma(_.l, H.l), J = nogamma(_.opacity, H.opacity);
      return function(Z) {
        return _.h = V(Z), _.s = q(Z), _.l = G(Z), _.opacity = J(Z), _ + "";
      };
    };
  }
  const hsl$1 = hsl(hue$1);
  var hslLong = hsl(nogamma);
  function lab(O, _) {
    var H = nogamma((O = lab$1(O)).l, (_ = lab$1(_)).l), V = nogamma(O.a, _.a), q = nogamma(O.b, _.b), G = nogamma(O.opacity, _.opacity);
    return function(J) {
      return O.l = H(J), O.a = V(J), O.b = q(J), O.opacity = G(J), O + "";
    };
  }
  function hcl(O) {
    return function(_, H) {
      var V = O((_ = hcl$2(_)).h, (H = hcl$2(H)).h), q = nogamma(_.c, H.c), G = nogamma(_.l, H.l), J = nogamma(_.opacity, H.opacity);
      return function(Z) {
        return _.h = V(Z), _.c = q(Z), _.l = G(Z), _.opacity = J(Z), _ + "";
      };
    };
  }
  const hcl$1 = hcl(hue$1);
  var hclLong = hcl(nogamma);
  function cubehelix(O) {
    return function _(H) {
      H = +H;
      function V(q, G) {
        var J = O((q = cubehelix$2(q)).h, (G = cubehelix$2(G)).h), Z = nogamma(q.s, G.s), te = nogamma(q.l, G.l), ne = nogamma(q.opacity, G.opacity);
        return function(ee) {
          return q.h = J(ee), q.s = Z(ee), q.l = te(Math.pow(ee, H)), q.opacity = ne(ee), q + "";
        };
      }
      return V.gamma = _, V;
    }(1);
  }
  const cubehelix$1 = cubehelix(hue$1);
  var cubehelixLong = cubehelix(nogamma);
  function piecewise(O, _) {
    _ === void 0 && (_ = O, O = interpolate$1$1);
    for (var H = 0, V = _.length - 1, q = _[0], G = new Array(V < 0 ? 0 : V); H < V; ) G[H] = O(q, q = _[++H]);
    return function(J) {
      var Z = Math.max(0, Math.min(V - 1, Math.floor(J *= V)));
      return G[Z](J - Z);
    };
  }
  function quantize$2(O, _) {
    for (var H = new Array(_), V = 0; V < _; ++V) H[V] = O(V / (_ - 1));
    return H;
  }
  const $$1$1 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    interpolate: interpolate$1$1,
    interpolateArray: array$3,
    interpolateBasis: basis$1,
    interpolateBasisClosed: basisClosed,
    interpolateCubehelix: cubehelix$1,
    interpolateCubehelixLong: cubehelixLong,
    interpolateDate: date$1,
    interpolateDiscrete: discrete$1,
    interpolateHcl: hcl$1,
    interpolateHclLong: hclLong,
    interpolateHsl: hsl$1,
    interpolateHslLong: hslLong,
    interpolateHue: hue,
    interpolateLab: lab,
    interpolateNumber,
    interpolateNumberArray: numberArray,
    interpolateObject: object,
    interpolateRgb: rgb,
    interpolateRgbBasis: rgbBasis,
    interpolateRgbBasisClosed: rgbBasisClosed,
    interpolateRound,
    interpolateString: string$1,
    interpolateTransformCss,
    interpolateTransformSvg,
    interpolateZoom: zoom$1,
    piecewise,
    quantize: quantize$2
  }, Symbol.toStringTag, { value: "Module" }));
  function constants$1(O) {
    return function() {
      return O;
    };
  }
  function number$6(O) {
    return +O;
  }
  var unit = [0, 1];
  function identity$3(O) {
    return O;
  }
  function normalize$2(O, _) {
    return (_ -= O = +O) ? function(H) {
      return (H - O) / _;
    } : constants$1(isNaN(_) ? NaN : 0.5);
  }
  function clamper(O, _) {
    var H;
    return O > _ && (H = O, O = _, _ = H), function(V) {
      return Math.max(O, Math.min(_, V));
    };
  }
  function bimap(O, _, H) {
    var V = O[0], q = O[1], G = _[0], J = _[1];
    return q < V ? (V = normalize$2(q, V), G = H(J, G)) : (V = normalize$2(V, q), G = H(G, J)), function(Z) {
      return G(V(Z));
    };
  }
  function polymap(O, _, H) {
    var V = Math.min(O.length, _.length) - 1, q = new Array(V), G = new Array(V), J = -1;
    for (O[V] < O[0] && (O = O.slice().reverse(), _ = _.slice().reverse()); ++J < V; )
      q[J] = normalize$2(O[J], O[J + 1]), G[J] = H(_[J], _[J + 1]);
    return function(Z) {
      var te = bisectRight$1(O, Z, 1, V) - 1;
      return G[te](q[te](Z));
    };
  }
  function copy$2(O, _) {
    return _.domain(O.domain()).range(O.range()).interpolate(O.interpolate()).clamp(O.clamp()).unknown(O.unknown());
  }
  function transformer$3() {
    var O = unit, _ = unit, H = interpolate$1$1, V, q, G, J = identity$3, Z, te, ne;
    function ee() {
      var oe = Math.min(O.length, _.length);
      return J !== identity$3 && (J = clamper(O[0], O[oe - 1])), Z = oe > 2 ? polymap : bimap, te = ne = null, ae;
    }
    function ae(oe) {
      return oe == null || isNaN(oe = +oe) ? G : (te || (te = Z(O.map(V), _, H)))(V(J(oe)));
    }
    return ae.invert = function(oe) {
      return J(q((ne || (ne = Z(_, O.map(V), interpolateNumber)))(oe)));
    }, ae.domain = function(oe) {
      return arguments.length ? (O = Array.from(oe, number$6), ee()) : O.slice();
    }, ae.range = function(oe) {
      return arguments.length ? (_ = Array.from(oe), ee()) : _.slice();
    }, ae.rangeRound = function(oe) {
      return _ = Array.from(oe), H = interpolateRound, ee();
    }, ae.clamp = function(oe) {
      return arguments.length ? (J = oe ? !0 : identity$3, ee()) : J !== identity$3;
    }, ae.interpolate = function(oe) {
      return arguments.length ? (H = oe, ee()) : H;
    }, ae.unknown = function(oe) {
      return arguments.length ? (G = oe, ae) : G;
    }, function(oe, se) {
      return V = oe, q = se, ee();
    };
  }
  function continuous$1() {
    return transformer$3()(identity$3, identity$3);
  }
  function tickFormat$1(O, _, H, V) {
    var q = tickStep(O, _, H), G;
    switch (V = formatSpecifier(V ?? ",f"), V.type) {
      case "s": {
        var J = Math.max(Math.abs(O), Math.abs(_));
        return V.precision == null && !isNaN(G = precisionPrefix(q, J)) && (V.precision = G), formatPrefix(V, J);
      }
      case "":
      case "e":
      case "g":
      case "p":
      case "r": {
        V.precision == null && !isNaN(G = precisionRound(q, Math.max(Math.abs(O), Math.abs(_)))) && (V.precision = G - (V.type === "e"));
        break;
      }
      case "f":
      case "%": {
        V.precision == null && !isNaN(G = precisionFixed(q)) && (V.precision = G - (V.type === "%") * 2);
        break;
      }
    }
    return format$3(V);
  }
  function linearish(O) {
    var _ = O.domain;
    return O.ticks = function(H) {
      var V = _();
      return ticks(V[0], V[V.length - 1], H ?? 10);
    }, O.tickFormat = function(H, V) {
      var q = _();
      return tickFormat$1(q[0], q[q.length - 1], H ?? 10, V);
    }, O.nice = function(H) {
      H == null && (H = 10);
      var V = _(), q = 0, G = V.length - 1, J = V[q], Z = V[G], te, ne, ee = 10;
      for (Z < J && (ne = J, J = Z, Z = ne, ne = q, q = G, G = ne); ee-- > 0; ) {
        if (ne = tickIncrement(J, Z, H), ne === te)
          return V[q] = J, V[G] = Z, _(V);
        if (ne > 0)
          J = Math.floor(J / ne) * ne, Z = Math.ceil(Z / ne) * ne;
        else if (ne < 0)
          J = Math.ceil(J * ne) / ne, Z = Math.floor(Z * ne) / ne;
        else
          break;
        te = ne;
      }
      return O;
    }, O;
  }
  function linear() {
    var O = continuous$1();
    return O.copy = function() {
      return copy$2(O, linear());
    }, initRange.apply(O, arguments), linearish(O);
  }
  function identity$2(O) {
    var _;
    function H(V) {
      return V == null || isNaN(V = +V) ? _ : V;
    }
    return H.invert = H, H.domain = H.range = function(V) {
      return arguments.length ? (O = Array.from(V, number$6), H) : O.slice();
    }, H.unknown = function(V) {
      return arguments.length ? (_ = V, H) : _;
    }, H.copy = function() {
      return identity$2(O).unknown(_);
    }, O = arguments.length ? Array.from(O, number$6) : [0, 1], linearish(H);
  }
  function nice$1(O, _) {
    O = O.slice();
    var H = 0, V = O.length - 1, q = O[H], G = O[V], J;
    return G < q && (J = H, H = V, V = J, J = q, q = G, G = J), O[H] = _.floor(q), O[V] = _.ceil(G), O;
  }
  function transformLog(O) {
    return Math.log(O);
  }
  function transformExp(O) {
    return Math.exp(O);
  }
  function transformLogn(O) {
    return -Math.log(-O);
  }
  function transformExpn(O) {
    return -Math.exp(-O);
  }
  function pow10(O) {
    return isFinite(O) ? +("1e" + O) : O < 0 ? 0 : O;
  }
  function powp(O) {
    return O === 10 ? pow10 : O === Math.E ? Math.exp : (_) => Math.pow(O, _);
  }
  function logp(O) {
    return O === Math.E ? Math.log : O === 10 && Math.log10 || O === 2 && Math.log2 || (O = Math.log(O), (_) => Math.log(_) / O);
  }
  function reflect(O) {
    return (_, H) => -O(-_, H);
  }
  function loggish(O) {
    const _ = O(transformLog, transformExp), H = _.domain;
    let V = 10, q, G;
    function J() {
      return q = logp(V), G = powp(V), H()[0] < 0 ? (q = reflect(q), G = reflect(G), O(transformLogn, transformExpn)) : O(transformLog, transformExp), _;
    }
    return _.base = function(Z) {
      return arguments.length ? (V = +Z, J()) : V;
    }, _.domain = function(Z) {
      return arguments.length ? (H(Z), J()) : H();
    }, _.ticks = (Z) => {
      const te = H();
      let ne = te[0], ee = te[te.length - 1];
      const ae = ee < ne;
      ae && ([ne, ee] = [ee, ne]);
      let oe = q(ne), se = q(ee), ce, le;
      const ue = Z == null ? 10 : +Z;
      let de = [];
      if (!(V % 1) && se - oe < ue) {
        if (oe = Math.floor(oe), se = Math.ceil(se), ne > 0) {
          for (; oe <= se; ++oe)
            for (ce = 1; ce < V; ++ce)
              if (le = oe < 0 ? ce / G(-oe) : ce * G(oe), !(le < ne)) {
                if (le > ee) break;
                de.push(le);
              }
        } else for (; oe <= se; ++oe)
          for (ce = V - 1; ce >= 1; --ce)
            if (le = oe > 0 ? ce / G(-oe) : ce * G(oe), !(le < ne)) {
              if (le > ee) break;
              de.push(le);
            }
        de.length * 2 < ue && (de = ticks(ne, ee, ue));
      } else
        de = ticks(oe, se, Math.min(se - oe, ue)).map(G);
      return ae ? de.reverse() : de;
    }, _.tickFormat = (Z, te) => {
      if (Z == null && (Z = 10), te == null && (te = V === 10 ? "s" : ","), typeof te != "function" && (!(V % 1) && (te = formatSpecifier(te)).precision == null && (te.trim = !0), te = format$3(te)), Z === 1 / 0) return te;
      const ne = Math.max(1, V * Z / _.ticks().length);
      return (ee) => {
        let ae = ee / G(Math.round(q(ee)));
        return ae * V < V - 0.5 && (ae *= V), ae <= ne ? te(ee) : "";
      };
    }, _.nice = () => H(nice$1(H(), {
      floor: (Z) => G(Math.floor(q(Z))),
      ceil: (Z) => G(Math.ceil(q(Z)))
    })), _;
  }
  function log$2() {
    const O = loggish(transformer$3()).domain([1, 10]);
    return O.copy = () => copy$2(O, log$2()).base(O.base()), initRange.apply(O, arguments), O;
  }
  function transformSymlog(O) {
    return function(_) {
      return Math.sign(_) * Math.log1p(Math.abs(_ / O));
    };
  }
  function transformSymexp(O) {
    return function(_) {
      return Math.sign(_) * Math.expm1(Math.abs(_)) * O;
    };
  }
  function symlogish(O) {
    var _ = 1, H = O(transformSymlog(_), transformSymexp(_));
    return H.constant = function(V) {
      return arguments.length ? O(transformSymlog(_ = +V), transformSymexp(_)) : _;
    }, linearish(H);
  }
  function symlog() {
    var O = symlogish(transformer$3());
    return O.copy = function() {
      return copy$2(O, symlog()).constant(O.constant());
    }, initRange.apply(O, arguments);
  }
  function transformPow(O) {
    return function(_) {
      return _ < 0 ? -Math.pow(-_, O) : Math.pow(_, O);
    };
  }
  function transformSqrt(O) {
    return O < 0 ? -Math.sqrt(-O) : Math.sqrt(O);
  }
  function transformSquare(O) {
    return O < 0 ? -O * O : O * O;
  }
  function powish(O) {
    var _ = O(identity$3, identity$3), H = 1;
    function V() {
      return H === 1 ? O(identity$3, identity$3) : H === 0.5 ? O(transformSqrt, transformSquare) : O(transformPow(H), transformPow(1 / H));
    }
    return _.exponent = function(q) {
      return arguments.length ? (H = +q, V()) : H;
    }, linearish(_);
  }
  function pow$2() {
    var O = powish(transformer$3());
    return O.copy = function() {
      return copy$2(O, pow$2()).exponent(O.exponent());
    }, initRange.apply(O, arguments), O;
  }
  function sqrt$2() {
    return pow$2.apply(null, arguments).exponent(0.5);
  }
  function quantile() {
    var O = [], _ = [], H = [], V;
    function q() {
      var J = 0, Z = Math.max(1, _.length);
      for (H = new Array(Z - 1); ++J < Z; ) H[J - 1] = quantileSorted(O, J / Z);
      return G;
    }
    function G(J) {
      return J == null || isNaN(J = +J) ? V : _[bisectRight$1(H, J)];
    }
    return G.invertExtent = function(J) {
      var Z = _.indexOf(J);
      return Z < 0 ? [NaN, NaN] : [
        Z > 0 ? H[Z - 1] : O[0],
        Z < H.length ? H[Z] : O[O.length - 1]
      ];
    }, G.domain = function(J) {
      if (!arguments.length) return O.slice();
      O = [];
      for (let Z of J) Z != null && !isNaN(Z = +Z) && O.push(Z);
      return O.sort(ascending$1), q();
    }, G.range = function(J) {
      return arguments.length ? (_ = Array.from(J), q()) : _.slice();
    }, G.unknown = function(J) {
      return arguments.length ? (V = J, G) : V;
    }, G.quantiles = function() {
      return H.slice();
    }, G.copy = function() {
      return quantile().domain(O).range(_).unknown(V);
    }, initRange.apply(G, arguments);
  }
  function quantize$1() {
    var O = 0, _ = 1, H = 1, V = [0.5], q = [0, 1], G;
    function J(te) {
      return te != null && te <= te ? q[bisectRight$1(V, te, 0, H)] : G;
    }
    function Z() {
      var te = -1;
      for (V = new Array(H); ++te < H; ) V[te] = ((te + 1) * _ - (te - H) * O) / (H + 1);
      return J;
    }
    return J.domain = function(te) {
      return arguments.length ? ([O, _] = te, O = +O, _ = +_, Z()) : [O, _];
    }, J.range = function(te) {
      return arguments.length ? (H = (q = Array.from(te)).length - 1, Z()) : q.slice();
    }, J.invertExtent = function(te) {
      var ne = q.indexOf(te);
      return ne < 0 ? [NaN, NaN] : ne < 1 ? [O, V[0]] : ne >= H ? [V[H - 1], _] : [V[ne - 1], V[ne]];
    }, J.unknown = function(te) {
      return arguments.length && (G = te), J;
    }, J.thresholds = function() {
      return V.slice();
    }, J.copy = function() {
      return quantize$1().domain([O, _]).range(q).unknown(G);
    }, initRange.apply(linearish(J), arguments);
  }
  function threshold() {
    var O = [0.5], _ = [0, 1], H, V = 1;
    function q(G) {
      return G != null && G <= G ? _[bisectRight$1(O, G, 0, V)] : H;
    }
    return q.domain = function(G) {
      return arguments.length ? (O = Array.from(G), V = Math.min(O.length, _.length - 1), q) : O.slice();
    }, q.range = function(G) {
      return arguments.length ? (_ = Array.from(G), V = Math.min(O.length, _.length - 1), q) : _.slice();
    }, q.invertExtent = function(G) {
      var J = _.indexOf(G);
      return [O[J - 1], O[J]];
    }, q.unknown = function(G) {
      return arguments.length ? (H = G, q) : H;
    }, q.copy = function() {
      return threshold().domain(O).range(_).unknown(H);
    }, initRange.apply(q, arguments);
  }
  function date(O) {
    return new Date(O);
  }
  function number$5(O) {
    return O instanceof Date ? +O : +/* @__PURE__ */ new Date(+O);
  }
  function calendar(O, _, H, V, q, G, J, Z, te, ne) {
    var ee = continuous$1(), ae = ee.invert, oe = ee.domain, se = ne(".%L"), ce = ne(":%S"), le = ne("%I:%M"), ue = ne("%I %p"), de = ne("%a %d"), fe = ne("%b %d"), he = ne("%B"), me = ne("%Y");
    function pe(ye) {
      return (te(ye) < ye ? se : Z(ye) < ye ? ce : J(ye) < ye ? le : G(ye) < ye ? ue : V(ye) < ye ? q(ye) < ye ? de : fe : H(ye) < ye ? he : me)(ye);
    }
    return ee.invert = function(ye) {
      return new Date(ae(ye));
    }, ee.domain = function(ye) {
      return arguments.length ? oe(Array.from(ye, number$5)) : oe().map(date);
    }, ee.ticks = function(ye) {
      var ge = oe();
      return O(ge[0], ge[ge.length - 1], ye ?? 10);
    }, ee.tickFormat = function(ye, ge) {
      return ge == null ? pe : ne(ge);
    }, ee.nice = function(ye) {
      var ge = oe();
      return (!ye || typeof ye.range != "function") && (ye = _(ge[0], ge[ge.length - 1], ye ?? 10)), ye ? oe(nice$1(ge, ye)) : ee;
    }, ee.copy = function() {
      return copy$2(ee, calendar(O, _, H, V, q, G, J, Z, te, ne));
    }, ee;
  }
  function time$2() {
    return initRange.apply(calendar(timeTicks, timeTickInterval, timeYear, timeMonth, timeSunday, timeDay, timeHour, timeMinute, second, timeFormat$2).domain([new Date(2e3, 0, 1), new Date(2e3, 0, 2)]), arguments);
  }
  function utcTime() {
    return initRange.apply(calendar(utcTicks, utcTickInterval, utcYear, utcMonth, utcSunday, utcDay, utcHour, utcMinute, second, utcFormat$1).domain([Date.UTC(2e3, 0, 1), Date.UTC(2e3, 0, 2)]), arguments);
  }
  function transformer$2() {
    var O = 0, _ = 1, H, V, q, G, J = identity$3, Z = !1, te;
    function ne(ae) {
      return ae == null || isNaN(ae = +ae) ? te : J(q === 0 ? 0.5 : (ae = (G(ae) - H) * q, Z ? Math.max(0, Math.min(1, ae)) : ae));
    }
    ne.domain = function(ae) {
      return arguments.length ? ([O, _] = ae, H = G(O = +O), V = G(_ = +_), q = H === V ? 0 : 1 / (V - H), ne) : [O, _];
    }, ne.clamp = function(ae) {
      return arguments.length ? (Z = !!ae, ne) : Z;
    }, ne.interpolator = function(ae) {
      return arguments.length ? (J = ae, ne) : J;
    };
    function ee(ae) {
      return function(oe) {
        var se, ce;
        return arguments.length ? ([se, ce] = oe, J = ae(se, ce), ne) : [J(0), J(1)];
      };
    }
    return ne.range = ee(interpolate$1$1), ne.rangeRound = ee(interpolateRound), ne.unknown = function(ae) {
      return arguments.length ? (te = ae, ne) : te;
    }, function(ae) {
      return G = ae, H = ae(O), V = ae(_), q = H === V ? 0 : 1 / (V - H), ne;
    };
  }
  function copy$1(O, _) {
    return _.domain(O.domain()).interpolator(O.interpolator()).clamp(O.clamp()).unknown(O.unknown());
  }
  function sequential() {
    var O = linearish(transformer$2()(identity$3));
    return O.copy = function() {
      return copy$1(O, sequential());
    }, initInterpolator.apply(O, arguments);
  }
  function sequentialLog() {
    var O = loggish(transformer$2()).domain([1, 10]);
    return O.copy = function() {
      return copy$1(O, sequentialLog()).base(O.base());
    }, initInterpolator.apply(O, arguments);
  }
  function sequentialSymlog() {
    var O = symlogish(transformer$2());
    return O.copy = function() {
      return copy$1(O, sequentialSymlog()).constant(O.constant());
    }, initInterpolator.apply(O, arguments);
  }
  function sequentialPow() {
    var O = powish(transformer$2());
    return O.copy = function() {
      return copy$1(O, sequentialPow()).exponent(O.exponent());
    }, initInterpolator.apply(O, arguments);
  }
  function sequentialSqrt() {
    return sequentialPow.apply(null, arguments).exponent(0.5);
  }
  function transformer$1() {
    var O = 0, _ = 0.5, H = 1, V = 1, q, G, J, Z, te, ne = identity$3, ee, ae = !1, oe;
    function se(le) {
      return isNaN(le = +le) ? oe : (le = 0.5 + ((le = +ee(le)) - G) * (V * le < V * G ? Z : te), ne(ae ? Math.max(0, Math.min(1, le)) : le));
    }
    se.domain = function(le) {
      return arguments.length ? ([O, _, H] = le, q = ee(O = +O), G = ee(_ = +_), J = ee(H = +H), Z = q === G ? 0 : 0.5 / (G - q), te = G === J ? 0 : 0.5 / (J - G), V = G < q ? -1 : 1, se) : [O, _, H];
    }, se.clamp = function(le) {
      return arguments.length ? (ae = !!le, se) : ae;
    }, se.interpolator = function(le) {
      return arguments.length ? (ne = le, se) : ne;
    };
    function ce(le) {
      return function(ue) {
        var de, fe, he;
        return arguments.length ? ([de, fe, he] = ue, ne = piecewise(le, [de, fe, he]), se) : [ne(0), ne(0.5), ne(1)];
      };
    }
    return se.range = ce(interpolate$1$1), se.rangeRound = ce(interpolateRound), se.unknown = function(le) {
      return arguments.length ? (oe = le, se) : oe;
    }, function(le) {
      return ee = le, q = le(O), G = le(_), J = le(H), Z = q === G ? 0 : 0.5 / (G - q), te = G === J ? 0 : 0.5 / (J - G), V = G < q ? -1 : 1, se;
    };
  }
  function diverging() {
    var O = linearish(transformer$1()(identity$3));
    return O.copy = function() {
      return copy$1(O, diverging());
    }, initInterpolator.apply(O, arguments);
  }
  function divergingLog() {
    var O = loggish(transformer$1()).domain([0.1, 1, 10]);
    return O.copy = function() {
      return copy$1(O, divergingLog()).base(O.base());
    }, initInterpolator.apply(O, arguments);
  }
  function divergingSymlog() {
    var O = symlogish(transformer$1());
    return O.copy = function() {
      return copy$1(O, divergingSymlog()).constant(O.constant());
    }, initInterpolator.apply(O, arguments);
  }
  function divergingPow() {
    var O = powish(transformer$1());
    return O.copy = function() {
      return copy$1(O, divergingPow()).exponent(O.exponent());
    }, initInterpolator.apply(O, arguments);
  }
  function divergingSqrt() {
    return divergingPow.apply(null, arguments).exponent(0.5);
  }
  function colors$1(O) {
    for (var _ = O.length / 6 | 0, H = new Array(_), V = 0; V < _; ) H[V] = "#" + O.slice(V * 6, ++V * 6);
    return H;
  }
  const schemeCategory10 = colors$1("1f77b4ff7f0e2ca02cd627289467bd8c564be377c27f7f7fbcbd2217becf"), schemeAccent = colors$1("7fc97fbeaed4fdc086ffff99386cb0f0027fbf5b17666666"), schemeDark2 = colors$1("1b9e77d95f027570b3e7298a66a61ee6ab02a6761d666666"), schemeObservable10 = colors$1("4269d0efb118ff725c6cc5b03ca951ff8ab7a463f297bbf59c6b4e9498a0"), schemePaired = colors$1("a6cee31f78b4b2df8a33a02cfb9a99e31a1cfdbf6fff7f00cab2d66a3d9affff99b15928"), schemePastel1 = colors$1("fbb4aeb3cde3ccebc5decbe4fed9a6ffffcce5d8bdfddaecf2f2f2"), schemePastel2 = colors$1("b3e2cdfdcdaccbd5e8f4cae4e6f5c9fff2aef1e2cccccccc"), schemeSet1 = colors$1("e41a1c377eb84daf4a984ea3ff7f00ffff33a65628f781bf999999"), schemeSet2 = colors$1("66c2a5fc8d628da0cbe78ac3a6d854ffd92fe5c494b3b3b3"), schemeSet3 = colors$1("8dd3c7ffffb3bebadafb807280b1d3fdb462b3de69fccde5d9d9d9bc80bdccebc5ffed6f");
  function bandSpace(O, _, H) {
    const V = O - _ + H * 2;
    return O ? V > 0 ? V : 1 : 0;
  }
  const Identity = "identity", Linear = "linear", Log = "log", Pow = "pow", Sqrt = "sqrt", Symlog = "symlog", Time$1 = "time", UTC = "utc", Sequential = "sequential", Diverging = "diverging", Quantile = "quantile", Quantize = "quantize", Threshold = "threshold", Ordinal = "ordinal", Point = "point", Band = "band", BinOrdinal = "bin-ordinal", Continuous = "continuous", Discrete$1 = "discrete", Discretizing = "discretizing", Interpolating = "interpolating", Temporal = "temporal";
  function invertRange(O) {
    return function(_) {
      let H = _[0], V = _[1], q;
      return V < H && (q = H, H = V, V = q), [O.invert(H), O.invert(V)];
    };
  }
  function invertRangeExtent(O) {
    return function(_) {
      const H = O.range();
      let V = _[0], q = _[1], G = -1, J, Z, te, ne;
      for (q < V && (Z = V, V = q, q = Z), te = 0, ne = H.length; te < ne; ++te)
        H[te] >= V && H[te] <= q && (G < 0 && (G = te), J = te);
      if (!(G < 0))
        return V = O.invertExtent(H[G]), q = O.invertExtent(H[J]), [V[0] === void 0 ? V[1] : V[0], q[1] === void 0 ? q[0] : q[1]];
    };
  }
  function band() {
    const O = ordinal().unknown(void 0), _ = O.domain, H = O.range;
    let V = [0, 1], q, G, J = !1, Z = 0, te = 0, ne = 0.5;
    delete O.unknown;
    function ee() {
      const ae = _().length, oe = V[1] < V[0], se = V[1 - oe], ce = bandSpace(ae, Z, te);
      let le = V[oe - 0];
      q = (se - le) / (ce || 1), J && (q = Math.floor(q)), le += (se - le - q * (ae - Z)) * ne, G = q * (1 - Z), J && (le = Math.round(le), G = Math.round(G));
      const ue = range$7(ae).map((de) => le + q * de);
      return H(oe ? ue.reverse() : ue);
    }
    return O.domain = function(ae) {
      return arguments.length ? (_(ae), ee()) : _();
    }, O.range = function(ae) {
      return arguments.length ? (V = [+ae[0], +ae[1]], ee()) : V.slice();
    }, O.rangeRound = function(ae) {
      return V = [+ae[0], +ae[1]], J = !0, ee();
    }, O.bandwidth = function() {
      return G;
    }, O.step = function() {
      return q;
    }, O.round = function(ae) {
      return arguments.length ? (J = !!ae, ee()) : J;
    }, O.padding = function(ae) {
      return arguments.length ? (te = Math.max(0, Math.min(1, ae)), Z = te, ee()) : Z;
    }, O.paddingInner = function(ae) {
      return arguments.length ? (Z = Math.max(0, Math.min(1, ae)), ee()) : Z;
    }, O.paddingOuter = function(ae) {
      return arguments.length ? (te = Math.max(0, Math.min(1, ae)), ee()) : te;
    }, O.align = function(ae) {
      return arguments.length ? (ne = Math.max(0, Math.min(1, ae)), ee()) : ne;
    }, O.invertRange = function(ae) {
      if (ae[0] == null || ae[1] == null) return;
      const oe = V[1] < V[0], se = oe ? H().reverse() : H(), ce = se.length - 1;
      let le = +ae[0], ue = +ae[1], de, fe, he;
      if (!(le !== le || ue !== ue) && (ue < le && (he = le, le = ue, ue = he), !(ue < se[0] || le > V[1 - oe])))
        return de = Math.max(0, bisectRight$1(se, le) - 1), fe = le === ue ? de : bisectRight$1(se, ue) - 1, le - se[de] > G + 1e-10 && ++de, oe && (he = de, de = ce - fe, fe = ce - he), de > fe ? void 0 : _().slice(de, fe + 1);
    }, O.invert = function(ae) {
      const oe = O.invertRange([ae, ae]);
      return oe && oe[0];
    }, O.copy = function() {
      return band().domain(_()).range(V).round(J).paddingInner(Z).paddingOuter(te).align(ne);
    }, ee();
  }
  function pointish(O) {
    const _ = O.copy;
    return O.padding = O.paddingOuter, delete O.paddingInner, O.copy = function() {
      return pointish(_());
    }, O;
  }
  function point$3() {
    return pointish(band().paddingInner(1));
  }
  var map$1 = Array.prototype.map;
  function numbers(O) {
    return map$1.call(O, toNumber$1);
  }
  const slice$4 = Array.prototype.slice;
  function scaleBinOrdinal() {
    let O = [], _ = [];
    function H(V) {
      return V == null || V !== V ? void 0 : _[(bisectRight$1(O, V) - 1) % _.length];
    }
    return H.domain = function(V) {
      return arguments.length ? (O = numbers(V), H) : O.slice();
    }, H.range = function(V) {
      return arguments.length ? (_ = slice$4.call(V), H) : _.slice();
    }, H.tickFormat = function(V, q) {
      return tickFormat$1(O[0], peek$1(O), V ?? 10, q);
    }, H.copy = function() {
      return scaleBinOrdinal().domain(H.domain()).range(H.range());
    }, H;
  }
  const scales = /* @__PURE__ */ new Map(), VEGA_SCALE = Symbol("vega_scale");
  function registerScale(O) {
    return O[VEGA_SCALE] = !0, O;
  }
  function isRegisteredScale(O) {
    return O && O[VEGA_SCALE] === !0;
  }
  function create$2(O, _, H) {
    const V = function() {
      const G = _();
      return G.invertRange || (G.invertRange = G.invert ? invertRange(G) : G.invertExtent ? invertRangeExtent(G) : void 0), G.type = O, registerScale(G);
    };
    return V.metadata = toSet$1(array$5(H)), V;
  }
  function scale$4(O, _, H) {
    return arguments.length > 1 ? (scales.set(O, create$2(O, _, H)), this) : isValidScaleType(O) ? scales.get(O) : void 0;
  }
  scale$4(Identity, identity$2);
  scale$4(Linear, linear, Continuous);
  scale$4(Log, log$2, [Continuous, Log]);
  scale$4(Pow, pow$2, Continuous);
  scale$4(Sqrt, sqrt$2, Continuous);
  scale$4(Symlog, symlog, Continuous);
  scale$4(Time$1, time$2, [Continuous, Temporal]);
  scale$4(UTC, utcTime, [Continuous, Temporal]);
  scale$4(Sequential, sequential, [Continuous, Interpolating]);
  scale$4(`${Sequential}-${Linear}`, sequential, [Continuous, Interpolating]);
  scale$4(`${Sequential}-${Log}`, sequentialLog, [Continuous, Interpolating, Log]);
  scale$4(`${Sequential}-${Pow}`, sequentialPow, [Continuous, Interpolating]);
  scale$4(`${Sequential}-${Sqrt}`, sequentialSqrt, [Continuous, Interpolating]);
  scale$4(`${Sequential}-${Symlog}`, sequentialSymlog, [Continuous, Interpolating]);
  scale$4(`${Diverging}-${Linear}`, diverging, [Continuous, Interpolating]);
  scale$4(`${Diverging}-${Log}`, divergingLog, [Continuous, Interpolating, Log]);
  scale$4(`${Diverging}-${Pow}`, divergingPow, [Continuous, Interpolating]);
  scale$4(`${Diverging}-${Sqrt}`, divergingSqrt, [Continuous, Interpolating]);
  scale$4(`${Diverging}-${Symlog}`, divergingSymlog, [Continuous, Interpolating]);
  scale$4(Quantile, quantile, [Discretizing, Quantile]);
  scale$4(Quantize, quantize$1, Discretizing);
  scale$4(Threshold, threshold, Discretizing);
  scale$4(BinOrdinal, scaleBinOrdinal, [Discrete$1, Discretizing]);
  scale$4(Ordinal, ordinal, Discrete$1);
  scale$4(Band, band, Discrete$1);
  scale$4(Point, point$3, Discrete$1);
  function isValidScaleType(O) {
    return scales.has(O);
  }
  function hasType(O, _) {
    const H = scales.get(O);
    return H && H.metadata[_];
  }
  function isContinuous$1(O) {
    return hasType(O, Continuous);
  }
  function isDiscrete$2(O) {
    return hasType(O, Discrete$1);
  }
  function isDiscretizing$1(O) {
    return hasType(O, Discretizing);
  }
  function isLogarithmic(O) {
    return hasType(O, Log);
  }
  function isTemporal(O) {
    return hasType(O, Temporal);
  }
  function isInterpolating(O) {
    return hasType(O, Interpolating);
  }
  function isQuantile$1(O) {
    return hasType(O, Quantile);
  }
  const scaleProps = ["clamp", "base", "constant", "exponent"];
  function interpolateRange$1(O, _) {
    const H = _[0], V = peek$1(_) - H;
    return function(q) {
      return O(H + q * V);
    };
  }
  function interpolateColors(O, _, H) {
    return piecewise(interpolate$1(_ || "rgb", H), O);
  }
  function quantizeInterpolator(O, _) {
    const H = new Array(_), V = _ + 1;
    for (let q = 0; q < _; ) H[q] = O(++q / V);
    return H;
  }
  function scaleFraction(O, _, H) {
    const V = H - _;
    let q, G, J;
    return !V || !Number.isFinite(V) ? constant$5(0.5) : (q = (G = O.type).indexOf("-"), G = q < 0 ? G : G.slice(q + 1), J = scale$4(G)().domain([_, H]).range([0, 1]), scaleProps.forEach((Z) => O[Z] ? J[Z](O[Z]()) : 0), J);
  }
  function interpolate$1(O, _) {
    const H = $$1$1[method(O)];
    return _ != null && H && H.gamma ? H.gamma(_) : H;
  }
  function method(O) {
    return "interpolate" + O.toLowerCase().split("-").map((_) => _[0].toUpperCase() + _.slice(1)).join("");
  }
  const continuous = {
    blues: "cfe1f2bed8eca8cee58fc1de74b2d75ba3cf4592c63181bd206fb2125ca40a4a90",
    greens: "d3eecdc0e6baabdda594d3917bc77d60ba6c46ab5e329a512089430e7735036429",
    greys: "e2e2e2d4d4d4c4c4c4b1b1b19d9d9d8888887575756262624d4d4d3535351e1e1e",
    oranges: "fdd8b3fdc998fdb87bfda55efc9244f87f2cf06b18e4580bd14904b93d029f3303",
    purples: "e2e1efd4d4e8c4c5e0b4b3d6a3a0cc928ec3827cb97566ae684ea25c3696501f8c",
    reds: "fdc9b4fcb49afc9e80fc8767fa7051f6573fec3f2fdc2a25c81b1db21218970b13",
    blueGreen: "d5efedc1e8e0a7ddd18bd2be70c6a958ba9144ad77319c5d2089460e7736036429",
    bluePurple: "ccddecbad0e4a8c2dd9ab0d4919cc98d85be8b6db28a55a6873c99822287730f71",
    greenBlue: "d3eecec5e8c3b1e1bb9bd8bb82cec269c2ca51b2cd3c9fc7288abd1675b10b60a1",
    orangeRed: "fddcaffdcf9bfdc18afdad77fb9562f67d53ee6545e24932d32d1ebf130da70403",
    purpleBlue: "dbdaebc8cee4b1c3de97b7d87bacd15b9fc93a90c01e7fb70b70ab056199045281",
    purpleBlueGreen: "dbd8eac8cee4b0c3de93b7d872acd1549fc83892bb1c88a3097f8702736b016353",
    purpleRed: "dcc9e2d3b3d7ce9eccd186c0da6bb2e14da0e23189d91e6fc61159ab07498f023a",
    redPurple: "fccfccfcbec0faa9b8f98faff571a5ec539ddb3695c41b8aa908808d0179700174",
    yellowGreen: "e4f4acd1eca0b9e2949ed68880c97c62bb6e47aa5e3297502083440e723b036034",
    yellowOrangeBrown: "feeaa1fedd84fecc63feb746fca031f68921eb7215db5e0bc54c05ab3d038f3204",
    yellowOrangeRed: "fee087fed16ffebd59fea849fd903efc7335f9522bee3423de1b20ca0b22af0225",
    blueOrange: "134b852f78b35da2cb9dcae1d2e5eff2f0ebfce0bafbbf74e8932fc5690d994a07",
    brownBlueGreen: "704108a0651ac79548e3c78af3e6c6eef1eac9e9e48ed1c74da79e187a72025147",
    purpleGreen: "5b1667834792a67fb6c9aed3e6d6e8eff0efd9efd5aedda971bb75368e490e5e29",
    purpleOrange: "4114696647968f83b7b9b4d6dadbebf3eeeafce0bafbbf74e8932fc5690d994a07",
    redBlue: "8c0d25bf363adf745ef4ae91fbdbc9f2efeed2e5ef9dcae15da2cb2f78b3134b85",
    redGrey: "8c0d25bf363adf745ef4ae91fcdccbfaf4f1e2e2e2c0c0c0969696646464343434",
    yellowGreenBlue: "eff9bddbf1b4bde5b594d5b969c5be45b4c22c9ec02182b82163aa23479c1c3185",
    redYellowBlue: "a50026d4322cf16e43fcac64fedd90faf8c1dcf1ecabd6e875abd04a74b4313695",
    redYellowGreen: "a50026d4322cf16e43fcac63fedd8df9f7aed7ee8ea4d86e64bc6122964f006837",
    pinkYellowGreen: "8e0152c0267edd72adf0b3d6faddedf5f3efe1f2cab6de8780bb474f9125276419",
    spectral: "9e0142d13c4bf0704afcac63fedd8dfbf8b0e0f3a1a9dda269bda94288b55e4fa2",
    viridis: "440154470e61481a6c482575472f7d443a834144873d4e8a39568c35608d31688e2d708e2a788e27818e23888e21918d1f988b1fa08822a8842ab07f35b77943bf7154c56866cc5d7ad1518fd744a5db36bcdf27d2e21be9e51afde725",
    magma: "0000040404130b0924150e3720114b2c11603b0f704a107957157e651a80721f817f24828c29819a2e80a8327db6377ac43c75d1426fde4968e95462f1605df76f5cfa7f5efc8f65fe9f6dfeaf78febf84fece91fddea0fcedaffcfdbf",
    inferno: "0000040403130c0826170c3b240c4f330a5f420a68500d6c5d126e6b176e781c6d86216b932667a12b62ae305cbb3755c73e4cd24644dd513ae65c30ed6925f3771af8850ffb9506fca50afcb519fac62df6d645f2e661f3f484fcffa4",
    plasma: "0d088723069033059742039d5002a25d01a66a00a87801a88405a7900da49c179ea72198b12a90ba3488c33d80cb4779d35171da5a69e16462e76e5bed7953f2834cf68f44fa9a3dfca636fdb32ffec029fcce25f9dc24f5ea27f0f921",
    cividis: "00205100235800265d002961012b65042e670831690d346b11366c16396d1c3c6e213f6e26426e2c456e31476e374a6e3c4d6e42506e47536d4c566d51586e555b6e5a5e6e5e616e62646f66676f6a6a706e6d717270717573727976737c79747f7c75827f758682768985778c8877908b78938e789691789a94789e9778a19b78a59e77a9a177aea575b2a874b6ab73bbaf71c0b26fc5b66dc9b96acebd68d3c065d8c462ddc85fe2cb5ce7cf58ebd355f0d652f3da4ff7de4cfae249fce647",
    rainbow: "6e40aa883eb1a43db3bf3cafd83fa4ee4395fe4b83ff576eff6659ff7847ff8c38f3a130e2b72fcfcc36bee044aff05b8ff4576ff65b52f6673af27828ea8d1ddfa319d0b81cbecb23abd82f96e03d82e14c6edb5a5dd0664dbf6e40aa",
    sinebow: "ff4040fc582af47218e78d0bd5a703bfbf00a7d5038de70b72f41858fc2a40ff402afc5818f4720be78d03d5a700bfbf03a7d50b8de71872f42a58fc4040ff582afc7218f48d0be7a703d5bf00bfd503a7e70b8df41872fc2a58ff4040",
    turbo: "23171b32204a3e2a71453493493eae4b49c54a53d7485ee44569ee4074f53c7ff8378af93295f72e9ff42ba9ef28b3e926bce125c5d925cdcf27d5c629dcbc2de3b232e9a738ee9d3ff39347f68950f9805afc7765fd6e70fe667cfd5e88fc5795fb51a1f84badf545b9f140c5ec3cd0e637dae034e4d931ecd12ef4c92bfac029ffb626ffad24ffa223ff9821ff8d1fff821dff771cfd6c1af76118f05616e84b14df4111d5380fcb2f0dc0260ab61f07ac1805a313029b0f00950c00910b00",
    browns: "eedbbdecca96e9b97ae4a865dc9856d18954c7784cc0673fb85536ad44339f3632",
    tealBlues: "bce4d89dd3d181c3cb65b3c245a2b9368fae347da0306a932c5985",
    teals: "bbdfdfa2d4d58ac9c975bcbb61b0af4da5a43799982b8b8c1e7f7f127273006667",
    warmGreys: "dcd4d0cec5c1c0b8b4b3aaa7a59c9998908c8b827f7e7673726866665c5a59504e",
    goldGreen: "f4d166d5ca60b6c35c98bb597cb25760a6564b9c533f8f4f33834a257740146c36",
    goldOrange: "f4d166f8be5cf8aa4cf5983bf3852aef701be2621fd65322c54923b142239e3a26",
    goldRed: "f4d166f6be59f9aa51fc964ef6834bee734ae56249db5247cf4244c43141b71d3e",
    lightGreyRed: "efe9e6e1dad7d5cbc8c8bdb9bbaea9cd967ddc7b43e15f19df4011dc000b",
    lightGreyTeal: "e4eaead6dcddc8ced2b7c2c7a6b4bc64b0bf22a6c32295c11f85be1876bc",
    lightMulti: "e0f1f2c4e9d0b0de9fd0e181f6e072f6c053f3993ef77440ef4a3c",
    lightOrange: "f2e7daf7d5baf9c499fab184fa9c73f68967ef7860e8645bde515bd43d5b",
    lightTealBlue: "e3e9e0c0dccf9aceca7abfc859afc0389fb9328dad2f7ca0276b95255988",
    darkBlue: "3232322d46681a5c930074af008cbf05a7ce25c0dd38daed50f3faffffff",
    darkGold: "3c3c3c584b37725e348c7631ae8b2bcfa424ecc31ef9de30fff184ffffff",
    darkGreen: "3a3a3a215748006f4d048942489e4276b340a6c63dd2d836ffeb2cffffaa",
    darkMulti: "3737371f5287197d8c29a86995ce3fffe800ffffff",
    darkRed: "3434347036339e3c38cc4037e75d1eec8620eeab29f0ce32ffeb2c"
  }, discrete = {
    accent: schemeAccent,
    category10: schemeCategory10,
    category20: "1f77b4aec7e8ff7f0effbb782ca02c98df8ad62728ff98969467bdc5b0d58c564bc49c94e377c2f7b6d27f7f7fc7c7c7bcbd22dbdb8d17becf9edae5",
    category20b: "393b795254a36b6ecf9c9ede6379398ca252b5cf6bcedb9c8c6d31bd9e39e7ba52e7cb94843c39ad494ad6616be7969c7b4173a55194ce6dbdde9ed6",
    category20c: "3182bd6baed69ecae1c6dbefe6550dfd8d3cfdae6bfdd0a231a35474c476a1d99bc7e9c0756bb19e9ac8bcbddcdadaeb636363969696bdbdbdd9d9d9",
    dark2: schemeDark2,
    observable10: schemeObservable10,
    paired: schemePaired,
    pastel1: schemePastel1,
    pastel2: schemePastel2,
    set1: schemeSet1,
    set2: schemeSet2,
    set3: schemeSet3,
    tableau10: "4c78a8f58518e4575672b7b254a24beeca3bb279a2ff9da69d755dbab0ac",
    tableau20: "4c78a89ecae9f58518ffbf7954a24b88d27ab79a20f2cf5b43989483bcb6e45756ff9d9879706ebab0acd67195fcbfd2b279a2d6a5c99e765fd8b5a5"
  };
  function colors(O) {
    if (isArray$2(O)) return O;
    const _ = O.length / 6 | 0, H = new Array(_);
    for (let V = 0; V < _; )
      H[V] = "#" + O.slice(V * 6, ++V * 6);
    return H;
  }
  function apply$1(O, _) {
    for (const H in O) scheme$1(H, _(O[H]));
  }
  const schemes = {};
  apply$1(discrete, colors);
  apply$1(continuous, (O) => interpolateColors(colors(O)));
  function scheme$1(O, _) {
    return O = O && O.toLowerCase(), arguments.length > 1 ? (schemes[O] = _, this) : schemes[O];
  }
  const SymbolLegend = "symbol", DiscreteLegend = "discrete", GradientLegend = "gradient", defaultFormatter = (O) => isArray$2(O) ? O.map((_) => String(_)) : String(O), ascending = (O, _) => O[1] - _[1], descending = (O, _) => _[1] - O[1];
  function tickCount(O, _, H) {
    let V;
    return isNumber$2(_) && (O.bins && (_ = Math.max(_, O.bins.length)), H != null && (_ = Math.min(_, Math.floor(span(O.domain()) / H || 1) + 1))), isObject$2(_) && (V = _.step, _ = _.interval), isString$1(_) && (_ = O.type === Time$1 ? timeInterval(_) : O.type == UTC ? utcInterval(_) : error$1("Only time and utc scales accept interval strings."), V && (_ = _.every(V))), _;
  }
  function validTicks(O, _, H) {
    let V = O.range(), q = V[0], G = peek$1(V), J = ascending;
    if (q > G && (V = G, G = q, q = V, J = descending), q = Math.floor(q), G = Math.ceil(G), _ = _.map((Z) => [Z, O(Z)]).filter((Z) => q <= Z[1] && Z[1] <= G).sort(J).map((Z) => Z[0]), H > 0 && _.length > 1) {
      const Z = [_[0], peek$1(_)];
      for (; _.length > H && _.length >= 3; )
        _ = _.filter((te, ne) => !(ne % 2));
      _.length < 3 && (_ = Z);
    }
    return _;
  }
  function tickValues(O, _) {
    return O.bins ? validTicks(O, O.bins, _) : O.ticks ? O.ticks(_) : O.domain();
  }
  function tickFormat(O, _, H, V, q, G) {
    const J = _.type;
    let Z = defaultFormatter;
    if (J === Time$1 || q === Time$1)
      Z = O.timeFormat(V);
    else if (J === UTC || q === UTC)
      Z = O.utcFormat(V);
    else if (isLogarithmic(J)) {
      const te = O.formatFloat(V);
      if (G || _.bins)
        Z = te;
      else {
        const ne = tickLog(_, H, !1);
        Z = (ee) => ne(ee) ? te(ee) : "";
      }
    } else if (_.tickFormat) {
      const te = _.domain();
      Z = O.formatSpan(te[0], te[te.length - 1], H, V);
    } else V && (Z = O.format(V));
    return Z;
  }
  function tickLog(O, _, H) {
    const V = tickValues(O, _), q = O.base(), G = Math.log(q), J = Math.max(1, q * _ / V.length), Z = (te) => {
      let ne = te / Math.pow(q, Math.round(Math.log(te) / G));
      return ne * q < q - 0.5 && (ne *= q), ne <= J;
    };
    return H ? V.filter(Z) : Z;
  }
  const symbols$2 = {
    [Quantile]: "quantiles",
    [Quantize]: "thresholds",
    [Threshold]: "domain"
  }, formats = {
    [Quantile]: "quantiles",
    [Quantize]: "domain"
  };
  function labelValues(O, _) {
    return O.bins ? binValues(O.bins) : O.type === Log ? tickLog(O, _, !0) : symbols$2[O.type] ? thresholdValues(O[symbols$2[O.type]]()) : tickValues(O, _);
  }
  function thresholdFormat(O, _, H) {
    const V = _[formats[_.type]](), q = V.length;
    let G = q > 1 ? V[1] - V[0] : V[0], J;
    for (J = 1; J < q; ++J)
      G = Math.min(G, V[J] - V[J - 1]);
    return O.formatSpan(0, G, 3 * 10, H);
  }
  function thresholdValues(O) {
    const _ = [-1 / 0].concat(O);
    return _.max = 1 / 0, _;
  }
  function binValues(O) {
    const _ = O.slice(0, -1);
    return _.max = peek$1(O), _;
  }
  const isDiscreteRange = (O) => symbols$2[O.type] || O.bins;
  function labelFormat(O, _, H, V, q, G, J) {
    const Z = formats[_.type] && G !== Time$1 && G !== UTC ? thresholdFormat(O, _, q) : tickFormat(O, _, H, q, G, J);
    return V === SymbolLegend && isDiscreteRange(_) ? formatRange(Z) : V === DiscreteLegend ? formatDiscrete(Z) : formatPoint(Z);
  }
  const formatRange = (O) => (_, H, V) => {
    const q = get$4(V[H + 1], get$4(V.max, 1 / 0)), G = formatValue$2(_, O), J = formatValue$2(q, O);
    return G && J ? G + " – " + J : J ? "< " + J : "≥ " + G;
  }, get$4 = (O, _) => O ?? _, formatDiscrete = (O) => (_, H) => H ? O(_) : null, formatPoint = (O) => (_) => O(_), formatValue$2 = (O, _) => Number.isFinite(O) ? _(O) : null;
  function labelFraction(O) {
    const _ = O.domain(), H = _.length - 1;
    let V = +_[0], q = +peek$1(_), G = q - V;
    if (O.type === Threshold) {
      const J = H ? G / H : 0.1;
      V -= J, q += J, G = q - V;
    }
    return (J) => (J - V) / G;
  }
  function format$1(O, _, H, V) {
    const q = V || _.type;
    return isString$1(H) && isTemporal(q) && (H = H.replace(/%a/g, "%A").replace(/%b/g, "%B")), !H && q === Time$1 ? O.timeFormat("%A, %d %B %Y, %X") : !H && q === UTC ? O.utcFormat("%A, %d %B %Y, %X UTC") : labelFormat(O, _, 5, null, H, V, !0);
  }
  function domainCaption(O, _, H) {
    H = H || {};
    const V = Math.max(3, H.maxlen || 7), q = format$1(O, _, H.format, H.formatType);
    if (isDiscretizing$1(_.type)) {
      const G = labelValues(_).slice(1).map(q), J = G.length;
      return `${J} boundar${J === 1 ? "y" : "ies"}: ${G.join(", ")}`;
    } else if (isDiscrete$2(_.type)) {
      const G = _.domain(), J = G.length, Z = J > V ? G.slice(0, V - 2).map(q).join(", ") + ", ending with " + G.slice(-1).map(q) : G.map(q).join(", ");
      return `${J} value${J === 1 ? "" : "s"}: ${Z}`;
    } else {
      const G = _.domain();
      return `values from ${q(G[0])} to ${q(peek$1(G))}`;
    }
  }
  let gradient_id = 0;
  function resetSVGGradientId() {
    gradient_id = 0;
  }
  const patternPrefix = "p_";
  function isGradient(O) {
    return O && O.gradient;
  }
  function gradientRef(O, _, H) {
    const V = O.gradient;
    let q = O.id, G = V === "radial" ? patternPrefix : "";
    return q || (q = O.id = "gradient_" + gradient_id++, V === "radial" ? (O.x1 = get$3(O.x1, 0.5), O.y1 = get$3(O.y1, 0.5), O.r1 = get$3(O.r1, 0), O.x2 = get$3(O.x2, 0.5), O.y2 = get$3(O.y2, 0.5), O.r2 = get$3(O.r2, 0.5), G = patternPrefix) : (O.x1 = get$3(O.x1, 0), O.y1 = get$3(O.y1, 0), O.x2 = get$3(O.x2, 1), O.y2 = get$3(O.y2, 0))), _[q] = O, "url(" + (H || "") + "#" + G + q + ")";
  }
  function get$3(O, _) {
    return O ?? _;
  }
  function Gradient$1(O, _) {
    var H = [], V;
    return V = {
      gradient: "linear",
      x1: O ? O[0] : 0,
      y1: O ? O[1] : 0,
      x2: _ ? _[0] : 1,
      y2: _ ? _[1] : 0,
      stops: H,
      stop: function(q, G) {
        return H.push({
          offset: q,
          color: G
        }), V;
      }
    };
  }
  const lookup$4 = {
    basis: {
      curve: curveBasis
    },
    "basis-closed": {
      curve: curveBasisClosed
    },
    "basis-open": {
      curve: curveBasisOpen
    },
    bundle: {
      curve: curveBundle,
      tension: "beta",
      value: 0.85
    },
    cardinal: {
      curve: curveCardinal,
      tension: "tension",
      value: 0
    },
    "cardinal-open": {
      curve: curveCardinalOpen,
      tension: "tension",
      value: 0
    },
    "cardinal-closed": {
      curve: curveCardinalClosed,
      tension: "tension",
      value: 0
    },
    "catmull-rom": {
      curve: curveCatmullRom,
      tension: "alpha",
      value: 0.5
    },
    "catmull-rom-closed": {
      curve: curveCatmullRomClosed,
      tension: "alpha",
      value: 0.5
    },
    "catmull-rom-open": {
      curve: curveCatmullRomOpen,
      tension: "alpha",
      value: 0.5
    },
    linear: {
      curve: curveLinear
    },
    "linear-closed": {
      curve: curveLinearClosed
    },
    monotone: {
      horizontal: monotoneY,
      vertical: monotoneX
    },
    natural: {
      curve: curveNatural
    },
    step: {
      curve: curveStep
    },
    "step-after": {
      curve: stepAfter
    },
    "step-before": {
      curve: stepBefore
    }
  };
  function curves(O, _, H) {
    var V = has$1(lookup$4, O) && lookup$4[O], q = null;
    return V && (q = V.curve || V[_ || "vertical"], V.tension && H != null && (q = q[V.tension](H))), q;
  }
  const paramCounts = {
    m: 2,
    l: 2,
    h: 1,
    v: 1,
    z: 0,
    c: 6,
    s: 4,
    q: 4,
    t: 2,
    a: 7
  }, commandPattern = /[mlhvzcsqta]([^mlhvzcsqta]+|$)/gi, numberPattern = /^[+-]?(([0-9]*\.[0-9]+)|([0-9]+\.)|([0-9]+))([eE][+-]?[0-9]+)?/, spacePattern = /^((\s+,?\s*)|(,\s*))/, flagPattern = /^[01]/;
  function parse$3(O) {
    const _ = [];
    return (O.match(commandPattern) || []).forEach((V) => {
      let q = V[0];
      const G = q.toLowerCase(), J = paramCounts[G], Z = parseParams(G, J, V.slice(1).trim()), te = Z.length;
      if (te < J || te && te % J !== 0)
        throw Error("Invalid SVG path, incorrect parameter count");
      if (_.push([q, ...Z.slice(0, J)]), te !== J) {
        G === "m" && (q = q === "M" ? "L" : "l");
        for (let ne = J; ne < te; ne += J)
          _.push([q, ...Z.slice(ne, ne + J)]);
      }
    }), _;
  }
  function parseParams(O, _, H) {
    const V = [];
    for (let q = 0; _ && q < H.length; )
      for (let G = 0; G < _; ++G) {
        const J = O === "a" && (G === 3 || G === 4) ? flagPattern : numberPattern, Z = H.slice(q).match(J);
        if (Z === null)
          throw Error("Invalid SVG path, incorrect parameter type");
        q += Z[0].length, V.push(+Z[0]);
        const te = H.slice(q).match(spacePattern);
        te !== null && (q += te[0].length);
      }
    return V;
  }
  const DegToRad = Math.PI / 180, Epsilon$1 = 1e-14, HalfPi = Math.PI / 2, Tau = Math.PI * 2, HalfSqrt3 = Math.sqrt(3) / 2;
  var segmentCache = {}, bezierCache = {}, join$1 = [].join;
  function segments(O, _, H, V, q, G, J, Z, te) {
    const ne = join$1.call(arguments);
    if (segmentCache[ne])
      return segmentCache[ne];
    const ee = J * DegToRad, ae = Math.sin(ee), oe = Math.cos(ee);
    H = Math.abs(H), V = Math.abs(V);
    const se = oe * (Z - O) * 0.5 + ae * (te - _) * 0.5, ce = oe * (te - _) * 0.5 - ae * (Z - O) * 0.5;
    let le = se * se / (H * H) + ce * ce / (V * V);
    le > 1 && (le = Math.sqrt(le), H *= le, V *= le);
    const ue = oe / H, de = ae / H, fe = -ae / V, he = oe / V, me = ue * Z + de * te, pe = fe * Z + he * te, ye = ue * O + de * _, ge = fe * O + he * _;
    let be = 1 / ((ye - me) * (ye - me) + (ge - pe) * (ge - pe)) - 0.25;
    be < 0 && (be = 0);
    let Ce = Math.sqrt(be);
    G == q && (Ce = -Ce);
    const Se = 0.5 * (me + ye) - Ce * (ge - pe), we = 0.5 * (pe + ge) + Ce * (ye - me), Ee = Math.atan2(pe - we, me - Se);
    let Le = Math.atan2(ge - we, ye - Se) - Ee;
    Le < 0 && G === 1 ? Le += Tau : Le > 0 && G === 0 && (Le -= Tau);
    const Qe = Math.ceil(Math.abs(Le / (HalfPi + 1e-3))), ke = [];
    for (let Me = 0; Me < Qe; ++Me) {
      const Re = Ee + Me * Le / Qe, De = Ee + (Me + 1) * Le / Qe;
      ke[Me] = [Se, we, Re, De, H, V, ae, oe];
    }
    return segmentCache[ne] = ke;
  }
  function bezier(O) {
    const _ = join$1.call(O);
    if (bezierCache[_])
      return bezierCache[_];
    var H = O[0], V = O[1], q = O[2], G = O[3], J = O[4], Z = O[5], te = O[6], ne = O[7];
    const ee = ne * J, ae = -te * Z, oe = te * J, se = ne * Z, ce = Math.cos(q), le = Math.sin(q), ue = Math.cos(G), de = Math.sin(G), fe = 0.5 * (G - q), he = Math.sin(fe * 0.5), me = 8 / 3 * he * he / Math.sin(fe), pe = H + ce - me * le, ye = V + le + me * ce, ge = H + ue, ve = V + de, be = ge + me * de, Ce = ve - me * ue;
    return bezierCache[_] = [ee * pe + ae * ye, oe * pe + se * ye, ee * be + ae * Ce, oe * be + se * Ce, ee * ge + ae * ve, oe * ge + se * ve];
  }
  const temp = ["l", 0, 0, 0, 0, 0, 0, 0];
  function scale$1$1(O, _, H) {
    const V = temp[0] = O[0];
    if (V === "a" || V === "A")
      temp[1] = _ * O[1], temp[2] = H * O[2], temp[3] = O[3], temp[4] = O[4], temp[5] = O[5], temp[6] = _ * O[6], temp[7] = H * O[7];
    else if (V === "h" || V === "H")
      temp[1] = _ * O[1];
    else if (V === "v" || V === "V")
      temp[1] = H * O[1];
    else
      for (var q = 1, G = O.length; q < G; ++q)
        temp[q] = (q % 2 == 1 ? _ : H) * O[q];
    return temp;
  }
  function pathRender(O, _, H, V, q, G) {
    var J, Z = null, te = 0, ne = 0, ee = 0, ae = 0, oe, se, ce, le, ue = 0, de = 0;
    H == null && (H = 0), V == null && (V = 0), q == null && (q = 1), G == null && (G = q), O.beginPath && O.beginPath();
    for (var fe = 0, he = _.length; fe < he; ++fe) {
      switch (J = _[fe], (q !== 1 || G !== 1) && (J = scale$1$1(J, q, G)), J[0]) {
        // first letter
        case "l":
          te += J[1], ne += J[2], O.lineTo(te + H, ne + V);
          break;
        case "L":
          te = J[1], ne = J[2], O.lineTo(te + H, ne + V);
          break;
        case "h":
          te += J[1], O.lineTo(te + H, ne + V);
          break;
        case "H":
          te = J[1], O.lineTo(te + H, ne + V);
          break;
        case "v":
          ne += J[1], O.lineTo(te + H, ne + V);
          break;
        case "V":
          ne = J[1], O.lineTo(te + H, ne + V);
          break;
        case "m":
          te += J[1], ne += J[2], ue = te, de = ne, O.moveTo(te + H, ne + V);
          break;
        case "M":
          te = J[1], ne = J[2], ue = te, de = ne, O.moveTo(te + H, ne + V);
          break;
        case "c":
          oe = te + J[5], se = ne + J[6], ee = te + J[3], ae = ne + J[4], O.bezierCurveTo(
            te + J[1] + H,
            // x1
            ne + J[2] + V,
            // y1
            ee + H,
            // x2
            ae + V,
            // y2
            oe + H,
            se + V
          ), te = oe, ne = se;
          break;
        case "C":
          te = J[5], ne = J[6], ee = J[3], ae = J[4], O.bezierCurveTo(J[1] + H, J[2] + V, ee + H, ae + V, te + H, ne + V);
          break;
        case "s":
          oe = te + J[3], se = ne + J[4], ee = 2 * te - ee, ae = 2 * ne - ae, O.bezierCurveTo(ee + H, ae + V, te + J[1] + H, ne + J[2] + V, oe + H, se + V), ee = te + J[1], ae = ne + J[2], te = oe, ne = se;
          break;
        case "S":
          oe = J[3], se = J[4], ee = 2 * te - ee, ae = 2 * ne - ae, O.bezierCurveTo(ee + H, ae + V, J[1] + H, J[2] + V, oe + H, se + V), te = oe, ne = se, ee = J[1], ae = J[2];
          break;
        case "q":
          oe = te + J[3], se = ne + J[4], ee = te + J[1], ae = ne + J[2], O.quadraticCurveTo(ee + H, ae + V, oe + H, se + V), te = oe, ne = se;
          break;
        case "Q":
          oe = J[3], se = J[4], O.quadraticCurveTo(J[1] + H, J[2] + V, oe + H, se + V), te = oe, ne = se, ee = J[1], ae = J[2];
          break;
        case "t":
          oe = te + J[1], se = ne + J[2], Z[0].match(/[QqTt]/) === null ? (ee = te, ae = ne) : Z[0] === "t" ? (ee = 2 * te - ce, ae = 2 * ne - le) : Z[0] === "q" && (ee = 2 * te - ee, ae = 2 * ne - ae), ce = ee, le = ae, O.quadraticCurveTo(ee + H, ae + V, oe + H, se + V), te = oe, ne = se, ee = te + J[1], ae = ne + J[2];
          break;
        case "T":
          oe = J[1], se = J[2], ee = 2 * te - ee, ae = 2 * ne - ae, O.quadraticCurveTo(ee + H, ae + V, oe + H, se + V), te = oe, ne = se;
          break;
        case "a":
          drawArc(O, te + H, ne + V, [J[1], J[2], J[3], J[4], J[5], J[6] + te + H, J[7] + ne + V]), te += J[6], ne += J[7];
          break;
        case "A":
          drawArc(O, te + H, ne + V, [J[1], J[2], J[3], J[4], J[5], J[6] + H, J[7] + V]), te = J[6], ne = J[7];
          break;
        case "z":
        case "Z":
          te = ue, ne = de, O.closePath();
          break;
      }
      Z = J;
    }
  }
  function drawArc(O, _, H, V) {
    const q = segments(
      V[5],
      // end x
      V[6],
      // end y
      V[0],
      // radius x
      V[1],
      // radius y
      V[3],
      // large flag
      V[4],
      // sweep flag
      V[2],
      // rotation
      _,
      H
    );
    for (let G = 0; G < q.length; ++G) {
      const J = bezier(q[G]);
      O.bezierCurveTo(J[0], J[1], J[2], J[3], J[4], J[5]);
    }
  }
  const Tan30 = 0.5773502691896257, builtins = {
    circle: {
      draw: function(O, _) {
        const H = Math.sqrt(_) / 2;
        O.moveTo(H, 0), O.arc(0, 0, H, 0, Tau);
      }
    },
    cross: {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = H / 2.5;
        O.moveTo(-H, -V), O.lineTo(-H, V), O.lineTo(-V, V), O.lineTo(-V, H), O.lineTo(V, H), O.lineTo(V, V), O.lineTo(H, V), O.lineTo(H, -V), O.lineTo(V, -V), O.lineTo(V, -H), O.lineTo(-V, -H), O.lineTo(-V, -V), O.closePath();
      }
    },
    diamond: {
      draw: function(O, _) {
        const H = Math.sqrt(_) / 2;
        O.moveTo(-H, 0), O.lineTo(0, -H), O.lineTo(H, 0), O.lineTo(0, H), O.closePath();
      }
    },
    square: {
      draw: function(O, _) {
        var H = Math.sqrt(_), V = -H / 2;
        O.rect(V, V, H, H);
      }
    },
    arrow: {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = H / 7, q = H / 2.5, G = H / 8;
        O.moveTo(-V, H), O.lineTo(V, H), O.lineTo(V, -G), O.lineTo(q, -G), O.lineTo(0, -H), O.lineTo(-q, -G), O.lineTo(-V, -G), O.closePath();
      }
    },
    wedge: {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = HalfSqrt3 * H, q = V - H * Tan30, G = H / 4;
        O.moveTo(0, -V - q), O.lineTo(-G, V - q), O.lineTo(G, V - q), O.closePath();
      }
    },
    triangle: {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = HalfSqrt3 * H, q = V - H * Tan30;
        O.moveTo(0, -V - q), O.lineTo(-H, V - q), O.lineTo(H, V - q), O.closePath();
      }
    },
    "triangle-up": {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = HalfSqrt3 * H;
        O.moveTo(0, -V), O.lineTo(-H, V), O.lineTo(H, V), O.closePath();
      }
    },
    "triangle-down": {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = HalfSqrt3 * H;
        O.moveTo(0, V), O.lineTo(-H, -V), O.lineTo(H, -V), O.closePath();
      }
    },
    "triangle-right": {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = HalfSqrt3 * H;
        O.moveTo(V, 0), O.lineTo(-V, -H), O.lineTo(-V, H), O.closePath();
      }
    },
    "triangle-left": {
      draw: function(O, _) {
        var H = Math.sqrt(_) / 2, V = HalfSqrt3 * H;
        O.moveTo(-V, 0), O.lineTo(V, -H), O.lineTo(V, H), O.closePath();
      }
    },
    stroke: {
      draw: function(O, _) {
        const H = Math.sqrt(_) / 2;
        O.moveTo(-H, 0), O.lineTo(H, 0);
      }
    }
  };
  function symbols$1(O) {
    return has$1(builtins, O) ? builtins[O] : customSymbol(O);
  }
  var custom = {};
  function customSymbol(O) {
    if (!has$1(custom, O)) {
      const _ = parse$3(O);
      custom[O] = {
        draw: function(H, V) {
          pathRender(H, _, 0, 0, Math.sqrt(V) / 2);
        }
      };
    }
    return custom[O];
  }
  const C$1 = 0.448084975506;
  function rectangleX(O) {
    return O.x;
  }
  function rectangleY(O) {
    return O.y;
  }
  function rectangleWidth(O) {
    return O.width;
  }
  function rectangleHeight(O) {
    return O.height;
  }
  function number$4(O) {
    return typeof O == "function" ? O : () => +O;
  }
  function clamp(O, _, H) {
    return Math.max(_, Math.min(O, H));
  }
  function vg_rect() {
    var O = rectangleX, _ = rectangleY, H = rectangleWidth, V = rectangleHeight, q = number$4(0), G = q, J = q, Z = q, te = null;
    function ne(ee, ae, oe) {
      var se, ce = ae ?? +O.call(this, ee), le = oe ?? +_.call(this, ee), ue = +H.call(this, ee), de = +V.call(this, ee), fe = Math.min(ue, de) / 2, he = clamp(+q.call(this, ee), 0, fe), me = clamp(+G.call(this, ee), 0, fe), pe = clamp(+J.call(this, ee), 0, fe), ye = clamp(+Z.call(this, ee), 0, fe);
      if (te || (te = se = path$3()), he <= 0 && me <= 0 && pe <= 0 && ye <= 0)
        te.rect(ce, le, ue, de);
      else {
        var ge = ce + ue, ve = le + de;
        te.moveTo(ce + he, le), te.lineTo(ge - me, le), te.bezierCurveTo(ge - C$1 * me, le, ge, le + C$1 * me, ge, le + me), te.lineTo(ge, ve - ye), te.bezierCurveTo(ge, ve - C$1 * ye, ge - C$1 * ye, ve, ge - ye, ve), te.lineTo(ce + pe, ve), te.bezierCurveTo(ce + C$1 * pe, ve, ce, ve - C$1 * pe, ce, ve - pe), te.lineTo(ce, le + he), te.bezierCurveTo(ce, le + C$1 * he, ce + C$1 * he, le, ce + he, le), te.closePath();
      }
      if (se)
        return te = null, se + "" || null;
    }
    return ne.x = function(ee) {
      return arguments.length ? (O = number$4(ee), ne) : O;
    }, ne.y = function(ee) {
      return arguments.length ? (_ = number$4(ee), ne) : _;
    }, ne.width = function(ee) {
      return arguments.length ? (H = number$4(ee), ne) : H;
    }, ne.height = function(ee) {
      return arguments.length ? (V = number$4(ee), ne) : V;
    }, ne.cornerRadius = function(ee, ae, oe, se) {
      return arguments.length ? (q = number$4(ee), G = ae != null ? number$4(ae) : q, Z = oe != null ? number$4(oe) : q, J = se != null ? number$4(se) : G, ne) : q;
    }, ne.context = function(ee) {
      return arguments.length ? (te = ee ?? null, ne) : te;
    }, ne;
  }
  function vg_trail() {
    var O, _, H, V, q = null, G, J, Z, te;
    function ne(ae, oe, se) {
      const ce = se / 2;
      if (G) {
        var le = Z - oe, ue = ae - J;
        if (le || ue) {
          var de = Math.hypot(le, ue), fe = (le /= de) * te, he = (ue /= de) * te, me = Math.atan2(ue, le);
          q.moveTo(J - fe, Z - he), q.lineTo(ae - le * ce, oe - ue * ce), q.arc(ae, oe, ce, me - Math.PI, me), q.lineTo(J + fe, Z + he), q.arc(J, Z, te, me, me + Math.PI);
        } else
          q.arc(ae, oe, ce, 0, Tau);
        q.closePath();
      } else
        G = 1;
      J = ae, Z = oe, te = ce;
    }
    function ee(ae) {
      var oe, se = ae.length, ce, le = !1, ue;
      for (q == null && (q = ue = path$3()), oe = 0; oe <= se; ++oe)
        !(oe < se && V(ce = ae[oe], oe, ae)) === le && (le = !le) && (G = 0), le && ne(+O(ce, oe, ae), +_(ce, oe, ae), +H(ce, oe, ae));
      if (ue)
        return q = null, ue + "" || null;
    }
    return ee.x = function(ae) {
      return arguments.length ? (O = ae, ee) : O;
    }, ee.y = function(ae) {
      return arguments.length ? (_ = ae, ee) : _;
    }, ee.size = function(ae) {
      return arguments.length ? (H = ae, ee) : H;
    }, ee.defined = function(ae) {
      return arguments.length ? (V = ae, ee) : V;
    }, ee.context = function(ae) {
      return arguments.length ? (ae == null ? q = null : q = ae, ee) : q;
    }, ee;
  }
  function value$1(O, _) {
    return O ?? _;
  }
  const x$3 = (O) => O.x || 0, y$2 = (O) => O.y || 0, w$1 = (O) => O.width || 0, h$1 = (O) => O.height || 0, xw = (O) => (O.x || 0) + (O.width || 0), yh = (O) => (O.y || 0) + (O.height || 0), sa = (O) => O.startAngle || 0, ea = (O) => O.endAngle || 0, pa = (O) => O.padAngle || 0, ir = (O) => O.innerRadius || 0, or = (O) => O.outerRadius || 0, cr = (O) => O.cornerRadius || 0, tl = (O) => value$1(O.cornerRadiusTopLeft, O.cornerRadius) || 0, tr = (O) => value$1(O.cornerRadiusTopRight, O.cornerRadius) || 0, br = (O) => value$1(O.cornerRadiusBottomRight, O.cornerRadius) || 0, bl = (O) => value$1(O.cornerRadiusBottomLeft, O.cornerRadius) || 0, sz = (O) => value$1(O.size, 64), ts = (O) => O.size || 1, def = (O) => O.defined !== !1, type$1 = (O) => symbols$1(O.shape || "circle"), arcShape = arc$2$1().startAngle(sa).endAngle(ea).padAngle(pa).innerRadius(ir).outerRadius(or).cornerRadius(cr), areavShape = area$2$1().x(x$3).y1(y$2).y0(yh).defined(def), areahShape = area$2$1().y(y$2).x1(x$3).x0(xw).defined(def), lineShape = line$2$1().x(x$3).y(y$2).defined(def), rectShape = vg_rect().x(x$3).y(y$2).width(w$1).height(h$1).cornerRadius(tl, tr, br, bl), symbolShape = Symbol$2().type(type$1).size(sz), trailShape = vg_trail().x(x$3).y(y$2).defined(def).size(ts);
  function hasCornerRadius(O) {
    return O.cornerRadius || O.cornerRadiusTopLeft || O.cornerRadiusTopRight || O.cornerRadiusBottomRight || O.cornerRadiusBottomLeft;
  }
  function arc$1$1(O, _) {
    return arcShape.context(O)(_);
  }
  function area$1$1(O, _) {
    const H = _[0], V = H.interpolate || "linear";
    return (H.orient === "horizontal" ? areahShape : areavShape).curve(curves(V, H.orient, H.tension)).context(O)(_);
  }
  function line$1$1(O, _) {
    const H = _[0], V = H.interpolate || "linear";
    return lineShape.curve(curves(V, H.orient, H.tension)).context(O)(_);
  }
  function rectangle(O, _, H, V) {
    return rectShape.context(O)(_, H, V);
  }
  function shape$1(O, _) {
    return (_.mark.shape || _.shape).context(O)(_);
  }
  function symbol$1(O, _) {
    return symbolShape.context(O)(_);
  }
  function trail$1(O, _) {
    return trailShape.context(O)(_);
  }
  var clip_id = 1;
  function resetSVGClipId() {
    clip_id = 1;
  }
  function clip$1$1(O, _, H) {
    var V = _.clip, q = O._defs, G = _.clip_id || (_.clip_id = "clip" + clip_id++), J = q.clipping[G] || (q.clipping[G] = {
      id: G
    });
    return isFunction$2(V) ? J.path = V(null) : hasCornerRadius(H) ? J.path = rectangle(null, H, 0, 0) : (J.width = H.width || 0, J.height = H.height || 0), "url(#" + G + ")";
  }
  function Bounds(O) {
    this.clear(), O && this.union(O);
  }
  Bounds.prototype = {
    clone() {
      return new Bounds(this);
    },
    clear() {
      return this.x1 = +Number.MAX_VALUE, this.y1 = +Number.MAX_VALUE, this.x2 = -Number.MAX_VALUE, this.y2 = -Number.MAX_VALUE, this;
    },
    empty() {
      return this.x1 === +Number.MAX_VALUE && this.y1 === +Number.MAX_VALUE && this.x2 === -Number.MAX_VALUE && this.y2 === -Number.MAX_VALUE;
    },
    equals(O) {
      return this.x1 === O.x1 && this.y1 === O.y1 && this.x2 === O.x2 && this.y2 === O.y2;
    },
    set(O, _, H, V) {
      return H < O ? (this.x2 = O, this.x1 = H) : (this.x1 = O, this.x2 = H), V < _ ? (this.y2 = _, this.y1 = V) : (this.y1 = _, this.y2 = V), this;
    },
    add(O, _) {
      return O < this.x1 && (this.x1 = O), _ < this.y1 && (this.y1 = _), O > this.x2 && (this.x2 = O), _ > this.y2 && (this.y2 = _), this;
    },
    expand(O) {
      return this.x1 -= O, this.y1 -= O, this.x2 += O, this.y2 += O, this;
    },
    round() {
      return this.x1 = Math.floor(this.x1), this.y1 = Math.floor(this.y1), this.x2 = Math.ceil(this.x2), this.y2 = Math.ceil(this.y2), this;
    },
    scale(O) {
      return this.x1 *= O, this.y1 *= O, this.x2 *= O, this.y2 *= O, this;
    },
    translate(O, _) {
      return this.x1 += O, this.x2 += O, this.y1 += _, this.y2 += _, this;
    },
    rotate(O, _, H) {
      const V = this.rotatedPoints(O, _, H);
      return this.clear().add(V[0], V[1]).add(V[2], V[3]).add(V[4], V[5]).add(V[6], V[7]);
    },
    rotatedPoints(O, _, H) {
      var {
        x1: V,
        y1: q,
        x2: G,
        y2: J
      } = this, Z = Math.cos(O), te = Math.sin(O), ne = _ - _ * Z + H * te, ee = H - _ * te - H * Z;
      return [Z * V - te * q + ne, te * V + Z * q + ee, Z * V - te * J + ne, te * V + Z * J + ee, Z * G - te * q + ne, te * G + Z * q + ee, Z * G - te * J + ne, te * G + Z * J + ee];
    },
    union(O) {
      return O.x1 < this.x1 && (this.x1 = O.x1), O.y1 < this.y1 && (this.y1 = O.y1), O.x2 > this.x2 && (this.x2 = O.x2), O.y2 > this.y2 && (this.y2 = O.y2), this;
    },
    intersect(O) {
      return O.x1 > this.x1 && (this.x1 = O.x1), O.y1 > this.y1 && (this.y1 = O.y1), O.x2 < this.x2 && (this.x2 = O.x2), O.y2 < this.y2 && (this.y2 = O.y2), this;
    },
    encloses(O) {
      return O && this.x1 <= O.x1 && this.x2 >= O.x2 && this.y1 <= O.y1 && this.y2 >= O.y2;
    },
    alignsWith(O) {
      return O && (this.x1 == O.x1 || this.x2 == O.x2 || this.y1 == O.y1 || this.y2 == O.y2);
    },
    intersects(O) {
      return O && !(this.x2 < O.x1 || this.x1 > O.x2 || this.y2 < O.y1 || this.y1 > O.y2);
    },
    contains(O, _) {
      return !(O < this.x1 || O > this.x2 || _ < this.y1 || _ > this.y2);
    },
    width() {
      return this.x2 - this.x1;
    },
    height() {
      return this.y2 - this.y1;
    }
  };
  function Item(O) {
    this.mark = O, this.bounds = this.bounds || new Bounds();
  }
  function GroupItem(O) {
    Item.call(this, O), this.items = this.items || [];
  }
  inherits(GroupItem, Item);
  class ResourceLoader {
    constructor(_) {
      this._pending = 0, this._loader = _ || loader();
    }
    pending() {
      return this._pending;
    }
    sanitizeURL(_) {
      const H = this;
      return increment(H), H._loader.sanitize(_, {
        context: "href"
      }).then((V) => (decrement(H), V)).catch(() => (decrement(H), null));
    }
    loadImage(_) {
      const H = this, V = domImage();
      return increment(H), H._loader.sanitize(_, {
        context: "image"
      }).then((q) => {
        const G = q.href;
        if (!G || !V) throw {
          url: G
        };
        const J = new V(), Z = has$1(q, "crossOrigin") ? q.crossOrigin : "anonymous";
        return Z != null && (J.crossOrigin = Z), J.onload = () => decrement(H), J.onerror = () => decrement(H), J.src = G, J;
      }).catch((q) => (decrement(H), {
        complete: !1,
        width: 0,
        height: 0,
        src: q && q.url || ""
      }));
    }
    ready() {
      const _ = this;
      return new Promise((H) => {
        function V(q) {
          _.pending() ? setTimeout(() => {
            V(!0);
          }, 10) : H(q);
        }
        V(!1);
      });
    }
  }
  function increment(O) {
    O._pending += 1;
  }
  function decrement(O) {
    O._pending -= 1;
  }
  function boundStroke(O, _, H) {
    if (_.stroke && _.opacity !== 0 && _.strokeOpacity !== 0) {
      const V = _.strokeWidth != null ? +_.strokeWidth : 1;
      O.expand(V + (H ? miterAdjustment(_, V) : 0));
    }
    return O;
  }
  function miterAdjustment(O, _) {
    return O.strokeJoin && O.strokeJoin !== "miter" ? 0 : _;
  }
  const circleThreshold = Tau - 1e-8;
  let bounds, lx, ly, rot, ma, mb, mc, md;
  const add$3 = (O, _) => bounds.add(O, _), addL = (O, _) => add$3(lx = O, ly = _), addX = (O) => add$3(O, bounds.y1), addY = (O) => add$3(bounds.x1, O), px = (O, _) => ma * O + mc * _, py = (O, _) => mb * O + md * _, addp = (O, _) => add$3(px(O, _), py(O, _)), addpL = (O, _) => addL(px(O, _), py(O, _));
  function boundContext(O, _) {
    return bounds = O, _ ? (rot = _ * DegToRad, ma = md = Math.cos(rot), mb = Math.sin(rot), mc = -mb) : (ma = md = 1, rot = mb = mc = 0), context$1;
  }
  const context$1 = {
    beginPath() {
    },
    closePath() {
    },
    moveTo: addpL,
    lineTo: addpL,
    rect(O, _, H, V) {
      rot ? (addp(O + H, _), addp(O + H, _ + V), addp(O, _ + V), addpL(O, _)) : (add$3(O + H, _ + V), addL(O, _));
    },
    quadraticCurveTo(O, _, H, V) {
      const q = px(O, _), G = py(O, _), J = px(H, V), Z = py(H, V);
      quadExtrema(lx, q, J, addX), quadExtrema(ly, G, Z, addY), addL(J, Z);
    },
    bezierCurveTo(O, _, H, V, q, G) {
      const J = px(O, _), Z = py(O, _), te = px(H, V), ne = py(H, V), ee = px(q, G), ae = py(q, G);
      cubicExtrema(lx, J, te, ee, addX), cubicExtrema(ly, Z, ne, ae, addY), addL(ee, ae);
    },
    arc(O, _, H, V, q, G) {
      if (V += rot, q += rot, lx = H * Math.cos(q) + O, ly = H * Math.sin(q) + _, Math.abs(q - V) > circleThreshold)
        add$3(O - H, _ - H), add$3(O + H, _ + H);
      else {
        const J = (ne) => add$3(H * Math.cos(ne) + O, H * Math.sin(ne) + _);
        let Z, te;
        if (J(V), J(q), q !== V)
          if (V = V % Tau, V < 0 && (V += Tau), q = q % Tau, q < 0 && (q += Tau), q < V && (G = !G, Z = V, V = q, q = Z), G)
            for (q -= Tau, Z = V - V % HalfPi, te = 0; te < 4 && Z > q; ++te, Z -= HalfPi) J(Z);
          else
            for (Z = V - V % HalfPi + HalfPi, te = 0; te < 4 && Z < q; ++te, Z = Z + HalfPi) J(Z);
      }
    }
  };
  function quadExtrema(O, _, H, V) {
    const q = (O - _) / (O + H - 2 * _);
    0 < q && q < 1 && V(O + (_ - O) * q);
  }
  function cubicExtrema(O, _, H, V, q) {
    const G = V - O + 3 * _ - 3 * H, J = O + H - 2 * _, Z = O - _;
    let te = 0, ne = 0, ee;
    Math.abs(G) > Epsilon$1 ? (ee = J * J + Z * G, ee >= 0 && (ee = Math.sqrt(ee), te = (-J + ee) / G, ne = (-J - ee) / G)) : te = 0.5 * Z / J, 0 < te && te < 1 && q(cubic(te, O, _, H, V)), 0 < ne && ne < 1 && q(cubic(ne, O, _, H, V));
  }
  function cubic(O, _, H, V, q) {
    const G = 1 - O, J = G * G, Z = O * O;
    return J * G * _ + 3 * J * O * H + 3 * G * Z * V + Z * O * q;
  }
  var context$2 = (context$2 = domCanvas(1, 1)) ? context$2.getContext("2d") : null;
  const b = new Bounds();
  function intersectPath(O) {
    return function(_, H) {
      if (!context$2) return !0;
      O(context$2, _), b.clear().union(_.bounds).intersect(H).round();
      const {
        x1: V,
        y1: q,
        x2: G,
        y2: J
      } = b;
      for (let Z = q; Z <= J; ++Z)
        for (let te = V; te <= G; ++te)
          if (context$2.isPointInPath(te, Z))
            return !0;
      return !1;
    };
  }
  function intersectPoint(O, _) {
    return _.contains(O.x || 0, O.y || 0);
  }
  function intersectRect(O, _) {
    const H = O.x || 0, V = O.y || 0, q = O.width || 0, G = O.height || 0;
    return _.intersects(b.set(H, V, H + q, V + G));
  }
  function intersectRule(O, _) {
    const H = O.x || 0, V = O.y || 0, q = O.x2 != null ? O.x2 : H, G = O.y2 != null ? O.y2 : V;
    return intersectBoxLine(_, H, V, q, G);
  }
  function intersectBoxLine(O, _, H, V, q) {
    const {
      x1: G,
      y1: J,
      x2: Z,
      y2: te
    } = O, ne = V - _, ee = q - H;
    let ae = 0, oe = 1, se, ce, le, ue;
    for (ue = 0; ue < 4; ++ue) {
      if (ue === 0 && (se = -ne, ce = -(G - _)), ue === 1 && (se = ne, ce = Z - _), ue === 2 && (se = -ee, ce = -(J - H)), ue === 3 && (se = ee, ce = te - H), Math.abs(se) < 1e-10 && ce < 0) return !1;
      if (le = ce / se, se < 0) {
        if (le > oe) return !1;
        le > ae && (ae = le);
      } else if (se > 0) {
        if (le < ae) return !1;
        le < oe && (oe = le);
      }
    }
    return !0;
  }
  function blend(O, _) {
    O.globalCompositeOperation = _.blend || "source-over";
  }
  function value$2(O, _) {
    return O ?? _;
  }
  function addStops(O, _) {
    const H = _.length;
    for (let V = 0; V < H; ++V)
      O.addColorStop(_[V].offset, _[V].color);
    return O;
  }
  function gradient$2(O, _, H) {
    const V = H.width(), q = H.height();
    let G;
    if (_.gradient === "radial")
      G = O.createRadialGradient(H.x1 + value$2(_.x1, 0.5) * V, H.y1 + value$2(_.y1, 0.5) * q, Math.max(V, q) * value$2(_.r1, 0), H.x1 + value$2(_.x2, 0.5) * V, H.y1 + value$2(_.y2, 0.5) * q, Math.max(V, q) * value$2(_.r2, 0.5));
    else {
      const J = value$2(_.x1, 0), Z = value$2(_.y1, 0), te = value$2(_.x2, 1), ne = value$2(_.y2, 0);
      if (J === te || Z === ne || V === q)
        G = O.createLinearGradient(H.x1 + J * V, H.y1 + Z * q, H.x1 + te * V, H.y1 + ne * q);
      else {
        const ee = domCanvas(Math.ceil(V), Math.ceil(q)), ae = ee.getContext("2d");
        return ae.scale(V, q), ae.fillStyle = addStops(ae.createLinearGradient(J, Z, te, ne), _.stops), ae.fillRect(0, 0, V, q), O.createPattern(ee, "no-repeat");
      }
    }
    return addStops(G, _.stops);
  }
  function color$2(O, _, H) {
    return isGradient(H) ? gradient$2(O, H, _.bounds) : H;
  }
  function fill$1(O, _, H) {
    return H *= _.fillOpacity == null ? 1 : _.fillOpacity, H > 0 ? (O.globalAlpha = H, O.fillStyle = color$2(O, _, _.fill), !0) : !1;
  }
  var Empty = [];
  function stroke(O, _, H) {
    var V = (V = _.strokeWidth) != null ? V : 1;
    return V <= 0 ? !1 : (H *= _.strokeOpacity == null ? 1 : _.strokeOpacity, H > 0 ? (O.globalAlpha = H, O.strokeStyle = color$2(O, _, _.stroke), O.lineWidth = V, O.lineCap = _.strokeCap || "butt", O.lineJoin = _.strokeJoin || "miter", O.miterLimit = _.strokeMiterLimit || 10, O.setLineDash && (O.setLineDash(_.strokeDash || Empty), O.lineDashOffset = _.strokeDashOffset || 0), !0) : !1);
  }
  function compare$2(O, _) {
    return O.zindex - _.zindex || O.index - _.index;
  }
  function zorder(O) {
    if (!O.zdirty) return O.zitems;
    var _ = O.items, H = [], V, q, G;
    for (q = 0, G = _.length; q < G; ++q)
      V = _[q], V.index = q, V.zindex && H.push(V);
    return O.zdirty = !1, O.zitems = H.sort(compare$2);
  }
  function visit(O, _) {
    var H = O.items, V, q;
    if (!H || !H.length) return;
    const G = zorder(O);
    if (G && G.length) {
      for (V = 0, q = H.length; V < q; ++V)
        H[V].zindex || _(H[V]);
      H = G;
    }
    for (V = 0, q = H.length; V < q; ++V)
      _(H[V]);
  }
  function pickVisit(O, _) {
    var H = O.items, V, q;
    if (!H || !H.length) return null;
    const G = zorder(O);
    for (G && G.length && (H = G), q = H.length; --q >= 0; )
      if (V = _(H[q])) return V;
    if (H === G) {
      for (H = O.items, q = H.length; --q >= 0; )
        if (!H[q].zindex && (V = _(H[q])))
          return V;
    }
    return null;
  }
  function drawAll(O) {
    return function(_, H, V) {
      visit(H, (q) => {
        (!V || V.intersects(q.bounds)) && drawPath(O, _, q, q);
      });
    };
  }
  function drawOne(O) {
    return function(_, H, V) {
      H.items.length && (!V || V.intersects(H.bounds)) && drawPath(O, _, H.items[0], H.items);
    };
  }
  function drawPath(O, _, H, V) {
    var q = H.opacity == null ? 1 : H.opacity;
    q !== 0 && (O(_, V) || (blend(_, H), H.fill && fill$1(_, H, q) && _.fill(), H.stroke && stroke(_, H, q) && _.stroke()));
  }
  function pick$1$1(O) {
    return O = O || truthy, function(_, H, V, q, G, J) {
      return V *= _.pixelRatio, q *= _.pixelRatio, pickVisit(H, (Z) => {
        const te = Z.bounds;
        if (!(te && !te.contains(G, J) || !te) && O(_, Z, V, q, G, J))
          return Z;
      });
    };
  }
  function hitPath(O, _) {
    return function(H, V, q, G) {
      var J = Array.isArray(V) ? V[0] : V, Z = _ ?? J.fill, te = J.stroke && H.isPointInStroke, ne, ee;
      return te && (ne = J.strokeWidth, ee = J.strokeCap, H.lineWidth = ne ?? 1, H.lineCap = ee ?? "butt"), O(H, V) ? !1 : Z && H.isPointInPath(q, G) || te && H.isPointInStroke(q, G);
    };
  }
  function pickPath(O) {
    return pick$1$1(hitPath(O));
  }
  function translate$2(O, _) {
    return "translate(" + O + "," + _ + ")";
  }
  function rotate(O) {
    return "rotate(" + O + ")";
  }
  function scale$3(O, _) {
    return "scale(" + O + "," + _ + ")";
  }
  function translateItem(O) {
    return translate$2(O.x || 0, O.y || 0);
  }
  function rotateItem(O) {
    return translate$2(O.x || 0, O.y || 0) + (O.angle ? " " + rotate(O.angle) : "");
  }
  function transformItem(O) {
    return translate$2(O.x || 0, O.y || 0) + (O.angle ? " " + rotate(O.angle) : "") + (O.scaleX || O.scaleY ? " " + scale$3(O.scaleX || 1, O.scaleY || 1) : "");
  }
  function markItemPath(O, _, H) {
    function V(J, Z) {
      J("transform", rotateItem(Z)), J("d", _(null, Z));
    }
    function q(J, Z) {
      return _(boundContext(J, Z.angle), Z), boundStroke(J, Z).translate(Z.x || 0, Z.y || 0);
    }
    function G(J, Z) {
      var te = Z.x || 0, ne = Z.y || 0, ee = Z.angle || 0;
      J.translate(te, ne), ee && J.rotate(ee *= DegToRad), J.beginPath(), _(J, Z), ee && J.rotate(-ee), J.translate(-te, -ne);
    }
    return {
      type: O,
      tag: "path",
      nested: !1,
      attr: V,
      bound: q,
      draw: drawAll(G),
      pick: pickPath(G),
      isect: H || intersectPath(G)
    };
  }
  var arc$2 = markItemPath("arc", arc$1$1);
  function pickArea(O, _) {
    for (var H = O[0].orient === "horizontal" ? _[1] : _[0], V = O[0].orient === "horizontal" ? "y" : "x", q = O.length, G = 1 / 0, J, Z; --q >= 0; )
      O[q].defined !== !1 && (Z = Math.abs(O[q][V] - H), Z < G && (G = Z, J = O[q]));
    return J;
  }
  function pickLine(O, _) {
    for (var H = Math.pow(O[0].strokeWidth || 1, 2), V = O.length, q, G, J; --V >= 0; )
      if (O[V].defined !== !1 && (q = O[V].x - _[0], G = O[V].y - _[1], J = q * q + G * G, J < H))
        return O[V];
    return null;
  }
  function pickTrail(O, _) {
    for (var H = O.length, V, q, G; --H >= 0; )
      if (O[H].defined !== !1 && (V = O[H].x - _[0], q = O[H].y - _[1], G = V * V + q * q, V = O[H].size || 1, G < V * V))
        return O[H];
    return null;
  }
  function markMultiItemPath(O, _, H) {
    function V(te, ne) {
      var ee = ne.mark.items;
      ee.length && te("d", _(null, ee));
    }
    function q(te, ne) {
      var ee = ne.items;
      return ee.length === 0 ? te : (_(boundContext(te), ee), boundStroke(te, ee[0]));
    }
    function G(te, ne) {
      te.beginPath(), _(te, ne);
    }
    const J = hitPath(G);
    function Z(te, ne, ee, ae, oe, se) {
      var ce = ne.items, le = ne.bounds;
      return !ce || !ce.length || le && !le.contains(oe, se) ? null : (ee *= te.pixelRatio, ae *= te.pixelRatio, J(te, ce, ee, ae) ? ce[0] : null);
    }
    return {
      type: O,
      tag: "path",
      nested: !0,
      attr: V,
      bound: q,
      draw: drawOne(G),
      pick: Z,
      isect: intersectPoint,
      tip: H
    };
  }
  var area$2 = markMultiItemPath("area", area$1$1, pickArea);
  function clip$3(O, _) {
    var H = _.clip;
    O.save(), isFunction$2(H) ? (O.beginPath(), H(O), O.clip()) : clipGroup(O, _.group);
  }
  function clipGroup(O, _) {
    O.beginPath(), hasCornerRadius(_) ? rectangle(O, _, 0, 0) : O.rect(0, 0, _.width || 0, _.height || 0), O.clip();
  }
  function offset$1(O) {
    const _ = value$2(O.strokeWidth, 1);
    return O.strokeOffset != null ? O.strokeOffset : O.stroke && _ > 0.5 && _ < 1.5 ? 0.5 - Math.abs(_ - 1) : 0;
  }
  function attr$5(O, _) {
    O("transform", translateItem(_));
  }
  function emitRectangle(O, _) {
    const H = offset$1(_);
    O("d", rectangle(null, _, H, H));
  }
  function background$1(O, _) {
    O("class", "background"), O("aria-hidden", !0), emitRectangle(O, _);
  }
  function foreground(O, _) {
    O("class", "foreground"), O("aria-hidden", !0), _.strokeForeground ? emitRectangle(O, _) : O("d", "");
  }
  function content$1(O, _, H) {
    const V = _.clip ? clip$1$1(H, _, _) : null;
    O("clip-path", V);
  }
  function bound$5(O, _) {
    if (!_.clip && _.items) {
      const H = _.items, V = H.length;
      for (let q = 0; q < V; ++q)
        O.union(H[q].bounds);
    }
    return (_.clip || _.width || _.height) && !_.noBound && O.add(0, 0).add(_.width || 0, _.height || 0), boundStroke(O, _), O.translate(_.x || 0, _.y || 0);
  }
  function rectanglePath(O, _, H, V) {
    const q = offset$1(_);
    O.beginPath(), rectangle(O, _, (H || 0) + q, (V || 0) + q);
  }
  const hitBackground = hitPath(rectanglePath), hitForeground = hitPath(rectanglePath, !1), hitCorner = hitPath(rectanglePath, !0);
  function draw$4(O, _, H, V) {
    visit(_, (q) => {
      const G = q.x || 0, J = q.y || 0, Z = q.strokeForeground, te = q.opacity == null ? 1 : q.opacity;
      (q.stroke || q.fill) && te && (rectanglePath(O, q, G, J), blend(O, q), q.fill && fill$1(O, q, te) && O.fill(), q.stroke && !Z && stroke(O, q, te) && O.stroke()), O.save(), O.translate(G, J), q.clip && clipGroup(O, q), H && H.translate(-G, -J), visit(q, (ne) => {
        (ne.marktype === "group" || V == null || V.includes(ne.marktype)) && this.draw(O, ne, H, V);
      }), H && H.translate(G, J), O.restore(), Z && q.stroke && te && (rectanglePath(O, q, G, J), blend(O, q), stroke(O, q, te) && O.stroke());
    });
  }
  function pick$2(O, _, H, V, q, G) {
    if (_.bounds && !_.bounds.contains(q, G) || !_.items)
      return null;
    const J = H * O.pixelRatio, Z = V * O.pixelRatio;
    return pickVisit(_, (te) => {
      let ne, ee, ae;
      const oe = te.bounds;
      if (oe && !oe.contains(q, G)) return;
      ee = te.x || 0, ae = te.y || 0;
      const se = ee + (te.width || 0), ce = ae + (te.height || 0), le = te.clip;
      if (le && (q < ee || q > se || G < ae || G > ce)) return;
      if (O.save(), O.translate(ee, ae), ee = q - ee, ae = G - ae, le && hasCornerRadius(te) && !hitCorner(O, te, J, Z))
        return O.restore(), null;
      const ue = te.strokeForeground, de = _.interactive !== !1;
      return de && ue && te.stroke && hitForeground(O, te, J, Z) ? (O.restore(), te) : (ne = pickVisit(te, (fe) => pickMark(fe, ee, ae) ? this.pick(fe, H, V, ee, ae) : null), !ne && de && (te.fill || !ue && te.stroke) && hitBackground(O, te, J, Z) && (ne = te), O.restore(), ne || null);
    });
  }
  function pickMark(O, _, H) {
    return (O.interactive !== !1 || O.marktype === "group") && O.bounds && O.bounds.contains(_, H);
  }
  var group = {
    type: "group",
    tag: "g",
    nested: !1,
    attr: attr$5,
    bound: bound$5,
    draw: draw$4,
    pick: pick$2,
    isect: intersectRect,
    content: content$1,
    background: background$1,
    foreground
  }, metadata = {
    xmlns: "http://www.w3.org/2000/svg",
    "xmlns:xlink": "http://www.w3.org/1999/xlink",
    version: "1.1"
  };
  function getImage(O, _) {
    var H = O.image;
    return (!H || O.url && O.url !== H.url) && (H = {
      complete: !1,
      width: 0,
      height: 0
    }, _.loadImage(O.url).then((V) => {
      O.image = V, O.image.url = O.url;
    })), H;
  }
  function imageWidth(O, _) {
    return O.width != null ? O.width : !_ || !_.width ? 0 : O.aspect !== !1 && O.height ? O.height * _.width / _.height : _.width;
  }
  function imageHeight(O, _) {
    return O.height != null ? O.height : !_ || !_.height ? 0 : O.aspect !== !1 && O.width ? O.width * _.height / _.width : _.height;
  }
  function imageXOffset(O, _) {
    return O === "center" ? _ / 2 : O === "right" ? _ : 0;
  }
  function imageYOffset(O, _) {
    return O === "middle" ? _ / 2 : O === "bottom" ? _ : 0;
  }
  function attr$4(O, _, H) {
    const V = getImage(_, H), q = imageWidth(_, V), G = imageHeight(_, V), J = (_.x || 0) - imageXOffset(_.align, q), Z = (_.y || 0) - imageYOffset(_.baseline, G), te = !V.src && V.toDataURL ? V.toDataURL() : V.src || "";
    O("href", te, metadata["xmlns:xlink"], "xlink:href"), O("transform", translate$2(J, Z)), O("width", q), O("height", G), O("preserveAspectRatio", _.aspect === !1 ? "none" : "xMidYMid");
  }
  function bound$4(O, _) {
    const H = _.image, V = imageWidth(_, H), q = imageHeight(_, H), G = (_.x || 0) - imageXOffset(_.align, V), J = (_.y || 0) - imageYOffset(_.baseline, q);
    return O.set(G, J, G + V, J + q);
  }
  function draw$3(O, _, H) {
    visit(_, (V) => {
      if (H && !H.intersects(V.bounds)) return;
      const q = getImage(V, this);
      let G = imageWidth(V, q), J = imageHeight(V, q);
      if (G === 0 || J === 0) return;
      let Z = (V.x || 0) - imageXOffset(V.align, G), te = (V.y || 0) - imageYOffset(V.baseline, J), ne, ee, ae, oe;
      V.aspect !== !1 && (ee = q.width / q.height, ae = V.width / V.height, ee === ee && ae === ae && ee !== ae && (ae < ee ? (oe = G / ee, te += (J - oe) / 2, J = oe) : (oe = J * ee, Z += (G - oe) / 2, G = oe))), (q.complete || q.toDataURL) && (blend(O, V), O.globalAlpha = (ne = V.opacity) != null ? ne : 1, O.imageSmoothingEnabled = V.smooth !== !1, O.drawImage(q, Z, te, G, J));
    });
  }
  var image$1 = {
    type: "image",
    tag: "image",
    nested: !1,
    attr: attr$4,
    bound: bound$4,
    draw: draw$3,
    pick: pick$1$1(),
    isect: truthy,
    // bounds check is sufficient
    get: getImage,
    xOffset: imageXOffset,
    yOffset: imageYOffset
  }, line$2 = markMultiItemPath("line", line$1$1, pickLine);
  function attr$3(O, _) {
    var H = _.scaleX || 1, V = _.scaleY || 1;
    (H !== 1 || V !== 1) && O("vector-effect", "non-scaling-stroke"), O("transform", transformItem(_)), O("d", _.path);
  }
  function path$1(O, _) {
    var H = _.path;
    if (H == null) return !0;
    var V = _.x || 0, q = _.y || 0, G = _.scaleX || 1, J = _.scaleY || 1, Z = (_.angle || 0) * DegToRad, te = _.pathCache;
    (!te || te.path !== H) && ((_.pathCache = te = parse$3(H)).path = H), Z && O.rotate && O.translate ? (O.translate(V, q), O.rotate(Z), pathRender(O, te, 0, 0, G, J), O.rotate(-Z), O.translate(-V, -q)) : pathRender(O, te, V, q, G, J);
  }
  function bound$3(O, _) {
    return path$1(boundContext(O, _.angle), _) ? O.set(0, 0, 0, 0) : boundStroke(O, _, !0);
  }
  var path$2 = {
    type: "path",
    tag: "path",
    nested: !1,
    attr: attr$3,
    bound: bound$3,
    draw: drawAll(path$1),
    pick: pickPath(path$1),
    isect: intersectPath(path$1)
  };
  function attr$2(O, _) {
    O("d", rectangle(null, _));
  }
  function bound$2(O, _) {
    var H, V;
    return boundStroke(O.set(H = _.x || 0, V = _.y || 0, H + _.width || 0, V + _.height || 0), _);
  }
  function draw$2(O, _) {
    O.beginPath(), rectangle(O, _);
  }
  var rect$1 = {
    type: "rect",
    tag: "path",
    nested: !1,
    attr: attr$2,
    bound: bound$2,
    draw: drawAll(draw$2),
    pick: pickPath(draw$2),
    isect: intersectRect
  };
  function attr$1(O, _) {
    O("transform", translateItem(_)), O("x2", _.x2 != null ? _.x2 - (_.x || 0) : 0), O("y2", _.y2 != null ? _.y2 - (_.y || 0) : 0);
  }
  function bound$1(O, _) {
    var H, V;
    return boundStroke(O.set(H = _.x || 0, V = _.y || 0, _.x2 != null ? _.x2 : H, _.y2 != null ? _.y2 : V), _);
  }
  function path(O, _, H) {
    var V, q, G, J;
    return _.stroke && stroke(O, _, H) ? (V = _.x || 0, q = _.y || 0, G = _.x2 != null ? _.x2 : V, J = _.y2 != null ? _.y2 : q, O.beginPath(), O.moveTo(V, q), O.lineTo(G, J), !0) : !1;
  }
  function draw$1(O, _, H) {
    visit(_, (V) => {
      if (!(H && !H.intersects(V.bounds))) {
        var q = V.opacity == null ? 1 : V.opacity;
        q && path(O, V, q) && (blend(O, V), O.stroke());
      }
    });
  }
  function hit$1(O, _, H, V) {
    return O.isPointInStroke ? path(O, _, 1) && O.isPointInStroke(H, V) : !1;
  }
  var rule$2 = {
    type: "rule",
    tag: "line",
    nested: !1,
    attr: attr$1,
    bound: bound$1,
    draw: draw$1,
    pick: pick$1$1(hit$1),
    isect: intersectRule
  }, shape = markItemPath("shape", shape$1), symbol = markItemPath("symbol", symbol$1, intersectPoint);
  const widthCache = lruCache();
  var textMetrics = {
    height: fontSize,
    measureWidth,
    estimateWidth,
    width: estimateWidth,
    canvas: useCanvas
  };
  useCanvas(!0);
  function useCanvas(O) {
    textMetrics.width = O && context$2 ? measureWidth : estimateWidth;
  }
  function estimateWidth(O, _) {
    return _estimateWidth(textValue(O, _), fontSize(O));
  }
  function _estimateWidth(O, _) {
    return ~~(0.8 * O.length * _);
  }
  function measureWidth(O, _) {
    return fontSize(O) <= 0 || !(_ = textValue(O, _)) ? 0 : _measureWidth(_, font$1(O));
  }
  function _measureWidth(O, _) {
    const H = `(${_}) ${O}`;
    let V = widthCache.get(H);
    return V === void 0 && (context$2.font = _, V = context$2.measureText(O).width, widthCache.set(H, V)), V;
  }
  function fontSize(O) {
    return O.fontSize != null ? +O.fontSize || 0 : 11;
  }
  function lineHeight(O) {
    return O.lineHeight != null ? O.lineHeight : fontSize(O) + 2;
  }
  function lineArray(O) {
    return isArray$2(O) ? O.length > 1 ? O : O[0] : O;
  }
  function textLines(O) {
    return lineArray(O.lineBreak && O.text && !isArray$2(O.text) ? O.text.split(O.lineBreak) : O.text);
  }
  function multiLineOffset(O) {
    const _ = textLines(O);
    return (isArray$2(_) ? _.length - 1 : 0) * lineHeight(O);
  }
  function textValue(O, _) {
    const H = _ == null ? "" : (_ + "").trim();
    return O.limit > 0 && H.length ? truncate(O, H) : H;
  }
  function widthGetter(O) {
    if (textMetrics.width === measureWidth) {
      const _ = font$1(O);
      return (H) => _measureWidth(H, _);
    } else if (textMetrics.width === estimateWidth) {
      const _ = fontSize(O);
      return (H) => _estimateWidth(H, _);
    } else
      return (_) => textMetrics.width(O, _);
  }
  function truncate(O, _) {
    var H = +O.limit, V = widthGetter(O);
    if (V(_) < H) return _;
    var q = O.ellipsis || "…", G = O.dir === "rtl", J = 0, Z = _.length, te;
    if (H -= V(q), G) {
      for (; J < Z; )
        te = J + Z >>> 1, V(_.slice(te)) > H ? J = te + 1 : Z = te;
      return q + _.slice(J);
    } else {
      for (; J < Z; )
        te = 1 + (J + Z >>> 1), V(_.slice(0, te)) < H ? J = te : Z = te - 1;
      return _.slice(0, J) + q;
    }
  }
  function fontFamily(O, _) {
    var H = O.font;
    return (_ && H ? String(H).replace(/"/g, "'") : H) || "sans-serif";
  }
  function font$1(O, _) {
    return (O.fontStyle ? O.fontStyle + " " : "") + (O.fontVariant ? O.fontVariant + " " : "") + (O.fontWeight ? O.fontWeight + " " : "") + fontSize(O) + "px " + fontFamily(O, _);
  }
  function offset$2(O) {
    var _ = O.baseline, H = fontSize(O);
    return Math.round(_ === "top" ? 0.79 * H : _ === "middle" ? 0.3 * H : _ === "bottom" ? -0.21 * H : _ === "line-top" ? 0.29 * H + 0.5 * lineHeight(O) : _ === "line-bottom" ? 0.29 * H - 0.5 * lineHeight(O) : 0);
  }
  const textAlign = {
    left: "start",
    center: "middle",
    right: "end"
  }, tempBounds$1 = new Bounds();
  function anchorPoint(O) {
    var _ = O.x || 0, H = O.y || 0, V = O.radius || 0, q;
    return V && (q = (O.theta || 0) - HalfPi, _ += V * Math.cos(q), H += V * Math.sin(q)), tempBounds$1.x1 = _, tempBounds$1.y1 = H, tempBounds$1;
  }
  function attr(O, _) {
    var H = _.dx || 0, V = (_.dy || 0) + offset$2(_), q = anchorPoint(_), G = q.x1, J = q.y1, Z = _.angle || 0, te;
    O("text-anchor", textAlign[_.align] || "start"), Z ? (te = translate$2(G, J) + " " + rotate(Z), (H || V) && (te += " " + translate$2(H, V))) : te = translate$2(G + H, J + V), O("transform", te);
  }
  function bound(O, _, H) {
    var V = textMetrics.height(_), q = _.align, G = anchorPoint(_), J = G.x1, Z = G.y1, te = _.dx || 0, ne = (_.dy || 0) + offset$2(_) - Math.round(0.8 * V), ee = textLines(_), ae;
    if (isArray$2(ee) ? (V += lineHeight(_) * (ee.length - 1), ae = ee.reduce((oe, se) => Math.max(oe, textMetrics.width(_, se)), 0)) : ae = textMetrics.width(_, ee), q === "center" ? te -= ae / 2 : q === "right" && (te -= ae), O.set(te += J, ne += Z, te + ae, ne + V), _.angle && !H)
      O.rotate(_.angle * DegToRad, J, Z);
    else if (H === 2)
      return O.rotatedPoints(_.angle * DegToRad, J, Z);
    return O;
  }
  function draw$5(O, _, H) {
    visit(_, (V) => {
      var q = V.opacity == null ? 1 : V.opacity, G, J, Z, te, ne, ee, ae;
      if (!(H && !H.intersects(V.bounds) || // bounds check
      q === 0 || V.fontSize <= 0 || V.text == null || V.text.length === 0)) {
        if (O.font = font$1(V), O.textAlign = V.align || "left", G = anchorPoint(V), J = G.x1, Z = G.y1, V.angle && (O.save(), O.translate(J, Z), O.rotate(V.angle * DegToRad), J = Z = 0), J += V.dx || 0, Z += (V.dy || 0) + offset$2(V), ee = textLines(V), blend(O, V), isArray$2(ee))
          for (ne = lineHeight(V), te = 0; te < ee.length; ++te)
            ae = textValue(V, ee[te]), V.fill && fill$1(O, V, q) && O.fillText(ae, J, Z), V.stroke && stroke(O, V, q) && O.strokeText(ae, J, Z), Z += ne;
        else
          ae = textValue(V, ee), V.fill && fill$1(O, V, q) && O.fillText(ae, J, Z), V.stroke && stroke(O, V, q) && O.strokeText(ae, J, Z);
        V.angle && O.restore();
      }
    });
  }
  function hit(O, _, H, V, q, G) {
    if (_.fontSize <= 0) return !1;
    if (!_.angle) return !0;
    var J = anchorPoint(_), Z = J.x1, te = J.y1, ne = bound(tempBounds$1, _, 1), ee = -_.angle * DegToRad, ae = Math.cos(ee), oe = Math.sin(ee), se = ae * q - oe * G + (Z - ae * Z + oe * te), ce = oe * q + ae * G + (te - oe * Z - ae * te);
    return ne.contains(se, ce);
  }
  function intersectText(O, _) {
    const H = bound(tempBounds$1, O, 2);
    return intersectBoxLine(_, H[0], H[1], H[2], H[3]) || intersectBoxLine(_, H[0], H[1], H[4], H[5]) || intersectBoxLine(_, H[4], H[5], H[6], H[7]) || intersectBoxLine(_, H[2], H[3], H[6], H[7]);
  }
  var text$2 = {
    type: "text",
    tag: "text",
    nested: !1,
    attr,
    bound,
    draw: draw$5,
    pick: pick$1$1(hit),
    isect: intersectText
  }, trail$2 = markMultiItemPath("trail", trail$1, pickTrail), Marks = {
    arc: arc$2,
    area: area$2,
    group,
    image: image$1,
    line: line$2,
    path: path$2,
    rect: rect$1,
    rule: rule$2,
    shape,
    symbol,
    text: text$2,
    trail: trail$2
  };
  function boundItem$1(O, _, H) {
    var V = Marks[O.mark.marktype], q = _ || V.bound;
    return V.nested && (O = O.mark), q(O.bounds || (O.bounds = new Bounds()), O, H);
  }
  var DUMMY = {
    mark: null
  };
  function boundMark(O, _, H) {
    var V = Marks[O.marktype], q = V.bound, G = O.items, J = G && G.length, Z, te, ne, ee;
    if (V.nested)
      return J ? ne = G[0] : (DUMMY.mark = O, ne = DUMMY), ee = boundItem$1(ne, q, H), _ = _ && _.union(ee) || ee, _;
    if (_ = _ || O.bounds && O.bounds.clear() || new Bounds(), J)
      for (Z = 0, te = G.length; Z < te; ++Z)
        _.union(boundItem$1(G[Z], q, H));
    return O.bounds = _;
  }
  const keys$4 = [
    "marktype",
    "name",
    "role",
    "interactive",
    "clip",
    "items",
    "zindex",
    "x",
    "y",
    "width",
    "height",
    "align",
    "baseline",
    // layout
    "fill",
    "fillOpacity",
    "opacity",
    "blend",
    // fill
    "stroke",
    "strokeOpacity",
    "strokeWidth",
    "strokeCap",
    // stroke
    "strokeDash",
    "strokeDashOffset",
    // stroke dash
    "strokeForeground",
    "strokeOffset",
    // group
    "startAngle",
    "endAngle",
    "innerRadius",
    "outerRadius",
    // arc
    "cornerRadius",
    "padAngle",
    // arc, rect
    "cornerRadiusTopLeft",
    "cornerRadiusTopRight",
    // rect, group
    "cornerRadiusBottomLeft",
    "cornerRadiusBottomRight",
    "interpolate",
    "tension",
    "orient",
    "defined",
    // area, line
    "url",
    "aspect",
    "smooth",
    // image
    "path",
    "scaleX",
    "scaleY",
    // path
    "x2",
    "y2",
    // rule
    "size",
    "shape",
    // symbol
    "text",
    "angle",
    "theta",
    "radius",
    "dir",
    "dx",
    "dy",
    // text
    "ellipsis",
    "limit",
    "lineBreak",
    "lineHeight",
    "font",
    "fontSize",
    "fontWeight",
    "fontStyle",
    "fontVariant",
    // font
    "description",
    "aria",
    "ariaRole",
    "ariaRoleDescription"
    // aria
  ];
  function sceneToJSON(O, _) {
    return JSON.stringify(O, keys$4, _);
  }
  function sceneFromJSON(O) {
    const _ = typeof O == "string" ? JSON.parse(O) : O;
    return initialize$1(_);
  }
  function initialize$1(O) {
    var _ = O.marktype, H = O.items, V, q, G;
    if (H)
      for (q = 0, G = H.length; q < G; ++q)
        V = _ ? "mark" : "group", H[q][V] = O, H[q].zindex && (H[q][V].zdirty = !0), (_ || V) === "group" && initialize$1(H[q]);
    return _ && boundMark(O), O;
  }
  class Scenegraph {
    constructor(_) {
      arguments.length ? this.root = sceneFromJSON(_) : (this.root = createMark({
        marktype: "group",
        name: "root",
        role: "frame"
      }), this.root.items = [new GroupItem(this.root)]);
    }
    toJSON(_) {
      return sceneToJSON(this.root, _ || 0);
    }
    mark(_, H, V) {
      H = H || this.root.items[0];
      const q = createMark(_, H);
      return H.items[V] = q, q.zindex && (q.group.zdirty = !0), q;
    }
  }
  function createMark(O, _) {
    const H = {
      bounds: new Bounds(),
      clip: !!O.clip,
      group: _,
      interactive: O.interactive !== !1,
      items: [],
      marktype: O.marktype,
      name: O.name || void 0,
      role: O.role || void 0,
      zindex: O.zindex || 0
    };
    return O.aria != null && (H.aria = O.aria), O.description && (H.description = O.description), H;
  }
  function domCreate(O, _, H) {
    return !O && typeof document < "u" && document.createElement && (O = document), O ? H ? O.createElementNS(H, _) : O.createElement(_) : null;
  }
  function domFind(O, _) {
    _ = _.toLowerCase();
    for (var H = O.childNodes, V = 0, q = H.length; V < q; ++V) if (H[V].tagName.toLowerCase() === _)
      return H[V];
  }
  function domChild(O, _, H, V) {
    var q = O.childNodes[_], G;
    return (!q || q.tagName.toLowerCase() !== H.toLowerCase()) && (G = q || null, q = domCreate(O.ownerDocument, H, V), O.insertBefore(q, G)), q;
  }
  function domClear(O, _) {
    for (var H = O.childNodes, V = H.length; V > _; ) O.removeChild(H[--V]);
    return O;
  }
  function cssClass(O) {
    return "mark-" + O.marktype + (O.role ? " role-" + O.role : "") + (O.name ? " " + O.name : "");
  }
  function point$2(O, _) {
    const H = _.getBoundingClientRect();
    return [O.clientX - H.left - (_.clientLeft || 0), O.clientY - H.top - (_.clientTop || 0)];
  }
  function resolveItem(O, _, H, V) {
    var q = O && O.mark, G, J;
    if (q && (G = Marks[q.marktype]).tip) {
      for (J = point$2(_, H), J[0] -= V[0], J[1] -= V[1]; O = O.mark.group; )
        J[0] -= O.x || 0, J[1] -= O.y || 0;
      O = G.tip(q.items, J);
    }
    return O;
  }
  let Handler$1 = class {
    /**
     * Create a new Handler instance.
     * @param {object} [customLoader] - Optional loader instance for
     *   href URL sanitization. If not specified, a standard loader
     *   instance will be generated.
     * @param {function} [customTooltip] - Optional tooltip handler
     *   function for custom tooltip display.
     * @constructor
     */
    constructor(_, H) {
      this._active = null, this._handlers = {}, this._loader = _ || loader(), this._tooltip = H || defaultTooltip$1;
    }
    /**
     * Initialize a new Handler instance.
     * @param {DOMElement} el - The containing DOM element for the display.
     * @param {Array<number>} origin - The origin of the display, in pixels.
     *   The coordinate system will be translated to this point.
     * @param {object} [obj] - Optional context object that should serve as
     *   the "this" context for event callbacks.
     * @return {Handler} - This handler instance.
     */
    initialize(_, H, V) {
      return this._el = _, this._obj = V || null, this.origin(H);
    }
    /**
     * Returns the parent container element for a visualization.
     * @return {DOMElement} - The containing DOM element.
     */
    element() {
      return this._el;
    }
    /**
     * Returns the scene element (e.g., canvas or SVG) of the visualization
     * Subclasses must override if the first child is not the scene element.
     * @return {DOMElement} - The scene (e.g., canvas or SVG) element.
     */
    canvas() {
      return this._el && this._el.firstChild;
    }
    /**
     * Get / set the origin coordinates of the visualization.
     */
    origin(_) {
      return arguments.length ? (this._origin = _ || [0, 0], this) : this._origin.slice();
    }
    /**
     * Get / set the scenegraph root.
     */
    scene(_) {
      return arguments.length ? (this._scene = _, this) : this._scene;
    }
    /**
     * Add an event handler. Subclasses should override this method.
     */
    on() {
    }
    /**
     * Remove an event handler. Subclasses should override this method.
     */
    off() {
    }
    /**
     * Utility method for finding the array index of an event handler.
     * @param {Array} h - An array of registered event handlers.
     * @param {string} type - The event type.
     * @param {function} handler - The event handler instance to find.
     * @return {number} - The handler's array index or -1 if not registered.
     */
    _handlerIndex(_, H, V) {
      for (let q = _ ? _.length : 0; --q >= 0; )
        if (_[q].type === H && (!V || _[q].handler === V))
          return q;
      return -1;
    }
    /**
     * Returns an array with registered event handlers.
     * @param {string} [type] - The event type to query. Any annotations
     *   are ignored; for example, for the argument "click.foo", ".foo" will
     *   be ignored and the method returns all "click" handlers. If type is
     *   null or unspecified, this method returns handlers for all types.
     * @return {Array} - A new array containing all registered event handlers.
     */
    handlers(_) {
      const H = this._handlers, V = [];
      if (_)
        V.push(...H[this.eventName(_)]);
      else
        for (const q in H)
          V.push(...H[q]);
      return V;
    }
    /**
     * Parses an event name string to return the specific event type.
     * For example, given "click.foo" returns "click"
     * @param {string} name - The input event type string.
     * @return {string} - A string with the event type only.
     */
    eventName(_) {
      const H = _.indexOf(".");
      return H < 0 ? _ : _.slice(0, H);
    }
    /**
     * Handle hyperlink navigation in response to an item.href value.
     * @param {Event} event - The event triggering hyperlink navigation.
     * @param {Item} item - The scenegraph item.
     * @param {string} href - The URL to navigate to.
     */
    handleHref(_, H, V) {
      this._loader.sanitize(V, {
        context: "href"
      }).then((q) => {
        const G = new MouseEvent(_.type, _), J = domCreate(null, "a");
        for (const Z in q) J.setAttribute(Z, q[Z]);
        J.dispatchEvent(G);
      }).catch(() => {
      });
    }
    /**
     * Handle tooltip display in response to an item.tooltip value.
     * @param {Event} event - The event triggering tooltip display.
     * @param {Item} item - The scenegraph item.
     * @param {boolean} show - A boolean flag indicating whether
     *   to show or hide a tooltip for the given item.
     */
    handleTooltip(_, H, V) {
      if (H && H.tooltip != null) {
        H = resolveItem(H, _, this.canvas(), this._origin);
        const q = V && H && H.tooltip || null;
        this._tooltip.call(this._obj, this, _, H, q);
      }
    }
    /**
     * Returns the size of a scenegraph item and its position relative
     * to the viewport.
     * @param {Item} item - The scenegraph item.
     * @return {object} - A bounding box object (compatible with the
     *   DOMRect type) consisting of x, y, width, heigh, top, left,
     *   right, and bottom properties.
     */
    getItemBoundingClientRect(_) {
      const H = this.canvas();
      if (!H) return;
      const V = H.getBoundingClientRect(), q = this._origin, G = _.bounds, J = G.width(), Z = G.height();
      let te = G.x1 + q[0] + V.left, ne = G.y1 + q[1] + V.top;
      for (; _.mark && (_ = _.mark.group); )
        te += _.x || 0, ne += _.y || 0;
      return {
        x: te,
        y: ne,
        width: J,
        height: Z,
        left: te,
        top: ne,
        right: te + J,
        bottom: ne + Z
      };
    }
  };
  function defaultTooltip$1(O, _, H, V) {
    O.element().setAttribute("title", V || "");
  }
  class Renderer {
    /**
     * Create a new Renderer instance.
     * @param {object} [loader] - Optional loader instance for
     *   image and href URL sanitization. If not specified, a
     *   standard loader instance will be generated.
     * @constructor
     */
    constructor(_) {
      this._el = null, this._bgcolor = null, this._loader = new ResourceLoader(_);
    }
    /**
     * Initialize a new Renderer instance.
     * @param {DOMElement} el - The containing DOM element for the display.
     * @param {number} width - The coordinate width of the display, in pixels.
     * @param {number} height - The coordinate height of the display, in pixels.
     * @param {Array<number>} origin - The origin of the display, in pixels.
     *   The coordinate system will be translated to this point.
     * @param {number} [scaleFactor=1] - Optional scaleFactor by which to multiply
     *   the width and height to determine the final pixel size.
     * @return {Renderer} - This renderer instance.
     */
    initialize(_, H, V, q, G) {
      return this._el = _, this.resize(H, V, q, G);
    }
    /**
     * Returns the parent container element for a visualization.
     * @return {DOMElement} - The containing DOM element.
     */
    element() {
      return this._el;
    }
    /**
     * Returns the scene element (e.g., canvas or SVG) of the visualization
     * Subclasses must override if the first child is not the scene element.
     * @return {DOMElement} - The scene (e.g., canvas or SVG) element.
     */
    canvas() {
      return this._el && this._el.firstChild;
    }
    /**
     * Get / set the background color.
     */
    background(_) {
      return arguments.length === 0 ? this._bgcolor : (this._bgcolor = _, this);
    }
    /**
     * Resize the display.
     * @param {number} width - The new coordinate width of the display, in pixels.
     * @param {number} height - The new coordinate height of the display, in pixels.
     * @param {Array<number>} origin - The new origin of the display, in pixels.
     *   The coordinate system will be translated to this point.
     * @param {number} [scaleFactor=1] - Optional scaleFactor by which to multiply
     *   the width and height to determine the final pixel size.
     * @return {Renderer} - This renderer instance;
     */
    resize(_, H, V, q) {
      return this._width = _, this._height = H, this._origin = V || [0, 0], this._scale = q || 1, this;
    }
    /**
     * Report a dirty item whose bounds should be redrawn.
     * This base class method does nothing. Subclasses that perform
     * incremental should implement this method.
     * @param {Item} item - The dirty item whose bounds should be redrawn.
     */
    dirty() {
    }
    /**
     * Render an input scenegraph, potentially with a set of dirty items.
     * This method will perform an immediate rendering with available resources.
     * The renderer may also need to perform image loading to perform a complete
     * render. This process can lead to asynchronous re-rendering of the scene
     * after this method returns. To receive notification when rendering is
     * complete, use the renderAsync method instead.
     * @param {object} scene - The root mark of a scenegraph to render.
     * @param {Array} markTypes - Array of the mark types to render.
     *                            If undefined, render all mark types
     * @return {Renderer} - This renderer instance.
     */
    render(_, H) {
      const V = this;
      return V._call = function() {
        V._render(_, H);
      }, V._call(), V._call = null, V;
    }
    /**
     * Internal rendering method. Renderer subclasses should override this
     * method to actually perform rendering.
     * @param {object} scene - The root mark of a scenegraph to render.
     * @param {Array} markTypes - Array of the mark types to render.
     *                            If undefined, render all mark types
     */
    _render() {
    }
    /**
     * Asynchronous rendering method. Similar to render, but returns a Promise
     * that resolves when all rendering is completed. Sometimes a renderer must
     * perform image loading to get a complete rendering. The returned
     * Promise will not resolve until this process completes.
     * @param {object} scene - The root mark of a scenegraph to render.
     * @param {Array} markTypes - Array of the mark types to render.
     *                            If undefined, render all mark types
     * @return {Promise} - A Promise that resolves when rendering is complete.
     */
    renderAsync(_, H) {
      const V = this.render(_, H);
      return this._ready ? this._ready.then(() => V) : Promise.resolve(V);
    }
    /**
     * Internal method for asynchronous resource loading.
     * Proxies method calls to the ImageLoader, and tracks loading
     * progress to invoke a re-render once complete.
     * @param {string} method - The method name to invoke on the ImageLoader.
     * @param {string} uri - The URI for the requested resource.
     * @return {Promise} - A Promise that resolves to the requested resource.
     */
    _load(_, H) {
      var V = this, q = V._loader[_](H);
      if (!V._ready) {
        const G = V._call;
        V._ready = V._loader.ready().then((J) => {
          J && G(), V._ready = null;
        });
      }
      return q;
    }
    /**
     * Sanitize a URL to include as a hyperlink in the rendered scene.
     * This method proxies a call to ImageLoader.sanitizeURL, but also tracks
     * image loading progress and invokes a re-render once complete.
     * @param {string} uri - The URI string to sanitize.
     * @return {Promise} - A Promise that resolves to the sanitized URL.
     */
    sanitizeURL(_) {
      return this._load("sanitizeURL", _);
    }
    /**
     * Requests an image to include in the rendered scene.
     * This method proxies a call to ImageLoader.loadImage, but also tracks
     * image loading progress and invokes a re-render once complete.
     * @param {string} uri - The URI string of the image.
     * @return {Promise} - A Promise that resolves to the loaded Image.
     */
    loadImage(_) {
      return this._load("loadImage", _);
    }
  }
  const KeyDownEvent = "keydown", KeyPressEvent = "keypress", KeyUpEvent = "keyup", DragEnterEvent = "dragenter", DragLeaveEvent = "dragleave", DragOverEvent = "dragover", PointerDownEvent = "pointerdown", PointerUpEvent = "pointerup", PointerMoveEvent = "pointermove", PointerOutEvent = "pointerout", PointerOverEvent = "pointerover", MouseDownEvent = "mousedown", MouseUpEvent = "mouseup", MouseMoveEvent = "mousemove", MouseOutEvent = "mouseout", MouseOverEvent = "mouseover", ClickEvent = "click", DoubleClickEvent = "dblclick", WheelEvent$1 = "wheel", MouseWheelEvent = "mousewheel", TouchStartEvent = "touchstart", TouchMoveEvent = "touchmove", TouchEndEvent = "touchend", Events = [KeyDownEvent, KeyPressEvent, KeyUpEvent, DragEnterEvent, DragLeaveEvent, DragOverEvent, PointerDownEvent, PointerUpEvent, PointerMoveEvent, PointerOutEvent, PointerOverEvent, MouseDownEvent, MouseUpEvent, MouseMoveEvent, MouseOutEvent, MouseOverEvent, ClickEvent, DoubleClickEvent, WheelEvent$1, MouseWheelEvent, TouchStartEvent, TouchMoveEvent, TouchEndEvent], TooltipShowEvent = PointerMoveEvent, TooltipHideEvent = MouseOutEvent, HrefEvent = ClickEvent;
  class CanvasHandler extends Handler$1 {
    constructor(_, H) {
      super(_, H), this._down = null, this._touch = null, this._first = !0, this._events = {}, this.events = Events, this.pointermove = move([PointerMoveEvent, MouseMoveEvent], [PointerOverEvent, MouseOverEvent], [PointerOutEvent, MouseOutEvent]), this.dragover = move([DragOverEvent], [DragEnterEvent], [DragLeaveEvent]), this.pointerout = inactive([PointerOutEvent, MouseOutEvent]), this.dragleave = inactive([DragLeaveEvent]);
    }
    initialize(_, H, V) {
      return this._canvas = _ && domFind(_, "canvas"), [ClickEvent, MouseDownEvent, PointerDownEvent, PointerMoveEvent, PointerOutEvent, DragLeaveEvent].forEach((q) => eventListenerCheck(this, q)), super.initialize(_, H, V);
    }
    // return the backing canvas instance
    canvas() {
      return this._canvas;
    }
    // retrieve the current canvas context
    context() {
      return this._canvas.getContext("2d");
    }
    // to keep old versions of firefox happy
    DOMMouseScroll(_) {
      this.fire(MouseWheelEvent, _);
    }
    pointerdown(_) {
      this._down = this._active, this.fire(PointerDownEvent, _);
    }
    mousedown(_) {
      this._down = this._active, this.fire(MouseDownEvent, _);
    }
    click(_) {
      this._down === this._active && (this.fire(ClickEvent, _), this._down = null);
    }
    touchstart(_) {
      this._touch = this.pickEvent(_.changedTouches[0]), this._first && (this._active = this._touch, this._first = !1), this.fire(TouchStartEvent, _, !0);
    }
    touchmove(_) {
      this.fire(TouchMoveEvent, _, !0);
    }
    touchend(_) {
      this.fire(TouchEndEvent, _, !0), this._touch = null;
    }
    // fire an event
    fire(_, H, V) {
      const q = V ? this._touch : this._active, G = this._handlers[_];
      if (H.vegaType = _, _ === HrefEvent && q && q.href ? this.handleHref(H, q, q.href) : (_ === TooltipShowEvent || _ === TooltipHideEvent) && this.handleTooltip(H, q, _ !== TooltipHideEvent), G)
        for (let J = 0, Z = G.length; J < Z; ++J)
          G[J].handler.call(this._obj, H, q);
    }
    // add an event handler
    on(_, H) {
      const V = this.eventName(_), q = this._handlers;
      return this._handlerIndex(q[V], _, H) < 0 && (eventListenerCheck(this, _), (q[V] || (q[V] = [])).push({
        type: _,
        handler: H
      })), this;
    }
    // remove an event handler
    off(_, H) {
      const V = this.eventName(_), q = this._handlers[V], G = this._handlerIndex(q, _, H);
      return G >= 0 && q.splice(G, 1), this;
    }
    pickEvent(_) {
      const H = point$2(_, this._canvas), V = this._origin;
      return this.pick(this._scene, H[0], H[1], H[0] - V[0], H[1] - V[1]);
    }
    // find the scenegraph item at the current pointer position
    // x, y -- the absolute x, y pointer coordinates on the canvas element
    // gx, gy -- the relative coordinates within the current group
    pick(_, H, V, q, G) {
      const J = this.context();
      return Marks[_.marktype].pick.call(this, J, _, H, V, q, G);
    }
  }
  const eventBundle = (O) => O === TouchStartEvent || O === TouchMoveEvent || O === TouchEndEvent ? [TouchStartEvent, TouchMoveEvent, TouchEndEvent] : [O];
  function eventListenerCheck(O, _) {
    eventBundle(_).forEach((H) => addEventListener$1(O, H));
  }
  function addEventListener$1(O, _) {
    const H = O.canvas();
    H && !O._events[_] && (O._events[_] = 1, H.addEventListener(_, O[_] ? (V) => O[_](V) : (V) => O.fire(_, V)));
  }
  function fireAll(O, _, H) {
    _.forEach((V) => O.fire(V, H));
  }
  function move(O, _, H) {
    return function(V) {
      const q = this._active, G = this.pickEvent(V);
      G === q ? fireAll(this, O, V) : ((!q || !q.exit) && fireAll(this, H, V), this._active = G, fireAll(this, _, V), fireAll(this, O, V));
    };
  }
  function inactive(O) {
    return function(_) {
      fireAll(this, O, _), this._active = null;
    };
  }
  function devicePixelRatio$1() {
    return typeof window < "u" && window.devicePixelRatio || 1;
  }
  function resize(O, _, H, V, q, G) {
    const J = typeof HTMLElement < "u" && O instanceof HTMLElement && O.parentNode != null, Z = O.getContext("2d"), te = J ? devicePixelRatio$1() : q;
    O.width = _ * te, O.height = H * te;
    for (const ne in G)
      Z[ne] = G[ne];
    return J && te !== 1 && (O.style.width = _ + "px", O.style.height = H + "px"), Z.pixelRatio = te, Z.setTransform(te, 0, 0, te, te * V[0], te * V[1]), O;
  }
  class CanvasRenderer extends Renderer {
    constructor(_) {
      super(_), this._options = {}, this._redraw = !1, this._dirty = new Bounds(), this._tempb = new Bounds();
    }
    initialize(_, H, V, q, G, J) {
      return this._options = J || {}, this._canvas = this._options.externalContext ? null : domCanvas(1, 1, this._options.type), _ && this._canvas && (domClear(_, 0).appendChild(this._canvas), this._canvas.setAttribute("class", "marks")), super.initialize(_, H, V, q, G);
    }
    resize(_, H, V, q) {
      if (super.resize(_, H, V, q), this._canvas)
        resize(this._canvas, this._width, this._height, this._origin, this._scale, this._options.context);
      else {
        const G = this._options.externalContext;
        G || error$1("CanvasRenderer is missing a valid canvas or context"), G.scale(this._scale, this._scale), G.translate(this._origin[0], this._origin[1]);
      }
      return this._redraw = !0, this;
    }
    canvas() {
      return this._canvas;
    }
    context() {
      return this._options.externalContext || (this._canvas ? this._canvas.getContext("2d") : null);
    }
    dirty(_) {
      const H = this._tempb.clear().union(_.bounds);
      let V = _.mark.group;
      for (; V; )
        H.translate(V.x || 0, V.y || 0), V = V.mark.group;
      this._dirty.union(H);
    }
    _render(_, H) {
      const V = this.context(), q = this._origin, G = this._width, J = this._height, Z = this._dirty, te = viewBounds(q, G, J);
      V.save();
      const ne = this._redraw || Z.empty() ? (this._redraw = !1, te.expand(1)) : clipToBounds(V, te.intersect(Z), q);
      return this.clear(-q[0], -q[1], G, J), this.draw(V, _, ne, H), V.restore(), Z.clear(), this;
    }
    draw(_, H, V, q) {
      if (H.marktype !== "group" && q != null && !q.includes(H.marktype))
        return;
      const G = Marks[H.marktype];
      H.clip && clip$3(_, H), G.draw.call(this, _, H, V, q), H.clip && _.restore();
    }
    clear(_, H, V, q) {
      const G = this._options, J = this.context();
      G.type !== "pdf" && !G.externalContext && J.clearRect(_, H, V, q), this._bgcolor != null && (J.fillStyle = this._bgcolor, J.fillRect(_, H, V, q));
    }
  }
  const viewBounds = (O, _, H) => new Bounds().set(0, 0, _, H).translate(-O[0], -O[1]);
  function clipToBounds(O, _, H) {
    return _.expand(1).round(), O.pixelRatio % 1 && _.scale(O.pixelRatio).round().scale(1 / O.pixelRatio), _.translate(-(H[0] % 1), -(H[1] % 1)), O.beginPath(), O.rect(_.x1, _.y1, _.width(), _.height()), O.clip(), _;
  }
  class SVGHandler extends Handler$1 {
    constructor(_, H) {
      super(_, H);
      const V = this;
      V._hrefHandler = listener(V, (q, G) => {
        G && G.href && V.handleHref(q, G, G.href);
      }), V._tooltipHandler = listener(V, (q, G) => {
        V.handleTooltip(q, G, q.type !== TooltipHideEvent);
      });
    }
    initialize(_, H, V) {
      let q = this._svg;
      return q && (q.removeEventListener(HrefEvent, this._hrefHandler), q.removeEventListener(TooltipShowEvent, this._tooltipHandler), q.removeEventListener(TooltipHideEvent, this._tooltipHandler)), this._svg = q = _ && domFind(_, "svg"), q && (q.addEventListener(HrefEvent, this._hrefHandler), q.addEventListener(TooltipShowEvent, this._tooltipHandler), q.addEventListener(TooltipHideEvent, this._tooltipHandler)), super.initialize(_, H, V);
    }
    canvas() {
      return this._svg;
    }
    // add an event handler
    on(_, H) {
      const V = this.eventName(_), q = this._handlers;
      if (this._handlerIndex(q[V], _, H) < 0) {
        const J = {
          type: _,
          handler: H,
          listener: listener(this, H)
        };
        (q[V] || (q[V] = [])).push(J), this._svg && this._svg.addEventListener(V, J.listener);
      }
      return this;
    }
    // remove an event handler
    off(_, H) {
      const V = this.eventName(_), q = this._handlers[V], G = this._handlerIndex(q, _, H);
      return G >= 0 && (this._svg && this._svg.removeEventListener(V, q[G].listener), q.splice(G, 1)), this;
    }
  }
  const listener = (O, _) => (H) => {
    let V = H.target.__data__;
    V = Array.isArray(V) ? V[0] : V, H.vegaType = H.type, _.call(O._obj, H, V);
  }, ARIA_HIDDEN = "aria-hidden", ARIA_LABEL = "aria-label", ARIA_ROLE = "role", ARIA_ROLEDESCRIPTION = "aria-roledescription", GRAPHICS_OBJECT = "graphics-object", GRAPHICS_SYMBOL = "graphics-symbol", bundle = (O, _, H) => ({
    [ARIA_ROLE]: O,
    [ARIA_ROLEDESCRIPTION]: _,
    [ARIA_LABEL]: H || void 0
  }), AriaIgnore = toSet$1(["axis-domain", "axis-grid", "axis-label", "axis-tick", "axis-title", "legend-band", "legend-entry", "legend-gradient", "legend-label", "legend-title", "legend-symbol", "title"]), AriaGuides = {
    axis: {
      desc: "axis",
      caption: axisCaption
    },
    legend: {
      desc: "legend",
      caption: legendCaption
    },
    "title-text": {
      desc: "title",
      caption: (O) => `Title text '${titleCaption(O)}'`
    },
    "title-subtitle": {
      desc: "subtitle",
      caption: (O) => `Subtitle text '${titleCaption(O)}'`
    }
  }, AriaEncode = {
    ariaRole: ARIA_ROLE,
    ariaRoleDescription: ARIA_ROLEDESCRIPTION,
    description: ARIA_LABEL
  };
  function ariaItemAttributes(O, _) {
    const H = _.aria === !1;
    if (O(ARIA_HIDDEN, H || void 0), H || _.description == null)
      for (const V in AriaEncode)
        O(AriaEncode[V], void 0);
    else {
      const V = _.mark.marktype;
      O(ARIA_LABEL, _.description), O(ARIA_ROLE, _.ariaRole || (V === "group" ? GRAPHICS_OBJECT : GRAPHICS_SYMBOL)), O(ARIA_ROLEDESCRIPTION, _.ariaRoleDescription || `${V} mark`);
    }
  }
  function ariaMarkAttributes(O) {
    return O.aria === !1 ? {
      [ARIA_HIDDEN]: !0
    } : AriaIgnore[O.role] ? null : AriaGuides[O.role] ? ariaGuide(O, AriaGuides[O.role]) : ariaMark(O);
  }
  function ariaMark(O) {
    const _ = O.marktype, H = _ === "group" || _ === "text" || O.items.some((V) => V.description != null && V.aria !== !1);
    return bundle(H ? GRAPHICS_OBJECT : GRAPHICS_SYMBOL, `${_} mark container`, O.description);
  }
  function ariaGuide(O, _) {
    try {
      const H = O.items[0], V = _.caption || (() => "");
      return bundle(_.role || GRAPHICS_SYMBOL, _.desc, H.description || V(H));
    } catch {
      return null;
    }
  }
  function titleCaption(O) {
    return array$5(O.text).join(" ");
  }
  function axisCaption(O) {
    const _ = O.datum, H = O.orient, V = _.title ? extractTitle(O) : null, q = O.context, G = q.scales[_.scale].value, J = q.dataflow.locale(), Z = G.type;
    return `${H === "left" || H === "right" ? "Y" : "X"}-axis` + (V ? ` titled '${V}'` : "") + ` for a ${isDiscrete$2(Z) ? "discrete" : Z} scale with ${domainCaption(J, G, O)}`;
  }
  function legendCaption(O) {
    const _ = O.datum, H = _.title ? extractTitle(O) : null, V = `${_.type || ""} legend`.trim(), q = _.scales, G = Object.keys(q), J = O.context, Z = J.scales[q[G[0]]].value, te = J.dataflow.locale();
    return capitalize(V) + (H ? ` titled '${H}'` : "") + ` for ${channelCaption(G)} with ${domainCaption(te, Z, O)}`;
  }
  function extractTitle(O) {
    try {
      return array$5(peek$1(O.items).items[0].text).join(" ");
    } catch {
      return null;
    }
  }
  function channelCaption(O) {
    return O = O.map((_) => _ + (_ === "fill" || _ === "stroke" ? " color" : "")), O.length < 2 ? O[0] : O.slice(0, -1).join(", ") + " and " + peek$1(O);
  }
  function capitalize(O) {
    return O.length ? O[0].toUpperCase() + O.slice(1) : O;
  }
  const innerText = (O) => (O + "").replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;"), attrText = (O) => innerText(O).replace(/"/g, "&quot;").replace(/\t/g, "&#x9;").replace(/\n/g, "&#xA;").replace(/\r/g, "&#xD;");
  function markup() {
    let O = "", _ = "", H = "";
    const V = [], q = () => _ = H = "", G = (te) => {
      _ && (O += `${_}>${H}`, q()), V.push(te);
    }, J = (te, ne) => (ne != null && (_ += ` ${te}="${attrText(ne)}"`), Z), Z = {
      open(te) {
        G(te), _ = "<" + te;
        for (var ne = arguments.length, ee = new Array(ne > 1 ? ne - 1 : 0), ae = 1; ae < ne; ae++)
          ee[ae - 1] = arguments[ae];
        for (const oe of ee)
          for (const se in oe) J(se, oe[se]);
        return Z;
      },
      close() {
        const te = V.pop();
        return _ ? O += _ + (H ? `>${H}</${te}>` : "/>") : O += `</${te}>`, q(), Z;
      },
      attr: J,
      text: (te) => (H += innerText(te), Z),
      toString: () => O
    };
    return Z;
  }
  const serializeXML = (O) => _serialize(markup(), O) + "";
  function _serialize(O, _) {
    if (O.open(_.tagName), _.hasAttributes()) {
      const H = _.attributes, V = H.length;
      for (let q = 0; q < V; ++q)
        O.attr(H[q].name, H[q].value);
    }
    if (_.hasChildNodes()) {
      const H = _.childNodes;
      for (const V of H)
        V.nodeType === 3 ? O.text(V.nodeValue) : _serialize(O, V);
    }
    return O.close();
  }
  const stylesAttr = {
    fill: "fill",
    fillOpacity: "fill-opacity",
    stroke: "stroke",
    strokeOpacity: "stroke-opacity",
    strokeWidth: "stroke-width",
    strokeCap: "stroke-linecap",
    strokeJoin: "stroke-linejoin",
    strokeDash: "stroke-dasharray",
    strokeDashOffset: "stroke-dashoffset",
    strokeMiterLimit: "stroke-miterlimit",
    opacity: "opacity"
  }, stylesCss = {
    blend: "mix-blend-mode"
  }, rootAttributes = {
    fill: "none",
    "stroke-miterlimit": 10
  }, RootIndex = 0, xmlns = "http://www.w3.org/2000/xmlns/", svgns = metadata.xmlns;
  class SVGRenderer extends Renderer {
    constructor(_) {
      super(_), this._dirtyID = 0, this._dirty = [], this._svg = null, this._root = null, this._defs = null;
    }
    /**
     * Initialize a new SVGRenderer instance.
     * @param {DOMElement} el - The containing DOM element for the display.
     * @param {number} width - The coordinate width of the display, in pixels.
     * @param {number} height - The coordinate height of the display, in pixels.
     * @param {Array<number>} origin - The origin of the display, in pixels.
     *   The coordinate system will be translated to this point.
     * @param {number} [scaleFactor=1] - Optional scaleFactor by which to multiply
     *   the width and height to determine the final pixel size.
     * @return {SVGRenderer} - This renderer instance.
     */
    initialize(_, H, V, q, G) {
      return this._defs = {}, this._clearDefs(), _ && (this._svg = domChild(_, 0, "svg", svgns), this._svg.setAttributeNS(xmlns, "xmlns", svgns), this._svg.setAttributeNS(xmlns, "xmlns:xlink", metadata["xmlns:xlink"]), this._svg.setAttribute("version", metadata.version), this._svg.setAttribute("class", "marks"), domClear(_, 1), this._root = domChild(this._svg, RootIndex, "g", svgns), setAttributes(this._root, rootAttributes), domClear(this._svg, RootIndex + 1)), this.background(this._bgcolor), super.initialize(_, H, V, q, G);
    }
    /**
     * Get / set the background color.
     */
    background(_) {
      return arguments.length && this._svg && this._svg.style.setProperty("background-color", _), super.background(...arguments);
    }
    /**
     * Resize the display.
     * @param {number} width - The new coordinate width of the display, in pixels.
     * @param {number} height - The new coordinate height of the display, in pixels.
     * @param {Array<number>} origin - The new origin of the display, in pixels.
     *   The coordinate system will be translated to this point.
     * @param {number} [scaleFactor=1] - Optional scaleFactor by which to multiply
     *   the width and height to determine the final pixel size.
     * @return {SVGRenderer} - This renderer instance;
     */
    resize(_, H, V, q) {
      return super.resize(_, H, V, q), this._svg && (setAttributes(this._svg, {
        width: this._width * this._scale,
        height: this._height * this._scale,
        viewBox: `0 0 ${this._width} ${this._height}`
      }), this._root.setAttribute("transform", `translate(${this._origin})`)), this._dirty = [], this;
    }
    /**
     * Returns the SVG element of the visualization.
     * @return {DOMElement} - The SVG element.
     */
    canvas() {
      return this._svg;
    }
    /**
     * Returns an SVG text string for the rendered content,
     * or null if this renderer is currently headless.
     */
    svg() {
      const _ = this._svg, H = this._bgcolor;
      if (!_) return null;
      let V;
      H && (_.removeAttribute("style"), V = domChild(_, RootIndex, "rect", svgns), setAttributes(V, {
        width: this._width,
        height: this._height,
        fill: H
      }));
      const q = serializeXML(_);
      return H && (_.removeChild(V), this._svg.style.setProperty("background-color", H)), q;
    }
    /**
     * Internal rendering method.
     * @param {object} scene - The root mark of a scenegraph to render.
     * @param {Array} markTypes - Array of the mark types to render.
     *                            If undefined, render all mark types
     */
    _render(_, H) {
      return this._dirtyCheck() && (this._dirtyAll && this._clearDefs(), this.mark(this._root, _, void 0, H), domClear(this._root, 1)), this.defs(), this._dirty = [], ++this._dirtyID, this;
    }
    // -- Manage rendering of items marked as dirty --
    /**
     * Flag a mark item as dirty.
     * @param {Item} item - The mark item.
     */
    dirty(_) {
      _.dirty !== this._dirtyID && (_.dirty = this._dirtyID, this._dirty.push(_));
    }
    /**
     * Check if a mark item is considered dirty.
     * @param {Item} item - The mark item.
     */
    isDirty(_) {
      return this._dirtyAll || !_._svg || !_._svg.ownerSVGElement || _.dirty === this._dirtyID;
    }
    /**
     * Internal method to check dirty status and, if possible,
     * make targetted updates without a full rendering pass.
     */
    _dirtyCheck() {
      this._dirtyAll = !0;
      const _ = this._dirty;
      if (!_.length || !this._dirtyID) return !0;
      const H = ++this._dirtyID;
      let V, q, G, J, Z, te, ne;
      for (Z = 0, te = _.length; Z < te; ++Z)
        if (V = _[Z], q = V.mark, q.marktype !== G && (G = q.marktype, J = Marks[G]), q.zdirty && q.dirty !== H && (this._dirtyAll = !1, dirtyParents(V, H), q.items.forEach((ee) => {
          ee.dirty = H;
        })), !q.zdirty) {
          if (V.exit) {
            J.nested && q.items.length ? (ne = q.items[0], ne._svg && this._update(J, ne._svg, ne)) : V._svg && (ne = V._svg.parentNode, ne && ne.removeChild(V._svg)), V._svg = null;
            continue;
          }
          V = J.nested ? q.items[0] : V, V._update !== H && (!V._svg || !V._svg.ownerSVGElement ? (this._dirtyAll = !1, dirtyParents(V, H)) : this._update(J, V._svg, V), V._update = H);
        }
      return !this._dirtyAll;
    }
    // -- Construct & maintain scenegraph to SVG mapping ---
    /**
     * Render a set of mark items.
     * @param {SVGElement} el - The parent element in the SVG tree.
     * @param {object} scene - The mark parent to render.
     * @param {SVGElement} prev - The previous sibling in the SVG tree.
     * @param {Array} markTypes - Array of the mark types to render.
     *                            If undefined, render all mark types
     */
    mark(_, H, V, q) {
      if (!this.isDirty(H))
        return H._svg;
      const G = this._svg, J = H.marktype, Z = Marks[J], te = H.interactive === !1 ? "none" : null, ne = Z.tag === "g", ee = bind$2(H, _, V, "g", G);
      if (J !== "group" && q != null && !q.includes(J))
        return domClear(ee, 0), H._svg;
      ee.setAttribute("class", cssClass(H));
      const ae = ariaMarkAttributes(H);
      for (const le in ae) setAttribute(ee, le, ae[le]);
      ne || setAttribute(ee, "pointer-events", te), setAttribute(ee, "clip-path", H.clip ? clip$1$1(this, H, H.group) : null);
      let oe = null, se = 0;
      const ce = (le) => {
        const ue = this.isDirty(le), de = bind$2(le, ee, oe, Z.tag, G);
        ue && (this._update(Z, de, le), ne && recurse(this, de, le, q)), oe = de, ++se;
      };
      return Z.nested ? H.items.length && ce(H.items[0]) : visit(H, ce), domClear(ee, se), ee;
    }
    /**
     * Update the attributes of an SVG element for a mark item.
     * @param {object} mdef - The mark definition object
     * @param {SVGElement} el - The SVG element.
     * @param {Item} item - The mark item.
     */
    _update(_, H, V) {
      element$1 = H, values$2 = H.__values__, ariaItemAttributes(emit, V), _.attr(emit, V, this);
      const q = mark_extras[_.type];
      q && q.call(this, _, H, V), element$1 && this.style(element$1, V);
    }
    /**
     * Update the presentation attributes of an SVG element for a mark item.
     * @param {SVGElement} el - The SVG element.
     * @param {Item} item - The mark item.
     */
    style(_, H) {
      if (H != null) {
        for (const V in stylesAttr) {
          let q = V === "font" ? fontFamily(H) : H[V];
          if (q === values$2[V]) continue;
          const G = stylesAttr[V];
          q == null ? _.removeAttribute(G) : (isGradient(q) && (q = gradientRef(q, this._defs.gradient, href())), _.setAttribute(G, q + "")), values$2[V] = q;
        }
        for (const V in stylesCss)
          setStyle(_, stylesCss[V], H[V]);
      }
    }
    /**
     * Render SVG defs, as needed.
     * Must be called *after* marks have been processed to ensure the
     * collected state is current and accurate.
     */
    defs() {
      const _ = this._svg, H = this._defs;
      let V = H.el, q = 0;
      for (const G in H.gradient)
        V || (H.el = V = domChild(_, RootIndex + 1, "defs", svgns)), q = updateGradient(V, H.gradient[G], q);
      for (const G in H.clipping)
        V || (H.el = V = domChild(_, RootIndex + 1, "defs", svgns)), q = updateClipping(V, H.clipping[G], q);
      V && (q === 0 ? (_.removeChild(V), H.el = null) : domClear(V, q));
    }
    /**
     * Clear defs caches.
     */
    _clearDefs() {
      const _ = this._defs;
      _.gradient = {}, _.clipping = {};
    }
  }
  function dirtyParents(O, _) {
    for (; O && O.dirty !== _; O = O.mark.group)
      if (O.dirty = _, O.mark && O.mark.dirty !== _)
        O.mark.dirty = _;
      else return;
  }
  function updateGradient(O, _, H) {
    let V, q, G;
    if (_.gradient === "radial") {
      let J = domChild(O, H++, "pattern", svgns);
      setAttributes(J, {
        id: patternPrefix + _.id,
        viewBox: "0,0,1,1",
        width: "100%",
        height: "100%",
        preserveAspectRatio: "xMidYMid slice"
      }), J = domChild(J, 0, "rect", svgns), setAttributes(J, {
        width: 1,
        height: 1,
        fill: `url(${href()}#${_.id})`
      }), O = domChild(O, H++, "radialGradient", svgns), setAttributes(O, {
        id: _.id,
        fx: _.x1,
        fy: _.y1,
        fr: _.r1,
        cx: _.x2,
        cy: _.y2,
        r: _.r2
      });
    } else
      O = domChild(O, H++, "linearGradient", svgns), setAttributes(O, {
        id: _.id,
        x1: _.x1,
        x2: _.x2,
        y1: _.y1,
        y2: _.y2
      });
    for (V = 0, q = _.stops.length; V < q; ++V)
      G = domChild(O, V, "stop", svgns), G.setAttribute("offset", _.stops[V].offset), G.setAttribute("stop-color", _.stops[V].color);
    return domClear(O, V), H;
  }
  function updateClipping(O, _, H) {
    let V;
    return O = domChild(O, H, "clipPath", svgns), O.setAttribute("id", _.id), _.path ? (V = domChild(O, 0, "path", svgns), V.setAttribute("d", _.path)) : (V = domChild(O, 0, "rect", svgns), setAttributes(V, {
      x: 0,
      y: 0,
      width: _.width,
      height: _.height
    })), domClear(O, 1), H + 1;
  }
  function recurse(O, _, H, V) {
    _ = _.lastChild.previousSibling;
    let q, G = 0;
    visit(H, (J) => {
      q = O.mark(_, J, q, V), ++G;
    }), domClear(_, 1 + G);
  }
  function bind$2(O, _, H, V, q) {
    let G = O._svg, J;
    if (!G && (J = _.ownerDocument, G = domCreate(J, V, svgns), O._svg = G, O.mark && (G.__data__ = O, G.__values__ = {
      fill: "default"
    }, V === "g"))) {
      const Z = domCreate(J, "path", svgns);
      G.appendChild(Z), Z.__data__ = O;
      const te = domCreate(J, "g", svgns);
      G.appendChild(te), te.__data__ = O;
      const ne = domCreate(J, "path", svgns);
      G.appendChild(ne), ne.__data__ = O, ne.__values__ = {
        fill: "default"
      };
    }
    return (G.ownerSVGElement !== q || siblingCheck(G, H)) && _.insertBefore(G, H ? H.nextSibling : _.firstChild), G;
  }
  function siblingCheck(O, _) {
    return O.parentNode && O.parentNode.childNodes.length > 1 && O.previousSibling != _;
  }
  let element$1 = null, values$2 = null;
  const mark_extras = {
    group(O, _, H) {
      const V = element$1 = _.childNodes[2];
      values$2 = V.__values__, O.foreground(emit, H, this), values$2 = _.__values__, element$1 = _.childNodes[1], O.content(emit, H, this);
      const q = element$1 = _.childNodes[0];
      O.background(emit, H, this);
      const G = H.mark.interactive === !1 ? "none" : null;
      if (G !== values$2.events && (setAttribute(V, "pointer-events", G), setAttribute(q, "pointer-events", G), values$2.events = G), H.strokeForeground && H.stroke) {
        const J = H.fill;
        setAttribute(V, "display", null), this.style(q, H), setAttribute(q, "stroke", null), J && (H.fill = null), values$2 = V.__values__, this.style(V, H), J && (H.fill = J), element$1 = null;
      } else
        setAttribute(V, "display", "none");
    },
    image(O, _, H) {
      H.smooth === !1 ? (setStyle(_, "image-rendering", "optimizeSpeed"), setStyle(_, "image-rendering", "pixelated")) : setStyle(_, "image-rendering", null);
    },
    text(O, _, H) {
      const V = textLines(H);
      let q, G, J, Z;
      isArray$2(V) ? (G = V.map((te) => textValue(H, te)), q = G.join(`
`), q !== values$2.text && (domClear(_, 0), J = _.ownerDocument, Z = lineHeight(H), G.forEach((te, ne) => {
        const ee = domCreate(J, "tspan", svgns);
        ee.__data__ = H, ee.textContent = te, ne && (ee.setAttribute("x", 0), ee.setAttribute("dy", Z)), _.appendChild(ee);
      }), values$2.text = q)) : (G = textValue(H, V), G !== values$2.text && (_.textContent = G, values$2.text = G)), setAttribute(_, "font-family", fontFamily(H)), setAttribute(_, "font-size", fontSize(H) + "px"), setAttribute(_, "font-style", H.fontStyle), setAttribute(_, "font-variant", H.fontVariant), setAttribute(_, "font-weight", H.fontWeight);
    }
  };
  function emit(O, _, H) {
    _ !== values$2[O] && (H ? setAttributeNS(element$1, O, _, H) : setAttribute(element$1, O, _), values$2[O] = _);
  }
  function setStyle(O, _, H) {
    H !== values$2[_] && (H == null ? O.style.removeProperty(_) : O.style.setProperty(_, H + ""), values$2[_] = H);
  }
  function setAttributes(O, _) {
    for (const H in _)
      setAttribute(O, H, _[H]);
  }
  function setAttribute(O, _, H) {
    H != null ? O.setAttribute(_, H) : O.removeAttribute(_);
  }
  function setAttributeNS(O, _, H, V) {
    H != null ? O.setAttributeNS(V, _, H) : O.removeAttributeNS(V, _);
  }
  function href() {
    let O;
    return typeof window > "u" ? "" : (O = window.location).hash ? O.href.slice(0, -O.hash.length) : O.href;
  }
  class SVGStringRenderer extends Renderer {
    constructor(_) {
      super(_), this._text = null, this._defs = {
        gradient: {},
        clipping: {}
      };
    }
    /**
     * Returns the rendered SVG text string,
     * or null if rendering has not yet occurred.
     */
    svg() {
      return this._text;
    }
    /**
     * Internal rendering method.
     * @param {object} scene - The root mark of a scenegraph to render.
     */
    _render(_) {
      const H = markup();
      H.open("svg", extend$2({}, metadata, {
        class: "marks",
        width: this._width * this._scale,
        height: this._height * this._scale,
        viewBox: `0 0 ${this._width} ${this._height}`
      }));
      const V = this._bgcolor;
      return V && V !== "transparent" && V !== "none" && H.open("rect", {
        width: this._width,
        height: this._height,
        fill: V
      }).close(), H.open("g", rootAttributes, {
        transform: "translate(" + this._origin + ")"
      }), this.mark(H, _), H.close(), this.defs(H), this._text = H.close() + "", this;
    }
    /**
     * Render a set of mark items.
     * @param {object} m - The markup context.
     * @param {object} scene - The mark parent to render.
     */
    mark(_, H) {
      const V = Marks[H.marktype], q = V.tag, G = [ariaItemAttributes, V.attr];
      _.open("g", {
        class: cssClass(H),
        "clip-path": H.clip ? clip$1$1(this, H, H.group) : null
      }, ariaMarkAttributes(H), {
        "pointer-events": q !== "g" && H.interactive === !1 ? "none" : null
      });
      const J = (Z) => {
        const te = this.href(Z);
        if (te && _.open("a", te), _.open(q, this.attr(H, Z, G, q !== "g" ? q : null)), q === "text") {
          const ne = textLines(Z);
          if (isArray$2(ne)) {
            const ee = {
              x: 0,
              dy: lineHeight(Z)
            };
            for (let ae = 0; ae < ne.length; ++ae)
              _.open("tspan", ae ? ee : null).text(textValue(Z, ne[ae])).close();
          } else
            _.text(textValue(Z, ne));
        } else if (q === "g") {
          const ne = Z.strokeForeground, ee = Z.fill, ae = Z.stroke;
          ne && ae && (Z.stroke = null), _.open("path", this.attr(H, Z, V.background, "bgrect")).close(), _.open("g", this.attr(H, Z, V.content)), visit(Z, (oe) => this.mark(_, oe)), _.close(), ne && ae ? (ee && (Z.fill = null), Z.stroke = ae, _.open("path", this.attr(H, Z, V.foreground, "bgrect")).close(), ee && (Z.fill = ee)) : _.open("path", this.attr(H, Z, V.foreground, "bgfore")).close();
        }
        _.close(), te && _.close();
      };
      return V.nested ? H.items && H.items.length && J(H.items[0]) : visit(H, J), _.close();
    }
    /**
     * Get href attributes for a hyperlinked mark item.
     * @param {Item} item - The mark item.
     */
    href(_) {
      const H = _.href;
      let V;
      if (H) {
        if (V = this._hrefs && this._hrefs[H])
          return V;
        this.sanitizeURL(H).then((q) => {
          q["xlink:href"] = q.href, q.href = null, (this._hrefs || (this._hrefs = {}))[H] = q;
        });
      }
      return null;
    }
    /**
     * Get an object of SVG attributes for a mark item.
     * @param {object} scene - The mark parent.
     * @param {Item} item - The mark item.
     * @param {array|function} attrs - One or more attribute emitters.
     * @param {string} tag - The tag being rendered.
     */
    attr(_, H, V, q) {
      const G = {}, J = (Z, te, ne, ee) => {
        G[ee || Z] = te;
      };
      return Array.isArray(V) ? V.forEach((Z) => Z(J, H, this)) : V(J, H, this), q && style(G, H, _, q, this._defs), G;
    }
    /**
     * Render SVG defs, as needed.
     * Must be called *after* marks have been processed to ensure the
     * collected state is current and accurate.
     * @param {object} m - The markup context.
     */
    defs(_) {
      const H = this._defs.gradient, V = this._defs.clipping;
      if (Object.keys(H).length + Object.keys(V).length !== 0) {
        _.open("defs");
        for (const G in H) {
          const J = H[G], Z = J.stops;
          J.gradient === "radial" ? (_.open("pattern", {
            id: patternPrefix + G,
            viewBox: "0,0,1,1",
            width: "100%",
            height: "100%",
            preserveAspectRatio: "xMidYMid slice"
          }), _.open("rect", {
            width: "1",
            height: "1",
            fill: "url(#" + G + ")"
          }).close(), _.close(), _.open("radialGradient", {
            id: G,
            fx: J.x1,
            fy: J.y1,
            fr: J.r1,
            cx: J.x2,
            cy: J.y2,
            r: J.r2
          })) : _.open("linearGradient", {
            id: G,
            x1: J.x1,
            x2: J.x2,
            y1: J.y1,
            y2: J.y2
          });
          for (let te = 0; te < Z.length; ++te)
            _.open("stop", {
              offset: Z[te].offset,
              "stop-color": Z[te].color
            }).close();
          _.close();
        }
        for (const G in V) {
          const J = V[G];
          _.open("clipPath", {
            id: G
          }), J.path ? _.open("path", {
            d: J.path
          }).close() : _.open("rect", {
            x: 0,
            y: 0,
            width: J.width,
            height: J.height
          }).close(), _.close();
        }
        _.close();
      }
    }
  }
  function style(O, _, H, V, q) {
    let G;
    if (_ == null || (V === "bgrect" && H.interactive === !1 && (O["pointer-events"] = "none"), V === "bgfore" && (H.interactive === !1 && (O["pointer-events"] = "none"), O.display = "none", _.fill !== null)))
      return O;
    V === "image" && _.smooth === !1 && (G = ["image-rendering: optimizeSpeed;", "image-rendering: pixelated;"]), V === "text" && (O["font-family"] = fontFamily(_), O["font-size"] = fontSize(_) + "px", O["font-style"] = _.fontStyle, O["font-variant"] = _.fontVariant, O["font-weight"] = _.fontWeight);
    for (const J in stylesAttr) {
      let Z = _[J];
      const te = stylesAttr[J];
      Z === "transparent" && (te === "fill" || te === "stroke") || Z != null && (isGradient(Z) && (Z = gradientRef(Z, q.gradient, "")), O[te] = Z);
    }
    for (const J in stylesCss) {
      const Z = _[J];
      Z != null && (G = G || [], G.push(`${stylesCss[J]}: ${Z};`));
    }
    return G && (O.style = G.join(" ")), O;
  }
  const OPTS = {
    svgMarkTypes: ["text"],
    svgOnTop: !0,
    debug: !1
  };
  function setHybridRendererOptions(O) {
    OPTS.svgMarkTypes = O.svgMarkTypes ?? ["text"], OPTS.svgOnTop = O.svgOnTop ?? !0, OPTS.debug = O.debug ?? !1;
  }
  class HybridRenderer extends Renderer {
    constructor(_) {
      super(_), this._svgRenderer = new SVGRenderer(_), this._canvasRenderer = new CanvasRenderer(_);
    }
    /**
     * Initialize a new HybridRenderer instance.
     * @param {DOMElement} el - The containing DOM element for the display.
     * @param {number} width - The coordinate width of the display, in pixels.
     * @param {number} height - The coordinate height of the display, in pixels.
     * @param {Array<number>} origin - The origin of the display, in pixels.
     *   The coordinate system will be translated to this point.
     * @param {number} [scaleFactor=1] - Optional scaleFactor by which to multiply
     *   the width and height to determine the final pixel size.
     * @return {HybridRenderer} - This renderer instance.
     */
    initialize(_, H, V, q, G) {
      this._root_el = domChild(_, 0, "div");
      const J = domChild(this._root_el, 0, "div"), Z = domChild(this._root_el, 1, "div");
      return this._root_el.style.position = "relative", OPTS.debug || (J.style.height = "100%", Z.style.position = "absolute", Z.style.top = "0", Z.style.left = "0", Z.style.height = "100%", Z.style.width = "100%"), this._svgEl = OPTS.svgOnTop ? Z : J, this._canvasEl = OPTS.svgOnTop ? J : Z, this._svgEl.style.pointerEvents = "none", this._canvasRenderer.initialize(this._canvasEl, H, V, q, G), this._svgRenderer.initialize(this._svgEl, H, V, q, G), super.initialize(_, H, V, q, G);
    }
    /**
     * Flag a mark item as dirty.
     * @param {Item} item - The mark item.
     */
    dirty(_) {
      return OPTS.svgMarkTypes.includes(_.mark.marktype) ? this._svgRenderer.dirty(_) : this._canvasRenderer.dirty(_), this;
    }
    /**
     * Internal rendering method.
     * @param {object} scene - The root mark of a scenegraph to render.
     * @param {Array} markTypes - Array of the mark types to render.
     *                            If undefined, render all mark types
     */
    _render(_, H) {
      const q = (H ?? ["arc", "area", "image", "line", "path", "rect", "rule", "shape", "symbol", "text", "trail"]).filter((G) => !OPTS.svgMarkTypes.includes(G));
      this._svgRenderer.render(_, OPTS.svgMarkTypes), this._canvasRenderer.render(_, q);
    }
    /**
     * Resize the display.
     * @param {number} width - The new coordinate width of the display, in pixels.
     * @param {number} height - The new coordinate height of the display, in pixels.
     * @param {Array<number>} origin - The new origin of the display, in pixels.
     *   The coordinate system will be translated to this point.
     * @param {number} [scaleFactor=1] - Optional scaleFactor by which to multiply
     *   the width and height to determine the final pixel size.
     * @return {SVGRenderer} - This renderer instance;
     */
    resize(_, H, V, q) {
      return super.resize(_, H, V, q), this._svgRenderer.resize(_, H, V, q), this._canvasRenderer.resize(_, H, V, q), this;
    }
    background(_) {
      return OPTS.svgOnTop ? this._canvasRenderer.background(_) : this._svgRenderer.background(_), this;
    }
  }
  class HybridHandler extends CanvasHandler {
    constructor(_, H) {
      super(_, H);
    }
    initialize(_, H, V) {
      const q = domChild(domChild(_, 0, "div"), OPTS.svgOnTop ? 0 : 1, "div");
      return super.initialize(q, H, V);
    }
  }
  const Canvas = "canvas", Hybrid = "hybrid", PNG = "png", SVG = "svg", None$1 = "none", RenderType = {
    Canvas,
    PNG,
    SVG,
    Hybrid,
    None: None$1
  }, modules = {};
  modules[Canvas] = modules[PNG] = {
    renderer: CanvasRenderer,
    headless: CanvasRenderer,
    handler: CanvasHandler
  };
  modules[SVG] = {
    renderer: SVGRenderer,
    headless: SVGStringRenderer,
    handler: SVGHandler
  };
  modules[Hybrid] = {
    renderer: HybridRenderer,
    headless: HybridRenderer,
    handler: HybridHandler
  };
  modules[None$1] = {};
  function renderModule(O, _) {
    return O = String(O || "").toLowerCase(), arguments.length > 1 ? (modules[O] = _, this) : modules[O];
  }
  function intersect$2(O, _, H) {
    const V = [], q = new Bounds().union(_), G = O.marktype;
    return G ? intersectMark(O, q, H, V) : G === "group" ? intersectGroup(O, q, H, V) : error$1("Intersect scene must be mark node or group item.");
  }
  function intersectMark(O, _, H, V) {
    if (visitMark(O, _, H)) {
      const q = O.items, G = O.marktype, J = q.length;
      let Z = 0;
      if (G === "group")
        for (; Z < J; ++Z)
          intersectGroup(q[Z], _, H, V);
      else
        for (const te = Marks[G].isect; Z < J; ++Z) {
          const ne = q[Z];
          intersectItem(ne, _, te) && V.push(ne);
        }
    }
    return V;
  }
  function visitMark(O, _, H) {
    return O.bounds && _.intersects(O.bounds) && (O.marktype === "group" || O.interactive !== !1 && (!H || H(O)));
  }
  function intersectGroup(O, _, H, V) {
    H && H(O.mark) && intersectItem(O, _, Marks.group.isect) && V.push(O);
    const q = O.items, G = q && q.length;
    if (G) {
      const J = O.x || 0, Z = O.y || 0;
      _.translate(-J, -Z);
      for (let te = 0; te < G; ++te)
        intersectMark(q[te], _, H, V);
      _.translate(J, Z);
    }
    return V;
  }
  function intersectItem(O, _, H) {
    const V = O.bounds;
    return _.encloses(V) || _.intersects(V) && H(O, _);
  }
  const clipBounds = new Bounds();
  function boundClip(O) {
    const _ = O.clip;
    if (isFunction$2(_))
      _(boundContext(clipBounds.clear()));
    else if (_)
      clipBounds.set(0, 0, O.group.width, O.group.height);
    else return;
    O.bounds.intersect(clipBounds);
  }
  const TOLERANCE = 1e-9;
  function sceneEqual(O, _, H) {
    return O === _ ? !0 : H === "path" ? pathEqual(O, _) : O instanceof Date && _ instanceof Date ? +O == +_ : isNumber$2(O) && isNumber$2(_) ? Math.abs(O - _) <= TOLERANCE : !O || !_ || !isObject$2(O) && !isObject$2(_) ? O == _ : objectEqual(O, _);
  }
  function pathEqual(O, _) {
    return sceneEqual(parse$3(O), parse$3(_));
  }
  function objectEqual(O, _) {
    var H = Object.keys(O), V = Object.keys(_), q, G;
    if (H.length !== V.length) return !1;
    for (H.sort(), V.sort(), G = H.length - 1; G >= 0; G--)
      if (H[G] != V[G]) return !1;
    for (G = H.length - 1; G >= 0; G--)
      if (q = H[G], !sceneEqual(O[q], _[q], q)) return !1;
    return typeof O == typeof _;
  }
  function resetSVGDefIds() {
    resetSVGClipId(), resetSVGGradientId();
  }
  const Top$1 = "top", Left$1 = "left", Right$1 = "right", Bottom$1 = "bottom", TopLeft = "top-left", TopRight = "top-right", BottomLeft = "bottom-left", BottomRight = "bottom-right", Start$1 = "start", Middle$1 = "middle", End$1 = "end", X$1 = "x", Y$2 = "y", Group = "group", AxisRole$1 = "axis", TitleRole$1 = "title", FrameRole$1 = "frame", ScopeRole$1 = "scope", LegendRole$1 = "legend", RowHeader = "row-header", RowFooter = "row-footer", RowTitle = "row-title", ColHeader = "column-header", ColFooter = "column-footer", ColTitle = "column-title", Padding$1 = "padding", Symbols$1 = "symbol", Fit = "fit", FitX = "fit-x", FitY = "fit-y", Pad = "pad", None = "none", All = "all", Each = "each", Flush = "flush", Column = "column", Row = "row";
  function Bound$1(O) {
    Transform.call(this, null, O);
  }
  inherits(Bound$1, Transform, {
    transform(O, _) {
      const H = _.dataflow, V = O.mark, q = V.marktype, G = Marks[q], J = G.bound;
      let Z = V.bounds, te;
      if (G.nested)
        V.items.length && H.dirty(V.items[0]), Z = boundItem(V, J), V.items.forEach((ne) => {
          ne.bounds.clear().union(Z);
        });
      else if (q === Group || O.modified())
        switch (_.visit(_.MOD, (ne) => H.dirty(ne)), Z.clear(), V.items.forEach((ne) => Z.union(boundItem(ne, J))), V.role) {
          case AxisRole$1:
          case LegendRole$1:
          case TitleRole$1:
            _.reflow();
        }
      else
        te = _.changed(_.REM), _.visit(_.ADD, (ne) => {
          Z.union(boundItem(ne, J));
        }), _.visit(_.MOD, (ne) => {
          te = te || Z.alignsWith(ne.bounds), H.dirty(ne), Z.union(boundItem(ne, J));
        }), te && (Z.clear(), V.items.forEach((ne) => Z.union(ne.bounds)));
      return boundClip(V), _.modifies("bounds");
    }
  });
  function boundItem(O, _, H) {
    return _(O.bounds.clear(), O, H);
  }
  const COUNTER_NAME = ":vega_identifier:";
  function Identifier$2(O) {
    Transform.call(this, 0, O);
  }
  Identifier$2.Definition = {
    type: "Identifier",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "as",
      type: "string",
      required: !0
    }]
  };
  inherits(Identifier$2, Transform, {
    transform(O, _) {
      const H = getCounter(_.dataflow), V = O.as;
      let q = H.value;
      return _.visit(_.ADD, (G) => G[V] = G[V] || ++q), H.set(this.value = q), _;
    }
  });
  function getCounter(O) {
    return O._signals[COUNTER_NAME] || (O._signals[COUNTER_NAME] = O.add(0));
  }
  function Mark$2(O) {
    Transform.call(this, null, O);
  }
  inherits(Mark$2, Transform, {
    transform(O, _) {
      let H = this.value;
      H || (H = _.dataflow.scenegraph().mark(O.markdef, lookup$1$1(O), O.index), H.group.context = O.context, O.context.group || (O.context.group = H.group), H.source = this.source, H.clip = O.clip, H.interactive = O.interactive, this.value = H);
      const V = H.marktype === Group ? GroupItem : Item;
      return _.visit(_.ADD, (q) => V.call(q, H)), (O.modified("clip") || O.modified("interactive")) && (H.clip = O.clip, H.interactive = !!O.interactive, H.zdirty = !0, _.reflow()), H.items = _.source, _;
    }
  });
  function lookup$1$1(O) {
    const _ = O.groups, H = O.parent;
    return _ && _.size === 1 ? _.get(Object.keys(_.object)[0]) : _ && H ? _.lookup(H) : null;
  }
  function Overlap$1(O) {
    Transform.call(this, null, O);
  }
  const methods = {
    parity: (O) => O.filter((_, H) => H % 2 ? _.opacity = 0 : 1),
    greedy: (O, _) => {
      let H;
      return O.filter((V, q) => !q || !intersect$1(H.bounds, V.bounds, _) ? (H = V, 1) : V.opacity = 0);
    }
  }, intersect$1 = (O, _, H) => H > Math.max(_.x1 - O.x2, O.x1 - _.x2, _.y1 - O.y2, O.y1 - _.y2), hasOverlap = (O, _) => {
    for (var H = 1, V = O.length, q = O[0].bounds, G; H < V; q = G, ++H)
      if (intersect$1(q, G = O[H].bounds, _)) return !0;
  }, hasBounds = (O) => {
    const _ = O.bounds;
    return _.width() > 1 && _.height() > 1;
  }, boundTest = (O, _, H) => {
    var V = O.range(), q = new Bounds();
    return _ === Top$1 || _ === Bottom$1 ? q.set(V[0], -1 / 0, V[1], 1 / 0) : q.set(-1 / 0, V[0], 1 / 0, V[1]), q.expand(H || 1), (G) => q.encloses(G.bounds);
  }, reset$1 = (O) => (O.forEach((_) => _.opacity = 1), O), reflow = (O, _) => O.reflow(_.modified()).modifies("opacity");
  inherits(Overlap$1, Transform, {
    transform(O, _) {
      const H = methods[O.method] || methods.parity, V = O.separation || 0;
      let q = _.materialize(_.SOURCE).source, G, J;
      if (!q || !q.length) return;
      if (!O.method)
        return O.modified("method") && (reset$1(q), _ = reflow(_, O)), _;
      if (q = q.filter(hasBounds), !q.length) return;
      if (O.sort && (q = q.slice().sort(O.sort)), G = reset$1(q), _ = reflow(_, O), G.length >= 3 && hasOverlap(G, V)) {
        do
          G = H(G, V);
        while (G.length >= 3 && hasOverlap(G, V));
        G.length < 3 && !peek$1(q).opacity && (G.length > 1 && (peek$1(G).opacity = 0), peek$1(q).opacity = 1);
      }
      O.boundScale && O.boundTolerance >= 0 && (J = boundTest(O.boundScale, O.boundOrient, +O.boundTolerance), q.forEach((te) => {
        J(te) || (te.opacity = 0);
      }));
      const Z = G[0].mark.bounds.clear();
      return q.forEach((te) => {
        te.opacity && Z.union(te.bounds);
      }), _;
    }
  });
  function Render$1(O) {
    Transform.call(this, null, O);
  }
  inherits(Render$1, Transform, {
    transform(O, _) {
      const H = _.dataflow;
      if (_.visit(_.ALL, (V) => H.dirty(V)), _.fields && _.fields.zindex) {
        const V = _.source && _.source[0];
        V && (V.mark.zdirty = !0);
      }
    }
  });
  const tempBounds = new Bounds();
  function set$4(O, _, H) {
    return O[_] === H ? 0 : (O[_] = H, 1);
  }
  function isYAxis(O) {
    var _ = O.items[0].orient;
    return _ === Left$1 || _ === Right$1;
  }
  function axisIndices(O) {
    let _ = +O.grid;
    return [
      O.ticks ? _++ : -1,
      // ticks index
      O.labels ? _++ : -1,
      // labels index
      _ + +O.domain
      // title index
    ];
  }
  function axisLayout(O, _, H, V) {
    var q = _.items[0], G = q.datum, J = q.translate != null ? q.translate : 0.5, Z = q.orient, te = axisIndices(G), ne = q.range, ee = q.offset, ae = q.position, oe = q.minExtent, se = q.maxExtent, ce = G.title && q.items[te[2]].items[0], le = q.titlePadding, ue = q.bounds, de = ce && multiLineOffset(ce), fe = 0, he = 0, me, pe;
    switch (tempBounds.clear().union(ue), ue.clear(), (me = te[0]) > -1 && ue.union(q.items[me].bounds), (me = te[1]) > -1 && ue.union(q.items[me].bounds), Z) {
      case Top$1:
        fe = ae || 0, he = -ee, pe = Math.max(oe, Math.min(se, -ue.y1)), ue.add(0, -pe).add(ne, 0), ce && axisTitleLayout(O, ce, pe, le, de, 0, -1, ue);
        break;
      case Left$1:
        fe = -ee, he = ae || 0, pe = Math.max(oe, Math.min(se, -ue.x1)), ue.add(-pe, 0).add(0, ne), ce && axisTitleLayout(O, ce, pe, le, de, 1, -1, ue);
        break;
      case Right$1:
        fe = H + ee, he = ae || 0, pe = Math.max(oe, Math.min(se, ue.x2)), ue.add(0, 0).add(pe, ne), ce && axisTitleLayout(O, ce, pe, le, de, 1, 1, ue);
        break;
      case Bottom$1:
        fe = ae || 0, he = V + ee, pe = Math.max(oe, Math.min(se, ue.y2)), ue.add(0, 0).add(ne, pe), ce && axisTitleLayout(O, ce, pe, le, 0, 0, 1, ue);
        break;
      default:
        fe = q.x, he = q.y;
    }
    return boundStroke(ue.translate(fe, he), q), set$4(q, "x", fe + J) | set$4(q, "y", he + J) && (q.bounds = tempBounds, O.dirty(q), q.bounds = ue, O.dirty(q)), q.mark.bounds.clear().union(ue);
  }
  function axisTitleLayout(O, _, H, V, q, G, J, Z) {
    const te = _.bounds;
    if (_.auto) {
      const ne = J * (H + q + V);
      let ee = 0, ae = 0;
      O.dirty(_), G ? ee = (_.x || 0) - (_.x = ne) : ae = (_.y || 0) - (_.y = ne), _.mark.bounds.clear().union(te.translate(-ee, -ae)), O.dirty(_);
    }
    Z.union(te);
  }
  const min = (O, _) => Math.floor(Math.min(O, _)), max = (O, _) => Math.ceil(Math.max(O, _));
  function gridLayoutGroups(O) {
    var _ = O.items, H = _.length, V = 0, q, G;
    const J = {
      marks: [],
      rowheaders: [],
      rowfooters: [],
      colheaders: [],
      colfooters: [],
      rowtitle: null,
      coltitle: null
    };
    for (; V < H; ++V)
      if (q = _[V], G = q.items, q.marktype === Group)
        switch (q.role) {
          case AxisRole$1:
          case LegendRole$1:
          case TitleRole$1:
            break;
          case RowHeader:
            J.rowheaders.push(...G);
            break;
          case RowFooter:
            J.rowfooters.push(...G);
            break;
          case ColHeader:
            J.colheaders.push(...G);
            break;
          case ColFooter:
            J.colfooters.push(...G);
            break;
          case RowTitle:
            J.rowtitle = G[0];
            break;
          case ColTitle:
            J.coltitle = G[0];
            break;
          default:
            J.marks.push(...G);
        }
    return J;
  }
  function bboxFlush(O) {
    return new Bounds().set(0, 0, O.width || 0, O.height || 0);
  }
  function bboxFull(O) {
    const _ = O.bounds.clone();
    return _.empty() ? _.set(0, 0, 0, 0) : _.translate(-(O.x || 0), -(O.y || 0));
  }
  function get$2(O, _, H) {
    const V = isObject$2(O) ? O[_] : O;
    return V ?? (H !== void 0 ? H : 0);
  }
  function offsetValue$1(O) {
    return O < 0 ? Math.ceil(-O) : 0;
  }
  function gridLayout(O, _, H) {
    var V = !H.nodirty, q = H.bounds === Flush ? bboxFlush : bboxFull, G = tempBounds.set(0, 0, 0, 0), J = get$2(H.align, Column), Z = get$2(H.align, Row), te = get$2(H.padding, Column), ne = get$2(H.padding, Row), ee = H.columns || _.length, ae = ee <= 0 ? 1 : Math.ceil(_.length / ee), oe = _.length, se = Array(oe), ce = Array(ee), le = 0, ue = Array(oe), de = Array(ae), fe = 0, he = Array(oe), me = Array(oe), pe = Array(oe), ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe;
    for (ge = 0; ge < ee; ++ge) ce[ge] = 0;
    for (ge = 0; ge < ae; ++ge) de[ge] = 0;
    for (ge = 0; ge < oe; ++ge)
      Se = _[ge], Ce = pe[ge] = q(Se), Se.x = Se.x || 0, he[ge] = 0, Se.y = Se.y || 0, me[ge] = 0, ve = ge % ee, be = ~~(ge / ee), le = Math.max(le, we = Math.ceil(Ce.x2)), fe = Math.max(fe, Ee = Math.ceil(Ce.y2)), ce[ve] = Math.max(ce[ve], we), de[be] = Math.max(de[be], Ee), se[ge] = te + offsetValue$1(Ce.x1), ue[ge] = ne + offsetValue$1(Ce.y1), V && O.dirty(_[ge]);
    for (ge = 0; ge < oe; ++ge)
      ge % ee === 0 && (se[ge] = 0), ge < ee && (ue[ge] = 0);
    if (J === Each)
      for (ve = 1; ve < ee; ++ve) {
        for (Qe = 0, ge = ve; ge < oe; ge += ee)
          Qe < se[ge] && (Qe = se[ge]);
        for (ge = ve; ge < oe; ge += ee)
          se[ge] = Qe + ce[ve - 1];
      }
    else if (J === All) {
      for (Qe = 0, ge = 0; ge < oe; ++ge)
        ge % ee && Qe < se[ge] && (Qe = se[ge]);
      for (ge = 0; ge < oe; ++ge)
        ge % ee && (se[ge] = Qe + le);
    } else
      for (J = !1, ve = 1; ve < ee; ++ve)
        for (ge = ve; ge < oe; ge += ee)
          se[ge] += ce[ve - 1];
    if (Z === Each)
      for (be = 1; be < ae; ++be) {
        for (Qe = 0, ge = be * ee, ye = ge + ee; ge < ye; ++ge)
          Qe < ue[ge] && (Qe = ue[ge]);
        for (ge = be * ee; ge < ye; ++ge)
          ue[ge] = Qe + de[be - 1];
      }
    else if (Z === All) {
      for (Qe = 0, ge = ee; ge < oe; ++ge)
        Qe < ue[ge] && (Qe = ue[ge]);
      for (ge = ee; ge < oe; ++ge)
        ue[ge] = Qe + fe;
    } else
      for (Z = !1, be = 1; be < ae; ++be)
        for (ge = be * ee, ye = ge + ee; ge < ye; ++ge)
          ue[ge] += de[be - 1];
    for (xe = 0, ge = 0; ge < oe; ++ge)
      xe = se[ge] + (ge % ee ? xe : 0), he[ge] += xe - _[ge].x;
    for (ve = 0; ve < ee; ++ve)
      for (Le = 0, ge = ve; ge < oe; ge += ee)
        Le += ue[ge], me[ge] += Le - _[ge].y;
    if (J && get$2(H.center, Column) && ae > 1)
      for (ge = 0; ge < oe; ++ge)
        Ce = J === All ? le : ce[ge % ee], xe = Ce - pe[ge].x2 - _[ge].x - he[ge], xe > 0 && (he[ge] += xe / 2);
    if (Z && get$2(H.center, Row) && ee !== 1)
      for (ge = 0; ge < oe; ++ge)
        Ce = Z === All ? fe : de[~~(ge / ee)], Le = Ce - pe[ge].y2 - _[ge].y - me[ge], Le > 0 && (me[ge] += Le / 2);
    for (ge = 0; ge < oe; ++ge)
      G.union(pe[ge].translate(he[ge], me[ge]));
    switch (xe = get$2(H.anchor, X$1), Le = get$2(H.anchor, Y$2), get$2(H.anchor, Column)) {
      case End$1:
        xe -= G.width();
        break;
      case Middle$1:
        xe -= G.width() / 2;
    }
    switch (get$2(H.anchor, Row)) {
      case End$1:
        Le -= G.height();
        break;
      case Middle$1:
        Le -= G.height() / 2;
    }
    for (xe = Math.round(xe), Le = Math.round(Le), G.clear(), ge = 0; ge < oe; ++ge)
      _[ge].mark.bounds.clear();
    for (ge = 0; ge < oe; ++ge)
      Se = _[ge], Se.x += he[ge] += xe, Se.y += me[ge] += Le, G.union(Se.mark.bounds.union(Se.bounds.translate(he[ge], me[ge]))), V && O.dirty(Se);
    return G;
  }
  function trellisLayout(O, _, H) {
    var V = gridLayoutGroups(_), q = V.marks, G = H.bounds === Flush ? boundFlush : boundFull, J = H.offset, Z = H.columns || q.length, te = Z <= 0 ? 1 : Math.ceil(q.length / Z), ne = te * Z, ee, ae, oe, se, ce, le, ue;
    const de = gridLayout(O, q, H);
    de.empty() && de.set(0, 0, 0, 0), V.rowheaders && (le = get$2(H.headerBand, Row, null), ee = layoutHeaders(O, V.rowheaders, q, Z, te, -get$2(J, "rowHeader"), min, 0, G, "x1", 0, Z, 1, le)), V.colheaders && (le = get$2(H.headerBand, Column, null), ae = layoutHeaders(O, V.colheaders, q, Z, Z, -get$2(J, "columnHeader"), min, 1, G, "y1", 0, 1, Z, le)), V.rowfooters && (le = get$2(H.footerBand, Row, null), oe = layoutHeaders(O, V.rowfooters, q, Z, te, get$2(J, "rowFooter"), max, 0, G, "x2", Z - 1, Z, 1, le)), V.colfooters && (le = get$2(H.footerBand, Column, null), se = layoutHeaders(O, V.colfooters, q, Z, Z, get$2(J, "columnFooter"), max, 1, G, "y2", ne - Z, 1, Z, le)), V.rowtitle && (ce = get$2(H.titleAnchor, Row), ue = get$2(J, "rowTitle"), ue = ce === End$1 ? oe + ue : ee - ue, le = get$2(H.titleBand, Row, 0.5), layoutTitle(O, V.rowtitle, ue, 0, de, le)), V.coltitle && (ce = get$2(H.titleAnchor, Column), ue = get$2(J, "columnTitle"), ue = ce === End$1 ? se + ue : ae - ue, le = get$2(H.titleBand, Column, 0.5), layoutTitle(O, V.coltitle, ue, 1, de, le));
  }
  function boundFlush(O, _) {
    return _ === "x1" ? O.x || 0 : _ === "y1" ? O.y || 0 : _ === "x2" ? (O.x || 0) + (O.width || 0) : _ === "y2" ? (O.y || 0) + (O.height || 0) : void 0;
  }
  function boundFull(O, _) {
    return O.bounds[_];
  }
  function layoutHeaders(O, _, H, V, q, G, J, Z, te, ne, ee, ae, oe, se) {
    var ce = H.length, le = 0, ue = 0, de, fe, he, me, pe, ye, ge, ve, be;
    if (!ce) return le;
    for (de = ee; de < ce; de += ae)
      H[de] && (le = J(le, te(H[de], ne)));
    if (!_.length) return le;
    for (_.length > q && (O.warn("Grid headers exceed limit: " + q), _ = _.slice(0, q)), le += G, fe = 0, me = _.length; fe < me; ++fe)
      O.dirty(_[fe]), _[fe].mark.bounds.clear();
    for (de = ee, fe = 0, me = _.length; fe < me; ++fe, de += ae) {
      for (ye = _[fe], pe = ye.mark.bounds, he = de; he >= 0 && (ge = H[he]) == null; he -= oe) ;
      Z ? (ve = se == null ? ge.x : Math.round(ge.bounds.x1 + se * ge.bounds.width()), be = le) : (ve = le, be = se == null ? ge.y : Math.round(ge.bounds.y1 + se * ge.bounds.height())), pe.union(ye.bounds.translate(ve - (ye.x || 0), be - (ye.y || 0))), ye.x = ve, ye.y = be, O.dirty(ye), ue = J(ue, pe[ne]);
    }
    return ue;
  }
  function layoutTitle(O, _, H, V, q, G) {
    if (_) {
      O.dirty(_);
      var J = H, Z = H;
      V ? J = Math.round(q.x1 + G * q.width()) : Z = Math.round(q.y1 + G * q.height()), _.bounds.translate(J - (_.x || 0), Z - (_.y || 0)), _.mark.bounds.clear().union(_.bounds), _.x = J, _.y = Z, O.dirty(_);
    }
  }
  function lookup$3(O, _) {
    const H = O[_] || {};
    return (V, q) => H[V] != null ? H[V] : O[V] != null ? O[V] : q;
  }
  function offsets(O, _) {
    let H = -1 / 0;
    return O.forEach((V) => {
      V.offset != null && (H = Math.max(H, V.offset));
    }), H > -1 / 0 ? H : _;
  }
  function legendParams(O, _, H, V, q, G, J) {
    const Z = lookup$3(H, _), te = offsets(O, Z("offset", 0)), ne = Z("anchor", Start$1), ee = ne === End$1 ? 1 : ne === Middle$1 ? 0.5 : 0, ae = {
      align: Each,
      bounds: Z("bounds", Flush),
      columns: Z("direction") === "vertical" ? 1 : O.length,
      padding: Z("margin", 8),
      center: Z("center"),
      nodirty: !0
    };
    switch (_) {
      case Left$1:
        ae.anchor = {
          x: Math.floor(V.x1) - te,
          column: End$1,
          y: ee * (J || V.height() + 2 * V.y1),
          row: ne
        };
        break;
      case Right$1:
        ae.anchor = {
          x: Math.ceil(V.x2) + te,
          y: ee * (J || V.height() + 2 * V.y1),
          row: ne
        };
        break;
      case Top$1:
        ae.anchor = {
          y: Math.floor(q.y1) - te,
          row: End$1,
          x: ee * (G || q.width() + 2 * q.x1),
          column: ne
        };
        break;
      case Bottom$1:
        ae.anchor = {
          y: Math.ceil(q.y2) + te,
          x: ee * (G || q.width() + 2 * q.x1),
          column: ne
        };
        break;
      case TopLeft:
        ae.anchor = {
          x: te,
          y: te
        };
        break;
      case TopRight:
        ae.anchor = {
          x: G - te,
          y: te,
          column: End$1
        };
        break;
      case BottomLeft:
        ae.anchor = {
          x: te,
          y: J - te,
          row: End$1
        };
        break;
      case BottomRight:
        ae.anchor = {
          x: G - te,
          y: J - te,
          column: End$1,
          row: End$1
        };
        break;
    }
    return ae;
  }
  function legendLayout(O, _) {
    var H = _.items[0], V = H.datum, q = H.orient, G = H.bounds, J = H.x, Z = H.y, te, ne;
    return H._bounds ? H._bounds.clear().union(G) : H._bounds = G.clone(), G.clear(), legendGroupLayout(O, H, H.items[0].items[0]), G = legendBounds(H, G), te = 2 * H.padding, ne = 2 * H.padding, G.empty() || (te = Math.ceil(G.width() + te), ne = Math.ceil(G.height() + ne)), V.type === Symbols$1 && legendEntryLayout(H.items[0].items[0].items[0].items), q !== None && (H.x = J = 0, H.y = Z = 0), H.width = te, H.height = ne, boundStroke(G.set(J, Z, J + te, Z + ne), H), H.mark.bounds.clear().union(G), H;
  }
  function legendBounds(O, _) {
    return O.items.forEach((H) => _.union(H.bounds)), _.x1 = O.padding, _.y1 = O.padding, _;
  }
  function legendGroupLayout(O, _, H) {
    var V = _.padding, q = V - H.x, G = V - H.y;
    if (!_.datum.title)
      (q || G) && translate$1(O, H, q, G);
    else {
      var J = _.items[1].items[0], Z = J.anchor, te = _.titlePadding || 0, ne = V - J.x, ee = V - J.y;
      switch (J.orient) {
        case Left$1:
          q += Math.ceil(J.bounds.width()) + te;
          break;
        case Right$1:
        case Bottom$1:
          break;
        default:
          G += J.bounds.height() + te;
      }
      switch ((q || G) && translate$1(O, H, q, G), J.orient) {
        case Left$1:
          ee += legendTitleOffset(_, H, J, Z, 1, 1);
          break;
        case Right$1:
          ne += legendTitleOffset(_, H, J, End$1, 0, 0) + te, ee += legendTitleOffset(_, H, J, Z, 1, 1);
          break;
        case Bottom$1:
          ne += legendTitleOffset(_, H, J, Z, 0, 0), ee += legendTitleOffset(_, H, J, End$1, -1, 0, 1) + te;
          break;
        default:
          ne += legendTitleOffset(_, H, J, Z, 0, 0);
      }
      (ne || ee) && translate$1(O, J, ne, ee), (ne = Math.round(J.bounds.x1 - V)) < 0 && (translate$1(O, H, -ne, 0), translate$1(O, J, -ne, 0));
    }
  }
  function legendTitleOffset(O, _, H, V, q, G, J) {
    const Z = O.datum.type !== "symbol", te = H.datum.vgrad, ne = Z && (G || !te) && !J ? _.items[0] : _, ee = ne.bounds[q ? "y2" : "x2"] - O.padding, ae = te && G ? ee : 0, oe = te && G ? 0 : ee, se = q <= 0 ? 0 : multiLineOffset(H);
    return Math.round(V === Start$1 ? ae : V === End$1 ? oe - se : 0.5 * (ee - se));
  }
  function translate$1(O, _, H, V) {
    _.x += H, _.y += V, _.bounds.translate(H, V), _.mark.bounds.translate(H, V), O.dirty(_);
  }
  function legendEntryLayout(O) {
    const _ = O.reduce((H, V) => (H[V.column] = Math.max(V.bounds.x2 - V.x, H[V.column] || 0), H), {});
    O.forEach((H) => {
      H.width = _[H.column], H.height = H.bounds.y2 - H.y;
    });
  }
  function titleLayout(O, _, H, V, q) {
    var G = _.items[0], J = G.frame, Z = G.orient, te = G.anchor, ne = G.offset, ee = G.padding, ae = G.items[0].items[0], oe = G.items[1] && G.items[1].items[0], se = Z === Left$1 || Z === Right$1 ? V : H, ce = 0, le = 0, ue = 0, de = 0, fe = 0, he;
    if (J !== Group ? Z === Left$1 ? (ce = q.y2, se = q.y1) : Z === Right$1 ? (ce = q.y1, se = q.y2) : (ce = q.x1, se = q.x2) : Z === Left$1 && (ce = V, se = 0), he = te === Start$1 ? ce : te === End$1 ? se : (ce + se) / 2, oe && oe.text) {
      switch (Z) {
        case Top$1:
        case Bottom$1:
          fe = ae.bounds.height() + ee;
          break;
        case Left$1:
          de = ae.bounds.width() + ee;
          break;
        case Right$1:
          de = -ae.bounds.width() - ee;
          break;
      }
      tempBounds.clear().union(oe.bounds), tempBounds.translate(de - (oe.x || 0), fe - (oe.y || 0)), set$4(oe, "x", de) | set$4(oe, "y", fe) && (O.dirty(oe), oe.bounds.clear().union(tempBounds), oe.mark.bounds.clear().union(tempBounds), O.dirty(oe)), tempBounds.clear().union(oe.bounds);
    } else
      tempBounds.clear();
    switch (tempBounds.union(ae.bounds), Z) {
      case Top$1:
        le = he, ue = q.y1 - tempBounds.height() - ne;
        break;
      case Left$1:
        le = q.x1 - tempBounds.width() - ne, ue = he;
        break;
      case Right$1:
        le = q.x2 + tempBounds.width() + ne, ue = he;
        break;
      case Bottom$1:
        le = he, ue = q.y2 + ne;
        break;
      default:
        le = G.x, ue = G.y;
    }
    return set$4(G, "x", le) | set$4(G, "y", ue) && (tempBounds.translate(le, ue), O.dirty(G), G.bounds.clear().union(tempBounds), _.bounds.clear().union(tempBounds), O.dirty(G)), G.bounds;
  }
  function ViewLayout$1(O) {
    Transform.call(this, null, O);
  }
  inherits(ViewLayout$1, Transform, {
    transform(O, _) {
      const H = _.dataflow;
      return O.mark.items.forEach((V) => {
        O.layout && trellisLayout(H, V, O.layout), layoutGroup(H, V, O);
      }), shouldReflow(O.mark.group) ? _.reflow() : _;
    }
  });
  function shouldReflow(O) {
    return O && O.mark.role !== "legend-entry";
  }
  function layoutGroup(O, _, H) {
    var V = _.items, q = Math.max(0, _.width || 0), G = Math.max(0, _.height || 0), J = new Bounds().set(0, 0, q, G), Z = J.clone(), te = J.clone(), ne = [], ee, ae, oe, se, ce, le;
    for (ce = 0, le = V.length; ce < le; ++ce)
      switch (ae = V[ce], ae.role) {
        case AxisRole$1:
          se = isYAxis(ae) ? Z : te, se.union(axisLayout(O, ae, q, G));
          break;
        case TitleRole$1:
          ee = ae;
          break;
        case LegendRole$1:
          ne.push(legendLayout(O, ae));
          break;
        case FrameRole$1:
        case ScopeRole$1:
        case RowHeader:
        case RowFooter:
        case RowTitle:
        case ColHeader:
        case ColFooter:
        case ColTitle:
          Z.union(ae.bounds), te.union(ae.bounds);
          break;
        default:
          J.union(ae.bounds);
      }
    if (ne.length) {
      const ue = {};
      ne.forEach((de) => {
        oe = de.orient || Right$1, oe !== None && (ue[oe] || (ue[oe] = [])).push(de);
      });
      for (const de in ue) {
        const fe = ue[de];
        gridLayout(O, fe, legendParams(fe, de, H.legends, Z, te, q, G));
      }
      ne.forEach((de) => {
        const fe = de.bounds;
        if (fe.equals(de._bounds) || (de.bounds = de._bounds, O.dirty(de), de.bounds = fe, O.dirty(de)), H.autosize && (H.autosize.type === Fit || H.autosize.type === FitX || H.autosize.type === FitY))
          switch (de.orient) {
            case Left$1:
            case Right$1:
              J.add(fe.x1, 0).add(fe.x2, 0);
              break;
            case Top$1:
            case Bottom$1:
              J.add(0, fe.y1).add(0, fe.y2);
          }
        else
          J.union(fe);
      });
    }
    J.union(Z).union(te), ee && J.union(titleLayout(O, ee, q, G, J)), _.clip && J.set(0, 0, _.width || 0, _.height || 0), viewSizeLayout(O, _, J, H);
  }
  function viewSizeLayout(O, _, H, V) {
    const q = V.autosize || {}, G = q.type;
    if (O._autosize < 1 || !G) return;
    let J = O._width, Z = O._height, te = Math.max(0, _.width || 0), ne = Math.max(0, Math.ceil(-H.x1)), ee = Math.max(0, _.height || 0), ae = Math.max(0, Math.ceil(-H.y1));
    const oe = Math.max(0, Math.ceil(H.x2 - te)), se = Math.max(0, Math.ceil(H.y2 - ee));
    if (q.contains === Padding$1) {
      const ce = O.padding();
      J -= ce.left + ce.right, Z -= ce.top + ce.bottom;
    }
    G === None ? (ne = 0, ae = 0, te = J, ee = Z) : G === Fit ? (te = Math.max(0, J - ne - oe), ee = Math.max(0, Z - ae - se)) : G === FitX ? (te = Math.max(0, J - ne - oe), Z = ee + ae + se) : G === FitY ? (J = te + ne + oe, ee = Math.max(0, Z - ae - se)) : G === Pad && (J = te + ne + oe, Z = ee + ae + se), O._resizeView(J, Z, te, ee, [ne, ae], q.resize);
  }
  const vtx = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    bound: Bound$1,
    identifier: Identifier$2,
    mark: Mark$2,
    overlap: Overlap$1,
    render: Render$1,
    viewlayout: ViewLayout$1
  }, Symbol.toStringTag, { value: "Module" }));
  function AxisTicks$1(O) {
    Transform.call(this, null, O);
  }
  inherits(AxisTicks$1, Transform, {
    transform(O, _) {
      if (this.value && !O.modified())
        return _.StopPropagation;
      var H = _.dataflow.locale(), V = _.fork(_.NO_SOURCE | _.NO_FIELDS), q = this.value, G = O.scale, J = O.count == null ? O.values ? O.values.length : 10 : O.count, Z = tickCount(G, J, O.minstep), te = O.format || tickFormat(H, G, Z, O.formatSpecifier, O.formatType, !!O.values), ne = O.values ? validTicks(G, O.values, Z) : tickValues(G, Z);
      return q && (V.rem = q), q = ne.map((ee, ae) => ingest$1({
        index: ae / (ne.length - 1 || 1),
        value: ee,
        label: te(ee)
      })), O.extra && q.length && q.push(ingest$1({
        index: -1,
        extra: {
          value: q[0].value
        },
        label: ""
      })), V.source = q, V.add = q, this.value = q, V;
    }
  });
  function DataJoin$1(O) {
    Transform.call(this, null, O);
  }
  function defaultItemCreate() {
    return ingest$1({});
  }
  function newMap(O) {
    const _ = fastmap().test((H) => H.exit);
    return _.lookup = (H) => _.get(O(H)), _;
  }
  inherits(DataJoin$1, Transform, {
    transform(O, _) {
      var H = _.dataflow, V = _.fork(_.NO_SOURCE | _.NO_FIELDS), q = O.item || defaultItemCreate, G = O.key || tupleid, J = this.value;
      return isArray$2(V.encode) && (V.encode = null), J && (O.modified("key") || _.modified(G)) && error$1("DataJoin does not support modified key function or fields."), J || (_ = _.addAll(), this.value = J = newMap(G)), _.visit(_.ADD, (Z) => {
        const te = G(Z);
        let ne = J.get(te);
        ne ? ne.exit ? (J.empty--, V.add.push(ne)) : V.mod.push(ne) : (ne = q(Z), J.set(te, ne), V.add.push(ne)), ne.datum = Z, ne.exit = !1;
      }), _.visit(_.MOD, (Z) => {
        const te = G(Z), ne = J.get(te);
        ne && (ne.datum = Z, V.mod.push(ne));
      }), _.visit(_.REM, (Z) => {
        const te = G(Z), ne = J.get(te);
        Z === ne.datum && !ne.exit && (V.rem.push(ne), ne.exit = !0, ++J.empty);
      }), _.changed(_.ADD_MOD) && V.modifies("datum"), (_.clean() || O.clean && J.empty > H.cleanThreshold) && H.runAfter(J.clean), V;
    }
  });
  function Encode$1(O) {
    Transform.call(this, null, O);
  }
  inherits(Encode$1, Transform, {
    transform(O, _) {
      var H = _.fork(_.ADD_REM), V = O.mod || !1, q = O.encoders, G = _.encode;
      if (isArray$2(G))
        if (H.changed() || G.every((ae) => q[ae]))
          G = G[0], H.encode = null;
        else
          return _.StopPropagation;
      var J = G === "enter", Z = q.update || falsy, te = q.enter || falsy, ne = q.exit || falsy, ee = (G && !J ? q[G] : Z) || falsy;
      if (_.changed(_.ADD) && (_.visit(_.ADD, (ae) => {
        te(ae, O), Z(ae, O);
      }), H.modifies(te.output), H.modifies(Z.output), ee !== falsy && ee !== Z && (_.visit(_.ADD, (ae) => {
        ee(ae, O);
      }), H.modifies(ee.output))), _.changed(_.REM) && ne !== falsy && (_.visit(_.REM, (ae) => {
        ne(ae, O);
      }), H.modifies(ne.output)), J || ee !== falsy) {
        const ae = _.MOD | (O.modified() ? _.REFLOW : 0);
        J ? (_.visit(ae, (oe) => {
          const se = te(oe, O) || V;
          (ee(oe, O) || se) && H.mod.push(oe);
        }), H.mod.length && H.modifies(te.output)) : _.visit(ae, (oe) => {
          (ee(oe, O) || V) && H.mod.push(oe);
        }), H.mod.length && H.modifies(ee.output);
      }
      return H.changed() ? H : _.StopPropagation;
    }
  });
  function LegendEntries$1(O) {
    Transform.call(this, [], O);
  }
  inherits(LegendEntries$1, Transform, {
    transform(O, _) {
      if (this.value != null && !O.modified())
        return _.StopPropagation;
      var H = _.dataflow.locale(), V = _.fork(_.NO_SOURCE | _.NO_FIELDS), q = this.value, G = O.type || SymbolLegend, J = O.scale, Z = +O.limit, te = tickCount(J, O.count == null ? 5 : O.count, O.minstep), ne = !!O.values || G === SymbolLegend, ee = O.format || labelFormat(H, J, te, G, O.formatSpecifier, O.formatType, ne), ae = O.values || labelValues(J, te), oe, se, ce, le, ue;
      return q && (V.rem = q), G === SymbolLegend ? (Z && ae.length > Z ? (_.dataflow.warn("Symbol legend count exceeds limit, filtering items."), q = ae.slice(0, Z - 1), ue = !0) : q = ae, isFunction$2(ce = O.size) ? (!O.values && J(q[0]) === 0 && (q = q.slice(1)), le = q.reduce((de, fe) => Math.max(de, ce(fe, O)), 0)) : ce = constant$5(le = ce || 8), q = q.map((de, fe) => ingest$1({
        index: fe,
        label: ee(de, fe, q),
        value: de,
        offset: le,
        size: ce(de, O)
      })), ue && (ue = ae[q.length], q.push(ingest$1({
        index: q.length,
        label: `…${ae.length - q.length} entries`,
        value: ue,
        offset: le,
        size: ce(ue, O)
      })))) : G === GradientLegend ? (oe = J.domain(), se = scaleFraction(J, oe[0], peek$1(oe)), ae.length < 3 && !O.values && oe[0] !== peek$1(oe) && (ae = [oe[0], peek$1(oe)]), q = ae.map((de, fe) => ingest$1({
        index: fe,
        label: ee(de, fe, ae),
        value: de,
        perc: se(de)
      }))) : (ce = ae.length - 1, se = labelFraction(J), q = ae.map((de, fe) => ingest$1({
        index: fe,
        label: ee(de, fe, ae),
        value: de,
        perc: fe ? se(de) : 0,
        perc2: fe === ce ? 1 : se(ae[fe + 1])
      }))), V.source = q, V.add = q, this.value = q, V;
    }
  });
  const sourceX = (O) => O.source.x, sourceY = (O) => O.source.y, targetX = (O) => O.target.x, targetY = (O) => O.target.y;
  function LinkPath(O) {
    Transform.call(this, {}, O);
  }
  LinkPath.Definition = {
    type: "LinkPath",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "sourceX",
      type: "field",
      default: "source.x"
    }, {
      name: "sourceY",
      type: "field",
      default: "source.y"
    }, {
      name: "targetX",
      type: "field",
      default: "target.x"
    }, {
      name: "targetY",
      type: "field",
      default: "target.y"
    }, {
      name: "orient",
      type: "enum",
      default: "vertical",
      values: ["horizontal", "vertical", "radial"]
    }, {
      name: "shape",
      type: "enum",
      default: "line",
      values: ["line", "arc", "curve", "diagonal", "orthogonal"]
    }, {
      name: "require",
      type: "signal"
    }, {
      name: "as",
      type: "string",
      default: "path"
    }]
  };
  inherits(LinkPath, Transform, {
    transform(O, _) {
      var H = O.sourceX || sourceX, V = O.sourceY || sourceY, q = O.targetX || targetX, G = O.targetY || targetY, J = O.as || "path", Z = O.orient || "vertical", te = O.shape || "line", ne = Paths.get(te + "-" + Z) || Paths.get(te);
      return ne || error$1("LinkPath unsupported type: " + O.shape + (O.orient ? "-" + O.orient : "")), _.visit(_.SOURCE, (ee) => {
        ee[J] = ne(H(ee), V(ee), q(ee), G(ee));
      }), _.reflow(O.modified()).modifies(J);
    }
  });
  const line$1 = (O, _, H, V) => "M" + O + "," + _ + "L" + H + "," + V, lineR = (O, _, H, V) => line$1(_ * Math.cos(O), _ * Math.sin(O), V * Math.cos(H), V * Math.sin(H)), arc$1 = (O, _, H, V) => {
    var q = H - O, G = V - _, J = Math.hypot(q, G) / 2, Z = 180 * Math.atan2(G, q) / Math.PI;
    return "M" + O + "," + _ + "A" + J + "," + J + " " + Z + " 0 1 " + H + "," + V;
  }, arcR = (O, _, H, V) => arc$1(_ * Math.cos(O), _ * Math.sin(O), V * Math.cos(H), V * Math.sin(H)), curve = (O, _, H, V) => {
    const q = H - O, G = V - _, J = 0.2 * (q + G), Z = 0.2 * (G - q);
    return "M" + O + "," + _ + "C" + (O + J) + "," + (_ + Z) + " " + (H + Z) + "," + (V - J) + " " + H + "," + V;
  }, curveR = (O, _, H, V) => curve(_ * Math.cos(O), _ * Math.sin(O), V * Math.cos(H), V * Math.sin(H)), orthoX = (O, _, H, V) => "M" + O + "," + _ + "V" + V + "H" + H, orthoY = (O, _, H, V) => "M" + O + "," + _ + "H" + H + "V" + V, orthoR = (O, _, H, V) => {
    const q = Math.cos(O), G = Math.sin(O), J = Math.cos(H), Z = Math.sin(H), te = Math.abs(H - O) > Math.PI ? H <= O : H > O;
    return "M" + _ * q + "," + _ * G + "A" + _ + "," + _ + " 0 0," + (te ? 1 : 0) + " " + _ * J + "," + _ * Z + "L" + V * J + "," + V * Z;
  }, diagonalX = (O, _, H, V) => {
    const q = (O + H) / 2;
    return "M" + O + "," + _ + "C" + q + "," + _ + " " + q + "," + V + " " + H + "," + V;
  }, diagonalY = (O, _, H, V) => {
    const q = (_ + V) / 2;
    return "M" + O + "," + _ + "C" + O + "," + q + " " + H + "," + q + " " + H + "," + V;
  }, diagonalR = (O, _, H, V) => {
    const q = Math.cos(O), G = Math.sin(O), J = Math.cos(H), Z = Math.sin(H), te = (_ + V) / 2;
    return "M" + _ * q + "," + _ * G + "C" + te * q + "," + te * G + " " + te * J + "," + te * Z + " " + V * J + "," + V * Z;
  }, Paths = fastmap({
    line: line$1,
    "line-radial": lineR,
    arc: arc$1,
    "arc-radial": arcR,
    curve,
    "curve-radial": curveR,
    "orthogonal-horizontal": orthoX,
    "orthogonal-vertical": orthoY,
    "orthogonal-radial": orthoR,
    "diagonal-horizontal": diagonalX,
    "diagonal-vertical": diagonalY,
    "diagonal-radial": diagonalR
  });
  function Pie(O) {
    Transform.call(this, null, O);
  }
  Pie.Definition = {
    type: "Pie",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "startAngle",
      type: "number",
      default: 0
    }, {
      name: "endAngle",
      type: "number",
      default: 6.283185307179586
    }, {
      name: "sort",
      type: "boolean",
      default: !1
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: ["startAngle", "endAngle"]
    }]
  };
  inherits(Pie, Transform, {
    transform(O, _) {
      var H = O.as || ["startAngle", "endAngle"], V = H[0], q = H[1], G = O.field || one$2, J = O.startAngle || 0, Z = O.endAngle != null ? O.endAngle : 2 * Math.PI, te = _.source, ne = te.map(G), ee = ne.length, ae = J, oe = (Z - J) / sum$1(ne), se = range$7(ee), ce, le, ue;
      for (O.sort && se.sort((de, fe) => ne[de] - ne[fe]), ce = 0; ce < ee; ++ce)
        ue = ne[se[ce]], le = te[se[ce]], le[V] = ae, le[q] = ae += ue * oe;
      return this.value = ne, _.reflow(O.modified()).modifies(H);
    }
  });
  const DEFAULT_COUNT = 5;
  function includeZero(O) {
    const _ = O.type;
    return !O.bins && (_ === Linear || _ === Pow || _ === Sqrt);
  }
  function includePad(O) {
    return isContinuous$1(O) && O !== Sequential;
  }
  const SKIP$1 = toSet$1(["set", "modified", "clear", "type", "scheme", "schemeExtent", "schemeCount", "domain", "domainMin", "domainMid", "domainMax", "domainRaw", "domainImplicit", "nice", "zero", "bins", "range", "rangeStep", "round", "reverse", "interpolate", "interpolateGamma"]);
  function Scale$1(O) {
    Transform.call(this, null, O), this.modified(!0);
  }
  inherits(Scale$1, Transform, {
    transform(O, _) {
      var H = _.dataflow, V = this.value, q = scaleKey(O);
      (!V || q !== V.type) && (this.value = V = scale$4(q)());
      for (q in O) if (!SKIP$1[q]) {
        if (q === "padding" && includePad(V.type)) continue;
        isFunction$2(V[q]) ? V[q](O[q]) : H.warn("Unsupported scale property: " + q);
      }
      return configureRange(V, O, configureBins(V, O, configureDomain(V, O, H))), _.fork(_.NO_SOURCE | _.NO_FIELDS);
    }
  });
  function scaleKey(O) {
    var _ = O.type, H = "", V;
    return _ === Sequential ? Sequential + "-" + Linear : (isContinuousColor(O) && (V = O.rawDomain ? O.rawDomain.length : O.domain ? O.domain.length + +(O.domainMid != null) : 0, H = V === 2 ? Sequential + "-" : V === 3 ? Diverging + "-" : ""), (H + _ || Linear).toLowerCase());
  }
  function isContinuousColor(O) {
    const _ = O.type;
    return isContinuous$1(_) && _ !== Time$1 && _ !== UTC && (O.scheme || O.range && O.range.length && O.range.every(isString$1));
  }
  function configureDomain(O, _, H) {
    const V = rawDomain(O, _.domainRaw, H);
    if (V > -1) return V;
    var q = _.domain, G = O.type, J = _.zero || _.zero === void 0 && includeZero(O), Z, te;
    if (!q) return 0;
    if ((J || _.domainMin != null || _.domainMax != null || _.domainMid != null) && (Z = (q = q.slice()).length - 1 || 1, J && (q[0] > 0 && (q[0] = 0), q[Z] < 0 && (q[Z] = 0)), _.domainMin != null && (q[0] = _.domainMin), _.domainMax != null && (q[Z] = _.domainMax), _.domainMid != null)) {
      te = _.domainMid;
      const ne = te > q[Z] ? Z + 1 : te < q[0] ? 0 : Z;
      ne !== Z && H.warn("Scale domainMid exceeds domain min or max.", te), q.splice(ne, 0, te);
    }
    return includePad(G) && _.padding && q[0] !== peek$1(q) && (q = padDomain(G, q, _.range, _.padding, _.exponent, _.constant)), O.domain(domainCheck(G, q, H)), G === Ordinal && O.unknown(_.domainImplicit ? implicit : void 0), _.nice && O.nice && O.nice(_.nice !== !0 && tickCount(O, _.nice) || null), q.length;
  }
  function rawDomain(O, _, H) {
    return _ ? (O.domain(domainCheck(O.type, _, H)), _.length) : -1;
  }
  function padDomain(O, _, H, V, q, G) {
    var J = Math.abs(peek$1(H) - H[0]), Z = J / (J - 2 * V), te = O === Log ? zoomLog(_, null, Z) : O === Sqrt ? zoomPow(_, null, Z, 0.5) : O === Pow ? zoomPow(_, null, Z, q || 1) : O === Symlog ? zoomSymlog(_, null, Z, G || 1) : zoomLinear(_, null, Z);
    return _ = _.slice(), _[0] = te[0], _[_.length - 1] = te[1], _;
  }
  function domainCheck(O, _, H) {
    if (isLogarithmic(O)) {
      var V = Math.abs(_.reduce((q, G) => q + (G < 0 ? -1 : G > 0 ? 1 : 0), 0));
      V !== _.length && H.warn("Log scale domain includes zero: " + $$2(_));
    }
    return _;
  }
  function configureBins(O, _, H) {
    let V = _.bins;
    if (V && !isArray$2(V)) {
      const q = O.domain(), G = q[0], J = peek$1(q), Z = V.step;
      let te = V.start == null ? G : V.start, ne = V.stop == null ? J : V.stop;
      Z || error$1("Scale bins parameter missing step property."), te < G && (te = Z * Math.ceil(G / Z)), ne > J && (ne = Z * Math.floor(J / Z)), V = range$7(te, ne + Z / 2, Z);
    }
    return V ? O.bins = V : O.bins && delete O.bins, O.type === BinOrdinal && (V ? !_.domain && !_.domainRaw && (O.domain(V), H = V.length) : O.bins = O.domain()), H;
  }
  function configureRange(O, _, H) {
    var V = O.type, q = _.round || !1, G = _.range;
    if (_.rangeStep != null)
      G = configureRangeStep(V, _, H);
    else if (_.scheme && (G = configureScheme(V, _, H), isFunction$2(G))) {
      if (O.interpolator)
        return O.interpolator(G);
      error$1(`Scale type ${V} does not support interpolating color schemes.`);
    }
    if (G && isInterpolating(V))
      return O.interpolator(interpolateColors(flip(G, _.reverse), _.interpolate, _.interpolateGamma));
    G && _.interpolate && O.interpolate ? O.interpolate(interpolate$1(_.interpolate, _.interpolateGamma)) : isFunction$2(O.round) ? O.round(q) : isFunction$2(O.rangeRound) && O.interpolate(q ? interpolateRound : interpolate$1$1), G && O.range(flip(G, _.reverse));
  }
  function configureRangeStep(O, _, H) {
    O !== Band && O !== Point && error$1("Only band and point scales support rangeStep.");
    var V = (_.paddingOuter != null ? _.paddingOuter : _.padding) || 0, q = O === Point ? 1 : (_.paddingInner != null ? _.paddingInner : _.padding) || 0;
    return [0, _.rangeStep * bandSpace(H, q, V)];
  }
  function configureScheme(O, _, H) {
    var V = _.schemeExtent, q, G;
    return isArray$2(_.scheme) ? G = interpolateColors(_.scheme, _.interpolate, _.interpolateGamma) : (q = _.scheme.toLowerCase(), G = scheme$1(q), G || error$1(`Unrecognized scheme name: ${_.scheme}`)), H = O === Threshold ? H + 1 : O === BinOrdinal ? H - 1 : O === Quantile || O === Quantize ? +_.schemeCount || DEFAULT_COUNT : H, isInterpolating(O) ? adjustScheme(G, V, _.reverse) : isFunction$2(G) ? quantizeInterpolator(adjustScheme(G, V), H) : O === Ordinal ? G : G.slice(0, H);
  }
  function adjustScheme(O, _, H) {
    return isFunction$2(O) && (_ || H) ? interpolateRange$1(O, flip(_ || [0, 1], H)) : O;
  }
  function flip(O, _) {
    return _ ? O.slice().reverse() : O;
  }
  function SortItems$1(O) {
    Transform.call(this, null, O);
  }
  inherits(SortItems$1, Transform, {
    transform(O, _) {
      const H = O.modified("sort") || _.changed(_.ADD) || _.modified(O.sort.fields) || _.modified("datum");
      return H && _.source.sort(stableCompare(O.sort)), this.modified(H), _;
    }
  });
  const Zero = "zero", Center$1 = "center", Normalize = "normalize", DefOutput = ["y0", "y1"];
  function Stack$2(O) {
    Transform.call(this, null, O);
  }
  Stack$2.Definition = {
    type: "Stack",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "sort",
      type: "compare"
    }, {
      name: "offset",
      type: "enum",
      default: Zero,
      values: [Zero, Center$1, Normalize]
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: DefOutput
    }]
  };
  inherits(Stack$2, Transform, {
    transform(O, _) {
      var H = O.as || DefOutput, V = H[0], q = H[1], G = stableCompare(O.sort), J = O.field || one$2, Z = O.offset === Center$1 ? stackCenter : O.offset === Normalize ? stackNormalize : stackZero, te, ne, ee, ae;
      for (te = partition$3(_.source, O.groupby, G, J), ne = 0, ee = te.length, ae = te.max; ne < ee; ++ne)
        Z(te[ne], ae, J, V, q);
      return _.reflow(O.modified()).modifies(H);
    }
  });
  function stackCenter(O, _, H, V, q) {
    for (var G = (_ - O.sum) / 2, J = O.length, Z = 0, te; Z < J; ++Z)
      te = O[Z], te[V] = G, te[q] = G += Math.abs(H(te));
  }
  function stackNormalize(O, _, H, V, q) {
    for (var G = 1 / O.sum, J = 0, Z = O.length, te = 0, ne = 0, ee; te < Z; ++te)
      ee = O[te], ee[V] = J, ee[q] = J = G * (ne += Math.abs(H(ee)));
  }
  function stackZero(O, _, H, V, q) {
    for (var G = 0, J = 0, Z = O.length, te = 0, ne, ee; te < Z; ++te)
      ee = O[te], ne = +H(ee), ne < 0 ? (ee[V] = J, ee[q] = J += ne) : (ee[V] = G, ee[q] = G += ne);
  }
  function partition$3(O, _, H, V) {
    var q = [], G = (le) => le(ee), J, Z, te, ne, ee, ae, oe, se, ce;
    if (_ == null)
      q.push(O.slice());
    else
      for (J = {}, Z = 0, te = O.length; Z < te; ++Z)
        ee = O[Z], ae = _.map(G), oe = J[ae], oe || (J[ae] = oe = [], q.push(oe)), oe.push(ee);
    for (ae = 0, ce = 0, ne = q.length; ae < ne; ++ae) {
      for (oe = q[ae], Z = 0, se = 0, te = oe.length; Z < te; ++Z)
        se += Math.abs(V(oe[Z]));
      oe.sum = se, se > ce && (ce = se), H && oe.sort(H);
    }
    return q.max = ce, q;
  }
  const encode$1 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    axisticks: AxisTicks$1,
    datajoin: DataJoin$1,
    encode: Encode$1,
    legendentries: LegendEntries$1,
    linkpath: LinkPath,
    pie: Pie,
    scale: Scale$1,
    sortitems: SortItems$1,
    stack: Stack$2
  }, Symbol.toStringTag, { value: "Module" }));
  var epsilon$3 = 1e-6, epsilon2 = 1e-12, pi$1 = Math.PI, halfPi$1 = pi$1 / 2, quarterPi = pi$1 / 4, tau$1 = pi$1 * 2, degrees = 180 / pi$1, radians = pi$1 / 180, abs$1 = Math.abs, atan = Math.atan, atan2 = Math.atan2, cos$1 = Math.cos, ceil = Math.ceil, exp = Math.exp, hypot = Math.hypot, log$1 = Math.log, pow$1 = Math.pow, sin$1 = Math.sin, sign = Math.sign || function(O) {
    return O > 0 ? 1 : O < 0 ? -1 : 0;
  }, sqrt$1 = Math.sqrt, tan = Math.tan;
  function acos(O) {
    return O > 1 ? 0 : O < -1 ? pi$1 : Math.acos(O);
  }
  function asin$1(O) {
    return O > 1 ? halfPi$1 : O < -1 ? -halfPi$1 : Math.asin(O);
  }
  function noop$2() {
  }
  function streamGeometry(O, _) {
    O && streamGeometryType.hasOwnProperty(O.type) && streamGeometryType[O.type](O, _);
  }
  var streamObjectType = {
    Feature: function(O, _) {
      streamGeometry(O.geometry, _);
    },
    FeatureCollection: function(O, _) {
      for (var H = O.features, V = -1, q = H.length; ++V < q; ) streamGeometry(H[V].geometry, _);
    }
  }, streamGeometryType = {
    Sphere: function(O, _) {
      _.sphere();
    },
    Point: function(O, _) {
      O = O.coordinates, _.point(O[0], O[1], O[2]);
    },
    MultiPoint: function(O, _) {
      for (var H = O.coordinates, V = -1, q = H.length; ++V < q; ) O = H[V], _.point(O[0], O[1], O[2]);
    },
    LineString: function(O, _) {
      streamLine(O.coordinates, _, 0);
    },
    MultiLineString: function(O, _) {
      for (var H = O.coordinates, V = -1, q = H.length; ++V < q; ) streamLine(H[V], _, 0);
    },
    Polygon: function(O, _) {
      streamPolygon(O.coordinates, _);
    },
    MultiPolygon: function(O, _) {
      for (var H = O.coordinates, V = -1, q = H.length; ++V < q; ) streamPolygon(H[V], _);
    },
    GeometryCollection: function(O, _) {
      for (var H = O.geometries, V = -1, q = H.length; ++V < q; ) streamGeometry(H[V], _);
    }
  };
  function streamLine(O, _, H) {
    var V = -1, q = O.length - H, G;
    for (_.lineStart(); ++V < q; ) G = O[V], _.point(G[0], G[1], G[2]);
    _.lineEnd();
  }
  function streamPolygon(O, _) {
    var H = -1, V = O.length;
    for (_.polygonStart(); ++H < V; ) streamLine(O[H], _, 1);
    _.polygonEnd();
  }
  function geoStream(O, _) {
    O && streamObjectType.hasOwnProperty(O.type) ? streamObjectType[O.type](O, _) : streamGeometry(O, _);
  }
  var areaRingSum$1 = new Adder(), areaSum$1 = new Adder(), lambda00$2, phi00$2, lambda0$1, cosPhi0, sinPhi0, areaStream$1 = {
    point: noop$2,
    lineStart: noop$2,
    lineEnd: noop$2,
    polygonStart: function() {
      areaRingSum$1 = new Adder(), areaStream$1.lineStart = areaRingStart$1, areaStream$1.lineEnd = areaRingEnd$1;
    },
    polygonEnd: function() {
      var O = +areaRingSum$1;
      areaSum$1.add(O < 0 ? tau$1 + O : O), this.lineStart = this.lineEnd = this.point = noop$2;
    },
    sphere: function() {
      areaSum$1.add(tau$1);
    }
  };
  function areaRingStart$1() {
    areaStream$1.point = areaPointFirst$1;
  }
  function areaRingEnd$1() {
    areaPoint$1(lambda00$2, phi00$2);
  }
  function areaPointFirst$1(O, _) {
    areaStream$1.point = areaPoint$1, lambda00$2 = O, phi00$2 = _, O *= radians, _ *= radians, lambda0$1 = O, cosPhi0 = cos$1(_ = _ / 2 + quarterPi), sinPhi0 = sin$1(_);
  }
  function areaPoint$1(O, _) {
    O *= radians, _ *= radians, _ = _ / 2 + quarterPi;
    var H = O - lambda0$1, V = H >= 0 ? 1 : -1, q = V * H, G = cos$1(_), J = sin$1(_), Z = sinPhi0 * J, te = cosPhi0 * G + Z * cos$1(q), ne = Z * V * sin$1(q);
    areaRingSum$1.add(atan2(ne, te)), lambda0$1 = O, cosPhi0 = G, sinPhi0 = J;
  }
  function geoArea$1(O) {
    return areaSum$1 = new Adder(), geoStream(O, areaStream$1), areaSum$1 * 2;
  }
  function spherical(O) {
    return [atan2(O[1], O[0]), asin$1(O[2])];
  }
  function cartesian(O) {
    var _ = O[0], H = O[1], V = cos$1(H);
    return [V * cos$1(_), V * sin$1(_), sin$1(H)];
  }
  function cartesianDot(O, _) {
    return O[0] * _[0] + O[1] * _[1] + O[2] * _[2];
  }
  function cartesianCross(O, _) {
    return [O[1] * _[2] - O[2] * _[1], O[2] * _[0] - O[0] * _[2], O[0] * _[1] - O[1] * _[0]];
  }
  function cartesianAddInPlace(O, _) {
    O[0] += _[0], O[1] += _[1], O[2] += _[2];
  }
  function cartesianScale(O, _) {
    return [O[0] * _, O[1] * _, O[2] * _];
  }
  function cartesianNormalizeInPlace(O) {
    var _ = sqrt$1(O[0] * O[0] + O[1] * O[1] + O[2] * O[2]);
    O[0] /= _, O[1] /= _, O[2] /= _;
  }
  var lambda0, phi0, lambda1, phi1, lambda2, lambda00$1, phi00$1, p0, deltaSum, ranges, range$6, boundsStream$1 = {
    point: boundsPoint$1,
    lineStart: boundsLineStart,
    lineEnd: boundsLineEnd,
    polygonStart: function() {
      boundsStream$1.point = boundsRingPoint, boundsStream$1.lineStart = boundsRingStart, boundsStream$1.lineEnd = boundsRingEnd, deltaSum = new Adder(), areaStream$1.polygonStart();
    },
    polygonEnd: function() {
      areaStream$1.polygonEnd(), boundsStream$1.point = boundsPoint$1, boundsStream$1.lineStart = boundsLineStart, boundsStream$1.lineEnd = boundsLineEnd, areaRingSum$1 < 0 ? (lambda0 = -(lambda1 = 180), phi0 = -(phi1 = 90)) : deltaSum > epsilon$3 ? phi1 = 90 : deltaSum < -1e-6 && (phi0 = -90), range$6[0] = lambda0, range$6[1] = lambda1;
    },
    sphere: function() {
      lambda0 = -(lambda1 = 180), phi0 = -(phi1 = 90);
    }
  };
  function boundsPoint$1(O, _) {
    ranges.push(range$6 = [lambda0 = O, lambda1 = O]), _ < phi0 && (phi0 = _), _ > phi1 && (phi1 = _);
  }
  function linePoint(O, _) {
    var H = cartesian([O * radians, _ * radians]);
    if (p0) {
      var V = cartesianCross(p0, H), q = [V[1], -V[0], 0], G = cartesianCross(q, V);
      cartesianNormalizeInPlace(G), G = spherical(G);
      var J = O - lambda2, Z = J > 0 ? 1 : -1, te = G[0] * degrees * Z, ne, ee = abs$1(J) > 180;
      ee ^ (Z * lambda2 < te && te < Z * O) ? (ne = G[1] * degrees, ne > phi1 && (phi1 = ne)) : (te = (te + 360) % 360 - 180, ee ^ (Z * lambda2 < te && te < Z * O) ? (ne = -G[1] * degrees, ne < phi0 && (phi0 = ne)) : (_ < phi0 && (phi0 = _), _ > phi1 && (phi1 = _))), ee ? O < lambda2 ? angle(lambda0, O) > angle(lambda0, lambda1) && (lambda1 = O) : angle(O, lambda1) > angle(lambda0, lambda1) && (lambda0 = O) : lambda1 >= lambda0 ? (O < lambda0 && (lambda0 = O), O > lambda1 && (lambda1 = O)) : O > lambda2 ? angle(lambda0, O) > angle(lambda0, lambda1) && (lambda1 = O) : angle(O, lambda1) > angle(lambda0, lambda1) && (lambda0 = O);
    } else
      ranges.push(range$6 = [lambda0 = O, lambda1 = O]);
    _ < phi0 && (phi0 = _), _ > phi1 && (phi1 = _), p0 = H, lambda2 = O;
  }
  function boundsLineStart() {
    boundsStream$1.point = linePoint;
  }
  function boundsLineEnd() {
    range$6[0] = lambda0, range$6[1] = lambda1, boundsStream$1.point = boundsPoint$1, p0 = null;
  }
  function boundsRingPoint(O, _) {
    if (p0) {
      var H = O - lambda2;
      deltaSum.add(abs$1(H) > 180 ? H + (H > 0 ? 360 : -360) : H);
    } else
      lambda00$1 = O, phi00$1 = _;
    areaStream$1.point(O, _), linePoint(O, _);
  }
  function boundsRingStart() {
    areaStream$1.lineStart();
  }
  function boundsRingEnd() {
    boundsRingPoint(lambda00$1, phi00$1), areaStream$1.lineEnd(), abs$1(deltaSum) > epsilon$3 && (lambda0 = -(lambda1 = 180)), range$6[0] = lambda0, range$6[1] = lambda1, p0 = null;
  }
  function angle(O, _) {
    return (_ -= O) < 0 ? _ + 360 : _;
  }
  function rangeCompare(O, _) {
    return O[0] - _[0];
  }
  function rangeContains(O, _) {
    return O[0] <= O[1] ? O[0] <= _ && _ <= O[1] : _ < O[0] || O[1] < _;
  }
  function geoBounds$1(O) {
    var _, H, V, q, G, J, Z;
    if (phi1 = lambda1 = -(lambda0 = phi0 = 1 / 0), ranges = [], geoStream(O, boundsStream$1), H = ranges.length) {
      for (ranges.sort(rangeCompare), _ = 1, V = ranges[0], G = [V]; _ < H; ++_)
        q = ranges[_], rangeContains(V, q[0]) || rangeContains(V, q[1]) ? (angle(V[0], q[1]) > angle(V[0], V[1]) && (V[1] = q[1]), angle(q[0], V[1]) > angle(V[0], V[1]) && (V[0] = q[0])) : G.push(V = q);
      for (J = -1 / 0, H = G.length - 1, _ = 0, V = G[H]; _ <= H; V = q, ++_)
        q = G[_], (Z = angle(V[1], q[0])) > J && (J = Z, lambda0 = q[0], lambda1 = V[1]);
    }
    return ranges = range$6 = null, lambda0 === 1 / 0 || phi0 === 1 / 0 ? [[NaN, NaN], [NaN, NaN]] : [[lambda0, phi0], [lambda1, phi1]];
  }
  var W0, W1, X0$1, Y0$1, Z0$1, X1$1, Y1$1, Z1$1, X2$2, Y2$2, Z2$1, lambda00, phi00, x0$4, y0$4, z0, centroidStream$1 = {
    sphere: noop$2,
    point: centroidPoint$1,
    lineStart: centroidLineStart$1,
    lineEnd: centroidLineEnd$1,
    polygonStart: function() {
      centroidStream$1.lineStart = centroidRingStart$1, centroidStream$1.lineEnd = centroidRingEnd$1;
    },
    polygonEnd: function() {
      centroidStream$1.lineStart = centroidLineStart$1, centroidStream$1.lineEnd = centroidLineEnd$1;
    }
  };
  function centroidPoint$1(O, _) {
    O *= radians, _ *= radians;
    var H = cos$1(_);
    centroidPointCartesian(H * cos$1(O), H * sin$1(O), sin$1(_));
  }
  function centroidPointCartesian(O, _, H) {
    ++W0, X0$1 += (O - X0$1) / W0, Y0$1 += (_ - Y0$1) / W0, Z0$1 += (H - Z0$1) / W0;
  }
  function centroidLineStart$1() {
    centroidStream$1.point = centroidLinePointFirst;
  }
  function centroidLinePointFirst(O, _) {
    O *= radians, _ *= radians;
    var H = cos$1(_);
    x0$4 = H * cos$1(O), y0$4 = H * sin$1(O), z0 = sin$1(_), centroidStream$1.point = centroidLinePoint, centroidPointCartesian(x0$4, y0$4, z0);
  }
  function centroidLinePoint(O, _) {
    O *= radians, _ *= radians;
    var H = cos$1(_), V = H * cos$1(O), q = H * sin$1(O), G = sin$1(_), J = atan2(sqrt$1((J = y0$4 * G - z0 * q) * J + (J = z0 * V - x0$4 * G) * J + (J = x0$4 * q - y0$4 * V) * J), x0$4 * V + y0$4 * q + z0 * G);
    W1 += J, X1$1 += J * (x0$4 + (x0$4 = V)), Y1$1 += J * (y0$4 + (y0$4 = q)), Z1$1 += J * (z0 + (z0 = G)), centroidPointCartesian(x0$4, y0$4, z0);
  }
  function centroidLineEnd$1() {
    centroidStream$1.point = centroidPoint$1;
  }
  function centroidRingStart$1() {
    centroidStream$1.point = centroidRingPointFirst;
  }
  function centroidRingEnd$1() {
    centroidRingPoint(lambda00, phi00), centroidStream$1.point = centroidPoint$1;
  }
  function centroidRingPointFirst(O, _) {
    lambda00 = O, phi00 = _, O *= radians, _ *= radians, centroidStream$1.point = centroidRingPoint;
    var H = cos$1(_);
    x0$4 = H * cos$1(O), y0$4 = H * sin$1(O), z0 = sin$1(_), centroidPointCartesian(x0$4, y0$4, z0);
  }
  function centroidRingPoint(O, _) {
    O *= radians, _ *= radians;
    var H = cos$1(_), V = H * cos$1(O), q = H * sin$1(O), G = sin$1(_), J = y0$4 * G - z0 * q, Z = z0 * V - x0$4 * G, te = x0$4 * q - y0$4 * V, ne = hypot(J, Z, te), ee = asin$1(ne), ae = ne && -ee / ne;
    X2$2.add(ae * J), Y2$2.add(ae * Z), Z2$1.add(ae * te), W1 += ee, X1$1 += ee * (x0$4 + (x0$4 = V)), Y1$1 += ee * (y0$4 + (y0$4 = q)), Z1$1 += ee * (z0 + (z0 = G)), centroidPointCartesian(x0$4, y0$4, z0);
  }
  function geoCentroid$1(O) {
    W0 = W1 = X0$1 = Y0$1 = Z0$1 = X1$1 = Y1$1 = Z1$1 = 0, X2$2 = new Adder(), Y2$2 = new Adder(), Z2$1 = new Adder(), geoStream(O, centroidStream$1);
    var _ = +X2$2, H = +Y2$2, V = +Z2$1, q = hypot(_, H, V);
    return q < epsilon2 && (_ = X1$1, H = Y1$1, V = Z1$1, W1 < epsilon$3 && (_ = X0$1, H = Y0$1, V = Z0$1), q = hypot(_, H, V), q < epsilon2) ? [NaN, NaN] : [atan2(H, _) * degrees, asin$1(V / q) * degrees];
  }
  function compose$1(O, _) {
    function H(V, q) {
      return V = O(V, q), _(V[0], V[1]);
    }
    return O.invert && _.invert && (H.invert = function(V, q) {
      return V = _.invert(V, q), V && O.invert(V[0], V[1]);
    }), H;
  }
  function rotationIdentity(O, _) {
    return abs$1(O) > pi$1 && (O -= Math.round(O / tau$1) * tau$1), [O, _];
  }
  rotationIdentity.invert = rotationIdentity;
  function rotateRadians(O, _, H) {
    return (O %= tau$1) ? _ || H ? compose$1(rotationLambda(O), rotationPhiGamma(_, H)) : rotationLambda(O) : _ || H ? rotationPhiGamma(_, H) : rotationIdentity;
  }
  function forwardRotationLambda(O) {
    return function(_, H) {
      return _ += O, abs$1(_) > pi$1 && (_ -= Math.round(_ / tau$1) * tau$1), [_, H];
    };
  }
  function rotationLambda(O) {
    var _ = forwardRotationLambda(O);
    return _.invert = forwardRotationLambda(-O), _;
  }
  function rotationPhiGamma(O, _) {
    var H = cos$1(O), V = sin$1(O), q = cos$1(_), G = sin$1(_);
    function J(Z, te) {
      var ne = cos$1(te), ee = cos$1(Z) * ne, ae = sin$1(Z) * ne, oe = sin$1(te), se = oe * H + ee * V;
      return [
        atan2(ae * q - se * G, ee * H - oe * V),
        asin$1(se * q + ae * G)
      ];
    }
    return J.invert = function(Z, te) {
      var ne = cos$1(te), ee = cos$1(Z) * ne, ae = sin$1(Z) * ne, oe = sin$1(te), se = oe * q - ae * G;
      return [
        atan2(ae * q + oe * G, ee * H + se * V),
        asin$1(se * H - ee * V)
      ];
    }, J;
  }
  function rotation(O) {
    O = rotateRadians(O[0] * radians, O[1] * radians, O.length > 2 ? O[2] * radians : 0);
    function _(H) {
      return H = O(H[0] * radians, H[1] * radians), H[0] *= degrees, H[1] *= degrees, H;
    }
    return _.invert = function(H) {
      return H = O.invert(H[0] * radians, H[1] * radians), H[0] *= degrees, H[1] *= degrees, H;
    }, _;
  }
  function circleStream(O, _, H, V, q, G) {
    if (H) {
      var J = cos$1(_), Z = sin$1(_), te = V * H;
      q == null ? (q = _ + V * tau$1, G = _ - te / 2) : (q = circleRadius(J, q), G = circleRadius(J, G), (V > 0 ? q < G : q > G) && (q += V * tau$1));
      for (var ne, ee = q; V > 0 ? ee > G : ee < G; ee -= te)
        ne = spherical([J, -Z * cos$1(ee), -Z * sin$1(ee)]), O.point(ne[0], ne[1]);
    }
  }
  function circleRadius(O, _) {
    _ = cartesian(_), _[0] -= O, cartesianNormalizeInPlace(_);
    var H = acos(-_[1]);
    return ((-_[2] < 0 ? -H : H) + tau$1 - epsilon$3) % tau$1;
  }
  function clipBuffer() {
    var O = [], _;
    return {
      point: function(H, V, q) {
        _.push([H, V, q]);
      },
      lineStart: function() {
        O.push(_ = []);
      },
      lineEnd: noop$2,
      rejoin: function() {
        O.length > 1 && O.push(O.pop().concat(O.shift()));
      },
      result: function() {
        var H = O;
        return O = [], _ = null, H;
      }
    };
  }
  function pointEqual(O, _) {
    return abs$1(O[0] - _[0]) < epsilon$3 && abs$1(O[1] - _[1]) < epsilon$3;
  }
  function Intersection(O, _, H, V) {
    this.x = O, this.z = _, this.o = H, this.e = V, this.v = !1, this.n = this.p = null;
  }
  function clipRejoin(O, _, H, V, q) {
    var G = [], J = [], Z, te;
    if (O.forEach(function(ce) {
      if (!((le = ce.length - 1) <= 0)) {
        var le, ue = ce[0], de = ce[le], fe;
        if (pointEqual(ue, de)) {
          if (!ue[2] && !de[2]) {
            for (q.lineStart(), Z = 0; Z < le; ++Z) q.point((ue = ce[Z])[0], ue[1]);
            q.lineEnd();
            return;
          }
          de[0] += 2 * epsilon$3;
        }
        G.push(fe = new Intersection(ue, ce, null, !0)), J.push(fe.o = new Intersection(ue, null, fe, !1)), G.push(fe = new Intersection(de, ce, null, !1)), J.push(fe.o = new Intersection(de, null, fe, !0));
      }
    }), !!G.length) {
      for (J.sort(_), link(G), link(J), Z = 0, te = J.length; Z < te; ++Z)
        J[Z].e = H = !H;
      for (var ne = G[0], ee, ae; ; ) {
        for (var oe = ne, se = !0; oe.v; ) if ((oe = oe.n) === ne) return;
        ee = oe.z, q.lineStart();
        do {
          if (oe.v = oe.o.v = !0, oe.e) {
            if (se)
              for (Z = 0, te = ee.length; Z < te; ++Z) q.point((ae = ee[Z])[0], ae[1]);
            else
              V(oe.x, oe.n.x, 1, q);
            oe = oe.n;
          } else {
            if (se)
              for (ee = oe.p.z, Z = ee.length - 1; Z >= 0; --Z) q.point((ae = ee[Z])[0], ae[1]);
            else
              V(oe.x, oe.p.x, -1, q);
            oe = oe.p;
          }
          oe = oe.o, ee = oe.z, se = !se;
        } while (!oe.v);
        q.lineEnd();
      }
    }
  }
  function link(O) {
    if (_ = O.length) {
      for (var _, H = 0, V = O[0], q; ++H < _; )
        V.n = q = O[H], q.p = V, V = q;
      V.n = q = O[0], q.p = V;
    }
  }
  function longitude(O) {
    return abs$1(O[0]) <= pi$1 ? O[0] : sign(O[0]) * ((abs$1(O[0]) + pi$1) % tau$1 - pi$1);
  }
  function polygonContains(O, _) {
    var H = longitude(_), V = _[1], q = sin$1(V), G = [sin$1(H), -cos$1(H), 0], J = 0, Z = 0, te = new Adder();
    q === 1 ? V = halfPi$1 + epsilon$3 : q === -1 && (V = -halfPi$1 - epsilon$3);
    for (var ne = 0, ee = O.length; ne < ee; ++ne)
      if (oe = (ae = O[ne]).length)
        for (var ae, oe, se = ae[oe - 1], ce = longitude(se), le = se[1] / 2 + quarterPi, ue = sin$1(le), de = cos$1(le), fe = 0; fe < oe; ++fe, ce = me, ue = ye, de = ge, se = he) {
          var he = ae[fe], me = longitude(he), pe = he[1] / 2 + quarterPi, ye = sin$1(pe), ge = cos$1(pe), ve = me - ce, be = ve >= 0 ? 1 : -1, Ce = be * ve, Se = Ce > pi$1, we = ue * ye;
          if (te.add(atan2(we * be * sin$1(Ce), de * ge + we * cos$1(Ce))), J += Se ? ve + be * tau$1 : ve, Se ^ ce >= H ^ me >= H) {
            var Ee = cartesianCross(cartesian(se), cartesian(he));
            cartesianNormalizeInPlace(Ee);
            var xe = cartesianCross(G, Ee);
            cartesianNormalizeInPlace(xe);
            var Le = (Se ^ ve >= 0 ? -1 : 1) * asin$1(xe[2]);
            (V > Le || V === Le && (Ee[0] || Ee[1])) && (Z += Se ^ ve >= 0 ? 1 : -1);
          }
        }
    return (J < -1e-6 || J < epsilon$3 && te < -1e-12) ^ Z & 1;
  }
  function clip$2(O, _, H, V) {
    return function(q) {
      var G = _(q), J = clipBuffer(), Z = _(J), te = !1, ne, ee, ae, oe = {
        point: se,
        lineStart: le,
        lineEnd: ue,
        polygonStart: function() {
          oe.point = de, oe.lineStart = fe, oe.lineEnd = he, ee = [], ne = [];
        },
        polygonEnd: function() {
          oe.point = se, oe.lineStart = le, oe.lineEnd = ue, ee = merge$2(ee);
          var me = polygonContains(ne, V);
          ee.length ? (te || (q.polygonStart(), te = !0), clipRejoin(ee, compareIntersection, me, H, q)) : me && (te || (q.polygonStart(), te = !0), q.lineStart(), H(null, null, 1, q), q.lineEnd()), te && (q.polygonEnd(), te = !1), ee = ne = null;
        },
        sphere: function() {
          q.polygonStart(), q.lineStart(), H(null, null, 1, q), q.lineEnd(), q.polygonEnd();
        }
      };
      function se(me, pe) {
        O(me, pe) && q.point(me, pe);
      }
      function ce(me, pe) {
        G.point(me, pe);
      }
      function le() {
        oe.point = ce, G.lineStart();
      }
      function ue() {
        oe.point = se, G.lineEnd();
      }
      function de(me, pe) {
        ae.push([me, pe]), Z.point(me, pe);
      }
      function fe() {
        Z.lineStart(), ae = [];
      }
      function he() {
        de(ae[0][0], ae[0][1]), Z.lineEnd();
        var me = Z.clean(), pe = J.result(), ye, ge = pe.length, ve, be, Ce;
        if (ae.pop(), ne.push(ae), ae = null, !!ge) {
          if (me & 1) {
            if (be = pe[0], (ve = be.length - 1) > 0) {
              for (te || (q.polygonStart(), te = !0), q.lineStart(), ye = 0; ye < ve; ++ye) q.point((Ce = be[ye])[0], Ce[1]);
              q.lineEnd();
            }
            return;
          }
          ge > 1 && me & 2 && pe.push(pe.pop().concat(pe.shift())), ee.push(pe.filter(validSegment));
        }
      }
      return oe;
    };
  }
  function validSegment(O) {
    return O.length > 1;
  }
  function compareIntersection(O, _) {
    return ((O = O.x)[0] < 0 ? O[1] - halfPi$1 - epsilon$3 : halfPi$1 - O[1]) - ((_ = _.x)[0] < 0 ? _[1] - halfPi$1 - epsilon$3 : halfPi$1 - _[1]);
  }
  const clipAntimeridian = clip$2(
    function() {
      return !0;
    },
    clipAntimeridianLine,
    clipAntimeridianInterpolate,
    [-pi$1, -halfPi$1]
  );
  function clipAntimeridianLine(O) {
    var _ = NaN, H = NaN, V = NaN, q;
    return {
      lineStart: function() {
        O.lineStart(), q = 1;
      },
      point: function(G, J) {
        var Z = G > 0 ? pi$1 : -pi$1, te = abs$1(G - _);
        abs$1(te - pi$1) < epsilon$3 ? (O.point(_, H = (H + J) / 2 > 0 ? halfPi$1 : -halfPi$1), O.point(V, H), O.lineEnd(), O.lineStart(), O.point(Z, H), O.point(G, H), q = 0) : V !== Z && te >= pi$1 && (abs$1(_ - V) < epsilon$3 && (_ -= V * epsilon$3), abs$1(G - Z) < epsilon$3 && (G -= Z * epsilon$3), H = clipAntimeridianIntersect(_, H, G, J), O.point(V, H), O.lineEnd(), O.lineStart(), O.point(Z, H), q = 0), O.point(_ = G, H = J), V = Z;
      },
      lineEnd: function() {
        O.lineEnd(), _ = H = NaN;
      },
      clean: function() {
        return 2 - q;
      }
    };
  }
  function clipAntimeridianIntersect(O, _, H, V) {
    var q, G, J = sin$1(O - H);
    return abs$1(J) > epsilon$3 ? atan((sin$1(_) * (G = cos$1(V)) * sin$1(H) - sin$1(V) * (q = cos$1(_)) * sin$1(O)) / (q * G * J)) : (_ + V) / 2;
  }
  function clipAntimeridianInterpolate(O, _, H, V) {
    var q;
    if (O == null)
      q = H * halfPi$1, V.point(-pi$1, q), V.point(0, q), V.point(pi$1, q), V.point(pi$1, 0), V.point(pi$1, -q), V.point(0, -q), V.point(-pi$1, -q), V.point(-pi$1, 0), V.point(-pi$1, q);
    else if (abs$1(O[0] - _[0]) > epsilon$3) {
      var G = O[0] < _[0] ? pi$1 : -pi$1;
      q = H * G / 2, V.point(-G, q), V.point(0, q), V.point(G, q);
    } else
      V.point(_[0], _[1]);
  }
  function clipCircle(O) {
    var _ = cos$1(O), H = 2 * radians, V = _ > 0, q = abs$1(_) > epsilon$3;
    function G(ee, ae, oe, se) {
      circleStream(se, O, H, oe, ee, ae);
    }
    function J(ee, ae) {
      return cos$1(ee) * cos$1(ae) > _;
    }
    function Z(ee) {
      var ae, oe, se, ce, le;
      return {
        lineStart: function() {
          ce = se = !1, le = 1;
        },
        point: function(ue, de) {
          var fe = [ue, de], he, me = J(ue, de), pe = V ? me ? 0 : ne(ue, de) : me ? ne(ue + (ue < 0 ? pi$1 : -pi$1), de) : 0;
          if (!ae && (ce = se = me) && ee.lineStart(), me !== se && (he = te(ae, fe), (!he || pointEqual(ae, he) || pointEqual(fe, he)) && (fe[2] = 1)), me !== se)
            le = 0, me ? (ee.lineStart(), he = te(fe, ae), ee.point(he[0], he[1])) : (he = te(ae, fe), ee.point(he[0], he[1], 2), ee.lineEnd()), ae = he;
          else if (q && ae && V ^ me) {
            var ye;
            !(pe & oe) && (ye = te(fe, ae, !0)) && (le = 0, V ? (ee.lineStart(), ee.point(ye[0][0], ye[0][1]), ee.point(ye[1][0], ye[1][1]), ee.lineEnd()) : (ee.point(ye[1][0], ye[1][1]), ee.lineEnd(), ee.lineStart(), ee.point(ye[0][0], ye[0][1], 3)));
          }
          me && (!ae || !pointEqual(ae, fe)) && ee.point(fe[0], fe[1]), ae = fe, se = me, oe = pe;
        },
        lineEnd: function() {
          se && ee.lineEnd(), ae = null;
        },
        // Rejoin first and last segments if there were intersections and the first
        // and last points were visible.
        clean: function() {
          return le | (ce && se) << 1;
        }
      };
    }
    function te(ee, ae, oe) {
      var se = cartesian(ee), ce = cartesian(ae), le = [1, 0, 0], ue = cartesianCross(se, ce), de = cartesianDot(ue, ue), fe = ue[0], he = de - fe * fe;
      if (!he) return !oe && ee;
      var me = _ * de / he, pe = -_ * fe / he, ye = cartesianCross(le, ue), ge = cartesianScale(le, me), ve = cartesianScale(ue, pe);
      cartesianAddInPlace(ge, ve);
      var be = ye, Ce = cartesianDot(ge, be), Se = cartesianDot(be, be), we = Ce * Ce - Se * (cartesianDot(ge, ge) - 1);
      if (!(we < 0)) {
        var Ee = sqrt$1(we), xe = cartesianScale(be, (-Ce - Ee) / Se);
        if (cartesianAddInPlace(xe, ge), xe = spherical(xe), !oe) return xe;
        var Le = ee[0], Qe = ae[0], ke = ee[1], Me = ae[1], Re;
        Qe < Le && (Re = Le, Le = Qe, Qe = Re);
        var De = Qe - Le, Pe = abs$1(De - pi$1) < epsilon$3, $e = Pe || De < epsilon$3;
        if (!Pe && Me < ke && (Re = ke, ke = Me, Me = Re), $e ? Pe ? ke + Me > 0 ^ xe[1] < (abs$1(xe[0] - Le) < epsilon$3 ? ke : Me) : ke <= xe[1] && xe[1] <= Me : De > pi$1 ^ (Le <= xe[0] && xe[0] <= Qe)) {
          var qe = cartesianScale(be, (-Ce + Ee) / Se);
          return cartesianAddInPlace(qe, ge), [xe, spherical(qe)];
        }
      }
    }
    function ne(ee, ae) {
      var oe = V ? O : pi$1 - O, se = 0;
      return ee < -oe ? se |= 1 : ee > oe && (se |= 2), ae < -oe ? se |= 4 : ae > oe && (se |= 8), se;
    }
    return clip$2(J, Z, G, V ? [0, -O] : [-pi$1, O - pi$1]);
  }
  function clipLine(O, _, H, V, q, G) {
    var J = O[0], Z = O[1], te = _[0], ne = _[1], ee = 0, ae = 1, oe = te - J, se = ne - Z, ce;
    if (ce = H - J, !(!oe && ce > 0)) {
      if (ce /= oe, oe < 0) {
        if (ce < ee) return;
        ce < ae && (ae = ce);
      } else if (oe > 0) {
        if (ce > ae) return;
        ce > ee && (ee = ce);
      }
      if (ce = q - J, !(!oe && ce < 0)) {
        if (ce /= oe, oe < 0) {
          if (ce > ae) return;
          ce > ee && (ee = ce);
        } else if (oe > 0) {
          if (ce < ee) return;
          ce < ae && (ae = ce);
        }
        if (ce = V - Z, !(!se && ce > 0)) {
          if (ce /= se, se < 0) {
            if (ce < ee) return;
            ce < ae && (ae = ce);
          } else if (se > 0) {
            if (ce > ae) return;
            ce > ee && (ee = ce);
          }
          if (ce = G - Z, !(!se && ce < 0)) {
            if (ce /= se, se < 0) {
              if (ce > ae) return;
              ce > ee && (ee = ce);
            } else if (se > 0) {
              if (ce < ee) return;
              ce < ae && (ae = ce);
            }
            return ee > 0 && (O[0] = J + ee * oe, O[1] = Z + ee * se), ae < 1 && (_[0] = J + ae * oe, _[1] = Z + ae * se), !0;
          }
        }
      }
    }
  }
  var clipMax = 1e9, clipMin = -1e9;
  function clipRectangle(O, _, H, V) {
    function q(ne, ee) {
      return O <= ne && ne <= H && _ <= ee && ee <= V;
    }
    function G(ne, ee, ae, oe) {
      var se = 0, ce = 0;
      if (ne == null || (se = J(ne, ae)) !== (ce = J(ee, ae)) || te(ne, ee) < 0 ^ ae > 0)
        do
          oe.point(se === 0 || se === 3 ? O : H, se > 1 ? V : _);
        while ((se = (se + ae + 4) % 4) !== ce);
      else
        oe.point(ee[0], ee[1]);
    }
    function J(ne, ee) {
      return abs$1(ne[0] - O) < epsilon$3 ? ee > 0 ? 0 : 3 : abs$1(ne[0] - H) < epsilon$3 ? ee > 0 ? 2 : 1 : abs$1(ne[1] - _) < epsilon$3 ? ee > 0 ? 1 : 0 : ee > 0 ? 3 : 2;
    }
    function Z(ne, ee) {
      return te(ne.x, ee.x);
    }
    function te(ne, ee) {
      var ae = J(ne, 1), oe = J(ee, 1);
      return ae !== oe ? ae - oe : ae === 0 ? ee[1] - ne[1] : ae === 1 ? ne[0] - ee[0] : ae === 2 ? ne[1] - ee[1] : ee[0] - ne[0];
    }
    return function(ne) {
      var ee = ne, ae = clipBuffer(), oe, se, ce, le, ue, de, fe, he, me, pe, ye, ge = {
        point: ve,
        lineStart: we,
        lineEnd: Ee,
        polygonStart: Ce,
        polygonEnd: Se
      };
      function ve(Le, Qe) {
        q(Le, Qe) && ee.point(Le, Qe);
      }
      function be() {
        for (var Le = 0, Qe = 0, ke = se.length; Qe < ke; ++Qe)
          for (var Me = se[Qe], Re = 1, De = Me.length, Pe = Me[0], $e, qe, Xe = Pe[0], Ve = Pe[1]; Re < De; ++Re)
            $e = Xe, qe = Ve, Pe = Me[Re], Xe = Pe[0], Ve = Pe[1], qe <= V ? Ve > V && (Xe - $e) * (V - qe) > (Ve - qe) * (O - $e) && ++Le : Ve <= V && (Xe - $e) * (V - qe) < (Ve - qe) * (O - $e) && --Le;
        return Le;
      }
      function Ce() {
        ee = ae, oe = [], se = [], ye = !0;
      }
      function Se() {
        var Le = be(), Qe = ye && Le, ke = (oe = merge$2(oe)).length;
        (Qe || ke) && (ne.polygonStart(), Qe && (ne.lineStart(), G(null, null, 1, ne), ne.lineEnd()), ke && clipRejoin(oe, Z, Le, G, ne), ne.polygonEnd()), ee = ne, oe = se = ce = null;
      }
      function we() {
        ge.point = xe, se && se.push(ce = []), pe = !0, me = !1, fe = he = NaN;
      }
      function Ee() {
        oe && (xe(le, ue), de && me && ae.rejoin(), oe.push(ae.result())), ge.point = ve, me && ee.lineEnd();
      }
      function xe(Le, Qe) {
        var ke = q(Le, Qe);
        if (se && ce.push([Le, Qe]), pe)
          le = Le, ue = Qe, de = ke, pe = !1, ke && (ee.lineStart(), ee.point(Le, Qe));
        else if (ke && me) ee.point(Le, Qe);
        else {
          var Me = [fe = Math.max(clipMin, Math.min(clipMax, fe)), he = Math.max(clipMin, Math.min(clipMax, he))], Re = [Le = Math.max(clipMin, Math.min(clipMax, Le)), Qe = Math.max(clipMin, Math.min(clipMax, Qe))];
          clipLine(Me, Re, O, _, H, V) ? (me || (ee.lineStart(), ee.point(Me[0], Me[1])), ee.point(Re[0], Re[1]), ke || ee.lineEnd(), ye = !1) : ke && (ee.lineStart(), ee.point(Le, Qe), ye = !1);
        }
        fe = Le, he = Qe, me = ke;
      }
      return ge;
    };
  }
  function graticuleX(O, _, H) {
    var V = range$7(O, _ - epsilon$3, H).concat(_);
    return function(q) {
      return V.map(function(G) {
        return [q, G];
      });
    };
  }
  function graticuleY(O, _, H) {
    var V = range$7(O, _ - epsilon$3, H).concat(_);
    return function(q) {
      return V.map(function(G) {
        return [G, q];
      });
    };
  }
  function graticule() {
    var O, _, H, V, q, G, J, Z, te = 10, ne = te, ee = 90, ae = 360, oe, se, ce, le, ue = 2.5;
    function de() {
      return { type: "MultiLineString", coordinates: fe() };
    }
    function fe() {
      return range$7(ceil(V / ee) * ee, H, ee).map(ce).concat(range$7(ceil(Z / ae) * ae, J, ae).map(le)).concat(range$7(ceil(_ / te) * te, O, te).filter(function(he) {
        return abs$1(he % ee) > epsilon$3;
      }).map(oe)).concat(range$7(ceil(G / ne) * ne, q, ne).filter(function(he) {
        return abs$1(he % ae) > epsilon$3;
      }).map(se));
    }
    return de.lines = function() {
      return fe().map(function(he) {
        return { type: "LineString", coordinates: he };
      });
    }, de.outline = function() {
      return {
        type: "Polygon",
        coordinates: [
          ce(V).concat(
            le(J).slice(1),
            ce(H).reverse().slice(1),
            le(Z).reverse().slice(1)
          )
        ]
      };
    }, de.extent = function(he) {
      return arguments.length ? de.extentMajor(he).extentMinor(he) : de.extentMinor();
    }, de.extentMajor = function(he) {
      return arguments.length ? (V = +he[0][0], H = +he[1][0], Z = +he[0][1], J = +he[1][1], V > H && (he = V, V = H, H = he), Z > J && (he = Z, Z = J, J = he), de.precision(ue)) : [[V, Z], [H, J]];
    }, de.extentMinor = function(he) {
      return arguments.length ? (_ = +he[0][0], O = +he[1][0], G = +he[0][1], q = +he[1][1], _ > O && (he = _, _ = O, O = he), G > q && (he = G, G = q, q = he), de.precision(ue)) : [[_, G], [O, q]];
    }, de.step = function(he) {
      return arguments.length ? de.stepMajor(he).stepMinor(he) : de.stepMinor();
    }, de.stepMajor = function(he) {
      return arguments.length ? (ee = +he[0], ae = +he[1], de) : [ee, ae];
    }, de.stepMinor = function(he) {
      return arguments.length ? (te = +he[0], ne = +he[1], de) : [te, ne];
    }, de.precision = function(he) {
      return arguments.length ? (ue = +he, oe = graticuleX(G, q, 90), se = graticuleY(_, O, ue), ce = graticuleX(Z, J, 90), le = graticuleY(V, H, ue), de) : ue;
    }, de.extentMajor([[-180, -90 + epsilon$3], [180, 90 - epsilon$3]]).extentMinor([[-180, -80 - epsilon$3], [180, 80 + epsilon$3]]);
  }
  const identity$1 = (O) => O;
  var areaSum = new Adder(), areaRingSum = new Adder(), x00$2, y00$2, x0$3, y0$3, areaStream = {
    point: noop$2,
    lineStart: noop$2,
    lineEnd: noop$2,
    polygonStart: function() {
      areaStream.lineStart = areaRingStart, areaStream.lineEnd = areaRingEnd;
    },
    polygonEnd: function() {
      areaStream.lineStart = areaStream.lineEnd = areaStream.point = noop$2, areaSum.add(abs$1(areaRingSum)), areaRingSum = new Adder();
    },
    result: function() {
      var O = areaSum / 2;
      return areaSum = new Adder(), O;
    }
  };
  function areaRingStart() {
    areaStream.point = areaPointFirst;
  }
  function areaPointFirst(O, _) {
    areaStream.point = areaPoint, x00$2 = x0$3 = O, y00$2 = y0$3 = _;
  }
  function areaPoint(O, _) {
    areaRingSum.add(y0$3 * O - x0$3 * _), x0$3 = O, y0$3 = _;
  }
  function areaRingEnd() {
    areaPoint(x00$2, y00$2);
  }
  var x0$2 = 1 / 0, y0$2 = x0$2, x1 = -x0$2, y1 = x1, boundsStream = {
    point: boundsPoint,
    lineStart: noop$2,
    lineEnd: noop$2,
    polygonStart: noop$2,
    polygonEnd: noop$2,
    result: function() {
      var O = [[x0$2, y0$2], [x1, y1]];
      return x1 = y1 = -(y0$2 = x0$2 = 1 / 0), O;
    }
  };
  function boundsPoint(O, _) {
    O < x0$2 && (x0$2 = O), O > x1 && (x1 = O), _ < y0$2 && (y0$2 = _), _ > y1 && (y1 = _);
  }
  var X0 = 0, Y0 = 0, Z0 = 0, X1 = 0, Y1 = 0, Z1 = 0, X2$1 = 0, Y2$1 = 0, Z2 = 0, x00$1, y00$1, x0$1, y0$1, centroidStream = {
    point: centroidPoint,
    lineStart: centroidLineStart,
    lineEnd: centroidLineEnd,
    polygonStart: function() {
      centroidStream.lineStart = centroidRingStart, centroidStream.lineEnd = centroidRingEnd;
    },
    polygonEnd: function() {
      centroidStream.point = centroidPoint, centroidStream.lineStart = centroidLineStart, centroidStream.lineEnd = centroidLineEnd;
    },
    result: function() {
      var O = Z2 ? [X2$1 / Z2, Y2$1 / Z2] : Z1 ? [X1 / Z1, Y1 / Z1] : Z0 ? [X0 / Z0, Y0 / Z0] : [NaN, NaN];
      return X0 = Y0 = Z0 = X1 = Y1 = Z1 = X2$1 = Y2$1 = Z2 = 0, O;
    }
  };
  function centroidPoint(O, _) {
    X0 += O, Y0 += _, ++Z0;
  }
  function centroidLineStart() {
    centroidStream.point = centroidPointFirstLine;
  }
  function centroidPointFirstLine(O, _) {
    centroidStream.point = centroidPointLine, centroidPoint(x0$1 = O, y0$1 = _);
  }
  function centroidPointLine(O, _) {
    var H = O - x0$1, V = _ - y0$1, q = sqrt$1(H * H + V * V);
    X1 += q * (x0$1 + O) / 2, Y1 += q * (y0$1 + _) / 2, Z1 += q, centroidPoint(x0$1 = O, y0$1 = _);
  }
  function centroidLineEnd() {
    centroidStream.point = centroidPoint;
  }
  function centroidRingStart() {
    centroidStream.point = centroidPointFirstRing;
  }
  function centroidRingEnd() {
    centroidPointRing(x00$1, y00$1);
  }
  function centroidPointFirstRing(O, _) {
    centroidStream.point = centroidPointRing, centroidPoint(x00$1 = x0$1 = O, y00$1 = y0$1 = _);
  }
  function centroidPointRing(O, _) {
    var H = O - x0$1, V = _ - y0$1, q = sqrt$1(H * H + V * V);
    X1 += q * (x0$1 + O) / 2, Y1 += q * (y0$1 + _) / 2, Z1 += q, q = y0$1 * O - x0$1 * _, X2$1 += q * (x0$1 + O), Y2$1 += q * (y0$1 + _), Z2 += q * 3, centroidPoint(x0$1 = O, y0$1 = _);
  }
  function PathContext(O) {
    this._context = O;
  }
  PathContext.prototype = {
    _radius: 4.5,
    pointRadius: function(O) {
      return this._radius = O, this;
    },
    polygonStart: function() {
      this._line = 0;
    },
    polygonEnd: function() {
      this._line = NaN;
    },
    lineStart: function() {
      this._point = 0;
    },
    lineEnd: function() {
      this._line === 0 && this._context.closePath(), this._point = NaN;
    },
    point: function(O, _) {
      switch (this._point) {
        case 0: {
          this._context.moveTo(O, _), this._point = 1;
          break;
        }
        case 1: {
          this._context.lineTo(O, _);
          break;
        }
        default: {
          this._context.moveTo(O + this._radius, _), this._context.arc(O, _, this._radius, 0, tau$1);
          break;
        }
      }
    },
    result: noop$2
  };
  var lengthSum = new Adder(), lengthRing, x00, y00, x0, y0, lengthStream = {
    point: noop$2,
    lineStart: function() {
      lengthStream.point = lengthPointFirst;
    },
    lineEnd: function() {
      lengthRing && lengthPoint(x00, y00), lengthStream.point = noop$2;
    },
    polygonStart: function() {
      lengthRing = !0;
    },
    polygonEnd: function() {
      lengthRing = null;
    },
    result: function() {
      var O = +lengthSum;
      return lengthSum = new Adder(), O;
    }
  };
  function lengthPointFirst(O, _) {
    lengthStream.point = lengthPoint, x00 = x0 = O, y00 = y0 = _;
  }
  function lengthPoint(O, _) {
    x0 -= O, y0 -= _, lengthSum.add(sqrt$1(x0 * x0 + y0 * y0)), x0 = O, y0 = _;
  }
  let cacheDigits, cacheAppend, cacheRadius, cacheCircle;
  class PathString {
    constructor(_) {
      this._append = _ == null ? append : appendRound(_), this._radius = 4.5, this._ = "";
    }
    pointRadius(_) {
      return this._radius = +_, this;
    }
    polygonStart() {
      this._line = 0;
    }
    polygonEnd() {
      this._line = NaN;
    }
    lineStart() {
      this._point = 0;
    }
    lineEnd() {
      this._line === 0 && (this._ += "Z"), this._point = NaN;
    }
    point(_, H) {
      switch (this._point) {
        case 0: {
          this._append`M${_},${H}`, this._point = 1;
          break;
        }
        case 1: {
          this._append`L${_},${H}`;
          break;
        }
        default: {
          if (this._append`M${_},${H}`, this._radius !== cacheRadius || this._append !== cacheAppend) {
            const V = this._radius, q = this._;
            this._ = "", this._append`m0,${V}a${V},${V} 0 1,1 0,${-2 * V}a${V},${V} 0 1,1 0,${2 * V}z`, cacheRadius = V, cacheAppend = this._append, cacheCircle = this._, this._ = q;
          }
          this._ += cacheCircle;
          break;
        }
      }
    }
    result() {
      const _ = this._;
      return this._ = "", _.length ? _ : null;
    }
  }
  function append(O) {
    let _ = 1;
    this._ += O[0];
    for (const H = O.length; _ < H; ++_)
      this._ += arguments[_] + O[_];
  }
  function appendRound(O) {
    const _ = Math.floor(O);
    if (!(_ >= 0)) throw new RangeError(`invalid digits: ${O}`);
    if (_ > 15) return append;
    if (_ !== cacheDigits) {
      const H = 10 ** _;
      cacheDigits = _, cacheAppend = function(q) {
        let G = 1;
        this._ += q[0];
        for (const J = q.length; G < J; ++G)
          this._ += Math.round(arguments[G] * H) / H + q[G];
      };
    }
    return cacheAppend;
  }
  function geoPath(O, _) {
    let H = 3, V = 4.5, q, G;
    function J(Z) {
      return Z && (typeof V == "function" && G.pointRadius(+V.apply(this, arguments)), geoStream(Z, q(G))), G.result();
    }
    return J.area = function(Z) {
      return geoStream(Z, q(areaStream)), areaStream.result();
    }, J.measure = function(Z) {
      return geoStream(Z, q(lengthStream)), lengthStream.result();
    }, J.bounds = function(Z) {
      return geoStream(Z, q(boundsStream)), boundsStream.result();
    }, J.centroid = function(Z) {
      return geoStream(Z, q(centroidStream)), centroidStream.result();
    }, J.projection = function(Z) {
      return arguments.length ? (q = Z == null ? (O = null, identity$1) : (O = Z).stream, J) : O;
    }, J.context = function(Z) {
      return arguments.length ? (G = Z == null ? (_ = null, new PathString(H)) : new PathContext(_ = Z), typeof V != "function" && G.pointRadius(V), J) : _;
    }, J.pointRadius = function(Z) {
      return arguments.length ? (V = typeof Z == "function" ? Z : (G.pointRadius(+Z), +Z), J) : V;
    }, J.digits = function(Z) {
      if (!arguments.length) return H;
      if (Z == null) H = null;
      else {
        const te = Math.floor(Z);
        if (!(te >= 0)) throw new RangeError(`invalid digits: ${Z}`);
        H = te;
      }
      return _ === null && (G = new PathString(H)), J;
    }, J.projection(O).digits(H).context(_);
  }
  function transformer(O) {
    return function(_) {
      var H = new TransformStream();
      for (var V in O) H[V] = O[V];
      return H.stream = _, H;
    };
  }
  function TransformStream() {
  }
  TransformStream.prototype = {
    constructor: TransformStream,
    point: function(O, _) {
      this.stream.point(O, _);
    },
    sphere: function() {
      this.stream.sphere();
    },
    lineStart: function() {
      this.stream.lineStart();
    },
    lineEnd: function() {
      this.stream.lineEnd();
    },
    polygonStart: function() {
      this.stream.polygonStart();
    },
    polygonEnd: function() {
      this.stream.polygonEnd();
    }
  };
  function fit$1(O, _, H) {
    var V = O.clipExtent && O.clipExtent();
    return O.scale(150).translate([0, 0]), V != null && O.clipExtent(null), geoStream(H, O.stream(boundsStream)), _(boundsStream.result()), V != null && O.clipExtent(V), O;
  }
  function fitExtent(O, _, H) {
    return fit$1(O, function(V) {
      var q = _[1][0] - _[0][0], G = _[1][1] - _[0][1], J = Math.min(q / (V[1][0] - V[0][0]), G / (V[1][1] - V[0][1])), Z = +_[0][0] + (q - J * (V[1][0] + V[0][0])) / 2, te = +_[0][1] + (G - J * (V[1][1] + V[0][1])) / 2;
      O.scale(150 * J).translate([Z, te]);
    }, H);
  }
  function fitSize(O, _, H) {
    return fitExtent(O, [[0, 0], _], H);
  }
  function fitWidth(O, _, H) {
    return fit$1(O, function(V) {
      var q = +_, G = q / (V[1][0] - V[0][0]), J = (q - G * (V[1][0] + V[0][0])) / 2, Z = -G * V[0][1];
      O.scale(150 * G).translate([J, Z]);
    }, H);
  }
  function fitHeight(O, _, H) {
    return fit$1(O, function(V) {
      var q = +_, G = q / (V[1][1] - V[0][1]), J = -G * V[0][0], Z = (q - G * (V[1][1] + V[0][1])) / 2;
      O.scale(150 * G).translate([J, Z]);
    }, H);
  }
  var maxDepth = 16, cosMinDistance = cos$1(30 * radians);
  function resample(O, _) {
    return +_ ? resample$1(O, _) : resampleNone(O);
  }
  function resampleNone(O) {
    return transformer({
      point: function(_, H) {
        _ = O(_, H), this.stream.point(_[0], _[1]);
      }
    });
  }
  function resample$1(O, _) {
    function H(V, q, G, J, Z, te, ne, ee, ae, oe, se, ce, le, ue) {
      var de = ne - V, fe = ee - q, he = de * de + fe * fe;
      if (he > 4 * _ && le--) {
        var me = J + oe, pe = Z + se, ye = te + ce, ge = sqrt$1(me * me + pe * pe + ye * ye), ve = asin$1(ye /= ge), be = abs$1(abs$1(ye) - 1) < epsilon$3 || abs$1(G - ae) < epsilon$3 ? (G + ae) / 2 : atan2(pe, me), Ce = O(be, ve), Se = Ce[0], we = Ce[1], Ee = Se - V, xe = we - q, Le = fe * Ee - de * xe;
        (Le * Le / he > _ || abs$1((de * Ee + fe * xe) / he - 0.5) > 0.3 || J * oe + Z * se + te * ce < cosMinDistance) && (H(V, q, G, J, Z, te, Se, we, be, me /= ge, pe /= ge, ye, le, ue), ue.point(Se, we), H(Se, we, be, me, pe, ye, ne, ee, ae, oe, se, ce, le, ue));
      }
    }
    return function(V) {
      var q, G, J, Z, te, ne, ee, ae, oe, se, ce, le, ue = {
        point: de,
        lineStart: fe,
        lineEnd: me,
        polygonStart: function() {
          V.polygonStart(), ue.lineStart = pe;
        },
        polygonEnd: function() {
          V.polygonEnd(), ue.lineStart = fe;
        }
      };
      function de(ve, be) {
        ve = O(ve, be), V.point(ve[0], ve[1]);
      }
      function fe() {
        ae = NaN, ue.point = he, V.lineStart();
      }
      function he(ve, be) {
        var Ce = cartesian([ve, be]), Se = O(ve, be);
        H(ae, oe, ee, se, ce, le, ae = Se[0], oe = Se[1], ee = ve, se = Ce[0], ce = Ce[1], le = Ce[2], maxDepth, V), V.point(ae, oe);
      }
      function me() {
        ue.point = de, V.lineEnd();
      }
      function pe() {
        fe(), ue.point = ye, ue.lineEnd = ge;
      }
      function ye(ve, be) {
        he(q = ve, be), G = ae, J = oe, Z = se, te = ce, ne = le, ue.point = he;
      }
      function ge() {
        H(ae, oe, ee, se, ce, le, G, J, q, Z, te, ne, maxDepth, V), ue.lineEnd = me, me();
      }
      return ue;
    };
  }
  var transformRadians = transformer({
    point: function(O, _) {
      this.stream.point(O * radians, _ * radians);
    }
  });
  function transformRotate(O) {
    return transformer({
      point: function(_, H) {
        var V = O(_, H);
        return this.stream.point(V[0], V[1]);
      }
    });
  }
  function scaleTranslate(O, _, H, V, q) {
    function G(J, Z) {
      return J *= V, Z *= q, [_ + O * J, H - O * Z];
    }
    return G.invert = function(J, Z) {
      return [(J - _) / O * V, (H - Z) / O * q];
    }, G;
  }
  function scaleTranslateRotate(O, _, H, V, q, G) {
    if (!G) return scaleTranslate(O, _, H, V, q);
    var J = cos$1(G), Z = sin$1(G), te = J * O, ne = Z * O, ee = J / O, ae = Z / O, oe = (Z * H - J * _) / O, se = (Z * _ + J * H) / O;
    function ce(le, ue) {
      return le *= V, ue *= q, [te * le - ne * ue + _, H - ne * le - te * ue];
    }
    return ce.invert = function(le, ue) {
      return [V * (ee * le - ae * ue + oe), q * (se - ae * le - ee * ue)];
    }, ce;
  }
  function projection$1(O) {
    return projectionMutator(function() {
      return O;
    })();
  }
  function projectionMutator(O) {
    var _, H = 150, V = 480, q = 250, G = 0, J = 0, Z = 0, te = 0, ne = 0, ee, ae = 0, oe = 1, se = 1, ce = null, le = clipAntimeridian, ue = null, de, fe, he, me = identity$1, pe = 0.5, ye, ge, ve, be, Ce;
    function Se(Le) {
      return ve(Le[0] * radians, Le[1] * radians);
    }
    function we(Le) {
      return Le = ve.invert(Le[0], Le[1]), Le && [Le[0] * degrees, Le[1] * degrees];
    }
    Se.stream = function(Le) {
      return be && Ce === Le ? be : be = transformRadians(transformRotate(ee)(le(ye(me(Ce = Le)))));
    }, Se.preclip = function(Le) {
      return arguments.length ? (le = Le, ce = void 0, xe()) : le;
    }, Se.postclip = function(Le) {
      return arguments.length ? (me = Le, ue = de = fe = he = null, xe()) : me;
    }, Se.clipAngle = function(Le) {
      return arguments.length ? (le = +Le ? clipCircle(ce = Le * radians) : (ce = null, clipAntimeridian), xe()) : ce * degrees;
    }, Se.clipExtent = function(Le) {
      return arguments.length ? (me = Le == null ? (ue = de = fe = he = null, identity$1) : clipRectangle(ue = +Le[0][0], de = +Le[0][1], fe = +Le[1][0], he = +Le[1][1]), xe()) : ue == null ? null : [[ue, de], [fe, he]];
    }, Se.scale = function(Le) {
      return arguments.length ? (H = +Le, Ee()) : H;
    }, Se.translate = function(Le) {
      return arguments.length ? (V = +Le[0], q = +Le[1], Ee()) : [V, q];
    }, Se.center = function(Le) {
      return arguments.length ? (G = Le[0] % 360 * radians, J = Le[1] % 360 * radians, Ee()) : [G * degrees, J * degrees];
    }, Se.rotate = function(Le) {
      return arguments.length ? (Z = Le[0] % 360 * radians, te = Le[1] % 360 * radians, ne = Le.length > 2 ? Le[2] % 360 * radians : 0, Ee()) : [Z * degrees, te * degrees, ne * degrees];
    }, Se.angle = function(Le) {
      return arguments.length ? (ae = Le % 360 * radians, Ee()) : ae * degrees;
    }, Se.reflectX = function(Le) {
      return arguments.length ? (oe = Le ? -1 : 1, Ee()) : oe < 0;
    }, Se.reflectY = function(Le) {
      return arguments.length ? (se = Le ? -1 : 1, Ee()) : se < 0;
    }, Se.precision = function(Le) {
      return arguments.length ? (ye = resample(ge, pe = Le * Le), xe()) : sqrt$1(pe);
    }, Se.fitExtent = function(Le, Qe) {
      return fitExtent(Se, Le, Qe);
    }, Se.fitSize = function(Le, Qe) {
      return fitSize(Se, Le, Qe);
    }, Se.fitWidth = function(Le, Qe) {
      return fitWidth(Se, Le, Qe);
    }, Se.fitHeight = function(Le, Qe) {
      return fitHeight(Se, Le, Qe);
    };
    function Ee() {
      var Le = scaleTranslateRotate(H, 0, 0, oe, se, ae).apply(null, _(G, J)), Qe = scaleTranslateRotate(H, V - Le[0], q - Le[1], oe, se, ae);
      return ee = rotateRadians(Z, te, ne), ge = compose$1(_, Qe), ve = compose$1(ee, ge), ye = resample(ge, pe), xe();
    }
    function xe() {
      return be = Ce = null, Se;
    }
    return function() {
      return _ = O.apply(this, arguments), Se.invert = _.invert && we, Ee();
    };
  }
  function conicProjection(O) {
    var _ = 0, H = pi$1 / 3, V = projectionMutator(O), q = V(_, H);
    return q.parallels = function(G) {
      return arguments.length ? V(_ = G[0] * radians, H = G[1] * radians) : [_ * degrees, H * degrees];
    }, q;
  }
  function cylindricalEqualAreaRaw(O) {
    var _ = cos$1(O);
    function H(V, q) {
      return [V * _, sin$1(q) / _];
    }
    return H.invert = function(V, q) {
      return [V / _, asin$1(q * _)];
    }, H;
  }
  function conicEqualAreaRaw(O, _) {
    var H = sin$1(O), V = (H + sin$1(_)) / 2;
    if (abs$1(V) < epsilon$3) return cylindricalEqualAreaRaw(O);
    var q = 1 + H * (2 * V - H), G = sqrt$1(q) / V;
    function J(Z, te) {
      var ne = sqrt$1(q - 2 * V * sin$1(te)) / V;
      return [ne * sin$1(Z *= V), G - ne * cos$1(Z)];
    }
    return J.invert = function(Z, te) {
      var ne = G - te, ee = atan2(Z, abs$1(ne)) * sign(ne);
      return ne * V < 0 && (ee -= pi$1 * sign(Z) * sign(ne)), [ee / V, asin$1((q - (Z * Z + ne * ne) * V * V) / (2 * V))];
    }, J;
  }
  function geoConicEqualArea() {
    return conicProjection(conicEqualAreaRaw).scale(155.424).center([0, 33.6442]);
  }
  function geoAlbers() {
    return geoConicEqualArea().parallels([29.5, 45.5]).scale(1070).translate([480, 250]).rotate([96, 0]).center([-0.6, 38.7]);
  }
  function multiplex(O) {
    var _ = O.length;
    return {
      point: function(H, V) {
        for (var q = -1; ++q < _; ) O[q].point(H, V);
      },
      sphere: function() {
        for (var H = -1; ++H < _; ) O[H].sphere();
      },
      lineStart: function() {
        for (var H = -1; ++H < _; ) O[H].lineStart();
      },
      lineEnd: function() {
        for (var H = -1; ++H < _; ) O[H].lineEnd();
      },
      polygonStart: function() {
        for (var H = -1; ++H < _; ) O[H].polygonStart();
      },
      polygonEnd: function() {
        for (var H = -1; ++H < _; ) O[H].polygonEnd();
      }
    };
  }
  function geoAlbersUsa() {
    var O, _, H = geoAlbers(), V, q = geoConicEqualArea().rotate([154, 0]).center([-2, 58.5]).parallels([55, 65]), G, J = geoConicEqualArea().rotate([157, 0]).center([-3, 19.9]).parallels([8, 18]), Z, te, ne = { point: function(oe, se) {
      te = [oe, se];
    } };
    function ee(oe) {
      var se = oe[0], ce = oe[1];
      return te = null, V.point(se, ce), te || (G.point(se, ce), te) || (Z.point(se, ce), te);
    }
    ee.invert = function(oe) {
      var se = H.scale(), ce = H.translate(), le = (oe[0] - ce[0]) / se, ue = (oe[1] - ce[1]) / se;
      return (ue >= 0.12 && ue < 0.234 && le >= -0.425 && le < -0.214 ? q : ue >= 0.166 && ue < 0.234 && le >= -0.214 && le < -0.115 ? J : H).invert(oe);
    }, ee.stream = function(oe) {
      return O && _ === oe ? O : O = multiplex([H.stream(_ = oe), q.stream(oe), J.stream(oe)]);
    }, ee.precision = function(oe) {
      return arguments.length ? (H.precision(oe), q.precision(oe), J.precision(oe), ae()) : H.precision();
    }, ee.scale = function(oe) {
      return arguments.length ? (H.scale(oe), q.scale(oe * 0.35), J.scale(oe), ee.translate(H.translate())) : H.scale();
    }, ee.translate = function(oe) {
      if (!arguments.length) return H.translate();
      var se = H.scale(), ce = +oe[0], le = +oe[1];
      return V = H.translate(oe).clipExtent([[ce - 0.455 * se, le - 0.238 * se], [ce + 0.455 * se, le + 0.238 * se]]).stream(ne), G = q.translate([ce - 0.307 * se, le + 0.201 * se]).clipExtent([[ce - 0.425 * se + epsilon$3, le + 0.12 * se + epsilon$3], [ce - 0.214 * se - epsilon$3, le + 0.234 * se - epsilon$3]]).stream(ne), Z = J.translate([ce - 0.205 * se, le + 0.212 * se]).clipExtent([[ce - 0.214 * se + epsilon$3, le + 0.166 * se + epsilon$3], [ce - 0.115 * se - epsilon$3, le + 0.234 * se - epsilon$3]]).stream(ne), ae();
    }, ee.fitExtent = function(oe, se) {
      return fitExtent(ee, oe, se);
    }, ee.fitSize = function(oe, se) {
      return fitSize(ee, oe, se);
    }, ee.fitWidth = function(oe, se) {
      return fitWidth(ee, oe, se);
    }, ee.fitHeight = function(oe, se) {
      return fitHeight(ee, oe, se);
    };
    function ae() {
      return O = _ = null, ee;
    }
    return ee.scale(1070);
  }
  function azimuthalRaw(O) {
    return function(_, H) {
      var V = cos$1(_), q = cos$1(H), G = O(V * q);
      return G === 1 / 0 ? [2, 0] : [
        G * q * sin$1(_),
        G * sin$1(H)
      ];
    };
  }
  function azimuthalInvert(O) {
    return function(_, H) {
      var V = sqrt$1(_ * _ + H * H), q = O(V), G = sin$1(q), J = cos$1(q);
      return [
        atan2(_ * G, V * J),
        asin$1(V && H * G / V)
      ];
    };
  }
  var azimuthalEqualAreaRaw = azimuthalRaw(function(O) {
    return sqrt$1(2 / (1 + O));
  });
  azimuthalEqualAreaRaw.invert = azimuthalInvert(function(O) {
    return 2 * asin$1(O / 2);
  });
  function geoAzimuthalEqualArea() {
    return projection$1(azimuthalEqualAreaRaw).scale(124.75).clipAngle(180 - 1e-3);
  }
  var azimuthalEquidistantRaw = azimuthalRaw(function(O) {
    return (O = acos(O)) && O / sin$1(O);
  });
  azimuthalEquidistantRaw.invert = azimuthalInvert(function(O) {
    return O;
  });
  function geoAzimuthalEquidistant() {
    return projection$1(azimuthalEquidistantRaw).scale(79.4188).clipAngle(180 - 1e-3);
  }
  function mercatorRaw(O, _) {
    return [O, log$1(tan((halfPi$1 + _) / 2))];
  }
  mercatorRaw.invert = function(O, _) {
    return [O, 2 * atan(exp(_)) - halfPi$1];
  };
  function geoMercator() {
    return mercatorProjection(mercatorRaw).scale(961 / tau$1);
  }
  function mercatorProjection(O) {
    var _ = projection$1(O), H = _.center, V = _.scale, q = _.translate, G = _.clipExtent, J = null, Z, te, ne;
    _.scale = function(ae) {
      return arguments.length ? (V(ae), ee()) : V();
    }, _.translate = function(ae) {
      return arguments.length ? (q(ae), ee()) : q();
    }, _.center = function(ae) {
      return arguments.length ? (H(ae), ee()) : H();
    }, _.clipExtent = function(ae) {
      return arguments.length ? (ae == null ? J = Z = te = ne = null : (J = +ae[0][0], Z = +ae[0][1], te = +ae[1][0], ne = +ae[1][1]), ee()) : J == null ? null : [[J, Z], [te, ne]];
    };
    function ee() {
      var ae = pi$1 * V(), oe = _(rotation(_.rotate()).invert([0, 0]));
      return G(J == null ? [[oe[0] - ae, oe[1] - ae], [oe[0] + ae, oe[1] + ae]] : O === mercatorRaw ? [[Math.max(oe[0] - ae, J), Z], [Math.min(oe[0] + ae, te), ne]] : [[J, Math.max(oe[1] - ae, Z)], [te, Math.min(oe[1] + ae, ne)]]);
    }
    return ee();
  }
  function tany(O) {
    return tan((halfPi$1 + O) / 2);
  }
  function conicConformalRaw(O, _) {
    var H = cos$1(O), V = O === _ ? sin$1(O) : log$1(H / cos$1(_)) / log$1(tany(_) / tany(O)), q = H * pow$1(tany(O), V) / V;
    if (!V) return mercatorRaw;
    function G(J, Z) {
      q > 0 ? Z < -halfPi$1 + epsilon$3 && (Z = -halfPi$1 + epsilon$3) : Z > halfPi$1 - epsilon$3 && (Z = halfPi$1 - epsilon$3);
      var te = q / pow$1(tany(Z), V);
      return [te * sin$1(V * J), q - te * cos$1(V * J)];
    }
    return G.invert = function(J, Z) {
      var te = q - Z, ne = sign(V) * sqrt$1(J * J + te * te), ee = atan2(J, abs$1(te)) * sign(te);
      return te * V < 0 && (ee -= pi$1 * sign(J) * sign(te)), [ee / V, 2 * atan(pow$1(q / ne, 1 / V)) - halfPi$1];
    }, G;
  }
  function geoConicConformal() {
    return conicProjection(conicConformalRaw).scale(109.5).parallels([30, 30]);
  }
  function equirectangularRaw(O, _) {
    return [O, _];
  }
  equirectangularRaw.invert = equirectangularRaw;
  function geoEquirectangular() {
    return projection$1(equirectangularRaw).scale(152.63);
  }
  function conicEquidistantRaw(O, _) {
    var H = cos$1(O), V = O === _ ? sin$1(O) : (H - cos$1(_)) / (_ - O), q = H / V + O;
    if (abs$1(V) < epsilon$3) return equirectangularRaw;
    function G(J, Z) {
      var te = q - Z, ne = V * J;
      return [te * sin$1(ne), q - te * cos$1(ne)];
    }
    return G.invert = function(J, Z) {
      var te = q - Z, ne = atan2(J, abs$1(te)) * sign(te);
      return te * V < 0 && (ne -= pi$1 * sign(J) * sign(te)), [ne / V, q - sign(V) * sqrt$1(J * J + te * te)];
    }, G;
  }
  function geoConicEquidistant() {
    return conicProjection(conicEquidistantRaw).scale(131.154).center([0, 13.9389]);
  }
  var A1 = 1.340264, A2 = -0.081106, A3 = 893e-6, A4 = 3796e-6, M = sqrt$1(3) / 2, iterations = 12;
  function equalEarthRaw(O, _) {
    var H = asin$1(M * sin$1(_)), V = H * H, q = V * V * V;
    return [
      O * cos$1(H) / (M * (A1 + 3 * A2 * V + q * (7 * A3 + 9 * A4 * V))),
      H * (A1 + A2 * V + q * (A3 + A4 * V))
    ];
  }
  equalEarthRaw.invert = function(O, _) {
    for (var H = _, V = H * H, q = V * V * V, G = 0, J, Z, te; G < iterations && (Z = H * (A1 + A2 * V + q * (A3 + A4 * V)) - _, te = A1 + 3 * A2 * V + q * (7 * A3 + 9 * A4 * V), H -= J = Z / te, V = H * H, q = V * V * V, !(abs$1(J) < epsilon2)); ++G)
      ;
    return [
      M * O * (A1 + 3 * A2 * V + q * (7 * A3 + 9 * A4 * V)) / cos$1(H),
      asin$1(sin$1(H) / M)
    ];
  };
  function geoEqualEarth() {
    return projection$1(equalEarthRaw).scale(177.158);
  }
  function gnomonicRaw(O, _) {
    var H = cos$1(_), V = cos$1(O) * H;
    return [H * sin$1(O) / V, sin$1(_) / V];
  }
  gnomonicRaw.invert = azimuthalInvert(atan);
  function geoGnomonic() {
    return projection$1(gnomonicRaw).scale(144.049).clipAngle(60);
  }
  function geoIdentity() {
    var O = 1, _ = 0, H = 0, V = 1, q = 1, G = 0, J, Z, te = null, ne, ee, ae, oe = 1, se = 1, ce = transformer({
      point: function(me, pe) {
        var ye = he([me, pe]);
        this.stream.point(ye[0], ye[1]);
      }
    }), le = identity$1, ue, de;
    function fe() {
      return oe = O * V, se = O * q, ue = de = null, he;
    }
    function he(me) {
      var pe = me[0] * oe, ye = me[1] * se;
      if (G) {
        var ge = ye * J - pe * Z;
        pe = pe * J + ye * Z, ye = ge;
      }
      return [pe + _, ye + H];
    }
    return he.invert = function(me) {
      var pe = me[0] - _, ye = me[1] - H;
      if (G) {
        var ge = ye * J + pe * Z;
        pe = pe * J - ye * Z, ye = ge;
      }
      return [pe / oe, ye / se];
    }, he.stream = function(me) {
      return ue && de === me ? ue : ue = ce(le(de = me));
    }, he.postclip = function(me) {
      return arguments.length ? (le = me, te = ne = ee = ae = null, fe()) : le;
    }, he.clipExtent = function(me) {
      return arguments.length ? (le = me == null ? (te = ne = ee = ae = null, identity$1) : clipRectangle(te = +me[0][0], ne = +me[0][1], ee = +me[1][0], ae = +me[1][1]), fe()) : te == null ? null : [[te, ne], [ee, ae]];
    }, he.scale = function(me) {
      return arguments.length ? (O = +me, fe()) : O;
    }, he.translate = function(me) {
      return arguments.length ? (_ = +me[0], H = +me[1], fe()) : [_, H];
    }, he.angle = function(me) {
      return arguments.length ? (G = me % 360 * radians, Z = sin$1(G), J = cos$1(G), fe()) : G * degrees;
    }, he.reflectX = function(me) {
      return arguments.length ? (V = me ? -1 : 1, fe()) : V < 0;
    }, he.reflectY = function(me) {
      return arguments.length ? (q = me ? -1 : 1, fe()) : q < 0;
    }, he.fitExtent = function(me, pe) {
      return fitExtent(he, me, pe);
    }, he.fitSize = function(me, pe) {
      return fitSize(he, me, pe);
    }, he.fitWidth = function(me, pe) {
      return fitWidth(he, me, pe);
    }, he.fitHeight = function(me, pe) {
      return fitHeight(he, me, pe);
    }, he;
  }
  function naturalEarth1Raw(O, _) {
    var H = _ * _, V = H * H;
    return [
      O * (0.8707 - 0.131979 * H + V * (-0.013791 + V * (3971e-6 * H - 1529e-6 * V))),
      _ * (1.007226 + H * (0.015085 + V * (-0.044475 + 0.028874 * H - 5916e-6 * V)))
    ];
  }
  naturalEarth1Raw.invert = function(O, _) {
    var H = _, V = 25, q;
    do {
      var G = H * H, J = G * G;
      H -= q = (H * (1.007226 + G * (0.015085 + J * (-0.044475 + 0.028874 * G - 5916e-6 * J))) - _) / (1.007226 + G * (0.015085 * 3 + J * (-0.044475 * 7 + 0.028874 * 9 * G - 5916e-6 * 11 * J)));
    } while (abs$1(q) > epsilon$3 && --V > 0);
    return [
      O / (0.8707 + (G = H * H) * (-0.131979 + G * (-0.013791 + G * G * G * (3971e-6 - 1529e-6 * G)))),
      H
    ];
  };
  function geoNaturalEarth1() {
    return projection$1(naturalEarth1Raw).scale(175.295);
  }
  function orthographicRaw(O, _) {
    return [cos$1(_) * sin$1(O), sin$1(_)];
  }
  orthographicRaw.invert = azimuthalInvert(asin$1);
  function geoOrthographic() {
    return projection$1(orthographicRaw).scale(249.5).clipAngle(90 + epsilon$3);
  }
  function stereographicRaw(O, _) {
    var H = cos$1(_), V = 1 + cos$1(O) * H;
    return [H * sin$1(O) / V, sin$1(_) / V];
  }
  stereographicRaw.invert = azimuthalInvert(function(O) {
    return 2 * atan(O);
  });
  function geoStereographic() {
    return projection$1(stereographicRaw).scale(250).clipAngle(142);
  }
  function transverseMercatorRaw(O, _) {
    return [log$1(tan((halfPi$1 + _) / 2)), -O];
  }
  transverseMercatorRaw.invert = function(O, _) {
    return [-_, 2 * atan(exp(O)) - halfPi$1];
  };
  function geoTransverseMercator() {
    var O = mercatorProjection(transverseMercatorRaw), _ = O.center, H = O.rotate;
    return O.center = function(V) {
      return arguments.length ? _([-V[1], V[0]]) : (V = _(), [V[1], -V[0]]);
    }, O.rotate = function(V) {
      return arguments.length ? H([V[0], V[1], V.length > 2 ? V[2] + 90 : 90]) : (V = H(), [V[0], V[1], V[2] - 90]);
    }, H([0, 0, 90]).scale(159.155);
  }
  var abs = Math.abs, cos = Math.cos, sin = Math.sin, epsilon$2 = 1e-6, pi = Math.PI, halfPi = pi / 2, sqrt2 = sqrt(2);
  function asin(O) {
    return O > 1 ? halfPi : O < -1 ? -halfPi : Math.asin(O);
  }
  function sqrt(O) {
    return O > 0 ? Math.sqrt(O) : 0;
  }
  function mollweideBromleyTheta(O, _) {
    var H = O * sin(_), V = 30, q;
    do
      _ -= q = (_ + sin(_) - H) / (1 + cos(_));
    while (abs(q) > epsilon$2 && --V > 0);
    return _ / 2;
  }
  function mollweideBromleyRaw(O, _, H) {
    function V(q, G) {
      return [O * q * cos(G = mollweideBromleyTheta(H, G)), _ * sin(G)];
    }
    return V.invert = function(q, G) {
      return G = asin(G / _), [q / (O * cos(G)), asin((2 * G + sin(2 * G)) / H)];
    }, V;
  }
  var mollweideRaw = mollweideBromleyRaw(sqrt2 / halfPi, sqrt2, pi);
  function geoMollweide() {
    return projection$1(mollweideRaw).scale(169.529);
  }
  const defaultPath = geoPath(), projectionProperties = [
    // standard properties in d3-geo
    "clipAngle",
    "clipExtent",
    "scale",
    "translate",
    "center",
    "rotate",
    "parallels",
    "precision",
    "reflectX",
    "reflectY",
    // extended properties in d3-geo-projections
    "coefficient",
    "distance",
    "fraction",
    "lobes",
    "parallel",
    "radius",
    "ratio",
    "spacing",
    "tilt"
  ];
  function create$1(O, _) {
    return function H() {
      const V = _();
      return V.type = O, V.path = geoPath().projection(V), V.copy = V.copy || function() {
        const q = H();
        return projectionProperties.forEach((G) => {
          V[G] && q[G](V[G]());
        }), q.path.pointRadius(V.path.pointRadius()), q;
      }, registerScale(V);
    };
  }
  function projection(O, _) {
    if (!O || typeof O != "string")
      throw new Error("Projection type must be a name string.");
    return O = O.toLowerCase(), arguments.length > 1 ? (projections[O] = create$1(O, _), this) : projections[O] || null;
  }
  function getProjectionPath(O) {
    return O && O.path || defaultPath;
  }
  const projections = {
    // base d3-geo projection types
    albers: geoAlbers,
    albersusa: geoAlbersUsa,
    azimuthalequalarea: geoAzimuthalEqualArea,
    azimuthalequidistant: geoAzimuthalEquidistant,
    conicconformal: geoConicConformal,
    conicequalarea: geoConicEqualArea,
    conicequidistant: geoConicEquidistant,
    equalEarth: geoEqualEarth,
    equirectangular: geoEquirectangular,
    gnomonic: geoGnomonic,
    identity: geoIdentity,
    mercator: geoMercator,
    mollweide: geoMollweide,
    naturalEarth1: geoNaturalEarth1,
    orthographic: geoOrthographic,
    stereographic: geoStereographic,
    transversemercator: geoTransverseMercator
  };
  for (const O in projections)
    projection(O, projections[O]);
  function noop$1() {
  }
  const cases = [[], [[[1, 1.5], [0.5, 1]]], [[[1.5, 1], [1, 1.5]]], [[[1.5, 1], [0.5, 1]]], [[[1, 0.5], [1.5, 1]]], [[[1, 1.5], [0.5, 1]], [[1, 0.5], [1.5, 1]]], [[[1, 0.5], [1, 1.5]]], [[[1, 0.5], [0.5, 1]]], [[[0.5, 1], [1, 0.5]]], [[[1, 1.5], [1, 0.5]]], [[[0.5, 1], [1, 0.5]], [[1.5, 1], [1, 1.5]]], [[[1.5, 1], [1, 0.5]]], [[[0.5, 1], [1.5, 1]]], [[[1, 1.5], [1.5, 1]]], [[[0.5, 1], [1, 1.5]]], []];
  function contours() {
    var O = 1, _ = 1, H = Z;
    function V(te, ne) {
      return ne.map((ee) => q(te, ee));
    }
    function q(te, ne) {
      var ee = [], ae = [];
      return G(te, ne, (oe) => {
        H(oe, te, ne), area$1(oe) > 0 ? ee.push([oe]) : ae.push(oe);
      }), ae.forEach((oe) => {
        for (var se = 0, ce = ee.length, le; se < ce; ++se)
          if (contains$3((le = ee[se])[0], oe) !== -1) {
            le.push(oe);
            return;
          }
      }), {
        type: "MultiPolygon",
        value: ne,
        coordinates: ee
      };
    }
    function G(te, ne, ee) {
      var ae = [], oe = [], se, ce, le, ue, de, fe;
      for (se = ce = -1, ue = te[0] >= ne, cases[ue << 1].forEach(he); ++se < O - 1; )
        le = ue, ue = te[se + 1] >= ne, cases[le | ue << 1].forEach(he);
      for (cases[ue << 0].forEach(he); ++ce < _ - 1; ) {
        for (se = -1, ue = te[ce * O + O] >= ne, de = te[ce * O] >= ne, cases[ue << 1 | de << 2].forEach(he); ++se < O - 1; )
          le = ue, ue = te[ce * O + O + se + 1] >= ne, fe = de, de = te[ce * O + se + 1] >= ne, cases[le | ue << 1 | de << 2 | fe << 3].forEach(he);
        cases[ue | de << 3].forEach(he);
      }
      for (se = -1, de = te[ce * O] >= ne, cases[de << 2].forEach(he); ++se < O - 1; )
        fe = de, de = te[ce * O + se + 1] >= ne, cases[de << 2 | fe << 3].forEach(he);
      cases[de << 3].forEach(he);
      function he(me) {
        var pe = [me[0][0] + se, me[0][1] + ce], ye = [me[1][0] + se, me[1][1] + ce], ge = J(pe), ve = J(ye), be, Ce;
        (be = oe[ge]) ? (Ce = ae[ve]) ? (delete oe[be.end], delete ae[Ce.start], be === Ce ? (be.ring.push(ye), ee(be.ring)) : ae[be.start] = oe[Ce.end] = {
          start: be.start,
          end: Ce.end,
          ring: be.ring.concat(Ce.ring)
        }) : (delete oe[be.end], be.ring.push(ye), oe[be.end = ve] = be) : (be = ae[ve]) ? (Ce = oe[ge]) ? (delete ae[be.start], delete oe[Ce.end], be === Ce ? (be.ring.push(ye), ee(be.ring)) : ae[Ce.start] = oe[be.end] = {
          start: Ce.start,
          end: be.end,
          ring: Ce.ring.concat(be.ring)
        }) : (delete ae[be.start], be.ring.unshift(pe), ae[be.start = ge] = be) : ae[ge] = oe[ve] = {
          start: ge,
          end: ve,
          ring: [pe, ye]
        };
      }
    }
    function J(te) {
      return te[0] * 2 + te[1] * (O + 1) * 4;
    }
    function Z(te, ne, ee) {
      te.forEach((ae) => {
        var oe = ae[0], se = ae[1], ce = oe | 0, le = se | 0, ue, de = ne[le * O + ce];
        oe > 0 && oe < O && ce === oe && (ue = ne[le * O + ce - 1], ae[0] = oe + (ee - ue) / (de - ue) - 0.5), se > 0 && se < _ && le === se && (ue = ne[(le - 1) * O + ce], ae[1] = se + (ee - ue) / (de - ue) - 0.5);
      });
    }
    return V.contour = q, V.size = function(te) {
      if (!arguments.length) return [O, _];
      var ne = Math.floor(te[0]), ee = Math.floor(te[1]);
      return ne >= 0 && ee >= 0 || error$1("invalid size"), O = ne, _ = ee, V;
    }, V.smooth = function(te) {
      return arguments.length ? (H = te ? Z : noop$1, V) : H === Z;
    }, V;
  }
  function area$1(O) {
    for (var _ = 0, H = O.length, V = O[H - 1][1] * O[0][0] - O[H - 1][0] * O[0][1]; ++_ < H; ) V += O[_ - 1][1] * O[_][0] - O[_ - 1][0] * O[_][1];
    return V;
  }
  function contains$3(O, _) {
    for (var H = -1, V = _.length, q; ++H < V; ) if (q = ringContains(O, _[H])) return q;
    return 0;
  }
  function ringContains(O, _) {
    for (var H = _[0], V = _[1], q = -1, G = 0, J = O.length, Z = J - 1; G < J; Z = G++) {
      var te = O[G], ne = te[0], ee = te[1], ae = O[Z], oe = ae[0], se = ae[1];
      if (segmentContains(te, ae, _)) return 0;
      ee > V != se > V && H < (oe - ne) * (V - ee) / (se - ee) + ne && (q = -q);
    }
    return q;
  }
  function segmentContains(O, _, H) {
    var V;
    return collinear$1(O, _, H) && within(O[V = +(O[0] === _[0])], H[V], _[V]);
  }
  function collinear$1(O, _, H) {
    return (_[0] - O[0]) * (H[1] - O[1]) === (H[0] - O[0]) * (_[1] - O[1]);
  }
  function within(O, _, H) {
    return O <= _ && _ <= H || H <= _ && _ <= O;
  }
  function quantize(O, _, H) {
    return function(V) {
      var q = extent(V), G = H ? Math.min(q[0], 0) : q[0], J = q[1], Z = J - G, te = _ ? tickStep(G, J, O) : Z / (O + 1);
      return range$7(G + te, J, te);
    };
  }
  function Isocontour(O) {
    Transform.call(this, null, O);
  }
  Isocontour.Definition = {
    type: "Isocontour",
    metadata: {
      generates: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "thresholds",
      type: "number",
      array: !0
    }, {
      name: "levels",
      type: "number"
    }, {
      name: "nice",
      type: "boolean",
      default: !1
    }, {
      name: "resolve",
      type: "enum",
      values: ["shared", "independent"],
      default: "independent"
    }, {
      name: "zero",
      type: "boolean",
      default: !0
    }, {
      name: "smooth",
      type: "boolean",
      default: !0
    }, {
      name: "scale",
      type: "number",
      expr: !0
    }, {
      name: "translate",
      type: "number",
      array: !0,
      expr: !0
    }, {
      name: "as",
      type: "string",
      null: !0,
      default: "contour"
    }]
  };
  inherits(Isocontour, Transform, {
    transform(O, _) {
      if (this.value && !_.changed() && !O.modified())
        return _.StopPropagation;
      var H = _.fork(_.NO_SOURCE | _.NO_FIELDS), V = _.materialize(_.SOURCE).source, q = O.field || identity$7, G = contours().smooth(O.smooth !== !1), J = O.thresholds || levels(V, q, O), Z = O.as === null ? null : O.as || "contour", te = [];
      return V.forEach((ne) => {
        const ee = q(ne), ae = G.size([ee.width, ee.height])(ee.values, isArray$2(J) ? J : J(ee.values));
        transformPaths(ae, ee, ne, O), ae.forEach((oe) => {
          te.push(rederive(ne, ingest$1(Z != null ? {
            [Z]: oe
          } : oe)));
        });
      }), this.value && (H.rem = this.value), this.value = H.source = H.add = te, H;
    }
  });
  function levels(O, _, H) {
    const V = quantize(H.levels || 10, H.nice, H.zero !== !1);
    return H.resolve !== "shared" ? V : V(O.map((q) => max$2(_(q).values)));
  }
  function transformPaths(O, _, H, V) {
    let q = V.scale || _.scale, G = V.translate || _.translate;
    if (isFunction$2(q) && (q = q(H, V)), isFunction$2(G) && (G = G(H, V)), (q === 1 || q == null) && !G) return;
    const J = (isNumber$2(q) ? q : q[0]) || 1, Z = (isNumber$2(q) ? q : q[1]) || 1, te = G && G[0] || 0, ne = G && G[1] || 0;
    O.forEach(transform$1(_, J, Z, te, ne));
  }
  function transform$1(O, _, H, V, q) {
    const G = O.x1 || 0, J = O.y1 || 0, Z = _ * H < 0;
    function te(ae) {
      ae.forEach(ne);
    }
    function ne(ae) {
      Z && ae.reverse(), ae.forEach(ee);
    }
    function ee(ae) {
      ae[0] = (ae[0] - G) * _ + V, ae[1] = (ae[1] - J) * H + q;
    }
    return function(ae) {
      return ae.coordinates.forEach(te), ae;
    };
  }
  function radius(O, _, H) {
    const V = O >= 0 ? O : estimateBandwidth(_, H);
    return Math.round((Math.sqrt(4 * V * V + 1) - 1) / 2);
  }
  function number$3(O) {
    return isFunction$2(O) ? O : constant$5(+O);
  }
  function density2D() {
    var O = (te) => te[0], _ = (te) => te[1], H = one$2, V = [-1, -1], q = 960, G = 500, J = 2;
    function Z(te, ne) {
      const ee = radius(V[0], te, O) >> J, ae = radius(V[1], te, _) >> J, oe = ee ? ee + 2 : 0, se = ae ? ae + 2 : 0, ce = 2 * oe + (q >> J), le = 2 * se + (G >> J), ue = new Float32Array(ce * le), de = new Float32Array(ce * le);
      let fe = ue;
      te.forEach((me) => {
        const pe = oe + (+O(me) >> J), ye = se + (+_(me) >> J);
        pe >= 0 && pe < ce && ye >= 0 && ye < le && (ue[pe + ye * ce] += +H(me));
      }), ee > 0 && ae > 0 ? (blurX(ce, le, ue, de, ee), blurY(ce, le, de, ue, ae), blurX(ce, le, ue, de, ee), blurY(ce, le, de, ue, ae), blurX(ce, le, ue, de, ee), blurY(ce, le, de, ue, ae)) : ee > 0 ? (blurX(ce, le, ue, de, ee), blurX(ce, le, de, ue, ee), blurX(ce, le, ue, de, ee), fe = de) : ae > 0 && (blurY(ce, le, ue, de, ae), blurY(ce, le, de, ue, ae), blurY(ce, le, ue, de, ae), fe = de);
      const he = ne ? Math.pow(2, -2 * J) : 1 / sum$1(fe);
      for (let me = 0, pe = ce * le; me < pe; ++me) fe[me] *= he;
      return {
        values: fe,
        scale: 1 << J,
        width: ce,
        height: le,
        x1: oe,
        y1: se,
        x2: oe + (q >> J),
        y2: se + (G >> J)
      };
    }
    return Z.x = function(te) {
      return arguments.length ? (O = number$3(te), Z) : O;
    }, Z.y = function(te) {
      return arguments.length ? (_ = number$3(te), Z) : _;
    }, Z.weight = function(te) {
      return arguments.length ? (H = number$3(te), Z) : H;
    }, Z.size = function(te) {
      if (!arguments.length) return [q, G];
      var ne = +te[0], ee = +te[1];
      return ne >= 0 && ee >= 0 || error$1("invalid size"), q = ne, G = ee, Z;
    }, Z.cellSize = function(te) {
      return arguments.length ? ((te = +te) >= 1 || error$1("invalid cell size"), J = Math.floor(Math.log(te) / Math.LN2), Z) : 1 << J;
    }, Z.bandwidth = function(te) {
      return arguments.length ? (te = array$5(te), te.length === 1 && (te = [+te[0], +te[0]]), te.length !== 2 && error$1("invalid bandwidth"), V = te, Z) : V;
    }, Z;
  }
  function blurX(O, _, H, V, q) {
    const G = (q << 1) + 1;
    for (let J = 0; J < _; ++J)
      for (let Z = 0, te = 0; Z < O + q; ++Z)
        Z < O && (te += H[Z + J * O]), Z >= q && (Z >= G && (te -= H[Z - G + J * O]), V[Z - q + J * O] = te / Math.min(Z + 1, O - 1 + G - Z, G));
  }
  function blurY(O, _, H, V, q) {
    const G = (q << 1) + 1;
    for (let J = 0; J < O; ++J)
      for (let Z = 0, te = 0; Z < _ + q; ++Z)
        Z < _ && (te += H[J + Z * O]), Z >= q && (Z >= G && (te -= H[J + (Z - G) * O]), V[J + (Z - q) * O] = te / Math.min(Z + 1, _ - 1 + G - Z, G));
  }
  function KDE2D(O) {
    Transform.call(this, null, O);
  }
  KDE2D.Definition = {
    type: "KDE2D",
    metadata: {
      generates: !0
    },
    params: [{
      name: "size",
      type: "number",
      array: !0,
      length: 2,
      required: !0
    }, {
      name: "x",
      type: "field",
      required: !0
    }, {
      name: "y",
      type: "field",
      required: !0
    }, {
      name: "weight",
      type: "field"
    }, {
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "cellSize",
      type: "number"
    }, {
      name: "bandwidth",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "counts",
      type: "boolean",
      default: !1
    }, {
      name: "as",
      type: "string",
      default: "grid"
    }]
  };
  const PARAMS = ["x", "y", "weight", "size", "cellSize", "bandwidth"];
  function params(O, _) {
    return PARAMS.forEach((H) => _[H] != null ? O[H](_[H]) : 0), O;
  }
  inherits(KDE2D, Transform, {
    transform(O, _) {
      if (this.value && !_.changed() && !O.modified()) return _.StopPropagation;
      var H = _.fork(_.NO_SOURCE | _.NO_FIELDS), V = _.materialize(_.SOURCE).source, q = partition$2(V, O.groupby), G = (O.groupby || []).map(accessorName), J = params(density2D(), O), Z = O.as || "grid", te = [];
      function ne(ee, ae) {
        for (let oe = 0; oe < G.length; ++oe) ee[G[oe]] = ae[oe];
        return ee;
      }
      return te = q.map((ee) => ingest$1(ne({
        [Z]: J(ee, O.counts)
      }, ee.dims))), this.value && (H.rem = this.value), this.value = H.source = H.add = te, H;
    }
  });
  function partition$2(O, _) {
    var H = [], V = (ee) => ee(Z), q, G, J, Z, te, ne;
    if (_ == null)
      H.push(O);
    else
      for (q = {}, G = 0, J = O.length; G < J; ++G)
        Z = O[G], te = _.map(V), ne = q[te], ne || (q[te] = ne = [], ne.dims = te, H.push(ne)), ne.push(Z);
    return H;
  }
  function Contour(O) {
    Transform.call(this, null, O);
  }
  Contour.Definition = {
    type: "Contour",
    metadata: {
      generates: !0
    },
    params: [{
      name: "size",
      type: "number",
      array: !0,
      length: 2,
      required: !0
    }, {
      name: "values",
      type: "number",
      array: !0
    }, {
      name: "x",
      type: "field"
    }, {
      name: "y",
      type: "field"
    }, {
      name: "weight",
      type: "field"
    }, {
      name: "cellSize",
      type: "number"
    }, {
      name: "bandwidth",
      type: "number"
    }, {
      name: "count",
      type: "number"
    }, {
      name: "nice",
      type: "boolean",
      default: !1
    }, {
      name: "thresholds",
      type: "number",
      array: !0
    }, {
      name: "smooth",
      type: "boolean",
      default: !0
    }]
  };
  inherits(Contour, Transform, {
    transform(O, _) {
      if (this.value && !_.changed() && !O.modified())
        return _.StopPropagation;
      var H = _.fork(_.NO_SOURCE | _.NO_FIELDS), V = contours().smooth(O.smooth !== !1), q = O.values, G = O.thresholds || quantize(O.count || 10, O.nice, !!q), J = O.size, Z, te;
      return q || (q = _.materialize(_.SOURCE).source, Z = params(density2D(), O)(q, !0), te = transform$1(Z, Z.scale || 1, Z.scale || 1, 0, 0), J = [Z.width, Z.height], q = Z.values), G = isArray$2(G) ? G : G(q), q = V.size(J)(q, G), te && q.forEach(te), this.value && (H.rem = this.value), this.value = H.source = H.add = (q || []).map(ingest$1), H;
    }
  });
  const Feature = "Feature", FeatureCollection = "FeatureCollection", MultiPoint = "MultiPoint";
  function GeoJSON(O) {
    Transform.call(this, null, O);
  }
  GeoJSON.Definition = {
    type: "GeoJSON",
    metadata: {},
    params: [{
      name: "fields",
      type: "field",
      array: !0,
      length: 2
    }, {
      name: "geojson",
      type: "field"
    }]
  };
  inherits(GeoJSON, Transform, {
    transform(O, _) {
      var H = this._features, V = this._points, q = O.fields, G = q && q[0], J = q && q[1], Z = O.geojson || !q && identity$7, te = _.ADD, ne;
      ne = O.modified() || _.changed(_.REM) || _.modified(accessorFields(Z)) || G && _.modified(accessorFields(G)) || J && _.modified(accessorFields(J)), (!this.value || ne) && (te = _.SOURCE, this._features = H = [], this._points = V = []), Z && _.visit(te, (ee) => H.push(Z(ee))), G && J && (_.visit(te, (ee) => {
        var ae = G(ee), oe = J(ee);
        ae != null && oe != null && (ae = +ae) === ae && (oe = +oe) === oe && V.push([ae, oe]);
      }), H = H.concat({
        type: Feature,
        geometry: {
          type: MultiPoint,
          coordinates: V
        }
      })), this.value = {
        type: FeatureCollection,
        features: H
      };
    }
  });
  function GeoPath(O) {
    Transform.call(this, null, O);
  }
  GeoPath.Definition = {
    type: "GeoPath",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "projection",
      type: "projection"
    }, {
      name: "field",
      type: "field"
    }, {
      name: "pointRadius",
      type: "number",
      expr: !0
    }, {
      name: "as",
      type: "string",
      default: "path"
    }]
  };
  inherits(GeoPath, Transform, {
    transform(O, _) {
      var H = _.fork(_.ALL), V = this.value, q = O.field || identity$7, G = O.as || "path", J = H.SOURCE;
      !V || O.modified() ? (this.value = V = getProjectionPath(O.projection), H.materialize().reflow()) : J = q === identity$7 || _.modified(q.fields) ? H.ADD_MOD : H.ADD;
      const Z = initPath(V, O.pointRadius);
      return H.visit(J, (te) => te[G] = V(q(te))), V.pointRadius(Z), H.modifies(G);
    }
  });
  function initPath(O, _) {
    const H = O.pointRadius();
    return O.context(null), _ != null && O.pointRadius(_), H;
  }
  function GeoPoint(O) {
    Transform.call(this, null, O);
  }
  GeoPoint.Definition = {
    type: "GeoPoint",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "projection",
      type: "projection",
      required: !0
    }, {
      name: "fields",
      type: "field",
      array: !0,
      required: !0,
      length: 2
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 2,
      default: ["x", "y"]
    }]
  };
  inherits(GeoPoint, Transform, {
    transform(O, _) {
      var H = O.projection, V = O.fields[0], q = O.fields[1], G = O.as || ["x", "y"], J = G[0], Z = G[1], te;
      function ne(ee) {
        const ae = H([V(ee), q(ee)]);
        ae ? (ee[J] = ae[0], ee[Z] = ae[1]) : (ee[J] = void 0, ee[Z] = void 0);
      }
      return O.modified() ? _ = _.materialize().reflow(!0).visit(_.SOURCE, ne) : (te = _.modified(V.fields) || _.modified(q.fields), _.visit(te ? _.ADD_MOD : _.ADD, ne)), _.modifies(G);
    }
  });
  function GeoShape(O) {
    Transform.call(this, null, O);
  }
  GeoShape.Definition = {
    type: "GeoShape",
    metadata: {
      modifies: !0,
      nomod: !0
    },
    params: [{
      name: "projection",
      type: "projection"
    }, {
      name: "field",
      type: "field",
      default: "datum"
    }, {
      name: "pointRadius",
      type: "number",
      expr: !0
    }, {
      name: "as",
      type: "string",
      default: "shape"
    }]
  };
  inherits(GeoShape, Transform, {
    transform(O, _) {
      var H = _.fork(_.ALL), V = this.value, q = O.as || "shape", G = H.ADD;
      return (!V || O.modified()) && (this.value = V = shapeGenerator(getProjectionPath(O.projection), O.field || field$1("datum"), O.pointRadius), H.materialize().reflow(), G = H.SOURCE), H.visit(G, (J) => J[q] = V), H.modifies(q);
    }
  });
  function shapeGenerator(O, _, H) {
    const V = H == null ? (q) => O(_(q)) : (q) => {
      var G = O.pointRadius(), J = O.pointRadius(H)(_(q));
      return O.pointRadius(G), J;
    };
    return V.context = (q) => (O.context(q), V), V;
  }
  function Graticule(O) {
    Transform.call(this, [], O), this.generator = graticule();
  }
  Graticule.Definition = {
    type: "Graticule",
    metadata: {
      changes: !0,
      generates: !0
    },
    params: [{
      name: "extent",
      type: "array",
      array: !0,
      length: 2,
      content: {
        type: "number",
        array: !0,
        length: 2
      }
    }, {
      name: "extentMajor",
      type: "array",
      array: !0,
      length: 2,
      content: {
        type: "number",
        array: !0,
        length: 2
      }
    }, {
      name: "extentMinor",
      type: "array",
      array: !0,
      length: 2,
      content: {
        type: "number",
        array: !0,
        length: 2
      }
    }, {
      name: "step",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "stepMajor",
      type: "number",
      array: !0,
      length: 2,
      default: [90, 360]
    }, {
      name: "stepMinor",
      type: "number",
      array: !0,
      length: 2,
      default: [10, 10]
    }, {
      name: "precision",
      type: "number",
      default: 2.5
    }]
  };
  inherits(Graticule, Transform, {
    transform(O, _) {
      var H = this.value, V = this.generator, q;
      if (!H.length || O.modified())
        for (const G in O)
          isFunction$2(V[G]) && V[G](O[G]);
      return q = V(), H.length ? _.mod.push(replace$2(H[0], q)) : _.add.push(ingest$1(q)), H[0] = q, _;
    }
  });
  function Heatmap(O) {
    Transform.call(this, null, O);
  }
  Heatmap.Definition = {
    type: "heatmap",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "color",
      type: "string",
      expr: !0
    }, {
      name: "opacity",
      type: "number",
      expr: !0
    }, {
      name: "resolve",
      type: "enum",
      values: ["shared", "independent"],
      default: "independent"
    }, {
      name: "as",
      type: "string",
      default: "image"
    }]
  };
  inherits(Heatmap, Transform, {
    transform(O, _) {
      if (!_.changed() && !O.modified())
        return _.StopPropagation;
      var H = _.materialize(_.SOURCE).source, V = O.resolve === "shared", q = O.field || identity$7, G = opacity_(O.opacity, O), J = color_(O.color, O), Z = O.as || "image", te = {
        $x: 0,
        $y: 0,
        $value: 0,
        $max: V ? max$2(H.map((ne) => max$2(q(ne).values))) : 0
      };
      return H.forEach((ne) => {
        const ee = q(ne), ae = extend$2({}, ne, te);
        V || (ae.$max = max$2(ee.values || [])), ne[Z] = toCanvas(ee, ae, J.dep ? J : constant$5(J(ae)), G.dep ? G : constant$5(G(ae)));
      }), _.reflow(!0).modifies(Z);
    }
  });
  function color_(O, _) {
    let H;
    return isFunction$2(O) ? (H = (V) => rgb$1(O(V, _)), H.dep = dependency(O)) : H = constant$5(rgb$1(O || "#888")), H;
  }
  function opacity_(O, _) {
    let H;
    return isFunction$2(O) ? (H = (V) => O(V, _), H.dep = dependency(O)) : O ? H = constant$5(O) : (H = (V) => V.$value / V.$max || 0, H.dep = !0), H;
  }
  function dependency(O) {
    if (!isFunction$2(O)) return !1;
    const _ = toSet$1(accessorFields(O));
    return _.$x || _.$y || _.$value || _.$max;
  }
  function toCanvas(O, _, H, V) {
    const q = O.width, G = O.height, J = O.x1 || 0, Z = O.y1 || 0, te = O.x2 || q, ne = O.y2 || G, ee = O.values, ae = ee ? (ue) => ee[ue] : zero$4, oe = domCanvas(te - J, ne - Z), se = oe.getContext("2d"), ce = se.getImageData(0, 0, te - J, ne - Z), le = ce.data;
    for (let ue = Z, de = 0; ue < ne; ++ue) {
      _.$y = ue - Z;
      for (let fe = J, he = ue * q; fe < te; ++fe, de += 4) {
        _.$x = fe - J, _.$value = ae(fe + he);
        const me = H(_);
        le[de + 0] = me.r, le[de + 1] = me.g, le[de + 2] = me.b, le[de + 3] = ~~(255 * V(_));
      }
    }
    return se.putImageData(ce, 0, 0), oe;
  }
  function Projection$1(O) {
    Transform.call(this, null, O), this.modified(!0);
  }
  inherits(Projection$1, Transform, {
    transform(O, _) {
      let H = this.value;
      return !H || O.modified("type") ? (this.value = H = create(O.type), projectionProperties.forEach((V) => {
        O[V] != null && set$3(H, V, O[V]);
      })) : projectionProperties.forEach((V) => {
        O.modified(V) && set$3(H, V, O[V]);
      }), O.pointRadius != null && H.path.pointRadius(O.pointRadius), O.fit && fit(H, O), _.fork(_.NO_SOURCE | _.NO_FIELDS);
    }
  });
  function fit(O, _) {
    const H = collectGeoJSON(_.fit);
    _.extent ? O.fitExtent(_.extent, H) : _.size && O.fitSize(_.size, H);
  }
  function create(O) {
    const _ = projection((O || "mercator").toLowerCase());
    return _ || error$1("Unrecognized projection type: " + O), _();
  }
  function set$3(O, _, H) {
    isFunction$2(O[_]) && O[_](H);
  }
  function collectGeoJSON(O) {
    return O = array$5(O), O.length === 1 ? O[0] : {
      type: FeatureCollection,
      features: O.reduce((_, H) => _.concat(featurize(H)), [])
    };
  }
  function featurize(O) {
    return O.type === FeatureCollection ? O.features : array$5(O).filter((_) => _ != null).map((_) => _.type === Feature ? _ : {
      type: Feature,
      geometry: _
    });
  }
  const geo = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    contour: Contour,
    geojson: GeoJSON,
    geopath: GeoPath,
    geopoint: GeoPoint,
    geoshape: GeoShape,
    graticule: Graticule,
    heatmap: Heatmap,
    isocontour: Isocontour,
    kde2d: KDE2D,
    projection: Projection$1
  }, Symbol.toStringTag, { value: "Module" }));
  function forceCenter(O, _) {
    var H, V = 1;
    O == null && (O = 0), _ == null && (_ = 0);
    function q() {
      var G, J = H.length, Z, te = 0, ne = 0;
      for (G = 0; G < J; ++G)
        Z = H[G], te += Z.x, ne += Z.y;
      for (te = (te / J - O) * V, ne = (ne / J - _) * V, G = 0; G < J; ++G)
        Z = H[G], Z.x -= te, Z.y -= ne;
    }
    return q.initialize = function(G) {
      H = G;
    }, q.x = function(G) {
      return arguments.length ? (O = +G, q) : O;
    }, q.y = function(G) {
      return arguments.length ? (_ = +G, q) : _;
    }, q.strength = function(G) {
      return arguments.length ? (V = +G, q) : V;
    }, q;
  }
  function tree_add(O) {
    const _ = +this._x.call(null, O), H = +this._y.call(null, O);
    return add$2(this.cover(_, H), _, H, O);
  }
  function add$2(O, _, H, V) {
    if (isNaN(_) || isNaN(H)) return O;
    var q, G = O._root, J = { data: V }, Z = O._x0, te = O._y0, ne = O._x1, ee = O._y1, ae, oe, se, ce, le, ue, de, fe;
    if (!G) return O._root = J, O;
    for (; G.length; )
      if ((le = _ >= (ae = (Z + ne) / 2)) ? Z = ae : ne = ae, (ue = H >= (oe = (te + ee) / 2)) ? te = oe : ee = oe, q = G, !(G = G[de = ue << 1 | le])) return q[de] = J, O;
    if (se = +O._x.call(null, G.data), ce = +O._y.call(null, G.data), _ === se && H === ce) return J.next = G, q ? q[de] = J : O._root = J, O;
    do
      q = q ? q[de] = new Array(4) : O._root = new Array(4), (le = _ >= (ae = (Z + ne) / 2)) ? Z = ae : ne = ae, (ue = H >= (oe = (te + ee) / 2)) ? te = oe : ee = oe;
    while ((de = ue << 1 | le) === (fe = (ce >= oe) << 1 | se >= ae));
    return q[fe] = G, q[de] = J, O;
  }
  function addAll(O) {
    var _, H, V = O.length, q, G, J = new Array(V), Z = new Array(V), te = 1 / 0, ne = 1 / 0, ee = -1 / 0, ae = -1 / 0;
    for (H = 0; H < V; ++H)
      isNaN(q = +this._x.call(null, _ = O[H])) || isNaN(G = +this._y.call(null, _)) || (J[H] = q, Z[H] = G, q < te && (te = q), q > ee && (ee = q), G < ne && (ne = G), G > ae && (ae = G));
    if (te > ee || ne > ae) return this;
    for (this.cover(te, ne).cover(ee, ae), H = 0; H < V; ++H)
      add$2(this, J[H], Z[H], O[H]);
    return this;
  }
  function tree_cover(O, _) {
    if (isNaN(O = +O) || isNaN(_ = +_)) return this;
    var H = this._x0, V = this._y0, q = this._x1, G = this._y1;
    if (isNaN(H))
      q = (H = Math.floor(O)) + 1, G = (V = Math.floor(_)) + 1;
    else {
      for (var J = q - H || 1, Z = this._root, te, ne; H > O || O >= q || V > _ || _ >= G; )
        switch (ne = (_ < V) << 1 | O < H, te = new Array(4), te[ne] = Z, Z = te, J *= 2, ne) {
          case 0:
            q = H + J, G = V + J;
            break;
          case 1:
            H = q - J, G = V + J;
            break;
          case 2:
            q = H + J, V = G - J;
            break;
          case 3:
            H = q - J, V = G - J;
            break;
        }
      this._root && this._root.length && (this._root = Z);
    }
    return this._x0 = H, this._y0 = V, this._x1 = q, this._y1 = G, this;
  }
  function tree_data() {
    var O = [];
    return this.visit(function(_) {
      if (!_.length) do
        O.push(_.data);
      while (_ = _.next);
    }), O;
  }
  function tree_extent(O) {
    return arguments.length ? this.cover(+O[0][0], +O[0][1]).cover(+O[1][0], +O[1][1]) : isNaN(this._x0) ? void 0 : [[this._x0, this._y0], [this._x1, this._y1]];
  }
  function Quad(O, _, H, V, q) {
    this.node = O, this.x0 = _, this.y0 = H, this.x1 = V, this.y1 = q;
  }
  function tree_find(O, _, H) {
    var V, q = this._x0, G = this._y0, J, Z, te, ne, ee = this._x1, ae = this._y1, oe = [], se = this._root, ce, le;
    for (se && oe.push(new Quad(se, q, G, ee, ae)), H == null ? H = 1 / 0 : (q = O - H, G = _ - H, ee = O + H, ae = _ + H, H *= H); ce = oe.pop(); )
      if (!(!(se = ce.node) || (J = ce.x0) > ee || (Z = ce.y0) > ae || (te = ce.x1) < q || (ne = ce.y1) < G))
        if (se.length) {
          var ue = (J + te) / 2, de = (Z + ne) / 2;
          oe.push(
            new Quad(se[3], ue, de, te, ne),
            new Quad(se[2], J, de, ue, ne),
            new Quad(se[1], ue, Z, te, de),
            new Quad(se[0], J, Z, ue, de)
          ), (le = (_ >= de) << 1 | O >= ue) && (ce = oe[oe.length - 1], oe[oe.length - 1] = oe[oe.length - 1 - le], oe[oe.length - 1 - le] = ce);
        } else {
          var fe = O - +this._x.call(null, se.data), he = _ - +this._y.call(null, se.data), me = fe * fe + he * he;
          if (me < H) {
            var pe = Math.sqrt(H = me);
            q = O - pe, G = _ - pe, ee = O + pe, ae = _ + pe, V = se.data;
          }
        }
    return V;
  }
  function tree_remove(O) {
    if (isNaN(ee = +this._x.call(null, O)) || isNaN(ae = +this._y.call(null, O))) return this;
    var _, H = this._root, V, q, G, J = this._x0, Z = this._y0, te = this._x1, ne = this._y1, ee, ae, oe, se, ce, le, ue, de;
    if (!H) return this;
    if (H.length) for (; ; ) {
      if ((ce = ee >= (oe = (J + te) / 2)) ? J = oe : te = oe, (le = ae >= (se = (Z + ne) / 2)) ? Z = se : ne = se, _ = H, !(H = H[ue = le << 1 | ce])) return this;
      if (!H.length) break;
      (_[ue + 1 & 3] || _[ue + 2 & 3] || _[ue + 3 & 3]) && (V = _, de = ue);
    }
    for (; H.data !== O; ) if (q = H, !(H = H.next)) return this;
    return (G = H.next) && delete H.next, q ? (G ? q.next = G : delete q.next, this) : _ ? (G ? _[ue] = G : delete _[ue], (H = _[0] || _[1] || _[2] || _[3]) && H === (_[3] || _[2] || _[1] || _[0]) && !H.length && (V ? V[de] = H : this._root = H), this) : (this._root = G, this);
  }
  function removeAll(O) {
    for (var _ = 0, H = O.length; _ < H; ++_) this.remove(O[_]);
    return this;
  }
  function tree_root() {
    return this._root;
  }
  function tree_size() {
    var O = 0;
    return this.visit(function(_) {
      if (!_.length) do
        ++O;
      while (_ = _.next);
    }), O;
  }
  function tree_visit(O) {
    var _ = [], H, V = this._root, q, G, J, Z, te;
    for (V && _.push(new Quad(V, this._x0, this._y0, this._x1, this._y1)); H = _.pop(); )
      if (!O(V = H.node, G = H.x0, J = H.y0, Z = H.x1, te = H.y1) && V.length) {
        var ne = (G + Z) / 2, ee = (J + te) / 2;
        (q = V[3]) && _.push(new Quad(q, ne, ee, Z, te)), (q = V[2]) && _.push(new Quad(q, G, ee, ne, te)), (q = V[1]) && _.push(new Quad(q, ne, J, Z, ee)), (q = V[0]) && _.push(new Quad(q, G, J, ne, ee));
      }
    return this;
  }
  function tree_visitAfter(O) {
    var _ = [], H = [], V;
    for (this._root && _.push(new Quad(this._root, this._x0, this._y0, this._x1, this._y1)); V = _.pop(); ) {
      var q = V.node;
      if (q.length) {
        var G, J = V.x0, Z = V.y0, te = V.x1, ne = V.y1, ee = (J + te) / 2, ae = (Z + ne) / 2;
        (G = q[0]) && _.push(new Quad(G, J, Z, ee, ae)), (G = q[1]) && _.push(new Quad(G, ee, Z, te, ae)), (G = q[2]) && _.push(new Quad(G, J, ae, ee, ne)), (G = q[3]) && _.push(new Quad(G, ee, ae, te, ne));
      }
      H.push(V);
    }
    for (; V = H.pop(); )
      O(V.node, V.x0, V.y0, V.x1, V.y1);
    return this;
  }
  function defaultX(O) {
    return O[0];
  }
  function tree_x(O) {
    return arguments.length ? (this._x = O, this) : this._x;
  }
  function defaultY(O) {
    return O[1];
  }
  function tree_y(O) {
    return arguments.length ? (this._y = O, this) : this._y;
  }
  function quadtree(O, _, H) {
    var V = new Quadtree(_ ?? defaultX, H ?? defaultY, NaN, NaN, NaN, NaN);
    return O == null ? V : V.addAll(O);
  }
  function Quadtree(O, _, H, V, q, G) {
    this._x = O, this._y = _, this._x0 = H, this._y0 = V, this._x1 = q, this._y1 = G, this._root = void 0;
  }
  function leaf_copy(O) {
    for (var _ = { data: O.data }, H = _; O = O.next; ) H = H.next = { data: O.data };
    return _;
  }
  var treeProto = quadtree.prototype = Quadtree.prototype;
  treeProto.copy = function() {
    var O = new Quadtree(this._x, this._y, this._x0, this._y0, this._x1, this._y1), _ = this._root, H, V;
    if (!_) return O;
    if (!_.length) return O._root = leaf_copy(_), O;
    for (H = [{ source: _, target: O._root = new Array(4) }]; _ = H.pop(); )
      for (var q = 0; q < 4; ++q)
        (V = _.source[q]) && (V.length ? H.push({ source: V, target: _.target[q] = new Array(4) }) : _.target[q] = leaf_copy(V));
    return O;
  };
  treeProto.add = tree_add;
  treeProto.addAll = addAll;
  treeProto.cover = tree_cover;
  treeProto.data = tree_data;
  treeProto.extent = tree_extent;
  treeProto.find = tree_find;
  treeProto.remove = tree_remove;
  treeProto.removeAll = removeAll;
  treeProto.root = tree_root;
  treeProto.size = tree_size;
  treeProto.visit = tree_visit;
  treeProto.visitAfter = tree_visitAfter;
  treeProto.x = tree_x;
  treeProto.y = tree_y;
  function constant$1(O) {
    return function() {
      return O;
    };
  }
  function jiggle(O) {
    return (O() - 0.5) * 1e-6;
  }
  function x$2(O) {
    return O.x + O.vx;
  }
  function y$1(O) {
    return O.y + O.vy;
  }
  function forceCollide(O) {
    var _, H, V, q = 1, G = 1;
    typeof O != "function" && (O = constant$1(O == null ? 1 : +O));
    function J() {
      for (var ne, ee = _.length, ae, oe, se, ce, le, ue, de = 0; de < G; ++de)
        for (ae = quadtree(_, x$2, y$1).visitAfter(Z), ne = 0; ne < ee; ++ne)
          oe = _[ne], le = H[oe.index], ue = le * le, se = oe.x + oe.vx, ce = oe.y + oe.vy, ae.visit(fe);
      function fe(he, me, pe, ye, ge) {
        var ve = he.data, be = he.r, Ce = le + be;
        if (ve) {
          if (ve.index > oe.index) {
            var Se = se - ve.x - ve.vx, we = ce - ve.y - ve.vy, Ee = Se * Se + we * we;
            Ee < Ce * Ce && (Se === 0 && (Se = jiggle(V), Ee += Se * Se), we === 0 && (we = jiggle(V), Ee += we * we), Ee = (Ce - (Ee = Math.sqrt(Ee))) / Ee * q, oe.vx += (Se *= Ee) * (Ce = (be *= be) / (ue + be)), oe.vy += (we *= Ee) * Ce, ve.vx -= Se * (Ce = 1 - Ce), ve.vy -= we * Ce);
          }
          return;
        }
        return me > se + Ce || ye < se - Ce || pe > ce + Ce || ge < ce - Ce;
      }
    }
    function Z(ne) {
      if (ne.data) return ne.r = H[ne.data.index];
      for (var ee = ne.r = 0; ee < 4; ++ee)
        ne[ee] && ne[ee].r > ne.r && (ne.r = ne[ee].r);
    }
    function te() {
      if (_) {
        var ne, ee = _.length, ae;
        for (H = new Array(ee), ne = 0; ne < ee; ++ne) ae = _[ne], H[ae.index] = +O(ae, ne, _);
      }
    }
    return J.initialize = function(ne, ee) {
      _ = ne, V = ee, te();
    }, J.iterations = function(ne) {
      return arguments.length ? (G = +ne, J) : G;
    }, J.strength = function(ne) {
      return arguments.length ? (q = +ne, J) : q;
    }, J.radius = function(ne) {
      return arguments.length ? (O = typeof ne == "function" ? ne : constant$1(+ne), te(), J) : O;
    }, J;
  }
  function index$1(O) {
    return O.index;
  }
  function find$2(O, _) {
    var H = O.get(_);
    if (!H) throw new Error("node not found: " + _);
    return H;
  }
  function forceLink(O) {
    var _ = index$1, H = ae, V, q = constant$1(30), G, J, Z, te, ne, ee = 1;
    O == null && (O = []);
    function ae(ue) {
      return 1 / Math.min(Z[ue.source.index], Z[ue.target.index]);
    }
    function oe(ue) {
      for (var de = 0, fe = O.length; de < ee; ++de)
        for (var he = 0, me, pe, ye, ge, ve, be, Ce; he < fe; ++he)
          me = O[he], pe = me.source, ye = me.target, ge = ye.x + ye.vx - pe.x - pe.vx || jiggle(ne), ve = ye.y + ye.vy - pe.y - pe.vy || jiggle(ne), be = Math.sqrt(ge * ge + ve * ve), be = (be - G[he]) / be * ue * V[he], ge *= be, ve *= be, ye.vx -= ge * (Ce = te[he]), ye.vy -= ve * Ce, pe.vx += ge * (Ce = 1 - Ce), pe.vy += ve * Ce;
    }
    function se() {
      if (J) {
        var ue, de = J.length, fe = O.length, he = new Map(J.map((pe, ye) => [_(pe, ye, J), pe])), me;
        for (ue = 0, Z = new Array(de); ue < fe; ++ue)
          me = O[ue], me.index = ue, typeof me.source != "object" && (me.source = find$2(he, me.source)), typeof me.target != "object" && (me.target = find$2(he, me.target)), Z[me.source.index] = (Z[me.source.index] || 0) + 1, Z[me.target.index] = (Z[me.target.index] || 0) + 1;
        for (ue = 0, te = new Array(fe); ue < fe; ++ue)
          me = O[ue], te[ue] = Z[me.source.index] / (Z[me.source.index] + Z[me.target.index]);
        V = new Array(fe), ce(), G = new Array(fe), le();
      }
    }
    function ce() {
      if (J)
        for (var ue = 0, de = O.length; ue < de; ++ue)
          V[ue] = +H(O[ue], ue, O);
    }
    function le() {
      if (J)
        for (var ue = 0, de = O.length; ue < de; ++ue)
          G[ue] = +q(O[ue], ue, O);
    }
    return oe.initialize = function(ue, de) {
      J = ue, ne = de, se();
    }, oe.links = function(ue) {
      return arguments.length ? (O = ue, se(), oe) : O;
    }, oe.id = function(ue) {
      return arguments.length ? (_ = ue, oe) : _;
    }, oe.iterations = function(ue) {
      return arguments.length ? (ee = +ue, oe) : ee;
    }, oe.strength = function(ue) {
      return arguments.length ? (H = typeof ue == "function" ? ue : constant$1(+ue), ce(), oe) : H;
    }, oe.distance = function(ue) {
      return arguments.length ? (q = typeof ue == "function" ? ue : constant$1(+ue), le(), oe) : q;
    }, oe;
  }
  var noop = { value: () => {
  } };
  function dispatch$1() {
    for (var O = 0, _ = arguments.length, H = {}, V; O < _; ++O) {
      if (!(V = arguments[O] + "") || V in H || /[\s.]/.test(V)) throw new Error("illegal type: " + V);
      H[V] = [];
    }
    return new Dispatch(H);
  }
  function Dispatch(O) {
    this._ = O;
  }
  function parseTypenames(O, _) {
    return O.trim().split(/^|\s+/).map(function(H) {
      var V = "", q = H.indexOf(".");
      if (q >= 0 && (V = H.slice(q + 1), H = H.slice(0, q)), H && !_.hasOwnProperty(H)) throw new Error("unknown type: " + H);
      return { type: H, name: V };
    });
  }
  Dispatch.prototype = dispatch$1.prototype = {
    constructor: Dispatch,
    on: function(O, _) {
      var H = this._, V = parseTypenames(O + "", H), q, G = -1, J = V.length;
      if (arguments.length < 2) {
        for (; ++G < J; ) if ((q = (O = V[G]).type) && (q = get$1(H[q], O.name))) return q;
        return;
      }
      if (_ != null && typeof _ != "function") throw new Error("invalid callback: " + _);
      for (; ++G < J; )
        if (q = (O = V[G]).type) H[q] = set$2(H[q], O.name, _);
        else if (_ == null) for (q in H) H[q] = set$2(H[q], O.name, null);
      return this;
    },
    copy: function() {
      var O = {}, _ = this._;
      for (var H in _) O[H] = _[H].slice();
      return new Dispatch(O);
    },
    call: function(O, _) {
      if ((q = arguments.length - 2) > 0) for (var H = new Array(q), V = 0, q, G; V < q; ++V) H[V] = arguments[V + 2];
      if (!this._.hasOwnProperty(O)) throw new Error("unknown type: " + O);
      for (G = this._[O], V = 0, q = G.length; V < q; ++V) G[V].value.apply(_, H);
    },
    apply: function(O, _, H) {
      if (!this._.hasOwnProperty(O)) throw new Error("unknown type: " + O);
      for (var V = this._[O], q = 0, G = V.length; q < G; ++q) V[q].value.apply(_, H);
    }
  };
  function get$1(O, _) {
    for (var H = 0, V = O.length, q; H < V; ++H)
      if ((q = O[H]).name === _)
        return q.value;
  }
  function set$2(O, _, H) {
    for (var V = 0, q = O.length; V < q; ++V)
      if (O[V].name === _) {
        O[V] = noop, O = O.slice(0, V).concat(O.slice(V + 1));
        break;
      }
    return H != null && O.push({ name: _, value: H }), O;
  }
  var frame = 0, timeout$1 = 0, interval$2 = 0, pokeDelay = 1e3, taskHead, taskTail, clockLast = 0, clockNow = 0, clockSkew = 0, clock = typeof performance == "object" && performance.now ? performance : Date, setFrame = typeof window == "object" && window.requestAnimationFrame ? window.requestAnimationFrame.bind(window) : function(O) {
    setTimeout(O, 17);
  };
  function now$2() {
    return clockNow || (setFrame(clearNow), clockNow = clock.now() + clockSkew);
  }
  function clearNow() {
    clockNow = 0;
  }
  function Timer$1() {
    this._call = this._time = this._next = null;
  }
  Timer$1.prototype = timer$1.prototype = {
    constructor: Timer$1,
    restart: function(O, _, H) {
      if (typeof O != "function") throw new TypeError("callback is not a function");
      H = (H == null ? now$2() : +H) + (_ == null ? 0 : +_), !this._next && taskTail !== this && (taskTail ? taskTail._next = this : taskHead = this, taskTail = this), this._call = O, this._time = H, sleep();
    },
    stop: function() {
      this._call && (this._call = null, this._time = 1 / 0, sleep());
    }
  };
  function timer$1(O, _, H) {
    var V = new Timer$1();
    return V.restart(O, _, H), V;
  }
  function timerFlush() {
    now$2(), ++frame;
    for (var O = taskHead, _; O; )
      (_ = clockNow - O._time) >= 0 && O._call.call(void 0, _), O = O._next;
    --frame;
  }
  function wake() {
    clockNow = (clockLast = clock.now()) + clockSkew, frame = timeout$1 = 0;
    try {
      timerFlush();
    } finally {
      frame = 0, nap(), clockNow = 0;
    }
  }
  function poke() {
    var O = clock.now(), _ = O - clockLast;
    _ > pokeDelay && (clockSkew -= _, clockLast = O);
  }
  function nap() {
    for (var O, _ = taskHead, H, V = 1 / 0; _; )
      _._call ? (V > _._time && (V = _._time), O = _, _ = _._next) : (H = _._next, _._next = null, _ = O ? O._next = H : taskHead = H);
    taskTail = O, sleep(V);
  }
  function sleep(O) {
    if (!frame) {
      timeout$1 && (timeout$1 = clearTimeout(timeout$1));
      var _ = O - clockNow;
      _ > 24 ? (O < 1 / 0 && (timeout$1 = setTimeout(wake, O - clock.now() - clockSkew)), interval$2 && (interval$2 = clearInterval(interval$2))) : (interval$2 || (clockLast = clock.now(), interval$2 = setInterval(poke, pokeDelay)), frame = 1, setFrame(wake));
    }
  }
  function interval$1(O, _, H) {
    var V = new Timer$1(), q = _;
    return _ == null ? (V.restart(O, _, H), V) : (V._restart = V.restart, V.restart = function(G, J, Z) {
      J = +J, Z = Z == null ? now$2() : +Z, V._restart(function te(ne) {
        ne += q, V._restart(te, q += J, Z), G(ne);
      }, J, Z);
    }, V.restart(O, _, H), V);
  }
  const a$1 = 1664525, c$1 = 1013904223, m$2 = 4294967296;
  function lcg$1() {
    let O = 1;
    return () => (O = (a$1 * O + c$1) % m$2) / m$2;
  }
  function x$1(O) {
    return O.x;
  }
  function y(O) {
    return O.y;
  }
  var initialRadius = 10, initialAngle = Math.PI * (3 - Math.sqrt(5));
  function forceSimulation(O) {
    var _, H = 1, V = 1e-3, q = 1 - Math.pow(V, 1 / 300), G = 0, J = 0.6, Z = /* @__PURE__ */ new Map(), te = timer$1(ae), ne = dispatch$1("tick", "end"), ee = lcg$1();
    O == null && (O = []);
    function ae() {
      oe(), ne.call("tick", _), H < V && (te.stop(), ne.call("end", _));
    }
    function oe(le) {
      var ue, de = O.length, fe;
      le === void 0 && (le = 1);
      for (var he = 0; he < le; ++he)
        for (H += (G - H) * q, Z.forEach(function(me) {
          me(H);
        }), ue = 0; ue < de; ++ue)
          fe = O[ue], fe.fx == null ? fe.x += fe.vx *= J : (fe.x = fe.fx, fe.vx = 0), fe.fy == null ? fe.y += fe.vy *= J : (fe.y = fe.fy, fe.vy = 0);
      return _;
    }
    function se() {
      for (var le = 0, ue = O.length, de; le < ue; ++le) {
        if (de = O[le], de.index = le, de.fx != null && (de.x = de.fx), de.fy != null && (de.y = de.fy), isNaN(de.x) || isNaN(de.y)) {
          var fe = initialRadius * Math.sqrt(0.5 + le), he = le * initialAngle;
          de.x = fe * Math.cos(he), de.y = fe * Math.sin(he);
        }
        (isNaN(de.vx) || isNaN(de.vy)) && (de.vx = de.vy = 0);
      }
    }
    function ce(le) {
      return le.initialize && le.initialize(O, ee), le;
    }
    return se(), _ = {
      tick: oe,
      restart: function() {
        return te.restart(ae), _;
      },
      stop: function() {
        return te.stop(), _;
      },
      nodes: function(le) {
        return arguments.length ? (O = le, se(), Z.forEach(ce), _) : O;
      },
      alpha: function(le) {
        return arguments.length ? (H = +le, _) : H;
      },
      alphaMin: function(le) {
        return arguments.length ? (V = +le, _) : V;
      },
      alphaDecay: function(le) {
        return arguments.length ? (q = +le, _) : +q;
      },
      alphaTarget: function(le) {
        return arguments.length ? (G = +le, _) : G;
      },
      velocityDecay: function(le) {
        return arguments.length ? (J = 1 - le, _) : 1 - J;
      },
      randomSource: function(le) {
        return arguments.length ? (ee = le, Z.forEach(ce), _) : ee;
      },
      force: function(le, ue) {
        return arguments.length > 1 ? (ue == null ? Z.delete(le) : Z.set(le, ce(ue)), _) : Z.get(le);
      },
      find: function(le, ue, de) {
        var fe = 0, he = O.length, me, pe, ye, ge, ve;
        for (de == null ? de = 1 / 0 : de *= de, fe = 0; fe < he; ++fe)
          ge = O[fe], me = le - ge.x, pe = ue - ge.y, ye = me * me + pe * pe, ye < de && (ve = ge, de = ye);
        return ve;
      },
      on: function(le, ue) {
        return arguments.length > 1 ? (ne.on(le, ue), _) : ne.on(le);
      }
    };
  }
  function forceManyBody() {
    var O, _, H, V, q = constant$1(-30), G, J = 1, Z = 1 / 0, te = 0.81;
    function ne(se) {
      var ce, le = O.length, ue = quadtree(O, x$1, y).visitAfter(ae);
      for (V = se, ce = 0; ce < le; ++ce) _ = O[ce], ue.visit(oe);
    }
    function ee() {
      if (O) {
        var se, ce = O.length, le;
        for (G = new Array(ce), se = 0; se < ce; ++se) le = O[se], G[le.index] = +q(le, se, O);
      }
    }
    function ae(se) {
      var ce = 0, le, ue, de = 0, fe, he, me;
      if (se.length) {
        for (fe = he = me = 0; me < 4; ++me)
          (le = se[me]) && (ue = Math.abs(le.value)) && (ce += le.value, de += ue, fe += ue * le.x, he += ue * le.y);
        se.x = fe / de, se.y = he / de;
      } else {
        le = se, le.x = le.data.x, le.y = le.data.y;
        do
          ce += G[le.data.index];
        while (le = le.next);
      }
      se.value = ce;
    }
    function oe(se, ce, le, ue) {
      if (!se.value) return !0;
      var de = se.x - _.x, fe = se.y - _.y, he = ue - ce, me = de * de + fe * fe;
      if (he * he / te < me)
        return me < Z && (de === 0 && (de = jiggle(H), me += de * de), fe === 0 && (fe = jiggle(H), me += fe * fe), me < J && (me = Math.sqrt(J * me)), _.vx += de * se.value * V / me, _.vy += fe * se.value * V / me), !0;
      if (se.length || me >= Z) return;
      (se.data !== _ || se.next) && (de === 0 && (de = jiggle(H), me += de * de), fe === 0 && (fe = jiggle(H), me += fe * fe), me < J && (me = Math.sqrt(J * me)));
      do
        se.data !== _ && (he = G[se.data.index] * V / me, _.vx += de * he, _.vy += fe * he);
      while (se = se.next);
    }
    return ne.initialize = function(se, ce) {
      O = se, H = ce, ee();
    }, ne.strength = function(se) {
      return arguments.length ? (q = typeof se == "function" ? se : constant$1(+se), ee(), ne) : q;
    }, ne.distanceMin = function(se) {
      return arguments.length ? (J = se * se, ne) : Math.sqrt(J);
    }, ne.distanceMax = function(se) {
      return arguments.length ? (Z = se * se, ne) : Math.sqrt(Z);
    }, ne.theta = function(se) {
      return arguments.length ? (te = se * se, ne) : Math.sqrt(te);
    }, ne;
  }
  function forceX(O) {
    var _ = constant$1(0.1), H, V, q;
    typeof O != "function" && (O = constant$1(O == null ? 0 : +O));
    function G(Z) {
      for (var te = 0, ne = H.length, ee; te < ne; ++te)
        ee = H[te], ee.vx += (q[te] - ee.x) * V[te] * Z;
    }
    function J() {
      if (H) {
        var Z, te = H.length;
        for (V = new Array(te), q = new Array(te), Z = 0; Z < te; ++Z)
          V[Z] = isNaN(q[Z] = +O(H[Z], Z, H)) ? 0 : +_(H[Z], Z, H);
      }
    }
    return G.initialize = function(Z) {
      H = Z, J();
    }, G.strength = function(Z) {
      return arguments.length ? (_ = typeof Z == "function" ? Z : constant$1(+Z), J(), G) : _;
    }, G.x = function(Z) {
      return arguments.length ? (O = typeof Z == "function" ? Z : constant$1(+Z), J(), G) : O;
    }, G;
  }
  function forceY(O) {
    var _ = constant$1(0.1), H, V, q;
    typeof O != "function" && (O = constant$1(O == null ? 0 : +O));
    function G(Z) {
      for (var te = 0, ne = H.length, ee; te < ne; ++te)
        ee = H[te], ee.vy += (q[te] - ee.y) * V[te] * Z;
    }
    function J() {
      if (H) {
        var Z, te = H.length;
        for (V = new Array(te), q = new Array(te), Z = 0; Z < te; ++Z)
          V[Z] = isNaN(q[Z] = +O(H[Z], Z, H)) ? 0 : +_(H[Z], Z, H);
      }
    }
    return G.initialize = function(Z) {
      H = Z, J();
    }, G.strength = function(Z) {
      return arguments.length ? (_ = typeof Z == "function" ? Z : constant$1(+Z), J(), G) : _;
    }, G.y = function(Z) {
      return arguments.length ? (O = typeof Z == "function" ? Z : constant$1(+Z), J(), G) : O;
    }, G;
  }
  const ForceMap = {
    center: forceCenter,
    collide: forceCollide,
    nbody: forceManyBody,
    link: forceLink,
    x: forceX,
    y: forceY
  }, Forces = "forces", ForceParams = ["alpha", "alphaMin", "alphaTarget", "velocityDecay", "forces"], ForceConfig = ["static", "iterations"], ForceOutput = ["x", "y", "vx", "vy"];
  function Force(O) {
    Transform.call(this, null, O);
  }
  Force.Definition = {
    type: "Force",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "static",
      type: "boolean",
      default: !1
    }, {
      name: "restart",
      type: "boolean",
      default: !1
    }, {
      name: "iterations",
      type: "number",
      default: 300
    }, {
      name: "alpha",
      type: "number",
      default: 1
    }, {
      name: "alphaMin",
      type: "number",
      default: 1e-3
    }, {
      name: "alphaTarget",
      type: "number",
      default: 0
    }, {
      name: "velocityDecay",
      type: "number",
      default: 0.4
    }, {
      name: "forces",
      type: "param",
      array: !0,
      params: [{
        key: {
          force: "center"
        },
        params: [{
          name: "x",
          type: "number",
          default: 0
        }, {
          name: "y",
          type: "number",
          default: 0
        }]
      }, {
        key: {
          force: "collide"
        },
        params: [{
          name: "radius",
          type: "number",
          expr: !0
        }, {
          name: "strength",
          type: "number",
          default: 0.7
        }, {
          name: "iterations",
          type: "number",
          default: 1
        }]
      }, {
        key: {
          force: "nbody"
        },
        params: [{
          name: "strength",
          type: "number",
          default: -30,
          expr: !0
        }, {
          name: "theta",
          type: "number",
          default: 0.9
        }, {
          name: "distanceMin",
          type: "number",
          default: 1
        }, {
          name: "distanceMax",
          type: "number"
        }]
      }, {
        key: {
          force: "link"
        },
        params: [{
          name: "links",
          type: "data"
        }, {
          name: "id",
          type: "field"
        }, {
          name: "distance",
          type: "number",
          default: 30,
          expr: !0
        }, {
          name: "strength",
          type: "number",
          expr: !0
        }, {
          name: "iterations",
          type: "number",
          default: 1
        }]
      }, {
        key: {
          force: "x"
        },
        params: [{
          name: "strength",
          type: "number",
          default: 0.1
        }, {
          name: "x",
          type: "field"
        }]
      }, {
        key: {
          force: "y"
        },
        params: [{
          name: "strength",
          type: "number",
          default: 0.1
        }, {
          name: "y",
          type: "field"
        }]
      }]
    }, {
      name: "as",
      type: "string",
      array: !0,
      modify: !1,
      default: ForceOutput
    }]
  };
  inherits(Force, Transform, {
    transform(O, _) {
      var H = this.value, V = _.changed(_.ADD_REM), q = O.modified(ForceParams), G = O.iterations || 300;
      if (H ? (V && (_.modifies("index"), H.nodes(_.source)), (q || _.changed(_.MOD)) && setup(H, O, 0, _)) : (this.value = H = simulation(_.source, O), H.on("tick", rerun(_.dataflow, this)), O.static || (V = !0, H.tick()), _.modifies("index")), q || V || O.modified(ForceConfig) || _.changed() && O.restart) {
        if (H.alpha(Math.max(H.alpha(), O.alpha || 1)).alphaDecay(1 - Math.pow(H.alphaMin(), 1 / G)), O.static)
          for (H.stop(); --G >= 0; ) H.tick();
        else if (H.stopped() && H.restart(), !V) return _.StopPropagation;
      }
      return this.finish(O, _);
    },
    finish(O, _) {
      const H = _.dataflow;
      for (let Z = this._argops, te = 0, ne = Z.length, ee; te < ne; ++te)
        if (ee = Z[te], !(ee.name !== Forces || ee.op._argval.force !== "link")) {
          for (var V = ee.op._argops, q = 0, G = V.length, J; q < G; ++q)
            if (V[q].name === "links" && (J = V[q].op.source)) {
              H.pulse(J, H.changeset().reflow());
              break;
            }
        }
      return _.reflow(O.modified()).modifies(ForceOutput);
    }
  });
  function rerun(O, _) {
    return () => O.touch(_).run();
  }
  function simulation(O, _) {
    const H = forceSimulation(O), V = H.stop, q = H.restart;
    let G = !1;
    return H.stopped = () => G, H.restart = () => (G = !1, q()), H.stop = () => (G = !0, V()), setup(H, _, !0).on("end", () => G = !0);
  }
  function setup(O, _, H, V) {
    var q = array$5(_.forces), G, J, Z, te;
    for (G = 0, J = ForceParams.length; G < J; ++G)
      Z = ForceParams[G], Z !== Forces && _.modified(Z) && O[Z](_[Z]);
    for (G = 0, J = q.length; G < J; ++G)
      te = Forces + G, Z = H || _.modified(Forces, G) ? getForce(q[G]) : V && modified(q[G], V) ? O.force(te) : null, Z && O.force(te, Z);
    for (J = O.numForces || 0; G < J; ++G)
      O.force(Forces + G, null);
    return O.numForces = q.length, O;
  }
  function modified(O, _) {
    var H, V;
    for (H in O)
      if (isFunction$2(V = O[H]) && _.modified(accessorFields(V))) return 1;
    return 0;
  }
  function getForce(O) {
    var _, H;
    has$1(ForceMap, O.force) || error$1("Unrecognized force: " + O.force), _ = ForceMap[O.force]();
    for (H in O)
      isFunction$2(_[H]) && setForceParam(_[H], O[H], O);
    return _;
  }
  function setForceParam(O, _, H) {
    O(isFunction$2(_) ? (V) => _(V, H) : _);
  }
  const force = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    force: Force
  }, Symbol.toStringTag, { value: "Module" }));
  function defaultSeparation$2(O, _) {
    return O.parent === _.parent ? 1 : 2;
  }
  function meanX(O) {
    return O.reduce(meanXReduce, 0) / O.length;
  }
  function meanXReduce(O, _) {
    return O + _.x;
  }
  function maxY(O) {
    return 1 + O.reduce(maxYReduce, 0);
  }
  function maxYReduce(O, _) {
    return Math.max(O, _.y);
  }
  function leafLeft(O) {
    for (var _; _ = O.children; ) O = _[0];
    return O;
  }
  function leafRight(O) {
    for (var _; _ = O.children; ) O = _[_.length - 1];
    return O;
  }
  function cluster() {
    var O = defaultSeparation$2, _ = 1, H = 1, V = !1;
    function q(G) {
      var J, Z = 0;
      G.eachAfter(function(oe) {
        var se = oe.children;
        se ? (oe.x = meanX(se), oe.y = maxY(se)) : (oe.x = J ? Z += O(oe, J) : 0, oe.y = 0, J = oe);
      });
      var te = leafLeft(G), ne = leafRight(G), ee = te.x - O(te, ne) / 2, ae = ne.x + O(ne, te) / 2;
      return G.eachAfter(V ? function(oe) {
        oe.x = (oe.x - G.x) * _, oe.y = (G.y - oe.y) * H;
      } : function(oe) {
        oe.x = (oe.x - ee) / (ae - ee) * _, oe.y = (1 - (G.y ? oe.y / G.y : 1)) * H;
      });
    }
    return q.separation = function(G) {
      return arguments.length ? (O = G, q) : O;
    }, q.size = function(G) {
      return arguments.length ? (V = !1, _ = +G[0], H = +G[1], q) : V ? null : [_, H];
    }, q.nodeSize = function(G) {
      return arguments.length ? (V = !0, _ = +G[0], H = +G[1], q) : V ? [_, H] : null;
    }, q;
  }
  function count(O) {
    var _ = 0, H = O.children, V = H && H.length;
    if (!V) _ = 1;
    else for (; --V >= 0; ) _ += H[V].value;
    O.value = _;
  }
  function node_count() {
    return this.eachAfter(count);
  }
  function node_each(O, _) {
    let H = -1;
    for (const V of this)
      O.call(_, V, ++H, this);
    return this;
  }
  function node_eachBefore(O, _) {
    for (var H = this, V = [H], q, G, J = -1; H = V.pop(); )
      if (O.call(_, H, ++J, this), q = H.children)
        for (G = q.length - 1; G >= 0; --G)
          V.push(q[G]);
    return this;
  }
  function node_eachAfter(O, _) {
    for (var H = this, V = [H], q = [], G, J, Z, te = -1; H = V.pop(); )
      if (q.push(H), G = H.children)
        for (J = 0, Z = G.length; J < Z; ++J)
          V.push(G[J]);
    for (; H = q.pop(); )
      O.call(_, H, ++te, this);
    return this;
  }
  function node_find(O, _) {
    let H = -1;
    for (const V of this)
      if (O.call(_, V, ++H, this))
        return V;
  }
  function node_sum(O) {
    return this.eachAfter(function(_) {
      for (var H = +O(_.data) || 0, V = _.children, q = V && V.length; --q >= 0; ) H += V[q].value;
      _.value = H;
    });
  }
  function node_sort(O) {
    return this.eachBefore(function(_) {
      _.children && _.children.sort(O);
    });
  }
  function node_path(O) {
    for (var _ = this, H = leastCommonAncestor(_, O), V = [_]; _ !== H; )
      _ = _.parent, V.push(_);
    for (var q = V.length; O !== H; )
      V.splice(q, 0, O), O = O.parent;
    return V;
  }
  function leastCommonAncestor(O, _) {
    if (O === _) return O;
    var H = O.ancestors(), V = _.ancestors(), q = null;
    for (O = H.pop(), _ = V.pop(); O === _; )
      q = O, O = H.pop(), _ = V.pop();
    return q;
  }
  function node_ancestors() {
    for (var O = this, _ = [O]; O = O.parent; )
      _.push(O);
    return _;
  }
  function node_descendants() {
    return Array.from(this);
  }
  function node_leaves() {
    var O = [];
    return this.eachBefore(function(_) {
      _.children || O.push(_);
    }), O;
  }
  function node_links() {
    var O = this, _ = [];
    return O.each(function(H) {
      H !== O && _.push({ source: H.parent, target: H });
    }), _;
  }
  function* node_iterator() {
    var O = this, _, H = [O], V, q, G;
    do
      for (_ = H.reverse(), H = []; O = _.pop(); )
        if (yield O, V = O.children)
          for (q = 0, G = V.length; q < G; ++q)
            H.push(V[q]);
    while (H.length);
  }
  function hierarchy(O, _) {
    O instanceof Map ? (O = [void 0, O], _ === void 0 && (_ = mapChildren)) : _ === void 0 && (_ = objectChildren);
    for (var H = new Node$2(O), V, q = [H], G, J, Z, te; V = q.pop(); )
      if ((J = _(V.data)) && (te = (J = Array.from(J)).length))
        for (V.children = J, Z = te - 1; Z >= 0; --Z)
          q.push(G = J[Z] = new Node$2(J[Z])), G.parent = V, G.depth = V.depth + 1;
    return H.eachBefore(computeHeight);
  }
  function node_copy() {
    return hierarchy(this).eachBefore(copyData);
  }
  function objectChildren(O) {
    return O.children;
  }
  function mapChildren(O) {
    return Array.isArray(O) ? O[1] : null;
  }
  function copyData(O) {
    O.data.value !== void 0 && (O.value = O.data.value), O.data = O.data.data;
  }
  function computeHeight(O) {
    var _ = 0;
    do
      O.height = _;
    while ((O = O.parent) && O.height < ++_);
  }
  function Node$2(O) {
    this.data = O, this.depth = this.height = 0, this.parent = null;
  }
  Node$2.prototype = hierarchy.prototype = {
    constructor: Node$2,
    count: node_count,
    each: node_each,
    eachAfter: node_eachAfter,
    eachBefore: node_eachBefore,
    find: node_find,
    sum: node_sum,
    sort: node_sort,
    path: node_path,
    ancestors: node_ancestors,
    descendants: node_descendants,
    leaves: node_leaves,
    links: node_links,
    copy: node_copy,
    [Symbol.iterator]: node_iterator
  };
  function optional(O) {
    return O == null ? null : required(O);
  }
  function required(O) {
    if (typeof O != "function") throw new Error();
    return O;
  }
  function constantZero() {
    return 0;
  }
  function constant(O) {
    return function() {
      return O;
    };
  }
  const a = 1664525, c = 1013904223, m$1 = 4294967296;
  function lcg() {
    let O = 1;
    return () => (O = (a * O + c) % m$1) / m$1;
  }
  function array$2(O) {
    return typeof O == "object" && "length" in O ? O : Array.from(O);
  }
  function shuffle(O, _) {
    let H = O.length, V, q;
    for (; H; )
      q = _() * H-- | 0, V = O[H], O[H] = O[q], O[q] = V;
    return O;
  }
  function packEncloseRandom(O, _) {
    for (var H = 0, V = (O = shuffle(Array.from(O), _)).length, q = [], G, J; H < V; )
      G = O[H], J && enclosesWeak(J, G) ? ++H : (J = encloseBasis(q = extendBasis(q, G)), H = 0);
    return J;
  }
  function extendBasis(O, _) {
    var H, V;
    if (enclosesWeakAll(_, O)) return [_];
    for (H = 0; H < O.length; ++H)
      if (enclosesNot(_, O[H]) && enclosesWeakAll(encloseBasis2(O[H], _), O))
        return [O[H], _];
    for (H = 0; H < O.length - 1; ++H)
      for (V = H + 1; V < O.length; ++V)
        if (enclosesNot(encloseBasis2(O[H], O[V]), _) && enclosesNot(encloseBasis2(O[H], _), O[V]) && enclosesNot(encloseBasis2(O[V], _), O[H]) && enclosesWeakAll(encloseBasis3(O[H], O[V], _), O))
          return [O[H], O[V], _];
    throw new Error();
  }
  function enclosesNot(O, _) {
    var H = O.r - _.r, V = _.x - O.x, q = _.y - O.y;
    return H < 0 || H * H < V * V + q * q;
  }
  function enclosesWeak(O, _) {
    var H = O.r - _.r + Math.max(O.r, _.r, 1) * 1e-9, V = _.x - O.x, q = _.y - O.y;
    return H > 0 && H * H > V * V + q * q;
  }
  function enclosesWeakAll(O, _) {
    for (var H = 0; H < _.length; ++H)
      if (!enclosesWeak(O, _[H]))
        return !1;
    return !0;
  }
  function encloseBasis(O) {
    switch (O.length) {
      case 1:
        return encloseBasis1(O[0]);
      case 2:
        return encloseBasis2(O[0], O[1]);
      case 3:
        return encloseBasis3(O[0], O[1], O[2]);
    }
  }
  function encloseBasis1(O) {
    return {
      x: O.x,
      y: O.y,
      r: O.r
    };
  }
  function encloseBasis2(O, _) {
    var H = O.x, V = O.y, q = O.r, G = _.x, J = _.y, Z = _.r, te = G - H, ne = J - V, ee = Z - q, ae = Math.sqrt(te * te + ne * ne);
    return {
      x: (H + G + te / ae * ee) / 2,
      y: (V + J + ne / ae * ee) / 2,
      r: (ae + q + Z) / 2
    };
  }
  function encloseBasis3(O, _, H) {
    var V = O.x, q = O.y, G = O.r, J = _.x, Z = _.y, te = _.r, ne = H.x, ee = H.y, ae = H.r, oe = V - J, se = V - ne, ce = q - Z, le = q - ee, ue = te - G, de = ae - G, fe = V * V + q * q - G * G, he = fe - J * J - Z * Z + te * te, me = fe - ne * ne - ee * ee + ae * ae, pe = se * ce - oe * le, ye = (ce * me - le * he) / (pe * 2) - V, ge = (le * ue - ce * de) / pe, ve = (se * he - oe * me) / (pe * 2) - q, be = (oe * de - se * ue) / pe, Ce = ge * ge + be * be - 1, Se = 2 * (G + ye * ge + ve * be), we = ye * ye + ve * ve - G * G, Ee = -(Math.abs(Ce) > 1e-6 ? (Se + Math.sqrt(Se * Se - 4 * Ce * we)) / (2 * Ce) : we / Se);
    return {
      x: V + ye + ge * Ee,
      y: q + ve + be * Ee,
      r: Ee
    };
  }
  function place(O, _, H) {
    var V = O.x - _.x, q, G, J = O.y - _.y, Z, te, ne = V * V + J * J;
    ne ? (G = _.r + H.r, G *= G, te = O.r + H.r, te *= te, G > te ? (q = (ne + te - G) / (2 * ne), Z = Math.sqrt(Math.max(0, te / ne - q * q)), H.x = O.x - q * V - Z * J, H.y = O.y - q * J + Z * V) : (q = (ne + G - te) / (2 * ne), Z = Math.sqrt(Math.max(0, G / ne - q * q)), H.x = _.x + q * V - Z * J, H.y = _.y + q * J + Z * V)) : (H.x = _.x + H.r, H.y = _.y);
  }
  function intersects(O, _) {
    var H = O.r + _.r - 1e-6, V = _.x - O.x, q = _.y - O.y;
    return H > 0 && H * H > V * V + q * q;
  }
  function score$1(O) {
    var _ = O._, H = O.next._, V = _.r + H.r, q = (_.x * H.r + H.x * _.r) / V, G = (_.y * H.r + H.y * _.r) / V;
    return q * q + G * G;
  }
  function Node$1(O) {
    this._ = O, this.next = null, this.previous = null;
  }
  function packSiblingsRandom(O, _) {
    if (!(G = (O = array$2(O)).length)) return 0;
    var H, V, q, G, J, Z, te, ne, ee, ae, oe;
    if (H = O[0], H.x = 0, H.y = 0, !(G > 1)) return H.r;
    if (V = O[1], H.x = -V.r, V.x = H.r, V.y = 0, !(G > 2)) return H.r + V.r;
    place(V, H, q = O[2]), H = new Node$1(H), V = new Node$1(V), q = new Node$1(q), H.next = q.previous = V, V.next = H.previous = q, q.next = V.previous = H;
    e: for (te = 3; te < G; ++te) {
      place(H._, V._, q = O[te]), q = new Node$1(q), ne = V.next, ee = H.previous, ae = V._.r, oe = H._.r;
      do
        if (ae <= oe) {
          if (intersects(ne._, q._)) {
            V = ne, H.next = V, V.previous = H, --te;
            continue e;
          }
          ae += ne._.r, ne = ne.next;
        } else {
          if (intersects(ee._, q._)) {
            H = ee, H.next = V, V.previous = H, --te;
            continue e;
          }
          oe += ee._.r, ee = ee.previous;
        }
      while (ne !== ee.next);
      for (q.previous = H, q.next = V, H.next = V.previous = V = q, J = score$1(H); (q = q.next) !== V; )
        (Z = score$1(q)) < J && (H = q, J = Z);
      V = H.next;
    }
    for (H = [V._], q = V; (q = q.next) !== V; ) H.push(q._);
    for (q = packEncloseRandom(H, _), te = 0; te < G; ++te) H = O[te], H.x -= q.x, H.y -= q.y;
    return q.r;
  }
  function defaultRadius(O) {
    return Math.sqrt(O.value);
  }
  function pack() {
    var O = null, _ = 1, H = 1, V = constantZero;
    function q(G) {
      const J = lcg();
      return G.x = _ / 2, G.y = H / 2, O ? G.eachBefore(radiusLeaf(O)).eachAfter(packChildrenRandom(V, 0.5, J)).eachBefore(translateChild(1)) : G.eachBefore(radiusLeaf(defaultRadius)).eachAfter(packChildrenRandom(constantZero, 1, J)).eachAfter(packChildrenRandom(V, G.r / Math.min(_, H), J)).eachBefore(translateChild(Math.min(_, H) / (2 * G.r))), G;
    }
    return q.radius = function(G) {
      return arguments.length ? (O = optional(G), q) : O;
    }, q.size = function(G) {
      return arguments.length ? (_ = +G[0], H = +G[1], q) : [_, H];
    }, q.padding = function(G) {
      return arguments.length ? (V = typeof G == "function" ? G : constant(+G), q) : V;
    }, q;
  }
  function radiusLeaf(O) {
    return function(_) {
      _.children || (_.r = Math.max(0, +O(_) || 0));
    };
  }
  function packChildrenRandom(O, _, H) {
    return function(V) {
      if (q = V.children) {
        var q, G, J = q.length, Z = O(V) * _ || 0, te;
        if (Z) for (G = 0; G < J; ++G) q[G].r += Z;
        if (te = packSiblingsRandom(q, H), Z) for (G = 0; G < J; ++G) q[G].r -= Z;
        V.r = te + Z;
      }
    };
  }
  function translateChild(O) {
    return function(_) {
      var H = _.parent;
      _.r *= O, H && (_.x = H.x + O * _.x, _.y = H.y + O * _.y);
    };
  }
  function roundNode(O) {
    O.x0 = Math.round(O.x0), O.y0 = Math.round(O.y0), O.x1 = Math.round(O.x1), O.y1 = Math.round(O.y1);
  }
  function treemapDice(O, _, H, V, q) {
    for (var G = O.children, J, Z = -1, te = G.length, ne = O.value && (V - _) / O.value; ++Z < te; )
      J = G[Z], J.y0 = H, J.y1 = q, J.x0 = _, J.x1 = _ += J.value * ne;
  }
  function partition$1() {
    var O = 1, _ = 1, H = 0, V = !1;
    function q(J) {
      var Z = J.height + 1;
      return J.x0 = J.y0 = H, J.x1 = O, J.y1 = _ / Z, J.eachBefore(G(_, Z)), V && J.eachBefore(roundNode), J;
    }
    function G(J, Z) {
      return function(te) {
        te.children && treemapDice(te, te.x0, J * (te.depth + 1) / Z, te.x1, J * (te.depth + 2) / Z);
        var ne = te.x0, ee = te.y0, ae = te.x1 - H, oe = te.y1 - H;
        ae < ne && (ne = ae = (ne + ae) / 2), oe < ee && (ee = oe = (ee + oe) / 2), te.x0 = ne, te.y0 = ee, te.x1 = ae, te.y1 = oe;
      };
    }
    return q.round = function(J) {
      return arguments.length ? (V = !!J, q) : V;
    }, q.size = function(J) {
      return arguments.length ? (O = +J[0], _ = +J[1], q) : [O, _];
    }, q.padding = function(J) {
      return arguments.length ? (H = +J, q) : H;
    }, q;
  }
  var preroot = { depth: -1 }, ambiguous = {}, imputed = {};
  function defaultId(O) {
    return O.id;
  }
  function defaultParentId(O) {
    return O.parentId;
  }
  function stratify() {
    var O = defaultId, _ = defaultParentId, H;
    function V(q) {
      var G = Array.from(q), J = O, Z = _, te, ne, ee, ae, oe, se, ce, le, ue = /* @__PURE__ */ new Map();
      if (H != null) {
        const de = G.map((me, pe) => normalize$1(H(me, pe, q))), fe = de.map(parentof), he = new Set(de).add("");
        for (const me of fe)
          he.has(me) || (he.add(me), de.push(me), fe.push(parentof(me)), G.push(imputed));
        J = (me, pe) => de[pe], Z = (me, pe) => fe[pe];
      }
      for (ee = 0, te = G.length; ee < te; ++ee)
        ne = G[ee], se = G[ee] = new Node$2(ne), (ce = J(ne, ee, q)) != null && (ce += "") && (le = se.id = ce, ue.set(le, ue.has(le) ? ambiguous : se)), (ce = Z(ne, ee, q)) != null && (ce += "") && (se.parent = ce);
      for (ee = 0; ee < te; ++ee)
        if (se = G[ee], ce = se.parent) {
          if (oe = ue.get(ce), !oe) throw new Error("missing: " + ce);
          if (oe === ambiguous) throw new Error("ambiguous: " + ce);
          oe.children ? oe.children.push(se) : oe.children = [se], se.parent = oe;
        } else {
          if (ae) throw new Error("multiple roots");
          ae = se;
        }
      if (!ae) throw new Error("no root");
      if (H != null) {
        for (; ae.data === imputed && ae.children.length === 1; )
          ae = ae.children[0], --te;
        for (let de = G.length - 1; de >= 0 && (se = G[de], se.data === imputed); --de)
          se.data = null;
      }
      if (ae.parent = preroot, ae.eachBefore(function(de) {
        de.depth = de.parent.depth + 1, --te;
      }).eachBefore(computeHeight), ae.parent = null, te > 0) throw new Error("cycle");
      return ae;
    }
    return V.id = function(q) {
      return arguments.length ? (O = optional(q), V) : O;
    }, V.parentId = function(q) {
      return arguments.length ? (_ = optional(q), V) : _;
    }, V.path = function(q) {
      return arguments.length ? (H = optional(q), V) : H;
    }, V;
  }
  function normalize$1(O) {
    O = `${O}`;
    let _ = O.length;
    return slash(O, _ - 1) && !slash(O, _ - 2) && (O = O.slice(0, -1)), O[0] === "/" ? O : `/${O}`;
  }
  function parentof(O) {
    let _ = O.length;
    if (_ < 2) return "";
    for (; --_ > 1 && !slash(O, _); ) ;
    return O.slice(0, _);
  }
  function slash(O, _) {
    if (O[_] === "/") {
      let H = 0;
      for (; _ > 0 && O[--_] === "\\"; ) ++H;
      if (!(H & 1)) return !0;
    }
    return !1;
  }
  function defaultSeparation$1(O, _) {
    return O.parent === _.parent ? 1 : 2;
  }
  function nextLeft(O) {
    var _ = O.children;
    return _ ? _[0] : O.t;
  }
  function nextRight(O) {
    var _ = O.children;
    return _ ? _[_.length - 1] : O.t;
  }
  function moveSubtree(O, _, H) {
    var V = H / (_.i - O.i);
    _.c -= V, _.s += H, O.c += V, _.z += H, _.m += H;
  }
  function executeShifts(O) {
    for (var _ = 0, H = 0, V = O.children, q = V.length, G; --q >= 0; )
      G = V[q], G.z += _, G.m += _, _ += G.s + (H += G.c);
  }
  function nextAncestor(O, _, H) {
    return O.a.parent === _.parent ? O.a : H;
  }
  function TreeNode$1(O, _) {
    this._ = O, this.parent = null, this.children = null, this.A = null, this.a = this, this.z = 0, this.m = 0, this.c = 0, this.s = 0, this.t = null, this.i = _;
  }
  TreeNode$1.prototype = Object.create(Node$2.prototype);
  function treeRoot(O) {
    for (var _ = new TreeNode$1(O, 0), H, V = [_], q, G, J, Z; H = V.pop(); )
      if (G = H._.children)
        for (H.children = new Array(Z = G.length), J = Z - 1; J >= 0; --J)
          V.push(q = H.children[J] = new TreeNode$1(G[J], J)), q.parent = H;
    return (_.parent = new TreeNode$1(null, 0)).children = [_], _;
  }
  function tree$1() {
    var O = defaultSeparation$1, _ = 1, H = 1, V = null;
    function q(ne) {
      var ee = treeRoot(ne);
      if (ee.eachAfter(G), ee.parent.m = -ee.z, ee.eachBefore(J), V) ne.eachBefore(te);
      else {
        var ae = ne, oe = ne, se = ne;
        ne.eachBefore(function(fe) {
          fe.x < ae.x && (ae = fe), fe.x > oe.x && (oe = fe), fe.depth > se.depth && (se = fe);
        });
        var ce = ae === oe ? 1 : O(ae, oe) / 2, le = ce - ae.x, ue = _ / (oe.x + ce + le), de = H / (se.depth || 1);
        ne.eachBefore(function(fe) {
          fe.x = (fe.x + le) * ue, fe.y = fe.depth * de;
        });
      }
      return ne;
    }
    function G(ne) {
      var ee = ne.children, ae = ne.parent.children, oe = ne.i ? ae[ne.i - 1] : null;
      if (ee) {
        executeShifts(ne);
        var se = (ee[0].z + ee[ee.length - 1].z) / 2;
        oe ? (ne.z = oe.z + O(ne._, oe._), ne.m = ne.z - se) : ne.z = se;
      } else oe && (ne.z = oe.z + O(ne._, oe._));
      ne.parent.A = Z(ne, oe, ne.parent.A || ae[0]);
    }
    function J(ne) {
      ne._.x = ne.z + ne.parent.m, ne.m += ne.parent.m;
    }
    function Z(ne, ee, ae) {
      if (ee) {
        for (var oe = ne, se = ne, ce = ee, le = oe.parent.children[0], ue = oe.m, de = se.m, fe = ce.m, he = le.m, me; ce = nextRight(ce), oe = nextLeft(oe), ce && oe; )
          le = nextLeft(le), se = nextRight(se), se.a = ne, me = ce.z + fe - oe.z - ue + O(ce._, oe._), me > 0 && (moveSubtree(nextAncestor(ce, ne, ae), ne, me), ue += me, de += me), fe += ce.m, ue += oe.m, he += le.m, de += se.m;
        ce && !nextRight(se) && (se.t = ce, se.m += fe - de), oe && !nextLeft(le) && (le.t = oe, le.m += ue - he, ae = ne);
      }
      return ae;
    }
    function te(ne) {
      ne.x *= _, ne.y = ne.depth * H;
    }
    return q.separation = function(ne) {
      return arguments.length ? (O = ne, q) : O;
    }, q.size = function(ne) {
      return arguments.length ? (V = !1, _ = +ne[0], H = +ne[1], q) : V ? null : [_, H];
    }, q.nodeSize = function(ne) {
      return arguments.length ? (V = !0, _ = +ne[0], H = +ne[1], q) : V ? [_, H] : null;
    }, q;
  }
  function treemapSlice(O, _, H, V, q) {
    for (var G = O.children, J, Z = -1, te = G.length, ne = O.value && (q - H) / O.value; ++Z < te; )
      J = G[Z], J.x0 = _, J.x1 = V, J.y0 = H, J.y1 = H += J.value * ne;
  }
  var phi = (1 + Math.sqrt(5)) / 2;
  function squarifyRatio(O, _, H, V, q, G) {
    for (var J = [], Z = _.children, te, ne, ee = 0, ae = 0, oe = Z.length, se, ce, le = _.value, ue, de, fe, he, me, pe, ye; ee < oe; ) {
      se = q - H, ce = G - V;
      do
        ue = Z[ae++].value;
      while (!ue && ae < oe);
      for (de = fe = ue, pe = Math.max(ce / se, se / ce) / (le * O), ye = ue * ue * pe, me = Math.max(fe / ye, ye / de); ae < oe; ++ae) {
        if (ue += ne = Z[ae].value, ne < de && (de = ne), ne > fe && (fe = ne), ye = ue * ue * pe, he = Math.max(fe / ye, ye / de), he > me) {
          ue -= ne;
          break;
        }
        me = he;
      }
      J.push(te = { value: ue, dice: se < ce, children: Z.slice(ee, ae) }), te.dice ? treemapDice(te, H, V, q, le ? V += ce * ue / le : G) : treemapSlice(te, H, V, le ? H += se * ue / le : q, G), le -= ue, ee = ae;
    }
    return J;
  }
  const treemapSquarify = function O(_) {
    function H(V, q, G, J, Z) {
      squarifyRatio(_, V, q, G, J, Z);
    }
    return H.ratio = function(V) {
      return O((V = +V) > 1 ? V : 1);
    }, H;
  }(phi);
  function treemap() {
    var O = treemapSquarify, _ = !1, H = 1, V = 1, q = [0], G = constantZero, J = constantZero, Z = constantZero, te = constantZero, ne = constantZero;
    function ee(oe) {
      return oe.x0 = oe.y0 = 0, oe.x1 = H, oe.y1 = V, oe.eachBefore(ae), q = [0], _ && oe.eachBefore(roundNode), oe;
    }
    function ae(oe) {
      var se = q[oe.depth], ce = oe.x0 + se, le = oe.y0 + se, ue = oe.x1 - se, de = oe.y1 - se;
      ue < ce && (ce = ue = (ce + ue) / 2), de < le && (le = de = (le + de) / 2), oe.x0 = ce, oe.y0 = le, oe.x1 = ue, oe.y1 = de, oe.children && (se = q[oe.depth + 1] = G(oe) / 2, ce += ne(oe) - se, le += J(oe) - se, ue -= Z(oe) - se, de -= te(oe) - se, ue < ce && (ce = ue = (ce + ue) / 2), de < le && (le = de = (le + de) / 2), O(oe, ce, le, ue, de));
    }
    return ee.round = function(oe) {
      return arguments.length ? (_ = !!oe, ee) : _;
    }, ee.size = function(oe) {
      return arguments.length ? (H = +oe[0], V = +oe[1], ee) : [H, V];
    }, ee.tile = function(oe) {
      return arguments.length ? (O = required(oe), ee) : O;
    }, ee.padding = function(oe) {
      return arguments.length ? ee.paddingInner(oe).paddingOuter(oe) : ee.paddingInner();
    }, ee.paddingInner = function(oe) {
      return arguments.length ? (G = typeof oe == "function" ? oe : constant(+oe), ee) : G;
    }, ee.paddingOuter = function(oe) {
      return arguments.length ? ee.paddingTop(oe).paddingRight(oe).paddingBottom(oe).paddingLeft(oe) : ee.paddingTop();
    }, ee.paddingTop = function(oe) {
      return arguments.length ? (J = typeof oe == "function" ? oe : constant(+oe), ee) : J;
    }, ee.paddingRight = function(oe) {
      return arguments.length ? (Z = typeof oe == "function" ? oe : constant(+oe), ee) : Z;
    }, ee.paddingBottom = function(oe) {
      return arguments.length ? (te = typeof oe == "function" ? oe : constant(+oe), ee) : te;
    }, ee.paddingLeft = function(oe) {
      return arguments.length ? (ne = typeof oe == "function" ? oe : constant(+oe), ee) : ne;
    }, ee;
  }
  function treemapBinary(O, _, H, V, q) {
    var G = O.children, J, Z = G.length, te, ne = new Array(Z + 1);
    for (ne[0] = te = J = 0; J < Z; ++J)
      ne[J + 1] = te += G[J].value;
    ee(0, Z, O.value, _, H, V, q);
    function ee(ae, oe, se, ce, le, ue, de) {
      if (ae >= oe - 1) {
        var fe = G[ae];
        fe.x0 = ce, fe.y0 = le, fe.x1 = ue, fe.y1 = de;
        return;
      }
      for (var he = ne[ae], me = se / 2 + he, pe = ae + 1, ye = oe - 1; pe < ye; ) {
        var ge = pe + ye >>> 1;
        ne[ge] < me ? pe = ge + 1 : ye = ge;
      }
      me - ne[pe - 1] < ne[pe] - me && ae + 1 < pe && --pe;
      var ve = ne[pe] - he, be = se - ve;
      if (ue - ce > de - le) {
        var Ce = se ? (ce * be + ue * ve) / se : ue;
        ee(ae, pe, ve, ce, le, Ce, de), ee(pe, oe, be, Ce, le, ue, de);
      } else {
        var Se = se ? (le * be + de * ve) / se : de;
        ee(ae, pe, ve, ce, le, ue, Se), ee(pe, oe, be, ce, Se, ue, de);
      }
    }
  }
  function treemapSliceDice(O, _, H, V, q) {
    (O.depth & 1 ? treemapSlice : treemapDice)(O, _, H, V, q);
  }
  const treemapResquarify = function O(_) {
    function H(V, q, G, J, Z) {
      if ((te = V._squarify) && te.ratio === _)
        for (var te, ne, ee, ae, oe = -1, se, ce = te.length, le = V.value; ++oe < ce; ) {
          for (ne = te[oe], ee = ne.children, ae = ne.value = 0, se = ee.length; ae < se; ++ae) ne.value += ee[ae].value;
          ne.dice ? treemapDice(ne, q, G, J, le ? G += (Z - G) * ne.value / le : Z) : treemapSlice(ne, q, G, le ? q += (J - q) * ne.value / le : J, Z), le -= ne.value;
        }
      else
        V._squarify = te = squarifyRatio(_, V, q, G, J, Z), te.ratio = _;
    }
    return H.ratio = function(V) {
      return O((V = +V) > 1 ? V : 1);
    }, H;
  }(phi);
  function lookup$2(O, _, H) {
    const V = {};
    return O.each((q) => {
      const G = q.data;
      H(G) && (V[_(G)] = q);
    }), O.lookup = V, O;
  }
  function Nest(O) {
    Transform.call(this, null, O);
  }
  Nest.Definition = {
    type: "Nest",
    metadata: {
      treesource: !0,
      changes: !0
    },
    params: [{
      name: "keys",
      type: "field",
      array: !0
    }, {
      name: "generate",
      type: "boolean"
    }]
  };
  const children$1 = (O) => O.values;
  inherits(Nest, Transform, {
    transform(O, _) {
      _.source || error$1("Nest transform requires an upstream data source.");
      var H = O.generate, V = O.modified(), q = _.clone(), G = this.value;
      return (!G || V || _.changed()) && (G && G.each((J) => {
        J.children && isTuple(J.data) && q.rem.push(J.data);
      }), this.value = G = hierarchy({
        values: array$5(O.keys).reduce((J, Z) => (J.key(Z), J), nest()).entries(q.source)
      }, children$1), H && G.each((J) => {
        J.children && (J = ingest$1(J.data), q.add.push(J), q.source.push(J));
      }), lookup$2(G, tupleid, tupleid)), q.source.root = G, q;
    }
  });
  function nest() {
    const O = [], _ = {
      entries: (q) => V(H(q, 0), 0),
      key: (q) => (O.push(q), _)
    };
    function H(q, G) {
      if (G >= O.length)
        return q;
      const J = q.length, Z = O[G++], te = {}, ne = {};
      let ee = -1, ae, oe, se;
      for (; ++ee < J; )
        ae = Z(oe = q[ee]) + "", (se = te[ae]) ? se.push(oe) : te[ae] = [oe];
      for (ae in te)
        ne[ae] = H(te[ae], G);
      return ne;
    }
    function V(q, G) {
      if (++G > O.length) return q;
      const J = [];
      for (const Z in q)
        J.push({
          key: Z,
          values: V(q[Z], G)
        });
      return J;
    }
    return _;
  }
  function HierarchyLayout(O) {
    Transform.call(this, null, O);
  }
  const defaultSeparation = (O, _) => O.parent === _.parent ? 1 : 2;
  inherits(HierarchyLayout, Transform, {
    transform(O, _) {
      (!_.source || !_.source.root) && error$1(this.constructor.name + " transform requires a backing tree data source.");
      const H = this.layout(O.method), V = this.fields, q = _.source.root, G = O.as || V;
      O.field ? q.sum(O.field) : q.count(), O.sort && q.sort(stableCompare(O.sort, (J) => J.data)), setParams(H, this.params, O), H.separation && H.separation(O.separation !== !1 ? defaultSeparation : one$2);
      try {
        this.value = H(q);
      } catch (J) {
        error$1(J);
      }
      return q.each((J) => setFields(J, V, G)), _.reflow(O.modified()).modifies(G).modifies("leaf");
    }
  });
  function setParams(O, _, H) {
    for (let V, q = 0, G = _.length; q < G; ++q)
      V = _[q], V in H && O[V](H[V]);
  }
  function setFields(O, _, H) {
    const V = O.data, q = _.length - 1;
    for (let G = 0; G < q; ++G)
      V[H[G]] = O[_[G]];
    V[H[q]] = O.children ? O.children.length : 0;
  }
  const Output$3 = ["x", "y", "r", "depth", "children"];
  function Pack(O) {
    HierarchyLayout.call(this, O);
  }
  Pack.Definition = {
    type: "Pack",
    metadata: {
      tree: !0,
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "sort",
      type: "compare"
    }, {
      name: "padding",
      type: "number",
      default: 0
    }, {
      name: "radius",
      type: "field",
      default: null
    }, {
      name: "size",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: Output$3.length,
      default: Output$3
    }]
  };
  inherits(Pack, HierarchyLayout, {
    layout: pack,
    params: ["radius", "size", "padding"],
    fields: Output$3
  });
  const Output$2 = ["x0", "y0", "x1", "y1", "depth", "children"];
  function Partition(O) {
    HierarchyLayout.call(this, O);
  }
  Partition.Definition = {
    type: "Partition",
    metadata: {
      tree: !0,
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "sort",
      type: "compare"
    }, {
      name: "padding",
      type: "number",
      default: 0
    }, {
      name: "round",
      type: "boolean",
      default: !1
    }, {
      name: "size",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: Output$2.length,
      default: Output$2
    }]
  };
  inherits(Partition, HierarchyLayout, {
    layout: partition$1,
    params: ["size", "round", "padding"],
    fields: Output$2
  });
  function Stratify(O) {
    Transform.call(this, null, O);
  }
  Stratify.Definition = {
    type: "Stratify",
    metadata: {
      treesource: !0
    },
    params: [{
      name: "key",
      type: "field",
      required: !0
    }, {
      name: "parentKey",
      type: "field",
      required: !0
    }]
  };
  inherits(Stratify, Transform, {
    transform(O, _) {
      _.source || error$1("Stratify transform requires an upstream data source.");
      let H = this.value;
      const V = O.modified(), q = _.fork(_.ALL).materialize(_.SOURCE), G = !H || V || _.changed(_.ADD_REM) || _.modified(O.key.fields) || _.modified(O.parentKey.fields);
      return q.source = q.source.slice(), G && (H = q.source.length ? lookup$2(stratify().id(O.key).parentId(O.parentKey)(q.source), O.key, truthy) : lookup$2(stratify()([{}]), O.key, O.key)), q.source.root = this.value = H, q;
    }
  });
  const Layouts = {
    tidy: tree$1,
    cluster
  }, Output$1$1 = ["x", "y", "depth", "children"];
  function Tree$1(O) {
    HierarchyLayout.call(this, O);
  }
  Tree$1.Definition = {
    type: "Tree",
    metadata: {
      tree: !0,
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "sort",
      type: "compare"
    }, {
      name: "method",
      type: "enum",
      default: "tidy",
      values: ["tidy", "cluster"]
    }, {
      name: "size",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "nodeSize",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "separation",
      type: "boolean",
      default: !0
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: Output$1$1.length,
      default: Output$1$1
    }]
  };
  inherits(Tree$1, HierarchyLayout, {
    /**
     * Tree layout generator. Supports both 'tidy' and 'cluster' layouts.
     */
    layout(O) {
      const _ = O || "tidy";
      if (has$1(Layouts, _)) return Layouts[_]();
      error$1("Unrecognized Tree layout method: " + _);
    },
    params: ["size", "nodeSize"],
    fields: Output$1$1
  });
  function TreeLinks(O) {
    Transform.call(this, [], O);
  }
  TreeLinks.Definition = {
    type: "TreeLinks",
    metadata: {
      tree: !0,
      generates: !0,
      changes: !0
    },
    params: []
  };
  inherits(TreeLinks, Transform, {
    transform(O, _) {
      const H = this.value, V = _.source && _.source.root, q = _.fork(_.NO_SOURCE), G = {};
      return V || error$1("TreeLinks transform requires a tree data source."), _.changed(_.ADD_REM) ? (q.rem = H, _.visit(_.SOURCE, (J) => G[tupleid(J)] = 1), V.each((J) => {
        const Z = J.data, te = J.parent && J.parent.data;
        te && G[tupleid(Z)] && G[tupleid(te)] && q.add.push(ingest$1({
          source: te,
          target: Z
        }));
      }), this.value = q.add) : _.changed(_.MOD) && (_.visit(_.MOD, (J) => G[tupleid(J)] = 1), H.forEach((J) => {
        (G[tupleid(J.source)] || G[tupleid(J.target)]) && q.mod.push(J);
      })), q;
    }
  });
  const Tiles = {
    binary: treemapBinary,
    dice: treemapDice,
    slice: treemapSlice,
    slicedice: treemapSliceDice,
    squarify: treemapSquarify,
    resquarify: treemapResquarify
  }, Output$4 = ["x0", "y0", "x1", "y1", "depth", "children"];
  function Treemap(O) {
    HierarchyLayout.call(this, O);
  }
  Treemap.Definition = {
    type: "Treemap",
    metadata: {
      tree: !0,
      modifies: !0
    },
    params: [{
      name: "field",
      type: "field"
    }, {
      name: "sort",
      type: "compare"
    }, {
      name: "method",
      type: "enum",
      default: "squarify",
      values: ["squarify", "resquarify", "binary", "dice", "slice", "slicedice"]
    }, {
      name: "padding",
      type: "number",
      default: 0
    }, {
      name: "paddingInner",
      type: "number",
      default: 0
    }, {
      name: "paddingOuter",
      type: "number",
      default: 0
    }, {
      name: "paddingTop",
      type: "number",
      default: 0
    }, {
      name: "paddingRight",
      type: "number",
      default: 0
    }, {
      name: "paddingBottom",
      type: "number",
      default: 0
    }, {
      name: "paddingLeft",
      type: "number",
      default: 0
    }, {
      name: "ratio",
      type: "number",
      default: 1.618033988749895
    }, {
      name: "round",
      type: "boolean",
      default: !1
    }, {
      name: "size",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: Output$4.length,
      default: Output$4
    }]
  };
  inherits(Treemap, HierarchyLayout, {
    /**
     * Treemap layout generator. Adds 'method' and 'ratio' parameters
     * to configure the underlying tile method.
     */
    layout() {
      const O = treemap();
      return O.ratio = (_) => {
        const H = O.tile();
        H.ratio && O.tile(H.ratio(_));
      }, O.method = (_) => {
        has$1(Tiles, _) ? O.tile(Tiles[_]) : error$1("Unrecognized Treemap layout method: " + _);
      }, O;
    },
    params: ["method", "ratio", "size", "round", "padding", "paddingInner", "paddingOuter", "paddingTop", "paddingRight", "paddingBottom", "paddingLeft"],
    fields: Output$4
  });
  const tree = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    nest: Nest,
    pack: Pack,
    partition: Partition,
    stratify: Stratify,
    tree: Tree$1,
    treelinks: TreeLinks,
    treemap: Treemap
  }, Symbol.toStringTag, { value: "Module" })), ALPHA_MASK = 4278190080;
  function baseBitmaps(O, _) {
    const H = O.bitmap();
    return (_ || []).forEach((V) => H.set(O(V.boundary[0]), O(V.boundary[3]))), [H, void 0];
  }
  function markBitmaps(O, _, H, V, q) {
    const G = O.width, J = O.height, Z = V || q, te = domCanvas(G, J).getContext("2d"), ne = domCanvas(G, J).getContext("2d"), ee = Z && domCanvas(G, J).getContext("2d");
    H.forEach((ve) => draw(te, ve, !1)), draw(ne, _, !1), Z && draw(ee, _, !0);
    const ae = getBuffer(te, G, J), oe = getBuffer(ne, G, J), se = Z && getBuffer(ee, G, J), ce = O.bitmap(), le = Z && O.bitmap();
    let ue, de, fe, he, me, pe, ye, ge;
    for (de = 0; de < J; ++de)
      for (ue = 0; ue < G; ++ue)
        me = de * G + ue, pe = ae[me] & ALPHA_MASK, ge = oe[me] & ALPHA_MASK, ye = Z && se[me] & ALPHA_MASK, (pe || ye || ge) && (fe = O(ue), he = O(de), !q && (pe || ge) && ce.set(fe, he), Z && (pe || ye) && le.set(fe, he));
    return [ce, le];
  }
  function getBuffer(O, _, H) {
    return new Uint32Array(O.getImageData(0, 0, _, H).data.buffer);
  }
  function draw(O, _, H) {
    if (!_.length) return;
    const V = _[0].mark.marktype;
    V === "group" ? _.forEach((q) => {
      q.items.forEach((G) => draw(O, G.items, H));
    }) : Marks[V].draw(O, {
      items: H ? _.map(prepare) : _
    });
  }
  function prepare(O) {
    const _ = rederive(O, {});
    return _.stroke && _.strokeOpacity !== 0 || _.fill && _.fillOpacity !== 0 ? {
      ..._,
      strokeOpacity: 1,
      stroke: "#000",
      fillOpacity: 0
    } : _;
  }
  const DIV = 5, MOD = 31, SIZE$1 = 32, RIGHT0 = new Uint32Array(SIZE$1 + 1), RIGHT1 = new Uint32Array(SIZE$1 + 1);
  RIGHT1[0] = 0;
  RIGHT0[0] = ~RIGHT1[0];
  for (let O = 1; O <= SIZE$1; ++O)
    RIGHT1[O] = RIGHT1[O - 1] << 1 | 1, RIGHT0[O] = ~RIGHT1[O];
  function Bitmap(O, _) {
    const H = new Uint32Array(~~((O * _ + SIZE$1) / SIZE$1));
    function V(G, J) {
      H[G] |= J;
    }
    function q(G, J) {
      H[G] &= J;
    }
    return {
      array: H,
      get: (G, J) => {
        const Z = J * O + G;
        return H[Z >>> DIV] & 1 << (Z & MOD);
      },
      set: (G, J) => {
        const Z = J * O + G;
        V(Z >>> DIV, 1 << (Z & MOD));
      },
      clear: (G, J) => {
        const Z = J * O + G;
        q(Z >>> DIV, ~(1 << (Z & MOD)));
      },
      getRange: (G, J, Z, te) => {
        let ne = te, ee, ae, oe, se;
        for (; ne >= J; --ne)
          if (ee = ne * O + G, ae = ne * O + Z, oe = ee >>> DIV, se = ae >>> DIV, oe === se) {
            if (H[oe] & RIGHT0[ee & MOD] & RIGHT1[(ae & MOD) + 1])
              return !0;
          } else {
            if (H[oe] & RIGHT0[ee & MOD] || H[se] & RIGHT1[(ae & MOD) + 1]) return !0;
            for (let ce = oe + 1; ce < se; ++ce)
              if (H[ce]) return !0;
          }
        return !1;
      },
      setRange: (G, J, Z, te) => {
        let ne, ee, ae, oe, se;
        for (; J <= te; ++J)
          if (ne = J * O + G, ee = J * O + Z, ae = ne >>> DIV, oe = ee >>> DIV, ae === oe)
            V(ae, RIGHT0[ne & MOD] & RIGHT1[(ee & MOD) + 1]);
          else
            for (V(ae, RIGHT0[ne & MOD]), V(oe, RIGHT1[(ee & MOD) + 1]), se = ae + 1; se < oe; ++se) V(se, 4294967295);
      },
      clearRange: (G, J, Z, te) => {
        let ne, ee, ae, oe, se;
        for (; J <= te; ++J)
          if (ne = J * O + G, ee = J * O + Z, ae = ne >>> DIV, oe = ee >>> DIV, ae === oe)
            q(ae, RIGHT1[ne & MOD] | RIGHT0[(ee & MOD) + 1]);
          else
            for (q(ae, RIGHT1[ne & MOD]), q(oe, RIGHT0[(ee & MOD) + 1]), se = ae + 1; se < oe; ++se) q(se, 0);
      },
      outOfBounds: (G, J, Z, te) => G < 0 || J < 0 || te >= _ || Z >= O
    };
  }
  function scaler(O, _, H) {
    const V = Math.max(1, Math.sqrt(O * _ / 1e6)), q = ~~((O + 2 * H + V) / V), G = ~~((_ + 2 * H + V) / V), J = (Z) => ~~((Z + H) / V);
    return J.invert = (Z) => Z * V - H, J.bitmap = () => Bitmap(q, G), J.ratio = V, J.padding = H, J.width = O, J.height = _, J;
  }
  function placeAreaLabelNaive(O, _, H, V) {
    const q = O.width, G = O.height;
    return function(J) {
      const Z = J.datum.datum.items[V].items, te = Z.length, ne = J.datum.fontSize, ee = textMetrics.width(J.datum, J.datum.text);
      let ae = 0, oe, se, ce, le, ue, de, fe;
      for (let he = 0; he < te; ++he)
        oe = Z[he].x, ce = Z[he].y, se = Z[he].x2 === void 0 ? oe : Z[he].x2, le = Z[he].y2 === void 0 ? ce : Z[he].y2, ue = (oe + se) / 2, de = (ce + le) / 2, fe = Math.abs(se - oe + le - ce), fe >= ae && (ae = fe, J.x = ue, J.y = de);
      return ue = ee / 2, de = ne / 2, oe = J.x - ue, se = J.x + ue, ce = J.y - de, le = J.y + de, J.align = "center", oe < 0 && se <= q ? J.align = "left" : 0 <= oe && q < se && (J.align = "right"), J.baseline = "middle", ce < 0 && le <= G ? J.baseline = "top" : 0 <= ce && G < le && (J.baseline = "bottom"), !0;
    };
  }
  function outOfBounds(O, _, H, V, q, G) {
    let J = H / 2;
    return O - J < 0 || O + J > q || _ - (J = V / 2) < 0 || _ + J > G;
  }
  function collision(O, _, H, V, q, G, J, Z) {
    const te = q * G / (V * 2), ne = O(_ - te), ee = O(_ + te), ae = O(H - (G = G / 2)), oe = O(H + G);
    return J.outOfBounds(ne, ae, ee, oe) || J.getRange(ne, ae, ee, oe) || Z && Z.getRange(ne, ae, ee, oe);
  }
  function placeAreaLabelReducedSearch(O, _, H, V) {
    const q = O.width, G = O.height, J = _[0], Z = _[1];
    function te(ne, ee, ae, oe, se) {
      const ce = O.invert(ne), le = O.invert(ee);
      let ue = ae, de = G, fe;
      if (!outOfBounds(ce, le, oe, se, q, G) && !collision(O, ce, le, se, oe, ue, J, Z) && !collision(O, ce, le, se, oe, se, J, null)) {
        for (; de - ue >= 1; )
          fe = (ue + de) / 2, collision(O, ce, le, se, oe, fe, J, Z) ? de = fe : ue = fe;
        if (ue > ae)
          return [ce, le, ue, !0];
      }
    }
    return function(ne) {
      const ee = ne.datum.datum.items[V].items, ae = ee.length, oe = ne.datum.fontSize, se = textMetrics.width(ne.datum, ne.datum.text);
      let ce = H ? oe : 0, le = !1, ue = !1, de = 0, fe, he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe, Le, Qe, ke, Me;
      for (let Re = 0; Re < ae; ++Re) {
        for (fe = ee[Re].x, me = ee[Re].y, he = ee[Re].x2 === void 0 ? fe : ee[Re].x2, pe = ee[Re].y2 === void 0 ? me : ee[Re].y2, fe > he && (Me = fe, fe = he, he = Me), me > pe && (Me = me, me = pe, pe = Me), Ce = O(fe), we = O(he), Se = ~~((Ce + we) / 2), Ee = O(me), Le = O(pe), xe = ~~((Ee + Le) / 2), ve = Se; ve >= Ce; --ve)
          for (be = xe; be >= Ee; --be)
            ke = te(ve, be, ce, se, oe), ke && ([ne.x, ne.y, ce, le] = ke);
        for (ve = Se; ve <= we; ++ve)
          for (be = xe; be <= Le; ++be)
            ke = te(ve, be, ce, se, oe), ke && ([ne.x, ne.y, ce, le] = ke);
        !le && !H && (Qe = Math.abs(he - fe + pe - me), ye = (fe + he) / 2, ge = (me + pe) / 2, Qe >= de && !outOfBounds(ye, ge, se, oe, q, G) && !collision(O, ye, ge, oe, se, oe, J, null) && (de = Qe, ne.x = ye, ne.y = ge, ue = !0));
      }
      return le || ue ? (ye = se / 2, ge = oe / 2, J.setRange(O(ne.x - ye), O(ne.y - ge), O(ne.x + ye), O(ne.y + ge)), ne.align = "center", ne.baseline = "middle", !0) : !1;
    };
  }
  const X_DIR = [-1, -1, 1, 1], Y_DIR = [-1, 1, -1, 1];
  function placeAreaLabelFloodFill(O, _, H, V) {
    const q = O.width, G = O.height, J = _[0], Z = _[1], te = O.bitmap();
    return function(ne) {
      const ee = ne.datum.datum.items[V].items, ae = ee.length, oe = ne.datum.fontSize, se = textMetrics.width(ne.datum, ne.datum.text), ce = [];
      let le = H ? oe : 0, ue = !1, de = !1, fe = 0, he, me, pe, ye, ge, ve, be, Ce, Se, we, Ee, xe;
      for (let Le = 0; Le < ae; ++Le) {
        for (he = ee[Le].x, pe = ee[Le].y, me = ee[Le].x2 === void 0 ? he : ee[Le].x2, ye = ee[Le].y2 === void 0 ? pe : ee[Le].y2, ce.push([O((he + me) / 2), O((pe + ye) / 2)]); ce.length; )
          if ([be, Ce] = ce.pop(), !(J.get(be, Ce) || Z.get(be, Ce) || te.get(be, Ce))) {
            te.set(be, Ce);
            for (let Qe = 0; Qe < 4; ++Qe)
              ge = be + X_DIR[Qe], ve = Ce + Y_DIR[Qe], te.outOfBounds(ge, ve, ge, ve) || ce.push([ge, ve]);
            if (ge = O.invert(be), ve = O.invert(Ce), Se = le, we = G, !outOfBounds(ge, ve, se, oe, q, G) && !collision(O, ge, ve, oe, se, Se, J, Z) && !collision(O, ge, ve, oe, se, oe, J, null)) {
              for (; we - Se >= 1; )
                Ee = (Se + we) / 2, collision(O, ge, ve, oe, se, Ee, J, Z) ? we = Ee : Se = Ee;
              Se > le && (ne.x = ge, ne.y = ve, le = Se, ue = !0);
            }
          }
        !ue && !H && (xe = Math.abs(me - he + ye - pe), ge = (he + me) / 2, ve = (pe + ye) / 2, xe >= fe && !outOfBounds(ge, ve, se, oe, q, G) && !collision(O, ge, ve, oe, se, oe, J, null) && (fe = xe, ne.x = ge, ne.y = ve, de = !0));
      }
      return ue || de ? (ge = se / 2, ve = oe / 2, J.setRange(O(ne.x - ge), O(ne.y - ve), O(ne.x + ge), O(ne.y + ve)), ne.align = "center", ne.baseline = "middle", !0) : !1;
    };
  }
  const Aligns = ["right", "center", "left"], Baselines = ["bottom", "middle", "top"];
  function placeMarkLabel(O, _, H, V) {
    const q = O.width, G = O.height, J = _[0], Z = _[1], te = V.length;
    return function(ne) {
      const ee = ne.boundary, ae = ne.datum.fontSize;
      if (ee[2] < 0 || ee[5] < 0 || ee[0] > q || ee[3] > G)
        return !1;
      let oe = ne.textWidth ?? 0, se, ce, le, ue, de, fe, he, me, pe, ye, ge, ve, be, Ce, Se;
      for (let we = 0; we < te; ++we) {
        if (se = (H[we] & 3) - 1, ce = (H[we] >>> 2 & 3) - 1, le = se === 0 && ce === 0 || V[we] < 0, ue = se && ce ? Math.SQRT1_2 : 1, de = V[we] < 0 ? -1 : 1, fe = ee[1 + se] + V[we] * se * ue, ge = ee[4 + ce] + de * ae * ce / 2 + V[we] * ce * ue, me = ge - ae / 2, pe = ge + ae / 2, ve = O(fe), Ce = O(me), Se = O(pe), !oe)
          if (test(ve, ve, Ce, Se, J, Z, fe, fe, me, pe, ee, le))
            oe = textMetrics.width(ne.datum, ne.datum.text);
          else
            continue;
        if (ye = fe + de * oe * se / 2, fe = ye - oe / 2, he = ye + oe / 2, ve = O(fe), be = O(he), test(ve, be, Ce, Se, J, Z, fe, he, me, pe, ee, le))
          return ne.x = se ? se * de < 0 ? he : fe : ye, ne.y = ce ? ce * de < 0 ? pe : me : ge, ne.align = Aligns[se * de + 1], ne.baseline = Baselines[ce * de + 1], J.setRange(ve, Ce, be, Se), !0;
      }
      return !1;
    };
  }
  function test(O, _, H, V, q, G, J, Z, te, ne, ee, ae) {
    return !(q.outOfBounds(O, H, _, V) || (ae && G || q).getRange(O, H, _, V));
  }
  const TOP = 0, MIDDLE = 4, BOTTOM = 8, LEFT = 0, CENTER$1 = 1, RIGHT = 2, anchorCode = {
    "top-left": TOP + LEFT,
    top: TOP + CENTER$1,
    "top-right": TOP + RIGHT,
    left: MIDDLE + LEFT,
    middle: MIDDLE + CENTER$1,
    right: MIDDLE + RIGHT,
    "bottom-left": BOTTOM + LEFT,
    bottom: BOTTOM + CENTER$1,
    "bottom-right": BOTTOM + RIGHT
  }, placeAreaLabel = {
    naive: placeAreaLabelNaive,
    "reduced-search": placeAreaLabelReducedSearch,
    floodfill: placeAreaLabelFloodFill
  };
  function labelLayout(O, _, H, V, q, G, J, Z, te, ne, ee) {
    if (!O.length) return O;
    const ae = Math.max(V.length, q.length), oe = getOffsets(V, ae), se = getAnchors(q, ae), ce = markType(O[0].datum), le = ce === "group" && O[0].datum.items[te].marktype, ue = le === "area", de = markBoundary(ce, le, Z, te), fe = ne === null || ne === 1 / 0, he = ue && ee === "naive";
    let me = -1, pe = -1;
    const ye = O.map((Ce) => {
      const Se = fe ? textMetrics.width(Ce, Ce.text) : void 0;
      return me = Math.max(me, Se), pe = Math.max(pe, Ce.fontSize), {
        datum: Ce,
        opacity: 0,
        x: void 0,
        y: void 0,
        align: void 0,
        baseline: void 0,
        boundary: de(Ce),
        textWidth: Se
      };
    });
    ne = ne === null || ne === 1 / 0 ? Math.max(me, pe) + Math.max(...V) : ne;
    const ge = scaler(_[0], _[1], ne);
    let ve;
    if (!he) {
      H && ye.sort((we, Ee) => H(we.datum, Ee.datum));
      let Ce = !1;
      for (let we = 0; we < se.length && !Ce; ++we)
        Ce = se[we] === 5 || oe[we] < 0;
      const Se = (ce && J || ue) && O.map((we) => we.datum);
      ve = G.length || Se ? markBitmaps(ge, Se || [], G, Ce, ue) : baseBitmaps(ge, J && ye);
    }
    const be = ue ? placeAreaLabel[ee](ge, ve, J, te) : placeMarkLabel(ge, ve, se, oe);
    return ye.forEach((Ce) => Ce.opacity = +be(Ce)), ye;
  }
  function getOffsets(O, _) {
    const H = new Float64Array(_), V = O.length;
    for (let q = 0; q < V; ++q) H[q] = O[q] || 0;
    for (let q = V; q < _; ++q) H[q] = H[V - 1];
    return H;
  }
  function getAnchors(O, _) {
    const H = new Int8Array(_), V = O.length;
    for (let q = 0; q < V; ++q) H[q] |= anchorCode[O[q]];
    for (let q = V; q < _; ++q) H[q] = H[V - 1];
    return H;
  }
  function markType(O) {
    return O && O.mark && O.mark.marktype;
  }
  function markBoundary(O, _, H, V) {
    const q = (G) => [G.x, G.x, G.x, G.y, G.y, G.y];
    return O ? O === "line" || O === "area" ? (G) => q(G.datum) : _ === "line" ? (G) => {
      const J = G.datum.items[V].items;
      return q(J.length ? J[H === "start" ? 0 : J.length - 1] : {
        x: NaN,
        y: NaN
      });
    } : (G) => {
      const J = G.datum.bounds;
      return [J.x1, (J.x1 + J.x2) / 2, J.x2, J.y1, (J.y1 + J.y2) / 2, J.y2];
    } : q;
  }
  const Output$1 = ["x", "y", "opacity", "align", "baseline"], Anchors = ["top-left", "left", "bottom-left", "top", "bottom", "top-right", "right", "bottom-right"];
  function Label$1(O) {
    Transform.call(this, null, O);
  }
  Label$1.Definition = {
    type: "Label",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "size",
      type: "number",
      array: !0,
      length: 2,
      required: !0
    }, {
      name: "sort",
      type: "compare"
    }, {
      name: "anchor",
      type: "string",
      array: !0,
      default: Anchors
    }, {
      name: "offset",
      type: "number",
      array: !0,
      default: [1]
    }, {
      name: "padding",
      type: "number",
      default: 0,
      null: !0
    }, {
      name: "lineAnchor",
      type: "string",
      values: ["start", "end"],
      default: "end"
    }, {
      name: "markIndex",
      type: "number",
      default: 0
    }, {
      name: "avoidBaseMark",
      type: "boolean",
      default: !0
    }, {
      name: "avoidMarks",
      type: "data",
      array: !0
    }, {
      name: "method",
      type: "string",
      default: "naive"
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: Output$1.length,
      default: Output$1
    }]
  };
  inherits(Label$1, Transform, {
    transform(O, _) {
      function H(G) {
        const J = O[G];
        return isFunction$2(J) && _.modified(J.fields);
      }
      const V = O.modified();
      if (!(V || _.changed(_.ADD_REM) || H("sort"))) return;
      (!O.size || O.size.length !== 2) && error$1("Size parameter should be specified as a [width, height] array.");
      const q = O.as || Output$1;
      return labelLayout(_.materialize(_.SOURCE).source || [], O.size, O.sort, array$5(O.offset == null ? 1 : O.offset), array$5(O.anchor || Anchors), O.avoidMarks || [], O.avoidBaseMark !== !1, O.lineAnchor || "end", O.markIndex || 0, O.padding === void 0 ? 0 : O.padding, O.method || "naive").forEach((G) => {
        const J = G.datum;
        J[q[0]] = G.x, J[q[1]] = G.y, J[q[2]] = G.opacity, J[q[3]] = G.align, J[q[4]] = G.baseline;
      }), _.reflow(V).modifies(q);
    }
  });
  const label = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    label: Label$1
  }, Symbol.toStringTag, { value: "Module" }));
  function partition(O, _) {
    var H = [], V = function(ee) {
      return ee(Z);
    }, q, G, J, Z, te, ne;
    if (_ == null)
      H.push(O);
    else
      for (q = {}, G = 0, J = O.length; G < J; ++G)
        Z = O[G], te = _.map(V), ne = q[te], ne || (q[te] = ne = [], ne.dims = te, H.push(ne)), ne.push(Z);
    return H;
  }
  function Loess(O) {
    Transform.call(this, null, O);
  }
  Loess.Definition = {
    type: "Loess",
    metadata: {
      generates: !0
    },
    params: [{
      name: "x",
      type: "field",
      required: !0
    }, {
      name: "y",
      type: "field",
      required: !0
    }, {
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "bandwidth",
      type: "number",
      default: 0.3
    }, {
      name: "as",
      type: "string",
      array: !0
    }]
  };
  inherits(Loess, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE | _.NO_FIELDS);
      if (!this.value || _.changed() || O.modified()) {
        const V = _.materialize(_.SOURCE).source, q = partition(V, O.groupby), G = (O.groupby || []).map(accessorName), J = G.length, Z = O.as || [accessorName(O.x), accessorName(O.y)], te = [];
        q.forEach((ne) => {
          loess(ne, O.x, O.y, O.bandwidth || 0.3).forEach((ee) => {
            const ae = {};
            for (let oe = 0; oe < J; ++oe)
              ae[G[oe]] = ne.dims[oe];
            ae[Z[0]] = ee[0], ae[Z[1]] = ee[1], te.push(ingest$1(ae));
          });
        }), this.value && (H.rem = this.value), this.value = H.add = H.source = te;
      }
      return H;
    }
  });
  const Methods = {
    constant: constant$4,
    linear: linear$2,
    log: log$3,
    exp: exp$1,
    pow: pow$3,
    quad,
    poly
  }, degreesOfFreedom = (O, _) => O === "poly" ? _ : O === "quad" ? 2 : 1;
  function Regression(O) {
    Transform.call(this, null, O);
  }
  Regression.Definition = {
    type: "Regression",
    metadata: {
      generates: !0
    },
    params: [{
      name: "x",
      type: "field",
      required: !0
    }, {
      name: "y",
      type: "field",
      required: !0
    }, {
      name: "groupby",
      type: "field",
      array: !0
    }, {
      name: "method",
      type: "string",
      default: "linear",
      values: Object.keys(Methods)
    }, {
      name: "order",
      type: "number",
      default: 3
    }, {
      name: "extent",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "params",
      type: "boolean",
      default: !1
    }, {
      name: "as",
      type: "string",
      array: !0
    }]
  };
  inherits(Regression, Transform, {
    transform(O, _) {
      const H = _.fork(_.NO_SOURCE | _.NO_FIELDS);
      if (!this.value || _.changed() || O.modified()) {
        const V = _.materialize(_.SOURCE).source, q = partition(V, O.groupby), G = (O.groupby || []).map(accessorName), J = O.method || "linear", Z = O.order == null ? 3 : O.order, te = degreesOfFreedom(J, Z), ne = O.as || [accessorName(O.x), accessorName(O.y)], ee = Methods[J], ae = [];
        let oe = O.extent;
        has$1(Methods, J) || error$1("Invalid regression method: " + J), oe != null && J === "log" && oe[0] <= 0 && (_.dataflow.warn("Ignoring extent with values <= 0 for log regression."), oe = null), q.forEach((se) => {
          if (se.length <= te) {
            _.dataflow.warn("Skipping regression with more parameters than data points.");
            return;
          }
          const le = ee(se, O.x, O.y, Z);
          if (O.params) {
            ae.push(ingest$1({
              keys: se.dims,
              coef: le.coef,
              rSquared: le.rSquared
            }));
            return;
          }
          const ue = oe || extent(se, O.x), de = (fe) => {
            const he = {};
            for (let me = 0; me < G.length; ++me)
              he[G[me]] = se.dims[me];
            he[ne[0]] = fe[0], he[ne[1]] = fe[1], ae.push(ingest$1(he));
          };
          J === "linear" || J === "constant" ? ue.forEach((fe) => de([fe, le.predict(fe)])) : sampleCurve(le.predict, ue, 25, 200).forEach(de);
        }), this.value && (H.rem = this.value), this.value = H.add = H.source = ae;
      }
      return H;
    }
  });
  const reg = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    loess: Loess,
    regression: Regression
  }, Symbol.toStringTag, { value: "Module" })), epsilon$1 = 11102230246251565e-32, splitter = 134217729, resulterrbound = (3 + 8 * epsilon$1) * epsilon$1;
  function sum(O, _, H, V, q) {
    let G, J, Z, te, ne = _[0], ee = V[0], ae = 0, oe = 0;
    ee > ne == ee > -ne ? (G = ne, ne = _[++ae]) : (G = ee, ee = V[++oe]);
    let se = 0;
    if (ae < O && oe < H)
      for (ee > ne == ee > -ne ? (J = ne + G, Z = G - (J - ne), ne = _[++ae]) : (J = ee + G, Z = G - (J - ee), ee = V[++oe]), G = J, Z !== 0 && (q[se++] = Z); ae < O && oe < H; )
        ee > ne == ee > -ne ? (J = G + ne, te = J - G, Z = G - (J - te) + (ne - te), ne = _[++ae]) : (J = G + ee, te = J - G, Z = G - (J - te) + (ee - te), ee = V[++oe]), G = J, Z !== 0 && (q[se++] = Z);
    for (; ae < O; )
      J = G + ne, te = J - G, Z = G - (J - te) + (ne - te), ne = _[++ae], G = J, Z !== 0 && (q[se++] = Z);
    for (; oe < H; )
      J = G + ee, te = J - G, Z = G - (J - te) + (ee - te), ee = V[++oe], G = J, Z !== 0 && (q[se++] = Z);
    return (G !== 0 || se === 0) && (q[se++] = G), se;
  }
  function estimate(O, _) {
    let H = _[0];
    for (let V = 1; V < O; V++) H += _[V];
    return H;
  }
  function vec(O) {
    return new Float64Array(O);
  }
  const ccwerrboundA = (3 + 16 * epsilon$1) * epsilon$1, ccwerrboundB = (2 + 12 * epsilon$1) * epsilon$1, ccwerrboundC = (9 + 64 * epsilon$1) * epsilon$1 * epsilon$1, B = vec(4), C1 = vec(8), C2 = vec(12), D = vec(16), u = vec(4);
  function orient2dadapt(O, _, H, V, q, G, J) {
    let Z, te, ne, ee, ae, oe, se, ce, le, ue, de, fe, he, me, pe, ye, ge, ve;
    const be = O - q, Ce = H - q, Se = _ - G, we = V - G;
    me = be * we, oe = splitter * be, se = oe - (oe - be), ce = be - se, oe = splitter * we, le = oe - (oe - we), ue = we - le, pe = ce * ue - (me - se * le - ce * le - se * ue), ye = Se * Ce, oe = splitter * Se, se = oe - (oe - Se), ce = Se - se, oe = splitter * Ce, le = oe - (oe - Ce), ue = Ce - le, ge = ce * ue - (ye - se * le - ce * le - se * ue), de = pe - ge, ae = pe - de, B[0] = pe - (de + ae) + (ae - ge), fe = me + de, ae = fe - me, he = me - (fe - ae) + (de - ae), de = he - ye, ae = he - de, B[1] = he - (de + ae) + (ae - ye), ve = fe + de, ae = ve - fe, B[2] = fe - (ve - ae) + (de - ae), B[3] = ve;
    let Ee = estimate(4, B), xe = ccwerrboundB * J;
    if (Ee >= xe || -Ee >= xe || (ae = O - be, Z = O - (be + ae) + (ae - q), ae = H - Ce, ne = H - (Ce + ae) + (ae - q), ae = _ - Se, te = _ - (Se + ae) + (ae - G), ae = V - we, ee = V - (we + ae) + (ae - G), Z === 0 && te === 0 && ne === 0 && ee === 0) || (xe = ccwerrboundC * J + resulterrbound * Math.abs(Ee), Ee += be * ee + we * Z - (Se * ne + Ce * te), Ee >= xe || -Ee >= xe)) return Ee;
    me = Z * we, oe = splitter * Z, se = oe - (oe - Z), ce = Z - se, oe = splitter * we, le = oe - (oe - we), ue = we - le, pe = ce * ue - (me - se * le - ce * le - se * ue), ye = te * Ce, oe = splitter * te, se = oe - (oe - te), ce = te - se, oe = splitter * Ce, le = oe - (oe - Ce), ue = Ce - le, ge = ce * ue - (ye - se * le - ce * le - se * ue), de = pe - ge, ae = pe - de, u[0] = pe - (de + ae) + (ae - ge), fe = me + de, ae = fe - me, he = me - (fe - ae) + (de - ae), de = he - ye, ae = he - de, u[1] = he - (de + ae) + (ae - ye), ve = fe + de, ae = ve - fe, u[2] = fe - (ve - ae) + (de - ae), u[3] = ve;
    const Le = sum(4, B, 4, u, C1);
    me = be * ee, oe = splitter * be, se = oe - (oe - be), ce = be - se, oe = splitter * ee, le = oe - (oe - ee), ue = ee - le, pe = ce * ue - (me - se * le - ce * le - se * ue), ye = Se * ne, oe = splitter * Se, se = oe - (oe - Se), ce = Se - se, oe = splitter * ne, le = oe - (oe - ne), ue = ne - le, ge = ce * ue - (ye - se * le - ce * le - se * ue), de = pe - ge, ae = pe - de, u[0] = pe - (de + ae) + (ae - ge), fe = me + de, ae = fe - me, he = me - (fe - ae) + (de - ae), de = he - ye, ae = he - de, u[1] = he - (de + ae) + (ae - ye), ve = fe + de, ae = ve - fe, u[2] = fe - (ve - ae) + (de - ae), u[3] = ve;
    const Qe = sum(Le, C1, 4, u, C2);
    me = Z * ee, oe = splitter * Z, se = oe - (oe - Z), ce = Z - se, oe = splitter * ee, le = oe - (oe - ee), ue = ee - le, pe = ce * ue - (me - se * le - ce * le - se * ue), ye = te * ne, oe = splitter * te, se = oe - (oe - te), ce = te - se, oe = splitter * ne, le = oe - (oe - ne), ue = ne - le, ge = ce * ue - (ye - se * le - ce * le - se * ue), de = pe - ge, ae = pe - de, u[0] = pe - (de + ae) + (ae - ge), fe = me + de, ae = fe - me, he = me - (fe - ae) + (de - ae), de = he - ye, ae = he - de, u[1] = he - (de + ae) + (ae - ye), ve = fe + de, ae = ve - fe, u[2] = fe - (ve - ae) + (de - ae), u[3] = ve;
    const ke = sum(Qe, C2, 4, u, D);
    return D[ke - 1];
  }
  function orient2d(O, _, H, V, q, G) {
    const J = (_ - G) * (H - q), Z = (O - q) * (V - G), te = J - Z, ne = Math.abs(J + Z);
    return Math.abs(te) >= ccwerrboundA * ne ? te : -orient2dadapt(O, _, H, V, q, G, ne);
  }
  const EPSILON = Math.pow(2, -52), EDGE_STACK = new Uint32Array(512);
  class Delaunator {
    static from(_, H = defaultGetX, V = defaultGetY) {
      const q = _.length, G = new Float64Array(q * 2);
      for (let J = 0; J < q; J++) {
        const Z = _[J];
        G[2 * J] = H(Z), G[2 * J + 1] = V(Z);
      }
      return new Delaunator(G);
    }
    constructor(_) {
      const H = _.length >> 1;
      if (H > 0 && typeof _[0] != "number") throw new Error("Expected coords to contain numbers.");
      this.coords = _;
      const V = Math.max(2 * H - 5, 0);
      this._triangles = new Uint32Array(V * 3), this._halfedges = new Int32Array(V * 3), this._hashSize = Math.ceil(Math.sqrt(H)), this._hullPrev = new Uint32Array(H), this._hullNext = new Uint32Array(H), this._hullTri = new Uint32Array(H), this._hullHash = new Int32Array(this._hashSize), this._ids = new Uint32Array(H), this._dists = new Float64Array(H), this.update();
    }
    update() {
      const { coords: _, _hullPrev: H, _hullNext: V, _hullTri: q, _hullHash: G } = this, J = _.length >> 1;
      let Z = 1 / 0, te = 1 / 0, ne = -1 / 0, ee = -1 / 0;
      for (let be = 0; be < J; be++) {
        const Ce = _[2 * be], Se = _[2 * be + 1];
        Ce < Z && (Z = Ce), Se < te && (te = Se), Ce > ne && (ne = Ce), Se > ee && (ee = Se), this._ids[be] = be;
      }
      const ae = (Z + ne) / 2, oe = (te + ee) / 2;
      let se, ce, le;
      for (let be = 0, Ce = 1 / 0; be < J; be++) {
        const Se = dist$1(ae, oe, _[2 * be], _[2 * be + 1]);
        Se < Ce && (se = be, Ce = Se);
      }
      const ue = _[2 * se], de = _[2 * se + 1];
      for (let be = 0, Ce = 1 / 0; be < J; be++) {
        if (be === se) continue;
        const Se = dist$1(ue, de, _[2 * be], _[2 * be + 1]);
        Se < Ce && Se > 0 && (ce = be, Ce = Se);
      }
      let fe = _[2 * ce], he = _[2 * ce + 1], me = 1 / 0;
      for (let be = 0; be < J; be++) {
        if (be === se || be === ce) continue;
        const Ce = circumradius(ue, de, fe, he, _[2 * be], _[2 * be + 1]);
        Ce < me && (le = be, me = Ce);
      }
      let pe = _[2 * le], ye = _[2 * le + 1];
      if (me === 1 / 0) {
        for (let Se = 0; Se < J; Se++)
          this._dists[Se] = _[2 * Se] - _[0] || _[2 * Se + 1] - _[1];
        quicksort(this._ids, this._dists, 0, J - 1);
        const be = new Uint32Array(J);
        let Ce = 0;
        for (let Se = 0, we = -1 / 0; Se < J; Se++) {
          const Ee = this._ids[Se], xe = this._dists[Ee];
          xe > we && (be[Ce++] = Ee, we = xe);
        }
        this.hull = be.subarray(0, Ce), this.triangles = new Uint32Array(0), this.halfedges = new Uint32Array(0);
        return;
      }
      if (orient2d(ue, de, fe, he, pe, ye) < 0) {
        const be = ce, Ce = fe, Se = he;
        ce = le, fe = pe, he = ye, le = be, pe = Ce, ye = Se;
      }
      const ge = circumcenter(ue, de, fe, he, pe, ye);
      this._cx = ge.x, this._cy = ge.y;
      for (let be = 0; be < J; be++)
        this._dists[be] = dist$1(_[2 * be], _[2 * be + 1], ge.x, ge.y);
      quicksort(this._ids, this._dists, 0, J - 1), this._hullStart = se;
      let ve = 3;
      V[se] = H[le] = ce, V[ce] = H[se] = le, V[le] = H[ce] = se, q[se] = 0, q[ce] = 1, q[le] = 2, G.fill(-1), G[this._hashKey(ue, de)] = se, G[this._hashKey(fe, he)] = ce, G[this._hashKey(pe, ye)] = le, this.trianglesLen = 0, this._addTriangle(se, ce, le, -1, -1, -1);
      for (let be = 0, Ce, Se; be < this._ids.length; be++) {
        const we = this._ids[be], Ee = _[2 * we], xe = _[2 * we + 1];
        if (be > 0 && Math.abs(Ee - Ce) <= EPSILON && Math.abs(xe - Se) <= EPSILON || (Ce = Ee, Se = xe, we === se || we === ce || we === le)) continue;
        let Le = 0;
        for (let De = 0, Pe = this._hashKey(Ee, xe); De < this._hashSize && (Le = G[(Pe + De) % this._hashSize], !(Le !== -1 && Le !== V[Le])); De++)
          ;
        Le = H[Le];
        let Qe = Le, ke;
        for (; ke = V[Qe], orient2d(Ee, xe, _[2 * Qe], _[2 * Qe + 1], _[2 * ke], _[2 * ke + 1]) >= 0; )
          if (Qe = ke, Qe === Le) {
            Qe = -1;
            break;
          }
        if (Qe === -1) continue;
        let Me = this._addTriangle(Qe, we, V[Qe], -1, -1, q[Qe]);
        q[we] = this._legalize(Me + 2), q[Qe] = Me, ve++;
        let Re = V[Qe];
        for (; ke = V[Re], orient2d(Ee, xe, _[2 * Re], _[2 * Re + 1], _[2 * ke], _[2 * ke + 1]) < 0; )
          Me = this._addTriangle(Re, we, ke, q[we], -1, q[Re]), q[we] = this._legalize(Me + 2), V[Re] = Re, ve--, Re = ke;
        if (Qe === Le)
          for (; ke = H[Qe], orient2d(Ee, xe, _[2 * ke], _[2 * ke + 1], _[2 * Qe], _[2 * Qe + 1]) < 0; )
            Me = this._addTriangle(ke, we, Qe, -1, q[Qe], q[ke]), this._legalize(Me + 2), q[ke] = Me, V[Qe] = Qe, ve--, Qe = ke;
        this._hullStart = H[we] = Qe, V[Qe] = H[Re] = we, V[we] = Re, G[this._hashKey(Ee, xe)] = we, G[this._hashKey(_[2 * Qe], _[2 * Qe + 1])] = Qe;
      }
      this.hull = new Uint32Array(ve);
      for (let be = 0, Ce = this._hullStart; be < ve; be++)
        this.hull[be] = Ce, Ce = V[Ce];
      this.triangles = this._triangles.subarray(0, this.trianglesLen), this.halfedges = this._halfedges.subarray(0, this.trianglesLen);
    }
    _hashKey(_, H) {
      return Math.floor(pseudoAngle(_ - this._cx, H - this._cy) * this._hashSize) % this._hashSize;
    }
    _legalize(_) {
      const { _triangles: H, _halfedges: V, coords: q } = this;
      let G = 0, J = 0;
      for (; ; ) {
        const Z = V[_], te = _ - _ % 3;
        if (J = te + (_ + 2) % 3, Z === -1) {
          if (G === 0) break;
          _ = EDGE_STACK[--G];
          continue;
        }
        const ne = Z - Z % 3, ee = te + (_ + 1) % 3, ae = ne + (Z + 2) % 3, oe = H[J], se = H[_], ce = H[ee], le = H[ae];
        if (inCircle(
          q[2 * oe],
          q[2 * oe + 1],
          q[2 * se],
          q[2 * se + 1],
          q[2 * ce],
          q[2 * ce + 1],
          q[2 * le],
          q[2 * le + 1]
        )) {
          H[_] = le, H[Z] = oe;
          const de = V[ae];
          if (de === -1) {
            let he = this._hullStart;
            do {
              if (this._hullTri[he] === ae) {
                this._hullTri[he] = _;
                break;
              }
              he = this._hullPrev[he];
            } while (he !== this._hullStart);
          }
          this._link(_, de), this._link(Z, V[J]), this._link(J, ae);
          const fe = ne + (Z + 1) % 3;
          G < EDGE_STACK.length && (EDGE_STACK[G++] = fe);
        } else {
          if (G === 0) break;
          _ = EDGE_STACK[--G];
        }
      }
      return J;
    }
    _link(_, H) {
      this._halfedges[_] = H, H !== -1 && (this._halfedges[H] = _);
    }
    // add a new triangle given vertex indices and adjacent half-edge ids
    _addTriangle(_, H, V, q, G, J) {
      const Z = this.trianglesLen;
      return this._triangles[Z] = _, this._triangles[Z + 1] = H, this._triangles[Z + 2] = V, this._link(Z, q), this._link(Z + 1, G), this._link(Z + 2, J), this.trianglesLen += 3, Z;
    }
  }
  function pseudoAngle(O, _) {
    const H = O / (Math.abs(O) + Math.abs(_));
    return (_ > 0 ? 3 - H : 1 + H) / 4;
  }
  function dist$1(O, _, H, V) {
    const q = O - H, G = _ - V;
    return q * q + G * G;
  }
  function inCircle(O, _, H, V, q, G, J, Z) {
    const te = O - J, ne = _ - Z, ee = H - J, ae = V - Z, oe = q - J, se = G - Z, ce = te * te + ne * ne, le = ee * ee + ae * ae, ue = oe * oe + se * se;
    return te * (ae * ue - le * se) - ne * (ee * ue - le * oe) + ce * (ee * se - ae * oe) < 0;
  }
  function circumradius(O, _, H, V, q, G) {
    const J = H - O, Z = V - _, te = q - O, ne = G - _, ee = J * J + Z * Z, ae = te * te + ne * ne, oe = 0.5 / (J * ne - Z * te), se = (ne * ee - Z * ae) * oe, ce = (J * ae - te * ee) * oe;
    return se * se + ce * ce;
  }
  function circumcenter(O, _, H, V, q, G) {
    const J = H - O, Z = V - _, te = q - O, ne = G - _, ee = J * J + Z * Z, ae = te * te + ne * ne, oe = 0.5 / (J * ne - Z * te), se = O + (ne * ee - Z * ae) * oe, ce = _ + (J * ae - te * ee) * oe;
    return { x: se, y: ce };
  }
  function quicksort(O, _, H, V) {
    if (V - H <= 20)
      for (let q = H + 1; q <= V; q++) {
        const G = O[q], J = _[G];
        let Z = q - 1;
        for (; Z >= H && _[O[Z]] > J; ) O[Z + 1] = O[Z--];
        O[Z + 1] = G;
      }
    else {
      const q = H + V >> 1;
      let G = H + 1, J = V;
      swap(O, q, G), _[O[H]] > _[O[V]] && swap(O, H, V), _[O[G]] > _[O[V]] && swap(O, G, V), _[O[H]] > _[O[G]] && swap(O, H, G);
      const Z = O[G], te = _[Z];
      for (; ; ) {
        do
          G++;
        while (_[O[G]] < te);
        do
          J--;
        while (_[O[J]] > te);
        if (J < G) break;
        swap(O, G, J);
      }
      O[H + 1] = O[J], O[J] = Z, V - G + 1 >= J - H ? (quicksort(O, _, G, V), quicksort(O, _, H, J - 1)) : (quicksort(O, _, H, J - 1), quicksort(O, _, G, V));
    }
  }
  function swap(O, _, H) {
    const V = O[_];
    O[_] = O[H], O[H] = V;
  }
  function defaultGetX(O) {
    return O[0];
  }
  function defaultGetY(O) {
    return O[1];
  }
  const epsilon = 1e-6;
  class Path {
    constructor() {
      this._x0 = this._y0 = // start of current subpath
      this._x1 = this._y1 = null, this._ = "";
    }
    moveTo(_, H) {
      this._ += `M${this._x0 = this._x1 = +_},${this._y0 = this._y1 = +H}`;
    }
    closePath() {
      this._x1 !== null && (this._x1 = this._x0, this._y1 = this._y0, this._ += "Z");
    }
    lineTo(_, H) {
      this._ += `L${this._x1 = +_},${this._y1 = +H}`;
    }
    arc(_, H, V) {
      _ = +_, H = +H, V = +V;
      const q = _ + V, G = H;
      if (V < 0) throw new Error("negative radius");
      this._x1 === null ? this._ += `M${q},${G}` : (Math.abs(this._x1 - q) > epsilon || Math.abs(this._y1 - G) > epsilon) && (this._ += "L" + q + "," + G), V && (this._ += `A${V},${V},0,1,1,${_ - V},${H}A${V},${V},0,1,1,${this._x1 = q},${this._y1 = G}`);
    }
    rect(_, H, V, q) {
      this._ += `M${this._x0 = this._x1 = +_},${this._y0 = this._y1 = +H}h${+V}v${+q}h${-V}Z`;
    }
    value() {
      return this._ || null;
    }
  }
  class Polygon {
    constructor() {
      this._ = [];
    }
    moveTo(_, H) {
      this._.push([_, H]);
    }
    closePath() {
      this._.push(this._[0].slice());
    }
    lineTo(_, H) {
      this._.push([_, H]);
    }
    value() {
      return this._.length ? this._ : null;
    }
  }
  let Voronoi$1 = class {
    constructor(_, [H, V, q, G] = [0, 0, 960, 500]) {
      if (!((q = +q) >= (H = +H)) || !((G = +G) >= (V = +V))) throw new Error("invalid bounds");
      this.delaunay = _, this._circumcenters = new Float64Array(_.points.length * 2), this.vectors = new Float64Array(_.points.length * 2), this.xmax = q, this.xmin = H, this.ymax = G, this.ymin = V, this._init();
    }
    update() {
      return this.delaunay.update(), this._init(), this;
    }
    _init() {
      const { delaunay: { points: _, hull: H, triangles: V }, vectors: q } = this;
      let G, J;
      const Z = this.circumcenters = this._circumcenters.subarray(0, V.length / 3 * 2);
      for (let le = 0, ue = 0, de = V.length, fe, he; le < de; le += 3, ue += 2) {
        const me = V[le] * 2, pe = V[le + 1] * 2, ye = V[le + 2] * 2, ge = _[me], ve = _[me + 1], be = _[pe], Ce = _[pe + 1], Se = _[ye], we = _[ye + 1], Ee = be - ge, xe = Ce - ve, Le = Se - ge, Qe = we - ve, ke = (Ee * Qe - xe * Le) * 2;
        if (Math.abs(ke) < 1e-9) {
          if (G === void 0) {
            G = J = 0;
            for (const Re of H) G += _[Re * 2], J += _[Re * 2 + 1];
            G /= H.length, J /= H.length;
          }
          const Me = 1e9 * Math.sign((G - ge) * Qe - (J - ve) * Le);
          fe = (ge + Se) / 2 - Me * Qe, he = (ve + we) / 2 + Me * Le;
        } else {
          const Me = 1 / ke, Re = Ee * Ee + xe * xe, De = Le * Le + Qe * Qe;
          fe = ge + (Qe * Re - xe * De) * Me, he = ve + (Ee * De - Le * Re) * Me;
        }
        Z[ue] = fe, Z[ue + 1] = he;
      }
      let te = H[H.length - 1], ne, ee = te * 4, ae, oe = _[2 * te], se, ce = _[2 * te + 1];
      q.fill(0);
      for (let le = 0; le < H.length; ++le)
        te = H[le], ne = ee, ae = oe, se = ce, ee = te * 4, oe = _[2 * te], ce = _[2 * te + 1], q[ne + 2] = q[ee] = se - ce, q[ne + 3] = q[ee + 1] = oe - ae;
    }
    render(_) {
      const H = _ == null ? _ = new Path() : void 0, { delaunay: { halfedges: V, inedges: q, hull: G }, circumcenters: J, vectors: Z } = this;
      if (G.length <= 1) return null;
      for (let ee = 0, ae = V.length; ee < ae; ++ee) {
        const oe = V[ee];
        if (oe < ee) continue;
        const se = Math.floor(ee / 3) * 2, ce = Math.floor(oe / 3) * 2, le = J[se], ue = J[se + 1], de = J[ce], fe = J[ce + 1];
        this._renderSegment(le, ue, de, fe, _);
      }
      let te, ne = G[G.length - 1];
      for (let ee = 0; ee < G.length; ++ee) {
        te = ne, ne = G[ee];
        const ae = Math.floor(q[ne] / 3) * 2, oe = J[ae], se = J[ae + 1], ce = te * 4, le = this._project(oe, se, Z[ce + 2], Z[ce + 3]);
        le && this._renderSegment(oe, se, le[0], le[1], _);
      }
      return H && H.value();
    }
    renderBounds(_) {
      const H = _ == null ? _ = new Path() : void 0;
      return _.rect(this.xmin, this.ymin, this.xmax - this.xmin, this.ymax - this.ymin), H && H.value();
    }
    renderCell(_, H) {
      const V = H == null ? H = new Path() : void 0, q = this._clip(_);
      if (q === null || !q.length) return;
      H.moveTo(q[0], q[1]);
      let G = q.length;
      for (; q[0] === q[G - 2] && q[1] === q[G - 1] && G > 1; ) G -= 2;
      for (let J = 2; J < G; J += 2)
        (q[J] !== q[J - 2] || q[J + 1] !== q[J - 1]) && H.lineTo(q[J], q[J + 1]);
      return H.closePath(), V && V.value();
    }
    *cellPolygons() {
      const { delaunay: { points: _ } } = this;
      for (let H = 0, V = _.length / 2; H < V; ++H) {
        const q = this.cellPolygon(H);
        q && (q.index = H, yield q);
      }
    }
    cellPolygon(_) {
      const H = new Polygon();
      return this.renderCell(_, H), H.value();
    }
    _renderSegment(_, H, V, q, G) {
      let J;
      const Z = this._regioncode(_, H), te = this._regioncode(V, q);
      Z === 0 && te === 0 ? (G.moveTo(_, H), G.lineTo(V, q)) : (J = this._clipSegment(_, H, V, q, Z, te)) && (G.moveTo(J[0], J[1]), G.lineTo(J[2], J[3]));
    }
    contains(_, H, V) {
      return H = +H, H !== H || (V = +V, V !== V) ? !1 : this.delaunay._step(_, H, V) === _;
    }
    *neighbors(_) {
      const H = this._clip(_);
      if (H) for (const V of this.delaunay.neighbors(_)) {
        const q = this._clip(V);
        if (q) {
          e: for (let G = 0, J = H.length; G < J; G += 2)
            for (let Z = 0, te = q.length; Z < te; Z += 2)
              if (H[G] === q[Z] && H[G + 1] === q[Z + 1] && H[(G + 2) % J] === q[(Z + te - 2) % te] && H[(G + 3) % J] === q[(Z + te - 1) % te]) {
                yield V;
                break e;
              }
        }
      }
    }
    _cell(_) {
      const { circumcenters: H, delaunay: { inedges: V, halfedges: q, triangles: G } } = this, J = V[_];
      if (J === -1) return null;
      const Z = [];
      let te = J;
      do {
        const ne = Math.floor(te / 3);
        if (Z.push(H[ne * 2], H[ne * 2 + 1]), te = te % 3 === 2 ? te - 2 : te + 1, G[te] !== _) break;
        te = q[te];
      } while (te !== J && te !== -1);
      return Z;
    }
    _clip(_) {
      if (_ === 0 && this.delaunay.hull.length === 1)
        return [this.xmax, this.ymin, this.xmax, this.ymax, this.xmin, this.ymax, this.xmin, this.ymin];
      const H = this._cell(_);
      if (H === null) return null;
      const { vectors: V } = this, q = _ * 4;
      return this._simplify(V[q] || V[q + 1] ? this._clipInfinite(_, H, V[q], V[q + 1], V[q + 2], V[q + 3]) : this._clipFinite(_, H));
    }
    _clipFinite(_, H) {
      const V = H.length;
      let q = null, G, J, Z = H[V - 2], te = H[V - 1], ne, ee = this._regioncode(Z, te), ae, oe = 0;
      for (let se = 0; se < V; se += 2)
        if (G = Z, J = te, Z = H[se], te = H[se + 1], ne = ee, ee = this._regioncode(Z, te), ne === 0 && ee === 0)
          ae = oe, oe = 0, q ? q.push(Z, te) : q = [Z, te];
        else {
          let ce, le, ue, de, fe;
          if (ne === 0) {
            if ((ce = this._clipSegment(G, J, Z, te, ne, ee)) === null) continue;
            [le, ue, de, fe] = ce;
          } else {
            if ((ce = this._clipSegment(Z, te, G, J, ee, ne)) === null) continue;
            [de, fe, le, ue] = ce, ae = oe, oe = this._edgecode(le, ue), ae && oe && this._edge(_, ae, oe, q, q.length), q ? q.push(le, ue) : q = [le, ue];
          }
          ae = oe, oe = this._edgecode(de, fe), ae && oe && this._edge(_, ae, oe, q, q.length), q ? q.push(de, fe) : q = [de, fe];
        }
      if (q)
        ae = oe, oe = this._edgecode(q[0], q[1]), ae && oe && this._edge(_, ae, oe, q, q.length);
      else if (this.contains(_, (this.xmin + this.xmax) / 2, (this.ymin + this.ymax) / 2))
        return [this.xmax, this.ymin, this.xmax, this.ymax, this.xmin, this.ymax, this.xmin, this.ymin];
      return q;
    }
    _clipSegment(_, H, V, q, G, J) {
      const Z = G < J;
      for (Z && ([_, H, V, q, G, J] = [V, q, _, H, J, G]); ; ) {
        if (G === 0 && J === 0) return Z ? [V, q, _, H] : [_, H, V, q];
        if (G & J) return null;
        let te, ne, ee = G || J;
        ee & 8 ? (te = _ + (V - _) * (this.ymax - H) / (q - H), ne = this.ymax) : ee & 4 ? (te = _ + (V - _) * (this.ymin - H) / (q - H), ne = this.ymin) : ee & 2 ? (ne = H + (q - H) * (this.xmax - _) / (V - _), te = this.xmax) : (ne = H + (q - H) * (this.xmin - _) / (V - _), te = this.xmin), G ? (_ = te, H = ne, G = this._regioncode(_, H)) : (V = te, q = ne, J = this._regioncode(V, q));
      }
    }
    _clipInfinite(_, H, V, q, G, J) {
      let Z = Array.from(H), te;
      if ((te = this._project(Z[0], Z[1], V, q)) && Z.unshift(te[0], te[1]), (te = this._project(Z[Z.length - 2], Z[Z.length - 1], G, J)) && Z.push(te[0], te[1]), Z = this._clipFinite(_, Z))
        for (let ne = 0, ee = Z.length, ae, oe = this._edgecode(Z[ee - 2], Z[ee - 1]); ne < ee; ne += 2)
          ae = oe, oe = this._edgecode(Z[ne], Z[ne + 1]), ae && oe && (ne = this._edge(_, ae, oe, Z, ne), ee = Z.length);
      else this.contains(_, (this.xmin + this.xmax) / 2, (this.ymin + this.ymax) / 2) && (Z = [this.xmin, this.ymin, this.xmax, this.ymin, this.xmax, this.ymax, this.xmin, this.ymax]);
      return Z;
    }
    _edge(_, H, V, q, G) {
      for (; H !== V; ) {
        let J, Z;
        switch (H) {
          case 5:
            H = 4;
            continue;
          // top-left
          case 4:
            H = 6, J = this.xmax, Z = this.ymin;
            break;
          // top
          case 6:
            H = 2;
            continue;
          // top-right
          case 2:
            H = 10, J = this.xmax, Z = this.ymax;
            break;
          // right
          case 10:
            H = 8;
            continue;
          // bottom-right
          case 8:
            H = 9, J = this.xmin, Z = this.ymax;
            break;
          // bottom
          case 9:
            H = 1;
            continue;
          // bottom-left
          case 1:
            H = 5, J = this.xmin, Z = this.ymin;
            break;
        }
        (q[G] !== J || q[G + 1] !== Z) && this.contains(_, J, Z) && (q.splice(G, 0, J, Z), G += 2);
      }
      return G;
    }
    _project(_, H, V, q) {
      let G = 1 / 0, J, Z, te;
      if (q < 0) {
        if (H <= this.ymin) return null;
        (J = (this.ymin - H) / q) < G && (te = this.ymin, Z = _ + (G = J) * V);
      } else if (q > 0) {
        if (H >= this.ymax) return null;
        (J = (this.ymax - H) / q) < G && (te = this.ymax, Z = _ + (G = J) * V);
      }
      if (V > 0) {
        if (_ >= this.xmax) return null;
        (J = (this.xmax - _) / V) < G && (Z = this.xmax, te = H + (G = J) * q);
      } else if (V < 0) {
        if (_ <= this.xmin) return null;
        (J = (this.xmin - _) / V) < G && (Z = this.xmin, te = H + (G = J) * q);
      }
      return [Z, te];
    }
    _edgecode(_, H) {
      return (_ === this.xmin ? 1 : _ === this.xmax ? 2 : 0) | (H === this.ymin ? 4 : H === this.ymax ? 8 : 0);
    }
    _regioncode(_, H) {
      return (_ < this.xmin ? 1 : _ > this.xmax ? 2 : 0) | (H < this.ymin ? 4 : H > this.ymax ? 8 : 0);
    }
    _simplify(_) {
      if (_ && _.length > 4) {
        for (let H = 0; H < _.length; H += 2) {
          const V = (H + 2) % _.length, q = (H + 4) % _.length;
          (_[H] === _[V] && _[V] === _[q] || _[H + 1] === _[V + 1] && _[V + 1] === _[q + 1]) && (_.splice(V, 2), H -= 2);
        }
        _.length || (_ = null);
      }
      return _;
    }
  };
  const tau = 2 * Math.PI, pow = Math.pow;
  function pointX(O) {
    return O[0];
  }
  function pointY(O) {
    return O[1];
  }
  function collinear(O) {
    const { triangles: _, coords: H } = O;
    for (let V = 0; V < _.length; V += 3) {
      const q = 2 * _[V], G = 2 * _[V + 1], J = 2 * _[V + 2];
      if ((H[J] - H[q]) * (H[G + 1] - H[q + 1]) - (H[G] - H[q]) * (H[J + 1] - H[q + 1]) > 1e-10) return !1;
    }
    return !0;
  }
  function jitter(O, _, H) {
    return [O + Math.sin(O + _) * H, _ + Math.cos(O - _) * H];
  }
  class Delaunay {
    static from(_, H = pointX, V = pointY, q) {
      return new Delaunay("length" in _ ? flatArray(_, H, V, q) : Float64Array.from(flatIterable(_, H, V, q)));
    }
    constructor(_) {
      this._delaunator = new Delaunator(_), this.inedges = new Int32Array(_.length / 2), this._hullIndex = new Int32Array(_.length / 2), this.points = this._delaunator.coords, this._init();
    }
    update() {
      return this._delaunator.update(), this._init(), this;
    }
    _init() {
      const _ = this._delaunator, H = this.points;
      if (_.hull && _.hull.length > 2 && collinear(_)) {
        this.collinear = Int32Array.from({ length: H.length / 2 }, (oe, se) => se).sort((oe, se) => H[2 * oe] - H[2 * se] || H[2 * oe + 1] - H[2 * se + 1]);
        const te = this.collinear[0], ne = this.collinear[this.collinear.length - 1], ee = [H[2 * te], H[2 * te + 1], H[2 * ne], H[2 * ne + 1]], ae = 1e-8 * Math.hypot(ee[3] - ee[1], ee[2] - ee[0]);
        for (let oe = 0, se = H.length / 2; oe < se; ++oe) {
          const ce = jitter(H[2 * oe], H[2 * oe + 1], ae);
          H[2 * oe] = ce[0], H[2 * oe + 1] = ce[1];
        }
        this._delaunator = new Delaunator(H);
      } else
        delete this.collinear;
      const V = this.halfedges = this._delaunator.halfedges, q = this.hull = this._delaunator.hull, G = this.triangles = this._delaunator.triangles, J = this.inedges.fill(-1), Z = this._hullIndex.fill(-1);
      for (let te = 0, ne = V.length; te < ne; ++te) {
        const ee = G[te % 3 === 2 ? te - 2 : te + 1];
        (V[te] === -1 || J[ee] === -1) && (J[ee] = te);
      }
      for (let te = 0, ne = q.length; te < ne; ++te)
        Z[q[te]] = te;
      q.length <= 2 && q.length > 0 && (this.triangles = new Int32Array(3).fill(-1), this.halfedges = new Int32Array(3).fill(-1), this.triangles[0] = q[0], J[q[0]] = 1, q.length === 2 && (J[q[1]] = 0, this.triangles[1] = q[1], this.triangles[2] = q[1]));
    }
    voronoi(_) {
      return new Voronoi$1(this, _);
    }
    *neighbors(_) {
      const { inedges: H, hull: V, _hullIndex: q, halfedges: G, triangles: J, collinear: Z } = this;
      if (Z) {
        const ae = Z.indexOf(_);
        ae > 0 && (yield Z[ae - 1]), ae < Z.length - 1 && (yield Z[ae + 1]);
        return;
      }
      const te = H[_];
      if (te === -1) return;
      let ne = te, ee = -1;
      do {
        if (yield ee = J[ne], ne = ne % 3 === 2 ? ne - 2 : ne + 1, J[ne] !== _) return;
        if (ne = G[ne], ne === -1) {
          const ae = V[(q[_] + 1) % V.length];
          ae !== ee && (yield ae);
          return;
        }
      } while (ne !== te);
    }
    find(_, H, V = 0) {
      if (_ = +_, _ !== _ || (H = +H, H !== H)) return -1;
      const q = V;
      let G;
      for (; (G = this._step(V, _, H)) >= 0 && G !== V && G !== q; ) V = G;
      return G;
    }
    _step(_, H, V) {
      const { inedges: q, hull: G, _hullIndex: J, halfedges: Z, triangles: te, points: ne } = this;
      if (q[_] === -1 || !ne.length) return (_ + 1) % (ne.length >> 1);
      let ee = _, ae = pow(H - ne[_ * 2], 2) + pow(V - ne[_ * 2 + 1], 2);
      const oe = q[_];
      let se = oe;
      do {
        let ce = te[se];
        const le = pow(H - ne[ce * 2], 2) + pow(V - ne[ce * 2 + 1], 2);
        if (le < ae && (ae = le, ee = ce), se = se % 3 === 2 ? se - 2 : se + 1, te[se] !== _) break;
        if (se = Z[se], se === -1) {
          if (se = G[(J[_] + 1) % G.length], se !== ce && pow(H - ne[se * 2], 2) + pow(V - ne[se * 2 + 1], 2) < ae)
            return se;
          break;
        }
      } while (se !== oe);
      return ee;
    }
    render(_) {
      const H = _ == null ? _ = new Path() : void 0, { points: V, halfedges: q, triangles: G } = this;
      for (let J = 0, Z = q.length; J < Z; ++J) {
        const te = q[J];
        if (te < J) continue;
        const ne = G[J] * 2, ee = G[te] * 2;
        _.moveTo(V[ne], V[ne + 1]), _.lineTo(V[ee], V[ee + 1]);
      }
      return this.renderHull(_), H && H.value();
    }
    renderPoints(_, H) {
      H === void 0 && (!_ || typeof _.moveTo != "function") && (H = _, _ = null), H = H == null ? 2 : +H;
      const V = _ == null ? _ = new Path() : void 0, { points: q } = this;
      for (let G = 0, J = q.length; G < J; G += 2) {
        const Z = q[G], te = q[G + 1];
        _.moveTo(Z + H, te), _.arc(Z, te, H, 0, tau);
      }
      return V && V.value();
    }
    renderHull(_) {
      const H = _ == null ? _ = new Path() : void 0, { hull: V, points: q } = this, G = V[0] * 2, J = V.length;
      _.moveTo(q[G], q[G + 1]);
      for (let Z = 1; Z < J; ++Z) {
        const te = 2 * V[Z];
        _.lineTo(q[te], q[te + 1]);
      }
      return _.closePath(), H && H.value();
    }
    hullPolygon() {
      const _ = new Polygon();
      return this.renderHull(_), _.value();
    }
    renderTriangle(_, H) {
      const V = H == null ? H = new Path() : void 0, { points: q, triangles: G } = this, J = G[_ *= 3] * 2, Z = G[_ + 1] * 2, te = G[_ + 2] * 2;
      return H.moveTo(q[J], q[J + 1]), H.lineTo(q[Z], q[Z + 1]), H.lineTo(q[te], q[te + 1]), H.closePath(), V && V.value();
    }
    *trianglePolygons() {
      const { triangles: _ } = this;
      for (let H = 0, V = _.length / 3; H < V; ++H)
        yield this.trianglePolygon(H);
    }
    trianglePolygon(_) {
      const H = new Polygon();
      return this.renderTriangle(_, H), H.value();
    }
  }
  function flatArray(O, _, H, V) {
    const q = O.length, G = new Float64Array(q * 2);
    for (let J = 0; J < q; ++J) {
      const Z = O[J];
      G[J * 2] = _.call(V, Z, J, O), G[J * 2 + 1] = H.call(V, Z, J, O);
    }
    return G;
  }
  function* flatIterable(O, _, H, V) {
    let q = 0;
    for (const G of O)
      yield _.call(V, G, q, O), yield H.call(V, G, q, O), ++q;
  }
  function Voronoi(O) {
    Transform.call(this, null, O);
  }
  Voronoi.Definition = {
    type: "Voronoi",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "x",
      type: "field",
      required: !0
    }, {
      name: "y",
      type: "field",
      required: !0
    }, {
      name: "size",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "extent",
      type: "array",
      array: !0,
      length: 2,
      default: [[-1e5, -1e5], [1e5, 1e5]],
      content: {
        type: "number",
        array: !0,
        length: 2
      }
    }, {
      name: "as",
      type: "string",
      default: "path"
    }]
  };
  const defaultExtent = [-1e5, -1e5, 1e5, 1e5];
  inherits(Voronoi, Transform, {
    transform(O, _) {
      const H = O.as || "path", V = _.source;
      if (!V || !V.length) return _;
      let q = O.size;
      q = q ? [0, 0, q[0], q[1]] : (q = O.extent) ? [q[0][0], q[0][1], q[1][0], q[1][1]] : defaultExtent;
      const G = this.value = Delaunay.from(V, O.x, O.y).voronoi(q);
      for (let J = 0, Z = V.length; J < Z; ++J) {
        const te = G.cellPolygon(J);
        V[J][H] = te && !isPoint(te) ? toPathString(te) : null;
      }
      return _.reflow(O.modified()).modifies(H);
    }
  });
  function toPathString(O) {
    const _ = O[0][0], H = O[0][1];
    let V = O.length - 1;
    for (; O[V][0] === _ && O[V][1] === H; --V) ;
    return "M" + O.slice(0, V + 1).join("L") + "Z";
  }
  function isPoint(O) {
    return O.length === 2 && O[0][0] === O[1][0] && O[0][1] === O[1][1];
  }
  const voronoi = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    voronoi: Voronoi
  }, Symbol.toStringTag, { value: "Module" }));
  var cloudRadians = Math.PI / 180, cw = 64, ch = 2048;
  function cloud() {
    var O = [256, 256], _, H, V, q, G, J, Z, te = archimedeanSpiral, ne = [], ee = Math.random, ae = {};
    ae.layout = function() {
      for (var ce = oe(domCanvas()), le = zeroArray((O[0] >> 5) * O[1]), ue = null, de = ne.length, fe = -1, he = [], me = ne.map((ye) => ({
        text: _(ye),
        font: H(ye),
        style: q(ye),
        weight: G(ye),
        rotate: J(ye),
        size: ~~(V(ye) + 1e-14),
        padding: Z(ye),
        xoff: 0,
        yoff: 0,
        x1: 0,
        y1: 0,
        x0: 0,
        y0: 0,
        hasText: !1,
        sprite: null,
        datum: ye
      })).sort((ye, ge) => ge.size - ye.size); ++fe < de; ) {
        var pe = me[fe];
        pe.x = O[0] * (ee() + 0.5) >> 1, pe.y = O[1] * (ee() + 0.5) >> 1, cloudSprite(ce, pe, me, fe), pe.hasText && se(le, pe, ue) && (he.push(pe), ue ? cloudBounds(ue, pe) : ue = [{
          x: pe.x + pe.x0,
          y: pe.y + pe.y0
        }, {
          x: pe.x + pe.x1,
          y: pe.y + pe.y1
        }], pe.x -= O[0] >> 1, pe.y -= O[1] >> 1);
      }
      return he;
    };
    function oe(ce) {
      ce.width = ce.height = 1;
      var le = Math.sqrt(ce.getContext("2d").getImageData(0, 0, 1, 1).data.length >> 2);
      ce.width = (cw << 5) / le, ce.height = ch / le;
      var ue = ce.getContext("2d");
      return ue.fillStyle = ue.strokeStyle = "red", ue.textAlign = "center", {
        context: ue,
        ratio: le
      };
    }
    function se(ce, le, ue) {
      for (var de = le.x, fe = le.y, he = Math.hypot(O[0], O[1]), me = te(O), pe = ee() < 0.5 ? 1 : -1, ye = -pe, ge, ve, be; (ge = me(ye += pe)) && (ve = ~~ge[0], be = ~~ge[1], !(Math.min(Math.abs(ve), Math.abs(be)) >= he)); )
        if (le.x = de + ve, le.y = fe + be, !(le.x + le.x0 < 0 || le.y + le.y0 < 0 || le.x + le.x1 > O[0] || le.y + le.y1 > O[1]) && (!ue || !cloudCollide(le, ce, O[0])) && (!ue || collideRects(le, ue))) {
          for (var Ce = le.sprite, Se = le.width >> 5, we = O[0] >> 5, Ee = le.x - (Se << 4), xe = Ee & 127, Le = 32 - xe, Qe = le.y1 - le.y0, ke = (le.y + le.y0) * we + (Ee >> 5), Me, Re = 0; Re < Qe; Re++) {
            Me = 0;
            for (var De = 0; De <= Se; De++)
              ce[ke + De] |= Me << Le | (De < Se ? (Me = Ce[Re * Se + De]) >>> xe : 0);
            ke += we;
          }
          return le.sprite = null, !0;
        }
      return !1;
    }
    return ae.words = function(ce) {
      return arguments.length ? (ne = ce, ae) : ne;
    }, ae.size = function(ce) {
      return arguments.length ? (O = [+ce[0], +ce[1]], ae) : O;
    }, ae.font = function(ce) {
      return arguments.length ? (H = functor(ce), ae) : H;
    }, ae.fontStyle = function(ce) {
      return arguments.length ? (q = functor(ce), ae) : q;
    }, ae.fontWeight = function(ce) {
      return arguments.length ? (G = functor(ce), ae) : G;
    }, ae.rotate = function(ce) {
      return arguments.length ? (J = functor(ce), ae) : J;
    }, ae.text = function(ce) {
      return arguments.length ? (_ = functor(ce), ae) : _;
    }, ae.spiral = function(ce) {
      return arguments.length ? (te = spirals[ce] || ce, ae) : te;
    }, ae.fontSize = function(ce) {
      return arguments.length ? (V = functor(ce), ae) : V;
    }, ae.padding = function(ce) {
      return arguments.length ? (Z = functor(ce), ae) : Z;
    }, ae.random = function(ce) {
      return arguments.length ? (ee = ce, ae) : ee;
    }, ae;
  }
  function cloudSprite(O, _, H, V) {
    if (!_.sprite) {
      var q = O.context, G = O.ratio;
      q.clearRect(0, 0, (cw << 5) / G, ch / G);
      var J = 0, Z = 0, te = 0, ne = H.length, ee, ae, oe, se, ce;
      for (--V; ++V < ne; ) {
        if (_ = H[V], q.save(), q.font = _.style + " " + _.weight + " " + ~~((_.size + 1) / G) + "px " + _.font, ee = q.measureText(_.text + "m").width * G, oe = _.size << 1, _.rotate) {
          var le = Math.sin(_.rotate * cloudRadians), ue = Math.cos(_.rotate * cloudRadians), de = ee * ue, fe = ee * le, he = oe * ue, me = oe * le;
          ee = Math.max(Math.abs(de + me), Math.abs(de - me)) + 31 >> 5 << 5, oe = ~~Math.max(Math.abs(fe + he), Math.abs(fe - he));
        } else
          ee = ee + 31 >> 5 << 5;
        if (oe > te && (te = oe), J + ee >= cw << 5 && (J = 0, Z += te, te = 0), Z + oe >= ch) break;
        q.translate((J + (ee >> 1)) / G, (Z + (oe >> 1)) / G), _.rotate && q.rotate(_.rotate * cloudRadians), q.fillText(_.text, 0, 0), _.padding && (q.lineWidth = 2 * _.padding, q.strokeText(_.text, 0, 0)), q.restore(), _.width = ee, _.height = oe, _.xoff = J, _.yoff = Z, _.x1 = ee >> 1, _.y1 = oe >> 1, _.x0 = -_.x1, _.y0 = -_.y1, _.hasText = !0, J += ee;
      }
      for (var pe = q.getImageData(0, 0, (cw << 5) / G, ch / G).data, ye = []; --V >= 0; )
        if (_ = H[V], !!_.hasText) {
          for (ee = _.width, ae = ee >> 5, oe = _.y1 - _.y0, se = 0; se < oe * ae; se++) ye[se] = 0;
          if (J = _.xoff, J == null) return;
          Z = _.yoff;
          var ge = 0, ve = -1;
          for (ce = 0; ce < oe; ce++) {
            for (se = 0; se < ee; se++) {
              var be = ae * ce + (se >> 5), Ce = pe[(Z + ce) * (cw << 5) + (J + se) << 2] ? 1 << 31 - se % 32 : 0;
              ye[be] |= Ce, ge |= Ce;
            }
            ge ? ve = ce : (_.y0++, oe--, ce--, Z++);
          }
          _.y1 = _.y0 + ve, _.sprite = ye.slice(0, (_.y1 - _.y0) * ae);
        }
    }
  }
  function cloudCollide(O, _, H) {
    H >>= 5;
    for (var V = O.sprite, q = O.width >> 5, G = O.x - (q << 4), J = G & 127, Z = 32 - J, te = O.y1 - O.y0, ne = (O.y + O.y0) * H + (G >> 5), ee, ae = 0; ae < te; ae++) {
      ee = 0;
      for (var oe = 0; oe <= q; oe++)
        if ((ee << Z | (oe < q ? (ee = V[ae * q + oe]) >>> J : 0)) & _[ne + oe]) return !0;
      ne += H;
    }
    return !1;
  }
  function cloudBounds(O, _) {
    var H = O[0], V = O[1];
    _.x + _.x0 < H.x && (H.x = _.x + _.x0), _.y + _.y0 < H.y && (H.y = _.y + _.y0), _.x + _.x1 > V.x && (V.x = _.x + _.x1), _.y + _.y1 > V.y && (V.y = _.y + _.y1);
  }
  function collideRects(O, _) {
    return O.x + O.x1 > _[0].x && O.x + O.x0 < _[1].x && O.y + O.y1 > _[0].y && O.y + O.y0 < _[1].y;
  }
  function archimedeanSpiral(O) {
    var _ = O[0] / O[1];
    return function(H) {
      return [_ * (H *= 0.1) * Math.cos(H), H * Math.sin(H)];
    };
  }
  function rectangularSpiral(O) {
    var _ = 4, H = _ * O[0] / O[1], V = 0, q = 0;
    return function(G) {
      var J = G < 0 ? -1 : 1;
      switch (Math.sqrt(1 + 4 * J * G) - J & 3) {
        case 0:
          V += H;
          break;
        case 1:
          q += _;
          break;
        case 2:
          V -= H;
          break;
        default:
          q -= _;
          break;
      }
      return [V, q];
    };
  }
  function zeroArray(O) {
    for (var _ = [], H = -1; ++H < O; ) _[H] = 0;
    return _;
  }
  function functor(O) {
    return typeof O == "function" ? O : function() {
      return O;
    };
  }
  var spirals = {
    archimedean: archimedeanSpiral,
    rectangular: rectangularSpiral
  };
  const Output = ["x", "y", "font", "fontSize", "fontStyle", "fontWeight", "angle"], Params$1 = ["text", "font", "rotate", "fontSize", "fontStyle", "fontWeight"];
  function Wordcloud(O) {
    Transform.call(this, cloud(), O);
  }
  Wordcloud.Definition = {
    type: "Wordcloud",
    metadata: {
      modifies: !0
    },
    params: [{
      name: "size",
      type: "number",
      array: !0,
      length: 2
    }, {
      name: "font",
      type: "string",
      expr: !0,
      default: "sans-serif"
    }, {
      name: "fontStyle",
      type: "string",
      expr: !0,
      default: "normal"
    }, {
      name: "fontWeight",
      type: "string",
      expr: !0,
      default: "normal"
    }, {
      name: "fontSize",
      type: "number",
      expr: !0,
      default: 14
    }, {
      name: "fontSizeRange",
      type: "number",
      array: "nullable",
      default: [10, 50]
    }, {
      name: "rotate",
      type: "number",
      expr: !0,
      default: 0
    }, {
      name: "text",
      type: "field"
    }, {
      name: "spiral",
      type: "string",
      values: ["archimedean", "rectangular"]
    }, {
      name: "padding",
      type: "number",
      expr: !0
    }, {
      name: "as",
      type: "string",
      array: !0,
      length: 7,
      default: Output
    }]
  };
  inherits(Wordcloud, Transform, {
    transform(O, _) {
      O.size && !(O.size[0] && O.size[1]) && error$1("Wordcloud size dimensions must be non-zero.");
      function H(ce) {
        const le = O[ce];
        return isFunction$2(le) && _.modified(le.fields);
      }
      const V = O.modified();
      if (!(V || _.changed(_.ADD_REM) || Params$1.some(H))) return;
      const q = _.materialize(_.SOURCE).source, G = this.value, J = O.as || Output;
      let Z = O.fontSize || 14, te;
      if (isFunction$2(Z) ? te = O.fontSizeRange : Z = constant$5(Z), te) {
        const ce = Z, le = scale$4("sqrt")().domain(extent(q, ce)).range(te);
        Z = (ue) => le(ce(ue));
      }
      q.forEach((ce) => {
        ce[J[0]] = NaN, ce[J[1]] = NaN, ce[J[3]] = 0;
      });
      const ne = G.words(q).text(O.text).size(O.size || [500, 500]).padding(O.padding || 1).spiral(O.spiral || "archimedean").rotate(O.rotate || 0).font(O.font || "sans-serif").fontStyle(O.fontStyle || "normal").fontWeight(O.fontWeight || "normal").fontSize(Z).random(random).layout(), ee = G.size(), ae = ee[0] >> 1, oe = ee[1] >> 1, se = ne.length;
      for (let ce = 0, le, ue; ce < se; ++ce)
        le = ne[ce], ue = le.datum, ue[J[0]] = le.x + ae, ue[J[1]] = le.y + oe, ue[J[2]] = le.font, ue[J[3]] = le.size, ue[J[4]] = le.style, ue[J[5]] = le.weight, ue[J[6]] = le.rotate;
      return _.reflow(V).modifies(J);
    }
  });
  const wordcloud = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    wordcloud: Wordcloud
  }, Symbol.toStringTag, { value: "Module" })), array8 = (O) => new Uint8Array(O), array16 = (O) => new Uint16Array(O), array32 = (O) => new Uint32Array(O);
  function Bitmaps() {
    let O = 8, _ = [], H = array32(0), V = array$1(0, O), q = array$1(0, O);
    return {
      data: () => _,
      seen: () => H = lengthen(H, _.length),
      add(G) {
        for (let J = 0, Z = _.length, te = G.length, ne; J < te; ++J)
          ne = G[J], ne._index = Z++, _.push(ne);
      },
      remove(G, J) {
        const Z = _.length, te = Array(Z - G), ne = _;
        let ee, ae, oe;
        for (ae = 0; !J[ae] && ae < Z; ++ae)
          te[ae] = _[ae], ne[ae] = ae;
        for (oe = ae; ae < Z; ++ae)
          ee = _[ae], J[ae] ? ne[ae] = -1 : (ne[ae] = oe, V[oe] = V[ae], q[oe] = q[ae], te[oe] = ee, ee._index = oe++), V[ae] = 0;
        return _ = te, ne;
      },
      size: () => _.length,
      curr: () => V,
      prev: () => q,
      reset: (G) => q[G] = V[G],
      all: () => O < 257 ? 255 : O < 65537 ? 65535 : 4294967295,
      set(G, J) {
        V[G] |= J;
      },
      clear(G, J) {
        V[G] &= ~J;
      },
      resize(G, J) {
        const Z = V.length;
        (G > Z || J > O) && (O = Math.max(J, O), V = array$1(G, O, V), q = array$1(G, O));
      }
    };
  }
  function lengthen(O, _, H) {
    return O.length >= _ ? O : (H = H || new O.constructor(_), H.set(O), H);
  }
  function array$1(O, _, H) {
    const V = (_ < 257 ? array8 : _ < 65537 ? array16 : array32)(O);
    return H && V.set(H), V;
  }
  function Dimension(O, _, H) {
    const V = 1 << _;
    return {
      one: V,
      zero: ~V,
      range: H.slice(),
      bisect: O.bisect,
      index: O.index,
      size: O.size,
      onAdd(q, G) {
        const J = this, Z = J.bisect(J.range, q.value), te = q.index, ne = Z[0], ee = Z[1], ae = te.length;
        let oe;
        for (oe = 0; oe < ne; ++oe) G[te[oe]] |= V;
        for (oe = ee; oe < ae; ++oe) G[te[oe]] |= V;
        return J;
      }
    };
  }
  function SortedIndex() {
    let O = array32(0), _ = [], H = 0;
    function V(Z, te, ne) {
      if (!te.length) return [];
      const ee = H, ae = te.length, oe = array32(ae);
      let se = Array(ae), ce, le, ue;
      for (ue = 0; ue < ae; ++ue)
        se[ue] = Z(te[ue]), oe[ue] = ue;
      if (se = sort(se, oe), ee)
        ce = _, le = O, _ = Array(ee + ae), O = array32(ee + ae), merge$1(ne, ce, le, ee, se, oe, ae, _, O);
      else {
        if (ne > 0) for (ue = 0; ue < ae; ++ue)
          oe[ue] += ne;
        _ = se, O = oe;
      }
      return H = ee + ae, {
        index: oe,
        value: se
      };
    }
    function q(Z, te) {
      const ne = H;
      let ee, ae, oe;
      for (ae = 0; !te[O[ae]] && ae < ne; ++ae) ;
      for (oe = ae; ae < ne; ++ae)
        te[ee = O[ae]] || (O[oe] = ee, _[oe] = _[ae], ++oe);
      H = ne - Z;
    }
    function G(Z) {
      for (let te = 0, ne = H; te < ne; ++te)
        O[te] = Z[O[te]];
    }
    function J(Z, te) {
      let ne;
      return te ? ne = te.length : (te = _, ne = H), [bisectLeft$1(te, Z[0], 0, ne), bisectRight$1(te, Z[1], 0, ne)];
    }
    return {
      insert: V,
      remove: q,
      bisect: J,
      reindex: G,
      index: () => O,
      size: () => H
    };
  }
  function sort(O, _) {
    return O.sort.call(_, (H, V) => {
      const q = O[H], G = O[V];
      return q < G ? -1 : q > G ? 1 : 0;
    }), permute(O, _);
  }
  function merge$1(O, _, H, V, q, G, J, Z, te) {
    let ne = 0, ee = 0, ae;
    for (ae = 0; ne < V && ee < J; ++ae)
      _[ne] < q[ee] ? (Z[ae] = _[ne], te[ae] = H[ne++]) : (Z[ae] = q[ee], te[ae] = G[ee++] + O);
    for (; ne < V; ++ne, ++ae)
      Z[ae] = _[ne], te[ae] = H[ne];
    for (; ee < J; ++ee, ++ae)
      Z[ae] = q[ee], te[ae] = G[ee] + O;
  }
  function CrossFilter(O) {
    Transform.call(this, Bitmaps(), O), this._indices = null, this._dims = null;
  }
  CrossFilter.Definition = {
    type: "CrossFilter",
    metadata: {},
    params: [{
      name: "fields",
      type: "field",
      array: !0,
      required: !0
    }, {
      name: "query",
      type: "array",
      array: !0,
      required: !0,
      content: {
        type: "number",
        array: !0,
        length: 2
      }
    }]
  };
  inherits(CrossFilter, Transform, {
    transform(O, _) {
      if (this._dims) {
        var H = O.modified("fields") || O.fields.some((V) => _.modified(V.fields));
        return H ? this.reinit(O, _) : this.eval(O, _);
      } else
        return this.init(O, _);
    },
    init(O, _) {
      const H = O.fields, V = O.query, q = this._indices = {}, G = this._dims = [], J = V.length;
      let Z = 0, te, ne;
      for (; Z < J; ++Z)
        te = H[Z].fname, ne = q[te] || (q[te] = SortedIndex()), G.push(Dimension(ne, Z, V[Z]));
      return this.eval(O, _);
    },
    reinit(O, _) {
      const H = _.materialize().fork(), V = O.fields, q = O.query, G = this._indices, J = this._dims, Z = this.value, te = Z.curr(), ne = Z.prev(), ee = Z.all(), ae = H.rem = H.add, oe = H.mod, se = q.length, ce = {};
      let le, ue, de, fe, he, me, pe, ye, ge;
      if (ne.set(te), _.rem.length && (he = this.remove(O, _, H)), _.add.length && Z.add(_.add), _.mod.length)
        for (me = {}, fe = _.mod, pe = 0, ye = fe.length; pe < ye; ++pe)
          me[fe[pe]._index] = 1;
      for (pe = 0; pe < se; ++pe)
        ge = V[pe], (!J[pe] || O.modified("fields", pe) || _.modified(ge.fields)) && (de = ge.fname, (le = ce[de]) || (G[de] = ue = SortedIndex(), ce[de] = le = ue.insert(ge, _.source, 0)), J[pe] = Dimension(ue, pe, q[pe]).onAdd(le, te));
      for (pe = 0, ye = Z.data().length; pe < ye; ++pe)
        he[pe] || (ne[pe] !== te[pe] ? ae.push(pe) : me[pe] && te[pe] !== ee && oe.push(pe));
      return Z.mask = (1 << se) - 1, H;
    },
    eval(O, _) {
      const H = _.materialize().fork(), V = this._dims.length;
      let q = 0;
      return _.rem.length && (this.remove(O, _, H), q |= (1 << V) - 1), O.modified("query") && !O.modified("fields") && (q |= this.update(O, _, H)), _.add.length && (this.insert(O, _, H), q |= (1 << V) - 1), _.mod.length && (this.modify(_, H), q |= (1 << V) - 1), this.value.mask = q, H;
    },
    insert(O, _, H) {
      const V = _.add, q = this.value, G = this._dims, J = this._indices, Z = O.fields, te = {}, ne = H.add, ee = q.size() + V.length, ae = G.length;
      let oe = q.size(), se, ce, le;
      q.resize(ee, ae), q.add(V);
      const ue = q.curr(), de = q.prev(), fe = q.all();
      for (se = 0; se < ae; ++se)
        ce = Z[se].fname, le = te[ce] || (te[ce] = J[ce].insert(Z[se], V, oe)), G[se].onAdd(le, ue);
      for (; oe < ee; ++oe)
        de[oe] = fe, ue[oe] !== fe && ne.push(oe);
    },
    modify(O, _) {
      const H = _.mod, V = this.value, q = V.curr(), G = V.all(), J = O.mod;
      let Z, te, ne;
      for (Z = 0, te = J.length; Z < te; ++Z)
        ne = J[Z]._index, q[ne] !== G && H.push(ne);
    },
    remove(O, _, H) {
      const V = this._indices, q = this.value, G = q.curr(), J = q.prev(), Z = q.all(), te = {}, ne = H.rem, ee = _.rem;
      let ae, oe, se, ce;
      for (ae = 0, oe = ee.length; ae < oe; ++ae)
        se = ee[ae]._index, te[se] = 1, J[se] = ce = G[se], G[se] = Z, ce !== Z && ne.push(se);
      for (se in V)
        V[se].remove(oe, te);
      return this.reindex(_, oe, te), te;
    },
    // reindex filters and indices after propagation completes
    reindex(O, _, H) {
      const V = this._indices, q = this.value;
      O.runAfter(() => {
        const G = q.remove(_, H);
        for (const J in V) V[J].reindex(G);
      });
    },
    update(O, _, H) {
      const V = this._dims, q = O.query, G = _.stamp, J = V.length;
      let Z = 0, te, ne;
      for (H.filters = 0, ne = 0; ne < J; ++ne)
        O.modified("query", ne) && (te = ne, ++Z);
      if (Z === 1)
        Z = V[te].one, this.incrementOne(V[te], q[te], H.add, H.rem);
      else
        for (ne = 0, Z = 0; ne < J; ++ne)
          O.modified("query", ne) && (Z |= V[ne].one, this.incrementAll(V[ne], q[ne], G, H.add), H.rem = H.add);
      return Z;
    },
    incrementAll(O, _, H, V) {
      const q = this.value, G = q.seen(), J = q.curr(), Z = q.prev(), te = O.index(), ne = O.bisect(O.range), ee = O.bisect(_), ae = ee[0], oe = ee[1], se = ne[0], ce = ne[1], le = O.one;
      let ue, de, fe;
      if (ae < se)
        for (ue = ae, de = Math.min(se, oe); ue < de; ++ue)
          fe = te[ue], G[fe] !== H && (Z[fe] = J[fe], G[fe] = H, V.push(fe)), J[fe] ^= le;
      else if (ae > se)
        for (ue = se, de = Math.min(ae, ce); ue < de; ++ue)
          fe = te[ue], G[fe] !== H && (Z[fe] = J[fe], G[fe] = H, V.push(fe)), J[fe] ^= le;
      if (oe > ce)
        for (ue = Math.max(ae, ce), de = oe; ue < de; ++ue)
          fe = te[ue], G[fe] !== H && (Z[fe] = J[fe], G[fe] = H, V.push(fe)), J[fe] ^= le;
      else if (oe < ce)
        for (ue = Math.max(se, oe), de = ce; ue < de; ++ue)
          fe = te[ue], G[fe] !== H && (Z[fe] = J[fe], G[fe] = H, V.push(fe)), J[fe] ^= le;
      O.range = _.slice();
    },
    incrementOne(O, _, H, V) {
      const q = this.value, G = q.curr(), J = O.index(), Z = O.bisect(O.range), te = O.bisect(_), ne = te[0], ee = te[1], ae = Z[0], oe = Z[1], se = O.one;
      let ce, le, ue;
      if (ne < ae)
        for (ce = ne, le = Math.min(ae, ee); ce < le; ++ce)
          ue = J[ce], G[ue] ^= se, H.push(ue);
      else if (ne > ae)
        for (ce = ae, le = Math.min(ne, oe); ce < le; ++ce)
          ue = J[ce], G[ue] ^= se, V.push(ue);
      if (ee > oe)
        for (ce = Math.max(ne, oe), le = ee; ce < le; ++ce)
          ue = J[ce], G[ue] ^= se, H.push(ue);
      else if (ee < oe)
        for (ce = Math.max(ae, ee), le = oe; ce < le; ++ce)
          ue = J[ce], G[ue] ^= se, V.push(ue);
      O.range = _.slice();
    }
  });
  function ResolveFilter(O) {
    Transform.call(this, null, O);
  }
  ResolveFilter.Definition = {
    type: "ResolveFilter",
    metadata: {},
    params: [{
      name: "ignore",
      type: "number",
      required: !0,
      description: "A bit mask indicating which filters to ignore."
    }, {
      name: "filter",
      type: "object",
      required: !0,
      description: "Per-tuple filter bitmaps from a CrossFilter transform."
    }]
  };
  inherits(ResolveFilter, Transform, {
    transform(O, _) {
      const H = ~(O.ignore || 0), V = O.filter, q = V.mask;
      if (!(q & H)) return _.StopPropagation;
      const G = _.fork(_.ALL), J = V.data(), Z = V.curr(), te = V.prev(), ne = (ee) => Z[ee] & H ? null : J[ee];
      return G.filter(G.MOD, ne), q & q - 1 ? (G.filter(G.ADD, (ee) => {
        const ae = Z[ee] & H;
        return !ae && ae ^ te[ee] & H ? J[ee] : null;
      }), G.filter(G.REM, (ee) => {
        const ae = Z[ee] & H;
        return ae && !(ae ^ (ae ^ te[ee] & H)) ? J[ee] : null;
      })) : (G.filter(G.ADD, ne), G.filter(G.REM, (ee) => (Z[ee] & H) === q ? J[ee] : null)), G.filter(G.SOURCE, (ee) => ne(ee._index));
    }
  });
  const xf = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    crossfilter: CrossFilter,
    resolvefilter: ResolveFilter
  }, Symbol.toStringTag, { value: "Module" })), RawCode = "RawCode", Literal = "Literal", Property = "Property", Identifier$1 = "Identifier", ArrayExpression = "ArrayExpression", BinaryExpression = "BinaryExpression", CallExpression = "CallExpression", ConditionalExpression = "ConditionalExpression", LogicalExpression = "LogicalExpression", MemberExpression = "MemberExpression", ObjectExpression = "ObjectExpression", UnaryExpression = "UnaryExpression";
  function ASTNode(O) {
    this.type = O;
  }
  ASTNode.prototype.visit = function(O) {
    let _, H, V;
    if (O(this)) return 1;
    for (_ = children(this), H = 0, V = _.length; H < V; ++H)
      if (_[H].visit(O)) return 1;
  };
  function children(O) {
    switch (O.type) {
      case ArrayExpression:
        return O.elements;
      case BinaryExpression:
      case LogicalExpression:
        return [O.left, O.right];
      case CallExpression:
        return [O.callee].concat(O.arguments);
      case ConditionalExpression:
        return [O.test, O.consequent, O.alternate];
      case MemberExpression:
        return [O.object, O.property];
      case ObjectExpression:
        return O.properties;
      case Property:
        return [O.key, O.value];
      case UnaryExpression:
        return [O.argument];
      case Identifier$1:
      case Literal:
      case RawCode:
      default:
        return [];
    }
  }
  var TokenName, source, index, length, lookahead, TokenBooleanLiteral = 1, TokenEOF = 2, TokenIdentifier = 3, TokenKeyword = 4, TokenNullLiteral = 5, TokenNumericLiteral = 6, TokenPunctuator = 7, TokenStringLiteral = 8, TokenRegularExpression = 9;
  TokenName = {};
  TokenName[TokenBooleanLiteral] = "Boolean";
  TokenName[TokenEOF] = "<end>";
  TokenName[TokenIdentifier] = "Identifier";
  TokenName[TokenKeyword] = "Keyword";
  TokenName[TokenNullLiteral] = "Null";
  TokenName[TokenNumericLiteral] = "Numeric";
  TokenName[TokenPunctuator] = "Punctuator";
  TokenName[TokenStringLiteral] = "String";
  TokenName[TokenRegularExpression] = "RegularExpression";
  var SyntaxArrayExpression = "ArrayExpression", SyntaxBinaryExpression = "BinaryExpression", SyntaxCallExpression = "CallExpression", SyntaxConditionalExpression = "ConditionalExpression", SyntaxIdentifier = "Identifier", SyntaxLiteral = "Literal", SyntaxLogicalExpression = "LogicalExpression", SyntaxMemberExpression = "MemberExpression", SyntaxObjectExpression = "ObjectExpression", SyntaxProperty = "Property", SyntaxUnaryExpression = "UnaryExpression", MessageUnexpectedToken = "Unexpected token %0", MessageUnexpectedNumber = "Unexpected number", MessageUnexpectedString = "Unexpected string", MessageUnexpectedIdentifier = "Unexpected identifier", MessageUnexpectedReserved = "Unexpected reserved word", MessageUnexpectedEOS = "Unexpected end of input", MessageInvalidRegExp = "Invalid regular expression", MessageUnterminatedRegExp = "Invalid regular expression: missing /", MessageStrictOctalLiteral = "Octal literals are not allowed in strict mode.", MessageStrictDuplicateProperty = "Duplicate data property in object literal not allowed in strict mode", ILLEGAL$1 = "ILLEGAL", DISABLED = "Disabled.", RegexNonAsciiIdentifierStart = new RegExp("[\\xAA\\xB5\\xBA\\xC0-\\xD6\\xD8-\\xF6\\xF8-\\u02C1\\u02C6-\\u02D1\\u02E0-\\u02E4\\u02EC\\u02EE\\u0370-\\u0374\\u0376\\u0377\\u037A-\\u037D\\u037F\\u0386\\u0388-\\u038A\\u038C\\u038E-\\u03A1\\u03A3-\\u03F5\\u03F7-\\u0481\\u048A-\\u052F\\u0531-\\u0556\\u0559\\u0561-\\u0587\\u05D0-\\u05EA\\u05F0-\\u05F2\\u0620-\\u064A\\u066E\\u066F\\u0671-\\u06D3\\u06D5\\u06E5\\u06E6\\u06EE\\u06EF\\u06FA-\\u06FC\\u06FF\\u0710\\u0712-\\u072F\\u074D-\\u07A5\\u07B1\\u07CA-\\u07EA\\u07F4\\u07F5\\u07FA\\u0800-\\u0815\\u081A\\u0824\\u0828\\u0840-\\u0858\\u08A0-\\u08B2\\u0904-\\u0939\\u093D\\u0950\\u0958-\\u0961\\u0971-\\u0980\\u0985-\\u098C\\u098F\\u0990\\u0993-\\u09A8\\u09AA-\\u09B0\\u09B2\\u09B6-\\u09B9\\u09BD\\u09CE\\u09DC\\u09DD\\u09DF-\\u09E1\\u09F0\\u09F1\\u0A05-\\u0A0A\\u0A0F\\u0A10\\u0A13-\\u0A28\\u0A2A-\\u0A30\\u0A32\\u0A33\\u0A35\\u0A36\\u0A38\\u0A39\\u0A59-\\u0A5C\\u0A5E\\u0A72-\\u0A74\\u0A85-\\u0A8D\\u0A8F-\\u0A91\\u0A93-\\u0AA8\\u0AAA-\\u0AB0\\u0AB2\\u0AB3\\u0AB5-\\u0AB9\\u0ABD\\u0AD0\\u0AE0\\u0AE1\\u0B05-\\u0B0C\\u0B0F\\u0B10\\u0B13-\\u0B28\\u0B2A-\\u0B30\\u0B32\\u0B33\\u0B35-\\u0B39\\u0B3D\\u0B5C\\u0B5D\\u0B5F-\\u0B61\\u0B71\\u0B83\\u0B85-\\u0B8A\\u0B8E-\\u0B90\\u0B92-\\u0B95\\u0B99\\u0B9A\\u0B9C\\u0B9E\\u0B9F\\u0BA3\\u0BA4\\u0BA8-\\u0BAA\\u0BAE-\\u0BB9\\u0BD0\\u0C05-\\u0C0C\\u0C0E-\\u0C10\\u0C12-\\u0C28\\u0C2A-\\u0C39\\u0C3D\\u0C58\\u0C59\\u0C60\\u0C61\\u0C85-\\u0C8C\\u0C8E-\\u0C90\\u0C92-\\u0CA8\\u0CAA-\\u0CB3\\u0CB5-\\u0CB9\\u0CBD\\u0CDE\\u0CE0\\u0CE1\\u0CF1\\u0CF2\\u0D05-\\u0D0C\\u0D0E-\\u0D10\\u0D12-\\u0D3A\\u0D3D\\u0D4E\\u0D60\\u0D61\\u0D7A-\\u0D7F\\u0D85-\\u0D96\\u0D9A-\\u0DB1\\u0DB3-\\u0DBB\\u0DBD\\u0DC0-\\u0DC6\\u0E01-\\u0E30\\u0E32\\u0E33\\u0E40-\\u0E46\\u0E81\\u0E82\\u0E84\\u0E87\\u0E88\\u0E8A\\u0E8D\\u0E94-\\u0E97\\u0E99-\\u0E9F\\u0EA1-\\u0EA3\\u0EA5\\u0EA7\\u0EAA\\u0EAB\\u0EAD-\\u0EB0\\u0EB2\\u0EB3\\u0EBD\\u0EC0-\\u0EC4\\u0EC6\\u0EDC-\\u0EDF\\u0F00\\u0F40-\\u0F47\\u0F49-\\u0F6C\\u0F88-\\u0F8C\\u1000-\\u102A\\u103F\\u1050-\\u1055\\u105A-\\u105D\\u1061\\u1065\\u1066\\u106E-\\u1070\\u1075-\\u1081\\u108E\\u10A0-\\u10C5\\u10C7\\u10CD\\u10D0-\\u10FA\\u10FC-\\u1248\\u124A-\\u124D\\u1250-\\u1256\\u1258\\u125A-\\u125D\\u1260-\\u1288\\u128A-\\u128D\\u1290-\\u12B0\\u12B2-\\u12B5\\u12B8-\\u12BE\\u12C0\\u12C2-\\u12C5\\u12C8-\\u12D6\\u12D8-\\u1310\\u1312-\\u1315\\u1318-\\u135A\\u1380-\\u138F\\u13A0-\\u13F4\\u1401-\\u166C\\u166F-\\u167F\\u1681-\\u169A\\u16A0-\\u16EA\\u16EE-\\u16F8\\u1700-\\u170C\\u170E-\\u1711\\u1720-\\u1731\\u1740-\\u1751\\u1760-\\u176C\\u176E-\\u1770\\u1780-\\u17B3\\u17D7\\u17DC\\u1820-\\u1877\\u1880-\\u18A8\\u18AA\\u18B0-\\u18F5\\u1900-\\u191E\\u1950-\\u196D\\u1970-\\u1974\\u1980-\\u19AB\\u19C1-\\u19C7\\u1A00-\\u1A16\\u1A20-\\u1A54\\u1AA7\\u1B05-\\u1B33\\u1B45-\\u1B4B\\u1B83-\\u1BA0\\u1BAE\\u1BAF\\u1BBA-\\u1BE5\\u1C00-\\u1C23\\u1C4D-\\u1C4F\\u1C5A-\\u1C7D\\u1CE9-\\u1CEC\\u1CEE-\\u1CF1\\u1CF5\\u1CF6\\u1D00-\\u1DBF\\u1E00-\\u1F15\\u1F18-\\u1F1D\\u1F20-\\u1F45\\u1F48-\\u1F4D\\u1F50-\\u1F57\\u1F59\\u1F5B\\u1F5D\\u1F5F-\\u1F7D\\u1F80-\\u1FB4\\u1FB6-\\u1FBC\\u1FBE\\u1FC2-\\u1FC4\\u1FC6-\\u1FCC\\u1FD0-\\u1FD3\\u1FD6-\\u1FDB\\u1FE0-\\u1FEC\\u1FF2-\\u1FF4\\u1FF6-\\u1FFC\\u2071\\u207F\\u2090-\\u209C\\u2102\\u2107\\u210A-\\u2113\\u2115\\u2119-\\u211D\\u2124\\u2126\\u2128\\u212A-\\u212D\\u212F-\\u2139\\u213C-\\u213F\\u2145-\\u2149\\u214E\\u2160-\\u2188\\u2C00-\\u2C2E\\u2C30-\\u2C5E\\u2C60-\\u2CE4\\u2CEB-\\u2CEE\\u2CF2\\u2CF3\\u2D00-\\u2D25\\u2D27\\u2D2D\\u2D30-\\u2D67\\u2D6F\\u2D80-\\u2D96\\u2DA0-\\u2DA6\\u2DA8-\\u2DAE\\u2DB0-\\u2DB6\\u2DB8-\\u2DBE\\u2DC0-\\u2DC6\\u2DC8-\\u2DCE\\u2DD0-\\u2DD6\\u2DD8-\\u2DDE\\u2E2F\\u3005-\\u3007\\u3021-\\u3029\\u3031-\\u3035\\u3038-\\u303C\\u3041-\\u3096\\u309D-\\u309F\\u30A1-\\u30FA\\u30FC-\\u30FF\\u3105-\\u312D\\u3131-\\u318E\\u31A0-\\u31BA\\u31F0-\\u31FF\\u3400-\\u4DB5\\u4E00-\\u9FCC\\uA000-\\uA48C\\uA4D0-\\uA4FD\\uA500-\\uA60C\\uA610-\\uA61F\\uA62A\\uA62B\\uA640-\\uA66E\\uA67F-\\uA69D\\uA6A0-\\uA6EF\\uA717-\\uA71F\\uA722-\\uA788\\uA78B-\\uA78E\\uA790-\\uA7AD\\uA7B0\\uA7B1\\uA7F7-\\uA801\\uA803-\\uA805\\uA807-\\uA80A\\uA80C-\\uA822\\uA840-\\uA873\\uA882-\\uA8B3\\uA8F2-\\uA8F7\\uA8FB\\uA90A-\\uA925\\uA930-\\uA946\\uA960-\\uA97C\\uA984-\\uA9B2\\uA9CF\\uA9E0-\\uA9E4\\uA9E6-\\uA9EF\\uA9FA-\\uA9FE\\uAA00-\\uAA28\\uAA40-\\uAA42\\uAA44-\\uAA4B\\uAA60-\\uAA76\\uAA7A\\uAA7E-\\uAAAF\\uAAB1\\uAAB5\\uAAB6\\uAAB9-\\uAABD\\uAAC0\\uAAC2\\uAADB-\\uAADD\\uAAE0-\\uAAEA\\uAAF2-\\uAAF4\\uAB01-\\uAB06\\uAB09-\\uAB0E\\uAB11-\\uAB16\\uAB20-\\uAB26\\uAB28-\\uAB2E\\uAB30-\\uAB5A\\uAB5C-\\uAB5F\\uAB64\\uAB65\\uABC0-\\uABE2\\uAC00-\\uD7A3\\uD7B0-\\uD7C6\\uD7CB-\\uD7FB\\uF900-\\uFA6D\\uFA70-\\uFAD9\\uFB00-\\uFB06\\uFB13-\\uFB17\\uFB1D\\uFB1F-\\uFB28\\uFB2A-\\uFB36\\uFB38-\\uFB3C\\uFB3E\\uFB40\\uFB41\\uFB43\\uFB44\\uFB46-\\uFBB1\\uFBD3-\\uFD3D\\uFD50-\\uFD8F\\uFD92-\\uFDC7\\uFDF0-\\uFDFB\\uFE70-\\uFE74\\uFE76-\\uFEFC\\uFF21-\\uFF3A\\uFF41-\\uFF5A\\uFF66-\\uFFBE\\uFFC2-\\uFFC7\\uFFCA-\\uFFCF\\uFFD2-\\uFFD7\\uFFDA-\\uFFDC]"), RegexNonAsciiIdentifierPart = new RegExp("[\\xAA\\xB5\\xBA\\xC0-\\xD6\\xD8-\\xF6\\xF8-\\u02C1\\u02C6-\\u02D1\\u02E0-\\u02E4\\u02EC\\u02EE\\u0300-\\u0374\\u0376\\u0377\\u037A-\\u037D\\u037F\\u0386\\u0388-\\u038A\\u038C\\u038E-\\u03A1\\u03A3-\\u03F5\\u03F7-\\u0481\\u0483-\\u0487\\u048A-\\u052F\\u0531-\\u0556\\u0559\\u0561-\\u0587\\u0591-\\u05BD\\u05BF\\u05C1\\u05C2\\u05C4\\u05C5\\u05C7\\u05D0-\\u05EA\\u05F0-\\u05F2\\u0610-\\u061A\\u0620-\\u0669\\u066E-\\u06D3\\u06D5-\\u06DC\\u06DF-\\u06E8\\u06EA-\\u06FC\\u06FF\\u0710-\\u074A\\u074D-\\u07B1\\u07C0-\\u07F5\\u07FA\\u0800-\\u082D\\u0840-\\u085B\\u08A0-\\u08B2\\u08E4-\\u0963\\u0966-\\u096F\\u0971-\\u0983\\u0985-\\u098C\\u098F\\u0990\\u0993-\\u09A8\\u09AA-\\u09B0\\u09B2\\u09B6-\\u09B9\\u09BC-\\u09C4\\u09C7\\u09C8\\u09CB-\\u09CE\\u09D7\\u09DC\\u09DD\\u09DF-\\u09E3\\u09E6-\\u09F1\\u0A01-\\u0A03\\u0A05-\\u0A0A\\u0A0F\\u0A10\\u0A13-\\u0A28\\u0A2A-\\u0A30\\u0A32\\u0A33\\u0A35\\u0A36\\u0A38\\u0A39\\u0A3C\\u0A3E-\\u0A42\\u0A47\\u0A48\\u0A4B-\\u0A4D\\u0A51\\u0A59-\\u0A5C\\u0A5E\\u0A66-\\u0A75\\u0A81-\\u0A83\\u0A85-\\u0A8D\\u0A8F-\\u0A91\\u0A93-\\u0AA8\\u0AAA-\\u0AB0\\u0AB2\\u0AB3\\u0AB5-\\u0AB9\\u0ABC-\\u0AC5\\u0AC7-\\u0AC9\\u0ACB-\\u0ACD\\u0AD0\\u0AE0-\\u0AE3\\u0AE6-\\u0AEF\\u0B01-\\u0B03\\u0B05-\\u0B0C\\u0B0F\\u0B10\\u0B13-\\u0B28\\u0B2A-\\u0B30\\u0B32\\u0B33\\u0B35-\\u0B39\\u0B3C-\\u0B44\\u0B47\\u0B48\\u0B4B-\\u0B4D\\u0B56\\u0B57\\u0B5C\\u0B5D\\u0B5F-\\u0B63\\u0B66-\\u0B6F\\u0B71\\u0B82\\u0B83\\u0B85-\\u0B8A\\u0B8E-\\u0B90\\u0B92-\\u0B95\\u0B99\\u0B9A\\u0B9C\\u0B9E\\u0B9F\\u0BA3\\u0BA4\\u0BA8-\\u0BAA\\u0BAE-\\u0BB9\\u0BBE-\\u0BC2\\u0BC6-\\u0BC8\\u0BCA-\\u0BCD\\u0BD0\\u0BD7\\u0BE6-\\u0BEF\\u0C00-\\u0C03\\u0C05-\\u0C0C\\u0C0E-\\u0C10\\u0C12-\\u0C28\\u0C2A-\\u0C39\\u0C3D-\\u0C44\\u0C46-\\u0C48\\u0C4A-\\u0C4D\\u0C55\\u0C56\\u0C58\\u0C59\\u0C60-\\u0C63\\u0C66-\\u0C6F\\u0C81-\\u0C83\\u0C85-\\u0C8C\\u0C8E-\\u0C90\\u0C92-\\u0CA8\\u0CAA-\\u0CB3\\u0CB5-\\u0CB9\\u0CBC-\\u0CC4\\u0CC6-\\u0CC8\\u0CCA-\\u0CCD\\u0CD5\\u0CD6\\u0CDE\\u0CE0-\\u0CE3\\u0CE6-\\u0CEF\\u0CF1\\u0CF2\\u0D01-\\u0D03\\u0D05-\\u0D0C\\u0D0E-\\u0D10\\u0D12-\\u0D3A\\u0D3D-\\u0D44\\u0D46-\\u0D48\\u0D4A-\\u0D4E\\u0D57\\u0D60-\\u0D63\\u0D66-\\u0D6F\\u0D7A-\\u0D7F\\u0D82\\u0D83\\u0D85-\\u0D96\\u0D9A-\\u0DB1\\u0DB3-\\u0DBB\\u0DBD\\u0DC0-\\u0DC6\\u0DCA\\u0DCF-\\u0DD4\\u0DD6\\u0DD8-\\u0DDF\\u0DE6-\\u0DEF\\u0DF2\\u0DF3\\u0E01-\\u0E3A\\u0E40-\\u0E4E\\u0E50-\\u0E59\\u0E81\\u0E82\\u0E84\\u0E87\\u0E88\\u0E8A\\u0E8D\\u0E94-\\u0E97\\u0E99-\\u0E9F\\u0EA1-\\u0EA3\\u0EA5\\u0EA7\\u0EAA\\u0EAB\\u0EAD-\\u0EB9\\u0EBB-\\u0EBD\\u0EC0-\\u0EC4\\u0EC6\\u0EC8-\\u0ECD\\u0ED0-\\u0ED9\\u0EDC-\\u0EDF\\u0F00\\u0F18\\u0F19\\u0F20-\\u0F29\\u0F35\\u0F37\\u0F39\\u0F3E-\\u0F47\\u0F49-\\u0F6C\\u0F71-\\u0F84\\u0F86-\\u0F97\\u0F99-\\u0FBC\\u0FC6\\u1000-\\u1049\\u1050-\\u109D\\u10A0-\\u10C5\\u10C7\\u10CD\\u10D0-\\u10FA\\u10FC-\\u1248\\u124A-\\u124D\\u1250-\\u1256\\u1258\\u125A-\\u125D\\u1260-\\u1288\\u128A-\\u128D\\u1290-\\u12B0\\u12B2-\\u12B5\\u12B8-\\u12BE\\u12C0\\u12C2-\\u12C5\\u12C8-\\u12D6\\u12D8-\\u1310\\u1312-\\u1315\\u1318-\\u135A\\u135D-\\u135F\\u1380-\\u138F\\u13A0-\\u13F4\\u1401-\\u166C\\u166F-\\u167F\\u1681-\\u169A\\u16A0-\\u16EA\\u16EE-\\u16F8\\u1700-\\u170C\\u170E-\\u1714\\u1720-\\u1734\\u1740-\\u1753\\u1760-\\u176C\\u176E-\\u1770\\u1772\\u1773\\u1780-\\u17D3\\u17D7\\u17DC\\u17DD\\u17E0-\\u17E9\\u180B-\\u180D\\u1810-\\u1819\\u1820-\\u1877\\u1880-\\u18AA\\u18B0-\\u18F5\\u1900-\\u191E\\u1920-\\u192B\\u1930-\\u193B\\u1946-\\u196D\\u1970-\\u1974\\u1980-\\u19AB\\u19B0-\\u19C9\\u19D0-\\u19D9\\u1A00-\\u1A1B\\u1A20-\\u1A5E\\u1A60-\\u1A7C\\u1A7F-\\u1A89\\u1A90-\\u1A99\\u1AA7\\u1AB0-\\u1ABD\\u1B00-\\u1B4B\\u1B50-\\u1B59\\u1B6B-\\u1B73\\u1B80-\\u1BF3\\u1C00-\\u1C37\\u1C40-\\u1C49\\u1C4D-\\u1C7D\\u1CD0-\\u1CD2\\u1CD4-\\u1CF6\\u1CF8\\u1CF9\\u1D00-\\u1DF5\\u1DFC-\\u1F15\\u1F18-\\u1F1D\\u1F20-\\u1F45\\u1F48-\\u1F4D\\u1F50-\\u1F57\\u1F59\\u1F5B\\u1F5D\\u1F5F-\\u1F7D\\u1F80-\\u1FB4\\u1FB6-\\u1FBC\\u1FBE\\u1FC2-\\u1FC4\\u1FC6-\\u1FCC\\u1FD0-\\u1FD3\\u1FD6-\\u1FDB\\u1FE0-\\u1FEC\\u1FF2-\\u1FF4\\u1FF6-\\u1FFC\\u200C\\u200D\\u203F\\u2040\\u2054\\u2071\\u207F\\u2090-\\u209C\\u20D0-\\u20DC\\u20E1\\u20E5-\\u20F0\\u2102\\u2107\\u210A-\\u2113\\u2115\\u2119-\\u211D\\u2124\\u2126\\u2128\\u212A-\\u212D\\u212F-\\u2139\\u213C-\\u213F\\u2145-\\u2149\\u214E\\u2160-\\u2188\\u2C00-\\u2C2E\\u2C30-\\u2C5E\\u2C60-\\u2CE4\\u2CEB-\\u2CF3\\u2D00-\\u2D25\\u2D27\\u2D2D\\u2D30-\\u2D67\\u2D6F\\u2D7F-\\u2D96\\u2DA0-\\u2DA6\\u2DA8-\\u2DAE\\u2DB0-\\u2DB6\\u2DB8-\\u2DBE\\u2DC0-\\u2DC6\\u2DC8-\\u2DCE\\u2DD0-\\u2DD6\\u2DD8-\\u2DDE\\u2DE0-\\u2DFF\\u2E2F\\u3005-\\u3007\\u3021-\\u302F\\u3031-\\u3035\\u3038-\\u303C\\u3041-\\u3096\\u3099\\u309A\\u309D-\\u309F\\u30A1-\\u30FA\\u30FC-\\u30FF\\u3105-\\u312D\\u3131-\\u318E\\u31A0-\\u31BA\\u31F0-\\u31FF\\u3400-\\u4DB5\\u4E00-\\u9FCC\\uA000-\\uA48C\\uA4D0-\\uA4FD\\uA500-\\uA60C\\uA610-\\uA62B\\uA640-\\uA66F\\uA674-\\uA67D\\uA67F-\\uA69D\\uA69F-\\uA6F1\\uA717-\\uA71F\\uA722-\\uA788\\uA78B-\\uA78E\\uA790-\\uA7AD\\uA7B0\\uA7B1\\uA7F7-\\uA827\\uA840-\\uA873\\uA880-\\uA8C4\\uA8D0-\\uA8D9\\uA8E0-\\uA8F7\\uA8FB\\uA900-\\uA92D\\uA930-\\uA953\\uA960-\\uA97C\\uA980-\\uA9C0\\uA9CF-\\uA9D9\\uA9E0-\\uA9FE\\uAA00-\\uAA36\\uAA40-\\uAA4D\\uAA50-\\uAA59\\uAA60-\\uAA76\\uAA7A-\\uAAC2\\uAADB-\\uAADD\\uAAE0-\\uAAEF\\uAAF2-\\uAAF6\\uAB01-\\uAB06\\uAB09-\\uAB0E\\uAB11-\\uAB16\\uAB20-\\uAB26\\uAB28-\\uAB2E\\uAB30-\\uAB5A\\uAB5C-\\uAB5F\\uAB64\\uAB65\\uABC0-\\uABEA\\uABEC\\uABED\\uABF0-\\uABF9\\uAC00-\\uD7A3\\uD7B0-\\uD7C6\\uD7CB-\\uD7FB\\uF900-\\uFA6D\\uFA70-\\uFAD9\\uFB00-\\uFB06\\uFB13-\\uFB17\\uFB1D-\\uFB28\\uFB2A-\\uFB36\\uFB38-\\uFB3C\\uFB3E\\uFB40\\uFB41\\uFB43\\uFB44\\uFB46-\\uFBB1\\uFBD3-\\uFD3D\\uFD50-\\uFD8F\\uFD92-\\uFDC7\\uFDF0-\\uFDFB\\uFE00-\\uFE0F\\uFE20-\\uFE2D\\uFE33\\uFE34\\uFE4D-\\uFE4F\\uFE70-\\uFE74\\uFE76-\\uFEFC\\uFF10-\\uFF19\\uFF21-\\uFF3A\\uFF3F\\uFF41-\\uFF5A\\uFF66-\\uFFBE\\uFFC2-\\uFFC7\\uFFCA-\\uFFCF\\uFFD2-\\uFFD7\\uFFDA-\\uFFDC]");
  function assert(O, _) {
    if (!O)
      throw new Error("ASSERT: " + _);
  }
  function isDecimalDigit(O) {
    return O >= 48 && O <= 57;
  }
  function isHexDigit$1(O) {
    return "0123456789abcdefABCDEF".includes(O);
  }
  function isOctalDigit(O) {
    return "01234567".includes(O);
  }
  function isWhiteSpace(O) {
    return O === 32 || O === 9 || O === 11 || O === 12 || O === 160 || O >= 5760 && [5760, 6158, 8192, 8193, 8194, 8195, 8196, 8197, 8198, 8199, 8200, 8201, 8202, 8239, 8287, 12288, 65279].includes(O);
  }
  function isLineTerminator(O) {
    return O === 10 || O === 13 || O === 8232 || O === 8233;
  }
  function isIdentifierStart(O) {
    return O === 36 || O === 95 || // $ (dollar) and _ (underscore)
    O >= 65 && O <= 90 || // A..Z
    O >= 97 && O <= 122 || // a..z
    O === 92 || // \ (backslash)
    O >= 128 && RegexNonAsciiIdentifierStart.test(String.fromCharCode(O));
  }
  function isIdentifierPart(O) {
    return O === 36 || O === 95 || // $ (dollar) and _ (underscore)
    O >= 65 && O <= 90 || // A..Z
    O >= 97 && O <= 122 || // a..z
    O >= 48 && O <= 57 || // 0..9
    O === 92 || // \ (backslash)
    O >= 128 && RegexNonAsciiIdentifierPart.test(String.fromCharCode(O));
  }
  const keywords$3 = {
    if: 1,
    in: 1,
    do: 1,
    var: 1,
    for: 1,
    new: 1,
    try: 1,
    let: 1,
    this: 1,
    else: 1,
    case: 1,
    void: 1,
    with: 1,
    enum: 1,
    while: 1,
    break: 1,
    catch: 1,
    throw: 1,
    const: 1,
    yield: 1,
    class: 1,
    super: 1,
    return: 1,
    typeof: 1,
    delete: 1,
    switch: 1,
    export: 1,
    import: 1,
    public: 1,
    static: 1,
    default: 1,
    finally: 1,
    extends: 1,
    package: 1,
    private: 1,
    function: 1,
    continue: 1,
    debugger: 1,
    interface: 1,
    protected: 1,
    instanceof: 1,
    implements: 1
  };
  function skipComment() {
    for (; index < length; ) {
      const O = source.charCodeAt(index);
      if (isWhiteSpace(O) || isLineTerminator(O))
        ++index;
      else
        break;
    }
  }
  function scanHexEscape(O) {
    var _, H, V, q = 0;
    for (H = O === "u" ? 4 : 2, _ = 0; _ < H; ++_)
      index < length && isHexDigit$1(source[index]) ? (V = source[index++], q = q * 16 + "0123456789abcdef".indexOf(V.toLowerCase())) : throwError({}, MessageUnexpectedToken, ILLEGAL$1);
    return String.fromCharCode(q);
  }
  function scanUnicodeCodePointEscape() {
    var O, _, H, V;
    for (O = source[index], _ = 0, O === "}" && throwError({}, MessageUnexpectedToken, ILLEGAL$1); index < length && (O = source[index++], !!isHexDigit$1(O)); )
      _ = _ * 16 + "0123456789abcdef".indexOf(O.toLowerCase());
    return (_ > 1114111 || O !== "}") && throwError({}, MessageUnexpectedToken, ILLEGAL$1), _ <= 65535 ? String.fromCharCode(_) : (H = (_ - 65536 >> 10) + 55296, V = (_ - 65536 & 1023) + 56320, String.fromCharCode(H, V));
  }
  function getEscapedIdentifier() {
    var O, _;
    for (O = source.charCodeAt(index++), _ = String.fromCharCode(O), O === 92 && (source.charCodeAt(index) !== 117 && throwError({}, MessageUnexpectedToken, ILLEGAL$1), ++index, O = scanHexEscape("u"), (!O || O === "\\" || !isIdentifierStart(O.charCodeAt(0))) && throwError({}, MessageUnexpectedToken, ILLEGAL$1), _ = O); index < length && (O = source.charCodeAt(index), !!isIdentifierPart(O)); )
      ++index, _ += String.fromCharCode(O), O === 92 && (_ = _.substr(0, _.length - 1), source.charCodeAt(index) !== 117 && throwError({}, MessageUnexpectedToken, ILLEGAL$1), ++index, O = scanHexEscape("u"), (!O || O === "\\" || !isIdentifierPart(O.charCodeAt(0))) && throwError({}, MessageUnexpectedToken, ILLEGAL$1), _ += O);
    return _;
  }
  function getIdentifier() {
    var O, _;
    for (O = index++; index < length; ) {
      if (_ = source.charCodeAt(index), _ === 92)
        return index = O, getEscapedIdentifier();
      if (isIdentifierPart(_))
        ++index;
      else
        break;
    }
    return source.slice(O, index);
  }
  function scanIdentifier() {
    var O, _, H;
    return O = index, _ = source.charCodeAt(index) === 92 ? getEscapedIdentifier() : getIdentifier(), _.length === 1 ? H = TokenIdentifier : keywords$3.hasOwnProperty(_) ? H = TokenKeyword : _ === "null" ? H = TokenNullLiteral : _ === "true" || _ === "false" ? H = TokenBooleanLiteral : H = TokenIdentifier, {
      type: H,
      value: _,
      start: O,
      end: index
    };
  }
  function scanPunctuator() {
    var O = index, _ = source.charCodeAt(index), H, V = source[index], q, G, J;
    switch (_) {
      // Check for most common single-character punctuators.
      case 46:
      // . dot
      case 40:
      // ( open bracket
      case 41:
      // ) close bracket
      case 59:
      // ; semicolon
      case 44:
      // , comma
      case 123:
      // { open curly brace
      case 125:
      // } close curly brace
      case 91:
      // [
      case 93:
      // ]
      case 58:
      // :
      case 63:
      // ?
      case 126:
        return ++index, {
          type: TokenPunctuator,
          value: String.fromCharCode(_),
          start: O,
          end: index
        };
      default:
        if (H = source.charCodeAt(index + 1), H === 61)
          switch (_) {
            case 43:
            // +
            case 45:
            // -
            case 47:
            // /
            case 60:
            // <
            case 62:
            // >
            case 94:
            // ^
            case 124:
            // |
            case 37:
            // %
            case 38:
            // &
            case 42:
              return index += 2, {
                type: TokenPunctuator,
                value: String.fromCharCode(_) + String.fromCharCode(H),
                start: O,
                end: index
              };
            case 33:
            // !
            case 61:
              return index += 2, source.charCodeAt(index) === 61 && ++index, {
                type: TokenPunctuator,
                value: source.slice(O, index),
                start: O,
                end: index
              };
          }
    }
    if (J = source.substr(index, 4), J === ">>>=")
      return index += 4, {
        type: TokenPunctuator,
        value: J,
        start: O,
        end: index
      };
    if (G = J.substr(0, 3), G === ">>>" || G === "<<=" || G === ">>=")
      return index += 3, {
        type: TokenPunctuator,
        value: G,
        start: O,
        end: index
      };
    if (q = G.substr(0, 2), V === q[1] && "+-<>&|".includes(V) || q === "=>")
      return index += 2, {
        type: TokenPunctuator,
        value: q,
        start: O,
        end: index
      };
    if (q === "//" && throwError({}, MessageUnexpectedToken, ILLEGAL$1), "<>=!+-*%&|^/".includes(V))
      return ++index, {
        type: TokenPunctuator,
        value: V,
        start: O,
        end: index
      };
    throwError({}, MessageUnexpectedToken, ILLEGAL$1);
  }
  function scanHexLiteral(O) {
    let _ = "";
    for (; index < length && isHexDigit$1(source[index]); )
      _ += source[index++];
    return _.length === 0 && throwError({}, MessageUnexpectedToken, ILLEGAL$1), isIdentifierStart(source.charCodeAt(index)) && throwError({}, MessageUnexpectedToken, ILLEGAL$1), {
      type: TokenNumericLiteral,
      value: parseInt("0x" + _, 16),
      start: O,
      end: index
    };
  }
  function scanOctalLiteral(O) {
    let _ = "0" + source[index++];
    for (; index < length && isOctalDigit(source[index]); )
      _ += source[index++];
    return (isIdentifierStart(source.charCodeAt(index)) || isDecimalDigit(source.charCodeAt(index))) && throwError({}, MessageUnexpectedToken, ILLEGAL$1), {
      type: TokenNumericLiteral,
      value: parseInt(_, 8),
      octal: !0,
      start: O,
      end: index
    };
  }
  function scanNumericLiteral() {
    var O, _, H;
    if (H = source[index], assert(isDecimalDigit(H.charCodeAt(0)) || H === ".", "Numeric literal must start with a decimal digit or a decimal point"), _ = index, O = "", H !== ".") {
      if (O = source[index++], H = source[index], O === "0") {
        if (H === "x" || H === "X")
          return ++index, scanHexLiteral(_);
        if (isOctalDigit(H))
          return scanOctalLiteral(_);
        H && isDecimalDigit(H.charCodeAt(0)) && throwError({}, MessageUnexpectedToken, ILLEGAL$1);
      }
      for (; isDecimalDigit(source.charCodeAt(index)); )
        O += source[index++];
      H = source[index];
    }
    if (H === ".") {
      for (O += source[index++]; isDecimalDigit(source.charCodeAt(index)); )
        O += source[index++];
      H = source[index];
    }
    if (H === "e" || H === "E")
      if (O += source[index++], H = source[index], (H === "+" || H === "-") && (O += source[index++]), isDecimalDigit(source.charCodeAt(index)))
        for (; isDecimalDigit(source.charCodeAt(index)); )
          O += source[index++];
      else
        throwError({}, MessageUnexpectedToken, ILLEGAL$1);
    return isIdentifierStart(source.charCodeAt(index)) && throwError({}, MessageUnexpectedToken, ILLEGAL$1), {
      type: TokenNumericLiteral,
      value: parseFloat(O),
      start: _,
      end: index
    };
  }
  function scanStringLiteral() {
    var O = "", _, H, V, q, G = !1;
    for (_ = source[index], assert(_ === "'" || _ === '"', "String literal must starts with a quote"), H = index, ++index; index < length; )
      if (V = source[index++], V === _) {
        _ = "";
        break;
      } else if (V === "\\")
        if (V = source[index++], !V || !isLineTerminator(V.charCodeAt(0)))
          switch (V) {
            case "u":
            case "x":
              source[index] === "{" ? (++index, O += scanUnicodeCodePointEscape()) : O += scanHexEscape(V);
              break;
            case "n":
              O += `
`;
              break;
            case "r":
              O += "\r";
              break;
            case "t":
              O += "	";
              break;
            case "b":
              O += "\b";
              break;
            case "f":
              O += "\f";
              break;
            case "v":
              O += "\v";
              break;
            default:
              isOctalDigit(V) ? (q = "01234567".indexOf(V), q !== 0 && (G = !0), index < length && isOctalDigit(source[index]) && (G = !0, q = q * 8 + "01234567".indexOf(source[index++]), "0123".includes(V) && index < length && isOctalDigit(source[index]) && (q = q * 8 + "01234567".indexOf(source[index++]))), O += String.fromCharCode(q)) : O += V;
              break;
          }
        else
          V === "\r" && source[index] === `
` && ++index;
      else {
        if (isLineTerminator(V.charCodeAt(0)))
          break;
        O += V;
      }
    return _ !== "" && throwError({}, MessageUnexpectedToken, ILLEGAL$1), {
      type: TokenStringLiteral,
      value: O,
      octal: G,
      start: H,
      end: index
    };
  }
  function testRegExp(O, _) {
    let H = O;
    _.includes("u") && (H = H.replace(/\\u\{([0-9a-fA-F]+)\}/g, (V, q) => {
      if (parseInt(q, 16) <= 1114111)
        return "x";
      throwError({}, MessageInvalidRegExp);
    }).replace(/[\uD800-\uDBFF][\uDC00-\uDFFF]/g, "x"));
    try {
      new RegExp(H);
    } catch {
      throwError({}, MessageInvalidRegExp);
    }
    try {
      return new RegExp(O, _);
    } catch {
      return null;
    }
  }
  function scanRegExpBody() {
    var O, _, H, V, q;
    for (O = source[index], assert(O === "/", "Regular expression literal must start with a slash"), _ = source[index++], H = !1, V = !1; index < length; )
      if (O = source[index++], _ += O, O === "\\")
        O = source[index++], isLineTerminator(O.charCodeAt(0)) && throwError({}, MessageUnterminatedRegExp), _ += O;
      else if (isLineTerminator(O.charCodeAt(0)))
        throwError({}, MessageUnterminatedRegExp);
      else if (H)
        O === "]" && (H = !1);
      else if (O === "/") {
        V = !0;
        break;
      } else O === "[" && (H = !0);
    return V || throwError({}, MessageUnterminatedRegExp), q = _.substr(1, _.length - 2), {
      value: q,
      literal: _
    };
  }
  function scanRegExpFlags() {
    var O, _, H;
    for (_ = "", H = ""; index < length && (O = source[index], !!isIdentifierPart(O.charCodeAt(0))); )
      ++index, O === "\\" && index < length ? throwError({}, MessageUnexpectedToken, ILLEGAL$1) : (H += O, _ += O);
    return H.search(/[^gimuy]/g) >= 0 && throwError({}, MessageInvalidRegExp, H), {
      value: H,
      literal: _
    };
  }
  function scanRegExp() {
    var O, _, H, V;
    return lookahead = null, skipComment(), O = index, _ = scanRegExpBody(), H = scanRegExpFlags(), V = testRegExp(_.value, H.value), {
      literal: _.literal + H.literal,
      value: V,
      regex: {
        pattern: _.value,
        flags: H.value
      },
      start: O,
      end: index
    };
  }
  function isIdentifierName(O) {
    return O.type === TokenIdentifier || O.type === TokenKeyword || O.type === TokenBooleanLiteral || O.type === TokenNullLiteral;
  }
  function advance() {
    if (skipComment(), index >= length)
      return {
        type: TokenEOF,
        start: index,
        end: index
      };
    const O = source.charCodeAt(index);
    return isIdentifierStart(O) ? scanIdentifier() : O === 40 || O === 41 || O === 59 ? scanPunctuator() : O === 39 || O === 34 ? scanStringLiteral() : O === 46 ? isDecimalDigit(source.charCodeAt(index + 1)) ? scanNumericLiteral() : scanPunctuator() : isDecimalDigit(O) ? scanNumericLiteral() : scanPunctuator();
  }
  function lex() {
    const O = lookahead;
    return index = O.end, lookahead = advance(), index = O.end, O;
  }
  function peek() {
    const O = index;
    lookahead = advance(), index = O;
  }
  function finishArrayExpression(O) {
    const _ = new ASTNode(SyntaxArrayExpression);
    return _.elements = O, _;
  }
  function finishBinaryExpression(O, _, H) {
    const V = new ASTNode(O === "||" || O === "&&" ? SyntaxLogicalExpression : SyntaxBinaryExpression);
    return V.operator = O, V.left = _, V.right = H, V;
  }
  function finishCallExpression(O, _) {
    const H = new ASTNode(SyntaxCallExpression);
    return H.callee = O, H.arguments = _, H;
  }
  function finishConditionalExpression(O, _, H) {
    const V = new ASTNode(SyntaxConditionalExpression);
    return V.test = O, V.consequent = _, V.alternate = H, V;
  }
  function finishIdentifier(O) {
    const _ = new ASTNode(SyntaxIdentifier);
    return _.name = O, _;
  }
  function finishLiteral(O) {
    const _ = new ASTNode(SyntaxLiteral);
    return _.value = O.value, _.raw = source.slice(O.start, O.end), O.regex && (_.raw === "//" && (_.raw = "/(?:)/"), _.regex = O.regex), _;
  }
  function finishMemberExpression(O, _, H) {
    const V = new ASTNode(SyntaxMemberExpression);
    return V.computed = O === "[", V.object = _, V.property = H, V.computed || (H.member = !0), V;
  }
  function finishObjectExpression(O) {
    const _ = new ASTNode(SyntaxObjectExpression);
    return _.properties = O, _;
  }
  function finishProperty(O, _, H) {
    const V = new ASTNode(SyntaxProperty);
    return V.key = _, V.value = H, V.kind = O, V;
  }
  function finishUnaryExpression(O, _) {
    const H = new ASTNode(SyntaxUnaryExpression);
    return H.operator = O, H.argument = _, H.prefix = !0, H;
  }
  function throwError(O, _) {
    var H, V = Array.prototype.slice.call(arguments, 2), q = _.replace(/%(\d)/g, (G, J) => (assert(J < V.length, "Message reference must be in range"), V[J]));
    throw H = new Error(q), H.index = index, H.description = q, H;
  }
  function throwUnexpected(O) {
    O.type === TokenEOF && throwError(O, MessageUnexpectedEOS), O.type === TokenNumericLiteral && throwError(O, MessageUnexpectedNumber), O.type === TokenStringLiteral && throwError(O, MessageUnexpectedString), O.type === TokenIdentifier && throwError(O, MessageUnexpectedIdentifier), O.type === TokenKeyword && throwError(O, MessageUnexpectedReserved), throwError(O, MessageUnexpectedToken, O.value);
  }
  function expect(O) {
    const _ = lex();
    (_.type !== TokenPunctuator || _.value !== O) && throwUnexpected(_);
  }
  function match(O) {
    return lookahead.type === TokenPunctuator && lookahead.value === O;
  }
  function matchKeyword(O) {
    return lookahead.type === TokenKeyword && lookahead.value === O;
  }
  function parseArrayInitialiser() {
    const O = [];
    for (index = lookahead.start, expect("["); !match("]"); )
      match(",") ? (lex(), O.push(null)) : (O.push(parseConditionalExpression()), match("]") || expect(","));
    return lex(), finishArrayExpression(O);
  }
  function parseObjectPropertyKey() {
    index = lookahead.start;
    const O = lex();
    return O.type === TokenStringLiteral || O.type === TokenNumericLiteral ? (O.octal && throwError(O, MessageStrictOctalLiteral), finishLiteral(O)) : finishIdentifier(O.value);
  }
  function parseObjectProperty() {
    var O, _, H, V;
    if (index = lookahead.start, O = lookahead, O.type === TokenIdentifier)
      return H = parseObjectPropertyKey(), expect(":"), V = parseConditionalExpression(), finishProperty("init", H, V);
    if (O.type === TokenEOF || O.type === TokenPunctuator)
      throwUnexpected(O);
    else
      return _ = parseObjectPropertyKey(), expect(":"), V = parseConditionalExpression(), finishProperty("init", _, V);
  }
  function parseObjectInitialiser() {
    var O = [], _, H, V, q = {}, G = String;
    for (index = lookahead.start, expect("{"); !match("}"); )
      _ = parseObjectProperty(), _.key.type === SyntaxIdentifier ? H = _.key.name : H = G(_.key.value), V = "$" + H, Object.prototype.hasOwnProperty.call(q, V) ? throwError({}, MessageStrictDuplicateProperty) : q[V] = !0, O.push(_), match("}") || expect(",");
    return expect("}"), finishObjectExpression(O);
  }
  function parseGroupExpression() {
    expect("(");
    const O = parseExpression$1();
    return expect(")"), O;
  }
  const legalKeywords = {
    if: 1
  };
  function parsePrimaryExpression() {
    var O, _, H;
    if (match("("))
      return parseGroupExpression();
    if (match("["))
      return parseArrayInitialiser();
    if (match("{"))
      return parseObjectInitialiser();
    if (O = lookahead.type, index = lookahead.start, O === TokenIdentifier || legalKeywords[lookahead.value])
      H = finishIdentifier(lex().value);
    else if (O === TokenStringLiteral || O === TokenNumericLiteral)
      lookahead.octal && throwError(lookahead, MessageStrictOctalLiteral), H = finishLiteral(lex());
    else {
      if (O === TokenKeyword)
        throw new Error(DISABLED);
      O === TokenBooleanLiteral ? (_ = lex(), _.value = _.value === "true", H = finishLiteral(_)) : O === TokenNullLiteral ? (_ = lex(), _.value = null, H = finishLiteral(_)) : match("/") || match("/=") ? (H = finishLiteral(scanRegExp()), peek()) : throwUnexpected(lex());
    }
    return H;
  }
  function parseArguments() {
    const O = [];
    if (expect("("), !match(")"))
      for (; index < length && (O.push(parseConditionalExpression()), !match(")")); )
        expect(",");
    return expect(")"), O;
  }
  function parseNonComputedProperty() {
    index = lookahead.start;
    const O = lex();
    return isIdentifierName(O) || throwUnexpected(O), finishIdentifier(O.value);
  }
  function parseNonComputedMember() {
    return expect("."), parseNonComputedProperty();
  }
  function parseComputedMember() {
    expect("[");
    const O = parseExpression$1();
    return expect("]"), O;
  }
  function parseLeftHandSideExpressionAllowCall() {
    var O, _, H;
    for (O = parsePrimaryExpression(); ; )
      if (match("."))
        H = parseNonComputedMember(), O = finishMemberExpression(".", O, H);
      else if (match("("))
        _ = parseArguments(), O = finishCallExpression(O, _);
      else if (match("["))
        H = parseComputedMember(), O = finishMemberExpression("[", O, H);
      else
        break;
    return O;
  }
  function parsePostfixExpression() {
    const O = parseLeftHandSideExpressionAllowCall();
    if (lookahead.type === TokenPunctuator && (match("++") || match("--")))
      throw new Error(DISABLED);
    return O;
  }
  function parseUnaryExpression() {
    var O, _;
    if (lookahead.type !== TokenPunctuator && lookahead.type !== TokenKeyword)
      _ = parsePostfixExpression();
    else {
      if (match("++") || match("--"))
        throw new Error(DISABLED);
      if (match("+") || match("-") || match("~") || match("!"))
        O = lex(), _ = parseUnaryExpression(), _ = finishUnaryExpression(O.value, _);
      else {
        if (matchKeyword("delete") || matchKeyword("void") || matchKeyword("typeof"))
          throw new Error(DISABLED);
        _ = parsePostfixExpression();
      }
    }
    return _;
  }
  function binaryPrecedence(O) {
    let _ = 0;
    if (O.type !== TokenPunctuator && O.type !== TokenKeyword)
      return 0;
    switch (O.value) {
      case "||":
        _ = 1;
        break;
      case "&&":
        _ = 2;
        break;
      case "|":
        _ = 3;
        break;
      case "^":
        _ = 4;
        break;
      case "&":
        _ = 5;
        break;
      case "==":
      case "!=":
      case "===":
      case "!==":
        _ = 6;
        break;
      case "<":
      case ">":
      case "<=":
      case ">=":
      case "instanceof":
      case "in":
        _ = 7;
        break;
      case "<<":
      case ">>":
      case ">>>":
        _ = 8;
        break;
      case "+":
      case "-":
        _ = 9;
        break;
      case "*":
      case "/":
      case "%":
        _ = 11;
        break;
    }
    return _;
  }
  function parseBinaryExpression() {
    var O, _, H, V, q, G, J, Z, te, ne;
    if (O = lookahead, te = parseUnaryExpression(), V = lookahead, q = binaryPrecedence(V), q === 0)
      return te;
    for (V.prec = q, lex(), _ = [O, lookahead], J = parseUnaryExpression(), G = [te, V, J]; (q = binaryPrecedence(lookahead)) > 0; ) {
      for (; G.length > 2 && q <= G[G.length - 2].prec; )
        J = G.pop(), Z = G.pop().value, te = G.pop(), _.pop(), H = finishBinaryExpression(Z, te, J), G.push(H);
      V = lex(), V.prec = q, G.push(V), _.push(lookahead), H = parseUnaryExpression(), G.push(H);
    }
    for (ne = G.length - 1, H = G[ne], _.pop(); ne > 1; )
      _.pop(), H = finishBinaryExpression(G[ne - 1].value, G[ne - 2], H), ne -= 2;
    return H;
  }
  function parseConditionalExpression() {
    var O, _, H;
    return O = parseBinaryExpression(), match("?") && (lex(), _ = parseConditionalExpression(), expect(":"), H = parseConditionalExpression(), O = finishConditionalExpression(O, _, H)), O;
  }
  function parseExpression$1() {
    const O = parseConditionalExpression();
    if (match(","))
      throw new Error(DISABLED);
    return O;
  }
  function parser$3(O) {
    source = O, index = 0, length = source.length, lookahead = null, peek();
    const _ = parseExpression$1();
    if (lookahead.type !== TokenEOF)
      throw new Error("Unexpect token after expression.");
    return _;
  }
  var Constants$1 = {
    NaN: "NaN",
    E: "Math.E",
    LN2: "Math.LN2",
    LN10: "Math.LN10",
    LOG2E: "Math.LOG2E",
    LOG10E: "Math.LOG10E",
    PI: "Math.PI",
    SQRT1_2: "Math.SQRT1_2",
    SQRT2: "Math.SQRT2",
    MIN_VALUE: "Number.MIN_VALUE",
    MAX_VALUE: "Number.MAX_VALUE"
  };
  function Functions$1(O) {
    function _(J, Z, te, ne) {
      let ee = O(Z[0]);
      return te && (ee = te + "(" + ee + ")", te.lastIndexOf("new ", 0) === 0 && (ee = "(" + ee + ")")), ee + "." + J + (ne < 0 ? "" : ne === 0 ? "()" : "(" + Z.slice(1).map(O).join(",") + ")");
    }
    function H(J, Z, te) {
      return (ne) => _(J, ne, Z, te);
    }
    const V = "new Date", q = "String", G = "RegExp";
    return {
      // MATH functions
      isNaN: "Number.isNaN",
      isFinite: "Number.isFinite",
      abs: "Math.abs",
      acos: "Math.acos",
      asin: "Math.asin",
      atan: "Math.atan",
      atan2: "Math.atan2",
      ceil: "Math.ceil",
      cos: "Math.cos",
      exp: "Math.exp",
      floor: "Math.floor",
      hypot: "Math.hypot",
      log: "Math.log",
      max: "Math.max",
      min: "Math.min",
      pow: "Math.pow",
      random: "Math.random",
      round: "Math.round",
      sin: "Math.sin",
      sqrt: "Math.sqrt",
      tan: "Math.tan",
      clamp: function(J) {
        J.length < 3 && error$1("Missing arguments to clamp function."), J.length > 3 && error$1("Too many arguments to clamp function.");
        const Z = J.map(O);
        return "Math.max(" + Z[1] + ", Math.min(" + Z[2] + "," + Z[0] + "))";
      },
      // DATE functions
      now: "Date.now",
      utc: "Date.UTC",
      datetime: V,
      date: H("getDate", V, 0),
      day: H("getDay", V, 0),
      year: H("getFullYear", V, 0),
      month: H("getMonth", V, 0),
      hours: H("getHours", V, 0),
      minutes: H("getMinutes", V, 0),
      seconds: H("getSeconds", V, 0),
      milliseconds: H("getMilliseconds", V, 0),
      time: H("getTime", V, 0),
      timezoneoffset: H("getTimezoneOffset", V, 0),
      utcdate: H("getUTCDate", V, 0),
      utcday: H("getUTCDay", V, 0),
      utcyear: H("getUTCFullYear", V, 0),
      utcmonth: H("getUTCMonth", V, 0),
      utchours: H("getUTCHours", V, 0),
      utcminutes: H("getUTCMinutes", V, 0),
      utcseconds: H("getUTCSeconds", V, 0),
      utcmilliseconds: H("getUTCMilliseconds", V, 0),
      // sequence functions
      length: H("length", null, -1),
      // STRING functions
      parseFloat: "parseFloat",
      parseInt: "parseInt",
      upper: H("toUpperCase", q, 0),
      lower: H("toLowerCase", q, 0),
      substring: H("substring", q),
      split: H("split", q),
      trim: H("trim", q, 0),
      // REGEXP functions
      regexp: G,
      test: H("test", G),
      // Control Flow functions
      if: function(J) {
        J.length < 3 && error$1("Missing arguments to if function."), J.length > 3 && error$1("Too many arguments to if function.");
        const Z = J.map(O);
        return "(" + Z[0] + "?" + Z[1] + ":" + Z[2] + ")";
      }
    };
  }
  function stripQuotes(O) {
    const _ = O && O.length - 1;
    return _ && (O[0] === '"' && O[_] === '"' || O[0] === "'" && O[_] === "'") ? O.slice(1, -1) : O;
  }
  function codegen(O) {
    O = O || {};
    const _ = O.allowed ? toSet$1(O.allowed) : {}, H = O.forbidden ? toSet$1(O.forbidden) : {}, V = O.constants || Constants$1, q = (O.functions || Functions$1)(ae), G = O.globalvar, J = O.fieldvar, Z = isFunction$2(G) ? G : (ce) => `${G}["${ce}"]`;
    let te = {}, ne = {}, ee = 0;
    function ae(ce) {
      if (isString$1(ce)) return ce;
      const le = oe[ce.type];
      return le == null && error$1("Unsupported type: " + ce.type), le(ce);
    }
    const oe = {
      Literal: (ce) => ce.raw,
      Identifier: (ce) => {
        const le = ce.name;
        return ee > 0 ? le : has$1(H, le) ? error$1("Illegal identifier: " + le) : has$1(V, le) ? V[le] : has$1(_, le) ? le : (te[le] = 1, Z(le));
      },
      MemberExpression: (ce) => {
        const le = !ce.computed, ue = ae(ce.object);
        le && (ee += 1);
        const de = ae(ce.property);
        return ue === J && (ne[stripQuotes(de)] = 1), le && (ee -= 1), ue + (le ? "." + de : "[" + de + "]");
      },
      CallExpression: (ce) => {
        ce.callee.type !== "Identifier" && error$1("Illegal callee type: " + ce.callee.type);
        const le = ce.callee.name, ue = ce.arguments, de = has$1(q, le) && q[le];
        return de || error$1("Unrecognized function: " + le), isFunction$2(de) ? de(ue) : de + "(" + ue.map(ae).join(",") + ")";
      },
      ArrayExpression: (ce) => "[" + ce.elements.map(ae).join(",") + "]",
      BinaryExpression: (ce) => "(" + ae(ce.left) + " " + ce.operator + " " + ae(ce.right) + ")",
      UnaryExpression: (ce) => "(" + ce.operator + ae(ce.argument) + ")",
      ConditionalExpression: (ce) => "(" + ae(ce.test) + "?" + ae(ce.consequent) + ":" + ae(ce.alternate) + ")",
      LogicalExpression: (ce) => "(" + ae(ce.left) + ce.operator + ae(ce.right) + ")",
      ObjectExpression: (ce) => "{" + ce.properties.map(ae).join(",") + "}",
      Property: (ce) => {
        ee += 1;
        const le = ae(ce.key);
        return ee -= 1, le + ":" + ae(ce.value);
      }
    };
    function se(ce) {
      const le = {
        code: ae(ce),
        globals: Object.keys(te),
        fields: Object.keys(ne)
      };
      return te = {}, ne = {}, le;
    }
    return se.functions = q, se.constants = V, se;
  }
  const SELECTION_GETTER = Symbol("vega_selection_getter");
  function getter(O) {
    return (!O.getter || !O.getter[SELECTION_GETTER]) && (O.getter = field$1(O.field), O.getter[SELECTION_GETTER] = !0), O.getter;
  }
  const Intersect = "intersect", Union$1 = "union", VlMulti = "vlMulti", VlPoint = "vlPoint", Or = "or", And = "and", SelectionId = "_vgsid_", $selectionId = field$1(SelectionId), TYPE_ENUM = "E", TYPE_RANGE_INC = "R", TYPE_RANGE_EXC = "R-E", TYPE_RANGE_LE = "R-LE", TYPE_RANGE_RE = "R-RE", UNIT_INDEX = "index:unit";
  function testPoint(O, _) {
    for (var H = _.fields, V = _.values, q = H.length, G = 0, J, Z; G < q; ++G)
      if (Z = H[G], J = getter(Z)(O), isDate$1(J) && (J = toNumber$1(J)), isDate$1(V[G]) && (V[G] = toNumber$1(V[G])), isArray$2(V[G]) && isDate$1(V[G][0]) && (V[G] = V[G].map(toNumber$1)), Z.type === TYPE_ENUM) {
        if (isArray$2(V[G]) ? V[G].indexOf(J) < 0 : J !== V[G])
          return !1;
      } else if (Z.type === TYPE_RANGE_INC) {
        if (!inrange(J, V[G])) return !1;
      } else if (Z.type === TYPE_RANGE_RE) {
        if (!inrange(J, V[G], !0, !1)) return !1;
      } else if (Z.type === TYPE_RANGE_EXC) {
        if (!inrange(J, V[G], !1, !1)) return !1;
      } else if (Z.type === TYPE_RANGE_LE && !inrange(J, V[G], !1, !0))
        return !1;
    return !0;
  }
  function selectionTest(O, _, H) {
    for (var V = this.context.data[O], q = V ? V.values.value : [], G = V ? V[UNIT_INDEX] && V[UNIT_INDEX].value : void 0, J = H === Intersect, Z = q.length, te = 0, ne, ee, ae, oe, se; te < Z; ++te)
      if (ne = q[te], G && J) {
        if (ee = ee || {}, ae = ee[oe = ne.unit] || 0, ae === -1) continue;
        if (se = testPoint(_, ne), ee[oe] = se ? -1 : ++ae, se && G.size === 1) return !0;
        if (!se && ae === G.get(oe).count) return !1;
      } else if (se = testPoint(_, ne), J ^ se) return se;
    return Z && J;
  }
  const bisect = bisector($selectionId), bisectLeft = bisect.left, bisectRight = bisect.right;
  function selectionIdTest(O, _, H) {
    const V = this.context.data[O], q = V ? V.values.value : [], G = V ? V[UNIT_INDEX] && V[UNIT_INDEX].value : void 0, J = H === Intersect, Z = $selectionId(_), te = bisectLeft(q, Z);
    if (te === q.length || $selectionId(q[te]) !== Z) return !1;
    if (G && J) {
      if (G.size === 1) return !0;
      if (bisectRight(q, Z) - te < G.size) return !1;
    }
    return !0;
  }
  function selectionTuples(O, _) {
    return O.map((H) => extend$2(_.fields ? {
      values: _.fields.map((V) => getter(V)(H.datum))
    } : {
      [SelectionId]: $selectionId(H.datum)
    }, _));
  }
  function selectionResolve(O, _, H, V) {
    for (var q = this.context.data[O], G = q ? q.values.value : [], J = {}, Z = {}, te = {}, ne, ee, ae, oe, se, ce, le, ue, de, fe, he = G.length, me = 0, pe, ye; me < he; ++me)
      if (ne = G[me], oe = ne.unit, ee = ne.fields, ae = ne.values, ee && ae) {
        for (pe = 0, ye = ee.length; pe < ye; ++pe)
          se = ee[pe], le = J[se.field] || (J[se.field] = {}), ue = le[oe] || (le[oe] = []), te[se.field] = de = se.type.charAt(0), fe = ops[`${de}_union`], le[oe] = fe(ue, array$5(ae[pe]));
        H && (ue = Z[oe] || (Z[oe] = []), ue.push(array$5(ae).reduce((ge, ve, be) => (ge[ee[be].field] = ve, ge), {})));
      } else
        se = SelectionId, ce = $selectionId(ne), le = J[se] || (J[se] = {}), ue = le[oe] || (le[oe] = []), ue.push(ce), H && (ue = Z[oe] || (Z[oe] = []), ue.push({
          [SelectionId]: ce
        }));
    if (_ = _ || Union$1, J[SelectionId] ? J[SelectionId] = ops[`${SelectionId}_${_}`](...Object.values(J[SelectionId])) : Object.keys(J).forEach((ge) => {
      J[ge] = Object.keys(J[ge]).map((ve) => J[ge][ve]).reduce((ve, be) => ve === void 0 ? be : ops[`${te[ge]}_${_}`](ve, be));
    }), G = Object.keys(Z), H && G.length) {
      const ge = V ? VlPoint : VlMulti;
      J[ge] = _ === Union$1 ? {
        [Or]: G.reduce((ve, be) => (ve.push(...Z[be]), ve), [])
      } : {
        [And]: G.map((ve) => ({
          [Or]: Z[ve]
        }))
      };
    }
    return J;
  }
  var ops = {
    [`${SelectionId}_union`]: union,
    [`${SelectionId}_intersect`]: intersection,
    E_union: function(O, _) {
      if (!O.length) return _;
      for (var H = 0, V = _.length; H < V; ++H) O.indexOf(_[H]) < 0 && O.push(_[H]);
      return O;
    },
    E_intersect: function(O, _) {
      return O.length ? O.filter((H) => _.indexOf(H) >= 0) : _;
    },
    R_union: function(O, _) {
      var H = toNumber$1(_[0]), V = toNumber$1(_[1]);
      return H > V && (H = _[1], V = _[0]), O.length ? (O[0] > H && (O[0] = H), O[1] < V && (O[1] = V), O) : [H, V];
    },
    R_intersect: function(O, _) {
      var H = toNumber$1(_[0]), V = toNumber$1(_[1]);
      return H > V && (H = _[1], V = _[0]), O.length ? V < O[0] || O[1] < H ? [] : (O[0] < H && (O[0] = H), O[1] > V && (O[1] = V), O) : [H, V];
    }
  };
  const DataPrefix$1 = ":", IndexPrefix$1 = "@";
  function selectionVisitor(O, _, H, V) {
    _[0].type !== Literal && error$1("First argument to selection functions must be a string literal.");
    const q = _[0].value, G = _.length >= 2 && peek$1(_).value, J = "unit", Z = IndexPrefix$1 + J, te = DataPrefix$1 + q;
    G === Intersect && !has$1(V, Z) && (V[Z] = H.getData(q).indataRef(H, J)), has$1(V, te) || (V[te] = H.getData(q).tuplesRef());
  }
  function data$1(O) {
    const _ = this.context.data[O];
    return _ ? _.values.value : [];
  }
  function indata(O, _, H) {
    const V = this.context.data[O]["index:" + _], q = V ? V.value.get(H) : void 0;
    return q && q.count;
  }
  function setdata(O, _) {
    const H = this.context.dataflow, V = this.context.data[O], q = V.input;
    return H.pulse(q, H.changeset().remove(truthy).insert(_)), 1;
  }
  function encode(O, _, H) {
    if (O) {
      const V = this.context.dataflow, q = O.mark.source;
      V.pulse(q, V.changeset().encode(O, _));
    }
    return H !== void 0 ? H : O;
  }
  const wrap = (O) => function(_, H) {
    const V = this.context.dataflow.locale();
    return _ === null ? "null" : V[O](H)(_);
  }, format = wrap("format"), timeFormat$1 = wrap("timeFormat"), utcFormat = wrap("utcFormat"), timeParse = wrap("timeParse"), utcParse = wrap("utcParse"), dateObj = new Date(2e3, 0, 1);
  function time$1(O, _, H) {
    return !Number.isInteger(O) || !Number.isInteger(_) ? "" : (dateObj.setYear(2e3), dateObj.setMonth(O), dateObj.setDate(_), timeFormat$1.call(this, dateObj, H));
  }
  function monthFormat(O) {
    return time$1.call(this, O, 1, "%B");
  }
  function monthAbbrevFormat(O) {
    return time$1.call(this, O, 1, "%b");
  }
  function dayFormat(O) {
    return time$1.call(this, 0, 2 + O, "%A");
  }
  function dayAbbrevFormat(O) {
    return time$1.call(this, 0, 2 + O, "%a");
  }
  const DataPrefix = ":", IndexPrefix = "@", ScalePrefix = "%", SignalPrefix = "$";
  function dataVisitor(O, _, H, V) {
    _[0].type !== Literal && error$1("First argument to data functions must be a string literal.");
    const q = _[0].value, G = DataPrefix + q;
    if (!has$1(G, V))
      try {
        V[G] = H.getData(q).tuplesRef();
      } catch {
      }
  }
  function indataVisitor(O, _, H, V) {
    _[0].type !== Literal && error$1("First argument to indata must be a string literal."), _[1].type !== Literal && error$1("Second argument to indata must be a string literal.");
    const q = _[0].value, G = _[1].value, J = IndexPrefix + G;
    has$1(J, V) || (V[J] = H.getData(q).indataRef(H, G));
  }
  function scaleVisitor(O, _, H, V) {
    if (_[0].type === Literal)
      addScaleDependency(H, V, _[0].value);
    else
      for (O in H.scales)
        addScaleDependency(H, V, O);
  }
  function addScaleDependency(O, _, H) {
    const V = ScalePrefix + H;
    if (!has$1(_, V))
      try {
        _[V] = O.scaleRef(H);
      } catch {
      }
  }
  function getScale$1(O, _) {
    if (isFunction$2(O))
      return O;
    if (isString$1(O)) {
      const H = _.scales[O];
      return H && isRegisteredScale(H.value) ? H.value : void 0;
    }
  }
  function internalScaleFunctions(O, _, H) {
    _.__bandwidth = (q) => q && q.bandwidth ? q.bandwidth() : 0, H._bandwidth = scaleVisitor, H._range = scaleVisitor, H._scale = scaleVisitor;
    const V = (q) => "_[" + (q.type === Literal ? $$2(ScalePrefix + q.value) : $$2(ScalePrefix) + "+" + O(q)) + "]";
    return {
      _bandwidth: (q) => `this.__bandwidth(${V(q[0])})`,
      _range: (q) => `${V(q[0])}.range()`,
      _scale: (q) => `${V(q[0])}(${O(q[1])})`
    };
  }
  function geoMethod(O, _) {
    return function(H, V, q) {
      if (H) {
        const G = getScale$1(H, (q || this).context);
        return G && G.path[O](V);
      } else
        return _(V);
    };
  }
  const geoArea = geoMethod("area", geoArea$1), geoBounds = geoMethod("bounds", geoBounds$1), geoCentroid = geoMethod("centroid", geoCentroid$1);
  function geoScale(O, _) {
    const H = getScale$1(O, (_ || this).context);
    return H && H.scale();
  }
  function inScope(O) {
    const _ = this.context.group;
    let H = !1;
    if (_) for (; O; ) {
      if (O === _) {
        H = !0;
        break;
      }
      O = O.mark.group;
    }
    return H;
  }
  function log(O, _, H) {
    try {
      O[_].apply(O, ["EXPRESSION"].concat([].slice.call(H)));
    } catch (V) {
      O.warn(V);
    }
    return H[H.length - 1];
  }
  function warn$1() {
    return log(this.context.dataflow, "warn", arguments);
  }
  function info() {
    return log(this.context.dataflow, "info", arguments);
  }
  function debug$1() {
    return log(this.context.dataflow, "debug", arguments);
  }
  function channel_luminance_value(O) {
    const _ = O / 255;
    return _ <= 0.03928 ? _ / 12.92 : Math.pow((_ + 0.055) / 1.055, 2.4);
  }
  function luminance(O) {
    const _ = rgb$1(O), H = channel_luminance_value(_.r), V = channel_luminance_value(_.g), q = channel_luminance_value(_.b);
    return 0.2126 * H + 0.7152 * V + 0.0722 * q;
  }
  function contrast(O, _) {
    const H = luminance(O), V = luminance(_), q = Math.max(H, V), G = Math.min(H, V);
    return (q + 0.05) / (G + 0.05);
  }
  function merge() {
    const O = [].slice.call(arguments);
    return O.unshift({}), extend$2(...O);
  }
  function equal(O, _) {
    return O === _ || O !== O && _ !== _ ? !0 : isArray$2(O) ? isArray$2(_) && O.length === _.length ? equalArray(O, _) : !1 : isObject$2(O) && isObject$2(_) ? equalObject(O, _) : !1;
  }
  function equalArray(O, _) {
    for (let H = 0, V = O.length; H < V; ++H)
      if (!equal(O[H], _[H])) return !1;
    return !0;
  }
  function equalObject(O, _) {
    for (const H in O)
      if (!equal(O[H], _[H])) return !1;
    return !0;
  }
  function removePredicate(O) {
    return (_) => equalObject(O, _);
  }
  function modify(O, _, H, V, q, G) {
    const J = this.context.dataflow, Z = this.context.data[O], te = Z.input, ne = J.stamp();
    let ee = Z.changes, ae, oe;
    if (J._trigger === !1 || !(te.value.length || _ || V))
      return 0;
    if ((!ee || ee.stamp < ne) && (Z.changes = ee = J.changeset(), ee.stamp = ne, J.runAfter(() => {
      Z.modified = !0, J.pulse(te, ee).run();
    }, !0, 1)), H && (ae = H === !0 ? truthy : isArray$2(H) || isTuple(H) ? H : removePredicate(H), ee.remove(ae)), _ && ee.insert(_), V && (ae = removePredicate(V), te.value.some(ae) ? ee.remove(ae) : ee.insert(V)), q)
      for (oe in G)
        ee.modify(q, oe, G[oe]);
    return 1;
  }
  function pinchDistance(O) {
    const _ = O.touches, H = _[0].clientX - _[1].clientX, V = _[0].clientY - _[1].clientY;
    return Math.hypot(H, V);
  }
  function pinchAngle(O) {
    const _ = O.touches;
    return Math.atan2(_[0].clientY - _[1].clientY, _[0].clientX - _[1].clientX);
  }
  const accessors = {};
  function pluck(O, _) {
    const H = accessors[_] || (accessors[_] = field$1(_));
    return isArray$2(O) ? O.map(H) : H(O);
  }
  function array(O) {
    return isArray$2(O) || ArrayBuffer.isView(O) ? O : null;
  }
  function sequence(O) {
    return array(O) || (isString$1(O) ? O : null);
  }
  function join(O) {
    for (var _ = arguments.length, H = new Array(_ > 1 ? _ - 1 : 0), V = 1; V < _; V++)
      H[V - 1] = arguments[V];
    return array(O).join(...H);
  }
  function indexof(O) {
    for (var _ = arguments.length, H = new Array(_ > 1 ? _ - 1 : 0), V = 1; V < _; V++)
      H[V - 1] = arguments[V];
    return sequence(O).indexOf(...H);
  }
  function lastindexof(O) {
    for (var _ = arguments.length, H = new Array(_ > 1 ? _ - 1 : 0), V = 1; V < _; V++)
      H[V - 1] = arguments[V];
    return sequence(O).lastIndexOf(...H);
  }
  function slice$3(O) {
    for (var _ = arguments.length, H = new Array(_ > 1 ? _ - 1 : 0), V = 1; V < _; V++)
      H[V - 1] = arguments[V];
    return sequence(O).slice(...H);
  }
  function replace$1(O, _, H) {
    return isFunction$2(H) && error$1("Function argument passed to replace."), String(O).replace(_, H);
  }
  function reverse$2(O) {
    return array(O).slice().reverse();
  }
  function bandspace(O, _, H) {
    return bandSpace(O || 0, _ || 0, H || 0);
  }
  function bandwidth(O, _) {
    const H = getScale$1(O, (_ || this).context);
    return H && H.bandwidth ? H.bandwidth() : 0;
  }
  function copy(O, _) {
    const H = getScale$1(O, (_ || this).context);
    return H ? H.copy() : void 0;
  }
  function domain$2(O, _) {
    const H = getScale$1(O, (_ || this).context);
    return H ? H.domain() : [];
  }
  function invert(O, _, H) {
    const V = getScale$1(O, (H || this).context);
    return V ? isArray$2(_) ? (V.invertRange || V.invert)(_) : (V.invert || V.invertExtent)(_) : void 0;
  }
  function range$5(O, _) {
    const H = getScale$1(O, (_ || this).context);
    return H && H.range ? H.range() : [];
  }
  function scale$2(O, _, H) {
    const V = getScale$1(O, (H || this).context);
    return V ? V(_) : void 0;
  }
  function scaleGradient(O, _, H, V, q) {
    O = getScale$1(O, (q || this).context);
    const G = Gradient$1(_, H);
    let J = O.domain(), Z = J[0], te = peek$1(J), ne = identity$7;
    return te - Z ? ne = scaleFraction(O, Z, te) : O = (O.interpolator ? scale$4("sequential")().interpolator(O.interpolator()) : scale$4("linear")().interpolate(O.interpolate()).range(O.range())).domain([Z = 0, te = 1]), O.ticks && (J = O.ticks(+V || 15), Z !== J[0] && J.unshift(Z), te !== peek$1(J) && J.push(te)), J.forEach((ee) => G.stop(ne(ee), O(ee))), G;
  }
  function geoShape(O, _, H) {
    const V = getScale$1(O, (H || this).context);
    return function(q) {
      return V ? V.path.context(q)(_) : "";
    };
  }
  function pathShape(O) {
    let _ = null;
    return function(H) {
      return H ? pathRender(H, _ = _ || parse$3(O)) : O;
    };
  }
  const datum = (O) => O.data;
  function treeNodes(O, _) {
    const H = data$1.call(_, O);
    return H.root && H.root.lookup || {};
  }
  function treePath(O, _, H) {
    const V = treeNodes(O, this), q = V[_], G = V[H];
    return q && G ? q.path(G).map(datum) : void 0;
  }
  function treeAncestors(O, _) {
    const H = treeNodes(O, this)[_];
    return H ? H.ancestors().map(datum) : void 0;
  }
  const _window = () => typeof window < "u" && window || null;
  function screen() {
    const O = _window();
    return O ? O.screen : {};
  }
  function windowSize() {
    const O = _window();
    return O ? [O.innerWidth, O.innerHeight] : [void 0, void 0];
  }
  function containerSize() {
    const O = this.context.dataflow, _ = O.container && O.container();
    return _ ? [_.clientWidth, _.clientHeight] : [void 0, void 0];
  }
  function intersect(O, _, H) {
    if (!O) return [];
    const [V, q] = O, G = new Bounds().set(V[0], V[1], q[0], q[1]), J = H || this.context.dataflow.scenegraph().root;
    return intersect$2(J, G, filter(_));
  }
  function filter(O) {
    let _ = null;
    if (O) {
      const H = array$5(O.marktype), V = array$5(O.markname);
      _ = (q) => (!H.length || H.some((G) => q.marktype === G)) && (!V.length || V.some((G) => q.name === G));
    }
    return _;
  }
  function lassoAppend(O, _, H) {
    let V = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : 5;
    O = array$5(O);
    const q = O[O.length - 1];
    return q === void 0 || Math.hypot(q[0] - _, q[1] - H) > V ? [...O, [_, H]] : O;
  }
  function lassoPath(O) {
    return array$5(O).reduce((_, H, V) => {
      let [q, G] = H;
      return _ += V == 0 ? `M ${q},${G} ` : V === O.length - 1 ? " Z" : `L ${q},${G} `;
    }, "");
  }
  function intersectLasso(O, _, H) {
    const {
      x: V,
      y: q,
      mark: G
    } = H, J = new Bounds().set(Number.MAX_SAFE_INTEGER, Number.MAX_SAFE_INTEGER, Number.MIN_SAFE_INTEGER, Number.MIN_SAFE_INTEGER);
    for (const [te, ne] of _)
      te < J.x1 && (J.x1 = te), te > J.x2 && (J.x2 = te), ne < J.y1 && (J.y1 = ne), ne > J.y2 && (J.y2 = ne);
    return J.translate(V, q), intersect([[J.x1, J.y1], [J.x2, J.y2]], O, G).filter((te) => pointInPolygon(te.x, te.y, _));
  }
  function pointInPolygon(O, _, H) {
    let V = 0;
    for (let q = 0, G = H.length - 1; q < H.length; G = q++) {
      const [J, Z] = H[G], [te, ne] = H[q];
      ne > _ != Z > _ && O < (J - te) * (_ - ne) / (Z - ne) + te && V++;
    }
    return V & 1;
  }
  const functionContext = {
    random() {
      return random();
    },
    // override default
    cumulativeNormal,
    cumulativeLogNormal,
    cumulativeUniform,
    densityNormal,
    densityLogNormal,
    densityUniform,
    quantileNormal,
    quantileLogNormal,
    quantileUniform,
    sampleNormal,
    sampleLogNormal,
    sampleUniform,
    isArray: isArray$2,
    isBoolean: isBoolean$3,
    isDate: isDate$1,
    isDefined(O) {
      return O !== void 0;
    },
    isNumber: isNumber$2,
    isObject: isObject$2,
    isRegExp,
    isString: isString$1,
    isTuple,
    isValid(O) {
      return O != null && O === O;
    },
    toBoolean,
    toDate(O) {
      return toDate(O);
    },
    // suppress extra arguments
    toNumber: toNumber$1,
    toString: toString$1,
    indexof,
    join,
    lastindexof,
    replace: replace$1,
    reverse: reverse$2,
    slice: slice$3,
    flush: flush$1,
    lerp,
    merge,
    pad: pad$2,
    peek: peek$1,
    pluck,
    span,
    inrange,
    truncate: truncate$1,
    rgb: rgb$1,
    lab: lab$1,
    hcl: hcl$2,
    hsl: hsl$2,
    luminance,
    contrast,
    sequence: range$7,
    format,
    utcFormat,
    utcParse,
    utcOffset,
    utcSequence,
    timeFormat: timeFormat$1,
    timeParse,
    timeOffset,
    timeSequence,
    timeUnitSpecifier,
    monthFormat,
    monthAbbrevFormat,
    dayFormat,
    dayAbbrevFormat,
    quarter,
    utcquarter,
    week,
    utcweek,
    dayofyear,
    utcdayofyear,
    warn: warn$1,
    info,
    debug: debug$1,
    extent(O) {
      return extent(O);
    },
    // suppress extra arguments
    inScope,
    intersect,
    clampRange: clampRange$1,
    pinchDistance,
    pinchAngle,
    screen,
    containerSize,
    windowSize,
    bandspace,
    setdata,
    pathShape,
    panLinear,
    panLog,
    panPow,
    panSymlog,
    zoomLinear,
    zoomLog,
    zoomPow,
    zoomSymlog,
    encode,
    modify,
    lassoAppend,
    lassoPath,
    intersectLasso
  }, eventFunctions = ["view", "item", "group", "xy", "x", "y"], eventPrefix = "event.vega.", thisPrefix = "this.", astVisitors = {}, codegenParams = {
    forbidden: ["_"],
    allowed: ["datum", "event", "item"],
    fieldvar: "datum",
    globalvar: (O) => `_[${$$2(SignalPrefix + O)}]`,
    functions: buildFunctions,
    constants: Constants$1,
    visitors: astVisitors
  }, codeGenerator = codegen(codegenParams);
  function buildFunctions(O) {
    const _ = Functions$1(O);
    eventFunctions.forEach((H) => _[H] = eventPrefix + H);
    for (const H in functionContext)
      _[H] = thisPrefix + H;
    return extend$2(_, internalScaleFunctions(O, functionContext, astVisitors)), _;
  }
  function expressionFunction(O, _, H) {
    return arguments.length === 1 ? functionContext[O] : (functionContext[O] = _, H && (astVisitors[O] = H), codeGenerator.functions[O] = thisPrefix + O, this);
  }
  expressionFunction("bandwidth", bandwidth, scaleVisitor);
  expressionFunction("copy", copy, scaleVisitor);
  expressionFunction("domain", domain$2, scaleVisitor);
  expressionFunction("range", range$5, scaleVisitor);
  expressionFunction("invert", invert, scaleVisitor);
  expressionFunction("scale", scale$2, scaleVisitor);
  expressionFunction("gradient", scaleGradient, scaleVisitor);
  expressionFunction("geoArea", geoArea, scaleVisitor);
  expressionFunction("geoBounds", geoBounds, scaleVisitor);
  expressionFunction("geoCentroid", geoCentroid, scaleVisitor);
  expressionFunction("geoShape", geoShape, scaleVisitor);
  expressionFunction("geoScale", geoScale, scaleVisitor);
  expressionFunction("indata", indata, indataVisitor);
  expressionFunction("data", data$1, dataVisitor);
  expressionFunction("treePath", treePath, dataVisitor);
  expressionFunction("treeAncestors", treeAncestors, dataVisitor);
  expressionFunction("vlSelectionTest", selectionTest, selectionVisitor);
  expressionFunction("vlSelectionIdTest", selectionIdTest, selectionVisitor);
  expressionFunction("vlSelectionResolve", selectionResolve, selectionVisitor);
  expressionFunction("vlSelectionTuples", selectionTuples);
  function parser$2(O, _) {
    const H = {};
    let V;
    try {
      O = isString$1(O) ? O : $$2(O) + "", V = parser$3(O);
    } catch {
      error$1("Expression parse error: " + O);
    }
    V.visit((G) => {
      if (G.type !== CallExpression) return;
      const J = G.callee.name, Z = codegenParams.visitors[J];
      Z && Z(J, G.arguments, _, H);
    });
    const q = codeGenerator(V);
    return q.globals.forEach((G) => {
      const J = SignalPrefix + G;
      !has$1(H, J) && _.getSignal(G) && (H[J] = _.signalRef(G));
    }), {
      $expr: extend$2({
        code: q.code
      }, _.options.ast ? {
        ast: V
      } : null),
      $fields: q.fields,
      $params: H
    };
  }
  function parse$2(O) {
    const _ = this, H = O.operators || [];
    return O.background && (_.background = O.background), O.eventConfig && (_.eventConfig = O.eventConfig), O.locale && (_.locale = O.locale), H.forEach((V) => _.parseOperator(V)), H.forEach((V) => _.parseOperatorParameters(V)), (O.streams || []).forEach((V) => _.parseStream(V)), (O.updates || []).forEach((V) => _.parseUpdate(V)), _.resolve();
  }
  const Skip$2 = toSet$1(["rule"]), Swap = toSet$1(["group", "image", "rect"]);
  function adjustSpatial$1(O, _) {
    let H = "";
    return Skip$2[_] || (O.x2 && (O.x ? (Swap[_] && (H += "if(o.x>o.x2)$=o.x,o.x=o.x2,o.x2=$;"), H += "o.width=o.x2-o.x;") : H += "o.x=o.x2-(o.width||0);"), O.xc && (H += "o.x=o.xc-(o.width||0)/2;"), O.y2 && (O.y ? (Swap[_] && (H += "if(o.y>o.y2)$=o.y,o.y=o.y2,o.y2=$;"), H += "o.height=o.y2-o.y;") : H += "o.y=o.y2-(o.height||0);"), O.yc && (H += "o.y=o.yc-(o.height||0)/2;")), H;
  }
  function canonicalType(O) {
    return (O + "").toLowerCase();
  }
  function isOperator(O) {
    return canonicalType(O) === "operator";
  }
  function isCollect(O) {
    return canonicalType(O) === "collect";
  }
  function expression$2(O, _, H) {
    H.endsWith(";") || (H = "return(" + H + ");");
    const V = Function(..._.concat(H));
    return O && O.functions ? V.bind(O.functions) : V;
  }
  function _compare(O, _, H, V) {
    return `((u = ${O}) < (v = ${_}) || u == null) && v != null ? ${H}
  : (u > v || v == null) && u != null ? ${V}
  : ((v = v instanceof Date ? +v : v), (u = u instanceof Date ? +u : u)) !== u && v === v ? ${H}
  : v !== v && u === u ? ${V} : `;
  }
  var expressionCodegen = {
    /**
     * Parse an expression used to update an operator value.
     */
    operator: (O, _) => expression$2(O, ["_"], _.code),
    /**
     * Parse an expression provided as an operator parameter value.
     */
    parameter: (O, _) => expression$2(O, ["datum", "_"], _.code),
    /**
     * Parse an expression applied to an event stream.
     */
    event: (O, _) => expression$2(O, ["event"], _.code),
    /**
     * Parse an expression used to handle an event-driven operator update.
     */
    handler: (O, _) => {
      const H = `var datum=event.item&&event.item.datum;return ${_.code};`;
      return expression$2(O, ["_", "event"], H);
    },
    /**
     * Parse an expression that performs visual encoding.
     */
    encode: (O, _) => {
      const {
        marktype: H,
        channels: V
      } = _;
      let q = "var o=item,datum=o.datum,m=0,$;";
      for (const G in V) {
        const J = "o[" + $$2(G) + "]";
        q += `$=${V[G].code};if(${J}!==$)${J}=$,m=1;`;
      }
      return q += adjustSpatial$1(V, H), q += "return m;", expression$2(O, ["item", "_"], q);
    },
    /**
     * Optimized code generators for access and comparison.
     */
    codegen: {
      get(O) {
        const _ = `[${O.map($$2).join("][")}]`, H = Function("_", `return _${_};`);
        return H.path = _, H;
      },
      comparator(O, _) {
        let H;
        const V = (G, J) => {
          const Z = _[J];
          let te, ne;
          return G.path ? (te = `a${G.path}`, ne = `b${G.path}`) : ((H = H || {})["f" + J] = G, te = `this.f${J}(a)`, ne = `this.f${J}(b)`), _compare(te, ne, -Z, Z);
        }, q = Function("a", "b", "var u, v; return " + O.map(V).join("") + "0;");
        return H ? q.bind(H) : q;
      }
    }
  };
  function parseOperator(O) {
    const _ = this;
    isOperator(O.type) || !O.type ? _.operator(O, O.update ? _.operatorExpression(O.update) : null) : _.transform(O, O.type);
  }
  function parseOperatorParameters(O) {
    const _ = this;
    if (O.params) {
      const H = _.get(O.id);
      H || error$1("Invalid operator id: " + O.id), _.dataflow.connect(H, H.parameters(_.parseParameters(O.params), O.react, O.initonly));
    }
  }
  function parseParameters$1(O, _) {
    _ = _ || {};
    const H = this;
    for (const V in O) {
      const q = O[V];
      _[V] = isArray$2(q) ? q.map((G) => parseParameter$2(G, H, _)) : parseParameter$2(q, H, _);
    }
    return _;
  }
  function parseParameter$2(O, _, H) {
    if (!O || !isObject$2(O)) return O;
    for (let V = 0, q = PARSERS.length, G; V < q; ++V)
      if (G = PARSERS[V], has$1(O, G.key))
        return G.parse(O, _, H);
    return O;
  }
  var PARSERS = [{
    key: "$ref",
    parse: getOperator
  }, {
    key: "$key",
    parse: getKey
  }, {
    key: "$expr",
    parse: getExpression
  }, {
    key: "$field",
    parse: getField
  }, {
    key: "$encode",
    parse: getEncode
  }, {
    key: "$compare",
    parse: getCompare
  }, {
    key: "$context",
    parse: getContext
  }, {
    key: "$subflow",
    parse: getSubflow
  }, {
    key: "$tupleid",
    parse: getTupleId
  }];
  function getOperator(O, _) {
    return _.get(O.$ref) || error$1("Operator not defined: " + O.$ref);
  }
  function getExpression(O, _, H) {
    O.$params && _.parseParameters(O.$params, H);
    const V = "e:" + O.$expr.code;
    return _.fn[V] || (_.fn[V] = accessor(_.parameterExpression(O.$expr), O.$fields));
  }
  function getKey(O, _) {
    const H = "k:" + O.$key + "_" + !!O.$flat;
    return _.fn[H] || (_.fn[H] = key(O.$key, O.$flat, _.expr.codegen));
  }
  function getField(O, _) {
    if (!O.$field) return null;
    const H = "f:" + O.$field + "_" + O.$name;
    return _.fn[H] || (_.fn[H] = field$1(O.$field, O.$name, _.expr.codegen));
  }
  function getCompare(O, _) {
    const H = "c:" + O.$compare + "_" + O.$order, V = array$5(O.$compare).map((q) => q && q.$tupleid ? tupleid : q);
    return _.fn[H] || (_.fn[H] = compare$3(V, O.$order, _.expr.codegen));
  }
  function getEncode(O, _) {
    const H = O.$encode, V = {};
    for (const q in H) {
      const G = H[q];
      V[q] = accessor(_.encodeExpression(G.$expr), G.$fields), V[q].output = G.$output;
    }
    return V;
  }
  function getContext(O, _) {
    return _;
  }
  function getSubflow(O, _) {
    const H = O.$subflow;
    return function(V, q, G) {
      const J = _.fork().parse(H), Z = J.get(H.operators[0].id), te = J.signals.parent;
      return te && te.set(G), Z.detachSubflow = () => _.detach(J), Z;
    };
  }
  function getTupleId() {
    return tupleid;
  }
  function parseStream$2(O) {
    var _ = this, H = O.filter != null ? _.eventExpression(O.filter) : void 0, V = O.stream != null ? _.get(O.stream) : void 0, q;
    O.source ? V = _.events(O.source, O.type, H) : O.merge && (q = O.merge.map((G) => _.get(G)), V = q[0].merge.apply(q[0], q.slice(1))), O.between && (q = O.between.map((G) => _.get(G)), V = V.between(q[0], q[1])), O.filter && (V = V.filter(H)), O.throttle != null && (V = V.throttle(+O.throttle)), O.debounce != null && (V = V.debounce(+O.debounce)), V == null && error$1("Invalid stream definition: " + JSON.stringify(O)), O.consume && V.consume(!0), _.stream(O, V);
  }
  function parseUpdate$1(O) {
    var _ = this, H = isObject$2(H = O.source) ? H.$ref : H, V = _.get(H), q = null, G = O.update, J = void 0;
    V || error$1("Source not defined: " + O.source), q = O.target && O.target.$expr ? _.eventExpression(O.target.$expr) : _.get(O.target), G && G.$expr && (G.$params && (J = _.parseParameters(G.$params)), G = _.handlerExpression(G.$expr)), _.update(O, V, q, G, J);
  }
  const SKIP = {
    skip: !0
  };
  function getState$1(O) {
    var _ = this, H = {};
    if (O.signals) {
      var V = H.signals = {};
      Object.keys(_.signals).forEach((G) => {
        const J = _.signals[G];
        O.signals(G, J) && (V[G] = J.value);
      });
    }
    if (O.data) {
      var q = H.data = {};
      Object.keys(_.data).forEach((G) => {
        const J = _.data[G];
        O.data(G, J) && (q[G] = J.input.value);
      });
    }
    return _.subcontext && O.recurse !== !1 && (H.subcontext = _.subcontext.map((G) => G.getState(O))), H;
  }
  function setState$1(O) {
    var _ = this, H = _.dataflow, V = O.data, q = O.signals;
    Object.keys(q || {}).forEach((G) => {
      H.update(_.signals[G], q[G], SKIP);
    }), Object.keys(V || {}).forEach((G) => {
      H.pulse(_.data[G].input, H.changeset().remove(truthy).insert(V[G]));
    }), (O.subcontext || []).forEach((G, J) => {
      const Z = _.subcontext[J];
      Z && Z.setState(G);
    });
  }
  function context(O, _, H, V) {
    return new Context(O, _, H, V);
  }
  function Context(O, _, H, V) {
    this.dataflow = O, this.transforms = _, this.events = O.events.bind(O), this.expr = V || expressionCodegen, this.signals = {}, this.scales = {}, this.nodes = {}, this.data = {}, this.fn = {}, H && (this.functions = Object.create(H), this.functions.context = this);
  }
  function Subcontext(O) {
    this.dataflow = O.dataflow, this.transforms = O.transforms, this.events = O.events, this.expr = O.expr, this.signals = Object.create(O.signals), this.scales = Object.create(O.scales), this.nodes = Object.create(O.nodes), this.data = Object.create(O.data), this.fn = Object.create(O.fn), O.functions && (this.functions = Object.create(O.functions), this.functions.context = this);
  }
  Context.prototype = Subcontext.prototype = {
    fork() {
      const O = new Subcontext(this);
      return (this.subcontext || (this.subcontext = [])).push(O), O;
    },
    detach(O) {
      this.subcontext = this.subcontext.filter((H) => H !== O);
      const _ = Object.keys(O.nodes);
      for (const H of _) O.nodes[H]._targets = null;
      for (const H of _) O.nodes[H].detach();
      O.nodes = null;
    },
    get(O) {
      return this.nodes[O];
    },
    set(O, _) {
      return this.nodes[O] = _;
    },
    add(O, _) {
      const H = this, V = H.dataflow, q = O.value;
      if (H.set(O.id, _), isCollect(O.type) && q && (q.$ingest ? V.ingest(_, q.$ingest, q.$format) : q.$request ? V.preload(_, q.$request, q.$format) : V.pulse(_, V.changeset().insert(q))), O.root && (H.root = _), O.parent) {
        let G = H.get(O.parent.$ref);
        G ? (V.connect(G, [_]), _.targets().add(G)) : (H.unresolved = H.unresolved || []).push(() => {
          G = H.get(O.parent.$ref), V.connect(G, [_]), _.targets().add(G);
        });
      }
      if (O.signal && (H.signals[O.signal] = _), O.scale && (H.scales[O.scale] = _), O.data)
        for (const G in O.data) {
          const J = H.data[G] || (H.data[G] = {});
          O.data[G].forEach((Z) => J[Z] = _);
        }
    },
    resolve() {
      return (this.unresolved || []).forEach((O) => O()), delete this.unresolved, this;
    },
    operator(O, _) {
      this.add(O, this.dataflow.add(O.value, _));
    },
    transform(O, _) {
      this.add(O, this.dataflow.add(this.transforms[canonicalType(_)]));
    },
    stream(O, _) {
      this.set(O.id, _);
    },
    update(O, _, H, V, q) {
      this.dataflow.on(_, H, V, q, O.options);
    },
    // expression parsing
    operatorExpression(O) {
      return this.expr.operator(this, O);
    },
    parameterExpression(O) {
      return this.expr.parameter(this, O);
    },
    eventExpression(O) {
      return this.expr.event(this, O);
    },
    handlerExpression(O) {
      return this.expr.handler(this, O);
    },
    encodeExpression(O) {
      return this.expr.encode(this, O);
    },
    // parse methods
    parse: parse$2,
    parseOperator,
    parseOperatorParameters,
    parseParameters: parseParameters$1,
    parseStream: parseStream$2,
    parseUpdate: parseUpdate$1,
    // state methods
    getState: getState$1,
    setState: setState$1
  };
  function initializeAria(O) {
    const _ = O.container();
    _ && (_.setAttribute("role", "graphics-document"), _.setAttribute("aria-roleDescription", "visualization"), ariaLabel(_, O.description()));
  }
  function ariaLabel(O, _) {
    O && (_ == null ? O.removeAttribute("aria-label") : O.setAttribute("aria-label", _));
  }
  function background(O) {
    O.add(null, (_) => (O._background = _.bg, O._resize = 1, _.bg), {
      bg: O._signals.background
    });
  }
  const Default = "default";
  function cursor$1(O) {
    const _ = O._signals.cursor || (O._signals.cursor = O.add({
      user: Default,
      item: null
    }));
    O.on(O.events("view", "pointermove"), _, (H, V) => {
      const q = _.value, G = q ? isString$1(q) ? q : q.user : Default, J = V.item && V.item.cursor || null;
      return q && G === q.user && J == q.item ? q : {
        user: G,
        item: J
      };
    }), O.add(null, function(H) {
      let V = H.cursor, q = this.value;
      return isString$1(V) || (q = V.item, V = V.user), setCursor(O, V && V !== Default ? V : q || V), q;
    }, {
      cursor: _
    });
  }
  function setCursor(O, _) {
    const H = O.globalCursor() ? typeof document < "u" && document.body : O.container();
    if (H)
      return _ == null ? H.style.removeProperty("cursor") : H.style.cursor = _;
  }
  function dataref(O, _) {
    var H = O._runtime.data;
    return has$1(H, _) || error$1("Unrecognized data set: " + _), H[_];
  }
  function data(O, _) {
    return arguments.length < 2 ? dataref(this, O).values.value : change.call(this, O, changeset().remove(truthy).insert(_));
  }
  function change(O, _) {
    isChangeSet(_) || error$1("Second argument to changes must be a changeset.");
    const H = dataref(this, O);
    return H.modified = !0, this.pulse(H.input, _);
  }
  function insert$1(O, _) {
    return change.call(this, O, changeset().insert(_));
  }
  function remove$1(O, _) {
    return change.call(this, O, changeset().remove(_));
  }
  function width(O) {
    var _ = O.padding();
    return Math.max(0, O._viewWidth + _.left + _.right);
  }
  function height(O) {
    var _ = O.padding();
    return Math.max(0, O._viewHeight + _.top + _.bottom);
  }
  function offset(O) {
    var _ = O.padding(), H = O._origin;
    return [_.left + H[0], _.top + H[1]];
  }
  function resizeRenderer(O) {
    var _ = offset(O), H = width(O), V = height(O);
    O._renderer.background(O.background()), O._renderer.resize(H, V, _), O._handler.origin(_), O._resizeListeners.forEach((q) => {
      try {
        q(H, V);
      } catch (G) {
        O.error(G);
      }
    });
  }
  function eventExtend(O, _, H) {
    var V = O._renderer, q = V && V.canvas(), G, J, Z;
    return q && (Z = offset(O), J = _.changedTouches ? _.changedTouches[0] : _, G = point$2(J, q), G[0] -= Z[0], G[1] -= Z[1]), _.dataflow = O, _.item = H, _.vega = extension(O, H, G), _;
  }
  function extension(O, _, H) {
    const V = _ ? _.mark.marktype === "group" ? _ : _.mark.group : null;
    function q(J) {
      var Z = V, te;
      if (J) {
        for (te = _; te; te = te.mark.group)
          if (te.mark.name === J) {
            Z = te;
            break;
          }
      }
      return Z && Z.mark && Z.mark.interactive ? Z : {};
    }
    function G(J) {
      if (!J) return H;
      isString$1(J) && (J = q(J));
      const Z = H.slice();
      for (; J; )
        Z[0] -= J.x || 0, Z[1] -= J.y || 0, J = J.mark && J.mark.group;
      return Z;
    }
    return {
      view: constant$5(O),
      item: constant$5(_ || {}),
      group: q,
      xy: G,
      x: (J) => G(J)[0],
      y: (J) => G(J)[1]
    };
  }
  const VIEW$1 = "view", TIMER = "timer", WINDOW = "window", NO_TRAP = {
    trap: !1
  };
  function initializeEventConfig(O) {
    const _ = extend$2({
      defaults: {}
    }, O), H = (V, q) => {
      q.forEach((G) => {
        isArray$2(V[G]) && (V[G] = toSet$1(V[G]));
      });
    };
    return H(_.defaults, ["prevent", "allow"]), H(_, ["view", "window", "selector"]), _;
  }
  function trackEventListener(O, _, H, V) {
    O._eventListeners.push({
      type: H,
      sources: array$5(_),
      handler: V
    });
  }
  function prevent(O, _) {
    var H = O._eventConfig.defaults, V = H.prevent, q = H.allow;
    return V === !1 || q === !0 ? !1 : V === !0 || q === !1 ? !0 : V ? V[_] : q ? !q[_] : O.preventDefault();
  }
  function permit(O, _, H) {
    const V = O._eventConfig && O._eventConfig[_];
    return V === !1 || isObject$2(V) && !V[H] ? (O.warn(`Blocked ${_} ${H} event listener.`), !1) : !0;
  }
  function events$1(O, _, H) {
    var V = this, q = new EventStream(H), G = function(ne, ee) {
      V.runAsync(null, () => {
        O === VIEW$1 && prevent(V, _) && ne.preventDefault(), q.receive(eventExtend(V, ne, ee));
      });
    }, J;
    if (O === TIMER)
      permit(V, "timer", _) && V.timer(G, _);
    else if (O === VIEW$1)
      permit(V, "view", _) && V.addEventListener(_, G, NO_TRAP);
    else if (O === WINDOW ? permit(V, "window", _) && typeof window < "u" && (J = [window]) : typeof document < "u" && permit(V, "selector", _) && (J = Array.from(document.querySelectorAll(O))), !J)
      V.warn("Can not resolve event source: " + O);
    else {
      for (var Z = 0, te = J.length; Z < te; ++Z)
        J[Z].addEventListener(_, G);
      trackEventListener(V, J, _, G);
    }
    return q;
  }
  function itemFilter(O) {
    return O.item;
  }
  function markTarget(O) {
    return O.item.mark.source;
  }
  function invoke(O) {
    return function(_, H) {
      return H.vega.view().changeset().encode(H.item, O);
    };
  }
  function hover(O, _) {
    return O = [O || "hover"], _ = [_ || "update", O[0]], this.on(this.events("view", "pointerover", itemFilter), markTarget, invoke(O)), this.on(this.events("view", "pointerout", itemFilter), markTarget, invoke(_)), this;
  }
  function finalize() {
    var O = this._tooltip, _ = this._timers, H = this._handler.handlers(), V = this._eventListeners, q, G, J, Z, te;
    for (q = _.length; --q >= 0; )
      _[q].stop();
    for (q = V.length; --q >= 0; )
      for (J = V[q], G = J.sources.length; --G >= 0; )
        J.sources[G].removeEventListener(J.type, J.handler);
    for (O && O.call(this, this._handler, null, null, null), q = H.length; --q >= 0; )
      te = H[q].type, Z = H[q].handler, this._handler.off(te, Z);
    return this;
  }
  function element(O, _, H) {
    const V = document.createElement(O);
    for (const q in _) V.setAttribute(q, _[q]);
    return H != null && (V.textContent = H), V;
  }
  const BindClass = "vega-bind", NameClass = "vega-bind-name", RadioClass = "vega-bind-radio";
  function bind$1(O, _, H) {
    if (!_) return;
    const V = H.param;
    let q = H.state;
    return q || (q = H.state = {
      elements: null,
      active: !1,
      set: null,
      update: (J) => {
        J != O.signal(V.signal) && O.runAsync(null, () => {
          q.source = !0, O.signal(V.signal, J);
        });
      }
    }, V.debounce && (q.update = debounce$2(V.debounce, q.update))), (V.input == null && V.element ? target : generate$1)(q, _, V, O), q.active || (O.on(O._signals[V.signal], null, () => {
      q.source ? q.source = !1 : q.set(O.signal(V.signal));
    }), q.active = !0), q;
  }
  function target(O, _, H, V) {
    const q = H.event || "input", G = () => O.update(_.value);
    V.signal(H.signal, _.value), _.addEventListener(q, G), trackEventListener(V, _, q, G), O.set = (J) => {
      _.value = J, _.dispatchEvent(event(q));
    };
  }
  function event(O) {
    return typeof Event < "u" ? new Event(O) : {
      type: O
    };
  }
  function generate$1(O, _, H, V) {
    const q = V.signal(H.signal), G = element("div", {
      class: BindClass
    }), J = H.input === "radio" ? G : G.appendChild(element("label"));
    J.appendChild(element("span", {
      class: NameClass
    }, H.name || H.signal)), _.appendChild(G);
    let Z = form;
    switch (H.input) {
      case "checkbox":
        Z = checkbox;
        break;
      case "select":
        Z = select;
        break;
      case "radio":
        Z = radio;
        break;
      case "range":
        Z = range$4;
        break;
    }
    Z(O, J, H, q);
  }
  function form(O, _, H, V) {
    const q = element("input");
    for (const G in H)
      G !== "signal" && G !== "element" && q.setAttribute(G === "input" ? "type" : G, H[G]);
    q.setAttribute("name", H.signal), q.value = V, _.appendChild(q), q.addEventListener("input", () => O.update(q.value)), O.elements = [q], O.set = (G) => q.value = G;
  }
  function checkbox(O, _, H, V) {
    const q = {
      type: "checkbox",
      name: H.signal
    };
    V && (q.checked = !0);
    const G = element("input", q);
    _.appendChild(G), G.addEventListener("change", () => O.update(G.checked)), O.elements = [G], O.set = (J) => G.checked = !!J || null;
  }
  function select(O, _, H, V) {
    const q = element("select", {
      name: H.signal
    }), G = H.labels || [];
    H.options.forEach((J, Z) => {
      const te = {
        value: J
      };
      valuesEqual(J, V) && (te.selected = !0), q.appendChild(element("option", te, (G[Z] || J) + ""));
    }), _.appendChild(q), q.addEventListener("change", () => {
      O.update(H.options[q.selectedIndex]);
    }), O.elements = [q], O.set = (J) => {
      for (let Z = 0, te = H.options.length; Z < te; ++Z)
        if (valuesEqual(H.options[Z], J)) {
          q.selectedIndex = Z;
          return;
        }
    };
  }
  function radio(O, _, H, V) {
    const q = element("span", {
      class: RadioClass
    }), G = H.labels || [];
    _.appendChild(q), O.elements = H.options.map((J, Z) => {
      const te = {
        type: "radio",
        name: H.signal,
        value: J
      };
      valuesEqual(J, V) && (te.checked = !0);
      const ne = element("input", te);
      ne.addEventListener("change", () => O.update(J));
      const ee = element("label", {}, (G[Z] || J) + "");
      return ee.prepend(ne), q.appendChild(ee), ne;
    }), O.set = (J) => {
      const Z = O.elements, te = Z.length;
      for (let ne = 0; ne < te; ++ne)
        valuesEqual(Z[ne].value, J) && (Z[ne].checked = !0);
    };
  }
  function range$4(O, _, H, V) {
    V = V !== void 0 ? V : (+H.max + +H.min) / 2;
    const q = H.max != null ? H.max : Math.max(100, +V) || 100, G = H.min || Math.min(0, q, +V) || 0, J = H.step || tickStep(G, q, 100), Z = element("input", {
      type: "range",
      name: H.signal,
      min: G,
      max: q,
      step: J
    });
    Z.value = V;
    const te = element("span", {}, +V);
    _.appendChild(Z), _.appendChild(te);
    const ne = () => {
      te.textContent = Z.value, O.update(+Z.value);
    };
    Z.addEventListener("input", ne), Z.addEventListener("change", ne), O.elements = [Z], O.set = (ee) => {
      Z.value = ee, te.textContent = ee;
    };
  }
  function valuesEqual(O, _) {
    return O === _ || O + "" == _ + "";
  }
  function initializeRenderer(O, _, H, V, q, G) {
    return _ = _ || new V(O.loader()), _.initialize(H, width(O), height(O), offset(O), q, G).background(O.background());
  }
  function trap(O, _) {
    return _ ? function() {
      try {
        _.apply(this, arguments);
      } catch (H) {
        O.error(H);
      }
    } : null;
  }
  function initializeHandler(O, _, H, V) {
    const q = new V(O.loader(), trap(O, O.tooltip())).scene(O.scenegraph().root).initialize(H, offset(O), O);
    return _ && _.handlers().forEach((G) => {
      q.on(G.type, G.handler);
    }), q;
  }
  function initialize(O, _) {
    const H = this, V = H._renderType, q = H._eventConfig.bind, G = renderModule(V);
    O = H._el = O ? lookup$1(H, O, !0) : null, initializeAria(H), G || H.error("Unrecognized renderer type: " + V);
    const J = G.handler || CanvasHandler, Z = O ? G.renderer : G.headless;
    return H._renderer = Z ? initializeRenderer(H, H._renderer, O, Z) : null, H._handler = initializeHandler(H, H._handler, O, J), H._redraw = !0, O && q !== "none" && (_ = _ ? H._elBind = lookup$1(H, _, !0) : O.appendChild(element("form", {
      class: "vega-bindings"
    })), H._bind.forEach((te) => {
      te.param.element && q !== "container" && (te.element = lookup$1(H, te.param.element, !!te.param.input));
    }), H._bind.forEach((te) => {
      bind$1(H, te.element || _, te);
    })), H;
  }
  function lookup$1(O, _, H) {
    if (typeof _ == "string")
      if (typeof document < "u") {
        if (_ = document.querySelector(_), !_)
          return O.error("Signal bind element not found: " + _), null;
      } else
        return O.error("DOM document instance not found."), null;
    if (_ && H)
      try {
        _.textContent = "";
      } catch (V) {
        _ = null, O.error(V);
      }
    return _;
  }
  const number$2 = (O) => +O || 0, paddingObject$1 = (O) => ({
    top: O,
    bottom: O,
    left: O,
    right: O
  });
  function padding$1(O) {
    return isObject$2(O) ? {
      top: number$2(O.top),
      bottom: number$2(O.bottom),
      left: number$2(O.left),
      right: number$2(O.right)
    } : paddingObject$1(number$2(O));
  }
  async function renderHeadless(O, _, H, V) {
    const q = renderModule(_), G = q && q.headless;
    return G || error$1("Unrecognized renderer type: " + _), await O.runAsync(), initializeRenderer(O, null, null, G, H, V).renderAsync(O._scenegraph.root);
  }
  async function renderToImageURL(O, _) {
    O !== RenderType.Canvas && O !== RenderType.SVG && O !== RenderType.PNG && error$1("Unrecognized image type: " + O);
    const H = await renderHeadless(this, O, _);
    return O === RenderType.SVG ? toBlobURL(H.svg(), "image/svg+xml") : H.canvas().toDataURL("image/png");
  }
  function toBlobURL(O, _) {
    const H = new Blob([O], {
      type: _
    });
    return window.URL.createObjectURL(H);
  }
  async function renderToCanvas(O, _) {
    return (await renderHeadless(this, RenderType.Canvas, O, _)).canvas();
  }
  async function renderToSVG(O) {
    return (await renderHeadless(this, RenderType.SVG, O)).svg();
  }
  function runtime(O, _, H) {
    return context(O, transforms, functionContext, H).parse(_);
  }
  function scale$1(O) {
    var _ = this._runtime.scales;
    return has$1(_, O) || error$1("Unrecognized scale or projection: " + O), _[O].value;
  }
  var Width = "width", Height = "height", Padding = "padding", Skip$1 = {
    skip: !0
  };
  function viewWidth(O, _) {
    var H = O.autosize(), V = O.padding();
    return _ - (H && H.contains === Padding ? V.left + V.right : 0);
  }
  function viewHeight(O, _) {
    var H = O.autosize(), V = O.padding();
    return _ - (H && H.contains === Padding ? V.top + V.bottom : 0);
  }
  function initializeResize(O) {
    var _ = O._signals, H = _[Width], V = _[Height], q = _[Padding];
    function G() {
      O._autosize = O._resize = 1;
    }
    O._resizeWidth = O.add(null, (Z) => {
      O._width = Z.size, O._viewWidth = viewWidth(O, Z.size), G();
    }, {
      size: H
    }), O._resizeHeight = O.add(null, (Z) => {
      O._height = Z.size, O._viewHeight = viewHeight(O, Z.size), G();
    }, {
      size: V
    });
    const J = O.add(null, G, {
      pad: q
    });
    O._resizeWidth.rank = H.rank + 1, O._resizeHeight.rank = V.rank + 1, J.rank = q.rank + 1;
  }
  function resizeView(O, _, H, V, q, G) {
    this.runAfter((J) => {
      let Z = 0;
      J._autosize = 0, J.width() !== H && (Z = 1, J.signal(Width, H, Skip$1), J._resizeWidth.skip(!0)), J.height() !== V && (Z = 1, J.signal(Height, V, Skip$1), J._resizeHeight.skip(!0)), J._viewWidth !== O && (J._resize = 1, J._viewWidth = O), J._viewHeight !== _ && (J._resize = 1, J._viewHeight = _), (J._origin[0] !== q[0] || J._origin[1] !== q[1]) && (J._resize = 1, J._origin = q), Z && J.run("enter"), G && J.runAfter((te) => te.resize());
    }, !1, 1);
  }
  function getState(O) {
    return this._runtime.getState(O || {
      data: dataTest,
      signals: signalTest,
      recurse: !0
    });
  }
  function dataTest(O, _) {
    return _.modified && isArray$2(_.input.value) && !O.startsWith("_:vega:_");
  }
  function signalTest(O, _) {
    return !(O === "parent" || _ instanceof transforms.proxy);
  }
  function setState(O) {
    return this.runAsync(null, (_) => {
      _._trigger = !1, _._runtime.setState(O);
    }, (_) => {
      _._trigger = !0;
    }), this;
  }
  function timer(O, _) {
    function H(V) {
      O({
        timestamp: Date.now(),
        elapsed: V
      });
    }
    this._timers.push(interval$1(H, _));
  }
  function defaultTooltip(O, _, H, V) {
    const q = O.element();
    q && q.setAttribute("title", formatTooltip(V));
  }
  function formatTooltip(O) {
    return O == null ? "" : isArray$2(O) ? formatArray(O) : isObject$2(O) && !isDate$1(O) ? formatObject(O) : O + "";
  }
  function formatObject(O) {
    return Object.keys(O).map((_) => {
      const H = O[_];
      return _ + ": " + (isArray$2(H) ? formatArray(H) : formatValue$1(H));
    }).join(`
`);
  }
  function formatArray(O) {
    return "[" + O.map(formatValue$1).join(", ") + "]";
  }
  function formatValue$1(O) {
    return isArray$2(O) ? "[…]" : isObject$2(O) && !isDate$1(O) ? "{…}" : O;
  }
  function watchPixelRatio() {
    if (this.renderer() === "canvas" && this._renderer._canvas) {
      let O = null;
      const _ = () => {
        O != null && O();
        const H = matchMedia(`(resolution: ${window.devicePixelRatio}dppx)`);
        H.addEventListener("change", _), O = () => {
          H.removeEventListener("change", _);
        }, this._renderer._canvas.getContext("2d").pixelRatio = window.devicePixelRatio || 1, this._redraw = !0, this._resize = 1, this.resize().runAsync();
      };
      _();
    }
  }
  function View$1(O, _) {
    const H = this;
    if (_ = _ || {}, Dataflow.call(H), _.loader && H.loader(_.loader), _.logger && H.logger(_.logger), _.logLevel != null && H.logLevel(_.logLevel), _.locale || O.locale) {
      const G = extend$2({}, O.locale, _.locale);
      H.locale(locale(G.number, G.time));
    }
    H._el = null, H._elBind = null, H._renderType = _.renderer || RenderType.Canvas, H._scenegraph = new Scenegraph();
    const V = H._scenegraph.root;
    H._renderer = null, H._tooltip = _.tooltip || defaultTooltip, H._redraw = !0, H._handler = new CanvasHandler().scene(V), H._globalCursor = !1, H._preventDefault = !1, H._timers = [], H._eventListeners = [], H._resizeListeners = [], H._eventConfig = initializeEventConfig(O.eventConfig), H.globalCursor(H._eventConfig.globalCursor);
    const q = runtime(H, O, _.expr);
    H._runtime = q, H._signals = q.signals, H._bind = (O.bindings || []).map((G) => ({
      state: null,
      param: extend$2({}, G)
    })), q.root && q.root.set(V), V.source = q.data.root.input, H.pulse(q.data.root.input, H.changeset().insert(V.items)), H._width = H.width(), H._height = H.height(), H._viewWidth = viewWidth(H, H._width), H._viewHeight = viewHeight(H, H._height), H._origin = [0, 0], H._resize = 0, H._autosize = 1, initializeResize(H), background(H), cursor$1(H), H.description(O.description), _.hover && H.hover(), _.container && H.initialize(_.container, _.bind), _.watchPixelRatio && H._watchPixelRatio();
  }
  function lookupSignal(O, _) {
    return has$1(O._signals, _) ? O._signals[_] : error$1("Unrecognized signal name: " + $$2(_));
  }
  function findOperatorHandler(O, _) {
    const H = (O._targets || []).filter((V) => V._update && V._update.handler === _);
    return H.length ? H[0] : null;
  }
  function addOperatorListener(O, _, H, V) {
    let q = findOperatorHandler(H, V);
    return q || (q = trap(O, () => V(_, H.value)), q.handler = V, O.on(H, null, q)), O;
  }
  function removeOperatorListener(O, _, H) {
    const V = findOperatorHandler(_, H);
    return V && _._targets.remove(V), O;
  }
  inherits(View$1, Dataflow, {
    // -- DATAFLOW / RENDERING ----
    async evaluate(O, _, H) {
      if (await Dataflow.prototype.evaluate.call(this, O, _), this._redraw || this._resize)
        try {
          this._renderer && (this._resize && (this._resize = 0, resizeRenderer(this)), await this._renderer.renderAsync(this._scenegraph.root)), this._redraw = !1;
        } catch (V) {
          this.error(V);
        }
      return H && asyncCallback(this, H), this;
    },
    dirty(O) {
      this._redraw = !0, this._renderer && this._renderer.dirty(O);
    },
    // -- GET / SET ----
    description(O) {
      if (arguments.length) {
        const _ = O != null ? O + "" : null;
        return _ !== this._desc && ariaLabel(this._el, this._desc = _), this;
      }
      return this._desc;
    },
    container() {
      return this._el;
    },
    scenegraph() {
      return this._scenegraph;
    },
    origin() {
      return this._origin.slice();
    },
    signal(O, _, H) {
      const V = lookupSignal(this, O);
      return arguments.length === 1 ? V.value : this.update(V, _, H);
    },
    width(O) {
      return arguments.length ? this.signal("width", O) : this.signal("width");
    },
    height(O) {
      return arguments.length ? this.signal("height", O) : this.signal("height");
    },
    padding(O) {
      return arguments.length ? this.signal("padding", padding$1(O)) : padding$1(this.signal("padding"));
    },
    autosize(O) {
      return arguments.length ? this.signal("autosize", O) : this.signal("autosize");
    },
    background(O) {
      return arguments.length ? this.signal("background", O) : this.signal("background");
    },
    renderer(O) {
      return arguments.length ? (renderModule(O) || error$1("Unrecognized renderer type: " + O), O !== this._renderType && (this._renderType = O, this._resetRenderer()), this) : this._renderType;
    },
    tooltip(O) {
      return arguments.length ? (O !== this._tooltip && (this._tooltip = O, this._resetRenderer()), this) : this._tooltip;
    },
    loader(O) {
      return arguments.length ? (O !== this._loader && (Dataflow.prototype.loader.call(this, O), this._resetRenderer()), this) : this._loader;
    },
    resize() {
      return this._autosize = 1, this.touch(lookupSignal(this, "autosize"));
    },
    _resetRenderer() {
      this._renderer && (this._renderer = null, this.initialize(this._el, this._elBind));
    },
    // -- SIZING ----
    _resizeView: resizeView,
    // -- EVENT HANDLING ----
    addEventListener(O, _, H) {
      let V = _;
      return H && H.trap === !1 || (V = trap(this, _), V.raw = _), this._handler.on(O, V), this;
    },
    removeEventListener(O, _) {
      for (var H = this._handler.handlers(O), V = H.length, q, G; --V >= 0; )
        if (G = H[V].type, q = H[V].handler, O === G && (_ === q || _ === q.raw)) {
          this._handler.off(G, q);
          break;
        }
      return this;
    },
    addResizeListener(O) {
      const _ = this._resizeListeners;
      return _.includes(O) || _.push(O), this;
    },
    removeResizeListener(O) {
      var _ = this._resizeListeners, H = _.indexOf(O);
      return H >= 0 && _.splice(H, 1), this;
    },
    addSignalListener(O, _) {
      return addOperatorListener(this, O, lookupSignal(this, O), _);
    },
    removeSignalListener(O, _) {
      return removeOperatorListener(this, lookupSignal(this, O), _);
    },
    addDataListener(O, _) {
      return addOperatorListener(this, O, dataref(this, O).values, _);
    },
    removeDataListener(O, _) {
      return removeOperatorListener(this, dataref(this, O).values, _);
    },
    globalCursor(O) {
      if (arguments.length) {
        if (this._globalCursor !== !!O) {
          const _ = setCursor(this, null);
          this._globalCursor = !!O, _ && setCursor(this, _);
        }
        return this;
      } else
        return this._globalCursor;
    },
    preventDefault(O) {
      return arguments.length ? (this._preventDefault = O, this) : this._preventDefault;
    },
    timer,
    events: events$1,
    finalize,
    hover,
    // -- DATA ----
    data,
    change,
    insert: insert$1,
    remove: remove$1,
    // -- SCALES --
    scale: scale$1,
    // -- INITIALIZATION ----
    initialize,
    // -- HEADLESS RENDERING ----
    toImageURL: renderToImageURL,
    toCanvas: renderToCanvas,
    toSVG: renderToSVG,
    // -- SAVE / RESTORE STATE ----
    getState,
    setState,
    // RE-RENDER ON ZOOM
    _watchPixelRatio: watchPixelRatio
  });
  const VIEW = "view", LBRACK = "[", RBRACK = "]", LBRACE = "{", RBRACE = "}", COLON = ":", COMMA = ",", NAME = "@", GT = ">", ILLEGAL = /[[\]{}]/, DEFAULT_MARKS = {
    "*": 1,
    arc: 1,
    area: 1,
    group: 1,
    image: 1,
    line: 1,
    path: 1,
    rect: 1,
    rule: 1,
    shape: 1,
    symbol: 1,
    text: 1,
    trail: 1
  };
  let DEFAULT_SOURCE, MARKS;
  function eventSelector(O, _, H) {
    return DEFAULT_SOURCE = _ || VIEW, MARKS = H || DEFAULT_MARKS, parseMerge(O.trim()).map(parseSelector);
  }
  function isMarkType(O) {
    return MARKS[O];
  }
  function find$1(O, _, H, V, q) {
    const G = O.length;
    let J = 0, Z;
    for (; _ < G; ++_) {
      if (Z = O[_], !J && Z === H) return _;
      q && q.indexOf(Z) >= 0 ? --J : V && V.indexOf(Z) >= 0 && ++J;
    }
    return _;
  }
  function parseMerge(O) {
    const _ = [], H = O.length;
    let V = 0, q = 0;
    for (; q < H; )
      q = find$1(O, q, COMMA, LBRACK + LBRACE, RBRACK + RBRACE), _.push(O.substring(V, q).trim()), V = ++q;
    if (_.length === 0)
      throw "Empty event selector: " + O;
    return _;
  }
  function parseSelector(O) {
    return O[0] === "[" ? parseBetween(O) : parseStream$1(O);
  }
  function parseBetween(O) {
    const _ = O.length;
    let H = 1, V;
    if (H = find$1(O, H, RBRACK, LBRACK, RBRACK), H === _)
      throw "Empty between selector: " + O;
    if (V = parseMerge(O.substring(1, H)), V.length !== 2)
      throw "Between selector must have two elements: " + O;
    if (O = O.slice(H + 1).trim(), O[0] !== GT)
      throw "Expected '>' after between selector: " + O;
    V = V.map(parseSelector);
    const q = parseSelector(O.slice(1).trim());
    return q.between ? {
      between: V,
      stream: q
    } : (q.between = V, q);
  }
  function parseStream$1(O) {
    const _ = {
      source: DEFAULT_SOURCE
    }, H = [];
    let V = [0, 0], q = 0, G = 0, J = O.length, Z = 0, te, ne;
    if (O[J - 1] === RBRACE) {
      if (Z = O.lastIndexOf(LBRACE), Z >= 0) {
        try {
          V = parseThrottle(O.substring(Z + 1, J - 1));
        } catch {
          throw "Invalid throttle specification: " + O;
        }
        O = O.slice(0, Z).trim(), J = O.length;
      } else throw "Unmatched right brace: " + O;
      Z = 0;
    }
    if (!J) throw O;
    if (O[0] === NAME && (q = ++Z), te = find$1(O, Z, COLON), te < J && (H.push(O.substring(G, te).trim()), G = Z = ++te), Z = find$1(O, Z, LBRACK), Z === J)
      H.push(O.substring(G, J).trim());
    else if (H.push(O.substring(G, Z).trim()), ne = [], G = ++Z, G === J) throw "Unmatched left bracket: " + O;
    for (; Z < J; ) {
      if (Z = find$1(O, Z, RBRACK), Z === J) throw "Unmatched left bracket: " + O;
      if (ne.push(O.substring(G, Z).trim()), Z < J - 1 && O[++Z] !== LBRACK) throw "Expected left bracket: " + O;
      G = ++Z;
    }
    if (!(J = H.length) || ILLEGAL.test(H[J - 1]))
      throw "Invalid event selector: " + O;
    return J > 1 ? (_.type = H[1], q ? _.markname = H[0].slice(1) : isMarkType(H[0]) ? _.marktype = H[0] : _.source = H[0]) : _.type = H[0], _.type.slice(-1) === "!" && (_.consume = !0, _.type = _.type.slice(0, -1)), ne != null && (_.filter = ne), V[0] && (_.throttle = V[0]), V[1] && (_.debounce = V[1]), _;
  }
  function parseThrottle(O) {
    const _ = O.split(COMMA);
    if (!O.length || _.length > 2) throw O;
    return _.map((H) => {
      const V = +H;
      if (V !== V) throw O;
      return V;
    });
  }
  function parseAutosize(O) {
    return isObject$2(O) ? O : {
      type: O || "pad"
    };
  }
  const number$1 = (O) => +O || 0, paddingObject = (O) => ({
    top: O,
    bottom: O,
    left: O,
    right: O
  });
  function parsePadding(O) {
    return isObject$2(O) ? O.signal ? O : {
      top: number$1(O.top),
      bottom: number$1(O.bottom),
      left: number$1(O.left),
      right: number$1(O.right)
    } : paddingObject(number$1(O));
  }
  const encoder$1 = (O) => isObject$2(O) && !isArray$2(O) ? extend$2({}, O) : {
    value: O
  };
  function addEncode(O, _, H, V) {
    return H != null ? (isObject$2(H) && !isArray$2(H) || isArray$2(H) && H.length && isObject$2(H[0]) ? O.update[_] = H : O[V || "enter"][_] = {
      value: H
    }, 1) : 0;
  }
  function addEncoders(O, _, H) {
    for (const V in _)
      addEncode(O, V, _[V]);
    for (const V in H)
      addEncode(O, V, H[V], "update");
  }
  function extendEncode(O, _, H) {
    for (const V in _)
      H && has$1(H, V) || (O[V] = extend$2(O[V] || {}, _[V]));
    return O;
  }
  function has(O, _) {
    return _ && (_.enter && _.enter[O] || _.update && _.update[O]);
  }
  const MarkRole = "mark", FrameRole = "frame", ScopeRole = "scope", AxisRole = "axis", AxisDomainRole = "axis-domain", AxisGridRole = "axis-grid", AxisLabelRole = "axis-label", AxisTickRole = "axis-tick", AxisTitleRole = "axis-title", LegendRole = "legend", LegendBandRole = "legend-band", LegendEntryRole = "legend-entry", LegendGradientRole = "legend-gradient", LegendLabelRole = "legend-label", LegendSymbolRole = "legend-symbol", LegendTitleRole = "legend-title", TitleRole = "title", TitleTextRole = "title-text", TitleSubtitleRole = "title-subtitle";
  function applyDefaults(O, _, H, V, q) {
    const G = {}, J = {};
    let Z, te, ne, ee;
    te = "lineBreak", _ === "text" && q[te] != null && !has(te, O) && applyDefault(G, te, q[te]), (H == "legend" || String(H).startsWith("axis")) && (H = null), ee = H === FrameRole ? q.group : H === MarkRole ? extend$2({}, q.mark, q[_]) : null;
    for (te in ee)
      ne = has(te, O) || (te === "fill" || te === "stroke") && (has("fill", O) || has("stroke", O)), ne || applyDefault(G, te, ee[te]);
    array$5(V).forEach((ae) => {
      const oe = q.style && q.style[ae];
      for (const se in oe)
        has(se, O) || applyDefault(G, se, oe[se]);
    }), O = extend$2({}, O);
    for (te in G)
      ee = G[te], ee.signal ? (Z = Z || {})[te] = ee : J[te] = ee;
    return O.enter = extend$2(J, O.enter), Z && (O.update = extend$2(Z, O.update)), O;
  }
  function applyDefault(O, _, H) {
    O[_] = H && H.signal ? {
      signal: H.signal
    } : {
      value: H
    };
  }
  const scaleRef = (O) => isString$1(O) ? $$2(O) : O.signal ? `(${O.signal})` : field(O);
  function entry$1(O) {
    if (O.gradient != null)
      return gradient$1(O);
    let _ = O.signal ? `(${O.signal})` : O.color ? color$1(O.color) : O.field != null ? field(O.field) : O.value !== void 0 ? $$2(O.value) : void 0;
    return O.scale != null && (_ = scale(O, _)), _ === void 0 && (_ = null), O.exponent != null && (_ = `pow(${_},${property$1(O.exponent)})`), O.mult != null && (_ += `*${property$1(O.mult)}`), O.offset != null && (_ += `+${property$1(O.offset)}`), O.round && (_ = `round(${_})`), _;
  }
  const _color = (O, _, H, V) => `(${O}(${[_, H, V].map(entry$1).join(",")})+'')`;
  function color$1(O) {
    return O.c ? _color("hcl", O.h, O.c, O.l) : O.h || O.s ? _color("hsl", O.h, O.s, O.l) : O.l || O.a ? _color("lab", O.l, O.a, O.b) : O.r || O.g || O.b ? _color("rgb", O.r, O.g, O.b) : null;
  }
  function gradient$1(O) {
    const _ = [O.start, O.stop, O.count].map((H) => H == null ? null : $$2(H));
    for (; _.length && peek$1(_) == null; ) _.pop();
    return _.unshift(scaleRef(O.gradient)), `gradient(${_.join(",")})`;
  }
  function property$1(O) {
    return isObject$2(O) ? "(" + entry$1(O) + ")" : O;
  }
  function field(O) {
    return resolveField(isObject$2(O) ? O : {
      datum: O
    });
  }
  function resolveField(O) {
    let _, H, V;
    if (O.signal)
      _ = "datum", V = O.signal;
    else if (O.group || O.parent) {
      for (H = Math.max(1, O.level || 1), _ = "item"; H-- > 0; )
        _ += ".mark.group";
      O.parent ? (V = O.parent, _ += ".datum") : V = O.group;
    } else O.datum ? (_ = "datum", V = O.datum) : error$1("Invalid field reference: " + $$2(O));
    return O.signal || (V = isString$1(V) ? splitAccessPath(V).map($$2).join("][") : resolveField(V)), _ + "[" + V + "]";
  }
  function scale(O, _) {
    const H = scaleRef(O.scale);
    return O.range != null ? _ = `lerp(_range(${H}), ${+O.range})` : (_ !== void 0 && (_ = `_scale(${H}, ${_})`), O.band && (_ = (_ ? _ + "+" : "") + `_bandwidth(${H})` + (+O.band == 1 ? "" : "*" + property$1(O.band)), O.extra && (_ = `(datum.extra ? _scale(${H}, datum.extra.value) : ${_})`)), _ == null && (_ = "0")), _;
  }
  function rule$1(O) {
    let _ = "";
    return O.forEach((H) => {
      const V = entry$1(H);
      _ += H.test ? `(${H.test})?${V}:` : V;
    }), peek$1(_) === ":" && (_ += "null"), _;
  }
  function parseEncode(O, _, H, V, q, G) {
    const J = {};
    G = G || {}, G.encoders = {
      $encode: J
    }, O = applyDefaults(O, _, H, V, q.config);
    for (const Z in O)
      J[Z] = parseBlock(O[Z], _, G, q);
    return G;
  }
  function parseBlock(O, _, H, V) {
    const q = {}, G = {};
    for (const J in O)
      O[J] != null && (q[J] = parse$1(expr(O[J]), V, H, G));
    return {
      $expr: {
        marktype: _,
        channels: q
      },
      $fields: Object.keys(G),
      $output: Object.keys(O)
    };
  }
  function expr(O) {
    return isArray$2(O) ? rule$1(O) : entry$1(O);
  }
  function parse$1(O, _, H, V) {
    const q = parser$2(O, _);
    return q.$fields.forEach((G) => V[G] = 1), extend$2(H, q.$params), q.$expr;
  }
  const OUTER = "outer", OUTER_INVALID = ["value", "update", "init", "react", "bind"];
  function outerError(O, _) {
    error$1(O + ' for "outer" push: ' + $$2(_));
  }
  function parseSignal(O, _) {
    const H = O.name;
    if (O.push === OUTER)
      _.signals[H] || outerError("No prior signal definition", H), OUTER_INVALID.forEach((V) => {
        O[V] !== void 0 && outerError("Invalid property ", V);
      });
    else {
      const V = _.addSignal(H, O.value);
      O.react === !1 && (V.react = !1), O.bind && _.addBinding(H, O.bind);
    }
  }
  function Entry(O, _, H, V) {
    this.id = -1, this.type = O, this.value = _, this.params = H, V && (this.parent = V);
  }
  function entry(O, _, H, V) {
    return new Entry(O, _, H, V);
  }
  function operator$1(O, _) {
    return entry("operator", O, _);
  }
  function ref(O) {
    const _ = {
      $ref: O.id
    };
    return O.id < 0 && (O.refs = O.refs || []).push(_), _;
  }
  function fieldRef$1(O, _) {
    return _ ? {
      $field: O,
      $name: _
    } : {
      $field: O
    };
  }
  const keyFieldRef = fieldRef$1("key");
  function compareRef(O, _) {
    return {
      $compare: O,
      $order: _
    };
  }
  function keyRef(O, _) {
    const H = {
      $key: O
    };
    return _ && (H.$flat = !0), H;
  }
  const Ascending = "ascending", Descending = "descending";
  function sortKey(O) {
    return isObject$2(O) ? (O.order === Descending ? "-" : "+") + aggrField(O.op, O.field) : "";
  }
  function aggrField(O, _) {
    return (O && O.signal ? "$" + O.signal : O || "") + (O && _ ? "_" : "") + (_ && _.signal ? "$" + _.signal : _ || "");
  }
  const Scope$1 = "scope", View = "view";
  function isSignal(O) {
    return O && O.signal;
  }
  function isExpr$1(O) {
    return O && O.expr;
  }
  function hasSignal(O) {
    if (isSignal(O)) return !0;
    if (isObject$2(O)) {
      for (const _ in O)
        if (hasSignal(O[_])) return !0;
    }
    return !1;
  }
  function value(O, _) {
    return O ?? _;
  }
  function deref(O) {
    return O && O.signal || O;
  }
  const Timer = "timer";
  function parseStream(O, _) {
    return (O.merge ? mergeStream : O.stream ? nestedStream : O.type ? eventStream : error$1("Invalid stream specification: " + $$2(O)))(O, _);
  }
  function eventSource(O) {
    return O === Scope$1 ? View : O || View;
  }
  function mergeStream(O, _) {
    const H = O.merge.map((q) => parseStream(q, _)), V = streamParameters({
      merge: H
    }, O, _);
    return _.addStream(V).id;
  }
  function nestedStream(O, _) {
    const H = parseStream(O.stream, _), V = streamParameters({
      stream: H
    }, O, _);
    return _.addStream(V).id;
  }
  function eventStream(O, _) {
    let H;
    O.type === Timer ? (H = _.event(Timer, O.throttle), O = {
      between: O.between,
      filter: O.filter
    }) : H = _.event(eventSource(O.source), O.type);
    const V = streamParameters({
      stream: H
    }, O, _);
    return Object.keys(V).length === 1 ? H : _.addStream(V).id;
  }
  function streamParameters(O, _, H) {
    let V = _.between;
    return V && (V.length !== 2 && error$1('Stream "between" parameter must have 2 entries: ' + $$2(_)), O.between = [parseStream(V[0], H), parseStream(V[1], H)]), V = _.filter ? [].concat(_.filter) : [], (_.marktype || _.markname || _.markrole) && V.push(filterMark(_.marktype, _.markname, _.markrole)), _.source === Scope$1 && V.push("inScope(event.item)"), V.length && (O.filter = parser$2("(" + V.join(")&&(") + ")", H).$expr), (V = _.throttle) != null && (O.throttle = +V), (V = _.debounce) != null && (O.debounce = +V), _.consume && (O.consume = !0), O;
  }
  function filterMark(O, _, H) {
    const V = "event.item";
    return V + (O && O !== "*" ? "&&" + V + ".mark.marktype==='" + O + "'" : "") + (H ? "&&" + V + ".mark.role==='" + H + "'" : "") + (_ ? "&&" + V + ".mark.name==='" + _ + "'" : "");
  }
  const OP_VALUE_EXPR = {
    code: "_.$value",
    ast: {
      type: "Identifier",
      value: "value"
    }
  };
  function parseUpdate(O, _, H) {
    const V = O.encode, q = {
      target: H
    };
    let G = O.events, J = O.update, Z = [];
    G || error$1("Signal update missing events specification."), isString$1(G) && (G = eventSelector(G, _.isSubscope() ? Scope$1 : View)), G = array$5(G).filter((te) => te.signal || te.scale ? (Z.push(te), 0) : 1), Z.length > 1 && (Z = [mergeSources(Z)]), G.length && Z.push(G.length > 1 ? {
      merge: G
    } : G[0]), V != null && (J && error$1("Signal encode and update are mutually exclusive."), J = "encode(item()," + $$2(V) + ")"), q.update = isString$1(J) ? parser$2(J, _) : J.expr != null ? parser$2(J.expr, _) : J.value != null ? J.value : J.signal != null ? {
      $expr: OP_VALUE_EXPR,
      $params: {
        $value: _.signalRef(J.signal)
      }
    } : error$1("Invalid signal update specification."), O.force && (q.options = {
      force: !0
    }), Z.forEach((te) => _.addUpdate(extend$2(streamSource(te, _), q)));
  }
  function streamSource(O, _) {
    return {
      source: O.signal ? _.signalRef(O.signal) : O.scale ? _.scaleRef(O.scale) : parseStream(O, _)
    };
  }
  function mergeSources(O) {
    return {
      signal: "[" + O.map((_) => _.scale ? 'scale("' + _.scale + '")' : _.signal) + "]"
    };
  }
  function parseSignalUpdates(O, _) {
    const H = _.getSignal(O.name);
    let V = O.update;
    O.init && (V ? error$1("Signals can not include both init and update expressions.") : (V = O.init, H.initonly = !0)), V && (V = parser$2(V, _), H.update = V.$expr, H.params = V.$params), O.on && O.on.forEach((q) => parseUpdate(q, _, H.id));
  }
  const transform = (O) => (_, H, V) => entry(O, H, _ || void 0, V), Aggregate = transform("aggregate"), AxisTicks = transform("axisticks"), Bound = transform("bound"), Collect = transform("collect"), Compare = transform("compare"), DataJoin = transform("datajoin"), Encode = transform("encode"), Expression = transform("expression"), Facet$1 = transform("facet"), Field$2 = transform("field"), Key = transform("key"), LegendEntries = transform("legendentries"), Load = transform("load"), Mark$1 = transform("mark"), MultiExtent = transform("multiextent"), MultiValues = transform("multivalues"), Overlap = transform("overlap"), Params = transform("params"), PreFacet = transform("prefacet"), Projection = transform("projection"), Proxy$1 = transform("proxy"), Relay = transform("relay"), Render = transform("render"), Scale = transform("scale"), Sieve = transform("sieve"), SortItems = transform("sortitems"), ViewLayout = transform("viewlayout"), Values = transform("values");
  let FIELD_REF_ID = 0;
  const MULTIDOMAIN_SORT_OPS = {
    min: "min",
    max: "max",
    count: "sum"
  };
  function initScale(O, _) {
    const H = O.type || "linear";
    isValidScaleType(H) || error$1("Unrecognized scale type: " + $$2(H)), _.addScale(O.name, {
      type: H,
      domain: void 0
    });
  }
  function parseScale(O, _) {
    const H = _.getScale(O.name).params;
    let V;
    H.domain = parseScaleDomain$1(O.domain, O, _), O.range != null && (H.range = parseScaleRange$1(O, _, H)), O.interpolate != null && parseScaleInterpolate(O.interpolate, H), O.nice != null && (H.nice = parseScaleNice(O.nice, _)), O.bins != null && (H.bins = parseScaleBins(O.bins, _));
    for (V in O)
      has$1(H, V) || V === "name" || (H[V] = parseLiteral(O[V], _));
  }
  function parseLiteral(O, _) {
    return isObject$2(O) ? O.signal ? _.signalRef(O.signal) : error$1("Unsupported object: " + $$2(O)) : O;
  }
  function parseArray(O, _) {
    return O.signal ? _.signalRef(O.signal) : O.map((H) => parseLiteral(H, _));
  }
  function dataLookupError(O) {
    error$1("Can not find data set: " + $$2(O));
  }
  function parseScaleDomain$1(O, _, H) {
    if (!O) {
      (_.domainMin != null || _.domainMax != null) && error$1("No scale domain defined for domainMin/domainMax to override.");
      return;
    }
    return O.signal ? H.signalRef(O.signal) : (isArray$2(O) ? explicitDomain : O.fields ? multipleDomain : singularDomain)(O, _, H);
  }
  function explicitDomain(O, _, H) {
    return O.map((V) => parseLiteral(V, H));
  }
  function singularDomain(O, _, H) {
    const V = H.getData(O.data);
    return V || dataLookupError(O.data), isDiscrete$2(_.type) ? V.valuesRef(H, O.field, parseSort(O.sort, !1)) : isQuantile$1(_.type) ? V.domainRef(H, O.field) : V.extentRef(H, O.field);
  }
  function multipleDomain(O, _, H) {
    const V = O.data, q = O.fields.reduce((G, J) => (J = isString$1(J) ? {
      data: V,
      field: J
    } : isArray$2(J) || J.signal ? fieldRef(J, H) : J, G.push(J), G), []);
    return (isDiscrete$2(_.type) ? ordinalMultipleDomain : isQuantile$1(_.type) ? quantileMultipleDomain : numericMultipleDomain)(O, H, q);
  }
  function fieldRef(O, _) {
    const H = "_:vega:_" + FIELD_REF_ID++, V = Collect({});
    if (isArray$2(O))
      V.value = {
        $ingest: O
      };
    else if (O.signal) {
      const q = "setdata(" + $$2(H) + "," + O.signal + ")";
      V.params.input = _.signalRef(q);
    }
    return _.addDataPipeline(H, [V, Sieve({})]), {
      data: H,
      field: "data"
    };
  }
  function ordinalMultipleDomain(O, _, H) {
    const V = parseSort(O.sort, !0);
    let q, G;
    const J = H.map((ne) => {
      const ee = _.getData(ne.data);
      return ee || dataLookupError(ne.data), ee.countsRef(_, ne.field, V);
    }), Z = {
      groupby: keyFieldRef,
      pulse: J
    };
    V && (q = V.op || "count", G = V.field ? aggrField(q, V.field) : "count", Z.ops = [MULTIDOMAIN_SORT_OPS[q]], Z.fields = [_.fieldRef(G)], Z.as = [G]), q = _.add(Aggregate(Z));
    const te = _.add(Collect({
      pulse: ref(q)
    }));
    return G = _.add(Values({
      field: keyFieldRef,
      sort: _.sortRef(V),
      pulse: ref(te)
    })), ref(G);
  }
  function parseSort(O, _) {
    return O && (!O.field && !O.op ? isObject$2(O) ? O.field = "key" : O = {
      field: "key"
    } : !O.field && O.op !== "count" ? error$1("No field provided for sort aggregate op: " + O.op) : _ && O.field && O.op && !MULTIDOMAIN_SORT_OPS[O.op] && error$1("Multiple domain scales can not be sorted using " + O.op)), O;
  }
  function quantileMultipleDomain(O, _, H) {
    const V = H.map((q) => {
      const G = _.getData(q.data);
      return G || dataLookupError(q.data), G.domainRef(_, q.field);
    });
    return ref(_.add(MultiValues({
      values: V
    })));
  }
  function numericMultipleDomain(O, _, H) {
    const V = H.map((q) => {
      const G = _.getData(q.data);
      return G || dataLookupError(q.data), G.extentRef(_, q.field);
    });
    return ref(_.add(MultiExtent({
      extents: V
    })));
  }
  function parseScaleBins(O, _) {
    return O.signal || isArray$2(O) ? parseArray(O, _) : _.objectProperty(O);
  }
  function parseScaleNice(O, _) {
    return O.signal ? _.signalRef(O.signal) : isObject$2(O) ? {
      interval: parseLiteral(O.interval),
      step: parseLiteral(O.step)
    } : parseLiteral(O);
  }
  function parseScaleInterpolate(O, _) {
    _.interpolate = parseLiteral(O.type || O), O.gamma != null && (_.interpolateGamma = parseLiteral(O.gamma));
  }
  function parseScaleRange$1(O, _, H) {
    const V = _.config.range;
    let q = O.range;
    if (q.signal)
      return _.signalRef(q.signal);
    if (isString$1(q)) {
      if (V && has$1(V, q))
        return O = extend$2({}, O, {
          range: V[q]
        }), parseScaleRange$1(O, _, H);
      q === "width" ? q = [0, {
        signal: "width"
      }] : q === "height" ? q = isDiscrete$2(O.type) ? [0, {
        signal: "height"
      }] : [{
        signal: "height"
      }, 0] : error$1("Unrecognized scale range value: " + $$2(q));
    } else if (q.scheme) {
      H.scheme = isArray$2(q.scheme) ? parseArray(q.scheme, _) : parseLiteral(q.scheme, _), q.extent && (H.schemeExtent = parseArray(q.extent, _)), q.count && (H.schemeCount = parseLiteral(q.count, _));
      return;
    } else if (q.step) {
      H.rangeStep = parseLiteral(q.step, _);
      return;
    } else {
      if (isDiscrete$2(O.type) && !isArray$2(q))
        return parseScaleDomain$1(q, O, _);
      isArray$2(q) || error$1("Unsupported range type: " + $$2(q));
    }
    return q.map((G) => (isArray$2(G) ? parseArray : parseLiteral)(G, _));
  }
  function parseProjection$1(O, _) {
    const H = _.config.projection || {}, V = {};
    for (const q in O)
      q !== "name" && (V[q] = parseParameter$1(O[q], q, _));
    for (const q in H)
      V[q] == null && (V[q] = parseParameter$1(H[q], q, _));
    _.addProjection(O.name, V);
  }
  function parseParameter$1(O, _, H) {
    return isArray$2(O) ? O.map((V) => parseParameter$1(V, _, H)) : isObject$2(O) ? O.signal ? H.signalRef(O.signal) : _ === "fit" ? O : error$1("Unsupported parameter object: " + $$2(O)) : O;
  }
  const Top = "top", Left = "left", Right = "right", Bottom = "bottom", Center = "center", Vertical = "vertical", Start = "start", Middle = "middle", End = "end", Index = "index", Label = "label", Offset = "offset", Perc = "perc", Perc2 = "perc2", Value = "value", GuideLabelStyle = "guide-label", GuideTitleStyle = "guide-title", GroupTitleStyle = "group-title", GroupSubtitleStyle = "group-subtitle", Symbols = "symbol", Gradient = "gradient", Discrete = "discrete", Size = "size", Shape = "shape", Fill = "fill", Stroke = "stroke", StrokeWidth = "strokeWidth", StrokeDash = "strokeDash", Opacity = "opacity", LegendScales = [Size, Shape, Fill, Stroke, StrokeWidth, StrokeDash, Opacity], Skip = {
    name: 1,
    style: 1,
    interactive: 1
  }, zero$1 = {
    value: 0
  }, one = {
    value: 1
  }, GroupMark = "group", RectMark = "rect", RuleMark = "rule", SymbolMark = "symbol", TextMark = "text";
  function guideGroup(O) {
    return O.type = GroupMark, O.interactive = O.interactive || !1, O;
  }
  function lookup(O, _) {
    const H = (V, q) => value(O[V], value(_[V], q));
    return H.isVertical = (V) => Vertical === value(O.direction, _.direction || (V ? _.symbolDirection : _.gradientDirection)), H.gradientLength = () => value(O.gradientLength, _.gradientLength || _.gradientWidth), H.gradientThickness = () => value(O.gradientThickness, _.gradientThickness || _.gradientHeight), H.entryColumns = () => value(O.columns, value(_.columns, +H.isVertical(!0))), H;
  }
  function getEncoding(O, _) {
    const H = _ && (_.update && _.update[O] || _.enter && _.enter[O]);
    return H && H.signal ? H : H ? H.value : null;
  }
  function getStyle(O, _, H) {
    const V = _.config.style[H];
    return V && V[O];
  }
  function anchorExpr(O, _, H) {
    return `item.anchor === '${Start}' ? ${O} : item.anchor === '${End}' ? ${_} : ${H}`;
  }
  const alignExpr$1 = anchorExpr($$2(Left), $$2(Right), $$2(Center));
  function tickBand(O) {
    const _ = O("tickBand");
    let H = O("tickOffset"), V, q;
    return _ ? _.signal ? (V = {
      signal: `(${_.signal}) === 'extent' ? 1 : 0.5`
    }, q = {
      signal: `(${_.signal}) === 'extent'`
    }, isObject$2(H) || (H = {
      signal: `(${_.signal}) === 'extent' ? 0 : ${H}`
    })) : _ === "extent" ? (V = 1, q = !0, H = 0) : (V = 0.5, q = !1) : (V = O("bandPosition"), q = O("tickExtra")), {
      extra: q,
      band: V,
      offset: H
    };
  }
  function extendOffset(O, _) {
    return _ ? O ? isObject$2(O) ? Object.assign({}, O, {
      offset: extendOffset(O.offset, _)
    }) : {
      value: O,
      offset: _
    } : _ : O;
  }
  function guideMark(O, _) {
    return _ ? (O.name = _.name, O.style = _.style || O.style, O.interactive = !!_.interactive, O.encode = extendEncode(O.encode, _, Skip)) : O.interactive = !1, O;
  }
  function legendGradient(O, _, H, V) {
    const q = lookup(O, H), G = q.isVertical(), J = q.gradientThickness(), Z = q.gradientLength();
    let te, ne, ee, ae, oe;
    G ? (ne = [0, 1], ee = [0, 0], ae = J, oe = Z) : (ne = [0, 0], ee = [1, 0], ae = Z, oe = J);
    const se = {
      enter: te = {
        opacity: zero$1,
        x: zero$1,
        y: zero$1,
        width: encoder$1(ae),
        height: encoder$1(oe)
      },
      update: extend$2({}, te, {
        opacity: one,
        fill: {
          gradient: _,
          start: ne,
          stop: ee
        }
      }),
      exit: {
        opacity: zero$1
      }
    };
    return addEncoders(se, {
      stroke: q("gradientStrokeColor"),
      strokeWidth: q("gradientStrokeWidth")
    }, {
      // update
      opacity: q("gradientOpacity")
    }), guideMark({
      type: RectMark,
      role: LegendGradientRole,
      encode: se
    }, V);
  }
  function legendGradientDiscrete(O, _, H, V, q) {
    const G = lookup(O, H), J = G.isVertical(), Z = G.gradientThickness(), te = G.gradientLength();
    let ne, ee, ae, oe, se = "";
    J ? (ne = "y", ae = "y2", ee = "x", oe = "width", se = "1-") : (ne = "x", ae = "x2", ee = "y", oe = "height");
    const ce = {
      opacity: zero$1,
      fill: {
        scale: _,
        field: Value
      }
    };
    ce[ne] = {
      signal: se + "datum." + Perc,
      mult: te
    }, ce[ee] = zero$1, ce[ae] = {
      signal: se + "datum." + Perc2,
      mult: te
    }, ce[oe] = encoder$1(Z);
    const le = {
      enter: ce,
      update: extend$2({}, ce, {
        opacity: one
      }),
      exit: {
        opacity: zero$1
      }
    };
    return addEncoders(le, {
      stroke: G("gradientStrokeColor"),
      strokeWidth: G("gradientStrokeWidth")
    }, {
      // update
      opacity: G("gradientOpacity")
    }), guideMark({
      type: RectMark,
      role: LegendBandRole,
      key: Value,
      from: q,
      encode: le
    }, V);
  }
  const alignExpr = `datum.${Perc}<=0?"${Left}":datum.${Perc}>=1?"${Right}":"${Center}"`, baselineExpr = `datum.${Perc}<=0?"${Bottom}":datum.${Perc}>=1?"${Top}":"${Middle}"`;
  function legendGradientLabels(O, _, H, V) {
    const q = lookup(O, _), G = q.isVertical(), J = encoder$1(q.gradientThickness()), Z = q.gradientLength();
    let te = q("labelOverlap"), ne, ee, ae, oe, se = "";
    const ce = {
      enter: ne = {
        opacity: zero$1
      },
      update: ee = {
        opacity: one,
        text: {
          field: Label
        }
      },
      exit: {
        opacity: zero$1
      }
    };
    return addEncoders(ce, {
      fill: q("labelColor"),
      fillOpacity: q("labelOpacity"),
      font: q("labelFont"),
      fontSize: q("labelFontSize"),
      fontStyle: q("labelFontStyle"),
      fontWeight: q("labelFontWeight"),
      limit: value(O.labelLimit, _.gradientLabelLimit)
    }), G ? (ne.align = {
      value: "left"
    }, ne.baseline = ee.baseline = {
      signal: baselineExpr
    }, ae = "y", oe = "x", se = "1-") : (ne.align = ee.align = {
      signal: alignExpr
    }, ne.baseline = {
      value: "top"
    }, ae = "x", oe = "y"), ne[ae] = ee[ae] = {
      signal: se + "datum." + Perc,
      mult: Z
    }, ne[oe] = ee[oe] = J, J.offset = value(O.labelOffset, _.gradientLabelOffset) || 0, te = te ? {
      separation: q("labelSeparation"),
      method: te,
      order: "datum." + Index
    } : void 0, guideMark({
      type: TextMark,
      role: LegendLabelRole,
      style: GuideLabelStyle,
      key: Value,
      from: V,
      encode: ce,
      overlap: te
    }, H);
  }
  function legendSymbolGroups(O, _, H, V, q) {
    const G = lookup(O, _), J = H.entries, Z = !!(J && J.interactive), te = J ? J.name : void 0, ne = G("clipHeight"), ee = G("symbolOffset"), ae = {
      data: "value"
    }, oe = `(${q}) ? datum.${Offset} : datum.${Size}`, se = ne ? encoder$1(ne) : {
      field: Size
    }, ce = `datum.${Index}`, le = `max(1, ${q})`;
    let ue, de, fe, he, me;
    se.mult = 0.5, ue = {
      enter: de = {
        opacity: zero$1,
        x: {
          signal: oe,
          mult: 0.5,
          offset: ee
        },
        y: se
      },
      update: fe = {
        opacity: one,
        x: de.x,
        y: de.y
      },
      exit: {
        opacity: zero$1
      }
    };
    let pe = null, ye = null;
    O.fill || (pe = _.symbolBaseFillColor, ye = _.symbolBaseStrokeColor), addEncoders(ue, {
      fill: G("symbolFillColor", pe),
      shape: G("symbolType"),
      size: G("symbolSize"),
      stroke: G("symbolStrokeColor", ye),
      strokeDash: G("symbolDash"),
      strokeDashOffset: G("symbolDashOffset"),
      strokeWidth: G("symbolStrokeWidth")
    }, {
      // update
      opacity: G("symbolOpacity")
    }), LegendScales.forEach((Ce) => {
      O[Ce] && (fe[Ce] = de[Ce] = {
        scale: O[Ce],
        field: Value
      });
    });
    const ge = guideMark({
      type: SymbolMark,
      role: LegendSymbolRole,
      key: Value,
      from: ae,
      clip: ne ? !0 : void 0,
      encode: ue
    }, H.symbols), ve = encoder$1(ee);
    ve.offset = G("labelOffset"), ue = {
      enter: de = {
        opacity: zero$1,
        x: {
          signal: oe,
          offset: ve
        },
        y: se
      },
      update: fe = {
        opacity: one,
        text: {
          field: Label
        },
        x: de.x,
        y: de.y
      },
      exit: {
        opacity: zero$1
      }
    }, addEncoders(ue, {
      align: G("labelAlign"),
      baseline: G("labelBaseline"),
      fill: G("labelColor"),
      fillOpacity: G("labelOpacity"),
      font: G("labelFont"),
      fontSize: G("labelFontSize"),
      fontStyle: G("labelFontStyle"),
      fontWeight: G("labelFontWeight"),
      limit: G("labelLimit")
    });
    const be = guideMark({
      type: TextMark,
      role: LegendLabelRole,
      style: GuideLabelStyle,
      key: Value,
      from: ae,
      encode: ue
    }, H.labels);
    return ue = {
      enter: {
        noBound: {
          value: !ne
        },
        // ignore width/height in bounds calc
        width: zero$1,
        height: ne ? encoder$1(ne) : zero$1,
        opacity: zero$1
      },
      exit: {
        opacity: zero$1
      },
      update: fe = {
        opacity: one,
        row: {
          signal: null
        },
        column: {
          signal: null
        }
      }
    }, G.isVertical(!0) ? (he = `ceil(item.mark.items.length / ${le})`, fe.row.signal = `${ce}%${he}`, fe.column.signal = `floor(${ce} / ${he})`, me = {
      field: ["row", ce]
    }) : (fe.row.signal = `floor(${ce} / ${le})`, fe.column.signal = `${ce} % ${le}`, me = {
      field: ce
    }), fe.column.signal = `(${q})?${fe.column.signal}:${ce}`, V = {
      facet: {
        data: V,
        name: "value",
        groupby: Index
      }
    }, guideGroup({
      role: ScopeRole,
      from: V,
      encode: extendEncode(ue, J, Skip),
      marks: [ge, be],
      name: te,
      interactive: Z,
      sort: me
    });
  }
  function legendSymbolLayout(O, _) {
    const H = lookup(O, _);
    return {
      align: H("gridAlign"),
      columns: H.entryColumns(),
      center: {
        row: !0,
        column: !1
      },
      padding: {
        row: H("rowPadding"),
        column: H("columnPadding")
      }
    };
  }
  const isL = 'item.orient === "left"', isR = 'item.orient === "right"', isLR = `(${isL} || ${isR})`, isVG = `datum.vgrad && ${isLR}`, baseline$1 = anchorExpr('"top"', '"bottom"', '"middle"'), alignFlip = anchorExpr('"right"', '"left"', '"center"'), exprAlign = `datum.vgrad && ${isR} ? (${alignFlip}) : (${isLR} && !(datum.vgrad && ${isL})) ? "left" : ${alignExpr$1}`, exprAnchor = `item._anchor || (${isLR} ? "middle" : "start")`, exprAngle = `${isVG} ? (${isL} ? -90 : 90) : 0`, exprBaseline = `${isLR} ? (datum.vgrad ? (${isR} ? "bottom" : "top") : ${baseline$1}) : "top"`;
  function legendTitle(O, _, H, V) {
    const q = lookup(O, _), G = {
      enter: {
        opacity: zero$1
      },
      update: {
        opacity: one,
        x: {
          field: {
            group: "padding"
          }
        },
        y: {
          field: {
            group: "padding"
          }
        }
      },
      exit: {
        opacity: zero$1
      }
    };
    return addEncoders(G, {
      orient: q("titleOrient"),
      _anchor: q("titleAnchor"),
      anchor: {
        signal: exprAnchor
      },
      angle: {
        signal: exprAngle
      },
      align: {
        signal: exprAlign
      },
      baseline: {
        signal: exprBaseline
      },
      text: O.title,
      fill: q("titleColor"),
      fillOpacity: q("titleOpacity"),
      font: q("titleFont"),
      fontSize: q("titleFontSize"),
      fontStyle: q("titleFontStyle"),
      fontWeight: q("titleFontWeight"),
      limit: q("titleLimit"),
      lineHeight: q("titleLineHeight")
    }, {
      // require update
      align: q("titleAlign"),
      baseline: q("titleBaseline")
    }), guideMark({
      type: TextMark,
      role: LegendTitleRole,
      style: GuideTitleStyle,
      from: V,
      encode: G
    }, H);
  }
  function clip$1(O, _) {
    let H;
    return isObject$2(O) && (O.signal ? H = O.signal : O.path ? H = "pathShape(" + param(O.path) + ")" : O.sphere && (H = "geoShape(" + param(O.sphere) + ', {type: "Sphere"})')), H ? _.signalRef(H) : !!O;
  }
  function param(O) {
    return isObject$2(O) && O.signal ? O.signal : $$2(O);
  }
  function getRole(O) {
    const _ = O.role || "";
    return _.startsWith("axis") || _.startsWith("legend") || _.startsWith("title") ? _ : O.type === GroupMark ? ScopeRole : _ || MarkRole;
  }
  function definition(O) {
    return {
      marktype: O.type,
      name: O.name || void 0,
      role: O.role || getRole(O),
      zindex: +O.zindex || void 0,
      aria: O.aria,
      description: O.description
    };
  }
  function interactive(O, _) {
    return O && O.signal ? _.signalRef(O.signal) : O !== !1;
  }
  function parseTransform(O, _) {
    const H = definition$1(O.type);
    H || error$1("Unrecognized transform type: " + $$2(O.type));
    const V = entry(H.type.toLowerCase(), null, parseParameters(H, O, _));
    return O.signal && _.addSignal(O.signal, _.proxy(V)), V.metadata = H.metadata || {}, V;
  }
  function parseParameters(O, _, H) {
    const V = {}, q = O.params.length;
    for (let G = 0; G < q; ++G) {
      const J = O.params[G];
      V[J.name] = parseParameter(J, _, H);
    }
    return V;
  }
  function parseParameter(O, _, H) {
    const V = O.type, q = _[O.name];
    if (V === "index")
      return parseIndexParameter(O, _, H);
    if (q === void 0) {
      O.required && error$1("Missing required " + $$2(_.type) + " parameter: " + $$2(O.name));
      return;
    } else {
      if (V === "param")
        return parseSubParameters(O, _, H);
      if (V === "projection")
        return H.projectionRef(_[O.name]);
    }
    return O.array && !isSignal(q) ? q.map((G) => parameterValue(O, G, H)) : parameterValue(O, q, H);
  }
  function parameterValue(O, _, H) {
    const V = O.type;
    if (isSignal(_))
      return isExpr(V) ? error$1("Expression references can not be signals.") : isField(V) ? H.fieldRef(_) : isCompare(V) ? H.compareRef(_) : H.signalRef(_.signal);
    {
      const q = O.expr || isField(V);
      return q && outerExpr(_) ? H.exprRef(_.expr, _.as) : q && outerField(_) ? fieldRef$1(_.field, _.as) : isExpr(V) ? parser$2(_, H) : isData(V) ? ref(H.getData(_).values) : isField(V) ? fieldRef$1(_) : isCompare(V) ? H.compareRef(_) : _;
    }
  }
  function parseIndexParameter(O, _, H) {
    return isString$1(_.from) || error$1('Lookup "from" parameter must be a string literal.'), H.getData(_.from).lookupRef(H, _.key);
  }
  function parseSubParameters(O, _, H) {
    const V = _[O.name];
    return O.array ? (isArray$2(V) || error$1("Expected an array of sub-parameters. Instead: " + $$2(V)), V.map((q) => parseSubParameter(O, q, H))) : parseSubParameter(O, V, H);
  }
  function parseSubParameter(O, _, H) {
    const V = O.params.length;
    let q;
    for (let J = 0; J < V; ++J) {
      q = O.params[J];
      for (const Z in q.key)
        if (q.key[Z] !== _[Z]) {
          q = null;
          break;
        }
      if (q) break;
    }
    q || error$1("Unsupported parameter: " + $$2(_));
    const G = extend$2(parseParameters(q, _, H), q.key);
    return ref(H.add(Params(G)));
  }
  const outerExpr = (O) => O && O.expr, outerField = (O) => O && O.field, isData = (O) => O === "data", isExpr = (O) => O === "expr", isField = (O) => O === "field", isCompare = (O) => O === "compare";
  function parseData$1(O, _, H) {
    let V, q, G, J, Z;
    return O ? (V = O.facet) && (_ || error$1("Only group marks can be faceted."), V.field != null ? J = Z = getDataRef(V, H) : (O.data ? Z = ref(H.getData(O.data).aggregate) : (G = parseTransform(extend$2({
      type: "aggregate",
      groupby: array$5(V.groupby)
    }, V.aggregate), H), G.params.key = H.keyRef(V.groupby), G.params.pulse = getDataRef(V, H), J = Z = ref(H.add(G))), q = H.keyRef(V.groupby, !0))) : J = ref(H.add(Collect(null, [{}]))), J || (J = getDataRef(O, H)), {
      key: q,
      pulse: J,
      parent: Z
    };
  }
  function getDataRef(O, _) {
    return O.$ref ? O : O.data && O.data.$ref ? O.data : ref(_.getData(O.data).output);
  }
  function DataScope(O, _, H, V, q) {
    this.scope = O, this.input = _, this.output = H, this.values = V, this.aggregate = q, this.index = {};
  }
  DataScope.fromEntries = function(O, _) {
    const H = _.length, V = _[H - 1], q = _[H - 2];
    let G = _[0], J = null, Z = 1;
    for (G && G.type === "load" && (G = _[1]), O.add(_[0]); Z < H; ++Z)
      _[Z].params.pulse = ref(_[Z - 1]), O.add(_[Z]), _[Z].type === "aggregate" && (J = _[Z]);
    return new DataScope(O, G, q, V, J);
  };
  function fieldKey(O) {
    return isString$1(O) ? O : null;
  }
  function addSortField(O, _, H) {
    const V = aggrField(H.op, H.field);
    let q;
    if (_.ops) {
      for (let G = 0, J = _.as.length; G < J; ++G)
        if (_.as[G] === V) return;
    } else
      _.ops = ["count"], _.fields = [null], _.as = ["count"];
    H.op && (_.ops.push((q = H.op.signal) ? O.signalRef(q) : H.op), _.fields.push(O.fieldRef(H.field)), _.as.push(V));
  }
  function cache$1(O, _, H, V, q, G, J) {
    const Z = _[H] || (_[H] = {}), te = sortKey(G);
    let ne = fieldKey(q), ee, ae;
    if (ne != null && (O = _.scope, ne = ne + (te ? "|" + te : ""), ee = Z[ne]), !ee) {
      const oe = G ? {
        field: keyFieldRef,
        pulse: _.countsRef(O, q, G)
      } : {
        field: O.fieldRef(q),
        pulse: ref(_.output)
      };
      te && (oe.sort = O.sortRef(G)), ae = O.add(entry(V, void 0, oe)), J && (_.index[q] = ae), ee = ref(ae), ne != null && (Z[ne] = ee);
    }
    return ee;
  }
  DataScope.prototype = {
    countsRef(O, _, H) {
      const V = this, q = V.counts || (V.counts = {}), G = fieldKey(_);
      let J, Z, te;
      return G != null && (O = V.scope, J = q[G]), J ? H && H.field && addSortField(O, J.agg.params, H) : (te = {
        groupby: O.fieldRef(_, "key"),
        pulse: ref(V.output)
      }, H && H.field && addSortField(O, te, H), Z = O.add(Aggregate(te)), J = O.add(Collect({
        pulse: ref(Z)
      })), J = {
        agg: Z,
        ref: ref(J)
      }, G != null && (q[G] = J)), J.ref;
    },
    tuplesRef() {
      return ref(this.values);
    },
    extentRef(O, _) {
      return cache$1(O, this, "extent", "extent", _, !1);
    },
    domainRef(O, _) {
      return cache$1(O, this, "domain", "values", _, !1);
    },
    valuesRef(O, _, H) {
      return cache$1(O, this, "vals", "values", _, H || !0);
    },
    lookupRef(O, _) {
      return cache$1(O, this, "lookup", "tupleindex", _, !1);
    },
    indataRef(O, _) {
      return cache$1(O, this, "indata", "tupleindex", _, !0, !0);
    }
  };
  function parseFacet(O, _, H) {
    const V = O.from.facet, q = V.name, G = getDataRef(V, _);
    let J;
    V.name || error$1("Facet must have a name: " + $$2(V)), V.data || error$1("Facet must reference a data set: " + $$2(V)), V.field ? J = _.add(PreFacet({
      field: _.fieldRef(V.field),
      pulse: G
    })) : V.groupby ? J = _.add(Facet$1({
      key: _.keyRef(V.groupby),
      group: ref(_.proxy(H.parent)),
      pulse: G
    })) : error$1("Facet must specify groupby or field: " + $$2(V));
    const Z = _.fork(), te = Z.add(Collect()), ne = Z.add(Sieve({
      pulse: ref(te)
    }));
    Z.addData(q, new DataScope(Z, te, te, ne)), Z.addSignal("parent", null), J.params.subflow = {
      $subflow: Z.parse(O).toRuntime()
    };
  }
  function parseSubflow(O, _, H) {
    const V = _.add(PreFacet({
      pulse: H.pulse
    })), q = _.fork();
    q.add(Sieve()), q.addSignal("parent", null), V.params.subflow = {
      $subflow: q.parse(O).toRuntime()
    };
  }
  function parseTrigger(O, _, H) {
    const V = O.remove, q = O.insert, G = O.toggle, J = O.modify, Z = O.values, te = _.add(operator$1()), ne = "if(" + O.trigger + ',modify("' + H + '",' + [q, V, G, J, Z].map((ae) => ae ?? "null").join(",") + "),0)", ee = parser$2(ne, _);
    te.update = ee.$expr, te.params = ee.$params;
  }
  function parseMark(O, _) {
    const H = getRole(O), V = O.type === GroupMark, q = O.from && O.from.facet, G = O.overlap;
    let J = O.layout || H === ScopeRole || H === FrameRole, Z, te, ne, ee, ae, oe, se;
    const ce = H === MarkRole || J || q, le = parseData$1(O.from, V, _);
    te = _.add(DataJoin({
      key: le.key || (O.key ? fieldRef$1(O.key) : void 0),
      pulse: le.pulse,
      clean: !V
    }));
    const ue = ref(te);
    te = ne = _.add(Collect({
      pulse: ue
    })), te = _.add(Mark$1({
      markdef: definition(O),
      interactive: interactive(O.interactive, _),
      clip: clip$1(O.clip, _),
      context: {
        $context: !0
      },
      groups: _.lookup(),
      parent: _.signals.parent ? _.signalRef("parent") : null,
      index: _.markpath(),
      pulse: ref(te)
    }));
    const de = ref(te);
    te = ee = _.add(Encode(parseEncode(O.encode, O.type, H, O.style, _, {
      mod: !1,
      pulse: de
    }))), te.params.parent = _.encode(), O.transform && O.transform.forEach((ye) => {
      const ge = parseTransform(ye, _), ve = ge.metadata;
      (ve.generates || ve.changes) && error$1("Mark transforms should not generate new data."), ve.nomod || (ee.params.mod = !0), ge.params.pulse = ref(te), _.add(te = ge);
    }), O.sort && (te = _.add(SortItems({
      sort: _.compareRef(O.sort),
      pulse: ref(te)
    })));
    const fe = ref(te);
    (q || J) && (J = _.add(ViewLayout({
      layout: _.objectProperty(O.layout),
      legends: _.legends,
      mark: de,
      pulse: fe
    })), oe = ref(J));
    const he = _.add(Bound({
      mark: de,
      pulse: oe || fe
    }));
    se = ref(he), V && (ce && (Z = _.operators, Z.pop(), J && Z.pop()), _.pushState(fe, oe || se, ue), q ? parseFacet(O, _, le) : ce ? parseSubflow(O, _, le) : _.parse(O), _.popState(), ce && (J && Z.push(J), Z.push(he))), G && (se = parseOverlap(G, se, _));
    const me = _.add(Render({
      pulse: se
    })), pe = _.add(Sieve({
      pulse: ref(me)
    }, void 0, _.parent()));
    O.name != null && (ae = O.name, _.addData(ae, new DataScope(_, ne, me, pe)), O.on && O.on.forEach((ye) => {
      (ye.insert || ye.remove || ye.toggle) && error$1("Marks only support modify triggers."), parseTrigger(ye, _, ae);
    }));
  }
  function parseOverlap(O, _, H) {
    const V = O.method, q = O.bound, G = O.separation, J = {
      separation: isSignal(G) ? H.signalRef(G.signal) : G,
      method: isSignal(V) ? H.signalRef(V.signal) : V,
      pulse: _
    };
    if (O.order && (J.sort = H.compareRef({
      field: O.order
    })), q) {
      const Z = q.tolerance;
      J.boundTolerance = isSignal(Z) ? H.signalRef(Z.signal) : +Z, J.boundScale = H.scaleRef(q.scale), J.boundOrient = q.orient;
    }
    return ref(H.add(Overlap(J)));
  }
  function parseLegend$1(O, _) {
    const H = _.config.legend, V = O.encode || {}, q = lookup(O, H), G = V.legend || {}, J = G.name || void 0, Z = G.interactive, te = G.style, ne = {};
    let ee = 0, ae, oe, se;
    LegendScales.forEach((he) => O[he] ? (ne[he] = O[he], ee = ee || O[he]) : 0), ee || error$1("Missing valid scale for legend.");
    const ce = legendType(O, _.scaleType(ee)), le = {
      title: O.title != null,
      scales: ne,
      type: ce,
      vgrad: ce !== "symbol" && q.isVertical()
    }, ue = ref(_.add(Collect(null, [le]))), de = {
      enter: {
        x: {
          value: 0
        },
        y: {
          value: 0
        }
      }
    }, fe = ref(_.add(LegendEntries(oe = {
      type: ce,
      scale: _.scaleRef(ee),
      count: _.objectProperty(q("tickCount")),
      limit: _.property(q("symbolLimit")),
      values: _.objectProperty(O.values),
      minstep: _.property(O.tickMinStep),
      formatType: _.property(O.formatType),
      formatSpecifier: _.property(O.format)
    })));
    return ce === Gradient ? (se = [legendGradient(O, ee, H, V.gradient), legendGradientLabels(O, H, V.labels, fe)], oe.count = oe.count || _.signalRef(`max(2,2*floor((${deref(q.gradientLength())})/100))`)) : ce === Discrete ? se = [legendGradientDiscrete(O, ee, H, V.gradient, fe), legendGradientLabels(O, H, V.labels, fe)] : (ae = legendSymbolLayout(O, H), se = [legendSymbolGroups(O, H, V, fe, deref(ae.columns))], oe.size = sizeExpression(O, _, se[0].marks)), se = [guideGroup({
      role: LegendEntryRole,
      from: ue,
      encode: de,
      marks: se,
      layout: ae,
      interactive: Z
    })], le.title && se.push(legendTitle(O, H, V.title, ue)), parseMark(guideGroup({
      role: LegendRole,
      from: ue,
      encode: extendEncode(buildLegendEncode(q, O, H), G, Skip),
      marks: se,
      aria: q("aria"),
      description: q("description"),
      zindex: q("zindex"),
      name: J,
      interactive: Z,
      style: te
    }), _);
  }
  function legendType(O, _) {
    let H = O.type || Symbols;
    return !O.type && scaleCount(O) === 1 && (O.fill || O.stroke) && (H = isContinuous$1(_) ? Gradient : isDiscretizing$1(_) ? Discrete : Symbols), H !== Gradient ? H : isDiscretizing$1(_) ? Discrete : Gradient;
  }
  function scaleCount(O) {
    return LegendScales.reduce((_, H) => _ + (O[H] ? 1 : 0), 0);
  }
  function buildLegendEncode(O, _, H) {
    const V = {
      enter: {},
      update: {}
    };
    return addEncoders(V, {
      orient: O("orient"),
      offset: O("offset"),
      padding: O("padding"),
      titlePadding: O("titlePadding"),
      cornerRadius: O("cornerRadius"),
      fill: O("fillColor"),
      stroke: O("strokeColor"),
      strokeWidth: H.strokeWidth,
      strokeDash: H.strokeDash,
      x: O("legendX"),
      y: O("legendY"),
      // accessibility support
      format: _.format,
      formatType: _.formatType
    }), V;
  }
  function sizeExpression(O, _, H) {
    const V = deref(getChannel("size", O, H)), q = deref(getChannel("strokeWidth", O, H)), G = deref(getFontSize(H[1].encode, _, GuideLabelStyle));
    return parser$2(`max(ceil(sqrt(${V})+${q}),${G})`, _);
  }
  function getChannel(O, _, H) {
    return _[O] ? `scale("${_[O]}",datum)` : getEncoding(O, H[0].encode);
  }
  function getFontSize(O, _, H) {
    return getEncoding("fontSize", O) || getStyle("fontSize", _, H);
  }
  const angleExpr = `item.orient==="${Left}"?-90:item.orient==="${Right}"?90:0`;
  function parseTitle(O, _) {
    O = isString$1(O) ? {
      text: O
    } : O;
    const H = lookup(O, _.config.title), V = O.encode || {}, q = V.group || {}, G = q.name || void 0, J = q.interactive, Z = q.style, te = [], ne = {}, ee = ref(_.add(Collect(null, [ne])));
    return te.push(buildTitle(O, H, titleEncode(O), ee)), O.subtitle && te.push(buildSubTitle(O, H, V.subtitle, ee)), parseMark(guideGroup({
      role: TitleRole,
      from: ee,
      encode: groupEncode(H, q),
      marks: te,
      aria: H("aria"),
      description: H("description"),
      zindex: H("zindex"),
      name: G,
      interactive: J,
      style: Z
    }), _);
  }
  function titleEncode(O) {
    const _ = O.encode;
    return _ && _.title || extend$2({
      name: O.name,
      interactive: O.interactive,
      style: O.style
    }, _);
  }
  function groupEncode(O, _) {
    const H = {
      enter: {},
      update: {}
    };
    return addEncoders(H, {
      orient: O("orient"),
      anchor: O("anchor"),
      align: {
        signal: alignExpr$1
      },
      angle: {
        signal: angleExpr
      },
      limit: O("limit"),
      frame: O("frame"),
      offset: O("offset") || 0,
      padding: O("subtitlePadding")
    }), extendEncode(H, _, Skip);
  }
  function buildTitle(O, _, H, V) {
    const q = {
      value: 0
    }, G = O.text, J = {
      enter: {
        opacity: q
      },
      update: {
        opacity: {
          value: 1
        }
      },
      exit: {
        opacity: q
      }
    };
    return addEncoders(J, {
      text: G,
      align: {
        signal: "item.mark.group.align"
      },
      angle: {
        signal: "item.mark.group.angle"
      },
      limit: {
        signal: "item.mark.group.limit"
      },
      baseline: "top",
      dx: _("dx"),
      dy: _("dy"),
      fill: _("color"),
      font: _("font"),
      fontSize: _("fontSize"),
      fontStyle: _("fontStyle"),
      fontWeight: _("fontWeight"),
      lineHeight: _("lineHeight")
    }, {
      // update
      align: _("align"),
      angle: _("angle"),
      baseline: _("baseline")
    }), guideMark({
      type: TextMark,
      role: TitleTextRole,
      style: GroupTitleStyle,
      from: V,
      encode: J
    }, H);
  }
  function buildSubTitle(O, _, H, V) {
    const q = {
      value: 0
    }, G = O.subtitle, J = {
      enter: {
        opacity: q
      },
      update: {
        opacity: {
          value: 1
        }
      },
      exit: {
        opacity: q
      }
    };
    return addEncoders(J, {
      text: G,
      align: {
        signal: "item.mark.group.align"
      },
      angle: {
        signal: "item.mark.group.angle"
      },
      limit: {
        signal: "item.mark.group.limit"
      },
      baseline: "top",
      dx: _("dx"),
      dy: _("dy"),
      fill: _("subtitleColor"),
      font: _("subtitleFont"),
      fontSize: _("subtitleFontSize"),
      fontStyle: _("subtitleFontStyle"),
      fontWeight: _("subtitleFontWeight"),
      lineHeight: _("subtitleLineHeight")
    }, {
      // update
      align: _("align"),
      angle: _("angle"),
      baseline: _("baseline")
    }), guideMark({
      type: TextMark,
      role: TitleSubtitleRole,
      style: GroupSubtitleStyle,
      from: V,
      encode: J
    }, H);
  }
  function parseData$2(O, _) {
    const H = [];
    O.transform && O.transform.forEach((V) => {
      H.push(parseTransform(V, _));
    }), O.on && O.on.forEach((V) => {
      parseTrigger(V, _, O.name);
    }), _.addDataPipeline(O.name, analyze(O, _, H));
  }
  function analyze(O, _, H) {
    const V = [];
    let q = null, G = !1, J = !1, Z, te, ne, ee, ae;
    for (O.values ? isSignal(O.values) || hasSignal(O.format) ? (V.push(load(_, O)), V.push(q = collect())) : V.push(q = collect({
      $ingest: O.values,
      $format: O.format
    })) : O.url ? hasSignal(O.url) || hasSignal(O.format) ? (V.push(load(_, O)), V.push(q = collect())) : V.push(q = collect({
      $request: O.url,
      $format: O.format
    })) : O.source && (q = Z = array$5(O.source).map((oe) => ref(_.getData(oe).output)), V.push(null)), te = 0, ne = H.length; te < ne; ++te)
      ee = H[te], ae = ee.metadata, !q && !ae.source && V.push(q = collect()), V.push(ee), ae.generates && (J = !0), ae.modifies && !J && (G = !0), ae.source ? q = ee : ae.changes && (q = null);
    return Z && (ne = Z.length - 1, V[0] = Relay({
      derive: G,
      pulse: ne ? Z : Z[0]
    }), (G || ne) && V.splice(1, 0, collect())), q || V.push(collect()), V.push(Sieve({})), V;
  }
  function collect(O) {
    const _ = Collect({}, O);
    return _.metadata = {
      source: !0
    }, _;
  }
  function load(O, _) {
    return Load({
      url: _.url ? O.property(_.url) : void 0,
      async: _.async ? O.property(_.async) : void 0,
      values: _.values ? O.property(_.values) : void 0,
      format: O.objectProperty(_.format)
    });
  }
  const isX = (O) => O === Bottom || O === Top, getSign = (O, _, H) => isSignal(O) ? ifLeftTopExpr(O.signal, _, H) : O === Left || O === Top ? _ : H, ifX = (O, _, H) => isSignal(O) ? ifXEnc(O.signal, _, H) : isX(O) ? _ : H, ifY = (O, _, H) => isSignal(O) ? ifYEnc(O.signal, _, H) : isX(O) ? H : _, ifTop = (O, _, H) => isSignal(O) ? ifTopExpr(O.signal, _, H) : O === Top ? {
    value: _
  } : {
    value: H
  }, ifRight = (O, _, H) => isSignal(O) ? ifRightExpr(O.signal, _, H) : O === Right ? {
    value: _
  } : {
    value: H
  }, ifXEnc = (O, _, H) => ifEnc(`${O} === '${Top}' || ${O} === '${Bottom}'`, _, H), ifYEnc = (O, _, H) => ifEnc(`${O} !== '${Top}' && ${O} !== '${Bottom}'`, _, H), ifLeftTopExpr = (O, _, H) => ifExpr(`${O} === '${Left}' || ${O} === '${Top}'`, _, H), ifTopExpr = (O, _, H) => ifExpr(`${O} === '${Top}'`, _, H), ifRightExpr = (O, _, H) => ifExpr(`${O} === '${Right}'`, _, H), ifEnc = (O, _, H) => (_ = _ != null ? encoder$1(_) : _, H = H != null ? encoder$1(H) : H, isSimple(_) && isSimple(H) ? (_ = _ ? _.signal || $$2(_.value) : null, H = H ? H.signal || $$2(H.value) : null, {
    signal: `${O} ? (${_}) : (${H})`
  }) : [extend$2({
    test: O
  }, _)].concat(H || [])), isSimple = (O) => O == null || Object.keys(O).length === 1, ifExpr = (O, _, H) => ({
    signal: `${O} ? (${toExpr(_)}) : (${toExpr(H)})`
  }), ifOrient = (O, _, H, V, q) => ({
    signal: (V != null ? `${O} === '${Left}' ? (${toExpr(V)}) : ` : "") + (H != null ? `${O} === '${Bottom}' ? (${toExpr(H)}) : ` : "") + (q != null ? `${O} === '${Right}' ? (${toExpr(q)}) : ` : "") + (_ != null ? `${O} === '${Top}' ? (${toExpr(_)}) : ` : "") + "(null)"
  }), toExpr = (O) => isSignal(O) ? O.signal : O == null ? null : $$2(O), mult = (O, _) => _ === 0 ? 0 : isSignal(O) ? {
    signal: `(${O.signal}) * ${_}`
  } : {
    value: O * _
  }, patch$1 = (O, _) => {
    const H = O.signal;
    return H && H.endsWith("(null)") ? {
      signal: H.slice(0, -6) + _.signal
    } : O;
  };
  function fallback(O, _, H, V) {
    let q;
    if (_ && has$1(_, O))
      return _[O];
    if (has$1(H, O))
      return H[O];
    if (O.startsWith("title")) {
      switch (O) {
        case "titleColor":
          q = "fill";
          break;
        case "titleFont":
        case "titleFontSize":
        case "titleFontWeight":
          q = O[5].toLowerCase() + O.slice(6);
      }
      return V[GuideTitleStyle][q];
    } else if (O.startsWith("label")) {
      switch (O) {
        case "labelColor":
          q = "fill";
          break;
        case "labelFont":
        case "labelFontSize":
          q = O[5].toLowerCase() + O.slice(6);
      }
      return V[GuideLabelStyle][q];
    }
    return null;
  }
  function keys$3(O) {
    const _ = {};
    for (const H of O)
      if (H)
        for (const V in H) _[V] = 1;
    return Object.keys(_);
  }
  function axisConfig(O, _) {
    var H = _.config, V = H.style, q = H.axis, G = _.scaleType(O.scale) === "band" && H.axisBand, J = O.orient, Z, te, ne;
    if (isSignal(J)) {
      const ae = keys$3([H.axisX, H.axisY]), oe = keys$3([H.axisTop, H.axisBottom, H.axisLeft, H.axisRight]);
      Z = {};
      for (ne of ae)
        Z[ne] = ifX(J, fallback(ne, H.axisX, q, V), fallback(ne, H.axisY, q, V));
      te = {};
      for (ne of oe)
        te[ne] = ifOrient(J.signal, fallback(ne, H.axisTop, q, V), fallback(ne, H.axisBottom, q, V), fallback(ne, H.axisLeft, q, V), fallback(ne, H.axisRight, q, V));
    } else
      Z = J === Top || J === Bottom ? H.axisX : H.axisY, te = H["axis" + J[0].toUpperCase() + J.slice(1)];
    return Z || te || G ? extend$2({}, q, Z, te, G) : q;
  }
  function axisDomain(O, _, H, V) {
    const q = lookup(O, _), G = O.orient;
    let J, Z;
    const te = {
      enter: J = {
        opacity: zero$1
      },
      update: Z = {
        opacity: one
      },
      exit: {
        opacity: zero$1
      }
    };
    addEncoders(te, {
      stroke: q("domainColor"),
      strokeCap: q("domainCap"),
      strokeDash: q("domainDash"),
      strokeDashOffset: q("domainDashOffset"),
      strokeWidth: q("domainWidth"),
      strokeOpacity: q("domainOpacity")
    });
    const ne = position(O, 0), ee = position(O, 1);
    return J.x = Z.x = ifX(G, ne, zero$1), J.x2 = Z.x2 = ifX(G, ee), J.y = Z.y = ifY(G, ne, zero$1), J.y2 = Z.y2 = ifY(G, ee), guideMark({
      type: RuleMark,
      role: AxisDomainRole,
      from: V,
      encode: te
    }, H);
  }
  function position(O, _) {
    return {
      scale: O.scale,
      range: _
    };
  }
  function axisGrid(O, _, H, V, q) {
    const G = lookup(O, _), J = O.orient, Z = O.gridScale, te = getSign(J, 1, -1), ne = offsetValue(O.offset, te);
    let ee, ae, oe;
    const se = {
      enter: ee = {
        opacity: zero$1
      },
      update: oe = {
        opacity: one
      },
      exit: ae = {
        opacity: zero$1
      }
    };
    addEncoders(se, {
      stroke: G("gridColor"),
      strokeCap: G("gridCap"),
      strokeDash: G("gridDash"),
      strokeDashOffset: G("gridDashOffset"),
      strokeOpacity: G("gridOpacity"),
      strokeWidth: G("gridWidth")
    });
    const ce = {
      scale: O.scale,
      field: Value,
      band: q.band,
      extra: q.extra,
      offset: q.offset,
      round: G("tickRound")
    }, le = ifX(J, {
      signal: "height"
    }, {
      signal: "width"
    }), ue = Z ? {
      scale: Z,
      range: 0,
      mult: te,
      offset: ne
    } : {
      value: 0,
      offset: ne
    }, de = Z ? {
      scale: Z,
      range: 1,
      mult: te,
      offset: ne
    } : extend$2(le, {
      mult: te,
      offset: ne
    });
    return ee.x = oe.x = ifX(J, ce, ue), ee.y = oe.y = ifY(J, ce, ue), ee.x2 = oe.x2 = ifY(J, de), ee.y2 = oe.y2 = ifX(J, de), ae.x = ifX(J, ce), ae.y = ifY(J, ce), guideMark({
      type: RuleMark,
      role: AxisGridRole,
      key: Value,
      from: V,
      encode: se
    }, H);
  }
  function offsetValue(O, _) {
    if (_ !== 1) if (!isObject$2(O))
      O = isSignal(_) ? {
        signal: `(${_.signal}) * (${O || 0})`
      } : _ * (O || 0);
    else {
      let H = O = extend$2({}, O);
      for (; H.mult != null; )
        if (isObject$2(H.mult))
          H = H.mult = extend$2({}, H.mult);
        else
          return H.mult = isSignal(_) ? {
            signal: `(${H.mult}) * (${_.signal})`
          } : H.mult * _, O;
      H.mult = _;
    }
    return O;
  }
  function axisTicks(O, _, H, V, q, G) {
    const J = lookup(O, _), Z = O.orient, te = getSign(Z, -1, 1);
    let ne, ee, ae;
    const oe = {
      enter: ne = {
        opacity: zero$1
      },
      update: ae = {
        opacity: one
      },
      exit: ee = {
        opacity: zero$1
      }
    };
    addEncoders(oe, {
      stroke: J("tickColor"),
      strokeCap: J("tickCap"),
      strokeDash: J("tickDash"),
      strokeDashOffset: J("tickDashOffset"),
      strokeOpacity: J("tickOpacity"),
      strokeWidth: J("tickWidth")
    });
    const se = encoder$1(q);
    se.mult = te;
    const ce = {
      scale: O.scale,
      field: Value,
      band: G.band,
      extra: G.extra,
      offset: G.offset,
      round: J("tickRound")
    };
    return ae.y = ne.y = ifX(Z, zero$1, ce), ae.y2 = ne.y2 = ifX(Z, se), ee.x = ifX(Z, ce), ae.x = ne.x = ifY(Z, zero$1, ce), ae.x2 = ne.x2 = ifY(Z, se), ee.y = ifY(Z, ce), guideMark({
      type: RuleMark,
      role: AxisTickRole,
      key: Value,
      from: V,
      encode: oe
    }, H);
  }
  function flushExpr(O, _, H, V, q) {
    return {
      signal: 'flush(range("' + O + '"), scale("' + O + '", datum.value), ' + _ + "," + H + "," + V + "," + q + ")"
    };
  }
  function axisLabels(O, _, H, V, q, G) {
    const J = lookup(O, _), Z = O.orient, te = O.scale, ne = getSign(Z, -1, 1), ee = deref(J("labelFlush")), ae = deref(J("labelFlushOffset")), oe = J("labelAlign"), se = J("labelBaseline");
    let ce = ee === 0 || !!ee, le;
    const ue = encoder$1(q);
    ue.mult = ne, ue.offset = encoder$1(J("labelPadding") || 0), ue.offset.mult = ne;
    const de = {
      scale: te,
      field: Value,
      band: 0.5,
      offset: extendOffset(G.offset, J("labelOffset"))
    }, fe = ifX(Z, ce ? flushExpr(te, ee, '"left"', '"right"', '"center"') : {
      value: "center"
    }, ifRight(Z, "left", "right")), he = ifX(Z, ifTop(Z, "bottom", "top"), ce ? flushExpr(te, ee, '"top"', '"bottom"', '"middle"') : {
      value: "middle"
    }), me = flushExpr(te, ee, `-(${ae})`, ae, 0);
    ce = ce && ae;
    const pe = {
      opacity: zero$1,
      x: ifX(Z, de, ue),
      y: ifY(Z, de, ue)
    }, ye = {
      enter: pe,
      update: le = {
        opacity: one,
        text: {
          field: Label
        },
        x: pe.x,
        y: pe.y,
        align: fe,
        baseline: he
      },
      exit: {
        opacity: zero$1,
        x: pe.x,
        y: pe.y
      }
    };
    addEncoders(ye, {
      dx: !oe && ce ? ifX(Z, me) : null,
      dy: !se && ce ? ifY(Z, me) : null
    }), addEncoders(ye, {
      angle: J("labelAngle"),
      fill: J("labelColor"),
      fillOpacity: J("labelOpacity"),
      font: J("labelFont"),
      fontSize: J("labelFontSize"),
      fontWeight: J("labelFontWeight"),
      fontStyle: J("labelFontStyle"),
      limit: J("labelLimit"),
      lineHeight: J("labelLineHeight")
    }, {
      align: oe,
      baseline: se
    });
    const ge = J("labelBound");
    let ve = J("labelOverlap");
    return ve = ve || ge ? {
      separation: J("labelSeparation"),
      method: ve,
      order: "datum.index",
      bound: ge ? {
        scale: te,
        orient: Z,
        tolerance: ge
      } : null
    } : void 0, le.align !== fe && (le.align = patch$1(le.align, fe)), le.baseline !== he && (le.baseline = patch$1(le.baseline, he)), guideMark({
      type: TextMark,
      role: AxisLabelRole,
      style: GuideLabelStyle,
      key: Value,
      from: V,
      encode: ye,
      overlap: ve
    }, H);
  }
  function axisTitle(O, _, H, V) {
    const q = lookup(O, _), G = O.orient, J = getSign(G, -1, 1);
    let Z, te;
    const ne = {
      enter: Z = {
        opacity: zero$1,
        anchor: encoder$1(q("titleAnchor", null)),
        align: {
          signal: alignExpr$1
        }
      },
      update: te = extend$2({}, Z, {
        opacity: one,
        text: encoder$1(O.title)
      }),
      exit: {
        opacity: zero$1
      }
    }, ee = {
      signal: `lerp(range("${O.scale}"), ${anchorExpr(0, 1, 0.5)})`
    };
    return te.x = ifX(G, ee), te.y = ifY(G, ee), Z.angle = ifX(G, zero$1, mult(J, 90)), Z.baseline = ifX(G, ifTop(G, Bottom, Top), {
      value: Bottom
    }), te.angle = Z.angle, te.baseline = Z.baseline, addEncoders(ne, {
      fill: q("titleColor"),
      fillOpacity: q("titleOpacity"),
      font: q("titleFont"),
      fontSize: q("titleFontSize"),
      fontStyle: q("titleFontStyle"),
      fontWeight: q("titleFontWeight"),
      limit: q("titleLimit"),
      lineHeight: q("titleLineHeight")
    }, {
      // require update
      align: q("titleAlign"),
      angle: q("titleAngle"),
      baseline: q("titleBaseline")
    }), autoLayout(q, G, ne, H), ne.update.align = patch$1(ne.update.align, Z.align), ne.update.angle = patch$1(ne.update.angle, Z.angle), ne.update.baseline = patch$1(ne.update.baseline, Z.baseline), guideMark({
      type: TextMark,
      role: AxisTitleRole,
      style: GuideTitleStyle,
      from: V,
      encode: ne
    }, H);
  }
  function autoLayout(O, _, H, V) {
    const q = (Z, te) => Z != null ? (H.update[te] = patch$1(encoder$1(Z), H.update[te]), !1) : !has(te, V), G = q(O("titleX"), "x"), J = q(O("titleY"), "y");
    H.enter.auto = J === G ? encoder$1(J) : ifX(_, encoder$1(J), encoder$1(G));
  }
  function parseAxis$1(O, _) {
    const H = axisConfig(O, _), V = O.encode || {}, q = V.axis || {}, G = q.name || void 0, J = q.interactive, Z = q.style, te = lookup(O, H), ne = tickBand(te), ee = {
      scale: O.scale,
      ticks: !!te("ticks"),
      labels: !!te("labels"),
      grid: !!te("grid"),
      domain: !!te("domain"),
      title: O.title != null
    }, ae = ref(_.add(Collect({}, [ee]))), oe = ref(_.add(AxisTicks({
      scale: _.scaleRef(O.scale),
      extra: _.property(ne.extra),
      count: _.objectProperty(O.tickCount),
      values: _.objectProperty(O.values),
      minstep: _.property(O.tickMinStep),
      formatType: _.property(O.formatType),
      formatSpecifier: _.property(O.format)
    }))), se = [];
    let ce;
    return ee.grid && se.push(axisGrid(O, H, V.grid, oe, ne)), ee.ticks && (ce = te("tickSize"), se.push(axisTicks(O, H, V.ticks, oe, ce, ne))), ee.labels && (ce = ee.ticks ? ce : 0, se.push(axisLabels(O, H, V.labels, oe, ce, ne))), ee.domain && se.push(axisDomain(O, H, V.domain, ae)), ee.title && se.push(axisTitle(O, H, V.title, ae)), parseMark(guideGroup({
      role: AxisRole,
      from: ae,
      encode: extendEncode(buildAxisEncode(te, O), q, Skip),
      marks: se,
      aria: te("aria"),
      description: te("description"),
      zindex: te("zindex"),
      name: G,
      interactive: J,
      style: Z
    }), _);
  }
  function buildAxisEncode(O, _) {
    const H = {
      enter: {},
      update: {}
    };
    return addEncoders(H, {
      orient: O("orient"),
      offset: O("offset") || 0,
      position: value(_.position, 0),
      titlePadding: O("titlePadding"),
      minExtent: O("minExtent"),
      maxExtent: O("maxExtent"),
      range: {
        signal: `abs(span(range("${_.scale}")))`
      },
      translate: O("translate"),
      // accessibility support
      format: _.format,
      formatType: _.formatType
    }), H;
  }
  function parseScope(O, _, H) {
    const V = array$5(O.signals), q = array$5(O.scales);
    return H || V.forEach((G) => parseSignal(G, _)), array$5(O.projections).forEach((G) => parseProjection$1(G, _)), q.forEach((G) => initScale(G, _)), array$5(O.data).forEach((G) => parseData$2(G, _)), q.forEach((G) => parseScale(G, _)), (H || V).forEach((G) => parseSignalUpdates(G, _)), array$5(O.axes).forEach((G) => parseAxis$1(G, _)), array$5(O.marks).forEach((G) => parseMark(G, _)), array$5(O.legends).forEach((G) => parseLegend$1(G, _)), O.title && parseTitle(O.title, _), _.parseLambdas(), _;
  }
  const rootEncode = (O) => extendEncode({
    enter: {
      x: {
        value: 0
      },
      y: {
        value: 0
      }
    },
    update: {
      width: {
        signal: "width"
      },
      height: {
        signal: "height"
      }
    }
  }, O);
  function parseView(O, _) {
    const H = _.config, V = ref(_.root = _.add(operator$1())), q = collectSignals(O, H);
    q.forEach((ne) => parseSignal(ne, _)), _.description = O.description || H.description, _.eventConfig = H.events, _.legends = _.objectProperty(H.legend && H.legend.layout), _.locale = H.locale;
    const G = _.add(Collect()), J = _.add(Encode(parseEncode(rootEncode(O.encode), GroupMark, FrameRole, O.style, _, {
      pulse: ref(G)
    }))), Z = _.add(ViewLayout({
      layout: _.objectProperty(O.layout),
      legends: _.legends,
      autosize: _.signalRef("autosize"),
      mark: V,
      pulse: ref(J)
    }));
    _.operators.pop(), _.pushState(ref(J), ref(Z), null), parseScope(O, _, q), _.operators.push(Z);
    let te = _.add(Bound({
      mark: V,
      pulse: ref(Z)
    }));
    return te = _.add(Render({
      pulse: ref(te)
    })), te = _.add(Sieve({
      pulse: ref(te)
    })), _.addData("root", new DataScope(_, G, G, te)), _;
  }
  function signalObject(O, _) {
    return _ && _.signal ? {
      name: O,
      update: _.signal
    } : {
      name: O,
      value: _
    };
  }
  function collectSignals(O, _) {
    const H = (J) => value(O[J], _[J]), V = [signalObject("background", H("background")), signalObject("autosize", parseAutosize(H("autosize"))), signalObject("padding", parsePadding(H("padding"))), signalObject("width", H("width") || 0), signalObject("height", H("height") || 0)], q = V.reduce((J, Z) => (J[Z.name] = Z, J), {}), G = {};
    return array$5(O.signals).forEach((J) => {
      has$1(q, J.name) ? J = extend$2(q[J.name], J) : V.push(J), G[J.name] = J;
    }), array$5(_.signals).forEach((J) => {
      !has$1(G, J.name) && !has$1(q, J.name) && V.push(J);
    }), V;
  }
  function Scope(O, _) {
    this.config = O || {}, this.options = _ || {}, this.bindings = [], this.field = {}, this.signals = {}, this.lambdas = {}, this.scales = {}, this.events = {}, this.data = {}, this.streams = [], this.updates = [], this.operators = [], this.eventConfig = null, this.locale = null, this._id = 0, this._subid = 0, this._nextsub = [0], this._parent = [], this._encode = [], this._lookup = [], this._markpath = [];
  }
  function Subscope(O) {
    this.config = O.config, this.options = O.options, this.legends = O.legends, this.field = Object.create(O.field), this.signals = Object.create(O.signals), this.lambdas = Object.create(O.lambdas), this.scales = Object.create(O.scales), this.events = Object.create(O.events), this.data = Object.create(O.data), this.streams = [], this.updates = [], this.operators = [], this._id = 0, this._subid = ++O._nextsub[0], this._nextsub = O._nextsub, this._parent = O._parent.slice(), this._encode = O._encode.slice(), this._lookup = O._lookup.slice(), this._markpath = O._markpath;
  }
  Scope.prototype = Subscope.prototype = {
    parse(O) {
      return parseScope(O, this);
    },
    fork() {
      return new Subscope(this);
    },
    isSubscope() {
      return this._subid > 0;
    },
    toRuntime() {
      return this.finish(), {
        description: this.description,
        operators: this.operators,
        streams: this.streams,
        updates: this.updates,
        bindings: this.bindings,
        eventConfig: this.eventConfig,
        locale: this.locale
      };
    },
    id() {
      return (this._subid ? this._subid + ":" : 0) + this._id++;
    },
    add(O) {
      return this.operators.push(O), O.id = this.id(), O.refs && (O.refs.forEach((_) => {
        _.$ref = O.id;
      }), O.refs = null), O;
    },
    proxy(O) {
      const _ = O instanceof Entry ? ref(O) : O;
      return this.add(Proxy$1({
        value: _
      }));
    },
    addStream(O) {
      return this.streams.push(O), O.id = this.id(), O;
    },
    addUpdate(O) {
      return this.updates.push(O), O;
    },
    // Apply metadata
    finish() {
      let O, _;
      this.root && (this.root.root = !0);
      for (O in this.signals)
        this.signals[O].signal = O;
      for (O in this.scales)
        this.scales[O].scale = O;
      function H(V, q, G) {
        let J, Z;
        V && (J = V.data || (V.data = {}), Z = J[q] || (J[q] = []), Z.push(G));
      }
      for (O in this.data) {
        _ = this.data[O], H(_.input, O, "input"), H(_.output, O, "output"), H(_.values, O, "values");
        for (const V in _.index)
          H(_.index[V], O, "index:" + V);
      }
      return this;
    },
    // ----
    pushState(O, _, H) {
      this._encode.push(ref(this.add(Sieve({
        pulse: O
      })))), this._parent.push(_), this._lookup.push(H ? ref(this.proxy(H)) : null), this._markpath.push(-1);
    },
    popState() {
      this._encode.pop(), this._parent.pop(), this._lookup.pop(), this._markpath.pop();
    },
    parent() {
      return peek$1(this._parent);
    },
    encode() {
      return peek$1(this._encode);
    },
    lookup() {
      return peek$1(this._lookup);
    },
    markpath() {
      const O = this._markpath;
      return ++O[O.length - 1];
    },
    // ----
    fieldRef(O, _) {
      if (isString$1(O)) return fieldRef$1(O, _);
      O.signal || error$1("Unsupported field reference: " + $$2(O));
      const H = O.signal;
      let V = this.field[H];
      if (!V) {
        const q = {
          name: this.signalRef(H)
        };
        _ && (q.as = _), this.field[H] = V = ref(this.add(Field$2(q)));
      }
      return V;
    },
    compareRef(O) {
      let _ = !1;
      const H = (G) => isSignal(G) ? (_ = !0, this.signalRef(G.signal)) : isExpr$1(G) ? (_ = !0, this.exprRef(G.expr)) : G, V = array$5(O.field).map(H), q = array$5(O.order).map(H);
      return _ ? ref(this.add(Compare({
        fields: V,
        orders: q
      }))) : compareRef(V, q);
    },
    keyRef(O, _) {
      let H = !1;
      const V = (G) => isSignal(G) ? (H = !0, ref(q[G.signal])) : G, q = this.signals;
      return O = array$5(O).map(V), H ? ref(this.add(Key({
        fields: O,
        flat: _
      }))) : keyRef(O, _);
    },
    sortRef(O) {
      if (!O) return O;
      const _ = aggrField(O.op, O.field), H = O.order || Ascending;
      return H.signal ? ref(this.add(Compare({
        fields: _,
        orders: this.signalRef(H.signal)
      }))) : compareRef(_, H);
    },
    // ----
    event(O, _) {
      const H = O + ":" + _;
      if (!this.events[H]) {
        const V = this.id();
        this.streams.push({
          id: V,
          source: O,
          type: _
        }), this.events[H] = V;
      }
      return this.events[H];
    },
    // ----
    hasOwnSignal(O) {
      return has$1(this.signals, O);
    },
    addSignal(O, _) {
      this.hasOwnSignal(O) && error$1("Duplicate signal name: " + $$2(O));
      const H = _ instanceof Entry ? _ : this.add(operator$1(_));
      return this.signals[O] = H;
    },
    getSignal(O) {
      return this.signals[O] || error$1("Unrecognized signal name: " + $$2(O)), this.signals[O];
    },
    signalRef(O) {
      return this.signals[O] ? ref(this.signals[O]) : (has$1(this.lambdas, O) || (this.lambdas[O] = this.add(operator$1(null))), ref(this.lambdas[O]));
    },
    parseLambdas() {
      const O = Object.keys(this.lambdas);
      for (let _ = 0, H = O.length; _ < H; ++_) {
        const V = O[_], q = parser$2(V, this), G = this.lambdas[V];
        G.params = q.$params, G.update = q.$expr;
      }
    },
    property(O) {
      return O && O.signal ? this.signalRef(O.signal) : O;
    },
    objectProperty(O) {
      return !O || !isObject$2(O) ? O : this.signalRef(O.signal || propertyLambda(O));
    },
    exprRef(O, _) {
      const H = {
        expr: parser$2(O, this)
      };
      return _ && (H.expr.$name = _), ref(this.add(Expression(H)));
    },
    addBinding(O, _) {
      this.bindings || error$1("Nested signals do not support binding: " + $$2(O)), this.bindings.push(extend$2({
        signal: O
      }, _));
    },
    // ----
    addScaleProj(O, _) {
      has$1(this.scales, O) && error$1("Duplicate scale or projection name: " + $$2(O)), this.scales[O] = this.add(_);
    },
    addScale(O, _) {
      this.addScaleProj(O, Scale(_));
    },
    addProjection(O, _) {
      this.addScaleProj(O, Projection(_));
    },
    getScale(O) {
      return this.scales[O] || error$1("Unrecognized scale name: " + $$2(O)), this.scales[O];
    },
    scaleRef(O) {
      return ref(this.getScale(O));
    },
    scaleType(O) {
      return this.getScale(O).params.type;
    },
    projectionRef(O) {
      return this.scaleRef(O);
    },
    projectionType(O) {
      return this.scaleType(O);
    },
    // ----
    addData(O, _) {
      return has$1(this.data, O) && error$1("Duplicate data set name: " + $$2(O)), this.data[O] = _;
    },
    getData(O) {
      return this.data[O] || error$1("Undefined data set name: " + $$2(O)), this.data[O];
    },
    addDataPipeline(O, _) {
      return has$1(this.data, O) && error$1("Duplicate data set name: " + $$2(O)), this.addData(O, DataScope.fromEntries(this, _));
    }
  };
  function propertyLambda(O) {
    return (isArray$2(O) ? arrayLambda : objectLambda)(O);
  }
  function arrayLambda(O) {
    const _ = O.length;
    let H = "[";
    for (let V = 0; V < _; ++V) {
      const q = O[V];
      H += (V > 0 ? "," : "") + (isObject$2(q) ? q.signal || propertyLambda(q) : $$2(q));
    }
    return H + "]";
  }
  function objectLambda(O) {
    let _ = "{", H = 0, V, q;
    for (V in O)
      q = O[V], _ += (++H > 1 ? "," : "") + $$2(V) + ":" + (isObject$2(q) ? q.signal || propertyLambda(q) : $$2(q));
    return _ + "}";
  }
  function defaults$3() {
    const O = "sans-serif", V = "#4c78a8", q = "#000", G = "#888", J = "#ddd";
    return {
      // default visualization description
      description: "Vega visualization",
      // default padding around visualization
      padding: 0,
      // default for automatic sizing; options: 'none', 'pad', 'fit'
      // or provide an object (e.g., {'type': 'pad', 'resize': true})
      autosize: "pad",
      // default view background color
      // covers the entire view component
      background: null,
      // default event handling configuration
      // preventDefault for view-sourced event types except 'wheel'
      events: {
        defaults: {
          allow: ["wheel"]
        }
      },
      // defaults for top-level group marks
      // accepts mark properties (fill, stroke, etc)
      // covers the data rectangle within group width/height
      group: null,
      // defaults for basic mark types
      // each subset accepts mark properties (fill, stroke, etc)
      mark: null,
      arc: {
        fill: V
      },
      area: {
        fill: V
      },
      image: null,
      line: {
        stroke: V,
        strokeWidth: 2
      },
      path: {
        stroke: V
      },
      rect: {
        fill: V
      },
      rule: {
        stroke: q
      },
      shape: {
        stroke: V
      },
      symbol: {
        fill: V,
        size: 64
      },
      text: {
        fill: q,
        font: O,
        fontSize: 11
      },
      trail: {
        fill: V,
        size: 2
      },
      // style definitions
      style: {
        // axis & legend labels
        "guide-label": {
          fill: q,
          font: O,
          fontSize: 10
        },
        // axis & legend titles
        "guide-title": {
          fill: q,
          font: O,
          fontSize: 11,
          fontWeight: "bold"
        },
        // headers, including chart title
        "group-title": {
          fill: q,
          font: O,
          fontSize: 13,
          fontWeight: "bold"
        },
        // chart subtitle
        "group-subtitle": {
          fill: q,
          font: O,
          fontSize: 12
        },
        // defaults for styled point marks in Vega-Lite
        point: {
          size: 30,
          strokeWidth: 2,
          shape: "circle"
        },
        circle: {
          size: 30,
          strokeWidth: 2
        },
        square: {
          size: 30,
          strokeWidth: 2,
          shape: "square"
        },
        // defaults for styled group marks in Vega-Lite
        cell: {
          fill: "transparent",
          stroke: J
        },
        view: {
          fill: "transparent"
        }
      },
      // defaults for title
      title: {
        orient: "top",
        anchor: "middle",
        offset: 4,
        subtitlePadding: 3
      },
      // defaults for axes
      axis: {
        minExtent: 0,
        maxExtent: 200,
        bandPosition: 0.5,
        domain: !0,
        domainWidth: 1,
        domainColor: G,
        grid: !1,
        gridWidth: 1,
        gridColor: J,
        labels: !0,
        labelAngle: 0,
        labelLimit: 180,
        labelOffset: 0,
        labelPadding: 2,
        ticks: !0,
        tickColor: G,
        tickOffset: 0,
        tickRound: !0,
        tickSize: 5,
        tickWidth: 1,
        titlePadding: 4
      },
      // correction for centering bias
      axisBand: {
        tickOffset: -0.5
      },
      // defaults for cartographic projection
      projection: {
        type: "mercator"
      },
      // defaults for legends
      legend: {
        orient: "right",
        padding: 0,
        gridAlign: "each",
        columnPadding: 10,
        rowPadding: 2,
        symbolDirection: "vertical",
        gradientDirection: "vertical",
        gradientLength: 200,
        gradientThickness: 16,
        gradientStrokeColor: J,
        gradientStrokeWidth: 0,
        gradientLabelOffset: 2,
        labelAlign: "left",
        labelBaseline: "middle",
        labelLimit: 160,
        labelOffset: 4,
        labelOverlap: !0,
        symbolLimit: 30,
        symbolType: "circle",
        symbolSize: 100,
        symbolOffset: 0,
        symbolStrokeWidth: 1.5,
        symbolBaseFillColor: "transparent",
        symbolBaseStrokeColor: G,
        titleLimit: 180,
        titleOrient: "top",
        titlePadding: 5,
        layout: {
          offset: 18,
          direction: "horizontal",
          left: {
            direction: "vertical"
          },
          right: {
            direction: "vertical"
          }
        }
      },
      // defaults for scale ranges
      range: {
        category: {
          scheme: "tableau10"
        },
        ordinal: {
          scheme: "blues"
        },
        heatmap: {
          scheme: "yellowgreenblue"
        },
        ramp: {
          scheme: "blues"
        },
        diverging: {
          scheme: "blueorange",
          extent: [1, 0]
        },
        symbol: ["circle", "square", "triangle-up", "cross", "diamond", "triangle-right", "triangle-down", "triangle-left"]
      }
    };
  }
  function parse(O, _, H) {
    return isObject$2(O) || error$1("Input Vega specification must be an object."), _ = mergeConfig(defaults$3(), _, O.config), parseView(O, new Scope(_, H)).toRuntime();
  }
  var version$4 = "5.30.0";
  extend$2(transforms, tx, vtx, encode$1, geo, force, label, tree, reg, voronoi, wordcloud, xf);
  const vegaImport = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    Bounds,
    CanvasHandler,
    CanvasRenderer,
    DATE,
    DAY,
    DAYOFYEAR,
    Dataflow,
    Debug,
    Error: Error$1,
    EventStream,
    Gradient: Gradient$1,
    GroupItem,
    HOURS,
    Handler: Handler$1,
    HybridHandler,
    HybridRenderer,
    Info,
    Item,
    MILLISECONDS,
    MINUTES,
    MONTH,
    Marks,
    MultiPulse,
    None: None$2,
    Operator: Operator$1,
    Parameters,
    Pulse,
    QUARTER,
    RenderType,
    Renderer,
    ResourceLoader,
    SECONDS,
    SVGHandler,
    SVGRenderer,
    SVGStringRenderer,
    Scenegraph,
    TIME_UNITS,
    Transform,
    View: View$1,
    WEEK,
    Warn,
    YEAR,
    accessor,
    accessorFields,
    accessorName,
    array: array$5,
    ascending: ascending$2,
    bandwidthNRD: estimateBandwidth,
    bin: bin$1,
    bootstrapCI,
    boundClip,
    boundContext,
    boundItem: boundItem$1,
    boundMark,
    boundStroke,
    changeset,
    clampRange: clampRange$1,
    codegenExpression: codegen,
    compare: compare$3,
    constant: constant$5,
    cumulativeLogNormal,
    cumulativeNormal,
    cumulativeUniform,
    dayofyear,
    debounce: debounce$2,
    defaultLocale,
    definition: definition$1,
    densityLogNormal,
    densityNormal,
    densityUniform,
    domChild,
    domClear,
    domCreate,
    domFind,
    dotbin,
    error: error$1,
    expressionFunction,
    extend: extend$2,
    extent,
    extentIndex,
    falsy,
    fastmap,
    field: field$1,
    flush: flush$1,
    font: font$1,
    fontFamily,
    fontSize,
    format: format$2,
    formatLocale: numberFormatDefaultLocale,
    formats: formats$1,
    hasOwnProperty: has$1,
    id,
    identity: identity$7,
    inferType,
    inferTypes,
    ingest: ingest$1,
    inherits,
    inrange,
    interpolate: interpolate$1,
    interpolateColors,
    interpolateRange: interpolateRange$1,
    intersect: intersect$2,
    intersectBoxLine,
    intersectPath,
    intersectPoint,
    intersectRule,
    isArray: isArray$2,
    isBoolean: isBoolean$3,
    isDate: isDate$1,
    isFunction: isFunction$2,
    isIterable: isIterable$1,
    isNumber: isNumber$2,
    isObject: isObject$2,
    isRegExp,
    isString: isString$1,
    isTuple,
    key,
    lerp,
    lineHeight,
    loader,
    locale,
    logger,
    lruCache,
    markup,
    merge: merge$3,
    mergeConfig,
    multiLineOffset,
    one: one$2,
    pad: pad$2,
    panLinear,
    panLog,
    panPow,
    panSymlog,
    parse,
    parseExpression: parser$3,
    parseSelector: eventSelector,
    path: path$3,
    pathCurves: curves,
    pathEqual,
    pathParse: parse$3,
    pathRectangle: vg_rect,
    pathRender,
    pathSymbols: symbols$1,
    pathTrail: vg_trail,
    peek: peek$1,
    point: point$2,
    projection,
    quantileLogNormal,
    quantileNormal,
    quantileUniform,
    quantiles,
    quantizeInterpolator,
    quarter,
    quartiles,
    get random() {
      return random;
    },
    randomInteger: integer,
    randomKDE: kde,
    randomLCG: lcg$2,
    randomLogNormal: lognormal,
    randomMixture: mixture$1,
    randomNormal: gaussian,
    randomUniform: uniform,
    read,
    regressionConstant: constant$4,
    regressionExp: exp$1,
    regressionLinear: linear$2,
    regressionLoess: loess,
    regressionLog: log$3,
    regressionPoly: poly,
    regressionPow: pow$3,
    regressionQuad: quad,
    renderModule,
    repeat,
    resetDefaultLocale,
    resetSVGClipId,
    resetSVGDefIds,
    responseType,
    runtimeContext: context,
    sampleCurve,
    sampleLogNormal,
    sampleNormal,
    sampleUniform,
    scale: scale$4,
    sceneEqual,
    sceneFromJSON,
    scenePickVisit: pickVisit,
    sceneToJSON,
    sceneVisit: visit,
    sceneZOrder: zorder,
    scheme: scheme$1,
    serializeXML,
    setHybridRendererOptions,
    setRandom,
    span,
    splitAccessPath,
    stringValue: $$2,
    textMetrics,
    timeBin: bin$2,
    timeFloor,
    timeFormatLocale: timeFormatDefaultLocale,
    timeInterval,
    timeOffset,
    timeSequence,
    timeUnitSpecifier,
    timeUnits,
    toBoolean,
    toDate,
    toNumber: toNumber$1,
    toSet: toSet$1,
    toString: toString$1,
    transform: transform$2,
    transforms,
    truncate: truncate$1,
    truthy,
    tupleid,
    typeParsers,
    utcFloor,
    utcInterval,
    utcOffset,
    utcSequence,
    utcdayofyear,
    utcquarter,
    utcweek,
    version: version$4,
    visitArray,
    week,
    writeConfig,
    zero: zero$4,
    zoomLinear,
    zoomLog,
    zoomPow,
    zoomSymlog
  }, Symbol.toStringTag, { value: "Module" }));
  function adjustSpatial(O, _, H) {
    let V;
    _.x2 && (_.x ? (H && O.x > O.x2 && (V = O.x, O.x = O.x2, O.x2 = V), O.width = O.x2 - O.x) : O.x = O.x2 - (O.width || 0)), _.xc && (O.x = O.xc - (O.width || 0) / 2), _.y2 && (_.y ? (H && O.y > O.y2 && (V = O.y, O.y = O.y2, O.y2 = V), O.height = O.y2 - O.y) : O.y = O.y2 - (O.height || 0)), _.yc && (O.y = O.yc - (O.height || 0) / 2);
  }
  var Constants = {
    NaN: NaN,
    E: Math.E,
    LN2: Math.LN2,
    LN10: Math.LN10,
    LOG2E: Math.LOG2E,
    LOG10E: Math.LOG10E,
    PI: Math.PI,
    SQRT1_2: Math.SQRT1_2,
    SQRT2: Math.SQRT2,
    MIN_VALUE: Number.MIN_VALUE,
    MAX_VALUE: Number.MAX_VALUE
  }, Ops = {
    "*": (O, _) => O * _,
    "+": (O, _) => O + _,
    "-": (O, _) => O - _,
    "/": (O, _) => O / _,
    "%": (O, _) => O % _,
    ">": (O, _) => O > _,
    "<": (O, _) => O < _,
    "<=": (O, _) => O <= _,
    ">=": (O, _) => O >= _,
    "==": (O, _) => O == _,
    "!=": (O, _) => O != _,
    "===": (O, _) => O === _,
    "!==": (O, _) => O !== _,
    "&": (O, _) => O & _,
    "|": (O, _) => O | _,
    "^": (O, _) => O ^ _,
    "<<": (O, _) => O << _,
    ">>": (O, _) => O >> _,
    ">>>": (O, _) => O >>> _
  }, Unary = {
    "+": (O) => +O,
    "-": (O) => -O,
    "~": (O) => ~O,
    "!": (O) => !O
  };
  const slice$2 = Array.prototype.slice, apply = (O, _, H) => {
    const V = H ? H(_[0]) : _[0];
    return V[O].apply(V, slice$2.call(_, 1));
  }, datetime = (O, _, H, V, q, G, J) => new Date(O, _ || 0, H ?? 1, V || 0, q || 0, G || 0, J || 0);
  var Functions = {
    // math functions
    isNaN: Number.isNaN,
    isFinite: Number.isFinite,
    abs: Math.abs,
    acos: Math.acos,
    asin: Math.asin,
    atan: Math.atan,
    atan2: Math.atan2,
    ceil: Math.ceil,
    cos: Math.cos,
    exp: Math.exp,
    floor: Math.floor,
    log: Math.log,
    max: Math.max,
    min: Math.min,
    pow: Math.pow,
    random: Math.random,
    round: Math.round,
    sin: Math.sin,
    sqrt: Math.sqrt,
    tan: Math.tan,
    clamp: (O, _, H) => Math.max(_, Math.min(H, O)),
    // date functions
    now: Date.now,
    utc: Date.UTC,
    datetime,
    date: (O) => new Date(O).getDate(),
    day: (O) => new Date(O).getDay(),
    year: (O) => new Date(O).getFullYear(),
    month: (O) => new Date(O).getMonth(),
    hours: (O) => new Date(O).getHours(),
    minutes: (O) => new Date(O).getMinutes(),
    seconds: (O) => new Date(O).getSeconds(),
    milliseconds: (O) => new Date(O).getMilliseconds(),
    time: (O) => new Date(O).getTime(),
    timezoneoffset: (O) => new Date(O).getTimezoneOffset(),
    utcdate: (O) => new Date(O).getUTCDate(),
    utcday: (O) => new Date(O).getUTCDay(),
    utcyear: (O) => new Date(O).getUTCFullYear(),
    utcmonth: (O) => new Date(O).getUTCMonth(),
    utchours: (O) => new Date(O).getUTCHours(),
    utcminutes: (O) => new Date(O).getUTCMinutes(),
    utcseconds: (O) => new Date(O).getUTCSeconds(),
    utcmilliseconds: (O) => new Date(O).getUTCMilliseconds(),
    // sequence functions
    length: (O) => O.length,
    join: function() {
      return apply("join", arguments);
    },
    indexof: function() {
      return apply("indexOf", arguments);
    },
    lastindexof: function() {
      return apply("lastIndexOf", arguments);
    },
    slice: function() {
      return apply("slice", arguments);
    },
    reverse: (O) => O.slice().reverse(),
    // string functions
    parseFloat,
    parseInt,
    upper: (O) => String(O).toUpperCase(),
    lower: (O) => String(O).toLowerCase(),
    substring: function() {
      return apply("substring", arguments, String);
    },
    split: function() {
      return apply("split", arguments, String);
    },
    replace: function() {
      return apply("replace", arguments, String);
    },
    trim: (O) => String(O).trim(),
    // regexp functions
    regexp: RegExp,
    test: (O, _) => RegExp(O).test(_)
  };
  const EventFunctions = ["view", "item", "group", "xy", "x", "y"], DisallowedMethods = /* @__PURE__ */ new Set([Function, eval, setTimeout, setInterval]);
  typeof setImmediate == "function" && DisallowedMethods.add(setImmediate);
  const Visitors = {
    Literal: (O, _) => _.value,
    Identifier: (O, _) => {
      const H = _.name;
      return O.memberDepth > 0 ? H : H === "datum" ? O.datum : H === "event" ? O.event : H === "item" ? O.item : Constants[H] || O.params["$" + H];
    },
    MemberExpression: (O, _) => {
      const H = !_.computed, V = O(_.object);
      H && (O.memberDepth += 1);
      const q = O(_.property);
      if (H && (O.memberDepth -= 1), DisallowedMethods.has(V[q])) {
        console.error(`Prevented interpretation of member "${q}" which could lead to insecure code execution`);
        return;
      }
      return V[q];
    },
    CallExpression: (O, _) => {
      const H = _.arguments;
      let V = _.callee.name;
      return V.startsWith("_") && (V = V.slice(1)), V === "if" ? O(H[0]) ? O(H[1]) : O(H[2]) : (O.fn[V] || Functions[V]).apply(O.fn, H.map(O));
    },
    ArrayExpression: (O, _) => _.elements.map(O),
    BinaryExpression: (O, _) => Ops[_.operator](O(_.left), O(_.right)),
    UnaryExpression: (O, _) => Unary[_.operator](O(_.argument)),
    ConditionalExpression: (O, _) => O(_.test) ? O(_.consequent) : O(_.alternate),
    LogicalExpression: (O, _) => _.operator === "&&" ? O(_.left) && O(_.right) : O(_.left) || O(_.right),
    ObjectExpression: (O, _) => _.properties.reduce((H, V) => {
      O.memberDepth += 1;
      const q = O(V.key);
      return O.memberDepth -= 1, DisallowedMethods.has(O(V.value)) ? console.error(`Prevented interpretation of property "${q}" which could lead to insecure code execution`) : H[q] = O(V.value), H;
    }, {})
  };
  function interpret(O, _, H, V, q, G) {
    const J = (Z) => Visitors[Z.type](J, Z);
    return J.memberDepth = 0, J.fn = Object.create(_), J.params = H, J.datum = V, J.event = q, J.item = G, EventFunctions.forEach((Z) => J.fn[Z] = function() {
      return q.vega[Z](...arguments);
    }), J(O);
  }
  var expression$1 = {
    /**
     * Parse an expression used to update an operator value.
     */
    operator(O, _) {
      const H = _.ast, V = O.functions;
      return (q) => interpret(H, V, q);
    },
    /**
     * Parse an expression provided as an operator parameter value.
     */
    parameter(O, _) {
      const H = _.ast, V = O.functions;
      return (q, G) => interpret(H, V, G, q);
    },
    /**
     * Parse an expression applied to an event stream.
     */
    event(O, _) {
      const H = _.ast, V = O.functions;
      return (q) => interpret(H, V, void 0, void 0, q);
    },
    /**
     * Parse an expression used to handle an event-driven operator update.
     */
    handler(O, _) {
      const H = _.ast, V = O.functions;
      return (q, G) => {
        const J = G.item && G.item.datum;
        return interpret(H, V, q, J, G);
      };
    },
    /**
     * Parse an expression that performs visual encoding.
     */
    encode(O, _) {
      const {
        marktype: H,
        channels: V
      } = _, q = O.functions, G = H === "group" || H === "image" || H === "rect";
      return (J, Z) => {
        const te = J.datum;
        let ne = 0, ee;
        for (const ae in V)
          ee = interpret(V[ae].ast, q, Z, te, void 0, J), J[ae] !== ee && (J[ae] = ee, ne = 1);
        return H !== "rule" && adjustSpatial(J, V, G), ne;
      };
    }
  };
  const name$3 = "vega-lite", author$1 = 'Dominik Moritz, Kanit "Ham" Wongsuphasawat, Arvind Satyanarayan, Jeffrey Heer', version$3 = "5.23.0", collaborators = ["Kanit Wongsuphasawat (http://kanitw.yellowpigz.com)", "Dominik Moritz (https://www.domoritz.de)", "Arvind Satyanarayan (https://arvindsatya.com)", "Jeffrey Heer (https://jheer.org)"], homepage = "https://vega.github.io/vega-lite/", description$2 = "Vega-Lite is a concise high-level language for interactive visualization.", keywords$2 = ["vega", "chart", "visualization"], main$2 = "build/vega-lite.js", unpkg$1 = "build/vega-lite.min.js", jsdelivr$1 = "build/vega-lite.min.js", module$2 = "build/src/index", types$2 = "build/src/index.d.ts", bin = { vl2pdf: "./bin/vl2pdf", vl2png: "./bin/vl2png", vl2svg: "./bin/vl2svg", vl2vg: "./bin/vl2vg" }, files$1 = ["bin", "build", "src", "vega-lite*", "tsconfig.json"], scripts$1 = { changelog: "conventional-changelog -p angular -r 2", prebuild: "yarn clean:build", build: "yarn build:only", "build:only": "tsc -p tsconfig.build.json && rollup -c", "prebuild:examples": "yarn build:only", "build:examples": "yarn data && TZ=America/Los_Angeles scripts/build-examples.sh", "prebuild:examples-full": "yarn build:only", "build:examples-full": "TZ=America/Los_Angeles scripts/build-examples.sh 1", "build:example": "TZ=America/Los_Angeles scripts/build-example.sh", "build:toc": "yarn build:jekyll && scripts/generate-toc", "build:site": "rollup -c site/rollup.config.mjs", "build:jekyll": "pushd site && bundle exec jekyll build -q && popd", "build:versions": "scripts/update-version.sh", clean: "yarn clean:build && del-cli 'site/data/*' 'examples/compiled/*.png' && find site/examples ! -name 'index.md' ! -name 'data' -type f -delete", "clean:build": "del-cli 'build/*' !build/vega-lite-schema.json", data: "rsync -r node_modules/vega-datasets/data/* site/data", "build-editor-preview": "scripts/build-editor-preview.sh", schema: "mkdir -p build && ts-json-schema-generator -f tsconfig.json -p src/index.ts -t TopLevelSpec --no-type-check --no-ref-encode > build/vega-lite-schema.json && yarn renameschema && cp build/vega-lite-schema.json site/_data/", renameschema: "scripts/rename-schema.sh", presite: "yarn data && yarn schema && yarn build:site && yarn build:versions && scripts/create-example-pages.sh", site: "yarn site:only", "site:only": "pushd site && bundle exec jekyll serve -I -l && popd", prettierbase: "prettier '**/*.{md,css,yml}'", format: "eslint . --fix && yarn prettierbase --write", lint: "eslint . && yarn prettierbase --check", test: "yarn jest test/ && yarn lint && yarn schema && yarn jest examples/ && yarn test:runtime", "test:cover": "yarn jest --collectCoverage test/", "test:inspect": "node --inspect-brk ./node_modules/.bin/jest --runInBand test", "test:runtime": "TZ=America/Los_Angeles npx jest test-runtime/ --config test-runtime/jest-config.json", "test:runtime:generate": "yarn build:only && del-cli test-runtime/resources && VL_GENERATE_TESTS=true yarn test:runtime", watch: "tsc -p tsconfig.build.json -w", "watch:site": "yarn build:site -w", "watch:test": "yarn jest --watch test/", "watch:test:runtime": "TZ=America/Los_Angeles npx jest --watch test-runtime/ --config test-runtime/jest-config.json", release: "release-it" }, repository$1 = { type: "git", url: "https://github.com/vega/vega-lite.git" }, license$1 = "BSD-3-Clause", bugs = { url: "https://github.com/vega/vega-lite/issues" }, devDependencies$1 = { "@babel/core": "^7.26.0", "@babel/preset-env": "^7.26.0", "@babel/preset-typescript": "^7.26.0", "@release-it/conventional-changelog": "^9.0.3", "@rollup/plugin-alias": "^5.1.1", "@rollup/plugin-babel": "^6.0.4", "@rollup/plugin-commonjs": "^28.0.1", "@rollup/plugin-json": "^6.1.0", "@rollup/plugin-node-resolve": "^15.3.0", "@rollup/plugin-terser": "^0.4.4", "@types/d3": "^7.4.3", "@types/jest": "^29.5.14", "@types/pako": "^2.0.3", "@typescript-eslint/eslint-plugin": "^7.17.0", "@typescript-eslint/parser": "^7.17.0", ajv: "^8.17.1", "ajv-formats": "^3.0.1", cheerio: "^1.0.0", "conventional-changelog-cli": "^5.0.0", d3: "^7.9.0", "del-cli": "^6.0.0", eslint: "^8.57.0", "eslint-config-prettier": "^9.1.0", "eslint-plugin-jest": "^27.9.0", "eslint-plugin-prettier": "^5.2.1", "fast-json-stable-stringify": "~2.1.0", "highlight.js": "^11.10.0", jest: "^29.7.0", "jest-dev-server": "^10.1.4", mkdirp: "^3.0.1", pako: "^2.1.0", prettier: "^3.3.3", puppeteer: "^15.0.0", "release-it": "17.10.0", rollup: "^4.27.3", "rollup-plugin-bundle-size": "^1.0.3", serve: "^14.2.4", terser: "^5.36.0", "ts-jest": "^29.2.5", "ts-json-schema-generator": "^2.3.0", typescript: "~5.7.2", "vega-cli": "^5.30.0", "vega-datasets": "^2.11.0", "vega-embed": "^6.28.0", "vega-tooltip": "^0.35.2", "yaml-front-matter": "^4.1.1" }, dependencies$1 = { "json-stringify-pretty-compact": "~4.0.0", tslib: "~2.8.1", "vega-event-selector": "~3.0.1", "vega-expression": "~5.1.1", "vega-util": "~1.17.2", yargs: "~17.7.2" }, peerDependencies$1 = { vega: "^5.24.0" }, engines = { node: ">=18" }, packageManager = "yarn@1.22.22", pkg$1 = {
    name: name$3,
    author: author$1,
    version: version$3,
    collaborators,
    homepage,
    description: description$2,
    keywords: keywords$2,
    main: main$2,
    unpkg: unpkg$1,
    jsdelivr: jsdelivr$1,
    module: module$2,
    types: types$2,
    bin,
    files: files$1,
    scripts: scripts$1,
    repository: repository$1,
    license: license$1,
    bugs,
    devDependencies: devDependencies$1,
    dependencies: dependencies$1,
    peerDependencies: peerDependencies$1,
    engines,
    packageManager
  };
  function isLogicalOr(O) {
    return hasProperty(O, "or");
  }
  function isLogicalAnd(O) {
    return hasProperty(O, "and");
  }
  function isLogicalNot(O) {
    return hasProperty(O, "not");
  }
  function forEachLeaf(O, _) {
    if (isLogicalNot(O))
      forEachLeaf(O.not, _);
    else if (isLogicalAnd(O))
      for (const H of O.and)
        forEachLeaf(H, _);
    else if (isLogicalOr(O))
      for (const H of O.or)
        forEachLeaf(H, _);
    else
      _(O);
  }
  function normalizeLogicalComposition(O, _) {
    return isLogicalNot(O) ? { not: normalizeLogicalComposition(O.not, _) } : isLogicalAnd(O) ? { and: O.and.map((H) => normalizeLogicalComposition(H, _)) } : isLogicalOr(O) ? { or: O.or.map((H) => normalizeLogicalComposition(H, _)) } : _(O);
  }
  const duplicate = structuredClone;
  function never(O) {
    throw new Error(O);
  }
  function pick$1(O, _) {
    const H = {};
    for (const V of _)
      has$1(O, V) && (H[V] = O[V]);
    return H;
  }
  function omit(O, _) {
    const H = { ...O };
    for (const V of _)
      delete H[V];
    return H;
  }
  Set.prototype.toJSON = function() {
    return `Set(${[...this].map((O) => stringify$1(O)).join(",")})`;
  };
  function hash(O) {
    if (isNumber$2(O))
      return O;
    const _ = isString$1(O) ? O : stringify$1(O);
    if (_.length < 250)
      return _;
    let H = 0;
    for (let V = 0; V < _.length; V++) {
      const q = _.charCodeAt(V);
      H = (H << 5) - H + q, H = H & H;
    }
    return H;
  }
  function isNullOrFalse(O) {
    return O === !1 || O === null;
  }
  function contains$2(O, _) {
    return O.includes(_);
  }
  function some(O, _) {
    let H = 0;
    for (const [V, q] of O.entries())
      if (_(q, V, H++))
        return !0;
    return !1;
  }
  function every(O, _) {
    let H = 0;
    for (const [V, q] of O.entries())
      if (!_(q, V, H++))
        return !1;
    return !0;
  }
  function mergeDeep$1(O, ..._) {
    for (const H of _)
      deepMerge_$1(O, H ?? {});
    return O;
  }
  function deepMerge_$1(O, _) {
    for (const H of keys$2(_))
      writeConfig(O, H, _[H], !0);
  }
  function unique(O, _) {
    const H = [], V = {};
    let q;
    for (const G of O)
      q = _(G), !(q in V) && (V[q] = 1, H.push(G));
    return H;
  }
  function isEqual(O, _) {
    const H = keys$2(O), V = keys$2(_);
    if (H.length !== V.length)
      return !1;
    for (const q of H)
      if (O[q] !== _[q])
        return !1;
    return !0;
  }
  function setEqual(O, _) {
    if (O.size !== _.size)
      return !1;
    for (const H of O)
      if (!_.has(H))
        return !1;
    return !0;
  }
  function hasIntersection(O, _) {
    for (const H of O)
      if (_.has(H))
        return !0;
    return !1;
  }
  function prefixGenerator(O) {
    const _ = /* @__PURE__ */ new Set();
    for (const H of O) {
      const q = splitAccessPath(H).map((J, Z) => Z === 0 ? J : `[${J}]`), G = q.map((J, Z) => q.slice(0, Z + 1).join(""));
      for (const J of G)
        _.add(J);
    }
    return _;
  }
  function fieldIntersection(O, _) {
    return O === void 0 || _ === void 0 ? !0 : hasIntersection(prefixGenerator(O), prefixGenerator(_));
  }
  function isEmpty(O) {
    return keys$2(O).length === 0;
  }
  const keys$2 = Object.keys, vals = Object.values, entries$1 = Object.entries;
  function isBoolean$1(O) {
    return O === !0 || O === !1;
  }
  function varName(O) {
    const _ = O.replace(/\W/g, "_");
    return (O.match(/^\d+/) ? "_" : "") + _;
  }
  function logicalExpr(O, _) {
    return isLogicalNot(O) ? `!(${logicalExpr(O.not, _)})` : isLogicalAnd(O) ? `(${O.and.map((H) => logicalExpr(H, _)).join(") && (")})` : isLogicalOr(O) ? `(${O.or.map((H) => logicalExpr(H, _)).join(") || (")})` : _(O);
  }
  function deleteNestedProperty(O, _) {
    if (_.length === 0)
      return !0;
    const H = _.shift();
    return H in O && deleteNestedProperty(O[H], _) && delete O[H], isEmpty(O);
  }
  function titleCase(O) {
    return O.charAt(0).toUpperCase() + O.substr(1);
  }
  function accessPathWithDatum(O, _ = "datum") {
    const H = splitAccessPath(O), V = [];
    for (let q = 1; q <= H.length; q++) {
      const G = `[${H.slice(0, q).map($$2).join("][")}]`;
      V.push(`${_}${G}`);
    }
    return V.join(" && ");
  }
  function flatAccessWithDatum(O, _ = "datum") {
    return `${_}[${$$2(splitAccessPath(O).join("."))}]`;
  }
  function accessWithDatumToUnescapedPath(O) {
    return `datum['${O.replaceAll("'", "\\'")}']`;
  }
  function escapePathAccess(O) {
    return O.replace(/(\[|\]|\.|'|")/g, "\\$1");
  }
  function replacePathInField(O) {
    return `${splitAccessPath(O).map(escapePathAccess).join("\\.")}`;
  }
  function replaceAll$1(O, _, H) {
    return O.replace(new RegExp(_.replace(/[-/\\^$*+?.()|[\]{}]/g, "\\$&"), "g"), H);
  }
  function removePathFromField(O) {
    return `${splitAccessPath(O).join(".")}`;
  }
  function accessPathDepth(O) {
    return O ? splitAccessPath(O).length : 0;
  }
  function getFirstDefined(...O) {
    return O.find((_) => _ !== void 0);
  }
  let idCounter = 42;
  function uniqueId(O) {
    const _ = ++idCounter;
    return O ? String(O) + _ : _;
  }
  function resetIdCounter() {
    idCounter = 42;
  }
  function internalField(O) {
    return isInternalField(O) ? O : `__${O}`;
  }
  function isInternalField(O) {
    return O.startsWith("__");
  }
  function normalizeAngle(O) {
    if (O !== void 0)
      return (O % 360 + 360) % 360;
  }
  function isNumeric(O) {
    return isNumber$2(O) ? !0 : !isNaN(O) && !isNaN(parseFloat(O));
  }
  const clonedProto = Object.getPrototypeOf(structuredClone({}));
  function deepEqual(O, _) {
    if (O === _)
      return !0;
    if (O && _ && typeof O == "object" && typeof _ == "object") {
      if (O.constructor.name !== _.constructor.name)
        return !1;
      let H, V;
      if (Array.isArray(O)) {
        if (H = O.length, H != _.length)
          return !1;
        for (V = H; V-- !== 0; )
          if (!deepEqual(O[V], _[V]))
            return !1;
        return !0;
      }
      if (O instanceof Map && _ instanceof Map) {
        if (O.size !== _.size)
          return !1;
        for (const G of O.entries())
          if (!_.has(G[0]))
            return !1;
        for (const G of O.entries())
          if (!deepEqual(G[1], _.get(G[0])))
            return !1;
        return !0;
      }
      if (O instanceof Set && _ instanceof Set) {
        if (O.size !== _.size)
          return !1;
        for (const G of O.entries())
          if (!_.has(G[0]))
            return !1;
        return !0;
      }
      if (ArrayBuffer.isView(O) && ArrayBuffer.isView(_)) {
        if (H = O.length, H != _.length)
          return !1;
        for (V = H; V-- !== 0; )
          if (O[V] !== _[V])
            return !1;
        return !0;
      }
      if (O.constructor === RegExp)
        return O.source === _.source && O.flags === _.flags;
      if (O.valueOf !== Object.prototype.valueOf && O.valueOf !== clonedProto.valueOf)
        return O.valueOf() === _.valueOf();
      if (O.toString !== Object.prototype.toString && O.toString !== clonedProto.toString)
        return O.toString() === _.toString();
      const q = Object.keys(O);
      if (H = q.length, H !== Object.keys(_).length)
        return !1;
      for (V = H; V-- !== 0; )
        if (!Object.prototype.hasOwnProperty.call(_, q[V]))
          return !1;
      for (V = H; V-- !== 0; ) {
        const G = q[V];
        if (!deepEqual(O[G], _[G]))
          return !1;
      }
      return !0;
    }
    return O !== O && _ !== _;
  }
  function stringify$1(O) {
    const _ = [];
    return function H(V) {
      if (V && V.toJSON && typeof V.toJSON == "function" && (V = V.toJSON()), V === void 0)
        return;
      if (typeof V == "number")
        return isFinite(V) ? "" + V : "null";
      if (typeof V != "object")
        return JSON.stringify(V);
      let q, G;
      if (Array.isArray(V)) {
        for (G = "[", q = 0; q < V.length; q++)
          q && (G += ","), G += H(V[q]) || "null";
        return G + "]";
      }
      if (V === null)
        return "null";
      if (_.includes(V))
        throw new TypeError("Converting circular structure to JSON");
      const J = _.push(V) - 1, Z = Object.keys(V).sort();
      for (G = "", q = 0; q < Z.length; q++) {
        const te = Z[q], ne = H(V[te]);
        ne && (G && (G += ","), G += JSON.stringify(te) + ":" + ne);
      }
      return _.splice(J, 1), `{${G}}`;
    }(O);
  }
  function hasProperty(O, _) {
    return isObject$2(O) && has$1(O, _) && O[_] !== void 0;
  }
  const ROW = "row", COLUMN = "column", FACET = "facet", X = "x", Y$1 = "y", X2 = "x2", Y2 = "y2", XOFFSET = "xOffset", YOFFSET = "yOffset", RADIUS = "radius", RADIUS2 = "radius2", THETA = "theta", THETA2 = "theta2", LATITUDE = "latitude", LONGITUDE = "longitude", LATITUDE2 = "latitude2", LONGITUDE2 = "longitude2", TIME = "time", COLOR = "color", FILL = "fill", STROKE = "stroke", SHAPE = "shape", SIZE = "size", ANGLE = "angle", OPACITY = "opacity", FILLOPACITY = "fillOpacity", STROKEOPACITY = "strokeOpacity", STROKEWIDTH = "strokeWidth", STROKEDASH = "strokeDash", TEXT$1 = "text", ORDER = "order", DETAIL = "detail", KEY = "key", TOOLTIP = "tooltip", HREF = "href", URL$1 = "url", DESCRIPTION = "description", POSITION_CHANNEL_INDEX = {
    x: 1,
    y: 1,
    x2: 1,
    y2: 1
  }, POLAR_POSITION_CHANNEL_INDEX = {
    theta: 1,
    theta2: 1,
    radius: 1,
    radius2: 1
  };
  function isPolarPositionChannel(O) {
    return has$1(POLAR_POSITION_CHANNEL_INDEX, O);
  }
  const GEO_POSIITON_CHANNEL_INDEX = {
    longitude: 1,
    longitude2: 1,
    latitude: 1,
    latitude2: 1
  };
  function getPositionChannelFromLatLong(O) {
    switch (O) {
      case LATITUDE:
        return "y";
      case LATITUDE2:
        return "y2";
      case LONGITUDE:
        return "x";
      case LONGITUDE2:
        return "x2";
    }
  }
  function isGeoPositionChannel(O) {
    return has$1(GEO_POSIITON_CHANNEL_INDEX, O);
  }
  const GEOPOSITION_CHANNELS = keys$2(GEO_POSIITON_CHANNEL_INDEX), UNIT_CHANNEL_INDEX = {
    ...POSITION_CHANNEL_INDEX,
    ...POLAR_POSITION_CHANNEL_INDEX,
    ...GEO_POSIITON_CHANNEL_INDEX,
    xOffset: 1,
    yOffset: 1,
    // color
    color: 1,
    fill: 1,
    stroke: 1,
    // time
    time: 1,
    // other non-position with scale
    opacity: 1,
    fillOpacity: 1,
    strokeOpacity: 1,
    strokeWidth: 1,
    strokeDash: 1,
    size: 1,
    angle: 1,
    shape: 1,
    // channels without scales
    order: 1,
    text: 1,
    detail: 1,
    key: 1,
    tooltip: 1,
    href: 1,
    url: 1,
    description: 1
  };
  function isColorChannel(O) {
    return O === COLOR || O === FILL || O === STROKE;
  }
  const FACET_CHANNEL_INDEX = {
    row: 1,
    column: 1,
    facet: 1
  }, FACET_CHANNELS = keys$2(FACET_CHANNEL_INDEX), CHANNEL_INDEX = {
    ...UNIT_CHANNEL_INDEX,
    ...FACET_CHANNEL_INDEX
  }, CHANNELS = keys$2(CHANNEL_INDEX), { order: _o$1, detail: _d$1, tooltip: _tt1, ...SINGLE_DEF_CHANNEL_INDEX } = CHANNEL_INDEX, { row: _r$1, column: _c$1, facet: _f$1, ...SINGLE_DEF_UNIT_CHANNEL_INDEX } = SINGLE_DEF_CHANNEL_INDEX;
  function isSingleDefUnitChannel(O) {
    return has$1(SINGLE_DEF_UNIT_CHANNEL_INDEX, O);
  }
  function isChannel(O) {
    return has$1(CHANNEL_INDEX, O);
  }
  const SECONDARY_RANGE_CHANNEL = [X2, Y2, LATITUDE2, LONGITUDE2, THETA2, RADIUS2];
  function isSecondaryRangeChannel(O) {
    return getMainRangeChannel(O) !== O;
  }
  function getMainRangeChannel(O) {
    switch (O) {
      case X2:
        return X;
      case Y2:
        return Y$1;
      case LATITUDE2:
        return LATITUDE;
      case LONGITUDE2:
        return LONGITUDE;
      case THETA2:
        return THETA;
      case RADIUS2:
        return RADIUS;
    }
    return O;
  }
  function getVgPositionChannel(O) {
    if (isPolarPositionChannel(O))
      switch (O) {
        case THETA:
          return "startAngle";
        case THETA2:
          return "endAngle";
        case RADIUS:
          return "outerRadius";
        case RADIUS2:
          return "innerRadius";
      }
    return O;
  }
  function getSecondaryRangeChannel(O) {
    switch (O) {
      case X:
        return X2;
      case Y$1:
        return Y2;
      case LATITUDE:
        return LATITUDE2;
      case LONGITUDE:
        return LONGITUDE2;
      case THETA:
        return THETA2;
      case RADIUS:
        return RADIUS2;
    }
  }
  function getSizeChannel(O) {
    switch (O) {
      case X:
      case X2:
        return "width";
      case Y$1:
      case Y2:
        return "height";
    }
  }
  function getOffsetChannel(O) {
    switch (O) {
      case X:
        return "xOffset";
      case Y$1:
        return "yOffset";
      case X2:
        return "x2Offset";
      case Y2:
        return "y2Offset";
      case THETA:
        return "thetaOffset";
      case RADIUS:
        return "radiusOffset";
      case THETA2:
        return "theta2Offset";
      case RADIUS2:
        return "radius2Offset";
    }
  }
  function getOffsetScaleChannel(O) {
    switch (O) {
      case X:
        return "xOffset";
      case Y$1:
        return "yOffset";
    }
  }
  function getMainChannelFromOffsetChannel(O) {
    switch (O) {
      case "xOffset":
        return "x";
      case "yOffset":
        return "y";
    }
  }
  const UNIT_CHANNELS = keys$2(UNIT_CHANNEL_INDEX), {
    x: _x$1,
    y: _y,
    // x2 and y2 share the same scale as x and y
    x2: _x2,
    y2: _y2,
    //
    xOffset: _xo,
    yOffset: _yo,
    latitude: _latitude,
    longitude: _longitude,
    latitude2: _latitude2,
    longitude2: _longitude2,
    theta: _theta,
    theta2: _theta2,
    radius: _radius,
    radius2: _radius2,
    // The rest of unit channels then have scale
    ...NONPOSITION_CHANNEL_INDEX
  } = UNIT_CHANNEL_INDEX, NONPOSITION_CHANNELS = keys$2(NONPOSITION_CHANNEL_INDEX), POSITION_SCALE_CHANNEL_INDEX = {
    x: 1,
    y: 1
  }, POSITION_SCALE_CHANNELS = keys$2(POSITION_SCALE_CHANNEL_INDEX);
  function isXorY(O) {
    return has$1(POSITION_SCALE_CHANNEL_INDEX, O);
  }
  const POLAR_POSITION_SCALE_CHANNEL_INDEX = {
    theta: 1,
    radius: 1
  }, POLAR_POSITION_SCALE_CHANNELS = keys$2(POLAR_POSITION_SCALE_CHANNEL_INDEX);
  function getPositionScaleChannel(O) {
    return O === "width" ? X : Y$1;
  }
  const OFFSET_SCALE_CHANNEL_INDEX = { xOffset: 1, yOffset: 1 };
  function isXorYOffset(O) {
    return has$1(OFFSET_SCALE_CHANNEL_INDEX, O);
  }
  const TIME_SCALE_CHANNEL_INDEX = {
    time: 1
  };
  function isTime(O) {
    return O in TIME_SCALE_CHANNEL_INDEX;
  }
  const {
    // x2 and y2 share the same scale as x and y
    // text and tooltip have format instead of scale,
    // href has neither format, nor scale
    text: _t$1,
    tooltip: _tt,
    href: _hr,
    url: _u$1,
    description: _al,
    // detail and order have no scale
    detail: _dd,
    key: _k$1,
    order: _oo,
    ...NONPOSITION_SCALE_CHANNEL_INDEX
  } = NONPOSITION_CHANNEL_INDEX, NONPOSITION_SCALE_CHANNELS = keys$2(NONPOSITION_SCALE_CHANNEL_INDEX);
  function isNonPositionScaleChannel(O) {
    return has$1(NONPOSITION_CHANNEL_INDEX, O);
  }
  function supportLegend(O) {
    switch (O) {
      case COLOR:
      case FILL:
      case STROKE:
      case SIZE:
      case SHAPE:
      case OPACITY:
      case STROKEWIDTH:
      case STROKEDASH:
        return !0;
      case FILLOPACITY:
      case STROKEOPACITY:
      case ANGLE:
      case TIME:
        return !1;
    }
  }
  const SCALE_CHANNEL_INDEX = {
    ...POSITION_SCALE_CHANNEL_INDEX,
    ...POLAR_POSITION_SCALE_CHANNEL_INDEX,
    ...OFFSET_SCALE_CHANNEL_INDEX,
    ...NONPOSITION_SCALE_CHANNEL_INDEX
  }, SCALE_CHANNELS = keys$2(SCALE_CHANNEL_INDEX);
  function isScaleChannel(O) {
    return has$1(SCALE_CHANNEL_INDEX, O);
  }
  function supportMark(O, _) {
    return getSupportedMark(O)[_];
  }
  const ALL_MARKS = {
    // all marks
    arc: "always",
    area: "always",
    bar: "always",
    circle: "always",
    geoshape: "always",
    image: "always",
    line: "always",
    rule: "always",
    point: "always",
    rect: "always",
    square: "always",
    trail: "always",
    text: "always",
    tick: "always"
  }, { geoshape: _g$1, ...ALL_MARKS_EXCEPT_GEOSHAPE } = ALL_MARKS;
  function getSupportedMark(O) {
    switch (O) {
      case COLOR:
      case FILL:
      case STROKE:
      // falls through
      case DESCRIPTION:
      case DETAIL:
      case KEY:
      case TOOLTIP:
      case HREF:
      case ORDER:
      // TODO: revise (order might not support rect, which is not stackable?)
      case OPACITY:
      case FILLOPACITY:
      case STROKEOPACITY:
      case STROKEWIDTH:
      // falls through
      case FACET:
      case ROW:
      // falls through
      case COLUMN:
        return ALL_MARKS;
      case X:
      case Y$1:
      case XOFFSET:
      case YOFFSET:
      case LATITUDE:
      case LONGITUDE:
      case TIME:
        return ALL_MARKS_EXCEPT_GEOSHAPE;
      case X2:
      case Y2:
      case LATITUDE2:
      case LONGITUDE2:
        return {
          area: "always",
          bar: "always",
          image: "always",
          rect: "always",
          rule: "always",
          circle: "binned",
          point: "binned",
          square: "binned",
          tick: "binned",
          line: "binned",
          trail: "binned"
        };
      case SIZE:
        return {
          point: "always",
          tick: "always",
          rule: "always",
          circle: "always",
          square: "always",
          bar: "always",
          text: "always",
          line: "always",
          trail: "always"
        };
      case STROKEDASH:
        return {
          line: "always",
          point: "always",
          tick: "always",
          rule: "always",
          circle: "always",
          square: "always",
          bar: "always",
          geoshape: "always"
        };
      case SHAPE:
        return { point: "always", geoshape: "always" };
      case TEXT$1:
        return { text: "always" };
      case ANGLE:
        return { point: "always", square: "always", text: "always" };
      case URL$1:
        return { image: "always" };
      case THETA:
        return { text: "always", arc: "always" };
      case RADIUS:
        return { text: "always", arc: "always" };
      case THETA2:
      case RADIUS2:
        return { arc: "always" };
    }
  }
  function rangeType(O) {
    switch (O) {
      case X:
      case Y$1:
      case THETA:
      case RADIUS:
      case XOFFSET:
      case YOFFSET:
      case SIZE:
      case ANGLE:
      case STROKEWIDTH:
      case OPACITY:
      case FILLOPACITY:
      case STROKEOPACITY:
      case TIME:
      // X2 and Y2 use X and Y scales, so they similarly have continuous range. [falls through]
      case X2:
      case Y2:
      case THETA2:
      case RADIUS2:
        return;
      case FACET:
      case ROW:
      case COLUMN:
      case SHAPE:
      case STROKEDASH:
      // TEXT, TOOLTIP, URL, and HREF have no scale but have discrete output [falls through]
      case TEXT$1:
      case TOOLTIP:
      case HREF:
      case URL$1:
      case DESCRIPTION:
        return "discrete";
      // Color can be either continuous or discrete, depending on scale type.
      case COLOR:
      case FILL:
      case STROKE:
        return "flexible";
      // No scale, no range type.
      case LATITUDE:
      case LONGITUDE:
      case LATITUDE2:
      case LONGITUDE2:
      case DETAIL:
      case KEY:
      case ORDER:
        return;
    }
  }
  const AGGREGATE_OP_INDEX = {
    argmax: 1,
    argmin: 1,
    average: 1,
    count: 1,
    distinct: 1,
    exponential: 1,
    exponentialb: 1,
    product: 1,
    max: 1,
    mean: 1,
    median: 1,
    min: 1,
    missing: 1,
    q1: 1,
    q3: 1,
    ci0: 1,
    ci1: 1,
    stderr: 1,
    stdev: 1,
    stdevp: 1,
    sum: 1,
    valid: 1,
    values: 1,
    variance: 1,
    variancep: 1
  }, MULTIDOMAIN_SORT_OP_INDEX = {
    count: 1,
    min: 1,
    max: 1
  };
  function isArgminDef(O) {
    return hasProperty(O, "argmin");
  }
  function isArgmaxDef(O) {
    return hasProperty(O, "argmax");
  }
  function isAggregateOp(O) {
    return isString$1(O) && has$1(AGGREGATE_OP_INDEX, O);
  }
  const COUNTING_OPS = /* @__PURE__ */ new Set([
    "count",
    "valid",
    "missing",
    "distinct"
  ]);
  function isCountingAggregateOp(O) {
    return isString$1(O) && COUNTING_OPS.has(O);
  }
  function isMinMaxOp(O) {
    return isString$1(O) && contains$2(["min", "max"], O);
  }
  const SUM_OPS = /* @__PURE__ */ new Set([
    "count",
    "sum",
    "distinct",
    "valid",
    "missing"
  ]), SHARED_DOMAIN_OPS = /* @__PURE__ */ new Set([
    "mean",
    "average",
    "median",
    "q1",
    "q3",
    "min",
    "max"
  ]);
  function binToString(O) {
    return isBoolean$3(O) && (O = normalizeBin(O, void 0)), "bin" + keys$2(O).map((_) => isParameterExtent(O[_]) ? varName(`_${_}_${entries$1(O[_])}`) : varName(`_${_}_${O[_]}`)).join("");
  }
  function isBinning(O) {
    return O === !0 || isBinParams(O) && !O.binned;
  }
  function isBinned(O) {
    return O === "binned" || isBinParams(O) && O.binned === !0;
  }
  function isBinParams(O) {
    return isObject$2(O);
  }
  function isParameterExtent(O) {
    return hasProperty(O, "param");
  }
  function autoMaxBins(O) {
    switch (O) {
      case ROW:
      case COLUMN:
      case SIZE:
      case COLOR:
      case FILL:
      case STROKE:
      case STROKEWIDTH:
      case OPACITY:
      case FILLOPACITY:
      case STROKEOPACITY:
      // Facets and Size shouldn't have too many bins
      // We choose 6 like shape to simplify the rule [falls through]
      case SHAPE:
        return 6;
      // Vega's "shape" has 6 distinct values
      case STROKEDASH:
        return 4;
      // We only provide 5 different stroke dash values (but 4 is more effective)
      default:
        return 10;
    }
  }
  function isExprRef(O) {
    return hasProperty(O, "expr");
  }
  function replaceExprRef(O, { level: _ } = { level: 0 }) {
    const H = keys$2(O || {}), V = {};
    for (const q of H)
      V[q] = _ === 0 ? signalRefOrValue(O[q]) : replaceExprRef(O[q], { level: _ - 1 });
    return V;
  }
  function extractTitleConfig(O) {
    const {
      // These are non-mark title config that need to be hardcoded
      anchor: _,
      frame: H,
      offset: V,
      orient: q,
      angle: G,
      limit: J,
      // color needs to be redirect to fill
      color: Z,
      // subtitle properties
      subtitleColor: te,
      subtitleFont: ne,
      subtitleFontSize: ee,
      subtitleFontStyle: ae,
      subtitleFontWeight: oe,
      subtitleLineHeight: se,
      subtitlePadding: ce,
      // The rest are mark config.
      ...le
    } = O, ue = {
      ...le,
      ...Z ? { fill: Z } : {}
    }, de = {
      ..._ ? { anchor: _ } : {},
      ...H ? { frame: H } : {},
      ...V ? { offset: V } : {},
      ...q ? { orient: q } : {},
      ...G !== void 0 ? { angle: G } : {},
      ...J !== void 0 ? { limit: J } : {}
    }, fe = {
      ...te ? { subtitleColor: te } : {},
      ...ne ? { subtitleFont: ne } : {},
      ...ee ? { subtitleFontSize: ee } : {},
      ...ae ? { subtitleFontStyle: ae } : {},
      ...oe ? { subtitleFontWeight: oe } : {},
      ...se ? { subtitleLineHeight: se } : {},
      ...ce ? { subtitlePadding: ce } : {}
    }, he = pick$1(O, ["align", "baseline", "dx", "dy", "limit"]);
    return { titleMarkConfig: ue, subtitleMarkConfig: he, nonMarkTitleProperties: de, subtitle: fe };
  }
  function isText(O) {
    return isString$1(O) || isArray$2(O) && isString$1(O[0]);
  }
  function isSignalRef(O) {
    return hasProperty(O, "signal");
  }
  function isVgRangeStep(O) {
    return hasProperty(O, "step");
  }
  function isDataRefUnionedDomain(O) {
    return isArray$2(O) ? !1 : hasProperty(O, "fields") && !hasProperty(O, "data");
  }
  function isFieldRefUnionDomain(O) {
    return isArray$2(O) ? !1 : hasProperty(O, "fields") && hasProperty(O, "data");
  }
  function isDataRefDomain(O) {
    return isArray$2(O) ? !1 : hasProperty(O, "field") && hasProperty(O, "data");
  }
  const VG_MARK_CONFIG_INDEX = {
    aria: 1,
    description: 1,
    ariaRole: 1,
    ariaRoleDescription: 1,
    blend: 1,
    opacity: 1,
    fill: 1,
    fillOpacity: 1,
    stroke: 1,
    strokeCap: 1,
    strokeWidth: 1,
    strokeOpacity: 1,
    strokeDash: 1,
    strokeDashOffset: 1,
    strokeJoin: 1,
    strokeOffset: 1,
    strokeMiterLimit: 1,
    startAngle: 1,
    endAngle: 1,
    padAngle: 1,
    innerRadius: 1,
    outerRadius: 1,
    size: 1,
    shape: 1,
    interpolate: 1,
    tension: 1,
    orient: 1,
    align: 1,
    baseline: 1,
    text: 1,
    dir: 1,
    dx: 1,
    dy: 1,
    ellipsis: 1,
    limit: 1,
    radius: 1,
    theta: 1,
    angle: 1,
    font: 1,
    fontSize: 1,
    fontWeight: 1,
    fontStyle: 1,
    lineBreak: 1,
    lineHeight: 1,
    cursor: 1,
    href: 1,
    tooltip: 1,
    cornerRadius: 1,
    cornerRadiusTopLeft: 1,
    cornerRadiusTopRight: 1,
    cornerRadiusBottomLeft: 1,
    cornerRadiusBottomRight: 1,
    aspect: 1,
    width: 1,
    height: 1,
    url: 1,
    smooth: 1
    // commented below are vg channel that do not have mark config.
    // x: 1,
    // y: 1,
    // x2: 1,
    // y2: 1,
    // xc'|'yc'
    // clip: 1,
    // path: 1,
    // url: 1,
  }, VG_MARK_CONFIGS = keys$2(VG_MARK_CONFIG_INDEX), VG_MARK_INDEX = {
    arc: 1,
    area: 1,
    group: 1,
    image: 1,
    line: 1,
    path: 1,
    rect: 1,
    rule: 1,
    shape: 1,
    symbol: 1,
    text: 1,
    trail: 1
  }, VG_CORNERRADIUS_CHANNELS = [
    "cornerRadius",
    "cornerRadiusTopLeft",
    "cornerRadiusTopRight",
    "cornerRadiusBottomLeft",
    "cornerRadiusBottomRight"
  ];
  function signalOrValueRefWithCondition(O) {
    const _ = isArray$2(O.condition) ? O.condition.map(conditionalSignalRefOrValue) : conditionalSignalRefOrValue(O.condition);
    return {
      ...signalRefOrValue(O),
      condition: _
    };
  }
  function signalRefOrValue(O) {
    if (isExprRef(O)) {
      const { expr: _, ...H } = O;
      return { signal: _, ...H };
    }
    return O;
  }
  function conditionalSignalRefOrValue(O) {
    if (isExprRef(O)) {
      const { expr: _, ...H } = O;
      return { signal: _, ...H };
    }
    return O;
  }
  function signalOrValueRef(O) {
    if (isExprRef(O)) {
      const { expr: _, ...H } = O;
      return { signal: _, ...H };
    }
    return isSignalRef(O) ? O : O !== void 0 ? { value: O } : void 0;
  }
  function exprFromSignalRefOrValue(O) {
    return isSignalRef(O) ? O.signal : $$2(O);
  }
  function exprFromValueRefOrSignalRef(O) {
    return isSignalRef(O) ? O.signal : $$2(O.value);
  }
  function signalOrStringValue(O) {
    return isSignalRef(O) ? O.signal : O == null ? null : $$2(O);
  }
  function applyMarkConfig(O, _, H) {
    for (const V of H) {
      const q = getMarkConfig(V, _.markDef, _.config);
      q !== void 0 && (O[V] = signalOrValueRef(q));
    }
    return O;
  }
  function getStyles(O) {
    return [].concat(O.type, O.style ?? []);
  }
  function getMarkPropOrConfig(O, _, H, V = {}) {
    const { vgChannel: q, ignoreVgConfig: G } = V;
    return q && hasProperty(_, q) ? _[q] : _[O] !== void 0 ? _[O] : G && (!q || q === O) ? void 0 : getMarkConfig(O, _, H, V);
  }
  function getMarkConfig(O, _, H, { vgChannel: V } = {}) {
    const q = getMarkStyleConfig(O, _, H.style);
    return getFirstDefined(
      // style config has highest precedence
      V ? q : void 0,
      q,
      // then mark-specific config
      V ? H[_.type][V] : void 0,
      H[_.type][O],
      // Need to cast because MarkDef doesn't perfectly match with AnyMarkConfig, but if the type isn't available, we'll get nothing here, which is fine
      // If there is vgChannel, skip vl channel.
      // For example, vl size for text is vg fontSize, but config.mark.size is only for point size.
      V ? H.mark[V] : H.mark[O]
      // Need to cast for the same reason as above
    );
  }
  function getMarkStyleConfig(O, _, H) {
    return getStyleConfig(O, getStyles(_), H);
  }
  function getStyleConfig(O, _, H) {
    _ = array$5(_);
    let V;
    for (const q of _) {
      const G = H[q];
      hasProperty(G, O) && (V = G[O]);
    }
    return V;
  }
  function sortParams(O, _) {
    return array$5(O).reduce((H, V) => (H.field.push(vgField(V, _)), H.order.push(V.sort ?? "ascending"), H), { field: [], order: [] });
  }
  function mergeTitleFieldDefs(O, _) {
    const H = [...O];
    return _.forEach((V) => {
      for (const q of H)
        if (deepEqual(q, V))
          return;
      H.push(V);
    }), H;
  }
  function mergeTitle(O, _) {
    return deepEqual(O, _) || !_ ? O : O ? [...array$5(O), ...array$5(_)].join(", ") : _;
  }
  function mergeTitleComponent(O, _) {
    const H = O.value, V = _.value;
    if (H == null || V === null)
      return {
        explicit: O.explicit,
        value: null
      };
    if ((isText(H) || isSignalRef(H)) && (isText(V) || isSignalRef(V)))
      return {
        explicit: O.explicit,
        value: mergeTitle(H, V)
      };
    if (isText(H) || isSignalRef(H))
      return {
        explicit: O.explicit,
        value: H
      };
    if (isText(V) || isSignalRef(V))
      return {
        explicit: O.explicit,
        value: V
      };
    if (!isText(H) && !isSignalRef(H) && !isText(V) && !isSignalRef(V))
      return {
        explicit: O.explicit,
        value: mergeTitleFieldDefs(H, V)
      };
    throw new Error("It should never reach here");
  }
  function invalidSpec(O) {
    return `Invalid specification ${stringify$1(O)}. Make sure the specification includes at least one of the following properties: "mark", "layer", "facet", "hconcat", "vconcat", "concat", or "repeat".`;
  }
  const FIT_NON_SINGLE = 'Autosize "fit" only works for single views and layered views.';
  function containerSizeNonSingle(O) {
    return `${O == "width" ? "Width" : "Height"} "container" only works for single views and layered views.`;
  }
  function containerSizeNotCompatibleWithAutosize(O) {
    const _ = O == "width" ? "Width" : "Height", H = O == "width" ? "x" : "y";
    return `${_} "container" only works well with autosize "fit" or "fit-${H}".`;
  }
  function droppingFit(O) {
    return O ? `Dropping "fit-${O}" because spec has discrete ${getSizeChannel(O)}.` : 'Dropping "fit" because spec has discrete size.';
  }
  function unknownField(O) {
    return `Unknown field for ${O}. Cannot calculate view size.`;
  }
  function cannotProjectOnChannelWithoutField(O) {
    return `Cannot project a selection on encoding channel "${O}", which has no field.`;
  }
  function cannotProjectAggregate(O, _) {
    return `Cannot project a selection on encoding channel "${O}" as it uses an aggregate function ("${_}").`;
  }
  function nearestNotSupportForContinuous(O) {
    return `The "nearest" transform is not supported for ${O} marks.`;
  }
  function selectionNotSupported(O) {
    return `Selection not supported for ${O} yet.`;
  }
  function selectionNotFound(O) {
    return `Cannot find a selection named "${O}".`;
  }
  const SCALE_BINDINGS_CONTINUOUS = "Scale bindings are currently only supported for scales with unbinned, continuous domains.", SEQUENTIAL_SCALE_DEPRECATED = "Sequntial scales are deprecated. The available quantitative scale type values are linear, log, pow, sqrt, symlog, time and utc", LEGEND_BINDINGS_MUST_HAVE_PROJECTION = "Legend bindings are only supported for selections over an individual field or encoding channel.";
  function cannotLookupVariableParameter(O) {
    return `Lookups can only be performed on selection parameters. "${O}" is a variable parameter.`;
  }
  function noSameUnitLookup(O) {
    return `Cannot define and lookup the "${O}" selection in the same view. Try moving the lookup into a second, layered view?`;
  }
  const NEEDS_SAME_SELECTION = "The same selection must be used to override scale domains in a layered view.", INTERVAL_INITIALIZED_WITH_POS = 'Interval selections should be initialized using "x", "y", "longitude", or "latitude" keys.';
  function noSuchRepeatedValue(O) {
    return `Unknown repeated value "${O}".`;
  }
  function columnsNotSupportByRowCol(O) {
    return `The "columns" property cannot be used when "${O}" has nested row/column.`;
  }
  const MULTIPLE_TIMER_ANIMATION_SELECTION = "Multiple timer selections in one unit spec are not supported. Ignoring all but the first.", MULTI_VIEW_ANIMATION_UNSUPPORTED = "Animation involving facet, layer, or concat is currently unsupported.";
  function selectionAsScaleDomainWithoutField(O) {
    return `A "field" or "encoding" must be specified when using a selection as a scale domain. Using "field": ${$$2(O)}.`;
  }
  function selectionAsScaleDomainWrongEncodings(O, _, H, V) {
    return (O.length ? "Multiple " : "No ") + `matching ${$$2(_)} encoding found for selection ${$$2(H.param)}. Using "field": ${$$2(V)}.`;
  }
  const CONCAT_CANNOT_SHARE_AXIS = "Axes cannot be shared in concatenated or repeated views yet (https://github.com/vega/vega-lite/issues/2415).";
  function unrecognizedParse(O) {
    return `Unrecognized parse "${O}".`;
  }
  function differentParse(O, _, H) {
    return `An ancestor parsed field "${O}" as ${H} but a child wants to parse the field as ${_}.`;
  }
  const ADD_SAME_CHILD_TWICE = "Attempt to add the same child twice.";
  function invalidTransformIgnored(O) {
    return `Ignoring an invalid transform: ${stringify$1(O)}.`;
  }
  const NO_FIELDS_NEEDS_AS = 'If "from.fields" is not specified, "as" has to be a string that specifies the key to be used for the data from the secondary source.';
  function customFormatTypeNotAllowed(O) {
    return `Config.customFormatTypes is not true, thus custom format type and format for channel ${O} are dropped.`;
  }
  function projectionOverridden(O) {
    const { parentProjection: _, projection: H } = O;
    return `Layer's shared projection ${stringify$1(_)} is overridden by a child projection ${stringify$1(H)}.`;
  }
  const REPLACE_ANGLE_WITH_THETA = "Arc marks uses theta channel rather than angle, replacing angle with theta.";
  function offsetNestedInsideContinuousPositionScaleDropped(O) {
    return `${O}Offset dropped because ${O} is continuous`;
  }
  function primitiveChannelDef(O, _, H) {
    return `Channel ${O} is a ${_}. Converted to {value: ${stringify$1(H)}}.`;
  }
  function invalidFieldType(O) {
    return `Invalid field type "${O}".`;
  }
  function invalidFieldTypeForCountAggregate(O, _) {
    return `Invalid field type "${O}" for aggregate: "${_}", using "quantitative" instead.`;
  }
  function invalidAggregate(O) {
    return `Invalid aggregation operator "${O}".`;
  }
  function droppingColor(O, _) {
    const { fill: H, stroke: V } = _;
    return `Dropping color ${O} as the plot also has ${H && V ? "fill and stroke" : H ? "fill" : "stroke"}.`;
  }
  function relativeBandSizeNotSupported(O) {
    return `Position range does not support relative band size for ${O}.`;
  }
  function emptyFieldDef(O, _) {
    return `Dropping ${stringify$1(O)} from channel "${_}" since it does not contain any data field, datum, value, or signal.`;
  }
  const LINE_WITH_VARYING_SIZE = "Line marks cannot encode size with a non-groupby field. You may want to use trail marks instead.";
  function incompatibleChannel(O, _, H) {
    return `${O} dropped as it is incompatible with "${_}".`;
  }
  function invalidEncodingChannel(O) {
    return `${O}-encoding is dropped as ${O} is not a valid encoding channel.`;
  }
  function channelShouldBeDiscrete(O) {
    return `${O} encoding should be discrete (ordinal / nominal / binned).`;
  }
  function channelShouldBeDiscreteOrDiscretizing(O) {
    return `${O} encoding should be discrete (ordinal / nominal / binned) or use a discretizing scale (e.g. threshold).`;
  }
  function facetChannelDropped(O) {
    return `Facet encoding dropped as ${O.join(" and ")} ${O.length > 1 ? "are" : "is"} also specified.`;
  }
  function discreteChannelCannotEncode(O, _) {
    return `Using discrete channel "${O}" to encode "${_}" field can be misleading as it does not encode ${_ === "ordinal" ? "order" : "magnitude"}.`;
  }
  function rangeMarkAlignmentCannotBeExpression(O) {
    return `The ${O} for range marks cannot be an expression`;
  }
  function lineWithRange(O, _) {
    return `Line mark is for continuous lines and thus cannot be used with ${O && _ ? "x2 and y2" : O ? "x2" : "y2"}. We will use the rule mark (line segments) instead.`;
  }
  function orientOverridden(O, _) {
    return `Specified orient "${O}" overridden with "${_}".`;
  }
  function cannotUseScalePropertyWithNonColor(O) {
    return `Cannot use the scale property "${O}" with non-color channel.`;
  }
  function cannotUseRelativeBandSizeWithNonBandScale(O) {
    return `Cannot use the relative band size with ${O} scale.`;
  }
  function unaggregateDomainHasNoEffectForRawField(O) {
    return `Using unaggregated domain with raw field has no effect (${stringify$1(O)}).`;
  }
  function unaggregateDomainWithNonSharedDomainOp(O) {
    return `Unaggregated domain not applicable for "${O}" since it produces values outside the origin domain of the source data.`;
  }
  function unaggregatedDomainWithLogScale(O) {
    return `Unaggregated domain is currently unsupported for log scale (${stringify$1(O)}).`;
  }
  function cannotApplySizeToNonOrientedMark(O) {
    return `Cannot apply size to non-oriented mark "${O}".`;
  }
  function scaleTypeNotWorkWithChannel(O, _, H) {
    return `Channel "${O}" does not work with "${_}" scale. We are using "${H}" scale instead.`;
  }
  function scaleTypeNotWorkWithFieldDef(O, _) {
    return `FieldDef does not work with "${O}" scale. We are using "${_}" scale instead.`;
  }
  function scalePropertyNotWorkWithScaleType(O, _, H) {
    return `${H}-scale's "${_}" is dropped as it does not work with ${O} scale.`;
  }
  function stepDropped(O) {
    return `The step for "${O}" is dropped because the ${O === "width" ? "x" : "y"} is continuous.`;
  }
  function mergeConflictingProperty(O, _, H, V) {
    return `Conflicting ${_.toString()} property "${O.toString()}" (${stringify$1(H)} and ${stringify$1(V)}). Using ${stringify$1(H)}.`;
  }
  function mergeConflictingDomainProperty(O, _, H, V) {
    return `Conflicting ${_.toString()} property "${O.toString()}" (${stringify$1(H)} and ${stringify$1(V)}). Using the union of the two domains.`;
  }
  function independentScaleMeansIndependentGuide(O) {
    return `Setting the scale to be independent for "${O}" means we also have to set the guide (axis or legend) to be independent.`;
  }
  function domainSortDropped(O) {
    return `Dropping sort property ${stringify$1(O)} as unioned domains only support boolean or op "count", "min", and "max".`;
  }
  const MORE_THAN_ONE_SORT = "Domains that should be unioned has conflicting sort properties. Sort will be set to true.", FACETED_INDEPENDENT_DIFFERENT_SOURCES = "Detected faceted independent scales that union domain of multiple fields from different data sources. We will use the first field. The result view size may be incorrect.", FACETED_INDEPENDENT_SAME_FIELDS_DIFFERENT_SOURCES = "Detected faceted independent scales that union domain of the same fields from different source. We will assume that this is the same field from a different fork of the same data source. However, if this is not the case, the result view size may be incorrect.", FACETED_INDEPENDENT_SAME_SOURCE = "Detected faceted independent scales that union domain of multiple fields from the same data source. We will use the first field. The result view size may be incorrect.";
  function cannotStackRangedMark(O) {
    return `Cannot stack "${O}" if there is already "${O}2".`;
  }
  function stackNonLinearScale(O) {
    return `Stack is applied to a non-linear scale (${O}).`;
  }
  function stackNonSummativeAggregate(O) {
    return `Stacking is applied even though the aggregate function is non-summative ("${O}").`;
  }
  function invalidTimeUnit(O, _) {
    return `Invalid ${O}: ${stringify$1(_)}.`;
  }
  function droppedDay(O) {
    return `Dropping day from datetime ${stringify$1(O)} as day cannot be combined with other units.`;
  }
  function errorBarCenterAndExtentAreNotNeeded(O, _) {
    return `${_ ? "extent " : ""}${_ && O ? "and " : ""}${O ? "center " : ""}${_ && O ? "are " : "is "}not needed when data are aggregated.`;
  }
  function errorBarCenterIsUsedWithWrongExtent(O, _, H) {
    return `${O} is not usually used with ${_} for ${H}.`;
  }
  function errorBarContinuousAxisHasCustomizedAggregate(O, _) {
    return `Continuous axis should not have customized aggregation function ${O}; ${_} already agregates the axis.`;
  }
  function errorBand1DNotSupport(O) {
    return `1D error band does not support ${O}.`;
  }
  function channelRequiredForBinned(O) {
    return `Channel ${O} is required for "binned" bin.`;
  }
  function channelShouldNotBeUsedForBinned(O) {
    return `Channel ${O} should not be used with "binned" bin.`;
  }
  function domainRequiredForThresholdScale(O) {
    return `Domain for ${O} is required for threshold scale.`;
  }
  const main$1 = logger(Warn);
  let current = main$1;
  function set$1(O) {
    return current = O, current;
  }
  function reset() {
    return current = main$1, current;
  }
  function error(...O) {
    current.error(...O);
  }
  function warn(...O) {
    current.warn(...O);
  }
  function debug(...O) {
    current.debug(...O);
  }
  function isDateTime(O) {
    if (O && isObject$2(O)) {
      for (const _ of TIMEUNIT_PARTS)
        if (hasProperty(O, _))
          return !0;
    }
    return !1;
  }
  const MONTHS = [
    "january",
    "february",
    "march",
    "april",
    "may",
    "june",
    "july",
    "august",
    "september",
    "october",
    "november",
    "december"
  ], SHORT_MONTHS = MONTHS.map((O) => O.substr(0, 3)), DAYS = ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"], SHORT_DAYS = DAYS.map((O) => O.substr(0, 3));
  function normalizeQuarter(O) {
    if (isNumeric(O) && (O = +O), isNumber$2(O))
      return O > 4 && warn(invalidTimeUnit("quarter", O)), O - 1;
    throw new Error(invalidTimeUnit("quarter", O));
  }
  function normalizeMonth(O) {
    if (isNumeric(O) && (O = +O), isNumber$2(O))
      return O - 1;
    {
      const _ = O.toLowerCase(), H = MONTHS.indexOf(_);
      if (H !== -1)
        return H;
      const V = _.substr(0, 3), q = SHORT_MONTHS.indexOf(V);
      if (q !== -1)
        return q;
      throw new Error(invalidTimeUnit("month", O));
    }
  }
  function normalizeDay(O) {
    if (isNumeric(O) && (O = +O), isNumber$2(O))
      return O % 7;
    {
      const _ = O.toLowerCase(), H = DAYS.indexOf(_);
      if (H !== -1)
        return H;
      const V = _.substr(0, 3), q = SHORT_DAYS.indexOf(V);
      if (q !== -1)
        return q;
      throw new Error(invalidTimeUnit("day", O));
    }
  }
  function dateTimeParts(O, _) {
    const H = [];
    if (_ && O.day !== void 0 && keys$2(O).length > 1 && (warn(droppedDay(O)), O = duplicate(O), delete O.day), O.year !== void 0 ? H.push(O.year) : H.push(2012), O.month !== void 0) {
      const V = _ ? normalizeMonth(O.month) : O.month;
      H.push(V);
    } else if (O.quarter !== void 0) {
      const V = _ ? normalizeQuarter(O.quarter) : O.quarter;
      H.push(isNumber$2(V) ? V * 3 : `${V}*3`);
    } else
      H.push(0);
    if (O.date !== void 0)
      H.push(O.date);
    else if (O.day !== void 0) {
      const V = _ ? normalizeDay(O.day) : O.day;
      H.push(isNumber$2(V) ? V + 1 : `${V}+1`);
    } else
      H.push(1);
    for (const V of ["hours", "minutes", "seconds", "milliseconds"]) {
      const q = O[V];
      H.push(typeof q > "u" ? 0 : q);
    }
    return H;
  }
  function dateTimeToExpr(O) {
    const H = dateTimeParts(O, !0).join(", ");
    return O.utc ? `utc(${H})` : `datetime(${H})`;
  }
  function dateTimeExprToExpr(O) {
    const H = dateTimeParts(O, !1).join(", ");
    return O.utc ? `utc(${H})` : `datetime(${H})`;
  }
  function dateTimeToTimestamp(O) {
    const _ = dateTimeParts(O, !0);
    return O.utc ? +new Date(Date.UTC(..._)) : +new Date(..._);
  }
  const LOCAL_SINGLE_TIMEUNIT_INDEX = {
    year: 1,
    quarter: 1,
    month: 1,
    week: 1,
    day: 1,
    dayofyear: 1,
    date: 1,
    hours: 1,
    minutes: 1,
    seconds: 1,
    milliseconds: 1
  }, TIMEUNIT_PARTS = keys$2(LOCAL_SINGLE_TIMEUNIT_INDEX);
  function isLocalSingleTimeUnit(O) {
    return has$1(LOCAL_SINGLE_TIMEUNIT_INDEX, O);
  }
  function isBinnedTimeUnit(O) {
    return isObject$2(O) ? O.binned : isBinnedTimeUnitString(O);
  }
  function isBinnedTimeUnitString(O) {
    return O && O.startsWith("binned");
  }
  function isUTCTimeUnit(O) {
    return O.startsWith("utc");
  }
  function getLocalTimeUnitFromUTCTimeUnit(O) {
    return O.substring(3);
  }
  const VEGALITE_TIMEFORMAT = {
    "year-month": "%b %Y ",
    "year-month-date": "%b %d, %Y "
  };
  function getTimeUnitParts(O) {
    return TIMEUNIT_PARTS.filter((_) => containsTimeUnit(O, _));
  }
  function getSmallestTimeUnitPart(O) {
    const _ = getTimeUnitParts(O);
    return _[_.length - 1];
  }
  function containsTimeUnit(O, _) {
    const H = O.indexOf(_);
    return !(H < 0 || H > 0 && _ === "seconds" && O.charAt(H - 1) === "i" || O.length > H + 3 && _ === "day" && O.charAt(H + 3) === "o" || H > 0 && _ === "year" && O.charAt(H - 1) === "f");
  }
  function fieldExpr(O, _, { end: H } = { end: !1 }) {
    const V = accessPathWithDatum(_), q = isUTCTimeUnit(O) ? "utc" : "";
    function G(te) {
      return te === "quarter" ? `(${q}quarter(${V})-1)` : `${q}${te}(${V})`;
    }
    let J;
    const Z = {};
    for (const te of TIMEUNIT_PARTS)
      containsTimeUnit(O, te) && (Z[te] = G(te), J = te);
    return H && (Z[J] += "+1"), dateTimeExprToExpr(Z);
  }
  function timeUnitSpecifierExpression(O) {
    if (!O)
      return;
    const _ = getTimeUnitParts(O);
    return `timeUnitSpecifier(${stringify$1(_)}, ${stringify$1(VEGALITE_TIMEFORMAT)})`;
  }
  function formatExpression(O, _, H) {
    if (!O)
      return;
    const V = timeUnitSpecifierExpression(O);
    return `${H || isUTCTimeUnit(O) ? "utc" : "time"}Format(${_}, ${V})`;
  }
  function normalizeTimeUnit(O) {
    if (!O)
      return;
    let _;
    return isString$1(O) ? isBinnedTimeUnitString(O) ? _ = {
      unit: O.substring(6),
      binned: !0
    } : _ = {
      unit: O
    } : isObject$2(O) && (_ = {
      ...O,
      ...O.unit ? { unit: O.unit } : {}
    }), isUTCTimeUnit(_.unit) && (_.utc = !0, _.unit = getLocalTimeUnitFromUTCTimeUnit(_.unit)), _;
  }
  function timeUnitToString(O) {
    const { utc: _, ...H } = normalizeTimeUnit(O);
    return H.unit ? (_ ? "utc" : "") + keys$2(H).map((V) => varName(`${V === "unit" ? "" : `_${V}_`}${H[V]}`)).join("") : (_ ? "utc" : "") + "timeunit" + keys$2(H).map((V) => varName(`_${V}_${H[V]}`)).join("");
  }
  function durationExpr(O, _ = (H) => H) {
    const H = normalizeTimeUnit(O), V = getSmallestTimeUnitPart(H.unit);
    if (V && V !== "day") {
      const q = {
        year: 2001,
        // pick a non-leap year
        month: 1,
        date: 1,
        hours: 0,
        minutes: 0,
        seconds: 0,
        milliseconds: 0
      }, { step: G, part: J } = getDateTimePartAndStep(V, H.step), Z = {
        ...q,
        [J]: +q[J] + G
      };
      return `${_(dateTimeToExpr(Z))} - ${_(dateTimeToExpr(q))}`;
    }
  }
  const DATE_PARTS = {
    year: 1,
    month: 1,
    date: 1,
    hours: 1,
    minutes: 1,
    seconds: 1,
    milliseconds: 1
  };
  function isDatePart(O) {
    return has$1(DATE_PARTS, O);
  }
  function getDateTimePartAndStep(O, _ = 1) {
    if (isDatePart(O))
      return { part: O, step: _ };
    switch (O) {
      case "day":
      case "dayofyear":
        return { part: "date", step: _ };
      case "quarter":
        return { part: "month", step: _ * 3 };
      case "week":
        return { part: "date", step: _ * 7 };
    }
  }
  function isSelectionPredicate(O) {
    return hasProperty(O, "param");
  }
  function isFieldEqualPredicate(O) {
    return !!(O != null && O.field) && O.equal !== void 0;
  }
  function isFieldLTPredicate(O) {
    return !!(O != null && O.field) && O.lt !== void 0;
  }
  function isFieldLTEPredicate(O) {
    return !!(O != null && O.field) && O.lte !== void 0;
  }
  function isFieldGTPredicate(O) {
    return !!(O != null && O.field) && O.gt !== void 0;
  }
  function isFieldGTEPredicate(O) {
    return !!(O != null && O.field) && O.gte !== void 0;
  }
  function isFieldRangePredicate(O) {
    if (O != null && O.field) {
      if (isArray$2(O.range) && O.range.length === 2)
        return !0;
      if (isSignalRef(O.range))
        return !0;
    }
    return !1;
  }
  function isFieldOneOfPredicate(O) {
    return !!(O != null && O.field) && (isArray$2(O.oneOf) || isArray$2(O.in));
  }
  function isFieldValidPredicate(O) {
    return !!(O != null && O.field) && O.valid !== void 0;
  }
  function isFieldPredicate(O) {
    return isFieldOneOfPredicate(O) || isFieldEqualPredicate(O) || isFieldRangePredicate(O) || isFieldLTPredicate(O) || isFieldGTPredicate(O) || isFieldLTEPredicate(O) || isFieldGTEPredicate(O);
  }
  function predicateValueExpr(O, _) {
    return valueExpr(O, { timeUnit: _, wrapTime: !0 });
  }
  function predicateValuesExpr(O, _) {
    return O.map((H) => predicateValueExpr(H, _));
  }
  function fieldFilterExpression(O, _ = !0) {
    const { field: H } = O, V = normalizeTimeUnit(O.timeUnit), { unit: q, binned: G } = V || {}, J = vgField(O, { expr: "datum" }), Z = q ? (
      // For timeUnit, cast into integer with time() so we can use ===, inrange, indexOf to compare values directly.
      // TODO: We calculate timeUnit on the fly here. Consider if we would like to consolidate this with timeUnit pipeline
      // TODO: support utc
      `time(${G ? J : fieldExpr(q, H)})`
    ) : J;
    if (isFieldEqualPredicate(O))
      return `${Z}===${predicateValueExpr(O.equal, q)}`;
    if (isFieldLTPredicate(O)) {
      const te = O.lt;
      return `${Z}<${predicateValueExpr(te, q)}`;
    } else if (isFieldGTPredicate(O)) {
      const te = O.gt;
      return `${Z}>${predicateValueExpr(te, q)}`;
    } else if (isFieldLTEPredicate(O)) {
      const te = O.lte;
      return `${Z}<=${predicateValueExpr(te, q)}`;
    } else if (isFieldGTEPredicate(O)) {
      const te = O.gte;
      return `${Z}>=${predicateValueExpr(te, q)}`;
    } else {
      if (isFieldOneOfPredicate(O))
        return `indexof([${predicateValuesExpr(O.oneOf, q).join(",")}], ${Z}) !== -1`;
      if (isFieldValidPredicate(O))
        return fieldValidPredicate(Z, O.valid);
      if (isFieldRangePredicate(O)) {
        const { range: te } = replaceExprRef(O), ne = isSignalRef(te) ? { signal: `${te.signal}[0]` } : te[0], ee = isSignalRef(te) ? { signal: `${te.signal}[1]` } : te[1];
        if (ne !== null && ee !== null && _)
          return "inrange(" + Z + ", [" + predicateValueExpr(ne, q) + ", " + predicateValueExpr(ee, q) + "])";
        const ae = [];
        return ne !== null && ae.push(`${Z} >= ${predicateValueExpr(ne, q)}`), ee !== null && ae.push(`${Z} <= ${predicateValueExpr(ee, q)}`), ae.length > 0 ? ae.join(" && ") : "true";
      }
    }
    throw new Error(`Invalid field predicate: ${stringify$1(O)}`);
  }
  function fieldValidPredicate(O, _ = !0) {
    return _ ? `isValid(${O}) && isFinite(+${O})` : `!isValid(${O}) || !isFinite(+${O})`;
  }
  function normalizePredicate$1(O) {
    return isFieldPredicate(O) && O.timeUnit ? {
      ...O,
      timeUnit: normalizeTimeUnit(O.timeUnit)
    } : O;
  }
  const Type$3 = {
    quantitative: "quantitative",
    ordinal: "ordinal",
    temporal: "temporal",
    nominal: "nominal",
    geojson: "geojson"
  };
  function isContinuous(O) {
    return O === "quantitative" || O === "temporal";
  }
  function isDiscrete$1(O) {
    return O === "ordinal" || O === "nominal";
  }
  const QUANTITATIVE = Type$3.quantitative, ORDINAL = Type$3.ordinal, TEMPORAL = Type$3.temporal, NOMINAL = Type$3.nominal, GEOJSON = Type$3.geojson;
  function getFullName(O) {
    if (O)
      switch (O = O.toLowerCase(), O) {
        case "q":
        case QUANTITATIVE:
          return "quantitative";
        case "t":
        case TEMPORAL:
          return "temporal";
        case "o":
        case ORDINAL:
          return "ordinal";
        case "n":
        case NOMINAL:
          return "nominal";
        case GEOJSON:
          return "geojson";
      }
  }
  const ScaleType = {
    // Continuous - Quantitative
    LINEAR: "linear",
    LOG: "log",
    POW: "pow",
    SQRT: "sqrt",
    SYMLOG: "symlog",
    IDENTITY: "identity",
    SEQUENTIAL: "sequential",
    // Continuous - Time
    TIME: "time",
    UTC: "utc",
    // Discretizing scales
    QUANTILE: "quantile",
    QUANTIZE: "quantize",
    THRESHOLD: "threshold",
    BIN_ORDINAL: "bin-ordinal",
    // Discrete scales
    ORDINAL: "ordinal",
    POINT: "point",
    BAND: "band"
  }, SCALE_CATEGORY_INDEX = {
    linear: "numeric",
    log: "numeric",
    pow: "numeric",
    sqrt: "numeric",
    symlog: "numeric",
    identity: "numeric",
    sequential: "numeric",
    time: "time",
    utc: "time",
    ordinal: "ordinal",
    "bin-ordinal": "bin-ordinal",
    // TODO: should bin-ordinal support merging with other
    point: "ordinal-position",
    band: "ordinal-position",
    quantile: "discretizing",
    quantize: "discretizing",
    threshold: "discretizing"
  };
  function scaleCompatible(O, _) {
    const H = SCALE_CATEGORY_INDEX[O], V = SCALE_CATEGORY_INDEX[_];
    return H === V || H === "ordinal-position" && V === "time" || V === "ordinal-position" && H === "time";
  }
  const SCALE_PRECEDENCE_INDEX = {
    // numeric
    linear: 0,
    log: 1,
    pow: 1,
    sqrt: 1,
    symlog: 1,
    identity: 1,
    sequential: 1,
    // time
    time: 0,
    utc: 0,
    // ordinal-position -- these have higher precedence than continuous scales as they support more types of data
    point: 10,
    band: 11,
    // band has higher precedence as it is better for interaction
    // non grouped types
    ordinal: 0,
    "bin-ordinal": 0,
    quantile: 0,
    quantize: 0,
    threshold: 0
  };
  function scaleTypePrecedence(O) {
    return SCALE_PRECEDENCE_INDEX[O];
  }
  const QUANTITATIVE_SCALES = /* @__PURE__ */ new Set([
    "linear",
    "log",
    "pow",
    "sqrt",
    "symlog"
  ]), CONTINUOUS_TO_CONTINUOUS_SCALES = /* @__PURE__ */ new Set([
    ...QUANTITATIVE_SCALES,
    "time",
    "utc"
  ]);
  function isQuantitative(O) {
    return QUANTITATIVE_SCALES.has(O);
  }
  const CONTINUOUS_TO_DISCRETE_SCALES = /* @__PURE__ */ new Set([
    "quantile",
    "quantize",
    "threshold"
  ]), CONTINUOUS_DOMAIN_SCALES = /* @__PURE__ */ new Set([
    ...CONTINUOUS_TO_CONTINUOUS_SCALES,
    ...CONTINUOUS_TO_DISCRETE_SCALES,
    "sequential",
    "identity"
  ]), DISCRETE_DOMAIN_SCALES = /* @__PURE__ */ new Set([
    "ordinal",
    "bin-ordinal",
    "point",
    "band"
  ]);
  function hasDiscreteDomain(O) {
    return DISCRETE_DOMAIN_SCALES.has(O);
  }
  function hasContinuousDomain(O) {
    return CONTINUOUS_DOMAIN_SCALES.has(O);
  }
  function isContinuousToContinuous(O) {
    return CONTINUOUS_TO_CONTINUOUS_SCALES.has(O);
  }
  function isContinuousToDiscrete(O) {
    return CONTINUOUS_TO_DISCRETE_SCALES.has(O);
  }
  const defaultScaleConfig = {
    pointPadding: 0.5,
    barBandPaddingInner: 0.1,
    rectBandPaddingInner: 0,
    tickBandPaddingInner: 0.25,
    bandWithNestedOffsetPaddingInner: 0.2,
    bandWithNestedOffsetPaddingOuter: 0.2,
    minBandSize: 2,
    minFontSize: 8,
    maxFontSize: 40,
    minOpacity: 0.3,
    maxOpacity: 0.8,
    // FIXME: revise if these *can* become ratios of width/height step
    minSize: 4,
    // Point size is area. For square point, 9 = 3 pixel ^ 2, not too small!
    minStrokeWidth: 1,
    maxStrokeWidth: 4,
    quantileCount: 4,
    quantizeCount: 4,
    zero: !0,
    framesPerSecond: 2,
    animationDuration: 5
  };
  function isExtendedScheme(O) {
    return !isString$1(O) && hasProperty(O, "name");
  }
  function isParameterDomain(O) {
    return hasProperty(O, "param");
  }
  function isDomainUnionWith(O) {
    return hasProperty(O, "unionWith");
  }
  function isFieldRange(O) {
    return isObject$2(O) && "field" in O;
  }
  const SCALE_PROPERTY_INDEX = {
    type: 1,
    domain: 1,
    domainMax: 1,
    domainMin: 1,
    domainMid: 1,
    domainRaw: 1,
    align: 1,
    range: 1,
    rangeMax: 1,
    rangeMin: 1,
    scheme: 1,
    bins: 1,
    // Other properties
    reverse: 1,
    round: 1,
    // quantitative / time
    clamp: 1,
    nice: 1,
    // quantitative
    base: 1,
    exponent: 1,
    constant: 1,
    interpolate: 1,
    zero: 1,
    // zero depends on domain
    // band/point
    padding: 1,
    paddingInner: 1,
    paddingOuter: 1
  }, { type, domain: domain$1, range: range$3, rangeMax, rangeMin, scheme, ...NON_TYPE_DOMAIN_RANGE_VEGA_SCALE_PROPERTY_INDEX } = SCALE_PROPERTY_INDEX, NON_TYPE_DOMAIN_RANGE_VEGA_SCALE_PROPERTIES = keys$2(NON_TYPE_DOMAIN_RANGE_VEGA_SCALE_PROPERTY_INDEX);
  function scaleTypeSupportProperty(O, _) {
    switch (_) {
      case "type":
      case "domain":
      case "reverse":
      case "range":
        return !0;
      case "scheme":
      case "interpolate":
        return !["point", "band", "identity"].includes(O);
      case "bins":
        return !["point", "band", "identity", "ordinal"].includes(O);
      case "round":
        return isContinuousToContinuous(O) || O === "band" || O === "point";
      case "padding":
      case "rangeMin":
      case "rangeMax":
        return isContinuousToContinuous(O) || ["point", "band"].includes(O);
      case "paddingOuter":
      case "align":
        return ["point", "band"].includes(O);
      case "paddingInner":
        return O === "band";
      case "domainMax":
      case "domainMid":
      case "domainMin":
      case "domainRaw":
      case "clamp":
        return isContinuousToContinuous(O);
      case "nice":
        return isContinuousToContinuous(O) || O === "quantize" || O === "threshold";
      case "exponent":
        return O === "pow";
      case "base":
        return O === "log";
      case "constant":
        return O === "symlog";
      case "zero":
        return hasContinuousDomain(O) && !contains$2([
          "log",
          // log scale cannot have zero value
          "time",
          "utc",
          // zero is not meaningful for time
          "threshold",
          // threshold requires custom domain so zero does not matter
          "quantile"
          // quantile depends on distribution so zero does not matter
        ], O);
    }
  }
  function channelScalePropertyIncompatability(O, _) {
    switch (_) {
      case "interpolate":
      case "scheme":
      case "domainMid":
        return isColorChannel(O) ? void 0 : cannotUseScalePropertyWithNonColor(_);
      case "align":
      case "type":
      case "bins":
      case "domain":
      case "domainMax":
      case "domainMin":
      case "domainRaw":
      case "range":
      case "base":
      case "exponent":
      case "constant":
      case "nice":
      case "padding":
      case "paddingInner":
      case "paddingOuter":
      case "rangeMax":
      case "rangeMin":
      case "reverse":
      case "round":
      case "clamp":
      case "zero":
        return;
    }
  }
  function scaleTypeSupportDataType(O, _) {
    return contains$2([ORDINAL, NOMINAL], _) ? O === void 0 || hasDiscreteDomain(O) : _ === TEMPORAL ? contains$2([ScaleType.TIME, ScaleType.UTC, void 0], O) : _ === QUANTITATIVE ? isQuantitative(O) || isContinuousToDiscrete(O) || O === void 0 : !0;
  }
  function channelSupportScaleType(O, _, H = !1) {
    if (!isScaleChannel(O))
      return !1;
    switch (O) {
      case X:
      case Y$1:
      case XOFFSET:
      case YOFFSET:
      case THETA:
      case RADIUS:
        return isContinuousToContinuous(_) || _ === "band" ? !0 : _ === "point" ? !H : !1;
      case TIME:
        return contains$2(["linear", "band"], _);
      case SIZE:
      // TODO: size and opacity can support ordinal with more modification
      case STROKEWIDTH:
      case OPACITY:
      case FILLOPACITY:
      case STROKEOPACITY:
      case ANGLE:
        return isContinuousToContinuous(_) || isContinuousToDiscrete(_) || contains$2(["band", "point", "ordinal"], _);
      case COLOR:
      case FILL:
      case STROKE:
        return _ !== "band";
      // band does not make sense with color
      case STROKEDASH:
      case SHAPE:
        return _ === "ordinal" || isContinuousToDiscrete(_);
    }
  }
  function isScaleInvalidDataIncludeAsValue(O) {
    return isObject$2(O) && "value" in O;
  }
  const Mark = {
    arc: "arc",
    area: "area",
    bar: "bar",
    image: "image",
    line: "line",
    point: "point",
    rect: "rect",
    rule: "rule",
    text: "text",
    tick: "tick",
    trail: "trail",
    circle: "circle",
    square: "square",
    geoshape: "geoshape"
  }, ARC = Mark.arc, AREA = Mark.area, BAR = Mark.bar, IMAGE = Mark.image, LINE = Mark.line, POINT = Mark.point, RECT = Mark.rect, RULE = Mark.rule, TEXT = Mark.text, TICK = Mark.tick, TRAIL = Mark.trail, CIRCLE = Mark.circle, SQUARE = Mark.square, GEOSHAPE = Mark.geoshape;
  function isPathMark(O) {
    return ["line", "area", "trail"].includes(O);
  }
  function isRectBasedMark(O) {
    return [
      "rect",
      "bar",
      "image",
      "arc",
      "tick"
      /* arc is rect/interval in polar coordinate */
    ].includes(O);
  }
  const PRIMITIVE_MARKS = new Set(keys$2(Mark));
  function isMarkDef(O) {
    return hasProperty(O, "type");
  }
  const STROKE_CONFIG = [
    "stroke",
    "strokeWidth",
    "strokeDash",
    "strokeDashOffset",
    "strokeOpacity",
    "strokeJoin",
    "strokeMiterLimit"
  ], FILL_CONFIG = ["fill", "fillOpacity"], FILL_STROKE_CONFIG = [...STROKE_CONFIG, ...FILL_CONFIG], VL_ONLY_MARK_CONFIG_INDEX = {
    color: 1,
    filled: 1,
    invalid: 1,
    order: 1,
    radius2: 1,
    theta2: 1,
    timeUnitBandSize: 1,
    timeUnitBandPosition: 1
  }, VL_ONLY_MARK_CONFIG_PROPERTIES = keys$2(VL_ONLY_MARK_CONFIG_INDEX), VL_ONLY_RECT_CONFIG = [
    "binSpacing",
    "continuousBandSize",
    "discreteBandSize",
    "minBandSize"
  ], VL_ONLY_MARK_SPECIFIC_CONFIG_PROPERTY_INDEX = {
    area: ["line", "point"],
    bar: VL_ONLY_RECT_CONFIG,
    rect: VL_ONLY_RECT_CONFIG,
    line: ["point"],
    tick: ["bandSize", "thickness", ...VL_ONLY_RECT_CONFIG]
  }, defaultMarkConfig = {
    color: "#4c78a8",
    invalid: "break-paths-show-path-domains",
    timeUnitBandSize: 1
  }, MARK_CONFIG_INDEX = {
    mark: 1,
    arc: 1,
    area: 1,
    bar: 1,
    circle: 1,
    image: 1,
    line: 1,
    point: 1,
    rect: 1,
    rule: 1,
    square: 1,
    text: 1,
    tick: 1,
    trail: 1,
    geoshape: 1
  }, MARK_CONFIGS = keys$2(MARK_CONFIG_INDEX);
  function isRelativeBandSize(O) {
    return hasProperty(O, "band");
  }
  const BAR_CORNER_RADIUS_INDEX = {
    horizontal: ["cornerRadiusTopRight", "cornerRadiusBottomRight"],
    vertical: ["cornerRadiusTopLeft", "cornerRadiusTopRight"]
  }, DEFAULT_RECT_BAND_SIZE = 5, defaultRectConfig = {
    binSpacing: 0,
    continuousBandSize: DEFAULT_RECT_BAND_SIZE,
    minBandSize: 0.25,
    timeUnitBandPosition: 0.5
  }, defaultBarConfig = {
    ...defaultRectConfig,
    binSpacing: 1
  }, defaultTickConfig = {
    ...defaultRectConfig,
    thickness: 1
  };
  function getMarkType(O) {
    return isMarkDef(O) ? O.type : O;
  }
  function normalizeInvalidDataMode(O, { isPath: _ }) {
    return O === void 0 || O === "break-paths-show-path-domains" ? _ ? "break-paths-show-domains" : "filter" : O === null ? "show" : O;
  }
  function getScaleInvalidDataMode({ markDef: O, config: _, scaleChannel: H, scaleType: V, isCountAggregate: q }) {
    var Z, te;
    if (!V || !hasContinuousDomain(V) || q)
      return "always-valid";
    const G = normalizeInvalidDataMode(getMarkPropOrConfig("invalid", O, _), {
      isPath: isPathMark(O.type)
    });
    return ((te = (Z = _.scale) == null ? void 0 : Z.invalid) == null ? void 0 : te[H]) !== void 0 ? "show" : G;
  }
  function shouldBreakPath(O) {
    return O === "break-paths-filter-domains" || O === "break-paths-show-domains";
  }
  function scaledZeroOrMinOrMax({ scaleName: O, scale: _, mode: H }) {
    const V = `domain('${O}')`;
    if (!_ || !O)
      return;
    const q = `${V}[0]`, G = `peek(${V})`, J = _.domainHasZero();
    return J === "definitely" ? {
      scale: O,
      value: 0
    } : J === "maybe" ? { signal: `scale('${O}', inrange(0, ${V}) ? 0 : ${H === "zeroOrMin" ? q : G})` } : { signal: `scale('${O}', ${H === "zeroOrMin" ? q : G})` };
  }
  function getConditionalValueRefForIncludingInvalidValue({ scaleChannel: O, channelDef: _, scale: H, scaleName: V, markDef: q, config: G }) {
    var ee;
    const J = H == null ? void 0 : H.get("type"), Z = getFieldDef(_), te = isCountingAggregateOp(Z == null ? void 0 : Z.aggregate), ne = getScaleInvalidDataMode({
      scaleChannel: O,
      markDef: q,
      config: G,
      scaleType: J,
      isCountAggregate: te
    });
    if (Z && ne === "show") {
      const ae = ((ee = G.scale.invalid) == null ? void 0 : ee[O]) ?? "zero-or-min";
      return {
        test: fieldValidPredicate(vgField(Z, { expr: "datum" }), !1),
        ...refForInvalidValues(ae, H, V)
      };
    }
  }
  function refForInvalidValues(O, _, H) {
    if (isScaleInvalidDataIncludeAsValue(O)) {
      const { value: V } = O;
      return isSignalRef(V) ? { signal: V.signal } : { value: V };
    }
    return scaledZeroOrMinOrMax({
      scale: _,
      scaleName: H,
      mode: "zeroOrMin"
    });
  }
  function midPointRefWithPositionInvalidTest(O) {
    const { channel: _, channelDef: H, markDef: V, scale: q, scaleName: G, config: J } = O, Z = getMainRangeChannel(_), te = midPoint(O), ne = getConditionalValueRefForIncludingInvalidValue({
      scaleChannel: Z,
      channelDef: H,
      scale: q,
      scaleName: G,
      markDef: V,
      config: J
    });
    return ne !== void 0 ? [ne, te] : te;
  }
  function datumDefToExpr(O) {
    const { datum: _ } = O;
    return isDateTime(_) ? dateTimeToExpr(_) : `${stringify$1(_)}`;
  }
  function valueRefForFieldOrDatumDef(O, _, H, V) {
    const q = {};
    if (_ && (q.scale = _), isDatumDef(O)) {
      const { datum: G } = O;
      isDateTime(G) ? q.signal = dateTimeToExpr(G) : isSignalRef(G) ? q.signal = G.signal : isExprRef(G) ? q.signal = G.expr : q.value = G;
    } else
      q.field = vgField(O, H);
    if (V) {
      const { offset: G, band: J } = V;
      G && (q.offset = G), J && (q.band = J);
    }
    return q;
  }
  function interpolatedSignalRef({ scaleName: O, fieldOrDatumDef: _, fieldOrDatumDef2: H, offset: V, startSuffix: q, endSuffix: G = "end", bandPosition: J = 0.5 }) {
    const Z = !isSignalRef(J) && 0 < J && J < 1 ? "datum" : void 0, te = vgField(_, { expr: Z, suffix: q }), ne = H !== void 0 ? vgField(H, { expr: Z }) : vgField(_, { suffix: G, expr: Z }), ee = {};
    if (J === 0 || J === 1) {
      ee.scale = O;
      const ae = J === 0 ? te : ne;
      ee.field = ae;
    } else {
      const ae = isSignalRef(J) ? `(1-${J.signal}) * ${te} + ${J.signal} * ${ne}` : `${1 - J} * ${te} + ${J} * ${ne}`;
      ee.signal = `scale("${O}", ${ae})`;
    }
    return V && (ee.offset = V), ee;
  }
  function binSizeExpr({ scaleName: O, fieldDef: _ }) {
    const H = vgField(_, { expr: "datum" }), V = vgField(_, { expr: "datum", suffix: "end" });
    return `abs(scale("${O}", ${V}) - scale("${O}", ${H}))`;
  }
  function midPoint({ channel: O, channelDef: _, channel2Def: H, markDef: V, config: q, scaleName: G, scale: J, stack: Z, offset: te, defaultRef: ne, bandPosition: ee }) {
    if (_) {
      if (isFieldOrDatumDef(_)) {
        const ae = J == null ? void 0 : J.get("type");
        if (isTypedFieldDef(_)) {
          ee ?? (ee = getBandPosition({
            fieldDef: _,
            fieldDef2: H,
            markDef: V,
            config: q
          }));
          const { bin: oe, timeUnit: se, type: ce } = _;
          if (isBinning(oe) || ee && se && ce === TEMPORAL)
            return Z != null && Z.impute ? valueRefForFieldOrDatumDef(_, G, { binSuffix: "mid" }, { offset: te }) : ee && !hasDiscreteDomain(ae) ? interpolatedSignalRef({ scaleName: G, fieldOrDatumDef: _, bandPosition: ee, offset: te }) : valueRefForFieldOrDatumDef(_, G, binRequiresRange(_, O) ? { binSuffix: "range" } : {}, {
              offset: te
            });
          if (isBinned(oe)) {
            if (isFieldDef(H))
              return interpolatedSignalRef({
                scaleName: G,
                fieldOrDatumDef: _,
                fieldOrDatumDef2: H,
                bandPosition: ee,
                offset: te
              });
            warn(channelRequiredForBinned(O === X ? X2 : Y2));
          }
        }
        return valueRefForFieldOrDatumDef(
          _,
          G,
          hasDiscreteDomain(ae) ? { binSuffix: "range" } : {},
          // no need for bin suffix if there is no scale
          {
            offset: te,
            // For band, to get mid point, need to offset by half of the band
            band: ae === "band" ? ee ?? _.bandPosition ?? 0.5 : void 0
          }
        );
      } else if (isValueDef(_)) {
        const ae = _.value, oe = te ? { offset: te } : {};
        return { ...widthHeightValueOrSignalRef(O, ae), ...oe };
      }
    }
    return isFunction$2(ne) && (ne = ne()), ne && {
      ...ne,
      // only include offset when it is non-zero (zero = no offset)
      ...te ? { offset: te } : {}
    };
  }
  function widthHeightValueOrSignalRef(O, _) {
    return contains$2(["x", "x2"], O) && _ === "width" ? { field: { group: "width" } } : contains$2(["y", "y2"], O) && _ === "height" ? { field: { group: "height" } } : signalOrValueRef(_);
  }
  function isCustomFormatType(O) {
    return O && O !== "number" && O !== "time";
  }
  function customFormatExpr(O, _, H) {
    return `${O}(${_}${H ? `, ${stringify$1(H)}` : ""})`;
  }
  const BIN_RANGE_DELIMITER = " – ";
  function formatSignalRef({ fieldOrDatumDef: O, format: _, formatType: H, expr: V, normalizeStack: q, config: G }) {
    var te, ne;
    if (isCustomFormatType(H))
      return formatCustomType({
        fieldOrDatumDef: O,
        format: _,
        formatType: H,
        expr: V,
        config: G
      });
    const J = fieldToFormat(O, V, q), Z = channelDefType(O);
    if (_ === void 0 && H === void 0 && G.customFormatTypes) {
      if (Z === "quantitative") {
        if (q && G.normalizedNumberFormatType)
          return formatCustomType({
            fieldOrDatumDef: O,
            format: G.normalizedNumberFormat,
            formatType: G.normalizedNumberFormatType,
            expr: V,
            config: G
          });
        if (G.numberFormatType)
          return formatCustomType({
            fieldOrDatumDef: O,
            format: G.numberFormat,
            formatType: G.numberFormatType,
            expr: V,
            config: G
          });
      }
      if (Z === "temporal" && G.timeFormatType && isFieldDef(O) && O.timeUnit === void 0)
        return formatCustomType({
          fieldOrDatumDef: O,
          format: G.timeFormat,
          formatType: G.timeFormatType,
          expr: V,
          config: G
        });
    }
    if (isFieldOrDatumDefForTimeFormat(O)) {
      const ee = timeFormatExpression({
        field: J,
        timeUnit: isFieldDef(O) ? (te = normalizeTimeUnit(O.timeUnit)) == null ? void 0 : te.unit : void 0,
        format: _,
        formatType: G.timeFormatType,
        rawTimeFormat: G.timeFormat,
        isUTCScale: isScaleFieldDef(O) && ((ne = O.scale) == null ? void 0 : ne.type) === ScaleType.UTC
      });
      return ee ? { signal: ee } : void 0;
    }
    if (_ = numberFormat({ type: Z, specifiedFormat: _, config: G, normalizeStack: q }), isFieldDef(O) && isBinning(O.bin)) {
      const ee = vgField(O, { expr: V, binSuffix: "end" });
      return {
        signal: binFormatExpression(J, ee, _, H, G)
      };
    } else return _ || channelDefType(O) === "quantitative" ? {
      signal: `${formatExpr(J, _)}`
    } : { signal: `isValid(${J}) ? ${J} : ""+${J}` };
  }
  function fieldToFormat(O, _, H) {
    return isFieldDef(O) ? H ? `${vgField(O, { expr: _, suffix: "end" })}-${vgField(O, {
      expr: _,
      suffix: "start"
    })}` : vgField(O, { expr: _ }) : datumDefToExpr(O);
  }
  function formatCustomType({ fieldOrDatumDef: O, format: _, formatType: H, expr: V, normalizeStack: q, config: G, field: J }) {
    if (J ?? (J = fieldToFormat(O, V, q)), J !== "datum.value" && // For axis/legend, we can't correctly know the end of the bin from `datum`
    isFieldDef(O) && isBinning(O.bin)) {
      const Z = vgField(O, { expr: V, binSuffix: "end" });
      return {
        signal: binFormatExpression(J, Z, _, H, G)
      };
    }
    return { signal: customFormatExpr(H, J, _) };
  }
  function guideFormat(O, _, H, V, q, G) {
    var J;
    if (!(isString$1(V) && isCustomFormatType(V)) && !(H === void 0 && V === void 0 && q.customFormatTypes && channelDefType(O) === "quantitative" && (q.normalizedNumberFormatType && isPositionFieldOrDatumDef(O) && O.stack === "normalize" || q.numberFormatType))) {
      if (isPositionFieldOrDatumDef(O) && O.stack === "normalize" && q.normalizedNumberFormat)
        return numberFormat({
          type: "quantitative",
          config: q,
          normalizeStack: !0
        });
      if (isFieldOrDatumDefForTimeFormat(O)) {
        const Z = isFieldDef(O) ? (J = normalizeTimeUnit(O.timeUnit)) == null ? void 0 : J.unit : void 0;
        return Z === void 0 && q.customFormatTypes && q.timeFormatType ? void 0 : timeFormat({ specifiedFormat: H, timeUnit: Z, config: q, omitTimeFormatConfig: G });
      }
      return numberFormat({ type: _, specifiedFormat: H, config: q });
    }
  }
  function guideFormatType(O, _, H) {
    var V;
    if (O && (isSignalRef(O) || O === "number" || O === "time"))
      return O;
    if (isFieldOrDatumDefForTimeFormat(_) && H !== "time" && H !== "utc")
      return isFieldDef(_) && ((V = normalizeTimeUnit(_ == null ? void 0 : _.timeUnit)) != null && V.utc) ? "utc" : "time";
  }
  function numberFormat({ type: O, specifiedFormat: _, config: H, normalizeStack: V }) {
    if (isString$1(_))
      return _;
    if (O === QUANTITATIVE)
      return V ? H.normalizedNumberFormat : H.numberFormat;
  }
  function timeFormat({ specifiedFormat: O, timeUnit: _, config: H, omitTimeFormatConfig: V }) {
    return O || (_ ? {
      signal: timeUnitSpecifierExpression(_)
    } : V ? void 0 : H.timeFormat);
  }
  function formatExpr(O, _) {
    return `format(${O}, "${_ || ""}")`;
  }
  function binNumberFormatExpr(O, _, H, V) {
    return isCustomFormatType(H) ? customFormatExpr(H, O, _) : formatExpr(O, (isString$1(_) ? _ : void 0) ?? V.numberFormat);
  }
  function binFormatExpression(O, _, H, V, q) {
    if (H === void 0 && V === void 0 && q.customFormatTypes && q.numberFormatType)
      return binFormatExpression(O, _, q.numberFormat, q.numberFormatType, q);
    const G = binNumberFormatExpr(O, H, V, q), J = binNumberFormatExpr(_, H, V, q);
    return `${fieldValidPredicate(O, !1)} ? "null" : ${G} + "${BIN_RANGE_DELIMITER}" + ${J}`;
  }
  function timeFormatExpression({ field: O, timeUnit: _, format: H, formatType: V, rawTimeFormat: q, isUTCScale: G }) {
    return !_ || H ? !_ && V ? `${V}(${O}, '${H}')` : (H = isString$1(H) ? H : q, `${G ? "utc" : "time"}Format(${O}, '${H}')`) : formatExpression(_, O, G);
  }
  const DEFAULT_SORT_OP = "min", SORT_BY_CHANNEL_INDEX = {
    x: 1,
    y: 1,
    color: 1,
    fill: 1,
    stroke: 1,
    strokeWidth: 1,
    size: 1,
    shape: 1,
    fillOpacity: 1,
    strokeOpacity: 1,
    opacity: 1,
    text: 1
  };
  function isSortByChannel(O) {
    return has$1(SORT_BY_CHANNEL_INDEX, O);
  }
  function isSortByEncoding(O) {
    return hasProperty(O, "encoding");
  }
  function isSortField(O) {
    return O && (O.op === "count" || hasProperty(O, "field"));
  }
  function isSortArray(O) {
    return O && isArray$2(O);
  }
  function isFacetMapping(O) {
    return hasProperty(O, "row") || hasProperty(O, "column");
  }
  function isFacetFieldDef(O) {
    return hasProperty(O, "header");
  }
  function isFacetSpec(O) {
    return hasProperty(O, "facet");
  }
  function isConditionalParameter(O) {
    return hasProperty(O, "param");
  }
  function isRepeatRef(O) {
    return !isString$1(O) && hasProperty(O, "repeat");
  }
  function toFieldDefBase(O) {
    const { field: _, timeUnit: H, bin: V, aggregate: q } = O;
    return {
      ...H ? { timeUnit: H } : {},
      ...V ? { bin: V } : {},
      ...q ? { aggregate: q } : {},
      field: _
    };
  }
  function isSortableFieldDef(O) {
    return hasProperty(O, "sort");
  }
  function getBandPosition({ fieldDef: O, fieldDef2: _, markDef: H, config: V }) {
    if (isFieldOrDatumDef(O) && O.bandPosition !== void 0)
      return O.bandPosition;
    if (isFieldDef(O)) {
      const { timeUnit: q, bin: G } = O;
      if (q && !_)
        return getMarkConfig("timeUnitBandPosition", H, V);
      if (isBinning(G))
        return 0.5;
    }
  }
  function getBandSize({ channel: O, fieldDef: _, fieldDef2: H, markDef: V, config: q, scaleType: G, useVlSizeChannel: J }) {
    var ne, ee, ae;
    const Z = getSizeChannel(O), te = getMarkPropOrConfig(J ? "size" : Z, V, q, {
      vgChannel: Z
    });
    if (te !== void 0)
      return te;
    if (isFieldDef(_)) {
      const { timeUnit: oe, bin: se } = _;
      if (oe && !H)
        return { band: getMarkConfig("timeUnitBandSize", V, q) };
      if (isBinning(se) && !hasDiscreteDomain(G))
        return { band: 1 };
    }
    if (isRectBasedMark(V.type))
      return G ? hasDiscreteDomain(G) ? ((ne = q[V.type]) == null ? void 0 : ne.discreteBandSize) || { band: 1 } : (ee = q[V.type]) == null ? void 0 : ee.continuousBandSize : (ae = q[V.type]) == null ? void 0 : ae.discreteBandSize;
  }
  function hasBandEnd(O, _, H, V) {
    return isBinning(O.bin) || O.timeUnit && isTypedFieldDef(O) && O.type === "temporal" ? getBandPosition({ fieldDef: O, fieldDef2: _, markDef: H, config: V }) !== void 0 : !1;
  }
  function isOrderOnlyDef(O) {
    return hasProperty(O, "sort") && !hasProperty(O, "field");
  }
  function isConditionalDef(O) {
    return hasProperty(O, "condition");
  }
  function hasConditionalFieldDef(O) {
    const _ = O == null ? void 0 : O.condition;
    return !!_ && !isArray$2(_) && isFieldDef(_);
  }
  function hasConditionalFieldOrDatumDef(O) {
    const _ = O == null ? void 0 : O.condition;
    return !!_ && !isArray$2(_) && isFieldOrDatumDef(_);
  }
  function hasConditionalValueDef(O) {
    const _ = O == null ? void 0 : O.condition;
    return !!_ && (isArray$2(_) || isValueDef(_));
  }
  function isFieldDef(O) {
    return hasProperty(O, "field") || (O == null ? void 0 : O.aggregate) === "count";
  }
  function channelDefType(O) {
    return O == null ? void 0 : O.type;
  }
  function isDatumDef(O) {
    return hasProperty(O, "datum");
  }
  function isContinuousFieldOrDatumDef(O) {
    return isTypedFieldDef(O) && !isDiscrete(O) || isNumericDataDef(O);
  }
  function isUnbinnedQuantitativeFieldOrDatumDef(O) {
    return isTypedFieldDef(O) && O.type === "quantitative" && !O.bin || isNumericDataDef(O);
  }
  function isNumericDataDef(O) {
    return isDatumDef(O) && isNumber$2(O.datum);
  }
  function isFieldOrDatumDef(O) {
    return isFieldDef(O) || isDatumDef(O);
  }
  function isTypedFieldDef(O) {
    return O && (hasProperty(O, "field") || O.aggregate === "count") && hasProperty(O, "type");
  }
  function isValueDef(O) {
    return hasProperty(O, "value");
  }
  function isScaleFieldDef(O) {
    return hasProperty(O, "scale") || hasProperty(O, "sort");
  }
  function isPositionFieldOrDatumDef(O) {
    return hasProperty(O, "axis") || hasProperty(O, "stack") || hasProperty(O, "impute");
  }
  function isMarkPropFieldOrDatumDef(O) {
    return hasProperty(O, "legend");
  }
  function isStringFieldOrDatumDef(O) {
    return hasProperty(O, "format") || hasProperty(O, "formatType");
  }
  function toStringFieldDef(O) {
    return omit(O, ["legend", "axis", "header", "scale"]);
  }
  function isOpFieldDef(O) {
    return hasProperty(O, "op");
  }
  function vgField(O, _ = {}) {
    let H = O.field;
    const V = _.prefix;
    let q = _.suffix, G = "";
    if (isCount(O))
      H = internalField("count");
    else {
      let J;
      if (!_.nofn)
        if (isOpFieldDef(O))
          J = O.op;
        else {
          const { bin: Z, aggregate: te, timeUnit: ne } = O;
          isBinning(Z) ? (J = binToString(Z), q = (_.binSuffix ?? "") + (_.suffix ?? "")) : te ? isArgmaxDef(te) ? (G = `["${H}"]`, H = `argmax_${te.argmax}`) : isArgminDef(te) ? (G = `["${H}"]`, H = `argmin_${te.argmin}`) : J = String(te) : ne && !isBinnedTimeUnit(ne) && (J = timeUnitToString(ne), q = (!["range", "mid"].includes(_.binSuffix) && _.binSuffix || "") + (_.suffix ?? ""));
        }
      J && (H = H ? `${J}_${H}` : J);
    }
    return q && (H = `${H}_${q}`), V && (H = `${V}_${H}`), _.forAs ? removePathFromField(H) : _.expr ? flatAccessWithDatum(H, _.expr) + G : replacePathInField(H) + G;
  }
  function isDiscrete(O) {
    switch (O.type) {
      case "nominal":
      case "ordinal":
      case "geojson":
        return !0;
      case "quantitative":
        return isFieldDef(O) && !!O.bin;
      case "temporal":
        return !1;
    }
    throw new Error(invalidFieldType(O.type));
  }
  function isDiscretizing(O) {
    var _;
    return isScaleFieldDef(O) && isContinuousToDiscrete((_ = O.scale) == null ? void 0 : _.type);
  }
  function isCount(O) {
    return O.aggregate === "count";
  }
  function verbalTitleFormatter(O, _) {
    var J;
    const { field: H, bin: V, timeUnit: q, aggregate: G } = O;
    if (G === "count")
      return _.countTitle;
    if (isBinning(V))
      return `${H} (binned)`;
    if (q && !isBinnedTimeUnit(q)) {
      const Z = (J = normalizeTimeUnit(q)) == null ? void 0 : J.unit;
      if (Z)
        return `${H} (${getTimeUnitParts(Z).join("-")})`;
    } else if (G)
      return isArgmaxDef(G) ? `${H} for max ${G.argmax}` : isArgminDef(G) ? `${H} for min ${G.argmin}` : `${titleCase(G)} of ${H}`;
    return H;
  }
  function functionalTitleFormatter(O) {
    const { aggregate: _, bin: H, timeUnit: V, field: q } = O;
    if (isArgmaxDef(_))
      return `${q} for argmax(${_.argmax})`;
    if (isArgminDef(_))
      return `${q} for argmin(${_.argmin})`;
    const G = V && !isBinnedTimeUnit(V) ? normalizeTimeUnit(V) : void 0, J = _ || (G == null ? void 0 : G.unit) || (G == null ? void 0 : G.maxbins) && "timeunit" || isBinning(H) && "bin";
    return J ? `${J.toUpperCase()}(${q})` : q;
  }
  const defaultTitleFormatter = (O, _) => {
    switch (_.fieldTitle) {
      case "plain":
        return O.field;
      case "functional":
        return functionalTitleFormatter(O);
      default:
        return verbalTitleFormatter(O, _);
    }
  };
  let titleFormatter = defaultTitleFormatter;
  function setTitleFormatter(O) {
    titleFormatter = O;
  }
  function resetTitleFormatter() {
    setTitleFormatter(defaultTitleFormatter);
  }
  function title(O, _, { allowDisabling: H, includeDefault: V = !0 }) {
    var Z;
    const q = (Z = getGuide(O)) == null ? void 0 : Z.title;
    if (!isFieldDef(O))
      return q ?? O.title;
    const G = O, J = V ? defaultTitle(G, _) : void 0;
    return H ? getFirstDefined(q, G.title, J) : q ?? G.title ?? J;
  }
  function getGuide(O) {
    if (isPositionFieldOrDatumDef(O) && O.axis)
      return O.axis;
    if (isMarkPropFieldOrDatumDef(O) && O.legend)
      return O.legend;
    if (isFacetFieldDef(O) && O.header)
      return O.header;
  }
  function defaultTitle(O, _) {
    return titleFormatter(O, _);
  }
  function getFormatMixins(O) {
    if (isStringFieldOrDatumDef(O)) {
      const { format: _, formatType: H } = O;
      return { format: _, formatType: H };
    } else {
      const _ = getGuide(O) ?? {}, { format: H, formatType: V } = _;
      return { format: H, formatType: V };
    }
  }
  function defaultType$3(O, _) {
    var G;
    switch (_) {
      case "latitude":
      case "longitude":
        return "quantitative";
      case "row":
      case "column":
      case "facet":
      case "shape":
      case "strokeDash":
        return "nominal";
      case "order":
        return "ordinal";
    }
    if (isSortableFieldDef(O) && isArray$2(O.sort))
      return "ordinal";
    const { aggregate: H, bin: V, timeUnit: q } = O;
    if (q)
      return "temporal";
    if (V || H && !isArgmaxDef(H) && !isArgminDef(H))
      return "quantitative";
    if (isScaleFieldDef(O) && ((G = O.scale) != null && G.type))
      switch (SCALE_CATEGORY_INDEX[O.scale.type]) {
        case "numeric":
        case "discretizing":
          return "quantitative";
        case "time":
          return "temporal";
      }
    return "nominal";
  }
  function getFieldDef(O) {
    if (isFieldDef(O))
      return O;
    if (hasConditionalFieldDef(O))
      return O.condition;
  }
  function getFieldOrDatumDef(O) {
    if (isFieldOrDatumDef(O))
      return O;
    if (hasConditionalFieldOrDatumDef(O))
      return O.condition;
  }
  function initChannelDef(O, _, H, V = {}) {
    if (isString$1(O) || isNumber$2(O) || isBoolean$3(O)) {
      const q = isString$1(O) ? "string" : isNumber$2(O) ? "number" : "boolean";
      return warn(primitiveChannelDef(_, q, O)), { value: O };
    }
    return isFieldOrDatumDef(O) ? initFieldOrDatumDef(O, _, H, V) : hasConditionalFieldOrDatumDef(O) ? {
      ...O,
      // Need to cast as normalizeFieldDef normally return FieldDef, but here we know that it is definitely Condition<FieldDef>
      condition: initFieldOrDatumDef(O.condition, _, H, V)
    } : O;
  }
  function initFieldOrDatumDef(O, _, H, V) {
    if (isStringFieldOrDatumDef(O)) {
      const { format: q, formatType: G, ...J } = O;
      if (isCustomFormatType(G) && !H.customFormatTypes)
        return warn(customFormatTypeNotAllowed(_)), initFieldOrDatumDef(J, _, H, V);
    } else {
      const q = isPositionFieldOrDatumDef(O) ? "axis" : isMarkPropFieldOrDatumDef(O) ? "legend" : isFacetFieldDef(O) ? "header" : null;
      if (q && O[q]) {
        const { format: G, formatType: J, ...Z } = O[q];
        if (isCustomFormatType(J) && !H.customFormatTypes)
          return warn(customFormatTypeNotAllowed(_)), initFieldOrDatumDef({ ...O, [q]: Z }, _, H, V);
      }
    }
    return isFieldDef(O) ? initFieldDef(O, _, V) : initDatumDef(O);
  }
  function initDatumDef(O) {
    let _ = O.type;
    if (_)
      return O;
    const { datum: H } = O;
    return _ = isNumber$2(H) ? "quantitative" : isString$1(H) ? "nominal" : isDateTime(H) ? "temporal" : void 0, { ...O, type: _ };
  }
  function initFieldDef(O, _, { compositeMark: H = !1 } = {}) {
    const { aggregate: V, timeUnit: q, bin: G, field: J } = O, Z = { ...O };
    if (!H && V && !isAggregateOp(V) && !isArgmaxDef(V) && !isArgminDef(V) && (warn(invalidAggregate(V)), delete Z.aggregate), q && (Z.timeUnit = normalizeTimeUnit(q)), J && (Z.field = `${J}`), isBinning(G) && (Z.bin = normalizeBin(G, _)), isBinned(G) && !isXorY(_) && warn(channelShouldNotBeUsedForBinned(_)), isTypedFieldDef(Z)) {
      const { type: te } = Z, ne = getFullName(te);
      te !== ne && (Z.type = ne), te !== "quantitative" && isCountingAggregateOp(V) && (warn(invalidFieldTypeForCountAggregate(te, V)), Z.type = "quantitative");
    } else if (!isSecondaryRangeChannel(_)) {
      const te = defaultType$3(Z, _);
      Z.type = te;
    }
    if (isTypedFieldDef(Z)) {
      const { compatible: te, warning: ne } = channelCompatibility(Z, _) || {};
      te === !1 && warn(ne);
    }
    if (isSortableFieldDef(Z) && isString$1(Z.sort)) {
      const { sort: te } = Z;
      if (isSortByChannel(te))
        return {
          ...Z,
          sort: { encoding: te }
        };
      const ne = te.substring(1);
      if (te.charAt(0) === "-" && isSortByChannel(ne))
        return {
          ...Z,
          sort: { encoding: ne, order: "descending" }
        };
    }
    if (isFacetFieldDef(Z)) {
      const { header: te } = Z;
      if (te) {
        const { orient: ne, ...ee } = te;
        if (ne)
          return {
            ...Z,
            header: {
              ...ee,
              labelOrient: te.labelOrient || ne,
              titleOrient: te.titleOrient || ne
            }
          };
      }
    }
    return Z;
  }
  function normalizeBin(O, _) {
    return isBoolean$3(O) ? { maxbins: autoMaxBins(_) } : O === "binned" ? {
      binned: !0
    } : !O.maxbins && !O.step ? { ...O, maxbins: autoMaxBins(_) } : O;
  }
  const COMPATIBLE = { compatible: !0 };
  function channelCompatibility(O, _) {
    const H = O.type;
    if (H === "geojson" && _ !== "shape")
      return {
        compatible: !1,
        warning: `Channel ${_} should not be used with a geojson data.`
      };
    switch (_) {
      case ROW:
      case COLUMN:
      case FACET:
        return isDiscrete(O) ? COMPATIBLE : {
          compatible: !1,
          warning: channelShouldBeDiscrete(_)
        };
      case X:
      case Y$1:
      case XOFFSET:
      case YOFFSET:
      case COLOR:
      case FILL:
      case STROKE:
      case TEXT$1:
      case DETAIL:
      case KEY:
      case TOOLTIP:
      case HREF:
      case URL$1:
      case ANGLE:
      case THETA:
      case RADIUS:
      case DESCRIPTION:
        return COMPATIBLE;
      case LONGITUDE:
      case LONGITUDE2:
      case LATITUDE:
      case LATITUDE2:
        return H !== QUANTITATIVE ? {
          compatible: !1,
          warning: `Channel ${_} should be used with a quantitative field only, not ${O.type} field.`
        } : COMPATIBLE;
      case OPACITY:
      case FILLOPACITY:
      case STROKEOPACITY:
      case STROKEWIDTH:
      case SIZE:
      case THETA2:
      case RADIUS2:
      case X2:
      case Y2:
      case TIME:
        return H === "nominal" && !O.sort ? {
          compatible: !1,
          warning: `Channel ${_} should not be used with an unsorted discrete field.`
        } : COMPATIBLE;
      case SHAPE:
      case STROKEDASH:
        return !isDiscrete(O) && !isDiscretizing(O) ? {
          compatible: !1,
          warning: channelShouldBeDiscreteOrDiscretizing(_)
        } : COMPATIBLE;
      case ORDER:
        return O.type === "nominal" && !("sort" in O) ? {
          compatible: !1,
          warning: "Channel order is inappropriate for nominal field, which has no inherent order."
        } : COMPATIBLE;
    }
  }
  function isFieldOrDatumDefForTimeFormat(O) {
    const { formatType: _ } = getFormatMixins(O);
    return _ === "time" || !_ && isTemporalFieldDef(O);
  }
  function isTemporalFieldDef(O) {
    return O && (O.type === "temporal" || isFieldDef(O) && !!O.timeUnit);
  }
  function valueExpr(O, { timeUnit: _, type: H, wrapTime: V, undefinedIfExprNotRequired: q }) {
    var te;
    const G = _ && ((te = normalizeTimeUnit(_)) == null ? void 0 : te.unit);
    let J = G || H === "temporal", Z;
    return isExprRef(O) ? Z = O.expr : isSignalRef(O) ? Z = O.signal : isDateTime(O) ? (J = !0, Z = dateTimeToExpr(O)) : (isString$1(O) || isNumber$2(O)) && J && (Z = `datetime(${stringify$1(O)})`, isLocalSingleTimeUnit(G) && (isNumber$2(O) && O < 1e4 || isString$1(O) && isNaN(Date.parse(O))) && (Z = dateTimeToExpr({ [G]: O }))), Z ? V && J ? `time(${Z})` : Z : q ? void 0 : stringify$1(O);
  }
  function valueArray(O, _) {
    const { type: H } = O;
    return _.map((V) => {
      const q = isFieldDef(O) && !isBinnedTimeUnit(O.timeUnit) ? O.timeUnit : void 0, G = valueExpr(V, {
        timeUnit: q,
        type: H,
        undefinedIfExprNotRequired: !0
      });
      return G !== void 0 ? { signal: G } : V;
    });
  }
  function binRequiresRange(O, _) {
    return isBinning(O.bin) ? isScaleChannel(_) && ["ordinal", "nominal"].includes(O.type) : (console.warn("Only call this method for binned field defs."), !1);
  }
  const CONDITIONAL_AXIS_PROP_INDEX = {
    labelAlign: {
      part: "labels",
      vgProp: "align"
    },
    labelBaseline: {
      part: "labels",
      vgProp: "baseline"
    },
    labelColor: {
      part: "labels",
      vgProp: "fill"
    },
    labelFont: {
      part: "labels",
      vgProp: "font"
    },
    labelFontSize: {
      part: "labels",
      vgProp: "fontSize"
    },
    labelFontStyle: {
      part: "labels",
      vgProp: "fontStyle"
    },
    labelFontWeight: {
      part: "labels",
      vgProp: "fontWeight"
    },
    labelOpacity: {
      part: "labels",
      vgProp: "opacity"
    },
    labelOffset: null,
    labelPadding: null,
    // There is no fixed vgProp for tickSize, need to use signal.
    gridColor: {
      part: "grid",
      vgProp: "stroke"
    },
    gridDash: {
      part: "grid",
      vgProp: "strokeDash"
    },
    gridDashOffset: {
      part: "grid",
      vgProp: "strokeDashOffset"
    },
    gridOpacity: {
      part: "grid",
      vgProp: "opacity"
    },
    gridWidth: {
      part: "grid",
      vgProp: "strokeWidth"
    },
    tickColor: {
      part: "ticks",
      vgProp: "stroke"
    },
    tickDash: {
      part: "ticks",
      vgProp: "strokeDash"
    },
    tickDashOffset: {
      part: "ticks",
      vgProp: "strokeDashOffset"
    },
    tickOpacity: {
      part: "ticks",
      vgProp: "opacity"
    },
    tickSize: null,
    // There is no fixed vgProp for tickSize, need to use signal.
    tickWidth: {
      part: "ticks",
      vgProp: "strokeWidth"
    }
  };
  function isConditionalAxisValue(O) {
    return O == null ? void 0 : O.condition;
  }
  const AXIS_PARTS = ["domain", "grid", "labels", "ticks", "title"], AXIS_PROPERTY_TYPE = {
    grid: "grid",
    gridCap: "grid",
    gridColor: "grid",
    gridDash: "grid",
    gridDashOffset: "grid",
    gridOpacity: "grid",
    gridScale: "grid",
    gridWidth: "grid",
    orient: "main",
    bandPosition: "both",
    // Need to be applied to grid axis too, so the grid will align with ticks.
    aria: "main",
    description: "main",
    domain: "main",
    domainCap: "main",
    domainColor: "main",
    domainDash: "main",
    domainDashOffset: "main",
    domainOpacity: "main",
    domainWidth: "main",
    format: "main",
    formatType: "main",
    labelAlign: "main",
    labelAngle: "main",
    labelBaseline: "main",
    labelBound: "main",
    labelColor: "main",
    labelFlush: "main",
    labelFlushOffset: "main",
    labelFont: "main",
    labelFontSize: "main",
    labelFontStyle: "main",
    labelFontWeight: "main",
    labelLimit: "main",
    labelLineHeight: "main",
    labelOffset: "main",
    labelOpacity: "main",
    labelOverlap: "main",
    labelPadding: "main",
    labels: "main",
    labelSeparation: "main",
    maxExtent: "main",
    minExtent: "main",
    offset: "both",
    position: "main",
    tickCap: "main",
    tickColor: "main",
    tickDash: "main",
    tickDashOffset: "main",
    tickMinStep: "both",
    tickOffset: "both",
    // Need to be applied to grid axis too, so the grid will align with ticks.
    tickOpacity: "main",
    tickRound: "both",
    // Apply rounding to grid and ticks so they are aligned.
    ticks: "main",
    tickSize: "main",
    tickWidth: "both",
    title: "main",
    titleAlign: "main",
    titleAnchor: "main",
    titleAngle: "main",
    titleBaseline: "main",
    titleColor: "main",
    titleFont: "main",
    titleFontSize: "main",
    titleFontStyle: "main",
    titleFontWeight: "main",
    titleLimit: "main",
    titleLineHeight: "main",
    titleOpacity: "main",
    titlePadding: "main",
    titleX: "main",
    titleY: "main",
    encode: "both",
    // we hide this in Vega-Lite
    scale: "both",
    tickBand: "both",
    tickCount: "both",
    tickExtra: "both",
    translate: "both",
    values: "both",
    zindex: "both"
    // this is actually set afterward, so it doesn't matter
  }, COMMON_AXIS_PROPERTIES_INDEX = {
    orient: 1,
    // other things can depend on orient
    aria: 1,
    bandPosition: 1,
    description: 1,
    domain: 1,
    domainCap: 1,
    domainColor: 1,
    domainDash: 1,
    domainDashOffset: 1,
    domainOpacity: 1,
    domainWidth: 1,
    format: 1,
    formatType: 1,
    grid: 1,
    gridCap: 1,
    gridColor: 1,
    gridDash: 1,
    gridDashOffset: 1,
    gridOpacity: 1,
    gridWidth: 1,
    labelAlign: 1,
    labelAngle: 1,
    labelBaseline: 1,
    labelBound: 1,
    labelColor: 1,
    labelFlush: 1,
    labelFlushOffset: 1,
    labelFont: 1,
    labelFontSize: 1,
    labelFontStyle: 1,
    labelFontWeight: 1,
    labelLimit: 1,
    labelLineHeight: 1,
    labelOffset: 1,
    labelOpacity: 1,
    labelOverlap: 1,
    labelPadding: 1,
    labels: 1,
    labelSeparation: 1,
    maxExtent: 1,
    minExtent: 1,
    offset: 1,
    position: 1,
    tickBand: 1,
    tickCap: 1,
    tickColor: 1,
    tickCount: 1,
    tickDash: 1,
    tickDashOffset: 1,
    tickExtra: 1,
    tickMinStep: 1,
    tickOffset: 1,
    tickOpacity: 1,
    tickRound: 1,
    ticks: 1,
    tickSize: 1,
    tickWidth: 1,
    title: 1,
    titleAlign: 1,
    titleAnchor: 1,
    titleAngle: 1,
    titleBaseline: 1,
    titleColor: 1,
    titleFont: 1,
    titleFontSize: 1,
    titleFontStyle: 1,
    titleFontWeight: 1,
    titleLimit: 1,
    titleLineHeight: 1,
    titleOpacity: 1,
    titlePadding: 1,
    titleX: 1,
    titleY: 1,
    translate: 1,
    values: 1,
    zindex: 1
  }, AXIS_PROPERTIES_INDEX = {
    ...COMMON_AXIS_PROPERTIES_INDEX,
    style: 1,
    labelExpr: 1,
    encoding: 1
  };
  function isAxisProperty(O) {
    return has$1(AXIS_PROPERTIES_INDEX, O);
  }
  const AXIS_CONFIGS_INDEX = {
    axis: 1,
    axisBand: 1,
    axisBottom: 1,
    axisDiscrete: 1,
    axisLeft: 1,
    axisPoint: 1,
    axisQuantitative: 1,
    axisRight: 1,
    axisTemporal: 1,
    axisTop: 1,
    axisX: 1,
    axisXBand: 1,
    axisXDiscrete: 1,
    axisXPoint: 1,
    axisXQuantitative: 1,
    axisXTemporal: 1,
    axisY: 1,
    axisYBand: 1,
    axisYDiscrete: 1,
    axisYPoint: 1,
    axisYQuantitative: 1,
    axisYTemporal: 1
  }, AXIS_CONFIGS = keys$2(AXIS_CONFIGS_INDEX);
  function isUnitSpec(O) {
    return hasProperty(O, "mark");
  }
  class CompositeMarkNormalizer {
    constructor(_, H) {
      this.name = _, this.run = H;
    }
    hasMatchingType(_) {
      return isUnitSpec(_) ? getMarkType(_.mark) === this.name : !1;
    }
  }
  function channelHasField(O, _) {
    const H = O && O[_];
    return H ? isArray$2(H) ? some(H, (V) => !!V.field) : isFieldDef(H) || hasConditionalFieldDef(H) : !1;
  }
  function channelHasFieldOrDatum(O, _) {
    const H = O && O[_];
    return H ? isArray$2(H) ? some(H, (V) => !!V.field) : isFieldDef(H) || isDatumDef(H) || hasConditionalFieldOrDatumDef(H) : !1;
  }
  function channelHasNestedOffsetScale(O, _) {
    if (isXorY(_)) {
      const H = O[_];
      if ((isFieldDef(H) || isDatumDef(H)) && (isDiscrete$1(H.type) || isFieldDef(H) && H.timeUnit)) {
        const V = getOffsetScaleChannel(_);
        return channelHasFieldOrDatum(O, V);
      }
    }
    return !1;
  }
  function isAggregate$1(O) {
    return some(CHANNELS, (_) => {
      if (channelHasField(O, _)) {
        const H = O[_];
        if (isArray$2(H))
          return some(H, (V) => !!V.aggregate);
        {
          const V = getFieldDef(H);
          return V && !!V.aggregate;
        }
      }
      return !1;
    });
  }
  function extractTransformsFromEncoding(O, _) {
    const H = [], V = [], q = [], G = [], J = {};
    return forEach$1(O, (Z, te) => {
      if (isFieldDef(Z)) {
        const { field: ne, aggregate: ee, bin: ae, timeUnit: oe, ...se } = Z;
        if (ee || oe || ae) {
          const ce = getGuide(Z), le = ce == null ? void 0 : ce.title;
          let ue = vgField(Z, { forAs: !0 });
          const de = {
            // Only add title if it doesn't exist
            ...le ? [] : { title: title(Z, _, { allowDisabling: !0 }) },
            ...se,
            // Always overwrite field
            field: ue
          };
          if (ee) {
            let fe;
            if (isArgmaxDef(ee) ? (fe = "argmax", ue = vgField({ op: "argmax", field: ee.argmax }, { forAs: !0 }), de.field = `${ue}.${ne}`) : isArgminDef(ee) ? (fe = "argmin", ue = vgField({ op: "argmin", field: ee.argmin }, { forAs: !0 }), de.field = `${ue}.${ne}`) : ee !== "boxplot" && ee !== "errorbar" && ee !== "errorband" && (fe = ee), fe) {
              const he = {
                op: fe,
                as: ue
              };
              ne && (he.field = ne), G.push(he);
            }
          } else if (H.push(ue), isTypedFieldDef(Z) && isBinning(ae)) {
            if (V.push({ bin: ae, field: ne, as: ue }), H.push(vgField(Z, { binSuffix: "end" })), binRequiresRange(Z, te) && H.push(vgField(Z, { binSuffix: "range" })), isXorY(te)) {
              const fe = {
                field: `${ue}_end`
              };
              J[`${te}2`] = fe;
            }
            de.bin = "binned", isSecondaryRangeChannel(te) || (de.type = QUANTITATIVE);
          } else if (oe && !isBinnedTimeUnit(oe)) {
            q.push({
              timeUnit: oe,
              field: ne,
              as: ue
            });
            const fe = isTypedFieldDef(Z) && Z.type !== TEMPORAL && "time";
            fe && (te === TEXT$1 || te === TOOLTIP ? de.formatType = fe : isNonPositionScaleChannel(te) ? de.legend = {
              formatType: fe,
              ...de.legend
            } : isXorY(te) && (de.axis = {
              formatType: fe,
              ...de.axis
            }));
          }
          J[te] = de;
        } else
          H.push(ne), J[te] = O[te];
      } else
        J[te] = O[te];
    }), {
      bins: V,
      timeUnits: q,
      aggregate: G,
      groupby: H,
      encoding: J
    };
  }
  function markChannelCompatible(O, _, H) {
    const V = supportMark(_, H);
    if (V) {
      if (V === "binned") {
        const q = O[_ === X2 ? X : Y$1];
        return !!(isFieldDef(q) && isFieldDef(O[_]) && isBinned(q.bin));
      }
    } else return !1;
    return !0;
  }
  function initEncoding(O, _, H, V) {
    const q = {};
    for (const G of keys$2(O))
      isChannel(G) || warn(invalidEncodingChannel(G));
    for (let G of UNIT_CHANNELS) {
      if (!O[G])
        continue;
      const J = O[G];
      if (isXorYOffset(G)) {
        const Z = getMainChannelFromOffsetChannel(G), te = q[Z];
        if (isFieldDef(te) && isContinuous(te.type) && isFieldDef(J) && !te.timeUnit) {
          warn(offsetNestedInsideContinuousPositionScaleDropped(Z));
          continue;
        }
      }
      if (G === "angle" && _ === "arc" && !O.theta && (warn(REPLACE_ANGLE_WITH_THETA), G = THETA), !markChannelCompatible(O, G, _)) {
        warn(incompatibleChannel(G, _));
        continue;
      }
      if (G === SIZE && _ === "line") {
        const Z = getFieldDef(O[G]);
        if (Z != null && Z.aggregate) {
          warn(LINE_WITH_VARYING_SIZE);
          continue;
        }
      }
      if (G === COLOR && (H ? "fill" in O : "stroke" in O)) {
        warn(droppingColor("encoding", { fill: "fill" in O, stroke: "stroke" in O }));
        continue;
      }
      if (G === DETAIL || G === ORDER && !isArray$2(J) && !isValueDef(J) || G === TOOLTIP && isArray$2(J)) {
        if (J) {
          if (G === ORDER) {
            const Z = O[G];
            if (isOrderOnlyDef(Z)) {
              q[G] = Z;
              continue;
            }
          }
          q[G] = array$5(J).reduce((Z, te) => (isFieldDef(te) ? Z.push(initFieldDef(te, G)) : warn(emptyFieldDef(te, G)), Z), []);
        }
      } else {
        if (G === TOOLTIP && J === null)
          q[G] = null;
        else if (!isFieldDef(J) && !isDatumDef(J) && !isValueDef(J) && !isConditionalDef(J) && !isSignalRef(J)) {
          warn(emptyFieldDef(J, G));
          continue;
        }
        q[G] = initChannelDef(J, G, V);
      }
    }
    return q;
  }
  function normalizeEncoding(O, _) {
    const H = {};
    for (const V of keys$2(O)) {
      const q = initChannelDef(O[V], V, _, { compositeMark: !0 });
      H[V] = q;
    }
    return H;
  }
  function fieldDefs(O) {
    const _ = [];
    for (const H of keys$2(O))
      if (channelHasField(O, H)) {
        const V = O[H], q = array$5(V);
        for (const G of q)
          isFieldDef(G) ? _.push(G) : hasConditionalFieldDef(G) && _.push(G.condition);
      }
    return _;
  }
  function forEach$1(O, _, H) {
    if (O)
      for (const V of keys$2(O)) {
        const q = O[V];
        if (isArray$2(q))
          for (const G of q)
            _.call(H, G, V);
        else
          _.call(H, q, V);
      }
  }
  function reduce$2(O, _, H, V) {
    return O ? keys$2(O).reduce((q, G) => {
      const J = O[G];
      return isArray$2(J) ? J.reduce((Z, te) => _.call(V, Z, te, G), q) : _.call(V, q, J, G);
    }, H) : H;
  }
  function pathGroupingFields(O, _) {
    return keys$2(_).reduce((H, V) => {
      switch (V) {
        // x, y, x2, y2, lat, long, lat1, long2, order, tooltip, href, aria label, cursor should not cause lines to group
        case X:
        case Y$1:
        case HREF:
        case DESCRIPTION:
        case URL$1:
        case X2:
        case Y2:
        case XOFFSET:
        case YOFFSET:
        case THETA:
        case THETA2:
        case RADIUS:
        case RADIUS2:
        case TIME:
        // falls through
        case LATITUDE:
        case LONGITUDE:
        case LATITUDE2:
        case LONGITUDE2:
        // TODO: case 'cursor':
        // text, shape, shouldn't be a part of line/trail/area [falls through]
        case TEXT$1:
        case SHAPE:
        case ANGLE:
        // falls through
        // tooltip fields should not be added to group by [falls through]
        case TOOLTIP:
          return H;
        case ORDER:
          if (O === "line" || O === "trail")
            return H;
        // but order should group area for stacking (falls through)
        case DETAIL:
        case KEY: {
          const q = _[V];
          if (isArray$2(q) || isFieldDef(q))
            for (const G of array$5(q))
              G.aggregate || H.push(vgField(G, {}));
          return H;
        }
        case SIZE:
          if (O === "trail")
            return H;
        // For line, size should group lines.
        // falls through
        case COLOR:
        case FILL:
        case STROKE:
        case OPACITY:
        case FILLOPACITY:
        case STROKEOPACITY:
        case STROKEDASH:
        case STROKEWIDTH: {
          const q = getFieldDef(_[V]);
          return q && !q.aggregate && H.push(vgField(q, {})), H;
        }
      }
    }, []);
  }
  function filterTooltipWithAggregatedField(O) {
    const { tooltip: _, ...H } = O;
    if (!_)
      return { filteredEncoding: H };
    let V, q;
    if (isArray$2(_)) {
      for (const G of _)
        G.aggregate ? (V || (V = []), V.push(G)) : (q || (q = []), q.push(G));
      V && (H.tooltip = V);
    } else
      _.aggregate ? H.tooltip = _ : q = _;
    return isArray$2(q) && q.length === 1 && (q = q[0]), { customTooltipWithoutAggregatedField: q, filteredEncoding: H };
  }
  function getCompositeMarkTooltip(O, _, H, V = !0) {
    if ("tooltip" in H)
      return { tooltip: H.tooltip };
    const q = O.map(({ fieldPrefix: J, titlePrefix: Z }) => {
      const te = V ? ` of ${getTitle(_)}` : "";
      return {
        field: J + _.field,
        type: _.type,
        title: isSignalRef(Z) ? { signal: `${Z}"${escape(te)}"` } : Z + te
      };
    }), G = fieldDefs(H).map(toStringFieldDef);
    return {
      tooltip: [
        ...q,
        // need to cast because TextFieldDef supports fewer types of bin
        ...unique(G, hash)
      ]
    };
  }
  function getTitle(O) {
    const { title: _, field: H } = O;
    return getFirstDefined(_, H);
  }
  function makeCompositeAggregatePartFactory(O, _, H, V, q) {
    const { scale: G, axis: J } = H;
    return ({ partName: Z, mark: te, positionPrefix: ne, endPositionPrefix: ee = void 0, extraEncoding: ae = {} }) => {
      const oe = getTitle(H);
      return partLayerMixins(O, Z, q, {
        mark: te,
        // TODO better remove this method and just have mark as a parameter of the method
        encoding: {
          [_]: {
            field: `${ne}_${H.field}`,
            type: H.type,
            ...oe !== void 0 ? { title: oe } : {},
            ...G !== void 0 ? { scale: G } : {},
            ...J !== void 0 ? { axis: J } : {}
          },
          ...isString$1(ee) ? {
            [`${_}2`]: {
              field: `${ee}_${H.field}`
            }
          } : {},
          ...V,
          ...ae
        }
      });
    };
  }
  function partLayerMixins(O, _, H, V) {
    const { clip: q, color: G, opacity: J } = O, Z = O.type;
    return O[_] || O[_] === void 0 && H[_] ? [
      {
        ...V,
        mark: {
          ...H[_],
          ...q ? { clip: q } : {},
          ...G ? { color: G } : {},
          ...J ? { opacity: J } : {},
          ...isMarkDef(V.mark) ? V.mark : { type: V.mark },
          style: `${Z}-${String(_)}`,
          ...isBoolean$3(O[_]) ? {} : O[_]
        }
      }
    ] : [];
  }
  function compositeMarkContinuousAxis(O, _, H) {
    const { encoding: V } = O, q = _ === "vertical" ? "y" : "x", G = V[q], J = V[`${q}2`], Z = V[`${q}Error`], te = V[`${q}Error2`];
    return {
      continuousAxisChannelDef: filterAggregateFromChannelDef(G, H),
      continuousAxisChannelDef2: filterAggregateFromChannelDef(J, H),
      continuousAxisChannelDefError: filterAggregateFromChannelDef(Z, H),
      continuousAxisChannelDefError2: filterAggregateFromChannelDef(te, H),
      continuousAxis: q
    };
  }
  function filterAggregateFromChannelDef(O, _) {
    if (O != null && O.aggregate) {
      const { aggregate: H, ...V } = O;
      return H !== _ && warn(errorBarContinuousAxisHasCustomizedAggregate(H, _)), V;
    } else
      return O;
  }
  function compositeMarkOrient(O, _) {
    const { mark: H, encoding: V } = O, { x: q, y: G } = V;
    if (isMarkDef(H) && H.orient)
      return H.orient;
    if (isContinuousFieldOrDatumDef(q)) {
      if (isContinuousFieldOrDatumDef(G)) {
        const J = isFieldDef(q) && q.aggregate, Z = isFieldDef(G) && G.aggregate;
        if (!J && Z === _)
          return "vertical";
        if (!Z && J === _)
          return "horizontal";
        if (J === _ && Z === _)
          throw new Error("Both x and y cannot have aggregate");
        return isFieldOrDatumDefForTimeFormat(G) && !isFieldOrDatumDefForTimeFormat(q) ? "horizontal" : "vertical";
      }
      return "horizontal";
    } else {
      if (isContinuousFieldOrDatumDef(G))
        return "vertical";
      throw new Error(`Need a valid continuous axis for ${_}s`);
    }
  }
  const BOXPLOT = "boxplot", BOXPLOT_PARTS = ["box", "median", "outliers", "rule", "ticks"], boxPlotNormalizer = new CompositeMarkNormalizer(BOXPLOT, normalizeBoxPlot);
  function getBoxPlotType(O) {
    return isNumber$2(O) ? "tukey" : O;
  }
  function normalizeBoxPlot(O, { config: _ }) {
    O = {
      ...O,
      encoding: normalizeEncoding(O.encoding, _)
    };
    const { mark: H, encoding: V, params: q, projection: G, ...J } = O, Z = isMarkDef(H) ? H : { type: H };
    q && warn(selectionNotSupported("boxplot"));
    const te = Z.extent ?? _.boxplot.extent, ne = getMarkPropOrConfig(
      "size",
      Z,
      // TODO: https://github.com/vega/vega-lite/issues/6245
      _
    ), ee = Z.invalid, ae = getBoxPlotType(te), { bins: oe, timeUnits: se, transform: ce, continuousAxisChannelDef: le, continuousAxis: ue, groupby: de, aggregate: fe, encodingWithoutContinuousAxis: he, ticksOrient: me, boxOrient: pe, customTooltipWithoutAggregatedField: ye } = boxParams(O, te, _), ge = removePathFromField(le.field), { color: ve, size: be, ...Ce } = he, Se = (bt) => makeCompositeAggregatePartFactory(Z, ue, le, bt, _.boxplot), we = Se(Ce), Ee = Se(he), xe = (isObject$2(_.boxplot.box) ? _.boxplot.box.color : _.mark.color) || "#4c78a8", Le = Se({
      ...Ce,
      ...be ? { size: be } : {},
      color: {
        condition: {
          test: `${accessWithDatumToUnescapedPath(`lower_box_${le.field}`)} >= ${accessWithDatumToUnescapedPath(`upper_box_${le.field}`)}`,
          ...ve || { value: xe }
        }
      }
    }), Qe = getCompositeMarkTooltip([
      { fieldPrefix: ae === "min-max" ? "upper_whisker_" : "max_", titlePrefix: "Max" },
      { fieldPrefix: "upper_box_", titlePrefix: "Q3" },
      { fieldPrefix: "mid_box_", titlePrefix: "Median" },
      { fieldPrefix: "lower_box_", titlePrefix: "Q1" },
      { fieldPrefix: ae === "min-max" ? "lower_whisker_" : "min_", titlePrefix: "Min" }
    ], le, he), ke = { type: "tick", color: "black", opacity: 1, orient: me, invalid: ee, aria: !1 }, Me = ae === "min-max" ? Qe : (
      // for tukey / k-IQR, just show upper/lower-whisker
      getCompositeMarkTooltip([
        { fieldPrefix: "upper_whisker_", titlePrefix: "Upper Whisker" },
        { fieldPrefix: "lower_whisker_", titlePrefix: "Lower Whisker" }
      ], le, he)
    ), Re = [
      ...we({
        partName: "rule",
        mark: { type: "rule", invalid: ee, aria: !1 },
        positionPrefix: "lower_whisker",
        endPositionPrefix: "lower_box",
        extraEncoding: Me
      }),
      ...we({
        partName: "rule",
        mark: { type: "rule", invalid: ee, aria: !1 },
        positionPrefix: "upper_box",
        endPositionPrefix: "upper_whisker",
        extraEncoding: Me
      }),
      ...we({
        partName: "ticks",
        mark: ke,
        positionPrefix: "lower_whisker",
        extraEncoding: Me
      }),
      ...we({
        partName: "ticks",
        mark: ke,
        positionPrefix: "upper_whisker",
        extraEncoding: Me
      })
    ], De = [
      ...ae !== "tukey" ? Re : [],
      ...Ee({
        partName: "box",
        mark: {
          type: "bar",
          ...ne ? { size: ne } : {},
          orient: pe,
          invalid: ee,
          ariaRoleDescription: "box"
        },
        positionPrefix: "lower_box",
        endPositionPrefix: "upper_box",
        extraEncoding: Qe
      }),
      ...Le({
        partName: "median",
        mark: {
          type: "tick",
          invalid: ee,
          ...isObject$2(_.boxplot.median) && _.boxplot.median.color ? { color: _.boxplot.median.color } : {},
          ...ne ? { size: ne } : {},
          orient: me,
          aria: !1
        },
        positionPrefix: "mid_box",
        extraEncoding: Qe
      })
    ];
    if (ae === "min-max")
      return {
        ...J,
        transform: (J.transform ?? []).concat(ce),
        layer: De
      };
    const Pe = accessWithDatumToUnescapedPath(`lower_box_${le.field}`), $e = accessWithDatumToUnescapedPath(`upper_box_${le.field}`), qe = `(${$e} - ${Pe})`, Xe = `${Pe} - ${te} * ${qe}`, Ve = `${$e} + ${te} * ${qe}`, Ze = accessWithDatumToUnescapedPath(le.field), Ge = {
      joinaggregate: boxParamsQuartiles(le.field),
      groupby: de
    }, ze = {
      transform: [
        {
          filter: `(${Xe} <= ${Ze}) && (${Ze} <= ${Ve})`
        },
        {
          aggregate: [
            {
              op: "min",
              field: le.field,
              as: `lower_whisker_${ge}`
            },
            {
              op: "max",
              field: le.field,
              as: `upper_whisker_${ge}`
            },
            // preserve lower_box / upper_box
            {
              op: "min",
              field: `lower_box_${le.field}`,
              as: `lower_box_${ge}`
            },
            {
              op: "max",
              field: `upper_box_${le.field}`,
              as: `upper_box_${ge}`
            },
            ...fe
          ],
          groupby: de
        }
      ],
      layer: Re
    }, { tooltip: We, ...at } = Ce, { scale: ct, axis: et } = le, dt = getTitle(le), Je = omit(et, ["title"]), it = partLayerMixins(Z, "outliers", _.boxplot, {
      transform: [{ filter: `(${Ze} < ${Xe}) || (${Ze} > ${Ve})` }],
      mark: "point",
      encoding: {
        [ue]: {
          field: le.field,
          type: le.type,
          ...dt !== void 0 ? { title: dt } : {},
          ...ct !== void 0 ? { scale: ct } : {},
          // add axis without title since we already added the title above
          ...isEmpty(Je) ? {} : { axis: Je }
        },
        ...at,
        ...ve ? { color: ve } : {},
        ...ye ? { tooltip: ye } : {}
      }
    })[0];
    let st;
    const gt = [...oe, ...se, Ge];
    return it ? st = {
      transform: gt,
      layer: [it, ze]
    } : (st = ze, st.transform.unshift(...gt)), {
      ...J,
      layer: [
        st,
        {
          // boxplot
          transform: ce,
          layer: De
        }
      ]
    };
  }
  function boxParamsQuartiles(O) {
    const _ = removePathFromField(O);
    return [
      {
        op: "q1",
        field: O,
        as: `lower_box_${_}`
      },
      {
        op: "q3",
        field: O,
        as: `upper_box_${_}`
      }
    ];
  }
  function boxParams(O, _, H) {
    const V = compositeMarkOrient(O, BOXPLOT), { continuousAxisChannelDef: q, continuousAxis: G } = compositeMarkContinuousAxis(O, V, BOXPLOT), J = q.field, Z = removePathFromField(J), te = getBoxPlotType(_), ne = [
      ...boxParamsQuartiles(J),
      {
        op: "median",
        field: J,
        as: `mid_box_${Z}`
      },
      {
        op: "min",
        field: J,
        as: (te === "min-max" ? "lower_whisker_" : "min_") + Z
      },
      {
        op: "max",
        field: J,
        as: (te === "min-max" ? "upper_whisker_" : "max_") + Z
      }
    ], ee = te === "min-max" || te === "tukey" ? [] : [
      // This is for the  original k-IQR, which we do not expose
      {
        calculate: `${accessWithDatumToUnescapedPath(`upper_box_${Z}`)} - ${accessWithDatumToUnescapedPath(`lower_box_${Z}`)}`,
        as: `iqr_${Z}`
      },
      {
        calculate: `min(${accessWithDatumToUnescapedPath(`upper_box_${Z}`)} + ${accessWithDatumToUnescapedPath(`iqr_${Z}`)} * ${_}, ${accessWithDatumToUnescapedPath(`max_${Z}`)})`,
        as: `upper_whisker_${Z}`
      },
      {
        calculate: `max(${accessWithDatumToUnescapedPath(`lower_box_${Z}`)} - ${accessWithDatumToUnescapedPath(`iqr_${Z}`)} * ${_}, ${accessWithDatumToUnescapedPath(`min_${Z}`)})`,
        as: `lower_whisker_${Z}`
      }
    ], { [G]: ae, ...oe } = O.encoding, { customTooltipWithoutAggregatedField: se, filteredEncoding: ce } = filterTooltipWithAggregatedField(oe), { bins: le, timeUnits: ue, aggregate: de, groupby: fe, encoding: he } = extractTransformsFromEncoding(ce, H), me = V === "vertical" ? "horizontal" : "vertical", pe = V, ye = [
      ...le,
      ...ue,
      {
        aggregate: [...de, ...ne],
        groupby: fe
      },
      ...ee
    ];
    return {
      bins: le,
      timeUnits: ue,
      transform: ye,
      groupby: fe,
      aggregate: de,
      continuousAxisChannelDef: q,
      continuousAxis: G,
      encodingWithoutContinuousAxis: he,
      ticksOrient: me,
      boxOrient: pe,
      customTooltipWithoutAggregatedField: se
    };
  }
  const ERRORBAR = "errorbar", ERRORBAR_PARTS = ["ticks", "rule"], errorBarNormalizer = new CompositeMarkNormalizer(ERRORBAR, normalizeErrorBar);
  function normalizeErrorBar(O, { config: _ }) {
    O = {
      ...O,
      encoding: normalizeEncoding(O.encoding, _)
    };
    const { transform: H, continuousAxisChannelDef: V, continuousAxis: q, encodingWithoutContinuousAxis: G, ticksOrient: J, markDef: Z, outerSpec: te, tooltipEncoding: ne } = errorBarParams(O, ERRORBAR, _);
    delete G.size;
    const ee = makeCompositeAggregatePartFactory(Z, q, V, G, _.errorbar), ae = Z.thickness, oe = Z.size, se = {
      type: "tick",
      orient: J,
      aria: !1,
      ...ae !== void 0 ? { thickness: ae } : {},
      ...oe !== void 0 ? { size: oe } : {}
    }, ce = [
      ...ee({
        partName: "ticks",
        mark: se,
        positionPrefix: "lower",
        extraEncoding: ne
      }),
      ...ee({
        partName: "ticks",
        mark: se,
        positionPrefix: "upper",
        extraEncoding: ne
      }),
      ...ee({
        partName: "rule",
        mark: {
          type: "rule",
          ariaRoleDescription: "errorbar",
          ...ae !== void 0 ? { size: ae } : {}
        },
        positionPrefix: "lower",
        endPositionPrefix: "upper",
        extraEncoding: ne
      })
    ];
    return {
      ...te,
      transform: H,
      ...ce.length > 1 ? { layer: ce } : { ...ce[0] }
    };
  }
  function errorBarOrientAndInputType(O, _) {
    const { encoding: H } = O;
    if (errorBarIsInputTypeRaw(H))
      return {
        orient: compositeMarkOrient(O, _),
        inputType: "raw"
      };
    const V = errorBarIsInputTypeAggregatedUpperLower(H), q = errorBarIsInputTypeAggregatedError(H), G = H.x, J = H.y;
    if (V) {
      if (q)
        throw new Error(`${_} cannot be both type aggregated-upper-lower and aggregated-error`);
      const Z = H.x2, te = H.y2;
      if (isFieldOrDatumDef(Z) && isFieldOrDatumDef(te))
        throw new Error(`${_} cannot have both x2 and y2`);
      if (isFieldOrDatumDef(Z)) {
        if (isContinuousFieldOrDatumDef(G))
          return { orient: "horizontal", inputType: "aggregated-upper-lower" };
        throw new Error(`Both x and x2 have to be quantitative in ${_}`);
      } else if (isFieldOrDatumDef(te)) {
        if (isContinuousFieldOrDatumDef(J))
          return { orient: "vertical", inputType: "aggregated-upper-lower" };
        throw new Error(`Both y and y2 have to be quantitative in ${_}`);
      }
      throw new Error("No ranged axis");
    } else {
      const Z = H.xError, te = H.xError2, ne = H.yError, ee = H.yError2;
      if (isFieldOrDatumDef(te) && !isFieldOrDatumDef(Z))
        throw new Error(`${_} cannot have xError2 without xError`);
      if (isFieldOrDatumDef(ee) && !isFieldOrDatumDef(ne))
        throw new Error(`${_} cannot have yError2 without yError`);
      if (isFieldOrDatumDef(Z) && isFieldOrDatumDef(ne))
        throw new Error(`${_} cannot have both xError and yError with both are quantiative`);
      if (isFieldOrDatumDef(Z)) {
        if (isContinuousFieldOrDatumDef(G))
          return { orient: "horizontal", inputType: "aggregated-error" };
        throw new Error("All x, xError, and xError2 (if exist) have to be quantitative");
      } else if (isFieldOrDatumDef(ne)) {
        if (isContinuousFieldOrDatumDef(J))
          return { orient: "vertical", inputType: "aggregated-error" };
        throw new Error("All y, yError, and yError2 (if exist) have to be quantitative");
      }
      throw new Error("No ranged axis");
    }
  }
  function errorBarIsInputTypeRaw(O) {
    return (isFieldOrDatumDef(O.x) || isFieldOrDatumDef(O.y)) && !isFieldOrDatumDef(O.x2) && !isFieldOrDatumDef(O.y2) && !isFieldOrDatumDef(O.xError) && !isFieldOrDatumDef(O.xError2) && !isFieldOrDatumDef(O.yError) && !isFieldOrDatumDef(O.yError2);
  }
  function errorBarIsInputTypeAggregatedUpperLower(O) {
    return isFieldOrDatumDef(O.x2) || isFieldOrDatumDef(O.y2);
  }
  function errorBarIsInputTypeAggregatedError(O) {
    return isFieldOrDatumDef(O.xError) || isFieldOrDatumDef(O.xError2) || isFieldOrDatumDef(O.yError) || isFieldOrDatumDef(O.yError2);
  }
  function errorBarParams(O, _, H) {
    const { mark: V, encoding: q, params: G, projection: J, ...Z } = O, te = isMarkDef(V) ? V : { type: V };
    G && warn(selectionNotSupported(_));
    const { orient: ne, inputType: ee } = errorBarOrientAndInputType(O, _), { continuousAxisChannelDef: ae, continuousAxisChannelDef2: oe, continuousAxisChannelDefError: se, continuousAxisChannelDefError2: ce, continuousAxis: le } = compositeMarkContinuousAxis(O, ne, _), { errorBarSpecificAggregate: ue, postAggregateCalculates: de, tooltipSummary: fe, tooltipTitleWithFieldName: he } = errorBarAggregationAndCalculation(te, ae, oe, se, ce, ee, _, H), { [le]: me, [le === "x" ? "x2" : "y2"]: pe, [le === "x" ? "xError" : "yError"]: ye, [le === "x" ? "xError2" : "yError2"]: ge, ...ve } = q, { bins: be, timeUnits: Ce, aggregate: Se, groupby: we, encoding: Ee } = extractTransformsFromEncoding(ve, H), xe = [...Se, ...ue], Le = ee !== "raw" ? [] : we, Qe = getCompositeMarkTooltip(fe, ae, Ee, he);
    return {
      transform: [
        ...Z.transform ?? [],
        ...be,
        ...Ce,
        ...xe.length === 0 ? [] : [{ aggregate: xe, groupby: Le }],
        ...de
      ],
      groupby: Le,
      continuousAxisChannelDef: ae,
      continuousAxis: le,
      encodingWithoutContinuousAxis: Ee,
      ticksOrient: ne === "vertical" ? "horizontal" : "vertical",
      markDef: te,
      outerSpec: Z,
      tooltipEncoding: Qe
    };
  }
  function errorBarAggregationAndCalculation(O, _, H, V, q, G, J, Z) {
    let te = [], ne = [];
    const ee = _.field;
    let ae, oe = !1;
    if (G === "raw") {
      const se = O.center ? O.center : O.extent ? O.extent === "iqr" ? "median" : "mean" : Z.errorbar.center, ce = O.extent ? O.extent : se === "mean" ? "stderr" : "iqr";
      if (se === "median" != (ce === "iqr") && warn(errorBarCenterIsUsedWithWrongExtent(se, ce, J)), ce === "stderr" || ce === "stdev")
        te = [
          { op: ce, field: ee, as: `extent_${ee}` },
          { op: se, field: ee, as: `center_${ee}` }
        ], ne = [
          {
            calculate: `${accessWithDatumToUnescapedPath(`center_${ee}`)} + ${accessWithDatumToUnescapedPath(`extent_${ee}`)}`,
            as: `upper_${ee}`
          },
          {
            calculate: `${accessWithDatumToUnescapedPath(`center_${ee}`)} - ${accessWithDatumToUnescapedPath(`extent_${ee}`)}`,
            as: `lower_${ee}`
          }
        ], ae = [
          { fieldPrefix: "center_", titlePrefix: titleCase(se) },
          { fieldPrefix: "upper_", titlePrefix: getTitlePrefix(se, ce, "+") },
          { fieldPrefix: "lower_", titlePrefix: getTitlePrefix(se, ce, "-") }
        ], oe = !0;
      else {
        let le, ue, de;
        ce === "ci" ? (le = "mean", ue = "ci0", de = "ci1") : (le = "median", ue = "q1", de = "q3"), te = [
          { op: ue, field: ee, as: `lower_${ee}` },
          { op: de, field: ee, as: `upper_${ee}` },
          { op: le, field: ee, as: `center_${ee}` }
        ], ae = [
          {
            fieldPrefix: "upper_",
            titlePrefix: title({ field: ee, aggregate: de, type: "quantitative" }, Z, {
              allowDisabling: !1
            })
          },
          {
            fieldPrefix: "lower_",
            titlePrefix: title({ field: ee, aggregate: ue, type: "quantitative" }, Z, {
              allowDisabling: !1
            })
          },
          {
            fieldPrefix: "center_",
            titlePrefix: title({ field: ee, aggregate: le, type: "quantitative" }, Z, {
              allowDisabling: !1
            })
          }
        ];
      }
    } else {
      (O.center || O.extent) && warn(errorBarCenterAndExtentAreNotNeeded(O.center, O.extent)), G === "aggregated-upper-lower" ? (ae = [], ne = [
        {
          calculate: accessWithDatumToUnescapedPath(H.field),
          as: `upper_${ee}`
        },
        { calculate: accessWithDatumToUnescapedPath(ee), as: `lower_${ee}` }
      ]) : G === "aggregated-error" && (ae = [{ fieldPrefix: "", titlePrefix: ee }], ne = [
        {
          calculate: `${accessWithDatumToUnescapedPath(ee)} + ${accessWithDatumToUnescapedPath(V.field)}`,
          as: `upper_${ee}`
        }
      ], q ? ne.push({
        calculate: `${accessWithDatumToUnescapedPath(ee)} + ${accessWithDatumToUnescapedPath(q.field)}`,
        as: `lower_${ee}`
      }) : ne.push({
        calculate: `${accessWithDatumToUnescapedPath(ee)} - ${accessWithDatumToUnescapedPath(V.field)}`,
        as: `lower_${ee}`
      }));
      for (const se of ne)
        ae.push({
          fieldPrefix: se.as.substring(0, 6),
          titlePrefix: replaceAll$1(replaceAll$1(se.calculate, "datum['", ""), "']", "")
        });
    }
    return { postAggregateCalculates: ne, errorBarSpecificAggregate: te, tooltipSummary: ae, tooltipTitleWithFieldName: oe };
  }
  function getTitlePrefix(O, _, H) {
    return `${titleCase(O)} ${H} ${_}`;
  }
  const ERRORBAND = "errorband", ERRORBAND_PARTS = ["band", "borders"], errorBandNormalizer = new CompositeMarkNormalizer(ERRORBAND, normalizeErrorBand);
  function normalizeErrorBand(O, { config: _ }) {
    O = {
      ...O,
      encoding: normalizeEncoding(O.encoding, _)
    };
    const { transform: H, continuousAxisChannelDef: V, continuousAxis: q, encodingWithoutContinuousAxis: G, markDef: J, outerSpec: Z, tooltipEncoding: te } = errorBarParams(O, ERRORBAND, _), ne = J, ee = makeCompositeAggregatePartFactory(ne, q, V, G, _.errorband), ae = O.encoding.x !== void 0 && O.encoding.y !== void 0;
    let oe = { type: ae ? "area" : "rect" }, se = { type: ae ? "line" : "rule" };
    const ce = {
      ...ne.interpolate ? { interpolate: ne.interpolate } : {},
      ...ne.tension && ne.interpolate ? { tension: ne.tension } : {}
    };
    return ae ? (oe = {
      ...oe,
      ...ce,
      ariaRoleDescription: "errorband"
    }, se = {
      ...se,
      ...ce,
      aria: !1
    }) : ne.interpolate ? warn(errorBand1DNotSupport("interpolate")) : ne.tension && warn(errorBand1DNotSupport("tension")), {
      ...Z,
      transform: H,
      layer: [
        ...ee({
          partName: "band",
          mark: oe,
          positionPrefix: "lower",
          endPositionPrefix: "upper",
          extraEncoding: te
        }),
        ...ee({
          partName: "borders",
          mark: se,
          positionPrefix: "lower",
          extraEncoding: te
        }),
        ...ee({
          partName: "borders",
          mark: se,
          positionPrefix: "upper",
          extraEncoding: te
        })
      ]
    };
  }
  const compositeMarkRegistry = {};
  function add$1(O, _, H) {
    const V = new CompositeMarkNormalizer(O, _);
    compositeMarkRegistry[O] = { normalizer: V, parts: H };
  }
  function getAllCompositeMarks() {
    return keys$2(compositeMarkRegistry);
  }
  add$1(BOXPLOT, normalizeBoxPlot, BOXPLOT_PARTS);
  add$1(ERRORBAR, normalizeErrorBar, ERRORBAR_PARTS);
  add$1(ERRORBAND, normalizeErrorBand, ERRORBAND_PARTS);
  const VL_ONLY_LEGEND_CONFIG = [
    "gradientHorizontalMaxLength",
    "gradientHorizontalMinLength",
    "gradientVerticalMaxLength",
    "gradientVerticalMinLength",
    "unselectedOpacity"
  ], HEADER_TITLE_PROPERTIES_MAP = {
    titleAlign: "align",
    titleAnchor: "anchor",
    titleAngle: "angle",
    titleBaseline: "baseline",
    titleColor: "color",
    titleFont: "font",
    titleFontSize: "fontSize",
    titleFontStyle: "fontStyle",
    titleFontWeight: "fontWeight",
    titleLimit: "limit",
    titleLineHeight: "lineHeight",
    titleOrient: "orient",
    titlePadding: "offset"
  }, HEADER_LABEL_PROPERTIES_MAP = {
    labelAlign: "align",
    labelAnchor: "anchor",
    labelAngle: "angle",
    labelBaseline: "baseline",
    labelColor: "color",
    labelFont: "font",
    labelFontSize: "fontSize",
    labelFontStyle: "fontStyle",
    labelFontWeight: "fontWeight",
    labelLimit: "limit",
    labelLineHeight: "lineHeight",
    labelOrient: "orient",
    labelPadding: "offset"
  }, HEADER_TITLE_PROPERTIES = keys$2(HEADER_TITLE_PROPERTIES_MAP), HEADER_LABEL_PROPERTIES = keys$2(HEADER_LABEL_PROPERTIES_MAP), HEADER_CONFIGS_INDEX = {
    header: 1,
    headerRow: 1,
    headerColumn: 1,
    headerFacet: 1
  }, HEADER_CONFIGS = keys$2(HEADER_CONFIGS_INDEX), LEGEND_SCALE_CHANNELS = [
    "size",
    "shape",
    "fill",
    "stroke",
    "strokeDash",
    "strokeWidth",
    "opacity"
  ], defaultLegendConfig = {
    gradientHorizontalMaxLength: 200,
    gradientHorizontalMinLength: 100,
    gradientVerticalMaxLength: 200,
    gradientVerticalMinLength: 64,
    // This is Vega's minimum.
    unselectedOpacity: 0.35
  }, COMMON_LEGEND_PROPERTY_INDEX = {
    aria: 1,
    clipHeight: 1,
    columnPadding: 1,
    columns: 1,
    cornerRadius: 1,
    description: 1,
    direction: 1,
    fillColor: 1,
    format: 1,
    formatType: 1,
    gradientLength: 1,
    gradientOpacity: 1,
    gradientStrokeColor: 1,
    gradientStrokeWidth: 1,
    gradientThickness: 1,
    gridAlign: 1,
    labelAlign: 1,
    labelBaseline: 1,
    labelColor: 1,
    labelFont: 1,
    labelFontSize: 1,
    labelFontStyle: 1,
    labelFontWeight: 1,
    labelLimit: 1,
    labelOffset: 1,
    labelOpacity: 1,
    labelOverlap: 1,
    labelPadding: 1,
    labelSeparation: 1,
    legendX: 1,
    legendY: 1,
    offset: 1,
    orient: 1,
    padding: 1,
    rowPadding: 1,
    strokeColor: 1,
    symbolDash: 1,
    symbolDashOffset: 1,
    symbolFillColor: 1,
    symbolLimit: 1,
    symbolOffset: 1,
    symbolOpacity: 1,
    symbolSize: 1,
    symbolStrokeColor: 1,
    symbolStrokeWidth: 1,
    symbolType: 1,
    tickCount: 1,
    tickMinStep: 1,
    title: 1,
    titleAlign: 1,
    titleAnchor: 1,
    titleBaseline: 1,
    titleColor: 1,
    titleFont: 1,
    titleFontSize: 1,
    titleFontStyle: 1,
    titleFontWeight: 1,
    titleLimit: 1,
    titleLineHeight: 1,
    titleOpacity: 1,
    titleOrient: 1,
    titlePadding: 1,
    type: 1,
    values: 1,
    zindex: 1
  }, SELECTION_ID = "_vgsid_", defaultConfig$2 = {
    point: {
      on: "click",
      fields: [SELECTION_ID],
      toggle: "event.shiftKey",
      resolve: "global",
      clear: "dblclick"
    },
    interval: {
      on: "[pointerdown, window:pointerup] > window:pointermove!",
      encodings: ["x", "y"],
      translate: "[pointerdown, window:pointerup] > window:pointermove!",
      zoom: "wheel!",
      mark: { fill: "#333", fillOpacity: 0.125, stroke: "white" },
      resolve: "global",
      clear: "dblclick"
    }
  };
  function isLegendBinding(O) {
    return O === "legend" || !!(O != null && O.legend);
  }
  function isLegendStreamBinding(O) {
    return isLegendBinding(O) && isObject$2(O);
  }
  function isSelectionParameter(O) {
    return !!(O != null && O.select);
  }
  function assembleParameterSignals(O) {
    const _ = [];
    for (const H of O || []) {
      if (isSelectionParameter(H))
        continue;
      const { expr: V, bind: q, ...G } = H;
      if (q && V) {
        const J = {
          ...G,
          bind: q,
          init: V
        };
        _.push(J);
      } else {
        const J = {
          ...G,
          ...V ? { update: V } : {},
          ...q ? { bind: q } : {}
        };
        _.push(J);
      }
    }
    return _;
  }
  function isAnyConcatSpec(O) {
    return isVConcatSpec(O) || isHConcatSpec(O) || isConcatSpec(O);
  }
  function isConcatSpec(O) {
    return hasProperty(O, "concat");
  }
  function isVConcatSpec(O) {
    return hasProperty(O, "vconcat");
  }
  function isHConcatSpec(O) {
    return hasProperty(O, "hconcat");
  }
  function getStepFor({ step: O, offsetIsDiscrete: _ }) {
    return _ ? O.for ?? "offset" : "position";
  }
  function isStep(O) {
    return hasProperty(O, "step");
  }
  function isFrameMixins(O) {
    return hasProperty(O, "view") || hasProperty(O, "width") || hasProperty(O, "height");
  }
  const DEFAULT_SPACING = 20, COMPOSITION_LAYOUT_INDEX = {
    align: 1,
    bounds: 1,
    center: 1,
    columns: 1,
    spacing: 1
  }, COMPOSITION_LAYOUT_PROPERTIES = keys$2(COMPOSITION_LAYOUT_INDEX);
  function extractCompositionLayout(O, _, H) {
    const V = H[_], q = {}, { spacing: G, columns: J } = V;
    G !== void 0 && (q.spacing = G), J !== void 0 && (isFacetSpec(O) && !isFacetMapping(O.facet) || isConcatSpec(O)) && (q.columns = J), isVConcatSpec(O) && (q.columns = 1);
    for (const Z of COMPOSITION_LAYOUT_PROPERTIES)
      if (O[Z] !== void 0)
        if (Z === "spacing") {
          const te = O[Z];
          q[Z] = isNumber$2(te) ? te : {
            row: te.row ?? G,
            column: te.column ?? G
          };
        } else
          q[Z] = O[Z];
    return q;
  }
  function getViewConfigContinuousSize(O, _) {
    return O[_] ?? O[_ === "width" ? "continuousWidth" : "continuousHeight"];
  }
  function getViewConfigDiscreteStep(O, _) {
    const H = getViewConfigDiscreteSize(O, _);
    return isStep(H) ? H.step : DEFAULT_STEP;
  }
  function getViewConfigDiscreteSize(O, _) {
    const H = O[_] ?? O[_ === "width" ? "discreteWidth" : "discreteHeight"];
    return getFirstDefined(H, { step: O.step });
  }
  const DEFAULT_STEP = 20, defaultViewConfig = {
    continuousWidth: 200,
    continuousHeight: 200,
    step: DEFAULT_STEP
  }, defaultConfig$1 = {
    background: "white",
    padding: 5,
    timeFormat: "%b %d, %Y",
    countTitle: "Count of Records",
    view: defaultViewConfig,
    mark: defaultMarkConfig,
    arc: {},
    area: {},
    bar: defaultBarConfig,
    circle: {},
    geoshape: {},
    image: {},
    line: {},
    point: {},
    rect: defaultRectConfig,
    rule: { color: "black" },
    // Need this to override default color in mark config
    square: {},
    text: { color: "black" },
    // Need this to override default color in mark config
    tick: defaultTickConfig,
    trail: {},
    boxplot: {
      size: 14,
      extent: 1.5,
      box: {},
      median: { color: "white" },
      outliers: {},
      rule: {},
      ticks: null
    },
    errorbar: {
      center: "mean",
      rule: !0,
      ticks: !1
    },
    errorband: {
      band: {
        opacity: 0.3
      },
      borders: !1
    },
    scale: defaultScaleConfig,
    projection: {},
    legend: defaultLegendConfig,
    header: { titlePadding: 10, labelPadding: 10 },
    headerColumn: {},
    headerRow: {},
    headerFacet: {},
    selection: defaultConfig$2,
    style: {},
    title: {},
    facet: { spacing: DEFAULT_SPACING },
    concat: { spacing: DEFAULT_SPACING },
    normalizedNumberFormat: ".0%"
  }, tab10 = [
    "#4c78a8",
    "#f58518",
    "#e45756",
    "#72b7b2",
    "#54a24b",
    "#eeca3b",
    "#b279a2",
    "#ff9da6",
    "#9d755d",
    "#bab0ac"
  ], DEFAULT_FONT_SIZE = {
    text: 11,
    guideLabel: 10,
    guideTitle: 11,
    groupTitle: 13,
    groupSubtitle: 12
  }, DEFAULT_COLOR = {
    blue: tab10[0],
    orange: tab10[1],
    red: tab10[2],
    teal: tab10[3],
    green: tab10[4],
    yellow: tab10[5],
    purple: tab10[6],
    pink: tab10[7],
    brown: tab10[8],
    gray0: "#000",
    gray1: "#111",
    gray2: "#222",
    gray3: "#333",
    gray4: "#444",
    gray5: "#555",
    gray6: "#666",
    gray7: "#777",
    gray8: "#888",
    gray9: "#999",
    gray10: "#aaa",
    gray11: "#bbb",
    gray12: "#ccc",
    gray13: "#ddd",
    gray14: "#eee",
    gray15: "#fff"
  };
  function colorSignalConfig(O = {}) {
    return {
      signals: [
        {
          name: "color",
          value: isObject$2(O) ? { ...DEFAULT_COLOR, ...O } : DEFAULT_COLOR
        }
      ],
      mark: { color: { signal: "color.blue" } },
      rule: { color: { signal: "color.gray0" } },
      text: {
        color: { signal: "color.gray0" }
      },
      style: {
        "guide-label": {
          fill: { signal: "color.gray0" }
        },
        "guide-title": {
          fill: { signal: "color.gray0" }
        },
        "group-title": {
          fill: { signal: "color.gray0" }
        },
        "group-subtitle": {
          fill: { signal: "color.gray0" }
        },
        cell: {
          stroke: { signal: "color.gray8" }
        }
      },
      axis: {
        domainColor: { signal: "color.gray13" },
        gridColor: { signal: "color.gray8" },
        tickColor: { signal: "color.gray13" }
      },
      range: {
        category: [
          { signal: "color.blue" },
          { signal: "color.orange" },
          { signal: "color.red" },
          { signal: "color.teal" },
          { signal: "color.green" },
          { signal: "color.yellow" },
          { signal: "color.purple" },
          { signal: "color.pink" },
          { signal: "color.brown" },
          { signal: "color.grey8" }
        ]
      }
    };
  }
  function fontSizeSignalConfig(O) {
    return {
      signals: [
        {
          name: "fontSize",
          value: isObject$2(O) ? { ...DEFAULT_FONT_SIZE, ...O } : DEFAULT_FONT_SIZE
        }
      ],
      text: {
        fontSize: { signal: "fontSize.text" }
      },
      style: {
        "guide-label": {
          fontSize: { signal: "fontSize.guideLabel" }
        },
        "guide-title": {
          fontSize: { signal: "fontSize.guideTitle" }
        },
        "group-title": {
          fontSize: { signal: "fontSize.groupTitle" }
        },
        "group-subtitle": {
          fontSize: { signal: "fontSize.groupSubtitle" }
        }
      }
    };
  }
  function fontConfig(O) {
    return {
      text: { font: O },
      style: {
        "guide-label": { font: O },
        "guide-title": { font: O },
        "group-title": { font: O },
        "group-subtitle": { font: O }
      }
    };
  }
  function getAxisConfigInternal(O) {
    const _ = keys$2(O || {}), H = {};
    for (const V of _) {
      const q = O[V];
      H[V] = isConditionalAxisValue(q) ? signalOrValueRefWithCondition(q) : signalRefOrValue(q);
    }
    return H;
  }
  function getStyleConfigInternal(O) {
    const _ = keys$2(O), H = {};
    for (const V of _)
      H[V] = getAxisConfigInternal(O[V]);
    return H;
  }
  const configPropsWithExpr = [
    ...MARK_CONFIGS,
    ...AXIS_CONFIGS,
    ...HEADER_CONFIGS,
    "background",
    "padding",
    "legend",
    "lineBreak",
    "scale",
    "style",
    "title",
    "view"
  ];
  function initConfig(O = {}) {
    const { color: _, font: H, fontSize: V, selection: q, ...G } = O, J = mergeConfig({}, duplicate(defaultConfig$1), H ? fontConfig(H) : {}, _ ? colorSignalConfig(_) : {}, V ? fontSizeSignalConfig(V) : {}, G || {});
    q && writeConfig(J, "selection", q, !0);
    const Z = omit(J, configPropsWithExpr);
    for (const te of ["background", "lineBreak", "padding"])
      J[te] && (Z[te] = signalRefOrValue(J[te]));
    for (const te of MARK_CONFIGS)
      J[te] && (Z[te] = replaceExprRef(J[te]));
    for (const te of AXIS_CONFIGS)
      J[te] && (Z[te] = getAxisConfigInternal(J[te]));
    for (const te of HEADER_CONFIGS)
      J[te] && (Z[te] = replaceExprRef(J[te]));
    if (J.legend && (Z.legend = replaceExprRef(J.legend)), J.scale) {
      const { invalid: te, ...ne } = J.scale, ee = replaceExprRef(te, { level: 1 });
      Z.scale = {
        ...replaceExprRef(ne),
        ...keys$2(ee).length > 0 ? { invalid: ee } : {}
      };
    }
    return J.style && (Z.style = getStyleConfigInternal(J.style)), J.title && (Z.title = replaceExprRef(J.title)), J.view && (Z.view = replaceExprRef(J.view)), Z;
  }
  const MARK_STYLES = /* @__PURE__ */ new Set(["view", ...PRIMITIVE_MARKS]), VL_ONLY_CONFIG_PROPERTIES = [
    "color",
    "fontSize",
    "background",
    // We apply background to the spec directly.
    "padding",
    "facet",
    "concat",
    "numberFormat",
    "numberFormatType",
    "normalizedNumberFormat",
    "normalizedNumberFormatType",
    "timeFormat",
    "countTitle",
    "header",
    "axisQuantitative",
    "axisTemporal",
    "axisDiscrete",
    "axisPoint",
    "axisXBand",
    "axisXPoint",
    "axisXDiscrete",
    "axisXQuantitative",
    "axisXTemporal",
    "axisYBand",
    "axisYPoint",
    "axisYDiscrete",
    "axisYQuantitative",
    "axisYTemporal",
    "scale",
    "selection",
    "overlay"
    // FIXME: Redesign and unhide this
  ], VL_ONLY_ALL_MARK_SPECIFIC_CONFIG_PROPERTY_INDEX = {
    view: ["continuousWidth", "continuousHeight", "discreteWidth", "discreteHeight", "step"],
    ...VL_ONLY_MARK_SPECIFIC_CONFIG_PROPERTY_INDEX
  };
  function stripAndRedirectConfig(O) {
    O = duplicate(O);
    for (const _ of VL_ONLY_CONFIG_PROPERTIES)
      delete O[_];
    if (O.axis)
      for (const _ in O.axis)
        isConditionalAxisValue(O.axis[_]) && delete O.axis[_];
    if (O.legend)
      for (const _ of VL_ONLY_LEGEND_CONFIG)
        delete O.legend[_];
    if (O.mark) {
      for (const _ of VL_ONLY_MARK_CONFIG_PROPERTIES)
        delete O.mark[_];
      O.mark.tooltip && isObject$2(O.mark.tooltip) && delete O.mark.tooltip;
    }
    O.params && (O.signals = (O.signals || []).concat(assembleParameterSignals(O.params)), delete O.params);
    for (const _ of MARK_STYLES) {
      for (const V of VL_ONLY_MARK_CONFIG_PROPERTIES)
        delete O[_][V];
      const H = VL_ONLY_ALL_MARK_SPECIFIC_CONFIG_PROPERTY_INDEX[_];
      if (H)
        for (const V of H)
          delete O[_][V];
      redirectConfigToStyleConfig(O, _);
    }
    for (const _ of getAllCompositeMarks())
      delete O[_];
    redirectTitleConfig(O);
    for (const _ in O)
      isObject$2(O[_]) && isEmpty(O[_]) && delete O[_];
    return isEmpty(O) ? void 0 : O;
  }
  function redirectTitleConfig(O) {
    const { titleMarkConfig: _, subtitleMarkConfig: H, subtitle: V } = extractTitleConfig(O.title);
    isEmpty(_) || (O.style["group-title"] = {
      ...O.style["group-title"],
      ..._
      // config.title has higher precedence than config.style.group-title in Vega
    }), isEmpty(H) || (O.style["group-subtitle"] = {
      ...O.style["group-subtitle"],
      ...H
    }), isEmpty(V) ? delete O.title : O.title = V;
  }
  function redirectConfigToStyleConfig(O, _, H, V) {
    const q = O[_];
    _ === "view" && (H = "cell");
    const G = {
      ...q,
      ...O.style[H ?? _]
    };
    isEmpty(G) || (O.style[H ?? _] = G), delete O[_];
  }
  function isLayerSpec(O) {
    return hasProperty(O, "layer");
  }
  function isRepeatSpec(O) {
    return hasProperty(O, "repeat");
  }
  function isLayerRepeatSpec(O) {
    return !isArray$2(O.repeat) && hasProperty(O.repeat, "layer");
  }
  class SpecMapper {
    map(_, H) {
      return isFacetSpec(_) ? this.mapFacet(_, H) : isRepeatSpec(_) ? this.mapRepeat(_, H) : isHConcatSpec(_) ? this.mapHConcat(_, H) : isVConcatSpec(_) ? this.mapVConcat(_, H) : isConcatSpec(_) ? this.mapConcat(_, H) : this.mapLayerOrUnit(_, H);
    }
    mapLayerOrUnit(_, H) {
      if (isLayerSpec(_))
        return this.mapLayer(_, H);
      if (isUnitSpec(_))
        return this.mapUnit(_, H);
      throw new Error(invalidSpec(_));
    }
    mapLayer(_, H) {
      return {
        ..._,
        layer: _.layer.map((V) => this.mapLayerOrUnit(V, H))
      };
    }
    mapHConcat(_, H) {
      return {
        ..._,
        hconcat: _.hconcat.map((V) => this.map(V, H))
      };
    }
    mapVConcat(_, H) {
      return {
        ..._,
        vconcat: _.vconcat.map((V) => this.map(V, H))
      };
    }
    mapConcat(_, H) {
      const { concat: V, ...q } = _;
      return {
        ...q,
        concat: V.map((G) => this.map(G, H))
      };
    }
    mapFacet(_, H) {
      return {
        // as any is required here since TS cannot infer that FO may only be FieldName or Field, but not RepeatRef
        ..._,
        // TODO: remove "any" once we support all facet listed in https://github.com/vega/vega-lite/issues/2760
        spec: this.map(_.spec, H)
      };
    }
    mapRepeat(_, H) {
      return {
        ..._,
        // as any is required here since TS cannot infer that the output type satisfies the input type
        spec: this.map(_.spec, H)
      };
    }
  }
  const STACK_OFFSET_INDEX = {
    zero: 1,
    center: 1,
    normalize: 1
  };
  function isStackOffset(O) {
    return has$1(STACK_OFFSET_INDEX, O);
  }
  const STACKABLE_MARKS = /* @__PURE__ */ new Set([ARC, BAR, AREA, RULE, POINT, CIRCLE, SQUARE, LINE, TEXT, TICK]), STACK_BY_DEFAULT_MARKS = /* @__PURE__ */ new Set([BAR, AREA, ARC]);
  function isUnbinnedQuantitative(O) {
    return isFieldDef(O) && channelDefType(O) === "quantitative" && !O.bin;
  }
  function potentialStackedChannel(O, _, { orient: H, type: V }) {
    const q = _ === "x" ? "y" : "radius", G = _ === "x" && ["bar", "area"].includes(V), J = O[_], Z = O[q];
    if (isFieldDef(J) && isFieldDef(Z))
      if (isUnbinnedQuantitative(J) && isUnbinnedQuantitative(Z)) {
        if (J.stack)
          return _;
        if (Z.stack)
          return q;
        const te = isFieldDef(J) && !!J.aggregate, ne = isFieldDef(Z) && !!Z.aggregate;
        if (te !== ne)
          return te ? _ : q;
        if (G) {
          if (H === "vertical")
            return q;
          if (H === "horizontal")
            return _;
        }
      } else {
        if (isUnbinnedQuantitative(J))
          return _;
        if (isUnbinnedQuantitative(Z))
          return q;
      }
    else {
      if (isUnbinnedQuantitative(J))
        return G && H === "vertical" ? void 0 : _;
      if (isUnbinnedQuantitative(Z))
        return G && H === "horizontal" ? void 0 : q;
    }
  }
  function getDimensionChannel(O) {
    switch (O) {
      case "x":
        return "y";
      case "y":
        return "x";
      case "theta":
        return "radius";
      case "radius":
        return "theta";
    }
  }
  function stack(O, _) {
    var le, ue;
    const H = isMarkDef(O) ? O : { type: O }, V = H.type;
    if (!STACKABLE_MARKS.has(V))
      return null;
    const q = potentialStackedChannel(_, "x", H) || potentialStackedChannel(_, "theta", H);
    if (!q)
      return null;
    const G = _[q], J = isFieldDef(G) ? vgField(G, {}) : void 0, Z = getDimensionChannel(q), te = [], ne = /* @__PURE__ */ new Set();
    if (_[Z]) {
      const de = _[Z], fe = isFieldDef(de) ? vgField(de, {}) : void 0;
      fe && fe !== J && (te.push(Z), ne.add(fe));
    }
    const ee = Z === "x" ? "xOffset" : "yOffset", ae = _[ee], oe = isFieldDef(ae) ? vgField(ae, {}) : void 0;
    oe && oe !== J && (te.push(ee), ne.add(oe));
    const se = NONPOSITION_CHANNELS.reduce((de, fe) => {
      if (fe !== "tooltip" && channelHasField(_, fe)) {
        const he = _[fe];
        for (const me of array$5(he)) {
          const pe = getFieldDef(me);
          if (pe.aggregate)
            continue;
          const ye = vgField(pe, {});
          // if fielddef is a repeat, just include it in the stack by
          (!ye || // otherwise, the field must be different from the groupBy fields.
          !ne.has(ye)) && de.push({ channel: fe, fieldDef: pe });
        }
      }
      return de;
    }, []);
    let ce;
    return G.stack !== void 0 ? isBoolean$3(G.stack) ? ce = G.stack ? "zero" : null : ce = G.stack : STACK_BY_DEFAULT_MARKS.has(V) && (ce = "zero"), !ce || !isStackOffset(ce) || isAggregate$1(_) && se.length === 0 ? null : ((le = G == null ? void 0 : G.scale) != null && le.type && ((ue = G == null ? void 0 : G.scale) == null ? void 0 : ue.type) !== ScaleType.LINEAR && G != null && G.stack && warn(stackNonLinearScale(G.scale.type)), isFieldOrDatumDef(_[getSecondaryRangeChannel(q)]) ? (G.stack !== void 0 && warn(cannotStackRangedMark(q)), null) : (isFieldDef(G) && G.aggregate && !SUM_OPS.has(G.aggregate) && warn(stackNonSummativeAggregate(G.aggregate)), {
      groupbyChannels: te,
      groupbyFields: ne,
      fieldChannel: q,
      impute: G.impute === null ? !1 : isPathMark(V),
      stackBy: se,
      offset: ce
    }));
  }
  function initMarkdef(O, _, H) {
    const V = replaceExprRef(O), q = getMarkPropOrConfig("orient", V, H);
    if (V.orient = orient(V.type, _, q), q !== void 0 && q !== V.orient && warn(orientOverridden(V.orient, q)), V.type === "bar" && V.orient) {
      const te = getMarkPropOrConfig("cornerRadiusEnd", V, H);
      if (te !== void 0) {
        const ne = V.orient === "horizontal" && _.x2 || V.orient === "vertical" && _.y2 ? ["cornerRadius"] : BAR_CORNER_RADIUS_INDEX[V.orient];
        for (const ee of ne)
          V[ee] = te;
        V.cornerRadiusEnd !== void 0 && delete V.cornerRadiusEnd;
      }
    }
    const G = getMarkPropOrConfig("opacity", V, H), J = getMarkPropOrConfig("fillOpacity", V, H);
    return G === void 0 && J === void 0 && (V.opacity = opacity(V.type, _)), getMarkPropOrConfig("cursor", V, H) === void 0 && (V.cursor = cursor(V, _, H)), V;
  }
  function cursor(O, _, H) {
    return _.href || O.href || getMarkPropOrConfig("href", O, H) ? "pointer" : O.cursor;
  }
  function opacity(O, _) {
    if (contains$2([POINT, TICK, CIRCLE, SQUARE], O) && !isAggregate$1(_))
      return 0.7;
  }
  function defaultFilled(O, _, { graticule: H }) {
    if (H)
      return !1;
    const V = getMarkConfig("filled", O, _), q = O.type;
    return getFirstDefined(V, q !== POINT && q !== LINE && q !== RULE);
  }
  function orient(O, _, H) {
    switch (O) {
      case POINT:
      case CIRCLE:
      case SQUARE:
      case TEXT:
      case RECT:
      case IMAGE:
        return;
    }
    const { x: V, y: q, x2: G, y2: J } = _;
    switch (O) {
      case BAR:
        if (isFieldDef(V) && (isBinned(V.bin) || isFieldDef(q) && q.aggregate && !V.aggregate))
          return "vertical";
        if (isFieldDef(q) && (isBinned(q.bin) || isFieldDef(V) && V.aggregate && !q.aggregate))
          return "horizontal";
        if (J || G) {
          if (H)
            return H;
          if (!G)
            return (isFieldDef(V) && V.type === QUANTITATIVE && !isBinning(V.bin) || isNumericDataDef(V)) && isFieldDef(q) && isBinned(q.bin) ? "horizontal" : "vertical";
          if (!J)
            return (isFieldDef(q) && q.type === QUANTITATIVE && !isBinning(q.bin) || isNumericDataDef(q)) && isFieldDef(V) && isBinned(V.bin) ? "vertical" : "horizontal";
        }
      // falls through
      case RULE:
        if (G && !(isFieldDef(V) && isBinned(V.bin)) && J && !(isFieldDef(q) && isBinned(q.bin)))
          return;
      // falls through
      case AREA:
        if (J)
          return isFieldDef(q) && isBinned(q.bin) ? "horizontal" : "vertical";
        if (G)
          return isFieldDef(V) && isBinned(V.bin) ? "vertical" : "horizontal";
        if (O === RULE) {
          if (V && !q)
            return "vertical";
          if (q && !V)
            return "horizontal";
        }
      // falls through
      case LINE:
      case TICK: {
        const Z = isUnbinnedQuantitativeFieldOrDatumDef(V), te = isUnbinnedQuantitativeFieldOrDatumDef(q);
        if (H)
          return H;
        if (Z && !te)
          return O !== "tick" ? "horizontal" : "vertical";
        if (!Z && te)
          return O !== "tick" ? "vertical" : "horizontal";
        if (Z && te)
          return "vertical";
        {
          const ne = isTypedFieldDef(V) && V.type === TEMPORAL, ee = isTypedFieldDef(q) && q.type === TEMPORAL;
          if (ne && !ee)
            return "vertical";
          if (!ne && ee)
            return "horizontal";
        }
        return;
      }
    }
    return "vertical";
  }
  function dropLineAndPoint(O) {
    const { point: _, line: H, ...V } = O;
    return keys$2(V).length > 1 ? V : V.type;
  }
  function dropLineAndPointFromConfig(O) {
    for (const _ of ["line", "area", "rule", "trail"])
      O[_] && (O = {
        ...O,
        // TODO: remove as any
        [_]: omit(O[_], ["point", "line"])
      });
    return O;
  }
  function getPointOverlay(O, _ = {}, H) {
    return O.point === "transparent" ? { opacity: 0 } : O.point ? isObject$2(O.point) ? O.point : {} : O.point !== void 0 ? null : _.point || H.shape ? isObject$2(_.point) ? _.point : {} : void 0;
  }
  function getLineOverlay(O, _ = {}) {
    return O.line ? O.line === !0 ? {} : O.line : O.line !== void 0 ? null : _.line ? _.line === !0 ? {} : _.line : void 0;
  }
  class PathOverlayNormalizer {
    constructor() {
      this.name = "path-overlay";
    }
    hasMatchingType(_, H) {
      if (isUnitSpec(_)) {
        const { mark: V, encoding: q } = _, G = isMarkDef(V) ? V : { type: V };
        switch (G.type) {
          case "line":
          case "rule":
          case "trail":
            return !!getPointOverlay(G, H[G.type], q);
          case "area":
            return (
              // false / null are also included as we want to remove the properties
              !!getPointOverlay(G, H[G.type], q) || !!getLineOverlay(G, H[G.type])
            );
        }
      }
      return !1;
    }
    run(_, H, V) {
      const { config: q } = H, { params: G, projection: J, mark: Z, name: te, encoding: ne, ...ee } = _, ae = normalizeEncoding(ne, q), oe = isMarkDef(Z) ? Z : { type: Z }, se = getPointOverlay(oe, q[oe.type], ae), ce = oe.type === "area" && getLineOverlay(oe, q[oe.type]), le = [
        {
          name: te,
          ...G ? { params: G } : {},
          mark: dropLineAndPoint({
            // TODO: extract this 0.7 to be shared with default opacity for point/tick/...
            ...oe.type === "area" && oe.opacity === void 0 && oe.fillOpacity === void 0 ? { opacity: 0.7 } : {},
            ...oe
          }),
          // drop shape from encoding as this might be used to trigger point overlay
          encoding: omit(ae, ["shape"])
        }
      ], ue = stack(initMarkdef(oe, ae, q), ae);
      let de = ae;
      if (ue) {
        const { fieldChannel: fe, offset: he } = ue;
        de = {
          ...ae,
          [fe]: {
            ...ae[fe],
            ...he ? { stack: he } : {}
          }
        };
      }
      return de = omit(de, ["y2", "x2"]), ce && le.push({
        ...J ? { projection: J } : {},
        mark: {
          type: "line",
          ...pick$1(oe, ["clip", "interpolate", "tension", "tooltip"]),
          ...ce
        },
        encoding: de
      }), se && le.push({
        ...J ? { projection: J } : {},
        mark: {
          type: "point",
          opacity: 1,
          filled: !0,
          ...pick$1(oe, ["clip", "tooltip"]),
          ...se
        },
        encoding: de
      }), V({
        ...ee,
        layer: le
      }, {
        ...H,
        config: dropLineAndPointFromConfig(q)
      });
    }
  }
  function replaceRepeaterInFacet(O, _) {
    return _ ? isFacetMapping(O) ? replaceRepeaterInMapping(O, _) : replaceRepeaterInFieldDef(O, _) : O;
  }
  function replaceRepeaterInEncoding(O, _) {
    return _ ? replaceRepeaterInMapping(O, _) : O;
  }
  function replaceRepeatInProp(O, _, H) {
    const V = _[O];
    if (isRepeatRef(V)) {
      if (V.repeat in H)
        return { ..._, [O]: H[V.repeat] };
      warn(noSuchRepeatedValue(V.repeat));
      return;
    }
    return _;
  }
  function replaceRepeaterInFieldDef(O, _) {
    if (O = replaceRepeatInProp("field", O, _), O !== void 0) {
      if (O === null)
        return null;
      if (isSortableFieldDef(O) && isSortField(O.sort)) {
        const H = replaceRepeatInProp("field", O.sort, _);
        O = {
          ...O,
          ...H ? { sort: H } : {}
        };
      }
      return O;
    }
  }
  function replaceRepeaterInFieldOrDatumDef(O, _) {
    if (isFieldDef(O))
      return replaceRepeaterInFieldDef(O, _);
    {
      const H = replaceRepeatInProp("datum", O, _);
      return H !== O && !H.type && (H.type = "nominal"), H;
    }
  }
  function replaceRepeaterInChannelDef(O, _) {
    if (isFieldOrDatumDef(O)) {
      const H = replaceRepeaterInFieldOrDatumDef(O, _);
      if (H)
        return H;
      if (isConditionalDef(O))
        return { condition: O.condition };
    } else {
      if (hasConditionalFieldOrDatumDef(O)) {
        const H = replaceRepeaterInFieldOrDatumDef(O.condition, _);
        if (H)
          return {
            ...O,
            condition: H
          };
        {
          const { condition: V, ...q } = O;
          return q;
        }
      }
      return O;
    }
  }
  function replaceRepeaterInMapping(O, _) {
    const H = {};
    for (const V in O)
      if (hasProperty(O, V)) {
        const q = O[V];
        if (isArray$2(q))
          H[V] = q.map((G) => replaceRepeaterInChannelDef(G, _)).filter((G) => G);
        else {
          const G = replaceRepeaterInChannelDef(q, _);
          G !== void 0 && (H[V] = G);
        }
      }
    return H;
  }
  class RuleForRangedLineNormalizer {
    constructor() {
      this.name = "RuleForRangedLine";
    }
    hasMatchingType(_) {
      if (isUnitSpec(_)) {
        const { encoding: H, mark: V } = _;
        if (V === "line" || isMarkDef(V) && V.type === "line")
          for (const q of SECONDARY_RANGE_CHANNEL) {
            const G = getMainRangeChannel(q), J = H[G];
            if (H[q] && (isFieldDef(J) && !isBinned(J.bin) || isDatumDef(J)))
              return !0;
          }
      }
      return !1;
    }
    run(_, H, V) {
      const { encoding: q, mark: G } = _;
      return warn(lineWithRange(!!q.x2, !!q.y2)), V({
        ..._,
        mark: isObject$2(G) ? { ...G, type: "rule" } : "rule"
      }, H);
    }
  }
  class CoreNormalizer extends SpecMapper {
    constructor() {
      super(...arguments), this.nonFacetUnitNormalizers = [
        boxPlotNormalizer,
        errorBarNormalizer,
        errorBandNormalizer,
        new PathOverlayNormalizer(),
        new RuleForRangedLineNormalizer()
      ];
    }
    map(_, H) {
      if (isUnitSpec(_)) {
        const V = channelHasField(_.encoding, ROW), q = channelHasField(_.encoding, COLUMN), G = channelHasField(_.encoding, FACET);
        if (V || q || G)
          return this.mapFacetedUnit(_, H);
      }
      return super.map(_, H);
    }
    // This is for normalizing non-facet unit
    mapUnit(_, H) {
      const { parentEncoding: V, parentProjection: q } = H, G = replaceRepeaterInEncoding(_.encoding, H.repeater), J = {
        ..._,
        ..._.name ? { name: [H.repeaterPrefix, _.name].filter((te) => te).join("_") } : {},
        ...G ? { encoding: G } : {}
      };
      if (V || q)
        return this.mapUnitWithParentEncodingOrProjection(J, H);
      const Z = this.mapLayerOrUnit.bind(this);
      for (const te of this.nonFacetUnitNormalizers)
        if (te.hasMatchingType(J, H.config))
          return te.run(J, H, Z);
      return J;
    }
    mapRepeat(_, H) {
      return isLayerRepeatSpec(_) ? this.mapLayerRepeat(_, H) : this.mapNonLayerRepeat(_, H);
    }
    mapLayerRepeat(_, H) {
      const { repeat: V, spec: q, ...G } = _, { row: J, column: Z, layer: te } = V, { repeater: ne = {}, repeaterPrefix: ee = "" } = H;
      return J || Z ? this.mapRepeat({
        ..._,
        repeat: {
          ...J ? { row: J } : {},
          ...Z ? { column: Z } : {}
        },
        spec: {
          repeat: { layer: te },
          spec: q
        }
      }, H) : {
        ...G,
        layer: te.map((ae) => {
          const oe = {
            ...ne,
            layer: ae
          }, se = `${(q.name ? `${q.name}_` : "") + ee}child__layer_${varName(ae)}`, ce = this.mapLayerOrUnit(q, { ...H, repeater: oe, repeaterPrefix: se });
          return ce.name = se, ce;
        })
      };
    }
    mapNonLayerRepeat(_, H) {
      const { repeat: V, spec: q, data: G, ...J } = _;
      !isArray$2(V) && _.columns && (_ = omit(_, ["columns"]), warn(columnsNotSupportByRowCol("repeat")));
      const Z = [], { repeater: te = {}, repeaterPrefix: ne = "" } = H, ee = !isArray$2(V) && V.row || [te ? te.row : null], ae = !isArray$2(V) && V.column || [te ? te.column : null], oe = isArray$2(V) && V || [te ? te.repeat : null];
      for (const ce of oe)
        for (const le of ee)
          for (const ue of ae) {
            const de = {
              repeat: ce,
              row: le,
              column: ue,
              layer: te.layer
            }, fe = (q.name ? `${q.name}_` : "") + ne + "child__" + (isArray$2(V) ? `${varName(ce)}` : (V.row ? `row_${varName(le)}` : "") + (V.column ? `column_${varName(ue)}` : "")), he = this.map(q, { ...H, repeater: de, repeaterPrefix: fe });
            he.name = fe, Z.push(omit(he, ["data"]));
          }
      const se = isArray$2(V) ? _.columns : V.column ? V.column.length : 1;
      return {
        data: q.data ?? G,
        // data from child spec should have precedence
        align: "all",
        ...J,
        columns: se,
        concat: Z
      };
    }
    mapFacet(_, H) {
      const { facet: V } = _;
      return isFacetMapping(V) && _.columns && (_ = omit(_, ["columns"]), warn(columnsNotSupportByRowCol("facet"))), super.mapFacet(_, H);
    }
    mapUnitWithParentEncodingOrProjection(_, H) {
      const { encoding: V, projection: q } = _, { parentEncoding: G, parentProjection: J, config: Z } = H, te = mergeProjection({ parentProjection: J, projection: q }), ne = mergeEncoding({
        parentEncoding: G,
        encoding: replaceRepeaterInEncoding(V, H.repeater)
      });
      return this.mapUnit({
        ..._,
        ...te ? { projection: te } : {},
        ...ne ? { encoding: ne } : {}
      }, { config: Z });
    }
    mapFacetedUnit(_, H) {
      const { row: V, column: q, facet: G, ...J } = _.encoding, { mark: Z, width: te, projection: ne, height: ee, view: ae, params: oe, encoding: se, ...ce } = _, { facetMapping: le, layout: ue } = this.getFacetMappingAndLayout({ row: V, column: q, facet: G }, H), de = replaceRepeaterInEncoding(J, H.repeater);
      return this.mapFacet({
        ...ce,
        ...ue,
        // row / column has higher precedence than facet
        facet: le,
        spec: {
          ...te ? { width: te } : {},
          ...ee ? { height: ee } : {},
          ...ae ? { view: ae } : {},
          ...ne ? { projection: ne } : {},
          mark: Z,
          encoding: de,
          ...oe ? { params: oe } : {}
        }
      }, H);
    }
    getFacetMappingAndLayout(_, H) {
      const { row: V, column: q, facet: G } = _;
      if (V || q) {
        G && warn(facetChannelDropped([...V ? [ROW] : [], ...q ? [COLUMN] : []]));
        const J = {}, Z = {};
        for (const te of [ROW, COLUMN]) {
          const ne = _[te];
          if (ne) {
            const { align: ee, center: ae, spacing: oe, columns: se, ...ce } = ne;
            J[te] = ce;
            for (const le of ["align", "center", "spacing"])
              ne[le] !== void 0 && (Z[le] ?? (Z[le] = {}), Z[le][te] = ne[le]);
          }
        }
        return { facetMapping: J, layout: Z };
      } else {
        const { align: J, center: Z, spacing: te, columns: ne, ...ee } = G;
        return {
          facetMapping: replaceRepeaterInFacet(ee, H.repeater),
          layout: {
            ...J ? { align: J } : {},
            ...Z ? { center: Z } : {},
            ...te ? { spacing: te } : {},
            ...ne ? { columns: ne } : {}
          }
        };
      }
    }
    mapLayer(_, { parentEncoding: H, parentProjection: V, ...q }) {
      const { encoding: G, projection: J, ...Z } = _, te = {
        ...q,
        parentEncoding: mergeEncoding({ parentEncoding: H, encoding: G, layer: !0 }),
        parentProjection: mergeProjection({ parentProjection: V, projection: J })
      };
      return super.mapLayer({
        ...Z,
        ..._.name ? { name: [te.repeaterPrefix, _.name].filter((ne) => ne).join("_") } : {}
      }, te);
    }
  }
  function mergeEncoding({ parentEncoding: O, encoding: _ = {}, layer: H }) {
    let V = {};
    if (O) {
      const q = /* @__PURE__ */ new Set([...keys$2(O), ...keys$2(_)]);
      for (const G of q) {
        const J = _[G], Z = O[G];
        if (isFieldOrDatumDef(J)) {
          const te = {
            ...Z,
            ...J
          };
          V[G] = te;
        } else hasConditionalFieldOrDatumDef(J) ? V[G] = {
          ...J,
          condition: {
            ...Z,
            ...J.condition
          }
        } : J || J === null ? V[G] = J : (H || isValueDef(Z) || isSignalRef(Z) || isFieldOrDatumDef(Z) || isArray$2(Z)) && (V[G] = Z);
      }
    } else
      V = _;
    return !V || isEmpty(V) ? void 0 : V;
  }
  function mergeProjection(O) {
    const { parentProjection: _, projection: H } = O;
    return _ && H && warn(projectionOverridden({ parentProjection: _, projection: H })), H ?? _;
  }
  function isFilter(O) {
    return hasProperty(O, "filter");
  }
  function isImputeSequence(O) {
    return hasProperty(O, "stop");
  }
  function isLookup(O) {
    return hasProperty(O, "lookup");
  }
  function isLookupData(O) {
    return hasProperty(O, "data");
  }
  function isLookupSelection(O) {
    return hasProperty(O, "param");
  }
  function isPivot(O) {
    return hasProperty(O, "pivot");
  }
  function isDensity(O) {
    return hasProperty(O, "density");
  }
  function isQuantile(O) {
    return hasProperty(O, "quantile");
  }
  function isRegression(O) {
    return hasProperty(O, "regression");
  }
  function isLoess(O) {
    return hasProperty(O, "loess");
  }
  function isSample(O) {
    return hasProperty(O, "sample");
  }
  function isWindow(O) {
    return hasProperty(O, "window");
  }
  function isJoinAggregate(O) {
    return hasProperty(O, "joinaggregate");
  }
  function isFlatten(O) {
    return hasProperty(O, "flatten");
  }
  function isCalculate(O) {
    return hasProperty(O, "calculate");
  }
  function isBin(O) {
    return hasProperty(O, "bin");
  }
  function isImpute(O) {
    return hasProperty(O, "impute");
  }
  function isTimeUnit(O) {
    return hasProperty(O, "timeUnit");
  }
  function isAggregate(O) {
    return hasProperty(O, "aggregate");
  }
  function isStack(O) {
    return hasProperty(O, "stack");
  }
  function isFold(O) {
    return hasProperty(O, "fold");
  }
  function isExtent(O) {
    return hasProperty(O, "extent") && !hasProperty(O, "density") && !hasProperty(O, "regression");
  }
  function normalizeTransform(O) {
    return O.map((_) => isFilter(_) ? {
      filter: normalizeLogicalComposition(_.filter, normalizePredicate$1)
    } : _);
  }
  class SelectionCompatibilityNormalizer extends SpecMapper {
    map(_, H) {
      return H.emptySelections ?? (H.emptySelections = {}), H.selectionPredicates ?? (H.selectionPredicates = {}), _ = normalizeTransforms(_, H), super.map(_, H);
    }
    mapLayerOrUnit(_, H) {
      if (_ = normalizeTransforms(_, H), _.encoding) {
        const V = {};
        for (const [q, G] of entries$1(_.encoding))
          V[q] = normalizeChannelDef(G, H);
        _ = { ..._, encoding: V };
      }
      return super.mapLayerOrUnit(_, H);
    }
    mapUnit(_, H) {
      const { selection: V, ...q } = _;
      return V ? {
        ...q,
        params: entries$1(V).map(([G, J]) => {
          const { init: Z, bind: te, empty: ne, ...ee } = J;
          ee.type === "single" ? (ee.type = "point", ee.toggle = !1) : ee.type === "multi" && (ee.type = "point"), H.emptySelections[G] = ne !== "none";
          for (const ae of vals(H.selectionPredicates[G] ?? {}))
            ae.empty = ne !== "none";
          return { name: G, value: Z, select: ee, bind: te };
        })
      } : _;
    }
  }
  function normalizeTransforms(O, _) {
    const { transform: H, ...V } = O;
    if (H) {
      const q = H.map((G) => {
        if (isFilter(G))
          return { filter: normalizePredicate(G, _) };
        if (isBin(G) && isBinParams(G.bin))
          return {
            ...G,
            bin: normalizeBinExtent(G.bin)
          };
        if (isLookup(G)) {
          const { selection: J, ...Z } = G.from;
          return J ? {
            ...G,
            from: { param: J, ...Z }
          } : G;
        }
        return G;
      });
      return { ...V, transform: q };
    }
    return O;
  }
  function normalizeChannelDef(O, _) {
    var V, q;
    const H = duplicate(O);
    if (isFieldDef(H) && isBinParams(H.bin) && (H.bin = normalizeBinExtent(H.bin)), isScaleFieldDef(H) && ((q = (V = H.scale) == null ? void 0 : V.domain) != null && q.selection)) {
      const { selection: G, ...J } = H.scale.domain;
      H.scale.domain = { ...J, ...G ? { param: G } : {} };
    }
    if (isConditionalDef(H))
      if (isArray$2(H.condition))
        H.condition = H.condition.map((G) => {
          const { selection: J, param: Z, test: te, ...ne } = G;
          return Z ? G : { ...ne, test: normalizePredicate(G, _) };
        });
      else {
        const { selection: G, param: J, test: Z, ...te } = normalizeChannelDef(H.condition, _);
        H.condition = J ? H.condition : {
          ...te,
          test: normalizePredicate(H.condition, _)
        };
      }
    return H;
  }
  function normalizeBinExtent(O) {
    const _ = O.extent;
    if (_ != null && _.selection) {
      const { selection: H, ...V } = _;
      return { ...O, extent: { ...V, param: H } };
    }
    return O;
  }
  function normalizePredicate(O, _) {
    const H = (V) => normalizeLogicalComposition(V, (q) => {
      var G;
      const J = _.emptySelections[q] ?? !0, Z = { param: q, empty: J };
      return (G = _.selectionPredicates)[q] ?? (G[q] = []), _.selectionPredicates[q].push(Z), Z;
    });
    return O.selection ? H(O.selection) : normalizeLogicalComposition(O.test || O.filter, (V) => V.selection ? H(V.selection) : V);
  }
  class TopLevelSelectionsNormalizer extends SpecMapper {
    map(_, H) {
      const V = H.selections ?? [];
      if (_.params && !isUnitSpec(_)) {
        const q = [];
        for (const G of _.params)
          isSelectionParameter(G) ? V.push(G) : q.push(G);
        _.params = q;
      }
      return H.selections = V, super.map(_, H);
    }
    mapUnit(_, H) {
      const V = H.selections;
      if (!V || !V.length)
        return _;
      const q = (H.path ?? []).concat(_.name), G = [];
      for (const J of V)
        if (!J.views || !J.views.length)
          G.push(J);
        else
          for (const Z of J.views)
            (isString$1(Z) && (Z === _.name || q.includes(Z)) || isArray$2(Z) && // logic for backwards compatibility with view paths before we had unique names
            // @ts-ignore
            Z.map((te) => q.indexOf(te)).every((te, ne, ee) => te !== -1 && (ne === 0 || te > ee[ne - 1]))) && G.push(J);
      return G.length && (_.params = G), _;
    }
  }
  for (const O of ["mapFacet", "mapRepeat", "mapHConcat", "mapVConcat", "mapLayer"]) {
    const _ = TopLevelSelectionsNormalizer.prototype[O];
    TopLevelSelectionsNormalizer.prototype[O] = function(H, V) {
      return _.call(this, H, addSpecNameToParams(H, V));
    };
  }
  function addSpecNameToParams(O, _) {
    return O.name ? {
      ..._,
      path: (_.path ?? []).concat(O.name)
    } : _;
  }
  function normalize(O, _) {
    _ === void 0 && (_ = initConfig(O.config));
    const H = normalizeGenericSpec(O, _), { width: V, height: q } = O, G = normalizeAutoSize(H, { width: V, height: q, autosize: O.autosize }, _);
    return {
      ...H,
      ...G ? { autosize: G } : {}
    };
  }
  const coreNormalizer = new CoreNormalizer(), selectionCompatNormalizer = new SelectionCompatibilityNormalizer(), topLevelSelectionNormalizer = new TopLevelSelectionsNormalizer();
  function normalizeGenericSpec(O, _ = {}) {
    const H = { config: _ };
    return topLevelSelectionNormalizer.map(coreNormalizer.map(selectionCompatNormalizer.map(O, H), H), H);
  }
  function _normalizeAutoSize(O) {
    return isString$1(O) ? { type: O } : O ?? {};
  }
  function normalizeAutoSize(O, _, H) {
    let { width: V, height: q } = _;
    const G = isUnitSpec(O) || isLayerSpec(O), J = {};
    G ? V == "container" && q == "container" ? (J.type = "fit", J.contains = "padding") : V == "container" ? (J.type = "fit-x", J.contains = "padding") : q == "container" && (J.type = "fit-y", J.contains = "padding") : (V == "container" && (warn(containerSizeNonSingle("width")), V = void 0), q == "container" && (warn(containerSizeNonSingle("height")), q = void 0));
    const Z = {
      type: "pad",
      ...J,
      ...H ? _normalizeAutoSize(H.autosize) : {},
      ..._normalizeAutoSize(O.autosize)
    };
    if (Z.type === "fit" && !G && (warn(FIT_NON_SINGLE), Z.type = "pad"), V == "container" && !(Z.type == "fit" || Z.type == "fit-x") && warn(containerSizeNotCompatibleWithAutosize("width")), q == "container" && !(Z.type == "fit" || Z.type == "fit-y") && warn(containerSizeNotCompatibleWithAutosize("height")), !deepEqual(Z, { type: "pad" }))
      return Z;
  }
  function isFitType(O) {
    return ["fit", "fit-x", "fit-y"].includes(O);
  }
  function getFitType(O) {
    return O ? `fit-${getPositionScaleChannel(O)}` : "fit";
  }
  const TOP_LEVEL_PROPERTIES = [
    "background",
    "padding"
    // We do not include "autosize" here as it is supported by only unit and layer specs and thus need to be normalized
  ];
  function extractTopLevelProperties(O, _) {
    const H = {};
    for (const V of TOP_LEVEL_PROPERTIES)
      O && O[V] !== void 0 && (H[V] = signalRefOrValue(O[V]));
    return _ && (H.params = O.params), H;
  }
  class Split {
    constructor(_ = {}, H = {}) {
      this.explicit = _, this.implicit = H;
    }
    clone() {
      return new Split(duplicate(this.explicit), duplicate(this.implicit));
    }
    combine() {
      return {
        ...this.explicit,
        // Explicit properties comes first
        ...this.implicit
      };
    }
    get(_) {
      return getFirstDefined(this.explicit[_], this.implicit[_]);
    }
    getWithExplicit(_) {
      return this.explicit[_] !== void 0 ? { explicit: !0, value: this.explicit[_] } : this.implicit[_] !== void 0 ? { explicit: !1, value: this.implicit[_] } : { explicit: !1, value: void 0 };
    }
    setWithExplicit(_, { value: H, explicit: V }) {
      H !== void 0 && this.set(_, H, V);
    }
    set(_, H, V) {
      return delete this[V ? "implicit" : "explicit"][_], this[V ? "explicit" : "implicit"][_] = H, this;
    }
    copyKeyFromSplit(_, { explicit: H, implicit: V }) {
      H[_] !== void 0 ? this.set(_, H[_], !0) : V[_] !== void 0 && this.set(_, V[_], !1);
    }
    copyKeyFromObject(_, H) {
      H[_] !== void 0 && this.set(_, H[_], !0);
    }
    /**
     * Merge split object into this split object. Properties from the other split
     * overwrite properties from this split.
     */
    copyAll(_) {
      for (const H of keys$2(_.combine())) {
        const V = _.getWithExplicit(H);
        this.setWithExplicit(H, V);
      }
    }
  }
  function makeExplicit(O) {
    return {
      explicit: !0,
      value: O
    };
  }
  function makeImplicit(O) {
    return {
      explicit: !1,
      value: O
    };
  }
  function tieBreakByComparing(O) {
    return (_, H, V, q) => {
      const G = O(_.value, H.value);
      return G > 0 ? _ : G < 0 ? H : defaultTieBreaker(_, H, V, q);
    };
  }
  function defaultTieBreaker(O, _, H, V) {
    return O.explicit && _.explicit && warn(mergeConflictingProperty(H, V, O.value, _.value)), O;
  }
  function mergeValuesWithExplicit(O, _, H, V, q = defaultTieBreaker) {
    return O === void 0 || O.value === void 0 ? _ : O.explicit && !_.explicit ? O : _.explicit && !O.explicit ? _ : deepEqual(O.value, _.value) ? O : q(O, _, H, V);
  }
  class AncestorParse extends Split {
    constructor(_ = {}, H = {}, V = !1) {
      super(_, H), this.explicit = _, this.implicit = H, this.parseNothing = V;
    }
    clone() {
      const _ = super.clone();
      return _.parseNothing = this.parseNothing, _;
    }
  }
  function isUrlData(O) {
    return hasProperty(O, "url");
  }
  function isInlineData(O) {
    return hasProperty(O, "values");
  }
  function isNamedData(O) {
    return hasProperty(O, "name") && !isUrlData(O) && !isInlineData(O) && !isGenerator(O);
  }
  function isGenerator(O) {
    return O && (isSequenceGenerator(O) || isSphereGenerator(O) || isGraticuleGenerator(O));
  }
  function isSequenceGenerator(O) {
    return hasProperty(O, "sequence");
  }
  function isSphereGenerator(O) {
    return hasProperty(O, "sphere");
  }
  function isGraticuleGenerator(O) {
    return hasProperty(O, "graticule");
  }
  var DataSourceType;
  (function(O) {
    O[O.Raw = 0] = "Raw", O[O.Main = 1] = "Main", O[O.Row = 2] = "Row", O[O.Column = 3] = "Column", O[O.Lookup = 4] = "Lookup", O[O.PreFilterInvalid = 5] = "PreFilterInvalid", O[O.PostFilterInvalid = 6] = "PostFilterInvalid";
  })(DataSourceType || (DataSourceType = {}));
  function getDataSourcesForHandlingInvalidValues({ invalid: O, isPath: _ }) {
    switch (normalizeInvalidDataMode(O, { isPath: _ })) {
      case "filter":
        return {
          marks: "exclude-invalid-values",
          scales: "exclude-invalid-values"
        };
      case "break-paths-show-domains":
        return {
          // Path-based marks use pre-filter data so we know to skip these invalid points in the path.
          // For non-path based marks, we skip by not showing them at all.
          marks: _ ? "include-invalid-values" : "exclude-invalid-values",
          scales: "include-invalid-values"
        };
      case "break-paths-filter-domains":
        return {
          marks: _ ? "include-invalid-values" : "exclude-invalid-values",
          // Unlike 'break-paths-show-domains', 'break-paths-filter-domains' uses post-filter data to feed scale.
          scales: "exclude-invalid-values"
        };
      case "show":
        return {
          marks: "include-invalid-values",
          scales: "include-invalid-values"
        };
    }
  }
  function getScaleDataSourceForHandlingInvalidValues(O) {
    const { marks: _, scales: H } = getDataSourcesForHandlingInvalidValues(O);
    return _ === H ? DataSourceType.Main : H === "include-invalid-values" ? DataSourceType.PreFilterInvalid : DataSourceType.PostFilterInvalid;
  }
  class DataFlowNode {
    constructor(_, H) {
      this.debugName = H, this._children = [], this._parent = null, _ && (this.parent = _);
    }
    /**
     * Clone this node with a deep copy but don't clone links to children or parents.
     */
    clone() {
      throw new Error("Cannot clone node");
    }
    get parent() {
      return this._parent;
    }
    /**
     * Set the parent of the node and also add this node to the parent's children.
     */
    set parent(_) {
      this._parent = _, _ && _.addChild(this);
    }
    get children() {
      return this._children;
    }
    numChildren() {
      return this._children.length;
    }
    addChild(_, H) {
      if (this._children.includes(_)) {
        warn(ADD_SAME_CHILD_TWICE);
        return;
      }
      H !== void 0 ? this._children.splice(H, 0, _) : this._children.push(_);
    }
    removeChild(_) {
      const H = this._children.indexOf(_);
      return this._children.splice(H, 1), H;
    }
    /**
     * Remove node from the dataflow.
     */
    remove() {
      let _ = this._parent.removeChild(this);
      for (const H of this._children)
        H._parent = this._parent, this._parent.addChild(H, _++);
    }
    /**
     * Insert another node as a parent of this node.
     */
    insertAsParentOf(_) {
      const H = _.parent;
      H.removeChild(this), this.parent = H, _.parent = this;
    }
    swapWithParent() {
      const _ = this._parent, H = _.parent;
      for (const q of this._children)
        q.parent = _;
      this._children = [], _.removeChild(this);
      const V = _.parent.removeChild(_);
      this._parent = H, H.addChild(this, V), _.parent = this;
    }
  }
  class OutputNode extends DataFlowNode {
    clone() {
      const _ = new this.constructor();
      return _.debugName = `clone_${this.debugName}`, _._source = this._source, _._name = `clone_${this._name}`, _.type = this.type, _.refCounts = this.refCounts, _.refCounts[_._name] = 0, _;
    }
    /**
     * @param source The name of the source. Will change in assemble.
     * @param type The type of the output node.
     * @param refCounts A global ref counter map.
     */
    constructor(_, H, V, q) {
      super(_, H), this.type = V, this.refCounts = q, this._source = this._name = H, this.refCounts && !(this._name in this.refCounts) && (this.refCounts[this._name] = 0);
    }
    dependentFields() {
      return /* @__PURE__ */ new Set();
    }
    producedFields() {
      return /* @__PURE__ */ new Set();
    }
    hash() {
      return this._hash === void 0 && (this._hash = `Output ${uniqueId()}`), this._hash;
    }
    /**
     * Request the datasource name and increase the ref counter.
     *
     * During the parsing phase, this will return the simple name such as 'main' or 'raw'.
     * It is crucial to request the name from an output node to mark it as a required node.
     * If nobody ever requests the name, this datasource will not be instantiated in the assemble phase.
     *
     * In the assemble phase, this will return the correct name.
     */
    getSource() {
      return this.refCounts[this._name]++, this._source;
    }
    isRequired() {
      return !!this.refCounts[this._name];
    }
    setSource(_) {
      this._source = _;
    }
  }
  function isTimeUnitTransformComponent(O) {
    return O.as !== void 0;
  }
  function offsetAs(O) {
    return `${O}_end`;
  }
  class TimeUnitNode extends DataFlowNode {
    clone() {
      return new TimeUnitNode(null, duplicate(this.timeUnits));
    }
    constructor(_, H) {
      super(_), this.timeUnits = H;
    }
    static makeFromEncoding(_, H) {
      const V = H.reduceFieldDef((q, G, J) => {
        const { field: Z, timeUnit: te } = G;
        if (te) {
          let ne;
          if (isBinnedTimeUnit(te)) {
            if (isUnitModel(H)) {
              const { mark: ee, markDef: ae, config: oe } = H, se = getBandPosition({ fieldDef: G, markDef: ae, config: oe });
              (isRectBasedMark(ee) || se) && (ne = {
                timeUnit: normalizeTimeUnit(te),
                field: Z
              });
            }
          } else
            ne = {
              as: vgField(G, { forAs: !0 }),
              field: Z,
              timeUnit: te
            };
          if (isUnitModel(H)) {
            const { mark: ee, markDef: ae, config: oe } = H, se = getBandPosition({ fieldDef: G, markDef: ae, config: oe });
            isRectBasedMark(ee) && isXorY(J) && se !== 0.5 && (ne.rectBandPosition = se);
          }
          ne && (q[hash(ne)] = ne);
        }
        return q;
      }, {});
      return isEmpty(V) ? null : new TimeUnitNode(_, V);
    }
    static makeFromTransform(_, H) {
      const { timeUnit: V, ...q } = { ...H }, G = normalizeTimeUnit(V), J = {
        ...q,
        timeUnit: G
      };
      return new TimeUnitNode(_, {
        [hash(J)]: J
      });
    }
    /**
     * Merge together TimeUnitNodes assigning the children of `other` to `this`
     * and removing `other`.
     */
    merge(_) {
      this.timeUnits = { ...this.timeUnits };
      for (const H in _.timeUnits)
        this.timeUnits[H] || (this.timeUnits[H] = _.timeUnits[H]);
      for (const H of _.children)
        _.removeChild(H), H.parent = this;
      _.remove();
    }
    /**
     * Remove time units coming from the other node.
     */
    removeFormulas(_) {
      const H = {};
      for (const [V, q] of entries$1(this.timeUnits)) {
        const G = isTimeUnitTransformComponent(q) ? q.as : `${q.field}_end`;
        _.has(G) || (H[V] = q);
      }
      this.timeUnits = H;
    }
    producedFields() {
      return new Set(vals(this.timeUnits).map((_) => isTimeUnitTransformComponent(_) ? _.as : offsetAs(_.field)));
    }
    dependentFields() {
      return new Set(vals(this.timeUnits).map((_) => _.field));
    }
    hash() {
      return `TimeUnit ${hash(this.timeUnits)}`;
    }
    assemble() {
      const _ = [];
      for (const H of vals(this.timeUnits)) {
        const { rectBandPosition: V } = H, q = normalizeTimeUnit(H.timeUnit);
        if (isTimeUnitTransformComponent(H)) {
          const { field: G, as: J } = H, { unit: Z, utc: te, ...ne } = q, ee = [J, `${J}_end`];
          _.push({
            field: replacePathInField(G),
            type: "timeunit",
            ...Z ? { units: getTimeUnitParts(Z) } : {},
            ...te ? { timezone: "utc" } : {},
            ...ne,
            as: ee
          }), _.push(...offsetedRectFormulas(ee, V, q));
        } else if (H) {
          const { field: G } = H, J = G.replaceAll("\\.", "."), Z = offsetExpr({ timeUnit: q, field: J }), te = offsetAs(J);
          _.push({
            type: "formula",
            expr: Z,
            as: te
          }), _.push(...offsetedRectFormulas([J, te], V, q));
        }
      }
      return _;
    }
  }
  const OFFSETTED_RECT_START_SUFFIX = "offsetted_rect_start", OFFSETTED_RECT_END_SUFFIX = "offsetted_rect_end";
  function offsetExpr({ timeUnit: O, field: _, reverse: H }) {
    const { unit: V, utc: q } = O, G = getSmallestTimeUnitPart(V), { part: J, step: Z } = getDateTimePartAndStep(G, O.step);
    return `${q ? "utcOffset" : "timeOffset"}('${J}', ${accessWithDatumToUnescapedPath(_)}, ${H ? -Z : Z})`;
  }
  function offsetedRectFormulas([O, _], H, V) {
    if (H !== void 0 && H !== 0.5) {
      const q = accessWithDatumToUnescapedPath(O), G = accessWithDatumToUnescapedPath(_);
      return [
        {
          type: "formula",
          expr: interpolateExpr([
            offsetExpr({
              timeUnit: V,
              field: O,
              reverse: !0
            }),
            q
          ], H + 0.5),
          as: `${O}_${OFFSETTED_RECT_START_SUFFIX}`
        },
        {
          type: "formula",
          expr: interpolateExpr([q, G], H + 0.5),
          as: `${O}_${OFFSETTED_RECT_END_SUFFIX}`
        }
      ];
    }
    return [];
  }
  function interpolateExpr([O, _], H) {
    return `${1 - H} * ${O} + ${H} * ${_}`;
  }
  const TUPLE_FIELDS = "_tuple_fields";
  class SelectionProjectionComponent {
    constructor(..._) {
      this.items = _, this.hasChannel = {}, this.hasField = {}, this.hasSelectionId = !1;
    }
  }
  const project = {
    defined: () => !0,
    parse: (O, _, H) => {
      const V = _.name, q = _.project ?? (_.project = new SelectionProjectionComponent()), G = {}, J = {}, Z = /* @__PURE__ */ new Set(), te = (ce, le) => {
        const ue = le === "visual" ? ce.channel : ce.field;
        let de = varName(`${V}_${ue}`);
        for (let fe = 1; Z.has(de); fe++)
          de = varName(`${V}_${ue}_${fe}`);
        return Z.add(de), { [le]: de };
      }, ne = _.type, ee = O.config.selection[ne], ae = H.value !== void 0 ? array$5(H.value) : null;
      let { fields: oe, encodings: se } = isObject$2(H.select) ? H.select : {};
      if (!oe && !se && ae) {
        for (const ce of ae)
          if (isObject$2(ce))
            for (const le of keys$2(ce))
              isSingleDefUnitChannel(le) ? (se || (se = [])).push(le) : ne === "interval" ? (warn(INTERVAL_INITIALIZED_WITH_POS), se = ee.encodings) : (oe ?? (oe = [])).push(le);
      }
      !oe && !se && (se = ee.encodings, "fields" in ee && (oe = ee.fields));
      for (const ce of se ?? []) {
        const le = O.fieldDef(ce);
        if (le) {
          let ue = le.field;
          if (le.aggregate) {
            warn(cannotProjectAggregate(ce, le.aggregate));
            continue;
          } else if (!ue) {
            warn(cannotProjectOnChannelWithoutField(ce));
            continue;
          }
          if (le.timeUnit && !isBinnedTimeUnit(le.timeUnit)) {
            ue = O.vgField(ce);
            const de = {
              timeUnit: le.timeUnit,
              as: ue,
              field: le.field
            };
            J[hash(de)] = de;
          }
          if (!G[ue]) {
            const de = ne === "interval" && isScaleChannel(ce) && hasContinuousDomain(O.getScaleComponent(ce).get("type")) ? "R" : le.bin ? "R-RE" : "E", fe = { field: ue, channel: ce, type: de, index: q.items.length };
            fe.signals = { ...te(fe, "data"), ...te(fe, "visual") }, q.items.push(G[ue] = fe), q.hasField[ue] = G[ue], q.hasSelectionId = q.hasSelectionId || ue === SELECTION_ID, isGeoPositionChannel(ce) ? (fe.geoChannel = ce, fe.channel = getPositionChannelFromLatLong(ce), q.hasChannel[fe.channel] = G[ue]) : q.hasChannel[ce] = G[ue];
          }
        } else
          warn(cannotProjectOnChannelWithoutField(ce));
      }
      for (const ce of oe ?? []) {
        if (q.hasField[ce])
          continue;
        const le = { type: "E", field: ce, index: q.items.length };
        le.signals = { ...te(le, "data") }, q.items.push(le), q.hasField[ce] = le, q.hasSelectionId = q.hasSelectionId || ce === SELECTION_ID;
      }
      ae && (_.init = ae.map((ce) => q.items.map((le) => isObject$2(ce) ? ce[le.geoChannel || le.channel] !== void 0 ? ce[le.geoChannel || le.channel] : ce[le.field] : ce))), isEmpty(J) || (q.timeUnit = new TimeUnitNode(null, J));
    },
    signals: (O, _, H) => {
      const V = _.name + TUPLE_FIELDS;
      return H.filter((G) => G.name === V).length > 0 || _.project.hasSelectionId ? H : H.concat({
        name: V,
        value: _.project.items.map(assembleProjection)
      });
    }
  }, CURR = "_curr", ANIM_VALUE = "anim_value", ANIM_CLOCK = "anim_clock", EASED_ANIM_CLOCK = "eased_anim_clock", MIN_EXTENT = "min_extent", MAX_RANGE_EXTENT = "max_range_extent", LAST_TICK = "last_tick_at", IS_PLAYING = "is_playing", THROTTLE = 1 / 60 * 1e3, animationSignals = (O, _) => [
    // timer signals
    {
      name: EASED_ANIM_CLOCK,
      // update: 'easeLinear(anim_clock / max_range_extent) * max_range_extent'
      update: ANIM_CLOCK
      // TODO: replace with above once easing functions are implemented in vega-functions
    },
    // scale signals
    // TODO(jzong): uncomment commented signals below when implementing interpolation
    { name: `${O}_domain`, init: `domain('${_}')` },
    { name: MIN_EXTENT, init: `extent(${O}_domain)[0]` },
    // {name: 'max_extent', init: `extent(${selectionName}_domain)[1]`},
    { name: MAX_RANGE_EXTENT, init: `extent(range('${_}'))[1]` },
    // {name: 't_index', update: `indexof(${selectionName}_domain, anim_value)`},
    { name: ANIM_VALUE, update: `invert('${_}', ${EASED_ANIM_CLOCK})` }
  ], point$1 = {
    defined: (O) => O.type === "point",
    topLevelSignals: (O, _, H) => (isTimerSelection(_) && (H = H.concat([
      {
        name: ANIM_CLOCK,
        init: "0",
        on: [
          {
            events: { type: "timer", throttle: THROTTLE },
            update: `${IS_PLAYING} ? (${ANIM_CLOCK} + (now() - ${LAST_TICK}) > ${MAX_RANGE_EXTENT} ? 0 : ${ANIM_CLOCK} + (now() - ${LAST_TICK})) : ${ANIM_CLOCK}`
          }
        ]
      },
      {
        name: LAST_TICK,
        init: "now()",
        on: [{ events: [{ signal: ANIM_CLOCK }, { signal: IS_PLAYING }], update: "now()" }]
      },
      {
        name: IS_PLAYING,
        init: "true"
      }
    ])), H),
    signals: (O, _, H) => {
      const V = _.name, q = V + TUPLE_FIELDS, G = _.project, J = "(item().isVoronoi ? datum.datum : datum)", Z = vals(O.component.selection ?? {}).reduce((ee, ae) => ae.type === "interval" ? ee.concat(ae.name + BRUSH) : ee, []).map((ee) => `indexof(item().mark.name, '${ee}') < 0`).join(" && "), te = `datum && item().mark.marktype !== 'group' && indexof(item().mark.role, 'legend') < 0${Z ? ` && ${Z}` : ""}`;
      let ne = `unit: ${unitName(O)}, `;
      if (_.project.hasSelectionId)
        ne += `${SELECTION_ID}: ${J}[${$$2(SELECTION_ID)}]`;
      else if (isTimerSelection(_))
        ne += `fields: ${q}, values: [${ANIM_VALUE} ? ${ANIM_VALUE} : ${MIN_EXTENT}]`;
      else {
        const ee = G.items.map((ae) => {
          const oe = O.fieldDef(ae.channel);
          return oe != null && oe.bin ? `[${J}[${$$2(O.vgField(ae.channel, {}))}], ${J}[${$$2(O.vgField(ae.channel, { binSuffix: "end" }))}]]` : `${J}[${$$2(ae.field)}]`;
        }).join(", ");
        ne += `fields: ${q}, values: [${ee}]`;
      }
      if (isTimerSelection(_))
        return H.concat(animationSignals(_.name, O.scaleName(TIME)), [
          {
            name: V + TUPLE,
            on: [
              {
                events: [{ signal: EASED_ANIM_CLOCK }, { signal: ANIM_VALUE }],
                update: `{${ne}}`,
                force: !0
              }
            ]
          }
        ]);
      {
        const ee = _.events;
        return H.concat([
          {
            name: V + TUPLE,
            on: ee ? [
              {
                events: ee,
                update: `${te} ? {${ne}} : null`,
                force: !0
              }
            ] : []
          }
        ]);
      }
    }
  };
  function assembleProjection(O) {
    const { signals: _, hasLegend: H, index: V, ...q } = O;
    return q.field = replacePathInField(q.field), q;
  }
  function assembleInit(O, _ = !0, H = identity$7) {
    if (isArray$2(O)) {
      const V = O.map((q) => assembleInit(q, _, H));
      return _ ? `[${V.join(", ")}]` : V;
    } else if (isDateTime(O))
      return H(_ ? dateTimeToExpr(O) : dateTimeToTimestamp(O));
    return _ ? H(stringify$1(O)) : O;
  }
  function assembleUnitSelectionSignals(O, _) {
    for (const H of vals(O.component.selection ?? {})) {
      const V = H.name;
      let q = `${V}${TUPLE}, ${H.resolve === "global" ? "true" : `{unit: ${unitName(O)}}`}`;
      for (const G of selectionCompilers)
        G.defined(H) && (G.signals && (_ = G.signals(O, H, _)), G.modifyExpr && (q = G.modifyExpr(O, H, q)));
      _.push({
        name: V + MODIFY,
        on: [
          {
            events: { signal: H.name + TUPLE },
            update: `modify(${$$2(H.name + STORE)}, ${q})`
          }
        ]
      });
    }
    return cleanupEmptyOnArray(_);
  }
  function assembleFacetSignals(O, _) {
    if (O.component.selection && keys$2(O.component.selection).length) {
      const H = $$2(O.getName("cell"));
      _.unshift({
        name: "facet",
        value: {},
        on: [
          {
            events: eventSelector("pointermove", "scope"),
            update: `isTuple(facet) ? facet : group(${H}).datum`
          }
        ]
      });
    }
    return cleanupEmptyOnArray(_);
  }
  function assembleTopLevelSignals(O, _) {
    let H = !1;
    for (const V of vals(O.component.selection ?? {})) {
      const q = V.name, G = $$2(q + STORE);
      if (_.filter((Z) => Z.name === q).length === 0) {
        const Z = V.resolve === "global" ? "union" : V.resolve, te = V.type === "point" ? ", true, true)" : ")";
        _.push({
          name: V.name,
          update: `${VL_SELECTION_RESOLVE}(${G}, ${$$2(Z)}${te}`
        });
      }
      H = !0;
      for (const Z of selectionCompilers)
        Z.defined(V) && Z.topLevelSignals && (_ = Z.topLevelSignals(O, V, _));
    }
    return H && _.filter((q) => q.name === "unit").length === 0 && _.unshift({
      name: "unit",
      value: {},
      on: [{ events: "pointermove", update: "isTuple(group()) ? group() : unit" }]
    }), cleanupEmptyOnArray(_);
  }
  function assembleUnitSelectionData(O, _) {
    const H = [], V = [], q = unitName(O, { escape: !1 });
    for (const G of vals(O.component.selection ?? {})) {
      const J = { name: G.name + STORE };
      if (G.project.hasSelectionId && (J.transform = [{ type: "collect", sort: { field: SELECTION_ID } }]), G.init) {
        const te = G.project.items.map(assembleProjection);
        J.values = G.project.hasSelectionId ? G.init.map((ne) => ({ unit: q, [SELECTION_ID]: assembleInit(ne, !1)[0] })) : G.init.map((ne) => ({ unit: q, fields: te, values: assembleInit(ne, !1) }));
      }
      if ([...H, ..._].filter((te) => te.name === G.name + STORE).length || H.push(J), isTimerSelection(G) && _.length) {
        const te = O.lookupDataSource(O.getDataName(DataSourceType.Main)), ne = _.find((ae) => ae.name === te), ee = ne.transform.find((ae) => ae.type === "filter" && ae.expr.includes("vlSelectionTest"));
        if (ee) {
          ne.transform = ne.transform.filter((oe) => oe !== ee);
          const ae = {
            name: ne.name + CURR,
            source: ne.name,
            transform: [ee]
            // add the selection filter to the animation dataset
          };
          V.push(ae);
        }
      }
    }
    return H.concat(_, V);
  }
  function assembleUnitSelectionMarks(O, _) {
    for (const H of vals(O.component.selection ?? {}))
      for (const V of selectionCompilers)
        V.defined(H) && V.marks && (_ = V.marks(O, H, _));
    return _;
  }
  function assembleLayerSelectionMarks(O, _) {
    for (const H of O.children)
      isUnitModel(H) && (_ = assembleUnitSelectionMarks(H, _));
    return _;
  }
  function assembleSelectionScaleDomain(O, _, H, V) {
    const q = parseSelectionExtent(O, _.param, _);
    return {
      signal: hasContinuousDomain(H.get("type")) && isArray$2(V) && V[0] > V[1] ? `isValid(${q}) && reverse(${q})` : q
    };
  }
  function cleanupEmptyOnArray(O) {
    return O.map((_) => (_.on && !_.on.length && delete _.on, _));
  }
  const scaleBindings = {
    defined: (O) => O.type === "interval" && O.resolve === "global" && O.bind && O.bind === "scales",
    parse: (O, _) => {
      const H = _.scales = [];
      for (const V of _.project.items) {
        const q = V.channel;
        if (!isScaleChannel(q))
          continue;
        const G = O.getScaleComponent(q), J = G ? G.get("type") : void 0;
        if (J == "sequential" && warn(SEQUENTIAL_SCALE_DEPRECATED), !G || !hasContinuousDomain(J)) {
          warn(SCALE_BINDINGS_CONTINUOUS);
          continue;
        }
        G.set("selectionExtent", { param: _.name, field: V.field }, !0), H.push(V);
      }
    },
    topLevelSignals: (O, _, H) => {
      const V = _.scales.filter((J) => H.filter((Z) => Z.name === J.signals.data).length === 0);
      if (!O.parent || isTopLevelLayer(O) || V.length === 0)
        return H;
      const q = H.find((J) => J.name === _.name);
      let G = q.update;
      if (G.includes(VL_SELECTION_RESOLVE))
        q.update = `{${V.map((J) => `${$$2(replacePathInField(J.field))}: ${J.signals.data}`).join(", ")}}`;
      else {
        for (const J of V) {
          const Z = `${$$2(replacePathInField(J.field))}: ${J.signals.data}`;
          G.includes(Z) || (G = `${G.substring(0, G.length - 1)}, ${Z}}`);
        }
        q.update = G;
      }
      return H.concat(V.map((J) => ({ name: J.signals.data })));
    },
    signals: (O, _, H) => {
      if (O.parent && !isTopLevelLayer(O))
        for (const V of _.scales) {
          const q = H.find((G) => G.name === V.signals.data);
          q.push = "outer", delete q.value, delete q.update;
        }
      return H;
    }
  };
  function domain(O, _) {
    return `domain(${$$2(O.scaleName(_))})`;
  }
  function isTopLevelLayer(O) {
    return O.parent && isLayerModel(O.parent) && (!O.parent.parent || isTopLevelLayer(O.parent.parent));
  }
  const BRUSH = "_brush", SCALE_TRIGGER = "_scale_trigger", GEO_INIT_TICK = "geo_interval_init_tick", INIT = "_init", CENTER = "_center", interval = {
    defined: (O) => O.type === "interval",
    parse: (O, _, H) => {
      var V;
      if (O.hasProjection) {
        const q = { ...isObject$2(H.select) ? H.select : {} };
        q.fields = [SELECTION_ID], q.encodings || (q.encodings = H.value ? keys$2(H.value) : [LONGITUDE, LATITUDE]), H.select = { type: "interval", ...q };
      }
      if (_.translate && !scaleBindings.defined(_)) {
        const q = `!event.item || event.item.mark.name !== ${$$2(_.name + BRUSH)}`;
        for (const G of _.events) {
          if (!G.between) {
            warn(`${G} is not an ordered event stream for interval selections.`);
            continue;
          }
          const J = array$5((V = G.between[0]).filter ?? (V.filter = []));
          J.includes(q) || J.push(q);
        }
      }
    },
    signals: (O, _, H) => {
      const V = _.name, q = V + TUPLE, G = vals(_.project.hasChannel).filter((Z) => Z.channel === X || Z.channel === Y$1), J = _.init ? _.init[0] : null;
      if (H.push(...G.reduce((Z, te) => Z.concat(channelSignals(O, _, te, J && J[te.index])), [])), O.hasProjection) {
        const Z = $$2(O.projectionName()), te = O.projectionName() + CENTER, { x: ne, y: ee } = _.project.hasChannel, ae = ne && ne.signals.visual, oe = ee && ee.signals.visual, se = ne ? J && J[ne.index] : `${te}[0]`, ce = ee ? J && J[ee.index] : `${te}[1]`, le = (pe) => O.getSizeSignalRef(pe).signal, ue = `[[${ae ? ae + "[0]" : "0"}, ${oe ? oe + "[0]" : "0"}],[${ae ? ae + "[1]" : le("width")}, ${oe ? oe + "[1]" : le("height")}]]`;
        J && (H.unshift({
          name: V + INIT,
          init: `[scale(${Z}, [${ne ? se[0] : se}, ${ee ? ce[0] : ce}]), scale(${Z}, [${ne ? se[1] : se}, ${ee ? ce[1] : ce}])]`
        }), (!ne || !ee) && (H.find((ye) => ye.name === te) || H.unshift({
          name: te,
          update: `invert(${Z}, [${le("width")}/2, ${le("height")}/2])`
        })));
        const de = `intersect(${ue}, {markname: ${$$2(O.getName("marks"))}}, unit.mark)`, fe = `{unit: ${unitName(O)}}`, he = `vlSelectionTuples(${de}, ${fe})`, me = G.map((pe) => pe.signals.visual);
        return H.concat({
          name: q,
          on: [
            {
              events: [
                ...me.length ? [{ signal: me.join(" || ") }] : [],
                ...J ? [{ signal: GEO_INIT_TICK }] : []
              ],
              update: he
            }
          ]
        });
      } else {
        if (!scaleBindings.defined(_)) {
          const ne = V + SCALE_TRIGGER, ee = G.map((ae) => {
            const oe = ae.channel, { data: se, visual: ce } = ae.signals, le = $$2(O.scaleName(oe)), ue = O.getScaleComponent(oe).get("type"), de = hasContinuousDomain(ue) ? "+" : "";
            return `(!isArray(${se}) || (${de}invert(${le}, ${ce})[0] === ${de}${se}[0] && ${de}invert(${le}, ${ce})[1] === ${de}${se}[1]))`;
          });
          ee.length && H.push({
            name: ne,
            value: {},
            on: [
              {
                events: G.map((ae) => ({ scale: O.scaleName(ae.channel) })),
                update: ee.join(" && ") + ` ? ${ne} : {}`
              }
            ]
          });
        }
        const Z = G.map((ne) => ne.signals.data), te = `unit: ${unitName(O)}, fields: ${V + TUPLE_FIELDS}, values`;
        return H.concat({
          name: q,
          ...J ? { init: `{${te}: ${assembleInit(J)}}` } : {},
          ...Z.length ? {
            on: [
              {
                events: [{ signal: Z.join(" || ") }],
                // Prevents double invocation, see https://github.com/vega/vega/issues/1672.
                update: `${Z.join(" && ")} ? {${te}: [${Z}]} : null`
              }
            ]
          } : {}
        });
      }
    },
    topLevelSignals: (O, _, H) => (isUnitModel(O) && O.hasProjection && _.init && (H.filter((q) => q.name === GEO_INIT_TICK).length || H.unshift({
      name: GEO_INIT_TICK,
      value: null,
      on: [
        {
          events: "timer{1}",
          update: `${GEO_INIT_TICK} === null ? {} : ${GEO_INIT_TICK}`
        }
      ]
    })), H),
    marks: (O, _, H) => {
      const V = _.name, { x: q, y: G } = _.project.hasChannel, J = q == null ? void 0 : q.signals.visual, Z = G == null ? void 0 : G.signals.visual, te = `data(${$$2(_.name + STORE)})`;
      if (scaleBindings.defined(_) || !q && !G)
        return H;
      const ne = {
        x: q !== void 0 ? { signal: `${J}[0]` } : { value: 0 },
        y: G !== void 0 ? { signal: `${Z}[0]` } : { value: 0 },
        x2: q !== void 0 ? { signal: `${J}[1]` } : { field: { group: "width" } },
        y2: G !== void 0 ? { signal: `${Z}[1]` } : { field: { group: "height" } }
      };
      if (_.resolve === "global")
        for (const ue of keys$2(ne))
          ne[ue] = [
            {
              test: `${te}.length && ${te}[0].unit === ${unitName(O)}`,
              ...ne[ue]
            },
            { value: 0 }
          ];
      const { fill: ee, fillOpacity: ae, cursor: oe, ...se } = _.mark, ce = keys$2(se).reduce((ue, de) => (ue[de] = [
        {
          test: [
            q !== void 0 && `${J}[0] !== ${J}[1]`,
            G !== void 0 && `${Z}[0] !== ${Z}[1]`
          ].filter((fe) => fe).join(" && "),
          value: se[de]
        },
        { value: null }
      ], ue), {}), le = oe ?? (_.translate ? "move" : null);
      return [
        {
          name: `${V + BRUSH}_bg`,
          type: "rect",
          clip: !0,
          encode: {
            enter: {
              fill: { value: ee },
              fillOpacity: { value: ae }
            },
            update: ne
          }
        },
        ...H,
        {
          name: V + BRUSH,
          type: "rect",
          clip: !0,
          encode: {
            enter: {
              ...le ? { cursor: { value: le } } : {},
              fill: { value: "transparent" }
            },
            update: { ...ne, ...ce }
          }
        }
      ];
    }
  };
  function channelSignals(O, _, H, V) {
    const q = !O.hasProjection, G = H.channel, J = H.signals.visual, Z = $$2(q ? O.scaleName(G) : O.projectionName()), te = (oe) => `scale(${Z}, ${oe})`, ne = O.getSizeSignalRef(G === X ? "width" : "height").signal, ee = `${G}(unit)`, ae = _.events.reduce((oe, se) => [
      ...oe,
      { events: se.between[0], update: `[${ee}, ${ee}]` },
      // Brush Start
      { events: se, update: `[${J}[0], clamp(${ee}, 0, ${ne})]` }
      // Brush End
    ], []);
    if (q) {
      const oe = H.signals.data, se = scaleBindings.defined(_), ce = O.getScaleComponent(G), le = ce ? ce.get("type") : void 0, ue = V ? { init: assembleInit(V, !0, te) } : { value: [] };
      return ae.push({
        events: { signal: _.name + SCALE_TRIGGER },
        update: hasContinuousDomain(le) ? `[${te(`${oe}[0]`)}, ${te(`${oe}[1]`)}]` : "[0, 0]"
      }), se ? [{ name: oe, on: [] }] : [
        { name: J, ...ue, on: ae },
        {
          name: oe,
          ...V ? { init: assembleInit(V) } : {},
          // Cannot be `value` as `init` may require datetime exprs.
          on: [
            {
              events: { signal: J },
              update: `${J}[0] === ${J}[1] ? null : invert(${Z}, ${J})`
            }
          ]
        }
      ];
    } else {
      const oe = G === X ? 0 : 1, se = _.name + INIT, ce = V ? { init: `[${se}[0][${oe}], ${se}[1][${oe}]]` } : { value: [] };
      return [{ name: J, ...ce, on: ae }];
    }
  }
  function wrapCondition({ model: O, channelDef: _, vgChannel: H, invalidValueRef: V, mainRefFn: q }) {
    const G = isConditionalDef(_) && _.condition;
    let J = [];
    G && (J = array$5(G).map((ne) => {
      const ee = q(ne);
      if (isConditionalParameter(ne)) {
        const { param: ae, empty: oe } = ne;
        return { test: parseSelectionPredicate(O, { param: ae, empty: oe }), ...ee };
      } else
        return { test: expression(O, ne.test), ...ee };
    })), V !== void 0 && J.push(V);
    const Z = q(_);
    return Z !== void 0 && J.push(Z), J.length > 1 || J.length === 1 && J[0].test ? { [H]: J } : J.length === 1 ? { [H]: J[0] } : {};
  }
  function text$1(O, _ = "text") {
    const H = O.encoding[_];
    return wrapCondition({
      model: O,
      channelDef: H,
      vgChannel: _,
      mainRefFn: (V) => textRef(V, O.config),
      invalidValueRef: void 0
      // text encoding doesn't have continuous scales and thus can't have invalid values
    });
  }
  function textRef(O, _, H = "datum") {
    if (O) {
      if (isValueDef(O))
        return signalOrValueRef(O.value);
      if (isFieldOrDatumDef(O)) {
        const { format: V, formatType: q } = getFormatMixins(O);
        return formatSignalRef({ fieldOrDatumDef: O, format: V, formatType: q, expr: H, config: _ });
      }
    }
  }
  function tooltip(O, _ = {}) {
    const { encoding: H, markDef: V, config: q, stack: G } = O, J = H.tooltip;
    if (isArray$2(J))
      return { tooltip: tooltipRefForEncoding({ tooltip: J }, G, q, _) };
    {
      const Z = _.reactiveGeom ? "datum.datum" : "datum";
      return wrapCondition({
        model: O,
        channelDef: J,
        vgChannel: "tooltip",
        mainRefFn: (ne) => {
          const ee = textRef(ne, q, Z);
          if (ee)
            return ee;
          if (ne === null)
            return;
          let ae = getMarkPropOrConfig("tooltip", V, q);
          if (ae === !0 && (ae = { content: "encoding" }), isString$1(ae))
            return { value: ae };
          if (isObject$2(ae))
            return isSignalRef(ae) ? ae : ae.content === "encoding" ? tooltipRefForEncoding(H, G, q, _) : { signal: Z };
        },
        invalidValueRef: void 0
        // tooltip encoding doesn't have continuous scales and thus can't have invalid values
      });
    }
  }
  function tooltipData(O, _, H, { reactiveGeom: V } = {}) {
    const q = { ...H, ...H.tooltipFormat }, G = /* @__PURE__ */ new Set(), J = V ? "datum.datum" : "datum", Z = [];
    function te(ee, ae) {
      const oe = getMainRangeChannel(ae), se = isTypedFieldDef(ee) ? ee : {
        ...ee,
        type: O[oe].type
        // for secondary field def, copy type from main channel
      }, ce = se.title || defaultTitle(se, q), le = array$5(ce).join(", ").replaceAll(/"/g, '\\"');
      let ue;
      if (isXorY(ae)) {
        const de = ae === "x" ? "x2" : "y2", fe = getFieldDef(O[de]);
        if (isBinned(se.bin) && fe) {
          const he = vgField(se, { expr: J }), me = vgField(fe, { expr: J }), { format: pe, formatType: ye } = getFormatMixins(se);
          ue = binFormatExpression(he, me, pe, ye, q), G.add(de);
        }
      }
      if ((isXorY(ae) || ae === THETA || ae === RADIUS) && _ && _.fieldChannel === ae && _.offset === "normalize") {
        const { format: de, formatType: fe } = getFormatMixins(se);
        ue = formatSignalRef({
          fieldOrDatumDef: se,
          format: de,
          formatType: fe,
          expr: J,
          config: q,
          normalizeStack: !0
        }).signal;
      }
      ue ?? (ue = textRef(se, q, J).signal), Z.push({ channel: ae, key: le, value: ue });
    }
    forEach$1(O, (ee, ae) => {
      isFieldDef(ee) ? te(ee, ae) : hasConditionalFieldDef(ee) && te(ee.condition, ae);
    });
    const ne = {};
    for (const { channel: ee, key: ae, value: oe } of Z)
      !G.has(ee) && !ne[ae] && (ne[ae] = oe);
    return ne;
  }
  function tooltipRefForEncoding(O, _, H, { reactiveGeom: V } = {}) {
    const q = tooltipData(O, _, H, { reactiveGeom: V }), G = entries$1(q).map(([J, Z]) => `"${J}": ${Z}`);
    return G.length > 0 ? { signal: `{${G.join(", ")}}` } : void 0;
  }
  function aria(O) {
    const { markDef: _, config: H } = O, V = getMarkPropOrConfig("aria", _, H);
    return V === !1 ? {} : {
      ...V ? { aria: V } : {},
      ...ariaRoleDescription(O),
      ...description$1(O)
    };
  }
  function ariaRoleDescription(O) {
    const { mark: _, markDef: H, config: V } = O;
    if (V.aria === !1)
      return {};
    const q = getMarkPropOrConfig("ariaRoleDescription", H, V);
    return q != null ? { ariaRoleDescription: { value: q } } : has$1(VG_MARK_INDEX, _) ? {} : { ariaRoleDescription: { value: _ } };
  }
  function description$1(O) {
    const { encoding: _, markDef: H, config: V, stack: q } = O, G = _.description;
    if (G)
      return wrapCondition({
        model: O,
        channelDef: G,
        vgChannel: "description",
        mainRefFn: (te) => textRef(te, O.config),
        invalidValueRef: void 0
        // aria encoding doesn't have continuous scales and thus can't have invalid values
      });
    const J = getMarkPropOrConfig("description", H, V);
    if (J != null)
      return {
        description: signalOrValueRef(J)
      };
    if (V.aria === !1)
      return {};
    const Z = tooltipData(_, q, V);
    if (!isEmpty(Z))
      return {
        description: {
          signal: entries$1(Z).map(([te, ne], ee) => `"${ee > 0 ? "; " : ""}${te}: " + (${ne})`).join(" + ")
        }
      };
  }
  function nonPosition(O, _, H = {}) {
    const { markDef: V, encoding: q, config: G } = _, { vgChannel: J } = H;
    let { defaultRef: Z, defaultValue: te } = H;
    const ne = q[O];
    Z === void 0 && (te ?? (te = getMarkPropOrConfig(O, V, G, {
      vgChannel: J,
      // If there is no conditonal def, we ignore vgConfig so the output spec is concise.
      // However, if there is a conditional def, we must include vgConfig so the default is respected.
      ignoreVgConfig: !isConditionalDef(ne)
    })), te !== void 0 && (Z = signalOrValueRef(te)));
    const ee = {
      markDef: V,
      config: G,
      scaleName: _.scaleName(O),
      scale: _.getScaleComponent(O)
    }, ae = getConditionalValueRefForIncludingInvalidValue({
      ...ee,
      scaleChannel: O,
      channelDef: ne
    });
    return wrapCondition({
      model: _,
      channelDef: ne,
      vgChannel: J ?? O,
      invalidValueRef: ae,
      mainRefFn: (se) => midPoint({
        ...ee,
        channel: O,
        channelDef: se,
        stack: null,
        // No need to provide stack for non-position as it does not affect mid point
        defaultRef: Z
      })
    });
  }
  function color(O, _ = { filled: void 0 }) {
    const { markDef: H, encoding: V, config: q } = O, { type: G } = H, J = _.filled ?? getMarkPropOrConfig("filled", H, q), Z = contains$2(["bar", "point", "circle", "square", "geoshape"], G) ? "transparent" : void 0, te = getMarkPropOrConfig(J === !0 ? "color" : void 0, H, q, { vgChannel: "fill" }) ?? // need to add this manually as getMarkConfig normally drops config.mark[channel] if vgChannel is specified
    q.mark[J === !0 && "color"] ?? // If there is no fill, always fill symbols, bar, geoshape
    // with transparent fills https://github.com/vega/vega-lite/issues/1316
    Z, ne = getMarkPropOrConfig(J === !1 ? "color" : void 0, H, q, { vgChannel: "stroke" }) ?? // need to add this manually as getMarkConfig normally drops config.mark[channel] if vgChannel is specified
    q.mark[J === !1 && "color"], ee = J ? "fill" : "stroke", ae = {
      ...te ? { fill: signalOrValueRef(te) } : {},
      ...ne ? { stroke: signalOrValueRef(ne) } : {}
    };
    return H.color && (J ? H.fill : H.stroke) && warn(droppingColor("property", { fill: "fill" in H, stroke: "stroke" in H })), {
      ...ae,
      ...nonPosition("color", O, {
        vgChannel: ee,
        defaultValue: J ? te : ne
      }),
      ...nonPosition("fill", O, {
        // if there is encoding.fill, include default fill just in case we have conditional-only fill encoding
        defaultValue: V.fill ? te : void 0
      }),
      ...nonPosition("stroke", O, {
        // if there is encoding.stroke, include default fill just in case we have conditional-only stroke encoding
        defaultValue: V.stroke ? ne : void 0
      })
    };
  }
  function zindex(O) {
    const { encoding: _, mark: H } = O, V = _.order;
    return !isPathMark(H) && isValueDef(V) ? wrapCondition({
      model: O,
      channelDef: V,
      vgChannel: "zindex",
      mainRefFn: (q) => signalOrValueRef(q.value),
      invalidValueRef: void 0
      // zindex encoding doesn't have continuous scales and thus can't have invalid values
    }) : {};
  }
  function positionOffset({ channel: O, markDef: _, encoding: H = {}, model: V, bandPosition: q }) {
    const G = `${O}Offset`, J = _[G], Z = H[G];
    if ((G === "xOffset" || G === "yOffset") && Z)
      return { offsetType: "encoding", offset: midPoint({
        channel: G,
        channelDef: Z,
        markDef: _,
        config: V == null ? void 0 : V.config,
        scaleName: V.scaleName(G),
        scale: V.getScaleComponent(G),
        stack: null,
        defaultRef: signalOrValueRef(J),
        bandPosition: q
      }) };
    const te = _[G];
    return te ? { offsetType: "visual", offset: te } : {};
  }
  function pointPosition(O, _, { defaultPos: H, vgChannel: V }) {
    const { encoding: q, markDef: G, config: J, stack: Z } = _, te = q[O], ne = q[getSecondaryRangeChannel(O)], ee = _.scaleName(O), ae = _.getScaleComponent(O), { offset: oe, offsetType: se } = positionOffset({
      channel: O,
      markDef: G,
      encoding: q,
      model: _,
      bandPosition: 0.5
    }), ce = pointPositionDefaultRef({
      model: _,
      defaultPos: H,
      channel: O,
      scaleName: ee,
      scale: ae
    }), le = !te && isXorY(O) && (q.latitude || q.longitude) ? (
      // use geopoint output if there are lat/long and there is no point position overriding lat/long.
      { field: _.getName(O) }
    ) : positionRef({
      channel: O,
      channelDef: te,
      channel2Def: ne,
      markDef: G,
      config: J,
      scaleName: ee,
      scale: ae,
      stack: Z,
      offset: oe,
      defaultRef: ce,
      bandPosition: se === "encoding" ? 0 : void 0
    });
    return le ? { [V || O]: le } : void 0;
  }
  function positionRef(O) {
    const { channel: _, channelDef: H, scaleName: V, stack: q, offset: G, markDef: J } = O;
    if (isFieldOrDatumDef(H) && q && _ === q.fieldChannel) {
      if (isFieldDef(H)) {
        let Z = H.bandPosition;
        if (Z === void 0 && J.type === "text" && (_ === "radius" || _ === "theta") && (Z = 0.5), Z !== void 0)
          return interpolatedSignalRef({
            scaleName: V,
            fieldOrDatumDef: H,
            // positionRef always have type
            startSuffix: "start",
            bandPosition: Z,
            offset: G
          });
      }
      return valueRefForFieldOrDatumDef(H, V, { suffix: "end" }, { offset: G });
    }
    return midPointRefWithPositionInvalidTest(O);
  }
  function pointPositionDefaultRef({ model: O, defaultPos: _, channel: H, scaleName: V, scale: q }) {
    const { markDef: G, config: J } = O;
    return () => {
      const Z = getMainRangeChannel(H), te = getVgPositionChannel(H), ne = getMarkPropOrConfig(H, G, J, { vgChannel: te });
      if (ne !== void 0)
        return widthHeightValueOrSignalRef(H, ne);
      switch (_) {
        case "zeroOrMin":
          return zeroOrMinOrMaxPosition({ scaleName: V, scale: q, mode: "zeroOrMin", mainChannel: Z, config: J });
        case "zeroOrMax":
          return zeroOrMinOrMaxPosition({
            scaleName: V,
            scale: q,
            mode: { zeroOrMax: { widthSignal: O.width.signal, heightSignal: O.height.signal } },
            mainChannel: Z,
            config: J
          });
        case "mid":
          return { ...O[getSizeChannel(H)], mult: 0.5 };
      }
    };
  }
  function zeroOrMinOrMaxPosition({ mainChannel: O, config: _, ...H }) {
    const V = scaledZeroOrMinOrMax(H), { mode: q } = H;
    if (V)
      return V;
    switch (O) {
      case "radius": {
        if (q === "zeroOrMin")
          return { value: 0 };
        const { widthSignal: G, heightSignal: J } = q.zeroOrMax;
        return {
          signal: `min(${G},${J})/2`
        };
      }
      case "theta":
        return q === "zeroOrMin" ? { value: 0 } : { signal: "2*PI" };
      case "x":
        return q === "zeroOrMin" ? { value: 0 } : { field: { group: "width" } };
      case "y":
        return q === "zeroOrMin" ? { field: { group: "height" } } : { value: 0 };
    }
  }
  const ALIGNED_X_CHANNEL = {
    left: "x",
    center: "xc",
    right: "x2"
  }, BASELINED_Y_CHANNEL = {
    top: "y",
    middle: "yc",
    bottom: "y2"
  };
  function vgAlignedPositionChannel(O, _, H, V = "middle") {
    if (O === "radius" || O === "theta")
      return getVgPositionChannel(O);
    const q = O === "x" ? "align" : "baseline", G = getMarkPropOrConfig(q, _, H);
    let J;
    return isSignalRef(G) ? (warn(rangeMarkAlignmentCannotBeExpression(q)), J = void 0) : J = G, O === "x" ? ALIGNED_X_CHANNEL[J || (V === "top" ? "left" : "center")] : BASELINED_Y_CHANNEL[J || V];
  }
  function pointOrRangePosition(O, _, { defaultPos: H, defaultPos2: V, range: q }) {
    return q ? rangePosition(O, _, { defaultPos: H, defaultPos2: V }) : pointPosition(O, _, { defaultPos: H });
  }
  function rangePosition(O, _, { defaultPos: H, defaultPos2: V }) {
    const { markDef: q, config: G } = _, J = getSecondaryRangeChannel(O), Z = getSizeChannel(O), te = pointPosition2OrSize(_, V, J), ne = te[Z] ? (
      // If there is width/height, we need to position the marks based on the alignment.
      vgAlignedPositionChannel(O, q, G)
    ) : (
      // Otherwise, make sure to apply to the right Vg Channel (for arc mark)
      getVgPositionChannel(O)
    );
    return {
      ...pointPosition(O, _, { defaultPos: H, vgChannel: ne }),
      ...te
    };
  }
  function pointPosition2OrSize(O, _, H) {
    const { encoding: V, mark: q, markDef: G, stack: J, config: Z } = O, te = getMainRangeChannel(H), ne = getSizeChannel(H), ee = getVgPositionChannel(H), ae = V[te], oe = O.scaleName(te), se = O.getScaleComponent(te), { offset: ce } = H in V || H in G ? positionOffset({ channel: H, markDef: G, encoding: V, model: O }) : positionOffset({ channel: te, markDef: G, encoding: V, model: O });
    if (!ae && (H === "x2" || H === "y2") && (V.latitude || V.longitude)) {
      const ue = getSizeChannel(H), de = O.markDef[ue];
      return de != null ? {
        [ue]: { value: de }
      } : {
        [ee]: { field: O.getName(H) }
      };
    }
    const le = position2Ref({
      channel: H,
      channelDef: ae,
      channel2Def: V[H],
      markDef: G,
      config: Z,
      scaleName: oe,
      scale: se,
      stack: J,
      offset: ce,
      defaultRef: void 0
    });
    return le !== void 0 ? { [ee]: le } : position2orSize(H, G) || position2orSize(H, {
      [H]: getMarkStyleConfig(H, G, Z.style),
      [ne]: getMarkStyleConfig(ne, G, Z.style)
    }) || position2orSize(H, Z[q]) || position2orSize(H, Z.mark) || {
      [ee]: pointPositionDefaultRef({
        model: O,
        defaultPos: _,
        channel: H,
        scaleName: oe,
        scale: se
      })()
    };
  }
  function position2Ref({ channel: O, channelDef: _, channel2Def: H, markDef: V, config: q, scaleName: G, scale: J, stack: Z, offset: te, defaultRef: ne }) {
    return isFieldOrDatumDef(_) && Z && // If fieldChannel is X and channel is X2 (or Y and Y2)
    O.charAt(0) === Z.fieldChannel.charAt(0) ? valueRefForFieldOrDatumDef(_, G, { suffix: "start" }, { offset: te }) : midPointRefWithPositionInvalidTest({
      channel: O,
      channelDef: H,
      scaleName: G,
      scale: J,
      stack: Z,
      markDef: V,
      config: q,
      offset: te,
      defaultRef: ne
    });
  }
  function position2orSize(O, _) {
    const H = getSizeChannel(O), V = getVgPositionChannel(O);
    if (_[V] !== void 0)
      return { [V]: widthHeightValueOrSignalRef(O, _[V]) };
    if (_[O] !== void 0)
      return { [V]: widthHeightValueOrSignalRef(O, _[O]) };
    if (_[H]) {
      const q = _[H];
      if (isRelativeBandSize(q))
        warn(relativeBandSizeNotSupported(H));
      else
        return { [H]: widthHeightValueOrSignalRef(O, q) };
    }
  }
  function rectPosition(O, _) {
    const { config: H, encoding: V, markDef: q } = O, G = q.type, J = getSecondaryRangeChannel(_), Z = getSizeChannel(_), te = V[_], ne = V[J], ee = O.getScaleComponent(_), ae = ee ? ee.get("type") : void 0, oe = q.orient, se = V[Z] ?? V.size ?? getMarkPropOrConfig("size", q, H, { vgChannel: Z }), ce = getOffsetChannel(_), le = G === "bar" && (_ === "x" ? oe === "vertical" : oe === "horizontal") || G === "tick" && (_ === "y" ? oe === "vertical" : oe === "horizontal");
    return isFieldDef(te) && (isBinning(te.bin) || isBinned(te.bin) || te.timeUnit && !ne) && !(se && !isRelativeBandSize(se)) && !V[ce] && !hasDiscreteDomain(ae) ? rectBinPosition({
      fieldDef: te,
      fieldDef2: ne,
      channel: _,
      model: O
    }) : (isFieldOrDatumDef(te) && hasDiscreteDomain(ae) || le) && !ne ? positionAndSize(te, _, O) : rangePosition(_, O, { defaultPos: "zeroOrMax", defaultPos2: "zeroOrMin" });
  }
  function defaultSizeRef(O, _, H, V, q, G, J) {
    if (isRelativeBandSize(q))
      if (H) {
        const te = H.get("type");
        if (te === "band") {
          let ne = `bandwidth('${_}')`;
          q.band !== 1 && (ne = `${q.band} * ${ne}`);
          const ee = getMarkConfig("minBandSize", { type: J }, V);
          return { signal: ee ? `max(${signalOrStringValue(ee)}, ${ne})` : ne };
        } else q.band !== 1 && (warn(cannotUseRelativeBandSizeWithNonBandScale(te)), q = void 0);
      } else
        return {
          mult: q.band,
          field: { group: O }
        };
    else {
      if (isSignalRef(q))
        return q;
      if (q)
        return { value: q };
    }
    if (H) {
      const te = H.get("range");
      if (isVgRangeStep(te) && isNumber$2(te.step))
        return { value: te.step - 2 };
    }
    if (!G) {
      const { bandPaddingInner: te, barBandPaddingInner: ne, rectBandPaddingInner: ee, tickBandPaddingInner: ae } = V.scale, oe = getFirstDefined(te, J === "tick" ? ae : J === "bar" ? ne : ee);
      if (isSignalRef(oe))
        return { signal: `(1 - (${oe.signal})) * ${O}` };
      if (isNumber$2(oe))
        return { signal: `${1 - oe} * ${O}` };
    }
    return { value: getViewConfigDiscreteStep(V.view, O) - 2 };
  }
  function positionAndSize(O, _, H) {
    var be, Ce;
    const { markDef: V, encoding: q, config: G, stack: J } = H, Z = V.orient, te = H.scaleName(_), ne = H.getScaleComponent(_), ee = getSizeChannel(_), ae = getSecondaryRangeChannel(_), oe = getOffsetChannel(_), se = H.scaleName(oe), ce = H.getScaleComponent(getOffsetScaleChannel(_)), le = (
      // Always uses size channel for ticks, because tick only calls rectPosition() for the size channel
      V.type === "tick" || // use "size" channel for bars, if there is orient and the channel matches the right orientation
      Z === "horizontal" && _ === "y" || Z === "vertical" && _ === "x"
    );
    let ue;
    (q.size || V.size) && (le ? ue = nonPosition("size", H, {
      vgChannel: ee,
      defaultRef: signalOrValueRef(V.size)
    }) : warn(cannotApplySizeToNonOrientedMark(V.type)));
    const de = !!ue, fe = getBandSize({
      channel: _,
      fieldDef: O,
      markDef: V,
      config: G,
      scaleType: (be = ne || ce) == null ? void 0 : be.get("type"),
      useVlSizeChannel: le
    });
    ue = ue || {
      [ee]: defaultSizeRef(ee, se || te, ce || ne, G, fe, !!O, V.type)
    };
    const he = ((Ce = ne || ce) == null ? void 0 : Ce.get("type")) === "band" && isRelativeBandSize(fe) && !de ? "top" : "middle", me = vgAlignedPositionChannel(_, V, G, he), pe = me === "xc" || me === "yc", { offset: ye, offsetType: ge } = positionOffset({ channel: _, markDef: V, encoding: q, model: H, bandPosition: pe ? 0.5 : 0 }), ve = midPointRefWithPositionInvalidTest({
      channel: _,
      channelDef: O,
      markDef: V,
      config: G,
      scaleName: te,
      scale: ne,
      stack: J,
      offset: ye,
      defaultRef: pointPositionDefaultRef({ model: H, defaultPos: "mid", channel: _, scaleName: te, scale: ne }),
      bandPosition: pe ? ge === "encoding" ? 0 : 0.5 : isSignalRef(fe) ? { signal: `(1-${fe})/2` } : isRelativeBandSize(fe) ? (1 - fe.band) / 2 : 0
    });
    if (ee)
      return { [me]: ve, ...ue };
    {
      const Se = getVgPositionChannel(ae), we = ue[ee], Ee = ye ? { ...we, offset: ye } : we;
      return {
        [me]: ve,
        // posRef might be an array that wraps position invalid test
        [Se]: isArray$2(ve) ? [ve[0], { ...ve[1], offset: Ee }] : {
          ...ve,
          offset: Ee
        }
      };
    }
  }
  function getBinSpacing(O, _, H, V, q, G, J) {
    if (isPolarPositionChannel(O))
      return 0;
    const Z = O === "x" || O === "y2", te = Z ? -_ / 2 : _ / 2;
    if (isSignalRef(H) || isSignalRef(q) || isSignalRef(V) || G) {
      const ne = signalOrStringValue(H), ee = signalOrStringValue(q), ae = signalOrStringValue(V), oe = signalOrStringValue(G), ce = G ? `(${J} < ${oe} ? ${Z ? "" : "-"}0.5 * (${oe} - (${J})) : ${te})` : te, le = ae ? `${ae} + ` : "", ue = ne ? `(${ne} ? -1 : 1) * ` : "", de = ee ? `(${ee} + ${ce})` : ce;
      return {
        signal: le + ue + de
      };
    } else
      return q = q || 0, V + (H ? -q - te : +q + te);
  }
  function rectBinPosition({ fieldDef: O, fieldDef2: _, channel: H, model: V }) {
    var Ce;
    const { config: q, markDef: G, encoding: J } = V, Z = V.getScaleComponent(H), te = V.scaleName(H), ne = Z ? Z.get("type") : void 0, ee = Z.get("reverse"), ae = getBandSize({ channel: H, fieldDef: O, markDef: G, config: q, scaleType: ne }), oe = (Ce = V.component.axes[H]) == null ? void 0 : Ce[0], se = (oe == null ? void 0 : oe.get("translate")) ?? 0.5, ce = isXorY(H) ? getMarkPropOrConfig("binSpacing", G, q) ?? 0 : 0, le = getSecondaryRangeChannel(H), ue = getVgPositionChannel(H), de = getVgPositionChannel(le), fe = getMarkConfig("minBandSize", G, q), { offset: he } = positionOffset({ channel: H, markDef: G, encoding: J, model: V, bandPosition: 0 }), { offset: me } = positionOffset({ channel: le, markDef: G, encoding: J, model: V, bandPosition: 0 }), pe = binSizeExpr({ fieldDef: O, scaleName: te }), ye = getBinSpacing(H, ce, ee, se, he, fe, pe), ge = getBinSpacing(le, ce, ee, se, me ?? he, fe, pe), ve = isSignalRef(ae) ? { signal: `(1-${ae.signal})/2` } : isRelativeBandSize(ae) ? (1 - ae.band) / 2 : 0.5, be = getBandPosition({ fieldDef: O, fieldDef2: _, markDef: G, config: q });
    if (isBinning(O.bin) || O.timeUnit) {
      const Se = O.timeUnit && be !== 0.5;
      return {
        [de]: rectBinRef({
          fieldDef: O,
          scaleName: te,
          bandPosition: ve,
          offset: ge,
          useRectOffsetField: Se
        }),
        [ue]: rectBinRef({
          fieldDef: O,
          scaleName: te,
          bandPosition: isSignalRef(ve) ? { signal: `1-${ve.signal}` } : 1 - ve,
          offset: ye,
          useRectOffsetField: Se
        })
      };
    } else if (isBinned(O.bin)) {
      const Se = valueRefForFieldOrDatumDef(O, te, {}, { offset: ge });
      if (isFieldDef(_))
        return {
          [de]: Se,
          [ue]: valueRefForFieldOrDatumDef(_, te, {}, { offset: ye })
        };
      if (isBinParams(O.bin) && O.bin.step)
        return {
          [de]: Se,
          [ue]: {
            signal: `scale("${te}", ${vgField(O, { expr: "datum" })} + ${O.bin.step})`,
            offset: ye
          }
        };
    }
    warn(channelRequiredForBinned(le));
  }
  function rectBinRef({ fieldDef: O, scaleName: _, bandPosition: H, offset: V, useRectOffsetField: q }) {
    return interpolatedSignalRef({
      scaleName: _,
      fieldOrDatumDef: O,
      bandPosition: H,
      offset: V,
      ...q ? {
        startSuffix: OFFSETTED_RECT_START_SUFFIX,
        endSuffix: OFFSETTED_RECT_END_SUFFIX
      } : {}
    });
  }
  const ALWAYS_IGNORE = /* @__PURE__ */ new Set(["aria", "width", "height"]);
  function baseEncodeEntry(O, _) {
    const { fill: H = void 0, stroke: V = void 0 } = _.color === "include" ? color(O) : {};
    return {
      ...markDefProperties(O.markDef, _),
      ...colorRef("fill", H),
      ...colorRef("stroke", V),
      ...nonPosition("opacity", O),
      ...nonPosition("fillOpacity", O),
      ...nonPosition("strokeOpacity", O),
      ...nonPosition("strokeWidth", O),
      ...nonPosition("strokeDash", O),
      ...zindex(O),
      ...tooltip(O),
      ...text$1(O, "href"),
      ...aria(O)
    };
  }
  function colorRef(O, _) {
    return _ ? { [O]: _ } : {};
  }
  function markDefProperties(O, _) {
    return VG_MARK_CONFIGS.reduce((H, V) => (!ALWAYS_IGNORE.has(V) && hasProperty(O, V) && _[V] !== "ignore" && (H[V] = signalOrValueRef(O[V])), H), {});
  }
  function defined(O) {
    const { config: _, markDef: H } = O, V = /* @__PURE__ */ new Set();
    if (O.forEachFieldDef((q, G) => {
      var ne;
      let J;
      if (!isScaleChannel(G) || !(J = O.getScaleType(G)))
        return;
      const Z = isCountingAggregateOp(q.aggregate), te = getScaleInvalidDataMode({
        scaleChannel: G,
        markDef: H,
        config: _,
        scaleType: J,
        isCountAggregate: Z
      });
      if (shouldBreakPath(te)) {
        const ee = O.vgField(G, { expr: "datum", binSuffix: (ne = O.stack) != null && ne.impute ? "mid" : void 0 });
        ee && V.add(ee);
      }
    }), V.size > 0)
      return { defined: { signal: [...V].map((G) => fieldValidPredicate(G, !0)).join(" && ") } };
  }
  function valueIfDefined(O, _) {
    if (_ !== void 0)
      return { [O]: signalOrValueRef(_) };
  }
  const VORONOI = "voronoi", nearest = {
    defined: (O) => O.type === "point" && O.nearest,
    parse: (O, _) => {
      if (_.events)
        for (const H of _.events)
          H.markname = O.getName(VORONOI);
    },
    marks: (O, _, H) => {
      const { x: V, y: q } = _.project.hasChannel, G = O.mark;
      if (isPathMark(G))
        return warn(nearestNotSupportForContinuous(G)), H;
      const J = {
        name: O.getName(VORONOI),
        type: "path",
        interactive: !0,
        from: { data: O.getName("marks") },
        encode: {
          update: {
            fill: { value: "transparent" },
            strokeWidth: { value: 0.35 },
            stroke: { value: "transparent" },
            isVoronoi: { value: !0 },
            ...tooltip(O, { reactiveGeom: !0 })
          }
        },
        transform: [
          {
            type: "voronoi",
            x: { expr: V || !q ? "datum.datum.x || 0" : "0" },
            y: { expr: q || !V ? "datum.datum.y || 0" : "0" },
            size: [O.getSizeSignalRef("width"), O.getSizeSignalRef("height")]
          }
        ]
      };
      let Z = 0, te = !1;
      return H.forEach((ne, ee) => {
        const ae = ne.name ?? "";
        ae === O.component.mark[0].name ? Z = ee : ae.includes(VORONOI) && (te = !0);
      }), te || H.splice(Z + 1, 0, J), H;
    }
  }, inputBindings = {
    defined: (O) => O.type === "point" && O.resolve === "global" && O.bind && O.bind !== "scales" && !isLegendBinding(O.bind),
    parse: (O, _, H) => disableDirectManipulation(_, H),
    topLevelSignals: (O, _, H) => {
      const V = _.name, q = _.project, G = _.bind, J = _.init && _.init[0], Z = nearest.defined(_) ? "(item().isVoronoi ? datum.datum : datum)" : "datum";
      return q.items.forEach((te, ne) => {
        const ee = varName(`${V}_${te.field}`);
        H.filter((oe) => oe.name === ee).length || H.unshift({
          name: ee,
          ...J ? { init: assembleInit(J[ne]) } : { value: null },
          on: _.events ? [
            {
              events: _.events,
              update: `datum && item().mark.marktype !== 'group' ? ${Z}[${$$2(te.field)}] : null`
            }
          ] : [],
          bind: G[te.field] ?? G[te.channel] ?? G
        });
      }), H;
    },
    signals: (O, _, H) => {
      const V = _.name, q = _.project, G = H.find((ne) => ne.name === V + TUPLE), J = V + TUPLE_FIELDS, Z = q.items.map((ne) => varName(`${V}_${ne.field}`)), te = Z.map((ne) => `${ne} !== null`).join(" && ");
      return Z.length && (G.update = `${te} ? {fields: ${J}, values: [${Z.join(", ")}]} : null`), delete G.value, delete G.on, H;
    }
  }, TOGGLE = "_toggle", toggle = {
    defined: (O) => O.type === "point" && !isTimerSelection(O) && !!O.toggle,
    signals: (O, _, H) => H.concat({
      name: _.name + TOGGLE,
      value: !1,
      on: [{ events: _.events, update: _.toggle }]
    }),
    modifyExpr: (O, _) => {
      const H = _.name + TUPLE, V = _.name + TOGGLE;
      return `${V} ? null : ${H}, ` + (_.resolve === "global" ? `${V} ? null : true, ` : `${V} ? null : {unit: ${unitName(O)}}, `) + `${V} ? ${H} : null`;
    }
  }, clear = {
    defined: (O) => O.clear !== void 0 && O.clear !== !1 && !isTimerSelection(O),
    parse: (O, _) => {
      _.clear && (_.clear = isString$1(_.clear) ? eventSelector(_.clear, "view") : _.clear);
    },
    topLevelSignals: (O, _, H) => {
      if (inputBindings.defined(_))
        for (const V of _.project.items) {
          const q = H.findIndex((G) => G.name === varName(`${_.name}_${V.field}`));
          q !== -1 && H[q].on.push({ events: _.clear, update: "null" });
        }
      return H;
    },
    signals: (O, _, H) => {
      function V(q, G) {
        q !== -1 && H[q].on && H[q].on.push({ events: _.clear, update: G });
      }
      if (_.type === "interval")
        for (const q of _.project.items) {
          const G = H.findIndex((J) => J.name === q.signals.visual);
          if (V(G, "[0, 0]"), G === -1) {
            const J = H.findIndex((Z) => Z.name === q.signals.data);
            V(J, "null");
          }
        }
      else {
        let q = H.findIndex((G) => G.name === _.name + TUPLE);
        V(q, "null"), toggle.defined(_) && (q = H.findIndex((G) => G.name === _.name + TOGGLE), V(q, "false"));
      }
      return H;
    }
  }, legendBindings = {
    defined: (O) => {
      const _ = O.resolve === "global" && O.bind && isLegendBinding(O.bind), H = O.project.items.length === 1 && O.project.items[0].field !== SELECTION_ID;
      return _ && !H && warn(LEGEND_BINDINGS_MUST_HAVE_PROJECTION), _ && H;
    },
    parse: (O, _, H) => {
      const V = duplicate(H);
      if (V.select = isString$1(V.select) ? { type: V.select, toggle: _.toggle } : { ...V.select, toggle: _.toggle }, disableDirectManipulation(_, V), isObject$2(H.select) && (H.select.on || H.select.clear)) {
        const J = 'event.item && indexof(event.item.mark.role, "legend") < 0';
        for (const Z of _.events)
          Z.filter = array$5(Z.filter ?? []), Z.filter.includes(J) || Z.filter.push(J);
      }
      const q = isLegendStreamBinding(_.bind) ? _.bind.legend : "click", G = isString$1(q) ? eventSelector(q, "view") : array$5(q);
      _.bind = { legend: { merge: G } };
    },
    topLevelSignals: (O, _, H) => {
      const V = _.name, q = isLegendStreamBinding(_.bind) && _.bind.legend, G = (J) => (Z) => {
        const te = duplicate(Z);
        return te.markname = J, te;
      };
      for (const J of _.project.items) {
        if (!J.hasLegend)
          continue;
        const Z = `${varName(J.field)}_legend`, te = `${V}_${Z}`;
        if (H.filter((ee) => ee.name === te).length === 0) {
          const ee = q.merge.map(G(`${Z}_symbols`)).concat(q.merge.map(G(`${Z}_labels`))).concat(q.merge.map(G(`${Z}_entries`)));
          H.unshift({
            name: te,
            ..._.init ? {} : { value: null },
            on: [
              // Legend entries do not store values, so we need to walk the scenegraph to the symbol datum.
              {
                events: ee,
                update: "isDefined(datum.value) ? datum.value : item().items[0].items[0].datum.value",
                force: !0
              },
              { events: q.merge, update: `!event.item || !datum ? null : ${te}`, force: !0 }
            ]
          });
        }
      }
      return H;
    },
    signals: (O, _, H) => {
      const V = _.name, q = _.project, G = H.find((oe) => oe.name === V + TUPLE), J = V + TUPLE_FIELDS, Z = q.items.filter((oe) => oe.hasLegend).map((oe) => varName(`${V}_${varName(oe.field)}_legend`)), ne = `${Z.map((oe) => `${oe} !== null`).join(" && ")} ? {fields: ${J}, values: [${Z.join(", ")}]} : null`;
      _.events && Z.length > 0 ? G.on.push({
        events: Z.map((oe) => ({ signal: oe })),
        update: ne
      }) : Z.length > 0 && (G.update = ne, delete G.value, delete G.on);
      const ee = H.find((oe) => oe.name === V + TOGGLE), ae = isLegendStreamBinding(_.bind) && _.bind.legend;
      return ee && (_.events ? ee.on.push({ ...ee.on[0], events: ae }) : ee.on[0].events = ae), H;
    }
  };
  function parseInteractiveLegend(O, _, H) {
    var q;
    const V = (q = O.fieldDef(_)) == null ? void 0 : q.field;
    for (const G of vals(O.component.selection ?? {})) {
      const J = G.project.hasField[V] ?? G.project.hasChannel[_];
      if (J && legendBindings.defined(G)) {
        const Z = H.get("selections") ?? [];
        Z.push(G.name), H.set("selections", Z, !1), J.hasLegend = !0;
      }
    }
  }
  const ANCHOR$1 = "_translate_anchor", DELTA$1 = "_translate_delta", translate = {
    defined: (O) => O.type === "interval" && O.translate,
    signals: (O, _, H) => {
      const V = _.name, q = scaleBindings.defined(_), G = V + ANCHOR$1, { x: J, y: Z } = _.project.hasChannel;
      let te = eventSelector(_.translate, "scope");
      return q || (te = te.map((ne) => (ne.between[0].markname = V + BRUSH, ne))), H.push({
        name: G,
        value: {},
        on: [
          {
            events: te.map((ne) => ne.between[0]),
            update: "{x: x(unit), y: y(unit)" + (J !== void 0 ? `, extent_x: ${q ? domain(O, X) : `slice(${J.signals.visual})`}` : "") + (Z !== void 0 ? `, extent_y: ${q ? domain(O, Y$1) : `slice(${Z.signals.visual})`}` : "") + "}"
          }
        ]
      }, {
        name: V + DELTA$1,
        value: {},
        on: [
          {
            events: te,
            update: `{x: ${G}.x - x(unit), y: ${G}.y - y(unit)}`
          }
        ]
      }), J !== void 0 && onDelta$1(O, _, J, "width", H), Z !== void 0 && onDelta$1(O, _, Z, "height", H), H;
    }
  };
  function onDelta$1(O, _, H, V, q) {
    const G = _.name, J = G + ANCHOR$1, Z = G + DELTA$1, te = H.channel, ne = scaleBindings.defined(_), ee = q.find((pe) => pe.name === H.signals[ne ? "data" : "visual"]), ae = O.getSizeSignalRef(V).signal, oe = O.getScaleComponent(te), se = oe && oe.get("type"), ce = oe && oe.get("reverse"), le = ne ? te === X ? ce ? "" : "-" : ce ? "-" : "" : "", ue = `${J}.extent_${te}`, de = `${le}${Z}.${te} / ${ne ? `${ae}` : `span(${ue})`}`, fe = !ne || !oe ? "panLinear" : se === "log" ? "panLog" : se === "symlog" ? "panSymlog" : se === "pow" ? "panPow" : "panLinear", he = ne ? se === "pow" ? `, ${oe.get("exponent") ?? 1}` : se === "symlog" ? `, ${oe.get("constant") ?? 1}` : "" : "", me = `${fe}(${ue}, ${de}${he})`;
    ee.on.push({
      events: { signal: Z },
      update: ne ? me : `clampRange(${me}, 0, ${ae})`
    });
  }
  const ANCHOR = "_zoom_anchor", DELTA = "_zoom_delta", zoom = {
    defined: (O) => O.type === "interval" && O.zoom,
    signals: (O, _, H) => {
      const V = _.name, q = scaleBindings.defined(_), G = V + DELTA, { x: J, y: Z } = _.project.hasChannel, te = $$2(O.scaleName(X)), ne = $$2(O.scaleName(Y$1));
      let ee = eventSelector(_.zoom, "scope");
      return q || (ee = ee.map((ae) => (ae.markname = V + BRUSH, ae))), H.push({
        name: V + ANCHOR,
        on: [
          {
            events: ee,
            update: q ? "{" + [te ? `x: invert(${te}, x(unit))` : "", ne ? `y: invert(${ne}, y(unit))` : ""].filter((ae) => ae).join(", ") + "}" : "{x: x(unit), y: y(unit)}"
          }
        ]
      }, {
        name: G,
        on: [
          {
            events: ee,
            force: !0,
            update: "pow(1.001, event.deltaY * pow(16, event.deltaMode))"
          }
        ]
      }), J !== void 0 && onDelta(O, _, J, "width", H), Z !== void 0 && onDelta(O, _, Z, "height", H), H;
    }
  };
  function onDelta(O, _, H, V, q) {
    const G = _.name, J = H.channel, Z = scaleBindings.defined(_), te = q.find((fe) => fe.name === H.signals[Z ? "data" : "visual"]), ne = O.getSizeSignalRef(V).signal, ee = O.getScaleComponent(J), ae = ee && ee.get("type"), oe = Z ? domain(O, J) : te.name, se = G + DELTA, ce = `${G}${ANCHOR}.${J}`, le = !Z || !ee ? "zoomLinear" : ae === "log" ? "zoomLog" : ae === "symlog" ? "zoomSymlog" : ae === "pow" ? "zoomPow" : "zoomLinear", ue = Z ? ae === "pow" ? `, ${ee.get("exponent") ?? 1}` : ae === "symlog" ? `, ${ee.get("constant") ?? 1}` : "" : "", de = `${le}(${oe}, ${ce}, ${se}${ue})`;
    te.on.push({
      events: { signal: se },
      update: Z ? de : `clampRange(${de}, 0, ${ne})`
    });
  }
  const STORE = "_store", TUPLE = "_tuple", MODIFY = "_modify", VL_SELECTION_RESOLVE = "vlSelectionResolve", selectionCompilers = [
    point$1,
    interval,
    project,
    toggle,
    // Bindings may disable direct manipulation.
    inputBindings,
    scaleBindings,
    legendBindings,
    clear,
    translate,
    zoom,
    nearest
  ];
  function getFacetModel(O) {
    let _ = O.parent;
    for (; _ && !isFacetModel(_); )
      _ = _.parent;
    return _;
  }
  function unitName(O, { escape: _ } = { escape: !0 }) {
    let H = _ ? $$2(O.name) : O.name;
    const V = getFacetModel(O);
    if (V) {
      const { facet: q } = V;
      for (const G of FACET_CHANNELS)
        q[G] && (H += ` + '__facet_${G}_' + (facet[${$$2(V.vgField(G))}])`);
    }
    return H;
  }
  function requiresSelectionId(O) {
    return vals(O.component.selection ?? {}).reduce((_, H) => _ || H.project.hasSelectionId, !1);
  }
  function disableDirectManipulation(O, _) {
    (isString$1(_.select) || !_.select.on) && delete O.events, (isString$1(_.select) || !_.select.clear) && delete O.clear, (isString$1(_.select) || !_.select.toggle) && delete O.toggle;
  }
  function isTimerSelection(O) {
    var _;
    return (_ = O.events) == null ? void 0 : _.find((H) => "type" in H && H.type === "timer");
  }
  function getName(O) {
    const _ = [];
    return O.type === "Identifier" ? [O.name] : O.type === "Literal" ? [O.value] : (O.type === "MemberExpression" && (_.push(...getName(O.object)), _.push(...getName(O.property))), _);
  }
  function startsWithDatum(O) {
    return O.object.type === "MemberExpression" ? startsWithDatum(O.object) : O.object.name === "datum";
  }
  function getDependentFields(O) {
    const _ = parser$3(O), H = /* @__PURE__ */ new Set();
    return _.visit((V) => {
      V.type === "MemberExpression" && startsWithDatum(V) && H.add(getName(V).slice(1).join("."));
    }), H;
  }
  class FilterNode extends DataFlowNode {
    clone() {
      return new FilterNode(null, this.model, duplicate(this.filter));
    }
    constructor(_, H, V) {
      super(_), this.model = H, this.filter = V, this.expr = expression(this.model, this.filter, this), this._dependentFields = getDependentFields(this.expr);
    }
    dependentFields() {
      return this._dependentFields;
    }
    producedFields() {
      return /* @__PURE__ */ new Set();
    }
    assemble() {
      return {
        type: "filter",
        expr: this.expr
      };
    }
    hash() {
      return `Filter ${this.expr}`;
    }
  }
  function parseUnitSelection(O, _) {
    const H = {}, V = O.config.selection;
    if (!_ || !_.length)
      return H;
    let q = 0;
    for (const G of _) {
      const J = varName(G.name), Z = G.select, te = isString$1(Z) ? Z : Z.type, ne = isObject$2(Z) ? duplicate(Z) : { type: te }, ee = V[te];
      for (const se in ee)
        se === "fields" || se === "encodings" || (se === "mark" && (ne.mark = { ...ee.mark, ...ne.mark }), (ne[se] === void 0 || ne[se] === !0) && (ne[se] = duplicate(ee[se] ?? ne[se])));
      const ae = H[J] = {
        ...ne,
        name: J,
        type: te,
        init: G.value,
        bind: G.bind,
        events: isString$1(ne.on) ? eventSelector(ne.on, "scope") : array$5(duplicate(ne.on))
      };
      if (isTimerSelection(ae) && (q++, q > 1)) {
        delete H[J];
        continue;
      }
      const oe = duplicate(G);
      for (const se of selectionCompilers)
        se.defined(ae) && se.parse && se.parse(O, ae, oe);
    }
    return q > 1 && warn(MULTIPLE_TIMER_ANIMATION_SELECTION), H;
  }
  function parseSelectionPredicate(O, _, H, V = "datum") {
    const q = isString$1(_) ? _ : _.param, G = varName(q), J = $$2(G + STORE);
    let Z;
    try {
      Z = O.getSelectionComponent(G, q);
    } catch {
      return `!!${G}`;
    }
    if (Z.project.timeUnit) {
      const oe = H ?? O.component.data.raw, se = Z.project.timeUnit.clone();
      oe.parent ? se.insertAsParentOf(oe) : oe.parent = se;
    }
    const te = Z.project.hasSelectionId ? "vlSelectionIdTest(" : "vlSelectionTest(", ne = Z.resolve === "global" ? ")" : `, ${$$2(Z.resolve)})`, ee = `${te}${J}, ${V}${ne}`, ae = `length(data(${J}))`;
    return _.empty === !1 ? `${ae} && ${ee}` : `!${ae} || ${ee}`;
  }
  function parseSelectionExtent(O, _, H) {
    const V = varName(_), q = H.encoding;
    let G = H.field, J;
    try {
      J = O.getSelectionComponent(V, _);
    } catch {
      return V;
    }
    if (!q && !G)
      G = J.project.items[0].field, J.project.items.length > 1 && warn(selectionAsScaleDomainWithoutField(G));
    else if (q && !G) {
      const Z = J.project.items.filter((te) => te.channel === q);
      !Z.length || Z.length > 1 ? (G = J.project.items[0].field, warn(selectionAsScaleDomainWrongEncodings(Z, q, H, G))) : G = Z[0].field;
    }
    return `${J.name}[${$$2(replacePathInField(G))}]`;
  }
  function materializeSelections(O, _) {
    for (const [H, V] of entries$1(O.component.selection ?? {})) {
      const q = O.getName(`lookup_${H}`);
      O.component.data.outputNodes[q] = V.materialized = new OutputNode(new FilterNode(_, O, { param: H }), q, DataSourceType.Lookup, O.component.data.outputNodeRefCounts);
    }
  }
  function expression(O, _, H) {
    return logicalExpr(_, (V) => isString$1(V) ? V : isSelectionPredicate(V) ? parseSelectionPredicate(O, V, H) : fieldFilterExpression(V));
  }
  function assembleTitle(O, _) {
    if (O)
      return isArray$2(O) && !isText(O) ? O.map((H) => defaultTitle(H, _)).join(", ") : O;
  }
  function setAxisEncode(O, _, H, V) {
    var q, G;
    O.encode ?? (O.encode = {}), (q = O.encode)[_] ?? (q[_] = {}), (G = O.encode[_]).update ?? (G.update = {}), O.encode[_].update[H] = V;
  }
  function assembleAxis(O, _, H, V = { header: !1 }) {
    var ae, oe;
    const { disable: q, orient: G, scale: J, labelExpr: Z, title: te, zindex: ne, ...ee } = O.combine();
    if (!q) {
      for (const se in ee) {
        const ce = se, le = AXIS_PROPERTY_TYPE[ce], ue = ee[ce];
        if (le && le !== _ && le !== "both")
          delete ee[ce];
        else if (isConditionalAxisValue(ue)) {
          const { condition: de, ...fe } = ue, he = array$5(de), me = CONDITIONAL_AXIS_PROP_INDEX[ce];
          if (me) {
            const { vgProp: pe, part: ye } = me, ge = [
              ...he.map((ve) => {
                const { test: be, ...Ce } = ve;
                return {
                  test: expression(null, be),
                  ...Ce
                };
              }),
              fe
            ];
            setAxisEncode(ee, ye, pe, ge), delete ee[ce];
          } else if (me === null) {
            const pe = {
              signal: he.map((ye) => {
                const { test: ge, ...ve } = ye;
                return `${expression(null, ge)} ? ${exprFromValueRefOrSignalRef(ve)} : `;
              }).join("") + exprFromValueRefOrSignalRef(fe)
            };
            ee[ce] = pe;
          }
        } else if (isSignalRef(ue)) {
          const de = CONDITIONAL_AXIS_PROP_INDEX[ce];
          if (de) {
            const { vgProp: fe, part: he } = de;
            setAxisEncode(ee, he, fe, ue), delete ee[ce];
          }
        }
        contains$2(["labelAlign", "labelBaseline"], ce) && ee[ce] === null && delete ee[ce];
      }
      if (_ === "grid") {
        if (!ee.grid)
          return;
        if (ee.encode) {
          const { grid: se } = ee.encode;
          ee.encode = {
            ...se ? { grid: se } : {}
          }, isEmpty(ee.encode) && delete ee.encode;
        }
        return {
          scale: J,
          orient: G,
          ...ee,
          domain: !1,
          labels: !1,
          aria: !1,
          // always hide grid axis
          // Always set min/maxExtent to 0 to ensure that `config.axis*.minExtent` and `config.axis*.maxExtent`
          // would not affect gridAxis
          maxExtent: 0,
          minExtent: 0,
          ticks: !1,
          zindex: getFirstDefined(ne, 0)
          // put grid behind marks by default
        };
      } else {
        if (!V.header && O.mainExtracted)
          return;
        if (Z !== void 0) {
          let ce = Z;
          (oe = (ae = ee.encode) == null ? void 0 : ae.labels) != null && oe.update && isSignalRef(ee.encode.labels.update.text) && (ce = replaceAll$1(Z, "datum.label", ee.encode.labels.update.text.signal)), setAxisEncode(ee, "labels", "text", { signal: ce });
        }
        if (ee.labelAlign === null && delete ee.labelAlign, ee.encode) {
          for (const ce of AXIS_PARTS)
            O.hasAxisPart(ce) || delete ee.encode[ce];
          isEmpty(ee.encode) && delete ee.encode;
        }
        const se = assembleTitle(te, H);
        return {
          scale: J,
          orient: G,
          grid: !1,
          ...se ? { title: se } : {},
          ...ee,
          ...H.aria === !1 ? { aria: !1 } : {},
          zindex: getFirstDefined(ne, 0)
          // put axis line above marks by default
        };
      }
    }
  }
  function assembleAxisSignals(O) {
    const { axes: _ } = O.component, H = [];
    for (const V of POSITION_SCALE_CHANNELS)
      if (_[V]) {
        for (const q of _[V])
          if (!q.get("disable") && !q.get("gridScale")) {
            const G = V === "x" ? "height" : "width", J = O.getSizeSignalRef(G).signal;
            G !== J && H.push({
              name: G,
              update: J
            });
          }
      }
    return H;
  }
  function assembleAxes(O, _) {
    const { x: H = [], y: V = [] } = O;
    return [
      ...H.map((q) => assembleAxis(q, "grid", _)),
      ...V.map((q) => assembleAxis(q, "grid", _)),
      ...H.map((q) => assembleAxis(q, "main", _)),
      ...V.map((q) => assembleAxis(q, "main", _))
    ].filter((q) => q);
  }
  function getAxisConfigFromConfigTypes(O, _, H, V) {
    return Object.assign.apply(null, [
      {},
      ...O.map((q) => {
        if (q === "axisOrient") {
          const G = H === "x" ? "bottom" : "left", J = _[H === "x" ? "axisBottom" : "axisLeft"] || {}, Z = _[H === "x" ? "axisTop" : "axisRight"] || {}, te = /* @__PURE__ */ new Set([...keys$2(J), ...keys$2(Z)]), ne = {};
          for (const ee of te.values())
            ne[ee] = {
              // orient is surely signal in this case
              signal: `${V.signal} === "${G}" ? ${signalOrStringValue(J[ee])} : ${signalOrStringValue(Z[ee])}`
            };
          return ne;
        }
        return _[q];
      })
    ]);
  }
  function getAxisConfigs(O, _, H, V) {
    const q = _ === "band" ? ["axisDiscrete", "axisBand"] : _ === "point" ? ["axisDiscrete", "axisPoint"] : isQuantitative(_) ? ["axisQuantitative"] : _ === "time" || _ === "utc" ? ["axisTemporal"] : [], G = O === "x" ? "axisX" : "axisY", J = isSignalRef(H) ? "axisOrient" : `axis${titleCase(H)}`, Z = [
      // technically Vega does have axisBand, but if we make another separation here,
      // it will further introduce complexity in the code
      ...q,
      ...q.map((ne) => G + ne.substr(4))
    ], te = ["axis", J, G];
    return {
      vlOnlyAxisConfig: getAxisConfigFromConfigTypes(Z, V, O, H),
      vgAxisConfig: getAxisConfigFromConfigTypes(te, V, O, H),
      axisConfigStyle: getAxisConfigStyle([...te, ...Z], V)
    };
  }
  function getAxisConfigStyle(O, _) {
    var V;
    const H = [{}];
    for (const q of O) {
      let G = (V = _[q]) == null ? void 0 : V.style;
      if (G) {
        G = array$5(G);
        for (const J of G)
          H.push(_.style[J]);
      }
    }
    return Object.assign.apply(null, H);
  }
  function getAxisConfig(O, _, H, V = {}) {
    var G;
    const q = getStyleConfig(O, H, _);
    if (q !== void 0)
      return {
        configFrom: "style",
        configValue: q
      };
    for (const J of ["vlOnlyAxisConfig", "vgAxisConfig", "axisConfigStyle"])
      if (((G = V[J]) == null ? void 0 : G[O]) !== void 0)
        return { configFrom: J, configValue: V[J][O] };
    return {};
  }
  const axisRules = {
    scale: ({ model: O, channel: _ }) => O.scaleName(_),
    format: ({ format: O }) => O,
    // we already calculate this in parse
    formatType: ({ formatType: O }) => O,
    // we already calculate this in parse
    grid: ({ fieldOrDatumDef: O, axis: _, scaleType: H }) => _.grid ?? defaultGrid(H, O),
    gridScale: ({ model: O, channel: _ }) => gridScale(O, _),
    labelAlign: ({ axis: O, labelAngle: _, orient: H, channel: V }) => O.labelAlign || defaultLabelAlign(_, H, V),
    labelAngle: ({ labelAngle: O }) => O,
    // we already calculate this in parse
    labelBaseline: ({ axis: O, labelAngle: _, orient: H, channel: V }) => O.labelBaseline || defaultLabelBaseline(_, H, V),
    labelFlush: ({ axis: O, fieldOrDatumDef: _, channel: H }) => O.labelFlush ?? defaultLabelFlush(_.type, H),
    labelOverlap: ({ axis: O, fieldOrDatumDef: _, scaleType: H }) => O.labelOverlap ?? defaultLabelOverlap$1(_.type, H, isFieldDef(_) && !!_.timeUnit, isFieldDef(_) ? _.sort : void 0),
    // we already calculate orient in parse
    orient: ({ orient: O }) => O,
    // Need to cast until Vega supports signal
    tickCount: ({ channel: O, model: _, axis: H, fieldOrDatumDef: V, scaleType: q }) => {
      const G = O === "x" ? "width" : O === "y" ? "height" : void 0, J = G ? _.getSizeSignalRef(G) : void 0;
      return H.tickCount ?? defaultTickCount({ fieldOrDatumDef: V, scaleType: q, size: J, values: H.values });
    },
    tickMinStep: defaultTickMinStep,
    title: ({ axis: O, model: _, channel: H }) => {
      if (O.title !== void 0)
        return O.title;
      const V = getFieldDefTitle(_, H);
      if (V !== void 0)
        return V;
      const q = _.typedFieldDef(H), G = H === "x" ? "x2" : "y2", J = _.fieldDef(G);
      return mergeTitleFieldDefs(q ? [toFieldDefBase(q)] : [], isFieldDef(J) ? [toFieldDefBase(J)] : []);
    },
    values: ({ axis: O, fieldOrDatumDef: _ }) => values$1(O, _),
    zindex: ({ axis: O, fieldOrDatumDef: _, mark: H }) => O.zindex ?? defaultZindex(H, _)
  };
  function defaultGrid(O, _) {
    return !hasDiscreteDomain(O) && isFieldDef(_) && !isBinning(_ == null ? void 0 : _.bin) && !isBinned(_ == null ? void 0 : _.bin);
  }
  function gridScale(O, _) {
    const H = _ === "x" ? "y" : "x";
    if (O.getScaleComponent(H))
      return O.scaleName(H);
  }
  function getLabelAngle(O, _, H, V, q) {
    const G = _ == null ? void 0 : _.labelAngle;
    if (G !== void 0)
      return isSignalRef(G) ? G : normalizeAngle(G);
    {
      const { configValue: J } = getAxisConfig("labelAngle", V, _ == null ? void 0 : _.style, q);
      return J !== void 0 ? normalizeAngle(J) : H === X && contains$2([NOMINAL, ORDINAL], O.type) && !(isFieldDef(O) && O.timeUnit) ? 270 : void 0;
    }
  }
  function normalizeAngleExpr(O) {
    return `(((${O.signal} % 360) + 360) % 360)`;
  }
  function defaultLabelBaseline(O, _, H, V) {
    if (O !== void 0)
      if (H === "x") {
        if (isSignalRef(O)) {
          const q = normalizeAngleExpr(O), G = isSignalRef(_) ? `(${_.signal} === "top")` : _ === "top";
          return {
            signal: `(45 < ${q} && ${q} < 135) || (225 < ${q} && ${q} < 315) ? "middle" :(${q} <= 45 || 315 <= ${q}) === ${G} ? "bottom" : "top"`
          };
        }
        if (45 < O && O < 135 || 225 < O && O < 315)
          return "middle";
        if (isSignalRef(_)) {
          const q = O <= 45 || 315 <= O ? "===" : "!==";
          return { signal: `${_.signal} ${q} "top" ? "bottom" : "top"` };
        }
        return (O <= 45 || 315 <= O) == (_ === "top") ? "bottom" : "top";
      } else {
        if (isSignalRef(O)) {
          const q = normalizeAngleExpr(O), G = isSignalRef(_) ? `(${_.signal} === "left")` : _ === "left";
          return {
            signal: `${q} <= 45 || 315 <= ${q} || (135 <= ${q} && ${q} <= 225) ? ${V ? '"middle"' : "null"} : (45 <= ${q} && ${q} <= 135) === ${G} ? "top" : "bottom"`
          };
        }
        if (O <= 45 || 315 <= O || 135 <= O && O <= 225)
          return V ? "middle" : null;
        if (isSignalRef(_)) {
          const q = 45 <= O && O <= 135 ? "===" : "!==";
          return { signal: `${_.signal} ${q} "left" ? "top" : "bottom"` };
        }
        return (45 <= O && O <= 135) == (_ === "left") ? "top" : "bottom";
      }
  }
  function defaultLabelAlign(O, _, H) {
    if (O === void 0)
      return;
    const V = H === "x", q = V ? 0 : 90, G = V ? "bottom" : "left";
    if (isSignalRef(O)) {
      const J = normalizeAngleExpr(O), Z = isSignalRef(_) ? `(${_.signal} === "${G}")` : _ === G;
      return {
        signal: `(${q ? `(${J} + 90)` : J} % 180 === 0) ? ${V ? null : '"center"'} :(${q} < ${J} && ${J} < ${180 + q}) === ${Z} ? "left" : "right"`
      };
    }
    if ((O + q) % 180 === 0)
      return V ? null : "center";
    if (isSignalRef(_)) {
      const J = q < O && O < 180 + q ? "===" : "!==";
      return {
        signal: `${`${_.signal} ${J} "${G}"`} ? "left" : "right"`
      };
    }
    return (q < O && O < 180 + q) == (_ === G) ? "left" : "right";
  }
  function defaultLabelFlush(O, _) {
    if (_ === "x" && contains$2(["quantitative", "temporal"], O))
      return !0;
  }
  function defaultLabelOverlap$1(O, _, H, V) {
    if (H && !isObject$2(V) || O !== "nominal" && O !== "ordinal")
      return _ === "log" || _ === "symlog" ? "greedy" : !0;
  }
  function defaultOrient(O) {
    return O === "x" ? "bottom" : "left";
  }
  function defaultTickCount({ fieldOrDatumDef: O, scaleType: _, size: H, values: V }) {
    var q;
    if (!V && !hasDiscreteDomain(_) && _ !== "log") {
      if (isFieldDef(O)) {
        if (isBinning(O.bin))
          return { signal: `ceil(${H.signal}/10)` };
        if (O.timeUnit && contains$2(["month", "hours", "day", "quarter"], (q = normalizeTimeUnit(O.timeUnit)) == null ? void 0 : q.unit))
          return;
      }
      return { signal: `ceil(${H.signal}/40)` };
    }
  }
  function defaultTickMinStep({ format: O, fieldOrDatumDef: _ }) {
    if (O === "d")
      return 1;
    if (isFieldDef(_)) {
      const { timeUnit: H } = _;
      if (H) {
        const V = durationExpr(H);
        if (V)
          return { signal: V };
      }
    }
  }
  function getFieldDefTitle(O, _) {
    const H = _ === "x" ? "x2" : "y2", V = O.fieldDef(_), q = O.fieldDef(H), G = V ? V.title : void 0, J = q ? q.title : void 0;
    if (G && J)
      return mergeTitle(G, J);
    if (G)
      return G;
    if (J)
      return J;
    if (G !== void 0)
      return G;
    if (J !== void 0)
      return J;
  }
  function values$1(O, _) {
    const H = O.values;
    if (isArray$2(H))
      return valueArray(_, H);
    if (isSignalRef(H))
      return H;
  }
  function defaultZindex(O, _) {
    return O === "rect" && isDiscrete(_) ? 1 : 0;
  }
  class CalculateNode extends DataFlowNode {
    clone() {
      return new CalculateNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this._dependentFields = getDependentFields(this.transform.calculate);
    }
    static parseAllForSortIndex(_, H) {
      return H.forEachFieldDef((V, q) => {
        if (isScaleFieldDef(V) && isSortArray(V.sort)) {
          const { field: G, timeUnit: J } = V, Z = V.sort, te = Z.map((ne, ee) => `${fieldFilterExpression({ field: G, timeUnit: J, equal: ne })} ? ${ee} : `).join("") + Z.length;
          _ = new CalculateNode(_, {
            calculate: te,
            as: sortArrayIndexField(V, q, { forAs: !0 })
          });
        }
      }), _;
    }
    producedFields() {
      return /* @__PURE__ */ new Set([this.transform.as]);
    }
    dependentFields() {
      return this._dependentFields;
    }
    assemble() {
      return {
        type: "formula",
        expr: this.transform.calculate,
        as: this.transform.as
      };
    }
    hash() {
      return `Calculate ${hash(this.transform)}`;
    }
  }
  function sortArrayIndexField(O, _, H) {
    return vgField(O, { prefix: _, suffix: "sort_index", ...H });
  }
  function getHeaderChannel(O, _) {
    return contains$2(["top", "bottom"], _) ? "column" : contains$2(["left", "right"], _) || O === "row" ? "row" : "column";
  }
  function getHeaderProperty(O, _, H, V) {
    const q = V === "row" ? H.headerRow : V === "column" ? H.headerColumn : H.headerFacet;
    return getFirstDefined((_ || {})[O], q[O], H.header[O]);
  }
  function getHeaderProperties(O, _, H, V) {
    const q = {};
    for (const G of O) {
      const J = getHeaderProperty(G, _ || {}, H, V);
      J !== void 0 && (q[G] = J);
    }
    return q;
  }
  const HEADER_CHANNELS = ["row", "column"], HEADER_TYPES = ["header", "footer"];
  function assembleTitleGroup(O, _) {
    const H = O.component.layoutHeaders[_].title, V = O.config ? O.config : void 0, q = O.component.layoutHeaders[_].facetFieldDef ? O.component.layoutHeaders[_].facetFieldDef : void 0, { titleAnchor: G, titleAngle: J, titleOrient: Z } = getHeaderProperties(["titleAnchor", "titleAngle", "titleOrient"], q.header, V, _), te = getHeaderChannel(_, Z), ne = normalizeAngle(J);
    return {
      name: `${_}-title`,
      type: "group",
      role: `${te}-title`,
      title: {
        text: H,
        ..._ === "row" ? { orient: "left" } : {},
        style: "guide-title",
        ...defaultHeaderGuideBaseline(ne, te),
        ...defaultHeaderGuideAlign(te, ne, G),
        ...assembleHeaderProperties(V, q, _, HEADER_TITLE_PROPERTIES, HEADER_TITLE_PROPERTIES_MAP)
      }
    };
  }
  function defaultHeaderGuideAlign(O, _, H = "middle") {
    switch (H) {
      case "start":
        return { align: "left" };
      case "end":
        return { align: "right" };
    }
    const V = defaultLabelAlign(_, O === "row" ? "left" : "top", O === "row" ? "y" : "x");
    return V ? { align: V } : {};
  }
  function defaultHeaderGuideBaseline(O, _) {
    const H = defaultLabelBaseline(O, _ === "row" ? "left" : "top", _ === "row" ? "y" : "x", !0);
    return H ? { baseline: H } : {};
  }
  function assembleHeaderGroups(O, _) {
    const H = O.component.layoutHeaders[_], V = [];
    for (const q of HEADER_TYPES)
      if (H[q])
        for (const G of H[q]) {
          const J = assembleHeaderGroup(O, _, q, H, G);
          J != null && V.push(J);
        }
    return V;
  }
  function getSort$1(O, _) {
    const { sort: H } = O;
    return isSortField(H) ? {
      field: vgField(H, { expr: "datum" }),
      order: H.order ?? "ascending"
    } : isArray$2(H) ? {
      field: sortArrayIndexField(O, _, { expr: "datum" }),
      order: "ascending"
    } : {
      field: vgField(O, { expr: "datum" }),
      order: H ?? "ascending"
    };
  }
  function assembleLabelTitle(O, _, H) {
    const { format: V, formatType: q, labelAngle: G, labelAnchor: J, labelOrient: Z, labelExpr: te } = getHeaderProperties(["format", "formatType", "labelAngle", "labelAnchor", "labelOrient", "labelExpr"], O.header, H, _), ne = formatSignalRef({
      fieldOrDatumDef: O,
      format: V,
      formatType: q,
      expr: "parent",
      config: H
    }).signal, ee = getHeaderChannel(_, Z);
    return {
      text: {
        signal: te ? replaceAll$1(replaceAll$1(te, "datum.label", ne), "datum.value", vgField(O, { expr: "parent" })) : ne
      },
      ..._ === "row" ? { orient: "left" } : {},
      style: "guide-label",
      frame: "group",
      ...defaultHeaderGuideBaseline(G, ee),
      ...defaultHeaderGuideAlign(ee, G, J),
      ...assembleHeaderProperties(H, O, _, HEADER_LABEL_PROPERTIES, HEADER_LABEL_PROPERTIES_MAP)
    };
  }
  function assembleHeaderGroup(O, _, H, V, q) {
    if (q) {
      let G = null;
      const { facetFieldDef: J } = V, Z = O.config ? O.config : void 0;
      if (J && q.labels) {
        const { labelOrient: ae } = getHeaderProperties(["labelOrient"], J.header, Z, _);
        (_ === "row" && !contains$2(["top", "bottom"], ae) || _ === "column" && !contains$2(["left", "right"], ae)) && (G = assembleLabelTitle(J, _, Z));
      }
      const te = isFacetModel(O) && !isFacetMapping(O.facet), ne = q.axes, ee = (ne == null ? void 0 : ne.length) > 0;
      if (G || ee) {
        const ae = _ === "row" ? "height" : "width";
        return {
          name: O.getName(`${_}_${H}`),
          type: "group",
          role: `${_}-${H}`,
          ...V.facetFieldDef ? {
            from: { data: O.getName(`${_}_domain`) },
            sort: getSort$1(J, _)
          } : {},
          ...ee && te ? {
            from: { data: O.getName(`facet_domain_${_}`) }
          } : {},
          ...G ? { title: G } : {},
          ...q.sizeSignal ? {
            encode: {
              update: {
                [ae]: q.sizeSignal
              }
            }
          } : {},
          ...ee ? { axes: ne } : {}
        };
      }
    }
    return null;
  }
  const LAYOUT_TITLE_BAND = {
    column: {
      start: 0,
      end: 1
    },
    row: {
      start: 1,
      end: 0
    }
  };
  function getLayoutTitleBand(O, _) {
    return LAYOUT_TITLE_BAND[_][O];
  }
  function assembleLayoutTitleBand(O, _) {
    const H = {};
    for (const V of FACET_CHANNELS) {
      const q = O[V];
      if (q != null && q.facetFieldDef) {
        const { titleAnchor: G, titleOrient: J } = getHeaderProperties(["titleAnchor", "titleOrient"], q.facetFieldDef.header, _, V), Z = getHeaderChannel(V, J), te = getLayoutTitleBand(G, Z);
        te !== void 0 && (H[Z] = te);
      }
    }
    return isEmpty(H) ? void 0 : H;
  }
  function assembleHeaderProperties(O, _, H, V, q) {
    const G = {};
    for (const J of V) {
      if (!q[J])
        continue;
      const Z = getHeaderProperty(J, _ == null ? void 0 : _.header, O, H);
      Z !== void 0 && (G[q[J]] = Z);
    }
    return G;
  }
  function assembleLayoutSignals(O) {
    return [
      ...sizeSignals(O, "width"),
      ...sizeSignals(O, "height"),
      ...sizeSignals(O, "childWidth"),
      ...sizeSignals(O, "childHeight")
    ];
  }
  function sizeSignals(O, _) {
    const H = _ === "width" ? "x" : "y", V = O.component.layoutSize.get(_);
    if (!V || V === "merged")
      return [];
    const q = O.getSizeSignalRef(_).signal;
    if (V === "step") {
      const G = O.getScaleComponent(H);
      if (G) {
        const J = G.get("type"), Z = G.get("range");
        if (hasDiscreteDomain(J) && isVgRangeStep(Z)) {
          const te = O.scaleName(H);
          return isFacetModel(O.parent) && O.parent.component.resolve.scale[H] === "independent" ? [stepSignal(te, Z)] : [
            stepSignal(te, Z),
            {
              name: q,
              update: sizeExpr(te, G, `domain('${te}').length`)
            }
          ];
        }
      }
      throw new Error("layout size is step although width/height is not step.");
    } else if (V == "container") {
      const G = q.endsWith("width"), J = G ? "containerSize()[0]" : "containerSize()[1]", Z = getViewConfigContinuousSize(O.config.view, G ? "width" : "height"), te = `isFinite(${J}) ? ${J} : ${Z}`;
      return [{ name: q, init: te, on: [{ update: te, events: "window:resize" }] }];
    } else
      return [
        {
          name: q,
          value: V
        }
      ];
  }
  function stepSignal(O, _) {
    const H = `${O}_step`;
    return isSignalRef(_.step) ? { name: H, update: _.step.signal } : { name: H, value: _.step };
  }
  function sizeExpr(O, _, H) {
    const V = _.get("type"), q = _.get("padding"), G = getFirstDefined(_.get("paddingOuter"), q);
    let J = _.get("paddingInner");
    return J = V === "band" ? (
      // only band has real paddingInner
      J !== void 0 ? J : q
    ) : (
      // For point, as calculated in https://github.com/vega/vega-scale/blob/master/src/band.js#L128,
      // it's equivalent to have paddingInner = 1 since there is only n-1 steps between n points.
      1
    ), `bandspace(${H}, ${signalOrStringValue(J)}, ${signalOrStringValue(G)}) * ${O}_step`;
  }
  function getSizeTypeFromLayoutSizeType(O) {
    return O === "childWidth" ? "width" : O === "childHeight" ? "height" : O;
  }
  function guideEncodeEntry(O, _) {
    return keys$2(O).reduce((H, V) => ({
      ...H,
      ...wrapCondition({
        model: _,
        channelDef: O[V],
        vgChannel: V,
        mainRefFn: (q) => signalOrValueRef(q.value),
        invalidValueRef: void 0
        // guide encoding won't show invalid values for the scale
      })
    }), {});
  }
  function defaultScaleResolve(O, _) {
    if (isFacetModel(_))
      return O === "theta" ? "independent" : "shared";
    if (isLayerModel(_))
      return "shared";
    if (isConcatModel(_))
      return isXorY(O) || O === "theta" || O === "radius" ? "independent" : "shared";
    throw new Error("invalid model type for resolve");
  }
  function parseGuideResolve(O, _) {
    const H = O.scale[_], V = isXorY(_) ? "axis" : "legend";
    return H === "independent" ? (O[V][_] === "shared" && warn(independentScaleMeansIndependentGuide(_)), "independent") : O[V][_] || "shared";
  }
  const LEGEND_COMPONENT_PROPERTY_INDEX = {
    ...COMMON_LEGEND_PROPERTY_INDEX,
    disable: 1,
    labelExpr: 1,
    selections: 1,
    // channel scales
    opacity: 1,
    shape: 1,
    stroke: 1,
    fill: 1,
    size: 1,
    strokeWidth: 1,
    strokeDash: 1,
    // encode
    encode: 1
  }, LEGEND_COMPONENT_PROPERTIES = keys$2(LEGEND_COMPONENT_PROPERTY_INDEX);
  class LegendComponent extends Split {
  }
  const legendEncodeRules = {
    symbols,
    gradient,
    labels: labels$1,
    entries
  };
  function symbols(O, { fieldOrDatumDef: _, model: H, channel: V, legendCmpt: q, legendType: G }) {
    if (G !== "symbol")
      return;
    const { markDef: J, encoding: Z, config: te, mark: ne } = H, ee = J.filled && ne !== "trail";
    let ae = {
      ...applyMarkConfig({}, H, FILL_STROKE_CONFIG),
      ...color(H, { filled: ee })
    };
    const oe = q.get("symbolOpacity") ?? te.legend.symbolOpacity, se = q.get("symbolFillColor") ?? te.legend.symbolFillColor, ce = q.get("symbolStrokeColor") ?? te.legend.symbolStrokeColor, le = oe === void 0 ? getMaxValue(Z.opacity) ?? J.opacity : void 0;
    if (ae.fill) {
      if (V === "fill" || ee && V === COLOR)
        delete ae.fill;
      else if (hasProperty(ae.fill, "field"))
        se ? delete ae.fill : (ae.fill = signalOrValueRef(te.legend.symbolBaseFillColor ?? "black"), ae.fillOpacity = signalOrValueRef(le ?? 1));
      else if (isArray$2(ae.fill)) {
        const ue = getFirstConditionValue(Z.fill ?? Z.color) ?? J.fill ?? (ee && J.color);
        ue && (ae.fill = signalOrValueRef(ue));
      }
    }
    if (ae.stroke) {
      if (V === "stroke" || !ee && V === COLOR)
        delete ae.stroke;
      else if (hasProperty(ae.stroke, "field") || ce)
        delete ae.stroke;
      else if (isArray$2(ae.stroke)) {
        const ue = getFirstDefined(getFirstConditionValue(Z.stroke || Z.color), J.stroke, ee ? J.color : void 0);
        ue && (ae.stroke = { value: ue });
      }
    }
    if (V !== OPACITY) {
      const ue = isFieldDef(_) && selectedCondition(H, q, _);
      ue ? ae.opacity = [
        { test: ue, ...signalOrValueRef(le ?? 1) },
        signalOrValueRef(te.legend.unselectedOpacity)
      ] : le && (ae.opacity = signalOrValueRef(le));
    }
    return ae = { ...ae, ...O }, isEmpty(ae) ? void 0 : ae;
  }
  function gradient(O, { model: _, legendType: H, legendCmpt: V }) {
    if (H !== "gradient")
      return;
    const { config: q, markDef: G, encoding: J } = _;
    let Z = {};
    const ne = (V.get("gradientOpacity") ?? q.legend.gradientOpacity) === void 0 ? getMaxValue(J.opacity) || G.opacity : void 0;
    return ne && (Z.opacity = signalOrValueRef(ne)), Z = { ...Z, ...O }, isEmpty(Z) ? void 0 : Z;
  }
  function labels$1(O, { fieldOrDatumDef: _, model: H, channel: V, legendCmpt: q }) {
    const G = H.legend(V) || {}, J = H.config, Z = isFieldDef(_) ? selectedCondition(H, q, _) : void 0, te = Z ? [{ test: Z, value: 1 }, { value: J.legend.unselectedOpacity }] : void 0, { format: ne, formatType: ee } = G;
    let ae;
    isCustomFormatType(ee) ? ae = formatCustomType({
      fieldOrDatumDef: _,
      field: "datum.value",
      format: ne,
      formatType: ee,
      config: J
    }) : ne === void 0 && ee === void 0 && J.customFormatTypes && (_.type === "quantitative" && J.numberFormatType ? ae = formatCustomType({
      fieldOrDatumDef: _,
      field: "datum.value",
      format: J.numberFormat,
      formatType: J.numberFormatType,
      config: J
    }) : _.type === "temporal" && J.timeFormatType && isFieldDef(_) && _.timeUnit === void 0 && (ae = formatCustomType({
      fieldOrDatumDef: _,
      field: "datum.value",
      format: J.timeFormat,
      formatType: J.timeFormatType,
      config: J
    })));
    const oe = {
      ...te ? { opacity: te } : {},
      ...ae ? { text: ae } : {},
      ...O
    };
    return isEmpty(oe) ? void 0 : oe;
  }
  function entries(O, { legendCmpt: _ }) {
    const H = _.get("selections");
    return H != null && H.length ? { ...O, fill: { value: "transparent" } } : O;
  }
  function getMaxValue(O) {
    return getConditionValue(O, (_, H) => Math.max(_, H.value));
  }
  function getFirstConditionValue(O) {
    return getConditionValue(O, (_, H) => getFirstDefined(_, H.value));
  }
  function getConditionValue(O, _) {
    if (hasConditionalValueDef(O))
      return array$5(O.condition).reduce(_, O.value);
    if (isValueDef(O))
      return O.value;
  }
  function selectedCondition(O, _, H) {
    const V = _.get("selections");
    if (!(V != null && V.length))
      return;
    const q = $$2(H.field);
    return V.map((G) => `(!length(data(${$$2(varName(G) + STORE)})) || (${G}[${q}] && indexof(${G}[${q}], datum.value) >= 0))`).join(" || ");
  }
  const legendRules = {
    direction: ({ direction: O }) => O,
    format: ({ fieldOrDatumDef: O, legend: _, config: H }) => {
      const { format: V, formatType: q } = _;
      return guideFormat(O, O.type, V, q, H, !1);
    },
    formatType: ({ legend: O, fieldOrDatumDef: _, scaleType: H }) => {
      const { formatType: V } = O;
      return guideFormatType(V, _, H);
    },
    gradientLength: (O) => {
      const { legend: _, legendConfig: H } = O;
      return _.gradientLength ?? H.gradientLength ?? defaultGradientLength(O);
    },
    labelOverlap: ({ legend: O, legendConfig: _, scaleType: H }) => O.labelOverlap ?? _.labelOverlap ?? defaultLabelOverlap(H),
    symbolType: ({ legend: O, markDef: _, channel: H, encoding: V }) => O.symbolType ?? defaultSymbolType(_.type, H, V.shape, _.shape),
    title: ({ fieldOrDatumDef: O, config: _ }) => title(O, _, { allowDisabling: !0 }),
    type: ({ legendType: O, scaleType: _, channel: H }) => {
      if (isColorChannel(H) && isContinuousToContinuous(_)) {
        if (O === "gradient")
          return;
      } else if (O === "symbol")
        return;
      return O;
    },
    // depended by other property, let's define upfront
    values: ({ fieldOrDatumDef: O, legend: _ }) => values(_, O)
  };
  function values(O, _) {
    const H = O.values;
    if (isArray$2(H))
      return valueArray(_, H);
    if (isSignalRef(H))
      return H;
  }
  function defaultSymbolType(O, _, H, V) {
    if (_ !== "shape") {
      const q = getFirstConditionValue(H) ?? V;
      if (q)
        return q;
    }
    switch (O) {
      case "bar":
      case "rect":
      case "image":
      case "square":
        return "square";
      case "line":
      case "trail":
      case "rule":
        return "stroke";
      case "arc":
      case "point":
      case "circle":
      case "tick":
      case "geoshape":
      case "area":
      case "text":
        return "circle";
    }
  }
  function getLegendType(O) {
    const { legend: _ } = O;
    return getFirstDefined(_.type, defaultType$2(O));
  }
  function defaultType$2({ channel: O, timeUnit: _, scaleType: H }) {
    if (isColorChannel(O)) {
      if (contains$2(["quarter", "month", "day"], _))
        return "symbol";
      if (isContinuousToContinuous(H))
        return "gradient";
    }
    return "symbol";
  }
  function getDirection({ legendConfig: O, legendType: _, orient: H, legend: V }) {
    return V.direction ?? O[_ ? "gradientDirection" : "symbolDirection"] ?? defaultDirection(H, _);
  }
  function defaultDirection(O, _) {
    switch (O) {
      case "top":
      case "bottom":
        return "horizontal";
      case "left":
      case "right":
      case "none":
      case void 0:
        return;
      // vertical is Vega's default
      default:
        return _ === "gradient" ? "horizontal" : void 0;
    }
  }
  function defaultGradientLength({ legendConfig: O, model: _, direction: H, orient: V, scaleType: q }) {
    const { gradientHorizontalMaxLength: G, gradientHorizontalMinLength: J, gradientVerticalMaxLength: Z, gradientVerticalMinLength: te } = O;
    if (isContinuousToContinuous(q))
      return H === "horizontal" ? V === "top" || V === "bottom" ? gradientLengthSignal(_, "width", J, G) : J : gradientLengthSignal(_, "height", te, Z);
  }
  function gradientLengthSignal(O, _, H, V) {
    return { signal: `clamp(${O.getSizeSignalRef(_).signal}, ${H}, ${V})` };
  }
  function defaultLabelOverlap(O) {
    if (contains$2(["quantile", "threshold", "log", "symlog"], O))
      return "greedy";
  }
  function parseLegend(O) {
    const _ = isUnitModel(O) ? parseUnitLegend(O) : parseNonUnitLegend(O);
    return O.component.legends = _, _;
  }
  function parseUnitLegend(O) {
    const { encoding: _ } = O, H = {};
    for (const V of [COLOR, ...LEGEND_SCALE_CHANNELS]) {
      const q = getFieldOrDatumDef(_[V]);
      !q || !O.getScaleComponent(V) || V === SHAPE && isFieldDef(q) && q.type === GEOJSON || (H[V] = parseLegendForChannel(O, V));
    }
    return H;
  }
  function getLegendDefWithScale(O, _) {
    const H = O.scaleName(_);
    if (O.mark === "trail") {
      if (_ === "color")
        return { stroke: H };
      if (_ === "size")
        return { strokeWidth: H };
    }
    return _ === "color" ? O.markDef.filled ? { fill: H } : { stroke: H } : { [_]: H };
  }
  function isExplicit$1(O, _, H, V) {
    switch (_) {
      case "disable":
        return H !== void 0;
      // if axis is specified or null/false, then its enable/disable state is explicit
      case "values":
        return !!(H != null && H.values);
      case "title":
        if (_ === "title" && O === (V == null ? void 0 : V.title))
          return !0;
    }
    return O === (H || {})[_];
  }
  function parseLegendForChannel(O, _) {
    var me;
    let H = O.legend(_);
    const { markDef: V, encoding: q, config: G } = O, J = G.legend, Z = new LegendComponent({}, getLegendDefWithScale(O, _));
    parseInteractiveLegend(O, _, Z);
    const te = H !== void 0 ? !H : J.disable;
    if (Z.set("disable", te, H !== void 0), te)
      return Z;
    H = H || {};
    const ne = O.getScaleComponent(_).get("type"), ee = getFieldOrDatumDef(q[_]), ae = isFieldDef(ee) ? (me = normalizeTimeUnit(ee.timeUnit)) == null ? void 0 : me.unit : void 0, oe = H.orient || G.legend.orient || "right", se = getLegendType({ legend: H, channel: _, timeUnit: ae, scaleType: ne }), ce = getDirection({ legend: H, legendType: se, orient: oe, legendConfig: J }), le = {
      legend: H,
      channel: _,
      model: O,
      markDef: V,
      encoding: q,
      fieldOrDatumDef: ee,
      legendConfig: J,
      config: G,
      scaleType: ne,
      orient: oe,
      legendType: se,
      direction: ce
    };
    for (const pe of LEGEND_COMPONENT_PROPERTIES) {
      if (se === "gradient" && pe.startsWith("symbol") || se === "symbol" && pe.startsWith("gradient"))
        continue;
      const ye = pe in legendRules ? legendRules[pe](le) : H[pe];
      if (ye !== void 0) {
        const ge = isExplicit$1(ye, pe, H, O.fieldDef(_));
        (ge || G.legend[pe] === void 0) && Z.set(pe, ye, ge);
      }
    }
    const ue = (H == null ? void 0 : H.encoding) ?? {}, de = Z.get("selections"), fe = {}, he = { fieldOrDatumDef: ee, model: O, channel: _, legendCmpt: Z, legendType: se };
    for (const pe of ["labels", "legend", "title", "symbols", "gradient", "entries"]) {
      const ye = guideEncodeEntry(ue[pe] ?? {}, O), ge = pe in legendEncodeRules ? legendEncodeRules[pe](ye, he) : ye;
      ge !== void 0 && !isEmpty(ge) && (fe[pe] = {
        ...de != null && de.length && isFieldDef(ee) ? { name: `${varName(ee.field)}_legend_${pe}` } : {},
        ...de != null && de.length ? { interactive: !!de } : {},
        update: ge
      });
    }
    return isEmpty(fe) || Z.set("encode", fe, !!(H != null && H.encoding)), Z;
  }
  function parseNonUnitLegend(O) {
    const { legends: _, resolve: H } = O.component;
    for (const V of O.children) {
      parseLegend(V);
      for (const q of keys$2(V.component.legends))
        H.legend[q] = parseGuideResolve(O.component.resolve, q), H.legend[q] === "shared" && (_[q] = mergeLegendComponent(_[q], V.component.legends[q]), _[q] || (H.legend[q] = "independent", delete _[q]));
    }
    for (const V of keys$2(_))
      for (const q of O.children)
        q.component.legends[V] && H.legend[V] === "shared" && delete q.component.legends[V];
    return _;
  }
  function mergeLegendComponent(O, _) {
    var G, J, Z, te;
    if (!O)
      return _.clone();
    const H = O.getWithExplicit("orient"), V = _.getWithExplicit("orient");
    if (H.explicit && V.explicit && H.value !== V.value)
      return;
    let q = !1;
    for (const ne of LEGEND_COMPONENT_PROPERTIES) {
      const ee = mergeValuesWithExplicit(
        O.getWithExplicit(ne),
        _.getWithExplicit(ne),
        ne,
        "legend",
        // Tie breaker function
        (ae, oe) => {
          switch (ne) {
            case "symbolType":
              return mergeSymbolType(ae, oe);
            case "title":
              return mergeTitleComponent(ae, oe);
            case "type":
              return q = !0, makeImplicit("symbol");
          }
          return defaultTieBreaker(ae, oe, ne, "legend");
        }
      );
      O.setWithExplicit(ne, ee);
    }
    return q && ((J = (G = O.implicit) == null ? void 0 : G.encode) != null && J.gradient && deleteNestedProperty(O.implicit, ["encode", "gradient"]), (te = (Z = O.explicit) == null ? void 0 : Z.encode) != null && te.gradient && deleteNestedProperty(O.explicit, ["encode", "gradient"])), O;
  }
  function mergeSymbolType(O, _) {
    return _.value === "circle" ? _ : O;
  }
  function setLegendEncode(O, _, H, V) {
    var q, G;
    O.encode ?? (O.encode = {}), (q = O.encode)[_] ?? (q[_] = {}), (G = O.encode[_]).update ?? (G.update = {}), O.encode[_].update[H] = V;
  }
  function assembleLegends(O) {
    const _ = O.component.legends, H = {};
    for (const q of keys$2(_)) {
      const G = O.getScaleComponent(q), J = stringify$1(G.get("domains"));
      if (H[J])
        for (const Z of H[J])
          mergeLegendComponent(Z, _[q]) || H[J].push(_[q]);
      else
        H[J] = [_[q].clone()];
    }
    return vals(H).flat().map((q) => assembleLegend(q, O.config)).filter((q) => q !== void 0);
  }
  function assembleLegend(O, _) {
    var J, Z, te;
    const { disable: H, labelExpr: V, selections: q, ...G } = O.combine();
    if (!H) {
      if (_.aria === !1 && G.aria == null && (G.aria = !1), (J = G.encode) != null && J.symbols) {
        const ne = G.encode.symbols.update;
        ne.fill && ne.fill.value !== "transparent" && !ne.stroke && !G.stroke && (ne.stroke = { value: "transparent" });
        for (const ee of LEGEND_SCALE_CHANNELS)
          G[ee] && delete ne[ee];
      }
      if (G.title || delete G.title, V !== void 0) {
        let ne = V;
        (te = (Z = G.encode) == null ? void 0 : Z.labels) != null && te.update && isSignalRef(G.encode.labels.update.text) && (ne = replaceAll$1(V, "datum.label", G.encode.labels.update.text.signal)), setLegendEncode(G, "labels", "text", { signal: ne });
      }
      return G;
    }
  }
  function assembleProjections(O) {
    return isLayerModel(O) || isConcatModel(O) ? assembleProjectionsForModelAndChildren(O) : assembleProjectionForModel(O);
  }
  function assembleProjectionsForModelAndChildren(O) {
    return O.children.reduce((_, H) => _.concat(H.assembleProjections()), assembleProjectionForModel(O));
  }
  function assembleProjectionForModel(O) {
    const _ = O.component.projection;
    if (!_ || _.merged)
      return [];
    const H = _.combine(), { name: V } = H;
    if (_.data) {
      const q = {
        signal: `[${_.size.map((J) => J.signal).join(", ")}]`
      }, G = _.data.reduce((J, Z) => {
        const te = isSignalRef(Z) ? Z.signal : `data('${O.lookupDataSource(Z)}')`;
        return contains$2(J, te) || J.push(te), J;
      }, []);
      if (G.length <= 0)
        throw new Error("Projection's fit didn't find any data sources");
      return [
        {
          name: V,
          size: q,
          fit: {
            signal: G.length > 1 ? `[${G.join(", ")}]` : G[0]
          },
          ...H
        }
      ];
    } else
      return [
        {
          name: V,
          // translate to center by default
          translate: { signal: "[width / 2, height / 2]" },
          // parameters, overwrite default translate if specified
          ...H
        }
      ];
  }
  const PROJECTION_PROPERTIES = [
    "type",
    "clipAngle",
    "clipExtent",
    "center",
    "rotate",
    "precision",
    "reflectX",
    "reflectY",
    "coefficient",
    "distance",
    "fraction",
    "lobes",
    "parallel",
    "radius",
    "ratio",
    "spacing",
    "tilt"
  ];
  class ProjectionComponent extends Split {
    constructor(_, H, V, q) {
      super(
        { ...H },
        // all explicit properties of projection
        { name: _ }
        // name as initial implicit property
      ), this.specifiedProjection = H, this.size = V, this.data = q, this.merged = !1;
    }
    /**
     * Whether the projection parameters should fit provided data.
     */
    get isFit() {
      return !!this.data;
    }
  }
  function parseProjection(O) {
    O.component.projection = isUnitModel(O) ? parseUnitProjection(O) : parseNonUnitProjections(O);
  }
  function parseUnitProjection(O) {
    if (O.hasProjection) {
      const _ = replaceExprRef(O.specifiedProjection), H = !(_.scale != null || _.translate != null), V = H ? [O.getSizeSignalRef("width"), O.getSizeSignalRef("height")] : void 0, q = H ? gatherFitData(O) : void 0, G = new ProjectionComponent(O.projectionName(!0), {
        ...replaceExprRef(O.config.projection),
        ..._
      }, V, q);
      return G.get("type") || G.set("type", "equalEarth", !1), G;
    }
  }
  function gatherFitData(O) {
    const _ = [], { encoding: H } = O;
    for (const V of [
      [LONGITUDE, LATITUDE],
      [LONGITUDE2, LATITUDE2]
    ])
      (getFieldOrDatumDef(H[V[0]]) || getFieldOrDatumDef(H[V[1]])) && _.push({
        signal: O.getName(`geojson_${_.length}`)
      });
    return O.channelHasField(SHAPE) && O.typedFieldDef(SHAPE).type === GEOJSON && _.push({
      signal: O.getName(`geojson_${_.length}`)
    }), _.length === 0 && _.push(O.requestDataName(DataSourceType.Main)), _;
  }
  function mergeIfNoConflict(O, _) {
    const H = every(PROJECTION_PROPERTIES, (q) => !!(!has$1(O.explicit, q) && !has$1(_.explicit, q) || has$1(O.explicit, q) && has$1(_.explicit, q) && // some properties might be signals or objects and require hashing for comparison
    deepEqual(O.get(q), _.get(q))));
    if (deepEqual(O.size, _.size)) {
      if (H)
        return O;
      if (deepEqual(O.explicit, {}))
        return _;
      if (deepEqual(_.explicit, {}))
        return O;
    }
    return null;
  }
  function parseNonUnitProjections(O) {
    if (O.children.length === 0)
      return;
    let _;
    for (const V of O.children)
      parseProjection(V);
    const H = every(O.children, (V) => {
      const q = V.component.projection;
      if (q)
        if (_) {
          const G = mergeIfNoConflict(_, q);
          return G && (_ = G), !!G;
        } else
          return _ = q, !0;
      else return !0;
    });
    if (_ && H) {
      const V = O.projectionName(!0), q = new ProjectionComponent(V, _.specifiedProjection, _.size, duplicate(_.data));
      for (const G of O.children) {
        const J = G.component.projection;
        J && (J.isFit && q.data.push(...G.component.projection.data), G.renameProjection(J.get("name"), V), J.merged = !0);
      }
      return q;
    }
  }
  function rangeFormula(O, _, H, V) {
    if (binRequiresRange(_, H)) {
      const q = isUnitModel(O) ? O.axis(H) ?? O.legend(H) ?? {} : {}, G = vgField(_, { expr: "datum" }), J = vgField(_, { expr: "datum", binSuffix: "end" });
      return {
        formulaAs: vgField(_, { binSuffix: "range", forAs: !0 }),
        formula: binFormatExpression(G, J, q.format, q.formatType, V)
      };
    }
    return {};
  }
  function binKey(O, _) {
    return `${binToString(O)}_${_}`;
  }
  function getSignalsFromModel(O, _) {
    return {
      signal: O.getName(`${_}_bins`),
      extentSignal: O.getName(`${_}_extent`)
    };
  }
  function getBinSignalName(O, _, H) {
    const V = normalizeBin(H, void 0) ?? {}, q = binKey(V, _);
    return O.getName(`${q}_bins`);
  }
  function isBinTransform(O) {
    return "as" in O;
  }
  function createBinComponent(O, _, H) {
    let V, q;
    isBinTransform(O) ? V = isString$1(O.as) ? [O.as, `${O.as}_end`] : [O.as[0], O.as[1]] : V = [vgField(O, { forAs: !0 }), vgField(O, { binSuffix: "end", forAs: !0 })];
    const G = { ...normalizeBin(_, void 0) }, J = binKey(G, O.field), { signal: Z, extentSignal: te } = getSignalsFromModel(H, J);
    if (isParameterExtent(G.extent)) {
      const ee = G.extent;
      q = parseSelectionExtent(H, ee.param, ee), delete G.extent;
    }
    const ne = {
      bin: G,
      field: O.field,
      as: [V],
      ...Z ? { signal: Z } : {},
      ...te ? { extentSignal: te } : {},
      ...q ? { span: q } : {}
    };
    return { key: J, binComponent: ne };
  }
  class BinNode extends DataFlowNode {
    clone() {
      return new BinNode(null, duplicate(this.bins));
    }
    constructor(_, H) {
      super(_), this.bins = H;
    }
    static makeFromEncoding(_, H) {
      const V = H.reduceFieldDef((q, G, J) => {
        if (isTypedFieldDef(G) && isBinning(G.bin)) {
          const { key: Z, binComponent: te } = createBinComponent(G, G.bin, H);
          q[Z] = {
            ...te,
            ...q[Z],
            ...rangeFormula(H, G, J, H.config)
          };
        }
        return q;
      }, {});
      return isEmpty(V) ? null : new BinNode(_, V);
    }
    /**
     * Creates a bin node from BinTransform.
     * The optional parameter should provide
     */
    static makeFromTransform(_, H, V) {
      const { key: q, binComponent: G } = createBinComponent(H, H.bin, V);
      return new BinNode(_, {
        [q]: G
      });
    }
    /**
     * Merge bin nodes. This method either integrates the bin config from the other node
     * or if this node already has a bin config, renames the corresponding signal in the model.
     */
    merge(_, H) {
      for (const V of keys$2(_.bins))
        V in this.bins ? (H(_.bins[V].signal, this.bins[V].signal), this.bins[V].as = unique([...this.bins[V].as, ..._.bins[V].as], hash)) : this.bins[V] = _.bins[V];
      for (const V of _.children)
        _.removeChild(V), V.parent = this;
      _.remove();
    }
    producedFields() {
      return new Set(vals(this.bins).map((_) => _.as).flat(2));
    }
    dependentFields() {
      return new Set(vals(this.bins).map((_) => _.field));
    }
    hash() {
      return `Bin ${hash(this.bins)}`;
    }
    assemble() {
      return vals(this.bins).flatMap((_) => {
        const H = [], [V, ...q] = _.as, { extent: G, ...J } = _.bin, Z = {
          type: "bin",
          field: replacePathInField(_.field),
          as: V,
          signal: _.signal,
          ...isParameterExtent(G) ? { extent: null } : { extent: G },
          ..._.span ? { span: { signal: `span(${_.span})` } } : {},
          ...J
        };
        !G && _.extentSignal && (H.push({
          type: "extent",
          field: replacePathInField(_.field),
          signal: _.extentSignal
        }), Z.extent = { signal: _.extentSignal }), H.push(Z);
        for (const te of q)
          for (let ne = 0; ne < 2; ne++)
            H.push({
              type: "formula",
              expr: vgField({ field: V[ne] }, { expr: "datum" }),
              as: te[ne]
            });
        return _.formula && H.push({
          type: "formula",
          expr: _.formula,
          as: _.formulaAs
        }), H;
      });
    }
  }
  function addDimension(O, _, H, V) {
    var G;
    const q = isUnitModel(V) ? V.encoding[getSecondaryRangeChannel(_)] : void 0;
    if (isTypedFieldDef(H) && isUnitModel(V) && hasBandEnd(H, q, V.markDef, V.config)) {
      O.add(vgField(H, {})), O.add(vgField(H, { suffix: "end" }));
      const { mark: J, markDef: Z, config: te } = V, ne = getBandPosition({ fieldDef: H, markDef: Z, config: te });
      isRectBasedMark(J) && ne !== 0.5 && isXorY(_) && (O.add(vgField(H, { suffix: OFFSETTED_RECT_START_SUFFIX })), O.add(vgField(H, { suffix: OFFSETTED_RECT_END_SUFFIX }))), H.bin && binRequiresRange(H, _) && O.add(vgField(H, { binSuffix: "range" }));
    } else if (isGeoPositionChannel(_)) {
      const J = getPositionChannelFromLatLong(_);
      O.add(V.getName(J));
    } else
      O.add(vgField(H));
    return isScaleFieldDef(H) && isFieldRange((G = H.scale) == null ? void 0 : G.range) && O.add(H.scale.range.field), O;
  }
  function mergeMeasures(O, _) {
    for (const H of keys$2(_)) {
      const V = _[H];
      for (const q of keys$2(V))
        H in O ? O[H][q] = /* @__PURE__ */ new Set([...O[H][q] ?? [], ...V[q]]) : O[H] = { [q]: V[q] };
    }
  }
  class AggregateNode extends DataFlowNode {
    clone() {
      return new AggregateNode(null, new Set(this.dimensions), duplicate(this.measures));
    }
    /**
     * @param dimensions string set for dimensions
     * @param measures dictionary mapping field name => dict of aggregation functions and names to use
     */
    constructor(_, H, V) {
      super(_), this.dimensions = H, this.measures = V;
    }
    get groupBy() {
      return this.dimensions;
    }
    static makeFromEncoding(_, H) {
      let V = !1;
      H.forEachFieldDef((J) => {
        J.aggregate && (V = !0);
      });
      const q = {}, G = /* @__PURE__ */ new Set();
      return !V || (H.forEachFieldDef((J, Z) => {
        const { aggregate: te, field: ne } = J;
        if (te)
          if (te === "count")
            q["*"] ?? (q["*"] = {}), q["*"].count = /* @__PURE__ */ new Set([vgField(J, { forAs: !0 })]);
          else {
            if (isArgminDef(te) || isArgmaxDef(te)) {
              const ee = isArgminDef(te) ? "argmin" : "argmax", ae = te[ee];
              q[ae] ?? (q[ae] = {}), q[ae][ee] = /* @__PURE__ */ new Set([vgField({ op: ee, field: ae }, { forAs: !0 })]);
            } else
              q[ne] ?? (q[ne] = {}), q[ne][te] = /* @__PURE__ */ new Set([vgField(J, { forAs: !0 })]);
            isScaleChannel(Z) && H.scaleDomain(Z) === "unaggregated" && (q[ne] ?? (q[ne] = {}), q[ne].min = /* @__PURE__ */ new Set([vgField({ field: ne, aggregate: "min" }, { forAs: !0 })]), q[ne].max = /* @__PURE__ */ new Set([vgField({ field: ne, aggregate: "max" }, { forAs: !0 })]));
          }
        else
          addDimension(G, Z, J, H);
      }), G.size + keys$2(q).length === 0) ? null : new AggregateNode(_, G, q);
    }
    static makeFromTransform(_, H) {
      var V;
      const q = /* @__PURE__ */ new Set(), G = {};
      for (const J of H.aggregate) {
        const { op: Z, field: te, as: ne } = J;
        Z && (Z === "count" ? (G["*"] ?? (G["*"] = {}), G["*"].count = /* @__PURE__ */ new Set([ne || vgField(J, { forAs: !0 })])) : (G[te] ?? (G[te] = {}), (V = G[te])[Z] ?? (V[Z] = /* @__PURE__ */ new Set()), G[te][Z].add(ne || vgField(J, { forAs: !0 }))));
      }
      for (const J of H.groupby ?? [])
        q.add(J);
      return q.size + keys$2(G).length === 0 ? null : new AggregateNode(_, q, G);
    }
    merge(_) {
      return setEqual(this.dimensions, _.dimensions) ? (mergeMeasures(this.measures, _.measures), !0) : (debug("different dimensions, cannot merge"), !1);
    }
    addDimensions(_) {
      _.forEach(this.dimensions.add, this.dimensions);
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([...this.dimensions, ...keys$2(this.measures)]);
    }
    producedFields() {
      const _ = /* @__PURE__ */ new Set();
      for (const H of keys$2(this.measures))
        for (const V of keys$2(this.measures[H])) {
          const q = this.measures[H][V];
          q.size === 0 ? _.add(`${V}_${H}`) : q.forEach(_.add, _);
        }
      return _;
    }
    hash() {
      return `Aggregate ${hash({ dimensions: this.dimensions, measures: this.measures })}`;
    }
    assemble() {
      const _ = [], H = [], V = [];
      for (const G of keys$2(this.measures))
        for (const J of keys$2(this.measures[G]))
          for (const Z of this.measures[G][J])
            V.push(Z), _.push(J), H.push(G === "*" ? null : replacePathInField(G));
      return {
        type: "aggregate",
        groupby: [...this.dimensions].map(replacePathInField),
        ops: _,
        fields: H,
        as: V
      };
    }
  }
  class FacetNode extends DataFlowNode {
    /**
     * @param model The facet model.
     * @param name The name that this facet source will have.
     * @param data The source data for this facet data.
     */
    constructor(_, H, V, q) {
      super(_), this.model = H, this.name = V, this.data = q;
      for (const G of FACET_CHANNELS) {
        const J = H.facet[G];
        if (J) {
          const { bin: Z, sort: te } = J;
          this[G] = {
            name: H.getName(`${G}_domain`),
            fields: [vgField(J), ...isBinning(Z) ? [vgField(J, { binSuffix: "end" })] : []],
            ...isSortField(te) ? { sortField: te } : isArray$2(te) ? { sortIndexField: sortArrayIndexField(J, G) } : {}
          };
        }
      }
      this.childModel = H.child;
    }
    hash() {
      let _ = "Facet";
      for (const H of FACET_CHANNELS)
        this[H] && (_ += ` ${H.charAt(0)}:${hash(this[H])}`);
      return _;
    }
    get fields() {
      var H;
      const _ = [];
      for (const V of FACET_CHANNELS)
        (H = this[V]) != null && H.fields && _.push(...this[V].fields);
      return _;
    }
    dependentFields() {
      const _ = new Set(this.fields);
      for (const H of FACET_CHANNELS)
        this[H] && (this[H].sortField && _.add(this[H].sortField.field), this[H].sortIndexField && _.add(this[H].sortIndexField));
      return _;
    }
    producedFields() {
      return /* @__PURE__ */ new Set();
    }
    /**
     * The name to reference this source is its name.
     */
    getSource() {
      return this.name;
    }
    getChildIndependentFieldsWithStep() {
      const _ = {};
      for (const H of POSITION_SCALE_CHANNELS) {
        const V = this.childModel.component.scales[H];
        if (V && !V.merged) {
          const q = V.get("type"), G = V.get("range");
          if (hasDiscreteDomain(q) && isVgRangeStep(G)) {
            const J = assembleDomain(this.childModel, H), Z = getFieldFromDomain(J);
            Z ? _[H] = Z : warn(unknownField(H));
          }
        }
      }
      return _;
    }
    assembleRowColumnHeaderData(_, H, V) {
      const q = { row: "y", column: "x", facet: void 0 }[_], G = [], J = [], Z = [];
      q && V && V[q] && (H ? (G.push(`distinct_${V[q]}`), J.push("max")) : (G.push(V[q]), J.push("distinct")), Z.push(`distinct_${V[q]}`));
      const { sortField: te, sortIndexField: ne } = this[_];
      if (te) {
        const { op: ee = DEFAULT_SORT_OP, field: ae } = te;
        G.push(ae), J.push(ee), Z.push(vgField(te, { forAs: !0 }));
      } else ne && (G.push(ne), J.push("max"), Z.push(ne));
      return {
        name: this[_].name,
        // Use data from the crossed one if it exist
        source: H ?? this.data,
        transform: [
          {
            type: "aggregate",
            groupby: this[_].fields,
            ...G.length ? {
              fields: G,
              ops: J,
              as: Z
            } : {}
          }
        ]
      };
    }
    assembleFacetHeaderData(_) {
      var te;
      const { columns: H } = this.model.layout, { layoutHeaders: V } = this.model.component, q = [], G = {};
      for (const ne of HEADER_CHANNELS) {
        for (const ee of HEADER_TYPES) {
          const ae = (V[ne] && V[ne][ee]) ?? [];
          for (const oe of ae)
            if (((te = oe.axes) == null ? void 0 : te.length) > 0) {
              G[ne] = !0;
              break;
            }
        }
        if (G[ne]) {
          const ee = `length(data("${this.facet.name}"))`, ae = ne === "row" ? H ? { signal: `ceil(${ee} / ${H})` } : 1 : H ? { signal: `min(${ee}, ${H})` } : { signal: ee };
          q.push({
            name: `${this.facet.name}_${ne}`,
            transform: [
              {
                type: "sequence",
                start: 0,
                stop: ae
              }
            ]
          });
        }
      }
      const { row: J, column: Z } = G;
      return (J || Z) && q.unshift(this.assembleRowColumnHeaderData("facet", null, _)), q;
    }
    assemble() {
      const _ = [];
      let H = null;
      const V = this.getChildIndependentFieldsWithStep(), { column: q, row: G, facet: J } = this;
      if (q && G && (V.x || V.y)) {
        H = `cross_${this.column.name}_${this.row.name}`;
        const Z = [].concat(V.x ?? [], V.y ?? []), te = Z.map(() => "distinct");
        _.push({
          name: H,
          source: this.data,
          transform: [
            {
              type: "aggregate",
              groupby: this.fields,
              fields: Z,
              ops: te
            }
          ]
        });
      }
      for (const Z of [COLUMN, ROW])
        this[Z] && _.push(this.assembleRowColumnHeaderData(Z, H, V));
      if (J) {
        const Z = this.assembleFacetHeaderData(V);
        Z && _.push(...Z);
      }
      return _;
    }
  }
  function unquote(O) {
    return O.startsWith("'") && O.endsWith("'") || O.startsWith('"') && O.endsWith('"') ? O.slice(1, -1) : O;
  }
  function parseExpression(O, _) {
    const H = accessPathWithDatum(O);
    if (_ === "number")
      return `toNumber(${H})`;
    if (_ === "boolean")
      return `toBoolean(${H})`;
    if (_ === "string")
      return `toString(${H})`;
    if (_ === "date")
      return `toDate(${H})`;
    if (_ === "flatten")
      return H;
    if (_.startsWith("date:")) {
      const V = unquote(_.slice(5, _.length));
      return `timeParse(${H},'${V}')`;
    } else if (_.startsWith("utc:")) {
      const V = unquote(_.slice(4, _.length));
      return `utcParse(${H},'${V}')`;
    } else
      return warn(unrecognizedParse(_)), null;
  }
  function getImplicitFromFilterTransform(O) {
    const _ = {};
    return forEachLeaf(O.filter, (H) => {
      if (isFieldPredicate(H)) {
        let V = null;
        isFieldEqualPredicate(H) ? V = signalRefOrValue(H.equal) : isFieldLTEPredicate(H) ? V = signalRefOrValue(H.lte) : isFieldLTPredicate(H) ? V = signalRefOrValue(H.lt) : isFieldGTPredicate(H) ? V = signalRefOrValue(H.gt) : isFieldGTEPredicate(H) ? V = signalRefOrValue(H.gte) : isFieldRangePredicate(H) ? V = H.range[0] : isFieldOneOfPredicate(H) && (V = (H.oneOf ?? H.in)[0]), V && (isDateTime(V) ? _[H.field] = "date" : isNumber$2(V) ? _[H.field] = "number" : isString$1(V) && (_[H.field] = "string")), H.timeUnit && (_[H.field] = "date");
      }
    }), _;
  }
  function getImplicitFromEncoding(O) {
    const _ = {};
    function H(V) {
      isFieldOrDatumDefForTimeFormat(V) ? _[V.field] = "date" : V.type === "quantitative" && isMinMaxOp(V.aggregate) ? _[V.field] = "number" : accessPathDepth(V.field) > 1 ? V.field in _ || (_[V.field] = "flatten") : isScaleFieldDef(V) && isSortField(V.sort) && accessPathDepth(V.sort.field) > 1 && (V.sort.field in _ || (_[V.sort.field] = "flatten"));
    }
    if ((isUnitModel(O) || isFacetModel(O)) && O.forEachFieldDef((V, q) => {
      if (isTypedFieldDef(V))
        H(V);
      else {
        const G = getMainRangeChannel(q), J = O.fieldDef(G);
        H({
          ...V,
          type: J.type
        });
      }
    }), isUnitModel(O)) {
      const { mark: V, markDef: q, encoding: G } = O;
      if (isPathMark(V) && // No need to sort by dimension if we have a connected scatterplot (order channel is present)
      !O.encoding.order) {
        const J = q.orient === "horizontal" ? "y" : "x", Z = G[J];
        isFieldDef(Z) && Z.type === "quantitative" && !(Z.field in _) && (_[Z.field] = "number");
      }
    }
    return _;
  }
  function getImplicitFromSelection(O) {
    const _ = {};
    if (isUnitModel(O) && O.component.selection)
      for (const H of keys$2(O.component.selection)) {
        const V = O.component.selection[H];
        for (const q of V.project.items)
          !q.channel && accessPathDepth(q.field) > 1 && (_[q.field] = "flatten");
      }
    return _;
  }
  class ParseNode extends DataFlowNode {
    clone() {
      return new ParseNode(null, duplicate(this._parse));
    }
    constructor(_, H) {
      super(_), this._parse = H;
    }
    hash() {
      return `Parse ${hash(this._parse)}`;
    }
    /**
     * Creates a parse node from a data.format.parse and updates ancestorParse.
     */
    static makeExplicit(_, H, V) {
      var J;
      let q = {};
      const G = H.data;
      return !isGenerator(G) && ((J = G == null ? void 0 : G.format) != null && J.parse) && (q = G.format.parse), this.makeWithAncestors(_, q, {}, V);
    }
    /**
     * Creates a parse node from "explicit" parse and "implicit" parse and updates ancestorParse.
     */
    static makeWithAncestors(_, H, V, q) {
      for (const Z of keys$2(V)) {
        const te = q.getWithExplicit(Z);
        te.value !== void 0 && (te.explicit || te.value === V[Z] || te.value === "derived" || V[Z] === "flatten" ? delete V[Z] : warn(differentParse(Z, V[Z], te.value)));
      }
      for (const Z of keys$2(H)) {
        const te = q.get(Z);
        te !== void 0 && (te === H[Z] ? delete H[Z] : warn(differentParse(Z, H[Z], te)));
      }
      const G = new Split(H, V);
      q.copyAll(G);
      const J = {};
      for (const Z of keys$2(G.combine())) {
        const te = G.get(Z);
        te !== null && (J[Z] = te);
      }
      return keys$2(J).length === 0 || q.parseNothing ? null : new ParseNode(_, J);
    }
    get parse() {
      return this._parse;
    }
    merge(_) {
      this._parse = { ...this._parse, ..._.parse }, _.remove();
    }
    /**
     * Assemble an object for Vega's format.parse property.
     */
    assembleFormatParse() {
      const _ = {};
      for (const H of keys$2(this._parse)) {
        const V = this._parse[H];
        accessPathDepth(H) === 1 && (_[H] = V);
      }
      return _;
    }
    // format parse depends and produces all fields in its parse
    producedFields() {
      return new Set(keys$2(this._parse));
    }
    dependentFields() {
      return new Set(keys$2(this._parse));
    }
    assembleTransforms(_ = !1) {
      return keys$2(this._parse).filter((H) => _ ? accessPathDepth(H) > 1 : !0).map((H) => {
        const V = parseExpression(H, this._parse[H]);
        return V ? {
          type: "formula",
          expr: V,
          as: removePathFromField(H)
          // Vega output is always flattened
        } : null;
      }).filter((H) => H !== null);
    }
  }
  class IdentifierNode extends DataFlowNode {
    clone() {
      return new IdentifierNode(null);
    }
    constructor(_) {
      super(_);
    }
    dependentFields() {
      return /* @__PURE__ */ new Set();
    }
    producedFields() {
      return /* @__PURE__ */ new Set([SELECTION_ID]);
    }
    hash() {
      return "Identifier";
    }
    assemble() {
      return { type: "identifier", as: SELECTION_ID };
    }
  }
  class GraticuleNode extends DataFlowNode {
    clone() {
      return new GraticuleNode(null, this.params);
    }
    constructor(_, H) {
      super(_), this.params = H;
    }
    dependentFields() {
      return /* @__PURE__ */ new Set();
    }
    producedFields() {
    }
    hash() {
      return `Graticule ${hash(this.params)}`;
    }
    assemble() {
      return {
        type: "graticule",
        ...this.params === !0 ? {} : this.params
      };
    }
  }
  class SequenceNode extends DataFlowNode {
    clone() {
      return new SequenceNode(null, this.params);
    }
    constructor(_, H) {
      super(_), this.params = H;
    }
    dependentFields() {
      return /* @__PURE__ */ new Set();
    }
    producedFields() {
      return /* @__PURE__ */ new Set([this.params.as ?? "data"]);
    }
    hash() {
      return `Hash ${hash(this.params)}`;
    }
    assemble() {
      return {
        type: "sequence",
        ...this.params
      };
    }
  }
  class SourceNode extends DataFlowNode {
    constructor(_) {
      super(null), _ ?? (_ = { name: "source" });
      let H;
      if (isGenerator(_) || (H = _.format ? { ...omit(_.format, ["parse"]) } : {}), isInlineData(_))
        this._data = { values: _.values };
      else if (isUrlData(_)) {
        if (this._data = { url: _.url }, !H.type) {
          let V = /(?:\.([^.]+))?$/.exec(_.url)[1];
          contains$2(["json", "csv", "tsv", "dsv", "topojson"], V) || (V = "json"), H.type = V;
        }
      } else isSphereGenerator(_) ? this._data = { values: [{ type: "Sphere" }] } : (isNamedData(_) || isGenerator(_)) && (this._data = {});
      this._generator = isGenerator(_), _.name && (this._name = _.name), H && !isEmpty(H) && (this._data.format = H);
    }
    dependentFields() {
      return /* @__PURE__ */ new Set();
    }
    producedFields() {
    }
    get data() {
      return this._data;
    }
    hasName() {
      return !!this._name;
    }
    get isGenerator() {
      return this._generator;
    }
    get dataName() {
      return this._name;
    }
    set dataName(_) {
      this._name = _;
    }
    set parent(_) {
      throw new Error("Source nodes have to be roots.");
    }
    remove() {
      throw new Error("Source nodes are roots and cannot be removed.");
    }
    hash() {
      throw new Error("Cannot hash sources");
    }
    assemble() {
      return {
        name: this._name,
        ...this._data,
        transform: []
      };
    }
  }
  var __classPrivateFieldSet = function(O, _, H, V, q) {
    if (V === "m") throw new TypeError("Private method is not writable");
    if (V === "a" && !q) throw new TypeError("Private accessor was defined without a setter");
    if (typeof _ == "function" ? O !== _ || !q : !_.has(O)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return V === "a" ? q.call(O, H) : q ? q.value = H : _.set(O, H), H;
  }, __classPrivateFieldGet = function(O, _, H, V) {
    if (H === "a" && !V) throw new TypeError("Private accessor was defined without a getter");
    if (typeof _ == "function" ? O !== _ || !V : !_.has(O)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return H === "m" ? V : H === "a" ? V.call(O) : V ? V.value : _.get(O);
  }, _Optimizer_modified;
  function isDataSourceNode(O) {
    return O instanceof SourceNode || O instanceof GraticuleNode || O instanceof SequenceNode;
  }
  class Optimizer {
    constructor() {
      _Optimizer_modified.set(this, void 0), __classPrivateFieldSet(this, _Optimizer_modified, !1, "f");
    }
    // Once true, #modified is never set to false
    setModified() {
      __classPrivateFieldSet(this, _Optimizer_modified, !0, "f");
    }
    get modifiedFlag() {
      return __classPrivateFieldGet(this, _Optimizer_modified, "f");
    }
  }
  _Optimizer_modified = /* @__PURE__ */ new WeakMap();
  class BottomUpOptimizer extends Optimizer {
    /**
     * Compute a map of node depths that we can use to determine a topological sort order.
     */
    getNodeDepths(_, H, V) {
      V.set(_, H);
      for (const q of _.children)
        this.getNodeDepths(q, H + 1, V);
      return V;
    }
    /**
     * Run the optimizer on all nodes starting from the leaves.
     */
    optimize(_) {
      const V = [...this.getNodeDepths(_, 0, /* @__PURE__ */ new Map()).entries()].sort((q, G) => G[1] - q[1]);
      for (const q of V)
        this.run(q[0]);
      return this.modifiedFlag;
    }
  }
  class TopDownOptimizer extends Optimizer {
    /**
     * Run the optimizer depth first on all nodes starting from the roots.
     */
    optimize(_) {
      this.run(_);
      for (const H of _.children)
        this.optimize(H);
      return this.modifiedFlag;
    }
  }
  class MergeIdenticalNodes extends TopDownOptimizer {
    mergeNodes(_, H) {
      const V = H.shift();
      for (const q of H)
        _.removeChild(q), q.parent = V, q.remove();
    }
    run(_) {
      const H = _.children.map((q) => q.hash()), V = {};
      for (let q = 0; q < H.length; q++)
        V[H[q]] === void 0 ? V[H[q]] = [_.children[q]] : V[H[q]].push(_.children[q]);
      for (const q of keys$2(V))
        V[q].length > 1 && (this.setModified(), this.mergeNodes(_, V[q]));
    }
  }
  class RemoveUnnecessaryIdentifierNodes extends TopDownOptimizer {
    constructor(_) {
      super(), this.requiresSelectionId = _ && requiresSelectionId(_);
    }
    run(_) {
      _ instanceof IdentifierNode && (this.requiresSelectionId && (isDataSourceNode(_.parent) || _.parent instanceof AggregateNode || _.parent instanceof ParseNode) || (this.setModified(), _.remove()));
    }
  }
  class RemoveDuplicateTimeUnits extends Optimizer {
    optimize(_) {
      return this.run(_, /* @__PURE__ */ new Set()), this.modifiedFlag;
    }
    run(_, H) {
      let V = /* @__PURE__ */ new Set();
      _ instanceof TimeUnitNode && (V = _.producedFields(), hasIntersection(V, H) && (this.setModified(), _.removeFormulas(H), _.producedFields.length === 0 && _.remove()));
      for (const q of _.children)
        this.run(q, /* @__PURE__ */ new Set([...H, ...V]));
    }
  }
  class RemoveUnnecessaryOutputNodes extends TopDownOptimizer {
    constructor() {
      super();
    }
    run(_) {
      _ instanceof OutputNode && !_.isRequired() && (this.setModified(), _.remove());
    }
  }
  class MoveParseUp extends BottomUpOptimizer {
    run(_) {
      if (!isDataSourceNode(_) && !(_.numChildren() > 1)) {
        for (const H of _.children)
          if (H instanceof ParseNode)
            if (_ instanceof ParseNode)
              this.setModified(), _.merge(H);
            else {
              if (fieldIntersection(_.producedFields(), H.dependentFields()))
                continue;
              this.setModified(), H.swapWithParent();
            }
      }
    }
  }
  class MergeParse extends BottomUpOptimizer {
    run(_) {
      const H = [..._.children], V = _.children.filter((q) => q instanceof ParseNode);
      if (_.numChildren() > 1 && V.length >= 1) {
        const q = {}, G = /* @__PURE__ */ new Set();
        for (const J of V) {
          const Z = J.parse;
          for (const te of keys$2(Z))
            te in q ? q[te] !== Z[te] && G.add(te) : q[te] = Z[te];
        }
        for (const J of G)
          delete q[J];
        if (!isEmpty(q)) {
          this.setModified();
          const J = new ParseNode(_, q);
          for (const Z of H) {
            if (Z instanceof ParseNode)
              for (const te of keys$2(q))
                delete Z.parse[te];
            _.removeChild(Z), Z.parent = J, Z instanceof ParseNode && keys$2(Z.parse).length === 0 && Z.remove();
          }
        }
      }
    }
  }
  class RemoveUnusedSubtrees extends BottomUpOptimizer {
    run(_) {
      _ instanceof OutputNode || _.numChildren() > 0 || _ instanceof FacetNode || _ instanceof SourceNode || (this.setModified(), _.remove());
    }
  }
  class MergeTimeUnits extends BottomUpOptimizer {
    run(_) {
      const H = _.children.filter((q) => q instanceof TimeUnitNode), V = H.pop();
      for (const q of H)
        this.setModified(), V.merge(q);
    }
  }
  class MergeAggregates extends BottomUpOptimizer {
    run(_) {
      const H = _.children.filter((q) => q instanceof AggregateNode), V = {};
      for (const q of H) {
        const G = hash(q.groupBy);
        G in V || (V[G] = []), V[G].push(q);
      }
      for (const q of keys$2(V)) {
        const G = V[q];
        if (G.length > 1) {
          const J = G.pop();
          for (const Z of G)
            J.merge(Z) && (_.removeChild(Z), Z.parent = J, Z.remove(), this.setModified());
        }
      }
    }
  }
  class MergeBins extends BottomUpOptimizer {
    constructor(_) {
      super(), this.model = _;
    }
    run(_) {
      const H = !(isDataSourceNode(_) || _ instanceof FilterNode || _ instanceof ParseNode || _ instanceof IdentifierNode), V = [], q = [];
      for (const G of _.children)
        G instanceof BinNode && (H && !fieldIntersection(_.producedFields(), G.dependentFields()) ? V.push(G) : q.push(G));
      if (V.length > 0) {
        const G = V.pop();
        for (const J of V)
          G.merge(J, this.model.renameSignal.bind(this.model));
        this.setModified(), _ instanceof BinNode ? _.merge(G, this.model.renameSignal.bind(this.model)) : G.swapWithParent();
      }
      if (q.length > 1) {
        const G = q.pop();
        for (const J of q)
          G.merge(J, this.model.renameSignal.bind(this.model));
        this.setModified();
      }
    }
  }
  class MergeOutputs extends BottomUpOptimizer {
    run(_) {
      const H = [..._.children];
      if (!some(H, (J) => J instanceof OutputNode) || _.numChildren() <= 1)
        return;
      const q = [];
      let G;
      for (const J of H)
        if (J instanceof OutputNode) {
          let Z = J;
          for (; Z.numChildren() === 1; ) {
            const [te] = Z.children;
            if (te instanceof OutputNode)
              Z = te;
            else
              break;
          }
          q.push(...Z.children), G ? (_.removeChild(J), J.parent = G.parent, G.parent.removeChild(G), G.parent = Z, this.setModified()) : G = Z;
        } else
          q.push(J);
      if (q.length) {
        this.setModified();
        for (const J of q)
          J.parent.removeChild(J), J.parent = G;
      }
    }
  }
  class JoinAggregateTransformNode extends DataFlowNode {
    clone() {
      return new JoinAggregateTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H;
    }
    addDimensions(_) {
      this.transform.groupby = unique(this.transform.groupby.concat(_), (H) => H);
    }
    dependentFields() {
      const _ = /* @__PURE__ */ new Set();
      return this.transform.groupby && this.transform.groupby.forEach(_.add, _), this.transform.joinaggregate.map((H) => H.field).filter((H) => H !== void 0).forEach(_.add, _), _;
    }
    producedFields() {
      return new Set(this.transform.joinaggregate.map(this.getDefaultName));
    }
    getDefaultName(_) {
      return _.as ?? vgField(_);
    }
    hash() {
      return `JoinAggregateTransform ${hash(this.transform)}`;
    }
    assemble() {
      const _ = [], H = [], V = [];
      for (const G of this.transform.joinaggregate)
        H.push(G.op), V.push(this.getDefaultName(G)), _.push(G.field === void 0 ? null : G.field);
      const q = this.transform.groupby;
      return {
        type: "joinaggregate",
        as: V,
        ops: H,
        fields: _,
        ...q !== void 0 ? { groupby: q } : {}
      };
    }
  }
  class FilterInvalidNode extends DataFlowNode {
    clone() {
      return new FilterInvalidNode(null, { ...this.filter });
    }
    constructor(_, H) {
      super(_), this.filter = H;
    }
    static make(_, H, V) {
      const { config: q, markDef: G } = H, { marks: J, scales: Z } = V;
      if (J === "include-invalid-values" && Z === "include-invalid-values")
        return null;
      const te = H.reduceFieldDef((ne, ee, ae) => {
        const oe = isScaleChannel(ae) && H.getScaleComponent(ae);
        if (oe) {
          const se = oe.get("type"), { aggregate: ce } = ee, le = getScaleInvalidDataMode({
            scaleChannel: ae,
            markDef: G,
            config: q,
            scaleType: se,
            isCountAggregate: isCountingAggregateOp(ce)
          });
          le !== "show" && le !== "always-valid" && (ne[ee.field] = ee);
        }
        return ne;
      }, {});
      return keys$2(te).length ? new FilterInvalidNode(_, te) : null;
    }
    dependentFields() {
      return new Set(keys$2(this.filter));
    }
    producedFields() {
      return /* @__PURE__ */ new Set();
    }
    hash() {
      return `FilterInvalid ${hash(this.filter)}`;
    }
    /**
     * Create the VgTransforms for each of the filtered fields.
     */
    assemble() {
      const _ = keys$2(this.filter).reduce((H, V) => {
        const q = this.filter[V], G = vgField(q, { expr: "datum" });
        return q !== null && (q.type === "temporal" ? H.push(`(isDate(${G}) || (${isValidFiniteNumberExpr(G)}))`) : q.type === "quantitative" && H.push(isValidFiniteNumberExpr(G))), H;
      }, []);
      return _.length > 0 ? {
        type: "filter",
        expr: _.join(" && ")
      } : null;
    }
  }
  function isValidFiniteNumberExpr(O) {
    return `isValid(${O}) && isFinite(+${O})`;
  }
  function getStackByFields(O) {
    return O.stack.stackBy.reduce((_, H) => {
      const V = H.fieldDef, q = vgField(V);
      return q && _.push(q), _;
    }, []);
  }
  function isValidAsArray(O) {
    return isArray$2(O) && O.every((_) => isString$1(_)) && O.length > 1;
  }
  class StackNode extends DataFlowNode {
    clone() {
      return new StackNode(null, duplicate(this._stack));
    }
    constructor(_, H) {
      super(_), this._stack = H;
    }
    static makeFromTransform(_, H) {
      const { stack: V, groupby: q, as: G, offset: J = "zero" } = H, Z = [], te = [];
      if (H.sort !== void 0)
        for (const ae of H.sort)
          Z.push(ae.field), te.push(getFirstDefined(ae.order, "ascending"));
      const ne = {
        field: Z,
        order: te
      };
      let ee;
      return isValidAsArray(G) ? ee = G : isString$1(G) ? ee = [G, `${G}_end`] : ee = [`${H.stack}_start`, `${H.stack}_end`], new StackNode(_, {
        dimensionFieldDefs: [],
        stackField: V,
        groupby: q,
        offset: J,
        sort: ne,
        facetby: [],
        as: ee
      });
    }
    static makeFromEncoding(_, H) {
      const V = H.stack, { encoding: q } = H;
      if (!V)
        return null;
      const { groupbyChannels: G, fieldChannel: J, offset: Z, impute: te } = V, ne = G.map((se) => {
        const ce = q[se];
        return getFieldDef(ce);
      }).filter((se) => !!se), ee = getStackByFields(H), ae = H.encoding.order;
      let oe;
      if (isArray$2(ae) || isFieldDef(ae))
        oe = sortParams(ae);
      else {
        const se = isOrderOnlyDef(ae) ? ae.sort : J === "y" ? "descending" : "ascending";
        oe = ee.reduce((ce, le) => (ce.field.includes(le) || (ce.field.push(le), ce.order.push(se)), ce), { field: [], order: [] });
      }
      return new StackNode(_, {
        dimensionFieldDefs: ne,
        stackField: H.vgField(J),
        facetby: [],
        stackby: ee,
        sort: oe,
        offset: Z,
        impute: te,
        as: [
          H.vgField(J, { suffix: "start", forAs: !0 }),
          H.vgField(J, { suffix: "end", forAs: !0 })
        ]
      });
    }
    get stack() {
      return this._stack;
    }
    addDimensions(_) {
      this._stack.facetby.push(..._);
    }
    dependentFields() {
      const _ = /* @__PURE__ */ new Set();
      return _.add(this._stack.stackField), this.getGroupbyFields().forEach(_.add, _), this._stack.facetby.forEach(_.add, _), this._stack.sort.field.forEach(_.add, _), _;
    }
    producedFields() {
      return new Set(this._stack.as);
    }
    hash() {
      return `Stack ${hash(this._stack)}`;
    }
    getGroupbyFields() {
      const { dimensionFieldDefs: _, impute: H, groupby: V } = this._stack;
      return _.length > 0 ? _.map((q) => q.bin ? H ? [vgField(q, { binSuffix: "mid" })] : [
        // For binned group by field without impute, we need both bin (start) and bin_end
        vgField(q, {}),
        vgField(q, { binSuffix: "end" })
      ] : [vgField(q)]).flat() : V ?? [];
    }
    assemble() {
      const _ = [], { facetby: H, dimensionFieldDefs: V, stackField: q, stackby: G, sort: J, offset: Z, impute: te, as: ne } = this._stack;
      if (te)
        for (const ee of V) {
          const { bandPosition: ae = 0.5, bin: oe } = ee;
          if (oe) {
            const se = vgField(ee, { expr: "datum" }), ce = vgField(ee, { expr: "datum", binSuffix: "end" });
            _.push({
              type: "formula",
              expr: `${isValidFiniteNumberExpr(se)} ? ${ae}*${se}+${1 - ae}*${ce} : ${se}`,
              as: vgField(ee, { binSuffix: "mid", forAs: !0 })
            });
          }
          _.push({
            type: "impute",
            field: q,
            groupby: [...G, ...H],
            key: vgField(ee, { binSuffix: "mid" }),
            method: "value",
            value: 0
          });
        }
      return _.push({
        type: "stack",
        groupby: [...this.getGroupbyFields(), ...H],
        field: q,
        sort: J,
        as: ne,
        offset: Z
      }), _;
    }
  }
  class WindowTransformNode extends DataFlowNode {
    clone() {
      return new WindowTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H;
    }
    addDimensions(_) {
      this.transform.groupby = unique(this.transform.groupby.concat(_), (H) => H);
    }
    dependentFields() {
      const _ = /* @__PURE__ */ new Set();
      return (this.transform.groupby ?? []).forEach(_.add, _), (this.transform.sort ?? []).forEach((H) => _.add(H.field)), this.transform.window.map((H) => H.field).filter((H) => H !== void 0).forEach(_.add, _), _;
    }
    producedFields() {
      return new Set(this.transform.window.map(this.getDefaultName));
    }
    getDefaultName(_) {
      return _.as ?? vgField(_);
    }
    hash() {
      return `WindowTransform ${hash(this.transform)}`;
    }
    assemble() {
      const _ = [], H = [], V = [], q = [];
      for (const ae of this.transform.window)
        H.push(ae.op), V.push(this.getDefaultName(ae)), q.push(ae.param === void 0 ? null : ae.param), _.push(ae.field === void 0 ? null : ae.field);
      const G = this.transform.frame, J = this.transform.groupby;
      if (G && G[0] === null && G[1] === null && H.every((ae) => isAggregateOp(ae)))
        return {
          type: "joinaggregate",
          as: V,
          ops: H,
          fields: _,
          ...J !== void 0 ? { groupby: J } : {}
        };
      const Z = [], te = [];
      if (this.transform.sort !== void 0)
        for (const ae of this.transform.sort)
          Z.push(ae.field), te.push(ae.order ?? "ascending");
      const ne = {
        field: Z,
        order: te
      }, ee = this.transform.ignorePeers;
      return {
        type: "window",
        params: q,
        as: V,
        ops: H,
        fields: _,
        sort: ne,
        ...ee !== void 0 ? { ignorePeers: ee } : {},
        ...J !== void 0 ? { groupby: J } : {},
        ...G !== void 0 ? { frame: G } : {}
      };
    }
  }
  function cloneSubtree(O) {
    function _(H) {
      if (!(H instanceof FacetNode)) {
        const V = H.clone();
        if (V instanceof OutputNode) {
          const q = FACET_SCALE_PREFIX + V.getSource();
          V.setSource(q), O.model.component.data.outputNodes[q] = V;
        } else (V instanceof AggregateNode || V instanceof StackNode || V instanceof WindowTransformNode || V instanceof JoinAggregateTransformNode) && V.addDimensions(O.fields);
        for (const q of H.children.flatMap(_))
          q.parent = V;
        return [V];
      }
      return H.children.flatMap(_);
    }
    return _;
  }
  function moveFacetDown(O) {
    if (O instanceof FacetNode)
      if (O.numChildren() === 1 && !(O.children[0] instanceof OutputNode)) {
        const _ = O.children[0];
        (_ instanceof AggregateNode || _ instanceof StackNode || _ instanceof WindowTransformNode || _ instanceof JoinAggregateTransformNode) && _.addDimensions(O.fields), _.swapWithParent(), moveFacetDown(O);
      } else {
        const _ = O.model.component.data.main;
        moveMainDownToFacet(_);
        const H = cloneSubtree(O), V = O.children.map(H).flat();
        for (const q of V)
          q.parent = _;
      }
    else
      O.children.map(moveFacetDown);
  }
  function moveMainDownToFacet(O) {
    if (O instanceof OutputNode && O.type === DataSourceType.Main && O.numChildren() === 1) {
      const _ = O.children[0];
      _ instanceof FacetNode || (_.swapWithParent(), moveMainDownToFacet(O));
    }
  }
  const FACET_SCALE_PREFIX = "scale_", MAX_OPTIMIZATION_RUNS = 5;
  function checkLinks(O) {
    for (const _ of O) {
      for (const H of _.children)
        if (H.parent !== _)
          return !1;
      if (!checkLinks(_.children))
        return !1;
    }
    return !0;
  }
  function runOptimizer(O, _) {
    let H = !1;
    for (const V of _)
      H = O.optimize(V) || H;
    return H;
  }
  function optimizationDataflowHelper(O, _, H) {
    let V = O.sources, q = !1;
    return q = runOptimizer(new RemoveUnnecessaryOutputNodes(), V) || q, q = runOptimizer(new RemoveUnnecessaryIdentifierNodes(_), V) || q, V = V.filter((G) => G.numChildren() > 0), q = runOptimizer(new RemoveUnusedSubtrees(), V) || q, V = V.filter((G) => G.numChildren() > 0), H || (q = runOptimizer(new MoveParseUp(), V) || q, q = runOptimizer(new MergeBins(_), V) || q, q = runOptimizer(new RemoveDuplicateTimeUnits(), V) || q, q = runOptimizer(new MergeParse(), V) || q, q = runOptimizer(new MergeAggregates(), V) || q, q = runOptimizer(new MergeTimeUnits(), V) || q, q = runOptimizer(new MergeIdenticalNodes(), V) || q, q = runOptimizer(new MergeOutputs(), V) || q), O.sources = V, q;
  }
  function optimizeDataflow(O, _) {
    checkLinks(O.sources);
    let H = 0, V = 0;
    for (let q = 0; q < MAX_OPTIMIZATION_RUNS && optimizationDataflowHelper(O, _, !0); q++)
      H++;
    O.sources.map(moveFacetDown);
    for (let q = 0; q < MAX_OPTIMIZATION_RUNS && optimizationDataflowHelper(O, _, !1); q++)
      V++;
    checkLinks(O.sources), Math.max(H, V) === MAX_OPTIMIZATION_RUNS && warn(`Maximum optimization runs(${MAX_OPTIMIZATION_RUNS}) reached.`);
  }
  class SignalRefWrapper {
    constructor(_) {
      Object.defineProperty(this, "signal", {
        enumerable: !0,
        get: _
      });
    }
    static fromName(_, H) {
      return new SignalRefWrapper(() => _(H));
    }
  }
  function parseScaleDomain(O) {
    isUnitModel(O) ? parseUnitScaleDomain(O) : parseNonUnitScaleDomain(O);
  }
  function parseUnitScaleDomain(O) {
    const _ = O.component.scales;
    for (const H of keys$2(_)) {
      const V = parseDomainForChannel(O, H);
      if (_[H].setWithExplicit("domains", V), parseSelectionDomain(O, H), O.component.data.isFaceted) {
        let G = O;
        for (; !isFacetModel(G) && G.parent; )
          G = G.parent;
        if (G.component.resolve.scale[H] === "shared")
          for (const Z of V.value)
            isDataRefDomain(Z) && (Z.data = FACET_SCALE_PREFIX + Z.data.replace(FACET_SCALE_PREFIX, ""));
      }
    }
  }
  function parseNonUnitScaleDomain(O) {
    for (const H of O.children)
      parseScaleDomain(H);
    const _ = O.component.scales;
    for (const H of keys$2(_)) {
      let V, q = null;
      for (const G of O.children) {
        const J = G.component.scales[H];
        if (J) {
          V === void 0 ? V = J.getWithExplicit("domains") : V = mergeValuesWithExplicit(V, J.getWithExplicit("domains"), "domains", "scale", domainsTieBreaker);
          const Z = J.get("selectionExtent");
          q && Z && q.param !== Z.param && warn(NEEDS_SAME_SELECTION), q = Z;
        }
      }
      _[H].setWithExplicit("domains", V), q && _[H].set("selectionExtent", q, !0);
    }
  }
  function normalizeUnaggregatedDomain(O, _, H, V) {
    if (O === "unaggregated") {
      const { valid: q, reason: G } = canUseUnaggregatedDomain(_, H);
      if (!q) {
        warn(G);
        return;
      }
    } else if (O === void 0 && V.useUnaggregatedDomain) {
      const { valid: q } = canUseUnaggregatedDomain(_, H);
      if (q)
        return "unaggregated";
    }
    return O;
  }
  function parseDomainForChannel(O, _) {
    const H = O.getScaleComponent(_).get("type"), { encoding: V } = O, q = normalizeUnaggregatedDomain(O.scaleDomain(_), O.typedFieldDef(_), H, O.config.scale);
    return q !== O.scaleDomain(_) && (O.specifiedScales[_] = {
      ...O.specifiedScales[_],
      domain: q
    }), _ === "x" && getFieldOrDatumDef(V.x2) ? getFieldOrDatumDef(V.x) ? mergeValuesWithExplicit(parseSingleChannelDomain(H, q, O, "x"), parseSingleChannelDomain(H, q, O, "x2"), "domain", "scale", domainsTieBreaker) : parseSingleChannelDomain(H, q, O, "x2") : _ === "y" && getFieldOrDatumDef(V.y2) ? getFieldOrDatumDef(V.y) ? mergeValuesWithExplicit(parseSingleChannelDomain(H, q, O, "y"), parseSingleChannelDomain(H, q, O, "y2"), "domain", "scale", domainsTieBreaker) : parseSingleChannelDomain(H, q, O, "y2") : parseSingleChannelDomain(H, q, O, _);
  }
  function mapDomainToDataSignal(O, _, H) {
    return O.map((V) => ({ signal: `{data: ${valueExpr(V, { timeUnit: H, type: _ })}}` }));
  }
  function convertDomainIfItIsDateTime(O, _, H) {
    var q;
    const V = (q = normalizeTimeUnit(H)) == null ? void 0 : q.unit;
    return _ === "temporal" || V ? mapDomainToDataSignal(O, _, V) : [O];
  }
  function parseSingleChannelDomain(O, _, H, V) {
    const { encoding: q, markDef: G, mark: J, config: Z, stack: te } = H, ne = getFieldOrDatumDef(q[V]), { type: ee } = ne, ae = ne.timeUnit, oe = getScaleDataSourceForHandlingInvalidValues({
      invalid: getMarkConfig("invalid", G, Z),
      isPath: isPathMark(J)
    });
    if (isDomainUnionWith(_)) {
      const le = parseSingleChannelDomain(O, void 0, H, V), ue = convertDomainIfItIsDateTime(_.unionWith, ee, ae);
      return makeExplicit([...ue, ...le.value]);
    } else {
      if (isSignalRef(_))
        return makeExplicit([_]);
      if (_ && _ !== "unaggregated" && !isParameterDomain(_))
        return makeExplicit(convertDomainIfItIsDateTime(_, ee, ae));
    }
    if (te && V === te.fieldChannel) {
      if (te.offset === "normalize")
        return makeImplicit([[0, 1]]);
      const le = H.requestDataName(oe);
      return makeImplicit([
        {
          data: le,
          field: H.vgField(V, { suffix: "start" })
        },
        {
          data: le,
          field: H.vgField(V, { suffix: "end" })
        }
      ]);
    }
    const se = isScaleChannel(V) && isFieldDef(ne) ? domainSort(H, V, O) : void 0;
    if (isDatumDef(ne)) {
      const le = convertDomainIfItIsDateTime([ne.datum], ee, ae);
      return makeImplicit(le);
    }
    const ce = ne;
    if (_ === "unaggregated") {
      const { field: le } = ne;
      return makeImplicit([
        {
          data: H.requestDataName(oe),
          field: vgField({ field: le, aggregate: "min" })
        },
        {
          data: H.requestDataName(oe),
          field: vgField({ field: le, aggregate: "max" })
        }
      ]);
    } else if (isBinning(ce.bin)) {
      if (hasDiscreteDomain(O))
        return makeImplicit(O === "bin-ordinal" ? [] : [
          {
            // If sort by aggregation of a specified sort field, we need to use RAW table,
            // so we can aggregate values for the scale independently from the main aggregation.
            data: isBoolean$1(se) ? H.requestDataName(oe) : H.requestDataName(DataSourceType.Raw),
            // Use range if we added it and the scale does not support computing a range as a signal.
            field: H.vgField(V, binRequiresRange(ce, V) ? { binSuffix: "range" } : {}),
            // we have to use a sort object if sort = true to make the sort correct by bin start
            sort: se === !0 || !isObject$2(se) ? {
              field: H.vgField(V, {}),
              op: "min"
              // min or max doesn't matter since we sort by the start of the bin range
            } : se
          }
        ]);
      {
        const { bin: le } = ce;
        if (isBinning(le)) {
          const ue = getBinSignalName(H, ce.field, le);
          return makeImplicit([
            new SignalRefWrapper(() => {
              const de = H.getSignalName(ue);
              return `[${de}.start, ${de}.stop]`;
            })
          ]);
        } else
          return makeImplicit([
            {
              data: H.requestDataName(oe),
              field: H.vgField(V, {})
            }
          ]);
      }
    } else if (ce.timeUnit && contains$2(["time", "utc"], O)) {
      const le = q[getSecondaryRangeChannel(V)];
      if (hasBandEnd(ce, le, G, Z)) {
        const ue = H.requestDataName(oe), de = getBandPosition({ fieldDef: ce, fieldDef2: le, markDef: G, config: Z }), fe = isRectBasedMark(J) && de !== 0.5 && isXorY(V);
        return makeImplicit([
          {
            data: ue,
            field: H.vgField(V, fe ? { suffix: OFFSETTED_RECT_START_SUFFIX } : {})
          },
          {
            data: ue,
            field: H.vgField(V, { suffix: fe ? OFFSETTED_RECT_END_SUFFIX : "end" })
          }
        ]);
      }
    }
    return makeImplicit(se ? [
      {
        // If sort by aggregation of a specified sort field, we need to use RAW table,
        // so we can aggregate values for the scale independently from the main aggregation.
        data: isBoolean$1(se) ? H.requestDataName(oe) : H.requestDataName(DataSourceType.Raw),
        field: H.vgField(V),
        sort: se
      }
    ] : [
      {
        data: H.requestDataName(oe),
        field: H.vgField(V)
      }
    ]);
  }
  function normalizeSortField(O, _) {
    const { op: H, field: V, order: q } = O;
    return {
      // Apply default op
      op: H ?? (_ ? "sum" : DEFAULT_SORT_OP),
      // flatten nested fields
      ...V ? { field: replacePathInField(V) } : {},
      ...q ? { order: q } : {}
    };
  }
  function parseSelectionDomain(O, _) {
    var Z;
    const H = O.component.scales[_], V = O.specifiedScales[_].domain, q = (Z = O.fieldDef(_)) == null ? void 0 : Z.bin, G = isParameterDomain(V) ? V : void 0, J = isBinParams(q) && isParameterExtent(q.extent) ? q.extent : void 0;
    (G || J) && H.set("selectionExtent", G ?? J, !0);
  }
  function domainSort(O, _, H) {
    if (!hasDiscreteDomain(H))
      return;
    const V = O.fieldDef(_), q = V.sort;
    if (isSortArray(q))
      return {
        op: "min",
        field: sortArrayIndexField(V, _),
        order: "ascending"
      };
    const { stack: G } = O, J = G ? /* @__PURE__ */ new Set([...G.groupbyFields, ...G.stackBy.map((Z) => Z.fieldDef.field)]) : void 0;
    if (isSortField(q)) {
      const Z = G && !J.has(q.field);
      return normalizeSortField(q, Z);
    } else if (isSortByEncoding(q)) {
      const { encoding: Z, order: te } = q, ne = O.fieldDef(Z), { aggregate: ee, field: ae } = ne, oe = G && !J.has(ae);
      if (isArgminDef(ee) || isArgmaxDef(ee))
        return normalizeSortField({
          field: vgField(ne),
          order: te
        }, oe);
      if (isAggregateOp(ee) || !ee)
        return normalizeSortField({
          op: ee,
          // can't be argmin/argmax since we don't support them in encoding field def
          field: ae,
          order: te
        }, oe);
    } else {
      if (q === "descending")
        return {
          op: "min",
          field: O.vgField(_),
          order: "descending"
        };
      if (contains$2([
        "ascending",
        void 0
        /* default =ascending*/
      ], q))
        return !0;
    }
  }
  function canUseUnaggregatedDomain(O, _) {
    const { aggregate: H, type: V } = O;
    return H ? isString$1(H) && !SHARED_DOMAIN_OPS.has(H) ? {
      valid: !1,
      reason: unaggregateDomainWithNonSharedDomainOp(H)
    } : V === "quantitative" && _ === "log" ? {
      valid: !1,
      reason: unaggregatedDomainWithLogScale(O)
    } : { valid: !0 } : {
      valid: !1,
      reason: unaggregateDomainHasNoEffectForRawField(O)
    };
  }
  function domainsTieBreaker(O, _, H, V) {
    return O.explicit && _.explicit && warn(mergeConflictingDomainProperty(H, V, O.value, _.value)), { explicit: O.explicit, value: [...O.value, ..._.value] };
  }
  function mergeDomains(O) {
    const _ = unique(O.map((J) => {
      if (isDataRefDomain(J)) {
        const { sort: Z, ...te } = J;
        return te;
      }
      return J;
    }), hash), H = unique(O.map((J) => {
      if (isDataRefDomain(J)) {
        const Z = J.sort;
        return Z !== void 0 && !isBoolean$1(Z) && ("op" in Z && Z.op === "count" && delete Z.field, Z.order === "ascending" && delete Z.order), Z;
      }
    }).filter((J) => J !== void 0), hash);
    if (_.length === 0)
      return;
    if (_.length === 1) {
      const J = O[0];
      if (isDataRefDomain(J) && H.length > 0) {
        let Z = H[0];
        if (H.length > 1) {
          warn(MORE_THAN_ONE_SORT);
          const te = H.filter((ne) => isObject$2(ne) && "op" in ne && ne.op !== "min");
          H.every((ne) => isObject$2(ne) && "op" in ne) && te.length === 1 ? Z = te[0] : Z = !0;
        } else if (isObject$2(Z) && "field" in Z) {
          const te = Z.field;
          J.field === te && (Z = Z.order ? { order: Z.order } : !0);
        }
        return {
          ...J,
          sort: Z
        };
      }
      return J;
    }
    const V = unique(H.map((J) => isBoolean$1(J) || !("op" in J) || isString$1(J.op) && has$1(MULTIDOMAIN_SORT_OP_INDEX, J.op) ? J : (warn(domainSortDropped(J)), !0)), hash);
    let q;
    V.length === 1 ? q = V[0] : V.length > 1 && (warn(MORE_THAN_ONE_SORT), q = !0);
    const G = unique(O.map((J) => isDataRefDomain(J) ? J.data : null), (J) => J);
    return G.length === 1 && G[0] !== null ? {
      data: G[0],
      fields: _.map((Z) => Z.field),
      ...q ? { sort: q } : {}
    } : { fields: _, ...q ? { sort: q } : {} };
  }
  function getFieldFromDomain(O) {
    if (isDataRefDomain(O) && isString$1(O.field))
      return O.field;
    if (isDataRefUnionedDomain(O)) {
      let _;
      for (const H of O.fields)
        if (isDataRefDomain(H) && isString$1(H.field)) {
          if (!_)
            _ = H.field;
          else if (_ !== H.field)
            return warn(FACETED_INDEPENDENT_DIFFERENT_SOURCES), _;
        }
      return warn(FACETED_INDEPENDENT_SAME_FIELDS_DIFFERENT_SOURCES), _;
    } else if (isFieldRefUnionDomain(O)) {
      warn(FACETED_INDEPENDENT_SAME_SOURCE);
      const _ = O.fields[0];
      return isString$1(_) ? _ : void 0;
    }
  }
  function assembleDomain(O, _) {
    const V = O.component.scales[_].get("domains").map((q) => (isDataRefDomain(q) && (q.data = O.lookupDataSource(q.data)), q));
    return mergeDomains(V);
  }
  function assembleScales(O) {
    return isLayerModel(O) || isConcatModel(O) ? O.children.reduce((_, H) => _.concat(assembleScales(H)), assembleScalesForModel(O)) : assembleScalesForModel(O);
  }
  function assembleScalesForModel(O) {
    return keys$2(O.component.scales).reduce((_, H) => {
      const V = O.component.scales[H];
      if (V.merged)
        return _;
      const q = V.combine(), { name: G, type: J, selectionExtent: Z, domains: te, range: ne, reverse: ee, ...ae } = q, oe = assembleScaleRange(q.range, G, H, O), se = assembleDomain(O, H), ce = Z ? assembleSelectionScaleDomain(O, Z, V, se) : null;
      return _.push({
        name: G,
        type: J,
        ...se ? { domain: se } : {},
        ...ce ? { domainRaw: ce } : {},
        range: oe,
        ...ee !== void 0 ? { reverse: ee } : {},
        ...ae
      }), _;
    }, []);
  }
  function assembleScaleRange(O, _, H, V) {
    if (isXorY(H)) {
      if (isVgRangeStep(O))
        return {
          step: { signal: `${_}_step` }
        };
    } else if (isObject$2(O) && isDataRefDomain(O))
      return {
        ...O,
        data: V.lookupDataSource(O.data)
      };
    return O;
  }
  class ScaleComponent extends Split {
    constructor(_, H) {
      super(
        {},
        // no initial explicit property
        { name: _ }
        // name as initial implicit property
      ), this.merged = !1, this.setWithExplicit("type", H);
    }
    /**
     * Whether the scale definitely includes or not include zero in the domain
     */
    domainHasZero() {
      const _ = this.get("type");
      if (contains$2([ScaleType.LOG, ScaleType.TIME, ScaleType.UTC], _))
        return "definitely-not";
      const H = this.get("zero");
      if (H === !0 || // If zero is undefined, linear/sqrt/pow scales have zero by default.
      H === void 0 && contains$2([ScaleType.LINEAR, ScaleType.SQRT, ScaleType.POW], _))
        return "definitely";
      const V = this.get("domains");
      if (V.length > 0) {
        let q = !1, G = !1, J = !1;
        for (const Z of V) {
          if (isArray$2(Z)) {
            const te = Z[0], ne = Z[Z.length - 1];
            if (isNumber$2(te) && isNumber$2(ne))
              if (te <= 0 && ne >= 0) {
                q = !0;
                continue;
              } else {
                G = !0;
                continue;
              }
          }
          J = !0;
        }
        if (q)
          return "definitely";
        if (G && !J)
          return "definitely-not";
      }
      return "maybe";
    }
  }
  const RANGE_PROPERTIES = ["range", "scheme"];
  function parseUnitScaleRange(O) {
    const _ = O.component.scales;
    for (const H of SCALE_CHANNELS) {
      const V = _[H];
      if (!V)
        continue;
      const q = parseRangeForChannel(H, O);
      V.setWithExplicit("range", q);
    }
  }
  function getBinStepSignal(O, _) {
    const H = O.fieldDef(_);
    if (H != null && H.bin) {
      const { bin: V, field: q } = H, G = getSizeChannel(_), J = O.getName(G);
      if (isObject$2(V) && V.binned && V.step !== void 0)
        return new SignalRefWrapper(() => {
          const Z = O.scaleName(_), te = `(domain("${Z}")[1] - domain("${Z}")[0]) / ${V.step}`;
          return `${O.getSignalName(J)} / (${te})`;
        });
      if (isBinning(V)) {
        const Z = getBinSignalName(O, q, V);
        return new SignalRefWrapper(() => {
          const te = O.getSignalName(Z), ne = `(${te}.stop - ${te}.start) / ${te}.step`;
          return `${O.getSignalName(J)} / (${ne})`;
        });
      }
    }
  }
  function parseRangeForChannel(O, _) {
    const H = _.specifiedScales[O], { size: V } = _, G = _.getScaleComponent(O).get("type");
    for (const ae of RANGE_PROPERTIES)
      if (H[ae] !== void 0) {
        const oe = scaleTypeSupportProperty(G, ae), se = channelScalePropertyIncompatability(O, ae);
        if (!oe)
          warn(scalePropertyNotWorkWithScaleType(G, ae, O));
        else if (se)
          warn(se);
        else
          switch (ae) {
            case "range": {
              const ce = H.range;
              if (isArray$2(ce)) {
                if (isXorY(O))
                  return makeExplicit(ce.map((le) => {
                    if (le === "width" || le === "height") {
                      const ue = _.getName(le), de = _.getSignalName.bind(_);
                      return SignalRefWrapper.fromName(de, ue);
                    }
                    return le;
                  }));
              } else if (isObject$2(ce))
                return makeExplicit({
                  data: _.requestDataName(DataSourceType.Main),
                  field: ce.field,
                  sort: { op: "min", field: _.vgField(O) }
                });
              return makeExplicit(ce);
            }
            case "scheme":
              return makeExplicit(parseScheme(H[ae]));
          }
      }
    const J = O === X || O === "xOffset" ? "width" : "height", Z = V[J];
    if (isStep(Z)) {
      if (isXorY(O))
        if (hasDiscreteDomain(G)) {
          const ae = getPositionStep(Z, _, O);
          if (ae)
            return makeExplicit({ step: ae });
        } else
          warn(stepDropped(J));
      else if (isXorYOffset(O)) {
        const ae = O === XOFFSET ? "x" : "y";
        if (_.getScaleComponent(ae).get("type") === "band") {
          const ce = getOffsetStep(Z, G);
          if (ce)
            return makeExplicit(ce);
        }
      }
    }
    const { rangeMin: te, rangeMax: ne } = H, ee = defaultRange(O, _);
    return (te !== void 0 || ne !== void 0) && // it's ok to check just rangeMin's compatibility since rangeMin/rangeMax are the same
    scaleTypeSupportProperty(G, "rangeMin") && isArray$2(ee) && ee.length === 2 ? makeExplicit([te ?? ee[0], ne ?? ee[1]]) : makeImplicit(ee);
  }
  function parseScheme(O) {
    return isExtendedScheme(O) ? {
      scheme: O.name,
      ...omit(O, ["name"])
    } : { scheme: O };
  }
  function fullWidthOrHeightRange(O, _, H, { center: V } = {}) {
    const q = getSizeChannel(O), G = _.getName(q), J = _.getSignalName.bind(_);
    return O === Y$1 && hasContinuousDomain(H) ? V ? [
      SignalRefWrapper.fromName((Z) => `${J(Z)}/2`, G),
      SignalRefWrapper.fromName((Z) => `-${J(Z)}/2`, G)
    ] : [SignalRefWrapper.fromName(J, G), 0] : V ? [
      SignalRefWrapper.fromName((Z) => `-${J(Z)}/2`, G),
      SignalRefWrapper.fromName((Z) => `${J(Z)}/2`, G)
    ] : [0, SignalRefWrapper.fromName(J, G)];
  }
  function defaultRange(O, _) {
    const { size: H, config: V, mark: q, encoding: G } = _, { type: J } = getFieldOrDatumDef(G[O]), te = _.getScaleComponent(O).get("type"), { domain: ne, domainMid: ee } = _.specifiedScales[O];
    switch (O) {
      case X:
      case Y$1: {
        if (contains$2(["point", "band"], te)) {
          const ae = getDiscretePositionSize(O, H, V.view);
          if (isStep(ae))
            return { step: getPositionStep(ae, _, O) };
        }
        return fullWidthOrHeightRange(O, _, te);
      }
      case XOFFSET:
      case YOFFSET:
        return getOffsetRange(O, _, te);
      case SIZE: {
        const ae = sizeRangeMin(q, V), oe = sizeRangeMax(q, H, _, V);
        return isContinuousToDiscrete(te) ? interpolateRange(ae, oe, defaultContinuousToDiscreteCount(te, V, ne, O)) : [ae, oe];
      }
      case THETA:
        return [0, Math.PI * 2];
      case ANGLE:
        return [0, 360];
      case RADIUS:
        return [
          0,
          new SignalRefWrapper(() => {
            const ae = _.getSignalName(isFacetModel(_.parent) ? "child_width" : "width"), oe = _.getSignalName(isFacetModel(_.parent) ? "child_height" : "height");
            return `min(${ae},${oe})/2`;
          })
        ];
      case TIME:
        return { step: 1e3 / V.scale.framesPerSecond };
      case STROKEWIDTH:
        return [V.scale.minStrokeWidth, V.scale.maxStrokeWidth];
      case STROKEDASH:
        return [
          // TODO: add this to Vega's config.range?
          [1, 0],
          [4, 2],
          [2, 1],
          [1, 1],
          [1, 2, 4, 2]
        ];
      case SHAPE:
        return "symbol";
      case COLOR:
      case FILL:
      case STROKE:
        return te === "ordinal" ? J === "nominal" ? "category" : "ordinal" : ee !== void 0 ? "diverging" : q === "rect" || q === "geoshape" ? "heatmap" : "ramp";
      case OPACITY:
      case FILLOPACITY:
      case STROKEOPACITY:
        return [V.scale.minOpacity, V.scale.maxOpacity];
    }
  }
  function getPositionStep(O, _, H) {
    const { encoding: V } = _, q = _.getScaleComponent(H), G = getOffsetScaleChannel(H), J = V[G];
    if (getStepFor({ step: O, offsetIsDiscrete: isFieldOrDatumDef(J) && isDiscrete$1(J.type) }) === "offset" && channelHasFieldOrDatum(V, G)) {
      const te = _.getScaleComponent(G);
      let ee = `domain('${_.scaleName(G)}').length`;
      if (te.get("type") === "band") {
        const oe = te.get("paddingInner") ?? te.get("padding") ?? 0, se = te.get("paddingOuter") ?? te.get("padding") ?? 0;
        ee = `bandspace(${ee}, ${oe}, ${se})`;
      }
      const ae = q.get("paddingInner") ?? q.get("padding");
      return {
        signal: `${O.step} * ${ee} / (1-${exprFromSignalRefOrValue(ae)})`
      };
    } else
      return O.step;
  }
  function getOffsetStep(O, _) {
    if (getStepFor({ step: O, offsetIsDiscrete: hasDiscreteDomain(_) }) === "offset")
      return { step: O.step };
  }
  function getOffsetRange(O, _, H) {
    const V = O === XOFFSET ? "x" : "y", q = _.getScaleComponent(V);
    if (!q)
      return fullWidthOrHeightRange(V, _, H, { center: !0 });
    const G = q.get("type"), J = _.scaleName(V), { markDef: Z, config: te } = _;
    if (G === "band") {
      const ne = getDiscretePositionSize(V, _.size, _.config.view);
      if (isStep(ne)) {
        const ee = getOffsetStep(ne, H);
        if (ee)
          return ee;
      }
      return [0, { signal: `bandwidth('${J}')` }];
    } else {
      const ne = _.encoding[V];
      if (isFieldDef(ne) && ne.timeUnit) {
        const ee = durationExpr(ne.timeUnit, (ce) => `scale('${J}', ${ce})`), ae = _.config.scale.bandWithNestedOffsetPaddingInner, oe = getBandPosition({
          fieldDef: ne,
          markDef: Z,
          config: te
        }) - 0.5, se = oe !== 0 ? ` + ${oe}` : "";
        if (ae) {
          const ce = isSignalRef(ae) ? `${ae.signal}/2` + se : `${ae / 2 + oe}`, le = isSignalRef(ae) ? `(1 - ${ae.signal}/2)` + se : `${1 - ae / 2 + oe}`;
          return [{ signal: `${ce} * (${ee})` }, { signal: `${le} * (${ee})` }];
        }
        return [0, { signal: ee }];
      }
      return never(`Cannot use ${O} scale if ${V} scale is not discrete.`);
    }
  }
  function getDiscretePositionSize(O, _, H) {
    const V = O === X ? "width" : "height", q = _[V];
    return q || getViewConfigDiscreteSize(H, V);
  }
  function defaultContinuousToDiscreteCount(O, _, H, V) {
    switch (O) {
      case "quantile":
        return _.scale.quantileCount;
      case "quantize":
        return _.scale.quantizeCount;
      case "threshold":
        return H !== void 0 && isArray$2(H) ? H.length + 1 : (warn(domainRequiredForThresholdScale(V)), 3);
    }
  }
  function interpolateRange(O, _, H) {
    const V = () => {
      const q = signalOrStringValue(_), G = signalOrStringValue(O), J = `(${q} - ${G}) / (${H} - 1)`;
      return `sequence(${G}, ${q} + ${J}, ${J})`;
    };
    return isSignalRef(_) ? new SignalRefWrapper(V) : { signal: V() };
  }
  function sizeRangeMin(O, _) {
    switch (O) {
      case "bar":
      case "tick":
        return _.scale.minBandSize;
      case "line":
      case "trail":
      case "rule":
        return _.scale.minStrokeWidth;
      case "text":
        return _.scale.minFontSize;
      case "point":
      case "square":
      case "circle":
        return _.scale.minSize;
    }
    throw new Error(incompatibleChannel("size", O));
  }
  const MAX_SIZE_RANGE_STEP_RATIO = 0.95;
  function sizeRangeMax(O, _, H, V) {
    const q = {
      x: getBinStepSignal(H, "x"),
      y: getBinStepSignal(H, "y")
    };
    switch (O) {
      case "bar":
      case "tick": {
        if (V.scale.maxBandSize !== void 0)
          return V.scale.maxBandSize;
        const G = minXYStep(_, q, V.view);
        return isNumber$2(G) ? G - 1 : new SignalRefWrapper(() => `${G.signal} - 1`);
      }
      case "line":
      case "trail":
      case "rule":
        return V.scale.maxStrokeWidth;
      case "text":
        return V.scale.maxFontSize;
      case "point":
      case "square":
      case "circle": {
        if (V.scale.maxSize)
          return V.scale.maxSize;
        const G = minXYStep(_, q, V.view);
        return isNumber$2(G) ? Math.pow(MAX_SIZE_RANGE_STEP_RATIO * G, 2) : new SignalRefWrapper(() => `pow(${MAX_SIZE_RANGE_STEP_RATIO} * ${G.signal}, 2)`);
      }
    }
    throw new Error(incompatibleChannel("size", O));
  }
  function minXYStep(O, _, H) {
    const V = isStep(O.width) ? O.width.step : getViewConfigDiscreteStep(H, "width"), q = isStep(O.height) ? O.height.step : getViewConfigDiscreteStep(H, "height");
    return _.x || _.y ? new SignalRefWrapper(() => `min(${[
      _.x ? _.x.signal : V,
      _.y ? _.y.signal : q
    ].join(", ")})`) : Math.min(V, q);
  }
  function parseScaleProperty(O, _) {
    isUnitModel(O) ? parseUnitScaleProperty(O, _) : parseNonUnitScaleProperty(O, _);
  }
  function parseUnitScaleProperty(O, _) {
    const H = O.component.scales, { config: V, encoding: q, markDef: G, specifiedScales: J } = O;
    for (const Z of keys$2(H)) {
      const te = J[Z], ne = H[Z], ee = O.getScaleComponent(Z), ae = getFieldOrDatumDef(q[Z]), oe = te[_], se = ee.get("type"), ce = ee.get("padding"), le = ee.get("paddingInner"), ue = scaleTypeSupportProperty(se, _), de = channelScalePropertyIncompatability(Z, _);
      if (oe !== void 0 && (ue ? de && warn(de) : warn(scalePropertyNotWorkWithScaleType(se, _, Z))), ue && de === void 0)
        if (oe !== void 0) {
          const fe = ae.timeUnit, he = ae.type;
          switch (_) {
            // domainMax/Min to signal if the value is a datetime object
            case "domainMax":
            case "domainMin":
              isDateTime(te[_]) || he === "temporal" || fe ? ne.set(_, { signal: valueExpr(te[_], { type: he, timeUnit: fe }) }, !0) : ne.set(_, te[_], !0);
              break;
            default:
              ne.copyKeyFromObject(_, te);
          }
        } else {
          const fe = hasProperty(scaleRules, _) ? scaleRules[_]({
            model: O,
            channel: Z,
            fieldOrDatumDef: ae,
            scaleType: se,
            scalePadding: ce,
            scalePaddingInner: le,
            domain: te.domain,
            domainMin: te.domainMin,
            domainMax: te.domainMax,
            markDef: G,
            config: V,
            hasNestedOffsetScale: channelHasNestedOffsetScale(q, Z),
            hasSecondaryRangeChannel: !!q[getSecondaryRangeChannel(Z)]
          }) : V.scale[_];
          fe !== void 0 && ne.set(_, fe, !1);
        }
    }
  }
  const scaleRules = {
    bins: ({ model: O, fieldOrDatumDef: _ }) => isFieldDef(_) ? bins(O, _) : void 0,
    interpolate: ({ channel: O, fieldOrDatumDef: _ }) => interpolate(O, _.type),
    nice: ({ scaleType: O, channel: _, domain: H, domainMin: V, domainMax: q, fieldOrDatumDef: G }) => nice(O, _, H, V, q, G),
    padding: ({ channel: O, scaleType: _, fieldOrDatumDef: H, markDef: V, config: q }) => padding(O, _, q.scale, H, V, q.bar),
    paddingInner: ({ scalePadding: O, channel: _, markDef: H, scaleType: V, config: q, hasNestedOffsetScale: G }) => paddingInner(O, _, H.type, V, q.scale, G),
    paddingOuter: ({ scalePadding: O, channel: _, scaleType: H, scalePaddingInner: V, config: q, hasNestedOffsetScale: G }) => paddingOuter(O, _, H, V, q.scale, G),
    reverse: ({ fieldOrDatumDef: O, scaleType: _, channel: H, config: V }) => {
      const q = isFieldDef(O) ? O.sort : void 0;
      return reverse$1(_, q, H, V.scale);
    },
    zero: ({ channel: O, fieldOrDatumDef: _, domain: H, markDef: V, scaleType: q, config: G, hasSecondaryRangeChannel: J }) => zero(O, _, H, V, q, G.scale, J)
  };
  function parseScaleRange(O) {
    isUnitModel(O) ? parseUnitScaleRange(O) : parseNonUnitScaleProperty(O, "range");
  }
  function parseNonUnitScaleProperty(O, _) {
    const H = O.component.scales;
    for (const V of O.children)
      _ === "range" ? parseScaleRange(V) : parseScaleProperty(V, _);
    for (const V of keys$2(H)) {
      let q;
      for (const G of O.children) {
        const J = G.component.scales[V];
        if (J) {
          const Z = J.getWithExplicit(_);
          q = mergeValuesWithExplicit(q, Z, _, "scale", tieBreakByComparing((te, ne) => {
            switch (_) {
              case "range":
                return te.step && ne.step ? te.step - ne.step : 0;
            }
            return 0;
          }));
        }
      }
      H[V].setWithExplicit(_, q);
    }
  }
  function bins(O, _) {
    const H = _.bin;
    if (isBinning(H)) {
      const V = getBinSignalName(O, _.field, H);
      return new SignalRefWrapper(() => O.getSignalName(V));
    } else if (isBinned(H) && isBinParams(H) && H.step !== void 0)
      return {
        step: H.step
      };
  }
  function interpolate(O, _) {
    if (contains$2([COLOR, FILL, STROKE], O) && _ !== "nominal")
      return "hcl";
  }
  function nice(O, _, H, V, q, G) {
    var J;
    if (!((J = getFieldDef(G)) != null && J.bin || isArray$2(H) || q != null || V != null || contains$2([ScaleType.TIME, ScaleType.UTC], O)))
      return isXorY(_) ? !0 : void 0;
  }
  function padding(O, _, H, V, q, G) {
    if (isXorY(O)) {
      if (isContinuousToContinuous(_)) {
        if (H.continuousPadding !== void 0)
          return H.continuousPadding;
        const { type: J, orient: Z } = q;
        if (J === "bar" && !(isFieldDef(V) && (V.bin || V.timeUnit)) && (Z === "vertical" && O === "x" || Z === "horizontal" && O === "y"))
          return G.continuousBandSize;
      }
      if (_ === ScaleType.POINT)
        return H.pointPadding;
    }
  }
  function paddingInner(O, _, H, V, q, G = !1) {
    if (O === void 0) {
      if (isXorY(_)) {
        const { bandPaddingInner: J, barBandPaddingInner: Z, rectBandPaddingInner: te, tickBandPaddingInner: ne, bandWithNestedOffsetPaddingInner: ee } = q;
        return G ? ee : getFirstDefined(J, H === "bar" ? Z : H === "tick" ? ne : te);
      } else if (isXorYOffset(_) && V === ScaleType.BAND)
        return q.offsetBandPaddingInner;
    }
  }
  function paddingOuter(O, _, H, V, q, G = !1) {
    if (O === void 0) {
      if (isXorY(_)) {
        const { bandPaddingOuter: J, bandWithNestedOffsetPaddingOuter: Z } = q;
        if (G)
          return Z;
        if (H === ScaleType.BAND)
          return getFirstDefined(
            J,
            /* By default, paddingOuter is paddingInner / 2. The reason is that
              size (width/height) = step * (cardinality - paddingInner + 2 * paddingOuter).
              and we want the width/height to be integer by default.
              Note that step (by default) and cardinality are integers.) */
            isSignalRef(V) ? { signal: `${V.signal}/2` } : V / 2
          );
      } else if (isXorYOffset(_)) {
        if (H === ScaleType.POINT)
          return 0.5;
        if (H === ScaleType.BAND)
          return q.offsetBandPaddingOuter;
      }
    }
  }
  function reverse$1(O, _, H, V) {
    if (H === "x" && V.xReverse !== void 0)
      return hasContinuousDomain(O) && _ === "descending" ? isSignalRef(V.xReverse) ? { signal: `!${V.xReverse.signal}` } : !V.xReverse : V.xReverse;
    if (hasContinuousDomain(O) && _ === "descending")
      return !0;
  }
  function zero(O, _, H, V, q, G, J) {
    if (!!H && H !== "unaggregated" && hasContinuousDomain(q)) {
      if (isArray$2(H)) {
        const te = H[0], ne = H[H.length - 1];
        if (isNumber$2(te) && te <= 0 && isNumber$2(ne) && ne >= 0)
          return !0;
      }
      return !1;
    }
    if (O === "size" && _.type === "quantitative" && !isContinuousToDiscrete(q))
      return !0;
    if (!(isFieldDef(_) && _.bin) && contains$2([...POSITION_SCALE_CHANNELS, ...POLAR_POSITION_SCALE_CHANNELS], O)) {
      const { orient: te, type: ne } = V;
      return contains$2(["bar", "area", "line", "trail"], ne) && (te === "horizontal" && O === "y" || te === "vertical" && O === "x") ? !1 : contains$2(["bar", "area"], ne) && !J ? !0 : G == null ? void 0 : G.zero;
    }
    return !1;
  }
  function scaleType(O, _, H, V, q = !1) {
    const G = defaultType$1(_, H, V, q), { type: J } = O;
    return isScaleChannel(_) ? J !== void 0 ? channelSupportScaleType(_, J) ? isFieldDef(H) && !scaleTypeSupportDataType(J, H.type) ? (warn(scaleTypeNotWorkWithFieldDef(J, G)), G) : J : (warn(scaleTypeNotWorkWithChannel(_, J, G)), G) : G : null;
  }
  function defaultType$1(O, _, H, V) {
    var q;
    switch (_.type) {
      case "nominal":
      case "ordinal": {
        if (isColorChannel(O) || rangeType(O) === "discrete")
          return O === "shape" && _.type === "ordinal" && warn(discreteChannelCannotEncode(O, "ordinal")), "ordinal";
        if (isTime(O))
          return "band";
        if (isXorY(O) || isXorYOffset(O)) {
          if (contains$2(["rect", "bar", "image", "rule", "tick"], H.type) || V)
            return "band";
        } else if (H.type === "arc" && O in POLAR_POSITION_SCALE_CHANNEL_INDEX)
          return "band";
        const G = H[getSizeChannel(O)];
        return isRelativeBandSize(G) || isPositionFieldOrDatumDef(_) && ((q = _.axis) != null && q.tickBand) ? "band" : "point";
      }
      case "temporal":
        return isColorChannel(O) ? "time" : rangeType(O) === "discrete" ? (warn(discreteChannelCannotEncode(O, "temporal")), "ordinal") : isFieldDef(_) && _.timeUnit && normalizeTimeUnit(_.timeUnit).utc ? "utc" : isTime(O) ? "band" : "time";
      case "quantitative":
        return isColorChannel(O) ? isFieldDef(_) && isBinning(_.bin) ? "bin-ordinal" : "linear" : rangeType(O) === "discrete" ? (warn(discreteChannelCannotEncode(O, "quantitative")), "ordinal") : isTime(O) ? "band" : "linear";
      case "geojson":
        return;
    }
    throw new Error(invalidFieldType(_.type));
  }
  function parseScales(O, { ignoreRange: _ } = {}) {
    parseScaleCore(O), parseScaleDomain(O);
    for (const H of NON_TYPE_DOMAIN_RANGE_VEGA_SCALE_PROPERTIES)
      parseScaleProperty(O, H);
    _ || parseScaleRange(O);
  }
  function parseScaleCore(O) {
    isUnitModel(O) ? O.component.scales = parseUnitScaleCore(O) : O.component.scales = parseNonUnitScaleCore(O);
  }
  function parseUnitScaleCore(O) {
    const { encoding: _, mark: H, markDef: V } = O, q = {};
    for (const G of SCALE_CHANNELS) {
      const J = getFieldOrDatumDef(_[G]);
      if (J && H === GEOSHAPE && G === SHAPE && J.type === GEOJSON)
        continue;
      let Z = J && J.scale;
      if (J && Z !== null && Z !== !1) {
        Z ?? (Z = {});
        const te = channelHasNestedOffsetScale(_, G), ne = scaleType(Z, G, J, V, te);
        q[G] = new ScaleComponent(O.scaleName(`${G}`, !0), {
          value: ne,
          explicit: Z.type === ne
        });
      }
    }
    return q;
  }
  const scaleTypeTieBreaker = tieBreakByComparing((O, _) => scaleTypePrecedence(O) - scaleTypePrecedence(_));
  function parseNonUnitScaleCore(O) {
    var _;
    const H = O.component.scales = {}, V = {}, q = O.component.resolve;
    for (const G of O.children) {
      parseScaleCore(G);
      for (const J of keys$2(G.component.scales))
        if ((_ = q.scale)[J] ?? (_[J] = defaultScaleResolve(J, O)), q.scale[J] === "shared") {
          const Z = V[J], te = G.component.scales[J].getWithExplicit("type");
          Z ? scaleCompatible(Z.value, te.value) ? V[J] = mergeValuesWithExplicit(Z, te, "type", "scale", scaleTypeTieBreaker) : (q.scale[J] = "independent", delete V[J]) : V[J] = te;
        }
    }
    for (const G of keys$2(V)) {
      const J = O.scaleName(G, !0), Z = V[G];
      H[G] = new ScaleComponent(J, Z);
      for (const te of O.children) {
        const ne = te.component.scales[G];
        ne && (te.renameScale(ne.get("name"), J), ne.merged = !0);
      }
    }
    return H;
  }
  class NameMap {
    constructor() {
      this.nameMap = {};
    }
    rename(_, H) {
      this.nameMap[_] = H;
    }
    has(_) {
      return this.nameMap[_] !== void 0;
    }
    get(_) {
      for (; this.nameMap[_] && _ !== this.nameMap[_]; )
        _ = this.nameMap[_];
      return _;
    }
  }
  function isUnitModel(O) {
    return (O == null ? void 0 : O.type) === "unit";
  }
  function isFacetModel(O) {
    return (O == null ? void 0 : O.type) === "facet";
  }
  function isConcatModel(O) {
    return (O == null ? void 0 : O.type) === "concat";
  }
  function isLayerModel(O) {
    return (O == null ? void 0 : O.type) === "layer";
  }
  class Model {
    constructor(_, H, V, q, G, J, Z) {
      this.type = H, this.parent = V, this.config = G, this.parent = V, this.config = G, this.view = replaceExprRef(Z), this.name = _.name ?? q, this.title = isText(_.title) ? { text: _.title } : _.title ? replaceExprRef(_.title) : void 0, this.scaleNameMap = V ? V.scaleNameMap : new NameMap(), this.projectionNameMap = V ? V.projectionNameMap : new NameMap(), this.signalNameMap = V ? V.signalNameMap : new NameMap(), this.data = _.data, this.description = _.description, this.transforms = normalizeTransform(_.transform ?? []), this.layout = H === "layer" || H === "unit" ? {} : extractCompositionLayout(_, H, G), this.component = {
        data: {
          sources: V ? V.component.data.sources : [],
          outputNodes: V ? V.component.data.outputNodes : {},
          outputNodeRefCounts: V ? V.component.data.outputNodeRefCounts : {},
          // data is faceted if the spec is a facet spec or the parent has faceted data and data is undefined
          isFaceted: isFacetSpec(_) || (V == null ? void 0 : V.component.data.isFaceted) && _.data === void 0
        },
        layoutSize: new Split(),
        layoutHeaders: { row: {}, column: {}, facet: {} },
        mark: null,
        resolve: {
          scale: {},
          axis: {},
          legend: {},
          ...J ? duplicate(J) : {}
        },
        selection: null,
        scales: null,
        projection: null,
        axes: {},
        legends: {}
      };
    }
    get width() {
      return this.getSizeSignalRef("width");
    }
    get height() {
      return this.getSizeSignalRef("height");
    }
    parse() {
      this.parseScale(), this.parseLayoutSize(), this.renameTopLevelLayoutSizeSignal(), this.parseSelections(), this.parseProjection(), this.parseData(), this.parseAxesAndHeaders(), this.parseLegends(), this.parseMarkGroup();
    }
    parseScale() {
      parseScales(this);
    }
    parseProjection() {
      parseProjection(this);
    }
    /**
     * Rename top-level spec's size to be just width / height, ignoring model name.
     * This essentially merges the top-level spec's width/height signals with the width/height signals
     * to help us reduce redundant signals declaration.
     */
    renameTopLevelLayoutSizeSignal() {
      this.getName("width") !== "width" && this.renameSignal(this.getName("width"), "width"), this.getName("height") !== "height" && this.renameSignal(this.getName("height"), "height");
    }
    parseLegends() {
      parseLegend(this);
    }
    assembleEncodeFromView(_) {
      const { style: H, ...V } = _, q = {};
      for (const G of keys$2(V)) {
        const J = V[G];
        J !== void 0 && (q[G] = signalOrValueRef(J));
      }
      return q;
    }
    assembleGroupEncodeEntry(_) {
      let H = {};
      return this.view && (H = this.assembleEncodeFromView(this.view)), !_ && (this.description && (H.description = signalOrValueRef(this.description)), this.type === "unit" || this.type === "layer") ? {
        width: this.getSizeSignalRef("width"),
        height: this.getSizeSignalRef("height"),
        ...H
      } : isEmpty(H) ? void 0 : H;
    }
    assembleLayout() {
      if (!this.layout)
        return;
      const { spacing: _, ...H } = this.layout, { component: V, config: q } = this, G = assembleLayoutTitleBand(V.layoutHeaders, q);
      return {
        padding: _,
        ...this.assembleDefaultLayout(),
        ...H,
        ...G ? { titleBand: G } : {}
      };
    }
    assembleDefaultLayout() {
      return {};
    }
    assembleHeaderMarks() {
      const { layoutHeaders: _ } = this.component;
      let H = [];
      for (const V of FACET_CHANNELS)
        _[V].title && H.push(assembleTitleGroup(this, V));
      for (const V of HEADER_CHANNELS)
        H = H.concat(assembleHeaderGroups(this, V));
      return H;
    }
    assembleAxes() {
      return assembleAxes(this.component.axes, this.config);
    }
    assembleLegends() {
      return assembleLegends(this);
    }
    assembleProjections() {
      return assembleProjections(this);
    }
    assembleTitle() {
      const { encoding: _, ...H } = this.title ?? {}, V = {
        ...extractTitleConfig(this.config.title).nonMarkTitleProperties,
        ...H,
        ..._ ? { encode: { update: _ } } : {}
      };
      if (V.text)
        return contains$2(["unit", "layer"], this.type) ? contains$2(["middle", void 0], V.anchor) && (V.frame ?? (V.frame = "group")) : V.anchor ?? (V.anchor = "start"), isEmpty(V) ? void 0 : V;
    }
    /**
     * Assemble the mark group for this model. We accept optional `signals` so that we can include concat top-level signals with the top-level model's local signals.
     */
    assembleGroup(_ = []) {
      const H = {};
      _ = _.concat(this.assembleSignals()), _.length > 0 && (H.signals = _);
      const V = this.assembleLayout();
      V && (H.layout = V), H.marks = [].concat(this.assembleHeaderMarks(), this.assembleMarks());
      const q = !this.parent || isFacetModel(this.parent) ? assembleScales(this) : [];
      q.length > 0 && (H.scales = q);
      const G = this.assembleAxes();
      G.length > 0 && (H.axes = G);
      const J = this.assembleLegends();
      return J.length > 0 && (H.legends = J), H;
    }
    getName(_) {
      return varName((this.name ? `${this.name}_` : "") + _);
    }
    getDataName(_) {
      return this.getName(DataSourceType[_].toLowerCase());
    }
    /**
     * Request a data source name for the given data source type and mark that data source as required.
     * This method should be called in parse, so that all used data source can be correctly instantiated in assembleData().
     * You can lookup the correct dataset name in assemble with `lookupDataSource`.
     */
    requestDataName(_) {
      const H = this.getDataName(_), V = this.component.data.outputNodeRefCounts;
      return V[H] = (V[H] || 0) + 1, H;
    }
    getSizeSignalRef(_) {
      if (isFacetModel(this.parent)) {
        const H = getSizeTypeFromLayoutSizeType(_), V = getPositionScaleChannel(H), q = this.component.scales[V];
        if (q && !q.merged) {
          const G = q.get("type"), J = q.get("range");
          if (hasDiscreteDomain(G) && isVgRangeStep(J)) {
            const Z = q.get("name"), te = assembleDomain(this, V), ne = getFieldFromDomain(te);
            if (ne) {
              const ee = vgField({ aggregate: "distinct", field: ne }, { expr: "datum" });
              return {
                signal: sizeExpr(Z, q, ee)
              };
            } else
              return warn(unknownField(V)), null;
          }
        }
      }
      return {
        signal: this.signalNameMap.get(this.getName(_))
      };
    }
    /**
     * Lookup the name of the datasource for an output node. You probably want to call this in assemble.
     */
    lookupDataSource(_) {
      const H = this.component.data.outputNodes[_];
      return H ? H.getSource() : _;
    }
    getSignalName(_) {
      return this.signalNameMap.get(_);
    }
    renameSignal(_, H) {
      this.signalNameMap.rename(_, H);
    }
    renameScale(_, H) {
      this.scaleNameMap.rename(_, H);
    }
    renameProjection(_, H) {
      this.projectionNameMap.rename(_, H);
    }
    /**
     * @return scale name for a given channel after the scale has been parsed and named.
     */
    scaleName(_, H) {
      if (H)
        return this.getName(_);
      if (
        // If there is a scale for the channel, there should be a local scale component for it
        isChannel(_) && isScaleChannel(_) && this.component.scales[_] || // in the scale name map (the scale get merged by its parent)
        this.scaleNameMap.has(this.getName(_))
      )
        return this.scaleNameMap.get(this.getName(_));
    }
    /**
     * @return projection name after the projection has been parsed and named.
     */
    projectionName(_) {
      if (_)
        return this.getName("projection");
      if (this.component.projection && !this.component.projection.merged || this.projectionNameMap.has(this.getName("projection")))
        return this.projectionNameMap.get(this.getName("projection"));
    }
    /**
     * Traverse a model's hierarchy to get the scale component for a particular channel.
     */
    getScaleComponent(_) {
      if (!this.component.scales)
        throw new Error("getScaleComponent cannot be called before parseScale(). Make sure you have called parseScale or use parseUnitModelWithScale().");
      const H = this.component.scales[_];
      return H && !H.merged ? H : this.parent ? this.parent.getScaleComponent(_) : void 0;
    }
    getScaleType(_) {
      const H = this.getScaleComponent(_);
      return H ? H.get("type") : void 0;
    }
    /**
     * Traverse a model's hierarchy to get a particular selection component.
     */
    getSelectionComponent(_, H) {
      let V = this.component.selection[_];
      if (!V && this.parent && (V = this.parent.getSelectionComponent(_, H)), !V)
        throw new Error(selectionNotFound(H));
      return V;
    }
    /**
     * Returns true if the model has a signalRef for an axis orient.
     */
    hasAxisOrientSignalRef() {
      var _, H;
      return ((_ = this.component.axes.x) == null ? void 0 : _.some((V) => V.hasOrientSignalRef())) || ((H = this.component.axes.y) == null ? void 0 : H.some((V) => V.hasOrientSignalRef()));
    }
  }
  class ModelWithField extends Model {
    /** Get "field" reference for Vega */
    vgField(_, H = {}) {
      const V = this.fieldDef(_);
      if (V)
        return vgField(V, H);
    }
    reduceFieldDef(_, H) {
      return reduce$2(this.getMapping(), (V, q, G) => {
        const J = getFieldDef(q);
        return J ? _(V, J, G) : V;
      }, H);
    }
    forEachFieldDef(_, H) {
      forEach$1(this.getMapping(), (V, q) => {
        const G = getFieldDef(V);
        G && _(G, q);
      }, H);
    }
  }
  class DensityTransformNode extends DataFlowNode {
    clone() {
      return new DensityTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this.transform = duplicate(H);
      const V = this.transform.as ?? [void 0, void 0];
      this.transform.as = [V[0] ?? "value", V[1] ?? "density"];
      const q = this.transform.resolve ?? "shared";
      this.transform.resolve = q;
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.density, ...this.transform.groupby ?? []]);
    }
    producedFields() {
      return new Set(this.transform.as);
    }
    hash() {
      return `DensityTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { density: _, ...H } = this.transform, V = {
        type: "kde",
        field: _,
        ...H
      };
      return V.resolve = this.transform.resolve, V;
    }
  }
  class ExtentTransformNode extends DataFlowNode {
    clone() {
      return new ExtentTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this.transform = duplicate(H);
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.extent]);
    }
    producedFields() {
      return /* @__PURE__ */ new Set([]);
    }
    hash() {
      return `ExtentTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { extent: _, param: H } = this.transform;
      return {
        type: "extent",
        field: _,
        signal: H
      };
    }
  }
  class FlattenTransformNode extends DataFlowNode {
    clone() {
      return new FlattenTransformNode(this.parent, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this.transform = duplicate(H);
      const { flatten: V, as: q = [] } = this.transform;
      this.transform.as = V.map((G, J) => q[J] ?? G);
    }
    dependentFields() {
      return new Set(this.transform.flatten);
    }
    producedFields() {
      return new Set(this.transform.as);
    }
    hash() {
      return `FlattenTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { flatten: _, as: H } = this.transform;
      return {
        type: "flatten",
        fields: _,
        as: H
      };
    }
  }
  class FoldTransformNode extends DataFlowNode {
    clone() {
      return new FoldTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this.transform = duplicate(H);
      const V = this.transform.as ?? [void 0, void 0];
      this.transform.as = [V[0] ?? "key", V[1] ?? "value"];
    }
    dependentFields() {
      return new Set(this.transform.fold);
    }
    producedFields() {
      return new Set(this.transform.as);
    }
    hash() {
      return `FoldTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { fold: _, as: H } = this.transform;
      return {
        type: "fold",
        fields: _,
        as: H
      };
    }
  }
  class GeoJSONNode extends DataFlowNode {
    clone() {
      return new GeoJSONNode(null, duplicate(this.fields), this.geojson, this.signal);
    }
    static parseAll(_, H) {
      if (H.component.projection && !H.component.projection.isFit)
        return _;
      let V = 0;
      for (const q of [
        [LONGITUDE, LATITUDE],
        [LONGITUDE2, LATITUDE2]
      ]) {
        const G = q.map((J) => {
          const Z = getFieldOrDatumDef(H.encoding[J]);
          return isFieldDef(Z) ? Z.field : isDatumDef(Z) ? { expr: `${Z.datum}` } : isValueDef(Z) ? { expr: `${Z.value}` } : void 0;
        });
        (G[0] || G[1]) && (_ = new GeoJSONNode(_, G, null, H.getName(`geojson_${V++}`)));
      }
      if (H.channelHasField(SHAPE)) {
        const q = H.typedFieldDef(SHAPE);
        q.type === GEOJSON && (_ = new GeoJSONNode(_, null, q.field, H.getName(`geojson_${V++}`)));
      }
      return _;
    }
    constructor(_, H, V, q) {
      super(_), this.fields = H, this.geojson = V, this.signal = q;
    }
    dependentFields() {
      const _ = (this.fields ?? []).filter(isString$1);
      return /* @__PURE__ */ new Set([...this.geojson ? [this.geojson] : [], ..._]);
    }
    producedFields() {
      return /* @__PURE__ */ new Set();
    }
    hash() {
      return `GeoJSON ${this.geojson} ${this.signal} ${hash(this.fields)}`;
    }
    assemble() {
      return [
        ...this.geojson ? [
          {
            type: "filter",
            expr: `isValid(datum["${this.geojson}"])`
          }
        ] : [],
        {
          type: "geojson",
          ...this.fields ? { fields: this.fields } : {},
          ...this.geojson ? { geojson: this.geojson } : {},
          signal: this.signal
        }
      ];
    }
  }
  class GeoPointNode extends DataFlowNode {
    clone() {
      return new GeoPointNode(null, this.projection, duplicate(this.fields), duplicate(this.as));
    }
    constructor(_, H, V, q) {
      super(_), this.projection = H, this.fields = V, this.as = q;
    }
    static parseAll(_, H) {
      if (!H.projectionName())
        return _;
      for (const V of [
        [LONGITUDE, LATITUDE],
        [LONGITUDE2, LATITUDE2]
      ]) {
        const q = V.map((J) => {
          const Z = getFieldOrDatumDef(H.encoding[J]);
          return isFieldDef(Z) ? Z.field : isDatumDef(Z) ? { expr: `${Z.datum}` } : isValueDef(Z) ? { expr: `${Z.value}` } : void 0;
        }), G = V[0] === LONGITUDE2 ? "2" : "";
        (q[0] || q[1]) && (_ = new GeoPointNode(_, H.projectionName(), q, [
          H.getName(`x${G}`),
          H.getName(`y${G}`)
        ]));
      }
      return _;
    }
    dependentFields() {
      return new Set(this.fields.filter(isString$1));
    }
    producedFields() {
      return new Set(this.as);
    }
    hash() {
      return `Geopoint ${this.projection} ${hash(this.fields)} ${hash(this.as)}`;
    }
    assemble() {
      return {
        type: "geopoint",
        projection: this.projection,
        fields: this.fields,
        as: this.as
      };
    }
  }
  class ImputeNode extends DataFlowNode {
    clone() {
      return new ImputeNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H;
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.impute, this.transform.key, ...this.transform.groupby ?? []]);
    }
    producedFields() {
      return /* @__PURE__ */ new Set([this.transform.impute]);
    }
    processSequence(_) {
      const { start: H = 0, stop: V, step: q } = _;
      return { signal: `sequence(${[H, V, ...q ? [q] : []].join(",")})` };
    }
    static makeFromTransform(_, H) {
      return new ImputeNode(_, H);
    }
    static makeFromEncoding(_, H) {
      const V = H.encoding, q = V.x, G = V.y;
      if (isFieldDef(q) && isFieldDef(G)) {
        const J = q.impute ? q : G.impute ? G : void 0;
        if (J === void 0)
          return;
        const Z = q.impute ? G : G.impute ? q : void 0, { method: te, value: ne, frame: ee, keyvals: ae } = J.impute, oe = pathGroupingFields(H.mark, V);
        return new ImputeNode(_, {
          impute: J.field,
          key: Z.field,
          ...te ? { method: te } : {},
          ...ne !== void 0 ? { value: ne } : {},
          ...ee ? { frame: ee } : {},
          ...ae !== void 0 ? { keyvals: ae } : {},
          ...oe.length ? { groupby: oe } : {}
        });
      }
      return null;
    }
    hash() {
      return `Impute ${hash(this.transform)}`;
    }
    assemble() {
      const { impute: _, key: H, keyvals: V, method: q, groupby: G, value: J, frame: Z = [null, null] } = this.transform, te = {
        type: "impute",
        field: _,
        key: H,
        ...V ? { keyvals: isImputeSequence(V) ? this.processSequence(V) : V } : {},
        method: "value",
        ...G ? { groupby: G } : {},
        value: !q || q === "value" ? J : null
      };
      if (q && q !== "value") {
        const ne = {
          type: "window",
          as: [`imputed_${_}_value`],
          ops: [q],
          fields: [_],
          frame: Z,
          ignorePeers: !1,
          ...G ? { groupby: G } : {}
        }, ee = {
          type: "formula",
          expr: `datum.${_} === null ? datum.imputed_${_}_value : datum.${_}`,
          as: _
        };
        return [te, ne, ee];
      } else
        return [te];
    }
  }
  class LoessTransformNode extends DataFlowNode {
    clone() {
      return new LoessTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this.transform = duplicate(H);
      const V = this.transform.as ?? [void 0, void 0];
      this.transform.as = [V[0] ?? H.on, V[1] ?? H.loess];
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.loess, this.transform.on, ...this.transform.groupby ?? []]);
    }
    producedFields() {
      return new Set(this.transform.as);
    }
    hash() {
      return `LoessTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { loess: _, on: H, ...V } = this.transform;
      return {
        type: "loess",
        x: H,
        y: _,
        ...V
      };
    }
  }
  class LookupNode extends DataFlowNode {
    clone() {
      return new LookupNode(null, duplicate(this.transform), this.secondary);
    }
    constructor(_, H, V) {
      super(_), this.transform = H, this.secondary = V;
    }
    static make(_, H, V, q) {
      const G = H.component.data.sources, { from: J } = V;
      let Z = null;
      if (isLookupData(J)) {
        let te = findSource(J.data, G);
        te || (te = new SourceNode(J.data), G.push(te));
        const ne = H.getName(`lookup_${q}`);
        Z = new OutputNode(te, ne, DataSourceType.Lookup, H.component.data.outputNodeRefCounts), H.component.data.outputNodes[ne] = Z;
      } else if (isLookupSelection(J)) {
        const te = J.param;
        V = { as: te, ...V };
        let ne;
        try {
          ne = H.getSelectionComponent(varName(te), te);
        } catch {
          throw new Error(cannotLookupVariableParameter(te));
        }
        if (Z = ne.materialized, !Z)
          throw new Error(noSameUnitLookup(te));
      }
      return new LookupNode(_, V, Z.getSource());
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.lookup]);
    }
    producedFields() {
      return new Set(this.transform.as ? array$5(this.transform.as) : this.transform.from.fields);
    }
    hash() {
      return `Lookup ${hash({ transform: this.transform, secondary: this.secondary })}`;
    }
    assemble() {
      let _;
      if (this.transform.from.fields)
        _ = {
          values: this.transform.from.fields,
          ...this.transform.as ? { as: array$5(this.transform.as) } : {}
        };
      else {
        let H = this.transform.as;
        isString$1(H) || (warn(NO_FIELDS_NEEDS_AS), H = "_lookup"), _ = {
          as: [H]
        };
      }
      return {
        type: "lookup",
        from: this.secondary,
        key: this.transform.from.key,
        fields: [this.transform.lookup],
        ..._,
        ...this.transform.default ? { default: this.transform.default } : {}
      };
    }
  }
  class QuantileTransformNode extends DataFlowNode {
    clone() {
      return new QuantileTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this.transform = duplicate(H);
      const V = this.transform.as ?? [void 0, void 0];
      this.transform.as = [V[0] ?? "prob", V[1] ?? "value"];
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.quantile, ...this.transform.groupby ?? []]);
    }
    producedFields() {
      return new Set(this.transform.as);
    }
    hash() {
      return `QuantileTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { quantile: _, ...H } = this.transform;
      return {
        type: "quantile",
        field: _,
        ...H
      };
    }
  }
  class RegressionTransformNode extends DataFlowNode {
    clone() {
      return new RegressionTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H, this.transform = duplicate(H);
      const V = this.transform.as ?? [void 0, void 0];
      this.transform.as = [V[0] ?? H.on, V[1] ?? H.regression];
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.regression, this.transform.on, ...this.transform.groupby ?? []]);
    }
    producedFields() {
      return new Set(this.transform.as);
    }
    hash() {
      return `RegressionTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { regression: _, on: H, ...V } = this.transform;
      return {
        type: "regression",
        x: H,
        y: _,
        ...V
      };
    }
  }
  class PivotTransformNode extends DataFlowNode {
    clone() {
      return new PivotTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H;
    }
    addDimensions(_) {
      this.transform.groupby = unique((this.transform.groupby ?? []).concat(_), (H) => H);
    }
    producedFields() {
    }
    dependentFields() {
      return /* @__PURE__ */ new Set([this.transform.pivot, this.transform.value, ...this.transform.groupby ?? []]);
    }
    hash() {
      return `PivotTransform ${hash(this.transform)}`;
    }
    assemble() {
      const { pivot: _, value: H, groupby: V, limit: q, op: G } = this.transform;
      return {
        type: "pivot",
        field: _,
        value: H,
        ...q !== void 0 ? { limit: q } : {},
        ...G !== void 0 ? { op: G } : {},
        ...V !== void 0 ? { groupby: V } : {}
      };
    }
  }
  class SampleTransformNode extends DataFlowNode {
    clone() {
      return new SampleTransformNode(null, duplicate(this.transform));
    }
    constructor(_, H) {
      super(_), this.transform = H;
    }
    dependentFields() {
      return /* @__PURE__ */ new Set();
    }
    producedFields() {
      return /* @__PURE__ */ new Set();
    }
    hash() {
      return `SampleTransform ${hash(this.transform)}`;
    }
    assemble() {
      return {
        type: "sample",
        size: this.transform.sample
      };
    }
  }
  function makeWalkTree(O) {
    let _ = 0;
    function H(V, q) {
      if (V instanceof SourceNode && !V.isGenerator && !isUrlData(V.data) && (O.push(q), q = {
        name: null,
        source: q.name,
        transform: []
      }), V instanceof ParseNode && (V.parent instanceof SourceNode && !q.source ? (q.format = {
        ...q.format,
        parse: V.assembleFormatParse()
      }, q.transform.push(...V.assembleTransforms(!0))) : q.transform.push(...V.assembleTransforms())), V instanceof FacetNode) {
        q.name || (q.name = `data_${_++}`), !q.source || q.transform.length > 0 ? (O.push(q), V.data = q.name) : V.data = q.source, O.push(...V.assemble());
        return;
      }
      switch ((V instanceof GraticuleNode || V instanceof SequenceNode || V instanceof FilterInvalidNode || V instanceof FilterNode || V instanceof CalculateNode || V instanceof GeoPointNode || V instanceof AggregateNode || V instanceof LookupNode || V instanceof WindowTransformNode || V instanceof JoinAggregateTransformNode || V instanceof FoldTransformNode || V instanceof FlattenTransformNode || V instanceof DensityTransformNode || V instanceof LoessTransformNode || V instanceof QuantileTransformNode || V instanceof RegressionTransformNode || V instanceof IdentifierNode || V instanceof SampleTransformNode || V instanceof PivotTransformNode || V instanceof ExtentTransformNode) && q.transform.push(V.assemble()), (V instanceof BinNode || V instanceof TimeUnitNode || V instanceof ImputeNode || V instanceof StackNode || V instanceof GeoJSONNode) && q.transform.push(...V.assemble()), V instanceof OutputNode && (q.source && q.transform.length === 0 ? V.setSource(q.source) : V.parent instanceof OutputNode ? V.setSource(q.name) : (q.name || (q.name = `data_${_++}`), V.setSource(q.name), V.numChildren() === 1 && (O.push(q), q = {
        name: null,
        source: q.name,
        transform: []
      }))), V.numChildren()) {
        case 0:
          V instanceof OutputNode && (!q.source || q.transform.length > 0) && O.push(q);
          break;
        case 1:
          H(V.children[0], q);
          break;
        default: {
          q.name || (q.name = `data_${_++}`);
          let G = q.name;
          !q.source || q.transform.length > 0 ? O.push(q) : G = q.source;
          for (const J of V.children)
            H(J, {
              name: null,
              source: G,
              transform: []
            });
          break;
        }
      }
    }
    return H;
  }
  function assembleFacetData(O) {
    const _ = [], H = makeWalkTree(_);
    for (const V of O.children)
      H(V, {
        source: O.name,
        name: null,
        transform: []
      });
    return _;
  }
  function assembleRootData(O, _) {
    const H = [], V = makeWalkTree(H);
    let q = 0;
    for (const J of O.sources) {
      J.hasName() || (J.dataName = `source_${q++}`);
      const Z = J.assemble();
      V(J, Z);
    }
    for (const J of H)
      J.transform.length === 0 && delete J.transform;
    let G = 0;
    for (const [J, Z] of H.entries())
      (Z.transform ?? []).length === 0 && !Z.source && H.splice(G++, 0, H.splice(J, 1)[0]);
    for (const J of H)
      for (const Z of J.transform ?? [])
        Z.type === "lookup" && (Z.from = O.outputNodes[Z.from].getSource());
    for (const J of H)
      J.name in _ && (J.values = _[J.name]);
    return H;
  }
  function getHeaderType(O) {
    return O === "top" || O === "left" || isSignalRef(O) ? "header" : "footer";
  }
  function parseFacetHeaders(O) {
    for (const _ of FACET_CHANNELS)
      parseFacetHeader(O, _);
    mergeChildAxis(O, "x"), mergeChildAxis(O, "y");
  }
  function parseFacetHeader(O, _) {
    var J;
    const { facet: H, config: V, child: q, component: G } = O;
    if (O.channelHasField(_)) {
      const Z = H[_], te = getHeaderProperty("title", null, V, _);
      let ne = title(Z, V, {
        allowDisabling: !0,
        includeDefault: te === void 0 || !!te
      });
      q.component.layoutHeaders[_].title && (ne = isArray$2(ne) ? ne.join(", ") : ne, ne += ` / ${q.component.layoutHeaders[_].title}`, q.component.layoutHeaders[_].title = null);
      const ee = getHeaderProperty("labelOrient", Z.header, V, _), ae = Z.header !== null ? getFirstDefined((J = Z.header) == null ? void 0 : J.labels, V.header.labels, !0) : !1, oe = contains$2(["bottom", "right"], ee) ? "footer" : "header";
      G.layoutHeaders[_] = {
        title: Z.header !== null ? ne : null,
        facetFieldDef: Z,
        [oe]: _ === "facet" ? [] : [makeHeaderComponent(O, _, ae)]
      };
    }
  }
  function makeHeaderComponent(O, _, H) {
    const V = _ === "row" ? "height" : "width";
    return {
      labels: H,
      sizeSignal: O.child.component.layoutSize.get(V) ? O.child.getSizeSignalRef(V) : void 0,
      axes: []
    };
  }
  function mergeChildAxis(O, _) {
    const { child: H } = O;
    if (H.component.axes[_]) {
      const { layoutHeaders: V, resolve: q } = O.component;
      if (q.axis[_] = parseGuideResolve(q, _), q.axis[_] === "shared") {
        const G = _ === "x" ? "column" : "row", J = V[G];
        for (const Z of H.component.axes[_]) {
          const te = getHeaderType(Z.get("orient"));
          J[te] ?? (J[te] = [makeHeaderComponent(O, G, !1)]);
          const ne = assembleAxis(Z, "main", O.config, { header: !0 });
          ne && J[te][0].axes.push(ne), Z.mainExtracted = !0;
        }
      }
    }
  }
  function parseLayerLayoutSize(O) {
    parseChildrenLayoutSize(O), parseNonUnitLayoutSizeForChannel(O, "width"), parseNonUnitLayoutSizeForChannel(O, "height");
  }
  function parseConcatLayoutSize(O) {
    parseChildrenLayoutSize(O);
    const _ = O.layout.columns === 1 ? "width" : "childWidth", H = O.layout.columns === void 0 ? "height" : "childHeight";
    parseNonUnitLayoutSizeForChannel(O, _), parseNonUnitLayoutSizeForChannel(O, H);
  }
  function parseChildrenLayoutSize(O) {
    for (const _ of O.children)
      _.parseLayoutSize();
  }
  function parseNonUnitLayoutSizeForChannel(O, _) {
    const H = getSizeTypeFromLayoutSizeType(_), V = getPositionScaleChannel(H), q = O.component.resolve, G = O.component.layoutSize;
    let J;
    for (const Z of O.children) {
      const te = Z.component.layoutSize.getWithExplicit(H), ne = q.scale[V] ?? defaultScaleResolve(V, O);
      if (ne === "independent" && te.value === "step") {
        J = void 0;
        break;
      }
      if (J) {
        if (ne === "independent" && J.value !== te.value) {
          J = void 0;
          break;
        }
        J = mergeValuesWithExplicit(J, te, H, "");
      } else
        J = te;
    }
    if (J) {
      for (const Z of O.children)
        O.renameSignal(Z.getName(H), O.getName(_)), Z.component.layoutSize.set(H, "merged", !1);
      G.setWithExplicit(_, J);
    } else
      G.setWithExplicit(_, {
        explicit: !1,
        value: void 0
      });
  }
  function parseUnitLayoutSize(O) {
    const { size: _, component: H } = O;
    for (const V of POSITION_SCALE_CHANNELS) {
      const q = getSizeChannel(V);
      if (_[q]) {
        const G = _[q];
        H.layoutSize.set(q, isStep(G) ? "step" : G, !0);
      } else {
        const G = defaultUnitSize(O, q);
        H.layoutSize.set(q, G, !1);
      }
    }
  }
  function defaultUnitSize(O, _) {
    const H = _ === "width" ? "x" : "y", V = O.config, q = O.getScaleComponent(H);
    if (q) {
      const G = q.get("type"), J = q.get("range");
      if (hasDiscreteDomain(G)) {
        const Z = getViewConfigDiscreteSize(V.view, _);
        return isVgRangeStep(J) || isStep(Z) ? "step" : Z;
      } else
        return getViewConfigContinuousSize(V.view, _);
    } else {
      if (O.hasProjection || O.mark === "arc")
        return getViewConfigContinuousSize(V.view, _);
      {
        const G = getViewConfigDiscreteSize(V.view, _);
        return isStep(G) ? G.step : G;
      }
    }
  }
  function facetSortFieldName(O, _, H) {
    return vgField(_, { suffix: `by_${vgField(O)}`, ...H });
  }
  class FacetModel extends ModelWithField {
    constructor(_, H, V, q) {
      super(_, "facet", H, V, q, _.resolve), this.child = buildModel(_.spec, this, this.getName("child"), void 0, q), this.children = [this.child], this.facet = this.initFacet(_.facet);
    }
    initFacet(_) {
      if (!isFacetMapping(_))
        return { facet: this.initFacetFieldDef(_, "facet") };
      const H = keys$2(_), V = {};
      for (const q of H) {
        if (![ROW, COLUMN].includes(q)) {
          warn(incompatibleChannel(q, "facet"));
          break;
        }
        const G = _[q];
        if (G.field === void 0) {
          warn(emptyFieldDef(G, q));
          break;
        }
        V[q] = this.initFacetFieldDef(G, q);
      }
      return V;
    }
    initFacetFieldDef(_, H) {
      const V = initFieldDef(_, H);
      return V.header ? V.header = replaceExprRef(V.header) : V.header === null && (V.header = null), V;
    }
    channelHasField(_) {
      return hasProperty(this.facet, _);
    }
    fieldDef(_) {
      return this.facet[_];
    }
    parseData() {
      this.component.data = parseData(this), this.child.parseData();
    }
    parseLayoutSize() {
      parseChildrenLayoutSize(this);
    }
    parseSelections() {
      this.child.parseSelections(), this.component.selection = this.child.component.selection, Object.values(this.component.selection).some((_) => isTimerSelection(_)) && error(MULTI_VIEW_ANIMATION_UNSUPPORTED);
    }
    parseMarkGroup() {
      this.child.parseMarkGroup();
    }
    parseAxesAndHeaders() {
      this.child.parseAxesAndHeaders(), parseFacetHeaders(this);
    }
    assembleSelectionTopLevelSignals(_) {
      return this.child.assembleSelectionTopLevelSignals(_);
    }
    assembleSignals() {
      return this.child.assembleSignals(), [];
    }
    assembleSelectionData(_) {
      return this.child.assembleSelectionData(_);
    }
    getHeaderLayoutMixins() {
      const _ = {};
      for (const H of FACET_CHANNELS)
        for (const V of HEADER_TYPES) {
          const q = this.component.layoutHeaders[H], G = q[V], { facetFieldDef: J } = q;
          if (J) {
            const Z = getHeaderProperty("titleOrient", J.header, this.config, H);
            if (["right", "bottom"].includes(Z)) {
              const te = getHeaderChannel(H, Z);
              _.titleAnchor ?? (_.titleAnchor = {}), _.titleAnchor[te] = "end";
            }
          }
          if (G != null && G[0]) {
            const Z = H === "row" ? "height" : "width", te = V === "header" ? "headerBand" : "footerBand";
            H !== "facet" && !this.child.component.layoutSize.get(Z) && (_[te] ?? (_[te] = {}), _[te][H] = 0.5), q.title && (_.offset ?? (_.offset = {}), _.offset[H === "row" ? "rowTitle" : "columnTitle"] = 10);
          }
        }
      return _;
    }
    assembleDefaultLayout() {
      const { column: _, row: H } = this.facet, V = _ ? this.columnDistinctSignal() : H ? 1 : void 0;
      let q = "all";
      return (!H && this.component.resolve.scale.x === "independent" || !_ && this.component.resolve.scale.y === "independent") && (q = "none"), {
        ...this.getHeaderLayoutMixins(),
        ...V ? { columns: V } : {},
        bounds: "full",
        align: q
      };
    }
    assembleLayoutSignals() {
      return this.child.assembleLayoutSignals();
    }
    columnDistinctSignal() {
      if (!(this.parent && this.parent instanceof FacetModel))
        return { signal: `length(data('${this.getName("column_domain")}'))` };
    }
    assembleGroupStyle() {
    }
    assembleGroup(_) {
      return this.parent && this.parent instanceof FacetModel ? {
        ...this.channelHasField("column") ? {
          encode: {
            update: {
              // TODO(https://github.com/vega/vega-lite/issues/2759):
              // Correct the signal for facet of concat of facet_column
              columns: { field: vgField(this.facet.column, { prefix: "distinct" }) }
            }
          }
        } : {},
        ...super.assembleGroup(_)
      } : super.assembleGroup(_);
    }
    /**
     * Aggregate cardinality for calculating size
     */
    getCardinalityAggregateForChild() {
      const _ = [], H = [], V = [];
      if (this.child instanceof FacetModel) {
        if (this.child.channelHasField("column")) {
          const q = vgField(this.child.facet.column);
          _.push(q), H.push("distinct"), V.push(`distinct_${q}`);
        }
      } else
        for (const q of POSITION_SCALE_CHANNELS) {
          const G = this.child.component.scales[q];
          if (G && !G.merged) {
            const J = G.get("type"), Z = G.get("range");
            if (hasDiscreteDomain(J) && isVgRangeStep(Z)) {
              const te = assembleDomain(this.child, q), ne = getFieldFromDomain(te);
              ne ? (_.push(ne), H.push("distinct"), V.push(`distinct_${ne}`)) : warn(unknownField(q));
            }
          }
        }
      return { fields: _, ops: H, as: V };
    }
    assembleFacet() {
      const { name: _, data: H } = this.component.data.facetRoot, { row: V, column: q } = this.facet, { fields: G, ops: J, as: Z } = this.getCardinalityAggregateForChild(), te = [];
      for (const ee of FACET_CHANNELS) {
        const ae = this.facet[ee];
        if (ae) {
          te.push(vgField(ae));
          const { bin: oe, sort: se } = ae;
          if (isBinning(oe) && te.push(vgField(ae, { binSuffix: "end" })), isSortField(se)) {
            const { field: ce, op: le = DEFAULT_SORT_OP } = se, ue = facetSortFieldName(ae, se);
            V && q ? (G.push(ue), J.push("max"), Z.push(ue)) : (G.push(ce), J.push(le), Z.push(ue));
          } else if (isArray$2(se)) {
            const ce = sortArrayIndexField(ae, ee);
            G.push(ce), J.push("max"), Z.push(ce);
          }
        }
      }
      const ne = !!V && !!q;
      return {
        name: _,
        data: H,
        groupby: te,
        ...ne || G.length > 0 ? {
          aggregate: {
            ...ne ? { cross: ne } : {},
            ...G.length ? { fields: G, ops: J, as: Z } : {}
          }
        } : {}
      };
    }
    facetSortFields(_) {
      const { facet: H } = this, V = H[_];
      return V ? isSortField(V.sort) ? [facetSortFieldName(V, V.sort, { expr: "datum" })] : isArray$2(V.sort) ? [sortArrayIndexField(V, _, { expr: "datum" })] : [vgField(V, { expr: "datum" })] : [];
    }
    facetSortOrder(_) {
      const { facet: H } = this, V = H[_];
      if (V) {
        const { sort: q } = V;
        return [(isSortField(q) ? q.order : !isArray$2(q) && q) || "ascending"];
      }
      return [];
    }
    assembleLabelTitle() {
      var q;
      const { facet: _, config: H } = this;
      if (_.facet)
        return assembleLabelTitle(_.facet, "facet", H);
      const V = {
        row: ["top", "bottom"],
        column: ["left", "right"]
      };
      for (const G of HEADER_CHANNELS)
        if (_[G]) {
          const J = getHeaderProperty("labelOrient", (q = _[G]) == null ? void 0 : q.header, H, G);
          if (V[G].includes(J))
            return assembleLabelTitle(_[G], G, H);
        }
    }
    assembleMarks() {
      const { child: _ } = this, H = this.component.data.facetRoot, V = assembleFacetData(H), q = _.assembleGroupEncodeEntry(!1), G = this.assembleLabelTitle() || _.assembleTitle(), J = _.assembleGroupStyle();
      return [{
        name: this.getName("cell"),
        type: "group",
        ...G ? { title: G } : {},
        ...J ? { style: J } : {},
        from: {
          facet: this.assembleFacet()
        },
        // TODO: move this to after data
        sort: {
          field: FACET_CHANNELS.map((te) => this.facetSortFields(te)).flat(),
          order: FACET_CHANNELS.map((te) => this.facetSortOrder(te)).flat()
        },
        ...V.length > 0 ? { data: V } : {},
        ...q ? { encode: { update: q } } : {},
        ..._.assembleGroup(assembleFacetSignals(this, []))
      }];
    }
    getMapping() {
      return this.facet;
    }
  }
  function makeJoinAggregateFromFacet(O, _) {
    const { row: H, column: V } = _;
    if (H && V) {
      let q = null;
      for (const G of [H, V])
        if (isSortField(G.sort)) {
          const { field: J, op: Z = DEFAULT_SORT_OP } = G.sort;
          O = q = new JoinAggregateTransformNode(O, {
            joinaggregate: [
              {
                op: Z,
                field: J,
                as: facetSortFieldName(G, G.sort, { forAs: !0 })
              }
            ],
            groupby: [vgField(G)]
          });
        }
      return q;
    }
    return null;
  }
  function findSource(O, _) {
    var H, V, q, G;
    for (const J of _) {
      const Z = J.data;
      if (O.name && J.hasName() && O.name !== J.dataName)
        continue;
      const te = (H = O.format) == null ? void 0 : H.mesh, ne = (V = Z.format) == null ? void 0 : V.feature;
      if (te && ne)
        continue;
      const ee = (q = O.format) == null ? void 0 : q.feature;
      if ((ee || ne) && ee !== ne)
        continue;
      const ae = (G = Z.format) == null ? void 0 : G.mesh;
      if (!((te || ae) && te !== ae)) {
        if (isInlineData(O) && isInlineData(Z)) {
          if (deepEqual(O.values, Z.values))
            return J;
        } else if (isUrlData(O) && isUrlData(Z)) {
          if (O.url === Z.url)
            return J;
        } else if (isNamedData(O) && O.name === J.dataName)
          return J;
      }
    }
    return null;
  }
  function parseRoot(O, _) {
    if (O.data || !O.parent) {
      if (O.data === null) {
        const V = new SourceNode({ values: [] });
        return _.push(V), V;
      }
      const H = findSource(O.data, _);
      if (H)
        return isGenerator(O.data) || (H.data.format = mergeDeep$1({}, O.data.format, H.data.format)), !H.hasName() && O.data.name && (H.dataName = O.data.name), H;
      {
        const V = new SourceNode(O.data);
        return _.push(V), V;
      }
    } else
      return O.parent.component.data.facetRoot ? O.parent.component.data.facetRoot : O.parent.component.data.main;
  }
  function parseTransformArray(O, _, H) {
    let V = 0;
    for (const q of _.transforms) {
      let G, J;
      if (isCalculate(q))
        J = O = new CalculateNode(O, q), G = "derived";
      else if (isFilter(q)) {
        const Z = getImplicitFromFilterTransform(q);
        J = O = ParseNode.makeWithAncestors(O, {}, Z, H) ?? O, O = new FilterNode(O, _, q.filter);
      } else if (isBin(q))
        J = O = BinNode.makeFromTransform(O, q, _), G = "number";
      else if (isTimeUnit(q))
        G = "date", H.getWithExplicit(q.field).value === void 0 && (O = new ParseNode(O, { [q.field]: G }), H.set(q.field, G, !1)), J = O = TimeUnitNode.makeFromTransform(O, q);
      else if (isAggregate(q))
        J = O = AggregateNode.makeFromTransform(O, q), G = "number", requiresSelectionId(_) && (O = new IdentifierNode(O));
      else if (isLookup(q))
        J = O = LookupNode.make(O, _, q, V++), G = "derived";
      else if (isWindow(q))
        J = O = new WindowTransformNode(O, q), G = "number";
      else if (isJoinAggregate(q))
        J = O = new JoinAggregateTransformNode(O, q), G = "number";
      else if (isStack(q))
        J = O = StackNode.makeFromTransform(O, q), G = "derived";
      else if (isFold(q))
        J = O = new FoldTransformNode(O, q), G = "derived";
      else if (isExtent(q))
        J = O = new ExtentTransformNode(O, q), G = "derived";
      else if (isFlatten(q))
        J = O = new FlattenTransformNode(O, q), G = "derived";
      else if (isPivot(q))
        J = O = new PivotTransformNode(O, q), G = "derived";
      else if (isSample(q))
        O = new SampleTransformNode(O, q);
      else if (isImpute(q))
        J = O = ImputeNode.makeFromTransform(O, q), G = "derived";
      else if (isDensity(q))
        J = O = new DensityTransformNode(O, q), G = "derived";
      else if (isQuantile(q))
        J = O = new QuantileTransformNode(O, q), G = "derived";
      else if (isRegression(q))
        J = O = new RegressionTransformNode(O, q), G = "derived";
      else if (isLoess(q))
        J = O = new LoessTransformNode(O, q), G = "derived";
      else {
        warn(invalidTransformIgnored(q));
        continue;
      }
      if (J && G !== void 0)
        for (const Z of J.producedFields() ?? [])
          H.set(Z, G, !1);
    }
    return O;
  }
  function parseData(O) {
    var ue;
    let _ = parseRoot(O, O.component.data.sources);
    const { outputNodes: H, outputNodeRefCounts: V } = O.component.data, q = O.data, J = !(q && (isGenerator(q) || isUrlData(q) || isInlineData(q))) && O.parent ? O.parent.component.data.ancestorParse.clone() : new AncestorParse();
    isGenerator(q) ? (isSequenceGenerator(q) ? _ = new SequenceNode(_, q.sequence) : isGraticuleGenerator(q) && (_ = new GraticuleNode(_, q.graticule)), J.parseNothing = !0) : ((ue = q == null ? void 0 : q.format) == null ? void 0 : ue.parse) === null && (J.parseNothing = !0), _ = ParseNode.makeExplicit(_, O, J) ?? _, _ = new IdentifierNode(_);
    const Z = O.parent && isLayerModel(O.parent);
    (isUnitModel(O) || isFacetModel(O)) && Z && (_ = BinNode.makeFromEncoding(_, O) ?? _), O.transforms.length > 0 && (_ = parseTransformArray(_, O, J));
    const te = getImplicitFromSelection(O), ne = getImplicitFromEncoding(O);
    _ = ParseNode.makeWithAncestors(_, {}, { ...te, ...ne }, J) ?? _, isUnitModel(O) && (_ = GeoJSONNode.parseAll(_, O), _ = GeoPointNode.parseAll(_, O)), (isUnitModel(O) || isFacetModel(O)) && (Z || (_ = BinNode.makeFromEncoding(_, O) ?? _), _ = TimeUnitNode.makeFromEncoding(_, O) ?? _, _ = CalculateNode.parseAllForSortIndex(_, O));
    const ee = _ = makeOutputNode(DataSourceType.Raw, O, _);
    if (isUnitModel(O)) {
      const de = AggregateNode.makeFromEncoding(_, O);
      de && (_ = de, requiresSelectionId(O) && (_ = new IdentifierNode(_))), _ = ImputeNode.makeFromEncoding(_, O) ?? _, _ = StackNode.makeFromEncoding(_, O) ?? _;
    }
    let ae, oe;
    if (isUnitModel(O)) {
      const { markDef: de, mark: fe, config: he } = O, me = getMarkPropOrConfig("invalid", de, he), { marks: pe, scales: ye } = oe = getDataSourcesForHandlingInvalidValues({
        invalid: me,
        isPath: isPathMark(fe)
      });
      pe !== ye && ye === "include-invalid-values" && (ae = _ = makeOutputNode(DataSourceType.PreFilterInvalid, O, _)), pe === "exclude-invalid-values" && (_ = FilterInvalidNode.make(_, O, oe) ?? _);
    }
    const se = _ = makeOutputNode(DataSourceType.Main, O, _);
    let ce;
    if (isUnitModel(O) && oe) {
      const { marks: de, scales: fe } = oe;
      de === "include-invalid-values" && fe === "exclude-invalid-values" && (_ = FilterInvalidNode.make(_, O, oe) ?? _, ce = _ = makeOutputNode(DataSourceType.PostFilterInvalid, O, _));
    }
    isUnitModel(O) && materializeSelections(O, se);
    let le = null;
    if (isFacetModel(O)) {
      const de = O.getName("facet");
      _ = makeJoinAggregateFromFacet(_, O.facet) ?? _, le = new FacetNode(_, O, de, se.getSource()), H[de] = le;
    }
    return {
      ...O.component.data,
      outputNodes: H,
      outputNodeRefCounts: V,
      raw: ee,
      main: se,
      facetRoot: le,
      ancestorParse: J,
      preFilterInvalid: ae,
      postFilterInvalid: ce
    };
  }
  function makeOutputNode(O, _, H) {
    const { outputNodes: V, outputNodeRefCounts: q } = _.component.data, G = _.getDataName(O), J = new OutputNode(H, G, O, q);
    return V[G] = J, J;
  }
  class ConcatModel extends Model {
    constructor(_, H, V, q) {
      var G, J, Z, te;
      super(_, "concat", H, V, q, _.resolve), (((J = (G = _.resolve) == null ? void 0 : G.axis) == null ? void 0 : J.x) === "shared" || ((te = (Z = _.resolve) == null ? void 0 : Z.axis) == null ? void 0 : te.y) === "shared") && warn(CONCAT_CANNOT_SHARE_AXIS), this.children = this.getChildren(_).map((ne, ee) => buildModel(ne, this, this.getName(`concat_${ee}`), void 0, q));
    }
    parseData() {
      this.component.data = parseData(this);
      for (const _ of this.children)
        _.parseData();
    }
    parseSelections() {
      this.component.selection = {};
      for (const _ of this.children) {
        _.parseSelections();
        for (const H of keys$2(_.component.selection))
          this.component.selection[H] = _.component.selection[H];
      }
      Object.values(this.component.selection).some((_) => isTimerSelection(_)) && error(MULTI_VIEW_ANIMATION_UNSUPPORTED);
    }
    parseMarkGroup() {
      for (const _ of this.children)
        _.parseMarkGroup();
    }
    parseAxesAndHeaders() {
      for (const _ of this.children)
        _.parseAxesAndHeaders();
    }
    getChildren(_) {
      return isVConcatSpec(_) ? _.vconcat : isHConcatSpec(_) ? _.hconcat : _.concat;
    }
    parseLayoutSize() {
      parseConcatLayoutSize(this);
    }
    parseAxisGroup() {
      return null;
    }
    assembleSelectionTopLevelSignals(_) {
      return this.children.reduce((H, V) => V.assembleSelectionTopLevelSignals(H), _);
    }
    assembleSignals() {
      return this.children.forEach((_) => _.assembleSignals()), [];
    }
    assembleLayoutSignals() {
      const _ = assembleLayoutSignals(this);
      for (const H of this.children)
        _.push(...H.assembleLayoutSignals());
      return _;
    }
    assembleSelectionData(_) {
      return this.children.reduce((H, V) => V.assembleSelectionData(H), _);
    }
    assembleMarks() {
      return this.children.map((_) => {
        const H = _.assembleTitle(), V = _.assembleGroupStyle(), q = _.assembleGroupEncodeEntry(!1);
        return {
          type: "group",
          name: _.getName("group"),
          ...H ? { title: H } : {},
          ...V ? { style: V } : {},
          ...q ? { encode: { update: q } } : {},
          ..._.assembleGroup()
        };
      });
    }
    assembleGroupStyle() {
    }
    assembleDefaultLayout() {
      const _ = this.layout.columns;
      return {
        ..._ != null ? { columns: _ } : {},
        bounds: "full",
        // Use align each so it can work with multiple plots with different size
        align: "each"
      };
    }
  }
  function isFalseOrNull(O) {
    return O === !1 || O === null;
  }
  const AXIS_COMPONENT_PROPERTIES_INDEX = {
    disable: 1,
    gridScale: 1,
    scale: 1,
    ...COMMON_AXIS_PROPERTIES_INDEX,
    labelExpr: 1,
    encode: 1
  }, AXIS_COMPONENT_PROPERTIES = keys$2(AXIS_COMPONENT_PROPERTIES_INDEX);
  class AxisComponent extends Split {
    constructor(_ = {}, H = {}, V = !1) {
      super(), this.explicit = _, this.implicit = H, this.mainExtracted = V;
    }
    clone() {
      return new AxisComponent(duplicate(this.explicit), duplicate(this.implicit), this.mainExtracted);
    }
    hasAxisPart(_) {
      return _ === "axis" ? !0 : _ === "grid" || _ === "title" ? !!this.get(_) : !isFalseOrNull(this.get(_));
    }
    hasOrientSignalRef() {
      return isSignalRef(this.explicit.orient);
    }
  }
  function labels(O, _, H) {
    const { encoding: V, config: q } = O, G = getFieldOrDatumDef(V[_]) ?? getFieldOrDatumDef(V[getSecondaryRangeChannel(_)]), J = O.axis(_) || {}, { format: Z, formatType: te } = J;
    if (isCustomFormatType(te))
      return {
        text: formatCustomType({
          fieldOrDatumDef: G,
          field: "datum.value",
          format: Z,
          formatType: te,
          config: q
        }),
        ...H
      };
    if (Z === void 0 && te === void 0 && q.customFormatTypes) {
      if (channelDefType(G) === "quantitative") {
        if (isPositionFieldOrDatumDef(G) && G.stack === "normalize" && q.normalizedNumberFormatType)
          return {
            text: formatCustomType({
              fieldOrDatumDef: G,
              field: "datum.value",
              format: q.normalizedNumberFormat,
              formatType: q.normalizedNumberFormatType,
              config: q
            }),
            ...H
          };
        if (q.numberFormatType)
          return {
            text: formatCustomType({
              fieldOrDatumDef: G,
              field: "datum.value",
              format: q.numberFormat,
              formatType: q.numberFormatType,
              config: q
            }),
            ...H
          };
      }
      if (channelDefType(G) === "temporal" && q.timeFormatType && isFieldDef(G) && !G.timeUnit)
        return {
          text: formatCustomType({
            fieldOrDatumDef: G,
            field: "datum.value",
            format: q.timeFormat,
            formatType: q.timeFormatType,
            config: q
          }),
          ...H
        };
    }
    return H;
  }
  function parseUnitAxes(O) {
    return POSITION_SCALE_CHANNELS.reduce((_, H) => (O.component.scales[H] && (_[H] = [parseAxis(H, O)]), _), {});
  }
  const OPPOSITE_ORIENT = {
    bottom: "top",
    top: "bottom",
    left: "right",
    right: "left"
  };
  function parseLayerAxes(O) {
    const { axes: _, resolve: H } = O.component, V = { top: 0, bottom: 0, right: 0, left: 0 };
    for (const q of O.children) {
      q.parseAxesAndHeaders();
      for (const G of keys$2(q.component.axes))
        H.axis[G] = parseGuideResolve(O.component.resolve, G), H.axis[G] === "shared" && (_[G] = mergeAxisComponents(_[G], q.component.axes[G]), _[G] || (H.axis[G] = "independent", delete _[G]));
    }
    for (const q of POSITION_SCALE_CHANNELS) {
      for (const G of O.children)
        if (G.component.axes[q]) {
          if (H.axis[q] === "independent") {
            _[q] = (_[q] ?? []).concat(G.component.axes[q]);
            for (const J of G.component.axes[q]) {
              const { value: Z, explicit: te } = J.getWithExplicit("orient");
              if (!isSignalRef(Z)) {
                if (V[Z] > 0 && !te) {
                  const ne = OPPOSITE_ORIENT[Z];
                  V[Z] > V[ne] && J.set("orient", ne, !1);
                }
                V[Z]++;
              }
            }
          }
          delete G.component.axes[q];
        }
      if (H.axis[q] === "independent" && _[q] && _[q].length > 1)
        for (const [G, J] of (_[q] || []).entries())
          G > 0 && J.get("grid") && !J.explicit.grid && (J.implicit.grid = !1);
    }
  }
  function mergeAxisComponents(O, _) {
    if (O) {
      if (O.length !== _.length)
        return;
      const H = O.length;
      for (let V = 0; V < H; V++) {
        const q = O[V], G = _[V];
        if (!!q != !!G)
          return;
        if (q && G) {
          const J = q.getWithExplicit("orient"), Z = G.getWithExplicit("orient");
          if (J.explicit && Z.explicit && J.value !== Z.value)
            return;
          O[V] = mergeAxisComponent(q, G);
        }
      }
    } else
      return _.map((H) => H.clone());
    return O;
  }
  function mergeAxisComponent(O, _) {
    for (const H of AXIS_COMPONENT_PROPERTIES) {
      const V = mergeValuesWithExplicit(
        O.getWithExplicit(H),
        _.getWithExplicit(H),
        H,
        "axis",
        // Tie breaker function
        (q, G) => {
          switch (H) {
            case "title":
              return mergeTitleComponent(q, G);
            case "gridScale":
              return {
                explicit: q.explicit,
                // keep the old explicit
                value: getFirstDefined(q.value, G.value)
              };
          }
          return defaultTieBreaker(q, G, H, "axis");
        }
      );
      O.setWithExplicit(H, V);
    }
    return O;
  }
  function isExplicit(O, _, H, V, q) {
    if (_ === "disable")
      return H !== void 0;
    switch (H = H || {}, _) {
      case "titleAngle":
      case "labelAngle":
        return O === (isSignalRef(H.labelAngle) ? H.labelAngle : normalizeAngle(H.labelAngle));
      case "values":
        return !!H.values;
      // specified axis.values is already respected, but may get transformed.
      case "encode":
        return !!H.encoding || !!H.labelAngle;
      case "title":
        if (O === getFieldDefTitle(V, q))
          return !0;
    }
    return O === H[_];
  }
  const propsToAlwaysIncludeConfig = /* @__PURE__ */ new Set([
    "grid",
    // Grid is an exception because we need to set grid = true to generate another grid axis
    "translate",
    // translate has dependent logic for bar's bin position and it's 0.5 by default in Vega. If a config overrides this value, we need to know.
    // the rest are not axis configs in Vega, but are in VL, so we need to set too.
    "format",
    "formatType",
    "orient",
    "labelExpr",
    "tickCount",
    "position",
    "tickMinStep"
  ]);
  function parseAxis(O, _) {
    var de, fe;
    let H = _.axis(O);
    const V = new AxisComponent(), q = getFieldOrDatumDef(_.encoding[O]), { mark: G, config: J } = _, Z = (H == null ? void 0 : H.orient) || ((de = J[O === "x" ? "axisX" : "axisY"]) == null ? void 0 : de.orient) || ((fe = J.axis) == null ? void 0 : fe.orient) || defaultOrient(O), te = _.getScaleComponent(O).get("type"), ne = getAxisConfigs(O, te, Z, _.config), ee = H !== void 0 ? !H : getAxisConfig("disable", J.style, H == null ? void 0 : H.style, ne).configValue;
    if (V.set("disable", ee, H !== void 0), ee)
      return V;
    H = H || {};
    const ae = getLabelAngle(q, H, O, J.style, ne), oe = guideFormatType(H.formatType, q, te), se = guideFormat(q, q.type, H.format, H.formatType, J, !0), ce = {
      fieldOrDatumDef: q,
      axis: H,
      channel: O,
      model: _,
      scaleType: te,
      orient: Z,
      labelAngle: ae,
      format: se,
      formatType: oe,
      mark: G,
      config: J
    };
    for (const he of AXIS_COMPONENT_PROPERTIES) {
      const me = he in axisRules ? axisRules[he](ce) : isAxisProperty(he) ? H[he] : void 0, pe = me !== void 0, ye = isExplicit(me, he, H, _, O);
      if (pe && ye)
        V.set(he, me, ye);
      else {
        const { configValue: ge = void 0, configFrom: ve = void 0 } = isAxisProperty(he) && he !== "values" ? getAxisConfig(he, J.style, H.style, ne) : {}, be = ge !== void 0;
        pe && !be ? V.set(he, me, ye) : (
          // Cases need implicit values
          // 1. Axis config that aren't available in Vega
          (ve !== "vgAxisConfig" || // 2. Certain properties are always included (see `propsToAlwaysIncludeConfig`'s declaration for more details)
          propsToAlwaysIncludeConfig.has(he) && be || // 3. Conditional axis values and signals
          isConditionalAxisValue(ge) || isSignalRef(ge)) && V.set(he, ge, !1)
        );
      }
    }
    const le = H.encoding ?? {}, ue = AXIS_PARTS.reduce((he, me) => {
      if (!V.hasAxisPart(me))
        return he;
      const pe = guideEncodeEntry(le[me] ?? {}, _), ye = me === "labels" ? labels(_, O, pe) : pe;
      return ye !== void 0 && !isEmpty(ye) && (he[me] = { update: ye }), he;
    }, {});
    return isEmpty(ue) || V.set("encode", ue, !!H.encoding || H.labelAngle !== void 0), V;
  }
  function initLayoutSize({ encoding: O, size: _ }) {
    for (const H of POSITION_SCALE_CHANNELS) {
      const V = getSizeChannel(H);
      isStep(_[V]) && isContinuousFieldOrDatumDef(O[H]) && (delete _[V], warn(stepDropped(V)));
    }
    return _;
  }
  const arc = {
    vgMark: "arc",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        size: "ignore",
        orient: "ignore",
        theta: "ignore"
      }),
      ...pointPosition("x", O, { defaultPos: "mid" }),
      ...pointPosition("y", O, { defaultPos: "mid" }),
      // arcs are rectangles in polar coordinates
      ...rectPosition(O, "radius"),
      ...rectPosition(O, "theta")
    })
  }, area = {
    vgMark: "area",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        orient: "include",
        size: "ignore",
        theta: "ignore"
      }),
      ...pointOrRangePosition("x", O, {
        defaultPos: "zeroOrMin",
        defaultPos2: "zeroOrMin",
        range: O.markDef.orient === "horizontal"
      }),
      ...pointOrRangePosition("y", O, {
        defaultPos: "zeroOrMin",
        defaultPos2: "zeroOrMin",
        range: O.markDef.orient === "vertical"
      }),
      ...defined(O)
    })
  }, bar = {
    vgMark: "rect",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        orient: "ignore",
        size: "ignore",
        theta: "ignore"
      }),
      ...rectPosition(O, "x"),
      ...rectPosition(O, "y")
    })
  }, geoshape = {
    vgMark: "shape",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        size: "ignore",
        orient: "ignore",
        theta: "ignore"
      })
    }),
    postEncodingTransform: (O) => {
      const { encoding: _ } = O, H = _.shape;
      return [{
        type: "geoshape",
        projection: O.projectionName(),
        // as: 'shape',
        ...H && isFieldDef(H) && H.type === GEOJSON ? { field: vgField(H, { expr: "datum" }) } : {}
      }];
    }
  }, image = {
    vgMark: "image",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "ignore",
        orient: "ignore",
        size: "ignore",
        theta: "ignore"
      }),
      ...rectPosition(O, "x"),
      ...rectPosition(O, "y"),
      ...text$1(O, "url")
    })
  }, line = {
    vgMark: "line",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        size: "ignore",
        orient: "ignore",
        theta: "ignore"
      }),
      ...pointPosition("x", O, { defaultPos: "mid" }),
      ...pointPosition("y", O, { defaultPos: "mid" }),
      ...nonPosition("size", O, {
        vgChannel: "strokeWidth"
        // VL's line size is strokeWidth
      }),
      ...defined(O)
    })
  }, trail = {
    vgMark: "trail",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        size: "include",
        orient: "ignore",
        theta: "ignore"
      }),
      ...pointPosition("x", O, { defaultPos: "mid" }),
      ...pointPosition("y", O, { defaultPos: "mid" }),
      ...nonPosition("size", O),
      ...defined(O)
    })
  };
  function encodeEntry(O, _) {
    const { config: H } = O;
    return {
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        size: "include",
        orient: "ignore",
        theta: "ignore"
      }),
      ...pointPosition("x", O, { defaultPos: "mid" }),
      ...pointPosition("y", O, { defaultPos: "mid" }),
      ...nonPosition("size", O),
      ...nonPosition("angle", O),
      ...shapeMixins(O, H, _)
    };
  }
  function shapeMixins(O, _, H) {
    return H ? { shape: { value: H } } : nonPosition("shape", O);
  }
  const point = {
    vgMark: "symbol",
    encodeEntry: (O) => encodeEntry(O)
  }, circle = {
    vgMark: "symbol",
    encodeEntry: (O) => encodeEntry(O, "circle")
  }, square = {
    vgMark: "symbol",
    encodeEntry: (O) => encodeEntry(O, "square")
  }, rect = {
    vgMark: "rect",
    encodeEntry: (O) => ({
      ...baseEncodeEntry(O, {
        align: "ignore",
        baseline: "ignore",
        color: "include",
        orient: "ignore",
        size: "ignore",
        theta: "ignore"
      }),
      ...rectPosition(O, "x"),
      ...rectPosition(O, "y")
    })
  }, rule = {
    vgMark: "rule",
    encodeEntry: (O) => {
      const { markDef: _ } = O, H = _.orient;
      return !O.encoding.x && !O.encoding.y && !O.encoding.latitude && !O.encoding.longitude ? {} : {
        ...baseEncodeEntry(O, {
          align: "ignore",
          baseline: "ignore",
          color: "include",
          orient: "ignore",
          size: "ignore",
          theta: "ignore"
        }),
        ...pointOrRangePosition("x", O, {
          defaultPos: H === "horizontal" ? "zeroOrMax" : "mid",
          defaultPos2: "zeroOrMin",
          range: H !== "vertical"
          // include x2 for horizontal or line segment rule
        }),
        ...pointOrRangePosition("y", O, {
          defaultPos: H === "vertical" ? "zeroOrMax" : "mid",
          defaultPos2: "zeroOrMin",
          range: H !== "horizontal"
          // include y2 for vertical or line segment rule
        }),
        ...nonPosition("size", O, {
          vgChannel: "strokeWidth"
          // VL's rule size is strokeWidth
        })
      };
    }
  }, text = {
    vgMark: "text",
    encodeEntry: (O) => {
      const { config: _, encoding: H } = O;
      return {
        ...baseEncodeEntry(O, {
          align: "include",
          baseline: "include",
          color: "include",
          size: "ignore",
          orient: "ignore",
          theta: "include"
        }),
        ...pointPosition("x", O, { defaultPos: "mid" }),
        ...pointPosition("y", O, { defaultPos: "mid" }),
        ...text$1(O),
        ...nonPosition("size", O, {
          vgChannel: "fontSize"
          // VL's text size is fontSize
        }),
        ...nonPosition("angle", O),
        ...valueIfDefined("align", align(O.markDef, H, _)),
        ...valueIfDefined("baseline", baseline(O.markDef, H, _)),
        ...pointPosition("radius", O, { defaultPos: null }),
        ...pointPosition("theta", O, { defaultPos: null })
      };
    }
  };
  function align(O, _, H) {
    if (getMarkPropOrConfig("align", O, H) === void 0)
      return "center";
  }
  function baseline(O, _, H) {
    if (getMarkPropOrConfig("baseline", O, H) === void 0)
      return "middle";
  }
  const tick = {
    vgMark: "rect",
    encodeEntry: (O) => {
      const { config: _, markDef: H } = O, V = H.orient, q = V === "horizontal" ? "x" : "y", G = V === "horizontal" ? "y" : "x", J = V === "horizontal" ? "height" : "width";
      return {
        ...baseEncodeEntry(O, {
          align: "ignore",
          baseline: "ignore",
          color: "include",
          orient: "ignore",
          size: "ignore",
          theta: "ignore"
        }),
        ...rectPosition(O, q),
        ...pointPosition(G, O, {
          defaultPos: "mid",
          vgChannel: G === "y" ? "yc" : "xc"
        }),
        [J]: signalOrValueRef(getMarkPropOrConfig("thickness", H, _))
      };
    }
  }, markCompiler = {
    arc,
    area,
    bar,
    circle,
    geoshape,
    image,
    line,
    point,
    rect,
    rule,
    square,
    text,
    tick,
    trail
  };
  function parseMarkGroups(O) {
    if (contains$2([LINE, AREA, TRAIL], O.mark)) {
      const _ = pathGroupingFields(O.mark, O.encoding);
      if (_.length > 0)
        return getPathGroups(O, _);
    } else if (O.mark === BAR) {
      const _ = VG_CORNERRADIUS_CHANNELS.some((H) => getMarkPropOrConfig(H, O.markDef, O.config));
      if (O.stack && !O.fieldDef("size") && _)
        return getGroupsForStackedBarWithCornerRadius(O);
    }
    return getMarkGroup(O);
  }
  const FACETED_PATH_PREFIX = "faceted_path_";
  function getPathGroups(O, _) {
    return [
      {
        name: O.getName("pathgroup"),
        type: "group",
        from: {
          facet: {
            name: FACETED_PATH_PREFIX + O.requestDataName(DataSourceType.Main),
            data: O.requestDataName(DataSourceType.Main),
            groupby: _
          }
        },
        encode: {
          update: {
            width: { field: { group: "width" } },
            height: { field: { group: "height" } }
          }
        },
        // With subfacet for line/area group, need to use faceted data from above.
        marks: getMarkGroup(O, { fromPrefix: FACETED_PATH_PREFIX })
      }
    ];
  }
  const STACK_GROUP_PREFIX = "stack_group_";
  function getGroupsForStackedBarWithCornerRadius(O) {
    var ne;
    const [_] = getMarkGroup(O, { fromPrefix: STACK_GROUP_PREFIX }), H = O.scaleName(O.stack.fieldChannel), V = (ee = {}) => O.vgField(O.stack.fieldChannel, ee), q = (ee, ae) => {
      const oe = [
        V({ prefix: "min", suffix: "start", expr: ae }),
        V({ prefix: "max", suffix: "start", expr: ae }),
        V({ prefix: "min", suffix: "end", expr: ae }),
        V({ prefix: "max", suffix: "end", expr: ae })
      ];
      return `${ee}(${oe.map((se) => `scale('${H}',${se})`).join(",")})`;
    };
    let G, J;
    O.stack.fieldChannel === "x" ? (G = {
      ...pick$1(_.encode.update, ["y", "yc", "y2", "height", ...VG_CORNERRADIUS_CHANNELS]),
      x: { signal: q("min", "datum") },
      x2: { signal: q("max", "datum") },
      clip: { value: !0 }
    }, J = {
      x: { field: { group: "x" }, mult: -1 },
      height: { field: { group: "height" } }
    }, _.encode.update = {
      ...omit(_.encode.update, ["y", "yc", "y2"]),
      height: { field: { group: "height" } }
    }) : (G = {
      ...pick$1(_.encode.update, ["x", "xc", "x2", "width"]),
      y: { signal: q("min", "datum") },
      y2: { signal: q("max", "datum") },
      clip: { value: !0 }
    }, J = {
      y: { field: { group: "y" }, mult: -1 },
      width: { field: { group: "width" } }
    }, _.encode.update = {
      ...omit(_.encode.update, ["x", "xc", "x2"]),
      width: { field: { group: "width" } }
    });
    for (const ee of VG_CORNERRADIUS_CHANNELS) {
      const ae = getMarkConfig(ee, O.markDef, O.config);
      _.encode.update[ee] ? (G[ee] = _.encode.update[ee], delete _.encode.update[ee]) : ae && (G[ee] = signalOrValueRef(ae)), ae && (_.encode.update[ee] = { value: 0 });
    }
    const Z = [];
    if (((ne = O.stack.groupbyChannels) == null ? void 0 : ne.length) > 0)
      for (const ee of O.stack.groupbyChannels) {
        const ae = O.fieldDef(ee), oe = vgField(ae);
        oe && Z.push(oe), (ae != null && ae.bin || ae != null && ae.timeUnit) && Z.push(vgField(ae, { binSuffix: "end" }));
      }
    return G = [
      "stroke",
      "strokeWidth",
      "strokeJoin",
      "strokeCap",
      "strokeDash",
      "strokeDashOffset",
      "strokeMiterLimit",
      "strokeOpacity"
    ].reduce((ee, ae) => {
      if (_.encode.update[ae])
        return { ...ee, [ae]: _.encode.update[ae] };
      {
        const oe = getMarkConfig(ae, O.markDef, O.config);
        return oe !== void 0 ? { ...ee, [ae]: signalOrValueRef(oe) } : ee;
      }
    }, G), G.stroke && (G.strokeForeground = { value: !0 }, G.strokeOffset = { value: 0 }), [
      {
        type: "group",
        from: {
          facet: {
            data: O.requestDataName(DataSourceType.Main),
            name: STACK_GROUP_PREFIX + O.requestDataName(DataSourceType.Main),
            groupby: Z,
            aggregate: {
              fields: [
                V({ suffix: "start" }),
                V({ suffix: "start" }),
                V({ suffix: "end" }),
                V({ suffix: "end" })
              ],
              ops: ["min", "max", "min", "max"]
            }
          }
        },
        encode: {
          update: G
        },
        marks: [
          {
            type: "group",
            encode: { update: J },
            marks: [_]
          }
        ]
      }
    ];
  }
  function getSort(O) {
    const { encoding: _, stack: H, mark: V, markDef: q, config: G } = O, J = _.order;
    if (!(!isArray$2(J) && isValueDef(J) && isNullOrFalse(J.value) || !J && isNullOrFalse(getMarkPropOrConfig("order", q, G)))) {
      if ((isArray$2(J) || isFieldDef(J)) && !H)
        return sortParams(J, { expr: "datum" });
      if (isPathMark(V)) {
        const Z = q.orient === "horizontal" ? "y" : "x", te = _[Z];
        if (isFieldDef(te))
          return { field: Z };
      }
    }
  }
  function getMarkGroup(O, _ = { fromPrefix: "" }) {
    const { mark: H, markDef: V, encoding: q, config: G } = O, J = getFirstDefined(V.clip, scaleClip(O), projectionClip(O)), Z = getStyles(V), te = q.key, ne = getSort(O), ee = interactiveFlag(O), ae = getMarkPropOrConfig("aria", V, G), oe = markCompiler[H].postEncodingTransform ? markCompiler[H].postEncodingTransform(O) : null;
    return [
      {
        name: O.getName("marks"),
        type: markCompiler[H].vgMark,
        ...J ? { clip: J } : {},
        style: Z,
        ...te ? { key: te.field } : {},
        ...ne ? { sort: ne } : {},
        ...ee || {},
        ...ae === !1 ? { aria: ae } : {},
        from: { data: _.fromPrefix + O.requestDataName(DataSourceType.Main) },
        encode: {
          update: markCompiler[H].encodeEntry(O)
        },
        ...oe ? {
          transform: oe
        } : {}
      }
    ];
  }
  function scaleClip(O) {
    const _ = O.getScaleComponent("x"), H = O.getScaleComponent("y");
    return _ != null && _.get("selectionExtent") || H != null && H.get("selectionExtent") ? !0 : void 0;
  }
  function projectionClip(O) {
    const _ = O.component.projection;
    return _ && !_.isFit ? !0 : void 0;
  }
  function interactiveFlag(O) {
    if (!O.component.selection)
      return null;
    const _ = keys$2(O.component.selection).length;
    let H = _, V = O.parent;
    for (; V && H === 0; )
      H = keys$2(V.component.selection).length, V = V.parent;
    return H ? {
      interactive: _ > 0 || O.mark === "geoshape" || !!O.encoding.tooltip || !!O.markDef.tooltip
    } : null;
  }
  class UnitModel extends ModelWithField {
    constructor(_, H, V, q = {}, G) {
      super(_, "unit", H, V, G, void 0, isFrameMixins(_) ? _.view : void 0), this.specifiedScales = {}, this.specifiedAxes = {}, this.specifiedLegends = {}, this.specifiedProjection = {}, this.selection = [], this.children = [], this.correctDataNames = (ne) => {
        var ee, ae, oe;
        return (ee = ne.from) != null && ee.data && (ne.from.data = this.lookupDataSource(ne.from.data), "time" in this.encoding && (ne.from.data = ne.from.data + CURR)), (oe = (ae = ne.from) == null ? void 0 : ae.facet) != null && oe.data && (ne.from.facet.data = this.lookupDataSource(ne.from.facet.data)), ne;
      };
      const J = isMarkDef(_.mark) ? { ..._.mark } : { type: _.mark }, Z = J.type;
      J.filled === void 0 && (J.filled = defaultFilled(J, G, {
        graticule: _.data && isGraticuleGenerator(_.data)
      }));
      const te = this.encoding = initEncoding(_.encoding || {}, Z, J.filled, G);
      this.markDef = initMarkdef(J, te, G), this.size = initLayoutSize({
        encoding: te,
        size: isFrameMixins(_) ? {
          ...q,
          ..._.width ? { width: _.width } : {},
          ..._.height ? { height: _.height } : {}
        } : q
      }), this.stack = stack(this.markDef, te), this.specifiedScales = this.initScales(Z, te), this.specifiedAxes = this.initAxes(te), this.specifiedLegends = this.initLegends(te), this.specifiedProjection = _.projection, this.selection = (_.params ?? []).filter((ne) => isSelectionParameter(ne));
    }
    get hasProjection() {
      const { encoding: _ } = this, H = this.mark === GEOSHAPE, V = _ && GEOPOSITION_CHANNELS.some((q) => isFieldOrDatumDef(_[q]));
      return H || V;
    }
    /**
     * Return specified Vega-Lite scale domain for a particular channel
     * @param channel
     */
    scaleDomain(_) {
      const H = this.specifiedScales[_];
      return H ? H.domain : void 0;
    }
    axis(_) {
      return this.specifiedAxes[_];
    }
    legend(_) {
      return this.specifiedLegends[_];
    }
    initScales(_, H) {
      return SCALE_CHANNELS.reduce((V, q) => {
        const G = getFieldOrDatumDef(H[q]);
        return G && (V[q] = this.initScale(G.scale ?? {})), V;
      }, {});
    }
    initScale(_) {
      const { domain: H, range: V } = _, q = replaceExprRef(_);
      return isArray$2(H) && (q.domain = H.map(signalRefOrValue)), isArray$2(V) && (q.range = V.map(signalRefOrValue)), q;
    }
    initAxes(_) {
      return POSITION_SCALE_CHANNELS.reduce((H, V) => {
        const q = _[V];
        if (isFieldOrDatumDef(q) || V === X && isFieldOrDatumDef(_.x2) || V === Y$1 && isFieldOrDatumDef(_.y2)) {
          const G = isFieldOrDatumDef(q) ? q.axis : void 0;
          H[V] = G && this.initAxis({ ...G });
        }
        return H;
      }, {});
    }
    initAxis(_) {
      const H = keys$2(_), V = {};
      for (const q of H) {
        const G = _[q];
        V[q] = isConditionalAxisValue(G) ? signalOrValueRefWithCondition(G) : signalRefOrValue(G);
      }
      return V;
    }
    initLegends(_) {
      return NONPOSITION_SCALE_CHANNELS.reduce((H, V) => {
        const q = getFieldOrDatumDef(_[V]);
        if (q && supportLegend(V)) {
          const G = q.legend;
          H[V] = G && replaceExprRef(G);
        }
        return H;
      }, {});
    }
    parseData() {
      this.component.data = parseData(this);
    }
    parseLayoutSize() {
      parseUnitLayoutSize(this);
    }
    parseSelections() {
      this.component.selection = parseUnitSelection(this, this.selection);
    }
    parseMarkGroup() {
      this.component.mark = parseMarkGroups(this);
    }
    parseAxesAndHeaders() {
      this.component.axes = parseUnitAxes(this);
    }
    assembleSelectionTopLevelSignals(_) {
      return assembleTopLevelSignals(this, _);
    }
    assembleSignals() {
      return [...assembleAxisSignals(this), ...assembleUnitSelectionSignals(this, [])];
    }
    assembleSelectionData(_) {
      return assembleUnitSelectionData(this, _);
    }
    assembleLayout() {
      return null;
    }
    assembleLayoutSignals() {
      return assembleLayoutSignals(this);
    }
    assembleMarks() {
      let _ = this.component.mark ?? [];
      return (!this.parent || !isLayerModel(this.parent)) && (_ = assembleUnitSelectionMarks(this, _)), _.map(this.correctDataNames);
    }
    assembleGroupStyle() {
      const { style: _ } = this.view || {};
      return _ !== void 0 ? _ : this.encoding.x || this.encoding.y ? "cell" : "view";
    }
    getMapping() {
      return this.encoding;
    }
    get mark() {
      return this.markDef.type;
    }
    channelHasField(_) {
      return channelHasField(this.encoding, _);
    }
    fieldDef(_) {
      const H = this.encoding[_];
      return getFieldDef(H);
    }
    typedFieldDef(_) {
      const H = this.fieldDef(_);
      return isTypedFieldDef(H) ? H : null;
    }
  }
  class LayerModel extends Model {
    constructor(_, H, V, q, G) {
      super(_, "layer", H, V, G, _.resolve, _.view);
      const J = {
        ...q,
        ..._.width ? { width: _.width } : {},
        ..._.height ? { height: _.height } : {}
      };
      this.children = _.layer.map((Z, te) => {
        if (isLayerSpec(Z))
          return new LayerModel(Z, this, this.getName(`layer_${te}`), J, G);
        if (isUnitSpec(Z))
          return new UnitModel(Z, this, this.getName(`layer_${te}`), J, G);
        throw new Error(invalidSpec(Z));
      });
    }
    parseData() {
      this.component.data = parseData(this);
      for (const _ of this.children)
        _.parseData();
    }
    parseLayoutSize() {
      parseLayerLayoutSize(this);
    }
    parseSelections() {
      this.component.selection = {};
      for (const _ of this.children) {
        _.parseSelections();
        for (const H of keys$2(_.component.selection))
          this.component.selection[H] = _.component.selection[H];
      }
      Object.values(this.component.selection).some((_) => isTimerSelection(_)) && error(MULTI_VIEW_ANIMATION_UNSUPPORTED);
    }
    parseMarkGroup() {
      for (const _ of this.children)
        _.parseMarkGroup();
    }
    parseAxesAndHeaders() {
      parseLayerAxes(this);
    }
    assembleSelectionTopLevelSignals(_) {
      return this.children.reduce((H, V) => V.assembleSelectionTopLevelSignals(H), _);
    }
    // TODO: Support same named selections across children.
    assembleSignals() {
      return this.children.reduce((_, H) => _.concat(H.assembleSignals()), assembleAxisSignals(this));
    }
    assembleLayoutSignals() {
      return this.children.reduce((_, H) => _.concat(H.assembleLayoutSignals()), assembleLayoutSignals(this));
    }
    assembleSelectionData(_) {
      return this.children.reduce((H, V) => V.assembleSelectionData(H), _);
    }
    assembleGroupStyle() {
      const _ = /* @__PURE__ */ new Set();
      for (const V of this.children)
        for (const q of array$5(V.assembleGroupStyle()))
          _.add(q);
      const H = Array.from(_);
      return H.length > 1 ? H : H.length === 1 ? H[0] : void 0;
    }
    assembleTitle() {
      let _ = super.assembleTitle();
      if (_)
        return _;
      for (const H of this.children)
        if (_ = H.assembleTitle(), _)
          return _;
    }
    assembleLayout() {
      return null;
    }
    assembleMarks() {
      return assembleLayerSelectionMarks(this, this.children.flatMap((_) => _.assembleMarks()));
    }
    assembleLegends() {
      return this.children.reduce((_, H) => _.concat(H.assembleLegends()), assembleLegends(this));
    }
  }
  function buildModel(O, _, H, V, q) {
    if (isFacetSpec(O))
      return new FacetModel(O, _, H, q);
    if (isLayerSpec(O))
      return new LayerModel(O, _, H, V, q);
    if (isUnitSpec(O))
      return new UnitModel(O, _, H, V, q);
    if (isAnyConcatSpec(O))
      return new ConcatModel(O, _, H, q);
    throw new Error(invalidSpec(O));
  }
  function compile(O, _ = {}) {
    _.logger && set$1(_.logger), _.fieldTitle && setTitleFormatter(_.fieldTitle);
    try {
      const H = initConfig(mergeConfig(_.config, O.config)), V = normalize(O, H), q = buildModel(V, null, "", void 0, H);
      return q.parse(), optimizeDataflow(q.component.data, q), {
        spec: assembleTopLevelModel(q, getTopLevelProperties(O, V.autosize, H, q), O.datasets, O.usermeta),
        normalized: V
      };
    } finally {
      _.logger && reset(), _.fieldTitle && resetTitleFormatter();
    }
  }
  function getTopLevelProperties(O, _, H, V) {
    const q = V.component.layoutSize.get("width"), G = V.component.layoutSize.get("height");
    if (_ === void 0 ? (_ = { type: "pad" }, V.hasAxisOrientSignalRef() && (_.resize = !0)) : isString$1(_) && (_ = { type: _ }), q && G && isFitType(_.type)) {
      if (q === "step" && G === "step")
        warn(droppingFit()), _.type = "pad";
      else if (q === "step" || G === "step") {
        const J = q === "step" ? "width" : "height";
        warn(droppingFit(getPositionScaleChannel(J)));
        const Z = J === "width" ? "height" : "width";
        _.type = getFitType(Z);
      }
    }
    return {
      ...keys$2(_).length === 1 && _.type ? _.type === "pad" ? {} : { autosize: _.type } : { autosize: _ },
      ...extractTopLevelProperties(H, !1),
      ...extractTopLevelProperties(O, !0)
    };
  }
  function assembleTopLevelModel(O, _, H = {}, V) {
    const q = O.config ? stripAndRedirectConfig(O.config) : void 0, G = assembleRootData(O.component.data, H), J = O.assembleSelectionData(G), Z = O.assembleProjections(), te = O.assembleTitle(), ne = O.assembleGroupStyle(), ee = O.assembleGroupEncodeEntry(!0);
    let ae = O.assembleLayoutSignals();
    ae = ae.filter((ce) => (ce.name === "width" || ce.name === "height") && ce.value !== void 0 ? (_[ce.name] = +ce.value, !1) : !0);
    const { params: oe, ...se } = _;
    return {
      $schema: "https://vega.github.io/schema/vega/v5.json",
      ...O.description ? { description: O.description } : {},
      ...se,
      ...te ? { title: te } : {},
      ...ne ? { style: ne } : {},
      ...ee ? { encode: { update: ee } } : {},
      data: J,
      ...Z.length > 0 ? { projections: Z } : {},
      ...O.assembleGroup([
        ...ae,
        ...O.assembleSelectionTopLevelSignals([]),
        ...assembleParameterSignals(oe)
      ]),
      ...q ? { config: q } : {},
      ...V ? { usermeta: V } : {}
    };
  }
  const version$2 = pkg$1.version, vegaLiteImport = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    accessPathDepth,
    accessPathWithDatum,
    accessWithDatumToUnescapedPath,
    compile,
    contains: contains$2,
    deepEqual,
    deleteNestedProperty,
    duplicate,
    entries: entries$1,
    every,
    fieldIntersection,
    flatAccessWithDatum,
    getFirstDefined,
    hasIntersection,
    hasProperty,
    hash,
    internalField,
    isBoolean: isBoolean$1,
    isEmpty,
    isEqual,
    isInternalField,
    isNullOrFalse,
    isNumeric,
    keys: keys$2,
    logicalExpr,
    mergeDeep: mergeDeep$1,
    never,
    normalize,
    normalizeAngle,
    omit,
    pick: pick$1,
    prefixGenerator,
    removePathFromField,
    replaceAll: replaceAll$1,
    replacePathInField,
    resetIdCounter,
    setEqual,
    some,
    stringify: stringify$1,
    titleCase,
    unique,
    uniqueId,
    vals,
    varName,
    version: version$2
  }, Symbol.toStringTag, { value: "Module" }));
  function e(O) {
    const [_, H] = /schema\/([\w-]+)\/([\w\.\-]+)\.json$/g.exec(O).slice(1, 3);
    return { library: _, version: H };
  }
  var name$2 = "vega-themes", version$1 = "2.15.0", description = "Themes for stylized Vega and Vega-Lite visualizations.", keywords$1 = ["vega", "vega-lite", "themes", "style"], license = "BSD-3-Clause", author = {
    name: "UW Interactive Data Lab",
    url: "https://idl.cs.washington.edu"
  }, contributors = [{
    name: "Emily Gu",
    url: "https://github.com/emilygu"
  }, {
    name: "Arvind Satyanarayan",
    url: "http://arvindsatya.com"
  }, {
    name: "Jeffrey Heer",
    url: "https://idl.cs.washington.edu"
  }, {
    name: "Dominik Moritz",
    url: "https://www.domoritz.de"
  }], main = "build/vega-themes.js", module$1 = "build/vega-themes.module.js", unpkg = "build/vega-themes.min.js", jsdelivr = "build/vega-themes.min.js", types$1 = "build/vega-themes.module.d.ts", repository = {
    type: "git",
    url: "https://github.com/vega/vega-themes.git"
  }, files = ["src", "build"], scripts = {
    prebuild: "yarn clean",
    build: "rollup -c",
    clean: "rimraf build && rimraf examples/build",
    "copy:data": "rsync -r node_modules/vega-datasets/data/* examples/data",
    "copy:build": "rsync -r build/* examples/build",
    "deploy:gh": "yarn build && mkdir -p examples/build && rsync -r build/* examples/build && gh-pages -d examples",
    preversion: "yarn lint",
    serve: "browser-sync start -s -f build examples --serveStatic examples",
    start: "yarn build && concurrently --kill-others -n Server,Rollup 'yarn serve' 'rollup -c -w'",
    format: "eslint . --fix",
    lint: "eslint .",
    release: "release-it"
  }, devDependencies = {
    "@babel/core": "^7.24.6",
    "@babel/plugin-transform-runtime": "^7.24.6",
    "@babel/preset-env": "^7.24.6",
    "@babel/preset-typescript": "^7.24.6",
    "@release-it/conventional-changelog": "^8.0.1",
    "@rollup/plugin-json": "^6.1.0",
    "@rollup/plugin-node-resolve": "^15.2.3",
    "@rollup/plugin-terser": "^0.4.4",
    "@typescript-eslint/eslint-plugin": "^7.11.0",
    "@typescript-eslint/parser": "^7.11.0",
    "browser-sync": "^3.0.2",
    concurrently: "^8.2.2",
    eslint: "^8.45.0",
    "eslint-config-prettier": "^9.1.0",
    "eslint-plugin-prettier": "^5.1.3",
    "gh-pages": "^6.1.1",
    prettier: "^3.2.5",
    "release-it": "^17.3.0",
    rollup: "^4.18.0",
    "rollup-plugin-bundle-size": "^1.0.3",
    "rollup-plugin-ts": "^3.4.5",
    typescript: "^5.4.5",
    vega: "^5.25.0",
    "vega-lite": "^5.9.3"
  }, peerDependencies = {
    vega: "*",
    "vega-lite": "*"
  }, dependencies = {}, pkg = {
    name: name$2,
    version: version$1,
    description,
    keywords: keywords$1,
    license,
    author,
    contributors,
    main,
    module: module$1,
    unpkg,
    jsdelivr,
    types: types$1,
    repository,
    files,
    scripts,
    devDependencies,
    peerDependencies,
    dependencies
  };
  const lightColor = "#fff", medColor = "#888", darkTheme$1 = {
    background: "#333",
    view: {
      stroke: medColor
    },
    title: {
      color: lightColor,
      subtitleColor: lightColor
    },
    style: {
      "guide-label": {
        fill: lightColor
      },
      "guide-title": {
        fill: lightColor
      }
    },
    axis: {
      domainColor: lightColor,
      gridColor: medColor,
      tickColor: lightColor
    }
  }, markColor$7 = "#4572a7", excelTheme = {
    background: "#fff",
    arc: {
      fill: markColor$7
    },
    area: {
      fill: markColor$7
    },
    line: {
      stroke: markColor$7,
      strokeWidth: 2
    },
    path: {
      stroke: markColor$7
    },
    rect: {
      fill: markColor$7
    },
    shape: {
      stroke: markColor$7
    },
    symbol: {
      fill: markColor$7,
      strokeWidth: 1.5,
      size: 50
    },
    axis: {
      bandPosition: 0.5,
      grid: !0,
      gridColor: "#000000",
      gridOpacity: 1,
      gridWidth: 0.5,
      labelPadding: 10,
      tickSize: 5,
      tickWidth: 0.5
    },
    axisBand: {
      grid: !1,
      tickExtra: !0
    },
    legend: {
      labelBaseline: "middle",
      labelFontSize: 11,
      symbolSize: 50,
      symbolType: "square"
    },
    range: {
      category: ["#4572a7", "#aa4643", "#8aa453", "#71598e", "#4598ae", "#d98445", "#94aace", "#d09393", "#b9cc98", "#a99cbc"]
    }
  }, markColor$6 = "#30a2da", axisColor$2 = "#cbcbcb", guideLabelColor = "#999", guideTitleColor = "#333", backgroundColor$2 = "#f0f0f0", blackTitle = "#333", fiveThirtyEightTheme = {
    arc: {
      fill: markColor$6
    },
    area: {
      fill: markColor$6
    },
    axis: {
      domainColor: axisColor$2,
      grid: !0,
      gridColor: axisColor$2,
      gridWidth: 1,
      labelColor: guideLabelColor,
      labelFontSize: 10,
      titleColor: guideTitleColor,
      tickColor: axisColor$2,
      tickSize: 10,
      titleFontSize: 14,
      titlePadding: 10,
      labelPadding: 4
    },
    axisBand: {
      grid: !1
    },
    background: backgroundColor$2,
    group: {
      fill: backgroundColor$2
    },
    legend: {
      labelColor: blackTitle,
      labelFontSize: 11,
      padding: 1,
      symbolSize: 30,
      symbolType: "square",
      titleColor: blackTitle,
      titleFontSize: 14,
      titlePadding: 10
    },
    line: {
      stroke: markColor$6,
      strokeWidth: 2
    },
    path: {
      stroke: markColor$6,
      strokeWidth: 0.5
    },
    rect: {
      fill: markColor$6
    },
    range: {
      category: ["#30a2da", "#fc4f30", "#e5ae38", "#6d904f", "#8b8b8b", "#b96db8", "#ff9e27", "#56cc60", "#52d2ca", "#52689e", "#545454", "#9fe4f8"],
      diverging: ["#cc0020", "#e77866", "#f6e7e1", "#d6e8ed", "#91bfd9", "#1d78b5"],
      heatmap: ["#d6e8ed", "#cee0e5", "#91bfd9", "#549cc6", "#1d78b5"]
    },
    point: {
      filled: !0,
      shape: "circle"
    },
    shape: {
      stroke: markColor$6
    },
    bar: {
      binSpacing: 2,
      fill: markColor$6,
      stroke: null
    },
    title: {
      anchor: "start",
      fontSize: 24,
      fontWeight: 600,
      offset: 20
    }
  }, markColor$5 = "#000", ggplot2Theme = {
    group: {
      fill: "#e5e5e5"
    },
    arc: {
      fill: markColor$5
    },
    area: {
      fill: markColor$5
    },
    line: {
      stroke: markColor$5
    },
    path: {
      stroke: markColor$5
    },
    rect: {
      fill: markColor$5
    },
    shape: {
      stroke: markColor$5
    },
    symbol: {
      fill: markColor$5,
      size: 40
    },
    axis: {
      domain: !1,
      grid: !0,
      gridColor: "#FFFFFF",
      gridOpacity: 1,
      labelColor: "#7F7F7F",
      labelPadding: 4,
      tickColor: "#7F7F7F",
      tickSize: 5.67,
      titleFontSize: 16,
      titleFontWeight: "normal"
    },
    legend: {
      labelBaseline: "middle",
      labelFontSize: 11,
      symbolSize: 40
    },
    range: {
      category: ["#000000", "#7F7F7F", "#1A1A1A", "#999999", "#333333", "#B0B0B0", "#4D4D4D", "#C9C9C9", "#666666", "#DCDCDC"]
    }
  }, headlineFontSize = 22, headlineFontWeight = "normal", labelFont$1 = "Benton Gothic, sans-serif", labelFontSize = 11.5, labelFontWeight = "normal", markColor$4 = "#82c6df", titleFont = "Benton Gothic Bold, sans-serif", titleFontWeight = "normal", titleFontSize$1 = 13, colorSchemes$1 = {
    "category-6": ["#ec8431", "#829eb1", "#c89d29", "#3580b1", "#adc839", "#ab7fb4"],
    "fire-7": ["#fbf2c7", "#f9e39c", "#f8d36e", "#f4bb6a", "#e68a4f", "#d15a40", "#ab4232"],
    "fireandice-6": ["#e68a4f", "#f4bb6a", "#f9e39c", "#dadfe2", "#a6b7c6", "#849eae"],
    "ice-7": ["#edefee", "#dadfe2", "#c4ccd2", "#a6b7c6", "#849eae", "#607785", "#47525d"]
  }, latimesTheme = {
    background: "#ffffff",
    title: {
      anchor: "start",
      color: "#000000",
      font: titleFont,
      fontSize: headlineFontSize,
      fontWeight: headlineFontWeight
    },
    arc: {
      fill: markColor$4
    },
    area: {
      fill: markColor$4
    },
    line: {
      stroke: markColor$4,
      strokeWidth: 2
    },
    path: {
      stroke: markColor$4
    },
    rect: {
      fill: markColor$4
    },
    shape: {
      stroke: markColor$4
    },
    symbol: {
      fill: markColor$4,
      size: 30
    },
    axis: {
      labelFont: labelFont$1,
      labelFontSize,
      labelFontWeight,
      titleFont,
      titleFontSize: titleFontSize$1,
      titleFontWeight
    },
    axisX: {
      labelAngle: 0,
      labelPadding: 4,
      tickSize: 3
    },
    axisY: {
      labelBaseline: "middle",
      maxExtent: 45,
      minExtent: 45,
      tickSize: 2,
      titleAlign: "left",
      titleAngle: 0,
      titleX: -45,
      titleY: -11
    },
    legend: {
      labelFont: labelFont$1,
      labelFontSize,
      symbolType: "square",
      titleFont,
      titleFontSize: titleFontSize$1,
      titleFontWeight
    },
    range: {
      category: colorSchemes$1["category-6"],
      diverging: colorSchemes$1["fireandice-6"],
      heatmap: colorSchemes$1["fire-7"],
      ordinal: colorSchemes$1["fire-7"],
      ramp: colorSchemes$1["fire-7"]
    }
  }, markColor$3 = "#ab5787", axisColor$1 = "#979797", quartzTheme = {
    background: "#f9f9f9",
    arc: {
      fill: markColor$3
    },
    area: {
      fill: markColor$3
    },
    line: {
      stroke: markColor$3
    },
    path: {
      stroke: markColor$3
    },
    rect: {
      fill: markColor$3
    },
    shape: {
      stroke: markColor$3
    },
    symbol: {
      fill: markColor$3,
      size: 30
    },
    axis: {
      domainColor: axisColor$1,
      domainWidth: 0.5,
      gridWidth: 0.2,
      labelColor: axisColor$1,
      tickColor: axisColor$1,
      tickWidth: 0.2,
      titleColor: axisColor$1
    },
    axisBand: {
      grid: !1
    },
    axisX: {
      grid: !0,
      tickSize: 10
    },
    axisY: {
      domain: !1,
      grid: !0,
      tickSize: 0
    },
    legend: {
      labelFontSize: 11,
      padding: 1,
      symbolSize: 30,
      symbolType: "square"
    },
    range: {
      category: ["#ab5787", "#51b2e5", "#703c5c", "#168dd9", "#d190b6", "#00609f", "#d365ba", "#154866", "#666666", "#c4c4c4"]
    }
  }, markColor$2 = "#3e5c69", voxTheme = {
    background: "#fff",
    arc: {
      fill: markColor$2
    },
    area: {
      fill: markColor$2
    },
    line: {
      stroke: markColor$2
    },
    path: {
      stroke: markColor$2
    },
    rect: {
      fill: markColor$2
    },
    shape: {
      stroke: markColor$2
    },
    symbol: {
      fill: markColor$2
    },
    axis: {
      domainWidth: 0.5,
      grid: !0,
      labelPadding: 2,
      tickSize: 5,
      tickWidth: 0.5,
      titleFontWeight: "normal"
    },
    axisBand: {
      grid: !1
    },
    axisX: {
      gridWidth: 0.2
    },
    axisY: {
      gridDash: [3],
      gridWidth: 0.4
    },
    legend: {
      labelFontSize: 11,
      padding: 1,
      symbolType: "square"
    },
    range: {
      category: ["#3e5c69", "#6793a6", "#182429", "#0570b0", "#3690c0", "#74a9cf", "#a6bddb", "#e2ddf2"]
    }
  }, markColor$1 = "#1696d2", axisColor = "#000000", backgroundColor$1 = "#FFFFFF", font = "Lato", labelFont = "Lato", sourceFont = "Lato", gridColor$1 = "#DEDDDD", titleFontSize = 18, colorSchemes = {
    "main-colors": ["#1696d2", "#d2d2d2", "#000000", "#fdbf11", "#ec008b", "#55b748", "#5c5859", "#db2b27"],
    "shades-blue": ["#CFE8F3", "#A2D4EC", "#73BFE2", "#46ABDB", "#1696D2", "#12719E", "#0A4C6A", "#062635"],
    "shades-gray": ["#F5F5F5", "#ECECEC", "#E3E3E3", "#DCDBDB", "#D2D2D2", "#9D9D9D", "#696969", "#353535"],
    "shades-yellow": ["#FFF2CF", "#FCE39E", "#FDD870", "#FCCB41", "#FDBF11", "#E88E2D", "#CA5800", "#843215"],
    "shades-magenta": ["#F5CBDF", "#EB99C2", "#E46AA7", "#E54096", "#EC008B", "#AF1F6B", "#761548", "#351123"],
    "shades-green": ["#DCEDD9", "#BCDEB4", "#98CF90", "#78C26D", "#55B748", "#408941", "#2C5C2D", "#1A2E19"],
    "shades-black": ["#D5D5D4", "#ADABAC", "#848081", "#5C5859", "#332D2F", "#262223", "#1A1717", "#0E0C0D"],
    "shades-red": ["#F8D5D4", "#F1AAA9", "#E9807D", "#E25552", "#DB2B27", "#A4201D", "#6E1614", "#370B0A"],
    "one-group": ["#1696d2", "#000000"],
    "two-groups-cat-1": ["#1696d2", "#000000"],
    "two-groups-cat-2": ["#1696d2", "#fdbf11"],
    "two-groups-cat-3": ["#1696d2", "#db2b27"],
    "two-groups-seq": ["#a2d4ec", "#1696d2"],
    "three-groups-cat": ["#1696d2", "#fdbf11", "#000000"],
    "three-groups-seq": ["#a2d4ec", "#1696d2", "#0a4c6a"],
    "four-groups-cat-1": ["#000000", "#d2d2d2", "#fdbf11", "#1696d2"],
    "four-groups-cat-2": ["#1696d2", "#ec0008b", "#fdbf11", "#5c5859"],
    "four-groups-seq": ["#cfe8f3", "#73bf42", "#1696d2", "#0a4c6a"],
    "five-groups-cat-1": ["#1696d2", "#fdbf11", "#d2d2d2", "#ec008b", "#000000"],
    "five-groups-cat-2": ["#1696d2", "#0a4c6a", "#d2d2d2", "#fdbf11", "#332d2f"],
    "five-groups-seq": ["#cfe8f3", "#73bf42", "#1696d2", "#0a4c6a", "#000000"],
    "six-groups-cat-1": ["#1696d2", "#ec008b", "#fdbf11", "#000000", "#d2d2d2", "#55b748"],
    "six-groups-cat-2": ["#1696d2", "#d2d2d2", "#ec008b", "#fdbf11", "#332d2f", "#0a4c6a"],
    "six-groups-seq": ["#cfe8f3", "#a2d4ec", "#73bfe2", "#46abdb", "#1696d2", "#12719e"],
    "diverging-colors": ["#ca5800", "#fdbf11", "#fdd870", "#fff2cf", "#cfe8f3", "#73bfe2", "#1696d2", "#0a4c6a"]
  }, urbanInstituteTheme = {
    background: backgroundColor$1,
    title: {
      anchor: "start",
      fontSize: titleFontSize,
      font
    },
    axisX: {
      domain: !0,
      domainColor: axisColor,
      domainWidth: 1,
      grid: !1,
      labelFontSize: 12,
      labelFont,
      labelAngle: 0,
      tickColor: axisColor,
      tickSize: 5,
      titleFontSize: 12,
      titlePadding: 10,
      titleFont: font
    },
    axisY: {
      domain: !1,
      domainWidth: 1,
      grid: !0,
      gridColor: gridColor$1,
      gridWidth: 1,
      labelFontSize: 12,
      labelFont,
      labelPadding: 8,
      ticks: !1,
      titleFontSize: 12,
      titlePadding: 10,
      titleFont: font,
      titleAngle: 0,
      titleY: -10,
      titleX: 18
    },
    legend: {
      labelFontSize: 12,
      labelFont,
      symbolSize: 100,
      titleFontSize: 12,
      titlePadding: 10,
      titleFont: font,
      orient: "right",
      offset: 10
    },
    view: {
      stroke: "transparent"
    },
    range: {
      category: colorSchemes["six-groups-cat-1"],
      diverging: colorSchemes["diverging-colors"],
      heatmap: colorSchemes["diverging-colors"],
      ordinal: colorSchemes["six-groups-seq"],
      ramp: colorSchemes["shades-blue"]
    },
    area: {
      fill: markColor$1
    },
    rect: {
      fill: markColor$1
    },
    line: {
      color: markColor$1,
      stroke: markColor$1,
      strokeWidth: 5
    },
    trail: {
      color: markColor$1,
      stroke: markColor$1,
      strokeWidth: 0,
      size: 1
    },
    path: {
      stroke: markColor$1,
      strokeWidth: 0.5
    },
    point: {
      filled: !0
    },
    text: {
      font: sourceFont,
      color: markColor$1,
      fontSize: 11,
      align: "center",
      fontWeight: 400,
      size: 11
    },
    style: {
      bar: {
        fill: markColor$1,
        stroke: null
      }
    },
    arc: {
      fill: markColor$1
    },
    shape: {
      stroke: markColor$1
    },
    symbol: {
      fill: markColor$1,
      size: 30
    }
  }, markColor = "#3366CC", gridColor = "#ccc", defaultFont$1 = "Arial, sans-serif", googlechartsTheme = {
    arc: {
      fill: markColor
    },
    area: {
      fill: markColor
    },
    path: {
      stroke: markColor
    },
    rect: {
      fill: markColor
    },
    shape: {
      stroke: markColor
    },
    symbol: {
      stroke: markColor
    },
    circle: {
      fill: markColor
    },
    background: "#fff",
    padding: {
      top: 10,
      right: 10,
      bottom: 10,
      left: 10
    },
    style: {
      "guide-label": {
        font: defaultFont$1,
        fontSize: 12
      },
      "guide-title": {
        font: defaultFont$1,
        fontSize: 12
      },
      "group-title": {
        font: defaultFont$1,
        fontSize: 12
      }
    },
    title: {
      font: defaultFont$1,
      fontSize: 14,
      fontWeight: "bold",
      dy: -3,
      anchor: "start"
    },
    axis: {
      gridColor,
      tickColor: gridColor,
      domain: !1,
      grid: !0
    },
    range: {
      category: ["#4285F4", "#DB4437", "#F4B400", "#0F9D58", "#AB47BC", "#00ACC1", "#FF7043", "#9E9D24", "#5C6BC0", "#F06292", "#00796B", "#C2185B"],
      heatmap: ["#c6dafc", "#5e97f6", "#2a56c6"]
    }
  }, ptToPx = (O) => O * (1 / 3 + 1), fontSmallPx = ptToPx(9), legendFontPx = ptToPx(10), fontLargePx = ptToPx(12), fontStandard = "Segoe UI", fontTitle = "wf_standard-font, helvetica, arial, sans-serif", firstLevelElementColor = "#252423", secondLevelElementColor = "#605E5C", backgroundColor = "transparent", backgroundSecondaryColor = "#C8C6C4", paletteColor1 = "#118DFF", paletteColor2 = "#12239E", paletteColor3 = "#E66C37", paletteColor4 = "#6B007B", paletteColor5 = "#E044A7", paletteColor6 = "#744EC2", paletteColor7 = "#D9B300", paletteColor8 = "#D64550", divergentColorMax = paletteColor1, divergentColorMin = "#DEEFFF", divergentPalette = [divergentColorMin, divergentColorMax], ordinalPalette = [divergentColorMin, "#c7e4ff", "#b0d9ff", "#9aceff", "#83c3ff", "#6cb9ff", "#55aeff", "#3fa3ff", "#2898ff", divergentColorMax], powerbiTheme = {
    view: {
      stroke: backgroundColor
    },
    background: backgroundColor,
    font: fontStandard,
    header: {
      titleFont: fontTitle,
      titleFontSize: fontLargePx,
      titleColor: firstLevelElementColor,
      labelFont: fontStandard,
      labelFontSize: legendFontPx,
      labelColor: secondLevelElementColor
    },
    axis: {
      ticks: !1,
      grid: !1,
      domain: !1,
      labelColor: secondLevelElementColor,
      labelFontSize: fontSmallPx,
      titleFont: fontTitle,
      titleColor: firstLevelElementColor,
      titleFontSize: fontLargePx,
      titleFontWeight: "normal"
    },
    axisQuantitative: {
      tickCount: 3,
      grid: !0,
      gridColor: backgroundSecondaryColor,
      gridDash: [1, 5],
      labelFlush: !1
    },
    axisBand: {
      tickExtra: !0
    },
    axisX: {
      labelPadding: 5
    },
    axisY: {
      labelPadding: 10
    },
    bar: {
      fill: paletteColor1
    },
    line: {
      stroke: paletteColor1,
      strokeWidth: 3,
      strokeCap: "round",
      strokeJoin: "round"
    },
    text: {
      font: fontStandard,
      fontSize: fontSmallPx,
      fill: secondLevelElementColor
    },
    arc: {
      fill: paletteColor1
    },
    area: {
      fill: paletteColor1,
      line: !0,
      opacity: 0.6
    },
    path: {
      stroke: paletteColor1
    },
    rect: {
      fill: paletteColor1
    },
    point: {
      fill: paletteColor1,
      filled: !0,
      size: 75
    },
    shape: {
      stroke: paletteColor1
    },
    symbol: {
      fill: paletteColor1,
      strokeWidth: 1.5,
      size: 50
    },
    legend: {
      titleFont: fontStandard,
      titleFontWeight: "bold",
      titleColor: secondLevelElementColor,
      labelFont: fontStandard,
      labelFontSize: legendFontPx,
      labelColor: secondLevelElementColor,
      symbolType: "circle",
      symbolSize: 75
    },
    range: {
      category: [paletteColor1, paletteColor2, paletteColor3, paletteColor4, paletteColor5, paletteColor6, paletteColor7, paletteColor8],
      diverging: divergentPalette,
      heatmap: divergentPalette,
      ordinal: ordinalPalette
    }
  }, defaultFont = 'IBM Plex Sans,system-ui,-apple-system,BlinkMacSystemFont,".sfnstext-regular",sans-serif', condensedFont = 'IBM Plex Sans Condensed, system-ui, -apple-system, BlinkMacSystemFont, ".SFNSText-Regular", sans-serif', fontWeight = 400, TOKENS = {
    textPrimary: {
      g90: "#f4f4f4",
      g100: "#f4f4f4",
      white: "#161616",
      g10: "#161616"
    },
    textSecondary: {
      g90: "#c6c6c6",
      g100: "#c6c6c6",
      white: "#525252",
      g10: "#525252"
    },
    // layer
    layerAccent01: {
      white: "#e0e0e0",
      g10: "#e0e0e0",
      g90: "#525252",
      g100: "#393939"
    },
    // grid
    gridBg: {
      white: "#ffffff",
      g10: "#ffffff",
      g90: "#161616",
      g100: "#161616"
    }
  }, darkCategories = ["#8a3ffc", "#33b1ff", "#007d79", "#ff7eb6", "#fa4d56", "#fff1f1", "#6fdc8c", "#4589ff", "#d12771", "#d2a106", "#08bdba", "#bae6ff", "#ba4e00", "#d4bbff"], lightCategories = ["#6929c4", "#1192e8", "#005d5d", "#9f1853", "#fa4d56", "#570408", "#198038", "#002d9c", "#ee538b", "#b28600", "#009d9a", "#012749", "#8a3800", "#a56eff"];
  function genCarbonConfig({
    theme: O,
    background: _
  }) {
    const H = ["white", "g10"].includes(O) ? "light" : "dark", V = TOKENS.gridBg[O], q = TOKENS.textPrimary[O], G = TOKENS.textSecondary[O], J = H === "dark" ? darkCategories : lightCategories, Z = H === "dark" ? "#d4bbff" : "#6929c4";
    return {
      background: _,
      arc: {
        fill: Z
      },
      area: {
        fill: Z
      },
      path: {
        stroke: Z
      },
      rect: {
        fill: Z
      },
      shape: {
        stroke: Z
      },
      symbol: {
        stroke: Z
      },
      circle: {
        fill: Z
      },
      view: {
        fill: V,
        stroke: V
      },
      group: {
        fill: V
      },
      title: {
        color: q,
        anchor: "start",
        dy: -15,
        fontSize: 16,
        font: defaultFont,
        fontWeight: 600
      },
      axis: {
        // Axis labels
        labelColor: G,
        labelFontSize: 12,
        labelFont: condensedFont,
        labelFontWeight: fontWeight,
        // Axis titles
        titleColor: q,
        titleFontWeight: 600,
        titleFontSize: 12,
        // MISC
        grid: !0,
        gridColor: TOKENS.layerAccent01[O],
        labelAngle: 0
      },
      axisX: {
        titlePadding: 10
      },
      axisY: {
        titlePadding: 2.5
      },
      style: {
        "guide-label": {
          font: defaultFont,
          fill: G,
          fontWeight
        },
        "guide-title": {
          font: defaultFont,
          fill: G,
          fontWeight
        }
      },
      range: {
        category: J,
        diverging: ["#750e13", "#a2191f", "#da1e28", "#fa4d56", "#ff8389", "#ffb3b8", "#ffd7d9", "#fff1f1", "#e5f6ff", "#bae6ff", "#82cfff", "#33b1ff", "#1192e8", "#0072c3", "#00539a", "#003a6d"],
        heatmap: ["#f6f2ff", "#e8daff", "#d4bbff", "#be95ff", "#a56eff", "#8a3ffc", "#6929c4", "#491d8b", "#31135e", "#1c0f30"]
      }
    };
  }
  const carbonwhite = genCarbonConfig({
    theme: "white",
    background: "#ffffff"
  }), carbong10 = genCarbonConfig({
    theme: "g10",
    background: "#f4f4f4"
  }), carbong90 = genCarbonConfig({
    theme: "g90",
    background: "#262626"
  }), carbong100 = genCarbonConfig({
    theme: "g100",
    background: "#161616"
  }), version = pkg.version, themes = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    carbong10,
    carbong100,
    carbong90,
    carbonwhite,
    dark: darkTheme$1,
    excel: excelTheme,
    fivethirtyeight: fiveThirtyEightTheme,
    ggplot2: ggplot2Theme,
    googlecharts: googlechartsTheme,
    latimes: latimesTheme,
    powerbi: powerbiTheme,
    quartz: quartzTheme,
    urbaninstitute: urbanInstituteTheme,
    version,
    vox: voxTheme
  }, Symbol.toStringTag, { value: "Module" }));
  function formatValue(O, _, H, V) {
    if (isArray$2(O))
      return `[${O.map((q) => _(isString$1(q) ? q : stringify(q, H))).join(", ")}]`;
    if (isObject$2(O)) {
      let q = "";
      const { title: G, image: J, ...Z } = O;
      G && (q += `<h2>${_(G)}</h2>`), J && (q += `<img src="${new URL(_(J), V || location.href).href}">`);
      const te = Object.keys(Z);
      if (te.length > 0) {
        q += "<table>";
        for (const ne of te) {
          let ee = Z[ne];
          ee !== void 0 && (isObject$2(ee) && (ee = stringify(ee, H)), q += `<tr><td class="key">${_(ne)}</td><td class="value">${_(ee)}</td></tr>`);
        }
        q += "</table>";
      }
      return q || "{}";
    }
    return _(O);
  }
  function replacer(O) {
    const _ = [];
    return function(H, V) {
      if (typeof V != "object" || V === null)
        return V;
      const q = _.indexOf(this) + 1;
      return _.length = q, _.length > O ? "[Object]" : _.indexOf(V) >= 0 ? "[Circular]" : (_.push(V), V);
    };
  }
  function stringify(O, _) {
    return JSON.stringify(O, replacer(_));
  }
  var defaultStyle = `#vg-tooltip-element {
  visibility: hidden;
  padding: 8px;
  position: fixed;
  z-index: 1000;
  font-family: sans-serif;
  font-size: 11px;
  border-radius: 3px;
  box-shadow: 2px 2px 4px rgba(0, 0, 0, 0.1);
  /* The default theme is the light theme. */
  background-color: rgba(255, 255, 255, 0.95);
  border: 1px solid #d9d9d9;
  color: black;
}
#vg-tooltip-element.visible {
  visibility: visible;
}
#vg-tooltip-element h2 {
  margin-top: 0;
  margin-bottom: 10px;
  font-size: 13px;
}
#vg-tooltip-element table {
  border-spacing: 0;
}
#vg-tooltip-element table tr {
  border: none;
}
#vg-tooltip-element table tr td {
  overflow: hidden;
  text-overflow: ellipsis;
  padding-top: 2px;
  padding-bottom: 2px;
}
#vg-tooltip-element table tr td.key {
  color: #808080;
  max-width: 150px;
  text-align: right;
  padding-right: 4px;
}
#vg-tooltip-element table tr td.value {
  display: block;
  max-width: 300px;
  max-height: 7em;
  text-align: left;
}
#vg-tooltip-element.dark-theme {
  background-color: rgba(32, 32, 32, 0.9);
  border: 1px solid #f5f5f5;
  color: white;
}
#vg-tooltip-element.dark-theme td.key {
  color: #bfbfbf;
}
`;
  const EL_ID = "vg-tooltip-element", DEFAULT_OPTIONS = {
    offsetX: 10,
    offsetY: 10,
    id: EL_ID,
    styleId: "vega-tooltip-style",
    theme: "light",
    disableDefaultStyle: !1,
    sanitize: escapeHTML,
    maxDepth: 2,
    formatTooltip: formatValue,
    baseURL: "",
    anchor: "cursor",
    position: ["top", "bottom", "left", "right", "top-left", "top-right", "bottom-left", "bottom-right"]
  };
  function escapeHTML(O) {
    return String(O).replace(/&/g, "&amp;").replace(/</g, "&lt;");
  }
  function createDefaultStyle(O) {
    if (!/^[A-Za-z]+[-:.\w]*$/.test(O))
      throw new Error("Invalid HTML ID");
    return defaultStyle.toString().replace(EL_ID, O);
  }
  function calculatePositionRelativeToCursor(O, _, { offsetX: H, offsetY: V }) {
    const q = getPositions({ x1: O.clientX, x2: O.clientX, y1: O.clientY, y2: O.clientY }, _, H, V), G = ["bottom-right", "bottom-left", "top-right", "top-left"];
    for (const J of G)
      if (tooltipIsInViewport(q[J], _))
        return q[J];
    return q["top-left"];
  }
  function calculatePositionRelativeToMark(O, _, H, V, q) {
    const { position: G, offsetX: J, offsetY: Z } = q, te = O._el.getBoundingClientRect(), ne = O._origin, ee = getMarkBounds(te, ne, H), ae = getPositions(ee, V, J, Z), oe = Array.isArray(G) ? G : [G];
    for (const se of oe)
      if (tooltipIsInViewport(ae[se], V) && !mouseIsOnTooltip(_, ae[se], V))
        return ae[se];
    return calculatePositionRelativeToCursor(_, V, q);
  }
  function getMarkBounds(O, _, H) {
    const V = H.isVoronoi ? H.datum.bounds : H.bounds;
    let q = O.left + _[0] + V.x1, G = O.top + _[1] + V.y1, J = H;
    for (; J.mark.group; )
      J = J.mark.group, q += J.x ?? 0, G += J.y ?? 0;
    const Z = V.x2 - V.x1, te = V.y2 - V.y1;
    return {
      x1: q,
      x2: q + Z,
      y1: G,
      y2: G + te
    };
  }
  function getPositions(O, _, H, V) {
    const q = (O.x1 + O.x2) / 2, G = (O.y1 + O.y2) / 2, J = O.x1 - _.width - H, Z = q - _.width / 2, te = O.x2 + H, ne = O.y1 - _.height - V, ee = G - _.height / 2, ae = O.y2 + V;
    return {
      top: { x: Z, y: ne },
      bottom: { x: Z, y: ae },
      left: { x: J, y: ee },
      right: { x: te, y: ee },
      "top-left": { x: J, y: ne },
      "top-right": { x: te, y: ne },
      "bottom-left": { x: J, y: ae },
      "bottom-right": { x: te, y: ae }
    };
  }
  function tooltipIsInViewport(O, _) {
    return O.x >= 0 && O.y >= 0 && O.x + _.width <= window.innerWidth && O.y + _.height <= window.innerHeight;
  }
  function mouseIsOnTooltip(O, _, H) {
    return O.clientX >= _.x && O.clientX <= _.x + H.width && O.clientY >= _.y && O.clientY <= _.y + H.height;
  }
  class Handler {
    /**
     * Create the tooltip handler and initialize the element and style.
     *
     * @param options Tooltip Options
     */
    constructor(_) {
      this.options = { ...DEFAULT_OPTIONS, ..._ };
      const H = this.options.id;
      if (this.el = null, this.call = this.tooltipHandler.bind(this), !this.options.disableDefaultStyle && !document.getElementById(this.options.styleId)) {
        const V = document.createElement("style");
        V.setAttribute("id", this.options.styleId), V.innerHTML = createDefaultStyle(H);
        const q = document.head;
        q.childNodes.length > 0 ? q.insertBefore(V, q.childNodes[0]) : q.appendChild(V);
      }
    }
    /**
     * The tooltip handler function.
     */
    tooltipHandler(_, H, V, q) {
      if (this.el = document.getElementById(this.options.id), this.el || (this.el = document.createElement("div"), this.el.setAttribute("id", this.options.id), this.el.classList.add("vg-tooltip"), (document.fullscreenElement ?? document.body).appendChild(this.el)), q == null || q === "") {
        this.el.classList.remove("visible", `${this.options.theme}-theme`);
        return;
      }
      this.el.innerHTML = this.options.formatTooltip(q, this.options.sanitize, this.options.maxDepth, this.options.baseURL), this.el.classList.add("visible", `${this.options.theme}-theme`);
      const { x: G, y: J } = this.options.anchor === "mark" ? calculatePositionRelativeToMark(_, H, V, this.el.getBoundingClientRect(), this.options) : calculatePositionRelativeToCursor(H, this.el.getBoundingClientRect(), this.options);
      this.el.style.top = `${J}px`, this.el.style.left = `${G}px`;
    }
  }
  /*!
   * https://github.com/Starcounter-Jack/JSON-Patch
   * (c) 2017-2022 Joachim Wester
   * MIT licensed
   */
  var __extends = /* @__PURE__ */ function() {
    var O = function(_, H) {
      return O = Object.setPrototypeOf || { __proto__: [] } instanceof Array && function(V, q) {
        V.__proto__ = q;
      } || function(V, q) {
        for (var G in q) q.hasOwnProperty(G) && (V[G] = q[G]);
      }, O(_, H);
    };
    return function(_, H) {
      O(_, H);
      function V() {
        this.constructor = _;
      }
      _.prototype = H === null ? Object.create(H) : (V.prototype = H.prototype, new V());
    };
  }(), _hasOwnProperty = Object.prototype.hasOwnProperty;
  function hasOwnProperty$a(O, _) {
    return _hasOwnProperty.call(O, _);
  }
  function _objectKeys(O) {
    if (Array.isArray(O)) {
      for (var _ = new Array(O.length), H = 0; H < _.length; H++)
        _[H] = "" + H;
      return _;
    }
    if (Object.keys)
      return Object.keys(O);
    var V = [];
    for (var q in O)
      hasOwnProperty$a(O, q) && V.push(q);
    return V;
  }
  function _deepClone(O) {
    switch (typeof O) {
      case "object":
        return JSON.parse(JSON.stringify(O));
      //Faster than ES5 clone - http://jsperf.com/deep-cloning-of-objects/5
      case "undefined":
        return null;
      //this is how JSON.stringify behaves for array items
      default:
        return O;
    }
  }
  function isInteger(O) {
    for (var _ = 0, H = O.length, V; _ < H; ) {
      if (V = O.charCodeAt(_), V >= 48 && V <= 57) {
        _++;
        continue;
      }
      return !1;
    }
    return !0;
  }
  function escapePathComponent(O) {
    return O.indexOf("/") === -1 && O.indexOf("~") === -1 ? O : O.replace(/~/g, "~0").replace(/\//g, "~1");
  }
  function unescapePathComponent(O) {
    return O.replace(/~1/g, "/").replace(/~0/g, "~");
  }
  function hasUndefined(O) {
    if (O === void 0)
      return !0;
    if (O) {
      if (Array.isArray(O)) {
        for (var _ = 0, H = O.length; _ < H; _++)
          if (hasUndefined(O[_]))
            return !0;
      } else if (typeof O == "object") {
        for (var V = _objectKeys(O), q = V.length, G = 0; G < q; G++)
          if (hasUndefined(O[V[G]]))
            return !0;
      }
    }
    return !1;
  }
  function patchErrorMessageFormatter(O, _) {
    var H = [O];
    for (var V in _) {
      var q = typeof _[V] == "object" ? JSON.stringify(_[V], null, 2) : _[V];
      typeof q < "u" && H.push(V + ": " + q);
    }
    return H.join(`
`);
  }
  var PatchError = (
    /** @class */
    function(O) {
      __extends(_, O);
      function _(H, V, q, G, J) {
        var Z = this.constructor, te = O.call(this, patchErrorMessageFormatter(H, { name: V, index: q, operation: G, tree: J })) || this;
        return te.name = V, te.index = q, te.operation = G, te.tree = J, Object.setPrototypeOf(te, Z.prototype), te.message = patchErrorMessageFormatter(H, { name: V, index: q, operation: G, tree: J }), te;
      }
      return _;
    }(Error)
  ), JsonPatchError = PatchError, deepClone = _deepClone, objOps = {
    add: function(O, _, H) {
      return O[_] = this.value, { newDocument: H };
    },
    remove: function(O, _, H) {
      var V = O[_];
      return delete O[_], { newDocument: H, removed: V };
    },
    replace: function(O, _, H) {
      var V = O[_];
      return O[_] = this.value, { newDocument: H, removed: V };
    },
    move: function(O, _, H) {
      var V = getValueByPointer(H, this.path);
      V && (V = _deepClone(V));
      var q = applyOperation(H, { op: "remove", path: this.from }).removed;
      return applyOperation(H, { op: "add", path: this.path, value: q }), { newDocument: H, removed: V };
    },
    copy: function(O, _, H) {
      var V = getValueByPointer(H, this.from);
      return applyOperation(H, { op: "add", path: this.path, value: _deepClone(V) }), { newDocument: H };
    },
    test: function(O, _, H) {
      return { newDocument: H, test: _areEquals(O[_], this.value) };
    },
    _get: function(O, _, H) {
      return this.value = O[_], { newDocument: H };
    }
  }, arrOps = {
    add: function(O, _, H) {
      return isInteger(_) ? O.splice(_, 0, this.value) : O[_] = this.value, { newDocument: H, index: _ };
    },
    remove: function(O, _, H) {
      var V = O.splice(_, 1);
      return { newDocument: H, removed: V[0] };
    },
    replace: function(O, _, H) {
      var V = O[_];
      return O[_] = this.value, { newDocument: H, removed: V };
    },
    move: objOps.move,
    copy: objOps.copy,
    test: objOps.test,
    _get: objOps._get
  };
  function getValueByPointer(O, _) {
    if (_ == "")
      return O;
    var H = { op: "_get", path: _ };
    return applyOperation(O, H), H.value;
  }
  function applyOperation(O, _, H, V, q, G) {
    if (H === void 0 && (H = !1), V === void 0 && (V = !0), q === void 0 && (q = !0), G === void 0 && (G = 0), H && (typeof H == "function" ? H(_, 0, O, _.path) : validator(_, 0)), _.path === "") {
      var J = { newDocument: O };
      if (_.op === "add")
        return J.newDocument = _.value, J;
      if (_.op === "replace")
        return J.newDocument = _.value, J.removed = O, J;
      if (_.op === "move" || _.op === "copy")
        return J.newDocument = getValueByPointer(O, _.from), _.op === "move" && (J.removed = O), J;
      if (_.op === "test") {
        if (J.test = _areEquals(O, _.value), J.test === !1)
          throw new JsonPatchError("Test operation failed", "TEST_OPERATION_FAILED", G, _, O);
        return J.newDocument = O, J;
      } else {
        if (_.op === "remove")
          return J.removed = O, J.newDocument = null, J;
        if (_.op === "_get")
          return _.value = O, J;
        if (H)
          throw new JsonPatchError("Operation `op` property is not one of operations defined in RFC-6902", "OPERATION_OP_INVALID", G, _, O);
        return J;
      }
    } else {
      V || (O = _deepClone(O));
      var Z = _.path || "", te = Z.split("/"), ne = O, ee = 1, ae = te.length, oe = void 0, se = void 0, ce = void 0;
      for (typeof H == "function" ? ce = H : ce = validator; ; ) {
        if (se = te[ee], se && se.indexOf("~") != -1 && (se = unescapePathComponent(se)), q && (se == "__proto__" || se == "prototype" && ee > 0 && te[ee - 1] == "constructor"))
          throw new TypeError("JSON-Patch: modifying `__proto__` or `constructor/prototype` prop is banned for security reasons, if this was on purpose, please set `banPrototypeModifications` flag false and pass it to this function. More info in fast-json-patch README");
        if (H && oe === void 0 && (ne[se] === void 0 ? oe = te.slice(0, ee).join("/") : ee == ae - 1 && (oe = _.path), oe !== void 0 && ce(_, 0, O, oe)), ee++, Array.isArray(ne)) {
          if (se === "-")
            se = ne.length;
          else {
            if (H && !isInteger(se))
              throw new JsonPatchError("Expected an unsigned base-10 integer value, making the new referenced value the array element with the zero-based index", "OPERATION_PATH_ILLEGAL_ARRAY_INDEX", G, _, O);
            isInteger(se) && (se = ~~se);
          }
          if (ee >= ae) {
            if (H && _.op === "add" && se > ne.length)
              throw new JsonPatchError("The specified index MUST NOT be greater than the number of elements in the array", "OPERATION_VALUE_OUT_OF_BOUNDS", G, _, O);
            var J = arrOps[_.op].call(_, ne, se, O);
            if (J.test === !1)
              throw new JsonPatchError("Test operation failed", "TEST_OPERATION_FAILED", G, _, O);
            return J;
          }
        } else if (ee >= ae) {
          var J = objOps[_.op].call(_, ne, se, O);
          if (J.test === !1)
            throw new JsonPatchError("Test operation failed", "TEST_OPERATION_FAILED", G, _, O);
          return J;
        }
        if (ne = ne[se], H && ee < ae && (!ne || typeof ne != "object"))
          throw new JsonPatchError("Cannot perform operation at the desired path", "OPERATION_PATH_UNRESOLVABLE", G, _, O);
      }
    }
  }
  function applyPatch(O, _, H, V, q) {
    if (V === void 0 && (V = !0), q === void 0 && (q = !0), H && !Array.isArray(_))
      throw new JsonPatchError("Patch sequence must be an array", "SEQUENCE_NOT_AN_ARRAY");
    V || (O = _deepClone(O));
    for (var G = new Array(_.length), J = 0, Z = _.length; J < Z; J++)
      G[J] = applyOperation(O, _[J], H, !0, q, J), O = G[J].newDocument;
    return G.newDocument = O, G;
  }
  function applyReducer(O, _, H) {
    var V = applyOperation(O, _);
    if (V.test === !1)
      throw new JsonPatchError("Test operation failed", "TEST_OPERATION_FAILED", H, _, O);
    return V.newDocument;
  }
  function validator(O, _, H, V) {
    if (typeof O != "object" || O === null || Array.isArray(O))
      throw new JsonPatchError("Operation is not an object", "OPERATION_NOT_AN_OBJECT", _, O, H);
    if (objOps[O.op]) {
      if (typeof O.path != "string")
        throw new JsonPatchError("Operation `path` property is not a string", "OPERATION_PATH_INVALID", _, O, H);
      if (O.path.indexOf("/") !== 0 && O.path.length > 0)
        throw new JsonPatchError('Operation `path` property must start with "/"', "OPERATION_PATH_INVALID", _, O, H);
      if ((O.op === "move" || O.op === "copy") && typeof O.from != "string")
        throw new JsonPatchError("Operation `from` property is not present (applicable in `move` and `copy` operations)", "OPERATION_FROM_REQUIRED", _, O, H);
      if ((O.op === "add" || O.op === "replace" || O.op === "test") && O.value === void 0)
        throw new JsonPatchError("Operation `value` property is not present (applicable in `add`, `replace` and `test` operations)", "OPERATION_VALUE_REQUIRED", _, O, H);
      if ((O.op === "add" || O.op === "replace" || O.op === "test") && hasUndefined(O.value))
        throw new JsonPatchError("Operation `value` property is not present (applicable in `add`, `replace` and `test` operations)", "OPERATION_VALUE_CANNOT_CONTAIN_UNDEFINED", _, O, H);
      if (H) {
        if (O.op == "add") {
          var q = O.path.split("/").length, G = V.split("/").length;
          if (q !== G + 1 && q !== G)
            throw new JsonPatchError("Cannot perform an `add` operation at the desired path", "OPERATION_PATH_CANNOT_ADD", _, O, H);
        } else if (O.op === "replace" || O.op === "remove" || O.op === "_get") {
          if (O.path !== V)
            throw new JsonPatchError("Cannot perform the operation at a path that does not exist", "OPERATION_PATH_UNRESOLVABLE", _, O, H);
        } else if (O.op === "move" || O.op === "copy") {
          var J = { op: "_get", path: O.from, value: void 0 }, Z = validate([J], H);
          if (Z && Z.name === "OPERATION_PATH_UNRESOLVABLE")
            throw new JsonPatchError("Cannot perform the operation from a path that does not exist", "OPERATION_FROM_UNRESOLVABLE", _, O, H);
        }
      }
    } else throw new JsonPatchError("Operation `op` property is not one of operations defined in RFC-6902", "OPERATION_OP_INVALID", _, O, H);
  }
  function validate(O, _, H) {
    try {
      if (!Array.isArray(O))
        throw new JsonPatchError("Patch sequence must be an array", "SEQUENCE_NOT_AN_ARRAY");
      if (_)
        applyPatch(_deepClone(_), _deepClone(O), H || !0);
      else {
        H = H || validator;
        for (var V = 0; V < O.length; V++)
          H(O[V], V, _, void 0);
      }
    } catch (q) {
      if (q instanceof JsonPatchError)
        return q;
      throw q;
    }
  }
  function _areEquals(O, _) {
    if (O === _)
      return !0;
    if (O && _ && typeof O == "object" && typeof _ == "object") {
      var H = Array.isArray(O), V = Array.isArray(_), q, G, J;
      if (H && V) {
        if (G = O.length, G != _.length)
          return !1;
        for (q = G; q-- !== 0; )
          if (!_areEquals(O[q], _[q]))
            return !1;
        return !0;
      }
      if (H != V)
        return !1;
      var Z = Object.keys(O);
      if (G = Z.length, G !== Object.keys(_).length)
        return !1;
      for (q = G; q-- !== 0; )
        if (!_.hasOwnProperty(Z[q]))
          return !1;
      for (q = G; q-- !== 0; )
        if (J = Z[q], !_areEquals(O[J], _[J]))
          return !1;
      return !0;
    }
    return O !== O && _ !== _;
  }
  var core = /* @__PURE__ */ Object.freeze({
    __proto__: null,
    JsonPatchError,
    _areEquals,
    applyOperation,
    applyPatch,
    applyReducer,
    deepClone,
    getValueByPointer,
    validate,
    validator
  });
  /*!
   * https://github.com/Starcounter-Jack/JSON-Patch
   * (c) 2017-2021 Joachim Wester
   * MIT license
   */
  var beforeDict = /* @__PURE__ */ new WeakMap(), Mirror = (
    /** @class */
    /* @__PURE__ */ function() {
      function O(_) {
        this.observers = /* @__PURE__ */ new Map(), this.obj = _;
      }
      return O;
    }()
  ), ObserverInfo = (
    /** @class */
    /* @__PURE__ */ function() {
      function O(_, H) {
        this.callback = _, this.observer = H;
      }
      return O;
    }()
  );
  function getMirror(O) {
    return beforeDict.get(O);
  }
  function getObserverFromMirror(O, _) {
    return O.observers.get(_);
  }
  function removeObserverFromMirror(O, _) {
    O.observers.delete(_.callback);
  }
  function unobserve(O, _) {
    _.unobserve();
  }
  function observe(O, _) {
    var H = [], V, q = getMirror(O);
    if (!q)
      q = new Mirror(O), beforeDict.set(O, q);
    else {
      var G = getObserverFromMirror(q, _);
      V = G && G.observer;
    }
    if (V)
      return V;
    if (V = {}, q.value = _deepClone(O), _) {
      V.callback = _, V.next = null;
      var J = function() {
        generate(V);
      }, Z = function() {
        clearTimeout(V.next), V.next = setTimeout(J);
      };
      typeof window < "u" && (window.addEventListener("mouseup", Z), window.addEventListener("keyup", Z), window.addEventListener("mousedown", Z), window.addEventListener("keydown", Z), window.addEventListener("change", Z));
    }
    return V.patches = H, V.object = O, V.unobserve = function() {
      generate(V), clearTimeout(V.next), removeObserverFromMirror(q, V), typeof window < "u" && (window.removeEventListener("mouseup", Z), window.removeEventListener("keyup", Z), window.removeEventListener("mousedown", Z), window.removeEventListener("keydown", Z), window.removeEventListener("change", Z));
    }, q.observers.set(_, new ObserverInfo(_, V)), V;
  }
  function generate(O, _) {
    _ === void 0 && (_ = !1);
    var H = beforeDict.get(O.object);
    _generate(H.value, O.object, O.patches, "", _), O.patches.length && applyPatch(H.value, O.patches);
    var V = O.patches;
    return V.length > 0 && (O.patches = [], O.callback && O.callback(V)), V;
  }
  function _generate(O, _, H, V, q) {
    if (_ !== O) {
      typeof _.toJSON == "function" && (_ = _.toJSON());
      for (var G = _objectKeys(_), J = _objectKeys(O), Z = !1, te = J.length - 1; te >= 0; te--) {
        var ne = J[te], ee = O[ne];
        if (hasOwnProperty$a(_, ne) && !(_[ne] === void 0 && ee !== void 0 && Array.isArray(_) === !1)) {
          var ae = _[ne];
          typeof ee == "object" && ee != null && typeof ae == "object" && ae != null && Array.isArray(ee) === Array.isArray(ae) ? _generate(ee, ae, H, V + "/" + escapePathComponent(ne), q) : ee !== ae && (q && H.push({ op: "test", path: V + "/" + escapePathComponent(ne), value: _deepClone(ee) }), H.push({ op: "replace", path: V + "/" + escapePathComponent(ne), value: _deepClone(ae) }));
        } else Array.isArray(O) === Array.isArray(_) ? (q && H.push({ op: "test", path: V + "/" + escapePathComponent(ne), value: _deepClone(ee) }), H.push({ op: "remove", path: V + "/" + escapePathComponent(ne) }), Z = !0) : (q && H.push({ op: "test", path: V, value: O }), H.push({ op: "replace", path: V, value: _ }));
      }
      if (!(!Z && G.length == J.length))
        for (var te = 0; te < G.length; te++) {
          var ne = G[te];
          !hasOwnProperty$a(O, ne) && _[ne] !== void 0 && H.push({ op: "add", path: V + "/" + escapePathComponent(ne), value: _deepClone(_[ne]) });
        }
    }
  }
  function compare$1(O, _, H) {
    H === void 0 && (H = !1);
    var V = [];
    return _generate(O, _, V, "", H), V;
  }
  var duplex = /* @__PURE__ */ Object.freeze({
    __proto__: null,
    compare: compare$1,
    generate,
    observe,
    unobserve
  });
  Object.assign({}, core, duplex, {
    JsonPatchError: PatchError,
    deepClone: _deepClone,
    escapePathComponent,
    unescapePathComponent
  });
  function getDefaultExportFromCjs(O) {
    return O && O.__esModule && Object.prototype.hasOwnProperty.call(O, "default") ? O.default : O;
  }
  var lrucache, hasRequiredLrucache;
  function requireLrucache() {
    if (hasRequiredLrucache) return lrucache;
    hasRequiredLrucache = 1;
    class O {
      constructor() {
        this.max = 1e3, this.map = /* @__PURE__ */ new Map();
      }
      get(H) {
        const V = this.map.get(H);
        if (V !== void 0)
          return this.map.delete(H), this.map.set(H, V), V;
      }
      delete(H) {
        return this.map.delete(H);
      }
      set(H, V) {
        if (!this.delete(H) && V !== void 0) {
          if (this.map.size >= this.max) {
            const G = this.map.keys().next().value;
            this.delete(G);
          }
          this.map.set(H, V);
        }
        return this;
      }
    }
    return lrucache = O, lrucache;
  }
  var parseOptions_1, hasRequiredParseOptions;
  function requireParseOptions() {
    if (hasRequiredParseOptions) return parseOptions_1;
    hasRequiredParseOptions = 1;
    const O = Object.freeze({ loose: !0 }), _ = Object.freeze({});
    return parseOptions_1 = (V) => V ? typeof V != "object" ? O : V : _, parseOptions_1;
  }
  var re = { exports: {} }, constants, hasRequiredConstants;
  function requireConstants() {
    if (hasRequiredConstants) return constants;
    hasRequiredConstants = 1;
    const O = "2.0.0", _ = 256, H = Number.MAX_SAFE_INTEGER || /* istanbul ignore next */
    9007199254740991, V = 16, q = _ - 6;
    return constants = {
      MAX_LENGTH: _,
      MAX_SAFE_COMPONENT_LENGTH: V,
      MAX_SAFE_BUILD_LENGTH: q,
      MAX_SAFE_INTEGER: H,
      RELEASE_TYPES: [
        "major",
        "premajor",
        "minor",
        "preminor",
        "patch",
        "prepatch",
        "prerelease"
      ],
      SEMVER_SPEC_VERSION: O,
      FLAG_INCLUDE_PRERELEASE: 1,
      FLAG_LOOSE: 2
    }, constants;
  }
  var debug_1, hasRequiredDebug;
  function requireDebug() {
    return hasRequiredDebug || (hasRequiredDebug = 1, debug_1 = typeof process == "object" && process.env && process.env.NODE_DEBUG && /\bsemver\b/i.test(process.env.NODE_DEBUG) ? (..._) => console.error("SEMVER", ..._) : () => {
    }), debug_1;
  }
  var hasRequiredRe;
  function requireRe() {
    return hasRequiredRe || (hasRequiredRe = 1, function(O, _) {
      const {
        MAX_SAFE_COMPONENT_LENGTH: H,
        MAX_SAFE_BUILD_LENGTH: V,
        MAX_LENGTH: q
      } = requireConstants(), G = requireDebug();
      _ = O.exports = {};
      const J = _.re = [], Z = _.safeRe = [], te = _.src = [], ne = _.t = {};
      let ee = 0;
      const ae = "[a-zA-Z0-9-]", oe = [
        ["\\s", 1],
        ["\\d", q],
        [ae, V]
      ], se = (le) => {
        for (const [ue, de] of oe)
          le = le.split(`${ue}*`).join(`${ue}{0,${de}}`).split(`${ue}+`).join(`${ue}{1,${de}}`);
        return le;
      }, ce = (le, ue, de) => {
        const fe = se(ue), he = ee++;
        G(le, he, ue), ne[le] = he, te[he] = ue, J[he] = new RegExp(ue, de ? "g" : void 0), Z[he] = new RegExp(fe, de ? "g" : void 0);
      };
      ce("NUMERICIDENTIFIER", "0|[1-9]\\d*"), ce("NUMERICIDENTIFIERLOOSE", "\\d+"), ce("NONNUMERICIDENTIFIER", `\\d*[a-zA-Z-]${ae}*`), ce("MAINVERSION", `(${te[ne.NUMERICIDENTIFIER]})\\.(${te[ne.NUMERICIDENTIFIER]})\\.(${te[ne.NUMERICIDENTIFIER]})`), ce("MAINVERSIONLOOSE", `(${te[ne.NUMERICIDENTIFIERLOOSE]})\\.(${te[ne.NUMERICIDENTIFIERLOOSE]})\\.(${te[ne.NUMERICIDENTIFIERLOOSE]})`), ce("PRERELEASEIDENTIFIER", `(?:${te[ne.NUMERICIDENTIFIER]}|${te[ne.NONNUMERICIDENTIFIER]})`), ce("PRERELEASEIDENTIFIERLOOSE", `(?:${te[ne.NUMERICIDENTIFIERLOOSE]}|${te[ne.NONNUMERICIDENTIFIER]})`), ce("PRERELEASE", `(?:-(${te[ne.PRERELEASEIDENTIFIER]}(?:\\.${te[ne.PRERELEASEIDENTIFIER]})*))`), ce("PRERELEASELOOSE", `(?:-?(${te[ne.PRERELEASEIDENTIFIERLOOSE]}(?:\\.${te[ne.PRERELEASEIDENTIFIERLOOSE]})*))`), ce("BUILDIDENTIFIER", `${ae}+`), ce("BUILD", `(?:\\+(${te[ne.BUILDIDENTIFIER]}(?:\\.${te[ne.BUILDIDENTIFIER]})*))`), ce("FULLPLAIN", `v?${te[ne.MAINVERSION]}${te[ne.PRERELEASE]}?${te[ne.BUILD]}?`), ce("FULL", `^${te[ne.FULLPLAIN]}$`), ce("LOOSEPLAIN", `[v=\\s]*${te[ne.MAINVERSIONLOOSE]}${te[ne.PRERELEASELOOSE]}?${te[ne.BUILD]}?`), ce("LOOSE", `^${te[ne.LOOSEPLAIN]}$`), ce("GTLT", "((?:<|>)?=?)"), ce("XRANGEIDENTIFIERLOOSE", `${te[ne.NUMERICIDENTIFIERLOOSE]}|x|X|\\*`), ce("XRANGEIDENTIFIER", `${te[ne.NUMERICIDENTIFIER]}|x|X|\\*`), ce("XRANGEPLAIN", `[v=\\s]*(${te[ne.XRANGEIDENTIFIER]})(?:\\.(${te[ne.XRANGEIDENTIFIER]})(?:\\.(${te[ne.XRANGEIDENTIFIER]})(?:${te[ne.PRERELEASE]})?${te[ne.BUILD]}?)?)?`), ce("XRANGEPLAINLOOSE", `[v=\\s]*(${te[ne.XRANGEIDENTIFIERLOOSE]})(?:\\.(${te[ne.XRANGEIDENTIFIERLOOSE]})(?:\\.(${te[ne.XRANGEIDENTIFIERLOOSE]})(?:${te[ne.PRERELEASELOOSE]})?${te[ne.BUILD]}?)?)?`), ce("XRANGE", `^${te[ne.GTLT]}\\s*${te[ne.XRANGEPLAIN]}$`), ce("XRANGELOOSE", `^${te[ne.GTLT]}\\s*${te[ne.XRANGEPLAINLOOSE]}$`), ce("COERCEPLAIN", `(^|[^\\d])(\\d{1,${H}})(?:\\.(\\d{1,${H}}))?(?:\\.(\\d{1,${H}}))?`), ce("COERCE", `${te[ne.COERCEPLAIN]}(?:$|[^\\d])`), ce("COERCEFULL", te[ne.COERCEPLAIN] + `(?:${te[ne.PRERELEASE]})?(?:${te[ne.BUILD]})?(?:$|[^\\d])`), ce("COERCERTL", te[ne.COERCE], !0), ce("COERCERTLFULL", te[ne.COERCEFULL], !0), ce("LONETILDE", "(?:~>?)"), ce("TILDETRIM", `(\\s*)${te[ne.LONETILDE]}\\s+`, !0), _.tildeTrimReplace = "$1~", ce("TILDE", `^${te[ne.LONETILDE]}${te[ne.XRANGEPLAIN]}$`), ce("TILDELOOSE", `^${te[ne.LONETILDE]}${te[ne.XRANGEPLAINLOOSE]}$`), ce("LONECARET", "(?:\\^)"), ce("CARETTRIM", `(\\s*)${te[ne.LONECARET]}\\s+`, !0), _.caretTrimReplace = "$1^", ce("CARET", `^${te[ne.LONECARET]}${te[ne.XRANGEPLAIN]}$`), ce("CARETLOOSE", `^${te[ne.LONECARET]}${te[ne.XRANGEPLAINLOOSE]}$`), ce("COMPARATORLOOSE", `^${te[ne.GTLT]}\\s*(${te[ne.LOOSEPLAIN]})$|^$`), ce("COMPARATOR", `^${te[ne.GTLT]}\\s*(${te[ne.FULLPLAIN]})$|^$`), ce("COMPARATORTRIM", `(\\s*)${te[ne.GTLT]}\\s*(${te[ne.LOOSEPLAIN]}|${te[ne.XRANGEPLAIN]})`, !0), _.comparatorTrimReplace = "$1$2$3", ce("HYPHENRANGE", `^\\s*(${te[ne.XRANGEPLAIN]})\\s+-\\s+(${te[ne.XRANGEPLAIN]})\\s*$`), ce("HYPHENRANGELOOSE", `^\\s*(${te[ne.XRANGEPLAINLOOSE]})\\s+-\\s+(${te[ne.XRANGEPLAINLOOSE]})\\s*$`), ce("STAR", "(<|>)?=?\\s*\\*"), ce("GTE0", "^\\s*>=\\s*0\\.0\\.0\\s*$"), ce("GTE0PRE", "^\\s*>=\\s*0\\.0\\.0-0\\s*$");
    }(re, re.exports)), re.exports;
  }
  var identifiers, hasRequiredIdentifiers;
  function requireIdentifiers() {
    if (hasRequiredIdentifiers) return identifiers;
    hasRequiredIdentifiers = 1;
    const O = /^[0-9]+$/, _ = (V, q) => {
      const G = O.test(V), J = O.test(q);
      return G && J && (V = +V, q = +q), V === q ? 0 : G && !J ? -1 : J && !G ? 1 : V < q ? -1 : 1;
    };
    return identifiers = {
      compareIdentifiers: _,
      rcompareIdentifiers: (V, q) => _(q, V)
    }, identifiers;
  }
  var semver, hasRequiredSemver;
  function requireSemver() {
    if (hasRequiredSemver) return semver;
    hasRequiredSemver = 1;
    const O = requireDebug(), { MAX_LENGTH: _, MAX_SAFE_INTEGER: H } = requireConstants(), { safeRe: V, t: q } = requireRe(), G = requireParseOptions(), { compareIdentifiers: J } = requireIdentifiers();
    class Z {
      constructor(ne, ee) {
        if (ee = G(ee), ne instanceof Z) {
          if (ne.loose === !!ee.loose && ne.includePrerelease === !!ee.includePrerelease)
            return ne;
          ne = ne.version;
        } else if (typeof ne != "string")
          throw new TypeError(`Invalid version. Must be a string. Got type "${typeof ne}".`);
        if (ne.length > _)
          throw new TypeError(
            `version is longer than ${_} characters`
          );
        O("SemVer", ne, ee), this.options = ee, this.loose = !!ee.loose, this.includePrerelease = !!ee.includePrerelease;
        const ae = ne.trim().match(ee.loose ? V[q.LOOSE] : V[q.FULL]);
        if (!ae)
          throw new TypeError(`Invalid Version: ${ne}`);
        if (this.raw = ne, this.major = +ae[1], this.minor = +ae[2], this.patch = +ae[3], this.major > H || this.major < 0)
          throw new TypeError("Invalid major version");
        if (this.minor > H || this.minor < 0)
          throw new TypeError("Invalid minor version");
        if (this.patch > H || this.patch < 0)
          throw new TypeError("Invalid patch version");
        ae[4] ? this.prerelease = ae[4].split(".").map((oe) => {
          if (/^[0-9]+$/.test(oe)) {
            const se = +oe;
            if (se >= 0 && se < H)
              return se;
          }
          return oe;
        }) : this.prerelease = [], this.build = ae[5] ? ae[5].split(".") : [], this.format();
      }
      format() {
        return this.version = `${this.major}.${this.minor}.${this.patch}`, this.prerelease.length && (this.version += `-${this.prerelease.join(".")}`), this.version;
      }
      toString() {
        return this.version;
      }
      compare(ne) {
        if (O("SemVer.compare", this.version, this.options, ne), !(ne instanceof Z)) {
          if (typeof ne == "string" && ne === this.version)
            return 0;
          ne = new Z(ne, this.options);
        }
        return ne.version === this.version ? 0 : this.compareMain(ne) || this.comparePre(ne);
      }
      compareMain(ne) {
        return ne instanceof Z || (ne = new Z(ne, this.options)), J(this.major, ne.major) || J(this.minor, ne.minor) || J(this.patch, ne.patch);
      }
      comparePre(ne) {
        if (ne instanceof Z || (ne = new Z(ne, this.options)), this.prerelease.length && !ne.prerelease.length)
          return -1;
        if (!this.prerelease.length && ne.prerelease.length)
          return 1;
        if (!this.prerelease.length && !ne.prerelease.length)
          return 0;
        let ee = 0;
        do {
          const ae = this.prerelease[ee], oe = ne.prerelease[ee];
          if (O("prerelease compare", ee, ae, oe), ae === void 0 && oe === void 0)
            return 0;
          if (oe === void 0)
            return 1;
          if (ae === void 0)
            return -1;
          if (ae === oe)
            continue;
          return J(ae, oe);
        } while (++ee);
      }
      compareBuild(ne) {
        ne instanceof Z || (ne = new Z(ne, this.options));
        let ee = 0;
        do {
          const ae = this.build[ee], oe = ne.build[ee];
          if (O("build compare", ee, ae, oe), ae === void 0 && oe === void 0)
            return 0;
          if (oe === void 0)
            return 1;
          if (ae === void 0)
            return -1;
          if (ae === oe)
            continue;
          return J(ae, oe);
        } while (++ee);
      }
      // preminor will bump the version up to the next minor release, and immediately
      // down to pre-release. premajor and prepatch work the same way.
      inc(ne, ee, ae) {
        switch (ne) {
          case "premajor":
            this.prerelease.length = 0, this.patch = 0, this.minor = 0, this.major++, this.inc("pre", ee, ae);
            break;
          case "preminor":
            this.prerelease.length = 0, this.patch = 0, this.minor++, this.inc("pre", ee, ae);
            break;
          case "prepatch":
            this.prerelease.length = 0, this.inc("patch", ee, ae), this.inc("pre", ee, ae);
            break;
          // If the input is a non-prerelease version, this acts the same as
          // prepatch.
          case "prerelease":
            this.prerelease.length === 0 && this.inc("patch", ee, ae), this.inc("pre", ee, ae);
            break;
          case "major":
            (this.minor !== 0 || this.patch !== 0 || this.prerelease.length === 0) && this.major++, this.minor = 0, this.patch = 0, this.prerelease = [];
            break;
          case "minor":
            (this.patch !== 0 || this.prerelease.length === 0) && this.minor++, this.patch = 0, this.prerelease = [];
            break;
          case "patch":
            this.prerelease.length === 0 && this.patch++, this.prerelease = [];
            break;
          // This probably shouldn't be used publicly.
          // 1.0.0 'pre' would become 1.0.0-0 which is the wrong direction.
          case "pre": {
            const oe = Number(ae) ? 1 : 0;
            if (!ee && ae === !1)
              throw new Error("invalid increment argument: identifier is empty");
            if (this.prerelease.length === 0)
              this.prerelease = [oe];
            else {
              let se = this.prerelease.length;
              for (; --se >= 0; )
                typeof this.prerelease[se] == "number" && (this.prerelease[se]++, se = -2);
              if (se === -1) {
                if (ee === this.prerelease.join(".") && ae === !1)
                  throw new Error("invalid increment argument: identifier already exists");
                this.prerelease.push(oe);
              }
            }
            if (ee) {
              let se = [ee, oe];
              ae === !1 && (se = [ee]), J(this.prerelease[0], ee) === 0 ? isNaN(this.prerelease[1]) && (this.prerelease = se) : this.prerelease = se;
            }
            break;
          }
          default:
            throw new Error(`invalid increment argument: ${ne}`);
        }
        return this.raw = this.format(), this.build.length && (this.raw += `+${this.build.join(".")}`), this;
      }
    }
    return semver = Z, semver;
  }
  var compare_1, hasRequiredCompare;
  function requireCompare() {
    if (hasRequiredCompare) return compare_1;
    hasRequiredCompare = 1;
    const O = requireSemver();
    return compare_1 = (H, V, q) => new O(H, q).compare(new O(V, q)), compare_1;
  }
  var eq_1, hasRequiredEq;
  function requireEq() {
    if (hasRequiredEq) return eq_1;
    hasRequiredEq = 1;
    const O = requireCompare();
    return eq_1 = (H, V, q) => O(H, V, q) === 0, eq_1;
  }
  var neq_1, hasRequiredNeq;
  function requireNeq() {
    if (hasRequiredNeq) return neq_1;
    hasRequiredNeq = 1;
    const O = requireCompare();
    return neq_1 = (H, V, q) => O(H, V, q) !== 0, neq_1;
  }
  var gt_1, hasRequiredGt;
  function requireGt() {
    if (hasRequiredGt) return gt_1;
    hasRequiredGt = 1;
    const O = requireCompare();
    return gt_1 = (H, V, q) => O(H, V, q) > 0, gt_1;
  }
  var gte_1, hasRequiredGte;
  function requireGte() {
    if (hasRequiredGte) return gte_1;
    hasRequiredGte = 1;
    const O = requireCompare();
    return gte_1 = (H, V, q) => O(H, V, q) >= 0, gte_1;
  }
  var lt_1, hasRequiredLt;
  function requireLt() {
    if (hasRequiredLt) return lt_1;
    hasRequiredLt = 1;
    const O = requireCompare();
    return lt_1 = (H, V, q) => O(H, V, q) < 0, lt_1;
  }
  var lte_1, hasRequiredLte;
  function requireLte() {
    if (hasRequiredLte) return lte_1;
    hasRequiredLte = 1;
    const O = requireCompare();
    return lte_1 = (H, V, q) => O(H, V, q) <= 0, lte_1;
  }
  var cmp_1, hasRequiredCmp;
  function requireCmp() {
    if (hasRequiredCmp) return cmp_1;
    hasRequiredCmp = 1;
    const O = requireEq(), _ = requireNeq(), H = requireGt(), V = requireGte(), q = requireLt(), G = requireLte();
    return cmp_1 = (Z, te, ne, ee) => {
      switch (te) {
        case "===":
          return typeof Z == "object" && (Z = Z.version), typeof ne == "object" && (ne = ne.version), Z === ne;
        case "!==":
          return typeof Z == "object" && (Z = Z.version), typeof ne == "object" && (ne = ne.version), Z !== ne;
        case "":
        case "=":
        case "==":
          return O(Z, ne, ee);
        case "!=":
          return _(Z, ne, ee);
        case ">":
          return H(Z, ne, ee);
        case ">=":
          return V(Z, ne, ee);
        case "<":
          return q(Z, ne, ee);
        case "<=":
          return G(Z, ne, ee);
        default:
          throw new TypeError(`Invalid operator: ${te}`);
      }
    }, cmp_1;
  }
  var comparator, hasRequiredComparator;
  function requireComparator() {
    if (hasRequiredComparator) return comparator;
    hasRequiredComparator = 1;
    const O = Symbol("SemVer ANY");
    class _ {
      static get ANY() {
        return O;
      }
      constructor(ee, ae) {
        if (ae = H(ae), ee instanceof _) {
          if (ee.loose === !!ae.loose)
            return ee;
          ee = ee.value;
        }
        ee = ee.trim().split(/\s+/).join(" "), J("comparator", ee, ae), this.options = ae, this.loose = !!ae.loose, this.parse(ee), this.semver === O ? this.value = "" : this.value = this.operator + this.semver.version, J("comp", this);
      }
      parse(ee) {
        const ae = this.options.loose ? V[q.COMPARATORLOOSE] : V[q.COMPARATOR], oe = ee.match(ae);
        if (!oe)
          throw new TypeError(`Invalid comparator: ${ee}`);
        this.operator = oe[1] !== void 0 ? oe[1] : "", this.operator === "=" && (this.operator = ""), oe[2] ? this.semver = new Z(oe[2], this.options.loose) : this.semver = O;
      }
      toString() {
        return this.value;
      }
      test(ee) {
        if (J("Comparator.test", ee, this.options.loose), this.semver === O || ee === O)
          return !0;
        if (typeof ee == "string")
          try {
            ee = new Z(ee, this.options);
          } catch {
            return !1;
          }
        return G(ee, this.operator, this.semver, this.options);
      }
      intersects(ee, ae) {
        if (!(ee instanceof _))
          throw new TypeError("a Comparator is required");
        return this.operator === "" ? this.value === "" ? !0 : new te(ee.value, ae).test(this.value) : ee.operator === "" ? ee.value === "" ? !0 : new te(this.value, ae).test(ee.semver) : (ae = H(ae), ae.includePrerelease && (this.value === "<0.0.0-0" || ee.value === "<0.0.0-0") || !ae.includePrerelease && (this.value.startsWith("<0.0.0") || ee.value.startsWith("<0.0.0")) ? !1 : !!(this.operator.startsWith(">") && ee.operator.startsWith(">") || this.operator.startsWith("<") && ee.operator.startsWith("<") || this.semver.version === ee.semver.version && this.operator.includes("=") && ee.operator.includes("=") || G(this.semver, "<", ee.semver, ae) && this.operator.startsWith(">") && ee.operator.startsWith("<") || G(this.semver, ">", ee.semver, ae) && this.operator.startsWith("<") && ee.operator.startsWith(">")));
      }
    }
    comparator = _;
    const H = requireParseOptions(), { safeRe: V, t: q } = requireRe(), G = requireCmp(), J = requireDebug(), Z = requireSemver(), te = requireRange();
    return comparator;
  }
  var range, hasRequiredRange;
  function requireRange() {
    if (hasRequiredRange) return range;
    hasRequiredRange = 1;
    const O = /\s+/g;
    class _ {
      constructor(Le, Qe) {
        if (Qe = q(Qe), Le instanceof _)
          return Le.loose === !!Qe.loose && Le.includePrerelease === !!Qe.includePrerelease ? Le : new _(Le.raw, Qe);
        if (Le instanceof G)
          return this.raw = Le.value, this.set = [[Le]], this.formatted = void 0, this;
        if (this.options = Qe, this.loose = !!Qe.loose, this.includePrerelease = !!Qe.includePrerelease, this.raw = Le.trim().replace(O, " "), this.set = this.raw.split("||").map((ke) => this.parseRange(ke.trim())).filter((ke) => ke.length), !this.set.length)
          throw new TypeError(`Invalid SemVer Range: ${this.raw}`);
        if (this.set.length > 1) {
          const ke = this.set[0];
          if (this.set = this.set.filter((Me) => !le(Me[0])), this.set.length === 0)
            this.set = [ke];
          else if (this.set.length > 1) {
            for (const Me of this.set)
              if (Me.length === 1 && ue(Me[0])) {
                this.set = [Me];
                break;
              }
          }
        }
        this.formatted = void 0;
      }
      get range() {
        if (this.formatted === void 0) {
          this.formatted = "";
          for (let Le = 0; Le < this.set.length; Le++) {
            Le > 0 && (this.formatted += "||");
            const Qe = this.set[Le];
            for (let ke = 0; ke < Qe.length; ke++)
              ke > 0 && (this.formatted += " "), this.formatted += Qe[ke].toString().trim();
          }
        }
        return this.formatted;
      }
      format() {
        return this.range;
      }
      toString() {
        return this.range;
      }
      parseRange(Le) {
        const ke = ((this.options.includePrerelease && se) | (this.options.loose && ce)) + ":" + Le, Me = V.get(ke);
        if (Me)
          return Me;
        const Re = this.options.loose, De = Re ? te[ne.HYPHENRANGELOOSE] : te[ne.HYPHENRANGE];
        Le = Le.replace(De, we(this.options.includePrerelease)), J("hyphen replace", Le), Le = Le.replace(te[ne.COMPARATORTRIM], ee), J("comparator trim", Le), Le = Le.replace(te[ne.TILDETRIM], ae), J("tilde trim", Le), Le = Le.replace(te[ne.CARETTRIM], oe), J("caret trim", Le);
        let Pe = Le.split(" ").map((Ve) => fe(Ve, this.options)).join(" ").split(/\s+/).map((Ve) => Se(Ve, this.options));
        Re && (Pe = Pe.filter((Ve) => (J("loose invalid filter", Ve, this.options), !!Ve.match(te[ne.COMPARATORLOOSE])))), J("range list", Pe);
        const $e = /* @__PURE__ */ new Map(), qe = Pe.map((Ve) => new G(Ve, this.options));
        for (const Ve of qe) {
          if (le(Ve))
            return [Ve];
          $e.set(Ve.value, Ve);
        }
        $e.size > 1 && $e.has("") && $e.delete("");
        const Xe = [...$e.values()];
        return V.set(ke, Xe), Xe;
      }
      intersects(Le, Qe) {
        if (!(Le instanceof _))
          throw new TypeError("a Range is required");
        return this.set.some((ke) => de(ke, Qe) && Le.set.some((Me) => de(Me, Qe) && ke.every((Re) => Me.every((De) => Re.intersects(De, Qe)))));
      }
      // if ANY of the sets match ALL of its comparators, then pass
      test(Le) {
        if (!Le)
          return !1;
        if (typeof Le == "string")
          try {
            Le = new Z(Le, this.options);
          } catch {
            return !1;
          }
        for (let Qe = 0; Qe < this.set.length; Qe++)
          if (Ee(this.set[Qe], Le, this.options))
            return !0;
        return !1;
      }
    }
    range = _;
    const H = requireLrucache(), V = new H(), q = requireParseOptions(), G = requireComparator(), J = requireDebug(), Z = requireSemver(), {
      safeRe: te,
      t: ne,
      comparatorTrimReplace: ee,
      tildeTrimReplace: ae,
      caretTrimReplace: oe
    } = requireRe(), { FLAG_INCLUDE_PRERELEASE: se, FLAG_LOOSE: ce } = requireConstants(), le = (xe) => xe.value === "<0.0.0-0", ue = (xe) => xe.value === "", de = (xe, Le) => {
      let Qe = !0;
      const ke = xe.slice();
      let Me = ke.pop();
      for (; Qe && ke.length; )
        Qe = ke.every((Re) => Me.intersects(Re, Le)), Me = ke.pop();
      return Qe;
    }, fe = (xe, Le) => (J("comp", xe, Le), xe = ye(xe, Le), J("caret", xe), xe = me(xe, Le), J("tildes", xe), xe = ve(xe, Le), J("xrange", xe), xe = Ce(xe, Le), J("stars", xe), xe), he = (xe) => !xe || xe.toLowerCase() === "x" || xe === "*", me = (xe, Le) => xe.trim().split(/\s+/).map((Qe) => pe(Qe, Le)).join(" "), pe = (xe, Le) => {
      const Qe = Le.loose ? te[ne.TILDELOOSE] : te[ne.TILDE];
      return xe.replace(Qe, (ke, Me, Re, De, Pe) => {
        J("tilde", xe, ke, Me, Re, De, Pe);
        let $e;
        return he(Me) ? $e = "" : he(Re) ? $e = `>=${Me}.0.0 <${+Me + 1}.0.0-0` : he(De) ? $e = `>=${Me}.${Re}.0 <${Me}.${+Re + 1}.0-0` : Pe ? (J("replaceTilde pr", Pe), $e = `>=${Me}.${Re}.${De}-${Pe} <${Me}.${+Re + 1}.0-0`) : $e = `>=${Me}.${Re}.${De} <${Me}.${+Re + 1}.0-0`, J("tilde return", $e), $e;
      });
    }, ye = (xe, Le) => xe.trim().split(/\s+/).map((Qe) => ge(Qe, Le)).join(" "), ge = (xe, Le) => {
      J("caret", xe, Le);
      const Qe = Le.loose ? te[ne.CARETLOOSE] : te[ne.CARET], ke = Le.includePrerelease ? "-0" : "";
      return xe.replace(Qe, (Me, Re, De, Pe, $e) => {
        J("caret", xe, Me, Re, De, Pe, $e);
        let qe;
        return he(Re) ? qe = "" : he(De) ? qe = `>=${Re}.0.0${ke} <${+Re + 1}.0.0-0` : he(Pe) ? Re === "0" ? qe = `>=${Re}.${De}.0${ke} <${Re}.${+De + 1}.0-0` : qe = `>=${Re}.${De}.0${ke} <${+Re + 1}.0.0-0` : $e ? (J("replaceCaret pr", $e), Re === "0" ? De === "0" ? qe = `>=${Re}.${De}.${Pe}-${$e} <${Re}.${De}.${+Pe + 1}-0` : qe = `>=${Re}.${De}.${Pe}-${$e} <${Re}.${+De + 1}.0-0` : qe = `>=${Re}.${De}.${Pe}-${$e} <${+Re + 1}.0.0-0`) : (J("no pr"), Re === "0" ? De === "0" ? qe = `>=${Re}.${De}.${Pe}${ke} <${Re}.${De}.${+Pe + 1}-0` : qe = `>=${Re}.${De}.${Pe}${ke} <${Re}.${+De + 1}.0-0` : qe = `>=${Re}.${De}.${Pe} <${+Re + 1}.0.0-0`), J("caret return", qe), qe;
      });
    }, ve = (xe, Le) => (J("replaceXRanges", xe, Le), xe.split(/\s+/).map((Qe) => be(Qe, Le)).join(" ")), be = (xe, Le) => {
      xe = xe.trim();
      const Qe = Le.loose ? te[ne.XRANGELOOSE] : te[ne.XRANGE];
      return xe.replace(Qe, (ke, Me, Re, De, Pe, $e) => {
        J("xRange", xe, ke, Me, Re, De, Pe, $e);
        const qe = he(Re), Xe = qe || he(De), Ve = Xe || he(Pe), Ze = Ve;
        return Me === "=" && Ze && (Me = ""), $e = Le.includePrerelease ? "-0" : "", qe ? Me === ">" || Me === "<" ? ke = "<0.0.0-0" : ke = "*" : Me && Ze ? (Xe && (De = 0), Pe = 0, Me === ">" ? (Me = ">=", Xe ? (Re = +Re + 1, De = 0, Pe = 0) : (De = +De + 1, Pe = 0)) : Me === "<=" && (Me = "<", Xe ? Re = +Re + 1 : De = +De + 1), Me === "<" && ($e = "-0"), ke = `${Me + Re}.${De}.${Pe}${$e}`) : Xe ? ke = `>=${Re}.0.0${$e} <${+Re + 1}.0.0-0` : Ve && (ke = `>=${Re}.${De}.0${$e} <${Re}.${+De + 1}.0-0`), J("xRange return", ke), ke;
      });
    }, Ce = (xe, Le) => (J("replaceStars", xe, Le), xe.trim().replace(te[ne.STAR], "")), Se = (xe, Le) => (J("replaceGTE0", xe, Le), xe.trim().replace(te[Le.includePrerelease ? ne.GTE0PRE : ne.GTE0], "")), we = (xe) => (Le, Qe, ke, Me, Re, De, Pe, $e, qe, Xe, Ve, Ze) => (he(ke) ? Qe = "" : he(Me) ? Qe = `>=${ke}.0.0${xe ? "-0" : ""}` : he(Re) ? Qe = `>=${ke}.${Me}.0${xe ? "-0" : ""}` : De ? Qe = `>=${Qe}` : Qe = `>=${Qe}${xe ? "-0" : ""}`, he(qe) ? $e = "" : he(Xe) ? $e = `<${+qe + 1}.0.0-0` : he(Ve) ? $e = `<${qe}.${+Xe + 1}.0-0` : Ze ? $e = `<=${qe}.${Xe}.${Ve}-${Ze}` : xe ? $e = `<${qe}.${Xe}.${+Ve + 1}-0` : $e = `<=${$e}`, `${Qe} ${$e}`.trim()), Ee = (xe, Le, Qe) => {
      for (let ke = 0; ke < xe.length; ke++)
        if (!xe[ke].test(Le))
          return !1;
      if (Le.prerelease.length && !Qe.includePrerelease) {
        for (let ke = 0; ke < xe.length; ke++)
          if (J(xe[ke].semver), xe[ke].semver !== G.ANY && xe[ke].semver.prerelease.length > 0) {
            const Me = xe[ke].semver;
            if (Me.major === Le.major && Me.minor === Le.minor && Me.patch === Le.patch)
              return !0;
          }
        return !1;
      }
      return !0;
    };
    return range;
  }
  var satisfies_1, hasRequiredSatisfies;
  function requireSatisfies() {
    if (hasRequiredSatisfies) return satisfies_1;
    hasRequiredSatisfies = 1;
    const O = requireRange();
    return satisfies_1 = (H, V, q) => {
      try {
        V = new O(V, q);
      } catch {
        return !1;
      }
      return V.test(H);
    }, satisfies_1;
  }
  var satisfiesExports = requireSatisfies(), satisfies = /* @__PURE__ */ getDefaultExportFromCjs(satisfiesExports);
  function post(O, _, H) {
    const V = O.open(_), q = 250, { origin: G } = new URL(_);
    let J = 40;
    function Z(ne) {
      ne.source === V && (J = 0, O.removeEventListener("message", Z, !1));
    }
    O.addEventListener("message", Z, !1);
    function te() {
      J <= 0 || (V.postMessage(H, G), setTimeout(te, q), J -= 1);
    }
    setTimeout(te, q);
  }
  var embedStyle = `.vega-embed {
  position: relative;
  display: inline-block;
  box-sizing: border-box;
}
.vega-embed.has-actions {
  padding-right: 38px;
}
.vega-embed details:not([open]) > :not(summary) {
  display: none !important;
}
.vega-embed summary {
  list-style: none;
  position: absolute;
  top: 0;
  right: 0;
  padding: 6px;
  z-index: 1000;
  background: white;
  box-shadow: 1px 1px 3px rgba(0, 0, 0, 0.1);
  color: #1b1e23;
  border: 1px solid #aaa;
  border-radius: 999px;
  opacity: 0.2;
  transition: opacity 0.4s ease-in;
  cursor: pointer;
  line-height: 0px;
}
.vega-embed summary::-webkit-details-marker {
  display: none;
}
.vega-embed summary:active {
  box-shadow: #aaa 0px 0px 0px 1px inset;
}
.vega-embed summary svg {
  width: 14px;
  height: 14px;
}
.vega-embed details[open] summary {
  opacity: 0.7;
}
.vega-embed:hover summary, .vega-embed:focus-within summary {
  opacity: 1 !important;
  transition: opacity 0.2s ease;
}
.vega-embed .vega-actions {
  position: absolute;
  z-index: 1001;
  top: 35px;
  right: -9px;
  display: flex;
  flex-direction: column;
  padding-bottom: 8px;
  padding-top: 8px;
  border-radius: 4px;
  box-shadow: 0 2px 8px 0 rgba(0, 0, 0, 0.2);
  border: 1px solid #d9d9d9;
  background: white;
  animation-duration: 0.15s;
  animation-name: scale-in;
  animation-timing-function: cubic-bezier(0.2, 0, 0.13, 1.5);
  text-align: left;
}
.vega-embed .vega-actions a {
  padding: 8px 16px;
  font-family: sans-serif;
  font-size: 14px;
  font-weight: 600;
  white-space: nowrap;
  color: #434a56;
  text-decoration: none;
}
.vega-embed .vega-actions a:hover, .vega-embed .vega-actions a:focus {
  background-color: #f7f7f9;
  color: black;
}
.vega-embed .vega-actions::before, .vega-embed .vega-actions::after {
  content: "";
  display: inline-block;
  position: absolute;
}
.vega-embed .vega-actions::before {
  left: auto;
  right: 14px;
  top: -16px;
  border: 8px solid rgba(0, 0, 0, 0);
  border-bottom-color: #d9d9d9;
}
.vega-embed .vega-actions::after {
  left: auto;
  right: 15px;
  top: -14px;
  border: 7px solid rgba(0, 0, 0, 0);
  border-bottom-color: #fff;
}
.vega-embed .chart-wrapper.fit-x {
  width: 100%;
}
.vega-embed .chart-wrapper.fit-y {
  height: 100%;
}

.vega-embed-wrapper {
  max-width: 100%;
  overflow: auto;
  padding-right: 14px;
}

@keyframes scale-in {
  from {
    opacity: 0;
    transform: scale(0.6);
  }
  to {
    opacity: 1;
    transform: scale(1);
  }
}
`;
  function mergeDeep(O, ..._) {
    for (const H of _)
      deepMerge_(O, H);
    return O;
  }
  function deepMerge_(O, _) {
    for (const H of Object.keys(_))
      writeConfig(O, H, _[H], !0);
  }
  const vega = vegaImport;
  let vegaLite = vegaLiteImport;
  const w = typeof window < "u" ? window : void 0;
  var Zy;
  vegaLite === void 0 && ((Zy = w == null ? void 0 : w.vl) != null && Zy.compile) && (vegaLite = w.vl);
  const DEFAULT_ACTIONS = { export: { svg: !0, png: !0 }, source: !0, compiled: !0, editor: !0 }, I18N = {
    CLICK_TO_VIEW_ACTIONS: "Click to view actions",
    COMPILED_ACTION: "View Compiled Vega",
    EDITOR_ACTION: "Open in Vega Editor",
    PNG_ACTION: "Save as PNG",
    SOURCE_ACTION: "View Source",
    SVG_ACTION: "Save as SVG"
  }, NAMES = {
    vega: "Vega",
    "vega-lite": "Vega-Lite"
  }, VERSION = {
    vega: vega.version,
    "vega-lite": vegaLite ? vegaLite.version : "not available"
  }, PREPROCESSOR = {
    vega: (O) => O,
    "vega-lite": (O, _) => vegaLite.compile(O, { config: _ }).spec
  }, SVG_CIRCLES = `
<svg viewBox="0 0 16 16" fill="currentColor" stroke="none" stroke-width="1" stroke-linecap="round" stroke-linejoin="round">
  <circle r="2" cy="8" cx="2"></circle>
  <circle r="2" cy="8" cx="8"></circle>
  <circle r="2" cy="8" cx="14"></circle>
</svg>`, CHART_WRAPPER_CLASS = "chart-wrapper";
  function isTooltipHandler(O) {
    return typeof O == "function";
  }
  function viewSource(O, _, H, V) {
    const q = `<html><head>${_}</head><body><pre><code class="json">`, G = `</code></pre>${H}</body></html>`, J = window.open("");
    J.document.write(q + O + G), J.document.title = `${NAMES[V]} JSON Source`;
  }
  function guessMode(O, _) {
    if (O.$schema) {
      const H = e(O.$schema);
      _ && _ !== H.library && console.warn(`The given visualization spec is written in ${NAMES[H.library]}, but mode argument sets ${NAMES[_] ?? _}.`);
      const V = H.library;
      return satisfies(VERSION[V], `^${H.version.slice(1)}`) || console.warn(`The input spec uses ${NAMES[V]} ${H.version}, but the current version of ${NAMES[V]} is v${VERSION[V]}.`), V;
    }
    return "mark" in O || "encoding" in O || "layer" in O || "hconcat" in O || "vconcat" in O || "facet" in O || "repeat" in O ? "vega-lite" : "marks" in O || "signals" in O || "scales" in O || "axes" in O ? "vega" : _ ?? "vega";
  }
  function isLoader(O) {
    return !!(O && "load" in O);
  }
  function createLoader(O) {
    return isLoader(O) ? O : vega.loader(O);
  }
  function embedOptionsFromUsermeta(O) {
    var H;
    const _ = ((H = O.usermeta) == null ? void 0 : H.embedOptions) ?? {};
    return isString$1(_.defaultStyle) && (_.defaultStyle = !1), _;
  }
  async function embed(O, _, H = {}) {
    let V, q;
    isString$1(_) ? (q = createLoader(H.loader), V = JSON.parse(await q.load(_))) : V = _;
    const G = embedOptionsFromUsermeta(V), J = G.loader;
    (!q || J) && (q = createLoader(H.loader ?? J));
    const Z = await loadOpts(G, q), te = await loadOpts(H, q), ne = {
      ...mergeDeep(te, Z),
      config: mergeConfig(te.config ?? {}, Z.config ?? {})
    };
    return await _embed(O, V, ne, q);
  }
  async function loadOpts(O, _) {
    const H = isString$1(O.config) ? JSON.parse(await _.load(O.config)) : O.config ?? {}, V = isString$1(O.patch) ? JSON.parse(await _.load(O.patch)) : O.patch;
    return {
      ...O,
      ...V ? { patch: V } : {},
      ...H ? { config: H } : {}
    };
  }
  function getRoot$1(O) {
    const _ = O.getRootNode ? O.getRootNode() : document;
    return _ instanceof ShadowRoot ? { root: _, rootContainer: _ } : { root: document, rootContainer: document.head ?? document.body };
  }
  async function _embed(O, _, H = {}, V) {
    const q = H.theme ? mergeConfig(themes[H.theme], H.config ?? {}) : H.config, G = isBoolean$3(H.actions) ? H.actions : mergeDeep({}, DEFAULT_ACTIONS, H.actions ?? {}), J = { ...I18N, ...H.i18n }, Z = H.renderer ?? "canvas", te = H.logLevel ?? vega.Warn, ne = H.downloadFileName ?? "visualization", ee = typeof O == "string" ? document.querySelector(O) : O;
    if (!ee)
      throw new Error(`${O} does not exist`);
    if (H.defaultStyle !== !1) {
      const pe = "vega-embed-style", { root: ye, rootContainer: ge } = getRoot$1(ee);
      if (!ye.getElementById(pe)) {
        const ve = document.createElement("style");
        ve.id = pe, ve.innerHTML = H.defaultStyle === void 0 || H.defaultStyle === !0 ? embedStyle.toString() : H.defaultStyle, ge.appendChild(ve);
      }
    }
    const ae = guessMode(_, H.mode);
    let oe = PREPROCESSOR[ae](_, q);
    if (ae === "vega-lite" && oe.$schema) {
      const pe = e(oe.$schema);
      satisfies(VERSION.vega, `^${pe.version.slice(1)}`) || console.warn(`The compiled spec uses Vega ${pe.version}, but current version is v${VERSION.vega}.`);
    }
    ee.classList.add("vega-embed"), G && ee.classList.add("has-actions"), ee.innerHTML = "";
    let se = ee;
    if (G) {
      const pe = document.createElement("div");
      pe.classList.add(CHART_WRAPPER_CLASS), ee.appendChild(pe), se = pe;
    }
    const ce = H.patch;
    if (ce && (oe = ce instanceof Function ? ce(oe) : applyPatch(oe, ce, !0, !1).newDocument), H.formatLocale && vega.formatLocale(H.formatLocale), H.timeFormatLocale && vega.timeFormatLocale(H.timeFormatLocale), H.expressionFunctions)
      for (const pe in H.expressionFunctions) {
        const ye = H.expressionFunctions[pe];
        "fn" in ye ? vega.expressionFunction(pe, ye.fn, ye.visitor) : ye instanceof Function && vega.expressionFunction(pe, ye);
      }
    const { ast: le } = H, ue = vega.parse(oe, ae === "vega-lite" ? {} : q, { ast: le }), de = new (H.viewClass || vega.View)(ue, {
      loader: V,
      logLevel: te,
      renderer: Z,
      ...le ? { expr: vega.expressionInterpreter ?? H.expr ?? expression$1 } : {}
    });
    if (de.addSignalListener("autosize", (pe, ye) => {
      const { type: ge } = ye;
      ge == "fit-x" ? (se.classList.add("fit-x"), se.classList.remove("fit-y")) : ge == "fit-y" ? (se.classList.remove("fit-x"), se.classList.add("fit-y")) : ge == "fit" ? se.classList.add("fit-x", "fit-y") : se.classList.remove("fit-x", "fit-y");
    }), H.tooltip !== !1) {
      const { loader: pe, tooltip: ye } = H, ge = pe && !isLoader(pe) ? pe == null ? void 0 : pe.baseURL : void 0, ve = isTooltipHandler(ye) ? ye : (
        // user provided boolean true or tooltip options
        new Handler({ baseURL: ge, ...ye === !0 ? {} : ye }).call
      );
      de.tooltip(ve);
    }
    let { hover: fe } = H;
    if (fe === void 0 && (fe = ae === "vega"), fe) {
      const { hoverSet: pe, updateSet: ye } = typeof fe == "boolean" ? {} : fe;
      de.hover(pe, ye);
    }
    H && (H.width != null && de.width(H.width), H.height != null && de.height(H.height), H.padding != null && de.padding(H.padding)), await de.initialize(se, H.bind).runAsync();
    let he;
    if (G !== !1) {
      let pe = ee;
      if (H.defaultStyle !== !1 || H.forceActionsMenu) {
        const ge = document.createElement("details");
        ge.title = J.CLICK_TO_VIEW_ACTIONS, ee.append(ge), pe = ge;
        const ve = document.createElement("summary");
        ve.innerHTML = SVG_CIRCLES, ge.append(ve), he = (be) => {
          ge.contains(be.target) || ge.removeAttribute("open");
        }, document.addEventListener("click", he);
      }
      const ye = document.createElement("div");
      if (pe.append(ye), ye.classList.add("vega-actions"), G === !0 || G.export !== !1) {
        for (const ge of ["svg", "png"])
          if (G === !0 || G.export === !0 || G.export[ge]) {
            const ve = J[`${ge.toUpperCase()}_ACTION`], be = document.createElement("a"), Ce = isObject$2(H.scaleFactor) ? H.scaleFactor[ge] : H.scaleFactor;
            be.text = ve, be.href = "#", be.target = "_blank", be.download = `${ne}.${ge}`, be.addEventListener("mousedown", async function(Se) {
              Se.preventDefault();
              const we = await de.toImageURL(ge, Ce);
              this.href = we;
            }), ye.append(be);
          }
      }
      if (G === !0 || G.source !== !1) {
        const ge = document.createElement("a");
        ge.text = J.SOURCE_ACTION, ge.href = "#", ge.addEventListener("click", function(ve) {
          viewSource(stringify$2(_), H.sourceHeader ?? "", H.sourceFooter ?? "", ae), ve.preventDefault();
        }), ye.append(ge);
      }
      if (ae === "vega-lite" && (G === !0 || G.compiled !== !1)) {
        const ge = document.createElement("a");
        ge.text = J.COMPILED_ACTION, ge.href = "#", ge.addEventListener("click", function(ve) {
          viewSource(stringify$2(oe), H.sourceHeader ?? "", H.sourceFooter ?? "", "vega"), ve.preventDefault();
        }), ye.append(ge);
      }
      if (G === !0 || G.editor !== !1) {
        const ge = H.editorUrl ?? "https://vega.github.io/editor/", ve = document.createElement("a");
        ve.text = J.EDITOR_ACTION, ve.href = "#", ve.addEventListener("click", function(be) {
          post(window, ge, {
            config: q,
            mode: ce ? "vega" : ae,
            renderer: Z,
            spec: stringify$2(ce ? oe : _)
          }), be.preventDefault();
        }), ye.append(ve);
      }
    }
    function me() {
      he && document.removeEventListener("click", he), de.finalize();
    }
    return { view: de, spec: _, vgSpec: oe, finalize: me, embedOptions: H };
  }
  const _sfc_main$5 = /* @__PURE__ */ defineComponent({
    __name: "Vega",
    props: {
      plotJson: {},
      responsive: { type: Boolean },
      singleBlockEmbed: { type: Boolean }
    },
    setup(O) {
      const _ = O, H = `vega_${v4()}`, V = ref$1(), q = (Z) => {
        Z.width = "container", _.singleBlockEmbed && (Z.height = "container");
      }, G = () => {
        const Z = document.getElementById(H), te = Z && Z.querySelector(".vega-bindings"), ne = Z && Z.querySelector("canvas");
        ne && te && (ne.style.height = `${ne.offsetHeight - te.offsetHeight}px`);
      }, J = async () => {
        try {
          _.responsive && q(_.plotJson);
          const Z = toRaw(_.plotJson), te = await embed(`#${H}`, Z, {
            mode: "vega-lite",
            actions: !1
            // disable the altair action menu
          });
          V.value = te, _.singleBlockEmbed && G();
        } catch (Z) {
          console.error("An error occurred while rendering an Altair chart: ", Z);
        }
      };
      return onMounted(() => {
        J();
      }), onUnmounted(() => {
        var Z;
        return void ((Z = V.value) == null ? void 0 : Z.finalize());
      }), (Z, te) => (openBlock(), createElementBlock("div", {
        class: normalizeClass([
          "justify-center bg-white flex",
          { "w-full": _.responsive, "h-iframe": Z.singleBlockEmbed }
        ])
      }, [
        createBaseVNode("div", {
          id: H,
          class: normalizeClass({ "w-full": _.responsive, "h-iframe": Z.singleBlockEmbed }),
          "data-cy": "block-vega"
        }, null, 2)
      ], 2));
    }
  }), Vega = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    default: _sfc_main$5
  }, Symbol.toStringTag, { value: "Module" }));
  /*!
   * Built by Revolist OU ❤️
   */
  const NAMESPACE = "revo-grid", BUILD = (
    /* revo-grid */
    { allRenderFn: !1, appendChildSlotFix: !1, asyncLoading: !0, asyncQueue: !1, attachStyles: !0, cloneNodeFix: !1, cmpDidLoad: !0, cmpDidRender: !0, cmpDidUnload: !1, cmpDidUpdate: !1, cmpShouldUpdate: !1, cmpWillLoad: !0, cmpWillRender: !0, cmpWillUpdate: !1, connectedCallback: !0, constructableCSS: !0, cssAnnotations: !0, devTools: !1, disconnectedCallback: !0, element: !1, event: !0, experimentalScopedSlotChanges: !1, experimentalSlotFixes: !1, formAssociated: !1, hasRenderFn: !0, hostListener: !0, hostListenerTarget: !0, hostListenerTargetBody: !1, hostListenerTargetDocument: !0, hostListenerTargetParent: !1, hostListenerTargetWindow: !1, hotModuleReplacement: !1, hydrateClientSide: !0, hydrateServerSide: !1, hydratedAttribute: !1, hydratedClass: !0, hydratedSelectorName: "hydrated", initializeNextTick: !0, invisiblePrehydration: !1, isDebug: !1, isDev: !1, isTesting: !1, lazyLoad: !0, lifecycle: !0, lifecycleDOMEvents: !1, member: !0, method: !0, mode: !1, observeAttribute: !0, profile: !1, prop: !0, propBoolean: !0, propMutable: !0, propNumber: !0, propString: !0, reflect: !0, scoped: !1, scopedSlotTextContentFix: !1, scriptDataOpts: !1, shadowDelegatesFocus: !1, shadowDom: !1, slot: !0, slotChildNodesFix: !1, slotRelocation: !0, state: !0, style: !0, svg: !0, taskQueue: !0, transformTagName: !1, updatable: !0, vdomAttribute: !0, vdomClass: !0, vdomFunctional: !0, vdomKey: !0, vdomListener: !0, vdomPropOrAttr: !0, vdomRef: !0, vdomRender: !0, vdomStyle: !0, vdomText: !0, vdomXlink: !0, watchCallback: !0 }
  );
  var __defProp = Object.defineProperty, __export = (O, _) => {
    for (var H in _)
      __defProp(O, H, { get: _[H], enumerable: !0 });
  }, hostRefs = /* @__PURE__ */ new WeakMap(), getHostRef = (O) => hostRefs.get(O), registerInstance = (O, _) => hostRefs.set(_.$lazyInstance$ = O, _), registerHost = (O, _) => {
    const H = {
      $flags$: 0,
      $hostElement$: O,
      $cmpMeta$: _,
      $instanceValues$: /* @__PURE__ */ new Map()
    };
    return H.$onInstancePromise$ = new Promise((V) => H.$onInstanceResolve$ = V), H.$onReadyPromise$ = new Promise((V) => H.$onReadyResolve$ = V), O["s-p"] = [], O["s-rc"] = [], hostRefs.set(O, H);
  }, isMemberInElement = (O, _) => _ in O, consoleError = (O, _) => (0, console.error)(O, _), cmpModules = /* @__PURE__ */ new Map(), loadModule = (O, _, H) => {
    const V = O.$tagName$.replace(/-/g, "_"), q = O.$lazyBundleId$;
    if (!q)
      return;
    const G = cmpModules.get(q);
    if (G)
      return G[V];
    {
      const J = (Z) => (cmpModules.set(q, Z), Z[V]);
      switch (q) {
        case "revo-grid":
          return Promise.resolve().then(() => revoGrid_entry).then(J, consoleError);
        case "revogr-filter-panel":
          return Promise.resolve().then(() => revogrFilterPanel_entry).then(J, consoleError);
        case "revogr-attribution_7":
          return Promise.resolve().then(() => revogrAttribution_7_entry).then(J, consoleError);
        case "revogr-clipboard_3":
          return Promise.resolve().then(() => revogrClipboard_3_entry).then(J, consoleError);
        case "revogr-data_4":
          return Promise.resolve().then(() => revogrData_4_entry).then(J, consoleError);
      }
    }
    return import(
      /* @vite-ignore */
      /* webpackInclude: /\.entry\.js$/ */
      /* webpackExclude: /\.system\.entry\.js$/ */
      /* webpackMode: "lazy" */
      `./${q}.entry.js`
    ).then((J) => (cmpModules.set(q, J), J[V]), consoleError);
  }, styles = /* @__PURE__ */ new Map(), CONTENT_REF_ID = "r", ORG_LOCATION_ID = "o", SLOT_NODE_ID = "s", TEXT_NODE_ID = "t", COMMENT_NODE_ID = "c", HYDRATE_ID = "s-id", HYDRATED_STYLE_ID = "sty-id", HYDRATE_CHILD_ID = "c-id", SLOT_FB_CSS = "slot-fb{display:contents}slot-fb[hidden]{display:none}", XLINK_NS = "http://www.w3.org/1999/xlink", win = typeof window < "u" ? window : {}, doc$1 = win.document || { head: {} }, plt = {
    $flags$: 0,
    $resourcesUrl$: "",
    jmp: (O) => O(),
    raf: (O) => requestAnimationFrame(O),
    ael: (O, _, H, V) => O.addEventListener(_, H, V),
    rel: (O, _, H, V) => O.removeEventListener(_, H, V),
    ce: (O, _) => new CustomEvent(O, _)
  }, supportsShadow = BUILD.shadowDom, supportsListenerOptions = /* @__PURE__ */ (() => {
    let O = !1;
    try {
      doc$1.addEventListener(
        "e",
        null,
        Object.defineProperty({}, "passive", {
          get() {
            O = !0;
          }
        })
      );
    } catch {
    }
    return O;
  })(), promiseResolve = (O) => Promise.resolve(O), supportsConstructableStylesheets = /* @__PURE__ */ (() => {
    try {
      return new CSSStyleSheet(), typeof new CSSStyleSheet().replaceSync == "function";
    } catch {
    }
    return !1;
  })(), queuePending = !1, queueDomReads = [], queueDomWrites = [], queueTask = (O, _) => (H) => {
    O.push(H), queuePending || (queuePending = !0, plt.$flags$ & 4 ? nextTick(flush) : plt.raf(flush));
  }, consume = (O) => {
    for (let _ = 0; _ < O.length; _++)
      try {
        O[_](performance.now());
      } catch (H) {
        consoleError(H);
      }
    O.length = 0;
  }, flush = () => {
    consume(queueDomReads), consume(queueDomWrites), (queuePending = queueDomReads.length > 0) && plt.raf(flush);
  }, nextTick = (O) => promiseResolve().then(O), writeTask = /* @__PURE__ */ queueTask(queueDomWrites), EMPTY_OBJ = {}, SVG_NS = "http://www.w3.org/2000/svg", HTML_NS = "http://www.w3.org/1999/xhtml", isComplexType = (O) => (O = typeof O, O === "object" || O === "function");
  function queryNonceMetaTagContent(O) {
    var _, H, V;
    return (V = (H = (_ = O.head) == null ? void 0 : _.querySelector('meta[name="csp-nonce"]')) == null ? void 0 : H.getAttribute("content")) != null ? V : void 0;
  }
  var result_exports = {};
  __export(result_exports, {
    err: () => err,
    map: () => map,
    ok: () => ok,
    unwrap: () => unwrap$1,
    unwrapErr: () => unwrapErr
  });
  var ok = (O) => ({
    isOk: !0,
    isErr: !1,
    value: O
  }), err = (O) => ({
    isOk: !1,
    isErr: !0,
    value: O
  });
  function map(O, _) {
    if (O.isOk) {
      const H = _(O.value);
      return H instanceof Promise ? H.then((V) => ok(V)) : ok(H);
    }
    if (O.isErr) {
      const H = O.value;
      return err(H);
    }
    throw "should never get here";
  }
  var unwrap$1 = (O) => {
    if (O.isOk)
      return O.value;
    throw O.value;
  }, unwrapErr = (O) => {
    if (O.isErr)
      return O.value;
    throw O.value;
  }, h = (O, _, ...H) => {
    let V = null, q = null, G = null, J = !1, Z = !1;
    const te = [], ne = (ae) => {
      for (let oe = 0; oe < ae.length; oe++)
        V = ae[oe], Array.isArray(V) ? ne(V) : V != null && typeof V != "boolean" && ((J = typeof O != "function" && !isComplexType(V)) && (V = String(V)), J && Z ? te[te.length - 1].$text$ += V : te.push(J ? newVNode(null, V) : V), Z = J);
    };
    if (ne(H), _) {
      _.key && (q = _.key), _.name && (G = _.name);
      {
        const ae = _.className || _.class;
        ae && (_.class = typeof ae != "object" ? ae : Object.keys(ae).filter((oe) => ae[oe]).join(" "));
      }
    }
    if (typeof O == "function")
      return O(
        _ === null ? {} : _,
        te,
        vdomFnUtils
      );
    const ee = newVNode(O, null);
    return ee.$attrs$ = _, te.length > 0 && (ee.$children$ = te), ee.$key$ = q, ee.$name$ = G, ee;
  }, newVNode = (O, _) => {
    const H = {
      $flags$: 0,
      $tag$: O,
      $text$: _,
      $elm$: null,
      $children$: null
    };
    return H.$attrs$ = null, H.$key$ = null, H.$name$ = null, H;
  }, Host = {}, isHost = (O) => O && O.$tag$ === Host, vdomFnUtils = {
    forEach: (O, _) => O.map(convertToPublic).forEach(_),
    map: (O, _) => O.map(convertToPublic).map(_).map(convertToPrivate)
  }, convertToPublic = (O) => ({
    vattrs: O.$attrs$,
    vchildren: O.$children$,
    vkey: O.$key$,
    vname: O.$name$,
    vtag: O.$tag$,
    vtext: O.$text$
  }), convertToPrivate = (O) => {
    if (typeof O.vtag == "function") {
      const H = { ...O.vattrs };
      return O.vkey && (H.key = O.vkey), O.vname && (H.name = O.vname), h(O.vtag, H, ...O.vchildren || []);
    }
    const _ = newVNode(O.vtag, O.vtext);
    return _.$attrs$ = O.vattrs, _.$children$ = O.vchildren, _.$key$ = O.vkey, _.$name$ = O.vname, _;
  }, setAccessor = (O, _, H, V, q, G) => {
    if (H !== V) {
      let J = isMemberInElement(O, _), Z = _.toLowerCase();
      if (_ === "class") {
        const te = O.classList, ne = parseClassList(H), ee = parseClassList(V);
        O["s-si"] && ee.indexOf(O["s-si"]) < 0 && ee.push(O["s-si"]), te.remove(...ne.filter((ae) => ae && !ee.includes(ae))), te.add(...ee.filter((ae) => ae && !ne.includes(ae)));
      } else if (_ === "style") {
        for (const te in H)
          (!V || V[te] == null) && (te.includes("-") ? O.style.removeProperty(te) : O.style[te] = "");
        for (const te in V)
          (!H || V[te] !== H[te]) && (te.includes("-") ? O.style.setProperty(te, V[te]) : O.style[te] = V[te]);
      } else if (_ !== "key") if (_ === "ref")
        V && V(O);
      else if (!J && _[0] === "o" && _[1] === "n") {
        if (_[2] === "-" ? _ = _.slice(3) : isMemberInElement(win, Z) ? _ = Z.slice(2) : _ = Z[2] + _.slice(3), H || V) {
          const te = _.endsWith(CAPTURE_EVENT_SUFFIX);
          _ = _.replace(CAPTURE_EVENT_REGEX, ""), H && plt.rel(O, _, H, te), V && plt.ael(O, _, V, te);
        }
      } else {
        const te = isComplexType(V);
        if ((J || te && V !== null) && !q)
          try {
            if (O.tagName.includes("-"))
              O[_] = V;
            else {
              const ee = V ?? "";
              _ === "list" ? J = !1 : (H == null || O[_] != ee) && (typeof O.__lookupSetter__(_) == "function" ? O[_] = ee : O.setAttribute(_, ee));
            }
          } catch {
          }
        let ne = !1;
        Z !== (Z = Z.replace(/^xlink\:?/, "")) && (_ = Z, ne = !0), V == null || V === !1 ? (V !== !1 || O.getAttribute(_) === "") && (ne ? O.removeAttributeNS(XLINK_NS, _) : O.removeAttribute(_)) : (!J || G & 4 || q) && !te && (V = V === !0 ? "" : V, ne ? O.setAttributeNS(XLINK_NS, _, V) : O.setAttribute(_, V));
      }
    }
  }, parseClassListRegex = /\s/, parseClassList = (O) => O ? O.split(parseClassListRegex) : [], CAPTURE_EVENT_SUFFIX = "Capture", CAPTURE_EVENT_REGEX = new RegExp(CAPTURE_EVENT_SUFFIX + "$"), updateElement = (O, _, H) => {
    const V = _.$elm$.nodeType === 11 && _.$elm$.host ? _.$elm$.host : _.$elm$, q = O && O.$attrs$ || EMPTY_OBJ, G = _.$attrs$ || EMPTY_OBJ;
    for (const J of sortedAttrNames(Object.keys(q)))
      J in G || setAccessor(V, J, q[J], void 0, H, _.$flags$);
    for (const J of sortedAttrNames(Object.keys(G)))
      setAccessor(V, J, q[J], G[J], H, _.$flags$);
  };
  function sortedAttrNames(O) {
    return O.includes("ref") ? (
      // we need to sort these to ensure that `'ref'` is the last attr
      [...O.filter((_) => _ !== "ref"), "ref"]
    ) : (
      // no need to sort, return the original array
      O
    );
  }
  var contentRef, hostTagName, useNativeShadowDom = !1, checkSlotFallbackVisibility = !1, checkSlotRelocate = !1, isSvgMode = !1, createElm = (O, _, H, V) => {
    var q;
    const G = _.$children$[H];
    let J = 0, Z, te, ne;
    if (useNativeShadowDom || (checkSlotRelocate = !0, G.$tag$ === "slot" && (G.$flags$ |= G.$children$ ? (
      // slot element has fallback content
      // still create an element that "mocks" the slot element
      2
    ) : (
      // slot element does not have fallback content
      // create an html comment we'll use to always reference
      // where actual slot content should sit next to
      1
    ))), G.$text$ !== null)
      Z = G.$elm$ = doc$1.createTextNode(G.$text$);
    else if (G.$flags$ & 1)
      Z = G.$elm$ = doc$1.createTextNode("");
    else {
      if (isSvgMode || (isSvgMode = G.$tag$ === "svg"), Z = G.$elm$ = doc$1.createElementNS(
        isSvgMode ? SVG_NS : HTML_NS,
        !useNativeShadowDom && BUILD.slotRelocation && G.$flags$ & 2 ? "slot-fb" : G.$tag$
      ), isSvgMode && G.$tag$ === "foreignObject" && (isSvgMode = !1), updateElement(null, G, isSvgMode), Z.getRootNode().querySelector("body"), G.$children$)
        for (J = 0; J < G.$children$.length; ++J)
          te = createElm(O, G, J), te && Z.appendChild(te);
      G.$tag$ === "svg" ? isSvgMode = !1 : Z.tagName === "foreignObject" && (isSvgMode = !0);
    }
    return Z["s-hn"] = hostTagName, G.$flags$ & 3 && (Z["s-sr"] = !0, Z["s-cr"] = contentRef, Z["s-sn"] = G.$name$ || "", Z["s-rf"] = (q = G.$attrs$) == null ? void 0 : q.ref, ne = O && O.$children$ && O.$children$[H], ne && ne.$tag$ === G.$tag$ && O.$elm$ && putBackInOriginalLocation(O.$elm$, !1)), Z;
  }, putBackInOriginalLocation = (O, _) => {
    plt.$flags$ |= 1;
    const H = Array.from(O.__childNodes || O.childNodes);
    O["s-sr"] && BUILD.experimentalSlotFixes;
    for (let V = H.length - 1; V >= 0; V--) {
      const q = H[V];
      q["s-hn"] !== hostTagName && q["s-ol"] && (insertBefore(parentReferenceNode(q), q, referenceNode(q)), q["s-ol"].remove(), q["s-ol"] = void 0, q["s-sh"] = void 0, checkSlotRelocate = !0), _ && putBackInOriginalLocation(q, _);
    }
    plt.$flags$ &= -2;
  }, addVnodes = (O, _, H, V, q, G) => {
    let J = O["s-cr"] && O["s-cr"].parentNode || O, Z;
    for (; q <= G; ++q)
      V[q] && (Z = createElm(null, H, q), Z && (V[q].$elm$ = Z, insertBefore(J, Z, referenceNode(_))));
  }, removeVnodes = (O, _, H) => {
    for (let V = _; V <= H; ++V) {
      const q = O[V];
      if (q) {
        const G = q.$elm$;
        nullifyVNodeRefs(q), G && (checkSlotFallbackVisibility = !0, G["s-ol"] ? G["s-ol"].remove() : putBackInOriginalLocation(G, !0), G.remove());
      }
    }
  }, updateChildren = (O, _, H, V, q = !1) => {
    let G = 0, J = 0, Z = 0, te = 0, ne = _.length - 1, ee = _[0], ae = _[ne], oe = V.length - 1, se = V[0], ce = V[oe], le, ue;
    for (; G <= ne && J <= oe; )
      if (ee == null)
        ee = _[++G];
      else if (ae == null)
        ae = _[--ne];
      else if (se == null)
        se = V[++J];
      else if (ce == null)
        ce = V[--oe];
      else if (isSameVnode(ee, se, q))
        patch(ee, se, q), ee = _[++G], se = V[++J];
      else if (isSameVnode(ae, ce, q))
        patch(ae, ce, q), ae = _[--ne], ce = V[--oe];
      else if (isSameVnode(ee, ce, q))
        (ee.$tag$ === "slot" || ce.$tag$ === "slot") && putBackInOriginalLocation(ee.$elm$.parentNode, !1), patch(ee, ce, q), insertBefore(O, ee.$elm$, ae.$elm$.nextSibling), ee = _[++G], ce = V[--oe];
      else if (isSameVnode(ae, se, q))
        (ee.$tag$ === "slot" || ce.$tag$ === "slot") && putBackInOriginalLocation(ae.$elm$.parentNode, !1), patch(ae, se, q), insertBefore(O, ae.$elm$, ee.$elm$), ae = _[--ne], se = V[++J];
      else {
        for (Z = -1, te = G; te <= ne; ++te)
          if (_[te] && _[te].$key$ !== null && _[te].$key$ === se.$key$) {
            Z = te;
            break;
          }
        Z >= 0 ? (ue = _[Z], ue.$tag$ !== se.$tag$ ? le = createElm(_ && _[J], H, Z) : (patch(ue, se, q), _[Z] = void 0, le = ue.$elm$), se = V[++J]) : (le = createElm(_ && _[J], H, J), se = V[++J]), le && insertBefore(parentReferenceNode(ee.$elm$), le, referenceNode(ee.$elm$));
      }
    G > ne ? addVnodes(
      O,
      V[oe + 1] == null ? null : V[oe + 1].$elm$,
      H,
      V,
      J,
      oe
    ) : J > oe && removeVnodes(_, G, ne);
  }, isSameVnode = (O, _, H = !1) => O.$tag$ === _.$tag$ ? O.$tag$ === "slot" ? (
    // The component gets hydrated and no VDOM has been initialized.
    // Here the comparison can't happen as $name$ property is not set for `leftNode`.
    "$nodeId$" in O && H && // `leftNode` is not from type HTMLComment which would cause many
    // hydration comments to be removed
    O.$elm$.nodeType !== 8 ? !1 : O.$name$ === _.$name$
  ) : H ? !0 : O.$key$ === _.$key$ : !1, referenceNode = (O) => O && O["s-ol"] || O, parentReferenceNode = (O) => (O["s-ol"] ? O["s-ol"] : O).parentNode, patch = (O, _, H = !1) => {
    const V = _.$elm$ = O.$elm$, q = O.$children$, G = _.$children$, J = _.$tag$, Z = _.$text$;
    let te;
    Z === null ? (isSvgMode = J === "svg" ? !0 : J === "foreignObject" ? !1 : isSvgMode, J === "slot" && !useNativeShadowDom || updateElement(O, _, isSvgMode), q !== null && G !== null ? updateChildren(V, q, _, G, H) : G !== null ? (O.$text$ !== null && (V.textContent = ""), addVnodes(V, null, _, G, 0, G.length - 1)) : (
      // don't do this on initial render as it can cause non-hydrated content to be removed
      !H && BUILD.updatable && q !== null && removeVnodes(q, 0, q.length - 1)
    ), isSvgMode && J === "svg" && (isSvgMode = !1)) : (te = V["s-cr"]) ? te.parentNode.textContent = Z : O.$text$ !== Z && (V.data = Z);
  }, updateFallbackSlotVisibility = (O) => {
    const _ = O.__childNodes || O.childNodes;
    for (const H of _)
      if (H.nodeType === 1) {
        if (H["s-sr"]) {
          const V = H["s-sn"];
          H.hidden = !1;
          for (const q of _)
            if (q !== H) {
              if (q["s-hn"] !== H["s-hn"] || V !== "") {
                if (q.nodeType === 1 && (V === q.getAttribute("slot") || V === q["s-sn"]) || q.nodeType === 3 && V === q["s-sn"]) {
                  H.hidden = !0;
                  break;
                }
              } else if (V === q["s-sn"] && (q.nodeType === 1 || q.nodeType === 3 && q.textContent.trim() !== "")) {
                H.hidden = !0;
                break;
              }
            }
        }
        updateFallbackSlotVisibility(H);
      }
  }, relocateNodes = [], markSlotContentForRelocation = (O) => {
    let _, H, V;
    const q = O.__childNodes || O.childNodes;
    for (const G of q) {
      if (G["s-sr"] && (_ = G["s-cr"]) && _.parentNode) {
        H = _.parentNode.__childNodes || _.parentNode.childNodes;
        const J = G["s-sn"];
        for (V = H.length - 1; V >= 0; V--)
          if (_ = H[V], !_["s-cn"] && !_["s-nr"] && _["s-hn"] !== G["s-hn"])
            if (isNodeLocatedInSlot(_, J)) {
              let Z = relocateNodes.find((te) => te.$nodeToRelocate$ === _);
              checkSlotFallbackVisibility = !0, _["s-sn"] = _["s-sn"] || J, Z ? (Z.$nodeToRelocate$["s-sh"] = G["s-hn"], Z.$slotRefNode$ = G) : (_["s-sh"] = G["s-hn"], relocateNodes.push({
                $slotRefNode$: G,
                $nodeToRelocate$: _
              })), _["s-sr"] && relocateNodes.map((te) => {
                isNodeLocatedInSlot(te.$nodeToRelocate$, _["s-sn"]) && (Z = relocateNodes.find((ne) => ne.$nodeToRelocate$ === _), Z && !te.$slotRefNode$ && (te.$slotRefNode$ = Z.$slotRefNode$));
              });
            } else relocateNodes.some((Z) => Z.$nodeToRelocate$ === _) || relocateNodes.push({
              $nodeToRelocate$: _
            });
      }
      G.nodeType === 1 && markSlotContentForRelocation(G);
    }
  }, isNodeLocatedInSlot = (O, _) => O.nodeType === 1 ? O.getAttribute("slot") === null && _ === "" || O.getAttribute("slot") === _ : O["s-sn"] === _ ? !0 : _ === "", nullifyVNodeRefs = (O) => {
    O.$attrs$ && O.$attrs$.ref && O.$attrs$.ref(null), O.$children$ && O.$children$.map(nullifyVNodeRefs);
  }, insertBefore = (O, _, H) => O == null ? void 0 : O.insertBefore(_, H), renderVdom = (O, _, H = !1) => {
    var V, q, G, J;
    const Z = O.$hostElement$, te = O.$cmpMeta$, ne = O.$vnode$ || newVNode(null, null), ee = isHost(_) ? _ : h(null, null, _);
    if (hostTagName = Z.tagName, te.$attrsToReflect$ && (ee.$attrs$ = ee.$attrs$ || {}, te.$attrsToReflect$.map(
      ([ae, oe]) => ee.$attrs$[oe] = Z[ae]
    )), H && ee.$attrs$)
      for (const ae of Object.keys(ee.$attrs$))
        Z.hasAttribute(ae) && !["key", "ref", "style", "class"].includes(ae) && (ee.$attrs$[ae] = Z[ae]);
    ee.$tag$ = null, ee.$flags$ |= 4, O.$vnode$ = ee, ee.$elm$ = ne.$elm$ = Z, useNativeShadowDom = supportsShadow, contentRef = Z["s-cr"], checkSlotFallbackVisibility = !1, patch(ne, ee, H);
    {
      if (plt.$flags$ |= 1, checkSlotRelocate) {
        markSlotContentForRelocation(ee.$elm$);
        for (const ae of relocateNodes) {
          const oe = ae.$nodeToRelocate$;
          if (!oe["s-ol"]) {
            const se = doc$1.createTextNode("");
            se["s-nr"] = oe, insertBefore(oe.parentNode, oe["s-ol"] = se, oe);
          }
        }
        for (const ae of relocateNodes) {
          const oe = ae.$nodeToRelocate$, se = ae.$slotRefNode$;
          if (se) {
            const ce = se.parentNode;
            let le = se.nextSibling;
            {
              let ue = (V = oe["s-ol"]) == null ? void 0 : V.previousSibling;
              for (; ue; ) {
                let de = (q = ue["s-nr"]) != null ? q : null;
                if (de && de["s-sn"] === oe["s-sn"] && ce === de.parentNode) {
                  for (de = de.nextSibling; de === oe || de != null && de["s-sr"]; )
                    de = de == null ? void 0 : de.nextSibling;
                  if (!de || !de["s-nr"]) {
                    le = de;
                    break;
                  }
                }
                ue = ue.previousSibling;
              }
            }
            (!le && ce !== oe.parentNode || oe.nextSibling !== le) && oe !== le && (!oe["s-hn"] && oe["s-ol"] && (oe["s-hn"] = oe["s-ol"].parentNode.nodeName), insertBefore(ce, oe, le), oe.nodeType === 1 && (oe.hidden = (G = oe["s-ih"]) != null ? G : !1)), oe && typeof se["s-rf"] == "function" && se["s-rf"](oe);
          } else
            oe.nodeType === 1 && (H && (oe["s-ih"] = (J = oe.hidden) != null ? J : !1), oe.hidden = !0);
        }
      }
      checkSlotFallbackVisibility && updateFallbackSlotVisibility(ee.$elm$), plt.$flags$ &= -2, relocateNodes.length = 0;
    }
    contentRef = void 0;
  }, addSlotRelocateNode = (O, _, H, V) => {
    let q;
    if (O["s-ol"] && O["s-ol"].isConnected ? q = O["s-ol"] : (q = document.createTextNode(""), q["s-nr"] = O), !_["s-cr"] || !_["s-cr"].parentNode) return;
    const G = _["s-cr"].parentNode, J = G.__appendChild || G.appendChild;
    if (typeof V < "u") {
      q["s-oo"] = V;
      const Z = G.__childNodes || G.childNodes, te = [q];
      Z.forEach((ne) => {
        ne["s-nr"] && te.push(ne);
      }), te.sort((ne, ee) => !ne["s-oo"] || ne["s-oo"] < ee["s-oo"] ? -1 : !ee["s-oo"] || ee["s-oo"] < ne["s-oo"] ? 1 : 0), te.forEach((ne) => J.call(G, ne));
    } else
      J.call(G, q);
    O["s-ol"] = q, O["s-sh"] = _["s-hn"];
  }, createTime = (O, _ = "") => () => {
  }, uniqueTime = (O, _) => () => {
  }, initializeClientHydrate = (O, _, H, V) => {
    const q = createTime("hydrateClient", _), G = O.shadowRoot, J = [], Z = [], te = [], ne = null, ee = newVNode(_, null);
    ee.$elm$ = O, plt.$orgLocNodes$ || initializeDocumentHydrate(doc$1.body, plt.$orgLocNodes$ = /* @__PURE__ */ new Map()), O[HYDRATE_ID] = H, O.removeAttribute(HYDRATE_ID), V.$vnode$ = clientHydrate(
      ee,
      J,
      Z,
      ne,
      O,
      O,
      H,
      te
    );
    let ae = 0;
    const oe = J.length;
    let se;
    for (ae; ae < oe; ae++) {
      se = J[ae];
      const pe = se.$hostId$ + "." + se.$nodeId$, ye = plt.$orgLocNodes$.get(pe), ge = se.$elm$;
      G || (ge["s-hn"] = _.toUpperCase(), se.$tag$ === "slot" && (ge["s-cr"] = O["s-cr"])), ye && ye.isConnected && (G && ye["s-en"] === "" && ye.parentNode.insertBefore(ge, ye.nextSibling), ye.parentNode.removeChild(ye), G || (ge["s-oo"] = parseInt(se.$nodeId$))), plt.$orgLocNodes$.delete(pe);
    }
    const ce = [];
    let le = 0;
    const ue = te.length;
    let de, fe, he, me;
    for (le; le < ue; le++)
      if (de = te[le], !(!de || !de.length))
        for (he = de.length, fe = 0, fe; fe < he; fe++) {
          if (me = de[fe], ce[me.hostId] || (ce[me.hostId] = plt.$orgLocNodes$.get(me.hostId)), !ce[me.hostId]) continue;
          const pe = ce[me.hostId];
          if (!pe.shadowRoot || !G) {
            if (me.slot["s-cr"] = pe["s-cr"], !me.slot["s-cr"] && pe.shadowRoot)
              me.slot["s-cr"] = pe;
            else {
              const ye = pe.__childNodes || pe.childNodes;
              me.slot["s-cr"] = ye[0];
            }
            addSlotRelocateNode(me.node, me.slot, !1, me.node["s-oo"]);
          }
          pe.shadowRoot && me.node.parentElement !== pe && pe.appendChild(me.node);
        }
    V.$hostElement$ = O, q();
  }, clientHydrate = (O, _, H, V, q, G, J, Z = []) => {
    let te, ne, ee, ae;
    const oe = q["s-sc"];
    if (G.nodeType === 1) {
      if (te = G.getAttribute(HYDRATE_CHILD_ID), te && (ne = te.split("."), ne[0] === J || ne[0] === "0")) {
        ee = createSimpleVNode({
          $flags$: 0,
          $hostId$: ne[0],
          $nodeId$: ne[1],
          $depth$: ne[2],
          $index$: ne[3],
          $tag$: G.tagName.toLowerCase(),
          $elm$: G,
          // If we don't add the initial classes to the VNode, the first `vdom-render.ts` reconciliation will fail:
          // client side changes before componentDidLoad will be ignored, `set-accessor.ts` will just take the element's initial classes
          $attrs$: { class: G.className }
        }), _.push(ee), G.removeAttribute(HYDRATE_CHILD_ID), O.$children$ || (O.$children$ = []);
        const ce = ee.$elm$.getAttribute("s-sn");
        typeof ce == "string" && (ee.$tag$ === "slot-fb" && addSlot(
          ce,
          ne[2],
          ee,
          G,
          O,
          _,
          H,
          V,
          Z
        ), ee.$elm$["s-sn"] = ce, ee.$elm$.removeAttribute("s-sn")), ee.$index$ !== void 0 && (O.$children$[ee.$index$] = ee), oe && (G["s-si"] = oe), O = ee;
      }
      if (G.shadowRoot)
        for (ae = G.shadowRoot.childNodes.length - 1; ae >= 0; ae--)
          clientHydrate(
            O,
            _,
            H,
            V,
            q,
            G.shadowRoot.childNodes[ae],
            J,
            Z
          );
      const se = G.__childNodes || G.childNodes;
      for (ae = se.length - 1; ae >= 0; ae--)
        clientHydrate(
          O,
          _,
          H,
          V,
          q,
          se[ae],
          J,
          Z
        );
    } else if (G.nodeType === 8) {
      if (ne = G.nodeValue.split("."), ne[1] === J || ne[1] === "0") {
        if (te = ne[0], ee = createSimpleVNode({
          $hostId$: ne[1],
          $nodeId$: ne[2],
          $depth$: ne[3],
          $index$: ne[4] || "0",
          $elm$: G,
          $attrs$: null,
          $children$: null,
          $key$: null,
          $name$: null,
          $tag$: null,
          $text$: null
        }), te === TEXT_NODE_ID)
          ee.$elm$ = G.nextSibling, ee.$elm$ && ee.$elm$.nodeType === 3 && (ee.$text$ = ee.$elm$.textContent, _.push(ee), G.remove(), J === ee.$hostId$ && (O.$children$ || (O.$children$ = []), O.$children$[ee.$index$] = ee));
        else if (te === COMMENT_NODE_ID)
          ee.$elm$ = G.nextSibling, ee.$elm$ && ee.$elm$.nodeType === 8 && (_.push(ee), G.remove());
        else if (ee.$hostId$ === J)
          if (te === SLOT_NODE_ID) {
            ee.$tag$ = "slot";
            const se = G["s-sn"] = ee.$name$ = ne[5] || "";
            addSlot(
              se,
              ne[2],
              ee,
              G,
              O,
              _,
              H,
              V,
              Z
            );
          } else te === CONTENT_REF_ID && (q["s-cr"] = G, G["s-cn"] = !0);
      }
    } else if (O && O.$tag$ === "style") {
      const se = newVNode(null, G.textContent);
      se.$elm$ = G, se.$index$ = "0", O.$children$ = [se];
    }
    return O;
  }, initializeDocumentHydrate = (O, _) => {
    if (O.nodeType === 1) {
      const H = O[HYDRATE_ID] || O.getAttribute(HYDRATE_ID);
      H && _.set(H, O);
      let V = 0;
      if (O.shadowRoot)
        for (; V < O.shadowRoot.childNodes.length; V++)
          initializeDocumentHydrate(O.shadowRoot.childNodes[V], _);
      const q = O.__childNodes || O.childNodes;
      for (V = 0; V < q.length; V++)
        initializeDocumentHydrate(q[V], _);
    } else if (O.nodeType === 8) {
      const H = O.nodeValue.split(".");
      H[0] === ORG_LOCATION_ID && (_.set(H[1] + "." + H[2], O), O.nodeValue = "", O["s-en"] = H[3]);
    }
  }, createSimpleVNode = (O) => ({ ...{
    $flags$: 0,
    $hostId$: null,
    $nodeId$: null,
    $depth$: null,
    $index$: "0",
    $elm$: null,
    $attrs$: null,
    $children$: null,
    $key$: null,
    $name$: null,
    $tag$: null,
    $text$: null
  }, ...O });
  function addSlot(O, _, H, V, q, G, J, Z, te) {
    V["s-sr"] = !0;
    const ne = q != null && q.$elm$ ? q.$elm$["s-id"] || q.$elm$.getAttribute("s-id") : "";
    {
      const ee = H.$elm$, ae = ne && ne !== H.$hostId$ && q.$elm$.shadowRoot;
      addSlottedNodes(te, _, O, V, ae ? ne : H.$hostId$), ae && q.$elm$.insertBefore(ee, q.$elm$.children[0]), G.push(H);
    }
    J.push(H), q.$children$ || (q.$children$ = []), q.$children$[H.$index$] = H;
  }
  var addSlottedNodes = (O, _, H, V, q) => {
    let G = V.nextSibling;
    for (O[_] = O[_] || []; G && ((G.getAttribute && G.getAttribute("slot") || G["s-sn"]) === H || H === "" && !G["s-sn"] && (G.nodeType === 8 && G.nodeValue.indexOf(".") !== 1 || G.nodeType === 3)); )
      G["s-sn"] = H, O[_].push({ slot: V, node: G, hostId: q }), G = G.nextSibling;
  }, parsePropertyValue = (O, _) => O != null && !isComplexType(O) ? _ & 4 ? O === "false" ? !1 : O === "" || !!O : _ & 2 ? parseFloat(O) : _ & 1 ? String(O) : O : O, getElement = (O) => getHostRef(O).$hostElement$, createEvent = (O, _, H) => {
    const V = getElement(O);
    return {
      emit: (q) => emitEvent(V, _, {
        bubbles: !!(H & 4),
        composed: !!(H & 2),
        cancelable: !!(H & 1),
        detail: q
      })
    };
  }, emitEvent = (O, _, H) => {
    const V = plt.ce(_, H);
    return O.dispatchEvent(V), V;
  }, rootAppliedStyles = /* @__PURE__ */ new WeakMap(), registerStyle = (O, _, H) => {
    let V = styles.get(O);
    supportsConstructableStylesheets && H ? (V = V || new CSSStyleSheet(), typeof V == "string" ? V = _ : V.replaceSync(_)) : V = _, styles.set(O, V);
  }, addStyle = (O, _, H) => {
    var V;
    const q = getScopeId(_), G = styles.get(q);
    if (O = O.nodeType === 11 ? O : doc$1, G)
      if (typeof G == "string") {
        O = O.head || O;
        let J = rootAppliedStyles.get(O), Z;
        if (J || rootAppliedStyles.set(O, J = /* @__PURE__ */ new Set()), !J.has(q)) {
          if (O.host && (Z = O.querySelector(`[${HYDRATED_STYLE_ID}="${q}"]`)))
            Z.innerHTML = G;
          else {
            Z = doc$1.createElement("style"), Z.innerHTML = G;
            const te = (V = plt.$nonce$) != null ? V : queryNonceMetaTagContent(doc$1);
            if (te != null && Z.setAttribute("nonce", te), !(_.$flags$ & 1))
              if (O.nodeName === "HEAD") {
                const ne = O.querySelectorAll("link[rel=preconnect]"), ee = ne.length > 0 ? ne[ne.length - 1].nextSibling : O.querySelector("style");
                O.insertBefore(Z, ee);
              } else if ("host" in O)
                if (supportsConstructableStylesheets) {
                  const ne = new CSSStyleSheet();
                  ne.replaceSync(G), O.adoptedStyleSheets = [ne, ...O.adoptedStyleSheets];
                } else {
                  const ne = O.querySelector("style");
                  ne ? ne.innerHTML = G + ne.innerHTML : O.prepend(Z);
                }
              else
                O.append(Z);
            _.$flags$ & 1 && O.nodeName !== "HEAD" && O.insertBefore(Z, null);
          }
          _.$flags$ & 4 && (Z.innerHTML += SLOT_FB_CSS), J && J.add(q);
        }
      } else O.adoptedStyleSheets.includes(G) || (O.adoptedStyleSheets = [...O.adoptedStyleSheets, G]);
    return q;
  }, attachStyles = (O) => {
    const _ = O.$cmpMeta$, H = O.$hostElement$, V = createTime("attachStyles", _.$tagName$);
    addStyle(
      H.getRootNode(),
      _
    ), V();
  }, getScopeId = (O, _) => "sc-" + O.$tagName$, attachToAncestor = (O, _) => {
    _ && !O.$onRenderResolve$ && _["s-p"] && _["s-p"].push(new Promise((H) => O.$onRenderResolve$ = H));
  }, scheduleUpdate = (O, _) => {
    if (O.$flags$ |= 16, O.$flags$ & 4) {
      O.$flags$ |= 512;
      return;
    }
    return attachToAncestor(O, O.$ancestorComponent$), writeTask(() => dispatchHooks(O, _));
  }, dispatchHooks = (O, _) => {
    const H = O.$hostElement$, V = createTime("scheduleUpdate", O.$cmpMeta$.$tagName$), q = O.$lazyInstance$;
    if (!q)
      throw new Error(
        `Can't render component <${H.tagName.toLowerCase()} /> with invalid Stencil runtime! Make sure this imported component is compiled with a \`externalRuntime: true\` flag. For more information, please refer to https://stenciljs.com/docs/custom-elements#externalruntime`
      );
    let G;
    return _ && (O.$flags$ |= 256, O.$queuedListeners$ && (O.$queuedListeners$.map(([J, Z]) => safeCall(q, J, Z)), O.$queuedListeners$ = void 0), G = safeCall(q, "componentWillLoad")), G = enqueue(G, () => safeCall(q, "componentWillRender")), V(), enqueue(G, () => updateComponent(O, q, _));
  }, enqueue = (O, _) => isPromisey(O) ? O.then(_).catch((H) => {
    console.error(H), _();
  }) : _(), isPromisey = (O) => O instanceof Promise || O && O.then && typeof O.then == "function", updateComponent = async (O, _, H) => {
    var V;
    const q = O.$hostElement$, G = createTime("update", O.$cmpMeta$.$tagName$), J = q["s-rc"];
    H && attachStyles(O);
    const Z = createTime("render", O.$cmpMeta$.$tagName$);
    callRender(O, _, q, H), J && (J.map((te) => te()), q["s-rc"] = void 0), Z(), G();
    {
      const te = (V = q["s-p"]) != null ? V : [], ne = () => postUpdateComponent(O);
      te.length === 0 ? ne() : (Promise.all(te).then(ne), O.$flags$ |= 4, te.length = 0);
    }
  }, renderingRef = null, callRender = (O, _, H, V) => {
    try {
      renderingRef = _, _ = _.render && _.render(), O.$flags$ &= -17, O.$flags$ |= 2, renderVdom(O, _, V);
    } catch (q) {
      consoleError(q, O.$hostElement$);
    }
    return renderingRef = null, null;
  }, getRenderingRef = () => renderingRef, postUpdateComponent = (O) => {
    const _ = O.$cmpMeta$.$tagName$, H = O.$hostElement$, V = createTime("postUpdate", _), q = O.$lazyInstance$, G = O.$ancestorComponent$;
    safeCall(q, "componentDidRender"), O.$flags$ & 64 ? V() : (O.$flags$ |= 64, addHydratedFlag(H), safeCall(q, "componentDidLoad"), V(), O.$onReadyResolve$(H), G || appDidLoad()), O.$onInstanceResolve$(H), O.$onRenderResolve$ && (O.$onRenderResolve$(), O.$onRenderResolve$ = void 0), O.$flags$ & 512 && nextTick(() => scheduleUpdate(O, !1)), O.$flags$ &= -517;
  }, forceUpdate = (O) => {
    {
      const _ = getHostRef(O), H = _.$hostElement$.isConnected;
      return H && (_.$flags$ & 18) === 2 && scheduleUpdate(_, !1), H;
    }
  }, appDidLoad = (O) => {
    addHydratedFlag(doc$1.documentElement), nextTick(() => emitEvent(win, "appload", { detail: { namespace: NAMESPACE } }));
  }, safeCall = (O, _, H) => {
    if (O && O[_])
      try {
        return O[_](H);
      } catch (V) {
        consoleError(V);
      }
  }, addHydratedFlag = (O) => {
    var _;
    return O.classList.add((_ = BUILD.hydratedSelectorName) != null ? _ : "hydrated");
  }, getValue$1 = (O, _) => getHostRef(O).$instanceValues$.get(_), setValue = (O, _, H, V) => {
    const q = getHostRef(O);
    if (!q)
      throw new Error(
        `Couldn't find host element for "${V.$tagName$}" as it is unknown to this Stencil runtime. This usually happens when integrating a 3rd party Stencil component with another Stencil component or application. Please reach out to the maintainers of the 3rd party Stencil component or report this on the Stencil Discord server (https://chat.stenciljs.com) or comment on this similar [GitHub issue](https://github.com/ionic-team/stencil/issues/5457).`
      );
    const G = q.$hostElement$, J = q.$instanceValues$.get(_), Z = q.$flags$, te = q.$lazyInstance$;
    H = parsePropertyValue(H, V.$members$[_][0]);
    const ne = Number.isNaN(J) && Number.isNaN(H), ee = H !== J && !ne;
    if ((!(Z & 8) || J === void 0) && ee && (q.$instanceValues$.set(_, H), te)) {
      if (V.$watchers$ && Z & 128) {
        const ae = V.$watchers$[_];
        ae && ae.map((oe) => {
          try {
            te[oe](H, J, _);
          } catch (se) {
            consoleError(se, G);
          }
        });
      }
      (Z & 18) === 2 && scheduleUpdate(q, !1);
    }
  }, proxyComponent = (O, _, H) => {
    var V, q;
    const G = O.prototype;
    if (_.$members$ || _.$watchers$ || O.watchers) {
      O.watchers && !_.$watchers$ && (_.$watchers$ = O.watchers);
      const J = Object.entries((V = _.$members$) != null ? V : {});
      if (J.map(([Z, [te]]) => {
        if (te & 31 || H & 2 && te & 32) {
          if (!(te & 2048))
            Object.defineProperty(G, Z, {
              get() {
                return getValue$1(this, Z);
              },
              set(ne) {
                setValue(this, Z, ne, _);
              },
              configurable: !0,
              enumerable: !0
            });
          else if (H & 1 && te & 2048 && (Object.defineProperty(G, Z, {
            get() {
              const ne = getHostRef(this), ee = ne ? ne.$lazyInstance$ : G;
              if (ee)
                return ee[Z];
            },
            configurable: !0,
            enumerable: !0
          }), te & 4096)) {
            const ne = Object.getOwnPropertyDescriptor(G, Z).set;
            Object.defineProperty(G, Z, {
              set(ee) {
                const ae = getHostRef(this);
                if (ne) {
                  const se = ae.$hostElement$[Z];
                  !ae.$instanceValues$.get(Z) && se && ae.$instanceValues$.set(Z, se), ne.apply(this, [parsePropertyValue(ee, _.$members$[Z][0])]), setValue(this, Z, ae.$hostElement$[Z], _);
                  return;
                }
                if (!ae) return;
                const oe = () => {
                  const se = ae.$lazyInstance$[Z];
                  !ae.$instanceValues$.get(Z) && se && ae.$instanceValues$.set(Z, se), ae.$lazyInstance$[Z] = parsePropertyValue(ee, _.$members$[Z][0]), setValue(this, Z, ae.$lazyInstance$[Z], _);
                };
                ae.$lazyInstance$ ? oe() : ae.$onReadyPromise$.then(() => oe());
              }
            });
          }
        } else H & 1 && te & 64 && Object.defineProperty(G, Z, {
          value(...ne) {
            var ee;
            const ae = getHostRef(this);
            return (ee = ae == null ? void 0 : ae.$onInstancePromise$) == null ? void 0 : ee.then(() => {
              var oe;
              return (oe = ae.$lazyInstance$) == null ? void 0 : oe[Z](...ne);
            });
          }
        });
      }), H & 1) {
        const Z = /* @__PURE__ */ new Map();
        G.attributeChangedCallback = function(te, ne, ee) {
          plt.jmp(() => {
            var ae;
            const oe = Z.get(te);
            if (this.hasOwnProperty(oe))
              ee = this[oe], delete this[oe];
            else {
              if (G.hasOwnProperty(oe) && typeof this[oe] == "number" && // cast type to number to avoid TS compiler issues
              this[oe] == ee)
                return;
              if (oe == null) {
                const ce = getHostRef(this), le = ce == null ? void 0 : ce.$flags$;
                if (le && !(le & 8) && le & 128 && ee !== ne) {
                  const ue = ce.$lazyInstance$, de = (ae = _.$watchers$) == null ? void 0 : ae[te];
                  de == null || de.forEach((fe) => {
                    ue[fe] != null && ue[fe].call(ue, ee, ne, te);
                  });
                }
                return;
              }
            }
            const se = Object.getOwnPropertyDescriptor(G, oe);
            (!se.get || se.set) && (this[oe] = ee === null && typeof this[oe] == "boolean" ? !1 : ee);
          });
        }, O.observedAttributes = Array.from(
          /* @__PURE__ */ new Set([
            ...Object.keys((q = _.$watchers$) != null ? q : {}),
            ...J.filter(
              ([te, ne]) => ne[0] & 15
              /* HasAttribute */
            ).map(([te, ne]) => {
              var ee;
              const ae = ne[1] || te;
              return Z.set(ae, te), ne[0] & 512 && ((ee = _.$attrsToReflect$) == null || ee.push([te, ae])), ae;
            })
          ])
        );
      }
    }
    return O;
  }, initializeComponent = async (O, _, H, V) => {
    let q;
    if (!(_.$flags$ & 32)) {
      if (_.$flags$ |= 32, H.$lazyBundleId$) {
        const te = loadModule(H);
        if (te && "then" in te) {
          const ee = uniqueTime();
          q = await te, ee();
        } else
          q = te;
        if (!q)
          throw new Error(`Constructor for "${H.$tagName$}#${_.$modeName$}" was not found`);
        q.isProxied || (H.$watchers$ = q.watchers, proxyComponent(
          q,
          H,
          2
          /* proxyState */
        ), q.isProxied = !0);
        const ne = createTime("createInstance", H.$tagName$);
        _.$flags$ |= 8;
        try {
          new q(_);
        } catch (ee) {
          consoleError(ee);
        }
        _.$flags$ &= -9, _.$flags$ |= 128, ne(), fireConnectedCallback(_.$lazyInstance$);
      } else {
        q = O.constructor;
        const te = O.localName;
        customElements.whenDefined(te).then(
          () => _.$flags$ |= 128
          /* isWatchReady */
        );
      }
      if (q && q.style) {
        let te;
        typeof q.style == "string" && (te = q.style);
        const ne = getScopeId(H);
        if (!styles.has(ne)) {
          const ee = createTime("registerStyles", H.$tagName$);
          registerStyle(ne, te, !!(H.$flags$ & 1)), ee();
        }
      }
    }
    const G = _.$ancestorComponent$, J = () => scheduleUpdate(_, !0);
    G && G["s-rc"] ? G["s-rc"].push(J) : J();
  }, fireConnectedCallback = (O) => {
    safeCall(O, "connectedCallback");
  }, connectedCallback = (O) => {
    if (!(plt.$flags$ & 1)) {
      const _ = getHostRef(O), H = _.$cmpMeta$, V = createTime("connectedCallback", H.$tagName$);
      if (_.$flags$ & 1)
        addHostEventListeners(O, _, H.$listeners$), _ != null && _.$lazyInstance$ ? fireConnectedCallback(_.$lazyInstance$) : _ != null && _.$onReadyPromise$ && _.$onReadyPromise$.then(() => fireConnectedCallback(_.$lazyInstance$));
      else {
        _.$flags$ |= 1;
        let q;
        q = O.getAttribute(HYDRATE_ID), q && initializeClientHydrate(O, H.$tagName$, q, _), q || // TODO(STENCIL-854): Remove code related to legacy shadowDomShim field
        H.$flags$ & 12 && setContentReference(O);
        {
          let G = O;
          for (; G = G.parentNode || G.host; )
            if (G.nodeType === 1 && G.hasAttribute("s-id") && G["s-p"] || G["s-p"]) {
              attachToAncestor(_, _.$ancestorComponent$ = G);
              break;
            }
        }
        H.$members$ && Object.entries(H.$members$).map(([G, [J]]) => {
          if (J & 31 && O.hasOwnProperty(G)) {
            const Z = O[G];
            delete O[G], O[G] = Z;
          }
        }), nextTick(() => initializeComponent(O, _, H));
      }
      V();
    }
  }, setContentReference = (O) => {
    const _ = O["s-cr"] = doc$1.createComment(
      ""
    );
    _["s-cn"] = !0, insertBefore(O, _, O.firstChild);
  }, disconnectInstance = (O) => {
    safeCall(O, "disconnectedCallback");
  }, disconnectedCallback = async (O) => {
    if (!(plt.$flags$ & 1)) {
      const _ = getHostRef(O);
      _.$rmListeners$ && (_.$rmListeners$.map((H) => H()), _.$rmListeners$ = void 0), _ != null && _.$lazyInstance$ ? disconnectInstance(_.$lazyInstance$) : _ != null && _.$onReadyPromise$ && _.$onReadyPromise$.then(() => disconnectInstance(_.$lazyInstance$));
    }
  }, bootstrapLazy = (O, _ = {}) => {
    var H;
    const V = createTime(), q = [], G = _.exclude || [], J = win.customElements, Z = doc$1.head, te = /* @__PURE__ */ Z.querySelector("meta[charset]"), ne = /* @__PURE__ */ doc$1.createElement("style"), ee = [];
    let ae, oe = !0;
    Object.assign(plt, _), plt.$resourcesUrl$ = new URL(_.resourcesUrl || "./", doc$1.baseURI).href, plt.$flags$ |= 2;
    let se = !1;
    if (O.map((ce) => {
      ce[1].map((le) => {
        var ue;
        const de = {
          $flags$: le[0],
          $tagName$: le[1],
          $members$: le[2],
          $listeners$: le[3]
        };
        de.$flags$ & 4 && (se = !0), de.$members$ = le[2], de.$listeners$ = le[3], de.$attrsToReflect$ = [], de.$watchers$ = (ue = le[4]) != null ? ue : {};
        const fe = de.$tagName$, he = class extends HTMLElement {
          // StencilLazyHost
          constructor(me) {
            super(me), this.hasRegisteredEventListeners = !1, me = this, registerHost(me, de);
          }
          connectedCallback() {
            const me = getHostRef(this);
            this.hasRegisteredEventListeners || (this.hasRegisteredEventListeners = !0, addHostEventListeners(this, me, de.$listeners$)), ae && (clearTimeout(ae), ae = null), oe ? ee.push(this) : plt.jmp(() => connectedCallback(this));
          }
          disconnectedCallback() {
            plt.jmp(() => disconnectedCallback(this));
          }
          componentOnReady() {
            return getHostRef(this).$onReadyPromise$;
          }
        };
        de.$lazyBundleId$ = ce[0], !G.includes(fe) && !J.get(fe) && (q.push(fe), J.define(
          fe,
          proxyComponent(
            he,
            de,
            1
            /* isElementConstructor */
          )
        ));
      });
    }), q.length > 0 && (se && (ne.textContent += SLOT_FB_CSS), ne.innerHTML.length)) {
      ne.setAttribute("data-styles", "");
      const ce = (H = plt.$nonce$) != null ? H : queryNonceMetaTagContent(doc$1);
      ce != null && ne.setAttribute("nonce", ce), Z.insertBefore(ne, te ? te.nextSibling : Z.firstChild);
    }
    oe = !1, ee.length ? ee.map((ce) => ce.connectedCallback()) : plt.jmp(() => ae = setTimeout(appDidLoad, 30)), V();
  }, addHostEventListeners = (O, _, H, V) => {
    H && H.map(([q, G, J]) => {
      const Z = getHostListenerTarget(O, q), te = hostListenerProxy(_, J), ne = hostListenerOpts(q);
      plt.ael(Z, G, te, ne), (_.$rmListeners$ = _.$rmListeners$ || []).push(() => plt.rel(Z, G, te, ne));
    });
  }, hostListenerProxy = (O, _) => (H) => {
    var V;
    try {
      O.$flags$ & 256 ? (V = O.$lazyInstance$) == null || V[_](H) : (O.$queuedListeners$ = O.$queuedListeners$ || []).push([_, H]);
    } catch (q) {
      consoleError(q);
    }
  }, getHostListenerTarget = (O, _) => _ & 4 ? doc$1 : O, hostListenerOpts = (O) => supportsListenerOptions ? {
    passive: (O & 1) !== 0,
    capture: (O & 2) !== 0
  } : (O & 2) !== 0;
  /*!
   * Built by Revolist OU ❤️
   */
  const globalFn = () => {
  }, globalScripts = globalFn;
  /*!
   * Built by Revolist OU ❤️
   */
  const defineCustomElements = async (O, _) => {
    if (!(typeof window > "u"))
      return await globalScripts(), bootstrapLazy([["revo-grid", [[4, "revo-grid", { rowHeaders: [4, "row-headers"], frameSize: [2, "frame-size"], rowSize: [2, "row-size"], colSize: [2, "col-size"], range: [4], readonly: [4], resize: [4], canFocus: [4, "can-focus"], useClipboard: [4, "use-clipboard"], columns: [16], source: [16], pinnedTopSource: [16], pinnedBottomSource: [16], rowDefinitions: [16], editors: [16], applyOnClose: [4, "apply-on-close"], plugins: [16], columnTypes: [16], theme: [1537], rowClass: [513, "row-class"], autoSizeColumn: [4, "auto-size-column"], filter: [4], sorting: [16], focusTemplate: [16], canMoveColumns: [4, "can-move-columns"], trimmedRows: [16], exporting: [4], grouping: [16], stretch: [8], additionalData: [8, "additional-data"], disableVirtualX: [4, "disable-virtual-x"], disableVirtualY: [4, "disable-virtual-y"], hideAttribution: [4, "hide-attribution"], jobsBeforeRender: [16], registerVNode: [16], accessible: [4], refresh: [64], setDataAt: [64], scrollToRow: [64], scrollToColumnIndex: [64], scrollToColumnProp: [64], updateColumns: [64], addTrimmed: [64], scrollToCoordinate: [64], setCellEdit: [64], setCellsFocus: [64], getSource: [64], getVisibleSource: [64], getSourceStore: [64], getColumnStore: [64], updateColumnSorting: [64], clearSorting: [64], getColumns: [64], clearFocus: [64], getPlugins: [64], getFocused: [64], getContentSize: [64], getSelectedRange: [64], refreshExtraElements: [64] }, [[5, "touchstart", "mousedownHandle"], [5, "mousedown", "mousedownHandle"], [5, "touchend", "mouseupHandle"], [5, "mouseup", "mouseupHandle"], [0, "rowdragstartinit", "onRowDragStarted"], [0, "rowdragendinit", "onRowDragEnd"], [0, "roworderchange", "onRowOrderChange"], [0, "rowdragmoveinit", "onRowDrag"], [0, "rowdragmousemove", "onRowMouseMove"], [0, "celleditapply", "onCellEdit"], [0, "rangeeditapply", "onRangeEdit"], [0, "selectionchangeinit", "onRangeChanged"], [0, "rowdropinit", "onRowDropped"], [0, "beforeheaderclick", "onHeaderClick"], [0, "beforecellfocusinit", "onCellFocus"]], { columnTypes: ["columnTypesChanged"], columns: ["columnChanged"], disableVirtualX: ["disableVirtualXChanged"], rowSize: ["rowSizeChanged"], theme: ["themeChanged"], source: ["dataSourceChanged"], pinnedBottomSource: ["dataSourceChanged"], pinnedTopSource: ["dataSourceChanged"], disableVirtualY: ["disableVirtualYChanged"], rowDefinitions: ["rowDefChanged"], trimmedRows: ["trimmedRowsChanged"], grouping: ["groupingChanged"], stretch: ["applyStretch"], filter: ["applyFilter"], sorting: ["applySorting"], rowHeaders: ["rowHeadersChange"], registerVNode: ["registerOutsideVNodes"], additionalData: ["additionalDataChanged"], plugins: ["pluginsChanged"] }]]], ["revogr-filter-panel", [[4, "revogr-filter-panel", { filterNames: [16], filterEntities: [16], filterCaptions: [16], disableDynamicFiltering: [4, "disable-dynamic-filtering"], isFilterIdSet: [32], filterId: [32], currentFilterId: [32], currentFilterType: [32], changes: [32], filterItems: [32], show: [64], getChanges: [64] }, [[5, "mousedown", "onMouseDown"]]]]], ["revogr-clipboard_3", [[0, "revogr-clipboard", { readonly: [4], doCopy: [64] }, [[4, "paste", "onPaste"], [4, "copy", "copyStarted"], [4, "cut", "cutStarted"]]], [0, "revogr-edit", { editCell: [16], column: [16], editor: [16], saveOnClose: [4, "save-on-close"], additionalData: [8, "additional-data"], cancelChanges: [64], beforeDisconnect: [64] }], [0, "revogr-order-editor", { parent: [16], dimensionRow: [16], dimensionCol: [16], dataStore: [16], rowType: [1, "row-type"], dragStart: [64], endOrder: [64], clearOrder: [64] }]]], ["revogr-data_4", [[4, "revogr-data", { readonly: [4], range: [4], rowClass: [1, "row-class"], additionalData: [8, "additional-data"], rowSelectionStore: [16], viewportRow: [16], viewportCol: [16], dimensionRow: [16], colData: [16], dataStore: [16], type: [513], colType: [513, "col-type"], jobsBeforeRender: [16], providers: [32], updateCell: [64] }, null, { dataStore: ["onDataStoreChange"], colData: ["onColDataChange"] }], [0, "revogr-header", { viewportCol: [16], dimensionCol: [16], selectionStore: [16], groups: [16], groupingDepth: [2, "grouping-depth"], readonly: [4], canResize: [4, "can-resize"], resizeHandler: [16], colData: [16], columnFilter: [4, "column-filter"], type: [1], additionalData: [8, "additional-data"] }], [4, "revogr-viewport-scroll", { rowHeader: [4, "row-header"], contentWidth: [2, "content-width"], contentHeight: [2, "content-height"], colType: [1, "col-type"], setScroll: [64], changeScroll: [64], applyScroll: [64] }, [[0, "mousewheel-vertical", "mousewheelVertical"], [0, "mousewheel-horizontal", "mousewheelHorizontal"], [0, "scroll-coordinate", "scrollApply"]]], [0, "vnode-html", { redraw: [16] }]]], ["revogr-attribution_7", [[0, "revogr-row-headers", { height: [2], dataPorts: [16], headerProp: [16], rowClass: [1, "row-class"], resize: [4], rowHeaderColumn: [16], additionalData: [8, "additional-data"], jobsBeforeRender: [16] }], [4, "revogr-overlay-selection", { readonly: [4], range: [4], canDrag: [4, "can-drag"], useClipboard: [4, "use-clipboard"], selectionStore: [16], dimensionRow: [16], dimensionCol: [16], dataStore: [16], colData: [16], lastCell: [16], editors: [16], applyChangesOnClose: [4, "apply-changes-on-close"], additionalData: [8, "additional-data"], isMobileDevice: [4, "is-mobile-device"] }, [[5, "touchmove", "onMouseMove"], [5, "mousemove", "onMouseMove"], [5, "touchend", "onMouseUp"], [5, "mouseup", "onMouseUp"], [5, "mouseleave", "onMouseUp"], [0, "dragstartcell", "onCellDrag"], [4, "keyup", "onKeyUp"], [4, "keydown", "onKeyDown"]], { selectionStore: ["selectionServiceSet"], dimensionRow: ["createAutoFillService"], dimensionCol: ["createAutoFillService"], dataStore: ["columnServiceSet"], colData: ["columnServiceSet"] }], [0, "revogr-attribution"], [4, "revogr-focus", { colType: [1, "col-type"], rowType: [1, "row-type"], selectionStore: [16], dimensionRow: [16], dimensionCol: [16], dataStore: [16], colData: [16], focusTemplate: [16] }], [0, "revogr-scroll-virtual", { dimension: [1], viewportStore: [16], dimensionStore: [16], setScroll: [64], changeScroll: [64] }], [0, "revogr-temp-range", { selectionStore: [16], dimensionRow: [16], dimensionCol: [16] }], [0, "revogr-extra", { nodes: [16], update: [32], refresh: [64] }]]]], _);
  };
  /*!
   * Built by Revolist OU ❤️
   */
  (function() {
    if (typeof window < "u" && window.Reflect !== void 0 && window.customElements !== void 0) {
      var O = HTMLElement;
      window.HTMLElement = function() {
        return Reflect.construct(O, [], this.constructor);
      }, HTMLElement.prototype = O.prototype, HTMLElement.prototype.constructor = HTMLElement, Object.setPrototypeOf(HTMLElement, O);
    }
  })();
  /*!
   * Built by Revolist OU ❤️
   */
  var freeGlobal = typeof global == "object" && global && global.Object === Object && global;
  const freeGlobal$1 = freeGlobal;
  var freeSelf = typeof self == "object" && self && self.Object === Object && self, root = freeGlobal$1 || freeSelf || Function("return this")();
  const root$1 = root;
  var Symbol$1 = root$1.Symbol;
  const Symbol$1$1 = Symbol$1;
  var objectProto$1$1 = Object.prototype, hasOwnProperty$9 = objectProto$1$1.hasOwnProperty, nativeObjectToString$1 = objectProto$1$1.toString, symToStringTag$1 = Symbol$1$1 ? Symbol$1$1.toStringTag : void 0;
  function getRawTag(O) {
    var _ = hasOwnProperty$9.call(O, symToStringTag$1), H = O[symToStringTag$1];
    try {
      O[symToStringTag$1] = void 0;
      var V = !0;
    } catch {
    }
    var q = nativeObjectToString$1.call(O);
    return V && (_ ? O[symToStringTag$1] = H : delete O[symToStringTag$1]), q;
  }
  var objectProto$a = Object.prototype, nativeObjectToString = objectProto$a.toString;
  function objectToString(O) {
    return nativeObjectToString.call(O);
  }
  var nullTag = "[object Null]", undefinedTag = "[object Undefined]", symToStringTag = Symbol$1$1 ? Symbol$1$1.toStringTag : void 0;
  function baseGetTag(O) {
    return O == null ? O === void 0 ? undefinedTag : nullTag : symToStringTag && symToStringTag in Object(O) ? getRawTag(O) : objectToString(O);
  }
  function isObject$1(O) {
    var _ = typeof O;
    return O != null && (_ == "object" || _ == "function");
  }
  function isObjectLike(O) {
    return O != null && typeof O == "object";
  }
  var symbolTag$1 = "[object Symbol]";
  function isSymbol(O) {
    return typeof O == "symbol" || isObjectLike(O) && baseGetTag(O) == symbolTag$1;
  }
  var reWhitespace = /\s/;
  function trimmedEndIndex(O) {
    for (var _ = O.length; _-- && reWhitespace.test(O.charAt(_)); )
      ;
    return _;
  }
  var reTrimStart = /^\s+/;
  function baseTrim(O) {
    return O && O.slice(0, trimmedEndIndex(O) + 1).replace(reTrimStart, "");
  }
  var NAN = NaN, reIsBadHex = /^[-+]0x[0-9a-f]+$/i, reIsBinary = /^0b[01]+$/i, reIsOctal = /^0o[0-7]+$/i, freeParseInt = parseInt;
  function toNumber(O) {
    if (typeof O == "number")
      return O;
    if (isSymbol(O))
      return NAN;
    if (isObject$1(O)) {
      var _ = typeof O.valueOf == "function" ? O.valueOf() : O;
      O = isObject$1(_) ? _ + "" : _;
    }
    if (typeof O != "string")
      return O === 0 ? O : +O;
    O = baseTrim(O);
    var H = reIsBinary.test(O);
    return H || reIsOctal.test(O) ? freeParseInt(O.slice(2), H ? 2 : 8) : reIsBadHex.test(O) ? NAN : +O;
  }
  var now = function() {
    return root$1.Date.now();
  };
  const now$1 = now;
  var FUNC_ERROR_TEXT$2 = "Expected a function", nativeMax$1 = Math.max, nativeMin$1 = Math.min;
  function debounce$1(O, _, H) {
    var V, q, G, J, Z, te, ne = 0, ee = !1, ae = !1, oe = !0;
    if (typeof O != "function")
      throw new TypeError(FUNC_ERROR_TEXT$2);
    _ = toNumber(_) || 0, isObject$1(H) && (ee = !!H.leading, ae = "maxWait" in H, G = ae ? nativeMax$1(toNumber(H.maxWait) || 0, _) : G, oe = "trailing" in H ? !!H.trailing : oe);
    function se(ye) {
      var ge = V, ve = q;
      return V = q = void 0, ne = ye, J = O.apply(ve, ge), J;
    }
    function ce(ye) {
      return ne = ye, Z = setTimeout(de, _), ee ? se(ye) : J;
    }
    function le(ye) {
      var ge = ye - te, ve = ye - ne, be = _ - ge;
      return ae ? nativeMin$1(be, G - ve) : be;
    }
    function ue(ye) {
      var ge = ye - te, ve = ye - ne;
      return te === void 0 || ge >= _ || ge < 0 || ae && ve >= G;
    }
    function de() {
      var ye = now$1();
      if (ue(ye))
        return fe(ye);
      Z = setTimeout(de, le(ye));
    }
    function fe(ye) {
      return Z = void 0, oe && V ? se(ye) : (V = q = void 0, J);
    }
    function he() {
      Z !== void 0 && clearTimeout(Z), ne = 0, V = te = q = Z = void 0;
    }
    function me() {
      return Z === void 0 ? J : fe(now$1());
    }
    function pe() {
      var ye = now$1(), ge = ue(ye);
      if (V = arguments, q = this, te = ye, ge) {
        if (Z === void 0)
          return ce(te);
        if (ae)
          return clearTimeout(Z), Z = setTimeout(de, _), se(te);
      }
      return Z === void 0 && (Z = setTimeout(de, _)), J;
    }
    return pe.cancel = he, pe.flush = me, pe;
  }
  /*!
   * Built by Revolist OU ❤️
   */
  const proxyPlugin = (O) => ({
    /**
     * Set the value of a property in the store.
     * If the key is 'proxyItems' it will filter the items in the data source according to the new value.
     * The new value should be an array of numbers representing the indexes of the items that should be visible.
     * The method will return a new array of numbers with the indexes of the items that should be visible.
     * The method will also update the 'items' property of the store with the new array.
     */
    set(_, H) {
      if (_ !== "proxyItems")
        return;
      const V = O.get("items").reduce((G, J) => (G.add(J), G), /* @__PURE__ */ new Set()), q = H.reduce((G, J) => (V.has(J) && G.push(J), G), []);
      O.set("items", q);
    }
  });
  function baseFindIndex(O, _, H, V) {
    for (var q = O.length, G = H + -1; ++G < q; )
      if (_(O[G], G, O))
        return G;
    return -1;
  }
  function listCacheClear() {
    this.__data__ = [], this.size = 0;
  }
  function eq$1(O, _) {
    return O === _ || O !== O && _ !== _;
  }
  function assocIndexOf(O, _) {
    for (var H = O.length; H--; )
      if (eq$1(O[H][0], _))
        return H;
    return -1;
  }
  var arrayProto = Array.prototype, splice = arrayProto.splice;
  function listCacheDelete(O) {
    var _ = this.__data__, H = assocIndexOf(_, O);
    if (H < 0)
      return !1;
    var V = _.length - 1;
    return H == V ? _.pop() : splice.call(_, H, 1), --this.size, !0;
  }
  function listCacheGet(O) {
    var _ = this.__data__, H = assocIndexOf(_, O);
    return H < 0 ? void 0 : _[H][1];
  }
  function listCacheHas(O) {
    return assocIndexOf(this.__data__, O) > -1;
  }
  function listCacheSet(O, _) {
    var H = this.__data__, V = assocIndexOf(H, O);
    return V < 0 ? (++this.size, H.push([O, _])) : H[V][1] = _, this;
  }
  function ListCache(O) {
    var _ = -1, H = O == null ? 0 : O.length;
    for (this.clear(); ++_ < H; ) {
      var V = O[_];
      this.set(V[0], V[1]);
    }
  }
  ListCache.prototype.clear = listCacheClear;
  ListCache.prototype.delete = listCacheDelete;
  ListCache.prototype.get = listCacheGet;
  ListCache.prototype.has = listCacheHas;
  ListCache.prototype.set = listCacheSet;
  function stackClear() {
    this.__data__ = new ListCache(), this.size = 0;
  }
  function stackDelete(O) {
    var _ = this.__data__, H = _.delete(O);
    return this.size = _.size, H;
  }
  function stackGet(O) {
    return this.__data__.get(O);
  }
  function stackHas(O) {
    return this.__data__.has(O);
  }
  var asyncTag = "[object AsyncFunction]", funcTag$1 = "[object Function]", genTag = "[object GeneratorFunction]", proxyTag = "[object Proxy]";
  function isFunction$1(O) {
    if (!isObject$1(O))
      return !1;
    var _ = baseGetTag(O);
    return _ == funcTag$1 || _ == genTag || _ == asyncTag || _ == proxyTag;
  }
  var coreJsData = root$1["__core-js_shared__"];
  const coreJsData$1 = coreJsData;
  var maskSrcKey = function() {
    var O = /[^.]+$/.exec(coreJsData$1 && coreJsData$1.keys && coreJsData$1.keys.IE_PROTO || "");
    return O ? "Symbol(src)_1." + O : "";
  }();
  function isMasked(O) {
    return !!maskSrcKey && maskSrcKey in O;
  }
  var funcProto$1 = Function.prototype, funcToString$1 = funcProto$1.toString;
  function toSource(O) {
    if (O != null) {
      try {
        return funcToString$1.call(O);
      } catch {
      }
      try {
        return O + "";
      } catch {
      }
    }
    return "";
  }
  var reRegExpChar = /[\\^$.*+?()[\]{}|]/g, reIsHostCtor = /^\[object .+?Constructor\]$/, funcProto = Function.prototype, objectProto$9 = Object.prototype, funcToString = funcProto.toString, hasOwnProperty$7 = objectProto$9.hasOwnProperty, reIsNative = RegExp(
    "^" + funcToString.call(hasOwnProperty$7).replace(reRegExpChar, "\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, "$1.*?") + "$"
  );
  function baseIsNative(O) {
    if (!isObject$1(O) || isMasked(O))
      return !1;
    var _ = isFunction$1(O) ? reIsNative : reIsHostCtor;
    return _.test(toSource(O));
  }
  function getValue(O, _) {
    return O == null ? void 0 : O[_];
  }
  function getNative(O, _) {
    var H = getValue(O, _);
    return baseIsNative(H) ? H : void 0;
  }
  var Map$1$1 = getNative(root$1, "Map");
  const Map$2 = Map$1$1;
  var nativeCreate = getNative(Object, "create");
  const nativeCreate$1 = nativeCreate;
  function hashClear() {
    this.__data__ = nativeCreate$1 ? nativeCreate$1(null) : {}, this.size = 0;
  }
  function hashDelete(O) {
    var _ = this.has(O) && delete this.__data__[O];
    return this.size -= _ ? 1 : 0, _;
  }
  var HASH_UNDEFINED$2 = "__lodash_hash_undefined__", objectProto$8 = Object.prototype, hasOwnProperty$6 = objectProto$8.hasOwnProperty;
  function hashGet(O) {
    var _ = this.__data__;
    if (nativeCreate$1) {
      var H = _[O];
      return H === HASH_UNDEFINED$2 ? void 0 : H;
    }
    return hasOwnProperty$6.call(_, O) ? _[O] : void 0;
  }
  var objectProto$7 = Object.prototype, hasOwnProperty$5 = objectProto$7.hasOwnProperty;
  function hashHas(O) {
    var _ = this.__data__;
    return nativeCreate$1 ? _[O] !== void 0 : hasOwnProperty$5.call(_, O);
  }
  var HASH_UNDEFINED$1 = "__lodash_hash_undefined__";
  function hashSet(O, _) {
    var H = this.__data__;
    return this.size += this.has(O) ? 0 : 1, H[O] = nativeCreate$1 && _ === void 0 ? HASH_UNDEFINED$1 : _, this;
  }
  function Hash(O) {
    var _ = -1, H = O == null ? 0 : O.length;
    for (this.clear(); ++_ < H; ) {
      var V = O[_];
      this.set(V[0], V[1]);
    }
  }
  Hash.prototype.clear = hashClear;
  Hash.prototype.delete = hashDelete;
  Hash.prototype.get = hashGet;
  Hash.prototype.has = hashHas;
  Hash.prototype.set = hashSet;
  function mapCacheClear() {
    this.size = 0, this.__data__ = {
      hash: new Hash(),
      map: new (Map$2 || ListCache)(),
      string: new Hash()
    };
  }
  function isKeyable(O) {
    var _ = typeof O;
    return _ == "string" || _ == "number" || _ == "symbol" || _ == "boolean" ? O !== "__proto__" : O === null;
  }
  function getMapData(O, _) {
    var H = O.__data__;
    return isKeyable(_) ? H[typeof _ == "string" ? "string" : "hash"] : H.map;
  }
  function mapCacheDelete(O) {
    var _ = getMapData(this, O).delete(O);
    return this.size -= _ ? 1 : 0, _;
  }
  function mapCacheGet(O) {
    return getMapData(this, O).get(O);
  }
  function mapCacheHas(O) {
    return getMapData(this, O).has(O);
  }
  function mapCacheSet(O, _) {
    var H = getMapData(this, O), V = H.size;
    return H.set(O, _), this.size += H.size == V ? 0 : 1, this;
  }
  function MapCache(O) {
    var _ = -1, H = O == null ? 0 : O.length;
    for (this.clear(); ++_ < H; ) {
      var V = O[_];
      this.set(V[0], V[1]);
    }
  }
  MapCache.prototype.clear = mapCacheClear;
  MapCache.prototype.delete = mapCacheDelete;
  MapCache.prototype.get = mapCacheGet;
  MapCache.prototype.has = mapCacheHas;
  MapCache.prototype.set = mapCacheSet;
  var LARGE_ARRAY_SIZE = 200;
  function stackSet(O, _) {
    var H = this.__data__;
    if (H instanceof ListCache) {
      var V = H.__data__;
      if (!Map$2 || V.length < LARGE_ARRAY_SIZE - 1)
        return V.push([O, _]), this.size = ++H.size, this;
      H = this.__data__ = new MapCache(V);
    }
    return H.set(O, _), this.size = H.size, this;
  }
  function Stack$1(O) {
    var _ = this.__data__ = new ListCache(O);
    this.size = _.size;
  }
  Stack$1.prototype.clear = stackClear;
  Stack$1.prototype.delete = stackDelete;
  Stack$1.prototype.get = stackGet;
  Stack$1.prototype.has = stackHas;
  Stack$1.prototype.set = stackSet;
  var HASH_UNDEFINED = "__lodash_hash_undefined__";
  function setCacheAdd(O) {
    return this.__data__.set(O, HASH_UNDEFINED), this;
  }
  function setCacheHas(O) {
    return this.__data__.has(O);
  }
  function SetCache(O) {
    var _ = -1, H = O == null ? 0 : O.length;
    for (this.__data__ = new MapCache(); ++_ < H; )
      this.add(O[_]);
  }
  SetCache.prototype.add = SetCache.prototype.push = setCacheAdd;
  SetCache.prototype.has = setCacheHas;
  function arraySome(O, _) {
    for (var H = -1, V = O == null ? 0 : O.length; ++H < V; )
      if (_(O[H], H, O))
        return !0;
    return !1;
  }
  function cacheHas(O, _) {
    return O.has(_);
  }
  var COMPARE_PARTIAL_FLAG$5 = 1, COMPARE_UNORDERED_FLAG$3 = 2;
  function equalArrays(O, _, H, V, q, G) {
    var J = H & COMPARE_PARTIAL_FLAG$5, Z = O.length, te = _.length;
    if (Z != te && !(J && te > Z))
      return !1;
    var ne = G.get(O), ee = G.get(_);
    if (ne && ee)
      return ne == _ && ee == O;
    var ae = -1, oe = !0, se = H & COMPARE_UNORDERED_FLAG$3 ? new SetCache() : void 0;
    for (G.set(O, _), G.set(_, O); ++ae < Z; ) {
      var ce = O[ae], le = _[ae];
      if (V)
        var ue = J ? V(le, ce, ae, _, O, G) : V(ce, le, ae, O, _, G);
      if (ue !== void 0) {
        if (ue)
          continue;
        oe = !1;
        break;
      }
      if (se) {
        if (!arraySome(_, function(de, fe) {
          if (!cacheHas(se, fe) && (ce === de || q(ce, de, H, V, G)))
            return se.push(fe);
        })) {
          oe = !1;
          break;
        }
      } else if (!(ce === le || q(ce, le, H, V, G))) {
        oe = !1;
        break;
      }
    }
    return G.delete(O), G.delete(_), oe;
  }
  var Uint8Array$1 = root$1.Uint8Array;
  const Uint8Array$1$1 = Uint8Array$1;
  function mapToArray(O) {
    var _ = -1, H = Array(O.size);
    return O.forEach(function(V, q) {
      H[++_] = [q, V];
    }), H;
  }
  function setToArray(O) {
    var _ = -1, H = Array(O.size);
    return O.forEach(function(V) {
      H[++_] = V;
    }), H;
  }
  var COMPARE_PARTIAL_FLAG$4 = 1, COMPARE_UNORDERED_FLAG$2 = 2, boolTag$1 = "[object Boolean]", dateTag$1 = "[object Date]", errorTag$1 = "[object Error]", mapTag$2 = "[object Map]", numberTag$1 = "[object Number]", regexpTag$1 = "[object RegExp]", setTag$2 = "[object Set]", stringTag$1 = "[object String]", symbolTag = "[object Symbol]", arrayBufferTag$1 = "[object ArrayBuffer]", dataViewTag$2 = "[object DataView]", symbolProto$1 = Symbol$1$1 ? Symbol$1$1.prototype : void 0, symbolValueOf = symbolProto$1 ? symbolProto$1.valueOf : void 0;
  function equalByTag(O, _, H, V, q, G, J) {
    switch (H) {
      case dataViewTag$2:
        if (O.byteLength != _.byteLength || O.byteOffset != _.byteOffset)
          return !1;
        O = O.buffer, _ = _.buffer;
      case arrayBufferTag$1:
        return !(O.byteLength != _.byteLength || !G(new Uint8Array$1$1(O), new Uint8Array$1$1(_)));
      case boolTag$1:
      case dateTag$1:
      case numberTag$1:
        return eq$1(+O, +_);
      case errorTag$1:
        return O.name == _.name && O.message == _.message;
      case regexpTag$1:
      case stringTag$1:
        return O == _ + "";
      case mapTag$2:
        var Z = mapToArray;
      case setTag$2:
        var te = V & COMPARE_PARTIAL_FLAG$4;
        if (Z || (Z = setToArray), O.size != _.size && !te)
          return !1;
        var ne = J.get(O);
        if (ne)
          return ne == _;
        V |= COMPARE_UNORDERED_FLAG$2, J.set(O, _);
        var ee = equalArrays(Z(O), Z(_), V, q, G, J);
        return J.delete(O), ee;
      case symbolTag:
        if (symbolValueOf)
          return symbolValueOf.call(O) == symbolValueOf.call(_);
    }
    return !1;
  }
  function arrayPush(O, _) {
    for (var H = -1, V = _.length, q = O.length; ++H < V; )
      O[q + H] = _[H];
    return O;
  }
  var isArray = Array.isArray;
  const isArray$1 = isArray;
  function baseGetAllKeys(O, _, H) {
    var V = _(O);
    return isArray$1(O) ? V : arrayPush(V, H(O));
  }
  function arrayFilter(O, _) {
    for (var H = -1, V = O == null ? 0 : O.length, q = 0, G = []; ++H < V; ) {
      var J = O[H];
      _(J, H, O) && (G[q++] = J);
    }
    return G;
  }
  function stubArray() {
    return [];
  }
  var objectProto$6 = Object.prototype, propertyIsEnumerable$1 = objectProto$6.propertyIsEnumerable, nativeGetSymbols = Object.getOwnPropertySymbols, getSymbols = nativeGetSymbols ? function(O) {
    return O == null ? [] : (O = Object(O), arrayFilter(nativeGetSymbols(O), function(_) {
      return propertyIsEnumerable$1.call(O, _);
    }));
  } : stubArray;
  const getSymbols$1 = getSymbols;
  function baseTimes(O, _) {
    for (var H = -1, V = Array(O); ++H < O; )
      V[H] = _(H);
    return V;
  }
  var argsTag$2 = "[object Arguments]";
  function baseIsArguments(O) {
    return isObjectLike(O) && baseGetTag(O) == argsTag$2;
  }
  var objectProto$5 = Object.prototype, hasOwnProperty$4 = objectProto$5.hasOwnProperty, propertyIsEnumerable = objectProto$5.propertyIsEnumerable, isArguments = baseIsArguments(/* @__PURE__ */ function() {
    return arguments;
  }()) ? baseIsArguments : function(O) {
    return isObjectLike(O) && hasOwnProperty$4.call(O, "callee") && !propertyIsEnumerable.call(O, "callee");
  };
  const isArguments$1 = isArguments;
  function stubFalse() {
    return !1;
  }
  var freeExports$1 = typeof exports == "object" && exports && !exports.nodeType && exports, freeModule$1 = freeExports$1 && typeof module == "object" && module && !module.nodeType && module, moduleExports$1 = freeModule$1 && freeModule$1.exports === freeExports$1, Buffer$2 = moduleExports$1 ? root$1.Buffer : void 0, nativeIsBuffer = Buffer$2 ? Buffer$2.isBuffer : void 0, isBuffer = nativeIsBuffer || stubFalse;
  const isBuffer$1 = isBuffer;
  var MAX_SAFE_INTEGER$1 = 9007199254740991, reIsUint = /^(?:0|[1-9]\d*)$/;
  function isIndex(O, _) {
    var H = typeof O;
    return _ = _ ?? MAX_SAFE_INTEGER$1, !!_ && (H == "number" || H != "symbol" && reIsUint.test(O)) && O > -1 && O % 1 == 0 && O < _;
  }
  var MAX_SAFE_INTEGER = 9007199254740991;
  function isLength(O) {
    return typeof O == "number" && O > -1 && O % 1 == 0 && O <= MAX_SAFE_INTEGER;
  }
  var argsTag$1 = "[object Arguments]", arrayTag$1 = "[object Array]", boolTag = "[object Boolean]", dateTag = "[object Date]", errorTag = "[object Error]", funcTag = "[object Function]", mapTag$1 = "[object Map]", numberTag = "[object Number]", objectTag$2 = "[object Object]", regexpTag = "[object RegExp]", setTag$1 = "[object Set]", stringTag$2 = "[object String]", weakMapTag$1 = "[object WeakMap]", arrayBufferTag = "[object ArrayBuffer]", dataViewTag$1 = "[object DataView]", float32Tag = "[object Float32Array]", float64Tag = "[object Float64Array]", int8Tag = "[object Int8Array]", int16Tag = "[object Int16Array]", int32Tag = "[object Int32Array]", uint8Tag = "[object Uint8Array]", uint8ClampedTag = "[object Uint8ClampedArray]", uint16Tag = "[object Uint16Array]", uint32Tag = "[object Uint32Array]", typedArrayTags = {};
  typedArrayTags[float32Tag] = typedArrayTags[float64Tag] = typedArrayTags[int8Tag] = typedArrayTags[int16Tag] = typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] = typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] = typedArrayTags[uint32Tag] = !0;
  typedArrayTags[argsTag$1] = typedArrayTags[arrayTag$1] = typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] = typedArrayTags[dataViewTag$1] = typedArrayTags[dateTag] = typedArrayTags[errorTag] = typedArrayTags[funcTag] = typedArrayTags[mapTag$1] = typedArrayTags[numberTag] = typedArrayTags[objectTag$2] = typedArrayTags[regexpTag] = typedArrayTags[setTag$1] = typedArrayTags[stringTag$2] = typedArrayTags[weakMapTag$1] = !1;
  function baseIsTypedArray(O) {
    return isObjectLike(O) && isLength(O.length) && !!typedArrayTags[baseGetTag(O)];
  }
  function baseUnary(O) {
    return function(_) {
      return O(_);
    };
  }
  var freeExports = typeof exports == "object" && exports && !exports.nodeType && exports, freeModule = freeExports && typeof module == "object" && module && !module.nodeType && module, moduleExports = freeModule && freeModule.exports === freeExports, freeProcess = moduleExports && freeGlobal$1.process, nodeUtil = function() {
    try {
      var O = freeModule && freeModule.require && freeModule.require("util").types;
      return O || freeProcess && freeProcess.binding && freeProcess.binding("util");
    } catch {
    }
  }();
  const nodeUtil$1 = nodeUtil;
  var nodeIsTypedArray = nodeUtil$1 && nodeUtil$1.isTypedArray, isTypedArray = nodeIsTypedArray ? baseUnary(nodeIsTypedArray) : baseIsTypedArray;
  const isTypedArray$1 = isTypedArray;
  var objectProto$4 = Object.prototype, hasOwnProperty$3 = objectProto$4.hasOwnProperty;
  function arrayLikeKeys(O, _) {
    var H = isArray$1(O), V = !H && isArguments$1(O), q = !H && !V && isBuffer$1(O), G = !H && !V && !q && isTypedArray$1(O), J = H || V || q || G, Z = J ? baseTimes(O.length, String) : [], te = Z.length;
    for (var ne in O)
      hasOwnProperty$3.call(O, ne) && !(J && // Safari 9 has enumerable `arguments.length` in strict mode.
      (ne == "length" || // Node.js 0.10 has enumerable non-index properties on buffers.
      q && (ne == "offset" || ne == "parent") || // PhantomJS 2 has enumerable non-index properties on typed arrays.
      G && (ne == "buffer" || ne == "byteLength" || ne == "byteOffset") || // Skip index properties.
      isIndex(ne, te))) && Z.push(ne);
    return Z;
  }
  var objectProto$3 = Object.prototype;
  function isPrototype(O) {
    var _ = O && O.constructor, H = typeof _ == "function" && _.prototype || objectProto$3;
    return O === H;
  }
  function overArg(O, _) {
    return function(H) {
      return O(_(H));
    };
  }
  var nativeKeys = overArg(Object.keys, Object);
  const nativeKeys$1 = nativeKeys;
  var objectProto$2 = Object.prototype, hasOwnProperty$2 = objectProto$2.hasOwnProperty;
  function baseKeys(O) {
    if (!isPrototype(O))
      return nativeKeys$1(O);
    var _ = [];
    for (var H in Object(O))
      hasOwnProperty$2.call(O, H) && H != "constructor" && _.push(H);
    return _;
  }
  function isArrayLike(O) {
    return O != null && isLength(O.length) && !isFunction$1(O);
  }
  function keys$1(O) {
    return isArrayLike(O) ? arrayLikeKeys(O) : baseKeys(O);
  }
  function getAllKeys(O) {
    return baseGetAllKeys(O, keys$1, getSymbols$1);
  }
  var COMPARE_PARTIAL_FLAG$3 = 1, objectProto$1 = Object.prototype, hasOwnProperty$1 = objectProto$1.hasOwnProperty;
  function equalObjects(O, _, H, V, q, G) {
    var J = H & COMPARE_PARTIAL_FLAG$3, Z = getAllKeys(O), te = Z.length, ne = getAllKeys(_), ee = ne.length;
    if (te != ee && !J)
      return !1;
    for (var ae = te; ae--; ) {
      var oe = Z[ae];
      if (!(J ? oe in _ : hasOwnProperty$1.call(_, oe)))
        return !1;
    }
    var se = G.get(O), ce = G.get(_);
    if (se && ce)
      return se == _ && ce == O;
    var le = !0;
    G.set(O, _), G.set(_, O);
    for (var ue = J; ++ae < te; ) {
      oe = Z[ae];
      var de = O[oe], fe = _[oe];
      if (V)
        var he = J ? V(fe, de, oe, _, O, G) : V(de, fe, oe, O, _, G);
      if (!(he === void 0 ? de === fe || q(de, fe, H, V, G) : he)) {
        le = !1;
        break;
      }
      ue || (ue = oe == "constructor");
    }
    if (le && !ue) {
      var me = O.constructor, pe = _.constructor;
      me != pe && "constructor" in O && "constructor" in _ && !(typeof me == "function" && me instanceof me && typeof pe == "function" && pe instanceof pe) && (le = !1);
    }
    return G.delete(O), G.delete(_), le;
  }
  var DataView$1 = getNative(root$1, "DataView");
  const DataView$1$1 = DataView$1;
  var Promise$1 = getNative(root$1, "Promise");
  const Promise$2 = Promise$1;
  var Set$1 = getNative(root$1, "Set");
  const Set$2 = Set$1;
  var WeakMap$1 = getNative(root$1, "WeakMap");
  const WeakMap$1$1 = WeakMap$1;
  var mapTag$3 = "[object Map]", objectTag$1 = "[object Object]", promiseTag = "[object Promise]", setTag$3 = "[object Set]", weakMapTag = "[object WeakMap]", dataViewTag = "[object DataView]", dataViewCtorString = toSource(DataView$1$1), mapCtorString = toSource(Map$2), promiseCtorString = toSource(Promise$2), setCtorString = toSource(Set$2), weakMapCtorString = toSource(WeakMap$1$1), getTag = baseGetTag;
  (DataView$1$1 && getTag(new DataView$1$1(new ArrayBuffer(1))) != dataViewTag || Map$2 && getTag(new Map$2()) != mapTag$3 || Promise$2 && getTag(Promise$2.resolve()) != promiseTag || Set$2 && getTag(new Set$2()) != setTag$3 || WeakMap$1$1 && getTag(new WeakMap$1$1()) != weakMapTag) && (getTag = function(O) {
    var _ = baseGetTag(O), H = _ == objectTag$1 ? O.constructor : void 0, V = H ? toSource(H) : "";
    if (V)
      switch (V) {
        case dataViewCtorString:
          return dataViewTag;
        case mapCtorString:
          return mapTag$3;
        case promiseCtorString:
          return promiseTag;
        case setCtorString:
          return setTag$3;
        case weakMapCtorString:
          return weakMapTag;
      }
    return _;
  });
  const getTag$1 = getTag;
  var COMPARE_PARTIAL_FLAG$2 = 1, argsTag = "[object Arguments]", arrayTag = "[object Array]", objectTag = "[object Object]", objectProto = Object.prototype, hasOwnProperty$8 = objectProto.hasOwnProperty;
  function baseIsEqualDeep(O, _, H, V, q, G) {
    var J = isArray$1(O), Z = isArray$1(_), te = J ? arrayTag : getTag$1(O), ne = Z ? arrayTag : getTag$1(_);
    te = te == argsTag ? objectTag : te, ne = ne == argsTag ? objectTag : ne;
    var ee = te == objectTag, ae = ne == objectTag, oe = te == ne;
    if (oe && isBuffer$1(O)) {
      if (!isBuffer$1(_))
        return !1;
      J = !0, ee = !1;
    }
    if (oe && !ee)
      return G || (G = new Stack$1()), J || isTypedArray$1(O) ? equalArrays(O, _, H, V, q, G) : equalByTag(O, _, te, H, V, q, G);
    if (!(H & COMPARE_PARTIAL_FLAG$2)) {
      var se = ee && hasOwnProperty$8.call(O, "__wrapped__"), ce = ae && hasOwnProperty$8.call(_, "__wrapped__");
      if (se || ce) {
        var le = se ? O.value() : O, ue = ce ? _.value() : _;
        return G || (G = new Stack$1()), q(le, ue, H, V, G);
      }
    }
    return oe ? (G || (G = new Stack$1()), equalObjects(O, _, H, V, q, G)) : !1;
  }
  function baseIsEqual(O, _, H, V, q) {
    return O === _ ? !0 : O == null || _ == null || !isObjectLike(O) && !isObjectLike(_) ? O !== O && _ !== _ : baseIsEqualDeep(O, _, H, V, baseIsEqual, q);
  }
  var COMPARE_PARTIAL_FLAG$1 = 1, COMPARE_UNORDERED_FLAG$1 = 2;
  function baseIsMatch(O, _, H, V) {
    var q = H.length, G = q;
    if (O == null)
      return !G;
    for (O = Object(O); q--; ) {
      var J = H[q];
      if (J[2] ? J[1] !== O[J[0]] : !(J[0] in O))
        return !1;
    }
    for (; ++q < G; ) {
      J = H[q];
      var Z = J[0], te = O[Z], ne = J[1];
      if (J[2]) {
        if (te === void 0 && !(Z in O))
          return !1;
      } else {
        var ee = new Stack$1(), ae;
        if (!(ae === void 0 ? baseIsEqual(ne, te, COMPARE_PARTIAL_FLAG$1 | COMPARE_UNORDERED_FLAG$1, V, ee) : ae))
          return !1;
      }
    }
    return !0;
  }
  function isStrictComparable(O) {
    return O === O && !isObject$1(O);
  }
  function getMatchData(O) {
    for (var _ = keys$1(O), H = _.length; H--; ) {
      var V = _[H], q = O[V];
      _[H] = [V, q, isStrictComparable(q)];
    }
    return _;
  }
  function matchesStrictComparable(O, _) {
    return function(H) {
      return H == null ? !1 : H[O] === _ && (_ !== void 0 || O in Object(H));
    };
  }
  function baseMatches(O) {
    var _ = getMatchData(O);
    return _.length == 1 && _[0][2] ? matchesStrictComparable(_[0][0], _[0][1]) : function(H) {
      return H === O || baseIsMatch(H, O, _);
    };
  }
  var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/, reIsPlainProp = /^\w*$/;
  function isKey(O, _) {
    if (isArray$1(O))
      return !1;
    var H = typeof O;
    return H == "number" || H == "symbol" || H == "boolean" || O == null || isSymbol(O) ? !0 : reIsPlainProp.test(O) || !reIsDeepProp.test(O) || _ != null && O in Object(_);
  }
  var FUNC_ERROR_TEXT$1 = "Expected a function";
  function memoize(O, _) {
    if (typeof O != "function" || _ != null && typeof _ != "function")
      throw new TypeError(FUNC_ERROR_TEXT$1);
    var H = function() {
      var V = arguments, q = _ ? _.apply(this, V) : V[0], G = H.cache;
      if (G.has(q))
        return G.get(q);
      var J = O.apply(this, V);
      return H.cache = G.set(q, J) || G, J;
    };
    return H.cache = new (memoize.Cache || MapCache)(), H;
  }
  memoize.Cache = MapCache;
  var MAX_MEMOIZE_SIZE = 500;
  function memoizeCapped(O) {
    var _ = memoize(O, function(V) {
      return H.size === MAX_MEMOIZE_SIZE && H.clear(), V;
    }), H = _.cache;
    return _;
  }
  var rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g, reEscapeChar = /\\(\\)?/g, stringToPath = memoizeCapped(function(O) {
    var _ = [];
    return O.charCodeAt(0) === 46 && _.push(""), O.replace(rePropName, function(H, V, q, G) {
      _.push(q ? G.replace(reEscapeChar, "$1") : V || H);
    }), _;
  });
  const stringToPath$1 = stringToPath;
  function arrayMap(O, _) {
    for (var H = -1, V = O == null ? 0 : O.length, q = Array(V); ++H < V; )
      q[H] = _(O[H], H, O);
    return q;
  }
  var symbolProto = Symbol$1$1 ? Symbol$1$1.prototype : void 0, symbolToString = symbolProto ? symbolProto.toString : void 0;
  function baseToString(O) {
    if (typeof O == "string")
      return O;
    if (isArray$1(O))
      return arrayMap(O, baseToString) + "";
    if (isSymbol(O))
      return symbolToString ? symbolToString.call(O) : "";
    var _ = O + "";
    return _ == "0" && 1 / O == -1 / 0 ? "-0" : _;
  }
  function toString(O) {
    return O == null ? "" : baseToString(O);
  }
  function castPath(O, _) {
    return isArray$1(O) ? O : isKey(O, _) ? [O] : stringToPath$1(toString(O));
  }
  function toKey(O) {
    if (typeof O == "string" || isSymbol(O))
      return O;
    var _ = O + "";
    return _ == "0" && 1 / O == -1 / 0 ? "-0" : _;
  }
  function baseGet(O, _) {
    _ = castPath(_, O);
    for (var H = 0, V = _.length; O != null && H < V; )
      O = O[toKey(_[H++])];
    return H && H == V ? O : void 0;
  }
  function get(O, _, H) {
    var V = O == null ? void 0 : baseGet(O, _);
    return V === void 0 ? H : V;
  }
  function baseHasIn(O, _) {
    return O != null && _ in Object(O);
  }
  function hasPath(O, _, H) {
    _ = castPath(_, O);
    for (var V = -1, q = _.length, G = !1; ++V < q; ) {
      var J = toKey(_[V]);
      if (!(G = O != null && H(O, J)))
        break;
      O = O[J];
    }
    return G || ++V != q ? G : (q = O == null ? 0 : O.length, !!q && isLength(q) && isIndex(J, q) && (isArray$1(O) || isArguments$1(O)));
  }
  function hasIn(O, _) {
    return O != null && hasPath(O, _, baseHasIn);
  }
  var COMPARE_PARTIAL_FLAG = 1, COMPARE_UNORDERED_FLAG = 2;
  function baseMatchesProperty(O, _) {
    return isKey(O) && isStrictComparable(_) ? matchesStrictComparable(toKey(O), _) : function(H) {
      var V = get(H, O);
      return V === void 0 && V === _ ? hasIn(H, O) : baseIsEqual(_, V, COMPARE_PARTIAL_FLAG | COMPARE_UNORDERED_FLAG);
    };
  }
  function identity(O) {
    return O;
  }
  function baseProperty(O) {
    return function(_) {
      return _ == null ? void 0 : _[O];
    };
  }
  function basePropertyDeep(O) {
    return function(_) {
      return baseGet(_, O);
    };
  }
  function property(O) {
    return isKey(O) ? baseProperty(toKey(O)) : basePropertyDeep(O);
  }
  function baseIteratee(O) {
    return typeof O == "function" ? O : O == null ? identity : typeof O == "object" ? isArray$1(O) ? baseMatchesProperty(O[0], O[1]) : baseMatches(O) : property(O);
  }
  var INFINITY = 1 / 0, MAX_INTEGER = 17976931348623157e292;
  function toFinite(O) {
    if (!O)
      return O === 0 ? O : 0;
    if (O = toNumber(O), O === INFINITY || O === -1 / 0) {
      var _ = O < 0 ? -1 : 1;
      return _ * MAX_INTEGER;
    }
    return O === O ? O : 0;
  }
  function toInteger(O) {
    var _ = toFinite(O), H = _ % 1;
    return _ === _ ? H ? _ - H : _ : 0;
  }
  function findIndex(O, _, H) {
    var V = O == null ? 0 : O.length;
    if (!V)
      return -1;
    var q = 0;
    return baseFindIndex(O, baseIteratee(_), q);
  }
  var nativeCeil = Math.ceil, nativeMax = Math.max;
  function baseRange(O, _, H, V) {
    for (var q = -1, G = nativeMax(nativeCeil((_ - O) / (H || 1)), 0), J = Array(G); G--; )
      J[++q] = O, O += H;
    return J;
  }
  function isIterateeCall(O, _, H) {
    if (!isObject$1(H))
      return !1;
    var V = typeof _;
    return (V == "number" ? isArrayLike(H) && isIndex(_, H.length) : V == "string" && _ in H) ? eq$1(H[_], O) : !1;
  }
  function createRange(O) {
    return function(_, H, V) {
      return V && typeof V != "number" && isIterateeCall(_, H, V) && (H = V = void 0), _ = toFinite(_), H === void 0 ? (H = _, _ = 0) : H = toFinite(H), V = V === void 0 ? _ < H ? 1 : -1 : toFinite(V), baseRange(_, H, V);
    };
  }
  var range$1 = createRange();
  const range$2 = range$1, appendToMap = (O, _, H) => {
    const V = O.get(_);
    V ? V.includes(H) || V.push(H) : O.set(_, [H]);
  }, debounce = (O, _) => {
    let H;
    return (...V) => {
      H && clearTimeout(H), H = setTimeout(() => {
        H = 0, O(...V);
      }, _);
    };
  }, isConnected = (O) => !("isConnected" in O) || O.isConnected, cleanupElements = debounce((O) => {
    for (let _ of O.keys())
      O.set(_, O.get(_).filter(isConnected));
  }, 2e3), stencilSubscription = () => {
    if (typeof getRenderingRef != "function")
      return {};
    const O = /* @__PURE__ */ new Map();
    return {
      dispose: () => O.clear(),
      get: (_) => {
        const H = getRenderingRef();
        H && appendToMap(O, _, H);
      },
      set: (_) => {
        const H = O.get(_);
        H && O.set(_, H.filter(forceUpdate)), cleanupElements(O);
      },
      reset: () => {
        O.forEach((_) => _.forEach(forceUpdate)), cleanupElements(O);
      }
    };
  }, unwrap = (O) => typeof O == "function" ? O() : O, createObservableMap = (O, _ = (H, V) => H !== V) => {
    const H = unwrap(O);
    let V = new Map(Object.entries(H ?? {}));
    const q = {
      dispose: [],
      get: [],
      set: [],
      reset: []
    }, G = () => {
      var ce;
      V = new Map(Object.entries((ce = unwrap(O)) !== null && ce !== void 0 ? ce : {})), q.reset.forEach((le) => le());
    }, J = () => {
      q.dispose.forEach((ce) => ce()), G();
    }, Z = (ce) => (q.get.forEach((le) => le(ce)), V.get(ce)), te = (ce, le) => {
      const ue = V.get(ce);
      _(le, ue, ce) && (V.set(ce, le), q.set.forEach((de) => de(ce, le, ue)));
    }, ne = typeof Proxy > "u" ? {} : new Proxy(H, {
      get(ce, le) {
        return Z(le);
      },
      ownKeys(ce) {
        return Array.from(V.keys());
      },
      getOwnPropertyDescriptor() {
        return {
          enumerable: !0,
          configurable: !0
        };
      },
      has(ce, le) {
        return V.has(le);
      },
      set(ce, le, ue) {
        return te(le, ue), !0;
      }
    }), ee = (ce, le) => (q[ce].push(le), () => {
      removeFromArray(q[ce], le);
    });
    return {
      state: ne,
      get: Z,
      set: te,
      on: ee,
      onChange: (ce, le) => {
        const ue = ee("set", (fe, he) => {
          fe === ce && le(he);
        }), de = ee("reset", () => le(unwrap(O)[ce]));
        return () => {
          ue(), de();
        };
      },
      use: (...ce) => {
        const le = ce.reduce((ue, de) => (de.set && ue.push(ee("set", de.set)), de.get && ue.push(ee("get", de.get)), de.reset && ue.push(ee("reset", de.reset)), de.dispose && ue.push(ee("dispose", de.dispose)), ue), []);
        return () => le.forEach((ue) => ue());
      },
      dispose: J,
      reset: G,
      forceUpdate: (ce) => {
        const le = V.get(ce);
        q.set.forEach((ue) => ue(ce, le, le));
      }
    };
  }, removeFromArray = (O, _) => {
    const H = O.indexOf(_);
    H >= 0 && (O[H] = O[O.length - 1], O.length--);
  }, createStore = (O, _) => {
    const H = createObservableMap(O, _);
    return H.use(stencilSubscription()), H;
  }, trimmedPlugin$1 = (O) => ({
    set(_, H) {
      switch (_) {
        case "trimmed": {
          const V = O.get("proxyItems"), q = gatherTrimmedItems(H), G = V.filter((J) => !q[J]);
          O.set("items", G);
          break;
        }
      }
    }
  });
  function gatherTrimmedItems(O) {
    const _ = {};
    for (let H in O)
      for (let V in O[H])
        _[V] = _[V] || O[H][V];
    return _;
  }
  function setStore(O, _) {
    Object.entries(_).forEach(([H, V]) => {
      O.set(H, V);
    });
  }
  const MIN_COL_SIZE = 30, RESIZE_INTERVAL = 40, DATA_COL = "data-rgCol", DATA_ROW = "data-rgRow", DISABLED_CLASS = "disabled", CELL_CLASS = "rgCell", ROW_HEADER_TYPE = "rowHeaders", HEADER_CLASS = "rgHeaderCell", HEADER_SORTABLE_CLASS = "sortable", HEADER_ROW_CLASS = "header-rgRow", HEADER_ACTUAL_ROW_CLASS = "actual-rgRow", DRAG_ICON_CLASS = "revo-drag-icon", DRAGGABLE_CLASS = "revo-draggable", FOCUS_CLASS = "focused-cell", SELECTION_BORDER_CLASS = "selection-border-range", MOBILE_CLASS = "mobile-handler", TMP_SELECTION_BG_CLASS = "temp-bg-range", CELL_HANDLER_CLASS = "autofill-handle", EDIT_INPUT_WR = "edit-input-wrapper", DRAGG_TEXT = "Draggable item", GRID_INTERNALS = "__rvgr", ROW_FOCUSED_CLASS = "focused-rgRow";
  var codes;
  (function(O) {
    O[O.MOUSE_LEFT = 1] = "MOUSE_LEFT", O[O.MOUSE_RIGHT = 3] = "MOUSE_RIGHT", O[O.MOUSE_MIDDLE = 2] = "MOUSE_MIDDLE", O[O.BACKSPACE = 8] = "BACKSPACE", O[O.COMMA = 188] = "COMMA", O[O.INSERT = 45] = "INSERT", O[O.DELETE = 46] = "DELETE", O[O.END = 35] = "END", O[O.ENTER = 13] = "ENTER", O[O.ESCAPE = 27] = "ESCAPE", O[O.CONTROL = 17] = "CONTROL", O[O.COMMAND_LEFT = 91] = "COMMAND_LEFT", O[O.COMMAND_RIGHT = 93] = "COMMAND_RIGHT", O[O.COMMAND_FIREFOX = 224] = "COMMAND_FIREFOX", O[O.ALT = 18] = "ALT", O[O.HOME = 36] = "HOME", O[O.PAGE_DOWN = 34] = "PAGE_DOWN", O[O.PAGE_UP = 33] = "PAGE_UP", O[O.PERIOD = 190] = "PERIOD", O[O.SPACE = 32] = "SPACE", O[O.SHIFT = 16] = "SHIFT", O[O.CAPS_LOCK = 20] = "CAPS_LOCK", O[O.TAB = 9] = "TAB", O[O.ARROW_RIGHT = 39] = "ARROW_RIGHT", O[O.ARROW_LEFT = 37] = "ARROW_LEFT", O[O.ARROW_UP = 38] = "ARROW_UP", O[O.ARROW_DOWN = 40] = "ARROW_DOWN", O[O.F1 = 112] = "F1", O[O.F2 = 113] = "F2", O[O.F3 = 114] = "F3", O[O.F4 = 115] = "F4", O[O.F5 = 116] = "F5", O[O.F6 = 117] = "F6", O[O.F7 = 118] = "F7", O[O.F8 = 119] = "F8", O[O.F9 = 120] = "F9", O[O.F10 = 121] = "F10", O[O.F11 = 122] = "F11", O[O.F12 = 123] = "F12", O[O.A = 65] = "A", O[O.C = 67] = "C", O[O.D = 68] = "D", O[O.F = 70] = "F", O[O.L = 76] = "L", O[O.O = 79] = "O", O[O.P = 80] = "P", O[O.S = 83] = "S", O[O.V = 86] = "V", O[O.X = 88] = "X";
  })(codes || (codes = {}));
  var codesLetter;
  (function(O) {
    O.ENTER = "Enter", O.ENTER_NUM = "NumpadEnter", O.A = "KeyA", O.C = "KeyC", O.X = "KeyX", O.V = "KeyV", O.ESCAPE = "Escape", O.TAB = "Tab", O.BACKSPACE = "Backspace", O.DELETE = "Delete", O.ARROW_RIGHT = "ArrowRight", O.ARROW_LEFT = "ArrowLeft", O.ARROW_UP = "ArrowUp", O.ARROW_DOWN = "ArrowDown", O.SHIFT = "Shift";
  })(codesLetter || (codesLetter = {}));
  var keyValues;
  (function(O) {
    O.ENTER = "Enter", O.TAB = "Tab";
  })(keyValues || (keyValues = {}));
  var osPlatform;
  (function(O) {
    O.mac = "Mac";
  })(osPlatform || (osPlatform = {}));
  function getScrollbarSize(O) {
    const _ = O.createElement("div");
    _.style.overflow = "scroll", _.style.visibility = "hidden", _.style.position = "absolute", _.style.top = "-9999px", _.style.width = "50px", _.style.height = "50px", O.body.appendChild(_);
    const H = _.offsetWidth - _.clientWidth;
    return O.body.removeChild(_), H;
  }
  function scaleValue(O, _, H) {
    return (H[1] - H[0]) * (O - _[0]) / (_[1] - _[0]) + H[0];
  }
  async function timeout(O = 0) {
    await new Promise((_) => {
      setTimeout(() => _(), O);
    });
  }
  class DataStore {
    get store() {
      return this.dataStore;
    }
    constructor(_, H) {
      const V = this.dataStore = createStore(Object.assign({ items: [], proxyItems: [], source: [], groupingDepth: 0, groups: {}, type: _, trimmed: {}, groupingCustomRenderer: void 0 }, H));
      V.use(proxyPlugin(V)), V.use(trimmedPlugin$1(V));
    }
    /**
     * full data source update
     * @param source - data column/rgRow source
     * @param grouping - grouping information if present
     */
    updateData(_, H, V = !1) {
      V || this.store.set("trimmed", {}), this.store.set("items", []);
      const q = range$2(0, (_ == null ? void 0 : _.length) || 0);
      setStore(this.store, {
        source: _,
        proxyItems: [...q]
      }), this.store.set("items", q), H && setStore(this.store, {
        groupingDepth: H.depth,
        groups: H.groups,
        groupingCustomRenderer: H.customRenderer
      });
    }
    addTrimmed(_) {
      let H = this.store.get("trimmed");
      H = Object.assign(Object.assign({}, H), _), setStore(this.store, { trimmed: H });
    }
    setSourceData(_, H = !0) {
      setSourceByVirtualIndex(this.store, _, H);
    }
    // local data update
    setData(_) {
      const H = Object.assign({}, _);
      setStore(this.store, H);
    }
    refresh() {
      const _ = this.store.get("source");
      this.store.set("source", [..._]);
    }
  }
  function getPhysical(O, _) {
    return O.get("items")[_];
  }
  function getVisibleSourceItem(O) {
    const _ = O.get("source");
    return O.get("items").map((H) => _[H]);
  }
  const getSourceItem = (O, _) => O.get("source")[getSourcePhysicalIndex(O, _)], getSourcePhysicalIndex = (O, _) => O.get("items")[_];
  function setSourceByVirtualIndex(O, _, H = !0) {
    const V = O.get("items"), q = O.get("source");
    for (let G in _) {
      const J = V[G], Z = _[G];
      q[J] = Z;
    }
    H && O.set("source", [...q]);
  }
  function setSourceByPhysicalIndex(O, _, H = !0) {
    const V = O.get("source");
    for (let q in _)
      V[q] = _[q];
    H && O.set("source", [...V]);
  }
  function setItems(O, _) {
    O.set("items", _);
  }
  function getSourceItemVirtualIndexByProp(O, _) {
    const H = O.get("items"), V = O.get("source"), q = findIndex(V, { prop: _ });
    return H.indexOf(q);
  }
  var MAX_ARRAY_LENGTH$1 = 4294967295, MAX_ARRAY_INDEX = MAX_ARRAY_LENGTH$1 - 1, nativeFloor = Math.floor, nativeMin = Math.min;
  function baseSortedIndexBy(O, _, H, V) {
    var q = 0, G = O == null ? 0 : O.length;
    if (G === 0)
      return 0;
    _ = H(_);
    for (var J = _ !== _, Z = _ === null, te = isSymbol(_), ne = _ === void 0; q < G; ) {
      var ee = nativeFloor((q + G) / 2), ae = H(O[ee]), oe = ae !== void 0, se = ae === null, ce = ae === ae, le = isSymbol(ae);
      if (J)
        var ue = ce;
      else ne ? ue = ce && oe : Z ? ue = ce && oe && !se : te ? ue = ce && oe && !se && !le : se || le ? ue = !1 : ue = ae < _;
      ue ? q = ee + 1 : G = ee;
    }
    return nativeMin(G, MAX_ARRAY_INDEX);
  }
  var MAX_ARRAY_LENGTH$2 = 4294967295, HALF_MAX_ARRAY_LENGTH = MAX_ARRAY_LENGTH$2 >>> 1;
  function baseSortedIndex(O, _, H) {
    var V = 0, q = O == null ? V : O.length;
    if (typeof _ == "number" && _ === _ && q <= HALF_MAX_ARRAY_LENGTH) {
      for (; V < q; ) {
        var G = V + q >>> 1, J = O[G];
        J !== null && !isSymbol(J) && J < _ ? V = G + 1 : q = G;
      }
      return q;
    }
    return baseSortedIndexBy(O, _, identity);
  }
  function sortedIndex(O, _) {
    return baseSortedIndex(O, _);
  }
  function calculateDimensionData(O, _ = {}) {
    const H = [], V = {}, q = {}, G = Object.keys(_).map(Number).sort((Z, te) => Z - te);
    let J;
    for (let Z = 0; Z < G.length; Z++) {
      const te = G[Z], ne = {
        itemIndex: te,
        start: 0,
        end: 0
      };
      if (J) {
        const ee = (te - J.itemIndex - 1) * O;
        ne.start = ee + J.end;
      } else
        ne.start = te * O;
      ne.end = ne.start + _[te], H.push(ne.start), q[te] = V[Z] = ne, J = ne;
    }
    return {
      indexes: G,
      positionIndexes: [...H],
      positionIndexToItem: Object.assign({}, V),
      indexToItem: q
    };
  }
  const getItemByPosition = ({ indexes: O, positionIndexes: _, originItemSize: H, positionIndexToItem: V }, q) => {
    const G = {
      itemIndex: 0,
      start: 0,
      end: 0
    }, J = O.length ? sortedIndex(_, q) : 0;
    if (!J)
      return G.itemIndex = Math.floor(q / H), G.start = G.itemIndex * H, G.end = G.start + H, G;
    const Z = V[J - 1];
    if (Z.end > q)
      return Z;
    const te = q - Z.end, ne = Math.floor(te / H);
    return G.itemIndex = Z.itemIndex + 1 + ne, G.start = Z.end + ne * H, G.end = G.start + H, G;
  };
  function getItemByIndex(O, _) {
    let H = {
      itemIndex: _,
      start: 0,
      end: 0
    };
    if (O.indexToItem[_])
      return O.indexToItem[_];
    const V = O.indexes.length ? sortedIndex(O.indexes, _) : 0;
    if (!V)
      return H.start = H.itemIndex * O.originItemSize, H.end = H.start + O.originItemSize, H;
    const q = O.indexToItem[O.indexes[V - 1]];
    return H.start = q.end + (_ - q.itemIndex - 1) * O.originItemSize, H.end = H.start + O.originItemSize, H;
  }
  /*!
   * Built by Revolist OU ❤️
   */
  function arrayReduce(O, _, H, V) {
    var q = -1, G = O == null ? 0 : O.length;
    for (V && G && (H = O[++q]); ++q < G; )
      H = _(H, O[q], q, O);
    return H;
  }
  function createBaseFor(O) {
    return function(_, H, V) {
      for (var q = -1, G = Object(_), J = V(_), Z = J.length; Z--; ) {
        var te = J[++q];
        if (H(G[te], te, G) === !1)
          break;
      }
      return _;
    };
  }
  var baseFor = createBaseFor();
  const baseFor$1 = baseFor;
  function baseForOwn(O, _) {
    return O && baseFor$1(O, _, keys$1);
  }
  function createBaseEach(O, _) {
    return function(H, V) {
      if (H == null)
        return H;
      if (!isArrayLike(H))
        return O(H, V);
      for (var q = H.length, G = -1, J = Object(H); ++G < q && V(J[G], G, J) !== !1; )
        ;
      return H;
    };
  }
  var baseEach = createBaseEach(baseForOwn);
  const baseEach$1 = baseEach;
  function baseReduce(O, _, H, V, q) {
    return q(O, function(G, J, Z) {
      H = V ? (V = !1, G) : _(H, G, J, Z);
    }), H;
  }
  function reduce$1(O, _, H) {
    var V = isArray$1(O) ? arrayReduce : baseReduce, q = arguments.length < 3;
    return V(O, baseIteratee(_), H, q, baseEach$1);
  }
  function getCellData(O) {
    return typeof O > "u" || O === null ? "" : O;
  }
  function getCellRaw(O, _) {
    if (_)
      return _.cellParser ? _.cellParser(O, _) : O[_.prop];
  }
  function getCellDataParsed(O, _) {
    return getCellData(getCellRaw(O, _));
  }
  function getColumnType(O) {
    return O.pin ? O.pin : "rgCol";
  }
  function getColumnSizes(O) {
    const _ = {};
    for (const [H, V] of O.entries())
      V.size && (_[H] = V.size);
    return _;
  }
  function isColGrouping(O) {
    return !!O.children;
  }
  function getColumns(O, _ = 0, H) {
    return reduce$1(O, (q, G) => {
      if (isColGrouping(G))
        return q = gatherGroup(q, G, getColumns(G.children, _ + 1, H), _), q;
      const J = G.columnType && (H == null ? void 0 : H[G.columnType]), Z = Object.assign(Object.assign({}, J), G);
      return Z.pin ? q.columns[Z.pin].push(Z) : q.columns.rgCol.push(Z), Z.order && (q.sort[Z.prop] = Z), q.columnByProp[Z.prop] || (q.columnByProp[Z.prop] = []), q.columnByProp[Z.prop].push(Z), Z.beforeSetup && Z.beforeSetup(Z), q;
    }, {
      // columns as they are in stores per type
      columns: {
        rgCol: [],
        colPinStart: [],
        colPinEnd: []
      },
      // columns indexed by prop for quick access
      columnByProp: {},
      // column grouping
      columnGrouping: {
        rgCol: [],
        colPinStart: [],
        colPinEnd: []
      },
      // max depth level for column grouping
      maxLevel: _,
      // sorting
      sort: {}
    });
  }
  function gatherGroup(O, _, H, V = 0) {
    const q = Object.assign(Object.assign({}, _), { level: V, ids: [] });
    for (let G in H.columns) {
      const J = G, Z = O.columns[J], te = H.columns[J];
      isArray$1(Z) && isArray$1(te) && (Z.push(...te), te.length && O.columnGrouping[J].push(Object.assign(Object.assign({}, q), { ids: te.map((ne) => ne.prop) })));
    }
    for (let G in H.columnGrouping) {
      const J = G, Z = H.columnGrouping[J];
      O.columnGrouping[J].push(...Z);
    }
    return O.maxLevel = Math.max(O.maxLevel, H.maxLevel), O.sort = Object.assign(Object.assign({}, O.sort), H.sort), O.columnByProp = Object.assign(Object.assign({}, O.columnByProp), H.columnByProp), O;
  }
  function findColumn$1(O, _) {
    for (const H of O)
      if (isColGrouping(H)) {
        const V = findColumn$1(H.children, _);
        if (V)
          return V;
      } else if (H.prop === _)
        return H;
  }
  function getColumnByProp(O, _) {
    return findColumn$1(O, _);
  }
  function defaultState() {
    return {
      range: null,
      tempRange: null,
      tempRangeType: null,
      focus: null,
      edit: null,
      lastCell: null,
      nextFocus: null
    };
  }
  class SelectionStore {
    constructor() {
      this.unsubscribe = [], this.store = createStore(defaultState()), this.store.on("set", (_, H) => {
        _ === "tempRange" && !H && this.store.set("tempRangeType", null);
      });
    }
    onChange(_, H) {
      this.unsubscribe.push(this.store.onChange(_, H));
    }
    clearFocus() {
      setStore(this.store, { focus: null, range: null, edit: null, tempRange: null });
    }
    setFocus(_, H) {
      H ? setStore(this.store, {
        focus: _,
        range: getRange(_, H),
        edit: null,
        tempRange: null
      }) : setStore(this.store, { focus: _ });
    }
    setNextFocus(_) {
      setStore(this.store, { nextFocus: _ });
    }
    setTempArea(_) {
      setStore(this.store, { tempRange: _ == null ? void 0 : _.area, tempRangeType: _ == null ? void 0 : _.type, edit: null });
    }
    clearTemp() {
      setStore(this.store, { tempRange: null });
    }
    /** Can be applied from selection change or from simple keyboard change clicks */
    setRangeArea(_) {
      setStore(this.store, { range: _, edit: null, tempRange: null });
    }
    setRange(_, H) {
      const V = getRange(_, H);
      this.setRangeArea(V);
    }
    setLastCell(_) {
      setStore(this.store, { lastCell: _ });
    }
    setEdit(_) {
      const H = this.store.get("focus");
      if (H && typeof _ == "string") {
        setStore(this.store, {
          edit: { x: H.x, y: H.y, val: _ }
        });
        return;
      }
      setStore(this.store, { edit: null });
    }
    dispose() {
      this.unsubscribe.forEach((_) => _()), this.store.dispose();
    }
  }
  const EMPTY_INDEX = -1;
  class SelectionStoreConnector {
    constructor() {
      this.dirty = !1, this.stores = {}, this.columnStores = {}, this.rowStores = {}, this.storesByType = {}, this.storesXToType = {}, this.storesYToType = {}, this.sections = [];
    }
    get focusedStore() {
      var _;
      for (let H in this.stores)
        for (let V in this.stores[H]) {
          const q = (_ = this.stores[H][V]) === null || _ === void 0 ? void 0 : _.store.get("focus");
          if (q)
            return {
              entity: this.stores[H][V],
              cell: q,
              position: {
                x: parseInt(V, 10),
                y: parseInt(H, 10)
              }
            };
        }
      return null;
    }
    get edit() {
      var _;
      return (_ = this.focusedStore) === null || _ === void 0 ? void 0 : _.entity.store.get("edit");
    }
    get focused() {
      var _;
      return (_ = this.focusedStore) === null || _ === void 0 ? void 0 : _.entity.store.get("focus");
    }
    get selectedRange() {
      var _;
      return (_ = this.focusedStore) === null || _ === void 0 ? void 0 : _.entity.store.get("range");
    }
    registerSection(_) {
      if (!_) {
        this.sections.length = 0, this.dirty = !0;
        return;
      }
      this.sections.indexOf(_) === -1 && this.sections.push(_);
    }
    // check if require to cleanup all stores
    beforeUpdate() {
      if (this.dirty) {
        for (let _ in this.stores)
          for (let H in this.stores[_])
            this.stores[_][H].dispose();
        this.dirty = !1;
      }
    }
    registerColumn(_, H) {
      return isHiddenStore(_) ? new SelectionStore() : this.columnStores[_] ? this.columnStores[_] : (this.columnStores[_] = new SelectionStore(), this.storesByType[H] = _, this.storesXToType[_] = H, this.columnStores[_]);
    }
    registerRow(_, H) {
      return isHiddenStore(_) ? new SelectionStore() : this.rowStores[_] ? this.rowStores[_] : (this.rowStores[_] = new SelectionStore(), this.storesByType[H] = _, this.storesYToType[_] = H, this.rowStores[_]);
    }
    /**
     * Cross store proxy, based on multiple dimensions
     */
    register({ x: _, y: H }) {
      var V, q;
      return isHiddenStore(_) || isHiddenStore(H) ? new SelectionStore() : (this.stores[H] || (this.stores[H] = {}), this.stores[H][_] ? this.stores[H][_] : (this.stores[H][_] = new SelectionStore(), (V = this.stores[H][_]) === null || V === void 0 || V.onChange("range", (G) => {
        this.columnStores[_].setRangeArea(G), this.rowStores[H].setRangeArea(G);
      }), (q = this.stores[H][_]) === null || q === void 0 || q.store.on("dispose", () => this.destroy(_, H)), this.stores[H][_]));
    }
    destroy(_, H) {
      var V, q;
      if ((V = this.columnStores[_]) === null || V === void 0 || V.dispose(), (q = this.rowStores[H]) === null || q === void 0 || q.dispose(), delete this.rowStores[H], delete this.columnStores[_], this.storesXToType[_]) {
        const G = this.storesXToType[_];
        delete this.storesXToType[_], delete this.storesByType[G];
      }
      if (this.storesYToType[H]) {
        const G = this.storesYToType[H];
        delete this.storesYToType[H], delete this.storesByType[G];
      }
      this.stores[H] && delete this.stores[H][_], Object.keys(this.stores[H] || {}).length || delete this.stores[H];
    }
    setEditByCell(_, H) {
      this.focusByCell(_, H, H), this.setEdit("");
    }
    /**
     * Sets the next focus cell before the current one.
     *
     * @param focus - The cell to set as the next focus.
     */
    beforeNextFocusCell(_) {
      var H;
      if (!this.focusedStore)
        return;
      const V = this.focusedStore.entity.store.get("lastCell"), q = V && this.getNextStore(_, this.focusedStore.position, V);
      (H = q == null ? void 0 : q.store) === null || H === void 0 || H.setNextFocus(Object.assign(Object.assign({}, _), q.item));
    }
    focusByCell(_, H, V) {
      const q = this.stores[_.y][_.x];
      this.focus(q, { focus: H, end: V });
    }
    focus(_, { focus: H, end: V }) {
      const q = this.getCurrentStorePointer(_);
      if (!q)
        return null;
      const G = _.store.get("lastCell"), J = G && this.getNextStore(H, q, G);
      if (J != null && J.store) {
        const Z = Object.assign(Object.assign({}, H), J.item);
        return this.focus(J.store, { focus: Z, end: Z }), null;
      }
      return G && (H = cropCellToMax(H, G), V = cropCellToMax(V, G)), _.setFocus(H, V), H;
    }
    /**
     * Retrieves the current store pointer based on the active store.
     * Clears focus from all stores except the active one.
     */
    getCurrentStorePointer(_) {
      let H;
      for (let V in this.stores)
        for (let q in this.stores[V]) {
          const G = this.stores[V][q];
          G !== _ ? G.clearFocus() : H = {
            x: parseInt(q, 10),
            y: parseInt(V, 10)
          };
        }
      return H;
    }
    /**
     * Retrieves the next store based on the focus cell and current store pointer.
     * If the next store exists, returns an object with the next store and the item in the new store.
     * If the next store does not exist, returns null.
     */
    getNextStore(_, H, V) {
      const q = nextCell(_, V);
      let G;
      return q && Object.entries(q).forEach(([J, Z]) => {
        let te;
        switch (J) {
          case "x":
            te = this.getXStores(H.y);
            break;
          case "y":
            te = this.getYStores(H.x);
            break;
        }
        if (Z >= 0)
          G = te[++H[J]];
        else {
          G = te[--H[J]];
          const ne = G == null ? void 0 : G.store.get("lastCell");
          ne && (q[J] = ne[J] + Z);
        }
      }), {
        store: G,
        item: q
      };
    }
    clearAll() {
      var _;
      for (let H in this.stores)
        for (let V in this.stores[H])
          (_ = this.stores[H][V]) === null || _ === void 0 || _.clearFocus();
    }
    setEdit(_) {
      this.focusedStore && this.focusedStore.entity.setEdit(_);
    }
    /**
     * Select all cells across all stores
     */
    selectAll() {
      for (let _ in this.stores)
        for (let H in this.stores[_]) {
          const V = this.stores[_][H];
          if (!V)
            continue;
          const q = V.store.get("lastCell");
          q && V.setRange({ x: 0, y: 0 }, { x: q.x - 1, y: q.y - 1 });
        }
    }
    getXStores(_) {
      return this.stores[_];
    }
    getYStores(_) {
      const H = {};
      for (let V in this.stores)
        H[V] = this.stores[V][_];
      return H;
    }
  }
  function isHiddenStore(O) {
    return O === EMPTY_INDEX;
  }
  function nextCell(O, _) {
    const H = {};
    let V = ["x", "y"];
    for (let q of V)
      if (O[q] < 0)
        return H[q] = O[q], H;
    for (let q of V)
      if (O[q] >= _[q])
        return H[q] = O[q] - _[q], H;
    return null;
  }
  function cropCellToMax(O, _) {
    const H = Object.assign({}, O), V = ["x", "y"];
    for (const q of V)
      O[q] < 0 ? H[q] = 0 : O[q] >= _[q] && (H[q] = _[q] - 1);
    return H;
  }
  function getRange(O, _) {
    return O && _ ? {
      x: Math.min(O.x, _.x),
      y: Math.min(O.y, _.y),
      x1: Math.max(O.x, _.x),
      y1: Math.max(O.y, _.y)
    } : null;
  }
  function isRangeSingleCell(O) {
    return O.x === O.x1 && O.y === O.y1;
  }
  const GROUP_DEPTH = `${GRID_INTERNALS}-depth`, PSEUDO_GROUP_ITEM = `${GRID_INTERNALS}-name`, PSEUDO_GROUP_ITEM_ID = `${GRID_INTERNALS}-id`, PSEUDO_GROUP_ITEM_VALUE = `${GRID_INTERNALS}-value`, PSEUDO_GROUP_COLUMN = `${GRID_INTERNALS}-column`, GROUP_EXPANDED = `${GRID_INTERNALS}-expanded`, GROUP_COLUMN_PROP = `${GRID_INTERNALS}-prop`, GROUP_ORIGINAL_INDEX = `${GRID_INTERNALS}-original-index`, GROUP_EXPAND_BTN = "group-expand", GROUPING_ROW_TYPE = "rgRow";
  function getGroupValueDefault(O, _) {
    return O[_] || null;
  }
  function getSource(O, _, H = !1) {
    let V = 0;
    const q = {
      source: [],
      prevExpanded: {},
      oldNewIndexes: {}
    };
    return _.forEach((G) => {
      const J = O[G];
      if (!H) {
        q.source.push(J);
        return;
      }
      isGrouping(J) ? getExpanded(J) && (q.prevExpanded[J[PSEUDO_GROUP_ITEM_VALUE]] = !0) : (q.source.push(J), q.oldNewIndexes[G] = V, V++);
    }), q;
  }
  function getExpanded(O = {}) {
    return O[GROUP_EXPANDED];
  }
  function gatherGrouping(O, _, { prevExpanded: H, expandedAll: V, getGroupValue: q = getGroupValueDefault }) {
    const G = /* @__PURE__ */ new Map();
    O.forEach((se, ce) => {
      const le = _.map((he) => q(se, he)), ue = le.pop();
      let de = G;
      le.forEach((he) => {
        de.has(he) || de.set(he, /* @__PURE__ */ new Map()), de = de.get(he);
      }), de.has(ue) || de.set(ue, []), de.get(ue).push(Object.assign(Object.assign({}, se), { [GROUP_ORIGINAL_INDEX]: ce }));
    });
    let J = -1;
    const Z = _.length, te = {}, ne = {}, ee = {}, ae = [];
    function oe(se, ce, le) {
      const ue = ce.length;
      se.forEach((de, fe) => {
        const he = [...ce, fe], me = he.join(","), pe = le && (!!V || !!(H != null && H[me]));
        ae.push({
          [PSEUDO_GROUP_ITEM]: fe,
          [GROUP_DEPTH]: ue,
          [PSEUDO_GROUP_ITEM_ID]: JSON.stringify(he),
          [PSEUDO_GROUP_ITEM_VALUE]: me,
          [GROUP_EXPANDED]: pe,
          [GROUP_COLUMN_PROP]: _[ue],
          [_[ue]]: fe
        }), J += 1, !pe && ue && (te[J] = !0), Array.isArray(de) ? (de.forEach((ye) => {
          J += 1, pe || (te[J] = !0), ne[ye[GROUP_ORIGINAL_INDEX]] = J, he.map((ve, be) => he.slice(0, be + 1).join(",")).forEach((ve) => {
            ee[ve] || (ee[ve] = []), ee[ve].push(J);
          });
        }), ae.push(...de)) : oe(de, he, pe);
      });
    }
    return oe(G, [], !0), {
      sourceWithGroups: ae,
      // updates source mirror
      depth: Z,
      // largest depth for grouping
      trimmed: te,
      // used for expand/collapse grouping values
      oldNewIndexMap: ne,
      // used for mapping old values to new
      childrenByGroup: ee
      // used to get child items in group
    };
  }
  function getGroupingName(O) {
    return O && O[PSEUDO_GROUP_ITEM];
  }
  function isGrouping(O) {
    return typeof (O == null ? void 0 : O[PSEUDO_GROUP_ITEM]) < "u";
  }
  function isGroupingColumn(O) {
    return typeof (O == null ? void 0 : O[PSEUDO_GROUP_COLUMN]) < "u";
  }
  function measureEqualDepth(O, _) {
    const H = O.length;
    let V = 0;
    for (; V < H; V++)
      if (O[V] !== _[V])
        return V;
    return V;
  }
  function getParsedGroup(O) {
    const _ = JSON.parse(O);
    return Array.isArray(_) ? _ : null;
  }
  function isSameGroup(O, _, H) {
    const V = getParsedGroup(H[PSEUDO_GROUP_ITEM_ID]);
    if (!V)
      return !1;
    const q = measureEqualDepth(O, V);
    return _[GROUP_DEPTH] < q;
  }
  function baseSlice(O, _, H) {
    var V = -1, q = O.length;
    _ < 0 && (_ = -_ > q ? 0 : q + _), H = H > q ? q : H, H < 0 && (H += q), q = _ > H ? 0 : H - _ >>> 0, _ >>>= 0;
    for (var G = Array(q); ++V < q; )
      G[V] = O[V + _];
    return G;
  }
  function slice$1(O, _, H) {
    var V = O == null ? 0 : O.length;
    return V ? (H && typeof H != "number" && isIterateeCall(O, _, H) ? (_ = 0, H = V) : (_ = _ == null ? 0 : toInteger(_), H = H === void 0 ? V : toInteger(H)), baseSlice(O, _, H)) : [];
  }
  class ColumnService {
    get columns() {
      return getVisibleSourceItem(this.source);
    }
    constructor(_, H) {
      this.dataStore = _, this.source = H, this.unsubscribe = [], this.hasGrouping = !1, this.unsubscribe.push(H.onChange("source", (V) => this.checkGrouping(V))), this.checkGrouping(H.get("source")), this.type = H.get("type");
    }
    checkGrouping(_) {
      for (let H of _) {
        if (isGroupingColumn(H)) {
          this.hasGrouping = !0;
          return;
        }
        this.hasGrouping = !1;
      }
    }
    isReadOnly(_, H) {
      var V;
      const q = (V = this.columns[H]) === null || V === void 0 ? void 0 : V.readonly;
      if (typeof q == "function") {
        const G = this.rowDataModel(_, H);
        return q(G);
      }
      return !!q;
    }
    mergeProperties(_, H, V, q) {
      var G, J;
      const Z = Object.assign({}, V);
      Z.class = Object.assign(Object.assign({}, typeof Z.class == "string" ? { [Z.class]: !0 } : Z.class), { [CELL_CLASS]: !0, [DISABLED_CLASS]: this.isReadOnly(_, H) });
      const te = (J = (G = q.column) === null || G === void 0 ? void 0 : G.cellProperties) === null || J === void 0 ? void 0 : J.call(G, q);
      return te ? doPropMerge(Z, te) : Z;
    }
    getRowClass(_, H) {
      return (getSourceItem(this.dataStore, _) || {})[H] || "";
    }
    getSaveData(_, H, V) {
      const q = this.rowDataModel(_, H);
      return typeof V > "u" && (V = getCellData(q.value)), Object.assign(Object.assign({}, q), { val: V });
    }
    getCellEditor(_, H, V) {
      var q;
      const G = (q = this.columns[H]) === null || q === void 0 ? void 0 : q.editor;
      if (G)
        return typeof G == "string" ? V[G] : G;
    }
    /**
     * Get cell data model for given rowIndex and colIndex
     * Used to pass data to editor/renderer
     */
    rowDataModel(_, H) {
      const V = this.columns[H], q = V == null ? void 0 : V.prop, G = getSourceItem(this.dataStore, _) || {}, J = this.dataStore.get("type");
      return {
        prop: q,
        model: G,
        data: this.dataStore.get("source"),
        column: V,
        rowIndex: _,
        colIndex: H,
        colType: this.type,
        type: J,
        value: getCellRaw(G, V)
      };
    }
    getRangeData(_, H) {
      var V;
      const q = {}, G = _.oldRange.x1 - _.oldRange.x + 1, J = _.oldRange.y1 - _.oldRange.y + 1, Z = {};
      for (let te = _.newRange.y, ne = 0; te < _.newRange.y1 + 1; te++, ne++) {
        const ee = _.oldRange.y + ne % J, ae = getSourceItem(this.dataStore, ee) || {};
        for (let oe = _.newRange.x, se = 0; oe < _.newRange.x1 + 1; oe++, se++) {
          if (te >= _.oldRange.y && te <= _.oldRange.y1 && oe >= _.oldRange.x && oe <= _.oldRange.x1 || !this.columns[oe])
            continue;
          const ce = (V = this.columns[oe]) === null || V === void 0 ? void 0 : V.prop, le = _.oldRange.x + se % G, ue = H[le].prop;
          this.isReadOnly(te, oe) || (q[te] || (q[te] = {}), q[te][ce] = ae[ue], Z[te] || (Z[te] = {}), Z[te][ce] = {
            colIndex: le,
            colProp: ue,
            rowIndex: ee
          });
        }
      }
      return {
        changed: q,
        mapping: Z
      };
    }
    getTransformedDataToApply(_, H) {
      const V = {}, q = H.length, G = this.columns.length, J = this.dataStore.get("items").length;
      let Z = _.y, te = 0;
      for (let ee = 0; Z < J && ee < q; Z++, ee++) {
        const ae = H[ee % q], oe = (ae == null ? void 0 : ae.length) || 0;
        let se = _.x;
        for (let ce = 0; se < G && ce < oe; se++, ce++) {
          const le = this.columns[se].prop, ue = ce % G;
          this.isReadOnly(Z, se) || (V[Z] || (V[Z] = {}), V[Z][le] = ae[ue]);
        }
        te = Math.max(te, se - 1);
      }
      const ne = getRange(_, {
        y: Z - 1,
        x: te
      });
      return {
        changed: V,
        range: ne
      };
    }
    getRangeStaticData(_, H) {
      const V = {};
      for (let q = _.y, G = 0; q < _.y1 + 1; q++, G++)
        for (let J = _.x, Z = 0; J < _.x1 + 1; J++, Z++) {
          if (!this.columns[J])
            continue;
          const te = this.columns[J].prop;
          this.isReadOnly(q, J) || (V[q] || (V[q] = {}), V[q][te] = H);
        }
      return V;
    }
    getRangeTransformedToProps(_, H) {
      var V;
      const q = [], G = this.dataStore.get("type");
      for (let J = _.y, Z = 0; J < _.y1 + 1; J++, Z++)
        for (let te = _.x, ne = 0; te < _.x1 + 1; te++, ne++) {
          const ee = (V = this.columns[te]) === null || V === void 0 ? void 0 : V.prop;
          q.push({
            prop: ee,
            rowIndex: J,
            colIndex: te,
            model: getSourceItem(H, J),
            type: G,
            colType: this.type
          });
        }
      return q;
    }
    copyRangeArray(_, H) {
      const V = [...this.columns], q = slice$1(V, _.x, _.x1 + 1).map((Z) => Z.prop), G = [], J = {};
      for (let Z = _.y; Z <= _.y1; Z++) {
        const te = [];
        J[Z] = {};
        for (let ne of q) {
          const ee = getSourceItem(H, Z);
          if (!ee)
            continue;
          const ae = ee[ne];
          te.push(ae), J[Z][ne] = ae;
        }
        G.push(te);
      }
      return {
        data: G,
        mapping: J
      };
    }
    destroy() {
      this.unsubscribe.forEach((_) => _());
    }
  }
  function isRowDragService(O, _) {
    return typeof O == "function" ? O(_) : !!O;
  }
  function mergeClasses(O = {}, _ = {}) {
    return typeof O == "string" && (O = { [O]: !0 }), typeof _ == "string" && (_ = { [_]: !0 }), Object.assign(Object.assign({}, O), _);
  }
  function doPropMerge(O, _) {
    _.className && (_.class = mergeClasses(_.class, _.className), delete _.className);
    let H = Object.assign(Object.assign({}, _), O);
    return _.class && (H.class = mergeClasses(H.class, _.class)), _.style && (H.style = Object.assign(Object.assign({}, _.style), H.style)), H;
  }
  /*!
   * Built by Revolist OU ❤️
   */
  const LETTER_BLOCK_SIZE$1 = 10, calculateRowHeaderSize = (O, _, H = 50) => (_ == null ? void 0 : _.size) || Math.max((O.toString().length + 1) * LETTER_BLOCK_SIZE$1, H);
  function getUpdatedItemsByPosition(O, _, H, V, q) {
    const G = getItemByPosition(q, O), J = getFirstItem(_);
    let Z;
    if (J) {
      let ne = G.itemIndex - (J.itemIndex || 0);
      ne && (Z = recombineByOffset(Math.abs(ne), Object.assign(Object.assign({ positiveDirection: ne > -1 }, q), _)));
    }
    const te = getMaxVirtualSize(V, q.realSize, G);
    if (Z) {
      const ne = addMissingItems(G, H, te, Z, q);
      ne.length && updateMissingAndRange(Z.items, ne, Z);
    }
    if (!Z) {
      const ne = getItems({
        firstItemStart: G.start,
        firstItemIndex: G.itemIndex,
        origSize: q.originItemSize,
        maxSize: te,
        maxCount: H,
        sizes: q.sizes
      });
      Z = {
        items: ne,
        start: 0,
        end: ne.length - 1
      };
    }
    return Z;
  }
  function getMaxVirtualSize(O, _, H) {
    return Math.min(O + (H.end - H.start), _);
  }
  function updateMissingAndRange(O, _, H) {
    O.splice(H.end + 1, 0, ..._), H.start >= H.end && !(H.start === H.end && H.start === 0) && (H.start += _.length), H.end += _.length;
  }
  function addMissingItems(O, _, H, V, q) {
    const G = getLastItem(V);
    return getItems({
      sizes: q.sizes,
      firstItemStart: G.end,
      firstItemIndex: G.itemIndex + 1,
      origSize: q.originItemSize,
      maxSize: H - (G.end - O.start),
      maxCount: _
    });
  }
  function getItems(O, _ = 0) {
    const H = [];
    let V = O.firstItemIndex, q = _;
    for (; q <= O.maxSize && V < O.maxCount; ) {
      const G = getItemSize(V, O.sizes, O.origSize);
      H.push({
        start: O.firstItemStart + q,
        end: O.firstItemStart + q + G,
        itemIndex: V,
        size: G
      }), q += G, V++;
    }
    return H;
  }
  function recombineByOffset(O, _) {
    var H, V;
    const q = [..._.items], G = q.length;
    let J = {
      start: _.start,
      end: _.end
    };
    if (O > G)
      return;
    if (_.positiveDirection) {
      let te = getLastItem(_), ne = J.start;
      const ee = ne + O;
      for (; ne < ee; ne++) {
        const ae = te.itemIndex + 1, oe = getItemSize(ae, _.sizes, _.originItemSize);
        if (te.end + oe > _.realSize)
          break;
        let se = ne % G;
        if (!q[se])
          throw new Error("incorrect index");
        q[se] = te = {
          start: te.end,
          end: te.end + oe,
          itemIndex: ae,
          size: oe
        }, J.start++, J.end = se;
      }
    } else {
      let te = getFirstItem(_);
      const ne = J.end;
      for (let ee = 0; ee < O; ee++) {
        const ae = ((H = te == null ? void 0 : te.itemIndex) !== null && H !== void 0 ? H : 0) - 1, oe = getItemSize(ae, _.sizes, _.originItemSize);
        let se = ne - ee;
        if (se = (se < 0 ? G + se : se) % G, !q[se]) {
          console.error("incorrect index");
          break;
        }
        const ce = (V = te == null ? void 0 : te.start) !== null && V !== void 0 ? V : 0;
        q[se] = te = {
          start: ce - oe,
          end: ce,
          itemIndex: ae,
          size: oe
        }, J.start = se, J.end--;
      }
    }
    const Z = {
      start: (J.start < 0 ? G + J.start : J.start) % G,
      end: (J.end < 0 ? G + J.end : J.end) % G
    };
    return Object.assign({ items: q }, Z);
  }
  function getItemSize(O, _, H = 0) {
    return _ && _[O] ? _[O] : H;
  }
  function isActiveRange(O, _, H, V) {
    return !H || !V ? !1 : O >= H.start && O <= H.end || O > H.end && V.end === _;
  }
  function isActiveRangeOutsideLastItem(O, _, H, V) {
    var q;
    return H ? _ + O > ((q = V == null ? void 0 : V.end) !== null && q !== void 0 ? q : 0) : !1;
  }
  function getFirstItem(O) {
    return O.items[O.start];
  }
  function getLastItem(O) {
    return O.items[O.end];
  }
  function setItemSizes(O, _, H, V) {
    const q = [...O], G = q.length;
    let J = V, Z = 0, te = _;
    if (!G)
      return [];
    for (; Z < G; ) {
      const ne = q[te];
      ne.start = J, ne.size = H, ne.end = ne.start + H, J = ne.end, te++, Z++, te === G && (te = 0);
    }
    return q;
  }
  function initialState$1() {
    return {
      // virtual item information per rendered item
      items: [],
      // virtual dom item order to render
      start: 0,
      end: 0,
      // size of virtual viewport in px
      virtualSize: 0,
      // total number of items
      realCount: 0,
      // size of viewport in px
      clientSize: 0
    };
  }
  class ViewportStore {
    get lastCoordinate() {
      return this.lastKnownScroll;
    }
    set lastCoordinate(_) {
      this.lastKnownScroll = _;
    }
    constructor(_) {
      this.type = _, this.lastKnownScroll = 0, this.store = createStore(initialState$1());
    }
    /**
     * Render viewport based on coordinate
     * It's the main method for draw
     * Use force if you want to re-render viewport
     */
    setViewPortCoordinate(_, H, V = !1) {
      const q = this.store.get("virtualSize");
      if (!q)
        return;
      const J = H.originItemSize * 1, Z = J * 2, te = q + Z;
      let ne = 0;
      H.realSize > q && (ne = H.realSize - q - J);
      let ee = _;
      ee < 0 ? ee = 0 : ee > ne && (ee = ne), this.lastCoordinate = ee, ee -= J, ee = ee < 0 ? 0 : ee < ne ? ee : ne;
      let ae;
      V ? ae = {
        items: [],
        start: 0,
        end: 0
      } : ae = this.getItems();
      const oe = getFirstItem(ae), se = getLastItem(ae);
      let ce = {};
      if (!isActiveRange(ee, H.realSize, oe, se))
        ce = Object.assign(Object.assign({}, ce), getUpdatedItemsByPosition(ee, ae, this.store.get("realCount"), te, H)), this.setViewport(Object.assign({}, ce));
      else if (isActiveRangeOutsideLastItem(ee, te, oe, se)) {
        const le = [...ae.items], ue = addMissingItems(oe, this.store.get("realCount"), te + ee - oe.start, ae, {
          sizes: H.sizes,
          originItemSize: H.originItemSize
        });
        if (ue.length) {
          const de = {
            start: this.store.get("start"),
            end: this.store.get("end")
          };
          updateMissingAndRange(le, ue, de), ce = Object.assign(Object.assign(Object.assign({}, ce), { items: [...le] }), de), this.setViewport(Object.assign({}, ce));
        }
      }
    }
    /**
     * Set sizes for existing items
     */
    setOriginalSizes(_) {
      const H = this.store.get("items");
      H.length && setStore(this.store, {
        items: setItemSizes(H, this.store.get("start"), _, this.lastCoordinate)
      });
    }
    getItems() {
      return {
        items: this.store.get("items"),
        start: this.store.get("start"),
        end: this.store.get("end")
      };
    }
    setViewport(_) {
      (typeof _.realCount == "number" || typeof _.virtualSize == "number") && (_ = Object.assign(Object.assign({}, _), { items: _.items || [] })), setStore(this.store, _);
    }
  }
  /*!
   * Built by Revolist OU ❤️
   */
  const FILTER_BUTTON_CLASS = "rv-filter", FILTER_BUTTON_ACTIVE = "active", FILTER_PROP = "hasFilter", AND_OR_BUTTON = "and-or-button", TRASH_BUTTON = "trash-button", FilterButton = ({ column: O }) => h(
    "span",
    null,
    h(
      "button",
      { class: {
        [FILTER_BUTTON_CLASS]: !0,
        [FILTER_BUTTON_ACTIVE]: O && !!O[FILTER_PROP]
      } },
      h(
        "svg",
        { class: "filter-img", viewBox: "0 0 64 64" },
        h(
          "g",
          { stroke: "none", "stroke-width": "1", fill: "none", "fill-rule": "evenodd" },
          h("path", { d: "M43,48 L43,56 L21,56 L21,48 L43,48 Z M53,28 L53,36 L12,36 L12,28 L53,28 Z M64,8 L64,16 L0,16 L0,8 L64,8 Z", fill: "currentColor" })
        )
      )
    )
  ), TrashButton = () => h(
    "div",
    { class: { [TRASH_BUTTON]: !0 } },
    h(
      "svg",
      { class: "trash-img", viewBox: "0 0 24 24" },
      h("path", { fill: "currentColor", d: "M9,3V4H4V6H5V19A2,2 0 0,0 7,21H17A2,2 0 0,0 19,19V6H20V4H15V3H9M7,6H17V19H7V6M9,8V17H11V8H9M13,8V17H15V8H13Z" })
    )
  ), AndOrButton = ({ text: O }) => h("button", { class: { [AND_OR_BUTTON]: !0, "light revo-button": !0 } }, O);
  function isFilterBtn(O) {
    return O.classList.contains(FILTER_BUTTON_CLASS) ? !0 : O == null ? void 0 : O.closest(`.${FILTER_BUTTON_CLASS}`);
  }
  /*!
   * Built by Revolist OU ❤️
   */
  function dispatch(O, _, H) {
    const V = new CustomEvent(_, {
      detail: H,
      cancelable: !0,
      // Indicates whether the event can be canceled.
      bubbles: !0
      // Indicates whether the event bubbles up through the DOM.
    });
    return O == null || O.dispatchEvent(V), V;
  }
  var ResizeEvents;
  (function(O) {
    O.start = "resize:start", O.move = "resize:move", O.end = "resize:end";
  })(ResizeEvents || (ResizeEvents = {}));
  const RESIZE_MASK = {
    "resizable-r": { bit: 1, cursor: "ew-resize" },
    "resizable-rb": { bit: 3, cursor: "se-resize" },
    "resizable-b": { bit: 2, cursor: "s-resize" },
    "resizable-lb": { bit: 6, cursor: "sw-resize" },
    "resizable-l": { bit: 4, cursor: "w-resize" },
    "resizable-lt": { bit: 12, cursor: "nw-resize" },
    "resizable-t": { bit: 8, cursor: "n-resize" },
    "resizable-rt": { bit: 9, cursor: "ne-resize" }
  }, DISABLE_MASK = {
    l: 1,
    t: 2,
    w: 4,
    h: 8
  }, defaultProps = (O) => Object.assign(Object.assign({}, O), { fitParent: O.fitParent || !1, active: O.active || [], disableAttributes: O.disableAttributes || [], minWidth: O.minWidth || 0, minHeight: O.minHeight || 0 });
  class ResizeDirective {
    constructor(_, H) {
      var V, q;
      this.initialProps = _, this.$event = H, this.mouseX = 0, this.mouseY = 0, this.width = 0, this.height = 0, this.changeX = 0, this.changeY = 0, this.disableCalcMap = 15, this.props = defaultProps(_), this.mouseMoveFunc = this.handleMove.bind(this), this.mouseUpFunc = this.handleUp.bind(this), this.minW = this.props.minWidth, this.minH = this.props.minHeight, this.maxW = (V = this.props.maxWidth) !== null && V !== void 0 ? V : 0, this.maxH = (q = this.props.maxHeight) !== null && q !== void 0 ? q : 0, this.parent = { width: 0, height: 0 }, this.resizeState = 0;
    }
    set(_) {
      this.$el = _, this.props.disableAttributes.forEach((H) => {
        switch (H) {
          case "l":
            this.disableCalcMap &= -2;
            break;
          case "t":
            this.disableCalcMap &= -3;
            break;
          case "w":
            this.disableCalcMap &= -5;
            break;
          case "h":
            this.disableCalcMap &= -9;
        }
      });
    }
    emitEvent(_, H) {
      var V;
      if (!this.$event)
        return;
      const q = (V = this.activeResizer) === null || V === void 0 ? void 0 : V.classList.contains("resizable-l");
      this.$event(Object.assign({ eventName: _, width: this.width + this.changeX * (q ? -1 : 1), height: this.height + this.changeY, changedX: this.changeX, changedY: this.changeY }, H));
    }
    static isTouchEvent(_) {
      var H;
      return ((H = _.touches) === null || H === void 0 ? void 0 : H.length) >= 0;
    }
    handleMove(_) {
      var H;
      if (!this.resizeState)
        return;
      let V, q;
      ResizeDirective.isTouchEvent(_) ? (V = _.touches[0].clientY, q = _.touches[0].clientX) : (V = _.clientY, q = _.clientX);
      let G = this.resizeState & RESIZE_MASK["resizable-r"].bit || this.resizeState & RESIZE_MASK["resizable-l"].bit;
      if ((this.resizeState & RESIZE_MASK["resizable-t"].bit || this.resizeState & RESIZE_MASK["resizable-b"].bit) && this.disableCalcMap & DISABLE_MASK.h) {
        let Z = V - this.mouseY, te = this.changeY + Z;
        const ne = this.height + te;
        ne < this.minH && (te = -(this.height - this.minH)), this.maxH && ne > this.maxH && (te = this.maxH - this.height), this.changeY = te, this.mouseY = V, this.activeResizer && (this.activeResizer.style.bottom = `${-this.changeY}px`);
      }
      if (G && this.disableCalcMap & DISABLE_MASK.w) {
        const Z = (H = this.activeResizer) === null || H === void 0 ? void 0 : H.classList.contains("resizable-l");
        let te = q - this.mouseX, ne = this.changeX + te;
        const ee = this.width + ne * (Z ? -1 : 1);
        ee < this.minW && (ne = -(this.width - this.minW)), this.maxW && ee > this.maxW && (ne = this.maxW - this.width), this.changeX = ne, this.mouseX = q, this.activeResizer && (Z ? this.activeResizer.style.left = `${this.changeX}px` : this.activeResizer.style.right = `${-this.changeX}px`);
      }
      this.emitEvent(ResizeEvents.move);
    }
    handleDown(_) {
      if (!_.defaultPrevented) {
        _.preventDefault(), this.dropInitial();
        for (let H in RESIZE_MASK) {
          const V = _.target;
          if (this.$el.contains(V) && (V != null && V.classList.contains(H))) {
            document.body.style.cursor = RESIZE_MASK[H].cursor, ResizeDirective.isTouchEvent(_) ? this.setInitials(_.touches[0], V) : (_.preventDefault && _.preventDefault(), this.setInitials(_, V)), this.resizeState = RESIZE_MASK[H].bit;
            const q = ResizeEvents.start;
            this.emitEvent(q);
            break;
          }
        }
        this.bindMove();
      }
    }
    handleUp(_) {
      if (_.preventDefault(), this.resizeState !== 0) {
        this.resizeState = 0, document.body.style.cursor = "";
        const H = ResizeEvents.end;
        this.emitEvent(H);
      }
      this.dropInitial(), this.unbindMove();
    }
    setInitials({ clientX: _, clientY: H }, V) {
      var q, G, J, Z;
      const te = getComputedStyle(this.$el);
      if (this.$el.classList.add("active"), this.activeResizer = V, this.disableCalcMap & DISABLE_MASK.w) {
        this.mouseX = _, this.width = this.$el.clientWidth, this.parent.width = (G = (q = this.$el.parentElement) === null || q === void 0 ? void 0 : q.clientWidth) !== null && G !== void 0 ? G : 0;
        const ne = parseFloat(te.paddingLeft) + parseFloat(te.paddingRight);
        this.minW = Math.max(ne, this.initialProps.minWidth || 0), this.initialProps.maxWidth && (this.maxW = Math.max(this.width, this.initialProps.maxWidth));
      }
      if (this.disableCalcMap & DISABLE_MASK.h) {
        this.mouseY = H, this.height = this.$el.clientHeight, this.parent.height = (Z = (J = this.$el.parentElement) === null || J === void 0 ? void 0 : J.clientHeight) !== null && Z !== void 0 ? Z : 0;
        const ne = parseFloat(te.paddingTop) + parseFloat(te.paddingBottom);
        this.minH = Math.max(ne, this.initialProps.minHeight || 0), this.initialProps.maxHeight && (this.maxH = Math.max(this.height, this.initialProps.maxHeight));
      }
    }
    dropInitial() {
      this.changeX = this.changeY = this.minW = this.minH, this.width = this.height = 0, this.activeResizer && this.activeResizer.removeAttribute("style"), this.$el.classList.remove("active"), this.activeResizer = void 0;
    }
    bindMove() {
      document.documentElement.addEventListener("mouseup", this.mouseUpFunc, !0), document.documentElement.addEventListener("touchend", this.mouseUpFunc, !0), document.documentElement.addEventListener("mousemove", this.mouseMoveFunc, !0), document.documentElement.addEventListener("touchmove", this.mouseMoveFunc, !0), document.documentElement.addEventListener("mouseleave", this.mouseUpFunc);
    }
    unbindMove() {
      document.documentElement.removeEventListener("mouseup", this.mouseUpFunc, !0), document.documentElement.removeEventListener("touchend", this.mouseUpFunc, !0), document.documentElement.removeEventListener("mousemove", this.mouseMoveFunc, !0), document.documentElement.removeEventListener("touchmove", this.mouseMoveFunc, !0), document.documentElement.removeEventListener("mouseleave", this.mouseUpFunc);
    }
  }
  const ResizableElement = (O, _) => {
    const H = [], V = O.canResize && new ResizeDirective(O, (q) => {
      var G;
      q.eventName === ResizeEvents.end && ((G = O.onResize) === null || G === void 0 || G.call(O, q));
    }) || null;
    if (O.active)
      if (O.canResize)
        for (let q in O.active)
          H.push(h("div", { onClick: (G) => G.preventDefault(), onDblClick: (G) => {
            var J;
            G.preventDefault(), (J = O.onDblClick) === null || J === void 0 || J.call(O, G);
          }, onMouseDown: (G) => V == null ? void 0 : V.handleDown(G), onTouchStart: (G) => V == null ? void 0 : V.handleDown(G), class: `resizable resizable-${O.active[q]}` }));
      else
        for (let q in O.active)
          H.push(h("div", { onClick: (G) => G.preventDefault(), onTouchStart: (G) => G.preventDefault(), onDblClick: (G) => {
            var J;
            G.preventDefault(), (J = O.onDblClick) === null || J === void 0 || J.call(O, G);
          }, class: `no-resize resizable resizable-${O.active[q]}` }));
    return h(
      "div",
      Object.assign({}, O, { ref: (q) => q && (V == null ? void 0 : V.set(q)) }),
      _,
      H
    );
  }, ON_COLUMN_CLICK = "columnclick", HeaderCellRenderer = ({ data: O, props: _, additionalData: H }, V) => {
    let q = (O == null ? void 0 : O.name) || "", G = _;
    if (O != null && O.columnTemplate && (q = O.columnTemplate(h, O, H)), O != null && O.columnProperties) {
      const Z = O.columnProperties(O);
      Z && (G = doPropMerge(_, Z));
    }
    const J = Object.assign(Object.assign({}, G), { onMouseDown(Z) {
      dispatch(Z.currentTarget, ON_COLUMN_CLICK, {
        data: O,
        event: Z
      });
    } });
    return h(
      ResizableElement,
      Object.assign({}, J),
      h("div", { class: "header-content" }, q),
      V
    );
  };
  /*!
   * Built by Revolist OU ❤️
   */
  const recalculateRealSizePlugin = (O) => ({
    /**
     * Reacts on changes of count, sizes and originItemSize
     */
    set(_) {
      switch (_) {
        case "count":
        case "sizes":
        case "originItemSize": {
          let H = 0;
          const V = O.store.get("count");
          for (let q = 0; q < V; q++)
            H += O.store.get("sizes")[q] || O.store.get("originItemSize");
          O.setStore({ realSize: H });
          break;
        }
      }
    }
  }), trimmedPlugin = (O) => {
    let _ = null, H = null;
    return {
      set(V, q) {
        switch (V) {
          case "sizes": {
            if (_ && _ === q) {
              _ = null;
              return;
            }
            H = null;
            break;
          }
          case "trimmed": {
            const G = q;
            H || (H = O.store.get("sizes")), _ = removeMultipleAndShift(H, G || {}), O.setSizes(_);
            break;
          }
        }
      }
    };
  };
  function removeMultipleAndShift(O, _) {
    const H = {}, V = Object.keys(O || {}).map(Number).sort((J, Z) => J - Z), q = V[V.length - 1];
    let G = 0;
    for (let J = 0; J <= q; J++)
      _[J] !== void 0 && (G++, O[J] !== void 0) || O[J] !== void 0 && (H[J - G] = O[J]);
    return H;
  }
  function initialBase() {
    return {
      indexes: [],
      count: 0,
      // hidden items
      trimmed: null,
      // virtual item index to size
      sizes: {},
      // order in indexes[] to coordinate
      positionIndexToItem: {},
      // initial element to coordinate ^
      indexToItem: {},
      positionIndexes: []
    };
  }
  function initialState() {
    return Object.assign(Object.assign({}, initialBase()), {
      // size which all items can take
      realSize: 0,
      // initial item size if it wasn't changed
      originItemSize: 0
    });
  }
  class DimensionStore {
    constructor(_) {
      this.type = _, this.store = createStore(initialState()), this.store.use(trimmedPlugin({
        store: this.store,
        setSizes: this.setDimensionSize.bind(this)
      })), this.store.use(recalculateRealSizePlugin({
        store: this.store,
        setStore: this.setStore.bind(this)
      }));
    }
    getCurrentState() {
      const _ = initialState(), H = Object.keys(_);
      return reduce$1(H, (V, q) => {
        const G = this.store.get(q);
        return V[q] = G, V;
      }, _);
    }
    dispose() {
      setStore(this.store, initialState());
    }
    setStore(_) {
      setStore(this.store, _);
    }
    drop() {
      setStore(this.store, initialBase());
    }
    /**
     * Set custom dimension sizes and overwrite old
     * Generates new indexes based on sizes
     * @param sizes - sizes to set
     */
    setDimensionSize(_ = {}) {
      const H = calculateDimensionData(this.store.get("originItemSize"), _);
      setStore(this.store, Object.assign(Object.assign({}, H), { sizes: _ }));
    }
    updateSizesPositionByIndexes(_, H = []) {
      const V = Object.assign({}, this.store.get("sizes"));
      if (!Object.keys(V).length)
        return;
      const q = {};
      H.forEach((J, Z) => {
        q[J] || (q[J] = []), q[J].push(Z);
      });
      const G = {};
      _.forEach((J, Z) => {
        const te = q[J];
        if (te && te.length > 0) {
          const ne = te.shift();
          ne !== void 0 && ne !== Z && V[ne] && (G[Z] = V[ne], delete V[ne]);
        }
      }), Object.keys(G).length && this.setDimensionSize(Object.assign(Object.assign({}, V), G));
    }
  }
  const rowTypes = ["rowPinStart", "rgRow", "rowPinEnd"], columnTypes = [
    "colPinStart",
    "rgCol",
    "colPinEnd"
  ];
  function isRowType(O) {
    return rowTypes.indexOf(O) > -1;
  }
  class BasePlugin {
    constructor(_, H) {
      this.revogrid = _, this.providers = H, this.h = h, this.subscriptions = {};
    }
    /**
     *
     * @param eventName - event name to subscribe to in revo-grid component (e.g. 'beforeheaderclick')
     * @param callback - callback function for event
     */
    addEventListener(_, H) {
      this.revogrid.addEventListener(_, H), this.subscriptions[_] = H;
    }
    /**
     * Subscribe to property change in revo-grid component
     * You can return false in callback to prevent default value set
     *
     * @param prop - property name
     * @param callback - callback function
     * @param immediate - trigger callback immediately with current value
     */
    watch(_, H, { immediate: V } = { immediate: !1 }) {
      const q = Object.getOwnPropertyDescriptor(this.revogrid, _) || Object.getOwnPropertyDescriptor(this.revogrid.constructor.prototype, _);
      Object.defineProperty(this.revogrid, _, {
        set(G) {
          var J;
          if (H(G) !== !1)
            return (J = q == null ? void 0 : q.set) === null || J === void 0 ? void 0 : J.call(this, G);
        },
        get() {
          var G;
          return (G = q == null ? void 0 : q.get) === null || G === void 0 ? void 0 : G.call(this);
        }
      }), V && H(q == null ? void 0 : q.value);
    }
    /**
     * Remove event listener
     * @param eventName
     */
    removeEventListener(_) {
      this.revogrid.removeEventListener(_, this.subscriptions[_]), delete this.subscriptions[_];
    }
    /**
     * Emit event from revo-grid component
     * Event can be cancelled by calling event.preventDefault() in callback
     */
    emit(_, H) {
      const V = new CustomEvent(_, { detail: H, cancelable: !0 });
      return this.revogrid.dispatchEvent(V), V;
    }
    /**
     * Clear all subscriptions
     */
    clearSubscriptions() {
      for (let _ in this.subscriptions)
        this.removeEventListener(_);
    }
    /**
     * Destroy plugin and clear all subscriptions
     */
    destroy() {
      this.clearSubscriptions();
    }
  }
  function arrayEach(O, _) {
    for (var H = -1, V = O == null ? 0 : O.length; ++H < V && _(O[H], H, O) !== !1; )
      ;
    return O;
  }
  function castFunction(O) {
    return typeof O == "function" ? O : identity;
  }
  function forEach(O, _) {
    var H = isArray$1(O) ? arrayEach : baseEach$1;
    return H(O, castFunction(_));
  }
  const LETTER_BLOCK_SIZE = 7;
  var ColumnAutoSizeMode;
  (function(O) {
    O.headerClickAutosize = "headerClickAutoSize", O.autoSizeOnTextOverlap = "autoSizeOnTextOverlap", O.autoSizeAll = "autoSizeAll";
  })(ColumnAutoSizeMode || (ColumnAutoSizeMode = {}));
  class AutoSizeColumnPlugin extends BasePlugin {
    constructor(_, H, V) {
      super(_, H), this.providers = H, this.config = V, this.autoSizeColumns = null, this.dataResolve = null, this.dataReject = null, this.letterBlockSize = (V == null ? void 0 : V.letterBlockSize) || LETTER_BLOCK_SIZE, V != null && V.preciseSize && (this.precsizeCalculationArea = this.initiatePresizeElement(), _.appendChild(this.precsizeCalculationArea));
      const q = ({ detail: { source: ne } }) => {
        this.setSource(ne);
      }, G = ({ detail: ne }) => {
        this.afteredit(ne);
      }, J = ({ detail: ne }) => {
        this.afterEditAll(ne);
      }, Z = ({ detail: { columns: ne } }) => {
        this.columnSet(ne);
      }, te = ({ detail: ne }) => {
        const ee = getColumnType(ne.column), ae = this.getColumnSize(ne.index, ee);
        ae && this.providers.dimension.setCustomSizes(ee, {
          [ne.index]: ae
        }, !0);
      };
      switch (this.addEventListener("beforecolumnsset", Z), V == null ? void 0 : V.mode) {
        case ColumnAutoSizeMode.autoSizeOnTextOverlap:
          this.addEventListener("aftersourceset", q), this.addEventListener("afteredit", G);
          break;
        case ColumnAutoSizeMode.autoSizeAll:
          this.addEventListener("aftersourceset", q), this.addEventListener("afteredit", J);
          break;
        default:
          this.addEventListener("headerdblclick", te);
          break;
      }
    }
    async setSource(_) {
      let H = this.autoSizeColumns;
      if (this.dataReject && (this.dataReject(), this.clearPromise()), !H) {
        const V = new Promise((q, G) => {
          this.dataResolve = q, this.dataReject = G;
        });
        try {
          H = await V;
        } catch {
          return;
        }
      }
      forEach(H, (V, q) => {
        const G = {};
        forEach(H[q], (J) => {
          J.size = G[J.index] = _.reduce((Z, te) => Math.max(Z, this.getLength(te[J.prop])), this.getLength(J.name || ""));
        }), this.providers.dimension.setCustomSizes(q, G, !0);
      });
    }
    getLength(_) {
      var H;
      if (!_)
        return 0;
      try {
        const q = _.toString();
        return !((H = this.config) === null || H === void 0) && H.preciseSize ? (this.precsizeCalculationArea.innerText = q, this.precsizeCalculationArea.scrollWidth + 15 * 2) : q.length * this.letterBlockSize + 15 * 2;
      } catch {
        return 0;
      }
    }
    afteredit(_) {
      let H;
      this.isRangeEdit(_) ? H = _.data : H = { 0: { [_.prop]: _.val } }, forEach(this.autoSizeColumns, (V, q) => {
        const G = {};
        forEach(V, (J) => {
          var Z;
          const te = reduce$1(H, (ne, ee) => typeof ee[J.prop] > "u" ? ne : Math.max(ne || 0, this.getLength(ee[J.prop])), void 0);
          te && ((Z = J.size) !== null && Z !== void 0 ? Z : 0) < te && (J.size = G[J.index] = te);
        }), this.providers.dimension.setCustomSizes(q, G, !0);
      });
    }
    afterEditAll(_) {
      const H = {};
      this.isRangeEdit(_) ? forEach(_.data, (V) => forEach(V, (q, G) => H[G] = !0)) : H[_.prop] = !0, forEach(this.autoSizeColumns, (V, q) => {
        const G = {};
        forEach(V, (J) => {
          if (H[J.prop]) {
            const Z = this.getColumnSize(J.index, q);
            Z && (G[J.index] = Z);
          }
        }), this.providers.dimension.setCustomSizes(q, G, !0);
      });
    }
    getColumnSize(_, H) {
      var V, q;
      const G = (q = (V = this.autoSizeColumns) === null || V === void 0 ? void 0 : V[H]) === null || q === void 0 ? void 0 : q[_];
      return G ? reduce$1(this.providers.data.stores, (J, Z) => {
        const te = reduce$1(Z.store.get("items"), (ne, ee, ae) => {
          const oe = getSourceItem(Z.store, ae);
          return Math.max(ne || 0, this.getLength(oe == null ? void 0 : oe[G.prop]));
        }, 0);
        return Math.max(J, te);
      }, G.size || 0) : 0;
    }
    columnSet(_) {
      var H;
      for (let V of columnTypes) {
        const q = V, G = _[q];
        for (let J in G)
          (G[J].autoSize || !((H = this.config) === null || H === void 0) && H.allColumns) && (this.autoSizeColumns || (this.autoSizeColumns = {}), this.autoSizeColumns[q] || (this.autoSizeColumns[q] = {}), this.autoSizeColumns[q][J] = Object.assign(Object.assign({}, G[J]), { index: parseInt(J, 10) }));
      }
      this.dataResolve && (this.dataResolve(this.autoSizeColumns || {}), this.clearPromise());
    }
    clearPromise() {
      this.dataResolve = null, this.dataReject = null;
    }
    isRangeEdit(_) {
      return !!_.data;
    }
    initiatePresizeElement() {
      var _;
      const H = {
        position: "absolute",
        fontSize: "14px",
        height: "0",
        width: "0",
        whiteSpace: "nowrap",
        top: "0",
        overflowX: "scroll"
      }, V = document.createElement("div");
      for (let q in H)
        V.style[q] = (_ = H[q]) !== null && _ !== void 0 ? _ : "";
      return V.classList.add("revo-test-container"), V;
    }
    destroy() {
      var _;
      super.destroy(), (_ = this.precsizeCalculationArea) === null || _ === void 0 || _.remove();
    }
  }
  class StretchColumn extends BasePlugin {
    constructor(_, H) {
      super(_, H), this.providers = H, this.stretchedColumn = null, this.scrollSize = getScrollbarSize(document);
      const V = ({ detail: { columns: q } }) => this.applyStretch(q);
      this.addEventListener("beforecolumnapplied", V);
    }
    setScroll({ type: _, hasScroll: H }) {
      var V;
      _ === "rgRow" && this.stretchedColumn && ((V = this.stretchedColumn) === null || V === void 0 ? void 0 : V.initialSize) === this.stretchedColumn.size && H && (this.stretchedColumn.size -= this.scrollSize, this.apply(), this.dropChanges());
    }
    activateChanges() {
      const _ = ({ detail: H }) => this.setScroll(H);
      this.addEventListener("scrollchange", _);
    }
    dropChanges() {
      this.stretchedColumn = null, this.removeEventListener("scrollchange");
    }
    apply() {
      if (!this.stretchedColumn)
        return;
      const _ = "rgCol", H = this.providers.dimension.stores[_].store.get("sizes");
      this.providers.dimension.setCustomSizes(_, Object.assign(Object.assign({}, H), { [this.stretchedColumn.index]: this.stretchedColumn.size }), !0);
    }
    /**
     * Apply stretch changes
     */
    applyStretch(_) {
      this.dropChanges();
      let H = this.revogrid.clientWidth - 1;
      if (forEach(_, (V, q) => {
        const G = this.providers.dimension.stores[q].store.get("realSize");
        H -= G;
      }), this.revogrid.rowHeaders) {
        const V = this.providers.data.stores.rgRow.store.get("source").length, q = this.revogrid.rowHeaders, G = calculateRowHeaderSize(V, typeof q == "object" ? q : void 0);
        G && (H -= G);
      }
      if (H > 0) {
        const V = _.rgCol.length - 1, q = _.rgCol[V], G = (q == null ? void 0 : q.size) || this.revogrid.colSize || 0, J = H + G - 1;
        q && !q.autoSize && G < J && (this.stretchedColumn = {
          initialSize: J,
          index: V,
          size: J
        }, this.apply(), this.activateChanges());
      }
    }
  }
  function isStretchPlugin(O) {
    return !!O.applyStretch;
  }
  function baseClamp(O, _, H) {
    return O === O && (O = O <= H ? O : H, O = O >= _ ? O : _), O;
  }
  var MAX_ARRAY_LENGTH = 4294967295;
  function toLength(O) {
    return O ? baseClamp(toInteger(O), 0, MAX_ARRAY_LENGTH) : 0;
  }
  function baseFill(O, _, H, V) {
    var q = O.length;
    for (H = toInteger(H), H < 0 && (H = -H > q ? 0 : q + H), V = V === void 0 || V > q ? q : toInteger(V), V < 0 && (V += q), V = H > V ? 0 : toLength(V); H < V; )
      O[H++] = _;
    return O;
  }
  function fill(O, _, H, V) {
    var q = O == null ? 0 : O.length;
    return q ? baseFill(O, _, H, V) : [];
  }
  const INITIAL = {
    mime: "text/csv",
    fileKind: "csv",
    // BOM signature
    bom: !0,
    columnDelimiter: ",",
    rowDelimiter: `\r
`,
    encoding: ""
  }, CARRIAGE_RETURN = "\r", LINE_FEED = `
`, DOUBLE_QT = '"', NO_BREAK_SPACE = "\uFEFF", escapeRegex = new RegExp('"', "g");
  class ExportCsv {
    constructor(_ = {}) {
      this.options = Object.assign(Object.assign({}, INITIAL), _);
    }
    doExport({ data: _, headers: H, props: V }) {
      let q = this.options.bom ? NO_BREAK_SPACE : "";
      return (H == null ? void 0 : H.length) > 0 && H.forEach((G) => {
        G.length && (q += this.prepareHeader(G, this.options.columnDelimiter), q += this.options.rowDelimiter);
      }), _.forEach((G, J) => {
        if (J > 0 && (q += this.options.rowDelimiter), isGrouping(G)) {
          q += this.parseCell(getGroupingName(G), this.options.columnDelimiter);
          return;
        }
        q += V.map((Z) => this.parseCell(G[Z], this.options.columnDelimiter)).join(this.options.columnDelimiter);
      }), q;
    }
    prepareHeader(_, H) {
      let V = "";
      const q = _.map((G) => this.parseCell(G, H, !0));
      return V += q.join(H), V;
    }
    parseCell(_, H, V = !1) {
      let q = _;
      typeof _ != "string" && (q = JSON.stringify(_));
      const G = [CARRIAGE_RETURN, DOUBLE_QT, LINE_FEED, H];
      return typeof q > "u" ? "" : q !== "" && (V || G.some((J) => q.indexOf(J) >= 0)) ? `"${q.replace(escapeRegex, '""')}"` : q;
    }
  }
  var ExportTypes;
  (function(O) {
    O.csv = "csv";
  })(ExportTypes || (ExportTypes = {}));
  class ExportFilePlugin extends BasePlugin {
    /** Exports string */
    async exportString(_ = {}, H = ExportTypes.csv) {
      const V = await this.beforeexport();
      return V ? this.formatter(H, _).doExport(V) : null;
    }
    /** Exports Blob */
    async exportBlob(_ = {}, H = ExportTypes.csv) {
      return await this.getBlob(this.formatter(H, _));
    }
    /** Export file */
    async exportFile(_ = {}, H = ExportTypes.csv) {
      const V = this.formatter(H, _), q = window.URL || window.webkitURL, G = document.createElement("a"), { filename: J, fileKind: Z } = V.options, te = `${J}.${Z}`, ne = await this.getBlob(V), ee = ne ? q.createObjectURL(ne) : "";
      G.style.display = "none", G.setAttribute("href", ee), G.setAttribute("download", te), this.revogrid.appendChild(G), G.dispatchEvent(new MouseEvent("click")), this.revogrid.removeChild(G), await timeout(120), q.revokeObjectURL(ee);
    }
    /** Blob object */
    async getBlob(_) {
      const H = `${_.options.mime};charset=${_.options.encoding}`;
      if (typeof Blob < "u") {
        const V = await this.beforeexport();
        return V ? new Blob([_.doExport(V)], { type: H }) : null;
      }
      return null;
    }
    // before event
    async beforeexport() {
      let _ = await this.getData();
      const H = this.emit("beforeexport", { data: _ });
      return H.defaultPrevented ? null : H.detail.data;
    }
    async getData() {
      const _ = await this.getSource(), H = [], V = [];
      columnTypes.forEach((G, J) => {
        V.push(this.getColPerSource(G).then((Z) => H[J] = Z));
      }), await Promise.all(V);
      const q = {
        headers: [],
        props: []
      };
      for (let G of H)
        G.headers.forEach((J, Z) => {
          q.headers[Z] || (q.headers[Z] = []), q.headers[Z].push(...J);
        }), q.props.push(...G.props);
      return Object.assign({ data: _ }, q);
    }
    async getColPerSource(_) {
      const H = await this.revogrid.getColumnStore(_), V = H.get("source"), q = H.get("items"), G = H.get("groupingDepth"), J = H.get("groups"), Z = [], te = [], ne = q.reduce((ae, oe, se) => {
        const ce = V[oe].prop;
        return Z.push(V[oe].name || ""), te.push(ce), ae[ce] = se, ae;
      }, {}), ee = this.getGroupHeaders(G, J, q, ne);
      return ee.push(Z), {
        headers: ee,
        props: te
      };
    }
    getGroupHeaders(_, H, V, q) {
      const G = [], J = fill(new Array(V.length), "");
      for (let Z = 0; Z < _; Z++) {
        const te = [...J];
        if (G.push(te), !H[Z])
          continue;
        H[Z].forEach((ee) => {
          const ae = this.findGroupStartIndex(ee.ids, q);
          typeof ae == "number" && (te[ae] = ee.name);
        });
      }
      return G;
    }
    findGroupStartIndex(_, H) {
      let V;
      return _.forEach((q) => {
        const G = H[q];
        typeof G == "number" && (typeof V != "number" || V > G) && (V = G);
      }), V;
    }
    async getSource() {
      const _ = [], H = [];
      return rowTypes.forEach((V) => {
        const q = [];
        _.push(q);
        const G = this.revogrid.getVisibleSource(V).then((J) => q.push(...J));
        H.push(G);
      }), await Promise.all(H), _.reduce((V, q) => (V.push(...q), V), []);
    }
    // get correct class for future multiple types support
    formatter(_, H = {}) {
      switch (_) {
        case ExportTypes.csv:
          return new ExportCsv(H);
        default:
          throw new Error("Unknown format");
      }
    }
  }
  const eq = (O, _) => {
    if (typeof O > "u" || O === null && !_)
      return !0;
    typeof O != "string" && (O = JSON.stringify(O));
    const H = _ == null ? void 0 : _.toString().toLocaleLowerCase();
    return (H == null ? void 0 : H.length) === 0 ? !0 : O.toLocaleLowerCase() === H;
  }, notEq = (O, _) => !eq(O, _);
  notEq.extra = "input";
  eq.extra = "input";
  const gtThan = function(O, _) {
    let H;
    return typeof O == "number" && typeof _ < "u" && _ !== null ? (H = parseFloat(_ == null ? void 0 : _.toString()), O > H) : !1;
  };
  gtThan.extra = "input";
  const gtThanEq = function(O, _) {
    return eq(O, _) || gtThan(O, _);
  };
  gtThanEq.extra = "input";
  const lt = function(O, _) {
    let H;
    return typeof O == "number" && typeof _ < "u" && _ !== null ? (H = parseFloat(_.toString()), O < H) : !1;
  };
  lt.extra = "input";
  const lsEq = function(O, _) {
    return eq(O, _) || lt(O, _);
  };
  lsEq.extra = "input";
  const set = (O) => !(O === "" || O === null || O === void 0), notSet = (O) => !set(O), beginsWith = (O, _) => O ? _ ? (typeof O != "string" && (O = JSON.stringify(O)), typeof _ != "string" && (_ = JSON.stringify(_)), O.toLocaleLowerCase().indexOf(_.toLocaleLowerCase()) === 0) : !0 : !1;
  beginsWith.extra = "input";
  const contains$1 = (O, _) => _ ? O ? _ ? (typeof O != "string" && (O = JSON.stringify(O)), O.toLocaleLowerCase().indexOf(_.toString().toLowerCase()) > -1) : !0 : !1 : !0, notContains = (O, _) => !contains$1(O, _);
  notContains.extra = "input";
  contains$1.extra = "input";
  const filterCoreFunctionsIndexedByType = {
    none: () => !0,
    empty: notSet,
    notEmpty: set,
    eq,
    notEq,
    begins: beginsWith,
    contains: contains$1,
    notContains,
    eqN: eq,
    neqN: notEq,
    gt: gtThan,
    gte: gtThanEq,
    lt,
    lte: lsEq
  }, filterTypes = {
    string: ["notEmpty", "empty", "eq", "notEq", "begins", "contains", "notContains"],
    number: ["notEmpty", "empty", "eqN", "neqN", "gt", "gte", "lt", "lte"]
  }, filterNames = {
    none: "None",
    empty: "Not set",
    notEmpty: "Set",
    eq: "Equal",
    notEq: "Not equal",
    begins: "Begins with",
    contains: "Contains",
    notContains: "Does not contain",
    eqN: "=",
    neqN: "!=",
    gt: ">",
    gte: ">=",
    lt: "<",
    lte: "<="
  }, FILTER_TRIMMED_TYPE = "filter", FILTER_CONFIG_CHANGED_EVENT = "filterconfigchanged", FILTE_PANEL = "revogr-filter-panel";
  class FilterPlugin extends BasePlugin {
    constructor(_, H, V) {
      var q;
      super(_, H), this.revogrid = _, this.config = V, this.filterCollection = {}, this.multiFilterItems = {}, this.filterByType = Object.assign({}, filterTypes), this.filterNameIndexByType = Object.assign({}, filterNames), this.filterFunctionsIndexedByType = Object.assign({}, filterCoreFunctionsIndexedByType), this.filterProp = FILTER_PROP, V && this.initConfig(V);
      const G = this.revogrid.registerVNode.filter((Z) => typeof Z == "object" && Z.$tag$ !== FILTE_PANEL);
      this.revogrid.registerVNode = [
        ...G,
        h(
          "revogr-filter-panel",
          { filterNames: this.filterNameIndexByType, filterEntities: this.filterFunctionsIndexedByType, filterCaptions: (q = V == null ? void 0 : V.localization) === null || q === void 0 ? void 0 : q.captions, onFilterChange: (Z) => this.onFilterChange(Z.detail), onResetChange: (Z) => this.onFilterReset(Z.detail), disableDynamicFiltering: V == null ? void 0 : V.disableDynamicFiltering, ref: (Z) => this.pop = Z },
          " ",
          this.extraContent()
        )
      ];
      const J = async () => {
        const Z = Object.keys(this.filterCollection);
        Z.length > 0 && Z.forEach((te, ne) => {
          this.multiFilterItems[te] || (this.multiFilterItems[te] = [
            {
              id: ne,
              type: this.filterCollection[te].type,
              value: this.filterCollection[te].value,
              relation: "and"
            }
          ]);
        }), Object.keys(this.multiFilterItems).length !== 0 && await this.runFiltering(this.multiFilterItems);
      };
      this.addEventListener("headerclick", (Z) => this.headerclick(Z)), this.addEventListener(FILTER_CONFIG_CHANGED_EVENT, ({ detail: Z }) => {
        if (!Z || typeof Z == "object" && (!Z.multiFilterItems || !Object.keys(Z.multiFilterItems).length)) {
          this.clearFiltering();
          return;
        }
        typeof Z == "object" && this.initConfig(Z), J();
      }), this.addEventListener("aftersourceset", J), this.addEventListener("filter", ({ detail: Z }) => this.onFilterChange(Z));
    }
    beforeshow(_) {
    }
    extraContent() {
      return null;
    }
    initConfig(_) {
      if (_.multiFilterItems ? this.multiFilterItems = Object.assign({}, _.multiFilterItems) : this.multiFilterItems = {}, _.customFilters)
        for (let V in _.customFilters) {
          const q = _.customFilters[V];
          this.filterByType[q.columnFilterType] || (this.filterByType[q.columnFilterType] = []), this.filterByType[q.columnFilterType].push(V), this.filterFunctionsIndexedByType[V] = q.func, this.filterNameIndexByType[V] = q.name;
        }
      _.filterProp && (this.filterProp = _.filterProp);
      const H = _.include;
      if (H) {
        const V = {};
        for (let q in this.filterByType) {
          const G = this.filterByType[q].filter((J) => H.indexOf(J) > -1);
          G.length && (V[q] = G);
        }
        Object.keys(V).length > 0 && (this.filterByType = V);
      }
      if (_.collection) {
        const V = Object.entries(_.collection).filter(([, q]) => this.filterFunctionsIndexedByType[q.type]);
        this.filterCollection = Object.fromEntries(V);
      } else
        this.filterCollection = {};
      _.localization && _.localization.filterNames && Object.entries(_.localization.filterNames).forEach(([V, q]) => {
        this.filterNameIndexByType[V] != null && (this.filterNameIndexByType[V] = q);
      });
    }
    async headerclick(_) {
      var H, V;
      const q = (H = _.detail.originalEvent) === null || H === void 0 ? void 0 : H.target;
      if (!isFilterBtn(q) || (_.preventDefault(), !this.pop))
        return;
      const G = this.revogrid.getBoundingClientRect(), J = q.getBoundingClientRect(), Z = _.detail.prop, te = Object.assign(Object.assign(Object.assign({}, _.detail), this.filterCollection[Z]), { x: J.x - G.x, y: J.y - G.y + J.height, autoCorrect: !0, filterTypes: this.getColumnFilter(_.detail.filter), filterItems: this.multiFilterItems, extraContent: this.extraHyperContent });
      (V = this.beforeshow) === null || V === void 0 || V.call(this, te), this.pop.show(te);
    }
    getColumnFilter(_) {
      let H = "string";
      if (!_)
        return { [H]: this.filterByType[H] };
      if (this.isValidType(_))
        H = _;
      else if (typeof _ == "object" && _.length)
        return _.reduce((V, q) => (this.isValidType(q) && (V[q] = this.filterByType[q]), V), {});
      return { [H]: this.filterByType[H] };
    }
    isValidType(_) {
      return !!(typeof _ == "string" && this.filterByType[_]);
    }
    /**
     * Called on internal component change
     */
    async onFilterChange(_) {
      this.multiFilterItems = _, this.runFiltering(this.multiFilterItems);
    }
    onFilterReset(_) {
      delete this.multiFilterItems[_ ?? ""], this.onFilterChange(this.multiFilterItems);
    }
    /**
     * Triggers grid filtering
     */
    async doFiltering(_, H, V, q) {
      const G = [], J = {};
      V.forEach((ae) => {
        const oe = Object.assign({}, ae), se = q[oe.prop];
        J[oe.prop] = oe, oe[this.filterProp] && !se && (delete oe[this.filterProp], G.push(oe)), !oe[this.filterProp] && se && (G.push(oe), oe[this.filterProp] = !0);
      });
      const Z = this.getRowFilter(H, q, J), { defaultPrevented: te, detail: ne } = this.emit("beforefiltertrimmed", {
        collection: _,
        itemsToFilter: Z,
        source: H,
        filterItems: q
      });
      te || (await this.revogrid.addTrimmed(ne.itemsToFilter, FILTER_TRIMMED_TYPE)).defaultPrevented || (this.providers.column.updateColumns(G), this.emit("afterfilterapply", {
        multiFilterItems: q,
        source: H,
        collection: _
      }));
    }
    async clearFiltering() {
      this.multiFilterItems = {}, await this.runFiltering(this.multiFilterItems);
    }
    async runFiltering(_) {
      const H = {}, V = Object.keys(_);
      for (const te of V)
        if (_[te].length > 0) {
          const ne = _[te][0];
          H[te] = {
            type: ne.type,
            value: ne.value
          };
        }
      this.filterCollection = H;
      const q = this.providers.column.getColumns(), G = this.providers.data.stores.rgRow.store.get("source"), { defaultPrevented: J, detail: Z } = this.emit("beforefilterapply", {
        collection: this.filterCollection,
        source: G,
        columns: q,
        filterItems: this.multiFilterItems
      });
      J || this.doFiltering(Z.collection, Z.source, Z.columns, Z.filterItems);
    }
    /**
     * Get trimmed rows based on filter
     */
    getRowFilter(_, H, V) {
      const q = Object.keys(H), G = {};
      for (let J = 0; J < _.length; J++)
        for (const Z of q)
          this.shouldTrimRow(H[Z], Z, V[Z], _[J]) && (G[J] = !0);
      return G;
    }
    shouldTrimRow(_, H, V, q = {}) {
      let G = 0, J = [];
      for (const [Z, te] of _.entries()) {
        const ne = this.filterFunctionsIndexedByType[te.type], ee = V ? getCellDataParsed(q, V) : q[H];
        if (te.relation === "or") {
          if (J = [], ne(ee, te.value))
            continue;
          G++;
        } else if (J.push(!ne(ee, te.value)), isFinalAndFilter(Z, _)) {
          if (allAndConditionsSatisfied(J)) {
            J = [];
            continue;
          }
          G += J.length, J = [];
        }
      }
      return G === _.length;
    }
  }
  function isFinalAndFilter(O, _) {
    const H = _[O + 1];
    return !H || !!H.relation && H.relation !== "and";
  }
  function allAndConditionsSatisfied(O) {
    return !O.includes(!0);
  }
  function doCollapse(O, _) {
    const H = _[O], V = H[PSEUDO_GROUP_ITEM_VALUE], q = {};
    let G = O + 1;
    const J = _.length;
    for (; G < J; ) {
      const Z = _[G];
      if (isGrouping(Z)) {
        const te = Z[PSEUDO_GROUP_ITEM_VALUE];
        if (!te.length || !te.startsWith(V + ","))
          break;
        Z[GROUP_EXPANDED] = !1;
      }
      q[G++] = !0;
    }
    return H[GROUP_EXPANDED] = !1, { trimmed: q };
  }
  function doExpand(O, _, H) {
    const V = H[O], q = _[V], G = getParsedGroup(q[PSEUDO_GROUP_ITEM_ID]), J = {};
    if (!G)
      return { trimmed: J };
    const Z = [];
    q[GROUP_EXPANDED] = !0;
    let te = V + 1;
    const ne = _.length;
    let ee = 0;
    for (; te < ne; ) {
      const oe = _[te], se = isGrouping(oe);
      if (se)
        if (isSameGroup(G, q, oe))
          ee || (ee = oe[GROUP_DEPTH]);
        else break;
      (!ee || se && ee === oe[GROUP_DEPTH]) && (J[te] = !1, Z.push(te)), te++;
    }
    const ae = {
      trimmed: J
    };
    if (Z.length) {
      const oe = [...H];
      oe.splice(O + 1, 0, ...Z), ae.items = oe;
    }
    return ae;
  }
  const TRIMMED_GROUPING = "grouping";
  function processDoubleConversionTrimmed(O, _, H) {
    const V = {};
    for (let q in O) {
      if (q === TRIMMED_GROUPING)
        continue;
      const G = O[q], J = {};
      for (let Z in G) {
        let te = _[Z];
        H && (te = H[te]), G[Z] && (J[te] = !0, te !== parseInt(Z, 10) && (V[q] = J));
      }
    }
    return V;
  }
  var stringTag = "[object String]";
  function isString(O) {
    return typeof O == "string" || !isArray$1(O) && isObjectLike(O) && baseGetTag(O) == stringTag;
  }
  var asciiSize = baseProperty("length");
  const asciiSize$1 = asciiSize;
  var rsAstralRange$1 = "\\ud800-\\udfff", rsComboMarksRange$1 = "\\u0300-\\u036f", reComboHalfMarksRange$1 = "\\ufe20-\\ufe2f", rsComboSymbolsRange$1 = "\\u20d0-\\u20ff", rsComboRange$1 = rsComboMarksRange$1 + reComboHalfMarksRange$1 + rsComboSymbolsRange$1, rsVarRange$1 = "\\ufe0e\\ufe0f", rsZWJ$1 = "\\u200d", reHasUnicode = RegExp("[" + rsZWJ$1 + rsAstralRange$1 + rsComboRange$1 + rsVarRange$1 + "]");
  function hasUnicode(O) {
    return reHasUnicode.test(O);
  }
  var rsAstralRange = "\\ud800-\\udfff", rsComboMarksRange = "\\u0300-\\u036f", reComboHalfMarksRange = "\\ufe20-\\ufe2f", rsComboSymbolsRange = "\\u20d0-\\u20ff", rsComboRange = rsComboMarksRange + reComboHalfMarksRange + rsComboSymbolsRange, rsVarRange = "\\ufe0e\\ufe0f", rsAstral = "[" + rsAstralRange + "]", rsCombo = "[" + rsComboRange + "]", rsFitz = "\\ud83c[\\udffb-\\udfff]", rsModifier = "(?:" + rsCombo + "|" + rsFitz + ")", rsNonAstral = "[^" + rsAstralRange + "]", rsRegional = "(?:\\ud83c[\\udde6-\\uddff]){2}", rsSurrPair = "[\\ud800-\\udbff][\\udc00-\\udfff]", rsZWJ = "\\u200d", reOptMod = rsModifier + "?", rsOptVar = "[" + rsVarRange + "]?", rsOptJoin = "(?:" + rsZWJ + "(?:" + [rsNonAstral, rsRegional, rsSurrPair].join("|") + ")" + rsOptVar + reOptMod + ")*", rsSeq = rsOptVar + reOptMod + rsOptJoin, rsSymbol = "(?:" + [rsNonAstral + rsCombo + "?", rsCombo, rsRegional, rsSurrPair, rsAstral].join("|") + ")", reUnicode = RegExp(rsFitz + "(?=" + rsFitz + ")|" + rsSymbol + rsSeq, "g");
  function unicodeSize(O) {
    for (var _ = reUnicode.lastIndex = 0; reUnicode.test(O); )
      ++_;
    return _;
  }
  function stringSize(O) {
    return hasUnicode(O) ? unicodeSize(O) : asciiSize$1(O);
  }
  var mapTag = "[object Map]", setTag = "[object Set]";
  function size$1(O) {
    if (O == null)
      return 0;
    if (isArrayLike(O))
      return isString(O) ? stringSize(O) : O.length;
    var _ = getTag$1(O);
    return _ == mapTag || _ == setTag ? O.size : baseKeys(O).length;
  }
  function sortIndexByItems(O, _, H = {}) {
    return Object.entries(H).length === 0 ? [...Array(O.length).keys()] : O.sort((V, q) => {
      const G = _[V], J = _[q];
      for (const [Z, te] of Object.entries(H)) {
        if (isGrouping(G) && G["__rvgr-prop"] !== Z || isGrouping(J) && J["__rvgr-prop"] !== Z)
          return 0;
        const ne = te == null ? void 0 : te(Z, G, J);
        if (ne)
          return ne;
      }
      return 0;
    });
  }
  class SortingPlugin extends BasePlugin {
    runSorting(_, H, V) {
      var q;
      this.sort(_, H, void 0, V), (q = this.sortingPromise) === null || q === void 0 || q.call(this), this.sortingPromise = null;
    }
    constructor(_, H, V) {
      super(_, H), this.revogrid = _, this.sortingPromise = null, this.postponeSort = debounce$1((G, J, Z) => this.runSorting(G, J, Z), 50);
      const q = (G) => {
        var J;
        if (G) {
          const Z = {}, te = {};
          (J = G.columns) === null || J === void 0 || J.forEach((ne) => {
            Z[ne.prop] = this.getComparer(ne, ne.order), te[ne.prop] = ne.order;
          }), this.sorting = te, this.sortingFunc = Z;
        }
      };
      q(V), this.addEventListener("sortingconfigchanged", ({ detail: G }) => {
        V = G, q(G), this.startSorting(this.sorting, this.sortingFunc);
      }), this.addEventListener("beforeanysource", ({ detail: { type: G } }) => {
        if (this.sorting && this.sortingFunc) {
          if (this.emit("beforesourcesortingapply", { type: G, sorting: this.sorting }).defaultPrevented)
            return;
          this.startSorting(this.sorting, this.sortingFunc);
        }
      }), this.addEventListener("aftercolumnsset", ({ detail: { order: G } }) => {
        if (V)
          return;
        const J = this.providers.column.getColumns(), Z = {};
        for (let te in G) {
          const ne = this.getComparer(getColumnByProp(J, te), G[te]);
          Z[te] = ne;
        }
        this.sorting = G, this.sortingFunc = G && Z;
      }), this.addEventListener("beforeheaderclick", (G) => {
        var J, Z, te, ne;
        G.defaultPrevented || !((Z = (J = G.detail) === null || J === void 0 ? void 0 : J.column) === null || Z === void 0) && Z.sortable && this.headerclick(G.detail.column, G.detail.index, (ne = (te = G.detail) === null || te === void 0 ? void 0 : te.originalEvent) === null || ne === void 0 ? void 0 : ne.shiftKey);
      });
    }
    startSorting(_, H, V) {
      this.sortingPromise || this.revogrid.jobsBeforeRender.push(new Promise((q) => {
        this.sortingPromise = q;
      })), this.postponeSort(_, H, V);
    }
    getComparer(_, H) {
      var V, q;
      const G = ((V = _ == null ? void 0 : _.cellCompare) === null || V === void 0 ? void 0 : V.bind({ order: H })) || ((q = this.defaultCellCompare) === null || q === void 0 ? void 0 : q.bind({ column: _, order: H }));
      if (H == "asc")
        return G;
      if (H == "desc")
        return this.descCellCompare(G);
    }
    /**
     * Apply sorting to data on header click
     * If additive - add to existing sorting, multiple columns can be sorted
     */
    headerclick(_, H, V) {
      var q, G;
      let J = this.getNextOrder(_.order);
      const Z = this.emit("beforesorting", { column: _, order: J, additive: V });
      if (Z.defaultPrevented)
        return;
      J = Z.detail.order;
      const te = this.providers.column.updateColumnSorting(Z.detail.column, H, J, V), ne = this.emit("beforesortingapply", {
        column: te,
        order: J,
        additive: V
      });
      if (ne.defaultPrevented)
        return;
      const ee = this.getComparer(ne.detail.column, ne.detail.order);
      if (ne.detail.additive && this.sorting) {
        const ae = {}, oe = {};
        this.sorting = Object.assign(Object.assign({}, this.sorting), ae), this.sortingFunc = Object.assign(Object.assign({}, this.sortingFunc), oe), _.prop in ae && size$1(ae) > 1 && J === void 0 ? (delete ae[_.prop], delete oe[_.prop]) : (ae[_.prop] = J, oe[_.prop] = ee);
      } else
        J ? (this.sorting = { [_.prop]: J }, this.sortingFunc = { [_.prop]: ee }) : ((q = this.sorting) === null || q === void 0 || delete q[_.prop], (G = this.sortingFunc) === null || G === void 0 || delete G[_.prop]);
      this.startSorting(this.sorting, this.sortingFunc);
    }
    /**
     * Sort items by sorting function
     * @requires proxyItems applied to row store
     * @requires source applied to row store
     *
     * @param sorting - per column sorting
     * @param data - this.stores['rgRow'].store.get('source')
     */
    sort(_, H, V = rowTypes, q = !1) {
      if (Object.keys(_ || {}).length)
        for (let G of V) {
          const J = this.providers.data.stores[G], Z = J.store.get("source"), te = J.store.get("proxyItems"), ne = sortIndexByItems([...te], Z, H), ee = J.store.get("items");
          J.setData({
            proxyItems: ne,
            source: [...Z]
          });
          const ae = J.store.get("items");
          q || this.providers.dimension.updateSizesPositionByNewDataIndexes(G, ae, ee);
        }
      else
        for (let G of V) {
          const J = this.providers.data.stores[G], Z = J.store.get("source"), te = J.store.get("proxyItems"), ne = Array.from({ length: Z.length }, (ee, ae) => ae);
          this.providers.dimension.updateSizesPositionByNewDataIndexes(G, ne, te), J.setData({ proxyItems: ne, source: [...Z] });
        }
      this.emit("aftersortingapply");
    }
    defaultCellCompare(_, H, V) {
      const q = this.column ? getCellRaw(H, this.column) : H == null ? void 0 : H[_], G = this.column ? getCellRaw(V, this.column) : V == null ? void 0 : V[_], J = q == null ? void 0 : q.toString().toLowerCase(), Z = G == null ? void 0 : G.toString().toLowerCase();
      return J == Z ? 0 : J > Z ? 1 : -1;
    }
    descCellCompare(_) {
      return (H, V, q) => -1 * _(H, V, q);
    }
    getNextOrder(_) {
      switch (_) {
        case void 0:
          return "asc";
        case "asc":
          return "desc";
        case "desc":
          return;
      }
    }
  }
  class GroupingRowPlugin extends BasePlugin {
    getStore(_ = GROUPING_ROW_TYPE) {
      return this.providers.data.stores[_].store;
    }
    constructor(_, H) {
      super(_, H), this.revogrid = _, this.providers = H;
    }
    // befoce cell focus
    onFocus(_) {
      isGrouping(_.detail.model) && _.preventDefault();
    }
    // expand event triggered
    onExpand({ virtualIndex: _ }) {
      const { source: H } = getSource(this.getStore().get("source"), this.getStore().get("proxyItems"));
      let V = this.getStore().get("trimmed")[TRIMMED_GROUPING], q = getPhysical(this.getStore(), _);
      if (getExpanded(H[q])) {
        const { trimmed: J } = doCollapse(q, H);
        V = Object.assign(Object.assign({}, V), J), this.revogrid.clearFocus();
      } else {
        const { trimmed: J, items: Z } = doExpand(_, H, this.getStore().get("items"));
        V = Object.assign(Object.assign({}, V), J), Z && setItems(this.getStore(), Z);
      }
      this.getStore().set("source", H), this.revogrid.addTrimmed(V, TRIMMED_GROUPING);
    }
    setColumnGrouping(_) {
      return _ != null && _.length ? (_[0][PSEUDO_GROUP_COLUMN] = !0, !0) : !1;
    }
    setColumns({ columns: _ }) {
      for (let H of columnTypes)
        if (this.setColumnGrouping(_[H]))
          break;
    }
    // evaluate drag between groups
    onDrag(_) {
      const { from: H, to: V } = _.detail, q = V - H >= 0, { source: G } = getSource(this.getStore().get("source"), this.getStore().get("proxyItems")), J = this.getStore().get("items");
      let Z = q ? H : V;
      const te = q ? V : H;
      for (; Z < te; Z++) {
        const ne = G[J[Z]];
        if (isGrouping(ne)) {
          _.preventDefault();
          return;
        }
      }
    }
    beforeTrimmedApply(_, H) {
      if (H === FILTER_TRIMMED_TYPE) {
        const V = this.getStore().get("source");
        for (let q in _)
          _[q] && isGrouping(V[q]) && (_[q] = !1);
      }
    }
    isSortingRunning() {
      const _ = this.providers.plugins.getByClass(SortingPlugin);
      return !!(_ != null && _.sortingPromise);
    }
    // subscribe to grid events to process them accordingly
    subscribe() {
      this.addEventListener("beforesourceset", ({ detail: _ }) => {
        var H, V, q;
        !((V = (H = this.options) === null || H === void 0 ? void 0 : H.props) === null || V === void 0) && V.length && (!((q = _ == null ? void 0 : _.source) === null || q === void 0) && q.length) && (this.isSortingRunning() || this.onDataSet(_));
      }), this.addEventListener("beforecolumnsset", ({ detail: _ }) => {
        this.setColumns(_);
      }), this.addEventListener("beforetrimmed", ({ detail: { trimmed: _, trimmedType: H } }) => this.beforeTrimmedApply(_, H)), this.addEventListener("aftersortingapply", () => {
        var _, H;
        !((H = (_ = this.options) === null || _ === void 0 ? void 0 : _.props) === null || H === void 0) && H.length && this.doSourceUpdate(Object.assign({}, this.options));
      }), this.addEventListener("beforecellfocus", (_) => this.onFocus(_)), this.addEventListener("roworderchanged", (_) => this.onDrag(_)), this.addEventListener("groupexpandclick", (_) => this.onExpand(_.detail));
    }
    /**
     * Starts global source update with group clearing and applying new one
     * Initiated when need to reapply grouping
     */
    doSourceUpdate(_) {
      var H;
      const { source: V, prevExpanded: q, oldNewIndexes: G } = getSource(this.getStore().get("source"), this.getStore().get("proxyItems"), !0), J = Object.assign({ prevExpanded: q }, _), { sourceWithGroups: Z, depth: te, trimmed: ne, oldNewIndexMap: ee, childrenByGroup: ae } = gatherGrouping(V, ((H = this.options) === null || H === void 0 ? void 0 : H.props) || [], J), oe = _ == null ? void 0 : _.groupLabelTemplate;
      this.providers.data.setData(Z, GROUPING_ROW_TYPE, this.revogrid.disableVirtualY, { depth: te, customRenderer: oe }, !0), this.updateTrimmed(ne, ae, G ?? {}, ee);
    }
    /**
     * Apply grouping on data set
     * Clear grouping from source
     * If source came from other plugin
     */
    onDataSet(_) {
      var H, V;
      let q = {};
      if (((H = this.options) === null || H === void 0 ? void 0 : H.preserveGroupingOnUpdate) !== !1) {
        let { prevExpanded: oe } = getSource(this.getStore().get("source"), this.getStore().get("proxyItems"), !0);
        q = oe;
      }
      const G = _.source.filter((oe) => !isGrouping(oe)), J = Object.assign(Object.assign({}, this.revogrid.grouping || {}), { prevExpanded: q }), { sourceWithGroups: Z, depth: te, trimmed: ne, oldNewIndexMap: ee, childrenByGroup: ae } = gatherGrouping(G, ((V = this.options) === null || V === void 0 ? void 0 : V.props) || [], J);
      _.source = Z, this.providers.data.setGrouping({ depth: te }), this.updateTrimmed(ne, ae, ee);
    }
    /**
     * Externam call to apply grouping. Called by revogrid when prop changed.
     */
    setGrouping(_) {
      var H, V;
      if (this.clearSubscriptions(), this.options = _, !(!((V = (H = this.options) === null || H === void 0 ? void 0 : H.props) === null || V === void 0) && V.length)) {
        this.clearGrouping();
        return;
      }
      const { source: q } = getSource(this.getStore().get("source"), this.getStore().get("proxyItems"));
      q.length && this.doSourceUpdate(Object.assign({}, _));
      for (let G of columnTypes)
        if (this.setColumnGrouping(this.providers.column.getColumns(G))) {
          this.providers.column.refreshByType(G);
          break;
        }
      this.subscribe();
    }
    // clear grouping
    clearGrouping() {
      columnTypes.forEach((V) => {
        const q = this.providers.column.getColumns(V);
        let G = !1;
        q.forEach((J) => {
          isGroupingColumn(J) && (delete J[PSEUDO_GROUP_COLUMN], G = !0);
        }), G && this.providers.column.refreshByType(V);
      });
      const { source: _, oldNewIndexes: H } = getSource(this.getStore().get("source"), this.getStore().get("proxyItems"), !0);
      this.providers.data.setData(_, GROUPING_ROW_TYPE, this.revogrid.disableVirtualY, void 0, !0), this.updateTrimmed(void 0, void 0, H);
    }
    updateTrimmed(_ = {}, H = {}, V = {}, q) {
      const G = processDoubleConversionTrimmed(this.getStore().get("trimmed"), V, q);
      for (let J in G)
        this.revogrid.addTrimmed(G[J], J);
      this.revogrid.addTrimmed(Object.assign({}, _), TRIMMED_GROUPING);
    }
  }
  const COLUMN_DRAG_CLASS = "column-drag-start";
  class ColumnOrderHandler {
    constructor() {
      this.offset = 0;
    }
    renderAutoscroll(_, H) {
      H && (this.autoscrollEl = document.createElement("div"), this.autoscrollEl.classList.add("drag-auto-scroll-y"), H.appendChild(this.autoscrollEl));
    }
    autoscroll(_, H, V = "translateX") {
      if (!this.autoscrollEl)
        return;
      const G = Math.min(_ + 10, H - 3);
      this.autoscrollEl.style.transform = `${V}(${G}px)`, this.autoscrollEl.scrollIntoView({
        block: "nearest",
        inline: "nearest"
      });
    }
    start(_, { dataEl: H, gridRect: V, scrollEl: q, gridEl: G }, J = "left") {
      G.classList.add(COLUMN_DRAG_CLASS);
      const Z = q.getBoundingClientRect();
      Z && (this.offset = Z[J] - V[J]), this.renderAutoscroll(_, H);
    }
    stop(_) {
      var H;
      _.classList.remove(COLUMN_DRAG_CLASS), this.element && (this.element.hidden = !0), this.offset = 0, (H = this.autoscrollEl) === null || H === void 0 || H.remove(), this.autoscrollEl = void 0;
    }
    showHandler(_, H, V = "translateX") {
      this.element && (this.offset && (_ = Math.max(_, this.offset)), _ = Math.min(_, H), this.element.style.transform = `${V}(${_}px)`, this.element.hidden = !1);
    }
    render() {
      const _ = this.element = document.createElement("div");
      return _.classList.add("drag-position-y"), _.hidden = !0, _;
    }
  }
  const COLUMN_CLICK = ON_COLUMN_CLICK, MOVE = "columndragmousemove", DRAG_END = "columndragend", BEFORE_DRAG_END = "beforecolumndragend", DRAG_START = "columndragstart";
  class ColumnMovePlugin extends BasePlugin {
    constructor(_, H) {
      super(_, H), this.revogrid = _, this.providers = H, this.moveFunc = debounce$1((V) => this.doMove(V), 5), this.staticDragData = null, this.dragData = null, this.localSubscriptions = {}, this.orderUi = new ColumnOrderHandler(), _.appendChild(this.orderUi.render()), _.classList.add("column-draggable"), this.localSubscriptions.mouseleave = {
        target: document,
        callback: (V) => this.onMouseOut(V)
      }, this.localSubscriptions.mouseup = {
        target: document,
        callback: (V) => this.onMouseUp(V)
      }, this.localSubscriptions.mousemove = {
        target: document,
        callback: (V) => this.move(V)
      }, this.addEventListener(COLUMN_CLICK, ({ detail: V }) => this.dragStart(V));
    }
    dragStart({ event: _, data: H }) {
      if (_.defaultPrevented)
        return;
      const { defaultPrevented: V } = dispatch(this.revogrid, DRAG_START, H);
      if (V)
        return;
      this.clearOrder();
      const { mouseleave: q, mouseup: G, mousemove: J } = this.localSubscriptions;
      q.target.addEventListener("mouseleave", q.callback), G.target.addEventListener("mouseup", G.callback);
      const Z = _.target.closest("revogr-header"), te = _.target.closest("revogr-viewport-scroll");
      if (!Z || !te || isColGrouping(H) || H.providers.type === "rowHeaders")
        return;
      const ne = this.getDimension(H.pin || "rgCol"), ee = this.revogrid.getBoundingClientRect(), ae = Z.getBoundingClientRect(), oe = getItemByPosition(ne, getLeftRelative(_.x, ee.left, ae.left - ee.left));
      this.staticDragData = {
        startPos: _.x,
        startItem: oe,
        data: H,
        dataEl: Z,
        scrollEl: te,
        gridEl: this.revogrid,
        cols: ne
      }, this.dragData = this.getData(this.staticDragData), J.target.addEventListener("mousemove", J.callback), this.orderUi.start(_, Object.assign(Object.assign({}, this.dragData), this.staticDragData));
    }
    doMove(_) {
      if (!this.staticDragData)
        return;
      const H = this.dragData = this.getData(this.staticDragData);
      if (!H)
        return;
      const V = this.staticDragData.startPos;
      if (Math.abs(V - _.x) > 10) {
        const q = getLeftRelative(_.x, this.dragData.gridRect.left, this.dragData.scrollOffset), G = getItemByPosition(this.staticDragData.cols, q);
        if (this.orderUi.autoscroll(q, H.elRect.width), G.itemIndex >= this.staticDragData.cols.count)
          return;
        this.orderUi.showHandler(G.end + H.scrollOffset, H.gridRect.width);
      }
    }
    move(_) {
      dispatch(this.revogrid, MOVE, _), this.moveFunc(_);
    }
    onMouseOut(_) {
      this.clearOrder();
    }
    onMouseUp(_) {
      if (this.dragData && this.staticDragData) {
        let H = getLeftRelative(_.x, this.dragData.gridRect.left, this.dragData.scrollOffset);
        H < 0 && (H = 0);
        const V = getItemByPosition(this.staticDragData.cols, H), q = this.providers.column.stores[this.dragData.type].store, G = [...q.get("items")], { defaultPrevented: J } = dispatch(this.revogrid, BEFORE_DRAG_END, Object.assign(Object.assign({}, this.staticDragData), { startPosition: this.staticDragData.startItem, newPosition: V, newItem: q.get("source")[G[this.staticDragData.startItem.itemIndex]] }));
        if (!J) {
          const Z = [...G], te = G.splice(this.staticDragData.startItem.itemIndex, 1);
          G.splice(V.itemIndex, 0, ...te), q.set("items", G), this.providers.dimension.updateSizesPositionByNewDataIndexes(this.dragData.type, G, Z);
        }
        dispatch(this.revogrid, DRAG_END, this.dragData);
      }
      this.clearOrder();
    }
    clearLocalSubscriptions() {
      forEach(this.localSubscriptions, ({ target: _, callback: H }, V) => _.removeEventListener(V, H));
    }
    clearOrder() {
      this.staticDragData = null, this.dragData = null, this.clearLocalSubscriptions(), this.orderUi.stop(this.revogrid);
    }
    /**
     * Clearing subscription
     */
    clearSubscriptions() {
      super.clearSubscriptions(), this.clearLocalSubscriptions();
    }
    getData({ gridEl: _, dataEl: H, data: V }) {
      const q = _.getBoundingClientRect(), G = H.getBoundingClientRect(), J = G.left - q.left;
      return {
        elRect: G,
        gridRect: q,
        type: V.pin || "rgCol",
        scrollOffset: J
      };
    }
    getDimension(_) {
      return this.providers.dimension.stores[_].getCurrentState();
    }
  }
  function getLeftRelative(O, _, H) {
    return O - _ - H;
  }
  /*!
   * Built by Revolist OU ❤️
   */
  const SortingSign = ({ column: O }) => {
    var _;
    return h("i", { class: (_ = O == null ? void 0 : O.order) !== null && _ !== void 0 ? _ : "sort-off" });
  };
  /*!
   * Built by Revolist OU ❤️
   */
  function isClear(O) {
    return codesLetter.BACKSPACE === O || codesLetter.DELETE === O;
  }
  function isTab(O) {
    return codesLetter.TAB === O;
  }
  function isEnterKeyValue(O) {
    return keyValues.ENTER === O;
  }
  function isCut(O) {
    return O.ctrlKey && O.code === "KeyX" || // Ctrl + X on Windows
    O.metaKey && O.code === "KeyX";
  }
  function isCopy(O) {
    return O.ctrlKey && O.code === "KeyC" || // Ctrl + C on Windows
    O.metaKey && O.code === "KeyC";
  }
  function isPaste(O) {
    return O.ctrlKey && O.code === "KeyV" || // Ctrl + V on Windows
    O.metaKey && O.code === "KeyV";
  }
  function isAll(O) {
    return O.ctrlKey && O.code === "KeyA" || // Ctrl + A on Windows
    O.metaKey && O.code === "KeyA";
  }
  /*!
   * Built by Revolist OU ❤️
   */
  class TextEditor {
    constructor(_, H) {
      this.data = _, this.saveCallback = H, this.editInput = null, this.element = null, this.editCell = void 0;
    }
    /**
     * Callback triggered on cell editor render
     */
    async componentDidRender() {
      var _;
      this.editInput && (await timeout(), (_ = this.editInput) === null || _ === void 0 || _.focus());
    }
    onKeyDown(_) {
      const H = isEnterKeyValue(_.key), V = isTab(_.key);
      (V || H) && _.target && this.saveCallback && !_.isComposing && (this.beforeDisconnect(), this.saveCallback(this.getValue(), V));
    }
    /**
     * IMPORTANT: Prevent scroll glitches when editor is closed and focus is on current input element.
     */
    beforeDisconnect() {
      var _;
      (_ = this.editInput) === null || _ === void 0 || _.blur();
    }
    /**
     * Get value from input
     */
    getValue() {
      var _;
      return (_ = this.editInput) === null || _ === void 0 ? void 0 : _.value;
    }
    /**
     * Render method for Editor plugin.
     * Renders input element with passed data from cell.
     *
     * @required @method
     * @param {Function} h - h function from stencil render.
     * @param {Object} _additionalData - additional data from plugin.
     * @returns {VNode} - input element.
     */
    render(_, H) {
      var V, q;
      return _("input", {
        type: "text",
        enterKeyHint: "enter",
        // set input value from cell data
        value: (q = (V = this.editCell) === null || V === void 0 ? void 0 : V.val) !== null && q !== void 0 ? q : "",
        // save input element as ref for further usage
        ref: (G) => {
          this.editInput = G;
        },
        // listen to keydown event on input element
        onKeyDown: (G) => this.onKeyDown(G)
      });
    }
  }
  const p = "routerLink", k = "navManager", N = "router", U = "aria", m = Symbol(), E = { default: m }, P = (O) => (O == null ? void 0 : O.split(" ")) || [], F = (O, _, H = []) => {
    var V;
    return [...Array.from(((V = O.value) == null ? void 0 : V.classList) || []), ...H].filter(
      (q, G, J) => !_.has(q) && J.indexOf(q) === G
    );
  }, $$1 = (O, _, H = [], V, q) => {
    const G = /* @__PURE__ */ defineComponent((J, { attrs: Z, slots: te, emit: ne }) => {
      var ee;
      J[V];
      const ae = ref$1(), oe = new Set(P(Z.class)), se = getCurrentInstance(), ce = (ee = se == null ? void 0 : se.appContext) != null && ee.provides[k] ? inject(k) : void 0, le = (ue) => {
        const { routerLink: de } = J;
        if (de !== m)
          if (ce !== void 0) {
            ue.preventDefault();
            let fe = { event: ue };
            for (const he in J) {
              const me = J[he];
              J.hasOwnProperty(he) && he.startsWith(N) && me !== m && (fe[he] = me);
            }
            ce.navigate(fe);
          } else
            console.warn("Tried to navigate, but no router was found. Make sure you have mounted Vue Router.");
      };
      return () => {
        J[V], P(Z.class).forEach((he) => {
          oe.add(he);
        });
        const ue = J.onClick, de = (he) => {
          ue !== void 0 && ue(he), he.defaultPrevented || le(he);
        };
        let fe = {
          ref: ae,
          class: F(ae, oe),
          onClick: de
        };
        for (const he in J) {
          const me = J[he];
          (J.hasOwnProperty(he) && me !== m || he.startsWith(U)) && (fe[he] = me);
        }
        return J[p] !== m && (fe = {
          ...fe,
          href: J[p]
        }), h$3(O, fe, te.default && te.default());
      };
    });
    return typeof G != "function" && (G.name = O, G.props = {
      [p]: E
    }, H.forEach((J) => {
      G.props[J] = E;
    })), G;
  }, W = /* @__PURE__ */ $$1("revo-grid", void 0, [
    "rowHeaders",
    "frameSize",
    "rowSize",
    "colSize",
    "range",
    "readonly",
    "resize",
    "canFocus",
    "useClipboard",
    "columns",
    "source",
    "pinnedTopSource",
    "pinnedBottomSource",
    "rowDefinitions",
    "editors",
    "applyOnClose",
    "plugins",
    "columnTypes",
    "theme",
    "rowClass",
    "autoSizeColumn",
    "filter",
    "sorting",
    "focusTemplate",
    "canMoveColumns",
    "trimmedRows",
    "exporting",
    "grouping",
    "stretch",
    "additionalData",
    "disableVirtualX",
    "disableVirtualY",
    "hideAttribution",
    "jobsBeforeRender",
    "registerVNode",
    "accessible",
    "contentsizechanged",
    "beforeedit",
    "beforerangeedit",
    "afteredit",
    "beforeautofill",
    "beforerange",
    "afterfocus",
    "roworderchanged",
    "beforesorting",
    "beforesourcesortingapply",
    "beforesortingapply",
    "rowdragstart",
    "headerclick",
    "beforecellfocus",
    "beforefocuslost",
    "beforesourceset",
    "beforeanysource",
    "aftersourceset",
    "afteranysource",
    "beforecolumnsset",
    "beforecolumnapplied",
    "aftercolumnsset",
    "beforefilterapply",
    "beforefiltertrimmed",
    "beforetrimmed",
    "aftertrimmed",
    "viewportscroll",
    "beforeexport",
    "beforeeditstart",
    "aftercolumnresize",
    "beforerowdefinition",
    "filterconfigchanged",
    "sortingconfigchanged",
    "rowheaderschanged",
    "beforegridrender",
    "aftergridrender",
    "aftergridinit",
    "additionaldatachanged",
    "afterthemechanged",
    "created"
  ]);
  function Y(O) {
    return !!O && (typeof O == "object" || typeof O == "function") && typeof O.then == "function";
  }
  var x;
  const A = (x = defineCustomElements) == null ? void 0 : x(), Q = /* @__PURE__ */ defineAsyncComponent(async () => (Y(A) && await A, W));
  var alasql_min$1 = { exports: {} }, alasql_min = alasql_min$1.exports, hasRequiredAlasql_min;
  function requireAlasql_min() {
    return hasRequiredAlasql_min || (hasRequiredAlasql_min = 1, function(O, _) {
      /*
      		@module alasql
      		@version 4.6.1
      
      		AlaSQL - JavaScript SQL database
      		© 2014-2024	Andrey Gershun & Mathias Wulff
      
      		@license
      		The MIT License (MIT)
      
      		Copyright 2014-2024 Andrey Gershun (agershun@gmail.com) & Mathias Wulff (m@rawu.dk)
      
      		Permission is hereby granted, free of charge, to any person obtaining a copy
      		of this software and associated documentation files (the "Software"), to deal
      		in the Software without restriction, including without limitation the rights
      		to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
      		copies of the Software, and to permit persons to whom the Software is
      		furnished to do so, subject to the following conditions:
      
      		The above copyright notice and this permission notice shall be included in all
      		copies or substantial portions of the Software.
      
      		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
      		IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
      		FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
      		AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
      		LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
      		OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
      		SOFTWARE.
      		*/
      (function(H, V) {
        O.exports = V();
      })(alasql_min, function() {
        var wn, e7, Pn;
        let H = function(Ie, rt, ft, pt) {
          if (rt = rt || [], typeof importScripts != "function" && H.webworker) {
            var wt = H.lastid++;
            H.buffer[wt] = ft, H.webworker.postMessage({ id: wt, sql: Ie, params: rt });
            return;
          }
          return arguments.length === 0 ? new ze.Select({ columns: [new ze.Column({ columnid: "*" })], from: [new ze.ParamValue({ param: 0 })] }) : arguments.length === 1 && Ie.constructor === Array ? H.promise(Ie) : (typeof rt == "function" && (pt = ft, ft = rt, rt = []), typeof rt != "object" && (rt = [rt]), typeof Ie == "string" && Ie[0] === "#" && typeof document == "object" ? Ie = document.querySelector(Ie).textContent : typeof Ie == "object" && Ie instanceof HTMLElement ? Ie = Ie.textContent : typeof Ie == "function" && (Ie = Ie.toString(), Ie = (/\/\*([\S\s]+)\*\//m.exec(Ie) || ["", "Function given as SQL. Plese Provide SQL string or have a /* ... */ syle comment with SQL in the function."])[1]), H.exec(Ie, rt, ft, pt));
        };
        H.version = "4.6.1", H.build = "develop-a4c1a42d", H.debug = void 0;
        var V = function() {
          return null;
        }, q = "", G = function() {
          var Ie = function(bd, pr, oi, ar) {
            for (oi = oi || {}, ar = bd.length; ar--; oi[bd[ar]] = pr) ;
            return oi;
          }, rt = [2, 13], ft = [1, 104], pt = [1, 102], wt = [1, 103], Bt = [1, 6], At = [1, 42], Dt = [1, 79], Ot = [1, 76], un = [1, 94], yn = [1, 93], _n = [1, 69], gn = [1, 101], bn = [1, 85], an = [1, 64], Zt = [1, 71], Mn = [1, 84], Ln = [1, 66], In = [1, 70], kn = [1, 68], er = [1, 61], jn = [1, 74], ur = [1, 62], Kn = [1, 67], Jn = [1, 83], Lr = [1, 77], mr = [1, 86], Ar = [1, 87], Yr = [1, 81], jr = [1, 82], Mr = [1, 80], Wr = [1, 88], Bi = [1, 89], gi = [1, 90], Zr = [1, 91], fi = [1, 92], ri = [1, 98], bi = [1, 65], Zi = [1, 78], $i = [1, 72], Br = [1, 96], Pr = [1, 97], ei = [1, 63], Ci = [1, 73], oa = [1, 108], ka = [1, 107], aa = [10, 311, 607, 768], hi = [10, 311, 315, 607, 768], Gr = [1, 115], Ca = [1, 117], Sr = [1, 116], Dr = [1, 118], si = [1, 119], zr = [1, 120], di = [1, 121], Da = [130, 358, 415], la = [1, 129], wa = [1, 128], Wa = [1, 136], Qr = [1, 166], Ur = [1, 178], rr = [1, 181], vr = [1, 176], Rr = [1, 184], Vr = [1, 188], Xr = [1, 162], Jr = [1, 185], ti = [1, 172], Hr = [1, 174], Er = [1, 177], gr = [1, 186], $r = [1, 203], li = [1, 204], Qi = [1, 168], Ai = [1, 169], Li = [1, 196], Ni = [1, 191], Pi = [1, 192], Oi = [1, 197], ki = [1, 198], Mi = [1, 199], Vi = [1, 200], ua = [1, 201], Ki = [1, 202], Fi = [1, 205], na = [1, 206], nn = [1, 179], Tn = [1, 180], Dn = [1, 182], Fn = [1, 183], fr = [1, 189], dr = [1, 195], yr = [1, 187], ai = [1, 190], Fr = [1, 175], Ti = [1, 173], Si = [1, 194], Ei = [1, 207], Fa = [2, 4, 5], $o = [2, 480], Gs = [1, 210], Al = [1, 215], ul = [1, 224], gc = [1, 220], Gl = [10, 72, 78, 93, 98, 118, 128, 162, 168, 169, 183, 198, 232, 249, 251, 311, 315, 607, 768], Sl = [2, 4, 5, 10, 72, 76, 77, 78, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 183, 185, 187, 198, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 311, 315, 425, 429, 607, 768], ls = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], vs = [1, 253], Vl = [1, 260], Ll = [1, 261], Xl = [1, 270], yc = [1, 275], gu = [1, 274], Nc = [2, 4, 5, 10, 72, 77, 78, 93, 98, 107, 118, 128, 131, 132, 137, 143, 145, 149, 152, 154, 156, 162, 168, 169, 179, 180, 181, 183, 198, 232, 244, 245, 249, 251, 269, 270, 271, 275, 276, 278, 285, 286, 287, 288, 289, 290, 291, 292, 294, 295, 296, 297, 298, 299, 300, 301, 302, 303, 304, 307, 308, 311, 315, 317, 322, 425, 429, 607, 768], Cu = [2, 162], zc = [1, 286], Kc = [10, 74, 78, 311, 315, 510, 607, 768], zi = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 193, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 302, 305, 307, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 348, 349, 361, 373, 374, 375, 378, 379, 391, 394, 401, 405, 406, 407, 408, 409, 410, 411, 413, 414, 422, 423, 425, 429, 431, 438, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 519, 520, 521, 522, 607, 768], Fu = [2, 4, 5, 10, 53, 72, 89, 124, 146, 156, 189, 271, 272, 294, 311, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 510, 512, 513, 522, 607, 768], d1 = [1, 567], Gu = [1, 569], Ku = [1, 570], I1 = [2, 512], f1 = [1, 576], D1 = [1, 587], Hl = [1, 590], Kl = [1, 591], uu = [10, 78, 89, 132, 137, 146, 189, 301, 311, 315, 475, 607, 768], Hs = [10, 74, 311, 315, 607, 768], b1 = [2, 576], Wl = [1, 609], zl = [2, 4, 5, 156], os = [1, 647], jo = [1, 619], uo = [1, 653], so = [1, 654], So = [1, 627], Jc = [1, 638], No = [1, 625], ro = [1, 633], Io = [1, 626], bs = [1, 634], ws = [1, 636], zo = [1, 628], Yo = [1, 629], Ls = [1, 648], Ms = [1, 645], vi = [1, 646], Ri = [1, 622], ta = [1, 624], da = [1, 616], Ji = [1, 617], Pa = [1, 618], Ga = [1, 620], Ha = [1, 621], Ba = [1, 623], ao = [1, 630], io = [1, 631], To = [1, 635], Mo = [1, 637], po = [1, 639], Vo = [1, 640], Xo = [1, 641], Go = [1, 642], es = [1, 643], Ts = [1, 649], Fo = [1, 650], Oo = [1, 651], gs = [1, 652], Bu = [2, 4, 5, 10, 53, 72, 74, 76, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], Ql = [2, 291], dl = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 302, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 348, 361, 373, 374, 378, 379, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 431, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], h1 = [2, 368], z1 = [1, 675], J1 = [1, 685], Lu = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 431, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], M1 = [1, 701], ku = [1, 710], Wu = [1, 709], T1 = [2, 4, 5, 10, 72, 74, 78, 93, 98, 118, 128, 162, 168, 169, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 249, 251, 311, 315, 607, 768], kl = [10, 72, 74, 78, 93, 98, 118, 128, 162, 168, 169, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 249, 251, 311, 315, 607, 768], Yl = [2, 202], i1 = [1, 732], ic = [10, 72, 78, 93, 98, 118, 128, 162, 168, 169, 183, 232, 249, 251, 311, 315, 607, 768], iu = [2, 163], xu = [1, 735], vc = [2, 4, 5, 112], Zl = [1, 748], Ec = [1, 767], wu = [1, 747], Pc = [1, 746], du = [1, 741], Ac = [1, 742], yu = [1, 744], au = [1, 745], Qc = [1, 749], Cc = [1, 750], Ol = [1, 751], uc = [1, 752], cs = [1, 753], Mu = [1, 754], Hu = [1, 755], ou = [1, 756], Gc = [1, 757], Su = [1, 758], Lc = [1, 759], Do = [1, 760], Ru = [1, 761], dc = [1, 762], vl = [1, 763], Eu = [1, 764], fu = [1, 766], ac = [1, 768], bc = [1, 769], kc = [1, 770], fc = [1, 771], oc = [1, 772], ss = [1, 773], su = [1, 774], Tl = [1, 777], sc = [1, 778], hc = [1, 779], Vc = [1, 780], Rc = [1, 781], Zo = [1, 782], Ou = [1, 783], Uc = [1, 784], El = [1, 785], eu = [1, 786], Wc = [1, 787], hu = [1, 788], gd = [74, 89, 189], Jl = [10, 74, 78, 154, 187, 230, 302, 311, 315, 348, 361, 373, 374, 378, 379, 607, 768], Yc = [1, 805], nc = [10, 74, 78, 305, 311, 315, 607, 768], Ko = [1, 806], ks = [1, 812], qs = [1, 813], a1 = [1, 817], $l = [10, 74, 78, 311, 315, 607, 768], o1 = [2, 4, 5, 77, 131, 132, 137, 143, 145, 149, 152, 154, 156, 179, 180, 181, 244, 245, 269, 270, 271, 275, 276, 278, 285, 286, 287, 288, 289, 290, 291, 292, 294, 295, 296, 297, 298, 299, 300, 301, 302, 303, 304, 307, 308, 317, 322, 425, 429], $c = [10, 72, 78, 93, 98, 107, 118, 128, 162, 168, 169, 183, 198, 232, 249, 251, 311, 315, 607, 768], lu = [2, 4, 5, 10, 72, 77, 78, 93, 98, 107, 118, 128, 131, 132, 137, 143, 145, 149, 152, 154, 156, 162, 164, 168, 169, 179, 180, 181, 183, 185, 187, 195, 198, 232, 244, 245, 249, 251, 269, 270, 271, 275, 276, 278, 285, 286, 287, 288, 289, 290, 291, 292, 294, 295, 296, 297, 298, 299, 300, 301, 302, 303, 304, 307, 308, 311, 315, 317, 322, 425, 429, 607, 768], Ul = [2, 4, 5, 132, 301], zs = [1, 853], Os = [10, 74, 76, 78, 311, 315, 607, 768], lc = [2, 748], xc = [10, 74, 76, 78, 132, 139, 141, 145, 152, 311, 315, 425, 429, 607, 768], Iu = [2, 1171], tu = [10, 74, 76, 78, 139, 141, 145, 152, 311, 315, 425, 429, 607, 768], w1 = [10, 74, 76, 78, 139, 141, 145, 311, 315, 425, 429, 607, 768], $n = [10, 74, 78, 139, 141, 311, 315, 607, 768], zn = [10, 78, 89, 132, 146, 189, 301, 311, 315, 475, 607, 768], Gn = [340, 343, 344], Ke = [2, 774], Fe = [1, 878], Te = [1, 879], Ae = [1, 880], Oe = [1, 881], Be = [1, 890], Ue = [1, 889], nt = [164, 166, 339], ht = [2, 453], Et = [1, 945], Mt = [2, 4, 5, 77, 131, 156, 270, 294, 295, 296, 297, 298], qt = [1, 960], tn = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 318, 319, 320, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], xn = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], An = [2, 384], On = [1, 967], Bn = [311, 313, 315], qn = [74, 305], Vn = [74, 305, 431], sr = [1, 974], Un = [74, 431], wr = [1, 987], Cr = [1, 986], Tr = [1, 993], ii = [10, 72, 78, 93, 98, 118, 128, 162, 168, 169, 232, 249, 251, 311, 315, 607, 768], wi = [1, 1020], Wi = [10, 72, 78, 311, 315, 607, 768], Yi = [1, 1026], ra = [1, 1027], ha = [1, 1028], ya = [2, 4, 5, 10, 72, 74, 76, 77, 78, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 198, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 311, 315, 425, 429, 607, 768], va = [1, 1078], qa = [1, 1077], Aa = [1, 1091], Xa = [1, 1090], Oa = [1, 1098], Ya = [10, 72, 74, 78, 93, 98, 107, 118, 128, 162, 168, 169, 183, 198, 232, 249, 251, 311, 315, 607, 768], Wo = [1, 1130], Ja = [10, 78, 89, 146, 189, 311, 315, 475, 607, 768], Uo = [1, 1150], ys = [1, 1149], il = [1, 1148], Is = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 302, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 348, 361, 373, 374, 378, 379, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], ml = [1, 1164], nl = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 318, 319, 320, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], Il = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 318, 320, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], Fl = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 118, 122, 124, 128, 129, 130, 131, 132, 133, 134, 135, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 318, 319, 320, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], qc = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 318, 319, 320, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], ql = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 319, 325, 326, 327, 328, 329, 330, 331, 335, 336, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], jc = [2, 415], Dl = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 107, 118, 122, 128, 129, 130, 131, 132, 134, 135, 137, 143, 145, 146, 148, 149, 150, 152, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 319, 335, 336, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], Ml = [2, 289], pc = [2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 431, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], hl = [10, 78, 311, 315, 607, 768], xl = [1, 1200], Bl = [10, 77, 78, 143, 145, 152, 181, 307, 311, 315, 425, 429, 607, 768], al = [10, 74, 78, 311, 313, 315, 469, 607, 768], Oc = [1, 1211], nu = [10, 72, 78, 118, 128, 162, 168, 169, 232, 249, 251, 311, 315, 607, 768], Ic = [10, 72, 74, 78, 93, 98, 118, 128, 162, 168, 169, 183, 198, 232, 249, 251, 311, 315, 607, 768], $s = [2, 4, 5, 72, 76, 77, 78, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 185, 187, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 425, 429], Xc = [2, 4, 5, 72, 74, 76, 77, 78, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 185, 187, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 425, 429], wc = [2, 1095], zu = [2, 4, 5, 72, 74, 76, 77, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 185, 187, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 425, 429], Zc = [1, 1264], Ju = [10, 74, 78, 128, 311, 313, 315, 469, 607, 768], ru = [115, 116, 124], Bd = [2, 593], tf = [1, 1293], wd = [76, 139], od = [2, 734], yd = [1, 1310], Ef = [1, 1311], Md = [2, 4, 5, 10, 53, 72, 76, 89, 124, 146, 156, 189, 230, 271, 272, 294, 311, 315, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 510, 512, 513, 522, 607, 768], nf = [2, 336], Vd = [1, 1335], S1 = [1, 1349], Af = [1, 1351], Pd = [2, 496], Au = [74, 78], Vu = [10, 311, 313, 315, 469, 607, 768], Yh = [10, 72, 78, 118, 162, 168, 169, 232, 249, 251, 311, 315, 607, 768], Qf = [1, 1368], vd = [1, 1372], Ff = [1, 1373], Uu = [1, 1375], rf = [1, 1376], Ud = [1, 1377], $d = [1, 1378], Sd = [1, 1379], Yd = [1, 1380], af = [1, 1381], of = [1, 1382], Bf = [10, 72, 74, 78, 93, 98, 118, 128, 162, 168, 169, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 249, 251, 311, 315, 607, 768], qd = [1, 1407], Cf = [10, 72, 78, 118, 162, 168, 169, 249, 251, 311, 315, 607, 768], V1 = [10, 72, 78, 93, 98, 118, 128, 162, 168, 169, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 249, 251, 311, 315, 607, 768], jd = [1, 1505], E1 = [1, 1507], sd = [2, 4, 5, 77, 143, 145, 152, 156, 181, 270, 294, 295, 296, 297, 298, 307, 425, 429], Xd = [1, 1521], ld = [10, 72, 74, 78, 162, 168, 169, 249, 251, 311, 315, 607, 768], Ch = [1, 1539], sf = [1, 1541], Zd = [1, 1542], Lf = [1, 1538], uh = [1, 1537], Xf = [1, 1536], Lh = [1, 1543], p1 = [1, 1533], cd = [1, 1534], dh = [1, 1535], kh = [1, 1561], Mh = [2, 4, 5, 10, 53, 72, 89, 124, 146, 156, 189, 271, 272, 294, 311, 315, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 510, 512, 513, 522, 607, 768], h2 = [1, 1572], Rh = [1, 1580], Zf = [1, 1579], Rd = [10, 72, 78, 162, 168, 169, 249, 251, 311, 315, 607, 768], ed = [10, 72, 78, 93, 98, 118, 128, 162, 168, 169, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 249, 251, 311, 315, 607, 768], Hf = [2, 4, 5, 10, 72, 78, 93, 98, 118, 128, 162, 168, 169, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 249, 251, 311, 315, 607, 768], Od = [1, 1640], Oh = [1, 1642], Yu = [1, 1639], kf = [1, 1641], Mf = [187, 193, 373, 374, 375, 378], e1 = [2, 524], Id = [1, 1647], cu = [1, 1666], pu = [10, 72, 78, 162, 168, 169, 311, 315, 607, 768], gf = [1, 1676], lf = [1, 1677], yf = [1, 1678], Ih = [1, 1700], Kf = [4, 10, 247, 311, 315, 348, 361, 607, 768], zf = [1, 1748], cf = [10, 72, 74, 78, 118, 162, 168, 169, 239, 249, 251, 311, 315, 607, 768], Xh = [2, 4, 5, 77], P2 = [1, 1844], Jf = [1, 1856], eh = [1, 1875], td = [10, 72, 78, 162, 168, 169, 311, 315, 420, 607, 768], Rf = [10, 74, 78, 230, 311, 315, 607, 768], fh = { trace: function() {
          }, yy: {}, symbols_: { error: 2, Literal: 3, LITERAL: 4, BRALITERAL: 5, NonReserved: 6, LiteralWithSpaces: 7, main: 8, Statements: 9, EOF: 10, Statements_group0: 11, AStatement: 12, ExplainStatement: 13, EXPLAIN: 14, QUERY: 15, PLAN: 16, Statement: 17, AlterTable: 18, AttachDatabase: 19, Call: 20, CreateDatabase: 21, CreateIndex: 22, CreateGraph: 23, CreateTable: 24, CreateView: 25, CreateEdge: 26, CreateVertex: 27, Declare: 28, Delete: 29, DetachDatabase: 30, DropDatabase: 31, DropIndex: 32, DropTable: 33, DropView: 34, If: 35, Insert: 36, Merge: 37, Reindex: 38, RenameTable: 39, Select: 40, ShowCreateTable: 41, ShowColumns: 42, ShowDatabases: 43, ShowIndex: 44, ShowTables: 45, TruncateTable: 46, WithSelect: 47, CreateTrigger: 48, DropTrigger: 49, BeginTransaction: 50, CommitTransaction: 51, RollbackTransaction: 52, EndTransaction: 53, UseDatabase: 54, Update: 55, JavaScript: 56, Source: 57, Assert: 58, While: 59, Continue: 60, Break: 61, BeginEnd: 62, Print: 63, Require: 64, SetVariable: 65, ExpressionStatement: 66, AddRule: 67, Query: 68, Echo: 69, CreateFunction: 70, CreateAggregate: 71, WITH: 72, WithTablesList: 73, COMMA: 74, WithTable: 75, AS: 76, LPAR: 77, RPAR: 78, SelectClause: 79, Select_option0: 80, IntoClause: 81, FromClause: 82, Select_option1: 83, WhereClause: 84, GroupClause: 85, OrderClause: 86, LimitClause: 87, UnionClause: 88, SEARCH: 89, Select_repetition0: 90, Select_option2: 91, PivotClause: 92, PIVOT: 93, Expression: 94, FOR: 95, PivotClause_option0: 96, PivotClause_option1: 97, UNPIVOT: 98, IN: 99, ColumnsList: 100, PivotClause_option2: 101, PivotClause2: 102, AsList: 103, AsLiteral: 104, AsPart: 105, RemoveClause: 106, REMOVE: 107, RemoveClause_option0: 108, RemoveColumnsList: 109, RemoveColumn: 110, Column: 111, LIKE: 112, StringValue: 113, ArrowDot: 114, ARROW: 115, DOT: 116, SearchSelector: 117, ORDER: 118, BY: 119, OrderExpressionsList: 120, SearchSelector_option0: 121, DOTDOT: 122, CARET: 123, EQ: 124, SearchSelector_repetition_plus0: 125, SearchSelector_repetition_plus1: 126, SearchSelector_option1: 127, WHERE: 128, OF: 129, CLASS: 130, NUMBER: 131, STRING: 132, SLASH: 133, VERTEX: 134, EDGE: 135, EXCLAMATION: 136, SHARP: 137, MODULO: 138, GT: 139, LT: 140, GTGT: 141, LTLT: 142, DOLLAR: 143, Json: 144, AT: 145, SET: 146, SetColumnsList: 147, TO: 148, VALUE: 149, ROW: 150, ExprList: 151, COLON: 152, PlusStar: 153, NOT: 154, SearchSelector_repetition2: 155, IF: 156, SearchSelector_repetition3: 157, Aggregator: 158, SearchSelector_repetition4: 159, SearchSelector_group0: 160, SearchSelector_repetition5: 161, UNION: 162, SearchSelectorList: 163, ALL: 164, SearchSelector_repetition6: 165, ANY: 166, SearchSelector_repetition7: 167, INTERSECT: 168, EXCEPT: 169, AND: 170, OR: 171, PATH: 172, RETURN: 173, ResultColumns: 174, REPEAT: 175, SearchSelector_repetition8: 176, SearchSelectorList_repetition0: 177, SearchSelectorList_repetition1: 178, PLUS: 179, STAR: 180, QUESTION: 181, SearchFrom: 182, FROM: 183, SelectModifier: 184, DISTINCT: 185, TopClause: 186, UNIQUE: 187, SelectClause_option0: 188, SELECT: 189, COLUMN: 190, MATRIX: 191, TEXTSTRING: 192, INDEX: 193, RECORDSET: 194, TOP: 195, NumValue: 196, TopClause_option0: 197, INTO: 198, Table: 199, FuncValue: 200, ParamValue: 201, VarValue: 202, FromTablesList: 203, JoinTablesList: 204, ApplyClause: 205, CROSS: 206, APPLY: 207, OUTER: 208, FromTable: 209, FromTable_option0: 210, FromTable_option1: 211, INDEXED: 212, INSERTED: 213, FromString: 214, JoinTable: 215, JoinMode: 216, JoinTableAs: 217, OnClause: 218, JoinTableAs_option0: 219, JoinTableAs_option1: 220, JoinModeMode: 221, NATURAL: 222, JOIN: 223, INNER: 224, LEFT: 225, RIGHT: 226, FULL: 227, SEMI: 228, ANTI: 229, ON: 230, USING: 231, GROUP: 232, GroupExpressionsList: 233, HavingClause: 234, GroupExpression: 235, GROUPING: 236, ROLLUP: 237, CUBE: 238, HAVING: 239, CORRESPONDING: 240, OrderExpression: 241, NullsOrder: 242, NULLS: 243, FIRST: 244, LAST: 245, DIRECTION: 246, COLLATE: 247, NOCASE: 248, LIMIT: 249, OffsetClause: 250, OFFSET: 251, LimitClause_option0: 252, FETCH: 253, LimitClause_option1: 254, LimitClause_option2: 255, LimitClause_option3: 256, ResultColumn: 257, Star: 258, AggrValue: 259, Op: 260, LogicValue: 261, NullValue: 262, ExistsValue: 263, CaseValue: 264, CastClause: 265, ArrayValue: 266, NewClause: 267, Expression_group0: 268, CURRENT_TIMESTAMP: 269, CURRENT_DATE: 270, JAVASCRIPT: 271, CREATE: 272, FUNCTION: 273, AGGREGATE: 274, NEW: 275, CAST: 276, ColumnType: 277, CONVERT: 278, PrimitiveValue: 279, OverClause: 280, OVER: 281, OverPartitionClause: 282, OverOrderByClause: 283, PARTITION: 284, SUM: 285, TOTAL: 286, COUNT: 287, MIN: 288, MAX: 289, AVG: 290, AGGR: 291, ARRAY: 292, FuncValue_option0: 293, REPLACE: 294, DATEADD: 295, DATEDIFF: 296, TIMESTAMPDIFF: 297, INTERVAL: 298, TRUE: 299, FALSE: 300, NSTRING: 301, NULL: 302, EXISTS: 303, ARRAYLBRA: 304, RBRA: 305, ParamValue_group0: 306, BRAQUESTION: 307, CASE: 308, WhensList: 309, ElseClause: 310, END: 311, When: 312, WHEN: 313, THEN: 314, ELSE: 315, REGEXP: 316, TILDA: 317, GLOB: 318, ESCAPE: 319, NOT_LIKE: 320, BARBAR: 321, MINUS: 322, AMPERSAND: 323, BAR: 324, GE: 325, LE: 326, EQEQ: 327, EQEQEQ: 328, NE: 329, NEEQEQ: 330, NEEQEQEQ: 331, CondOp: 332, AllSome: 333, ColFunc: 334, BETWEEN: 335, NOT_BETWEEN: 336, IS: 337, DOUBLECOLON: 338, SOME: 339, UPDATE: 340, SetColumn: 341, SetColumn_group0: 342, DELETE: 343, INSERT: 344, Into: 345, Values: 346, ValuesListsList: 347, DEFAULT: 348, VALUES: 349, ValuesList: 350, Value: 351, DateValue: 352, TemporaryClause: 353, TableClass: 354, IfNotExists: 355, CreateTableDefClause: 356, CreateTableOptionsClause: 357, TABLE: 358, CreateTableOptions: 359, CreateTableOption: 360, IDENTITY: 361, TEMP: 362, ColumnDefsList: 363, ConstraintsList: 364, Constraint: 365, ConstraintName: 366, PrimaryKey: 367, ForeignKey: 368, UniqueKey: 369, IndexKey: 370, Check: 371, CONSTRAINT: 372, CHECK: 373, PRIMARY: 374, KEY: 375, PrimaryKey_option0: 376, ColsList: 377, FOREIGN: 378, REFERENCES: 379, ForeignKey_option0: 380, OnForeignKeyClause: 381, ParColsList: 382, OnDeleteClause: 383, OnUpdateClause: 384, NO: 385, ACTION: 386, UniqueKey_option0: 387, UniqueKey_option1: 388, ColumnDef: 389, ColumnConstraintsClause: 390, ColumnConstraints: 391, SingularColumnType: 392, NumberMax: 393, ENUM: 394, MAXNUM: 395, ColumnConstraintsList: 396, ColumnConstraint: 397, ParLiteral: 398, ColumnConstraint_option0: 399, ColumnConstraint_option1: 400, DROP: 401, DropTable_group0: 402, IfExists: 403, TablesList: 404, ALTER: 405, RENAME: 406, ADD: 407, MODIFY: 408, ATTACH: 409, DATABASE: 410, DETACH: 411, AsClause: 412, USE: 413, SHOW: 414, VIEW: 415, CreateView_option0: 416, CreateView_option1: 417, SubqueryRestriction: 418, READ: 419, ONLY: 420, OPTION: 421, SOURCE: 422, ASSERT: 423, JsonObject: 424, ATLBRA: 425, JsonArray: 426, JsonValue: 427, JsonPrimitiveValue: 428, LCUR: 429, JsonPropertiesList: 430, RCUR: 431, JsonElementsList: 432, JsonProperty: 433, OnOff: 434, SetPropsList: 435, AtDollar: 436, SetProp: 437, OFF: 438, COMMIT: 439, TRANSACTION: 440, ROLLBACK: 441, BEGIN: 442, ElseStatement: 443, WHILE: 444, CONTINUE: 445, BREAK: 446, PRINT: 447, REQUIRE: 448, StringValuesList: 449, PluginsList: 450, Plugin: 451, ECHO: 452, DECLARE: 453, DeclaresList: 454, DeclareItem: 455, TRUNCATE: 456, MERGE: 457, MergeInto: 458, MergeUsing: 459, MergeOn: 460, MergeMatchedList: 461, OutputClause: 462, MergeMatched: 463, MergeNotMatched: 464, MATCHED: 465, MergeMatchedAction: 466, MergeNotMatchedAction: 467, TARGET: 468, OUTPUT: 469, CreateVertex_option0: 470, CreateVertex_option1: 471, CreateVertex_option2: 472, CreateVertexSet: 473, SharpValue: 474, CONTENT: 475, CreateEdge_option0: 476, GRAPH: 477, GraphList: 478, GraphVertexEdge: 479, GraphElement: 480, GraphVertexEdge_option0: 481, GraphVertexEdge_option1: 482, GraphElementVar: 483, GraphVertexEdge_option2: 484, GraphVertexEdge_option3: 485, GraphVertexEdge_option4: 486, GraphVar: 487, GraphAsClause: 488, GraphAtClause: 489, GraphElement2: 490, GraphElement2_option0: 491, GraphElement2_option1: 492, GraphElement2_option2: 493, GraphElement2_option3: 494, GraphElement_option0: 495, GraphElement_option1: 496, GraphElement_option2: 497, SharpLiteral: 498, GraphElement_option3: 499, GraphElement_option4: 500, GraphElement_option5: 501, ColonLiteral: 502, DeleteVertex: 503, DeleteVertex_option0: 504, DeleteEdge: 505, DeleteEdge_option0: 506, DeleteEdge_option1: 507, DeleteEdge_option2: 508, Term: 509, COLONDASH: 510, TermsList: 511, QUESTIONDASH: 512, CALL: 513, TRIGGER: 514, BeforeAfter: 515, InsertDeleteUpdate: 516, CreateTrigger_option0: 517, CreateTrigger_option1: 518, BEFORE: 519, AFTER: 520, INSTEAD: 521, REINDEX: 522, A: 523, ABSENT: 524, ABSOLUTE: 525, ACCORDING: 526, ADA: 527, ADMIN: 528, ALWAYS: 529, ASC: 530, ASSERTION: 531, ASSIGNMENT: 532, ATTRIBUTE: 533, ATTRIBUTES: 534, BASE64: 535, BERNOULLI: 536, BLOCKED: 537, BOM: 538, BREADTH: 539, C: 540, CASCADE: 541, CATALOG: 542, CATALOG_NAME: 543, CHAIN: 544, CHARACTERISTICS: 545, CHARACTERS: 546, CHARACTER_SET_CATALOG: 547, CHARACTER_SET_NAME: 548, CHARACTER_SET_SCHEMA: 549, CLASS_ORIGIN: 550, COBOL: 551, COLLATION: 552, COLLATION_CATALOG: 553, COLLATION_NAME: 554, COLLATION_SCHEMA: 555, COLUMNS: 556, COLUMN_NAME: 557, COMMAND_FUNCTION: 558, COMMAND_FUNCTION_CODE: 559, COMMITTED: 560, CONDITION_NUMBER: 561, CONNECTION: 562, CONNECTION_NAME: 563, CONSTRAINTS: 564, CONSTRAINT_CATALOG: 565, CONSTRAINT_NAME: 566, CONSTRAINT_SCHEMA: 567, CONSTRUCTOR: 568, CONTROL: 569, CURSOR_NAME: 570, DATA: 571, DATETIME_INTERVAL_CODE: 572, DATETIME_INTERVAL_PRECISION: 573, DB: 574, DEFAULTS: 575, DEFERRABLE: 576, DEFERRED: 577, DEFINED: 578, DEFINER: 579, DEGREE: 580, DEPTH: 581, DERIVED: 582, DESC: 583, DESCRIPTOR: 584, DIAGNOSTICS: 585, DISPATCH: 586, DOCUMENT: 587, DOMAIN: 588, DYNAMIC_FUNCTION: 589, DYNAMIC_FUNCTION_CODE: 590, EMPTY: 591, ENCODING: 592, ENFORCED: 593, EXCLUDE: 594, EXCLUDING: 595, EXPRESSION: 596, FILE: 597, FINAL: 598, FLAG: 599, FOLLOWING: 600, FORTRAN: 601, FOUND: 602, FS: 603, G: 604, GENERAL: 605, GENERATED: 606, GO: 607, GOTO: 608, GRANTED: 609, HEX: 610, HIERARCHY: 611, ID: 612, IGNORE: 613, IMMEDIATE: 614, IMMEDIATELY: 615, IMPLEMENTATION: 616, INCLUDING: 617, INCREMENT: 618, INDENT: 619, INITIALLY: 620, INPUT: 621, INSTANCE: 622, INSTANTIABLE: 623, INTEGRITY: 624, INVOKER: 625, ISOLATION: 626, K: 627, KEY_MEMBER: 628, KEY_TYPE: 629, LENGTH: 630, LEVEL: 631, LIBRARY: 632, LINK: 633, LOCATION: 634, LOCATOR: 635, M: 636, MAP: 637, MAPPING: 638, MAXVALUE: 639, MESSAGE_LENGTH: 640, MESSAGE_OCTET_LENGTH: 641, MESSAGE_TEXT: 642, MINVALUE: 643, MORE: 644, MUMPS: 645, NAME: 646, NAMES: 647, NAMESPACE: 648, NESTING: 649, NEXT: 650, NFC: 651, NFD: 652, NFKC: 653, NFKD: 654, NIL: 655, NORMALIZED: 656, NULLABLE: 657, OBJECT: 658, OCTETS: 659, OPTIONS: 660, ORDERING: 661, ORDINALITY: 662, OTHERS: 663, OVERRIDING: 664, P: 665, PAD: 666, PARAMETER_MODE: 667, PARAMETER_NAME: 668, PARAMETER_ORDINAL_POSITION: 669, PARAMETER_SPECIFIC_CATALOG: 670, PARAMETER_SPECIFIC_NAME: 671, PARAMETER_SPECIFIC_SCHEMA: 672, PARTIAL: 673, PASCAL: 674, PASSING: 675, PASSTHROUGH: 676, PERMISSION: 677, PLACING: 678, PLI: 679, PRECEDING: 680, PRESERVE: 681, PRIOR: 682, PRIVILEGES: 683, PUBLIC: 684, RECOVERY: 685, RELATIVE: 686, REPEATABLE: 687, REQUIRING: 688, RESPECT: 689, RESTART: 690, RESTORE: 691, RESTRICT: 692, RETURNED_CARDINALITY: 693, RETURNED_LENGTH: 694, RETURNED_OCTET_LENGTH: 695, RETURNED_SQLSTATE: 696, RETURNING: 697, ROLE: 698, ROUTINE: 699, ROUTINE_CATALOG: 700, ROUTINE_NAME: 701, ROUTINE_SCHEMA: 702, ROW_COUNT: 703, SCALE: 704, SCHEMA: 705, SCHEMA_NAME: 706, SCOPE_CATALOG: 707, SCOPE_NAME: 708, SCOPE_SCHEMA: 709, SECTION: 710, SECURITY: 711, SELECTIVE: 712, SELF: 713, SEQUENCE: 714, SERIALIZABLE: 715, SERVER: 716, SERVER_NAME: 717, SESSION: 718, SETS: 719, SIMPLE: 720, SIZE: 721, SPACE: 722, SPECIFIC_NAME: 723, STANDALONE: 724, STATE: 725, STATEMENT: 726, STRIP: 727, STRUCTURE: 728, STYLE: 729, SUBCLASS_ORIGIN: 730, T: 731, TABLE_NAME: 732, TEMPORARY: 733, TIES: 734, TOKEN: 735, TOP_LEVEL_COUNT: 736, TRANSACTIONS_COMMITTED: 737, TRANSACTIONS_ROLLED_BACK: 738, TRANSACTION_ACTIVE: 739, TRANSFORM: 740, TRANSFORMS: 741, TRIGGER_CATALOG: 742, TRIGGER_NAME: 743, TRIGGER_SCHEMA: 744, TYPE: 745, UNBOUNDED: 746, UNCOMMITTED: 747, UNDER: 748, UNLINK: 749, UNNAMED: 750, UNTYPED: 751, URI: 752, USAGE: 753, USER_DEFINED_TYPE_CATALOG: 754, USER_DEFINED_TYPE_CODE: 755, USER_DEFINED_TYPE_NAME: 756, USER_DEFINED_TYPE_SCHEMA: 757, VALID: 758, VERSION: 759, WHITESPACE: 760, WORK: 761, WRAPPER: 762, WRITE: 763, XMLDECLARATION: 764, XMLSCHEMA: 765, YES: 766, ZONE: 767, SEMICOLON: 768, PERCENT: 769, ROWS: 770, FuncValue_option0_group0: 771, $accept: 0, $end: 1 }, terminals_: { 2: "error", 4: "LITERAL", 5: "BRALITERAL", 10: "EOF", 14: "EXPLAIN", 15: "QUERY", 16: "PLAN", 53: "EndTransaction", 72: "WITH", 74: "COMMA", 76: "AS", 77: "LPAR", 78: "RPAR", 89: "SEARCH", 93: "PIVOT", 95: "FOR", 98: "UNPIVOT", 99: "IN", 107: "REMOVE", 112: "LIKE", 115: "ARROW", 116: "DOT", 118: "ORDER", 119: "BY", 122: "DOTDOT", 123: "CARET", 124: "EQ", 128: "WHERE", 129: "OF", 130: "CLASS", 131: "NUMBER", 132: "STRING", 133: "SLASH", 134: "VERTEX", 135: "EDGE", 136: "EXCLAMATION", 137: "SHARP", 138: "MODULO", 139: "GT", 140: "LT", 141: "GTGT", 142: "LTLT", 143: "DOLLAR", 145: "AT", 146: "SET", 148: "TO", 149: "VALUE", 150: "ROW", 152: "COLON", 154: "NOT", 156: "IF", 162: "UNION", 164: "ALL", 166: "ANY", 168: "INTERSECT", 169: "EXCEPT", 170: "AND", 171: "OR", 172: "PATH", 173: "RETURN", 175: "REPEAT", 179: "PLUS", 180: "STAR", 181: "QUESTION", 183: "FROM", 185: "DISTINCT", 187: "UNIQUE", 189: "SELECT", 190: "COLUMN", 191: "MATRIX", 192: "TEXTSTRING", 193: "INDEX", 194: "RECORDSET", 195: "TOP", 198: "INTO", 206: "CROSS", 207: "APPLY", 208: "OUTER", 212: "INDEXED", 213: "INSERTED", 222: "NATURAL", 223: "JOIN", 224: "INNER", 225: "LEFT", 226: "RIGHT", 227: "FULL", 228: "SEMI", 229: "ANTI", 230: "ON", 231: "USING", 232: "GROUP", 236: "GROUPING", 237: "ROLLUP", 238: "CUBE", 239: "HAVING", 240: "CORRESPONDING", 243: "NULLS", 244: "FIRST", 245: "LAST", 246: "DIRECTION", 247: "COLLATE", 248: "NOCASE", 249: "LIMIT", 251: "OFFSET", 253: "FETCH", 269: "CURRENT_TIMESTAMP", 270: "CURRENT_DATE", 271: "JAVASCRIPT", 272: "CREATE", 273: "FUNCTION", 274: "AGGREGATE", 275: "NEW", 276: "CAST", 278: "CONVERT", 281: "OVER", 284: "PARTITION", 285: "SUM", 286: "TOTAL", 287: "COUNT", 288: "MIN", 289: "MAX", 290: "AVG", 291: "AGGR", 292: "ARRAY", 294: "REPLACE", 295: "DATEADD", 296: "DATEDIFF", 297: "TIMESTAMPDIFF", 298: "INTERVAL", 299: "TRUE", 300: "FALSE", 301: "NSTRING", 302: "NULL", 303: "EXISTS", 304: "ARRAYLBRA", 305: "RBRA", 307: "BRAQUESTION", 308: "CASE", 311: "END", 313: "WHEN", 314: "THEN", 315: "ELSE", 316: "REGEXP", 317: "TILDA", 318: "GLOB", 319: "ESCAPE", 320: "NOT_LIKE", 321: "BARBAR", 322: "MINUS", 323: "AMPERSAND", 324: "BAR", 325: "GE", 326: "LE", 327: "EQEQ", 328: "EQEQEQ", 329: "NE", 330: "NEEQEQ", 331: "NEEQEQEQ", 335: "BETWEEN", 336: "NOT_BETWEEN", 337: "IS", 338: "DOUBLECOLON", 339: "SOME", 340: "UPDATE", 343: "DELETE", 344: "INSERT", 348: "DEFAULT", 349: "VALUES", 352: "DateValue", 358: "TABLE", 361: "IDENTITY", 362: "TEMP", 372: "CONSTRAINT", 373: "CHECK", 374: "PRIMARY", 375: "KEY", 378: "FOREIGN", 379: "REFERENCES", 385: "NO", 386: "ACTION", 391: "ColumnConstraints", 394: "ENUM", 395: "MAXNUM", 401: "DROP", 405: "ALTER", 406: "RENAME", 407: "ADD", 408: "MODIFY", 409: "ATTACH", 410: "DATABASE", 411: "DETACH", 413: "USE", 414: "SHOW", 415: "VIEW", 419: "READ", 420: "ONLY", 421: "OPTION", 422: "SOURCE", 423: "ASSERT", 425: "ATLBRA", 429: "LCUR", 431: "RCUR", 438: "OFF", 439: "COMMIT", 440: "TRANSACTION", 441: "ROLLBACK", 442: "BEGIN", 444: "WHILE", 445: "CONTINUE", 446: "BREAK", 447: "PRINT", 448: "REQUIRE", 452: "ECHO", 453: "DECLARE", 456: "TRUNCATE", 457: "MERGE", 465: "MATCHED", 468: "TARGET", 469: "OUTPUT", 475: "CONTENT", 477: "GRAPH", 510: "COLONDASH", 512: "QUESTIONDASH", 513: "CALL", 514: "TRIGGER", 519: "BEFORE", 520: "AFTER", 521: "INSTEAD", 522: "REINDEX", 523: "A", 524: "ABSENT", 525: "ABSOLUTE", 526: "ACCORDING", 527: "ADA", 528: "ADMIN", 529: "ALWAYS", 530: "ASC", 531: "ASSERTION", 532: "ASSIGNMENT", 533: "ATTRIBUTE", 534: "ATTRIBUTES", 535: "BASE64", 536: "BERNOULLI", 537: "BLOCKED", 538: "BOM", 539: "BREADTH", 540: "C", 541: "CASCADE", 542: "CATALOG", 543: "CATALOG_NAME", 544: "CHAIN", 545: "CHARACTERISTICS", 546: "CHARACTERS", 547: "CHARACTER_SET_CATALOG", 548: "CHARACTER_SET_NAME", 549: "CHARACTER_SET_SCHEMA", 550: "CLASS_ORIGIN", 551: "COBOL", 552: "COLLATION", 553: "COLLATION_CATALOG", 554: "COLLATION_NAME", 555: "COLLATION_SCHEMA", 556: "COLUMNS", 557: "COLUMN_NAME", 558: "COMMAND_FUNCTION", 559: "COMMAND_FUNCTION_CODE", 560: "COMMITTED", 561: "CONDITION_NUMBER", 562: "CONNECTION", 563: "CONNECTION_NAME", 564: "CONSTRAINTS", 565: "CONSTRAINT_CATALOG", 566: "CONSTRAINT_NAME", 567: "CONSTRAINT_SCHEMA", 568: "CONSTRUCTOR", 569: "CONTROL", 570: "CURSOR_NAME", 571: "DATA", 572: "DATETIME_INTERVAL_CODE", 573: "DATETIME_INTERVAL_PRECISION", 574: "DB", 575: "DEFAULTS", 576: "DEFERRABLE", 577: "DEFERRED", 578: "DEFINED", 579: "DEFINER", 580: "DEGREE", 581: "DEPTH", 582: "DERIVED", 583: "DESC", 584: "DESCRIPTOR", 585: "DIAGNOSTICS", 586: "DISPATCH", 587: "DOCUMENT", 588: "DOMAIN", 589: "DYNAMIC_FUNCTION", 590: "DYNAMIC_FUNCTION_CODE", 591: "EMPTY", 592: "ENCODING", 593: "ENFORCED", 594: "EXCLUDE", 595: "EXCLUDING", 596: "EXPRESSION", 597: "FILE", 598: "FINAL", 599: "FLAG", 600: "FOLLOWING", 601: "FORTRAN", 602: "FOUND", 603: "FS", 604: "G", 605: "GENERAL", 606: "GENERATED", 607: "GO", 608: "GOTO", 609: "GRANTED", 610: "HEX", 611: "HIERARCHY", 612: "ID", 613: "IGNORE", 614: "IMMEDIATE", 615: "IMMEDIATELY", 616: "IMPLEMENTATION", 617: "INCLUDING", 618: "INCREMENT", 619: "INDENT", 620: "INITIALLY", 621: "INPUT", 622: "INSTANCE", 623: "INSTANTIABLE", 624: "INTEGRITY", 625: "INVOKER", 626: "ISOLATION", 627: "K", 628: "KEY_MEMBER", 629: "KEY_TYPE", 630: "LENGTH", 631: "LEVEL", 632: "LIBRARY", 633: "LINK", 634: "LOCATION", 635: "LOCATOR", 636: "M", 637: "MAP", 638: "MAPPING", 639: "MAXVALUE", 640: "MESSAGE_LENGTH", 641: "MESSAGE_OCTET_LENGTH", 642: "MESSAGE_TEXT", 643: "MINVALUE", 644: "MORE", 645: "MUMPS", 646: "NAME", 647: "NAMES", 648: "NAMESPACE", 649: "NESTING", 650: "NEXT", 651: "NFC", 652: "NFD", 653: "NFKC", 654: "NFKD", 655: "NIL", 656: "NORMALIZED", 657: "NULLABLE", 658: "OBJECT", 659: "OCTETS", 660: "OPTIONS", 661: "ORDERING", 662: "ORDINALITY", 663: "OTHERS", 664: "OVERRIDING", 665: "P", 666: "PAD", 667: "PARAMETER_MODE", 668: "PARAMETER_NAME", 669: "PARAMETER_ORDINAL_POSITION", 670: "PARAMETER_SPECIFIC_CATALOG", 671: "PARAMETER_SPECIFIC_NAME", 672: "PARAMETER_SPECIFIC_SCHEMA", 673: "PARTIAL", 674: "PASCAL", 675: "PASSING", 676: "PASSTHROUGH", 677: "PERMISSION", 678: "PLACING", 679: "PLI", 680: "PRECEDING", 681: "PRESERVE", 682: "PRIOR", 683: "PRIVILEGES", 684: "PUBLIC", 685: "RECOVERY", 686: "RELATIVE", 687: "REPEATABLE", 688: "REQUIRING", 689: "RESPECT", 690: "RESTART", 691: "RESTORE", 692: "RESTRICT", 693: "RETURNED_CARDINALITY", 694: "RETURNED_LENGTH", 695: "RETURNED_OCTET_LENGTH", 696: "RETURNED_SQLSTATE", 697: "RETURNING", 698: "ROLE", 699: "ROUTINE", 700: "ROUTINE_CATALOG", 701: "ROUTINE_NAME", 702: "ROUTINE_SCHEMA", 703: "ROW_COUNT", 704: "SCALE", 705: "SCHEMA", 706: "SCHEMA_NAME", 707: "SCOPE_CATALOG", 708: "SCOPE_NAME", 709: "SCOPE_SCHEMA", 710: "SECTION", 711: "SECURITY", 712: "SELECTIVE", 713: "SELF", 714: "SEQUENCE", 715: "SERIALIZABLE", 716: "SERVER", 717: "SERVER_NAME", 718: "SESSION", 719: "SETS", 720: "SIMPLE", 721: "SIZE", 722: "SPACE", 723: "SPECIFIC_NAME", 724: "STANDALONE", 725: "STATE", 726: "STATEMENT", 727: "STRIP", 728: "STRUCTURE", 729: "STYLE", 730: "SUBCLASS_ORIGIN", 731: "T", 732: "TABLE_NAME", 733: "TEMPORARY", 734: "TIES", 735: "TOKEN", 736: "TOP_LEVEL_COUNT", 737: "TRANSACTIONS_COMMITTED", 738: "TRANSACTIONS_ROLLED_BACK", 739: "TRANSACTION_ACTIVE", 740: "TRANSFORM", 741: "TRANSFORMS", 742: "TRIGGER_CATALOG", 743: "TRIGGER_NAME", 744: "TRIGGER_SCHEMA", 745: "TYPE", 746: "UNBOUNDED", 747: "UNCOMMITTED", 748: "UNDER", 749: "UNLINK", 750: "UNNAMED", 751: "UNTYPED", 752: "URI", 753: "USAGE", 754: "USER_DEFINED_TYPE_CATALOG", 755: "USER_DEFINED_TYPE_CODE", 756: "USER_DEFINED_TYPE_NAME", 757: "USER_DEFINED_TYPE_SCHEMA", 758: "VALID", 759: "VERSION", 760: "WHITESPACE", 761: "WORK", 762: "WRAPPER", 763: "WRITE", 764: "XMLDECLARATION", 765: "XMLSCHEMA", 766: "YES", 767: "ZONE", 768: "SEMICOLON", 769: "PERCENT", 770: "ROWS" }, productions_: [0, [3, 1], [3, 1], [3, 2], [7, 1], [7, 2], [8, 2], [9, 3], [9, 1], [9, 1], [13, 2], [13, 4], [12, 1], [17, 0], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [17, 1], [47, 3], [73, 3], [73, 1], [75, 5], [40, 10], [40, 4], [92, 8], [92, 11], [102, 4], [104, 2], [104, 1], [103, 3], [103, 1], [105, 1], [105, 3], [106, 3], [109, 3], [109, 1], [110, 1], [110, 2], [114, 1], [114, 1], [117, 1], [117, 5], [117, 5], [117, 1], [117, 2], [117, 1], [117, 2], [117, 2], [117, 3], [117, 4], [117, 4], [117, 4], [117, 4], [117, 4], [117, 1], [117, 1], [117, 1], [117, 1], [117, 1], [117, 1], [117, 2], [117, 2], [117, 2], [117, 1], [117, 1], [117, 1], [117, 1], [117, 1], [117, 1], [117, 2], [117, 3], [117, 4], [117, 3], [117, 1], [117, 4], [117, 2], [117, 2], [117, 4], [117, 4], [117, 4], [117, 4], [117, 4], [117, 5], [117, 4], [117, 4], [117, 4], [117, 4], [117, 4], [117, 4], [117, 4], [117, 4], [117, 6], [163, 3], [163, 1], [153, 1], [153, 1], [153, 1], [182, 2], [79, 4], [79, 4], [79, 4], [79, 3], [184, 1], [184, 2], [184, 2], [184, 2], [184, 2], [184, 2], [184, 2], [184, 2], [186, 3], [186, 4], [186, 0], [81, 0], [81, 2], [81, 2], [81, 2], [81, 2], [81, 2], [82, 2], [82, 3], [82, 5], [82, 0], [205, 6], [205, 7], [205, 6], [205, 7], [203, 1], [203, 3], [209, 4], [209, 5], [209, 3], [209, 3], [209, 2], [209, 3], [209, 1], [209, 3], [209, 2], [209, 3], [209, 1], [209, 1], [209, 2], [209, 3], [209, 1], [209, 1], [209, 2], [209, 3], [209, 1], [209, 2], [209, 3], [214, 1], [199, 3], [199, 1], [204, 2], [204, 2], [204, 1], [204, 1], [215, 3], [217, 1], [217, 2], [217, 3], [217, 3], [217, 2], [217, 3], [217, 4], [217, 5], [217, 1], [217, 2], [217, 3], [217, 1], [217, 2], [217, 3], [216, 1], [216, 2], [221, 1], [221, 2], [221, 2], [221, 3], [221, 2], [221, 3], [221, 2], [221, 3], [221, 2], [221, 2], [221, 2], [218, 2], [218, 2], [218, 4], [218, 0], [84, 0], [84, 2], [85, 0], [85, 4], [233, 1], [233, 3], [235, 5], [235, 4], [235, 4], [235, 1], [234, 0], [234, 2], [88, 0], [88, 2], [88, 3], [88, 2], [88, 2], [88, 3], [88, 4], [88, 3], [88, 3], [86, 0], [86, 3], [120, 1], [120, 3], [242, 2], [242, 2], [241, 1], [241, 2], [241, 3], [241, 3], [241, 4], [87, 0], [87, 3], [87, 8], [250, 0], [250, 2], [174, 3], [174, 1], [257, 3], [257, 2], [257, 3], [257, 2], [257, 3], [257, 2], [257, 1], [258, 5], [258, 3], [258, 1], [111, 5], [111, 3], [111, 3], [111, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 1], [94, 3], [94, 3], [94, 3], [94, 1], [94, 1], [94, 1], [56, 1], [70, 5], [71, 5], [267, 2], [267, 2], [265, 6], [265, 8], [265, 6], [265, 8], [279, 1], [279, 1], [279, 1], [279, 1], [279, 1], [279, 1], [279, 1], [279, 1], [259, 5], [259, 6], [259, 6], [280, 0], [280, 4], [280, 4], [280, 5], [282, 3], [283, 3], [158, 1], [158, 1], [158, 1], [158, 1], [158, 1], [158, 1], [158, 1], [158, 1], [158, 1], [158, 1], [200, 5], [200, 3], [200, 4], [200, 4], [200, 3], [200, 8], [200, 8], [200, 8], [200, 8], [200, 8], [200, 3], [151, 1], [151, 3], [196, 1], [261, 1], [261, 1], [113, 1], [113, 1], [262, 1], [202, 2], [263, 4], [266, 3], [201, 2], [201, 2], [201, 1], [201, 1], [264, 5], [264, 4], [309, 2], [309, 1], [312, 4], [310, 2], [310, 0], [260, 3], [260, 3], [260, 3], [260, 3], [260, 5], [260, 3], [260, 5], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 5], [260, 3], [260, 3], [260, 3], [260, 5], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [260, 6], [260, 6], [260, 3], [260, 3], [260, 2], [260, 2], [260, 2], [260, 2], [260, 2], [260, 3], [260, 5], [260, 6], [260, 5], [260, 6], [260, 4], [260, 5], [260, 3], [260, 4], [260, 3], [260, 4], [260, 3], [260, 3], [260, 3], [260, 3], [260, 3], [334, 1], [334, 1], [334, 4], [332, 1], [332, 1], [332, 1], [332, 1], [332, 1], [332, 1], [333, 1], [333, 1], [333, 1], [55, 6], [55, 4], [147, 1], [147, 3], [341, 3], [341, 4], [29, 5], [29, 3], [36, 5], [36, 4], [36, 7], [36, 6], [36, 5], [36, 4], [36, 5], [36, 8], [36, 7], [36, 4], [36, 6], [36, 7], [346, 1], [346, 1], [345, 0], [345, 1], [347, 3], [347, 1], [347, 1], [347, 5], [347, 3], [347, 3], [350, 1], [350, 3], [351, 1], [351, 1], [351, 1], [351, 1], [351, 1], [351, 1], [100, 1], [100, 3], [24, 9], [24, 5], [354, 1], [354, 1], [357, 0], [357, 1], [359, 2], [359, 1], [360, 1], [360, 3], [360, 3], [360, 3], [353, 0], [353, 1], [355, 0], [355, 3], [356, 3], [356, 1], [356, 2], [364, 1], [364, 3], [365, 2], [365, 2], [365, 2], [365, 2], [365, 2], [366, 0], [366, 2], [371, 4], [367, 6], [368, 9], [382, 3], [381, 0], [381, 2], [383, 4], [384, 4], [369, 6], [370, 5], [370, 5], [377, 1], [377, 1], [377, 3], [377, 3], [363, 1], [363, 3], [389, 3], [389, 2], [389, 1], [392, 6], [392, 4], [392, 1], [392, 4], [277, 2], [277, 1], [393, 1], [393, 1], [390, 0], [390, 1], [396, 2], [396, 1], [398, 3], [397, 2], [397, 5], [397, 3], [397, 6], [397, 1], [397, 2], [397, 4], [397, 2], [397, 1], [397, 2], [397, 1], [397, 1], [397, 3], [397, 5], [33, 4], [404, 3], [404, 1], [403, 0], [403, 2], [18, 6], [18, 6], [18, 6], [18, 8], [18, 6], [39, 5], [19, 4], [19, 7], [19, 6], [19, 9], [30, 3], [21, 4], [21, 6], [21, 9], [21, 6], [412, 0], [412, 2], [54, 3], [54, 2], [31, 4], [31, 5], [31, 5], [22, 8], [22, 9], [32, 3], [43, 2], [43, 4], [43, 3], [43, 5], [45, 2], [45, 4], [45, 4], [45, 6], [42, 4], [42, 6], [44, 4], [44, 6], [41, 4], [41, 6], [25, 11], [25, 8], [418, 3], [418, 3], [418, 5], [34, 4], [66, 2], [57, 2], [58, 2], [58, 2], [58, 4], [144, 4], [144, 2], [144, 2], [144, 2], [144, 2], [144, 1], [144, 2], [144, 2], [427, 1], [427, 1], [428, 1], [428, 1], [428, 1], [428, 1], [428, 1], [428, 1], [428, 1], [428, 3], [424, 3], [424, 4], [424, 2], [426, 2], [426, 3], [426, 1], [430, 3], [430, 1], [433, 3], [433, 3], [433, 3], [432, 3], [432, 1], [65, 4], [65, 3], [65, 4], [65, 5], [65, 5], [65, 6], [436, 1], [436, 1], [435, 3], [435, 2], [437, 1], [437, 1], [437, 3], [434, 1], [434, 1], [51, 2], [52, 2], [50, 2], [35, 4], [35, 3], [443, 2], [59, 3], [60, 1], [61, 1], [62, 3], [63, 2], [63, 2], [64, 2], [64, 2], [451, 1], [451, 1], [69, 2], [449, 3], [449, 1], [450, 3], [450, 1], [28, 2], [454, 1], [454, 3], [455, 3], [455, 4], [455, 5], [455, 6], [46, 3], [37, 6], [458, 1], [458, 2], [459, 2], [459, 4], [460, 2], [461, 2], [461, 2], [461, 1], [461, 1], [463, 4], [463, 6], [466, 1], [466, 3], [464, 5], [464, 7], [464, 7], [464, 9], [464, 7], [464, 9], [467, 3], [467, 6], [467, 3], [467, 6], [462, 0], [462, 2], [462, 5], [462, 4], [462, 7], [27, 6], [474, 2], [473, 0], [473, 2], [473, 2], [473, 1], [26, 8], [23, 3], [23, 4], [478, 3], [478, 1], [479, 3], [479, 7], [479, 6], [479, 3], [479, 4], [483, 1], [483, 1], [487, 2], [488, 3], [489, 2], [490, 4], [480, 4], [480, 3], [480, 2], [480, 1], [502, 2], [498, 2], [498, 2], [503, 4], [505, 6], [67, 3], [67, 2], [511, 3], [511, 1], [509, 1], [509, 4], [68, 2], [20, 2], [48, 9], [48, 8], [48, 9], [515, 0], [515, 1], [515, 1], [515, 1], [515, 2], [516, 1], [516, 1], [516, 1], [49, 3], [38, 2], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [6, 1], [11, 1], [11, 1], [80, 0], [80, 1], [83, 0], [83, 1], [90, 0], [90, 2], [91, 0], [91, 1], [96, 0], [96, 1], [97, 0], [97, 1], [101, 0], [101, 1], [108, 0], [108, 1], [121, 0], [121, 1], [125, 1], [125, 2], [126, 1], [126, 2], [127, 0], [127, 1], [155, 0], [155, 2], [157, 0], [157, 2], [159, 0], [159, 2], [160, 1], [160, 1], [161, 0], [161, 2], [165, 0], [165, 2], [167, 0], [167, 2], [176, 0], [176, 2], [177, 0], [177, 2], [178, 0], [178, 2], [188, 0], [188, 1], [197, 0], [197, 1], [210, 0], [210, 1], [211, 0], [211, 1], [219, 0], [219, 1], [220, 0], [220, 1], [252, 0], [252, 1], [254, 0], [254, 1], [255, 0], [255, 1], [256, 0], [256, 1], [268, 1], [268, 1], [771, 1], [771, 1], [293, 0], [293, 1], [306, 1], [306, 1], [342, 1], [342, 1], [376, 0], [376, 1], [380, 0], [380, 1], [387, 0], [387, 1], [388, 0], [388, 1], [399, 0], [399, 1], [400, 0], [400, 1], [402, 1], [402, 1], [416, 0], [416, 1], [417, 0], [417, 1], [470, 0], [470, 1], [471, 0], [471, 1], [472, 0], [472, 1], [476, 0], [476, 1], [481, 0], [481, 1], [482, 0], [482, 1], [484, 0], [484, 1], [485, 0], [485, 1], [486, 0], [486, 1], [491, 0], [491, 1], [492, 0], [492, 1], [493, 0], [493, 1], [494, 0], [494, 1], [495, 0], [495, 1], [496, 0], [496, 1], [497, 0], [497, 1], [499, 0], [499, 1], [500, 0], [500, 1], [501, 0], [501, 1], [504, 0], [504, 2], [506, 0], [506, 2], [507, 0], [507, 2], [508, 0], [508, 2], [517, 0], [517, 1], [518, 0], [518, 1]], performAction: function(bd, pr, oi, ar, Sa, Qn, za) {
            var En = Qn.length - 1;
            switch (Sa) {
              case 1:
                H.options.casesensitive ? this.$ = Qn[En] : this.$ = Qn[En].toLowerCase();
                break;
              case 2:
                this.$ = oe(Qn[En].substr(1, Qn[En].length - 2));
                break;
              case 3:
                this.$ = Qn[En].toLowerCase();
                break;
              case 4:
                this.$ = Qn[En];
                break;
              case 5:
                this.$ = Qn[En] ? Qn[En - 1] + " " + Qn[En] : Qn[En - 1];
                break;
              case 6:
                return new ar.Statements({ statements: Qn[En - 1] });
              case 7:
                this.$ = Qn[En - 2], Qn[En] && Qn[En - 2].push(Qn[En]);
                break;
              case 8:
              case 9:
              case 70:
              case 80:
              case 85:
              case 143:
              case 177:
              case 205:
              case 206:
              case 243:
              case 262:
              case 277:
              case 363:
              case 381:
              case 460:
              case 483:
              case 484:
              case 488:
              case 496:
              case 537:
              case 538:
              case 575:
              case 658:
              case 668:
              case 692:
              case 694:
              case 696:
              case 711:
              case 712:
              case 742:
              case 766:
                this.$ = [Qn[En]];
                break;
              case 10:
                this.$ = Qn[En], Qn[En].explain = !0;
                break;
              case 11:
                this.$ = Qn[En], Qn[En].explain = !0;
                break;
              case 12:
                this.$ = Qn[En], ar.exists && (this.$.exists = ar.exists), delete ar.exists, ar.queries && (this.$.queries = ar.queries), delete ar.queries;
                break;
              case 13:
              case 162:
              case 172:
              case 238:
              case 239:
              case 241:
              case 249:
              case 251:
              case 260:
              case 271:
              case 274:
              case 384:
              case 500:
              case 510:
              case 512:
              case 524:
              case 530:
              case 531:
              case 576:
                this.$ = void 0;
                break;
              case 68:
                this.$ = new ar.WithSelect({ withs: Qn[En - 1], select: Qn[En] });
                break;
              case 69:
              case 574:
                Qn[En - 2].push(Qn[En]), this.$ = Qn[En - 2];
                break;
              case 71:
                this.$ = { name: Qn[En - 4], select: Qn[En - 1] };
                break;
              case 72:
                ar.extend(this.$, Qn[En - 9]), ar.extend(this.$, Qn[En - 8]), ar.extend(this.$, Qn[En - 7]), ar.extend(this.$, Qn[En - 6]), ar.extend(this.$, Qn[En - 5]), ar.extend(this.$, Qn[En - 4]), ar.extend(this.$, Qn[En - 3]), ar.extend(this.$, Qn[En - 2]), ar.extend(this.$, Qn[En - 1]), ar.extend(this.$, Qn[En]), this.$ = Qn[En - 9], ar.exists && (this.$.exists = ar.exists.slice());
                break;
              case 73:
                this.$ = new ar.Search({ selectors: Qn[En - 2], from: Qn[En] }), ar.extend(this.$, Qn[En - 1]);
                break;
              case 74:
                this.$ = { pivot: { expr: Qn[En - 5], columnid: Qn[En - 3], inlist: Qn[En - 2], as: Qn[En] } };
                break;
              case 75:
                this.$ = { unpivot: { tocolumnid: Qn[En - 8], forcolumnid: Qn[En - 6], inlist: Qn[En - 3], as: Qn[En] } };
                break;
              case 76:
              case 529:
              case 558:
              case 594:
              case 628:
              case 645:
              case 646:
              case 649:
              case 671:
                this.$ = Qn[En - 1];
                break;
              case 77:
              case 78:
              case 86:
              case 147:
              case 185:
              case 248:
              case 284:
              case 292:
              case 293:
              case 294:
              case 295:
              case 296:
              case 297:
              case 298:
              case 299:
              case 300:
              case 301:
              case 302:
              case 303:
              case 304:
              case 305:
              case 308:
              case 309:
              case 325:
              case 326:
              case 327:
              case 328:
              case 329:
              case 330:
              case 383:
              case 449:
              case 450:
              case 451:
              case 452:
              case 453:
              case 454:
              case 525:
              case 551:
              case 555:
              case 557:
              case 632:
              case 633:
              case 634:
              case 635:
              case 636:
              case 637:
              case 641:
              case 643:
              case 644:
              case 653:
              case 669:
              case 670:
              case 733:
              case 748:
              case 749:
              case 751:
              case 752:
              case 758:
              case 759:
                this.$ = Qn[En];
                break;
              case 79:
              case 84:
              case 741:
              case 765:
                this.$ = Qn[En - 2], this.$.push(Qn[En]);
                break;
              case 81:
                this.$ = { expr: Qn[En] };
                break;
              case 82:
                this.$ = { expr: Qn[En - 2], as: Qn[En] };
                break;
              case 83:
                this.$ = { removecolumns: Qn[En] };
                break;
              case 87:
                this.$ = { like: Qn[En] };
                break;
              case 90:
              case 104:
                this.$ = { srchid: "PROP", args: [Qn[En]] };
                break;
              case 91:
                this.$ = { srchid: "ORDERBY", args: Qn[En - 1] };
                break;
              case 92:
                var Qo = Qn[En - 1];
                Qo || (Qo = "ASC"), this.$ = { srchid: "ORDERBY", args: [{ expression: new ar.Column({ columnid: "_" }), direction: Qo }] };
                break;
              case 93:
                this.$ = { srchid: "PARENT" };
                break;
              case 94:
                this.$ = { srchid: "APROP", args: [Qn[En]] };
                break;
              case 95:
                this.$ = { selid: "ROOT" };
                break;
              case 96:
                this.$ = { srchid: "EQ", args: [Qn[En]] };
                break;
              case 97:
                this.$ = { srchid: "LIKE", args: [Qn[En]] };
                break;
              case 98:
              case 99:
                this.$ = { selid: "WITH", args: Qn[En - 1] };
                break;
              case 100:
                this.$ = { srchid: Qn[En - 3].toUpperCase(), args: Qn[En - 1] };
                break;
              case 101:
                this.$ = { srchid: "WHERE", args: [Qn[En - 1]] };
                break;
              case 102:
                this.$ = { selid: "OF", args: [Qn[En - 1]] };
                break;
              case 103:
                this.$ = { srchid: "CLASS", args: [Qn[En - 1]] };
                break;
              case 105:
                this.$ = { srchid: "NAME", args: [Qn[En].substr(1, Qn[En].length - 2)] };
                break;
              case 106:
                this.$ = { srchid: "CHILD" };
                break;
              case 107:
                this.$ = { srchid: "VERTEX" };
                break;
              case 108:
                this.$ = { srchid: "EDGE" };
                break;
              case 109:
                this.$ = { srchid: "REF" };
                break;
              case 110:
                this.$ = { srchid: "SHARP", args: [Qn[En]] };
                break;
              case 111:
                this.$ = { srchid: "ATTR", args: typeof Qn[En] > "u" ? void 0 : [Qn[En]] };
                break;
              case 112:
                this.$ = { srchid: "ATTR" };
                break;
              case 113:
                this.$ = { srchid: "OUT" };
                break;
              case 114:
                this.$ = { srchid: "IN" };
                break;
              case 115:
                this.$ = { srchid: "OUTOUT" };
                break;
              case 116:
                this.$ = { srchid: "ININ" };
                break;
              case 117:
                this.$ = { srchid: "CONTENT" };
                break;
              case 118:
                this.$ = { srchid: "EX", args: [new ar.Json({ value: Qn[En] })] };
                break;
              case 119:
                this.$ = { srchid: "AT", args: [Qn[En]] };
                break;
              case 120:
                this.$ = { srchid: "AS", args: [Qn[En]] };
                break;
              case 121:
                this.$ = { srchid: "SET", args: Qn[En - 1] };
                break;
              case 122:
                this.$ = { selid: "TO", args: [Qn[En]] };
                break;
              case 123:
                this.$ = { srchid: "VALUE" };
                break;
              case 124:
                this.$ = { srchid: "ROW", args: Qn[En - 1] };
                break;
              case 125:
                this.$ = { srchid: "CLASS", args: [Qn[En]] };
                break;
              case 126:
                this.$ = { selid: Qn[En], args: [Qn[En - 1]] };
                break;
              case 127:
                this.$ = { selid: "NOT", args: Qn[En - 1] };
                break;
              case 128:
                this.$ = { selid: "IF", args: Qn[En - 1] };
                break;
              case 129:
                this.$ = { selid: Qn[En - 3], args: Qn[En - 1] };
                break;
              case 130:
                this.$ = { selid: "DISTINCT", args: Qn[En - 1] };
                break;
              case 131:
                this.$ = { selid: "UNION", args: Qn[En - 1] };
                break;
              case 132:
                this.$ = { selid: "UNIONALL", args: Qn[En - 1] };
                break;
              case 133:
                this.$ = { selid: "ALL", args: [Qn[En - 1]] };
                break;
              case 134:
                this.$ = { selid: "ANY", args: [Qn[En - 1]] };
                break;
              case 135:
                this.$ = { selid: "INTERSECT", args: Qn[En - 1] };
                break;
              case 136:
                this.$ = { selid: "EXCEPT", args: Qn[En - 1] };
                break;
              case 137:
                this.$ = { selid: "AND", args: Qn[En - 1] };
                break;
              case 138:
                this.$ = { selid: "OR", args: Qn[En - 1] };
                break;
              case 139:
                this.$ = { selid: "PATH", args: [Qn[En - 1]] };
                break;
              case 140:
                this.$ = { srchid: "RETURN", args: Qn[En - 1] };
                break;
              case 141:
                this.$ = { selid: "REPEAT", sels: Qn[En - 3], args: Qn[En - 1] };
                break;
              case 142:
                this.$ = Qn[En - 2], this.$.push(Qn[En]);
                break;
              case 144:
                this.$ = "PLUS";
                break;
              case 145:
                this.$ = "STAR";
                break;
              case 146:
                this.$ = "QUESTION";
                break;
              case 148:
                this.$ = new ar.Select({ columns: Qn[En], distinct: !0 }), ar.extend(this.$, Qn[En - 3]), ar.extend(this.$, Qn[En - 1]);
                break;
              case 149:
                this.$ = new ar.Select({ columns: Qn[En], distinct: !0 }), ar.extend(this.$, Qn[En - 3]), ar.extend(this.$, Qn[En - 1]);
                break;
              case 150:
                this.$ = new ar.Select({ columns: Qn[En], all: !0 }), ar.extend(this.$, Qn[En - 3]), ar.extend(this.$, Qn[En - 1]);
                break;
              case 151:
                Qn[En] ? (this.$ = new ar.Select({ columns: Qn[En] }), ar.extend(this.$, Qn[En - 2]), ar.extend(this.$, Qn[En - 1])) : this.$ = new ar.Select({ columns: [new ar.Column({ columnid: "_" })], modifier: "COLUMN" });
                break;
              case 152:
                Qn[En] == "SELECT" ? this.$ = void 0 : this.$ = { modifier: Qn[En] };
                break;
              case 153:
                this.$ = { modifier: "VALUE" };
                break;
              case 154:
                this.$ = { modifier: "ROW" };
                break;
              case 155:
                this.$ = { modifier: "COLUMN" };
                break;
              case 156:
                this.$ = { modifier: "MATRIX" };
                break;
              case 157:
                this.$ = { modifier: "TEXTSTRING" };
                break;
              case 158:
                this.$ = { modifier: "INDEX" };
                break;
              case 159:
                this.$ = { modifier: "RECORDSET" };
                break;
              case 160:
                this.$ = { top: Qn[En - 1], percent: typeof Qn[En] < "u" ? !0 : void 0 };
                break;
              case 161:
                this.$ = { top: Qn[En - 1] };
                break;
              case 163:
              case 336:
              case 532:
              case 533:
              case 734:
                this.$ = void 0;
                break;
              case 164:
              case 165:
              case 166:
              case 167:
                this.$ = { into: Qn[En] };
                break;
              case 168:
                var lo = Qn[En];
                lo = lo.substr(1, lo.length - 2);
                var Eo = lo.substr(-3).toUpperCase(), bo = lo.substr(-4).toUpperCase();
                lo[0] == "#" ? this.$ = { into: new ar.FuncValue({ funcid: "HTML", args: [new ar.StringValue({ value: lo }), new ar.Json({ value: { headers: !0 } })] }) } : Eo == "XLS" || Eo == "CSV" || Eo == "TAB" ? this.$ = { into: new ar.FuncValue({ funcid: Eo, args: [new ar.StringValue({ value: lo }), new ar.Json({ value: { headers: !0 } })] }) } : (bo == "XLSX" || bo == "JSON") && (this.$ = { into: new ar.FuncValue({ funcid: bo, args: [new ar.StringValue({ value: lo }), new ar.Json({ value: { headers: !0 } })] }) });
                break;
              case 169:
                this.$ = { from: Qn[En] };
                break;
              case 170:
                this.$ = { from: Qn[En - 1], joins: Qn[En] };
                break;
              case 171:
                this.$ = { from: Qn[En - 2], joins: Qn[En - 1] };
                break;
              case 173:
                this.$ = new ar.Apply({ select: Qn[En - 2], applymode: "CROSS", as: Qn[En] });
                break;
              case 174:
                this.$ = new ar.Apply({ select: Qn[En - 3], applymode: "CROSS", as: Qn[En] });
                break;
              case 175:
                this.$ = new ar.Apply({ select: Qn[En - 2], applymode: "OUTER", as: Qn[En] });
                break;
              case 176:
                this.$ = new ar.Apply({ select: Qn[En - 3], applymode: "OUTER", as: Qn[En] });
                break;
              case 178:
              case 244:
              case 461:
              case 539:
              case 540:
                this.$ = Qn[En - 2], Qn[En - 2].push(Qn[En]);
                break;
              case 179:
                this.$ = Qn[En - 2], this.$.as = Qn[En];
                break;
              case 180:
                this.$ = Qn[En - 3], this.$.as = Qn[En];
                break;
              case 181:
                this.$ = Qn[En - 1], this.$.as = "default";
                break;
              case 182:
                this.$ = new ar.Json({ value: Qn[En - 2] }), Qn[En - 2].as = Qn[En];
                break;
              case 183:
                this.$ = Qn[En - 1], Qn[En - 1].as = Qn[En];
                break;
              case 184:
                this.$ = Qn[En - 2], Qn[En - 2].as = Qn[En];
                break;
              case 186:
              case 647:
              case 650:
                this.$ = Qn[En - 2];
                break;
              case 187:
              case 191:
              case 195:
              case 198:
                this.$ = Qn[En - 1], Qn[En - 1].as = Qn[En];
                break;
              case 188:
              case 192:
              case 196:
              case 199:
                this.$ = Qn[En - 2], Qn[En - 2].as = Qn[En];
                break;
              case 189:
              case 190:
              case 194:
              case 197:
                this.$ = Qn[En], Qn[En].as = "default";
                break;
              case 193:
                this.$ = { inserted: !0 };
                break;
              case 200:
                var lo = Qn[En];
                lo = lo.substr(1, lo.length - 2);
                var Eo = lo.substr(-3).toUpperCase(), bo = lo.substr(-4).toUpperCase(), Ho;
                if (lo[0] == "#") Ho = new ar.FuncValue({ funcid: "HTML", args: [new ar.StringValue({ value: lo }), new ar.Json({ value: { headers: !0 } })] });
                else if (Eo == "XLS" || Eo == "CSV" || Eo == "TAB") Ho = new ar.FuncValue({ funcid: Eo, args: [new ar.StringValue({ value: lo }), new ar.Json({ value: { headers: !0 } })] });
                else if (bo == "XLSX" || bo == "JSON") Ho = new ar.FuncValue({ funcid: bo, args: [new ar.StringValue({ value: lo }), new ar.Json({ value: { headers: !0 } })] });
                else throw new Error("Unknown string in FROM clause");
                this.$ = Ho;
                break;
              case 201:
                Qn[En - 2] == "INFORMATION_SCHEMA" ? this.$ = new ar.FuncValue({ funcid: Qn[En - 2], args: [new ar.StringValue({ value: Qn[En] })] }) : this.$ = new ar.Table({ databaseid: Qn[En - 2], tableid: Qn[En] });
                break;
              case 202:
                this.$ = new ar.Table({ tableid: Qn[En] });
                break;
              case 203:
              case 204:
                this.$ = Qn[En - 1], Qn[En - 1].push(Qn[En]);
                break;
              case 207:
                this.$ = new ar.Join(Qn[En - 2]), ar.extend(this.$, Qn[En - 1]), ar.extend(this.$, Qn[En]);
                break;
              case 208:
                this.$ = { table: Qn[En] };
                break;
              case 209:
                this.$ = { table: Qn[En - 1], as: Qn[En] };
                break;
              case 210:
                this.$ = { table: Qn[En - 2], as: Qn[En] };
                break;
              case 211:
                this.$ = { json: new ar.Json({ value: Qn[En - 2], as: Qn[En] }) };
                break;
              case 212:
                this.$ = { param: Qn[En - 1], as: Qn[En] };
                break;
              case 213:
                this.$ = { param: Qn[En - 2], as: Qn[En] };
                break;
              case 214:
                this.$ = { select: Qn[En - 2], as: Qn[En] };
                break;
              case 215:
                this.$ = { select: Qn[En - 3], as: Qn[En] };
                break;
              case 216:
                this.$ = { func: Qn[En], as: "default" };
                break;
              case 217:
                this.$ = { func: Qn[En - 1], as: Qn[En] };
                break;
              case 218:
                this.$ = { func: Qn[En - 2], as: Qn[En] };
                break;
              case 219:
                this.$ = { variable: Qn[En], as: "default" };
                break;
              case 220:
                this.$ = { variable: Qn[En - 1], as: Qn[En] };
                break;
              case 221:
                this.$ = { variable: Qn[En - 2], as: Qn[En] };
                break;
              case 222:
                this.$ = { joinmode: Qn[En] };
                break;
              case 223:
                this.$ = { joinmode: Qn[En - 1], natural: !0 };
                break;
              case 224:
              case 225:
                this.$ = "INNER";
                break;
              case 226:
              case 227:
                this.$ = "LEFT";
                break;
              case 228:
              case 229:
                this.$ = "RIGHT";
                break;
              case 230:
              case 231:
                this.$ = "OUTER";
                break;
              case 232:
                this.$ = "SEMI";
                break;
              case 233:
                this.$ = "ANTI";
                break;
              case 234:
                this.$ = "CROSS";
                break;
              case 235:
                this.$ = { on: Qn[En] };
                break;
              case 236:
              case 706:
                this.$ = { using: Qn[En] };
                break;
              case 237:
              case 707:
                this.$ = { using: Qn[En - 1] };
                break;
              case 240:
                this.$ = { where: new ar.Expression({ expression: Qn[En] }) };
                break;
              case 242:
                this.$ = { group: Qn[En - 1] }, ar.extend(this.$, Qn[En]);
                break;
              case 245:
                this.$ = new ar.GroupExpression({ type: "GROUPING SETS", group: Qn[En - 1] });
                break;
              case 246:
                this.$ = new ar.GroupExpression({ type: "ROLLUP", group: Qn[En - 1] });
                break;
              case 247:
                this.$ = new ar.GroupExpression({ type: "CUBE", group: Qn[En - 1] });
                break;
              case 250:
                this.$ = { having: Qn[En] };
                break;
              case 252:
                this.$ = { union: Qn[En] };
                break;
              case 253:
                this.$ = { unionall: Qn[En] };
                break;
              case 254:
                this.$ = { except: Qn[En] };
                break;
              case 255:
                this.$ = { intersect: Qn[En] };
                break;
              case 256:
                this.$ = { union: Qn[En], corresponding: !0 };
                break;
              case 257:
                this.$ = { unionall: Qn[En], corresponding: !0 };
                break;
              case 258:
                this.$ = { except: Qn[En], corresponding: !0 };
                break;
              case 259:
                this.$ = { intersect: Qn[En], corresponding: !0 };
                break;
              case 261:
                this.$ = { order: Qn[En] };
                break;
              case 263:
                this.$ = Qn[En - 2], Qn[En - 2].push(Qn[En]);
                break;
              case 264:
                this.$ = { nullsOrder: "FIRST" };
                break;
              case 265:
                this.$ = { nullsOrder: "LAST" };
                break;
              case 266:
                this.$ = new ar.Expression({ expression: Qn[En], direction: "ASC" });
                break;
              case 267:
                this.$ = new ar.Expression({ expression: Qn[En - 1], direction: Qn[En].toUpperCase() });
                break;
              case 268:
                this.$ = new ar.Expression({ expression: Qn[En - 2], direction: Qn[En - 1].toUpperCase() }), ar.extend(this.$, Qn[En]);
                break;
              case 269:
                this.$ = new ar.Expression({ expression: Qn[En - 2], direction: "ASC", nocase: !0 });
                break;
              case 270:
                this.$ = new ar.Expression({ expression: Qn[En - 3], direction: Qn[En].toUpperCase(), nocase: !0 });
                break;
              case 272:
                this.$ = { limit: Qn[En - 1] }, ar.extend(this.$, Qn[En]);
                break;
              case 273:
                this.$ = { limit: Qn[En - 2], offset: Qn[En - 6] };
                break;
              case 275:
                this.$ = { offset: Qn[En] };
                break;
              case 276:
              case 518:
              case 542:
              case 657:
              case 667:
              case 691:
              case 693:
              case 697:
                Qn[En - 2].push(Qn[En]), this.$ = Qn[En - 2];
                break;
              case 278:
              case 280:
              case 282:
                Qn[En - 2].as = Qn[En], this.$ = Qn[En - 2];
                break;
              case 279:
              case 281:
              case 283:
                Qn[En - 1].as = Qn[En], this.$ = Qn[En - 1];
                break;
              case 285:
                this.$ = new ar.Column({ columid: Qn[En], tableid: Qn[En - 2], databaseid: Qn[En - 4] });
                break;
              case 286:
                this.$ = new ar.Column({ columnid: Qn[En], tableid: Qn[En - 2] });
                break;
              case 287:
                this.$ = new ar.Column({ columnid: Qn[En] });
                break;
              case 288:
                this.$ = new ar.Column({ columnid: Qn[En], tableid: Qn[En - 2], databaseid: Qn[En - 4] });
                break;
              case 289:
              case 290:
                this.$ = new ar.Column({ columnid: Qn[En], tableid: Qn[En - 2] });
                break;
              case 291:
                this.$ = new ar.Column({ columnid: Qn[En] });
                break;
              case 306:
                this.$ = new ar.DomainValueValue();
                break;
              case 307:
                this.$ = new ar.Json({ value: Qn[En] });
                break;
              case 310:
              case 311:
              case 312:
                ar.queries || (ar.queries = []), ar.queries.push(Qn[En - 1]), Qn[En - 1].queriesidx = ar.queries.length, this.$ = Qn[En - 1];
                break;
              case 313:
                this.$ = Qn[En];
                break;
              case 314:
                this.$ = new ar.FuncValue({ funcid: "CURRENT_TIMESTAMP" });
                break;
              case 315:
                this.$ = new ar.FuncValue({ funcid: "CURRENT_DATE" });
                break;
              case 316:
                this.$ = new ar.JavaScript({ value: Qn[En].substr(2, Qn[En].length - 4) });
                break;
              case 317:
                this.$ = new ar.JavaScript({ value: 'alasql.fn["' + Qn[En - 2] + '"] = ' + Qn[En].substr(2, Qn[En].length - 4) });
                break;
              case 318:
                this.$ = new ar.JavaScript({ value: 'alasql.aggr["' + Qn[En - 2] + '"] = ' + Qn[En].substr(2, Qn[En].length - 4) });
                break;
              case 319:
                this.$ = new ar.FuncValue({ funcid: Qn[En], newid: !0 });
                break;
              case 320:
                this.$ = Qn[En], ar.extend(this.$, { newid: !0 });
                break;
              case 321:
                this.$ = new ar.Convert({ expression: Qn[En - 3] }), ar.extend(this.$, Qn[En - 1]);
                break;
              case 322:
                this.$ = new ar.Convert({ expression: Qn[En - 5], style: Qn[En - 1] }), ar.extend(this.$, Qn[En - 3]);
                break;
              case 323:
                this.$ = new ar.Convert({ expression: Qn[En - 1] }), ar.extend(this.$, Qn[En - 3]);
                break;
              case 324:
                this.$ = new ar.Convert({ expression: Qn[En - 3], style: Qn[En - 1] }), ar.extend(this.$, Qn[En - 5]);
                break;
              case 331:
                this.$ = new ar.FuncValue({ funcid: "CURRENT_TIMESTAMP" });
                break;
              case 332:
                this.$ = new ar.FuncValue({ funcid: "CURRENT_DATE" });
                break;
              case 333:
                Qn[En - 2].length > 1 && (Qn[En - 4].toUpperCase() == "MAX" || Qn[En - 4].toUpperCase() == "MIN") ? this.$ = new ar.FuncValue({ funcid: Qn[En - 4], args: Qn[En - 2] }) : this.$ = new ar.AggrValue({ aggregatorid: Qn[En - 4].toUpperCase(), expression: Qn[En - 2].pop(), over: Qn[En] });
                break;
              case 334:
                this.$ = new ar.AggrValue({ aggregatorid: Qn[En - 5].toUpperCase(), expression: Qn[En - 2], distinct: !0, over: Qn[En] });
                break;
              case 335:
                this.$ = new ar.AggrValue({ aggregatorid: Qn[En - 5].toUpperCase(), expression: Qn[En - 2], over: Qn[En] });
                break;
              case 337:
              case 338:
                this.$ = new ar.Over(), ar.extend(this.$, Qn[En - 1]);
                break;
              case 339:
                this.$ = new ar.Over(), ar.extend(this.$, Qn[En - 2]), ar.extend(this.$, Qn[En - 1]);
                break;
              case 340:
                this.$ = { partition: Qn[En] };
                break;
              case 341:
                this.$ = { order: Qn[En] };
                break;
              case 342:
                this.$ = "SUM";
                break;
              case 343:
                this.$ = "TOTAL";
                break;
              case 344:
                this.$ = "COUNT";
                break;
              case 345:
                this.$ = "MIN";
                break;
              case 346:
              case 553:
                this.$ = "MAX";
                break;
              case 347:
                this.$ = "AVG";
                break;
              case 348:
                this.$ = "FIRST";
                break;
              case 349:
                this.$ = "LAST";
                break;
              case 350:
                this.$ = "AGGR";
                break;
              case 351:
                this.$ = "ARRAY";
                break;
              case 352:
                var Co = Qn[En - 4], Ao = Qn[En - 1];
                Ao.length > 1 && (Co.toUpperCase() == "MIN" || Co.toUpperCase() == "MAX") ? this.$ = new ar.FuncValue({ funcid: Co, args: Ao }) : H.aggr[Qn[En - 4]] ? this.$ = new ar.AggrValue({ aggregatorid: "REDUCE", funcid: Co, expression: Ao.pop(), distinct: Qn[En - 2] == "DISTINCT" }) : this.$ = new ar.FuncValue({ funcid: Co, args: Ao });
                break;
              case 353:
              case 356:
                this.$ = new ar.FuncValue({ funcid: Qn[En - 2] });
                break;
              case 354:
                this.$ = new ar.FuncValue({ funcid: "IIF", args: Qn[En - 1] });
                break;
              case 355:
                this.$ = new ar.FuncValue({ funcid: "REPLACE", args: Qn[En - 1] });
                break;
              case 357:
                this.$ = new ar.FuncValue({ funcid: "DATEADD", args: [new ar.StringValue({ value: Qn[En - 5] }), Qn[En - 3], Qn[En - 1]] });
                break;
              case 358:
                this.$ = new ar.FuncValue({ funcid: "DATEADD", args: [Qn[En - 5], Qn[En - 3], Qn[En - 1]] });
                break;
              case 359:
                this.$ = new ar.FuncValue({ funcid: "DATEDIFF", args: [new ar.StringValue({ value: Qn[En - 5] }), Qn[En - 3], Qn[En - 1]] });
                break;
              case 360:
                this.$ = new ar.FuncValue({ funcid: "DATEDIFF", args: [Qn[En - 5], Qn[En - 3], Qn[En - 1]] });
                break;
              case 361:
                this.$ = new ar.FuncValue({ funcid: "TIMESTAMPDIFF", args: [new ar.StringValue({ value: Qn[En - 5] }), Qn[En - 3], Qn[En - 1]] });
                break;
              case 362:
                this.$ = new ar.FuncValue({ funcid: "INTERVAL", args: [Qn[En - 1], new ar.StringValue({ value: Qn[En].toLowerCase() })] });
                break;
              case 364:
                Qn[En - 2].push(Qn[En]), this.$ = Qn[En - 2];
                break;
              case 365:
                this.$ = new ar.NumValue({ value: +Qn[En] });
                break;
              case 366:
                this.$ = new ar.LogicValue({ value: !0 });
                break;
              case 367:
                this.$ = new ar.LogicValue({ value: !1 });
                break;
              case 368:
                this.$ = new ar.StringValue({ value: Qn[En].substr(1, Qn[En].length - 2).replace(/(\\\')/g, "'").replace(/(\'\')/g, "'") });
                break;
              case 369:
                this.$ = new ar.StringValue({ value: Qn[En].substr(2, Qn[En].length - 3).replace(/(\\\')/g, "'").replace(/(\'\')/g, "'") });
                break;
              case 370:
                this.$ = new ar.NullValue({ value: void 0 });
                break;
              case 371:
                this.$ = new ar.VarValue({ variable: Qn[En] });
                break;
              case 372:
                ar.exists || (ar.exists = []), this.$ = new ar.ExistsValue({ value: Qn[En - 1], existsidx: ar.exists.length }), ar.exists.push(Qn[En - 1]);
                break;
              case 373:
                this.$ = new ar.ArrayValue({ value: Qn[En - 1] });
                break;
              case 374:
              case 375:
                this.$ = new ar.ParamValue({ param: Qn[En] });
                break;
              case 376:
                typeof ar.question > "u" && (ar.question = 0), this.$ = new ar.ParamValue({ param: ar.question++ });
                break;
              case 377:
                typeof ar.question > "u" && (ar.question = 0), this.$ = new ar.ParamValue({ param: ar.question++, array: !0 });
                break;
              case 378:
                this.$ = new ar.CaseValue({ expression: Qn[En - 3], whens: Qn[En - 2], elses: Qn[En - 1] });
                break;
              case 379:
                this.$ = new ar.CaseValue({ whens: Qn[En - 2], elses: Qn[En - 1] });
                break;
              case 380:
              case 709:
              case 710:
                this.$ = Qn[En - 1], this.$.push(Qn[En]);
                break;
              case 382:
                this.$ = { when: Qn[En - 2], then: Qn[En] };
                break;
              case 385:
              case 386:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "REGEXP", right: Qn[En] });
                break;
              case 387:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "GLOB", right: Qn[En] });
                break;
              case 388:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "LIKE", right: Qn[En] });
                break;
              case 389:
                this.$ = new ar.Op({ left: Qn[En - 4], op: "LIKE", right: Qn[En - 2], escape: Qn[En] });
                break;
              case 390:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "NOT LIKE", right: Qn[En] });
                break;
              case 391:
                this.$ = new ar.Op({ left: Qn[En - 4], op: "NOT LIKE", right: Qn[En - 2], escape: Qn[En] });
                break;
              case 392:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "||", right: Qn[En] });
                break;
              case 393:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "+", right: Qn[En] });
                break;
              case 394:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "-", right: Qn[En] });
                break;
              case 395:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "*", right: Qn[En] });
                break;
              case 396:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "/", right: Qn[En] });
                break;
              case 397:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "%", right: Qn[En] });
                break;
              case 398:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "^", right: Qn[En] });
                break;
              case 399:
                this.$ = new ar.Op({ left: Qn[En - 2], op: ">>", right: Qn[En] });
                break;
              case 400:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "<<", right: Qn[En] });
                break;
              case 401:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "&", right: Qn[En] });
                break;
              case 402:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "|", right: Qn[En] });
                break;
              case 403:
              case 404:
              case 406:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "->", right: Qn[En] });
                break;
              case 405:
                this.$ = new ar.Op({ left: Qn[En - 4], op: "->", right: Qn[En - 1] });
                break;
              case 407:
              case 408:
              case 410:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "!", right: Qn[En] });
                break;
              case 409:
                this.$ = new ar.Op({ left: Qn[En - 4], op: "!", right: Qn[En - 1] });
                break;
              case 411:
                this.$ = new ar.Op({ left: Qn[En - 2], op: ">", right: Qn[En] });
                break;
              case 412:
                this.$ = new ar.Op({ left: Qn[En - 2], op: ">=", right: Qn[En] });
                break;
              case 413:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "<", right: Qn[En] });
                break;
              case 414:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "<=", right: Qn[En] });
                break;
              case 415:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "=", right: Qn[En] });
                break;
              case 416:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "==", right: Qn[En] });
                break;
              case 417:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "===", right: Qn[En] });
                break;
              case 418:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "!=", right: Qn[En] });
                break;
              case 419:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "!==", right: Qn[En] });
                break;
              case 420:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "!===", right: Qn[En] });
                break;
              case 421:
                ar.queries || (ar.queries = []), this.$ = new ar.Op({ left: Qn[En - 5], op: Qn[En - 4], allsome: Qn[En - 3], right: Qn[En - 1], queriesidx: ar.queries.length }), ar.queries.push(Qn[En - 1]);
                break;
              case 422:
                this.$ = new ar.Op({ left: Qn[En - 5], op: Qn[En - 4], allsome: Qn[En - 3], right: Qn[En - 1] });
                break;
              case 423:
                Qn[En - 2].op == "BETWEEN1" ? Qn[En - 2].left.op == "AND" ? this.$ = new ar.Op({ left: Qn[En - 2].left.left, op: "AND", right: new ar.Op({ left: Qn[En - 2].left.right, op: "BETWEEN", right1: Qn[En - 2].right, right2: Qn[En] }) }) : this.$ = new ar.Op({ left: Qn[En - 2].left, op: "BETWEEN", right1: Qn[En - 2].right, right2: Qn[En] }) : Qn[En - 2].op == "NOT BETWEEN1" ? Qn[En - 2].left.op == "AND" ? this.$ = new ar.Op({ left: Qn[En - 2].left.left, op: "AND", right: new ar.Op({ left: Qn[En - 2].left.right, op: "NOT BETWEEN", right1: Qn[En - 2].right, right2: Qn[En] }) }) : this.$ = new ar.Op({ left: Qn[En - 2].left, op: "NOT BETWEEN", right1: Qn[En - 2].right, right2: Qn[En] }) : this.$ = new ar.Op({ left: Qn[En - 2], op: "AND", right: Qn[En] });
                break;
              case 424:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "OR", right: Qn[En] });
                break;
              case 425:
                this.$ = new ar.UniOp({ op: "NOT", right: Qn[En] });
                break;
              case 426:
                this.$ = new ar.UniOp({ op: "-", right: Qn[En] });
                break;
              case 427:
                this.$ = new ar.UniOp({ op: "+", right: Qn[En] });
                break;
              case 428:
                this.$ = new ar.UniOp({ op: "~", right: Qn[En] });
                break;
              case 429:
                this.$ = new ar.UniOp({ op: "#", right: Qn[En] });
                break;
              case 430:
                this.$ = new ar.UniOp({ right: Qn[En - 1] });
                break;
              case 431:
                ar.queries || (ar.queries = []), this.$ = new ar.Op({ left: Qn[En - 4], op: "IN", right: Qn[En - 1], queriesidx: ar.queries.length }), ar.queries.push(Qn[En - 1]);
                break;
              case 432:
                ar.queries || (ar.queries = []), this.$ = new ar.Op({ left: Qn[En - 5], op: "NOT IN", right: Qn[En - 1], queriesidx: ar.queries.length }), ar.queries.push(Qn[En - 1]);
                break;
              case 433:
                this.$ = new ar.Op({ left: Qn[En - 4], op: "IN", right: Qn[En - 1] });
                break;
              case 434:
                this.$ = new ar.Op({ left: Qn[En - 5], op: "NOT IN", right: Qn[En - 1] });
                break;
              case 435:
                this.$ = new ar.Op({ left: Qn[En - 3], op: "IN", right: [] });
                break;
              case 436:
                this.$ = new ar.Op({ left: Qn[En - 4], op: "NOT IN", right: [] });
                break;
              case 437:
              case 439:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "IN", right: Qn[En] });
                break;
              case 438:
              case 440:
                this.$ = new ar.Op({ left: Qn[En - 3], op: "NOT IN", right: Qn[En] });
                break;
              case 441:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "BETWEEN1", right: Qn[En] });
                break;
              case 442:
                this.$ = new ar.Op({ left: Qn[En - 2], op: "NOT BETWEEN1", right: Qn[En] });
                break;
              case 443:
                this.$ = new ar.Op({ op: "IS", left: Qn[En - 2], right: Qn[En] });
                break;
              case 444:
                this.$ = new ar.Op({ op: "IS", left: Qn[En - 2], right: new ar.UniOp({ op: "NOT", right: new ar.NullValue({ value: void 0 }) }) });
                break;
              case 445:
                this.$ = new ar.Convert({ expression: Qn[En - 2] }), ar.extend(this.$, Qn[En]);
                break;
              case 446:
              case 447:
                this.$ = Qn[En];
                break;
              case 448:
                this.$ = Qn[En - 1];
                break;
              case 455:
                this.$ = "ALL";
                break;
              case 456:
                this.$ = "SOME";
                break;
              case 457:
                this.$ = "ANY";
                break;
              case 458:
                this.$ = new ar.Update({ table: Qn[En - 4], columns: Qn[En - 2], where: Qn[En] });
                break;
              case 459:
                this.$ = new ar.Update({ table: Qn[En - 2], columns: Qn[En] });
                break;
              case 462:
                this.$ = new ar.SetColumn({ column: Qn[En - 2], expression: Qn[En] });
                break;
              case 463:
                this.$ = new ar.SetColumn({ variable: Qn[En - 2], expression: Qn[En], method: Qn[En - 3] });
                break;
              case 464:
                this.$ = new ar.Delete({ table: Qn[En - 2], where: Qn[En] });
                break;
              case 465:
                this.$ = new ar.Delete({ table: Qn[En] });
                break;
              case 466:
                this.$ = new ar.Insert({ into: Qn[En - 2], values: Qn[En] });
                break;
              case 467:
                this.$ = new ar.Insert({ into: Qn[En - 1], values: Qn[En] });
                break;
              case 468:
              case 470:
                this.$ = new ar.Insert({ into: Qn[En - 2], values: Qn[En], orreplace: !0 });
                break;
              case 469:
              case 471:
                this.$ = new ar.Insert({ into: Qn[En - 1], values: Qn[En], orreplace: !0 });
                break;
              case 472:
                this.$ = new ar.Insert({ into: Qn[En - 2], default: !0 });
                break;
              case 473:
                this.$ = new ar.Insert({ into: Qn[En - 5], columns: Qn[En - 3], values: Qn[En] });
                break;
              case 474:
                this.$ = new ar.Insert({ into: Qn[En - 4], columns: Qn[En - 2], values: Qn[En] });
                break;
              case 475:
                this.$ = new ar.Insert({ into: Qn[En - 1], select: Qn[En] });
                break;
              case 476:
                this.$ = new ar.Insert({ into: Qn[En - 1], select: Qn[En], orreplace: !0 });
                break;
              case 477:
                this.$ = new ar.Insert({ into: Qn[En - 4], columns: Qn[En - 2], select: Qn[En] });
                break;
              case 482:
                this.$ = [Qn[En - 1]];
                break;
              case 485:
                this.$ = Qn[En - 4], Qn[En - 4].push(Qn[En - 1]);
                break;
              case 486:
              case 487:
              case 489:
              case 497:
                this.$ = Qn[En - 2], Qn[En - 2].push(Qn[En]);
                break;
              case 498:
                this.$ = new ar.CreateTable({ table: Qn[En - 4] }), ar.extend(this.$, Qn[En - 7]), ar.extend(this.$, Qn[En - 6]), ar.extend(this.$, Qn[En - 5]), ar.extend(this.$, Qn[En - 2]), ar.extend(this.$, Qn[En]);
                break;
              case 499:
                this.$ = new ar.CreateTable({ table: Qn[En] }), ar.extend(this.$, Qn[En - 3]), ar.extend(this.$, Qn[En - 2]), ar.extend(this.$, Qn[En - 1]);
                break;
              case 501:
                this.$ = { class: !0 };
                break;
              case 511:
                this.$ = { temporary: !0 };
                break;
              case 513:
                this.$ = { ifnotexists: !0 };
                break;
              case 514:
                this.$ = { columns: Qn[En - 2], constraints: Qn[En] };
                break;
              case 515:
                this.$ = { columns: Qn[En] };
                break;
              case 516:
                this.$ = { as: Qn[En] };
                break;
              case 517:
              case 541:
                this.$ = [Qn[En]];
                break;
              case 519:
              case 520:
              case 521:
              case 522:
              case 523:
                Qn[En].constraintid = Qn[En - 1], this.$ = Qn[En];
                break;
              case 526:
                this.$ = { type: "CHECK", expression: Qn[En - 1] };
                break;
              case 527:
                this.$ = { type: "PRIMARY KEY", columns: Qn[En - 1], clustered: (Qn[En - 3] + "").toUpperCase() };
                break;
              case 528:
                this.$ = { type: "FOREIGN KEY", columns: Qn[En - 5], fktable: Qn[En - 2], fkcolumns: Qn[En - 1] };
                break;
              case 534:
                this.$ = { type: "UNIQUE", columns: Qn[En - 1], clustered: (Qn[En - 3] + "").toUpperCase() };
                break;
              case 543:
                this.$ = new ar.ColumnDef({ columnid: Qn[En - 2] }), ar.extend(this.$, Qn[En - 1]), ar.extend(this.$, Qn[En]);
                break;
              case 544:
                this.$ = new ar.ColumnDef({ columnid: Qn[En - 1] }), ar.extend(this.$, Qn[En]);
                break;
              case 545:
                this.$ = new ar.ColumnDef({ columnid: Qn[En], dbtypeid: "" });
                break;
              case 546:
                this.$ = { dbtypeid: Qn[En - 5], dbsize: Qn[En - 3], dbprecision: +Qn[En - 1] };
                break;
              case 547:
                this.$ = { dbtypeid: Qn[En - 3], dbsize: Qn[En - 1] };
                break;
              case 548:
                this.$ = { dbtypeid: Qn[En] };
                break;
              case 549:
                this.$ = { dbtypeid: "ENUM", enumvalues: Qn[En - 1] };
                break;
              case 550:
                this.$ = Qn[En - 1], Qn[En - 1].dbtypeid += "[" + Qn[En] + "]";
                break;
              case 552:
              case 760:
                this.$ = +Qn[En];
                break;
              case 554:
                this.$ = void 0;
                break;
              case 556:
                ar.extend(Qn[En - 1], Qn[En]), this.$ = Qn[En - 1];
                break;
              case 559:
                this.$ = { primarykey: !0 };
                break;
              case 560:
              case 561:
                this.$ = { foreignkey: { table: Qn[En - 1], columnid: Qn[En] } };
                break;
              case 562:
                this.$ = { identity: { value: Qn[En - 3], step: Qn[En - 1] } };
                break;
              case 563:
                this.$ = { identity: { value: 1, step: 1 } };
                break;
              case 564:
              case 566:
                this.$ = { default: Qn[En] };
                break;
              case 565:
                this.$ = { default: Qn[En - 1] };
                break;
              case 567:
                this.$ = { null: !0 };
                break;
              case 568:
                this.$ = { notnull: !0 };
                break;
              case 569:
                this.$ = { check: Qn[En] };
                break;
              case 570:
                this.$ = { unique: !0 };
                break;
              case 571:
                this.$ = { onupdate: Qn[En] };
                break;
              case 572:
                this.$ = { onupdate: Qn[En - 1] };
                break;
              case 573:
                this.$ = new ar.DropTable({ tables: Qn[En], type: Qn[En - 2] }), ar.extend(this.$, Qn[En - 1]);
                break;
              case 577:
                this.$ = { ifexists: !0 };
                break;
              case 578:
                this.$ = new ar.AlterTable({ table: Qn[En - 3], renameto: Qn[En] });
                break;
              case 579:
                this.$ = new ar.AlterTable({ table: Qn[En - 3], addcolumn: Qn[En] });
                break;
              case 580:
                this.$ = new ar.AlterTable({ table: Qn[En - 3], modifycolumn: Qn[En] });
                break;
              case 581:
                this.$ = new ar.AlterTable({ table: Qn[En - 5], renamecolumn: Qn[En - 2], to: Qn[En] });
                break;
              case 582:
                this.$ = new ar.AlterTable({ table: Qn[En - 3], dropcolumn: Qn[En] });
                break;
              case 583:
                this.$ = new ar.AlterTable({ table: Qn[En - 2], renameto: Qn[En] });
                break;
              case 584:
                this.$ = new ar.AttachDatabase({ databaseid: Qn[En], engineid: Qn[En - 2].toUpperCase() });
                break;
              case 585:
                this.$ = new ar.AttachDatabase({ databaseid: Qn[En - 3], engineid: Qn[En - 5].toUpperCase(), args: Qn[En - 1] });
                break;
              case 586:
                this.$ = new ar.AttachDatabase({ databaseid: Qn[En - 2], engineid: Qn[En - 4].toUpperCase(), as: Qn[En] });
                break;
              case 587:
                this.$ = new ar.AttachDatabase({ databaseid: Qn[En - 5], engineid: Qn[En - 7].toUpperCase(), as: Qn[En], args: Qn[En - 3] });
                break;
              case 588:
                this.$ = new ar.DetachDatabase({ databaseid: Qn[En] });
                break;
              case 589:
                this.$ = new ar.CreateDatabase({ databaseid: Qn[En] }), ar.extend(this.$, Qn[En]);
                break;
              case 590:
                this.$ = new ar.CreateDatabase({ engineid: Qn[En - 4].toUpperCase(), databaseid: Qn[En - 1], as: Qn[En] }), ar.extend(this.$, Qn[En - 2]);
                break;
              case 591:
                this.$ = new ar.CreateDatabase({ engineid: Qn[En - 7].toUpperCase(), databaseid: Qn[En - 4], args: Qn[En - 2], as: Qn[En] }), ar.extend(this.$, Qn[En - 5]);
                break;
              case 592:
                this.$ = new ar.CreateDatabase({ engineid: Qn[En - 4].toUpperCase(), as: Qn[En], args: [Qn[En - 1]] }), ar.extend(this.$, Qn[En - 2]);
                break;
              case 593:
                this.$ = void 0;
                break;
              case 595:
              case 596:
                this.$ = new ar.UseDatabase({ databaseid: Qn[En] });
                break;
              case 597:
                this.$ = new ar.DropDatabase({ databaseid: Qn[En] }), ar.extend(this.$, Qn[En - 1]);
                break;
              case 598:
              case 599:
                this.$ = new ar.DropDatabase({ databaseid: Qn[En], engineid: Qn[En - 3].toUpperCase() }), ar.extend(this.$, Qn[En - 1]);
                break;
              case 600:
                this.$ = new ar.CreateIndex({ indexid: Qn[En - 5], table: Qn[En - 3], columns: Qn[En - 1] });
                break;
              case 601:
                this.$ = new ar.CreateIndex({ indexid: Qn[En - 5], table: Qn[En - 3], columns: Qn[En - 1], unique: !0 });
                break;
              case 602:
                this.$ = new ar.DropIndex({ indexid: Qn[En] });
                break;
              case 603:
                this.$ = new ar.ShowDatabases();
                break;
              case 604:
                this.$ = new ar.ShowDatabases({ like: Qn[En] });
                break;
              case 605:
                this.$ = new ar.ShowDatabases({ engineid: Qn[En - 1].toUpperCase() });
                break;
              case 606:
                this.$ = new ar.ShowDatabases({ engineid: Qn[En - 3].toUpperCase(), like: Qn[En] });
                break;
              case 607:
                this.$ = new ar.ShowTables();
                break;
              case 608:
                this.$ = new ar.ShowTables({ like: Qn[En] });
                break;
              case 609:
                this.$ = new ar.ShowTables({ databaseid: Qn[En] });
                break;
              case 610:
                this.$ = new ar.ShowTables({ like: Qn[En], databaseid: Qn[En - 2] });
                break;
              case 611:
                this.$ = new ar.ShowColumns({ table: Qn[En] });
                break;
              case 612:
                this.$ = new ar.ShowColumns({ table: Qn[En - 2], databaseid: Qn[En] });
                break;
              case 613:
                this.$ = new ar.ShowIndex({ table: Qn[En] });
                break;
              case 614:
                this.$ = new ar.ShowIndex({ table: Qn[En - 2], databaseid: Qn[En] });
                break;
              case 615:
                this.$ = new ar.ShowCreateTable({ table: Qn[En] });
                break;
              case 616:
                this.$ = new ar.ShowCreateTable({ table: Qn[En - 2], databaseid: Qn[En] });
                break;
              case 617:
                this.$ = new ar.CreateTable({ table: Qn[En - 6], view: !0, select: Qn[En - 1], viewcolumns: Qn[En - 4] }), ar.extend(this.$, Qn[En - 9]), ar.extend(this.$, Qn[En - 7]);
                break;
              case 618:
                this.$ = new ar.CreateTable({ table: Qn[En - 3], view: !0, select: Qn[En - 1] }), ar.extend(this.$, Qn[En - 6]), ar.extend(this.$, Qn[En - 4]);
                break;
              case 622:
                this.$ = new ar.DropTable({ tables: Qn[En], view: !0 }), ar.extend(this.$, Qn[En - 1]);
                break;
              case 623:
              case 770:
                this.$ = new ar.ExpressionStatement({ expression: Qn[En] });
                break;
              case 624:
                this.$ = new ar.Source({ url: Qn[En].value });
                break;
              case 625:
                this.$ = new ar.Assert({ value: Qn[En] });
                break;
              case 626:
                this.$ = new ar.Assert({ value: Qn[En].value });
                break;
              case 627:
                this.$ = new ar.Assert({ value: Qn[En], message: Qn[En - 2] });
                break;
              case 629:
              case 640:
              case 642:
                this.$ = Qn[En].value;
                break;
              case 630:
              case 638:
                this.$ = +Qn[En].value;
                break;
              case 631:
                this.$ = !!Qn[En].value;
                break;
              case 639:
                this.$ = "" + Qn[En].value;
                break;
              case 648:
                this.$ = {};
                break;
              case 651:
                this.$ = [];
                break;
              case 652:
                ar.extend(Qn[En - 2], Qn[En]), this.$ = Qn[En - 2];
                break;
              case 654:
                this.$ = {}, this.$[Qn[En - 2].substr(1, Qn[En - 2].length - 2)] = Qn[En];
                break;
              case 655:
              case 656:
                this.$ = {}, this.$[Qn[En - 2]] = Qn[En];
                break;
              case 659:
                this.$ = new ar.SetVariable({ variable: Qn[En - 2].toLowerCase(), value: Qn[En] });
                break;
              case 660:
                this.$ = new ar.SetVariable({ variable: Qn[En - 1].toLowerCase(), value: Qn[En] });
                break;
              case 661:
                this.$ = new ar.SetVariable({ variable: Qn[En - 2], expression: Qn[En] });
                break;
              case 662:
                this.$ = new ar.SetVariable({ variable: Qn[En - 3], props: Qn[En - 2], expression: Qn[En] });
                break;
              case 663:
                this.$ = new ar.SetVariable({ variable: Qn[En - 2], expression: Qn[En], method: Qn[En - 3] });
                break;
              case 664:
                this.$ = new ar.SetVariable({ variable: Qn[En - 3], props: Qn[En - 2], expression: Qn[En], method: Qn[En - 4] });
                break;
              case 665:
                this.$ = "@";
                break;
              case 666:
                this.$ = "$";
                break;
              case 672:
                this.$ = !0;
                break;
              case 673:
                this.$ = !1;
                break;
              case 674:
                this.$ = new ar.CommitTransaction();
                break;
              case 675:
                this.$ = new ar.RollbackTransaction();
                break;
              case 676:
                this.$ = new ar.BeginTransaction();
                break;
              case 677:
                this.$ = new ar.If({ expression: Qn[En - 2], thenstat: Qn[En - 1], elsestat: Qn[En] }), Qn[En - 1].exists && (this.$.exists = Qn[En - 1].exists), Qn[En - 1].queries && (this.$.queries = Qn[En - 1].queries);
                break;
              case 678:
                this.$ = new ar.If({ expression: Qn[En - 1], thenstat: Qn[En] }), Qn[En].exists && (this.$.exists = Qn[En].exists), Qn[En].queries && (this.$.queries = Qn[En].queries);
                break;
              case 679:
                this.$ = Qn[En];
                break;
              case 680:
                this.$ = new ar.While({ expression: Qn[En - 1], loopstat: Qn[En] }), Qn[En].exists && (this.$.exists = Qn[En].exists), Qn[En].queries && (this.$.queries = Qn[En].queries);
                break;
              case 681:
                this.$ = new ar.Continue();
                break;
              case 682:
                this.$ = new ar.Break();
                break;
              case 683:
                this.$ = new ar.BeginEnd({ statements: Qn[En - 1] });
                break;
              case 684:
                this.$ = new ar.Print({ exprs: Qn[En] });
                break;
              case 685:
                this.$ = new ar.Print({ select: Qn[En] });
                break;
              case 686:
                this.$ = new ar.Require({ paths: Qn[En] });
                break;
              case 687:
                this.$ = new ar.Require({ plugins: Qn[En] });
                break;
              case 688:
              case 689:
                this.$ = Qn[En].toUpperCase();
                break;
              case 690:
                this.$ = new ar.Echo({ expr: Qn[En] });
                break;
              case 695:
                this.$ = new ar.Declare({ declares: Qn[En] });
                break;
              case 698:
                this.$ = { variable: Qn[En - 1] }, ar.extend(this.$, Qn[En]);
                break;
              case 699:
                this.$ = { variable: Qn[En - 2] }, ar.extend(this.$, Qn[En]);
                break;
              case 700:
                this.$ = { variable: Qn[En - 3], expression: Qn[En] }, ar.extend(this.$, Qn[En - 2]);
                break;
              case 701:
                this.$ = { variable: Qn[En - 4], expression: Qn[En] }, ar.extend(this.$, Qn[En - 2]);
                break;
              case 702:
                this.$ = new ar.TruncateTable({ table: Qn[En] });
                break;
              case 703:
                this.$ = new ar.Merge(), ar.extend(this.$, Qn[En - 4]), ar.extend(this.$, Qn[En - 3]), ar.extend(this.$, Qn[En - 2]), ar.extend(this.$, { matches: Qn[En - 1] }), ar.extend(this.$, Qn[En]);
                break;
              case 704:
              case 705:
                this.$ = { into: Qn[En] };
                break;
              case 708:
                this.$ = { on: Qn[En] };
                break;
              case 713:
                this.$ = { matched: !0, action: Qn[En] };
                break;
              case 714:
                this.$ = { matched: !0, expr: Qn[En - 2], action: Qn[En] };
                break;
              case 715:
                this.$ = { delete: !0 };
                break;
              case 716:
                this.$ = { update: Qn[En] };
                break;
              case 717:
              case 718:
                this.$ = { matched: !1, bytarget: !0, action: Qn[En] };
                break;
              case 719:
              case 720:
                this.$ = { matched: !1, bytarget: !0, expr: Qn[En - 2], action: Qn[En] };
                break;
              case 721:
                this.$ = { matched: !1, bysource: !0, action: Qn[En] };
                break;
              case 722:
                this.$ = { matched: !1, bysource: !0, expr: Qn[En - 2], action: Qn[En] };
                break;
              case 723:
                this.$ = { insert: !0, values: Qn[En] };
                break;
              case 724:
                this.$ = { insert: !0, values: Qn[En], columns: Qn[En - 3] };
                break;
              case 725:
                this.$ = { insert: !0, defaultvalues: !0 };
                break;
              case 726:
                this.$ = { insert: !0, defaultvalues: !0, columns: Qn[En - 3] };
                break;
              case 728:
                this.$ = { output: { columns: Qn[En] } };
                break;
              case 729:
                this.$ = { output: { columns: Qn[En - 3], intovar: Qn[En], method: Qn[En - 1] } };
                break;
              case 730:
                this.$ = { output: { columns: Qn[En - 2], intotable: Qn[En] } };
                break;
              case 731:
                this.$ = { output: { columns: Qn[En - 5], intotable: Qn[En - 3], intocolumns: Qn[En - 1] } };
                break;
              case 732:
                this.$ = new ar.CreateVertex({ class: Qn[En - 3], sharp: Qn[En - 2], name: Qn[En - 1] }), ar.extend(this.$, Qn[En]);
                break;
              case 735:
                this.$ = { sets: Qn[En] };
                break;
              case 736:
                this.$ = { content: Qn[En] };
                break;
              case 737:
                this.$ = { select: Qn[En] };
                break;
              case 738:
                this.$ = new ar.CreateEdge({ from: Qn[En - 3], to: Qn[En - 1], name: Qn[En - 5] }), ar.extend(this.$, Qn[En]);
                break;
              case 739:
                this.$ = new ar.CreateGraph({ graph: Qn[En] });
                break;
              case 740:
                this.$ = new ar.CreateGraph({ from: Qn[En] });
                break;
              case 743:
                this.$ = Qn[En - 2], Qn[En - 1] && (this.$.json = new ar.Json({ value: Qn[En - 1] })), Qn[En] && (this.$.as = Qn[En]);
                break;
              case 744:
                this.$ = { source: Qn[En - 6], target: Qn[En] }, Qn[En - 3] && (this.$.json = new ar.Json({ value: Qn[En - 3] })), Qn[En - 2] && (this.$.as = Qn[En - 2]), ar.extend(this.$, Qn[En - 4]);
                break;
              case 745:
                this.$ = { source: Qn[En - 5], target: Qn[En] }, Qn[En - 2] && (this.$.json = new ar.Json({ value: Qn[En - 3] })), Qn[En - 1] && (this.$.as = Qn[En - 2]);
                break;
              case 746:
                this.$ = { source: Qn[En - 2], target: Qn[En] };
                break;
              case 750:
                this.$ = { vars: Qn[En], method: Qn[En - 1] };
                break;
              case 753:
              case 754:
                var Ro = Qn[En - 1];
                this.$ = { prop: Qn[En - 3], sharp: Qn[En - 2], name: typeof Ro > "u" ? void 0 : Ro.substr(1, Ro.length - 2), class: Qn[En] };
                break;
              case 755:
                var Qa = Qn[En - 1];
                this.$ = { sharp: Qn[En - 2], name: typeof Qa > "u" ? void 0 : Qa.substr(1, Qa.length - 2), class: Qn[En] };
                break;
              case 756:
                var Ka = Qn[En - 1];
                this.$ = { name: typeof Ka > "u" ? void 0 : Ka.substr(1, Ka.length - 2), class: Qn[En] };
                break;
              case 757:
                this.$ = { class: Qn[En] };
                break;
              case 763:
                this.$ = new ar.AddRule({ left: Qn[En - 2], right: Qn[En] });
                break;
              case 764:
                this.$ = new ar.AddRule({ right: Qn[En] });
                break;
              case 767:
                this.$ = { termid: Qn[En] };
                break;
              case 768:
                this.$ = { termid: Qn[En - 3], args: Qn[En - 1] };
                break;
              case 771:
                this.$ = new ar.CreateTrigger({ trigger: Qn[En - 6], when: Qn[En - 5], action: Qn[En - 4], table: Qn[En - 2], statement: Qn[En] }), Qn[En].exists && (this.$.exists = Qn[En].exists), Qn[En].queries && (this.$.queries = Qn[En].queries);
                break;
              case 772:
                this.$ = new ar.CreateTrigger({ trigger: Qn[En - 5], when: Qn[En - 4], action: Qn[En - 3], table: Qn[En - 1], funcid: Qn[En] });
                break;
              case 773:
                this.$ = new ar.CreateTrigger({ trigger: Qn[En - 6], when: Qn[En - 4], action: Qn[En - 3], table: Qn[En - 5], statement: Qn[En] }), Qn[En].exists && (this.$.exists = Qn[En].exists), Qn[En].queries && (this.$.queries = Qn[En].queries);
                break;
              case 774:
              case 775:
              case 777:
                this.$ = "AFTER";
                break;
              case 776:
                this.$ = "BEFORE";
                break;
              case 778:
                this.$ = "INSTEADOF";
                break;
              case 779:
                this.$ = "INSERT";
                break;
              case 780:
                this.$ = "DELETE";
                break;
              case 781:
                this.$ = "UPDATE";
                break;
              case 782:
                this.$ = new ar.DropTrigger({ trigger: Qn[En] });
                break;
              case 783:
                this.$ = new ar.Reindex({ indexid: Qn[En] });
                break;
              case 1057:
              case 1077:
              case 1079:
              case 1081:
              case 1085:
              case 1087:
              case 1089:
              case 1091:
              case 1093:
              case 1095:
                this.$ = [];
                break;
              case 1058:
              case 1072:
              case 1074:
              case 1078:
              case 1080:
              case 1082:
              case 1086:
              case 1088:
              case 1090:
              case 1092:
              case 1094:
              case 1096:
                Qn[En - 1].push(Qn[En]);
                break;
              case 1071:
              case 1073:
                this.$ = [Qn[En]];
                break;
            }
          }, table: [Ie([10, 607, 768], rt, { 8: 1, 9: 2, 12: 3, 13: 4, 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 2: ft, 4: pt, 5: wt, 14: Bt, 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), { 1: [3] }, { 10: [1, 105], 11: 106, 607: oa, 768: ka }, Ie(aa, [2, 8]), Ie(aa, [2, 9]), Ie(hi, [2, 12]), Ie(aa, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 12: 109, 2: ft, 4: pt, 5: wt, 15: [1, 110], 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), Ie(hi, [2, 14]), Ie(hi, [2, 15]), Ie(hi, [2, 16]), Ie(hi, [2, 17]), Ie(hi, [2, 18]), Ie(hi, [2, 19]), Ie(hi, [2, 20]), Ie(hi, [2, 21]), Ie(hi, [2, 22]), Ie(hi, [2, 23]), Ie(hi, [2, 24]), Ie(hi, [2, 25]), Ie(hi, [2, 26]), Ie(hi, [2, 27]), Ie(hi, [2, 28]), Ie(hi, [2, 29]), Ie(hi, [2, 30]), Ie(hi, [2, 31]), Ie(hi, [2, 32]), Ie(hi, [2, 33]), Ie(hi, [2, 34]), Ie(hi, [2, 35]), Ie(hi, [2, 36]), Ie(hi, [2, 37]), Ie(hi, [2, 38]), Ie(hi, [2, 39]), Ie(hi, [2, 40]), Ie(hi, [2, 41]), Ie(hi, [2, 42]), Ie(hi, [2, 43]), Ie(hi, [2, 44]), Ie(hi, [2, 45]), Ie(hi, [2, 46]), Ie(hi, [2, 47]), Ie(hi, [2, 48]), Ie(hi, [2, 49]), Ie(hi, [2, 50]), Ie(hi, [2, 51]), Ie(hi, [2, 52]), Ie(hi, [2, 53]), Ie(hi, [2, 54]), Ie(hi, [2, 55]), Ie(hi, [2, 56]), Ie(hi, [2, 57]), Ie(hi, [2, 58]), Ie(hi, [2, 59]), Ie(hi, [2, 60]), Ie(hi, [2, 61]), Ie(hi, [2, 62]), Ie(hi, [2, 63]), Ie(hi, [2, 64]), Ie(hi, [2, 65]), Ie(hi, [2, 66]), Ie(hi, [2, 67]), { 358: [1, 111] }, { 2: ft, 3: 112, 4: pt, 5: wt }, { 2: ft, 3: 114, 4: pt, 5: wt, 156: Gr, 200: 113, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di }, Ie(Da, [2, 510], { 3: 123, 353: 127, 2: ft, 4: pt, 5: wt, 134: la, 135: wa, 187: [1, 125], 193: [1, 124], 273: [1, 131], 274: [1, 132], 362: [1, 133], 410: [1, 122], 477: [1, 126], 514: [1, 130] }), { 145: Wa, 454: 134, 455: 135 }, { 183: [1, 137] }, { 410: [1, 138] }, { 2: ft, 3: 140, 4: pt, 5: wt, 130: [1, 146], 193: [1, 141], 358: [1, 145], 402: 142, 410: [1, 139], 415: [1, 143], 514: [1, 144] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 147, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Fa, $o, { 345: 208, 171: [1, 209], 198: Gs }), Ie(Fa, $o, { 345: 211, 198: Gs }), { 2: ft, 3: 223, 4: pt, 5: wt, 77: Al, 132: ul, 143: Rr, 144: 216, 145: Vr, 152: Jr, 156: Gr, 181: gr, 198: [1, 214], 199: 217, 200: 219, 201: 218, 202: 221, 209: 213, 213: gc, 214: 222, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei, 458: 212 }, { 2: ft, 3: 225, 4: pt, 5: wt }, { 358: [1, 226] }, Ie(Gl, [2, 1053], { 80: 227, 106: 228, 107: [1, 229] }), Ie(Sl, [2, 1057], { 90: 230 }), { 2: ft, 3: 234, 4: pt, 5: wt, 190: [1, 232], 193: [1, 235], 272: [1, 231], 358: [1, 236], 410: [1, 233] }, { 358: [1, 237] }, { 2: ft, 3: 240, 4: pt, 5: wt, 73: 238, 75: 239 }, Ie([311, 607, 768], rt, { 12: 3, 13: 4, 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 9: 242, 2: ft, 4: pt, 5: wt, 14: Bt, 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 440: [1, 241], 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), { 440: [1, 243] }, { 440: [1, 244] }, { 2: ft, 3: 246, 4: pt, 5: wt, 410: [1, 245] }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 247 }, Ie(ls, [2, 316]), { 113: 249, 132: rr, 301: Dn }, { 2: ft, 3: 114, 4: pt, 5: wt, 113: 255, 131: Ur, 132: [1, 252], 143: Rr, 144: 250, 145: vs, 152: Jr, 156: Gr, 181: gr, 196: 254, 200: 259, 201: 258, 261: 256, 262: 257, 269: Vl, 270: Ll, 279: 251, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 307: yr, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 262, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(hi, [2, 681]), Ie(hi, [2, 682]), { 2: ft, 3: 171, 4: pt, 5: wt, 40: 264, 56: 167, 77: Qr, 79: 75, 89: Ot, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 263, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 184: 99, 189: gn, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 271, 4: pt, 5: wt, 113: 268, 132: rr, 301: Dn, 449: 266, 450: 267, 451: 269, 452: Xl }, { 2: ft, 3: 272, 4: pt, 5: wt, 143: yc, 145: gu, 436: 273 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 276, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 510: [1, 277] }, { 2: ft, 3: 100, 4: pt, 5: wt, 509: 279, 511: 278 }, { 2: ft, 3: 114, 4: pt, 5: wt, 156: Gr, 200: 280, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 281, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Nc, Cu, { 186: 285, 164: [1, 284], 185: [1, 282], 187: [1, 283], 195: zc }), Ie(Kc, [2, 767], { 77: [1, 287] }), Ie([2, 4, 5, 10, 72, 77, 78, 93, 98, 107, 118, 128, 131, 132, 137, 143, 145, 152, 154, 156, 162, 164, 168, 169, 179, 180, 181, 183, 185, 187, 195, 198, 232, 244, 245, 249, 251, 269, 270, 271, 275, 276, 278, 285, 286, 287, 288, 289, 290, 291, 292, 294, 295, 296, 297, 298, 299, 300, 301, 302, 303, 304, 307, 308, 311, 315, 317, 322, 425, 429, 607, 768], [2, 152], { 149: [1, 288], 150: [1, 289], 190: [1, 290], 191: [1, 291], 192: [1, 292], 193: [1, 293], 194: [1, 294] }), Ie(zi, [2, 1]), Ie(zi, [2, 2]), { 6: 295, 131: [1, 444], 172: [1, 467], 243: [1, 443], 244: [1, 378], 245: [1, 412], 249: [1, 416], 375: [1, 409], 386: [1, 300], 407: [1, 302], 415: [1, 554], 419: [1, 476], 421: [1, 448], 422: [1, 514], 438: [1, 447], 440: [1, 530], 445: [1, 347], 465: [1, 423], 469: [1, 453], 475: [1, 346], 519: [1, 312], 520: [1, 304], 521: [1, 404], 523: [1, 296], 524: [1, 297], 525: [1, 298], 526: [1, 299], 527: [1, 301], 528: [1, 303], 529: [1, 305], 530: [1, 306], 531: [1, 307], 532: [1, 308], 533: [1, 309], 534: [1, 310], 535: [1, 311], 536: [1, 313], 537: [1, 314], 538: [1, 315], 539: [1, 316], 540: [1, 317], 541: [1, 318], 542: [1, 319], 543: [1, 320], 544: [1, 321], 545: [1, 322], 546: [1, 323], 547: [1, 324], 548: [1, 325], 549: [1, 326], 550: [1, 327], 551: [1, 328], 552: [1, 329], 553: [1, 330], 554: [1, 331], 555: [1, 332], 556: [1, 333], 557: [1, 334], 558: [1, 335], 559: [1, 336], 560: [1, 337], 561: [1, 338], 562: [1, 339], 563: [1, 340], 564: [1, 341], 565: [1, 342], 566: [1, 343], 567: [1, 344], 568: [1, 345], 569: [1, 348], 570: [1, 349], 571: [1, 350], 572: [1, 351], 573: [1, 352], 574: [1, 353], 575: [1, 354], 576: [1, 355], 577: [1, 356], 578: [1, 357], 579: [1, 358], 580: [1, 359], 581: [1, 360], 582: [1, 361], 583: [1, 362], 584: [1, 363], 585: [1, 364], 586: [1, 365], 587: [1, 366], 588: [1, 367], 589: [1, 368], 590: [1, 369], 591: [1, 370], 592: [1, 371], 593: [1, 372], 594: [1, 373], 595: [1, 374], 596: [1, 375], 597: [1, 376], 598: [1, 377], 599: [1, 379], 600: [1, 380], 601: [1, 381], 602: [1, 382], 603: [1, 383], 604: [1, 384], 605: [1, 385], 606: [1, 386], 607: [1, 387], 608: [1, 388], 609: [1, 389], 610: [1, 390], 611: [1, 391], 612: [1, 392], 613: [1, 393], 614: [1, 394], 615: [1, 395], 616: [1, 396], 617: [1, 397], 618: [1, 398], 619: [1, 399], 620: [1, 400], 621: [1, 401], 622: [1, 402], 623: [1, 403], 624: [1, 405], 625: [1, 406], 626: [1, 407], 627: [1, 408], 628: [1, 410], 629: [1, 411], 630: [1, 413], 631: [1, 414], 632: [1, 415], 633: [1, 417], 634: [1, 418], 635: [1, 419], 636: [1, 420], 637: [1, 421], 638: [1, 422], 639: [1, 424], 640: [1, 425], 641: [1, 426], 642: [1, 427], 643: [1, 428], 644: [1, 429], 645: [1, 430], 646: [1, 431], 647: [1, 432], 648: [1, 433], 649: [1, 434], 650: [1, 435], 651: [1, 436], 652: [1, 437], 653: [1, 438], 654: [1, 439], 655: [1, 440], 656: [1, 441], 657: [1, 442], 658: [1, 445], 659: [1, 446], 660: [1, 449], 661: [1, 450], 662: [1, 451], 663: [1, 452], 664: [1, 454], 665: [1, 455], 666: [1, 456], 667: [1, 457], 668: [1, 458], 669: [1, 459], 670: [1, 460], 671: [1, 461], 672: [1, 462], 673: [1, 463], 674: [1, 464], 675: [1, 465], 676: [1, 466], 677: [1, 468], 678: [1, 469], 679: [1, 470], 680: [1, 471], 681: [1, 472], 682: [1, 473], 683: [1, 474], 684: [1, 475], 685: [1, 477], 686: [1, 478], 687: [1, 479], 688: [1, 480], 689: [1, 481], 690: [1, 482], 691: [1, 483], 692: [1, 484], 693: [1, 485], 694: [1, 486], 695: [1, 487], 696: [1, 488], 697: [1, 489], 698: [1, 490], 699: [1, 491], 700: [1, 492], 701: [1, 493], 702: [1, 494], 703: [1, 495], 704: [1, 496], 705: [1, 497], 706: [1, 498], 707: [1, 499], 708: [1, 500], 709: [1, 501], 710: [1, 502], 711: [1, 503], 712: [1, 504], 713: [1, 505], 714: [1, 506], 715: [1, 507], 716: [1, 508], 717: [1, 509], 718: [1, 510], 719: [1, 511], 720: [1, 512], 721: [1, 513], 722: [1, 515], 723: [1, 516], 724: [1, 517], 725: [1, 518], 726: [1, 519], 727: [1, 520], 728: [1, 521], 729: [1, 522], 730: [1, 523], 731: [1, 524], 732: [1, 525], 733: [1, 526], 734: [1, 527], 735: [1, 528], 736: [1, 529], 737: [1, 531], 738: [1, 532], 739: [1, 533], 740: [1, 534], 741: [1, 535], 742: [1, 536], 743: [1, 537], 744: [1, 538], 745: [1, 539], 746: [1, 540], 747: [1, 541], 748: [1, 542], 749: [1, 543], 750: [1, 544], 751: [1, 545], 752: [1, 546], 753: [1, 547], 754: [1, 548], 755: [1, 549], 756: [1, 550], 757: [1, 551], 758: [1, 552], 759: [1, 553], 760: [1, 555], 761: [1, 556], 762: [1, 557], 763: [1, 558], 764: [1, 559], 765: [1, 560], 766: [1, 561], 767: [1, 562] }, { 1: [2, 6] }, Ie(aa, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 12: 563, 2: ft, 4: pt, 5: wt, 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), Ie(Fu, [2, 1051]), Ie(Fu, [2, 1052]), Ie(aa, [2, 10]), { 16: [1, 564] }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 565 }, { 410: [1, 566] }, Ie(hi, [2, 770]), { 77: d1 }, { 77: [1, 568] }, { 77: Gu }, { 77: Ku }, { 77: [1, 571] }, { 77: [1, 572] }, { 77: [1, 573] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 574, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Fa, I1, { 355: 575, 156: f1 }), { 410: [1, 577] }, { 2: ft, 3: 578, 4: pt, 5: wt }, { 193: [1, 579] }, { 2: ft, 3: 585, 4: pt, 5: wt, 132: D1, 137: Hl, 143: yc, 145: gu, 152: Kl, 183: [1, 581], 436: 592, 478: 580, 479: 582, 480: 583, 483: 584, 487: 589, 498: 586, 502: 588 }, { 130: [1, 596], 354: 593, 358: [1, 595], 415: [1, 594] }, { 113: 598, 132: rr, 183: [2, 1151], 301: Dn, 476: 597 }, Ie(uu, [2, 1145], { 470: 599, 3: 600, 2: ft, 4: pt, 5: wt }), { 2: ft, 3: 601, 4: pt, 5: wt }, { 4: [1, 602] }, { 4: [1, 603] }, Ie(Da, [2, 511]), Ie(hi, [2, 695], { 74: [1, 604] }), Ie(Hs, [2, 696]), { 2: ft, 3: 605, 4: pt, 5: wt }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 606 }, { 2: ft, 3: 607, 4: pt, 5: wt }, Ie(Fa, b1, { 403: 608, 156: Wl }), { 410: [1, 610] }, { 2: ft, 3: 611, 4: pt, 5: wt }, Ie(Fa, b1, { 403: 612, 156: Wl }), Ie(Fa, b1, { 403: 613, 156: Wl }), { 2: ft, 3: 614, 4: pt, 5: wt }, Ie(zl, [2, 1139]), Ie(zl, [2, 1140]), Ie(hi, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 12: 615, 114: 632, 332: 644, 2: ft, 4: pt, 5: wt, 53: At, 72: Dt, 89: Ot, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Jc, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 146: yn, 154: Ls, 156: _n, 170: Ms, 171: vi, 179: Ri, 180: ta, 189: gn, 271: bn, 272: an, 294: Zt, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), Ie(ls, [2, 292]), Ie(ls, [2, 293]), Ie(ls, [2, 294]), Ie(ls, [2, 295]), Ie(ls, [2, 296]), Ie(ls, [2, 297]), Ie(ls, [2, 298]), Ie(ls, [2, 299]), Ie(ls, [2, 300]), Ie(ls, [2, 301]), Ie(ls, [2, 302]), Ie(ls, [2, 303]), Ie(ls, [2, 304]), Ie(ls, [2, 305]), Ie(ls, [2, 306]), Ie(ls, [2, 307]), Ie(ls, [2, 308]), Ie(ls, [2, 309]), { 2: ft, 3: 171, 4: pt, 5: wt, 26: 661, 27: 660, 36: 656, 40: 655, 56: 167, 77: Qr, 79: 75, 89: Ot, 94: 658, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 184: 99, 189: gn, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 268: 657, 269: Qi, 270: Ai, 271: bn, 272: [1, 662], 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: [1, 659], 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 344: In, 424: 193, 425: Si, 429: Ei }, Ie(ls, [2, 313]), Ie(ls, [2, 314]), Ie(Bu, [2, 315], { 77: Ku }), { 77: [1, 663] }, Ie([2, 4, 5, 10, 53, 72, 74, 76, 78, 89, 93, 95, 98, 99, 107, 112, 115, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], Ql, { 77: d1, 116: [1, 664] }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 665, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 666, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 667, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 668, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 669, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ls, [2, 287]), Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 239, 244, 245, 246, 247, 249, 251, 253, 269, 270, 271, 272, 275, 276, 278, 285, 286, 287, 288, 289, 290, 291, 292, 294, 295, 296, 297, 298, 299, 300, 301, 302, 303, 304, 305, 307, 308, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 348, 361, 373, 374, 378, 379, 401, 405, 406, 409, 411, 413, 414, 420, 422, 423, 425, 429, 431, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768, 769, 770], [2, 365]), Ie(dl, [2, 366]), Ie(dl, [2, 367]), Ie(dl, h1), Ie(dl, [2, 369]), Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 302, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 348, 361, 373, 374, 378, 379, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 431, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 370]), { 2: ft, 3: 671, 4: pt, 5: wt, 131: [1, 672], 306: 670 }, { 2: ft, 3: 673, 4: pt, 5: wt }, Ie(dl, [2, 376]), Ie(dl, [2, 377]), { 2: ft, 3: 674, 4: pt, 5: wt, 77: z1, 113: 676, 131: Ur, 132: rr, 143: Rr, 152: Jr, 181: gr, 196: 677, 201: 679, 261: 678, 299: nn, 300: Tn, 301: Dn, 307: yr, 424: 680, 429: Ei }, { 77: [1, 681] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 682, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 309: 683, 312: 684, 313: J1, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 77: [1, 686] }, { 77: [1, 687] }, Ie(Lu, [2, 633]), { 2: ft, 3: 702, 4: pt, 5: wt, 77: M1, 111: 697, 113: 695, 131: Ur, 132: rr, 143: Rr, 144: 692, 145: vs, 152: Jr, 156: Gr, 181: gr, 196: 694, 200: 700, 201: 699, 261: 696, 262: 698, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 305: [1, 690], 307: yr, 424: 193, 425: Si, 426: 688, 427: 691, 428: 693, 429: Ei, 432: 689 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 703, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 704, 4: pt, 5: wt, 156: Gr, 200: 705, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di }, { 77: [2, 342] }, { 77: [2, 343] }, { 77: [2, 344] }, { 77: [2, 345] }, { 77: [2, 346] }, { 77: [2, 347] }, { 77: [2, 348] }, { 77: [2, 349] }, { 77: [2, 350] }, { 77: [2, 351] }, { 2: ft, 3: 711, 4: pt, 5: wt, 131: ku, 132: Wu, 430: 706, 431: [1, 707], 433: 708 }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 712 }, { 294: [1, 713] }, Ie(Fa, [2, 481]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 714 }, { 231: [1, 716], 459: 715 }, { 231: [2, 704] }, { 2: ft, 3: 223, 4: pt, 5: wt, 77: Al, 132: ul, 143: Rr, 144: 216, 145: Vr, 152: Jr, 156: Gr, 181: gr, 199: 217, 200: 219, 201: 218, 202: 221, 209: 717, 213: gc, 214: 222, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei }, { 40: 718, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(T1, [2, 1101], { 210: 719, 76: [1, 720] }), Ie(kl, [2, 185], { 3: 721, 2: ft, 4: pt, 5: wt, 76: [1, 722], 154: [1, 723] }), Ie(kl, [2, 189], { 3: 724, 2: ft, 4: pt, 5: wt, 76: [1, 725] }), Ie(kl, [2, 190], { 3: 726, 2: ft, 4: pt, 5: wt, 76: [1, 727] }), Ie(kl, [2, 193]), Ie(kl, [2, 194], { 3: 728, 2: ft, 4: pt, 5: wt, 76: [1, 729] }), Ie(kl, [2, 197], { 3: 730, 2: ft, 4: pt, 5: wt, 76: [1, 731] }), Ie([2, 4, 5, 10, 72, 74, 76, 78, 93, 98, 118, 128, 154, 162, 168, 169, 183, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 249, 251, 311, 315, 607, 768], Yl, { 77: d1, 116: i1 }), Ie([2, 4, 5, 10, 72, 74, 76, 78, 93, 98, 118, 128, 162, 168, 169, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 249, 251, 311, 315, 607, 768], [2, 200]), Ie(hi, [2, 783]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 733 }, Ie(ic, iu, { 81: 734, 198: xu }), Ie(Gl, [2, 1054]), Ie(vc, [2, 1067], { 108: 736, 190: [1, 737] }), Ie([10, 78, 183, 311, 315, 607, 768], iu, { 424: 193, 81: 738, 117: 739, 3: 740, 114: 743, 144: 765, 158: 775, 160: 776, 2: ft, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 112: Pc, 115: uo, 116: so, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 198: xu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 425: Si, 429: Ei }), { 358: [1, 789] }, { 183: [1, 790] }, Ie(hi, [2, 603], { 112: [1, 791] }), { 410: [1, 792] }, { 183: [1, 793] }, Ie(hi, [2, 607], { 112: [1, 794], 183: [1, 795] }), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 796 }, { 40: 797, 74: [1, 798], 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(gd, [2, 70]), { 76: [1, 799] }, Ie(hi, [2, 676]), { 11: 106, 311: [1, 800], 607: oa, 768: ka }, Ie(hi, [2, 674]), Ie(hi, [2, 675]), { 2: ft, 3: 801, 4: pt, 5: wt }, Ie(hi, [2, 596]), { 146: [1, 802] }, Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 95, 124, 128, 143, 145, 146, 148, 149, 152, 154, 156, 181, 183, 187, 189, 230, 271, 272, 294, 302, 307, 311, 315, 340, 343, 344, 348, 349, 361, 373, 374, 378, 379, 401, 405, 406, 407, 408, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 510, 512, 513, 519, 520, 521, 522, 607, 768], Yl, { 116: i1 }), Ie(hi, [2, 624]), Ie(hi, [2, 625]), Ie(hi, [2, 626]), Ie(hi, h1, { 74: [1, 803] }), { 77: z1, 113: 676, 131: Ur, 132: rr, 143: Rr, 152: Jr, 181: gr, 196: 677, 201: 679, 261: 678, 299: nn, 300: Tn, 301: Dn, 307: yr, 424: 680, 429: Ei }, Ie(Jl, [2, 325]), Ie(Jl, [2, 326]), Ie(Jl, [2, 327]), Ie(Jl, [2, 328]), Ie(Jl, [2, 329]), Ie(Jl, [2, 330]), Ie(Jl, [2, 331]), Ie(Jl, [2, 332], { 77: Ku }), Ie(hi, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 114: 632, 332: 644, 12: 804, 2: ft, 4: pt, 5: wt, 53: At, 72: Dt, 89: Ot, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Jc, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 146: yn, 154: Ls, 156: _n, 170: Ms, 171: vi, 179: Ri, 180: ta, 189: gn, 271: bn, 272: an, 294: Zt, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), Ie(hi, [2, 684], { 74: Yc }), Ie(hi, [2, 685]), Ie(nc, [2, 363], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(hi, [2, 686], { 74: [1, 807] }), Ie(hi, [2, 687], { 74: [1, 808] }), Ie(Hs, [2, 692]), Ie(Hs, [2, 694]), Ie(Hs, [2, 688]), Ie(Hs, [2, 689]), { 114: 814, 115: uo, 116: so, 124: [1, 809], 230: ks, 434: 810, 435: 811, 438: qs }, { 2: ft, 3: 815, 4: pt, 5: wt }, Ie(Fa, [2, 665]), Ie(Fa, [2, 666]), Ie(hi, [2, 623], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 100, 4: pt, 5: wt, 509: 279, 511: 816 }, Ie(hi, [2, 764], { 74: a1 }), Ie($l, [2, 766]), Ie(hi, [2, 769]), Ie(hi, [2, 690], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(o1, Cu, { 186: 818, 195: zc }), Ie(o1, Cu, { 186: 819, 195: zc }), Ie(o1, Cu, { 186: 820, 195: zc }), Ie($c, [2, 1097], { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 188: 821, 174: 822, 257: 823, 94: 824, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), { 77: [1, 826], 131: Ur, 196: 825 }, { 2: ft, 3: 100, 4: pt, 5: wt, 509: 279, 511: 827 }, Ie(lu, [2, 153]), Ie(lu, [2, 154]), Ie(lu, [2, 155]), Ie(lu, [2, 156]), Ie(lu, [2, 157]), Ie(lu, [2, 158]), Ie(lu, [2, 159]), Ie(zi, [2, 3]), Ie(zi, [2, 784]), Ie(zi, [2, 785]), Ie(zi, [2, 786]), Ie(zi, [2, 787]), Ie(zi, [2, 788]), Ie(zi, [2, 789]), Ie(zi, [2, 790]), Ie(zi, [2, 791]), Ie(zi, [2, 792]), Ie(zi, [2, 793]), Ie(zi, [2, 794]), Ie(zi, [2, 795]), Ie(zi, [2, 796]), Ie(zi, [2, 797]), Ie(zi, [2, 798]), Ie(zi, [2, 799]), Ie(zi, [2, 800]), Ie(zi, [2, 801]), Ie(zi, [2, 802]), Ie(zi, [2, 803]), Ie(zi, [2, 804]), Ie(zi, [2, 805]), Ie(zi, [2, 806]), Ie(zi, [2, 807]), Ie(zi, [2, 808]), Ie(zi, [2, 809]), Ie(zi, [2, 810]), Ie(zi, [2, 811]), Ie(zi, [2, 812]), Ie(zi, [2, 813]), Ie(zi, [2, 814]), Ie(zi, [2, 815]), Ie(zi, [2, 816]), Ie(zi, [2, 817]), Ie(zi, [2, 818]), Ie(zi, [2, 819]), Ie(zi, [2, 820]), Ie(zi, [2, 821]), Ie(zi, [2, 822]), Ie(zi, [2, 823]), Ie(zi, [2, 824]), Ie(zi, [2, 825]), Ie(zi, [2, 826]), Ie(zi, [2, 827]), Ie(zi, [2, 828]), Ie(zi, [2, 829]), Ie(zi, [2, 830]), Ie(zi, [2, 831]), Ie(zi, [2, 832]), Ie(zi, [2, 833]), Ie(zi, [2, 834]), Ie(zi, [2, 835]), Ie(zi, [2, 836]), Ie(zi, [2, 837]), Ie(zi, [2, 838]), Ie(zi, [2, 839]), Ie(zi, [2, 840]), Ie(zi, [2, 841]), Ie(zi, [2, 842]), Ie(zi, [2, 843]), Ie(zi, [2, 844]), Ie(zi, [2, 845]), Ie(zi, [2, 846]), Ie(zi, [2, 847]), Ie(zi, [2, 848]), Ie(zi, [2, 849]), Ie(zi, [2, 850]), Ie(zi, [2, 851]), Ie(zi, [2, 852]), Ie(zi, [2, 853]), Ie(zi, [2, 854]), Ie(zi, [2, 855]), Ie(zi, [2, 856]), Ie(zi, [2, 857]), Ie(zi, [2, 858]), Ie(zi, [2, 859]), Ie(zi, [2, 860]), Ie(zi, [2, 861]), Ie(zi, [2, 862]), Ie(zi, [2, 863]), Ie(zi, [2, 864]), Ie(zi, [2, 865]), Ie(zi, [2, 866]), Ie(zi, [2, 867]), Ie(zi, [2, 868]), Ie(zi, [2, 869]), Ie(zi, [2, 870]), Ie(zi, [2, 871]), Ie(zi, [2, 872]), Ie(zi, [2, 873]), Ie(zi, [2, 874]), Ie(zi, [2, 875]), Ie(zi, [2, 876]), Ie(zi, [2, 877]), Ie(zi, [2, 878]), Ie(zi, [2, 879]), Ie(zi, [2, 880]), Ie(zi, [2, 881]), Ie(zi, [2, 882]), Ie(zi, [2, 883]), Ie(zi, [2, 884]), Ie(zi, [2, 885]), Ie(zi, [2, 886]), Ie(zi, [2, 887]), Ie(zi, [2, 888]), Ie(zi, [2, 889]), Ie(zi, [2, 890]), Ie(zi, [2, 891]), Ie(zi, [2, 892]), Ie(zi, [2, 893]), Ie(zi, [2, 894]), Ie(zi, [2, 895]), Ie(zi, [2, 896]), Ie(zi, [2, 897]), Ie(zi, [2, 898]), Ie(zi, [2, 899]), Ie(zi, [2, 900]), Ie(zi, [2, 901]), Ie(zi, [2, 902]), Ie(zi, [2, 903]), Ie(zi, [2, 904]), Ie(zi, [2, 905]), Ie(zi, [2, 906]), Ie(zi, [2, 907]), Ie(zi, [2, 908]), Ie(zi, [2, 909]), Ie(zi, [2, 910]), Ie(zi, [2, 911]), Ie(zi, [2, 912]), Ie(zi, [2, 913]), Ie(zi, [2, 914]), Ie(zi, [2, 915]), Ie(zi, [2, 916]), Ie(zi, [2, 917]), Ie(zi, [2, 918]), Ie(zi, [2, 919]), Ie(zi, [2, 920]), Ie(zi, [2, 921]), Ie(zi, [2, 922]), Ie(zi, [2, 923]), Ie(zi, [2, 924]), Ie(zi, [2, 925]), Ie(zi, [2, 926]), Ie(zi, [2, 927]), Ie(zi, [2, 928]), Ie(zi, [2, 929]), Ie(zi, [2, 930]), Ie(zi, [2, 931]), Ie(zi, [2, 932]), Ie(zi, [2, 933]), Ie(zi, [2, 934]), Ie(zi, [2, 935]), Ie(zi, [2, 936]), Ie(zi, [2, 937]), Ie(zi, [2, 938]), Ie(zi, [2, 939]), Ie(zi, [2, 940]), Ie(zi, [2, 941]), Ie(zi, [2, 942]), Ie(zi, [2, 943]), Ie(zi, [2, 944]), Ie(zi, [2, 945]), Ie(zi, [2, 946]), Ie(zi, [2, 947]), Ie(zi, [2, 948]), Ie(zi, [2, 949]), Ie(zi, [2, 950]), Ie(zi, [2, 951]), Ie(zi, [2, 952]), Ie(zi, [2, 953]), Ie(zi, [2, 954]), Ie(zi, [2, 955]), Ie(zi, [2, 956]), Ie(zi, [2, 957]), Ie(zi, [2, 958]), Ie(zi, [2, 959]), Ie(zi, [2, 960]), Ie(zi, [2, 961]), Ie(zi, [2, 962]), Ie(zi, [2, 963]), Ie(zi, [2, 964]), Ie(zi, [2, 965]), Ie(zi, [2, 966]), Ie(zi, [2, 967]), Ie(zi, [2, 968]), Ie(zi, [2, 969]), Ie(zi, [2, 970]), Ie(zi, [2, 971]), Ie(zi, [2, 972]), Ie(zi, [2, 973]), Ie(zi, [2, 974]), Ie(zi, [2, 975]), Ie(zi, [2, 976]), Ie(zi, [2, 977]), Ie(zi, [2, 978]), Ie(zi, [2, 979]), Ie(zi, [2, 980]), Ie(zi, [2, 981]), Ie(zi, [2, 982]), Ie(zi, [2, 983]), Ie(zi, [2, 984]), Ie(zi, [2, 985]), Ie(zi, [2, 986]), Ie(zi, [2, 987]), Ie(zi, [2, 988]), Ie(zi, [2, 989]), Ie(zi, [2, 990]), Ie(zi, [2, 991]), Ie(zi, [2, 992]), Ie(zi, [2, 993]), Ie(zi, [2, 994]), Ie(zi, [2, 995]), Ie(zi, [2, 996]), Ie(zi, [2, 997]), Ie(zi, [2, 998]), Ie(zi, [2, 999]), Ie(zi, [2, 1e3]), Ie(zi, [2, 1001]), Ie(zi, [2, 1002]), Ie(zi, [2, 1003]), Ie(zi, [2, 1004]), Ie(zi, [2, 1005]), Ie(zi, [2, 1006]), Ie(zi, [2, 1007]), Ie(zi, [2, 1008]), Ie(zi, [2, 1009]), Ie(zi, [2, 1010]), Ie(zi, [2, 1011]), Ie(zi, [2, 1012]), Ie(zi, [2, 1013]), Ie(zi, [2, 1014]), Ie(zi, [2, 1015]), Ie(zi, [2, 1016]), Ie(zi, [2, 1017]), Ie(zi, [2, 1018]), Ie(zi, [2, 1019]), Ie(zi, [2, 1020]), Ie(zi, [2, 1021]), Ie(zi, [2, 1022]), Ie(zi, [2, 1023]), Ie(zi, [2, 1024]), Ie(zi, [2, 1025]), Ie(zi, [2, 1026]), Ie(zi, [2, 1027]), Ie(zi, [2, 1028]), Ie(zi, [2, 1029]), Ie(zi, [2, 1030]), Ie(zi, [2, 1031]), Ie(zi, [2, 1032]), Ie(zi, [2, 1033]), Ie(zi, [2, 1034]), Ie(zi, [2, 1035]), Ie(zi, [2, 1036]), Ie(zi, [2, 1037]), Ie(zi, [2, 1038]), Ie(zi, [2, 1039]), Ie(zi, [2, 1040]), Ie(zi, [2, 1041]), Ie(zi, [2, 1042]), Ie(zi, [2, 1043]), Ie(zi, [2, 1044]), Ie(zi, [2, 1045]), Ie(zi, [2, 1046]), Ie(zi, [2, 1047]), Ie(zi, [2, 1048]), Ie(zi, [2, 1049]), Ie(zi, [2, 1050]), Ie(aa, [2, 7]), Ie(aa, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 12: 828, 2: ft, 4: pt, 5: wt, 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), { 401: [1, 832], 406: [1, 829], 407: [1, 830], 408: [1, 831] }, { 2: ft, 3: 833, 4: pt, 5: wt }, Ie(o1, [2, 1121], { 293: 834, 771: 836, 78: [1, 835], 164: [1, 838], 185: [1, 837] }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 839, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 840, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 78: [1, 841] }, { 2: ft, 3: 842, 4: pt, 5: wt, 132: [1, 843] }, { 2: ft, 3: 844, 4: pt, 5: wt, 132: [1, 845] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 846, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 847, 4: pt, 5: wt, 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 2: ft, 3: 848, 4: pt, 5: wt }, { 154: [1, 849] }, Ie(Ul, I1, { 355: 850, 156: f1 }), { 230: [1, 851] }, { 2: ft, 3: 852, 4: pt, 5: wt }, Ie(hi, [2, 739], { 74: zs }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 854, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie($l, [2, 742]), Ie(Os, [2, 1153], { 424: 193, 481: 855, 144: 856, 139: lc, 141: lc, 145: vs, 425: Si, 429: Ei }), { 139: [1, 857], 141: [1, 858] }, Ie(xc, Iu, { 495: 860, 498: 861, 77: [1, 859], 137: Hl }), Ie(tu, [2, 1177], { 499: 862, 132: [1, 863] }), Ie(w1, [2, 1181], { 501: 864, 502: 865, 152: Kl }), Ie(w1, [2, 757]), Ie($n, [2, 749]), { 2: ft, 3: 866, 4: pt, 5: wt, 131: [1, 867] }, { 2: ft, 3: 868, 4: pt, 5: wt }, { 2: ft, 3: 869, 4: pt, 5: wt }, Ie(Fa, I1, { 355: 870, 156: f1 }), Ie(Fa, I1, { 355: 871, 156: f1 }), Ie(zl, [2, 500]), Ie(zl, [2, 501]), { 183: [1, 872] }, { 183: [2, 1152] }, Ie(zn, [2, 1147], { 471: 873, 474: 874, 137: [1, 875] }), Ie(uu, [2, 1146]), Ie(Gn, Ke, { 515: 876, 95: Fe, 230: [1, 877], 519: Te, 520: Ae, 521: Oe }), { 76: [1, 882] }, { 76: [1, 883] }, { 145: Wa, 455: 884 }, { 4: Be, 7: 888, 76: [1, 886], 277: 885, 392: 887, 394: Ue }, Ie(hi, [2, 465], { 128: [1, 891] }), Ie(hi, [2, 588]), { 2: ft, 3: 892, 4: pt, 5: wt }, { 303: [1, 893] }, Ie(Ul, b1, { 403: 894, 156: Wl }), Ie(hi, [2, 602]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 896, 404: 895 }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 896, 404: 897 }, Ie(hi, [2, 782]), Ie(aa, [2, 678], { 443: 898, 315: [1, 899] }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 900, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 901, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 902, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 903, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 904, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 905, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 906, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 907, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 908, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 909, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 910, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 911, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 912, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 913, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 914, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 915, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 916, 4: pt, 5: wt, 77: [1, 918], 131: Ur, 156: Gr, 196: 917, 200: 919, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di }, { 2: ft, 3: 920, 4: pt, 5: wt, 77: [1, 922], 131: Ur, 156: Gr, 196: 921, 200: 923, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di }, Ie(nt, [2, 449], { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 94: 924, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), Ie(nt, [2, 450], { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 94: 925, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), Ie(nt, [2, 451], { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 94: 926, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), Ie(nt, [2, 452], { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 94: 927, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), Ie(nt, ht, { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 94: 928, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 929, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 930, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(nt, [2, 454], { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 94: 931, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 932, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 933, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 164: [1, 935], 166: [1, 937], 333: 934, 339: [1, 936] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 938, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 939, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 702, 4: pt, 5: wt, 77: [1, 940], 111: 943, 145: Et, 156: Gr, 200: 944, 202: 942, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 334: 941 }, { 99: [1, 946], 302: [1, 947] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 948, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 949, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 950, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 4: Be, 7: 888, 277: 951, 392: 887, 394: Ue }, Ie(Mt, [2, 88]), Ie(Mt, [2, 89]), { 78: [1, 952] }, { 78: [1, 953] }, { 78: [1, 954] }, { 78: [1, 955], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(Fa, $o, { 345: 211, 77: Gu, 198: Gs }), { 78: [2, 1117] }, { 78: [2, 1118] }, { 134: la, 135: wa }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 956, 152: Jr, 154: ti, 156: Gr, 158: 170, 164: [1, 958], 179: Hr, 180: Er, 181: gr, 185: [1, 957], 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 959, 4: pt, 5: wt, 149: qt, 180: [1, 961] }, Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 118, 122, 128, 129, 130, 131, 132, 134, 135, 137, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 319, 335, 336, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 425], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 337: Oo }), Ie(tn, [2, 426], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 180: ta, 317: Ji, 321: Ha }), Ie(tn, [2, 427], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 180: ta, 317: Ji, 321: Ha }), Ie(xn, [2, 428], { 114: 632, 332: 644, 321: Ha }), Ie(xn, [2, 429], { 114: 632, 332: 644, 321: Ha }), Ie(dl, [2, 374]), Ie(dl, [2, 1123]), Ie(dl, [2, 1124]), Ie(dl, [2, 375]), Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 371]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 962, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Lu, [2, 629]), Ie(Lu, [2, 630]), Ie(Lu, [2, 631]), Ie(Lu, [2, 632]), Ie(Lu, [2, 634]), { 40: 963, 79: 75, 89: Ot, 184: 99, 189: gn }, { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 309: 964, 312: 684, 313: J1, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 310: 965, 311: An, 312: 966, 313: J1, 315: On }, Ie(Bn, [2, 381]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 968, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 969, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 4: Be, 7: 888, 277: 970, 392: 887, 394: Ue }, Ie(Lu, [2, 635]), { 74: [1, 972], 305: [1, 971] }, Ie(Lu, [2, 651]), Ie(qn, [2, 658]), Ie(Vn, [2, 636]), Ie(Vn, [2, 637]), Ie(Vn, [2, 638]), Ie(Vn, [2, 639]), Ie(Vn, [2, 640]), Ie(Vn, [2, 641]), Ie(Vn, [2, 642]), Ie(Vn, [2, 643]), Ie(Vn, [2, 644]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 973, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie([2, 4, 5, 10, 53, 72, 74, 76, 78, 89, 93, 95, 98, 99, 107, 112, 115, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 431, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], Ql, { 77: d1, 116: sr }), { 74: Yc, 305: [1, 975] }, Ie(Bu, [2, 319], { 77: d1 }), Ie(ls, [2, 320]), { 74: [1, 977], 431: [1, 976] }, Ie(Lu, [2, 648]), Ie(Un, [2, 653]), { 152: [1, 978] }, { 152: [1, 979] }, { 152: [1, 980] }, { 40: 985, 77: [1, 984], 79: 75, 89: Ot, 143: Rr, 144: 988, 145: vs, 149: wr, 152: Jr, 181: gr, 184: 99, 189: gn, 201: 989, 307: yr, 346: 981, 347: 982, 348: [1, 983], 349: Cr, 424: 193, 425: Si, 429: Ei }, Ie(Fa, $o, { 345: 990, 198: Gs }), { 77: Tr, 143: Rr, 144: 988, 145: vs, 149: wr, 152: Jr, 181: gr, 201: 989, 307: yr, 346: 991, 347: 992, 349: Cr, 424: 193, 425: Si, 429: Ei }, { 230: [1, 995], 460: 994 }, { 2: ft, 3: 223, 4: pt, 5: wt, 77: [1, 997], 132: ul, 143: Rr, 144: 216, 145: Vr, 152: Jr, 156: Gr, 181: gr, 199: 217, 200: 219, 201: 218, 202: 221, 209: 996, 213: gc, 214: 222, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei }, { 231: [2, 705] }, { 78: [1, 998] }, Ie(kl, [2, 1103], { 211: 999, 3: 1e3, 2: ft, 4: pt, 5: wt }), Ie(T1, [2, 1102]), Ie(kl, [2, 183]), { 2: ft, 3: 1001, 4: pt, 5: wt }, { 212: [1, 1002] }, Ie(kl, [2, 187]), { 2: ft, 3: 1003, 4: pt, 5: wt }, Ie(kl, [2, 191]), { 2: ft, 3: 1004, 4: pt, 5: wt }, Ie(kl, [2, 195]), { 2: ft, 3: 1005, 4: pt, 5: wt }, Ie(kl, [2, 198]), { 2: ft, 3: 1006, 4: pt, 5: wt }, { 2: ft, 3: 1007, 4: pt, 5: wt }, { 148: [1, 1008] }, Ie(ii, [2, 172], { 82: 1009, 183: [1, 1010] }), { 2: ft, 3: 223, 4: pt, 5: wt, 132: [1, 1015], 143: Rr, 145: [1, 1016], 152: Jr, 156: Gr, 181: gr, 199: 1011, 200: 1012, 201: 1013, 202: 1014, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr }, { 2: ft, 3: 1021, 4: pt, 5: wt, 109: 1017, 110: 1018, 111: 1019, 112: wi }, Ie(vc, [2, 1068]), Ie(Wi, [2, 1059], { 91: 1022, 182: 1023, 183: [1, 1024] }), Ie(Sl, [2, 1058], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie([2, 4, 5, 10, 72, 74, 76, 78, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 198, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 311, 315, 425, 429, 607, 768], [2, 90], { 77: [1, 1029] }), { 119: [1, 1030] }, Ie(ya, [2, 93]), { 2: ft, 3: 1031, 4: pt, 5: wt }, Ie(ya, [2, 95]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1032, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1033, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 112: Pc, 114: 743, 115: uo, 116: so, 117: 1035, 118: du, 122: Ac, 123: yu, 124: au, 125: 1034, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 77: [1, 1036] }, { 77: [1, 1037] }, { 77: [1, 1038] }, { 77: [1, 1039] }, Ie(ya, [2, 104]), Ie(ya, [2, 105]), Ie(ya, [2, 106]), Ie(ya, [2, 107]), Ie(ya, [2, 108]), Ie(ya, [2, 109]), { 2: ft, 3: 1040, 4: pt, 5: wt }, { 2: ft, 3: 1041, 4: pt, 5: wt, 133: [1, 1042] }, Ie(ya, [2, 113]), Ie(ya, [2, 114]), Ie(ya, [2, 115]), Ie(ya, [2, 116]), Ie(ya, [2, 117]), Ie(ya, [2, 118]), { 2: ft, 3: 1043, 4: pt, 5: wt, 77: z1, 113: 676, 131: Ur, 132: rr, 143: Rr, 152: Jr, 181: gr, 196: 677, 201: 679, 261: 678, 299: nn, 300: Tn, 301: Dn, 307: yr, 424: 680, 429: Ei }, { 145: [1, 1044] }, { 77: [1, 1045] }, { 145: [1, 1046] }, Ie(ya, [2, 123]), { 77: [1, 1047] }, { 2: ft, 3: 1048, 4: pt, 5: wt }, { 77: [1, 1049] }, { 77: [1, 1050] }, { 77: [1, 1051] }, { 77: [1, 1052] }, { 77: [1, 1053], 164: [1, 1054] }, { 77: [1, 1055] }, { 77: [1, 1056] }, { 77: [1, 1057] }, { 77: [1, 1058] }, { 77: [1, 1059] }, { 77: [1, 1060] }, { 77: [1, 1061] }, { 77: [1, 1062] }, { 77: [1, 1063] }, { 77: [2, 1083] }, { 77: [2, 1084] }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1064 }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1065 }, { 113: 1066, 132: rr, 301: Dn }, Ie(hi, [2, 605], { 112: [1, 1067] }), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1068 }, { 113: 1069, 132: rr, 301: Dn }, { 2: ft, 3: 1070, 4: pt, 5: wt }, Ie(hi, [2, 702]), Ie(hi, [2, 68]), { 2: ft, 3: 240, 4: pt, 5: wt, 75: 1071 }, { 77: [1, 1072] }, Ie(hi, [2, 683]), Ie(hi, [2, 595]), { 2: ft, 3: 1021, 4: pt, 5: wt, 111: 1075, 143: va, 145: qa, 147: 1073, 341: 1074, 342: 1076 }, { 144: 1079, 145: vs, 424: 193, 425: Si, 429: Ei }, Ie(hi, [2, 680]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1080, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(nt, ht, { 259: 148, 200: 149, 260: 150, 111: 151, 258: 152, 196: 153, 261: 154, 113: 155, 262: 156, 201: 157, 202: 158, 263: 159, 264: 160, 265: 161, 144: 163, 266: 164, 267: 165, 56: 167, 158: 170, 3: 171, 424: 193, 94: 1081, 2: ft, 4: pt, 5: wt, 77: Qr, 131: Ur, 132: rr, 137: vr, 143: Rr, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 179: Hr, 180: Er, 181: gr, 244: $r, 245: li, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 425: Si, 429: Ei }), { 113: 1082, 132: rr, 301: Dn }, { 2: ft, 3: 271, 4: pt, 5: wt, 451: 1083, 452: Xl }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1085, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 230: ks, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei, 434: 1084, 438: qs }, Ie(hi, [2, 660]), { 114: 1087, 115: uo, 116: so, 124: [1, 1086] }, Ie(hi, [2, 672]), Ie(hi, [2, 673]), { 2: ft, 3: 1089, 4: pt, 5: wt, 77: Aa, 131: Xa, 437: 1088 }, { 114: 814, 115: uo, 116: so, 124: [1, 1092], 435: 1093 }, Ie(hi, [2, 763], { 74: a1 }), { 2: ft, 3: 100, 4: pt, 5: wt, 509: 1094 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 824, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 174: 1095, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 257: 823, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 824, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 174: 1096, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 257: 823, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 824, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 174: 1097, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 257: 823, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie($c, [2, 151]), Ie($c, [2, 1098], { 74: Oa }), Ie(Ya, [2, 277]), Ie(Ya, [2, 284], { 114: 632, 332: 644, 3: 1100, 113: 1102, 2: ft, 4: pt, 5: wt, 76: [1, 1099], 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 131: [1, 1101], 132: rr, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 301: Dn, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(Nc, [2, 1099], { 197: 1103, 769: [1, 1104] }), { 131: Ur, 196: 1105 }, { 74: a1, 78: [1, 1106] }, Ie(aa, [2, 11]), { 148: [1, 1107], 190: [1, 1108] }, { 190: [1, 1109] }, { 190: [1, 1110] }, { 190: [1, 1111] }, Ie(hi, [2, 584], { 76: [1, 1113], 77: [1, 1112] }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1114, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(dl, [2, 353]), Ie(o1, [2, 1122]), Ie(o1, [2, 1119]), Ie(o1, [2, 1120]), { 74: Yc, 78: [1, 1115] }, { 74: Yc, 78: [1, 1116] }, Ie(dl, [2, 356]), { 74: [1, 1117] }, { 74: [1, 1118] }, { 74: [1, 1119] }, { 74: [1, 1120] }, { 74: [1, 1121], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(dl, [2, 362]), Ie(hi, [2, 589]), { 303: [1, 1122] }, { 2: ft, 3: 1123, 4: pt, 5: wt, 113: 1124, 132: rr, 301: Dn }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1125 }, { 230: [1, 1126] }, { 2: ft, 3: 585, 4: pt, 5: wt, 132: D1, 137: Hl, 143: yc, 145: gu, 152: Kl, 436: 592, 479: 1127, 480: 583, 483: 584, 487: 589, 498: 586, 502: 588 }, Ie(hi, [2, 740], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie($l, [2, 1155], { 482: 1128, 488: 1129, 76: Wo }), Ie(Os, [2, 1154]), { 2: ft, 3: 1133, 4: pt, 5: wt, 132: D1, 137: Hl, 144: 1132, 145: vs, 152: Kl, 424: 193, 425: Si, 429: Ei, 480: 1131, 498: 586, 502: 588 }, { 2: ft, 3: 1133, 4: pt, 5: wt, 132: D1, 137: Hl, 143: yc, 145: gu, 152: Kl, 436: 592, 480: 1135, 483: 1134, 487: 589, 498: 586, 502: 588 }, { 2: ft, 3: 585, 4: pt, 5: wt, 132: D1, 137: Hl, 143: yc, 145: gu, 152: Kl, 436: 592, 478: 1136, 479: 582, 480: 583, 483: 584, 487: 589, 498: 586, 502: 588 }, Ie(tu, [2, 1173], { 496: 1137, 132: [1, 1138] }), Ie(xc, [2, 1172]), Ie(w1, [2, 1179], { 500: 1139, 502: 1140, 152: Kl }), Ie(tu, [2, 1178]), Ie(w1, [2, 756]), Ie(w1, [2, 1182]), Ie(xc, [2, 759]), Ie(xc, [2, 760]), Ie(w1, [2, 758]), Ie($n, [2, 750]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1141 }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1142 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1143, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Ja, [2, 1149], { 472: 1144, 113: 1145, 132: rr, 301: Dn }), Ie(zn, [2, 1148]), { 2: ft, 3: 1146, 4: pt, 5: wt }, { 340: Uo, 343: ys, 344: il, 516: 1147 }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1151 }, Ie(Gn, [2, 775]), Ie(Gn, [2, 776]), Ie(Gn, [2, 777]), { 129: [1, 1152] }, { 271: [1, 1153] }, { 271: [1, 1154] }, Ie(Hs, [2, 697]), Ie(Hs, [2, 698], { 124: [1, 1155] }), { 4: Be, 7: 888, 277: 1156, 392: 887, 394: Ue }, Ie([2, 4, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 302, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 348, 361, 373, 374, 378, 379, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 551], { 5: [1, 1157] }), Ie([2, 5, 10, 53, 72, 74, 76, 78, 89, 93, 95, 98, 99, 107, 112, 115, 116, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 302, 305, 311, 313, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 348, 361, 373, 374, 378, 379, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 548], { 4: [1, 1159], 77: [1, 1158] }), { 77: [1, 1160] }, Ie(Is, [2, 4]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1161, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(hi, [2, 597]), Ie(Ul, [2, 577]), { 2: ft, 3: 1162, 4: pt, 5: wt, 113: 1163, 132: rr, 301: Dn }, Ie(hi, [2, 573], { 74: ml }), Ie(Hs, [2, 575]), Ie(hi, [2, 622], { 74: ml }), Ie(hi, [2, 677]), Ie(hi, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 12: 1165, 2: ft, 4: pt, 5: wt, 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), Ie(nl, [2, 385], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba, 323: ao, 324: io }), Ie(xn, [2, 386], { 114: 632, 332: 644, 321: Ha }), Ie(nl, [2, 387], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba, 323: ao, 324: io }), Ie(Il, [2, 388], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 317: Ji, 319: [1, 1166], 321: Ha, 322: Ba, 323: ao, 324: io }), Ie(Il, [2, 390], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 317: Ji, 319: [1, 1167], 321: Ha, 322: Ba, 323: ao, 324: io }), Ie(ls, [2, 392], { 114: 632, 332: 644 }), Ie(tn, [2, 393], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 180: ta, 317: Ji, 321: Ha }), Ie(tn, [2, 394], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 180: ta, 317: Ji, 321: Ha }), Ie(Fl, [2, 395], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 136: ro, 317: Ji, 321: Ha }), Ie(Fl, [2, 396], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 136: ro, 317: Ji, 321: Ha }), Ie(Fl, [2, 397], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 136: ro, 317: Ji, 321: Ha }), Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 112, 118, 122, 123, 124, 128, 129, 130, 131, 132, 133, 134, 135, 137, 138, 139, 140, 141, 142, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 179, 180, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 316, 318, 319, 320, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 398], { 114: 632, 332: 644, 115: uo, 116: so, 136: ro, 317: Ji, 321: Ha }), Ie(qc, [2, 399], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba }), Ie(qc, [2, 400], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba }), Ie(qc, [2, 401], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba }), Ie(qc, [2, 402], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba }), Ie(Bu, [2, 403], { 77: d1 }), Ie(ls, [2, 404]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1168, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ls, [2, 406]), Ie(Bu, [2, 407], { 77: d1 }), Ie(ls, [2, 408]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1169, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ls, [2, 410]), Ie(ql, [2, 411], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 412], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 413], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 414], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie([2, 4, 5, 10, 53, 72, 89, 99, 124, 139, 140, 146, 154, 156, 170, 171, 189, 271, 272, 294, 311, 315, 325, 326, 327, 328, 329, 330, 331, 335, 336, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 510, 512, 513, 522, 607, 768], jc, { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 416], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 417], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 418], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 419], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(ql, [2, 420], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), { 77: [1, 1170] }, { 77: [2, 455] }, { 77: [2, 456] }, { 77: [2, 457] }, Ie(Dl, [2, 423], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 337: Oo }), Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 107, 118, 122, 128, 129, 130, 131, 132, 134, 135, 137, 143, 145, 146, 148, 149, 150, 152, 156, 162, 164, 166, 168, 169, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 319, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 424], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo }), { 2: ft, 3: 171, 4: pt, 5: wt, 40: 1171, 56: 167, 77: Qr, 78: [1, 1173], 79: 75, 89: Ot, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1172, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 184: 99, 189: gn, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ls, [2, 437]), Ie(ls, [2, 439]), Ie(ls, [2, 446]), Ie(ls, [2, 447]), { 2: ft, 3: 674, 4: pt, 5: wt, 77: [1, 1174] }, { 2: ft, 3: 702, 4: pt, 5: wt, 77: [1, 1175], 111: 943, 145: Et, 156: Gr, 200: 944, 202: 1177, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 334: 1176 }, Ie(ls, [2, 444]), Ie(Dl, [2, 441], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 337: Oo }), Ie(Dl, [2, 442], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 337: Oo }), Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 99, 107, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 189, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 239, 244, 245, 246, 247, 249, 251, 271, 272, 285, 286, 287, 288, 289, 290, 291, 292, 294, 301, 305, 311, 313, 314, 315, 319, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 469, 475, 510, 512, 513, 522, 607, 768], [2, 443], { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io }), Ie(ls, [2, 445]), Ie(ls, [2, 310]), Ie(ls, [2, 311]), Ie(ls, [2, 312]), Ie(ls, [2, 430]), { 74: Yc, 78: [1, 1178] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1179, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1180, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ls, Ml), Ie(pc, [2, 290]), Ie(ls, [2, 286]), { 78: [1, 1182], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1183] }, { 310: 1184, 311: An, 312: 966, 313: J1, 315: On }, { 311: [1, 1185] }, Ie(Bn, [2, 380]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1186, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 314: [1, 1187], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 76: [1, 1188], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 74: [1, 1189] }, Ie(Lu, [2, 649]), { 2: ft, 3: 702, 4: pt, 5: wt, 77: M1, 111: 697, 113: 695, 131: Ur, 132: rr, 143: Rr, 144: 692, 145: vs, 152: Jr, 156: Gr, 181: gr, 196: 694, 200: 700, 201: 699, 261: 696, 262: 698, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 305: [1, 1190], 307: yr, 424: 193, 425: Si, 427: 1191, 428: 693, 429: Ei }, { 78: [1, 1192], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 2: ft, 3: 1193, 4: pt, 5: wt, 149: qt }, Ie(ls, [2, 373]), Ie(Lu, [2, 646]), { 2: ft, 3: 711, 4: pt, 5: wt, 131: ku, 132: Wu, 431: [1, 1194], 433: 1195 }, { 2: ft, 3: 702, 4: pt, 5: wt, 77: M1, 111: 697, 113: 695, 131: Ur, 132: rr, 143: Rr, 144: 692, 145: vs, 152: Jr, 156: Gr, 181: gr, 196: 694, 200: 700, 201: 699, 261: 696, 262: 698, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 307: yr, 424: 193, 425: Si, 427: 1196, 428: 693, 429: Ei }, { 2: ft, 3: 702, 4: pt, 5: wt, 77: M1, 111: 697, 113: 695, 131: Ur, 132: rr, 143: Rr, 144: 692, 145: vs, 152: Jr, 156: Gr, 181: gr, 196: 694, 200: 700, 201: 699, 261: 696, 262: 698, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 307: yr, 424: 193, 425: Si, 427: 1197, 428: 693, 429: Ei }, { 2: ft, 3: 702, 4: pt, 5: wt, 77: M1, 111: 697, 113: 695, 131: Ur, 132: rr, 143: Rr, 144: 692, 145: vs, 152: Jr, 156: Gr, 181: gr, 196: 694, 200: 700, 201: 699, 261: 696, 262: 698, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 307: yr, 424: 193, 425: Si, 427: 1198, 428: 693, 429: Ei }, { 77: Tr, 143: Rr, 144: 988, 145: vs, 152: Jr, 181: gr, 201: 989, 307: yr, 347: 1199, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 467], { 74: xl }), { 149: wr, 346: 1201, 349: Cr }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1205, 100: 1202, 111: 1204, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 350: 1203, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 475]), Ie(Bl, [2, 478]), Ie(Bl, [2, 479]), Ie(al, [2, 483]), Ie(al, [2, 484]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1206 }, { 77: Tr, 143: Rr, 144: 988, 145: vs, 152: Jr, 181: gr, 201: 989, 307: yr, 347: 1207, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 471], { 74: xl }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1205, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 350: 1203, 424: 193, 425: Si, 429: Ei }, { 313: Oc, 461: 1208, 463: 1209, 464: 1210 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1212, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 230: [2, 706] }, { 2: ft, 3: 223, 4: pt, 5: wt, 40: 718, 77: Al, 79: 75, 89: Ot, 132: ul, 143: Rr, 144: 216, 145: Vr, 152: Jr, 156: Gr, 181: gr, 184: 99, 189: gn, 199: 217, 200: 219, 201: 218, 202: 221, 209: 1213, 213: gc, 214: 222, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei }, Ie(kl, [2, 181], { 3: 1214, 2: ft, 4: pt, 5: wt, 76: [1, 1215] }), Ie(kl, [2, 182]), Ie(kl, [2, 1104]), Ie(kl, [2, 184]), Ie(kl, [2, 186]), Ie(kl, [2, 188]), Ie(kl, [2, 192]), Ie(kl, [2, 196]), Ie(kl, [2, 199]), Ie([2, 4, 5, 10, 53, 72, 74, 76, 77, 78, 89, 93, 95, 98, 118, 124, 128, 143, 145, 146, 148, 149, 152, 154, 156, 162, 168, 169, 181, 183, 187, 189, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 249, 251, 271, 272, 294, 302, 307, 311, 315, 340, 343, 344, 348, 349, 361, 373, 374, 378, 379, 401, 405, 406, 407, 408, 409, 411, 413, 414, 422, 423, 425, 429, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 510, 512, 513, 519, 520, 521, 522, 607, 768], [2, 201]), { 2: ft, 3: 1216, 4: pt, 5: wt }, Ie(nu, [2, 1055], { 83: 1217, 92: 1218, 93: [1, 1219], 98: [1, 1220] }), { 2: ft, 3: 223, 4: pt, 5: wt, 77: [1, 1222], 132: ul, 143: Rr, 144: 216, 145: Vr, 152: Jr, 156: Gr, 181: gr, 199: 217, 200: 219, 201: 218, 202: 221, 203: 1221, 209: 1223, 213: gc, 214: 222, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei }, Ie(ic, [2, 164]), Ie(ic, [2, 165]), Ie(ic, [2, 166]), Ie(ic, [2, 167]), Ie(ic, [2, 168]), { 2: ft, 3: 674, 4: pt, 5: wt }, Ie(Gl, [2, 83], { 74: [1, 1224] }), Ie(Ic, [2, 85]), Ie(Ic, [2, 86]), { 113: 1225, 132: rr, 301: Dn }, Ie([10, 72, 74, 78, 93, 98, 118, 124, 128, 162, 168, 169, 183, 198, 206, 208, 222, 223, 224, 225, 226, 227, 228, 229, 232, 249, 251, 311, 315, 607, 768], Ql, { 116: sr }), Ie(Wi, [2, 73]), Ie(Wi, [2, 1060]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1226, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ya, [2, 126]), Ie(ya, [2, 144]), Ie(ya, [2, 145]), Ie(ya, [2, 146]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 78: [2, 1075], 94: 265, 111: 151, 113: 155, 127: 1227, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1228, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 77: [1, 1229] }, Ie(ya, [2, 94]), Ie([2, 4, 5, 10, 72, 74, 76, 77, 78, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 198, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 311, 315, 425, 429, 607, 768], [2, 96], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie([2, 4, 5, 10, 72, 74, 76, 77, 78, 112, 118, 122, 124, 128, 129, 130, 131, 132, 134, 135, 137, 139, 140, 143, 145, 146, 148, 149, 150, 152, 154, 156, 162, 164, 166, 168, 169, 170, 171, 172, 173, 175, 181, 183, 185, 187, 198, 244, 245, 285, 286, 287, 288, 289, 290, 291, 292, 311, 315, 425, 429, 607, 768], [2, 97], { 114: 632, 332: 644, 99: os, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1230], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1231, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, Ie($s, [2, 1071], { 153: 1025, 179: Yi, 180: ra, 181: ha }), { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 112: Pc, 114: 743, 115: uo, 116: so, 117: 1233, 118: du, 122: Ac, 123: yu, 124: au, 126: 1232, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1234, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1235, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1236, 4: pt, 5: wt }, Ie(ya, [2, 110]), Ie(ya, [2, 111]), Ie(ya, [2, 112]), Ie(ya, [2, 119]), { 2: ft, 3: 1237, 4: pt, 5: wt }, { 2: ft, 3: 1021, 4: pt, 5: wt, 111: 1075, 143: va, 145: qa, 147: 1238, 341: 1074, 342: 1076 }, { 2: ft, 3: 1239, 4: pt, 5: wt }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1240, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ya, [2, 125]), Ie($s, [2, 1077], { 155: 1241 }), Ie($s, [2, 1079], { 157: 1242 }), Ie($s, [2, 1081], { 159: 1243 }), Ie($s, [2, 1085], { 161: 1244 }), Ie(Xc, wc, { 163: 1245, 178: 1246 }), { 77: [1, 1247] }, Ie($s, [2, 1087], { 165: 1248 }), Ie($s, [2, 1089], { 167: 1249 }), Ie(Xc, wc, { 178: 1246, 163: 1250 }), Ie(Xc, wc, { 178: 1246, 163: 1251 }), Ie(Xc, wc, { 178: 1246, 163: 1252 }), Ie(Xc, wc, { 178: 1246, 163: 1253 }), { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 112: Pc, 114: 743, 115: uo, 116: so, 117: 1254, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 824, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 174: 1255, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 257: 823, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(zu, [2, 1091], { 176: 1256 }), Ie(hi, [2, 615], { 183: [1, 1257] }), Ie(hi, [2, 611], { 183: [1, 1258] }), Ie(hi, [2, 604]), { 113: 1259, 132: rr, 301: Dn }, Ie(hi, [2, 613], { 183: [1, 1260] }), Ie(hi, [2, 608]), Ie(hi, [2, 609], { 112: [1, 1261] }), Ie(gd, [2, 69]), { 40: 1262, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(hi, [2, 459], { 74: Zc, 128: [1, 1263] }), Ie(Ju, [2, 460]), { 124: [1, 1265] }, { 2: ft, 3: 1266, 4: pt, 5: wt }, Ie(Fa, [2, 1125]), Ie(Fa, [2, 1126]), Ie(hi, [2, 627]), Ie(nc, [2, 364], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(ql, jc, { 114: 632, 332: 644, 112: jo, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 337: Oo }), Ie(Hs, [2, 691]), Ie(Hs, [2, 693]), Ie(hi, [2, 659]), Ie(hi, [2, 661], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1267, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1089, 4: pt, 5: wt, 77: Aa, 131: Xa, 437: 1268 }, Ie(ru, [2, 668]), Ie(ru, [2, 669]), Ie(ru, [2, 670]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1269, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1270, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 114: 1087, 115: uo, 116: so, 124: [1, 1271] }, Ie($l, [2, 765]), Ie($c, [2, 148], { 74: Oa }), Ie($c, [2, 149], { 74: Oa }), Ie($c, [2, 150], { 74: Oa }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 824, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 257: 1272, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1273, 4: pt, 5: wt, 113: 1275, 131: [1, 1274], 132: rr, 301: Dn }, Ie(Ya, [2, 279]), Ie(Ya, [2, 281]), Ie(Ya, [2, 283]), Ie(Nc, [2, 160]), Ie(Nc, [2, 1100]), { 78: [1, 1276] }, Ie(Kc, [2, 768]), { 2: ft, 3: 1277, 4: pt, 5: wt }, { 2: ft, 3: 1278, 4: pt, 5: wt }, { 2: ft, 3: 1280, 4: pt, 5: wt, 389: 1279 }, { 2: ft, 3: 1280, 4: pt, 5: wt, 389: 1281 }, { 2: ft, 3: 1282, 4: pt, 5: wt }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1283, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1284, 4: pt, 5: wt }, { 74: Yc, 78: [1, 1285] }, Ie(dl, [2, 354]), Ie(dl, [2, 355]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1286, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1287, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1288, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1289, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1290, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Ul, [2, 513]), Ie(hi, Bd, { 412: 1291, 76: tf, 77: [1, 1292] }), Ie(hi, Bd, { 412: 1294, 76: tf }), { 77: [1, 1295] }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1296 }, Ie($l, [2, 741]), Ie($l, [2, 743]), Ie($l, [2, 1156]), { 143: yc, 145: gu, 436: 1297 }, Ie(wd, [2, 1157], { 424: 193, 484: 1298, 144: 1299, 145: vs, 425: Si, 429: Ei }), { 76: Wo, 139: [2, 1161], 486: 1300, 488: 1301 }, Ie([10, 74, 76, 78, 132, 139, 145, 152, 311, 315, 425, 429, 607, 768], Iu, { 495: 860, 498: 861, 137: Hl }), Ie($l, [2, 746]), Ie($l, lc), { 74: zs, 78: [1, 1302] }, Ie(w1, [2, 1175], { 497: 1303, 502: 1304, 152: Kl }), Ie(tu, [2, 1174]), Ie(w1, [2, 755]), Ie(w1, [2, 1180]), Ie(hi, [2, 499], { 77: [1, 1305] }), { 76: [1, 1307], 77: [1, 1306] }, { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 148: [1, 1308], 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(hl, od, { 79: 75, 184: 99, 473: 1309, 40: 1312, 89: Ot, 146: yd, 189: gn, 475: Ef }), Ie(Ja, [2, 1150]), Ie(zn, [2, 733]), { 230: [1, 1313] }, Ie(Md, [2, 779]), Ie(Md, [2, 780]), Ie(Md, [2, 781]), Ie(Gn, Ke, { 515: 1314, 95: Fe, 519: Te, 520: Ae, 521: Oe }), Ie(Gn, [2, 778]), Ie(hi, [2, 317]), Ie(hi, [2, 318]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1315, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Hs, [2, 699], { 124: [1, 1316] }), Ie(Is, [2, 550]), { 131: [1, 1318], 393: 1317, 395: [1, 1319] }, Ie(Is, [2, 5]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1205, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 350: 1320, 424: 193, 425: Si, 429: Ei }, Ie(hi, [2, 464], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(hi, [2, 598]), Ie(hi, [2, 599]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1321 }, Ie(hi, [2, 679]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1322, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1323, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 78: [1, 1324], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1325], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 2: ft, 3: 171, 4: pt, 5: wt, 40: 1326, 56: 167, 77: Qr, 79: 75, 89: Ot, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1327, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 184: 99, 189: gn, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 78: [1, 1328] }, { 74: Yc, 78: [1, 1329] }, Ie(ls, [2, 435]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1330, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 40: 1331, 56: 167, 77: Qr, 78: [1, 1333], 79: 75, 89: Ot, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1332, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 184: 99, 189: gn, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ls, [2, 438]), Ie(ls, [2, 440]), Ie(ls, nf, { 280: 1334, 281: Vd }), { 78: [1, 1336], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1337], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 2: ft, 3: 1338, 4: pt, 5: wt, 180: [1, 1339] }, Ie(Lu, [2, 628]), Ie(ls, [2, 372]), { 311: [1, 1340] }, Ie(ls, [2, 379]), { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 311: [2, 383], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1341, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 4: Be, 7: 888, 277: 1342, 392: 887, 394: Ue }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1343, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Lu, [2, 650]), Ie(qn, [2, 657]), Ie(Vn, [2, 645]), Ie(pc, Ml), Ie(Lu, [2, 647]), Ie(Un, [2, 652]), Ie(Un, [2, 654]), Ie(Un, [2, 655]), Ie(Un, [2, 656]), Ie(hl, [2, 466], { 74: xl }), { 77: [1, 1345], 143: Rr, 144: 1346, 145: vs, 152: Jr, 181: gr, 201: 1347, 307: yr, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 472]), { 74: S1, 78: [1, 1348] }, { 74: Af, 78: [1, 1350] }, Ie([74, 78, 99, 112, 115, 116, 123, 124, 133, 136, 138, 139, 140, 141, 142, 154, 170, 171, 179, 180, 316, 317, 318, 320, 321, 322, 323, 324, 325, 326, 327, 328, 329, 330, 331, 335, 336, 337, 338], Pd), Ie(Au, [2, 488], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 40: 1354, 77: Tr, 79: 75, 89: Ot, 143: Rr, 144: 988, 145: vs, 149: wr, 152: Jr, 181: gr, 184: 99, 189: gn, 201: 989, 307: yr, 346: 1352, 347: 1353, 349: Cr, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 470], { 74: xl }), Ie(hi, [2, 727], { 462: 1355, 463: 1356, 464: 1357, 313: Oc, 469: [1, 1358] }), Ie(Vu, [2, 711]), Ie(Vu, [2, 712]), { 154: [1, 1360], 465: [1, 1359] }, { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 313: [2, 708], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1361] }, Ie(kl, [2, 179]), { 2: ft, 3: 1362, 4: pt, 5: wt }, Ie(hi, [2, 583]), Ie(Yh, [2, 239], { 84: 1363, 128: [1, 1364] }), Ie(nu, [2, 1056]), { 77: [1, 1365] }, { 77: [1, 1366] }, Ie(ii, [2, 169], { 204: 1367, 215: 1369, 205: 1370, 216: 1371, 221: 1374, 74: Qf, 206: vd, 208: Ff, 222: Uu, 223: rf, 224: Ud, 225: $d, 226: Sd, 227: Yd, 228: af, 229: of }), { 2: ft, 3: 223, 4: pt, 5: wt, 40: 718, 77: Al, 79: 75, 89: Ot, 132: ul, 143: Rr, 144: 216, 145: Vr, 152: Jr, 156: Gr, 181: gr, 184: 99, 189: gn, 199: 217, 200: 219, 201: 218, 202: 221, 203: 1383, 209: 1223, 213: gc, 214: 222, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei }, Ie(Bf, [2, 177]), { 2: ft, 3: 1021, 4: pt, 5: wt, 110: 1384, 111: 1019, 112: wi }, Ie(Ic, [2, 87]), Ie(Wi, [2, 147], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 78: [1, 1385] }, { 74: Yc, 78: [2, 1076] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 78: [2, 1069], 94: 1390, 111: 151, 113: 155, 120: 1386, 121: 1387, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 241: 1388, 244: $r, 245: li, 246: [1, 1389], 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ya, [2, 98]), Ie($s, [2, 1072], { 153: 1025, 179: Yi, 180: ra, 181: ha }), { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1391], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1392, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, Ie($s, [2, 1073], { 153: 1025, 179: Yi, 180: ra, 181: ha }), { 78: [1, 1393], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1394], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1395] }, Ie(ya, [2, 120]), { 74: Zc, 78: [1, 1396] }, Ie(ya, [2, 122]), { 74: Yc, 78: [1, 1397] }, { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1398], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1399, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1400], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1401, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1402], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1403, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1404], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1405, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 74: qd, 78: [1, 1406] }, Ie(Au, [2, 143], { 424: 193, 3: 740, 114: 743, 144: 765, 158: 775, 160: 776, 117: 1408, 2: ft, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 112: Pc, 115: uo, 116: so, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 425: Si, 429: Ei }), Ie(Xc, wc, { 178: 1246, 163: 1409 }), { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1410], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1411, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 78: [1, 1412], 112: Pc, 114: 743, 115: uo, 116: so, 117: 1413, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 74: qd, 78: [1, 1414] }, { 74: qd, 78: [1, 1415] }, { 74: qd, 78: [1, 1416] }, { 74: qd, 78: [1, 1417] }, { 78: [1, 1418], 153: 1025, 179: Yi, 180: ra, 181: ha }, { 74: Oa, 78: [1, 1419] }, { 2: ft, 3: 740, 4: pt, 5: wt, 72: Zl, 74: [1, 1420], 76: Ec, 77: wu, 112: Pc, 114: 743, 115: uo, 116: so, 117: 1421, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 144: 765, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 158: 775, 160: 776, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1422, 4: pt, 5: wt }, { 2: ft, 3: 1423, 4: pt, 5: wt }, Ie(hi, [2, 606]), { 2: ft, 3: 1424, 4: pt, 5: wt }, { 113: 1425, 132: rr, 301: Dn }, { 78: [1, 1426] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1427, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1021, 4: pt, 5: wt, 111: 1075, 143: va, 145: qa, 341: 1428, 342: 1076 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1429, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 124: [1, 1430] }, Ie(hi, [2, 662], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(ru, [2, 667]), { 78: [1, 1431], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(hi, [2, 663], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1432, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Ya, [2, 276]), Ie(Ya, [2, 278]), Ie(Ya, [2, 280]), Ie(Ya, [2, 282]), Ie(Nc, [2, 161]), Ie(hi, [2, 578]), { 148: [1, 1433] }, Ie(hi, [2, 579]), Ie($l, [2, 545], { 392: 887, 7: 888, 277: 1434, 4: Be, 391: [1, 1435], 394: Ue }), Ie(hi, [2, 580]), Ie(hi, [2, 582]), { 74: Yc, 78: [1, 1436] }, Ie(hi, [2, 586]), Ie(dl, [2, 352]), { 74: [1, 1437], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 74: [1, 1438], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 74: [1, 1439], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 74: [1, 1440], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 74: [1, 1441], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(hi, [2, 590]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1442, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1443, 4: pt, 5: wt }, Ie(hi, [2, 592]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1390, 111: 151, 113: 155, 120: 1444, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 241: 1388, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 77: [1, 1445] }, { 2: ft, 3: 1446, 4: pt, 5: wt }, { 76: Wo, 139: [2, 1159], 485: 1447, 488: 1448 }, Ie(wd, [2, 1158]), { 139: [1, 1449] }, { 139: [2, 1162] }, Ie($l, [2, 747]), Ie(w1, [2, 754]), Ie(w1, [2, 1176]), { 2: ft, 3: 1280, 4: pt, 5: wt, 76: [1, 1452], 356: 1450, 363: 1451, 389: 1453 }, { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1454, 111: 1455 }, { 40: 1456, 79: 75, 89: Ot, 184: 99, 189: gn }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1457, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 732]), { 2: ft, 3: 1021, 4: pt, 5: wt, 111: 1075, 143: va, 145: qa, 147: 1458, 341: 1074, 342: 1076 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1459, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 737]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1460 }, { 340: Uo, 343: ys, 344: il, 516: 1461 }, Ie(Hs, [2, 700], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1462, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 74: [1, 1463], 78: [1, 1464] }, Ie(Au, [2, 552]), Ie(Au, [2, 553]), { 74: Af, 78: [1, 1465] }, Ie(Hs, [2, 574]), Ie(nl, [2, 389], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba, 323: ao, 324: io }), Ie(nl, [2, 391], { 114: 632, 332: 644, 115: uo, 116: so, 123: So, 133: No, 136: ro, 138: Io, 141: zo, 142: Yo, 179: Ri, 180: ta, 317: Ji, 321: Ha, 322: Ba, 323: ao, 324: io }), Ie(ls, [2, 405]), Ie(ls, [2, 409]), { 78: [1, 1466] }, { 74: Yc, 78: [1, 1467] }, Ie(ls, [2, 431]), Ie(ls, [2, 433]), { 78: [1, 1468], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1469] }, { 74: Yc, 78: [1, 1470] }, Ie(ls, [2, 436]), Ie(ls, [2, 333]), { 77: [1, 1471] }, Ie(ls, nf, { 280: 1472, 281: Vd }), Ie(ls, nf, { 280: 1473, 281: Vd }), Ie(pc, [2, 288]), Ie(ls, [2, 285]), Ie(ls, [2, 378]), Ie(Bn, [2, 382], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 74: [1, 1475], 78: [1, 1474] }, { 74: [1, 1477], 78: [1, 1476], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 2: ft, 3: 1338, 4: pt, 5: wt }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1205, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 350: 1478, 424: 193, 425: Si, 429: Ei }, Ie(al, [2, 486]), Ie(al, [2, 487]), { 40: 1481, 77: Tr, 79: 75, 89: Ot, 143: Rr, 144: 988, 145: vs, 149: wr, 152: Jr, 181: gr, 184: 99, 189: gn, 201: 989, 307: yr, 346: 1479, 347: 1480, 349: Cr, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1021, 4: pt, 5: wt, 111: 1482 }, Ie(al, [2, 482]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1483, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 77: Tr, 143: Rr, 144: 988, 145: vs, 152: Jr, 181: gr, 201: 989, 307: yr, 347: 1484, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 469], { 74: xl }), Ie(hl, [2, 476]), Ie(hi, [2, 703]), Ie(Vu, [2, 709]), Ie(Vu, [2, 710]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 824, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 174: 1485, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 257: 823, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 170: [1, 1487], 314: [1, 1486] }, { 465: [1, 1488] }, { 230: [2, 707] }, Ie(kl, [2, 180]), Ie(Cf, [2, 241], { 85: 1489, 232: [1, 1490] }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1491, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1492, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1493, 4: pt, 5: wt }, Ie(ii, [2, 170], { 216: 1371, 221: 1374, 215: 1494, 205: 1495, 206: vd, 208: Ff, 222: Uu, 223: rf, 224: Ud, 225: $d, 226: Sd, 227: Yd, 228: af, 229: of }), { 2: ft, 3: 223, 4: pt, 5: wt, 77: Al, 132: ul, 143: Rr, 144: 216, 145: Vr, 152: Jr, 156: Gr, 181: gr, 199: 217, 200: 219, 201: 218, 202: 221, 209: 1496, 213: gc, 214: 222, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei }, Ie(V1, [2, 205]), Ie(V1, [2, 206]), { 2: ft, 3: 223, 4: pt, 5: wt, 77: [1, 1501], 143: Rr, 144: 1499, 145: Vr, 152: Jr, 156: Gr, 181: gr, 199: 1498, 200: 1502, 201: 1500, 202: 1503, 217: 1497, 270: Ca, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 307: yr, 424: 193, 425: Si, 429: Ei }, { 207: [1, 1504], 223: jd }, { 207: [1, 1506], 223: E1 }, Ie(sd, [2, 222]), { 206: [1, 1510], 208: [1, 1509], 221: 1508, 223: rf, 224: Ud, 225: $d, 226: Sd, 227: Yd, 228: af, 229: of }, Ie(sd, [2, 224]), { 223: [1, 1511] }, { 208: [1, 1513], 223: [1, 1512] }, { 208: [1, 1515], 223: [1, 1514] }, { 208: [1, 1516] }, { 223: [1, 1517] }, { 223: [1, 1518] }, { 74: Qf, 204: 1519, 205: 1370, 206: vd, 208: Ff, 215: 1369, 216: 1371, 221: 1374, 222: Uu, 223: rf, 224: Ud, 225: $d, 226: Sd, 227: Yd, 228: af, 229: of }, Ie(Ic, [2, 84]), Ie(ya, [2, 100]), { 74: Xd, 78: [1, 1520] }, { 78: [1, 1522] }, Ie(ld, [2, 262]), { 78: [2, 1070] }, Ie(ld, [2, 266], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 246: [1, 1523], 247: [1, 1524], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(ya, [2, 99]), Ie($s, [2, 1074], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 101]), Ie(ya, [2, 102]), Ie(ya, [2, 103]), Ie(ya, [2, 121]), Ie(ya, [2, 124]), Ie(ya, [2, 127]), Ie($s, [2, 1078], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 128]), Ie($s, [2, 1080], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 129]), Ie($s, [2, 1082], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 130]), Ie($s, [2, 1086], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 131]), Ie(Xc, [2, 1093], { 177: 1525 }), Ie(Xc, [2, 1096], { 153: 1025, 179: Yi, 180: ra, 181: ha }), { 74: qd, 78: [1, 1526] }, Ie(ya, [2, 133]), Ie($s, [2, 1088], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 134]), Ie($s, [2, 1090], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 135]), Ie(ya, [2, 136]), Ie(ya, [2, 137]), Ie(ya, [2, 138]), Ie(ya, [2, 139]), Ie(ya, [2, 140]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 265, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 151: 1527, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(zu, [2, 1092], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(hi, [2, 616]), Ie(hi, [2, 612]), Ie(hi, [2, 614]), Ie(hi, [2, 610]), Ie(gd, [2, 71]), Ie(hi, [2, 458], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(Ju, [2, 461]), Ie(Ju, [2, 462], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1528, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ru, [2, 671]), Ie(hi, [2, 664], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 1529, 4: pt, 5: wt }, Ie($l, [2, 554], { 390: 1530, 396: 1531, 397: 1532, 371: 1540, 154: Ch, 187: sf, 230: Zd, 302: Lf, 348: uh, 361: Xf, 373: Lh, 374: p1, 378: cd, 379: dh }), Ie($l, [2, 544]), Ie(hi, [2, 585], { 76: [1, 1544] }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1545, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1546, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1547, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1548, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1549, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 74: Yc, 78: [1, 1550] }, Ie(hi, [2, 594]), { 74: Xd, 78: [1, 1551] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1390, 111: 151, 113: 155, 120: 1552, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 241: 1388, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie([10, 74, 78, 139, 311, 315, 607, 768], [2, 751]), { 139: [1, 1553] }, { 139: [2, 1160] }, { 2: ft, 3: 1133, 4: pt, 5: wt, 132: D1, 137: Hl, 143: yc, 145: gu, 152: Kl, 436: 592, 480: 1135, 483: 1554, 487: 589, 498: 586, 502: 588 }, { 78: [1, 1555] }, { 74: [1, 1556], 78: [2, 515] }, { 40: 1557, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(Au, [2, 541]), { 74: S1, 78: [1, 1558] }, Ie(Bf, Pd), Ie(hi, [2, 1143], { 417: 1559, 418: 1560, 72: kh }), Ie(hl, od, { 79: 75, 184: 99, 114: 632, 332: 644, 40: 1312, 473: 1562, 89: Ot, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 146: yd, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 189: gn, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs, 475: Ef }), Ie(hl, [2, 735], { 74: Zc }), Ie(hl, [2, 736], { 74: Yc }), Ie([10, 53, 72, 89, 124, 146, 156, 189, 271, 272, 294, 311, 315, 340, 343, 344, 401, 405, 406, 409, 411, 413, 414, 422, 423, 439, 441, 442, 444, 445, 446, 447, 448, 452, 453, 456, 457, 510, 512, 513, 522, 607, 768], [2, 1191], { 517: 1563, 3: 1564, 2: ft, 4: pt, 5: wt, 76: [1, 1565] }), Ie(Mh, [2, 1193], { 518: 1566, 76: [1, 1567] }), Ie(Hs, [2, 701], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 131: [1, 1568] }, Ie(Is, [2, 547]), Ie(Is, [2, 549]), Ie(ls, [2, 421]), Ie(ls, [2, 422]), Ie(ls, [2, 448]), Ie(ls, [2, 432]), Ie(ls, [2, 434]), { 118: h2, 282: 1569, 283: 1570, 284: [1, 1571] }, Ie(ls, [2, 334]), Ie(ls, [2, 335]), Ie(ls, [2, 321]), { 131: [1, 1573] }, Ie(ls, [2, 323]), { 131: [1, 1574] }, { 74: Af, 78: [1, 1575] }, { 77: Tr, 143: Rr, 144: 988, 145: vs, 152: Jr, 181: gr, 201: 989, 307: yr, 347: 1576, 424: 193, 425: Si, 429: Ei }, Ie(hl, [2, 474], { 74: xl }), Ie(hl, [2, 477]), Ie(Bf, [2, 497]), Ie(Au, [2, 489], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(hl, [2, 468], { 74: xl }), Ie(hi, [2, 728], { 74: Oa, 198: [1, 1577] }), { 340: Rh, 343: Zf, 466: 1578 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1581, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 119: [1, 1583], 170: [1, 1584], 314: [1, 1582] }, Ie(Rd, [2, 260], { 86: 1585, 118: [1, 1586] }), { 119: [1, 1587] }, Ie(Yh, [2, 240], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 95: [1, 1588], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 95: [1, 1589] }, Ie(V1, [2, 203]), Ie(V1, [2, 204]), Ie(Bf, [2, 178]), Ie(V1, [2, 238], { 218: 1590, 230: [1, 1591], 231: [1, 1592] }), Ie(ed, [2, 208], { 3: 1593, 2: ft, 4: pt, 5: wt, 76: [1, 1594] }), Ie(Hf, [2, 1105], { 219: 1595, 76: [1, 1596] }), { 2: ft, 3: 1597, 4: pt, 5: wt, 76: [1, 1598] }, { 40: 1599, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(ed, [2, 216], { 3: 1600, 2: ft, 4: pt, 5: wt, 76: [1, 1601] }), Ie(ed, [2, 219], { 3: 1602, 2: ft, 4: pt, 5: wt, 76: [1, 1603] }), { 77: [1, 1604] }, Ie(sd, [2, 234]), { 77: [1, 1605] }, Ie(sd, [2, 230]), Ie(sd, [2, 223]), { 223: E1 }, { 223: jd }, Ie(sd, [2, 225]), Ie(sd, [2, 226]), { 223: [1, 1606] }, Ie(sd, [2, 228]), { 223: [1, 1607] }, { 223: [1, 1608] }, Ie(sd, [2, 232]), Ie(sd, [2, 233]), { 78: [1, 1609], 205: 1495, 206: vd, 208: Ff, 215: 1494, 216: 1371, 221: 1374, 222: Uu, 223: rf, 224: Ud, 225: $d, 226: Sd, 227: Yd, 228: af, 229: of }, Ie(ya, [2, 91]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1390, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 241: 1610, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ya, [2, 92]), Ie(ld, [2, 267], { 242: 1611, 243: [1, 1612] }), { 248: [1, 1613] }, Ie(Au, [2, 142], { 424: 193, 3: 740, 114: 743, 144: 765, 158: 775, 160: 776, 117: 1614, 2: ft, 4: pt, 5: wt, 72: Zl, 76: Ec, 77: wu, 112: Pc, 115: uo, 116: so, 118: du, 122: Ac, 123: yu, 124: au, 128: Qc, 129: Cc, 130: Ol, 131: uc, 132: cs, 133: Mu, 134: Hu, 135: ou, 136: Gc, 137: Su, 138: Lc, 139: Do, 140: Ru, 141: dc, 142: vl, 143: Eu, 145: fu, 146: ac, 148: bc, 149: kc, 150: fc, 152: oc, 154: ss, 156: su, 162: Tl, 164: sc, 166: hc, 168: Vc, 169: Rc, 170: Zo, 171: Ou, 172: Uc, 173: El, 175: eu, 185: Wc, 187: hu, 244: $r, 245: li, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 425: Si, 429: Ei }), Ie(ya, [2, 132]), { 74: Yc, 78: [1, 1615] }, Ie(Ju, [2, 463], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(hi, [2, 581]), Ie($l, [2, 543]), Ie($l, [2, 555], { 371: 1540, 397: 1616, 154: Ch, 187: sf, 230: Zd, 302: Lf, 348: uh, 361: Xf, 373: Lh, 374: p1, 378: cd, 379: dh }), Ie(Jl, [2, 557]), { 375: [1, 1617] }, { 375: [1, 1618] }, { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1619 }, Ie(Jl, [2, 563], { 77: [1, 1620] }), { 2: ft, 3: 114, 4: pt, 5: wt, 77: [1, 1622], 113: 255, 131: Ur, 132: rr, 143: Rr, 152: Jr, 156: Gr, 181: gr, 196: 254, 200: 1623, 201: 258, 261: 256, 262: 257, 269: Vl, 270: Ll, 279: 1621, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 307: yr }, Ie(Jl, [2, 567]), { 302: [1, 1624] }, Ie(Jl, [2, 569]), Ie(Jl, [2, 570]), { 340: [1, 1625] }, { 77: [1, 1626] }, { 2: ft, 3: 1627, 4: pt, 5: wt }, { 78: [1, 1628], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1629], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1630], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1631], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 78: [1, 1632], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(hi, Bd, { 412: 1633, 76: tf }), Ie(hi, [2, 600]), { 74: Xd, 78: [1, 1634] }, { 2: ft, 3: 1133, 4: pt, 5: wt, 132: D1, 137: Hl, 143: yc, 145: gu, 152: Kl, 436: 592, 480: 1135, 483: 1635, 487: 589, 498: 586, 502: 588 }, Ie($l, [2, 745]), Ie(hi, [2, 502], { 357: 1636, 359: 1637, 360: 1638, 4: Od, 247: Oh, 348: Yu, 361: kf }), Ie(Mf, e1, { 3: 1280, 364: 1643, 389: 1644, 365: 1645, 366: 1646, 2: ft, 4: pt, 5: wt, 372: Id }), { 78: [2, 516] }, { 76: [1, 1648] }, Ie(hi, [2, 618]), Ie(hi, [2, 1144]), { 373: [1, 1650], 419: [1, 1649] }, Ie(hl, [2, 738]), Ie(hi, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 12: 1651, 2: ft, 4: pt, 5: wt, 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), Ie(hi, [2, 772]), Ie(Mh, [2, 1192]), Ie(hi, rt, { 17: 5, 18: 7, 19: 8, 20: 9, 21: 10, 22: 11, 23: 12, 24: 13, 25: 14, 26: 15, 27: 16, 28: 17, 29: 18, 30: 19, 31: 20, 32: 21, 33: 22, 34: 23, 35: 24, 36: 25, 37: 26, 38: 27, 39: 28, 40: 29, 41: 30, 42: 31, 43: 32, 44: 33, 45: 34, 46: 35, 47: 36, 48: 37, 49: 38, 50: 39, 51: 40, 52: 41, 54: 43, 55: 44, 56: 45, 57: 46, 58: 47, 59: 48, 60: 49, 61: 50, 62: 51, 63: 52, 64: 53, 65: 54, 66: 55, 67: 56, 68: 57, 69: 58, 70: 59, 71: 60, 79: 75, 509: 95, 184: 99, 3: 100, 12: 1652, 2: ft, 4: pt, 5: wt, 53: At, 72: Dt, 89: Ot, 124: un, 146: yn, 156: _n, 189: gn, 271: bn, 272: an, 294: Zt, 340: Mn, 343: Ln, 344: In, 401: kn, 405: er, 406: jn, 409: ur, 411: Kn, 413: Jn, 414: Lr, 422: mr, 423: Ar, 439: Yr, 441: jr, 442: Mr, 444: Wr, 445: Bi, 446: gi, 447: Zr, 448: fi, 452: ri, 453: bi, 456: Zi, 457: $i, 510: Br, 512: Pr, 513: ei, 522: Ci }), Ie(Mh, [2, 1194]), { 78: [1, 1653] }, { 78: [1, 1654], 118: h2, 283: 1655 }, { 78: [1, 1656] }, { 119: [1, 1657] }, { 119: [1, 1658] }, { 78: [1, 1659] }, { 78: [1, 1660] }, Ie(al, [2, 485]), Ie(hl, [2, 473], { 74: xl }), { 2: ft, 3: 248, 4: pt, 5: wt, 143: yc, 145: gu, 199: 1662, 436: 1661 }, Ie(Vu, [2, 713]), Ie(Vu, [2, 715]), { 146: [1, 1663] }, { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 314: [1, 1664], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, { 344: cu, 467: 1665 }, { 422: [1, 1668], 468: [1, 1667] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1669, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(pu, [2, 271], { 87: 1670, 249: [1, 1671], 251: [1, 1672] }), { 119: [1, 1673] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1679, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 233: 1674, 235: 1675, 236: gf, 237: lf, 238: yf, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1680, 4: pt, 5: wt }, { 2: ft, 3: 1681, 4: pt, 5: wt }, Ie(V1, [2, 207]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1682, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1021, 4: pt, 5: wt, 77: [1, 1684], 100: 1683, 111: 1455 }, Ie(ed, [2, 209]), { 2: ft, 3: 1685, 4: pt, 5: wt }, Ie(ed, [2, 1107], { 220: 1686, 3: 1687, 2: ft, 4: pt, 5: wt }), Ie(Hf, [2, 1106]), Ie(ed, [2, 212]), { 2: ft, 3: 1688, 4: pt, 5: wt }, { 78: [1, 1689] }, Ie(ed, [2, 217]), { 2: ft, 3: 1690, 4: pt, 5: wt }, Ie(ed, [2, 220]), { 2: ft, 3: 1691, 4: pt, 5: wt }, { 40: 1692, 79: 75, 89: Ot, 184: 99, 189: gn }, { 40: 1693, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(sd, [2, 227]), Ie(sd, [2, 229]), Ie(sd, [2, 231]), Ie(ii, [2, 171]), Ie(ld, [2, 263]), Ie(ld, [2, 268]), { 244: [1, 1694], 245: [1, 1695] }, Ie(ld, [2, 269], { 246: [1, 1696] }), Ie(Xc, [2, 1094], { 153: 1025, 179: Yi, 180: ra, 181: ha }), Ie(ya, [2, 141]), Ie(Jl, [2, 556]), Ie(Jl, [2, 559]), { 379: [1, 1697] }, Ie(Jl, [2, 1137], { 400: 1698, 398: 1699, 77: Ih }), { 131: Ur, 196: 1701 }, Ie(Jl, [2, 564]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1702, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Jl, [2, 566]), Ie(Jl, [2, 568]), { 2: ft, 3: 114, 4: pt, 5: wt, 77: [1, 1704], 113: 255, 131: Ur, 132: rr, 143: Rr, 152: Jr, 156: Gr, 181: gr, 196: 254, 200: 259, 201: 258, 261: 256, 262: 257, 269: Vl, 270: Ll, 279: 1703, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 307: yr }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1705, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(hi, [2, 587]), Ie(dl, [2, 357]), Ie(dl, [2, 358]), Ie(dl, [2, 359]), Ie(dl, [2, 360]), Ie(dl, [2, 361]), Ie(hi, [2, 591]), Ie(hi, [2, 601]), Ie($l, [2, 744]), Ie(hi, [2, 498]), Ie(hi, [2, 503], { 360: 1706, 4: Od, 247: Oh, 348: Yu, 361: kf }), Ie(Kf, [2, 505]), Ie(Kf, [2, 506]), { 124: [1, 1707] }, { 124: [1, 1708] }, { 124: [1, 1709] }, { 74: [1, 1710], 78: [2, 514] }, Ie(Au, [2, 542]), Ie(Au, [2, 517]), { 187: [1, 1718], 193: [1, 1719], 367: 1711, 368: 1712, 369: 1713, 370: 1714, 371: 1715, 373: Lh, 374: [1, 1716], 375: [1, 1720], 378: [1, 1717] }, { 2: ft, 3: 1721, 4: pt, 5: wt }, { 40: 1722, 79: 75, 89: Ot, 184: 99, 189: gn }, { 420: [1, 1723] }, { 421: [1, 1724] }, Ie(hi, [2, 771]), Ie(hi, [2, 773]), Ie(Is, [2, 546]), Ie(ls, [2, 337]), { 78: [1, 1725] }, Ie(ls, [2, 338]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1679, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 233: 1726, 235: 1675, 236: gf, 237: lf, 238: yf, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1390, 111: 151, 113: 155, 120: 1727, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 241: 1388, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(ls, [2, 322]), Ie(ls, [2, 324]), { 2: ft, 3: 1728, 4: pt, 5: wt }, Ie(hi, [2, 730], { 77: [1, 1729] }), { 2: ft, 3: 1021, 4: pt, 5: wt, 111: 1075, 143: va, 145: qa, 147: 1730, 341: 1074, 342: 1076 }, { 340: Rh, 343: Zf, 466: 1731 }, Ie(Vu, [2, 717]), { 77: [1, 1733], 348: [1, 1734], 349: [1, 1732] }, { 170: [1, 1736], 314: [1, 1735] }, { 170: [1, 1738], 314: [1, 1737] }, { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 314: [1, 1739], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(Wi, [2, 251], { 88: 1740, 162: [1, 1741], 168: [1, 1743], 169: [1, 1742] }), { 131: Ur, 196: 1744 }, { 131: Ur, 196: 1745 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1390, 111: 151, 113: 155, 120: 1746, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 241: 1388, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, Ie(Cf, [2, 249], { 234: 1747, 74: zf, 239: [1, 1749] }), Ie(cf, [2, 243]), { 146: [1, 1750] }, { 77: [1, 1751] }, { 77: [1, 1752] }, Ie(cf, [2, 248], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 78: [2, 1061], 96: 1753, 99: [1, 1755], 102: 1754 }, { 99: [1, 1756] }, Ie(V1, [2, 235], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), Ie(V1, [2, 236], { 74: S1 }), { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1757, 111: 1455 }, Ie(ed, [2, 210]), Ie(ed, [2, 211]), Ie(ed, [2, 1108]), Ie(ed, [2, 213]), { 2: ft, 3: 1758, 4: pt, 5: wt, 76: [1, 1759] }, Ie(ed, [2, 218]), Ie(ed, [2, 221]), { 78: [1, 1760] }, { 78: [1, 1761] }, Ie(ld, [2, 264]), Ie(ld, [2, 265]), Ie(ld, [2, 270]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1762 }, Ie(Jl, [2, 561]), Ie(Jl, [2, 1138]), { 2: ft, 3: 1763, 4: pt, 5: wt }, { 74: [1, 1764] }, { 78: [1, 1765], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(Jl, [2, 571]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1766, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 78: [1, 1767], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(Kf, [2, 504]), { 2: ft, 3: 1768, 4: pt, 5: wt }, { 131: Ur, 196: 1769 }, { 2: ft, 3: 1770, 4: pt, 5: wt }, Ie(Mf, e1, { 366: 1646, 365: 1771, 372: Id }), Ie($l, [2, 519]), Ie($l, [2, 520]), Ie($l, [2, 521]), Ie($l, [2, 522]), Ie($l, [2, 523]), { 375: [1, 1772] }, { 375: [1, 1773] }, Ie(Xh, [2, 1131], { 387: 1774, 375: [1, 1775] }), { 2: ft, 3: 1776, 4: pt, 5: wt }, { 2: ft, 3: 1777, 4: pt, 5: wt }, Ie(Mf, [2, 525]), Ie(hi, [2, 1141], { 416: 1778, 418: 1779, 72: kh }), Ie(hi, [2, 619]), Ie(hi, [2, 620], { 372: [1, 1780] }), Ie(ls, [2, 339]), Ie([78, 118], [2, 340], { 74: zf }), { 74: Xd, 78: [2, 341] }, Ie(hi, [2, 729]), { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1781, 111: 1455 }, Ie(Vu, [2, 716], { 74: Zc }), Ie(Vu, [2, 714]), { 77: Tr, 143: Rr, 144: 988, 145: vs, 152: Jr, 181: gr, 201: 989, 307: yr, 347: 1782, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1783, 111: 1455 }, { 349: [1, 1784] }, { 344: cu, 467: 1785 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1786, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 344: cu, 467: 1787 }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1788, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 344: cu, 467: 1789 }, Ie(Wi, [2, 72]), { 40: 1790, 79: 75, 89: Ot, 164: [1, 1791], 184: 99, 189: gn, 240: [1, 1792] }, { 40: 1793, 79: 75, 89: Ot, 184: 99, 189: gn, 240: [1, 1794] }, { 40: 1795, 79: 75, 89: Ot, 184: 99, 189: gn, 240: [1, 1796] }, Ie(pu, [2, 274], { 250: 1797, 251: [1, 1798] }), { 252: 1799, 253: [2, 1109], 770: [1, 1800] }, Ie(Rd, [2, 261], { 74: Xd }), Ie(Cf, [2, 242]), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1679, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 235: 1801, 236: gf, 237: lf, 238: yf, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1802, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 77: [1, 1803] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1679, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 233: 1804, 235: 1675, 236: gf, 237: lf, 238: yf, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1679, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 233: 1805, 235: 1675, 236: gf, 237: lf, 238: yf, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 78: [1, 1806] }, { 78: [2, 1062] }, { 77: [1, 1807] }, { 77: [1, 1808] }, { 74: S1, 78: [1, 1809] }, Ie(ed, [2, 214]), { 2: ft, 3: 1810, 4: pt, 5: wt }, { 2: ft, 3: 1811, 4: pt, 5: wt, 76: [1, 1812] }, { 2: ft, 3: 1813, 4: pt, 5: wt, 76: [1, 1814] }, Ie(Jl, [2, 1135], { 399: 1815, 398: 1816, 77: Ih }), { 78: [1, 1817] }, { 131: Ur, 196: 1818 }, Ie(Jl, [2, 565]), { 78: [1, 1819], 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(Jl, [2, 526]), Ie(Kf, [2, 507]), Ie(Kf, [2, 508]), Ie(Kf, [2, 509]), Ie(Au, [2, 518]), { 2: ft, 3: 1821, 4: pt, 5: wt, 77: [2, 1127], 376: 1820 }, { 77: [1, 1822] }, { 2: ft, 3: 1824, 4: pt, 5: wt, 77: [2, 1133], 388: 1823 }, Ie(Xh, [2, 1132]), { 77: [1, 1825] }, { 77: [1, 1826] }, Ie(hi, [2, 617]), Ie(hi, [2, 1142]), Ie(Mf, e1, { 366: 1646, 365: 1827, 372: Id }), { 74: S1, 78: [1, 1828] }, Ie(Vu, [2, 723], { 74: xl }), { 74: S1, 78: [1, 1829] }, Ie(Vu, [2, 725]), Ie(Vu, [2, 718]), { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 314: [1, 1830], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(Vu, [2, 721]), { 99: os, 112: jo, 114: 632, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 314: [1, 1831], 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 332: 644, 335: Ts, 336: Fo, 337: Oo, 338: gs }, Ie(Vu, [2, 719]), Ie(Wi, [2, 252]), { 40: 1832, 79: 75, 89: Ot, 184: 99, 189: gn, 240: [1, 1833] }, { 40: 1834, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(Wi, [2, 254]), { 40: 1835, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(Wi, [2, 255]), { 40: 1836, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(pu, [2, 272]), { 131: Ur, 196: 1837 }, { 253: [1, 1838] }, { 253: [2, 1110] }, Ie(cf, [2, 244]), Ie(Cf, [2, 250], { 114: 632, 332: 644, 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1679, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 233: 1839, 235: 1675, 236: gf, 237: lf, 238: yf, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 74: zf, 78: [1, 1840] }, { 74: zf, 78: [1, 1841] }, Ie(nu, [2, 1063], { 97: 1842, 104: 1843, 3: 1845, 2: ft, 4: pt, 5: wt, 76: P2 }), { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1848, 103: 1846, 105: 1847, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1849, 111: 1455 }, Ie(V1, [2, 237]), Ie(ed, [2, 215]), Ie(V1, [2, 173]), { 2: ft, 3: 1850, 4: pt, 5: wt }, Ie(V1, [2, 175]), { 2: ft, 3: 1851, 4: pt, 5: wt }, Ie(Jl, [2, 560]), Ie(Jl, [2, 1136]), Ie(Jl, [2, 558]), { 78: [1, 1852] }, Ie(Jl, [2, 572]), { 77: [1, 1853] }, { 77: [2, 1128] }, { 2: ft, 3: 1855, 4: pt, 5: wt, 132: Jf, 377: 1854 }, { 77: [1, 1857] }, { 77: [2, 1134] }, { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1858, 111: 1455 }, { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1859, 111: 1455 }, Ie(hi, [2, 621]), Ie(hi, [2, 731]), { 348: [1, 1861], 349: [1, 1860] }, { 344: cu, 467: 1862 }, { 340: Rh, 343: Zf, 466: 1863 }, Ie(Wi, [2, 253]), { 40: 1864, 79: 75, 89: Ot, 184: 99, 189: gn }, Ie(Wi, [2, 256]), Ie(Wi, [2, 258]), Ie(Wi, [2, 259]), Ie(pu, [2, 275]), { 131: [2, 1111], 254: 1865, 650: [1, 1866] }, { 74: zf, 78: [1, 1867] }, Ie(cf, [2, 246]), Ie(cf, [2, 247]), Ie(nu, [2, 74]), Ie(nu, [2, 1064]), { 2: ft, 3: 1868, 4: pt, 5: wt }, Ie(nu, [2, 78]), { 74: [1, 1870], 78: [1, 1869] }, Ie(Au, [2, 80]), Ie(Au, [2, 81], { 114: 632, 332: 644, 76: [1, 1871], 99: os, 112: jo, 115: uo, 116: so, 123: So, 124: Ko, 133: No, 136: ro, 138: Io, 139: bs, 140: ws, 141: zo, 142: Yo, 154: Ls, 170: Ms, 171: vi, 179: Ri, 180: ta, 316: da, 317: Ji, 318: Pa, 320: Ga, 321: Ha, 322: Ba, 323: ao, 324: io, 325: To, 326: Mo, 327: po, 328: Vo, 329: Xo, 330: Go, 331: es, 335: Ts, 336: Fo, 337: Oo, 338: gs }), { 74: S1, 78: [1, 1872] }, Ie(V1, [2, 174]), Ie(V1, [2, 176]), Ie(Jl, [2, 562]), { 2: ft, 3: 1855, 4: pt, 5: wt, 132: Jf, 377: 1873 }, { 74: eh, 78: [1, 1874] }, Ie(Au, [2, 537]), Ie(Au, [2, 538]), { 2: ft, 3: 1021, 4: pt, 5: wt, 100: 1876, 111: 1455 }, { 74: S1, 78: [1, 1877] }, { 74: S1, 78: [1, 1878] }, { 77: Tr, 143: Rr, 144: 988, 145: vs, 152: Jr, 181: gr, 201: 989, 307: yr, 347: 1879, 424: 193, 425: Si, 429: Ei }, { 349: [1, 1880] }, Ie(Vu, [2, 720]), Ie(Vu, [2, 722]), Ie(Wi, [2, 257]), { 131: Ur, 196: 1881 }, { 131: [2, 1112] }, Ie(cf, [2, 245]), Ie(nu, [2, 77]), { 78: [2, 76] }, { 2: ft, 3: 171, 4: pt, 5: wt, 56: 167, 77: Qr, 94: 1848, 105: 1882, 111: 151, 113: 155, 131: Ur, 132: rr, 137: vr, 143: Rr, 144: 163, 145: Vr, 149: Xr, 152: Jr, 154: ti, 156: Gr, 158: 170, 179: Hr, 180: Er, 181: gr, 196: 153, 200: 149, 201: 157, 202: 158, 244: $r, 245: li, 258: 152, 259: 148, 260: 150, 261: 154, 262: 156, 263: 159, 264: 160, 265: 161, 266: 164, 267: 165, 269: Qi, 270: Ai, 271: bn, 275: Li, 276: Ni, 278: Pi, 285: Oi, 286: ki, 287: Mi, 288: Vi, 289: ua, 290: Ki, 291: Fi, 292: na, 294: Sr, 295: Dr, 296: si, 297: zr, 298: di, 299: nn, 300: Tn, 301: Dn, 302: Fn, 303: fr, 304: dr, 307: yr, 308: ai, 317: Fr, 322: Ti, 424: 193, 425: Si, 429: Ei }, { 2: ft, 3: 1883, 4: pt, 5: wt }, { 78: [1, 1884] }, { 74: eh, 78: [1, 1885] }, { 379: [1, 1886] }, { 2: ft, 3: 1887, 4: pt, 5: wt, 132: [1, 1888] }, { 74: S1, 78: [1, 1889] }, Ie($l, [2, 535]), Ie($l, [2, 536]), Ie(Vu, [2, 724], { 74: xl }), Ie(Vu, [2, 726]), Ie(td, [2, 1113], { 255: 1890, 770: [1, 1891] }), Ie(Au, [2, 79]), Ie(Au, [2, 82]), Ie(nu, [2, 1065], { 3: 1845, 101: 1892, 104: 1893, 2: ft, 4: pt, 5: wt, 76: P2 }), Ie($l, [2, 527]), { 2: ft, 3: 248, 4: pt, 5: wt, 199: 1894 }, Ie(Au, [2, 539]), Ie(Au, [2, 540]), Ie($l, [2, 534]), Ie(pu, [2, 1115], { 256: 1895, 420: [1, 1896] }), Ie(td, [2, 1114]), Ie(nu, [2, 75]), Ie(nu, [2, 1066]), Ie(Rf, [2, 1129], { 380: 1897, 382: 1898, 77: [1, 1899] }), Ie(pu, [2, 273]), Ie(pu, [2, 1116]), Ie($l, [2, 530], { 381: 1900, 383: 1901, 230: [1, 1902] }), Ie(Rf, [2, 1130]), { 2: ft, 3: 1855, 4: pt, 5: wt, 132: Jf, 377: 1903 }, Ie($l, [2, 528]), { 230: [1, 1905], 384: 1904 }, { 343: [1, 1906] }, { 74: eh, 78: [1, 1907] }, Ie($l, [2, 531]), { 340: [1, 1908] }, { 385: [1, 1909] }, Ie(Rf, [2, 529]), { 385: [1, 1910] }, { 386: [1, 1911] }, { 386: [1, 1912] }, { 230: [2, 532] }, Ie($l, [2, 533])], defaultActions: { 105: [2, 6], 197: [2, 342], 198: [2, 343], 199: [2, 344], 200: [2, 345], 201: [2, 346], 202: [2, 347], 203: [2, 348], 204: [2, 349], 205: [2, 350], 206: [2, 351], 213: [2, 704], 598: [2, 1152], 660: [2, 1117], 661: [2, 1118], 717: [2, 705], 787: [2, 1083], 788: [2, 1084], 935: [2, 455], 936: [2, 456], 937: [2, 457], 996: [2, 706], 1301: [2, 1162], 1361: [2, 707], 1389: [2, 1070], 1448: [2, 1160], 1557: [2, 516], 1754: [2, 1062], 1800: [2, 1110], 1821: [2, 1128], 1824: [2, 1134], 1866: [2, 1112], 1869: [2, 76], 1911: [2, 532] }, parseError: function(bd, pr) {
            if (pr.recoverable) this.trace(bd);
            else {
              var oi = new Error(bd);
              throw oi.hash = pr, oi;
            }
          }, parse: function(bd) {
            var pr = this, oi = [0], ar = [null], Sa = [], Qn = this.table, za = "", En = 0, Qo = 0, lo = 0, Eo = 2, bo = 1, Ho = Sa.slice.call(arguments, 1), Co = Object.create(this.lexer), Ao = { yy: {} };
            for (var Ro in this.yy) Object.prototype.hasOwnProperty.call(this.yy, Ro) && (Ao.yy[Ro] = this.yy[Ro]);
            Co.setInput(bd, Ao.yy), Ao.yy.lexer = Co, Ao.yy.parser = this, typeof Co.yylloc > "u" && (Co.yylloc = {});
            var Qa = Co.yylloc;
            Sa.push(Qa);
            var Ka = Co.options && Co.options.ranges;
            typeof Ao.yy.parseError == "function" ? this.parseError = Ao.yy.parseError : this.parseError = Object.getPrototypeOf(this).parseError;
            function qo(ns) {
              oi.length = oi.length - 2 * ns, ar.length = ar.length - ns, Sa.length = Sa.length - ns;
            }
            for (var Ss = function() {
              var ns;
              return ns = Co.lex() || bo, typeof ns != "number" && (ns = pr.symbols_[ns] || ns), ns;
            }, ho, hs, Bo, Vs, vo, go, Qs = {}, us, Po, Jo, ps; ; ) {
              if (Bo = oi[oi.length - 1], this.defaultActions[Bo] ? Vs = this.defaultActions[Bo] : ((ho === null || typeof ho > "u") && (ho = Ss()), Vs = Qn[Bo] && Qn[Bo][ho]), typeof Vs > "u" || !Vs.length || !Vs[0]) {
                var wl, ol = "", xs = function(ns) {
                  for (var ll = oi.length - 1, Ws = 0; ; ) {
                    if (Eo.toString() in Qn[ns]) return Ws;
                    if (ns === 0 || ll < 2) return !1;
                    ll -= 2, ns = oi[ll], ++Ws;
                  }
                };
                if (lo) hs !== bo && (wl = xs(Bo));
                else {
                  wl = xs(Bo), ps = [];
                  for (us in Qn[Bo]) this.terminals_[us] && us > Eo && ps.push("'" + this.terminals_[us] + "'");
                  Co.showPosition ? ol = "Parse error on line " + (En + 1) + `:
` + Co.showPosition() + `
Expecting ` + ps.join(", ") + ", got '" + (this.terminals_[ho] || ho) + "'" : ol = "Parse error on line " + (En + 1) + ": Unexpected " + (ho == bo ? "end of input" : "'" + (this.terminals_[ho] || ho) + "'"), this.parseError(ol, { text: Co.match, token: this.terminals_[ho] || ho, line: Co.yylineno, loc: Qa, expected: ps, recoverable: wl !== !1 });
                }
                if (lo == 3) {
                  if (ho === bo || hs === bo) throw new Error(ol || "Parsing halted while starting to recover from another error.");
                  Qo = Co.yyleng, za = Co.yytext, En = Co.yylineno, Qa = Co.yylloc, ho = Ss();
                }
                if (wl === !1) throw new Error(ol || "Parsing halted. No suitable error recovery rule available.");
                qo(wl), hs = ho == Eo ? null : ho, ho = Eo, Bo = oi[oi.length - 1], Vs = Qn[Bo] && Qn[Bo][Eo], lo = 3;
              }
              if (Vs[0] instanceof Array && Vs.length > 1) throw new Error("Parse Error: multiple actions possible at state: " + Bo + ", token: " + ho);
              switch (Vs[0]) {
                case 1:
                  oi.push(ho), ar.push(Co.yytext), Sa.push(Co.yylloc), oi.push(Vs[1]), ho = null, hs ? (ho = hs, hs = null) : (Qo = Co.yyleng, za = Co.yytext, En = Co.yylineno, Qa = Co.yylloc, lo > 0 && lo--);
                  break;
                case 2:
                  if (Po = this.productions_[Vs[1]][1], Qs.$ = ar[ar.length - Po], Qs._$ = { first_line: Sa[Sa.length - (Po || 1)].first_line, last_line: Sa[Sa.length - 1].last_line, first_column: Sa[Sa.length - (Po || 1)].first_column, last_column: Sa[Sa.length - 1].last_column }, Ka && (Qs._$.range = [Sa[Sa.length - (Po || 1)].range[0], Sa[Sa.length - 1].range[1]]), go = this.performAction.apply(Qs, [za, Qo, En, Ao.yy, Vs[1], ar, Sa].concat(Ho)), typeof go < "u") return go;
                  Po && (oi = oi.slice(0, -1 * Po * 2), ar = ar.slice(0, -1 * Po), Sa = Sa.slice(0, -1 * Po)), oi.push(this.productions_[Vs[1]][0]), ar.push(Qs.$), Sa.push(Qs._$), Jo = Qn[oi[oi.length - 2]][oi[oi.length - 1]], oi.push(Jo);
                  break;
                case 3:
                  return !0;
              }
            }
            return !0;
          } }, $2 = ["A", "ABSENT", "ABSOLUTE", "ACCORDING", "ACTION", "ADA", "ADD", "ADMIN", "AFTER", "ALWAYS", "ASC", "ASSERTION", "ASSIGNMENT", "ATTRIBUTE", "ATTRIBUTES", "BASE64", "BEFORE", "BERNOULLI", "BLOCKED", "BOM", "BREADTH", "C", "CASCADE", "CATALOG", "CATALOG_NAME", "CHAIN", "CHARACTERISTICS", "CHARACTERS", "CHARACTER_SET_CATALOG", "CHARACTER_SET_NAME", "CHARACTER_SET_SCHEMA", "CLASS_ORIGIN", "COBOL", "COLLATION", "COLLATION_CATALOG", "COLLATION_NAME", "COLLATION_SCHEMA", "COLUMNS", "COLUMN_NAME", "COMMAND_FUNCTION", "COMMAND_FUNCTION_CODE", "COMMITTED", "CONDITION_NUMBER", "CONNECTION", "CONNECTION_NAME", "CONSTRAINTS", "CONSTRAINT_CATALOG", "CONSTRAINT_NAME", "CONSTRAINT_SCHEMA", "CONSTRUCTOR", "CONTENT", "CONTINUE", "CONTROL", "CURSOR_NAME", "DATA", "DATETIME_INTERVAL_CODE", "DATETIME_INTERVAL_PRECISION", "DB", "DEFAULTS", "DEFERRABLE", "DEFERRED", "DEFINED", "DEFINER", "DEGREE", "DEPTH", "DERIVED", "DESC", "DESCRIPTOR", "DIAGNOSTICS", "DISPATCH", "DOCUMENT", "DOMAIN", "DYNAMIC_FUNCTION", "DYNAMIC_FUNCTION_CODE", "EMPTY", "ENCODING", "ENFORCED", "EXCLUDE", "EXCLUDING", "EXPRESSION", "FILE", "FINAL", "FIRST", "FLAG", "FOLLOWING", "FORTRAN", "FOUND", "FS", "G", "GENERAL", "GENERATED", "GO", "GOTO", "GRANTED", "HEX", "HIERARCHY", "ID", "IGNORE", "IMMEDIATE", "IMMEDIATELY", "IMPLEMENTATION", "INCLUDING", "INCREMENT", "INDENT", "INITIALLY", "INPUT", "INSTANCE", "INSTANTIABLE", "INSTEAD", "INTEGRITY", "INVOKER", "ISOLATION", "K", "KEY", "KEY_MEMBER", "KEY_TYPE", "LAST", "LENGTH", "LEVEL", "LIBRARY", "LIMIT", "LINK", "LOCATION", "LOCATOR", "M", "MAP", "MAPPING", "MATCHED", "MAXVALUE", "MESSAGE_LENGTH", "MESSAGE_OCTET_LENGTH", "MESSAGE_TEXT", "MINVALUE", "MORE", "MUMPS", "NAME", "NAMES", "NAMESPACE", "NESTING", "NEXT", "NFC", "NFD", "NFKC", "NFKD", "NIL", "NORMALIZED", "NULLABLE", "NULLS", "NUMBER", "OBJECT", "OCTETS", "OFF", "OPTION", "OPTIONS", "ORDERING", "ORDINALITY", "OTHERS", "OUTPUT", "OVERRIDING", "P", "PAD", "PARAMETER_MODE", "PARAMETER_NAME", "PARAMETER_ORDINAL_POSITION", "PARAMETER_SPECIFIC_CATALOG", "PARAMETER_SPECIFIC_NAME", "PARAMETER_SPECIFIC_SCHEMA", "PARTIAL", "PASCAL", "PASSING", "PASSTHROUGH", "PATH", "PERMISSION", "PLACING", "PLI", "PRECEDING", "PRESERVE", "PRIOR", "PRIVILEGES", "PUBLIC", "READ", "RECOVERY", "RELATIVE", "REPEATABLE", "REQUIRING", "RESPECT", "RESTART", "RESTORE", "RESTRICT", "RETURNED_CARDINALITY", "RETURNED_LENGTH", "RETURNED_OCTET_LENGTH", "RETURNED_SQLSTATE", "RETURNING", "ROLE", "ROUTINE", "ROUTINE_CATALOG", "ROUTINE_NAME", "ROUTINE_SCHEMA", "ROW_COUNT", "SCALE", "SCHEMA", "SCHEMA_NAME", "SCOPE_CATALOG", "SCOPE_NAME", "SCOPE_SCHEMA", "SECTION", "SECURITY", "SELECTIVE", "SELF", "SEQUENCE", "SERIALIZABLE", "SERVER", "SERVER_NAME", "SESSION", "SETS", "SIMPLE", "SIZE", "SOURCE", "SPACE", "SPECIFIC_NAME", "STANDALONE", "STATE", "STATEMENT", "STRIP", "STRUCTURE", "STYLE", "SUBCLASS_ORIGIN", "T", "TABLE_NAME", "TEMPORARY", "TIES", "TOKEN", "TOP_LEVEL_COUNT", "TRANSACTION", "TRANSACTIONS_COMMITTED", "TRANSACTIONS_ROLLED_BACK", "TRANSACTION_ACTIVE", "TRANSFORM", "TRANSFORMS", "TRIGGER_CATALOG", "TRIGGER_NAME", "TRIGGER_SCHEMA", "TYPE", "UNBOUNDED", "UNCOMMITTED", "UNDER", "UNLINK", "UNNAMED", "UNTYPED", "URI", "USAGE", "USER_DEFINED_TYPE_CATALOG", "USER_DEFINED_TYPE_CODE", "USER_DEFINED_TYPE_NAME", "USER_DEFINED_TYPE_SCHEMA", "VALID", "VERSION", "VIEW", "WHITESPACE", "WORK", "WRAPPER", "WRITE", "XMLDECLARATION", "XMLSCHEMA", "YES", "ZONE"];
          fh.parseError = function(bd, pr) {
            if (!(pr.expected && pr.expected.indexOf("'LITERAL'") > -1 && /[a-zA-Z_][a-zA-Z_0-9]*/.test(pr.token) && $2.indexOf(pr.token) > -1)) throw new SyntaxError(bd);
          };
          var F2 = /* @__PURE__ */ function() {
            var bd = { EOF: 1, parseError: function(pr, oi) {
              if (this.yy.parser) this.yy.parser.parseError(pr, oi);
              else throw new Error(pr);
            }, setInput: function(pr, oi) {
              return this.yy = oi || this.yy || {}, this._input = pr, this._more = this._backtrack = this.done = !1, this.yylineno = this.yyleng = 0, this.yytext = this.matched = this.match = "", this.conditionStack = ["INITIAL"], this.yylloc = { first_line: 1, first_column: 0, last_line: 1, last_column: 0 }, this.options.ranges && (this.yylloc.range = [0, 0]), this.offset = 0, this;
            }, input: function() {
              var pr = this._input[0];
              this.yytext += pr, this.yyleng++, this.offset++, this.match += pr, this.matched += pr;
              var oi = pr.match(/(?:\r\n?|\n).*/g);
              return oi ? (this.yylineno++, this.yylloc.last_line++) : this.yylloc.last_column++, this.options.ranges && this.yylloc.range[1]++, this._input = this._input.slice(1), pr;
            }, unput: function(pr) {
              var oi = pr.length, ar = pr.split(/(?:\r\n?|\n)/g);
              this._input = pr + this._input, this.yytext = this.yytext.substr(0, this.yytext.length - oi), this.offset -= oi;
              var Sa = this.match.split(/(?:\r\n?|\n)/g);
              this.match = this.match.substr(0, this.match.length - 1), this.matched = this.matched.substr(0, this.matched.length - 1), ar.length - 1 && (this.yylineno -= ar.length - 1);
              var Qn = this.yylloc.range;
              return this.yylloc = { first_line: this.yylloc.first_line, last_line: this.yylineno + 1, first_column: this.yylloc.first_column, last_column: ar ? (ar.length === Sa.length ? this.yylloc.first_column : 0) + Sa[Sa.length - ar.length].length - ar[0].length : this.yylloc.first_column - oi }, this.options.ranges && (this.yylloc.range = [Qn[0], Qn[0] + this.yyleng - oi]), this.yyleng = this.yytext.length, this;
            }, more: function() {
              return this._more = !0, this;
            }, reject: function() {
              if (this.options.backtrack_lexer) this._backtrack = !0;
              else return this.parseError("Lexical error on line " + (this.yylineno + 1) + `. You can only invoke reject() in the lexer when the lexer is of the backtracking persuasion (options.backtrack_lexer = true).
` + this.showPosition(), { text: "", token: null, line: this.yylineno });
              return this;
            }, less: function(pr) {
              this.unput(this.match.slice(pr));
            }, pastInput: function() {
              var pr = this.matched.substr(0, this.matched.length - this.match.length);
              return (pr.length > 20 ? "..." : "") + pr.substr(-20).replace(/\n/g, "");
            }, upcomingInput: function() {
              var pr = this.match;
              return pr.length < 20 && (pr += this._input.substr(0, 20 - pr.length)), (pr.substr(0, 20) + (pr.length > 20 ? "..." : "")).replace(/\n/g, "");
            }, showPosition: function() {
              var pr = this.pastInput(), oi = new Array(pr.length + 1).join("-");
              return pr + this.upcomingInput() + `
` + oi + "^";
            }, test_match: function(pr, oi) {
              var ar, Sa, Qn;
              if (this.options.backtrack_lexer && (Qn = { yylineno: this.yylineno, yylloc: { first_line: this.yylloc.first_line, last_line: this.last_line, first_column: this.yylloc.first_column, last_column: this.yylloc.last_column }, yytext: this.yytext, match: this.match, matches: this.matches, matched: this.matched, yyleng: this.yyleng, offset: this.offset, _more: this._more, _input: this._input, yy: this.yy, conditionStack: this.conditionStack.slice(0), done: this.done }, this.options.ranges && (Qn.yylloc.range = this.yylloc.range.slice(0))), Sa = pr[0].match(/(?:\r\n?|\n).*/g), Sa && (this.yylineno += Sa.length), this.yylloc = { first_line: this.yylloc.last_line, last_line: this.yylineno + 1, first_column: this.yylloc.last_column, last_column: Sa ? Sa[Sa.length - 1].length - Sa[Sa.length - 1].match(/\r?\n?/)[0].length : this.yylloc.last_column + pr[0].length }, this.yytext += pr[0], this.match += pr[0], this.matches = pr, this.yyleng = this.yytext.length, this.options.ranges && (this.yylloc.range = [this.offset, this.offset += this.yyleng]), this._more = !1, this._backtrack = !1, this._input = this._input.slice(pr[0].length), this.matched += pr[0], ar = this.performAction.call(this, this.yy, this, oi, this.conditionStack[this.conditionStack.length - 1]), this.done && this._input && (this.done = !1), ar) return ar;
              if (this._backtrack) {
                for (var za in Qn) this[za] = Qn[za];
                return !1;
              }
              return !1;
            }, next: function() {
              if (this.done) return this.EOF;
              this._input || (this.done = !0);
              var pr, oi, ar, Sa;
              this._more || (this.yytext = "", this.match = "");
              for (var Qn = this._currentRules(), za = 0; za < Qn.length; za++) if (ar = this._input.match(this.rules[Qn[za]]), ar && (!oi || ar[0].length > oi[0].length)) {
                if (oi = ar, Sa = za, this.options.backtrack_lexer) {
                  if (pr = this.test_match(ar, Qn[za]), pr !== !1) return pr;
                  if (this._backtrack) {
                    oi = !1;
                    continue;
                  } else return !1;
                } else if (!this.options.flex) break;
              }
              return oi ? (pr = this.test_match(oi, Qn[Sa]), pr !== !1 ? pr : !1) : this._input === "" ? this.EOF : this.parseError("Lexical error on line " + (this.yylineno + 1) + `. Unrecognized text.
` + this.showPosition(), { text: "", token: null, line: this.yylineno });
            }, lex: function() {
              var pr = this.next();
              return pr || this.lex();
            }, begin: function(pr) {
              this.conditionStack.push(pr);
            }, popState: function() {
              var pr = this.conditionStack.length - 1;
              return pr > 0 ? this.conditionStack.pop() : this.conditionStack[0];
            }, _currentRules: function() {
              return this.conditionStack.length && this.conditionStack[this.conditionStack.length - 1] ? this.conditions[this.conditionStack[this.conditionStack.length - 1]].rules : this.conditions.INITIAL.rules;
            }, topState: function(pr) {
              return pr = this.conditionStack.length - 1 - Math.abs(pr || 0), pr >= 0 ? this.conditionStack[pr] : "INITIAL";
            }, pushState: function(pr) {
              this.begin(pr);
            }, stateStackSize: function() {
              return this.conditionStack.length;
            }, options: { "case-insensitive": !0 }, performAction: function(pr, oi, ar, Sa) {
              switch (ar) {
                case 0:
                  return 271;
                case 1:
                  return 307;
                case 2:
                  return 425;
                case 3:
                  return 304;
                case 4:
                  return 5;
                case 5:
                  return 5;
                case 6:
                  return 301;
                case 7:
                  return 301;
                case 8:
                  return 132;
                case 9:
                  return 132;
                case 10:
                  return;
                case 11:
                  break;
                case 12:
                  return 321;
                case 13:
                  return 324;
                case 14:
                  return oi.yytext = "VALUE", 89;
                case 15:
                  return oi.yytext = "VALUE", 189;
                case 16:
                  return oi.yytext = "ROW", 189;
                case 17:
                  return oi.yytext = "COLUMN", 189;
                case 18:
                  return oi.yytext = "MATRIX", 189;
                case 19:
                  return oi.yytext = "INDEX", 189;
                case 20:
                  return oi.yytext = "RECORDSET", 189;
                case 21:
                  return oi.yytext = "TEXT", 189;
                case 22:
                  return oi.yytext = "SELECT", 189;
                case 23:
                  return 525;
                case 24:
                  return 386;
                case 25:
                  return 407;
                case 26:
                  return 520;
                case 27:
                  return 291;
                case 28:
                  return 274;
                case 29:
                  return 274;
                case 30:
                  return 164;
                case 31:
                  return 405;
                case 32:
                  return 170;
                case 33:
                  return 229;
                case 34:
                  return 166;
                case 35:
                  return 207;
                case 36:
                  return 292;
                case 37:
                  return 76;
                case 38:
                  return 423;
                case 39:
                  return 246;
                case 40:
                  return 409;
                case 41:
                  return 361;
                case 42:
                  return 290;
                case 43:
                  return 519;
                case 44:
                  return 442;
                case 45:
                  return 335;
                case 46:
                  return 446;
                case 47:
                  return 336;
                case 48:
                  return 320;
                case 49:
                  return 119;
                case 50:
                  return 112;
                case 51:
                  return 320;
                case 52:
                  return 112;
                case 53:
                  return 320;
                case 54:
                  return 112;
                case 55:
                  return 320;
                case 56:
                  return 513;
                case 57:
                  return 308;
                case 58:
                  return 276;
                case 59:
                  return 373;
                case 60:
                  return 130;
                case 61:
                  return "CLOSE";
                case 62:
                  return 247;
                case 63:
                  return 190;
                case 64:
                  return 190;
                case 65:
                  return 439;
                case 66:
                  return 372;
                case 67:
                  return 475;
                case 68:
                  return 445;
                case 69:
                  return 278;
                case 70:
                  return 240;
                case 71:
                  return 287;
                case 72:
                  return 272;
                case 73:
                  return 206;
                case 74:
                  return 238;
                case 75:
                  return 269;
                case 76:
                  return 270;
                case 77:
                  return 270;
                case 78:
                  return "CURSOR";
                case 79:
                  return 410;
                case 80:
                  return 295;
                case 81:
                  return 296;
                case 82:
                  return 297;
                case 83:
                  return 453;
                case 84:
                  return 348;
                case 85:
                  return 343;
                case 86:
                  return "DELETED";
                case 87:
                  return 246;
                case 88:
                  return 411;
                case 89:
                  return 185;
                case 90:
                  return 401;
                case 91:
                  return 452;
                case 92:
                  return 135;
                case 93:
                  return 311;
                case 94:
                  return 394;
                case 95:
                  return 315;
                case 96:
                  return 319;
                case 97:
                  return 169;
                case 98:
                  return 513;
                case 99:
                  return 513;
                case 100:
                  return 303;
                case 101:
                  return 14;
                case 102:
                  return 300;
                case 103:
                  return 253;
                case 104:
                  return 244;
                case 105:
                  return 95;
                case 106:
                  return 378;
                case 107:
                  return 183;
                case 108:
                  return 227;
                case 109:
                  return 273;
                case 110:
                  return 318;
                case 111:
                  return 607;
                case 112:
                  return 477;
                case 113:
                  return 232;
                case 114:
                  return 236;
                case 115:
                  return 239;
                case 116:
                  return 156;
                case 117:
                  return 361;
                case 118:
                  return 337;
                case 119:
                  return 99;
                case 120:
                  return 193;
                case 121:
                  return 212;
                case 122:
                  return 224;
                case 123:
                  return 521;
                case 124:
                  return 344;
                case 125:
                  return 213;
                case 126:
                  return 168;
                case 127:
                  return 298;
                case 128:
                  return 198;
                case 129:
                  return 223;
                case 130:
                  return 375;
                case 131:
                  return 245;
                case 132:
                  return "LET";
                case 133:
                  return 225;
                case 134:
                  return 112;
                case 135:
                  return 249;
                case 136:
                  return 465;
                case 137:
                  return 191;
                case 138:
                  return 289;
                case 139:
                  return 395;
                case 140:
                  return 288;
                case 141:
                  return 457;
                case 142:
                  return 169;
                case 143:
                  return 408;
                case 144:
                  return 222;
                case 145:
                  return 650;
                case 146:
                  return 275;
                case 147:
                  return 248;
                case 148:
                  return 385;
                case 149:
                  return 154;
                case 150:
                  return 302;
                case 151:
                  return 243;
                case 152:
                  return 438;
                case 153:
                  return 230;
                case 154:
                  return 420;
                case 155:
                  return 129;
                case 156:
                  return 251;
                case 157:
                  return "OPEN";
                case 158:
                  return 421;
                case 159:
                  return 171;
                case 160:
                  return 118;
                case 161:
                  return 208;
                case 162:
                  return 281;
                case 163:
                  return 172;
                case 164:
                  return 284;
                case 165:
                  return 769;
                case 166:
                  return 93;
                case 167:
                  return 16;
                case 168:
                  return 374;
                case 169:
                  return 447;
                case 170:
                  return 682;
                case 171:
                  return 15;
                case 172:
                  return 419;
                case 173:
                  return 194;
                case 174:
                  return "REDUCE";
                case 175:
                  return 379;
                case 176:
                  return 316;
                case 177:
                  return 522;
                case 178:
                  return 686;
                case 179:
                  return 107;
                case 180:
                  return 406;
                case 181:
                  return 175;
                case 182:
                  return 294;
                case 183:
                  return 448;
                case 184:
                  return 691;
                case 185:
                  return 173;
                case 186:
                  return 173;
                case 187:
                  return 226;
                case 188:
                  return 441;
                case 189:
                  return 237;
                case 190:
                  return 150;
                case 191:
                  return 770;
                case 192:
                  return 410;
                case 193:
                  return 89;
                case 194:
                  return 228;
                case 195:
                  return 146;
                case 196:
                  return 146;
                case 197:
                  return 414;
                case 198:
                  return 339;
                case 199:
                  return 422;
                case 200:
                  return "STRATEGY";
                case 201:
                  return "STORE";
                case 202:
                  return 285;
                case 203:
                  return 286;
                case 204:
                  return 358;
                case 205:
                  return 358;
                case 206:
                  return 468;
                case 207:
                  return 362;
                case 208:
                  return 362;
                case 209:
                  return 192;
                case 210:
                  return 314;
                case 211:
                  return "TIMEOUT";
                case 212:
                  return 148;
                case 213:
                  return 195;
                case 214:
                  return 440;
                case 215:
                  return 440;
                case 216:
                  return 514;
                case 217:
                  return 299;
                case 218:
                  return 456;
                case 219:
                  return 162;
                case 220:
                  return 187;
                case 221:
                  return 98;
                case 222:
                  return 340;
                case 223:
                  return 413;
                case 224:
                  return 231;
                case 225:
                  return 149;
                case 226:
                  return 349;
                case 227:
                  return 134;
                case 228:
                  return 415;
                case 229:
                  return 313;
                case 230:
                  return 128;
                case 231:
                  return 444;
                case 232:
                  return 72;
                case 233:
                  return 440;
                case 234:
                  return 131;
                case 235:
                  return 115;
                case 236:
                  return 137;
                case 237:
                  return 179;
                case 238:
                  return 322;
                case 239:
                  return 180;
                case 240:
                  return 133;
                case 241:
                  return 138;
                case 242:
                  return 331;
                case 243:
                  return 328;
                case 244:
                  return 330;
                case 245:
                  return 327;
                case 246:
                  return 325;
                case 247:
                  return 323;
                case 248:
                  return 324;
                case 249:
                  return 142;
                case 250:
                  return 141;
                case 251:
                  return 139;
                case 252:
                  return 326;
                case 253:
                  return 329;
                case 254:
                  return 140;
                case 255:
                  return 124;
                case 256:
                  return 329;
                case 257:
                  return 77;
                case 258:
                  return 78;
                case 259:
                  return 145;
                case 260:
                  return 429;
                case 261:
                  return 431;
                case 262:
                  return 305;
                case 263:
                  return 510;
                case 264:
                  return 512;
                case 265:
                  return 122;
                case 266:
                  return 116;
                case 267:
                  return 74;
                case 268:
                  return 338;
                case 269:
                  return 152;
                case 270:
                  return 768;
                case 271:
                  return 143;
                case 272:
                  return 181;
                case 273:
                  return 136;
                case 274:
                  return 123;
                case 275:
                  return 317;
                case 276:
                  return 4;
                case 277:
                  return 10;
                case 278:
                  return "INVALID";
              }
            }, rules: [/^(?:``([^\`])+``)/i, /^(?:\[\?\])/i, /^(?:@\[)/i, /^(?:ARRAY\[)/i, /^(?:\[([^\]'])*?\])/i, /^(?:`([^\`'])*?`)/i, /^(?:N(['](\\.|[^']|\\')*?['])+)/i, /^(?:X(['](\\.|[^']|\\')*?['])+)/i, /^(?:(['](\\.|[^']|\\')*?['])+)/i, /^(?:(["](\\.|[^"]|\\")*?["])+)/i, /^(?:--(.*?)($|\r\n|\r|\n))/i, /^(?:\s+)/i, /^(?:\|\|)/i, /^(?:\|)/i, /^(?:VALUE\s+OF\s+SEARCH\b)/i, /^(?:VALUE\s+OF\s+SELECT\b)/i, /^(?:ROW\s+OF\s+SELECT\b)/i, /^(?:COLUMN\s+OF\s+SELECT\b)/i, /^(?:MATRIX\s+OF\s+SELECT\b)/i, /^(?:INDEX\s+OF\s+SELECT\b)/i, /^(?:RECORDSET\s+OF\s+SELECT\b)/i, /^(?:TEXT\s+OF\s+SELECT\b)/i, /^(?:SELECT\b)/i, /^(?:ABSOLUTE\b)/i, /^(?:ACTION\b)/i, /^(?:ADD\b)/i, /^(?:AFTER\b)/i, /^(?:AGGR\b)/i, /^(?:AGGREGATE\b)/i, /^(?:AGGREGATOR\b)/i, /^(?:ALL\b)/i, /^(?:ALTER\b)/i, /^(?:AND\b)/i, /^(?:ANTI\b)/i, /^(?:ANY\b)/i, /^(?:APPLY\b)/i, /^(?:ARRAY\b)/i, /^(?:AS\b)/i, /^(?:ASSERT\b)/i, /^(?:ASC\b)/i, /^(?:ATTACH\b)/i, /^(?:AUTO(_)?INCREMENT\b)/i, /^(?:AVG\b)/i, /^(?:BEFORE\b)/i, /^(?:BEGIN\b)/i, /^(?:BETWEEN\b)/i, /^(?:BREAK\b)/i, /^(?:NOT\s+BETWEEN\b)/i, /^(?:NOT\s+LIKE\b)/i, /^(?:BY\b)/i, /^(?:~~\*)/i, /^(?:!~~\*)/i, /^(?:~~)/i, /^(?:!~~)/i, /^(?:ILIKE\b)/i, /^(?:NOT\s+ILIKE\b)/i, /^(?:CALL\b)/i, /^(?:CASE\b)/i, /^(?:CAST\b)/i, /^(?:CHECK\b)/i, /^(?:CLASS\b)/i, /^(?:CLOSE\b)/i, /^(?:COLLATE\b)/i, /^(?:COLUMN\b)/i, /^(?:COLUMNS\b)/i, /^(?:COMMIT\b)/i, /^(?:CONSTRAINT\b)/i, /^(?:CONTENT\b)/i, /^(?:CONTINUE\b)/i, /^(?:CONVERT\b)/i, /^(?:CORRESPONDING\b)/i, /^(?:COUNT\b)/i, /^(?:CREATE\b)/i, /^(?:CROSS\b)/i, /^(?:CUBE\b)/i, /^(?:CURRENT_TIMESTAMP\b)/i, /^(?:CURRENT_DATE\b)/i, /^(?:CURDATE\b)/i, /^(?:CURSOR\b)/i, /^(?:DATABASE(S)?)/i, /^(?:DATEADD\b)/i, /^(?:DATEDIFF\b)/i, /^(?:TIMESTAMPDIFF\b)/i, /^(?:DECLARE\b)/i, /^(?:DEFAULT\b)/i, /^(?:DELETE\b)/i, /^(?:DELETED\b)/i, /^(?:DESC\b)/i, /^(?:DETACH\b)/i, /^(?:DISTINCT\b)/i, /^(?:DROP\b)/i, /^(?:ECHO\b)/i, /^(?:EDGE\b)/i, /^(?:END\b)/i, /^(?:ENUM\b)/i, /^(?:ELSE\b)/i, /^(?:ESCAPE\b)/i, /^(?:EXCEPT\b)/i, /^(?:EXEC\b)/i, /^(?:EXECUTE\b)/i, /^(?:EXISTS\b)/i, /^(?:EXPLAIN\b)/i, /^(?:FALSE\b)/i, /^(?:FETCH\b)/i, /^(?:FIRST\b)/i, /^(?:FOR\b)/i, /^(?:FOREIGN\b)/i, /^(?:FROM\b)/i, /^(?:FULL\b)/i, /^(?:FUNCTION\b)/i, /^(?:GLOB\b)/i, /^(?:GO\b)/i, /^(?:GRAPH\b)/i, /^(?:GROUP\b)/i, /^(?:GROUPING\b)/i, /^(?:HAVING\b)/i, /^(?:IF\b)/i, /^(?:IDENTITY\b)/i, /^(?:IS\b)/i, /^(?:IN\b)/i, /^(?:INDEX\b)/i, /^(?:INDEXED\b)/i, /^(?:INNER\b)/i, /^(?:INSTEAD\b)/i, /^(?:INSERT\b)/i, /^(?:INSERTED\b)/i, /^(?:INTERSECT\b)/i, /^(?:INTERVAL\b)/i, /^(?:INTO\b)/i, /^(?:JOIN\b)/i, /^(?:KEY\b)/i, /^(?:LAST\b)/i, /^(?:LET\b)/i, /^(?:LEFT\b)/i, /^(?:LIKE\b)/i, /^(?:LIMIT\b)/i, /^(?:MATCHED\b)/i, /^(?:MATRIX\b)/i, /^(?:MAX\s*(?=\())/i, /^(?:MAX\s*(?=(,|\))))/i, /^(?:MIN\s*(?=\())/i, /^(?:MERGE\b)/i, /^(?:MINUS\b)/i, /^(?:MODIFY\b)/i, /^(?:NATURAL\b)/i, /^(?:NEXT\b)/i, /^(?:NEW\b)/i, /^(?:NOCASE\b)/i, /^(?:NO\b)/i, /^(?:NOT\b)/i, /^(?:NULL\b)/i, /^(?:NULLS\b)/i, /^(?:OFF\b)/i, /^(?:ON\b)/i, /^(?:ONLY\b)/i, /^(?:OF\b)/i, /^(?:OFFSET\b)/i, /^(?:OPEN\b)/i, /^(?:OPTION\b)/i, /^(?:OR\b)/i, /^(?:ORDER\b)/i, /^(?:OUTER\b)/i, /^(?:OVER\b)/i, /^(?:PATH\b)/i, /^(?:PARTITION\b)/i, /^(?:PERCENT\b)/i, /^(?:PIVOT\b)/i, /^(?:PLAN\b)/i, /^(?:PRIMARY\b)/i, /^(?:PRINT\b)/i, /^(?:PRIOR\b)/i, /^(?:QUERY\b)/i, /^(?:READ\b)/i, /^(?:RECORDSET\b)/i, /^(?:REDUCE\b)/i, /^(?:REFERENCES\b)/i, /^(?:REGEXP\b)/i, /^(?:REINDEX\b)/i, /^(?:RELATIVE\b)/i, /^(?:REMOVE\b)/i, /^(?:RENAME\b)/i, /^(?:REPEAT\b)/i, /^(?:REPLACE\b)/i, /^(?:REQUIRE\b)/i, /^(?:RESTORE\b)/i, /^(?:RETURN\b)/i, /^(?:RETURNS\b)/i, /^(?:RIGHT\b)/i, /^(?:ROLLBACK\b)/i, /^(?:ROLLUP\b)/i, /^(?:ROW\b)/i, /^(?:ROWS\b)/i, /^(?:SCHEMA(S)?)/i, /^(?:SEARCH\b)/i, /^(?:SEMI\b)/i, /^(?:SET\b)/i, /^(?:SETS\b)/i, /^(?:SHOW\b)/i, /^(?:SOME\b)/i, /^(?:SOURCE\b)/i, /^(?:STRATEGY\b)/i, /^(?:STORE\b)/i, /^(?:SUM\b)/i, /^(?:TOTAL\b)/i, /^(?:TABLE\b)/i, /^(?:TABLES\b)/i, /^(?:TARGET\b)/i, /^(?:TEMP\b)/i, /^(?:TEMPORARY\b)/i, /^(?:TEXTSTRING\b)/i, /^(?:THEN\b)/i, /^(?:TIMEOUT\b)/i, /^(?:TO\b)/i, /^(?:TOP\b)/i, /^(?:TRAN\b)/i, /^(?:TRANSACTION\b)/i, /^(?:TRIGGER\b)/i, /^(?:TRUE\b)/i, /^(?:TRUNCATE\b)/i, /^(?:UNION\b)/i, /^(?:UNIQUE\b)/i, /^(?:UNPIVOT\b)/i, /^(?:UPDATE\b)/i, /^(?:USE\b)/i, /^(?:USING\b)/i, /^(?:VALUE\b)/i, /^(?:VALUES\b)/i, /^(?:VERTEX\b)/i, /^(?:VIEW\b)/i, /^(?:WHEN\b)/i, /^(?:WHERE\b)/i, /^(?:WHILE\b)/i, /^(?:WITH\b)/i, /^(?:WORK\b)/i, /^(?:(\d+\.?\d*|\.\d+)([eE][+-]?\d+)?)/i, /^(?:->)/i, /^(?:#)/i, /^(?:\+)/i, /^(?:-)/i, /^(?:\*)/i, /^(?:\/)/i, /^(?:%)/i, /^(?:!===)/i, /^(?:===)/i, /^(?:!==)/i, /^(?:==)/i, /^(?:>=)/i, /^(?:&)/i, /^(?:\|)/i, /^(?:<<)/i, /^(?:>>)/i, /^(?:>)/i, /^(?:<=)/i, /^(?:<>)/i, /^(?:<)/i, /^(?:=)/i, /^(?:!=)/i, /^(?:\()/i, /^(?:\))/i, /^(?:@)/i, /^(?:\{)/i, /^(?:\})/i, /^(?:\])/i, /^(?::-)/i, /^(?:\?-)/i, /^(?:\.\.)/i, /^(?:\.)/i, /^(?:,)/i, /^(?:::)/i, /^(?::)/i, /^(?:;)/i, /^(?:\$)/i, /^(?:\?)/i, /^(?:!)/i, /^(?:\^)/i, /^(?:~)/i, /^(?:[0-9]*[a-zA-Z_]+[a-zA-Z_0-9]*)/i, /^(?:$)/i, /^(?:.)/i], conditions: { INITIAL: { rules: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255, 256, 257, 258, 259, 260, 261, 262, 263, 264, 265, 266, 267, 268, 269, 270, 271, 272, 273, 274, 275, 276, 277, 278], inclusive: !0 } } };
            return bd;
          }();
          fh.lexer = F2;
          function hh() {
            this.yy = {};
          }
          return hh.prototype = fh, fh.Parser = hh, new hh();
        }();
        typeof V < "u" && (_.parser = G, _.Parser = G.Parser, _.parse = function() {
          return G.parse.apply(G, arguments);
        }, _.main = function(Ie) {
          Ie[1] || (console.log("Usage: " + Ie[0] + " FILE"), process.exit(1));
          var rt = V().readFileSync(V().normalize(Ie[1]), "utf8");
          return _.parser.parse(rt);
        }, V.main === O && _.main(process.argv.slice(1))), H.prettyflag = !1, H.pretty = function(Ie, rt) {
          var ft = H.prettyflag;
          H.prettyflag = !rt;
          var pt = H.parse(Ie).toString();
          return H.prettyflag = ft, pt;
        };
        var J = H.utils = {};
        function Z(Ie) {
          return "(y=" + Ie + ",y===y?y:undefined)";
        }
        function te(Ie, rt) {
          return "(y=" + Ie + ',typeof y=="undefined"?undefined:' + rt + ")";
        }
        function ne() {
          return !0;
        }
        var ee = J.escapeq = function(Ie) {
          return ("" + Ie).replace(/["'\\\n\r\u2028\u2029]/g, function(rt) {
            switch (rt) {
              case '"':
              case "'":
              case "\\":
                return "\\" + rt;
              case `
`:
                return "\\n";
              case "\r":
                return "\\r";
              case "\u2028":
                return "\\u2028";
              case "\u2029":
                return "\\u2029";
            }
          });
        }, ae = J.undoubleq = function(Ie) {
          return Ie.replace(/(\')/g, "''");
        }, oe = J.doubleq = function(Ie) {
          return Ie.replace(/(\'\')/g, "\\'");
        };
        J.doubleqq = function(Ie) {
          return Ie.replace(/'/g, "\\'");
        };
        var se = function(Ie) {
          return Ie[0] === "\uFEFF" && (Ie = Ie.substr(1)), Ie;
        };
        J.global = function() {
          return typeof self < "u" ? self : typeof window < "u" ? window : typeof commonjsGlobal < "u" ? commonjsGlobal : Function("return this")();
        }(), J.isNativeFunction = function(Ie) {
          return typeof Ie == "function" && !!~Ie.toString().indexOf("[native code]");
        }, J.isWebWorker = function() {
          try {
            var Ie = J.global.importScripts;
            return J.isNativeFunction(Ie);
          } catch {
            return !1;
          }
        }(), J.isNode = function() {
          try {
            return !(typeof process > "u" || !process.versions || !process.versions.node);
          } catch {
            return !1;
          }
        }(), J.isBrowser = function() {
          try {
            return J.isNativeFunction(J.global.location.reload);
          } catch {
            return !1;
          }
        }(), J.isBrowserify = function() {
          return J.isBrowser && typeof process < "u" && process.browser;
        }(), J.isRequireJS = function() {
          return J.isBrowser && typeof V == "function" && typeof V.specified == "function";
        }(), J.isMeteor = function() {
          return typeof Meteor < "u" && Meteor.release;
        }(), J.isMeteorClient = J.isMeteorClient = function() {
          return J.isMeteor && Meteor.isClient;
        }(), J.isMeteorServer = function() {
          return J.isMeteor && Meteor.isServer;
        }(), J.isCordova = /* @__PURE__ */ function() {
          return typeof cordova == "object";
        }(), J.isReactNative = /* @__PURE__ */ function() {
          var Ie = !1;
          return Ie;
        }(), J.hasIndexedDB = function() {
          return !!J.global.indexedDB;
        }(), J.isArray = function(Ie) {
          return Object.prototype.toString.call(Ie) === "[object Array]";
        };
        let ce = J.loadFile = function(Ie, rt, ft, pt) {
          var wt;
          if (!(J.isNode || J.isMeteorServer)) {
            if (J.isCordova) {
              J.global.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function(Dt) {
                Dt.root.getFile(Ie, { create: !1 }, function(Ot) {
                  Ot.file(function(un) {
                    var yn = new FileReader();
                    yn.onloadend = function(_n) {
                      ft(se(this.result));
                    }, yn.readAsText(un);
                  });
                });
              });
              return;
            }
            if (typeof Ie == "string") {
              if (Ie.substr(0, 1) === "#" && typeof document < "u") {
                wt = document.querySelector(Ie).textContent, ft(wt);
                return;
              }
              ue(Ie, (Dt) => ft(se(Dt)), pt, rt);
              return;
            }
            if (Ie instanceof Event) {
              var Bt = Ie.target.files, At = new FileReader();
              Bt[0].name, At.onload = function(Dt) {
                var Ot = Dt.target.result;
                ft(se(Ot));
              }, At.readAsText(Bt[0]);
            }
            ue(Ie, (Dt) => ft(se(Dt)), pt, rt);
          }
        }, le = typeof fetch < "u" ? fetch : null;
        async function ue(Ie, rt, ft, pt) {
          return pt ? de(Ie, rt, ft) : await de(Ie, rt, ft);
        }
        function de(Ie, rt, ft) {
          return le(Ie).then((pt) => pt.arrayBuffer()).then((pt) => {
            var wt = new Uint8Array(pt), Bt = [...wt].map((At) => String.fromCharCode(At)).join("");
            rt(Bt);
          }).catch((pt) => {
            if (ft) return ft(pt);
            throw console.error(pt), pt;
          });
        }
        J.loadBinaryFile = function(Ie, rt, ft, pt = (wt) => {
          throw wt;
        }) {
          if (!(J.isNode || J.isMeteorServer)) if (typeof Ie == "string") {
            var wt = new XMLHttpRequest();
            wt.open("GET", Ie, rt), wt.responseType = "arraybuffer", wt.onload = function() {
              for (var Dt = new Uint8Array(wt.response), Ot = [], un = 0; un < Dt.length; ++un) Ot[un] = String.fromCharCode(Dt[un]);
              ft(Ot.join(""));
            }, wt.onerror = pt, wt.send();
          } else if (Ie instanceof Event) {
            var Bt = Ie.target.files, At = new FileReader();
            Bt[0].name, At.onload = function(Dt) {
              var Ot = Dt.target.result;
              ft(Ot);
            }, At.onerror = pt, At.readAsArrayBuffer(Bt[0]);
          } else Ie instanceof Blob && ft(Ie);
        }, J.removeFile = function(Ie, rt) {
          if (!J.isNode) throw new Error("You can remove files only in Node.js and Apache Cordova");
        }, J.deleteFile = function(Ie, rt) {
        }, J.autoExtFilename = function(Ie, rt, ft) {
          return ft = ft || {}, typeof Ie != "string" || Ie.match(/^[A-Z]+:\/\/|\n|\..{2,6}$/i) || ft.autoExt === 0 || ft.autoExt === !1 ? Ie : Ie + "." + rt;
        }, J.fileExists = function(Ie, rt) {
          if (!J.isNode) throw new Error("You can use exists() only in Node.js or Apach Cordova");
        }, J.saveFile = function(Ie, rt, ft, pt) {
          var wt = 1;
          if (Ie === void 0) wt = rt, ft && (wt = ft(wt));
          else if (!J.isNode) {
            var Bt = { disableAutoBom: !1 };
            H.utils.extend(Bt, pt);
            var At = new Blob([rt], { type: "text/plain;charset=utf-8" });
            cn(At, Ie, Bt.disableAutoBom), ft && (wt = ft(wt));
          }
          return wt;
        };
        var fe = J.hash = function(Ie) {
          for (var rt = 2166136261, ft = Ie.length; ft; ) rt ^= Ie.charCodeAt(--ft), rt += (rt << 1) + (rt << 4) + (rt << 7) + (rt << 8) + (rt << 24);
          return rt;
        }, he = J.arrayUnion = function(Ie, rt) {
          var ft = rt.slice(0);
          return Ie.forEach(function(pt) {
            ft.indexOf(pt) < 0 && ft.push(pt);
          }), ft;
        }, me = J.arrayDiff = function(Ie, rt) {
          return Ie.filter(function(ft) {
            return rt.indexOf(ft) < 0;
          });
        }, pe = J.arrayIntersect = function(Ie, rt) {
          var ft = [];
          return Ie.forEach(function(pt) {
            var wt = !1;
            rt.forEach(function(Bt) {
              wt = wt || pt === Bt;
            }), wt && ft.push(pt);
          }), ft;
        }, ye = J.arrayUnionDeep = function(Ie, rt) {
          var ft = rt.slice(0);
          return Ie.forEach(function(pt) {
            var wt = !1;
            ft.forEach(function(Bt) {
              wt = wt || Ce(pt, Bt);
            }), wt || ft.push(pt);
          }), ft;
        }, ge = J.arrayExceptDeep = function(Ie, rt) {
          var ft = [];
          return Ie.forEach(function(pt) {
            var wt = !1;
            rt.forEach(function(Bt) {
              wt = wt || Ce(pt, Bt);
            }), wt || ft.push(pt);
          }), ft;
        }, ve = J.arrayIntersectDeep = function(Ie, rt) {
          var ft = [];
          return Ie.forEach(function(pt) {
            var wt = !1;
            rt.forEach(function(Bt) {
              wt = wt || Ce(pt, Bt, !0);
            }), wt && ft.push(pt);
          }), ft;
        }, be = J.cloneDeep = function Ie(rt) {
          if (rt === null || typeof rt != "object") return rt;
          if (rt instanceof Date) return new Date(rt);
          if (rt instanceof String) return rt.toString();
          if (rt instanceof Number) return +rt;
          var ft = new rt.constructor();
          for (var pt in rt) rt.hasOwnProperty(pt) && (ft[pt] = Ie(rt[pt]));
          return ft;
        }, Ce = J.deepEqual = function(Ie, rt) {
          if (Ie === rt) return !0;
          if (typeof Ie == "object" && Ie !== null && typeof rt == "object" && rt !== null) {
            if (Object.keys(Ie).length !== Object.keys(rt).length) return !1;
            for (var ft in Ie) if (!Ce(Ie[ft], rt[ft])) return !1;
            return !0;
          }
          return !1;
        }, Se = J.distinctArray = function(Ie) {
          for (var rt = {}, ft = 0, pt = Ie.length; ft < pt; ft++) {
            var wt;
            typeof Ie[ft] == "object" ? wt = Object.keys(Ie[ft]).sort().map(function(Dt) {
              return Dt + "`" + Ie[ft][Dt];
            }).join("`") : wt = Ie[ft], rt[wt] = Ie[ft];
          }
          var Bt = [];
          for (var At in rt) Bt.push(rt[At]);
          return Bt;
        }, we = J.extend = function(Ie, rt) {
          Ie = Ie || {};
          for (var ft in rt) rt.hasOwnProperty(ft) && (Ie[ft] = rt[ft]);
          return Ie;
        }, Ee = J.getValueOf = function(Ie) {
          return typeof Ie == "object" && (Ie instanceof String || Ie instanceof Number) ? Ie.valueOf() : Ie;
        }, xe = J.flatArray = function(Ie) {
          if (!Ie || Ie.length === 0) return [];
          if (typeof Ie == "object" && Ie instanceof H.Recordset) return Ie.data.map(function(ft) {
            return Ee(ft[Ie.columns[0].columnid]);
          });
          var rt = Object.keys(Ie[0])[0];
          return rt === void 0 ? [] : Ie.map(function(ft) {
            return ft[rt];
          });
        };
        J.arrayOfArrays = function(Ie) {
          return Ie.map(function(rt) {
            var ft = [];
            for (var pt in rt) ft.push(rt[pt]);
            return ft;
          });
        }, Array.isArray || (Array.isArray = function(Ie) {
          return Object.prototype.toString.call(Ie) === "[object Array]";
        }), J.xlsnc = function(Ie) {
          var rt = "";
          if (Ie > 701) {
            let pt = ((Ie - 26) / 676 | 0) - 1;
            rt = String.fromCharCode(65 + pt % 26), Ie = Ie % (26 * 26);
          }
          var ft = String.fromCharCode(65 + Ie % 26);
          return Ie >= 26 && (Ie = (Ie / 26 | 0) - 1, ft = String.fromCharCode(65 + Ie % 26) + ft, Ie > 26 && (Ie = (Ie / 26 | 0) - 1, ft = String.fromCharCode(65 + Ie % 26) + ft)), rt + ft;
        }, J.xlscn = function(Ie) {
          var rt = Ie.charCodeAt(0) - 65;
          return Ie.length > 1 && (rt = (rt + 1) * 26 + Ie.charCodeAt(1) - 65, Ie.length > 2 && (rt = (rt + 1) * 26 + Ie.charCodeAt(2) - 65)), rt;
        }, J.domEmptyChildren = function(Ie) {
          for (var rt = Ie.childNodes.length; rt--; ) Ie.removeChild(Ie.lastChild);
        };
        var Le = {};
        J.like = function(Ie, rt, ft) {
          if (!Le[Ie]) {
            ft || (ft = "");
            for (var pt = 0, wt = "^"; pt < Ie.length; ) {
              var Bt = Ie[pt], At = "";
              pt < Ie.length - 1 && (At = Ie[pt + 1]), Bt === ft ? (wt += "\\" + At, pt++) : Bt === "[" && At === "^" ? (wt += "[^", pt++) : Bt === "[" || Bt === "]" ? wt += Bt : Bt === "%" ? wt += "[\\s\\S]*" : Bt === "_" ? wt += "." : "/.*+?|(){}".indexOf(Bt) > -1 ? wt += "\\" + Bt : wt += Bt, pt++;
            }
            wt += "$", Le[Ie] = RegExp(wt, "i");
          }
          return ("" + (rt ?? "")).search(Le[Ie]) > -1;
        }, J.glob = function(Ie, rt) {
          for (var ft = 0, pt = "^"; ft < rt.length; ) {
            var wt = rt[ft], Bt = "";
            ft < rt.length - 1 && (Bt = rt[ft + 1]), wt === "[" && Bt === "^" ? (pt += "[^", ft++) : wt === "[" || wt === "]" ? pt += wt : wt === "*" ? pt += ".*" : wt === "?" ? pt += "." : "/.*+?|(){}".indexOf(wt) > -1 ? pt += "\\" + wt : pt += wt, ft++;
          }
          return pt += "$", ("" + (Ie || "")).toUpperCase().search(RegExp(pt.toUpperCase())) > -1;
        }, J.findAlaSQLPath = function() {
          if (J.isWebWorker) return "";
          if (J.isMeteorClient) return "/packages/dist/";
          if (J.isMeteorServer) return "assets/packages/dist/";
          if (J.isNode) return q;
          if (J.isBrowser) for (var Ie = document.getElementsByTagName("script"), rt = 0; rt < Ie.length; rt++) {
            if (Ie[rt].src.substr(-16).toLowerCase() === "alasql-worker.js") return Ie[rt].src.substr(0, Ie[rt].src.length - 16);
            if (Ie[rt].src.substr(-20).toLowerCase() === "alasql-worker.min.js") return Ie[rt].src.substr(0, Ie[rt].src.length - 20);
            if (Ie[rt].src.substr(-9).toLowerCase() === "alasql.js") return Ie[rt].src.substr(0, Ie[rt].src.length - 9);
            if (Ie[rt].src.substr(-13).toLowerCase() === "alasql.min.js") return Ie[rt].src.substr(0, Ie[rt].src.length - 13);
          }
          return "";
        };
        var Qe = function() {
          var Ie = H.private.externalXlsxLib || J.global.XLSX || null;
          if (Ie) return Ie;
          if (J.isNode || J.isBrowserify || J.isMeteorServer, !Ie) throw new Error("Please include the xlsx.js library");
          return Ie;
        };
        H.path = H.utils.findAlaSQLPath(), H.utils.uncomment = function(Ie) {
          Ie = ("__" + Ie + "__").split("");
          for (var rt = !1, ft, pt = !1, wt = !1, Bt = 0, At = Ie.length; Bt < At; Bt++) {
            var Dt = Ie[Bt - 1] !== "\\" || Ie[Bt - 2] === "\\";
            rt ? Ie[Bt] === ft && Dt && (rt = !1) : pt ? Ie[Bt] === "*" && Ie[Bt + 1] === "/" ? (Ie[Bt] = Ie[Bt + 1] = "", pt = !1, Bt++) : Ie[Bt] = "" : wt ? ((Ie[Bt + 1] === `
` || Ie[Bt + 1] === "\r" || Ie.length - 2 === Bt) && (wt = !1), Ie[Bt] = "") : Ie[Bt] === '"' || Ie[Bt] === "'" ? (rt = !0, ft = Ie[Bt]) : Ie[Bt] === "[" && Ie[Bt - 1] !== "@" ? (rt = !0, ft = "]") : Ie[Bt] === "-" && Ie[Bt + 1] === "-" ? (Ie[Bt] = "", wt = !0) : Ie[Bt] === "/" && Ie[Bt + 1] === "*" && (Ie[Bt] = "", pt = !0);
          }
          return Ie = Ie.join("").slice(2, -2), Ie;
        }, H.parser = G, H.parser.parseError = function(Ie, rt) {
          throw new Error("Have you used a reserved keyword without `escaping` it?\n" + Ie);
        }, H.parse = function(Ie) {
          return G.parse(H.utils.uncomment(Ie));
        }, H.engines = {}, H.databases = {}, H.databasenum = 0, H.options = { errorlog: !1, valueof: !0, dropifnotexists: !1, datetimeformat: "sql", casesensitive: !0, logtarget: "output", logprompt: !0, progress: !1, modifier: void 0, columnlookup: 10, autovertex: !0, usedbo: !0, autocommit: !0, cache: !0, tsql: !0, mysql: !0, postgres: !0, oracle: !0, sqlite: !0, orientdb: !0, nocount: !1, nan: !1, excel: { cellDates: !0 }, joinstar: "overwrite", loopbreak: 1e5, dateAsString: !0 }, H.vars = {}, H.declares = {}, H.prompthistory = [], H.plugins = {}, H.from = {}, H.into = {}, H.fn = {}, H.aggr = {}, H.busy = 0, H.MAXSQLCACHESIZE = 1e4, H.DEFAULTDATABASEID = "alasql", H.lastid = 0, H.buffer = {}, H.private = { externalXlsxLib: null }, H.setXLSX = function(Ie) {
          H.private.externalXlsxLib = Ie;
        }, H.use = function(Ie) {
          if (Ie || (Ie = H.DEFAULTDATABASEID), H.useid !== Ie) if (H.databases[Ie] !== void 0) {
            H.useid = Ie;
            let rt = H.databases[H.useid];
            H.tables = rt.tables, rt.resetSqlCache(), H.options.usedbo && (H.databases.dbo = rt);
          } else throw Error("Database does not exist: " + Ie);
        }, H.autoval = function(Ie, rt, ft, pt) {
          var wt = pt ? H.databases[pt] : H.databases[H.useid];
          if (!wt.tables[Ie]) throw new Error("Tablename not found: " + Ie);
          if (!wt.tables[Ie].identities[rt]) throw new Error("Colname not found: " + rt);
          return ft ? wt.tables[Ie].identities[rt].value || null : wt.tables[Ie].identities[rt].value - wt.tables[Ie].identities[rt].step || null;
        }, H.exec = function(Ie, rt, ft, pt) {
          if (typeof rt == "function" && (pt = ft, ft = rt, rt = {}), delete H.error, rt = rt || {}, H.options.errorlog) try {
            return H.dexec(H.useid, Ie, rt, ft, pt);
          } catch (wt) {
            H.error = wt, ft && ft(null, H.error);
          }
          else return H.dexec(H.useid, Ie, rt, ft, pt);
        };
        function ke(Ie) {
          Ie && H.options.cache && Ie && Ie.query && Ie.query.data && (Ie.query.data = []);
        }
        H.dexec = function(Ie, rt, ft, pt, wt) {
          var Bt = H.databases[Ie], At = fe(rt);
          if (H.options.cache) {
            let yn = Bt.sqlCache[At];
            if (yn && Bt.dbversion === yn.dbversion) {
              var Dt = yn(ft, pt);
              return ke(yn), Dt;
            }
          }
          let Ot = Bt.astCache[At];
          if (H.options.cache && !Ot ? (Ot = H.parse(rt), Ot && (Bt.astCache[At] = Ot)) : Ot = H.parse(rt), !!Ot.statements) {
            if (Ot.statements.length === 0) return 0;
            if (Ot.statements.length === 1) {
              if (Ot.statements[0].compile) {
                var un = Ot.statements[0].compile(Ie, ft);
                if (!un) return;
                un.sql = rt, un.dbversion = Bt.dbversion, H.options.cache && (Bt.sqlCacheSize > H.MAXSQLCACHESIZE && Bt.resetSqlCache(), Bt.sqlCacheSize++, Bt.sqlCache[At] = un);
                var Dt = H.res = un(ft, pt, wt);
                return ke(un), Dt;
              }
              H.precompile(Ot.statements[0], H.useid, ft);
              var Dt = H.res = Ot.statements[0].execute(Ie, ft, pt, wt);
              return Dt;
            }
            if (pt) {
              H.adrun(Ie, Ot, ft, pt, wt);
              return;
            }
            return H.drun(Ie, Ot, ft, pt, wt);
          }
        }, H.drun = function(Ie, rt, ft, pt, wt) {
          var Bt = H.useid;
          Bt !== Ie && H.use(Ie);
          for (var At = [], Dt = 0, Ot = rt.statements.length; Dt < Ot; Dt++) if (rt.statements[Dt]) if (rt.statements[Dt].compile) {
            var un = rt.statements[Dt].compile(H.useid);
            At.push(H.res = un(ft, null, wt));
          } else H.precompile(rt.statements[Dt], H.useid, ft), At.push(H.res = rt.statements[Dt].execute(H.useid, ft));
          return Bt !== Ie && H.use(Bt), pt && pt(At), H.res = At, At;
        }, H.adrun = function(Ie, rt, ft, pt, wt) {
          var Bt = 0, At = rt.statements.length;
          H.options.progress !== !1 && H.options.progress(At, Bt++);
          var Dt = H.useid;
          Dt !== Ie && H.use(Ie);
          var Ot = [];
          function un(yn) {
            yn !== void 0 && Ot.push(yn);
            var _n = rt.statements.shift();
            if (!_n) {
              Dt !== Ie && H.use(Dt), pt(Ot);
              return;
            }
            if (_n.compile) {
              var gn = _n.compile(H.useid);
              gn(ft, un, wt), H.options.progress !== !1 && H.options.progress(At, Bt++);
              return;
            }
            H.precompile(rt.statements[0], H.useid, ft), _n.execute(H.useid, ft, un), H.options.progress !== !1 && H.options.progress(At, Bt++);
          }
          un();
        }, H.compile = function(Ie, rt) {
          rt = rt || H.useid;
          let ft = H.parse(Ie);
          if (ft.statements.length !== 1) throw new Error("Cannot compile, because number of statements in SQL is not equal to 1");
          var pt = ft.statements[0].compile(rt);
          return pt.promise = function(wt) {
            return new Promise(function(Bt, At) {
              pt(wt, function(Dt, Ot) {
                Ot ? At(Ot) : Bt(Dt);
              });
            });
          }, pt;
        }, J.global.Promise || (J.global.Promise = Promise);
        var Me = function(Ie, rt, ft, pt) {
          return new J.global.Promise(function(wt, Bt) {
            H(Ie, rt, function(At, Dt) {
              Dt ? Bt(Dt) : (ft && pt && H.options.progress !== !1 && H.options.progress(ft, pt), wt(At));
            });
          });
        };
        const Re = (Ie, rt) => {
          var ft = rt.resolve([]);
          return Ie.forEach((pt) => {
            ft = ft.then((wt) => Me(pt.sql, pt.params, pt.i, pt.length).then((Bt) => [...wt, Bt]));
          }), ft;
        };
        var De = function(Ie) {
          if (!(Ie.length < 1)) {
            for (var rt, ft, pt, wt = [], Bt = 0; Bt < Ie.length; Bt++) {
              if (rt = Ie[Bt], typeof rt == "string" && (rt = [rt]), !J.isArray(rt) || rt.length < 1 || 2 < rt.length) throw new Error("Error in .promise parameter");
              ft = rt[0], pt = rt[1] || void 0, wt.push({ sql: ft, params: pt, i: Bt, length: Ie.length });
            }
            return Re(wt, J.global.Promise);
          }
        };
        H.promise = function(Ie, rt) {
          if (typeof Promise > "u") throw new Error("Please include a Promise/A+ library");
          if (typeof Ie == "string") return Me(Ie, rt);
          if (!J.isArray(Ie) || Ie.length < 1 || typeof rt < "u") throw new Error("Error in .promise parameters");
          return De(Ie);
        };
        var Pe = H.Database = function(Ie) {
          var rt = this;
          if (rt === H) if (Ie) {
            if (rt = H.databases[Ie], H.databases[Ie] = rt, !rt) throw new Error(`Database ${Ie} not found`);
          } else rt = H.databases.alasql, H.options.tsql && (H.databases.tempdb = H.databases.alasql);
          return Ie || (Ie = "db" + H.databasenum++), rt.databaseid = Ie, H.databases[Ie] = rt, rt.dbversion = 0, rt.tables = {}, rt.views = {}, rt.triggers = {}, rt.indices = {}, rt.objects = {}, rt.counter = 0, rt.resetSqlCache(), rt;
        };
        Pe.prototype.resetSqlCache = function() {
          this.sqlCache = {}, this.sqlCacheSize = 0, this.astCache = {};
        }, Pe.prototype.exec = function(Ie, rt, ft) {
          return H.dexec(this.databaseid, Ie, rt, ft);
        }, Pe.prototype.autoval = function(Ie, rt, ft) {
          return H.autoval(Ie, rt, ft, this.databaseid);
        }, Pe.prototype.transaction = function(Ie) {
          var rt = new H.Transaction(this.databaseid), ft = Ie(rt);
          return ft;
        };
        class $e {
          constructor(rt) {
            mt(this, "transactionid", Date.now());
            mt(this, "committed", !1);
            mt(this, "bank");
            this.databaseid = rt, this.dbversion = H.databases[rt].dbversion, this.bank = JSON.stringify(H.databases[rt]);
          }
          commit() {
            this.committed = !0, H.databases[this.databaseid].dbversion = Date.now(), delete this.bank;
          }
          rollback() {
            if (!this.committed) H.databases[this.databaseid] = JSON.parse(this.bank), delete this.bank;
            else throw new Error("Transaction already commited");
          }
          exec(rt, ft, pt) {
            return H.dexec(this.databaseid, rt, ft, pt);
          }
        }
        $e.prototype.executeSQL = $e.prototype.exec, H.Transaction = $e;
        var qe = H.Table = function(Ie) {
          this.data = [], this.columns = [], this.xcolumns = {}, this.inddefs = {}, this.indices = {}, this.uniqs = {}, this.uniqdefs = {}, this.identities = {}, this.checks = [], this.checkfns = [], this.beforeinsert = {}, this.afterinsert = {}, this.insteadofinsert = {}, this.beforedelete = {}, this.afterdelete = {}, this.insteadofdelete = {}, this.beforeupdate = {}, this.afterupdate = {}, this.insteadofupdate = {}, Object.assign(this, Ie);
        };
        qe.prototype.indexColumns = function() {
          var Ie = this;
          Ie.xcolumns = {}, Ie.columns.forEach(function(rt) {
            Ie.xcolumns[rt.columnid] = rt;
          });
        };
        class Xe {
          constructor(rt) {
            this.columns = [], this.xcolumns = {}, this.query = [], Object.assign(this, rt);
          }
        }
        H.View = Xe;
        class Ve {
          constructor(rt) {
            this.alasql = H, this.columns = [], this.xcolumns = {}, this.selectGroup = [], this.groupColumns = {}, Object.assign(this, rt);
          }
        }
        class Ze {
          constructor(rt) {
            Object.assign(this, rt);
          }
        }
        H.Recordset = Ze, H.Query = Ve;
        class Ge {
          constructor(rt) {
            Object.assign(this, rt);
          }
          toString() {
          }
          toType() {
          }
          toJS() {
          }
          exec() {
          }
          compile() {
          }
        }
        var ze = { extend: Object.assign, casesensitive: H.options.casesensitive, Base: Ge };
        G.yy = H.yy = ze, ze.Statements = class {
          constructor(Ie) {
            Object.assign(this, Ie);
          }
          toString() {
            return this.statements.map((Ie) => Ie.toString()).join("; ");
          }
          compile(Ie) {
            const rt = this.statements.map((ft) => ft.compile(Ie));
            return rt.length === 1 ? rt[0] : (ft, pt) => {
              const wt = rt.map((Bt) => Bt(ft));
              return pt && pt(wt), wt;
            };
          }
        }, ze.Search = (Pn = class {
          constructor(Ie) {
            Ly(this, wn);
            Object.assign(this, Ie);
          }
          toString() {
            let Ie = "SEARCH ";
            return this.selectors && (Ie += this.selectors.toString()), this.from && (Ie += "FROM " + this.from.toString()), Ie;
          }
          toJS(Ie) {
            return `this.queriesfn[${this.queriesidx - 1}](this.params,null,${Ie})`;
          }
          compile(Ie) {
            var rt = Ie, ft = (pt, wt) => {
              var Bt;
              return ky(this, wn, e7).call(this, rt, pt, function(At) {
                Bt = gt(ft.query, At), wt && (Bt = wt(Bt));
              }), Bt;
            };
            return ft.query = {}, ft;
          }
        }, wn = new WeakSet(), e7 = function(Ie, rt, ft) {
          var pt, wt = {}, Bt, At = be(this.selectors);
          function Dt(bn, an, Zt) {
            var Mn, Ln, bi, In = bn[an], kn = H.options.loopbreak || 1e5;
            if (In.selid) {
              if (In.selid === "PATH") {
                for (var er = [{ node: Zt, stack: [] }], jn = {}, ur = H.databases[H.useid].objects; er.length > 0; ) {
                  var Kn = er.shift(), Jn = Kn.node, Lr = Kn.stack, bi = Dt(In.args, 0, Jn);
                  if (bi.length > 0) {
                    if (an + 1 + 1 > bn.length) return Lr;
                    var mr = [];
                    return Lr && Lr.length > 0 && Lr.forEach(function(Pr) {
                      mr = mr.concat(Dt(bn, an + 1, Pr));
                    }), mr;
                  } else {
                    if (typeof jn[Jn.$id] < "u") continue;
                    jn[Jn.$id] = !0, Jn.$out && Jn.$out.length > 0 && Jn.$out.forEach(function(Pr) {
                      var ei = ur[Pr], Ci = Lr.concat(ei);
                      Ci.push(ur[ei.$out[0]]), er.push({ node: ur[ei.$out[0]], stack: Ci });
                    });
                  }
                }
                return [];
              }
              if (In.selid === "NOT") {
                var Ln = Dt(In.args, 0, Zt);
                return Ln.length > 0 ? [] : an + 1 + 1 > bn.length ? [Zt] : Dt(bn, an + 1, Zt);
              } else if (In.selid === "DISTINCT") {
                var Ln;
                if (typeof In.args > "u" || In.args.length === 0 ? Ln = Se(Zt) : Ln = Dt(In.args, 0, Zt), Ln.length === 0) return [];
                var Zi = Se(Ln);
                return an + 1 + 1 > bn.length ? Zi : Dt(bn, an + 1, Zi);
              } else if (In.selid === "AND") {
                var Zi = !0;
                return In.args.forEach(function(Pr) {
                  Zi = Zi && Dt(Pr, 0, Zt).length > 0;
                }), Zi ? an + 1 + 1 > bn.length ? [Zt] : Dt(bn, an + 1, Zt) : [];
              } else if (In.selid === "OR") {
                var Zi = !1;
                return In.args.forEach(function(Pr) {
                  Zi = Zi || Dt(Pr, 0, Zt).length > 0;
                }), Zi ? an + 1 + 1 > bn.length ? [Zt] : Dt(bn, an + 1, Zt) : [];
              } else if (In.selid === "ALL") {
                var Ln = Dt(In.args[0], 0, Zt);
                return Ln.length === 0 ? [] : an + 1 + 1 > bn.length ? Ln : Dt(bn, an + 1, Ln);
              } else if (In.selid === "ANY") {
                var Ln = Dt(In.args[0], 0, Zt);
                return Ln.length === 0 ? [] : an + 1 + 1 > bn.length ? [Ln[0]] : Dt(bn, an + 1, [Ln[0]]);
              } else if (In.selid === "UNIONALL") {
                var Ln = [];
                return In.args.forEach(function(Pr) {
                  Ln = Ln.concat(Dt(Pr, 0, Zt));
                }), Ln.length === 0 ? [] : an + 1 + 1 > bn.length ? Ln : Dt(bn, an + 1, Ln);
              } else if (In.selid === "UNION") {
                var Ln = [];
                In.args.forEach(function(Pr) {
                  Ln = Ln.concat(Dt(Pr, 0, Zt));
                });
                var Ln = Se(Ln);
                return Ln.length === 0 ? [] : an + 1 + 1 > bn.length ? Ln : Dt(bn, an + 1, Ln);
              } else if (In.selid === "IF") {
                var Ln = Dt(In.args, 0, Zt);
                return Ln.length === 0 ? [] : an + 1 + 1 > bn.length ? [Zt] : Dt(bn, an + 1, Zt);
              } else if (In.selid === "REPEAT") {
                var Ar, Yr, jr = In.args[0].value;
                In.args[1] ? Yr = In.args[1].value : Yr = jr, In.args[2] && (Ar = In.args[2].variable);
                var Mr = [];
                if (jr === 0 && (an + 1 + 1 > bn.length ? Mr = [Zt] : (Ar && (H.vars[Ar] = 0), Mr = Mr.concat(Dt(bn, an + 1, Zt)))), Yr > 0) for (var Wr = [{ value: Zt, lvl: 1 }], Bi = 0; Wr.length > 0; ) {
                  var Ln = Wr[0];
                  if (Wr.shift(), Ln.lvl <= Yr) {
                    Ar && (H.vars[Ar] = Ln.lvl);
                    var gi = Dt(In.sels, 0, Ln.value);
                    gi.forEach(function(Pr) {
                      Wr.push({ value: Pr, lvl: Ln.lvl + 1 });
                    }), Ln.lvl >= jr && (an + 1 + 1 > bn.length ? Mr = Mr.concat(gi) : gi.forEach(function(Pr) {
                      Mr = Mr.concat(Dt(bn, an + 1, Pr));
                    }));
                  }
                  if (Bi++, Bi > kn) throw new Error("Infinite loop brake. Number of iterations = " + Bi);
                }
                return Mr;
              } else if (In.selid === "OF") {
                if (an + 1 + 1 > bn.length) return [Zt];
                var Zr = [];
                return Object.keys(Zt).forEach(function(Br) {
                  H.vars[In.args[0].variable] = Br, Zr = Zr.concat(Dt(bn, an + 1, Zt[Br]));
                }), Zr;
              } else if (In.selid === "TO") {
                var fi = H.vars[In.args[0]], ri = [];
                if (fi !== void 0 ? ri = fi.slice(0) : ri = [], ri.push(Zt), an + 1 + 1 > bn.length) return [Zt];
                H.vars[In.args[0]] = ri;
                var Zr = Dt(bn, an + 1, Zt);
                return H.vars[In.args[0]] = fi, Zr;
              } else if (In.selid === "ARRAY") {
                var Ln = Dt(In.args, 0, Zt);
                if (Ln.length > 0) Mn = Ln;
                else return [];
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "SUM") {
                var Ln = Dt(In.args, 0, Zt);
                if (Ln.length > 0) var Mn = Ln.reduce(function(ei, Ci) {
                  return ei + Ci;
                }, 0);
                else return [];
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "AVG") {
                if (Ln = Dt(In.args, 0, Zt), Ln.length > 0) Mn = Ln.reduce(function(Br, Pr) {
                  return Br + Pr;
                }, 0) / Ln.length;
                else return [];
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "COUNT") {
                if (Ln = Dt(In.args, 0, Zt), Ln.length > 0) Mn = Ln.length;
                else return [];
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "FIRST") {
                if (Ln = Dt(In.args, 0, Zt), Ln.length > 0) Mn = Ln[0];
                else return [];
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "LAST") {
                if (Ln = Dt(In.args, 0, Zt), Ln.length > 0) Mn = Ln[Ln.length - 1];
                else return [];
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "MIN") {
                if (Ln = Dt(In.args, 0, Zt), Ln.length === 0) return [];
                var Mn = Ln.reduce(function(Pr, ei) {
                  return Math.min(Pr, ei);
                }, 1 / 0);
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "MAX") {
                var Ln = Dt(In.args, 0, Zt);
                if (Ln.length === 0) return [];
                var Mn = Ln.reduce(function(ei, Ci) {
                  return Math.max(ei, Ci);
                }, -1 / 0);
                return an + 1 + 1 > bn.length ? [Mn] : Dt(bn, an + 1, Mn);
              } else if (In.selid === "PLUS") {
                var Mr = [], Wr = Dt(In.args, 0, Zt).slice();
                an + 1 + 1 > bn.length ? Mr = Mr.concat(Wr) : Wr.forEach(function(ei) {
                  Mr = Mr.concat(Dt(bn, an + 1, ei));
                });
                for (var Bi = 0; Wr.length > 0; ) {
                  var Ln = Wr.shift();
                  if (Ln = Dt(In.args, 0, Ln), Wr = Wr.concat(Ln), an + 1 + 1 > bn.length ? Mr = Mr.concat(Ln) : Ln.forEach(function(oa) {
                    var ka = Dt(bn, an + 1, oa);
                    Mr = Mr.concat(ka);
                  }), Bi++, Bi > kn) throw new Error("Infinite loop brake. Number of iterations = " + Bi);
                }
                return Mr;
              } else if (In.selid === "STAR") {
                var Mr = [];
                Mr = Dt(bn, an + 1, Zt);
                var Wr = Dt(In.args, 0, Zt).slice();
                an + 1 + 1 > bn.length ? Mr = Mr.concat(Wr) : Wr.forEach(function(ei) {
                  Mr = Mr.concat(Dt(bn, an + 1, ei));
                });
                for (var Bi = 0; Wr.length > 0; ) {
                  var Ln = Wr[0];
                  if (Wr.shift(), Ln = Dt(In.args, 0, Ln), Wr = Wr.concat(Ln), an + 1 + 1 <= bn.length && Ln.forEach(function(oa) {
                    Mr = Mr.concat(Dt(bn, an + 1, oa));
                  }), Bi++, Bi > kn) throw new Error("Infinite loop brake. Number of iterations = " + Bi);
                }
                return Mr;
              } else if (In.selid === "QUESTION") {
                var Mr = [];
                Mr = Mr.concat(Dt(bn, an + 1, Zt));
                var Ln = Dt(In.args, 0, Zt);
                return an + 1 + 1 <= bn.length && Ln.forEach(function(ei) {
                  Mr = Mr.concat(Dt(bn, an + 1, ei));
                }), Mr;
              } else if (In.selid === "WITH") {
                var Ln = Dt(In.args, 0, Zt);
                if (Ln.length === 0) return [];
                var bi = { status: 1, values: Ln };
              } else {
                if (In.selid === "ROOT") return an + 1 + 1 > bn.length ? [Zt] : Dt(bn, an + 1, Bt);
                throw new Error("Wrong selector " + In.selid);
              }
            } else if (In.srchid) var bi = H.srch[In.srchid.toUpperCase()](Zt, In.args, wt, rt);
            else throw new Error("Selector not found");
            typeof bi > "u" && (bi = { status: 1, values: [Zt] });
            var Zi = [];
            if (bi.status === 1) {
              var $i = bi.values;
              if (an + 1 + 1 > bn.length) Zi = $i;
              else for (var Bi = 0; Bi < bi.values.length; Bi++) Zi = Zi.concat(Dt(bn, an + 1, $i[Bi]));
            }
            return Zi;
          }
          if (At !== void 0 && At.length > 0 && (At && At[0] && At[0].srchid === "PROP" && At[0].args && At[0].args[0] && (At[0].args[0].toUpperCase() === "XML" ? (wt.mode = "XML", At.shift()) : At[0].args[0].toUpperCase() === "HTML" ? (wt.mode = "HTML", At.shift()) : At[0].args[0].toUpperCase() === "JSON" && (wt.mode = "JSON", At.shift())), At.length > 0 && At[0].srchid === "VALUE" && (wt.value = !0, At.shift())), this.from instanceof ze.Column) {
            var Ot = this.from.databaseid || Ie;
            Bt = H.databases[Ot].tables[this.from.columnid].data;
          } else if (this.from instanceof ze.FuncValue && H.from[this.from.funcid.toUpperCase()]) {
            var un = this.from.args.map(function(bn) {
              var an = bn.toJS(), Zt = new Function("params,alasql", "var y;return " + an).bind(this);
              return Zt(rt, H);
            });
            Bt = H.from[this.from.funcid.toUpperCase()].apply(this, un);
          } else if (typeof this.from > "u") Bt = H.databases[Ie].objects;
          else {
            var yn = new Function("params,alasql", "var y;return " + this.from.toJS());
            Bt = yn(rt, H), typeof Mongo == "object" && typeof Mongo.Collection != "object" && Bt instanceof Mongo.Collection && (Bt = Bt.find().fetch());
          }
          if (At !== void 0 && At.length > 0 ? pt = Dt(At, 0, Bt) : pt = Bt, this.into) {
            var _n, gn;
            typeof this.into.args[0] < "u" && (_n = new Function("params,alasql", "var y;return " + this.into.args[0].toJS())(rt, H)), typeof this.into.args[1] < "u" && (gn = new Function("params,alasql", "var y;return " + this.into.args[1].toJS())(rt, H)), pt = H.into[this.into.funcid.toUpperCase()](_n, gn, pt, [], ft);
          } else wt.value && pt.length > 0 && (pt = pt[0]), ft && (pt = ft(pt));
          return pt;
        }, Pn), H.srch = { PROP(Ie, rt, ft) {
          if (ft.mode === "XML") {
            const pt = Ie.children.filter((wt) => wt.name.toUpperCase() === rt[0].toUpperCase());
            return { status: pt.length ? 1 : -1, values: pt };
          } else return typeof Ie != "object" || Ie === null || typeof rt != "object" || typeof Ie[rt[0]] > "u" ? { status: -1, values: [] } : { status: 1, values: [Ie[rt[0]]] };
        }, APROP(Ie, rt) {
          return typeof Ie != "object" || Ie === null || typeof rt != "object" || typeof Ie[rt[0]] > "u" ? { status: 1, values: [void 0] } : { status: 1, values: [Ie[rt[0]]] };
        }, EQ(Ie, rt, ft, pt) {
          var wt = rt[0].toJS("x", ""), Bt = new Function("x,alasql,params", "return " + wt);
          return Ie === Bt(Ie, H, pt) ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, LIKE(Ie, rt, ft, pt) {
          var wt = rt[0].toJS("x", ""), Bt = new Function("x,alasql,params", "return " + wt);
          return Ie.toUpperCase().match(new RegExp("^" + Bt(Ie, H, pt).toUpperCase().replace(/%/g, ".*").replace(/\?|_/g, ".") + "$"), "g") ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, ATTR(Ie, rt, ft) {
          if (ft.mode === "XML") return typeof rt > "u" ? { status: 1, values: [Ie.attributes] } : typeof Ie == "object" && typeof Ie.attributes == "object" && typeof Ie.attributes[rt[0]] < "u" ? { status: 1, values: [Ie.attributes[rt[0]]] } : { status: -1, values: [] };
          throw new Error("ATTR is not using in usual mode");
        }, CONTENT(Ie, rt, ft) {
          if (ft.mode !== "XML") throw new Error("ATTR is not using in usual mode");
          return { status: 1, values: [Ie.content] };
        }, SHARP(Ie, rt) {
          const ft = H.databases[H.useid].objects[rt[0]];
          return Ie !== void 0 && Ie === ft ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, PARENT() {
          return console.error("PARENT not implemented", arguments), { status: -1, values: [] };
        }, CHILD(Ie, rt, ft) {
          return typeof Ie == "object" ? Array.isArray(Ie) ? { status: 1, values: Ie } : ft.mode === "XML" ? { status: 1, values: Object.keys(Ie.children).map(function(pt) {
            return Ie.children[pt];
          }) } : { status: 1, values: Object.keys(Ie).map(function(pt) {
            return Ie[pt];
          }) } : { status: 1, values: [] };
        }, KEYS(Ie) {
          return typeof Ie == "object" && Ie !== null ? { status: 1, values: Object.keys(Ie) } : { status: 1, values: [] };
        }, WHERE(Ie, rt, ft, pt) {
          var wt = rt[0].toJS("x", ""), Bt = new Function("x,alasql,params", "return " + wt);
          return Bt(Ie, H, pt) ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, NAME(Ie, rt) {
          return Ie.name === rt[0] ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, CLASS(Ie, rt) {
          return Ie.$class == rt ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, VERTEX(Ie) {
          return Ie.$node === "VERTEX" ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, INSTANCEOF(Ie, rt) {
          return Ie instanceof H.fn[rt[0]] ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, EDGE(Ie) {
          return Ie.$node === "EDGE" ? { status: 1, values: [Ie] } : { status: -1, values: [] };
        }, EX(Ie, rt, ft, pt) {
          var wt = rt[0].toJS("x", ""), Bt = new Function("x,alasql,params", "return " + wt);
          return { status: 1, values: [Bt(Ie, H, pt)] };
        }, RETURN(Ie, rt, ft, pt) {
          var wt = {};
          return rt && rt.length > 0 && rt.forEach(function(Bt) {
            var At = Bt.toJS("x", ""), Dt = new Function("x,alasql,params", "return " + At);
            typeof Bt.as > "u" && (Bt.as = Bt.toString()), wt[Bt.as] = Dt(Ie, H, pt);
          }), { status: 1, values: [wt] };
        }, REF(Ie) {
          return { status: 1, values: [H.databases[H.useid].objects[Ie]] };
        }, OUT(Ie) {
          if (Ie.$out && Ie.$out.length > 0) {
            var rt = Ie.$out.map(function(ft) {
              return H.databases[H.useid].objects[ft];
            });
            return { status: 1, values: rt };
          } else return { status: -1, values: [] };
        }, OUTOUT(Ie) {
          if (Ie.$out && Ie.$out.length > 0) {
            var rt = [];
            return Ie.$out.forEach(function(ft) {
              var pt = H.databases[H.useid].objects[ft];
              pt && pt.$out && pt.$out.length > 0 && pt.$out.forEach(function(wt) {
                rt = rt.concat(H.databases[H.useid].objects[wt]);
              });
            }), { status: 1, values: rt };
          } else return { status: -1, values: [] };
        }, IN(Ie) {
          if (Ie.$in && Ie.$in.length > 0) {
            var rt = Ie.$in.map(function(ft) {
              return H.databases[H.useid].objects[ft];
            });
            return { status: 1, values: rt };
          } else return { status: -1, values: [] };
        }, ININ(Ie) {
          if (Ie.$in && Ie.$in.length > 0) {
            var rt = [];
            return Ie.$in.forEach(function(ft) {
              var pt = H.databases[H.useid].objects[ft];
              pt && pt.$in && pt.$in.length > 0 && pt.$in.forEach(function(wt) {
                rt = rt.concat(H.databases[H.useid].objects[wt]);
              });
            }), { status: 1, values: rt };
          } else return { status: -1, values: [] };
        }, AS(Ie, rt) {
          return H.vars[rt[0]] = Ie, { status: 1, values: [Ie] };
        }, AT(Ie, rt) {
          var ft = H.vars[rt[0]];
          return { status: 1, values: [ft] };
        }, CLONEDEEP(Ie) {
          var rt = be(Ie);
          return { status: 1, values: [rt] };
        }, SET(Ie, rt, ft, pt) {
          var wt = rt.map(function(At) {
            return At.method === "@" ? `alasql.vars[${JSON.stringify(At.variable)}]=` + At.expression.toJS("x", "") : At.method === "$" ? `params[${JSON.stringify(At.variable)}]=` + At.expression.toJS("x", "") : `x[${JSON.stringify(At.column.columnid)}]=` + At.expression.toJS("x", "");
          }).join(";"), Bt = new Function("x,params,alasql", wt);
          return Bt(Ie, pt, H), { status: 1, values: [Ie] };
        }, ROW(Ie, rt, ft, pt) {
          var wt = "var y;return [";
          wt += rt.map((Dt) => Dt.toJS("x", "")).join(","), wt += "]";
          var Bt = new Function("x,params,alasql", wt), At = Bt(Ie, pt, H);
          return { status: 1, values: [At] };
        }, D3(Ie) {
          return Ie.$node !== "VERTEX" && Ie.$node === "EDGE" && (Ie.source = Ie.$in[0], Ie.target = Ie.$out[0]), { status: 1, values: [Ie] };
        }, ORDERBY(Ie, rt) {
          var ft = Ie.sort(We(rt));
          return { status: 1, values: ft };
        } };
        var We = function(Ie) {
          var wt;
          if (Ie) {
            if (typeof ((wt = Ie == null ? void 0 : Ie[0]) == null ? void 0 : wt.expression) == "function") {
              var rt = Ie[0].expression;
              return function(Bt, At) {
                var Dt = rt(Bt), Ot = rt(At);
                return Dt > Ot ? 1 : Dt === Ot ? 0 : -1;
              };
            }
            var ft = "", pt = "";
            return Ie.forEach(function(Bt) {
              var At = "";
              if (Bt.expression instanceof ze.NumValue && (Bt.expression = self.columns[Bt.expression.value - 1]), Bt.expression instanceof ze.Column) {
                var Dt = Bt.expression.columnid;
                H.options.valueof && (At = ".valueOf()"), Bt.nocase && (At += ".toUpperCase()"), Dt === "_" ? (ft += "if(a" + At + (Bt.direction === "ASC" ? ">" : "<") + "b" + At + ")return 1;", ft += "if(a" + At + "==b" + At + "){") : ft += `if (
							(a[${JSON.stringify(Dt)}]||'')${At}
							${Bt.direction === "ASC" ? ">" : "<"}
							(b[${JSON.stringify(Dt)}]||'')${At}
						) return 1;

						if(
							(a[${JSON.stringify(Dt)}]||'')${At}
							==
							(b[${JSON.stringify(Dt)}]||'')${At}
						){
						`;
              } else At = ".valueOf()", Bt.nocase && (At += ".toUpperCase()"), ft += `
					if (
						(${Bt.toJS("a", "")} || '')${At}
						${Bt.direction === "ASC" ? ">" : "<"}
						(${Bt.toJS("b", "")} || '')${At}
					) return 1;

					if (
						(${Bt.toJS("a", "")} || '')${At} ==
						(${Bt.toJS("b", "")} || '')${At}
					) {`;
              pt += "}";
            }), ft += "return 0;", ft += pt + "return -1", new Function("a,b", ft);
          }
        };
        function at(Ie, rt, ft, pt, wt) {
          Ie.sourceslen = Ie.sources.length;
          let Bt = Ie.sourceslen;
          Ie.query = Ie, Ie.A = pt, Ie.B = wt, Ie.cb = ft, Ie.oldscope = rt, Ie.queriesfn && (Ie.sourceslen += Ie.queriesfn.length, Bt += Ie.queriesfn.length, Ie.queriesdata = [], Ie.queriesfn.forEach(function(Dt, Ot) {
            Dt.query.params = Ie.params, ct([], -Ot - 1, Ie);
          })), Ie.scope = rt ? be(rt) : {};
          let At;
          if (Ie.sources.forEach(function(Dt, Ot) {
            Dt.query = Ie;
            var un = Dt.datafn(Ie, Ie.params, ct, Ot, H);
            typeof un < "u" && ((Ie.intofn || Ie.intoallfn) && Array.isArray(un) && (un = un.length), At = un), Dt.queriesdata = Ie.queriesdata;
          }), Ie.sources.length == 0 || Bt === 0) try {
            At = et(Ie);
          } catch (Dt) {
            return ft(null, Dt);
          }
          return At;
        }
        function ct(Ie, rt, ft) {
          if (rt >= 0) {
            let pt = ft.sources[rt];
            pt.data = Ie, typeof pt.data == "function" && (pt.getfn = pt.data, pt.dontcache = pt.getfn.dontcache, ["OUTER", "RIGHT", "ANTI"].includes(pt.joinmode) && (pt.dontcache = !1), pt.data = {});
          } else ft.queriesdata[-rt - 1] = xe(Ie);
          if (ft.sourceslen--, !(ft.sourceslen > 0)) return et(ft);
        }
        function et(Ie) {
          var rt = Ie.scope, ft;
          it(Ie), Ie.data = [], Ie.xgroups = {}, Ie.groups = [];
          var pt = 0;
          if (st(Ie, rt, pt), Ie.groupfn) {
            if (Ie.data = [], Ie.groups.length === 0 && Ie.allgroups.length === 0) {
              var wt = {};
              Ie.selectGroup.length > 0 && Ie.selectGroup.forEach(function(kn) {
                kn.aggregatorid == "COUNT" || kn.aggregatorid == "SUM" || kn.aggregatorid == "TOTAL" ? wt[kn.nick] = 0 : wt[kn.nick] = void 0;
              }), Ie.groups = [wt];
            }
            if (Ie.aggrKeys.length > 0) {
              var Bt = "";
              Ie.aggrKeys.forEach(function(kn) {
                Bt += `
				g[${JSON.stringify(kn.nick)}] = alasql.aggr[${JSON.stringify(kn.funcid)}](undefined,g[${JSON.stringify(kn.nick)}],3); `;
              });
              var At = new Function("g,params,alasql", "var y;" + Bt);
            }
            for (var Dt = 0, Ot = Ie.groups.length; Dt < Ot; Dt++) {
              var wt = Ie.groups[Dt];
              if (At && At(wt, Ie.params, H), !Ie.havingfn || Ie.havingfn(wt, Ie.params, H)) {
                var un = Ie.selectgfn(wt, Ie.params, H);
                for (const er in Ie.groupColumns) Ie.groupColumns[er] !== er && un[Ie.groupColumns[er]] && !Ie.groupColumns[Ie.groupColumns[er]] && delete un[Ie.groupColumns[er]];
                Ie.data.push(un);
              }
            }
          }
          if (Je(Ie), Ie.unionallfn) {
            var yn, _n;
            if (Ie.corresponding) Ie.unionallfn.query.modifier || (Ie.unionallfn.query.modifier = void 0), yn = Ie.unionallfn(Ie.params);
            else {
              Ie.unionallfn.query.modifier || (Ie.unionallfn.query.modifier = "RECORDSET"), _n = Ie.unionallfn(Ie.params), yn = [], Ot = _n.data.length;
              for (var Dt = 0; Dt < Ot; Dt++) {
                var gn = {};
                if (Ie.columns.length) {
                  ft = Math.min(Ie.columns.length, _n.columns.length);
                  for (var bn = 0; bn < ft; bn++) gn[Ie.columns[bn].columnid] = _n.data[Dt][_n.columns[bn].columnid];
                } else {
                  ft = _n.columns.length;
                  for (var bn = 0; bn < ft; bn++) gn[_n.columns[bn].columnid] = _n.data[Dt][_n.columns[bn].columnid];
                }
                yn.push(gn);
              }
            }
            Ie.data = Ie.data.concat(yn);
          } else if (Ie.unionfn) {
            if (Ie.corresponding) Ie.unionfn.query.modifier || (Ie.unionfn.query.modifier = "ARRAY"), yn = Ie.unionfn(Ie.params);
            else {
              Ie.unionfn.query.modifier || (Ie.unionfn.query.modifier = "RECORDSET"), _n = Ie.unionfn(Ie.params), yn = [], Ot = _n.data.length;
              for (var Dt = 0; Dt < Ot; Dt++) {
                if (gn = {}, Ie.columns.length) {
                  ft = Math.min(Ie.columns.length, _n.columns.length);
                  for (var bn = 0; bn < ft; bn++) gn[Ie.columns[bn].columnid] = _n.data[Dt][_n.columns[bn].columnid];
                } else {
                  ft = _n.columns.length;
                  for (var bn = 0; bn < ft; bn++) gn[_n.columns[bn].columnid] = _n.data[Dt][_n.columns[bn].columnid];
                }
                yn.push(gn);
              }
            }
            Ie.data = ye(Ie.data, yn);
          } else if (Ie.exceptfn) {
            if (Ie.corresponding) {
              Ie.exceptfn.query.modifier || (Ie.exceptfn.query.modifier = "ARRAY");
              var yn = Ie.exceptfn(Ie.params);
            } else {
              Ie.exceptfn.query.modifier || (Ie.exceptfn.query.modifier = "RECORDSET");
              for (var _n = Ie.exceptfn(Ie.params), yn = [], Dt = 0, Ot = _n.data.length; Dt < Ot; Dt++) {
                for (var gn = {}, bn = Math.min(Ie.columns.length, _n.columns.length) - 1; 0 <= bn; bn--) gn[Ie.columns[bn].columnid] = _n.data[Dt][_n.columns[bn].columnid];
                yn.push(gn);
              }
            }
            Ie.data = ge(Ie.data, yn);
          } else if (Ie.intersectfn) {
            if (Ie.corresponding) Ie.intersectfn.query.modifier || (Ie.intersectfn.query.modifier = void 0), yn = Ie.intersectfn(Ie.params);
            else for (Ie.intersectfn.query.modifier || (Ie.intersectfn.query.modifier = "RECORDSET"), _n = Ie.intersectfn(Ie.params), yn = [], Ot = _n.data.length, Dt = 0; Dt < Ot; Dt++) {
              for (gn = {}, ft = Math.min(Ie.columns.length, _n.columns.length), bn = 0; bn < ft; bn++) gn[Ie.columns[bn].columnid] = _n.data[Dt][_n.columns[bn].columnid];
              yn.push(gn);
            }
            Ie.data = ve(Ie.data, yn);
          }
          if (Ie.orderfn) {
            if (Ie.explain) var an = Date.now();
            Ie.data = Ie.data.sort(Ie.orderfn), Ie.explain && Ie.explaination.push({ explid: Ie.explid++, description: "QUERY BY", ms: Date.now() - an });
          }
          if (dt(Ie), typeof angular < "u" && Ie.removeKeys.push("$$hashKey"), Ie.removeKeys.length > 0) {
            var Zt = Ie.removeKeys;
            if (ft = Zt.length, ft > 0) for (Ot = Ie.data.length, Dt = 0; Dt < Ot; Dt++) for (bn = 0; bn < ft; bn++) delete Ie.data[Dt][Zt[bn]];
            Ie.columns.length > 0 && (Ie.columns = Ie.columns.filter(function(kn) {
              var er = !1;
              return Zt.forEach(function(jn) {
                kn.columnid == jn && (er = !0);
              }), !er;
            }));
          }
          if (typeof Ie.removeLikeKeys < "u" && Ie.removeLikeKeys.length > 0) {
            for (var Mn = Ie.removeLikeKeys, Dt = 0, Ot = Ie.data.length; Dt < Ot; Dt++) {
              gn = Ie.data[Dt];
              for (var Ln in gn) for (bn = 0; bn < Ie.removeLikeKeys.length; bn++) H.utils.like(Ie.removeLikeKeys[bn], Ln) && delete gn[Ln];
            }
            Ie.columns.length > 0 && (Ie.columns = Ie.columns.filter(function(kn) {
              var er = !1;
              return Mn.forEach(function(jn) {
                H.utils.like(jn, kn.columnid) && (er = !0);
              }), !er;
            }));
          }
          if (Ie.pivotfn && Ie.pivotfn(), Ie.unpivotfn && Ie.unpivotfn(), Ie.intoallfn) {
            var In = Ie.intoallfn(Ie.columns, Ie.cb, Ie.params, Ie.alasql);
            return In;
          }
          if (Ie.intofn) {
            for (Ot = Ie.data.length, Dt = 0; Dt < Ot; Dt++) Ie.intofn(Ie.data[Dt], Dt, Ie.params, Ie.alasql);
            return Ie.cb && Ie.cb(Ie.data.length, Ie.A, Ie.B), Ie.data.length;
          }
          return In = Ie.data, Ie.cb && (In = Ie.cb(Ie.data, Ie.A, Ie.B)), In;
        }
        function dt(Ie) {
          if (Ie.limit) {
            var rt = 0;
            Ie.offset && (rt = Ie.offset | 0 || 0, rt = rt < 0 ? 0 : rt);
            var ft;
            Ie.percent ? ft = (Ie.data.length * Ie.limit / 100 | 0) + rt : ft = (Ie.limit | 0) + rt, Ie.data = Ie.data.slice(rt, ft);
          }
        }
        function Je(Ie) {
          if (Ie.distinct) {
            for (var rt = {}, ft = Object.keys(Ie.data[0] || []), pt = 0, wt = Ie.data.length; pt < wt; pt++) {
              var Bt = ft.map(function(Dt) {
                return Ie.data[pt][Dt];
              }).join("`");
              rt[Bt] = Ie.data[pt];
            }
            Ie.data = [];
            for (var At in rt) Ie.data.push(rt[At]);
          }
        }
        var it = function(Ie) {
          for (var rt = 0, ft = Ie.sources.length; rt < ft; rt++) {
            var pt = Ie.sources[rt];
            if (delete pt.ix, rt > 0 && pt.optimization == "ix" && pt.onleftfn && pt.onrightfn) {
              if (pt.databaseid && H.databases[pt.databaseid].tables[pt.tableid]) {
                H.databases[pt.databaseid].tables[pt.tableid].indices || (Ie.database.tables[pt.tableid].indices = {});
                let Dt = H.databases[pt.databaseid].tables[pt.tableid].indices[fe(pt.onrightfns + "`" + pt.srcwherefns)];
                !H.databases[pt.databaseid].tables[pt.tableid].dirty && Dt && (pt.ix = Dt);
              }
              if (!pt.ix) {
                pt.ix = {};
                let Dt = {}, Ot = 0, un = pt.data.length, yn;
                for (; (yn = pt.data[Ot]) || pt.getfn && (yn = pt.getfn(Ot)) || Ot < un; ) {
                  if (pt.getfn && !pt.dontcache && (pt.data[Ot] = yn), Dt[pt.alias || pt.tableid] = yn, pt.srcwherefn(Dt, Ie.params, H)) {
                    var wt = pt.onrightfn(Dt, Ie.params, H), Bt = pt.ix[wt];
                    Bt || (Bt = pt.ix[wt] = []), Bt.push(yn);
                  }
                  Ot++;
                }
                pt.databaseid && H.databases[pt.databaseid].tables[pt.tableid] && (H.databases[pt.databaseid].tables[pt.tableid].indices[fe(pt.onrightfns + "`" + pt.srcwherefns)] = pt.ix);
              }
            } else if (pt.wxleftfn) {
              if (H.databases[pt.databaseid].engineid || (ixx = H.databases[pt.databaseid].tables[pt.tableid].indices[fe(pt.wxleftfns + "`")]), !H.databases[pt.databaseid].tables[pt.tableid].dirty && ixx) pt.ix = ixx, pt.data = pt.ix[pt.wxrightfn(null, Ie.params, H)];
              else {
                for (pt.ix = {}, At = {}, Gt = 0, ilen = pt.data.length, dataw; (dataw = pt.data[Gt]) || pt.getfn && (dataw = pt.getfn(Gt)) || Gt < ilen; ) pt.getfn && !pt.dontcache && (pt.data[Gt] = dataw), At[pt.alias || pt.tableid] = pt.data[Gt], wt = pt.wxleftfn(At, Ie.params, H), Bt = pt.ix[wt], Bt || (Bt = pt.ix[wt] = []), Bt.push(pt.data[Gt]), Gt++;
                H.databases[pt.databaseid].engineid || (H.databases[pt.databaseid].tables[pt.tableid].indices[fe(pt.wxleftfns + "`")] = pt.ix);
              }
              pt.srcwherefns && (pt.data ? (At = {}, pt.data = pt.data.filter(function(Dt) {
                return At[pt.alias] = Dt, pt.srcwherefn(At, Ie.params, H);
              })) : pt.data = []);
            } else if (pt.srcwherefns && !pt.dontcache) if (pt.data) {
              var At = {};
              pt.data = pt.data.filter(function(Ot) {
                return At[pt.alias] = Ot, pt.srcwherefn(At, Ie.params, H);
              }), At = {}, Gt = 0, ilen = pt.data.length;
              let Dt = [];
              for (; (dataw = pt.data[Gt]) || pt.getfn && (dataw = pt.getfn(Gt)) || Gt < ilen; ) pt.getfn && !pt.dontcache && (pt.data[Gt] = dataw), At[pt.alias] = dataw, pt.srcwherefn(At, Ie.params, H) && Dt.push(dataw), Gt++;
              pt.data = Dt;
            } else pt.data = [];
            pt.databaseid && H.databases[pt.databaseid].tables[pt.tableid];
          }
        };
        function st(Ie, rt, ft) {
          if (ft >= Ie.sources.length) Ie.wherefn(rt, Ie.params, H) && (Ie.groupfn ? Ie.groupfn(rt, Ie.params, H) : Ie.data.push(Ie.selectfn(rt, Ie.params, H)));
          else if (Ie.sources[ft].applyselect) {
            var pt = Ie.sources[ft];
            pt.applyselect(Ie.params, function(Ot) {
              if (Ot.length > 0) for (var un = 0; un < Ot.length; un++) rt[pt.alias] = Ot[un], st(Ie, rt, ft + 1);
              else pt.applymode == "OUTER" && (rt[pt.alias] = {}, st(Ie, rt, ft + 1));
            }, rt);
          } else {
            let Ot = Ie.sources[ft], un = Ie.sources[ft + 1], yn = Ot.alias || Ot.tableid, _n = !1, gn = Ot.data, bn = !1;
            (!Ot.getfn || Ot.getfn && !Ot.dontcache) && Ot.joinmode != "RIGHT" && Ot.joinmode != "OUTER" && Ot.joinmode != "ANTI" && Ot.optimization == "ix" && (gn = Ot.ix[Ot.onleftfn(rt, Ie.params, H)] || [], bn = !0);
            let an = 0;
            if (typeof gn > "u") throw new Error("Data source number " + ft + " in undefined");
            let Zt = gn.length, Mn;
            for (; (Mn = gn[an]) || !bn && Ot.getfn && (Mn = Ot.getfn(an)) || an < Zt; ) {
              !bn && Ot.getfn && !Ot.dontcache && (gn[an] = Mn), rt[yn] = Mn;
              var wt = !Ot.onleftfn;
              if (!wt) {
                var Bt = Ot.onleftfn(rt, Ie.params, H), At = Ot.onrightfn(rt, Ie.params, H);
                (Bt instanceof String || Bt instanceof Number) && (Bt = Bt.valueOf()), (At instanceof String || At instanceof Number) && (At = Bt.valueOf()), wt = Bt == At;
              }
              wt && Ot.onmiddlefn(rt, Ie.params, H) && (Ot.joinmode != "SEMI" && Ot.joinmode != "ANTI" && st(Ie, rt, ft + 1), Ot.joinmode != "LEFT" && Ot.joinmode != "INNER" && (Mn._rightjoin = !0), _n = !0), an++;
            }
            if ((Ot.joinmode == "LEFT" || Ot.joinmode == "OUTER" || Ot.joinmode == "SEMI") && !_n && (rt[yn] = {}, st(Ie, rt, ft + 1)), ft == 0) for (var Dt = ft + 1; Dt < Ie.sources.length; Dt++) {
              if (un.joinmode == "OUTER" || un.joinmode == "RIGHT" || un.joinmode == "ANTI") {
                rt[Ot.alias] = {};
                let Ln = 0, In = un.data.length, kn;
                for (; (kn = un.data[Ln]) || un.getfn && (kn = un.getfn(Ln)) || Ln < In; ) un.getfn && !un.dontcache && (un.data[Ln] = kn), kn._rightjoin ? delete kn._rightjoin : (rt[un.alias] = kn, st(Ie, rt, Dt + 1)), Ln++;
              }
              Ot = Ie.sources[Dt], un = Ie.sources[Dt + 1];
            }
            rt[yn] = void 0;
          }
        }
        ze.Select = class {
          constructor(Ie) {
            Object.assign(this, Ie);
          }
          toString() {
            var Ie;
            return Ie = "", this.explain && (Ie += "EXPLAIN "), Ie += "SELECT ", this.modifier && (Ie += this.modifier + " "), this.distinct && (Ie += "DISTINCT "), this.top && (Ie += "TOP " + this.top.value + " ", this.percent && (Ie += "PERCENT ")), Ie += this.columns.map(function(rt) {
              var ft;
              return ft = rt.toString(), typeof rt.as < "u" && (ft += " AS " + rt.as), ft;
            }).join(", "), this.from && (Ie += " FROM " + this.from.map(function(rt) {
              var ft;
              return ft = rt.toString(), rt.as && (ft += " AS " + rt.as), ft;
            }).join(",")), this.joins && (Ie += this.joins.map(function(rt) {
              var ft;
              if (ft = " ", rt.joinmode && (ft += rt.joinmode + " "), rt.table) ft += "JOIN " + rt.table.toString();
              else if (rt.select) ft += "JOIN (" + rt.select.toString() + ")";
              else if (rt instanceof H.yy.Apply) ft += rt.toString();
              else throw new Error("Wrong type in JOIN mode");
              return rt.as && (ft += " AS " + rt.as), rt.using && (ft += " USING " + rt.using.toString()), rt.on && (ft += " ON " + rt.on.toString()), ft;
            }).join("")), this.where && (Ie += " WHERE " + this.where.toString()), this.group && this.group.length > 0 && (Ie += " GROUP BY " + this.group.map(function(rt) {
              return rt.toString();
            }).join(", ")), this.having && (Ie += " HAVING " + this.having.toString()), this.order && this.order.length > 0 && (Ie += " ORDER BY " + this.order.map(function(rt) {
              return rt.toString();
            }).join(", ")), this.limit && (Ie += " LIMIT " + this.limit.value), this.offset && (Ie += " OFFSET " + this.offset.value), this.union && (Ie += " UNION " + (this.corresponding ? "CORRESPONDING " : "") + this.union.toString()), this.unionall && (Ie += " UNION ALL " + (this.corresponding ? "CORRESPONDING " : "") + this.unionall.toString()), this.except && (Ie += " EXCEPT " + (this.corresponding ? "CORRESPONDING " : "") + this.except.toString()), this.intersect && (Ie += " INTERSECT " + (this.corresponding ? "CORRESPONDING " : "") + this.intersect.toString()), Ie;
          }
          toJS(Ie) {
            var rt = "alasql.utils.flatArray(this.queriesfn[" + (this.queriesidx - 1) + "](this.params,null," + Ie + "))[0]";
            return rt;
          }
          compile(Ie, rt) {
            var ft = H.databases[Ie], pt = new Ve();
            if (pt.removeKeys = [], pt.aggrKeys = [], pt.explain = this.explain, pt.explaination = [], pt.explid = 1, pt.modifier = this.modifier, pt.database = ft, this.compileWhereExists(pt), this.compileQueries(pt), pt.defcols = this.compileDefCols(pt, Ie), pt.fromfn = this.compileFrom(pt), this.joins && this.compileJoins(pt), pt.rownums = [], this.compileSelectGroup0(pt), this.group || pt.selectGroup.length > 0 ? pt.selectgfns = this.compileSelectGroup1(pt) : pt.selectfns = this.compileSelect1(pt, rt), this.compileRemoveColumns(pt), this.where && this.compileWhereJoins(pt), pt.wherefn = this.compileWhere(pt), (this.group || pt.selectGroup.length > 0) && (pt.groupfn = this.compileGroup(pt)), this.having && (pt.havingfn = this.compileHaving(pt)), this.order && (pt.orderfn = this.compileOrder(pt, rt)), this.group || pt.selectGroup.length > 0 ? pt.selectgfn = this.compileSelectGroup2(pt) : pt.selectfn = this.compileSelect2(pt, rt), pt.distinct = this.distinct, this.pivot && (pt.pivotfn = this.compilePivot(pt)), this.unpivot && (pt.pivotfn = this.compileUnpivot(pt)), this.top ? pt.limit = this.top.value : this.limit && (pt.limit = this.limit.value, this.offset && (pt.offset = this.offset.value)), pt.percent = this.percent, pt.corresponding = this.corresponding, this.union ? (pt.unionfn = this.union.compile(Ie), pt.orderfn = this.union.order ? this.union.compileOrder(pt, rt) : null) : this.unionall ? (pt.unionallfn = this.unionall.compile(Ie), pt.orderfn = this.unionall.order ? this.unionall.compileOrder(pt, rt) : null) : this.except ? (pt.exceptfn = this.except.compile(Ie), pt.orderfn = this.except.order ? this.except.compileOrder(pt, rt) : null) : this.intersect && (pt.intersectfn = this.intersect.compile(Ie), pt.orderfn = this.intersect.order ? this.intersect.compileOrder(pt, rt) : null), this.into) {
              if (this.into instanceof ze.Table) H.options.autocommit && H.databases[this.into.databaseid || Ie].engineid ? pt.intoallfns = `return alasql
								.engines[${JSON.stringify(H.databases[this.into.databaseid || Ie].engineid)}]
								.intoTable(
									${JSON.stringify(this.into.databaseid || Ie)},
									${JSON.stringify(this.into.tableid)},
									this.data,
									columns,
									cb
								);` : pt.intofns = `alasql
							.databases[${JSON.stringify(this.into.databaseid || Ie)}]
							.tables[${JSON.stringify(this.into.tableid)}]
							.data.push(r);
						`;
              else if (this.into instanceof ze.VarValue) pt.intoallfns = `
					alasql.vars[${JSON.stringify(this.into.variable)}]=this.data;
					res=this.data.length;
					if(cb) res = cb(res);
					return res;
				`;
              else if (this.into instanceof ze.FuncValue) {
                var wt = "return alasql.into[" + JSON.stringify(this.into.funcid.toUpperCase()) + "](";
                this.into.args && this.into.args.length > 0 ? (wt += this.into.args[0].toJS() + ",", this.into.args.length > 1 ? wt += this.into.args[1].toJS() + "," : wt += "undefined,") : wt += "undefined, undefined,", pt.intoallfns = wt + "this.data,columns,cb)";
              } else this.into instanceof ze.ParamValue && (pt.intofns = `params[${JSON.stringify(this.into.param)}].push(r)`);
              pt.intofns ? pt.intofn = new Function("r,i,params,alasql", "var y;" + pt.intofns) : pt.intoallfns && (pt.intoallfn = new Function("columns,cb,params,alasql", "var y;" + pt.intoallfns));
            }
            var Bt = function(At, Dt, Ot) {
              pt.params = At;
              var un = at(pt, Ot, function(yn, _n) {
                if (_n) {
                  if (Dt) return Dt(null, _n);
                  throw _n;
                }
                if (pt.rownums.length > 0) for (var gn = 0, bn = yn.length; gn < bn; gn++) for (var an = 0, Zt = pt.rownums.length; an < Zt; an++) yn[gn][pt.rownums[an]] = gn + 1;
                var Mn = gt(pt, yn);
                return Dt && Dt(Mn), Mn;
              });
              return un;
            };
            return Bt.query = pt, Bt;
          }
          execute(Ie, rt, ft) {
            return this.compile(Ie)(rt, ft);
          }
          compileWhereExists(Ie) {
            this.exists && (Ie.existsfn = this.exists.map(function(rt) {
              var ft = rt.compile(Ie.database.databaseid);
              return ft.query.modifier = "RECORDSET", ft;
            }));
          }
          compileQueries(Ie) {
            this.queries && (Ie.queriesfn = this.queries.map(function(rt) {
              var ft = rt.compile(Ie.database.databaseid);
              return ft.query.modifier = "RECORDSET", ft;
            }));
          }
        };
        function gt(Ie, rt) {
          if (typeof rt > "u" || typeof rt == "number" || typeof rt == "string" || typeof rt == "boolean") return rt;
          var ft = Ie.modifier || H.options.modifier, pt = Ie.columns;
          if (typeof pt > "u" || pt.length == 0) if (rt.length > 0) {
            for (var wt = {}, Bt = Math.min(rt.length, H.options.columnlookup || 10) - 1; 0 <= Bt; Bt--) for (var At in rt[Bt]) wt[At] = !0;
            pt = Object.keys(wt).map(function(Ot) {
              return { columnid: Ot };
            });
          } else pt = [];
          switch (ft) {
            case "VALUE":
              if (rt.length === 0) return;
              const Ot = pt && pt.length > 0 ? pt[0].columnid : Object.keys(rt[0])[0];
              return rt[0][Ot];
            case "ROW":
              return rt.length === 0 ? void 0 : Object.values(rt[0]);
            case "COLUMN":
              if (rt.length === 0) return [];
              let un;
              pt && pt.length > 0 ? un = pt[0].columnid : un = Object.keys(rt[0])[0];
              let yn = [];
              for (var Bt = 0, Dt = rt.length; Bt < Dt; Bt++) yn.push(rt[Bt][un]);
              return yn;
            case "MATRIX":
              return rt.length === 0 ? void 0 : rt.map((an) => pt.map((Zt) => an[Zt.columnid]));
            case "INDEX":
              if (rt.length === 0) return;
              const _n = pt && pt.length > 0 ? pt[0].columnid : Object.keys(rt[0])[0], gn = pt && pt.length > 1 ? pt[1].columnid : Object.keys(rt[0])[1];
              return rt.reduce((an, Zt) => ({ ...an, [Zt[_n]]: Zt[gn] }), {});
            case "RECORDSET":
              return new H.Recordset({ columns: pt, data: rt });
            case "TEXTSTRING":
              if (rt.length === 0) return;
              const bn = pt && pt.length > 0 ? pt[0].columnid : Object.keys(rt[0])[0];
              return rt.map((an) => an[bn]).join(`
`);
          }
          return rt;
        }
        ze.ExistsValue = class {
          constructor(Ie) {
            Object.assign(this, Ie);
          }
          toString() {
            return "EXISTS(" + this.value.toString() + ")";
          }
          toType() {
            return "boolean";
          }
          toJS(Ie, rt, ft) {
            return `!!this.existsfn[${this.existsidx}](params, null, ${Ie}).data.length`;
          }
        }, H.precompile = function(Ie, rt, ft) {
          Ie && (Ie.params = ft, Ie.queries && (Ie.queriesfn = Ie.queries.map(function(pt) {
            var wt = pt.compile(rt || Ie.database.databaseid);
            return wt.query.modifier = "RECORDSET", wt;
          })), Ie.exists && (Ie.existsfn = Ie.exists.map(function(pt) {
            var wt = pt.compile(rt || Ie.database.databaseid);
            return wt.query.modifier = "RECORDSET", wt;
          })));
        }, ze.Select.prototype.compileFrom = function(Ie) {
          const rt = this;
          Ie.sources = [], Ie.aliases = {}, rt.from && (rt.from.forEach((ft) => {
            const pt = ft.as || ft.tableid;
            if (ft instanceof ze.Table) Ie.aliases[pt] = { tableid: ft.tableid, databaseid: ft.databaseid || Ie.database.databaseid, type: "table" };
            else if (ft instanceof ze.Select) Ie.aliases[pt] = { type: "subquery" };
            else if (ft instanceof ze.Search) Ie.aliases[pt] = { type: "subsearch" };
            else if (ft instanceof ze.ParamValue) Ie.aliases[pt] = { type: "paramvalue" };
            else if (ft instanceof ze.FuncValue) Ie.aliases[pt] = { type: "funcvalue" };
            else if (ft instanceof ze.VarValue) Ie.aliases[pt] = { type: "varvalue" };
            else if (ft instanceof ze.FromData) Ie.aliases[pt] = { type: "fromdata" };
            else if (ft instanceof ze.Json) Ie.aliases[pt] = { type: "json" };
            else if (ft.inserted) Ie.aliases[pt] = { type: "inserted" };
            else throw new Error("Wrong table at FROM");
            const wt = { alias: pt, databaseid: ft.databaseid || Ie.database.databaseid, tableid: ft.tableid, joinmode: "INNER", onmiddlefn: ne, srcwherefns: "", srcwherefn: ne };
            if (ft instanceof ze.Table) wt.columns = H.databases[wt.databaseid].tables[wt.tableid].columns, H.options.autocommit && H.databases[wt.databaseid].engineid && !H.databases[wt.databaseid].tables[wt.tableid].view ? wt.datafn = (Bt, At, Dt, Ot, un) => un.engines[un.databases[wt.databaseid].engineid].fromTable(wt.databaseid, wt.tableid, Dt, Ot, Bt) : H.databases[wt.databaseid].tables[wt.tableid].view ? wt.datafn = (Bt, At, Dt, Ot, un) => {
              let yn = un.databases[wt.databaseid].tables[wt.tableid].select(At);
              return Dt && (yn = Dt(yn, Ot, Bt)), yn;
            } : wt.datafn = (Bt, At, Dt, Ot, un) => {
              let yn = un.databases[wt.databaseid].tables[wt.tableid].data;
              return Dt && (yn = Dt(yn, Ot, Bt)), yn;
            };
            else if (ft instanceof ze.Select) wt.subquery = ft.compile(Ie.database.databaseid), typeof wt.subquery.query.modifier > "u" && (wt.subquery.query.modifier = "RECORDSET"), wt.columns = wt.subquery.query.columns, wt.datafn = (Bt, At, Dt, Ot, un) => {
              let yn;
              return wt.subquery(Bt.params, (_n) => {
                yn = _n.data, Dt && (yn = Dt(yn, Ot, Bt));
              }), yn;
            };
            else if (ft instanceof ze.Search) wt.subsearch = ft, wt.columns = [], wt.datafn = (Bt, At, Dt, Ot, un) => {
              let yn;
              return wt.subsearch.execute(Bt.database.databaseid, Bt.params, (_n) => {
                yn = _n, Dt && (yn = Dt(yn, Ot, Bt));
              }), yn;
            };
            else if (ft instanceof ze.ParamValue) {
              let Bt = `var res = alasql.prepareFromData(params['${ft.param}']`;
              ft.array && (Bt += ",true"), Bt += ");if(cb)res=cb(res,idx,query);return res", wt.datafn = new Function("query,params,cb,idx,alasql", Bt);
            } else if (ft.inserted) {
              let Bt = "var res = alasql.prepareFromData(alasql.inserted";
              ft.array && (Bt += ",true"), Bt += ");if(cb)res=cb(res,idx,query);return res", wt.datafn = new Function("query,params,cb,idx,alasql", Bt);
            } else if (ft instanceof ze.Json) {
              let Bt = "var res = alasql.prepareFromData(" + ft.toJS();
              ft.array && (Bt += ",true"), Bt += ");if(cb)res=cb(res,idx,query);return res", wt.datafn = new Function("query,params,cb,idx,alasql", Bt);
            } else if (ft instanceof ze.VarValue) {
              let Bt = `var res = alasql.prepareFromData(alasql.vars['${ft.variable}']`;
              ft.array && (Bt += ",true"), Bt += ");if(cb)res=cb(res,idx,query);return res", wt.datafn = new Function("query,params,cb,idx,alasql", Bt);
            } else if (ft instanceof ze.FuncValue) {
              let Bt = "var res=alasql.from[" + JSON.stringify(ft.funcid.toUpperCase()) + "](";
              ft.args && ft.args.length > 0 ? (ft.args[0] ? Bt += ft.args[0].toJS("query.oldscope") + "," : Bt += "null,", ft.args[1] ? Bt += ft.args[1].toJS("query.oldscope") + "," : Bt += "null,") : Bt += "null,null,", Bt += "cb,idx,query); return res", wt.datafn = new Function("query,params,cb,idx,alasql", Bt);
            } else if (ft instanceof ze.FromData) wt.datafn = (Bt, At, Dt, Ot, un) => {
              let yn = ft.data;
              return Dt && (yn = Dt(yn, Ot, Bt)), yn;
            };
            else throw new Error("Wrong table at FROM");
            Ie.sources.push(wt);
          }), Ie.defaultTableid = Ie.sources[0].alias);
        }, H.prepareFromData = function(Ie, rt) {
          let ft = Ie;
          if (typeof Ie == "string") ft = Ie.split(/\r?\n/), rt && (ft = ft.map((pt) => [pt]));
          else if (rt) ft = Ie.map((pt) => [pt]);
          else if (typeof Ie == "object" && !Array.isArray(Ie)) if (typeof Mongo < "u" && typeof Mongo.Collection < "u" && Ie instanceof Mongo.Collection) ft = Ie.find().fetch();
          else {
            ft = [];
            for (const pt in Ie) Ie.hasOwnProperty(pt) && ft.push([pt, Ie[pt]]);
          }
          return ft;
        }, ze.Select.prototype.compileJoins = function(Ie) {
          this.joins.forEach((rt) => {
            let ft, pt, wt;
            if (rt.joinmode === "CROSS") {
              if (rt.using || rt.on) throw new Error("CROSS JOIN cannot have USING or ON clauses");
              rt.joinmode = "INNER";
            }
            if (rt instanceof ze.Apply) {
              wt = { alias: rt.as, applymode: rt.applymode, onmiddlefn: ne, srcwherefns: "", srcwherefn: ne, columns: [] }, wt.applyselect = rt.select.compile(Ie.database.databaseid), wt.columns = wt.applyselect.query.columns, wt.datafn = function(At, Dt, Ot, un, yn) {
                let _n;
                return Ot && (_n = Ot(_n, un, At)), _n;
              }, Ie.sources.push(wt);
              return;
            }
            if (rt.table) {
              if (ft = rt.table, wt = { alias: rt.as || ft.tableid, databaseid: ft.databaseid || Ie.database.databaseid, tableid: ft.tableid, joinmode: rt.joinmode, onmiddlefn: ne, srcwherefns: "", srcwherefn: ne, columns: [] }, !H.databases[wt.databaseid].tables[wt.tableid]) throw new Error("Table '" + wt.tableid + "' is not exists in database '" + wt.databaseid + "'");
              wt.columns = H.databases[wt.databaseid].tables[wt.tableid].columns, H.options.autocommit && H.databases[wt.databaseid].engineid ? wt.datafn = function(At, Dt, Ot, un, yn) {
                return yn.engines[yn.databases[wt.databaseid].engineid].fromTable(wt.databaseid, wt.tableid, Ot, un, At);
              } : H.databases[wt.databaseid].tables[wt.tableid].view ? wt.datafn = function(At, Dt, Ot, un, yn) {
                let _n = yn.databases[wt.databaseid].tables[wt.tableid].select(Dt);
                return Ot && (_n = Ot(_n, un, At)), _n;
              } : wt.datafn = function(At, Dt, Ot, un, yn) {
                let _n = yn.databases[wt.databaseid].tables[wt.tableid].data;
                return Ot && (_n = Ot(_n, un, At)), _n;
              }, Ie.aliases[wt.alias] = { tableid: ft.tableid, databaseid: ft.databaseid || Ie.database.databaseid };
            } else if (rt.select) ft = rt.select, wt = { alias: rt.as, joinmode: rt.joinmode, onmiddlefn: ne, srcwherefns: "", srcwherefn: ne, columns: [] }, wt.subquery = ft.compile(Ie.database.databaseid), typeof wt.subquery.query.modifier > "u" && (wt.subquery.query.modifier = "RECORDSET"), wt.columns = wt.subquery.query.columns, wt.datafn = function(At, Dt, Ot, un, yn) {
              wt.data = wt.subquery(At.params, null, Ot, un).data;
              let _n = wt.data;
              return Ot && (_n = Ot(_n, un, At)), _n;
            }, Ie.aliases[wt.alias] = { type: "subquery" };
            else if (rt.param) wt = { alias: rt.as, joinmode: rt.joinmode, onmiddlefn: ne, srcwherefns: "", srcwherefn: ne }, pt = "let res=alasql.prepareFromData(params['" + rt.param.param + "']", rt.array && (pt += ",true"), pt += "); if(cb) res=cb(res, idx, query); return res", wt.datafn = new Function("query,params,cb,idx, alasql", pt), Ie.aliases[wt.alias] = { type: "paramvalue" };
            else if (rt.variable) wt = { alias: rt.as, joinmode: rt.joinmode, onmiddlefn: ne, srcwherefns: "", srcwherefn: ne }, pt = "let res=alasql.prepareFromData(alasql.vars['" + rt.variable + "']", rt.array && (pt += ", true"), pt += "); if(cb)res=cb(res, idx, query);return res", wt.datafn = new Function("query,params,cb,idx, alasql", pt), Ie.aliases[wt.alias] = { type: "varvalue" };
            else if (rt.func) {
              wt = { alias: rt.as, joinmode: rt.joinmode, onmiddlefn: ne, srcwherefns: "", srcwherefn: ne };
              let At = "let res=alasql.from[" + JSON.stringify(rt.func.funcid.toUpperCase()) + "](";
              const Dt = rt.func.args;
              Dt && Dt.length > 0 ? (Dt[0] ? At += Dt[0].toJS("query.oldscope") + ", " : At += "null, ", Dt[1] ? At += Dt[1].toJS("query.oldscope") + ", " : At += "null, ") : At += "null, null, ", At += "cb, idx, query); return res", wt.datafn = new Function("query, params, cb, idx, alasql", At), Ie.aliases[wt.alias] = { type: "funcvalue" };
            }
            const Bt = wt.alias;
            if (rt.natural) {
              if (rt.using || rt.on) throw new Error("NATURAL JOIN cannot have USING or ON clauses");
              if (Ie.sources.length > 0) {
                const At = Ie.sources[Ie.sources.length - 1], Dt = H.databases[At.databaseid].tables[At.tableid], Ot = H.databases[wt.databaseid].tables[wt.tableid];
                if (Dt && Ot) {
                  const un = Dt.columns.map((_n) => _n.columnid), yn = Ot.columns.map((_n) => _n.columnid);
                  rt.using = pe(un, yn).map((_n) => ({ columnid: _n }));
                } else throw new Error("In this version of Alasql NATURAL JOIN works for tables with predefined columns only");
              }
            }
            if (rt.using) {
              const At = Ie.sources[Ie.sources.length - 1];
              wt.onleftfns = rt.using.map((Dt) => "p['" + (At.alias || At.tableid) + "']['" + Dt.columnid + "']").join('+"`"+'), wt.onleftfn = new Function("p,params,alasql", "let y;return " + wt.onleftfns), wt.onrightfns = rt.using.map((Dt) => "p['" + (wt.alias || wt.tableid) + "']['" + Dt.columnid + "']").join('+"`"+'), wt.onrightfn = new Function("p,params,alasql", "let y;return " + wt.onrightfns), wt.optimization = "ix";
            } else if (rt.on) if (rt.on instanceof ze.Op && rt.on.op === "=" && !rt.on.allsome) {
              wt.optimization = "ix";
              let At = "", Dt = "", Ot = "", un = !1;
              const yn = rt.on.left.toJS("p", Ie.defaultTableid, Ie.defcols), _n = rt.on.right.toJS("p", Ie.defaultTableid, Ie.defcols);
              yn.indexOf("p['" + Bt + "']") > -1 && !(_n.indexOf("p['" + Bt + "']") > -1) ? (yn.match(/p\['.*?'\]/g) || []).every((gn) => gn === "p['" + Bt + "']") ? Dt = yn : un = !0 : !(yn.indexOf("p['" + Bt + "']") > -1) && _n.indexOf("p['" + Bt + "']") > -1 && (_n.match(/p\['.*?'\]/g) || []).every((gn) => gn === "p['" + Bt + "']") ? At = yn : un = !0, _n.indexOf("p['" + Bt + "']") > -1 && !(yn.indexOf("p['" + Bt + "']") > -1) ? (_n.match(/p\['.*?'\]/g) || []).every((gn) => gn === "p['" + Bt + "']") ? Dt = _n : un = !0 : !(_n.indexOf("p['" + Bt + "']") > -1) && yn.indexOf("p['" + Bt + "']") > -1 && (yn.match(/p\['.*?'\]/g) || []).every((gn) => gn === "p['" + Bt + "']") ? At = _n : un = !0, un && (Dt = "", At = "", Ot = rt.on.toJS("p", Ie.defaultTableid, Ie.defcols), wt.optimization = "no"), wt.onleftfns = At, wt.onrightfns = Dt, wt.onmiddlefns = Ot || "true", wt.onleftfn = new Function("p,params,alasql", "let y;return " + wt.onleftfns), wt.onrightfn = new Function("p,params,alasql", "let y;return " + wt.onrightfns), wt.onmiddlefn = new Function("p,params,alasql", "let y;return " + wt.onmiddlefns);
            } else wt.optimization = "no", wt.onmiddlefns = rt.on.toJS("p", Ie.defaultTableid, Ie.defcols), wt.onmiddlefn = new Function("p,params,alasql", "let y;return " + rt.on.toJS("p", Ie.defaultTableid, Ie.defcols));
            Ie.sources.push(wt);
          });
        }, ze.Select.prototype.compileWhere = function(Ie) {
          if (this.where) {
            if (typeof this.where == "function") return this.where;
            var rt = this.where.toJS("p", Ie.defaultTableid, Ie.defcols);
            return Ie.wherefns = rt, new Function("p,params,alasql", "var y;return " + rt);
          } else return function() {
            return !0;
          };
        }, ze.Select.prototype.compileWhereJoins = function(Ie) {
        }, ze.Select.prototype.compileGroup = function(Ie) {
          if (Ie.sources.length > 0) var rt = Ie.sources[0].alias;
          else var rt = "";
          var ft = Ie.defcols, pt = [[]];
          this.group && (pt = vt(this.group, Ie));
          var wt = [];
          pt.forEach(function(At) {
            wt = he(wt, At);
          }), Ie.allgroups = wt, Ie.ingroup = [];
          var Bt = "";
          return pt.forEach(function(At) {
            Bt += "var g=this.xgroups[";
            var Dt = At.map(function(_n) {
              var gn = _n.split("	")[0], bn = _n.split("	")[1];
              return gn === "" ? "1" : (Ie.ingroup.push(gn), bn);
            });
            Dt.length === 0 && (Dt = ["''"]), Bt += Dt.join('+"`"+'), Bt += "];if(!g) {this.groups.push((g=this.xgroups[", Bt += Dt.join('+"`"+'), Bt += "] = {", Bt += At.map(function(_n) {
              var gn = _n.split("	")[0], bn = _n.split("	")[1];
              return gn === "" ? "" : "'" + gn + "':" + bn + ",";
            }).join("");
            var Ot = me(wt, At);
            Bt += Ot.map(function(_n) {
              var gn = _n.split("	")[0];
              return "'" + gn + "':null,";
            }).join("");
            var un = "", yn = "";
            typeof Ie.groupStar < "u" && (yn += "for(var f in p['" + Ie.groupStar + "']) {g[f]=p['" + Ie.groupStar + "'][f];};"), Bt += Ie.selectGroup.map(function(_n) {
              var gn = _n.expression.toJS("p", rt, ft), bn = _n.nick;
              let an = (Zt) => Zt.args[0].toJS("p", rt, ft);
              if (_n instanceof ze.AggrValue) {
                if (_n.distinct && (un += ",g['$$_VALUES_" + bn + "']={},g['$$_VALUES_" + bn + "'][" + gn + "]=true"), _n.aggregatorid === "SUM") {
                  if ("funcid" in _n.expression) {
                    let Zt = an(_n.expression);
                    return `'${bn}':(${Zt})|| typeof ${Zt} == 'number' ? ${gn} : null,`;
                  }
                  return `'${bn}':(${gn})|| typeof ${gn} == 'number' ? ${gn} : null,`;
                } else if (_n.aggregatorid === "TOTAL") {
                  if ("funcid" in _n.expression) {
                    let Zt = an(_n.expression);
                    return `'${bn}':(${Zt}) || typeof ${Zt} == 'number' ?
							${Zt} : ${Zt} == 'string' && typeof Number(${Zt}) == 'number' ? Number(${Zt}) :
							typeof ${Zt} == 'boolean' ?  Number(${Zt}) : 0,`;
                  }
                  return `'${bn}':(${gn})|| typeof ${gn} == 'number' ?
							${gn} : ${gn} == 'string' && typeof Number(${gn}) == 'number' ? Number(${gn}) :
							typeof ${gn} === 'boolean' ?  Number(${gn}) : 0,`;
                } else {
                  if (_n.aggregatorid === "FIRST" || _n.aggregatorid === "LAST") return "'" + bn + "':" + gn + ",";
                  if (_n.aggregatorid === "MIN") {
                    if ("funcid" in _n.expression) {
                      let Zt = an(_n.expression);
                      return `'${bn}': (typeof ${Zt} == 'number' || typeof ${Zt} == 'bigint' ? ${gn} : typeof ${Zt} == 'object' ?
							typeof Number(${Zt}) == 'number' && ${Zt}!== null? ${gn} : null : null),`;
                    }
                    return `'${bn}': (typeof ${gn} == 'number' || typeof ${gn} == 'bigint' ? ${gn} : typeof ${gn} == 'object' ?
							typeof Number(${gn}) == 'number' && ${gn}!== null? ${gn} : null : null),`;
                  } else if (_n.aggregatorid === "MAX") {
                    if ("funcid" in _n.expression) {
                      let Zt = an(_n.expression);
                      return `'${bn}' : (typeof ${Zt} == 'number' || typeof ${Zt} == 'bigint' ? ${gn} : typeof ${Zt} == 'object' ?
							typeof Number(${Zt}) == 'number' ? ${gn} : null : null),`;
                    }
                    return `'${bn}' : (typeof ${gn} == 'number' || typeof ${gn} == 'bigint' ? ${gn} : typeof ${gn} == 'object' ?
							typeof Number(${gn}) == 'number' ? ${gn} : null : null),`;
                  } else {
                    if (_n.aggregatorid === "ARRAY") return `'${bn}':[${gn}],`;
                    if (_n.aggregatorid === "COUNT") return _n.expression.columnid === "*" ? `'${bn}':1,` : `'${bn}':(typeof ${gn} == "undefined" || ${gn} === null) ? 0 : 1,`;
                    if (_n.aggregatorid === "AVG") return Ie.removeKeys.push(`_SUM_${bn}`), Ie.removeKeys.push(`_COUNT_${bn}`), `'${bn}':${gn},'_SUM_${bn}':(${gn})||0,'_COUNT_${bn}':(typeof ${gn} == "undefined" || ${gn} === null) ? 0 : 1,`;
                    if (_n.aggregatorid === "AGGR") return un += `,g['${bn}']=${_n.expression.toJS("g", -1)}`, "";
                    if (_n.aggregatorid === "REDUCE") return Ie.aggrKeys.push(_n), `'${bn}':alasql.aggr['${_n.funcid}'](${gn},undefined,1),`;
                  }
                }
                return "";
              }
              return "";
            }).join(""), Bt += "}" + un + ",g));" + yn + "} else {", Bt += Ie.selectGroup.map(function(_n) {
              var gn = _n.nick, bn = _n.expression.toJS("p", rt, ft);
              let an = (Ln) => Ln.args[0].toJS("p", rt, ft);
              if (_n instanceof ze.AggrValue) {
                var Zt = "", Mn = "";
                if (_n.distinct && (Zt = `if(typeof ${bn}!="undefined" && (!g['$$_VALUES_${gn}'][${bn}])) {`, Mn = `g['$$_VALUES_${gn}'][${bn}]=true;}`), _n.aggregatorid === "SUM") {
                  if ("funcid" in _n.expression) {
                    let Ln = an(_n.expression);
                    return Zt + `
								{
									const __g_colas = g['${gn}'];
									const __typeof_colexp1 = typeof ${Ln};
									const __colexp1 = ${Ln};

									if (__g_colas == null && ${Ln} == null) {
										g['${gn}'] = null;
									} else if (typeof __g_colas === 'bigint' || typeof __colexp1 === 'bigint') {
            					    	g['${gn}'] = BigInt(__g_colas) + BigInt(__colexp);
            						} else if ((typeof __g_colas !== 'object' && typeof __g_colas !== 'number' && __typeof_colexp1 !== 'object' && __typeof_colexp1 !== 'number') ||
											   (__g_colas == null || (typeof __g_colas !== 'number' && typeof __g_colas !== 'object')) && (${Ln} == null || (__typeof_colexp1 !== 'number' && __typeof_colexp1 !== 'object'))) {
										g['${gn}'] = null;
									} else if ((typeof __g_colas !== 'object' && typeof __g_colas !== 'number' && __typeof_colexp1 == 'number') ||
											   (__g_colas == null && __typeof_colexp1 == 'number')) {
										g['${gn}'] = ${bn};
									} else if (typeof __g_colas == 'number' && ${Ln} == null) {
										g['${gn}'] = __g_colas;
									} else {
										g['${gn}'] += ${bn} || 0;
									}
								}
								` + Mn;
                  }
                  return Zt + `
							{
								const __g_colas = g['${gn}'];
								const __typeof_colexp = typeof ${bn};
								const __colexp = ${bn};

								if (__g_colas == null && ${bn} == null) {
									g['${gn}'] = null;
								} else if (typeof __g_colas === 'bigint' || typeof __colexp === 'bigint') {
            					    g['${gn}'] = BigInt(__g_colas) + BigInt(__colexp);
            					} else if ((typeof __g_colas !== 'object' && typeof __g_colas !== 'number' && __typeof_colexp !== 'object' && __typeof_colexp !== 'number') ||
										   (__g_colas == null || (typeof __g_colas !== 'number' && typeof __g_colas !== 'object')) && (${bn} == null || (__typeof_colexp !== 'number' && __typeof_colexp !== 'object'))) {
									g['${gn}'] = null;
								} else if (typeof __g_colas !== 'object' && typeof __g_colas !== 'number' && __typeof_colexp == 'number') {
									g['${gn}'] = ${bn};
								} else if (typeof __g_colas == 'number' && ${bn} == null) {
									g['${gn}'] = __g_colas;
								} else if (__g_colas == null && __typeof_colexp == 'number') {
									g['${gn}'] = ${bn};
								} else {
									g['${gn}'] += ${bn} || 0;
								}
							}
							` + Mn;
                } else if (_n.aggregatorid === "TOTAL") {
                  if ("funcid" in _n.expression) {
                    let Ln = an(_n.expression);
                    return Zt + `{
									const __g_colas = g['${gn}'];
									const __colexp1 = ${Ln};
									const __typeof_g_colas = typeof __g_colas;
									const __typeof_colexp1 = typeof __colexp1;

									if (__typeof_g_colas == 'string' && !isNaN(__g_colas) && typeof Number(__g_colas) == 'number' &&
										__typeof_colexp1 == 'string' && !isNaN(__colexp1) && typeof Number(__colexp1) == 'number') {
										g['${gn}'] = Number(__g_colas) + Number(__colexp1);
									} else if (__typeof_g_colas === 'bigint' || __typeof_colexp1 === 'bigint') {
       							    	g['${gn}'] = BigInt(__g_colas || 0) + BigInt(__colexp1 || 0);
       								} else if (__typeof_g_colas == 'string' && __typeof_colexp1 == 'string') {
										g['${gn}'] = 0;
									} else if (__typeof_g_colas == 'string' && __typeof_colexp1 == 'number') {
										g['${gn}'] = __colexp1;
									} else if (__typeof_colexp1 == 'string' && __typeof_g_colas == 'number') {
										g['${gn}'] = __g_colas;
									} else {
										g['${gn}'] += __colexp1 || 0;
									}
								}` + Mn;
                  }
                  return Zt + `{
								const __g_colas = g['${gn}'];
								const __colexp = ${bn};
								const __typeof_g_colas = typeof __g_colas;
								const __typeof_colexp = typeof __colexp;

								if (__typeof_g_colas === 'string' && !isNaN(__g_colas) && typeof Number(__g_colas) === 'number' &&
									__typeof_colexp === 'string' && !isNaN(__colexp) && typeof Number(__colexp) === 'number') {
									g['${gn}'] = Number(__g_colas) + Number(__colexp);
								} else if (__typeof_g_colas === 'bigint' || __typeof_colexp === 'bigint') {
       							    g['${gn}'] = BigInt(__g_colas || 0) + BigInt(__colexp || 0);
       							} else if (__typeof_g_colas === 'string' && __typeof_colexp === 'string') {
									g['${gn}'] = 0;
								} else if (__typeof_g_colas === 'string' && __typeof_colexp === 'number') {
									g['${gn}'] = __colexp;
								} else if (__typeof_colexp === 'string' && __typeof_g_colas === 'number') {
									g['${gn}'] = __g_colas;
								} else {
									g['${gn}'] += __colexp || 0;
								}
							}

							` + Mn;
                } else {
                  if (_n.aggregatorid === "COUNT") return _n.expression.columnid === "*" ? `${Zt}
								g['${gn}']++;
								${Mn}` : `${Zt}
							if(typeof ${bn}!="undefined" && ${bn} !== null) g['${gn}']++;
							${Mn}`;
                  if (_n.aggregatorid === "ARRAY") return Zt + "g['" + gn + "'].push(" + bn + ");" + Mn;
                  if (_n.aggregatorid === "MIN") {
                    if ("funcid" in _n.expression) {
                      let Ln = an(_n.expression);
                      return Zt + `if ((g['${gn}'] == null && ${Ln} !== null) ? y = ${bn} : 
									(g['${gn}'] !== null && ${Ln} == null) ? y = g['${gn}'] : 
									((y = ${bn}) < g['${gn}'])) {
									if (typeof y == 'number' || typeof y == 'bigint') {
									  g['${gn}'] = y;
									} else if (typeof y == 'object' && y instanceof Date) {
									  g['${gn}'] = y;
									} else if (typeof y == 'object' && typeof Number(y) == 'number') {
									  g['${gn}'] = Number(y);
									}
								} else if (g['${gn}'] !== null && typeof g['${gn}'] == 'object' && y instanceof Date) {
									g['${gn}'] = g['${gn}'];
								} else if (g['${gn}'] !== null && typeof g['${gn}'] == 'object') {
									g['${gn}'] = Number(g['${gn}']);
								}` + Mn;
                    }
                    return Zt + `if((g['${gn}'] == null && ${bn}!== null) ? y = ${bn} : 
								(g['${gn}']!== null && ${bn} == null) ? y = g['${gn}'] : 
								((y=${bn}) < g['${gn}'])) { 
								if(typeof y == 'number' || typeof y == 'bigint') {
									g['${gn}'] = y;
								} else if(typeof y == 'object' && y instanceof Date) {
									g['${gn}'] = y;
								} else if(typeof y == 'object' && typeof Number(y) == 'number') {
									g['${gn}'] = Number(y);
								}
							} else if(g['${gn}']!== null && typeof g['${gn}'] == 'object' && y instanceof Date) {
								g['${gn}'] = g['${gn}'];
							} else if(g['${gn}']!== null && typeof g['${gn}'] == 'object') {
								g['${gn}'] = Number(g['${gn}']);
							}` + Mn;
                  } else if (_n.aggregatorid === "MAX") {
                    if ("funcid" in _n.expression) {
                      let Ln = an(_n.expression);
                      return Zt + `if ((g['${gn}'] == null && ${Ln} !== null) ? y = ${bn} : 
									(g['${gn}'] !== null && ${Ln} == null) ? y = g['${gn}'] : 
									((y = ${bn}) > g['${gn}'])) {
									if (typeof y == 'number' || typeof y == 'bigint') {
									  g['${gn}'] = y;
									} else if (typeof y == 'object' && y instanceof Date) {
									  g['${gn}'] = y;
									} else if (typeof y == 'object' && typeof Number(y) == 'number') {
									  g['${gn}'] = Number(y);
									}
								} else if (g['${gn}'] !== null && typeof g['${gn}'] == 'object' && y instanceof Date) {
									g['${gn}'] = g['${gn}'];
								} else if (g['${gn}'] !== null && typeof g['${gn}'] == 'object') {
									g['${gn}'] = Number(g['${gn}']);
								}` + Mn;
                    }
                    return Zt + `if((g['${gn}'] == null && ${bn}!== null) ? y = ${bn} : 
								(g['${gn}']!== null && ${bn} == null) ? y = g['${gn}'] : 
								((y=${bn}) > g['${gn}'])) { 
								if(typeof y == 'number' || typeof y == 'bigint') {
									g['${gn}'] = y;
								} else if(typeof y == 'object' && y instanceof Date) {
									g['${gn}'] = y;
								} else if(typeof y == 'object' && typeof Number(y) == 'number') {
									g['${gn}'] = Number(y);
								}
							} else if(g['${gn}']!== null && typeof g['${gn}'] == 'object' && y instanceof Date) {
								g['${gn}'] = g['${gn}'];
							} else if(g['${gn}']!== null && typeof g['${gn}'] == 'object') {
								g['${gn}'] = Number(g['${gn}']);
							}` + Mn;
                  } else {
                    if (_n.aggregatorid === "FIRST") return "";
                    if (_n.aggregatorid === "LAST") return `${Zt}g['${gn}']=${bn};${Mn}`;
                    if (_n.aggregatorid === "AVG") return `${Zt}
							y= (${bn});
							g['_COUNT_${gn}'] += (typeof y == "undefined" || y === null) ? 0 : 1;
							if (typeof g['_SUM_${gn}'] === 'bigint' || typeof y === 'bigint') {
								g['_SUM_${gn}'] = BigInt(g['_SUM_${gn}']);
								g['_SUM_${gn}'] += BigInt(y || 0);
    							g['${gn}'] = BigInt(g['_SUM_${gn}']) / BigInt(g['_COUNT_${gn}']); 
    						} else {
								g['_SUM_${gn}'] += (y || 0);
    							g['${gn}'] = g['_SUM_${gn}'] / g['_COUNT_${gn}']; 
    						}
							${Mn}`;
                    if (_n.aggregatorid === "AGGR") return `${Zt}
							g['${gn}']=${_n.expression.toJS("g", -1)};
							${Mn}`;
                    if (_n.aggregatorid === "REDUCE") return `${Zt}
							g['${gn}'] = alasql.aggr.${_n.funcid}(${bn},g['${gn}'],2);
							${Mn}`;
                  }
                }
                return "";
              }
              return "";
            }).join(""), Bt += "}";
          }), new Function("p,params,alasql", "var y;" + Bt);
        };
        function bt(Ie, rt, ft) {
          var pt = "", wt = [], Bt = {};
          return rt.forEach(function(At) {
            Ie.ixsources = {}, Ie.sources.forEach(function(Ot) {
              Ie.ixsources[Ot.alias] = Ot;
            });
            var Dt;
            if (Ie.ixsources[At]) var Dt = Ie.ixsources[At].columns;
            ft && H.options.joinstar == "json" && (pt += "r['" + At + "']={};"), Dt && Dt.length > 0 ? Dt.forEach(function(Ot) {
              const un = ee(Ot.columnid);
              if (ft && H.options.joinstar == "underscore") wt.push("'" + At + "_" + un + "':p['" + At + "']['" + un + "']");
              else if (ft && H.options.joinstar == "json") pt += "r['" + At + "']['" + un + "']=p['" + At + "']['" + un + "'];";
              else {
                var yn = "p['" + At + "']['" + un + "']";
                if (Bt[Ot.columnid]) {
                  var _n = yn + " !== undefined ? " + yn + " : " + Bt[Ot.columnid].value;
                  wt[Bt[Ot.columnid].id] = Bt[Ot.columnid].key + _n, Bt[Ot.columnid].value = _n;
                } else {
                  var gn = "'" + un + "':";
                  wt.push(gn + yn), Bt[Ot.columnid] = { id: wt.length - 1, value: yn, key: gn };
                }
              }
              Ie.selectColumns[un] = !0;
              var bn = { columnid: Ot.columnid, dbtypeid: Ot.dbtypeid, dbsize: Ot.dbsize, dbprecision: Ot.dbprecision, dbenum: Ot.dbenum };
              Ie.columns.push(bn), Ie.xcolumns[bn.columnid] = bn;
            }) : (pt += 'var w=p["' + At + '"];for(var k in w){r[k]=w[k]};', Ie.dirtyColumns = !0);
          }), { s: wt.join(","), sp: pt };
        }
        ze.Select.prototype.compileSelect1 = function(Ie, rt) {
          var ft = this;
          Ie.columns = [], Ie.xcolumns = {}, Ie.selectColumns = {}, Ie.dirtyColumns = !1;
          var pt = "var r={", wt = "", Bt = [];
          return this.columns.forEach(function(At) {
            if (At instanceof ze.Column) if (At.columnid === "*") if (At.func) wt += "r=params['" + At.param + "'](p['" + Ie.sources[0].alias + "'],p,params,alasql);";
            else if (At.tableid) {
              var Dt = bt(Ie, [At.tableid], !1);
              Dt.s && (Bt = Bt.concat(Dt.s)), wt += Dt.sp;
            } else {
              var Dt = bt(Ie, Object.keys(Ie.aliases), !0);
              Dt.s && (Bt = Bt.concat(Dt.s)), wt += Dt.sp;
            }
            else {
              var Ot = At.tableid, un = At.databaseid || Ie.sources[0].databaseid || Ie.database.databaseid;
              if (Ot || (Ot = Ie.defcols[At.columnid]), Ot || (Ot = Ie.defaultTableid), At.columnid !== "_") {
                var yn = rt && rt.length > 1 && Array.isArray(rt[0]) && rt[0].length >= 1 && rt[0][0].hasOwnProperty("sheetid");
                yn ? wt = 'var r={};var w=p["' + Ot + '"];var cols=[' + ft.columns.map(function(Zt) {
                  return "'" + Zt.columnid + "'";
                }).join(",") + "];var colas=[" + ft.columns.map(function(Zt) {
                  return "'" + (Zt.as || Zt.columnid) + "'";
                }).join(",") + "];for (var i=0;i<Object.keys(p['" + Ot + "']).length;i++) for(var k=0;k<cols.length;k++){if (!r.hasOwnProperty(i)) r[i]={}; r[i][colas[k]]=w[i][cols[k]];}" : Bt.push("'" + ee(At.as || At.columnid) + "':p['" + Ot + "']['" + At.columnid + "']");
              } else Bt.push("'" + ee(At.as || At.columnid) + "':p['" + Ot + "']");
              if (Ie.selectColumns[ee(At.as || At.columnid)] = !0, Ie.aliases[Ot] && Ie.aliases[Ot].type === "table") {
                if (!H.databases[un].tables[Ie.aliases[Ot].tableid]) throw new Error("Table '" + Ot + "' does not exist in database");
                var _n = H.databases[un].tables[Ie.aliases[Ot].tableid].columns, gn = H.databases[un].tables[Ie.aliases[Ot].tableid].xcolumns;
                if (gn && _n.length > 0) {
                  var bn = gn[At.columnid];
                  if (bn === void 0) throw new Error("Column does not exist: " + At.columnid);
                  var an = { columnid: At.as || At.columnid, dbtypeid: bn.dbtypeid, dbsize: bn.dbsize, dbpecision: bn.dbprecision, dbenum: bn.dbenum };
                  Ie.columns.push(an), Ie.xcolumns[an.columnid] = an;
                } else {
                  var an = { columnid: At.as || At.columnid };
                  Ie.columns.push(an), Ie.xcolumns[an.columnid] = an, Ie.dirtyColumns = !0;
                }
              } else {
                var an = { columnid: At.as || At.columnid };
                Ie.columns.push(an), Ie.xcolumns[an.columnid] = an;
              }
            }
            else if (At instanceof ze.AggrValue) {
              ft.group || (ft.group = [""]), At.as || (At.as = ee(At.toString())), At.aggregatorid === "SUM" || At.aggregatorid === "MAX" || At.aggregatorid === "MIN" || At.aggregatorid === "FIRST" || At.aggregatorid === "LAST" || At.aggregatorid === "AVG" || At.aggregatorid === "ARRAY" || At.aggregatorid === "REDUCE" || At.aggregatorid === "TOTAL" ? Bt.push("'" + ee(At.as) + "':" + Z(At.expression.toJS("p", Ie.defaultTableid, Ie.defcols))) : At.aggregatorid === "COUNT" && Bt.push("'" + ee(At.as) + "':1");
              var an = { columnid: At.as || At.columnid || At.toString() };
              Ie.columns.push(an), Ie.xcolumns[an.columnid] = an;
            } else {
              Bt.push("'" + ee(At.as || At.columnid || At.toString()) + "':" + Z(At.toJS("p", Ie.defaultTableid, Ie.defcols))), Ie.selectColumns[ee(At.as || At.columnid || At.toString())] = !0;
              var an = { columnid: At.as || At.columnid || At.toString() };
              Ie.columns.push(an), Ie.xcolumns[an.columnid] = an;
            }
          }), pt += Bt.join(",") + "};" + wt, pt;
        }, ze.Select.prototype.compileSelect2 = function(Ie, rt) {
          var ft = Ie.selectfns;
          return this.orderColumns && this.orderColumns.length > 0 && this.orderColumns.forEach(function(pt, wt) {
            var Bt = "$$$" + wt;
            pt instanceof ze.Column && Ie.xcolumns[pt.columnid] ? ft += "r['" + Bt + "']=r['" + pt.columnid + "'];" : pt instanceof ze.ParamValue && Ie.xcolumns[rt[pt.param]] ? ft += "r['" + Bt + "']=r['" + rt[pt.param] + "'];" : ft += "r['" + Bt + "']=" + pt.toJS("p", Ie.defaultTableid, Ie.defcols) + ";", Ie.removeKeys.push(Bt);
          }), new Function("p,params,alasql", "var y;" + ft + "return r");
        }, ze.Select.prototype.compileSelectGroup0 = function(Ie) {
          var rt = this;
          rt.columns.forEach(function(ft, pt) {
            if (ft instanceof ze.Column && ft.columnid === "*") Ie.groupStar = ft.tableid || "default";
            else {
              var wt;
              ft instanceof ze.Column ? wt = ee(ft.columnid) : wt = ee(ft.toString(!0));
              for (var Bt = 0; Bt < pt; Bt++) if (wt === rt.columns[Bt].nick) {
                wt = rt.columns[Bt].nick + ":" + pt;
                break;
              }
              if (ft.nick = wt, rt.group) {
                var At = rt.group.findIndex(function(Dt) {
                  return Dt.columnid === ft.columnid && Dt.tableid === ft.tableid;
                });
                At > -1 && (rt.group[At].nick = wt);
              }
              ft.funcid && (ft.funcid.toUpperCase() === "ROWNUM" || ft.funcid.toUpperCase() === "ROW_NUMBER") && Ie.rownums.push(ft.as);
            }
          }), this.columns.forEach(function(ft) {
            ft.findAggregator && ft.findAggregator(Ie);
          }), this.having && this.having.findAggregator && this.having.findAggregator(Ie);
        }, ze.Select.prototype.compileSelectGroup1 = function(Ie) {
          var rt = this, ft = "var r = {};";
          return rt.columns.forEach(function(pt) {
            if (pt instanceof ze.Column && pt.columnid === "*") return ft += "for(var k in g) {r[k]=g[k]};", "";
            var wt = pt.as;
            wt === void 0 && (pt instanceof ze.Column ? wt = ee(pt.columnid) : wt = pt.nick), Ie.groupColumns[wt] = pt.nick, ft += "r['" + wt + "']=", ft += Z(pt.toJS("g", "")) + ";";
            for (var Bt = 0; Bt < Ie.removeKeys.length; Bt++) if (Ie.removeKeys[Bt] === wt) {
              Ie.removeKeys.splice(Bt, 1);
              break;
            }
          }), ft;
        }, ze.Select.prototype.compileSelectGroup2 = function(Ie) {
          var rt = this, ft = Ie.selectgfns;
          return rt.columns.forEach(function(pt) {
            Ie.ingroup.indexOf(pt.nick) > -1 && (ft += "r['" + (pt.as || pt.nick) + "']=g['" + pt.nick + "'];");
          }), this.orderColumns && this.orderColumns.length > 0 && this.orderColumns.forEach(function(pt, wt) {
            var Bt = "$$$" + wt;
            pt instanceof ze.Column && Ie.groupColumns[pt.columnid] ? ft += "r['" + Bt + "']=r['" + pt.columnid + "'];" : ft += "r['" + Bt + "']=" + pt.toJS("g", "") + ";", Ie.removeKeys.push(Bt);
          }), new Function("g,params,alasql", "var y;" + ft + "return r");
        }, ze.Select.prototype.compileRemoveColumns = function(Ie) {
          typeof this.removecolumns < "u" && (Ie.removeKeys = Ie.removeKeys.concat(this.removecolumns.filter(function(rt) {
            return typeof rt.like > "u";
          }).map(function(rt) {
            return rt.columnid;
          })), Ie.removeLikeKeys = this.removecolumns.filter(function(rt) {
            return typeof rt.like < "u";
          }).map(function(rt) {
            return rt.like.value;
          }));
        }, ze.Select.prototype.compileHaving = function(Ie) {
          if (this.having) {
            var rt = this.having.toJS("g", -1);
            return Ie.havingfns = rt, new Function("g,params,alasql", "var y;return " + rt);
          }
          return function() {
            return !0;
          };
        }, ze.Select.prototype.compileOrder = function(Ie, rt) {
          var ft = this;
          if (ft.orderColumns = [], this.order) {
            if (this.order && this.order.length == 1 && this.order[0].expression && typeof this.order[0].expression == "function") {
              var pt = this.order[0].expression, wt = this.order[0].nullsOrder == "FIRST" ? -1 : this.order[0].nullsOrder == "LAST" ? 1 : 0;
              return function(Dt, Ot) {
                var un = pt(Dt), yn = pt(Ot);
                if (wt) {
                  if (un == null) return yn == null ? 0 : wt;
                  if (yn == null) return -wt;
                }
                return un > yn ? 1 : un == yn ? 0 : -1;
              };
            }
            var Bt = "", At = "";
            return this.order.forEach(function(Dt, Ot) {
              if (Dt.expression instanceof ze.NumValue) {
                if (Dt.expression.value > ft.columns.length) throw new Error(`You are trying to order by column number ${Dt.expression.value} but you have only selected ${ft.columns.length} columns.`);
                var un = ft.columns[Dt.expression.value - 1];
              } else var un = Dt.expression;
              ft.orderColumns.push(un);
              var yn = "$$$" + Ot, _n = "";
              if (Dt.expression instanceof ze.Column) {
                var gn = Dt.expression.columnid;
                if (H.options.valueof) _n = ".valueOf()";
                else if (Ie.xcolumns[gn]) {
                  var bn = Ie.xcolumns[gn].dbtypeid;
                  (bn == "DATE" || bn == "DATETIME" || bn == "DATETIME2" || bn == "STRING" || bn == "NUMBER") && (_n = ".valueOf()");
                }
              }
              if (Dt.expression instanceof ze.ParamValue) {
                var gn = rt[Dt.expression.param];
                if (H.options.valueof) _n = ".valueOf()";
                else if (Ie.xcolumns[gn]) {
                  var bn = Ie.xcolumns[gn].dbtypeid;
                  (bn == "DATE" || bn == "DATETIME" || bn == "DATETIME2" || bn == "STRING" || bn == "NUMBER") && (_n = ".valueOf()");
                }
              }
              Dt.nocase && (_n += ".toUpperCase()"), Dt.nullsOrder && (Dt.nullsOrder == "FIRST" ? Bt += "if((a['" + yn + "'] != null) && (b['" + yn + "'] == null)) return 1;" : Dt.nullsOrder == "LAST" && (Bt += "if((a['" + yn + "'] == null) && (b['" + yn + "'] != null)) return 1;"), Bt += "if((a['" + yn + "'] == null) == (b['" + yn + "'] == null)) {", At += "}"), Bt += "if((a['" + yn + "']||'')" + _n + (Dt.direction == "ASC" ? ">" : "<") + "(b['" + yn + "']||'')" + _n + ")return 1;", Bt += "if((a['" + yn + "']||'')" + _n + "==(b['" + yn + "']||'')" + _n + "){", At += "}";
            }), Bt += "return 0;", Bt += At + "return -1", Ie.orderfns = Bt, new Function("a,b", "var y;" + Bt);
          }
        }, ze.Select.prototype.compilePivot = function(Ie) {
          var rt = this, ft = rt.pivot.columnid, pt = rt.pivot.expr.aggregatorid, wt = rt.pivot.inlist, Bt = null;
          if (rt.pivot.expr.expression.hasOwnProperty("columnid") ? Bt = rt.pivot.expr.expression.columnid : Bt = rt.pivot.expr.expression.expression.columnid, Bt == null) throw "columnid not found";
          return wt && (wt = wt.map(function(At) {
            return At.expr.columnid;
          })), function() {
            var At = this, Dt = At.columns.filter(function(Ln) {
              return Ln.columnid != ft && Ln.columnid != Bt;
            }).map(function(Ln) {
              return Ln.columnid;
            }), Ot = [], un = {}, yn = {}, _n = {}, gn = [];
            if (At.data.forEach(function(Ln) {
              if (!wt || wt.indexOf(Ln[ft]) > -1) {
                var In = Dt.map(function(er) {
                  return Ln[er];
                }).join("`"), kn = yn[In];
                if (kn || (kn = {}, yn[In] = kn, gn.push(kn), Dt.forEach(function(er) {
                  kn[er] = Ln[er];
                })), _n[In] || (_n[In] = {}), _n[In][Ln[ft]] ? _n[In][Ln[ft]]++ : _n[In][Ln[ft]] = 1, un[Ln[ft]] || (un[Ln[ft]] = !0, Ot.push(Ln[ft])), pt == "SUM" || pt == "AVG" || pt == "TOTAL") typeof kn[Ln[ft]] > "u" && (kn[Ln[ft]] = 0), kn[Ln[ft]] += +Ln[Bt];
                else if (pt == "COUNT") typeof kn[Ln[ft]] > "u" && (kn[Ln[ft]] = 0), kn[Ln[ft]]++;
                else if (pt == "MIN") typeof kn[Ln[ft]] > "u" && (kn[Ln[ft]] = Ln[Bt]), Ln[Bt] < kn[Ln[ft]] && (kn[Ln[ft]] = Ln[Bt]);
                else if (pt == "MAX") typeof kn[Ln[ft]] > "u" && (kn[Ln[ft]] = Ln[Bt]), Ln[Bt] > kn[Ln[ft]] && (kn[Ln[ft]] = Ln[Bt]);
                else if (pt == "FIRST") typeof kn[Ln[ft]] > "u" && (kn[Ln[ft]] = Ln[Bt]);
                else if (pt == "LAST") kn[Ln[ft]] = Ln[Bt];
                else if (H.aggr[pt]) H.aggr[pt](kn[Ln[ft]], Ln[Bt]);
                else throw new Error("Wrong aggregator in PIVOT clause");
              }
            }), pt == "AVG") for (var bn in yn) {
              var an = yn[bn];
              for (var Zt in an) Dt.indexOf(Zt) == -1 && Zt != Bt && (an[Zt] = an[Zt] / _n[bn][Zt]);
            }
            At.data = gn, wt && (Ot = wt);
            var Mn = At.columns.filter(function(Ln) {
              return Ln.columnid == Bt;
            })[0];
            At.columns = At.columns.filter(function(Ln) {
              return !(Ln.columnid == ft || Ln.columnid == Bt);
            }), Ot.forEach(function(Ln) {
              var In = be(Mn);
              In.columnid = Ln, At.columns.push(In);
            });
          };
        }, ze.Select.prototype.compileUnpivot = function(Ie) {
          var rt = this, ft = rt.unpivot.tocolumnid, pt = rt.unpivot.forcolumnid, wt = rt.unpivot.inlist.map(function(Bt) {
            return Bt.columnid;
          });
          return function() {
            var Bt = [], At = Ie.columns.map(function(Dt) {
              return Dt.columnid;
            }).filter(function(Dt) {
              return wt.indexOf(Dt) == -1 && Dt != pt && Dt != ft;
            });
            Ie.data.forEach(function(Dt) {
              wt.forEach(function(Ot) {
                var un = {};
                At.forEach(function(yn) {
                  un[yn] = Dt[yn];
                }), un[pt] = Ot, un[ft] = Dt[Ot], Bt.push(un);
              });
            }), Ie.data = Bt;
          };
        };
        const yt = (Ie, rt) => {
          const ft = [];
          let pt = 0;
          const wt = Ie.length;
          for (let Bt = 0; Bt < wt + 1; Bt++) {
            const At = [];
            for (let Dt = 0; Dt < wt; Dt++) {
              let Ot;
              Ie[Dt] instanceof ze.Column ? (Ie[Dt].nick = ee(Ie[Dt].columnid), rt.groupColumns[ee(Ie[Dt].columnid)] = Ie[Dt].nick, Ot = `${Ie[Dt].nick}	${Ie[Dt].toJS("p", rt.sources[0].alias, rt.defcols)}`) : (rt.groupColumns[ee(Ie[Dt].toString())] = ee(Ie[Dt].toString()), Ot = `${ee(Ie[Dt].toString())}	${Ie[Dt].toJS("p", rt.sources[0].alias, rt.defcols)}`), pt & 1 << Dt && At.push(Ot);
            }
            ft.push(At), pt = (pt << 1) + 1;
          }
          return ft;
        }, St = (Ie, rt) => {
          const ft = [], pt = Ie.length, wt = 1 << pt;
          for (let Bt = 0; Bt < wt; Bt++) {
            let At = [];
            for (let Dt = 0; Dt < pt; Dt++) Bt & 1 << Dt && (At = At.concat(vt(Ie[Dt], rt)));
            ft.push(At);
          }
          return ft;
        }, Lt = (Ie, rt) => Ie.reduce((ft, pt) => (ft = ft.concat(vt(pt, rt)), ft), []), Rt = (Ie, rt) => {
          const ft = [];
          for (let pt = 0; pt < Ie.length; pt++) for (let wt = 0; wt < rt.length; wt++) ft.push(Ie[pt].concat(rt[wt]));
          return ft;
        };
        function vt(Ie, rt) {
          if (Array.isArray(Ie)) {
            let ft = [[]];
            for (let pt = 0; pt < Ie.length; pt++) if (Ie[pt] instanceof ze.Column) Ie[pt].nick = Ie[pt].nick ? ee(Ie[pt].nick) : ee(Ie[pt].columnid), rt.groupColumns[Ie[pt].nick] = Ie[pt].nick, ft = ft.map((wt) => wt.concat(`${Ie[pt].nick}	${Ie[pt].toJS("p", rt.sources[0].alias, rt.defcols)}`));
            else if (Ie[pt] instanceof ze.FuncValue) rt.groupColumns[ee(Ie[pt].toString())] = ee(Ie[pt].toString()), ft = ft.map((wt) => wt.concat(`${ee(Ie[pt].toString())}	${Ie[pt].toJS("p", rt.sources[0].alias, rt.defcols)}`));
            else if (Ie[pt] instanceof ze.GroupExpression) if (Ie[pt].type == "ROLLUP") ft = Rt(ft, yt(Ie[pt].group, rt));
            else if (Ie[pt].type == "CUBE") ft = Rt(ft, St(Ie[pt].group, rt));
            else if (Ie[pt].type == "GROUPING SETS") ft = Rt(ft, Lt(Ie[pt].group, rt));
            else throw new Error("Unknown grouping function");
            else Ie[pt] === "" ? ft = [["1	1"]] : ft = ft.map((wt) => wt.concat(`${ee(Ie[pt].toString())}	${Ie[pt].toJS("p", rt.sources[0].alias, rt.defcols)}`));
            return ft;
          }
          return Ie instanceof ze.FuncValue ? (rt.groupColumns[ee(Ie.toString())] = ee(Ie.toString()), [`${Ie.toString()}	${Ie.toJS("p", rt.sources[0].alias, rt.defcols)}`]) : Ie instanceof ze.Column ? (Ie.nick = ee(Ie.columnid), rt.groupColumns[Ie.nick] = Ie.nick, [`${Ie.nick}	${Ie.toJS("p", rt.sources[0].alias, rt.defcols)}`]) : (rt.groupColumns[ee(Ie.toString())] = ee(Ie.toString()), [`${ee(Ie.toString())}	${Ie.toJS("p", rt.sources[0].alias, rt.defcols)}`]);
        }
        ze.Select.prototype.compileDefCols = function(Ie, rt) {
          var ft = { ".": {} };
          return this.from && this.from.forEach(function(pt) {
            if (ft["."][pt.as || pt.tableid] = !0, pt instanceof ze.Table) {
              var wt = pt.as || pt.tableid, Bt = H.databases[pt.databaseid || rt].tables[pt.tableid];
              if (Bt === void 0) throw new Error("Table does not exist: " + pt.tableid);
              Bt.columns && Bt.columns.forEach(function(At) {
                ft[At.columnid] ? ft[At.columnid] = "-" : ft[At.columnid] = wt;
              });
            } else if (!(pt instanceof ze.Select) && !(pt instanceof ze.Search) && !(pt instanceof ze.ParamValue) && !(pt instanceof ze.VarValue) && !(pt instanceof ze.FuncValue) && !(pt instanceof ze.FromData) && !(pt instanceof ze.Json) && !pt.inserted)
              throw new Error("Unknown type of FROM clause");
          }), this.joins && this.joins.forEach(function(pt) {
            if (ft["."][pt.as || pt.table.tableid] = !0, pt.table) {
              var wt = pt.as || pt.table.tableid, Bt = pt.table.databaseid || rt, At = H.databases[Bt];
              if (At === void 0) throw new Error("Database does not exist: " + Bt);
              var Dt = At.tables[pt.table.tableid];
              if (Dt === void 0) throw new Error("Table does not exist: " + pt.table.tableid);
              Dt.columns && Dt.columns.forEach(function(Ot) {
                ft[Ot.columnid] ? ft[Ot.columnid] = "-" : ft[Ot.columnid] = wt;
              });
            } else if (!pt.select && !pt.param && !pt.func)
              throw new Error("Unknown type of FROM clause");
          }), ft;
        }, ze.Union = class {
          constructor(Ie) {
            Object.assign(this, Ie);
          }
          toString() {
            return "UNION";
          }
          compile(Ie) {
            return null;
          }
        }, ze.Apply = class {
          constructor(Ie) {
            Object.assign(this, Ie);
          }
          toString() {
            let Ie = `${this.applymode} APPLY (${this.select.toString()})`;
            return this.as && (Ie += ` AS ${this.as}`), Ie;
          }
        }, ze.Over = class {
          constructor(Ie) {
            Object.assign(this, Ie);
          }
          toString() {
            let Ie = "OVER (";
            return this.partition && (Ie += `PARTITION BY ${this.partition.toString()}`, this.order && (Ie += " ")), this.order && (Ie += `ORDER BY ${this.order.toString()}`), Ie += ")", Ie;
          }
        };
        {
          const Ie = Object.assign;
          class rt {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return this.expression.toString();
            }
            execute(Ar, Yr, jr) {
              if (this.expression) {
                H.precompile(this, Ar, Yr);
                var Mr = new Function("params,alasql,p", "var y;return " + this.expression.toJS("({})", "", null)).bind(this), Wr = Mr(Yr, H);
                return jr && (Wr = jr(Wr)), Wr;
              }
            }
          }
          class ft {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              var Ar = this.expression.toString();
              return this.order && (Ar += " " + this.order.toString()), this.nocase && (Ar += " COLLATE NOCASE"), this.direction && (Ar += " " + this.direction), Ar;
            }
            findAggregator(Ar) {
              this.expression.findAggregator && this.expression.findAggregator(Ar);
            }
            toJS(Ar, Yr, jr) {
              return this.expression.reduced ? "true" : this.expression.toJS(Ar, Yr, jr);
            }
            compile(Ar, Yr, jr) {
              return this.reduced ? ne() : new Function("p", "var y;return " + this.toJS(Ar, Yr, jr));
            }
          }
          class pt {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              var Ar = "``" + this.value + "``";
              return Ar;
            }
            toJS() {
              return "(" + this.value + ")";
            }
            execute(Ar, Yr, jr) {
              var Mr = 1, Wr = new Function("params,alasql,p", this.value);
              return Wr(Yr, H), jr && (Mr = jr(Mr)), Mr;
            }
          }
          class wt {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              var Ar = this.value;
              return this.value1 && (Ar = this.value1 + "." + Ar), Ar;
            }
          }
          class Bt {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              var Ar = " ";
              return this.joinmode && (Ar += this.joinmode + " "), Ar += "JOIN " + this.table.toString(), Ar;
            }
          }
          class At {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              var Ar = this.tableid;
              return this.databaseid && (Ar = this.databaseid + "." + Ar), Ar;
            }
          }
          class Dt {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              var Ar = this.viewid;
              return this.databaseid && (Ar = this.databaseid + "." + Ar), Ar;
            }
          }
          const Ot = /* @__PURE__ */ new Set(["-", "*", "/", "%", "^"]), un = /* @__PURE__ */ new Set(["||"]), yn = /* @__PURE__ */ new Set(["AND", "OR", "NOT", "=", "==", "===", "!=", "!==", "!===", ">", ">=", "<", "<=", "IN", "NOT IN", "LIKE", "NOT LIKE", "REGEXP", "GLOB", "BETWEEN", "NOT BETWEEN", "IS NULL", "IS NOT NULL"]);
          class _n {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              const Ar = this.left.toString();
              let Yr;
              return this.op === "IN" || this.op === "NOT IN" ? `${Ar} ${this.op} (${this.right.toString()})` : this.allsome ? `${Ar} ${this.op} ${this.allsome} (${this.right.toString()})` : this.op === "->" || this.op === "!" ? (Yr = `${Ar}${this.op}`, typeof this.right != "string" && typeof this.right != "number" ? Yr + `(${this.right.toString()})` : Yr + this.right.toString()) : this.op === "BETWEEN" || this.op === "NOT BETWEEN" ? `${Ar} ${this.op} ${this.right1.toString()} AND ${this.right2.toString()}` : `${Ar} ${this.op} ${this.allsome ? this.allsome + " " : ""}${this.right.toString()}`;
            }
            findAggregator(Ar) {
              this.left && this.left.findAggregator && this.left.findAggregator(Ar), this.right && this.right.findAggregator && !this.allsome && this.right.findAggregator(Ar);
            }
            toType(Ar) {
              if (Ot.has(this.op)) return "number";
              if (un.has(this.op)) return "string";
              if (this.op === "+") {
                const Yr = this.left.toType(Ar), jr = this.right.toType(Ar);
                if (Yr === "string" || jr === "string") return "string";
                if (Yr === "number" || jr === "number") return "number";
              }
              return yn.has(this.op) || this.allsome ? "boolean" : this.op ? "unknown" : this.left.toType(Ar);
            }
            toJS(Ar, Yr, jr) {
              var Mr;
              let Wr = [], Bi = this.op, gi = this, Zr = function($i) {
                return $i.toJS && ($i = $i.toJS(Ar, Yr, jr)), "y[" + (Wr.push($i) - 1) + "]";
              };
              var fi = function() {
                return Zr(gi.left);
              }, ri = function() {
                return Zr(gi.right);
              };
              if (this.op === "=") Bi = "===";
              else if (this.op === "<>") Bi = "!=";
              else if (this.op === "OR") Bi = "||";
              else if (this.op === "->") {
                const $i = `(${fi()} || {})`;
                if (typeof this.right == "string") Mr = `${$i}["${ee(this.right)}"]`;
                else if (typeof this.right == "number") Mr = `${$i}[${this.right}]`;
                else if (this.right instanceof ze.FuncValue) {
                  let Br = [];
                  this.right.args && this.right.args.length > 0 && (Br = this.right.args.map(Zr)), Mr = `${$i}[${JSON.stringify(this.right.funcid)}](${Br.join(",")})`;
                } else Mr = `${$i}[${ri()}]`;
              } else if (this.op === "!") typeof this.right == "string" && (Mr = `alasql.databases[alasql.useid].objects[${fi()}]["${this.right}"]`);
              else if (this.op === "IS") {
                const $i = fi(), Br = ri();
                this.right instanceof ze.NullValue || this.right.op === "NOT" && this.right.right instanceof ze.NullValue ? Mr = `((${$i} == null) === (${Br} == null))` : Mr = `((${$i} == ${Br}) || (${$i} < 0 && true == ${Br}))`;
              } else if (this.op === "==") Mr = `alasql.utils.deepEqual(${fi()}, ${ri()})`;
              else if (this.op === "===" || this.op === "!===") Mr = `(${this.op === "!===" ? "!" : ""}((${fi()}).valueOf() === (${ri()}).valueOf()))`;
              else if (this.op === "!==") Mr = `(!alasql.utils.deepEqual(${fi()}, ${ri()}))`;
              else if (this.op === "||") Mr = `(''+(${fi()} || '') + (${ri()} || ''))`;
              else if (this.op === "LIKE" || this.op === "NOT LIKE") Mr = `(${this.op === "NOT LIKE" ? "!" : ""}alasql.utils.like(${ri()}, ${fi()}${this.escape ? `, ${Zr(this.escape)}` : ""}))`;
              else if (this.op === "REGEXP") Mr = `alasql.stdfn.REGEXP_LIKE(${fi()}, ${ri()})`;
              else if (this.op === "GLOB") Mr = `alasql.utils.glob(${fi()}, ${ri()})`;
              else if (this.op === "BETWEEN" || this.op === "NOT BETWEEN") {
                const $i = fi();
                Mr = `(${this.op === "NOT BETWEEN" ? "!" : ""}((${Zr(this.right1)} <= ${$i}) && (${$i} <= ${Zr(this.right2)})))`;
              } else if (this.op === "IN") if (this.right instanceof ze.Select) Mr = `alasql.utils.flatArray(this.queriesfn[${this.queriesidx}](params, null, ${Ar})).indexOf(alasql.utils.getValueOf(${fi()})) > -1`;
              else if (Array.isArray(this.right)) if (!H.options.cache || this.right.some(($i) => $i instanceof ze.ParamValue)) Mr = `(new Set([${this.right.map(Zr).join(",")}]).has(alasql.utils.getValueOf(${fi()})))`;
              else {
                H.sets = H.sets || {};
                const $i = this.right.map((Pr) => Pr.value), Br = $i.join(",");
                H.sets[Br] = H.sets[Br] || new Set($i), Mr = `alasql.sets["${Br}"].has(alasql.utils.getValueOf(${fi()}))`;
              }
              else Mr = `(${ri()}.indexOf(${fi()}) > -1)`;
              else if (this.op === "NOT IN") if (this.right instanceof ze.Select) Mr = `alasql.utils.flatArray(this.queriesfn[${this.queriesidx}](params, null, p)).indexOf(alasql.utils.getValueOf(${fi()})) < 0`;
              else if (Array.isArray(this.right)) if (!H.options.cache || this.right.some(($i) => $i instanceof ze.ParamValue)) Mr = `(!(new Set([${this.right.map(Zr).join(",")}]).has(alasql.utils.getValueOf(${fi()}))))`;
              else {
                H.sets = H.sets || {};
                const $i = this.right.map((Pr) => Pr.value), Br = $i.join(",");
                H.sets[Br] = H.sets[Br] || new Set($i), Mr = `!alasql.sets["${Br}"].has(alasql.utils.getValueOf(${fi()}))`;
              }
              else Mr = `(${ri()}.indexOf(${fi()}) === -1)`;
              if (this.allsome === "ALL") {
                var Mr;
                if (this.right instanceof ze.Select) Mr = "alasql.utils.flatArray(this.query.queriesfn[" + this.queriesidx + "](params,null,p))", Mr += ".every(function(b){return (", Mr += fi() + ")" + Bi + "b})";
                else if (Array.isArray(this.right)) Mr = "" + (this.right.length == 1 ? Zr(this.right[0]) : "[" + this.right.map(Zr).join(",") + "]"), Mr += ".every(function(b){return (", Mr += fi() + ")" + Bi + "b})";
                else throw new Error("NOT IN operator without SELECT");
              }
              if (this.allsome === "SOME" || this.allsome === "ANY") {
                var Mr;
                if (this.right instanceof ze.Select) Mr = "alasql.utils.flatArray(this.query.queriesfn[" + this.queriesidx + "](params,null,p))", Mr += ".some(function(b){return (", Mr += fi() + ")" + Bi + "b})";
                else if (Array.isArray(this.right)) Mr = "" + (this.right.length == 1 ? Zr(this.right[0]) : "[" + this.right.map(Zr).join(",") + "]"), Mr += ".some(function(b){return (", Mr += fi() + ")" + Bi + "b})";
                else throw new Error("SOME/ANY operator without SELECT");
              }
              if (this.op === "AND") {
                if (this.left.reduced) {
                  if (this.right.reduced) return "true";
                  Mr = ri();
                } else this.right.reduced && (Mr = fi());
                Bi = "&&";
              }
              var bi = Mr || "(" + fi() + Bi + ri() + ")", Zi = "y=[(" + Wr.join("), (") + ")]";
              return Bi === "&&" || Bi === "||" || Bi === "IS" || Bi === "IS NULL" || Bi === "IS NOT NULL" ? "(" + Zi + ", " + bi + ")" : `(${Zi}, y.some(e => e == null) ? void 0 : ${bi})`;
            }
          }
          class gn {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return "@" + this.variable;
            }
            toType() {
              return "unknown";
            }
            toJS() {
              return "alasql.vars['" + ee(this.variable) + "']";
            }
          }
          class bn {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return this.value.toString();
            }
            toType() {
              return "number";
            }
            toJS() {
              return "" + this.value;
            }
          }
          class an {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return "'" + this.value.toString() + "'";
            }
            toType() {
              return "string";
            }
            toJS() {
              return "'" + ee(this.value) + "'";
            }
          }
          class Zt {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return "VALUE";
            }
            toType() {
              return "object";
            }
            toJS(Ar, Yr, jr) {
              return Ar;
            }
          }
          class Mn {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return "ARRAY[]";
            }
            toType() {
              return "object";
            }
            toJS(Ar, Yr, jr) {
              return "[(" + this.value.map(function(Mr) {
                return Mr.toJS(Ar, Yr, jr);
              }).join("), (") + ")]";
            }
          }
          class Ln {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return this.value ? "TRUE" : "FALSE";
            }
            toType() {
              return "boolean";
            }
            toJS() {
              return this.value ? "true" : "false";
            }
          }
          class In {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return "NULL";
            }
            toJS() {
              return "undefined";
            }
          }
          class kn {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return "$" + this.param;
            }
            toJS() {
              return typeof this.param == "string" ? "params['" + this.param + "']" : "params[" + this.param + "]";
            }
          }
          const er = { "~": "~", "-": "-", "+": "+", NOT: "!" };
          class jn {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              const { op: Ar, right: Yr } = this, jr = Yr.toString();
              switch (Ar) {
                case "~":
                case "-":
                case "+":
                case "#":
                  return Ar + jr;
                case "NOT":
                  return Ar + "(" + jr + ")";
                default:
                  return "(" + jr + ")";
              }
            }
            findAggregator(Ar) {
              this.right.findAggregator && this.right.findAggregator(Ar);
            }
            toType() {
              switch (this.op) {
                case "-":
                case "+":
                  return "number";
                case "NOT":
                  return "boolean";
                default:
                  return "string";
              }
            }
            toJS(Ar, Yr, jr) {
              if (this.right instanceof ur && this.op === "#") return `(alasql.databases[alasql.useid].objects['${this.right.columnid}'])`;
              const Mr = this.right.toJS(Ar, Yr, jr);
              if (er.hasOwnProperty(this.op)) return `(${er[this.op]}(${Mr}))`;
              if (this.op == null) return `(${Mr})`;
              throw new Error(`Unsupported operator: ${this.op}`);
            }
          }
          class ur {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              let Ar = this.columnid;
              return this.columnid == +this.columnid && (Ar = "[" + this.columnid + "]"), this.tableid && (Ar = this.tableid + (this.columnid === +this.columnid ? "" : ".") + Ar, this.databaseid && (Ar = this.databaseid + "." + Ar)), Ar;
            }
            toJS(Ar, Yr, jr) {
              if (!this.tableid && Yr === "" && !jr) return this.columnid !== "_" ? `${Ar}['${this.columnid}']` : Ar === "g" ? "g['_']" : Ar;
              if (Ar === "g") return `g['${this.nick}']`;
              if (this.tableid) return this.columnid !== "_" ? `${Ar}['${this.tableid}']['${this.columnid}']` : Ar === "g" ? "g['_']" : `${Ar}['${this.tableid}']`;
              if (jr) {
                const Mr = jr[this.columnid];
                if (Mr === "-") throw new Error(`Cannot resolve column "${this.columnid}" because it exists in two source tables`);
                return Mr ? this.columnid !== "_" ? `${Ar}['${Mr}']['${this.columnid}']` : `${Ar}['${Mr}']` : this.columnid !== "_" ? `${Ar}['${this.tableid || Yr}']['${this.columnid}']` : `${Ar}['${this.tableid || Yr}']`;
              }
              return Yr === -1 ? `${Ar}['${this.columnid}']` : this.columnid !== "_" ? `${Ar}['${this.tableid || Yr}']['${this.columnid}']` : `${Ar}['${this.tableid || Yr}']`;
            }
          }
          class Kn {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              const Ar = this.aggregatorid === "REDUCE" ? this.funcid.replace($t, "") : this.aggregatorid, Yr = this.distinct ? "DISTINCT " : "", jr = this.expression ? this.expression.toString() : "", Mr = this.over ? ` ${this.over.toString()}` : "";
              return `${Ar}(${Yr}${jr})${Mr}`;
            }
            findAggregator(Ar) {
              const Yr = ee(this.toString()) + ":" + Ar.selectGroup.length;
              this.nick || (this.nick = Yr, Ar.removeKeys.includes(Yr) || Ar.removeKeys.push(Yr)), Ar.selectGroup.push(this);
            }
            toType() {
              return ["SUM", "COUNT", "AVG", "MIN", "MAX", "AGGR", "VAR", "STDDEV", "TOTAL"].includes(this.aggregatorid) ? "number" : this.aggregatorid === "ARRAY" ? "array" : this.expression.toType();
            }
            toJS() {
              var Ar = this.nick;
              return Ar === void 0 && (Ar = ee(this.toString())), "g['" + Ar + "']";
            }
          }
          class Jn {
            constructor(Ar) {
              Ie(this, Ar);
            }
          }
          Jn.prototype.toString = ft.prototype.toString;
          class Lr {
            constructor(Ar) {
              Ie(this, Ar);
            }
            toString() {
              return this.type + "(" + this.group.toString() + ")";
            }
          }
          Ie(ze, { AggrValue: Kn, ArrayValue: Mn, Column: ur, DomainValueValue: Zt, Expression: ft, ExpressionStatement: rt, GroupExpression: Lr, JavaScript: pt, Join: Bt, Literal: wt, LogicValue: Ln, NullValue: In, NumValue: bn, Op: _n, OrderExpression: Jn, ParamValue: kn, StringValue: an, Table: At, UniOp: jn, VarValue: gn, View: Dt });
        }
        ze.FromData = function(Ie) {
          return ze.extend(this, Ie);
        }, ze.FromData.prototype.toString = function() {
          return this.data ? "DATA(" + (Math.random() * 1e16 | 0) + ")" : "?";
        }, ze.FromData.prototype.toJS = function() {
        }, ze.Select.prototype.exec = function(Ie, rt) {
          this.preparams && (Ie = this.preparams.concat(Ie));
          var ft = H.useid, pt = H.databases[ft], wt = this.toString(), Bt = fe(wt), At = this.compile(ft);
          if (At) {
            At.sql = wt, At.dbversion = pt.dbversion, pt.sqlCacheSize > H.MAXSQLCACHESIZE && pt.resetSqlCache(), pt.sqlCacheSize++, pt.sqlCache[Bt] = At;
            var Dt = H.res = At(Ie, rt);
            return Dt;
          }
        }, ze.Select.prototype.Select = function() {
          var Ie = this, rt = [];
          if (arguments.length > 1) rt = Array.prototype.slice.call(arguments);
          else if (arguments.length == 1) Array.isArray(arguments[0]) ? rt = arguments[0] : rt = [arguments[0]];
          else throw new Error("Wrong number of arguments of Select() function");
          return Ie.columns = [], rt.forEach(function(ft) {
            if (typeof ft == "string") Ie.columns.push(new ze.Column({ columnid: ft }));
            else if (typeof ft == "function") {
              var pt = 0;
              Ie.preparams ? pt = Ie.preparams.length : Ie.preparams = [], Ie.preparams.push(ft), Ie.columns.push(new ze.Column({ columnid: "*", func: ft, param: pt }));
            }
          }), Ie;
        }, ze.Select.prototype.From = function(Ie) {
          var rt = this;
          if (rt.from || (rt.from = []), Array.isArray(Ie)) {
            var ft = 0;
            rt.preparams ? ft = rt.preparams.length : rt.preparams = [], rt.preparams.push(Ie), rt.from.push(new ze.ParamValue({ param: ft }));
          } else if (typeof Ie == "string") rt.from.push(new ze.Table({ tableid: Ie }));
          else throw new Error("Unknown arguments in From() function");
          return rt;
        }, ze.Select.prototype.OrderBy = function() {
          var Ie = this, rt = [];
          if (Ie.order = [], arguments.length == 0) rt = ["_"];
          else if (arguments.length > 1) rt = Array.prototype.slice.call(arguments);
          else if (arguments.length == 1) Array.isArray(arguments[0]) ? rt = arguments[0] : rt = [arguments[0]];
          else throw new Error("Wrong number of arguments of Select() function");
          return rt.length > 0 && rt.forEach(function(ft) {
            var pt = new ze.Column({ columnid: ft });
            typeof ft == "function" && (pt = ft), Ie.order.push(new ze.OrderExpression({ expression: pt, direction: "ASC" }));
          }), Ie;
        }, ze.Select.prototype.Top = function(Ie) {
          var rt = this;
          return rt.top = new ze.NumValue({ value: Ie }), rt;
        }, ze.Select.prototype.GroupBy = function() {
          var Ie = this, rt = [];
          if (arguments.length > 1) rt = Array.prototype.slice.call(arguments);
          else if (arguments.length == 1) Array.isArray(arguments[0]) ? rt = arguments[0] : rt = [arguments[0]];
          else throw new Error("Wrong number of arguments of Select() function");
          return Ie.group = [], rt.forEach(function(ft) {
            var pt = new ze.Column({ columnid: ft });
            Ie.group.push(pt);
          }), Ie;
        }, ze.Select.prototype.Where = function(Ie) {
          var rt = this;
          return typeof Ie == "function" && (rt.where = Ie), rt;
        }, ze.FuncValue = function(Ie) {
          return Object.assign(this, Ie);
        };
        let $t = /[^0-9A-Z_$]+/i;
        ze.FuncValue.prototype.toString = function() {
          let Ie = "";
          return H.fn[this.funcid] ? Ie += this.funcid : H.aggr[this.funcid] ? Ie += this.funcid : (H.stdlib[this.funcid.toUpperCase()] || H.stdfn[this.funcid.toUpperCase()]) && (Ie += this.funcid.toUpperCase().replace($t, "")), this.funcid !== "CURRENT_TIMESTAMP" && (Ie += "(", this.args && this.args.length > 0 && (Ie += this.args.map(function(rt) {
            return rt.toString();
          }).join(",")), Ie += ")"), Ie;
        }, ze.FuncValue.prototype.execute = function(Ie, rt, ft) {
          let pt = 1;
          return H.precompile(this, Ie, rt), new Function("params,alasql", "var y;return " + this.toJS("", "", null))(rt, H), ft && (pt = ft(pt)), pt;
        }, ze.FuncValue.prototype.findAggregator = function(Ie) {
          this.args && this.args.length > 0 && this.args.forEach(function(rt) {
            rt.findAggregator && rt.findAggregator(Ie);
          });
        }, ze.FuncValue.prototype.toJS = function(Ie, rt, ft) {
          var pt = "", wt = this.funcid;
          return !H.fn[wt] && H.stdlib[wt.toUpperCase()] ? this.args && this.args.length > 0 ? pt += H.stdlib[wt.toUpperCase()].apply(this, this.args.map(function(Bt) {
            return Bt.toJS(Ie, rt);
          })) : pt += H.stdlib[wt.toUpperCase()]() : !H.fn[wt] && H.stdfn[wt.toUpperCase()] ? (this.newid && (pt += "new "), pt += "alasql.stdfn[" + JSON.stringify(this.funcid.toUpperCase()) + "](", this.args && this.args.length > 0 && (pt += this.args.map(function(Bt) {
            return Bt.toJS(Ie, rt, ft);
          }).join(",")), pt += ")") : (this.newid && (pt += "new "), pt += "alasql.fn[" + JSON.stringify(this.funcid) + "](", this.args && this.args.length > 0 && (pt += this.args.map(function(Bt) {
            return Bt.toJS(Ie, rt, ft);
          }).join(",")), pt += ")"), pt;
        };
        var Pt = H.stdlib = {}, Xt = H.stdfn = {};
        Pt.ABS = function(Ie) {
          return "Math.abs(" + Ie + ")";
        }, Pt.CLONEDEEP = function(Ie) {
          return "alasql.utils.cloneDeep(" + Ie + ")";
        }, Xt.CONCAT = function() {
          return Array.prototype.slice.call(arguments).join("");
        }, Pt.EXP = function(Ie) {
          return "Math.pow(Math.E," + Ie + ")";
        }, Pt.IIF = function(Ie, rt, ft) {
          if (arguments.length === 3) return `((${Ie}) ? (${rt}) : (${ft}))`;
          throw new Error("Number of arguments of IFF is not equals to 3");
        }, Pt.IFNULL = function(Ie, rt) {
          return `((typeof ${Ie} === "undefined" || ${Ie} === null) ? ${rt} : ${Ie})`;
        }, Pt.INSTR = function(Ie, rt) {
          return `((${Ie}).indexOf(${rt}) + 1)`;
        }, Pt.LEN = Pt.LENGTH = function(Ie) {
          return te(Ie, "y.length");
        }, Pt.LOWER = Pt.LCASE = function(Ie) {
          return te(Ie, "String(y).toLowerCase()");
        }, Pt.LTRIM = function(Ie) {
          return te(Ie, 'y.replace(/^[ ]+/,"")');
        }, Pt.RTRIM = function(Ie) {
          return te(Ie, 'y.replace(/[ ]+$/,"")');
        }, Pt.MAX = Pt.GREATEST = function() {
          return "[" + Array.prototype.join.call(arguments, ",") + "].reduce(function (a, b) { return a > b ? a : b; })";
        }, Pt.MIN = Pt.LEAST = function() {
          return "[" + Array.prototype.join.call(arguments, ",") + "].reduce(function (a, b) { return a < b ? a : b; })";
        }, Pt.SUBSTRING = Pt.SUBSTR = Pt.MID = function(Ie, rt, ft) {
          if (arguments.length == 2) return te(Ie, "y.substr(" + rt + "-1)");
          if (arguments.length == 3) return te(Ie, "y.substr(" + rt + "-1," + ft + ")");
        }, Xt.REGEXP_LIKE = function(Ie, rt, ft) {
          return (Ie || "").search(RegExp(rt, ft)) > -1;
        }, Pt.ISNULL = Pt.NULLIF = function(Ie, rt) {
          return "(" + Ie + "==" + rt + "?undefined:" + Ie + ")";
        }, Pt.POWER = function(Ie, rt) {
          return "Math.pow(" + Ie + "," + rt + ")";
        }, Pt.RANDOM = function(Ie) {
          return arguments.length == 0 ? "Math.random()" : "(Math.random()*(" + Ie + ")|0)";
        }, Pt.ROUND = function(Ie, rt) {
          return arguments.length == 2 ? "Math.round((" + Ie + ")*Math.pow(10,(" + rt + ")))/Math.pow(10,(" + rt + "))" : "Math.round(" + Ie + ")";
        }, Pt.CEIL = Pt.CEILING = function(Ie) {
          return "Math.ceil(" + Ie + ")";
        }, Pt.FLOOR = function(Ie) {
          return "Math.floor(" + Ie + ")";
        }, Pt.ROWNUM = function() {
          return "1";
        }, Pt.ROW_NUMBER = function() {
          return "1";
        }, Pt.SQRT = function(Ie) {
          return "Math.sqrt(" + Ie + ")";
        }, Pt.TRIM = function(Ie) {
          return te(Ie, "y.trim()");
        }, Pt.UPPER = Pt.UCASE = function(Ie) {
          return te(Ie, "String(y).toUpperCase()");
        }, Xt.CONCAT_WS = function() {
          var Ie = Array.prototype.slice.call(arguments);
          return Ie = Ie.filter((rt) => !(rt === null || typeof rt > "u")), Ie.slice(1, Ie.length).join(Ie[0] || "");
        }, H.aggr.group_concat = H.aggr.GROUP_CONCAT = function(Ie, rt, ft) {
          return ft === 1 ? "" + Ie : (ft === 2 && (rt += "," + Ie), rt);
        }, H.aggr.median = H.aggr.MEDIAN = function(Ie, rt, ft) {
          if (ft === 2) return Ie !== null && rt.push(Ie), rt;
          if (ft === 1) return Ie === null ? [] : [Ie];
          if (!rt.length) return null;
          let pt = rt.sort((Dt, Ot) => Dt > Ot ? 1 : Dt < Ot ? -1 : 0), wt = (pt.length + 1) / 2, Bt = wt | 0, At = pt[Bt - 1];
          return wt === Bt || typeof At != "number" && !(At instanceof Number) ? At : (At + pt[Bt]) / 2;
        }, H.aggr.QUART = function(Ie, rt, ft, pt) {
          if (ft === 2) return Ie !== null && rt.push(Ie), rt;
          if (ft === 1) return Ie === null ? [] : [Ie];
          if (!rt.length) return rt;
          pt = pt || 1;
          var wt = rt.sort(function(At, Dt) {
            return At === Dt ? 0 : At > Dt ? 1 : -1;
          });
          let Bt = pt * (wt.length + 1) / 4;
          return Number.isInteger(Bt) ? wt[Bt - 1] : wt[Math.floor(Bt)];
        }, H.aggr.QUART2 = function(Ie, rt, ft) {
          return H.aggr.QUART(Ie, rt, ft, 2);
        }, H.aggr.QUART3 = function(Ie, rt, ft) {
          return H.aggr.QUART(Ie, rt, ft, 3);
        }, H.aggr.VAR = function(Ie, rt, ft) {
          return ft === 1 ? Ie === null ? { sum: 0, sumSq: 0, count: 0 } : { sum: Ie, sumSq: Ie * Ie, count: 1 } : ft === 2 ? (Ie !== null && (rt.sum += Ie, rt.sumSq += Ie * Ie, rt.count++), rt) : rt.count > 1 ? (rt.sumSq - rt.sum * rt.sum / rt.count) / (rt.count - 1) : 0;
        }, H.aggr.STDEV = function(Ie, rt, ft) {
          return ft === 1 || ft === 2 ? H.aggr.VAR(Ie, rt, ft) : Math.sqrt(H.aggr.VAR(Ie, rt, ft));
        }, H.aggr.STDEV = function(Ie, rt, ft) {
          return ft === 1 || ft === 2 ? H.aggr.VAR(Ie, rt, ft) : Math.sqrt(H.aggr.VAR(Ie, rt, ft));
        }, H.aggr.VARP = function(Ie, rt, ft) {
          if (ft === 1) return { count: 1, sum: Ie, sumSq: Ie * Ie };
          if (ft === 2) return rt.count++, rt.sum += Ie, rt.sumSq += Ie * Ie, rt;
          if (rt.count > 0) {
            const pt = rt.sum / rt.count;
            return rt.sumSq / rt.count - pt * pt;
          } else return 0;
        }, H.aggr.STD = H.aggr.STDDEV = H.aggr.STDEVP = function(Ie, rt, ft) {
          return ft == 1 || ft == 2 ? H.aggr.VARP(Ie, rt, ft) : Math.sqrt(H.aggr.VARP(Ie, rt, ft));
        }, H._aggrOriginal = H.aggr, H.aggr = {}, Object.keys(H._aggrOriginal).forEach(function(Ie) {
          H.aggr[Ie] = function(rt, ft, pt) {
            if (!(pt === 3 && typeof ft > "u")) return H._aggrOriginal[Ie].apply(null, arguments);
          };
        }), Xt.REPLACE = function(Ie, rt, ft) {
          return (Ie || "").split(rt).join(ft);
        };
        for (var Nt = [], Gt = 0; Gt < 256; Gt++) Nt[Gt] = (Gt < 16 ? "0" : "") + Gt.toString(16);
        Xt.NEWID = Xt.UUID = Xt.GEN_RANDOM_UUID = function() {
          var Ie = Math.random() * 4294967295 | 0, rt = Math.random() * 4294967295 | 0, ft = Math.random() * 4294967295 | 0, pt = Math.random() * 4294967295 | 0;
          return Nt[Ie & 255] + Nt[Ie >> 8 & 255] + Nt[Ie >> 16 & 255] + Nt[Ie >> 24 & 255] + "-" + Nt[rt & 255] + Nt[rt >> 8 & 255] + "-" + Nt[rt >> 16 & 15 | 64] + Nt[rt >> 24 & 255] + "-" + Nt[ft & 63 | 128] + Nt[ft >> 8 & 255] + "-" + Nt[ft >> 16 & 255] + Nt[ft >> 24 & 255] + Nt[pt & 255] + Nt[pt >> 8 & 255] + Nt[pt >> 16 & 255] + Nt[pt >> 24 & 255];
        }, ze.CaseValue = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CaseValue.prototype.toString = function() {
          var Ie = "CASE ";
          return this.expression && (Ie += this.expression.toString()), this.whens && (Ie += this.whens.map(function(rt) {
            return " WHEN " + rt.when.toString() + " THEN " + rt.then.toString();
          }).join()), Ie += " END", Ie;
        }, ze.CaseValue.prototype.findAggregator = function(Ie) {
          this.expression && this.expression.findAggregator && this.expression.findAggregator(Ie), this.whens && this.whens.length > 0 && this.whens.forEach(function(rt) {
            rt.when.findAggregator && rt.when.findAggregator(Ie), rt.then.findAggregator && rt.then.findAggregator(Ie);
          }), this.elses && this.elses.findAggregator && this.elses.findAggregator(Ie);
        }, ze.CaseValue.prototype.toJS = function(Ie, rt, ft) {
          let pt = `(((${Ie}, params, alasql) => {
        let y, r;`;
          return this.expression ? (pt += `let v = ${this.expression.toJS(Ie, rt, ft)};`, this.whens.forEach((wt, Bt) => {
            const At = `v === ${wt.when.toJS(Ie, rt, ft)}`, Dt = `r = ${wt.then.toJS(Ie, rt, ft)}`;
            pt += `${Bt === 0 ? "if" : " else if"} (${At}) { ${Dt}; }`;
          })) : this.whens.forEach((wt, Bt) => {
            const At = wt.when.toJS(Ie, rt, ft), Dt = `r = ${wt.then.toJS(Ie, rt, ft)}`;
            pt += `${Bt === 0 ? "if" : " else if"} (${At}) { ${Dt}; }`;
          }), this.elses && (pt += ` else { r = ${this.elses.toJS(Ie, rt, ft)}; }`), pt += "; return r; }))(" + Ie + ", params, alasql)", pt;
        }, ze.Json = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Json.prototype.toString = function() {
          var Ie = "";
          return Ie += Ft(this.value), Ie += "", Ie;
        };
        const Ft = H.utils.JSONtoString = function(Ie) {
          if (typeof Ie == "string") return `"${Ie}"`;
          if (typeof Ie == "number" || typeof Ie == "boolean") return String(Ie);
          if (typeof Ie == "bigint") return `${Ie.toString()}n`;
          if (Array.isArray(Ie)) return `[${Ie.map((rt) => Ft(rt)).join(",")}]`;
          if (typeof Ie == "object") if (!Ie.toJS || Ie instanceof ze.Json) {
            const rt = [];
            for (const ft in Ie) {
              const pt = typeof ft == "string" ? `"${ft}"` : String(ft), wt = Ft(Ie[ft]);
              rt.push(`${pt}:${wt}`);
            }
            return `{${rt.join(",")}}`;
          } else {
            if (Ie.toString) return Ie.toString();
            throw new Error(`1: Cannot show JSON object ${JSON.stringify(Ie)}`);
          }
          else throw new Error(`2: Cannot show JSON object ${JSON.stringify(Ie)}`);
        };
        function Tt(Ie, rt, ft, pt) {
          var wt = "";
          if (typeof Ie == "string") wt = '"' + Ie + '"';
          else if (typeof Ie == "number") wt = "(" + Ie + ")";
          else if (typeof Ie == "boolean") wt = Ie;
          else if (typeof Ie == "bigint") wt = Ie.toString() + "n";
          else if (typeof Ie == "object") if (Array.isArray(Ie)) wt += `[${Ie.map((Bt) => Tt(Bt, rt, ft, pt)).join(",")}]`;
          else if (!Ie.toJS || Ie instanceof ze.Json) {
            let Bt = [];
            for (const At in Ie) {
              let Dt = typeof At == "string" ? `"${At}"` : At.toString(), Ot = Tt(Ie[At], rt, ft, pt);
              Bt.push(`${Dt}:${Ot}`);
            }
            wt = `{${Bt.join(",")}}`;
          } else if (Ie.toJS) wt = Ie.toJS(rt, ft, pt);
          else throw new Error(`Cannot parse JSON object ${JSON.stringify(Ie)}`);
          else throw new Error("2Can not parse JSON object " + JSON.stringify(Ie));
          return wt;
        }
        ze.Json.prototype.toJS = function(Ie, rt, ft) {
          return Tt(this.value, Ie, rt, ft);
        }, ze.Convert = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Convert.prototype.toString = function() {
          var Ie = "CONVERT(";
          return Ie += this.dbtypeid, typeof this.dbsize < "u" && (Ie += "(" + this.dbsize, this.dbprecision && (Ie += "," + this.dbprecision), Ie += ")"), Ie += "," + this.expression.toString(), this.style && (Ie += "," + this.style), Ie += ")", Ie;
        }, ze.Convert.prototype.toJS = function(Ie, rt, ft) {
          return `alasql.stdfn.CONVERT(${this.expression.toJS(Ie, rt, ft)}, {
        dbtypeid: "${this.dbtypeid}",
        dbsize: ${this.dbsize},
        dbprecision: ${this.dbprecision},
        style: ${this.style}
    })`;
        };
        function It(Ie) {
          var rt = Ie.getMonth() + 1, ft = Ie.getYear(), pt = Ie.getFullYear(), wt = Ie.getDate(), Bt = Ie.toString().substr(4, 3), At = ("0" + wt).substr(-2), Dt = ("0" + rt).substr(-2), Ot = ("0" + ft).substr(-2), un = ("0" + Ie.getHours()).substr(-2), yn = ("0" + Ie.getMinutes()).substr(-2), _n = ("0" + Ie.getSeconds()).substr(-2), gn = ("00" + Ie.getMilliseconds()).substr(-3);
          return { month: rt, year: ft, fullYear: pt, date: wt, day: Bt, formattedDate: At, formattedMonth: Dt, formattedYear: Ot, formattedHour: un, formattedMinutes: yn, formattedSeconds: _n, formattedMilliseconds: gn };
        }
        H.stdfn.CONVERT = function(Ie, rt) {
          var Dt;
          var ft = Ie, pt = (Dt = rt.dbtypeid) == null ? void 0 : Dt.toUpperCase(), wt, Bt;
          if ((rt.style || rt.dbtypeid == "Date" || ["DATE", "DATETIME", "DATETIME2"].indexOf(pt) > -1) && (/\d{8}/.test(ft) ? wt = new Date(+ft.substr(0, 4), +ft.substr(4, 2) - 1, +ft.substr(6, 2)) : wt = Vt(ft), Bt = It(wt)), rt.style) switch (rt.style) {
            case 1:
              ft = Bt.formattedMonth + "/" + Bt.formattedDate + "/" + Bt.formattedYear;
              break;
            case 2:
              ft = Bt.formattedYear + "." + Bt.formattedMonth + "." + Bt.formattedDate;
              break;
            case 3:
              ft = Bt.formattedDate + "/" + Bt.formattedMonth + "/" + Bt.formattedYear;
              break;
            case 4:
              ft = Bt.formattedDate + "." + Bt.formattedMonth + "." + Bt.formattedYear;
              break;
            case 5:
              ft = Bt.formattedDate + "-" + Bt.formattedMonth + "-" + Bt.formattedYear;
              break;
            case 6:
              ft = Bt.formattedDate + " " + Bt.day.toLowerCase() + " " + Bt.formattedYear;
              break;
            case 7:
              ft = Bt.day + " " + Bt.formattedDate + "," + Bt.formattedYear;
              break;
            case 8:
            case 108:
              ft = Bt.formattedHour + ":" + Bt.formattedMinutes + ":" + Bt.formattedSeconds;
              break;
            case 10:
              ft = Bt.formattedMonth + "-" + Bt.formattedDate + "-" + Bt.formattedYear;
              break;
            case 11:
              ft = Bt.formattedYear + "/" + Bt.formattedMonth + "/" + Bt.formattedDate;
              break;
            case 12:
              ft = Bt.formattedYear + Bt.formattedMonth + Bt.formattedDate;
              break;
            case 101:
              ft = Bt.formattedMonth + "/" + Bt.formattedDate + "/" + Bt.fullYear;
              break;
            case 102:
              ft = Bt.fullYear + "." + Bt.formattedMonth + "." + Bt.formattedDate;
              break;
            case 103:
              ft = Bt.formattedDate + "/" + Bt.formattedMonth + "/" + Bt.fullYear;
              break;
            case 104:
              ft = Bt.formattedDate + "." + Bt.formattedMonth + "." + Bt.fullYear;
              break;
            case 105:
              ft = Bt.formattedDate + "-" + Bt.formattedMonth + "-" + Bt.fullYear;
              break;
            case 106:
              ft = Bt.formattedDate + " " + Bt.day.toLowerCase() + " " + Bt.fullYear;
              break;
            case 107:
              ft = Bt.day + " " + Bt.formattedDate + "," + Bt.fullYear;
              break;
            case 110:
              ft = Bt.formattedMonth + "-" + Bt.formattedDate + "-" + Bt.fullYear;
              break;
            case 111:
              ft = Bt.fullYear + "/" + Bt.formattedMonth + "/" + Bt.formattedDate;
              break;
            case 112:
              ft = Bt.fullYear + Bt.formattedMonth + Bt.formattedDate;
              break;
            default:
              throw new Error("The CONVERT style " + rt.style + " is not realized yet.");
          }
          switch (pt) {
            case "DATE":
              return `${Bt.formattedYear}.${Bt.formattedMonth}.${Bt.formattedDate}`;
            case "DATETIME":
            case "DATETIME2":
              return `${Bt.fullYear}.${Bt.formattedMonth}.${Bt.formattedDate} ${Bt.formattedHour}:${Bt.formattedMinutes}:${Bt.formattedSeconds}.${Bt.formattedMilliseconds}`;
            case "MONEY":
              var At = +ft;
              return (At | 0) + At * 100 % 100 / 100;
            case "BOOLEAN":
              return !!ft;
            case "INT":
            case "INTEGER":
            case "SMALLINT":
            case "BIGINT":
            case "SERIAL":
            case "SMALLSERIAL":
            case "BIGSERIAL":
              return ft | 0;
            case "STRING":
            case "VARCHAR":
            case "NVARCHAR":
            case "CHARACTER VARIABLE":
              return rt.dbsize ? String(ft).substr(0, rt.dbsize) : String(ft);
            case "CHAR":
            case "CHARACTER":
            case "NCHAR":
              return (ft + " ".repeat(rt.dbsize)).substr(0, rt.dbsize);
            case "NUMBER":
            case "FLOAT":
            case "DECIMAL":
            case "NUMERIC":
              var At = +ft;
              return rt.dbsize !== void 0 && (At = parseFloat(At.toPrecision(rt.dbsize))), rt.dbprecision !== void 0 && (At = parseFloat(At.toFixed(rt.dbprecision))), At;
            case "JSON":
              if (typeof ft == "object") return ft;
              try {
                return JSON.parse(ft);
              } catch {
                throw new Error("Cannot convert string to JSON");
              }
            case "Date":
              return ft;
            default:
              return ft;
          }
        }, ze.ColumnDef = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.ColumnDef.prototype.toString = function() {
          let Ie = this.columnid;
          return this.dbtypeid && (Ie += " " + this.dbtypeid), this.dbsize && (Ie += "(" + this.dbsize, this.dbprecision && (Ie += "," + this.dbprecision), Ie += ")"), this.primarykey && (Ie += " PRIMARY KEY"), this.notnull && (Ie += " NOT NULL"), Ie;
        }, ze.CreateTable = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CreateTable.prototype.toString = function() {
          let Ie = `CREATE${this.temporary ? " TEMPORARY" : ""}${this.view ? " VIEW" : ` ${this.class ? "CLASS" : "TABLE"}`}${this.ifnotexists ? " IF NOT EXISTS" : ""} ${this.table.toString()}`;
          return this.viewcolumns && (Ie += `(${this.viewcolumns.map((rt) => rt.toString()).join(",")})`), this.as ? Ie += ` AS ${this.as}` : Ie += ` (${this.columns.map((rt) => rt.toString()).join(",")})`, this.view && this.select && (Ie += ` AS ${this.select.toString()}`), Ie;
        }, ze.CreateTable.prototype.execute = function(Ie, rt, ft) {
          var pt = H.databases[this.table.databaseid || Ie], wt = this.table.tableid;
          if (!wt) throw new Error("Table name is not defined");
          var Bt = this.columns, At = this.constraints || [];
          if (this.ifnotexists && pt.tables[wt]) return ft ? ft(0) : 0;
          if (pt.tables[wt]) throw new Error("Can not create table '" + wt + "', because it already exists in the database '" + pt.databaseid + "'");
          var Dt = pt.tables[wt] = new H.Table();
          this.class && (Dt.isclass = !0);
          var Ot = [], un = [];
          if (Bt && Bt.forEach(function(gn) {
            var bn = gn.dbtypeid;
            H.fn[bn] || (bn = bn.toUpperCase()), ["SERIAL", "SMALLSERIAL", "BIGSERIAL"].indexOf(bn) > -1 && (gn.identity = { value: 1, step: 1 });
            var an = { columnid: gn.columnid, dbtypeid: bn, dbsize: gn.dbsize, dbprecision: gn.dbprecision, notnull: gn.notnull, identity: gn.identity };
            if (gn.identity && (Dt.identities[gn.columnid] = { value: +gn.identity.value, step: +gn.identity.step }), gn.check && Dt.checks.push({ id: gn.check.constrantid, fn: new Function("r", "var y;return " + gn.check.expression.toJS("r", "")) }), gn.default && Ot.push(JSON.stringify("" + gn.columnid) + ":" + gn.default.toJS("r", "")), gn.primarykey) {
              var Zt = Dt.pk = {};
              Zt.columns = [gn.columnid], Zt.onrightfns = `r[${JSON.stringify(gn.columnid)}]`, Zt.onrightfn = new Function("r", "var y;return " + Zt.onrightfns), Zt.hh = fe(Zt.onrightfns), Dt.uniqs[Zt.hh] = {};
            }
            if (gn.unique) {
              var Mn = {};
              Dt.uk = Dt.uk || [], Dt.uk.push(Mn), Mn.columns = [gn.columnid], Mn.onrightfns = `r[${JSON.stringify(gn.columnid)}]`, Mn.onrightfn = new Function("r", "var y;return " + Mn.onrightfns), Mn.hh = fe(Mn.onrightfns), Dt.uniqs[Mn.hh] = {};
            }
            if (gn.foreignkey) {
              var Ln = gn.foreignkey.table, In = H.databases[Ln.databaseid || Ie].tables[Ln.tableid];
              if (typeof Ln.columnid > "u") if (In.pk.columns && In.pk.columns.length > 0) Ln.columnid = In.pk.columns[0];
              else throw new Error("FOREIGN KEY allowed only to tables with PRIMARY KEYs");
              var kn = function(er) {
                var jn = {};
                if (typeof er[gn.columnid] > "u") return !0;
                jn[Ln.columnid] = er[gn.columnid];
                var ur = In.pk.onrightfn(jn);
                if (!In.uniqs[In.pk.hh][ur]) throw new Error("Foreign key violation");
                return !0;
              };
              Dt.checks.push({ fn: kn });
            }
            gn.onupdate && un.push(`r[${JSON.stringify(gn.columnid)}]=` + gn.onupdate.toJS("r", "")), Dt.columns.push(an), Dt.xcolumns[an.columnid] = an;
          }), Dt.defaultfns = Ot.join(","), Dt.onupdatefns = un.join(";"), At.forEach(function(gn) {
            var bn;
            if (gn.type === "PRIMARY KEY") {
              if (Dt.pk) throw new Error("Primary key already exists");
              var an = Dt.pk = {};
              an.columns = gn.columns, an.onrightfns = an.columns.map(function(In) {
                return `r[${JSON.stringify(In)}]`;
              }).join("+'`'+"), an.onrightfn = new Function("r", "var y;return " + an.onrightfns), an.hh = fe(an.onrightfns), Dt.uniqs[an.hh] = {};
            } else if (gn.type === "CHECK") bn = new Function("r", "var y;return " + gn.expression.toJS("r", ""));
            else if (gn.type === "UNIQUE") {
              var Zt = {};
              Dt.uk = Dt.uk || [], Dt.uk.push(Zt), Zt.columns = gn.columns, Zt.onrightfns = Zt.columns.map(function(In) {
                return `r[${JSON.stringify(In)}]`;
              }).join("+'`'+"), Zt.onrightfn = new Function("r", "var y;return " + Zt.onrightfns), Zt.hh = fe(Zt.onrightfns), Dt.uniqs[Zt.hh] = {};
            } else if (gn.type === "FOREIGN KEY") {
              var Mn = gn.fktable;
              gn.fkcolumns && gn.fkcolumns.length > 0 && (Mn.fkcolumns = gn.fkcolumns);
              var Ln = H.databases[Mn.databaseid || Ie].tables[Mn.tableid];
              if (typeof Mn.fkcolumns > "u" && (Mn.fkcolumns = Ln.pk.columns), Mn.columns = gn.columns, Mn.fkcolumns.length > Mn.columns.length) throw new Error("Invalid foreign key on table " + Dt.tableid);
              bn = function(In) {
                var kn = {};
                if (Mn.fkcolumns.forEach(function(ur, Kn) {
                  In[Mn.columns[Kn]] != null && (kn[ur] = In[Mn.columns[Kn]]);
                }), Object.keys(kn).length === 0) return !0;
                if (Object.keys(kn).length !== Mn.columns.length) throw new Error("Invalid foreign key on table " + Dt.tableid);
                var er = H.databases[Mn.databaseid || Ie].tables[Mn.tableid], jn = er.pk.onrightfn(kn);
                if (!er.uniqs[er.pk.hh][jn]) throw new Error("Foreign key violation");
                return !0;
              };
            }
            bn && Dt.checks.push({ fn: bn, id: gn.constraintid, fk: gn.type === "FOREIGN KEY" });
          }), this.view && this.viewcolumns) {
            var yn = this;
            this.viewcolumns.forEach(function(gn, bn) {
              yn.select.columns[bn].as = gn.columnid;
            });
          }
          if (this.view && this.select && (Dt.view = !0, Dt.select = this.select.compile(this.table.databaseid || Ie)), pt.engineid) return H.engines[pt.engineid].createTable(this.table.databaseid || Ie, wt, this.ifnotexists, ft);
          Dt.insert = function(gn, bn) {
            var an = H.inserted;
            H.inserted = [gn];
            var Zt = this, Mn = !1, Ln = !1;
            for (var In in Zt.beforeinsert) {
              var kn = Zt.beforeinsert[In];
              kn && (kn.funcid ? H.fn[kn.funcid](gn) === !1 && (Ln = Ln || !0) : kn.statement && kn.statement.execute(Ie) === !1 && (Ln = Ln || !0));
            }
            if (!Ln) {
              var er = !1;
              for (In in Zt.insteadofinsert) er = !0, kn = Zt.insteadofinsert[In], kn && (kn.funcid ? H.fn[kn.funcid](gn) : kn.statement && kn.statement.execute(Ie));
              if (!er) {
                for (var jn in Zt.identities) {
                  var ur = Zt.identities[jn];
                  gn[jn] = ur.value;
                }
                if (Zt.checks && Zt.checks.length > 0 && Zt.checks.forEach(function(Lr) {
                  if (!Lr.fn(gn)) throw new Error("Violation of CHECK constraint " + (Lr.id || ""));
                }), Zt.columns.forEach(function(Lr) {
                  if (Lr.notnull && typeof gn[Lr.columnid] > "u") throw new Error("Wrong NULL value in NOT NULL column " + Lr.columnid);
                }), Zt.pk) {
                  var Kn = Zt.pk, Jn = Kn.onrightfn(gn);
                  if (typeof Zt.uniqs[Kn.hh][Jn] < "u") if (bn) Mn = Zt.uniqs[Kn.hh][Jn];
                  else throw new Error("Cannot insert record, because it already exists in primary key index");
                }
                if (Zt.uk && Zt.uk.length && Zt.uk.forEach(function(Lr) {
                  var mr = Lr.onrightfn(gn);
                  if (typeof Zt.uniqs[Lr.hh][mr] < "u") if (bn) Mn = Zt.uniqs[Lr.hh][mr];
                  else throw new Error("Cannot insert record, because it already exists in unique index");
                }), Mn) Zt.update(function(Lr) {
                  for (var mr in gn) Lr[mr] = gn[mr];
                }, Zt.data.indexOf(Mn), rt);
                else {
                  Zt.data.push(gn);
                  for (var jn in Zt.identities) {
                    var ur = Zt.identities[jn];
                    ur.value += ur.step;
                  }
                  if (Zt.pk) {
                    var Kn = Zt.pk, Jn = Kn.onrightfn(gn);
                    Zt.uniqs[Kn.hh][Jn] = gn;
                  }
                  Zt.uk && Zt.uk.length && Zt.uk.forEach(function(Lr) {
                    var mr = Lr.onrightfn(gn);
                    Zt.uniqs[Lr.hh][mr] = gn;
                  });
                }
                for (var In in Zt.afterinsert) {
                  var kn = Zt.afterinsert[In];
                  kn && (kn.funcid ? H.fn[kn.funcid](gn) : kn.statement && kn.statement.execute(Ie));
                }
                H.inserted = an;
              }
            }
          }, Dt.delete = function(gn) {
            var bn = this, an = bn.data[gn], Zt = !1;
            for (var Mn in bn.beforedelete) {
              var Ln = bn.beforedelete[Mn];
              Ln && (Ln.funcid ? H.fn[Ln.funcid](an) === !1 && (Zt = Zt || !0) : Ln.statement && Ln.statement.execute(Ie) === !1 && (Zt = Zt || !0));
            }
            if (Zt) return !1;
            var In = !1;
            for (var Mn in bn.insteadofdelete) {
              In = !0;
              var Ln = bn.insteadofdelete[Mn];
              Ln && (Ln.funcid ? H.fn[Ln.funcid](an) : Ln.statement && Ln.statement.execute(Ie));
            }
            if (!In) {
              if (this.pk) {
                var kn = this.pk, er = kn.onrightfn(an);
                if (typeof this.uniqs[kn.hh][er] > "u") throw new Error("Something wrong with primary key index on table");
                this.uniqs[kn.hh][er] = void 0;
              }
              bn.uk && bn.uk.length && bn.uk.forEach(function(jn) {
                var ur = jn.onrightfn(an);
                if (typeof bn.uniqs[jn.hh][ur] > "u") throw new Error("Something wrong with unique index on table");
                bn.uniqs[jn.hh][ur] = void 0;
              });
            }
          }, Dt.deleteall = function() {
            this.data.length = 0, this.pk && (this.uniqs[this.pk.hh] = {}), Dt.uk && Dt.uk.length && Dt.uk.forEach(function(gn) {
              Dt.uniqs[gn.hh] = {};
            });
          }, Dt.update = function(gn, bn, an) {
            var Zt = be(this.data[bn]), Mn;
            if (this.pk && (Mn = this.pk, Mn.pkaddr = Mn.onrightfn(Zt, an), typeof this.uniqs[Mn.hh][Mn.pkaddr] > "u")) throw new Error("Something wrong with index on table");
            Dt.uk && Dt.uk.length && Dt.uk.forEach(function(jn) {
              if (jn.ukaddr = jn.onrightfn(Zt), typeof Dt.uniqs[jn.hh][jn.ukaddr] > "u") throw new Error("Something wrong with unique index on table");
            }), gn(Zt, an, H);
            var Ln = !1;
            for (var In in Dt.beforeupdate) {
              var kn = Dt.beforeupdate[In];
              kn && (kn.funcid ? H.fn[kn.funcid](this.data[bn], Zt) === !1 && (Ln = Ln || !0) : kn.statement && kn.statement.execute(Ie) === !1 && (Ln = Ln || !0));
            }
            if (Ln) return !1;
            var er = !1;
            for (var In in Dt.insteadofupdate) {
              er = !0;
              var kn = Dt.insteadofupdate[In];
              kn && (kn.funcid ? H.fn[kn.funcid](this.data[bn], Zt) : kn.statement && kn.statement.execute(Ie));
            }
            if (!er) {
              if (Dt.checks && Dt.checks.length > 0 && Dt.checks.forEach(function(jn) {
                if (!jn.fn(Zt)) throw new Error("Violation of CHECK constraint " + (jn.id || ""));
              }), Dt.columns.forEach(function(jn) {
                if (jn.notnull && typeof Zt[jn.columnid] > "u") throw new Error("Wrong NULL value in NOT NULL column " + jn.columnid);
              }), this.pk && (Mn.newpkaddr = Mn.onrightfn(Zt), typeof this.uniqs[Mn.hh][Mn.newpkaddr] < "u" && Mn.newpkaddr !== Mn.pkaddr)) throw new Error("Record already exists");
              Dt.uk && Dt.uk.length && Dt.uk.forEach(function(jn) {
                if (jn.newukaddr = jn.onrightfn(Zt), typeof Dt.uniqs[jn.hh][jn.newukaddr] < "u" && jn.newukaddr !== jn.ukaddr) throw new Error("Record already exists");
              }), this.pk && (this.uniqs[Mn.hh][Mn.pkaddr] = void 0, this.uniqs[Mn.hh][Mn.newpkaddr] = Zt), Dt.uk && Dt.uk.length && Dt.uk.forEach(function(jn) {
                Dt.uniqs[jn.hh][jn.ukaddr] = void 0, Dt.uniqs[jn.hh][jn.newukaddr] = Zt;
              }), this.data[bn] = Zt;
              for (var In in Dt.afterupdate) {
                var kn = Dt.afterupdate[In];
                kn && (kn.funcid ? H.fn[kn.funcid](this.data[bn], Zt) : kn.statement && kn.statement.execute(Ie));
              }
            }
          };
          var _n;
          return H.options.nocount || (_n = 1), ft && (_n = ft(_n)), _n;
        }, H.fn.Date = Object, H.fn.Date = Date, H.fn.Number = Number, H.fn.String = String, H.fn.Boolean = Boolean, Xt.EXTEND = H.utils.extend, Xt.CHAR = String.fromCharCode.bind(String), Xt.ASCII = function(Ie) {
          return Ie.charCodeAt(0);
        }, Xt.COALESCE = function() {
          for (var Ie = 0; Ie < arguments.length; Ie++) if (arguments[Ie] !== null && !(typeof arguments[Ie] > "u") && !(typeof arguments[Ie] == "number" && isNaN(arguments[Ie]))) return arguments[Ie];
        }, Xt.USER = function() {
          return "alasql";
        }, Xt.OBJECT_ID = function(Ie) {
          return !!H.tables[Ie];
        }, Xt.DATE = function(Ie) {
          return !isNaN(Ie) && Ie.length === 8 ? new Date(+Ie.substr(0, 4), +Ie.substr(4, 2) - 1, +Ie.substr(6, 2)) : Vt(Ie);
        }, Xt.NOW = function() {
          if (H.options.dateAsString) {
            var Ie = /* @__PURE__ */ new Date(), rt = Ie.getFullYear() + "-" + ("0" + (Ie.getMonth() + 1)).substr(-2) + "-" + ("0" + Ie.getDate()).substr(-2);
            return rt += " " + ("0" + Ie.getHours()).substr(-2) + ":" + ("0" + Ie.getMinutes()).substr(-2) + ":" + ("0" + Ie.getSeconds()).substr(-2), rt += "." + ("00" + Ie.getMilliseconds()).substr(-3), rt;
          }
          return /* @__PURE__ */ new Date();
        }, Xt.GETDATE = Xt.NOW, Xt.CURRENT_TIMESTAMP = Xt.NOW, Xt.CURDATE = Xt.CURRENT_DATE = function() {
          var Ie = /* @__PURE__ */ new Date();
          if (Ie.setHours(0, 0, 0, 0), H.options.dateAsString) {
            var rt = Ie.getFullYear() + "-" + ("0" + (Ie.getMonth() + 1)).substr(-2) + "-" + ("0" + Ie.getDate()).substr(-2);
            return rt;
          }
          return Ie;
        }, Xt.SECOND = function(rt) {
          var rt = Vt(rt);
          return rt.getSeconds();
        }, Xt.MINUTE = function(rt) {
          var rt = Vt(rt);
          return rt.getMinutes();
        }, Xt.HOUR = function(rt) {
          var rt = Vt(rt);
          return rt.getHours();
        }, Xt.DAYOFWEEK = Xt.WEEKDAY = function(rt) {
          var rt = Vt(rt);
          return rt.getDay();
        }, Xt.DAY = Xt.DAYOFMONTH = function(rt) {
          var rt = Vt(rt);
          return rt.getDate();
        }, Xt.MONTH = function(rt) {
          var rt = Vt(rt);
          return rt.getMonth() + 1;
        }, Xt.YEAR = function(rt) {
          var rt = Vt(rt);
          return rt.getFullYear();
        };
        var Ct = { year: 1e3 * 3600 * 24 * 365, quarter: 1e3 * 3600 * 24 * 365 / 4, month: 1e3 * 3600 * 24 * 30, week: 1e3 * 3600 * 24 * 7, day: 1e3 * 3600 * 24, dayofyear: 1e3 * 3600 * 24, weekday: 1e3 * 3600 * 24, hour: 1e3 * 3600, minute: 1e3 * 60, second: 1e3, millisecond: 1, microsecond: 1e-3 };
        H.stdfn.DATEDIFF = function(Ie, rt, ft) {
          var pt = Vt(ft).getTime() - Vt(rt).getTime();
          return pt / Ct[Ie.toLowerCase()] | 0;
        }, H.stdfn.DATEADD = function(wt, rt, ft) {
          var pt = Vt(ft), wt = wt.toLowerCase();
          switch (wt) {
            case "year":
              pt.setFullYear(pt.getFullYear() + rt);
              break;
            case "quarter":
              pt.setMonth(pt.getMonth() + rt * 3);
              break;
            case "month":
              pt.setMonth(pt.getMonth() + rt);
              break;
            default:
              pt = new Date(pt.getTime() + rt * Ct[wt]);
              break;
          }
          return pt;
        }, H.stdfn.INTERVAL = function(Ie, rt) {
          return Ie * Ct[rt.toLowerCase()];
        }, H.stdfn.DATE_ADD = H.stdfn.ADDDATE = function(Ie, rt) {
          var ft = Vt(Ie).getTime() + rt;
          return new Date(ft);
        }, H.stdfn.DATE_SUB = H.stdfn.SUBDATE = function(Ie, rt) {
          var ft = Vt(Ie).getTime() - rt;
          return new Date(ft);
        };
        var Kt = /^\d{4}\.\d{2}\.\d{2} \d{2}:\d{2}:\d{2}/;
        function Vt(Ie) {
          return typeof Ie == "string" && Kt.test(Ie) && (Ie = Ie.replace(".", "-").replace(".", "-")), new Date(Ie);
        }
        ze.DropTable = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.DropTable.prototype.toString = function() {
          var Ie = "DROP ";
          return this.view ? Ie += "VIEW" : Ie += "TABLE", this.ifexists && (Ie += " IF EXISTS"), Ie += " " + this.tables.toString(), Ie;
        }, ze.DropTable.prototype.execute = function(Ie, rt, ft) {
          var pt = this.ifexists, wt = 0, Bt = 0, At = this.tables.length;
          return this.tables.forEach(function(Dt) {
            var Ot = H.databases[Dt.databaseid || Ie], un = Dt.tableid;
            if (!pt || pt && Ot.tables[un]) {
              if (Ot.tables[un]) Ot.engineid ? H.engines[Ot.engineid].dropTable(Dt.databaseid || Ie, un, pt, function(yn) {
                delete Ot.tables[un], wt += yn, Bt++, Bt == At && ft && ft(wt);
              }) : (delete Ot.tables[un], wt++, Bt++, Bt == At && ft && ft(wt));
              else if (!H.options.dropifnotexists) throw new Error(`Can not drop table ${JSON.stringify(Dt.tableid)} because it does not exist in the database.`);
            } else Bt++, Bt == At && ft && ft(wt);
          }), wt;
        }, ze.TruncateTable = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.TruncateTable.prototype.toString = function() {
          var Ie = "TRUNCATE TABLE";
          return Ie += " " + this.table.toString(), Ie;
        }, ze.TruncateTable.prototype.execute = function(Ie, rt, ft) {
          var pt = H.databases[this.table.databaseid || Ie], wt = this.table.tableid;
          if (pt.engineid) return H.engines[pt.engineid].truncateTable(this.table.databaseid || Ie, wt, this.ifexists, ft);
          if (pt.tables[wt]) pt.tables[wt].data = [];
          else throw new Error("Cannot truncate table becaues it does not exist");
          return ft ? ft(0) : 0;
        }, ze.CreateVertex = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CreateVertex.prototype.toString = function() {
          var Ie = "CREATE VERTEX ";
          return this.class && (Ie += this.class + " "), this.sharp && (Ie += "#" + this.sharp + " "), this.sets ? Ie += this.sets.toString() : this.content ? Ie += this.content.toString() : this.select && (Ie += this.select.toString()), Ie;
        }, ze.CreateVertex.prototype.toJS = function(Ie) {
          var rt = "this.queriesfn[" + (this.queriesidx - 1) + "](this.params,null," + Ie + ")";
          return rt;
        }, ze.CreateVertex.prototype.compile = function(Ie) {
          var rt = Ie, ft = this.sharp;
          if (typeof this.name < "u") var wt = "x.name=" + this.name.toJS(), pt = new Function("x", wt);
          if (this.sets && this.sets.length > 0) var wt = this.sets.map(function(Dt) {
            return `x[${JSON.stringify(Dt.column.columnid)}]=` + Dt.expression.toJS("x", "");
          }).join(";"), Bt = new Function("x,params,alasql", wt);
          var At = function(Dt, Ot) {
            var un, yn = H.databases[rt], _n;
            typeof ft < "u" ? _n = ft : _n = yn.counter++;
            var gn = { $id: _n, $node: "VERTEX" };
            return yn.objects[gn.$id] = gn, un = gn, pt && pt(gn), Bt && Bt(gn, Dt, H), Ot && (un = Ot(un)), un;
          };
          return At;
        }, ze.CreateEdge = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CreateEdge.prototype.toString = function() {
          var Ie = "CREATE EDGE ";
          return this.class && (Ie += this.class + " "), Ie;
        }, ze.CreateEdge.prototype.toJS = function(Ie) {
          var rt = "this.queriesfn[" + (this.queriesidx - 1) + "](this.params,null," + Ie + ")";
          return rt;
        }, ze.CreateEdge.prototype.compile = function(Ie) {
          var rt = Ie, ft = new Function("params,alasql", "var y;return " + this.from.toJS()), pt = new Function("params,alasql", "var y;return " + this.to.toJS());
          if (typeof this.name < "u") var Bt = "x.name=" + this.name.toJS(), wt = new Function("x", Bt);
          if (this.sets && this.sets.length > 0) var Bt = this.sets.map(function(Dt) {
            return `x[${JSON.stringify(Dt.column.columnid)}]=` + Dt.expression.toJS("x", "");
          }).join(";"), At = new Function("x,params,alasql", "var y;" + Bt);
          return (Dt, Ot) => {
            let un = 0, yn = H.databases[rt], _n = { $id: yn.counter++, $node: "EDGE" }, gn = ft(Dt, H), bn = pt(Dt, H);
            return _n.$in = [gn.$id], _n.$out = [bn.$id], gn.$out = gn.$out || [], gn.$out.push(_n.$id), bn.$in = bn.$in || [], bn.$in.push(_n.$id), yn.objects[_n.$id] = _n, un = _n, wt == null || wt(_n), At == null || At(_n, Dt, H), Ot ? Ot(un) : un;
          };
        }, ze.CreateGraph = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CreateGraph.prototype.toString = function() {
          var Ie = "CREATE GRAPH ";
          return this.class && (Ie += this.class + " "), Ie;
        }, ze.CreateGraph.prototype.execute = function(Ie, rt, ft) {
          var pt = [];
          return this.from && H.from[this.from.funcid] && (this.graph = H.from[this.from.funcid.toUpperCase()]), this.graph.forEach((At) => {
            if (!At.source) Bt(At);
            else {
              let Dt = {};
              At.as !== void 0 && (H.vars[At.as] = Dt), At.prop !== void 0 && (Dt.name = At.prop), At.sharp !== void 0 && (Dt.$id = At.sharp), At.name !== void 0 && (Dt.name = At.name), At.class !== void 0 && (Dt.$class = At.class);
              let Ot = H.databases[Ie];
              Dt.$id = Dt.$id !== void 0 ? Dt.$id : Ot.counter++, Dt.$node = "EDGE", At.json !== void 0 && Object.assign(Dt, new Function("params, alasql", `return ${At.json.toJS()}`)(rt, H));
              const un = (gn, bn) => {
                let an, Zt;
                if (gn.vars) Zt = H.vars[gn.vars], an = typeof Zt == "object" ? Zt : Ot.objects[Zt];
                else {
                  let Mn = gn.sharp || gn.prop;
                  an = Ot.objects[Mn], an === void 0 && H.options.autovertex && (gn.prop || gn.name) && (an = wt(gn.prop || gn.name) || Bt(gn));
                }
                return bn && an && typeof an.$out > "u" && (an.$out = []), !bn && an && typeof an.$in > "u" && (an.$in = []), an;
              };
              let yn = un(At.source, !0), _n = un(At.target, !1);
              if (Dt.$in = [yn.$id], Dt.$out = [_n.$id], yn.$out.push(Dt.$id), _n.$in.push(Dt.$id), Ot.objects[Dt.$id] = Dt, Dt.$class !== void 0) {
                let gn = H.databases[Ie].tables[Dt.$class];
                if (gn === void 0) throw new Error("No such class. Please use CREATE CLASS");
                gn.data.push(Dt);
              }
              pt.push(Dt.$id);
            }
          }), ft && (pt = ft(pt)), pt;
          function wt(At) {
            var Dt = H.databases[H.useid].objects;
            for (var Ot in Dt) if (Dt[Ot].name === At) return Dt[Ot];
          }
          function Bt(At) {
            var Dt = {};
            typeof At.as < "u" && (H.vars[At.as] = Dt), typeof At.prop < "u" && (Dt.$id = At.prop, Dt.name = At.prop), typeof At.sharp < "u" && (Dt.$id = At.sharp), typeof At.name < "u" && (Dt.name = At.name), typeof At.class < "u" && (Dt.$class = At.class);
            var Ot = H.databases[Ie];
            if (typeof Dt.$id > "u" && (Dt.$id = Ot.counter++), Dt.$node = "VERTEX", typeof At.json < "u" && we(Dt, new Function("params,alasql", "var y;return " + At.json.toJS())(rt, H)), Ot.objects[Dt.$id] = Dt, typeof Dt.$class < "u") {
              if (typeof H.databases[Ie].tables[Dt.$class] > "u") throw new Error("No such class. Pleace use CREATE CLASS");
              H.databases[Ie].tables[Dt.$class].data.push(Dt);
            }
            return pt.push(Dt.$id), Dt;
          }
        }, ze.CreateGraph.prototype.compile1 = function(Ie) {
          const rt = Ie, ft = new Function("params, alasql", `return ${this.from.toJS()}`), pt = new Function("params, alasql", `return ${this.to.toJS()}`);
          let wt, Bt;
          if (this.name !== void 0) {
            const At = `x.name = ${this.name.toJS()}`;
            wt = new Function("x", At);
          }
          if (this.sets && this.sets.length > 0) {
            const At = this.sets.map((Dt) => `x[${JSON.stringify(Dt.column.columnid)}] = ${Dt.expression.toJS("x", "")}`).join(";");
            Bt = new Function("x, params, alasql", `var y; ${At}`);
          }
          return (At, Dt) => {
            let Ot = 0;
            const un = H.databases[rt], yn = { $id: un.counter++, $node: "EDGE" }, _n = ft(At, H), gn = pt(At, H);
            return yn.$in = [_n.$id], yn.$out = [gn.$id], _n.$out = _n.$out || [], _n.$out.push(yn.$id), gn.$in = gn.$in || [], gn.$in.push(yn.$id), un.objects[yn.$id] = yn, Ot = yn, wt && wt(yn), Bt && Bt(yn, At, H), Dt && (Ot = Dt(Ot)), Ot;
          };
        }, ze.AlterTable = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.AlterTable.prototype.toString = function() {
          let Ie = "ALTER TABLE " + this.table.toString();
          return this.renameto && (Ie += " RENAME TO " + this.renameto), Ie;
        }, ze.AlterTable.prototype.execute = function(Ie, rt, ft) {
          let pt = H.databases[Ie];
          if (pt.dbversion = Date.now(), this.renameto) {
            var wt = this.table.tableid, Bt = this.renameto, At = 1;
            if (pt.tables[Bt]) throw new Error(`Can not rename a table "${wt}" to "${Bt}" because the table with this name already exists`);
            if (Bt === wt) throw new Error(`Can not rename a table "${wt}" to itself`);
            return pt.tables[Bt] = pt.tables[wt], delete pt.tables[wt], At = 1, ft && ft(At), At;
          }
          if (this.addcolumn) {
            pt = H.databases[this.table.databaseid || Ie], pt.dbversion++;
            var Dt = this.table.tableid, Ot = pt.tables[Dt], un = this.addcolumn.columnid;
            if (Ot.xcolumns[un]) throw new Error(`Cannot add column "${un}" because it already exists in table "${Dt}"`);
            var yn = { columnid: un, dbtypeid: this.addcolumn.dbtypeid, dbsize: this.dbsize, dbprecision: this.dbprecision, dbenum: this.dbenum, defaultfns: null }, _n = function() {
            };
            Ot.columns.push(yn), Ot.xcolumns[un] = yn;
            for (let Ln = 0, In = Ot.data.length; Ln < In; Ln++) Ot.data[Ln][un] = _n();
            return ft ? ft(1) : 1;
          }
          if (this.modifycolumn) {
            let Ln = H.databases[this.table.databaseid || Ie];
            Ln.dbversion++;
            var Dt = this.table.tableid, Ot = Ln.tables[Dt], un = this.modifycolumn.columnid;
            if (!Ot.xcolumns[un]) throw new Error(`Cannot modify column "${un}" because it was not found in table "${Dt}"`);
            return yn = Ot.xcolumns[un], yn.dbtypeid = this.dbtypeid, yn.dbsize = this.dbsize, yn.dbprecision = this.dbprecision, yn.dbenum = this.dbenum, ft ? ft(1) : 1;
          }
          if (this.renamecolumn) {
            let Ln = H.databases[this.table.databaseid || Ie];
            Ln.dbversion++;
            var Dt = this.table.tableid, Ot = Ln.tables[Dt], un = this.renamecolumn, gn = this.to, yn;
            if (!Ot.xcolumns[un]) throw new Error('Column "' + un + '" is not found in the table "' + Dt + '"');
            if (Ot.xcolumns[gn]) throw new Error('Column "' + gn + '" already exists in the table "' + Dt + '"');
            if (un != gn) {
              for (var bn = 0; bn < Ot.columns.length; bn++) Ot.columns[bn].columnid == un && (Ot.columns[bn].columnid = gn);
              Ot.xcolumns[gn] = Ot.xcolumns[un], delete Ot.xcolumns[un];
              for (var an = 0, Zt = Ot.data.length; an < Zt; an++) Ot.data[an][gn] = Ot.data[an][un], delete Ot.data[an][un];
              return Ot.data.length;
            }
            return ft ? ft(0) : 0;
          }
          if (this.dropcolumn) {
            let Ln = H.databases[this.table.databaseid || Ie];
            Ln.dbversion++;
            for (var Dt = this.table.tableid, Ot = Ln.tables[Dt], un = this.dropcolumn, Mn = !1, bn = 0; bn < Ot.columns.length; bn++) if (Ot.columns[bn].columnid == un) {
              Mn = !0, Ot.columns.splice(bn, 1);
              break;
            }
            if (!Mn) throw new Error(`Cannot drop column "${un}" because it was not found in the table ${Dt}"`);
            for (delete Ot.xcolumns[un], an = 0, Zt = Ot.data.length; an < Zt; an++) delete Ot.data[an][un];
            return ft ? ft(Ot.data.length) : Ot.data.length;
          }
          throw Error("Unknown ALTER TABLE method");
        }, ze.CreateIndex = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CreateIndex.prototype.toString = function() {
          var Ie = "CREATE";
          return this.unique && (Ie += " UNIQUE"), Ie += " INDEX " + this.indexid + " ON " + this.table.toString(), Ie += "(" + this.columns.toString() + ")", Ie;
        }, ze.CreateIndex.prototype.execute = function(Ie, rt, ft) {
          var pt = H.databases[Ie], wt = this.table.tableid, Bt = pt.tables[wt], At = this.indexid;
          pt.indices[At] = wt;
          var Dt = this.columns.map(function(Mn) {
            return Mn.expression.toJS("r", "");
          }).join("+'`'+"), Ot = new Function("r,params,alasql", "return " + Dt);
          if (this.unique) {
            Bt.uniqdefs[At] = { rightfns: Dt };
            var un = Bt.uniqs[At] = {};
            if (Bt.data.length > 0) for (var yn = 0, _n = Bt.data.length; yn < _n; yn++) {
              var gn = Dt(Bt.data[yn]);
              un[gn] || (un[gn] = { num: 0 }), un[gn].num++;
            }
          } else {
            var bn = fe(Dt);
            Bt.inddefs[At] = { rightfns: Dt, hh: bn }, Bt.indices[bn] = {};
            var an = Bt.indices[bn] = {};
            if (Bt.data.length > 0) for (var yn = 0, _n = Bt.data.length; yn < _n; yn++) {
              var gn = Ot(Bt.data[yn], rt, H);
              an[gn] || (an[gn] = []), an[gn].push(Bt.data[yn]);
            }
          }
          var Zt = 1;
          return ft && (Zt = ft(Zt)), Zt;
        }, ze.Reindex = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Reindex.prototype.toString = function() {
          var Ie = "REINDEX " + this.indexid;
          return Ie;
        }, ze.Reindex.prototype.execute = function(Ie, rt, ft) {
          var pt = H.databases[Ie], wt = this.indexid, Bt = pt.indices[wt], At = pt.tables[Bt];
          At.indexColumns();
          var Dt = 1;
          return ft && (Dt = ft(Dt)), Dt;
        }, ze.DropIndex = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.DropIndex.prototype.toString = function() {
          return "DROP INDEX" + this.indexid;
        }, ze.DropIndex.prototype.compile = function(Ie) {
          return this.indexid, function() {
            return 1;
          };
        }, ze.WithSelect = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.WithSelect.prototype.toString = function() {
          var Ie = "WITH ";
          return Ie += this.withs.map(function(rt) {
            return rt.name + " AS (" + rt.select.toString() + ")";
          }).join(",") + " ", Ie += this.select.toString(), Ie;
        }, ze.WithSelect.prototype.execute = function(Ie, rt, ft) {
          var pt = this, wt = [];
          pt.withs.forEach(function(At) {
            wt.push(H.databases[Ie].tables[At.name]);
            var Dt = H.databases[Ie].tables[At.name] = new qe({ tableid: At.name });
            Dt.data = At.select.execute(Ie, rt);
          });
          var Bt = 1;
          return Bt = this.select.execute(Ie, rt, function(At) {
            return pt.withs.forEach(function(Dt, Ot) {
              wt[Ot] ? H.databases[Ie].tables[Dt.name] = wt[Ot] : delete H.databases[Ie].tables[Dt.name];
            }), ft && (At = ft(At)), At;
          }), Bt;
        }, ze.If = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.If.prototype.toString = function() {
          var Ie = "IF ";
          return Ie += this.expression.toString(), Ie += " " + this.thenstat.toString(), this.elsestat && (Ie += " ELSE " + this.thenstat.toString()), Ie;
        }, ze.If.prototype.execute = function(Ie, rt, ft) {
          var pt, wt = new Function("params,alasql,p", "var y;return " + this.expression.toJS("({})", "", null)).bind(this);
          return wt(rt, H) ? pt = this.thenstat.execute(Ie, rt, ft) : this.elsestat ? pt = this.elsestat.execute(Ie, rt, ft) : ft && (pt = ft(pt)), pt;
        }, ze.While = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.While.prototype.toString = function() {
          var Ie = "WHILE ";
          return Ie += this.expression.toString(), Ie += " " + this.loopstat.toString(), Ie;
        }, ze.While.prototype.execute = function(Ie, rt, ft) {
          var pt = this, wt = [], Bt = new Function("params,alasql,p", "var y;return " + this.expression.toJS());
          if (ft) {
            var At = !1, Dt = function(un) {
              At ? wt.push(un) : At = !0, setTimeout(function() {
                Bt(rt, H) ? pt.loopstat.execute(Ie, rt, Dt) : wt = ft(wt);
              }, 0);
            };
            Dt();
          } else for (; Bt(rt, H); ) {
            var Ot = pt.loopstat.execute(Ie, rt);
            wt.push(Ot);
          }
          return wt;
        }, ze.Break = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Break.prototype.toString = function() {
          var Ie = "BREAK";
          return Ie;
        }, ze.Break.prototype.execute = function(Ie, rt, ft, pt) {
          var wt = 1;
          return ft && (wt = ft(wt)), wt;
        }, ze.Continue = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Continue.prototype.toString = function() {
          var Ie = "CONTINUE";
          return Ie;
        }, ze.Continue.prototype.execute = function(Ie, rt, ft, pt) {
          var wt = 1;
          return ft && (wt = ft(wt)), wt;
        }, ze.BeginEnd = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.BeginEnd.prototype.toString = function() {
          var Ie = "BEGIN " + this.statements.toString() + " END";
          return Ie;
        }, ze.BeginEnd.prototype.execute = function(Ie, rt, ft, pt) {
          var wt = this, Bt = [], At = 0;
          Dt();
          function Dt() {
            wt.statements[At].execute(Ie, rt, function(Ot) {
              if (Bt.push(Ot), At++, At < wt.statements.length) return Dt();
              ft && (Bt = ft(Bt));
            });
          }
          return Bt;
        }, ze.Insert = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Insert.prototype.toString = function() {
          var Ie = "INSERT ";
          if (this.orreplace && (Ie += "OR REPLACE "), this.replaceonly && (Ie = "REPLACE "), Ie += "INTO " + this.into.toString(), this.columns && (Ie += "(" + this.columns.toString() + ")"), this.values) {
            var rt = this.values.map(function(ft) {
              return "(" + ft.toString() + ")";
            });
            Ie += " VALUES " + rt.join(",");
          }
          return this.select && (Ie += " " + this.select.toString()), Ie;
        }, ze.Insert.prototype.toJS = function(Ie, rt, ft) {
          var pt = "this.queriesfn[" + (this.queriesidx - 1) + "](this.params,null," + Ie + ")";
          return pt;
        }, ze.Insert.prototype.compile = function(Ie) {
          var rt = this;
          Ie = rt.into.databaseid || Ie;
          var ft = H.databases[Ie], pt = rt.into.tableid, wt = ft.tables[pt];
          if (!wt) throw "Table '" + pt + "' could not be found";
          var At = "", Bt = "", At = "db.tables['" + pt + "'].dirty=true;", Dt = "var a,aa=[],x;", Ot;
          if (this.values) {
            this.exists && (this.existsfn = this.exists.map(function(Mn) {
              var Ln = Mn.compile(Ie);
              return Ln.query.modifier = "RECORDSET", Ln;
            })), this.queries && (this.queriesfn = this.queries.map(function(Mn) {
              var Ln = Mn.compile(Ie);
              return Ln.query.modifier = "RECORDSET", Ln;
            })), rt.values.forEach(function(Mn) {
              var Ln = [];
              rt.columns ? rt.columns.forEach(function(In, kn) {
                var er = "'" + In.columnid + "':";
                wt.xcolumns && wt.xcolumns[In.columnid] ? ["INT", "FLOAT", "NUMBER", "MONEY"].indexOf(wt.xcolumns[In.columnid].dbtypeid) >= 0 ? er += "(x=" + Mn[kn].toJS() + ",x==undefined?undefined:+x)" : H.fn[wt.xcolumns[In.columnid].dbtypeid] ? (er += "(new " + wt.xcolumns[In.columnid].dbtypeid + "(", er += Mn[kn].toJS(), er += "))") : er += Mn[kn].toJS() : er += Mn[kn].toJS(), Ln.push(er);
              }) : Array.isArray(Mn) && wt.columns && wt.columns.length > 0 ? wt.columns.forEach(function(In, kn) {
                var er = "'" + In.columnid + "':";
                ["INT", "FLOAT", "NUMBER", "MONEY"].indexOf(In.dbtypeid) >= 0 ? er += "+" + Mn[kn].toJS() : H.fn[In.dbtypeid] ? (er += "(new " + In.dbtypeid + "(", er += Mn[kn].toJS(), er += "))") : er += Mn[kn].toJS(), Ln.push(er);
              }) : Bt = Tt(Mn), ft.tables[pt].defaultfns && Ln.unshift(ft.tables[pt].defaultfns), Bt ? At += "a=" + Bt + ";" : At += "a={" + Ln.join(",") + "};", ft.tables[pt].isclass && (At += "var db=alasql.databases['" + Ie + "'];", At += 'a.$class="' + pt + '";', At += "a.$id=db.counter++;", At += "db.objects[a.$id]=a;"), ft.tables[pt].insert ? (At += "var db=alasql.databases['" + Ie + "'];", At += "db.tables['" + pt + "'].insert(a," + (rt.orreplace ? "true" : "false") + ");") : At += "aa.push(a);";
            }), Ot = Dt + At, ft.tables[pt].insert || (At += "alasql.databases['" + Ie + "'].tables['" + pt + "'].data=alasql.databases['" + Ie + "'].tables['" + pt + "'].data.concat(aa);"), ft.tables[pt].insert && ft.tables[pt].isclass ? At += "return a.$id;" : At += "return " + rt.values.length;
            var bn = new Function("db, params, alasql", "var y;" + Dt + At).bind(this);
          } else if (this.select) {
            this.select.modifier = "RECORDSET", this.queries && (this.select.queries = this.queries);
            var un = this.select.compile(Ie);
            if (ft.engineid && H.engines[ft.engineid].intoTable) {
              var an = function(Ln, In) {
                var kn = un(Ln), er = H.engines[ft.engineid].intoTable(ft.databaseid, pt, kn.data, null, In);
                return er;
              };
              return an;
            } else var yn = "return alasql.utils.extend(r,{" + wt.defaultfns + "})", _n = new Function("r,db,params,alasql", yn), bn = function(Mn, Ln, In) {
              var kn = un(Ln).data;
              if (Mn.tables[pt].insert) for (var er = 0, jn = kn.length; er < jn; er++) {
                var ur = be(kn[er]);
                _n(ur, Mn, Ln, In), Mn.tables[pt].insert(ur, rt.orreplace);
              }
              else Mn.tables[pt].data = Mn.tables[pt].data.concat(kn);
              if (!In.options.nocount) return kn.length;
            };
          } else if (this.default) var gn = "db.tables['" + pt + "'].data.push({" + wt.defaultfns + "});return 1;", bn = new Function("db,params,alasql", gn);
          else throw new Error("Wrong INSERT parameters");
          if (ft.engineid && H.engines[ft.engineid].intoTable && H.options.autocommit) var an = function(Zt, Mn) {
            var Ln = new Function("db,params", "var y;" + Ot + "return aa;")(ft, Zt), In = H.engines[ft.engineid].intoTable(ft.databaseid, pt, Ln, null, Mn);
            return In;
          };
          else var an = function(Zt, Mn) {
            var Ln = H.databases[Ie];
            H.options.autocommit && Ln.engineid && H.engines[Ln.engineid].loadTableData(Ie, pt);
            var In = bn(Ln, Zt, H);
            return H.options.autocommit && Ln.engineid && H.engines[Ln.engineid].saveTableData(Ie, pt), H.options.nocount && (In = void 0), Mn && Mn(In), In;
          };
          return an;
        }, ze.Insert.prototype.execute = function(Ie, rt, ft) {
          return this.compile(Ie)(rt, ft);
        }, ze.CreateTrigger = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CreateTrigger.prototype.toString = function() {
          var Ie = "CREATE TRIGGER " + this.trigger + " ";
          return this.when && (Ie += this.when + " "), Ie += this.action + " ON ", this.table.databaseid && (Ie += this.table.databaseid + "."), Ie += this.table.tableid + " ", Ie += this.statement.toString(), Ie;
        };
        const Yt = ["beforeinsert", "afterinsert", "insteadofinsert", "beforedelete", "afterdelete", "insteadofdelete", "beforeupdate", "afterupdate", "insteadofupdate"];
        ze.CreateTrigger.prototype.execute = function(Ie, rt, ft) {
          let pt = 1;
          const wt = this.trigger;
          Ie = this.table.databaseid || Ie;
          const Bt = H.databases[Ie], { tableid: At } = this.table, Dt = { action: this.action, when: this.when, statement: this.statement, funcid: this.funcid, tableid: At };
          Bt.triggers[wt] = Dt;
          const Ot = `${this.when}${this.action}`.toLowerCase();
          return Yt.includes(Ot) && (Bt.tables[At] = Bt.tables[At] || {}, Bt.tables[At][Ot] = Bt.tables[At][Ot] || {}, Bt.tables[At][Ot][wt] = Dt), ft && (pt = ft(pt)), pt;
        }, ze.DropTrigger = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.DropTrigger.prototype.toString = function() {
          var Ie = "DROP TRIGGER " + this.trigger;
          return Ie;
        }, ze.DropTrigger.prototype.execute = function(Ie, rt, ft) {
          let pt = 0;
          const wt = H.databases[Ie], Bt = this.trigger, At = wt.triggers[Bt];
          if (At) {
            const { tableid: Dt } = At;
            if (Dt) pt = 1, Yt.forEach((Ot) => {
              delete wt.tables[Dt][Ot][Bt];
            }), delete wt.triggers[Bt];
            else throw new Error("Trigger Table not found");
          } else throw new Error("Trigger not found");
          return ft && (pt = ft(pt)), pt;
        }, ze.Delete = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Delete.prototype.toString = function() {
          var Ie = "DELETE FROM " + this.table.toString();
          return this.where && (Ie += " WHERE " + this.where.toString()), Ie;
        }, ze.Delete.prototype.compile = function(Ie) {
          Ie = this.table.databaseid || Ie;
          var rt = this.table.tableid, ft, pt = H.databases[Ie];
          if (this.where) {
            this.exists && (this.existsfn = this.exists.map(function(Bt) {
              var At = Bt.compile(Ie);
              return At.query.modifier = "RECORDSET", At;
            })), this.queries && (this.queriesfn = this.queries.map(function(Bt) {
              var At = Bt.compile(Ie);
              return At.query.modifier = "RECORDSET", At;
            }));
            var wt = new Function("r,params,alasql", "var y;return (" + this.where.toJS("r", "") + ")").bind(this);
            ft = function(Bt, At) {
              if (pt.engineid && H.engines[pt.engineid].deleteFromTable) return H.engines[pt.engineid].deleteFromTable(Ie, rt, wt, Bt, At);
              H.options.autocommit && pt.engineid && (pt.engineid == "LOCALSTORAGE" || pt.engineid == "FILESTORAGE") && H.engines[pt.engineid].loadTableData(Ie, rt);
              for (var Dt = pt.tables[rt], Ot = Dt.data.length, un = [], yn = 0, _n = Dt.data.length; yn < _n; yn++) wt(Dt.data[yn], Bt, H) ? Dt.delete && Dt.delete(yn, Bt, H) : un.push(Dt.data[yn]);
              Dt.data = un;
              for (var gn in Dt.afterdelete) {
                var bn = Dt.afterdelete[gn];
                bn && (bn.funcid ? H.fn[bn.funcid]() : bn.statement && bn.statement.execute(Ie));
              }
              var an = Ot - Dt.data.length;
              return H.options.autocommit && pt.engineid && (pt.engineid == "LOCALSTORAGE" || pt.engineid == "FILESTORAGE") && H.engines[pt.engineid].saveTableData(Ie, rt), At && (an = At(an)), an;
            };
          } else ft = function(Bt, At) {
            H.options.autocommit && pt.engineid && H.engines[pt.engineid].loadTableData(Ie, rt);
            var Dt = pt.tables[rt];
            Dt.dirty = !0;
            var Ot = pt.tables[rt].data.length;
            pt.tables[rt].data.length = 0;
            for (var un in pt.tables[rt].uniqs) pt.tables[rt].uniqs[un] = {};
            for (var un in pt.tables[rt].indices) pt.tables[rt].indices[un] = {};
            return H.options.autocommit && pt.engineid && H.engines[pt.engineid].saveTableData(Ie, rt), At && At(Ot), Ot;
          };
          return ft;
        }, ze.Delete.prototype.execute = function(Ie, rt, ft) {
          return this.compile(Ie)(rt, ft);
        }, ze.Update = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Update.prototype.toString = function() {
          var Ie = "UPDATE " + this.table.toString();
          return this.columns && (Ie += " SET " + this.columns.toString()), this.where && (Ie += " WHERE " + this.where.toString()), Ie;
        }, ze.SetColumn = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.SetColumn.prototype.toString = function() {
          return this.column.toString() + "=" + this.expression.toString();
        }, ze.Update.prototype.compile = function(Ie) {
          Ie = this.table.databaseid || Ie;
          var rt = this.table.tableid;
          if (this.where) {
            this.exists && (this.existsfn = this.exists.map(function(At) {
              var Dt = At.compile(Ie);
              return Dt.query.modifier = "RECORDSET", Dt;
            })), this.queries && (this.queriesfn = this.queries.map(function(At) {
              var Dt = At.compile(Ie);
              return Dt.query.modifier = "RECORDSET", Dt;
            }));
            var ft = new Function("r,params,alasql", "var y;return " + this.where.toJS("r", "")).bind(this);
          }
          var pt = H.databases[Ie].tables[rt].onupdatefns || "";
          pt += ";", this.columns.forEach(function(At) {
            pt += "r['" + At.column.columnid + "']=" + At.expression.toJS("r", "") + ";";
          });
          var wt = new Function("r,params,alasql", "var y;" + pt), Bt = function(At, Dt) {
            var Ot = H.databases[Ie];
            if (Ot.engineid && H.engines[Ot.engineid].updateTable) return H.engines[Ot.engineid].updateTable(Ie, rt, wt, ft, At, Dt);
            H.options.autocommit && Ot.engineid && H.engines[Ot.engineid].loadTableData(Ie, rt);
            var un = Ot.tables[rt];
            if (!un) throw new Error("Table '" + rt + "' not exists");
            for (var yn = 0, _n = 0, gn = un.data.length; _n < gn; _n++) (!ft || ft(un.data[_n], At, H)) && (un.update ? un.update(wt, _n, At) : wt(un.data[_n], At, H), yn++);
            return H.options.autocommit && Ot.engineid && H.engines[Ot.engineid].saveTableData(Ie, rt), Dt && Dt(yn), yn;
          };
          return Bt;
        }, ze.Update.prototype.execute = function(Ie, rt, ft) {
          return this.compile(Ie)(rt, ft);
        }, ze.Merge = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Merge.prototype.toString = function() {
          let Ie = `MERGE ${this.into.tableid} `;
          return this.into.as && (Ie += `AS ${this.into.as} `), Ie += `USING ${this.using.tableid} `, this.using.as && (Ie += `AS ${this.using.as} `), Ie += `ON ${this.on.toString()} `, this.matches.forEach((rt) => {
            Ie += "WHEN ", rt.matched || (Ie += "NOT "), Ie += "MATCHED ", rt.bytarget && (Ie += "BY TARGET "), rt.bysource && (Ie += "BY SOURCE "), rt.expr && (Ie += `AND ${rt.expr.toString()} `), Ie += "THEN ", rt.action.delete && (Ie += "DELETE "), rt.action.insert && (Ie += "INSERT ", rt.action.columns && (Ie += `(${rt.action.columns.toString()}) `), rt.action.values && (Ie += `VALUES (${rt.action.values.toString()}) `), rt.action.defaultvalues && (Ie += "DEFAULT VALUES ")), rt.action.update && (Ie += "UPDATE ", Ie += rt.action.update.map((ft) => ft.toString()).join(", ") + " ");
          }), Ie;
        }, ze.Merge.prototype.execute = function(Ie, rt, ft) {
          var pt = 1;
          return ft && (pt = ft(pt)), pt;
        }, ze.CreateDatabase = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CreateDatabase.prototype.toString = function() {
          let Ie = "CREATE ";
          return this.engineid && (Ie += `${this.engineid} `), Ie += "DATABASE ", this.ifnotexists && (Ie += "IF NOT EXISTS "), Ie += `${this.databaseid} `, this.args && this.args.length > 0 && (Ie += `(${this.args.map((rt) => rt.toString()).join(", ")}) `), this.as && (Ie += `AS ${this.as}`), Ie;
        }, ze.CreateDatabase.prototype.execute = function(Ie, rt, ft) {
          if (this.args && this.args.length > 0 && this.args.map(function(Bt) {
            return new Function("params,alasql", "var y;return " + Bt.toJS())(rt, H);
          }), this.engineid) {
            var pt = H.engines[this.engineid].createDatabase(this.databaseid, this.args, this.ifnotexists, this.as, ft);
            return pt;
          } else {
            var wt = this.databaseid;
            if (H.databases[wt]) throw new Error("Database '" + wt + "' already exists");
            new H.Database(wt);
            var pt = 1;
            return ft ? ft(pt) : pt;
          }
        }, ze.AttachDatabase = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.AttachDatabase.prototype.toString = function(Ie) {
          let rt = "ATTACH";
          return this.engineid && (rt += ` ${this.engineid}`), rt += ` DATABASE ${this.databaseid}`, Ie && (rt += "(", Ie.length > 0 && (rt += Ie.map((ft) => ft.toString()).join(", ")), rt += ")"), this.as && (rt += ` AS ${this.as}`), rt;
        }, ze.AttachDatabase.prototype.execute = function(Ie, rt, ft) {
          if (!H.engines[this.engineid]) throw new Error('Engine "' + this.engineid + '" is not defined.');
          var pt = H.engines[this.engineid].attachDatabase(this.databaseid, this.as, this.args, rt, ft);
          return pt;
        }, ze.DetachDatabase = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.DetachDatabase.prototype.toString = function() {
          var Ie = "DETACH";
          return Ie += " DATABASE " + this.databaseid, Ie;
        }, ze.DetachDatabase.prototype.execute = function(Ie, rt, ft) {
          if (!H.databases[this.databaseid].engineid) throw new Error('Cannot detach database "' + this.engineid + '", because it was not attached.');
          var pt, wt = this.databaseid;
          if (wt === H.DEFAULTDATABASEID) throw new Error("Drop of default database is prohibited");
          if (H.databases[wt]) {
            var Bt = H.databases[wt].engineid && H.databases[wt].engineid == "FILESTORAGE", At = H.databases[wt].filename || "";
            delete H.databases[wt], Bt && (H.databases[wt] = {}, H.databases[wt].isDetached = !0, H.databases[wt].filename = At), wt === H.useid && H.use(), pt = 1;
          } else if (this.ifexists) pt = 0;
          else throw new Error("Database '" + wt + "' does not exist");
          return ft && ft(pt), pt;
        }, ze.UseDatabase = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.UseDatabase.prototype.toString = function() {
          return "USE DATABASE " + this.databaseid;
        }, ze.UseDatabase.prototype.execute = function(Ie, rt, ft) {
          var pt = this.databaseid;
          if (!H.databases[pt]) throw new Error("Database '" + pt + "' does not exist");
          H.use(pt);
          var wt = 1;
          return ft && ft(wt), wt;
        }, ze.DropDatabase = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.DropDatabase.prototype.toString = function() {
          var Ie = "DROP";
          return this.ifexists && (Ie += " IF EXISTS"), Ie += " DATABASE " + this.databaseid, Ie;
        }, ze.DropDatabase.prototype.execute = function(Ie, rt, ft) {
          if (this.engineid) return H.engines[this.engineid].dropDatabase(this.databaseid, this.ifexists, ft);
          let pt;
          const wt = this.databaseid;
          if (wt === H.DEFAULTDATABASEID) throw new Error("Drop of default database is prohibited");
          if (H.databases[wt]) {
            if (H.databases[wt].engineid) throw new Error(`Cannot drop database '${wt}', because it is attached. Detach it.`);
            delete H.databases[wt], wt === H.useid && H.use(), pt = 1;
          } else if (this.ifexists) pt = 0;
          else throw new Error(`Database '${wt}' does not exist`);
          return ft && ft(pt), pt;
        }, ze.Declare = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Declare.prototype.toString = function() {
          let Ie = "DECLARE ";
          return this.declares && this.declares.length > 0 && (Ie += this.declares.map((rt) => {
            let ft = `@${rt.variable} ${rt.dbtypeid}`;
            return rt.dbsize && (ft += `(${rt.dbsize}`, rt.dbprecision && (ft += `,${rt.dbprecision}`), ft += ")"), rt.expression && (ft += ` = ${rt.expression.toString()}`), ft;
          }).join(",")), Ie;
        }, ze.Declare.prototype.execute = function(Ie, rt, ft) {
          var pt = 1, wt = this;
          return wt.declares && wt.declares.length > 0 && wt.declares.forEach(function(Bt) {
            var At = Bt.dbtypeid;
            H.fn[At] || (At = At.toUpperCase()), H.declares[Bt.variable] = { dbtypeid: At, dbsize: Bt.dbsize, dbprecision: Bt.dbprecision }, Bt.expression && (H.vars[Bt.variable] = new Function("params,alasql", "return " + Bt.expression.toJS("({})", "", null)).bind(wt)(rt, H), H.declares[Bt.variable] && (H.vars[Bt.variable] = H.stdfn.CONVERT(H.vars[Bt.variable], H.declares[Bt.variable])));
          }), ft && (pt = ft(pt)), pt;
        }, ze.ShowDatabases = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.ShowDatabases.prototype.toString = function() {
          var Ie = "SHOW DATABASES";
          return this.like && (Ie += "LIKE " + this.like.toString()), Ie;
        }, ze.ShowDatabases.prototype.execute = function(Ie, rt, ft) {
          if (this.engineid) return H.engines[this.engineid].showDatabases(this.like, ft);
          var pt = this, wt = [];
          for (var Bt in H.databases) wt.push({ databaseid: Bt });
          return pt.like && wt && wt.length > 0 && (wt = wt.filter(function(At) {
            return H.utils.like(pt.like.value, At.databaseid);
          })), ft && ft(wt), wt;
        }, ze.ShowTables = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.ShowTables.prototype.toString = function() {
          var Ie = "SHOW TABLES";
          return this.databaseid && (Ie += " FROM " + this.databaseid), this.like && (Ie += " LIKE " + this.like.toString()), Ie;
        }, ze.ShowTables.prototype.execute = function(Ie, rt, ft) {
          var pt = H.databases[this.databaseid || Ie], wt = this, Bt = [];
          for (var At in pt.tables) Bt.push({ tableid: At });
          return wt.like && Bt && Bt.length > 0 && (Bt = Bt.filter(function(Dt) {
            return H.utils.like(wt.like.value, Dt.tableid);
          })), ft && ft(Bt), Bt;
        }, ze.ShowColumns = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.ShowColumns.prototype.toString = function() {
          var Ie = "SHOW COLUMNS";
          return this.table.tableid && (Ie += " FROM " + this.table.tableid), this.databaseid && (Ie += " FROM " + this.databaseid), Ie;
        }, ze.ShowColumns.prototype.execute = function(Ie, rt, ft) {
          var pt = H.databases[this.databaseid || Ie], wt = pt.tables[this.table.tableid];
          if (wt && wt.columns) {
            var Bt = wt.columns.map(function(At) {
              return { columnid: At.columnid, dbtypeid: At.dbtypeid, dbsize: At.dbsize };
            });
            return ft && ft(Bt), Bt;
          } else return ft && ft([]), [];
        }, ze.ShowIndex = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.ShowIndex.prototype.toString = function() {
          var Ie = "SHOW INDEX";
          return this.table.tableid && (Ie += " FROM " + this.table.tableid), this.databaseid && (Ie += " FROM " + this.databaseid), Ie;
        }, ze.ShowIndex.prototype.execute = function(Ie, rt, ft) {
          var pt = H.databases[this.databaseid || Ie], wt = pt.tables[this.table.tableid], Bt = [];
          if (wt && wt.indices) for (var At in wt.indices) Bt.push({ hh: At, len: Object.keys(wt.indices[At]).length });
          return ft && ft(Bt), Bt;
        }, ze.ShowCreateTable = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.ShowCreateTable.prototype.toString = function() {
          var Ie = "SHOW CREATE TABLE " + this.table.tableid;
          return this.databaseid && (Ie += " FROM " + this.databaseid), Ie;
        }, ze.ShowCreateTable.prototype.execute = function(Ie) {
          var rt = H.databases[this.databaseid || Ie], ft = rt.tables[this.table.tableid];
          if (ft) {
            var pt = "CREATE TABLE " + this.table.tableid + " (", wt = [];
            return ft.columns && (ft.columns.forEach(function(Bt) {
              var At = Bt.columnid + " " + Bt.dbtypeid;
              Bt.dbsize && (At += "(" + Bt.dbsize + ")"), Bt.primarykey && (At += " PRIMARY KEY"), wt.push(At);
            }), pt += wt.join(", ")), pt += ")", pt;
          } else throw new Error('There is no such table "' + this.table.tableid + '"');
        }, ze.SetVariable = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.SetVariable.prototype.toString = function() {
          var Ie = "SET ";
          return typeof this.value < "u" && (Ie += this.variable.toUpperCase() + " " + (this.value ? "ON" : "OFF")), this.expression && (Ie += this.method + this.variable + " = " + this.expression.toString()), Ie;
        }, ze.SetVariable.prototype.execute = function(Ie, rt, ft) {
          if (typeof this.value < "u") {
            let wt = this.value;
            wt === "ON" ? wt = !0 : wt === "OFF" && (wt = !1), H.options[this.variable] = wt;
          } else if (this.expression) {
            this.exists && (this.existsfn = this.exists.map((Bt) => {
              let At = Bt.compile(Ie);
              return At.query && !At.query.modifier && (At.query.modifier = "RECORDSET"), At;
            })), this.queries && (this.queriesfn = this.queries.map((Bt) => {
              let At = Bt.compile(Ie);
              return At.query && !At.query.modifier && (At.query.modifier = "RECORDSET"), At;
            }));
            let wt = new Function("params, alasql", "return " + this.expression.toJS("({})", "", null)).bind(this)(rt, H);
            if (H.declares[this.variable] && (wt = H.stdfn.CONVERT(wt, H.declares[this.variable])), this.props && this.props.length > 0) {
              let Bt;
              this.method === "@" ? Bt = `alasql.vars['${this.variable}']` : Bt = `params['${this.variable}']`, this.props.forEach((At) => {
                typeof At == "string" ? Bt += `['${At}']` : typeof At == "number" ? Bt += `[${At}]` : Bt += `[${At.toJS()}]`;
              }), new Function("value, params, alasql", `${Bt} = value`)(wt, rt, H);
            } else this.method === "@" ? H.vars[this.variable] = wt : rt[this.variable] = wt;
          }
          let pt = 1;
          return ft && (pt = ft(pt)), pt;
        }, H.test = function(Ie, rt, ft) {
          if (arguments.length === 0) {
            H.log(H.con.results);
            return;
          }
          var pt = Date.now();
          if (arguments.length === 1) {
            ft(), H.con.log(Date.now() - pt);
            return;
          }
          arguments.length === 2 && (ft = rt, rt = 1);
          for (var wt = 0; wt < rt; wt++) ft();
          H.con.results[Ie] = Date.now() - pt;
        }, H.log = function(Ie, rt) {
          var ft = H.useid, pt = H.options.logtarget;
          J.isNode && (pt = "console");
          var wt;
          if (typeof Ie == "string" ? wt = H(Ie, rt) : wt = Ie, pt === "console" || J.isNode) typeof Ie == "string" && H.options.logprompt && console.log(ft + ">", Ie), Array.isArray(wt) && console.table ? console.table(wt) : console.log(Ft(wt));
          else {
            var Bt;
            pt === "output" ? Bt = document.getElementsByTagName("output")[0] : typeof pt == "string" ? Bt = document.getElementById(pt) : Bt = pt;
            var At = "";
            if (typeof Ie == "string" && H.options.logprompt && (At += "<pre><code>" + H.pretty(Ie) + "</code></pre>"), Array.isArray(wt)) if (wt.length === 0) At += "<p>[ ]</p>";
            else if (typeof wt[0] != "object" || Array.isArray(wt[0])) for (var Dt = 0, Ot = wt.length; Dt < Ot; Dt++) At += "<p>" + sn(wt[Dt]) + "</p>";
            else At += sn(wt);
            else At += sn(wt);
            Bt.innerHTML += At;
          }
        }, H.clear = function() {
          var Ie = H.options.logtarget;
          if (J.isNode || J.isMeteorServer) console.clear && console.clear();
          else {
            var rt;
            Ie === "output" ? rt = document.getElementsByTagName("output")[0] : typeof Ie == "string" ? rt = document.getElementById(Ie) : rt = Ie, rt.innerHTML = "";
          }
        }, H.write = function(Ie) {
          var rt = H.options.logtarget;
          if (J.isNode || J.isMeteorServer) console.log && console.log(Ie);
          else {
            var ft;
            rt === "output" ? ft = document.getElementsByTagName("output")[0] : typeof rt == "string" ? ft = document.getElementById(rt) : ft = rt, ft.innerHTML += Ie;
          }
        };
        function sn(Ie) {
          var rt = "";
          if (Ie === void 0) rt += "undefined";
          else if (Array.isArray(Ie)) {
            rt += "<style>", rt += "table {border:1px black solid; border-collapse: collapse; border-spacing: 0px;}", rt += "td,th {border:1px black solid; padding-left:5px; padding-right:5px}", rt += "th {background-color: #EEE}", rt += "</style>", rt += "<table>";
            var ft = [];
            for (var pt in Ie[0]) ft.push(pt);
            rt += "<tr><th>#", ft.forEach(function(At) {
              rt += "<th>" + At;
            });
            for (var wt = 0, Bt = Ie.length; wt < Bt; wt++) rt += "<tr><th>" + (wt + 1), ft.forEach(function(At) {
              rt += "<td> ", Ie[wt][At] == +Ie[wt][At] ? (rt += '<div style="text-align:right">', typeof Ie[wt][At] > "u" ? rt += "NULL" : rt += Ie[wt][At], rt += "</div>") : typeof Ie[wt][At] > "u" ? rt += "NULL" : typeof Ie[wt][At] == "string" ? rt += Ie[wt][At] : rt += Ft(Ie[wt][At]);
            });
            rt += "</table>";
          } else rt += "<p>" + Ft(Ie) + "</p>";
          return rt;
        }
        function hn(Ie, rt, ft) {
          if (!(ft <= 0)) {
            var pt = rt - Ie.scrollTop, wt = pt / ft * 10;
            setTimeout(function() {
              Ie.scrollTop !== rt && (Ie.scrollTop = Ie.scrollTop + wt, hn(Ie, rt, ft - 10));
            }, 10);
          }
        }
        H.prompt = function(Ie, rt, ft) {
          if (J.isNode) throw new Error("The prompt not realized for Node.js");
          var pt = 0;
          if (typeof Ie == "string" && (Ie = document.getElementById(Ie)), typeof rt == "string" && (rt = document.getElementById(rt)), rt.textContent = H.useid, ft) {
            H.prompthistory.push(ft), pt = H.prompthistory.length;
            try {
              var wt = Date.now();
              H.log(ft), H.write('<p style="color:blue">' + (Date.now() - wt) + " ms</p>");
            } catch (At) {
              H.write("<p>" + H.useid + "&gt;&nbsp;<b>" + ft + "</b></p>"), H.write('<p style="color:red">' + At + "<p>");
            }
          }
          var Bt = Ie.getBoundingClientRect().top + document.getElementsByTagName("body")[0].scrollTop;
          hn(document.getElementsByTagName("body")[0], Bt, 500), Ie.onkeydown = function(At) {
            if (At.which === 13) {
              var Dt = Ie.value, Ot = H.useid;
              Ie.value = "", H.prompthistory.push(Dt), pt = H.prompthistory.length;
              try {
                var un = Date.now();
                H.log(Dt), H.write('<p style="color:blue">' + (Date.now() - un) + " ms</p>");
              } catch (_n) {
                H.write("<p>" + Ot + "&gt;&nbsp;" + H.pretty(Dt, !1) + "</p>"), H.write('<p style="color:red">' + _n + "<p>");
              }
              Ie.focus(), rt.textContent = H.useid;
              var yn = Ie.getBoundingClientRect().top + document.getElementsByTagName("body")[0].scrollTop;
              hn(document.getElementsByTagName("body")[0], yn, 500);
            } else At.which === 38 ? (pt--, pt < 0 && (pt = 0), H.prompthistory[pt] && (Ie.value = H.prompthistory[pt], At.preventDefault())) : At.which === 40 && (pt++, pt >= H.prompthistory.length ? (pt = H.prompthistory.length, Ie.value = "") : H.prompthistory[pt] && (Ie.value = H.prompthistory[pt], At.preventDefault()));
          };
        }, ze.BeginTransaction = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.BeginTransaction.prototype.toString = function() {
          return "BEGIN TRANSACTION";
        }, ze.BeginTransaction.prototype.execute = function(Ie, rt, ft) {
          var pt = 1;
          return H.databases[Ie].engineid ? H.engines[H.databases[H.useid].engineid].begin(Ie, ft) : (ft && (pt = ft(pt)), pt);
        }, ze.CommitTransaction = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.CommitTransaction.prototype.toString = function() {
          return "COMMIT TRANSACTION";
        }, ze.CommitTransaction.prototype.execute = function(Ie, rt, ft) {
          var pt = 1;
          return H.databases[Ie].engineid ? H.engines[H.databases[H.useid].engineid].commit(Ie, ft) : (ft && (pt = ft(pt)), pt);
        }, ze.RollbackTransaction = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.RollbackTransaction.prototype.toString = function() {
          return "ROLLBACK TRANSACTION";
        }, ze.RollbackTransaction.prototype.execute = function(Ie, rt, ft) {
          var pt = 1;
          return H.databases[Ie].engineid ? H.engines[H.databases[Ie].engineid].rollback(Ie, ft) : (ft && (pt = ft(pt)), pt);
        }, H.options.tsql && (H.stdfn.OBJECT_ID = function(Ie, rt) {
          typeof rt > "u" && (rt = "T"), rt = rt.toUpperCase();
          var ft = Ie.split("."), pt = H.useid, wt = ft[0];
          ft.length == 2 && (pt = ft[0], wt = ft[1]);
          var Bt = H.databases[pt].tables;
          pt = H.databases[pt].databaseid;
          for (var At in Bt) if (At == wt) return Bt[At].view && rt == "V" || !Bt[At].view && rt == "T" ? pt + "." + At : void 0;
        }), H.options.mysql && (H.fn.TIMESTAMPDIFF = function(Ie, rt, ft) {
          return H.stdfn.DATEDIFF(Ie, rt, ft);
        }), (H.options.mysql || H.options.sqlite) && (H.from.INFORMATION_SCHEMA = function(Ie, rt, ft, pt, wt) {
          if (Ie == "VIEWS" || Ie == "TABLES") {
            var Bt = [];
            for (var At in H.databases) {
              var Dt = H.databases[At].tables;
              for (var Ot in Dt) (Dt[Ot].view && Ie == "VIEWS" || !Dt[Ot].view && Ie == "TABLES") && Bt.push({ TABLE_CATALOG: At, TABLE_NAME: Ot });
            }
            return ft && (Bt = ft(Bt, pt, wt)), Bt;
          }
          throw new Error("Unknown INFORMATION_SCHEMA table");
        }), H.options.postgres, H.options.oracle, H.options.sqlite, H.into.SQL = function(Ie, rt, ft, pt, wt) {
          var Bt;
          typeof Ie == "object" && (rt = Ie, Ie = void 0);
          var At = {};
          if (H.utils.extend(At, rt), typeof At.tableid > "u") throw new Error("Table for INSERT TO is not defined.");
          var Dt = "";
          pt.length === 0 && typeof ft[0] == "object" && (pt = Object.keys(ft[0]).map(function(yn) {
            return { columnid: yn };
          }));
          for (var Ot = 0, un = ft.length; Ot < un; Ot++) Dt += "INSERT INTO " + rt.tableid + "(", Dt += pt.map(function(yn) {
            return yn.columnid;
          }).join(","), Dt += ") VALUES (", Dt += pt.map(function(yn) {
            var _n = ft[Ot][yn.columnid];
            return yn.typeid ? (yn.typeid === "STRING" || yn.typeid === "VARCHAR" || yn.typeid === "NVARCHAR" || yn.typeid === "CHAR" || yn.typeid === "NCHAR") && (_n = "'" + ae(_n) + "'") : typeof _n == "string" && (_n = "'" + ae(_n) + "'"), _n;
          }), Dt += `);
`;
          return Ie = H.utils.autoExtFilename(Ie, "sql", rt), Bt = H.utils.saveFile(Ie, Dt), wt && (Bt = wt(Bt)), Bt;
        }, H.into.HTML = function(Ie, rt, ft, pt, wt) {
          var Bt = 1;
          if (typeof document != "object") {
            var At = { headers: !0 };
            H.utils.extend(At, rt);
            var Dt = document.querySelector(Ie);
            if (!Dt) throw new Error("Selected HTML element is not found");
            pt.length === 0 && typeof ft[0] == "object" && (pt = Object.keys(ft[0]).map(function(Zt) {
              return { columnid: Zt };
            }));
            var Ot = document.createElement("table"), un = document.createElement("thead");
            if (Ot.appendChild(un), At.headers) {
              for (var yn = document.createElement("tr"), _n = 0; _n < pt.length; _n++) {
                var gn = document.createElement("th");
                gn.textContent = pt[_n].columnid, yn.appendChild(gn);
              }
              un.appendChild(yn);
            }
            var bn = document.createElement("tbody");
            Ot.appendChild(bn);
            for (var an = 0; an < ft.length; an++) {
              for (var yn = document.createElement("tr"), _n = 0; _n < pt.length; _n++) {
                var gn = document.createElement("td");
                gn.textContent = ft[an][pt[_n].columnid], yn.appendChild(gn);
              }
              bn.appendChild(yn);
            }
            H.utils.domEmptyChildren(Dt), Dt.appendChild(Ot);
          }
          return wt && (Bt = wt(Bt)), Bt;
        }, H.into.JSON = function(Ie, rt, ft, pt, wt) {
          var Bt = 1;
          typeof Ie == "object" && (rt = Ie, Ie = void 0);
          var At = JSON.stringify(ft);
          return Ie = H.utils.autoExtFilename(Ie, "json", rt), Bt = H.utils.saveFile(Ie, At), wt && (Bt = wt(Bt)), Bt;
        }, H.into.TXT = function(Ie, rt, ft, pt, wt) {
          pt.length === 0 && ft.length > 0 && (pt = Object.keys(ft[0]).map(function(Ot) {
            return { columnid: Ot };
          })), typeof Ie == "object" && (rt = Ie, Ie = void 0);
          var Bt = ft.length, At = "";
          if (ft.length > 0) {
            var Dt = pt[0].columnid;
            At += ft.map(function(Ot) {
              return Ot[Dt];
            }).join(`
`);
          }
          return Ie = H.utils.autoExtFilename(Ie, "txt", rt), Bt = H.utils.saveFile(Ie, At), wt && (Bt = wt(Bt)), Bt;
        }, H.into.TAB = H.into.TSV = function(Ie, rt, ft, pt, wt) {
          var Bt = {};
          return H.utils.extend(Bt, rt), Bt.separator = "	", Ie = H.utils.autoExtFilename(Ie, "tab", rt), Bt.autoExt = !1, H.into.CSV(Ie, Bt, ft, pt, wt);
        }, H.into.CSV = function(Ie, rt, ft, pt, wt) {
          pt.length === 0 && ft.length > 0 && (pt = Object.keys(ft[0]).map(function(Ot) {
            return { columnid: Ot };
          })), typeof Ie == "object" && (rt = Ie, Ie = void 0);
          var Bt = { headers: !0 };
          Bt.separator = ";", Bt.quote = '"', Bt.utf8Bom = !0, rt && !rt.headers && typeof rt.headers < "u" && (Bt.utf8Bom = !1), H.utils.extend(Bt, rt);
          var At = ft.length, Dt = Bt.utf8Bom ? "\uFEFF" : "";
          return Bt.headers && (Dt += Bt.quote + pt.map(function(Ot) {
            return Ot.columnid.trim();
          }).join(Bt.quote + Bt.separator + Bt.quote) + Bt.quote + `\r
`), ft.forEach(function(Ot) {
            Dt += pt.map(function(un) {
              var yn = Ot[un.columnid];
              return Bt.quote !== "" && (yn = (yn + "").replace(new RegExp("\\" + Bt.quote, "g"), Bt.quote + Bt.quote)), +yn != yn && (yn = Bt.quote + yn + Bt.quote), yn;
            }).join(Bt.separator) + `\r
`;
          }), Ie = H.utils.autoExtFilename(Ie, "csv", rt), At = H.utils.saveFile(Ie, Dt, null, { disableAutoBom: !0 }), wt && (At = wt(At)), At;
        }, H.into.XLS = function(Ie, rt, ft, pt, wt) {
          typeof Ie == "object" && (rt = Ie, Ie = void 0);
          var Bt = {};
          rt && rt.sheets && (Bt = rt.sheets);
          var At = { headers: !0 };
          typeof Bt.Sheet1 < "u" ? At = Bt[0] : typeof rt < "u" && (At = rt), typeof At.sheetid > "u" && (At.sheetid = "Sheet1");
          var Dt = un();
          Ie = H.utils.autoExtFilename(Ie, "xls", rt);
          var Ot = H.utils.saveFile(Ie, Dt);
          return wt && (Ot = wt(Ot)), Ot;
          function un() {
            var yn = '<html xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:x="urn:schemas-microsoft-com:office:excel" 		xmlns="http://www.w3.org/TR/REC-html40"><head> 		<meta charset="utf-8" /> 		<!--[if gte mso 9]><xml><x:ExcelWorkbook><x:ExcelWorksheets> ';
            if (yn += " <x:ExcelWorksheet><x:Name>" + At.sheetid + "</x:Name><x:WorksheetOptions><x:DisplayGridlines/>     </x:WorksheetOptions> 		</x:ExcelWorksheet>", yn += "</x:ExcelWorksheets></x:ExcelWorkbook></xml><![endif]--></head>", yn += "<body", typeof At.style < "u" && (yn += ' style="', typeof At.style == "function" ? yn += At.style(At) : yn += At.style, yn += '"'), yn += ">", yn += "<table>", typeof At.caption < "u") {
              var _n = At.caption;
              typeof _n == "string" && (_n = { title: _n }), yn += "<caption", typeof _n.style < "u" && (yn += ' style="', typeof _n.style == "function" ? yn += _n.style(At, _n) : yn += _n.style, yn += '" '), yn += ">", yn += _n.title, yn += "</caption>";
            }
            return typeof At.columns < "u" ? pt = At.columns : pt.length == 0 && ft.length > 0 && typeof ft[0] == "object" && (Array.isArray(ft[0]) ? pt = ft[0].map(function(gn, bn) {
              return { columnid: bn };
            }) : pt = Object.keys(ft[0]).map(function(gn) {
              return { columnid: gn };
            })), pt.forEach(function(gn, bn) {
              typeof At.column < "u" && we(gn, At.column), typeof gn.width > "u" && (At.column && At.column.width != "undefined" ? gn.width = At.column.width : gn.width = "120px"), typeof gn.width == "number" && (gn.width = gn.width + "px"), typeof gn.columnid > "u" && (gn.columnid = bn), typeof gn.title > "u" && (gn.title = "" + gn.columnid.trim()), At.headers && Array.isArray(At.headers) && (gn.title = At.headers[bn]);
            }), yn += "<colgroups>", pt.forEach(function(gn) {
              yn += '<col style="width: ' + gn.width + '"></col>';
            }), yn += "</colgroups>", At.headers && (yn += "<thead>", yn += "<tr>", pt.forEach(function(gn, bn) {
              yn += "<th ", typeof gn.style < "u" && (yn += ' style="', typeof gn.style == "function" ? yn += gn.style(At, gn, bn) : yn += gn.style, yn += '" '), yn += ">", typeof gn.title < "u" && (typeof gn.title == "function" ? yn += gn.title(At, gn, bn) : yn += gn.title), yn += "</th>";
            }), yn += "</tr>", yn += "</thead>"), yn += "<tbody>", ft && ft.length > 0 && ft.forEach(function(gn, bn) {
              if (!(bn > At.limit)) {
                yn += "<tr";
                var an = {};
                we(an, At.row), At.rows && At.rows[bn] && we(an, At.rows[bn]), typeof an < "u" && typeof an.style < "u" && (yn += ' style="', typeof an.style == "function" ? yn += an.style(At, gn, bn) : yn += an.style, yn += '" '), yn += ">", pt.forEach(function(Zt, Mn) {
                  var Ln = {};
                  we(Ln, At.cell), we(Ln, an.cell), typeof At.column < "u" && we(Ln, At.column.cell), we(Ln, Zt.cell), At.cells && At.cells[bn] && At.cells[bn][Mn] && we(Ln, At.cells[bn][Mn]);
                  var In = gn[Zt.columnid];
                  typeof Ln.value == "function" && (In = Ln.value(In, At, gn, Zt, Ln, bn, Mn));
                  var kn = Ln.typeid;
                  typeof kn == "function" && (kn = kn(In, At, gn, Zt, Ln, bn, Mn)), typeof kn > "u" && (typeof In == "number" ? kn = "number" : typeof In == "string" ? kn = "string" : typeof In == "boolean" ? kn = "boolean" : typeof In == "object" && In instanceof Date && (kn = "date"));
                  var er = "";
                  kn == "money" ? er = 'mso-number-format:"\\#\\,\\#\\#0\\\\ _р_\\.";white-space:normal;' : kn == "number" ? er = " " : kn == "date" ? er = 'mso-number-format:"Short Date";' : rt.types && rt.types[kn] && rt.types[kn].typestyle && (er = rt.types[kn].typestyle), er = er || 'mso-number-format:"\\@";', yn += "<td style='" + er + "' ", typeof Ln.style < "u" && (yn += ' style="', typeof Ln.style == "function" ? yn += Ln.style(In, At, gn, Zt, bn, Mn) : yn += Ln.style, yn += '" '), yn += ">";
                  var jn = Ln.format;
                  if (typeof In > "u") yn += "";
                  else if (typeof jn < "u") if (typeof jn == "function") yn += jn(In);
                  else if (typeof jn == "string") yn += In;
                  else throw new Error("Unknown format type. Should be function or string");
                  else kn == "number" || kn == "date" ? yn += In.toString() : kn == "money" ? yn += (+In).toFixed(2) : yn += In;
                  yn += "</td>";
                }), yn += "</tr>";
              }
            }), yn += "</tbody>", yn += "</table>", yn += "</body>", yn += "</html>", yn;
          }
        }, H.into.XLSXML = function(Ie, rt, ft, pt, wt) {
          rt = rt || {}, typeof Ie == "object" && (rt = Ie, Ie = void 0);
          var Bt = {}, At, Dt;
          rt && rt.sheets ? (Bt = rt.sheets, At = ft, Dt = pt) : (Bt.Sheet1 = rt, At = [ft], Dt = [pt]), Ie = H.utils.autoExtFilename(Ie, "xls", rt);
          var Ot = H.utils.saveFile(Ie, un());
          return wt && (Ot = wt(Ot)), Ot;
          function un() {
            var yn = '<?xml version="1.0"?> 		<Workbook xmlns="urn:schemas-microsoft-com:office:spreadsheet" 		 xmlns:o="urn:schemas-microsoft-com:office:office" 		 xmlns:x="urn:schemas-microsoft-com:office:excel" 		 xmlns:ss="urn:schemas-microsoft-com:office:spreadsheet" 		 xmlns:html="http://www.w3.org/TR/REC-html40"> 		 <DocumentProperties xmlns="urn:schemas-microsoft-com:office:office"> 		 </DocumentProperties> 		 <OfficeDocumentSettings xmlns="urn:schemas-microsoft-com:office:office"> 		  <AllowPNG/> 		 </OfficeDocumentSettings> 		 <ExcelWorkbook xmlns="urn:schemas-microsoft-com:office:excel"> 		  <ActiveSheet>0</ActiveSheet> 		 </ExcelWorkbook> 		 <Styles> 		  <Style ss:ID="Default" ss:Name="Normal"> 		   <Alignment ss:Vertical="Bottom"/> 		   <Borders/> 		   <Font ss:FontName="Calibri" x:Family="Swiss" ss:Size="12" ss:Color="#000000"/> 		   <Interior/> 		   <NumberFormat/> 		   <Protection/> 		  </Style>', _n = "", gn = " </Styles>", bn = {}, an = 62;
            function Zt(Kn) {
              var Jn = "";
              for (var Lr in Kn) {
                Jn += "<" + Lr;
                for (var mr in Kn[Lr]) Jn += " ", mr.substr(0, 2) == "x:" ? Jn += mr : Jn += "ss:", Jn += mr + "=" + JSON.stringify(Kn[Lr][mr]);
                Jn += "/>";
              }
              var Ar = fe(Jn);
              return bn[Ar] || (bn[Ar] = { styleid: an }, _n += `<Style ss:ID=${JSON.stringify("s" + an)}>`, _n += Jn.replace(/<\s*\/\s*Style /gi, "<"), _n += "</Style>", an++), "s" + bn[Ar].styleid;
            }
            function Mn(Kn) {
              try {
                return Object.values(Kn);
              } catch {
                return Object.keys(Kn).map(function(Jn) {
                  return Kn[Jn];
                });
              }
            }
            var Ln = 0;
            for (var In in Bt) {
              var kn = Bt[In], er = typeof kn.dataidx < "u" ? kn.dataidx : Ln++, jn = Mn(At[er]), ur = void 0;
              typeof kn.columns < "u" ? ur = kn.columns : (ur = Dt[er], (ur === void 0 || ur.length == 0 && jn.length > 0) && typeof jn[0] == "object" && (Array.isArray(jn[0]) ? ur = jn[0].map(function(Kn, Jn) {
                return { columnid: Jn };
              }) : ur = Object.keys(jn[0]).map(function(Kn) {
                return { columnid: Kn };
              }))), ur.forEach(function(Kn, Jn) {
                typeof kn.column < "u" && we(Kn, kn.column), typeof Kn.width > "u" && (kn.column && typeof kn.column.width < "u" ? Kn.width = kn.column.width : Kn.width = 120), typeof Kn.width == "number" && (Kn.width = Kn.width), typeof Kn.columnid > "u" && (Kn.columnid = Jn), typeof Kn.title > "u" && (Kn.title = "" + Kn.columnid.trim()), kn.headers && Array.isArray(kn.headers) && (Kn.title = kn.headers[Jn]);
              }), gn += "<Worksheet ss:Name=" + JSON.stringify(In) + '> 	  			<Table ss:ExpandedColumnCount="' + ur.length + '" ss:ExpandedRowCount="' + ((kn.headers ? 1 : 0) + Math.min(jn.length, kn.limit || jn.length)) + '" x:FullColumns="1" 	   			x:FullRows="1" ss:DefaultColumnWidth="65" ss:DefaultRowHeight="15">', ur.forEach(function(Kn, Jn) {
                gn += `
					<Column 
						ss:Index="${Jn + 1}" 
						ss:AutoFitWidth="0" 
						ss:Width=${JSON.stringify("" + Kn.width)}
					/>`;
              }), kn.headers && (gn += '<Row ss:AutoFitHeight="0">', ur.forEach(function(Kn, Jn) {
                if (gn += "<Cell ", typeof Kn.style < "u") {
                  var Lr = {};
                  typeof Kn.style == "function" ? we(Lr, Kn.style(kn, Kn, Jn)) : we(Lr, Kn.style), gn += "ss:StyleID=" + JSON.stringify(Zt(Lr));
                }
                gn += '><Data ss:Type="String">', typeof Kn.title < "u" && (typeof Kn.title == "function" ? gn += Kn.title(kn, Kn, Jn) : gn += Kn.title), gn += "</Data></Cell>";
              }), gn += "</Row>"), jn && jn.length > 0 && jn.forEach(function(Kn, Jn) {
                if (!(Jn > kn.limit)) {
                  var Lr = {};
                  if (we(Lr, kn.row), kn.rows && kn.rows[Jn] && we(Lr, kn.rows[Jn]), gn += "<Row ", typeof Lr < "u") {
                    var mr = {};
                    typeof Lr.style < "u" && (typeof Lr.style == "function" ? we(mr, Lr.style(kn, Kn, Jn)) : we(mr, Lr.style), gn += "ss:StyleID=" + JSON.stringify(Zt(mr)));
                  }
                  gn += ">", ur.forEach(function(Ar, Yr) {
                    var jr = {};
                    we(jr, kn.cell), we(jr, Lr.cell), typeof kn.column < "u" && we(jr, kn.column.cell), we(jr, Ar.cell), kn.cells && kn.cells[Jn] && kn.cells[Jn][Yr] && we(jr, kn.cells[Jn][Yr]);
                    var Mr = Kn[Ar.columnid];
                    typeof jr.value == "function" && (Mr = jr.value(Mr, kn, Kn, Ar, jr, Jn, Yr));
                    var Wr = jr.typeid;
                    typeof Wr == "function" && (Wr = Wr(Mr, kn, Kn, Ar, jr, Jn, Yr)), typeof Wr > "u" && (typeof Mr == "number" ? Wr = "number" : typeof Mr == "string" ? Wr = "string" : typeof Mr == "boolean" ? Wr = "boolean" : typeof Mr == "object" && Mr instanceof Date && (Wr = "date"));
                    var Bi = "String";
                    Wr == "number" ? Bi = "Number" : Wr == "date" && (Bi = "Date"), Wr == "money" || Wr == "number" || Wr == "date" || rt.types && rt.types[Wr] && rt.types[Wr].typestyle && rt.types[Wr].typestyle, gn += "<Cell ";
                    var gi = {};
                    typeof jr.style < "u" && (typeof jr.style == "function" ? we(gi, jr.style(Mr, kn, Kn, Ar, Jn, Yr)) : we(gi, jr.style), gn += "ss:StyleID=" + JSON.stringify(Zt(gi))), gn += ">", gn += "<Data ss:Type=" + JSON.stringify(Bi) + ">";
                    var Zr = jr.format;
                    if (typeof Mr > "u") gn += "";
                    else if (typeof Zr < "u") if (typeof Zr == "function") gn += Zr(Mr);
                    else if (typeof Zr == "string") gn += Mr;
                    else throw new Error("Unknown format type. Should be function or string");
                    else Wr == "number" || Wr == "date" ? gn += Mr.toString() : Wr == "money" ? gn += (+Mr).toFixed(2) : gn += Mr;
                    gn += "</Data></Cell>";
                  }), gn += "</Row>";
                }
              }), gn += "</Table></Worksheet>";
            }
            return gn += "</Workbook>", yn + _n + gn;
          }
        }, H.into.XLSX = function(Ie, rt, ft, pt, wt) {
          var Bt = 1;
          rt = rt || {}, Ce(pt, [{ columnid: "_" }]) && (ft = ft.map(function(_n) {
            return _n._;
          }), pt = void 0), Ie = H.utils.autoExtFilename(Ie, "xlsx", rt);
          var At = Qe();
          typeof Ie == "object" && (rt = Ie, Ie = void 0);
          var Dt = { SheetNames: [], Sheets: {} };
          return rt.sourcefilename ? H.utils.loadBinaryFile(rt.sourcefilename, !!wt, function(_n) {
            Dt = At.read(_n, { type: "binary", ...H.options.excel, ...rt }), Ot();
          }) : Ot(), wt && (Bt = wt(Bt)), Bt;
          function Ot() {
            typeof rt == "object" && Array.isArray(rt) ? ft && ft.length > 0 && ft.forEach(function(_n, gn) {
              un(rt[gn], _n, void 0, gn + 1);
            }) : un(rt, ft, pt, 1), yn();
          }
          function un(_n, gn, bn, an) {
            var Zt = { sheetid: "Sheet " + an, headers: !0 };
            H.utils.extend(Zt, _n);
            var Mn = Object.keys(gn).length;
            (!bn || bn.length == 0) && (Mn > 0 ? bn = Object.keys(gn[0]).map(function(jr) {
              return { columnid: jr };
            }) : bn = []);
            var Ln = {};
            Dt.SheetNames.indexOf(Zt.sheetid) > -1 || (Dt.SheetNames.push(Zt.sheetid), Dt.Sheets[Zt.sheetid] = {}), Ln = Dt.Sheets[Zt.sheetid];
            var In = "A1";
            Zt.range && (In = Zt.range);
            var kn = H.utils.xlscn(In.match(/[A-Z]+/)[0]), er = +In.match(/[0-9]+/)[0] - 1;
            if (Dt.Sheets[Zt.sheetid]["!ref"]) var jn = Dt.Sheets[Zt.sheetid]["!ref"], ur = H.utils.xlscn(jn.match(/[A-Z]+/)[0]), Kn = +jn.match(/[0-9]+/)[0] - 1;
            else var ur = 1, Kn = 1;
            var Jn = bn.length ? 0 : 1, Lr = Math.max(kn + bn.length - 1 + Jn, ur), mr = Math.max(er + Mn + 2, Kn), Ar = er + 1;
            Dt.Sheets[Zt.sheetid]["!ref"] = "A1:" + H.utils.xlsnc(Lr) + mr, Zt.headers && (bn.forEach(function(jr, Mr) {
              Ln[H.utils.xlsnc(kn + Mr) + "" + Ar] = { v: jr.columnid.trim() };
            }), Ar++);
            for (var Yr = 0; Yr < Mn; Yr++) bn.forEach(function(jr, Mr) {
              var Wr = { v: gn[Yr][jr.columnid] };
              typeof gn[Yr][jr.columnid] == "number" ? Wr.t = "n" : typeof gn[Yr][jr.columnid] == "string" ? Wr.t = "s" : typeof gn[Yr][jr.columnid] == "boolean" ? Wr.t = "b" : typeof gn[Yr][jr.columnid] == "object" && gn[Yr][jr.columnid] instanceof Date && (Wr.t = "d"), Ln[H.utils.xlsnc(kn + Mr) + "" + Ar] = Wr;
            }), Ar++;
          }
          function yn(_n) {
            var gn;
            if (typeof Ie > "u") Bt = Dt;
            else if (gn = Qe(), J.isNode || J.isMeteorServer) gn.writeFile(Dt, Ie);
            else {
              var bn = { bookType: "xlsx", bookSST: !1, type: "binary" }, an = gn.write(Dt, bn), Zt = function(Mn) {
                for (var Ln = new ArrayBuffer(Mn.length), In = new Uint8Array(Ln), kn = 0; kn != Mn.length; ++kn) In[kn] = Mn.charCodeAt(kn) & 255;
                return Ln;
              };
              cn(new Blob([Zt(an)], { type: "application/octet-stream" }), Ie);
            }
          }
        }, H.from.METEOR = function(Ie, rt, ft, pt, wt) {
          var Bt = Ie.find(rt).fetch();
          return ft && (Bt = ft(Bt, pt, wt)), Bt;
        }, H.from.TABLETOP = function(Ie, rt, ft, pt, wt) {
          var Bt = [], At = { headers: !0, simpleSheet: !0, key: Ie };
          return H.utils.extend(At, rt), At.callback = function(Dt) {
            Bt = Dt, ft && (Bt = ft(Bt, pt, wt));
          }, Tabletop.init(At), null;
        }, H.from.HTML = function(Ie, rt, ft, pt, wt) {
          var Bt = {};
          H.utils.extend(Bt, rt);
          var At = document.querySelector(Ie);
          if (!At && At.tagName !== "TABLE") throw new Error("Selected HTML element is not a TABLE");
          var Dt = [], Ot = Bt.headers;
          if (Ot && !Array.isArray(Ot)) {
            Ot = [];
            for (var un = At.querySelector("thead tr").children, yn = 0; yn < un.length; yn++) un.item(yn).style && un.item(yn).style.display === "none" && Bt.skipdisplaynone ? Ot.push(void 0) : Ot.push(un.item(yn).textContent);
          }
          for (var _n = At.querySelectorAll("tbody tr"), gn = 0; gn < _n.length; gn++) {
            var bn = _n.item(gn).children, an = {};
            for (yn = 0; yn < bn.length; yn++) bn.item(yn).style && bn.item(yn).style.display === "none" && Bt.skipdisplaynone || (Ot ? an[Ot[yn]] = bn.item(yn).textContent : an[yn] = bn.item(yn).textContent);
            Dt.push(an);
          }
          return ft && (Dt = ft(Dt, pt, wt)), Dt;
        }, H.from.RANGE = function(Ie, rt, ft, pt, wt) {
          for (var Bt = [], At = Ie; At <= rt; At++) Bt.push(At);
          return ft && (Bt = ft(Bt, pt, wt)), Bt;
        }, H.from.FILE = function(Ie, rt, ft, pt, wt) {
          var Bt;
          if (typeof Ie == "string") Bt = Ie;
          else if (Ie instanceof Event) Bt = Ie.target.files[0].name;
          else throw new Error("Wrong usage of FILE() function");
          var At = Bt.split("."), Dt = At[At.length - 1].toUpperCase();
          if (H.from[Dt]) return H.from[Dt](Ie, rt, ft, pt, wt);
          throw new Error("Cannot recognize file type for loading");
        }, H.from.JSON = function(Ie, rt, ft, pt, wt) {
          var Bt;
          return Ie = H.utils.autoExtFilename(Ie, "json", rt), H.utils.loadFile(Ie, !!ft, function(At) {
            Bt = JSON.parse(At), ft && (Bt = ft(Bt, pt, wt));
          }), Bt;
        };
        const Qt = (Ie) => function(rt, ft, pt, wt, Bt) {
          let At = [];
          return rt = H.utils.autoExtFilename(rt, Ie, ft), H.utils.loadFile(rt, !!pt, function(Dt) {
            Dt.split(/\r?\n/).forEach((Ot, un) => {
              const yn = Ot.trim();
              if (yn !== "") try {
                At.push(JSON.parse(yn));
              } catch (_n) {
                throw new Error(`Could not parse JSON at line ${un}: ${_n.toString()}`);
              }
            }), pt && (res = pt(At, wt, Bt));
          }, (Dt) => {
            throw new Error(Dt);
          }), At;
        };
        H.from.JSONL = Qt("jsonl"), H.from.NDJSON = Qt("ndjson"), H.from.TXT = function(Ie, rt, ft, pt, wt) {
          var Bt;
          return Ie = H.utils.autoExtFilename(Ie, "txt", rt), H.utils.loadFile(Ie, !!ft, function(At) {
            Bt = At.split(/\r?\n/), Bt[Bt.length - 1] === "" && Bt.pop();
            for (var Dt = 0, Ot = Bt.length; Dt < Ot; Dt++) Bt[Dt] == +Bt[Dt] && (Bt[Dt] = +Bt[Dt]), Bt[Dt] = [Bt[Dt]];
            ft && (Bt = ft(Bt, pt, wt));
          }), Bt;
        }, H.from.TAB = H.from.TSV = function(Ie, rt, ft, pt, wt) {
          return rt = rt || {}, rt.separator = "	", Ie = H.utils.autoExtFilename(Ie, "tab", rt), rt.autoext = !1, H.from.CSV(Ie, rt, ft, pt, wt);
        }, H.from.CSV = function(Ie, rt, ft, pt, wt) {
          Ie = "" + Ie;
          var Bt = { separator: ",", quote: '"', headers: !0, raw: !1 };
          H.utils.extend(Bt, rt);
          var At, Dt = [];
          function Ot(un) {
            var yn = Bt.separator.charCodeAt(0), _n = Bt.quote.charCodeAt(0), gn = {}, bn = {}, an = [], Zt = un.length, Mn = 0, Ln = 0, In, kn;
            function er() {
              if (Mn >= Zt) return bn;
              if (kn) return kn = !1, gn;
              var Jn = Mn;
              if (un.charCodeAt(Jn) === _n) {
                for (var Lr = Jn; Lr++ < Zt; ) if (un.charCodeAt(Lr) === _n) {
                  if (un.charCodeAt(Lr + 1) !== _n) break;
                  ++Lr;
                }
                Mn = Lr + 2;
                var mr = un.charCodeAt(Lr + 1);
                return mr === 13 ? (kn = !0, un.charCodeAt(Lr + 2) === 10 && ++Mn) : mr === 10 && (kn = !0), un.substring(Jn + 1, Lr).replace(/""/g, '"');
              }
              for (; Mn < Zt; ) {
                var mr = un.charCodeAt(Mn++), Ar = 1;
                if (mr === 10) kn = !0;
                else if (mr === 13) kn = !0, un.charCodeAt(Mn) === 10 && (++Mn, ++Ar);
                else if (mr !== yn) continue;
                return un.substring(Jn, Mn - Ar);
              }
              return un.substring(Jn);
            }
            for (; (In = er()) !== bn; ) {
              for (var jn = []; In !== gn && In !== bn; ) jn.push(In.trim()), In = er();
              if (Bt.headers) {
                if (Ln === 0) {
                  if (typeof Bt.headers == "boolean") Dt = jn;
                  else if (Array.isArray(Bt.headers)) {
                    Dt = Bt.headers;
                    var ur = {};
                    Dt.forEach(function(Jn, Lr) {
                      ur[Jn] = jn[Lr], !Bt.raw && typeof ur[Jn] < "u" && ur[Jn].length !== 0 && ur[Jn].trim() == +ur[Jn] && (ur[Jn] = +ur[Jn]);
                    }), an.push(ur);
                  }
                } else {
                  var ur = {};
                  Dt.forEach(function(Lr, mr) {
                    ur[Lr] = jn[mr], !Bt.raw && typeof ur[Lr] < "u" && ur[Lr].length !== 0 && ur[Lr].trim() == +ur[Lr] && (ur[Lr] = +ur[Lr]);
                  }), an.push(ur);
                }
                Ln++;
              } else {
                var ur = {};
                jn.forEach(function(Lr, mr) {
                  ur[mr] = jn[mr], !Bt.raw && typeof ur[mr] < "u" && ur[mr].length !== 0 && ur[mr].trim() == +ur[mr] && (ur[mr] = +ur[mr]);
                }), an.push(ur);
              }
            }
            if (At = an, Bt.headers && wt && wt.sources && wt.sources[pt]) {
              var Kn = wt.sources[pt].columns = [];
              Dt.forEach(function(Jn) {
                Kn.push({ columnid: Jn });
              });
            }
            ft && (At = ft(At, pt, wt));
          }
          return new RegExp(`
`).test(Ie) ? Ot(Ie) : (Ie = H.utils.autoExtFilename(Ie, "csv", rt), H.utils.loadFile(Ie, !!ft, Ot, wt.cb)), At;
        };
        function Ut(Ie, rt, ft, pt, wt, Bt) {
          var At = {};
          ft = ft || {}, H.utils.extend(At, ft), typeof At.headers > "u" && (At.headers = !0);
          var Dt;
          function Ot(yn) {
            for (var _n = "", gn = 0, bn = 10240; gn < yn.byteLength / bn; ++gn) _n += String.fromCharCode.apply(null, new Uint8Array(yn.slice(gn * bn, gn * bn + bn)));
            return _n += String.fromCharCode.apply(null, new Uint8Array(yn.slice(gn * bn))), _n;
          }
          function un(yn) {
            return yn && H.options.casesensitive === !1 ? yn.toLowerCase() : yn;
          }
          return rt = H.utils.autoExtFilename(rt, "xls", ft), H.utils.loadBinaryFile(rt, !!pt, function(yn) {
            if (yn instanceof ArrayBuffer) var _n = Ot(yn), gn = Ie.read(btoa(_n), { type: "base64", ...H.options.excel, ...ft });
            else var gn = Ie.read(yn, { type: "binary", ...H.options.excel, ...ft });
            var bn;
            typeof At.sheetid > "u" ? bn = gn.SheetNames[0] : typeof At.sheetid == "number" ? bn = gn.SheetNames[At.sheetid] : bn = At.sheetid;
            var an, Zt = [];
            if (typeof At.range > "u" ? an = gn.Sheets[bn]["!ref"] : (an = At.range, gn.Sheets[bn][an] && (an = gn.Sheets[bn][an])), an) {
              for (var Mn = an.split(":"), Ln = Mn[0].match(/[A-Z]+/)[0], In = +Mn[0].match(/[0-9]+/)[0], kn = Mn[1].match(/[A-Z]+/)[0], er = +Mn[1].match(/[0-9]+/)[0], jn = {}, ur = H.utils.xlscn(Ln), Kn = H.utils.xlscn(kn), Jn = ur; Jn <= Kn; Jn++) {
                var Lr = H.utils.xlsnc(Jn);
                At.headers ? gn.Sheets[bn][Lr + "" + In] ? jn[Lr] = un(gn.Sheets[bn][Lr + "" + In].v) : jn[Lr] = un(Lr) : jn[Lr] = Lr;
              }
              At.headers && In++;
              for (var mr = In; mr <= er; mr++) {
                for (var Ar = {}, Jn = ur; Jn <= Kn; Jn++) {
                  var Lr = H.utils.xlsnc(Jn);
                  gn.Sheets[bn][Lr + "" + mr] && (Ar[jn[Lr]] = gn.Sheets[bn][Lr + "" + mr].v);
                }
                Zt.push(Ar);
              }
            } else Zt.push([]);
            Zt.length > 0 && Zt[Zt.length - 1] && Object.keys(Zt[Zt.length - 1]).length == 0 && Zt.pop(), pt && (Zt = pt(Zt, wt, Bt));
          }, function(yn) {
            throw yn;
          }), Dt;
        }
        H.from.XLS = function(Ie, rt, ft, pt, wt) {
          return rt = rt || {}, Ie = H.utils.autoExtFilename(Ie, "xls", rt), rt.autoExt = !1, Ut(Qe(), Ie, rt, ft, pt, wt);
        }, H.from.XLSX = function(Ie, rt, ft, pt, wt) {
          return rt = rt || {}, Ie = H.utils.autoExtFilename(Ie, "xlsx", rt), rt.autoExt = !1, Ut(Qe(), Ie, rt, ft, pt, wt);
        }, H.from.ODS = function(Ie, rt, ft, pt, wt) {
          return rt = rt || {}, Ie = H.utils.autoExtFilename(Ie, "ods", rt), rt.autoExt = !1, Ut(Qe(), Ie, rt, ft, pt, wt);
        }, H.from.XML = function(Ie, rt, ft, pt, wt) {
          var Bt;
          return H.utils.loadFile(Ie, !!ft, function(At) {
            Bt = Wt(At).root, ft && (Bt = ft(Bt, pt, wt));
          }), Bt;
        };
        function Wt(Ie) {
          return Ie = Ie.trim(), Ie = Ie.replace(/<!--[\s\S]*?-->/g, ""), rt();
          function rt() {
            return { declaration: ft(), root: pt() };
          }
          function ft() {
            var yn = Dt(/^<\?xml\s*/);
            if (yn) {
              for (var _n = { attributes: {} }; !(Ot() || un("?>")); ) {
                var gn = Bt();
                if (!gn) return _n;
                _n.attributes[gn.name] = gn.value;
              }
              return Dt(/\?>\s*/), _n;
            }
          }
          function pt() {
            var yn = Dt(/^<([\w-:.]+)\s*/);
            if (yn) {
              for (var _n = { name: yn[1], attributes: {}, children: [] }; !(Ot() || un(">") || un("?>") || un("/>")); ) {
                var gn = Bt();
                if (!gn) return _n;
                _n.attributes[gn.name] = gn.value;
              }
              if (Dt(/^\s*\/>\s*/)) return _n;
              Dt(/\??>\s*/), _n.content = wt();
              for (var bn; bn = pt(); ) _n.children.push(bn);
              return Dt(/^<\/[\w-:.]+>\s*/), _n;
            }
          }
          function wt() {
            var yn = Dt(/^([^<]*)/);
            return yn ? yn[1] : "";
          }
          function Bt() {
            var yn = Dt(/([\w:-]+)\s*=\s*("[^"]*"|'[^']*'|\w+)\s*/);
            if (yn) return { name: yn[1], value: At(yn[2]) };
          }
          function At(yn) {
            return yn.replace(/^['"]|['"]$/g, "");
          }
          function Dt(yn) {
            var _n = Ie.match(yn);
            if (_n) return Ie = Ie.slice(_n[0].length), _n;
          }
          function Ot() {
            return Ie.length == 0;
          }
          function un(yn) {
            return Ie.indexOf(yn) == 0;
          }
        }
        H.from.GEXF = function(Ie, rt, ft, pt, wt) {
          var Bt;
          return H("SEARCH FROM XML(" + Ie + ")", [], function(At) {
            Bt = At, ft && (Bt = ft(Bt));
          }), Bt;
        }, ze.Print = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Print.prototype.toString = function() {
          var Ie = "PRINT";
          return this.statement && (Ie += " " + this.statement.toString()), Ie;
        }, ze.Print.prototype.execute = function(Ie, rt, ft) {
          var pt = this, wt = 1;
          if (H.precompile(this, Ie, rt), this.exprs && this.exprs.length > 0) {
            var Bt = this.exprs.map(function(Dt) {
              var Ot = new Function("params,alasql,p", "var y;return " + Dt.toJS("({})", "", null)).bind(pt), un = Ot(rt, H);
              return Ft(un);
            });
            console.log.apply(console, Bt);
          } else if (this.select) {
            var At = this.select.execute(Ie, rt);
            console.log(Ft(At));
          } else console.log();
          return ft && (wt = ft(wt)), wt;
        }, ze.Source = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Source.prototype.toString = function() {
          var Ie = "SOURCE";
          return this.url && (Ie += " '" + this.url + " '"), Ie;
        }, ze.Source.prototype.execute = function(Ie, rt, ft) {
          var pt;
          return ce(this.url, !!ft, function(wt) {
            return pt = H(wt), ft && (pt = ft(pt)), pt;
          }, function(wt) {
            throw wt;
          }), pt;
        }, ze.Require = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Require.prototype.toString = function() {
          var Ie = "REQUIRE";
          return this.paths && this.paths.length > 0 && (Ie += this.paths.map(function(rt) {
            return rt.toString();
          }).join(",")), this.plugins && this.plugins.length > 0 && (Ie += this.plugins.map(function(rt) {
            return rt.toUpperCase();
          }).join(",")), Ie;
        }, ze.Require.prototype.execute = function(Ie, rt, ft) {
          var pt = this, wt = 0, Bt = "";
          return this.paths && this.paths.length > 0 ? this.paths.forEach(function(At) {
            ce(At.value, !!ft, function(Dt) {
              wt++, Bt += Dt, !(wt < pt.paths.length) && (new Function("params,alasql", Bt)(rt, H), ft && (wt = ft(wt)));
            });
          }) : this.plugins && this.plugins.length > 0 ? this.plugins.forEach(function(At) {
            H.plugins[At] || ce(H.path + "/alasql-" + At.toLowerCase() + ".js", !!ft, function(Dt) {
              wt++, Bt += Dt, !(wt < pt.plugins.length) && (new Function("params,alasql", Bt)(rt, H), H.plugins[At] = !0, ft && (wt = ft(wt)));
            });
          }) : ft && (wt = ft(wt)), wt;
        }, ze.Assert = function(Ie) {
          return Object.assign(this, Ie);
        }, ze.Source.prototype.toString = function() {
          var Ie = "ASSERT";
          return this.value && (Ie += " " + JSON.stringify(this.value)), Ie;
        }, ze.Assert.prototype.execute = function(Ie) {
          if (!Ce(H.res, this.value)) throw new Error((this.message || "Assert wrong") + ": " + JSON.stringify(H.res) + " == " + JSON.stringify(this.value));
          return 1;
        };
        var rn = H.engines.INDEXEDDB = function() {
        };
        async function vn(Ie) {
          const rt = globalThis.indexedDB;
          if (!rt) throw new Error("IndexedDB is not supported in this browser");
          if (rt.databases) return (await rt.databases()).find((pt) => pt.name === Ie) || 0;
          const ft = rt.open(Ie);
          return new Promise(function(pt, wt) {
            ft.onsuccess = () => {
              ft.result.close(), pt({ name: Ie, version: ft.result.version });
            }, ft.onupgradeneeded = (Bt) => {
              Bt.target.transaction.abort(), pt(0);
            }, ft.onerror = () => {
              wt(new Error("IndexedDB error"));
            }, ft.onblocked = () => {
              pt({ name: Ie, version: ft.result.version });
            };
          });
        }
        rn.showDatabases = function(Ie, rt) {
          if (!indexedDB.databases) {
            rt(null, new Error("SHOW DATABASE is not supported in this browser"));
            return;
          }
          indexedDB.databases().then((ft) => {
            const pt = [], wt = Ie && new RegExp(Ie.value.replace(/\%/g, ".*"), "g");
            for (var Bt = 0; Bt < ft.length; Bt++) (!Ie || ft[Bt].name.match(wt)) && pt.push({ databaseid: ft[Bt].name });
            rt(pt);
          });
        }, rn.createDatabase = async function(Ie, rt, ft, pt, wt) {
          if (await vn(Ie).catch((Bt) => {
            throw wt && wt(null, Bt), Bt;
          })) if (ft) wt && wt(0);
          else {
            const Bt = new Error(`IndexedDB: Cannot create new database "${Ie}" because it already exists`);
            wt && wt(null, Bt);
          }
          else {
            const Bt = indexedDB.open(Ie, 1);
            Bt.onsuccess = () => {
              Bt.result.close(), wt(1);
            };
          }
        }, rn.dropDatabase = async function(Ie, rt, ft) {
          if (await vn(Ie).catch((pt) => {
            throw ft && ft(null, pt), pt;
          })) {
            const pt = indexedDB.deleteDatabase(Ie);
            pt.onsuccess = () => {
              ft && ft(1);
            };
          } else rt ? ft && ft(0) : ft && ft(null, new Error(`IndexedDB: Cannot drop new database "${Ie}" because it does not exist'`));
        }, rn.attachDatabase = async function(Ie, rt, ft, pt, wt) {
          if (!await vn(Ie).catch((Ot) => {
            throw wt && wt(null, Ot), Ot;
          })) {
            const Ot = new Error(`IndexedDB: Cannot attach database "${Ie}" because it does not exist`);
            throw wt && wt(null, Ot), Ot;
          }
          const Bt = await new Promise((Ot, un) => {
            const yn = indexedDB.open(Ie);
            yn.onsuccess = () => {
              Ot(yn.result.objectStoreNames), yn.result.close();
            };
          }), At = new H.Database(rt || Ie);
          At.engineid = "INDEXEDDB", At.ixdbid = Ie, At.tables = [];
          for (var Dt = 0; Dt < Bt.length; Dt++) At.tables[Bt[Dt]] = {};
          wt && wt(1);
        }, rn.createTable = async function(Ie, rt, ft, pt) {
          const wt = H.databases[Ie].ixdbid, Bt = await vn(wt).catch((Dt) => {
            throw pt && pt(null, Dt), Dt;
          });
          if (!Bt) {
            const Dt = new Error('IndexedDB: Cannot create table in database "' + wt + '" because it does not exist');
            throw pt && pt(null, Dt), Dt;
          }
          const At = indexedDB.open(wt, Bt.version + 1);
          At.onupgradeneeded = function(Dt) {
            At.result.createObjectStore(rt, { autoIncrement: !0 });
          }, At.onsuccess = function(Dt) {
            At.result.close(), pt && pt(1);
          }, At.onerror = (Dt) => {
            pt(null, Dt);
          }, At.onblocked = function(Dt) {
            pt(null, new Error(`Cannot create table "${rt}" because database "${Ie}"  is blocked`));
          };
        }, rn.dropTable = async function(Ie, rt, ft, pt) {
          const wt = H.databases[Ie].ixdbid, Bt = await vn(wt).catch((Ot) => {
            throw pt && pt(null, Ot), Ot;
          });
          if (!Bt) {
            const Ot = new Error('IndexedDB: Cannot drop table in database "' + wt + '" because it does not exist');
            throw pt && pt(null, Ot), Ot;
          }
          const At = indexedDB.open(wt, Bt.version + 1);
          let Dt;
          At.onupgradeneeded = function(Ot) {
            var un = At.result;
            un.objectStoreNames.contains(rt) ? (un.deleteObjectStore(rt), delete H.databases[Ie].tables[rt]) : ft || (Dt = new Error(`IndexedDB: Cannot drop table "${rt}" because it does not exist`), Ot.target.transaction.abort());
          }, At.onsuccess = function(Ot) {
            At.result.close(), pt && pt(1);
          }, At.onerror = function(Ot) {
            pt && pt(null, Dt || Ot);
          }, At.onblocked = function(Ot) {
            pt(null, new Error(`Cannot drop table "${rt}" because database "${Ie}" is blocked`));
          };
        }, rn.intoTable = function(Ie, rt, ft, pt, wt) {
          const Bt = H.databases[Ie].ixdbid, At = indexedDB.open(Bt);
          var Dt = H.databases[Ie], Ot = Dt.tables[rt];
          At.onupgradeneeded = (un) => {
            un.target.transaction.abort();
            const yn = new Error(`Cannot insert into table "${rt}" because database "${Ie}" does not exist`);
            wt && wt(null, yn);
          }, At.onsuccess = () => {
            for (var un = At.result, yn = un.transaction([rt], "readwrite"), _n = yn.objectStore(rt), gn = 0, bn = ft.length; gn < bn; gn++) _n.add(ft[gn]);
            yn.oncomplete = function() {
              un.close();
              for (var an in Ot.afterinsert) if (Ot.afterinsert[an]) {
                var Zt = Ot.afterinsert[an];
                Zt.funcid ? H.fn[Zt.funcid](ft) : Zt.statement && Zt.statement.execute(Ie);
              }
              wt && wt(bn);
            };
          };
        }, rn.fromTable = function(Ie, rt, ft, pt, wt) {
          const Bt = H.databases[Ie].ixdbid, At = indexedDB.open(Bt);
          At.onupgradeneeded = (Dt) => {
            Dt.target.transaction.abort();
            const Ot = new Error(`Cannot select from table "${rt}" because database "${Ie}" does not exist`);
            ft && ft(null, Ot);
          }, At.onsuccess = () => {
            const Dt = [], Ot = At.result, un = Ot.transaction([rt]).objectStore(rt).openCursor();
            un.onsuccess = () => {
              const yn = un.result;
              if (yn) {
                const _n = typeof yn == "object" ? yn.value : { [yn.key]: yn.value };
                Dt.push(_n), yn.continue();
              } else Ot.close(), ft && ft(Dt, pt, wt);
            };
          };
        }, rn.deleteFromTable = function(Ie, rt, ft, pt, wt) {
          const Bt = H.databases[Ie].ixdbid, At = indexedDB.open(Bt);
          At.onsuccess = () => {
            const Dt = At.result, Ot = Dt.transaction([rt], "readwrite").objectStore(rt).openCursor();
            let un = 0;
            Ot.onsuccess = () => {
              var yn = Ot.result;
              yn ? ((!ft || ft(yn.value, pt, H)) && (yn.delete(), un++), yn.continue()) : (Dt.close(), wt && wt(un));
            };
          };
        }, rn.updateTable = function(Ie, rt, ft, pt, wt, Bt) {
          const At = H.databases[Ie].ixdbid, Dt = indexedDB.open(At);
          Dt.onsuccess = function() {
            const Ot = Dt.result, un = Ot.transaction([rt], "readwrite").objectStore(rt).openCursor();
            let yn = 0;
            un.onsuccess = () => {
              var _n = un.result;
              if (_n) {
                if (!pt || pt(_n.value, wt)) {
                  var gn = _n.value;
                  ft(gn, wt), _n.update(gn), yn++;
                }
                _n.continue();
              } else Ot.close(), Bt && Bt(yn);
            };
          };
        };
        var dn = H.engines.LOCALSTORAGE = function() {
        };
        dn.get = function(Ie) {
          var rt = localStorage.getItem(Ie);
          if (!(typeof rt > "u")) {
            var ft;
            try {
              ft = JSON.parse(rt);
            } catch {
              throw new Error("Cannot parse JSON object from localStorage" + rt);
            }
            return ft;
          }
        }, dn.set = function(Ie, rt) {
          typeof rt > "u" ? localStorage.removeItem(Ie) : localStorage.setItem(Ie, JSON.stringify(rt));
        }, dn.storeTable = function(Ie, rt) {
          var ft = H.databases[Ie], pt = ft.tables[rt], wt = {};
          wt.columns = pt.columns, wt.data = pt.data, wt.identities = pt.identities, dn.set(ft.lsdbid + "." + rt, wt);
        }, dn.restoreTable = function(Ie, rt) {
          var ft = H.databases[Ie], pt = dn.get(ft.lsdbid + "." + rt), wt = new H.Table();
          for (var Bt in pt) wt[Bt] = pt[Bt];
          return ft.tables[rt] = wt, wt.indexColumns(), wt;
        }, dn.removeTable = function(Ie, rt) {
          var ft = H.databases[Ie];
          localStorage.removeItem(ft.lsdbid + "." + rt);
        }, dn.createDatabase = function(Ie, rt, ft, pt, wt) {
          var Bt = 1, At = dn.get("alasql");
          if (ft && At && At.databases && At.databases[Ie]) Bt = 0;
          else {
            if (At || (At = { databases: {} }), At.databases && At.databases[Ie]) throw new Error('localStorage: Cannot create new database "' + Ie + '" because it already exists');
            At.databases[Ie] = !0, dn.set("alasql", At), dn.set(Ie, { databaseid: Ie, tables: {} });
          }
          return wt && (Bt = wt(Bt)), Bt;
        }, dn.dropDatabase = function(Ie, rt, ft) {
          var pt = 1, wt = dn.get("alasql");
          if (rt && wt && wt.databases && !wt.databases[Ie]) pt = 0;
          else {
            if (!wt) {
              if (rt) return ft ? ft(0) : 0;
              throw new Error("There is no any AlaSQL databases in localStorage");
            }
            if (wt.databases && !wt.databases[Ie]) throw new Error('localStorage: Cannot drop database "' + Ie + '" because there is no such database');
            delete wt.databases[Ie], dn.set("alasql", wt);
            var Bt = dn.get(Ie);
            for (var At in Bt.tables) localStorage.removeItem(Ie + "." + At);
            localStorage.removeItem(Ie);
          }
          return ft && (pt = ft(pt)), pt;
        }, dn.attachDatabase = function(Ie, rt, ft, pt, wt) {
          var Bt = 1;
          if (H.databases[rt]) throw new Error('Unable to attach database as "' + rt + '" because it already exists');
          rt || (rt = Ie);
          var At = new H.Database(rt);
          if (At.engineid = "LOCALSTORAGE", At.lsdbid = Ie, At.tables = dn.get(Ie).tables, !H.options.autocommit && At.tables) for (var Dt in At.tables) dn.restoreTable(rt, Dt);
          return wt && (Bt = wt(Bt)), Bt;
        }, dn.showDatabases = function(Ie, rt) {
          var ft = [], pt = dn.get("alasql");
          if (Ie) var wt = new RegExp(Ie.value.replace(/%/g, ".*"), "g");
          if (pt && pt.databases) {
            for (var Bt in pt.databases) ft.push({ databaseid: Bt });
            Ie && ft && ft.length > 0 && (ft = ft.filter(function(At) {
              return At.databaseid.match(wt);
            }));
          }
          return rt && (ft = rt(ft)), ft;
        }, dn.createTable = function(Ie, rt, ft, pt) {
          var wt = 1, Bt = H.databases[Ie].lsdbid, At = dn.get(Bt + "." + rt);
          if (At && !ft) throw new Error('Table "' + rt + '" alsready exists in localStorage database "' + Bt + '"');
          var Dt = dn.get(Bt);
          return H.databases[Ie].tables[rt], Dt.tables[rt] = !0, dn.set(Bt, Dt), dn.storeTable(Ie, rt), pt && (wt = pt(wt)), wt;
        }, dn.truncateTable = function(Ie, rt, ft, pt) {
          var wt = 1, Bt = H.databases[Ie].lsdbid, At;
          if (H.options.autocommit ? At = dn.get(Bt) : At = H.databases[Ie], !ft && !At.tables[rt]) throw new Error('Cannot truncate table "' + rt + '" in localStorage, because it does not exist');
          var Dt = dn.restoreTable(Ie, rt);
          return Dt.data = [], dn.storeTable(Ie, rt), pt && (wt = pt(wt)), wt;
        }, dn.dropTable = function(Ie, rt, ft, pt) {
          var wt = 1, Bt = H.databases[Ie].lsdbid, At;
          if (H.options.autocommit ? At = dn.get(Bt) : At = H.databases[Ie], !ft && !At.tables[rt]) throw new Error('Cannot drop table "' + rt + '" in localStorage, because it does not exist');
          return delete At.tables[rt], dn.set(Bt, At), dn.removeTable(Ie, rt), pt && (wt = pt(wt)), wt;
        }, dn.fromTable = function(Ie, rt, ft, pt, wt) {
          H.databases[Ie].lsdbid;
          var Bt = dn.restoreTable(Ie, rt).data;
          return ft && (Bt = ft(Bt, pt, wt)), Bt;
        }, dn.intoTable = function(Ie, rt, ft, pt, wt) {
          H.databases[Ie].lsdbid;
          var Bt = ft.length, At = dn.restoreTable(Ie, rt);
          for (var Dt in At.identities) {
            var Ot = At.identities[Dt];
            for (var un in ft) ft[un][Dt] = Ot.value, Ot.value += Ot.step;
          }
          return At.data || (At.data = []), At.data = At.data.concat(ft), dn.storeTable(Ie, rt), wt && (Bt = wt(Bt)), Bt;
        }, dn.loadTableData = function(Ie, rt) {
          H.databases[Ie], H.databases[Ie].lsdbid, dn.restoreTable(Ie, rt);
        }, dn.saveTableData = function(Ie, rt) {
          var ft = H.databases[Ie], pt = H.databases[Ie].lsdbid;
          dn.storeTable(pt, rt), ft.tables[rt].data = void 0;
        }, dn.commit = function(Ie, rt) {
          var ft = H.databases[Ie], pt = H.databases[Ie].lsdbid, wt = { databaseid: pt, tables: {} };
          if (ft.tables) for (var Bt in ft.tables) wt.tables[Bt] = !0, dn.storeTable(Ie, Bt);
          return dn.set(pt, wt), rt ? rt(1) : 1;
        }, dn.begin = dn.commit, dn.rollback = function(Ie, rt) {
        };
        var Ht = H.engines.SQLITE = function() {
        };
        Ht.createDatabase = function(Ie, rt, ft, pt, wt) {
          throw new Error("Connot create SQLITE database in memory. Attach it.");
        }, Ht.dropDatabase = function(Ie) {
          throw new Error("This is impossible to drop SQLite database. Detach it.");
        }, Ht.attachDatabase = function(Ie, rt, ft, pt, wt) {
          var Bt = 1;
          if (H.databases[rt]) throw new Error('Unable to attach database as "' + rt + '" because it already exists');
          if (ft[0] && ft[0] instanceof ze.StringValue || ft[0] instanceof ze.ParamValue) {
            if (ft[0] instanceof ze.StringValue) var At = ft[0].value;
            else if (ft[0] instanceof ze.ParamValue) var At = pt[ft[0].param];
            return H.utils.loadBinaryFile(At, !0, function(Dt) {
              var Ot = new H.Database(rt || Ie);
              Ot.engineid = "SQLITE", Ot.sqldbid = Ie;
              var un = Ot.sqldb = new SQL.Database(Dt);
              Ot.tables = [];
              var yn = un.exec("SELECT * FROM sqlite_master WHERE type='table'")[0].values;
              yn.forEach(function(_n) {
                Ot.tables[_n[1]] = {};
                var gn = Ot.tables[_n[1]].columns = [], bn = H.parse(_n[4]), an = bn.statements[0].columns;
                an && an.length > 0 && an.forEach(function(Zt) {
                  gn.push(Zt);
                });
              }), wt(1);
            }, function(Dt) {
              throw new Error('Cannot open SQLite database file "' + ft[0].value + '"');
            }), Bt;
          } else throw new Error("Cannot attach SQLite database without a file");
        }, Ht.fromTable = function(Ie, rt, ft, pt, wt) {
          var Bt = H.databases[Ie].sqldb.exec("SELECT * FROM " + rt), At = wt.sources[pt].columns = [];
          Bt[0].columns.length > 0 && Bt[0].columns.forEach(function(Ot) {
            At.push({ columnid: Ot });
          });
          var Dt = [];
          Bt[0].values.length > 0 && Bt[0].values.forEach(function(Ot) {
            var un = {};
            At.forEach(function(yn, _n) {
              un[yn.columnid] = Ot[_n];
            }), Dt.push(un);
          }), ft && ft(Dt, pt, wt);
        }, Ht.intoTable = function(Ie, rt, ft, pt, wt) {
          for (var Bt = H.databases[Ie].sqldb, At = 0, Dt = ft.length; At < Dt; At++) {
            var Ot = "INSERT INTO " + rt + " (", un = ft[At], yn = Object.keys(un);
            Ot += yn.join(","), Ot += ") VALUES (", Ot += yn.map(function(gn) {
              var bn = un[gn];
              return typeof bn == "string" && (bn = "'" + bn + "'"), bn;
            }).join(","), Ot += ")", Bt.exec(Ot);
          }
          var _n = Dt;
          return wt && wt(_n), _n;
        };
        var jt = H.engines.FILESTORAGE = H.engines.FILE = function() {
        };
        if (jt.createDatabase = function(Ie, rt, ft, pt, wt) {
          var Bt = 1, At = rt[0].value;
          return H.utils.fileExists(At, function(Dt) {
            if (Dt) {
              if (ft) return Bt = 0, wt && (Bt = wt(Bt)), Bt;
              throw new Error("Cannot create new database file, because it already exists");
            } else {
              var Ot = { tables: {} };
              H.utils.saveFile(At, JSON.stringify(Ot), function(un) {
                wt && (Bt = wt(Bt));
              });
            }
          }), Bt;
        }, jt.dropDatabase = function(Ie, rt, ft) {
          var pt, wt = "";
          if (typeof Ie == "object" && Ie.value) wt = Ie.value;
          else {
            var Bt = H.databases[Ie] || {};
            wt = Bt.filename || "", delete H.databases[Ie];
          }
          return H.utils.fileExists(wt, function(At) {
            if (At) pt = 1, H.utils.deleteFile(wt, function() {
              pt = 1, ft && (pt = ft(pt));
            });
            else {
              if (!rt) throw new Error("Cannot drop database file, because it does not exist");
              pt = 0, ft && (pt = ft(pt));
            }
          }), pt;
        }, jt.attachDatabase = function(Ie, rt, ft, pt, wt) {
          var Bt = 1;
          if (H.databases[rt]) throw new Error('Unable to attach database as "' + rt + '" because it already exists');
          var At = new H.Database(rt || Ie);
          return At.engineid = "FILESTORAGE", At.filename = ft[0].value, ce(At.filename, !!wt, function(Dt) {
            try {
              At.data = JSON.parse(Dt);
            } catch {
              throw new Error("Data in FileStorage database are corrupted");
            }
            if (At.tables = At.data.tables, !H.options.autocommit && At.tables) for (var Ot in At.tables) At.tables[Ot].data = At.data[Ot];
            wt && (Bt = wt(Bt));
          }), Bt;
        }, jt.createTable = function(Ie, rt, ft, pt) {
          var wt = H.databases[Ie], Bt = wt.data[rt], At = 1;
          if (Bt && !ft) throw new Error('Table "' + rt + '" alsready exists in the database "' + fsdbid + '"');
          var Dt = H.databases[Ie].tables[rt];
          return wt.data.tables[rt] = { columns: Dt.columns }, wt.data[rt] = [], jt.updateFile(Ie), pt && pt(At), At;
        }, jt.updateFile = function(Ie) {
          var rt = H.databases[Ie];
          if (rt.issaving) {
            rt.postsave = !0;
            return;
          }
          rt.issaving = !0, rt.postsave = !1, H.utils.saveFile(rt.filename, JSON.stringify(rt.data), function() {
            rt.issaving = !1, rt.postsave && setTimeout(function() {
              jt.updateFile(Ie);
            }, 50);
          });
        }, jt.dropTable = function(Ie, rt, ft, pt) {
          var wt = 1, Bt = H.databases[Ie];
          if (!ft && !Bt.tables[rt]) throw new Error('Cannot drop table "' + rt + '" in fileStorage, because it does not exist');
          return delete Bt.tables[rt], delete Bt.data.tables[rt], delete Bt.data[rt], jt.updateFile(Ie), pt && pt(wt), wt;
        }, jt.fromTable = function(Ie, rt, ft, pt, wt) {
          var Bt = H.databases[Ie], At = Bt.data[rt];
          return ft && (At = ft(At, pt, wt)), At;
        }, jt.intoTable = function(Ie, rt, ft, pt, wt) {
          var Bt = H.databases[Ie], At = ft.length, Dt = Bt.data[rt];
          return Dt || (Dt = []), Bt.data[rt] = Dt.concat(ft), jt.updateFile(Ie), wt && wt(At), At;
        }, jt.loadTableData = function(Ie, rt) {
          var ft = H.databases[Ie];
          ft.tables[rt].data = ft.data[rt];
        }, jt.saveTableData = function(Ie, rt) {
          var ft = H.databases[Ie];
          ft.data[rt] = ft.tables[rt].data, ft.tables[rt].data = null, jt.updateFile(Ie);
        }, jt.commit = function(Ie, rt) {
          var ft = H.databases[Ie];
          if (ft.tables) for (var pt in ft.tables) ft.data.tables[pt] = { columns: ft.tables[pt].columns }, ft.data[pt] = ft.tables[pt].data;
          return jt.updateFile(Ie), rt ? rt(1) : 1;
        }, jt.begin = jt.commit, jt.rollback = function(Ie, rt) {
          var ft = 1, pt = H.databases[Ie];
          pt.dbversion++, wt();
          function wt() {
            setTimeout(function() {
              if (pt.issaving) return wt();
              H.loadFile(pt.filename, !!rt, function(Bt) {
                pt.data = Bt, pt.tables = {};
                for (var At in pt.data.tables) {
                  var Dt = new H.Table({ columns: pt.data.tables[At].columns });
                  we(Dt, pt.data.tables[At]), pt.tables[At] = Dt, H.options.autocommit || (pt.tables[At].data = pt.data[At]), pt.tables[At].indexColumns();
                }
                delete H.databases[Ie], H.databases[Ie] = new H.Database(Ie), we(H.databases[Ie], pt), H.databases[Ie].engineid = "FILESTORAGE", H.databases[Ie].filename = pt.filename, rt && (ft = rt(ft));
              });
            }, 100);
          }
        }, J.isBrowser && !J.isWebWorker) {
          if (H = H || !1, !H) throw new Error("alasql was not found");
          H.worker = function() {
            throw new Error("Can find webworker in this enviroment");
          }, typeof Worker < "u" && (H.worker = function(Ie, rt, ft) {
            if (Ie === !0 && (Ie = void 0), typeof Ie > "u") {
              for (var pt = document.getElementsByTagName("script"), wt = 0; wt < pt.length; wt++) if (pt[wt].src.substr(-16).toLowerCase() === "alasql-worker.js") {
                Ie = pt[wt].src.substr(0, pt[wt].src.length - 16) + "alasql.js";
                break;
              } else if (pt[wt].src.substr(-20).toLowerCase() === "alasql-worker.min.js") {
                Ie = pt[wt].src.substr(0, pt[wt].src.length - 20) + "alasql.min.js";
                break;
              } else if (pt[wt].src.substr(-9).toLowerCase() === "alasql.js") {
                Ie = pt[wt].src;
                break;
              } else if (pt[wt].src.substr(-13).toLowerCase() === "alasql.min.js") {
                Ie = pt[wt].src.substr(0, pt[wt].src.length - 13) + "alasql.min.js";
                break;
              }
            }
            if (typeof Ie > "u") throw new Error("Path to alasql.js is not specified");
            if (Ie !== !1) {
              var Bt = "importScripts('";
              Bt += Ie, Bt += "');self.onmessage = function(event) {alasql(event.data.sql,event.data.params, function(data){postMessage({id:event.data.id, data:data});});}";
              var At = new Blob([Bt], { type: "text/plain" });
              if (H.webworker = new Worker(URL.createObjectURL(At)), H.webworker.onmessage = function(Ot) {
                var un = Ot.data.id;
                H.buffer[un](Ot.data.data), delete H.buffer[un];
              }, H.webworker.onerror = function(Ot) {
                throw Ot;
              }, arguments.length > 1) {
                var Dt = "REQUIRE " + rt.map(function(Ot) {
                  return '"' + Ot + '"';
                }).join(",");
                H(Dt, [], ft);
              }
            } else if (Ie === !1) {
              delete H.webworker;
              return;
            }
          });
          /*! @source http://purl.eligrey.com/github/FileSaver.js/blob/master/FileSaver.js */
          var cn = cn || function(Ie) {
            if (!(typeof Ie > "u" || typeof navigator < "u" && /MSIE [1-9]\./.test(navigator.userAgent))) {
              var rt = Ie.document, ft = function() {
                return Ie.URL || Ie.webkitURL || Ie;
              }, pt = rt.createElementNS("http://www.w3.org/1999/xhtml", "a"), wt = "download" in pt, Bt = function(Ln) {
                var In = new MouseEvent("click");
                Ln.dispatchEvent(In);
              }, At = /constructor/i.test(Ie.HTMLElement) || Ie.safari, Dt = /CriOS\/[\d]+/.test(navigator.userAgent), Ot = function(Ln) {
                (Ie.setImmediate || Ie.setTimeout)(function() {
                  throw Ln;
                }, 0);
              }, un = "application/octet-stream", yn = 1e3 * 40, _n = function(Ln) {
                var In = function() {
                  typeof Ln == "string" ? ft().revokeObjectURL(Ln) : Ln.remove();
                };
                setTimeout(In, yn);
              }, gn = function(Ln, In, kn) {
                In = [].concat(In);
                for (var er = In.length; er--; ) {
                  var jn = Ln["on" + In[er]];
                  if (typeof jn == "function") try {
                    jn.call(Ln, kn || Ln);
                  } catch (ur) {
                    Ot(ur);
                  }
                }
              }, bn = function(Ln) {
                return /^\s*(?:text\/\S*|application\/xml|\S*\/\S*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(Ln.type) ? new Blob(["\uFEFF", Ln], { type: Ln.type }) : Ln;
              }, an = function(Ln, In, kn) {
                kn || (Ln = bn(Ln));
                var er = this, jn = Ln.type, ur = jn === un, Kn, Jn = function() {
                  gn(er, "writestart progress write writeend".split(" "));
                }, Lr = function() {
                  if ((Dt || ur && At) && Ie.FileReader) {
                    var mr = new FileReader();
                    mr.onloadend = function() {
                      var Yr = Dt ? mr.result : mr.result.replace(/^data:[^;]*;/, "data:attachment/file;"), jr = Ie.open(Yr, "_blank");
                      jr || (Ie.location.href = Yr), Yr = void 0, er.readyState = er.DONE, Jn();
                    }, mr.readAsDataURL(Ln), er.readyState = er.INIT;
                    return;
                  }
                  if (Kn || (Kn = ft().createObjectURL(Ln)), ur) Ie.location.href = Kn;
                  else {
                    var Ar = Ie.open(Kn, "_blank");
                    Ar || (Ie.location.href = Kn);
                  }
                  er.readyState = er.DONE, Jn(), _n(Kn);
                };
                if (er.readyState = er.INIT, wt) {
                  Kn = ft().createObjectURL(Ln), setTimeout(function() {
                    pt.href = Kn, pt.download = In, Bt(pt), Jn(), _n(Kn), er.readyState = er.DONE;
                  });
                  return;
                }
                Lr();
              }, Zt = an.prototype, Mn = function(Ln, In, kn) {
                return new an(Ln, In || Ln.name || "download", kn);
              };
              return typeof navigator < "u" && navigator.msSaveOrOpenBlob ? function(Ln, In, kn) {
                return In = In || Ln.name || "download", kn || (Ln = bn(Ln)), navigator.msSaveOrOpenBlob(Ln, In);
              } : (Zt.abort = function() {
              }, Zt.readyState = Zt.INIT = 0, Zt.WRITING = 1, Zt.DONE = 2, Zt.error = Zt.onwritestart = Zt.onprogress = Zt.onwrite = Zt.onabort = Zt.onerror = Zt.onwriteend = null, Mn);
            }
          }(typeof self < "u" && self || typeof window < "u" && window || this.content);
          O.exports && (O.exports.saveAs = cn), (J.isCordova || J.isMeteorServer || J.isNode) && console.log("It looks like you are using the browser version of AlaSQL. Please use the alasql.fs.js file instead."), H.utils.saveAs = cn;
        }
        return new Pe("alasql"), H.use("alasql"), H;
      });
    }(alasql_min$1, alasql_min$1.exports)), alasql_min$1.exports;
  }
  var alasql_minExports = requireAlasql_min();
  const alasql = /* @__PURE__ */ getDefaultExportFromCjs$1(alasql_minExports), _hoisted_1$4 = ["data-cy"], _hoisted_2$3 = {
    key: 0,
    class: "text-xs pl-2 py-2 font-semibold"
  }, _hoisted_3$2 = ["data-cy", "id", "onClick"], _sfc_main$4 = /* @__PURE__ */ defineComponent({
    __name: "ARDropdown",
    props: {
      name: {},
      sections: {},
      orientation: {}
    },
    setup(O) {
      const _ = O, H = ref$1(!1), V = () => void (H.value = !H.value), q = () => void setTimeout(() => H.value = !1, 200);
      return (G, J) => (openBlock(), createElementBlock("div", {
        "data-cy": `dropdown-${_.name.toLowerCase()}`,
        style: { zIndex: 5 },
        class: "relative inline-block text-left"
      }, [
        createBaseVNode("div", null, [
          createBaseVNode("button", {
            type: "button",
            class: "ar-btn-sm px-1 sm:px-2 sm:px-2 py-0",
            id: "options-menu",
            onClick: V,
            onBlur: q
          }, [
            createTextVNode(toDisplayString(_.name) + " ", 1),
            J[0] || (J[0] = createBaseVNode("svg", {
              class: "-mr-1 ml-1 h-5 w-5",
              xmlns: "http://www.w3.org/2000/svg",
              viewBox: "0 0 20 20",
              fill: "currentColor",
              "aria-hidden": "true"
            }, [
              createBaseVNode("path", {
                fillRule: "evenodd",
                d: "M5.293 7.293a1 1 0 011.414 0L10 10.586l3.293-3.293a1 1 0 111.414 1.414l-4 4a1 1 0 01-1.414 0l-4-4a1 1 0 010-1.414z",
                clipRule: "evenodd"
              })
            ], -1))
          ], 32)
        ]),
        createBaseVNode("div", {
          class: normalizeClass([
            "absolute  w-56 rounded-md shadow-lg bg-white ring-1 ring-black ring-opacity-5 divide-y divide-gray-100",
            {
              hidden: !H.value,
              "origin-top-left left-0": _.orientation === "left",
              "origin-top-right right-0": _.orientation !== "left"
            }
          ]),
          role: "menu",
          "aria-orientation": "vertical",
          "aria-labelledby": "options-menu"
        }, [
          (openBlock(!0), createElementBlock(Fragment, null, renderList(_.sections, (Z, te) => (openBlock(), createElementBlock("div", {
            class: "py-1",
            key: te
          }, [
            Z.title ? (openBlock(), createElementBlock("div", _hoisted_2$3, toDisplayString(Z.title), 1)) : createCommentVNode("", !0),
            (openBlock(!0), createElementBlock(Fragment, null, renderList(Z.options, (ne, ee) => (openBlock(), createElementBlock("a", {
              key: `${te}-${ee}`,
              "data-cy": `dropdown-option-${ne.id}`,
              id: ne.id,
              onClick: ne.onClick,
              class: "block px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 hover:text-gray-900 cursor-pointer",
              role: "menuitem"
            }, toDisplayString(ne.name), 9, _hoisted_3$2))), 128))
          ]))), 128))
        ], 2)
      ], 8, _hoisted_1$4));
    }
  });
  var numeral$2 = { exports: {} };
  /*! @preserve
   * numeral.js
   * version : 2.0.6
   * author : Adam Draper
   * license : MIT
   * http://adamwdraper.github.com/Numeral-js/
   */
  var numeral$1 = numeral$2.exports, hasRequiredNumeral;
  function requireNumeral() {
    return hasRequiredNumeral || (hasRequiredNumeral = 1, function(O) {
      (function(_, H) {
        O.exports ? O.exports = H() : _.numeral = H();
      })(numeral$1, function() {
        var _, H, V = "2.0.6", q = {}, G = {}, J = {
          currentLocale: "en",
          zeroFormat: null,
          nullFormat: null,
          defaultFormat: "0,0",
          scalePercentBy100: !0
        }, Z = {
          currentLocale: J.currentLocale,
          zeroFormat: J.zeroFormat,
          nullFormat: J.nullFormat,
          defaultFormat: J.defaultFormat,
          scalePercentBy100: J.scalePercentBy100
        };
        function te(ne, ee) {
          this._input = ne, this._value = ee;
        }
        return _ = function(ne) {
          var ee, ae, oe, se;
          if (_.isNumeral(ne))
            ee = ne.value();
          else if (ne === 0 || typeof ne > "u")
            ee = 0;
          else if (ne === null || H.isNaN(ne))
            ee = null;
          else if (typeof ne == "string")
            if (Z.zeroFormat && ne === Z.zeroFormat)
              ee = 0;
            else if (Z.nullFormat && ne === Z.nullFormat || !ne.replace(/[^0-9]+/g, "").length)
              ee = null;
            else {
              for (ae in q)
                if (se = typeof q[ae].regexps.unformat == "function" ? q[ae].regexps.unformat() : q[ae].regexps.unformat, se && ne.match(se)) {
                  oe = q[ae].unformat;
                  break;
                }
              oe = oe || _._.stringToNumber, ee = oe(ne);
            }
          else
            ee = Number(ne) || null;
          return new te(ne, ee);
        }, _.version = V, _.isNumeral = function(ne) {
          return ne instanceof te;
        }, _._ = H = {
          // formats numbers separators, decimals places, signs, abbreviations
          numberToFormat: function(ne, ee, ae) {
            var oe = G[_.options.currentLocale], se = !1, ce = !1, le = 0, ue = "", de = 1e12, fe = 1e9, he = 1e6, me = 1e3, pe = "", ye = !1, ge, ve, be, Ce, Se, we, Ee;
            if (ne = ne || 0, ve = Math.abs(ne), _._.includes(ee, "(") ? (se = !0, ee = ee.replace(/[\(|\)]/g, "")) : (_._.includes(ee, "+") || _._.includes(ee, "-")) && (Se = _._.includes(ee, "+") ? ee.indexOf("+") : ne < 0 ? ee.indexOf("-") : -1, ee = ee.replace(/[\+|\-]/g, "")), _._.includes(ee, "a") && (ge = ee.match(/a(k|m|b|t)?/), ge = ge ? ge[1] : !1, _._.includes(ee, " a") && (ue = " "), ee = ee.replace(new RegExp(ue + "a[kmbt]?"), ""), ve >= de && !ge || ge === "t" ? (ue += oe.abbreviations.trillion, ne = ne / de) : ve < de && ve >= fe && !ge || ge === "b" ? (ue += oe.abbreviations.billion, ne = ne / fe) : ve < fe && ve >= he && !ge || ge === "m" ? (ue += oe.abbreviations.million, ne = ne / he) : (ve < he && ve >= me && !ge || ge === "k") && (ue += oe.abbreviations.thousand, ne = ne / me)), _._.includes(ee, "[.]") && (ce = !0, ee = ee.replace("[.]", ".")), be = ne.toString().split(".")[0], Ce = ee.split(".")[1], we = ee.indexOf(","), le = (ee.split(".")[0].split(",")[0].match(/0/g) || []).length, Ce ? (_._.includes(Ce, "[") ? (Ce = Ce.replace("]", ""), Ce = Ce.split("["), pe = _._.toFixed(ne, Ce[0].length + Ce[1].length, ae, Ce[1].length)) : pe = _._.toFixed(ne, Ce.length, ae), be = pe.split(".")[0], _._.includes(pe, ".") ? pe = oe.delimiters.decimal + pe.split(".")[1] : pe = "", ce && Number(pe.slice(1)) === 0 && (pe = "")) : be = _._.toFixed(ne, 0, ae), ue && !ge && Number(be) >= 1e3 && ue !== oe.abbreviations.trillion)
              switch (be = String(Number(be) / 1e3), ue) {
                case oe.abbreviations.thousand:
                  ue = oe.abbreviations.million;
                  break;
                case oe.abbreviations.million:
                  ue = oe.abbreviations.billion;
                  break;
                case oe.abbreviations.billion:
                  ue = oe.abbreviations.trillion;
                  break;
              }
            if (_._.includes(be, "-") && (be = be.slice(1), ye = !0), be.length < le)
              for (var xe = le - be.length; xe > 0; xe--)
                be = "0" + be;
            return we > -1 && (be = be.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1" + oe.delimiters.thousands)), ee.indexOf(".") === 0 && (be = ""), Ee = be + pe + (ue || ""), se ? Ee = (se && ye ? "(" : "") + Ee + (se && ye ? ")" : "") : Se >= 0 ? Ee = Se === 0 ? (ye ? "-" : "+") + Ee : Ee + (ye ? "-" : "+") : ye && (Ee = "-" + Ee), Ee;
          },
          // unformats numbers separators, decimals places, signs, abbreviations
          stringToNumber: function(ne) {
            var ee = G[Z.currentLocale], ae = ne, oe = {
              thousand: 3,
              million: 6,
              billion: 9,
              trillion: 12
            }, se, ce, le;
            if (Z.zeroFormat && ne === Z.zeroFormat)
              ce = 0;
            else if (Z.nullFormat && ne === Z.nullFormat || !ne.replace(/[^0-9]+/g, "").length)
              ce = null;
            else {
              ce = 1, ee.delimiters.decimal !== "." && (ne = ne.replace(/\./g, "").replace(ee.delimiters.decimal, "."));
              for (se in oe)
                if (le = new RegExp("[^a-zA-Z]" + ee.abbreviations[se] + "(?:\\)|(\\" + ee.currency.symbol + ")?(?:\\))?)?$"), ae.match(le)) {
                  ce *= Math.pow(10, oe[se]);
                  break;
                }
              ce *= (ne.split("-").length + Math.min(ne.split("(").length - 1, ne.split(")").length - 1)) % 2 ? 1 : -1, ne = ne.replace(/[^0-9\.]+/g, ""), ce *= Number(ne);
            }
            return ce;
          },
          isNaN: function(ne) {
            return typeof ne == "number" && isNaN(ne);
          },
          includes: function(ne, ee) {
            return ne.indexOf(ee) !== -1;
          },
          insert: function(ne, ee, ae) {
            return ne.slice(0, ae) + ee + ne.slice(ae);
          },
          reduce: function(ne, ee) {
            if (this === null)
              throw new TypeError("Array.prototype.reduce called on null or undefined");
            if (typeof ee != "function")
              throw new TypeError(ee + " is not a function");
            var ae = Object(ne), oe = ae.length >>> 0, se = 0, ce;
            if (arguments.length === 3)
              ce = arguments[2];
            else {
              for (; se < oe && !(se in ae); )
                se++;
              if (se >= oe)
                throw new TypeError("Reduce of empty array with no initial value");
              ce = ae[se++];
            }
            for (; se < oe; se++)
              se in ae && (ce = ee(ce, ae[se], se, ae));
            return ce;
          },
          /**
           * Computes the multiplier necessary to make x >= 1,
           * effectively eliminating miscalculations caused by
           * finite precision.
           */
          multiplier: function(ne) {
            var ee = ne.toString().split(".");
            return ee.length < 2 ? 1 : Math.pow(10, ee[1].length);
          },
          /**
           * Given a variable number of arguments, returns the maximum
           * multiplier that must be used to normalize an operation involving
           * all of them.
           */
          correctionFactor: function() {
            var ne = Array.prototype.slice.call(arguments);
            return ne.reduce(function(ee, ae) {
              var oe = H.multiplier(ae);
              return ee > oe ? ee : oe;
            }, 1);
          },
          /**
           * Implementation of toFixed() that treats floats more like decimals
           *
           * Fixes binary rounding issues (eg. (0.615).toFixed(2) === '0.61') that present
           * problems for accounting- and finance-related software.
           */
          toFixed: function(ne, ee, ae, oe) {
            var se = ne.toString().split("."), ce = ee - (oe || 0), le, ue, de, fe;
            return se.length === 2 ? le = Math.min(Math.max(se[1].length, ce), ee) : le = ce, de = Math.pow(10, le), fe = (ae(ne + "e+" + le) / de).toFixed(le), oe > ee - le && (ue = new RegExp("\\.?0{1," + (oe - (ee - le)) + "}$"), fe = fe.replace(ue, "")), fe;
          }
        }, _.options = Z, _.formats = q, _.locales = G, _.locale = function(ne) {
          return ne && (Z.currentLocale = ne.toLowerCase()), Z.currentLocale;
        }, _.localeData = function(ne) {
          if (!ne)
            return G[Z.currentLocale];
          if (ne = ne.toLowerCase(), !G[ne])
            throw new Error("Unknown locale : " + ne);
          return G[ne];
        }, _.reset = function() {
          for (var ne in J)
            Z[ne] = J[ne];
        }, _.zeroFormat = function(ne) {
          Z.zeroFormat = typeof ne == "string" ? ne : null;
        }, _.nullFormat = function(ne) {
          Z.nullFormat = typeof ne == "string" ? ne : null;
        }, _.defaultFormat = function(ne) {
          Z.defaultFormat = typeof ne == "string" ? ne : "0.0";
        }, _.register = function(ne, ee, ae) {
          if (ee = ee.toLowerCase(), this[ne + "s"][ee])
            throw new TypeError(ee + " " + ne + " already registered.");
          return this[ne + "s"][ee] = ae, ae;
        }, _.validate = function(ne, ee) {
          var ae, oe, se, ce, le, ue, de, fe;
          if (typeof ne != "string" && (ne += "", console.warn && console.warn("Numeral.js: Value is not string. It has been co-erced to: ", ne)), ne = ne.trim(), ne.match(/^\d+$/))
            return !0;
          if (ne === "")
            return !1;
          try {
            de = _.localeData(ee);
          } catch {
            de = _.localeData(_.locale());
          }
          return se = de.currency.symbol, le = de.abbreviations, ae = de.delimiters.decimal, de.delimiters.thousands === "." ? oe = "\\." : oe = de.delimiters.thousands, fe = ne.match(/^[^\d]+/), fe !== null && (ne = ne.substr(1), fe[0] !== se) || (fe = ne.match(/[^\d]+$/), fe !== null && (ne = ne.slice(0, -1), fe[0] !== le.thousand && fe[0] !== le.million && fe[0] !== le.billion && fe[0] !== le.trillion)) ? !1 : (ue = new RegExp(oe + "{2}"), ne.match(/[^\d.,]/g) ? !1 : (ce = ne.split(ae), ce.length > 2 ? !1 : ce.length < 2 ? !!ce[0].match(/^\d+.*\d$/) && !ce[0].match(ue) : ce[0].length === 1 ? !!ce[0].match(/^\d+$/) && !ce[0].match(ue) && !!ce[1].match(/^\d+$/) : !!ce[0].match(/^\d+.*\d$/) && !ce[0].match(ue) && !!ce[1].match(/^\d+$/)));
        }, _.fn = te.prototype = {
          clone: function() {
            return _(this);
          },
          format: function(ne, ee) {
            var ae = this._value, oe = ne || Z.defaultFormat, se, ce, le;
            if (ee = ee || Math.round, ae === 0 && Z.zeroFormat !== null)
              ce = Z.zeroFormat;
            else if (ae === null && Z.nullFormat !== null)
              ce = Z.nullFormat;
            else {
              for (se in q)
                if (oe.match(q[se].regexps.format)) {
                  le = q[se].format;
                  break;
                }
              le = le || _._.numberToFormat, ce = le(ae, oe, ee);
            }
            return ce;
          },
          value: function() {
            return this._value;
          },
          input: function() {
            return this._input;
          },
          set: function(ne) {
            return this._value = Number(ne), this;
          },
          add: function(ne) {
            var ee = H.correctionFactor.call(null, this._value, ne);
            function ae(oe, se, ce, le) {
              return oe + Math.round(ee * se);
            }
            return this._value = H.reduce([this._value, ne], ae, 0) / ee, this;
          },
          subtract: function(ne) {
            var ee = H.correctionFactor.call(null, this._value, ne);
            function ae(oe, se, ce, le) {
              return oe - Math.round(ee * se);
            }
            return this._value = H.reduce([ne], ae, Math.round(this._value * ee)) / ee, this;
          },
          multiply: function(ne) {
            function ee(ae, oe, se, ce) {
              var le = H.correctionFactor(ae, oe);
              return Math.round(ae * le) * Math.round(oe * le) / Math.round(le * le);
            }
            return this._value = H.reduce([this._value, ne], ee, 1), this;
          },
          divide: function(ne) {
            function ee(ae, oe, se, ce) {
              var le = H.correctionFactor(ae, oe);
              return Math.round(ae * le) / Math.round(oe * le);
            }
            return this._value = H.reduce([this._value, ne], ee), this;
          },
          difference: function(ne) {
            return Math.abs(_(this._value).subtract(ne).value());
          }
        }, _.register("locale", "en", {
          delimiters: {
            thousands: ",",
            decimal: "."
          },
          abbreviations: {
            thousand: "k",
            million: "m",
            billion: "b",
            trillion: "t"
          },
          ordinal: function(ne) {
            var ee = ne % 10;
            return ~~(ne % 100 / 10) === 1 ? "th" : ee === 1 ? "st" : ee === 2 ? "nd" : ee === 3 ? "rd" : "th";
          },
          currency: {
            symbol: "$"
          }
        }), function() {
          _.register("format", "bps", {
            regexps: {
              format: /(BPS)/,
              unformat: /(BPS)/
            },
            format: function(ne, ee, ae) {
              var oe = _._.includes(ee, " BPS") ? " " : "", se;
              return ne = ne * 1e4, ee = ee.replace(/\s?BPS/, ""), se = _._.numberToFormat(ne, ee, ae), _._.includes(se, ")") ? (se = se.split(""), se.splice(-1, 0, oe + "BPS"), se = se.join("")) : se = se + oe + "BPS", se;
            },
            unformat: function(ne) {
              return +(_._.stringToNumber(ne) * 1e-4).toFixed(15);
            }
          });
        }(), function() {
          var ne = {
            base: 1e3,
            suffixes: ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"]
          }, ee = {
            base: 1024,
            suffixes: ["B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"]
          }, ae = ne.suffixes.concat(ee.suffixes.filter(function(se) {
            return ne.suffixes.indexOf(se) < 0;
          })), oe = ae.join("|");
          oe = "(" + oe.replace("B", "B(?!PS)") + ")", _.register("format", "bytes", {
            regexps: {
              format: /([0\s]i?b)/,
              unformat: new RegExp(oe)
            },
            format: function(se, ce, le) {
              var ue, de = _._.includes(ce, "ib") ? ee : ne, fe = _._.includes(ce, " b") || _._.includes(ce, " ib") ? " " : "", he, me, pe;
              for (ce = ce.replace(/\s?i?b/, ""), he = 0; he <= de.suffixes.length; he++)
                if (me = Math.pow(de.base, he), pe = Math.pow(de.base, he + 1), se === null || se === 0 || se >= me && se < pe) {
                  fe += de.suffixes[he], me > 0 && (se = se / me);
                  break;
                }
              return ue = _._.numberToFormat(se, ce, le), ue + fe;
            },
            unformat: function(se) {
              var ce = _._.stringToNumber(se), le, ue;
              if (ce) {
                for (le = ne.suffixes.length - 1; le >= 0; le--) {
                  if (_._.includes(se, ne.suffixes[le])) {
                    ue = Math.pow(ne.base, le);
                    break;
                  }
                  if (_._.includes(se, ee.suffixes[le])) {
                    ue = Math.pow(ee.base, le);
                    break;
                  }
                }
                ce *= ue || 1;
              }
              return ce;
            }
          });
        }(), function() {
          _.register("format", "currency", {
            regexps: {
              format: /(\$)/
            },
            format: function(ne, ee, ae) {
              var oe = _.locales[_.options.currentLocale], se = {
                before: ee.match(/^([\+|\-|\(|\s|\$]*)/)[0],
                after: ee.match(/([\+|\-|\)|\s|\$]*)$/)[0]
              }, ce, le, ue;
              for (ee = ee.replace(/\s?\$\s?/, ""), ce = _._.numberToFormat(ne, ee, ae), ne >= 0 ? (se.before = se.before.replace(/[\-\(]/, ""), se.after = se.after.replace(/[\-\)]/, "")) : ne < 0 && !_._.includes(se.before, "-") && !_._.includes(se.before, "(") && (se.before = "-" + se.before), ue = 0; ue < se.before.length; ue++)
                switch (le = se.before[ue], le) {
                  case "$":
                    ce = _._.insert(ce, oe.currency.symbol, ue);
                    break;
                  case " ":
                    ce = _._.insert(ce, " ", ue + oe.currency.symbol.length - 1);
                    break;
                }
              for (ue = se.after.length - 1; ue >= 0; ue--)
                switch (le = se.after[ue], le) {
                  case "$":
                    ce = ue === se.after.length - 1 ? ce + oe.currency.symbol : _._.insert(ce, oe.currency.symbol, -(se.after.length - (1 + ue)));
                    break;
                  case " ":
                    ce = ue === se.after.length - 1 ? ce + " " : _._.insert(ce, " ", -(se.after.length - (1 + ue) + oe.currency.symbol.length - 1));
                    break;
                }
              return ce;
            }
          });
        }(), function() {
          _.register("format", "exponential", {
            regexps: {
              format: /(e\+|e-)/,
              unformat: /(e\+|e-)/
            },
            format: function(ne, ee, ae) {
              var oe, se = typeof ne == "number" && !_._.isNaN(ne) ? ne.toExponential() : "0e+0", ce = se.split("e");
              return ee = ee.replace(/e[\+|\-]{1}0/, ""), oe = _._.numberToFormat(Number(ce[0]), ee, ae), oe + "e" + ce[1];
            },
            unformat: function(ne) {
              var ee = _._.includes(ne, "e+") ? ne.split("e+") : ne.split("e-"), ae = Number(ee[0]), oe = Number(ee[1]);
              oe = _._.includes(ne, "e-") ? oe *= -1 : oe;
              function se(ce, le, ue, de) {
                var fe = _._.correctionFactor(ce, le), he = ce * fe * (le * fe) / (fe * fe);
                return he;
              }
              return _._.reduce([ae, Math.pow(10, oe)], se, 1);
            }
          });
        }(), function() {
          _.register("format", "ordinal", {
            regexps: {
              format: /(o)/
            },
            format: function(ne, ee, ae) {
              var oe = _.locales[_.options.currentLocale], se, ce = _._.includes(ee, " o") ? " " : "";
              return ee = ee.replace(/\s?o/, ""), ce += oe.ordinal(ne), se = _._.numberToFormat(ne, ee, ae), se + ce;
            }
          });
        }(), function() {
          _.register("format", "percentage", {
            regexps: {
              format: /(%)/,
              unformat: /(%)/
            },
            format: function(ne, ee, ae) {
              var oe = _._.includes(ee, " %") ? " " : "", se;
              return _.options.scalePercentBy100 && (ne = ne * 100), ee = ee.replace(/\s?\%/, ""), se = _._.numberToFormat(ne, ee, ae), _._.includes(se, ")") ? (se = se.split(""), se.splice(-1, 0, oe + "%"), se = se.join("")) : se = se + oe + "%", se;
            },
            unformat: function(ne) {
              var ee = _._.stringToNumber(ne);
              return _.options.scalePercentBy100 ? ee * 0.01 : ee;
            }
          });
        }(), function() {
          _.register("format", "time", {
            regexps: {
              format: /(:)/,
              unformat: /(:)/
            },
            format: function(ne, ee, ae) {
              var oe = Math.floor(ne / 60 / 60), se = Math.floor((ne - oe * 60 * 60) / 60), ce = Math.round(ne - oe * 60 * 60 - se * 60);
              return oe + ":" + (se < 10 ? "0" + se : se) + ":" + (ce < 10 ? "0" + ce : ce);
            },
            unformat: function(ne) {
              var ee = ne.split(":"), ae = 0;
              return ee.length === 3 ? (ae = ae + Number(ee[0]) * 60 * 60, ae = ae + Number(ee[1]) * 60, ae = ae + Number(ee[2])) : ee.length === 2 && (ae = ae + Number(ee[0]) * 60, ae = ae + Number(ee[1])), Number(ae);
            }
          });
        }(), _;
      });
    }(numeral$2)), numeral$2.exports;
  }
  var numeralExports = requireNumeral();
  const numeral = /* @__PURE__ */ getDefaultExportFromCjs$1(numeralExports), formatNumber$1 = (O) => numeral(O).format("0[.][0]a"), _hoisted_1$3 = { class: "px-2.5 py-0.5 mr-0 flex items-center inline-flex items-center md:rounded text-xs font-medium leading-4 bg-gray-200 text-gray-700" }, _hoisted_2$2 = { className: "hidden lg:inline" }, _sfc_main$3 = /* @__PURE__ */ defineComponent({
    __name: "DataTag",
    props: {
      value: {},
      unit: {},
      icon: {}
    },
    setup(O) {
      return (_, H) => (openBlock(), createElementBlock("div", _hoisted_1$3, [
        createBaseVNode("div", null, [
          createBaseVNode("i", {
            class: normalizeClass(`pr-2 fa ${_.icon}`)
          }, null, 2)
        ]),
        createBaseVNode("div", null, [
          createBaseVNode("span", null, toDisplayString(unref(formatNumber$1)(+_.value)), 1),
          createBaseVNode("span", _hoisted_2$2, toDisplayString(" ") + toDisplayString(_.unit), 1)
        ])
      ]));
    }
  }), _hoisted_1$2 = {
    class: "bg-gray-100 py-2",
    "data-cy": "block-datatable"
  }, _hoisted_2$1 = { class: "flex justify-between items-center flex-wrap" }, _hoisted_3$1 = { class: "flex justify-end md:space-x-2 ml-2" }, _hoisted_4$1 = {
    key: 0,
    class: "min-w-0 flex items-center pr-2 sm:divide-x flex-wrap"
  }, _hoisted_5$1 = { class: "pr-2 sm:flex hidden space-x-2" }, _sfc_main$2 = /* @__PURE__ */ defineComponent({
    __name: "Header",
    props: {
      previewMode: { type: Boolean },
      queryOpen: { type: Boolean },
      rows: {},
      columns: {},
      cells: {},
      getCsvText: { type: Function },
      downloadLocal: { type: Function }
    },
    emits: ["toggle-query-open"],
    setup(O, { emit: _ }) {
      const H = O, V = _, J = [...[
        {
          title: "Current State",
          options: [
            {
              name: "Copy CSV to clipboard",
              onClick: async () => ARClipboard.copyOnce(await H.getCsvText()),
              id: "copy-clipboard"
            },
            {
              name: "Download CSV",
              onClick: function(Z) {
                return function(...te) {
                  return Z.apply(this, te).catch((ne) => {
                    console.error(ne);
                  });
                };
              }(H.downloadLocal),
              id: "download-csv"
            }
          ]
        }
      ]];
      return (Z, te) => (openBlock(), createElementBlock("div", _hoisted_1$2, [
        createBaseVNode("div", _hoisted_2$1, [
          createBaseVNode("div", _hoisted_3$1, [
            createVNode(_sfc_main$3, {
              value: Z.rows,
              icon: "fa-bars",
              unit: "rows"
            }, null, 8, ["value"]),
            createVNode(_sfc_main$3, {
              value: Z.columns,
              icon: "fa-columns",
              unit: "columns"
            }, null, 8, ["value"]),
            createVNode(_sfc_main$3, {
              value: Z.cells,
              icon: "fa-th-large",
              unit: "cells"
            }, null, 8, ["value"])
          ]),
          Z.previewMode ? createCommentVNode("", !0) : (openBlock(), createElementBlock("div", _hoisted_4$1, [
            createBaseVNode("div", _hoisted_5$1, [
              createVNode(_sfc_main$P, {
                onClick: te[0] || (te[0] = (ne) => V("toggle-query-open")),
                icon: `fa ${Z.queryOpen ? "fa-caret-up" : "fa-caret-down"}`,
                disabled: Z.previewMode,
                "data-cy": "btn-open-query",
                class: "ar-btn-info"
              }, {
                default: withCtx(() => te[1] || (te[1] = [
                  createTextVNode(" Run SQL Query ")
                ])),
                _: 1
              }, 8, ["icon", "disabled"])
            ]),
            createVNode(_sfc_main$4, {
              name: "Export",
              sections: J
            })
          ]))
        ])
      ]));
    }
  }), DefaultBufferLength = 1024;
  let nextPropID = 0, Range$1 = class {
    constructor(_, H) {
      this.from = _, this.to = H;
    }
  };
  class NodeProp {
    /**
    Create a new node prop type.
    */
    constructor(_ = {}) {
      this.id = nextPropID++, this.perNode = !!_.perNode, this.deserialize = _.deserialize || (() => {
        throw new Error("This node type doesn't define a deserialize function");
      });
    }
    /**
    This is meant to be used with
    [`NodeSet.extend`](#common.NodeSet.extend) or
    [`LRParser.configure`](#lr.ParserConfig.props) to compute
    prop values for each node type in the set. Takes a [match
    object](#common.NodeType^match) or function that returns undefined
    if the node type doesn't get this prop, and the prop's value if
    it does.
    */
    add(_) {
      if (this.perNode)
        throw new RangeError("Can't add per-node props to node types");
      return typeof _ != "function" && (_ = NodeType.match(_)), (H) => {
        let V = _(H);
        return V === void 0 ? null : [this, V];
      };
    }
  }
  NodeProp.closedBy = new NodeProp({ deserialize: (O) => O.split(" ") });
  NodeProp.openedBy = new NodeProp({ deserialize: (O) => O.split(" ") });
  NodeProp.group = new NodeProp({ deserialize: (O) => O.split(" ") });
  NodeProp.isolate = new NodeProp({ deserialize: (O) => {
    if (O && O != "rtl" && O != "ltr" && O != "auto")
      throw new RangeError("Invalid value for isolate: " + O);
    return O || "auto";
  } });
  NodeProp.contextHash = new NodeProp({ perNode: !0 });
  NodeProp.lookAhead = new NodeProp({ perNode: !0 });
  NodeProp.mounted = new NodeProp({ perNode: !0 });
  class MountedTree {
    constructor(_, H, V) {
      this.tree = _, this.overlay = H, this.parser = V;
    }
    /**
    @internal
    */
    static get(_) {
      return _ && _.props && _.props[NodeProp.mounted.id];
    }
  }
  const noProps = /* @__PURE__ */ Object.create(null);
  class NodeType {
    /**
    @internal
    */
    constructor(_, H, V, q = 0) {
      this.name = _, this.props = H, this.id = V, this.flags = q;
    }
    /**
    Define a node type.
    */
    static define(_) {
      let H = _.props && _.props.length ? /* @__PURE__ */ Object.create(null) : noProps, V = (_.top ? 1 : 0) | (_.skipped ? 2 : 0) | (_.error ? 4 : 0) | (_.name == null ? 8 : 0), q = new NodeType(_.name || "", H, _.id, V);
      if (_.props) {
        for (let G of _.props)
          if (Array.isArray(G) || (G = G(q)), G) {
            if (G[0].perNode)
              throw new RangeError("Can't store a per-node prop on a node type");
            H[G[0].id] = G[1];
          }
      }
      return q;
    }
    /**
    Retrieves a node prop for this type. Will return `undefined` if
    the prop isn't present on this node.
    */
    prop(_) {
      return this.props[_.id];
    }
    /**
    True when this is the top node of a grammar.
    */
    get isTop() {
      return (this.flags & 1) > 0;
    }
    /**
    True when this node is produced by a skip rule.
    */
    get isSkipped() {
      return (this.flags & 2) > 0;
    }
    /**
    Indicates whether this is an error node.
    */
    get isError() {
      return (this.flags & 4) > 0;
    }
    /**
    When true, this node type doesn't correspond to a user-declared
    named node, for example because it is used to cache repetition.
    */
    get isAnonymous() {
      return (this.flags & 8) > 0;
    }
    /**
    Returns true when this node's name or one of its
    [groups](#common.NodeProp^group) matches the given string.
    */
    is(_) {
      if (typeof _ == "string") {
        if (this.name == _)
          return !0;
        let H = this.prop(NodeProp.group);
        return H ? H.indexOf(_) > -1 : !1;
      }
      return this.id == _;
    }
    /**
    Create a function from node types to arbitrary values by
    specifying an object whose property names are node or
    [group](#common.NodeProp^group) names. Often useful with
    [`NodeProp.add`](#common.NodeProp.add). You can put multiple
    names, separated by spaces, in a single property name to map
    multiple node names to a single value.
    */
    static match(_) {
      let H = /* @__PURE__ */ Object.create(null);
      for (let V in _)
        for (let q of V.split(" "))
          H[q] = _[V];
      return (V) => {
        for (let q = V.prop(NodeProp.group), G = -1; G < (q ? q.length : 0); G++) {
          let J = H[G < 0 ? V.name : q[G]];
          if (J)
            return J;
        }
      };
    }
  }
  NodeType.none = new NodeType(
    "",
    /* @__PURE__ */ Object.create(null),
    0,
    8
    /* NodeFlag.Anonymous */
  );
  class NodeSet {
    /**
    Create a set with the given types. The `id` property of each
    type should correspond to its position within the array.
    */
    constructor(_) {
      this.types = _;
      for (let H = 0; H < _.length; H++)
        if (_[H].id != H)
          throw new RangeError("Node type ids should correspond to array positions when creating a node set");
    }
    /**
    Create a copy of this set with some node properties added. The
    arguments to this method can be created with
    [`NodeProp.add`](#common.NodeProp.add).
    */
    extend(..._) {
      let H = [];
      for (let V of this.types) {
        let q = null;
        for (let G of _) {
          let J = G(V);
          J && (q || (q = Object.assign({}, V.props)), q[J[0].id] = J[1]);
        }
        H.push(q ? new NodeType(V.name, q, V.id, V.flags) : V);
      }
      return new NodeSet(H);
    }
  }
  const CachedNode = /* @__PURE__ */ new WeakMap(), CachedInnerNode = /* @__PURE__ */ new WeakMap();
  var IterMode;
  (function(O) {
    O[O.ExcludeBuffers = 1] = "ExcludeBuffers", O[O.IncludeAnonymous = 2] = "IncludeAnonymous", O[O.IgnoreMounts = 4] = "IgnoreMounts", O[O.IgnoreOverlays = 8] = "IgnoreOverlays";
  })(IterMode || (IterMode = {}));
  class Tree {
    /**
    Construct a new tree. See also [`Tree.build`](#common.Tree^build).
    */
    constructor(_, H, V, q, G) {
      if (this.type = _, this.children = H, this.positions = V, this.length = q, this.props = null, G && G.length) {
        this.props = /* @__PURE__ */ Object.create(null);
        for (let [J, Z] of G)
          this.props[typeof J == "number" ? J : J.id] = Z;
      }
    }
    /**
    @internal
    */
    toString() {
      let _ = MountedTree.get(this);
      if (_ && !_.overlay)
        return _.tree.toString();
      let H = "";
      for (let V of this.children) {
        let q = V.toString();
        q && (H && (H += ","), H += q);
      }
      return this.type.name ? (/\W/.test(this.type.name) && !this.type.isError ? JSON.stringify(this.type.name) : this.type.name) + (H.length ? "(" + H + ")" : "") : H;
    }
    /**
    Get a [tree cursor](#common.TreeCursor) positioned at the top of
    the tree. Mode can be used to [control](#common.IterMode) which
    nodes the cursor visits.
    */
    cursor(_ = 0) {
      return new TreeCursor(this.topNode, _);
    }
    /**
    Get a [tree cursor](#common.TreeCursor) pointing into this tree
    at the given position and side (see
    [`moveTo`](#common.TreeCursor.moveTo).
    */
    cursorAt(_, H = 0, V = 0) {
      let q = CachedNode.get(this) || this.topNode, G = new TreeCursor(q);
      return G.moveTo(_, H), CachedNode.set(this, G._tree), G;
    }
    /**
    Get a [syntax node](#common.SyntaxNode) object for the top of the
    tree.
    */
    get topNode() {
      return new TreeNode(this, 0, 0, null);
    }
    /**
    Get the [syntax node](#common.SyntaxNode) at the given position.
    If `side` is -1, this will move into nodes that end at the
    position. If 1, it'll move into nodes that start at the
    position. With 0, it'll only enter nodes that cover the position
    from both sides.
    
    Note that this will not enter
    [overlays](#common.MountedTree.overlay), and you often want
    [`resolveInner`](#common.Tree.resolveInner) instead.
    */
    resolve(_, H = 0) {
      let V = resolveNode(CachedNode.get(this) || this.topNode, _, H, !1);
      return CachedNode.set(this, V), V;
    }
    /**
    Like [`resolve`](#common.Tree.resolve), but will enter
    [overlaid](#common.MountedTree.overlay) nodes, producing a syntax node
    pointing into the innermost overlaid tree at the given position
    (with parent links going through all parent structure, including
    the host trees).
    */
    resolveInner(_, H = 0) {
      let V = resolveNode(CachedInnerNode.get(this) || this.topNode, _, H, !0);
      return CachedInnerNode.set(this, V), V;
    }
    /**
    In some situations, it can be useful to iterate through all
    nodes around a position, including those in overlays that don't
    directly cover the position. This method gives you an iterator
    that will produce all nodes, from small to big, around the given
    position.
    */
    resolveStack(_, H = 0) {
      return stackIterator(this, _, H);
    }
    /**
    Iterate over the tree and its children, calling `enter` for any
    node that touches the `from`/`to` region (if given) before
    running over such a node's children, and `leave` (if given) when
    leaving the node. When `enter` returns `false`, that node will
    not have its children iterated over (or `leave` called).
    */
    iterate(_) {
      let { enter: H, leave: V, from: q = 0, to: G = this.length } = _, J = _.mode || 0, Z = (J & IterMode.IncludeAnonymous) > 0;
      for (let te = this.cursor(J | IterMode.IncludeAnonymous); ; ) {
        let ne = !1;
        if (te.from <= G && te.to >= q && (!Z && te.type.isAnonymous || H(te) !== !1)) {
          if (te.firstChild())
            continue;
          ne = !0;
        }
        for (; ne && V && (Z || !te.type.isAnonymous) && V(te), !te.nextSibling(); ) {
          if (!te.parent())
            return;
          ne = !0;
        }
      }
    }
    /**
    Get the value of the given [node prop](#common.NodeProp) for this
    node. Works with both per-node and per-type props.
    */
    prop(_) {
      return _.perNode ? this.props ? this.props[_.id] : void 0 : this.type.prop(_);
    }
    /**
    Returns the node's [per-node props](#common.NodeProp.perNode) in a
    format that can be passed to the [`Tree`](#common.Tree)
    constructor.
    */
    get propValues() {
      let _ = [];
      if (this.props)
        for (let H in this.props)
          _.push([+H, this.props[H]]);
      return _;
    }
    /**
    Balance the direct children of this tree, producing a copy of
    which may have children grouped into subtrees with type
    [`NodeType.none`](#common.NodeType^none).
    */
    balance(_ = {}) {
      return this.children.length <= 8 ? this : balanceRange(NodeType.none, this.children, this.positions, 0, this.children.length, 0, this.length, (H, V, q) => new Tree(this.type, H, V, q, this.propValues), _.makeTree || ((H, V, q) => new Tree(NodeType.none, H, V, q)));
    }
    /**
    Build a tree from a postfix-ordered buffer of node information,
    or a cursor over such a buffer.
    */
    static build(_) {
      return buildTree(_);
    }
  }
  Tree.empty = new Tree(NodeType.none, [], [], 0);
  class FlatBufferCursor {
    constructor(_, H) {
      this.buffer = _, this.index = H;
    }
    get id() {
      return this.buffer[this.index - 4];
    }
    get start() {
      return this.buffer[this.index - 3];
    }
    get end() {
      return this.buffer[this.index - 2];
    }
    get size() {
      return this.buffer[this.index - 1];
    }
    get pos() {
      return this.index;
    }
    next() {
      this.index -= 4;
    }
    fork() {
      return new FlatBufferCursor(this.buffer, this.index);
    }
  }
  class TreeBuffer {
    /**
    Create a tree buffer.
    */
    constructor(_, H, V) {
      this.buffer = _, this.length = H, this.set = V;
    }
    /**
    @internal
    */
    get type() {
      return NodeType.none;
    }
    /**
    @internal
    */
    toString() {
      let _ = [];
      for (let H = 0; H < this.buffer.length; )
        _.push(this.childString(H)), H = this.buffer[H + 3];
      return _.join(",");
    }
    /**
    @internal
    */
    childString(_) {
      let H = this.buffer[_], V = this.buffer[_ + 3], q = this.set.types[H], G = q.name;
      if (/\W/.test(G) && !q.isError && (G = JSON.stringify(G)), _ += 4, V == _)
        return G;
      let J = [];
      for (; _ < V; )
        J.push(this.childString(_)), _ = this.buffer[_ + 3];
      return G + "(" + J.join(",") + ")";
    }
    /**
    @internal
    */
    findChild(_, H, V, q, G) {
      let { buffer: J } = this, Z = -1;
      for (let te = _; te != H && !(checkSide(G, q, J[te + 1], J[te + 2]) && (Z = te, V > 0)); te = J[te + 3])
        ;
      return Z;
    }
    /**
    @internal
    */
    slice(_, H, V) {
      let q = this.buffer, G = new Uint16Array(H - _), J = 0;
      for (let Z = _, te = 0; Z < H; ) {
        G[te++] = q[Z++], G[te++] = q[Z++] - V;
        let ne = G[te++] = q[Z++] - V;
        G[te++] = q[Z++] - _, J = Math.max(J, ne);
      }
      return new TreeBuffer(G, J, this.set);
    }
  }
  function checkSide(O, _, H, V) {
    switch (O) {
      case -2:
        return H < _;
      case -1:
        return V >= _ && H < _;
      case 0:
        return H < _ && V > _;
      case 1:
        return H <= _ && V > _;
      case 2:
        return V > _;
      case 4:
        return !0;
    }
  }
  function resolveNode(O, _, H, V) {
    for (var q; O.from == O.to || (H < 1 ? O.from >= _ : O.from > _) || (H > -1 ? O.to <= _ : O.to < _); ) {
      let J = !V && O instanceof TreeNode && O.index < 0 ? null : O.parent;
      if (!J)
        return O;
      O = J;
    }
    let G = V ? 0 : IterMode.IgnoreOverlays;
    if (V)
      for (let J = O, Z = J.parent; Z; J = Z, Z = J.parent)
        J instanceof TreeNode && J.index < 0 && ((q = Z.enter(_, H, G)) === null || q === void 0 ? void 0 : q.from) != J.from && (O = Z);
    for (; ; ) {
      let J = O.enter(_, H, G);
      if (!J)
        return O;
      O = J;
    }
  }
  class BaseNode {
    cursor(_ = 0) {
      return new TreeCursor(this, _);
    }
    getChild(_, H = null, V = null) {
      let q = getChildren(this, _, H, V);
      return q.length ? q[0] : null;
    }
    getChildren(_, H = null, V = null) {
      return getChildren(this, _, H, V);
    }
    resolve(_, H = 0) {
      return resolveNode(this, _, H, !1);
    }
    resolveInner(_, H = 0) {
      return resolveNode(this, _, H, !0);
    }
    matchContext(_) {
      return matchNodeContext(this.parent, _);
    }
    enterUnfinishedNodesBefore(_) {
      let H = this.childBefore(_), V = this;
      for (; H; ) {
        let q = H.lastChild;
        if (!q || q.to != H.to)
          break;
        q.type.isError && q.from == q.to ? (V = H, H = q.prevSibling) : H = q;
      }
      return V;
    }
    get node() {
      return this;
    }
    get next() {
      return this.parent;
    }
  }
  class TreeNode extends BaseNode {
    constructor(_, H, V, q) {
      super(), this._tree = _, this.from = H, this.index = V, this._parent = q;
    }
    get type() {
      return this._tree.type;
    }
    get name() {
      return this._tree.type.name;
    }
    get to() {
      return this.from + this._tree.length;
    }
    nextChild(_, H, V, q, G = 0) {
      for (let J = this; ; ) {
        for (let { children: Z, positions: te } = J._tree, ne = H > 0 ? Z.length : -1; _ != ne; _ += H) {
          let ee = Z[_], ae = te[_] + J.from;
          if (checkSide(q, V, ae, ae + ee.length)) {
            if (ee instanceof TreeBuffer) {
              if (G & IterMode.ExcludeBuffers)
                continue;
              let oe = ee.findChild(0, ee.buffer.length, H, V - ae, q);
              if (oe > -1)
                return new BufferNode(new BufferContext(J, ee, _, ae), null, oe);
            } else if (G & IterMode.IncludeAnonymous || !ee.type.isAnonymous || hasChild(ee)) {
              let oe;
              if (!(G & IterMode.IgnoreMounts) && (oe = MountedTree.get(ee)) && !oe.overlay)
                return new TreeNode(oe.tree, ae, _, J);
              let se = new TreeNode(ee, ae, _, J);
              return G & IterMode.IncludeAnonymous || !se.type.isAnonymous ? se : se.nextChild(H < 0 ? ee.children.length - 1 : 0, H, V, q);
            }
          }
        }
        if (G & IterMode.IncludeAnonymous || !J.type.isAnonymous || (J.index >= 0 ? _ = J.index + H : _ = H < 0 ? -1 : J._parent._tree.children.length, J = J._parent, !J))
          return null;
      }
    }
    get firstChild() {
      return this.nextChild(
        0,
        1,
        0,
        4
        /* Side.DontCare */
      );
    }
    get lastChild() {
      return this.nextChild(
        this._tree.children.length - 1,
        -1,
        0,
        4
        /* Side.DontCare */
      );
    }
    childAfter(_) {
      return this.nextChild(
        0,
        1,
        _,
        2
        /* Side.After */
      );
    }
    childBefore(_) {
      return this.nextChild(
        this._tree.children.length - 1,
        -1,
        _,
        -2
        /* Side.Before */
      );
    }
    enter(_, H, V = 0) {
      let q;
      if (!(V & IterMode.IgnoreOverlays) && (q = MountedTree.get(this._tree)) && q.overlay) {
        let G = _ - this.from;
        for (let { from: J, to: Z } of q.overlay)
          if ((H > 0 ? J <= G : J < G) && (H < 0 ? Z >= G : Z > G))
            return new TreeNode(q.tree, q.overlay[0].from + this.from, -1, this);
      }
      return this.nextChild(0, 1, _, H, V);
    }
    nextSignificantParent() {
      let _ = this;
      for (; _.type.isAnonymous && _._parent; )
        _ = _._parent;
      return _;
    }
    get parent() {
      return this._parent ? this._parent.nextSignificantParent() : null;
    }
    get nextSibling() {
      return this._parent && this.index >= 0 ? this._parent.nextChild(
        this.index + 1,
        1,
        0,
        4
        /* Side.DontCare */
      ) : null;
    }
    get prevSibling() {
      return this._parent && this.index >= 0 ? this._parent.nextChild(
        this.index - 1,
        -1,
        0,
        4
        /* Side.DontCare */
      ) : null;
    }
    get tree() {
      return this._tree;
    }
    toTree() {
      return this._tree;
    }
    /**
    @internal
    */
    toString() {
      return this._tree.toString();
    }
  }
  function getChildren(O, _, H, V) {
    let q = O.cursor(), G = [];
    if (!q.firstChild())
      return G;
    if (H != null) {
      for (let J = !1; !J; )
        if (J = q.type.is(H), !q.nextSibling())
          return G;
    }
    for (; ; ) {
      if (V != null && q.type.is(V))
        return G;
      if (q.type.is(_) && G.push(q.node), !q.nextSibling())
        return V == null ? G : [];
    }
  }
  function matchNodeContext(O, _, H = _.length - 1) {
    for (let V = O; H >= 0; V = V.parent) {
      if (!V)
        return !1;
      if (!V.type.isAnonymous) {
        if (_[H] && _[H] != V.name)
          return !1;
        H--;
      }
    }
    return !0;
  }
  class BufferContext {
    constructor(_, H, V, q) {
      this.parent = _, this.buffer = H, this.index = V, this.start = q;
    }
  }
  class BufferNode extends BaseNode {
    get name() {
      return this.type.name;
    }
    get from() {
      return this.context.start + this.context.buffer.buffer[this.index + 1];
    }
    get to() {
      return this.context.start + this.context.buffer.buffer[this.index + 2];
    }
    constructor(_, H, V) {
      super(), this.context = _, this._parent = H, this.index = V, this.type = _.buffer.set.types[_.buffer.buffer[V]];
    }
    child(_, H, V) {
      let { buffer: q } = this.context, G = q.findChild(this.index + 4, q.buffer[this.index + 3], _, H - this.context.start, V);
      return G < 0 ? null : new BufferNode(this.context, this, G);
    }
    get firstChild() {
      return this.child(
        1,
        0,
        4
        /* Side.DontCare */
      );
    }
    get lastChild() {
      return this.child(
        -1,
        0,
        4
        /* Side.DontCare */
      );
    }
    childAfter(_) {
      return this.child(
        1,
        _,
        2
        /* Side.After */
      );
    }
    childBefore(_) {
      return this.child(
        -1,
        _,
        -2
        /* Side.Before */
      );
    }
    enter(_, H, V = 0) {
      if (V & IterMode.ExcludeBuffers)
        return null;
      let { buffer: q } = this.context, G = q.findChild(this.index + 4, q.buffer[this.index + 3], H > 0 ? 1 : -1, _ - this.context.start, H);
      return G < 0 ? null : new BufferNode(this.context, this, G);
    }
    get parent() {
      return this._parent || this.context.parent.nextSignificantParent();
    }
    externalSibling(_) {
      return this._parent ? null : this.context.parent.nextChild(
        this.context.index + _,
        _,
        0,
        4
        /* Side.DontCare */
      );
    }
    get nextSibling() {
      let { buffer: _ } = this.context, H = _.buffer[this.index + 3];
      return H < (this._parent ? _.buffer[this._parent.index + 3] : _.buffer.length) ? new BufferNode(this.context, this._parent, H) : this.externalSibling(1);
    }
    get prevSibling() {
      let { buffer: _ } = this.context, H = this._parent ? this._parent.index + 4 : 0;
      return this.index == H ? this.externalSibling(-1) : new BufferNode(this.context, this._parent, _.findChild(
        H,
        this.index,
        -1,
        0,
        4
        /* Side.DontCare */
      ));
    }
    get tree() {
      return null;
    }
    toTree() {
      let _ = [], H = [], { buffer: V } = this.context, q = this.index + 4, G = V.buffer[this.index + 3];
      if (G > q) {
        let J = V.buffer[this.index + 1];
        _.push(V.slice(q, G, J)), H.push(0);
      }
      return new Tree(this.type, _, H, this.to - this.from);
    }
    /**
    @internal
    */
    toString() {
      return this.context.buffer.childString(this.index);
    }
  }
  function iterStack(O) {
    if (!O.length)
      return null;
    let _ = 0, H = O[0];
    for (let G = 1; G < O.length; G++) {
      let J = O[G];
      (J.from > H.from || J.to < H.to) && (H = J, _ = G);
    }
    let V = H instanceof TreeNode && H.index < 0 ? null : H.parent, q = O.slice();
    return V ? q[_] = V : q.splice(_, 1), new StackIterator(q, H);
  }
  class StackIterator {
    constructor(_, H) {
      this.heads = _, this.node = H;
    }
    get next() {
      return iterStack(this.heads);
    }
  }
  function stackIterator(O, _, H) {
    let V = O.resolveInner(_, H), q = null;
    for (let G = V instanceof TreeNode ? V : V.context.parent; G; G = G.parent)
      if (G.index < 0) {
        let J = G.parent;
        (q || (q = [V])).push(J.resolve(_, H)), G = J;
      } else {
        let J = MountedTree.get(G.tree);
        if (J && J.overlay && J.overlay[0].from <= _ && J.overlay[J.overlay.length - 1].to >= _) {
          let Z = new TreeNode(J.tree, J.overlay[0].from + G.from, -1, G);
          (q || (q = [V])).push(resolveNode(Z, _, H, !1));
        }
      }
    return q ? iterStack(q) : V;
  }
  class TreeCursor {
    /**
    Shorthand for `.type.name`.
    */
    get name() {
      return this.type.name;
    }
    /**
    @internal
    */
    constructor(_, H = 0) {
      if (this.mode = H, this.buffer = null, this.stack = [], this.index = 0, this.bufferNode = null, _ instanceof TreeNode)
        this.yieldNode(_);
      else {
        this._tree = _.context.parent, this.buffer = _.context;
        for (let V = _._parent; V; V = V._parent)
          this.stack.unshift(V.index);
        this.bufferNode = _, this.yieldBuf(_.index);
      }
    }
    yieldNode(_) {
      return _ ? (this._tree = _, this.type = _.type, this.from = _.from, this.to = _.to, !0) : !1;
    }
    yieldBuf(_, H) {
      this.index = _;
      let { start: V, buffer: q } = this.buffer;
      return this.type = H || q.set.types[q.buffer[_]], this.from = V + q.buffer[_ + 1], this.to = V + q.buffer[_ + 2], !0;
    }
    /**
    @internal
    */
    yield(_) {
      return _ ? _ instanceof TreeNode ? (this.buffer = null, this.yieldNode(_)) : (this.buffer = _.context, this.yieldBuf(_.index, _.type)) : !1;
    }
    /**
    @internal
    */
    toString() {
      return this.buffer ? this.buffer.buffer.childString(this.index) : this._tree.toString();
    }
    /**
    @internal
    */
    enterChild(_, H, V) {
      if (!this.buffer)
        return this.yield(this._tree.nextChild(_ < 0 ? this._tree._tree.children.length - 1 : 0, _, H, V, this.mode));
      let { buffer: q } = this.buffer, G = q.findChild(this.index + 4, q.buffer[this.index + 3], _, H - this.buffer.start, V);
      return G < 0 ? !1 : (this.stack.push(this.index), this.yieldBuf(G));
    }
    /**
    Move the cursor to this node's first child. When this returns
    false, the node has no child, and the cursor has not been moved.
    */
    firstChild() {
      return this.enterChild(
        1,
        0,
        4
        /* Side.DontCare */
      );
    }
    /**
    Move the cursor to this node's last child.
    */
    lastChild() {
      return this.enterChild(
        -1,
        0,
        4
        /* Side.DontCare */
      );
    }
    /**
    Move the cursor to the first child that ends after `pos`.
    */
    childAfter(_) {
      return this.enterChild(
        1,
        _,
        2
        /* Side.After */
      );
    }
    /**
    Move to the last child that starts before `pos`.
    */
    childBefore(_) {
      return this.enterChild(
        -1,
        _,
        -2
        /* Side.Before */
      );
    }
    /**
    Move the cursor to the child around `pos`. If side is -1 the
    child may end at that position, when 1 it may start there. This
    will also enter [overlaid](#common.MountedTree.overlay)
    [mounted](#common.NodeProp^mounted) trees unless `overlays` is
    set to false.
    */
    enter(_, H, V = this.mode) {
      return this.buffer ? V & IterMode.ExcludeBuffers ? !1 : this.enterChild(1, _, H) : this.yield(this._tree.enter(_, H, V));
    }
    /**
    Move to the node's parent node, if this isn't the top node.
    */
    parent() {
      if (!this.buffer)
        return this.yieldNode(this.mode & IterMode.IncludeAnonymous ? this._tree._parent : this._tree.parent);
      if (this.stack.length)
        return this.yieldBuf(this.stack.pop());
      let _ = this.mode & IterMode.IncludeAnonymous ? this.buffer.parent : this.buffer.parent.nextSignificantParent();
      return this.buffer = null, this.yieldNode(_);
    }
    /**
    @internal
    */
    sibling(_) {
      if (!this.buffer)
        return this._tree._parent ? this.yield(this._tree.index < 0 ? null : this._tree._parent.nextChild(this._tree.index + _, _, 0, 4, this.mode)) : !1;
      let { buffer: H } = this.buffer, V = this.stack.length - 1;
      if (_ < 0) {
        let q = V < 0 ? 0 : this.stack[V] + 4;
        if (this.index != q)
          return this.yieldBuf(H.findChild(
            q,
            this.index,
            -1,
            0,
            4
            /* Side.DontCare */
          ));
      } else {
        let q = H.buffer[this.index + 3];
        if (q < (V < 0 ? H.buffer.length : H.buffer[this.stack[V] + 3]))
          return this.yieldBuf(q);
      }
      return V < 0 ? this.yield(this.buffer.parent.nextChild(this.buffer.index + _, _, 0, 4, this.mode)) : !1;
    }
    /**
    Move to this node's next sibling, if any.
    */
    nextSibling() {
      return this.sibling(1);
    }
    /**
    Move to this node's previous sibling, if any.
    */
    prevSibling() {
      return this.sibling(-1);
    }
    atLastNode(_) {
      let H, V, { buffer: q } = this;
      if (q) {
        if (_ > 0) {
          if (this.index < q.buffer.buffer.length)
            return !1;
        } else
          for (let G = 0; G < this.index; G++)
            if (q.buffer.buffer[G + 3] < this.index)
              return !1;
        ({ index: H, parent: V } = q);
      } else
        ({ index: H, _parent: V } = this._tree);
      for (; V; { index: H, _parent: V } = V)
        if (H > -1)
          for (let G = H + _, J = _ < 0 ? -1 : V._tree.children.length; G != J; G += _) {
            let Z = V._tree.children[G];
            if (this.mode & IterMode.IncludeAnonymous || Z instanceof TreeBuffer || !Z.type.isAnonymous || hasChild(Z))
              return !1;
          }
      return !0;
    }
    move(_, H) {
      if (H && this.enterChild(
        _,
        0,
        4
        /* Side.DontCare */
      ))
        return !0;
      for (; ; ) {
        if (this.sibling(_))
          return !0;
        if (this.atLastNode(_) || !this.parent())
          return !1;
      }
    }
    /**
    Move to the next node in a
    [pre-order](https://en.wikipedia.org/wiki/Tree_traversal#Pre-order,_NLR)
    traversal, going from a node to its first child or, if the
    current node is empty or `enter` is false, its next sibling or
    the next sibling of the first parent node that has one.
    */
    next(_ = !0) {
      return this.move(1, _);
    }
    /**
    Move to the next node in a last-to-first pre-order traversal. A
    node is followed by its last child or, if it has none, its
    previous sibling or the previous sibling of the first parent
    node that has one.
    */
    prev(_ = !0) {
      return this.move(-1, _);
    }
    /**
    Move the cursor to the innermost node that covers `pos`. If
    `side` is -1, it will enter nodes that end at `pos`. If it is 1,
    it will enter nodes that start at `pos`.
    */
    moveTo(_, H = 0) {
      for (; (this.from == this.to || (H < 1 ? this.from >= _ : this.from > _) || (H > -1 ? this.to <= _ : this.to < _)) && this.parent(); )
        ;
      for (; this.enterChild(1, _, H); )
        ;
      return this;
    }
    /**
    Get a [syntax node](#common.SyntaxNode) at the cursor's current
    position.
    */
    get node() {
      if (!this.buffer)
        return this._tree;
      let _ = this.bufferNode, H = null, V = 0;
      if (_ && _.context == this.buffer)
        e: for (let q = this.index, G = this.stack.length; G >= 0; ) {
          for (let J = _; J; J = J._parent)
            if (J.index == q) {
              if (q == this.index)
                return J;
              H = J, V = G + 1;
              break e;
            }
          q = this.stack[--G];
        }
      for (let q = V; q < this.stack.length; q++)
        H = new BufferNode(this.buffer, H, this.stack[q]);
      return this.bufferNode = new BufferNode(this.buffer, H, this.index);
    }
    /**
    Get the [tree](#common.Tree) that represents the current node, if
    any. Will return null when the node is in a [tree
    buffer](#common.TreeBuffer).
    */
    get tree() {
      return this.buffer ? null : this._tree._tree;
    }
    /**
    Iterate over the current node and all its descendants, calling
    `enter` when entering a node and `leave`, if given, when leaving
    one. When `enter` returns `false`, any children of that node are
    skipped, and `leave` isn't called for it.
    */
    iterate(_, H) {
      for (let V = 0; ; ) {
        let q = !1;
        if (this.type.isAnonymous || _(this) !== !1) {
          if (this.firstChild()) {
            V++;
            continue;
          }
          this.type.isAnonymous || (q = !0);
        }
        for (; ; ) {
          if (q && H && H(this), q = this.type.isAnonymous, !V)
            return;
          if (this.nextSibling())
            break;
          this.parent(), V--, q = !0;
        }
      }
    }
    /**
    Test whether the current node matches a given context—a sequence
    of direct parent node names. Empty strings in the context array
    are treated as wildcards.
    */
    matchContext(_) {
      if (!this.buffer)
        return matchNodeContext(this.node.parent, _);
      let { buffer: H } = this.buffer, { types: V } = H.set;
      for (let q = _.length - 1, G = this.stack.length - 1; q >= 0; G--) {
        if (G < 0)
          return matchNodeContext(this._tree, _, q);
        let J = V[H.buffer[this.stack[G]]];
        if (!J.isAnonymous) {
          if (_[q] && _[q] != J.name)
            return !1;
          q--;
        }
      }
      return !0;
    }
  }
  function hasChild(O) {
    return O.children.some((_) => _ instanceof TreeBuffer || !_.type.isAnonymous || hasChild(_));
  }
  function buildTree(O) {
    var _;
    let { buffer: H, nodeSet: V, maxBufferLength: q = DefaultBufferLength, reused: G = [], minRepeatType: J = V.types.length } = O, Z = Array.isArray(H) ? new FlatBufferCursor(H, H.length) : H, te = V.types, ne = 0, ee = 0;
    function ae(pe, ye, ge, ve, be, Ce) {
      let { id: Se, start: we, end: Ee, size: xe } = Z, Le = ee, Qe = ne;
      for (; xe < 0; )
        if (Z.next(), xe == -1) {
          let Pe = G[Se];
          ge.push(Pe), ve.push(we - pe);
          return;
        } else if (xe == -3) {
          ne = Se;
          return;
        } else if (xe == -4) {
          ee = Se;
          return;
        } else
          throw new RangeError(`Unrecognized record size: ${xe}`);
      let ke = te[Se], Me, Re, De = we - pe;
      if (Ee - we <= q && (Re = ue(Z.pos - ye, be))) {
        let Pe = new Uint16Array(Re.size - Re.skip), $e = Z.pos - Re.size, qe = Pe.length;
        for (; Z.pos > $e; )
          qe = de(Re.start, Pe, qe);
        Me = new TreeBuffer(Pe, Ee - Re.start, V), De = Re.start - pe;
      } else {
        let Pe = Z.pos - xe;
        Z.next();
        let $e = [], qe = [], Xe = Se >= J ? Se : -1, Ve = 0, Ze = Ee;
        for (; Z.pos > Pe; )
          Xe >= 0 && Z.id == Xe && Z.size >= 0 ? (Z.end <= Ze - q && (ce($e, qe, we, Ve, Z.end, Ze, Xe, Le, Qe), Ve = $e.length, Ze = Z.end), Z.next()) : Ce > 2500 ? oe(we, Pe, $e, qe) : ae(we, Pe, $e, qe, Xe, Ce + 1);
        if (Xe >= 0 && Ve > 0 && Ve < $e.length && ce($e, qe, we, Ve, we, Ze, Xe, Le, Qe), $e.reverse(), qe.reverse(), Xe > -1 && Ve > 0) {
          let Ge = se(ke, Qe);
          Me = balanceRange(ke, $e, qe, 0, $e.length, 0, Ee - we, Ge, Ge);
        } else
          Me = le(ke, $e, qe, Ee - we, Le - Ee, Qe);
      }
      ge.push(Me), ve.push(De);
    }
    function oe(pe, ye, ge, ve) {
      let be = [], Ce = 0, Se = -1;
      for (; Z.pos > ye; ) {
        let { id: we, start: Ee, end: xe, size: Le } = Z;
        if (Le > 4)
          Z.next();
        else {
          if (Se > -1 && Ee < Se)
            break;
          Se < 0 && (Se = xe - q), be.push(we, Ee, xe), Ce++, Z.next();
        }
      }
      if (Ce) {
        let we = new Uint16Array(Ce * 4), Ee = be[be.length - 2];
        for (let xe = be.length - 3, Le = 0; xe >= 0; xe -= 3)
          we[Le++] = be[xe], we[Le++] = be[xe + 1] - Ee, we[Le++] = be[xe + 2] - Ee, we[Le++] = Le;
        ge.push(new TreeBuffer(we, be[2] - Ee, V)), ve.push(Ee - pe);
      }
    }
    function se(pe, ye) {
      return (ge, ve, be) => {
        let Ce = 0, Se = ge.length - 1, we, Ee;
        if (Se >= 0 && (we = ge[Se]) instanceof Tree) {
          if (!Se && we.type == pe && we.length == be)
            return we;
          (Ee = we.prop(NodeProp.lookAhead)) && (Ce = ve[Se] + we.length + Ee);
        }
        return le(pe, ge, ve, be, Ce, ye);
      };
    }
    function ce(pe, ye, ge, ve, be, Ce, Se, we, Ee) {
      let xe = [], Le = [];
      for (; pe.length > ve; )
        xe.push(pe.pop()), Le.push(ye.pop() + ge - be);
      pe.push(le(V.types[Se], xe, Le, Ce - be, we - Ce, Ee)), ye.push(be - ge);
    }
    function le(pe, ye, ge, ve, be, Ce, Se) {
      if (Ce) {
        let we = [NodeProp.contextHash, Ce];
        Se = Se ? [we].concat(Se) : [we];
      }
      if (be > 25) {
        let we = [NodeProp.lookAhead, be];
        Se = Se ? [we].concat(Se) : [we];
      }
      return new Tree(pe, ye, ge, ve, Se);
    }
    function ue(pe, ye) {
      let ge = Z.fork(), ve = 0, be = 0, Ce = 0, Se = ge.end - q, we = { size: 0, start: 0, skip: 0 };
      e: for (let Ee = ge.pos - pe; ge.pos > Ee; ) {
        let xe = ge.size;
        if (ge.id == ye && xe >= 0) {
          we.size = ve, we.start = be, we.skip = Ce, Ce += 4, ve += 4, ge.next();
          continue;
        }
        let Le = ge.pos - xe;
        if (xe < 0 || Le < Ee || ge.start < Se)
          break;
        let Qe = ge.id >= J ? 4 : 0, ke = ge.start;
        for (ge.next(); ge.pos > Le; ) {
          if (ge.size < 0)
            if (ge.size == -3)
              Qe += 4;
            else
              break e;
          else ge.id >= J && (Qe += 4);
          ge.next();
        }
        be = ke, ve += xe, Ce += Qe;
      }
      return (ye < 0 || ve == pe) && (we.size = ve, we.start = be, we.skip = Ce), we.size > 4 ? we : void 0;
    }
    function de(pe, ye, ge) {
      let { id: ve, start: be, end: Ce, size: Se } = Z;
      if (Z.next(), Se >= 0 && ve < J) {
        let we = ge;
        if (Se > 4) {
          let Ee = Z.pos - (Se - 4);
          for (; Z.pos > Ee; )
            ge = de(pe, ye, ge);
        }
        ye[--ge] = we, ye[--ge] = Ce - pe, ye[--ge] = be - pe, ye[--ge] = ve;
      } else Se == -3 ? ne = ve : Se == -4 && (ee = ve);
      return ge;
    }
    let fe = [], he = [];
    for (; Z.pos > 0; )
      ae(O.start || 0, O.bufferStart || 0, fe, he, -1, 0);
    let me = (_ = O.length) !== null && _ !== void 0 ? _ : fe.length ? he[0] + fe[0].length : 0;
    return new Tree(te[O.topID], fe.reverse(), he.reverse(), me);
  }
  const nodeSizeCache = /* @__PURE__ */ new WeakMap();
  function nodeSize(O, _) {
    if (!O.isAnonymous || _ instanceof TreeBuffer || _.type != O)
      return 1;
    let H = nodeSizeCache.get(_);
    if (H == null) {
      H = 1;
      for (let V of _.children) {
        if (V.type != O || !(V instanceof Tree)) {
          H = 1;
          break;
        }
        H += nodeSize(O, V);
      }
      nodeSizeCache.set(_, H);
    }
    return H;
  }
  function balanceRange(O, _, H, V, q, G, J, Z, te) {
    let ne = 0;
    for (let ce = V; ce < q; ce++)
      ne += nodeSize(O, _[ce]);
    let ee = Math.ceil(
      ne * 1.5 / 8
      /* Balance.BranchFactor */
    ), ae = [], oe = [];
    function se(ce, le, ue, de, fe) {
      for (let he = ue; he < de; ) {
        let me = he, pe = le[he], ye = nodeSize(O, ce[he]);
        for (he++; he < de; he++) {
          let ge = nodeSize(O, ce[he]);
          if (ye + ge >= ee)
            break;
          ye += ge;
        }
        if (he == me + 1) {
          if (ye > ee) {
            let ge = ce[me];
            se(ge.children, ge.positions, 0, ge.children.length, le[me] + fe);
            continue;
          }
          ae.push(ce[me]);
        } else {
          let ge = le[he - 1] + ce[he - 1].length - pe;
          ae.push(balanceRange(O, ce, le, me, he, pe, ge, null, te));
        }
        oe.push(pe + fe - G);
      }
    }
    return se(_, H, V, q, 0), (Z || te)(ae, oe, J);
  }
  class TreeFragment {
    /**
    Construct a tree fragment. You'll usually want to use
    [`addTree`](#common.TreeFragment^addTree) and
    [`applyChanges`](#common.TreeFragment^applyChanges) instead of
    calling this directly.
    */
    constructor(_, H, V, q, G = !1, J = !1) {
      this.from = _, this.to = H, this.tree = V, this.offset = q, this.open = (G ? 1 : 0) | (J ? 2 : 0);
    }
    /**
    Whether the start of the fragment represents the start of a
    parse, or the end of a change. (In the second case, it may not
    be safe to reuse some nodes at the start, depending on the
    parsing algorithm.)
    */
    get openStart() {
      return (this.open & 1) > 0;
    }
    /**
    Whether the end of the fragment represents the end of a
    full-document parse, or the start of a change.
    */
    get openEnd() {
      return (this.open & 2) > 0;
    }
    /**
    Create a set of fragments from a freshly parsed tree, or update
    an existing set of fragments by replacing the ones that overlap
    with a tree with content from the new tree. When `partial` is
    true, the parse is treated as incomplete, and the resulting
    fragment has [`openEnd`](#common.TreeFragment.openEnd) set to
    true.
    */
    static addTree(_, H = [], V = !1) {
      let q = [new TreeFragment(0, _.length, _, 0, !1, V)];
      for (let G of H)
        G.to > _.length && q.push(G);
      return q;
    }
    /**
    Apply a set of edits to an array of fragments, removing or
    splitting fragments as necessary to remove edited ranges, and
    adjusting offsets for fragments that moved.
    */
    static applyChanges(_, H, V = 128) {
      if (!H.length)
        return _;
      let q = [], G = 1, J = _.length ? _[0] : null;
      for (let Z = 0, te = 0, ne = 0; ; Z++) {
        let ee = Z < H.length ? H[Z] : null, ae = ee ? ee.fromA : 1e9;
        if (ae - te >= V)
          for (; J && J.from < ae; ) {
            let oe = J;
            if (te >= oe.from || ae <= oe.to || ne) {
              let se = Math.max(oe.from, te) - ne, ce = Math.min(oe.to, ae) - ne;
              oe = se >= ce ? null : new TreeFragment(se, ce, oe.tree, oe.offset + ne, Z > 0, !!ee);
            }
            if (oe && q.push(oe), J.to > ae)
              break;
            J = G < _.length ? _[G++] : null;
          }
        if (!ee)
          break;
        te = ee.toA, ne = ee.toA - ee.toB;
      }
      return q;
    }
  }
  class Parser {
    /**
    Start a parse, returning a [partial parse](#common.PartialParse)
    object. [`fragments`](#common.TreeFragment) can be passed in to
    make the parse incremental.
    
    By default, the entire input is parsed. You can pass `ranges`,
    which should be a sorted array of non-empty, non-overlapping
    ranges, to parse only those ranges. The tree returned in that
    case will start at `ranges[0].from`.
    */
    startParse(_, H, V) {
      return typeof _ == "string" && (_ = new StringInput(_)), V = V ? V.length ? V.map((q) => new Range$1(q.from, q.to)) : [new Range$1(0, 0)] : [new Range$1(0, _.length)], this.createParse(_, H || [], V);
    }
    /**
    Run a full parse, returning the resulting tree.
    */
    parse(_, H, V) {
      let q = this.startParse(_, H, V);
      for (; ; ) {
        let G = q.advance();
        if (G)
          return G;
      }
    }
  }
  class StringInput {
    constructor(_) {
      this.string = _;
    }
    get length() {
      return this.string.length;
    }
    chunk(_) {
      return this.string.slice(_);
    }
    get lineChunks() {
      return !1;
    }
    read(_, H) {
      return this.string.slice(_, H);
    }
  }
  new NodeProp({ perNode: !0 });
  class Text {
    /**
    Get the line description around the given position.
    */
    lineAt(_) {
      if (_ < 0 || _ > this.length)
        throw new RangeError(`Invalid position ${_} in document of length ${this.length}`);
      return this.lineInner(_, !1, 1, 0);
    }
    /**
    Get the description for the given (1-based) line number.
    */
    line(_) {
      if (_ < 1 || _ > this.lines)
        throw new RangeError(`Invalid line number ${_} in ${this.lines}-line document`);
      return this.lineInner(_, !0, 1, 0);
    }
    /**
    Replace a range of the text with the given content.
    */
    replace(_, H, V) {
      [_, H] = clip(this, _, H);
      let q = [];
      return this.decompose(
        0,
        _,
        q,
        2
        /* Open.To */
      ), V.length && V.decompose(
        0,
        V.length,
        q,
        3
        /* Open.To */
      ), this.decompose(
        H,
        this.length,
        q,
        1
        /* Open.From */
      ), TextNode.from(q, this.length - (H - _) + V.length);
    }
    /**
    Append another document to this one.
    */
    append(_) {
      return this.replace(this.length, this.length, _);
    }
    /**
    Retrieve the text between the given points.
    */
    slice(_, H = this.length) {
      [_, H] = clip(this, _, H);
      let V = [];
      return this.decompose(_, H, V, 0), TextNode.from(V, H - _);
    }
    /**
    Test whether this text is equal to another instance.
    */
    eq(_) {
      if (_ == this)
        return !0;
      if (_.length != this.length || _.lines != this.lines)
        return !1;
      let H = this.scanIdentical(_, 1), V = this.length - this.scanIdentical(_, -1), q = new RawTextCursor(this), G = new RawTextCursor(_);
      for (let J = H, Z = H; ; ) {
        if (q.next(J), G.next(J), J = 0, q.lineBreak != G.lineBreak || q.done != G.done || q.value != G.value)
          return !1;
        if (Z += q.value.length, q.done || Z >= V)
          return !0;
      }
    }
    /**
    Iterate over the text. When `dir` is `-1`, iteration happens
    from end to start. This will return lines and the breaks between
    them as separate strings.
    */
    iter(_ = 1) {
      return new RawTextCursor(this, _);
    }
    /**
    Iterate over a range of the text. When `from` > `to`, the
    iterator will run in reverse.
    */
    iterRange(_, H = this.length) {
      return new PartialTextCursor(this, _, H);
    }
    /**
    Return a cursor that iterates over the given range of lines,
    _without_ returning the line breaks between, and yielding empty
    strings for empty lines.
    
    When `from` and `to` are given, they should be 1-based line numbers.
    */
    iterLines(_, H) {
      let V;
      if (_ == null)
        V = this.iter();
      else {
        H == null && (H = this.lines + 1);
        let q = this.line(_).from;
        V = this.iterRange(q, Math.max(q, H == this.lines + 1 ? this.length : H <= 1 ? 0 : this.line(H - 1).to));
      }
      return new LineCursor(V);
    }
    /**
    Return the document as a string, using newline characters to
    separate lines.
    */
    toString() {
      return this.sliceString(0);
    }
    /**
    Convert the document to an array of lines (which can be
    deserialized again via [`Text.of`](https://codemirror.net/6/docs/ref/#state.Text^of)).
    */
    toJSON() {
      let _ = [];
      return this.flatten(_), _;
    }
    /**
    @internal
    */
    constructor() {
    }
    /**
    Create a `Text` instance for the given array of lines.
    */
    static of(_) {
      if (_.length == 0)
        throw new RangeError("A document must have at least one line");
      return _.length == 1 && !_[0] ? Text.empty : _.length <= 32 ? new TextLeaf(_) : TextNode.from(TextLeaf.split(_, []));
    }
  }
  class TextLeaf extends Text {
    constructor(_, H = textLength(_)) {
      super(), this.text = _, this.length = H;
    }
    get lines() {
      return this.text.length;
    }
    get children() {
      return null;
    }
    lineInner(_, H, V, q) {
      for (let G = 0; ; G++) {
        let J = this.text[G], Z = q + J.length;
        if ((H ? V : Z) >= _)
          return new Line(q, Z, V, J);
        q = Z + 1, V++;
      }
    }
    decompose(_, H, V, q) {
      let G = _ <= 0 && H >= this.length ? this : new TextLeaf(sliceText(this.text, _, H), Math.min(H, this.length) - Math.max(0, _));
      if (q & 1) {
        let J = V.pop(), Z = appendText(G.text, J.text.slice(), 0, G.length);
        if (Z.length <= 32)
          V.push(new TextLeaf(Z, J.length + G.length));
        else {
          let te = Z.length >> 1;
          V.push(new TextLeaf(Z.slice(0, te)), new TextLeaf(Z.slice(te)));
        }
      } else
        V.push(G);
    }
    replace(_, H, V) {
      if (!(V instanceof TextLeaf))
        return super.replace(_, H, V);
      [_, H] = clip(this, _, H);
      let q = appendText(this.text, appendText(V.text, sliceText(this.text, 0, _)), H), G = this.length + V.length - (H - _);
      return q.length <= 32 ? new TextLeaf(q, G) : TextNode.from(TextLeaf.split(q, []), G);
    }
    sliceString(_, H = this.length, V = `
`) {
      [_, H] = clip(this, _, H);
      let q = "";
      for (let G = 0, J = 0; G <= H && J < this.text.length; J++) {
        let Z = this.text[J], te = G + Z.length;
        G > _ && J && (q += V), _ < te && H > G && (q += Z.slice(Math.max(0, _ - G), H - G)), G = te + 1;
      }
      return q;
    }
    flatten(_) {
      for (let H of this.text)
        _.push(H);
    }
    scanIdentical() {
      return 0;
    }
    static split(_, H) {
      let V = [], q = -1;
      for (let G of _)
        V.push(G), q += G.length + 1, V.length == 32 && (H.push(new TextLeaf(V, q)), V = [], q = -1);
      return q > -1 && H.push(new TextLeaf(V, q)), H;
    }
  }
  class TextNode extends Text {
    constructor(_, H) {
      super(), this.children = _, this.length = H, this.lines = 0;
      for (let V of _)
        this.lines += V.lines;
    }
    lineInner(_, H, V, q) {
      for (let G = 0; ; G++) {
        let J = this.children[G], Z = q + J.length, te = V + J.lines - 1;
        if ((H ? te : Z) >= _)
          return J.lineInner(_, H, V, q);
        q = Z + 1, V = te + 1;
      }
    }
    decompose(_, H, V, q) {
      for (let G = 0, J = 0; J <= H && G < this.children.length; G++) {
        let Z = this.children[G], te = J + Z.length;
        if (_ <= te && H >= J) {
          let ne = q & ((J <= _ ? 1 : 0) | (te >= H ? 2 : 0));
          J >= _ && te <= H && !ne ? V.push(Z) : Z.decompose(_ - J, H - J, V, ne);
        }
        J = te + 1;
      }
    }
    replace(_, H, V) {
      if ([_, H] = clip(this, _, H), V.lines < this.lines)
        for (let q = 0, G = 0; q < this.children.length; q++) {
          let J = this.children[q], Z = G + J.length;
          if (_ >= G && H <= Z) {
            let te = J.replace(_ - G, H - G, V), ne = this.lines - J.lines + te.lines;
            if (te.lines < ne >> 4 && te.lines > ne >> 6) {
              let ee = this.children.slice();
              return ee[q] = te, new TextNode(ee, this.length - (H - _) + V.length);
            }
            return super.replace(G, Z, te);
          }
          G = Z + 1;
        }
      return super.replace(_, H, V);
    }
    sliceString(_, H = this.length, V = `
`) {
      [_, H] = clip(this, _, H);
      let q = "";
      for (let G = 0, J = 0; G < this.children.length && J <= H; G++) {
        let Z = this.children[G], te = J + Z.length;
        J > _ && G && (q += V), _ < te && H > J && (q += Z.sliceString(_ - J, H - J, V)), J = te + 1;
      }
      return q;
    }
    flatten(_) {
      for (let H of this.children)
        H.flatten(_);
    }
    scanIdentical(_, H) {
      if (!(_ instanceof TextNode))
        return 0;
      let V = 0, [q, G, J, Z] = H > 0 ? [0, 0, this.children.length, _.children.length] : [this.children.length - 1, _.children.length - 1, -1, -1];
      for (; ; q += H, G += H) {
        if (q == J || G == Z)
          return V;
        let te = this.children[q], ne = _.children[G];
        if (te != ne)
          return V + te.scanIdentical(ne, H);
        V += te.length + 1;
      }
    }
    static from(_, H = _.reduce((V, q) => V + q.length + 1, -1)) {
      let V = 0;
      for (let se of _)
        V += se.lines;
      if (V < 32) {
        let se = [];
        for (let ce of _)
          ce.flatten(se);
        return new TextLeaf(se, H);
      }
      let q = Math.max(
        32,
        V >> 5
        /* Tree.BranchShift */
      ), G = q << 1, J = q >> 1, Z = [], te = 0, ne = -1, ee = [];
      function ae(se) {
        let ce;
        if (se.lines > G && se instanceof TextNode)
          for (let le of se.children)
            ae(le);
        else se.lines > J && (te > J || !te) ? (oe(), Z.push(se)) : se instanceof TextLeaf && te && (ce = ee[ee.length - 1]) instanceof TextLeaf && se.lines + ce.lines <= 32 ? (te += se.lines, ne += se.length + 1, ee[ee.length - 1] = new TextLeaf(ce.text.concat(se.text), ce.length + 1 + se.length)) : (te + se.lines > q && oe(), te += se.lines, ne += se.length + 1, ee.push(se));
      }
      function oe() {
        te != 0 && (Z.push(ee.length == 1 ? ee[0] : TextNode.from(ee, ne)), ne = -1, te = ee.length = 0);
      }
      for (let se of _)
        ae(se);
      return oe(), Z.length == 1 ? Z[0] : new TextNode(Z, H);
    }
  }
  Text.empty = /* @__PURE__ */ new TextLeaf([""], 0);
  function textLength(O) {
    let _ = -1;
    for (let H of O)
      _ += H.length + 1;
    return _;
  }
  function appendText(O, _, H = 0, V = 1e9) {
    for (let q = 0, G = 0, J = !0; G < O.length && q <= V; G++) {
      let Z = O[G], te = q + Z.length;
      te >= H && (te > V && (Z = Z.slice(0, V - q)), q < H && (Z = Z.slice(H - q)), J ? (_[_.length - 1] += Z, J = !1) : _.push(Z)), q = te + 1;
    }
    return _;
  }
  function sliceText(O, _, H) {
    return appendText(O, [""], _, H);
  }
  class RawTextCursor {
    constructor(_, H = 1) {
      this.dir = H, this.done = !1, this.lineBreak = !1, this.value = "", this.nodes = [_], this.offsets = [H > 0 ? 1 : (_ instanceof TextLeaf ? _.text.length : _.children.length) << 1];
    }
    nextInner(_, H) {
      for (this.done = this.lineBreak = !1; ; ) {
        let V = this.nodes.length - 1, q = this.nodes[V], G = this.offsets[V], J = G >> 1, Z = q instanceof TextLeaf ? q.text.length : q.children.length;
        if (J == (H > 0 ? Z : 0)) {
          if (V == 0)
            return this.done = !0, this.value = "", this;
          H > 0 && this.offsets[V - 1]++, this.nodes.pop(), this.offsets.pop();
        } else if ((G & 1) == (H > 0 ? 0 : 1)) {
          if (this.offsets[V] += H, _ == 0)
            return this.lineBreak = !0, this.value = `
`, this;
          _--;
        } else if (q instanceof TextLeaf) {
          let te = q.text[J + (H < 0 ? -1 : 0)];
          if (this.offsets[V] += H, te.length > Math.max(0, _))
            return this.value = _ == 0 ? te : H > 0 ? te.slice(_) : te.slice(0, te.length - _), this;
          _ -= te.length;
        } else {
          let te = q.children[J + (H < 0 ? -1 : 0)];
          _ > te.length ? (_ -= te.length, this.offsets[V] += H) : (H < 0 && this.offsets[V]--, this.nodes.push(te), this.offsets.push(H > 0 ? 1 : (te instanceof TextLeaf ? te.text.length : te.children.length) << 1));
        }
      }
    }
    next(_ = 0) {
      return _ < 0 && (this.nextInner(-_, -this.dir), _ = this.value.length), this.nextInner(_, this.dir);
    }
  }
  class PartialTextCursor {
    constructor(_, H, V) {
      this.value = "", this.done = !1, this.cursor = new RawTextCursor(_, H > V ? -1 : 1), this.pos = H > V ? _.length : 0, this.from = Math.min(H, V), this.to = Math.max(H, V);
    }
    nextInner(_, H) {
      if (H < 0 ? this.pos <= this.from : this.pos >= this.to)
        return this.value = "", this.done = !0, this;
      _ += Math.max(0, H < 0 ? this.pos - this.to : this.from - this.pos);
      let V = H < 0 ? this.pos - this.from : this.to - this.pos;
      _ > V && (_ = V), V -= _;
      let { value: q } = this.cursor.next(_);
      return this.pos += (q.length + _) * H, this.value = q.length <= V ? q : H < 0 ? q.slice(q.length - V) : q.slice(0, V), this.done = !this.value, this;
    }
    next(_ = 0) {
      return _ < 0 ? _ = Math.max(_, this.from - this.pos) : _ > 0 && (_ = Math.min(_, this.to - this.pos)), this.nextInner(_, this.cursor.dir);
    }
    get lineBreak() {
      return this.cursor.lineBreak && this.value != "";
    }
  }
  class LineCursor {
    constructor(_) {
      this.inner = _, this.afterBreak = !0, this.value = "", this.done = !1;
    }
    next(_ = 0) {
      let { done: H, lineBreak: V, value: q } = this.inner.next(_);
      return H && this.afterBreak ? (this.value = "", this.afterBreak = !1) : H ? (this.done = !0, this.value = "") : V ? this.afterBreak ? this.value = "" : (this.afterBreak = !0, this.next()) : (this.value = q, this.afterBreak = !1), this;
    }
    get lineBreak() {
      return !1;
    }
  }
  typeof Symbol < "u" && (Text.prototype[Symbol.iterator] = function() {
    return this.iter();
  }, RawTextCursor.prototype[Symbol.iterator] = PartialTextCursor.prototype[Symbol.iterator] = LineCursor.prototype[Symbol.iterator] = function() {
    return this;
  });
  class Line {
    /**
    @internal
    */
    constructor(_, H, V, q) {
      this.from = _, this.to = H, this.number = V, this.text = q;
    }
    /**
    The length of the line (not including any line break after it).
    */
    get length() {
      return this.to - this.from;
    }
  }
  function clip(O, _, H) {
    return _ = Math.max(0, Math.min(O.length, _)), [_, Math.max(_, Math.min(O.length, H))];
  }
  let extend = /* @__PURE__ */ "lc,34,7n,7,7b,19,,,,2,,2,,,20,b,1c,l,g,,2t,7,2,6,2,2,,4,z,,u,r,2j,b,1m,9,9,,o,4,,9,,3,,5,17,3,3b,f,,w,1j,,,,4,8,4,,3,7,a,2,t,,1m,,,,2,4,8,,9,,a,2,q,,2,2,1l,,4,2,4,2,2,3,3,,u,2,3,,b,2,1l,,4,5,,2,4,,k,2,m,6,,,1m,,,2,,4,8,,7,3,a,2,u,,1n,,,,c,,9,,14,,3,,1l,3,5,3,,4,7,2,b,2,t,,1m,,2,,2,,3,,5,2,7,2,b,2,s,2,1l,2,,,2,4,8,,9,,a,2,t,,20,,4,,2,3,,,8,,29,,2,7,c,8,2q,,2,9,b,6,22,2,r,,,,,,1j,e,,5,,2,5,b,,10,9,,2u,4,,6,,2,2,2,p,2,4,3,g,4,d,,2,2,6,,f,,jj,3,qa,3,t,3,t,2,u,2,1s,2,,7,8,,2,b,9,,19,3,3b,2,y,,3a,3,4,2,9,,6,3,63,2,2,,1m,,,7,,,,,2,8,6,a,2,,1c,h,1r,4,1c,7,,,5,,14,9,c,2,w,4,2,2,,3,1k,,,2,3,,,3,1m,8,2,2,48,3,,d,,7,4,,6,,3,2,5i,1m,,5,ek,,5f,x,2da,3,3x,,2o,w,fe,6,2x,2,n9w,4,,a,w,2,28,2,7k,,3,,4,,p,2,5,,47,2,q,i,d,,12,8,p,b,1a,3,1c,,2,4,2,2,13,,1v,6,2,2,2,2,c,,8,,1b,,1f,,,3,2,2,5,2,,,16,2,8,,6m,,2,,4,,fn4,,kh,g,g,g,a6,2,gt,,6a,,45,5,1ae,3,,2,5,4,14,3,4,,4l,2,fx,4,ar,2,49,b,4w,,1i,f,1k,3,1d,4,2,2,1x,3,10,5,,8,1q,,c,2,1g,9,a,4,2,,2n,3,2,,,2,6,,4g,,3,8,l,2,1l,2,,,,,m,,e,7,3,5,5f,8,2,3,,,n,,29,,2,6,,,2,,,2,,2,6j,,2,4,6,2,,2,r,2,2d,8,2,,,2,2y,,,,2,6,,,2t,3,2,4,,5,77,9,,2,6t,,a,2,,,4,,40,4,2,2,4,,w,a,14,6,2,4,8,,9,6,2,3,1a,d,,2,ba,7,,6,,,2a,m,2,7,,2,,2,3e,6,3,,,2,,7,,,20,2,3,,,,9n,2,f0b,5,1n,7,t4,,1r,4,29,,f5k,2,43q,,,3,4,5,8,8,2,7,u,4,44,3,1iz,1j,4,1e,8,,e,,m,5,,f,11s,7,,h,2,7,,2,,5,79,7,c5,4,15s,7,31,7,240,5,gx7k,2o,3k,6o".split(",").map((O) => O ? parseInt(O, 36) : 1);
  for (let O = 1; O < extend.length; O++)
    extend[O] += extend[O - 1];
  function isExtendingChar(O) {
    for (let _ = 1; _ < extend.length; _ += 2)
      if (extend[_] > O)
        return extend[_ - 1] <= O;
    return !1;
  }
  function isRegionalIndicator(O) {
    return O >= 127462 && O <= 127487;
  }
  const ZWJ = 8205;
  function findClusterBreak(O, _, H = !0, V = !0) {
    return (H ? nextClusterBreak : prevClusterBreak)(O, _, V);
  }
  function nextClusterBreak(O, _, H) {
    if (_ == O.length)
      return _;
    _ && surrogateLow(O.charCodeAt(_)) && surrogateHigh(O.charCodeAt(_ - 1)) && _--;
    let V = codePointAt(O, _);
    for (_ += codePointSize(V); _ < O.length; ) {
      let q = codePointAt(O, _);
      if (V == ZWJ || q == ZWJ || H && isExtendingChar(q))
        _ += codePointSize(q), V = q;
      else if (isRegionalIndicator(q)) {
        let G = 0, J = _ - 2;
        for (; J >= 0 && isRegionalIndicator(codePointAt(O, J)); )
          G++, J -= 2;
        if (G % 2 == 0)
          break;
        _ += 2;
      } else
        break;
    }
    return _;
  }
  function prevClusterBreak(O, _, H) {
    for (; _ > 0; ) {
      let V = nextClusterBreak(O, _ - 2, H);
      if (V < _)
        return V;
      _--;
    }
    return 0;
  }
  function surrogateLow(O) {
    return O >= 56320 && O < 57344;
  }
  function surrogateHigh(O) {
    return O >= 55296 && O < 56320;
  }
  function codePointAt(O, _) {
    let H = O.charCodeAt(_);
    if (!surrogateHigh(H) || _ + 1 == O.length)
      return H;
    let V = O.charCodeAt(_ + 1);
    return surrogateLow(V) ? (H - 55296 << 10) + (V - 56320) + 65536 : H;
  }
  function fromCodePoint(O) {
    return O <= 65535 ? String.fromCharCode(O) : (O -= 65536, String.fromCharCode((O >> 10) + 55296, (O & 1023) + 56320));
  }
  function codePointSize(O) {
    return O < 65536 ? 1 : 2;
  }
  const DefaultSplit = /\r\n?|\n/;
  var MapMode = /* @__PURE__ */ function(O) {
    return O[O.Simple = 0] = "Simple", O[O.TrackDel = 1] = "TrackDel", O[O.TrackBefore = 2] = "TrackBefore", O[O.TrackAfter = 3] = "TrackAfter", O;
  }(MapMode || (MapMode = {}));
  class ChangeDesc {
    // Sections are encoded as pairs of integers. The first is the
    // length in the current document, and the second is -1 for
    // unaffected sections, and the length of the replacement content
    // otherwise. So an insertion would be (0, n>0), a deletion (n>0,
    // 0), and a replacement two positive numbers.
    /**
    @internal
    */
    constructor(_) {
      this.sections = _;
    }
    /**
    The length of the document before the change.
    */
    get length() {
      let _ = 0;
      for (let H = 0; H < this.sections.length; H += 2)
        _ += this.sections[H];
      return _;
    }
    /**
    The length of the document after the change.
    */
    get newLength() {
      let _ = 0;
      for (let H = 0; H < this.sections.length; H += 2) {
        let V = this.sections[H + 1];
        _ += V < 0 ? this.sections[H] : V;
      }
      return _;
    }
    /**
    False when there are actual changes in this set.
    */
    get empty() {
      return this.sections.length == 0 || this.sections.length == 2 && this.sections[1] < 0;
    }
    /**
    Iterate over the unchanged parts left by these changes. `posA`
    provides the position of the range in the old document, `posB`
    the new position in the changed document.
    */
    iterGaps(_) {
      for (let H = 0, V = 0, q = 0; H < this.sections.length; ) {
        let G = this.sections[H++], J = this.sections[H++];
        J < 0 ? (_(V, q, G), q += G) : q += J, V += G;
      }
    }
    /**
    Iterate over the ranges changed by these changes. (See
    [`ChangeSet.iterChanges`](https://codemirror.net/6/docs/ref/#state.ChangeSet.iterChanges) for a
    variant that also provides you with the inserted text.)
    `fromA`/`toA` provides the extent of the change in the starting
    document, `fromB`/`toB` the extent of the replacement in the
    changed document.
    
    When `individual` is true, adjacent changes (which are kept
    separate for [position mapping](https://codemirror.net/6/docs/ref/#state.ChangeDesc.mapPos)) are
    reported separately.
    */
    iterChangedRanges(_, H = !1) {
      iterChanges(this, _, H);
    }
    /**
    Get a description of the inverted form of these changes.
    */
    get invertedDesc() {
      let _ = [];
      for (let H = 0; H < this.sections.length; ) {
        let V = this.sections[H++], q = this.sections[H++];
        q < 0 ? _.push(V, q) : _.push(q, V);
      }
      return new ChangeDesc(_);
    }
    /**
    Compute the combined effect of applying another set of changes
    after this one. The length of the document after this set should
    match the length before `other`.
    */
    composeDesc(_) {
      return this.empty ? _ : _.empty ? this : composeSets(this, _);
    }
    /**
    Map this description, which should start with the same document
    as `other`, over another set of changes, so that it can be
    applied after it. When `before` is true, map as if the changes
    in `other` happened before the ones in `this`.
    */
    mapDesc(_, H = !1) {
      return _.empty ? this : mapSet(this, _, H);
    }
    mapPos(_, H = -1, V = MapMode.Simple) {
      let q = 0, G = 0;
      for (let J = 0; J < this.sections.length; ) {
        let Z = this.sections[J++], te = this.sections[J++], ne = q + Z;
        if (te < 0) {
          if (ne > _)
            return G + (_ - q);
          G += Z;
        } else {
          if (V != MapMode.Simple && ne >= _ && (V == MapMode.TrackDel && q < _ && ne > _ || V == MapMode.TrackBefore && q < _ || V == MapMode.TrackAfter && ne > _))
            return null;
          if (ne > _ || ne == _ && H < 0 && !Z)
            return _ == q || H < 0 ? G : G + te;
          G += te;
        }
        q = ne;
      }
      if (_ > q)
        throw new RangeError(`Position ${_} is out of range for changeset of length ${q}`);
      return G;
    }
    /**
    Check whether these changes touch a given range. When one of the
    changes entirely covers the range, the string `"cover"` is
    returned.
    */
    touchesRange(_, H = _) {
      for (let V = 0, q = 0; V < this.sections.length && q <= H; ) {
        let G = this.sections[V++], J = this.sections[V++], Z = q + G;
        if (J >= 0 && q <= H && Z >= _)
          return q < _ && Z > H ? "cover" : !0;
        q = Z;
      }
      return !1;
    }
    /**
    @internal
    */
    toString() {
      let _ = "";
      for (let H = 0; H < this.sections.length; ) {
        let V = this.sections[H++], q = this.sections[H++];
        _ += (_ ? " " : "") + V + (q >= 0 ? ":" + q : "");
      }
      return _;
    }
    /**
    Serialize this change desc to a JSON-representable value.
    */
    toJSON() {
      return this.sections;
    }
    /**
    Create a change desc from its JSON representation (as produced
    by [`toJSON`](https://codemirror.net/6/docs/ref/#state.ChangeDesc.toJSON).
    */
    static fromJSON(_) {
      if (!Array.isArray(_) || _.length % 2 || _.some((H) => typeof H != "number"))
        throw new RangeError("Invalid JSON representation of ChangeDesc");
      return new ChangeDesc(_);
    }
    /**
    @internal
    */
    static create(_) {
      return new ChangeDesc(_);
    }
  }
  class ChangeSet extends ChangeDesc {
    constructor(_, H) {
      super(_), this.inserted = H;
    }
    /**
    Apply the changes to a document, returning the modified
    document.
    */
    apply(_) {
      if (this.length != _.length)
        throw new RangeError("Applying change set to a document with the wrong length");
      return iterChanges(this, (H, V, q, G, J) => _ = _.replace(q, q + (V - H), J), !1), _;
    }
    mapDesc(_, H = !1) {
      return mapSet(this, _, H, !0);
    }
    /**
    Given the document as it existed _before_ the changes, return a
    change set that represents the inverse of this set, which could
    be used to go from the document created by the changes back to
    the document as it existed before the changes.
    */
    invert(_) {
      let H = this.sections.slice(), V = [];
      for (let q = 0, G = 0; q < H.length; q += 2) {
        let J = H[q], Z = H[q + 1];
        if (Z >= 0) {
          H[q] = Z, H[q + 1] = J;
          let te = q >> 1;
          for (; V.length < te; )
            V.push(Text.empty);
          V.push(J ? _.slice(G, G + J) : Text.empty);
        }
        G += J;
      }
      return new ChangeSet(H, V);
    }
    /**
    Combine two subsequent change sets into a single set. `other`
    must start in the document produced by `this`. If `this` goes
    `docA` → `docB` and `other` represents `docB` → `docC`, the
    returned value will represent the change `docA` → `docC`.
    */
    compose(_) {
      return this.empty ? _ : _.empty ? this : composeSets(this, _, !0);
    }
    /**
    Given another change set starting in the same document, maps this
    change set over the other, producing a new change set that can be
    applied to the document produced by applying `other`. When
    `before` is `true`, order changes as if `this` comes before
    `other`, otherwise (the default) treat `other` as coming first.
    
    Given two changes `A` and `B`, `A.compose(B.map(A))` and
    `B.compose(A.map(B, true))` will produce the same document. This
    provides a basic form of [operational
    transformation](https://en.wikipedia.org/wiki/Operational_transformation),
    and can be used for collaborative editing.
    */
    map(_, H = !1) {
      return _.empty ? this : mapSet(this, _, H, !0);
    }
    /**
    Iterate over the changed ranges in the document, calling `f` for
    each, with the range in the original document (`fromA`-`toA`)
    and the range that replaces it in the new document
    (`fromB`-`toB`).
    
    When `individual` is true, adjacent changes are reported
    separately.
    */
    iterChanges(_, H = !1) {
      iterChanges(this, _, H);
    }
    /**
    Get a [change description](https://codemirror.net/6/docs/ref/#state.ChangeDesc) for this change
    set.
    */
    get desc() {
      return ChangeDesc.create(this.sections);
    }
    /**
    @internal
    */
    filter(_) {
      let H = [], V = [], q = [], G = new SectionIter(this);
      e: for (let J = 0, Z = 0; ; ) {
        let te = J == _.length ? 1e9 : _[J++];
        for (; Z < te || Z == te && G.len == 0; ) {
          if (G.done)
            break e;
          let ee = Math.min(G.len, te - Z);
          addSection(q, ee, -1);
          let ae = G.ins == -1 ? -1 : G.off == 0 ? G.ins : 0;
          addSection(H, ee, ae), ae > 0 && addInsert(V, H, G.text), G.forward(ee), Z += ee;
        }
        let ne = _[J++];
        for (; Z < ne; ) {
          if (G.done)
            break e;
          let ee = Math.min(G.len, ne - Z);
          addSection(H, ee, -1), addSection(q, ee, G.ins == -1 ? -1 : G.off == 0 ? G.ins : 0), G.forward(ee), Z += ee;
        }
      }
      return {
        changes: new ChangeSet(H, V),
        filtered: ChangeDesc.create(q)
      };
    }
    /**
    Serialize this change set to a JSON-representable value.
    */
    toJSON() {
      let _ = [];
      for (let H = 0; H < this.sections.length; H += 2) {
        let V = this.sections[H], q = this.sections[H + 1];
        q < 0 ? _.push(V) : q == 0 ? _.push([V]) : _.push([V].concat(this.inserted[H >> 1].toJSON()));
      }
      return _;
    }
    /**
    Create a change set for the given changes, for a document of the
    given length, using `lineSep` as line separator.
    */
    static of(_, H, V) {
      let q = [], G = [], J = 0, Z = null;
      function te(ee = !1) {
        if (!ee && !q.length)
          return;
        J < H && addSection(q, H - J, -1);
        let ae = new ChangeSet(q, G);
        Z = Z ? Z.compose(ae.map(Z)) : ae, q = [], G = [], J = 0;
      }
      function ne(ee) {
        if (Array.isArray(ee))
          for (let ae of ee)
            ne(ae);
        else if (ee instanceof ChangeSet) {
          if (ee.length != H)
            throw new RangeError(`Mismatched change set length (got ${ee.length}, expected ${H})`);
          te(), Z = Z ? Z.compose(ee.map(Z)) : ee;
        } else {
          let { from: ae, to: oe = ae, insert: se } = ee;
          if (ae > oe || ae < 0 || oe > H)
            throw new RangeError(`Invalid change range ${ae} to ${oe} (in doc of length ${H})`);
          let ce = se ? typeof se == "string" ? Text.of(se.split(V || DefaultSplit)) : se : Text.empty, le = ce.length;
          if (ae == oe && le == 0)
            return;
          ae < J && te(), ae > J && addSection(q, ae - J, -1), addSection(q, oe - ae, le), addInsert(G, q, ce), J = oe;
        }
      }
      return ne(_), te(!Z), Z;
    }
    /**
    Create an empty changeset of the given length.
    */
    static empty(_) {
      return new ChangeSet(_ ? [_, -1] : [], []);
    }
    /**
    Create a changeset from its JSON representation (as produced by
    [`toJSON`](https://codemirror.net/6/docs/ref/#state.ChangeSet.toJSON).
    */
    static fromJSON(_) {
      if (!Array.isArray(_))
        throw new RangeError("Invalid JSON representation of ChangeSet");
      let H = [], V = [];
      for (let q = 0; q < _.length; q++) {
        let G = _[q];
        if (typeof G == "number")
          H.push(G, -1);
        else {
          if (!Array.isArray(G) || typeof G[0] != "number" || G.some((J, Z) => Z && typeof J != "string"))
            throw new RangeError("Invalid JSON representation of ChangeSet");
          if (G.length == 1)
            H.push(G[0], 0);
          else {
            for (; V.length < q; )
              V.push(Text.empty);
            V[q] = Text.of(G.slice(1)), H.push(G[0], V[q].length);
          }
        }
      }
      return new ChangeSet(H, V);
    }
    /**
    @internal
    */
    static createSet(_, H) {
      return new ChangeSet(_, H);
    }
  }
  function addSection(O, _, H, V = !1) {
    if (_ == 0 && H <= 0)
      return;
    let q = O.length - 2;
    q >= 0 && H <= 0 && H == O[q + 1] ? O[q] += _ : _ == 0 && O[q] == 0 ? O[q + 1] += H : V ? (O[q] += _, O[q + 1] += H) : O.push(_, H);
  }
  function addInsert(O, _, H) {
    if (H.length == 0)
      return;
    let V = _.length - 2 >> 1;
    if (V < O.length)
      O[O.length - 1] = O[O.length - 1].append(H);
    else {
      for (; O.length < V; )
        O.push(Text.empty);
      O.push(H);
    }
  }
  function iterChanges(O, _, H) {
    let V = O.inserted;
    for (let q = 0, G = 0, J = 0; J < O.sections.length; ) {
      let Z = O.sections[J++], te = O.sections[J++];
      if (te < 0)
        q += Z, G += Z;
      else {
        let ne = q, ee = G, ae = Text.empty;
        for (; ne += Z, ee += te, te && V && (ae = ae.append(V[J - 2 >> 1])), !(H || J == O.sections.length || O.sections[J + 1] < 0); )
          Z = O.sections[J++], te = O.sections[J++];
        _(q, ne, G, ee, ae), q = ne, G = ee;
      }
    }
  }
  function mapSet(O, _, H, V = !1) {
    let q = [], G = V ? [] : null, J = new SectionIter(O), Z = new SectionIter(_);
    for (let te = -1; ; )
      if (J.ins == -1 && Z.ins == -1) {
        let ne = Math.min(J.len, Z.len);
        addSection(q, ne, -1), J.forward(ne), Z.forward(ne);
      } else if (Z.ins >= 0 && (J.ins < 0 || te == J.i || J.off == 0 && (Z.len < J.len || Z.len == J.len && !H))) {
        let ne = Z.len;
        for (addSection(q, Z.ins, -1); ne; ) {
          let ee = Math.min(J.len, ne);
          J.ins >= 0 && te < J.i && J.len <= ee && (addSection(q, 0, J.ins), G && addInsert(G, q, J.text), te = J.i), J.forward(ee), ne -= ee;
        }
        Z.next();
      } else if (J.ins >= 0) {
        let ne = 0, ee = J.len;
        for (; ee; )
          if (Z.ins == -1) {
            let ae = Math.min(ee, Z.len);
            ne += ae, ee -= ae, Z.forward(ae);
          } else if (Z.ins == 0 && Z.len < ee)
            ee -= Z.len, Z.next();
          else
            break;
        addSection(q, ne, te < J.i ? J.ins : 0), G && te < J.i && addInsert(G, q, J.text), te = J.i, J.forward(J.len - ee);
      } else {
        if (J.done && Z.done)
          return G ? ChangeSet.createSet(q, G) : ChangeDesc.create(q);
        throw new Error("Mismatched change set lengths");
      }
  }
  function composeSets(O, _, H = !1) {
    let V = [], q = H ? [] : null, G = new SectionIter(O), J = new SectionIter(_);
    for (let Z = !1; ; ) {
      if (G.done && J.done)
        return q ? ChangeSet.createSet(V, q) : ChangeDesc.create(V);
      if (G.ins == 0)
        addSection(V, G.len, 0, Z), G.next();
      else if (J.len == 0 && !J.done)
        addSection(V, 0, J.ins, Z), q && addInsert(q, V, J.text), J.next();
      else {
        if (G.done || J.done)
          throw new Error("Mismatched change set lengths");
        {
          let te = Math.min(G.len2, J.len), ne = V.length;
          if (G.ins == -1) {
            let ee = J.ins == -1 ? -1 : J.off ? 0 : J.ins;
            addSection(V, te, ee, Z), q && ee && addInsert(q, V, J.text);
          } else J.ins == -1 ? (addSection(V, G.off ? 0 : G.len, te, Z), q && addInsert(q, V, G.textBit(te))) : (addSection(V, G.off ? 0 : G.len, J.off ? 0 : J.ins, Z), q && !J.off && addInsert(q, V, J.text));
          Z = (G.ins > te || J.ins >= 0 && J.len > te) && (Z || V.length > ne), G.forward2(te), J.forward(te);
        }
      }
    }
  }
  class SectionIter {
    constructor(_) {
      this.set = _, this.i = 0, this.next();
    }
    next() {
      let { sections: _ } = this.set;
      this.i < _.length ? (this.len = _[this.i++], this.ins = _[this.i++]) : (this.len = 0, this.ins = -2), this.off = 0;
    }
    get done() {
      return this.ins == -2;
    }
    get len2() {
      return this.ins < 0 ? this.len : this.ins;
    }
    get text() {
      let { inserted: _ } = this.set, H = this.i - 2 >> 1;
      return H >= _.length ? Text.empty : _[H];
    }
    textBit(_) {
      let { inserted: H } = this.set, V = this.i - 2 >> 1;
      return V >= H.length && !_ ? Text.empty : H[V].slice(this.off, _ == null ? void 0 : this.off + _);
    }
    forward(_) {
      _ == this.len ? this.next() : (this.len -= _, this.off += _);
    }
    forward2(_) {
      this.ins == -1 ? this.forward(_) : _ == this.ins ? this.next() : (this.ins -= _, this.off += _);
    }
  }
  class SelectionRange {
    constructor(_, H, V) {
      this.from = _, this.to = H, this.flags = V;
    }
    /**
    The anchor of the range—the side that doesn't move when you
    extend it.
    */
    get anchor() {
      return this.flags & 32 ? this.to : this.from;
    }
    /**
    The head of the range, which is moved when the range is
    [extended](https://codemirror.net/6/docs/ref/#state.SelectionRange.extend).
    */
    get head() {
      return this.flags & 32 ? this.from : this.to;
    }
    /**
    True when `anchor` and `head` are at the same position.
    */
    get empty() {
      return this.from == this.to;
    }
    /**
    If this is a cursor that is explicitly associated with the
    character on one of its sides, this returns the side. -1 means
    the character before its position, 1 the character after, and 0
    means no association.
    */
    get assoc() {
      return this.flags & 8 ? -1 : this.flags & 16 ? 1 : 0;
    }
    /**
    The bidirectional text level associated with this cursor, if
    any.
    */
    get bidiLevel() {
      let _ = this.flags & 7;
      return _ == 7 ? null : _;
    }
    /**
    The goal column (stored vertical offset) associated with a
    cursor. This is used to preserve the vertical position when
    [moving](https://codemirror.net/6/docs/ref/#view.EditorView.moveVertically) across
    lines of different length.
    */
    get goalColumn() {
      let _ = this.flags >> 6;
      return _ == 16777215 ? void 0 : _;
    }
    /**
    Map this range through a change, producing a valid range in the
    updated document.
    */
    map(_, H = -1) {
      let V, q;
      return this.empty ? V = q = _.mapPos(this.from, H) : (V = _.mapPos(this.from, 1), q = _.mapPos(this.to, -1)), V == this.from && q == this.to ? this : new SelectionRange(V, q, this.flags);
    }
    /**
    Extend this range to cover at least `from` to `to`.
    */
    extend(_, H = _) {
      if (_ <= this.anchor && H >= this.anchor)
        return EditorSelection.range(_, H);
      let V = Math.abs(_ - this.anchor) > Math.abs(H - this.anchor) ? _ : H;
      return EditorSelection.range(this.anchor, V);
    }
    /**
    Compare this range to another range.
    */
    eq(_, H = !1) {
      return this.anchor == _.anchor && this.head == _.head && (!H || !this.empty || this.assoc == _.assoc);
    }
    /**
    Return a JSON-serializable object representing the range.
    */
    toJSON() {
      return { anchor: this.anchor, head: this.head };
    }
    /**
    Convert a JSON representation of a range to a `SelectionRange`
    instance.
    */
    static fromJSON(_) {
      if (!_ || typeof _.anchor != "number" || typeof _.head != "number")
        throw new RangeError("Invalid JSON representation for SelectionRange");
      return EditorSelection.range(_.anchor, _.head);
    }
    /**
    @internal
    */
    static create(_, H, V) {
      return new SelectionRange(_, H, V);
    }
  }
  class EditorSelection {
    constructor(_, H) {
      this.ranges = _, this.mainIndex = H;
    }
    /**
    Map a selection through a change. Used to adjust the selection
    position for changes.
    */
    map(_, H = -1) {
      return _.empty ? this : EditorSelection.create(this.ranges.map((V) => V.map(_, H)), this.mainIndex);
    }
    /**
    Compare this selection to another selection. By default, ranges
    are compared only by position. When `includeAssoc` is true,
    cursor ranges must also have the same
    [`assoc`](https://codemirror.net/6/docs/ref/#state.SelectionRange.assoc) value.
    */
    eq(_, H = !1) {
      if (this.ranges.length != _.ranges.length || this.mainIndex != _.mainIndex)
        return !1;
      for (let V = 0; V < this.ranges.length; V++)
        if (!this.ranges[V].eq(_.ranges[V], H))
          return !1;
      return !0;
    }
    /**
    Get the primary selection range. Usually, you should make sure
    your code applies to _all_ ranges, by using methods like
    [`changeByRange`](https://codemirror.net/6/docs/ref/#state.EditorState.changeByRange).
    */
    get main() {
      return this.ranges[this.mainIndex];
    }
    /**
    Make sure the selection only has one range. Returns a selection
    holding only the main range from this selection.
    */
    asSingle() {
      return this.ranges.length == 1 ? this : new EditorSelection([this.main], 0);
    }
    /**
    Extend this selection with an extra range.
    */
    addRange(_, H = !0) {
      return EditorSelection.create([_].concat(this.ranges), H ? 0 : this.mainIndex + 1);
    }
    /**
    Replace a given range with another range, and then normalize the
    selection to merge and sort ranges if necessary.
    */
    replaceRange(_, H = this.mainIndex) {
      let V = this.ranges.slice();
      return V[H] = _, EditorSelection.create(V, this.mainIndex);
    }
    /**
    Convert this selection to an object that can be serialized to
    JSON.
    */
    toJSON() {
      return { ranges: this.ranges.map((_) => _.toJSON()), main: this.mainIndex };
    }
    /**
    Create a selection from a JSON representation.
    */
    static fromJSON(_) {
      if (!_ || !Array.isArray(_.ranges) || typeof _.main != "number" || _.main >= _.ranges.length)
        throw new RangeError("Invalid JSON representation for EditorSelection");
      return new EditorSelection(_.ranges.map((H) => SelectionRange.fromJSON(H)), _.main);
    }
    /**
    Create a selection holding a single range.
    */
    static single(_, H = _) {
      return new EditorSelection([EditorSelection.range(_, H)], 0);
    }
    /**
    Sort and merge the given set of ranges, creating a valid
    selection.
    */
    static create(_, H = 0) {
      if (_.length == 0)
        throw new RangeError("A selection needs at least one range");
      for (let V = 0, q = 0; q < _.length; q++) {
        let G = _[q];
        if (G.empty ? G.from <= V : G.from < V)
          return EditorSelection.normalized(_.slice(), H);
        V = G.to;
      }
      return new EditorSelection(_, H);
    }
    /**
    Create a cursor selection range at the given position. You can
    safely ignore the optional arguments in most situations.
    */
    static cursor(_, H = 0, V, q) {
      return SelectionRange.create(_, _, (H == 0 ? 0 : H < 0 ? 8 : 16) | (V == null ? 7 : Math.min(6, V)) | (q ?? 16777215) << 6);
    }
    /**
    Create a selection range.
    */
    static range(_, H, V, q) {
      let G = (V ?? 16777215) << 6 | (q == null ? 7 : Math.min(6, q));
      return H < _ ? SelectionRange.create(H, _, 48 | G) : SelectionRange.create(_, H, (H > _ ? 8 : 0) | G);
    }
    /**
    @internal
    */
    static normalized(_, H = 0) {
      let V = _[H];
      _.sort((q, G) => q.from - G.from), H = _.indexOf(V);
      for (let q = 1; q < _.length; q++) {
        let G = _[q], J = _[q - 1];
        if (G.empty ? G.from <= J.to : G.from < J.to) {
          let Z = J.from, te = Math.max(G.to, J.to);
          q <= H && H--, _.splice(--q, 2, G.anchor > G.head ? EditorSelection.range(te, Z) : EditorSelection.range(Z, te));
        }
      }
      return new EditorSelection(_, H);
    }
  }
  function checkSelection(O, _) {
    for (let H of O.ranges)
      if (H.to > _)
        throw new RangeError("Selection points outside of document");
  }
  let nextID = 0;
  class Facet {
    constructor(_, H, V, q, G) {
      this.combine = _, this.compareInput = H, this.compare = V, this.isStatic = q, this.id = nextID++, this.default = _([]), this.extensions = typeof G == "function" ? G(this) : G;
    }
    /**
    Returns a facet reader for this facet, which can be used to
    [read](https://codemirror.net/6/docs/ref/#state.EditorState.facet) it but not to define values for it.
    */
    get reader() {
      return this;
    }
    /**
    Define a new facet.
    */
    static define(_ = {}) {
      return new Facet(_.combine || ((H) => H), _.compareInput || ((H, V) => H === V), _.compare || (_.combine ? (H, V) => H === V : sameArray$1), !!_.static, _.enables);
    }
    /**
    Returns an extension that adds the given value to this facet.
    */
    of(_) {
      return new FacetProvider([], this, 0, _);
    }
    /**
    Create an extension that computes a value for the facet from a
    state. You must take care to declare the parts of the state that
    this value depends on, since your function is only called again
    for a new state when one of those parts changed.
    
    In cases where your value depends only on a single field, you'll
    want to use the [`from`](https://codemirror.net/6/docs/ref/#state.Facet.from) method instead.
    */
    compute(_, H) {
      if (this.isStatic)
        throw new Error("Can't compute a static facet");
      return new FacetProvider(_, this, 1, H);
    }
    /**
    Create an extension that computes zero or more values for this
    facet from a state.
    */
    computeN(_, H) {
      if (this.isStatic)
        throw new Error("Can't compute a static facet");
      return new FacetProvider(_, this, 2, H);
    }
    from(_, H) {
      return H || (H = (V) => V), this.compute([_], (V) => H(V.field(_)));
    }
  }
  function sameArray$1(O, _) {
    return O == _ || O.length == _.length && O.every((H, V) => H === _[V]);
  }
  class FacetProvider {
    constructor(_, H, V, q) {
      this.dependencies = _, this.facet = H, this.type = V, this.value = q, this.id = nextID++;
    }
    dynamicSlot(_) {
      var H;
      let V = this.value, q = this.facet.compareInput, G = this.id, J = _[G] >> 1, Z = this.type == 2, te = !1, ne = !1, ee = [];
      for (let ae of this.dependencies)
        ae == "doc" ? te = !0 : ae == "selection" ? ne = !0 : ((H = _[ae.id]) !== null && H !== void 0 ? H : 1) & 1 || ee.push(_[ae.id]);
      return {
        create(ae) {
          return ae.values[J] = V(ae), 1;
        },
        update(ae, oe) {
          if (te && oe.docChanged || ne && (oe.docChanged || oe.selection) || ensureAll(ae, ee)) {
            let se = V(ae);
            if (Z ? !compareArray$1(se, ae.values[J], q) : !q(se, ae.values[J]))
              return ae.values[J] = se, 1;
          }
          return 0;
        },
        reconfigure: (ae, oe) => {
          let se, ce = oe.config.address[G];
          if (ce != null) {
            let le = getAddr(oe, ce);
            if (this.dependencies.every((ue) => ue instanceof Facet ? oe.facet(ue) === ae.facet(ue) : ue instanceof StateField ? oe.field(ue, !1) == ae.field(ue, !1) : !0) || (Z ? compareArray$1(se = V(ae), le, q) : q(se = V(ae), le)))
              return ae.values[J] = le, 0;
          } else
            se = V(ae);
          return ae.values[J] = se, 1;
        }
      };
    }
  }
  function compareArray$1(O, _, H) {
    if (O.length != _.length)
      return !1;
    for (let V = 0; V < O.length; V++)
      if (!H(O[V], _[V]))
        return !1;
    return !0;
  }
  function ensureAll(O, _) {
    let H = !1;
    for (let V of _)
      ensureAddr(O, V) & 1 && (H = !0);
    return H;
  }
  function dynamicFacetSlot(O, _, H) {
    let V = H.map((te) => O[te.id]), q = H.map((te) => te.type), G = V.filter((te) => !(te & 1)), J = O[_.id] >> 1;
    function Z(te) {
      let ne = [];
      for (let ee = 0; ee < V.length; ee++) {
        let ae = getAddr(te, V[ee]);
        if (q[ee] == 2)
          for (let oe of ae)
            ne.push(oe);
        else
          ne.push(ae);
      }
      return _.combine(ne);
    }
    return {
      create(te) {
        for (let ne of V)
          ensureAddr(te, ne);
        return te.values[J] = Z(te), 1;
      },
      update(te, ne) {
        if (!ensureAll(te, G))
          return 0;
        let ee = Z(te);
        return _.compare(ee, te.values[J]) ? 0 : (te.values[J] = ee, 1);
      },
      reconfigure(te, ne) {
        let ee = ensureAll(te, V), ae = ne.config.facets[_.id], oe = ne.facet(_);
        if (ae && !ee && sameArray$1(H, ae))
          return te.values[J] = oe, 0;
        let se = Z(te);
        return _.compare(se, oe) ? (te.values[J] = oe, 0) : (te.values[J] = se, 1);
      }
    };
  }
  const initField = /* @__PURE__ */ Facet.define({ static: !0 });
  class StateField {
    constructor(_, H, V, q, G) {
      this.id = _, this.createF = H, this.updateF = V, this.compareF = q, this.spec = G, this.provides = void 0;
    }
    /**
    Define a state field.
    */
    static define(_) {
      let H = new StateField(nextID++, _.create, _.update, _.compare || ((V, q) => V === q), _);
      return _.provide && (H.provides = _.provide(H)), H;
    }
    create(_) {
      let H = _.facet(initField).find((V) => V.field == this);
      return ((H == null ? void 0 : H.create) || this.createF)(_);
    }
    /**
    @internal
    */
    slot(_) {
      let H = _[this.id] >> 1;
      return {
        create: (V) => (V.values[H] = this.create(V), 1),
        update: (V, q) => {
          let G = V.values[H], J = this.updateF(G, q);
          return this.compareF(G, J) ? 0 : (V.values[H] = J, 1);
        },
        reconfigure: (V, q) => q.config.address[this.id] != null ? (V.values[H] = q.field(this), 0) : (V.values[H] = this.create(V), 1)
      };
    }
    /**
    Returns an extension that enables this field and overrides the
    way it is initialized. Can be useful when you need to provide a
    non-default starting value for the field.
    */
    init(_) {
      return [this, initField.of({ field: this, create: _ })];
    }
    /**
    State field instances can be used as
    [`Extension`](https://codemirror.net/6/docs/ref/#state.Extension) values to enable the field in a
    given state.
    */
    get extension() {
      return this;
    }
  }
  const Prec_ = { lowest: 4, low: 3, default: 2, high: 1, highest: 0 };
  function prec(O) {
    return (_) => new PrecExtension(_, O);
  }
  const Prec = {
    /**
    The highest precedence level, for extensions that should end up
    near the start of the precedence ordering.
    */
    highest: /* @__PURE__ */ prec(Prec_.highest),
    /**
    A higher-than-default precedence, for extensions that should
    come before those with default precedence.
    */
    high: /* @__PURE__ */ prec(Prec_.high),
    /**
    The default precedence, which is also used for extensions
    without an explicit precedence.
    */
    default: /* @__PURE__ */ prec(Prec_.default),
    /**
    A lower-than-default precedence.
    */
    low: /* @__PURE__ */ prec(Prec_.low),
    /**
    The lowest precedence level. Meant for things that should end up
    near the end of the extension order.
    */
    lowest: /* @__PURE__ */ prec(Prec_.lowest)
  };
  class PrecExtension {
    constructor(_, H) {
      this.inner = _, this.prec = H;
    }
  }
  class Compartment {
    /**
    Create an instance of this compartment to add to your [state
    configuration](https://codemirror.net/6/docs/ref/#state.EditorStateConfig.extensions).
    */
    of(_) {
      return new CompartmentInstance(this, _);
    }
    /**
    Create an [effect](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) that
    reconfigures this compartment.
    */
    reconfigure(_) {
      return Compartment.reconfigure.of({ compartment: this, extension: _ });
    }
    /**
    Get the current content of the compartment in the state, or
    `undefined` if it isn't present.
    */
    get(_) {
      return _.config.compartments.get(this);
    }
  }
  class CompartmentInstance {
    constructor(_, H) {
      this.compartment = _, this.inner = H;
    }
  }
  class Configuration {
    constructor(_, H, V, q, G, J) {
      for (this.base = _, this.compartments = H, this.dynamicSlots = V, this.address = q, this.staticValues = G, this.facets = J, this.statusTemplate = []; this.statusTemplate.length < V.length; )
        this.statusTemplate.push(
          0
          /* SlotStatus.Unresolved */
        );
    }
    staticFacet(_) {
      let H = this.address[_.id];
      return H == null ? _.default : this.staticValues[H >> 1];
    }
    static resolve(_, H, V) {
      let q = [], G = /* @__PURE__ */ Object.create(null), J = /* @__PURE__ */ new Map();
      for (let oe of flatten(_, H, J))
        oe instanceof StateField ? q.push(oe) : (G[oe.facet.id] || (G[oe.facet.id] = [])).push(oe);
      let Z = /* @__PURE__ */ Object.create(null), te = [], ne = [];
      for (let oe of q)
        Z[oe.id] = ne.length << 1, ne.push((se) => oe.slot(se));
      let ee = V == null ? void 0 : V.config.facets;
      for (let oe in G) {
        let se = G[oe], ce = se[0].facet, le = ee && ee[oe] || [];
        if (se.every(
          (ue) => ue.type == 0
          /* Provider.Static */
        ))
          if (Z[ce.id] = te.length << 1 | 1, sameArray$1(le, se))
            te.push(V.facet(ce));
          else {
            let ue = ce.combine(se.map((de) => de.value));
            te.push(V && ce.compare(ue, V.facet(ce)) ? V.facet(ce) : ue);
          }
        else {
          for (let ue of se)
            ue.type == 0 ? (Z[ue.id] = te.length << 1 | 1, te.push(ue.value)) : (Z[ue.id] = ne.length << 1, ne.push((de) => ue.dynamicSlot(de)));
          Z[ce.id] = ne.length << 1, ne.push((ue) => dynamicFacetSlot(ue, ce, se));
        }
      }
      let ae = ne.map((oe) => oe(Z));
      return new Configuration(_, J, ae, Z, te, G);
    }
  }
  function flatten(O, _, H) {
    let V = [[], [], [], [], []], q = /* @__PURE__ */ new Map();
    function G(J, Z) {
      let te = q.get(J);
      if (te != null) {
        if (te <= Z)
          return;
        let ne = V[te].indexOf(J);
        ne > -1 && V[te].splice(ne, 1), J instanceof CompartmentInstance && H.delete(J.compartment);
      }
      if (q.set(J, Z), Array.isArray(J))
        for (let ne of J)
          G(ne, Z);
      else if (J instanceof CompartmentInstance) {
        if (H.has(J.compartment))
          throw new RangeError("Duplicate use of compartment in extensions");
        let ne = _.get(J.compartment) || J.inner;
        H.set(J.compartment, ne), G(ne, Z);
      } else if (J instanceof PrecExtension)
        G(J.inner, J.prec);
      else if (J instanceof StateField)
        V[Z].push(J), J.provides && G(J.provides, Z);
      else if (J instanceof FacetProvider)
        V[Z].push(J), J.facet.extensions && G(J.facet.extensions, Prec_.default);
      else {
        let ne = J.extension;
        if (!ne)
          throw new Error(`Unrecognized extension value in extension set (${J}). This sometimes happens because multiple instances of @codemirror/state are loaded, breaking instanceof checks.`);
        G(ne, Z);
      }
    }
    return G(O, Prec_.default), V.reduce((J, Z) => J.concat(Z));
  }
  function ensureAddr(O, _) {
    if (_ & 1)
      return 2;
    let H = _ >> 1, V = O.status[H];
    if (V == 4)
      throw new Error("Cyclic dependency between fields and/or facets");
    if (V & 2)
      return V;
    O.status[H] = 4;
    let q = O.computeSlot(O, O.config.dynamicSlots[H]);
    return O.status[H] = 2 | q;
  }
  function getAddr(O, _) {
    return _ & 1 ? O.config.staticValues[_ >> 1] : O.values[_ >> 1];
  }
  const languageData = /* @__PURE__ */ Facet.define(), allowMultipleSelections = /* @__PURE__ */ Facet.define({
    combine: (O) => O.some((_) => _),
    static: !0
  }), lineSeparator = /* @__PURE__ */ Facet.define({
    combine: (O) => O.length ? O[0] : void 0,
    static: !0
  }), changeFilter = /* @__PURE__ */ Facet.define(), transactionFilter = /* @__PURE__ */ Facet.define(), transactionExtender = /* @__PURE__ */ Facet.define(), readOnly = /* @__PURE__ */ Facet.define({
    combine: (O) => O.length ? O[0] : !1
  });
  class Annotation {
    /**
    @internal
    */
    constructor(_, H) {
      this.type = _, this.value = H;
    }
    /**
    Define a new type of annotation.
    */
    static define() {
      return new AnnotationType();
    }
  }
  class AnnotationType {
    /**
    Create an instance of this annotation.
    */
    of(_) {
      return new Annotation(this, _);
    }
  }
  class StateEffectType {
    /**
    @internal
    */
    constructor(_) {
      this.map = _;
    }
    /**
    Create a [state effect](https://codemirror.net/6/docs/ref/#state.StateEffect) instance of this
    type.
    */
    of(_) {
      return new StateEffect(this, _);
    }
  }
  class StateEffect {
    /**
    @internal
    */
    constructor(_, H) {
      this.type = _, this.value = H;
    }
    /**
    Map this effect through a position mapping. Will return
    `undefined` when that ends up deleting the effect.
    */
    map(_) {
      let H = this.type.map(this.value, _);
      return H === void 0 ? void 0 : H == this.value ? this : new StateEffect(this.type, H);
    }
    /**
    Tells you whether this effect object is of a given
    [type](https://codemirror.net/6/docs/ref/#state.StateEffectType).
    */
    is(_) {
      return this.type == _;
    }
    /**
    Define a new effect type. The type parameter indicates the type
    of values that his effect holds. It should be a type that
    doesn't include `undefined`, since that is used in
    [mapping](https://codemirror.net/6/docs/ref/#state.StateEffect.map) to indicate that an effect is
    removed.
    */
    static define(_ = {}) {
      return new StateEffectType(_.map || ((H) => H));
    }
    /**
    Map an array of effects through a change set.
    */
    static mapEffects(_, H) {
      if (!_.length)
        return _;
      let V = [];
      for (let q of _) {
        let G = q.map(H);
        G && V.push(G);
      }
      return V;
    }
  }
  StateEffect.reconfigure = /* @__PURE__ */ StateEffect.define();
  StateEffect.appendConfig = /* @__PURE__ */ StateEffect.define();
  class Transaction {
    constructor(_, H, V, q, G, J) {
      this.startState = _, this.changes = H, this.selection = V, this.effects = q, this.annotations = G, this.scrollIntoView = J, this._doc = null, this._state = null, V && checkSelection(V, H.newLength), G.some((Z) => Z.type == Transaction.time) || (this.annotations = G.concat(Transaction.time.of(Date.now())));
    }
    /**
    @internal
    */
    static create(_, H, V, q, G, J) {
      return new Transaction(_, H, V, q, G, J);
    }
    /**
    The new document produced by the transaction. Contrary to
    [`.state`](https://codemirror.net/6/docs/ref/#state.Transaction.state)`.doc`, accessing this won't
    force the entire new state to be computed right away, so it is
    recommended that [transaction
    filters](https://codemirror.net/6/docs/ref/#state.EditorState^transactionFilter) use this getter
    when they need to look at the new document.
    */
    get newDoc() {
      return this._doc || (this._doc = this.changes.apply(this.startState.doc));
    }
    /**
    The new selection produced by the transaction. If
    [`this.selection`](https://codemirror.net/6/docs/ref/#state.Transaction.selection) is undefined,
    this will [map](https://codemirror.net/6/docs/ref/#state.EditorSelection.map) the start state's
    current selection through the changes made by the transaction.
    */
    get newSelection() {
      return this.selection || this.startState.selection.map(this.changes);
    }
    /**
    The new state created by the transaction. Computed on demand
    (but retained for subsequent access), so it is recommended not to
    access it in [transaction
    filters](https://codemirror.net/6/docs/ref/#state.EditorState^transactionFilter) when possible.
    */
    get state() {
      return this._state || this.startState.applyTransaction(this), this._state;
    }
    /**
    Get the value of the given annotation type, if any.
    */
    annotation(_) {
      for (let H of this.annotations)
        if (H.type == _)
          return H.value;
    }
    /**
    Indicates whether the transaction changed the document.
    */
    get docChanged() {
      return !this.changes.empty;
    }
    /**
    Indicates whether this transaction reconfigures the state
    (through a [configuration compartment](https://codemirror.net/6/docs/ref/#state.Compartment) or
    with a top-level configuration
    [effect](https://codemirror.net/6/docs/ref/#state.StateEffect^reconfigure).
    */
    get reconfigured() {
      return this.startState.config != this.state.config;
    }
    /**
    Returns true if the transaction has a [user
    event](https://codemirror.net/6/docs/ref/#state.Transaction^userEvent) annotation that is equal to
    or more specific than `event`. For example, if the transaction
    has `"select.pointer"` as user event, `"select"` and
    `"select.pointer"` will match it.
    */
    isUserEvent(_) {
      let H = this.annotation(Transaction.userEvent);
      return !!(H && (H == _ || H.length > _.length && H.slice(0, _.length) == _ && H[_.length] == "."));
    }
  }
  Transaction.time = /* @__PURE__ */ Annotation.define();
  Transaction.userEvent = /* @__PURE__ */ Annotation.define();
  Transaction.addToHistory = /* @__PURE__ */ Annotation.define();
  Transaction.remote = /* @__PURE__ */ Annotation.define();
  function joinRanges(O, _) {
    let H = [];
    for (let V = 0, q = 0; ; ) {
      let G, J;
      if (V < O.length && (q == _.length || _[q] >= O[V]))
        G = O[V++], J = O[V++];
      else if (q < _.length)
        G = _[q++], J = _[q++];
      else
        return H;
      !H.length || H[H.length - 1] < G ? H.push(G, J) : H[H.length - 1] < J && (H[H.length - 1] = J);
    }
  }
  function mergeTransaction(O, _, H) {
    var V;
    let q, G, J;
    return H ? (q = _.changes, G = ChangeSet.empty(_.changes.length), J = O.changes.compose(_.changes)) : (q = _.changes.map(O.changes), G = O.changes.mapDesc(_.changes, !0), J = O.changes.compose(q)), {
      changes: J,
      selection: _.selection ? _.selection.map(G) : (V = O.selection) === null || V === void 0 ? void 0 : V.map(q),
      effects: StateEffect.mapEffects(O.effects, q).concat(StateEffect.mapEffects(_.effects, G)),
      annotations: O.annotations.length ? O.annotations.concat(_.annotations) : _.annotations,
      scrollIntoView: O.scrollIntoView || _.scrollIntoView
    };
  }
  function resolveTransactionInner(O, _, H) {
    let V = _.selection, q = asArray$1(_.annotations);
    return _.userEvent && (q = q.concat(Transaction.userEvent.of(_.userEvent))), {
      changes: _.changes instanceof ChangeSet ? _.changes : ChangeSet.of(_.changes || [], H, O.facet(lineSeparator)),
      selection: V && (V instanceof EditorSelection ? V : EditorSelection.single(V.anchor, V.head)),
      effects: asArray$1(_.effects),
      annotations: q,
      scrollIntoView: !!_.scrollIntoView
    };
  }
  function resolveTransaction(O, _, H) {
    let V = resolveTransactionInner(O, _.length ? _[0] : {}, O.doc.length);
    _.length && _[0].filter === !1 && (H = !1);
    for (let G = 1; G < _.length; G++) {
      _[G].filter === !1 && (H = !1);
      let J = !!_[G].sequential;
      V = mergeTransaction(V, resolveTransactionInner(O, _[G], J ? V.changes.newLength : O.doc.length), J);
    }
    let q = Transaction.create(O, V.changes, V.selection, V.effects, V.annotations, V.scrollIntoView);
    return extendTransaction(H ? filterTransaction(q) : q);
  }
  function filterTransaction(O) {
    let _ = O.startState, H = !0;
    for (let q of _.facet(changeFilter)) {
      let G = q(O);
      if (G === !1) {
        H = !1;
        break;
      }
      Array.isArray(G) && (H = H === !0 ? G : joinRanges(H, G));
    }
    if (H !== !0) {
      let q, G;
      if (H === !1)
        G = O.changes.invertedDesc, q = ChangeSet.empty(_.doc.length);
      else {
        let J = O.changes.filter(H);
        q = J.changes, G = J.filtered.mapDesc(J.changes).invertedDesc;
      }
      O = Transaction.create(_, q, O.selection && O.selection.map(G), StateEffect.mapEffects(O.effects, G), O.annotations, O.scrollIntoView);
    }
    let V = _.facet(transactionFilter);
    for (let q = V.length - 1; q >= 0; q--) {
      let G = V[q](O);
      G instanceof Transaction ? O = G : Array.isArray(G) && G.length == 1 && G[0] instanceof Transaction ? O = G[0] : O = resolveTransaction(_, asArray$1(G), !1);
    }
    return O;
  }
  function extendTransaction(O) {
    let _ = O.startState, H = _.facet(transactionExtender), V = O;
    for (let q = H.length - 1; q >= 0; q--) {
      let G = H[q](O);
      G && Object.keys(G).length && (V = mergeTransaction(V, resolveTransactionInner(_, G, O.changes.newLength), !0));
    }
    return V == O ? O : Transaction.create(_, O.changes, O.selection, V.effects, V.annotations, V.scrollIntoView);
  }
  const none$2 = [];
  function asArray$1(O) {
    return O == null ? none$2 : Array.isArray(O) ? O : [O];
  }
  var CharCategory = /* @__PURE__ */ function(O) {
    return O[O.Word = 0] = "Word", O[O.Space = 1] = "Space", O[O.Other = 2] = "Other", O;
  }(CharCategory || (CharCategory = {}));
  const nonASCIISingleCaseWordChar = /[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;
  let wordChar;
  try {
    wordChar = /* @__PURE__ */ new RegExp("[\\p{Alphabetic}\\p{Number}_]", "u");
  } catch (O) {
  }
  function hasWordChar(O) {
    if (wordChar)
      return wordChar.test(O);
    for (let _ = 0; _ < O.length; _++) {
      let H = O[_];
      if (/\w/.test(H) || H > "" && (H.toUpperCase() != H.toLowerCase() || nonASCIISingleCaseWordChar.test(H)))
        return !0;
    }
    return !1;
  }
  function makeCategorizer(O) {
    return (_) => {
      if (!/\S/.test(_))
        return CharCategory.Space;
      if (hasWordChar(_))
        return CharCategory.Word;
      for (let H = 0; H < O.length; H++)
        if (_.indexOf(O[H]) > -1)
          return CharCategory.Word;
      return CharCategory.Other;
    };
  }
  class EditorState {
    constructor(_, H, V, q, G, J) {
      this.config = _, this.doc = H, this.selection = V, this.values = q, this.status = _.statusTemplate.slice(), this.computeSlot = G, J && (J._state = this);
      for (let Z = 0; Z < this.config.dynamicSlots.length; Z++)
        ensureAddr(this, Z << 1);
      this.computeSlot = null;
    }
    field(_, H = !0) {
      let V = this.config.address[_.id];
      if (V == null) {
        if (H)
          throw new RangeError("Field is not present in this state");
        return;
      }
      return ensureAddr(this, V), getAddr(this, V);
    }
    /**
    Create a [transaction](https://codemirror.net/6/docs/ref/#state.Transaction) that updates this
    state. Any number of [transaction specs](https://codemirror.net/6/docs/ref/#state.TransactionSpec)
    can be passed. Unless
    [`sequential`](https://codemirror.net/6/docs/ref/#state.TransactionSpec.sequential) is set, the
    [changes](https://codemirror.net/6/docs/ref/#state.TransactionSpec.changes) (if any) of each spec
    are assumed to start in the _current_ document (not the document
    produced by previous specs), and its
    [selection](https://codemirror.net/6/docs/ref/#state.TransactionSpec.selection) and
    [effects](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) are assumed to refer
    to the document created by its _own_ changes. The resulting
    transaction contains the combined effect of all the different
    specs. For [selection](https://codemirror.net/6/docs/ref/#state.TransactionSpec.selection), later
    specs take precedence over earlier ones.
    */
    update(..._) {
      return resolveTransaction(this, _, !0);
    }
    /**
    @internal
    */
    applyTransaction(_) {
      let H = this.config, { base: V, compartments: q } = H;
      for (let Z of _.effects)
        Z.is(Compartment.reconfigure) ? (H && (q = /* @__PURE__ */ new Map(), H.compartments.forEach((te, ne) => q.set(ne, te)), H = null), q.set(Z.value.compartment, Z.value.extension)) : Z.is(StateEffect.reconfigure) ? (H = null, V = Z.value) : Z.is(StateEffect.appendConfig) && (H = null, V = asArray$1(V).concat(Z.value));
      let G;
      H ? G = _.startState.values.slice() : (H = Configuration.resolve(V, q, this), G = new EditorState(H, this.doc, this.selection, H.dynamicSlots.map(() => null), (te, ne) => ne.reconfigure(te, this), null).values);
      let J = _.startState.facet(allowMultipleSelections) ? _.newSelection : _.newSelection.asSingle();
      new EditorState(H, _.newDoc, J, G, (Z, te) => te.update(Z, _), _);
    }
    /**
    Create a [transaction spec](https://codemirror.net/6/docs/ref/#state.TransactionSpec) that
    replaces every selection range with the given content.
    */
    replaceSelection(_) {
      return typeof _ == "string" && (_ = this.toText(_)), this.changeByRange((H) => ({
        changes: { from: H.from, to: H.to, insert: _ },
        range: EditorSelection.cursor(H.from + _.length)
      }));
    }
    /**
    Create a set of changes and a new selection by running the given
    function for each range in the active selection. The function
    can return an optional set of changes (in the coordinate space
    of the start document), plus an updated range (in the coordinate
    space of the document produced by the call's own changes). This
    method will merge all the changes and ranges into a single
    changeset and selection, and return it as a [transaction
    spec](https://codemirror.net/6/docs/ref/#state.TransactionSpec), which can be passed to
    [`update`](https://codemirror.net/6/docs/ref/#state.EditorState.update).
    */
    changeByRange(_) {
      let H = this.selection, V = _(H.ranges[0]), q = this.changes(V.changes), G = [V.range], J = asArray$1(V.effects);
      for (let Z = 1; Z < H.ranges.length; Z++) {
        let te = _(H.ranges[Z]), ne = this.changes(te.changes), ee = ne.map(q);
        for (let oe = 0; oe < Z; oe++)
          G[oe] = G[oe].map(ee);
        let ae = q.mapDesc(ne, !0);
        G.push(te.range.map(ae)), q = q.compose(ee), J = StateEffect.mapEffects(J, ee).concat(StateEffect.mapEffects(asArray$1(te.effects), ae));
      }
      return {
        changes: q,
        selection: EditorSelection.create(G, H.mainIndex),
        effects: J
      };
    }
    /**
    Create a [change set](https://codemirror.net/6/docs/ref/#state.ChangeSet) from the given change
    description, taking the state's document length and line
    separator into account.
    */
    changes(_ = []) {
      return _ instanceof ChangeSet ? _ : ChangeSet.of(_, this.doc.length, this.facet(EditorState.lineSeparator));
    }
    /**
    Using the state's [line
    separator](https://codemirror.net/6/docs/ref/#state.EditorState^lineSeparator), create a
    [`Text`](https://codemirror.net/6/docs/ref/#state.Text) instance from the given string.
    */
    toText(_) {
      return Text.of(_.split(this.facet(EditorState.lineSeparator) || DefaultSplit));
    }
    /**
    Return the given range of the document as a string.
    */
    sliceDoc(_ = 0, H = this.doc.length) {
      return this.doc.sliceString(_, H, this.lineBreak);
    }
    /**
    Get the value of a state [facet](https://codemirror.net/6/docs/ref/#state.Facet).
    */
    facet(_) {
      let H = this.config.address[_.id];
      return H == null ? _.default : (ensureAddr(this, H), getAddr(this, H));
    }
    /**
    Convert this state to a JSON-serializable object. When custom
    fields should be serialized, you can pass them in as an object
    mapping property names (in the resulting object, which should
    not use `doc` or `selection`) to fields.
    */
    toJSON(_) {
      let H = {
        doc: this.sliceDoc(),
        selection: this.selection.toJSON()
      };
      if (_)
        for (let V in _) {
          let q = _[V];
          q instanceof StateField && this.config.address[q.id] != null && (H[V] = q.spec.toJSON(this.field(_[V]), this));
        }
      return H;
    }
    /**
    Deserialize a state from its JSON representation. When custom
    fields should be deserialized, pass the same object you passed
    to [`toJSON`](https://codemirror.net/6/docs/ref/#state.EditorState.toJSON) when serializing as
    third argument.
    */
    static fromJSON(_, H = {}, V) {
      if (!_ || typeof _.doc != "string")
        throw new RangeError("Invalid JSON representation for EditorState");
      let q = [];
      if (V) {
        for (let G in V)
          if (Object.prototype.hasOwnProperty.call(_, G)) {
            let J = V[G], Z = _[G];
            q.push(J.init((te) => J.spec.fromJSON(Z, te)));
          }
      }
      return EditorState.create({
        doc: _.doc,
        selection: EditorSelection.fromJSON(_.selection),
        extensions: H.extensions ? q.concat([H.extensions]) : q
      });
    }
    /**
    Create a new state. You'll usually only need this when
    initializing an editor—updated states are created by applying
    transactions.
    */
    static create(_ = {}) {
      let H = Configuration.resolve(_.extensions || [], /* @__PURE__ */ new Map()), V = _.doc instanceof Text ? _.doc : Text.of((_.doc || "").split(H.staticFacet(EditorState.lineSeparator) || DefaultSplit)), q = _.selection ? _.selection instanceof EditorSelection ? _.selection : EditorSelection.single(_.selection.anchor, _.selection.head) : EditorSelection.single(0);
      return checkSelection(q, V.length), H.staticFacet(allowMultipleSelections) || (q = q.asSingle()), new EditorState(H, V, q, H.dynamicSlots.map(() => null), (G, J) => J.create(G), null);
    }
    /**
    The size (in columns) of a tab in the document, determined by
    the [`tabSize`](https://codemirror.net/6/docs/ref/#state.EditorState^tabSize) facet.
    */
    get tabSize() {
      return this.facet(EditorState.tabSize);
    }
    /**
    Get the proper [line-break](https://codemirror.net/6/docs/ref/#state.EditorState^lineSeparator)
    string for this state.
    */
    get lineBreak() {
      return this.facet(EditorState.lineSeparator) || `
`;
    }
    /**
    Returns true when the editor is
    [configured](https://codemirror.net/6/docs/ref/#state.EditorState^readOnly) to be read-only.
    */
    get readOnly() {
      return this.facet(readOnly);
    }
    /**
    Look up a translation for the given phrase (via the
    [`phrases`](https://codemirror.net/6/docs/ref/#state.EditorState^phrases) facet), or return the
    original string if no translation is found.
    
    If additional arguments are passed, they will be inserted in
    place of markers like `$1` (for the first value) and `$2`, etc.
    A single `$` is equivalent to `$1`, and `$$` will produce a
    literal dollar sign.
    */
    phrase(_, ...H) {
      for (let V of this.facet(EditorState.phrases))
        if (Object.prototype.hasOwnProperty.call(V, _)) {
          _ = V[_];
          break;
        }
      return H.length && (_ = _.replace(/\$(\$|\d*)/g, (V, q) => {
        if (q == "$")
          return "$";
        let G = +(q || 1);
        return !G || G > H.length ? V : H[G - 1];
      })), _;
    }
    /**
    Find the values for a given language data field, provided by the
    the [`languageData`](https://codemirror.net/6/docs/ref/#state.EditorState^languageData) facet.
    
    Examples of language data fields are...
    
    - [`"commentTokens"`](https://codemirror.net/6/docs/ref/#commands.CommentTokens) for specifying
      comment syntax.
    - [`"autocomplete"`](https://codemirror.net/6/docs/ref/#autocomplete.autocompletion^config.override)
      for providing language-specific completion sources.
    - [`"wordChars"`](https://codemirror.net/6/docs/ref/#state.EditorState.charCategorizer) for adding
      characters that should be considered part of words in this
      language.
    - [`"closeBrackets"`](https://codemirror.net/6/docs/ref/#autocomplete.CloseBracketConfig) controls
      bracket closing behavior.
    */
    languageDataAt(_, H, V = -1) {
      let q = [];
      for (let G of this.facet(languageData))
        for (let J of G(this, H, V))
          Object.prototype.hasOwnProperty.call(J, _) && q.push(J[_]);
      return q;
    }
    /**
    Return a function that can categorize strings (expected to
    represent a single [grapheme cluster](https://codemirror.net/6/docs/ref/#state.findClusterBreak))
    into one of:
    
     - Word (contains an alphanumeric character or a character
       explicitly listed in the local language's `"wordChars"`
       language data, which should be a string)
     - Space (contains only whitespace)
     - Other (anything else)
    */
    charCategorizer(_) {
      return makeCategorizer(this.languageDataAt("wordChars", _).join(""));
    }
    /**
    Find the word at the given position, meaning the range
    containing all [word](https://codemirror.net/6/docs/ref/#state.CharCategory.Word) characters
    around it. If no word characters are adjacent to the position,
    this returns null.
    */
    wordAt(_) {
      let { text: H, from: V, length: q } = this.doc.lineAt(_), G = this.charCategorizer(_), J = _ - V, Z = _ - V;
      for (; J > 0; ) {
        let te = findClusterBreak(H, J, !1);
        if (G(H.slice(te, J)) != CharCategory.Word)
          break;
        J = te;
      }
      for (; Z < q; ) {
        let te = findClusterBreak(H, Z);
        if (G(H.slice(Z, te)) != CharCategory.Word)
          break;
        Z = te;
      }
      return J == Z ? null : EditorSelection.range(J + V, Z + V);
    }
  }
  EditorState.allowMultipleSelections = allowMultipleSelections;
  EditorState.tabSize = /* @__PURE__ */ Facet.define({
    combine: (O) => O.length ? O[0] : 4
  });
  EditorState.lineSeparator = lineSeparator;
  EditorState.readOnly = readOnly;
  EditorState.phrases = /* @__PURE__ */ Facet.define({
    compare(O, _) {
      let H = Object.keys(O), V = Object.keys(_);
      return H.length == V.length && H.every((q) => O[q] == _[q]);
    }
  });
  EditorState.languageData = languageData;
  EditorState.changeFilter = changeFilter;
  EditorState.transactionFilter = transactionFilter;
  EditorState.transactionExtender = transactionExtender;
  Compartment.reconfigure = /* @__PURE__ */ StateEffect.define();
  function combineConfig(O, _, H = {}) {
    let V = {};
    for (let q of O)
      for (let G of Object.keys(q)) {
        let J = q[G], Z = V[G];
        if (Z === void 0)
          V[G] = J;
        else if (!(Z === J || J === void 0)) if (Object.hasOwnProperty.call(H, G))
          V[G] = H[G](Z, J);
        else
          throw new Error("Config merge conflict for field " + G);
      }
    for (let q in _)
      V[q] === void 0 && (V[q] = _[q]);
    return V;
  }
  class RangeValue {
    /**
    Compare this value with another value. Used when comparing
    rangesets. The default implementation compares by identity.
    Unless you are only creating a fixed number of unique instances
    of your value type, it is a good idea to implement this
    properly.
    */
    eq(_) {
      return this == _;
    }
    /**
    Create a [range](https://codemirror.net/6/docs/ref/#state.Range) with this value.
    */
    range(_, H = _) {
      return Range.create(_, H, this);
    }
  }
  RangeValue.prototype.startSide = RangeValue.prototype.endSide = 0;
  RangeValue.prototype.point = !1;
  RangeValue.prototype.mapMode = MapMode.TrackDel;
  class Range {
    constructor(_, H, V) {
      this.from = _, this.to = H, this.value = V;
    }
    /**
    @internal
    */
    static create(_, H, V) {
      return new Range(_, H, V);
    }
  }
  function cmpRange(O, _) {
    return O.from - _.from || O.value.startSide - _.value.startSide;
  }
  class Chunk {
    constructor(_, H, V, q) {
      this.from = _, this.to = H, this.value = V, this.maxPoint = q;
    }
    get length() {
      return this.to[this.to.length - 1];
    }
    // Find the index of the given position and side. Use the ranges'
    // `from` pos when `end == false`, `to` when `end == true`.
    findIndex(_, H, V, q = 0) {
      let G = V ? this.to : this.from;
      for (let J = q, Z = G.length; ; ) {
        if (J == Z)
          return J;
        let te = J + Z >> 1, ne = G[te] - _ || (V ? this.value[te].endSide : this.value[te].startSide) - H;
        if (te == J)
          return ne >= 0 ? J : Z;
        ne >= 0 ? Z = te : J = te + 1;
      }
    }
    between(_, H, V, q) {
      for (let G = this.findIndex(H, -1e9, !0), J = this.findIndex(V, 1e9, !1, G); G < J; G++)
        if (q(this.from[G] + _, this.to[G] + _, this.value[G]) === !1)
          return !1;
    }
    map(_, H) {
      let V = [], q = [], G = [], J = -1, Z = -1;
      for (let te = 0; te < this.value.length; te++) {
        let ne = this.value[te], ee = this.from[te] + _, ae = this.to[te] + _, oe, se;
        if (ee == ae) {
          let ce = H.mapPos(ee, ne.startSide, ne.mapMode);
          if (ce == null || (oe = se = ce, ne.startSide != ne.endSide && (se = H.mapPos(ee, ne.endSide), se < oe)))
            continue;
        } else if (oe = H.mapPos(ee, ne.startSide), se = H.mapPos(ae, ne.endSide), oe > se || oe == se && ne.startSide > 0 && ne.endSide <= 0)
          continue;
        (se - oe || ne.endSide - ne.startSide) < 0 || (J < 0 && (J = oe), ne.point && (Z = Math.max(Z, se - oe)), V.push(ne), q.push(oe - J), G.push(se - J));
      }
      return { mapped: V.length ? new Chunk(q, G, V, Z) : null, pos: J };
    }
  }
  class RangeSet {
    constructor(_, H, V, q) {
      this.chunkPos = _, this.chunk = H, this.nextLayer = V, this.maxPoint = q;
    }
    /**
    @internal
    */
    static create(_, H, V, q) {
      return new RangeSet(_, H, V, q);
    }
    /**
    @internal
    */
    get length() {
      let _ = this.chunk.length - 1;
      return _ < 0 ? 0 : Math.max(this.chunkEnd(_), this.nextLayer.length);
    }
    /**
    The number of ranges in the set.
    */
    get size() {
      if (this.isEmpty)
        return 0;
      let _ = this.nextLayer.size;
      for (let H of this.chunk)
        _ += H.value.length;
      return _;
    }
    /**
    @internal
    */
    chunkEnd(_) {
      return this.chunkPos[_] + this.chunk[_].length;
    }
    /**
    Update the range set, optionally adding new ranges or filtering
    out existing ones.
    
    (Note: The type parameter is just there as a kludge to work
    around TypeScript variance issues that prevented `RangeSet<X>`
    from being a subtype of `RangeSet<Y>` when `X` is a subtype of
    `Y`.)
    */
    update(_) {
      let { add: H = [], sort: V = !1, filterFrom: q = 0, filterTo: G = this.length } = _, J = _.filter;
      if (H.length == 0 && !J)
        return this;
      if (V && (H = H.slice().sort(cmpRange)), this.isEmpty)
        return H.length ? RangeSet.of(H) : this;
      let Z = new LayerCursor(this, null, -1).goto(0), te = 0, ne = [], ee = new RangeSetBuilder();
      for (; Z.value || te < H.length; )
        if (te < H.length && (Z.from - H[te].from || Z.startSide - H[te].value.startSide) >= 0) {
          let ae = H[te++];
          ee.addInner(ae.from, ae.to, ae.value) || ne.push(ae);
        } else Z.rangeIndex == 1 && Z.chunkIndex < this.chunk.length && (te == H.length || this.chunkEnd(Z.chunkIndex) < H[te].from) && (!J || q > this.chunkEnd(Z.chunkIndex) || G < this.chunkPos[Z.chunkIndex]) && ee.addChunk(this.chunkPos[Z.chunkIndex], this.chunk[Z.chunkIndex]) ? Z.nextChunk() : ((!J || q > Z.to || G < Z.from || J(Z.from, Z.to, Z.value)) && (ee.addInner(Z.from, Z.to, Z.value) || ne.push(Range.create(Z.from, Z.to, Z.value))), Z.next());
      return ee.finishInner(this.nextLayer.isEmpty && !ne.length ? RangeSet.empty : this.nextLayer.update({ add: ne, filter: J, filterFrom: q, filterTo: G }));
    }
    /**
    Map this range set through a set of changes, return the new set.
    */
    map(_) {
      if (_.empty || this.isEmpty)
        return this;
      let H = [], V = [], q = -1;
      for (let J = 0; J < this.chunk.length; J++) {
        let Z = this.chunkPos[J], te = this.chunk[J], ne = _.touchesRange(Z, Z + te.length);
        if (ne === !1)
          q = Math.max(q, te.maxPoint), H.push(te), V.push(_.mapPos(Z));
        else if (ne === !0) {
          let { mapped: ee, pos: ae } = te.map(Z, _);
          ee && (q = Math.max(q, ee.maxPoint), H.push(ee), V.push(ae));
        }
      }
      let G = this.nextLayer.map(_);
      return H.length == 0 ? G : new RangeSet(V, H, G || RangeSet.empty, q);
    }
    /**
    Iterate over the ranges that touch the region `from` to `to`,
    calling `f` for each. There is no guarantee that the ranges will
    be reported in any specific order. When the callback returns
    `false`, iteration stops.
    */
    between(_, H, V) {
      if (!this.isEmpty) {
        for (let q = 0; q < this.chunk.length; q++) {
          let G = this.chunkPos[q], J = this.chunk[q];
          if (H >= G && _ <= G + J.length && J.between(G, _ - G, H - G, V) === !1)
            return;
        }
        this.nextLayer.between(_, H, V);
      }
    }
    /**
    Iterate over the ranges in this set, in order, including all
    ranges that end at or after `from`.
    */
    iter(_ = 0) {
      return HeapCursor.from([this]).goto(_);
    }
    /**
    @internal
    */
    get isEmpty() {
      return this.nextLayer == this;
    }
    /**
    Iterate over the ranges in a collection of sets, in order,
    starting from `from`.
    */
    static iter(_, H = 0) {
      return HeapCursor.from(_).goto(H);
    }
    /**
    Iterate over two groups of sets, calling methods on `comparator`
    to notify it of possible differences.
    */
    static compare(_, H, V, q, G = -1) {
      let J = _.filter((ae) => ae.maxPoint > 0 || !ae.isEmpty && ae.maxPoint >= G), Z = H.filter((ae) => ae.maxPoint > 0 || !ae.isEmpty && ae.maxPoint >= G), te = findSharedChunks(J, Z, V), ne = new SpanCursor(J, te, G), ee = new SpanCursor(Z, te, G);
      V.iterGaps((ae, oe, se) => compare(ne, ae, ee, oe, se, q)), V.empty && V.length == 0 && compare(ne, 0, ee, 0, 0, q);
    }
    /**
    Compare the contents of two groups of range sets, returning true
    if they are equivalent in the given range.
    */
    static eq(_, H, V = 0, q) {
      q == null && (q = 999999999);
      let G = _.filter((ee) => !ee.isEmpty && H.indexOf(ee) < 0), J = H.filter((ee) => !ee.isEmpty && _.indexOf(ee) < 0);
      if (G.length != J.length)
        return !1;
      if (!G.length)
        return !0;
      let Z = findSharedChunks(G, J), te = new SpanCursor(G, Z, 0).goto(V), ne = new SpanCursor(J, Z, 0).goto(V);
      for (; ; ) {
        if (te.to != ne.to || !sameValues(te.active, ne.active) || te.point && (!ne.point || !te.point.eq(ne.point)))
          return !1;
        if (te.to > q)
          return !0;
        te.next(), ne.next();
      }
    }
    /**
    Iterate over a group of range sets at the same time, notifying
    the iterator about the ranges covering every given piece of
    content. Returns the open count (see
    [`SpanIterator.span`](https://codemirror.net/6/docs/ref/#state.SpanIterator.span)) at the end
    of the iteration.
    */
    static spans(_, H, V, q, G = -1) {
      let J = new SpanCursor(_, null, G).goto(H), Z = H, te = J.openStart;
      for (; ; ) {
        let ne = Math.min(J.to, V);
        if (J.point) {
          let ee = J.activeForPoint(J.to), ae = J.pointFrom < H ? ee.length + 1 : J.point.startSide < 0 ? ee.length : Math.min(ee.length, te);
          q.point(Z, ne, J.point, ee, ae, J.pointRank), te = Math.min(J.openEnd(ne), ee.length);
        } else ne > Z && (q.span(Z, ne, J.active, te), te = J.openEnd(ne));
        if (J.to > V)
          return te + (J.point && J.to > V ? 1 : 0);
        Z = J.to, J.next();
      }
    }
    /**
    Create a range set for the given range or array of ranges. By
    default, this expects the ranges to be _sorted_ (by start
    position and, if two start at the same position,
    `value.startSide`). You can pass `true` as second argument to
    cause the method to sort them.
    */
    static of(_, H = !1) {
      let V = new RangeSetBuilder();
      for (let q of _ instanceof Range ? [_] : H ? lazySort(_) : _)
        V.add(q.from, q.to, q.value);
      return V.finish();
    }
    /**
    Join an array of range sets into a single set.
    */
    static join(_) {
      if (!_.length)
        return RangeSet.empty;
      let H = _[_.length - 1];
      for (let V = _.length - 2; V >= 0; V--)
        for (let q = _[V]; q != RangeSet.empty; q = q.nextLayer)
          H = new RangeSet(q.chunkPos, q.chunk, H, Math.max(q.maxPoint, H.maxPoint));
      return H;
    }
  }
  RangeSet.empty = /* @__PURE__ */ new RangeSet([], [], null, -1);
  function lazySort(O) {
    if (O.length > 1)
      for (let _ = O[0], H = 1; H < O.length; H++) {
        let V = O[H];
        if (cmpRange(_, V) > 0)
          return O.slice().sort(cmpRange);
        _ = V;
      }
    return O;
  }
  RangeSet.empty.nextLayer = RangeSet.empty;
  class RangeSetBuilder {
    finishChunk(_) {
      this.chunks.push(new Chunk(this.from, this.to, this.value, this.maxPoint)), this.chunkPos.push(this.chunkStart), this.chunkStart = -1, this.setMaxPoint = Math.max(this.setMaxPoint, this.maxPoint), this.maxPoint = -1, _ && (this.from = [], this.to = [], this.value = []);
    }
    /**
    Create an empty builder.
    */
    constructor() {
      this.chunks = [], this.chunkPos = [], this.chunkStart = -1, this.last = null, this.lastFrom = -1e9, this.lastTo = -1e9, this.from = [], this.to = [], this.value = [], this.maxPoint = -1, this.setMaxPoint = -1, this.nextLayer = null;
    }
    /**
    Add a range. Ranges should be added in sorted (by `from` and
    `value.startSide`) order.
    */
    add(_, H, V) {
      this.addInner(_, H, V) || (this.nextLayer || (this.nextLayer = new RangeSetBuilder())).add(_, H, V);
    }
    /**
    @internal
    */
    addInner(_, H, V) {
      let q = _ - this.lastTo || V.startSide - this.last.endSide;
      if (q <= 0 && (_ - this.lastFrom || V.startSide - this.last.startSide) < 0)
        throw new Error("Ranges must be added sorted by `from` position and `startSide`");
      return q < 0 ? !1 : (this.from.length == 250 && this.finishChunk(!0), this.chunkStart < 0 && (this.chunkStart = _), this.from.push(_ - this.chunkStart), this.to.push(H - this.chunkStart), this.last = V, this.lastFrom = _, this.lastTo = H, this.value.push(V), V.point && (this.maxPoint = Math.max(this.maxPoint, H - _)), !0);
    }
    /**
    @internal
    */
    addChunk(_, H) {
      if ((_ - this.lastTo || H.value[0].startSide - this.last.endSide) < 0)
        return !1;
      this.from.length && this.finishChunk(!0), this.setMaxPoint = Math.max(this.setMaxPoint, H.maxPoint), this.chunks.push(H), this.chunkPos.push(_);
      let V = H.value.length - 1;
      return this.last = H.value[V], this.lastFrom = H.from[V] + _, this.lastTo = H.to[V] + _, !0;
    }
    /**
    Finish the range set. Returns the new set. The builder can't be
    used anymore after this has been called.
    */
    finish() {
      return this.finishInner(RangeSet.empty);
    }
    /**
    @internal
    */
    finishInner(_) {
      if (this.from.length && this.finishChunk(!1), this.chunks.length == 0)
        return _;
      let H = RangeSet.create(this.chunkPos, this.chunks, this.nextLayer ? this.nextLayer.finishInner(_) : _, this.setMaxPoint);
      return this.from = null, H;
    }
  }
  function findSharedChunks(O, _, H) {
    let V = /* @__PURE__ */ new Map();
    for (let G of O)
      for (let J = 0; J < G.chunk.length; J++)
        G.chunk[J].maxPoint <= 0 && V.set(G.chunk[J], G.chunkPos[J]);
    let q = /* @__PURE__ */ new Set();
    for (let G of _)
      for (let J = 0; J < G.chunk.length; J++) {
        let Z = V.get(G.chunk[J]);
        Z != null && (H ? H.mapPos(Z) : Z) == G.chunkPos[J] && !(H != null && H.touchesRange(Z, Z + G.chunk[J].length)) && q.add(G.chunk[J]);
      }
    return q;
  }
  class LayerCursor {
    constructor(_, H, V, q = 0) {
      this.layer = _, this.skip = H, this.minPoint = V, this.rank = q;
    }
    get startSide() {
      return this.value ? this.value.startSide : 0;
    }
    get endSide() {
      return this.value ? this.value.endSide : 0;
    }
    goto(_, H = -1e9) {
      return this.chunkIndex = this.rangeIndex = 0, this.gotoInner(_, H, !1), this;
    }
    gotoInner(_, H, V) {
      for (; this.chunkIndex < this.layer.chunk.length; ) {
        let q = this.layer.chunk[this.chunkIndex];
        if (!(this.skip && this.skip.has(q) || this.layer.chunkEnd(this.chunkIndex) < _ || q.maxPoint < this.minPoint))
          break;
        this.chunkIndex++, V = !1;
      }
      if (this.chunkIndex < this.layer.chunk.length) {
        let q = this.layer.chunk[this.chunkIndex].findIndex(_ - this.layer.chunkPos[this.chunkIndex], H, !0);
        (!V || this.rangeIndex < q) && this.setRangeIndex(q);
      }
      this.next();
    }
    forward(_, H) {
      (this.to - _ || this.endSide - H) < 0 && this.gotoInner(_, H, !0);
    }
    next() {
      for (; ; )
        if (this.chunkIndex == this.layer.chunk.length) {
          this.from = this.to = 1e9, this.value = null;
          break;
        } else {
          let _ = this.layer.chunkPos[this.chunkIndex], H = this.layer.chunk[this.chunkIndex], V = _ + H.from[this.rangeIndex];
          if (this.from = V, this.to = _ + H.to[this.rangeIndex], this.value = H.value[this.rangeIndex], this.setRangeIndex(this.rangeIndex + 1), this.minPoint < 0 || this.value.point && this.to - this.from >= this.minPoint)
            break;
        }
    }
    setRangeIndex(_) {
      if (_ == this.layer.chunk[this.chunkIndex].value.length) {
        if (this.chunkIndex++, this.skip)
          for (; this.chunkIndex < this.layer.chunk.length && this.skip.has(this.layer.chunk[this.chunkIndex]); )
            this.chunkIndex++;
        this.rangeIndex = 0;
      } else
        this.rangeIndex = _;
    }
    nextChunk() {
      this.chunkIndex++, this.rangeIndex = 0, this.next();
    }
    compare(_) {
      return this.from - _.from || this.startSide - _.startSide || this.rank - _.rank || this.to - _.to || this.endSide - _.endSide;
    }
  }
  class HeapCursor {
    constructor(_) {
      this.heap = _;
    }
    static from(_, H = null, V = -1) {
      let q = [];
      for (let G = 0; G < _.length; G++)
        for (let J = _[G]; !J.isEmpty; J = J.nextLayer)
          J.maxPoint >= V && q.push(new LayerCursor(J, H, V, G));
      return q.length == 1 ? q[0] : new HeapCursor(q);
    }
    get startSide() {
      return this.value ? this.value.startSide : 0;
    }
    goto(_, H = -1e9) {
      for (let V of this.heap)
        V.goto(_, H);
      for (let V = this.heap.length >> 1; V >= 0; V--)
        heapBubble(this.heap, V);
      return this.next(), this;
    }
    forward(_, H) {
      for (let V of this.heap)
        V.forward(_, H);
      for (let V = this.heap.length >> 1; V >= 0; V--)
        heapBubble(this.heap, V);
      (this.to - _ || this.value.endSide - H) < 0 && this.next();
    }
    next() {
      if (this.heap.length == 0)
        this.from = this.to = 1e9, this.value = null, this.rank = -1;
      else {
        let _ = this.heap[0];
        this.from = _.from, this.to = _.to, this.value = _.value, this.rank = _.rank, _.value && _.next(), heapBubble(this.heap, 0);
      }
    }
  }
  function heapBubble(O, _) {
    for (let H = O[_]; ; ) {
      let V = (_ << 1) + 1;
      if (V >= O.length)
        break;
      let q = O[V];
      if (V + 1 < O.length && q.compare(O[V + 1]) >= 0 && (q = O[V + 1], V++), H.compare(q) < 0)
        break;
      O[V] = H, O[_] = q, _ = V;
    }
  }
  class SpanCursor {
    constructor(_, H, V) {
      this.minPoint = V, this.active = [], this.activeTo = [], this.activeRank = [], this.minActive = -1, this.point = null, this.pointFrom = 0, this.pointRank = 0, this.to = -1e9, this.endSide = 0, this.openStart = -1, this.cursor = HeapCursor.from(_, H, V);
    }
    goto(_, H = -1e9) {
      return this.cursor.goto(_, H), this.active.length = this.activeTo.length = this.activeRank.length = 0, this.minActive = -1, this.to = _, this.endSide = H, this.openStart = -1, this.next(), this;
    }
    forward(_, H) {
      for (; this.minActive > -1 && (this.activeTo[this.minActive] - _ || this.active[this.minActive].endSide - H) < 0; )
        this.removeActive(this.minActive);
      this.cursor.forward(_, H);
    }
    removeActive(_) {
      remove(this.active, _), remove(this.activeTo, _), remove(this.activeRank, _), this.minActive = findMinIndex(this.active, this.activeTo);
    }
    addActive(_) {
      let H = 0, { value: V, to: q, rank: G } = this.cursor;
      for (; H < this.activeRank.length && (G - this.activeRank[H] || q - this.activeTo[H]) > 0; )
        H++;
      insert(this.active, H, V), insert(this.activeTo, H, q), insert(this.activeRank, H, G), _ && insert(_, H, this.cursor.from), this.minActive = findMinIndex(this.active, this.activeTo);
    }
    // After calling this, if `this.point` != null, the next range is a
    // point. Otherwise, it's a regular range, covered by `this.active`.
    next() {
      let _ = this.to, H = this.point;
      this.point = null;
      let V = this.openStart < 0 ? [] : null;
      for (; ; ) {
        let q = this.minActive;
        if (q > -1 && (this.activeTo[q] - this.cursor.from || this.active[q].endSide - this.cursor.startSide) < 0) {
          if (this.activeTo[q] > _) {
            this.to = this.activeTo[q], this.endSide = this.active[q].endSide;
            break;
          }
          this.removeActive(q), V && remove(V, q);
        } else if (this.cursor.value)
          if (this.cursor.from > _) {
            this.to = this.cursor.from, this.endSide = this.cursor.startSide;
            break;
          } else {
            let G = this.cursor.value;
            if (!G.point)
              this.addActive(V), this.cursor.next();
            else if (H && this.cursor.to == this.to && this.cursor.from < this.cursor.to)
              this.cursor.next();
            else {
              this.point = G, this.pointFrom = this.cursor.from, this.pointRank = this.cursor.rank, this.to = this.cursor.to, this.endSide = G.endSide, this.cursor.next(), this.forward(this.to, this.endSide);
              break;
            }
          }
        else {
          this.to = this.endSide = 1e9;
          break;
        }
      }
      if (V) {
        this.openStart = 0;
        for (let q = V.length - 1; q >= 0 && V[q] < _; q--)
          this.openStart++;
      }
    }
    activeForPoint(_) {
      if (!this.active.length)
        return this.active;
      let H = [];
      for (let V = this.active.length - 1; V >= 0 && !(this.activeRank[V] < this.pointRank); V--)
        (this.activeTo[V] > _ || this.activeTo[V] == _ && this.active[V].endSide >= this.point.endSide) && H.push(this.active[V]);
      return H.reverse();
    }
    openEnd(_) {
      let H = 0;
      for (let V = this.activeTo.length - 1; V >= 0 && this.activeTo[V] > _; V--)
        H++;
      return H;
    }
  }
  function compare(O, _, H, V, q, G) {
    O.goto(_), H.goto(V);
    let J = V + q, Z = V, te = V - _;
    for (; ; ) {
      let ne = O.to + te - H.to || O.endSide - H.endSide, ee = ne < 0 ? O.to + te : H.to, ae = Math.min(ee, J);
      if (O.point || H.point ? O.point && H.point && (O.point == H.point || O.point.eq(H.point)) && sameValues(O.activeForPoint(O.to), H.activeForPoint(H.to)) || G.comparePoint(Z, ae, O.point, H.point) : ae > Z && !sameValues(O.active, H.active) && G.compareRange(Z, ae, O.active, H.active), ee > J)
        break;
      Z = ee, ne <= 0 && O.next(), ne >= 0 && H.next();
    }
  }
  function sameValues(O, _) {
    if (O.length != _.length)
      return !1;
    for (let H = 0; H < O.length; H++)
      if (O[H] != _[H] && !O[H].eq(_[H]))
        return !1;
    return !0;
  }
  function remove(O, _) {
    for (let H = _, V = O.length - 1; H < V; H++)
      O[H] = O[H + 1];
    O.pop();
  }
  function insert(O, _, H) {
    for (let V = O.length - 1; V >= _; V--)
      O[V + 1] = O[V];
    O[_] = H;
  }
  function findMinIndex(O, _) {
    let H = -1, V = 1e9;
    for (let q = 0; q < _.length; q++)
      (_[q] - V || O[q].endSide - O[H].endSide) < 0 && (H = q, V = _[q]);
    return H;
  }
  function countColumn(O, _, H = O.length) {
    let V = 0;
    for (let q = 0; q < H; )
      O.charCodeAt(q) == 9 ? (V += _ - V % _, q++) : (V++, q = findClusterBreak(O, q));
    return V;
  }
  function findColumn(O, _, H, V) {
    for (let q = 0, G = 0; ; ) {
      if (G >= _)
        return q;
      if (q == O.length)
        break;
      G += O.charCodeAt(q) == 9 ? H - G % H : 1, q = findClusterBreak(O, q);
    }
    return V === !0 ? -1 : O.length;
  }
  const C = "ͼ", COUNT = typeof Symbol > "u" ? "__" + C : Symbol.for(C), SET = typeof Symbol > "u" ? "__styleSet" + Math.floor(Math.random() * 1e8) : Symbol("styleSet"), top = typeof globalThis < "u" ? globalThis : typeof window < "u" ? window : {};
  class StyleModule {
    // :: (Object<Style>, ?{finish: ?(string) → string})
    // Create a style module from the given spec.
    //
    // When `finish` is given, it is called on regular (non-`@`)
    // selectors (after `&` expansion) to compute the final selector.
    constructor(_, H) {
      this.rules = [];
      let { finish: V } = H || {};
      function q(J) {
        return /^@/.test(J) ? [J] : J.split(/,\s*/);
      }
      function G(J, Z, te, ne) {
        let ee = [], ae = /^@(\w+)\b/.exec(J[0]), oe = ae && ae[1] == "keyframes";
        if (ae && Z == null) return te.push(J[0] + ";");
        for (let se in Z) {
          let ce = Z[se];
          if (/&/.test(se))
            G(
              se.split(/,\s*/).map((le) => J.map((ue) => le.replace(/&/, ue))).reduce((le, ue) => le.concat(ue)),
              ce,
              te
            );
          else if (ce && typeof ce == "object") {
            if (!ae) throw new RangeError("The value of a property (" + se + ") should be a primitive value.");
            G(q(se), ce, ee, oe);
          } else ce != null && ee.push(se.replace(/_.*/, "").replace(/[A-Z]/g, (le) => "-" + le.toLowerCase()) + ": " + ce + ";");
        }
        (ee.length || oe) && te.push((V && !ae && !ne ? J.map(V) : J).join(", ") + " {" + ee.join(" ") + "}");
      }
      for (let J in _) G(q(J), _[J], this.rules);
    }
    // :: () → string
    // Returns a string containing the module's CSS rules.
    getRules() {
      return this.rules.join(`
`);
    }
    // :: () → string
    // Generate a new unique CSS class name.
    static newName() {
      let _ = top[COUNT] || 1;
      return top[COUNT] = _ + 1, C + _.toString(36);
    }
    // :: (union<Document, ShadowRoot>, union<[StyleModule], StyleModule>, ?{nonce: ?string})
    //
    // Mount the given set of modules in the given DOM root, which ensures
    // that the CSS rules defined by the module are available in that
    // context.
    //
    // Rules are only added to the document once per root.
    //
    // Rule order will follow the order of the modules, so that rules from
    // modules later in the array take precedence of those from earlier
    // modules. If you call this function multiple times for the same root
    // in a way that changes the order of already mounted modules, the old
    // order will be changed.
    //
    // If a Content Security Policy nonce is provided, it is added to
    // the `<style>` tag generated by the library.
    static mount(_, H, V) {
      let q = _[SET], G = V && V.nonce;
      q ? G && q.setNonce(G) : q = new StyleSet(_, G), q.mount(Array.isArray(H) ? H : [H], _);
    }
  }
  let adoptedSet = /* @__PURE__ */ new Map();
  class StyleSet {
    constructor(_, H) {
      let V = _.ownerDocument || _, q = V.defaultView;
      if (!_.head && _.adoptedStyleSheets && q.CSSStyleSheet) {
        let G = adoptedSet.get(V);
        if (G) return _[SET] = G;
        this.sheet = new q.CSSStyleSheet(), adoptedSet.set(V, this);
      } else
        this.styleTag = V.createElement("style"), H && this.styleTag.setAttribute("nonce", H);
      this.modules = [], _[SET] = this;
    }
    mount(_, H) {
      let V = this.sheet, q = 0, G = 0;
      for (let J = 0; J < _.length; J++) {
        let Z = _[J], te = this.modules.indexOf(Z);
        if (te < G && te > -1 && (this.modules.splice(te, 1), G--, te = -1), te == -1) {
          if (this.modules.splice(G++, 0, Z), V) for (let ne = 0; ne < Z.rules.length; ne++)
            V.insertRule(Z.rules[ne], q++);
        } else {
          for (; G < te; ) q += this.modules[G++].rules.length;
          q += Z.rules.length, G++;
        }
      }
      if (V)
        H.adoptedStyleSheets.indexOf(this.sheet) < 0 && (H.adoptedStyleSheets = [this.sheet, ...H.adoptedStyleSheets]);
      else {
        let J = "";
        for (let te = 0; te < this.modules.length; te++)
          J += this.modules[te].getRules() + `
`;
        this.styleTag.textContent = J;
        let Z = H.head || H;
        this.styleTag.parentNode != Z && Z.insertBefore(this.styleTag, Z.firstChild);
      }
    }
    setNonce(_) {
      this.styleTag && this.styleTag.getAttribute("nonce") != _ && this.styleTag.setAttribute("nonce", _);
    }
  }
  var base = {
    8: "Backspace",
    9: "Tab",
    10: "Enter",
    12: "NumLock",
    13: "Enter",
    16: "Shift",
    17: "Control",
    18: "Alt",
    20: "CapsLock",
    27: "Escape",
    32: " ",
    33: "PageUp",
    34: "PageDown",
    35: "End",
    36: "Home",
    37: "ArrowLeft",
    38: "ArrowUp",
    39: "ArrowRight",
    40: "ArrowDown",
    44: "PrintScreen",
    45: "Insert",
    46: "Delete",
    59: ";",
    61: "=",
    91: "Meta",
    92: "Meta",
    106: "*",
    107: "+",
    108: ",",
    109: "-",
    110: ".",
    111: "/",
    144: "NumLock",
    145: "ScrollLock",
    160: "Shift",
    161: "Shift",
    162: "Control",
    163: "Control",
    164: "Alt",
    165: "Alt",
    173: "-",
    186: ";",
    187: "=",
    188: ",",
    189: "-",
    190: ".",
    191: "/",
    192: "`",
    219: "[",
    220: "\\",
    221: "]",
    222: "'"
  }, shift = {
    48: ")",
    49: "!",
    50: "@",
    51: "#",
    52: "$",
    53: "%",
    54: "^",
    55: "&",
    56: "*",
    57: "(",
    59: ":",
    61: "+",
    173: "_",
    186: ":",
    187: "+",
    188: "<",
    189: "_",
    190: ">",
    191: "?",
    192: "~",
    219: "{",
    220: "|",
    221: "}",
    222: '"'
  }, mac = typeof navigator < "u" && /Mac/.test(navigator.platform), ie$1 = typeof navigator < "u" && /MSIE \d|Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(navigator.userAgent);
  for (var i = 0; i < 10; i++) base[48 + i] = base[96 + i] = String(i);
  for (var i = 1; i <= 24; i++) base[i + 111] = "F" + i;
  for (var i = 65; i <= 90; i++)
    base[i] = String.fromCharCode(i + 32), shift[i] = String.fromCharCode(i);
  for (var code in base) shift.hasOwnProperty(code) || (shift[code] = base[code]);
  function keyName(O) {
    var _ = mac && O.metaKey && O.shiftKey && !O.ctrlKey && !O.altKey || ie$1 && O.shiftKey && O.key && O.key.length == 1 || O.key == "Unidentified", H = !_ && O.key || (O.shiftKey ? shift : base)[O.keyCode] || O.key || "Unidentified";
    return H == "Esc" && (H = "Escape"), H == "Del" && (H = "Delete"), H == "Left" && (H = "ArrowLeft"), H == "Up" && (H = "ArrowUp"), H == "Right" && (H = "ArrowRight"), H == "Down" && (H = "ArrowDown"), H;
  }
  function getSelection(O) {
    let _;
    return O.nodeType == 11 ? _ = O.getSelection ? O : O.ownerDocument : _ = O, _.getSelection();
  }
  function contains(O, _) {
    return _ ? O == _ || O.contains(_.nodeType != 1 ? _.parentNode : _) : !1;
  }
  function hasSelection(O, _) {
    if (!_.anchorNode)
      return !1;
    try {
      return contains(O, _.anchorNode);
    } catch {
      return !1;
    }
  }
  function clientRectsFor(O) {
    return O.nodeType == 3 ? textRange(O, 0, O.nodeValue.length).getClientRects() : O.nodeType == 1 ? O.getClientRects() : [];
  }
  function isEquivalentPosition(O, _, H, V) {
    return H ? scanFor(O, _, H, V, -1) || scanFor(O, _, H, V, 1) : !1;
  }
  function domIndex(O) {
    for (var _ = 0; ; _++)
      if (O = O.previousSibling, !O)
        return _;
  }
  function isBlockElement(O) {
    return O.nodeType == 1 && /^(DIV|P|LI|UL|OL|BLOCKQUOTE|DD|DT|H\d|SECTION|PRE)$/.test(O.nodeName);
  }
  function scanFor(O, _, H, V, q) {
    for (; ; ) {
      if (O == H && _ == V)
        return !0;
      if (_ == (q < 0 ? 0 : maxOffset(O))) {
        if (O.nodeName == "DIV")
          return !1;
        let G = O.parentNode;
        if (!G || G.nodeType != 1)
          return !1;
        _ = domIndex(O) + (q < 0 ? 0 : 1), O = G;
      } else if (O.nodeType == 1) {
        if (O = O.childNodes[_ + (q < 0 ? -1 : 0)], O.nodeType == 1 && O.contentEditable == "false")
          return !1;
        _ = q < 0 ? maxOffset(O) : 0;
      } else
        return !1;
    }
  }
  function maxOffset(O) {
    return O.nodeType == 3 ? O.nodeValue.length : O.childNodes.length;
  }
  function flattenRect(O, _) {
    let H = _ ? O.left : O.right;
    return { left: H, right: H, top: O.top, bottom: O.bottom };
  }
  function windowRect(O) {
    let _ = O.visualViewport;
    return _ ? {
      left: 0,
      right: _.width,
      top: 0,
      bottom: _.height
    } : {
      left: 0,
      right: O.innerWidth,
      top: 0,
      bottom: O.innerHeight
    };
  }
  function getScale(O, _) {
    let H = _.width / O.offsetWidth, V = _.height / O.offsetHeight;
    return (H > 0.995 && H < 1.005 || !isFinite(H) || Math.abs(_.width - O.offsetWidth) < 1) && (H = 1), (V > 0.995 && V < 1.005 || !isFinite(V) || Math.abs(_.height - O.offsetHeight) < 1) && (V = 1), { scaleX: H, scaleY: V };
  }
  function scrollRectIntoView(O, _, H, V, q, G, J, Z) {
    let te = O.ownerDocument, ne = te.defaultView || window;
    for (let ee = O, ae = !1; ee && !ae; )
      if (ee.nodeType == 1) {
        let oe, se = ee == te.body, ce = 1, le = 1;
        if (se)
          oe = windowRect(ne);
        else {
          if (/^(fixed|sticky)$/.test(getComputedStyle(ee).position) && (ae = !0), ee.scrollHeight <= ee.clientHeight && ee.scrollWidth <= ee.clientWidth) {
            ee = ee.assignedSlot || ee.parentNode;
            continue;
          }
          let fe = ee.getBoundingClientRect();
          ({ scaleX: ce, scaleY: le } = getScale(ee, fe)), oe = {
            left: fe.left,
            right: fe.left + ee.clientWidth * ce,
            top: fe.top,
            bottom: fe.top + ee.clientHeight * le
          };
        }
        let ue = 0, de = 0;
        if (q == "nearest")
          _.top < oe.top ? (de = -(oe.top - _.top + J), H > 0 && _.bottom > oe.bottom + de && (de = _.bottom - oe.bottom + de + J)) : _.bottom > oe.bottom && (de = _.bottom - oe.bottom + J, H < 0 && _.top - de < oe.top && (de = -(oe.top + de - _.top + J)));
        else {
          let fe = _.bottom - _.top, he = oe.bottom - oe.top;
          de = (q == "center" && fe <= he ? _.top + fe / 2 - he / 2 : q == "start" || q == "center" && H < 0 ? _.top - J : _.bottom - he + J) - oe.top;
        }
        if (V == "nearest" ? _.left < oe.left ? (ue = -(oe.left - _.left + G), H > 0 && _.right > oe.right + ue && (ue = _.right - oe.right + ue + G)) : _.right > oe.right && (ue = _.right - oe.right + G, H < 0 && _.left < oe.left + ue && (ue = -(oe.left + ue - _.left + G))) : ue = (V == "center" ? _.left + (_.right - _.left) / 2 - (oe.right - oe.left) / 2 : V == "start" == Z ? _.left - G : _.right - (oe.right - oe.left) + G) - oe.left, ue || de)
          if (se)
            ne.scrollBy(ue, de);
          else {
            let fe = 0, he = 0;
            if (de) {
              let me = ee.scrollTop;
              ee.scrollTop += de / le, he = (ee.scrollTop - me) * le;
            }
            if (ue) {
              let me = ee.scrollLeft;
              ee.scrollLeft += ue / ce, fe = (ee.scrollLeft - me) * ce;
            }
            _ = {
              left: _.left - fe,
              top: _.top - he,
              right: _.right - fe,
              bottom: _.bottom - he
            }, fe && Math.abs(fe - ue) < 1 && (V = "nearest"), he && Math.abs(he - de) < 1 && (q = "nearest");
          }
        if (se)
          break;
        ee = ee.assignedSlot || ee.parentNode;
      } else if (ee.nodeType == 11)
        ee = ee.host;
      else
        break;
  }
  function scrollableParents(O) {
    let _ = O.ownerDocument, H, V;
    for (let q = O.parentNode; q && !(q == _.body || H && V); )
      if (q.nodeType == 1)
        !V && q.scrollHeight > q.clientHeight && (V = q), !H && q.scrollWidth > q.clientWidth && (H = q), q = q.assignedSlot || q.parentNode;
      else if (q.nodeType == 11)
        q = q.host;
      else
        break;
    return { x: H, y: V };
  }
  class DOMSelectionState {
    constructor() {
      this.anchorNode = null, this.anchorOffset = 0, this.focusNode = null, this.focusOffset = 0;
    }
    eq(_) {
      return this.anchorNode == _.anchorNode && this.anchorOffset == _.anchorOffset && this.focusNode == _.focusNode && this.focusOffset == _.focusOffset;
    }
    setRange(_) {
      let { anchorNode: H, focusNode: V } = _;
      this.set(H, Math.min(_.anchorOffset, H ? maxOffset(H) : 0), V, Math.min(_.focusOffset, V ? maxOffset(V) : 0));
    }
    set(_, H, V, q) {
      this.anchorNode = _, this.anchorOffset = H, this.focusNode = V, this.focusOffset = q;
    }
  }
  let preventScrollSupported = null;
  function focusPreventScroll(O) {
    if (O.setActive)
      return O.setActive();
    if (preventScrollSupported)
      return O.focus(preventScrollSupported);
    let _ = [];
    for (let H = O; H && (_.push(H, H.scrollTop, H.scrollLeft), H != H.ownerDocument); H = H.parentNode)
      ;
    if (O.focus(preventScrollSupported == null ? {
      get preventScroll() {
        return preventScrollSupported = { preventScroll: !0 }, !0;
      }
    } : void 0), !preventScrollSupported) {
      preventScrollSupported = !1;
      for (let H = 0; H < _.length; ) {
        let V = _[H++], q = _[H++], G = _[H++];
        V.scrollTop != q && (V.scrollTop = q), V.scrollLeft != G && (V.scrollLeft = G);
      }
    }
  }
  let scratchRange;
  function textRange(O, _, H = _) {
    let V = scratchRange || (scratchRange = document.createRange());
    return V.setEnd(O, H), V.setStart(O, _), V;
  }
  function dispatchKey(O, _, H, V) {
    let q = { key: _, code: _, keyCode: H, which: H, cancelable: !0 };
    V && ({ altKey: q.altKey, ctrlKey: q.ctrlKey, shiftKey: q.shiftKey, metaKey: q.metaKey } = V);
    let G = new KeyboardEvent("keydown", q);
    G.synthetic = !0, O.dispatchEvent(G);
    let J = new KeyboardEvent("keyup", q);
    return J.synthetic = !0, O.dispatchEvent(J), G.defaultPrevented || J.defaultPrevented;
  }
  function getRoot(O) {
    for (; O; ) {
      if (O && (O.nodeType == 9 || O.nodeType == 11 && O.host))
        return O;
      O = O.assignedSlot || O.parentNode;
    }
    return null;
  }
  function clearAttributes(O) {
    for (; O.attributes.length; )
      O.removeAttributeNode(O.attributes[0]);
  }
  function atElementStart(O, _) {
    let H = _.focusNode, V = _.focusOffset;
    if (!H || _.anchorNode != H || _.anchorOffset != V)
      return !1;
    for (V = Math.min(V, maxOffset(H)); ; )
      if (V) {
        if (H.nodeType != 1)
          return !1;
        let q = H.childNodes[V - 1];
        q.contentEditable == "false" ? V-- : (H = q, V = maxOffset(H));
      } else {
        if (H == O)
          return !0;
        V = domIndex(H), H = H.parentNode;
      }
  }
  function isScrolledToBottom(O) {
    return O.scrollTop > Math.max(1, O.scrollHeight - O.clientHeight - 4);
  }
  function textNodeBefore(O, _) {
    for (let H = O, V = _; ; ) {
      if (H.nodeType == 3 && V > 0)
        return { node: H, offset: V };
      if (H.nodeType == 1 && V > 0) {
        if (H.contentEditable == "false")
          return null;
        H = H.childNodes[V - 1], V = maxOffset(H);
      } else if (H.parentNode && !isBlockElement(H))
        V = domIndex(H), H = H.parentNode;
      else
        return null;
    }
  }
  function textNodeAfter(O, _) {
    for (let H = O, V = _; ; ) {
      if (H.nodeType == 3 && V < H.nodeValue.length)
        return { node: H, offset: V };
      if (H.nodeType == 1 && V < H.childNodes.length) {
        if (H.contentEditable == "false")
          return null;
        H = H.childNodes[V], V = 0;
      } else if (H.parentNode && !isBlockElement(H))
        V = domIndex(H) + 1, H = H.parentNode;
      else
        return null;
    }
  }
  class DOMPos {
    constructor(_, H, V = !0) {
      this.node = _, this.offset = H, this.precise = V;
    }
    static before(_, H) {
      return new DOMPos(_.parentNode, domIndex(_), H);
    }
    static after(_, H) {
      return new DOMPos(_.parentNode, domIndex(_) + 1, H);
    }
  }
  const noChildren = [];
  class ContentView {
    constructor() {
      this.parent = null, this.dom = null, this.flags = 2;
    }
    get overrideDOMText() {
      return null;
    }
    get posAtStart() {
      return this.parent ? this.parent.posBefore(this) : 0;
    }
    get posAtEnd() {
      return this.posAtStart + this.length;
    }
    posBefore(_) {
      let H = this.posAtStart;
      for (let V of this.children) {
        if (V == _)
          return H;
        H += V.length + V.breakAfter;
      }
      throw new RangeError("Invalid child in posBefore");
    }
    posAfter(_) {
      return this.posBefore(_) + _.length;
    }
    sync(_, H) {
      if (this.flags & 2) {
        let V = this.dom, q = null, G;
        for (let J of this.children) {
          if (J.flags & 7) {
            if (!J.dom && (G = q ? q.nextSibling : V.firstChild)) {
              let Z = ContentView.get(G);
              (!Z || !Z.parent && Z.canReuseDOM(J)) && J.reuseDOM(G);
            }
            J.sync(_, H), J.flags &= -8;
          }
          if (G = q ? q.nextSibling : V.firstChild, H && !H.written && H.node == V && G != J.dom && (H.written = !0), J.dom.parentNode == V)
            for (; G && G != J.dom; )
              G = rm$1(G);
          else
            V.insertBefore(J.dom, G);
          q = J.dom;
        }
        for (G = q ? q.nextSibling : V.firstChild, G && H && H.node == V && (H.written = !0); G; )
          G = rm$1(G);
      } else if (this.flags & 1)
        for (let V of this.children)
          V.flags & 7 && (V.sync(_, H), V.flags &= -8);
    }
    reuseDOM(_) {
    }
    localPosFromDOM(_, H) {
      let V;
      if (_ == this.dom)
        V = this.dom.childNodes[H];
      else {
        let q = maxOffset(_) == 0 ? 0 : H == 0 ? -1 : 1;
        for (; ; ) {
          let G = _.parentNode;
          if (G == this.dom)
            break;
          q == 0 && G.firstChild != G.lastChild && (_ == G.firstChild ? q = -1 : q = 1), _ = G;
        }
        q < 0 ? V = _ : V = _.nextSibling;
      }
      if (V == this.dom.firstChild)
        return 0;
      for (; V && !ContentView.get(V); )
        V = V.nextSibling;
      if (!V)
        return this.length;
      for (let q = 0, G = 0; ; q++) {
        let J = this.children[q];
        if (J.dom == V)
          return G;
        G += J.length + J.breakAfter;
      }
    }
    domBoundsAround(_, H, V = 0) {
      let q = -1, G = -1, J = -1, Z = -1;
      for (let te = 0, ne = V, ee = V; te < this.children.length; te++) {
        let ae = this.children[te], oe = ne + ae.length;
        if (ne < _ && oe > H)
          return ae.domBoundsAround(_, H, ne);
        if (oe >= _ && q == -1 && (q = te, G = ne), ne > H && ae.dom.parentNode == this.dom) {
          J = te, Z = ee;
          break;
        }
        ee = oe, ne = oe + ae.breakAfter;
      }
      return {
        from: G,
        to: Z < 0 ? V + this.length : Z,
        startDOM: (q ? this.children[q - 1].dom.nextSibling : null) || this.dom.firstChild,
        endDOM: J < this.children.length && J >= 0 ? this.children[J].dom : null
      };
    }
    markDirty(_ = !1) {
      this.flags |= 2, this.markParentsDirty(_);
    }
    markParentsDirty(_) {
      for (let H = this.parent; H; H = H.parent) {
        if (_ && (H.flags |= 2), H.flags & 1)
          return;
        H.flags |= 1, _ = !1;
      }
    }
    setParent(_) {
      this.parent != _ && (this.parent = _, this.flags & 7 && this.markParentsDirty(!0));
    }
    setDOM(_) {
      this.dom != _ && (this.dom && (this.dom.cmView = null), this.dom = _, _.cmView = this);
    }
    get rootView() {
      for (let _ = this; ; ) {
        let H = _.parent;
        if (!H)
          return _;
        _ = H;
      }
    }
    replaceChildren(_, H, V = noChildren) {
      this.markDirty();
      for (let q = _; q < H; q++) {
        let G = this.children[q];
        G.parent == this && V.indexOf(G) < 0 && G.destroy();
      }
      V.length < 250 ? this.children.splice(_, H - _, ...V) : this.children = [].concat(this.children.slice(0, _), V, this.children.slice(H));
      for (let q = 0; q < V.length; q++)
        V[q].setParent(this);
    }
    ignoreMutation(_) {
      return !1;
    }
    ignoreEvent(_) {
      return !1;
    }
    childCursor(_ = this.length) {
      return new ChildCursor(this.children, _, this.children.length);
    }
    childPos(_, H = 1) {
      return this.childCursor().findPos(_, H);
    }
    toString() {
      let _ = this.constructor.name.replace("View", "");
      return _ + (this.children.length ? "(" + this.children.join() + ")" : this.length ? "[" + (_ == "Text" ? this.text : this.length) + "]" : "") + (this.breakAfter ? "#" : "");
    }
    static get(_) {
      return _.cmView;
    }
    get isEditable() {
      return !0;
    }
    get isWidget() {
      return !1;
    }
    get isHidden() {
      return !1;
    }
    merge(_, H, V, q, G, J) {
      return !1;
    }
    become(_) {
      return !1;
    }
    canReuseDOM(_) {
      return _.constructor == this.constructor && !((this.flags | _.flags) & 8);
    }
    // When this is a zero-length view with a side, this should return a
    // number <= 0 to indicate it is before its position, or a
    // number > 0 when after its position.
    getSide() {
      return 0;
    }
    destroy() {
      for (let _ of this.children)
        _.parent == this && _.destroy();
      this.parent = null;
    }
  }
  ContentView.prototype.breakAfter = 0;
  function rm$1(O) {
    let _ = O.nextSibling;
    return O.parentNode.removeChild(O), _;
  }
  class ChildCursor {
    constructor(_, H, V) {
      this.children = _, this.pos = H, this.i = V, this.off = 0;
    }
    findPos(_, H = 1) {
      for (; ; ) {
        if (_ > this.pos || _ == this.pos && (H > 0 || this.i == 0 || this.children[this.i - 1].breakAfter))
          return this.off = _ - this.pos, this;
        let V = this.children[--this.i];
        this.pos -= V.length + V.breakAfter;
      }
    }
  }
  function replaceRange(O, _, H, V, q, G, J, Z, te) {
    let { children: ne } = O, ee = ne.length ? ne[_] : null, ae = G.length ? G[G.length - 1] : null, oe = ae ? ae.breakAfter : J;
    if (!(_ == V && ee && !J && !oe && G.length < 2 && ee.merge(H, q, G.length ? ae : null, H == 0, Z, te))) {
      if (V < ne.length) {
        let se = ne[V];
        se && (q < se.length || se.breakAfter && (ae != null && ae.breakAfter)) ? (_ == V && (se = se.split(q), q = 0), !oe && ae && se.merge(0, q, ae, !0, 0, te) ? G[G.length - 1] = se : ((q || se.children.length && !se.children[0].length) && se.merge(0, q, null, !1, 0, te), G.push(se))) : se != null && se.breakAfter && (ae ? ae.breakAfter = 1 : J = 1), V++;
      }
      for (ee && (ee.breakAfter = J, H > 0 && (!J && G.length && ee.merge(H, ee.length, G[0], !1, Z, 0) ? ee.breakAfter = G.shift().breakAfter : (H < ee.length || ee.children.length && ee.children[ee.children.length - 1].length == 0) && ee.merge(H, ee.length, null, !1, Z, 0), _++)); _ < V && G.length; )
        if (ne[V - 1].become(G[G.length - 1]))
          V--, G.pop(), te = G.length ? 0 : Z;
        else if (ne[_].become(G[0]))
          _++, G.shift(), Z = G.length ? 0 : te;
        else
          break;
      !G.length && _ && V < ne.length && !ne[_ - 1].breakAfter && ne[V].merge(0, 0, ne[_ - 1], !1, Z, te) && _--, (_ < V || G.length) && O.replaceChildren(_, V, G);
    }
  }
  function mergeChildrenInto(O, _, H, V, q, G) {
    let J = O.childCursor(), { i: Z, off: te } = J.findPos(H, 1), { i: ne, off: ee } = J.findPos(_, -1), ae = _ - H;
    for (let oe of V)
      ae += oe.length;
    O.length += ae, replaceRange(O, ne, ee, Z, te, V, 0, q, G);
  }
  let nav = typeof navigator < "u" ? navigator : { userAgent: "", vendor: "", platform: "" }, doc = typeof document < "u" ? document : { documentElement: { style: {} } };
  const ie_edge = /* @__PURE__ */ /Edge\/(\d+)/.exec(nav.userAgent), ie_upto10 = /* @__PURE__ */ /MSIE \d/.test(nav.userAgent), ie_11up = /* @__PURE__ */ /Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(nav.userAgent), ie = !!(ie_upto10 || ie_11up || ie_edge), gecko = !ie && /* @__PURE__ */ /gecko\/(\d+)/i.test(nav.userAgent), chrome$1 = !ie && /* @__PURE__ */ /Chrome\/(\d+)/.exec(nav.userAgent), webkit = "webkitFontSmoothing" in doc.documentElement.style, safari = !ie && /* @__PURE__ */ /Apple Computer/.test(nav.vendor), ios = safari && (/* @__PURE__ */ /Mobile\/\w+/.test(nav.userAgent) || nav.maxTouchPoints > 2);
  var browser = {
    mac: ios || /* @__PURE__ */ /Mac/.test(nav.platform),
    windows: /* @__PURE__ */ /Win/.test(nav.platform),
    linux: /* @__PURE__ */ /Linux|X11/.test(nav.platform),
    ie,
    ie_version: ie_upto10 ? doc.documentMode || 6 : ie_11up ? +ie_11up[1] : ie_edge ? +ie_edge[1] : 0,
    gecko,
    gecko_version: gecko ? +(/* @__PURE__ */ /Firefox\/(\d+)/.exec(nav.userAgent) || [0, 0])[1] : 0,
    chrome: !!chrome$1,
    chrome_version: chrome$1 ? +chrome$1[1] : 0,
    ios,
    android: /* @__PURE__ */ /Android\b/.test(nav.userAgent),
    webkit,
    safari,
    webkit_version: webkit ? +(/* @__PURE__ */ /\bAppleWebKit\/(\d+)/.exec(nav.userAgent) || [0, 0])[1] : 0,
    tabSize: doc.documentElement.style.tabSize != null ? "tab-size" : "-moz-tab-size"
  };
  const MaxJoinLen = 256;
  class TextView extends ContentView {
    constructor(_) {
      super(), this.text = _;
    }
    get length() {
      return this.text.length;
    }
    createDOM(_) {
      this.setDOM(_ || document.createTextNode(this.text));
    }
    sync(_, H) {
      this.dom || this.createDOM(), this.dom.nodeValue != this.text && (H && H.node == this.dom && (H.written = !0), this.dom.nodeValue = this.text);
    }
    reuseDOM(_) {
      _.nodeType == 3 && this.createDOM(_);
    }
    merge(_, H, V) {
      return this.flags & 8 || V && (!(V instanceof TextView) || this.length - (H - _) + V.length > MaxJoinLen || V.flags & 8) ? !1 : (this.text = this.text.slice(0, _) + (V ? V.text : "") + this.text.slice(H), this.markDirty(), !0);
    }
    split(_) {
      let H = new TextView(this.text.slice(_));
      return this.text = this.text.slice(0, _), this.markDirty(), H.flags |= this.flags & 8, H;
    }
    localPosFromDOM(_, H) {
      return _ == this.dom ? H : H ? this.text.length : 0;
    }
    domAtPos(_) {
      return new DOMPos(this.dom, _);
    }
    domBoundsAround(_, H, V) {
      return { from: V, to: V + this.length, startDOM: this.dom, endDOM: this.dom.nextSibling };
    }
    coordsAt(_, H) {
      return textCoords(this.dom, _, H);
    }
  }
  class MarkView extends ContentView {
    constructor(_, H = [], V = 0) {
      super(), this.mark = _, this.children = H, this.length = V;
      for (let q of H)
        q.setParent(this);
    }
    setAttrs(_) {
      if (clearAttributes(_), this.mark.class && (_.className = this.mark.class), this.mark.attrs)
        for (let H in this.mark.attrs)
          _.setAttribute(H, this.mark.attrs[H]);
      return _;
    }
    canReuseDOM(_) {
      return super.canReuseDOM(_) && !((this.flags | _.flags) & 8);
    }
    reuseDOM(_) {
      _.nodeName == this.mark.tagName.toUpperCase() && (this.setDOM(_), this.flags |= 6);
    }
    sync(_, H) {
      this.dom ? this.flags & 4 && this.setAttrs(this.dom) : this.setDOM(this.setAttrs(document.createElement(this.mark.tagName))), super.sync(_, H);
    }
    merge(_, H, V, q, G, J) {
      return V && (!(V instanceof MarkView && V.mark.eq(this.mark)) || _ && G <= 0 || H < this.length && J <= 0) ? !1 : (mergeChildrenInto(this, _, H, V ? V.children.slice() : [], G - 1, J - 1), this.markDirty(), !0);
    }
    split(_) {
      let H = [], V = 0, q = -1, G = 0;
      for (let Z of this.children) {
        let te = V + Z.length;
        te > _ && H.push(V < _ ? Z.split(_ - V) : Z), q < 0 && V >= _ && (q = G), V = te, G++;
      }
      let J = this.length - _;
      return this.length = _, q > -1 && (this.children.length = q, this.markDirty()), new MarkView(this.mark, H, J);
    }
    domAtPos(_) {
      return inlineDOMAtPos(this, _);
    }
    coordsAt(_, H) {
      return coordsInChildren(this, _, H);
    }
  }
  function textCoords(O, _, H) {
    let V = O.nodeValue.length;
    _ > V && (_ = V);
    let q = _, G = _, J = 0;
    _ == 0 && H < 0 || _ == V && H >= 0 ? browser.chrome || browser.gecko || (_ ? (q--, J = 1) : G < V && (G++, J = -1)) : H < 0 ? q-- : G < V && G++;
    let Z = textRange(O, q, G).getClientRects();
    if (!Z.length)
      return null;
    let te = Z[(J ? J < 0 : H >= 0) ? 0 : Z.length - 1];
    return browser.safari && !J && te.width == 0 && (te = Array.prototype.find.call(Z, (ne) => ne.width) || te), J ? flattenRect(te, J < 0) : te || null;
  }
  class WidgetView extends ContentView {
    static create(_, H, V) {
      return new WidgetView(_, H, V);
    }
    constructor(_, H, V) {
      super(), this.widget = _, this.length = H, this.side = V, this.prevWidget = null;
    }
    split(_) {
      let H = WidgetView.create(this.widget, this.length - _, this.side);
      return this.length -= _, H;
    }
    sync(_) {
      (!this.dom || !this.widget.updateDOM(this.dom, _)) && (this.dom && this.prevWidget && this.prevWidget.destroy(this.dom), this.prevWidget = null, this.setDOM(this.widget.toDOM(_)), this.widget.editable || (this.dom.contentEditable = "false"));
    }
    getSide() {
      return this.side;
    }
    merge(_, H, V, q, G, J) {
      return V && (!(V instanceof WidgetView) || !this.widget.compare(V.widget) || _ > 0 && G <= 0 || H < this.length && J <= 0) ? !1 : (this.length = _ + (V ? V.length : 0) + (this.length - H), !0);
    }
    become(_) {
      return _ instanceof WidgetView && _.side == this.side && this.widget.constructor == _.widget.constructor ? (this.widget.compare(_.widget) || this.markDirty(!0), this.dom && !this.prevWidget && (this.prevWidget = this.widget), this.widget = _.widget, this.length = _.length, !0) : !1;
    }
    ignoreMutation() {
      return !0;
    }
    ignoreEvent(_) {
      return this.widget.ignoreEvent(_);
    }
    get overrideDOMText() {
      if (this.length == 0)
        return Text.empty;
      let _ = this;
      for (; _.parent; )
        _ = _.parent;
      let { view: H } = _, V = H && H.state.doc, q = this.posAtStart;
      return V ? V.slice(q, q + this.length) : Text.empty;
    }
    domAtPos(_) {
      return (this.length ? _ == 0 : this.side > 0) ? DOMPos.before(this.dom) : DOMPos.after(this.dom, _ == this.length);
    }
    domBoundsAround() {
      return null;
    }
    coordsAt(_, H) {
      let V = this.widget.coordsAt(this.dom, _, H);
      if (V)
        return V;
      let q = this.dom.getClientRects(), G = null;
      if (!q.length)
        return null;
      let J = this.side ? this.side < 0 : _ > 0;
      for (let Z = J ? q.length - 1 : 0; G = q[Z], !(_ > 0 ? Z == 0 : Z == q.length - 1 || G.top < G.bottom); Z += J ? -1 : 1)
        ;
      return flattenRect(G, !J);
    }
    get isEditable() {
      return !1;
    }
    get isWidget() {
      return !0;
    }
    get isHidden() {
      return this.widget.isHidden;
    }
    destroy() {
      super.destroy(), this.dom && this.widget.destroy(this.dom);
    }
  }
  class WidgetBufferView extends ContentView {
    constructor(_) {
      super(), this.side = _;
    }
    get length() {
      return 0;
    }
    merge() {
      return !1;
    }
    become(_) {
      return _ instanceof WidgetBufferView && _.side == this.side;
    }
    split() {
      return new WidgetBufferView(this.side);
    }
    sync() {
      if (!this.dom) {
        let _ = document.createElement("img");
        _.className = "cm-widgetBuffer", _.setAttribute("aria-hidden", "true"), this.setDOM(_);
      }
    }
    getSide() {
      return this.side;
    }
    domAtPos(_) {
      return this.side > 0 ? DOMPos.before(this.dom) : DOMPos.after(this.dom);
    }
    localPosFromDOM() {
      return 0;
    }
    domBoundsAround() {
      return null;
    }
    coordsAt(_) {
      return this.dom.getBoundingClientRect();
    }
    get overrideDOMText() {
      return Text.empty;
    }
    get isHidden() {
      return !0;
    }
  }
  TextView.prototype.children = WidgetView.prototype.children = WidgetBufferView.prototype.children = noChildren;
  function inlineDOMAtPos(O, _) {
    let H = O.dom, { children: V } = O, q = 0;
    for (let G = 0; q < V.length; q++) {
      let J = V[q], Z = G + J.length;
      if (!(Z == G && J.getSide() <= 0)) {
        if (_ > G && _ < Z && J.dom.parentNode == H)
          return J.domAtPos(_ - G);
        if (_ <= G)
          break;
        G = Z;
      }
    }
    for (let G = q; G > 0; G--) {
      let J = V[G - 1];
      if (J.dom.parentNode == H)
        return J.domAtPos(J.length);
    }
    for (let G = q; G < V.length; G++) {
      let J = V[G];
      if (J.dom.parentNode == H)
        return J.domAtPos(0);
    }
    return new DOMPos(H, 0);
  }
  function joinInlineInto(O, _, H) {
    let V, { children: q } = O;
    H > 0 && _ instanceof MarkView && q.length && (V = q[q.length - 1]) instanceof MarkView && V.mark.eq(_.mark) ? joinInlineInto(V, _.children[0], H - 1) : (q.push(_), _.setParent(O)), O.length += _.length;
  }
  function coordsInChildren(O, _, H) {
    let V = null, q = -1, G = null, J = -1;
    function Z(ne, ee) {
      for (let ae = 0, oe = 0; ae < ne.children.length && oe <= ee; ae++) {
        let se = ne.children[ae], ce = oe + se.length;
        ce >= ee && (se.children.length ? Z(se, ee - oe) : (!G || G.isHidden && H > 0) && (ce > ee || oe == ce && se.getSide() > 0) ? (G = se, J = ee - oe) : (oe < ee || oe == ce && se.getSide() < 0 && !se.isHidden) && (V = se, q = ee - oe)), oe = ce;
      }
    }
    Z(O, _);
    let te = (H < 0 ? V : G) || V || G;
    return te ? te.coordsAt(Math.max(0, te == V ? q : J), H) : fallbackRect(O);
  }
  function fallbackRect(O) {
    let _ = O.dom.lastChild;
    if (!_)
      return O.dom.getBoundingClientRect();
    let H = clientRectsFor(_);
    return H[H.length - 1] || null;
  }
  function combineAttrs(O, _) {
    for (let H in O)
      H == "class" && _.class ? _.class += " " + O.class : H == "style" && _.style ? _.style += ";" + O.style : _[H] = O[H];
    return _;
  }
  const noAttrs$1 = /* @__PURE__ */ Object.create(null);
  function attrsEq(O, _, H) {
    if (O == _)
      return !0;
    O || (O = noAttrs$1), _ || (_ = noAttrs$1);
    let V = Object.keys(O), q = Object.keys(_);
    if (V.length - (H && V.indexOf(H) > -1 ? 1 : 0) != q.length - (H && q.indexOf(H) > -1 ? 1 : 0))
      return !1;
    for (let G of V)
      if (G != H && (q.indexOf(G) == -1 || O[G] !== _[G]))
        return !1;
    return !0;
  }
  function updateAttrs(O, _, H) {
    let V = !1;
    if (_)
      for (let q in _)
        H && q in H || (V = !0, q == "style" ? O.style.cssText = "" : O.removeAttribute(q));
    if (H)
      for (let q in H)
        _ && _[q] == H[q] || (V = !0, q == "style" ? O.style.cssText = H[q] : O.setAttribute(q, H[q]));
    return V;
  }
  function getAttrs(O) {
    let _ = /* @__PURE__ */ Object.create(null);
    for (let H = 0; H < O.attributes.length; H++) {
      let V = O.attributes[H];
      _[V.name] = V.value;
    }
    return _;
  }
  class WidgetType {
    /**
    Compare this instance to another instance of the same type.
    (TypeScript can't express this, but only instances of the same
    specific class will be passed to this method.) This is used to
    avoid redrawing widgets when they are replaced by a new
    decoration of the same type. The default implementation just
    returns `false`, which will cause new instances of the widget to
    always be redrawn.
    */
    eq(_) {
      return !1;
    }
    /**
    Update a DOM element created by a widget of the same type (but
    different, non-`eq` content) to reflect this widget. May return
    true to indicate that it could update, false to indicate it
    couldn't (in which case the widget will be redrawn). The default
    implementation just returns false.
    */
    updateDOM(_, H) {
      return !1;
    }
    /**
    @internal
    */
    compare(_) {
      return this == _ || this.constructor == _.constructor && this.eq(_);
    }
    /**
    The estimated height this widget will have, to be used when
    estimating the height of content that hasn't been drawn. May
    return -1 to indicate you don't know. The default implementation
    returns -1.
    */
    get estimatedHeight() {
      return -1;
    }
    /**
    For inline widgets that are displayed inline (as opposed to
    `inline-block`) and introduce line breaks (through `<br>` tags
    or textual newlines), this must indicate the amount of line
    breaks they introduce. Defaults to 0.
    */
    get lineBreaks() {
      return 0;
    }
    /**
    Can be used to configure which kinds of events inside the widget
    should be ignored by the editor. The default is to ignore all
    events.
    */
    ignoreEvent(_) {
      return !0;
    }
    /**
    Override the way screen coordinates for positions at/in the
    widget are found. `pos` will be the offset into the widget, and
    `side` the side of the position that is being queried—less than
    zero for before, greater than zero for after, and zero for
    directly at that position.
    */
    coordsAt(_, H, V) {
      return null;
    }
    /**
    @internal
    */
    get isHidden() {
      return !1;
    }
    /**
    @internal
    */
    get editable() {
      return !1;
    }
    /**
    This is called when the an instance of the widget is removed
    from the editor view.
    */
    destroy(_) {
    }
  }
  var BlockType = /* @__PURE__ */ function(O) {
    return O[O.Text = 0] = "Text", O[O.WidgetBefore = 1] = "WidgetBefore", O[O.WidgetAfter = 2] = "WidgetAfter", O[O.WidgetRange = 3] = "WidgetRange", O;
  }(BlockType || (BlockType = {}));
  class Decoration extends RangeValue {
    constructor(_, H, V, q) {
      super(), this.startSide = _, this.endSide = H, this.widget = V, this.spec = q;
    }
    /**
    @internal
    */
    get heightRelevant() {
      return !1;
    }
    /**
    Create a mark decoration, which influences the styling of the
    content in its range. Nested mark decorations will cause nested
    DOM elements to be created. Nesting order is determined by
    precedence of the [facet](https://codemirror.net/6/docs/ref/#view.EditorView^decorations), with
    the higher-precedence decorations creating the inner DOM nodes.
    Such elements are split on line boundaries and on the boundaries
    of lower-precedence decorations.
    */
    static mark(_) {
      return new MarkDecoration(_);
    }
    /**
    Create a widget decoration, which displays a DOM element at the
    given position.
    */
    static widget(_) {
      let H = Math.max(-1e4, Math.min(1e4, _.side || 0)), V = !!_.block;
      return H += V && !_.inlineOrder ? H > 0 ? 3e8 : -4e8 : H > 0 ? 1e8 : -1e8, new PointDecoration(_, H, H, V, _.widget || null, !1);
    }
    /**
    Create a replace decoration which replaces the given range with
    a widget, or simply hides it.
    */
    static replace(_) {
      let H = !!_.block, V, q;
      if (_.isBlockGap)
        V = -5e8, q = 4e8;
      else {
        let { start: G, end: J } = getInclusive(_, H);
        V = (G ? H ? -3e8 : -1 : 5e8) - 1, q = (J ? H ? 2e8 : 1 : -6e8) + 1;
      }
      return new PointDecoration(_, V, q, H, _.widget || null, !0);
    }
    /**
    Create a line decoration, which can add DOM attributes to the
    line starting at the given position.
    */
    static line(_) {
      return new LineDecoration(_);
    }
    /**
    Build a [`DecorationSet`](https://codemirror.net/6/docs/ref/#view.DecorationSet) from the given
    decorated range or ranges. If the ranges aren't already sorted,
    pass `true` for `sort` to make the library sort them for you.
    */
    static set(_, H = !1) {
      return RangeSet.of(_, H);
    }
    /**
    @internal
    */
    hasHeight() {
      return this.widget ? this.widget.estimatedHeight > -1 : !1;
    }
  }
  Decoration.none = RangeSet.empty;
  class MarkDecoration extends Decoration {
    constructor(_) {
      let { start: H, end: V } = getInclusive(_);
      super(H ? -1 : 5e8, V ? 1 : -6e8, null, _), this.tagName = _.tagName || "span", this.class = _.class || "", this.attrs = _.attributes || null;
    }
    eq(_) {
      var H, V;
      return this == _ || _ instanceof MarkDecoration && this.tagName == _.tagName && (this.class || ((H = this.attrs) === null || H === void 0 ? void 0 : H.class)) == (_.class || ((V = _.attrs) === null || V === void 0 ? void 0 : V.class)) && attrsEq(this.attrs, _.attrs, "class");
    }
    range(_, H = _) {
      if (_ >= H)
        throw new RangeError("Mark decorations may not be empty");
      return super.range(_, H);
    }
  }
  MarkDecoration.prototype.point = !1;
  class LineDecoration extends Decoration {
    constructor(_) {
      super(-2e8, -2e8, null, _);
    }
    eq(_) {
      return _ instanceof LineDecoration && this.spec.class == _.spec.class && attrsEq(this.spec.attributes, _.spec.attributes);
    }
    range(_, H = _) {
      if (H != _)
        throw new RangeError("Line decoration ranges must be zero-length");
      return super.range(_, H);
    }
  }
  LineDecoration.prototype.mapMode = MapMode.TrackBefore;
  LineDecoration.prototype.point = !0;
  class PointDecoration extends Decoration {
    constructor(_, H, V, q, G, J) {
      super(H, V, G, _), this.block = q, this.isReplace = J, this.mapMode = q ? H <= 0 ? MapMode.TrackBefore : MapMode.TrackAfter : MapMode.TrackDel;
    }
    // Only relevant when this.block == true
    get type() {
      return this.startSide != this.endSide ? BlockType.WidgetRange : this.startSide <= 0 ? BlockType.WidgetBefore : BlockType.WidgetAfter;
    }
    get heightRelevant() {
      return this.block || !!this.widget && (this.widget.estimatedHeight >= 5 || this.widget.lineBreaks > 0);
    }
    eq(_) {
      return _ instanceof PointDecoration && widgetsEq(this.widget, _.widget) && this.block == _.block && this.startSide == _.startSide && this.endSide == _.endSide;
    }
    range(_, H = _) {
      if (this.isReplace && (_ > H || _ == H && this.startSide > 0 && this.endSide <= 0))
        throw new RangeError("Invalid range for replacement decoration");
      if (!this.isReplace && H != _)
        throw new RangeError("Widget decorations can only have zero-length ranges");
      return super.range(_, H);
    }
  }
  PointDecoration.prototype.point = !0;
  function getInclusive(O, _ = !1) {
    let { inclusiveStart: H, inclusiveEnd: V } = O;
    return H == null && (H = O.inclusive), V == null && (V = O.inclusive), { start: H ?? _, end: V ?? _ };
  }
  function widgetsEq(O, _) {
    return O == _ || !!(O && _ && O.compare(_));
  }
  function addRange(O, _, H, V = 0) {
    let q = H.length - 1;
    q >= 0 && H[q] + V >= O ? H[q] = Math.max(H[q], _) : H.push(O, _);
  }
  class LineView extends ContentView {
    constructor() {
      super(...arguments), this.children = [], this.length = 0, this.prevAttrs = void 0, this.attrs = null, this.breakAfter = 0;
    }
    // Consumes source
    merge(_, H, V, q, G, J) {
      if (V) {
        if (!(V instanceof LineView))
          return !1;
        this.dom || V.transferDOM(this);
      }
      return q && this.setDeco(V ? V.attrs : null), mergeChildrenInto(this, _, H, V ? V.children.slice() : [], G, J), !0;
    }
    split(_) {
      let H = new LineView();
      if (H.breakAfter = this.breakAfter, this.length == 0)
        return H;
      let { i: V, off: q } = this.childPos(_);
      q && (H.append(this.children[V].split(q), 0), this.children[V].merge(q, this.children[V].length, null, !1, 0, 0), V++);
      for (let G = V; G < this.children.length; G++)
        H.append(this.children[G], 0);
      for (; V > 0 && this.children[V - 1].length == 0; )
        this.children[--V].destroy();
      return this.children.length = V, this.markDirty(), this.length = _, H;
    }
    transferDOM(_) {
      this.dom && (this.markDirty(), _.setDOM(this.dom), _.prevAttrs = this.prevAttrs === void 0 ? this.attrs : this.prevAttrs, this.prevAttrs = void 0, this.dom = null);
    }
    setDeco(_) {
      attrsEq(this.attrs, _) || (this.dom && (this.prevAttrs = this.attrs, this.markDirty()), this.attrs = _);
    }
    append(_, H) {
      joinInlineInto(this, _, H);
    }
    // Only called when building a line view in ContentBuilder
    addLineDeco(_) {
      let H = _.spec.attributes, V = _.spec.class;
      H && (this.attrs = combineAttrs(H, this.attrs || {})), V && (this.attrs = combineAttrs({ class: V }, this.attrs || {}));
    }
    domAtPos(_) {
      return inlineDOMAtPos(this, _);
    }
    reuseDOM(_) {
      _.nodeName == "DIV" && (this.setDOM(_), this.flags |= 6);
    }
    sync(_, H) {
      var V;
      this.dom ? this.flags & 4 && (clearAttributes(this.dom), this.dom.className = "cm-line", this.prevAttrs = this.attrs ? null : void 0) : (this.setDOM(document.createElement("div")), this.dom.className = "cm-line", this.prevAttrs = this.attrs ? null : void 0), this.prevAttrs !== void 0 && (updateAttrs(this.dom, this.prevAttrs, this.attrs), this.dom.classList.add("cm-line"), this.prevAttrs = void 0), super.sync(_, H);
      let q = this.dom.lastChild;
      for (; q && ContentView.get(q) instanceof MarkView; )
        q = q.lastChild;
      if (!q || !this.length || q.nodeName != "BR" && ((V = ContentView.get(q)) === null || V === void 0 ? void 0 : V.isEditable) == !1 && (!browser.ios || !this.children.some((G) => G instanceof TextView))) {
        let G = document.createElement("BR");
        G.cmIgnore = !0, this.dom.appendChild(G);
      }
    }
    measureTextSize() {
      if (this.children.length == 0 || this.length > 20)
        return null;
      let _ = 0, H;
      for (let V of this.children) {
        if (!(V instanceof TextView) || /[^ -~]/.test(V.text))
          return null;
        let q = clientRectsFor(V.dom);
        if (q.length != 1)
          return null;
        _ += q[0].width, H = q[0].height;
      }
      return _ ? {
        lineHeight: this.dom.getBoundingClientRect().height,
        charWidth: _ / this.length,
        textHeight: H
      } : null;
    }
    coordsAt(_, H) {
      let V = coordsInChildren(this, _, H);
      if (!this.children.length && V && this.parent) {
        let { heightOracle: q } = this.parent.view.viewState, G = V.bottom - V.top;
        if (Math.abs(G - q.lineHeight) < 2 && q.textHeight < G) {
          let J = (G - q.textHeight) / 2;
          return { top: V.top + J, bottom: V.bottom - J, left: V.left, right: V.left };
        }
      }
      return V;
    }
    become(_) {
      return _ instanceof LineView && this.children.length == 0 && _.children.length == 0 && attrsEq(this.attrs, _.attrs) && this.breakAfter == _.breakAfter;
    }
    covers() {
      return !0;
    }
    static find(_, H) {
      for (let V = 0, q = 0; V < _.children.length; V++) {
        let G = _.children[V], J = q + G.length;
        if (J >= H) {
          if (G instanceof LineView)
            return G;
          if (J > H)
            break;
        }
        q = J + G.breakAfter;
      }
      return null;
    }
  }
  class BlockWidgetView extends ContentView {
    constructor(_, H, V) {
      super(), this.widget = _, this.length = H, this.deco = V, this.breakAfter = 0, this.prevWidget = null;
    }
    merge(_, H, V, q, G, J) {
      return V && (!(V instanceof BlockWidgetView) || !this.widget.compare(V.widget) || _ > 0 && G <= 0 || H < this.length && J <= 0) ? !1 : (this.length = _ + (V ? V.length : 0) + (this.length - H), !0);
    }
    domAtPos(_) {
      return _ == 0 ? DOMPos.before(this.dom) : DOMPos.after(this.dom, _ == this.length);
    }
    split(_) {
      let H = this.length - _;
      this.length = _;
      let V = new BlockWidgetView(this.widget, H, this.deco);
      return V.breakAfter = this.breakAfter, V;
    }
    get children() {
      return noChildren;
    }
    sync(_) {
      (!this.dom || !this.widget.updateDOM(this.dom, _)) && (this.dom && this.prevWidget && this.prevWidget.destroy(this.dom), this.prevWidget = null, this.setDOM(this.widget.toDOM(_)), this.widget.editable || (this.dom.contentEditable = "false"));
    }
    get overrideDOMText() {
      return this.parent ? this.parent.view.state.doc.slice(this.posAtStart, this.posAtEnd) : Text.empty;
    }
    domBoundsAround() {
      return null;
    }
    become(_) {
      return _ instanceof BlockWidgetView && _.widget.constructor == this.widget.constructor ? (_.widget.compare(this.widget) || this.markDirty(!0), this.dom && !this.prevWidget && (this.prevWidget = this.widget), this.widget = _.widget, this.length = _.length, this.deco = _.deco, this.breakAfter = _.breakAfter, !0) : !1;
    }
    ignoreMutation() {
      return !0;
    }
    ignoreEvent(_) {
      return this.widget.ignoreEvent(_);
    }
    get isEditable() {
      return !1;
    }
    get isWidget() {
      return !0;
    }
    coordsAt(_, H) {
      let V = this.widget.coordsAt(this.dom, _, H);
      return V || (this.widget instanceof BlockGapWidget ? null : flattenRect(this.dom.getBoundingClientRect(), this.length ? _ == 0 : H <= 0));
    }
    destroy() {
      super.destroy(), this.dom && this.widget.destroy(this.dom);
    }
    covers(_) {
      let { startSide: H, endSide: V } = this.deco;
      return H == V ? !1 : _ < 0 ? H < 0 : V > 0;
    }
  }
  class BlockGapWidget extends WidgetType {
    constructor(_) {
      super(), this.height = _;
    }
    toDOM() {
      let _ = document.createElement("div");
      return _.className = "cm-gap", this.updateDOM(_), _;
    }
    eq(_) {
      return _.height == this.height;
    }
    updateDOM(_) {
      return _.style.height = this.height + "px", !0;
    }
    get editable() {
      return !0;
    }
    get estimatedHeight() {
      return this.height;
    }
    ignoreEvent() {
      return !1;
    }
  }
  class ContentBuilder {
    constructor(_, H, V, q) {
      this.doc = _, this.pos = H, this.end = V, this.disallowBlockEffectsFor = q, this.content = [], this.curLine = null, this.breakAtStart = 0, this.pendingBuffer = 0, this.bufferMarks = [], this.atCursorPos = !0, this.openStart = -1, this.openEnd = -1, this.text = "", this.textOff = 0, this.cursor = _.iter(), this.skip = H;
    }
    posCovered() {
      if (this.content.length == 0)
        return !this.breakAtStart && this.doc.lineAt(this.pos).from != this.pos;
      let _ = this.content[this.content.length - 1];
      return !(_.breakAfter || _ instanceof BlockWidgetView && _.deco.endSide < 0);
    }
    getLine() {
      return this.curLine || (this.content.push(this.curLine = new LineView()), this.atCursorPos = !0), this.curLine;
    }
    flushBuffer(_ = this.bufferMarks) {
      this.pendingBuffer && (this.curLine.append(wrapMarks(new WidgetBufferView(-1), _), _.length), this.pendingBuffer = 0);
    }
    addBlockWidget(_) {
      this.flushBuffer(), this.curLine = null, this.content.push(_);
    }
    finish(_) {
      this.pendingBuffer && _ <= this.bufferMarks.length ? this.flushBuffer() : this.pendingBuffer = 0, !this.posCovered() && !(_ && this.content.length && this.content[this.content.length - 1] instanceof BlockWidgetView) && this.getLine();
    }
    buildText(_, H, V) {
      for (; _ > 0; ) {
        if (this.textOff == this.text.length) {
          let { value: G, lineBreak: J, done: Z } = this.cursor.next(this.skip);
          if (this.skip = 0, Z)
            throw new Error("Ran out of text content when drawing inline views");
          if (J) {
            this.posCovered() || this.getLine(), this.content.length ? this.content[this.content.length - 1].breakAfter = 1 : this.breakAtStart = 1, this.flushBuffer(), this.curLine = null, this.atCursorPos = !0, _--;
            continue;
          } else
            this.text = G, this.textOff = 0;
        }
        let q = Math.min(
          this.text.length - this.textOff,
          _,
          512
          /* T.Chunk */
        );
        this.flushBuffer(H.slice(H.length - V)), this.getLine().append(wrapMarks(new TextView(this.text.slice(this.textOff, this.textOff + q)), H), V), this.atCursorPos = !0, this.textOff += q, _ -= q, V = 0;
      }
    }
    span(_, H, V, q) {
      this.buildText(H - _, V, q), this.pos = H, this.openStart < 0 && (this.openStart = q);
    }
    point(_, H, V, q, G, J) {
      if (this.disallowBlockEffectsFor[J] && V instanceof PointDecoration) {
        if (V.block)
          throw new RangeError("Block decorations may not be specified via plugins");
        if (H > this.doc.lineAt(this.pos).to)
          throw new RangeError("Decorations that replace line breaks may not be specified via plugins");
      }
      let Z = H - _;
      if (V instanceof PointDecoration)
        if (V.block)
          V.startSide > 0 && !this.posCovered() && this.getLine(), this.addBlockWidget(new BlockWidgetView(V.widget || NullWidget.block, Z, V));
        else {
          let te = WidgetView.create(V.widget || NullWidget.inline, Z, Z ? 0 : V.startSide), ne = this.atCursorPos && !te.isEditable && G <= q.length && (_ < H || V.startSide > 0), ee = !te.isEditable && (_ < H || G > q.length || V.startSide <= 0), ae = this.getLine();
          this.pendingBuffer == 2 && !ne && !te.isEditable && (this.pendingBuffer = 0), this.flushBuffer(q), ne && (ae.append(wrapMarks(new WidgetBufferView(1), q), G), G = q.length + Math.max(0, G - q.length)), ae.append(wrapMarks(te, q), G), this.atCursorPos = ee, this.pendingBuffer = ee ? _ < H || G > q.length ? 1 : 2 : 0, this.pendingBuffer && (this.bufferMarks = q.slice());
        }
      else this.doc.lineAt(this.pos).from == this.pos && this.getLine().addLineDeco(V);
      Z && (this.textOff + Z <= this.text.length ? this.textOff += Z : (this.skip += Z - (this.text.length - this.textOff), this.text = "", this.textOff = 0), this.pos = H), this.openStart < 0 && (this.openStart = G);
    }
    static build(_, H, V, q, G) {
      let J = new ContentBuilder(_, H, V, G);
      return J.openEnd = RangeSet.spans(q, H, V, J), J.openStart < 0 && (J.openStart = J.openEnd), J.finish(J.openEnd), J;
    }
  }
  function wrapMarks(O, _) {
    for (let H of _)
      O = new MarkView(H, [O], O.length);
    return O;
  }
  class NullWidget extends WidgetType {
    constructor(_) {
      super(), this.tag = _;
    }
    eq(_) {
      return _.tag == this.tag;
    }
    toDOM() {
      return document.createElement(this.tag);
    }
    updateDOM(_) {
      return _.nodeName.toLowerCase() == this.tag;
    }
    get isHidden() {
      return !0;
    }
  }
  NullWidget.inline = /* @__PURE__ */ new NullWidget("span");
  NullWidget.block = /* @__PURE__ */ new NullWidget("div");
  var Direction = /* @__PURE__ */ function(O) {
    return O[O.LTR = 0] = "LTR", O[O.RTL = 1] = "RTL", O;
  }(Direction || (Direction = {}));
  const LTR = Direction.LTR, RTL = Direction.RTL;
  function dec(O) {
    let _ = [];
    for (let H = 0; H < O.length; H++)
      _.push(1 << +O[H]);
    return _;
  }
  const LowTypes = /* @__PURE__ */ dec("88888888888888888888888888888888888666888888787833333333337888888000000000000000000000000008888880000000000000000000000000088888888888888888888888888888888888887866668888088888663380888308888800000000000000000000000800000000000000000000000000000008"), ArabicTypes = /* @__PURE__ */ dec("4444448826627288999999999992222222222222222222222222222222222222222222222229999999999999999999994444444444644222822222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222999999949999999229989999223333333333"), Brackets = /* @__PURE__ */ Object.create(null), BracketStack = [];
  for (let O of ["()", "[]", "{}"]) {
    let _ = /* @__PURE__ */ O.charCodeAt(0), H = /* @__PURE__ */ O.charCodeAt(1);
    Brackets[_] = H, Brackets[H] = -_;
  }
  function charType(O) {
    return O <= 247 ? LowTypes[O] : 1424 <= O && O <= 1524 ? 2 : 1536 <= O && O <= 1785 ? ArabicTypes[O - 1536] : 1774 <= O && O <= 2220 ? 4 : 8192 <= O && O <= 8204 ? 256 : 64336 <= O && O <= 65023 ? 4 : 1;
  }
  const BidiRE = /[\u0590-\u05f4\u0600-\u06ff\u0700-\u08ac\ufb50-\ufdff]/;
  class BidiSpan {
    /**
    The direction of this span.
    */
    get dir() {
      return this.level % 2 ? RTL : LTR;
    }
    /**
    @internal
    */
    constructor(_, H, V) {
      this.from = _, this.to = H, this.level = V;
    }
    /**
    @internal
    */
    side(_, H) {
      return this.dir == H == _ ? this.to : this.from;
    }
    /**
    @internal
    */
    forward(_, H) {
      return _ == (this.dir == H);
    }
    /**
    @internal
    */
    static find(_, H, V, q) {
      let G = -1;
      for (let J = 0; J < _.length; J++) {
        let Z = _[J];
        if (Z.from <= H && Z.to >= H) {
          if (Z.level == V)
            return J;
          (G < 0 || (q != 0 ? q < 0 ? Z.from < H : Z.to > H : _[G].level > Z.level)) && (G = J);
        }
      }
      if (G < 0)
        throw new RangeError("Index out of range");
      return G;
    }
  }
  function isolatesEq(O, _) {
    if (O.length != _.length)
      return !1;
    for (let H = 0; H < O.length; H++) {
      let V = O[H], q = _[H];
      if (V.from != q.from || V.to != q.to || V.direction != q.direction || !isolatesEq(V.inner, q.inner))
        return !1;
    }
    return !0;
  }
  const types = [];
  function computeCharTypes(O, _, H, V, q) {
    for (let G = 0; G <= V.length; G++) {
      let J = G ? V[G - 1].to : _, Z = G < V.length ? V[G].from : H, te = G ? 256 : q;
      for (let ne = J, ee = te, ae = te; ne < Z; ne++) {
        let oe = charType(O.charCodeAt(ne));
        oe == 512 ? oe = ee : oe == 8 && ae == 4 && (oe = 16), types[ne] = oe == 4 ? 2 : oe, oe & 7 && (ae = oe), ee = oe;
      }
      for (let ne = J, ee = te, ae = te; ne < Z; ne++) {
        let oe = types[ne];
        if (oe == 128)
          ne < Z - 1 && ee == types[ne + 1] && ee & 24 ? oe = types[ne] = ee : types[ne] = 256;
        else if (oe == 64) {
          let se = ne + 1;
          for (; se < Z && types[se] == 64; )
            se++;
          let ce = ne && ee == 8 || se < H && types[se] == 8 ? ae == 1 ? 1 : 8 : 256;
          for (let le = ne; le < se; le++)
            types[le] = ce;
          ne = se - 1;
        } else oe == 8 && ae == 1 && (types[ne] = 1);
        ee = oe, oe & 7 && (ae = oe);
      }
    }
  }
  function processBracketPairs(O, _, H, V, q) {
    let G = q == 1 ? 2 : 1;
    for (let J = 0, Z = 0, te = 0; J <= V.length; J++) {
      let ne = J ? V[J - 1].to : _, ee = J < V.length ? V[J].from : H;
      for (let ae = ne, oe, se, ce; ae < ee; ae++)
        if (se = Brackets[oe = O.charCodeAt(ae)])
          if (se < 0) {
            for (let le = Z - 3; le >= 0; le -= 3)
              if (BracketStack[le + 1] == -se) {
                let ue = BracketStack[le + 2], de = ue & 2 ? q : ue & 4 ? ue & 1 ? G : q : 0;
                de && (types[ae] = types[BracketStack[le]] = de), Z = le;
                break;
              }
          } else {
            if (BracketStack.length == 189)
              break;
            BracketStack[Z++] = ae, BracketStack[Z++] = oe, BracketStack[Z++] = te;
          }
        else if ((ce = types[ae]) == 2 || ce == 1) {
          let le = ce == q;
          te = le ? 0 : 1;
          for (let ue = Z - 3; ue >= 0; ue -= 3) {
            let de = BracketStack[ue + 2];
            if (de & 2)
              break;
            if (le)
              BracketStack[ue + 2] |= 2;
            else {
              if (de & 4)
                break;
              BracketStack[ue + 2] |= 4;
            }
          }
        }
    }
  }
  function processNeutrals(O, _, H, V) {
    for (let q = 0, G = V; q <= H.length; q++) {
      let J = q ? H[q - 1].to : O, Z = q < H.length ? H[q].from : _;
      for (let te = J; te < Z; ) {
        let ne = types[te];
        if (ne == 256) {
          let ee = te + 1;
          for (; ; )
            if (ee == Z) {
              if (q == H.length)
                break;
              ee = H[q++].to, Z = q < H.length ? H[q].from : _;
            } else if (types[ee] == 256)
              ee++;
            else
              break;
          let ae = G == 1, oe = (ee < _ ? types[ee] : V) == 1, se = ae == oe ? ae ? 1 : 2 : V;
          for (let ce = ee, le = q, ue = le ? H[le - 1].to : O; ce > te; )
            ce == ue && (ce = H[--le].from, ue = le ? H[le - 1].to : O), types[--ce] = se;
          te = ee;
        } else
          G = ne, te++;
      }
    }
  }
  function emitSpans(O, _, H, V, q, G, J) {
    let Z = V % 2 ? 2 : 1;
    if (V % 2 == q % 2)
      for (let te = _, ne = 0; te < H; ) {
        let ee = !0, ae = !1;
        if (ne == G.length || te < G[ne].from) {
          let le = types[te];
          le != Z && (ee = !1, ae = le == 16);
        }
        let oe = !ee && Z == 1 ? [] : null, se = ee ? V : V + 1, ce = te;
        e: for (; ; )
          if (ne < G.length && ce == G[ne].from) {
            if (ae)
              break e;
            let le = G[ne];
            if (!ee)
              for (let ue = le.to, de = ne + 1; ; ) {
                if (ue == H)
                  break e;
                if (de < G.length && G[de].from == ue)
                  ue = G[de++].to;
                else {
                  if (types[ue] == Z)
                    break e;
                  break;
                }
              }
            if (ne++, oe)
              oe.push(le);
            else {
              le.from > te && J.push(new BidiSpan(te, le.from, se));
              let ue = le.direction == LTR != !(se % 2);
              computeSectionOrder(O, ue ? V + 1 : V, q, le.inner, le.from, le.to, J), te = le.to;
            }
            ce = le.to;
          } else {
            if (ce == H || (ee ? types[ce] != Z : types[ce] == Z))
              break;
            ce++;
          }
        oe ? emitSpans(O, te, ce, V + 1, q, oe, J) : te < ce && J.push(new BidiSpan(te, ce, se)), te = ce;
      }
    else
      for (let te = H, ne = G.length; te > _; ) {
        let ee = !0, ae = !1;
        if (!ne || te > G[ne - 1].to) {
          let le = types[te - 1];
          le != Z && (ee = !1, ae = le == 16);
        }
        let oe = !ee && Z == 1 ? [] : null, se = ee ? V : V + 1, ce = te;
        e: for (; ; )
          if (ne && ce == G[ne - 1].to) {
            if (ae)
              break e;
            let le = G[--ne];
            if (!ee)
              for (let ue = le.from, de = ne; ; ) {
                if (ue == _)
                  break e;
                if (de && G[de - 1].to == ue)
                  ue = G[--de].from;
                else {
                  if (types[ue - 1] == Z)
                    break e;
                  break;
                }
              }
            if (oe)
              oe.push(le);
            else {
              le.to < te && J.push(new BidiSpan(le.to, te, se));
              let ue = le.direction == LTR != !(se % 2);
              computeSectionOrder(O, ue ? V + 1 : V, q, le.inner, le.from, le.to, J), te = le.from;
            }
            ce = le.from;
          } else {
            if (ce == _ || (ee ? types[ce - 1] != Z : types[ce - 1] == Z))
              break;
            ce--;
          }
        oe ? emitSpans(O, ce, te, V + 1, q, oe, J) : ce < te && J.push(new BidiSpan(ce, te, se)), te = ce;
      }
  }
  function computeSectionOrder(O, _, H, V, q, G, J) {
    let Z = _ % 2 ? 2 : 1;
    computeCharTypes(O, q, G, V, Z), processBracketPairs(O, q, G, V, Z), processNeutrals(q, G, V, Z), emitSpans(O, q, G, _, H, V, J);
  }
  function computeOrder(O, _, H) {
    if (!O)
      return [new BidiSpan(0, 0, _ == RTL ? 1 : 0)];
    if (_ == LTR && !H.length && !BidiRE.test(O))
      return trivialOrder(O.length);
    if (H.length)
      for (; O.length > types.length; )
        types[types.length] = 256;
    let V = [], q = _ == LTR ? 0 : 1;
    return computeSectionOrder(O, q, q, H, 0, O.length, V), V;
  }
  function trivialOrder(O) {
    return [new BidiSpan(0, O, 0)];
  }
  let movedOver = "";
  function moveVisually(O, _, H, V, q) {
    var G;
    let J = V.head - O.from, Z = BidiSpan.find(_, J, (G = V.bidiLevel) !== null && G !== void 0 ? G : -1, V.assoc), te = _[Z], ne = te.side(q, H);
    if (J == ne) {
      let oe = Z += q ? 1 : -1;
      if (oe < 0 || oe >= _.length)
        return null;
      te = _[Z = oe], J = te.side(!q, H), ne = te.side(q, H);
    }
    let ee = findClusterBreak(O.text, J, te.forward(q, H));
    (ee < te.from || ee > te.to) && (ee = ne), movedOver = O.text.slice(Math.min(J, ee), Math.max(J, ee));
    let ae = Z == (q ? _.length - 1 : 0) ? null : _[Z + (q ? 1 : -1)];
    return ae && ee == ne && ae.level + (q ? 0 : 1) < te.level ? EditorSelection.cursor(ae.side(!q, H) + O.from, ae.forward(q, H) ? 1 : -1, ae.level) : EditorSelection.cursor(ee + O.from, te.forward(q, H) ? -1 : 1, te.level);
  }
  function autoDirection(O, _, H) {
    for (let V = _; V < H; V++) {
      let q = charType(O.charCodeAt(V));
      if (q == 1)
        return LTR;
      if (q == 2 || q == 4)
        return RTL;
    }
    return LTR;
  }
  const clickAddsSelectionRange = /* @__PURE__ */ Facet.define(), dragMovesSelection$1 = /* @__PURE__ */ Facet.define(), mouseSelectionStyle = /* @__PURE__ */ Facet.define(), exceptionSink = /* @__PURE__ */ Facet.define(), updateListener = /* @__PURE__ */ Facet.define(), inputHandler$1 = /* @__PURE__ */ Facet.define(), focusChangeEffect = /* @__PURE__ */ Facet.define(), clipboardInputFilter = /* @__PURE__ */ Facet.define(), clipboardOutputFilter = /* @__PURE__ */ Facet.define(), perLineTextDirection = /* @__PURE__ */ Facet.define({
    combine: (O) => O.some((_) => _)
  }), nativeSelectionHidden = /* @__PURE__ */ Facet.define({
    combine: (O) => O.some((_) => _)
  }), scrollHandler = /* @__PURE__ */ Facet.define();
  class ScrollTarget {
    constructor(_, H = "nearest", V = "nearest", q = 5, G = 5, J = !1) {
      this.range = _, this.y = H, this.x = V, this.yMargin = q, this.xMargin = G, this.isSnapshot = J;
    }
    map(_) {
      return _.empty ? this : new ScrollTarget(this.range.map(_), this.y, this.x, this.yMargin, this.xMargin, this.isSnapshot);
    }
    clip(_) {
      return this.range.to <= _.doc.length ? this : new ScrollTarget(EditorSelection.cursor(_.doc.length), this.y, this.x, this.yMargin, this.xMargin, this.isSnapshot);
    }
  }
  const scrollIntoView$1 = /* @__PURE__ */ StateEffect.define({ map: (O, _) => O.map(_) }), setEditContextFormatting = /* @__PURE__ */ StateEffect.define();
  function logException(O, _, H) {
    let V = O.facet(exceptionSink);
    V.length ? V[0](_) : window.onerror ? window.onerror(String(_), H, void 0, void 0, _) : H ? console.error(H + ":", _) : console.error(_);
  }
  const editable = /* @__PURE__ */ Facet.define({ combine: (O) => O.length ? O[0] : !0 });
  let nextPluginID = 0;
  const viewPlugin = /* @__PURE__ */ Facet.define();
  class ViewPlugin {
    constructor(_, H, V, q, G) {
      this.id = _, this.create = H, this.domEventHandlers = V, this.domEventObservers = q, this.extension = G(this);
    }
    /**
    Define a plugin from a constructor function that creates the
    plugin's value, given an editor view.
    */
    static define(_, H) {
      const { eventHandlers: V, eventObservers: q, provide: G, decorations: J } = H || {};
      return new ViewPlugin(nextPluginID++, _, V, q, (Z) => {
        let te = [viewPlugin.of(Z)];
        return J && te.push(decorations.of((ne) => {
          let ee = ne.plugin(Z);
          return ee ? J(ee) : Decoration.none;
        })), G && te.push(G(Z)), te;
      });
    }
    /**
    Create a plugin for a class whose constructor takes a single
    editor view as argument.
    */
    static fromClass(_, H) {
      return ViewPlugin.define((V) => new _(V), H);
    }
  }
  class PluginInstance {
    constructor(_) {
      this.spec = _, this.mustUpdate = null, this.value = null;
    }
    update(_) {
      if (this.value) {
        if (this.mustUpdate) {
          let H = this.mustUpdate;
          if (this.mustUpdate = null, this.value.update)
            try {
              this.value.update(H);
            } catch (V) {
              if (logException(H.state, V, "CodeMirror plugin crashed"), this.value.destroy)
                try {
                  this.value.destroy();
                } catch {
                }
              this.deactivate();
            }
        }
      } else if (this.spec)
        try {
          this.value = this.spec.create(_);
        } catch (H) {
          logException(_.state, H, "CodeMirror plugin crashed"), this.deactivate();
        }
      return this;
    }
    destroy(_) {
      var H;
      if (!((H = this.value) === null || H === void 0) && H.destroy)
        try {
          this.value.destroy();
        } catch (V) {
          logException(_.state, V, "CodeMirror plugin crashed");
        }
    }
    deactivate() {
      this.spec = this.value = null;
    }
  }
  const editorAttributes = /* @__PURE__ */ Facet.define(), contentAttributes = /* @__PURE__ */ Facet.define(), decorations = /* @__PURE__ */ Facet.define(), outerDecorations = /* @__PURE__ */ Facet.define(), atomicRanges = /* @__PURE__ */ Facet.define(), bidiIsolatedRanges = /* @__PURE__ */ Facet.define();
  function getIsolatedRanges(O, _) {
    let H = O.state.facet(bidiIsolatedRanges);
    if (!H.length)
      return H;
    let V = H.map((G) => G instanceof Function ? G(O) : G), q = [];
    return RangeSet.spans(V, _.from, _.to, {
      point() {
      },
      span(G, J, Z, te) {
        let ne = G - _.from, ee = J - _.from, ae = q;
        for (let oe = Z.length - 1; oe >= 0; oe--, te--) {
          let se = Z[oe].spec.bidiIsolate, ce;
          if (se == null && (se = autoDirection(_.text, ne, ee)), te > 0 && ae.length && (ce = ae[ae.length - 1]).to == ne && ce.direction == se)
            ce.to = ee, ae = ce.inner;
          else {
            let le = { from: ne, to: ee, direction: se, inner: [] };
            ae.push(le), ae = le.inner;
          }
        }
      }
    }), q;
  }
  const scrollMargins = /* @__PURE__ */ Facet.define();
  function getScrollMargins(O) {
    let _ = 0, H = 0, V = 0, q = 0;
    for (let G of O.state.facet(scrollMargins)) {
      let J = G(O);
      J && (J.left != null && (_ = Math.max(_, J.left)), J.right != null && (H = Math.max(H, J.right)), J.top != null && (V = Math.max(V, J.top)), J.bottom != null && (q = Math.max(q, J.bottom)));
    }
    return { left: _, right: H, top: V, bottom: q };
  }
  const styleModule = /* @__PURE__ */ Facet.define();
  class ChangedRange {
    constructor(_, H, V, q) {
      this.fromA = _, this.toA = H, this.fromB = V, this.toB = q;
    }
    join(_) {
      return new ChangedRange(Math.min(this.fromA, _.fromA), Math.max(this.toA, _.toA), Math.min(this.fromB, _.fromB), Math.max(this.toB, _.toB));
    }
    addToSet(_) {
      let H = _.length, V = this;
      for (; H > 0; H--) {
        let q = _[H - 1];
        if (!(q.fromA > V.toA)) {
          if (q.toA < V.fromA)
            break;
          V = V.join(q), _.splice(H - 1, 1);
        }
      }
      return _.splice(H, 0, V), _;
    }
    static extendWithRanges(_, H) {
      if (H.length == 0)
        return _;
      let V = [];
      for (let q = 0, G = 0, J = 0, Z = 0; ; q++) {
        let te = q == _.length ? null : _[q], ne = J - Z, ee = te ? te.fromB : 1e9;
        for (; G < H.length && H[G] < ee; ) {
          let ae = H[G], oe = H[G + 1], se = Math.max(Z, ae), ce = Math.min(ee, oe);
          if (se <= ce && new ChangedRange(se + ne, ce + ne, se, ce).addToSet(V), oe > ee)
            break;
          G += 2;
        }
        if (!te)
          return V;
        new ChangedRange(te.fromA, te.toA, te.fromB, te.toB).addToSet(V), J = te.toA, Z = te.toB;
      }
    }
  }
  class ViewUpdate {
    constructor(_, H, V) {
      this.view = _, this.state = H, this.transactions = V, this.flags = 0, this.startState = _.state, this.changes = ChangeSet.empty(this.startState.doc.length);
      for (let G of V)
        this.changes = this.changes.compose(G.changes);
      let q = [];
      this.changes.iterChangedRanges((G, J, Z, te) => q.push(new ChangedRange(G, J, Z, te))), this.changedRanges = q;
    }
    /**
    @internal
    */
    static create(_, H, V) {
      return new ViewUpdate(_, H, V);
    }
    /**
    Tells you whether the [viewport](https://codemirror.net/6/docs/ref/#view.EditorView.viewport) or
    [visible ranges](https://codemirror.net/6/docs/ref/#view.EditorView.visibleRanges) changed in this
    update.
    */
    get viewportChanged() {
      return (this.flags & 4) > 0;
    }
    /**
    Indicates whether the height of a block element in the editor
    changed in this update.
    */
    get heightChanged() {
      return (this.flags & 2) > 0;
    }
    /**
    Returns true when the document was modified or the size of the
    editor, or elements within the editor, changed.
    */
    get geometryChanged() {
      return this.docChanged || (this.flags & 10) > 0;
    }
    /**
    True when this update indicates a focus change.
    */
    get focusChanged() {
      return (this.flags & 1) > 0;
    }
    /**
    Whether the document changed in this update.
    */
    get docChanged() {
      return !this.changes.empty;
    }
    /**
    Whether the selection was explicitly set in this update.
    */
    get selectionSet() {
      return this.transactions.some((_) => _.selection);
    }
    /**
    @internal
    */
    get empty() {
      return this.flags == 0 && this.transactions.length == 0;
    }
  }
  class DocView extends ContentView {
    get length() {
      return this.view.state.doc.length;
    }
    constructor(_) {
      super(), this.view = _, this.decorations = [], this.dynamicDecorationMap = [!1], this.domChanged = null, this.hasComposition = null, this.markedForComposition = /* @__PURE__ */ new Set(), this.editContextFormatting = Decoration.none, this.lastCompositionAfterCursor = !1, this.minWidth = 0, this.minWidthFrom = 0, this.minWidthTo = 0, this.impreciseAnchor = null, this.impreciseHead = null, this.forceSelection = !1, this.lastUpdate = Date.now(), this.setDOM(_.contentDOM), this.children = [new LineView()], this.children[0].setParent(this), this.updateDeco(), this.updateInner([new ChangedRange(0, 0, 0, _.state.doc.length)], 0, null);
    }
    // Update the document view to a given state.
    update(_) {
      var H;
      let V = _.changedRanges;
      this.minWidth > 0 && V.length && (V.every(({ fromA: ne, toA: ee }) => ee < this.minWidthFrom || ne > this.minWidthTo) ? (this.minWidthFrom = _.changes.mapPos(this.minWidthFrom, 1), this.minWidthTo = _.changes.mapPos(this.minWidthTo, 1)) : this.minWidth = this.minWidthFrom = this.minWidthTo = 0), this.updateEditContextFormatting(_);
      let q = -1;
      this.view.inputState.composing >= 0 && !this.view.observer.editContext && (!((H = this.domChanged) === null || H === void 0) && H.newSel ? q = this.domChanged.newSel.head : !touchesComposition(_.changes, this.hasComposition) && !_.selectionSet && (q = _.state.selection.main.head));
      let G = q > -1 ? findCompositionRange(this.view, _.changes, q) : null;
      if (this.domChanged = null, this.hasComposition) {
        this.markedForComposition.clear();
        let { from: ne, to: ee } = this.hasComposition;
        V = new ChangedRange(ne, ee, _.changes.mapPos(ne, -1), _.changes.mapPos(ee, 1)).addToSet(V.slice());
      }
      this.hasComposition = G ? { from: G.range.fromB, to: G.range.toB } : null, (browser.ie || browser.chrome) && !G && _ && _.state.doc.lines != _.startState.doc.lines && (this.forceSelection = !0);
      let J = this.decorations, Z = this.updateDeco(), te = findChangedDeco(J, Z, _.changes);
      return V = ChangedRange.extendWithRanges(V, te), !(this.flags & 7) && V.length == 0 ? !1 : (this.updateInner(V, _.startState.doc.length, G), _.transactions.length && (this.lastUpdate = Date.now()), !0);
    }
    // Used by update and the constructor do perform the actual DOM
    // update
    updateInner(_, H, V) {
      this.view.viewState.mustMeasureContent = !0, this.updateChildren(_, H, V);
      let { observer: q } = this.view;
      q.ignore(() => {
        this.dom.style.height = this.view.viewState.contentHeight / this.view.scaleY + "px", this.dom.style.flexBasis = this.minWidth ? this.minWidth + "px" : "";
        let J = browser.chrome || browser.ios ? { node: q.selectionRange.focusNode, written: !1 } : void 0;
        this.sync(this.view, J), this.flags &= -8, J && (J.written || q.selectionRange.focusNode != J.node) && (this.forceSelection = !0), this.dom.style.height = "";
      }), this.markedForComposition.forEach(
        (J) => J.flags &= -9
        /* ViewFlag.Composition */
      );
      let G = [];
      if (this.view.viewport.from || this.view.viewport.to < this.view.state.doc.length)
        for (let J of this.children)
          J instanceof BlockWidgetView && J.widget instanceof BlockGapWidget && G.push(J.dom);
      q.updateGaps(G);
    }
    updateChildren(_, H, V) {
      let q = V ? V.range.addToSet(_.slice()) : _, G = this.childCursor(H);
      for (let J = q.length - 1; ; J--) {
        let Z = J >= 0 ? q[J] : null;
        if (!Z)
          break;
        let { fromA: te, toA: ne, fromB: ee, toB: ae } = Z, oe, se, ce, le;
        if (V && V.range.fromB < ae && V.range.toB > ee) {
          let me = ContentBuilder.build(this.view.state.doc, ee, V.range.fromB, this.decorations, this.dynamicDecorationMap), pe = ContentBuilder.build(this.view.state.doc, V.range.toB, ae, this.decorations, this.dynamicDecorationMap);
          se = me.breakAtStart, ce = me.openStart, le = pe.openEnd;
          let ye = this.compositionView(V);
          pe.breakAtStart ? ye.breakAfter = 1 : pe.content.length && ye.merge(ye.length, ye.length, pe.content[0], !1, pe.openStart, 0) && (ye.breakAfter = pe.content[0].breakAfter, pe.content.shift()), me.content.length && ye.merge(0, 0, me.content[me.content.length - 1], !0, 0, me.openEnd) && me.content.pop(), oe = me.content.concat(ye).concat(pe.content);
        } else
          ({ content: oe, breakAtStart: se, openStart: ce, openEnd: le } = ContentBuilder.build(this.view.state.doc, ee, ae, this.decorations, this.dynamicDecorationMap));
        let { i: ue, off: de } = G.findPos(ne, 1), { i: fe, off: he } = G.findPos(te, -1);
        replaceRange(this, fe, he, ue, de, oe, se, ce, le);
      }
      V && this.fixCompositionDOM(V);
    }
    updateEditContextFormatting(_) {
      this.editContextFormatting = this.editContextFormatting.map(_.changes);
      for (let H of _.transactions)
        for (let V of H.effects)
          V.is(setEditContextFormatting) && (this.editContextFormatting = V.value);
    }
    compositionView(_) {
      let H = new TextView(_.text.nodeValue);
      H.flags |= 8;
      for (let { deco: q } of _.marks)
        H = new MarkView(q, [H], H.length);
      let V = new LineView();
      return V.append(H, 0), V;
    }
    fixCompositionDOM(_) {
      let H = (G, J) => {
        J.flags |= 8 | (J.children.some(
          (te) => te.flags & 7
          /* ViewFlag.Dirty */
        ) ? 1 : 0), this.markedForComposition.add(J);
        let Z = ContentView.get(G);
        Z && Z != J && (Z.dom = null), J.setDOM(G);
      }, V = this.childPos(_.range.fromB, 1), q = this.children[V.i];
      H(_.line, q);
      for (let G = _.marks.length - 1; G >= -1; G--)
        V = q.childPos(V.off, 1), q = q.children[V.i], H(G >= 0 ? _.marks[G].node : _.text, q);
    }
    // Sync the DOM selection to this.state.selection
    updateSelection(_ = !1, H = !1) {
      (_ || !this.view.observer.selectionRange.focusNode) && this.view.observer.readSelectionRange();
      let V = this.view.root.activeElement, q = V == this.dom, G = !q && hasSelection(this.dom, this.view.observer.selectionRange) && !(V && this.dom.contains(V));
      if (!(q || H || G))
        return;
      let J = this.forceSelection;
      this.forceSelection = !1;
      let Z = this.view.state.selection.main, te = this.moveToLine(this.domAtPos(Z.anchor)), ne = Z.empty ? te : this.moveToLine(this.domAtPos(Z.head));
      if (browser.gecko && Z.empty && !this.hasComposition && betweenUneditable(te)) {
        let ae = document.createTextNode("");
        this.view.observer.ignore(() => te.node.insertBefore(ae, te.node.childNodes[te.offset] || null)), te = ne = new DOMPos(ae, 0), J = !0;
      }
      let ee = this.view.observer.selectionRange;
      (J || !ee.focusNode || (!isEquivalentPosition(te.node, te.offset, ee.anchorNode, ee.anchorOffset) || !isEquivalentPosition(ne.node, ne.offset, ee.focusNode, ee.focusOffset)) && !this.suppressWidgetCursorChange(ee, Z)) && (this.view.observer.ignore(() => {
        browser.android && browser.chrome && this.dom.contains(ee.focusNode) && inUneditable(ee.focusNode, this.dom) && (this.dom.blur(), this.dom.focus({ preventScroll: !0 }));
        let ae = getSelection(this.view.root);
        if (ae) if (Z.empty) {
          if (browser.gecko) {
            let oe = nextToUneditable(te.node, te.offset);
            if (oe && oe != 3) {
              let se = (oe == 1 ? textNodeBefore : textNodeAfter)(te.node, te.offset);
              se && (te = new DOMPos(se.node, se.offset));
            }
          }
          ae.collapse(te.node, te.offset), Z.bidiLevel != null && ae.caretBidiLevel !== void 0 && (ae.caretBidiLevel = Z.bidiLevel);
        } else if (ae.extend) {
          ae.collapse(te.node, te.offset);
          try {
            ae.extend(ne.node, ne.offset);
          } catch {
          }
        } else {
          let oe = document.createRange();
          Z.anchor > Z.head && ([te, ne] = [ne, te]), oe.setEnd(ne.node, ne.offset), oe.setStart(te.node, te.offset), ae.removeAllRanges(), ae.addRange(oe);
        }
        G && this.view.root.activeElement == this.dom && (this.dom.blur(), V && V.focus());
      }), this.view.observer.setSelectionRange(te, ne)), this.impreciseAnchor = te.precise ? null : new DOMPos(ee.anchorNode, ee.anchorOffset), this.impreciseHead = ne.precise ? null : new DOMPos(ee.focusNode, ee.focusOffset);
    }
    // If a zero-length widget is inserted next to the cursor during
    // composition, avoid moving it across it and disrupting the
    // composition.
    suppressWidgetCursorChange(_, H) {
      return this.hasComposition && H.empty && isEquivalentPosition(_.focusNode, _.focusOffset, _.anchorNode, _.anchorOffset) && this.posFromDOM(_.focusNode, _.focusOffset) == H.head;
    }
    enforceCursorAssoc() {
      if (this.hasComposition)
        return;
      let { view: _ } = this, H = _.state.selection.main, V = getSelection(_.root), { anchorNode: q, anchorOffset: G } = _.observer.selectionRange;
      if (!V || !H.empty || !H.assoc || !V.modify)
        return;
      let J = LineView.find(this, H.head);
      if (!J)
        return;
      let Z = J.posAtStart;
      if (H.head == Z || H.head == Z + J.length)
        return;
      let te = this.coordsAt(H.head, -1), ne = this.coordsAt(H.head, 1);
      if (!te || !ne || te.bottom > ne.top)
        return;
      let ee = this.domAtPos(H.head + H.assoc);
      V.collapse(ee.node, ee.offset), V.modify("move", H.assoc < 0 ? "forward" : "backward", "lineboundary"), _.observer.readSelectionRange();
      let ae = _.observer.selectionRange;
      _.docView.posFromDOM(ae.anchorNode, ae.anchorOffset) != H.from && V.collapse(q, G);
    }
    // If a position is in/near a block widget, move it to a nearby text
    // line, since we don't want the cursor inside a block widget.
    moveToLine(_) {
      let H = this.dom, V;
      if (_.node != H)
        return _;
      for (let q = _.offset; !V && q < H.childNodes.length; q++) {
        let G = ContentView.get(H.childNodes[q]);
        G instanceof LineView && (V = G.domAtPos(0));
      }
      for (let q = _.offset - 1; !V && q >= 0; q--) {
        let G = ContentView.get(H.childNodes[q]);
        G instanceof LineView && (V = G.domAtPos(G.length));
      }
      return V ? new DOMPos(V.node, V.offset, !0) : _;
    }
    nearest(_) {
      for (let H = _; H; ) {
        let V = ContentView.get(H);
        if (V && V.rootView == this)
          return V;
        H = H.parentNode;
      }
      return null;
    }
    posFromDOM(_, H) {
      let V = this.nearest(_);
      if (!V)
        throw new RangeError("Trying to find position for a DOM position outside of the document");
      return V.localPosFromDOM(_, H) + V.posAtStart;
    }
    domAtPos(_) {
      let { i: H, off: V } = this.childCursor().findPos(_, -1);
      for (; H < this.children.length - 1; ) {
        let q = this.children[H];
        if (V < q.length || q instanceof LineView)
          break;
        H++, V = 0;
      }
      return this.children[H].domAtPos(V);
    }
    coordsAt(_, H) {
      let V = null, q = 0;
      for (let G = this.length, J = this.children.length - 1; J >= 0; J--) {
        let Z = this.children[J], te = G - Z.breakAfter, ne = te - Z.length;
        if (te < _)
          break;
        if (ne <= _ && (ne < _ || Z.covers(-1)) && (te > _ || Z.covers(1)) && (!V || Z instanceof LineView && !(V instanceof LineView && H >= 0)))
          V = Z, q = ne;
        else if (V && ne == _ && te == _ && Z instanceof BlockWidgetView && Math.abs(H) < 2) {
          if (Z.deco.startSide < 0)
            break;
          J && (V = null);
        }
        G = ne;
      }
      return V ? V.coordsAt(_ - q, H) : null;
    }
    coordsForChar(_) {
      let { i: H, off: V } = this.childPos(_, 1), q = this.children[H];
      if (!(q instanceof LineView))
        return null;
      for (; q.children.length; ) {
        let { i: Z, off: te } = q.childPos(V, 1);
        for (; ; Z++) {
          if (Z == q.children.length)
            return null;
          if ((q = q.children[Z]).length)
            break;
        }
        V = te;
      }
      if (!(q instanceof TextView))
        return null;
      let G = findClusterBreak(q.text, V);
      if (G == V)
        return null;
      let J = textRange(q.dom, V, G).getClientRects();
      for (let Z = 0; Z < J.length; Z++) {
        let te = J[Z];
        if (Z == J.length - 1 || te.top < te.bottom && te.left < te.right)
          return te;
      }
      return null;
    }
    measureVisibleLineHeights(_) {
      let H = [], { from: V, to: q } = _, G = this.view.contentDOM.clientWidth, J = G > Math.max(this.view.scrollDOM.clientWidth, this.minWidth) + 1, Z = -1, te = this.view.textDirection == Direction.LTR;
      for (let ne = 0, ee = 0; ee < this.children.length; ee++) {
        let ae = this.children[ee], oe = ne + ae.length;
        if (oe > q)
          break;
        if (ne >= V) {
          let se = ae.dom.getBoundingClientRect();
          if (H.push(se.height), J) {
            let ce = ae.dom.lastChild, le = ce ? clientRectsFor(ce) : [];
            if (le.length) {
              let ue = le[le.length - 1], de = te ? ue.right - se.left : se.right - ue.left;
              de > Z && (Z = de, this.minWidth = G, this.minWidthFrom = ne, this.minWidthTo = oe);
            }
          }
        }
        ne = oe + ae.breakAfter;
      }
      return H;
    }
    textDirectionAt(_) {
      let { i: H } = this.childPos(_, 1);
      return getComputedStyle(this.children[H].dom).direction == "rtl" ? Direction.RTL : Direction.LTR;
    }
    measureTextSize() {
      for (let G of this.children)
        if (G instanceof LineView) {
          let J = G.measureTextSize();
          if (J)
            return J;
        }
      let _ = document.createElement("div"), H, V, q;
      return _.className = "cm-line", _.style.width = "99999px", _.style.position = "absolute", _.textContent = "abc def ghi jkl mno pqr stu", this.view.observer.ignore(() => {
        this.dom.appendChild(_);
        let G = clientRectsFor(_.firstChild)[0];
        H = _.getBoundingClientRect().height, V = G ? G.width / 27 : 7, q = G ? G.height : H, _.remove();
      }), { lineHeight: H, charWidth: V, textHeight: q };
    }
    childCursor(_ = this.length) {
      let H = this.children.length;
      return H && (_ -= this.children[--H].length), new ChildCursor(this.children, _, H);
    }
    computeBlockGapDeco() {
      let _ = [], H = this.view.viewState;
      for (let V = 0, q = 0; ; q++) {
        let G = q == H.viewports.length ? null : H.viewports[q], J = G ? G.from - 1 : this.length;
        if (J > V) {
          let Z = (H.lineBlockAt(J).bottom - H.lineBlockAt(V).top) / this.view.scaleY;
          _.push(Decoration.replace({
            widget: new BlockGapWidget(Z),
            block: !0,
            inclusive: !0,
            isBlockGap: !0
          }).range(V, J));
        }
        if (!G)
          break;
        V = G.to + 1;
      }
      return Decoration.set(_);
    }
    updateDeco() {
      let _ = 1, H = this.view.state.facet(decorations).map((G) => (this.dynamicDecorationMap[_++] = typeof G == "function") ? G(this.view) : G), V = !1, q = this.view.state.facet(outerDecorations).map((G, J) => {
        let Z = typeof G == "function";
        return Z && (V = !0), Z ? G(this.view) : G;
      });
      for (q.length && (this.dynamicDecorationMap[_++] = V, H.push(RangeSet.join(q))), this.decorations = [
        this.editContextFormatting,
        ...H,
        this.computeBlockGapDeco(),
        this.view.viewState.lineGapDeco
      ]; _ < this.decorations.length; )
        this.dynamicDecorationMap[_++] = !1;
      return this.decorations;
    }
    scrollIntoView(_) {
      if (_.isSnapshot) {
        let ne = this.view.viewState.lineBlockAt(_.range.head);
        this.view.scrollDOM.scrollTop = ne.top - _.yMargin, this.view.scrollDOM.scrollLeft = _.xMargin;
        return;
      }
      for (let ne of this.view.state.facet(scrollHandler))
        try {
          if (ne(this.view, _.range, _))
            return !0;
        } catch (ee) {
          logException(this.view.state, ee, "scroll handler");
        }
      let { range: H } = _, V = this.coordsAt(H.head, H.empty ? H.assoc : H.head > H.anchor ? -1 : 1), q;
      if (!V)
        return;
      !H.empty && (q = this.coordsAt(H.anchor, H.anchor > H.head ? -1 : 1)) && (V = {
        left: Math.min(V.left, q.left),
        top: Math.min(V.top, q.top),
        right: Math.max(V.right, q.right),
        bottom: Math.max(V.bottom, q.bottom)
      });
      let G = getScrollMargins(this.view), J = {
        left: V.left - G.left,
        top: V.top - G.top,
        right: V.right + G.right,
        bottom: V.bottom + G.bottom
      }, { offsetWidth: Z, offsetHeight: te } = this.view.scrollDOM;
      scrollRectIntoView(this.view.scrollDOM, J, H.head < H.anchor ? -1 : 1, _.x, _.y, Math.max(Math.min(_.xMargin, Z), -Z), Math.max(Math.min(_.yMargin, te), -te), this.view.textDirection == Direction.LTR);
    }
  }
  function betweenUneditable(O) {
    return O.node.nodeType == 1 && O.node.firstChild && (O.offset == 0 || O.node.childNodes[O.offset - 1].contentEditable == "false") && (O.offset == O.node.childNodes.length || O.node.childNodes[O.offset].contentEditable == "false");
  }
  function findCompositionNode(O, _) {
    let H = O.observer.selectionRange;
    if (!H.focusNode)
      return null;
    let V = textNodeBefore(H.focusNode, H.focusOffset), q = textNodeAfter(H.focusNode, H.focusOffset), G = V || q;
    if (q && V && q.node != V.node) {
      let Z = ContentView.get(q.node);
      if (!Z || Z instanceof TextView && Z.text != q.node.nodeValue)
        G = q;
      else if (O.docView.lastCompositionAfterCursor) {
        let te = ContentView.get(V.node);
        !te || te instanceof TextView && te.text != V.node.nodeValue || (G = q);
      }
    }
    if (O.docView.lastCompositionAfterCursor = G != V, !G)
      return null;
    let J = _ - G.offset;
    return { from: J, to: J + G.node.nodeValue.length, node: G.node };
  }
  function findCompositionRange(O, _, H) {
    let V = findCompositionNode(O, H);
    if (!V)
      return null;
    let { node: q, from: G, to: J } = V, Z = q.nodeValue;
    if (/[\n\r]/.test(Z) || O.state.doc.sliceString(V.from, V.to) != Z)
      return null;
    let te = _.invertedDesc, ne = new ChangedRange(te.mapPos(G), te.mapPos(J), G, J), ee = [];
    for (let ae = q.parentNode; ; ae = ae.parentNode) {
      let oe = ContentView.get(ae);
      if (oe instanceof MarkView)
        ee.push({ node: ae, deco: oe.mark });
      else {
        if (oe instanceof LineView || ae.nodeName == "DIV" && ae.parentNode == O.contentDOM)
          return { range: ne, text: q, marks: ee, line: ae };
        if (ae != O.contentDOM)
          ee.push({ node: ae, deco: new MarkDecoration({
            inclusive: !0,
            attributes: getAttrs(ae),
            tagName: ae.tagName.toLowerCase()
          }) });
        else
          return null;
      }
    }
  }
  function nextToUneditable(O, _) {
    return O.nodeType != 1 ? 0 : (_ && O.childNodes[_ - 1].contentEditable == "false" ? 1 : 0) | (_ < O.childNodes.length && O.childNodes[_].contentEditable == "false" ? 2 : 0);
  }
  let DecorationComparator$1 = class {
    constructor() {
      this.changes = [];
    }
    compareRange(_, H) {
      addRange(_, H, this.changes);
    }
    comparePoint(_, H) {
      addRange(_, H, this.changes);
    }
  };
  function findChangedDeco(O, _, H) {
    let V = new DecorationComparator$1();
    return RangeSet.compare(O, _, H, V), V.changes;
  }
  function inUneditable(O, _) {
    for (let H = O; H && H != _; H = H.assignedSlot || H.parentNode)
      if (H.nodeType == 1 && H.contentEditable == "false")
        return !0;
    return !1;
  }
  function touchesComposition(O, _) {
    let H = !1;
    return _ && O.iterChangedRanges((V, q) => {
      V < _.to && q > _.from && (H = !0);
    }), H;
  }
  function groupAt(O, _, H = 1) {
    let V = O.charCategorizer(_), q = O.doc.lineAt(_), G = _ - q.from;
    if (q.length == 0)
      return EditorSelection.cursor(_);
    G == 0 ? H = 1 : G == q.length && (H = -1);
    let J = G, Z = G;
    H < 0 ? J = findClusterBreak(q.text, G, !1) : Z = findClusterBreak(q.text, G);
    let te = V(q.text.slice(J, Z));
    for (; J > 0; ) {
      let ne = findClusterBreak(q.text, J, !1);
      if (V(q.text.slice(ne, J)) != te)
        break;
      J = ne;
    }
    for (; Z < q.length; ) {
      let ne = findClusterBreak(q.text, Z);
      if (V(q.text.slice(Z, ne)) != te)
        break;
      Z = ne;
    }
    return EditorSelection.range(J + q.from, Z + q.from);
  }
  function getdx(O, _) {
    return _.left > O ? _.left - O : Math.max(0, O - _.right);
  }
  function getdy(O, _) {
    return _.top > O ? _.top - O : Math.max(0, O - _.bottom);
  }
  function yOverlap(O, _) {
    return O.top < _.bottom - 1 && O.bottom > _.top + 1;
  }
  function upTop(O, _) {
    return _ < O.top ? { top: _, left: O.left, right: O.right, bottom: O.bottom } : O;
  }
  function upBot(O, _) {
    return _ > O.bottom ? { top: O.top, left: O.left, right: O.right, bottom: _ } : O;
  }
  function domPosAtCoords(O, _, H) {
    let V, q, G, J, Z = !1, te, ne, ee, ae;
    for (let ce = O.firstChild; ce; ce = ce.nextSibling) {
      let le = clientRectsFor(ce);
      for (let ue = 0; ue < le.length; ue++) {
        let de = le[ue];
        q && yOverlap(q, de) && (de = upTop(upBot(de, q.bottom), q.top));
        let fe = getdx(_, de), he = getdy(H, de);
        if (fe == 0 && he == 0)
          return ce.nodeType == 3 ? domPosInText(ce, _, H) : domPosAtCoords(ce, _, H);
        if (!V || J > he || J == he && G > fe) {
          V = ce, q = de, G = fe, J = he;
          let me = he ? H < de.top ? -1 : 1 : fe ? _ < de.left ? -1 : 1 : 0;
          Z = !me || (me > 0 ? ue < le.length - 1 : ue > 0);
        }
        fe == 0 ? H > de.bottom && (!ee || ee.bottom < de.bottom) ? (te = ce, ee = de) : H < de.top && (!ae || ae.top > de.top) && (ne = ce, ae = de) : ee && yOverlap(ee, de) ? ee = upBot(ee, de.bottom) : ae && yOverlap(ae, de) && (ae = upTop(ae, de.top));
      }
    }
    if (ee && ee.bottom >= H ? (V = te, q = ee) : ae && ae.top <= H && (V = ne, q = ae), !V)
      return { node: O, offset: 0 };
    let oe = Math.max(q.left, Math.min(q.right, _));
    if (V.nodeType == 3)
      return domPosInText(V, oe, H);
    if (Z && V.contentEditable != "false")
      return domPosAtCoords(V, oe, H);
    let se = Array.prototype.indexOf.call(O.childNodes, V) + (_ >= (q.left + q.right) / 2 ? 1 : 0);
    return { node: O, offset: se };
  }
  function domPosInText(O, _, H) {
    let V = O.nodeValue.length, q = -1, G = 1e9, J = 0;
    for (let Z = 0; Z < V; Z++) {
      let te = textRange(O, Z, Z + 1).getClientRects();
      for (let ne = 0; ne < te.length; ne++) {
        let ee = te[ne];
        if (ee.top == ee.bottom)
          continue;
        J || (J = _ - ee.left);
        let ae = (ee.top > H ? ee.top - H : H - ee.bottom) - 1;
        if (ee.left - 1 <= _ && ee.right + 1 >= _ && ae < G) {
          let oe = _ >= (ee.left + ee.right) / 2, se = oe;
          if ((browser.chrome || browser.gecko) && textRange(O, Z).getBoundingClientRect().left == ee.right && (se = !oe), ae <= 0)
            return { node: O, offset: Z + (se ? 1 : 0) };
          q = Z + (se ? 1 : 0), G = ae;
        }
      }
    }
    return { node: O, offset: q > -1 ? q : J > 0 ? O.nodeValue.length : 0 };
  }
  function posAtCoords(O, _, H, V = -1) {
    var q, G;
    let J = O.contentDOM.getBoundingClientRect(), Z = J.top + O.viewState.paddingTop, te, { docHeight: ne } = O.viewState, { x: ee, y: ae } = _, oe = ae - Z;
    if (oe < 0)
      return 0;
    if (oe > ne)
      return O.state.doc.length;
    for (let me = O.viewState.heightOracle.textHeight / 2, pe = !1; te = O.elementAtHeight(oe), te.type != BlockType.Text; )
      for (; oe = V > 0 ? te.bottom + me : te.top - me, !(oe >= 0 && oe <= ne); ) {
        if (pe)
          return H ? null : 0;
        pe = !0, V = -V;
      }
    ae = Z + oe;
    let se = te.from;
    if (se < O.viewport.from)
      return O.viewport.from == 0 ? 0 : H ? null : posAtCoordsImprecise(O, J, te, ee, ae);
    if (se > O.viewport.to)
      return O.viewport.to == O.state.doc.length ? O.state.doc.length : H ? null : posAtCoordsImprecise(O, J, te, ee, ae);
    let ce = O.dom.ownerDocument, le = O.root.elementFromPoint ? O.root : ce, ue = le.elementFromPoint(ee, ae);
    ue && !O.contentDOM.contains(ue) && (ue = null), ue || (ee = Math.max(J.left + 1, Math.min(J.right - 1, ee)), ue = le.elementFromPoint(ee, ae), ue && !O.contentDOM.contains(ue) && (ue = null));
    let de, fe = -1;
    if (ue && ((q = O.docView.nearest(ue)) === null || q === void 0 ? void 0 : q.isEditable) != !1) {
      if (ce.caretPositionFromPoint) {
        let me = ce.caretPositionFromPoint(ee, ae);
        me && ({ offsetNode: de, offset: fe } = me);
      } else if (ce.caretRangeFromPoint) {
        let me = ce.caretRangeFromPoint(ee, ae);
        me && ({ startContainer: de, startOffset: fe } = me, (!O.contentDOM.contains(de) || browser.safari && isSuspiciousSafariCaretResult(de, fe, ee) || browser.chrome && isSuspiciousChromeCaretResult(de, fe, ee)) && (de = void 0));
      }
      de && (fe = Math.min(maxOffset(de), fe));
    }
    if (!de || !O.docView.dom.contains(de)) {
      let me = LineView.find(O.docView, se);
      if (!me)
        return oe > te.top + te.height / 2 ? te.to : te.from;
      ({ node: de, offset: fe } = domPosAtCoords(me.dom, ee, ae));
    }
    let he = O.docView.nearest(de);
    if (!he)
      return null;
    if (he.isWidget && ((G = he.dom) === null || G === void 0 ? void 0 : G.nodeType) == 1) {
      let me = he.dom.getBoundingClientRect();
      return _.y < me.top || _.y <= me.bottom && _.x <= (me.left + me.right) / 2 ? he.posAtStart : he.posAtEnd;
    } else
      return he.localPosFromDOM(de, fe) + he.posAtStart;
  }
  function posAtCoordsImprecise(O, _, H, V, q) {
    let G = Math.round((V - _.left) * O.defaultCharacterWidth);
    if (O.lineWrapping && H.height > O.defaultLineHeight * 1.5) {
      let Z = O.viewState.heightOracle.textHeight, te = Math.floor((q - H.top - (O.defaultLineHeight - Z) * 0.5) / Z);
      G += te * O.viewState.heightOracle.lineLength;
    }
    let J = O.state.sliceDoc(H.from, H.to);
    return H.from + findColumn(J, G, O.state.tabSize);
  }
  function isSuspiciousSafariCaretResult(O, _, H) {
    let V;
    if (O.nodeType != 3 || _ != (V = O.nodeValue.length))
      return !1;
    for (let q = O.nextSibling; q; q = q.nextSibling)
      if (q.nodeType != 1 || q.nodeName != "BR")
        return !1;
    return textRange(O, V - 1, V).getBoundingClientRect().left > H;
  }
  function isSuspiciousChromeCaretResult(O, _, H) {
    if (_ != 0)
      return !1;
    for (let q = O; ; ) {
      let G = q.parentNode;
      if (!G || G.nodeType != 1 || G.firstChild != q)
        return !1;
      if (G.classList.contains("cm-line"))
        break;
      q = G;
    }
    let V = O.nodeType == 1 ? O.getBoundingClientRect() : textRange(O, 0, Math.max(O.nodeValue.length, 1)).getBoundingClientRect();
    return H - V.left > 5;
  }
  function blockAt(O, _) {
    let H = O.lineBlockAt(_);
    if (Array.isArray(H.type)) {
      for (let V of H.type)
        if (V.to > _ || V.to == _ && (V.to == H.to || V.type == BlockType.Text))
          return V;
    }
    return H;
  }
  function moveToLineBoundary(O, _, H, V) {
    let q = blockAt(O, _.head), G = !V || q.type != BlockType.Text || !(O.lineWrapping || q.widgetLineBreaks) ? null : O.coordsAtPos(_.assoc < 0 && _.head > q.from ? _.head - 1 : _.head);
    if (G) {
      let J = O.dom.getBoundingClientRect(), Z = O.textDirectionAt(q.from), te = O.posAtCoords({
        x: H == (Z == Direction.LTR) ? J.right - 1 : J.left + 1,
        y: (G.top + G.bottom) / 2
      });
      if (te != null)
        return EditorSelection.cursor(te, H ? -1 : 1);
    }
    return EditorSelection.cursor(H ? q.to : q.from, H ? -1 : 1);
  }
  function moveByChar(O, _, H, V) {
    let q = O.state.doc.lineAt(_.head), G = O.bidiSpans(q), J = O.textDirectionAt(q.from);
    for (let Z = _, te = null; ; ) {
      let ne = moveVisually(q, G, J, Z, H), ee = movedOver;
      if (!ne) {
        if (q.number == (H ? O.state.doc.lines : 1))
          return Z;
        ee = `
`, q = O.state.doc.line(q.number + (H ? 1 : -1)), G = O.bidiSpans(q), ne = O.visualLineSide(q, !H);
      }
      if (te) {
        if (!te(ee))
          return Z;
      } else {
        if (!V)
          return ne;
        te = V(ee);
      }
      Z = ne;
    }
  }
  function byGroup(O, _, H) {
    let V = O.state.charCategorizer(_), q = V(H);
    return (G) => {
      let J = V(G);
      return q == CharCategory.Space && (q = J), q == J;
    };
  }
  function moveVertically(O, _, H, V) {
    let q = _.head, G = H ? 1 : -1;
    if (q == (H ? O.state.doc.length : 0))
      return EditorSelection.cursor(q, _.assoc);
    let J = _.goalColumn, Z, te = O.contentDOM.getBoundingClientRect(), ne = O.coordsAtPos(q, _.assoc || -1), ee = O.documentTop;
    if (ne)
      J == null && (J = ne.left - te.left), Z = G < 0 ? ne.top : ne.bottom;
    else {
      let se = O.viewState.lineBlockAt(q);
      J == null && (J = Math.min(te.right - te.left, O.defaultCharacterWidth * (q - se.from))), Z = (G < 0 ? se.top : se.bottom) + ee;
    }
    let ae = te.left + J, oe = V ?? O.viewState.heightOracle.textHeight >> 1;
    for (let se = 0; ; se += 10) {
      let ce = Z + (oe + se) * G, le = posAtCoords(O, { x: ae, y: ce }, !1, G);
      if (ce < te.top || ce > te.bottom || (G < 0 ? le < q : le > q)) {
        let ue = O.docView.coordsForChar(le), de = !ue || ce < ue.top ? -1 : 1;
        return EditorSelection.cursor(le, de, void 0, J);
      }
    }
  }
  function skipAtomicRanges(O, _, H) {
    for (; ; ) {
      let V = 0;
      for (let q of O)
        q.between(_ - 1, _ + 1, (G, J, Z) => {
          if (_ > G && _ < J) {
            let te = V || H || (_ - G < J - _ ? -1 : 1);
            _ = te < 0 ? G : J, V = te;
          }
        });
      if (!V)
        return _;
    }
  }
  function skipAtoms(O, _, H) {
    let V = skipAtomicRanges(O.state.facet(atomicRanges).map((q) => q(O)), H.from, _.head > H.from ? -1 : 1);
    return V == H.from ? H : EditorSelection.cursor(V, V < H.from ? 1 : -1);
  }
  const LineBreakPlaceholder = "";
  class DOMReader {
    constructor(_, H) {
      this.points = _, this.text = "", this.lineSeparator = H.facet(EditorState.lineSeparator);
    }
    append(_) {
      this.text += _;
    }
    lineBreak() {
      this.text += LineBreakPlaceholder;
    }
    readRange(_, H) {
      if (!_)
        return this;
      let V = _.parentNode;
      for (let q = _; ; ) {
        this.findPointBefore(V, q);
        let G = this.text.length;
        this.readNode(q);
        let J = q.nextSibling;
        if (J == H)
          break;
        let Z = ContentView.get(q), te = ContentView.get(J);
        (Z && te ? Z.breakAfter : (Z ? Z.breakAfter : isBlockElement(q)) || isBlockElement(J) && (q.nodeName != "BR" || q.cmIgnore) && this.text.length > G) && this.lineBreak(), q = J;
      }
      return this.findPointBefore(V, H), this;
    }
    readTextNode(_) {
      let H = _.nodeValue;
      for (let V of this.points)
        V.node == _ && (V.pos = this.text.length + Math.min(V.offset, H.length));
      for (let V = 0, q = this.lineSeparator ? null : /\r\n?|\n/g; ; ) {
        let G = -1, J = 1, Z;
        if (this.lineSeparator ? (G = H.indexOf(this.lineSeparator, V), J = this.lineSeparator.length) : (Z = q.exec(H)) && (G = Z.index, J = Z[0].length), this.append(H.slice(V, G < 0 ? H.length : G)), G < 0)
          break;
        if (this.lineBreak(), J > 1)
          for (let te of this.points)
            te.node == _ && te.pos > this.text.length && (te.pos -= J - 1);
        V = G + J;
      }
    }
    readNode(_) {
      if (_.cmIgnore)
        return;
      let H = ContentView.get(_), V = H && H.overrideDOMText;
      if (V != null) {
        this.findPointInside(_, V.length);
        for (let q = V.iter(); !q.next().done; )
          q.lineBreak ? this.lineBreak() : this.append(q.value);
      } else _.nodeType == 3 ? this.readTextNode(_) : _.nodeName == "BR" ? _.nextSibling && this.lineBreak() : _.nodeType == 1 && this.readRange(_.firstChild, null);
    }
    findPointBefore(_, H) {
      for (let V of this.points)
        V.node == _ && _.childNodes[V.offset] == H && (V.pos = this.text.length);
    }
    findPointInside(_, H) {
      for (let V of this.points)
        (_.nodeType == 3 ? V.node == _ : _.contains(V.node)) && (V.pos = this.text.length + (isAtEnd(_, V.node, V.offset) ? H : 0));
    }
  }
  function isAtEnd(O, _, H) {
    for (; ; ) {
      if (!_ || H < maxOffset(_))
        return !1;
      if (_ == O)
        return !0;
      H = domIndex(_) + 1, _ = _.parentNode;
    }
  }
  let DOMPoint$1 = class {
    constructor(_, H) {
      this.node = _, this.offset = H, this.pos = -1;
    }
  };
  class DOMChange {
    constructor(_, H, V, q) {
      this.typeOver = q, this.bounds = null, this.text = "", this.domChanged = H > -1;
      let { impreciseHead: G, impreciseAnchor: J } = _.docView;
      if (_.state.readOnly && H > -1)
        this.newSel = null;
      else if (H > -1 && (this.bounds = _.docView.domBoundsAround(H, V, 0))) {
        let Z = G || J ? [] : selectionPoints(_), te = new DOMReader(Z, _.state);
        te.readRange(this.bounds.startDOM, this.bounds.endDOM), this.text = te.text, this.newSel = selectionFromPoints(Z, this.bounds.from);
      } else {
        let Z = _.observer.selectionRange, te = G && G.node == Z.focusNode && G.offset == Z.focusOffset || !contains(_.contentDOM, Z.focusNode) ? _.state.selection.main.head : _.docView.posFromDOM(Z.focusNode, Z.focusOffset), ne = J && J.node == Z.anchorNode && J.offset == Z.anchorOffset || !contains(_.contentDOM, Z.anchorNode) ? _.state.selection.main.anchor : _.docView.posFromDOM(Z.anchorNode, Z.anchorOffset), ee = _.viewport;
        if ((browser.ios || browser.chrome) && _.state.selection.main.empty && te != ne && (ee.from > 0 || ee.to < _.state.doc.length)) {
          let ae = Math.min(te, ne), oe = Math.max(te, ne), se = ee.from - ae, ce = ee.to - oe;
          (se == 0 || se == 1 || ae == 0) && (ce == 0 || ce == -1 || oe == _.state.doc.length) && (te = 0, ne = _.state.doc.length);
        }
        this.newSel = EditorSelection.single(ne, te);
      }
    }
  }
  function applyDOMChange(O, _) {
    let H, { newSel: V } = _, q = O.state.selection.main, G = O.inputState.lastKeyTime > Date.now() - 100 ? O.inputState.lastKeyCode : -1;
    if (_.bounds) {
      let { from: J, to: Z } = _.bounds, te = q.from, ne = null;
      (G === 8 || browser.android && _.text.length < Z - J) && (te = q.to, ne = "end");
      let ee = findDiff(O.state.doc.sliceString(J, Z, LineBreakPlaceholder), _.text, te - J, ne);
      ee && (browser.chrome && G == 13 && ee.toB == ee.from + 2 && _.text.slice(ee.from, ee.toB) == LineBreakPlaceholder + LineBreakPlaceholder && ee.toB--, H = {
        from: J + ee.from,
        to: J + ee.toA,
        insert: Text.of(_.text.slice(ee.from, ee.toB).split(LineBreakPlaceholder))
      });
    } else V && (!O.hasFocus && O.state.facet(editable) || V.main.eq(q)) && (V = null);
    if (!H && !V)
      return !1;
    if (!H && _.typeOver && !q.empty && V && V.main.empty ? H = { from: q.from, to: q.to, insert: O.state.doc.slice(q.from, q.to) } : H && H.from >= q.from && H.to <= q.to && (H.from != q.from || H.to != q.to) && q.to - q.from - (H.to - H.from) <= 4 ? H = {
      from: q.from,
      to: q.to,
      insert: O.state.doc.slice(q.from, H.from).append(H.insert).append(O.state.doc.slice(H.to, q.to))
    } : (browser.mac || browser.android) && H && H.from == H.to && H.from == q.head - 1 && /^\. ?$/.test(H.insert.toString()) && O.contentDOM.getAttribute("autocorrect") == "off" ? (V && H.insert.length == 2 && (V = EditorSelection.single(V.main.anchor - 1, V.main.head - 1)), H = { from: q.from, to: q.to, insert: Text.of([" "]) }) : browser.chrome && H && H.from == H.to && H.from == q.head && H.insert.toString() == `
 ` && O.lineWrapping && (V && (V = EditorSelection.single(V.main.anchor - 1, V.main.head - 1)), H = { from: q.from, to: q.to, insert: Text.of([" "]) }), H)
      return applyDOMChangeInner(O, H, V, G);
    if (V && !V.main.eq(q)) {
      let J = !1, Z = "select";
      return O.inputState.lastSelectionTime > Date.now() - 50 && (O.inputState.lastSelectionOrigin == "select" && (J = !0), Z = O.inputState.lastSelectionOrigin), O.dispatch({ selection: V, scrollIntoView: J, userEvent: Z }), !0;
    } else
      return !1;
  }
  function applyDOMChangeInner(O, _, H, V = -1) {
    if (browser.ios && O.inputState.flushIOSKey(_))
      return !0;
    let q = O.state.selection.main;
    if (browser.android && (_.to == q.to && // GBoard will sometimes remove a space it just inserted
    // after a completion when you press enter
    (_.from == q.from || _.from == q.from - 1 && O.state.sliceDoc(_.from, q.from) == " ") && _.insert.length == 1 && _.insert.lines == 2 && dispatchKey(O.contentDOM, "Enter", 13) || (_.from == q.from - 1 && _.to == q.to && _.insert.length == 0 || V == 8 && _.insert.length < _.to - _.from && _.to > q.head) && dispatchKey(O.contentDOM, "Backspace", 8) || _.from == q.from && _.to == q.to + 1 && _.insert.length == 0 && dispatchKey(O.contentDOM, "Delete", 46)))
      return !0;
    let G = _.insert.toString();
    O.inputState.composing >= 0 && O.inputState.composing++;
    let J, Z = () => J || (J = applyDefaultInsert(O, _, H));
    return O.state.facet(inputHandler$1).some((te) => te(O, _.from, _.to, G, Z)) || O.dispatch(Z()), !0;
  }
  function applyDefaultInsert(O, _, H) {
    let V, q = O.state, G = q.selection.main;
    if (_.from >= G.from && _.to <= G.to && _.to - _.from >= (G.to - G.from) / 3 && (!H || H.main.empty && H.main.from == _.from + _.insert.length) && O.inputState.composing < 0) {
      let Z = G.from < _.from ? q.sliceDoc(G.from, _.from) : "", te = G.to > _.to ? q.sliceDoc(_.to, G.to) : "";
      V = q.replaceSelection(O.state.toText(Z + _.insert.sliceString(0, void 0, O.state.lineBreak) + te));
    } else {
      let Z = q.changes(_), te = H && H.main.to <= Z.newLength ? H.main : void 0;
      if (q.selection.ranges.length > 1 && O.inputState.composing >= 0 && _.to <= G.to && _.to >= G.to - 10) {
        let ne = O.state.sliceDoc(_.from, _.to), ee, ae = H && findCompositionNode(O, H.main.head);
        if (ae) {
          let ce = _.insert.length - (_.to - _.from);
          ee = { from: ae.from, to: ae.to - ce };
        } else
          ee = O.state.doc.lineAt(G.head);
        let oe = G.to - _.to, se = G.to - G.from;
        V = q.changeByRange((ce) => {
          if (ce.from == G.from && ce.to == G.to)
            return { changes: Z, range: te || ce.map(Z) };
          let le = ce.to - oe, ue = le - ne.length;
          if (ce.to - ce.from != se || O.state.sliceDoc(ue, le) != ne || // Unfortunately, there's no way to make multiple
          // changes in the same node work without aborting
          // composition, so cursors in the composition range are
          // ignored.
          ce.to >= ee.from && ce.from <= ee.to)
            return { range: ce };
          let de = q.changes({ from: ue, to: le, insert: _.insert }), fe = ce.to - G.to;
          return {
            changes: de,
            range: te ? EditorSelection.range(Math.max(0, te.anchor + fe), Math.max(0, te.head + fe)) : ce.map(de)
          };
        });
      } else
        V = {
          changes: Z,
          selection: te && q.selection.replaceRange(te)
        };
    }
    let J = "input.type";
    return (O.composing || O.inputState.compositionPendingChange && O.inputState.compositionEndedAt > Date.now() - 50) && (O.inputState.compositionPendingChange = !1, J += ".compose", O.inputState.compositionFirstChange && (J += ".start", O.inputState.compositionFirstChange = !1)), q.update(V, { userEvent: J, scrollIntoView: !0 });
  }
  function findDiff(O, _, H, V) {
    let q = Math.min(O.length, _.length), G = 0;
    for (; G < q && O.charCodeAt(G) == _.charCodeAt(G); )
      G++;
    if (G == q && O.length == _.length)
      return null;
    let J = O.length, Z = _.length;
    for (; J > 0 && Z > 0 && O.charCodeAt(J - 1) == _.charCodeAt(Z - 1); )
      J--, Z--;
    if (V == "end") {
      let te = Math.max(0, G - Math.min(J, Z));
      H -= J + te - G;
    }
    if (J < G && O.length < _.length) {
      let te = H <= G && H >= J ? G - H : 0;
      G -= te, Z = G + (Z - J), J = G;
    } else if (Z < G) {
      let te = H <= G && H >= Z ? G - H : 0;
      G -= te, J = G + (J - Z), Z = G;
    }
    return { from: G, toA: J, toB: Z };
  }
  function selectionPoints(O) {
    let _ = [];
    if (O.root.activeElement != O.contentDOM)
      return _;
    let { anchorNode: H, anchorOffset: V, focusNode: q, focusOffset: G } = O.observer.selectionRange;
    return H && (_.push(new DOMPoint$1(H, V)), (q != H || G != V) && _.push(new DOMPoint$1(q, G))), _;
  }
  function selectionFromPoints(O, _) {
    if (O.length == 0)
      return null;
    let H = O[0].pos, V = O.length == 2 ? O[1].pos : H;
    return H > -1 && V > -1 ? EditorSelection.single(H + _, V + _) : null;
  }
  class InputState {
    setSelectionOrigin(_) {
      this.lastSelectionOrigin = _, this.lastSelectionTime = Date.now();
    }
    constructor(_) {
      this.view = _, this.lastKeyCode = 0, this.lastKeyTime = 0, this.lastTouchTime = 0, this.lastFocusTime = 0, this.lastScrollTop = 0, this.lastScrollLeft = 0, this.pendingIOSKey = void 0, this.tabFocusMode = -1, this.lastSelectionOrigin = null, this.lastSelectionTime = 0, this.lastContextMenu = 0, this.scrollHandlers = [], this.handlers = /* @__PURE__ */ Object.create(null), this.composing = -1, this.compositionFirstChange = null, this.compositionEndedAt = 0, this.compositionPendingKey = !1, this.compositionPendingChange = !1, this.mouseSelection = null, this.draggedContent = null, this.handleEvent = this.handleEvent.bind(this), this.notifiedFocused = _.hasFocus, browser.safari && _.contentDOM.addEventListener("input", () => null), browser.gecko && firefoxCopyCutHack(_.contentDOM.ownerDocument);
    }
    handleEvent(_) {
      !eventBelongsToEditor(this.view, _) || this.ignoreDuringComposition(_) || _.type == "keydown" && this.keydown(_) || this.runHandlers(_.type, _);
    }
    runHandlers(_, H) {
      let V = this.handlers[_];
      if (V) {
        for (let q of V.observers)
          q(this.view, H);
        for (let q of V.handlers) {
          if (H.defaultPrevented)
            break;
          if (q(this.view, H)) {
            H.preventDefault();
            break;
          }
        }
      }
    }
    ensureHandlers(_) {
      let H = computeHandlers(_), V = this.handlers, q = this.view.contentDOM;
      for (let G in H)
        if (G != "scroll") {
          let J = !H[G].handlers.length, Z = V[G];
          Z && J != !Z.handlers.length && (q.removeEventListener(G, this.handleEvent), Z = null), Z || q.addEventListener(G, this.handleEvent, { passive: J });
        }
      for (let G in V)
        G != "scroll" && !H[G] && q.removeEventListener(G, this.handleEvent);
      this.handlers = H;
    }
    keydown(_) {
      if (this.lastKeyCode = _.keyCode, this.lastKeyTime = Date.now(), _.keyCode == 9 && this.tabFocusMode > -1 && (!this.tabFocusMode || Date.now() <= this.tabFocusMode))
        return !0;
      if (this.tabFocusMode > 0 && _.keyCode != 27 && modifierCodes.indexOf(_.keyCode) < 0 && (this.tabFocusMode = -1), browser.android && browser.chrome && !_.synthetic && (_.keyCode == 13 || _.keyCode == 8))
        return this.view.observer.delayAndroidKey(_.key, _.keyCode), !0;
      let H;
      return browser.ios && !_.synthetic && !_.altKey && !_.metaKey && ((H = PendingKeys.find((V) => V.keyCode == _.keyCode)) && !_.ctrlKey || EmacsyPendingKeys.indexOf(_.key) > -1 && _.ctrlKey && !_.shiftKey) ? (this.pendingIOSKey = H || _, setTimeout(() => this.flushIOSKey(), 250), !0) : (_.keyCode != 229 && this.view.observer.forceFlush(), !1);
    }
    flushIOSKey(_) {
      let H = this.pendingIOSKey;
      return !H || H.key == "Enter" && _ && _.from < _.to && /^\S+$/.test(_.insert.toString()) ? !1 : (this.pendingIOSKey = void 0, dispatchKey(this.view.contentDOM, H.key, H.keyCode, H instanceof KeyboardEvent ? H : void 0));
    }
    ignoreDuringComposition(_) {
      return /^key/.test(_.type) ? this.composing > 0 ? !0 : browser.safari && !browser.ios && this.compositionPendingKey && Date.now() - this.compositionEndedAt < 100 ? (this.compositionPendingKey = !1, !0) : !1 : !1;
    }
    startMouseSelection(_) {
      this.mouseSelection && this.mouseSelection.destroy(), this.mouseSelection = _;
    }
    update(_) {
      this.view.observer.update(_), this.mouseSelection && this.mouseSelection.update(_), this.draggedContent && _.docChanged && (this.draggedContent = this.draggedContent.map(_.changes)), _.transactions.length && (this.lastKeyCode = this.lastSelectionTime = 0);
    }
    destroy() {
      this.mouseSelection && this.mouseSelection.destroy();
    }
  }
  function bindHandler(O, _) {
    return (H, V) => {
      try {
        return _.call(O, V, H);
      } catch (q) {
        logException(H.state, q);
      }
    };
  }
  function computeHandlers(O) {
    let _ = /* @__PURE__ */ Object.create(null);
    function H(V) {
      return _[V] || (_[V] = { observers: [], handlers: [] });
    }
    for (let V of O) {
      let q = V.spec;
      if (q && q.domEventHandlers)
        for (let G in q.domEventHandlers) {
          let J = q.domEventHandlers[G];
          J && H(G).handlers.push(bindHandler(V.value, J));
        }
      if (q && q.domEventObservers)
        for (let G in q.domEventObservers) {
          let J = q.domEventObservers[G];
          J && H(G).observers.push(bindHandler(V.value, J));
        }
    }
    for (let V in handlers)
      H(V).handlers.push(handlers[V]);
    for (let V in observers)
      H(V).observers.push(observers[V]);
    return _;
  }
  const PendingKeys = [
    { key: "Backspace", keyCode: 8, inputType: "deleteContentBackward" },
    { key: "Enter", keyCode: 13, inputType: "insertParagraph" },
    { key: "Enter", keyCode: 13, inputType: "insertLineBreak" },
    { key: "Delete", keyCode: 46, inputType: "deleteContentForward" }
  ], EmacsyPendingKeys = "dthko", modifierCodes = [16, 17, 18, 20, 91, 92, 224, 225], dragScrollMargin = 6;
  function dragScrollSpeed(O) {
    return Math.max(0, O) * 0.7 + 8;
  }
  function dist(O, _) {
    return Math.max(Math.abs(O.clientX - _.clientX), Math.abs(O.clientY - _.clientY));
  }
  class MouseSelection {
    constructor(_, H, V, q) {
      this.view = _, this.startEvent = H, this.style = V, this.mustSelect = q, this.scrollSpeed = { x: 0, y: 0 }, this.scrolling = -1, this.lastEvent = H, this.scrollParents = scrollableParents(_.contentDOM), this.atoms = _.state.facet(atomicRanges).map((J) => J(_));
      let G = _.contentDOM.ownerDocument;
      G.addEventListener("mousemove", this.move = this.move.bind(this)), G.addEventListener("mouseup", this.up = this.up.bind(this)), this.extend = H.shiftKey, this.multiple = _.state.facet(EditorState.allowMultipleSelections) && addsSelectionRange(_, H), this.dragging = isInPrimarySelection(_, H) && getClickType(H) == 1 ? null : !1;
    }
    start(_) {
      this.dragging === !1 && this.select(_);
    }
    move(_) {
      if (_.buttons == 0)
        return this.destroy();
      if (this.dragging || this.dragging == null && dist(this.startEvent, _) < 10)
        return;
      this.select(this.lastEvent = _);
      let H = 0, V = 0, q = 0, G = 0, J = this.view.win.innerWidth, Z = this.view.win.innerHeight;
      this.scrollParents.x && ({ left: q, right: J } = this.scrollParents.x.getBoundingClientRect()), this.scrollParents.y && ({ top: G, bottom: Z } = this.scrollParents.y.getBoundingClientRect());
      let te = getScrollMargins(this.view);
      _.clientX - te.left <= q + dragScrollMargin ? H = -dragScrollSpeed(q - _.clientX) : _.clientX + te.right >= J - dragScrollMargin && (H = dragScrollSpeed(_.clientX - J)), _.clientY - te.top <= G + dragScrollMargin ? V = -dragScrollSpeed(G - _.clientY) : _.clientY + te.bottom >= Z - dragScrollMargin && (V = dragScrollSpeed(_.clientY - Z)), this.setScrollSpeed(H, V);
    }
    up(_) {
      this.dragging == null && this.select(this.lastEvent), this.dragging || _.preventDefault(), this.destroy();
    }
    destroy() {
      this.setScrollSpeed(0, 0);
      let _ = this.view.contentDOM.ownerDocument;
      _.removeEventListener("mousemove", this.move), _.removeEventListener("mouseup", this.up), this.view.inputState.mouseSelection = this.view.inputState.draggedContent = null;
    }
    setScrollSpeed(_, H) {
      this.scrollSpeed = { x: _, y: H }, _ || H ? this.scrolling < 0 && (this.scrolling = setInterval(() => this.scroll(), 50)) : this.scrolling > -1 && (clearInterval(this.scrolling), this.scrolling = -1);
    }
    scroll() {
      let { x: _, y: H } = this.scrollSpeed;
      _ && this.scrollParents.x && (this.scrollParents.x.scrollLeft += _, _ = 0), H && this.scrollParents.y && (this.scrollParents.y.scrollTop += H, H = 0), (_ || H) && this.view.win.scrollBy(_, H), this.dragging === !1 && this.select(this.lastEvent);
    }
    skipAtoms(_) {
      let H = null;
      for (let V = 0; V < _.ranges.length; V++) {
        let q = _.ranges[V], G = null;
        if (q.empty) {
          let J = skipAtomicRanges(this.atoms, q.from, 0);
          J != q.from && (G = EditorSelection.cursor(J, -1));
        } else {
          let J = skipAtomicRanges(this.atoms, q.from, -1), Z = skipAtomicRanges(this.atoms, q.to, 1);
          (J != q.from || Z != q.to) && (G = EditorSelection.range(q.from == q.anchor ? J : Z, q.from == q.head ? J : Z));
        }
        G && (H || (H = _.ranges.slice()), H[V] = G);
      }
      return H ? EditorSelection.create(H, _.mainIndex) : _;
    }
    select(_) {
      let { view: H } = this, V = this.skipAtoms(this.style.get(_, this.extend, this.multiple));
      (this.mustSelect || !V.eq(H.state.selection, this.dragging === !1)) && this.view.dispatch({
        selection: V,
        userEvent: "select.pointer"
      }), this.mustSelect = !1;
    }
    update(_) {
      _.transactions.some((H) => H.isUserEvent("input.type")) ? this.destroy() : this.style.update(_) && setTimeout(() => this.select(this.lastEvent), 20);
    }
  }
  function addsSelectionRange(O, _) {
    let H = O.state.facet(clickAddsSelectionRange);
    return H.length ? H[0](_) : browser.mac ? _.metaKey : _.ctrlKey;
  }
  function dragMovesSelection(O, _) {
    let H = O.state.facet(dragMovesSelection$1);
    return H.length ? H[0](_) : browser.mac ? !_.altKey : !_.ctrlKey;
  }
  function isInPrimarySelection(O, _) {
    let { main: H } = O.state.selection;
    if (H.empty)
      return !1;
    let V = getSelection(O.root);
    if (!V || V.rangeCount == 0)
      return !0;
    let q = V.getRangeAt(0).getClientRects();
    for (let G = 0; G < q.length; G++) {
      let J = q[G];
      if (J.left <= _.clientX && J.right >= _.clientX && J.top <= _.clientY && J.bottom >= _.clientY)
        return !0;
    }
    return !1;
  }
  function eventBelongsToEditor(O, _) {
    if (!_.bubbles)
      return !0;
    if (_.defaultPrevented)
      return !1;
    for (let H = _.target, V; H != O.contentDOM; H = H.parentNode)
      if (!H || H.nodeType == 11 || (V = ContentView.get(H)) && V.ignoreEvent(_))
        return !1;
    return !0;
  }
  const handlers = /* @__PURE__ */ Object.create(null), observers = /* @__PURE__ */ Object.create(null), brokenClipboardAPI = browser.ie && browser.ie_version < 15 || browser.ios && browser.webkit_version < 604;
  function capturePaste(O) {
    let _ = O.dom.parentNode;
    if (!_)
      return;
    let H = _.appendChild(document.createElement("textarea"));
    H.style.cssText = "position: fixed; left: -10000px; top: 10px", H.focus(), setTimeout(() => {
      O.focus(), H.remove(), doPaste(O, H.value);
    }, 50);
  }
  function textFilter(O, _, H) {
    for (let V of O.facet(_))
      H = V(H, O);
    return H;
  }
  function doPaste(O, _) {
    _ = textFilter(O.state, clipboardInputFilter, _);
    let { state: H } = O, V, q = 1, G = H.toText(_), J = G.lines == H.selection.ranges.length;
    if (lastLinewiseCopy != null && H.selection.ranges.every((te) => te.empty) && lastLinewiseCopy == G.toString()) {
      let te = -1;
      V = H.changeByRange((ne) => {
        let ee = H.doc.lineAt(ne.from);
        if (ee.from == te)
          return { range: ne };
        te = ee.from;
        let ae = H.toText((J ? G.line(q++).text : _) + H.lineBreak);
        return {
          changes: { from: ee.from, insert: ae },
          range: EditorSelection.cursor(ne.from + ae.length)
        };
      });
    } else J ? V = H.changeByRange((te) => {
      let ne = G.line(q++);
      return {
        changes: { from: te.from, to: te.to, insert: ne.text },
        range: EditorSelection.cursor(te.from + ne.length)
      };
    }) : V = H.replaceSelection(G);
    O.dispatch(V, {
      userEvent: "input.paste",
      scrollIntoView: !0
    });
  }
  observers.scroll = (O) => {
    O.inputState.lastScrollTop = O.scrollDOM.scrollTop, O.inputState.lastScrollLeft = O.scrollDOM.scrollLeft;
  };
  handlers.keydown = (O, _) => (O.inputState.setSelectionOrigin("select"), _.keyCode == 27 && O.inputState.tabFocusMode != 0 && (O.inputState.tabFocusMode = Date.now() + 2e3), !1);
  observers.touchstart = (O, _) => {
    O.inputState.lastTouchTime = Date.now(), O.inputState.setSelectionOrigin("select.pointer");
  };
  observers.touchmove = (O) => {
    O.inputState.setSelectionOrigin("select.pointer");
  };
  handlers.mousedown = (O, _) => {
    if (O.observer.flush(), O.inputState.lastTouchTime > Date.now() - 2e3)
      return !1;
    let H = null;
    for (let V of O.state.facet(mouseSelectionStyle))
      if (H = V(O, _), H)
        break;
    if (!H && _.button == 0 && (H = basicMouseSelection(O, _)), H) {
      let V = !O.hasFocus;
      O.inputState.startMouseSelection(new MouseSelection(O, _, H, V)), V && O.observer.ignore(() => {
        focusPreventScroll(O.contentDOM);
        let G = O.root.activeElement;
        G && !G.contains(O.contentDOM) && G.blur();
      });
      let q = O.inputState.mouseSelection;
      if (q)
        return q.start(_), q.dragging === !1;
    }
    return !1;
  };
  function rangeForClick(O, _, H, V) {
    if (V == 1)
      return EditorSelection.cursor(_, H);
    if (V == 2)
      return groupAt(O.state, _, H);
    {
      let q = LineView.find(O.docView, _), G = O.state.doc.lineAt(q ? q.posAtEnd : _), J = q ? q.posAtStart : G.from, Z = q ? q.posAtEnd : G.to;
      return Z < O.state.doc.length && Z == G.to && Z++, EditorSelection.range(J, Z);
    }
  }
  let inside = (O, _, H) => _ >= H.top && _ <= H.bottom && O >= H.left && O <= H.right;
  function findPositionSide(O, _, H, V) {
    let q = LineView.find(O.docView, _);
    if (!q)
      return 1;
    let G = _ - q.posAtStart;
    if (G == 0)
      return 1;
    if (G == q.length)
      return -1;
    let J = q.coordsAt(G, -1);
    if (J && inside(H, V, J))
      return -1;
    let Z = q.coordsAt(G, 1);
    return Z && inside(H, V, Z) ? 1 : J && J.bottom >= V ? -1 : 1;
  }
  function queryPos(O, _) {
    let H = O.posAtCoords({ x: _.clientX, y: _.clientY }, !1);
    return { pos: H, bias: findPositionSide(O, H, _.clientX, _.clientY) };
  }
  const BadMouseDetail = browser.ie && browser.ie_version <= 11;
  let lastMouseDown = null, lastMouseDownCount = 0, lastMouseDownTime = 0;
  function getClickType(O) {
    if (!BadMouseDetail)
      return O.detail;
    let _ = lastMouseDown, H = lastMouseDownTime;
    return lastMouseDown = O, lastMouseDownTime = Date.now(), lastMouseDownCount = !_ || H > Date.now() - 400 && Math.abs(_.clientX - O.clientX) < 2 && Math.abs(_.clientY - O.clientY) < 2 ? (lastMouseDownCount + 1) % 3 : 1;
  }
  function basicMouseSelection(O, _) {
    let H = queryPos(O, _), V = getClickType(_), q = O.state.selection;
    return {
      update(G) {
        G.docChanged && (H.pos = G.changes.mapPos(H.pos), q = q.map(G.changes));
      },
      get(G, J, Z) {
        let te = queryPos(O, G), ne, ee = rangeForClick(O, te.pos, te.bias, V);
        if (H.pos != te.pos && !J) {
          let ae = rangeForClick(O, H.pos, H.bias, V), oe = Math.min(ae.from, ee.from), se = Math.max(ae.to, ee.to);
          ee = oe < ee.from ? EditorSelection.range(oe, se) : EditorSelection.range(se, oe);
        }
        return J ? q.replaceRange(q.main.extend(ee.from, ee.to)) : Z && V == 1 && q.ranges.length > 1 && (ne = removeRangeAround(q, te.pos)) ? ne : Z ? q.addRange(ee) : EditorSelection.create([ee]);
      }
    };
  }
  function removeRangeAround(O, _) {
    for (let H = 0; H < O.ranges.length; H++) {
      let { from: V, to: q } = O.ranges[H];
      if (V <= _ && q >= _)
        return EditorSelection.create(O.ranges.slice(0, H).concat(O.ranges.slice(H + 1)), O.mainIndex == H ? 0 : O.mainIndex - (O.mainIndex > H ? 1 : 0));
    }
    return null;
  }
  handlers.dragstart = (O, _) => {
    let { selection: { main: H } } = O.state;
    if (_.target.draggable) {
      let q = O.docView.nearest(_.target);
      if (q && q.isWidget) {
        let G = q.posAtStart, J = G + q.length;
        (G >= H.to || J <= H.from) && (H = EditorSelection.range(G, J));
      }
    }
    let { inputState: V } = O;
    return V.mouseSelection && (V.mouseSelection.dragging = !0), V.draggedContent = H, _.dataTransfer && (_.dataTransfer.setData("Text", textFilter(O.state, clipboardOutputFilter, O.state.sliceDoc(H.from, H.to))), _.dataTransfer.effectAllowed = "copyMove"), !1;
  };
  handlers.dragend = (O) => (O.inputState.draggedContent = null, !1);
  function dropText(O, _, H, V) {
    if (H = textFilter(O.state, clipboardInputFilter, H), !H)
      return;
    let q = O.posAtCoords({ x: _.clientX, y: _.clientY }, !1), { draggedContent: G } = O.inputState, J = V && G && dragMovesSelection(O, _) ? { from: G.from, to: G.to } : null, Z = { from: q, insert: H }, te = O.state.changes(J ? [J, Z] : Z);
    O.focus(), O.dispatch({
      changes: te,
      selection: { anchor: te.mapPos(q, -1), head: te.mapPos(q, 1) },
      userEvent: J ? "move.drop" : "input.drop"
    }), O.inputState.draggedContent = null;
  }
  handlers.drop = (O, _) => {
    if (!_.dataTransfer)
      return !1;
    if (O.state.readOnly)
      return !0;
    let H = _.dataTransfer.files;
    if (H && H.length) {
      let V = Array(H.length), q = 0, G = () => {
        ++q == H.length && dropText(O, _, V.filter((J) => J != null).join(O.state.lineBreak), !1);
      };
      for (let J = 0; J < H.length; J++) {
        let Z = new FileReader();
        Z.onerror = G, Z.onload = () => {
          /[\x00-\x08\x0e-\x1f]{2}/.test(Z.result) || (V[J] = Z.result), G();
        }, Z.readAsText(H[J]);
      }
      return !0;
    } else {
      let V = _.dataTransfer.getData("Text");
      if (V)
        return dropText(O, _, V, !0), !0;
    }
    return !1;
  };
  handlers.paste = (O, _) => {
    if (O.state.readOnly)
      return !0;
    O.observer.flush();
    let H = brokenClipboardAPI ? null : _.clipboardData;
    return H ? (doPaste(O, H.getData("text/plain") || H.getData("text/uri-list")), !0) : (capturePaste(O), !1);
  };
  function captureCopy(O, _) {
    let H = O.dom.parentNode;
    if (!H)
      return;
    let V = H.appendChild(document.createElement("textarea"));
    V.style.cssText = "position: fixed; left: -10000px; top: 10px", V.value = _, V.focus(), V.selectionEnd = _.length, V.selectionStart = 0, setTimeout(() => {
      V.remove(), O.focus();
    }, 50);
  }
  function copiedRange(O) {
    let _ = [], H = [], V = !1;
    for (let q of O.selection.ranges)
      q.empty || (_.push(O.sliceDoc(q.from, q.to)), H.push(q));
    if (!_.length) {
      let q = -1;
      for (let { from: G } of O.selection.ranges) {
        let J = O.doc.lineAt(G);
        J.number > q && (_.push(J.text), H.push({ from: J.from, to: Math.min(O.doc.length, J.to + 1) })), q = J.number;
      }
      V = !0;
    }
    return { text: textFilter(O, clipboardOutputFilter, _.join(O.lineBreak)), ranges: H, linewise: V };
  }
  let lastLinewiseCopy = null;
  handlers.copy = handlers.cut = (O, _) => {
    let { text: H, ranges: V, linewise: q } = copiedRange(O.state);
    if (!H && !q)
      return !1;
    lastLinewiseCopy = q ? H : null, _.type == "cut" && !O.state.readOnly && O.dispatch({
      changes: V,
      scrollIntoView: !0,
      userEvent: "delete.cut"
    });
    let G = brokenClipboardAPI ? null : _.clipboardData;
    return G ? (G.clearData(), G.setData("text/plain", H), !0) : (captureCopy(O, H), !1);
  };
  const isFocusChange = /* @__PURE__ */ Annotation.define();
  function focusChangeTransaction(O, _) {
    let H = [];
    for (let V of O.facet(focusChangeEffect)) {
      let q = V(O, _);
      q && H.push(q);
    }
    return H ? O.update({ effects: H, annotations: isFocusChange.of(!0) }) : null;
  }
  function updateForFocusChange(O) {
    setTimeout(() => {
      let _ = O.hasFocus;
      if (_ != O.inputState.notifiedFocused) {
        let H = focusChangeTransaction(O.state, _);
        H ? O.dispatch(H) : O.update([]);
      }
    }, 10);
  }
  observers.focus = (O) => {
    O.inputState.lastFocusTime = Date.now(), !O.scrollDOM.scrollTop && (O.inputState.lastScrollTop || O.inputState.lastScrollLeft) && (O.scrollDOM.scrollTop = O.inputState.lastScrollTop, O.scrollDOM.scrollLeft = O.inputState.lastScrollLeft), updateForFocusChange(O);
  };
  observers.blur = (O) => {
    O.observer.clearSelectionRange(), updateForFocusChange(O);
  };
  observers.compositionstart = observers.compositionupdate = (O) => {
    O.observer.editContext || (O.inputState.compositionFirstChange == null && (O.inputState.compositionFirstChange = !0), O.inputState.composing < 0 && (O.inputState.composing = 0));
  };
  observers.compositionend = (O) => {
    O.observer.editContext || (O.inputState.composing = -1, O.inputState.compositionEndedAt = Date.now(), O.inputState.compositionPendingKey = !0, O.inputState.compositionPendingChange = O.observer.pendingRecords().length > 0, O.inputState.compositionFirstChange = null, browser.chrome && browser.android ? O.observer.flushSoon() : O.inputState.compositionPendingChange ? Promise.resolve().then(() => O.observer.flush()) : setTimeout(() => {
      O.inputState.composing < 0 && O.docView.hasComposition && O.update([]);
    }, 50));
  };
  observers.contextmenu = (O) => {
    O.inputState.lastContextMenu = Date.now();
  };
  handlers.beforeinput = (O, _) => {
    var H, V;
    if (_.inputType == "insertReplacementText" && O.observer.editContext) {
      let G = (H = _.dataTransfer) === null || H === void 0 ? void 0 : H.getData("text/plain"), J = _.getTargetRanges();
      if (G && J.length) {
        let Z = J[0], te = O.posAtDOM(Z.startContainer, Z.startOffset), ne = O.posAtDOM(Z.endContainer, Z.endOffset);
        return applyDOMChangeInner(O, { from: te, to: ne, insert: O.state.toText(G) }, null), !0;
      }
    }
    let q;
    if (browser.chrome && browser.android && (q = PendingKeys.find((G) => G.inputType == _.inputType)) && (O.observer.delayAndroidKey(q.key, q.keyCode), q.key == "Backspace" || q.key == "Delete")) {
      let G = ((V = window.visualViewport) === null || V === void 0 ? void 0 : V.height) || 0;
      setTimeout(() => {
        var J;
        (((J = window.visualViewport) === null || J === void 0 ? void 0 : J.height) || 0) > G + 10 && O.hasFocus && (O.contentDOM.blur(), O.focus());
      }, 100);
    }
    return browser.ios && _.inputType == "deleteContentForward" && O.observer.flushSoon(), browser.safari && _.inputType == "insertText" && O.inputState.composing >= 0 && setTimeout(() => observers.compositionend(O, _), 20), !1;
  };
  const appliedFirefoxHack = /* @__PURE__ */ new Set();
  function firefoxCopyCutHack(O) {
    appliedFirefoxHack.has(O) || (appliedFirefoxHack.add(O), O.addEventListener("copy", () => {
    }), O.addEventListener("cut", () => {
    }));
  }
  const wrappingWhiteSpace = ["pre-wrap", "normal", "pre-line", "break-spaces"];
  let heightChangeFlag = !1;
  function clearHeightChangeFlag() {
    heightChangeFlag = !1;
  }
  class HeightOracle {
    constructor(_) {
      this.lineWrapping = _, this.doc = Text.empty, this.heightSamples = {}, this.lineHeight = 14, this.charWidth = 7, this.textHeight = 14, this.lineLength = 30;
    }
    heightForGap(_, H) {
      let V = this.doc.lineAt(H).number - this.doc.lineAt(_).number + 1;
      return this.lineWrapping && (V += Math.max(0, Math.ceil((H - _ - V * this.lineLength * 0.5) / this.lineLength))), this.lineHeight * V;
    }
    heightForLine(_) {
      return this.lineWrapping ? (1 + Math.max(0, Math.ceil((_ - this.lineLength) / (this.lineLength - 5)))) * this.lineHeight : this.lineHeight;
    }
    setDoc(_) {
      return this.doc = _, this;
    }
    mustRefreshForWrapping(_) {
      return wrappingWhiteSpace.indexOf(_) > -1 != this.lineWrapping;
    }
    mustRefreshForHeights(_) {
      let H = !1;
      for (let V = 0; V < _.length; V++) {
        let q = _[V];
        q < 0 ? V++ : this.heightSamples[Math.floor(q * 10)] || (H = !0, this.heightSamples[Math.floor(q * 10)] = !0);
      }
      return H;
    }
    refresh(_, H, V, q, G, J) {
      let Z = wrappingWhiteSpace.indexOf(_) > -1, te = Math.round(H) != Math.round(this.lineHeight) || this.lineWrapping != Z;
      if (this.lineWrapping = Z, this.lineHeight = H, this.charWidth = V, this.textHeight = q, this.lineLength = G, te) {
        this.heightSamples = {};
        for (let ne = 0; ne < J.length; ne++) {
          let ee = J[ne];
          ee < 0 ? ne++ : this.heightSamples[Math.floor(ee * 10)] = !0;
        }
      }
      return te;
    }
  }
  class MeasuredHeights {
    constructor(_, H) {
      this.from = _, this.heights = H, this.index = 0;
    }
    get more() {
      return this.index < this.heights.length;
    }
  }
  class BlockInfo {
    /**
    @internal
    */
    constructor(_, H, V, q, G) {
      this.from = _, this.length = H, this.top = V, this.height = q, this._content = G;
    }
    /**
    The type of element this is. When querying lines, this may be
    an array of all the blocks that make up the line.
    */
    get type() {
      return typeof this._content == "number" ? BlockType.Text : Array.isArray(this._content) ? this._content : this._content.type;
    }
    /**
    The end of the element as a document position.
    */
    get to() {
      return this.from + this.length;
    }
    /**
    The bottom position of the element.
    */
    get bottom() {
      return this.top + this.height;
    }
    /**
    If this is a widget block, this will return the widget
    associated with it.
    */
    get widget() {
      return this._content instanceof PointDecoration ? this._content.widget : null;
    }
    /**
    If this is a textblock, this holds the number of line breaks
    that appear in widgets inside the block.
    */
    get widgetLineBreaks() {
      return typeof this._content == "number" ? this._content : 0;
    }
    /**
    @internal
    */
    join(_) {
      let H = (Array.isArray(this._content) ? this._content : [this]).concat(Array.isArray(_._content) ? _._content : [_]);
      return new BlockInfo(this.from, this.length + _.length, this.top, this.height + _.height, H);
    }
  }
  var QueryType$1 = /* @__PURE__ */ function(O) {
    return O[O.ByPos = 0] = "ByPos", O[O.ByHeight = 1] = "ByHeight", O[O.ByPosNoHeight = 2] = "ByPosNoHeight", O;
  }(QueryType$1 || (QueryType$1 = {}));
  const Epsilon = 1e-3;
  class HeightMap {
    constructor(_, H, V = 2) {
      this.length = _, this.height = H, this.flags = V;
    }
    get outdated() {
      return (this.flags & 2) > 0;
    }
    set outdated(_) {
      this.flags = (_ ? 2 : 0) | this.flags & -3;
    }
    setHeight(_) {
      this.height != _ && (Math.abs(this.height - _) > Epsilon && (heightChangeFlag = !0), this.height = _);
    }
    // Base case is to replace a leaf node, which simply builds a tree
    // from the new nodes and returns that (HeightMapBranch and
    // HeightMapGap override this to actually use from/to)
    replace(_, H, V) {
      return HeightMap.of(V);
    }
    // Again, these are base cases, and are overridden for branch and gap nodes.
    decomposeLeft(_, H) {
      H.push(this);
    }
    decomposeRight(_, H) {
      H.push(this);
    }
    applyChanges(_, H, V, q) {
      let G = this, J = V.doc;
      for (let Z = q.length - 1; Z >= 0; Z--) {
        let { fromA: te, toA: ne, fromB: ee, toB: ae } = q[Z], oe = G.lineAt(te, QueryType$1.ByPosNoHeight, V.setDoc(H), 0, 0), se = oe.to >= ne ? oe : G.lineAt(ne, QueryType$1.ByPosNoHeight, V, 0, 0);
        for (ae += se.to - ne, ne = se.to; Z > 0 && oe.from <= q[Z - 1].toA; )
          te = q[Z - 1].fromA, ee = q[Z - 1].fromB, Z--, te < oe.from && (oe = G.lineAt(te, QueryType$1.ByPosNoHeight, V, 0, 0));
        ee += oe.from - te, te = oe.from;
        let ce = NodeBuilder.build(V.setDoc(J), _, ee, ae);
        G = replace(G, G.replace(te, ne, ce));
      }
      return G.updateHeight(V, 0);
    }
    static empty() {
      return new HeightMapText(0, 0);
    }
    // nodes uses null values to indicate the position of line breaks.
    // There are never line breaks at the start or end of the array, or
    // two line breaks next to each other, and the array isn't allowed
    // to be empty (same restrictions as return value from the builder).
    static of(_) {
      if (_.length == 1)
        return _[0];
      let H = 0, V = _.length, q = 0, G = 0;
      for (; ; )
        if (H == V)
          if (q > G * 2) {
            let Z = _[H - 1];
            Z.break ? _.splice(--H, 1, Z.left, null, Z.right) : _.splice(--H, 1, Z.left, Z.right), V += 1 + Z.break, q -= Z.size;
          } else if (G > q * 2) {
            let Z = _[V];
            Z.break ? _.splice(V, 1, Z.left, null, Z.right) : _.splice(V, 1, Z.left, Z.right), V += 2 + Z.break, G -= Z.size;
          } else
            break;
        else if (q < G) {
          let Z = _[H++];
          Z && (q += Z.size);
        } else {
          let Z = _[--V];
          Z && (G += Z.size);
        }
      let J = 0;
      return _[H - 1] == null ? (J = 1, H--) : _[H] == null && (J = 1, V++), new HeightMapBranch(HeightMap.of(_.slice(0, H)), J, HeightMap.of(_.slice(V)));
    }
  }
  function replace(O, _) {
    return O == _ ? O : (O.constructor != _.constructor && (heightChangeFlag = !0), _);
  }
  HeightMap.prototype.size = 1;
  class HeightMapBlock extends HeightMap {
    constructor(_, H, V) {
      super(_, H), this.deco = V;
    }
    blockAt(_, H, V, q) {
      return new BlockInfo(q, this.length, V, this.height, this.deco || 0);
    }
    lineAt(_, H, V, q, G) {
      return this.blockAt(0, V, q, G);
    }
    forEachLine(_, H, V, q, G, J) {
      _ <= G + this.length && H >= G && J(this.blockAt(0, V, q, G));
    }
    updateHeight(_, H = 0, V = !1, q) {
      return q && q.from <= H && q.more && this.setHeight(q.heights[q.index++]), this.outdated = !1, this;
    }
    toString() {
      return `block(${this.length})`;
    }
  }
  class HeightMapText extends HeightMapBlock {
    constructor(_, H) {
      super(_, H, null), this.collapsed = 0, this.widgetHeight = 0, this.breaks = 0;
    }
    blockAt(_, H, V, q) {
      return new BlockInfo(q, this.length, V, this.height, this.breaks);
    }
    replace(_, H, V) {
      let q = V[0];
      return V.length == 1 && (q instanceof HeightMapText || q instanceof HeightMapGap && q.flags & 4) && Math.abs(this.length - q.length) < 10 ? (q instanceof HeightMapGap ? q = new HeightMapText(q.length, this.height) : q.height = this.height, this.outdated || (q.outdated = !1), q) : HeightMap.of(V);
    }
    updateHeight(_, H = 0, V = !1, q) {
      return q && q.from <= H && q.more ? this.setHeight(q.heights[q.index++]) : (V || this.outdated) && this.setHeight(Math.max(this.widgetHeight, _.heightForLine(this.length - this.collapsed)) + this.breaks * _.lineHeight), this.outdated = !1, this;
    }
    toString() {
      return `line(${this.length}${this.collapsed ? -this.collapsed : ""}${this.widgetHeight ? ":" + this.widgetHeight : ""})`;
    }
  }
  class HeightMapGap extends HeightMap {
    constructor(_) {
      super(_, 0);
    }
    heightMetrics(_, H) {
      let V = _.doc.lineAt(H).number, q = _.doc.lineAt(H + this.length).number, G = q - V + 1, J, Z = 0;
      if (_.lineWrapping) {
        let te = Math.min(this.height, _.lineHeight * G);
        J = te / G, this.length > G + 1 && (Z = (this.height - te) / (this.length - G - 1));
      } else
        J = this.height / G;
      return { firstLine: V, lastLine: q, perLine: J, perChar: Z };
    }
    blockAt(_, H, V, q) {
      let { firstLine: G, lastLine: J, perLine: Z, perChar: te } = this.heightMetrics(H, q);
      if (H.lineWrapping) {
        let ne = q + (_ < H.lineHeight ? 0 : Math.round(Math.max(0, Math.min(1, (_ - V) / this.height)) * this.length)), ee = H.doc.lineAt(ne), ae = Z + ee.length * te, oe = Math.max(V, _ - ae / 2);
        return new BlockInfo(ee.from, ee.length, oe, ae, 0);
      } else {
        let ne = Math.max(0, Math.min(J - G, Math.floor((_ - V) / Z))), { from: ee, length: ae } = H.doc.line(G + ne);
        return new BlockInfo(ee, ae, V + Z * ne, Z, 0);
      }
    }
    lineAt(_, H, V, q, G) {
      if (H == QueryType$1.ByHeight)
        return this.blockAt(_, V, q, G);
      if (H == QueryType$1.ByPosNoHeight) {
        let { from: se, to: ce } = V.doc.lineAt(_);
        return new BlockInfo(se, ce - se, 0, 0, 0);
      }
      let { firstLine: J, perLine: Z, perChar: te } = this.heightMetrics(V, G), ne = V.doc.lineAt(_), ee = Z + ne.length * te, ae = ne.number - J, oe = q + Z * ae + te * (ne.from - G - ae);
      return new BlockInfo(ne.from, ne.length, Math.max(q, Math.min(oe, q + this.height - ee)), ee, 0);
    }
    forEachLine(_, H, V, q, G, J) {
      _ = Math.max(_, G), H = Math.min(H, G + this.length);
      let { firstLine: Z, perLine: te, perChar: ne } = this.heightMetrics(V, G);
      for (let ee = _, ae = q; ee <= H; ) {
        let oe = V.doc.lineAt(ee);
        if (ee == _) {
          let ce = oe.number - Z;
          ae += te * ce + ne * (_ - G - ce);
        }
        let se = te + ne * oe.length;
        J(new BlockInfo(oe.from, oe.length, ae, se, 0)), ae += se, ee = oe.to + 1;
      }
    }
    replace(_, H, V) {
      let q = this.length - H;
      if (q > 0) {
        let G = V[V.length - 1];
        G instanceof HeightMapGap ? V[V.length - 1] = new HeightMapGap(G.length + q) : V.push(null, new HeightMapGap(q - 1));
      }
      if (_ > 0) {
        let G = V[0];
        G instanceof HeightMapGap ? V[0] = new HeightMapGap(_ + G.length) : V.unshift(new HeightMapGap(_ - 1), null);
      }
      return HeightMap.of(V);
    }
    decomposeLeft(_, H) {
      H.push(new HeightMapGap(_ - 1), null);
    }
    decomposeRight(_, H) {
      H.push(null, new HeightMapGap(this.length - _ - 1));
    }
    updateHeight(_, H = 0, V = !1, q) {
      let G = H + this.length;
      if (q && q.from <= H + this.length && q.more) {
        let J = [], Z = Math.max(H, q.from), te = -1;
        for (q.from > H && J.push(new HeightMapGap(q.from - H - 1).updateHeight(_, H)); Z <= G && q.more; ) {
          let ee = _.doc.lineAt(Z).length;
          J.length && J.push(null);
          let ae = q.heights[q.index++];
          te == -1 ? te = ae : Math.abs(ae - te) >= Epsilon && (te = -2);
          let oe = new HeightMapText(ee, ae);
          oe.outdated = !1, J.push(oe), Z += ee + 1;
        }
        Z <= G && J.push(null, new HeightMapGap(G - Z).updateHeight(_, Z));
        let ne = HeightMap.of(J);
        return (te < 0 || Math.abs(ne.height - this.height) >= Epsilon || Math.abs(te - this.heightMetrics(_, H).perLine) >= Epsilon) && (heightChangeFlag = !0), replace(this, ne);
      } else (V || this.outdated) && (this.setHeight(_.heightForGap(H, H + this.length)), this.outdated = !1);
      return this;
    }
    toString() {
      return `gap(${this.length})`;
    }
  }
  class HeightMapBranch extends HeightMap {
    constructor(_, H, V) {
      super(_.length + H + V.length, _.height + V.height, H | (_.outdated || V.outdated ? 2 : 0)), this.left = _, this.right = V, this.size = _.size + V.size;
    }
    get break() {
      return this.flags & 1;
    }
    blockAt(_, H, V, q) {
      let G = V + this.left.height;
      return _ < G ? this.left.blockAt(_, H, V, q) : this.right.blockAt(_, H, G, q + this.left.length + this.break);
    }
    lineAt(_, H, V, q, G) {
      let J = q + this.left.height, Z = G + this.left.length + this.break, te = H == QueryType$1.ByHeight ? _ < J : _ < Z, ne = te ? this.left.lineAt(_, H, V, q, G) : this.right.lineAt(_, H, V, J, Z);
      if (this.break || (te ? ne.to < Z : ne.from > Z))
        return ne;
      let ee = H == QueryType$1.ByPosNoHeight ? QueryType$1.ByPosNoHeight : QueryType$1.ByPos;
      return te ? ne.join(this.right.lineAt(Z, ee, V, J, Z)) : this.left.lineAt(Z, ee, V, q, G).join(ne);
    }
    forEachLine(_, H, V, q, G, J) {
      let Z = q + this.left.height, te = G + this.left.length + this.break;
      if (this.break)
        _ < te && this.left.forEachLine(_, H, V, q, G, J), H >= te && this.right.forEachLine(_, H, V, Z, te, J);
      else {
        let ne = this.lineAt(te, QueryType$1.ByPos, V, q, G);
        _ < ne.from && this.left.forEachLine(_, ne.from - 1, V, q, G, J), ne.to >= _ && ne.from <= H && J(ne), H > ne.to && this.right.forEachLine(ne.to + 1, H, V, Z, te, J);
      }
    }
    replace(_, H, V) {
      let q = this.left.length + this.break;
      if (H < q)
        return this.balanced(this.left.replace(_, H, V), this.right);
      if (_ > this.left.length)
        return this.balanced(this.left, this.right.replace(_ - q, H - q, V));
      let G = [];
      _ > 0 && this.decomposeLeft(_, G);
      let J = G.length;
      for (let Z of V)
        G.push(Z);
      if (_ > 0 && mergeGaps(G, J - 1), H < this.length) {
        let Z = G.length;
        this.decomposeRight(H, G), mergeGaps(G, Z);
      }
      return HeightMap.of(G);
    }
    decomposeLeft(_, H) {
      let V = this.left.length;
      if (_ <= V)
        return this.left.decomposeLeft(_, H);
      H.push(this.left), this.break && (V++, _ >= V && H.push(null)), _ > V && this.right.decomposeLeft(_ - V, H);
    }
    decomposeRight(_, H) {
      let V = this.left.length, q = V + this.break;
      if (_ >= q)
        return this.right.decomposeRight(_ - q, H);
      _ < V && this.left.decomposeRight(_, H), this.break && _ < q && H.push(null), H.push(this.right);
    }
    balanced(_, H) {
      return _.size > 2 * H.size || H.size > 2 * _.size ? HeightMap.of(this.break ? [_, null, H] : [_, H]) : (this.left = replace(this.left, _), this.right = replace(this.right, H), this.setHeight(_.height + H.height), this.outdated = _.outdated || H.outdated, this.size = _.size + H.size, this.length = _.length + this.break + H.length, this);
    }
    updateHeight(_, H = 0, V = !1, q) {
      let { left: G, right: J } = this, Z = H + G.length + this.break, te = null;
      return q && q.from <= H + G.length && q.more ? te = G = G.updateHeight(_, H, V, q) : G.updateHeight(_, H, V), q && q.from <= Z + J.length && q.more ? te = J = J.updateHeight(_, Z, V, q) : J.updateHeight(_, Z, V), te ? this.balanced(G, J) : (this.height = this.left.height + this.right.height, this.outdated = !1, this);
    }
    toString() {
      return this.left + (this.break ? " " : "-") + this.right;
    }
  }
  function mergeGaps(O, _) {
    let H, V;
    O[_] == null && (H = O[_ - 1]) instanceof HeightMapGap && (V = O[_ + 1]) instanceof HeightMapGap && O.splice(_ - 1, 3, new HeightMapGap(H.length + 1 + V.length));
  }
  const relevantWidgetHeight = 5;
  class NodeBuilder {
    constructor(_, H) {
      this.pos = _, this.oracle = H, this.nodes = [], this.lineStart = -1, this.lineEnd = -1, this.covering = null, this.writtenTo = _;
    }
    get isCovered() {
      return this.covering && this.nodes[this.nodes.length - 1] == this.covering;
    }
    span(_, H) {
      if (this.lineStart > -1) {
        let V = Math.min(H, this.lineEnd), q = this.nodes[this.nodes.length - 1];
        q instanceof HeightMapText ? q.length += V - this.pos : (V > this.pos || !this.isCovered) && this.nodes.push(new HeightMapText(V - this.pos, -1)), this.writtenTo = V, H > V && (this.nodes.push(null), this.writtenTo++, this.lineStart = -1);
      }
      this.pos = H;
    }
    point(_, H, V) {
      if (_ < H || V.heightRelevant) {
        let q = V.widget ? V.widget.estimatedHeight : 0, G = V.widget ? V.widget.lineBreaks : 0;
        q < 0 && (q = this.oracle.lineHeight);
        let J = H - _;
        V.block ? this.addBlock(new HeightMapBlock(J, q, V)) : (J || G || q >= relevantWidgetHeight) && this.addLineDeco(q, G, J);
      } else H > _ && this.span(_, H);
      this.lineEnd > -1 && this.lineEnd < this.pos && (this.lineEnd = this.oracle.doc.lineAt(this.pos).to);
    }
    enterLine() {
      if (this.lineStart > -1)
        return;
      let { from: _, to: H } = this.oracle.doc.lineAt(this.pos);
      this.lineStart = _, this.lineEnd = H, this.writtenTo < _ && ((this.writtenTo < _ - 1 || this.nodes[this.nodes.length - 1] == null) && this.nodes.push(this.blankContent(this.writtenTo, _ - 1)), this.nodes.push(null)), this.pos > _ && this.nodes.push(new HeightMapText(this.pos - _, -1)), this.writtenTo = this.pos;
    }
    blankContent(_, H) {
      let V = new HeightMapGap(H - _);
      return this.oracle.doc.lineAt(_).to == H && (V.flags |= 4), V;
    }
    ensureLine() {
      this.enterLine();
      let _ = this.nodes.length ? this.nodes[this.nodes.length - 1] : null;
      if (_ instanceof HeightMapText)
        return _;
      let H = new HeightMapText(0, -1);
      return this.nodes.push(H), H;
    }
    addBlock(_) {
      this.enterLine();
      let H = _.deco;
      H && H.startSide > 0 && !this.isCovered && this.ensureLine(), this.nodes.push(_), this.writtenTo = this.pos = this.pos + _.length, H && H.endSide > 0 && (this.covering = _);
    }
    addLineDeco(_, H, V) {
      let q = this.ensureLine();
      q.length += V, q.collapsed += V, q.widgetHeight = Math.max(q.widgetHeight, _), q.breaks += H, this.writtenTo = this.pos = this.pos + V;
    }
    finish(_) {
      let H = this.nodes.length == 0 ? null : this.nodes[this.nodes.length - 1];
      this.lineStart > -1 && !(H instanceof HeightMapText) && !this.isCovered ? this.nodes.push(new HeightMapText(0, -1)) : (this.writtenTo < this.pos || H == null) && this.nodes.push(this.blankContent(this.writtenTo, this.pos));
      let V = _;
      for (let q of this.nodes)
        q instanceof HeightMapText && q.updateHeight(this.oracle, V), V += q ? q.length : 1;
      return this.nodes;
    }
    // Always called with a region that on both sides either stretches
    // to a line break or the end of the document.
    // The returned array uses null to indicate line breaks, but never
    // starts or ends in a line break, or has multiple line breaks next
    // to each other.
    static build(_, H, V, q) {
      let G = new NodeBuilder(V, _);
      return RangeSet.spans(H, V, q, G, 0), G.finish(V);
    }
  }
  function heightRelevantDecoChanges(O, _, H) {
    let V = new DecorationComparator();
    return RangeSet.compare(O, _, H, V, 0), V.changes;
  }
  class DecorationComparator {
    constructor() {
      this.changes = [];
    }
    compareRange() {
    }
    comparePoint(_, H, V, q) {
      (_ < H || V && V.heightRelevant || q && q.heightRelevant) && addRange(_, H, this.changes, 5);
    }
  }
  function visiblePixelRange(O, _) {
    let H = O.getBoundingClientRect(), V = O.ownerDocument, q = V.defaultView || window, G = Math.max(0, H.left), J = Math.min(q.innerWidth, H.right), Z = Math.max(0, H.top), te = Math.min(q.innerHeight, H.bottom);
    for (let ne = O.parentNode; ne && ne != V.body; )
      if (ne.nodeType == 1) {
        let ee = ne, ae = window.getComputedStyle(ee);
        if ((ee.scrollHeight > ee.clientHeight || ee.scrollWidth > ee.clientWidth) && ae.overflow != "visible") {
          let oe = ee.getBoundingClientRect();
          G = Math.max(G, oe.left), J = Math.min(J, oe.right), Z = Math.max(Z, oe.top), te = Math.min(ne == O.parentNode ? q.innerHeight : te, oe.bottom);
        }
        ne = ae.position == "absolute" || ae.position == "fixed" ? ee.offsetParent : ee.parentNode;
      } else if (ne.nodeType == 11)
        ne = ne.host;
      else
        break;
    return {
      left: G - H.left,
      right: Math.max(G, J) - H.left,
      top: Z - (H.top + _),
      bottom: Math.max(Z, te) - (H.top + _)
    };
  }
  function fullPixelRange(O, _) {
    let H = O.getBoundingClientRect();
    return {
      left: 0,
      right: H.right - H.left,
      top: _,
      bottom: H.bottom - (H.top + _)
    };
  }
  class LineGap {
    constructor(_, H, V, q) {
      this.from = _, this.to = H, this.size = V, this.displaySize = q;
    }
    static same(_, H) {
      if (_.length != H.length)
        return !1;
      for (let V = 0; V < _.length; V++) {
        let q = _[V], G = H[V];
        if (q.from != G.from || q.to != G.to || q.size != G.size)
          return !1;
      }
      return !0;
    }
    draw(_, H) {
      return Decoration.replace({
        widget: new LineGapWidget(this.displaySize * (H ? _.scaleY : _.scaleX), H)
      }).range(this.from, this.to);
    }
  }
  class LineGapWidget extends WidgetType {
    constructor(_, H) {
      super(), this.size = _, this.vertical = H;
    }
    eq(_) {
      return _.size == this.size && _.vertical == this.vertical;
    }
    toDOM() {
      let _ = document.createElement("div");
      return this.vertical ? _.style.height = this.size + "px" : (_.style.width = this.size + "px", _.style.height = "2px", _.style.display = "inline-block"), _;
    }
    get estimatedHeight() {
      return this.vertical ? this.size : -1;
    }
  }
  class ViewState {
    constructor(_) {
      this.state = _, this.pixelViewport = { left: 0, right: window.innerWidth, top: 0, bottom: 0 }, this.inView = !0, this.paddingTop = 0, this.paddingBottom = 0, this.contentDOMWidth = 0, this.contentDOMHeight = 0, this.editorHeight = 0, this.editorWidth = 0, this.scrollTop = 0, this.scrolledToBottom = !1, this.scaleX = 1, this.scaleY = 1, this.scrollAnchorPos = 0, this.scrollAnchorHeight = -1, this.scaler = IdScaler, this.scrollTarget = null, this.printing = !1, this.mustMeasureContent = !0, this.defaultTextDirection = Direction.LTR, this.visibleRanges = [], this.mustEnforceCursorAssoc = !1;
      let H = _.facet(contentAttributes).some((V) => typeof V != "function" && V.class == "cm-lineWrapping");
      this.heightOracle = new HeightOracle(H), this.stateDeco = _.facet(decorations).filter((V) => typeof V != "function"), this.heightMap = HeightMap.empty().applyChanges(this.stateDeco, Text.empty, this.heightOracle.setDoc(_.doc), [new ChangedRange(0, 0, 0, _.doc.length)]);
      for (let V = 0; V < 2 && (this.viewport = this.getViewport(0, null), !!this.updateForViewport()); V++)
        ;
      this.updateViewportLines(), this.lineGaps = this.ensureLineGaps([]), this.lineGapDeco = Decoration.set(this.lineGaps.map((V) => V.draw(this, !1))), this.computeVisibleRanges();
    }
    updateForViewport() {
      let _ = [this.viewport], { main: H } = this.state.selection;
      for (let V = 0; V <= 1; V++) {
        let q = V ? H.head : H.anchor;
        if (!_.some(({ from: G, to: J }) => q >= G && q <= J)) {
          let { from: G, to: J } = this.lineBlockAt(q);
          _.push(new Viewport(G, J));
        }
      }
      return this.viewports = _.sort((V, q) => V.from - q.from), this.updateScaler();
    }
    updateScaler() {
      let _ = this.scaler;
      return this.scaler = this.heightMap.height <= 7e6 ? IdScaler : new BigScaler(this.heightOracle, this.heightMap, this.viewports), _.eq(this.scaler) ? 0 : 2;
    }
    updateViewportLines() {
      this.viewportLines = [], this.heightMap.forEachLine(this.viewport.from, this.viewport.to, this.heightOracle.setDoc(this.state.doc), 0, 0, (_) => {
        this.viewportLines.push(scaleBlock(_, this.scaler));
      });
    }
    update(_, H = null) {
      this.state = _.state;
      let V = this.stateDeco;
      this.stateDeco = this.state.facet(decorations).filter((ee) => typeof ee != "function");
      let q = _.changedRanges, G = ChangedRange.extendWithRanges(q, heightRelevantDecoChanges(V, this.stateDeco, _ ? _.changes : ChangeSet.empty(this.state.doc.length))), J = this.heightMap.height, Z = this.scrolledToBottom ? null : this.scrollAnchorAt(this.scrollTop);
      clearHeightChangeFlag(), this.heightMap = this.heightMap.applyChanges(this.stateDeco, _.startState.doc, this.heightOracle.setDoc(this.state.doc), G), (this.heightMap.height != J || heightChangeFlag) && (_.flags |= 2), Z ? (this.scrollAnchorPos = _.changes.mapPos(Z.from, -1), this.scrollAnchorHeight = Z.top) : (this.scrollAnchorPos = -1, this.scrollAnchorHeight = this.heightMap.height);
      let te = G.length ? this.mapViewport(this.viewport, _.changes) : this.viewport;
      (H && (H.range.head < te.from || H.range.head > te.to) || !this.viewportIsAppropriate(te)) && (te = this.getViewport(0, H));
      let ne = te.from != this.viewport.from || te.to != this.viewport.to;
      this.viewport = te, _.flags |= this.updateForViewport(), (ne || !_.changes.empty || _.flags & 2) && this.updateViewportLines(), (this.lineGaps.length || this.viewport.to - this.viewport.from > 4e3) && this.updateLineGaps(this.ensureLineGaps(this.mapLineGaps(this.lineGaps, _.changes))), _.flags |= this.computeVisibleRanges(), H && (this.scrollTarget = H), !this.mustEnforceCursorAssoc && _.selectionSet && _.view.lineWrapping && _.state.selection.main.empty && _.state.selection.main.assoc && !_.state.facet(nativeSelectionHidden) && (this.mustEnforceCursorAssoc = !0);
    }
    measure(_) {
      let H = _.contentDOM, V = window.getComputedStyle(H), q = this.heightOracle, G = V.whiteSpace;
      this.defaultTextDirection = V.direction == "rtl" ? Direction.RTL : Direction.LTR;
      let J = this.heightOracle.mustRefreshForWrapping(G), Z = H.getBoundingClientRect(), te = J || this.mustMeasureContent || this.contentDOMHeight != Z.height;
      this.contentDOMHeight = Z.height, this.mustMeasureContent = !1;
      let ne = 0, ee = 0;
      if (Z.width && Z.height) {
        let { scaleX: me, scaleY: pe } = getScale(H, Z);
        (me > 5e-3 && Math.abs(this.scaleX - me) > 5e-3 || pe > 5e-3 && Math.abs(this.scaleY - pe) > 5e-3) && (this.scaleX = me, this.scaleY = pe, ne |= 8, J = te = !0);
      }
      let ae = (parseInt(V.paddingTop) || 0) * this.scaleY, oe = (parseInt(V.paddingBottom) || 0) * this.scaleY;
      (this.paddingTop != ae || this.paddingBottom != oe) && (this.paddingTop = ae, this.paddingBottom = oe, ne |= 10), this.editorWidth != _.scrollDOM.clientWidth && (q.lineWrapping && (te = !0), this.editorWidth = _.scrollDOM.clientWidth, ne |= 8);
      let se = _.scrollDOM.scrollTop * this.scaleY;
      this.scrollTop != se && (this.scrollAnchorHeight = -1, this.scrollTop = se), this.scrolledToBottom = isScrolledToBottom(_.scrollDOM);
      let ce = (this.printing ? fullPixelRange : visiblePixelRange)(H, this.paddingTop), le = ce.top - this.pixelViewport.top, ue = ce.bottom - this.pixelViewport.bottom;
      this.pixelViewport = ce;
      let de = this.pixelViewport.bottom > this.pixelViewport.top && this.pixelViewport.right > this.pixelViewport.left;
      if (de != this.inView && (this.inView = de, de && (te = !0)), !this.inView && !this.scrollTarget)
        return 0;
      let fe = Z.width;
      if ((this.contentDOMWidth != fe || this.editorHeight != _.scrollDOM.clientHeight) && (this.contentDOMWidth = Z.width, this.editorHeight = _.scrollDOM.clientHeight, ne |= 8), te) {
        let me = _.docView.measureVisibleLineHeights(this.viewport);
        if (q.mustRefreshForHeights(me) && (J = !0), J || q.lineWrapping && Math.abs(fe - this.contentDOMWidth) > q.charWidth) {
          let { lineHeight: pe, charWidth: ye, textHeight: ge } = _.docView.measureTextSize();
          J = pe > 0 && q.refresh(G, pe, ye, ge, fe / ye, me), J && (_.docView.minWidth = 0, ne |= 8);
        }
        le > 0 && ue > 0 ? ee = Math.max(le, ue) : le < 0 && ue < 0 && (ee = Math.min(le, ue)), clearHeightChangeFlag();
        for (let pe of this.viewports) {
          let ye = pe.from == this.viewport.from ? me : _.docView.measureVisibleLineHeights(pe);
          this.heightMap = (J ? HeightMap.empty().applyChanges(this.stateDeco, Text.empty, this.heightOracle, [new ChangedRange(0, 0, 0, _.state.doc.length)]) : this.heightMap).updateHeight(q, 0, J, new MeasuredHeights(pe.from, ye));
        }
        heightChangeFlag && (ne |= 2);
      }
      let he = !this.viewportIsAppropriate(this.viewport, ee) || this.scrollTarget && (this.scrollTarget.range.head < this.viewport.from || this.scrollTarget.range.head > this.viewport.to);
      return he && (ne & 2 && (ne |= this.updateScaler()), this.viewport = this.getViewport(ee, this.scrollTarget), ne |= this.updateForViewport()), (ne & 2 || he) && this.updateViewportLines(), (this.lineGaps.length || this.viewport.to - this.viewport.from > 4e3) && this.updateLineGaps(this.ensureLineGaps(J ? [] : this.lineGaps, _)), ne |= this.computeVisibleRanges(), this.mustEnforceCursorAssoc && (this.mustEnforceCursorAssoc = !1, _.docView.enforceCursorAssoc()), ne;
    }
    get visibleTop() {
      return this.scaler.fromDOM(this.pixelViewport.top);
    }
    get visibleBottom() {
      return this.scaler.fromDOM(this.pixelViewport.bottom);
    }
    getViewport(_, H) {
      let V = 0.5 - Math.max(-0.5, Math.min(0.5, _ / 1e3 / 2)), q = this.heightMap, G = this.heightOracle, { visibleTop: J, visibleBottom: Z } = this, te = new Viewport(q.lineAt(J - V * 1e3, QueryType$1.ByHeight, G, 0, 0).from, q.lineAt(Z + (1 - V) * 1e3, QueryType$1.ByHeight, G, 0, 0).to);
      if (H) {
        let { head: ne } = H.range;
        if (ne < te.from || ne > te.to) {
          let ee = Math.min(this.editorHeight, this.pixelViewport.bottom - this.pixelViewport.top), ae = q.lineAt(ne, QueryType$1.ByPos, G, 0, 0), oe;
          H.y == "center" ? oe = (ae.top + ae.bottom) / 2 - ee / 2 : H.y == "start" || H.y == "nearest" && ne < te.from ? oe = ae.top : oe = ae.bottom - ee, te = new Viewport(q.lineAt(oe - 1e3 / 2, QueryType$1.ByHeight, G, 0, 0).from, q.lineAt(oe + ee + 1e3 / 2, QueryType$1.ByHeight, G, 0, 0).to);
        }
      }
      return te;
    }
    mapViewport(_, H) {
      let V = H.mapPos(_.from, -1), q = H.mapPos(_.to, 1);
      return new Viewport(this.heightMap.lineAt(V, QueryType$1.ByPos, this.heightOracle, 0, 0).from, this.heightMap.lineAt(q, QueryType$1.ByPos, this.heightOracle, 0, 0).to);
    }
    // Checks if a given viewport covers the visible part of the
    // document and not too much beyond that.
    viewportIsAppropriate({ from: _, to: H }, V = 0) {
      if (!this.inView)
        return !0;
      let { top: q } = this.heightMap.lineAt(_, QueryType$1.ByPos, this.heightOracle, 0, 0), { bottom: G } = this.heightMap.lineAt(H, QueryType$1.ByPos, this.heightOracle, 0, 0), { visibleTop: J, visibleBottom: Z } = this;
      return (_ == 0 || q <= J - Math.max(10, Math.min(
        -V,
        250
        /* VP.MaxCoverMargin */
      ))) && (H == this.state.doc.length || G >= Z + Math.max(10, Math.min(
        V,
        250
        /* VP.MaxCoverMargin */
      ))) && q > J - 2 * 1e3 && G < Z + 2 * 1e3;
    }
    mapLineGaps(_, H) {
      if (!_.length || H.empty)
        return _;
      let V = [];
      for (let q of _)
        H.touchesRange(q.from, q.to) || V.push(new LineGap(H.mapPos(q.from), H.mapPos(q.to), q.size, q.displaySize));
      return V;
    }
    // Computes positions in the viewport where the start or end of a
    // line should be hidden, trying to reuse existing line gaps when
    // appropriate to avoid unneccesary redraws.
    // Uses crude character-counting for the positioning and sizing,
    // since actual DOM coordinates aren't always available and
    // predictable. Relies on generous margins (see LG.Margin) to hide
    // the artifacts this might produce from the user.
    ensureLineGaps(_, H) {
      let V = this.heightOracle.lineWrapping, q = V ? 1e4 : 2e3, G = q >> 1, J = q << 1;
      if (this.defaultTextDirection != Direction.LTR && !V)
        return [];
      let Z = [], te = (ee, ae, oe, se) => {
        if (ae - ee < G)
          return;
        let ce = this.state.selection.main, le = [ce.from];
        ce.empty || le.push(ce.to);
        for (let de of le)
          if (de > ee && de < ae) {
            te(ee, de - 10, oe, se), te(de + 10, ae, oe, se);
            return;
          }
        let ue = find(_, (de) => de.from >= oe.from && de.to <= oe.to && Math.abs(de.from - ee) < G && Math.abs(de.to - ae) < G && !le.some((fe) => de.from < fe && de.to > fe));
        if (!ue) {
          if (ae < oe.to && H && V && H.visibleRanges.some((he) => he.from <= ae && he.to >= ae)) {
            let he = H.moveToLineBoundary(EditorSelection.cursor(ae), !1, !0).head;
            he > ee && (ae = he);
          }
          let de = this.gapSize(oe, ee, ae, se), fe = V || de < 2e6 ? de : 2e6;
          ue = new LineGap(ee, ae, de, fe);
        }
        Z.push(ue);
      }, ne = (ee) => {
        if (ee.length < J || ee.type != BlockType.Text)
          return;
        let ae = lineStructure(ee.from, ee.to, this.stateDeco);
        if (ae.total < J)
          return;
        let oe = this.scrollTarget ? this.scrollTarget.range.head : null, se, ce;
        if (V) {
          let le = q / this.heightOracle.lineLength * this.heightOracle.lineHeight, ue, de;
          if (oe != null) {
            let fe = findFraction(ae, oe), he = ((this.visibleBottom - this.visibleTop) / 2 + le) / ee.height;
            ue = fe - he, de = fe + he;
          } else
            ue = (this.visibleTop - ee.top - le) / ee.height, de = (this.visibleBottom - ee.top + le) / ee.height;
          se = findPosition(ae, ue), ce = findPosition(ae, de);
        } else {
          let le = ae.total * this.heightOracle.charWidth, ue = q * this.heightOracle.charWidth, de = 0;
          if (le > 2e6)
            for (let ye of _)
              ye.from >= ee.from && ye.from < ee.to && ye.size != ye.displaySize && ye.from * this.heightOracle.charWidth + de < this.pixelViewport.left && (de = ye.size - ye.displaySize);
          let fe = this.pixelViewport.left + de, he = this.pixelViewport.right + de, me, pe;
          if (oe != null) {
            let ye = findFraction(ae, oe), ge = ((he - fe) / 2 + ue) / le;
            me = ye - ge, pe = ye + ge;
          } else
            me = (fe - ue) / le, pe = (he + ue) / le;
          se = findPosition(ae, me), ce = findPosition(ae, pe);
        }
        se > ee.from && te(ee.from, se, ee, ae), ce < ee.to && te(ce, ee.to, ee, ae);
      };
      for (let ee of this.viewportLines)
        Array.isArray(ee.type) ? ee.type.forEach(ne) : ne(ee);
      return Z;
    }
    gapSize(_, H, V, q) {
      let G = findFraction(q, V) - findFraction(q, H);
      return this.heightOracle.lineWrapping ? _.height * G : q.total * this.heightOracle.charWidth * G;
    }
    updateLineGaps(_) {
      LineGap.same(_, this.lineGaps) || (this.lineGaps = _, this.lineGapDeco = Decoration.set(_.map((H) => H.draw(this, this.heightOracle.lineWrapping))));
    }
    computeVisibleRanges() {
      let _ = this.stateDeco;
      this.lineGaps.length && (_ = _.concat(this.lineGapDeco));
      let H = [];
      RangeSet.spans(_, this.viewport.from, this.viewport.to, {
        span(q, G) {
          H.push({ from: q, to: G });
        },
        point() {
        }
      }, 20);
      let V = H.length != this.visibleRanges.length || this.visibleRanges.some((q, G) => q.from != H[G].from || q.to != H[G].to);
      return this.visibleRanges = H, V ? 4 : 0;
    }
    lineBlockAt(_) {
      return _ >= this.viewport.from && _ <= this.viewport.to && this.viewportLines.find((H) => H.from <= _ && H.to >= _) || scaleBlock(this.heightMap.lineAt(_, QueryType$1.ByPos, this.heightOracle, 0, 0), this.scaler);
    }
    lineBlockAtHeight(_) {
      return _ >= this.viewportLines[0].top && _ <= this.viewportLines[this.viewportLines.length - 1].bottom && this.viewportLines.find((H) => H.top <= _ && H.bottom >= _) || scaleBlock(this.heightMap.lineAt(this.scaler.fromDOM(_), QueryType$1.ByHeight, this.heightOracle, 0, 0), this.scaler);
    }
    scrollAnchorAt(_) {
      let H = this.lineBlockAtHeight(_ + 8);
      return H.from >= this.viewport.from || this.viewportLines[0].top - _ > 200 ? H : this.viewportLines[0];
    }
    elementAtHeight(_) {
      return scaleBlock(this.heightMap.blockAt(this.scaler.fromDOM(_), this.heightOracle, 0, 0), this.scaler);
    }
    get docHeight() {
      return this.scaler.toDOM(this.heightMap.height);
    }
    get contentHeight() {
      return this.docHeight + this.paddingTop + this.paddingBottom;
    }
  }
  class Viewport {
    constructor(_, H) {
      this.from = _, this.to = H;
    }
  }
  function lineStructure(O, _, H) {
    let V = [], q = O, G = 0;
    return RangeSet.spans(H, O, _, {
      span() {
      },
      point(J, Z) {
        J > q && (V.push({ from: q, to: J }), G += J - q), q = Z;
      }
    }, 20), q < _ && (V.push({ from: q, to: _ }), G += _ - q), { total: G, ranges: V };
  }
  function findPosition({ total: O, ranges: _ }, H) {
    if (H <= 0)
      return _[0].from;
    if (H >= 1)
      return _[_.length - 1].to;
    let V = Math.floor(O * H);
    for (let q = 0; ; q++) {
      let { from: G, to: J } = _[q], Z = J - G;
      if (V <= Z)
        return G + V;
      V -= Z;
    }
  }
  function findFraction(O, _) {
    let H = 0;
    for (let { from: V, to: q } of O.ranges) {
      if (_ <= q) {
        H += _ - V;
        break;
      }
      H += q - V;
    }
    return H / O.total;
  }
  function find(O, _) {
    for (let H of O)
      if (_(H))
        return H;
  }
  const IdScaler = {
    toDOM(O) {
      return O;
    },
    fromDOM(O) {
      return O;
    },
    scale: 1,
    eq(O) {
      return O == this;
    }
  };
  class BigScaler {
    constructor(_, H, V) {
      let q = 0, G = 0, J = 0;
      this.viewports = V.map(({ from: Z, to: te }) => {
        let ne = H.lineAt(Z, QueryType$1.ByPos, _, 0, 0).top, ee = H.lineAt(te, QueryType$1.ByPos, _, 0, 0).bottom;
        return q += ee - ne, { from: Z, to: te, top: ne, bottom: ee, domTop: 0, domBottom: 0 };
      }), this.scale = (7e6 - q) / (H.height - q);
      for (let Z of this.viewports)
        Z.domTop = J + (Z.top - G) * this.scale, J = Z.domBottom = Z.domTop + (Z.bottom - Z.top), G = Z.bottom;
    }
    toDOM(_) {
      for (let H = 0, V = 0, q = 0; ; H++) {
        let G = H < this.viewports.length ? this.viewports[H] : null;
        if (!G || _ < G.top)
          return q + (_ - V) * this.scale;
        if (_ <= G.bottom)
          return G.domTop + (_ - G.top);
        V = G.bottom, q = G.domBottom;
      }
    }
    fromDOM(_) {
      for (let H = 0, V = 0, q = 0; ; H++) {
        let G = H < this.viewports.length ? this.viewports[H] : null;
        if (!G || _ < G.domTop)
          return V + (_ - q) / this.scale;
        if (_ <= G.domBottom)
          return G.top + (_ - G.domTop);
        V = G.bottom, q = G.domBottom;
      }
    }
    eq(_) {
      return _ instanceof BigScaler ? this.scale == _.scale && this.viewports.length == _.viewports.length && this.viewports.every((H, V) => H.from == _.viewports[V].from && H.to == _.viewports[V].to) : !1;
    }
  }
  function scaleBlock(O, _) {
    if (_.scale == 1)
      return O;
    let H = _.toDOM(O.top), V = _.toDOM(O.bottom);
    return new BlockInfo(O.from, O.length, H, V - H, Array.isArray(O._content) ? O._content.map((q) => scaleBlock(q, _)) : O._content);
  }
  const theme = /* @__PURE__ */ Facet.define({ combine: (O) => O.join(" ") }), darkTheme = /* @__PURE__ */ Facet.define({ combine: (O) => O.indexOf(!0) > -1 }), baseThemeID = /* @__PURE__ */ StyleModule.newName(), baseLightID = /* @__PURE__ */ StyleModule.newName(), baseDarkID = /* @__PURE__ */ StyleModule.newName(), lightDarkIDs = { "&light": "." + baseLightID, "&dark": "." + baseDarkID };
  function buildTheme(O, _, H) {
    return new StyleModule(_, {
      finish(V) {
        return /&/.test(V) ? V.replace(/&\w*/, (q) => {
          if (q == "&")
            return O;
          if (!H || !H[q])
            throw new RangeError(`Unsupported selector: ${q}`);
          return H[q];
        }) : O + " " + V;
      }
    });
  }
  const baseTheme$1$2 = /* @__PURE__ */ buildTheme("." + baseThemeID, {
    "&": {
      position: "relative !important",
      boxSizing: "border-box",
      "&.cm-focused": {
        // Provide a simple default outline to make sure a focused
        // editor is visually distinct. Can't leave the default behavior
        // because that will apply to the content element, which is
        // inside the scrollable container and doesn't include the
        // gutters. We also can't use an 'auto' outline, since those
        // are, for some reason, drawn behind the element content, which
        // will cause things like the active line background to cover
        // the outline (#297).
        outline: "1px dotted #212121"
      },
      display: "flex !important",
      flexDirection: "column"
    },
    ".cm-scroller": {
      display: "flex !important",
      alignItems: "flex-start !important",
      fontFamily: "monospace",
      lineHeight: 1.4,
      height: "100%",
      overflowX: "auto",
      position: "relative",
      zIndex: 0,
      overflowAnchor: "none"
    },
    ".cm-content": {
      margin: 0,
      flexGrow: 2,
      flexShrink: 0,
      display: "block",
      whiteSpace: "pre",
      wordWrap: "normal",
      // https://github.com/codemirror/dev/issues/456
      boxSizing: "border-box",
      minHeight: "100%",
      padding: "4px 0",
      outline: "none",
      "&[contenteditable=true]": {
        WebkitUserModify: "read-write-plaintext-only"
      }
    },
    ".cm-lineWrapping": {
      whiteSpace_fallback: "pre-wrap",
      // For IE
      whiteSpace: "break-spaces",
      wordBreak: "break-word",
      // For Safari, which doesn't support overflow-wrap: anywhere
      overflowWrap: "anywhere",
      flexShrink: 1
    },
    "&light .cm-content": { caretColor: "black" },
    "&dark .cm-content": { caretColor: "white" },
    ".cm-line": {
      display: "block",
      padding: "0 2px 0 6px"
    },
    ".cm-layer": {
      position: "absolute",
      left: 0,
      top: 0,
      contain: "size style",
      "& > *": {
        position: "absolute"
      }
    },
    "&light .cm-selectionBackground": {
      background: "#d9d9d9"
    },
    "&dark .cm-selectionBackground": {
      background: "#222"
    },
    "&light.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground": {
      background: "#d7d4f0"
    },
    "&dark.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground": {
      background: "#233"
    },
    ".cm-cursorLayer": {
      pointerEvents: "none"
    },
    "&.cm-focused > .cm-scroller > .cm-cursorLayer": {
      animation: "steps(1) cm-blink 1.2s infinite"
    },
    // Two animations defined so that we can switch between them to
    // restart the animation without forcing another style
    // recomputation.
    "@keyframes cm-blink": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
    "@keyframes cm-blink2": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
    ".cm-cursor, .cm-dropCursor": {
      borderLeft: "1.2px solid black",
      marginLeft: "-0.6px",
      pointerEvents: "none"
    },
    ".cm-cursor": {
      display: "none"
    },
    "&dark .cm-cursor": {
      borderLeftColor: "#444"
    },
    ".cm-dropCursor": {
      position: "absolute"
    },
    "&.cm-focused > .cm-scroller > .cm-cursorLayer .cm-cursor": {
      display: "block"
    },
    ".cm-iso": {
      unicodeBidi: "isolate"
    },
    ".cm-announced": {
      position: "fixed",
      top: "-10000px"
    },
    "@media print": {
      ".cm-announced": { display: "none" }
    },
    "&light .cm-activeLine": { backgroundColor: "#cceeff44" },
    "&dark .cm-activeLine": { backgroundColor: "#99eeff33" },
    "&light .cm-specialChar": { color: "red" },
    "&dark .cm-specialChar": { color: "#f78" },
    ".cm-gutters": {
      flexShrink: 0,
      display: "flex",
      height: "100%",
      boxSizing: "border-box",
      insetInlineStart: 0,
      zIndex: 200
    },
    "&light .cm-gutters": {
      backgroundColor: "#f5f5f5",
      color: "#6c6c6c",
      borderRight: "1px solid #ddd"
    },
    "&dark .cm-gutters": {
      backgroundColor: "#333338",
      color: "#ccc"
    },
    ".cm-gutter": {
      display: "flex !important",
      // Necessary -- prevents margin collapsing
      flexDirection: "column",
      flexShrink: 0,
      boxSizing: "border-box",
      minHeight: "100%",
      overflow: "hidden"
    },
    ".cm-gutterElement": {
      boxSizing: "border-box"
    },
    ".cm-lineNumbers .cm-gutterElement": {
      padding: "0 3px 0 5px",
      minWidth: "20px",
      textAlign: "right",
      whiteSpace: "nowrap"
    },
    "&light .cm-activeLineGutter": {
      backgroundColor: "#e2f2ff"
    },
    "&dark .cm-activeLineGutter": {
      backgroundColor: "#222227"
    },
    ".cm-panels": {
      boxSizing: "border-box",
      position: "sticky",
      left: 0,
      right: 0,
      zIndex: 300
    },
    "&light .cm-panels": {
      backgroundColor: "#f5f5f5",
      color: "black"
    },
    "&light .cm-panels-top": {
      borderBottom: "1px solid #ddd"
    },
    "&light .cm-panels-bottom": {
      borderTop: "1px solid #ddd"
    },
    "&dark .cm-panels": {
      backgroundColor: "#333338",
      color: "white"
    },
    ".cm-tab": {
      display: "inline-block",
      overflow: "hidden",
      verticalAlign: "bottom"
    },
    ".cm-widgetBuffer": {
      verticalAlign: "text-top",
      height: "1em",
      width: 0,
      display: "inline"
    },
    ".cm-placeholder": {
      color: "#888",
      display: "inline-block",
      verticalAlign: "top"
    },
    ".cm-highlightSpace": {
      backgroundImage: "radial-gradient(circle at 50% 55%, #aaa 20%, transparent 5%)",
      backgroundPosition: "center"
    },
    ".cm-highlightTab": {
      backgroundImage: `url('data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" width="200" height="20"><path stroke="%23888" stroke-width="1" fill="none" d="M1 10H196L190 5M190 15L196 10M197 4L197 16"/></svg>')`,
      backgroundSize: "auto 100%",
      backgroundPosition: "right 90%",
      backgroundRepeat: "no-repeat"
    },
    ".cm-trailingSpace": {
      backgroundColor: "#ff332255"
    },
    ".cm-button": {
      verticalAlign: "middle",
      color: "inherit",
      fontSize: "70%",
      padding: ".2em 1em",
      borderRadius: "1px"
    },
    "&light .cm-button": {
      backgroundImage: "linear-gradient(#eff1f5, #d9d9df)",
      border: "1px solid #888",
      "&:active": {
        backgroundImage: "linear-gradient(#b4b4b4, #d0d3d6)"
      }
    },
    "&dark .cm-button": {
      backgroundImage: "linear-gradient(#393939, #111)",
      border: "1px solid #888",
      "&:active": {
        backgroundImage: "linear-gradient(#111, #333)"
      }
    },
    ".cm-textfield": {
      verticalAlign: "middle",
      color: "inherit",
      fontSize: "70%",
      border: "1px solid silver",
      padding: ".2em .5em"
    },
    "&light .cm-textfield": {
      backgroundColor: "white"
    },
    "&dark .cm-textfield": {
      border: "1px solid #555",
      backgroundColor: "inherit"
    }
  }, lightDarkIDs), observeOptions = {
    childList: !0,
    characterData: !0,
    subtree: !0,
    attributes: !0,
    characterDataOldValue: !0
  }, useCharData = browser.ie && browser.ie_version <= 11;
  class DOMObserver {
    constructor(_) {
      this.view = _, this.active = !1, this.editContext = null, this.selectionRange = new DOMSelectionState(), this.selectionChanged = !1, this.delayedFlush = -1, this.resizeTimeout = -1, this.queue = [], this.delayedAndroidKey = null, this.flushingAndroidKey = -1, this.lastChange = 0, this.scrollTargets = [], this.intersection = null, this.resizeScroll = null, this.intersecting = !1, this.gapIntersection = null, this.gaps = [], this.printQuery = null, this.parentCheck = -1, this.dom = _.contentDOM, this.observer = new MutationObserver((H) => {
        for (let V of H)
          this.queue.push(V);
        (browser.ie && browser.ie_version <= 11 || browser.ios && _.composing) && H.some((V) => V.type == "childList" && V.removedNodes.length || V.type == "characterData" && V.oldValue.length > V.target.nodeValue.length) ? this.flushSoon() : this.flush();
      }), window.EditContext && _.constructor.EDIT_CONTEXT !== !1 && // Chrome <126 doesn't support inverted selections in edit context (#1392)
      !(browser.chrome && browser.chrome_version < 126) && (this.editContext = new EditContextManager(_), _.state.facet(editable) && (_.contentDOM.editContext = this.editContext.editContext)), useCharData && (this.onCharData = (H) => {
        this.queue.push({
          target: H.target,
          type: "characterData",
          oldValue: H.prevValue
        }), this.flushSoon();
      }), this.onSelectionChange = this.onSelectionChange.bind(this), this.onResize = this.onResize.bind(this), this.onPrint = this.onPrint.bind(this), this.onScroll = this.onScroll.bind(this), window.matchMedia && (this.printQuery = window.matchMedia("print")), typeof ResizeObserver == "function" && (this.resizeScroll = new ResizeObserver(() => {
        var H;
        ((H = this.view.docView) === null || H === void 0 ? void 0 : H.lastUpdate) < Date.now() - 75 && this.onResize();
      }), this.resizeScroll.observe(_.scrollDOM)), this.addWindowListeners(this.win = _.win), this.start(), typeof IntersectionObserver == "function" && (this.intersection = new IntersectionObserver((H) => {
        this.parentCheck < 0 && (this.parentCheck = setTimeout(this.listenForScroll.bind(this), 1e3)), H.length > 0 && H[H.length - 1].intersectionRatio > 0 != this.intersecting && (this.intersecting = !this.intersecting, this.intersecting != this.view.inView && this.onScrollChanged(document.createEvent("Event")));
      }, { threshold: [0, 1e-3] }), this.intersection.observe(this.dom), this.gapIntersection = new IntersectionObserver((H) => {
        H.length > 0 && H[H.length - 1].intersectionRatio > 0 && this.onScrollChanged(document.createEvent("Event"));
      }, {})), this.listenForScroll(), this.readSelectionRange();
    }
    onScrollChanged(_) {
      this.view.inputState.runHandlers("scroll", _), this.intersecting && this.view.measure();
    }
    onScroll(_) {
      this.intersecting && this.flush(!1), this.editContext && this.view.requestMeasure(this.editContext.measureReq), this.onScrollChanged(_);
    }
    onResize() {
      this.resizeTimeout < 0 && (this.resizeTimeout = setTimeout(() => {
        this.resizeTimeout = -1, this.view.requestMeasure();
      }, 50));
    }
    onPrint(_) {
      (_.type == "change" || !_.type) && !_.matches || (this.view.viewState.printing = !0, this.view.measure(), setTimeout(() => {
        this.view.viewState.printing = !1, this.view.requestMeasure();
      }, 500));
    }
    updateGaps(_) {
      if (this.gapIntersection && (_.length != this.gaps.length || this.gaps.some((H, V) => H != _[V]))) {
        this.gapIntersection.disconnect();
        for (let H of _)
          this.gapIntersection.observe(H);
        this.gaps = _;
      }
    }
    onSelectionChange(_) {
      let H = this.selectionChanged;
      if (!this.readSelectionRange() || this.delayedAndroidKey)
        return;
      let { view: V } = this, q = this.selectionRange;
      if (V.state.facet(editable) ? V.root.activeElement != this.dom : !hasSelection(this.dom, q))
        return;
      let G = q.anchorNode && V.docView.nearest(q.anchorNode);
      if (G && G.ignoreEvent(_)) {
        H || (this.selectionChanged = !1);
        return;
      }
      (browser.ie && browser.ie_version <= 11 || browser.android && browser.chrome) && !V.state.selection.main.empty && // (Selection.isCollapsed isn't reliable on IE)
      q.focusNode && isEquivalentPosition(q.focusNode, q.focusOffset, q.anchorNode, q.anchorOffset) ? this.flushSoon() : this.flush(!1);
    }
    readSelectionRange() {
      let { view: _ } = this, H = getSelection(_.root);
      if (!H)
        return !1;
      let V = browser.safari && _.root.nodeType == 11 && _.root.activeElement == this.dom && safariSelectionRangeHack(this.view, H) || H;
      if (!V || this.selectionRange.eq(V))
        return !1;
      let q = hasSelection(this.dom, V);
      return q && !this.selectionChanged && _.inputState.lastFocusTime > Date.now() - 200 && _.inputState.lastTouchTime < Date.now() - 300 && atElementStart(this.dom, V) ? (this.view.inputState.lastFocusTime = 0, _.docView.updateSelection(), !1) : (this.selectionRange.setRange(V), q && (this.selectionChanged = !0), !0);
    }
    setSelectionRange(_, H) {
      this.selectionRange.set(_.node, _.offset, H.node, H.offset), this.selectionChanged = !1;
    }
    clearSelectionRange() {
      this.selectionRange.set(null, 0, null, 0);
    }
    listenForScroll() {
      this.parentCheck = -1;
      let _ = 0, H = null;
      for (let V = this.dom; V; )
        if (V.nodeType == 1)
          !H && _ < this.scrollTargets.length && this.scrollTargets[_] == V ? _++ : H || (H = this.scrollTargets.slice(0, _)), H && H.push(V), V = V.assignedSlot || V.parentNode;
        else if (V.nodeType == 11)
          V = V.host;
        else
          break;
      if (_ < this.scrollTargets.length && !H && (H = this.scrollTargets.slice(0, _)), H) {
        for (let V of this.scrollTargets)
          V.removeEventListener("scroll", this.onScroll);
        for (let V of this.scrollTargets = H)
          V.addEventListener("scroll", this.onScroll);
      }
    }
    ignore(_) {
      if (!this.active)
        return _();
      try {
        return this.stop(), _();
      } finally {
        this.start(), this.clear();
      }
    }
    start() {
      this.active || (this.observer.observe(this.dom, observeOptions), useCharData && this.dom.addEventListener("DOMCharacterDataModified", this.onCharData), this.active = !0);
    }
    stop() {
      this.active && (this.active = !1, this.observer.disconnect(), useCharData && this.dom.removeEventListener("DOMCharacterDataModified", this.onCharData));
    }
    // Throw away any pending changes
    clear() {
      this.processRecords(), this.queue.length = 0, this.selectionChanged = !1;
    }
    // Chrome Android, especially in combination with GBoard, not only
    // doesn't reliably fire regular key events, but also often
    // surrounds the effect of enter or backspace with a bunch of
    // composition events that, when interrupted, cause text duplication
    // or other kinds of corruption. This hack makes the editor back off
    // from handling DOM changes for a moment when such a key is
    // detected (via beforeinput or keydown), and then tries to flush
    // them or, if that has no effect, dispatches the given key.
    delayAndroidKey(_, H) {
      var V;
      if (!this.delayedAndroidKey) {
        let q = () => {
          let G = this.delayedAndroidKey;
          G && (this.clearDelayedAndroidKey(), this.view.inputState.lastKeyCode = G.keyCode, this.view.inputState.lastKeyTime = Date.now(), !this.flush() && G.force && dispatchKey(this.dom, G.key, G.keyCode));
        };
        this.flushingAndroidKey = this.view.win.requestAnimationFrame(q);
      }
      (!this.delayedAndroidKey || _ == "Enter") && (this.delayedAndroidKey = {
        key: _,
        keyCode: H,
        // Only run the key handler when no changes are detected if
        // this isn't coming right after another change, in which case
        // it is probably part of a weird chain of updates, and should
        // be ignored if it returns the DOM to its previous state.
        force: this.lastChange < Date.now() - 50 || !!(!((V = this.delayedAndroidKey) === null || V === void 0) && V.force)
      });
    }
    clearDelayedAndroidKey() {
      this.win.cancelAnimationFrame(this.flushingAndroidKey), this.delayedAndroidKey = null, this.flushingAndroidKey = -1;
    }
    flushSoon() {
      this.delayedFlush < 0 && (this.delayedFlush = this.view.win.requestAnimationFrame(() => {
        this.delayedFlush = -1, this.flush();
      }));
    }
    forceFlush() {
      this.delayedFlush >= 0 && (this.view.win.cancelAnimationFrame(this.delayedFlush), this.delayedFlush = -1), this.flush();
    }
    pendingRecords() {
      for (let _ of this.observer.takeRecords())
        this.queue.push(_);
      return this.queue;
    }
    processRecords() {
      let _ = this.pendingRecords();
      _.length && (this.queue = []);
      let H = -1, V = -1, q = !1;
      for (let G of _) {
        let J = this.readMutation(G);
        J && (J.typeOver && (q = !0), H == -1 ? { from: H, to: V } = J : (H = Math.min(J.from, H), V = Math.max(J.to, V)));
      }
      return { from: H, to: V, typeOver: q };
    }
    readChange() {
      let { from: _, to: H, typeOver: V } = this.processRecords(), q = this.selectionChanged && hasSelection(this.dom, this.selectionRange);
      if (_ < 0 && !q)
        return null;
      _ > -1 && (this.lastChange = Date.now()), this.view.inputState.lastFocusTime = 0, this.selectionChanged = !1;
      let G = new DOMChange(this.view, _, H, V);
      return this.view.docView.domChanged = { newSel: G.newSel ? G.newSel.main : null }, G;
    }
    // Apply pending changes, if any
    flush(_ = !0) {
      if (this.delayedFlush >= 0 || this.delayedAndroidKey)
        return !1;
      _ && this.readSelectionRange();
      let H = this.readChange();
      if (!H)
        return this.view.requestMeasure(), !1;
      let V = this.view.state, q = applyDOMChange(this.view, H);
      return this.view.state == V && (H.domChanged || H.newSel && !H.newSel.main.eq(this.view.state.selection.main)) && this.view.update([]), q;
    }
    readMutation(_) {
      let H = this.view.docView.nearest(_.target);
      if (!H || H.ignoreMutation(_))
        return null;
      if (H.markDirty(_.type == "attributes"), _.type == "attributes" && (H.flags |= 4), _.type == "childList") {
        let V = findChild(H, _.previousSibling || _.target.previousSibling, -1), q = findChild(H, _.nextSibling || _.target.nextSibling, 1);
        return {
          from: V ? H.posAfter(V) : H.posAtStart,
          to: q ? H.posBefore(q) : H.posAtEnd,
          typeOver: !1
        };
      } else return _.type == "characterData" ? { from: H.posAtStart, to: H.posAtEnd, typeOver: _.target.nodeValue == _.oldValue } : null;
    }
    setWindow(_) {
      _ != this.win && (this.removeWindowListeners(this.win), this.win = _, this.addWindowListeners(this.win));
    }
    addWindowListeners(_) {
      _.addEventListener("resize", this.onResize), this.printQuery ? this.printQuery.addEventListener ? this.printQuery.addEventListener("change", this.onPrint) : this.printQuery.addListener(this.onPrint) : _.addEventListener("beforeprint", this.onPrint), _.addEventListener("scroll", this.onScroll), _.document.addEventListener("selectionchange", this.onSelectionChange);
    }
    removeWindowListeners(_) {
      _.removeEventListener("scroll", this.onScroll), _.removeEventListener("resize", this.onResize), this.printQuery ? this.printQuery.removeEventListener ? this.printQuery.removeEventListener("change", this.onPrint) : this.printQuery.removeListener(this.onPrint) : _.removeEventListener("beforeprint", this.onPrint), _.document.removeEventListener("selectionchange", this.onSelectionChange);
    }
    update(_) {
      this.editContext && (this.editContext.update(_), _.startState.facet(editable) != _.state.facet(editable) && (_.view.contentDOM.editContext = _.state.facet(editable) ? this.editContext.editContext : null));
    }
    destroy() {
      var _, H, V;
      this.stop(), (_ = this.intersection) === null || _ === void 0 || _.disconnect(), (H = this.gapIntersection) === null || H === void 0 || H.disconnect(), (V = this.resizeScroll) === null || V === void 0 || V.disconnect();
      for (let q of this.scrollTargets)
        q.removeEventListener("scroll", this.onScroll);
      this.removeWindowListeners(this.win), clearTimeout(this.parentCheck), clearTimeout(this.resizeTimeout), this.win.cancelAnimationFrame(this.delayedFlush), this.win.cancelAnimationFrame(this.flushingAndroidKey), this.editContext && (this.view.contentDOM.editContext = null, this.editContext.destroy());
    }
  }
  function findChild(O, _, H) {
    for (; _; ) {
      let V = ContentView.get(_);
      if (V && V.parent == O)
        return V;
      let q = _.parentNode;
      _ = q != O.dom ? q : H > 0 ? _.nextSibling : _.previousSibling;
    }
    return null;
  }
  function buildSelectionRangeFromRange(O, _) {
    let H = _.startContainer, V = _.startOffset, q = _.endContainer, G = _.endOffset, J = O.docView.domAtPos(O.state.selection.main.anchor);
    return isEquivalentPosition(J.node, J.offset, q, G) && ([H, V, q, G] = [q, G, H, V]), { anchorNode: H, anchorOffset: V, focusNode: q, focusOffset: G };
  }
  function safariSelectionRangeHack(O, _) {
    if (_.getComposedRanges) {
      let q = _.getComposedRanges(O.root)[0];
      if (q)
        return buildSelectionRangeFromRange(O, q);
    }
    let H = null;
    function V(q) {
      q.preventDefault(), q.stopImmediatePropagation(), H = q.getTargetRanges()[0];
    }
    return O.contentDOM.addEventListener("beforeinput", V, !0), O.dom.ownerDocument.execCommand("indent"), O.contentDOM.removeEventListener("beforeinput", V, !0), H ? buildSelectionRangeFromRange(O, H) : null;
  }
  class EditContextManager {
    constructor(_) {
      this.from = 0, this.to = 0, this.pendingContextChange = null, this.handlers = /* @__PURE__ */ Object.create(null), this.resetRange(_.state);
      let H = this.editContext = new window.EditContext({
        text: _.state.doc.sliceString(this.from, this.to),
        selectionStart: this.toContextPos(Math.max(this.from, Math.min(this.to, _.state.selection.main.anchor))),
        selectionEnd: this.toContextPos(_.state.selection.main.head)
      });
      this.handlers.textupdate = (V) => {
        let { anchor: q } = _.state.selection.main, G = {
          from: this.toEditorPos(V.updateRangeStart),
          to: this.toEditorPos(V.updateRangeEnd),
          insert: Text.of(V.text.split(`
`))
        };
        G.from == this.from && q < this.from ? G.from = q : G.to == this.to && q > this.to && (G.to = q), !(G.from == G.to && !G.insert.length) && (this.pendingContextChange = G, _.state.readOnly || applyDOMChangeInner(_, G, EditorSelection.single(this.toEditorPos(V.selectionStart), this.toEditorPos(V.selectionEnd))), this.pendingContextChange && (this.revertPending(_.state), this.setSelection(_.state)));
      }, this.handlers.characterboundsupdate = (V) => {
        let q = [], G = null;
        for (let J = this.toEditorPos(V.rangeStart), Z = this.toEditorPos(V.rangeEnd); J < Z; J++) {
          let te = _.coordsForChar(J);
          G = te && new DOMRect(te.left, te.top, te.right - te.left, te.bottom - te.top) || G || new DOMRect(), q.push(G);
        }
        H.updateCharacterBounds(V.rangeStart, q);
      }, this.handlers.textformatupdate = (V) => {
        let q = [];
        for (let G of V.getTextFormats()) {
          let J = G.underlineStyle, Z = G.underlineThickness;
          if (J != "None" && Z != "None") {
            let te = `text-decoration: underline ${J == "Dashed" ? "dashed " : J == "Squiggle" ? "wavy " : ""}${Z == "Thin" ? 1 : 2}px`;
            q.push(Decoration.mark({ attributes: { style: te } }).range(this.toEditorPos(G.rangeStart), this.toEditorPos(G.rangeEnd)));
          }
        }
        _.dispatch({ effects: setEditContextFormatting.of(Decoration.set(q)) });
      }, this.handlers.compositionstart = () => {
        _.inputState.composing < 0 && (_.inputState.composing = 0, _.inputState.compositionFirstChange = !0);
      }, this.handlers.compositionend = () => {
        _.inputState.composing = -1, _.inputState.compositionFirstChange = null;
      };
      for (let V in this.handlers)
        H.addEventListener(V, this.handlers[V]);
      this.measureReq = { read: (V) => {
        this.editContext.updateControlBounds(V.contentDOM.getBoundingClientRect());
        let q = getSelection(V.root);
        q && q.rangeCount && this.editContext.updateSelectionBounds(q.getRangeAt(0).getBoundingClientRect());
      } };
    }
    applyEdits(_) {
      let H = 0, V = !1, q = this.pendingContextChange;
      return _.changes.iterChanges((G, J, Z, te, ne) => {
        if (V)
          return;
        let ee = ne.length - (J - G);
        if (q && J >= q.to)
          if (q.from == G && q.to == J && q.insert.eq(ne)) {
            q = this.pendingContextChange = null, H += ee, this.to += ee;
            return;
          } else
            q = null, this.revertPending(_.state);
        if (G += H, J += H, J <= this.from)
          this.from += ee, this.to += ee;
        else if (G < this.to) {
          if (G < this.from || J > this.to || this.to - this.from + ne.length > 3e4) {
            V = !0;
            return;
          }
          this.editContext.updateText(this.toContextPos(G), this.toContextPos(J), ne.toString()), this.to += ee;
        }
        H += ee;
      }), q && !V && this.revertPending(_.state), !V;
    }
    update(_) {
      let H = this.pendingContextChange;
      !this.applyEdits(_) || !this.rangeIsValid(_.state) ? (this.pendingContextChange = null, this.resetRange(_.state), this.editContext.updateText(0, this.editContext.text.length, _.state.doc.sliceString(this.from, this.to)), this.setSelection(_.state)) : (_.docChanged || _.selectionSet || H) && this.setSelection(_.state), (_.geometryChanged || _.docChanged || _.selectionSet) && _.view.requestMeasure(this.measureReq);
    }
    resetRange(_) {
      let { head: H } = _.selection.main;
      this.from = Math.max(
        0,
        H - 1e4
        /* CxVp.Margin */
      ), this.to = Math.min(
        _.doc.length,
        H + 1e4
        /* CxVp.Margin */
      );
    }
    revertPending(_) {
      let H = this.pendingContextChange;
      this.pendingContextChange = null, this.editContext.updateText(this.toContextPos(H.from), this.toContextPos(H.from + H.insert.length), _.doc.sliceString(H.from, H.to));
    }
    setSelection(_) {
      let { main: H } = _.selection, V = this.toContextPos(Math.max(this.from, Math.min(this.to, H.anchor))), q = this.toContextPos(H.head);
      (this.editContext.selectionStart != V || this.editContext.selectionEnd != q) && this.editContext.updateSelection(V, q);
    }
    rangeIsValid(_) {
      let { head: H } = _.selection.main;
      return !(this.from > 0 && H - this.from < 500 || this.to < _.doc.length && this.to - H < 500 || this.to - this.from > 1e4 * 3);
    }
    toEditorPos(_) {
      return _ + this.from;
    }
    toContextPos(_) {
      return _ - this.from;
    }
    destroy() {
      for (let _ in this.handlers)
        this.editContext.removeEventListener(_, this.handlers[_]);
    }
  }
  class EditorView {
    /**
    The current editor state.
    */
    get state() {
      return this.viewState.state;
    }
    /**
    To be able to display large documents without consuming too much
    memory or overloading the browser, CodeMirror only draws the
    code that is visible (plus a margin around it) to the DOM. This
    property tells you the extent of the current drawn viewport, in
    document positions.
    */
    get viewport() {
      return this.viewState.viewport;
    }
    /**
    When there are, for example, large collapsed ranges in the
    viewport, its size can be a lot bigger than the actual visible
    content. Thus, if you are doing something like styling the
    content in the viewport, it is preferable to only do so for
    these ranges, which are the subset of the viewport that is
    actually drawn.
    */
    get visibleRanges() {
      return this.viewState.visibleRanges;
    }
    /**
    Returns false when the editor is entirely scrolled out of view
    or otherwise hidden.
    */
    get inView() {
      return this.viewState.inView;
    }
    /**
    Indicates whether the user is currently composing text via
    [IME](https://en.wikipedia.org/wiki/Input_method), and at least
    one change has been made in the current composition.
    */
    get composing() {
      return this.inputState.composing > 0;
    }
    /**
    Indicates whether the user is currently in composing state. Note
    that on some platforms, like Android, this will be the case a
    lot, since just putting the cursor on a word starts a
    composition there.
    */
    get compositionStarted() {
      return this.inputState.composing >= 0;
    }
    /**
    The document or shadow root that the view lives in.
    */
    get root() {
      return this._root;
    }
    /**
    @internal
    */
    get win() {
      return this.dom.ownerDocument.defaultView || window;
    }
    /**
    Construct a new view. You'll want to either provide a `parent`
    option, or put `view.dom` into your document after creating a
    view, so that the user can see the editor.
    */
    constructor(_ = {}) {
      var H;
      this.plugins = [], this.pluginMap = /* @__PURE__ */ new Map(), this.editorAttrs = {}, this.contentAttrs = {}, this.bidiCache = [], this.destroyed = !1, this.updateState = 2, this.measureScheduled = -1, this.measureRequests = [], this.contentDOM = document.createElement("div"), this.scrollDOM = document.createElement("div"), this.scrollDOM.tabIndex = -1, this.scrollDOM.className = "cm-scroller", this.scrollDOM.appendChild(this.contentDOM), this.announceDOM = document.createElement("div"), this.announceDOM.className = "cm-announced", this.announceDOM.setAttribute("aria-live", "polite"), this.dom = document.createElement("div"), this.dom.appendChild(this.announceDOM), this.dom.appendChild(this.scrollDOM), _.parent && _.parent.appendChild(this.dom);
      let { dispatch: V } = _;
      this.dispatchTransactions = _.dispatchTransactions || V && ((q) => q.forEach((G) => V(G, this))) || ((q) => this.update(q)), this.dispatch = this.dispatch.bind(this), this._root = _.root || getRoot(_.parent) || document, this.viewState = new ViewState(_.state || EditorState.create(_)), _.scrollTo && _.scrollTo.is(scrollIntoView$1) && (this.viewState.scrollTarget = _.scrollTo.value.clip(this.viewState.state)), this.plugins = this.state.facet(viewPlugin).map((q) => new PluginInstance(q));
      for (let q of this.plugins)
        q.update(this);
      this.observer = new DOMObserver(this), this.inputState = new InputState(this), this.inputState.ensureHandlers(this.plugins), this.docView = new DocView(this), this.mountStyles(), this.updateAttrs(), this.updateState = 0, this.requestMeasure(), !((H = document.fonts) === null || H === void 0) && H.ready && document.fonts.ready.then(() => this.requestMeasure());
    }
    dispatch(..._) {
      let H = _.length == 1 && _[0] instanceof Transaction ? _ : _.length == 1 && Array.isArray(_[0]) ? _[0] : [this.state.update(..._)];
      this.dispatchTransactions(H, this);
    }
    /**
    Update the view for the given array of transactions. This will
    update the visible document and selection to match the state
    produced by the transactions, and notify view plugins of the
    change. You should usually call
    [`dispatch`](https://codemirror.net/6/docs/ref/#view.EditorView.dispatch) instead, which uses this
    as a primitive.
    */
    update(_) {
      if (this.updateState != 0)
        throw new Error("Calls to EditorView.update are not allowed while an update is in progress");
      let H = !1, V = !1, q, G = this.state;
      for (let oe of _) {
        if (oe.startState != G)
          throw new RangeError("Trying to update state with a transaction that doesn't start from the previous state.");
        G = oe.state;
      }
      if (this.destroyed) {
        this.viewState.state = G;
        return;
      }
      let J = this.hasFocus, Z = 0, te = null;
      _.some((oe) => oe.annotation(isFocusChange)) ? (this.inputState.notifiedFocused = J, Z = 1) : J != this.inputState.notifiedFocused && (this.inputState.notifiedFocused = J, te = focusChangeTransaction(G, J), te || (Z = 1));
      let ne = this.observer.delayedAndroidKey, ee = null;
      if (ne ? (this.observer.clearDelayedAndroidKey(), ee = this.observer.readChange(), (ee && !this.state.doc.eq(G.doc) || !this.state.selection.eq(G.selection)) && (ee = null)) : this.observer.clear(), G.facet(EditorState.phrases) != this.state.facet(EditorState.phrases))
        return this.setState(G);
      q = ViewUpdate.create(this, G, _), q.flags |= Z;
      let ae = this.viewState.scrollTarget;
      try {
        this.updateState = 2;
        for (let oe of _) {
          if (ae && (ae = ae.map(oe.changes)), oe.scrollIntoView) {
            let { main: se } = oe.state.selection;
            ae = new ScrollTarget(se.empty ? se : EditorSelection.cursor(se.head, se.head > se.anchor ? -1 : 1));
          }
          for (let se of oe.effects)
            se.is(scrollIntoView$1) && (ae = se.value.clip(this.state));
        }
        this.viewState.update(q, ae), this.bidiCache = CachedOrder.update(this.bidiCache, q.changes), q.empty || (this.updatePlugins(q), this.inputState.update(q)), H = this.docView.update(q), this.state.facet(styleModule) != this.styleModules && this.mountStyles(), V = this.updateAttrs(), this.showAnnouncements(_), this.docView.updateSelection(H, _.some((oe) => oe.isUserEvent("select.pointer")));
      } finally {
        this.updateState = 0;
      }
      if (q.startState.facet(theme) != q.state.facet(theme) && (this.viewState.mustMeasureContent = !0), (H || V || ae || this.viewState.mustEnforceCursorAssoc || this.viewState.mustMeasureContent) && this.requestMeasure(), H && this.docViewUpdate(), !q.empty)
        for (let oe of this.state.facet(updateListener))
          try {
            oe(q);
          } catch (se) {
            logException(this.state, se, "update listener");
          }
      (te || ee) && Promise.resolve().then(() => {
        te && this.state == te.startState && this.dispatch(te), ee && !applyDOMChange(this, ee) && ne.force && dispatchKey(this.contentDOM, ne.key, ne.keyCode);
      });
    }
    /**
    Reset the view to the given state. (This will cause the entire
    document to be redrawn and all view plugins to be reinitialized,
    so you should probably only use it when the new state isn't
    derived from the old state. Otherwise, use
    [`dispatch`](https://codemirror.net/6/docs/ref/#view.EditorView.dispatch) instead.)
    */
    setState(_) {
      if (this.updateState != 0)
        throw new Error("Calls to EditorView.setState are not allowed while an update is in progress");
      if (this.destroyed) {
        this.viewState.state = _;
        return;
      }
      this.updateState = 2;
      let H = this.hasFocus;
      try {
        for (let V of this.plugins)
          V.destroy(this);
        this.viewState = new ViewState(_), this.plugins = _.facet(viewPlugin).map((V) => new PluginInstance(V)), this.pluginMap.clear();
        for (let V of this.plugins)
          V.update(this);
        this.docView.destroy(), this.docView = new DocView(this), this.inputState.ensureHandlers(this.plugins), this.mountStyles(), this.updateAttrs(), this.bidiCache = [];
      } finally {
        this.updateState = 0;
      }
      H && this.focus(), this.requestMeasure();
    }
    updatePlugins(_) {
      let H = _.startState.facet(viewPlugin), V = _.state.facet(viewPlugin);
      if (H != V) {
        let q = [];
        for (let G of V) {
          let J = H.indexOf(G);
          if (J < 0)
            q.push(new PluginInstance(G));
          else {
            let Z = this.plugins[J];
            Z.mustUpdate = _, q.push(Z);
          }
        }
        for (let G of this.plugins)
          G.mustUpdate != _ && G.destroy(this);
        this.plugins = q, this.pluginMap.clear();
      } else
        for (let q of this.plugins)
          q.mustUpdate = _;
      for (let q = 0; q < this.plugins.length; q++)
        this.plugins[q].update(this);
      H != V && this.inputState.ensureHandlers(this.plugins);
    }
    docViewUpdate() {
      for (let _ of this.plugins) {
        let H = _.value;
        if (H && H.docViewUpdate)
          try {
            H.docViewUpdate(this);
          } catch (V) {
            logException(this.state, V, "doc view update listener");
          }
      }
    }
    /**
    @internal
    */
    measure(_ = !0) {
      if (this.destroyed)
        return;
      if (this.measureScheduled > -1 && this.win.cancelAnimationFrame(this.measureScheduled), this.observer.delayedAndroidKey) {
        this.measureScheduled = -1, this.requestMeasure();
        return;
      }
      this.measureScheduled = 0, _ && this.observer.forceFlush();
      let H = null, V = this.scrollDOM, q = V.scrollTop * this.scaleY, { scrollAnchorPos: G, scrollAnchorHeight: J } = this.viewState;
      Math.abs(q - this.viewState.scrollTop) > 1 && (J = -1), this.viewState.scrollAnchorHeight = -1;
      try {
        for (let Z = 0; ; Z++) {
          if (J < 0)
            if (isScrolledToBottom(V))
              G = -1, J = this.viewState.heightMap.height;
            else {
              let se = this.viewState.scrollAnchorAt(q);
              G = se.from, J = se.top;
            }
          this.updateState = 1;
          let te = this.viewState.measure(this);
          if (!te && !this.measureRequests.length && this.viewState.scrollTarget == null)
            break;
          if (Z > 5) {
            console.warn(this.measureRequests.length ? "Measure loop restarted more than 5 times" : "Viewport failed to stabilize");
            break;
          }
          let ne = [];
          te & 4 || ([this.measureRequests, ne] = [ne, this.measureRequests]);
          let ee = ne.map((se) => {
            try {
              return se.read(this);
            } catch (ce) {
              return logException(this.state, ce), BadMeasure;
            }
          }), ae = ViewUpdate.create(this, this.state, []), oe = !1;
          ae.flags |= te, H ? H.flags |= te : H = ae, this.updateState = 2, ae.empty || (this.updatePlugins(ae), this.inputState.update(ae), this.updateAttrs(), oe = this.docView.update(ae), oe && this.docViewUpdate());
          for (let se = 0; se < ne.length; se++)
            if (ee[se] != BadMeasure)
              try {
                let ce = ne[se];
                ce.write && ce.write(ee[se], this);
              } catch (ce) {
                logException(this.state, ce);
              }
          if (oe && this.docView.updateSelection(!0), !ae.viewportChanged && this.measureRequests.length == 0) {
            if (this.viewState.editorHeight)
              if (this.viewState.scrollTarget) {
                this.docView.scrollIntoView(this.viewState.scrollTarget), this.viewState.scrollTarget = null, J = -1;
                continue;
              } else {
                let ce = (G < 0 ? this.viewState.heightMap.height : this.viewState.lineBlockAt(G).top) - J;
                if (ce > 1 || ce < -1) {
                  q = q + ce, V.scrollTop = q / this.scaleY, J = -1;
                  continue;
                }
              }
            break;
          }
        }
      } finally {
        this.updateState = 0, this.measureScheduled = -1;
      }
      if (H && !H.empty)
        for (let Z of this.state.facet(updateListener))
          Z(H);
    }
    /**
    Get the CSS classes for the currently active editor themes.
    */
    get themeClasses() {
      return baseThemeID + " " + (this.state.facet(darkTheme) ? baseDarkID : baseLightID) + " " + this.state.facet(theme);
    }
    updateAttrs() {
      let _ = attrsFromFacet(this, editorAttributes, {
        class: "cm-editor" + (this.hasFocus ? " cm-focused " : " ") + this.themeClasses
      }), H = {
        spellcheck: "false",
        autocorrect: "off",
        autocapitalize: "off",
        translate: "no",
        contenteditable: this.state.facet(editable) ? "true" : "false",
        class: "cm-content",
        style: `${browser.tabSize}: ${this.state.tabSize}`,
        role: "textbox",
        "aria-multiline": "true"
      };
      this.state.readOnly && (H["aria-readonly"] = "true"), attrsFromFacet(this, contentAttributes, H);
      let V = this.observer.ignore(() => {
        let q = updateAttrs(this.contentDOM, this.contentAttrs, H), G = updateAttrs(this.dom, this.editorAttrs, _);
        return q || G;
      });
      return this.editorAttrs = _, this.contentAttrs = H, V;
    }
    showAnnouncements(_) {
      let H = !0;
      for (let V of _)
        for (let q of V.effects)
          if (q.is(EditorView.announce)) {
            H && (this.announceDOM.textContent = ""), H = !1;
            let G = this.announceDOM.appendChild(document.createElement("div"));
            G.textContent = q.value;
          }
    }
    mountStyles() {
      this.styleModules = this.state.facet(styleModule);
      let _ = this.state.facet(EditorView.cspNonce);
      StyleModule.mount(this.root, this.styleModules.concat(baseTheme$1$2).reverse(), _ ? { nonce: _ } : void 0);
    }
    readMeasured() {
      if (this.updateState == 2)
        throw new Error("Reading the editor layout isn't allowed during an update");
      this.updateState == 0 && this.measureScheduled > -1 && this.measure(!1);
    }
    /**
    Schedule a layout measurement, optionally providing callbacks to
    do custom DOM measuring followed by a DOM write phase. Using
    this is preferable reading DOM layout directly from, for
    example, an event handler, because it'll make sure measuring and
    drawing done by other components is synchronized, avoiding
    unnecessary DOM layout computations.
    */
    requestMeasure(_) {
      if (this.measureScheduled < 0 && (this.measureScheduled = this.win.requestAnimationFrame(() => this.measure())), _) {
        if (this.measureRequests.indexOf(_) > -1)
          return;
        if (_.key != null) {
          for (let H = 0; H < this.measureRequests.length; H++)
            if (this.measureRequests[H].key === _.key) {
              this.measureRequests[H] = _;
              return;
            }
        }
        this.measureRequests.push(_);
      }
    }
    /**
    Get the value of a specific plugin, if present. Note that
    plugins that crash can be dropped from a view, so even when you
    know you registered a given plugin, it is recommended to check
    the return value of this method.
    */
    plugin(_) {
      let H = this.pluginMap.get(_);
      return (H === void 0 || H && H.spec != _) && this.pluginMap.set(_, H = this.plugins.find((V) => V.spec == _) || null), H && H.update(this).value;
    }
    /**
    The top position of the document, in screen coordinates. This
    may be negative when the editor is scrolled down. Points
    directly to the top of the first line, not above the padding.
    */
    get documentTop() {
      return this.contentDOM.getBoundingClientRect().top + this.viewState.paddingTop;
    }
    /**
    Reports the padding above and below the document.
    */
    get documentPadding() {
      return { top: this.viewState.paddingTop, bottom: this.viewState.paddingBottom };
    }
    /**
    If the editor is transformed with CSS, this provides the scale
    along the X axis. Otherwise, it will just be 1. Note that
    transforms other than translation and scaling are not supported.
    */
    get scaleX() {
      return this.viewState.scaleX;
    }
    /**
    Provide the CSS transformed scale along the Y axis.
    */
    get scaleY() {
      return this.viewState.scaleY;
    }
    /**
    Find the text line or block widget at the given vertical
    position (which is interpreted as relative to the [top of the
    document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop)).
    */
    elementAtHeight(_) {
      return this.readMeasured(), this.viewState.elementAtHeight(_);
    }
    /**
    Find the line block (see
    [`lineBlockAt`](https://codemirror.net/6/docs/ref/#view.EditorView.lineBlockAt) at the given
    height, again interpreted relative to the [top of the
    document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop).
    */
    lineBlockAtHeight(_) {
      return this.readMeasured(), this.viewState.lineBlockAtHeight(_);
    }
    /**
    Get the extent and vertical position of all [line
    blocks](https://codemirror.net/6/docs/ref/#view.EditorView.lineBlockAt) in the viewport. Positions
    are relative to the [top of the
    document](https://codemirror.net/6/docs/ref/#view.EditorView.documentTop);
    */
    get viewportLineBlocks() {
      return this.viewState.viewportLines;
    }
    /**
    Find the line block around the given document position. A line
    block is a range delimited on both sides by either a
    non-[hidden](https://codemirror.net/6/docs/ref/#view.Decoration^replace) line break, or the
    start/end of the document. It will usually just hold a line of
    text, but may be broken into multiple textblocks by block
    widgets.
    */
    lineBlockAt(_) {
      return this.viewState.lineBlockAt(_);
    }
    /**
    The editor's total content height.
    */
    get contentHeight() {
      return this.viewState.contentHeight;
    }
    /**
    Move a cursor position by [grapheme
    cluster](https://codemirror.net/6/docs/ref/#state.findClusterBreak). `forward` determines whether
    the motion is away from the line start, or towards it. In
    bidirectional text, the line is traversed in visual order, using
    the editor's [text direction](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection).
    When the start position was the last one on the line, the
    returned position will be across the line break. If there is no
    further line, the original position is returned.
    
    By default, this method moves over a single cluster. The
    optional `by` argument can be used to move across more. It will
    be called with the first cluster as argument, and should return
    a predicate that determines, for each subsequent cluster,
    whether it should also be moved over.
    */
    moveByChar(_, H, V) {
      return skipAtoms(this, _, moveByChar(this, _, H, V));
    }
    /**
    Move a cursor position across the next group of either
    [letters](https://codemirror.net/6/docs/ref/#state.EditorState.charCategorizer) or non-letter
    non-whitespace characters.
    */
    moveByGroup(_, H) {
      return skipAtoms(this, _, moveByChar(this, _, H, (V) => byGroup(this, _.head, V)));
    }
    /**
    Get the cursor position visually at the start or end of a line.
    Note that this may differ from the _logical_ position at its
    start or end (which is simply at `line.from`/`line.to`) if text
    at the start or end goes against the line's base text direction.
    */
    visualLineSide(_, H) {
      let V = this.bidiSpans(_), q = this.textDirectionAt(_.from), G = V[H ? V.length - 1 : 0];
      return EditorSelection.cursor(G.side(H, q) + _.from, G.forward(!H, q) ? 1 : -1);
    }
    /**
    Move to the next line boundary in the given direction. If
    `includeWrap` is true, line wrapping is on, and there is a
    further wrap point on the current line, the wrap point will be
    returned. Otherwise this function will return the start or end
    of the line.
    */
    moveToLineBoundary(_, H, V = !0) {
      return moveToLineBoundary(this, _, H, V);
    }
    /**
    Move a cursor position vertically. When `distance` isn't given,
    it defaults to moving to the next line (including wrapped
    lines). Otherwise, `distance` should provide a positive distance
    in pixels.
    
    When `start` has a
    [`goalColumn`](https://codemirror.net/6/docs/ref/#state.SelectionRange.goalColumn), the vertical
    motion will use that as a target horizontal position. Otherwise,
    the cursor's own horizontal position is used. The returned
    cursor will have its goal column set to whichever column was
    used.
    */
    moveVertically(_, H, V) {
      return skipAtoms(this, _, moveVertically(this, _, H, V));
    }
    /**
    Find the DOM parent node and offset (child offset if `node` is
    an element, character offset when it is a text node) at the
    given document position.
    
    Note that for positions that aren't currently in
    `visibleRanges`, the resulting DOM position isn't necessarily
    meaningful (it may just point before or after a placeholder
    element).
    */
    domAtPos(_) {
      return this.docView.domAtPos(_);
    }
    /**
    Find the document position at the given DOM node. Can be useful
    for associating positions with DOM events. Will raise an error
    when `node` isn't part of the editor content.
    */
    posAtDOM(_, H = 0) {
      return this.docView.posFromDOM(_, H);
    }
    posAtCoords(_, H = !0) {
      return this.readMeasured(), posAtCoords(this, _, H);
    }
    /**
    Get the screen coordinates at the given document position.
    `side` determines whether the coordinates are based on the
    element before (-1) or after (1) the position (if no element is
    available on the given side, the method will transparently use
    another strategy to get reasonable coordinates).
    */
    coordsAtPos(_, H = 1) {
      this.readMeasured();
      let V = this.docView.coordsAt(_, H);
      if (!V || V.left == V.right)
        return V;
      let q = this.state.doc.lineAt(_), G = this.bidiSpans(q), J = G[BidiSpan.find(G, _ - q.from, -1, H)];
      return flattenRect(V, J.dir == Direction.LTR == H > 0);
    }
    /**
    Return the rectangle around a given character. If `pos` does not
    point in front of a character that is in the viewport and
    rendered (i.e. not replaced, not a line break), this will return
    null. For space characters that are a line wrap point, this will
    return the position before the line break.
    */
    coordsForChar(_) {
      return this.readMeasured(), this.docView.coordsForChar(_);
    }
    /**
    The default width of a character in the editor. May not
    accurately reflect the width of all characters (given variable
    width fonts or styling of invididual ranges).
    */
    get defaultCharacterWidth() {
      return this.viewState.heightOracle.charWidth;
    }
    /**
    The default height of a line in the editor. May not be accurate
    for all lines.
    */
    get defaultLineHeight() {
      return this.viewState.heightOracle.lineHeight;
    }
    /**
    The text direction
    ([`direction`](https://developer.mozilla.org/en-US/docs/Web/CSS/direction)
    CSS property) of the editor's content element.
    */
    get textDirection() {
      return this.viewState.defaultTextDirection;
    }
    /**
    Find the text direction of the block at the given position, as
    assigned by CSS. If
    [`perLineTextDirection`](https://codemirror.net/6/docs/ref/#view.EditorView^perLineTextDirection)
    isn't enabled, or the given position is outside of the viewport,
    this will always return the same as
    [`textDirection`](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection). Note that
    this may trigger a DOM layout.
    */
    textDirectionAt(_) {
      return !this.state.facet(perLineTextDirection) || _ < this.viewport.from || _ > this.viewport.to ? this.textDirection : (this.readMeasured(), this.docView.textDirectionAt(_));
    }
    /**
    Whether this editor [wraps lines](https://codemirror.net/6/docs/ref/#view.EditorView.lineWrapping)
    (as determined by the
    [`white-space`](https://developer.mozilla.org/en-US/docs/Web/CSS/white-space)
    CSS property of its content element).
    */
    get lineWrapping() {
      return this.viewState.heightOracle.lineWrapping;
    }
    /**
    Returns the bidirectional text structure of the given line
    (which should be in the current document) as an array of span
    objects. The order of these spans matches the [text
    direction](https://codemirror.net/6/docs/ref/#view.EditorView.textDirection)—if that is
    left-to-right, the leftmost spans come first, otherwise the
    rightmost spans come first.
    */
    bidiSpans(_) {
      if (_.length > MaxBidiLine)
        return trivialOrder(_.length);
      let H = this.textDirectionAt(_.from), V;
      for (let G of this.bidiCache)
        if (G.from == _.from && G.dir == H && (G.fresh || isolatesEq(G.isolates, V = getIsolatedRanges(this, _))))
          return G.order;
      V || (V = getIsolatedRanges(this, _));
      let q = computeOrder(_.text, H, V);
      return this.bidiCache.push(new CachedOrder(_.from, _.to, H, V, !0, q)), q;
    }
    /**
    Check whether the editor has focus.
    */
    get hasFocus() {
      var _;
      return (this.dom.ownerDocument.hasFocus() || browser.safari && ((_ = this.inputState) === null || _ === void 0 ? void 0 : _.lastContextMenu) > Date.now() - 3e4) && this.root.activeElement == this.contentDOM;
    }
    /**
    Put focus on the editor.
    */
    focus() {
      this.observer.ignore(() => {
        focusPreventScroll(this.contentDOM), this.docView.updateSelection();
      });
    }
    /**
    Update the [root](https://codemirror.net/6/docs/ref/##view.EditorViewConfig.root) in which the editor lives. This is only
    necessary when moving the editor's existing DOM to a new window or shadow root.
    */
    setRoot(_) {
      this._root != _ && (this._root = _, this.observer.setWindow((_.nodeType == 9 ? _ : _.ownerDocument).defaultView || window), this.mountStyles());
    }
    /**
    Clean up this editor view, removing its element from the
    document, unregistering event handlers, and notifying
    plugins. The view instance can no longer be used after
    calling this.
    */
    destroy() {
      this.root.activeElement == this.contentDOM && this.contentDOM.blur();
      for (let _ of this.plugins)
        _.destroy(this);
      this.plugins = [], this.inputState.destroy(), this.docView.destroy(), this.dom.remove(), this.observer.destroy(), this.measureScheduled > -1 && this.win.cancelAnimationFrame(this.measureScheduled), this.destroyed = !0;
    }
    /**
    Returns an effect that can be
    [added](https://codemirror.net/6/docs/ref/#state.TransactionSpec.effects) to a transaction to
    cause it to scroll the given position or range into view.
    */
    static scrollIntoView(_, H = {}) {
      return scrollIntoView$1.of(new ScrollTarget(typeof _ == "number" ? EditorSelection.cursor(_) : _, H.y, H.x, H.yMargin, H.xMargin));
    }
    /**
    Return an effect that resets the editor to its current (at the
    time this method was called) scroll position. Note that this
    only affects the editor's own scrollable element, not parents.
    See also
    [`EditorViewConfig.scrollTo`](https://codemirror.net/6/docs/ref/#view.EditorViewConfig.scrollTo).
    
    The effect should be used with a document identical to the one
    it was created for. Failing to do so is not an error, but may
    not scroll to the expected position. You can
    [map](https://codemirror.net/6/docs/ref/#state.StateEffect.map) the effect to account for changes.
    */
    scrollSnapshot() {
      let { scrollTop: _, scrollLeft: H } = this.scrollDOM, V = this.viewState.scrollAnchorAt(_);
      return scrollIntoView$1.of(new ScrollTarget(EditorSelection.cursor(V.from), "start", "start", V.top - _, H, !0));
    }
    /**
    Enable or disable tab-focus mode, which disables key bindings
    for Tab and Shift-Tab, letting the browser's default
    focus-changing behavior go through instead. This is useful to
    prevent trapping keyboard users in your editor.
    
    Without argument, this toggles the mode. With a boolean, it
    enables (true) or disables it (false). Given a number, it
    temporarily enables the mode until that number of milliseconds
    have passed or another non-Tab key is pressed.
    */
    setTabFocusMode(_) {
      _ == null ? this.inputState.tabFocusMode = this.inputState.tabFocusMode < 0 ? 0 : -1 : typeof _ == "boolean" ? this.inputState.tabFocusMode = _ ? 0 : -1 : this.inputState.tabFocusMode != 0 && (this.inputState.tabFocusMode = Date.now() + _);
    }
    /**
    Returns an extension that can be used to add DOM event handlers.
    The value should be an object mapping event names to handler
    functions. For any given event, such functions are ordered by
    extension precedence, and the first handler to return true will
    be assumed to have handled that event, and no other handlers or
    built-in behavior will be activated for it. These are registered
    on the [content element](https://codemirror.net/6/docs/ref/#view.EditorView.contentDOM), except
    for `scroll` handlers, which will be called any time the
    editor's [scroll element](https://codemirror.net/6/docs/ref/#view.EditorView.scrollDOM) or one of
    its parent nodes is scrolled.
    */
    static domEventHandlers(_) {
      return ViewPlugin.define(() => ({}), { eventHandlers: _ });
    }
    /**
    Create an extension that registers DOM event observers. Contrary
    to event [handlers](https://codemirror.net/6/docs/ref/#view.EditorView^domEventHandlers),
    observers can't be prevented from running by a higher-precedence
    handler returning true. They also don't prevent other handlers
    and observers from running when they return true, and should not
    call `preventDefault`.
    */
    static domEventObservers(_) {
      return ViewPlugin.define(() => ({}), { eventObservers: _ });
    }
    /**
    Create a theme extension. The first argument can be a
    [`style-mod`](https://github.com/marijnh/style-mod#documentation)
    style spec providing the styles for the theme. These will be
    prefixed with a generated class for the style.
    
    Because the selectors will be prefixed with a scope class, rule
    that directly match the editor's [wrapper
    element](https://codemirror.net/6/docs/ref/#view.EditorView.dom)—to which the scope class will be
    added—need to be explicitly differentiated by adding an `&` to
    the selector for that element—for example
    `&.cm-focused`.
    
    When `dark` is set to true, the theme will be marked as dark,
    which will cause the `&dark` rules from [base
    themes](https://codemirror.net/6/docs/ref/#view.EditorView^baseTheme) to be used (as opposed to
    `&light` when a light theme is active).
    */
    static theme(_, H) {
      let V = StyleModule.newName(), q = [theme.of(V), styleModule.of(buildTheme(`.${V}`, _))];
      return H && H.dark && q.push(darkTheme.of(!0)), q;
    }
    /**
    Create an extension that adds styles to the base theme. Like
    with [`theme`](https://codemirror.net/6/docs/ref/#view.EditorView^theme), use `&` to indicate the
    place of the editor wrapper element when directly targeting
    that. You can also use `&dark` or `&light` instead to only
    target editors with a dark or light theme.
    */
    static baseTheme(_) {
      return Prec.lowest(styleModule.of(buildTheme("." + baseThemeID, _, lightDarkIDs)));
    }
    /**
    Retrieve an editor view instance from the view's DOM
    representation.
    */
    static findFromDOM(_) {
      var H;
      let V = _.querySelector(".cm-content"), q = V && ContentView.get(V) || ContentView.get(_);
      return ((H = q == null ? void 0 : q.rootView) === null || H === void 0 ? void 0 : H.view) || null;
    }
  }
  EditorView.styleModule = styleModule;
  EditorView.inputHandler = inputHandler$1;
  EditorView.clipboardInputFilter = clipboardInputFilter;
  EditorView.clipboardOutputFilter = clipboardOutputFilter;
  EditorView.scrollHandler = scrollHandler;
  EditorView.focusChangeEffect = focusChangeEffect;
  EditorView.perLineTextDirection = perLineTextDirection;
  EditorView.exceptionSink = exceptionSink;
  EditorView.updateListener = updateListener;
  EditorView.editable = editable;
  EditorView.mouseSelectionStyle = mouseSelectionStyle;
  EditorView.dragMovesSelection = dragMovesSelection$1;
  EditorView.clickAddsSelectionRange = clickAddsSelectionRange;
  EditorView.decorations = decorations;
  EditorView.outerDecorations = outerDecorations;
  EditorView.atomicRanges = atomicRanges;
  EditorView.bidiIsolatedRanges = bidiIsolatedRanges;
  EditorView.scrollMargins = scrollMargins;
  EditorView.darkTheme = darkTheme;
  EditorView.cspNonce = /* @__PURE__ */ Facet.define({ combine: (O) => O.length ? O[0] : "" });
  EditorView.contentAttributes = contentAttributes;
  EditorView.editorAttributes = editorAttributes;
  EditorView.lineWrapping = /* @__PURE__ */ EditorView.contentAttributes.of({ class: "cm-lineWrapping" });
  EditorView.announce = /* @__PURE__ */ StateEffect.define();
  const MaxBidiLine = 4096, BadMeasure = {};
  class CachedOrder {
    constructor(_, H, V, q, G, J) {
      this.from = _, this.to = H, this.dir = V, this.isolates = q, this.fresh = G, this.order = J;
    }
    static update(_, H) {
      if (H.empty && !_.some((G) => G.fresh))
        return _;
      let V = [], q = _.length ? _[_.length - 1].dir : Direction.LTR;
      for (let G = Math.max(0, _.length - 10); G < _.length; G++) {
        let J = _[G];
        J.dir == q && !H.touchesRange(J.from, J.to) && V.push(new CachedOrder(H.mapPos(J.from, 1), H.mapPos(J.to, -1), J.dir, J.isolates, !1, J.order));
      }
      return V;
    }
  }
  function attrsFromFacet(O, _, H) {
    for (let V = O.state.facet(_), q = V.length - 1; q >= 0; q--) {
      let G = V[q], J = typeof G == "function" ? G(O) : G;
      J && combineAttrs(J, H);
    }
    return H;
  }
  const currentPlatform = browser.mac ? "mac" : browser.windows ? "win" : browser.linux ? "linux" : "key";
  function normalizeKeyName(O, _) {
    const H = O.split(/-(?!$)/);
    let V = H[H.length - 1];
    V == "Space" && (V = " ");
    let q, G, J, Z;
    for (let te = 0; te < H.length - 1; ++te) {
      const ne = H[te];
      if (/^(cmd|meta|m)$/i.test(ne))
        Z = !0;
      else if (/^a(lt)?$/i.test(ne))
        q = !0;
      else if (/^(c|ctrl|control)$/i.test(ne))
        G = !0;
      else if (/^s(hift)?$/i.test(ne))
        J = !0;
      else if (/^mod$/i.test(ne))
        _ == "mac" ? Z = !0 : G = !0;
      else
        throw new Error("Unrecognized modifier name: " + ne);
    }
    return q && (V = "Alt-" + V), G && (V = "Ctrl-" + V), Z && (V = "Meta-" + V), J && (V = "Shift-" + V), V;
  }
  function modifiers(O, _, H) {
    return _.altKey && (O = "Alt-" + O), _.ctrlKey && (O = "Ctrl-" + O), _.metaKey && (O = "Meta-" + O), H !== !1 && _.shiftKey && (O = "Shift-" + O), O;
  }
  const handleKeyEvents = /* @__PURE__ */ Prec.default(/* @__PURE__ */ EditorView.domEventHandlers({
    keydown(O, _) {
      return runHandlers(getKeymap(_.state), O, _, "editor");
    }
  })), keymap = /* @__PURE__ */ Facet.define({ enables: handleKeyEvents }), Keymaps = /* @__PURE__ */ new WeakMap();
  function getKeymap(O) {
    let _ = O.facet(keymap), H = Keymaps.get(_);
    return H || Keymaps.set(_, H = buildKeymap(_.reduce((V, q) => V.concat(q), []))), H;
  }
  function runScopeHandlers(O, _, H) {
    return runHandlers(getKeymap(O.state), _, O, H);
  }
  let storedPrefix = null;
  const PrefixTimeout = 4e3;
  function buildKeymap(O, _ = currentPlatform) {
    let H = /* @__PURE__ */ Object.create(null), V = /* @__PURE__ */ Object.create(null), q = (J, Z) => {
      let te = V[J];
      if (te == null)
        V[J] = Z;
      else if (te != Z)
        throw new Error("Key binding " + J + " is used both as a regular binding and as a multi-stroke prefix");
    }, G = (J, Z, te, ne, ee) => {
      var ae, oe;
      let se = H[J] || (H[J] = /* @__PURE__ */ Object.create(null)), ce = Z.split(/ (?!$)/).map((de) => normalizeKeyName(de, _));
      for (let de = 1; de < ce.length; de++) {
        let fe = ce.slice(0, de).join(" ");
        q(fe, !0), se[fe] || (se[fe] = {
          preventDefault: !0,
          stopPropagation: !1,
          run: [(he) => {
            let me = storedPrefix = { view: he, prefix: fe, scope: J };
            return setTimeout(() => {
              storedPrefix == me && (storedPrefix = null);
            }, PrefixTimeout), !0;
          }]
        });
      }
      let le = ce.join(" ");
      q(le, !1);
      let ue = se[le] || (se[le] = {
        preventDefault: !1,
        stopPropagation: !1,
        run: ((oe = (ae = se._any) === null || ae === void 0 ? void 0 : ae.run) === null || oe === void 0 ? void 0 : oe.slice()) || []
      });
      te && ue.run.push(te), ne && (ue.preventDefault = !0), ee && (ue.stopPropagation = !0);
    };
    for (let J of O) {
      let Z = J.scope ? J.scope.split(" ") : ["editor"];
      if (J.any)
        for (let ne of Z) {
          let ee = H[ne] || (H[ne] = /* @__PURE__ */ Object.create(null));
          ee._any || (ee._any = { preventDefault: !1, stopPropagation: !1, run: [] });
          let { any: ae } = J;
          for (let oe in ee)
            ee[oe].run.push((se) => ae(se, currentKeyEvent));
        }
      let te = J[_] || J.key;
      if (te)
        for (let ne of Z)
          G(ne, te, J.run, J.preventDefault, J.stopPropagation), J.shift && G(ne, "Shift-" + te, J.shift, J.preventDefault, J.stopPropagation);
    }
    return H;
  }
  let currentKeyEvent = null;
  function runHandlers(O, _, H, V) {
    currentKeyEvent = _;
    let q = keyName(_), G = codePointAt(q, 0), J = codePointSize(G) == q.length && q != " ", Z = "", te = !1, ne = !1, ee = !1;
    storedPrefix && storedPrefix.view == H && storedPrefix.scope == V && (Z = storedPrefix.prefix + " ", modifierCodes.indexOf(_.keyCode) < 0 && (ne = !0, storedPrefix = null));
    let ae = /* @__PURE__ */ new Set(), oe = (ue) => {
      if (ue) {
        for (let de of ue.run)
          if (!ae.has(de) && (ae.add(de), de(H)))
            return ue.stopPropagation && (ee = !0), !0;
        ue.preventDefault && (ue.stopPropagation && (ee = !0), ne = !0);
      }
      return !1;
    }, se = O[V], ce, le;
    return se && (oe(se[Z + modifiers(q, _, !J)]) ? te = !0 : J && (_.altKey || _.metaKey || _.ctrlKey) && // Ctrl-Alt may be used for AltGr on Windows
    !(browser.windows && _.ctrlKey && _.altKey) && (ce = base[_.keyCode]) && ce != q ? (oe(se[Z + modifiers(ce, _, !0)]) || _.shiftKey && (le = shift[_.keyCode]) != q && le != ce && oe(se[Z + modifiers(le, _, !1)])) && (te = !0) : J && _.shiftKey && oe(se[Z + modifiers(q, _, !0)]) && (te = !0), !te && oe(se._any) && (te = !0)), ne && (te = !0), te && ee && _.stopPropagation(), currentKeyEvent = null, te;
  }
  class RectangleMarker {
    /**
    Create a marker with the given class and dimensions. If `width`
    is null, the DOM element will get no width style.
    */
    constructor(_, H, V, q, G) {
      this.className = _, this.left = H, this.top = V, this.width = q, this.height = G;
    }
    draw() {
      let _ = document.createElement("div");
      return _.className = this.className, this.adjust(_), _;
    }
    update(_, H) {
      return H.className != this.className ? !1 : (this.adjust(_), !0);
    }
    adjust(_) {
      _.style.left = this.left + "px", _.style.top = this.top + "px", this.width != null && (_.style.width = this.width + "px"), _.style.height = this.height + "px";
    }
    eq(_) {
      return this.left == _.left && this.top == _.top && this.width == _.width && this.height == _.height && this.className == _.className;
    }
    /**
    Create a set of rectangles for the given selection range,
    assigning them theclass`className`. Will create a single
    rectangle for empty ranges, and a set of selection-style
    rectangles covering the range's content (in a bidi-aware
    way) for non-empty ones.
    */
    static forRange(_, H, V) {
      if (V.empty) {
        let q = _.coordsAtPos(V.head, V.assoc || 1);
        if (!q)
          return [];
        let G = getBase(_);
        return [new RectangleMarker(H, q.left - G.left, q.top - G.top, null, q.bottom - q.top)];
      } else
        return rectanglesForRange(_, H, V);
    }
  }
  function getBase(O) {
    let _ = O.scrollDOM.getBoundingClientRect();
    return { left: (O.textDirection == Direction.LTR ? _.left : _.right - O.scrollDOM.clientWidth * O.scaleX) - O.scrollDOM.scrollLeft * O.scaleX, top: _.top - O.scrollDOM.scrollTop * O.scaleY };
  }
  function wrappedLine(O, _, H, V) {
    let q = O.coordsAtPos(_, H * 2);
    if (!q)
      return V;
    let G = O.dom.getBoundingClientRect(), J = (q.top + q.bottom) / 2, Z = O.posAtCoords({ x: G.left + 1, y: J }), te = O.posAtCoords({ x: G.right - 1, y: J });
    return Z == null || te == null ? V : { from: Math.max(V.from, Math.min(Z, te)), to: Math.min(V.to, Math.max(Z, te)) };
  }
  function rectanglesForRange(O, _, H) {
    if (H.to <= O.viewport.from || H.from >= O.viewport.to)
      return [];
    let V = Math.max(H.from, O.viewport.from), q = Math.min(H.to, O.viewport.to), G = O.textDirection == Direction.LTR, J = O.contentDOM, Z = J.getBoundingClientRect(), te = getBase(O), ne = J.querySelector(".cm-line"), ee = ne && window.getComputedStyle(ne), ae = Z.left + (ee ? parseInt(ee.paddingLeft) + Math.min(0, parseInt(ee.textIndent)) : 0), oe = Z.right - (ee ? parseInt(ee.paddingRight) : 0), se = blockAt(O, V), ce = blockAt(O, q), le = se.type == BlockType.Text ? se : null, ue = ce.type == BlockType.Text ? ce : null;
    if (le && (O.lineWrapping || se.widgetLineBreaks) && (le = wrappedLine(O, V, 1, le)), ue && (O.lineWrapping || ce.widgetLineBreaks) && (ue = wrappedLine(O, q, -1, ue)), le && ue && le.from == ue.from && le.to == ue.to)
      return fe(he(H.from, H.to, le));
    {
      let pe = le ? he(H.from, null, le) : me(se, !1), ye = ue ? he(null, H.to, ue) : me(ce, !0), ge = [];
      return (le || se).to < (ue || ce).from - (le && ue ? 1 : 0) || se.widgetLineBreaks > 1 && pe.bottom + O.defaultLineHeight / 2 < ye.top ? ge.push(de(ae, pe.bottom, oe, ye.top)) : pe.bottom < ye.top && O.elementAtHeight((pe.bottom + ye.top) / 2).type == BlockType.Text && (pe.bottom = ye.top = (pe.bottom + ye.top) / 2), fe(pe).concat(ge).concat(fe(ye));
    }
    function de(pe, ye, ge, ve) {
      return new RectangleMarker(
        _,
        pe - te.left,
        ye - te.top - 0.01,
        ge - pe,
        ve - ye + 0.01
        /* C.Epsilon */
      );
    }
    function fe({ top: pe, bottom: ye, horizontal: ge }) {
      let ve = [];
      for (let be = 0; be < ge.length; be += 2)
        ve.push(de(ge[be], pe, ge[be + 1], ye));
      return ve;
    }
    function he(pe, ye, ge) {
      let ve = 1e9, be = -1e9, Ce = [];
      function Se(xe, Le, Qe, ke, Me) {
        let Re = O.coordsAtPos(xe, xe == ge.to ? -2 : 2), De = O.coordsAtPos(Qe, Qe == ge.from ? 2 : -2);
        !Re || !De || (ve = Math.min(Re.top, De.top, ve), be = Math.max(Re.bottom, De.bottom, be), Me == Direction.LTR ? Ce.push(G && Le ? ae : Re.left, G && ke ? oe : De.right) : Ce.push(!G && ke ? ae : De.left, !G && Le ? oe : Re.right));
      }
      let we = pe ?? ge.from, Ee = ye ?? ge.to;
      for (let xe of O.visibleRanges)
        if (xe.to > we && xe.from < Ee)
          for (let Le = Math.max(xe.from, we), Qe = Math.min(xe.to, Ee); ; ) {
            let ke = O.state.doc.lineAt(Le);
            for (let Me of O.bidiSpans(ke)) {
              let Re = Me.from + ke.from, De = Me.to + ke.from;
              if (Re >= Qe)
                break;
              De > Le && Se(Math.max(Re, Le), pe == null && Re <= we, Math.min(De, Qe), ye == null && De >= Ee, Me.dir);
            }
            if (Le = ke.to + 1, Le >= Qe)
              break;
          }
      return Ce.length == 0 && Se(we, pe == null, Ee, ye == null, O.textDirection), { top: ve, bottom: be, horizontal: Ce };
    }
    function me(pe, ye) {
      let ge = Z.top + (ye ? pe.top : pe.bottom);
      return { top: ge, bottom: ge, horizontal: [] };
    }
  }
  function sameMarker(O, _) {
    return O.constructor == _.constructor && O.eq(_);
  }
  class LayerView {
    constructor(_, H) {
      this.view = _, this.layer = H, this.drawn = [], this.scaleX = 1, this.scaleY = 1, this.measureReq = { read: this.measure.bind(this), write: this.draw.bind(this) }, this.dom = _.scrollDOM.appendChild(document.createElement("div")), this.dom.classList.add("cm-layer"), H.above && this.dom.classList.add("cm-layer-above"), H.class && this.dom.classList.add(H.class), this.scale(), this.dom.setAttribute("aria-hidden", "true"), this.setOrder(_.state), _.requestMeasure(this.measureReq), H.mount && H.mount(this.dom, _);
    }
    update(_) {
      _.startState.facet(layerOrder) != _.state.facet(layerOrder) && this.setOrder(_.state), (this.layer.update(_, this.dom) || _.geometryChanged) && (this.scale(), _.view.requestMeasure(this.measureReq));
    }
    docViewUpdate(_) {
      this.layer.updateOnDocViewUpdate !== !1 && _.requestMeasure(this.measureReq);
    }
    setOrder(_) {
      let H = 0, V = _.facet(layerOrder);
      for (; H < V.length && V[H] != this.layer; )
        H++;
      this.dom.style.zIndex = String((this.layer.above ? 150 : -1) - H);
    }
    measure() {
      return this.layer.markers(this.view);
    }
    scale() {
      let { scaleX: _, scaleY: H } = this.view;
      (_ != this.scaleX || H != this.scaleY) && (this.scaleX = _, this.scaleY = H, this.dom.style.transform = `scale(${1 / _}, ${1 / H})`);
    }
    draw(_) {
      if (_.length != this.drawn.length || _.some((H, V) => !sameMarker(H, this.drawn[V]))) {
        let H = this.dom.firstChild, V = 0;
        for (let q of _)
          q.update && H && q.constructor && this.drawn[V].constructor && q.update(H, this.drawn[V]) ? (H = H.nextSibling, V++) : this.dom.insertBefore(q.draw(), H);
        for (; H; ) {
          let q = H.nextSibling;
          H.remove(), H = q;
        }
        this.drawn = _;
      }
    }
    destroy() {
      this.layer.destroy && this.layer.destroy(this.dom, this.view), this.dom.remove();
    }
  }
  const layerOrder = /* @__PURE__ */ Facet.define();
  function layer(O) {
    return [
      ViewPlugin.define((_) => new LayerView(_, O)),
      layerOrder.of(O)
    ];
  }
  const CanHidePrimary = !browser.ios, selectionConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, {
        cursorBlinkRate: 1200,
        drawRangeCursor: !0
      }, {
        cursorBlinkRate: (_, H) => Math.min(_, H),
        drawRangeCursor: (_, H) => _ || H
      });
    }
  });
  function drawSelection(O = {}) {
    return [
      selectionConfig.of(O),
      cursorLayer,
      selectionLayer,
      hideNativeSelection,
      nativeSelectionHidden.of(!0)
    ];
  }
  function configChanged(O) {
    return O.startState.facet(selectionConfig) != O.state.facet(selectionConfig);
  }
  const cursorLayer = /* @__PURE__ */ layer({
    above: !0,
    markers(O) {
      let { state: _ } = O, H = _.facet(selectionConfig), V = [];
      for (let q of _.selection.ranges) {
        let G = q == _.selection.main;
        if (q.empty ? !G || CanHidePrimary : H.drawRangeCursor) {
          let J = G ? "cm-cursor cm-cursor-primary" : "cm-cursor cm-cursor-secondary", Z = q.empty ? q : EditorSelection.cursor(q.head, q.head > q.anchor ? -1 : 1);
          for (let te of RectangleMarker.forRange(O, J, Z))
            V.push(te);
        }
      }
      return V;
    },
    update(O, _) {
      O.transactions.some((V) => V.selection) && (_.style.animationName = _.style.animationName == "cm-blink" ? "cm-blink2" : "cm-blink");
      let H = configChanged(O);
      return H && setBlinkRate(O.state, _), O.docChanged || O.selectionSet || H;
    },
    mount(O, _) {
      setBlinkRate(_.state, O);
    },
    class: "cm-cursorLayer"
  });
  function setBlinkRate(O, _) {
    _.style.animationDuration = O.facet(selectionConfig).cursorBlinkRate + "ms";
  }
  const selectionLayer = /* @__PURE__ */ layer({
    above: !1,
    markers(O) {
      return O.state.selection.ranges.map((_) => _.empty ? [] : RectangleMarker.forRange(O, "cm-selectionBackground", _)).reduce((_, H) => _.concat(H));
    },
    update(O, _) {
      return O.docChanged || O.selectionSet || O.viewportChanged || configChanged(O);
    },
    class: "cm-selectionLayer"
  }), themeSpec = {
    ".cm-line": {
      "& ::selection, &::selection": { backgroundColor: "transparent !important" }
    },
    ".cm-content": {
      "& :focus": {
        caretColor: "initial !important",
        "&::selection, & ::selection": {
          backgroundColor: "Highlight !important"
        }
      }
    }
  };
  CanHidePrimary && (themeSpec[".cm-line"].caretColor = themeSpec[".cm-content"].caretColor = "transparent !important");
  const hideNativeSelection = /* @__PURE__ */ Prec.highest(/* @__PURE__ */ EditorView.theme(themeSpec)), setDropCursorPos = /* @__PURE__ */ StateEffect.define({
    map(O, _) {
      return O == null ? null : _.mapPos(O);
    }
  }), dropCursorPos = /* @__PURE__ */ StateField.define({
    create() {
      return null;
    },
    update(O, _) {
      return O != null && (O = _.changes.mapPos(O)), _.effects.reduce((H, V) => V.is(setDropCursorPos) ? V.value : H, O);
    }
  }), drawDropCursor = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.view = O, this.cursor = null, this.measureReq = { read: this.readPos.bind(this), write: this.drawCursor.bind(this) };
    }
    update(O) {
      var _;
      let H = O.state.field(dropCursorPos);
      H == null ? this.cursor != null && ((_ = this.cursor) === null || _ === void 0 || _.remove(), this.cursor = null) : (this.cursor || (this.cursor = this.view.scrollDOM.appendChild(document.createElement("div")), this.cursor.className = "cm-dropCursor"), (O.startState.field(dropCursorPos) != H || O.docChanged || O.geometryChanged) && this.view.requestMeasure(this.measureReq));
    }
    readPos() {
      let { view: O } = this, _ = O.state.field(dropCursorPos), H = _ != null && O.coordsAtPos(_);
      if (!H)
        return null;
      let V = O.scrollDOM.getBoundingClientRect();
      return {
        left: H.left - V.left + O.scrollDOM.scrollLeft * O.scaleX,
        top: H.top - V.top + O.scrollDOM.scrollTop * O.scaleY,
        height: H.bottom - H.top
      };
    }
    drawCursor(O) {
      if (this.cursor) {
        let { scaleX: _, scaleY: H } = this.view;
        O ? (this.cursor.style.left = O.left / _ + "px", this.cursor.style.top = O.top / H + "px", this.cursor.style.height = O.height / H + "px") : this.cursor.style.left = "-100000px";
      }
    }
    destroy() {
      this.cursor && this.cursor.remove();
    }
    setDropPos(O) {
      this.view.state.field(dropCursorPos) != O && this.view.dispatch({ effects: setDropCursorPos.of(O) });
    }
  }, {
    eventObservers: {
      dragover(O) {
        this.setDropPos(this.view.posAtCoords({ x: O.clientX, y: O.clientY }));
      },
      dragleave(O) {
        (O.target == this.view.contentDOM || !this.view.contentDOM.contains(O.relatedTarget)) && this.setDropPos(null);
      },
      dragend() {
        this.setDropPos(null);
      },
      drop() {
        this.setDropPos(null);
      }
    }
  });
  function dropCursor() {
    return [dropCursorPos, drawDropCursor];
  }
  function iterMatches(O, _, H, V, q) {
    _.lastIndex = 0;
    for (let G = O.iterRange(H, V), J = H, Z; !G.next().done; J += G.value.length)
      if (!G.lineBreak)
        for (; Z = _.exec(G.value); )
          q(J + Z.index, Z);
  }
  function matchRanges(O, _) {
    let H = O.visibleRanges;
    if (H.length == 1 && H[0].from == O.viewport.from && H[0].to == O.viewport.to)
      return H;
    let V = [];
    for (let { from: q, to: G } of H)
      q = Math.max(O.state.doc.lineAt(q).from, q - _), G = Math.min(O.state.doc.lineAt(G).to, G + _), V.length && V[V.length - 1].to >= q ? V[V.length - 1].to = G : V.push({ from: q, to: G });
    return V;
  }
  class MatchDecorator {
    /**
    Create a decorator.
    */
    constructor(_) {
      const { regexp: H, decoration: V, decorate: q, boundary: G, maxLength: J = 1e3 } = _;
      if (!H.global)
        throw new RangeError("The regular expression given to MatchDecorator should have its 'g' flag set");
      if (this.regexp = H, q)
        this.addMatch = (Z, te, ne, ee) => q(ee, ne, ne + Z[0].length, Z, te);
      else if (typeof V == "function")
        this.addMatch = (Z, te, ne, ee) => {
          let ae = V(Z, te, ne);
          ae && ee(ne, ne + Z[0].length, ae);
        };
      else if (V)
        this.addMatch = (Z, te, ne, ee) => ee(ne, ne + Z[0].length, V);
      else
        throw new RangeError("Either 'decorate' or 'decoration' should be provided to MatchDecorator");
      this.boundary = G, this.maxLength = J;
    }
    /**
    Compute the full set of decorations for matches in the given
    view's viewport. You'll want to call this when initializing your
    plugin.
    */
    createDeco(_) {
      let H = new RangeSetBuilder(), V = H.add.bind(H);
      for (let { from: q, to: G } of matchRanges(_, this.maxLength))
        iterMatches(_.state.doc, this.regexp, q, G, (J, Z) => this.addMatch(Z, _, J, V));
      return H.finish();
    }
    /**
    Update a set of decorations for a view update. `deco` _must_ be
    the set of decorations produced by _this_ `MatchDecorator` for
    the view state before the update.
    */
    updateDeco(_, H) {
      let V = 1e9, q = -1;
      return _.docChanged && _.changes.iterChanges((G, J, Z, te) => {
        te > _.view.viewport.from && Z < _.view.viewport.to && (V = Math.min(Z, V), q = Math.max(te, q));
      }), _.viewportChanged || q - V > 1e3 ? this.createDeco(_.view) : q > -1 ? this.updateRange(_.view, H.map(_.changes), V, q) : H;
    }
    updateRange(_, H, V, q) {
      for (let G of _.visibleRanges) {
        let J = Math.max(G.from, V), Z = Math.min(G.to, q);
        if (Z > J) {
          let te = _.state.doc.lineAt(J), ne = te.to < Z ? _.state.doc.lineAt(Z) : te, ee = Math.max(G.from, te.from), ae = Math.min(G.to, ne.to);
          if (this.boundary) {
            for (; J > te.from; J--)
              if (this.boundary.test(te.text[J - 1 - te.from])) {
                ee = J;
                break;
              }
            for (; Z < ne.to; Z++)
              if (this.boundary.test(ne.text[Z - ne.from])) {
                ae = Z;
                break;
              }
          }
          let oe = [], se, ce = (le, ue, de) => oe.push(de.range(le, ue));
          if (te == ne)
            for (this.regexp.lastIndex = ee - te.from; (se = this.regexp.exec(te.text)) && se.index < ae - te.from; )
              this.addMatch(se, _, se.index + te.from, ce);
          else
            iterMatches(_.state.doc, this.regexp, ee, ae, (le, ue) => this.addMatch(ue, _, le, ce));
          H = H.update({ filterFrom: ee, filterTo: ae, filter: (le, ue) => le < ee || ue > ae, add: oe });
        }
      }
      return H;
    }
  }
  const UnicodeRegexpSupport = /x/.unicode != null ? "gu" : "g", Specials = /* @__PURE__ */ new RegExp(`[\0-\b
--­؜​‎‏\u2028\u2029‭‮⁦⁧⁩\uFEFF￹-￼]`, UnicodeRegexpSupport), Names = {
    0: "null",
    7: "bell",
    8: "backspace",
    10: "newline",
    11: "vertical tab",
    13: "carriage return",
    27: "escape",
    8203: "zero width space",
    8204: "zero width non-joiner",
    8205: "zero width joiner",
    8206: "left-to-right mark",
    8207: "right-to-left mark",
    8232: "line separator",
    8237: "left-to-right override",
    8238: "right-to-left override",
    8294: "left-to-right isolate",
    8295: "right-to-left isolate",
    8297: "pop directional isolate",
    8233: "paragraph separator",
    65279: "zero width no-break space",
    65532: "object replacement"
  };
  let _supportsTabSize = null;
  function supportsTabSize() {
    var O;
    if (_supportsTabSize == null && typeof document < "u" && document.body) {
      let _ = document.body.style;
      _supportsTabSize = ((O = _.tabSize) !== null && O !== void 0 ? O : _.MozTabSize) != null;
    }
    return _supportsTabSize || !1;
  }
  const specialCharConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      let _ = combineConfig(O, {
        render: null,
        specialChars: Specials,
        addSpecialChars: null
      });
      return (_.replaceTabs = !supportsTabSize()) && (_.specialChars = new RegExp("	|" + _.specialChars.source, UnicodeRegexpSupport)), _.addSpecialChars && (_.specialChars = new RegExp(_.specialChars.source + "|" + _.addSpecialChars.source, UnicodeRegexpSupport)), _;
    }
  });
  function highlightSpecialChars(O = {}) {
    return [specialCharConfig.of(O), specialCharPlugin()];
  }
  let _plugin = null;
  function specialCharPlugin() {
    return _plugin || (_plugin = ViewPlugin.fromClass(class {
      constructor(O) {
        this.view = O, this.decorations = Decoration.none, this.decorationCache = /* @__PURE__ */ Object.create(null), this.decorator = this.makeDecorator(O.state.facet(specialCharConfig)), this.decorations = this.decorator.createDeco(O);
      }
      makeDecorator(O) {
        return new MatchDecorator({
          regexp: O.specialChars,
          decoration: (_, H, V) => {
            let { doc: q } = H.state, G = codePointAt(_[0], 0);
            if (G == 9) {
              let J = q.lineAt(V), Z = H.state.tabSize, te = countColumn(J.text, Z, V - J.from);
              return Decoration.replace({
                widget: new TabWidget((Z - te % Z) * this.view.defaultCharacterWidth / this.view.scaleX)
              });
            }
            return this.decorationCache[G] || (this.decorationCache[G] = Decoration.replace({ widget: new SpecialCharWidget(O, G) }));
          },
          boundary: O.replaceTabs ? void 0 : /[^]/
        });
      }
      update(O) {
        let _ = O.state.facet(specialCharConfig);
        O.startState.facet(specialCharConfig) != _ ? (this.decorator = this.makeDecorator(_), this.decorations = this.decorator.createDeco(O.view)) : this.decorations = this.decorator.updateDeco(O, this.decorations);
      }
    }, {
      decorations: (O) => O.decorations
    }));
  }
  const DefaultPlaceholder = "•";
  function placeholder$1(O) {
    return O >= 32 ? DefaultPlaceholder : O == 10 ? "␤" : String.fromCharCode(9216 + O);
  }
  class SpecialCharWidget extends WidgetType {
    constructor(_, H) {
      super(), this.options = _, this.code = H;
    }
    eq(_) {
      return _.code == this.code;
    }
    toDOM(_) {
      let H = placeholder$1(this.code), V = _.state.phrase("Control character") + " " + (Names[this.code] || "0x" + this.code.toString(16)), q = this.options.render && this.options.render(this.code, V, H);
      if (q)
        return q;
      let G = document.createElement("span");
      return G.textContent = H, G.title = V, G.setAttribute("aria-label", V), G.className = "cm-specialChar", G;
    }
    ignoreEvent() {
      return !1;
    }
  }
  class TabWidget extends WidgetType {
    constructor(_) {
      super(), this.width = _;
    }
    eq(_) {
      return _.width == this.width;
    }
    toDOM() {
      let _ = document.createElement("span");
      return _.textContent = "	", _.className = "cm-tab", _.style.width = this.width + "px", _;
    }
    ignoreEvent() {
      return !1;
    }
  }
  function highlightActiveLine() {
    return activeLineHighlighter;
  }
  const lineDeco = /* @__PURE__ */ Decoration.line({ class: "cm-activeLine" }), activeLineHighlighter = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.decorations = this.getDeco(O);
    }
    update(O) {
      (O.docChanged || O.selectionSet) && (this.decorations = this.getDeco(O.view));
    }
    getDeco(O) {
      let _ = -1, H = [];
      for (let V of O.state.selection.ranges) {
        let q = O.lineBlockAt(V.head);
        q.from > _ && (H.push(lineDeco.range(q.from)), _ = q.from);
      }
      return Decoration.set(H);
    }
  }, {
    decorations: (O) => O.decorations
  }), MaxOff = 2e3;
  function rectangleFor(O, _, H) {
    let V = Math.min(_.line, H.line), q = Math.max(_.line, H.line), G = [];
    if (_.off > MaxOff || H.off > MaxOff || _.col < 0 || H.col < 0) {
      let J = Math.min(_.off, H.off), Z = Math.max(_.off, H.off);
      for (let te = V; te <= q; te++) {
        let ne = O.doc.line(te);
        ne.length <= Z && G.push(EditorSelection.range(ne.from + J, ne.to + Z));
      }
    } else {
      let J = Math.min(_.col, H.col), Z = Math.max(_.col, H.col);
      for (let te = V; te <= q; te++) {
        let ne = O.doc.line(te), ee = findColumn(ne.text, J, O.tabSize, !0);
        if (ee < 0)
          G.push(EditorSelection.cursor(ne.to));
        else {
          let ae = findColumn(ne.text, Z, O.tabSize);
          G.push(EditorSelection.range(ne.from + ee, ne.from + ae));
        }
      }
    }
    return G;
  }
  function absoluteColumn(O, _) {
    let H = O.coordsAtPos(O.viewport.from);
    return H ? Math.round(Math.abs((H.left - _) / O.defaultCharacterWidth)) : -1;
  }
  function getPos(O, _) {
    let H = O.posAtCoords({ x: _.clientX, y: _.clientY }, !1), V = O.state.doc.lineAt(H), q = H - V.from, G = q > MaxOff ? -1 : q == V.length ? absoluteColumn(O, _.clientX) : countColumn(V.text, O.state.tabSize, H - V.from);
    return { line: V.number, col: G, off: q };
  }
  function rectangleSelectionStyle(O, _) {
    let H = getPos(O, _), V = O.state.selection;
    return H ? {
      update(q) {
        if (q.docChanged) {
          let G = q.changes.mapPos(q.startState.doc.line(H.line).from), J = q.state.doc.lineAt(G);
          H = { line: J.number, col: H.col, off: Math.min(H.off, J.length) }, V = V.map(q.changes);
        }
      },
      get(q, G, J) {
        let Z = getPos(O, q);
        if (!Z)
          return V;
        let te = rectangleFor(O.state, H, Z);
        return te.length ? J ? EditorSelection.create(te.concat(V.ranges)) : EditorSelection.create(te) : V;
      }
    } : null;
  }
  function rectangularSelection(O) {
    let _ = (H) => H.altKey && H.button == 0;
    return EditorView.mouseSelectionStyle.of((H, V) => _(V) ? rectangleSelectionStyle(H, V) : null);
  }
  const keys = {
    Alt: [18, (O) => !!O.altKey],
    Control: [17, (O) => !!O.ctrlKey],
    Shift: [16, (O) => !!O.shiftKey],
    Meta: [91, (O) => !!O.metaKey]
  }, showCrosshair = { style: "cursor: crosshair" };
  function crosshairCursor(O = {}) {
    let [_, H] = keys[O.key || "Alt"], V = ViewPlugin.fromClass(class {
      constructor(q) {
        this.view = q, this.isDown = !1;
      }
      set(q) {
        this.isDown != q && (this.isDown = q, this.view.update([]));
      }
    }, {
      eventObservers: {
        keydown(q) {
          this.set(q.keyCode == _ || H(q));
        },
        keyup(q) {
          (q.keyCode == _ || !H(q)) && this.set(!1);
        },
        mousemove(q) {
          this.set(H(q));
        }
      }
    });
    return [
      V,
      EditorView.contentAttributes.of((q) => {
        var G;
        return !((G = q.plugin(V)) === null || G === void 0) && G.isDown ? showCrosshair : null;
      })
    ];
  }
  const Outside = "-10000px";
  class TooltipViewManager {
    constructor(_, H, V, q) {
      this.facet = H, this.createTooltipView = V, this.removeTooltipView = q, this.input = _.state.facet(H), this.tooltips = this.input.filter((J) => J);
      let G = null;
      this.tooltipViews = this.tooltips.map((J) => G = V(J, G));
    }
    update(_, H) {
      var V;
      let q = _.state.facet(this.facet), G = q.filter((te) => te);
      if (q === this.input) {
        for (let te of this.tooltipViews)
          te.update && te.update(_);
        return !1;
      }
      let J = [], Z = H ? [] : null;
      for (let te = 0; te < G.length; te++) {
        let ne = G[te], ee = -1;
        if (ne) {
          for (let ae = 0; ae < this.tooltips.length; ae++) {
            let oe = this.tooltips[ae];
            oe && oe.create == ne.create && (ee = ae);
          }
          if (ee < 0)
            J[te] = this.createTooltipView(ne, te ? J[te - 1] : null), Z && (Z[te] = !!ne.above);
          else {
            let ae = J[te] = this.tooltipViews[ee];
            Z && (Z[te] = H[ee]), ae.update && ae.update(_);
          }
        }
      }
      for (let te of this.tooltipViews)
        J.indexOf(te) < 0 && (this.removeTooltipView(te), (V = te.destroy) === null || V === void 0 || V.call(te));
      return H && (Z.forEach((te, ne) => H[ne] = te), H.length = Z.length), this.input = q, this.tooltips = G, this.tooltipViews = J, !0;
    }
  }
  function windowSpace(O) {
    let { win: _ } = O;
    return { top: 0, left: 0, bottom: _.innerHeight, right: _.innerWidth };
  }
  const tooltipConfig = /* @__PURE__ */ Facet.define({
    combine: (O) => {
      var _, H, V;
      return {
        position: browser.ios ? "absolute" : ((_ = O.find((q) => q.position)) === null || _ === void 0 ? void 0 : _.position) || "fixed",
        parent: ((H = O.find((q) => q.parent)) === null || H === void 0 ? void 0 : H.parent) || null,
        tooltipSpace: ((V = O.find((q) => q.tooltipSpace)) === null || V === void 0 ? void 0 : V.tooltipSpace) || windowSpace
      };
    }
  }), knownHeight = /* @__PURE__ */ new WeakMap(), tooltipPlugin = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.view = O, this.above = [], this.inView = !0, this.madeAbsolute = !1, this.lastTransaction = 0, this.measureTimeout = -1;
      let _ = O.state.facet(tooltipConfig);
      this.position = _.position, this.parent = _.parent, this.classes = O.themeClasses, this.createContainer(), this.measureReq = { read: this.readMeasure.bind(this), write: this.writeMeasure.bind(this), key: this }, this.resizeObserver = typeof ResizeObserver == "function" ? new ResizeObserver(() => this.measureSoon()) : null, this.manager = new TooltipViewManager(O, showTooltip, (H, V) => this.createTooltip(H, V), (H) => {
        this.resizeObserver && this.resizeObserver.unobserve(H.dom), H.dom.remove();
      }), this.above = this.manager.tooltips.map((H) => !!H.above), this.intersectionObserver = typeof IntersectionObserver == "function" ? new IntersectionObserver((H) => {
        Date.now() > this.lastTransaction - 50 && H.length > 0 && H[H.length - 1].intersectionRatio < 1 && this.measureSoon();
      }, { threshold: [1] }) : null, this.observeIntersection(), O.win.addEventListener("resize", this.measureSoon = this.measureSoon.bind(this)), this.maybeMeasure();
    }
    createContainer() {
      this.parent ? (this.container = document.createElement("div"), this.container.style.position = "relative", this.container.className = this.view.themeClasses, this.parent.appendChild(this.container)) : this.container = this.view.dom;
    }
    observeIntersection() {
      if (this.intersectionObserver) {
        this.intersectionObserver.disconnect();
        for (let O of this.manager.tooltipViews)
          this.intersectionObserver.observe(O.dom);
      }
    }
    measureSoon() {
      this.measureTimeout < 0 && (this.measureTimeout = setTimeout(() => {
        this.measureTimeout = -1, this.maybeMeasure();
      }, 50));
    }
    update(O) {
      O.transactions.length && (this.lastTransaction = Date.now());
      let _ = this.manager.update(O, this.above);
      _ && this.observeIntersection();
      let H = _ || O.geometryChanged, V = O.state.facet(tooltipConfig);
      if (V.position != this.position && !this.madeAbsolute) {
        this.position = V.position;
        for (let q of this.manager.tooltipViews)
          q.dom.style.position = this.position;
        H = !0;
      }
      if (V.parent != this.parent) {
        this.parent && this.container.remove(), this.parent = V.parent, this.createContainer();
        for (let q of this.manager.tooltipViews)
          this.container.appendChild(q.dom);
        H = !0;
      } else this.parent && this.view.themeClasses != this.classes && (this.classes = this.container.className = this.view.themeClasses);
      H && this.maybeMeasure();
    }
    createTooltip(O, _) {
      let H = O.create(this.view), V = _ ? _.dom : null;
      if (H.dom.classList.add("cm-tooltip"), O.arrow && !H.dom.querySelector(".cm-tooltip > .cm-tooltip-arrow")) {
        let q = document.createElement("div");
        q.className = "cm-tooltip-arrow", H.dom.appendChild(q);
      }
      return H.dom.style.position = this.position, H.dom.style.top = Outside, H.dom.style.left = "0px", this.container.insertBefore(H.dom, V), H.mount && H.mount(this.view), this.resizeObserver && this.resizeObserver.observe(H.dom), H;
    }
    destroy() {
      var O, _, H;
      this.view.win.removeEventListener("resize", this.measureSoon);
      for (let V of this.manager.tooltipViews)
        V.dom.remove(), (O = V.destroy) === null || O === void 0 || O.call(V);
      this.parent && this.container.remove(), (_ = this.resizeObserver) === null || _ === void 0 || _.disconnect(), (H = this.intersectionObserver) === null || H === void 0 || H.disconnect(), clearTimeout(this.measureTimeout);
    }
    readMeasure() {
      let O = this.view.dom.getBoundingClientRect(), _ = 1, H = 1, V = !1;
      if (this.position == "fixed" && this.manager.tooltipViews.length) {
        let { dom: q } = this.manager.tooltipViews[0];
        if (browser.gecko)
          V = q.offsetParent != this.container.ownerDocument.body;
        else if (q.style.top == Outside && q.style.left == "0px") {
          let G = q.getBoundingClientRect();
          V = Math.abs(G.top + 1e4) > 1 || Math.abs(G.left) > 1;
        }
      }
      if (V || this.position == "absolute")
        if (this.parent) {
          let q = this.parent.getBoundingClientRect();
          q.width && q.height && (_ = q.width / this.parent.offsetWidth, H = q.height / this.parent.offsetHeight);
        } else
          ({ scaleX: _, scaleY: H } = this.view.viewState);
      return {
        editor: O,
        parent: this.parent ? this.container.getBoundingClientRect() : O,
        pos: this.manager.tooltips.map((q, G) => {
          let J = this.manager.tooltipViews[G];
          return J.getCoords ? J.getCoords(q.pos) : this.view.coordsAtPos(q.pos);
        }),
        size: this.manager.tooltipViews.map(({ dom: q }) => q.getBoundingClientRect()),
        space: this.view.state.facet(tooltipConfig).tooltipSpace(this.view),
        scaleX: _,
        scaleY: H,
        makeAbsolute: V
      };
    }
    writeMeasure(O) {
      var _;
      if (O.makeAbsolute) {
        this.madeAbsolute = !0, this.position = "absolute";
        for (let Z of this.manager.tooltipViews)
          Z.dom.style.position = "absolute";
      }
      let { editor: H, space: V, scaleX: q, scaleY: G } = O, J = [];
      for (let Z = 0; Z < this.manager.tooltips.length; Z++) {
        let te = this.manager.tooltips[Z], ne = this.manager.tooltipViews[Z], { dom: ee } = ne, ae = O.pos[Z], oe = O.size[Z];
        if (!ae || ae.bottom <= Math.max(H.top, V.top) || ae.top >= Math.min(H.bottom, V.bottom) || ae.right < Math.max(H.left, V.left) - 0.1 || ae.left > Math.min(H.right, V.right) + 0.1) {
          ee.style.top = Outside;
          continue;
        }
        let se = te.arrow ? ne.dom.querySelector(".cm-tooltip-arrow") : null, ce = se ? 7 : 0, le = oe.right - oe.left, ue = (_ = knownHeight.get(ne)) !== null && _ !== void 0 ? _ : oe.bottom - oe.top, de = ne.offset || noOffset, fe = this.view.textDirection == Direction.LTR, he = oe.width > V.right - V.left ? fe ? V.left : V.right - oe.width : fe ? Math.max(V.left, Math.min(ae.left - (se ? 14 : 0) + de.x, V.right - le)) : Math.min(Math.max(V.left, ae.left - le + (se ? 14 : 0) - de.x), V.right - le), me = this.above[Z];
        !te.strictSide && (me ? ae.top - (oe.bottom - oe.top) - de.y < V.top : ae.bottom + (oe.bottom - oe.top) + de.y > V.bottom) && me == V.bottom - ae.bottom > ae.top - V.top && (me = this.above[Z] = !me);
        let pe = (me ? ae.top - V.top : V.bottom - ae.bottom) - ce;
        if (pe < ue && ne.resize !== !1) {
          if (pe < this.view.defaultLineHeight) {
            ee.style.top = Outside;
            continue;
          }
          knownHeight.set(ne, ue), ee.style.height = (ue = pe) / G + "px";
        } else ee.style.height && (ee.style.height = "");
        let ye = me ? ae.top - ue - ce - de.y : ae.bottom + ce + de.y, ge = he + le;
        if (ne.overlap !== !0)
          for (let ve of J)
            ve.left < ge && ve.right > he && ve.top < ye + ue && ve.bottom > ye && (ye = me ? ve.top - ue - 2 - ce : ve.bottom + ce + 2);
        if (this.position == "absolute" ? (ee.style.top = (ye - O.parent.top) / G + "px", ee.style.left = (he - O.parent.left) / q + "px") : (ee.style.top = ye / G + "px", ee.style.left = he / q + "px"), se) {
          let ve = ae.left + (fe ? de.x : -de.x) - (he + 14 - 7);
          se.style.left = ve / q + "px";
        }
        ne.overlap !== !0 && J.push({ left: he, top: ye, right: ge, bottom: ye + ue }), ee.classList.toggle("cm-tooltip-above", me), ee.classList.toggle("cm-tooltip-below", !me), ne.positioned && ne.positioned(O.space);
      }
    }
    maybeMeasure() {
      if (this.manager.tooltips.length && (this.view.inView && this.view.requestMeasure(this.measureReq), this.inView != this.view.inView && (this.inView = this.view.inView, !this.inView)))
        for (let O of this.manager.tooltipViews)
          O.dom.style.top = Outside;
    }
  }, {
    eventObservers: {
      scroll() {
        this.maybeMeasure();
      }
    }
  }), baseTheme$5 = /* @__PURE__ */ EditorView.baseTheme({
    ".cm-tooltip": {
      zIndex: 100,
      boxSizing: "border-box"
    },
    "&light .cm-tooltip": {
      border: "1px solid #bbb",
      backgroundColor: "#f5f5f5"
    },
    "&light .cm-tooltip-section:not(:first-child)": {
      borderTop: "1px solid #bbb"
    },
    "&dark .cm-tooltip": {
      backgroundColor: "#333338",
      color: "white"
    },
    ".cm-tooltip-arrow": {
      height: "7px",
      width: `${7 * 2}px`,
      position: "absolute",
      zIndex: -1,
      overflow: "hidden",
      "&:before, &:after": {
        content: "''",
        position: "absolute",
        width: 0,
        height: 0,
        borderLeft: "7px solid transparent",
        borderRight: "7px solid transparent"
      },
      ".cm-tooltip-above &": {
        bottom: "-7px",
        "&:before": {
          borderTop: "7px solid #bbb"
        },
        "&:after": {
          borderTop: "7px solid #f5f5f5",
          bottom: "1px"
        }
      },
      ".cm-tooltip-below &": {
        top: "-7px",
        "&:before": {
          borderBottom: "7px solid #bbb"
        },
        "&:after": {
          borderBottom: "7px solid #f5f5f5",
          top: "1px"
        }
      }
    },
    "&dark .cm-tooltip .cm-tooltip-arrow": {
      "&:before": {
        borderTopColor: "#333338",
        borderBottomColor: "#333338"
      },
      "&:after": {
        borderTopColor: "transparent",
        borderBottomColor: "transparent"
      }
    }
  }), noOffset = { x: 0, y: 0 }, showTooltip = /* @__PURE__ */ Facet.define({
    enables: [tooltipPlugin, baseTheme$5]
  }), showHoverTooltip = /* @__PURE__ */ Facet.define({
    combine: (O) => O.reduce((_, H) => _.concat(H), [])
  });
  class HoverTooltipHost {
    // Needs to be static so that host tooltip instances always match
    static create(_) {
      return new HoverTooltipHost(_);
    }
    constructor(_) {
      this.view = _, this.mounted = !1, this.dom = document.createElement("div"), this.dom.classList.add("cm-tooltip-hover"), this.manager = new TooltipViewManager(_, showHoverTooltip, (H, V) => this.createHostedView(H, V), (H) => H.dom.remove());
    }
    createHostedView(_, H) {
      let V = _.create(this.view);
      return V.dom.classList.add("cm-tooltip-section"), this.dom.insertBefore(V.dom, H ? H.dom.nextSibling : this.dom.firstChild), this.mounted && V.mount && V.mount(this.view), V;
    }
    mount(_) {
      for (let H of this.manager.tooltipViews)
        H.mount && H.mount(_);
      this.mounted = !0;
    }
    positioned(_) {
      for (let H of this.manager.tooltipViews)
        H.positioned && H.positioned(_);
    }
    update(_) {
      this.manager.update(_);
    }
    destroy() {
      var _;
      for (let H of this.manager.tooltipViews)
        (_ = H.destroy) === null || _ === void 0 || _.call(H);
    }
    passProp(_) {
      let H;
      for (let V of this.manager.tooltipViews) {
        let q = V[_];
        if (q !== void 0) {
          if (H === void 0)
            H = q;
          else if (H !== q)
            return;
        }
      }
      return H;
    }
    get offset() {
      return this.passProp("offset");
    }
    get getCoords() {
      return this.passProp("getCoords");
    }
    get overlap() {
      return this.passProp("overlap");
    }
    get resize() {
      return this.passProp("resize");
    }
  }
  const showHoverTooltipHost = /* @__PURE__ */ showTooltip.compute([showHoverTooltip], (O) => {
    let _ = O.facet(showHoverTooltip);
    return _.length === 0 ? null : {
      pos: Math.min(..._.map((H) => H.pos)),
      end: Math.max(..._.map((H) => {
        var V;
        return (V = H.end) !== null && V !== void 0 ? V : H.pos;
      })),
      create: HoverTooltipHost.create,
      above: _[0].above,
      arrow: _.some((H) => H.arrow)
    };
  });
  class HoverPlugin {
    constructor(_, H, V, q, G) {
      this.view = _, this.source = H, this.field = V, this.setHover = q, this.hoverTime = G, this.hoverTimeout = -1, this.restartTimeout = -1, this.pending = null, this.lastMove = { x: 0, y: 0, target: _.dom, time: 0 }, this.checkHover = this.checkHover.bind(this), _.dom.addEventListener("mouseleave", this.mouseleave = this.mouseleave.bind(this)), _.dom.addEventListener("mousemove", this.mousemove = this.mousemove.bind(this));
    }
    update() {
      this.pending && (this.pending = null, clearTimeout(this.restartTimeout), this.restartTimeout = setTimeout(() => this.startHover(), 20));
    }
    get active() {
      return this.view.state.field(this.field);
    }
    checkHover() {
      if (this.hoverTimeout = -1, this.active.length)
        return;
      let _ = Date.now() - this.lastMove.time;
      _ < this.hoverTime ? this.hoverTimeout = setTimeout(this.checkHover, this.hoverTime - _) : this.startHover();
    }
    startHover() {
      clearTimeout(this.restartTimeout);
      let { view: _, lastMove: H } = this, V = _.docView.nearest(H.target);
      if (!V)
        return;
      let q, G = 1;
      if (V instanceof WidgetView)
        q = V.posAtStart;
      else {
        if (q = _.posAtCoords(H), q == null)
          return;
        let Z = _.coordsAtPos(q);
        if (!Z || H.y < Z.top || H.y > Z.bottom || H.x < Z.left - _.defaultCharacterWidth || H.x > Z.right + _.defaultCharacterWidth)
          return;
        let te = _.bidiSpans(_.state.doc.lineAt(q)).find((ee) => ee.from <= q && ee.to >= q), ne = te && te.dir == Direction.RTL ? -1 : 1;
        G = H.x < Z.left ? -ne : ne;
      }
      let J = this.source(_, q, G);
      if (J != null && J.then) {
        let Z = this.pending = { pos: q };
        J.then((te) => {
          this.pending == Z && (this.pending = null, te && !(Array.isArray(te) && !te.length) && _.dispatch({ effects: this.setHover.of(Array.isArray(te) ? te : [te]) }));
        }, (te) => logException(_.state, te, "hover tooltip"));
      } else J && !(Array.isArray(J) && !J.length) && _.dispatch({ effects: this.setHover.of(Array.isArray(J) ? J : [J]) });
    }
    get tooltip() {
      let _ = this.view.plugin(tooltipPlugin), H = _ ? _.manager.tooltips.findIndex((V) => V.create == HoverTooltipHost.create) : -1;
      return H > -1 ? _.manager.tooltipViews[H] : null;
    }
    mousemove(_) {
      var H, V;
      this.lastMove = { x: _.clientX, y: _.clientY, target: _.target, time: Date.now() }, this.hoverTimeout < 0 && (this.hoverTimeout = setTimeout(this.checkHover, this.hoverTime));
      let { active: q, tooltip: G } = this;
      if (q.length && G && !isInTooltip(G.dom, _) || this.pending) {
        let { pos: J } = q[0] || this.pending, Z = (V = (H = q[0]) === null || H === void 0 ? void 0 : H.end) !== null && V !== void 0 ? V : J;
        (J == Z ? this.view.posAtCoords(this.lastMove) != J : !isOverRange(this.view, J, Z, _.clientX, _.clientY)) && (this.view.dispatch({ effects: this.setHover.of([]) }), this.pending = null);
      }
    }
    mouseleave(_) {
      clearTimeout(this.hoverTimeout), this.hoverTimeout = -1;
      let { active: H } = this;
      if (H.length) {
        let { tooltip: V } = this;
        V && V.dom.contains(_.relatedTarget) ? this.watchTooltipLeave(V.dom) : this.view.dispatch({ effects: this.setHover.of([]) });
      }
    }
    watchTooltipLeave(_) {
      let H = (V) => {
        _.removeEventListener("mouseleave", H), this.active.length && !this.view.dom.contains(V.relatedTarget) && this.view.dispatch({ effects: this.setHover.of([]) });
      };
      _.addEventListener("mouseleave", H);
    }
    destroy() {
      clearTimeout(this.hoverTimeout), this.view.dom.removeEventListener("mouseleave", this.mouseleave), this.view.dom.removeEventListener("mousemove", this.mousemove);
    }
  }
  const tooltipMargin = 4;
  function isInTooltip(O, _) {
    let { left: H, right: V, top: q, bottom: G } = O.getBoundingClientRect(), J;
    if (J = O.querySelector(".cm-tooltip-arrow")) {
      let Z = J.getBoundingClientRect();
      q = Math.min(Z.top, q), G = Math.max(Z.bottom, G);
    }
    return _.clientX >= H - tooltipMargin && _.clientX <= V + tooltipMargin && _.clientY >= q - tooltipMargin && _.clientY <= G + tooltipMargin;
  }
  function isOverRange(O, _, H, V, q, G) {
    let J = O.scrollDOM.getBoundingClientRect(), Z = O.documentTop + O.documentPadding.top + O.contentHeight;
    if (J.left > V || J.right < V || J.top > q || Math.min(J.bottom, Z) < q)
      return !1;
    let te = O.posAtCoords({ x: V, y: q }, !1);
    return te >= _ && te <= H;
  }
  function hoverTooltip(O, _ = {}) {
    let H = StateEffect.define(), V = StateField.define({
      create() {
        return [];
      },
      update(q, G) {
        if (q.length && (_.hideOnChange && (G.docChanged || G.selection) ? q = [] : _.hideOn && (q = q.filter((J) => !_.hideOn(G, J))), G.docChanged)) {
          let J = [];
          for (let Z of q) {
            let te = G.changes.mapPos(Z.pos, -1, MapMode.TrackDel);
            if (te != null) {
              let ne = Object.assign(/* @__PURE__ */ Object.create(null), Z);
              ne.pos = te, ne.end != null && (ne.end = G.changes.mapPos(ne.end)), J.push(ne);
            }
          }
          q = J;
        }
        for (let J of G.effects)
          J.is(H) && (q = J.value), J.is(closeHoverTooltipEffect) && (q = []);
        return q;
      },
      provide: (q) => showHoverTooltip.from(q)
    });
    return {
      active: V,
      extension: [
        V,
        ViewPlugin.define((q) => new HoverPlugin(
          q,
          O,
          V,
          H,
          _.hoverTime || 300
          /* Hover.Time */
        )),
        showHoverTooltipHost
      ]
    };
  }
  function getTooltip(O, _) {
    let H = O.plugin(tooltipPlugin);
    if (!H)
      return null;
    let V = H.manager.tooltips.indexOf(_);
    return V < 0 ? null : H.manager.tooltipViews[V];
  }
  const closeHoverTooltipEffect = /* @__PURE__ */ StateEffect.define(), panelConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      let _, H;
      for (let V of O)
        _ = _ || V.topContainer, H = H || V.bottomContainer;
      return { topContainer: _, bottomContainer: H };
    }
  });
  function getPanel(O, _) {
    let H = O.plugin(panelPlugin), V = H ? H.specs.indexOf(_) : -1;
    return V > -1 ? H.panels[V] : null;
  }
  const panelPlugin = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.input = O.state.facet(showPanel), this.specs = this.input.filter((H) => H), this.panels = this.specs.map((H) => H(O));
      let _ = O.state.facet(panelConfig);
      this.top = new PanelGroup(O, !0, _.topContainer), this.bottom = new PanelGroup(O, !1, _.bottomContainer), this.top.sync(this.panels.filter((H) => H.top)), this.bottom.sync(this.panels.filter((H) => !H.top));
      for (let H of this.panels)
        H.dom.classList.add("cm-panel"), H.mount && H.mount();
    }
    update(O) {
      let _ = O.state.facet(panelConfig);
      this.top.container != _.topContainer && (this.top.sync([]), this.top = new PanelGroup(O.view, !0, _.topContainer)), this.bottom.container != _.bottomContainer && (this.bottom.sync([]), this.bottom = new PanelGroup(O.view, !1, _.bottomContainer)), this.top.syncClasses(), this.bottom.syncClasses();
      let H = O.state.facet(showPanel);
      if (H != this.input) {
        let V = H.filter((te) => te), q = [], G = [], J = [], Z = [];
        for (let te of V) {
          let ne = this.specs.indexOf(te), ee;
          ne < 0 ? (ee = te(O.view), Z.push(ee)) : (ee = this.panels[ne], ee.update && ee.update(O)), q.push(ee), (ee.top ? G : J).push(ee);
        }
        this.specs = V, this.panels = q, this.top.sync(G), this.bottom.sync(J);
        for (let te of Z)
          te.dom.classList.add("cm-panel"), te.mount && te.mount();
      } else
        for (let V of this.panels)
          V.update && V.update(O);
    }
    destroy() {
      this.top.sync([]), this.bottom.sync([]);
    }
  }, {
    provide: (O) => EditorView.scrollMargins.of((_) => {
      let H = _.plugin(O);
      return H && { top: H.top.scrollMargin(), bottom: H.bottom.scrollMargin() };
    })
  });
  class PanelGroup {
    constructor(_, H, V) {
      this.view = _, this.top = H, this.container = V, this.dom = void 0, this.classes = "", this.panels = [], this.syncClasses();
    }
    sync(_) {
      for (let H of this.panels)
        H.destroy && _.indexOf(H) < 0 && H.destroy();
      this.panels = _, this.syncDOM();
    }
    syncDOM() {
      if (this.panels.length == 0) {
        this.dom && (this.dom.remove(), this.dom = void 0);
        return;
      }
      if (!this.dom) {
        this.dom = document.createElement("div"), this.dom.className = this.top ? "cm-panels cm-panels-top" : "cm-panels cm-panels-bottom", this.dom.style[this.top ? "top" : "bottom"] = "0";
        let H = this.container || this.view.dom;
        H.insertBefore(this.dom, this.top ? H.firstChild : null);
      }
      let _ = this.dom.firstChild;
      for (let H of this.panels)
        if (H.dom.parentNode == this.dom) {
          for (; _ != H.dom; )
            _ = rm(_);
          _ = _.nextSibling;
        } else
          this.dom.insertBefore(H.dom, _);
      for (; _; )
        _ = rm(_);
    }
    scrollMargin() {
      return !this.dom || this.container ? 0 : Math.max(0, this.top ? this.dom.getBoundingClientRect().bottom - Math.max(0, this.view.scrollDOM.getBoundingClientRect().top) : Math.min(innerHeight, this.view.scrollDOM.getBoundingClientRect().bottom) - this.dom.getBoundingClientRect().top);
    }
    syncClasses() {
      if (!(!this.container || this.classes == this.view.themeClasses)) {
        for (let _ of this.classes.split(" "))
          _ && this.container.classList.remove(_);
        for (let _ of (this.classes = this.view.themeClasses).split(" "))
          _ && this.container.classList.add(_);
      }
    }
  }
  function rm(O) {
    let _ = O.nextSibling;
    return O.remove(), _;
  }
  const showPanel = /* @__PURE__ */ Facet.define({
    enables: panelPlugin
  });
  class GutterMarker extends RangeValue {
    /**
    @internal
    */
    compare(_) {
      return this == _ || this.constructor == _.constructor && this.eq(_);
    }
    /**
    Compare this marker to another marker of the same type.
    */
    eq(_) {
      return !1;
    }
    /**
    Called if the marker has a `toDOM` method and its representation
    was removed from a gutter.
    */
    destroy(_) {
    }
  }
  GutterMarker.prototype.elementClass = "";
  GutterMarker.prototype.toDOM = void 0;
  GutterMarker.prototype.mapMode = MapMode.TrackBefore;
  GutterMarker.prototype.startSide = GutterMarker.prototype.endSide = -1;
  GutterMarker.prototype.point = !0;
  const gutterLineClass = /* @__PURE__ */ Facet.define(), gutterWidgetClass = /* @__PURE__ */ Facet.define(), defaults$2 = {
    class: "",
    renderEmptyElements: !1,
    elementStyle: "",
    markers: () => RangeSet.empty,
    lineMarker: () => null,
    widgetMarker: () => null,
    lineMarkerChange: null,
    initialSpacer: null,
    updateSpacer: null,
    domEventHandlers: {}
  }, activeGutters = /* @__PURE__ */ Facet.define();
  function gutter(O) {
    return [gutters(), activeGutters.of(Object.assign(Object.assign({}, defaults$2), O))];
  }
  const unfixGutters = /* @__PURE__ */ Facet.define({
    combine: (O) => O.some((_) => _)
  });
  function gutters(O) {
    return [
      gutterView
    ];
  }
  const gutterView = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.view = O, this.prevViewport = O.viewport, this.dom = document.createElement("div"), this.dom.className = "cm-gutters", this.dom.setAttribute("aria-hidden", "true"), this.dom.style.minHeight = this.view.contentHeight / this.view.scaleY + "px", this.gutters = O.state.facet(activeGutters).map((_) => new SingleGutterView(O, _));
      for (let _ of this.gutters)
        this.dom.appendChild(_.dom);
      this.fixed = !O.state.facet(unfixGutters), this.fixed && (this.dom.style.position = "sticky"), this.syncGutters(!1), O.scrollDOM.insertBefore(this.dom, O.contentDOM);
    }
    update(O) {
      if (this.updateGutters(O)) {
        let _ = this.prevViewport, H = O.view.viewport, V = Math.min(_.to, H.to) - Math.max(_.from, H.from);
        this.syncGutters(V < (H.to - H.from) * 0.8);
      }
      O.geometryChanged && (this.dom.style.minHeight = this.view.contentHeight / this.view.scaleY + "px"), this.view.state.facet(unfixGutters) != !this.fixed && (this.fixed = !this.fixed, this.dom.style.position = this.fixed ? "sticky" : ""), this.prevViewport = O.view.viewport;
    }
    syncGutters(O) {
      let _ = this.dom.nextSibling;
      O && this.dom.remove();
      let H = RangeSet.iter(this.view.state.facet(gutterLineClass), this.view.viewport.from), V = [], q = this.gutters.map((G) => new UpdateContext(G, this.view.viewport, -this.view.documentPadding.top));
      for (let G of this.view.viewportLineBlocks)
        if (V.length && (V = []), Array.isArray(G.type)) {
          let J = !0;
          for (let Z of G.type)
            if (Z.type == BlockType.Text && J) {
              advanceCursor(H, V, Z.from);
              for (let te of q)
                te.line(this.view, Z, V);
              J = !1;
            } else if (Z.widget)
              for (let te of q)
                te.widget(this.view, Z);
        } else if (G.type == BlockType.Text) {
          advanceCursor(H, V, G.from);
          for (let J of q)
            J.line(this.view, G, V);
        } else if (G.widget)
          for (let J of q)
            J.widget(this.view, G);
      for (let G of q)
        G.finish();
      O && this.view.scrollDOM.insertBefore(this.dom, _);
    }
    updateGutters(O) {
      let _ = O.startState.facet(activeGutters), H = O.state.facet(activeGutters), V = O.docChanged || O.heightChanged || O.viewportChanged || !RangeSet.eq(O.startState.facet(gutterLineClass), O.state.facet(gutterLineClass), O.view.viewport.from, O.view.viewport.to);
      if (_ == H)
        for (let q of this.gutters)
          q.update(O) && (V = !0);
      else {
        V = !0;
        let q = [];
        for (let G of H) {
          let J = _.indexOf(G);
          J < 0 ? q.push(new SingleGutterView(this.view, G)) : (this.gutters[J].update(O), q.push(this.gutters[J]));
        }
        for (let G of this.gutters)
          G.dom.remove(), q.indexOf(G) < 0 && G.destroy();
        for (let G of q)
          this.dom.appendChild(G.dom);
        this.gutters = q;
      }
      return V;
    }
    destroy() {
      for (let O of this.gutters)
        O.destroy();
      this.dom.remove();
    }
  }, {
    provide: (O) => EditorView.scrollMargins.of((_) => {
      let H = _.plugin(O);
      return !H || H.gutters.length == 0 || !H.fixed ? null : _.textDirection == Direction.LTR ? { left: H.dom.offsetWidth * _.scaleX } : { right: H.dom.offsetWidth * _.scaleX };
    })
  });
  function asArray(O) {
    return Array.isArray(O) ? O : [O];
  }
  function advanceCursor(O, _, H) {
    for (; O.value && O.from <= H; )
      O.from == H && _.push(O.value), O.next();
  }
  class UpdateContext {
    constructor(_, H, V) {
      this.gutter = _, this.height = V, this.i = 0, this.cursor = RangeSet.iter(_.markers, H.from);
    }
    addElement(_, H, V) {
      let { gutter: q } = this, G = (H.top - this.height) / _.scaleY, J = H.height / _.scaleY;
      if (this.i == q.elements.length) {
        let Z = new GutterElement(_, J, G, V);
        q.elements.push(Z), q.dom.appendChild(Z.dom);
      } else
        q.elements[this.i].update(_, J, G, V);
      this.height = H.bottom, this.i++;
    }
    line(_, H, V) {
      let q = [];
      advanceCursor(this.cursor, q, H.from), V.length && (q = q.concat(V));
      let G = this.gutter.config.lineMarker(_, H, q);
      G && q.unshift(G);
      let J = this.gutter;
      q.length == 0 && !J.config.renderEmptyElements || this.addElement(_, H, q);
    }
    widget(_, H) {
      let V = this.gutter.config.widgetMarker(_, H.widget, H), q = V ? [V] : null;
      for (let G of _.state.facet(gutterWidgetClass)) {
        let J = G(_, H.widget, H);
        J && (q || (q = [])).push(J);
      }
      q && this.addElement(_, H, q);
    }
    finish() {
      let _ = this.gutter;
      for (; _.elements.length > this.i; ) {
        let H = _.elements.pop();
        _.dom.removeChild(H.dom), H.destroy();
      }
    }
  }
  class SingleGutterView {
    constructor(_, H) {
      this.view = _, this.config = H, this.elements = [], this.spacer = null, this.dom = document.createElement("div"), this.dom.className = "cm-gutter" + (this.config.class ? " " + this.config.class : "");
      for (let V in H.domEventHandlers)
        this.dom.addEventListener(V, (q) => {
          let G = q.target, J;
          if (G != this.dom && this.dom.contains(G)) {
            for (; G.parentNode != this.dom; )
              G = G.parentNode;
            let te = G.getBoundingClientRect();
            J = (te.top + te.bottom) / 2;
          } else
            J = q.clientY;
          let Z = _.lineBlockAtHeight(J - _.documentTop);
          H.domEventHandlers[V](_, Z, q) && q.preventDefault();
        });
      this.markers = asArray(H.markers(_)), H.initialSpacer && (this.spacer = new GutterElement(_, 0, 0, [H.initialSpacer(_)]), this.dom.appendChild(this.spacer.dom), this.spacer.dom.style.cssText += "visibility: hidden; pointer-events: none");
    }
    update(_) {
      let H = this.markers;
      if (this.markers = asArray(this.config.markers(_.view)), this.spacer && this.config.updateSpacer) {
        let q = this.config.updateSpacer(this.spacer.markers[0], _);
        q != this.spacer.markers[0] && this.spacer.update(_.view, 0, 0, [q]);
      }
      let V = _.view.viewport;
      return !RangeSet.eq(this.markers, H, V.from, V.to) || (this.config.lineMarkerChange ? this.config.lineMarkerChange(_) : !1);
    }
    destroy() {
      for (let _ of this.elements)
        _.destroy();
    }
  }
  class GutterElement {
    constructor(_, H, V, q) {
      this.height = -1, this.above = 0, this.markers = [], this.dom = document.createElement("div"), this.dom.className = "cm-gutterElement", this.update(_, H, V, q);
    }
    update(_, H, V, q) {
      this.height != H && (this.height = H, this.dom.style.height = H + "px"), this.above != V && (this.dom.style.marginTop = (this.above = V) ? V + "px" : ""), sameMarkers(this.markers, q) || this.setMarkers(_, q);
    }
    setMarkers(_, H) {
      let V = "cm-gutterElement", q = this.dom.firstChild;
      for (let G = 0, J = 0; ; ) {
        let Z = J, te = G < H.length ? H[G++] : null, ne = !1;
        if (te) {
          let ee = te.elementClass;
          ee && (V += " " + ee);
          for (let ae = J; ae < this.markers.length; ae++)
            if (this.markers[ae].compare(te)) {
              Z = ae, ne = !0;
              break;
            }
        } else
          Z = this.markers.length;
        for (; J < Z; ) {
          let ee = this.markers[J++];
          if (ee.toDOM) {
            ee.destroy(q);
            let ae = q.nextSibling;
            q.remove(), q = ae;
          }
        }
        if (!te)
          break;
        te.toDOM && (ne ? q = q.nextSibling : this.dom.insertBefore(te.toDOM(_), q)), ne && J++;
      }
      this.dom.className = V, this.markers = H;
    }
    destroy() {
      this.setMarkers(null, []);
    }
  }
  function sameMarkers(O, _) {
    if (O.length != _.length)
      return !1;
    for (let H = 0; H < O.length; H++)
      if (!O[H].compare(_[H]))
        return !1;
    return !0;
  }
  const lineNumberMarkers = /* @__PURE__ */ Facet.define(), lineNumberWidgetMarker = /* @__PURE__ */ Facet.define(), lineNumberConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, { formatNumber: String, domEventHandlers: {} }, {
        domEventHandlers(_, H) {
          let V = Object.assign({}, _);
          for (let q in H) {
            let G = V[q], J = H[q];
            V[q] = G ? (Z, te, ne) => G(Z, te, ne) || J(Z, te, ne) : J;
          }
          return V;
        }
      });
    }
  });
  class NumberMarker extends GutterMarker {
    constructor(_) {
      super(), this.number = _;
    }
    eq(_) {
      return this.number == _.number;
    }
    toDOM() {
      return document.createTextNode(this.number);
    }
  }
  function formatNumber(O, _) {
    return O.state.facet(lineNumberConfig).formatNumber(_, O.state);
  }
  const lineNumberGutter = /* @__PURE__ */ activeGutters.compute([lineNumberConfig], (O) => ({
    class: "cm-lineNumbers",
    renderEmptyElements: !1,
    markers(_) {
      return _.state.facet(lineNumberMarkers);
    },
    lineMarker(_, H, V) {
      return V.some((q) => q.toDOM) ? null : new NumberMarker(formatNumber(_, _.state.doc.lineAt(H.from).number));
    },
    widgetMarker: (_, H, V) => {
      for (let q of _.state.facet(lineNumberWidgetMarker)) {
        let G = q(_, H, V);
        if (G)
          return G;
      }
      return null;
    },
    lineMarkerChange: (_) => _.startState.facet(lineNumberConfig) != _.state.facet(lineNumberConfig),
    initialSpacer(_) {
      return new NumberMarker(formatNumber(_, maxLineNumber(_.state.doc.lines)));
    },
    updateSpacer(_, H) {
      let V = formatNumber(H.view, maxLineNumber(H.view.state.doc.lines));
      return V == _.number ? _ : new NumberMarker(V);
    },
    domEventHandlers: O.facet(lineNumberConfig).domEventHandlers
  }));
  function lineNumbers(O = {}) {
    return [
      lineNumberConfig.of(O),
      gutters(),
      lineNumberGutter
    ];
  }
  function maxLineNumber(O) {
    let _ = 9;
    for (; _ < O; )
      _ = _ * 10 + 9;
    return _;
  }
  const activeLineGutterMarker = /* @__PURE__ */ new class extends GutterMarker {
    constructor() {
      super(...arguments), this.elementClass = "cm-activeLineGutter";
    }
  }(), activeLineGutterHighlighter = /* @__PURE__ */ gutterLineClass.compute(["selection"], (O) => {
    let _ = [], H = -1;
    for (let V of O.selection.ranges) {
      let q = O.doc.lineAt(V.head).from;
      q > H && (H = q, _.push(activeLineGutterMarker.range(q)));
    }
    return RangeSet.of(_);
  });
  function highlightActiveLineGutter() {
    return activeLineGutterHighlighter;
  }
  let nextTagID = 0;
  class Tag {
    /**
    @internal
    */
    constructor(_, H, V, q) {
      this.name = _, this.set = H, this.base = V, this.modified = q, this.id = nextTagID++;
    }
    toString() {
      let { name: _ } = this;
      for (let H of this.modified)
        H.name && (_ = `${H.name}(${_})`);
      return _;
    }
    static define(_, H) {
      let V = typeof _ == "string" ? _ : "?";
      if (_ instanceof Tag && (H = _), H != null && H.base)
        throw new Error("Can not derive from a modified tag");
      let q = new Tag(V, [], null, []);
      if (q.set.push(q), H)
        for (let G of H.set)
          q.set.push(G);
      return q;
    }
    /**
    Define a tag _modifier_, which is a function that, given a tag,
    will return a tag that is a subtag of the original. Applying the
    same modifier to a twice tag will return the same value (`m1(t1)
    == m1(t1)`) and applying multiple modifiers will, regardless or
    order, produce the same tag (`m1(m2(t1)) == m2(m1(t1))`).
    
    When multiple modifiers are applied to a given base tag, each
    smaller set of modifiers is registered as a parent, so that for
    example `m1(m2(m3(t1)))` is a subtype of `m1(m2(t1))`,
    `m1(m3(t1)`, and so on.
    */
    static defineModifier(_) {
      let H = new Modifier(_);
      return (V) => V.modified.indexOf(H) > -1 ? V : Modifier.get(V.base || V, V.modified.concat(H).sort((q, G) => q.id - G.id));
    }
  }
  let nextModifierID = 0;
  class Modifier {
    constructor(_) {
      this.name = _, this.instances = [], this.id = nextModifierID++;
    }
    static get(_, H) {
      if (!H.length)
        return _;
      let V = H[0].instances.find((Z) => Z.base == _ && sameArray(H, Z.modified));
      if (V)
        return V;
      let q = [], G = new Tag(_.name, q, _, H);
      for (let Z of H)
        Z.instances.push(G);
      let J = powerSet(H);
      for (let Z of _.set)
        if (!Z.modified.length)
          for (let te of J)
            q.push(Modifier.get(Z, te));
      return G;
    }
  }
  function sameArray(O, _) {
    return O.length == _.length && O.every((H, V) => H == _[V]);
  }
  function powerSet(O) {
    let _ = [[]];
    for (let H = 0; H < O.length; H++)
      for (let V = 0, q = _.length; V < q; V++)
        _.push(_[V].concat(O[H]));
    return _.sort((H, V) => V.length - H.length);
  }
  function styleTags(O) {
    let _ = /* @__PURE__ */ Object.create(null);
    for (let H in O) {
      let V = O[H];
      Array.isArray(V) || (V = [V]);
      for (let q of H.split(" "))
        if (q) {
          let G = [], J = 2, Z = q;
          for (let ae = 0; ; ) {
            if (Z == "..." && ae > 0 && ae + 3 == q.length) {
              J = 1;
              break;
            }
            let oe = /^"(?:[^"\\]|\\.)*?"|[^\/!]+/.exec(Z);
            if (!oe)
              throw new RangeError("Invalid path: " + q);
            if (G.push(oe[0] == "*" ? "" : oe[0][0] == '"' ? JSON.parse(oe[0]) : oe[0]), ae += oe[0].length, ae == q.length)
              break;
            let se = q[ae++];
            if (ae == q.length && se == "!") {
              J = 0;
              break;
            }
            if (se != "/")
              throw new RangeError("Invalid path: " + q);
            Z = q.slice(ae);
          }
          let te = G.length - 1, ne = G[te];
          if (!ne)
            throw new RangeError("Invalid path: " + q);
          let ee = new Rule(V, J, te > 0 ? G.slice(0, te) : null);
          _[ne] = ee.sort(_[ne]);
        }
    }
    return ruleNodeProp.add(_);
  }
  const ruleNodeProp = new NodeProp();
  class Rule {
    constructor(_, H, V, q) {
      this.tags = _, this.mode = H, this.context = V, this.next = q;
    }
    get opaque() {
      return this.mode == 0;
    }
    get inherit() {
      return this.mode == 1;
    }
    sort(_) {
      return !_ || _.depth < this.depth ? (this.next = _, this) : (_.next = this.sort(_.next), _);
    }
    get depth() {
      return this.context ? this.context.length : 0;
    }
  }
  Rule.empty = new Rule([], 2, null);
  function tagHighlighter(O, _) {
    let H = /* @__PURE__ */ Object.create(null);
    for (let G of O)
      if (!Array.isArray(G.tag))
        H[G.tag.id] = G.class;
      else
        for (let J of G.tag)
          H[J.id] = G.class;
    let { scope: V, all: q = null } = _ || {};
    return {
      style: (G) => {
        let J = q;
        for (let Z of G)
          for (let te of Z.set) {
            let ne = H[te.id];
            if (ne) {
              J = J ? J + " " + ne : ne;
              break;
            }
          }
        return J;
      },
      scope: V
    };
  }
  function highlightTags(O, _) {
    let H = null;
    for (let V of O) {
      let q = V.style(_);
      q && (H = H ? H + " " + q : q);
    }
    return H;
  }
  function highlightTree(O, _, H, V = 0, q = O.length) {
    let G = new HighlightBuilder(V, Array.isArray(_) ? _ : [_], H);
    G.highlightRange(O.cursor(), V, q, "", G.highlighters), G.flush(q);
  }
  class HighlightBuilder {
    constructor(_, H, V) {
      this.at = _, this.highlighters = H, this.span = V, this.class = "";
    }
    startSpan(_, H) {
      H != this.class && (this.flush(_), _ > this.at && (this.at = _), this.class = H);
    }
    flush(_) {
      _ > this.at && this.class && this.span(this.at, _, this.class);
    }
    highlightRange(_, H, V, q, G) {
      let { type: J, from: Z, to: te } = _;
      if (Z >= V || te <= H)
        return;
      J.isTop && (G = this.highlighters.filter((se) => !se.scope || se.scope(J)));
      let ne = q, ee = getStyleTags(_) || Rule.empty, ae = highlightTags(G, ee.tags);
      if (ae && (ne && (ne += " "), ne += ae, ee.mode == 1 && (q += (q ? " " : "") + ae)), this.startSpan(Math.max(H, Z), ne), ee.opaque)
        return;
      let oe = _.tree && _.tree.prop(NodeProp.mounted);
      if (oe && oe.overlay) {
        let se = _.node.enter(oe.overlay[0].from + Z, 1), ce = this.highlighters.filter((ue) => !ue.scope || ue.scope(oe.tree.type)), le = _.firstChild();
        for (let ue = 0, de = Z; ; ue++) {
          let fe = ue < oe.overlay.length ? oe.overlay[ue] : null, he = fe ? fe.from + Z : te, me = Math.max(H, de), pe = Math.min(V, he);
          if (me < pe && le)
            for (; _.from < pe && (this.highlightRange(_, me, pe, q, G), this.startSpan(Math.min(pe, _.to), ne), !(_.to >= he || !_.nextSibling())); )
              ;
          if (!fe || he > V)
            break;
          de = fe.to + Z, de > H && (this.highlightRange(se.cursor(), Math.max(H, fe.from + Z), Math.min(V, de), "", ce), this.startSpan(Math.min(V, de), ne));
        }
        le && _.parent();
      } else if (_.firstChild()) {
        oe && (q = "");
        do
          if (!(_.to <= H)) {
            if (_.from >= V)
              break;
            this.highlightRange(_, H, V, q, G), this.startSpan(Math.min(V, _.to), ne);
          }
        while (_.nextSibling());
        _.parent();
      }
    }
  }
  function getStyleTags(O) {
    let _ = O.type.prop(ruleNodeProp);
    for (; _ && _.context && !O.matchContext(_.context); )
      _ = _.next;
    return _ || null;
  }
  const t = Tag.define, comment = t(), name$1 = t(), typeName = t(name$1), propertyName = t(name$1), literal = t(), string = t(literal), number = t(literal), content = t(), heading = t(content), keyword = t(), operator = t(), punctuation = t(), bracket = t(punctuation), meta = t(), tags = {
    /**
    A comment.
    */
    comment,
    /**
    A line [comment](#highlight.tags.comment).
    */
    lineComment: t(comment),
    /**
    A block [comment](#highlight.tags.comment).
    */
    blockComment: t(comment),
    /**
    A documentation [comment](#highlight.tags.comment).
    */
    docComment: t(comment),
    /**
    Any kind of identifier.
    */
    name: name$1,
    /**
    The [name](#highlight.tags.name) of a variable.
    */
    variableName: t(name$1),
    /**
    A type [name](#highlight.tags.name).
    */
    typeName,
    /**
    A tag name (subtag of [`typeName`](#highlight.tags.typeName)).
    */
    tagName: t(typeName),
    /**
    A property or field [name](#highlight.tags.name).
    */
    propertyName,
    /**
    An attribute name (subtag of [`propertyName`](#highlight.tags.propertyName)).
    */
    attributeName: t(propertyName),
    /**
    The [name](#highlight.tags.name) of a class.
    */
    className: t(name$1),
    /**
    A label [name](#highlight.tags.name).
    */
    labelName: t(name$1),
    /**
    A namespace [name](#highlight.tags.name).
    */
    namespace: t(name$1),
    /**
    The [name](#highlight.tags.name) of a macro.
    */
    macroName: t(name$1),
    /**
    A literal value.
    */
    literal,
    /**
    A string [literal](#highlight.tags.literal).
    */
    string,
    /**
    A documentation [string](#highlight.tags.string).
    */
    docString: t(string),
    /**
    A character literal (subtag of [string](#highlight.tags.string)).
    */
    character: t(string),
    /**
    An attribute value (subtag of [string](#highlight.tags.string)).
    */
    attributeValue: t(string),
    /**
    A number [literal](#highlight.tags.literal).
    */
    number,
    /**
    An integer [number](#highlight.tags.number) literal.
    */
    integer: t(number),
    /**
    A floating-point [number](#highlight.tags.number) literal.
    */
    float: t(number),
    /**
    A boolean [literal](#highlight.tags.literal).
    */
    bool: t(literal),
    /**
    Regular expression [literal](#highlight.tags.literal).
    */
    regexp: t(literal),
    /**
    An escape [literal](#highlight.tags.literal), for example a
    backslash escape in a string.
    */
    escape: t(literal),
    /**
    A color [literal](#highlight.tags.literal).
    */
    color: t(literal),
    /**
    A URL [literal](#highlight.tags.literal).
    */
    url: t(literal),
    /**
    A language keyword.
    */
    keyword,
    /**
    The [keyword](#highlight.tags.keyword) for the self or this
    object.
    */
    self: t(keyword),
    /**
    The [keyword](#highlight.tags.keyword) for null.
    */
    null: t(keyword),
    /**
    A [keyword](#highlight.tags.keyword) denoting some atomic value.
    */
    atom: t(keyword),
    /**
    A [keyword](#highlight.tags.keyword) that represents a unit.
    */
    unit: t(keyword),
    /**
    A modifier [keyword](#highlight.tags.keyword).
    */
    modifier: t(keyword),
    /**
    A [keyword](#highlight.tags.keyword) that acts as an operator.
    */
    operatorKeyword: t(keyword),
    /**
    A control-flow related [keyword](#highlight.tags.keyword).
    */
    controlKeyword: t(keyword),
    /**
    A [keyword](#highlight.tags.keyword) that defines something.
    */
    definitionKeyword: t(keyword),
    /**
    A [keyword](#highlight.tags.keyword) related to defining or
    interfacing with modules.
    */
    moduleKeyword: t(keyword),
    /**
    An operator.
    */
    operator,
    /**
    An [operator](#highlight.tags.operator) that dereferences something.
    */
    derefOperator: t(operator),
    /**
    Arithmetic-related [operator](#highlight.tags.operator).
    */
    arithmeticOperator: t(operator),
    /**
    Logical [operator](#highlight.tags.operator).
    */
    logicOperator: t(operator),
    /**
    Bit [operator](#highlight.tags.operator).
    */
    bitwiseOperator: t(operator),
    /**
    Comparison [operator](#highlight.tags.operator).
    */
    compareOperator: t(operator),
    /**
    [Operator](#highlight.tags.operator) that updates its operand.
    */
    updateOperator: t(operator),
    /**
    [Operator](#highlight.tags.operator) that defines something.
    */
    definitionOperator: t(operator),
    /**
    Type-related [operator](#highlight.tags.operator).
    */
    typeOperator: t(operator),
    /**
    Control-flow [operator](#highlight.tags.operator).
    */
    controlOperator: t(operator),
    /**
    Program or markup punctuation.
    */
    punctuation,
    /**
    [Punctuation](#highlight.tags.punctuation) that separates
    things.
    */
    separator: t(punctuation),
    /**
    Bracket-style [punctuation](#highlight.tags.punctuation).
    */
    bracket,
    /**
    Angle [brackets](#highlight.tags.bracket) (usually `<` and `>`
    tokens).
    */
    angleBracket: t(bracket),
    /**
    Square [brackets](#highlight.tags.bracket) (usually `[` and `]`
    tokens).
    */
    squareBracket: t(bracket),
    /**
    Parentheses (usually `(` and `)` tokens). Subtag of
    [bracket](#highlight.tags.bracket).
    */
    paren: t(bracket),
    /**
    Braces (usually `{` and `}` tokens). Subtag of
    [bracket](#highlight.tags.bracket).
    */
    brace: t(bracket),
    /**
    Content, for example plain text in XML or markup documents.
    */
    content,
    /**
    [Content](#highlight.tags.content) that represents a heading.
    */
    heading,
    /**
    A level 1 [heading](#highlight.tags.heading).
    */
    heading1: t(heading),
    /**
    A level 2 [heading](#highlight.tags.heading).
    */
    heading2: t(heading),
    /**
    A level 3 [heading](#highlight.tags.heading).
    */
    heading3: t(heading),
    /**
    A level 4 [heading](#highlight.tags.heading).
    */
    heading4: t(heading),
    /**
    A level 5 [heading](#highlight.tags.heading).
    */
    heading5: t(heading),
    /**
    A level 6 [heading](#highlight.tags.heading).
    */
    heading6: t(heading),
    /**
    A prose [content](#highlight.tags.content) separator (such as a horizontal rule).
    */
    contentSeparator: t(content),
    /**
    [Content](#highlight.tags.content) that represents a list.
    */
    list: t(content),
    /**
    [Content](#highlight.tags.content) that represents a quote.
    */
    quote: t(content),
    /**
    [Content](#highlight.tags.content) that is emphasized.
    */
    emphasis: t(content),
    /**
    [Content](#highlight.tags.content) that is styled strong.
    */
    strong: t(content),
    /**
    [Content](#highlight.tags.content) that is part of a link.
    */
    link: t(content),
    /**
    [Content](#highlight.tags.content) that is styled as code or
    monospace.
    */
    monospace: t(content),
    /**
    [Content](#highlight.tags.content) that has a strike-through
    style.
    */
    strikethrough: t(content),
    /**
    Inserted text in a change-tracking format.
    */
    inserted: t(),
    /**
    Deleted text.
    */
    deleted: t(),
    /**
    Changed text.
    */
    changed: t(),
    /**
    An invalid or unsyntactic element.
    */
    invalid: t(),
    /**
    Metadata or meta-instruction.
    */
    meta,
    /**
    [Metadata](#highlight.tags.meta) that applies to the entire
    document.
    */
    documentMeta: t(meta),
    /**
    [Metadata](#highlight.tags.meta) that annotates or adds
    attributes to a given syntactic element.
    */
    annotation: t(meta),
    /**
    Processing instruction or preprocessor directive. Subtag of
    [meta](#highlight.tags.meta).
    */
    processingInstruction: t(meta),
    /**
    [Modifier](#highlight.Tag^defineModifier) that indicates that a
    given element is being defined. Expected to be used with the
    various [name](#highlight.tags.name) tags.
    */
    definition: Tag.defineModifier("definition"),
    /**
    [Modifier](#highlight.Tag^defineModifier) that indicates that
    something is constant. Mostly expected to be used with
    [variable names](#highlight.tags.variableName).
    */
    constant: Tag.defineModifier("constant"),
    /**
    [Modifier](#highlight.Tag^defineModifier) used to indicate that
    a [variable](#highlight.tags.variableName) or [property
    name](#highlight.tags.propertyName) is being called or defined
    as a function.
    */
    function: Tag.defineModifier("function"),
    /**
    [Modifier](#highlight.Tag^defineModifier) that can be applied to
    [names](#highlight.tags.name) to indicate that they belong to
    the language's standard environment.
    */
    standard: Tag.defineModifier("standard"),
    /**
    [Modifier](#highlight.Tag^defineModifier) that indicates a given
    [names](#highlight.tags.name) is local to some scope.
    */
    local: Tag.defineModifier("local"),
    /**
    A generic variant [modifier](#highlight.Tag^defineModifier) that
    can be used to tag language-specific alternative variants of
    some common tag. It is recommended for themes to define special
    forms of at least the [string](#highlight.tags.string) and
    [variable name](#highlight.tags.variableName) tags, since those
    come up a lot.
    */
    special: Tag.defineModifier("special")
  };
  for (let O in tags) {
    let _ = tags[O];
    _ instanceof Tag && (_.name = O);
  }
  tagHighlighter([
    { tag: tags.link, class: "tok-link" },
    { tag: tags.heading, class: "tok-heading" },
    { tag: tags.emphasis, class: "tok-emphasis" },
    { tag: tags.strong, class: "tok-strong" },
    { tag: tags.keyword, class: "tok-keyword" },
    { tag: tags.atom, class: "tok-atom" },
    { tag: tags.bool, class: "tok-bool" },
    { tag: tags.url, class: "tok-url" },
    { tag: tags.labelName, class: "tok-labelName" },
    { tag: tags.inserted, class: "tok-inserted" },
    { tag: tags.deleted, class: "tok-deleted" },
    { tag: tags.literal, class: "tok-literal" },
    { tag: tags.string, class: "tok-string" },
    { tag: tags.number, class: "tok-number" },
    { tag: [tags.regexp, tags.escape, tags.special(tags.string)], class: "tok-string2" },
    { tag: tags.variableName, class: "tok-variableName" },
    { tag: tags.local(tags.variableName), class: "tok-variableName tok-local" },
    { tag: tags.definition(tags.variableName), class: "tok-variableName tok-definition" },
    { tag: tags.special(tags.variableName), class: "tok-variableName2" },
    { tag: tags.definition(tags.propertyName), class: "tok-propertyName tok-definition" },
    { tag: tags.typeName, class: "tok-typeName" },
    { tag: tags.namespace, class: "tok-namespace" },
    { tag: tags.className, class: "tok-className" },
    { tag: tags.macroName, class: "tok-macroName" },
    { tag: tags.propertyName, class: "tok-propertyName" },
    { tag: tags.operator, class: "tok-operator" },
    { tag: tags.comment, class: "tok-comment" },
    { tag: tags.meta, class: "tok-meta" },
    { tag: tags.invalid, class: "tok-invalid" },
    { tag: tags.punctuation, class: "tok-punctuation" }
  ]);
  var _a$4;
  const languageDataProp = /* @__PURE__ */ new NodeProp();
  function defineLanguageFacet(O) {
    return Facet.define({
      combine: O ? (_) => _.concat(O) : void 0
    });
  }
  const sublanguageProp = /* @__PURE__ */ new NodeProp();
  class Language {
    /**
    Construct a language object. If you need to invoke this
    directly, first define a data facet with
    [`defineLanguageFacet`](https://codemirror.net/6/docs/ref/#language.defineLanguageFacet), and then
    configure your parser to [attach](https://codemirror.net/6/docs/ref/#language.languageDataProp) it
    to the language's outer syntax node.
    */
    constructor(_, H, V = [], q = "") {
      this.data = _, this.name = q, EditorState.prototype.hasOwnProperty("tree") || Object.defineProperty(EditorState.prototype, "tree", { get() {
        return syntaxTree(this);
      } }), this.parser = H, this.extension = [
        language.of(this),
        EditorState.languageData.of((G, J, Z) => {
          let te = topNodeAt(G, J, Z), ne = te.type.prop(languageDataProp);
          if (!ne)
            return [];
          let ee = G.facet(ne), ae = te.type.prop(sublanguageProp);
          if (ae) {
            let oe = te.resolve(J - te.from, Z);
            for (let se of ae)
              if (se.test(oe, G)) {
                let ce = G.facet(se.facet);
                return se.type == "replace" ? ce : ce.concat(ee);
              }
          }
          return ee;
        })
      ].concat(V);
    }
    /**
    Query whether this language is active at the given position.
    */
    isActiveAt(_, H, V = -1) {
      return topNodeAt(_, H, V).type.prop(languageDataProp) == this.data;
    }
    /**
    Find the document regions that were parsed using this language.
    The returned regions will _include_ any nested languages rooted
    in this language, when those exist.
    */
    findRegions(_) {
      let H = _.facet(language);
      if ((H == null ? void 0 : H.data) == this.data)
        return [{ from: 0, to: _.doc.length }];
      if (!H || !H.allowsNesting)
        return [];
      let V = [], q = (G, J) => {
        if (G.prop(languageDataProp) == this.data) {
          V.push({ from: J, to: J + G.length });
          return;
        }
        let Z = G.prop(NodeProp.mounted);
        if (Z) {
          if (Z.tree.prop(languageDataProp) == this.data) {
            if (Z.overlay)
              for (let te of Z.overlay)
                V.push({ from: te.from + J, to: te.to + J });
            else
              V.push({ from: J, to: J + G.length });
            return;
          } else if (Z.overlay) {
            let te = V.length;
            if (q(Z.tree, Z.overlay[0].from + J), V.length > te)
              return;
          }
        }
        for (let te = 0; te < G.children.length; te++) {
          let ne = G.children[te];
          ne instanceof Tree && q(ne, G.positions[te] + J);
        }
      };
      return q(syntaxTree(_), 0), V;
    }
    /**
    Indicates whether this language allows nested languages. The
    default implementation returns true.
    */
    get allowsNesting() {
      return !0;
    }
  }
  Language.setState = /* @__PURE__ */ StateEffect.define();
  function topNodeAt(O, _, H) {
    let V = O.facet(language), q = syntaxTree(O).topNode;
    if (!V || V.allowsNesting)
      for (let G = q; G; G = G.enter(_, H, IterMode.ExcludeBuffers))
        G.type.isTop && (q = G);
    return q;
  }
  class LRLanguage extends Language {
    constructor(_, H, V) {
      super(_, H, [], V), this.parser = H;
    }
    /**
    Define a language from a parser.
    */
    static define(_) {
      let H = defineLanguageFacet(_.languageData);
      return new LRLanguage(H, _.parser.configure({
        props: [languageDataProp.add((V) => V.isTop ? H : void 0)]
      }), _.name);
    }
    /**
    Create a new instance of this language with a reconfigured
    version of its parser and optionally a new name.
    */
    configure(_, H) {
      return new LRLanguage(this.data, this.parser.configure(_), H || this.name);
    }
    get allowsNesting() {
      return this.parser.hasWrappers();
    }
  }
  function syntaxTree(O) {
    let _ = O.field(Language.state, !1);
    return _ ? _.tree : Tree.empty;
  }
  class DocInput {
    /**
    Create an input object for the given document.
    */
    constructor(_) {
      this.doc = _, this.cursorPos = 0, this.string = "", this.cursor = _.iter();
    }
    get length() {
      return this.doc.length;
    }
    syncTo(_) {
      return this.string = this.cursor.next(_ - this.cursorPos).value, this.cursorPos = _ + this.string.length, this.cursorPos - this.string.length;
    }
    chunk(_) {
      return this.syncTo(_), this.string;
    }
    get lineChunks() {
      return !0;
    }
    read(_, H) {
      let V = this.cursorPos - this.string.length;
      return _ < V || H >= this.cursorPos ? this.doc.sliceString(_, H) : this.string.slice(_ - V, H - V);
    }
  }
  let currentContext = null;
  class ParseContext {
    constructor(_, H, V = [], q, G, J, Z, te) {
      this.parser = _, this.state = H, this.fragments = V, this.tree = q, this.treeLen = G, this.viewport = J, this.skipped = Z, this.scheduleOn = te, this.parse = null, this.tempSkipped = [];
    }
    /**
    @internal
    */
    static create(_, H, V) {
      return new ParseContext(_, H, [], Tree.empty, 0, V, [], null);
    }
    startParse() {
      return this.parser.startParse(new DocInput(this.state.doc), this.fragments);
    }
    /**
    @internal
    */
    work(_, H) {
      return H != null && H >= this.state.doc.length && (H = void 0), this.tree != Tree.empty && this.isDone(H ?? this.state.doc.length) ? (this.takeTree(), !0) : this.withContext(() => {
        var V;
        if (typeof _ == "number") {
          let q = Date.now() + _;
          _ = () => Date.now() > q;
        }
        for (this.parse || (this.parse = this.startParse()), H != null && (this.parse.stoppedAt == null || this.parse.stoppedAt > H) && H < this.state.doc.length && this.parse.stopAt(H); ; ) {
          let q = this.parse.advance();
          if (q)
            if (this.fragments = this.withoutTempSkipped(TreeFragment.addTree(q, this.fragments, this.parse.stoppedAt != null)), this.treeLen = (V = this.parse.stoppedAt) !== null && V !== void 0 ? V : this.state.doc.length, this.tree = q, this.parse = null, this.treeLen < (H ?? this.state.doc.length))
              this.parse = this.startParse();
            else
              return !0;
          if (_())
            return !1;
        }
      });
    }
    /**
    @internal
    */
    takeTree() {
      let _, H;
      this.parse && (_ = this.parse.parsedPos) >= this.treeLen && ((this.parse.stoppedAt == null || this.parse.stoppedAt > _) && this.parse.stopAt(_), this.withContext(() => {
        for (; !(H = this.parse.advance()); )
          ;
      }), this.treeLen = _, this.tree = H, this.fragments = this.withoutTempSkipped(TreeFragment.addTree(this.tree, this.fragments, !0)), this.parse = null);
    }
    withContext(_) {
      let H = currentContext;
      currentContext = this;
      try {
        return _();
      } finally {
        currentContext = H;
      }
    }
    withoutTempSkipped(_) {
      for (let H; H = this.tempSkipped.pop(); )
        _ = cutFragments(_, H.from, H.to);
      return _;
    }
    /**
    @internal
    */
    changes(_, H) {
      let { fragments: V, tree: q, treeLen: G, viewport: J, skipped: Z } = this;
      if (this.takeTree(), !_.empty) {
        let te = [];
        if (_.iterChangedRanges((ne, ee, ae, oe) => te.push({ fromA: ne, toA: ee, fromB: ae, toB: oe })), V = TreeFragment.applyChanges(V, te), q = Tree.empty, G = 0, J = { from: _.mapPos(J.from, -1), to: _.mapPos(J.to, 1) }, this.skipped.length) {
          Z = [];
          for (let ne of this.skipped) {
            let ee = _.mapPos(ne.from, 1), ae = _.mapPos(ne.to, -1);
            ee < ae && Z.push({ from: ee, to: ae });
          }
        }
      }
      return new ParseContext(this.parser, H, V, q, G, J, Z, this.scheduleOn);
    }
    /**
    @internal
    */
    updateViewport(_) {
      if (this.viewport.from == _.from && this.viewport.to == _.to)
        return !1;
      this.viewport = _;
      let H = this.skipped.length;
      for (let V = 0; V < this.skipped.length; V++) {
        let { from: q, to: G } = this.skipped[V];
        q < _.to && G > _.from && (this.fragments = cutFragments(this.fragments, q, G), this.skipped.splice(V--, 1));
      }
      return this.skipped.length >= H ? !1 : (this.reset(), !0);
    }
    /**
    @internal
    */
    reset() {
      this.parse && (this.takeTree(), this.parse = null);
    }
    /**
    Notify the parse scheduler that the given region was skipped
    because it wasn't in view, and the parse should be restarted
    when it comes into view.
    */
    skipUntilInView(_, H) {
      this.skipped.push({ from: _, to: H });
    }
    /**
    Returns a parser intended to be used as placeholder when
    asynchronously loading a nested parser. It'll skip its input and
    mark it as not-really-parsed, so that the next update will parse
    it again.
    
    When `until` is given, a reparse will be scheduled when that
    promise resolves.
    */
    static getSkippingParser(_) {
      return new class extends Parser {
        createParse(H, V, q) {
          let G = q[0].from, J = q[q.length - 1].to;
          return {
            parsedPos: G,
            advance() {
              let te = currentContext;
              if (te) {
                for (let ne of q)
                  te.tempSkipped.push(ne);
                _ && (te.scheduleOn = te.scheduleOn ? Promise.all([te.scheduleOn, _]) : _);
              }
              return this.parsedPos = J, new Tree(NodeType.none, [], [], J - G);
            },
            stoppedAt: null,
            stopAt() {
            }
          };
        }
      }();
    }
    /**
    @internal
    */
    isDone(_) {
      _ = Math.min(_, this.state.doc.length);
      let H = this.fragments;
      return this.treeLen >= _ && H.length && H[0].from == 0 && H[0].to >= _;
    }
    /**
    Get the context for the current parse, or `null` if no editor
    parse is in progress.
    */
    static get() {
      return currentContext;
    }
  }
  function cutFragments(O, _, H) {
    return TreeFragment.applyChanges(O, [{ fromA: _, toA: H, fromB: _, toB: H }]);
  }
  class LanguageState {
    constructor(_) {
      this.context = _, this.tree = _.tree;
    }
    apply(_) {
      if (!_.docChanged && this.tree == this.context.tree)
        return this;
      let H = this.context.changes(_.changes, _.state), V = this.context.treeLen == _.startState.doc.length ? void 0 : Math.max(_.changes.mapPos(this.context.treeLen), H.viewport.to);
      return H.work(20, V) || H.takeTree(), new LanguageState(H);
    }
    static init(_) {
      let H = Math.min(3e3, _.doc.length), V = ParseContext.create(_.facet(language).parser, _, { from: 0, to: H });
      return V.work(20, H) || V.takeTree(), new LanguageState(V);
    }
  }
  Language.state = /* @__PURE__ */ StateField.define({
    create: LanguageState.init,
    update(O, _) {
      for (let H of _.effects)
        if (H.is(Language.setState))
          return H.value;
      return _.startState.facet(language) != _.state.facet(language) ? LanguageState.init(_.state) : O.apply(_);
    }
  });
  let requestIdle = (O) => {
    let _ = setTimeout(
      () => O(),
      500
      /* Work.MaxPause */
    );
    return () => clearTimeout(_);
  };
  typeof requestIdleCallback < "u" && (requestIdle = (O) => {
    let _ = -1, H = setTimeout(
      () => {
        _ = requestIdleCallback(O, {
          timeout: 400
          /* Work.MinPause */
        });
      },
      100
      /* Work.MinPause */
    );
    return () => _ < 0 ? clearTimeout(H) : cancelIdleCallback(_);
  });
  const isInputPending = typeof navigator < "u" && (!((_a$4 = navigator.scheduling) === null || _a$4 === void 0) && _a$4.isInputPending) ? () => navigator.scheduling.isInputPending() : null, parseWorker = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(_) {
      this.view = _, this.working = null, this.workScheduled = 0, this.chunkEnd = -1, this.chunkBudget = -1, this.work = this.work.bind(this), this.scheduleWork();
    }
    update(_) {
      let H = this.view.state.field(Language.state).context;
      (H.updateViewport(_.view.viewport) || this.view.viewport.to > H.treeLen) && this.scheduleWork(), (_.docChanged || _.selectionSet) && (this.view.hasFocus && (this.chunkBudget += 50), this.scheduleWork()), this.checkAsyncSchedule(H);
    }
    scheduleWork() {
      if (this.working)
        return;
      let { state: _ } = this.view, H = _.field(Language.state);
      (H.tree != H.context.tree || !H.context.isDone(_.doc.length)) && (this.working = requestIdle(this.work));
    }
    work(_) {
      this.working = null;
      let H = Date.now();
      if (this.chunkEnd < H && (this.chunkEnd < 0 || this.view.hasFocus) && (this.chunkEnd = H + 3e4, this.chunkBudget = 3e3), this.chunkBudget <= 0)
        return;
      let { state: V, viewport: { to: q } } = this.view, G = V.field(Language.state);
      if (G.tree == G.context.tree && G.context.isDone(
        q + 1e5
        /* Work.MaxParseAhead */
      ))
        return;
      let J = Date.now() + Math.min(this.chunkBudget, 100, _ && !isInputPending ? Math.max(25, _.timeRemaining() - 5) : 1e9), Z = G.context.treeLen < q && V.doc.length > q + 1e3, te = G.context.work(() => isInputPending && isInputPending() || Date.now() > J, q + (Z ? 0 : 1e5));
      this.chunkBudget -= Date.now() - H, (te || this.chunkBudget <= 0) && (G.context.takeTree(), this.view.dispatch({ effects: Language.setState.of(new LanguageState(G.context)) })), this.chunkBudget > 0 && !(te && !Z) && this.scheduleWork(), this.checkAsyncSchedule(G.context);
    }
    checkAsyncSchedule(_) {
      _.scheduleOn && (this.workScheduled++, _.scheduleOn.then(() => this.scheduleWork()).catch((H) => logException(this.view.state, H)).then(() => this.workScheduled--), _.scheduleOn = null);
    }
    destroy() {
      this.working && this.working();
    }
    isWorking() {
      return !!(this.working || this.workScheduled > 0);
    }
  }, {
    eventHandlers: { focus() {
      this.scheduleWork();
    } }
  }), language = /* @__PURE__ */ Facet.define({
    combine(O) {
      return O.length ? O[0] : null;
    },
    enables: (O) => [
      Language.state,
      parseWorker,
      EditorView.contentAttributes.compute([O], (_) => {
        let H = _.facet(O);
        return H && H.name ? { "data-language": H.name } : {};
      })
    ]
  });
  class LanguageSupport {
    /**
    Create a language support object.
    */
    constructor(_, H = []) {
      this.language = _, this.support = H, this.extension = [_, H];
    }
  }
  const indentService = /* @__PURE__ */ Facet.define(), indentUnit = /* @__PURE__ */ Facet.define({
    combine: (O) => {
      if (!O.length)
        return "  ";
      let _ = O[0];
      if (!_ || /\S/.test(_) || Array.from(_).some((H) => H != _[0]))
        throw new Error("Invalid indent unit: " + JSON.stringify(O[0]));
      return _;
    }
  });
  function getIndentUnit(O) {
    let _ = O.facet(indentUnit);
    return _.charCodeAt(0) == 9 ? O.tabSize * _.length : _.length;
  }
  function indentString(O, _) {
    let H = "", V = O.tabSize, q = O.facet(indentUnit)[0];
    if (q == "	") {
      for (; _ >= V; )
        H += "	", _ -= V;
      q = " ";
    }
    for (let G = 0; G < _; G++)
      H += q;
    return H;
  }
  function getIndentation(O, _) {
    O instanceof EditorState && (O = new IndentContext(O));
    for (let V of O.state.facet(indentService)) {
      let q = V(O, _);
      if (q !== void 0)
        return q;
    }
    let H = syntaxTree(O.state);
    return H.length >= _ ? syntaxIndentation(O, H, _) : null;
  }
  class IndentContext {
    /**
    Create an indent context.
    */
    constructor(_, H = {}) {
      this.state = _, this.options = H, this.unit = getIndentUnit(_);
    }
    /**
    Get a description of the line at the given position, taking
    [simulated line
    breaks](https://codemirror.net/6/docs/ref/#language.IndentContext.constructor^options.simulateBreak)
    into account. If there is such a break at `pos`, the `bias`
    argument determines whether the part of the line line before or
    after the break is used.
    */
    lineAt(_, H = 1) {
      let V = this.state.doc.lineAt(_), { simulateBreak: q, simulateDoubleBreak: G } = this.options;
      return q != null && q >= V.from && q <= V.to ? G && q == _ ? { text: "", from: _ } : (H < 0 ? q < _ : q <= _) ? { text: V.text.slice(q - V.from), from: q } : { text: V.text.slice(0, q - V.from), from: V.from } : V;
    }
    /**
    Get the text directly after `pos`, either the entire line
    or the next 100 characters, whichever is shorter.
    */
    textAfterPos(_, H = 1) {
      if (this.options.simulateDoubleBreak && _ == this.options.simulateBreak)
        return "";
      let { text: V, from: q } = this.lineAt(_, H);
      return V.slice(_ - q, Math.min(V.length, _ + 100 - q));
    }
    /**
    Find the column for the given position.
    */
    column(_, H = 1) {
      let { text: V, from: q } = this.lineAt(_, H), G = this.countColumn(V, _ - q), J = this.options.overrideIndentation ? this.options.overrideIndentation(q) : -1;
      return J > -1 && (G += J - this.countColumn(V, V.search(/\S|$/))), G;
    }
    /**
    Find the column position (taking tabs into account) of the given
    position in the given string.
    */
    countColumn(_, H = _.length) {
      return countColumn(_, this.state.tabSize, H);
    }
    /**
    Find the indentation column of the line at the given point.
    */
    lineIndent(_, H = 1) {
      let { text: V, from: q } = this.lineAt(_, H), G = this.options.overrideIndentation;
      if (G) {
        let J = G(q);
        if (J > -1)
          return J;
      }
      return this.countColumn(V, V.search(/\S|$/));
    }
    /**
    Returns the [simulated line
    break](https://codemirror.net/6/docs/ref/#language.IndentContext.constructor^options.simulateBreak)
    for this context, if any.
    */
    get simulatedBreak() {
      return this.options.simulateBreak || null;
    }
  }
  const indentNodeProp = /* @__PURE__ */ new NodeProp();
  function syntaxIndentation(O, _, H) {
    let V = _.resolveStack(H), q = V.node.enterUnfinishedNodesBefore(H);
    if (q != V.node) {
      let G = [];
      for (let J = q; J != V.node; J = J.parent)
        G.push(J);
      for (let J = G.length - 1; J >= 0; J--)
        V = { node: G[J], next: V };
    }
    return indentFor(V, O, H);
  }
  function indentFor(O, _, H) {
    for (let V = O; V; V = V.next) {
      let q = indentStrategy(V.node);
      if (q)
        return q(TreeIndentContext.create(_, H, V));
    }
    return 0;
  }
  function ignoreClosed(O) {
    return O.pos == O.options.simulateBreak && O.options.simulateDoubleBreak;
  }
  function indentStrategy(O) {
    let _ = O.type.prop(indentNodeProp);
    if (_)
      return _;
    let H = O.firstChild, V;
    if (H && (V = H.type.prop(NodeProp.closedBy))) {
      let q = O.lastChild, G = q && V.indexOf(q.name) > -1;
      return (J) => delimitedStrategy(J, !0, 1, void 0, G && !ignoreClosed(J) ? q.from : void 0);
    }
    return O.parent == null ? topIndent : null;
  }
  function topIndent() {
    return 0;
  }
  class TreeIndentContext extends IndentContext {
    constructor(_, H, V) {
      super(_.state, _.options), this.base = _, this.pos = H, this.context = V;
    }
    /**
    The syntax tree node to which the indentation strategy
    applies.
    */
    get node() {
      return this.context.node;
    }
    /**
    @internal
    */
    static create(_, H, V) {
      return new TreeIndentContext(_, H, V);
    }
    /**
    Get the text directly after `this.pos`, either the entire line
    or the next 100 characters, whichever is shorter.
    */
    get textAfter() {
      return this.textAfterPos(this.pos);
    }
    /**
    Get the indentation at the reference line for `this.node`, which
    is the line on which it starts, unless there is a node that is
    _not_ a parent of this node covering the start of that line. If
    so, the line at the start of that node is tried, again skipping
    on if it is covered by another such node.
    */
    get baseIndent() {
      return this.baseIndentFor(this.node);
    }
    /**
    Get the indentation for the reference line of the given node
    (see [`baseIndent`](https://codemirror.net/6/docs/ref/#language.TreeIndentContext.baseIndent)).
    */
    baseIndentFor(_) {
      let H = this.state.doc.lineAt(_.from);
      for (; ; ) {
        let V = _.resolve(H.from);
        for (; V.parent && V.parent.from == V.from; )
          V = V.parent;
        if (isParent(V, _))
          break;
        H = this.state.doc.lineAt(V.from);
      }
      return this.lineIndent(H.from);
    }
    /**
    Continue looking for indentations in the node's parent nodes,
    and return the result of that.
    */
    continue() {
      return indentFor(this.context.next, this.base, this.pos);
    }
  }
  function isParent(O, _) {
    for (let H = _; H; H = H.parent)
      if (O == H)
        return !0;
    return !1;
  }
  function bracketedAligned(O) {
    let _ = O.node, H = _.childAfter(_.from), V = _.lastChild;
    if (!H)
      return null;
    let q = O.options.simulateBreak, G = O.state.doc.lineAt(H.from), J = q == null || q <= G.from ? G.to : Math.min(G.to, q);
    for (let Z = H.to; ; ) {
      let te = _.childAfter(Z);
      if (!te || te == V)
        return null;
      if (!te.type.isSkipped) {
        if (te.from >= J)
          return null;
        let ne = /^ */.exec(G.text.slice(H.to - G.from))[0].length;
        return { from: H.from, to: H.to + ne };
      }
      Z = te.to;
    }
  }
  function delimitedStrategy(O, _, H, V, q) {
    let G = O.textAfter, J = G.match(/^\s*/)[0].length, Z = q == O.pos + J, te = bracketedAligned(O);
    return te ? Z ? O.column(te.from) : O.column(te.to) : O.baseIndent + (Z ? 0 : O.unit * H);
  }
  function continuedIndent({ except: O, units: _ = 1 } = {}) {
    return (H) => {
      let V = O && O.test(H.textAfter);
      return H.baseIndent + (V ? 0 : _ * H.unit);
    };
  }
  const DontIndentBeyond = 200;
  function indentOnInput() {
    return EditorState.transactionFilter.of((O) => {
      if (!O.docChanged || !O.isUserEvent("input.type") && !O.isUserEvent("input.complete"))
        return O;
      let _ = O.startState.languageDataAt("indentOnInput", O.startState.selection.main.head);
      if (!_.length)
        return O;
      let H = O.newDoc, { head: V } = O.newSelection.main, q = H.lineAt(V);
      if (V > q.from + DontIndentBeyond)
        return O;
      let G = H.sliceString(q.from, V);
      if (!_.some((ne) => ne.test(G)))
        return O;
      let { state: J } = O, Z = -1, te = [];
      for (let { head: ne } of J.selection.ranges) {
        let ee = J.doc.lineAt(ne);
        if (ee.from == Z)
          continue;
        Z = ee.from;
        let ae = getIndentation(J, ee.from);
        if (ae == null)
          continue;
        let oe = /^\s*/.exec(ee.text)[0], se = indentString(J, ae);
        oe != se && te.push({ from: ee.from, to: ee.from + oe.length, insert: se });
      }
      return te.length ? [O, { changes: te, sequential: !0 }] : O;
    });
  }
  const foldService = /* @__PURE__ */ Facet.define(), foldNodeProp = /* @__PURE__ */ new NodeProp();
  function syntaxFolding(O, _, H) {
    let V = syntaxTree(O);
    if (V.length < H)
      return null;
    let q = V.resolveStack(H, 1), G = null;
    for (let J = q; J; J = J.next) {
      let Z = J.node;
      if (Z.to <= H || Z.from > H)
        continue;
      if (G && Z.from < _)
        break;
      let te = Z.type.prop(foldNodeProp);
      if (te && (Z.to < V.length - 50 || V.length == O.doc.length || !isUnfinished(Z))) {
        let ne = te(Z, O);
        ne && ne.from <= H && ne.from >= _ && ne.to > H && (G = ne);
      }
    }
    return G;
  }
  function isUnfinished(O) {
    let _ = O.lastChild;
    return _ && _.to == O.to && _.type.isError;
  }
  function foldable(O, _, H) {
    for (let V of O.facet(foldService)) {
      let q = V(O, _, H);
      if (q)
        return q;
    }
    return syntaxFolding(O, _, H);
  }
  function mapRange(O, _) {
    let H = _.mapPos(O.from, 1), V = _.mapPos(O.to, -1);
    return H >= V ? void 0 : { from: H, to: V };
  }
  const foldEffect = /* @__PURE__ */ StateEffect.define({ map: mapRange }), unfoldEffect = /* @__PURE__ */ StateEffect.define({ map: mapRange });
  function selectedLines(O) {
    let _ = [];
    for (let { head: H } of O.state.selection.ranges)
      _.some((V) => V.from <= H && V.to >= H) || _.push(O.lineBlockAt(H));
    return _;
  }
  const foldState = /* @__PURE__ */ StateField.define({
    create() {
      return Decoration.none;
    },
    update(O, _) {
      O = O.map(_.changes);
      for (let H of _.effects)
        if (H.is(foldEffect) && !foldExists(O, H.value.from, H.value.to)) {
          let { preparePlaceholder: V } = _.state.facet(foldConfig), q = V ? Decoration.replace({ widget: new PreparedFoldWidget(V(_.state, H.value)) }) : foldWidget;
          O = O.update({ add: [q.range(H.value.from, H.value.to)] });
        } else H.is(unfoldEffect) && (O = O.update({
          filter: (V, q) => H.value.from != V || H.value.to != q,
          filterFrom: H.value.from,
          filterTo: H.value.to
        }));
      if (_.selection) {
        let H = !1, { head: V } = _.selection.main;
        O.between(V, V, (q, G) => {
          q < V && G > V && (H = !0);
        }), H && (O = O.update({
          filterFrom: V,
          filterTo: V,
          filter: (q, G) => G <= V || q >= V
        }));
      }
      return O;
    },
    provide: (O) => EditorView.decorations.from(O),
    toJSON(O, _) {
      let H = [];
      return O.between(0, _.doc.length, (V, q) => {
        H.push(V, q);
      }), H;
    },
    fromJSON(O) {
      if (!Array.isArray(O) || O.length % 2)
        throw new RangeError("Invalid JSON for fold state");
      let _ = [];
      for (let H = 0; H < O.length; ) {
        let V = O[H++], q = O[H++];
        if (typeof V != "number" || typeof q != "number")
          throw new RangeError("Invalid JSON for fold state");
        _.push(foldWidget.range(V, q));
      }
      return Decoration.set(_, !0);
    }
  });
  function findFold(O, _, H) {
    var V;
    let q = null;
    return (V = O.field(foldState, !1)) === null || V === void 0 || V.between(_, H, (G, J) => {
      (!q || q.from > G) && (q = { from: G, to: J });
    }), q;
  }
  function foldExists(O, _, H) {
    let V = !1;
    return O.between(_, _, (q, G) => {
      q == _ && G == H && (V = !0);
    }), V;
  }
  function maybeEnable(O, _) {
    return O.field(foldState, !1) ? _ : _.concat(StateEffect.appendConfig.of(codeFolding()));
  }
  const foldCode = (O) => {
    for (let _ of selectedLines(O)) {
      let H = foldable(O.state, _.from, _.to);
      if (H)
        return O.dispatch({ effects: maybeEnable(O.state, [foldEffect.of(H), announceFold(O, H)]) }), !0;
    }
    return !1;
  }, unfoldCode = (O) => {
    if (!O.state.field(foldState, !1))
      return !1;
    let _ = [];
    for (let H of selectedLines(O)) {
      let V = findFold(O.state, H.from, H.to);
      V && _.push(unfoldEffect.of(V), announceFold(O, V, !1));
    }
    return _.length && O.dispatch({ effects: _ }), _.length > 0;
  };
  function announceFold(O, _, H = !0) {
    let V = O.state.doc.lineAt(_.from).number, q = O.state.doc.lineAt(_.to).number;
    return EditorView.announce.of(`${O.state.phrase(H ? "Folded lines" : "Unfolded lines")} ${V} ${O.state.phrase("to")} ${q}.`);
  }
  const foldAll = (O) => {
    let { state: _ } = O, H = [];
    for (let V = 0; V < _.doc.length; ) {
      let q = O.lineBlockAt(V), G = foldable(_, q.from, q.to);
      G && H.push(foldEffect.of(G)), V = (G ? O.lineBlockAt(G.to) : q).to + 1;
    }
    return H.length && O.dispatch({ effects: maybeEnable(O.state, H) }), !!H.length;
  }, unfoldAll = (O) => {
    let _ = O.state.field(foldState, !1);
    if (!_ || !_.size)
      return !1;
    let H = [];
    return _.between(0, O.state.doc.length, (V, q) => {
      H.push(unfoldEffect.of({ from: V, to: q }));
    }), O.dispatch({ effects: H }), !0;
  }, foldKeymap = [
    { key: "Ctrl-Shift-[", mac: "Cmd-Alt-[", run: foldCode },
    { key: "Ctrl-Shift-]", mac: "Cmd-Alt-]", run: unfoldCode },
    { key: "Ctrl-Alt-[", run: foldAll },
    { key: "Ctrl-Alt-]", run: unfoldAll }
  ], defaultConfig = {
    placeholderDOM: null,
    preparePlaceholder: null,
    placeholderText: "…"
  }, foldConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, defaultConfig);
    }
  });
  function codeFolding(O) {
    return [foldState, baseTheme$1$1];
  }
  function widgetToDOM(O, _) {
    let { state: H } = O, V = H.facet(foldConfig), q = (J) => {
      let Z = O.lineBlockAt(O.posAtDOM(J.target)), te = findFold(O.state, Z.from, Z.to);
      te && O.dispatch({ effects: unfoldEffect.of(te) }), J.preventDefault();
    };
    if (V.placeholderDOM)
      return V.placeholderDOM(O, q, _);
    let G = document.createElement("span");
    return G.textContent = V.placeholderText, G.setAttribute("aria-label", H.phrase("folded code")), G.title = H.phrase("unfold"), G.className = "cm-foldPlaceholder", G.onclick = q, G;
  }
  const foldWidget = /* @__PURE__ */ Decoration.replace({ widget: /* @__PURE__ */ new class extends WidgetType {
    toDOM(O) {
      return widgetToDOM(O, null);
    }
  }() });
  class PreparedFoldWidget extends WidgetType {
    constructor(_) {
      super(), this.value = _;
    }
    eq(_) {
      return this.value == _.value;
    }
    toDOM(_) {
      return widgetToDOM(_, this.value);
    }
  }
  const foldGutterDefaults = {
    openText: "⌄",
    closedText: "›",
    markerDOM: null,
    domEventHandlers: {},
    foldingChanged: () => !1
  };
  class FoldMarker extends GutterMarker {
    constructor(_, H) {
      super(), this.config = _, this.open = H;
    }
    eq(_) {
      return this.config == _.config && this.open == _.open;
    }
    toDOM(_) {
      if (this.config.markerDOM)
        return this.config.markerDOM(this.open);
      let H = document.createElement("span");
      return H.textContent = this.open ? this.config.openText : this.config.closedText, H.title = _.state.phrase(this.open ? "Fold line" : "Unfold line"), H;
    }
  }
  function foldGutter(O = {}) {
    let _ = Object.assign(Object.assign({}, foldGutterDefaults), O), H = new FoldMarker(_, !0), V = new FoldMarker(_, !1), q = ViewPlugin.fromClass(class {
      constructor(J) {
        this.from = J.viewport.from, this.markers = this.buildMarkers(J);
      }
      update(J) {
        (J.docChanged || J.viewportChanged || J.startState.facet(language) != J.state.facet(language) || J.startState.field(foldState, !1) != J.state.field(foldState, !1) || syntaxTree(J.startState) != syntaxTree(J.state) || _.foldingChanged(J)) && (this.markers = this.buildMarkers(J.view));
      }
      buildMarkers(J) {
        let Z = new RangeSetBuilder();
        for (let te of J.viewportLineBlocks) {
          let ne = findFold(J.state, te.from, te.to) ? V : foldable(J.state, te.from, te.to) ? H : null;
          ne && Z.add(te.from, te.from, ne);
        }
        return Z.finish();
      }
    }), { domEventHandlers: G } = _;
    return [
      q,
      gutter({
        class: "cm-foldGutter",
        markers(J) {
          var Z;
          return ((Z = J.plugin(q)) === null || Z === void 0 ? void 0 : Z.markers) || RangeSet.empty;
        },
        initialSpacer() {
          return new FoldMarker(_, !1);
        },
        domEventHandlers: Object.assign(Object.assign({}, G), { click: (J, Z, te) => {
          if (G.click && G.click(J, Z, te))
            return !0;
          let ne = findFold(J.state, Z.from, Z.to);
          if (ne)
            return J.dispatch({ effects: unfoldEffect.of(ne) }), !0;
          let ee = foldable(J.state, Z.from, Z.to);
          return ee ? (J.dispatch({ effects: foldEffect.of(ee) }), !0) : !1;
        } })
      }),
      codeFolding()
    ];
  }
  const baseTheme$1$1 = /* @__PURE__ */ EditorView.baseTheme({
    ".cm-foldPlaceholder": {
      backgroundColor: "#eee",
      border: "1px solid #ddd",
      color: "#888",
      borderRadius: ".2em",
      margin: "0 1px",
      padding: "0 1px",
      cursor: "pointer"
    },
    ".cm-foldGutter span": {
      padding: "0 1px",
      cursor: "pointer"
    }
  });
  class HighlightStyle {
    constructor(_, H) {
      this.specs = _;
      let V;
      function q(Z) {
        let te = StyleModule.newName();
        return (V || (V = /* @__PURE__ */ Object.create(null)))["." + te] = Z, te;
      }
      const G = typeof H.all == "string" ? H.all : H.all ? q(H.all) : void 0, J = H.scope;
      this.scope = J instanceof Language ? (Z) => Z.prop(languageDataProp) == J.data : J ? (Z) => Z == J : void 0, this.style = tagHighlighter(_.map((Z) => ({
        tag: Z.tag,
        class: Z.class || q(Object.assign({}, Z, { tag: null }))
      })), {
        all: G
      }).style, this.module = V ? new StyleModule(V) : null, this.themeType = H.themeType;
    }
    /**
    Create a highlighter style that associates the given styles to
    the given tags. The specs must be objects that hold a style tag
    or array of tags in their `tag` property, and either a single
    `class` property providing a static CSS class (for highlighter
    that rely on external styling), or a
    [`style-mod`](https://github.com/marijnh/style-mod#documentation)-style
    set of CSS properties (which define the styling for those tags).
    
    The CSS rules created for a highlighter will be emitted in the
    order of the spec's properties. That means that for elements that
    have multiple tags associated with them, styles defined further
    down in the list will have a higher CSS precedence than styles
    defined earlier.
    */
    static define(_, H) {
      return new HighlightStyle(_, H || {});
    }
  }
  const highlighterFacet = /* @__PURE__ */ Facet.define(), fallbackHighlighter = /* @__PURE__ */ Facet.define({
    combine(O) {
      return O.length ? [O[0]] : null;
    }
  });
  function getHighlighters(O) {
    let _ = O.facet(highlighterFacet);
    return _.length ? _ : O.facet(fallbackHighlighter);
  }
  function syntaxHighlighting(O, _) {
    let H = [treeHighlighter], V;
    return O instanceof HighlightStyle && (O.module && H.push(EditorView.styleModule.of(O.module)), V = O.themeType), _ != null && _.fallback ? H.push(fallbackHighlighter.of(O)) : V ? H.push(highlighterFacet.computeN([EditorView.darkTheme], (q) => q.facet(EditorView.darkTheme) == (V == "dark") ? [O] : [])) : H.push(highlighterFacet.of(O)), H;
  }
  class TreeHighlighter {
    constructor(_) {
      this.markCache = /* @__PURE__ */ Object.create(null), this.tree = syntaxTree(_.state), this.decorations = this.buildDeco(_, getHighlighters(_.state)), this.decoratedTo = _.viewport.to;
    }
    update(_) {
      let H = syntaxTree(_.state), V = getHighlighters(_.state), q = V != getHighlighters(_.startState), { viewport: G } = _.view, J = _.changes.mapPos(this.decoratedTo, 1);
      H.length < G.to && !q && H.type == this.tree.type && J >= G.to ? (this.decorations = this.decorations.map(_.changes), this.decoratedTo = J) : (H != this.tree || _.viewportChanged || q) && (this.tree = H, this.decorations = this.buildDeco(_.view, V), this.decoratedTo = G.to);
    }
    buildDeco(_, H) {
      if (!H || !this.tree.length)
        return Decoration.none;
      let V = new RangeSetBuilder();
      for (let { from: q, to: G } of _.visibleRanges)
        highlightTree(this.tree, H, (J, Z, te) => {
          V.add(J, Z, this.markCache[te] || (this.markCache[te] = Decoration.mark({ class: te })));
        }, q, G);
      return V.finish();
    }
  }
  const treeHighlighter = /* @__PURE__ */ Prec.high(/* @__PURE__ */ ViewPlugin.fromClass(TreeHighlighter, {
    decorations: (O) => O.decorations
  })), defaultHighlightStyle = /* @__PURE__ */ HighlightStyle.define([
    {
      tag: tags.meta,
      color: "#404740"
    },
    {
      tag: tags.link,
      textDecoration: "underline"
    },
    {
      tag: tags.heading,
      textDecoration: "underline",
      fontWeight: "bold"
    },
    {
      tag: tags.emphasis,
      fontStyle: "italic"
    },
    {
      tag: tags.strong,
      fontWeight: "bold"
    },
    {
      tag: tags.strikethrough,
      textDecoration: "line-through"
    },
    {
      tag: tags.keyword,
      color: "#708"
    },
    {
      tag: [tags.atom, tags.bool, tags.url, tags.contentSeparator, tags.labelName],
      color: "#219"
    },
    {
      tag: [tags.literal, tags.inserted],
      color: "#164"
    },
    {
      tag: [tags.string, tags.deleted],
      color: "#a11"
    },
    {
      tag: [tags.regexp, tags.escape, /* @__PURE__ */ tags.special(tags.string)],
      color: "#e40"
    },
    {
      tag: /* @__PURE__ */ tags.definition(tags.variableName),
      color: "#00f"
    },
    {
      tag: /* @__PURE__ */ tags.local(tags.variableName),
      color: "#30a"
    },
    {
      tag: [tags.typeName, tags.namespace],
      color: "#085"
    },
    {
      tag: tags.className,
      color: "#167"
    },
    {
      tag: [/* @__PURE__ */ tags.special(tags.variableName), tags.macroName],
      color: "#256"
    },
    {
      tag: /* @__PURE__ */ tags.definition(tags.propertyName),
      color: "#00c"
    },
    {
      tag: tags.comment,
      color: "#940"
    },
    {
      tag: tags.invalid,
      color: "#f00"
    }
  ]), baseTheme$4 = /* @__PURE__ */ EditorView.baseTheme({
    "&.cm-focused .cm-matchingBracket": { backgroundColor: "#328c8252" },
    "&.cm-focused .cm-nonmatchingBracket": { backgroundColor: "#bb555544" }
  }), DefaultScanDist = 1e4, DefaultBrackets = "()[]{}", bracketMatchingConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, {
        afterCursor: !0,
        brackets: DefaultBrackets,
        maxScanDistance: DefaultScanDist,
        renderMatch: defaultRenderMatch
      });
    }
  }), matchingMark = /* @__PURE__ */ Decoration.mark({ class: "cm-matchingBracket" }), nonmatchingMark = /* @__PURE__ */ Decoration.mark({ class: "cm-nonmatchingBracket" });
  function defaultRenderMatch(O) {
    let _ = [], H = O.matched ? matchingMark : nonmatchingMark;
    return _.push(H.range(O.start.from, O.start.to)), O.end && _.push(H.range(O.end.from, O.end.to)), _;
  }
  const bracketMatchingState = /* @__PURE__ */ StateField.define({
    create() {
      return Decoration.none;
    },
    update(O, _) {
      if (!_.docChanged && !_.selection)
        return O;
      let H = [], V = _.state.facet(bracketMatchingConfig);
      for (let q of _.state.selection.ranges) {
        if (!q.empty)
          continue;
        let G = matchBrackets(_.state, q.head, -1, V) || q.head > 0 && matchBrackets(_.state, q.head - 1, 1, V) || V.afterCursor && (matchBrackets(_.state, q.head, 1, V) || q.head < _.state.doc.length && matchBrackets(_.state, q.head + 1, -1, V));
        G && (H = H.concat(V.renderMatch(G, _.state)));
      }
      return Decoration.set(H, !0);
    },
    provide: (O) => EditorView.decorations.from(O)
  }), bracketMatchingUnique = [
    bracketMatchingState,
    baseTheme$4
  ];
  function bracketMatching(O = {}) {
    return [bracketMatchingConfig.of(O), bracketMatchingUnique];
  }
  const bracketMatchingHandle = /* @__PURE__ */ new NodeProp();
  function matchingNodes(O, _, H) {
    let V = O.prop(_ < 0 ? NodeProp.openedBy : NodeProp.closedBy);
    if (V)
      return V;
    if (O.name.length == 1) {
      let q = H.indexOf(O.name);
      if (q > -1 && q % 2 == (_ < 0 ? 1 : 0))
        return [H[q + _]];
    }
    return null;
  }
  function findHandle(O) {
    let _ = O.type.prop(bracketMatchingHandle);
    return _ ? _(O.node) : O;
  }
  function matchBrackets(O, _, H, V = {}) {
    let q = V.maxScanDistance || DefaultScanDist, G = V.brackets || DefaultBrackets, J = syntaxTree(O), Z = J.resolveInner(_, H);
    for (let te = Z; te; te = te.parent) {
      let ne = matchingNodes(te.type, H, G);
      if (ne && te.from < te.to) {
        let ee = findHandle(te);
        if (ee && (H > 0 ? _ >= ee.from && _ < ee.to : _ > ee.from && _ <= ee.to))
          return matchMarkedBrackets(O, _, H, te, ee, ne, G);
      }
    }
    return matchPlainBrackets(O, _, H, J, Z.type, q, G);
  }
  function matchMarkedBrackets(O, _, H, V, q, G, J) {
    let Z = V.parent, te = { from: q.from, to: q.to }, ne = 0, ee = Z == null ? void 0 : Z.cursor();
    if (ee && (H < 0 ? ee.childBefore(V.from) : ee.childAfter(V.to)))
      do
        if (H < 0 ? ee.to <= V.from : ee.from >= V.to) {
          if (ne == 0 && G.indexOf(ee.type.name) > -1 && ee.from < ee.to) {
            let ae = findHandle(ee);
            return { start: te, end: ae ? { from: ae.from, to: ae.to } : void 0, matched: !0 };
          } else if (matchingNodes(ee.type, H, J))
            ne++;
          else if (matchingNodes(ee.type, -H, J)) {
            if (ne == 0) {
              let ae = findHandle(ee);
              return {
                start: te,
                end: ae && ae.from < ae.to ? { from: ae.from, to: ae.to } : void 0,
                matched: !1
              };
            }
            ne--;
          }
        }
      while (H < 0 ? ee.prevSibling() : ee.nextSibling());
    return { start: te, matched: !1 };
  }
  function matchPlainBrackets(O, _, H, V, q, G, J) {
    let Z = H < 0 ? O.sliceDoc(_ - 1, _) : O.sliceDoc(_, _ + 1), te = J.indexOf(Z);
    if (te < 0 || te % 2 == 0 != H > 0)
      return null;
    let ne = { from: H < 0 ? _ - 1 : _, to: H > 0 ? _ + 1 : _ }, ee = O.doc.iterRange(_, H > 0 ? O.doc.length : 0), ae = 0;
    for (let oe = 0; !ee.next().done && oe <= G; ) {
      let se = ee.value;
      H < 0 && (oe += se.length);
      let ce = _ + oe * H;
      for (let le = H > 0 ? 0 : se.length - 1, ue = H > 0 ? se.length : -1; le != ue; le += H) {
        let de = J.indexOf(se[le]);
        if (!(de < 0 || V.resolveInner(ce + le, 1).type != q))
          if (de % 2 == 0 == H > 0)
            ae++;
          else {
            if (ae == 1)
              return { start: ne, end: { from: ce + le, to: ce + le + 1 }, matched: de >> 1 == te >> 1 };
            ae--;
          }
      }
      H > 0 && (oe += se.length);
    }
    return ee.done ? { start: ne, matched: !1 } : null;
  }
  const noTokens = /* @__PURE__ */ Object.create(null), typeArray = [NodeType.none], warned = [], byTag = /* @__PURE__ */ Object.create(null), defaultTable = /* @__PURE__ */ Object.create(null);
  for (let [O, _] of [
    ["variable", "variableName"],
    ["variable-2", "variableName.special"],
    ["string-2", "string.special"],
    ["def", "variableName.definition"],
    ["tag", "tagName"],
    ["attribute", "attributeName"],
    ["type", "typeName"],
    ["builtin", "variableName.standard"],
    ["qualifier", "modifier"],
    ["error", "invalid"],
    ["header", "heading"],
    ["property", "propertyName"]
  ])
    defaultTable[O] = /* @__PURE__ */ createTokenType(noTokens, _);
  function warnForPart(O, _) {
    warned.indexOf(O) > -1 || (warned.push(O), console.warn(_));
  }
  function createTokenType(O, _) {
    let H = [];
    for (let Z of _.split(" ")) {
      let te = [];
      for (let ne of Z.split(".")) {
        let ee = O[ne] || tags[ne];
        ee ? typeof ee == "function" ? te.length ? te = te.map(ee) : warnForPart(ne, `Modifier ${ne} used at start of tag`) : te.length ? warnForPart(ne, `Tag ${ne} used as modifier`) : te = Array.isArray(ee) ? ee : [ee] : warnForPart(ne, `Unknown highlighting tag ${ne}`);
      }
      for (let ne of te)
        H.push(ne);
    }
    if (!H.length)
      return 0;
    let V = _.replace(/ /g, "_"), q = V + " " + H.map((Z) => Z.id), G = byTag[q];
    if (G)
      return G.id;
    let J = byTag[q] = NodeType.define({
      id: typeArray.length,
      name: V,
      props: [styleTags({ [V]: H })]
    });
    return typeArray.push(J), J.id;
  }
  Direction.RTL, Direction.LTR;
  class Stack {
    /**
    @internal
    */
    constructor(_, H, V, q, G, J, Z, te, ne, ee = 0, ae) {
      this.p = _, this.stack = H, this.state = V, this.reducePos = q, this.pos = G, this.score = J, this.buffer = Z, this.bufferBase = te, this.curContext = ne, this.lookAhead = ee, this.parent = ae;
    }
    /**
    @internal
    */
    toString() {
      return `[${this.stack.filter((_, H) => H % 3 == 0).concat(this.state)}]@${this.pos}${this.score ? "!" + this.score : ""}`;
    }
    // Start an empty stack
    /**
    @internal
    */
    static start(_, H, V = 0) {
      let q = _.parser.context;
      return new Stack(_, [], H, V, V, 0, [], 0, q ? new StackContext(q, q.start) : null, 0, null);
    }
    /**
    The stack's current [context](#lr.ContextTracker) value, if
    any. Its type will depend on the context tracker's type
    parameter, or it will be `null` if there is no context
    tracker.
    */
    get context() {
      return this.curContext ? this.curContext.context : null;
    }
    // Push a state onto the stack, tracking its start position as well
    // as the buffer base at that point.
    /**
    @internal
    */
    pushState(_, H) {
      this.stack.push(this.state, H, this.bufferBase + this.buffer.length), this.state = _;
    }
    // Apply a reduce action
    /**
    @internal
    */
    reduce(_) {
      var H;
      let V = _ >> 19, q = _ & 65535, { parser: G } = this.p, J = this.reducePos < this.pos - 25;
      J && this.setLookAhead(this.pos);
      let Z = G.dynamicPrecedence(q);
      if (Z && (this.score += Z), V == 0) {
        this.pushState(G.getGoto(this.state, q, !0), this.reducePos), q < G.minRepeatTerm && this.storeNode(q, this.reducePos, this.reducePos, J ? 8 : 4, !0), this.reduceContext(q, this.reducePos);
        return;
      }
      let te = this.stack.length - (V - 1) * 3 - (_ & 262144 ? 6 : 0), ne = te ? this.stack[te - 2] : this.p.ranges[0].from, ee = this.reducePos - ne;
      ee >= 2e3 && !(!((H = this.p.parser.nodeSet.types[q]) === null || H === void 0) && H.isAnonymous) && (ne == this.p.lastBigReductionStart ? (this.p.bigReductionCount++, this.p.lastBigReductionSize = ee) : this.p.lastBigReductionSize < ee && (this.p.bigReductionCount = 1, this.p.lastBigReductionStart = ne, this.p.lastBigReductionSize = ee));
      let ae = te ? this.stack[te - 1] : 0, oe = this.bufferBase + this.buffer.length - ae;
      if (q < G.minRepeatTerm || _ & 131072) {
        let se = G.stateFlag(
          this.state,
          1
          /* StateFlag.Skipped */
        ) ? this.pos : this.reducePos;
        this.storeNode(q, ne, se, oe + 4, !0);
      }
      if (_ & 262144)
        this.state = this.stack[te];
      else {
        let se = this.stack[te - 3];
        this.state = G.getGoto(se, q, !0);
      }
      for (; this.stack.length > te; )
        this.stack.pop();
      this.reduceContext(q, ne);
    }
    // Shift a value into the buffer
    /**
    @internal
    */
    storeNode(_, H, V, q = 4, G = !1) {
      if (_ == 0 && (!this.stack.length || this.stack[this.stack.length - 1] < this.buffer.length + this.bufferBase)) {
        let J = this, Z = this.buffer.length;
        if (Z == 0 && J.parent && (Z = J.bufferBase - J.parent.bufferBase, J = J.parent), Z > 0 && J.buffer[Z - 4] == 0 && J.buffer[Z - 1] > -1) {
          if (H == V)
            return;
          if (J.buffer[Z - 2] >= H) {
            J.buffer[Z - 2] = V;
            return;
          }
        }
      }
      if (!G || this.pos == V)
        this.buffer.push(_, H, V, q);
      else {
        let J = this.buffer.length;
        if (J > 0 && this.buffer[J - 4] != 0) {
          let Z = !1;
          for (let te = J; te > 0 && this.buffer[te - 2] > V; te -= 4)
            if (this.buffer[te - 1] >= 0) {
              Z = !0;
              break;
            }
          if (Z)
            for (; J > 0 && this.buffer[J - 2] > V; )
              this.buffer[J] = this.buffer[J - 4], this.buffer[J + 1] = this.buffer[J - 3], this.buffer[J + 2] = this.buffer[J - 2], this.buffer[J + 3] = this.buffer[J - 1], J -= 4, q > 4 && (q -= 4);
        }
        this.buffer[J] = _, this.buffer[J + 1] = H, this.buffer[J + 2] = V, this.buffer[J + 3] = q;
      }
    }
    // Apply a shift action
    /**
    @internal
    */
    shift(_, H, V, q) {
      if (_ & 131072)
        this.pushState(_ & 65535, this.pos);
      else if (_ & 262144)
        this.pos = q, this.shiftContext(H, V), H <= this.p.parser.maxNode && this.buffer.push(H, V, q, 4);
      else {
        let G = _, { parser: J } = this.p;
        (q > this.pos || H <= J.maxNode) && (this.pos = q, J.stateFlag(
          G,
          1
          /* StateFlag.Skipped */
        ) || (this.reducePos = q)), this.pushState(G, V), this.shiftContext(H, V), H <= J.maxNode && this.buffer.push(H, V, q, 4);
      }
    }
    // Apply an action
    /**
    @internal
    */
    apply(_, H, V, q) {
      _ & 65536 ? this.reduce(_) : this.shift(_, H, V, q);
    }
    // Add a prebuilt (reused) node into the buffer.
    /**
    @internal
    */
    useNode(_, H) {
      let V = this.p.reused.length - 1;
      (V < 0 || this.p.reused[V] != _) && (this.p.reused.push(_), V++);
      let q = this.pos;
      this.reducePos = this.pos = q + _.length, this.pushState(H, q), this.buffer.push(
        V,
        q,
        this.reducePos,
        -1
        /* size == -1 means this is a reused value */
      ), this.curContext && this.updateContext(this.curContext.tracker.reuse(this.curContext.context, _, this, this.p.stream.reset(this.pos - _.length)));
    }
    // Split the stack. Due to the buffer sharing and the fact
    // that `this.stack` tends to stay quite shallow, this isn't very
    // expensive.
    /**
    @internal
    */
    split() {
      let _ = this, H = _.buffer.length;
      for (; H > 0 && _.buffer[H - 2] > _.reducePos; )
        H -= 4;
      let V = _.buffer.slice(H), q = _.bufferBase + H;
      for (; _ && q == _.bufferBase; )
        _ = _.parent;
      return new Stack(this.p, this.stack.slice(), this.state, this.reducePos, this.pos, this.score, V, q, this.curContext, this.lookAhead, _);
    }
    // Try to recover from an error by 'deleting' (ignoring) one token.
    /**
    @internal
    */
    recoverByDelete(_, H) {
      let V = _ <= this.p.parser.maxNode;
      V && this.storeNode(_, this.pos, H, 4), this.storeNode(0, this.pos, H, V ? 8 : 4), this.pos = this.reducePos = H, this.score -= 190;
    }
    /**
    Check if the given term would be able to be shifted (optionally
    after some reductions) on this stack. This can be useful for
    external tokenizers that want to make sure they only provide a
    given token when it applies.
    */
    canShift(_) {
      for (let H = new SimulatedStack(this); ; ) {
        let V = this.p.parser.stateSlot(
          H.state,
          4
          /* ParseState.DefaultReduce */
        ) || this.p.parser.hasAction(H.state, _);
        if (V == 0)
          return !1;
        if (!(V & 65536))
          return !0;
        H.reduce(V);
      }
    }
    // Apply up to Recover.MaxNext recovery actions that conceptually
    // inserts some missing token or rule.
    /**
    @internal
    */
    recoverByInsert(_) {
      if (this.stack.length >= 300)
        return [];
      let H = this.p.parser.nextStates(this.state);
      if (H.length > 8 || this.stack.length >= 120) {
        let q = [];
        for (let G = 0, J; G < H.length; G += 2)
          (J = H[G + 1]) != this.state && this.p.parser.hasAction(J, _) && q.push(H[G], J);
        if (this.stack.length < 120)
          for (let G = 0; q.length < 8 && G < H.length; G += 2) {
            let J = H[G + 1];
            q.some((Z, te) => te & 1 && Z == J) || q.push(H[G], J);
          }
        H = q;
      }
      let V = [];
      for (let q = 0; q < H.length && V.length < 4; q += 2) {
        let G = H[q + 1];
        if (G == this.state)
          continue;
        let J = this.split();
        J.pushState(G, this.pos), J.storeNode(0, J.pos, J.pos, 4, !0), J.shiftContext(H[q], this.pos), J.reducePos = this.pos, J.score -= 200, V.push(J);
      }
      return V;
    }
    // Force a reduce, if possible. Return false if that can't
    // be done.
    /**
    @internal
    */
    forceReduce() {
      let { parser: _ } = this.p, H = _.stateSlot(
        this.state,
        5
        /* ParseState.ForcedReduce */
      );
      if (!(H & 65536))
        return !1;
      if (!_.validAction(this.state, H)) {
        let V = H >> 19, q = H & 65535, G = this.stack.length - V * 3;
        if (G < 0 || _.getGoto(this.stack[G], q, !1) < 0) {
          let J = this.findForcedReduction();
          if (J == null)
            return !1;
          H = J;
        }
        this.storeNode(0, this.pos, this.pos, 4, !0), this.score -= 100;
      }
      return this.reducePos = this.pos, this.reduce(H), !0;
    }
    /**
    Try to scan through the automaton to find some kind of reduction
    that can be applied. Used when the regular ForcedReduce field
    isn't a valid action. @internal
    */
    findForcedReduction() {
      let { parser: _ } = this.p, H = [], V = (q, G) => {
        if (!H.includes(q))
          return H.push(q), _.allActions(q, (J) => {
            if (!(J & 393216)) if (J & 65536) {
              let Z = (J >> 19) - G;
              if (Z > 1) {
                let te = J & 65535, ne = this.stack.length - Z * 3;
                if (ne >= 0 && _.getGoto(this.stack[ne], te, !1) >= 0)
                  return Z << 19 | 65536 | te;
              }
            } else {
              let Z = V(J, G + 1);
              if (Z != null)
                return Z;
            }
          });
      };
      return V(this.state, 0);
    }
    /**
    @internal
    */
    forceAll() {
      for (; !this.p.parser.stateFlag(
        this.state,
        2
        /* StateFlag.Accepting */
      ); )
        if (!this.forceReduce()) {
          this.storeNode(0, this.pos, this.pos, 4, !0);
          break;
        }
      return this;
    }
    /**
    Check whether this state has no further actions (assumed to be a direct descendant of the
    top state, since any other states must be able to continue
    somehow). @internal
    */
    get deadEnd() {
      if (this.stack.length != 3)
        return !1;
      let { parser: _ } = this.p;
      return _.data[_.stateSlot(
        this.state,
        1
        /* ParseState.Actions */
      )] == 65535 && !_.stateSlot(
        this.state,
        4
        /* ParseState.DefaultReduce */
      );
    }
    /**
    Restart the stack (put it back in its start state). Only safe
    when this.stack.length == 3 (state is directly below the top
    state). @internal
    */
    restart() {
      this.storeNode(0, this.pos, this.pos, 4, !0), this.state = this.stack[0], this.stack.length = 0;
    }
    /**
    @internal
    */
    sameState(_) {
      if (this.state != _.state || this.stack.length != _.stack.length)
        return !1;
      for (let H = 0; H < this.stack.length; H += 3)
        if (this.stack[H] != _.stack[H])
          return !1;
      return !0;
    }
    /**
    Get the parser used by this stack.
    */
    get parser() {
      return this.p.parser;
    }
    /**
    Test whether a given dialect (by numeric ID, as exported from
    the terms file) is enabled.
    */
    dialectEnabled(_) {
      return this.p.parser.dialect.flags[_];
    }
    shiftContext(_, H) {
      this.curContext && this.updateContext(this.curContext.tracker.shift(this.curContext.context, _, this, this.p.stream.reset(H)));
    }
    reduceContext(_, H) {
      this.curContext && this.updateContext(this.curContext.tracker.reduce(this.curContext.context, _, this, this.p.stream.reset(H)));
    }
    /**
    @internal
    */
    emitContext() {
      let _ = this.buffer.length - 1;
      (_ < 0 || this.buffer[_] != -3) && this.buffer.push(this.curContext.hash, this.pos, this.pos, -3);
    }
    /**
    @internal
    */
    emitLookAhead() {
      let _ = this.buffer.length - 1;
      (_ < 0 || this.buffer[_] != -4) && this.buffer.push(this.lookAhead, this.pos, this.pos, -4);
    }
    updateContext(_) {
      if (_ != this.curContext.context) {
        let H = new StackContext(this.curContext.tracker, _);
        H.hash != this.curContext.hash && this.emitContext(), this.curContext = H;
      }
    }
    /**
    @internal
    */
    setLookAhead(_) {
      _ > this.lookAhead && (this.emitLookAhead(), this.lookAhead = _);
    }
    /**
    @internal
    */
    close() {
      this.curContext && this.curContext.tracker.strict && this.emitContext(), this.lookAhead > 0 && this.emitLookAhead();
    }
  }
  class StackContext {
    constructor(_, H) {
      this.tracker = _, this.context = H, this.hash = _.strict ? _.hash(H) : 0;
    }
  }
  class SimulatedStack {
    constructor(_) {
      this.start = _, this.state = _.state, this.stack = _.stack, this.base = this.stack.length;
    }
    reduce(_) {
      let H = _ & 65535, V = _ >> 19;
      V == 0 ? (this.stack == this.start.stack && (this.stack = this.stack.slice()), this.stack.push(this.state, 0, 0), this.base += 3) : this.base -= (V - 1) * 3;
      let q = this.start.p.parser.getGoto(this.stack[this.base - 3], H, !0);
      this.state = q;
    }
  }
  class StackBufferCursor {
    constructor(_, H, V) {
      this.stack = _, this.pos = H, this.index = V, this.buffer = _.buffer, this.index == 0 && this.maybeNext();
    }
    static create(_, H = _.bufferBase + _.buffer.length) {
      return new StackBufferCursor(_, H, H - _.bufferBase);
    }
    maybeNext() {
      let _ = this.stack.parent;
      _ != null && (this.index = this.stack.bufferBase - _.bufferBase, this.stack = _, this.buffer = _.buffer);
    }
    get id() {
      return this.buffer[this.index - 4];
    }
    get start() {
      return this.buffer[this.index - 3];
    }
    get end() {
      return this.buffer[this.index - 2];
    }
    get size() {
      return this.buffer[this.index - 1];
    }
    next() {
      this.index -= 4, this.pos -= 4, this.index == 0 && this.maybeNext();
    }
    fork() {
      return new StackBufferCursor(this.stack, this.pos, this.index);
    }
  }
  function decodeArray(O, _ = Uint16Array) {
    if (typeof O != "string")
      return O;
    let H = null;
    for (let V = 0, q = 0; V < O.length; ) {
      let G = 0;
      for (; ; ) {
        let J = O.charCodeAt(V++), Z = !1;
        if (J == 126) {
          G = 65535;
          break;
        }
        J >= 92 && J--, J >= 34 && J--;
        let te = J - 32;
        if (te >= 46 && (te -= 46, Z = !0), G += te, Z)
          break;
        G *= 46;
      }
      H ? H[q++] = G : H = new _(G);
    }
    return H;
  }
  class CachedToken {
    constructor() {
      this.start = -1, this.value = -1, this.end = -1, this.extended = -1, this.lookAhead = 0, this.mask = 0, this.context = 0;
    }
  }
  const nullToken = new CachedToken();
  class InputStream {
    /**
    @internal
    */
    constructor(_, H) {
      this.input = _, this.ranges = H, this.chunk = "", this.chunkOff = 0, this.chunk2 = "", this.chunk2Pos = 0, this.next = -1, this.token = nullToken, this.rangeIndex = 0, this.pos = this.chunkPos = H[0].from, this.range = H[0], this.end = H[H.length - 1].to, this.readNext();
    }
    /**
    @internal
    */
    resolveOffset(_, H) {
      let V = this.range, q = this.rangeIndex, G = this.pos + _;
      for (; G < V.from; ) {
        if (!q)
          return null;
        let J = this.ranges[--q];
        G -= V.from - J.to, V = J;
      }
      for (; H < 0 ? G > V.to : G >= V.to; ) {
        if (q == this.ranges.length - 1)
          return null;
        let J = this.ranges[++q];
        G += J.from - V.to, V = J;
      }
      return G;
    }
    /**
    @internal
    */
    clipPos(_) {
      if (_ >= this.range.from && _ < this.range.to)
        return _;
      for (let H of this.ranges)
        if (H.to > _)
          return Math.max(_, H.from);
      return this.end;
    }
    /**
    Look at a code unit near the stream position. `.peek(0)` equals
    `.next`, `.peek(-1)` gives you the previous character, and so
    on.
    
    Note that looking around during tokenizing creates dependencies
    on potentially far-away content, which may reduce the
    effectiveness incremental parsing—when looking forward—or even
    cause invalid reparses when looking backward more than 25 code
    units, since the library does not track lookbehind.
    */
    peek(_) {
      let H = this.chunkOff + _, V, q;
      if (H >= 0 && H < this.chunk.length)
        V = this.pos + _, q = this.chunk.charCodeAt(H);
      else {
        let G = this.resolveOffset(_, 1);
        if (G == null)
          return -1;
        if (V = G, V >= this.chunk2Pos && V < this.chunk2Pos + this.chunk2.length)
          q = this.chunk2.charCodeAt(V - this.chunk2Pos);
        else {
          let J = this.rangeIndex, Z = this.range;
          for (; Z.to <= V; )
            Z = this.ranges[++J];
          this.chunk2 = this.input.chunk(this.chunk2Pos = V), V + this.chunk2.length > Z.to && (this.chunk2 = this.chunk2.slice(0, Z.to - V)), q = this.chunk2.charCodeAt(0);
        }
      }
      return V >= this.token.lookAhead && (this.token.lookAhead = V + 1), q;
    }
    /**
    Accept a token. By default, the end of the token is set to the
    current stream position, but you can pass an offset (relative to
    the stream position) to change that.
    */
    acceptToken(_, H = 0) {
      let V = H ? this.resolveOffset(H, -1) : this.pos;
      if (V == null || V < this.token.start)
        throw new RangeError("Token end out of bounds");
      this.token.value = _, this.token.end = V;
    }
    /**
    Accept a token ending at a specific given position.
    */
    acceptTokenTo(_, H) {
      this.token.value = _, this.token.end = H;
    }
    getChunk() {
      if (this.pos >= this.chunk2Pos && this.pos < this.chunk2Pos + this.chunk2.length) {
        let { chunk: _, chunkPos: H } = this;
        this.chunk = this.chunk2, this.chunkPos = this.chunk2Pos, this.chunk2 = _, this.chunk2Pos = H, this.chunkOff = this.pos - this.chunkPos;
      } else {
        this.chunk2 = this.chunk, this.chunk2Pos = this.chunkPos;
        let _ = this.input.chunk(this.pos), H = this.pos + _.length;
        this.chunk = H > this.range.to ? _.slice(0, this.range.to - this.pos) : _, this.chunkPos = this.pos, this.chunkOff = 0;
      }
    }
    readNext() {
      return this.chunkOff >= this.chunk.length && (this.getChunk(), this.chunkOff == this.chunk.length) ? this.next = -1 : this.next = this.chunk.charCodeAt(this.chunkOff);
    }
    /**
    Move the stream forward N (defaults to 1) code units. Returns
    the new value of [`next`](#lr.InputStream.next).
    */
    advance(_ = 1) {
      for (this.chunkOff += _; this.pos + _ >= this.range.to; ) {
        if (this.rangeIndex == this.ranges.length - 1)
          return this.setDone();
        _ -= this.range.to - this.pos, this.range = this.ranges[++this.rangeIndex], this.pos = this.range.from;
      }
      return this.pos += _, this.pos >= this.token.lookAhead && (this.token.lookAhead = this.pos + 1), this.readNext();
    }
    setDone() {
      return this.pos = this.chunkPos = this.end, this.range = this.ranges[this.rangeIndex = this.ranges.length - 1], this.chunk = "", this.next = -1;
    }
    /**
    @internal
    */
    reset(_, H) {
      if (H ? (this.token = H, H.start = _, H.lookAhead = _ + 1, H.value = H.extended = -1) : this.token = nullToken, this.pos != _) {
        if (this.pos = _, _ == this.end)
          return this.setDone(), this;
        for (; _ < this.range.from; )
          this.range = this.ranges[--this.rangeIndex];
        for (; _ >= this.range.to; )
          this.range = this.ranges[++this.rangeIndex];
        _ >= this.chunkPos && _ < this.chunkPos + this.chunk.length ? this.chunkOff = _ - this.chunkPos : (this.chunk = "", this.chunkOff = 0), this.readNext();
      }
      return this;
    }
    /**
    @internal
    */
    read(_, H) {
      if (_ >= this.chunkPos && H <= this.chunkPos + this.chunk.length)
        return this.chunk.slice(_ - this.chunkPos, H - this.chunkPos);
      if (_ >= this.chunk2Pos && H <= this.chunk2Pos + this.chunk2.length)
        return this.chunk2.slice(_ - this.chunk2Pos, H - this.chunk2Pos);
      if (_ >= this.range.from && H <= this.range.to)
        return this.input.read(_, H);
      let V = "";
      for (let q of this.ranges) {
        if (q.from >= H)
          break;
        q.to > _ && (V += this.input.read(Math.max(q.from, _), Math.min(q.to, H)));
      }
      return V;
    }
  }
  class TokenGroup {
    constructor(_, H) {
      this.data = _, this.id = H;
    }
    token(_, H) {
      let { parser: V } = H.p;
      readToken(this.data, _, H, this.id, V.data, V.tokenPrecTable);
    }
  }
  TokenGroup.prototype.contextual = TokenGroup.prototype.fallback = TokenGroup.prototype.extend = !1;
  TokenGroup.prototype.fallback = TokenGroup.prototype.extend = !1;
  class ExternalTokenizer {
    /**
    Create a tokenizer. The first argument is the function that,
    given an input stream, scans for the types of tokens it
    recognizes at the stream's position, and calls
    [`acceptToken`](#lr.InputStream.acceptToken) when it finds
    one.
    */
    constructor(_, H = {}) {
      this.token = _, this.contextual = !!H.contextual, this.fallback = !!H.fallback, this.extend = !!H.extend;
    }
  }
  function readToken(O, _, H, V, q, G) {
    let J = 0, Z = 1 << V, { dialect: te } = H.p.parser;
    e: for (; Z & O[J]; ) {
      let ne = O[J + 1];
      for (let se = J + 3; se < ne; se += 2)
        if ((O[se + 1] & Z) > 0) {
          let ce = O[se];
          if (te.allows(ce) && (_.token.value == -1 || _.token.value == ce || overrides(ce, _.token.value, q, G))) {
            _.acceptToken(ce);
            break;
          }
        }
      let ee = _.next, ae = 0, oe = O[J + 2];
      if (_.next < 0 && oe > ae && O[ne + oe * 3 - 3] == 65535) {
        J = O[ne + oe * 3 - 1];
        continue e;
      }
      for (; ae < oe; ) {
        let se = ae + oe >> 1, ce = ne + se + (se << 1), le = O[ce], ue = O[ce + 1] || 65536;
        if (ee < le)
          oe = se;
        else if (ee >= ue)
          ae = se + 1;
        else {
          J = O[ce + 2], _.advance();
          continue e;
        }
      }
      break;
    }
  }
  function findOffset(O, _, H) {
    for (let V = _, q; (q = O[V]) != 65535; V++)
      if (q == H)
        return V - _;
    return -1;
  }
  function overrides(O, _, H, V) {
    let q = findOffset(H, V, _);
    return q < 0 || findOffset(H, V, O) < q;
  }
  const verbose = typeof process < "u" && process.env && /\bparse\b/.test(process.env.LOG);
  let stackIDs = null;
  function cutAt(O, _, H) {
    let V = O.cursor(IterMode.IncludeAnonymous);
    for (V.moveTo(_); ; )
      if (!(H < 0 ? V.childBefore(_) : V.childAfter(_)))
        for (; ; ) {
          if ((H < 0 ? V.to < _ : V.from > _) && !V.type.isError)
            return H < 0 ? Math.max(0, Math.min(
              V.to - 1,
              _ - 25
              /* Lookahead.Margin */
            )) : Math.min(O.length, Math.max(
              V.from + 1,
              _ + 25
              /* Lookahead.Margin */
            ));
          if (H < 0 ? V.prevSibling() : V.nextSibling())
            break;
          if (!V.parent())
            return H < 0 ? 0 : O.length;
        }
  }
  class FragmentCursor {
    constructor(_, H) {
      this.fragments = _, this.nodeSet = H, this.i = 0, this.fragment = null, this.safeFrom = -1, this.safeTo = -1, this.trees = [], this.start = [], this.index = [], this.nextFragment();
    }
    nextFragment() {
      let _ = this.fragment = this.i == this.fragments.length ? null : this.fragments[this.i++];
      if (_) {
        for (this.safeFrom = _.openStart ? cutAt(_.tree, _.from + _.offset, 1) - _.offset : _.from, this.safeTo = _.openEnd ? cutAt(_.tree, _.to + _.offset, -1) - _.offset : _.to; this.trees.length; )
          this.trees.pop(), this.start.pop(), this.index.pop();
        this.trees.push(_.tree), this.start.push(-_.offset), this.index.push(0), this.nextStart = this.safeFrom;
      } else
        this.nextStart = 1e9;
    }
    // `pos` must be >= any previously given `pos` for this cursor
    nodeAt(_) {
      if (_ < this.nextStart)
        return null;
      for (; this.fragment && this.safeTo <= _; )
        this.nextFragment();
      if (!this.fragment)
        return null;
      for (; ; ) {
        let H = this.trees.length - 1;
        if (H < 0)
          return this.nextFragment(), null;
        let V = this.trees[H], q = this.index[H];
        if (q == V.children.length) {
          this.trees.pop(), this.start.pop(), this.index.pop();
          continue;
        }
        let G = V.children[q], J = this.start[H] + V.positions[q];
        if (J > _)
          return this.nextStart = J, null;
        if (G instanceof Tree) {
          if (J == _) {
            if (J < this.safeFrom)
              return null;
            let Z = J + G.length;
            if (Z <= this.safeTo) {
              let te = G.prop(NodeProp.lookAhead);
              if (!te || Z + te < this.fragment.to)
                return G;
            }
          }
          this.index[H]++, J + G.length >= Math.max(this.safeFrom, _) && (this.trees.push(G), this.start.push(J), this.index.push(0));
        } else
          this.index[H]++, this.nextStart = J + G.length;
      }
    }
  }
  class TokenCache {
    constructor(_, H) {
      this.stream = H, this.tokens = [], this.mainToken = null, this.actions = [], this.tokens = _.tokenizers.map((V) => new CachedToken());
    }
    getActions(_) {
      let H = 0, V = null, { parser: q } = _.p, { tokenizers: G } = q, J = q.stateSlot(
        _.state,
        3
        /* ParseState.TokenizerMask */
      ), Z = _.curContext ? _.curContext.hash : 0, te = 0;
      for (let ne = 0; ne < G.length; ne++) {
        if (!(1 << ne & J))
          continue;
        let ee = G[ne], ae = this.tokens[ne];
        if (!(V && !ee.fallback) && ((ee.contextual || ae.start != _.pos || ae.mask != J || ae.context != Z) && (this.updateCachedToken(ae, ee, _), ae.mask = J, ae.context = Z), ae.lookAhead > ae.end + 25 && (te = Math.max(ae.lookAhead, te)), ae.value != 0)) {
          let oe = H;
          if (ae.extended > -1 && (H = this.addActions(_, ae.extended, ae.end, H)), H = this.addActions(_, ae.value, ae.end, H), !ee.extend && (V = ae, H > oe))
            break;
        }
      }
      for (; this.actions.length > H; )
        this.actions.pop();
      return te && _.setLookAhead(te), !V && _.pos == this.stream.end && (V = new CachedToken(), V.value = _.p.parser.eofTerm, V.start = V.end = _.pos, H = this.addActions(_, V.value, V.end, H)), this.mainToken = V, this.actions;
    }
    getMainToken(_) {
      if (this.mainToken)
        return this.mainToken;
      let H = new CachedToken(), { pos: V, p: q } = _;
      return H.start = V, H.end = Math.min(V + 1, q.stream.end), H.value = V == q.stream.end ? q.parser.eofTerm : 0, H;
    }
    updateCachedToken(_, H, V) {
      let q = this.stream.clipPos(V.pos);
      if (H.token(this.stream.reset(q, _), V), _.value > -1) {
        let { parser: G } = V.p;
        for (let J = 0; J < G.specialized.length; J++)
          if (G.specialized[J] == _.value) {
            let Z = G.specializers[J](this.stream.read(_.start, _.end), V);
            if (Z >= 0 && V.p.parser.dialect.allows(Z >> 1)) {
              Z & 1 ? _.extended = Z >> 1 : _.value = Z >> 1;
              break;
            }
          }
      } else
        _.value = 0, _.end = this.stream.clipPos(q + 1);
    }
    putAction(_, H, V, q) {
      for (let G = 0; G < q; G += 3)
        if (this.actions[G] == _)
          return q;
      return this.actions[q++] = _, this.actions[q++] = H, this.actions[q++] = V, q;
    }
    addActions(_, H, V, q) {
      let { state: G } = _, { parser: J } = _.p, { data: Z } = J;
      for (let te = 0; te < 2; te++)
        for (let ne = J.stateSlot(
          G,
          te ? 2 : 1
          /* ParseState.Actions */
        ); ; ne += 3) {
          if (Z[ne] == 65535)
            if (Z[ne + 1] == 1)
              ne = pair(Z, ne + 2);
            else {
              q == 0 && Z[ne + 1] == 2 && (q = this.putAction(pair(Z, ne + 2), H, V, q));
              break;
            }
          Z[ne] == H && (q = this.putAction(pair(Z, ne + 1), H, V, q));
        }
      return q;
    }
  }
  class Parse {
    constructor(_, H, V, q) {
      this.parser = _, this.input = H, this.ranges = q, this.recovering = 0, this.nextStackID = 9812, this.minStackPos = 0, this.reused = [], this.stoppedAt = null, this.lastBigReductionStart = -1, this.lastBigReductionSize = 0, this.bigReductionCount = 0, this.stream = new InputStream(H, q), this.tokens = new TokenCache(_, this.stream), this.topTerm = _.top[1];
      let { from: G } = q[0];
      this.stacks = [Stack.start(this, _.top[0], G)], this.fragments = V.length && this.stream.end - G > _.bufferLength * 4 ? new FragmentCursor(V, _.nodeSet) : null;
    }
    get parsedPos() {
      return this.minStackPos;
    }
    // Move the parser forward. This will process all parse stacks at
    // `this.pos` and try to advance them to a further position. If no
    // stack for such a position is found, it'll start error-recovery.
    //
    // When the parse is finished, this will return a syntax tree. When
    // not, it returns `null`.
    advance() {
      let _ = this.stacks, H = this.minStackPos, V = this.stacks = [], q, G;
      if (this.bigReductionCount > 300 && _.length == 1) {
        let [J] = _;
        for (; J.forceReduce() && J.stack.length && J.stack[J.stack.length - 2] >= this.lastBigReductionStart; )
          ;
        this.bigReductionCount = this.lastBigReductionSize = 0;
      }
      for (let J = 0; J < _.length; J++) {
        let Z = _[J];
        for (; ; ) {
          if (this.tokens.mainToken = null, Z.pos > H)
            V.push(Z);
          else {
            if (this.advanceStack(Z, V, _))
              continue;
            {
              q || (q = [], G = []), q.push(Z);
              let te = this.tokens.getMainToken(Z);
              G.push(te.value, te.end);
            }
          }
          break;
        }
      }
      if (!V.length) {
        let J = q && findFinished(q);
        if (J)
          return verbose && console.log("Finish with " + this.stackID(J)), this.stackToTree(J);
        if (this.parser.strict)
          throw verbose && q && console.log("Stuck with token " + (this.tokens.mainToken ? this.parser.getName(this.tokens.mainToken.value) : "none")), new SyntaxError("No parse at " + H);
        this.recovering || (this.recovering = 5);
      }
      if (this.recovering && q) {
        let J = this.stoppedAt != null && q[0].pos > this.stoppedAt ? q[0] : this.runRecovery(q, G, V);
        if (J)
          return verbose && console.log("Force-finish " + this.stackID(J)), this.stackToTree(J.forceAll());
      }
      if (this.recovering) {
        let J = this.recovering == 1 ? 1 : this.recovering * 3;
        if (V.length > J)
          for (V.sort((Z, te) => te.score - Z.score); V.length > J; )
            V.pop();
        V.some((Z) => Z.reducePos > H) && this.recovering--;
      } else if (V.length > 1) {
        e: for (let J = 0; J < V.length - 1; J++) {
          let Z = V[J];
          for (let te = J + 1; te < V.length; te++) {
            let ne = V[te];
            if (Z.sameState(ne) || Z.buffer.length > 500 && ne.buffer.length > 500)
              if ((Z.score - ne.score || Z.buffer.length - ne.buffer.length) > 0)
                V.splice(te--, 1);
              else {
                V.splice(J--, 1);
                continue e;
              }
          }
        }
        V.length > 12 && V.splice(
          12,
          V.length - 12
          /* Rec.MaxStackCount */
        );
      }
      this.minStackPos = V[0].pos;
      for (let J = 1; J < V.length; J++)
        V[J].pos < this.minStackPos && (this.minStackPos = V[J].pos);
      return null;
    }
    stopAt(_) {
      if (this.stoppedAt != null && this.stoppedAt < _)
        throw new RangeError("Can't move stoppedAt forward");
      this.stoppedAt = _;
    }
    // Returns an updated version of the given stack, or null if the
    // stack can't advance normally. When `split` and `stacks` are
    // given, stacks split off by ambiguous operations will be pushed to
    // `split`, or added to `stacks` if they move `pos` forward.
    advanceStack(_, H, V) {
      let q = _.pos, { parser: G } = this, J = verbose ? this.stackID(_) + " -> " : "";
      if (this.stoppedAt != null && q > this.stoppedAt)
        return _.forceReduce() ? _ : null;
      if (this.fragments) {
        let ne = _.curContext && _.curContext.tracker.strict, ee = ne ? _.curContext.hash : 0;
        for (let ae = this.fragments.nodeAt(q); ae; ) {
          let oe = this.parser.nodeSet.types[ae.type.id] == ae.type ? G.getGoto(_.state, ae.type.id) : -1;
          if (oe > -1 && ae.length && (!ne || (ae.prop(NodeProp.contextHash) || 0) == ee))
            return _.useNode(ae, oe), verbose && console.log(J + this.stackID(_) + ` (via reuse of ${G.getName(ae.type.id)})`), !0;
          if (!(ae instanceof Tree) || ae.children.length == 0 || ae.positions[0] > 0)
            break;
          let se = ae.children[0];
          if (se instanceof Tree && ae.positions[0] == 0)
            ae = se;
          else
            break;
        }
      }
      let Z = G.stateSlot(
        _.state,
        4
        /* ParseState.DefaultReduce */
      );
      if (Z > 0)
        return _.reduce(Z), verbose && console.log(J + this.stackID(_) + ` (via always-reduce ${G.getName(
          Z & 65535
          /* Action.ValueMask */
        )})`), !0;
      if (_.stack.length >= 8400)
        for (; _.stack.length > 6e3 && _.forceReduce(); )
          ;
      let te = this.tokens.getActions(_);
      for (let ne = 0; ne < te.length; ) {
        let ee = te[ne++], ae = te[ne++], oe = te[ne++], se = ne == te.length || !V, ce = se ? _ : _.split(), le = this.tokens.mainToken;
        if (ce.apply(ee, ae, le ? le.start : ce.pos, oe), verbose && console.log(J + this.stackID(ce) + ` (via ${ee & 65536 ? `reduce of ${G.getName(
          ee & 65535
          /* Action.ValueMask */
        )}` : "shift"} for ${G.getName(ae)} @ ${q}${ce == _ ? "" : ", split"})`), se)
          return !0;
        ce.pos > q ? H.push(ce) : V.push(ce);
      }
      return !1;
    }
    // Advance a given stack forward as far as it will go. Returns the
    // (possibly updated) stack if it got stuck, or null if it moved
    // forward and was given to `pushStackDedup`.
    advanceFully(_, H) {
      let V = _.pos;
      for (; ; ) {
        if (!this.advanceStack(_, null, null))
          return !1;
        if (_.pos > V)
          return pushStackDedup(_, H), !0;
      }
    }
    runRecovery(_, H, V) {
      let q = null, G = !1;
      for (let J = 0; J < _.length; J++) {
        let Z = _[J], te = H[J << 1], ne = H[(J << 1) + 1], ee = verbose ? this.stackID(Z) + " -> " : "";
        if (Z.deadEnd && (G || (G = !0, Z.restart(), verbose && console.log(ee + this.stackID(Z) + " (restarted)"), this.advanceFully(Z, V))))
          continue;
        let ae = Z.split(), oe = ee;
        for (let se = 0; ae.forceReduce() && se < 10 && (verbose && console.log(oe + this.stackID(ae) + " (via force-reduce)"), !this.advanceFully(ae, V)); se++)
          verbose && (oe = this.stackID(ae) + " -> ");
        for (let se of Z.recoverByInsert(te))
          verbose && console.log(ee + this.stackID(se) + " (via recover-insert)"), this.advanceFully(se, V);
        this.stream.end > Z.pos ? (ne == Z.pos && (ne++, te = 0), Z.recoverByDelete(te, ne), verbose && console.log(ee + this.stackID(Z) + ` (via recover-delete ${this.parser.getName(te)})`), pushStackDedup(Z, V)) : (!q || q.score < Z.score) && (q = Z);
      }
      return q;
    }
    // Convert the stack's buffer to a syntax tree.
    stackToTree(_) {
      return _.close(), Tree.build({
        buffer: StackBufferCursor.create(_),
        nodeSet: this.parser.nodeSet,
        topID: this.topTerm,
        maxBufferLength: this.parser.bufferLength,
        reused: this.reused,
        start: this.ranges[0].from,
        length: _.pos - this.ranges[0].from,
        minRepeatType: this.parser.minRepeatTerm
      });
    }
    stackID(_) {
      let H = (stackIDs || (stackIDs = /* @__PURE__ */ new WeakMap())).get(_);
      return H || stackIDs.set(_, H = String.fromCodePoint(this.nextStackID++)), H + _;
    }
  }
  function pushStackDedup(O, _) {
    for (let H = 0; H < _.length; H++) {
      let V = _[H];
      if (V.pos == O.pos && V.sameState(O)) {
        _[H].score < O.score && (_[H] = O);
        return;
      }
    }
    _.push(O);
  }
  class Dialect {
    constructor(_, H, V) {
      this.source = _, this.flags = H, this.disabled = V;
    }
    allows(_) {
      return !this.disabled || this.disabled[_] == 0;
    }
  }
  class LRParser extends Parser {
    /**
    @internal
    */
    constructor(_) {
      if (super(), this.wrappers = [], _.version != 14)
        throw new RangeError(`Parser version (${_.version}) doesn't match runtime version (14)`);
      let H = _.nodeNames.split(" ");
      this.minRepeatTerm = H.length;
      for (let Z = 0; Z < _.repeatNodeCount; Z++)
        H.push("");
      let V = Object.keys(_.topRules).map((Z) => _.topRules[Z][1]), q = [];
      for (let Z = 0; Z < H.length; Z++)
        q.push([]);
      function G(Z, te, ne) {
        q[Z].push([te, te.deserialize(String(ne))]);
      }
      if (_.nodeProps)
        for (let Z of _.nodeProps) {
          let te = Z[0];
          typeof te == "string" && (te = NodeProp[te]);
          for (let ne = 1; ne < Z.length; ) {
            let ee = Z[ne++];
            if (ee >= 0)
              G(ee, te, Z[ne++]);
            else {
              let ae = Z[ne + -ee];
              for (let oe = -ee; oe > 0; oe--)
                G(Z[ne++], te, ae);
              ne++;
            }
          }
        }
      this.nodeSet = new NodeSet(H.map((Z, te) => NodeType.define({
        name: te >= this.minRepeatTerm ? void 0 : Z,
        id: te,
        props: q[te],
        top: V.indexOf(te) > -1,
        error: te == 0,
        skipped: _.skippedNodes && _.skippedNodes.indexOf(te) > -1
      }))), _.propSources && (this.nodeSet = this.nodeSet.extend(..._.propSources)), this.strict = !1, this.bufferLength = DefaultBufferLength;
      let J = decodeArray(_.tokenData);
      this.context = _.context, this.specializerSpecs = _.specialized || [], this.specialized = new Uint16Array(this.specializerSpecs.length);
      for (let Z = 0; Z < this.specializerSpecs.length; Z++)
        this.specialized[Z] = this.specializerSpecs[Z].term;
      this.specializers = this.specializerSpecs.map(getSpecializer), this.states = decodeArray(_.states, Uint32Array), this.data = decodeArray(_.stateData), this.goto = decodeArray(_.goto), this.maxTerm = _.maxTerm, this.tokenizers = _.tokenizers.map((Z) => typeof Z == "number" ? new TokenGroup(J, Z) : Z), this.topRules = _.topRules, this.dialects = _.dialects || {}, this.dynamicPrecedences = _.dynamicPrecedences || null, this.tokenPrecTable = _.tokenPrec, this.termNames = _.termNames || null, this.maxNode = this.nodeSet.types.length - 1, this.dialect = this.parseDialect(), this.top = this.topRules[Object.keys(this.topRules)[0]];
    }
    createParse(_, H, V) {
      let q = new Parse(this, _, H, V);
      for (let G of this.wrappers)
        q = G(q, _, H, V);
      return q;
    }
    /**
    Get a goto table entry @internal
    */
    getGoto(_, H, V = !1) {
      let q = this.goto;
      if (H >= q[0])
        return -1;
      for (let G = q[H + 1]; ; ) {
        let J = q[G++], Z = J & 1, te = q[G++];
        if (Z && V)
          return te;
        for (let ne = G + (J >> 1); G < ne; G++)
          if (q[G] == _)
            return te;
        if (Z)
          return -1;
      }
    }
    /**
    Check if this state has an action for a given terminal @internal
    */
    hasAction(_, H) {
      let V = this.data;
      for (let q = 0; q < 2; q++)
        for (let G = this.stateSlot(
          _,
          q ? 2 : 1
          /* ParseState.Actions */
        ), J; ; G += 3) {
          if ((J = V[G]) == 65535)
            if (V[G + 1] == 1)
              J = V[G = pair(V, G + 2)];
            else {
              if (V[G + 1] == 2)
                return pair(V, G + 2);
              break;
            }
          if (J == H || J == 0)
            return pair(V, G + 1);
        }
      return 0;
    }
    /**
    @internal
    */
    stateSlot(_, H) {
      return this.states[_ * 6 + H];
    }
    /**
    @internal
    */
    stateFlag(_, H) {
      return (this.stateSlot(
        _,
        0
        /* ParseState.Flags */
      ) & H) > 0;
    }
    /**
    @internal
    */
    validAction(_, H) {
      return !!this.allActions(_, (V) => V == H ? !0 : null);
    }
    /**
    @internal
    */
    allActions(_, H) {
      let V = this.stateSlot(
        _,
        4
        /* ParseState.DefaultReduce */
      ), q = V ? H(V) : void 0;
      for (let G = this.stateSlot(
        _,
        1
        /* ParseState.Actions */
      ); q == null; G += 3) {
        if (this.data[G] == 65535)
          if (this.data[G + 1] == 1)
            G = pair(this.data, G + 2);
          else
            break;
        q = H(pair(this.data, G + 1));
      }
      return q;
    }
    /**
    Get the states that can follow this one through shift actions or
    goto jumps. @internal
    */
    nextStates(_) {
      let H = [];
      for (let V = this.stateSlot(
        _,
        1
        /* ParseState.Actions */
      ); ; V += 3) {
        if (this.data[V] == 65535)
          if (this.data[V + 1] == 1)
            V = pair(this.data, V + 2);
          else
            break;
        if (!(this.data[V + 2] & 1)) {
          let q = this.data[V + 1];
          H.some((G, J) => J & 1 && G == q) || H.push(this.data[V], q);
        }
      }
      return H;
    }
    /**
    Configure the parser. Returns a new parser instance that has the
    given settings modified. Settings not provided in `config` are
    kept from the original parser.
    */
    configure(_) {
      let H = Object.assign(Object.create(LRParser.prototype), this);
      if (_.props && (H.nodeSet = this.nodeSet.extend(..._.props)), _.top) {
        let V = this.topRules[_.top];
        if (!V)
          throw new RangeError(`Invalid top rule name ${_.top}`);
        H.top = V;
      }
      return _.tokenizers && (H.tokenizers = this.tokenizers.map((V) => {
        let q = _.tokenizers.find((G) => G.from == V);
        return q ? q.to : V;
      })), _.specializers && (H.specializers = this.specializers.slice(), H.specializerSpecs = this.specializerSpecs.map((V, q) => {
        let G = _.specializers.find((Z) => Z.from == V.external);
        if (!G)
          return V;
        let J = Object.assign(Object.assign({}, V), { external: G.to });
        return H.specializers[q] = getSpecializer(J), J;
      })), _.contextTracker && (H.context = _.contextTracker), _.dialect && (H.dialect = this.parseDialect(_.dialect)), _.strict != null && (H.strict = _.strict), _.wrap && (H.wrappers = H.wrappers.concat(_.wrap)), _.bufferLength != null && (H.bufferLength = _.bufferLength), H;
    }
    /**
    Tells you whether any [parse wrappers](#lr.ParserConfig.wrap)
    are registered for this parser.
    */
    hasWrappers() {
      return this.wrappers.length > 0;
    }
    /**
    Returns the name associated with a given term. This will only
    work for all terms when the parser was generated with the
    `--names` option. By default, only the names of tagged terms are
    stored.
    */
    getName(_) {
      return this.termNames ? this.termNames[_] : String(_ <= this.maxNode && this.nodeSet.types[_].name || _);
    }
    /**
    The eof term id is always allocated directly after the node
    types. @internal
    */
    get eofTerm() {
      return this.maxNode + 1;
    }
    /**
    The type of top node produced by the parser.
    */
    get topNode() {
      return this.nodeSet.types[this.top[1]];
    }
    /**
    @internal
    */
    dynamicPrecedence(_) {
      let H = this.dynamicPrecedences;
      return H == null ? 0 : H[_] || 0;
    }
    /**
    @internal
    */
    parseDialect(_) {
      let H = Object.keys(this.dialects), V = H.map(() => !1);
      if (_)
        for (let G of _.split(" ")) {
          let J = H.indexOf(G);
          J >= 0 && (V[J] = !0);
        }
      let q = null;
      for (let G = 0; G < H.length; G++)
        if (!V[G])
          for (let J = this.dialects[H[G]], Z; (Z = this.data[J++]) != 65535; )
            (q || (q = new Uint8Array(this.maxTerm + 1)))[Z] = 1;
      return new Dialect(_, V, q);
    }
    /**
    Used by the output of the parser generator. Not available to
    user code. @hide
    */
    static deserialize(_) {
      return new LRParser(_);
    }
  }
  function pair(O, _) {
    return O[_] | O[_ + 1] << 16;
  }
  function findFinished(O) {
    let _ = null;
    for (let H of O) {
      let V = H.p.stoppedAt;
      (H.pos == H.p.stream.end || V != null && H.pos > V) && H.p.parser.stateFlag(
        H.state,
        2
        /* StateFlag.Accepting */
      ) && (!_ || _.score < H.score) && (_ = H);
    }
    return _;
  }
  function getSpecializer(O) {
    if (O.external) {
      let _ = O.extend ? 1 : 0;
      return (H, V) => O.external(H, V) << 1 | _;
    }
    return O.get;
  }
  class CompletionContext {
    /**
    Create a new completion context. (Mostly useful for testing
    completion sources—in the editor, the extension will create
    these for you.)
    */
    constructor(_, H, V, q) {
      this.state = _, this.pos = H, this.explicit = V, this.view = q, this.abortListeners = [], this.abortOnDocChange = !1;
    }
    /**
    Get the extent, content, and (if there is a token) type of the
    token before `this.pos`.
    */
    tokenBefore(_) {
      let H = syntaxTree(this.state).resolveInner(this.pos, -1);
      for (; H && _.indexOf(H.name) < 0; )
        H = H.parent;
      return H ? {
        from: H.from,
        to: this.pos,
        text: this.state.sliceDoc(H.from, this.pos),
        type: H.type
      } : null;
    }
    /**
    Get the match of the given expression directly before the
    cursor.
    */
    matchBefore(_) {
      let H = this.state.doc.lineAt(this.pos), V = Math.max(H.from, this.pos - 250), q = H.text.slice(V - H.from, this.pos - H.from), G = q.search(ensureAnchor(_, !1));
      return G < 0 ? null : { from: V + G, to: this.pos, text: q.slice(G) };
    }
    /**
    Yields true when the query has been aborted. Can be useful in
    asynchronous queries to avoid doing work that will be ignored.
    */
    get aborted() {
      return this.abortListeners == null;
    }
    /**
    Allows you to register abort handlers, which will be called when
    the query is
    [aborted](https://codemirror.net/6/docs/ref/#autocomplete.CompletionContext.aborted).
    
    By default, running queries will not be aborted for regular
    typing or backspacing, on the assumption that they are likely to
    return a result with a
    [`validFor`](https://codemirror.net/6/docs/ref/#autocomplete.CompletionResult.validFor) field that
    allows the result to be used after all. Passing `onDocChange:
    true` will cause this query to be aborted for any document
    change.
    */
    addEventListener(_, H, V) {
      _ == "abort" && this.abortListeners && (this.abortListeners.push(H), V && V.onDocChange && (this.abortOnDocChange = !0));
    }
  }
  function toSet(O) {
    let _ = Object.keys(O).join(""), H = /\w/.test(_);
    return H && (_ = _.replace(/\w/g, "")), `[${H ? "\\w" : ""}${_.replace(/[^\w\s]/g, "\\$&")}]`;
  }
  function prefixMatch(O) {
    let _ = /* @__PURE__ */ Object.create(null), H = /* @__PURE__ */ Object.create(null);
    for (let { label: q } of O) {
      _[q[0]] = !0;
      for (let G = 1; G < q.length; G++)
        H[q[G]] = !0;
    }
    let V = toSet(_) + toSet(H) + "*$";
    return [new RegExp("^" + V), new RegExp(V)];
  }
  function completeFromList(O) {
    let _ = O.map((q) => typeof q == "string" ? { label: q } : q), [H, V] = _.every((q) => /^\w+$/.test(q.label)) ? [/\w*$/, /\w+$/] : prefixMatch(_);
    return (q) => {
      let G = q.matchBefore(V);
      return G || q.explicit ? { from: G ? G.from : q.pos, options: _, validFor: H } : null;
    };
  }
  function ifNotIn(O, _) {
    return (H) => {
      for (let V = syntaxTree(H.state).resolveInner(H.pos, -1); V; V = V.parent) {
        if (O.indexOf(V.name) > -1)
          return null;
        if (V.type.isTop)
          break;
      }
      return _(H);
    };
  }
  let Option$1 = class {
    constructor(_, H, V, q) {
      this.completion = _, this.source = H, this.match = V, this.score = q;
    }
  };
  function cur(O) {
    return O.selection.main.from;
  }
  function ensureAnchor(O, _) {
    var H;
    let { source: V } = O, q = _ && V[0] != "^", G = V[V.length - 1] != "$";
    return !q && !G ? O : new RegExp(`${q ? "^" : ""}(?:${V})${G ? "$" : ""}`, (H = O.flags) !== null && H !== void 0 ? H : O.ignoreCase ? "i" : "");
  }
  const pickedCompletion = /* @__PURE__ */ Annotation.define();
  function insertCompletionText(O, _, H, V) {
    let { main: q } = O.selection, G = H - q.from, J = V - q.from;
    return Object.assign(Object.assign({}, O.changeByRange((Z) => {
      if (Z != q && H != V && O.sliceDoc(Z.from + G, Z.from + J) != O.sliceDoc(H, V))
        return { range: Z };
      let te = O.toText(_);
      return {
        changes: { from: Z.from + G, to: V == q.from ? Z.to : Z.from + J, insert: te },
        range: EditorSelection.cursor(Z.from + G + te.length)
      };
    })), { scrollIntoView: !0, userEvent: "input.complete" });
  }
  const SourceCache = /* @__PURE__ */ new WeakMap();
  function asSource(O) {
    if (!Array.isArray(O))
      return O;
    let _ = SourceCache.get(O);
    return _ || SourceCache.set(O, _ = completeFromList(O)), _;
  }
  const startCompletionEffect = /* @__PURE__ */ StateEffect.define(), closeCompletionEffect = /* @__PURE__ */ StateEffect.define();
  class FuzzyMatcher {
    constructor(_) {
      this.pattern = _, this.chars = [], this.folded = [], this.any = [], this.precise = [], this.byWord = [], this.score = 0, this.matched = [];
      for (let H = 0; H < _.length; ) {
        let V = codePointAt(_, H), q = codePointSize(V);
        this.chars.push(V);
        let G = _.slice(H, H + q), J = G.toUpperCase();
        this.folded.push(codePointAt(J == G ? G.toLowerCase() : J, 0)), H += q;
      }
      this.astral = _.length != this.chars.length;
    }
    ret(_, H) {
      return this.score = _, this.matched = H, this;
    }
    // Matches a given word (completion) against the pattern (input).
    // Will return a boolean indicating whether there was a match and,
    // on success, set `this.score` to the score, `this.matched` to an
    // array of `from, to` pairs indicating the matched parts of `word`.
    //
    // The score is a number that is more negative the worse the match
    // is. See `Penalty` above.
    match(_) {
      if (this.pattern.length == 0)
        return this.ret(-100, []);
      if (_.length < this.pattern.length)
        return null;
      let { chars: H, folded: V, any: q, precise: G, byWord: J } = this;
      if (H.length == 1) {
        let fe = codePointAt(_, 0), he = codePointSize(fe), me = he == _.length ? 0 : -100;
        if (fe != H[0]) if (fe == V[0])
          me += -200;
        else
          return null;
        return this.ret(me, [0, he]);
      }
      let Z = _.indexOf(this.pattern);
      if (Z == 0)
        return this.ret(_.length == this.pattern.length ? 0 : -100, [0, this.pattern.length]);
      let te = H.length, ne = 0;
      if (Z < 0) {
        for (let fe = 0, he = Math.min(_.length, 200); fe < he && ne < te; ) {
          let me = codePointAt(_, fe);
          (me == H[ne] || me == V[ne]) && (q[ne++] = fe), fe += codePointSize(me);
        }
        if (ne < te)
          return null;
      }
      let ee = 0, ae = 0, oe = !1, se = 0, ce = -1, le = -1, ue = /[a-z]/.test(_), de = !0;
      for (let fe = 0, he = Math.min(_.length, 200), me = 0; fe < he && ae < te; ) {
        let pe = codePointAt(_, fe);
        Z < 0 && (ee < te && pe == H[ee] && (G[ee++] = fe), se < te && (pe == H[se] || pe == V[se] ? (se == 0 && (ce = fe), le = fe + 1, se++) : se = 0));
        let ye, ge = pe < 255 ? pe >= 48 && pe <= 57 || pe >= 97 && pe <= 122 ? 2 : pe >= 65 && pe <= 90 ? 1 : 0 : (ye = fromCodePoint(pe)) != ye.toLowerCase() ? 1 : ye != ye.toUpperCase() ? 2 : 0;
        (!fe || ge == 1 && ue || me == 0 && ge != 0) && (H[ae] == pe || V[ae] == pe && (oe = !0) ? J[ae++] = fe : J.length && (de = !1)), me = ge, fe += codePointSize(pe);
      }
      return ae == te && J[0] == 0 && de ? this.result(-100 + (oe ? -200 : 0), J, _) : se == te && ce == 0 ? this.ret(-200 - _.length + (le == _.length ? 0 : -100), [0, le]) : Z > -1 ? this.ret(-700 - _.length, [Z, Z + this.pattern.length]) : se == te ? this.ret(-900 - _.length, [ce, le]) : ae == te ? this.result(-100 + (oe ? -200 : 0) + -700 + (de ? 0 : -1100), J, _) : H.length == 2 ? null : this.result((q[0] ? -700 : 0) + -200 + -1100, q, _);
    }
    result(_, H, V) {
      let q = [], G = 0;
      for (let J of H) {
        let Z = J + (this.astral ? codePointSize(codePointAt(V, J)) : 1);
        G && q[G - 1] == J ? q[G - 1] = Z : (q[G++] = J, q[G++] = Z);
      }
      return this.ret(_ - V.length, q);
    }
  }
  class StrictMatcher {
    constructor(_) {
      this.pattern = _, this.matched = [], this.score = 0, this.folded = _.toLowerCase();
    }
    match(_) {
      if (_.length < this.pattern.length)
        return null;
      let H = _.slice(0, this.pattern.length), V = H == this.pattern ? 0 : H.toLowerCase() == this.folded ? -200 : null;
      return V == null ? null : (this.matched = [0, H.length], this.score = V + (_.length == this.pattern.length ? 0 : -100), this);
    }
  }
  const completionConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, {
        activateOnTyping: !0,
        activateOnCompletion: () => !1,
        activateOnTypingDelay: 100,
        selectOnOpen: !0,
        override: null,
        closeOnBlur: !0,
        maxRenderedOptions: 100,
        defaultKeymap: !0,
        tooltipClass: () => "",
        optionClass: () => "",
        aboveCursor: !1,
        icons: !0,
        addToOptions: [],
        positionInfo: defaultPositionInfo,
        filterStrict: !1,
        compareCompletions: (_, H) => _.label.localeCompare(H.label),
        interactionDelay: 75,
        updateSyncTime: 100
      }, {
        defaultKeymap: (_, H) => _ && H,
        closeOnBlur: (_, H) => _ && H,
        icons: (_, H) => _ && H,
        tooltipClass: (_, H) => (V) => joinClass(_(V), H(V)),
        optionClass: (_, H) => (V) => joinClass(_(V), H(V)),
        addToOptions: (_, H) => _.concat(H),
        filterStrict: (_, H) => _ || H
      });
    }
  });
  function joinClass(O, _) {
    return O ? _ ? O + " " + _ : O : _;
  }
  function defaultPositionInfo(O, _, H, V, q, G) {
    let J = O.textDirection == Direction.RTL, Z = J, te = !1, ne = "top", ee, ae, oe = _.left - q.left, se = q.right - _.right, ce = V.right - V.left, le = V.bottom - V.top;
    if (Z && oe < Math.min(ce, se) ? Z = !1 : !Z && se < Math.min(ce, oe) && (Z = !0), ce <= (Z ? oe : se))
      ee = Math.max(q.top, Math.min(H.top, q.bottom - le)) - _.top, ae = Math.min(400, Z ? oe : se);
    else {
      te = !0, ae = Math.min(
        400,
        (J ? _.right : q.right - _.left) - 30
        /* Info.Margin */
      );
      let fe = q.bottom - _.bottom;
      fe >= le || fe > _.top ? ee = H.bottom - _.top : (ne = "bottom", ee = _.bottom - H.top);
    }
    let ue = (_.bottom - _.top) / G.offsetHeight, de = (_.right - _.left) / G.offsetWidth;
    return {
      style: `${ne}: ${ee / ue}px; max-width: ${ae / de}px`,
      class: "cm-completionInfo-" + (te ? J ? "left-narrow" : "right-narrow" : Z ? "left" : "right")
    };
  }
  function optionContent(O) {
    let _ = O.addToOptions.slice();
    return O.icons && _.push({
      render(H) {
        let V = document.createElement("div");
        return V.classList.add("cm-completionIcon"), H.type && V.classList.add(...H.type.split(/\s+/g).map((q) => "cm-completionIcon-" + q)), V.setAttribute("aria-hidden", "true"), V;
      },
      position: 20
    }), _.push({
      render(H, V, q, G) {
        let J = document.createElement("span");
        J.className = "cm-completionLabel";
        let Z = H.displayLabel || H.label, te = 0;
        for (let ne = 0; ne < G.length; ) {
          let ee = G[ne++], ae = G[ne++];
          ee > te && J.appendChild(document.createTextNode(Z.slice(te, ee)));
          let oe = J.appendChild(document.createElement("span"));
          oe.appendChild(document.createTextNode(Z.slice(ee, ae))), oe.className = "cm-completionMatchedText", te = ae;
        }
        return te < Z.length && J.appendChild(document.createTextNode(Z.slice(te))), J;
      },
      position: 50
    }, {
      render(H) {
        if (!H.detail)
          return null;
        let V = document.createElement("span");
        return V.className = "cm-completionDetail", V.textContent = H.detail, V;
      },
      position: 80
    }), _.sort((H, V) => H.position - V.position).map((H) => H.render);
  }
  function rangeAroundSelected(O, _, H) {
    if (O <= H)
      return { from: 0, to: O };
    if (_ < 0 && (_ = 0), _ <= O >> 1) {
      let q = Math.floor(_ / H);
      return { from: q * H, to: (q + 1) * H };
    }
    let V = Math.floor((O - _) / H);
    return { from: O - (V + 1) * H, to: O - V * H };
  }
  class CompletionTooltip {
    constructor(_, H, V) {
      this.view = _, this.stateField = H, this.applyCompletion = V, this.info = null, this.infoDestroy = null, this.placeInfoReq = {
        read: () => this.measureInfo(),
        write: (te) => this.placeInfo(te),
        key: this
      }, this.space = null, this.currentClass = "";
      let q = _.state.field(H), { options: G, selected: J } = q.open, Z = _.state.facet(completionConfig);
      this.optionContent = optionContent(Z), this.optionClass = Z.optionClass, this.tooltipClass = Z.tooltipClass, this.range = rangeAroundSelected(G.length, J, Z.maxRenderedOptions), this.dom = document.createElement("div"), this.dom.className = "cm-tooltip-autocomplete", this.updateTooltipClass(_.state), this.dom.addEventListener("mousedown", (te) => {
        let { options: ne } = _.state.field(H).open;
        for (let ee = te.target, ae; ee && ee != this.dom; ee = ee.parentNode)
          if (ee.nodeName == "LI" && (ae = /-(\d+)$/.exec(ee.id)) && +ae[1] < ne.length) {
            this.applyCompletion(_, ne[+ae[1]]), te.preventDefault();
            return;
          }
      }), this.dom.addEventListener("focusout", (te) => {
        let ne = _.state.field(this.stateField, !1);
        ne && ne.tooltip && _.state.facet(completionConfig).closeOnBlur && te.relatedTarget != _.contentDOM && _.dispatch({ effects: closeCompletionEffect.of(null) });
      }), this.showOptions(G, q.id);
    }
    mount() {
      this.updateSel();
    }
    showOptions(_, H) {
      this.list && this.list.remove(), this.list = this.dom.appendChild(this.createListBox(_, H, this.range)), this.list.addEventListener("scroll", () => {
        this.info && this.view.requestMeasure(this.placeInfoReq);
      });
    }
    update(_) {
      var H;
      let V = _.state.field(this.stateField), q = _.startState.field(this.stateField);
      if (this.updateTooltipClass(_.state), V != q) {
        let { options: G, selected: J, disabled: Z } = V.open;
        (!q.open || q.open.options != G) && (this.range = rangeAroundSelected(G.length, J, _.state.facet(completionConfig).maxRenderedOptions), this.showOptions(G, V.id)), this.updateSel(), Z != ((H = q.open) === null || H === void 0 ? void 0 : H.disabled) && this.dom.classList.toggle("cm-tooltip-autocomplete-disabled", !!Z);
      }
    }
    updateTooltipClass(_) {
      let H = this.tooltipClass(_);
      if (H != this.currentClass) {
        for (let V of this.currentClass.split(" "))
          V && this.dom.classList.remove(V);
        for (let V of H.split(" "))
          V && this.dom.classList.add(V);
        this.currentClass = H;
      }
    }
    positioned(_) {
      this.space = _, this.info && this.view.requestMeasure(this.placeInfoReq);
    }
    updateSel() {
      let _ = this.view.state.field(this.stateField), H = _.open;
      if ((H.selected > -1 && H.selected < this.range.from || H.selected >= this.range.to) && (this.range = rangeAroundSelected(H.options.length, H.selected, this.view.state.facet(completionConfig).maxRenderedOptions), this.showOptions(H.options, _.id)), this.updateSelectedOption(H.selected)) {
        this.destroyInfo();
        let { completion: V } = H.options[H.selected], { info: q } = V;
        if (!q)
          return;
        let G = typeof q == "string" ? document.createTextNode(q) : q(V);
        if (!G)
          return;
        "then" in G ? G.then((J) => {
          J && this.view.state.field(this.stateField, !1) == _ && this.addInfoPane(J, V);
        }).catch((J) => logException(this.view.state, J, "completion info")) : this.addInfoPane(G, V);
      }
    }
    addInfoPane(_, H) {
      this.destroyInfo();
      let V = this.info = document.createElement("div");
      if (V.className = "cm-tooltip cm-completionInfo", _.nodeType != null)
        V.appendChild(_), this.infoDestroy = null;
      else {
        let { dom: q, destroy: G } = _;
        V.appendChild(q), this.infoDestroy = G || null;
      }
      this.dom.appendChild(V), this.view.requestMeasure(this.placeInfoReq);
    }
    updateSelectedOption(_) {
      let H = null;
      for (let V = this.list.firstChild, q = this.range.from; V; V = V.nextSibling, q++)
        V.nodeName != "LI" || !V.id ? q-- : q == _ ? V.hasAttribute("aria-selected") || (V.setAttribute("aria-selected", "true"), H = V) : V.hasAttribute("aria-selected") && V.removeAttribute("aria-selected");
      return H && scrollIntoView(this.list, H), H;
    }
    measureInfo() {
      let _ = this.dom.querySelector("[aria-selected]");
      if (!_ || !this.info)
        return null;
      let H = this.dom.getBoundingClientRect(), V = this.info.getBoundingClientRect(), q = _.getBoundingClientRect(), G = this.space;
      if (!G) {
        let J = this.dom.ownerDocument.defaultView || window;
        G = { left: 0, top: 0, right: J.innerWidth, bottom: J.innerHeight };
      }
      return q.top > Math.min(G.bottom, H.bottom) - 10 || q.bottom < Math.max(G.top, H.top) + 10 ? null : this.view.state.facet(completionConfig).positionInfo(this.view, H, q, V, G, this.dom);
    }
    placeInfo(_) {
      this.info && (_ ? (_.style && (this.info.style.cssText = _.style), this.info.className = "cm-tooltip cm-completionInfo " + (_.class || "")) : this.info.style.cssText = "top: -1e6px");
    }
    createListBox(_, H, V) {
      const q = document.createElement("ul");
      q.id = H, q.setAttribute("role", "listbox"), q.setAttribute("aria-expanded", "true"), q.setAttribute("aria-label", this.view.state.phrase("Completions"));
      let G = null;
      for (let J = V.from; J < V.to; J++) {
        let { completion: Z, match: te } = _[J], { section: ne } = Z;
        if (ne) {
          let oe = typeof ne == "string" ? ne : ne.name;
          if (oe != G && (J > V.from || V.from == 0))
            if (G = oe, typeof ne != "string" && ne.header)
              q.appendChild(ne.header(ne));
            else {
              let se = q.appendChild(document.createElement("completion-section"));
              se.textContent = oe;
            }
        }
        const ee = q.appendChild(document.createElement("li"));
        ee.id = H + "-" + J, ee.setAttribute("role", "option");
        let ae = this.optionClass(Z);
        ae && (ee.className = ae);
        for (let oe of this.optionContent) {
          let se = oe(Z, this.view.state, this.view, te);
          se && ee.appendChild(se);
        }
      }
      return V.from && q.classList.add("cm-completionListIncompleteTop"), V.to < _.length && q.classList.add("cm-completionListIncompleteBottom"), q;
    }
    destroyInfo() {
      this.info && (this.infoDestroy && this.infoDestroy(), this.info.remove(), this.info = null);
    }
    destroy() {
      this.destroyInfo();
    }
  }
  function completionTooltip(O, _) {
    return (H) => new CompletionTooltip(H, O, _);
  }
  function scrollIntoView(O, _) {
    let H = O.getBoundingClientRect(), V = _.getBoundingClientRect(), q = H.height / O.offsetHeight;
    V.top < H.top ? O.scrollTop -= (H.top - V.top) / q : V.bottom > H.bottom && (O.scrollTop += (V.bottom - H.bottom) / q);
  }
  function score(O) {
    return (O.boost || 0) * 100 + (O.apply ? 10 : 0) + (O.info ? 5 : 0) + (O.type ? 1 : 0);
  }
  function sortOptions(O, _) {
    let H = [], V = null, q = (ne) => {
      H.push(ne);
      let { section: ee } = ne.completion;
      if (ee) {
        V || (V = []);
        let ae = typeof ee == "string" ? ee : ee.name;
        V.some((oe) => oe.name == ae) || V.push(typeof ee == "string" ? { name: ae } : ee);
      }
    }, G = _.facet(completionConfig);
    for (let ne of O)
      if (ne.hasResult()) {
        let ee = ne.result.getMatch;
        if (ne.result.filter === !1)
          for (let ae of ne.result.options)
            q(new Option$1(ae, ne.source, ee ? ee(ae) : [], 1e9 - H.length));
        else {
          let ae = _.sliceDoc(ne.from, ne.to), oe, se = G.filterStrict ? new StrictMatcher(ae) : new FuzzyMatcher(ae);
          for (let ce of ne.result.options)
            if (oe = se.match(ce.label)) {
              let le = ce.displayLabel ? ee ? ee(ce, oe.matched) : [] : oe.matched;
              q(new Option$1(ce, ne.source, le, oe.score + (ce.boost || 0)));
            }
        }
      }
    if (V) {
      let ne = /* @__PURE__ */ Object.create(null), ee = 0, ae = (oe, se) => {
        var ce, le;
        return ((ce = oe.rank) !== null && ce !== void 0 ? ce : 1e9) - ((le = se.rank) !== null && le !== void 0 ? le : 1e9) || (oe.name < se.name ? -1 : 1);
      };
      for (let oe of V.sort(ae))
        ee -= 1e5, ne[oe.name] = ee;
      for (let oe of H) {
        let { section: se } = oe.completion;
        se && (oe.score += ne[typeof se == "string" ? se : se.name]);
      }
    }
    let J = [], Z = null, te = G.compareCompletions;
    for (let ne of H.sort((ee, ae) => ae.score - ee.score || te(ee.completion, ae.completion))) {
      let ee = ne.completion;
      !Z || Z.label != ee.label || Z.detail != ee.detail || Z.type != null && ee.type != null && Z.type != ee.type || Z.apply != ee.apply || Z.boost != ee.boost ? J.push(ne) : score(ne.completion) > score(Z) && (J[J.length - 1] = ne), Z = ne.completion;
    }
    return J;
  }
  class CompletionDialog {
    constructor(_, H, V, q, G, J) {
      this.options = _, this.attrs = H, this.tooltip = V, this.timestamp = q, this.selected = G, this.disabled = J;
    }
    setSelected(_, H) {
      return _ == this.selected || _ >= this.options.length ? this : new CompletionDialog(this.options, makeAttrs(H, _), this.tooltip, this.timestamp, _, this.disabled);
    }
    static build(_, H, V, q, G) {
      let J = sortOptions(_, H);
      if (!J.length)
        return q && _.some(
          (te) => te.state == 1
          /* State.Pending */
        ) ? new CompletionDialog(q.options, q.attrs, q.tooltip, q.timestamp, q.selected, !0) : null;
      let Z = H.facet(completionConfig).selectOnOpen ? 0 : -1;
      if (q && q.selected != Z && q.selected != -1) {
        let te = q.options[q.selected].completion;
        for (let ne = 0; ne < J.length; ne++)
          if (J[ne].completion == te) {
            Z = ne;
            break;
          }
      }
      return new CompletionDialog(J, makeAttrs(V, Z), {
        pos: _.reduce((te, ne) => ne.hasResult() ? Math.min(te, ne.from) : te, 1e8),
        create: createTooltip,
        above: G.aboveCursor
      }, q ? q.timestamp : Date.now(), Z, !1);
    }
    map(_) {
      return new CompletionDialog(this.options, this.attrs, Object.assign(Object.assign({}, this.tooltip), { pos: _.mapPos(this.tooltip.pos) }), this.timestamp, this.selected, this.disabled);
    }
  }
  class CompletionState {
    constructor(_, H, V) {
      this.active = _, this.id = H, this.open = V;
    }
    static start() {
      return new CompletionState(none$1, "cm-ac-" + Math.floor(Math.random() * 2e6).toString(36), null);
    }
    update(_) {
      let { state: H } = _, V = H.facet(completionConfig), G = (V.override || H.languageDataAt("autocomplete", cur(H)).map(asSource)).map((Z) => (this.active.find((ne) => ne.source == Z) || new ActiveSource(
        Z,
        this.active.some(
          (ne) => ne.state != 0
          /* State.Inactive */
        ) ? 1 : 0
        /* State.Inactive */
      )).update(_, V));
      G.length == this.active.length && G.every((Z, te) => Z == this.active[te]) && (G = this.active);
      let J = this.open;
      J && _.docChanged && (J = J.map(_.changes)), _.selection || G.some((Z) => Z.hasResult() && _.changes.touchesRange(Z.from, Z.to)) || !sameResults(G, this.active) ? J = CompletionDialog.build(G, H, this.id, J, V) : J && J.disabled && !G.some(
        (Z) => Z.state == 1
        /* State.Pending */
      ) && (J = null), !J && G.every(
        (Z) => Z.state != 1
        /* State.Pending */
      ) && G.some((Z) => Z.hasResult()) && (G = G.map((Z) => Z.hasResult() ? new ActiveSource(
        Z.source,
        0
        /* State.Inactive */
      ) : Z));
      for (let Z of _.effects)
        Z.is(setSelectedEffect) && (J = J && J.setSelected(Z.value, this.id));
      return G == this.active && J == this.open ? this : new CompletionState(G, this.id, J);
    }
    get tooltip() {
      return this.open ? this.open.tooltip : null;
    }
    get attrs() {
      return this.open ? this.open.attrs : this.active.length ? baseAttrs : noAttrs;
    }
  }
  function sameResults(O, _) {
    if (O == _)
      return !0;
    for (let H = 0, V = 0; ; ) {
      for (; H < O.length && !O[H].hasResult; )
        H++;
      for (; V < _.length && !_[V].hasResult; )
        V++;
      let q = H == O.length, G = V == _.length;
      if (q || G)
        return q == G;
      if (O[H++].result != _[V++].result)
        return !1;
    }
  }
  const baseAttrs = {
    "aria-autocomplete": "list"
  }, noAttrs = {};
  function makeAttrs(O, _) {
    let H = {
      "aria-autocomplete": "list",
      "aria-haspopup": "listbox",
      "aria-controls": O
    };
    return _ > -1 && (H["aria-activedescendant"] = O + "-" + _), H;
  }
  const none$1 = [];
  function getUpdateType(O, _) {
    if (O.isUserEvent("input.complete")) {
      let V = O.annotation(pickedCompletion);
      if (V && _.activateOnCompletion(V))
        return 12;
    }
    let H = O.isUserEvent("input.type");
    return H && _.activateOnTyping ? 5 : H ? 1 : O.isUserEvent("delete.backward") ? 2 : O.selection ? 8 : O.docChanged ? 16 : 0;
  }
  class ActiveSource {
    constructor(_, H, V = -1) {
      this.source = _, this.state = H, this.explicitPos = V;
    }
    hasResult() {
      return !1;
    }
    update(_, H) {
      let V = getUpdateType(_, H), q = this;
      (V & 8 || V & 16 && this.touches(_)) && (q = new ActiveSource(
        q.source,
        0
        /* State.Inactive */
      )), V & 4 && q.state == 0 && (q = new ActiveSource(
        this.source,
        1
        /* State.Pending */
      )), q = q.updateFor(_, V);
      for (let G of _.effects)
        if (G.is(startCompletionEffect))
          q = new ActiveSource(q.source, 1, G.value ? cur(_.state) : -1);
        else if (G.is(closeCompletionEffect))
          q = new ActiveSource(
            q.source,
            0
            /* State.Inactive */
          );
        else if (G.is(setActiveEffect))
          for (let J of G.value)
            J.source == q.source && (q = J);
      return q;
    }
    updateFor(_, H) {
      return this.map(_.changes);
    }
    map(_) {
      return _.empty || this.explicitPos < 0 ? this : new ActiveSource(this.source, this.state, _.mapPos(this.explicitPos));
    }
    touches(_) {
      return _.changes.touchesRange(cur(_.state));
    }
  }
  class ActiveResult extends ActiveSource {
    constructor(_, H, V, q, G) {
      super(_, 2, H), this.result = V, this.from = q, this.to = G;
    }
    hasResult() {
      return !0;
    }
    updateFor(_, H) {
      var V;
      if (!(H & 3))
        return this.map(_.changes);
      let q = this.result;
      q.map && !_.changes.empty && (q = q.map(q, _.changes));
      let G = _.changes.mapPos(this.from), J = _.changes.mapPos(this.to, 1), Z = cur(_.state);
      if ((this.explicitPos < 0 ? Z <= G : Z < this.from) || Z > J || !q || H & 2 && cur(_.startState) == this.from)
        return new ActiveSource(
          this.source,
          H & 4 ? 1 : 0
          /* State.Inactive */
        );
      let te = this.explicitPos < 0 ? -1 : _.changes.mapPos(this.explicitPos);
      return checkValid(q.validFor, _.state, G, J) ? new ActiveResult(this.source, te, q, G, J) : q.update && (q = q.update(q, G, J, new CompletionContext(_.state, Z, te >= 0))) ? new ActiveResult(this.source, te, q, q.from, (V = q.to) !== null && V !== void 0 ? V : cur(_.state)) : new ActiveSource(this.source, 1, te);
    }
    map(_) {
      return _.empty ? this : (this.result.map ? this.result.map(this.result, _) : this.result) ? new ActiveResult(this.source, this.explicitPos < 0 ? -1 : _.mapPos(this.explicitPos), this.result, _.mapPos(this.from), _.mapPos(this.to, 1)) : new ActiveSource(
        this.source,
        0
        /* State.Inactive */
      );
    }
    touches(_) {
      return _.changes.touchesRange(this.from, this.to);
    }
  }
  function checkValid(O, _, H, V) {
    if (!O)
      return !1;
    let q = _.sliceDoc(H, V);
    return typeof O == "function" ? O(q, H, V, _) : ensureAnchor(O, !0).test(q);
  }
  const setActiveEffect = /* @__PURE__ */ StateEffect.define({
    map(O, _) {
      return O.map((H) => H.map(_));
    }
  }), setSelectedEffect = /* @__PURE__ */ StateEffect.define(), completionState = /* @__PURE__ */ StateField.define({
    create() {
      return CompletionState.start();
    },
    update(O, _) {
      return O.update(_);
    },
    provide: (O) => [
      showTooltip.from(O, (_) => _.tooltip),
      EditorView.contentAttributes.from(O, (_) => _.attrs)
    ]
  });
  function applyCompletion(O, _) {
    const H = _.completion.apply || _.completion.label;
    let V = O.state.field(completionState).active.find((q) => q.source == _.source);
    return V instanceof ActiveResult ? (typeof H == "string" ? O.dispatch(Object.assign(Object.assign({}, insertCompletionText(O.state, H, V.from, V.to)), { annotations: pickedCompletion.of(_.completion) })) : H(O, _.completion, V.from, V.to), !0) : !1;
  }
  const createTooltip = /* @__PURE__ */ completionTooltip(completionState, applyCompletion);
  function moveCompletionSelection(O, _ = "option") {
    return (H) => {
      let V = H.state.field(completionState, !1);
      if (!V || !V.open || V.open.disabled || Date.now() - V.open.timestamp < H.state.facet(completionConfig).interactionDelay)
        return !1;
      let q = 1, G;
      _ == "page" && (G = getTooltip(H, V.open.tooltip)) && (q = Math.max(2, Math.floor(G.dom.offsetHeight / G.dom.querySelector("li").offsetHeight) - 1));
      let { length: J } = V.open.options, Z = V.open.selected > -1 ? V.open.selected + q * (O ? 1 : -1) : O ? 0 : J - 1;
      return Z < 0 ? Z = _ == "page" ? 0 : J - 1 : Z >= J && (Z = _ == "page" ? J - 1 : 0), H.dispatch({ effects: setSelectedEffect.of(Z) }), !0;
    };
  }
  const acceptCompletion = (O) => {
    let _ = O.state.field(completionState, !1);
    return O.state.readOnly || !_ || !_.open || _.open.selected < 0 || _.open.disabled || Date.now() - _.open.timestamp < O.state.facet(completionConfig).interactionDelay ? !1 : applyCompletion(O, _.open.options[_.open.selected]);
  }, startCompletion = (O) => O.state.field(completionState, !1) ? (O.dispatch({ effects: startCompletionEffect.of(!0) }), !0) : !1, closeCompletion = (O) => {
    let _ = O.state.field(completionState, !1);
    return !_ || !_.active.some(
      (H) => H.state != 0
      /* State.Inactive */
    ) ? !1 : (O.dispatch({ effects: closeCompletionEffect.of(null) }), !0);
  };
  class RunningQuery {
    constructor(_, H) {
      this.active = _, this.context = H, this.time = Date.now(), this.updates = [], this.done = void 0;
    }
  }
  const MaxUpdateCount = 50, MinAbortTime = 1e3, completionPlugin = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.view = O, this.debounceUpdate = -1, this.running = [], this.debounceAccept = -1, this.pendingStart = !1, this.composing = 0;
      for (let _ of O.state.field(completionState).active)
        _.state == 1 && this.startQuery(_);
    }
    update(O) {
      let _ = O.state.field(completionState), H = O.state.facet(completionConfig);
      if (!O.selectionSet && !O.docChanged && O.startState.field(completionState) == _)
        return;
      let V = O.transactions.some((G) => {
        let J = getUpdateType(G, H);
        return J & 8 || (G.selection || G.docChanged) && !(J & 3);
      });
      for (let G = 0; G < this.running.length; G++) {
        let J = this.running[G];
        if (V || J.context.abortOnDocChange && O.docChanged || J.updates.length + O.transactions.length > MaxUpdateCount && Date.now() - J.time > MinAbortTime) {
          for (let Z of J.context.abortListeners)
            try {
              Z();
            } catch (te) {
              logException(this.view.state, te);
            }
          J.context.abortListeners = null, this.running.splice(G--, 1);
        } else
          J.updates.push(...O.transactions);
      }
      this.debounceUpdate > -1 && clearTimeout(this.debounceUpdate), O.transactions.some((G) => G.effects.some((J) => J.is(startCompletionEffect))) && (this.pendingStart = !0);
      let q = this.pendingStart ? 50 : H.activateOnTypingDelay;
      if (this.debounceUpdate = _.active.some((G) => G.state == 1 && !this.running.some((J) => J.active.source == G.source)) ? setTimeout(() => this.startUpdate(), q) : -1, this.composing != 0)
        for (let G of O.transactions)
          G.isUserEvent("input.type") ? this.composing = 2 : this.composing == 2 && G.selection && (this.composing = 3);
    }
    startUpdate() {
      this.debounceUpdate = -1, this.pendingStart = !1;
      let { state: O } = this.view, _ = O.field(completionState);
      for (let H of _.active)
        H.state == 1 && !this.running.some((V) => V.active.source == H.source) && this.startQuery(H);
    }
    startQuery(O) {
      let { state: _ } = this.view, H = cur(_), V = new CompletionContext(_, H, O.explicitPos == H, this.view), q = new RunningQuery(O, V);
      this.running.push(q), Promise.resolve(O.source(V)).then((G) => {
        q.context.aborted || (q.done = G || null, this.scheduleAccept());
      }, (G) => {
        this.view.dispatch({ effects: closeCompletionEffect.of(null) }), logException(this.view.state, G);
      });
    }
    scheduleAccept() {
      this.running.every((O) => O.done !== void 0) ? this.accept() : this.debounceAccept < 0 && (this.debounceAccept = setTimeout(() => this.accept(), this.view.state.facet(completionConfig).updateSyncTime));
    }
    // For each finished query in this.running, try to create a result
    // or, if appropriate, restart the query.
    accept() {
      var O;
      this.debounceAccept > -1 && clearTimeout(this.debounceAccept), this.debounceAccept = -1;
      let _ = [], H = this.view.state.facet(completionConfig);
      for (let V = 0; V < this.running.length; V++) {
        let q = this.running[V];
        if (q.done === void 0)
          continue;
        if (this.running.splice(V--, 1), q.done) {
          let J = new ActiveResult(q.active.source, q.active.explicitPos, q.done, q.done.from, (O = q.done.to) !== null && O !== void 0 ? O : cur(q.updates.length ? q.updates[0].startState : this.view.state));
          for (let Z of q.updates)
            J = J.update(Z, H);
          if (J.hasResult()) {
            _.push(J);
            continue;
          }
        }
        let G = this.view.state.field(completionState).active.find((J) => J.source == q.active.source);
        if (G && G.state == 1)
          if (q.done == null) {
            let J = new ActiveSource(
              q.active.source,
              0
              /* State.Inactive */
            );
            for (let Z of q.updates)
              J = J.update(Z, H);
            J.state != 1 && _.push(J);
          } else
            this.startQuery(G);
      }
      _.length && this.view.dispatch({ effects: setActiveEffect.of(_) });
    }
  }, {
    eventHandlers: {
      blur(O) {
        let _ = this.view.state.field(completionState, !1);
        if (_ && _.tooltip && this.view.state.facet(completionConfig).closeOnBlur) {
          let H = _.open && getTooltip(this.view, _.open.tooltip);
          (!H || !H.dom.contains(O.relatedTarget)) && setTimeout(() => this.view.dispatch({ effects: closeCompletionEffect.of(null) }), 10);
        }
      },
      compositionstart() {
        this.composing = 1;
      },
      compositionend() {
        this.composing == 3 && setTimeout(() => this.view.dispatch({ effects: startCompletionEffect.of(!1) }), 20), this.composing = 0;
      }
    }
  }), windows = typeof navigator == "object" && /* @__PURE__ */ /Win/.test(navigator.platform), commitCharacters = /* @__PURE__ */ Prec.highest(/* @__PURE__ */ EditorView.domEventHandlers({
    keydown(O, _) {
      let H = _.state.field(completionState, !1);
      if (!H || !H.open || H.open.disabled || H.open.selected < 0 || O.key.length > 1 || O.ctrlKey && !(windows && O.altKey) || O.metaKey)
        return !1;
      let V = H.open.options[H.open.selected], q = H.active.find((J) => J.source == V.source), G = V.completion.commitCharacters || q.result.commitCharacters;
      return G && G.indexOf(O.key) > -1 && applyCompletion(_, V), !1;
    }
  })), baseTheme$3 = /* @__PURE__ */ EditorView.baseTheme({
    ".cm-tooltip.cm-tooltip-autocomplete": {
      "& > ul": {
        fontFamily: "monospace",
        whiteSpace: "nowrap",
        overflow: "hidden auto",
        maxWidth_fallback: "700px",
        maxWidth: "min(700px, 95vw)",
        minWidth: "250px",
        maxHeight: "10em",
        height: "100%",
        listStyle: "none",
        margin: 0,
        padding: 0,
        "& > li, & > completion-section": {
          padding: "1px 3px",
          lineHeight: 1.2
        },
        "& > li": {
          overflowX: "hidden",
          textOverflow: "ellipsis",
          cursor: "pointer"
        },
        "& > completion-section": {
          display: "list-item",
          borderBottom: "1px solid silver",
          paddingLeft: "0.5em",
          opacity: 0.7
        }
      }
    },
    "&light .cm-tooltip-autocomplete ul li[aria-selected]": {
      background: "#17c",
      color: "white"
    },
    "&light .cm-tooltip-autocomplete-disabled ul li[aria-selected]": {
      background: "#777"
    },
    "&dark .cm-tooltip-autocomplete ul li[aria-selected]": {
      background: "#347",
      color: "white"
    },
    "&dark .cm-tooltip-autocomplete-disabled ul li[aria-selected]": {
      background: "#444"
    },
    ".cm-completionListIncompleteTop:before, .cm-completionListIncompleteBottom:after": {
      content: '"···"',
      opacity: 0.5,
      display: "block",
      textAlign: "center"
    },
    ".cm-tooltip.cm-completionInfo": {
      position: "absolute",
      padding: "3px 9px",
      width: "max-content",
      maxWidth: "400px",
      boxSizing: "border-box",
      whiteSpace: "pre-line"
    },
    ".cm-completionInfo.cm-completionInfo-left": { right: "100%" },
    ".cm-completionInfo.cm-completionInfo-right": { left: "100%" },
    ".cm-completionInfo.cm-completionInfo-left-narrow": { right: "30px" },
    ".cm-completionInfo.cm-completionInfo-right-narrow": { left: "30px" },
    "&light .cm-snippetField": { backgroundColor: "#00000022" },
    "&dark .cm-snippetField": { backgroundColor: "#ffffff22" },
    ".cm-snippetFieldPosition": {
      verticalAlign: "text-top",
      width: 0,
      height: "1.15em",
      display: "inline-block",
      margin: "0 -0.7px -.7em",
      borderLeft: "1.4px dotted #888"
    },
    ".cm-completionMatchedText": {
      textDecoration: "underline"
    },
    ".cm-completionDetail": {
      marginLeft: "0.5em",
      fontStyle: "italic"
    },
    ".cm-completionIcon": {
      fontSize: "90%",
      width: ".8em",
      display: "inline-block",
      textAlign: "center",
      paddingRight: ".6em",
      opacity: "0.6",
      boxSizing: "content-box"
    },
    ".cm-completionIcon-function, .cm-completionIcon-method": {
      "&:after": { content: "'ƒ'" }
    },
    ".cm-completionIcon-class": {
      "&:after": { content: "'○'" }
    },
    ".cm-completionIcon-interface": {
      "&:after": { content: "'◌'" }
    },
    ".cm-completionIcon-variable": {
      "&:after": { content: "'𝑥'" }
    },
    ".cm-completionIcon-constant": {
      "&:after": { content: "'𝐶'" }
    },
    ".cm-completionIcon-type": {
      "&:after": { content: "'𝑡'" }
    },
    ".cm-completionIcon-enum": {
      "&:after": { content: "'∪'" }
    },
    ".cm-completionIcon-property": {
      "&:after": { content: "'□'" }
    },
    ".cm-completionIcon-keyword": {
      "&:after": { content: "'🔑︎'" }
      // Disable emoji rendering
    },
    ".cm-completionIcon-namespace": {
      "&:after": { content: "'▢'" }
    },
    ".cm-completionIcon-text": {
      "&:after": { content: "'abc'", fontSize: "50%", verticalAlign: "middle" }
    }
  }), defaults$1 = {
    brackets: ["(", "[", "{", "'", '"'],
    before: ")]}:;>",
    stringPrefixes: []
  }, closeBracketEffect = /* @__PURE__ */ StateEffect.define({
    map(O, _) {
      let H = _.mapPos(O, -1, MapMode.TrackAfter);
      return H ?? void 0;
    }
  }), closedBracket = /* @__PURE__ */ new class extends RangeValue {
  }();
  closedBracket.startSide = 1;
  closedBracket.endSide = -1;
  const bracketState = /* @__PURE__ */ StateField.define({
    create() {
      return RangeSet.empty;
    },
    update(O, _) {
      if (O = O.map(_.changes), _.selection) {
        let H = _.state.doc.lineAt(_.selection.main.head);
        O = O.update({ filter: (V) => V >= H.from && V <= H.to });
      }
      for (let H of _.effects)
        H.is(closeBracketEffect) && (O = O.update({ add: [closedBracket.range(H.value, H.value + 1)] }));
      return O;
    }
  });
  function closeBrackets() {
    return [inputHandler, bracketState];
  }
  const definedClosing = "()[]{}<>";
  function closing(O) {
    for (let _ = 0; _ < definedClosing.length; _ += 2)
      if (definedClosing.charCodeAt(_) == O)
        return definedClosing.charAt(_ + 1);
    return fromCodePoint(O < 128 ? O : O + 1);
  }
  function config(O, _) {
    return O.languageDataAt("closeBrackets", _)[0] || defaults$1;
  }
  const android = typeof navigator == "object" && /* @__PURE__ */ /Android\b/.test(navigator.userAgent), inputHandler = /* @__PURE__ */ EditorView.inputHandler.of((O, _, H, V) => {
    if ((android ? O.composing : O.compositionStarted) || O.state.readOnly)
      return !1;
    let q = O.state.selection.main;
    if (V.length > 2 || V.length == 2 && codePointSize(codePointAt(V, 0)) == 1 || _ != q.from || H != q.to)
      return !1;
    let G = insertBracket(O.state, V);
    return G ? (O.dispatch(G), !0) : !1;
  }), deleteBracketPair = ({ state: O, dispatch: _ }) => {
    if (O.readOnly)
      return !1;
    let V = config(O, O.selection.main.head).brackets || defaults$1.brackets, q = null, G = O.changeByRange((J) => {
      if (J.empty) {
        let Z = prevChar(O.doc, J.head);
        for (let te of V)
          if (te == Z && nextChar(O.doc, J.head) == closing(codePointAt(te, 0)))
            return {
              changes: { from: J.head - te.length, to: J.head + te.length },
              range: EditorSelection.cursor(J.head - te.length)
            };
      }
      return { range: q = J };
    });
    return q || _(O.update(G, { scrollIntoView: !0, userEvent: "delete.backward" })), !q;
  }, closeBracketsKeymap = [
    { key: "Backspace", run: deleteBracketPair }
  ];
  function insertBracket(O, _) {
    let H = config(O, O.selection.main.head), V = H.brackets || defaults$1.brackets;
    for (let q of V) {
      let G = closing(codePointAt(q, 0));
      if (_ == q)
        return G == q ? handleSame(O, q, V.indexOf(q + q + q) > -1, H) : handleOpen(O, q, G, H.before || defaults$1.before);
      if (_ == G && closedBracketAt(O, O.selection.main.from))
        return handleClose(O, q, G);
    }
    return null;
  }
  function closedBracketAt(O, _) {
    let H = !1;
    return O.field(bracketState).between(0, O.doc.length, (V) => {
      V == _ && (H = !0);
    }), H;
  }
  function nextChar(O, _) {
    let H = O.sliceString(_, _ + 2);
    return H.slice(0, codePointSize(codePointAt(H, 0)));
  }
  function prevChar(O, _) {
    let H = O.sliceString(_ - 2, _);
    return codePointSize(codePointAt(H, 0)) == H.length ? H : H.slice(1);
  }
  function handleOpen(O, _, H, V) {
    let q = null, G = O.changeByRange((J) => {
      if (!J.empty)
        return {
          changes: [{ insert: _, from: J.from }, { insert: H, from: J.to }],
          effects: closeBracketEffect.of(J.to + _.length),
          range: EditorSelection.range(J.anchor + _.length, J.head + _.length)
        };
      let Z = nextChar(O.doc, J.head);
      return !Z || /\s/.test(Z) || V.indexOf(Z) > -1 ? {
        changes: { insert: _ + H, from: J.head },
        effects: closeBracketEffect.of(J.head + _.length),
        range: EditorSelection.cursor(J.head + _.length)
      } : { range: q = J };
    });
    return q ? null : O.update(G, {
      scrollIntoView: !0,
      userEvent: "input.type"
    });
  }
  function handleClose(O, _, H) {
    let V = null, q = O.changeByRange((G) => G.empty && nextChar(O.doc, G.head) == H ? {
      changes: { from: G.head, to: G.head + H.length, insert: H },
      range: EditorSelection.cursor(G.head + H.length)
    } : V = { range: G });
    return V ? null : O.update(q, {
      scrollIntoView: !0,
      userEvent: "input.type"
    });
  }
  function handleSame(O, _, H, V) {
    let q = V.stringPrefixes || defaults$1.stringPrefixes, G = null, J = O.changeByRange((Z) => {
      if (!Z.empty)
        return {
          changes: [{ insert: _, from: Z.from }, { insert: _, from: Z.to }],
          effects: closeBracketEffect.of(Z.to + _.length),
          range: EditorSelection.range(Z.anchor + _.length, Z.head + _.length)
        };
      let te = Z.head, ne = nextChar(O.doc, te), ee;
      if (ne == _) {
        if (nodeStart(O, te))
          return {
            changes: { insert: _ + _, from: te },
            effects: closeBracketEffect.of(te + _.length),
            range: EditorSelection.cursor(te + _.length)
          };
        if (closedBracketAt(O, te)) {
          let oe = H && O.sliceDoc(te, te + _.length * 3) == _ + _ + _ ? _ + _ + _ : _;
          return {
            changes: { from: te, to: te + oe.length, insert: oe },
            range: EditorSelection.cursor(te + oe.length)
          };
        }
      } else {
        if (H && O.sliceDoc(te - 2 * _.length, te) == _ + _ && (ee = canStartStringAt(O, te - 2 * _.length, q)) > -1 && nodeStart(O, ee))
          return {
            changes: { insert: _ + _ + _ + _, from: te },
            effects: closeBracketEffect.of(te + _.length),
            range: EditorSelection.cursor(te + _.length)
          };
        if (O.charCategorizer(te)(ne) != CharCategory.Word && canStartStringAt(O, te, q) > -1 && !probablyInString(O, te, _, q))
          return {
            changes: { insert: _ + _, from: te },
            effects: closeBracketEffect.of(te + _.length),
            range: EditorSelection.cursor(te + _.length)
          };
      }
      return { range: G = Z };
    });
    return G ? null : O.update(J, {
      scrollIntoView: !0,
      userEvent: "input.type"
    });
  }
  function nodeStart(O, _) {
    let H = syntaxTree(O).resolveInner(_ + 1);
    return H.parent && H.from == _;
  }
  function probablyInString(O, _, H, V) {
    let q = syntaxTree(O).resolveInner(_, -1), G = V.reduce((J, Z) => Math.max(J, Z.length), 0);
    for (let J = 0; J < 5; J++) {
      let Z = O.sliceDoc(q.from, Math.min(q.to, q.from + H.length + G)), te = Z.indexOf(H);
      if (!te || te > -1 && V.indexOf(Z.slice(0, te)) > -1) {
        let ee = q.firstChild;
        for (; ee && ee.from == q.from && ee.to - ee.from > H.length + te; ) {
          if (O.sliceDoc(ee.to - H.length, ee.to) == H)
            return !1;
          ee = ee.firstChild;
        }
        return !0;
      }
      let ne = q.to == _ && q.parent;
      if (!ne)
        break;
      q = ne;
    }
    return !1;
  }
  function canStartStringAt(O, _, H) {
    let V = O.charCategorizer(_);
    if (V(O.sliceDoc(_ - 1, _)) != CharCategory.Word)
      return _;
    for (let q of H) {
      let G = _ - q.length;
      if (O.sliceDoc(G, _) == q && V(O.sliceDoc(G - 1, G)) != CharCategory.Word)
        return G;
    }
    return -1;
  }
  function autocompletion(O = {}) {
    return [
      commitCharacters,
      completionState,
      completionConfig.of(O),
      completionPlugin,
      completionKeymapExt,
      baseTheme$3
    ];
  }
  const completionKeymap = [
    { key: "Ctrl-Space", run: startCompletion },
    { mac: "Alt-`", run: startCompletion },
    { key: "Escape", run: closeCompletion },
    { key: "ArrowDown", run: /* @__PURE__ */ moveCompletionSelection(!0) },
    { key: "ArrowUp", run: /* @__PURE__ */ moveCompletionSelection(!1) },
    { key: "PageDown", run: /* @__PURE__ */ moveCompletionSelection(!0, "page") },
    { key: "PageUp", run: /* @__PURE__ */ moveCompletionSelection(!1, "page") },
    { key: "Enter", run: acceptCompletion }
  ], completionKeymapExt = /* @__PURE__ */ Prec.highest(/* @__PURE__ */ keymap.computeN([completionConfig], (O) => O.facet(completionConfig).defaultKeymap ? [completionKeymap] : [])), whitespace = 36, LineComment = 1, BlockComment = 2, String$1 = 3, Number$1 = 4, Bool$2 = 5, Null$2 = 6, ParenL = 7, ParenR = 8, BraceL = 9, BraceR = 10, BracketL = 11, BracketR = 12, Semi = 13, Dot = 14, Operator = 15, Punctuation = 16, SpecialVar = 17, Identifier = 18, QuotedIdentifier = 19, Keyword = 20, Type$2 = 21, Bits = 22, Bytes = 23, Builtin = 24;
  function isAlpha(O) {
    return O >= 65 && O <= 90 || O >= 97 && O <= 122 || O >= 48 && O <= 57;
  }
  function isHexDigit(O) {
    return O >= 48 && O <= 57 || O >= 97 && O <= 102 || O >= 65 && O <= 70;
  }
  function readLiteral(O, _, H) {
    for (let V = !1; ; ) {
      if (O.next < 0)
        return;
      if (O.next == _ && !V) {
        O.advance();
        return;
      }
      V = H && !V && O.next == 92, O.advance();
    }
  }
  function readDoubleDollarLiteral(O, _) {
    e: for (; ; ) {
      if (O.next < 0)
        return;
      if (O.next == 36) {
        O.advance();
        for (let H = 0; H < _.length; H++) {
          if (O.next != _.charCodeAt(H))
            continue e;
          O.advance();
        }
        if (O.next == 36) {
          O.advance();
          return;
        }
      } else
        O.advance();
    }
  }
  function readPLSQLQuotedLiteral(O, _) {
    let H = "[{<(".indexOf(String.fromCharCode(_)), V = H < 0 ? _ : "]}>)".charCodeAt(H);
    for (; ; ) {
      if (O.next < 0)
        return;
      if (O.next == V && O.peek(1) == 39) {
        O.advance(2);
        return;
      }
      O.advance();
    }
  }
  function readWord(O, _) {
    for (; !(O.next != 95 && !isAlpha(O.next)); )
      _ != null && (_ += String.fromCharCode(O.next)), O.advance();
    return _;
  }
  function readWordOrQuoted(O) {
    if (O.next == 39 || O.next == 34 || O.next == 96) {
      let _ = O.next;
      O.advance(), readLiteral(O, _, !1);
    } else
      readWord(O);
  }
  function readBits(O, _) {
    for (; O.next == 48 || O.next == 49; )
      O.advance();
    _ && O.next == _ && O.advance();
  }
  function readNumber(O, _) {
    for (; ; ) {
      if (O.next == 46) {
        if (_)
          break;
        _ = !0;
      } else if (O.next < 48 || O.next > 57)
        break;
      O.advance();
    }
    if (O.next == 69 || O.next == 101)
      for (O.advance(), (O.next == 43 || O.next == 45) && O.advance(); O.next >= 48 && O.next <= 57; )
        O.advance();
  }
  function eol(O) {
    for (; !(O.next < 0 || O.next == 10); )
      O.advance();
  }
  function inString(O, _) {
    for (let H = 0; H < _.length; H++)
      if (_.charCodeAt(H) == O)
        return !0;
    return !1;
  }
  const Space = ` 	\r
`;
  function keywords(O, _, H) {
    let V = /* @__PURE__ */ Object.create(null);
    V.true = V.false = Bool$2, V.null = V.unknown = Null$2;
    for (let q of O.split(" "))
      q && (V[q] = Keyword);
    for (let q of _.split(" "))
      q && (V[q] = Type$2);
    for (let q of (H || "").split(" "))
      q && (V[q] = Builtin);
    return V;
  }
  const SQLTypes = "array binary bit boolean char character clob date decimal double float int integer interval large national nchar nclob numeric object precision real smallint time timestamp varchar varying ", SQLKeywords = "absolute action add after all allocate alter and any are as asc assertion at authorization before begin between both breadth by call cascade cascaded case cast catalog check close collate collation column commit condition connect connection constraint constraints constructor continue corresponding count create cross cube current current_date current_default_transform_group current_transform_group_for_type current_path current_role current_time current_timestamp current_user cursor cycle data day deallocate declare default deferrable deferred delete depth deref desc describe descriptor deterministic diagnostics disconnect distinct do domain drop dynamic each else elseif end end-exec equals escape except exception exec execute exists exit external fetch first for foreign found from free full function general get global go goto grant group grouping handle having hold hour identity if immediate in indicator initially inner inout input insert intersect into is isolation join key language last lateral leading leave left level like limit local localtime localtimestamp locator loop map match method minute modifies module month names natural nesting new next no none not of old on only open option or order ordinality out outer output overlaps pad parameter partial path prepare preserve primary prior privileges procedure public read reads recursive redo ref references referencing relative release repeat resignal restrict result return returns revoke right role rollback rollup routine row rows savepoint schema scroll search second section select session session_user set sets signal similar size some space specific specifictype sql sqlexception sqlstate sqlwarning start state static system_user table temporary then timezone_hour timezone_minute to trailing transaction translation treat trigger under undo union unique unnest until update usage user using value values view when whenever where while with without work write year zone ", defaults = {
    backslashEscapes: !1,
    hashComments: !1,
    spaceAfterDashes: !1,
    slashComments: !1,
    doubleQuotedStrings: !1,
    doubleDollarQuotedStrings: !1,
    unquotedBitLiterals: !1,
    treatBitsAsBytes: !1,
    charSetCasts: !1,
    plsqlQuotingMechanism: !1,
    operatorChars: "*+-%<>!=&|~^/",
    specialVar: "?",
    identifierQuotes: '"',
    caseInsensitiveIdentifiers: !1,
    words: /* @__PURE__ */ keywords(SQLKeywords, SQLTypes)
  };
  function dialect(O, _, H, V) {
    let q = {};
    for (let G in defaults)
      q[G] = (O.hasOwnProperty(G) ? O : defaults)[G];
    return _ && (q.words = keywords(_, H || "", V)), q;
  }
  function tokensFor(O) {
    return new ExternalTokenizer((_) => {
      var H;
      let { next: V } = _;
      if (_.advance(), inString(V, Space)) {
        for (; inString(_.next, Space); )
          _.advance();
        _.acceptToken(whitespace);
      } else if (V == 36 && O.doubleDollarQuotedStrings) {
        let q = readWord(_, "");
        _.next == 36 && (_.advance(), readDoubleDollarLiteral(_, q), _.acceptToken(String$1));
      } else if (V == 39 || V == 34 && O.doubleQuotedStrings)
        readLiteral(_, V, O.backslashEscapes), _.acceptToken(String$1);
      else if (V == 35 && O.hashComments || V == 47 && _.next == 47 && O.slashComments)
        eol(_), _.acceptToken(LineComment);
      else if (V == 45 && _.next == 45 && (!O.spaceAfterDashes || _.peek(1) == 32))
        eol(_), _.acceptToken(LineComment);
      else if (V == 47 && _.next == 42) {
        _.advance();
        for (let q = 1; ; ) {
          let G = _.next;
          if (_.next < 0)
            break;
          if (_.advance(), G == 42 && _.next == 47) {
            if (q--, _.advance(), !q)
              break;
          } else G == 47 && _.next == 42 && (q++, _.advance());
        }
        _.acceptToken(BlockComment);
      } else if ((V == 101 || V == 69) && _.next == 39)
        _.advance(), readLiteral(_, 39, !0), _.acceptToken(String$1);
      else if ((V == 110 || V == 78) && _.next == 39 && O.charSetCasts)
        _.advance(), readLiteral(_, 39, O.backslashEscapes), _.acceptToken(String$1);
      else if (V == 95 && O.charSetCasts)
        for (let q = 0; ; q++) {
          if (_.next == 39 && q > 1) {
            _.advance(), readLiteral(_, 39, O.backslashEscapes), _.acceptToken(String$1);
            break;
          }
          if (!isAlpha(_.next))
            break;
          _.advance();
        }
      else if (O.plsqlQuotingMechanism && (V == 113 || V == 81) && _.next == 39 && _.peek(1) > 0 && !inString(_.peek(1), Space)) {
        let q = _.peek(1);
        _.advance(2), readPLSQLQuotedLiteral(_, q), _.acceptToken(String$1);
      } else if (V == 40)
        _.acceptToken(ParenL);
      else if (V == 41)
        _.acceptToken(ParenR);
      else if (V == 123)
        _.acceptToken(BraceL);
      else if (V == 125)
        _.acceptToken(BraceR);
      else if (V == 91)
        _.acceptToken(BracketL);
      else if (V == 93)
        _.acceptToken(BracketR);
      else if (V == 59)
        _.acceptToken(Semi);
      else if (O.unquotedBitLiterals && V == 48 && _.next == 98)
        _.advance(), readBits(_), _.acceptToken(Bits);
      else if ((V == 98 || V == 66) && (_.next == 39 || _.next == 34)) {
        const q = _.next;
        _.advance(), O.treatBitsAsBytes ? (readLiteral(_, q, O.backslashEscapes), _.acceptToken(Bytes)) : (readBits(_, q), _.acceptToken(Bits));
      } else if (V == 48 && (_.next == 120 || _.next == 88) || (V == 120 || V == 88) && _.next == 39) {
        let q = _.next == 39;
        for (_.advance(); isHexDigit(_.next); )
          _.advance();
        q && _.next == 39 && _.advance(), _.acceptToken(Number$1);
      } else if (V == 46 && _.next >= 48 && _.next <= 57)
        readNumber(_, !0), _.acceptToken(Number$1);
      else if (V == 46)
        _.acceptToken(Dot);
      else if (V >= 48 && V <= 57)
        readNumber(_, !1), _.acceptToken(Number$1);
      else if (inString(V, O.operatorChars)) {
        for (; inString(_.next, O.operatorChars); )
          _.advance();
        _.acceptToken(Operator);
      } else if (inString(V, O.specialVar))
        _.next == V && _.advance(), readWordOrQuoted(_), _.acceptToken(SpecialVar);
      else if (inString(V, O.identifierQuotes))
        readLiteral(_, V, !1), _.acceptToken(QuotedIdentifier);
      else if (V == 58 || V == 44)
        _.acceptToken(Punctuation);
      else if (isAlpha(V)) {
        let q = readWord(_, String.fromCharCode(V));
        _.acceptToken(_.next == 46 || _.peek(-q.length - 1) == 46 ? Identifier : (H = O.words[q.toLowerCase()]) !== null && H !== void 0 ? H : Identifier);
      }
    });
  }
  const tokens = /* @__PURE__ */ tokensFor(defaults), parser$1 = /* @__PURE__ */ LRParser.deserialize({
    version: 14,
    states: "%vQ]QQOOO#wQRO'#DSO$OQQO'#CwO%eQQO'#CxO%lQQO'#CyO%sQQO'#CzOOQQ'#DS'#DSOOQQ'#C}'#C}O'UQRO'#C{OOQQ'#Cv'#CvOOQQ'#C|'#C|Q]QQOOQOQQOOO'`QQO'#DOO(xQRO,59cO)PQQO,59cO)UQQO'#DSOOQQ,59d,59dO)cQQO,59dOOQQ,59e,59eO)jQQO,59eOOQQ,59f,59fO)qQQO,59fOOQQ-E6{-E6{OOQQ,59b,59bOOQQ-E6z-E6zOOQQ,59j,59jOOQQ-E6|-E6|O+VQRO1G.}O+^QQO,59cOOQQ1G/O1G/OOOQQ1G/P1G/POOQQ1G/Q1G/QP+kQQO'#C}O+rQQO1G.}O)PQQO,59cO,PQQO'#Cw",
    stateData: ",[~OtOSPOSQOS~ORUOSUOTUOUUOVROXSOZTO]XO^QO_UO`UOaPObPOcPOdUOeUOfUOgUOhUO~O^]ORvXSvXTvXUvXVvXXvXZvX]vX_vX`vXavXbvXcvXdvXevXfvXgvXhvX~OsvX~P!jOa_Ob_Oc_O~ORUOSUOTUOUUOVROXSOZTO^tO_UO`UOa`Ob`Oc`OdUOeUOfUOgUOhUO~OWaO~P$ZOYcO~P$ZO[eO~P$ZORUOSUOTUOUUOVROXSOZTO^QO_UO`UOaPObPOcPOdUOeUOfUOgUOhUO~O]hOsoX~P%zOajObjOcjO~O^]ORkaSkaTkaUkaVkaXkaZka]ka_ka`kaakabkackadkaekafkagkahka~Oska~P'kO^]O~OWvXYvX[vX~P!jOWnO~P$ZOYoO~P$ZO[pO~P$ZO^]ORkiSkiTkiUkiVkiXkiZki]ki_ki`kiakibkickidkiekifkigkihki~Oski~P)xOWkaYka[ka~P'kO]hO~P$ZOWkiYki[ki~P)xOasObsOcsO~O",
    goto: "#hwPPPPPPPPPPPPPPPPPPPPPPPPPPx||||!Y!^!d!xPPP#[TYOZeUORSTWZbdfqT[OZQZORiZSWOZQbRQdSQfTZgWbdfqQ^PWk^lmrQl_Qm`RrseVORSTWZbdfq",
    nodeNames: "⚠ LineComment BlockComment String Number Bool Null ( ) { } [ ] ; . Operator Punctuation SpecialVar Identifier QuotedIdentifier Keyword Type Bits Bytes Builtin Script Statement CompositeIdentifier Parens Braces Brackets Statement",
    maxTerm: 38,
    nodeProps: [
      ["isolate", -4, 1, 2, 3, 19, ""]
    ],
    skippedNodes: [0, 1, 2],
    repeatNodeCount: 3,
    tokenData: "RORO",
    tokenizers: [0, tokens],
    topRules: { Script: [0, 25] },
    tokenPrec: 0
  });
  function tokenBefore(O) {
    let _ = O.cursor().moveTo(O.from, -1);
    for (; /Comment/.test(_.name); )
      _.moveTo(_.from, -1);
    return _.node;
  }
  function idName(O, _) {
    let H = O.sliceString(_.from, _.to), V = /^([`'"])(.*)\1$/.exec(H);
    return V ? V[2] : H;
  }
  function plainID(O) {
    return O && (O.name == "Identifier" || O.name == "QuotedIdentifier");
  }
  function pathFor(O, _) {
    if (_.name == "CompositeIdentifier") {
      let H = [];
      for (let V = _.firstChild; V; V = V.nextSibling)
        plainID(V) && H.push(idName(O, V));
      return H;
    }
    return [idName(O, _)];
  }
  function parentsFor(O, _) {
    for (let H = []; ; ) {
      if (!_ || _.name != ".")
        return H;
      let V = tokenBefore(_);
      if (!plainID(V))
        return H;
      H.unshift(idName(O, V)), _ = tokenBefore(V);
    }
  }
  function sourceContext(O, _) {
    let H = syntaxTree(O).resolveInner(_, -1), V = getAliases(O.doc, H);
    return H.name == "Identifier" || H.name == "QuotedIdentifier" || H.name == "Keyword" ? {
      from: H.from,
      quoted: H.name == "QuotedIdentifier" ? O.doc.sliceString(H.from, H.from + 1) : null,
      parents: parentsFor(O.doc, tokenBefore(H)),
      aliases: V
    } : H.name == "." ? { from: _, quoted: null, parents: parentsFor(O.doc, H), aliases: V } : { from: _, quoted: null, parents: [], empty: !0, aliases: V };
  }
  const EndFrom = /* @__PURE__ */ new Set(/* @__PURE__ */ "where group having order union intersect except all distinct limit offset fetch for".split(" "));
  function getAliases(O, _) {
    let H;
    for (let q = _; !H; q = q.parent) {
      if (!q)
        return null;
      q.name == "Statement" && (H = q);
    }
    let V = null;
    for (let q = H.firstChild, G = !1, J = null; q; q = q.nextSibling) {
      let Z = q.name == "Keyword" ? O.sliceString(q.from, q.to).toLowerCase() : null, te = null;
      if (!G)
        G = Z == "from";
      else if (Z == "as" && J && plainID(q.nextSibling))
        te = idName(O, q.nextSibling);
      else {
        if (Z && EndFrom.has(Z))
          break;
        J && plainID(q) && (te = idName(O, q));
      }
      te && (V || (V = /* @__PURE__ */ Object.create(null)), V[te] = pathFor(O, J)), J = /Identifier$/.test(q.name) ? q : null;
    }
    return V;
  }
  function maybeQuoteCompletions(O, _) {
    return O ? _.map((H) => Object.assign(Object.assign({}, H), { label: H.label[0] == O ? H.label : O + H.label + O, apply: void 0 })) : _;
  }
  const Span = /^\w*$/, QuotedSpan = /^[`'"]?\w*[`'"]?$/;
  function isSelfTag(O) {
    return O.self && typeof O.self.label == "string";
  }
  class CompletionLevel {
    constructor(_, H) {
      this.idQuote = _, this.idCaseInsensitive = H, this.list = [], this.children = void 0;
    }
    child(_) {
      let H = this.children || (this.children = /* @__PURE__ */ Object.create(null)), V = H[_];
      return V || (_ && !this.list.some((q) => q.label == _) && this.list.push(nameCompletion(_, "type", this.idQuote, this.idCaseInsensitive)), H[_] = new CompletionLevel(this.idQuote, this.idCaseInsensitive));
    }
    maybeChild(_) {
      return this.children ? this.children[_] : null;
    }
    addCompletion(_) {
      let H = this.list.findIndex((V) => V.label == _.label);
      H > -1 ? this.list[H] = _ : this.list.push(_);
    }
    addCompletions(_) {
      for (let H of _)
        this.addCompletion(typeof H == "string" ? nameCompletion(H, "property", this.idQuote, this.idCaseInsensitive) : H);
    }
    addNamespace(_) {
      Array.isArray(_) ? this.addCompletions(_) : isSelfTag(_) ? this.addNamespace(_.children) : this.addNamespaceObject(_);
    }
    addNamespaceObject(_) {
      for (let H of Object.keys(_)) {
        let V = _[H], q = null, G = H.replace(/\\?\./g, (Z) => Z == "." ? "\0" : Z).split("\0"), J = this;
        isSelfTag(V) && (q = V.self, V = V.children);
        for (let Z = 0; Z < G.length; Z++)
          q && Z == G.length - 1 && J.addCompletion(q), J = J.child(G[Z].replace(/\\\./g, "."));
        J.addNamespace(V);
      }
    }
  }
  function nameCompletion(O, _, H, V) {
    return new RegExp("^[a-z_][a-z_\\d]*$", V ? "i" : "").test(O) ? { label: O, type: _ } : { label: O, type: _, apply: H + O + H };
  }
  function completeFromSchema(O, _, H, V, q, G) {
    var J;
    let Z = ((J = G == null ? void 0 : G.spec.identifierQuotes) === null || J === void 0 ? void 0 : J[0]) || '"', te = new CompletionLevel(Z, !!(G != null && G.spec.caseInsensitiveIdentifiers)), ne = q ? te.child(q) : null;
    return te.addNamespace(O), _ && (ne || te).addCompletions(_), H && te.addCompletions(H), ne && te.addCompletions(ne.list), V && te.addCompletions((ne || te).child(V).list), (ee) => {
      let { parents: ae, from: oe, quoted: se, empty: ce, aliases: le } = sourceContext(ee.state, ee.pos);
      if (ce && !ee.explicit)
        return null;
      le && ae.length == 1 && (ae = le[ae[0]] || ae);
      let ue = te;
      for (let he of ae) {
        for (; !ue.children || !ue.children[he]; )
          if (ue == te && ne)
            ue = ne;
          else if (ue == ne && V)
            ue = ue.child(V);
          else
            return null;
        let me = ue.maybeChild(he);
        if (!me)
          return null;
        ue = me;
      }
      let de = se && ee.state.sliceDoc(ee.pos, ee.pos + 1) == se, fe = ue.list;
      return ue == te && le && (fe = fe.concat(Object.keys(le).map((he) => ({ label: he, type: "constant" })))), {
        from: oe,
        to: de ? ee.pos + 1 : void 0,
        options: maybeQuoteCompletions(se, fe),
        validFor: se ? QuotedSpan : Span
      };
    };
  }
  function completionType(O) {
    return O == Type$2 ? "type" : O == Keyword ? "keyword" : "variable";
  }
  function completeKeywords(O, _, H) {
    let V = Object.keys(O).map((q) => H(_ ? q.toUpperCase() : q, completionType(O[q])));
    return ifNotIn(["QuotedIdentifier", "SpecialVar", "String", "LineComment", "BlockComment", "."], completeFromList(V));
  }
  let parser = /* @__PURE__ */ parser$1.configure({
    props: [
      /* @__PURE__ */ indentNodeProp.add({
        Statement: /* @__PURE__ */ continuedIndent()
      }),
      /* @__PURE__ */ foldNodeProp.add({
        Statement(O, _) {
          return { from: Math.min(O.from + 100, _.doc.lineAt(O.from).to), to: O.to };
        },
        BlockComment(O) {
          return { from: O.from + 2, to: O.to - 2 };
        }
      }),
      /* @__PURE__ */ styleTags({
        Keyword: tags.keyword,
        Type: tags.typeName,
        Builtin: /* @__PURE__ */ tags.standard(tags.name),
        Bits: tags.number,
        Bytes: tags.string,
        Bool: tags.bool,
        Null: tags.null,
        Number: tags.number,
        String: tags.string,
        Identifier: tags.name,
        QuotedIdentifier: /* @__PURE__ */ tags.special(tags.string),
        SpecialVar: /* @__PURE__ */ tags.special(tags.name),
        LineComment: tags.lineComment,
        BlockComment: tags.blockComment,
        Operator: tags.operator,
        "Semi Punctuation": tags.punctuation,
        "( )": tags.paren,
        "{ }": tags.brace,
        "[ ]": tags.squareBracket
      })
    ]
  });
  class SQLDialect {
    constructor(_, H, V) {
      this.dialect = _, this.language = H, this.spec = V;
    }
    /**
    Returns the language for this dialect as an extension.
    */
    get extension() {
      return this.language.extension;
    }
    /**
    Define a new dialect.
    */
    static define(_) {
      let H = dialect(_, _.keywords, _.types, _.builtin), V = LRLanguage.define({
        name: "sql",
        parser: parser.configure({
          tokenizers: [{ from: tokens, to: tokensFor(H) }]
        }),
        languageData: {
          commentTokens: { line: "--", block: { open: "/*", close: "*/" } },
          closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] }
        }
      });
      return new SQLDialect(H, V, _);
    }
  }
  function defaultKeyword(O, _) {
    return { label: O, type: _, boost: -1 };
  }
  function keywordCompletionSource(O, _ = !1, H) {
    return completeKeywords(O.dialect.words, _, H || defaultKeyword);
  }
  function schemaCompletionSource(O) {
    return O.schema ? completeFromSchema(O.schema, O.tables, O.schemas, O.defaultTable, O.defaultSchema, O.dialect || StandardSQL) : () => null;
  }
  function schemaCompletion(O) {
    return O.schema ? (O.dialect || StandardSQL).language.data.of({
      autocomplete: schemaCompletionSource(O)
    }) : [];
  }
  function sql(O = {}) {
    let _ = O.dialect || StandardSQL;
    return new LanguageSupport(_.language, [
      schemaCompletion(O),
      _.language.data.of({
        autocomplete: keywordCompletionSource(_, O.upperCaseKeywords, O.keywordCompletion)
      })
    ]);
  }
  const StandardSQL = /* @__PURE__ */ SQLDialect.define({}), toggleComment = (O) => {
    let { state: _ } = O, H = _.doc.lineAt(_.selection.main.from), V = getConfig(O.state, H.from);
    return V.line ? toggleLineComment(O) : V.block ? toggleBlockCommentByLine(O) : !1;
  };
  function command(O, _) {
    return ({ state: H, dispatch: V }) => {
      if (H.readOnly)
        return !1;
      let q = O(_, H);
      return q ? (V(H.update(q)), !0) : !1;
    };
  }
  const toggleLineComment = /* @__PURE__ */ command(
    changeLineComment,
    0
    /* CommentOption.Toggle */
  ), toggleBlockComment = /* @__PURE__ */ command(
    changeBlockComment,
    0
    /* CommentOption.Toggle */
  ), toggleBlockCommentByLine = /* @__PURE__ */ command(
    (O, _) => changeBlockComment(O, _, selectedLineRanges(_)),
    0
    /* CommentOption.Toggle */
  );
  function getConfig(O, _) {
    let H = O.languageDataAt("commentTokens", _);
    return H.length ? H[0] : {};
  }
  const SearchMargin = 50;
  function findBlockComment(O, { open: _, close: H }, V, q) {
    let G = O.sliceDoc(V - SearchMargin, V), J = O.sliceDoc(q, q + SearchMargin), Z = /\s*$/.exec(G)[0].length, te = /^\s*/.exec(J)[0].length, ne = G.length - Z;
    if (G.slice(ne - _.length, ne) == _ && J.slice(te, te + H.length) == H)
      return {
        open: { pos: V - Z, margin: Z && 1 },
        close: { pos: q + te, margin: te && 1 }
      };
    let ee, ae;
    q - V <= 2 * SearchMargin ? ee = ae = O.sliceDoc(V, q) : (ee = O.sliceDoc(V, V + SearchMargin), ae = O.sliceDoc(q - SearchMargin, q));
    let oe = /^\s*/.exec(ee)[0].length, se = /\s*$/.exec(ae)[0].length, ce = ae.length - se - H.length;
    return ee.slice(oe, oe + _.length) == _ && ae.slice(ce, ce + H.length) == H ? {
      open: {
        pos: V + oe + _.length,
        margin: /\s/.test(ee.charAt(oe + _.length)) ? 1 : 0
      },
      close: {
        pos: q - se - H.length,
        margin: /\s/.test(ae.charAt(ce - 1)) ? 1 : 0
      }
    } : null;
  }
  function selectedLineRanges(O) {
    let _ = [];
    for (let H of O.selection.ranges) {
      let V = O.doc.lineAt(H.from), q = H.to <= V.to ? V : O.doc.lineAt(H.to), G = _.length - 1;
      G >= 0 && _[G].to > V.from ? _[G].to = q.to : _.push({ from: V.from + /^\s*/.exec(V.text)[0].length, to: q.to });
    }
    return _;
  }
  function changeBlockComment(O, _, H = _.selection.ranges) {
    let V = H.map((G) => getConfig(_, G.from).block);
    if (!V.every((G) => G))
      return null;
    let q = H.map((G, J) => findBlockComment(_, V[J], G.from, G.to));
    if (O != 2 && !q.every((G) => G))
      return { changes: _.changes(H.map((G, J) => q[J] ? [] : [{ from: G.from, insert: V[J].open + " " }, { from: G.to, insert: " " + V[J].close }])) };
    if (O != 1 && q.some((G) => G)) {
      let G = [];
      for (let J = 0, Z; J < q.length; J++)
        if (Z = q[J]) {
          let te = V[J], { open: ne, close: ee } = Z;
          G.push({ from: ne.pos - te.open.length, to: ne.pos + ne.margin }, { from: ee.pos - ee.margin, to: ee.pos + te.close.length });
        }
      return { changes: G };
    }
    return null;
  }
  function changeLineComment(O, _, H = _.selection.ranges) {
    let V = [], q = -1;
    for (let { from: G, to: J } of H) {
      let Z = V.length, te = 1e9, ne = getConfig(_, G).line;
      if (ne) {
        for (let ee = G; ee <= J; ) {
          let ae = _.doc.lineAt(ee);
          if (ae.from > q && (G == J || J > ae.from)) {
            q = ae.from;
            let oe = /^\s*/.exec(ae.text)[0].length, se = oe == ae.length, ce = ae.text.slice(oe, oe + ne.length) == ne ? oe : -1;
            oe < ae.text.length && oe < te && (te = oe), V.push({ line: ae, comment: ce, token: ne, indent: oe, empty: se, single: !1 });
          }
          ee = ae.to + 1;
        }
        if (te < 1e9)
          for (let ee = Z; ee < V.length; ee++)
            V[ee].indent < V[ee].line.text.length && (V[ee].indent = te);
        V.length == Z + 1 && (V[Z].single = !0);
      }
    }
    if (O != 2 && V.some((G) => G.comment < 0 && (!G.empty || G.single))) {
      let G = [];
      for (let { line: Z, token: te, indent: ne, empty: ee, single: ae } of V)
        (ae || !ee) && G.push({ from: Z.from + ne, insert: te + " " });
      let J = _.changes(G);
      return { changes: J, selection: _.selection.map(J, 1) };
    } else if (O != 1 && V.some((G) => G.comment >= 0)) {
      let G = [];
      for (let { line: J, comment: Z, token: te } of V)
        if (Z >= 0) {
          let ne = J.from + Z, ee = ne + te.length;
          J.text[ee - J.from] == " " && ee++, G.push({ from: ne, to: ee });
        }
      return { changes: G };
    }
    return null;
  }
  const fromHistory = /* @__PURE__ */ Annotation.define(), isolateHistory = /* @__PURE__ */ Annotation.define(), invertedEffects = /* @__PURE__ */ Facet.define(), historyConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, {
        minDepth: 100,
        newGroupDelay: 500,
        joinToEvent: (_, H) => H
      }, {
        minDepth: Math.max,
        newGroupDelay: Math.min,
        joinToEvent: (_, H) => (V, q) => _(V, q) || H(V, q)
      });
    }
  }), historyField_ = /* @__PURE__ */ StateField.define({
    create() {
      return HistoryState.empty;
    },
    update(O, _) {
      let H = _.state.facet(historyConfig), V = _.annotation(fromHistory);
      if (V) {
        let te = HistEvent.fromTransaction(_, V.selection), ne = V.side, ee = ne == 0 ? O.undone : O.done;
        return te ? ee = updateBranch(ee, ee.length, H.minDepth, te) : ee = addSelection(ee, _.startState.selection), new HistoryState(ne == 0 ? V.rest : ee, ne == 0 ? ee : V.rest);
      }
      let q = _.annotation(isolateHistory);
      if ((q == "full" || q == "before") && (O = O.isolate()), _.annotation(Transaction.addToHistory) === !1)
        return _.changes.empty ? O : O.addMapping(_.changes.desc);
      let G = HistEvent.fromTransaction(_), J = _.annotation(Transaction.time), Z = _.annotation(Transaction.userEvent);
      return G ? O = O.addChanges(G, J, Z, H, _) : _.selection && (O = O.addSelection(_.startState.selection, J, Z, H.newGroupDelay)), (q == "full" || q == "after") && (O = O.isolate()), O;
    },
    toJSON(O) {
      return { done: O.done.map((_) => _.toJSON()), undone: O.undone.map((_) => _.toJSON()) };
    },
    fromJSON(O) {
      return new HistoryState(O.done.map(HistEvent.fromJSON), O.undone.map(HistEvent.fromJSON));
    }
  });
  function history(O = {}) {
    return [
      historyField_,
      historyConfig.of(O),
      EditorView.domEventHandlers({
        beforeinput(_, H) {
          let V = _.inputType == "historyUndo" ? undo : _.inputType == "historyRedo" ? redo : null;
          return V ? (_.preventDefault(), V(H)) : !1;
        }
      })
    ];
  }
  function cmd(O, _) {
    return function({ state: H, dispatch: V }) {
      if (!_ && H.readOnly)
        return !1;
      let q = H.field(historyField_, !1);
      if (!q)
        return !1;
      let G = q.pop(O, H, _);
      return G ? (V(G), !0) : !1;
    };
  }
  const undo = /* @__PURE__ */ cmd(0, !1), redo = /* @__PURE__ */ cmd(1, !1), undoSelection = /* @__PURE__ */ cmd(0, !0), redoSelection = /* @__PURE__ */ cmd(1, !0);
  class HistEvent {
    constructor(_, H, V, q, G) {
      this.changes = _, this.effects = H, this.mapped = V, this.startSelection = q, this.selectionsAfter = G;
    }
    setSelAfter(_) {
      return new HistEvent(this.changes, this.effects, this.mapped, this.startSelection, _);
    }
    toJSON() {
      var _, H, V;
      return {
        changes: (_ = this.changes) === null || _ === void 0 ? void 0 : _.toJSON(),
        mapped: (H = this.mapped) === null || H === void 0 ? void 0 : H.toJSON(),
        startSelection: (V = this.startSelection) === null || V === void 0 ? void 0 : V.toJSON(),
        selectionsAfter: this.selectionsAfter.map((q) => q.toJSON())
      };
    }
    static fromJSON(_) {
      return new HistEvent(_.changes && ChangeSet.fromJSON(_.changes), [], _.mapped && ChangeDesc.fromJSON(_.mapped), _.startSelection && EditorSelection.fromJSON(_.startSelection), _.selectionsAfter.map(EditorSelection.fromJSON));
    }
    // This does not check `addToHistory` and such, it assumes the
    // transaction needs to be converted to an item. Returns null when
    // there are no changes or effects in the transaction.
    static fromTransaction(_, H) {
      let V = none;
      for (let q of _.startState.facet(invertedEffects)) {
        let G = q(_);
        G.length && (V = V.concat(G));
      }
      return !V.length && _.changes.empty ? null : new HistEvent(_.changes.invert(_.startState.doc), V, void 0, H || _.startState.selection, none);
    }
    static selection(_) {
      return new HistEvent(void 0, none, void 0, void 0, _);
    }
  }
  function updateBranch(O, _, H, V) {
    let q = _ + 1 > H + 20 ? _ - H - 1 : 0, G = O.slice(q, _);
    return G.push(V), G;
  }
  function isAdjacent(O, _) {
    let H = [], V = !1;
    return O.iterChangedRanges((q, G) => H.push(q, G)), _.iterChangedRanges((q, G, J, Z) => {
      for (let te = 0; te < H.length; ) {
        let ne = H[te++], ee = H[te++];
        Z >= ne && J <= ee && (V = !0);
      }
    }), V;
  }
  function eqSelectionShape(O, _) {
    return O.ranges.length == _.ranges.length && O.ranges.filter((H, V) => H.empty != _.ranges[V].empty).length === 0;
  }
  function conc(O, _) {
    return O.length ? _.length ? O.concat(_) : O : _;
  }
  const none = [], MaxSelectionsPerEvent = 200;
  function addSelection(O, _) {
    if (O.length) {
      let H = O[O.length - 1], V = H.selectionsAfter.slice(Math.max(0, H.selectionsAfter.length - MaxSelectionsPerEvent));
      return V.length && V[V.length - 1].eq(_) ? O : (V.push(_), updateBranch(O, O.length - 1, 1e9, H.setSelAfter(V)));
    } else
      return [HistEvent.selection([_])];
  }
  function popSelection(O) {
    let _ = O[O.length - 1], H = O.slice();
    return H[O.length - 1] = _.setSelAfter(_.selectionsAfter.slice(0, _.selectionsAfter.length - 1)), H;
  }
  function addMappingToBranch(O, _) {
    if (!O.length)
      return O;
    let H = O.length, V = none;
    for (; H; ) {
      let q = mapEvent(O[H - 1], _, V);
      if (q.changes && !q.changes.empty || q.effects.length) {
        let G = O.slice(0, H);
        return G[H - 1] = q, G;
      } else
        _ = q.mapped, H--, V = q.selectionsAfter;
    }
    return V.length ? [HistEvent.selection(V)] : none;
  }
  function mapEvent(O, _, H) {
    let V = conc(O.selectionsAfter.length ? O.selectionsAfter.map((Z) => Z.map(_)) : none, H);
    if (!O.changes)
      return HistEvent.selection(V);
    let q = O.changes.map(_), G = _.mapDesc(O.changes, !0), J = O.mapped ? O.mapped.composeDesc(G) : G;
    return new HistEvent(q, StateEffect.mapEffects(O.effects, _), J, O.startSelection.map(G), V);
  }
  const joinableUserEvent = /^(input\.type|delete)($|\.)/;
  class HistoryState {
    constructor(_, H, V = 0, q = void 0) {
      this.done = _, this.undone = H, this.prevTime = V, this.prevUserEvent = q;
    }
    isolate() {
      return this.prevTime ? new HistoryState(this.done, this.undone) : this;
    }
    addChanges(_, H, V, q, G) {
      let J = this.done, Z = J[J.length - 1];
      return Z && Z.changes && !Z.changes.empty && _.changes && (!V || joinableUserEvent.test(V)) && (!Z.selectionsAfter.length && H - this.prevTime < q.newGroupDelay && q.joinToEvent(G, isAdjacent(Z.changes, _.changes)) || // For compose (but not compose.start) events, always join with previous event
      V == "input.type.compose") ? J = updateBranch(J, J.length - 1, q.minDepth, new HistEvent(_.changes.compose(Z.changes), conc(StateEffect.mapEffects(_.effects, Z.changes), Z.effects), Z.mapped, Z.startSelection, none)) : J = updateBranch(J, J.length, q.minDepth, _), new HistoryState(J, none, H, V);
    }
    addSelection(_, H, V, q) {
      let G = this.done.length ? this.done[this.done.length - 1].selectionsAfter : none;
      return G.length > 0 && H - this.prevTime < q && V == this.prevUserEvent && V && /^select($|\.)/.test(V) && eqSelectionShape(G[G.length - 1], _) ? this : new HistoryState(addSelection(this.done, _), this.undone, H, V);
    }
    addMapping(_) {
      return new HistoryState(addMappingToBranch(this.done, _), addMappingToBranch(this.undone, _), this.prevTime, this.prevUserEvent);
    }
    pop(_, H, V) {
      let q = _ == 0 ? this.done : this.undone;
      if (q.length == 0)
        return null;
      let G = q[q.length - 1], J = G.selectionsAfter[0] || H.selection;
      if (V && G.selectionsAfter.length)
        return H.update({
          selection: G.selectionsAfter[G.selectionsAfter.length - 1],
          annotations: fromHistory.of({ side: _, rest: popSelection(q), selection: J }),
          userEvent: _ == 0 ? "select.undo" : "select.redo",
          scrollIntoView: !0
        });
      if (G.changes) {
        let Z = q.length == 1 ? none : q.slice(0, q.length - 1);
        return G.mapped && (Z = addMappingToBranch(Z, G.mapped)), H.update({
          changes: G.changes,
          selection: G.startSelection,
          effects: G.effects,
          annotations: fromHistory.of({ side: _, rest: Z, selection: J }),
          filter: !1,
          userEvent: _ == 0 ? "undo" : "redo",
          scrollIntoView: !0
        });
      } else
        return null;
    }
  }
  HistoryState.empty = /* @__PURE__ */ new HistoryState(none, none);
  const historyKeymap = [
    { key: "Mod-z", run: undo, preventDefault: !0 },
    { key: "Mod-y", mac: "Mod-Shift-z", run: redo, preventDefault: !0 },
    { linux: "Ctrl-Shift-z", run: redo, preventDefault: !0 },
    { key: "Mod-u", run: undoSelection, preventDefault: !0 },
    { key: "Alt-u", mac: "Mod-Shift-u", run: redoSelection, preventDefault: !0 }
  ];
  function updateSel(O, _) {
    return EditorSelection.create(O.ranges.map(_), O.mainIndex);
  }
  function setSel(O, _) {
    return O.update({ selection: _, scrollIntoView: !0, userEvent: "select" });
  }
  function moveSel({ state: O, dispatch: _ }, H) {
    let V = updateSel(O.selection, H);
    return V.eq(O.selection, !0) ? !1 : (_(setSel(O, V)), !0);
  }
  function rangeEnd(O, _) {
    return EditorSelection.cursor(_ ? O.to : O.from);
  }
  function cursorByChar(O, _) {
    return moveSel(O, (H) => H.empty ? O.moveByChar(H, _) : rangeEnd(H, _));
  }
  function ltrAtCursor(O) {
    return O.textDirectionAt(O.state.selection.main.head) == Direction.LTR;
  }
  const cursorCharLeft = (O) => cursorByChar(O, !ltrAtCursor(O)), cursorCharRight = (O) => cursorByChar(O, ltrAtCursor(O));
  function cursorByGroup(O, _) {
    return moveSel(O, (H) => H.empty ? O.moveByGroup(H, _) : rangeEnd(H, _));
  }
  const cursorGroupLeft = (O) => cursorByGroup(O, !ltrAtCursor(O)), cursorGroupRight = (O) => cursorByGroup(O, ltrAtCursor(O));
  function interestingNode(O, _, H) {
    if (_.type.prop(H))
      return !0;
    let V = _.to - _.from;
    return V && (V > 2 || /[^\s,.;:]/.test(O.sliceDoc(_.from, _.to))) || _.firstChild;
  }
  function moveBySyntax(O, _, H) {
    let V = syntaxTree(O).resolveInner(_.head), q = H ? NodeProp.closedBy : NodeProp.openedBy;
    for (let te = _.head; ; ) {
      let ne = H ? V.childAfter(te) : V.childBefore(te);
      if (!ne)
        break;
      interestingNode(O, ne, q) ? V = ne : te = H ? ne.to : ne.from;
    }
    let G = V.type.prop(q), J, Z;
    return G && (J = H ? matchBrackets(O, V.from, 1) : matchBrackets(O, V.to, -1)) && J.matched ? Z = H ? J.end.to : J.end.from : Z = H ? V.to : V.from, EditorSelection.cursor(Z, H ? -1 : 1);
  }
  const cursorSyntaxLeft = (O) => moveSel(O, (_) => moveBySyntax(O.state, _, !ltrAtCursor(O))), cursorSyntaxRight = (O) => moveSel(O, (_) => moveBySyntax(O.state, _, ltrAtCursor(O)));
  function cursorByLine(O, _) {
    return moveSel(O, (H) => {
      if (!H.empty)
        return rangeEnd(H, _);
      let V = O.moveVertically(H, _);
      return V.head != H.head ? V : O.moveToLineBoundary(H, _);
    });
  }
  const cursorLineUp = (O) => cursorByLine(O, !1), cursorLineDown = (O) => cursorByLine(O, !0);
  function pageInfo(O) {
    let _ = O.scrollDOM.clientHeight < O.scrollDOM.scrollHeight - 2, H = 0, V = 0, q;
    if (_) {
      for (let G of O.state.facet(EditorView.scrollMargins)) {
        let J = G(O);
        J != null && J.top && (H = Math.max(J == null ? void 0 : J.top, H)), J != null && J.bottom && (V = Math.max(J == null ? void 0 : J.bottom, V));
      }
      q = O.scrollDOM.clientHeight - H - V;
    } else
      q = (O.dom.ownerDocument.defaultView || window).innerHeight;
    return {
      marginTop: H,
      marginBottom: V,
      selfScroll: _,
      height: Math.max(O.defaultLineHeight, q - 5)
    };
  }
  function cursorByPage(O, _) {
    let H = pageInfo(O), { state: V } = O, q = updateSel(V.selection, (J) => J.empty ? O.moveVertically(J, _, H.height) : rangeEnd(J, _));
    if (q.eq(V.selection))
      return !1;
    let G;
    if (H.selfScroll) {
      let J = O.coordsAtPos(V.selection.main.head), Z = O.scrollDOM.getBoundingClientRect(), te = Z.top + H.marginTop, ne = Z.bottom - H.marginBottom;
      J && J.top > te && J.bottom < ne && (G = EditorView.scrollIntoView(q.main.head, { y: "start", yMargin: J.top - te }));
    }
    return O.dispatch(setSel(V, q), { effects: G }), !0;
  }
  const cursorPageUp = (O) => cursorByPage(O, !1), cursorPageDown = (O) => cursorByPage(O, !0);
  function moveByLineBoundary(O, _, H) {
    let V = O.lineBlockAt(_.head), q = O.moveToLineBoundary(_, H);
    if (q.head == _.head && q.head != (H ? V.to : V.from) && (q = O.moveToLineBoundary(_, H, !1)), !H && q.head == V.from && V.length) {
      let G = /^\s*/.exec(O.state.sliceDoc(V.from, Math.min(V.from + 100, V.to)))[0].length;
      G && _.head != V.from + G && (q = EditorSelection.cursor(V.from + G));
    }
    return q;
  }
  const cursorLineBoundaryForward = (O) => moveSel(O, (_) => moveByLineBoundary(O, _, !0)), cursorLineBoundaryBackward = (O) => moveSel(O, (_) => moveByLineBoundary(O, _, !1)), cursorLineBoundaryLeft = (O) => moveSel(O, (_) => moveByLineBoundary(O, _, !ltrAtCursor(O))), cursorLineBoundaryRight = (O) => moveSel(O, (_) => moveByLineBoundary(O, _, ltrAtCursor(O))), cursorLineStart = (O) => moveSel(O, (_) => EditorSelection.cursor(O.lineBlockAt(_.head).from, 1)), cursorLineEnd = (O) => moveSel(O, (_) => EditorSelection.cursor(O.lineBlockAt(_.head).to, -1));
  function toMatchingBracket(O, _, H) {
    let V = !1, q = updateSel(O.selection, (G) => {
      let J = matchBrackets(O, G.head, -1) || matchBrackets(O, G.head, 1) || G.head > 0 && matchBrackets(O, G.head - 1, 1) || G.head < O.doc.length && matchBrackets(O, G.head + 1, -1);
      if (!J || !J.end)
        return G;
      V = !0;
      let Z = J.start.from == G.head ? J.end.to : J.end.from;
      return EditorSelection.cursor(Z);
    });
    return V ? (_(setSel(O, q)), !0) : !1;
  }
  const cursorMatchingBracket = ({ state: O, dispatch: _ }) => toMatchingBracket(O, _);
  function extendSel(O, _) {
    let H = updateSel(O.state.selection, (V) => {
      let q = _(V);
      return EditorSelection.range(V.anchor, q.head, q.goalColumn, q.bidiLevel || void 0);
    });
    return H.eq(O.state.selection) ? !1 : (O.dispatch(setSel(O.state, H)), !0);
  }
  function selectByChar(O, _) {
    return extendSel(O, (H) => O.moveByChar(H, _));
  }
  const selectCharLeft = (O) => selectByChar(O, !ltrAtCursor(O)), selectCharRight = (O) => selectByChar(O, ltrAtCursor(O));
  function selectByGroup(O, _) {
    return extendSel(O, (H) => O.moveByGroup(H, _));
  }
  const selectGroupLeft = (O) => selectByGroup(O, !ltrAtCursor(O)), selectGroupRight = (O) => selectByGroup(O, ltrAtCursor(O)), selectSyntaxLeft = (O) => extendSel(O, (_) => moveBySyntax(O.state, _, !ltrAtCursor(O))), selectSyntaxRight = (O) => extendSel(O, (_) => moveBySyntax(O.state, _, ltrAtCursor(O)));
  function selectByLine(O, _) {
    return extendSel(O, (H) => O.moveVertically(H, _));
  }
  const selectLineUp = (O) => selectByLine(O, !1), selectLineDown = (O) => selectByLine(O, !0);
  function selectByPage(O, _) {
    return extendSel(O, (H) => O.moveVertically(H, _, pageInfo(O).height));
  }
  const selectPageUp = (O) => selectByPage(O, !1), selectPageDown = (O) => selectByPage(O, !0), selectLineBoundaryForward = (O) => extendSel(O, (_) => moveByLineBoundary(O, _, !0)), selectLineBoundaryBackward = (O) => extendSel(O, (_) => moveByLineBoundary(O, _, !1)), selectLineBoundaryLeft = (O) => extendSel(O, (_) => moveByLineBoundary(O, _, !ltrAtCursor(O))), selectLineBoundaryRight = (O) => extendSel(O, (_) => moveByLineBoundary(O, _, ltrAtCursor(O))), selectLineStart = (O) => extendSel(O, (_) => EditorSelection.cursor(O.lineBlockAt(_.head).from)), selectLineEnd = (O) => extendSel(O, (_) => EditorSelection.cursor(O.lineBlockAt(_.head).to)), cursorDocStart = ({ state: O, dispatch: _ }) => (_(setSel(O, { anchor: 0 })), !0), cursorDocEnd = ({ state: O, dispatch: _ }) => (_(setSel(O, { anchor: O.doc.length })), !0), selectDocStart = ({ state: O, dispatch: _ }) => (_(setSel(O, { anchor: O.selection.main.anchor, head: 0 })), !0), selectDocEnd = ({ state: O, dispatch: _ }) => (_(setSel(O, { anchor: O.selection.main.anchor, head: O.doc.length })), !0), selectAll = ({ state: O, dispatch: _ }) => (_(O.update({ selection: { anchor: 0, head: O.doc.length }, userEvent: "select" })), !0), selectLine = ({ state: O, dispatch: _ }) => {
    let H = selectedLineBlocks(O).map(({ from: V, to: q }) => EditorSelection.range(V, Math.min(q + 1, O.doc.length)));
    return _(O.update({ selection: EditorSelection.create(H), userEvent: "select" })), !0;
  }, selectParentSyntax = ({ state: O, dispatch: _ }) => {
    let H = updateSel(O.selection, (V) => {
      let q = syntaxTree(O), G = q.resolveStack(V.from, 1);
      if (V.empty) {
        let J = q.resolveStack(V.from, -1);
        J.node.from >= G.node.from && J.node.to <= G.node.to && (G = J);
      }
      for (let J = G; J; J = J.next) {
        let { node: Z } = J;
        if ((Z.from < V.from && Z.to >= V.to || Z.to > V.to && Z.from <= V.from) && J.next)
          return EditorSelection.range(Z.to, Z.from);
      }
      return V;
    });
    return H.eq(O.selection) ? !1 : (_(setSel(O, H)), !0);
  }, simplifySelection = ({ state: O, dispatch: _ }) => {
    let H = O.selection, V = null;
    return H.ranges.length > 1 ? V = EditorSelection.create([H.main]) : H.main.empty || (V = EditorSelection.create([EditorSelection.cursor(H.main.head)])), V ? (_(setSel(O, V)), !0) : !1;
  };
  function deleteBy(O, _) {
    if (O.state.readOnly)
      return !1;
    let H = "delete.selection", { state: V } = O, q = V.changeByRange((G) => {
      let { from: J, to: Z } = G;
      if (J == Z) {
        let te = _(G);
        te < J ? (H = "delete.backward", te = skipAtomic(O, te, !1)) : te > J && (H = "delete.forward", te = skipAtomic(O, te, !0)), J = Math.min(J, te), Z = Math.max(Z, te);
      } else
        J = skipAtomic(O, J, !1), Z = skipAtomic(O, Z, !0);
      return J == Z ? { range: G } : { changes: { from: J, to: Z }, range: EditorSelection.cursor(J, J < G.head ? -1 : 1) };
    });
    return q.changes.empty ? !1 : (O.dispatch(V.update(q, {
      scrollIntoView: !0,
      userEvent: H,
      effects: H == "delete.selection" ? EditorView.announce.of(V.phrase("Selection deleted")) : void 0
    })), !0);
  }
  function skipAtomic(O, _, H) {
    if (O instanceof EditorView)
      for (let V of O.state.facet(EditorView.atomicRanges).map((q) => q(O)))
        V.between(_, _, (q, G) => {
          q < _ && G > _ && (_ = H ? G : q);
        });
    return _;
  }
  const deleteByChar = (O, _, H) => deleteBy(O, (V) => {
    let q = V.from, { state: G } = O, J = G.doc.lineAt(q), Z, te;
    if (H && !_ && q > J.from && q < J.from + 200 && !/[^ \t]/.test(Z = J.text.slice(0, q - J.from))) {
      if (Z[Z.length - 1] == "	")
        return q - 1;
      let ne = countColumn(Z, G.tabSize), ee = ne % getIndentUnit(G) || getIndentUnit(G);
      for (let ae = 0; ae < ee && Z[Z.length - 1 - ae] == " "; ae++)
        q--;
      te = q;
    } else
      te = findClusterBreak(J.text, q - J.from, _, _) + J.from, te == q && J.number != (_ ? G.doc.lines : 1) ? te += _ ? 1 : -1 : !_ && /[\ufe00-\ufe0f]/.test(J.text.slice(te - J.from, q - J.from)) && (te = findClusterBreak(J.text, te - J.from, !1, !1) + J.from);
    return te;
  }), deleteCharBackward = (O) => deleteByChar(O, !1, !0), deleteCharForward = (O) => deleteByChar(O, !0, !1), deleteByGroup = (O, _) => deleteBy(O, (H) => {
    let V = H.head, { state: q } = O, G = q.doc.lineAt(V), J = q.charCategorizer(V);
    for (let Z = null; ; ) {
      if (V == (_ ? G.to : G.from)) {
        V == H.head && G.number != (_ ? q.doc.lines : 1) && (V += _ ? 1 : -1);
        break;
      }
      let te = findClusterBreak(G.text, V - G.from, _) + G.from, ne = G.text.slice(Math.min(V, te) - G.from, Math.max(V, te) - G.from), ee = J(ne);
      if (Z != null && ee != Z)
        break;
      (ne != " " || V != H.head) && (Z = ee), V = te;
    }
    return V;
  }), deleteGroupBackward = (O) => deleteByGroup(O, !1), deleteGroupForward = (O) => deleteByGroup(O, !0), deleteToLineEnd = (O) => deleteBy(O, (_) => {
    let H = O.lineBlockAt(_.head).to;
    return _.head < H ? H : Math.min(O.state.doc.length, _.head + 1);
  }), deleteLineBoundaryBackward = (O) => deleteBy(O, (_) => {
    let H = O.moveToLineBoundary(_, !1).head;
    return _.head > H ? H : Math.max(0, _.head - 1);
  }), deleteLineBoundaryForward = (O) => deleteBy(O, (_) => {
    let H = O.moveToLineBoundary(_, !0).head;
    return _.head < H ? H : Math.min(O.state.doc.length, _.head + 1);
  }), splitLine = ({ state: O, dispatch: _ }) => {
    if (O.readOnly)
      return !1;
    let H = O.changeByRange((V) => ({
      changes: { from: V.from, to: V.to, insert: Text.of(["", ""]) },
      range: EditorSelection.cursor(V.from)
    }));
    return _(O.update(H, { scrollIntoView: !0, userEvent: "input" })), !0;
  }, transposeChars = ({ state: O, dispatch: _ }) => {
    if (O.readOnly)
      return !1;
    let H = O.changeByRange((V) => {
      if (!V.empty || V.from == 0 || V.from == O.doc.length)
        return { range: V };
      let q = V.from, G = O.doc.lineAt(q), J = q == G.from ? q - 1 : findClusterBreak(G.text, q - G.from, !1) + G.from, Z = q == G.to ? q + 1 : findClusterBreak(G.text, q - G.from, !0) + G.from;
      return {
        changes: { from: J, to: Z, insert: O.doc.slice(q, Z).append(O.doc.slice(J, q)) },
        range: EditorSelection.cursor(Z)
      };
    });
    return H.changes.empty ? !1 : (_(O.update(H, { scrollIntoView: !0, userEvent: "move.character" })), !0);
  };
  function selectedLineBlocks(O) {
    let _ = [], H = -1;
    for (let V of O.selection.ranges) {
      let q = O.doc.lineAt(V.from), G = O.doc.lineAt(V.to);
      if (!V.empty && V.to == G.from && (G = O.doc.lineAt(V.to - 1)), H >= q.number) {
        let J = _[_.length - 1];
        J.to = G.to, J.ranges.push(V);
      } else
        _.push({ from: q.from, to: G.to, ranges: [V] });
      H = G.number + 1;
    }
    return _;
  }
  function moveLine(O, _, H) {
    if (O.readOnly)
      return !1;
    let V = [], q = [];
    for (let G of selectedLineBlocks(O)) {
      if (H ? G.to == O.doc.length : G.from == 0)
        continue;
      let J = O.doc.lineAt(H ? G.to + 1 : G.from - 1), Z = J.length + 1;
      if (H) {
        V.push({ from: G.to, to: J.to }, { from: G.from, insert: J.text + O.lineBreak });
        for (let te of G.ranges)
          q.push(EditorSelection.range(Math.min(O.doc.length, te.anchor + Z), Math.min(O.doc.length, te.head + Z)));
      } else {
        V.push({ from: J.from, to: G.from }, { from: G.to, insert: O.lineBreak + J.text });
        for (let te of G.ranges)
          q.push(EditorSelection.range(te.anchor - Z, te.head - Z));
      }
    }
    return V.length ? (_(O.update({
      changes: V,
      scrollIntoView: !0,
      selection: EditorSelection.create(q, O.selection.mainIndex),
      userEvent: "move.line"
    })), !0) : !1;
  }
  const moveLineUp = ({ state: O, dispatch: _ }) => moveLine(O, _, !1), moveLineDown = ({ state: O, dispatch: _ }) => moveLine(O, _, !0);
  function copyLine(O, _, H) {
    if (O.readOnly)
      return !1;
    let V = [];
    for (let q of selectedLineBlocks(O))
      H ? V.push({ from: q.from, insert: O.doc.slice(q.from, q.to) + O.lineBreak }) : V.push({ from: q.to, insert: O.lineBreak + O.doc.slice(q.from, q.to) });
    return _(O.update({ changes: V, scrollIntoView: !0, userEvent: "input.copyline" })), !0;
  }
  const copyLineUp = ({ state: O, dispatch: _ }) => copyLine(O, _, !1), copyLineDown = ({ state: O, dispatch: _ }) => copyLine(O, _, !0), deleteLine = (O) => {
    if (O.state.readOnly)
      return !1;
    let { state: _ } = O, H = _.changes(selectedLineBlocks(_).map(({ from: q, to: G }) => (q > 0 ? q-- : G < _.doc.length && G++, { from: q, to: G }))), V = updateSel(_.selection, (q) => {
      let G;
      if (O.lineWrapping) {
        let J = O.lineBlockAt(q.head), Z = O.coordsAtPos(q.head, q.assoc || 1);
        Z && (G = J.bottom + O.documentTop - Z.bottom + O.defaultLineHeight / 2);
      }
      return O.moveVertically(q, !0, G);
    }).map(H);
    return O.dispatch({ changes: H, selection: V, scrollIntoView: !0, userEvent: "delete.line" }), !0;
  };
  function isBetweenBrackets(O, _) {
    if (/\(\)|\[\]|\{\}/.test(O.sliceDoc(_ - 1, _ + 1)))
      return { from: _, to: _ };
    let H = syntaxTree(O).resolveInner(_), V = H.childBefore(_), q = H.childAfter(_), G;
    return V && q && V.to <= _ && q.from >= _ && (G = V.type.prop(NodeProp.closedBy)) && G.indexOf(q.name) > -1 && O.doc.lineAt(V.to).from == O.doc.lineAt(q.from).from && !/\S/.test(O.sliceDoc(V.to, q.from)) ? { from: V.to, to: q.from } : null;
  }
  const insertNewlineAndIndent = /* @__PURE__ */ newlineAndIndent(!1), insertBlankLine = /* @__PURE__ */ newlineAndIndent(!0);
  function newlineAndIndent(O) {
    return ({ state: _, dispatch: H }) => {
      if (_.readOnly)
        return !1;
      let V = _.changeByRange((q) => {
        let { from: G, to: J } = q, Z = _.doc.lineAt(G), te = !O && G == J && isBetweenBrackets(_, G);
        O && (G = J = (J <= Z.to ? Z : _.doc.lineAt(J)).to);
        let ne = new IndentContext(_, { simulateBreak: G, simulateDoubleBreak: !!te }), ee = getIndentation(ne, G);
        for (ee == null && (ee = countColumn(/^\s*/.exec(_.doc.lineAt(G).text)[0], _.tabSize)); J < Z.to && /\s/.test(Z.text[J - Z.from]); )
          J++;
        te ? { from: G, to: J } = te : G > Z.from && G < Z.from + 100 && !/\S/.test(Z.text.slice(0, G)) && (G = Z.from);
        let ae = ["", indentString(_, ee)];
        return te && ae.push(indentString(_, ne.lineIndent(Z.from, -1))), {
          changes: { from: G, to: J, insert: Text.of(ae) },
          range: EditorSelection.cursor(G + 1 + ae[1].length)
        };
      });
      return H(_.update(V, { scrollIntoView: !0, userEvent: "input" })), !0;
    };
  }
  function changeBySelectedLine(O, _) {
    let H = -1;
    return O.changeByRange((V) => {
      let q = [];
      for (let J = V.from; J <= V.to; ) {
        let Z = O.doc.lineAt(J);
        Z.number > H && (V.empty || V.to > Z.from) && (_(Z, q, V), H = Z.number), J = Z.to + 1;
      }
      let G = O.changes(q);
      return {
        changes: q,
        range: EditorSelection.range(G.mapPos(V.anchor, 1), G.mapPos(V.head, 1))
      };
    });
  }
  const indentSelection = ({ state: O, dispatch: _ }) => {
    if (O.readOnly)
      return !1;
    let H = /* @__PURE__ */ Object.create(null), V = new IndentContext(O, { overrideIndentation: (G) => {
      let J = H[G];
      return J ?? -1;
    } }), q = changeBySelectedLine(O, (G, J, Z) => {
      let te = getIndentation(V, G.from);
      if (te == null)
        return;
      /\S/.test(G.text) || (te = 0);
      let ne = /^\s*/.exec(G.text)[0], ee = indentString(O, te);
      (ne != ee || Z.from < G.from + ne.length) && (H[G.from] = te, J.push({ from: G.from, to: G.from + ne.length, insert: ee }));
    });
    return q.changes.empty || _(O.update(q, { userEvent: "indent" })), !0;
  }, indentMore = ({ state: O, dispatch: _ }) => O.readOnly ? !1 : (_(O.update(changeBySelectedLine(O, (H, V) => {
    V.push({ from: H.from, insert: O.facet(indentUnit) });
  }), { userEvent: "input.indent" })), !0), indentLess = ({ state: O, dispatch: _ }) => O.readOnly ? !1 : (_(O.update(changeBySelectedLine(O, (H, V) => {
    let q = /^\s*/.exec(H.text)[0];
    if (!q)
      return;
    let G = countColumn(q, O.tabSize), J = 0, Z = indentString(O, Math.max(0, G - getIndentUnit(O)));
    for (; J < q.length && J < Z.length && q.charCodeAt(J) == Z.charCodeAt(J); )
      J++;
    V.push({ from: H.from + J, to: H.from + q.length, insert: Z.slice(J) });
  }), { userEvent: "delete.dedent" })), !0), toggleTabFocusMode = (O) => (O.setTabFocusMode(), !0), emacsStyleKeymap = [
    { key: "Ctrl-b", run: cursorCharLeft, shift: selectCharLeft, preventDefault: !0 },
    { key: "Ctrl-f", run: cursorCharRight, shift: selectCharRight },
    { key: "Ctrl-p", run: cursorLineUp, shift: selectLineUp },
    { key: "Ctrl-n", run: cursorLineDown, shift: selectLineDown },
    { key: "Ctrl-a", run: cursorLineStart, shift: selectLineStart },
    { key: "Ctrl-e", run: cursorLineEnd, shift: selectLineEnd },
    { key: "Ctrl-d", run: deleteCharForward },
    { key: "Ctrl-h", run: deleteCharBackward },
    { key: "Ctrl-k", run: deleteToLineEnd },
    { key: "Ctrl-Alt-h", run: deleteGroupBackward },
    { key: "Ctrl-o", run: splitLine },
    { key: "Ctrl-t", run: transposeChars },
    { key: "Ctrl-v", run: cursorPageDown }
  ], standardKeymap = /* @__PURE__ */ [
    { key: "ArrowLeft", run: cursorCharLeft, shift: selectCharLeft, preventDefault: !0 },
    { key: "Mod-ArrowLeft", mac: "Alt-ArrowLeft", run: cursorGroupLeft, shift: selectGroupLeft, preventDefault: !0 },
    { mac: "Cmd-ArrowLeft", run: cursorLineBoundaryLeft, shift: selectLineBoundaryLeft, preventDefault: !0 },
    { key: "ArrowRight", run: cursorCharRight, shift: selectCharRight, preventDefault: !0 },
    { key: "Mod-ArrowRight", mac: "Alt-ArrowRight", run: cursorGroupRight, shift: selectGroupRight, preventDefault: !0 },
    { mac: "Cmd-ArrowRight", run: cursorLineBoundaryRight, shift: selectLineBoundaryRight, preventDefault: !0 },
    { key: "ArrowUp", run: cursorLineUp, shift: selectLineUp, preventDefault: !0 },
    { mac: "Cmd-ArrowUp", run: cursorDocStart, shift: selectDocStart },
    { mac: "Ctrl-ArrowUp", run: cursorPageUp, shift: selectPageUp },
    { key: "ArrowDown", run: cursorLineDown, shift: selectLineDown, preventDefault: !0 },
    { mac: "Cmd-ArrowDown", run: cursorDocEnd, shift: selectDocEnd },
    { mac: "Ctrl-ArrowDown", run: cursorPageDown, shift: selectPageDown },
    { key: "PageUp", run: cursorPageUp, shift: selectPageUp },
    { key: "PageDown", run: cursorPageDown, shift: selectPageDown },
    { key: "Home", run: cursorLineBoundaryBackward, shift: selectLineBoundaryBackward, preventDefault: !0 },
    { key: "Mod-Home", run: cursorDocStart, shift: selectDocStart },
    { key: "End", run: cursorLineBoundaryForward, shift: selectLineBoundaryForward, preventDefault: !0 },
    { key: "Mod-End", run: cursorDocEnd, shift: selectDocEnd },
    { key: "Enter", run: insertNewlineAndIndent, shift: insertNewlineAndIndent },
    { key: "Mod-a", run: selectAll },
    { key: "Backspace", run: deleteCharBackward, shift: deleteCharBackward },
    { key: "Delete", run: deleteCharForward },
    { key: "Mod-Backspace", mac: "Alt-Backspace", run: deleteGroupBackward },
    { key: "Mod-Delete", mac: "Alt-Delete", run: deleteGroupForward },
    { mac: "Mod-Backspace", run: deleteLineBoundaryBackward },
    { mac: "Mod-Delete", run: deleteLineBoundaryForward }
  ].concat(/* @__PURE__ */ emacsStyleKeymap.map((O) => ({ mac: O.key, run: O.run, shift: O.shift }))), defaultKeymap = /* @__PURE__ */ [
    { key: "Alt-ArrowLeft", mac: "Ctrl-ArrowLeft", run: cursorSyntaxLeft, shift: selectSyntaxLeft },
    { key: "Alt-ArrowRight", mac: "Ctrl-ArrowRight", run: cursorSyntaxRight, shift: selectSyntaxRight },
    { key: "Alt-ArrowUp", run: moveLineUp },
    { key: "Shift-Alt-ArrowUp", run: copyLineUp },
    { key: "Alt-ArrowDown", run: moveLineDown },
    { key: "Shift-Alt-ArrowDown", run: copyLineDown },
    { key: "Escape", run: simplifySelection },
    { key: "Mod-Enter", run: insertBlankLine },
    { key: "Alt-l", mac: "Ctrl-l", run: selectLine },
    { key: "Mod-i", run: selectParentSyntax, preventDefault: !0 },
    { key: "Mod-[", run: indentLess },
    { key: "Mod-]", run: indentMore },
    { key: "Mod-Alt-\\", run: indentSelection },
    { key: "Shift-Mod-k", run: deleteLine },
    { key: "Shift-Mod-\\", run: cursorMatchingBracket },
    { key: "Mod-/", run: toggleComment },
    { key: "Alt-A", run: toggleBlockComment },
    { key: "Ctrl-m", mac: "Shift-Alt-m", run: toggleTabFocusMode }
  ].concat(standardKeymap);
  function crelt() {
    var O = arguments[0];
    typeof O == "string" && (O = document.createElement(O));
    var _ = 1, H = arguments[1];
    if (H && typeof H == "object" && H.nodeType == null && !Array.isArray(H)) {
      for (var V in H) if (Object.prototype.hasOwnProperty.call(H, V)) {
        var q = H[V];
        typeof q == "string" ? O.setAttribute(V, q) : q != null && (O[V] = q);
      }
      _++;
    }
    for (; _ < arguments.length; _++) add(O, arguments[_]);
    return O;
  }
  function add(O, _) {
    if (typeof _ == "string")
      O.appendChild(document.createTextNode(_));
    else if (_ != null) if (_.nodeType != null)
      O.appendChild(_);
    else if (Array.isArray(_))
      for (var H = 0; H < _.length; H++) add(O, _[H]);
    else
      throw new RangeError("Unsupported child node: " + _);
  }
  const basicNormalize = typeof String.prototype.normalize == "function" ? (O) => O.normalize("NFKD") : (O) => O;
  class SearchCursor {
    /**
    Create a text cursor. The query is the search string, `from` to
    `to` provides the region to search.
    
    When `normalize` is given, it will be called, on both the query
    string and the content it is matched against, before comparing.
    You can, for example, create a case-insensitive search by
    passing `s => s.toLowerCase()`.
    
    Text is always normalized with
    [`.normalize("NFKD")`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/normalize)
    (when supported).
    */
    constructor(_, H, V = 0, q = _.length, G, J) {
      this.test = J, this.value = { from: 0, to: 0 }, this.done = !1, this.matches = [], this.buffer = "", this.bufferPos = 0, this.iter = _.iterRange(V, q), this.bufferStart = V, this.normalize = G ? (Z) => G(basicNormalize(Z)) : basicNormalize, this.query = this.normalize(H);
    }
    peek() {
      if (this.bufferPos == this.buffer.length) {
        if (this.bufferStart += this.buffer.length, this.iter.next(), this.iter.done)
          return -1;
        this.bufferPos = 0, this.buffer = this.iter.value;
      }
      return codePointAt(this.buffer, this.bufferPos);
    }
    /**
    Look for the next match. Updates the iterator's
    [`value`](https://codemirror.net/6/docs/ref/#search.SearchCursor.value) and
    [`done`](https://codemirror.net/6/docs/ref/#search.SearchCursor.done) properties. Should be called
    at least once before using the cursor.
    */
    next() {
      for (; this.matches.length; )
        this.matches.pop();
      return this.nextOverlapping();
    }
    /**
    The `next` method will ignore matches that partially overlap a
    previous match. This method behaves like `next`, but includes
    such matches.
    */
    nextOverlapping() {
      for (; ; ) {
        let _ = this.peek();
        if (_ < 0)
          return this.done = !0, this;
        let H = fromCodePoint(_), V = this.bufferStart + this.bufferPos;
        this.bufferPos += codePointSize(_);
        let q = this.normalize(H);
        for (let G = 0, J = V; ; G++) {
          let Z = q.charCodeAt(G), te = this.match(Z, J, this.bufferPos + this.bufferStart);
          if (G == q.length - 1) {
            if (te)
              return this.value = te, this;
            break;
          }
          J == V && G < H.length && H.charCodeAt(G) == Z && J++;
        }
      }
    }
    match(_, H, V) {
      let q = null;
      for (let G = 0; G < this.matches.length; G += 2) {
        let J = this.matches[G], Z = !1;
        this.query.charCodeAt(J) == _ && (J == this.query.length - 1 ? q = { from: this.matches[G + 1], to: V } : (this.matches[G]++, Z = !0)), Z || (this.matches.splice(G, 2), G -= 2);
      }
      return this.query.charCodeAt(0) == _ && (this.query.length == 1 ? q = { from: H, to: V } : this.matches.push(1, H)), q && this.test && !this.test(q.from, q.to, this.buffer, this.bufferStart) && (q = null), q;
    }
  }
  typeof Symbol < "u" && (SearchCursor.prototype[Symbol.iterator] = function() {
    return this;
  });
  const empty = { from: -1, to: -1, match: /* @__PURE__ */ /.*/.exec("") }, baseFlags = "gm" + (/x/.unicode == null ? "" : "u");
  class RegExpCursor {
    /**
    Create a cursor that will search the given range in the given
    document. `query` should be the raw pattern (as you'd pass it to
    `new RegExp`).
    */
    constructor(_, H, V, q = 0, G = _.length) {
      if (this.text = _, this.to = G, this.curLine = "", this.done = !1, this.value = empty, /\\[sWDnr]|\n|\r|\[\^/.test(H))
        return new MultilineRegExpCursor(_, H, V, q, G);
      this.re = new RegExp(H, baseFlags + (V != null && V.ignoreCase ? "i" : "")), this.test = V == null ? void 0 : V.test, this.iter = _.iter();
      let J = _.lineAt(q);
      this.curLineStart = J.from, this.matchPos = toCharEnd(_, q), this.getLine(this.curLineStart);
    }
    getLine(_) {
      this.iter.next(_), this.iter.lineBreak ? this.curLine = "" : (this.curLine = this.iter.value, this.curLineStart + this.curLine.length > this.to && (this.curLine = this.curLine.slice(0, this.to - this.curLineStart)), this.iter.next());
    }
    nextLine() {
      this.curLineStart = this.curLineStart + this.curLine.length + 1, this.curLineStart > this.to ? this.curLine = "" : this.getLine(0);
    }
    /**
    Move to the next match, if there is one.
    */
    next() {
      for (let _ = this.matchPos - this.curLineStart; ; ) {
        this.re.lastIndex = _;
        let H = this.matchPos <= this.to && this.re.exec(this.curLine);
        if (H) {
          let V = this.curLineStart + H.index, q = V + H[0].length;
          if (this.matchPos = toCharEnd(this.text, q + (V == q ? 1 : 0)), V == this.curLineStart + this.curLine.length && this.nextLine(), (V < q || V > this.value.to) && (!this.test || this.test(V, q, H)))
            return this.value = { from: V, to: q, match: H }, this;
          _ = this.matchPos - this.curLineStart;
        } else if (this.curLineStart + this.curLine.length < this.to)
          this.nextLine(), _ = 0;
        else
          return this.done = !0, this;
      }
    }
  }
  const flattened = /* @__PURE__ */ new WeakMap();
  class FlattenedDoc {
    constructor(_, H) {
      this.from = _, this.text = H;
    }
    get to() {
      return this.from + this.text.length;
    }
    static get(_, H, V) {
      let q = flattened.get(_);
      if (!q || q.from >= V || q.to <= H) {
        let Z = new FlattenedDoc(H, _.sliceString(H, V));
        return flattened.set(_, Z), Z;
      }
      if (q.from == H && q.to == V)
        return q;
      let { text: G, from: J } = q;
      return J > H && (G = _.sliceString(H, J) + G, J = H), q.to < V && (G += _.sliceString(q.to, V)), flattened.set(_, new FlattenedDoc(J, G)), new FlattenedDoc(H, G.slice(H - J, V - J));
    }
  }
  class MultilineRegExpCursor {
    constructor(_, H, V, q, G) {
      this.text = _, this.to = G, this.done = !1, this.value = empty, this.matchPos = toCharEnd(_, q), this.re = new RegExp(H, baseFlags + (V != null && V.ignoreCase ? "i" : "")), this.test = V == null ? void 0 : V.test, this.flat = FlattenedDoc.get(_, q, this.chunkEnd(
        q + 5e3
        /* Chunk.Base */
      ));
    }
    chunkEnd(_) {
      return _ >= this.to ? this.to : this.text.lineAt(_).to;
    }
    next() {
      for (; ; ) {
        let _ = this.re.lastIndex = this.matchPos - this.flat.from, H = this.re.exec(this.flat.text);
        if (H && !H[0] && H.index == _ && (this.re.lastIndex = _ + 1, H = this.re.exec(this.flat.text)), H) {
          let V = this.flat.from + H.index, q = V + H[0].length;
          if ((this.flat.to >= this.to || H.index + H[0].length <= this.flat.text.length - 10) && (!this.test || this.test(V, q, H)))
            return this.value = { from: V, to: q, match: H }, this.matchPos = toCharEnd(this.text, q + (V == q ? 1 : 0)), this;
        }
        if (this.flat.to == this.to)
          return this.done = !0, this;
        this.flat = FlattenedDoc.get(this.text, this.flat.from, this.chunkEnd(this.flat.from + this.flat.text.length * 2));
      }
    }
  }
  typeof Symbol < "u" && (RegExpCursor.prototype[Symbol.iterator] = MultilineRegExpCursor.prototype[Symbol.iterator] = function() {
    return this;
  });
  function validRegExp(O) {
    try {
      return new RegExp(O, baseFlags), !0;
    } catch {
      return !1;
    }
  }
  function toCharEnd(O, _) {
    if (_ >= O.length)
      return _;
    let H = O.lineAt(_), V;
    for (; _ < H.to && (V = H.text.charCodeAt(_ - H.from)) >= 56320 && V < 57344; )
      _++;
    return _;
  }
  function createLineDialog(O) {
    let _ = String(O.state.doc.lineAt(O.state.selection.main.head).number), H = crelt("input", { class: "cm-textfield", name: "line", value: _ }), V = crelt("form", {
      class: "cm-gotoLine",
      onkeydown: (G) => {
        G.keyCode == 27 ? (G.preventDefault(), O.dispatch({ effects: dialogEffect.of(!1) }), O.focus()) : G.keyCode == 13 && (G.preventDefault(), q());
      },
      onsubmit: (G) => {
        G.preventDefault(), q();
      }
    }, crelt("label", O.state.phrase("Go to line"), ": ", H), " ", crelt("button", { class: "cm-button", type: "submit" }, O.state.phrase("go")));
    function q() {
      let G = /^([+-])?(\d+)?(:\d+)?(%)?$/.exec(H.value);
      if (!G)
        return;
      let { state: J } = O, Z = J.doc.lineAt(J.selection.main.head), [, te, ne, ee, ae] = G, oe = ee ? +ee.slice(1) : 0, se = ne ? +ne : Z.number;
      if (ne && ae) {
        let ue = se / 100;
        te && (ue = ue * (te == "-" ? -1 : 1) + Z.number / J.doc.lines), se = Math.round(J.doc.lines * ue);
      } else ne && te && (se = se * (te == "-" ? -1 : 1) + Z.number);
      let ce = J.doc.line(Math.max(1, Math.min(J.doc.lines, se))), le = EditorSelection.cursor(ce.from + Math.max(0, Math.min(oe, ce.length)));
      O.dispatch({
        effects: [dialogEffect.of(!1), EditorView.scrollIntoView(le.from, { y: "center" })],
        selection: le
      }), O.focus();
    }
    return { dom: V };
  }
  const dialogEffect = /* @__PURE__ */ StateEffect.define(), dialogField = /* @__PURE__ */ StateField.define({
    create() {
      return !0;
    },
    update(O, _) {
      for (let H of _.effects)
        H.is(dialogEffect) && (O = H.value);
      return O;
    },
    provide: (O) => showPanel.from(O, (_) => _ ? createLineDialog : null)
  }), gotoLine = (O) => {
    let _ = getPanel(O, createLineDialog);
    if (!_) {
      let H = [dialogEffect.of(!0)];
      O.state.field(dialogField, !1) == null && H.push(StateEffect.appendConfig.of([dialogField, baseTheme$1])), O.dispatch({ effects: H }), _ = getPanel(O, createLineDialog);
    }
    return _ && _.dom.querySelector("input").select(), !0;
  }, baseTheme$1 = /* @__PURE__ */ EditorView.baseTheme({
    ".cm-panel.cm-gotoLine": {
      padding: "2px 6px 4px",
      "& label": { fontSize: "80%" }
    }
  }), defaultHighlightOptions = {
    highlightWordAroundCursor: !1,
    minSelectionLength: 1,
    maxMatches: 100,
    wholeWords: !1
  }, highlightConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, defaultHighlightOptions, {
        highlightWordAroundCursor: (_, H) => _ || H,
        minSelectionLength: Math.min,
        maxMatches: Math.min
      });
    }
  });
  function highlightSelectionMatches(O) {
    return [defaultTheme, matchHighlighter];
  }
  const matchDeco = /* @__PURE__ */ Decoration.mark({ class: "cm-selectionMatch" }), mainMatchDeco = /* @__PURE__ */ Decoration.mark({ class: "cm-selectionMatch cm-selectionMatch-main" });
  function insideWordBoundaries(O, _, H, V) {
    return (H == 0 || O(_.sliceDoc(H - 1, H)) != CharCategory.Word) && (V == _.doc.length || O(_.sliceDoc(V, V + 1)) != CharCategory.Word);
  }
  function insideWord(O, _, H, V) {
    return O(_.sliceDoc(H, H + 1)) == CharCategory.Word && O(_.sliceDoc(V - 1, V)) == CharCategory.Word;
  }
  const matchHighlighter = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.decorations = this.getDeco(O);
    }
    update(O) {
      (O.selectionSet || O.docChanged || O.viewportChanged) && (this.decorations = this.getDeco(O.view));
    }
    getDeco(O) {
      let _ = O.state.facet(highlightConfig), { state: H } = O, V = H.selection;
      if (V.ranges.length > 1)
        return Decoration.none;
      let q = V.main, G, J = null;
      if (q.empty) {
        if (!_.highlightWordAroundCursor)
          return Decoration.none;
        let te = H.wordAt(q.head);
        if (!te)
          return Decoration.none;
        J = H.charCategorizer(q.head), G = H.sliceDoc(te.from, te.to);
      } else {
        let te = q.to - q.from;
        if (te < _.minSelectionLength || te > 200)
          return Decoration.none;
        if (_.wholeWords) {
          if (G = H.sliceDoc(q.from, q.to), J = H.charCategorizer(q.head), !(insideWordBoundaries(J, H, q.from, q.to) && insideWord(J, H, q.from, q.to)))
            return Decoration.none;
        } else if (G = H.sliceDoc(q.from, q.to), !G)
          return Decoration.none;
      }
      let Z = [];
      for (let te of O.visibleRanges) {
        let ne = new SearchCursor(H.doc, G, te.from, te.to);
        for (; !ne.next().done; ) {
          let { from: ee, to: ae } = ne.value;
          if ((!J || insideWordBoundaries(J, H, ee, ae)) && (q.empty && ee <= q.from && ae >= q.to ? Z.push(mainMatchDeco.range(ee, ae)) : (ee >= q.to || ae <= q.from) && Z.push(matchDeco.range(ee, ae)), Z.length > _.maxMatches))
            return Decoration.none;
        }
      }
      return Decoration.set(Z);
    }
  }, {
    decorations: (O) => O.decorations
  }), defaultTheme = /* @__PURE__ */ EditorView.baseTheme({
    ".cm-selectionMatch": { backgroundColor: "#99ff7780" },
    ".cm-searchMatch .cm-selectionMatch": { backgroundColor: "transparent" }
  }), selectWord = ({ state: O, dispatch: _ }) => {
    let { selection: H } = O, V = EditorSelection.create(H.ranges.map((q) => O.wordAt(q.head) || EditorSelection.cursor(q.head)), H.mainIndex);
    return V.eq(H) ? !1 : (_(O.update({ selection: V })), !0);
  };
  function findNextOccurrence(O, _) {
    let { main: H, ranges: V } = O.selection, q = O.wordAt(H.head), G = q && q.from == H.from && q.to == H.to;
    for (let J = !1, Z = new SearchCursor(O.doc, _, V[V.length - 1].to); ; )
      if (Z.next(), Z.done) {
        if (J)
          return null;
        Z = new SearchCursor(O.doc, _, 0, Math.max(0, V[V.length - 1].from - 1)), J = !0;
      } else {
        if (J && V.some((te) => te.from == Z.value.from))
          continue;
        if (G) {
          let te = O.wordAt(Z.value.from);
          if (!te || te.from != Z.value.from || te.to != Z.value.to)
            continue;
        }
        return Z.value;
      }
  }
  const selectNextOccurrence = ({ state: O, dispatch: _ }) => {
    let { ranges: H } = O.selection;
    if (H.some((G) => G.from === G.to))
      return selectWord({ state: O, dispatch: _ });
    let V = O.sliceDoc(H[0].from, H[0].to);
    if (O.selection.ranges.some((G) => O.sliceDoc(G.from, G.to) != V))
      return !1;
    let q = findNextOccurrence(O, V);
    return q ? (_(O.update({
      selection: O.selection.addRange(EditorSelection.range(q.from, q.to), !1),
      effects: EditorView.scrollIntoView(q.to)
    })), !0) : !1;
  }, searchConfigFacet = /* @__PURE__ */ Facet.define({
    combine(O) {
      return combineConfig(O, {
        top: !1,
        caseSensitive: !1,
        literal: !1,
        regexp: !1,
        wholeWord: !1,
        createPanel: (_) => new SearchPanel(_),
        scrollToMatch: (_) => EditorView.scrollIntoView(_)
      });
    }
  });
  class SearchQuery {
    /**
    Create a query object.
    */
    constructor(_) {
      this.search = _.search, this.caseSensitive = !!_.caseSensitive, this.literal = !!_.literal, this.regexp = !!_.regexp, this.replace = _.replace || "", this.valid = !!this.search && (!this.regexp || validRegExp(this.search)), this.unquoted = this.unquote(this.search), this.wholeWord = !!_.wholeWord;
    }
    /**
    @internal
    */
    unquote(_) {
      return this.literal ? _ : _.replace(/\\([nrt\\])/g, (H, V) => V == "n" ? `
` : V == "r" ? "\r" : V == "t" ? "	" : "\\");
    }
    /**
    Compare this query to another query.
    */
    eq(_) {
      return this.search == _.search && this.replace == _.replace && this.caseSensitive == _.caseSensitive && this.regexp == _.regexp && this.wholeWord == _.wholeWord;
    }
    /**
    @internal
    */
    create() {
      return this.regexp ? new RegExpQuery(this) : new StringQuery(this);
    }
    /**
    Get a search cursor for this query, searching through the given
    range in the given state.
    */
    getCursor(_, H = 0, V) {
      let q = _.doc ? _ : EditorState.create({ doc: _ });
      return V == null && (V = q.doc.length), this.regexp ? regexpCursor(this, q, H, V) : stringCursor(this, q, H, V);
    }
  }
  class QueryType {
    constructor(_) {
      this.spec = _;
    }
  }
  function stringCursor(O, _, H, V) {
    return new SearchCursor(_.doc, O.unquoted, H, V, O.caseSensitive ? void 0 : (q) => q.toLowerCase(), O.wholeWord ? stringWordTest(_.doc, _.charCategorizer(_.selection.main.head)) : void 0);
  }
  function stringWordTest(O, _) {
    return (H, V, q, G) => ((G > H || G + q.length < V) && (G = Math.max(0, H - 2), q = O.sliceString(G, Math.min(O.length, V + 2))), (_(charBefore(q, H - G)) != CharCategory.Word || _(charAfter(q, H - G)) != CharCategory.Word) && (_(charAfter(q, V - G)) != CharCategory.Word || _(charBefore(q, V - G)) != CharCategory.Word));
  }
  class StringQuery extends QueryType {
    constructor(_) {
      super(_);
    }
    nextMatch(_, H, V) {
      let q = stringCursor(this.spec, _, V, _.doc.length).nextOverlapping();
      return q.done && (q = stringCursor(this.spec, _, 0, H).nextOverlapping()), q.done ? null : q.value;
    }
    // Searching in reverse is, rather than implementing an inverted search
    // cursor, done by scanning chunk after chunk forward.
    prevMatchInRange(_, H, V) {
      for (let q = V; ; ) {
        let G = Math.max(H, q - 1e4 - this.spec.unquoted.length), J = stringCursor(this.spec, _, G, q), Z = null;
        for (; !J.nextOverlapping().done; )
          Z = J.value;
        if (Z)
          return Z;
        if (G == H)
          return null;
        q -= 1e4;
      }
    }
    prevMatch(_, H, V) {
      return this.prevMatchInRange(_, 0, H) || this.prevMatchInRange(_, V, _.doc.length);
    }
    getReplacement(_) {
      return this.spec.unquote(this.spec.replace);
    }
    matchAll(_, H) {
      let V = stringCursor(this.spec, _, 0, _.doc.length), q = [];
      for (; !V.next().done; ) {
        if (q.length >= H)
          return null;
        q.push(V.value);
      }
      return q;
    }
    highlight(_, H, V, q) {
      let G = stringCursor(this.spec, _, Math.max(0, H - this.spec.unquoted.length), Math.min(V + this.spec.unquoted.length, _.doc.length));
      for (; !G.next().done; )
        q(G.value.from, G.value.to);
    }
  }
  function regexpCursor(O, _, H, V) {
    return new RegExpCursor(_.doc, O.search, {
      ignoreCase: !O.caseSensitive,
      test: O.wholeWord ? regexpWordTest(_.charCategorizer(_.selection.main.head)) : void 0
    }, H, V);
  }
  function charBefore(O, _) {
    return O.slice(findClusterBreak(O, _, !1), _);
  }
  function charAfter(O, _) {
    return O.slice(_, findClusterBreak(O, _));
  }
  function regexpWordTest(O) {
    return (_, H, V) => !V[0].length || (O(charBefore(V.input, V.index)) != CharCategory.Word || O(charAfter(V.input, V.index)) != CharCategory.Word) && (O(charAfter(V.input, V.index + V[0].length)) != CharCategory.Word || O(charBefore(V.input, V.index + V[0].length)) != CharCategory.Word);
  }
  class RegExpQuery extends QueryType {
    nextMatch(_, H, V) {
      let q = regexpCursor(this.spec, _, V, _.doc.length).next();
      return q.done && (q = regexpCursor(this.spec, _, 0, H).next()), q.done ? null : q.value;
    }
    prevMatchInRange(_, H, V) {
      for (let q = 1; ; q++) {
        let G = Math.max(
          H,
          V - q * 1e4
          /* FindPrev.ChunkSize */
        ), J = regexpCursor(this.spec, _, G, V), Z = null;
        for (; !J.next().done; )
          Z = J.value;
        if (Z && (G == H || Z.from > G + 10))
          return Z;
        if (G == H)
          return null;
      }
    }
    prevMatch(_, H, V) {
      return this.prevMatchInRange(_, 0, H) || this.prevMatchInRange(_, V, _.doc.length);
    }
    getReplacement(_) {
      return this.spec.unquote(this.spec.replace).replace(/\$([$&\d+])/g, (H, V) => V == "$" ? "$" : V == "&" ? _.match[0] : V != "0" && +V < _.match.length ? _.match[V] : H);
    }
    matchAll(_, H) {
      let V = regexpCursor(this.spec, _, 0, _.doc.length), q = [];
      for (; !V.next().done; ) {
        if (q.length >= H)
          return null;
        q.push(V.value);
      }
      return q;
    }
    highlight(_, H, V, q) {
      let G = regexpCursor(this.spec, _, Math.max(
        0,
        H - 250
        /* RegExp.HighlightMargin */
      ), Math.min(V + 250, _.doc.length));
      for (; !G.next().done; )
        q(G.value.from, G.value.to);
    }
  }
  const setSearchQuery = /* @__PURE__ */ StateEffect.define(), togglePanel$1 = /* @__PURE__ */ StateEffect.define(), searchState = /* @__PURE__ */ StateField.define({
    create(O) {
      return new SearchState(defaultQuery(O).create(), null);
    },
    update(O, _) {
      for (let H of _.effects)
        H.is(setSearchQuery) ? O = new SearchState(H.value.create(), O.panel) : H.is(togglePanel$1) && (O = new SearchState(O.query, H.value ? createSearchPanel : null));
      return O;
    },
    provide: (O) => showPanel.from(O, (_) => _.panel)
  });
  class SearchState {
    constructor(_, H) {
      this.query = _, this.panel = H;
    }
  }
  const matchMark = /* @__PURE__ */ Decoration.mark({ class: "cm-searchMatch" }), selectedMatchMark = /* @__PURE__ */ Decoration.mark({ class: "cm-searchMatch cm-searchMatch-selected" }), searchHighlighter = /* @__PURE__ */ ViewPlugin.fromClass(class {
    constructor(O) {
      this.view = O, this.decorations = this.highlight(O.state.field(searchState));
    }
    update(O) {
      let _ = O.state.field(searchState);
      (_ != O.startState.field(searchState) || O.docChanged || O.selectionSet || O.viewportChanged) && (this.decorations = this.highlight(_));
    }
    highlight({ query: O, panel: _ }) {
      if (!_ || !O.spec.valid)
        return Decoration.none;
      let { view: H } = this, V = new RangeSetBuilder();
      for (let q = 0, G = H.visibleRanges, J = G.length; q < J; q++) {
        let { from: Z, to: te } = G[q];
        for (; q < J - 1 && te > G[q + 1].from - 2 * 250; )
          te = G[++q].to;
        O.highlight(H.state, Z, te, (ne, ee) => {
          let ae = H.state.selection.ranges.some((oe) => oe.from == ne && oe.to == ee);
          V.add(ne, ee, ae ? selectedMatchMark : matchMark);
        });
      }
      return V.finish();
    }
  }, {
    decorations: (O) => O.decorations
  });
  function searchCommand(O) {
    return (_) => {
      let H = _.state.field(searchState, !1);
      return H && H.query.spec.valid ? O(_, H) : openSearchPanel(_);
    };
  }
  const findNext = /* @__PURE__ */ searchCommand((O, { query: _ }) => {
    let { to: H } = O.state.selection.main, V = _.nextMatch(O.state, H, H);
    if (!V)
      return !1;
    let q = EditorSelection.single(V.from, V.to), G = O.state.facet(searchConfigFacet);
    return O.dispatch({
      selection: q,
      effects: [announceMatch(O, V), G.scrollToMatch(q.main, O)],
      userEvent: "select.search"
    }), selectSearchInput(O), !0;
  }), findPrevious = /* @__PURE__ */ searchCommand((O, { query: _ }) => {
    let { state: H } = O, { from: V } = H.selection.main, q = _.prevMatch(H, V, V);
    if (!q)
      return !1;
    let G = EditorSelection.single(q.from, q.to), J = O.state.facet(searchConfigFacet);
    return O.dispatch({
      selection: G,
      effects: [announceMatch(O, q), J.scrollToMatch(G.main, O)],
      userEvent: "select.search"
    }), selectSearchInput(O), !0;
  }), selectMatches = /* @__PURE__ */ searchCommand((O, { query: _ }) => {
    let H = _.matchAll(O.state, 1e3);
    return !H || !H.length ? !1 : (O.dispatch({
      selection: EditorSelection.create(H.map((V) => EditorSelection.range(V.from, V.to))),
      userEvent: "select.search.matches"
    }), !0);
  }), selectSelectionMatches = ({ state: O, dispatch: _ }) => {
    let H = O.selection;
    if (H.ranges.length > 1 || H.main.empty)
      return !1;
    let { from: V, to: q } = H.main, G = [], J = 0;
    for (let Z = new SearchCursor(O.doc, O.sliceDoc(V, q)); !Z.next().done; ) {
      if (G.length > 1e3)
        return !1;
      Z.value.from == V && (J = G.length), G.push(EditorSelection.range(Z.value.from, Z.value.to));
    }
    return _(O.update({
      selection: EditorSelection.create(G, J),
      userEvent: "select.search.matches"
    })), !0;
  }, replaceNext = /* @__PURE__ */ searchCommand((O, { query: _ }) => {
    let { state: H } = O, { from: V, to: q } = H.selection.main;
    if (H.readOnly)
      return !1;
    let G = _.nextMatch(H, V, V);
    if (!G)
      return !1;
    let J = [], Z, te, ne = [];
    if (G.from == V && G.to == q && (te = H.toText(_.getReplacement(G)), J.push({ from: G.from, to: G.to, insert: te }), G = _.nextMatch(H, G.from, G.to), ne.push(EditorView.announce.of(H.phrase("replaced match on line $", H.doc.lineAt(V).number) + "."))), G) {
      let ee = J.length == 0 || J[0].from >= G.to ? 0 : G.to - G.from - te.length;
      Z = EditorSelection.single(G.from - ee, G.to - ee), ne.push(announceMatch(O, G)), ne.push(H.facet(searchConfigFacet).scrollToMatch(Z.main, O));
    }
    return O.dispatch({
      changes: J,
      selection: Z,
      effects: ne,
      userEvent: "input.replace"
    }), !0;
  }), replaceAll = /* @__PURE__ */ searchCommand((O, { query: _ }) => {
    if (O.state.readOnly)
      return !1;
    let H = _.matchAll(O.state, 1e9).map((q) => {
      let { from: G, to: J } = q;
      return { from: G, to: J, insert: _.getReplacement(q) };
    });
    if (!H.length)
      return !1;
    let V = O.state.phrase("replaced $ matches", H.length) + ".";
    return O.dispatch({
      changes: H,
      effects: EditorView.announce.of(V),
      userEvent: "input.replace.all"
    }), !0;
  });
  function createSearchPanel(O) {
    return O.state.facet(searchConfigFacet).createPanel(O);
  }
  function defaultQuery(O, _) {
    var H, V, q, G, J;
    let Z = O.selection.main, te = Z.empty || Z.to > Z.from + 100 ? "" : O.sliceDoc(Z.from, Z.to);
    if (_ && !te)
      return _;
    let ne = O.facet(searchConfigFacet);
    return new SearchQuery({
      search: ((H = _ == null ? void 0 : _.literal) !== null && H !== void 0 ? H : ne.literal) ? te : te.replace(/\n/g, "\\n"),
      caseSensitive: (V = _ == null ? void 0 : _.caseSensitive) !== null && V !== void 0 ? V : ne.caseSensitive,
      literal: (q = _ == null ? void 0 : _.literal) !== null && q !== void 0 ? q : ne.literal,
      regexp: (G = _ == null ? void 0 : _.regexp) !== null && G !== void 0 ? G : ne.regexp,
      wholeWord: (J = _ == null ? void 0 : _.wholeWord) !== null && J !== void 0 ? J : ne.wholeWord
    });
  }
  function getSearchInput(O) {
    let _ = getPanel(O, createSearchPanel);
    return _ && _.dom.querySelector("[main-field]");
  }
  function selectSearchInput(O) {
    let _ = getSearchInput(O);
    _ && _ == O.root.activeElement && _.select();
  }
  const openSearchPanel = (O) => {
    let _ = O.state.field(searchState, !1);
    if (_ && _.panel) {
      let H = getSearchInput(O);
      if (H && H != O.root.activeElement) {
        let V = defaultQuery(O.state, _.query.spec);
        V.valid && O.dispatch({ effects: setSearchQuery.of(V) }), H.focus(), H.select();
      }
    } else
      O.dispatch({ effects: [
        togglePanel$1.of(!0),
        _ ? setSearchQuery.of(defaultQuery(O.state, _.query.spec)) : StateEffect.appendConfig.of(searchExtensions)
      ] });
    return !0;
  }, closeSearchPanel = (O) => {
    let _ = O.state.field(searchState, !1);
    if (!_ || !_.panel)
      return !1;
    let H = getPanel(O, createSearchPanel);
    return H && H.dom.contains(O.root.activeElement) && O.focus(), O.dispatch({ effects: togglePanel$1.of(!1) }), !0;
  }, searchKeymap = [
    { key: "Mod-f", run: openSearchPanel, scope: "editor search-panel" },
    { key: "F3", run: findNext, shift: findPrevious, scope: "editor search-panel", preventDefault: !0 },
    { key: "Mod-g", run: findNext, shift: findPrevious, scope: "editor search-panel", preventDefault: !0 },
    { key: "Escape", run: closeSearchPanel, scope: "editor search-panel" },
    { key: "Mod-Shift-l", run: selectSelectionMatches },
    { key: "Mod-Alt-g", run: gotoLine },
    { key: "Mod-d", run: selectNextOccurrence, preventDefault: !0 }
  ];
  class SearchPanel {
    constructor(_) {
      this.view = _;
      let H = this.query = _.state.field(searchState).query.spec;
      this.commit = this.commit.bind(this), this.searchField = crelt("input", {
        value: H.search,
        placeholder: phrase(_, "Find"),
        "aria-label": phrase(_, "Find"),
        class: "cm-textfield",
        name: "search",
        form: "",
        "main-field": "true",
        onchange: this.commit,
        onkeyup: this.commit
      }), this.replaceField = crelt("input", {
        value: H.replace,
        placeholder: phrase(_, "Replace"),
        "aria-label": phrase(_, "Replace"),
        class: "cm-textfield",
        name: "replace",
        form: "",
        onchange: this.commit,
        onkeyup: this.commit
      }), this.caseField = crelt("input", {
        type: "checkbox",
        name: "case",
        form: "",
        checked: H.caseSensitive,
        onchange: this.commit
      }), this.reField = crelt("input", {
        type: "checkbox",
        name: "re",
        form: "",
        checked: H.regexp,
        onchange: this.commit
      }), this.wordField = crelt("input", {
        type: "checkbox",
        name: "word",
        form: "",
        checked: H.wholeWord,
        onchange: this.commit
      });
      function V(q, G, J) {
        return crelt("button", { class: "cm-button", name: q, onclick: G, type: "button" }, J);
      }
      this.dom = crelt("div", { onkeydown: (q) => this.keydown(q), class: "cm-search" }, [
        this.searchField,
        V("next", () => findNext(_), [phrase(_, "next")]),
        V("prev", () => findPrevious(_), [phrase(_, "previous")]),
        V("select", () => selectMatches(_), [phrase(_, "all")]),
        crelt("label", null, [this.caseField, phrase(_, "match case")]),
        crelt("label", null, [this.reField, phrase(_, "regexp")]),
        crelt("label", null, [this.wordField, phrase(_, "by word")]),
        ..._.state.readOnly ? [] : [
          crelt("br"),
          this.replaceField,
          V("replace", () => replaceNext(_), [phrase(_, "replace")]),
          V("replaceAll", () => replaceAll(_), [phrase(_, "replace all")])
        ],
        crelt("button", {
          name: "close",
          onclick: () => closeSearchPanel(_),
          "aria-label": phrase(_, "close"),
          type: "button"
        }, ["×"])
      ]);
    }
    commit() {
      let _ = new SearchQuery({
        search: this.searchField.value,
        caseSensitive: this.caseField.checked,
        regexp: this.reField.checked,
        wholeWord: this.wordField.checked,
        replace: this.replaceField.value
      });
      _.eq(this.query) || (this.query = _, this.view.dispatch({ effects: setSearchQuery.of(_) }));
    }
    keydown(_) {
      runScopeHandlers(this.view, _, "search-panel") ? _.preventDefault() : _.keyCode == 13 && _.target == this.searchField ? (_.preventDefault(), (_.shiftKey ? findPrevious : findNext)(this.view)) : _.keyCode == 13 && _.target == this.replaceField && (_.preventDefault(), replaceNext(this.view));
    }
    update(_) {
      for (let H of _.transactions)
        for (let V of H.effects)
          V.is(setSearchQuery) && !V.value.eq(this.query) && this.setQuery(V.value);
    }
    setQuery(_) {
      this.query = _, this.searchField.value = _.search, this.replaceField.value = _.replace, this.caseField.checked = _.caseSensitive, this.reField.checked = _.regexp, this.wordField.checked = _.wholeWord;
    }
    mount() {
      this.searchField.select();
    }
    get pos() {
      return 80;
    }
    get top() {
      return this.view.state.facet(searchConfigFacet).top;
    }
  }
  function phrase(O, _) {
    return O.state.phrase(_);
  }
  const AnnounceMargin = 30, Break = /[\s\.,:;?!]/;
  function announceMatch(O, { from: _, to: H }) {
    let V = O.state.doc.lineAt(_), q = O.state.doc.lineAt(H).to, G = Math.max(V.from, _ - AnnounceMargin), J = Math.min(q, H + AnnounceMargin), Z = O.state.sliceDoc(G, J);
    if (G != V.from) {
      for (let te = 0; te < AnnounceMargin; te++)
        if (!Break.test(Z[te + 1]) && Break.test(Z[te])) {
          Z = Z.slice(te);
          break;
        }
    }
    if (J != q) {
      for (let te = Z.length - 1; te > Z.length - AnnounceMargin; te--)
        if (!Break.test(Z[te - 1]) && Break.test(Z[te])) {
          Z = Z.slice(0, te);
          break;
        }
    }
    return EditorView.announce.of(`${O.state.phrase("current match")}. ${Z} ${O.state.phrase("on line")} ${V.number}.`);
  }
  const baseTheme$2 = /* @__PURE__ */ EditorView.baseTheme({
    ".cm-panel.cm-search": {
      padding: "2px 6px 4px",
      position: "relative",
      "& [name=close]": {
        position: "absolute",
        top: "0",
        right: "4px",
        backgroundColor: "inherit",
        border: "none",
        font: "inherit",
        padding: 0,
        margin: 0
      },
      "& input, & button, & label": {
        margin: ".2em .6em .2em 0"
      },
      "& input[type=checkbox]": {
        marginRight: ".2em"
      },
      "& label": {
        fontSize: "80%",
        whiteSpace: "pre"
      }
    },
    "&light .cm-searchMatch": { backgroundColor: "#ffff0054" },
    "&dark .cm-searchMatch": { backgroundColor: "#00ffff8a" },
    "&light .cm-searchMatch-selected": { backgroundColor: "#ff6a0054" },
    "&dark .cm-searchMatch-selected": { backgroundColor: "#ff00ff8a" }
  }), searchExtensions = [
    searchState,
    /* @__PURE__ */ Prec.low(searchHighlighter),
    baseTheme$2
  ];
  class SelectedDiagnostic {
    constructor(_, H, V) {
      this.from = _, this.to = H, this.diagnostic = V;
    }
  }
  class LintState {
    constructor(_, H, V) {
      this.diagnostics = _, this.panel = H, this.selected = V;
    }
    static init(_, H, V) {
      let q = _, G = V.facet(lintConfig).markerFilter;
      G && (q = G(q, V));
      let J = Decoration.set(q.map((Z) => Z.from == Z.to || Z.from == Z.to - 1 && V.doc.lineAt(Z.from).to == Z.from ? Decoration.widget({
        widget: new DiagnosticWidget(Z),
        diagnostic: Z
      }).range(Z.from) : Decoration.mark({
        attributes: { class: "cm-lintRange cm-lintRange-" + Z.severity + (Z.markClass ? " " + Z.markClass : "") },
        diagnostic: Z
      }).range(Z.from, Z.to)), !0);
      return new LintState(J, H, findDiagnostic(J));
    }
  }
  function findDiagnostic(O, _ = null, H = 0) {
    let V = null;
    return O.between(H, 1e9, (q, G, { spec: J }) => {
      if (!(_ && J.diagnostic != _))
        return V = new SelectedDiagnostic(q, G, J.diagnostic), !1;
    }), V;
  }
  function hideTooltip(O, _) {
    let H = _.pos, V = _.end || H, q = O.state.facet(lintConfig).hideOn(O, H, V);
    if (q != null)
      return q;
    let G = O.startState.doc.lineAt(_.pos);
    return !!(O.effects.some((J) => J.is(setDiagnosticsEffect)) || O.changes.touchesRange(G.from, Math.max(G.to, V)));
  }
  function maybeEnableLint(O, _) {
    return O.field(lintState, !1) ? _ : _.concat(StateEffect.appendConfig.of(lintExtensions));
  }
  const setDiagnosticsEffect = /* @__PURE__ */ StateEffect.define(), togglePanel = /* @__PURE__ */ StateEffect.define(), movePanelSelection = /* @__PURE__ */ StateEffect.define(), lintState = /* @__PURE__ */ StateField.define({
    create() {
      return new LintState(Decoration.none, null, null);
    },
    update(O, _) {
      if (_.docChanged && O.diagnostics.size) {
        let H = O.diagnostics.map(_.changes), V = null, q = O.panel;
        if (O.selected) {
          let G = _.changes.mapPos(O.selected.from, 1);
          V = findDiagnostic(H, O.selected.diagnostic, G) || findDiagnostic(H, null, G);
        }
        !H.size && q && _.state.facet(lintConfig).autoPanel && (q = null), O = new LintState(H, q, V);
      }
      for (let H of _.effects)
        if (H.is(setDiagnosticsEffect)) {
          let V = _.state.facet(lintConfig).autoPanel ? H.value.length ? LintPanel.open : null : O.panel;
          O = LintState.init(H.value, V, _.state);
        } else H.is(togglePanel) ? O = new LintState(O.diagnostics, H.value ? LintPanel.open : null, O.selected) : H.is(movePanelSelection) && (O = new LintState(O.diagnostics, O.panel, H.value));
      return O;
    },
    provide: (O) => [
      showPanel.from(O, (_) => _.panel),
      EditorView.decorations.from(O, (_) => _.diagnostics)
    ]
  }), activeMark = /* @__PURE__ */ Decoration.mark({ class: "cm-lintRange cm-lintRange-active" });
  function lintTooltip(O, _, H) {
    let { diagnostics: V } = O.state.field(lintState), q = [], G = 2e8, J = 0;
    V.between(_ - (H < 0 ? 1 : 0), _ + (H > 0 ? 1 : 0), (te, ne, { spec: ee }) => {
      _ >= te && _ <= ne && (te == ne || (_ > te || H > 0) && (_ < ne || H < 0)) && (q.push(ee.diagnostic), G = Math.min(te, G), J = Math.max(ne, J));
    });
    let Z = O.state.facet(lintConfig).tooltipFilter;
    return Z && (q = Z(q, O.state)), q.length ? {
      pos: G,
      end: J,
      above: O.state.doc.lineAt(G).to < J,
      create() {
        return { dom: diagnosticsTooltip(O, q) };
      }
    } : null;
  }
  function diagnosticsTooltip(O, _) {
    return crelt("ul", { class: "cm-tooltip-lint" }, _.map((H) => renderDiagnostic(O, H, !1)));
  }
  const openLintPanel = (O) => {
    let _ = O.state.field(lintState, !1);
    (!_ || !_.panel) && O.dispatch({ effects: maybeEnableLint(O.state, [togglePanel.of(!0)]) });
    let H = getPanel(O, LintPanel.open);
    return H && H.dom.querySelector(".cm-panel-lint ul").focus(), !0;
  }, closeLintPanel = (O) => {
    let _ = O.state.field(lintState, !1);
    return !_ || !_.panel ? !1 : (O.dispatch({ effects: togglePanel.of(!1) }), !0);
  }, nextDiagnostic = (O) => {
    let _ = O.state.field(lintState, !1);
    if (!_)
      return !1;
    let H = O.state.selection.main, V = _.diagnostics.iter(H.to + 1);
    return !V.value && (V = _.diagnostics.iter(0), !V.value || V.from == H.from && V.to == H.to) ? !1 : (O.dispatch({ selection: { anchor: V.from, head: V.to }, scrollIntoView: !0 }), !0);
  }, lintKeymap = [
    { key: "Mod-Shift-m", run: openLintPanel, preventDefault: !0 },
    { key: "F8", run: nextDiagnostic }
  ], lintConfig = /* @__PURE__ */ Facet.define({
    combine(O) {
      return Object.assign({ sources: O.map((_) => _.source).filter((_) => _ != null) }, combineConfig(O.map((_) => _.config), {
        delay: 750,
        markerFilter: null,
        tooltipFilter: null,
        needsRefresh: null,
        hideOn: () => null
      }, {
        needsRefresh: (_, H) => _ ? H ? (V) => _(V) || H(V) : _ : H
      }));
    }
  });
  function assignKeys(O) {
    let _ = [];
    if (O)
      e: for (let { name: H } of O) {
        for (let V = 0; V < H.length; V++) {
          let q = H[V];
          if (/[a-zA-Z]/.test(q) && !_.some((G) => G.toLowerCase() == q.toLowerCase())) {
            _.push(q);
            continue e;
          }
        }
        _.push("");
      }
    return _;
  }
  function renderDiagnostic(O, _, H) {
    var V;
    let q = H ? assignKeys(_.actions) : [];
    return crelt("li", { class: "cm-diagnostic cm-diagnostic-" + _.severity }, crelt("span", { class: "cm-diagnosticText" }, _.renderMessage ? _.renderMessage(O) : _.message), (V = _.actions) === null || V === void 0 ? void 0 : V.map((G, J) => {
      let Z = !1, te = (oe) => {
        if (oe.preventDefault(), Z)
          return;
        Z = !0;
        let se = findDiagnostic(O.state.field(lintState).diagnostics, _);
        se && G.apply(O, se.from, se.to);
      }, { name: ne } = G, ee = q[J] ? ne.indexOf(q[J]) : -1, ae = ee < 0 ? ne : [
        ne.slice(0, ee),
        crelt("u", ne.slice(ee, ee + 1)),
        ne.slice(ee + 1)
      ];
      return crelt("button", {
        type: "button",
        class: "cm-diagnosticAction",
        onclick: te,
        onmousedown: te,
        "aria-label": ` Action: ${ne}${ee < 0 ? "" : ` (access key "${q[J]})"`}.`
      }, ae);
    }), _.source && crelt("div", { class: "cm-diagnosticSource" }, _.source));
  }
  class DiagnosticWidget extends WidgetType {
    constructor(_) {
      super(), this.diagnostic = _;
    }
    eq(_) {
      return _.diagnostic == this.diagnostic;
    }
    toDOM() {
      return crelt("span", { class: "cm-lintPoint cm-lintPoint-" + this.diagnostic.severity });
    }
  }
  class PanelItem {
    constructor(_, H) {
      this.diagnostic = H, this.id = "item_" + Math.floor(Math.random() * 4294967295).toString(16), this.dom = renderDiagnostic(_, H, !0), this.dom.id = this.id, this.dom.setAttribute("role", "option");
    }
  }
  class LintPanel {
    constructor(_) {
      this.view = _, this.items = [];
      let H = (q) => {
        if (q.keyCode == 27)
          closeLintPanel(this.view), this.view.focus();
        else if (q.keyCode == 38 || q.keyCode == 33)
          this.moveSelection((this.selectedIndex - 1 + this.items.length) % this.items.length);
        else if (q.keyCode == 40 || q.keyCode == 34)
          this.moveSelection((this.selectedIndex + 1) % this.items.length);
        else if (q.keyCode == 36)
          this.moveSelection(0);
        else if (q.keyCode == 35)
          this.moveSelection(this.items.length - 1);
        else if (q.keyCode == 13)
          this.view.focus();
        else if (q.keyCode >= 65 && q.keyCode <= 90 && this.selectedIndex >= 0) {
          let { diagnostic: G } = this.items[this.selectedIndex], J = assignKeys(G.actions);
          for (let Z = 0; Z < J.length; Z++)
            if (J[Z].toUpperCase().charCodeAt(0) == q.keyCode) {
              let te = findDiagnostic(this.view.state.field(lintState).diagnostics, G);
              te && G.actions[Z].apply(_, te.from, te.to);
            }
        } else
          return;
        q.preventDefault();
      }, V = (q) => {
        for (let G = 0; G < this.items.length; G++)
          this.items[G].dom.contains(q.target) && this.moveSelection(G);
      };
      this.list = crelt("ul", {
        tabIndex: 0,
        role: "listbox",
        "aria-label": this.view.state.phrase("Diagnostics"),
        onkeydown: H,
        onclick: V
      }), this.dom = crelt("div", { class: "cm-panel-lint" }, this.list, crelt("button", {
        type: "button",
        name: "close",
        "aria-label": this.view.state.phrase("close"),
        onclick: () => closeLintPanel(this.view)
      }, "×")), this.update();
    }
    get selectedIndex() {
      let _ = this.view.state.field(lintState).selected;
      if (!_)
        return -1;
      for (let H = 0; H < this.items.length; H++)
        if (this.items[H].diagnostic == _.diagnostic)
          return H;
      return -1;
    }
    update() {
      let { diagnostics: _, selected: H } = this.view.state.field(lintState), V = 0, q = !1, G = null;
      for (_.between(0, this.view.state.doc.length, (J, Z, { spec: te }) => {
        let ne = -1, ee;
        for (let ae = V; ae < this.items.length; ae++)
          if (this.items[ae].diagnostic == te.diagnostic) {
            ne = ae;
            break;
          }
        ne < 0 ? (ee = new PanelItem(this.view, te.diagnostic), this.items.splice(V, 0, ee), q = !0) : (ee = this.items[ne], ne > V && (this.items.splice(V, ne - V), q = !0)), H && ee.diagnostic == H.diagnostic ? ee.dom.hasAttribute("aria-selected") || (ee.dom.setAttribute("aria-selected", "true"), G = ee) : ee.dom.hasAttribute("aria-selected") && ee.dom.removeAttribute("aria-selected"), V++;
      }); V < this.items.length && !(this.items.length == 1 && this.items[0].diagnostic.from < 0); )
        q = !0, this.items.pop();
      this.items.length == 0 && (this.items.push(new PanelItem(this.view, {
        from: -1,
        to: -1,
        severity: "info",
        message: this.view.state.phrase("No diagnostics")
      })), q = !0), G ? (this.list.setAttribute("aria-activedescendant", G.id), this.view.requestMeasure({
        key: this,
        read: () => ({ sel: G.dom.getBoundingClientRect(), panel: this.list.getBoundingClientRect() }),
        write: ({ sel: J, panel: Z }) => {
          let te = Z.height / this.list.offsetHeight;
          J.top < Z.top ? this.list.scrollTop -= (Z.top - J.top) / te : J.bottom > Z.bottom && (this.list.scrollTop += (J.bottom - Z.bottom) / te);
        }
      })) : this.selectedIndex < 0 && this.list.removeAttribute("aria-activedescendant"), q && this.sync();
    }
    sync() {
      let _ = this.list.firstChild;
      function H() {
        let V = _;
        _ = V.nextSibling, V.remove();
      }
      for (let V of this.items)
        if (V.dom.parentNode == this.list) {
          for (; _ != V.dom; )
            H();
          _ = V.dom.nextSibling;
        } else
          this.list.insertBefore(V.dom, _);
      for (; _; )
        H();
    }
    moveSelection(_) {
      if (this.selectedIndex < 0)
        return;
      let H = this.view.state.field(lintState), V = findDiagnostic(H.diagnostics, this.items[_].diagnostic);
      V && this.view.dispatch({
        selection: { anchor: V.from, head: V.to },
        scrollIntoView: !0,
        effects: movePanelSelection.of(V)
      });
    }
    static open(_) {
      return new LintPanel(_);
    }
  }
  function svg(O, _ = 'viewBox="0 0 40 40"') {
    return `url('data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" ${_}>${encodeURIComponent(O)}</svg>')`;
  }
  function underline(O) {
    return svg(`<path d="m0 2.5 l2 -1.5 l1 0 l2 1.5 l1 0" stroke="${O}" fill="none" stroke-width=".7"/>`, 'width="6" height="3"');
  }
  const baseTheme = /* @__PURE__ */ EditorView.baseTheme({
    ".cm-diagnostic": {
      padding: "3px 6px 3px 8px",
      marginLeft: "-1px",
      display: "block",
      whiteSpace: "pre-wrap"
    },
    ".cm-diagnostic-error": { borderLeft: "5px solid #d11" },
    ".cm-diagnostic-warning": { borderLeft: "5px solid orange" },
    ".cm-diagnostic-info": { borderLeft: "5px solid #999" },
    ".cm-diagnostic-hint": { borderLeft: "5px solid #66d" },
    ".cm-diagnosticAction": {
      font: "inherit",
      border: "none",
      padding: "2px 4px",
      backgroundColor: "#444",
      color: "white",
      borderRadius: "3px",
      marginLeft: "8px",
      cursor: "pointer"
    },
    ".cm-diagnosticSource": {
      fontSize: "70%",
      opacity: 0.7
    },
    ".cm-lintRange": {
      backgroundPosition: "left bottom",
      backgroundRepeat: "repeat-x",
      paddingBottom: "0.7px"
    },
    ".cm-lintRange-error": { backgroundImage: /* @__PURE__ */ underline("#d11") },
    ".cm-lintRange-warning": { backgroundImage: /* @__PURE__ */ underline("orange") },
    ".cm-lintRange-info": { backgroundImage: /* @__PURE__ */ underline("#999") },
    ".cm-lintRange-hint": { backgroundImage: /* @__PURE__ */ underline("#66d") },
    ".cm-lintRange-active": { backgroundColor: "#ffdd9980" },
    ".cm-tooltip-lint": {
      padding: 0,
      margin: 0
    },
    ".cm-lintPoint": {
      position: "relative",
      "&:after": {
        content: '""',
        position: "absolute",
        bottom: 0,
        left: "-2px",
        borderLeft: "3px solid transparent",
        borderRight: "3px solid transparent",
        borderBottom: "4px solid #d11"
      }
    },
    ".cm-lintPoint-warning": {
      "&:after": { borderBottomColor: "orange" }
    },
    ".cm-lintPoint-info": {
      "&:after": { borderBottomColor: "#999" }
    },
    ".cm-lintPoint-hint": {
      "&:after": { borderBottomColor: "#66d" }
    },
    ".cm-panel.cm-panel-lint": {
      position: "relative",
      "& ul": {
        maxHeight: "100px",
        overflowY: "auto",
        "& [aria-selected]": {
          backgroundColor: "#ddd",
          "& u": { textDecoration: "underline" }
        },
        "&:focus [aria-selected]": {
          background_fallback: "#bdf",
          backgroundColor: "Highlight",
          color_fallback: "white",
          color: "HighlightText"
        },
        "& u": { textDecoration: "none" },
        padding: 0,
        margin: 0
      },
      "& [name=close]": {
        position: "absolute",
        top: "0",
        right: "2px",
        background: "inherit",
        border: "none",
        font: "inherit",
        padding: 0,
        margin: 0
      }
    }
  }), lintExtensions = [
    lintState,
    /* @__PURE__ */ EditorView.decorations.compute([lintState], (O) => {
      let { selected: _, panel: H } = O.field(lintState);
      return !_ || !H || _.from == _.to ? Decoration.none : Decoration.set([
        activeMark.range(_.from, _.to)
      ]);
    }),
    /* @__PURE__ */ hoverTooltip(lintTooltip, { hideOn: hideTooltip }),
    baseTheme
  ], basicSetup = [
    lineNumbers(),
    highlightActiveLineGutter(),
    highlightSpecialChars(),
    history(),
    foldGutter(),
    drawSelection(),
    dropCursor(),
    EditorState.allowMultipleSelections.of(!0),
    indentOnInput(),
    syntaxHighlighting(defaultHighlightStyle, { fallback: !0 }),
    bracketMatching(),
    closeBrackets(),
    autocompletion(),
    rectangularSelection(),
    crosshairCursor(),
    highlightActiveLine(),
    highlightSelectionMatches(),
    keymap.of([
      ...closeBracketsKeymap,
      ...defaultKeymap,
      ...searchKeymap,
      ...historyKeymap,
      ...foldKeymap,
      ...completionKeymap,
      ...lintKeymap
    ])
  ], _hoisted_1$1 = { class: "flex flex-col justify-start border-b border-gray-200" }, _hoisted_2 = { class: "flex flex-col flex-fixed h-full" }, _hoisted_3 = ["value"], _hoisted_4 = { class: "flex justify-start flex-fixed my-2 px-2" }, _hoisted_5 = {
    key: 0,
    class: "w-full bg-red-100",
    "data-cy": "alasql-error-msg"
  }, _sfc_main$1 = /* @__PURE__ */ defineComponent({
    __name: "QueryArea",
    props: {
      initialQuery: {},
      errors: {}
    },
    emits: ["query-change", "run-query", "clear-query"],
    setup(O, { emit: _ }) {
      function H(Z, te) {
        const ne = new EditorView({ doc: Z.value, extensions: te });
        return Z.parentNode.insertBefore(ne.dom, Z), Z.style.display = "none", Z.form && Z.form.addEventListener("submit", () => {
          Z.value = ne.state.doc.toString();
        }), ne;
      }
      const V = _, q = ref$1(), G = ref$1(), J = (Z) => void V("query-change", Z);
      return onMounted(() => {
        q.value ? (G.value = H(q.value, [
          basicSetup,
          sql(),
          EditorView.updateListener.of((Z) => {
            J(Z.state.doc.toString());
          })
        ]), J(G.value.state.doc.toString())) : console.error("Couldn't find codemirror textarea element");
      }), (Z, te) => (openBlock(), createElementBlock(Fragment, null, [
        createBaseVNode("div", _hoisted_1$1, [
          createBaseVNode("div", _hoisted_2, [
            createBaseVNode("textarea", {
              ref_key: "cmEl",
              ref: q,
              value: Z.initialQuery
            }, null, 8, _hoisted_3),
            createBaseVNode("div", _hoisted_4, [
              createVNode(_sfc_main$P, {
                onClick: te[0] || (te[0] = (ne) => V("run-query")),
                icon: "fa fa-play",
                "data-cy": "btn-run-query",
                class: "w-28 ar-btn-primary"
              }, {
                default: withCtx(() => te[2] || (te[2] = [
                  createTextVNode(" Run Query ")
                ])),
                _: 1
              }),
              createVNode(_sfc_main$P, {
                onClick: te[1] || (te[1] = (ne) => V("clear-query")),
                icon: "fa fa-undo",
                class: "ml-2 w-28 ar-btn-secondary-gray",
                "data-cy": "btn-reset-data"
              }, {
                default: withCtx(() => te[3] || (te[3] = [
                  createTextVNode(" Reset Data ")
                ])),
                _: 1
              })
            ])
          ])
        ]),
        Z.errors ? (openBlock(), createElementBlock("div", _hoisted_5, toDisplayString(Z.errors), 1)) : createCommentVNode("", !0)
      ], 64));
    }
  }), _hoisted_1 = {
    key: 2,
    class: "w-full flex justify-center"
  }, TableColors = {
    string: "green",
    double: "blue",
    boolean: "indigo",
    timestamp: "red",
    category: "orange",
    integer: "teal",
    index: "black",
    unknown: "black"
  }, TableIcons = {
    string: "font",
    timestamp: "calendar",
    category: "cubes",
    double: "bar-chart",
    boolean: "toggle-on",
    integer: "bar-chart",
    index: "list-ol",
    unknown: "circle"
  }, DEFAULT_QUERY = "SELECT * FROM $table", SCHEMA_SEARCH_LIMIT = 10, DISTINCT_CATEGORIES_LIMIT = 8, _sfc_main = /* @__PURE__ */ defineComponent({
    __name: "DataTable",
    props: {
      singleBlockEmbed: { type: Boolean },
      data: {},
      cells: {},
      schema: {},
      previewMode: { type: Boolean },
      refId: {},
      getCsvText: { type: Function },
      downloadLocal: { type: Function }
    },
    emits: ["load-full"],
    setup(O, { emit: _ }) {
      const H = O, V = _, q = ref$1(DEFAULT_QUERY), G = ref$1(), J = ref$1(!1), Z = ref$1(), te = computed(() => {
        var de;
        return ((de = G.value) == null ? void 0 : de.schema) ?? H.schema;
      }), ne = computed(() => {
        var de;
        return ((de = G.value) == null ? void 0 : de.data) ?? H.data;
      }), ee = (de, fe, he) => {
        const me = fe[de], pe = he[de];
        return me === pe ? 0 : me > pe ? 1 : -1;
      }, ae = (de, fe) => {
        const he = (() => {
          var ye;
          return (ye = fe.type) != null && ye.toString().toLowerCase().includes("utf8") ? "category" : fe.type || "unknown";
        })(), me = TableIcons[he], pe = TableColors[he];
        return de(
          "div",
          {
            class: "flex items-center w-full whitespace-nowrap overflow-hidden"
          },
          de("i", {
            class: `fa fa-${me} pr-2 text-${pe}-400`
          }),
          de("div", {}, fe.name)
        );
      }, oe = (de) => {
        switch (de) {
          case "double":
          case "integer":
            return "number";
          case "timestamp":
            return "date";
          case "category":
            return "select";
          default:
            return "string";
        }
      }, se = computed(() => {
        const de = H.data[0];
        return H.previewMode || !de ? [] : (te.value && te.value.length ? te.value.map((he) => he.name) : Object.keys(de)).map((he) => {
          const me = te.value && te.value.find((ge) => ge.name === he), pe = me && me.type ? me.type : "string", ye = oe(pe);
          return {
            prop: he,
            name: he,
            size: 200,
            sortable: !0,
            columnType: ye,
            type: pe,
            filter: ye,
            autoSize: !0,
            cellCompare: ye === "number" ? ee : void 0,
            columnTemplate: ae
          };
        });
      }), ce = () => {
        var he;
        const de = (me, pe) => {
          const ye = (Se) => Number(Se) === Se && `${Se}`.includes(".") ? "double" : Number(Se) === Se ? "integer" : !!Se === Se ? "boolean" : "string", ge = Math.min(SCHEMA_SEARCH_LIMIT, H.data.length - 1), ve = pe.slice(0, ge).map((Se) => Se[me]), be = ve.map(ye);
          return be.every((Se) => Se === be[0]) && be[0] !== "string" ? be[0] : [...new Set(ve)].length <= DISTINCT_CATEGORIES_LIMIT ? "category" : "string";
        }, fe = (me) => Object.keys(me[0]).map((ye) => ({
          name: ye,
          type: de(ye, me)
        }));
        Z.value = null;
        try {
          const me = (he = q.value) == null ? void 0 : he.replace(/\$table/g, "?"), pe = alasql(me, [H.data]), ye = pe.length ? fe(pe) : [];
          G.value = { data: pe, schema: ye };
        } catch (me) {
          Z.value = me;
        }
      }, le = (de) => {
        q.value = de;
      }, ue = () => {
        G.value = null, Z.value = null;
      };
      return (de, fe) => (openBlock(), createElementBlock("div", {
        "data-cy": "block-datatable",
        class: normalizeClass([
          "rounded shadow w-full",
          { "h-full flex flex-col": de.singleBlockEmbed }
        ])
      }, [
        createVNode(_sfc_main$2, {
          "single-block-embed": de.singleBlockEmbed,
          "preview-mode": de.previewMode,
          "query-open": J.value,
          rows: ne.value.length,
          columns: se.value.length,
          cells: de.cells,
          "get-csv-text": de.getCsvText,
          "download-local": de.downloadLocal,
          onToggleQueryOpen: fe[0] || (fe[0] = (he) => J.value = !J.value)
        }, null, 8, ["single-block-embed", "preview-mode", "query-open", "rows", "columns", "cells", "get-csv-text", "download-local"]),
        J.value ? (openBlock(), createBlock(_sfc_main$1, {
          key: 0,
          errors: Z.value,
          initialQuery: q.value,
          onQueryChange: le,
          onRunQuery: ce,
          onClearQuery: ue
        }, null, 8, ["errors", "initialQuery"])) : createCommentVNode("", !0),
        se.value.length && !de.previewMode ? (openBlock(), createBlock(unref(Q), {
          key: 1,
          theme: "compact",
          source: ne.value,
          columns: se.value,
          class: normalizeClass({
            "flex-1": de.singleBlockEmbed,
            "h-96": !de.singleBlockEmbed
          }),
          resize: !0,
          autoSizeColumn: !0,
          rowHeaders: !0,
          filter: !0,
          readonly: !0,
          exporting: !0,
          id: `grid-${de.refId}`
        }, null, 8, ["source", "columns", "class", "id"])) : createCommentVNode("", !0),
        de.previewMode ? (openBlock(), createElementBlock("div", _hoisted_1, [
          createVNode(_sfc_main$P, {
            dataCy: "button-load-dataset",
            onClick: fe[1] || (fe[1] = (he) => V("load-full")),
            icon: "fa fa-table"
          }, {
            default: withCtx(() => [
              createTextVNode(" Click to load dataset (" + toDisplayString(unref(formatNumber$1)(de.cells)) + " cells) ", 1)
            ]),
            _: 1
          })
        ])) : createCommentVNode("", !0)
      ], 2));
    }
  }), DataTable = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    default: _sfc_main
  }, Symbol.toStringTag, { value: "Module" }));
  function __awaiter(O, _, H, V) {
    function q(G) {
      return G instanceof H ? G : new H(function(J) {
        J(G);
      });
    }
    return new (H || (H = Promise))(function(G, J) {
      function Z(ee) {
        try {
          ne(V.next(ee));
        } catch (ae) {
          J(ae);
        }
      }
      function te(ee) {
        try {
          ne(V.throw(ee));
        } catch (ae) {
          J(ae);
        }
      }
      function ne(ee) {
        ee.done ? G(ee.value) : q(ee.value).then(Z, te);
      }
      ne((V = V.apply(O, _ || [])).next());
    });
  }
  function __values(O) {
    var _ = typeof Symbol == "function" && Symbol.iterator, H = _ && O[_], V = 0;
    if (H) return H.call(O);
    if (O && typeof O.length == "number") return {
      next: function() {
        return O && V >= O.length && (O = void 0), { value: O && O[V++], done: !O };
      }
    };
    throw new TypeError(_ ? "Object is not iterable." : "Symbol.iterator is not defined.");
  }
  function __await(O) {
    return this instanceof __await ? (this.v = O, this) : new __await(O);
  }
  function __asyncGenerator(O, _, H) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var V = H.apply(O, _ || []), q, G = [];
    return q = Object.create((typeof AsyncIterator == "function" ? AsyncIterator : Object).prototype), Z("next"), Z("throw"), Z("return", J), q[Symbol.asyncIterator] = function() {
      return this;
    }, q;
    function J(se) {
      return function(ce) {
        return Promise.resolve(ce).then(se, ae);
      };
    }
    function Z(se, ce) {
      V[se] && (q[se] = function(le) {
        return new Promise(function(ue, de) {
          G.push([se, le, ue, de]) > 1 || te(se, le);
        });
      }, ce && (q[se] = ce(q[se])));
    }
    function te(se, ce) {
      try {
        ne(V[se](ce));
      } catch (le) {
        oe(G[0][3], le);
      }
    }
    function ne(se) {
      se.value instanceof __await ? Promise.resolve(se.value.v).then(ee, ae) : oe(G[0][2], se);
    }
    function ee(se) {
      te("next", se);
    }
    function ae(se) {
      te("throw", se);
    }
    function oe(se, ce) {
      se(ce), G.shift(), G.length && te(G[0][0], G[0][1]);
    }
  }
  function __asyncDelegator(O) {
    var _, H;
    return _ = {}, V("next"), V("throw", function(q) {
      throw q;
    }), V("return"), _[Symbol.iterator] = function() {
      return this;
    }, _;
    function V(q, G) {
      _[q] = O[q] ? function(J) {
        return (H = !H) ? { value: __await(O[q](J)), done: !1 } : G ? G(J) : J;
      } : G;
    }
  }
  function __asyncValues(O) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var _ = O[Symbol.asyncIterator], H;
    return _ ? _.call(O) : (O = typeof __values == "function" ? __values(O) : O[Symbol.iterator](), H = {}, V("next"), V("throw"), V("return"), H[Symbol.asyncIterator] = function() {
      return this;
    }, H);
    function V(G) {
      H[G] = O[G] && function(J) {
        return new Promise(function(Z, te) {
          J = O[G](J), q(Z, te, J.done, J.value);
        });
      };
    }
    function q(G, J, Z, te) {
      Promise.resolve(te).then(function(ne) {
        G({ value: ne, done: Z });
      }, J);
    }
  }
  typeof SuppressedError == "function" && SuppressedError;
  const decoder = new TextDecoder("utf-8"), decodeUtf8 = (O) => decoder.decode(O), encoder = new TextEncoder(), encodeUtf8 = (O) => encoder.encode(O), isNumber = (O) => typeof O == "number", isBoolean = (O) => typeof O == "boolean", isFunction = (O) => typeof O == "function", isObject = (O) => O != null && Object(O) === O, isPromise = (O) => isObject(O) && isFunction(O.then), isIterable = (O) => isObject(O) && isFunction(O[Symbol.iterator]), isAsyncIterable = (O) => isObject(O) && isFunction(O[Symbol.asyncIterator]), isArrowJSON = (O) => isObject(O) && isObject(O.schema), isIteratorResult = (O) => isObject(O) && "done" in O && "value" in O, isFileHandle = (O) => isObject(O) && isFunction(O.stat) && isNumber(O.fd), isFetchResponse = (O) => isObject(O) && isReadableDOMStream(O.body), isReadableInterop = (O) => "_getDOMStream" in O && "_getNodeStream" in O, isReadableDOMStream = (O) => isObject(O) && isFunction(O.cancel) && isFunction(O.getReader) && !isReadableInterop(O), isReadableNodeStream = (O) => isObject(O) && isFunction(O.read) && isFunction(O.pipe) && isBoolean(O.readable) && !isReadableInterop(O), isFlatbuffersByteBuffer = (O) => isObject(O) && isFunction(O.clear) && isFunction(O.bytes) && isFunction(O.position) && isFunction(O.setPosition) && isFunction(O.capacity) && isFunction(O.getBufferIdentifier) && isFunction(O.createLong), SharedArrayBuf = typeof SharedArrayBuffer < "u" ? SharedArrayBuffer : ArrayBuffer;
  function collapseContiguousByteRanges(O) {
    const _ = O[0] ? [O[0]] : [];
    let H, V, q, G;
    for (let J, Z, te = 0, ne = 0, ee = O.length; ++te < ee; ) {
      if (J = _[ne], Z = O[te], !J || !Z || J.buffer !== Z.buffer || Z.byteOffset < J.byteOffset) {
        Z && (_[++ne] = Z);
        continue;
      }
      if ({ byteOffset: H, byteLength: q } = J, { byteOffset: V, byteLength: G } = Z, H + q < V || V + G < H) {
        Z && (_[++ne] = Z);
        continue;
      }
      _[ne] = new Uint8Array(J.buffer, H, V - H + G);
    }
    return _;
  }
  function memcpy(O, _, H = 0, V = _.byteLength) {
    const q = O.byteLength, G = new Uint8Array(O.buffer, O.byteOffset, q), J = new Uint8Array(_.buffer, _.byteOffset, Math.min(V, q));
    return G.set(J, H), O;
  }
  function joinUint8Arrays(O, _) {
    const H = collapseContiguousByteRanges(O), V = H.reduce((ee, ae) => ee + ae.byteLength, 0);
    let q, G, J, Z = 0, te = -1;
    const ne = Math.min(_ || Number.POSITIVE_INFINITY, V);
    for (const ee = H.length; ++te < ee; ) {
      if (q = H[te], G = q.subarray(0, Math.min(q.length, ne - Z)), ne <= Z + G.length) {
        G.length < q.length ? H[te] = q.subarray(G.length) : G.length === q.length && te++, J ? memcpy(J, G, Z) : J = G;
        break;
      }
      memcpy(J || (J = new Uint8Array(ne)), G, Z), Z += G.length;
    }
    return [J || new Uint8Array(0), H.slice(te), V - (J ? J.byteLength : 0)];
  }
  function toArrayBufferView(O, _) {
    let H = isIteratorResult(_) ? _.value : _;
    return H instanceof O ? O === Uint8Array ? new O(H.buffer, H.byteOffset, H.byteLength) : H : H ? (typeof H == "string" && (H = encodeUtf8(H)), H instanceof ArrayBuffer ? new O(H) : H instanceof SharedArrayBuf ? new O(H) : isFlatbuffersByteBuffer(H) ? toArrayBufferView(O, H.bytes()) : ArrayBuffer.isView(H) ? H.byteLength <= 0 ? new O(0) : new O(H.buffer, H.byteOffset, H.byteLength / O.BYTES_PER_ELEMENT) : O.from(H)) : new O(0);
  }
  const toInt32Array = (O) => toArrayBufferView(Int32Array, O), toBigInt64Array = (O) => toArrayBufferView(BigInt64Array, O), toUint8Array = (O) => toArrayBufferView(Uint8Array, O), pump$1 = (O) => (O.next(), O);
  function* toArrayBufferViewIterator(O, _) {
    const H = function* (q) {
      yield q;
    }, V = typeof _ == "string" || ArrayBuffer.isView(_) || _ instanceof ArrayBuffer || _ instanceof SharedArrayBuf ? H(_) : isIterable(_) ? _ : H(_);
    return yield* pump$1(function* (q) {
      let G = null;
      do
        G = q.next(yield toArrayBufferView(O, G));
      while (!G.done);
    }(V[Symbol.iterator]())), new O();
  }
  const toUint8ArrayIterator = (O) => toArrayBufferViewIterator(Uint8Array, O);
  function toArrayBufferViewAsyncIterator(O, _) {
    return __asyncGenerator(this, arguments, function* () {
      if (isPromise(_))
        return yield __await(yield __await(yield* __asyncDelegator(__asyncValues(toArrayBufferViewAsyncIterator(O, yield __await(_))))));
      const V = function(J) {
        return __asyncGenerator(this, arguments, function* () {
          yield yield __await(yield __await(J));
        });
      }, q = function(J) {
        return __asyncGenerator(this, arguments, function* () {
          yield __await(yield* __asyncDelegator(__asyncValues(pump$1(function* (Z) {
            let te = null;
            do
              te = Z.next(yield te == null ? void 0 : te.value);
            while (!te.done);
          }(J[Symbol.iterator]())))));
        });
      }, G = typeof _ == "string" || ArrayBuffer.isView(_) || _ instanceof ArrayBuffer || _ instanceof SharedArrayBuf ? V(_) : isIterable(_) ? q(_) : isAsyncIterable(_) ? _ : V(_);
      return yield __await(
        // otherwise if AsyncIterable, use it
        yield* __asyncDelegator(__asyncValues(pump$1(function(J) {
          return __asyncGenerator(this, arguments, function* () {
            let Z = null;
            do
              Z = yield __await(J.next(yield yield __await(toArrayBufferView(O, Z))));
            while (!Z.done);
          });
        }(G[Symbol.asyncIterator]()))))
      ), yield __await(new O());
    });
  }
  const toUint8ArrayAsyncIterator = (O) => toArrayBufferViewAsyncIterator(Uint8Array, O);
  function compareArrayLike(O, _) {
    let H = 0;
    const V = O.length;
    if (V !== _.length)
      return !1;
    if (V > 0)
      do
        if (O[H] !== _[H])
          return !1;
      while (++H < V);
    return !0;
  }
  const streamAdapters = {
    fromIterable(O) {
      return pump(fromIterable(O));
    },
    fromAsyncIterable(O) {
      return pump(fromAsyncIterable(O));
    },
    fromDOMStream(O) {
      return pump(fromDOMStream(O));
    },
    fromNodeStream(O) {
      return pump(fromNodeStream(O));
    },
    // @ts-ignore
    toDOMStream(O, _) {
      throw new Error('"toDOMStream" not available in this environment');
    },
    // @ts-ignore
    toNodeStream(O, _) {
      throw new Error('"toNodeStream" not available in this environment');
    }
  }, pump = (O) => (O.next(), O);
  function* fromIterable(O) {
    let _, H = !1, V = [], q, G, J, Z = 0;
    function te() {
      return G === "peek" ? joinUint8Arrays(V, J)[0] : ([q, V, Z] = joinUint8Arrays(V, J), q);
    }
    ({ cmd: G, size: J } = (yield null) || { cmd: "read", size: 0 });
    const ne = toUint8ArrayIterator(O)[Symbol.iterator]();
    try {
      do
        if ({ done: _, value: q } = Number.isNaN(J - Z) ? ne.next() : ne.next(J - Z), !_ && q.byteLength > 0 && (V.push(q), Z += q.byteLength), _ || J <= Z)
          do
            ({ cmd: G, size: J } = yield te());
          while (J < Z);
      while (!_);
    } catch (ee) {
      (H = !0) && typeof ne.throw == "function" && ne.throw(ee);
    } finally {
      H === !1 && typeof ne.return == "function" && ne.return(null);
    }
    return null;
  }
  function fromAsyncIterable(O) {
    return __asyncGenerator(this, arguments, function* () {
      let H, V = !1, q = [], G, J, Z, te = 0;
      function ne() {
        return J === "peek" ? joinUint8Arrays(q, Z)[0] : ([G, q, te] = joinUint8Arrays(q, Z), G);
      }
      ({ cmd: J, size: Z } = (yield yield __await(null)) || { cmd: "read", size: 0 });
      const ee = toUint8ArrayAsyncIterator(O)[Symbol.asyncIterator]();
      try {
        do
          if ({ done: H, value: G } = Number.isNaN(Z - te) ? yield __await(ee.next()) : yield __await(ee.next(Z - te)), !H && G.byteLength > 0 && (q.push(G), te += G.byteLength), H || Z <= te)
            do
              ({ cmd: J, size: Z } = yield yield __await(ne()));
            while (Z < te);
        while (!H);
      } catch (ae) {
        (V = !0) && typeof ee.throw == "function" && (yield __await(ee.throw(ae)));
      } finally {
        V === !1 && typeof ee.return == "function" && (yield __await(ee.return(new Uint8Array(0))));
      }
      return yield __await(null);
    });
  }
  function fromDOMStream(O) {
    return __asyncGenerator(this, arguments, function* () {
      let H = !1, V = !1, q = [], G, J, Z, te = 0;
      function ne() {
        return J === "peek" ? joinUint8Arrays(q, Z)[0] : ([G, q, te] = joinUint8Arrays(q, Z), G);
      }
      ({ cmd: J, size: Z } = (yield yield __await(null)) || { cmd: "read", size: 0 });
      const ee = new AdaptiveByteReader(O);
      try {
        do
          if ({ done: H, value: G } = Number.isNaN(Z - te) ? yield __await(ee.read()) : yield __await(ee.read(Z - te)), !H && G.byteLength > 0 && (q.push(toUint8Array(G)), te += G.byteLength), H || Z <= te)
            do
              ({ cmd: J, size: Z } = yield yield __await(ne()));
            while (Z < te);
        while (!H);
      } catch (ae) {
        (V = !0) && (yield __await(ee.cancel(ae)));
      } finally {
        V === !1 ? yield __await(ee.cancel()) : O.locked && ee.releaseLock();
      }
      return yield __await(null);
    });
  }
  class AdaptiveByteReader {
    constructor(_) {
      this.source = _, this.reader = null, this.reader = this.source.getReader(), this.reader.closed.catch(() => {
      });
    }
    get closed() {
      return this.reader ? this.reader.closed.catch(() => {
      }) : Promise.resolve();
    }
    releaseLock() {
      this.reader && this.reader.releaseLock(), this.reader = null;
    }
    cancel(_) {
      return __awaiter(this, void 0, void 0, function* () {
        const { reader: H, source: V } = this;
        H && (yield H.cancel(_).catch(() => {
        })), V && V.locked && this.releaseLock();
      });
    }
    read(_) {
      return __awaiter(this, void 0, void 0, function* () {
        if (_ === 0)
          return { done: this.reader == null, value: new Uint8Array(0) };
        const H = yield this.reader.read();
        return !H.done && (H.value = toUint8Array(H)), H;
      });
    }
  }
  const onEvent = (O, _) => {
    const H = (q) => V([_, q]);
    let V;
    return [_, H, new Promise((q) => (V = q) && O.once(_, H))];
  };
  function fromNodeStream(O) {
    return __asyncGenerator(this, arguments, function* () {
      const H = [];
      let V = "error", q = !1, G = null, J, Z, te = 0, ne = [], ee;
      function ae() {
        return J === "peek" ? joinUint8Arrays(ne, Z)[0] : ([ee, ne, te] = joinUint8Arrays(ne, Z), ee);
      }
      if ({ cmd: J, size: Z } = (yield yield __await(null)) || { cmd: "read", size: 0 }, O.isTTY)
        return yield yield __await(new Uint8Array(0)), yield __await(null);
      try {
        H[0] = onEvent(O, "end"), H[1] = onEvent(O, "error");
        do {
          if (H[2] = onEvent(O, "readable"), [V, G] = yield __await(Promise.race(H.map((se) => se[2]))), V === "error")
            break;
          if ((q = V === "end") || (Number.isFinite(Z - te) ? (ee = toUint8Array(O.read(Z - te)), ee.byteLength < Z - te && (ee = toUint8Array(O.read()))) : ee = toUint8Array(O.read()), ee.byteLength > 0 && (ne.push(ee), te += ee.byteLength)), q || Z <= te)
            do
              ({ cmd: J, size: Z } = yield yield __await(ae()));
            while (Z < te);
        } while (!q);
      } finally {
        yield __await(oe(H, V === "error" ? G : null));
      }
      return yield __await(null);
      function oe(se, ce) {
        return ee = ne = null, new Promise((le, ue) => {
          for (const [de, fe] of se)
            O.off(de, fe);
          try {
            const de = O.destroy;
            de && de.call(O, ce), ce = void 0;
          } catch (de) {
            ce = de || ce;
          } finally {
            ce != null ? ue(ce) : le();
          }
        });
      }
    });
  }
  var MetadataVersion;
  (function(O) {
    O[O.V1 = 0] = "V1", O[O.V2 = 1] = "V2", O[O.V3 = 2] = "V3", O[O.V4 = 3] = "V4", O[O.V5 = 4] = "V5";
  })(MetadataVersion || (MetadataVersion = {}));
  var UnionMode;
  (function(O) {
    O[O.Sparse = 0] = "Sparse", O[O.Dense = 1] = "Dense";
  })(UnionMode || (UnionMode = {}));
  var Precision;
  (function(O) {
    O[O.HALF = 0] = "HALF", O[O.SINGLE = 1] = "SINGLE", O[O.DOUBLE = 2] = "DOUBLE";
  })(Precision || (Precision = {}));
  var DateUnit;
  (function(O) {
    O[O.DAY = 0] = "DAY", O[O.MILLISECOND = 1] = "MILLISECOND";
  })(DateUnit || (DateUnit = {}));
  var TimeUnit;
  (function(O) {
    O[O.SECOND = 0] = "SECOND", O[O.MILLISECOND = 1] = "MILLISECOND", O[O.MICROSECOND = 2] = "MICROSECOND", O[O.NANOSECOND = 3] = "NANOSECOND";
  })(TimeUnit || (TimeUnit = {}));
  var IntervalUnit;
  (function(O) {
    O[O.YEAR_MONTH = 0] = "YEAR_MONTH", O[O.DAY_TIME = 1] = "DAY_TIME", O[O.MONTH_DAY_NANO = 2] = "MONTH_DAY_NANO";
  })(IntervalUnit || (IntervalUnit = {}));
  const SIZEOF_SHORT = 2, SIZEOF_INT = 4, FILE_IDENTIFIER_LENGTH = 4, SIZE_PREFIX_LENGTH = 4, int32 = new Int32Array(2), float32 = new Float32Array(int32.buffer), float64 = new Float64Array(int32.buffer), isLittleEndian = new Uint16Array(new Uint8Array([1, 0]).buffer)[0] === 1;
  var Encoding;
  (function(O) {
    O[O.UTF8_BYTES = 1] = "UTF8_BYTES", O[O.UTF16_STRING = 2] = "UTF16_STRING";
  })(Encoding || (Encoding = {}));
  let ByteBuffer$2 = class t7 {
    /**
     * Create a new ByteBuffer with a given array of bytes (`Uint8Array`)
     */
    constructor(_) {
      this.bytes_ = _, this.position_ = 0, this.text_decoder_ = new TextDecoder();
    }
    /**
     * Create and allocate a new ByteBuffer with a given size.
     */
    static allocate(_) {
      return new t7(new Uint8Array(_));
    }
    clear() {
      this.position_ = 0;
    }
    /**
     * Get the underlying `Uint8Array`.
     */
    bytes() {
      return this.bytes_;
    }
    /**
     * Get the buffer's position.
     */
    position() {
      return this.position_;
    }
    /**
     * Set the buffer's position.
     */
    setPosition(_) {
      this.position_ = _;
    }
    /**
     * Get the buffer's capacity.
     */
    capacity() {
      return this.bytes_.length;
    }
    readInt8(_) {
      return this.readUint8(_) << 24 >> 24;
    }
    readUint8(_) {
      return this.bytes_[_];
    }
    readInt16(_) {
      return this.readUint16(_) << 16 >> 16;
    }
    readUint16(_) {
      return this.bytes_[_] | this.bytes_[_ + 1] << 8;
    }
    readInt32(_) {
      return this.bytes_[_] | this.bytes_[_ + 1] << 8 | this.bytes_[_ + 2] << 16 | this.bytes_[_ + 3] << 24;
    }
    readUint32(_) {
      return this.readInt32(_) >>> 0;
    }
    readInt64(_) {
      return BigInt.asIntN(64, BigInt(this.readUint32(_)) + (BigInt(this.readUint32(_ + 4)) << BigInt(32)));
    }
    readUint64(_) {
      return BigInt.asUintN(64, BigInt(this.readUint32(_)) + (BigInt(this.readUint32(_ + 4)) << BigInt(32)));
    }
    readFloat32(_) {
      return int32[0] = this.readInt32(_), float32[0];
    }
    readFloat64(_) {
      return int32[isLittleEndian ? 0 : 1] = this.readInt32(_), int32[isLittleEndian ? 1 : 0] = this.readInt32(_ + 4), float64[0];
    }
    writeInt8(_, H) {
      this.bytes_[_] = H;
    }
    writeUint8(_, H) {
      this.bytes_[_] = H;
    }
    writeInt16(_, H) {
      this.bytes_[_] = H, this.bytes_[_ + 1] = H >> 8;
    }
    writeUint16(_, H) {
      this.bytes_[_] = H, this.bytes_[_ + 1] = H >> 8;
    }
    writeInt32(_, H) {
      this.bytes_[_] = H, this.bytes_[_ + 1] = H >> 8, this.bytes_[_ + 2] = H >> 16, this.bytes_[_ + 3] = H >> 24;
    }
    writeUint32(_, H) {
      this.bytes_[_] = H, this.bytes_[_ + 1] = H >> 8, this.bytes_[_ + 2] = H >> 16, this.bytes_[_ + 3] = H >> 24;
    }
    writeInt64(_, H) {
      this.writeInt32(_, Number(BigInt.asIntN(32, H))), this.writeInt32(_ + 4, Number(BigInt.asIntN(32, H >> BigInt(32))));
    }
    writeUint64(_, H) {
      this.writeUint32(_, Number(BigInt.asUintN(32, H))), this.writeUint32(_ + 4, Number(BigInt.asUintN(32, H >> BigInt(32))));
    }
    writeFloat32(_, H) {
      float32[0] = H, this.writeInt32(_, int32[0]);
    }
    writeFloat64(_, H) {
      float64[0] = H, this.writeInt32(_, int32[isLittleEndian ? 0 : 1]), this.writeInt32(_ + 4, int32[isLittleEndian ? 1 : 0]);
    }
    /**
     * Return the file identifier.   Behavior is undefined for FlatBuffers whose
     * schema does not include a file_identifier (likely points at padding or the
     * start of a the root vtable).
     */
    getBufferIdentifier() {
      if (this.bytes_.length < this.position_ + SIZEOF_INT + FILE_IDENTIFIER_LENGTH)
        throw new Error("FlatBuffers: ByteBuffer is too short to contain an identifier.");
      let _ = "";
      for (let H = 0; H < FILE_IDENTIFIER_LENGTH; H++)
        _ += String.fromCharCode(this.readInt8(this.position_ + SIZEOF_INT + H));
      return _;
    }
    /**
     * Look up a field in the vtable, return an offset into the object, or 0 if the
     * field is not present.
     */
    __offset(_, H) {
      const V = _ - this.readInt32(_);
      return H < this.readInt16(V) ? this.readInt16(V + H) : 0;
    }
    /**
     * Initialize any Table-derived type to point to the union at the given offset.
     */
    __union(_, H) {
      return _.bb_pos = H + this.readInt32(H), _.bb = this, _;
    }
    /**
     * Create a JavaScript string from UTF-8 data stored inside the FlatBuffer.
     * This allocates a new string and converts to wide chars upon each access.
     *
     * To avoid the conversion to string, pass Encoding.UTF8_BYTES as the
     * "optionalEncoding" argument. This is useful for avoiding conversion when
     * the data will just be packaged back up in another FlatBuffer later on.
     *
     * @param offset
     * @param opt_encoding Defaults to UTF16_STRING
     */
    __string(_, H) {
      _ += this.readInt32(_);
      const V = this.readInt32(_);
      _ += SIZEOF_INT;
      const q = this.bytes_.subarray(_, _ + V);
      return H === Encoding.UTF8_BYTES ? q : this.text_decoder_.decode(q);
    }
    /**
     * Handle unions that can contain string as its member, if a Table-derived type then initialize it,
     * if a string then return a new one
     *
     * WARNING: strings are immutable in JS so we can't change the string that the user gave us, this
     * makes the behaviour of __union_with_string different compared to __union
     */
    __union_with_string(_, H) {
      return typeof _ == "string" ? this.__string(H) : this.__union(_, H);
    }
    /**
     * Retrieve the relative offset stored at "offset"
     */
    __indirect(_) {
      return _ + this.readInt32(_);
    }
    /**
     * Get the start of data of a vector whose offset is stored at "offset" in this object.
     */
    __vector(_) {
      return _ + this.readInt32(_) + SIZEOF_INT;
    }
    /**
     * Get the length of a vector whose offset is stored at "offset" in this object.
     */
    __vector_len(_) {
      return this.readInt32(_ + this.readInt32(_));
    }
    __has_identifier(_) {
      if (_.length != FILE_IDENTIFIER_LENGTH)
        throw new Error("FlatBuffers: file identifier must be length " + FILE_IDENTIFIER_LENGTH);
      for (let H = 0; H < FILE_IDENTIFIER_LENGTH; H++)
        if (_.charCodeAt(H) != this.readInt8(this.position() + SIZEOF_INT + H))
          return !1;
      return !0;
    }
    /**
     * A helper function for generating list for obj api
     */
    createScalarList(_, H) {
      const V = [];
      for (let q = 0; q < H; ++q) {
        const G = _(q);
        G !== null && V.push(G);
      }
      return V;
    }
    /**
     * A helper function for generating list for obj api
     * @param listAccessor function that accepts an index and return data at that index
     * @param listLength listLength
     * @param res result list
     */
    createObjList(_, H) {
      const V = [];
      for (let q = 0; q < H; ++q) {
        const G = _(q);
        G !== null && V.push(G.unpack());
      }
      return V;
    }
  }, Builder$2 = class n7 {
    /**
     * Create a FlatBufferBuilder.
     */
    constructor(_) {
      this.minalign = 1, this.vtable = null, this.vtable_in_use = 0, this.isNested = !1, this.object_start = 0, this.vtables = [], this.vector_num_elems = 0, this.force_defaults = !1, this.string_maps = null, this.text_encoder = new TextEncoder();
      let H;
      _ ? H = _ : H = 1024, this.bb = ByteBuffer$2.allocate(H), this.space = H;
    }
    clear() {
      this.bb.clear(), this.space = this.bb.capacity(), this.minalign = 1, this.vtable = null, this.vtable_in_use = 0, this.isNested = !1, this.object_start = 0, this.vtables = [], this.vector_num_elems = 0, this.force_defaults = !1, this.string_maps = null;
    }
    /**
     * In order to save space, fields that are set to their default value
     * don't get serialized into the buffer. Forcing defaults provides a
     * way to manually disable this optimization.
     *
     * @param forceDefaults true always serializes default values
     */
    forceDefaults(_) {
      this.force_defaults = _;
    }
    /**
     * Get the ByteBuffer representing the FlatBuffer. Only call this after you've
     * called finish(). The actual data starts at the ByteBuffer's current position,
     * not necessarily at 0.
     */
    dataBuffer() {
      return this.bb;
    }
    /**
     * Get the bytes representing the FlatBuffer. Only call this after you've
     * called finish().
     */
    asUint8Array() {
      return this.bb.bytes().subarray(this.bb.position(), this.bb.position() + this.offset());
    }
    /**
     * Prepare to write an element of `size` after `additional_bytes` have been
     * written, e.g. if you write a string, you need to align such the int length
     * field is aligned to 4 bytes, and the string data follows it directly. If all
     * you need to do is alignment, `additional_bytes` will be 0.
     *
     * @param size This is the of the new element to write
     * @param additional_bytes The padding size
     */
    prep(_, H) {
      _ > this.minalign && (this.minalign = _);
      const V = ~(this.bb.capacity() - this.space + H) + 1 & _ - 1;
      for (; this.space < V + _ + H; ) {
        const q = this.bb.capacity();
        this.bb = n7.growByteBuffer(this.bb), this.space += this.bb.capacity() - q;
      }
      this.pad(V);
    }
    pad(_) {
      for (let H = 0; H < _; H++)
        this.bb.writeInt8(--this.space, 0);
    }
    writeInt8(_) {
      this.bb.writeInt8(this.space -= 1, _);
    }
    writeInt16(_) {
      this.bb.writeInt16(this.space -= 2, _);
    }
    writeInt32(_) {
      this.bb.writeInt32(this.space -= 4, _);
    }
    writeInt64(_) {
      this.bb.writeInt64(this.space -= 8, _);
    }
    writeFloat32(_) {
      this.bb.writeFloat32(this.space -= 4, _);
    }
    writeFloat64(_) {
      this.bb.writeFloat64(this.space -= 8, _);
    }
    /**
     * Add an `int8` to the buffer, properly aligned, and grows the buffer (if necessary).
     * @param value The `int8` to add the buffer.
     */
    addInt8(_) {
      this.prep(1, 0), this.writeInt8(_);
    }
    /**
     * Add an `int16` to the buffer, properly aligned, and grows the buffer (if necessary).
     * @param value The `int16` to add the buffer.
     */
    addInt16(_) {
      this.prep(2, 0), this.writeInt16(_);
    }
    /**
     * Add an `int32` to the buffer, properly aligned, and grows the buffer (if necessary).
     * @param value The `int32` to add the buffer.
     */
    addInt32(_) {
      this.prep(4, 0), this.writeInt32(_);
    }
    /**
     * Add an `int64` to the buffer, properly aligned, and grows the buffer (if necessary).
     * @param value The `int64` to add the buffer.
     */
    addInt64(_) {
      this.prep(8, 0), this.writeInt64(_);
    }
    /**
     * Add a `float32` to the buffer, properly aligned, and grows the buffer (if necessary).
     * @param value The `float32` to add the buffer.
     */
    addFloat32(_) {
      this.prep(4, 0), this.writeFloat32(_);
    }
    /**
     * Add a `float64` to the buffer, properly aligned, and grows the buffer (if necessary).
     * @param value The `float64` to add the buffer.
     */
    addFloat64(_) {
      this.prep(8, 0), this.writeFloat64(_);
    }
    addFieldInt8(_, H, V) {
      (this.force_defaults || H != V) && (this.addInt8(H), this.slot(_));
    }
    addFieldInt16(_, H, V) {
      (this.force_defaults || H != V) && (this.addInt16(H), this.slot(_));
    }
    addFieldInt32(_, H, V) {
      (this.force_defaults || H != V) && (this.addInt32(H), this.slot(_));
    }
    addFieldInt64(_, H, V) {
      (this.force_defaults || H !== V) && (this.addInt64(H), this.slot(_));
    }
    addFieldFloat32(_, H, V) {
      (this.force_defaults || H != V) && (this.addFloat32(H), this.slot(_));
    }
    addFieldFloat64(_, H, V) {
      (this.force_defaults || H != V) && (this.addFloat64(H), this.slot(_));
    }
    addFieldOffset(_, H, V) {
      (this.force_defaults || H != V) && (this.addOffset(H), this.slot(_));
    }
    /**
     * Structs are stored inline, so nothing additional is being added. `d` is always 0.
     */
    addFieldStruct(_, H, V) {
      H != V && (this.nested(H), this.slot(_));
    }
    /**
     * Structures are always stored inline, they need to be created right
     * where they're used.  You'll get this assertion failure if you
     * created it elsewhere.
     */
    nested(_) {
      if (_ != this.offset())
        throw new TypeError("FlatBuffers: struct must be serialized inline.");
    }
    /**
     * Should not be creating any other object, string or vector
     * while an object is being constructed
     */
    notNested() {
      if (this.isNested)
        throw new TypeError("FlatBuffers: object serialization must not be nested.");
    }
    /**
     * Set the current vtable at `voffset` to the current location in the buffer.
     */
    slot(_) {
      this.vtable !== null && (this.vtable[_] = this.offset());
    }
    /**
     * @returns Offset relative to the end of the buffer.
     */
    offset() {
      return this.bb.capacity() - this.space;
    }
    /**
     * Doubles the size of the backing ByteBuffer and copies the old data towards
     * the end of the new buffer (since we build the buffer backwards).
     *
     * @param bb The current buffer with the existing data
     * @returns A new byte buffer with the old data copied
     * to it. The data is located at the end of the buffer.
     *
     * uint8Array.set() formally takes {Array<number>|ArrayBufferView}, so to pass
     * it a uint8Array we need to suppress the type check:
     * @suppress {checkTypes}
     */
    static growByteBuffer(_) {
      const H = _.capacity();
      if (H & 3221225472)
        throw new Error("FlatBuffers: cannot grow buffer beyond 2 gigabytes.");
      const V = H << 1, q = ByteBuffer$2.allocate(V);
      return q.setPosition(V - H), q.bytes().set(_.bytes(), V - H), q;
    }
    /**
     * Adds on offset, relative to where it will be written.
     *
     * @param offset The offset to add.
     */
    addOffset(_) {
      this.prep(SIZEOF_INT, 0), this.writeInt32(this.offset() - _ + SIZEOF_INT);
    }
    /**
     * Start encoding a new object in the buffer.  Users will not usually need to
     * call this directly. The FlatBuffers compiler will generate helper methods
     * that call this method internally.
     */
    startObject(_) {
      this.notNested(), this.vtable == null && (this.vtable = []), this.vtable_in_use = _;
      for (let H = 0; H < _; H++)
        this.vtable[H] = 0;
      this.isNested = !0, this.object_start = this.offset();
    }
    /**
     * Finish off writing the object that is under construction.
     *
     * @returns The offset to the object inside `dataBuffer`
     */
    endObject() {
      if (this.vtable == null || !this.isNested)
        throw new Error("FlatBuffers: endObject called without startObject");
      this.addInt32(0);
      const _ = this.offset();
      let H = this.vtable_in_use - 1;
      for (; H >= 0 && this.vtable[H] == 0; H--)
        ;
      const V = H + 1;
      for (; H >= 0; H--)
        this.addInt16(this.vtable[H] != 0 ? _ - this.vtable[H] : 0);
      const q = 2;
      this.addInt16(_ - this.object_start);
      const G = (V + q) * SIZEOF_SHORT;
      this.addInt16(G);
      let J = 0;
      const Z = this.space;
      e: for (H = 0; H < this.vtables.length; H++) {
        const te = this.bb.capacity() - this.vtables[H];
        if (G == this.bb.readInt16(te)) {
          for (let ne = SIZEOF_SHORT; ne < G; ne += SIZEOF_SHORT)
            if (this.bb.readInt16(Z + ne) != this.bb.readInt16(te + ne))
              continue e;
          J = this.vtables[H];
          break;
        }
      }
      return J ? (this.space = this.bb.capacity() - _, this.bb.writeInt32(this.space, J - _)) : (this.vtables.push(this.offset()), this.bb.writeInt32(this.bb.capacity() - _, this.offset() - _)), this.isNested = !1, _;
    }
    /**
     * Finalize a buffer, poiting to the given `root_table`.
     */
    finish(_, H, V) {
      const q = V ? SIZE_PREFIX_LENGTH : 0;
      if (H) {
        const G = H;
        if (this.prep(this.minalign, SIZEOF_INT + FILE_IDENTIFIER_LENGTH + q), G.length != FILE_IDENTIFIER_LENGTH)
          throw new TypeError("FlatBuffers: file identifier must be length " + FILE_IDENTIFIER_LENGTH);
        for (let J = FILE_IDENTIFIER_LENGTH - 1; J >= 0; J--)
          this.writeInt8(G.charCodeAt(J));
      }
      this.prep(this.minalign, SIZEOF_INT + q), this.addOffset(_), q && this.addInt32(this.bb.capacity() - this.space), this.bb.setPosition(this.space);
    }
    /**
     * Finalize a size prefixed buffer, pointing to the given `root_table`.
     */
    finishSizePrefixed(_, H) {
      this.finish(_, H, !0);
    }
    /**
     * This checks a required field has been set in a given table that has
     * just been constructed.
     */
    requiredField(_, H) {
      const V = this.bb.capacity() - _, q = V - this.bb.readInt32(V);
      if (!(H < this.bb.readInt16(q) && this.bb.readInt16(q + H) != 0))
        throw new TypeError("FlatBuffers: field " + H + " must be set");
    }
    /**
     * Start a new array/vector of objects.  Users usually will not call
     * this directly. The FlatBuffers compiler will create a start/end
     * method for vector types in generated code.
     *
     * @param elem_size The size of each element in the array
     * @param num_elems The number of elements in the array
     * @param alignment The alignment of the array
     */
    startVector(_, H, V) {
      this.notNested(), this.vector_num_elems = H, this.prep(SIZEOF_INT, _ * H), this.prep(V, _ * H);
    }
    /**
     * Finish off the creation of an array and all its elements. The array must be
     * created with `startVector`.
     *
     * @returns The offset at which the newly created array
     * starts.
     */
    endVector() {
      return this.writeInt32(this.vector_num_elems), this.offset();
    }
    /**
     * Encode the string `s` in the buffer using UTF-8. If the string passed has
     * already been seen, we return the offset of the already written string
     *
     * @param s The string to encode
     * @return The offset in the buffer where the encoded string starts
     */
    createSharedString(_) {
      if (!_)
        return 0;
      if (this.string_maps || (this.string_maps = /* @__PURE__ */ new Map()), this.string_maps.has(_))
        return this.string_maps.get(_);
      const H = this.createString(_);
      return this.string_maps.set(_, H), H;
    }
    /**
     * Encode the string `s` in the buffer using UTF-8. If a Uint8Array is passed
     * instead of a string, it is assumed to contain valid UTF-8 encoded data.
     *
     * @param s The string to encode
     * @return The offset in the buffer where the encoded string starts
     */
    createString(_) {
      if (_ == null)
        return 0;
      let H;
      return _ instanceof Uint8Array ? H = _ : H = this.text_encoder.encode(_), this.addInt8(0), this.startVector(1, H.length, 1), this.bb.setPosition(this.space -= H.length), this.bb.bytes().set(H, this.space), this.endVector();
    }
    /**
     * Create a byte vector.
     *
     * @param v The bytes to add
     * @returns The offset in the buffer where the byte vector starts
     */
    createByteVector(_) {
      return _ == null ? 0 : (this.startVector(1, _.length, 1), this.bb.setPosition(this.space -= _.length), this.bb.bytes().set(_, this.space), this.endVector());
    }
    /**
     * A helper function to pack an object
     *
     * @returns offset of obj
     */
    createObjectOffset(_) {
      return _ === null ? 0 : typeof _ == "string" ? this.createString(_) : _.pack(this);
    }
    /**
     * A helper function to pack a list of object
     *
     * @returns list of offsets of each non null object
     */
    createObjectOffsetList(_) {
      const H = [];
      for (let V = 0; V < _.length; ++V) {
        const q = _[V];
        if (q !== null)
          H.push(this.createObjectOffset(q));
        else
          throw new TypeError("FlatBuffers: Argument for createObjectOffsetList cannot contain null.");
      }
      return H;
    }
    createStructOffsetList(_, H) {
      return H(this, _.length), this.createObjectOffsetList(_.slice().reverse()), this.endVector();
    }
  };
  var BodyCompressionMethod;
  (function(O) {
    O[O.BUFFER = 0] = "BUFFER";
  })(BodyCompressionMethod || (BodyCompressionMethod = {}));
  var CompressionType;
  (function(O) {
    O[O.LZ4_FRAME = 0] = "LZ4_FRAME", O[O.ZSTD = 1] = "ZSTD";
  })(CompressionType || (CompressionType = {}));
  class BodyCompression {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsBodyCompression(_, H) {
      return (H || new BodyCompression()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsBodyCompression(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new BodyCompression()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * Compressor library.
     * For LZ4_FRAME, each compressed buffer must consist of a single frame.
     */
    codec() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt8(this.bb_pos + _) : CompressionType.LZ4_FRAME;
    }
    /**
     * Indicates the way the record batch body was compressed
     */
    method() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? this.bb.readInt8(this.bb_pos + _) : BodyCompressionMethod.BUFFER;
    }
    static startBodyCompression(_) {
      _.startObject(2);
    }
    static addCodec(_, H) {
      _.addFieldInt8(0, H, CompressionType.LZ4_FRAME);
    }
    static addMethod(_, H) {
      _.addFieldInt8(1, H, BodyCompressionMethod.BUFFER);
    }
    static endBodyCompression(_) {
      return _.endObject();
    }
    static createBodyCompression(_, H, V) {
      return BodyCompression.startBodyCompression(_), BodyCompression.addCodec(_, H), BodyCompression.addMethod(_, V), BodyCompression.endBodyCompression(_);
    }
  }
  let Buffer$1 = class {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    /**
     * The relative offset into the shared memory page where the bytes for this
     * buffer starts
     */
    offset() {
      return this.bb.readInt64(this.bb_pos);
    }
    /**
     * The absolute length (in bytes) of the memory buffer. The memory is found
     * from offset (inclusive) to offset + length (non-inclusive). When building
     * messages using the encapsulated IPC message, padding bytes may be written
     * after a buffer, but such padding bytes do not need to be accounted for in
     * the size here.
     */
    length() {
      return this.bb.readInt64(this.bb_pos + 8);
    }
    static sizeOf() {
      return 16;
    }
    static createBuffer(_, H, V) {
      return _.prep(8, 16), _.writeInt64(BigInt(V ?? 0)), _.writeInt64(BigInt(H ?? 0)), _.offset();
    }
  }, FieldNode$1 = class {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    /**
     * The number of value slots in the Arrow array at this level of a nested
     * tree
     */
    length() {
      return this.bb.readInt64(this.bb_pos);
    }
    /**
     * The number of observed nulls. Fields with null_count == 0 may choose not
     * to write their physical validity bitmap out as a materialized buffer,
     * instead setting the length of the bitmap buffer to 0.
     */
    nullCount() {
      return this.bb.readInt64(this.bb_pos + 8);
    }
    static sizeOf() {
      return 16;
    }
    static createFieldNode(_, H, V) {
      return _.prep(8, 16), _.writeInt64(BigInt(V ?? 0)), _.writeInt64(BigInt(H ?? 0)), _.offset();
    }
  }, RecordBatch$2 = class fy {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsRecordBatch(_, H) {
      return (H || new fy()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsRecordBatch(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new fy()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * number of records / rows. The arrays in the batch should all have this
     * length
     */
    length() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt64(this.bb_pos + _) : BigInt("0");
    }
    /**
     * Nodes correspond to the pre-ordered flattened logical schema
     */
    nodes(_, H) {
      const V = this.bb.__offset(this.bb_pos, 6);
      return V ? (H || new FieldNode$1()).__init(this.bb.__vector(this.bb_pos + V) + _ * 16, this.bb) : null;
    }
    nodesLength() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    /**
     * Buffers correspond to the pre-ordered flattened buffer tree
     *
     * The number of buffers appended to this list depends on the schema. For
     * example, most primitive arrays will have 2 buffers, 1 for the validity
     * bitmap and 1 for the values. For struct arrays, there will only be a
     * single buffer for the validity (nulls) bitmap
     */
    buffers(_, H) {
      const V = this.bb.__offset(this.bb_pos, 8);
      return V ? (H || new Buffer$1()).__init(this.bb.__vector(this.bb_pos + V) + _ * 16, this.bb) : null;
    }
    buffersLength() {
      const _ = this.bb.__offset(this.bb_pos, 8);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    /**
     * Optional compression of the message body
     */
    compression(_) {
      const H = this.bb.__offset(this.bb_pos, 10);
      return H ? (_ || new BodyCompression()).__init(this.bb.__indirect(this.bb_pos + H), this.bb) : null;
    }
    static startRecordBatch(_) {
      _.startObject(4);
    }
    static addLength(_, H) {
      _.addFieldInt64(0, H, BigInt("0"));
    }
    static addNodes(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static startNodesVector(_, H) {
      _.startVector(16, H, 8);
    }
    static addBuffers(_, H) {
      _.addFieldOffset(2, H, 0);
    }
    static startBuffersVector(_, H) {
      _.startVector(16, H, 8);
    }
    static addCompression(_, H) {
      _.addFieldOffset(3, H, 0);
    }
    static endRecordBatch(_) {
      return _.endObject();
    }
  }, DictionaryBatch$1 = class hy {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsDictionaryBatch(_, H) {
      return (H || new hy()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsDictionaryBatch(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new hy()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    id() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt64(this.bb_pos + _) : BigInt("0");
    }
    data(_) {
      const H = this.bb.__offset(this.bb_pos, 6);
      return H ? (_ || new RecordBatch$2()).__init(this.bb.__indirect(this.bb_pos + H), this.bb) : null;
    }
    /**
     * If isDelta is true the values in the dictionary are to be appended to a
     * dictionary with the indicated id. If isDelta is false this dictionary
     * should replace the existing dictionary.
     */
    isDelta() {
      const _ = this.bb.__offset(this.bb_pos, 8);
      return _ ? !!this.bb.readInt8(this.bb_pos + _) : !1;
    }
    static startDictionaryBatch(_) {
      _.startObject(3);
    }
    static addId(_, H) {
      _.addFieldInt64(0, H, BigInt("0"));
    }
    static addData(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static addIsDelta(_, H) {
      _.addFieldInt8(2, +H, 0);
    }
    static endDictionaryBatch(_) {
      return _.endObject();
    }
  };
  var Endianness;
  (function(O) {
    O[O.Little = 0] = "Little", O[O.Big = 1] = "Big";
  })(Endianness || (Endianness = {}));
  var DictionaryKind;
  (function(O) {
    O[O.DenseArray = 0] = "DenseArray";
  })(DictionaryKind || (DictionaryKind = {}));
  class Int {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsInt(_, H) {
      return (H || new Int()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsInt(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Int()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    bitWidth() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt32(this.bb_pos + _) : 0;
    }
    isSigned() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? !!this.bb.readInt8(this.bb_pos + _) : !1;
    }
    static startInt(_) {
      _.startObject(2);
    }
    static addBitWidth(_, H) {
      _.addFieldInt32(0, H, 0);
    }
    static addIsSigned(_, H) {
      _.addFieldInt8(1, +H, 0);
    }
    static endInt(_) {
      return _.endObject();
    }
    static createInt(_, H, V) {
      return Int.startInt(_), Int.addBitWidth(_, H), Int.addIsSigned(_, V), Int.endInt(_);
    }
  }
  class DictionaryEncoding {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsDictionaryEncoding(_, H) {
      return (H || new DictionaryEncoding()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsDictionaryEncoding(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new DictionaryEncoding()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * The known dictionary id in the application where this data is used. In
     * the file or streaming formats, the dictionary ids are found in the
     * DictionaryBatch messages
     */
    id() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt64(this.bb_pos + _) : BigInt("0");
    }
    /**
     * The dictionary indices are constrained to be non-negative integers. If
     * this field is null, the indices must be signed int32. To maximize
     * cross-language compatibility and performance, implementations are
     * recommended to prefer signed integer types over unsigned integer types
     * and to avoid uint64 indices unless they are required by an application.
     */
    indexType(_) {
      const H = this.bb.__offset(this.bb_pos, 6);
      return H ? (_ || new Int()).__init(this.bb.__indirect(this.bb_pos + H), this.bb) : null;
    }
    /**
     * By default, dictionaries are not ordered, or the order does not have
     * semantic meaning. In some statistical, applications, dictionary-encoding
     * is used to represent ordered categorical data, and we provide a way to
     * preserve that metadata here
     */
    isOrdered() {
      const _ = this.bb.__offset(this.bb_pos, 8);
      return _ ? !!this.bb.readInt8(this.bb_pos + _) : !1;
    }
    dictionaryKind() {
      const _ = this.bb.__offset(this.bb_pos, 10);
      return _ ? this.bb.readInt16(this.bb_pos + _) : DictionaryKind.DenseArray;
    }
    static startDictionaryEncoding(_) {
      _.startObject(4);
    }
    static addId(_, H) {
      _.addFieldInt64(0, H, BigInt("0"));
    }
    static addIndexType(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static addIsOrdered(_, H) {
      _.addFieldInt8(2, +H, 0);
    }
    static addDictionaryKind(_, H) {
      _.addFieldInt16(3, H, DictionaryKind.DenseArray);
    }
    static endDictionaryEncoding(_) {
      return _.endObject();
    }
  }
  class KeyValue {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsKeyValue(_, H) {
      return (H || new KeyValue()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsKeyValue(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new KeyValue()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    key(_) {
      const H = this.bb.__offset(this.bb_pos, 4);
      return H ? this.bb.__string(this.bb_pos + H, _) : null;
    }
    value(_) {
      const H = this.bb.__offset(this.bb_pos, 6);
      return H ? this.bb.__string(this.bb_pos + H, _) : null;
    }
    static startKeyValue(_) {
      _.startObject(2);
    }
    static addKey(_, H) {
      _.addFieldOffset(0, H, 0);
    }
    static addValue(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static endKeyValue(_) {
      return _.endObject();
    }
    static createKeyValue(_, H, V) {
      return KeyValue.startKeyValue(_), KeyValue.addKey(_, H), KeyValue.addValue(_, V), KeyValue.endKeyValue(_);
    }
  }
  let Binary$1 = class v3 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsBinary(_, H) {
      return (H || new v3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsBinary(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new v3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startBinary(_) {
      _.startObject(0);
    }
    static endBinary(_) {
      return _.endObject();
    }
    static createBinary(_) {
      return v3.startBinary(_), v3.endBinary(_);
    }
  }, Bool$1 = class b3 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsBool(_, H) {
      return (H || new b3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsBool(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new b3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startBool(_) {
      _.startObject(0);
    }
    static endBool(_) {
      return _.endObject();
    }
    static createBool(_) {
      return b3.startBool(_), b3.endBool(_);
    }
  }, Date$1 = class sp {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsDate(_, H) {
      return (H || new sp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsDate(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new sp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    unit() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : DateUnit.MILLISECOND;
    }
    static startDate(_) {
      _.startObject(1);
    }
    static addUnit(_, H) {
      _.addFieldInt16(0, H, DateUnit.MILLISECOND);
    }
    static endDate(_) {
      return _.endObject();
    }
    static createDate(_, H) {
      return sp.startDate(_), sp.addUnit(_, H), sp.endDate(_);
    }
  }, Decimal$1 = class b2 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsDecimal(_, H) {
      return (H || new b2()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsDecimal(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new b2()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * Total number of decimal digits
     */
    precision() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt32(this.bb_pos + _) : 0;
    }
    /**
     * Number of digits after the decimal point "."
     */
    scale() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? this.bb.readInt32(this.bb_pos + _) : 0;
    }
    /**
     * Number of bits per value. The only accepted widths are 128 and 256.
     * We use bitWidth for consistency with Int::bitWidth.
     */
    bitWidth() {
      const _ = this.bb.__offset(this.bb_pos, 8);
      return _ ? this.bb.readInt32(this.bb_pos + _) : 128;
    }
    static startDecimal(_) {
      _.startObject(3);
    }
    static addPrecision(_, H) {
      _.addFieldInt32(0, H, 0);
    }
    static addScale(_, H) {
      _.addFieldInt32(1, H, 0);
    }
    static addBitWidth(_, H) {
      _.addFieldInt32(2, H, 128);
    }
    static endDecimal(_) {
      return _.endObject();
    }
    static createDecimal(_, H, V, q) {
      return b2.startDecimal(_), b2.addPrecision(_, H), b2.addScale(_, V), b2.addBitWidth(_, q), b2.endDecimal(_);
    }
  }, Duration$1 = class lp {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsDuration(_, H) {
      return (H || new lp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsDuration(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new lp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    unit() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : TimeUnit.MILLISECOND;
    }
    static startDuration(_) {
      _.startObject(1);
    }
    static addUnit(_, H) {
      _.addFieldInt16(0, H, TimeUnit.MILLISECOND);
    }
    static endDuration(_) {
      return _.endObject();
    }
    static createDuration(_, H) {
      return lp.startDuration(_), lp.addUnit(_, H), lp.endDuration(_);
    }
  }, FixedSizeBinary$1 = class cp {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsFixedSizeBinary(_, H) {
      return (H || new cp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsFixedSizeBinary(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new cp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * Number of bytes per value
     */
    byteWidth() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt32(this.bb_pos + _) : 0;
    }
    static startFixedSizeBinary(_) {
      _.startObject(1);
    }
    static addByteWidth(_, H) {
      _.addFieldInt32(0, H, 0);
    }
    static endFixedSizeBinary(_) {
      return _.endObject();
    }
    static createFixedSizeBinary(_, H) {
      return cp.startFixedSizeBinary(_), cp.addByteWidth(_, H), cp.endFixedSizeBinary(_);
    }
  }, FixedSizeList$1 = class up {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsFixedSizeList(_, H) {
      return (H || new up()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsFixedSizeList(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new up()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * Number of list items per value
     */
    listSize() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt32(this.bb_pos + _) : 0;
    }
    static startFixedSizeList(_) {
      _.startObject(1);
    }
    static addListSize(_, H) {
      _.addFieldInt32(0, H, 0);
    }
    static endFixedSizeList(_) {
      return _.endObject();
    }
    static createFixedSizeList(_, H) {
      return up.startFixedSizeList(_), up.addListSize(_, H), up.endFixedSizeList(_);
    }
  };
  class FloatingPoint {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsFloatingPoint(_, H) {
      return (H || new FloatingPoint()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsFloatingPoint(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new FloatingPoint()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    precision() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : Precision.HALF;
    }
    static startFloatingPoint(_) {
      _.startObject(1);
    }
    static addPrecision(_, H) {
      _.addFieldInt16(0, H, Precision.HALF);
    }
    static endFloatingPoint(_) {
      return _.endObject();
    }
    static createFloatingPoint(_, H) {
      return FloatingPoint.startFloatingPoint(_), FloatingPoint.addPrecision(_, H), FloatingPoint.endFloatingPoint(_);
    }
  }
  class Interval {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsInterval(_, H) {
      return (H || new Interval()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsInterval(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Interval()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    unit() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : IntervalUnit.YEAR_MONTH;
    }
    static startInterval(_) {
      _.startObject(1);
    }
    static addUnit(_, H) {
      _.addFieldInt16(0, H, IntervalUnit.YEAR_MONTH);
    }
    static endInterval(_) {
      return _.endObject();
    }
    static createInterval(_, H) {
      return Interval.startInterval(_), Interval.addUnit(_, H), Interval.endInterval(_);
    }
  }
  let LargeBinary$1 = class T3 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsLargeBinary(_, H) {
      return (H || new T3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsLargeBinary(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new T3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startLargeBinary(_) {
      _.startObject(0);
    }
    static endLargeBinary(_) {
      return _.endObject();
    }
    static createLargeBinary(_) {
      return T3.startLargeBinary(_), T3.endLargeBinary(_);
    }
  }, LargeUtf8$1 = class x3 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsLargeUtf8(_, H) {
      return (H || new x3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsLargeUtf8(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new x3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startLargeUtf8(_) {
      _.startObject(0);
    }
    static endLargeUtf8(_) {
      return _.endObject();
    }
    static createLargeUtf8(_) {
      return x3.startLargeUtf8(_), x3.endLargeUtf8(_);
    }
  }, List$1 = class w3 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsList(_, H) {
      return (H || new w3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsList(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new w3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startList(_) {
      _.startObject(0);
    }
    static endList(_) {
      return _.endObject();
    }
    static createList(_) {
      return w3.startList(_), w3.endList(_);
    }
  }, Map$1 = class dp {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsMap(_, H) {
      return (H || new dp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsMap(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new dp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * Set to true if the keys within each value are sorted
     */
    keysSorted() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? !!this.bb.readInt8(this.bb_pos + _) : !1;
    }
    static startMap(_) {
      _.startObject(1);
    }
    static addKeysSorted(_, H) {
      _.addFieldInt8(0, +H, 0);
    }
    static endMap(_) {
      return _.endObject();
    }
    static createMap(_, H) {
      return dp.startMap(_), dp.addKeysSorted(_, H), dp.endMap(_);
    }
  }, Null$1 = class S3 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsNull(_, H) {
      return (H || new S3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsNull(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new S3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startNull(_) {
      _.startObject(0);
    }
    static endNull(_) {
      return _.endObject();
    }
    static createNull(_) {
      return S3.startNull(_), S3.endNull(_);
    }
  };
  class Struct_ {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsStruct_(_, H) {
      return (H || new Struct_()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsStruct_(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Struct_()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startStruct_(_) {
      _.startObject(0);
    }
    static endStruct_(_) {
      return _.endObject();
    }
    static createStruct_(_) {
      return Struct_.startStruct_(_), Struct_.endStruct_(_);
    }
  }
  class Time {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsTime(_, H) {
      return (H || new Time()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsTime(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Time()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    unit() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : TimeUnit.MILLISECOND;
    }
    bitWidth() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? this.bb.readInt32(this.bb_pos + _) : 32;
    }
    static startTime(_) {
      _.startObject(2);
    }
    static addUnit(_, H) {
      _.addFieldInt16(0, H, TimeUnit.MILLISECOND);
    }
    static addBitWidth(_, H) {
      _.addFieldInt32(1, H, 32);
    }
    static endTime(_) {
      return _.endObject();
    }
    static createTime(_, H, V) {
      return Time.startTime(_), Time.addUnit(_, H), Time.addBitWidth(_, V), Time.endTime(_);
    }
  }
  class Timestamp {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsTimestamp(_, H) {
      return (H || new Timestamp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsTimestamp(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Timestamp()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    unit() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : TimeUnit.SECOND;
    }
    timezone(_) {
      const H = this.bb.__offset(this.bb_pos, 6);
      return H ? this.bb.__string(this.bb_pos + H, _) : null;
    }
    static startTimestamp(_) {
      _.startObject(2);
    }
    static addUnit(_, H) {
      _.addFieldInt16(0, H, TimeUnit.SECOND);
    }
    static addTimezone(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static endTimestamp(_) {
      return _.endObject();
    }
    static createTimestamp(_, H, V) {
      return Timestamp.startTimestamp(_), Timestamp.addUnit(_, H), Timestamp.addTimezone(_, V), Timestamp.endTimestamp(_);
    }
  }
  class Union {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsUnion(_, H) {
      return (H || new Union()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsUnion(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Union()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    mode() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : UnionMode.Sparse;
    }
    typeIds(_) {
      const H = this.bb.__offset(this.bb_pos, 6);
      return H ? this.bb.readInt32(this.bb.__vector(this.bb_pos + H) + _ * 4) : 0;
    }
    typeIdsLength() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    typeIdsArray() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? new Int32Array(this.bb.bytes().buffer, this.bb.bytes().byteOffset + this.bb.__vector(this.bb_pos + _), this.bb.__vector_len(this.bb_pos + _)) : null;
    }
    static startUnion(_) {
      _.startObject(2);
    }
    static addMode(_, H) {
      _.addFieldInt16(0, H, UnionMode.Sparse);
    }
    static addTypeIds(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static createTypeIdsVector(_, H) {
      _.startVector(4, H.length, 4);
      for (let V = H.length - 1; V >= 0; V--)
        _.addInt32(H[V]);
      return _.endVector();
    }
    static startTypeIdsVector(_, H) {
      _.startVector(4, H, 4);
    }
    static endUnion(_) {
      return _.endObject();
    }
    static createUnion(_, H, V) {
      return Union.startUnion(_), Union.addMode(_, H), Union.addTypeIds(_, V), Union.endUnion(_);
    }
  }
  let Utf8$1 = class E3 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsUtf8(_, H) {
      return (H || new E3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsUtf8(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new E3()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static startUtf8(_) {
      _.startObject(0);
    }
    static endUtf8(_) {
      return _.endObject();
    }
    static createUtf8(_) {
      return E3.startUtf8(_), E3.endUtf8(_);
    }
  };
  var Type$1;
  (function(O) {
    O[O.NONE = 0] = "NONE", O[O.Null = 1] = "Null", O[O.Int = 2] = "Int", O[O.FloatingPoint = 3] = "FloatingPoint", O[O.Binary = 4] = "Binary", O[O.Utf8 = 5] = "Utf8", O[O.Bool = 6] = "Bool", O[O.Decimal = 7] = "Decimal", O[O.Date = 8] = "Date", O[O.Time = 9] = "Time", O[O.Timestamp = 10] = "Timestamp", O[O.Interval = 11] = "Interval", O[O.List = 12] = "List", O[O.Struct_ = 13] = "Struct_", O[O.Union = 14] = "Union", O[O.FixedSizeBinary = 15] = "FixedSizeBinary", O[O.FixedSizeList = 16] = "FixedSizeList", O[O.Map = 17] = "Map", O[O.Duration = 18] = "Duration", O[O.LargeBinary = 19] = "LargeBinary", O[O.LargeUtf8 = 20] = "LargeUtf8", O[O.LargeList = 21] = "LargeList", O[O.RunEndEncoded = 22] = "RunEndEncoded";
  })(Type$1 || (Type$1 = {}));
  let Field$1 = class X4 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsField(_, H) {
      return (H || new X4()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsField(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new X4()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    name(_) {
      const H = this.bb.__offset(this.bb_pos, 4);
      return H ? this.bb.__string(this.bb_pos + H, _) : null;
    }
    /**
     * Whether or not this field can contain nulls. Should be true in general.
     */
    nullable() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? !!this.bb.readInt8(this.bb_pos + _) : !1;
    }
    typeType() {
      const _ = this.bb.__offset(this.bb_pos, 8);
      return _ ? this.bb.readUint8(this.bb_pos + _) : Type$1.NONE;
    }
    /**
     * This is the type of the decoded value if the field is dictionary encoded.
     */
    type(_) {
      const H = this.bb.__offset(this.bb_pos, 10);
      return H ? this.bb.__union(_, this.bb_pos + H) : null;
    }
    /**
     * Present only if the field is dictionary encoded.
     */
    dictionary(_) {
      const H = this.bb.__offset(this.bb_pos, 12);
      return H ? (_ || new DictionaryEncoding()).__init(this.bb.__indirect(this.bb_pos + H), this.bb) : null;
    }
    /**
     * children apply only to nested data types like Struct, List and Union. For
     * primitive types children will have length 0.
     */
    children(_, H) {
      const V = this.bb.__offset(this.bb_pos, 14);
      return V ? (H || new X4()).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos + V) + _ * 4), this.bb) : null;
    }
    childrenLength() {
      const _ = this.bb.__offset(this.bb_pos, 14);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    /**
     * User-defined metadata
     */
    customMetadata(_, H) {
      const V = this.bb.__offset(this.bb_pos, 16);
      return V ? (H || new KeyValue()).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos + V) + _ * 4), this.bb) : null;
    }
    customMetadataLength() {
      const _ = this.bb.__offset(this.bb_pos, 16);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    static startField(_) {
      _.startObject(7);
    }
    static addName(_, H) {
      _.addFieldOffset(0, H, 0);
    }
    static addNullable(_, H) {
      _.addFieldInt8(1, +H, 0);
    }
    static addTypeType(_, H) {
      _.addFieldInt8(2, H, Type$1.NONE);
    }
    static addType(_, H) {
      _.addFieldOffset(3, H, 0);
    }
    static addDictionary(_, H) {
      _.addFieldOffset(4, H, 0);
    }
    static addChildren(_, H) {
      _.addFieldOffset(5, H, 0);
    }
    static createChildrenVector(_, H) {
      _.startVector(4, H.length, 4);
      for (let V = H.length - 1; V >= 0; V--)
        _.addOffset(H[V]);
      return _.endVector();
    }
    static startChildrenVector(_, H) {
      _.startVector(4, H, 4);
    }
    static addCustomMetadata(_, H) {
      _.addFieldOffset(6, H, 0);
    }
    static createCustomMetadataVector(_, H) {
      _.startVector(4, H.length, 4);
      for (let V = H.length - 1; V >= 0; V--)
        _.addOffset(H[V]);
      return _.endVector();
    }
    static startCustomMetadataVector(_, H) {
      _.startVector(4, H, 4);
    }
    static endField(_) {
      return _.endObject();
    }
  }, Schema$1 = class i2 {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsSchema(_, H) {
      return (H || new i2()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsSchema(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new i2()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    /**
     * endianness of the buffer
     * it is Little Endian by default
     * if endianness doesn't match the underlying system then the vectors need to be converted
     */
    endianness() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : Endianness.Little;
    }
    fields(_, H) {
      const V = this.bb.__offset(this.bb_pos, 6);
      return V ? (H || new Field$1()).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos + V) + _ * 4), this.bb) : null;
    }
    fieldsLength() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    customMetadata(_, H) {
      const V = this.bb.__offset(this.bb_pos, 8);
      return V ? (H || new KeyValue()).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos + V) + _ * 4), this.bb) : null;
    }
    customMetadataLength() {
      const _ = this.bb.__offset(this.bb_pos, 8);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    /**
     * Features used in the stream/file.
     */
    features(_) {
      const H = this.bb.__offset(this.bb_pos, 10);
      return H ? this.bb.readInt64(this.bb.__vector(this.bb_pos + H) + _ * 8) : BigInt(0);
    }
    featuresLength() {
      const _ = this.bb.__offset(this.bb_pos, 10);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    static startSchema(_) {
      _.startObject(4);
    }
    static addEndianness(_, H) {
      _.addFieldInt16(0, H, Endianness.Little);
    }
    static addFields(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static createFieldsVector(_, H) {
      _.startVector(4, H.length, 4);
      for (let V = H.length - 1; V >= 0; V--)
        _.addOffset(H[V]);
      return _.endVector();
    }
    static startFieldsVector(_, H) {
      _.startVector(4, H, 4);
    }
    static addCustomMetadata(_, H) {
      _.addFieldOffset(2, H, 0);
    }
    static createCustomMetadataVector(_, H) {
      _.startVector(4, H.length, 4);
      for (let V = H.length - 1; V >= 0; V--)
        _.addOffset(H[V]);
      return _.endVector();
    }
    static startCustomMetadataVector(_, H) {
      _.startVector(4, H, 4);
    }
    static addFeatures(_, H) {
      _.addFieldOffset(3, H, 0);
    }
    static createFeaturesVector(_, H) {
      _.startVector(8, H.length, 8);
      for (let V = H.length - 1; V >= 0; V--)
        _.addInt64(H[V]);
      return _.endVector();
    }
    static startFeaturesVector(_, H) {
      _.startVector(8, H, 8);
    }
    static endSchema(_) {
      return _.endObject();
    }
    static finishSchemaBuffer(_, H) {
      _.finish(H);
    }
    static finishSizePrefixedSchemaBuffer(_, H) {
      _.finish(H, void 0, !0);
    }
    static createSchema(_, H, V, q, G) {
      return i2.startSchema(_), i2.addEndianness(_, H), i2.addFields(_, V), i2.addCustomMetadata(_, q), i2.addFeatures(_, G), i2.endSchema(_);
    }
  };
  var MessageHeader;
  (function(O) {
    O[O.NONE = 0] = "NONE", O[O.Schema = 1] = "Schema", O[O.DictionaryBatch = 2] = "DictionaryBatch", O[O.RecordBatch = 3] = "RecordBatch", O[O.Tensor = 4] = "Tensor", O[O.SparseTensor = 5] = "SparseTensor";
  })(MessageHeader || (MessageHeader = {}));
  var Type;
  (function(O) {
    O[O.NONE = 0] = "NONE", O[O.Null = 1] = "Null", O[O.Int = 2] = "Int", O[O.Float = 3] = "Float", O[O.Binary = 4] = "Binary", O[O.Utf8 = 5] = "Utf8", O[O.Bool = 6] = "Bool", O[O.Decimal = 7] = "Decimal", O[O.Date = 8] = "Date", O[O.Time = 9] = "Time", O[O.Timestamp = 10] = "Timestamp", O[O.Interval = 11] = "Interval", O[O.List = 12] = "List", O[O.Struct = 13] = "Struct", O[O.Union = 14] = "Union", O[O.FixedSizeBinary = 15] = "FixedSizeBinary", O[O.FixedSizeList = 16] = "FixedSizeList", O[O.Map = 17] = "Map", O[O.Duration = 18] = "Duration", O[O.LargeBinary = 19] = "LargeBinary", O[O.LargeUtf8 = 20] = "LargeUtf8", O[O.Dictionary = -1] = "Dictionary", O[O.Int8 = -2] = "Int8", O[O.Int16 = -3] = "Int16", O[O.Int32 = -4] = "Int32", O[O.Int64 = -5] = "Int64", O[O.Uint8 = -6] = "Uint8", O[O.Uint16 = -7] = "Uint16", O[O.Uint32 = -8] = "Uint32", O[O.Uint64 = -9] = "Uint64", O[O.Float16 = -10] = "Float16", O[O.Float32 = -11] = "Float32", O[O.Float64 = -12] = "Float64", O[O.DateDay = -13] = "DateDay", O[O.DateMillisecond = -14] = "DateMillisecond", O[O.TimestampSecond = -15] = "TimestampSecond", O[O.TimestampMillisecond = -16] = "TimestampMillisecond", O[O.TimestampMicrosecond = -17] = "TimestampMicrosecond", O[O.TimestampNanosecond = -18] = "TimestampNanosecond", O[O.TimeSecond = -19] = "TimeSecond", O[O.TimeMillisecond = -20] = "TimeMillisecond", O[O.TimeMicrosecond = -21] = "TimeMicrosecond", O[O.TimeNanosecond = -22] = "TimeNanosecond", O[O.DenseUnion = -23] = "DenseUnion", O[O.SparseUnion = -24] = "SparseUnion", O[O.IntervalDayTime = -25] = "IntervalDayTime", O[O.IntervalYearMonth = -26] = "IntervalYearMonth", O[O.DurationSecond = -27] = "DurationSecond", O[O.DurationMillisecond = -28] = "DurationMillisecond", O[O.DurationMicrosecond = -29] = "DurationMicrosecond", O[O.DurationNanosecond = -30] = "DurationNanosecond";
  })(Type || (Type = {}));
  var BufferType;
  (function(O) {
    O[O.OFFSET = 0] = "OFFSET", O[O.DATA = 1] = "DATA", O[O.VALIDITY = 2] = "VALIDITY", O[O.TYPE = 3] = "TYPE";
  })(BufferType || (BufferType = {}));
  const undf = void 0;
  function valueToString(O) {
    if (O === null)
      return "null";
    if (O === undf)
      return "undefined";
    switch (typeof O) {
      case "number":
        return `${O}`;
      case "bigint":
        return `${O}`;
      case "string":
        return `"${O}"`;
    }
    return typeof O[Symbol.toPrimitive] == "function" ? O[Symbol.toPrimitive]("string") : ArrayBuffer.isView(O) ? O instanceof BigInt64Array || O instanceof BigUint64Array ? `[${[...O].map((_) => valueToString(_))}]` : `[${O}]` : ArrayBuffer.isView(O) ? `[${O}]` : JSON.stringify(O, (_, H) => typeof H == "bigint" ? `${H}` : H);
  }
  function bigIntToNumber(O) {
    if (typeof O == "bigint" && (O < Number.MIN_SAFE_INTEGER || O > Number.MAX_SAFE_INTEGER))
      throw new TypeError(`${O} is not safe to convert to a number.`);
    return Number(O);
  }
  function divideBigInts(O, _) {
    return bigIntToNumber(O / _) + bigIntToNumber(O % _) / bigIntToNumber(_);
  }
  const isArrowBigNumSymbol = Symbol.for("isArrowBigNum");
  function BigNum(O, ..._) {
    return _.length === 0 ? Object.setPrototypeOf(toArrayBufferView(this.TypedArray, O), this.constructor.prototype) : Object.setPrototypeOf(new this.TypedArray(O, ..._), this.constructor.prototype);
  }
  BigNum.prototype[isArrowBigNumSymbol] = !0;
  BigNum.prototype.toJSON = function() {
    return `"${bigNumToString(this)}"`;
  };
  BigNum.prototype.valueOf = function(O) {
    return bigNumToNumber(this, O);
  };
  BigNum.prototype.toString = function() {
    return bigNumToString(this);
  };
  BigNum.prototype[Symbol.toPrimitive] = function(O = "default") {
    switch (O) {
      case "number":
        return bigNumToNumber(this);
      case "string":
        return bigNumToString(this);
      case "default":
        return bigNumToBigInt(this);
    }
    return bigNumToString(this);
  };
  function SignedBigNum(...O) {
    return BigNum.apply(this, O);
  }
  function UnsignedBigNum(...O) {
    return BigNum.apply(this, O);
  }
  function DecimalBigNum(...O) {
    return BigNum.apply(this, O);
  }
  Object.setPrototypeOf(SignedBigNum.prototype, Object.create(Int32Array.prototype));
  Object.setPrototypeOf(UnsignedBigNum.prototype, Object.create(Uint32Array.prototype));
  Object.setPrototypeOf(DecimalBigNum.prototype, Object.create(Uint32Array.prototype));
  Object.assign(SignedBigNum.prototype, BigNum.prototype, { constructor: SignedBigNum, signed: !0, TypedArray: Int32Array, BigIntArray: BigInt64Array });
  Object.assign(UnsignedBigNum.prototype, BigNum.prototype, { constructor: UnsignedBigNum, signed: !1, TypedArray: Uint32Array, BigIntArray: BigUint64Array });
  Object.assign(DecimalBigNum.prototype, BigNum.prototype, { constructor: DecimalBigNum, signed: !0, TypedArray: Uint32Array, BigIntArray: BigUint64Array });
  const TWO_TO_THE_64 = BigInt(4294967296) * BigInt(4294967296), TWO_TO_THE_64_MINUS_1 = TWO_TO_THE_64 - BigInt(1);
  function bigNumToNumber(O, _) {
    const { buffer: H, byteOffset: V, byteLength: q, signed: G } = O, J = new BigUint64Array(H, V, q / 8), Z = G && J.at(-1) & BigInt(1) << BigInt(63);
    let te = BigInt(0), ne = 0;
    if (Z) {
      for (const ee of J)
        te |= (ee ^ TWO_TO_THE_64_MINUS_1) * (BigInt(1) << BigInt(64 * ne++));
      te *= BigInt(-1), te -= BigInt(1);
    } else
      for (const ee of J)
        te |= ee * (BigInt(1) << BigInt(64 * ne++));
    if (typeof _ == "number") {
      const ee = BigInt(Math.pow(10, _)), ae = te / ee, oe = te % ee;
      return bigIntToNumber(ae) + bigIntToNumber(oe) / bigIntToNumber(ee);
    }
    return bigIntToNumber(te);
  }
  function bigNumToString(O) {
    if (O.byteLength === 8)
      return `${new O.BigIntArray(O.buffer, O.byteOffset, 1)[0]}`;
    if (!O.signed)
      return unsignedBigNumToString(O);
    let _ = new Uint16Array(O.buffer, O.byteOffset, O.byteLength / 2);
    if (new Int16Array([_.at(-1)])[0] >= 0)
      return unsignedBigNumToString(O);
    _ = _.slice();
    let V = 1;
    for (let G = 0; G < _.length; G++) {
      const J = _[G], Z = ~J + V;
      _[G] = Z, V &= J === 0 ? 1 : 0;
    }
    return `-${unsignedBigNumToString(_)}`;
  }
  function bigNumToBigInt(O) {
    return O.byteLength === 8 ? new O.BigIntArray(O.buffer, O.byteOffset, 1)[0] : bigNumToString(O);
  }
  function unsignedBigNumToString(O) {
    let _ = "";
    const H = new Uint32Array(2);
    let V = new Uint16Array(O.buffer, O.byteOffset, O.byteLength / 2);
    const q = new Uint32Array((V = new Uint16Array(V).reverse()).buffer);
    let G = -1;
    const J = V.length - 1;
    do {
      for (H[0] = V[G = 0]; G < J; )
        V[G++] = H[1] = H[0] / 10, H[0] = (H[0] - H[1] * 10 << 16) + V[G];
      V[G] = H[1] = H[0] / 10, H[0] = H[0] - H[1] * 10, _ = `${H[0]}${_}`;
    } while (q[0] || q[1] || q[2] || q[3]);
    return _ ?? "0";
  }
  class BN {
    /** @nocollapse */
    static new(_, H) {
      switch (H) {
        case !0:
          return new SignedBigNum(_);
        case !1:
          return new UnsignedBigNum(_);
      }
      switch (_.constructor) {
        case Int8Array:
        case Int16Array:
        case Int32Array:
        case BigInt64Array:
          return new SignedBigNum(_);
      }
      return _.byteLength === 16 ? new DecimalBigNum(_) : new UnsignedBigNum(_);
    }
    /** @nocollapse */
    static signed(_) {
      return new SignedBigNum(_);
    }
    /** @nocollapse */
    static unsigned(_) {
      return new UnsignedBigNum(_);
    }
    /** @nocollapse */
    static decimal(_) {
      return new DecimalBigNum(_);
    }
    constructor(_, H) {
      return BN.new(_, H);
    }
  }
  var _a$3, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x;
  class DataType {
    /** @nocollapse */
    static isNull(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Null;
    }
    /** @nocollapse */
    static isInt(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Int;
    }
    /** @nocollapse */
    static isFloat(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Float;
    }
    /** @nocollapse */
    static isBinary(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Binary;
    }
    /** @nocollapse */
    static isLargeBinary(_) {
      return (_ == null ? void 0 : _.typeId) === Type.LargeBinary;
    }
    /** @nocollapse */
    static isUtf8(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Utf8;
    }
    /** @nocollapse */
    static isLargeUtf8(_) {
      return (_ == null ? void 0 : _.typeId) === Type.LargeUtf8;
    }
    /** @nocollapse */
    static isBool(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Bool;
    }
    /** @nocollapse */
    static isDecimal(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Decimal;
    }
    /** @nocollapse */
    static isDate(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Date;
    }
    /** @nocollapse */
    static isTime(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Time;
    }
    /** @nocollapse */
    static isTimestamp(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Timestamp;
    }
    /** @nocollapse */
    static isInterval(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Interval;
    }
    /** @nocollapse */
    static isDuration(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Duration;
    }
    /** @nocollapse */
    static isList(_) {
      return (_ == null ? void 0 : _.typeId) === Type.List;
    }
    /** @nocollapse */
    static isStruct(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Struct;
    }
    /** @nocollapse */
    static isUnion(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Union;
    }
    /** @nocollapse */
    static isFixedSizeBinary(_) {
      return (_ == null ? void 0 : _.typeId) === Type.FixedSizeBinary;
    }
    /** @nocollapse */
    static isFixedSizeList(_) {
      return (_ == null ? void 0 : _.typeId) === Type.FixedSizeList;
    }
    /** @nocollapse */
    static isMap(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Map;
    }
    /** @nocollapse */
    static isDictionary(_) {
      return (_ == null ? void 0 : _.typeId) === Type.Dictionary;
    }
    /** @nocollapse */
    static isDenseUnion(_) {
      return DataType.isUnion(_) && _.mode === UnionMode.Dense;
    }
    /** @nocollapse */
    static isSparseUnion(_) {
      return DataType.isUnion(_) && _.mode === UnionMode.Sparse;
    }
    constructor(_) {
      this.typeId = _;
    }
  }
  _a$3 = Symbol.toStringTag;
  DataType[_a$3] = ((O) => (O.children = null, O.ArrayType = Array, O.OffsetArrayType = Int32Array, O[Symbol.toStringTag] = "DataType"))(DataType.prototype);
  class Null extends DataType {
    constructor() {
      super(Type.Null);
    }
    toString() {
      return "Null";
    }
  }
  _b = Symbol.toStringTag;
  Null[_b] = ((O) => O[Symbol.toStringTag] = "Null")(Null.prototype);
  class Int_ extends DataType {
    constructor(_, H) {
      super(Type.Int), this.isSigned = _, this.bitWidth = H;
    }
    get ArrayType() {
      switch (this.bitWidth) {
        case 8:
          return this.isSigned ? Int8Array : Uint8Array;
        case 16:
          return this.isSigned ? Int16Array : Uint16Array;
        case 32:
          return this.isSigned ? Int32Array : Uint32Array;
        case 64:
          return this.isSigned ? BigInt64Array : BigUint64Array;
      }
      throw new Error(`Unrecognized ${this[Symbol.toStringTag]} type`);
    }
    toString() {
      return `${this.isSigned ? "I" : "Ui"}nt${this.bitWidth}`;
    }
  }
  _c = Symbol.toStringTag;
  Int_[_c] = ((O) => (O.isSigned = null, O.bitWidth = null, O[Symbol.toStringTag] = "Int"))(Int_.prototype);
  class Int32 extends Int_ {
    constructor() {
      super(!0, 32);
    }
    get ArrayType() {
      return Int32Array;
    }
  }
  Object.defineProperty(Int32.prototype, "ArrayType", { value: Int32Array });
  class Float extends DataType {
    constructor(_) {
      super(Type.Float), this.precision = _;
    }
    get ArrayType() {
      switch (this.precision) {
        case Precision.HALF:
          return Uint16Array;
        case Precision.SINGLE:
          return Float32Array;
        case Precision.DOUBLE:
          return Float64Array;
      }
      throw new Error(`Unrecognized ${this[Symbol.toStringTag]} type`);
    }
    toString() {
      return `Float${this.precision << 5 || 16}`;
    }
  }
  _d = Symbol.toStringTag;
  Float[_d] = ((O) => (O.precision = null, O[Symbol.toStringTag] = "Float"))(Float.prototype);
  class Binary extends DataType {
    constructor() {
      super(Type.Binary);
    }
    toString() {
      return "Binary";
    }
  }
  _e = Symbol.toStringTag;
  Binary[_e] = ((O) => (O.ArrayType = Uint8Array, O[Symbol.toStringTag] = "Binary"))(Binary.prototype);
  class LargeBinary extends DataType {
    constructor() {
      super(Type.LargeBinary);
    }
    toString() {
      return "LargeBinary";
    }
  }
  _f = Symbol.toStringTag;
  LargeBinary[_f] = ((O) => (O.ArrayType = Uint8Array, O.OffsetArrayType = BigInt64Array, O[Symbol.toStringTag] = "LargeBinary"))(LargeBinary.prototype);
  class Utf8 extends DataType {
    constructor() {
      super(Type.Utf8);
    }
    toString() {
      return "Utf8";
    }
  }
  _g = Symbol.toStringTag;
  Utf8[_g] = ((O) => (O.ArrayType = Uint8Array, O[Symbol.toStringTag] = "Utf8"))(Utf8.prototype);
  class LargeUtf8 extends DataType {
    constructor() {
      super(Type.LargeUtf8);
    }
    toString() {
      return "LargeUtf8";
    }
  }
  _h = Symbol.toStringTag;
  LargeUtf8[_h] = ((O) => (O.ArrayType = Uint8Array, O.OffsetArrayType = BigInt64Array, O[Symbol.toStringTag] = "LargeUtf8"))(LargeUtf8.prototype);
  class Bool extends DataType {
    constructor() {
      super(Type.Bool);
    }
    toString() {
      return "Bool";
    }
  }
  _j = Symbol.toStringTag;
  Bool[_j] = ((O) => (O.ArrayType = Uint8Array, O[Symbol.toStringTag] = "Bool"))(Bool.prototype);
  class Decimal extends DataType {
    constructor(_, H, V = 128) {
      super(Type.Decimal), this.scale = _, this.precision = H, this.bitWidth = V;
    }
    toString() {
      return `Decimal[${this.precision}e${this.scale > 0 ? "+" : ""}${this.scale}]`;
    }
  }
  _k = Symbol.toStringTag;
  Decimal[_k] = ((O) => (O.scale = null, O.precision = null, O.ArrayType = Uint32Array, O[Symbol.toStringTag] = "Decimal"))(Decimal.prototype);
  class Date_ extends DataType {
    constructor(_) {
      super(Type.Date), this.unit = _;
    }
    toString() {
      return `Date${(this.unit + 1) * 32}<${DateUnit[this.unit]}>`;
    }
    get ArrayType() {
      return this.unit === DateUnit.DAY ? Int32Array : BigInt64Array;
    }
  }
  _l = Symbol.toStringTag;
  Date_[_l] = ((O) => (O.unit = null, O[Symbol.toStringTag] = "Date"))(Date_.prototype);
  class Time_ extends DataType {
    constructor(_, H) {
      super(Type.Time), this.unit = _, this.bitWidth = H;
    }
    toString() {
      return `Time${this.bitWidth}<${TimeUnit[this.unit]}>`;
    }
    get ArrayType() {
      switch (this.bitWidth) {
        case 32:
          return Int32Array;
        case 64:
          return BigInt64Array;
      }
      throw new Error(`Unrecognized ${this[Symbol.toStringTag]} type`);
    }
  }
  _m = Symbol.toStringTag;
  Time_[_m] = ((O) => (O.unit = null, O.bitWidth = null, O[Symbol.toStringTag] = "Time"))(Time_.prototype);
  class Timestamp_ extends DataType {
    constructor(_, H) {
      super(Type.Timestamp), this.unit = _, this.timezone = H;
    }
    toString() {
      return `Timestamp<${TimeUnit[this.unit]}${this.timezone ? `, ${this.timezone}` : ""}>`;
    }
  }
  _o = Symbol.toStringTag;
  Timestamp_[_o] = ((O) => (O.unit = null, O.timezone = null, O.ArrayType = BigInt64Array, O[Symbol.toStringTag] = "Timestamp"))(Timestamp_.prototype);
  class Interval_ extends DataType {
    constructor(_) {
      super(Type.Interval), this.unit = _;
    }
    toString() {
      return `Interval<${IntervalUnit[this.unit]}>`;
    }
  }
  _p = Symbol.toStringTag;
  Interval_[_p] = ((O) => (O.unit = null, O.ArrayType = Int32Array, O[Symbol.toStringTag] = "Interval"))(Interval_.prototype);
  class Duration extends DataType {
    constructor(_) {
      super(Type.Duration), this.unit = _;
    }
    toString() {
      return `Duration<${TimeUnit[this.unit]}>`;
    }
  }
  _q = Symbol.toStringTag;
  Duration[_q] = ((O) => (O.unit = null, O.ArrayType = BigInt64Array, O[Symbol.toStringTag] = "Duration"))(Duration.prototype);
  class List extends DataType {
    constructor(_) {
      super(Type.List), this.children = [_];
    }
    toString() {
      return `List<${this.valueType}>`;
    }
    get valueType() {
      return this.children[0].type;
    }
    get valueField() {
      return this.children[0];
    }
    get ArrayType() {
      return this.valueType.ArrayType;
    }
  }
  _r = Symbol.toStringTag;
  List[_r] = ((O) => (O.children = null, O[Symbol.toStringTag] = "List"))(List.prototype);
  class Struct extends DataType {
    constructor(_) {
      super(Type.Struct), this.children = _;
    }
    toString() {
      return `Struct<{${this.children.map((_) => `${_.name}:${_.type}`).join(", ")}}>`;
    }
  }
  _s = Symbol.toStringTag;
  Struct[_s] = ((O) => (O.children = null, O[Symbol.toStringTag] = "Struct"))(Struct.prototype);
  class Union_ extends DataType {
    constructor(_, H, V) {
      super(Type.Union), this.mode = _, this.children = V, this.typeIds = H = Int32Array.from(H), this.typeIdToChildIndex = H.reduce((q, G, J) => (q[G] = J) && q || q, /* @__PURE__ */ Object.create(null));
    }
    toString() {
      return `${this[Symbol.toStringTag]}<${this.children.map((_) => `${_.type}`).join(" | ")}>`;
    }
  }
  _t = Symbol.toStringTag;
  Union_[_t] = ((O) => (O.mode = null, O.typeIds = null, O.children = null, O.typeIdToChildIndex = null, O.ArrayType = Int8Array, O[Symbol.toStringTag] = "Union"))(Union_.prototype);
  class FixedSizeBinary extends DataType {
    constructor(_) {
      super(Type.FixedSizeBinary), this.byteWidth = _;
    }
    toString() {
      return `FixedSizeBinary[${this.byteWidth}]`;
    }
  }
  _u = Symbol.toStringTag;
  FixedSizeBinary[_u] = ((O) => (O.byteWidth = null, O.ArrayType = Uint8Array, O[Symbol.toStringTag] = "FixedSizeBinary"))(FixedSizeBinary.prototype);
  class FixedSizeList extends DataType {
    constructor(_, H) {
      super(Type.FixedSizeList), this.listSize = _, this.children = [H];
    }
    get valueType() {
      return this.children[0].type;
    }
    get valueField() {
      return this.children[0];
    }
    get ArrayType() {
      return this.valueType.ArrayType;
    }
    toString() {
      return `FixedSizeList[${this.listSize}]<${this.valueType}>`;
    }
  }
  _v = Symbol.toStringTag;
  FixedSizeList[_v] = ((O) => (O.children = null, O.listSize = null, O[Symbol.toStringTag] = "FixedSizeList"))(FixedSizeList.prototype);
  class Map_ extends DataType {
    constructor(_, H = !1) {
      var V, q, G;
      if (super(Type.Map), this.children = [_], this.keysSorted = H, _ && (_.name = "entries", !((V = _ == null ? void 0 : _.type) === null || V === void 0) && V.children)) {
        const J = (q = _ == null ? void 0 : _.type) === null || q === void 0 ? void 0 : q.children[0];
        J && (J.name = "key");
        const Z = (G = _ == null ? void 0 : _.type) === null || G === void 0 ? void 0 : G.children[1];
        Z && (Z.name = "value");
      }
    }
    get keyType() {
      return this.children[0].type.children[0].type;
    }
    get valueType() {
      return this.children[0].type.children[1].type;
    }
    get childType() {
      return this.children[0].type;
    }
    toString() {
      return `Map<{${this.children[0].type.children.map((_) => `${_.name}:${_.type}`).join(", ")}}>`;
    }
  }
  _w = Symbol.toStringTag;
  Map_[_w] = ((O) => (O.children = null, O.keysSorted = null, O[Symbol.toStringTag] = "Map_"))(Map_.prototype);
  const getId = /* @__PURE__ */ ((O) => () => ++O)(-1);
  class Dictionary extends DataType {
    constructor(_, H, V, q) {
      super(Type.Dictionary), this.indices = H, this.dictionary = _, this.isOrdered = q || !1, this.id = V == null ? getId() : bigIntToNumber(V);
    }
    get children() {
      return this.dictionary.children;
    }
    get valueType() {
      return this.dictionary;
    }
    get ArrayType() {
      return this.dictionary.ArrayType;
    }
    toString() {
      return `Dictionary<${this.indices}, ${this.dictionary}>`;
    }
  }
  _x = Symbol.toStringTag;
  Dictionary[_x] = ((O) => (O.id = null, O.indices = null, O.isOrdered = null, O.dictionary = null, O[Symbol.toStringTag] = "Dictionary"))(Dictionary.prototype);
  function strideForType(O) {
    const _ = O;
    switch (O.typeId) {
      case Type.Decimal:
        return O.bitWidth / 32;
      case Type.Interval:
        return 1 + _.unit;
      // case Type.Int: return 1 + +((t as Int_).bitWidth > 32);
      // case Type.Time: return 1 + +((t as Time_).bitWidth > 32);
      case Type.FixedSizeList:
        return _.listSize;
      case Type.FixedSizeBinary:
        return _.byteWidth;
      default:
        return 1;
    }
  }
  class Visitor {
    visitMany(_, ...H) {
      return _.map((V, q) => this.visit(V, ...H.map((G) => G[q])));
    }
    visit(..._) {
      return this.getVisitFn(_[0], !1).apply(this, _);
    }
    getVisitFn(_, H = !0) {
      return getVisitFn(this, _, H);
    }
    getVisitFnByTypeId(_, H = !0) {
      return getVisitFnByTypeId(this, _, H);
    }
    visitNull(_, ...H) {
      return null;
    }
    visitBool(_, ...H) {
      return null;
    }
    visitInt(_, ...H) {
      return null;
    }
    visitFloat(_, ...H) {
      return null;
    }
    visitUtf8(_, ...H) {
      return null;
    }
    visitLargeUtf8(_, ...H) {
      return null;
    }
    visitBinary(_, ...H) {
      return null;
    }
    visitLargeBinary(_, ...H) {
      return null;
    }
    visitFixedSizeBinary(_, ...H) {
      return null;
    }
    visitDate(_, ...H) {
      return null;
    }
    visitTimestamp(_, ...H) {
      return null;
    }
    visitTime(_, ...H) {
      return null;
    }
    visitDecimal(_, ...H) {
      return null;
    }
    visitList(_, ...H) {
      return null;
    }
    visitStruct(_, ...H) {
      return null;
    }
    visitUnion(_, ...H) {
      return null;
    }
    visitDictionary(_, ...H) {
      return null;
    }
    visitInterval(_, ...H) {
      return null;
    }
    visitDuration(_, ...H) {
      return null;
    }
    visitFixedSizeList(_, ...H) {
      return null;
    }
    visitMap(_, ...H) {
      return null;
    }
  }
  function getVisitFn(O, _, H = !0) {
    return typeof _ == "number" ? getVisitFnByTypeId(O, _, H) : typeof _ == "string" && _ in Type ? getVisitFnByTypeId(O, Type[_], H) : _ && _ instanceof DataType ? getVisitFnByTypeId(O, inferDType(_), H) : _ != null && _.type && _.type instanceof DataType ? getVisitFnByTypeId(O, inferDType(_.type), H) : getVisitFnByTypeId(O, Type.NONE, H);
  }
  function getVisitFnByTypeId(O, _, H = !0) {
    let V = null;
    switch (_) {
      case Type.Null:
        V = O.visitNull;
        break;
      case Type.Bool:
        V = O.visitBool;
        break;
      case Type.Int:
        V = O.visitInt;
        break;
      case Type.Int8:
        V = O.visitInt8 || O.visitInt;
        break;
      case Type.Int16:
        V = O.visitInt16 || O.visitInt;
        break;
      case Type.Int32:
        V = O.visitInt32 || O.visitInt;
        break;
      case Type.Int64:
        V = O.visitInt64 || O.visitInt;
        break;
      case Type.Uint8:
        V = O.visitUint8 || O.visitInt;
        break;
      case Type.Uint16:
        V = O.visitUint16 || O.visitInt;
        break;
      case Type.Uint32:
        V = O.visitUint32 || O.visitInt;
        break;
      case Type.Uint64:
        V = O.visitUint64 || O.visitInt;
        break;
      case Type.Float:
        V = O.visitFloat;
        break;
      case Type.Float16:
        V = O.visitFloat16 || O.visitFloat;
        break;
      case Type.Float32:
        V = O.visitFloat32 || O.visitFloat;
        break;
      case Type.Float64:
        V = O.visitFloat64 || O.visitFloat;
        break;
      case Type.Utf8:
        V = O.visitUtf8;
        break;
      case Type.LargeUtf8:
        V = O.visitLargeUtf8;
        break;
      case Type.Binary:
        V = O.visitBinary;
        break;
      case Type.LargeBinary:
        V = O.visitLargeBinary;
        break;
      case Type.FixedSizeBinary:
        V = O.visitFixedSizeBinary;
        break;
      case Type.Date:
        V = O.visitDate;
        break;
      case Type.DateDay:
        V = O.visitDateDay || O.visitDate;
        break;
      case Type.DateMillisecond:
        V = O.visitDateMillisecond || O.visitDate;
        break;
      case Type.Timestamp:
        V = O.visitTimestamp;
        break;
      case Type.TimestampSecond:
        V = O.visitTimestampSecond || O.visitTimestamp;
        break;
      case Type.TimestampMillisecond:
        V = O.visitTimestampMillisecond || O.visitTimestamp;
        break;
      case Type.TimestampMicrosecond:
        V = O.visitTimestampMicrosecond || O.visitTimestamp;
        break;
      case Type.TimestampNanosecond:
        V = O.visitTimestampNanosecond || O.visitTimestamp;
        break;
      case Type.Time:
        V = O.visitTime;
        break;
      case Type.TimeSecond:
        V = O.visitTimeSecond || O.visitTime;
        break;
      case Type.TimeMillisecond:
        V = O.visitTimeMillisecond || O.visitTime;
        break;
      case Type.TimeMicrosecond:
        V = O.visitTimeMicrosecond || O.visitTime;
        break;
      case Type.TimeNanosecond:
        V = O.visitTimeNanosecond || O.visitTime;
        break;
      case Type.Decimal:
        V = O.visitDecimal;
        break;
      case Type.List:
        V = O.visitList;
        break;
      case Type.Struct:
        V = O.visitStruct;
        break;
      case Type.Union:
        V = O.visitUnion;
        break;
      case Type.DenseUnion:
        V = O.visitDenseUnion || O.visitUnion;
        break;
      case Type.SparseUnion:
        V = O.visitSparseUnion || O.visitUnion;
        break;
      case Type.Dictionary:
        V = O.visitDictionary;
        break;
      case Type.Interval:
        V = O.visitInterval;
        break;
      case Type.IntervalDayTime:
        V = O.visitIntervalDayTime || O.visitInterval;
        break;
      case Type.IntervalYearMonth:
        V = O.visitIntervalYearMonth || O.visitInterval;
        break;
      case Type.Duration:
        V = O.visitDuration;
        break;
      case Type.DurationSecond:
        V = O.visitDurationSecond || O.visitDuration;
        break;
      case Type.DurationMillisecond:
        V = O.visitDurationMillisecond || O.visitDuration;
        break;
      case Type.DurationMicrosecond:
        V = O.visitDurationMicrosecond || O.visitDuration;
        break;
      case Type.DurationNanosecond:
        V = O.visitDurationNanosecond || O.visitDuration;
        break;
      case Type.FixedSizeList:
        V = O.visitFixedSizeList;
        break;
      case Type.Map:
        V = O.visitMap;
        break;
    }
    if (typeof V == "function")
      return V;
    if (!H)
      return () => null;
    throw new Error(`Unrecognized type '${Type[_]}'`);
  }
  function inferDType(O) {
    switch (O.typeId) {
      case Type.Null:
        return Type.Null;
      case Type.Int: {
        const { bitWidth: _, isSigned: H } = O;
        switch (_) {
          case 8:
            return H ? Type.Int8 : Type.Uint8;
          case 16:
            return H ? Type.Int16 : Type.Uint16;
          case 32:
            return H ? Type.Int32 : Type.Uint32;
          case 64:
            return H ? Type.Int64 : Type.Uint64;
        }
        return Type.Int;
      }
      case Type.Float:
        switch (O.precision) {
          case Precision.HALF:
            return Type.Float16;
          case Precision.SINGLE:
            return Type.Float32;
          case Precision.DOUBLE:
            return Type.Float64;
        }
        return Type.Float;
      case Type.Binary:
        return Type.Binary;
      case Type.LargeBinary:
        return Type.LargeBinary;
      case Type.Utf8:
        return Type.Utf8;
      case Type.LargeUtf8:
        return Type.LargeUtf8;
      case Type.Bool:
        return Type.Bool;
      case Type.Decimal:
        return Type.Decimal;
      case Type.Time:
        switch (O.unit) {
          case TimeUnit.SECOND:
            return Type.TimeSecond;
          case TimeUnit.MILLISECOND:
            return Type.TimeMillisecond;
          case TimeUnit.MICROSECOND:
            return Type.TimeMicrosecond;
          case TimeUnit.NANOSECOND:
            return Type.TimeNanosecond;
        }
        return Type.Time;
      case Type.Timestamp:
        switch (O.unit) {
          case TimeUnit.SECOND:
            return Type.TimestampSecond;
          case TimeUnit.MILLISECOND:
            return Type.TimestampMillisecond;
          case TimeUnit.MICROSECOND:
            return Type.TimestampMicrosecond;
          case TimeUnit.NANOSECOND:
            return Type.TimestampNanosecond;
        }
        return Type.Timestamp;
      case Type.Date:
        switch (O.unit) {
          case DateUnit.DAY:
            return Type.DateDay;
          case DateUnit.MILLISECOND:
            return Type.DateMillisecond;
        }
        return Type.Date;
      case Type.Interval:
        switch (O.unit) {
          case IntervalUnit.DAY_TIME:
            return Type.IntervalDayTime;
          case IntervalUnit.YEAR_MONTH:
            return Type.IntervalYearMonth;
        }
        return Type.Interval;
      case Type.Duration:
        switch (O.unit) {
          case TimeUnit.SECOND:
            return Type.DurationSecond;
          case TimeUnit.MILLISECOND:
            return Type.DurationMillisecond;
          case TimeUnit.MICROSECOND:
            return Type.DurationMicrosecond;
          case TimeUnit.NANOSECOND:
            return Type.DurationNanosecond;
        }
        return Type.Duration;
      case Type.Map:
        return Type.Map;
      case Type.List:
        return Type.List;
      case Type.Struct:
        return Type.Struct;
      case Type.Union:
        switch (O.mode) {
          case UnionMode.Dense:
            return Type.DenseUnion;
          case UnionMode.Sparse:
            return Type.SparseUnion;
        }
        return Type.Union;
      case Type.FixedSizeBinary:
        return Type.FixedSizeBinary;
      case Type.FixedSizeList:
        return Type.FixedSizeList;
      case Type.Dictionary:
        return Type.Dictionary;
    }
    throw new Error(`Unrecognized type '${Type[O.typeId]}'`);
  }
  Visitor.prototype.visitInt8 = null;
  Visitor.prototype.visitInt16 = null;
  Visitor.prototype.visitInt32 = null;
  Visitor.prototype.visitInt64 = null;
  Visitor.prototype.visitUint8 = null;
  Visitor.prototype.visitUint16 = null;
  Visitor.prototype.visitUint32 = null;
  Visitor.prototype.visitUint64 = null;
  Visitor.prototype.visitFloat16 = null;
  Visitor.prototype.visitFloat32 = null;
  Visitor.prototype.visitFloat64 = null;
  Visitor.prototype.visitDateDay = null;
  Visitor.prototype.visitDateMillisecond = null;
  Visitor.prototype.visitTimestampSecond = null;
  Visitor.prototype.visitTimestampMillisecond = null;
  Visitor.prototype.visitTimestampMicrosecond = null;
  Visitor.prototype.visitTimestampNanosecond = null;
  Visitor.prototype.visitTimeSecond = null;
  Visitor.prototype.visitTimeMillisecond = null;
  Visitor.prototype.visitTimeMicrosecond = null;
  Visitor.prototype.visitTimeNanosecond = null;
  Visitor.prototype.visitDenseUnion = null;
  Visitor.prototype.visitSparseUnion = null;
  Visitor.prototype.visitIntervalDayTime = null;
  Visitor.prototype.visitIntervalYearMonth = null;
  Visitor.prototype.visitDuration = null;
  Visitor.prototype.visitDurationSecond = null;
  Visitor.prototype.visitDurationMillisecond = null;
  Visitor.prototype.visitDurationMicrosecond = null;
  Visitor.prototype.visitDurationNanosecond = null;
  const f64 = new Float64Array(1), u32 = new Uint32Array(f64.buffer);
  function uint16ToFloat64(O) {
    const _ = (O & 31744) >> 10, H = (O & 1023) / 1024, V = Math.pow(-1, (O & 32768) >> 15);
    switch (_) {
      case 31:
        return V * (H ? Number.NaN : 1 / 0);
      case 0:
        return V * (H ? 6103515625e-14 * H : 0);
    }
    return V * Math.pow(2, _ - 15) * (1 + H);
  }
  function float64ToUint16(O) {
    if (O !== O)
      return 32256;
    f64[0] = O;
    const _ = (u32[1] & 2147483648) >> 16 & 65535;
    let H = u32[1] & 2146435072, V = 0;
    return H >= 1089470464 ? u32[0] > 0 ? H = 31744 : (H = (H & 2080374784) >> 16, V = (u32[1] & 1048575) >> 10) : H <= 1056964608 ? (V = 1048576 + (u32[1] & 1048575), V = 1048576 + (V << (H >> 20) - 998) >> 21, H = 0) : (H = H - 1056964608 >> 10, V = (u32[1] & 1048575) + 512 >> 10), _ | H | V & 65535;
  }
  class SetVisitor extends Visitor {
  }
  function wrapSet(O) {
    return (_, H, V) => {
      if (_.setValid(H, V != null))
        return O(_, H, V);
    };
  }
  const setEpochMsToDays = (O, _, H) => {
    O[_] = Math.floor(H / 864e5);
  }, setVariableWidthBytes = (O, _, H, V) => {
    if (H + 1 < _.length) {
      const q = bigIntToNumber(_[H]), G = bigIntToNumber(_[H + 1]);
      O.set(V.subarray(0, G - q), q);
    }
  }, setBool = ({ offset: O, values: _ }, H, V) => {
    const q = O + H;
    V ? _[q >> 3] |= 1 << q % 8 : _[q >> 3] &= ~(1 << q % 8);
  }, setInt = ({ values: O }, _, H) => {
    O[_] = H;
  }, setFloat = ({ values: O }, _, H) => {
    O[_] = H;
  }, setFloat16 = ({ values: O }, _, H) => {
    O[_] = float64ToUint16(H);
  }, setAnyFloat = (O, _, H) => {
    switch (O.type.precision) {
      case Precision.HALF:
        return setFloat16(O, _, H);
      case Precision.SINGLE:
      case Precision.DOUBLE:
        return setFloat(O, _, H);
    }
  }, setDateDay = ({ values: O }, _, H) => {
    setEpochMsToDays(O, _, H.valueOf());
  }, setDateMillisecond = ({ values: O }, _, H) => {
    O[_] = BigInt(H);
  }, setFixedSizeBinary = ({ stride: O, values: _ }, H, V) => {
    _.set(V.subarray(0, O), O * H);
  }, setBinary = ({ values: O, valueOffsets: _ }, H, V) => setVariableWidthBytes(O, _, H, V), setUtf8 = ({ values: O, valueOffsets: _ }, H, V) => setVariableWidthBytes(O, _, H, encodeUtf8(V)), setDate = (O, _, H) => {
    O.type.unit === DateUnit.DAY ? setDateDay(O, _, H) : setDateMillisecond(O, _, H);
  }, setTimestampSecond = ({ values: O }, _, H) => {
    O[_] = BigInt(H / 1e3);
  }, setTimestampMillisecond = ({ values: O }, _, H) => {
    O[_] = BigInt(H);
  }, setTimestampMicrosecond = ({ values: O }, _, H) => {
    O[_] = BigInt(H * 1e3);
  }, setTimestampNanosecond = ({ values: O }, _, H) => {
    O[_] = BigInt(H * 1e6);
  }, setTimestamp = (O, _, H) => {
    switch (O.type.unit) {
      case TimeUnit.SECOND:
        return setTimestampSecond(O, _, H);
      case TimeUnit.MILLISECOND:
        return setTimestampMillisecond(O, _, H);
      case TimeUnit.MICROSECOND:
        return setTimestampMicrosecond(O, _, H);
      case TimeUnit.NANOSECOND:
        return setTimestampNanosecond(O, _, H);
    }
  }, setTimeSecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setTimeMillisecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setTimeMicrosecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setTimeNanosecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setTime = (O, _, H) => {
    switch (O.type.unit) {
      case TimeUnit.SECOND:
        return setTimeSecond(O, _, H);
      case TimeUnit.MILLISECOND:
        return setTimeMillisecond(O, _, H);
      case TimeUnit.MICROSECOND:
        return setTimeMicrosecond(O, _, H);
      case TimeUnit.NANOSECOND:
        return setTimeNanosecond(O, _, H);
    }
  }, setDecimal = ({ values: O, stride: _ }, H, V) => {
    O.set(V.subarray(0, _), _ * H);
  }, setList = (O, _, H) => {
    const V = O.children[0], q = O.valueOffsets, G = instance$5.getVisitFn(V);
    if (Array.isArray(H))
      for (let J = -1, Z = q[_], te = q[_ + 1]; Z < te; )
        G(V, Z++, H[++J]);
    else
      for (let J = -1, Z = q[_], te = q[_ + 1]; Z < te; )
        G(V, Z++, H.get(++J));
  }, setMap = (O, _, H) => {
    const V = O.children[0], { valueOffsets: q } = O, G = instance$5.getVisitFn(V);
    let { [_]: J, [_ + 1]: Z } = q;
    const te = H instanceof Map ? H.entries() : Object.entries(H);
    for (const ne of te)
      if (G(V, J, ne), ++J >= Z)
        break;
  }, _setStructArrayValue = (O, _) => (H, V, q, G) => V && H(V, O, _[G]), _setStructVectorValue = (O, _) => (H, V, q, G) => V && H(V, O, _.get(G)), _setStructMapValue = (O, _) => (H, V, q, G) => V && H(V, O, _.get(q.name)), _setStructObjectValue = (O, _) => (H, V, q, G) => V && H(V, O, _[q.name]), setStruct = (O, _, H) => {
    const V = O.type.children.map((G) => instance$5.getVisitFn(G.type)), q = H instanceof Map ? _setStructMapValue(_, H) : H instanceof Vector ? _setStructVectorValue(_, H) : Array.isArray(H) ? _setStructArrayValue(_, H) : _setStructObjectValue(_, H);
    O.type.children.forEach((G, J) => q(V[J], O.children[J], G, J));
  }, setUnion = (O, _, H) => {
    O.type.mode === UnionMode.Dense ? setDenseUnion(O, _, H) : setSparseUnion(O, _, H);
  }, setDenseUnion = (O, _, H) => {
    const V = O.type.typeIdToChildIndex[O.typeIds[_]], q = O.children[V];
    instance$5.visit(q, O.valueOffsets[_], H);
  }, setSparseUnion = (O, _, H) => {
    const V = O.type.typeIdToChildIndex[O.typeIds[_]], q = O.children[V];
    instance$5.visit(q, _, H);
  }, setDictionary = (O, _, H) => {
    var V;
    (V = O.dictionary) === null || V === void 0 || V.set(O.values[_], H);
  }, setIntervalValue = (O, _, H) => {
    O.type.unit === IntervalUnit.DAY_TIME ? setIntervalDayTime(O, _, H) : setIntervalYearMonth(O, _, H);
  }, setIntervalDayTime = ({ values: O }, _, H) => {
    O.set(H.subarray(0, 2), 2 * _);
  }, setIntervalYearMonth = ({ values: O }, _, H) => {
    O[_] = H[0] * 12 + H[1] % 12;
  }, setDurationSecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setDurationMillisecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setDurationMicrosecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setDurationNanosecond = ({ values: O }, _, H) => {
    O[_] = H;
  }, setDuration = (O, _, H) => {
    switch (O.type.unit) {
      case TimeUnit.SECOND:
        return setDurationSecond(O, _, H);
      case TimeUnit.MILLISECOND:
        return setDurationMillisecond(O, _, H);
      case TimeUnit.MICROSECOND:
        return setDurationMicrosecond(O, _, H);
      case TimeUnit.NANOSECOND:
        return setDurationNanosecond(O, _, H);
    }
  }, setFixedSizeList = (O, _, H) => {
    const { stride: V } = O, q = O.children[0], G = instance$5.getVisitFn(q);
    if (Array.isArray(H))
      for (let J = -1, Z = _ * V; ++J < V; )
        G(q, Z + J, H[J]);
    else
      for (let J = -1, Z = _ * V; ++J < V; )
        G(q, Z + J, H.get(J));
  };
  SetVisitor.prototype.visitBool = wrapSet(setBool);
  SetVisitor.prototype.visitInt = wrapSet(setInt);
  SetVisitor.prototype.visitInt8 = wrapSet(setInt);
  SetVisitor.prototype.visitInt16 = wrapSet(setInt);
  SetVisitor.prototype.visitInt32 = wrapSet(setInt);
  SetVisitor.prototype.visitInt64 = wrapSet(setInt);
  SetVisitor.prototype.visitUint8 = wrapSet(setInt);
  SetVisitor.prototype.visitUint16 = wrapSet(setInt);
  SetVisitor.prototype.visitUint32 = wrapSet(setInt);
  SetVisitor.prototype.visitUint64 = wrapSet(setInt);
  SetVisitor.prototype.visitFloat = wrapSet(setAnyFloat);
  SetVisitor.prototype.visitFloat16 = wrapSet(setFloat16);
  SetVisitor.prototype.visitFloat32 = wrapSet(setFloat);
  SetVisitor.prototype.visitFloat64 = wrapSet(setFloat);
  SetVisitor.prototype.visitUtf8 = wrapSet(setUtf8);
  SetVisitor.prototype.visitLargeUtf8 = wrapSet(setUtf8);
  SetVisitor.prototype.visitBinary = wrapSet(setBinary);
  SetVisitor.prototype.visitLargeBinary = wrapSet(setBinary);
  SetVisitor.prototype.visitFixedSizeBinary = wrapSet(setFixedSizeBinary);
  SetVisitor.prototype.visitDate = wrapSet(setDate);
  SetVisitor.prototype.visitDateDay = wrapSet(setDateDay);
  SetVisitor.prototype.visitDateMillisecond = wrapSet(setDateMillisecond);
  SetVisitor.prototype.visitTimestamp = wrapSet(setTimestamp);
  SetVisitor.prototype.visitTimestampSecond = wrapSet(setTimestampSecond);
  SetVisitor.prototype.visitTimestampMillisecond = wrapSet(setTimestampMillisecond);
  SetVisitor.prototype.visitTimestampMicrosecond = wrapSet(setTimestampMicrosecond);
  SetVisitor.prototype.visitTimestampNanosecond = wrapSet(setTimestampNanosecond);
  SetVisitor.prototype.visitTime = wrapSet(setTime);
  SetVisitor.prototype.visitTimeSecond = wrapSet(setTimeSecond);
  SetVisitor.prototype.visitTimeMillisecond = wrapSet(setTimeMillisecond);
  SetVisitor.prototype.visitTimeMicrosecond = wrapSet(setTimeMicrosecond);
  SetVisitor.prototype.visitTimeNanosecond = wrapSet(setTimeNanosecond);
  SetVisitor.prototype.visitDecimal = wrapSet(setDecimal);
  SetVisitor.prototype.visitList = wrapSet(setList);
  SetVisitor.prototype.visitStruct = wrapSet(setStruct);
  SetVisitor.prototype.visitUnion = wrapSet(setUnion);
  SetVisitor.prototype.visitDenseUnion = wrapSet(setDenseUnion);
  SetVisitor.prototype.visitSparseUnion = wrapSet(setSparseUnion);
  SetVisitor.prototype.visitDictionary = wrapSet(setDictionary);
  SetVisitor.prototype.visitInterval = wrapSet(setIntervalValue);
  SetVisitor.prototype.visitIntervalDayTime = wrapSet(setIntervalDayTime);
  SetVisitor.prototype.visitIntervalYearMonth = wrapSet(setIntervalYearMonth);
  SetVisitor.prototype.visitDuration = wrapSet(setDuration);
  SetVisitor.prototype.visitDurationSecond = wrapSet(setDurationSecond);
  SetVisitor.prototype.visitDurationMillisecond = wrapSet(setDurationMillisecond);
  SetVisitor.prototype.visitDurationMicrosecond = wrapSet(setDurationMicrosecond);
  SetVisitor.prototype.visitDurationNanosecond = wrapSet(setDurationNanosecond);
  SetVisitor.prototype.visitFixedSizeList = wrapSet(setFixedSizeList);
  SetVisitor.prototype.visitMap = wrapSet(setMap);
  const instance$5 = new SetVisitor(), kParent = Symbol.for("parent"), kRowIndex = Symbol.for("rowIndex");
  class StructRow {
    constructor(_, H) {
      return this[kParent] = _, this[kRowIndex] = H, new Proxy(this, structRowProxyHandler);
    }
    toArray() {
      return Object.values(this.toJSON());
    }
    toJSON() {
      const _ = this[kRowIndex], H = this[kParent], V = H.type.children, q = {};
      for (let G = -1, J = V.length; ++G < J; )
        q[V[G].name] = instance$4.visit(H.children[G], _);
      return q;
    }
    toString() {
      return `{${[...this].map(([_, H]) => `${valueToString(_)}: ${valueToString(H)}`).join(", ")}}`;
    }
    [Symbol.for("nodejs.util.inspect.custom")]() {
      return this.toString();
    }
    [Symbol.iterator]() {
      return new StructRowIterator(this[kParent], this[kRowIndex]);
    }
  }
  class StructRowIterator {
    constructor(_, H) {
      this.childIndex = 0, this.children = _.children, this.rowIndex = H, this.childFields = _.type.children, this.numChildren = this.childFields.length;
    }
    [Symbol.iterator]() {
      return this;
    }
    next() {
      const _ = this.childIndex;
      return _ < this.numChildren ? (this.childIndex = _ + 1, {
        done: !1,
        value: [
          this.childFields[_].name,
          instance$4.visit(this.children[_], this.rowIndex)
        ]
      }) : { done: !0, value: null };
    }
  }
  Object.defineProperties(StructRow.prototype, {
    [Symbol.toStringTag]: { enumerable: !1, configurable: !1, value: "Row" },
    [kParent]: { writable: !0, enumerable: !1, configurable: !1, value: null },
    [kRowIndex]: { writable: !0, enumerable: !1, configurable: !1, value: -1 }
  });
  class StructRowProxyHandler {
    isExtensible() {
      return !1;
    }
    deleteProperty() {
      return !1;
    }
    preventExtensions() {
      return !0;
    }
    ownKeys(_) {
      return _[kParent].type.children.map((H) => H.name);
    }
    has(_, H) {
      return _[kParent].type.children.some((V) => V.name === H);
    }
    getOwnPropertyDescriptor(_, H) {
      if (_[kParent].type.children.some((V) => V.name === H))
        return { writable: !0, enumerable: !0, configurable: !0 };
    }
    get(_, H) {
      if (Reflect.has(_, H))
        return _[H];
      const V = _[kParent].type.children.findIndex((q) => q.name === H);
      if (V !== -1) {
        const q = instance$4.visit(_[kParent].children[V], _[kRowIndex]);
        return Reflect.set(_, H, q), q;
      }
    }
    set(_, H, V) {
      const q = _[kParent].type.children.findIndex((G) => G.name === H);
      return q !== -1 ? (instance$5.visit(_[kParent].children[q], _[kRowIndex], V), Reflect.set(_, H, V)) : Reflect.has(_, H) || typeof H == "symbol" ? Reflect.set(_, H, V) : !1;
    }
  }
  const structRowProxyHandler = new StructRowProxyHandler();
  class GetVisitor extends Visitor {
  }
  function wrapGet(O) {
    return (_, H) => _.getValid(H) ? O(_, H) : null;
  }
  const epochDaysToMs = (O, _) => 864e5 * O[_], getNull = (O, _) => null, getVariableWidthBytes = (O, _, H) => {
    if (H + 1 >= _.length)
      return null;
    const V = bigIntToNumber(_[H]), q = bigIntToNumber(_[H + 1]);
    return O.subarray(V, q);
  }, getBool$1 = ({ offset: O, values: _ }, H) => {
    const V = O + H;
    return (_[V >> 3] & 1 << V % 8) !== 0;
  }, getDateDay = ({ values: O }, _) => epochDaysToMs(O, _), getDateMillisecond = ({ values: O }, _) => bigIntToNumber(O[_]), getNumeric = ({ stride: O, values: _ }, H) => _[O * H], getFloat16 = ({ stride: O, values: _ }, H) => uint16ToFloat64(_[O * H]), getBigInts = ({ values: O }, _) => O[_], getFixedSizeBinary = ({ stride: O, values: _ }, H) => _.subarray(O * H, O * (H + 1)), getBinary = ({ values: O, valueOffsets: _ }, H) => getVariableWidthBytes(O, _, H), getUtf8 = ({ values: O, valueOffsets: _ }, H) => {
    const V = getVariableWidthBytes(O, _, H);
    return V !== null ? decodeUtf8(V) : null;
  }, getInt = ({ values: O }, _) => O[_], getFloat = ({ type: O, values: _ }, H) => O.precision !== Precision.HALF ? _[H] : uint16ToFloat64(_[H]), getDate = (O, _) => O.type.unit === DateUnit.DAY ? getDateDay(O, _) : getDateMillisecond(O, _), getTimestampSecond = ({ values: O }, _) => 1e3 * bigIntToNumber(O[_]), getTimestampMillisecond = ({ values: O }, _) => bigIntToNumber(O[_]), getTimestampMicrosecond = ({ values: O }, _) => divideBigInts(O[_], BigInt(1e3)), getTimestampNanosecond = ({ values: O }, _) => divideBigInts(O[_], BigInt(1e6)), getTimestamp = (O, _) => {
    switch (O.type.unit) {
      case TimeUnit.SECOND:
        return getTimestampSecond(O, _);
      case TimeUnit.MILLISECOND:
        return getTimestampMillisecond(O, _);
      case TimeUnit.MICROSECOND:
        return getTimestampMicrosecond(O, _);
      case TimeUnit.NANOSECOND:
        return getTimestampNanosecond(O, _);
    }
  }, getTimeSecond = ({ values: O }, _) => O[_], getTimeMillisecond = ({ values: O }, _) => O[_], getTimeMicrosecond = ({ values: O }, _) => O[_], getTimeNanosecond = ({ values: O }, _) => O[_], getTime = (O, _) => {
    switch (O.type.unit) {
      case TimeUnit.SECOND:
        return getTimeSecond(O, _);
      case TimeUnit.MILLISECOND:
        return getTimeMillisecond(O, _);
      case TimeUnit.MICROSECOND:
        return getTimeMicrosecond(O, _);
      case TimeUnit.NANOSECOND:
        return getTimeNanosecond(O, _);
    }
  }, getDecimal = ({ values: O, stride: _ }, H) => BN.decimal(O.subarray(_ * H, _ * (H + 1))), getList = (O, _) => {
    const { valueOffsets: H, stride: V, children: q } = O, { [_ * V]: G, [_ * V + 1]: J } = H, te = q[0].slice(G, J - G);
    return new Vector([te]);
  }, getMap = (O, _) => {
    const { valueOffsets: H, children: V } = O, { [_]: q, [_ + 1]: G } = H, J = V[0];
    return new MapRow(J.slice(q, G - q));
  }, getStruct = (O, _) => new StructRow(O, _), getUnion = (O, _) => O.type.mode === UnionMode.Dense ? getDenseUnion(O, _) : getSparseUnion(O, _), getDenseUnion = (O, _) => {
    const H = O.type.typeIdToChildIndex[O.typeIds[_]], V = O.children[H];
    return instance$4.visit(V, O.valueOffsets[_]);
  }, getSparseUnion = (O, _) => {
    const H = O.type.typeIdToChildIndex[O.typeIds[_]], V = O.children[H];
    return instance$4.visit(V, _);
  }, getDictionary = (O, _) => {
    var H;
    return (H = O.dictionary) === null || H === void 0 ? void 0 : H.get(O.values[_]);
  }, getInterval = (O, _) => O.type.unit === IntervalUnit.DAY_TIME ? getIntervalDayTime(O, _) : getIntervalYearMonth(O, _), getIntervalDayTime = ({ values: O }, _) => O.subarray(2 * _, 2 * (_ + 1)), getIntervalYearMonth = ({ values: O }, _) => {
    const H = O[_], V = new Int32Array(2);
    return V[0] = Math.trunc(H / 12), V[1] = Math.trunc(H % 12), V;
  }, getDurationSecond = ({ values: O }, _) => O[_], getDurationMillisecond = ({ values: O }, _) => O[_], getDurationMicrosecond = ({ values: O }, _) => O[_], getDurationNanosecond = ({ values: O }, _) => O[_], getDuration = (O, _) => {
    switch (O.type.unit) {
      case TimeUnit.SECOND:
        return getDurationSecond(O, _);
      case TimeUnit.MILLISECOND:
        return getDurationMillisecond(O, _);
      case TimeUnit.MICROSECOND:
        return getDurationMicrosecond(O, _);
      case TimeUnit.NANOSECOND:
        return getDurationNanosecond(O, _);
    }
  }, getFixedSizeList = (O, _) => {
    const { stride: H, children: V } = O, G = V[0].slice(_ * H, H);
    return new Vector([G]);
  };
  GetVisitor.prototype.visitNull = wrapGet(getNull);
  GetVisitor.prototype.visitBool = wrapGet(getBool$1);
  GetVisitor.prototype.visitInt = wrapGet(getInt);
  GetVisitor.prototype.visitInt8 = wrapGet(getNumeric);
  GetVisitor.prototype.visitInt16 = wrapGet(getNumeric);
  GetVisitor.prototype.visitInt32 = wrapGet(getNumeric);
  GetVisitor.prototype.visitInt64 = wrapGet(getBigInts);
  GetVisitor.prototype.visitUint8 = wrapGet(getNumeric);
  GetVisitor.prototype.visitUint16 = wrapGet(getNumeric);
  GetVisitor.prototype.visitUint32 = wrapGet(getNumeric);
  GetVisitor.prototype.visitUint64 = wrapGet(getBigInts);
  GetVisitor.prototype.visitFloat = wrapGet(getFloat);
  GetVisitor.prototype.visitFloat16 = wrapGet(getFloat16);
  GetVisitor.prototype.visitFloat32 = wrapGet(getNumeric);
  GetVisitor.prototype.visitFloat64 = wrapGet(getNumeric);
  GetVisitor.prototype.visitUtf8 = wrapGet(getUtf8);
  GetVisitor.prototype.visitLargeUtf8 = wrapGet(getUtf8);
  GetVisitor.prototype.visitBinary = wrapGet(getBinary);
  GetVisitor.prototype.visitLargeBinary = wrapGet(getBinary);
  GetVisitor.prototype.visitFixedSizeBinary = wrapGet(getFixedSizeBinary);
  GetVisitor.prototype.visitDate = wrapGet(getDate);
  GetVisitor.prototype.visitDateDay = wrapGet(getDateDay);
  GetVisitor.prototype.visitDateMillisecond = wrapGet(getDateMillisecond);
  GetVisitor.prototype.visitTimestamp = wrapGet(getTimestamp);
  GetVisitor.prototype.visitTimestampSecond = wrapGet(getTimestampSecond);
  GetVisitor.prototype.visitTimestampMillisecond = wrapGet(getTimestampMillisecond);
  GetVisitor.prototype.visitTimestampMicrosecond = wrapGet(getTimestampMicrosecond);
  GetVisitor.prototype.visitTimestampNanosecond = wrapGet(getTimestampNanosecond);
  GetVisitor.prototype.visitTime = wrapGet(getTime);
  GetVisitor.prototype.visitTimeSecond = wrapGet(getTimeSecond);
  GetVisitor.prototype.visitTimeMillisecond = wrapGet(getTimeMillisecond);
  GetVisitor.prototype.visitTimeMicrosecond = wrapGet(getTimeMicrosecond);
  GetVisitor.prototype.visitTimeNanosecond = wrapGet(getTimeNanosecond);
  GetVisitor.prototype.visitDecimal = wrapGet(getDecimal);
  GetVisitor.prototype.visitList = wrapGet(getList);
  GetVisitor.prototype.visitStruct = wrapGet(getStruct);
  GetVisitor.prototype.visitUnion = wrapGet(getUnion);
  GetVisitor.prototype.visitDenseUnion = wrapGet(getDenseUnion);
  GetVisitor.prototype.visitSparseUnion = wrapGet(getSparseUnion);
  GetVisitor.prototype.visitDictionary = wrapGet(getDictionary);
  GetVisitor.prototype.visitInterval = wrapGet(getInterval);
  GetVisitor.prototype.visitIntervalDayTime = wrapGet(getIntervalDayTime);
  GetVisitor.prototype.visitIntervalYearMonth = wrapGet(getIntervalYearMonth);
  GetVisitor.prototype.visitDuration = wrapGet(getDuration);
  GetVisitor.prototype.visitDurationSecond = wrapGet(getDurationSecond);
  GetVisitor.prototype.visitDurationMillisecond = wrapGet(getDurationMillisecond);
  GetVisitor.prototype.visitDurationMicrosecond = wrapGet(getDurationMicrosecond);
  GetVisitor.prototype.visitDurationNanosecond = wrapGet(getDurationNanosecond);
  GetVisitor.prototype.visitFixedSizeList = wrapGet(getFixedSizeList);
  GetVisitor.prototype.visitMap = wrapGet(getMap);
  const instance$4 = new GetVisitor(), kKeys = Symbol.for("keys"), kVals = Symbol.for("vals"), kKeysAsStrings = Symbol.for("kKeysAsStrings"), _kKeysAsStrings = Symbol.for("_kKeysAsStrings");
  class MapRow {
    constructor(_) {
      return this[kKeys] = new Vector([_.children[0]]).memoize(), this[kVals] = _.children[1], new Proxy(this, new MapRowProxyHandler());
    }
    /** @ignore */
    get [kKeysAsStrings]() {
      return this[_kKeysAsStrings] || (this[_kKeysAsStrings] = Array.from(this[kKeys].toArray(), String));
    }
    [Symbol.iterator]() {
      return new MapRowIterator(this[kKeys], this[kVals]);
    }
    get size() {
      return this[kKeys].length;
    }
    toArray() {
      return Object.values(this.toJSON());
    }
    toJSON() {
      const _ = this[kKeys], H = this[kVals], V = {};
      for (let q = -1, G = _.length; ++q < G; )
        V[_.get(q)] = instance$4.visit(H, q);
      return V;
    }
    toString() {
      return `{${[...this].map(([_, H]) => `${valueToString(_)}: ${valueToString(H)}`).join(", ")}}`;
    }
    [Symbol.for("nodejs.util.inspect.custom")]() {
      return this.toString();
    }
  }
  class MapRowIterator {
    constructor(_, H) {
      this.keys = _, this.vals = H, this.keyIndex = 0, this.numKeys = _.length;
    }
    [Symbol.iterator]() {
      return this;
    }
    next() {
      const _ = this.keyIndex;
      return _ === this.numKeys ? { done: !0, value: null } : (this.keyIndex++, {
        done: !1,
        value: [
          this.keys.get(_),
          instance$4.visit(this.vals, _)
        ]
      });
    }
  }
  class MapRowProxyHandler {
    isExtensible() {
      return !1;
    }
    deleteProperty() {
      return !1;
    }
    preventExtensions() {
      return !0;
    }
    ownKeys(_) {
      return _[kKeysAsStrings];
    }
    has(_, H) {
      return _[kKeysAsStrings].includes(H);
    }
    getOwnPropertyDescriptor(_, H) {
      if (_[kKeysAsStrings].indexOf(H) !== -1)
        return { writable: !0, enumerable: !0, configurable: !0 };
    }
    get(_, H) {
      if (Reflect.has(_, H))
        return _[H];
      const V = _[kKeysAsStrings].indexOf(H);
      if (V !== -1) {
        const q = instance$4.visit(Reflect.get(_, kVals), V);
        return Reflect.set(_, H, q), q;
      }
    }
    set(_, H, V) {
      const q = _[kKeysAsStrings].indexOf(H);
      return q !== -1 ? (instance$5.visit(Reflect.get(_, kVals), q, V), Reflect.set(_, H, V)) : Reflect.has(_, H) ? Reflect.set(_, H, V) : !1;
    }
  }
  Object.defineProperties(MapRow.prototype, {
    [Symbol.toStringTag]: { enumerable: !1, configurable: !1, value: "Row" },
    [kKeys]: { writable: !0, enumerable: !1, configurable: !1, value: null },
    [kVals]: { writable: !0, enumerable: !1, configurable: !1, value: null },
    [_kKeysAsStrings]: { writable: !0, enumerable: !1, configurable: !1, value: null }
  });
  let tmp;
  function clampRange(O, _, H, V) {
    const { length: q = 0 } = O;
    let G = typeof _ != "number" ? 0 : _, J = typeof H != "number" ? q : H;
    return G < 0 && (G = (G % q + q) % q), J < 0 && (J = (J % q + q) % q), J < G && (tmp = G, G = J, J = tmp), J > q && (J = q), V ? V(O, G, J) : [G, J];
  }
  const wrapIndex = (O, _) => O < 0 ? _ + O : O, isNaNFast = (O) => O !== O;
  function createElementComparator(O) {
    if (typeof O !== "object" || O === null)
      return isNaNFast(O) ? isNaNFast : (H) => H === O;
    if (O instanceof Date) {
      const H = O.valueOf();
      return (V) => V instanceof Date ? V.valueOf() === H : !1;
    }
    return ArrayBuffer.isView(O) ? (H) => H ? compareArrayLike(O, H) : !1 : O instanceof Map ? createMapComparator(O) : Array.isArray(O) ? createArrayLikeComparator(O) : O instanceof Vector ? createVectorComparator(O) : createObjectComparator(O, !0);
  }
  function createArrayLikeComparator(O) {
    const _ = [];
    for (let H = -1, V = O.length; ++H < V; )
      _[H] = createElementComparator(O[H]);
    return createSubElementsComparator(_);
  }
  function createMapComparator(O) {
    let _ = -1;
    const H = [];
    for (const V of O.values())
      H[++_] = createElementComparator(V);
    return createSubElementsComparator(H);
  }
  function createVectorComparator(O) {
    const _ = [];
    for (let H = -1, V = O.length; ++H < V; )
      _[H] = createElementComparator(O.get(H));
    return createSubElementsComparator(_);
  }
  function createObjectComparator(O, _ = !1) {
    const H = Object.keys(O);
    if (!_ && H.length === 0)
      return () => !1;
    const V = [];
    for (let q = -1, G = H.length; ++q < G; )
      V[q] = createElementComparator(O[H[q]]);
    return createSubElementsComparator(V, H);
  }
  function createSubElementsComparator(O, _) {
    return (H) => {
      if (!H || typeof H != "object")
        return !1;
      switch (H.constructor) {
        case Array:
          return compareArray(O, H);
        case Map:
          return compareObject(O, H, H.keys());
        case MapRow:
        case StructRow:
        case Object:
        case void 0:
          return compareObject(O, H, _ || Object.keys(H));
      }
      return H instanceof Vector ? compareVector(O, H) : !1;
    };
  }
  function compareArray(O, _) {
    const H = O.length;
    if (_.length !== H)
      return !1;
    for (let V = -1; ++V < H; )
      if (!O[V](_[V]))
        return !1;
    return !0;
  }
  function compareVector(O, _) {
    const H = O.length;
    if (_.length !== H)
      return !1;
    for (let V = -1; ++V < H; )
      if (!O[V](_.get(V)))
        return !1;
    return !0;
  }
  function compareObject(O, _, H) {
    const V = H[Symbol.iterator](), q = _ instanceof Map ? _.keys() : Object.keys(_)[Symbol.iterator](), G = _ instanceof Map ? _.values() : Object.values(_)[Symbol.iterator]();
    let J = 0;
    const Z = O.length;
    let te = G.next(), ne = V.next(), ee = q.next();
    for (; J < Z && !ne.done && !ee.done && !te.done && !(ne.value !== ee.value || !O[J](te.value)); ++J, ne = V.next(), ee = q.next(), te = G.next())
      ;
    return J === Z && ne.done && ee.done && te.done ? !0 : (V.return && V.return(), q.return && q.return(), G.return && G.return(), !1);
  }
  function getBool(O, _, H, V) {
    return (H & 1 << V) !== 0;
  }
  function getBit(O, _, H, V) {
    return (H & 1 << V) >> V;
  }
  function truncateBitmap(O, _, H) {
    const V = H.byteLength + 7 & -8;
    if (O > 0 || H.byteLength < V) {
      const q = new Uint8Array(V);
      return q.set(O % 8 === 0 ? H.subarray(O >> 3) : (
        // Otherwise iterate each bit from the offset and return a new one
        packBools(new BitIterator(H, O, _, null, getBool)).subarray(0, V)
      )), q;
    }
    return H;
  }
  function packBools(O) {
    const _ = [];
    let H = 0, V = 0, q = 0;
    for (const J of O)
      J && (q |= 1 << V), ++V === 8 && (_[H++] = q, q = V = 0);
    (H === 0 || V > 0) && (_[H++] = q);
    const G = new Uint8Array(_.length + 7 & -8);
    return G.set(_), G;
  }
  class BitIterator {
    constructor(_, H, V, q, G) {
      this.bytes = _, this.length = V, this.context = q, this.get = G, this.bit = H % 8, this.byteIndex = H >> 3, this.byte = _[this.byteIndex++], this.index = 0;
    }
    next() {
      return this.index < this.length ? (this.bit === 8 && (this.bit = 0, this.byte = this.bytes[this.byteIndex++]), {
        value: this.get(this.context, this.index++, this.byte, this.bit++)
      }) : { done: !0, value: null };
    }
    [Symbol.iterator]() {
      return this;
    }
  }
  function popcnt_bit_range(O, _, H) {
    if (H - _ <= 0)
      return 0;
    if (H - _ < 8) {
      let G = 0;
      for (const J of new BitIterator(O, _, H - _, O, getBit))
        G += J;
      return G;
    }
    const V = H >> 3 << 3, q = _ + (_ % 8 === 0 ? 0 : 8 - _ % 8);
    return (
      // Get the popcnt of bits between the left hand side, and the next highest multiple of 8
      popcnt_bit_range(O, _, q) + // Get the popcnt of bits between the right hand side, and the next lowest multiple of 8
      popcnt_bit_range(O, V, H) + // Get the popcnt of all bits between the left and right hand sides' multiples of 8
      popcnt_array(O, q >> 3, V - q >> 3)
    );
  }
  function popcnt_array(O, _, H) {
    let V = 0, q = Math.trunc(_);
    const G = new DataView(O.buffer, O.byteOffset, O.byteLength), J = H === void 0 ? O.byteLength : q + H;
    for (; J - q >= 4; )
      V += popcnt_uint32(G.getUint32(q)), q += 4;
    for (; J - q >= 2; )
      V += popcnt_uint32(G.getUint16(q)), q += 2;
    for (; J - q >= 1; )
      V += popcnt_uint32(G.getUint8(q)), q += 1;
    return V;
  }
  function popcnt_uint32(O) {
    let _ = Math.trunc(O);
    return _ = _ - (_ >>> 1 & 1431655765), _ = (_ & 858993459) + (_ >>> 2 & 858993459), (_ + (_ >>> 4) & 252645135) * 16843009 >>> 24;
  }
  const kUnknownNullCount = -1;
  class Data {
    get typeId() {
      return this.type.typeId;
    }
    get ArrayType() {
      return this.type.ArrayType;
    }
    get buffers() {
      return [this.valueOffsets, this.values, this.nullBitmap, this.typeIds];
    }
    get nullable() {
      if (this._nullCount !== 0) {
        const { type: _ } = this;
        return DataType.isSparseUnion(_) ? this.children.some((H) => H.nullable) : DataType.isDenseUnion(_) ? this.children.some((H) => H.nullable) : this.nullBitmap && this.nullBitmap.byteLength > 0;
      }
      return !0;
    }
    get byteLength() {
      let _ = 0;
      const { valueOffsets: H, values: V, nullBitmap: q, typeIds: G } = this;
      return H && (_ += H.byteLength), V && (_ += V.byteLength), q && (_ += q.byteLength), G && (_ += G.byteLength), this.children.reduce((J, Z) => J + Z.byteLength, _);
    }
    get nullCount() {
      if (DataType.isUnion(this.type))
        return this.children.reduce((V, q) => V + q.nullCount, 0);
      let _ = this._nullCount, H;
      return _ <= kUnknownNullCount && (H = this.nullBitmap) && (this._nullCount = _ = H.length === 0 ? (
        // no null bitmap, so all values are valid
        0
      ) : this.length - popcnt_bit_range(H, this.offset, this.offset + this.length)), _;
    }
    constructor(_, H, V, q, G, J = [], Z) {
      this.type = _, this.children = J, this.dictionary = Z, this.offset = Math.floor(Math.max(H || 0, 0)), this.length = Math.floor(Math.max(V || 0, 0)), this._nullCount = Math.floor(Math.max(q || 0, -1));
      let te;
      G instanceof Data ? (this.stride = G.stride, this.values = G.values, this.typeIds = G.typeIds, this.nullBitmap = G.nullBitmap, this.valueOffsets = G.valueOffsets) : (this.stride = strideForType(_), G && ((te = G[0]) && (this.valueOffsets = te), (te = G[1]) && (this.values = te), (te = G[2]) && (this.nullBitmap = te), (te = G[3]) && (this.typeIds = te)));
    }
    getValid(_) {
      const { type: H } = this;
      if (DataType.isUnion(H)) {
        const V = H, q = this.children[V.typeIdToChildIndex[this.typeIds[_]]], G = V.mode === UnionMode.Dense ? this.valueOffsets[_] : _;
        return q.getValid(G);
      }
      if (this.nullable && this.nullCount > 0) {
        const V = this.offset + _;
        return (this.nullBitmap[V >> 3] & 1 << V % 8) !== 0;
      }
      return !0;
    }
    setValid(_, H) {
      let V;
      const { type: q } = this;
      if (DataType.isUnion(q)) {
        const G = q, J = this.children[G.typeIdToChildIndex[this.typeIds[_]]], Z = G.mode === UnionMode.Dense ? this.valueOffsets[_] : _;
        V = J.getValid(Z), J.setValid(Z, H);
      } else {
        let { nullBitmap: G } = this;
        const { offset: J, length: Z } = this, te = J + _, ne = 1 << te % 8, ee = te >> 3;
        (!G || G.byteLength <= ee) && (G = new Uint8Array((J + Z + 63 & -64) >> 3).fill(255), this.nullCount > 0 ? (G.set(truncateBitmap(J, Z, this.nullBitmap), 0), Object.assign(this, { nullBitmap: G })) : Object.assign(this, { nullBitmap: G, _nullCount: 0 }));
        const ae = G[ee];
        V = (ae & ne) !== 0, G[ee] = H ? ae | ne : ae & ~ne;
      }
      return V !== !!H && (this._nullCount = this.nullCount + (H ? -1 : 1)), H;
    }
    clone(_ = this.type, H = this.offset, V = this.length, q = this._nullCount, G = this, J = this.children) {
      return new Data(_, H, V, q, G, J, this.dictionary);
    }
    slice(_, H) {
      const { stride: V, typeId: q, children: G } = this, J = +(this._nullCount === 0) - 1, Z = q === 16 ? V : 1, te = this._sliceBuffers(_, H, V, q);
      return this.clone(
        this.type,
        this.offset + _,
        H,
        J,
        te,
        // Don't slice children if we have value offsets (the variable-width types)
        G.length === 0 || this.valueOffsets ? G : this._sliceChildren(G, Z * _, Z * H)
      );
    }
    _changeLengthAndBackfillNullBitmap(_) {
      if (this.typeId === Type.Null)
        return this.clone(this.type, 0, _, 0);
      const { length: H, nullCount: V } = this, q = new Uint8Array((_ + 63 & -64) >> 3).fill(255, 0, H >> 3);
      q[H >> 3] = (1 << H - (H & -8)) - 1, V > 0 && q.set(truncateBitmap(this.offset, H, this.nullBitmap), 0);
      const G = this.buffers;
      return G[BufferType.VALIDITY] = q, this.clone(this.type, 0, _, V + (_ - H), G);
    }
    _sliceBuffers(_, H, V, q) {
      let G;
      const { buffers: J } = this;
      return (G = J[BufferType.TYPE]) && (J[BufferType.TYPE] = G.subarray(_, _ + H)), (G = J[BufferType.OFFSET]) && (J[BufferType.OFFSET] = G.subarray(_, _ + H + 1)) || // Otherwise if no offsets, slice the data buffer. Don't slice the data vector for Booleans, since the offset goes by bits not bytes
      (G = J[BufferType.DATA]) && (J[BufferType.DATA] = q === 6 ? G : G.subarray(V * _, V * (_ + H))), J;
    }
    _sliceChildren(_, H, V) {
      return _.map((q) => q.slice(H, V));
    }
  }
  Data.prototype.children = Object.freeze([]);
  class MakeDataVisitor extends Visitor {
    visit(_) {
      return this.getVisitFn(_.type).call(this, _);
    }
    visitNull(_) {
      const { ["type"]: H, ["offset"]: V = 0, ["length"]: q = 0 } = _;
      return new Data(H, V, q, q);
    }
    visitBool(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length >> 3, ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitInt(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length, ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitFloat(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length, ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitUtf8(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.data), G = toUint8Array(_.nullBitmap), J = toInt32Array(_.valueOffsets), { ["length"]: Z = J.length - 1, ["nullCount"]: te = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, Z, te, [J, q, G]);
    }
    visitLargeUtf8(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.data), G = toUint8Array(_.nullBitmap), J = toBigInt64Array(_.valueOffsets), { ["length"]: Z = J.length - 1, ["nullCount"]: te = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, Z, te, [J, q, G]);
    }
    visitBinary(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.data), G = toUint8Array(_.nullBitmap), J = toInt32Array(_.valueOffsets), { ["length"]: Z = J.length - 1, ["nullCount"]: te = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, Z, te, [J, q, G]);
    }
    visitLargeBinary(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.data), G = toUint8Array(_.nullBitmap), J = toBigInt64Array(_.valueOffsets), { ["length"]: Z = J.length - 1, ["nullCount"]: te = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, Z, te, [J, q, G]);
    }
    visitFixedSizeBinary(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length / strideForType(H), ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitDate(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length / strideForType(H), ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitTimestamp(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length / strideForType(H), ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitTime(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length / strideForType(H), ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitDecimal(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length / strideForType(H), ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitList(_) {
      const { ["type"]: H, ["offset"]: V = 0, ["child"]: q } = _, G = toUint8Array(_.nullBitmap), J = toInt32Array(_.valueOffsets), { ["length"]: Z = J.length - 1, ["nullCount"]: te = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, Z, te, [J, void 0, G], [q]);
    }
    visitStruct(_) {
      const { ["type"]: H, ["offset"]: V = 0, ["children"]: q = [] } = _, G = toUint8Array(_.nullBitmap), { length: J = q.reduce((te, { length: ne }) => Math.max(te, ne), 0), nullCount: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, void 0, G], q);
    }
    visitUnion(_) {
      const { ["type"]: H, ["offset"]: V = 0, ["children"]: q = [] } = _, G = toArrayBufferView(H.ArrayType, _.typeIds), { ["length"]: J = G.length, ["nullCount"]: Z = -1 } = _;
      if (DataType.isSparseUnion(H))
        return new Data(H, V, J, Z, [void 0, void 0, void 0, G], q);
      const te = toInt32Array(_.valueOffsets);
      return new Data(H, V, J, Z, [te, void 0, void 0, G], q);
    }
    visitDictionary(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.indices.ArrayType, _.data), { ["dictionary"]: J = new Vector([new MakeDataVisitor().visit({ type: H.dictionary })]) } = _, { ["length"]: Z = G.length, ["nullCount"]: te = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, Z, te, [void 0, G, q], [], J);
    }
    visitInterval(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length / strideForType(H), ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitDuration(_) {
      const { ["type"]: H, ["offset"]: V = 0 } = _, q = toUint8Array(_.nullBitmap), G = toArrayBufferView(H.ArrayType, _.data), { ["length"]: J = G.length, ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, G, q]);
    }
    visitFixedSizeList(_) {
      const { ["type"]: H, ["offset"]: V = 0, ["child"]: q = new MakeDataVisitor().visit({ type: H.valueType }) } = _, G = toUint8Array(_.nullBitmap), { ["length"]: J = q.length / strideForType(H), ["nullCount"]: Z = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, J, Z, [void 0, void 0, G], [q]);
    }
    visitMap(_) {
      const { ["type"]: H, ["offset"]: V = 0, ["child"]: q = new MakeDataVisitor().visit({ type: H.childType }) } = _, G = toUint8Array(_.nullBitmap), J = toInt32Array(_.valueOffsets), { ["length"]: Z = J.length - 1, ["nullCount"]: te = _.nullBitmap ? -1 : 0 } = _;
      return new Data(H, V, Z, te, [J, void 0, G], [q]);
    }
  }
  const makeDataVisitor = new MakeDataVisitor();
  function makeData(O) {
    return makeDataVisitor.visit(O);
  }
  class ChunkedIterator {
    constructor(_ = 0, H) {
      this.numChunks = _, this.getChunkIterator = H, this.chunkIndex = 0, this.chunkIterator = this.getChunkIterator(0);
    }
    next() {
      for (; this.chunkIndex < this.numChunks; ) {
        const _ = this.chunkIterator.next();
        if (!_.done)
          return _;
        ++this.chunkIndex < this.numChunks && (this.chunkIterator = this.getChunkIterator(this.chunkIndex));
      }
      return { done: !0, value: null };
    }
    [Symbol.iterator]() {
      return this;
    }
  }
  function computeChunkNullable(O) {
    return O.some((_) => _.nullable);
  }
  function computeChunkNullCounts(O) {
    return O.reduce((_, H) => _ + H.nullCount, 0);
  }
  function computeChunkOffsets(O) {
    return O.reduce((_, H, V) => (_[V + 1] = _[V] + H.length, _), new Uint32Array(O.length + 1));
  }
  function sliceChunks(O, _, H, V) {
    const q = [];
    for (let G = -1, J = O.length; ++G < J; ) {
      const Z = O[G], te = _[G], { length: ne } = Z;
      if (te >= V)
        break;
      if (H >= te + ne)
        continue;
      if (te >= H && te + ne <= V) {
        q.push(Z);
        continue;
      }
      const ee = Math.max(0, H - te), ae = Math.min(V - te, ne);
      q.push(Z.slice(ee, ae - ee));
    }
    return q.length === 0 && q.push(O[0].slice(0, 0)), q;
  }
  function binarySearch(O, _, H, V) {
    let q = 0, G = 0, J = _.length - 1;
    do {
      if (q >= J - 1)
        return H < _[J] ? V(O, q, H - _[q]) : null;
      G = q + Math.trunc((J - q) * 0.5), H < _[G] ? J = G : q = G;
    } while (q < J);
  }
  function isChunkedValid(O, _) {
    return O.getValid(_);
  }
  function wrapChunkedCall1(O) {
    function _(H, V, q) {
      return O(H[V], q);
    }
    return function(H) {
      const V = this.data;
      return binarySearch(V, this._offsets, H, _);
    };
  }
  function wrapChunkedCall2(O) {
    let _;
    function H(V, q, G) {
      return O(V[q], G, _);
    }
    return function(V, q) {
      const G = this.data;
      _ = q;
      const J = binarySearch(G, this._offsets, V, H);
      return _ = void 0, J;
    };
  }
  function wrapChunkedIndexOf(O) {
    let _;
    function H(V, q, G) {
      let J = G, Z = 0, te = 0;
      for (let ne = q - 1, ee = V.length; ++ne < ee; ) {
        const ae = V[ne];
        if (~(Z = O(ae, _, J)))
          return te + Z;
        J = 0, te += ae.length;
      }
      return -1;
    }
    return function(V, q) {
      _ = V;
      const G = this.data, J = typeof q != "number" ? H(G, 0, 0) : binarySearch(G, this._offsets, q, H);
      return _ = void 0, J;
    };
  }
  class IndexOfVisitor extends Visitor {
  }
  function nullIndexOf(O, _) {
    return _ === null && O.length > 0 ? 0 : -1;
  }
  function indexOfNull(O, _) {
    const { nullBitmap: H } = O;
    if (!H || O.nullCount <= 0)
      return -1;
    let V = 0;
    for (const q of new BitIterator(H, O.offset + (_ || 0), O.length, H, getBool)) {
      if (!q)
        return V;
      ++V;
    }
    return -1;
  }
  function indexOfValue(O, _, H) {
    if (_ === void 0)
      return -1;
    if (_ === null)
      switch (O.typeId) {
        // Unions don't have a nullBitmap of its own, so compare the `searchElement` to `get()`.
        case Type.Union:
          break;
        // Dictionaries do have a nullBitmap, but their dictionary could also have null elements.
        case Type.Dictionary:
          break;
        // All other types can iterate the null bitmap
        default:
          return indexOfNull(O, H);
      }
    const V = instance$4.getVisitFn(O), q = createElementComparator(_);
    for (let G = (H || 0) - 1, J = O.length; ++G < J; )
      if (q(V(O, G)))
        return G;
    return -1;
  }
  function indexOfUnion(O, _, H) {
    const V = instance$4.getVisitFn(O), q = createElementComparator(_);
    for (let G = (H || 0) - 1, J = O.length; ++G < J; )
      if (q(V(O, G)))
        return G;
    return -1;
  }
  IndexOfVisitor.prototype.visitNull = nullIndexOf;
  IndexOfVisitor.prototype.visitBool = indexOfValue;
  IndexOfVisitor.prototype.visitInt = indexOfValue;
  IndexOfVisitor.prototype.visitInt8 = indexOfValue;
  IndexOfVisitor.prototype.visitInt16 = indexOfValue;
  IndexOfVisitor.prototype.visitInt32 = indexOfValue;
  IndexOfVisitor.prototype.visitInt64 = indexOfValue;
  IndexOfVisitor.prototype.visitUint8 = indexOfValue;
  IndexOfVisitor.prototype.visitUint16 = indexOfValue;
  IndexOfVisitor.prototype.visitUint32 = indexOfValue;
  IndexOfVisitor.prototype.visitUint64 = indexOfValue;
  IndexOfVisitor.prototype.visitFloat = indexOfValue;
  IndexOfVisitor.prototype.visitFloat16 = indexOfValue;
  IndexOfVisitor.prototype.visitFloat32 = indexOfValue;
  IndexOfVisitor.prototype.visitFloat64 = indexOfValue;
  IndexOfVisitor.prototype.visitUtf8 = indexOfValue;
  IndexOfVisitor.prototype.visitLargeUtf8 = indexOfValue;
  IndexOfVisitor.prototype.visitBinary = indexOfValue;
  IndexOfVisitor.prototype.visitLargeBinary = indexOfValue;
  IndexOfVisitor.prototype.visitFixedSizeBinary = indexOfValue;
  IndexOfVisitor.prototype.visitDate = indexOfValue;
  IndexOfVisitor.prototype.visitDateDay = indexOfValue;
  IndexOfVisitor.prototype.visitDateMillisecond = indexOfValue;
  IndexOfVisitor.prototype.visitTimestamp = indexOfValue;
  IndexOfVisitor.prototype.visitTimestampSecond = indexOfValue;
  IndexOfVisitor.prototype.visitTimestampMillisecond = indexOfValue;
  IndexOfVisitor.prototype.visitTimestampMicrosecond = indexOfValue;
  IndexOfVisitor.prototype.visitTimestampNanosecond = indexOfValue;
  IndexOfVisitor.prototype.visitTime = indexOfValue;
  IndexOfVisitor.prototype.visitTimeSecond = indexOfValue;
  IndexOfVisitor.prototype.visitTimeMillisecond = indexOfValue;
  IndexOfVisitor.prototype.visitTimeMicrosecond = indexOfValue;
  IndexOfVisitor.prototype.visitTimeNanosecond = indexOfValue;
  IndexOfVisitor.prototype.visitDecimal = indexOfValue;
  IndexOfVisitor.prototype.visitList = indexOfValue;
  IndexOfVisitor.prototype.visitStruct = indexOfValue;
  IndexOfVisitor.prototype.visitUnion = indexOfValue;
  IndexOfVisitor.prototype.visitDenseUnion = indexOfUnion;
  IndexOfVisitor.prototype.visitSparseUnion = indexOfUnion;
  IndexOfVisitor.prototype.visitDictionary = indexOfValue;
  IndexOfVisitor.prototype.visitInterval = indexOfValue;
  IndexOfVisitor.prototype.visitIntervalDayTime = indexOfValue;
  IndexOfVisitor.prototype.visitIntervalYearMonth = indexOfValue;
  IndexOfVisitor.prototype.visitDuration = indexOfValue;
  IndexOfVisitor.prototype.visitDurationSecond = indexOfValue;
  IndexOfVisitor.prototype.visitDurationMillisecond = indexOfValue;
  IndexOfVisitor.prototype.visitDurationMicrosecond = indexOfValue;
  IndexOfVisitor.prototype.visitDurationNanosecond = indexOfValue;
  IndexOfVisitor.prototype.visitFixedSizeList = indexOfValue;
  IndexOfVisitor.prototype.visitMap = indexOfValue;
  const instance$3 = new IndexOfVisitor();
  class IteratorVisitor extends Visitor {
  }
  function vectorIterator(O) {
    const { type: _ } = O;
    if (O.nullCount === 0 && O.stride === 1 && // Don't defer to native iterator for timestamps since Numbers are expected
    // (DataType.isTimestamp(type)) && type.unit === TimeUnit.MILLISECOND ||
    (DataType.isInt(_) && _.bitWidth !== 64 || DataType.isTime(_) && _.bitWidth !== 64 || DataType.isFloat(_) && _.precision !== Precision.HALF))
      return new ChunkedIterator(O.data.length, (V) => {
        const q = O.data[V];
        return q.values.subarray(0, q.length)[Symbol.iterator]();
      });
    let H = 0;
    return new ChunkedIterator(O.data.length, (V) => {
      const G = O.data[V].length, J = O.slice(H, H + G);
      return H += G, new VectorIterator(J);
    });
  }
  class VectorIterator {
    constructor(_) {
      this.vector = _, this.index = 0;
    }
    next() {
      return this.index < this.vector.length ? {
        value: this.vector.get(this.index++)
      } : { done: !0, value: null };
    }
    [Symbol.iterator]() {
      return this;
    }
  }
  IteratorVisitor.prototype.visitNull = vectorIterator;
  IteratorVisitor.prototype.visitBool = vectorIterator;
  IteratorVisitor.prototype.visitInt = vectorIterator;
  IteratorVisitor.prototype.visitInt8 = vectorIterator;
  IteratorVisitor.prototype.visitInt16 = vectorIterator;
  IteratorVisitor.prototype.visitInt32 = vectorIterator;
  IteratorVisitor.prototype.visitInt64 = vectorIterator;
  IteratorVisitor.prototype.visitUint8 = vectorIterator;
  IteratorVisitor.prototype.visitUint16 = vectorIterator;
  IteratorVisitor.prototype.visitUint32 = vectorIterator;
  IteratorVisitor.prototype.visitUint64 = vectorIterator;
  IteratorVisitor.prototype.visitFloat = vectorIterator;
  IteratorVisitor.prototype.visitFloat16 = vectorIterator;
  IteratorVisitor.prototype.visitFloat32 = vectorIterator;
  IteratorVisitor.prototype.visitFloat64 = vectorIterator;
  IteratorVisitor.prototype.visitUtf8 = vectorIterator;
  IteratorVisitor.prototype.visitLargeUtf8 = vectorIterator;
  IteratorVisitor.prototype.visitBinary = vectorIterator;
  IteratorVisitor.prototype.visitLargeBinary = vectorIterator;
  IteratorVisitor.prototype.visitFixedSizeBinary = vectorIterator;
  IteratorVisitor.prototype.visitDate = vectorIterator;
  IteratorVisitor.prototype.visitDateDay = vectorIterator;
  IteratorVisitor.prototype.visitDateMillisecond = vectorIterator;
  IteratorVisitor.prototype.visitTimestamp = vectorIterator;
  IteratorVisitor.prototype.visitTimestampSecond = vectorIterator;
  IteratorVisitor.prototype.visitTimestampMillisecond = vectorIterator;
  IteratorVisitor.prototype.visitTimestampMicrosecond = vectorIterator;
  IteratorVisitor.prototype.visitTimestampNanosecond = vectorIterator;
  IteratorVisitor.prototype.visitTime = vectorIterator;
  IteratorVisitor.prototype.visitTimeSecond = vectorIterator;
  IteratorVisitor.prototype.visitTimeMillisecond = vectorIterator;
  IteratorVisitor.prototype.visitTimeMicrosecond = vectorIterator;
  IteratorVisitor.prototype.visitTimeNanosecond = vectorIterator;
  IteratorVisitor.prototype.visitDecimal = vectorIterator;
  IteratorVisitor.prototype.visitList = vectorIterator;
  IteratorVisitor.prototype.visitStruct = vectorIterator;
  IteratorVisitor.prototype.visitUnion = vectorIterator;
  IteratorVisitor.prototype.visitDenseUnion = vectorIterator;
  IteratorVisitor.prototype.visitSparseUnion = vectorIterator;
  IteratorVisitor.prototype.visitDictionary = vectorIterator;
  IteratorVisitor.prototype.visitInterval = vectorIterator;
  IteratorVisitor.prototype.visitIntervalDayTime = vectorIterator;
  IteratorVisitor.prototype.visitIntervalYearMonth = vectorIterator;
  IteratorVisitor.prototype.visitDuration = vectorIterator;
  IteratorVisitor.prototype.visitDurationSecond = vectorIterator;
  IteratorVisitor.prototype.visitDurationMillisecond = vectorIterator;
  IteratorVisitor.prototype.visitDurationMicrosecond = vectorIterator;
  IteratorVisitor.prototype.visitDurationNanosecond = vectorIterator;
  IteratorVisitor.prototype.visitFixedSizeList = vectorIterator;
  IteratorVisitor.prototype.visitMap = vectorIterator;
  const instance$2 = new IteratorVisitor();
  var _a$2;
  const visitorsByTypeId = {}, vectorPrototypesByTypeId = {};
  class Vector {
    constructor(_) {
      var H, V, q;
      const G = _[0] instanceof Vector ? _.flatMap((Z) => Z.data) : _;
      if (G.length === 0 || G.some((Z) => !(Z instanceof Data)))
        throw new TypeError("Vector constructor expects an Array of Data instances.");
      const J = (H = G[0]) === null || H === void 0 ? void 0 : H.type;
      switch (G.length) {
        case 0:
          this._offsets = [0];
          break;
        case 1: {
          const { get: Z, set: te, indexOf: ne } = visitorsByTypeId[J.typeId], ee = G[0];
          this.isValid = (ae) => isChunkedValid(ee, ae), this.get = (ae) => Z(ee, ae), this.set = (ae, oe) => te(ee, ae, oe), this.indexOf = (ae) => ne(ee, ae), this._offsets = [0, ee.length];
          break;
        }
        default:
          Object.setPrototypeOf(this, vectorPrototypesByTypeId[J.typeId]), this._offsets = computeChunkOffsets(G);
          break;
      }
      this.data = G, this.type = J, this.stride = strideForType(J), this.numChildren = (q = (V = J.children) === null || V === void 0 ? void 0 : V.length) !== null && q !== void 0 ? q : 0, this.length = this._offsets.at(-1);
    }
    /**
     * The aggregate size (in bytes) of this Vector's buffers and/or child Vectors.
     */
    get byteLength() {
      return this.data.reduce((_, H) => _ + H.byteLength, 0);
    }
    /**
     * Whether this Vector's elements can contain null values.
     */
    get nullable() {
      return computeChunkNullable(this.data);
    }
    /**
     * The number of null elements in this Vector.
     */
    get nullCount() {
      return computeChunkNullCounts(this.data);
    }
    /**
     * The Array or TypedArray constructor used for the JS representation
     *  of the element's values in {@link Vector.prototype.toArray `toArray()`}.
     */
    get ArrayType() {
      return this.type.ArrayType;
    }
    /**
     * The name that should be printed when the Vector is logged in a message.
     */
    get [Symbol.toStringTag]() {
      return `${this.VectorName}<${this.type[Symbol.toStringTag]}>`;
    }
    /**
     * The name of this Vector.
     */
    get VectorName() {
      return `${Type[this.type.typeId]}Vector`;
    }
    /**
     * Check whether an element is null.
     * @param index The index at which to read the validity bitmap.
     */
    // @ts-ignore
    isValid(_) {
      return !1;
    }
    /**
     * Get an element value by position.
     * @param index The index of the element to read.
     */
    // @ts-ignore
    get(_) {
      return null;
    }
    /**
     * Get an element value by position.
     * @param index The index of the element to read. A negative index will count back from the last element.
     */
    at(_) {
      return this.get(wrapIndex(_, this.length));
    }
    /**
     * Set an element value by position.
     * @param index The index of the element to write.
     * @param value The value to set.
     */
    // @ts-ignore
    set(_, H) {
    }
    /**
     * Retrieve the index of the first occurrence of a value in an Vector.
     * @param element The value to locate in the Vector.
     * @param offset The index at which to begin the search. If offset is omitted, the search starts at index 0.
     */
    // @ts-ignore
    indexOf(_, H) {
      return -1;
    }
    includes(_, H) {
      return this.indexOf(_, H) > -1;
    }
    /**
     * Iterator for the Vector's elements.
     */
    [Symbol.iterator]() {
      return instance$2.visit(this);
    }
    /**
     * Combines two or more Vectors of the same type.
     * @param others Additional Vectors to add to the end of this Vector.
     */
    concat(..._) {
      return new Vector(this.data.concat(_.flatMap((H) => H.data).flat(Number.POSITIVE_INFINITY)));
    }
    /**
     * Return a zero-copy sub-section of this Vector.
     * @param start The beginning of the specified portion of the Vector.
     * @param end The end of the specified portion of the Vector. This is exclusive of the element at the index 'end'.
     */
    slice(_, H) {
      return new Vector(clampRange(this, _, H, ({ data: V, _offsets: q }, G, J) => sliceChunks(V, q, G, J)));
    }
    toJSON() {
      return [...this];
    }
    /**
     * Return a JavaScript Array or TypedArray of the Vector's elements.
     *
     * @note If this Vector contains a single Data chunk and the Vector's type is a
     *  primitive numeric type corresponding to one of the JavaScript TypedArrays, this
     *  method returns a zero-copy slice of the underlying TypedArray values. If there's
     *  more than one chunk, the resulting TypedArray will be a copy of the data from each
     *  chunk's underlying TypedArray values.
     *
     * @returns An Array or TypedArray of the Vector's elements, based on the Vector's DataType.
     */
    toArray() {
      const { type: _, data: H, length: V, stride: q, ArrayType: G } = this;
      switch (_.typeId) {
        case Type.Int:
        case Type.Float:
        case Type.Decimal:
        case Type.Time:
        case Type.Timestamp:
          switch (H.length) {
            case 0:
              return new G();
            case 1:
              return H[0].values.subarray(0, V * q);
            default:
              return H.reduce((J, { values: Z, length: te }) => (J.array.set(Z.subarray(0, te * q), J.offset), J.offset += te * q, J), { array: new G(V * q), offset: 0 }).array;
          }
      }
      return [...this];
    }
    /**
     * Returns a string representation of the Vector.
     *
     * @returns A string representation of the Vector.
     */
    toString() {
      return `[${[...this].join(",")}]`;
    }
    /**
     * Returns a child Vector by name, or null if this Vector has no child with the given name.
     * @param name The name of the child to retrieve.
     */
    getChild(_) {
      var H;
      return this.getChildAt((H = this.type.children) === null || H === void 0 ? void 0 : H.findIndex((V) => V.name === _));
    }
    /**
     * Returns a child Vector by index, or null if this Vector has no child at the supplied index.
     * @param index The index of the child to retrieve.
     */
    getChildAt(_) {
      return _ > -1 && _ < this.numChildren ? new Vector(this.data.map(({ children: H }) => H[_])) : null;
    }
    get isMemoized() {
      return DataType.isDictionary(this.type) ? this.data[0].dictionary.isMemoized : !1;
    }
    /**
     * Adds memoization to the Vector's {@link get} method. For dictionary
     * vectors, this method return a vector that memoizes only the dictionary
     * values.
     *
     * Memoization is very useful when decoding a value is expensive such as
     * Utf8. The memoization creates a cache of the size of the Vector and
     * therefore increases memory usage.
     *
     * @returns A new vector that memoizes calls to {@link get}.
     */
    memoize() {
      if (DataType.isDictionary(this.type)) {
        const _ = new MemoizedVector(this.data[0].dictionary), H = this.data.map((V) => {
          const q = V.clone();
          return q.dictionary = _, q;
        });
        return new Vector(H);
      }
      return new MemoizedVector(this);
    }
    /**
     * Returns a vector without memoization of the {@link get} method. If this
     * vector is not memoized, this method returns this vector.
     *
     * @returns A new vector without memoization.
     */
    unmemoize() {
      if (DataType.isDictionary(this.type) && this.isMemoized) {
        const _ = this.data[0].dictionary.unmemoize(), H = this.data.map((V) => {
          const q = V.clone();
          return q.dictionary = _, q;
        });
        return new Vector(H);
      }
      return this;
    }
  }
  _a$2 = Symbol.toStringTag;
  Vector[_a$2] = ((O) => {
    O.type = DataType.prototype, O.data = [], O.length = 0, O.stride = 1, O.numChildren = 0, O._offsets = new Uint32Array([0]), O[Symbol.isConcatSpreadable] = !0;
    const _ = Object.keys(Type).map((H) => Type[H]).filter((H) => typeof H == "number" && H !== Type.NONE);
    for (const H of _) {
      const V = instance$4.getVisitFnByTypeId(H), q = instance$5.getVisitFnByTypeId(H), G = instance$3.getVisitFnByTypeId(H);
      visitorsByTypeId[H] = { get: V, set: q, indexOf: G }, vectorPrototypesByTypeId[H] = Object.create(O, {
        isValid: { value: wrapChunkedCall1(isChunkedValid) },
        get: { value: wrapChunkedCall1(instance$4.getVisitFnByTypeId(H)) },
        set: { value: wrapChunkedCall2(instance$5.getVisitFnByTypeId(H)) },
        indexOf: { value: wrapChunkedIndexOf(instance$3.getVisitFnByTypeId(H)) }
      });
    }
    return "Vector";
  })(Vector.prototype);
  class MemoizedVector extends Vector {
    constructor(_) {
      super(_.data);
      const H = this.get, V = this.set, q = this.slice, G = new Array(this.length);
      Object.defineProperty(this, "get", {
        value(J) {
          const Z = G[J];
          if (Z !== void 0)
            return Z;
          const te = H.call(this, J);
          return G[J] = te, te;
        }
      }), Object.defineProperty(this, "set", {
        value(J, Z) {
          V.call(this, J, Z), G[J] = Z;
        }
      }), Object.defineProperty(this, "slice", {
        value: (J, Z) => new MemoizedVector(q.call(this, J, Z))
      }), Object.defineProperty(this, "isMemoized", { value: !0 }), Object.defineProperty(this, "unmemoize", {
        value: () => new Vector(this.data)
      }), Object.defineProperty(this, "memoize", {
        value: () => this
      });
    }
  }
  class Block {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    /**
     * Index to the start of the RecordBlock (note this is past the Message header)
     */
    offset() {
      return this.bb.readInt64(this.bb_pos);
    }
    /**
     * Length of the metadata
     */
    metaDataLength() {
      return this.bb.readInt32(this.bb_pos + 8);
    }
    /**
     * Length of the data (this is aligned so there can be a gap between this and
     * the metadata).
     */
    bodyLength() {
      return this.bb.readInt64(this.bb_pos + 16);
    }
    static sizeOf() {
      return 24;
    }
    static createBlock(_, H, V, q) {
      return _.prep(8, 24), _.writeInt64(BigInt(q ?? 0)), _.pad(4), _.writeInt32(V), _.writeInt64(BigInt(H ?? 0)), _.offset();
    }
  }
  class Footer {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsFooter(_, H) {
      return (H || new Footer()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsFooter(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Footer()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    version() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : MetadataVersion.V1;
    }
    schema(_) {
      const H = this.bb.__offset(this.bb_pos, 6);
      return H ? (_ || new Schema$1()).__init(this.bb.__indirect(this.bb_pos + H), this.bb) : null;
    }
    dictionaries(_, H) {
      const V = this.bb.__offset(this.bb_pos, 8);
      return V ? (H || new Block()).__init(this.bb.__vector(this.bb_pos + V) + _ * 24, this.bb) : null;
    }
    dictionariesLength() {
      const _ = this.bb.__offset(this.bb_pos, 8);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    recordBatches(_, H) {
      const V = this.bb.__offset(this.bb_pos, 10);
      return V ? (H || new Block()).__init(this.bb.__vector(this.bb_pos + V) + _ * 24, this.bb) : null;
    }
    recordBatchesLength() {
      const _ = this.bb.__offset(this.bb_pos, 10);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    /**
     * User-defined metadata
     */
    customMetadata(_, H) {
      const V = this.bb.__offset(this.bb_pos, 12);
      return V ? (H || new KeyValue()).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos + V) + _ * 4), this.bb) : null;
    }
    customMetadataLength() {
      const _ = this.bb.__offset(this.bb_pos, 12);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    static startFooter(_) {
      _.startObject(5);
    }
    static addVersion(_, H) {
      _.addFieldInt16(0, H, MetadataVersion.V1);
    }
    static addSchema(_, H) {
      _.addFieldOffset(1, H, 0);
    }
    static addDictionaries(_, H) {
      _.addFieldOffset(2, H, 0);
    }
    static startDictionariesVector(_, H) {
      _.startVector(24, H, 8);
    }
    static addRecordBatches(_, H) {
      _.addFieldOffset(3, H, 0);
    }
    static startRecordBatchesVector(_, H) {
      _.startVector(24, H, 8);
    }
    static addCustomMetadata(_, H) {
      _.addFieldOffset(4, H, 0);
    }
    static createCustomMetadataVector(_, H) {
      _.startVector(4, H.length, 4);
      for (let V = H.length - 1; V >= 0; V--)
        _.addOffset(H[V]);
      return _.endVector();
    }
    static startCustomMetadataVector(_, H) {
      _.startVector(4, H, 4);
    }
    static endFooter(_) {
      return _.endObject();
    }
    static finishFooterBuffer(_, H) {
      _.finish(H);
    }
    static finishSizePrefixedFooterBuffer(_, H) {
      _.finish(H, void 0, !0);
    }
  }
  class Schema {
    constructor(_ = [], H, V, q = MetadataVersion.V5) {
      this.fields = _ || [], this.metadata = H || /* @__PURE__ */ new Map(), V || (V = generateDictionaryMap(this.fields)), this.dictionaries = V, this.metadataVersion = q;
    }
    get [Symbol.toStringTag]() {
      return "Schema";
    }
    get names() {
      return this.fields.map((_) => _.name);
    }
    toString() {
      return `Schema<{ ${this.fields.map((_, H) => `${H}: ${_}`).join(", ")} }>`;
    }
    /**
     * Construct a new Schema containing only specified fields.
     *
     * @param fieldNames Names of fields to keep.
     * @returns A new Schema of fields matching the specified names.
     */
    select(_) {
      const H = new Set(_), V = this.fields.filter((q) => H.has(q.name));
      return new Schema(V, this.metadata);
    }
    /**
     * Construct a new Schema containing only fields at the specified indices.
     *
     * @param fieldIndices Indices of fields to keep.
     * @returns A new Schema of fields at the specified indices.
     */
    selectAt(_) {
      const H = _.map((V) => this.fields[V]).filter(Boolean);
      return new Schema(H, this.metadata);
    }
    assign(..._) {
      const H = _[0] instanceof Schema ? _[0] : Array.isArray(_[0]) ? new Schema(_[0]) : new Schema(_), V = [...this.fields], q = mergeMaps(mergeMaps(/* @__PURE__ */ new Map(), this.metadata), H.metadata), G = H.fields.filter((Z) => {
        const te = V.findIndex((ne) => ne.name === Z.name);
        return ~te ? (V[te] = Z.clone({
          metadata: mergeMaps(mergeMaps(/* @__PURE__ */ new Map(), V[te].metadata), Z.metadata)
        })) && !1 : !0;
      }), J = generateDictionaryMap(G, /* @__PURE__ */ new Map());
      return new Schema([...V, ...G], q, new Map([...this.dictionaries, ...J]));
    }
  }
  Schema.prototype.fields = null;
  Schema.prototype.metadata = null;
  Schema.prototype.dictionaries = null;
  class Field {
    /** @nocollapse */
    static new(..._) {
      let [H, V, q, G] = _;
      return _[0] && typeof _[0] == "object" && ({ name: H } = _[0], V === void 0 && (V = _[0].type), q === void 0 && (q = _[0].nullable), G === void 0 && (G = _[0].metadata)), new Field(`${H}`, V, q, G);
    }
    constructor(_, H, V = !1, q) {
      this.name = _, this.type = H, this.nullable = V, this.metadata = q || /* @__PURE__ */ new Map();
    }
    get typeId() {
      return this.type.typeId;
    }
    get [Symbol.toStringTag]() {
      return "Field";
    }
    toString() {
      return `${this.name}: ${this.type}`;
    }
    clone(..._) {
      let [H, V, q, G] = _;
      return !_[0] || typeof _[0] != "object" ? [H = this.name, V = this.type, q = this.nullable, G = this.metadata] = _ : { name: H = this.name, type: V = this.type, nullable: q = this.nullable, metadata: G = this.metadata } = _[0], Field.new(H, V, q, G);
    }
  }
  Field.prototype.type = null;
  Field.prototype.name = null;
  Field.prototype.nullable = null;
  Field.prototype.metadata = null;
  function mergeMaps(O, _) {
    return new Map([...O || /* @__PURE__ */ new Map(), ..._ || /* @__PURE__ */ new Map()]);
  }
  function generateDictionaryMap(O, _ = /* @__PURE__ */ new Map()) {
    for (let H = -1, V = O.length; ++H < V; ) {
      const G = O[H].type;
      if (DataType.isDictionary(G)) {
        if (!_.has(G.id))
          _.set(G.id, G.dictionary);
        else if (_.get(G.id) !== G.dictionary)
          throw new Error("Cannot create Schema containing two different dictionaries with the same Id");
      }
      G.children && G.children.length > 0 && generateDictionaryMap(G.children, _);
    }
    return _;
  }
  var Builder$1 = Builder$2, ByteBuffer$1 = ByteBuffer$2;
  class Footer_ {
    /** @nocollapse */
    static decode(_) {
      _ = new ByteBuffer$1(toUint8Array(_));
      const H = Footer.getRootAsFooter(_), V = Schema.decode(H.schema(), /* @__PURE__ */ new Map(), H.version());
      return new OffHeapFooter(V, H);
    }
    /** @nocollapse */
    static encode(_) {
      const H = new Builder$1(), V = Schema.encode(H, _.schema);
      Footer.startRecordBatchesVector(H, _.numRecordBatches);
      for (const J of [..._.recordBatches()].slice().reverse())
        FileBlock.encode(H, J);
      const q = H.endVector();
      Footer.startDictionariesVector(H, _.numDictionaries);
      for (const J of [..._.dictionaryBatches()].slice().reverse())
        FileBlock.encode(H, J);
      const G = H.endVector();
      return Footer.startFooter(H), Footer.addSchema(H, V), Footer.addVersion(H, MetadataVersion.V5), Footer.addRecordBatches(H, q), Footer.addDictionaries(H, G), Footer.finishFooterBuffer(H, Footer.endFooter(H)), H.asUint8Array();
    }
    get numRecordBatches() {
      return this._recordBatches.length;
    }
    get numDictionaries() {
      return this._dictionaryBatches.length;
    }
    constructor(_, H = MetadataVersion.V5, V, q) {
      this.schema = _, this.version = H, V && (this._recordBatches = V), q && (this._dictionaryBatches = q);
    }
    *recordBatches() {
      for (let _, H = -1, V = this.numRecordBatches; ++H < V; )
        (_ = this.getRecordBatch(H)) && (yield _);
    }
    *dictionaryBatches() {
      for (let _, H = -1, V = this.numDictionaries; ++H < V; )
        (_ = this.getDictionaryBatch(H)) && (yield _);
    }
    getRecordBatch(_) {
      return _ >= 0 && _ < this.numRecordBatches && this._recordBatches[_] || null;
    }
    getDictionaryBatch(_) {
      return _ >= 0 && _ < this.numDictionaries && this._dictionaryBatches[_] || null;
    }
  }
  class OffHeapFooter extends Footer_ {
    get numRecordBatches() {
      return this._footer.recordBatchesLength();
    }
    get numDictionaries() {
      return this._footer.dictionariesLength();
    }
    constructor(_, H) {
      super(_, H.version()), this._footer = H;
    }
    getRecordBatch(_) {
      if (_ >= 0 && _ < this.numRecordBatches) {
        const H = this._footer.recordBatches(_);
        if (H)
          return FileBlock.decode(H);
      }
      return null;
    }
    getDictionaryBatch(_) {
      if (_ >= 0 && _ < this.numDictionaries) {
        const H = this._footer.dictionaries(_);
        if (H)
          return FileBlock.decode(H);
      }
      return null;
    }
  }
  class FileBlock {
    /** @nocollapse */
    static decode(_) {
      return new FileBlock(_.metaDataLength(), _.bodyLength(), _.offset());
    }
    /** @nocollapse */
    static encode(_, H) {
      const { metaDataLength: V } = H, q = BigInt(H.offset), G = BigInt(H.bodyLength);
      return Block.createBlock(_, q, V, G);
    }
    constructor(_, H, V) {
      this.metaDataLength = _, this.offset = bigIntToNumber(V), this.bodyLength = bigIntToNumber(H);
    }
  }
  const ITERATOR_DONE = Object.freeze({ done: !0, value: void 0 });
  class ArrowJSON {
    constructor(_) {
      this._json = _;
    }
    get schema() {
      return this._json.schema;
    }
    get batches() {
      return this._json.batches || [];
    }
    get dictionaries() {
      return this._json.dictionaries || [];
    }
  }
  class ReadableInterop {
    tee() {
      return this._getDOMStream().tee();
    }
    pipe(_, H) {
      return this._getNodeStream().pipe(_, H);
    }
    pipeTo(_, H) {
      return this._getDOMStream().pipeTo(_, H);
    }
    pipeThrough(_, H) {
      return this._getDOMStream().pipeThrough(_, H);
    }
    _getDOMStream() {
      return this._DOMStream || (this._DOMStream = this.toDOMStream());
    }
    _getNodeStream() {
      return this._nodeStream || (this._nodeStream = this.toNodeStream());
    }
  }
  class AsyncQueue extends ReadableInterop {
    constructor() {
      super(), this._values = [], this.resolvers = [], this._closedPromise = new Promise((_) => this._closedPromiseResolve = _);
    }
    get closed() {
      return this._closedPromise;
    }
    cancel(_) {
      return __awaiter(this, void 0, void 0, function* () {
        yield this.return(_);
      });
    }
    write(_) {
      this._ensureOpen() && (this.resolvers.length <= 0 ? this._values.push(_) : this.resolvers.shift().resolve({ done: !1, value: _ }));
    }
    abort(_) {
      this._closedPromiseResolve && (this.resolvers.length <= 0 ? this._error = { error: _ } : this.resolvers.shift().reject({ done: !0, value: _ }));
    }
    close() {
      if (this._closedPromiseResolve) {
        const { resolvers: _ } = this;
        for (; _.length > 0; )
          _.shift().resolve(ITERATOR_DONE);
        this._closedPromiseResolve(), this._closedPromiseResolve = void 0;
      }
    }
    [Symbol.asyncIterator]() {
      return this;
    }
    toDOMStream(_) {
      return streamAdapters.toDOMStream(this._closedPromiseResolve || this._error ? this : this._values, _);
    }
    toNodeStream(_) {
      return streamAdapters.toNodeStream(this._closedPromiseResolve || this._error ? this : this._values, _);
    }
    throw(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return yield this.abort(_), ITERATOR_DONE;
      });
    }
    return(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return yield this.close(), ITERATOR_DONE;
      });
    }
    read(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return (yield this.next(_, "read")).value;
      });
    }
    peek(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return (yield this.next(_, "peek")).value;
      });
    }
    next(..._) {
      return this._values.length > 0 ? Promise.resolve({ done: !1, value: this._values.shift() }) : this._error ? Promise.reject({ done: !0, value: this._error.error }) : this._closedPromiseResolve ? new Promise((H, V) => {
        this.resolvers.push({ resolve: H, reject: V });
      }) : Promise.resolve(ITERATOR_DONE);
    }
    _ensureOpen() {
      if (this._closedPromiseResolve)
        return !0;
      throw new Error("AsyncQueue is closed");
    }
  }
  class AsyncByteQueue extends AsyncQueue {
    write(_) {
      if ((_ = toUint8Array(_)).byteLength > 0)
        return super.write(_);
    }
    toString(_ = !1) {
      return _ ? decodeUtf8(this.toUint8Array(!0)) : this.toUint8Array(!1).then(decodeUtf8);
    }
    toUint8Array(_ = !1) {
      return _ ? joinUint8Arrays(this._values)[0] : __awaiter(this, void 0, void 0, function* () {
        var H, V, q, G;
        const J = [];
        let Z = 0;
        try {
          for (var te = !0, ne = __asyncValues(this), ee; ee = yield ne.next(), H = ee.done, !H; te = !0) {
            G = ee.value, te = !1;
            const ae = G;
            J.push(ae), Z += ae.byteLength;
          }
        } catch (ae) {
          V = { error: ae };
        } finally {
          try {
            !te && !H && (q = ne.return) && (yield q.call(ne));
          } finally {
            if (V) throw V.error;
          }
        }
        return joinUint8Arrays(J, Z)[0];
      });
    }
  }
  class ByteStream {
    constructor(_) {
      _ && (this.source = new ByteStreamSource(streamAdapters.fromIterable(_)));
    }
    [Symbol.iterator]() {
      return this;
    }
    next(_) {
      return this.source.next(_);
    }
    throw(_) {
      return this.source.throw(_);
    }
    return(_) {
      return this.source.return(_);
    }
    peek(_) {
      return this.source.peek(_);
    }
    read(_) {
      return this.source.read(_);
    }
  }
  class AsyncByteStream {
    constructor(_) {
      _ instanceof AsyncByteStream ? this.source = _.source : _ instanceof AsyncByteQueue ? this.source = new AsyncByteStreamSource(streamAdapters.fromAsyncIterable(_)) : isReadableNodeStream(_) ? this.source = new AsyncByteStreamSource(streamAdapters.fromNodeStream(_)) : isReadableDOMStream(_) ? this.source = new AsyncByteStreamSource(streamAdapters.fromDOMStream(_)) : isFetchResponse(_) ? this.source = new AsyncByteStreamSource(streamAdapters.fromDOMStream(_.body)) : isIterable(_) ? this.source = new AsyncByteStreamSource(streamAdapters.fromIterable(_)) : isPromise(_) ? this.source = new AsyncByteStreamSource(streamAdapters.fromAsyncIterable(_)) : isAsyncIterable(_) && (this.source = new AsyncByteStreamSource(streamAdapters.fromAsyncIterable(_)));
    }
    [Symbol.asyncIterator]() {
      return this;
    }
    next(_) {
      return this.source.next(_);
    }
    throw(_) {
      return this.source.throw(_);
    }
    return(_) {
      return this.source.return(_);
    }
    get closed() {
      return this.source.closed;
    }
    cancel(_) {
      return this.source.cancel(_);
    }
    peek(_) {
      return this.source.peek(_);
    }
    read(_) {
      return this.source.read(_);
    }
  }
  class ByteStreamSource {
    constructor(_) {
      this.source = _;
    }
    cancel(_) {
      this.return(_);
    }
    peek(_) {
      return this.next(_, "peek").value;
    }
    read(_) {
      return this.next(_, "read").value;
    }
    next(_, H = "read") {
      return this.source.next({ cmd: H, size: _ });
    }
    throw(_) {
      return Object.create(this.source.throw && this.source.throw(_) || ITERATOR_DONE);
    }
    return(_) {
      return Object.create(this.source.return && this.source.return(_) || ITERATOR_DONE);
    }
  }
  class AsyncByteStreamSource {
    constructor(_) {
      this.source = _, this._closedPromise = new Promise((H) => this._closedPromiseResolve = H);
    }
    cancel(_) {
      return __awaiter(this, void 0, void 0, function* () {
        yield this.return(_);
      });
    }
    get closed() {
      return this._closedPromise;
    }
    read(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return (yield this.next(_, "read")).value;
      });
    }
    peek(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return (yield this.next(_, "peek")).value;
      });
    }
    next(_) {
      return __awaiter(this, arguments, void 0, function* (H, V = "read") {
        return yield this.source.next({ cmd: V, size: H });
      });
    }
    throw(_) {
      return __awaiter(this, void 0, void 0, function* () {
        const H = this.source.throw && (yield this.source.throw(_)) || ITERATOR_DONE;
        return this._closedPromiseResolve && this._closedPromiseResolve(), this._closedPromiseResolve = void 0, Object.create(H);
      });
    }
    return(_) {
      return __awaiter(this, void 0, void 0, function* () {
        const H = this.source.return && (yield this.source.return(_)) || ITERATOR_DONE;
        return this._closedPromiseResolve && this._closedPromiseResolve(), this._closedPromiseResolve = void 0, Object.create(H);
      });
    }
  }
  class RandomAccessFile extends ByteStream {
    constructor(_, H) {
      super(), this.position = 0, this.buffer = toUint8Array(_), this.size = H === void 0 ? this.buffer.byteLength : H;
    }
    readInt32(_) {
      const { buffer: H, byteOffset: V } = this.readAt(_, 4);
      return new DataView(H, V).getInt32(0, !0);
    }
    seek(_) {
      return this.position = Math.min(_, this.size), _ < this.size;
    }
    read(_) {
      const { buffer: H, size: V, position: q } = this;
      return H && q < V ? (typeof _ != "number" && (_ = Number.POSITIVE_INFINITY), this.position = Math.min(V, q + Math.min(V - q, _)), H.subarray(q, this.position)) : null;
    }
    readAt(_, H) {
      const V = this.buffer, q = Math.min(this.size, _ + H);
      return V ? V.subarray(_, q) : new Uint8Array(H);
    }
    close() {
      this.buffer && (this.buffer = null);
    }
    throw(_) {
      return this.close(), { done: !0, value: _ };
    }
    return(_) {
      return this.close(), { done: !0, value: _ };
    }
  }
  class AsyncRandomAccessFile extends AsyncByteStream {
    constructor(_, H) {
      super(), this.position = 0, this._handle = _, typeof H == "number" ? this.size = H : this._pending = __awaiter(this, void 0, void 0, function* () {
        this.size = (yield _.stat()).size, delete this._pending;
      });
    }
    readInt32(_) {
      return __awaiter(this, void 0, void 0, function* () {
        const { buffer: H, byteOffset: V } = yield this.readAt(_, 4);
        return new DataView(H, V).getInt32(0, !0);
      });
    }
    seek(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return this._pending && (yield this._pending), this.position = Math.min(_, this.size), _ < this.size;
      });
    }
    read(_) {
      return __awaiter(this, void 0, void 0, function* () {
        this._pending && (yield this._pending);
        const { _handle: H, size: V, position: q } = this;
        if (H && q < V) {
          typeof _ != "number" && (_ = Number.POSITIVE_INFINITY);
          let G = q, J = 0, Z = 0;
          const te = Math.min(V, G + Math.min(V - G, _)), ne = new Uint8Array(Math.max(0, (this.position = te) - G));
          for (; (G += Z) < te && (J += Z) < ne.byteLength; )
            ({ bytesRead: Z } = yield H.read(ne, J, ne.byteLength - J, G));
          return ne;
        }
        return null;
      });
    }
    readAt(_, H) {
      return __awaiter(this, void 0, void 0, function* () {
        this._pending && (yield this._pending);
        const { _handle: V, size: q } = this;
        if (V && _ + H < q) {
          const G = Math.min(q, _ + H), J = new Uint8Array(G - _);
          return (yield V.read(J, 0, H, _)).buffer;
        }
        return new Uint8Array(H);
      });
    }
    close() {
      return __awaiter(this, void 0, void 0, function* () {
        const _ = this._handle;
        this._handle = null, _ && (yield _.close());
      });
    }
    throw(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return yield this.close(), { done: !0, value: _ };
      });
    }
    return(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return yield this.close(), { done: !0, value: _ };
      });
    }
  }
  const carryBit16 = 65536;
  function intAsHex(O) {
    return O < 0 && (O = 4294967295 + O + 1), `0x${O.toString(16)}`;
  }
  const kInt32DecimalDigits = 8, kPowersOfTen = [
    1,
    10,
    100,
    1e3,
    1e4,
    1e5,
    1e6,
    1e7,
    1e8
  ];
  class BaseInt64 {
    constructor(_) {
      this.buffer = _;
    }
    high() {
      return this.buffer[1];
    }
    low() {
      return this.buffer[0];
    }
    _times(_) {
      const H = new Uint32Array([
        this.buffer[1] >>> 16,
        this.buffer[1] & 65535,
        this.buffer[0] >>> 16,
        this.buffer[0] & 65535
      ]), V = new Uint32Array([
        _.buffer[1] >>> 16,
        _.buffer[1] & 65535,
        _.buffer[0] >>> 16,
        _.buffer[0] & 65535
      ]);
      let q = H[3] * V[3];
      this.buffer[0] = q & 65535;
      let G = q >>> 16;
      return q = H[2] * V[3], G += q, q = H[3] * V[2] >>> 0, G += q, this.buffer[0] += G << 16, this.buffer[1] = G >>> 0 < q ? carryBit16 : 0, this.buffer[1] += G >>> 16, this.buffer[1] += H[1] * V[3] + H[2] * V[2] + H[3] * V[1], this.buffer[1] += H[0] * V[3] + H[1] * V[2] + H[2] * V[1] + H[3] * V[0] << 16, this;
    }
    _plus(_) {
      const H = this.buffer[0] + _.buffer[0] >>> 0;
      this.buffer[1] += _.buffer[1], H < this.buffer[0] >>> 0 && ++this.buffer[1], this.buffer[0] = H;
    }
    lessThan(_) {
      return this.buffer[1] < _.buffer[1] || this.buffer[1] === _.buffer[1] && this.buffer[0] < _.buffer[0];
    }
    equals(_) {
      return this.buffer[1] === _.buffer[1] && this.buffer[0] == _.buffer[0];
    }
    greaterThan(_) {
      return _.lessThan(this);
    }
    hex() {
      return `${intAsHex(this.buffer[1])} ${intAsHex(this.buffer[0])}`;
    }
  }
  class Uint64 extends BaseInt64 {
    times(_) {
      return this._times(_), this;
    }
    plus(_) {
      return this._plus(_), this;
    }
    /** @nocollapse */
    static from(_, H = new Uint32Array(2)) {
      return Uint64.fromString(typeof _ == "string" ? _ : _.toString(), H);
    }
    /** @nocollapse */
    static fromNumber(_, H = new Uint32Array(2)) {
      return Uint64.fromString(_.toString(), H);
    }
    /** @nocollapse */
    static fromString(_, H = new Uint32Array(2)) {
      const V = _.length, q = new Uint64(H);
      for (let G = 0; G < V; ) {
        const J = kInt32DecimalDigits < V - G ? kInt32DecimalDigits : V - G, Z = new Uint64(new Uint32Array([Number.parseInt(_.slice(G, G + J), 10), 0])), te = new Uint64(new Uint32Array([kPowersOfTen[J], 0]));
        q.times(te), q.plus(Z), G += J;
      }
      return q;
    }
    /** @nocollapse */
    static convertArray(_) {
      const H = new Uint32Array(_.length * 2);
      for (let V = -1, q = _.length; ++V < q; )
        Uint64.from(_[V], new Uint32Array(H.buffer, H.byteOffset + 2 * V * 4, 2));
      return H;
    }
    /** @nocollapse */
    static multiply(_, H) {
      return new Uint64(new Uint32Array(_.buffer)).times(H);
    }
    /** @nocollapse */
    static add(_, H) {
      return new Uint64(new Uint32Array(_.buffer)).plus(H);
    }
  }
  class Int64 extends BaseInt64 {
    negate() {
      return this.buffer[0] = ~this.buffer[0] + 1, this.buffer[1] = ~this.buffer[1], this.buffer[0] == 0 && ++this.buffer[1], this;
    }
    times(_) {
      return this._times(_), this;
    }
    plus(_) {
      return this._plus(_), this;
    }
    lessThan(_) {
      const H = this.buffer[1] << 0, V = _.buffer[1] << 0;
      return H < V || H === V && this.buffer[0] < _.buffer[0];
    }
    /** @nocollapse */
    static from(_, H = new Uint32Array(2)) {
      return Int64.fromString(typeof _ == "string" ? _ : _.toString(), H);
    }
    /** @nocollapse */
    static fromNumber(_, H = new Uint32Array(2)) {
      return Int64.fromString(_.toString(), H);
    }
    /** @nocollapse */
    static fromString(_, H = new Uint32Array(2)) {
      const V = _.startsWith("-"), q = _.length, G = new Int64(H);
      for (let J = V ? 1 : 0; J < q; ) {
        const Z = kInt32DecimalDigits < q - J ? kInt32DecimalDigits : q - J, te = new Int64(new Uint32Array([Number.parseInt(_.slice(J, J + Z), 10), 0])), ne = new Int64(new Uint32Array([kPowersOfTen[Z], 0]));
        G.times(ne), G.plus(te), J += Z;
      }
      return V ? G.negate() : G;
    }
    /** @nocollapse */
    static convertArray(_) {
      const H = new Uint32Array(_.length * 2);
      for (let V = -1, q = _.length; ++V < q; )
        Int64.from(_[V], new Uint32Array(H.buffer, H.byteOffset + 2 * V * 4, 2));
      return H;
    }
    /** @nocollapse */
    static multiply(_, H) {
      return new Int64(new Uint32Array(_.buffer)).times(H);
    }
    /** @nocollapse */
    static add(_, H) {
      return new Int64(new Uint32Array(_.buffer)).plus(H);
    }
  }
  class Int128 {
    constructor(_) {
      this.buffer = _;
    }
    high() {
      return new Int64(new Uint32Array(this.buffer.buffer, this.buffer.byteOffset + 8, 2));
    }
    low() {
      return new Int64(new Uint32Array(this.buffer.buffer, this.buffer.byteOffset, 2));
    }
    negate() {
      return this.buffer[0] = ~this.buffer[0] + 1, this.buffer[1] = ~this.buffer[1], this.buffer[2] = ~this.buffer[2], this.buffer[3] = ~this.buffer[3], this.buffer[0] == 0 && ++this.buffer[1], this.buffer[1] == 0 && ++this.buffer[2], this.buffer[2] == 0 && ++this.buffer[3], this;
    }
    times(_) {
      const H = new Uint64(new Uint32Array([this.buffer[3], 0])), V = new Uint64(new Uint32Array([this.buffer[2], 0])), q = new Uint64(new Uint32Array([this.buffer[1], 0])), G = new Uint64(new Uint32Array([this.buffer[0], 0])), J = new Uint64(new Uint32Array([_.buffer[3], 0])), Z = new Uint64(new Uint32Array([_.buffer[2], 0])), te = new Uint64(new Uint32Array([_.buffer[1], 0])), ne = new Uint64(new Uint32Array([_.buffer[0], 0]));
      let ee = Uint64.multiply(G, ne);
      this.buffer[0] = ee.low();
      const ae = new Uint64(new Uint32Array([ee.high(), 0]));
      return ee = Uint64.multiply(q, ne), ae.plus(ee), ee = Uint64.multiply(G, te), ae.plus(ee), this.buffer[1] = ae.low(), this.buffer[3] = ae.lessThan(ee) ? 1 : 0, this.buffer[2] = ae.high(), new Uint64(new Uint32Array(this.buffer.buffer, this.buffer.byteOffset + 8, 2)).plus(Uint64.multiply(V, ne)).plus(Uint64.multiply(q, te)).plus(Uint64.multiply(G, Z)), this.buffer[3] += Uint64.multiply(H, ne).plus(Uint64.multiply(V, te)).plus(Uint64.multiply(q, Z)).plus(Uint64.multiply(G, J)).low(), this;
    }
    plus(_) {
      const H = new Uint32Array(4);
      return H[3] = this.buffer[3] + _.buffer[3] >>> 0, H[2] = this.buffer[2] + _.buffer[2] >>> 0, H[1] = this.buffer[1] + _.buffer[1] >>> 0, H[0] = this.buffer[0] + _.buffer[0] >>> 0, H[0] < this.buffer[0] >>> 0 && ++H[1], H[1] < this.buffer[1] >>> 0 && ++H[2], H[2] < this.buffer[2] >>> 0 && ++H[3], this.buffer[3] = H[3], this.buffer[2] = H[2], this.buffer[1] = H[1], this.buffer[0] = H[0], this;
    }
    hex() {
      return `${intAsHex(this.buffer[3])} ${intAsHex(this.buffer[2])} ${intAsHex(this.buffer[1])} ${intAsHex(this.buffer[0])}`;
    }
    /** @nocollapse */
    static multiply(_, H) {
      return new Int128(new Uint32Array(_.buffer)).times(H);
    }
    /** @nocollapse */
    static add(_, H) {
      return new Int128(new Uint32Array(_.buffer)).plus(H);
    }
    /** @nocollapse */
    static from(_, H = new Uint32Array(4)) {
      return Int128.fromString(typeof _ == "string" ? _ : _.toString(), H);
    }
    /** @nocollapse */
    static fromNumber(_, H = new Uint32Array(4)) {
      return Int128.fromString(_.toString(), H);
    }
    /** @nocollapse */
    static fromString(_, H = new Uint32Array(4)) {
      const V = _.startsWith("-"), q = _.length, G = new Int128(H);
      for (let J = V ? 1 : 0; J < q; ) {
        const Z = kInt32DecimalDigits < q - J ? kInt32DecimalDigits : q - J, te = new Int128(new Uint32Array([Number.parseInt(_.slice(J, J + Z), 10), 0, 0, 0])), ne = new Int128(new Uint32Array([kPowersOfTen[Z], 0, 0, 0]));
        G.times(ne), G.plus(te), J += Z;
      }
      return V ? G.negate() : G;
    }
    /** @nocollapse */
    static convertArray(_) {
      const H = new Uint32Array(_.length * 4);
      for (let V = -1, q = _.length; ++V < q; )
        Int128.from(_[V], new Uint32Array(H.buffer, H.byteOffset + 4 * 4 * V, 4));
      return H;
    }
  }
  class VectorLoader extends Visitor {
    constructor(_, H, V, q, G = MetadataVersion.V5) {
      super(), this.nodesIndex = -1, this.buffersIndex = -1, this.bytes = _, this.nodes = H, this.buffers = V, this.dictionaries = q, this.metadataVersion = G;
    }
    visit(_) {
      return super.visit(_ instanceof Field ? _.type : _);
    }
    visitNull(_, { length: H } = this.nextFieldNode()) {
      return makeData({ type: _, length: H });
    }
    visitBool(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitInt(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitFloat(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitUtf8(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), valueOffsets: this.readOffsets(_), data: this.readData(_) });
    }
    visitLargeUtf8(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), valueOffsets: this.readOffsets(_), data: this.readData(_) });
    }
    visitBinary(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), valueOffsets: this.readOffsets(_), data: this.readData(_) });
    }
    visitLargeBinary(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), valueOffsets: this.readOffsets(_), data: this.readData(_) });
    }
    visitFixedSizeBinary(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitDate(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitTimestamp(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitTime(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitDecimal(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitList(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), valueOffsets: this.readOffsets(_), child: this.visit(_.children[0]) });
    }
    visitStruct(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), children: this.visitMany(_.children) });
    }
    visitUnion(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return this.metadataVersion < MetadataVersion.V5 && this.readNullBitmap(_, V), _.mode === UnionMode.Sparse ? this.visitSparseUnion(_, { length: H, nullCount: V }) : this.visitDenseUnion(_, { length: H, nullCount: V });
    }
    visitDenseUnion(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, typeIds: this.readTypeIds(_), valueOffsets: this.readOffsets(_), children: this.visitMany(_.children) });
    }
    visitSparseUnion(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, typeIds: this.readTypeIds(_), children: this.visitMany(_.children) });
    }
    visitDictionary(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_.indices), dictionary: this.readDictionary(_) });
    }
    visitInterval(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitDuration(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), data: this.readData(_) });
    }
    visitFixedSizeList(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), child: this.visit(_.children[0]) });
    }
    visitMap(_, { length: H, nullCount: V } = this.nextFieldNode()) {
      return makeData({ type: _, length: H, nullCount: V, nullBitmap: this.readNullBitmap(_, V), valueOffsets: this.readOffsets(_), child: this.visit(_.children[0]) });
    }
    nextFieldNode() {
      return this.nodes[++this.nodesIndex];
    }
    nextBufferRange() {
      return this.buffers[++this.buffersIndex];
    }
    readNullBitmap(_, H, V = this.nextBufferRange()) {
      return H > 0 && this.readData(_, V) || new Uint8Array(0);
    }
    readOffsets(_, H) {
      return this.readData(_, H);
    }
    readTypeIds(_, H) {
      return this.readData(_, H);
    }
    readData(_, { length: H, offset: V } = this.nextBufferRange()) {
      return this.bytes.subarray(V, V + H);
    }
    readDictionary(_) {
      return this.dictionaries.get(_.id);
    }
  }
  class JSONVectorLoader extends VectorLoader {
    constructor(_, H, V, q, G) {
      super(new Uint8Array(0), H, V, q, G), this.sources = _;
    }
    readNullBitmap(_, H, { offset: V } = this.nextBufferRange()) {
      return H <= 0 ? new Uint8Array(0) : packBools(this.sources[V]);
    }
    readOffsets(_, { offset: H } = this.nextBufferRange()) {
      return toArrayBufferView(Uint8Array, toArrayBufferView(_.OffsetArrayType, this.sources[H]));
    }
    readTypeIds(_, { offset: H } = this.nextBufferRange()) {
      return toArrayBufferView(Uint8Array, toArrayBufferView(_.ArrayType, this.sources[H]));
    }
    readData(_, { offset: H } = this.nextBufferRange()) {
      const { sources: V } = this;
      return DataType.isTimestamp(_) || (DataType.isInt(_) || DataType.isTime(_)) && _.bitWidth === 64 || DataType.isDuration(_) || DataType.isDate(_) && _.unit === DateUnit.MILLISECOND ? toArrayBufferView(Uint8Array, Int64.convertArray(V[H])) : DataType.isDecimal(_) ? toArrayBufferView(Uint8Array, Int128.convertArray(V[H])) : DataType.isBinary(_) || DataType.isLargeBinary(_) || DataType.isFixedSizeBinary(_) ? binaryDataFromJSON(V[H]) : DataType.isBool(_) ? packBools(V[H]) : DataType.isUtf8(_) || DataType.isLargeUtf8(_) ? encodeUtf8(V[H].join("")) : toArrayBufferView(Uint8Array, toArrayBufferView(_.ArrayType, V[H].map((q) => +q)));
    }
  }
  function binaryDataFromJSON(O) {
    const _ = O.join(""), H = new Uint8Array(_.length / 2);
    for (let V = 0; V < _.length; V += 2)
      H[V >> 1] = Number.parseInt(_.slice(V, V + 2), 16);
    return H;
  }
  class TypeComparator extends Visitor {
    compareSchemas(_, H) {
      return _ === H || H instanceof _.constructor && this.compareManyFields(_.fields, H.fields);
    }
    compareManyFields(_, H) {
      return _ === H || Array.isArray(_) && Array.isArray(H) && _.length === H.length && _.every((V, q) => this.compareFields(V, H[q]));
    }
    compareFields(_, H) {
      return _ === H || H instanceof _.constructor && _.name === H.name && _.nullable === H.nullable && this.visit(_.type, H.type);
    }
  }
  function compareConstructor(O, _) {
    return _ instanceof O.constructor;
  }
  function compareAny(O, _) {
    return O === _ || compareConstructor(O, _);
  }
  function compareInt(O, _) {
    return O === _ || compareConstructor(O, _) && O.bitWidth === _.bitWidth && O.isSigned === _.isSigned;
  }
  function compareFloat(O, _) {
    return O === _ || compareConstructor(O, _) && O.precision === _.precision;
  }
  function compareFixedSizeBinary(O, _) {
    return O === _ || compareConstructor(O, _) && O.byteWidth === _.byteWidth;
  }
  function compareDate(O, _) {
    return O === _ || compareConstructor(O, _) && O.unit === _.unit;
  }
  function compareTimestamp(O, _) {
    return O === _ || compareConstructor(O, _) && O.unit === _.unit && O.timezone === _.timezone;
  }
  function compareTime(O, _) {
    return O === _ || compareConstructor(O, _) && O.unit === _.unit && O.bitWidth === _.bitWidth;
  }
  function compareList(O, _) {
    return O === _ || compareConstructor(O, _) && O.children.length === _.children.length && instance$1.compareManyFields(O.children, _.children);
  }
  function compareStruct(O, _) {
    return O === _ || compareConstructor(O, _) && O.children.length === _.children.length && instance$1.compareManyFields(O.children, _.children);
  }
  function compareUnion(O, _) {
    return O === _ || compareConstructor(O, _) && O.mode === _.mode && O.typeIds.every((H, V) => H === _.typeIds[V]) && instance$1.compareManyFields(O.children, _.children);
  }
  function compareDictionary(O, _) {
    return O === _ || compareConstructor(O, _) && O.id === _.id && O.isOrdered === _.isOrdered && instance$1.visit(O.indices, _.indices) && instance$1.visit(O.dictionary, _.dictionary);
  }
  function compareInterval(O, _) {
    return O === _ || compareConstructor(O, _) && O.unit === _.unit;
  }
  function compareDuration(O, _) {
    return O === _ || compareConstructor(O, _) && O.unit === _.unit;
  }
  function compareFixedSizeList(O, _) {
    return O === _ || compareConstructor(O, _) && O.listSize === _.listSize && O.children.length === _.children.length && instance$1.compareManyFields(O.children, _.children);
  }
  function compareMap(O, _) {
    return O === _ || compareConstructor(O, _) && O.keysSorted === _.keysSorted && O.children.length === _.children.length && instance$1.compareManyFields(O.children, _.children);
  }
  TypeComparator.prototype.visitNull = compareAny;
  TypeComparator.prototype.visitBool = compareAny;
  TypeComparator.prototype.visitInt = compareInt;
  TypeComparator.prototype.visitInt8 = compareInt;
  TypeComparator.prototype.visitInt16 = compareInt;
  TypeComparator.prototype.visitInt32 = compareInt;
  TypeComparator.prototype.visitInt64 = compareInt;
  TypeComparator.prototype.visitUint8 = compareInt;
  TypeComparator.prototype.visitUint16 = compareInt;
  TypeComparator.prototype.visitUint32 = compareInt;
  TypeComparator.prototype.visitUint64 = compareInt;
  TypeComparator.prototype.visitFloat = compareFloat;
  TypeComparator.prototype.visitFloat16 = compareFloat;
  TypeComparator.prototype.visitFloat32 = compareFloat;
  TypeComparator.prototype.visitFloat64 = compareFloat;
  TypeComparator.prototype.visitUtf8 = compareAny;
  TypeComparator.prototype.visitLargeUtf8 = compareAny;
  TypeComparator.prototype.visitBinary = compareAny;
  TypeComparator.prototype.visitLargeBinary = compareAny;
  TypeComparator.prototype.visitFixedSizeBinary = compareFixedSizeBinary;
  TypeComparator.prototype.visitDate = compareDate;
  TypeComparator.prototype.visitDateDay = compareDate;
  TypeComparator.prototype.visitDateMillisecond = compareDate;
  TypeComparator.prototype.visitTimestamp = compareTimestamp;
  TypeComparator.prototype.visitTimestampSecond = compareTimestamp;
  TypeComparator.prototype.visitTimestampMillisecond = compareTimestamp;
  TypeComparator.prototype.visitTimestampMicrosecond = compareTimestamp;
  TypeComparator.prototype.visitTimestampNanosecond = compareTimestamp;
  TypeComparator.prototype.visitTime = compareTime;
  TypeComparator.prototype.visitTimeSecond = compareTime;
  TypeComparator.prototype.visitTimeMillisecond = compareTime;
  TypeComparator.prototype.visitTimeMicrosecond = compareTime;
  TypeComparator.prototype.visitTimeNanosecond = compareTime;
  TypeComparator.prototype.visitDecimal = compareAny;
  TypeComparator.prototype.visitList = compareList;
  TypeComparator.prototype.visitStruct = compareStruct;
  TypeComparator.prototype.visitUnion = compareUnion;
  TypeComparator.prototype.visitDenseUnion = compareUnion;
  TypeComparator.prototype.visitSparseUnion = compareUnion;
  TypeComparator.prototype.visitDictionary = compareDictionary;
  TypeComparator.prototype.visitInterval = compareInterval;
  TypeComparator.prototype.visitIntervalDayTime = compareInterval;
  TypeComparator.prototype.visitIntervalYearMonth = compareInterval;
  TypeComparator.prototype.visitDuration = compareDuration;
  TypeComparator.prototype.visitDurationSecond = compareDuration;
  TypeComparator.prototype.visitDurationMillisecond = compareDuration;
  TypeComparator.prototype.visitDurationMicrosecond = compareDuration;
  TypeComparator.prototype.visitDurationNanosecond = compareDuration;
  TypeComparator.prototype.visitFixedSizeList = compareFixedSizeList;
  TypeComparator.prototype.visitMap = compareMap;
  const instance$1 = new TypeComparator();
  function compareSchemas(O, _) {
    return instance$1.compareSchemas(O, _);
  }
  function distributeVectorsIntoRecordBatches(O, _) {
    return uniformlyDistributeChunksAcrossRecordBatches(O, _.map((H) => H.data.concat()));
  }
  function uniformlyDistributeChunksAcrossRecordBatches(O, _) {
    const H = [...O.fields], V = [], q = { numBatches: _.reduce((ae, oe) => Math.max(ae, oe.length), 0) };
    let G = 0, J = 0, Z = -1;
    const te = _.length;
    let ne, ee = [];
    for (; q.numBatches-- > 0; ) {
      for (J = Number.POSITIVE_INFINITY, Z = -1; ++Z < te; )
        ee[Z] = ne = _[Z].shift(), J = Math.min(J, ne ? ne.length : J);
      Number.isFinite(J) && (ee = distributeChildren(H, J, ee, _, q), J > 0 && (V[G++] = makeData({
        type: new Struct(H),
        length: J,
        nullCount: 0,
        children: ee.slice()
      })));
    }
    return [
      O = O.assign(H),
      V.map((ae) => new RecordBatch$1(O, ae))
    ];
  }
  function distributeChildren(O, _, H, V, q) {
    var G;
    const J = (_ + 63 & -64) >> 3;
    for (let Z = -1, te = V.length; ++Z < te; ) {
      const ne = H[Z], ee = ne == null ? void 0 : ne.length;
      if (ee >= _)
        ee === _ ? H[Z] = ne : (H[Z] = ne.slice(0, _), q.numBatches = Math.max(q.numBatches, V[Z].unshift(ne.slice(_, ee - _))));
      else {
        const ae = O[Z];
        O[Z] = ae.clone({ nullable: !0 }), H[Z] = (G = ne == null ? void 0 : ne._changeLengthAndBackfillNullBitmap(_)) !== null && G !== void 0 ? G : makeData({
          type: ae.type,
          length: _,
          nullCount: _,
          nullBitmap: new Uint8Array(J)
        });
      }
    }
    return H;
  }
  var _a$1;
  class Table {
    constructor(..._) {
      var H, V;
      if (_.length === 0)
        return this.batches = [], this.schema = new Schema([]), this._offsets = [0], this;
      let q, G;
      _[0] instanceof Schema && (q = _.shift()), _.at(-1) instanceof Uint32Array && (G = _.pop());
      const J = (te) => {
        if (te) {
          if (te instanceof RecordBatch$1)
            return [te];
          if (te instanceof Table)
            return te.batches;
          if (te instanceof Data) {
            if (te.type instanceof Struct)
              return [new RecordBatch$1(new Schema(te.type.children), te)];
          } else {
            if (Array.isArray(te))
              return te.flatMap((ne) => J(ne));
            if (typeof te[Symbol.iterator] == "function")
              return [...te].flatMap((ne) => J(ne));
            if (typeof te == "object") {
              const ne = Object.keys(te), ee = ne.map((se) => new Vector([te[se]])), ae = q ?? new Schema(ne.map((se, ce) => new Field(String(se), ee[ce].type, ee[ce].nullable))), [, oe] = distributeVectorsIntoRecordBatches(ae, ee);
              return oe.length === 0 ? [new RecordBatch$1(te)] : oe;
            }
          }
        }
        return [];
      }, Z = _.flatMap((te) => J(te));
      if (q = (V = q ?? ((H = Z[0]) === null || H === void 0 ? void 0 : H.schema)) !== null && V !== void 0 ? V : new Schema([]), !(q instanceof Schema))
        throw new TypeError("Table constructor expects a [Schema, RecordBatch[]] pair.");
      for (const te of Z) {
        if (!(te instanceof RecordBatch$1))
          throw new TypeError("Table constructor expects a [Schema, RecordBatch[]] pair.");
        if (!compareSchemas(q, te.schema))
          throw new TypeError("Table and inner RecordBatch schemas must be equivalent.");
      }
      this.schema = q, this.batches = Z, this._offsets = G ?? computeChunkOffsets(this.data);
    }
    /**
     * The contiguous {@link RecordBatch `RecordBatch`} chunks of the Table rows.
     */
    get data() {
      return this.batches.map(({ data: _ }) => _);
    }
    /**
     * The number of columns in this Table.
     */
    get numCols() {
      return this.schema.fields.length;
    }
    /**
     * The number of rows in this Table.
     */
    get numRows() {
      return this.data.reduce((_, H) => _ + H.length, 0);
    }
    /**
     * The number of null rows in this Table.
     */
    get nullCount() {
      return this._nullCount === -1 && (this._nullCount = computeChunkNullCounts(this.data)), this._nullCount;
    }
    /**
     * Check whether an element is null.
     *
     * @param index The index at which to read the validity bitmap.
     */
    // @ts-ignore
    isValid(_) {
      return !1;
    }
    /**
     * Get an element value by position.
     *
     * @param index The index of the element to read.
     */
    // @ts-ignore
    get(_) {
      return null;
    }
    /**
      * Get an element value by position.
      * @param index The index of the element to read. A negative index will count back from the last element.
      */
    // @ts-ignore
    at(_) {
      return this.get(wrapIndex(_, this.numRows));
    }
    /**
     * Set an element value by position.
     *
     * @param index The index of the element to write.
     * @param value The value to set.
     */
    // @ts-ignore
    set(_, H) {
    }
    /**
     * Retrieve the index of the first occurrence of a value in an Vector.
     *
     * @param element The value to locate in the Vector.
     * @param offset The index at which to begin the search. If offset is omitted, the search starts at index 0.
     */
    // @ts-ignore
    indexOf(_, H) {
      return -1;
    }
    /**
     * Iterator for rows in this Table.
     */
    [Symbol.iterator]() {
      return this.batches.length > 0 ? instance$2.visit(new Vector(this.data)) : new Array(0)[Symbol.iterator]();
    }
    /**
     * Return a JavaScript Array of the Table rows.
     *
     * @returns An Array of Table rows.
     */
    toArray() {
      return [...this];
    }
    /**
     * Returns a string representation of the Table rows.
     *
     * @returns A string representation of the Table rows.
     */
    toString() {
      return `[
  ${this.toArray().join(`,
  `)}
]`;
    }
    /**
     * Combines two or more Tables of the same schema.
     *
     * @param others Additional Tables to add to the end of this Tables.
     */
    concat(..._) {
      const H = this.schema, V = this.data.concat(_.flatMap(({ data: q }) => q));
      return new Table(H, V.map((q) => new RecordBatch$1(H, q)));
    }
    /**
     * Return a zero-copy sub-section of this Table.
     *
     * @param begin The beginning of the specified portion of the Table.
     * @param end The end of the specified portion of the Table. This is exclusive of the element at the index 'end'.
     */
    slice(_, H) {
      const V = this.schema;
      [_, H] = clampRange({ length: this.numRows }, _, H);
      const q = sliceChunks(this.data, this._offsets, _, H);
      return new Table(V, q.map((G) => new RecordBatch$1(V, G)));
    }
    /**
     * Returns a child Vector by name, or null if this Vector has no child with the given name.
     *
     * @param name The name of the child to retrieve.
     */
    getChild(_) {
      return this.getChildAt(this.schema.fields.findIndex((H) => H.name === _));
    }
    /**
     * Returns a child Vector by index, or null if this Vector has no child at the supplied index.
     *
     * @param index The index of the child to retrieve.
     */
    getChildAt(_) {
      if (_ > -1 && _ < this.schema.fields.length) {
        const H = this.data.map((V) => V.children[_]);
        if (H.length === 0) {
          const { type: V } = this.schema.fields[_], q = makeData({ type: V, length: 0, nullCount: 0 });
          H.push(q._changeLengthAndBackfillNullBitmap(this.numRows));
        }
        return new Vector(H);
      }
      return null;
    }
    /**
     * Sets a child Vector by name.
     *
     * @param name The name of the child to overwrite.
     * @returns A new Table with the supplied child for the specified name.
     */
    setChild(_, H) {
      var V;
      return this.setChildAt((V = this.schema.fields) === null || V === void 0 ? void 0 : V.findIndex((q) => q.name === _), H);
    }
    setChildAt(_, H) {
      let V = this.schema, q = [...this.batches];
      if (_ > -1 && _ < this.numCols) {
        H || (H = new Vector([makeData({ type: new Null(), length: this.numRows })]));
        const G = V.fields.slice(), J = G[_].clone({ type: H.type }), Z = this.schema.fields.map((te, ne) => this.getChildAt(ne));
        [G[_], Z[_]] = [J, H], [V, q] = distributeVectorsIntoRecordBatches(V, Z);
      }
      return new Table(V, q);
    }
    /**
     * Construct a new Table containing only specified columns.
     *
     * @param columnNames Names of columns to keep.
     * @returns A new Table of columns matching the specified names.
     */
    select(_) {
      const H = this.schema.fields.reduce((V, q, G) => V.set(q.name, G), /* @__PURE__ */ new Map());
      return this.selectAt(_.map((V) => H.get(V)).filter((V) => V > -1));
    }
    /**
     * Construct a new Table containing only columns at the specified indices.
     *
     * @param columnIndices Indices of columns to keep.
     * @returns A new Table of columns at the specified indices.
     */
    selectAt(_) {
      const H = this.schema.selectAt(_), V = this.batches.map((q) => q.selectAt(_));
      return new Table(H, V);
    }
    assign(_) {
      const H = this.schema.fields, [V, q] = _.schema.fields.reduce((Z, te, ne) => {
        const [ee, ae] = Z, oe = H.findIndex((se) => se.name === te.name);
        return ~oe ? ae[oe] = ne : ee.push(ne), Z;
      }, [[], []]), G = this.schema.assign(_.schema), J = [
        ...H.map((Z, te) => [te, q[te]]).map(([Z, te]) => te === void 0 ? this.getChildAt(Z) : _.getChildAt(te)),
        ...V.map((Z) => _.getChildAt(Z))
      ].filter(Boolean);
      return new Table(...distributeVectorsIntoRecordBatches(G, J));
    }
  }
  _a$1 = Symbol.toStringTag;
  Table[_a$1] = ((O) => (O.schema = null, O.batches = [], O._offsets = new Uint32Array([0]), O._nullCount = -1, O[Symbol.isConcatSpreadable] = !0, O.isValid = wrapChunkedCall1(isChunkedValid), O.get = wrapChunkedCall1(instance$4.getVisitFn(Type.Struct)), O.set = wrapChunkedCall2(instance$5.getVisitFn(Type.Struct)), O.indexOf = wrapChunkedIndexOf(instance$3.getVisitFn(Type.Struct)), "Table"))(Table.prototype);
  var _a;
  let RecordBatch$1 = class Q3 {
    constructor(..._) {
      switch (_.length) {
        case 2: {
          if ([this.schema] = _, !(this.schema instanceof Schema))
            throw new TypeError("RecordBatch constructor expects a [Schema, Data] pair.");
          if ([
            ,
            this.data = makeData({
              nullCount: 0,
              type: new Struct(this.schema.fields),
              children: this.schema.fields.map((H) => makeData({ type: H.type, nullCount: 0 }))
            })
          ] = _, !(this.data instanceof Data))
            throw new TypeError("RecordBatch constructor expects a [Schema, Data] pair.");
          [this.schema, this.data] = ensureSameLengthData(this.schema, this.data.children);
          break;
        }
        case 1: {
          const [H] = _, { fields: V, children: q, length: G } = Object.keys(H).reduce((te, ne, ee) => (te.children[ee] = H[ne], te.length = Math.max(te.length, H[ne].length), te.fields[ee] = Field.new({ name: ne, type: H[ne].type, nullable: !0 }), te), {
            length: 0,
            fields: new Array(),
            children: new Array()
          }), J = new Schema(V), Z = makeData({ type: new Struct(V), length: G, children: q, nullCount: 0 });
          [this.schema, this.data] = ensureSameLengthData(J, Z.children, G);
          break;
        }
        default:
          throw new TypeError("RecordBatch constructor expects an Object mapping names to child Data, or a [Schema, Data] pair.");
      }
    }
    get dictionaries() {
      return this._dictionaries || (this._dictionaries = collectDictionaries(this.schema.fields, this.data.children));
    }
    /**
     * The number of columns in this RecordBatch.
     */
    get numCols() {
      return this.schema.fields.length;
    }
    /**
     * The number of rows in this RecordBatch.
     */
    get numRows() {
      return this.data.length;
    }
    /**
     * The number of null rows in this RecordBatch.
     */
    get nullCount() {
      return this.data.nullCount;
    }
    /**
     * Check whether an row is null.
     * @param index The index at which to read the validity bitmap.
     */
    isValid(_) {
      return this.data.getValid(_);
    }
    /**
     * Get a row by position.
     * @param index The index of the row to read.
     */
    get(_) {
      return instance$4.visit(this.data, _);
    }
    /**
      * Get a row value by position.
      * @param index The index of the row to read. A negative index will count back from the last row.
      */
    at(_) {
      return this.get(wrapIndex(_, this.numRows));
    }
    /**
     * Set a row by position.
     * @param index The index of the row to write.
     * @param value The value to set.
     */
    set(_, H) {
      return instance$5.visit(this.data, _, H);
    }
    /**
     * Retrieve the index of the first occurrence of a row in an RecordBatch.
     * @param element The row to locate in the RecordBatch.
     * @param offset The index at which to begin the search. If offset is omitted, the search starts at index 0.
     */
    indexOf(_, H) {
      return instance$3.visit(this.data, _, H);
    }
    /**
     * Iterator for rows in this RecordBatch.
     */
    [Symbol.iterator]() {
      return instance$2.visit(new Vector([this.data]));
    }
    /**
     * Return a JavaScript Array of the RecordBatch rows.
     * @returns An Array of RecordBatch rows.
     */
    toArray() {
      return [...this];
    }
    /**
     * Combines two or more RecordBatch of the same schema.
     * @param others Additional RecordBatch to add to the end of this RecordBatch.
     */
    concat(..._) {
      return new Table(this.schema, [this, ..._]);
    }
    /**
     * Return a zero-copy sub-section of this RecordBatch.
     * @param start The beginning of the specified portion of the RecordBatch.
     * @param end The end of the specified portion of the RecordBatch. This is exclusive of the row at the index 'end'.
     */
    slice(_, H) {
      const [V] = new Vector([this.data]).slice(_, H).data;
      return new Q3(this.schema, V);
    }
    /**
     * Returns a child Vector by name, or null if this Vector has no child with the given name.
     * @param name The name of the child to retrieve.
     */
    getChild(_) {
      var H;
      return this.getChildAt((H = this.schema.fields) === null || H === void 0 ? void 0 : H.findIndex((V) => V.name === _));
    }
    /**
     * Returns a child Vector by index, or null if this Vector has no child at the supplied index.
     * @param index The index of the child to retrieve.
     */
    getChildAt(_) {
      return _ > -1 && _ < this.schema.fields.length ? new Vector([this.data.children[_]]) : null;
    }
    /**
     * Sets a child Vector by name.
     * @param name The name of the child to overwrite.
     * @returns A new RecordBatch with the new child for the specified name.
     */
    setChild(_, H) {
      var V;
      return this.setChildAt((V = this.schema.fields) === null || V === void 0 ? void 0 : V.findIndex((q) => q.name === _), H);
    }
    setChildAt(_, H) {
      let V = this.schema, q = this.data;
      if (_ > -1 && _ < this.numCols) {
        H || (H = new Vector([makeData({ type: new Null(), length: this.numRows })]));
        const G = V.fields.slice(), J = q.children.slice(), Z = G[_].clone({ type: H.type });
        [G[_], J[_]] = [Z, H.data[0]], V = new Schema(G, new Map(this.schema.metadata)), q = makeData({ type: new Struct(G), children: J });
      }
      return new Q3(V, q);
    }
    /**
     * Construct a new RecordBatch containing only specified columns.
     *
     * @param columnNames Names of columns to keep.
     * @returns A new RecordBatch of columns matching the specified names.
     */
    select(_) {
      const H = this.schema.select(_), V = new Struct(H.fields), q = [];
      for (const G of _) {
        const J = this.schema.fields.findIndex((Z) => Z.name === G);
        ~J && (q[J] = this.data.children[J]);
      }
      return new Q3(H, makeData({ type: V, length: this.numRows, children: q }));
    }
    /**
     * Construct a new RecordBatch containing only columns at the specified indices.
     *
     * @param columnIndices Indices of columns to keep.
     * @returns A new RecordBatch of columns matching at the specified indices.
     */
    selectAt(_) {
      const H = this.schema.selectAt(_), V = _.map((G) => this.data.children[G]).filter(Boolean), q = makeData({ type: new Struct(H.fields), length: this.numRows, children: V });
      return new Q3(H, q);
    }
  };
  _a = Symbol.toStringTag;
  RecordBatch$1[_a] = ((O) => (O._nullCount = -1, O[Symbol.isConcatSpreadable] = !0, "RecordBatch"))(RecordBatch$1.prototype);
  function ensureSameLengthData(O, _, H = _.reduce((V, q) => Math.max(V, q.length), 0)) {
    var V;
    const q = [...O.fields], G = [..._], J = (H + 63 & -64) >> 3;
    for (const [Z, te] of O.fields.entries()) {
      const ne = _[Z];
      (!ne || ne.length !== H) && (q[Z] = te.clone({ nullable: !0 }), G[Z] = (V = ne == null ? void 0 : ne._changeLengthAndBackfillNullBitmap(H)) !== null && V !== void 0 ? V : makeData({
        type: te.type,
        length: H,
        nullCount: H,
        nullBitmap: new Uint8Array(J)
      }));
    }
    return [
      O.assign(q),
      makeData({ type: new Struct(q), length: H, children: G })
    ];
  }
  function collectDictionaries(O, _, H = /* @__PURE__ */ new Map()) {
    var V, q;
    if (((V = O == null ? void 0 : O.length) !== null && V !== void 0 ? V : 0) > 0 && (O == null ? void 0 : O.length) === (_ == null ? void 0 : _.length))
      for (let G = -1, J = O.length; ++G < J; ) {
        const { type: Z } = O[G], te = _[G];
        for (const ne of [te, ...((q = te == null ? void 0 : te.dictionary) === null || q === void 0 ? void 0 : q.data) || []])
          collectDictionaries(Z.children, ne == null ? void 0 : ne.children, H);
        if (DataType.isDictionary(Z)) {
          const { id: ne } = Z;
          if (!H.has(ne))
            te != null && te.dictionary && H.set(ne, te.dictionary);
          else if (H.get(ne) !== te.dictionary)
            throw new Error("Cannot create Schema containing two different dictionaries with the same Id");
        }
      }
    return H;
  }
  class _InternalEmptyPlaceholderRecordBatch extends RecordBatch$1 {
    constructor(_) {
      const H = _.fields.map((q) => makeData({ type: q.type })), V = makeData({ type: new Struct(_.fields), nullCount: 0, children: H });
      super(_, V);
    }
  }
  let Message$1 = class Uh {
    constructor() {
      this.bb = null, this.bb_pos = 0;
    }
    __init(_, H) {
      return this.bb_pos = _, this.bb = H, this;
    }
    static getRootAsMessage(_, H) {
      return (H || new Uh()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    static getSizePrefixedRootAsMessage(_, H) {
      return _.setPosition(_.position() + SIZE_PREFIX_LENGTH), (H || new Uh()).__init(_.readInt32(_.position()) + _.position(), _);
    }
    version() {
      const _ = this.bb.__offset(this.bb_pos, 4);
      return _ ? this.bb.readInt16(this.bb_pos + _) : MetadataVersion.V1;
    }
    headerType() {
      const _ = this.bb.__offset(this.bb_pos, 6);
      return _ ? this.bb.readUint8(this.bb_pos + _) : MessageHeader.NONE;
    }
    header(_) {
      const H = this.bb.__offset(this.bb_pos, 8);
      return H ? this.bb.__union(_, this.bb_pos + H) : null;
    }
    bodyLength() {
      const _ = this.bb.__offset(this.bb_pos, 10);
      return _ ? this.bb.readInt64(this.bb_pos + _) : BigInt("0");
    }
    customMetadata(_, H) {
      const V = this.bb.__offset(this.bb_pos, 12);
      return V ? (H || new KeyValue()).__init(this.bb.__indirect(this.bb.__vector(this.bb_pos + V) + _ * 4), this.bb) : null;
    }
    customMetadataLength() {
      const _ = this.bb.__offset(this.bb_pos, 12);
      return _ ? this.bb.__vector_len(this.bb_pos + _) : 0;
    }
    static startMessage(_) {
      _.startObject(5);
    }
    static addVersion(_, H) {
      _.addFieldInt16(0, H, MetadataVersion.V1);
    }
    static addHeaderType(_, H) {
      _.addFieldInt8(1, H, MessageHeader.NONE);
    }
    static addHeader(_, H) {
      _.addFieldOffset(2, H, 0);
    }
    static addBodyLength(_, H) {
      _.addFieldInt64(3, H, BigInt("0"));
    }
    static addCustomMetadata(_, H) {
      _.addFieldOffset(4, H, 0);
    }
    static createCustomMetadataVector(_, H) {
      _.startVector(4, H.length, 4);
      for (let V = H.length - 1; V >= 0; V--)
        _.addOffset(H[V]);
      return _.endVector();
    }
    static startCustomMetadataVector(_, H) {
      _.startVector(4, H, 4);
    }
    static endMessage(_) {
      return _.endObject();
    }
    static finishMessageBuffer(_, H) {
      _.finish(H);
    }
    static finishSizePrefixedMessageBuffer(_, H) {
      _.finish(H, void 0, !0);
    }
    static createMessage(_, H, V, q, G, J) {
      return Uh.startMessage(_), Uh.addVersion(_, H), Uh.addHeaderType(_, V), Uh.addHeader(_, q), Uh.addBodyLength(_, G), Uh.addCustomMetadata(_, J), Uh.endMessage(_);
    }
  };
  class TypeAssembler extends Visitor {
    visit(_, H) {
      return _ == null || H == null ? void 0 : super.visit(_, H);
    }
    visitNull(_, H) {
      return Null$1.startNull(H), Null$1.endNull(H);
    }
    visitInt(_, H) {
      return Int.startInt(H), Int.addBitWidth(H, _.bitWidth), Int.addIsSigned(H, _.isSigned), Int.endInt(H);
    }
    visitFloat(_, H) {
      return FloatingPoint.startFloatingPoint(H), FloatingPoint.addPrecision(H, _.precision), FloatingPoint.endFloatingPoint(H);
    }
    visitBinary(_, H) {
      return Binary$1.startBinary(H), Binary$1.endBinary(H);
    }
    visitLargeBinary(_, H) {
      return LargeBinary$1.startLargeBinary(H), LargeBinary$1.endLargeBinary(H);
    }
    visitBool(_, H) {
      return Bool$1.startBool(H), Bool$1.endBool(H);
    }
    visitUtf8(_, H) {
      return Utf8$1.startUtf8(H), Utf8$1.endUtf8(H);
    }
    visitLargeUtf8(_, H) {
      return LargeUtf8$1.startLargeUtf8(H), LargeUtf8$1.endLargeUtf8(H);
    }
    visitDecimal(_, H) {
      return Decimal$1.startDecimal(H), Decimal$1.addScale(H, _.scale), Decimal$1.addPrecision(H, _.precision), Decimal$1.addBitWidth(H, _.bitWidth), Decimal$1.endDecimal(H);
    }
    visitDate(_, H) {
      return Date$1.startDate(H), Date$1.addUnit(H, _.unit), Date$1.endDate(H);
    }
    visitTime(_, H) {
      return Time.startTime(H), Time.addUnit(H, _.unit), Time.addBitWidth(H, _.bitWidth), Time.endTime(H);
    }
    visitTimestamp(_, H) {
      const V = _.timezone && H.createString(_.timezone) || void 0;
      return Timestamp.startTimestamp(H), Timestamp.addUnit(H, _.unit), V !== void 0 && Timestamp.addTimezone(H, V), Timestamp.endTimestamp(H);
    }
    visitInterval(_, H) {
      return Interval.startInterval(H), Interval.addUnit(H, _.unit), Interval.endInterval(H);
    }
    visitDuration(_, H) {
      return Duration$1.startDuration(H), Duration$1.addUnit(H, _.unit), Duration$1.endDuration(H);
    }
    visitList(_, H) {
      return List$1.startList(H), List$1.endList(H);
    }
    visitStruct(_, H) {
      return Struct_.startStruct_(H), Struct_.endStruct_(H);
    }
    visitUnion(_, H) {
      Union.startTypeIdsVector(H, _.typeIds.length);
      const V = Union.createTypeIdsVector(H, _.typeIds);
      return Union.startUnion(H), Union.addMode(H, _.mode), Union.addTypeIds(H, V), Union.endUnion(H);
    }
    visitDictionary(_, H) {
      const V = this.visit(_.indices, H);
      return DictionaryEncoding.startDictionaryEncoding(H), DictionaryEncoding.addId(H, BigInt(_.id)), DictionaryEncoding.addIsOrdered(H, _.isOrdered), V !== void 0 && DictionaryEncoding.addIndexType(H, V), DictionaryEncoding.endDictionaryEncoding(H);
    }
    visitFixedSizeBinary(_, H) {
      return FixedSizeBinary$1.startFixedSizeBinary(H), FixedSizeBinary$1.addByteWidth(H, _.byteWidth), FixedSizeBinary$1.endFixedSizeBinary(H);
    }
    visitFixedSizeList(_, H) {
      return FixedSizeList$1.startFixedSizeList(H), FixedSizeList$1.addListSize(H, _.listSize), FixedSizeList$1.endFixedSizeList(H);
    }
    visitMap(_, H) {
      return Map$1.startMap(H), Map$1.addKeysSorted(H, _.keysSorted), Map$1.endMap(H);
    }
  }
  const instance = new TypeAssembler();
  function schemaFromJSON(O, _ = /* @__PURE__ */ new Map()) {
    return new Schema(schemaFieldsFromJSON(O, _), customMetadataFromJSON(O.metadata), _);
  }
  function recordBatchFromJSON(O) {
    return new RecordBatch(O.count, fieldNodesFromJSON(O.columns), buffersFromJSON(O.columns));
  }
  function dictionaryBatchFromJSON(O) {
    return new DictionaryBatch(recordBatchFromJSON(O.data), O.id, O.isDelta);
  }
  function schemaFieldsFromJSON(O, _) {
    return (O.fields || []).filter(Boolean).map((H) => Field.fromJSON(H, _));
  }
  function fieldChildrenFromJSON(O, _) {
    return (O.children || []).filter(Boolean).map((H) => Field.fromJSON(H, _));
  }
  function fieldNodesFromJSON(O) {
    return (O || []).reduce((_, H) => [
      ..._,
      new FieldNode(H.count, nullCountFromJSON(H.VALIDITY)),
      ...fieldNodesFromJSON(H.children)
    ], []);
  }
  function buffersFromJSON(O, _ = []) {
    for (let H = -1, V = (O || []).length; ++H < V; ) {
      const q = O[H];
      q.VALIDITY && _.push(new BufferRegion(_.length, q.VALIDITY.length)), q.TYPE_ID && _.push(new BufferRegion(_.length, q.TYPE_ID.length)), q.OFFSET && _.push(new BufferRegion(_.length, q.OFFSET.length)), q.DATA && _.push(new BufferRegion(_.length, q.DATA.length)), _ = buffersFromJSON(q.children, _);
    }
    return _;
  }
  function nullCountFromJSON(O) {
    return (O || []).reduce((_, H) => _ + +(H === 0), 0);
  }
  function fieldFromJSON(O, _) {
    let H, V, q, G, J, Z;
    return !_ || !(G = O.dictionary) ? (J = typeFromJSON(O, fieldChildrenFromJSON(O, _)), q = new Field(O.name, J, O.nullable, customMetadataFromJSON(O.metadata))) : _.has(H = G.id) ? (V = (V = G.indexType) ? indexTypeFromJSON(V) : new Int32(), Z = new Dictionary(_.get(H), V, H, G.isOrdered), q = new Field(O.name, Z, O.nullable, customMetadataFromJSON(O.metadata))) : (V = (V = G.indexType) ? indexTypeFromJSON(V) : new Int32(), _.set(H, J = typeFromJSON(O, fieldChildrenFromJSON(O, _))), Z = new Dictionary(J, V, H, G.isOrdered), q = new Field(O.name, Z, O.nullable, customMetadataFromJSON(O.metadata))), q || null;
  }
  function customMetadataFromJSON(O = []) {
    return new Map(O.map(({ key: _, value: H }) => [_, H]));
  }
  function indexTypeFromJSON(O) {
    return new Int_(O.isSigned, O.bitWidth);
  }
  function typeFromJSON(O, _) {
    const H = O.type.name;
    switch (H) {
      case "NONE":
        return new Null();
      case "null":
        return new Null();
      case "binary":
        return new Binary();
      case "largebinary":
        return new LargeBinary();
      case "utf8":
        return new Utf8();
      case "largeutf8":
        return new LargeUtf8();
      case "bool":
        return new Bool();
      case "list":
        return new List((_ || [])[0]);
      case "struct":
        return new Struct(_ || []);
      case "struct_":
        return new Struct(_ || []);
    }
    switch (H) {
      case "int": {
        const V = O.type;
        return new Int_(V.isSigned, V.bitWidth);
      }
      case "floatingpoint": {
        const V = O.type;
        return new Float(Precision[V.precision]);
      }
      case "decimal": {
        const V = O.type;
        return new Decimal(V.scale, V.precision, V.bitWidth);
      }
      case "date": {
        const V = O.type;
        return new Date_(DateUnit[V.unit]);
      }
      case "time": {
        const V = O.type;
        return new Time_(TimeUnit[V.unit], V.bitWidth);
      }
      case "timestamp": {
        const V = O.type;
        return new Timestamp_(TimeUnit[V.unit], V.timezone);
      }
      case "interval": {
        const V = O.type;
        return new Interval_(IntervalUnit[V.unit]);
      }
      case "duration": {
        const V = O.type;
        return new Duration(TimeUnit[V.unit]);
      }
      case "union": {
        const V = O.type, [q, ...G] = (V.mode + "").toLowerCase(), J = q.toUpperCase() + G.join("");
        return new Union_(UnionMode[J], V.typeIds || [], _ || []);
      }
      case "fixedsizebinary": {
        const V = O.type;
        return new FixedSizeBinary(V.byteWidth);
      }
      case "fixedsizelist": {
        const V = O.type;
        return new FixedSizeList(V.listSize, (_ || [])[0]);
      }
      case "map": {
        const V = O.type;
        return new Map_((_ || [])[0], V.keysSorted);
      }
    }
    throw new Error(`Unrecognized type: "${H}"`);
  }
  var Builder = Builder$2, ByteBuffer = ByteBuffer$2;
  class Message {
    /** @nocollapse */
    static fromJSON(_, H) {
      const V = new Message(0, MetadataVersion.V5, H);
      return V._createHeader = messageHeaderFromJSON(_, H), V;
    }
    /** @nocollapse */
    static decode(_) {
      _ = new ByteBuffer(toUint8Array(_));
      const H = Message$1.getRootAsMessage(_), V = H.bodyLength(), q = H.version(), G = H.headerType(), J = new Message(V, q, G);
      return J._createHeader = decodeMessageHeader(H, G), J;
    }
    /** @nocollapse */
    static encode(_) {
      const H = new Builder();
      let V = -1;
      return _.isSchema() ? V = Schema.encode(H, _.header()) : _.isRecordBatch() ? V = RecordBatch.encode(H, _.header()) : _.isDictionaryBatch() && (V = DictionaryBatch.encode(H, _.header())), Message$1.startMessage(H), Message$1.addVersion(H, MetadataVersion.V5), Message$1.addHeader(H, V), Message$1.addHeaderType(H, _.headerType), Message$1.addBodyLength(H, BigInt(_.bodyLength)), Message$1.finishMessageBuffer(H, Message$1.endMessage(H)), H.asUint8Array();
    }
    /** @nocollapse */
    static from(_, H = 0) {
      if (_ instanceof Schema)
        return new Message(0, MetadataVersion.V5, MessageHeader.Schema, _);
      if (_ instanceof RecordBatch)
        return new Message(H, MetadataVersion.V5, MessageHeader.RecordBatch, _);
      if (_ instanceof DictionaryBatch)
        return new Message(H, MetadataVersion.V5, MessageHeader.DictionaryBatch, _);
      throw new Error(`Unrecognized Message header: ${_}`);
    }
    get type() {
      return this.headerType;
    }
    get version() {
      return this._version;
    }
    get headerType() {
      return this._headerType;
    }
    get bodyLength() {
      return this._bodyLength;
    }
    header() {
      return this._createHeader();
    }
    isSchema() {
      return this.headerType === MessageHeader.Schema;
    }
    isRecordBatch() {
      return this.headerType === MessageHeader.RecordBatch;
    }
    isDictionaryBatch() {
      return this.headerType === MessageHeader.DictionaryBatch;
    }
    constructor(_, H, V, q) {
      this._version = H, this._headerType = V, this.body = new Uint8Array(0), q && (this._createHeader = () => q), this._bodyLength = bigIntToNumber(_);
    }
  }
  class RecordBatch {
    get nodes() {
      return this._nodes;
    }
    get length() {
      return this._length;
    }
    get buffers() {
      return this._buffers;
    }
    constructor(_, H, V) {
      this._nodes = H, this._buffers = V, this._length = bigIntToNumber(_);
    }
  }
  class DictionaryBatch {
    get id() {
      return this._id;
    }
    get data() {
      return this._data;
    }
    get isDelta() {
      return this._isDelta;
    }
    get length() {
      return this.data.length;
    }
    get nodes() {
      return this.data.nodes;
    }
    get buffers() {
      return this.data.buffers;
    }
    constructor(_, H, V = !1) {
      this._data = _, this._isDelta = V, this._id = bigIntToNumber(H);
    }
  }
  class BufferRegion {
    constructor(_, H) {
      this.offset = bigIntToNumber(_), this.length = bigIntToNumber(H);
    }
  }
  class FieldNode {
    constructor(_, H) {
      this.length = bigIntToNumber(_), this.nullCount = bigIntToNumber(H);
    }
  }
  function messageHeaderFromJSON(O, _) {
    return () => {
      switch (_) {
        case MessageHeader.Schema:
          return Schema.fromJSON(O);
        case MessageHeader.RecordBatch:
          return RecordBatch.fromJSON(O);
        case MessageHeader.DictionaryBatch:
          return DictionaryBatch.fromJSON(O);
      }
      throw new Error(`Unrecognized Message type: { name: ${MessageHeader[_]}, type: ${_} }`);
    };
  }
  function decodeMessageHeader(O, _) {
    return () => {
      switch (_) {
        case MessageHeader.Schema:
          return Schema.decode(O.header(new Schema$1()), /* @__PURE__ */ new Map(), O.version());
        case MessageHeader.RecordBatch:
          return RecordBatch.decode(O.header(new RecordBatch$2()), O.version());
        case MessageHeader.DictionaryBatch:
          return DictionaryBatch.decode(O.header(new DictionaryBatch$1()), O.version());
      }
      throw new Error(`Unrecognized Message type: { name: ${MessageHeader[_]}, type: ${_} }`);
    };
  }
  Field.encode = encodeField;
  Field.decode = decodeField;
  Field.fromJSON = fieldFromJSON;
  Schema.encode = encodeSchema;
  Schema.decode = decodeSchema;
  Schema.fromJSON = schemaFromJSON;
  RecordBatch.encode = encodeRecordBatch;
  RecordBatch.decode = decodeRecordBatch;
  RecordBatch.fromJSON = recordBatchFromJSON;
  DictionaryBatch.encode = encodeDictionaryBatch;
  DictionaryBatch.decode = decodeDictionaryBatch;
  DictionaryBatch.fromJSON = dictionaryBatchFromJSON;
  FieldNode.encode = encodeFieldNode;
  FieldNode.decode = decodeFieldNode;
  BufferRegion.encode = encodeBufferRegion;
  BufferRegion.decode = decodeBufferRegion;
  function decodeSchema(O, _ = /* @__PURE__ */ new Map(), H = MetadataVersion.V5) {
    const V = decodeSchemaFields(O, _);
    return new Schema(V, decodeCustomMetadata(O), _, H);
  }
  function decodeRecordBatch(O, _ = MetadataVersion.V5) {
    if (O.compression() !== null)
      throw new Error("Record batch compression not implemented");
    return new RecordBatch(O.length(), decodeFieldNodes(O), decodeBuffers(O, _));
  }
  function decodeDictionaryBatch(O, _ = MetadataVersion.V5) {
    return new DictionaryBatch(RecordBatch.decode(O.data(), _), O.id(), O.isDelta());
  }
  function decodeBufferRegion(O) {
    return new BufferRegion(O.offset(), O.length());
  }
  function decodeFieldNode(O) {
    return new FieldNode(O.length(), O.nullCount());
  }
  function decodeFieldNodes(O) {
    const _ = [];
    for (let H, V = -1, q = -1, G = O.nodesLength(); ++V < G; )
      (H = O.nodes(V)) && (_[++q] = FieldNode.decode(H));
    return _;
  }
  function decodeBuffers(O, _) {
    const H = [];
    for (let V, q = -1, G = -1, J = O.buffersLength(); ++q < J; )
      (V = O.buffers(q)) && (_ < MetadataVersion.V4 && (V.bb_pos += 8 * (q + 1)), H[++G] = BufferRegion.decode(V));
    return H;
  }
  function decodeSchemaFields(O, _) {
    const H = [];
    for (let V, q = -1, G = -1, J = O.fieldsLength(); ++q < J; )
      (V = O.fields(q)) && (H[++G] = Field.decode(V, _));
    return H;
  }
  function decodeFieldChildren(O, _) {
    const H = [];
    for (let V, q = -1, G = -1, J = O.childrenLength(); ++q < J; )
      (V = O.children(q)) && (H[++G] = Field.decode(V, _));
    return H;
  }
  function decodeField(O, _) {
    let H, V, q, G, J, Z;
    return !_ || !(Z = O.dictionary()) ? (q = decodeFieldType(O, decodeFieldChildren(O, _)), V = new Field(O.name(), q, O.nullable(), decodeCustomMetadata(O))) : _.has(H = bigIntToNumber(Z.id())) ? (G = (G = Z.indexType()) ? decodeIndexType(G) : new Int32(), J = new Dictionary(_.get(H), G, H, Z.isOrdered()), V = new Field(O.name(), J, O.nullable(), decodeCustomMetadata(O))) : (G = (G = Z.indexType()) ? decodeIndexType(G) : new Int32(), _.set(H, q = decodeFieldType(O, decodeFieldChildren(O, _))), J = new Dictionary(q, G, H, Z.isOrdered()), V = new Field(O.name(), J, O.nullable(), decodeCustomMetadata(O))), V || null;
  }
  function decodeCustomMetadata(O) {
    const _ = /* @__PURE__ */ new Map();
    if (O)
      for (let H, V, q = -1, G = Math.trunc(O.customMetadataLength()); ++q < G; )
        (H = O.customMetadata(q)) && (V = H.key()) != null && _.set(V, H.value());
    return _;
  }
  function decodeIndexType(O) {
    return new Int_(O.isSigned(), O.bitWidth());
  }
  function decodeFieldType(O, _) {
    const H = O.typeType();
    switch (H) {
      case Type$1.NONE:
        return new Null();
      case Type$1.Null:
        return new Null();
      case Type$1.Binary:
        return new Binary();
      case Type$1.LargeBinary:
        return new LargeBinary();
      case Type$1.Utf8:
        return new Utf8();
      case Type$1.LargeUtf8:
        return new LargeUtf8();
      case Type$1.Bool:
        return new Bool();
      case Type$1.List:
        return new List((_ || [])[0]);
      case Type$1.Struct_:
        return new Struct(_ || []);
    }
    switch (H) {
      case Type$1.Int: {
        const V = O.type(new Int());
        return new Int_(V.isSigned(), V.bitWidth());
      }
      case Type$1.FloatingPoint: {
        const V = O.type(new FloatingPoint());
        return new Float(V.precision());
      }
      case Type$1.Decimal: {
        const V = O.type(new Decimal$1());
        return new Decimal(V.scale(), V.precision(), V.bitWidth());
      }
      case Type$1.Date: {
        const V = O.type(new Date$1());
        return new Date_(V.unit());
      }
      case Type$1.Time: {
        const V = O.type(new Time());
        return new Time_(V.unit(), V.bitWidth());
      }
      case Type$1.Timestamp: {
        const V = O.type(new Timestamp());
        return new Timestamp_(V.unit(), V.timezone());
      }
      case Type$1.Interval: {
        const V = O.type(new Interval());
        return new Interval_(V.unit());
      }
      case Type$1.Duration: {
        const V = O.type(new Duration$1());
        return new Duration(V.unit());
      }
      case Type$1.Union: {
        const V = O.type(new Union());
        return new Union_(V.mode(), V.typeIdsArray() || [], _ || []);
      }
      case Type$1.FixedSizeBinary: {
        const V = O.type(new FixedSizeBinary$1());
        return new FixedSizeBinary(V.byteWidth());
      }
      case Type$1.FixedSizeList: {
        const V = O.type(new FixedSizeList$1());
        return new FixedSizeList(V.listSize(), (_ || [])[0]);
      }
      case Type$1.Map: {
        const V = O.type(new Map$1());
        return new Map_((_ || [])[0], V.keysSorted());
      }
    }
    throw new Error(`Unrecognized type: "${Type$1[H]}" (${H})`);
  }
  function encodeSchema(O, _) {
    const H = _.fields.map((G) => Field.encode(O, G));
    Schema$1.startFieldsVector(O, H.length);
    const V = Schema$1.createFieldsVector(O, H), q = _.metadata && _.metadata.size > 0 ? Schema$1.createCustomMetadataVector(O, [..._.metadata].map(([G, J]) => {
      const Z = O.createString(`${G}`), te = O.createString(`${J}`);
      return KeyValue.startKeyValue(O), KeyValue.addKey(O, Z), KeyValue.addValue(O, te), KeyValue.endKeyValue(O);
    })) : -1;
    return Schema$1.startSchema(O), Schema$1.addFields(O, V), Schema$1.addEndianness(O, platformIsLittleEndian ? Endianness.Little : Endianness.Big), q !== -1 && Schema$1.addCustomMetadata(O, q), Schema$1.endSchema(O);
  }
  function encodeField(O, _) {
    let H = -1, V = -1, q = -1;
    const G = _.type;
    let J = _.typeId;
    DataType.isDictionary(G) ? (J = G.dictionary.typeId, q = instance.visit(G, O), V = instance.visit(G.dictionary, O)) : V = instance.visit(G, O);
    const Z = (G.children || []).map((ee) => Field.encode(O, ee)), te = Field$1.createChildrenVector(O, Z), ne = _.metadata && _.metadata.size > 0 ? Field$1.createCustomMetadataVector(O, [..._.metadata].map(([ee, ae]) => {
      const oe = O.createString(`${ee}`), se = O.createString(`${ae}`);
      return KeyValue.startKeyValue(O), KeyValue.addKey(O, oe), KeyValue.addValue(O, se), KeyValue.endKeyValue(O);
    })) : -1;
    return _.name && (H = O.createString(_.name)), Field$1.startField(O), Field$1.addType(O, V), Field$1.addTypeType(O, J), Field$1.addChildren(O, te), Field$1.addNullable(O, !!_.nullable), H !== -1 && Field$1.addName(O, H), q !== -1 && Field$1.addDictionary(O, q), ne !== -1 && Field$1.addCustomMetadata(O, ne), Field$1.endField(O);
  }
  function encodeRecordBatch(O, _) {
    const H = _.nodes || [], V = _.buffers || [];
    RecordBatch$2.startNodesVector(O, H.length);
    for (const J of H.slice().reverse())
      FieldNode.encode(O, J);
    const q = O.endVector();
    RecordBatch$2.startBuffersVector(O, V.length);
    for (const J of V.slice().reverse())
      BufferRegion.encode(O, J);
    const G = O.endVector();
    return RecordBatch$2.startRecordBatch(O), RecordBatch$2.addLength(O, BigInt(_.length)), RecordBatch$2.addNodes(O, q), RecordBatch$2.addBuffers(O, G), RecordBatch$2.endRecordBatch(O);
  }
  function encodeDictionaryBatch(O, _) {
    const H = RecordBatch.encode(O, _.data);
    return DictionaryBatch$1.startDictionaryBatch(O), DictionaryBatch$1.addId(O, BigInt(_.id)), DictionaryBatch$1.addIsDelta(O, _.isDelta), DictionaryBatch$1.addData(O, H), DictionaryBatch$1.endDictionaryBatch(O);
  }
  function encodeFieldNode(O, _) {
    return FieldNode$1.createFieldNode(O, BigInt(_.length), BigInt(_.nullCount));
  }
  function encodeBufferRegion(O, _) {
    return Buffer$1.createBuffer(O, BigInt(_.offset), BigInt(_.length));
  }
  const platformIsLittleEndian = (() => {
    const O = new ArrayBuffer(2);
    return new DataView(O).setInt16(
      0,
      256,
      !0
      /* littleEndian */
    ), new Int16Array(O)[0] === 256;
  })(), invalidMessageType = (O) => `Expected ${MessageHeader[O]} Message in stream, but was null or length 0.`, nullMessage = (O) => `Header pointer of flatbuffer-encoded ${MessageHeader[O]} Message is null or length 0.`, invalidMessageMetadata = (O, _) => `Expected to read ${O} metadata bytes, but only read ${_}.`, invalidMessageBodyLength = (O, _) => `Expected to read ${O} bytes for message body, but only read ${_}.`;
  class MessageReader {
    constructor(_) {
      this.source = _ instanceof ByteStream ? _ : new ByteStream(_);
    }
    [Symbol.iterator]() {
      return this;
    }
    next() {
      let _;
      return (_ = this.readMetadataLength()).done || _.value === -1 && (_ = this.readMetadataLength()).done || (_ = this.readMetadata(_.value)).done ? ITERATOR_DONE : _;
    }
    throw(_) {
      return this.source.throw(_);
    }
    return(_) {
      return this.source.return(_);
    }
    readMessage(_) {
      let H;
      if ((H = this.next()).done)
        return null;
      if (_ != null && H.value.headerType !== _)
        throw new Error(invalidMessageType(_));
      return H.value;
    }
    readMessageBody(_) {
      if (_ <= 0)
        return new Uint8Array(0);
      const H = toUint8Array(this.source.read(_));
      if (H.byteLength < _)
        throw new Error(invalidMessageBodyLength(_, H.byteLength));
      return (
        /* 1. */
        H.byteOffset % 8 === 0 && /* 2. */
        H.byteOffset + H.byteLength <= H.buffer.byteLength ? H : H.slice()
      );
    }
    readSchema(_ = !1) {
      const H = MessageHeader.Schema, V = this.readMessage(H), q = V == null ? void 0 : V.header();
      if (_ && !q)
        throw new Error(nullMessage(H));
      return q;
    }
    readMetadataLength() {
      const _ = this.source.read(PADDING), H = _ && new ByteBuffer$2(_), V = (H == null ? void 0 : H.readInt32(0)) || 0;
      return { done: V === 0, value: V };
    }
    readMetadata(_) {
      const H = this.source.read(_);
      if (!H)
        return ITERATOR_DONE;
      if (H.byteLength < _)
        throw new Error(invalidMessageMetadata(_, H.byteLength));
      return { done: !1, value: Message.decode(H) };
    }
  }
  class AsyncMessageReader {
    constructor(_, H) {
      this.source = _ instanceof AsyncByteStream ? _ : isFileHandle(_) ? new AsyncRandomAccessFile(_, H) : new AsyncByteStream(_);
    }
    [Symbol.asyncIterator]() {
      return this;
    }
    next() {
      return __awaiter(this, void 0, void 0, function* () {
        let _;
        return (_ = yield this.readMetadataLength()).done || _.value === -1 && (_ = yield this.readMetadataLength()).done || (_ = yield this.readMetadata(_.value)).done ? ITERATOR_DONE : _;
      });
    }
    throw(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return yield this.source.throw(_);
      });
    }
    return(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return yield this.source.return(_);
      });
    }
    readMessage(_) {
      return __awaiter(this, void 0, void 0, function* () {
        let H;
        if ((H = yield this.next()).done)
          return null;
        if (_ != null && H.value.headerType !== _)
          throw new Error(invalidMessageType(_));
        return H.value;
      });
    }
    readMessageBody(_) {
      return __awaiter(this, void 0, void 0, function* () {
        if (_ <= 0)
          return new Uint8Array(0);
        const H = toUint8Array(yield this.source.read(_));
        if (H.byteLength < _)
          throw new Error(invalidMessageBodyLength(_, H.byteLength));
        return (
          /* 1. */
          H.byteOffset % 8 === 0 && /* 2. */
          H.byteOffset + H.byteLength <= H.buffer.byteLength ? H : H.slice()
        );
      });
    }
    readSchema() {
      return __awaiter(this, arguments, void 0, function* (_ = !1) {
        const H = MessageHeader.Schema, V = yield this.readMessage(H), q = V == null ? void 0 : V.header();
        if (_ && !q)
          throw new Error(nullMessage(H));
        return q;
      });
    }
    readMetadataLength() {
      return __awaiter(this, void 0, void 0, function* () {
        const _ = yield this.source.read(PADDING), H = _ && new ByteBuffer$2(_), V = (H == null ? void 0 : H.readInt32(0)) || 0;
        return { done: V === 0, value: V };
      });
    }
    readMetadata(_) {
      return __awaiter(this, void 0, void 0, function* () {
        const H = yield this.source.read(_);
        if (!H)
          return ITERATOR_DONE;
        if (H.byteLength < _)
          throw new Error(invalidMessageMetadata(_, H.byteLength));
        return { done: !1, value: Message.decode(H) };
      });
    }
  }
  class JSONMessageReader extends MessageReader {
    constructor(_) {
      super(new Uint8Array(0)), this._schema = !1, this._body = [], this._batchIndex = 0, this._dictionaryIndex = 0, this._json = _ instanceof ArrowJSON ? _ : new ArrowJSON(_);
    }
    next() {
      const { _json: _ } = this;
      if (!this._schema)
        return this._schema = !0, { done: !1, value: Message.fromJSON(_.schema, MessageHeader.Schema) };
      if (this._dictionaryIndex < _.dictionaries.length) {
        const H = _.dictionaries[this._dictionaryIndex++];
        return this._body = H.data.columns, { done: !1, value: Message.fromJSON(H, MessageHeader.DictionaryBatch) };
      }
      if (this._batchIndex < _.batches.length) {
        const H = _.batches[this._batchIndex++];
        return this._body = H.columns, { done: !1, value: Message.fromJSON(H, MessageHeader.RecordBatch) };
      }
      return this._body = [], ITERATOR_DONE;
    }
    readMessageBody(_) {
      return H(this._body);
      function H(V) {
        return (V || []).reduce((q, G) => [
          ...q,
          ...G.VALIDITY && [G.VALIDITY] || [],
          ...G.TYPE_ID && [G.TYPE_ID] || [],
          ...G.OFFSET && [G.OFFSET] || [],
          ...G.DATA && [G.DATA] || [],
          ...H(G.children)
        ], []);
      }
    }
    readMessage(_) {
      let H;
      if ((H = this.next()).done)
        return null;
      if (_ != null && H.value.headerType !== _)
        throw new Error(invalidMessageType(_));
      return H.value;
    }
    readSchema() {
      const _ = MessageHeader.Schema, H = this.readMessage(_), V = H == null ? void 0 : H.header();
      if (!H || !V)
        throw new Error(nullMessage(_));
      return V;
    }
  }
  const PADDING = 4, MAGIC_STR = "ARROW1", MAGIC = new Uint8Array(MAGIC_STR.length);
  for (let O = 0; O < MAGIC_STR.length; O += 1)
    MAGIC[O] = MAGIC_STR.codePointAt(O);
  function checkForMagicArrowString(O, _ = 0) {
    for (let H = -1, V = MAGIC.length; ++H < V; )
      if (MAGIC[H] !== O[_ + H])
        return !1;
    return !0;
  }
  const magicLength = MAGIC.length, magicAndPadding = magicLength + PADDING, magicX2AndPadding = magicLength * 2 + PADDING;
  class RecordBatchReader extends ReadableInterop {
    constructor(_) {
      super(), this._impl = _;
    }
    get closed() {
      return this._impl.closed;
    }
    get schema() {
      return this._impl.schema;
    }
    get autoDestroy() {
      return this._impl.autoDestroy;
    }
    get dictionaries() {
      return this._impl.dictionaries;
    }
    get numDictionaries() {
      return this._impl.numDictionaries;
    }
    get numRecordBatches() {
      return this._impl.numRecordBatches;
    }
    get footer() {
      return this._impl.isFile() ? this._impl.footer : null;
    }
    isSync() {
      return this._impl.isSync();
    }
    isAsync() {
      return this._impl.isAsync();
    }
    isFile() {
      return this._impl.isFile();
    }
    isStream() {
      return this._impl.isStream();
    }
    next() {
      return this._impl.next();
    }
    throw(_) {
      return this._impl.throw(_);
    }
    return(_) {
      return this._impl.return(_);
    }
    cancel() {
      return this._impl.cancel();
    }
    reset(_) {
      return this._impl.reset(_), this._DOMStream = void 0, this._nodeStream = void 0, this;
    }
    open(_) {
      const H = this._impl.open(_);
      return isPromise(H) ? H.then(() => this) : this;
    }
    readRecordBatch(_) {
      return this._impl.isFile() ? this._impl.readRecordBatch(_) : null;
    }
    [Symbol.iterator]() {
      return this._impl[Symbol.iterator]();
    }
    [Symbol.asyncIterator]() {
      return this._impl[Symbol.asyncIterator]();
    }
    toDOMStream() {
      return streamAdapters.toDOMStream(this.isSync() ? { [Symbol.iterator]: () => this } : { [Symbol.asyncIterator]: () => this });
    }
    toNodeStream() {
      return streamAdapters.toNodeStream(this.isSync() ? { [Symbol.iterator]: () => this } : { [Symbol.asyncIterator]: () => this }, { objectMode: !0 });
    }
    /** @nocollapse */
    // @ts-ignore
    static throughNode(_) {
      throw new Error('"throughNode" not available in this environment');
    }
    /** @nocollapse */
    static throughDOM(_, H) {
      throw new Error('"throughDOM" not available in this environment');
    }
    /** @nocollapse */
    static from(_) {
      return _ instanceof RecordBatchReader ? _ : isArrowJSON(_) ? fromArrowJSON(_) : isFileHandle(_) ? fromFileHandle(_) : isPromise(_) ? __awaiter(this, void 0, void 0, function* () {
        return yield RecordBatchReader.from(yield _);
      }) : isFetchResponse(_) || isReadableDOMStream(_) || isReadableNodeStream(_) || isAsyncIterable(_) ? fromAsyncByteStream(new AsyncByteStream(_)) : fromByteStream(new ByteStream(_));
    }
    /** @nocollapse */
    static readAll(_) {
      return _ instanceof RecordBatchReader ? _.isSync() ? readAllSync(_) : readAllAsync(_) : isArrowJSON(_) || ArrayBuffer.isView(_) || isIterable(_) || isIteratorResult(_) ? readAllSync(_) : readAllAsync(_);
    }
  }
  class RecordBatchStreamReader extends RecordBatchReader {
    constructor(_) {
      super(_), this._impl = _;
    }
    readAll() {
      return [...this];
    }
    [Symbol.iterator]() {
      return this._impl[Symbol.iterator]();
    }
    [Symbol.asyncIterator]() {
      return __asyncGenerator(this, arguments, function* () {
        yield __await(yield* __asyncDelegator(__asyncValues(this[Symbol.iterator]())));
      });
    }
  }
  class AsyncRecordBatchStreamReader extends RecordBatchReader {
    constructor(_) {
      super(_), this._impl = _;
    }
    readAll() {
      return __awaiter(this, void 0, void 0, function* () {
        var _, H, V, q;
        const G = new Array();
        try {
          for (var J = !0, Z = __asyncValues(this), te; te = yield Z.next(), _ = te.done, !_; J = !0) {
            q = te.value, J = !1;
            const ne = q;
            G.push(ne);
          }
        } catch (ne) {
          H = { error: ne };
        } finally {
          try {
            !J && !_ && (V = Z.return) && (yield V.call(Z));
          } finally {
            if (H) throw H.error;
          }
        }
        return G;
      });
    }
    [Symbol.iterator]() {
      throw new Error("AsyncRecordBatchStreamReader is not Iterable");
    }
    [Symbol.asyncIterator]() {
      return this._impl[Symbol.asyncIterator]();
    }
  }
  class RecordBatchFileReader extends RecordBatchStreamReader {
    constructor(_) {
      super(_), this._impl = _;
    }
  }
  class AsyncRecordBatchFileReader extends AsyncRecordBatchStreamReader {
    constructor(_) {
      super(_), this._impl = _;
    }
  }
  class RecordBatchReaderImpl {
    get numDictionaries() {
      return this._dictionaryIndex;
    }
    get numRecordBatches() {
      return this._recordBatchIndex;
    }
    constructor(_ = /* @__PURE__ */ new Map()) {
      this.closed = !1, this.autoDestroy = !0, this._dictionaryIndex = 0, this._recordBatchIndex = 0, this.dictionaries = _;
    }
    isSync() {
      return !1;
    }
    isAsync() {
      return !1;
    }
    isFile() {
      return !1;
    }
    isStream() {
      return !1;
    }
    reset(_) {
      return this._dictionaryIndex = 0, this._recordBatchIndex = 0, this.schema = _, this.dictionaries = /* @__PURE__ */ new Map(), this;
    }
    _loadRecordBatch(_, H) {
      const V = this._loadVectors(_, H, this.schema.fields), q = makeData({ type: new Struct(this.schema.fields), length: _.length, children: V });
      return new RecordBatch$1(this.schema, q);
    }
    _loadDictionaryBatch(_, H) {
      const { id: V, isDelta: q } = _, { dictionaries: G, schema: J } = this, Z = G.get(V), te = J.dictionaries.get(V), ne = this._loadVectors(_.data, H, [te]);
      return (Z && q ? Z.concat(new Vector(ne)) : new Vector(ne)).memoize();
    }
    _loadVectors(_, H, V) {
      return new VectorLoader(H, _.nodes, _.buffers, this.dictionaries, this.schema.metadataVersion).visitMany(V);
    }
  }
  class RecordBatchStreamReaderImpl extends RecordBatchReaderImpl {
    constructor(_, H) {
      super(H), this._reader = isArrowJSON(_) ? new JSONMessageReader(this._handle = _) : new MessageReader(this._handle = _);
    }
    isSync() {
      return !0;
    }
    isStream() {
      return !0;
    }
    [Symbol.iterator]() {
      return this;
    }
    cancel() {
      !this.closed && (this.closed = !0) && (this.reset()._reader.return(), this._reader = null, this.dictionaries = null);
    }
    open(_) {
      return this.closed || (this.autoDestroy = shouldAutoDestroy(this, _), this.schema || (this.schema = this._reader.readSchema()) || this.cancel()), this;
    }
    throw(_) {
      return !this.closed && this.autoDestroy && (this.closed = !0) ? this.reset()._reader.throw(_) : ITERATOR_DONE;
    }
    return(_) {
      return !this.closed && this.autoDestroy && (this.closed = !0) ? this.reset()._reader.return(_) : ITERATOR_DONE;
    }
    next() {
      if (this.closed)
        return ITERATOR_DONE;
      let _;
      const { _reader: H } = this;
      for (; _ = this._readNextMessageAndValidate(); )
        if (_.isSchema())
          this.reset(_.header());
        else if (_.isRecordBatch()) {
          this._recordBatchIndex++;
          const V = _.header(), q = H.readMessageBody(_.bodyLength);
          return { done: !1, value: this._loadRecordBatch(V, q) };
        } else if (_.isDictionaryBatch()) {
          this._dictionaryIndex++;
          const V = _.header(), q = H.readMessageBody(_.bodyLength), G = this._loadDictionaryBatch(V, q);
          this.dictionaries.set(V.id, G);
        }
      return this.schema && this._recordBatchIndex === 0 ? (this._recordBatchIndex++, { done: !1, value: new _InternalEmptyPlaceholderRecordBatch(this.schema) }) : this.return();
    }
    _readNextMessageAndValidate(_) {
      return this._reader.readMessage(_);
    }
  }
  class AsyncRecordBatchStreamReaderImpl extends RecordBatchReaderImpl {
    constructor(_, H) {
      super(H), this._reader = new AsyncMessageReader(this._handle = _);
    }
    isAsync() {
      return !0;
    }
    isStream() {
      return !0;
    }
    [Symbol.asyncIterator]() {
      return this;
    }
    cancel() {
      return __awaiter(this, void 0, void 0, function* () {
        !this.closed && (this.closed = !0) && (yield this.reset()._reader.return(), this._reader = null, this.dictionaries = null);
      });
    }
    open(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return this.closed || (this.autoDestroy = shouldAutoDestroy(this, _), this.schema || (this.schema = yield this._reader.readSchema()) || (yield this.cancel())), this;
      });
    }
    throw(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return !this.closed && this.autoDestroy && (this.closed = !0) ? yield this.reset()._reader.throw(_) : ITERATOR_DONE;
      });
    }
    return(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return !this.closed && this.autoDestroy && (this.closed = !0) ? yield this.reset()._reader.return(_) : ITERATOR_DONE;
      });
    }
    next() {
      return __awaiter(this, void 0, void 0, function* () {
        if (this.closed)
          return ITERATOR_DONE;
        let _;
        const { _reader: H } = this;
        for (; _ = yield this._readNextMessageAndValidate(); )
          if (_.isSchema())
            yield this.reset(_.header());
          else if (_.isRecordBatch()) {
            this._recordBatchIndex++;
            const V = _.header(), q = yield H.readMessageBody(_.bodyLength);
            return { done: !1, value: this._loadRecordBatch(V, q) };
          } else if (_.isDictionaryBatch()) {
            this._dictionaryIndex++;
            const V = _.header(), q = yield H.readMessageBody(_.bodyLength), G = this._loadDictionaryBatch(V, q);
            this.dictionaries.set(V.id, G);
          }
        return this.schema && this._recordBatchIndex === 0 ? (this._recordBatchIndex++, { done: !1, value: new _InternalEmptyPlaceholderRecordBatch(this.schema) }) : yield this.return();
      });
    }
    _readNextMessageAndValidate(_) {
      return __awaiter(this, void 0, void 0, function* () {
        return yield this._reader.readMessage(_);
      });
    }
  }
  class RecordBatchFileReaderImpl extends RecordBatchStreamReaderImpl {
    get footer() {
      return this._footer;
    }
    get numDictionaries() {
      return this._footer ? this._footer.numDictionaries : 0;
    }
    get numRecordBatches() {
      return this._footer ? this._footer.numRecordBatches : 0;
    }
    constructor(_, H) {
      super(_ instanceof RandomAccessFile ? _ : new RandomAccessFile(_), H);
    }
    isSync() {
      return !0;
    }
    isFile() {
      return !0;
    }
    open(_) {
      if (!this.closed && !this._footer) {
        this.schema = (this._footer = this._readFooter()).schema;
        for (const H of this._footer.dictionaryBatches())
          H && this._readDictionaryBatch(this._dictionaryIndex++);
      }
      return super.open(_);
    }
    readRecordBatch(_) {
      var H;
      if (this.closed)
        return null;
      this._footer || this.open();
      const V = (H = this._footer) === null || H === void 0 ? void 0 : H.getRecordBatch(_);
      if (V && this._handle.seek(V.offset)) {
        const q = this._reader.readMessage(MessageHeader.RecordBatch);
        if (q != null && q.isRecordBatch()) {
          const G = q.header(), J = this._reader.readMessageBody(q.bodyLength);
          return this._loadRecordBatch(G, J);
        }
      }
      return null;
    }
    _readDictionaryBatch(_) {
      var H;
      const V = (H = this._footer) === null || H === void 0 ? void 0 : H.getDictionaryBatch(_);
      if (V && this._handle.seek(V.offset)) {
        const q = this._reader.readMessage(MessageHeader.DictionaryBatch);
        if (q != null && q.isDictionaryBatch()) {
          const G = q.header(), J = this._reader.readMessageBody(q.bodyLength), Z = this._loadDictionaryBatch(G, J);
          this.dictionaries.set(G.id, Z);
        }
      }
    }
    _readFooter() {
      const { _handle: _ } = this, H = _.size - magicAndPadding, V = _.readInt32(H), q = _.readAt(H - V, V);
      return Footer_.decode(q);
    }
    _readNextMessageAndValidate(_) {
      var H;
      if (this._footer || this.open(), this._footer && this._recordBatchIndex < this.numRecordBatches) {
        const V = (H = this._footer) === null || H === void 0 ? void 0 : H.getRecordBatch(this._recordBatchIndex);
        if (V && this._handle.seek(V.offset))
          return this._reader.readMessage(_);
      }
      return null;
    }
  }
  class AsyncRecordBatchFileReaderImpl extends AsyncRecordBatchStreamReaderImpl {
    get footer() {
      return this._footer;
    }
    get numDictionaries() {
      return this._footer ? this._footer.numDictionaries : 0;
    }
    get numRecordBatches() {
      return this._footer ? this._footer.numRecordBatches : 0;
    }
    constructor(_, ...H) {
      const V = typeof H[0] != "number" ? H.shift() : void 0, q = H[0] instanceof Map ? H.shift() : void 0;
      super(_ instanceof AsyncRandomAccessFile ? _ : new AsyncRandomAccessFile(_, V), q);
    }
    isFile() {
      return !0;
    }
    isAsync() {
      return !0;
    }
    open(_) {
      const H = Object.create(null, {
        open: { get: () => super.open }
      });
      return __awaiter(this, void 0, void 0, function* () {
        if (!this.closed && !this._footer) {
          this.schema = (this._footer = yield this._readFooter()).schema;
          for (const V of this._footer.dictionaryBatches())
            V && (yield this._readDictionaryBatch(this._dictionaryIndex++));
        }
        return yield H.open.call(this, _);
      });
    }
    readRecordBatch(_) {
      return __awaiter(this, void 0, void 0, function* () {
        var H;
        if (this.closed)
          return null;
        this._footer || (yield this.open());
        const V = (H = this._footer) === null || H === void 0 ? void 0 : H.getRecordBatch(_);
        if (V && (yield this._handle.seek(V.offset))) {
          const q = yield this._reader.readMessage(MessageHeader.RecordBatch);
          if (q != null && q.isRecordBatch()) {
            const G = q.header(), J = yield this._reader.readMessageBody(q.bodyLength);
            return this._loadRecordBatch(G, J);
          }
        }
        return null;
      });
    }
    _readDictionaryBatch(_) {
      return __awaiter(this, void 0, void 0, function* () {
        var H;
        const V = (H = this._footer) === null || H === void 0 ? void 0 : H.getDictionaryBatch(_);
        if (V && (yield this._handle.seek(V.offset))) {
          const q = yield this._reader.readMessage(MessageHeader.DictionaryBatch);
          if (q != null && q.isDictionaryBatch()) {
            const G = q.header(), J = yield this._reader.readMessageBody(q.bodyLength), Z = this._loadDictionaryBatch(G, J);
            this.dictionaries.set(G.id, Z);
          }
        }
      });
    }
    _readFooter() {
      return __awaiter(this, void 0, void 0, function* () {
        const { _handle: _ } = this;
        _._pending && (yield _._pending);
        const H = _.size - magicAndPadding, V = yield _.readInt32(H), q = yield _.readAt(H - V, V);
        return Footer_.decode(q);
      });
    }
    _readNextMessageAndValidate(_) {
      return __awaiter(this, void 0, void 0, function* () {
        if (this._footer || (yield this.open()), this._footer && this._recordBatchIndex < this.numRecordBatches) {
          const H = this._footer.getRecordBatch(this._recordBatchIndex);
          if (H && (yield this._handle.seek(H.offset)))
            return yield this._reader.readMessage(_);
        }
        return null;
      });
    }
  }
  class RecordBatchJSONReaderImpl extends RecordBatchStreamReaderImpl {
    constructor(_, H) {
      super(_, H);
    }
    _loadVectors(_, H, V) {
      return new JSONVectorLoader(H, _.nodes, _.buffers, this.dictionaries, this.schema.metadataVersion).visitMany(V);
    }
  }
  function shouldAutoDestroy(O, _) {
    return _ && typeof _.autoDestroy == "boolean" ? _.autoDestroy : O.autoDestroy;
  }
  function* readAllSync(O) {
    const _ = RecordBatchReader.from(O);
    try {
      if (!_.open({ autoDestroy: !1 }).closed)
        do
          yield _;
        while (!_.reset().open().closed);
    } finally {
      _.cancel();
    }
  }
  function readAllAsync(O) {
    return __asyncGenerator(this, arguments, function* () {
      const H = yield __await(RecordBatchReader.from(O));
      try {
        if (!(yield __await(H.open({ autoDestroy: !1 }))).closed)
          do
            yield yield __await(H);
          while (!(yield __await(H.reset().open())).closed);
      } finally {
        yield __await(H.cancel());
      }
    });
  }
  function fromArrowJSON(O) {
    return new RecordBatchStreamReader(new RecordBatchJSONReaderImpl(O));
  }
  function fromByteStream(O) {
    const _ = O.peek(magicLength + 7 & -8);
    return _ && _.byteLength >= 4 ? checkForMagicArrowString(_) ? new RecordBatchFileReader(new RecordBatchFileReaderImpl(O.read())) : new RecordBatchStreamReader(new RecordBatchStreamReaderImpl(O)) : new RecordBatchStreamReader(new RecordBatchStreamReaderImpl(function* () {
    }()));
  }
  function fromAsyncByteStream(O) {
    return __awaiter(this, void 0, void 0, function* () {
      const _ = yield O.peek(magicLength + 7 & -8);
      return _ && _.byteLength >= 4 ? checkForMagicArrowString(_) ? new RecordBatchFileReader(new RecordBatchFileReaderImpl(yield O.read())) : new AsyncRecordBatchStreamReader(new AsyncRecordBatchStreamReaderImpl(O)) : new AsyncRecordBatchStreamReader(new AsyncRecordBatchStreamReaderImpl(function() {
        return __asyncGenerator(this, arguments, function* () {
        });
      }()));
    });
  }
  function fromFileHandle(O) {
    return __awaiter(this, void 0, void 0, function* () {
      const { size: _ } = yield O.stat(), H = new AsyncRandomAccessFile(O, _);
      return _ >= magicX2AndPadding && checkForMagicArrowString(yield H.readAt(0, magicLength + 7 & -8)) ? new AsyncRecordBatchFileReader(new AsyncRecordBatchFileReaderImpl(H)) : new AsyncRecordBatchStreamReader(new AsyncRecordBatchStreamReaderImpl(H));
    });
  }
  function tableFromIPC(O) {
    const _ = RecordBatchReader.from(O);
    return isPromise(_) ? _.then((H) => tableFromIPC(H)) : _.isAsync() ? _.readAll().then((H) => new Table(H)) : new Table(_.readAll());
  }
  function _isPlaceholder(O) {
    return O != null && typeof O == "object" && O["@@functional/placeholder"] === !0;
  }
  function _curry1(O) {
    return function _(H) {
      return arguments.length === 0 || _isPlaceholder(H) ? _ : O.apply(this, arguments);
    };
  }
  function _curry2(O) {
    return function _(H, V) {
      switch (arguments.length) {
        case 0:
          return _;
        case 1:
          return _isPlaceholder(H) ? _ : _curry1(function(q) {
            return O(H, q);
          });
        default:
          return _isPlaceholder(H) && _isPlaceholder(V) ? _ : _isPlaceholder(H) ? _curry1(function(q) {
            return O(q, V);
          }) : _isPlaceholder(V) ? _curry1(function(q) {
            return O(H, q);
          }) : O(H, V);
      }
    };
  }
  function _arity(O, _) {
    switch (O) {
      case 0:
        return function() {
          return _.apply(this, arguments);
        };
      case 1:
        return function(H) {
          return _.apply(this, arguments);
        };
      case 2:
        return function(H, V) {
          return _.apply(this, arguments);
        };
      case 3:
        return function(H, V, q) {
          return _.apply(this, arguments);
        };
      case 4:
        return function(H, V, q, G) {
          return _.apply(this, arguments);
        };
      case 5:
        return function(H, V, q, G, J) {
          return _.apply(this, arguments);
        };
      case 6:
        return function(H, V, q, G, J, Z) {
          return _.apply(this, arguments);
        };
      case 7:
        return function(H, V, q, G, J, Z, te) {
          return _.apply(this, arguments);
        };
      case 8:
        return function(H, V, q, G, J, Z, te, ne) {
          return _.apply(this, arguments);
        };
      case 9:
        return function(H, V, q, G, J, Z, te, ne, ee) {
          return _.apply(this, arguments);
        };
      case 10:
        return function(H, V, q, G, J, Z, te, ne, ee, ae) {
          return _.apply(this, arguments);
        };
      default:
        throw new Error("First argument to _arity must be a non-negative integer no greater than ten");
    }
  }
  function _curry3(O) {
    return function _(H, V, q) {
      switch (arguments.length) {
        case 0:
          return _;
        case 1:
          return _isPlaceholder(H) ? _ : _curry2(function(G, J) {
            return O(H, G, J);
          });
        case 2:
          return _isPlaceholder(H) && _isPlaceholder(V) ? _ : _isPlaceholder(H) ? _curry2(function(G, J) {
            return O(G, V, J);
          }) : _isPlaceholder(V) ? _curry2(function(G, J) {
            return O(H, G, J);
          }) : _curry1(function(G) {
            return O(H, V, G);
          });
        default:
          return _isPlaceholder(H) && _isPlaceholder(V) && _isPlaceholder(q) ? _ : _isPlaceholder(H) && _isPlaceholder(V) ? _curry2(function(G, J) {
            return O(G, J, q);
          }) : _isPlaceholder(H) && _isPlaceholder(q) ? _curry2(function(G, J) {
            return O(G, V, J);
          }) : _isPlaceholder(V) && _isPlaceholder(q) ? _curry2(function(G, J) {
            return O(H, G, J);
          }) : _isPlaceholder(H) ? _curry1(function(G) {
            return O(G, V, q);
          }) : _isPlaceholder(V) ? _curry1(function(G) {
            return O(H, G, q);
          }) : _isPlaceholder(q) ? _curry1(function(G) {
            return O(H, V, G);
          }) : O(H, V, q);
      }
    };
  }
  const _isArray = Array.isArray || function O(_) {
    return _ != null && _.length >= 0 && Object.prototype.toString.call(_) === "[object Array]";
  };
  function _isString(O) {
    return Object.prototype.toString.call(O) === "[object String]";
  }
  var _isArrayLike = /* @__PURE__ */ _curry1(function O(_) {
    return _isArray(_) ? !0 : !_ || typeof _ != "object" || _isString(_) ? !1 : _.length === 0 ? !0 : _.length > 0 ? _.hasOwnProperty(0) && _.hasOwnProperty(_.length - 1) : !1;
  }), symIterator = typeof Symbol < "u" ? Symbol.iterator : "@@iterator";
  function _createReduce(O, _, H) {
    return function(q, G, J) {
      if (_isArrayLike(J))
        return O(q, G, J);
      if (J == null)
        return G;
      if (typeof J["fantasy-land/reduce"] == "function")
        return _(q, G, J, "fantasy-land/reduce");
      if (J[symIterator] != null)
        return H(q, G, J[symIterator]());
      if (typeof J.next == "function")
        return H(q, G, J);
      if (typeof J.reduce == "function")
        return _(q, G, J, "reduce");
      throw new TypeError("reduce: list must be array or iterable");
    };
  }
  function _xArrayReduce(O, _, H) {
    for (var V = 0, q = H.length; V < q; ) {
      if (_ = O["@@transducer/step"](_, H[V]), _ && _["@@transducer/reduced"]) {
        _ = _["@@transducer/value"];
        break;
      }
      V += 1;
    }
    return O["@@transducer/result"](_);
  }
  var bind = /* @__PURE__ */ _curry2(function O(_, H) {
    return _arity(_.length, function() {
      return _.apply(H, arguments);
    });
  });
  function _xIterableReduce(O, _, H) {
    for (var V = H.next(); !V.done; ) {
      if (_ = O["@@transducer/step"](_, V.value), _ && _["@@transducer/reduced"]) {
        _ = _["@@transducer/value"];
        break;
      }
      V = H.next();
    }
    return O["@@transducer/result"](_);
  }
  function _xMethodReduce(O, _, H, V) {
    return O["@@transducer/result"](H[V](bind(O["@@transducer/step"], O), _));
  }
  var _xReduce = /* @__PURE__ */ _createReduce(_xArrayReduce, _xMethodReduce, _xIterableReduce), XWrap = /* @__PURE__ */ function() {
    function O(_) {
      this.f = _;
    }
    return O.prototype["@@transducer/init"] = function() {
      throw new Error("init not implemented on XWrap");
    }, O.prototype["@@transducer/result"] = function(_) {
      return _;
    }, O.prototype["@@transducer/step"] = function(_, H) {
      return this.f(_, H);
    }, O;
  }();
  function _xwrap(O) {
    return new XWrap(O);
  }
  var reduce = /* @__PURE__ */ _curry3(function(O, _, H) {
    return _xReduce(typeof O == "function" ? _xwrap(O) : O, _, H);
  });
  function _pipe(O, _) {
    return function() {
      return _.call(this, O.apply(this, arguments));
    };
  }
  function _checkForMethod(O, _) {
    return function() {
      var H = arguments.length;
      if (H === 0)
        return _();
      var V = arguments[H - 1];
      return _isArray(V) || typeof V[O] != "function" ? _.apply(this, arguments) : V[O].apply(V, Array.prototype.slice.call(arguments, 0, H - 1));
    };
  }
  var slice = /* @__PURE__ */ _curry3(/* @__PURE__ */ _checkForMethod("slice", function O(_, H, V) {
    return Array.prototype.slice.call(V, _, H);
  })), tail = /* @__PURE__ */ _curry1(/* @__PURE__ */ _checkForMethod("tail", /* @__PURE__ */ slice(1, 1 / 0)));
  function pipe() {
    if (arguments.length === 0)
      throw new Error("pipe requires at least one argument");
    return _arity(arguments[0].length, reduce(_pipe, arguments[0], tail(arguments)));
  }
  var reverse = /* @__PURE__ */ _curry1(function O(_) {
    return _isString(_) ? _.split("").reverse().join("") : Array.prototype.slice.call(_, 0).reverse();
  });
  function compose() {
    if (arguments.length === 0)
      throw new Error("compose requires at least one argument");
    return pipe.apply(this, reverse(arguments));
  }
  var pick = /* @__PURE__ */ _curry2(function O(_, H) {
    for (var V = {}, q = 0; q < _.length; )
      _[q] in H && (V[_[q]] = H[_[q]]), q += 1;
    return V;
  });
  class Coerce {
    constructor(_) {
      mt(this, "fieldNames", []);
      mt(this, "coerceRow", (_) => pick(this.fieldNames, _));
      for (const H of _)
        DataType.isTimestamp(H) ? this.composeTimestampField(H) : DataType.isInt(H) ? this.composeIntField(H) : DataType.isFloat(H) && this.composeFloatField(H), this.fieldNames.push(H.name);
    }
    composeIntField(_) {
      _.type.bitWidth >= 64 && (this.coerceRow = compose((H) => H, this.coerceRow));
    }
    composeFloatField(_) {
      this.coerceRow = compose((H) => {
        const V = H[_.name];
        return V && (H[_.name] = parseFloat(V)), H;
      }, this.coerceRow);
    }
    composeTimestampField(_) {
      // only handle UTC or local timetamps
      (_.type.timezone === null || _.type.timezone === "UTC") && (this.coerceRow = compose((H) => {
        const V = H[_.name];
        return V && (H[_.name] = new Date(V).toISOString()), H;
      }, this.coerceRow));
    }
  }
  const dataTypeToString = (O) => {
    switch (O.typeId) {
      case Type.Null:
        return "null";
      case Type.Int:
        return "integer";
      case Type.Float:
        return O.precision === Precision.DOUBLE ? "double" : "float";
      case Type.Binary:
      case Type.FixedSizeBinary:
        return "binary";
      case Type.Utf8:
        return "string";
      case Type.Bool:
        return "boolean";
      case Type.Decimal:
        return "decimal";
      case Type.Date:
        return "date";
      case Type.Time:
        return "time";
      case Type.Timestamp:
        return "timestamp";
      case Type.Interval:
        return "interval";
      case Type.List:
      case Type.FixedSizeList:
        return `[${dataTypeToString(O.children[0].type)}]`;
      case Type.Struct:
        return "struct";
      case Type.Union:
        return O.children.map((_) => dataTypeToString(_.type)).filter((_) => _ !== "null").join(" or ");
      case Type.Map:
        return "map";
      case Type.Dictionary:
        return DataType.isUtf8(O.valueType) ? "category" : dataTypeToString(O.valueType);
      default:
        return O.toString();
    }
  }, extractArrowSchema = (O) => O.fields.map((_) => ({
    name: _.name,
    type: dataTypeToString(_.type)
  })), apiResponseToArrow = (O) => {
    const _ = [], H = tableFromIPC(O), V = new Coerce(H.schema.fields);
    for (const q of H)
      _.push(V.coerceRow(q));
    return {
      schema: extractArrowSchema(H.schema),
      data: _,
      // For now we want to disable sanddance for datasets containing int64 columns
      containsBigInt: H.schema.fields.some(
        (q) => q.type.bitWidth && q.type.bitWidth >= 64
      )
    };
  }, arrowUtils = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    apiResponseToArrow
  }, Symbol.toStringTag, { value: "Module" })), tailwind = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null
  }, Symbol.toStringTag, { value: "Module" }));
  /*!
   * Built by Revolist OU ❤️
   */
  const HEADER_SLOT = "header", FOOTER_SLOT = "footer", CONTENT_SLOT = "content", DATA_SLOT = "data";
  function getLastCell(O, _) {
    const H = O.viewports[O.colType].store.get("realCount"), V = O.viewports[_].store.get("realCount");
    return {
      x: H,
      y: V
    };
  }
  function viewportDataPartition(O, _, H, V) {
    return {
      colData: O.colStore,
      viewportCol: O.viewports[O.colType].store,
      viewportRow: O.viewports[_].store,
      // lastCell is the last real coordinate + 1
      lastCell: getLastCell(O, _),
      slot: H,
      type: _,
      canDrag: !V,
      position: O.position,
      dataStore: O.rowStores[_].store,
      dimensionCol: O.dimensions[O.colType].store,
      dimensionRow: O.dimensions[_].store,
      style: V ? { height: `${O.dimensions[_].store.get("realSize")}px` } : void 0
    };
  }
  /*!
   * Built by Revolist OU ❤️
   */
  function isTouch(O) {
    return !!O.touches;
  }
  function verifyTouchTarget(O, _) {
    return !(_ && O && !(O.target instanceof Element && O.target.classList.contains(_)));
  }
  function getPropertyFromEvent(O, _, H) {
    if (isTouch(O)) {
      if (O.touches.length > 0) {
        const V = O.touches[0];
        return verifyTouchTarget(V, H) ? V[_] || 0 : null;
      }
      return null;
    }
    return O[_] || 0;
  }
  /*!
   * Built by Revolist OU ❤️
   */
  class ThemeCompact {
    constructor() {
      this.defaultRowSize = 32;
    }
  }
  class ThemeDefault {
    constructor() {
      this.defaultRowSize = 27;
    }
  }
  class ThemeMaterial {
    constructor() {
      this.defaultRowSize = 42;
    }
  }
  const DEFAULT_THEME = "default", allowedThemes = [
    DEFAULT_THEME,
    "material",
    "compact",
    "darkMaterial",
    "darkCompact"
  ];
  class ThemeService {
    get theme() {
      return this.currentTheme;
    }
    get rowSize() {
      return this.customRowSize || this.currentTheme.defaultRowSize;
    }
    set rowSize(_) {
      this.customRowSize = _;
    }
    constructor(_) {
      this.customRowSize = 0, this.customRowSize = _.rowSize, this.register("default");
    }
    register(_) {
      switch (getTheme(_)) {
        case "material":
        case "darkMaterial":
          this.currentTheme = new ThemeMaterial();
          break;
        case "compact":
        case "darkCompact":
          this.currentTheme = new ThemeCompact();
          break;
        default:
          this.currentTheme = new ThemeDefault();
          break;
      }
    }
  }
  function getTheme(O) {
    return O && allowedThemes.indexOf(O) > -1 ? O : DEFAULT_THEME;
  }
  class ColumnDataProvider {
    get order() {
      const _ = {}, H = this.sorting;
      return H && Object.keys(H).forEach((V) => {
        _[V] = H[V].order;
      }), _;
    }
    get stores() {
      return this.dataSources;
    }
    constructor() {
      this.sorting = null, this.collection = null, this.dataSources = columnTypes.reduce((_, H) => (_[H] = new DataStore(H), _), {});
    }
    column(_, H = "rgCol") {
      return this.getColumn(_, H);
    }
    getColumn(_, H) {
      return getSourceItem(this.dataSources[H].store, _);
    }
    getRawColumns() {
      return reduce$1(this.dataSources, (_, H, V) => (_[V] = H.store.get("source"), _), {
        rgCol: [],
        colPinStart: [],
        colPinEnd: []
      });
    }
    getColumns(_ = "all") {
      const H = this.getRawColumns();
      return _ !== "all" ? H[_] : columnTypes.reduce((V, q) => [...V, ...H[q]], []);
    }
    getColumnIndexByProp(_, H) {
      return getSourceItemVirtualIndexByProp(this.dataSources[H].store, _);
    }
    getColumnByProp(_) {
      var H;
      return (H = this.collection) === null || H === void 0 ? void 0 : H.columnByProp[_];
    }
    refreshByType(_) {
      this.dataSources[_].refresh();
    }
    /**
     * Main method to set columns
     */
    setColumns(_) {
      return columnTypes.forEach((H) => {
        this.dataSources[H].updateData(_.columns[H], {
          // max depth level
          depth: _.maxLevel,
          // groups
          groups: _.columnGrouping[H].reduce((V, q) => (V[q.level] || (V[q.level] = []), V[q.level].push(q), V), {})
        });
      }), this.sorting = _.sort, this.collection = _, _;
    }
    /**
     * Used in plugins
     * Modify columns in store
     */
    updateColumns(_) {
      const H = _.reduce((q, G) => {
        const J = getColumnType(G);
        return q[J] || (q[J] = {}), q[J][G.prop] = G, q;
      }, {}), V = {};
      for (const q in H) {
        if (!H.hasOwnProperty(q))
          continue;
        const G = q, J = H[G], Z = this.dataSources[G].store.get("source");
        V[G] = {};
        for (let te = 0; te < Z.length; te++) {
          const ne = Z[te], ee = J == null ? void 0 : J[ne.prop];
          ee && (V[G][te] = ee);
        }
      }
      for (const q in V) {
        if (!V.hasOwnProperty(q))
          continue;
        const G = q;
        setSourceByPhysicalIndex(this.dataSources[G].store, V[G] || {});
      }
    }
    updateColumn(_, H) {
      const V = getColumnType(_);
      setSourceByVirtualIndex(this.dataSources[V].store, { [H]: _ });
    }
    updateColumnSorting(_, H, V, q) {
      return q || this.clearSorting(), _.order = V, this.sorting || (this.sorting = {}), this.sorting[_.prop] = _, this.updateColumn(_, H), _;
    }
    clearSorting() {
      const _ = reduce$1(this.sorting, (H, V) => {
        const q = getColumnType(V);
        return H[q] = !0, H;
      }, {});
      forEach(_, (H, V) => {
        const q = this.dataSources[V].store.get("source");
        forEach(q, (G) => G.order = void 0), this.dataSources[V].setData({ source: [...q] });
      }), this.sorting = {};
    }
  }
  class DataProvider {
    constructor(_) {
      this.dimensionProvider = _, this.stores = reduce$1(rowTypes, (H, V) => (H[V] = new DataStore(V), H), {});
    }
    setData(_, H = "rgRow", V = !1, q, G = !1) {
      this.stores[H].updateData([..._], q, G);
      const J = H !== "rgRow" || V;
      return this.dimensionProvider.setData(_.length, H, J), _;
    }
    getModel(_, H = "rgRow") {
      const V = this.stores[H].store;
      return getSourceItem(V, _);
    }
    changeOrder({ rowType: _ = "rgRow", from: H, to: V }) {
      const q = this.stores[_], G = [...q.store.get("proxyItems")], J = q.store.get("items"), Z = G.splice(
        G.indexOf(J[H]),
        // get index in proxy
        1
      );
      G.splice(
        G.indexOf(J[V]),
        // get index in proxy
        0,
        ...Z
      ), q.setData({
        proxyItems: G
      });
      const te = q.store.get("items");
      this.dimensionProvider.updateSizesPositionByNewDataIndexes(_, te, J);
    }
    setCellData({ type: _, rowIndex: H, prop: V, val: q }, G = !0) {
      const J = this.getModel(H, _);
      J[V] = q, this.stores[_].setSourceData({ [H]: J }, G);
    }
    setRangeData(_, H) {
      const V = {};
      for (let q in _) {
        const G = V[q] = getSourceItem(this.stores[H].store, parseInt(q, 10));
        if (G)
          for (let J in _[q])
            G[J] = _[q][J];
      }
      this.stores[H].setSourceData(V);
    }
    refresh(_ = "all") {
      isRowType(_) && this.refreshItems(_), rowTypes.forEach((H) => this.refreshItems(H));
    }
    refreshItems(_ = "rgRow") {
      const H = this.stores[_].store.get("items");
      this.stores[_].setData({ items: [...H] });
    }
    setGrouping({ depth: _ }, H = "rgRow") {
      this.stores[H].setData({ groupingDepth: _ });
    }
    setTrimmed(_, H = "rgRow") {
      const V = this.stores[H];
      V.addTrimmed(_), this.dimensionProvider.setTrimmed(_, H), H === "rgRow" && this.dimensionProvider.setData(getVisibleSourceItem(V.store).length, H);
    }
  }
  class DimensionProvider {
    constructor(_, H) {
      this.viewports = _;
      const V = debounce$1((q) => H.realSizeChanged(q), RESIZE_INTERVAL);
      this.stores = reduce$1([...rowTypes, ...columnTypes], (q, G) => (q[G] = new DimensionStore(G), q[G].store.onChange("realSize", () => V(G)), q), {});
    }
    /**
     * Clear old sizes from dimension and viewports
     * @param type - dimension type
     * @param count - count of items
     */
    clearSize(_, H) {
      this.stores[_].drop(), this.viewports.stores[_].setOriginalSizes(this.stores[_].store.get("originItemSize")), this.setItemCount(H, _);
    }
    /**
     * Apply new custom sizes to dimension and view port
     * @param type - dimension type
     * @param sizes - new custom sizes
     * @param keepOld - keep old sizes merge new with old
     */
    setCustomSizes(_, H, V = !1) {
      let q = H;
      if (V) {
        const G = this.stores[_].store.get("sizes");
        q = Object.assign(Object.assign({}, G), H);
      }
      this.stores[_].setDimensionSize(q), this.setViewPortCoordinate({
        type: _,
        force: !0
      });
    }
    setItemCount(_, H) {
      this.viewports.stores[H].setViewport({ realCount: _ }), this.stores[H].setStore({ count: _ });
    }
    /**
     * Apply trimmed items
     * @param trimmed - trimmed items
     * @param type
     */
    setTrimmed(_, H) {
      const V = gatherTrimmedItems(_);
      this.stores[H].setStore({ trimmed: V }), this.setViewPortCoordinate({
        type: H,
        force: !0
      });
    }
    /**
     * Sets dimension data and viewport coordinate
     * @param itemCount
     * @param type - dimension type
     * @param noVirtual - disable virtual data
     */
    setData(_, H, V = !1) {
      if (this.setItemCount(_, H), V) {
        const q = this.stores[H].getCurrentState();
        this.viewports.stores[H].setViewport({
          virtualSize: q.realSize
        });
      }
      this.setViewPortCoordinate({
        type: H
      });
    }
    /**
     * Applies new columns to the dimension provider
     * @param columns - new columns data
     * @param disableVirtualX - disable virtual data for X axis
     */
    applyNewColumns(_, H, V = !1) {
      for (let q of columnTypes) {
        V || this.stores[q].drop();
        const G = _[q], J = q !== "rgCol" || H;
        this.stores[q].setStore({ count: G.length });
        const Z = getColumnSizes(G);
        this.stores[q].setDimensionSize(Z);
        const te = {
          // This triggers drop on realCount change
          realCount: G.length
        };
        J && (te.virtualSize = this.stores[q].getCurrentState().realSize), this.viewports.stores[q].setViewport(te), this.setViewPortCoordinate({
          type: q
        });
      }
    }
    /**
     * Gets the full size of the grid by summing up the sizes of all dimensions
     * Goes through all dimensions columnTypes (x) and rowTypes (y) and sums up their sizes
     */
    getFullSize() {
      var _, H;
      let V = 0, q = 0;
      for (let G of columnTypes)
        V += ((_ = this.stores[G]) === null || _ === void 0 ? void 0 : _.store.get("realSize")) || 0;
      for (let G of rowTypes)
        q += ((H = this.stores[G]) === null || H === void 0 ? void 0 : H.store.get("realSize")) || 0;
      return { y: q, x: V };
    }
    setViewPortCoordinate({ type: _, coordinate: H = this.viewports.stores[_].lastCoordinate, force: V = !1 }) {
      const q = this.stores[_].getCurrentState();
      this.viewports.stores[_].setViewPortCoordinate(H, q, V);
    }
    getViewPortPos(_) {
      const H = this.stores[_.dimension].getCurrentState();
      return getItemByIndex(H, _.coordinate).start;
    }
    setSettings(_, H) {
      let V = [];
      switch (H) {
        case "rgCol":
          V = columnTypes;
          break;
        case "rgRow":
          V = rowTypes;
          break;
      }
      for (let q of V)
        this.stores[q].setStore(_);
    }
    updateSizesPositionByNewDataIndexes(_, H, V = []) {
      this.stores[_].updateSizesPositionByIndexes(H, V), this.setViewPortCoordinate({
        type: _,
        force: !0
      });
    }
  }
  class ViewportProvider {
    constructor() {
      this.stores = reduce$1([...rowTypes, ...columnTypes], (_, H) => (_[H] = new ViewportStore(H), _), {});
    }
    setViewport(_, H) {
      this.stores[_].setViewport(H);
    }
  }
  function gatherColumnData(O) {
    const _ = O.dimensions[O.colType].store, H = _.get("realSize"), V = {
      contentWidth: H,
      class: O.colType,
      contentHeight: O.contentHeight,
      key: O.colType,
      colType: O.colType,
      onResizeviewport: O.onResizeviewport,
      // set viewport size to real size
      style: O.fixWidth ? { minWidth: `${H}px` } : void 0
    }, q = {
      colData: getVisibleSourceItem(O.colStore),
      dimensionCol: _,
      type: O.colType,
      groups: O.colStore.get("groups"),
      groupingDepth: O.colStore.get("groupingDepth"),
      resizeHandler: O.colType === "colPinEnd" ? ["l"] : void 0,
      onHeaderresize: O.onHeaderresize
    };
    return {
      prop: V,
      type: O.colType,
      position: O.position,
      headerProp: q,
      viewportCol: O.viewports[O.colType].store
    };
  }
  class ViewportService {
    constructor(_, H) {
      var V, q;
      this.config = _, (V = this.config.selectionStoreConnector) === null || V === void 0 || V.beforeUpdate();
      const G = [];
      let J = 0;
      columnTypes.forEach((Z) => {
        const te = _.columnProvider.stores[Z].store;
        if (!te.get("items").length)
          return;
        const ne = {
          colType: Z,
          position: { x: J, y: 1 },
          contentHeight: H,
          // only central column has dynamic width
          fixWidth: Z !== "rgCol",
          viewports: _.viewportProvider.stores,
          dimensions: _.dimensionProvider.stores,
          rowStores: _.dataProvider.stores,
          colStore: te,
          onHeaderresize: (se) => this.onColumnResize(Z, se, te)
        };
        Z === "rgCol" && (ne.onResizeviewport = (se) => {
          var ce;
          const le = {
            clientSize: se.detail.size
          };
          (se.detail.dimension === "rgRow" && !_.disableVirtualY || se.detail.dimension === "rgCol" && !_.disableVirtualX) && (le.virtualSize = se.detail.size), (ce = _.viewportProvider) === null || ce === void 0 || ce.setViewport(se.detail.dimension, le);
        });
        const ee = gatherColumnData(ne), ae = this.registerCol(ee.position.x, Z), oe = this.dataViewPort(ne).reduce((se, ce) => {
          const le = this.registerSegment(ce.position);
          le.setLastCell(ce.lastCell);
          const ue = this.registerRow(ce.position.y, ce.type), de = Object.assign(Object.assign({ colType: Z }, ce), { rowSelectionStore: ue, selectionStore: le.store, ref: (fe) => _.selectionStoreConnector.registerSection(fe), onSetrange: (fe) => {
            le.setRangeArea(fe.detail);
          }, onSettemprange: (fe) => le.setTempArea(fe.detail), onFocuscell: (fe) => {
            le.clearFocus(), _.selectionStoreConnector.focus(le, fe.detail);
          } });
          return se.push(de), se;
        }, []);
        G.push(Object.assign(Object.assign({}, ee), {
          columnSelectionStore: ae,
          dataPorts: oe
        })), J++;
      }), this.columns = G, (q = this.config.scrollingService) === null || q === void 0 || q.unregister();
    }
    onColumnResize(_, { detail: H }, V) {
      var q;
      (q = this.config.dimensionProvider) === null || q === void 0 || q.setCustomSizes(_, H, !0);
      const G = {};
      for (const [J, Z] of Object.entries(H || {})) {
        const te = parseInt(J, 10), ne = getSourceItem(V, te);
        ne && (G[te] = Object.assign(Object.assign({}, ne), { size: Z }));
      }
      this.config.resize(G);
    }
    /** register selection store for Segment */
    registerSegment(_) {
      return this.config.selectionStoreConnector.register(_);
    }
    /** register selection store for Row */
    registerRow(_, H) {
      return this.config.selectionStoreConnector.registerRow(_, H).store;
    }
    /** register selection store for Column */
    registerCol(_, H) {
      return this.config.selectionStoreConnector.registerColumn(_, H).store;
    }
    /** Collect Row data */
    dataViewPort(_) {
      const H = {
        rowPinStart: HEADER_SLOT,
        rgRow: CONTENT_SLOT,
        rowPinEnd: FOOTER_SLOT
      };
      let V = 0;
      return rowTypes.reduce((q, G) => {
        const J = _.viewports[G].store.get("realCount") || G === "rgRow", Z = Object.assign(Object.assign({}, _), { position: Object.assign(Object.assign({}, _.position), { y: J ? V : EMPTY_INDEX }) }), te = viewportDataPartition(Z, G, H[G], G !== "rgRow");
        return q.push(te), J && V++, q;
      }, []);
    }
    scrollToCell(_) {
      for (let H in _) {
        const V = _[H];
        typeof V == "number" && this.config.scrollingService.proxyScroll({
          dimension: H === "x" ? "rgCol" : "rgRow",
          coordinate: V
        });
      }
    }
    /**
     * Clear current grid focus
     */
    clearFocused() {
      this.config.selectionStoreConnector.clearAll();
    }
    clearEdit() {
      this.config.selectionStoreConnector.setEdit(!1);
    }
    /**
     * Collect focused element data
     */
    getFocused() {
      const _ = this.config.selectionStoreConnector.focusedStore;
      if (!_)
        return null;
      const H = this.config.selectionStoreConnector.storesXToType[_.position.x], V = this.config.columnProvider.getColumn(_.cell.x, H), q = this.config.selectionStoreConnector.storesYToType[_.position.y], G = this.config.dataProvider.getModel(_.cell.y, q);
      return {
        column: V,
        model: G,
        cell: _.cell,
        colType: H,
        rowType: q
      };
    }
    getStoreCoordinateByType(_, H) {
      const V = this.config.selectionStoreConnector.storesByType;
      if (!(typeof V[_] > "u" || typeof V[H] > "u"))
        return {
          x: V[_],
          y: V[H]
        };
    }
    setFocus(_, H, V, q) {
      var G;
      const J = this.getStoreCoordinateByType(_, H);
      J && ((G = this.config.selectionStoreConnector) === null || G === void 0 || G.focusByCell(J, V, q));
    }
    getSelectedRange() {
      return this.config.selectionStoreConnector.selectedRange;
    }
    setEdit(_, H, V, q) {
      var G;
      const J = this.getStoreCoordinateByType(V, q);
      J && ((G = this.config.selectionStoreConnector) === null || G === void 0 || G.setEditByCell(J, { x: H, y: _ }));
    }
  }
  class GridScrollingService {
    constructor(_) {
      this.setViewport = _, this.elements = {};
    }
    async proxyScroll(_, H) {
      var V;
      let q, G = _;
      for (let Z in this.elements)
        if (!(_.dimension === "rgCol" && Z === "headerRow"))
          if (this.isPinnedColumn(H) && _.dimension === "rgCol") {
            if (Z === H || !_.delta)
              continue;
            for (let te of this.elements[Z])
              te.changeScroll && (q = te.changeScroll(_));
          } else
            for (let te of this.elements[Z])
              await ((V = te.setScroll) === null || V === void 0 ? void 0 : V.call(te, _));
      const J = await q;
      J && (G = J), this.setViewport(G);
    }
    /**
     * Silent scroll update for mobile devices when we have negative scroll top
     */
    async scrollSilentService(_, H) {
      var V;
      for (let q in this.elements)
        if (q !== H && columnTypes.includes(H) && (q === "headerRow" || columnTypes.includes(q))) {
          for (let G of this.elements[q])
            await ((V = G.changeScroll) === null || V === void 0 ? void 0 : V.call(G, _, !0));
          continue;
        }
    }
    isPinnedColumn(_) {
      return !!_ && ["colPinStart", "colPinEnd"].indexOf(_) > -1;
    }
    registerElements(_) {
      this.elements = _;
    }
    /**
     * Register new element for farther scroll support
     * @param el - can be null if holder removed
     * @param key - element key
     */
    registerElement(_, H) {
      this.elements[H] || (this.elements[H] = []), _ ? this.elements[H].push(_) : this.elements[H] && delete this.elements[H];
    }
    unregister() {
      this.elements = {};
    }
  }
  class OrdererService {
    constructor() {
      this.parentY = 0;
    }
    start(_, { pos: H, text: V, event: q }) {
      var G;
      const { top: J } = _.getBoundingClientRect();
      this.parentY = J, this.text && (this.text.innerText = V), this.move(H), this.moveTip({ x: q.x, y: q.y }), (G = this.el) === null || G === void 0 || G.classList.remove("hidden");
    }
    end() {
      var _;
      (_ = this.el) === null || _ === void 0 || _.classList.add("hidden");
    }
    move(_) {
      this.moveElement(_.end - this.parentY);
    }
    moveTip({ x: _, y: H }) {
      this.draggable && (this.draggable.style.left = `${_}px`, this.draggable.style.top = `${H}px`);
    }
    moveElement(_) {
      this.rgRow && (this.rgRow.style.transform = `translateY(${_}px)`);
    }
  }
  const OrderRenderer = ({ ref: O }) => {
    const _ = new OrdererService();
    return O(_), h(
      "div",
      { class: "draggable-wrapper hidden", ref: (H) => _.el = H },
      h(
        "div",
        { class: "draggable", ref: (H) => _.draggable = H },
        h("span", { class: "revo-alt-icon" }),
        h("span", { ref: (H) => _.text = H })
      ),
      h("div", { class: "drag-position", ref: (H) => _.rgRow = H })
    );
  }, rowDefinitionByType = (O = []) => {
    const _ = {};
    for (const H of O) {
      let V = _[H.type];
      V || (V = _[H.type] = {}), H.size && (V.sizes || (V.sizes = {}), V.sizes[H.index] = H.size);
    }
    return _;
  }, rowDefinitionRemoveByType = (O = []) => {
    const _ = {};
    for (const H of O) {
      let V = _[H.type];
      V || (V = _[H.type] = []), H.size && V.push(H.index);
    }
    return _;
  };
  function isMobileDevice() {
    return /Mobi/i.test(navigator.userAgent) || /Android/i.test(navigator.userAgent) || navigator.maxTouchPoints > 0;
  }
  class WCAGPlugin extends BasePlugin {
    constructor(_, H) {
      super(_, H), _.setAttribute("dir", "ltr"), _.setAttribute("role", "treegrid"), _.setAttribute("aria-keyshortcuts", "Enter"), _.setAttribute("aria-multiselectable", "true"), _.setAttribute("tabindex", "0"), this.addEventListener("beforecolumnsset", ({ detail: V }) => {
        const q = [
          ...V.columns.colPinStart,
          ...V.columns.rgCol,
          ...V.columns.colPinEnd
        ];
        _.setAttribute("aria-colcount", `${q.length}`), q.forEach((G, J) => {
          const { columnProperties: Z, cellProperties: te } = G;
          G.columnProperties = (...ne) => {
            const ee = (Z == null ? void 0 : Z(...ne)) || {};
            return ee.role = "columnheader", ee["aria-colindex"] = J, ee;
          }, G.cellProperties = (...ne) => {
            const ee = {
              role: "gridcell",
              "aria-colindex": J,
              "aria-rowindex": ne[0].rowIndex,
              tabindex: -1
            }, ae = (te == null ? void 0 : te(...ne)) || {};
            return Object.assign(Object.assign({}, ee), ae);
          };
        });
      }), this.addEventListener("beforesourceset", ({ detail: V }) => {
        _.setAttribute("aria-rowcount", `${V.source.length}`);
      }), this.addEventListener("beforerowrender", ({ detail: V }) => {
        V.node.$attrs$ = Object.assign(Object.assign({}, V.node.$attrs$), { role: "row", "aria-rowindex": V.item.itemIndex });
      }), this.addEventListener("afterfocus", async (V) => {
        if (V.defaultPrevented)
          return;
        const q = this.revogrid.querySelector(`revogr-data[type="${V.detail.rowType}"][col-type="${V.detail.colType}"] [data-rgrow="${V.detail.rowIndex}"][data-rgcol="${V.detail.colIndex}"]`);
        q instanceof HTMLElement && q.focus();
      });
    }
  }
  class PluginService {
    constructor() {
      this.internalPlugins = [];
    }
    /**
     * Get all plugins
     */
    get() {
      return [...this.internalPlugins];
    }
    /**
     * Add plugin to collection
     */
    add(_) {
      this.internalPlugins.push(_);
    }
    /**
     * Add user plugins and create
     */
    addUserPluginsAndCreate(_, H = [], V, q) {
      if (!q)
        return;
      ((V == null ? void 0 : V.filter((J) => !H.some((Z) => Z === J))) || []).forEach((J) => {
        var Z, te;
        const ne = this.internalPlugins.findIndex((ee) => ee instanceof J);
        ne !== -1 && ((te = (Z = this.internalPlugins[ne]).destroy) === null || te === void 0 || te.call(Z), this.internalPlugins.splice(ne, 1));
      }), H == null || H.forEach((J) => {
        this.internalPlugins.find((te) => te instanceof J) || this.add(new J(_, q));
      });
    }
    /**
     * Get plugin by class
     */
    getByClass(_) {
      return this.internalPlugins.find((H) => H instanceof _);
    }
    /**
     * Remove plugin
     */
    remove(_) {
      var H, V;
      const q = this.internalPlugins.indexOf(_);
      q > -1 && ((V = (H = this.internalPlugins[q]).destroy) === null || V === void 0 || V.call(H), this.internalPlugins.splice(q, 1));
    }
    /**
     * Remove all plugins
     */
    destroy() {
      this.internalPlugins.forEach((_) => {
        var H;
        return (H = _.destroy) === null || H === void 0 ? void 0 : H.call(_);
      }), this.internalPlugins = [];
    }
  }
  const revoGridStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revo-grid[theme=default],revo-grid:not([theme]){border:1px solid #cecece;font-size:12px}revo-grid[theme=default] .rowHeaders revogr-header,revo-grid:not([theme]) .rowHeaders revogr-header{box-shadow:-1px 0 0 0 #cecece inset}revo-grid[theme=default] revogr-header,revo-grid:not([theme]) revogr-header{text-align:center;line-height:30px;background-color:#f8f9fa}revo-grid[theme=default] revogr-header .group-rgRow,revo-grid:not([theme]) revogr-header .group-rgRow{box-shadow:none}revo-grid[theme=default] revogr-header .header-rgRow,revo-grid[theme=default] revogr-header .group-rgRow,revo-grid:not([theme]) revogr-header .header-rgRow,revo-grid:not([theme]) revogr-header .group-rgRow{text-transform:uppercase;font-size:12px;color:#000}revo-grid[theme=default] revogr-header .header-rgRow,revo-grid:not([theme]) revogr-header .header-rgRow{height:30px;box-shadow:0 -1px 0 0 #cecece inset}revo-grid[theme=default] revogr-header .rgHeaderCell,revo-grid:not([theme]) revogr-header .rgHeaderCell{box-shadow:-1px 0 0 0 #cecece inset, 0 -1px 0 0 #cecece, 0 -1px 0 0 #cecece inset}revo-grid[theme=default] revogr-header .rgHeaderCell.focused-cell,revo-grid:not([theme]) revogr-header .rgHeaderCell.focused-cell{background:rgba(233, 234, 237, 0.5)}revo-grid[theme=default] .rowHeaders,revo-grid:not([theme]) .rowHeaders{background-color:#f8f9fa}revo-grid[theme=default] .rowHeaders revogr-data .rgCell,revo-grid:not([theme]) .rowHeaders revogr-data .rgCell{color:#000}revo-grid[theme=default] .rowHeaders revogr-data .rgCell:first-child,revo-grid:not([theme]) .rowHeaders revogr-data .rgCell:first-child{box-shadow:0 -1px 0 0 #cecece inset}revo-grid[theme=default] .rowHeaders revogr-data .rgCell:not(:first-child),revo-grid:not([theme]) .rowHeaders revogr-data .rgCell:not(:first-child){box-shadow:0 -1px 0 0 #cecece inset, 1px 0 0 0 #cecece inset}revo-grid[theme=default] .rowHeaders revogr-data .rgCell:last-child,revo-grid:not([theme]) .rowHeaders revogr-data .rgCell:last-child{border-right:1px solid #cecece}revo-grid[theme=default] .rowHeaders revogr-data revogr-header,revo-grid:not([theme]) .rowHeaders revogr-data revogr-header{box-shadow:0 -1px 0 0 #cecece inset}revo-grid[theme=default] revogr-viewport-scroll.colPinStart revogr-data .rgRow .rgCell:last-child,revo-grid:not([theme]) revogr-viewport-scroll.colPinStart revogr-data .rgRow .rgCell:last-child{box-shadow:0 -1px 0 0 #e2e3e3 inset, -1px 0 0 0 #cecece inset}revo-grid[theme=default] revogr-viewport-scroll.colPinStart .footer-wrapper revogr-data .rgRow:first-child .rgCell,revo-grid:not([theme]) revogr-viewport-scroll.colPinStart .footer-wrapper revogr-data .rgRow:first-child .rgCell{box-shadow:0 1px 0 0 #cecece inset, -1px 0 0 0 #cecece inset}revo-grid[theme=default] revogr-viewport-scroll.colPinEnd,revo-grid[theme=default] revogr-viewport-scroll.colPinEnd revogr-header,revo-grid:not([theme]) revogr-viewport-scroll.colPinEnd,revo-grid:not([theme]) revogr-viewport-scroll.colPinEnd revogr-header{box-shadow:1px 0 0 #cecece inset}revo-grid[theme=default] .footer-wrapper revogr-data .rgRow:first-child .rgCell,revo-grid:not([theme]) .footer-wrapper revogr-data .rgRow:first-child .rgCell{box-shadow:0 1px 0 0 #e2e3e3 inset, -1px 0 0 0 #e2e3e3 inset, 0 -1px 0 0 #e2e3e3 inset}revo-grid[theme=default] revogr-data,revo-grid:not([theme]) revogr-data{text-align:center}revo-grid[theme=default] revogr-data .revo-draggable,revo-grid:not([theme]) revogr-data .revo-draggable{float:left}revo-grid[theme=default] revogr-data .rgRow,revo-grid:not([theme]) revogr-data .rgRow{line-height:27px}revo-grid[theme=default] revogr-data .rgRow.focused-rgRow,revo-grid:not([theme]) revogr-data .rgRow.focused-rgRow{background-color:rgba(233, 234, 237, 0.5)}revo-grid[theme=default] revogr-data .rgCell,revo-grid:not([theme]) revogr-data .rgCell{box-shadow:0 -1px 0 0 #e2e3e3 inset, -1px 0 0 0 #e2e3e3 inset}revo-grid[theme=default] revogr-data .rgCell.disabled,revo-grid:not([theme]) revogr-data .rgCell.disabled{background-color:0 -1px 0 0 #e2e3e3 inset, -1px 0 0 0 #e2e3e3 inset}revo-grid[theme=material]{font-family:Nunito, -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol"}revo-grid[theme=material] revogr-header{line-height:50px;font-weight:600;text-align:left}revo-grid[theme=material] revogr-header .rgHeaderCell{padding:0 15px;text-overflow:ellipsis}revo-grid[theme=material] revogr-header .header-rgRow{height:50px}revo-grid[theme=material] revogr-data{text-align:left}revo-grid[theme=material] revogr-data .rgRow{line-height:42px}revo-grid[theme=material] revogr-data .rgCell{padding:0 15px}revo-grid[theme=material] .rowHeaders{background-color:#f7faff}revo-grid[theme=material] .rowHeaders revogr-data .rgCell{color:#757a82}revo-grid[theme=material] revogr-header .header-rgRow.group{box-shadow:0 -1px 0 0 #f1f1f1 inset}revo-grid[theme=material] revogr-header .header-rgRow:not(.group){box-shadow:0 -1px 0 0 #f1f1f1, 0 -1px 0 0 #f1f1f1 inset}revo-grid[theme=material] revogr-header .rgHeaderCell.sortable:hover{background-color:#f1f1f1}revo-grid[theme=material] revogr-header .rgHeaderCell.focused-cell{background:rgba(233, 234, 237, 0.5)}revo-grid[theme=material] .footer-wrapper revogr-data{box-shadow:0 -1px 0 #f1f1f1}revo-grid[theme=material] revogr-viewport-scroll.colPinStart{box-shadow:-1px 0 0 #f1f1f1 inset}revo-grid[theme=material] revogr-viewport-scroll.colPinEnd{box-shadow:-1px 0 0 #f1f1f1}revo-grid[theme=material] revogr-data .rgRow{box-shadow:0 -1px 0 0 #f1f1f1 inset}revo-grid[theme=material] revogr-data .rgRow.focused-rgRow{background-color:rgba(233, 234, 237, 0.5)}revo-grid[theme=material] revogr-data .rgCell{color:rgba(0, 0, 0, 0.87)}revo-grid[theme=material] revogr-data .rgCell.disabled{background-color:#f7f7f7}revo-grid[theme=darkMaterial]{font-family:Nunito, -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";color:#d8d8d8}revo-grid[theme=darkMaterial] revogr-header{line-height:50px;font-weight:600;text-align:left}revo-grid[theme=darkMaterial] revogr-header .rgHeaderCell{padding:0 15px;text-overflow:ellipsis}revo-grid[theme=darkMaterial] revogr-header .header-rgRow{height:50px}revo-grid[theme=darkMaterial] revogr-data{text-align:left}revo-grid[theme=darkMaterial] revogr-data .rgRow{line-height:42px}revo-grid[theme=darkMaterial] revogr-data .rgCell{padding:0 15px}revo-grid[theme=darkMaterial] .rowHeaders{background-color:rgba(40, 39, 43, 0.8)}revo-grid[theme=darkMaterial] .rowHeaders revogr-data .rgCell{color:rgba(216, 216, 216, 0.8)}revo-grid[theme=darkMaterial] revogr-edit{background-color:#28272b}revo-grid[theme=darkMaterial] revogr-header .header-rgRow.group{box-shadow:0 -1px 0 0 #404040 inset}revo-grid[theme=darkMaterial] revogr-header .header-rgRow:not(.group){box-shadow:0 -1px 0 0 #404040, 0 -1px 0 0 #404040 inset}revo-grid[theme=darkMaterial] revogr-header .rgHeaderCell.sortable:hover{background-color:rgba(64, 64, 64, 0.5)}revo-grid[theme=darkMaterial] revogr-header .rgHeaderCell.focused-cell{background:rgba(115, 148, 160, 0.15)}revo-grid[theme=darkMaterial] .footer-wrapper revogr-data{box-shadow:0 -1px 0 #404040}revo-grid[theme=darkMaterial] revogr-data .rgCell{color:rgba(216, 216, 216, 0.9)}revo-grid[theme=darkMaterial] revogr-data .rgRow{box-shadow:0 -1px 0 0 #404040 inset}revo-grid[theme=darkMaterial] revogr-data .rgRow.focused-rgRow{background-color:rgba(115, 148, 160, 0.15)}revo-grid[theme=darkMaterial] revogr-viewport-scroll.colPinStart{box-shadow:-1px 0 0 #404040 inset}revo-grid[theme=darkMaterial] revogr-viewport-scroll.colPinEnd{box-shadow:-1px 0 0 #404040}revo-grid[theme=darkCompact]{font-family:Nunito, -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";color:#d8d8d8}revo-grid[theme=darkCompact] revogr-header{line-height:45px;font-weight:600;text-align:left}revo-grid[theme=darkCompact] revogr-header .rgHeaderCell{padding:0 15px;text-overflow:ellipsis}revo-grid[theme=darkCompact] revogr-header .header-rgRow{height:45px}revo-grid[theme=darkCompact] revogr-data{text-align:left}revo-grid[theme=darkCompact] revogr-data .rgRow{line-height:32px}revo-grid[theme=darkCompact] revogr-data .rgCell{padding:0 15px}revo-grid[theme=darkCompact] .rowHeaders{background-color:rgba(40, 39, 43, 0.8)}revo-grid[theme=darkCompact] .rowHeaders revogr-data .rgCell{color:rgba(216, 216, 216, 0.8)}revo-grid[theme=darkCompact] revogr-edit{background-color:#28272b}revo-grid[theme=darkCompact] revogr-header .header-rgRow.group{box-shadow:0 -1px 0 0 #404040 inset}revo-grid[theme=darkCompact] revogr-header .header-rgRow:not(.group){box-shadow:0 -1px 0 0 #404040, 0 -1px 0 0 #404040 inset}revo-grid[theme=darkCompact] revogr-header .rgHeaderCell.sortable:hover{background-color:rgba(64, 64, 64, 0.5)}revo-grid[theme=darkCompact] revogr-header .rgHeaderCell.focused-cell{background:rgba(115, 148, 160, 0.15)}revo-grid[theme=darkCompact] .footer-wrapper revogr-data{box-shadow:0 -1px 0 #404040}revo-grid[theme=darkCompact] revogr-data .rgCell{color:rgba(216, 216, 216, 0.9)}revo-grid[theme=darkCompact] revogr-data .rgRow{box-shadow:0 -1px 0 0 #404040 inset}revo-grid[theme=darkCompact] revogr-data .rgRow.focused-rgRow{background-color:rgba(115, 148, 160, 0.15)}revo-grid[theme=darkCompact] revogr-viewport-scroll.colPinStart{box-shadow:-1px 0 0 #404040 inset}revo-grid[theme=darkCompact] revogr-viewport-scroll.colPinEnd{box-shadow:-1px 0 0 #404040}revo-grid[theme=compact]{font-family:Nunito, -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol"}revo-grid[theme=compact] revogr-header{line-height:45px;font-weight:600;text-align:left}revo-grid[theme=compact] revogr-header .rgHeaderCell{padding:0 15px;text-overflow:ellipsis}revo-grid[theme=compact] revogr-header .header-rgRow{height:45px}revo-grid[theme=compact] revogr-data{text-align:left}revo-grid[theme=compact] revogr-data .rgRow{line-height:32px}revo-grid[theme=compact] revogr-data .rgCell{padding:0 15px}revo-grid[theme=compact] .rowHeaders{background-color:#f7faff}revo-grid[theme=compact] .rowHeaders revogr-data .rgCell{color:#757a82}revo-grid[theme=compact] revogr-header .header-rgRow.group{box-shadow:0 -1px 0 0 #f1f1f1 inset}revo-grid[theme=compact] revogr-header .header-rgRow:not(.group){box-shadow:0 -1px 0 0 #f1f1f1, 0 -1px 0 0 #f1f1f1 inset}revo-grid[theme=compact] revogr-header .rgHeaderCell.sortable:hover{background-color:#f1f1f1}revo-grid[theme=compact] revogr-header .rgHeaderCell.focused-cell{background:rgba(233, 234, 237, 0.5)}revo-grid[theme=compact] .footer-wrapper revogr-data{box-shadow:0 -1px 0 #f1f1f1}revo-grid[theme=compact] revogr-viewport-scroll.colPinStart{box-shadow:-1px 0 0 #f1f1f1 inset}revo-grid[theme=compact] revogr-viewport-scroll.colPinEnd{box-shadow:-1px 0 0 #f1f1f1}revo-grid[theme=compact] revogr-data .rgRow{box-shadow:0 -1px 0 0 #f1f1f1 inset}revo-grid[theme=compact] revogr-data .rgRow.focused-rgRow{background-color:rgba(233, 234, 237, 0.5)}revo-grid[theme=compact] revogr-data .rgCell{color:rgba(0, 0, 0, 0.87)}revo-grid[theme=compact] revogr-data .rgCell.disabled{background-color:#f7f7f7}revo-grid[theme=compact] revo-dropdown .rv-dr-root{padding:0px 9px}revo-grid{display:flex !important;height:100%;min-height:300px;font-family:Helvetica, Arial, Sans-Serif, serif;font-size:14px;position:relative;-webkit-touch-callout:none;-webkit-user-select:none;-khtml-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;display:flex;flex-direction:column;width:100%;height:100%}revo-grid .attribution{position:absolute;bottom:0;left:0;right:0;z-index:1000;width:0;height:0;border-left:4px solid rgba(38, 106, 232, 0.9);border-bottom:4px solid rgba(38, 106, 232, 0.9);border-top:4px solid transparent;border-right:4px solid transparent;cursor:pointer}revo-grid .attribution .value{position:absolute;bottom:0;left:0;background-color:rgba(255, 255, 255, 0.9);padding:4px;border-radius:4px;box-shadow:0 1px 10px rgba(0, 0, 0, 0.2);white-space:nowrap;text-decoration:none;color:black;letter-spacing:0.3px;font-size:11px;opacity:0;width:4px;overflow:hidden;transition:opacity 0.5s ease-in-out, width 0.3s ease-in-out}revo-grid .attribution:hover .value{width:63px;opacity:1}revo-grid.column-draggable.column-drag-start:hover,revo-grid.column-draggable.column-drag-start *:hover{cursor:grabbing}revo-grid .footer-wrapper,revo-grid .header-wrapper{width:100%}revo-grid .footer-wrapper revogr-data,revo-grid .header-wrapper revogr-data{z-index:3}revo-grid revo-dropdown{width:100%}revo-grid revo-dropdown .rv-dr-root{max-height:100%}revo-grid revo-dropdown.shrink label{opacity:0}revo-grid .viewports{max-width:100%;display:flex;flex-direction:row;align-items:flex-start;flex-grow:1}revo-grid .main-viewport{flex-grow:1;height:0;display:flex;justify-content:space-between;flex-direction:row}revo-grid .draggable{position:fixed;height:30px;line-height:30px;background:#fff;border-radius:3px;display:block;z-index:100;margin-top:5px;margin-right:-20px;box-shadow:0 4px 20px 0 rgba(0, 0, 0, 0.15);padding-left:20px;padding-right:5px}revo-grid .draggable.hidden{display:none}revo-grid .draggable .revo-alt-icon{background-color:black;position:absolute;left:5px;top:10px}revo-grid .draggable-wrapper.hidden{display:none}revo-grid .drag-position{position:absolute;left:0;right:0;height:1px;z-index:2;background:gray;pointer-events:none}revo-grid .drag-position-y{position:absolute;top:0;left:0;bottom:0;width:1px;z-index:2;background:gray;pointer-events:none}revo-grid .drag-auto-scroll-y{pointer-events:none;position:absolute;left:0;top:0;height:50px;width:1px}revo-grid .clipboard{position:absolute;left:0;top:0}revo-grid revogr-scroll-virtual{position:relative}revo-grid revogr-scroll-virtual.vertical,revo-grid revogr-scroll-virtual.horizontal{z-index:3}`, RevoGridStyle0 = revoGridStyleCss, RevoGridComponent = class {
    constructor(O) {
      registerInstance(this, O), this.contentsizechanged = createEvent(this, "contentsizechanged", 7), this.beforeedit = createEvent(this, "beforeedit", 7), this.beforerangeedit = createEvent(this, "beforerangeedit", 7), this.afteredit = createEvent(this, "afteredit", 7), this.beforeautofill = createEvent(this, "beforeautofill", 7), this.beforerange = createEvent(this, "beforerange", 7), this.afterfocus = createEvent(this, "afterfocus", 7), this.roworderchanged = createEvent(this, "roworderchanged", 7), this.beforesorting = createEvent(this, "beforesorting", 7), this.beforesourcesortingapply = createEvent(this, "beforesourcesortingapply", 7), this.beforesortingapply = createEvent(this, "beforesortingapply", 7), this.rowdragstart = createEvent(this, "rowdragstart", 7), this.headerclick = createEvent(this, "headerclick", 7), this.beforecellfocus = createEvent(this, "beforecellfocus", 7), this.beforefocuslost = createEvent(this, "beforefocuslost", 7), this.beforesourceset = createEvent(this, "beforesourceset", 7), this.beforeanysource = createEvent(this, "beforeanysource", 7), this.aftersourceset = createEvent(this, "aftersourceset", 7), this.afteranysource = createEvent(this, "afteranysource", 7), this.beforecolumnsset = createEvent(this, "beforecolumnsset", 7), this.beforecolumnapplied = createEvent(this, "beforecolumnapplied", 7), this.aftercolumnsset = createEvent(this, "aftercolumnsset", 7), this.beforefilterapply = createEvent(this, "beforefilterapply", 7), this.beforefiltertrimmed = createEvent(this, "beforefiltertrimmed", 7), this.beforetrimmed = createEvent(this, "beforetrimmed", 7), this.aftertrimmed = createEvent(this, "aftertrimmed", 7), this.viewportscroll = createEvent(this, "viewportscroll", 7), this.beforeexport = createEvent(this, "beforeexport", 7), this.beforeeditstart = createEvent(this, "beforeeditstart", 7), this.aftercolumnresize = createEvent(this, "aftercolumnresize", 7), this.beforerowdefinition = createEvent(this, "beforerowdefinition", 7), this.filterconfigchanged = createEvent(this, "filterconfigchanged", 7), this.sortingconfigchanged = createEvent(this, "sortingconfigchanged", 7), this.rowheaderschanged = createEvent(this, "rowheaderschanged", 7), this.beforegridrender = createEvent(this, "beforegridrender", 7), this.aftergridrender = createEvent(this, "aftergridrender", 7), this.aftergridinit = createEvent(this, "aftergridinit", 7), this.additionaldatachanged = createEvent(this, "additionaldatachanged", 7), this.afterthemechanged = createEvent(this, "afterthemechanged", 7), this.created = createEvent(this, "created", 7), this.extraElements = [], this.pluginService = new PluginService(), this.viewport = null, this.isInited = !1, this.rowHeaders = void 0, this.frameSize = 1, this.rowSize = 0, this.colSize = 100, this.range = !1, this.readonly = !1, this.resize = !1, this.canFocus = !0, this.useClipboard = !0, this.columns = [], this.source = [], this.pinnedTopSource = [], this.pinnedBottomSource = [], this.rowDefinitions = [], this.editors = {}, this.applyOnClose = !1, this.plugins = [], this.columnTypes = {}, this.theme = "default", this.rowClass = "", this.autoSizeColumn = !1, this.filter = !1, this.sorting = void 0, this.focusTemplate = void 0, this.canMoveColumns = !1, this.trimmedRows = {}, this.exporting = !1, this.grouping = void 0, this.stretch = !1, this.additionalData = {}, this.disableVirtualX = !1, this.disableVirtualY = !1, this.hideAttribution = !1, this.jobsBeforeRender = [], this.registerVNode = [], this.accessible = !0;
    }
    // #endregion
    // #region Methods
    /**
     * Refreshes data viewport.
     * Can be specific part as rgRow or pinned rgRow or 'all' by default.
     */
    async refresh(O = "all") {
      if (!this.dataProvider)
        throw new Error("Not connected");
      this.dataProvider.refresh(O);
    }
    /**
     * Refreshes data at specified cell.
     * Useful for performance optimization.
     * No viewport update will be triggered.
     *
     * @example
     * const grid = document.querySelector('revo-grid');
     * grid.setDataAt({ row: 0, col: 0, val: 'test' }); // refresh
     */
    async setDataAt({ row: O, col: _, colType: H = "rgCol", rowType: V = "rgRow", val: q, skipDataUpdate: G = !1 }) {
      var J;
      if (this.dataProvider && this.columnProvider && !G) {
        const te = (J = this.columnProvider.getColumn(_, H)) === null || J === void 0 ? void 0 : J.prop;
        typeof te < "u" && this.dataProvider.setCellData({
          type: V,
          rowIndex: O,
          prop: te,
          val: q
        }, !1);
      }
      const Z = this.element.querySelector(`revogr-data[type="${V}"][col-type="${H}"]`);
      return Z == null ? void 0 : Z.updateCell({
        row: O,
        col: _
      });
    }
    /**
     * Scrolls viewport to specified row by index.
     */
    async scrollToRow(O = 0) {
      if (!this.dimensionProvider)
        throw new Error("Not connected");
      const _ = this.dimensionProvider.getViewPortPos({
        coordinate: O,
        dimension: "rgRow"
      });
      await this.scrollToCoordinate({ y: _ });
    }
    /**
     * Scrolls viewport to specified column by index.
     */
    async scrollToColumnIndex(O = 0) {
      if (!this.dimensionProvider)
        throw new Error("Not connected");
      const _ = this.dimensionProvider.getViewPortPos({
        coordinate: O,
        dimension: "rgCol"
      });
      await this.scrollToCoordinate({ x: _ });
    }
    /**
     * Scrolls viewport to specified column by prop
     */
    async scrollToColumnProp(O, _ = "rgCol") {
      if (!this.dimensionProvider || !this.columnProvider)
        throw new Error("Not connected");
      const H = this.columnProvider.getColumnIndexByProp(O, _);
      if (H < 0)
        return;
      const V = this.dimensionProvider.getViewPortPos({
        coordinate: H,
        dimension: _
      });
      await this.scrollToCoordinate({ x: V });
    }
    /** Update columns */
    async updateColumns(O) {
      var _;
      (_ = this.columnProvider) === null || _ === void 0 || _.updateColumns(O);
    }
    /** Add trimmed by type */
    async addTrimmed(O, _ = "external", H = "rgRow") {
      if (!this.dataProvider)
        throw new Error("Not connected");
      const V = this.beforetrimmed.emit({
        trimmed: O,
        trimmedType: _,
        type: H
      });
      return V.defaultPrevented || (this.dataProvider.setTrimmed({ [_]: V.detail.trimmed }, H), this.aftertrimmed.emit()), V;
    }
    /**  Scrolls view port to coordinate */
    async scrollToCoordinate(O) {
      var _;
      (_ = this.viewport) === null || _ === void 0 || _.scrollToCell(O);
    }
    /**  Open editor for cell. */
    async setCellEdit(O, _, H = "rgRow") {
      var V;
      const q = getColumnByProp(this.columns, _);
      if (!q)
        return;
      await timeout();
      const G = q.pin || "rgCol";
      if (!this.columnProvider)
        throw new Error("Not connected");
      (V = this.viewport) === null || V === void 0 || V.setEdit(O, this.columnProvider.getColumnIndexByProp(_, G), G, H);
    }
    /**  Set focus range. */
    async setCellsFocus(O = { x: 0, y: 0 }, _ = { x: 0, y: 0 }, H = "rgCol", V = "rgRow") {
      var q;
      (q = this.viewport) === null || q === void 0 || q.setFocus(H, V, O, _);
    }
    /**  Get data from source */
    async getSource(O = "rgRow") {
      if (!this.dataProvider)
        throw new Error("Not connected");
      return this.dataProvider.stores[O].store.get("source");
    }
    /**
     * Get data from visible part of source
     * Trimmed/filtered rows will be excluded
     * @param type - type of source
     */
    async getVisibleSource(O = "rgRow") {
      if (!this.dataProvider)
        throw new Error("Not connected");
      return getVisibleSourceItem(this.dataProvider.stores[O].store);
    }
    /**
     * Provides access to rows internal store observer
     * Can be used for plugin support
     * @param type - type of source
     */
    async getSourceStore(O = "rgRow") {
      if (!this.dataProvider)
        throw new Error("Not connected");
      return this.dataProvider.stores[O].store;
    }
    /**
     * Provides access to column internal store observer
     * Can be used for plugin support
     * @param type - type of column
     */
    async getColumnStore(O = "rgCol") {
      if (!this.columnProvider)
        throw new Error("Not connected");
      return this.columnProvider.stores[O].store;
    }
    /**
     * Update column sorting
     * @param column - full column details to update
     * @param index - virtual column index
     * @param order - order to apply
     * @param additive - if false will replace current order
     */
    async updateColumnSorting(O, _, H, V) {
      if (!this.columnProvider)
        throw new Error("Not connected");
      return this.columnProvider.updateColumnSorting(O, _, H, V);
    }
    /**
     * Clears column sorting
     */
    async clearSorting() {
      if (!this.columnProvider)
        throw new Error("Not connected");
      this.columnProvider.clearSorting();
    }
    /**
     * Receive all columns in data source
     */
    async getColumns() {
      if (!this.columnProvider)
        throw new Error("Not connected");
      return this.columnProvider.getColumns();
    }
    /**
     * Clear current grid focus. Grid has no longer focus on it.
     */
    async clearFocus() {
      var O, _;
      const H = (O = this.viewport) === null || O === void 0 ? void 0 : O.getFocused();
      this.beforefocuslost.emit(H).defaultPrevented || (_ = this.selectionStoreConnector) === null || _ === void 0 || _.clearAll();
    }
    /**
     * Get all active plugins instances
     */
    async getPlugins() {
      return this.pluginService.get();
    }
    /**
     * Get the currently focused cell.
     */
    async getFocused() {
      var O, _;
      return (_ = (O = this.viewport) === null || O === void 0 ? void 0 : O.getFocused()) !== null && _ !== void 0 ? _ : null;
    }
    /**
     * Get size of content
     * Including all pinned data
     */
    async getContentSize() {
      var O;
      if (!this.dimensionProvider)
        throw new Error("Not connected");
      return (O = this.dimensionProvider) === null || O === void 0 ? void 0 : O.getFullSize();
    }
    /**
     * Get the currently selected Range.
     */
    async getSelectedRange() {
      var O, _;
      return (_ = (O = this.viewport) === null || O === void 0 ? void 0 : O.getSelectedRange()) !== null && _ !== void 0 ? _ : null;
    }
    /**
     * Refresh extra elements. Triggers re-rendering of extra elements and functions.
     * Part of extraElements and registerVNode methods.
     * Useful for plugins.
     */
    async refreshExtraElements() {
      var O;
      (O = this.extraService) === null || O === void 0 || O.refresh();
    }
    mousedownHandle(O) {
      const _ = getPropertyFromEvent(O, "screenX"), H = getPropertyFromEvent(O, "screenY");
      _ === null || H === null || (this.clickTrackForFocusClear = _ + H);
    }
    /**
     * To keep your elements from losing focus use mouseup/touchend e.preventDefault();
     */
    async mouseupHandle(O) {
      var _;
      const H = getPropertyFromEvent(O, "screenX"), V = getPropertyFromEvent(O, "screenY");
      if (H === null || V === null || O.defaultPrevented)
        return;
      const q = H + V;
      if (Math.abs(((_ = this.clickTrackForFocusClear) !== null && _ !== void 0 ? _ : 0) - q) > 10)
        return;
      const G = O.composedPath();
      !G.includes(this.element) && !(this.element.shadowRoot && G.includes(this.element.shadowRoot)) && await this.clearFocus();
    }
    // #endregion
    // #region Listeners
    /** Drag events */
    onRowDragStarted(O) {
      var _;
      const H = this.rowdragstart.emit(O.detail);
      if (H.defaultPrevented) {
        O.preventDefault();
        return;
      }
      (_ = this.orderService) === null || _ === void 0 || _.start(this.element, Object.assign(Object.assign({}, O.detail), H.detail));
    }
    onRowDragEnd() {
      var O;
      (O = this.orderService) === null || O === void 0 || O.end();
    }
    onRowOrderChange(O) {
      var _;
      (_ = this.dataProvider) === null || _ === void 0 || _.changeOrder(O.detail);
    }
    onRowDrag({ detail: O }) {
      var _;
      (_ = this.orderService) === null || _ === void 0 || _.move(O);
    }
    onRowMouseMove(O) {
      var _;
      (_ = this.orderService) === null || _ === void 0 || _.moveTip(O.detail);
    }
    async onCellEdit(O) {
      var _;
      const { defaultPrevented: H, detail: V } = this.beforeedit.emit(O.detail);
      await timeout(), H || ((_ = this.dataProvider) === null || _ === void 0 || _.setCellData(V), this.afteredit.emit(V));
    }
    onRangeEdit(O) {
      if (!this.dataProvider)
        throw new Error("Not connected");
      const { defaultPrevented: _, detail: H } = this.beforerangeedit.emit(O.detail);
      if (_) {
        O.preventDefault();
        return;
      }
      this.dataProvider.setRangeData(H.data, H.type), this.afteredit.emit(H);
    }
    onRangeChanged(O) {
      const _ = this.beforerange.emit(O.detail);
      _.defaultPrevented && O.preventDefault(), this.beforeautofill.emit(_.detail).defaultPrevented && O.preventDefault();
    }
    onRowDropped(O) {
      const { defaultPrevented: _ } = this.roworderchanged.emit(O.detail);
      _ && O.preventDefault();
    }
    onHeaderClick(O) {
      const { defaultPrevented: _ } = this.headerclick.emit(Object.assign(Object.assign({}, O.detail.column), { originalEvent: O.detail.originalEvent }));
      _ && O.preventDefault();
    }
    onCellFocus(O) {
      const { defaultPrevented: _ } = this.beforecellfocus.emit(O.detail);
      (!this.canFocus || _) && O.preventDefault();
    }
    // #endregion
    // #region Watchers
    columnTypesChanged() {
      this.columnChanged(this.columns);
    }
    columnChanged(O = [], _ = void 0, H = "columns", V = !1) {
      if (!this.dimensionProvider || !this.columnProvider)
        return;
      const q = getColumns(O, 0, this.columnTypes), G = this.beforecolumnsset.emit(q);
      if (G.defaultPrevented)
        return;
      this.dimensionProvider.applyNewColumns(G.detail.columns, this.disableVirtualX, V);
      const J = this.beforecolumnapplied.emit(q);
      if (J.defaultPrevented)
        return;
      const Z = this.columnProvider.setColumns(J.detail);
      this.aftercolumnsset.emit({
        columns: Z,
        order: this.columnProvider.order
      });
    }
    disableVirtualXChanged(O = !1, _ = !1) {
      O !== _ && this.columnChanged(this.columns);
    }
    rowSizeChanged(O) {
      this.dimensionProvider && (this.dimensionProvider.setSettings({ originItemSize: O }, "rgRow"), this.rowDefChanged(this.rowDefinitions, this.rowDefinitions, "rowSize", !0));
    }
    themeChanged(O, _, H = "theme", V = !1) {
      this.dimensionProvider && (this.themeService.register(O), this.dimensionProvider.setSettings({ originItemSize: this.themeService.rowSize }, "rgRow"), this.dimensionProvider.setSettings({ originItemSize: this.colSize }, "rgCol"), V || (this.dimensionProvider.setSettings({ originItemSize: this.themeService.rowSize }, "rgRow"), this.rowDefChanged(
        // for cases when some custom size present and not
        this.rowDefinitions,
        this.rowDefinitions,
        "theme",
        !0
      )), this.afterthemechanged.emit(O));
    }
    dataSourceChanged(O = [], _, H) {
      if (!this.dataProvider)
        return;
      let V = "rgRow";
      switch (H) {
        case "pinnedBottomSource":
          V = "rowPinEnd";
          break;
        case "pinnedTopSource":
          V = "rowPinStart";
          break;
        case "source":
          V = "rgRow", O = this.beforesourceset.emit({
            type: V,
            source: O
          }).detail.source;
          break;
      }
      const G = [...this.beforeanysource.emit({
        type: V,
        source: O
      }).detail.source];
      this.dataProvider.setData(G, V, this.disableVirtualY), H === "source" && this.aftersourceset.emit({
        type: V,
        source: O
      }), this.afteranysource.emit({
        type: V,
        source: O
      });
    }
    disableVirtualYChanged(O = !1, _ = !1) {
      O !== _ && this.dataSourceChanged(this.source, this.source, "source");
    }
    rowDefChanged(O, _, H, V = !0) {
      if (!this.dimensionProvider || !this.dataProvider)
        return;
      const { detail: { vals: q, oldVals: G } } = this.beforerowdefinition.emit({
        vals: O,
        oldVals: _
      }), J = rowDefinitionByType(q);
      if (G) {
        const Z = rowDefinitionRemoveByType(G);
        for (const te in Z)
          if (Z.hasOwnProperty(te)) {
            const ne = te, ae = this.dataProvider.stores[ne].store.get("source").length;
            this.dimensionProvider.clearSize(ne, ae);
          }
      }
      rowTypes.forEach((Z) => {
        var te;
        const ne = J[Z];
        (ne || V) && ((te = this.dimensionProvider) === null || te === void 0 || te.setCustomSizes(Z, (ne == null ? void 0 : ne.sizes) || {}));
      });
    }
    trimmedRowsChanged(O = {}) {
      this.addTrimmed(O);
    }
    /**
     * Grouping
     */
    groupingChanged(O = {}) {
      var _;
      (_ = this.pluginService.getByClass(GroupingRowPlugin)) === null || _ === void 0 || _.setGrouping(O || {});
    }
    /**
     * Stretch Plugin Apply
     */
    applyStretch(O) {
      if (!this.dimensionProvider || !this.dataProvider || !this.columnProvider || !this.viewportProvider)
        return;
      O === "false" && (O = !1);
      const _ = this.getPluginData();
      if (!_)
        return;
      const H = this.pluginService.getByClass(StretchColumn);
      typeof O == "boolean" && O || O === "true" ? H ? isStretchPlugin(H) && H.applyStretch(this.columnProvider.getRawColumns()) : this.pluginService.add(new StretchColumn(this.element, _)) : H && this.pluginService.remove(H);
    }
    applyFilter(O) {
      this.filterconfigchanged.emit(O);
    }
    applySorting(O) {
      this.sortingconfigchanged.emit(O);
    }
    rowHeadersChange(O) {
      this.rowheaderschanged.emit(O);
    }
    /**
     * Register external VNodes
     */
    registerOutsideVNodes(O = []) {
      this.extraElements = O;
    }
    additionalDataChanged(O) {
      this.additionaldatachanged.emit(O);
    }
    /**
     * User can add plugins via plugins property
     */
    pluginsChanged(O = [], _) {
      this.pluginService.addUserPluginsAndCreate(this.element, O, _, this.getPluginData());
    }
    // #endregion
    // #region Plugins
    setPlugins() {
      this.removePlugins();
      const O = this.getPluginData();
      O && (this.setCorePlugins(O), this.pluginsChanged(this.plugins));
    }
    setCorePlugins(O) {
      this.accessible && this.pluginService.add(new WCAGPlugin(this.element, O)), this.autoSizeColumn && this.pluginService.add(new AutoSizeColumnPlugin(this.element, O, typeof this.autoSizeColumn == "object" ? this.autoSizeColumn : void 0)), this.filter && this.pluginService.add(new FilterPlugin(this.element, O, typeof this.filter == "object" ? this.filter : void 0)), this.exporting && this.pluginService.add(new ExportFilePlugin(this.element, O)), this.pluginService.add(new SortingPlugin(this.element, O)), this.pluginService.add(new GroupingRowPlugin(this.element, O)), this.canMoveColumns && this.pluginService.add(new ColumnMovePlugin(this.element, O));
    }
    getPluginData() {
      return !this.dimensionProvider || !this.dataProvider || !this.columnProvider || !this.viewportProvider || !this.selectionStoreConnector ? void 0 : {
        data: this.dataProvider,
        column: this.columnProvider,
        dimension: this.dimensionProvider,
        viewport: this.viewportProvider,
        selection: this.selectionStoreConnector,
        plugins: this.pluginService
      };
    }
    removePlugins() {
      this.pluginService.destroy();
    }
    // #endregion
    // if reconnect to dom we need to set up plugins
    connectedCallback() {
      this.isInited && this.setPlugins(), this.created.emit();
    }
    /**
     * Called once just after the component is first connected to the DOM.
     * Since this method is only called once, it's a good place to load data asynchronously and to setup the state
     * without triggering extra re-renders.
     * A promise can be returned, that can be used to wait for the first render().
     */
    componentWillLoad() {
      var O;
      this.viewportProvider = new ViewportProvider(), this.themeService = new ThemeService({
        rowSize: this.rowSize
      }), this.dimensionProvider = new DimensionProvider(this.viewportProvider, {
        realSizeChanged: (_) => this.contentsizechanged.emit(_)
      }), this.columnProvider = new ColumnDataProvider(), this.selectionStoreConnector = new SelectionStoreConnector(), this.dataProvider = new DataProvider(this.dimensionProvider), this.registerOutsideVNodes(this.registerVNode), this.setPlugins(), this.applyStretch(this.stretch), this.themeChanged(this.theme, void 0, void 0, !0), this.columnChanged(this.columns, void 0, void 0, !0), this.dataSourceChanged(this.source, void 0, "source"), this.dataSourceChanged(this.pinnedTopSource, void 0, "pinnedTopSource"), this.dataSourceChanged(this.pinnedBottomSource, void 0, "pinnedBottomSource"), Object.keys((O = this.trimmedRows) !== null && O !== void 0 ? O : {}).length > 0 && this.trimmedRowsChanged(this.trimmedRows), this.rowDefChanged(this.rowDefinitions), this.grouping && Object.keys(this.grouping).length > 0 && this.groupingChanged(this.grouping), this.scrollingService = new GridScrollingService((_) => {
        var H;
        (H = this.dimensionProvider) === null || H === void 0 || H.setViewPortCoordinate({
          coordinate: _.coordinate,
          type: _.dimension
        }), this.viewportscroll.emit(_);
      }), this.aftergridinit.emit(), this.isInited = !0;
    }
    componentWillRender() {
      return this.beforegridrender.emit().defaultPrevented ? !1 : Promise.all(this.jobsBeforeRender);
    }
    componentDidRender() {
      this.aftergridrender.emit();
    }
    render() {
      if (!this.dimensionProvider || !this.dataProvider || !this.columnProvider || !this.viewportProvider || !this.selectionStoreConnector)
        return;
      const O = this.dimensionProvider.stores.rgRow.store.get("realSize");
      this.viewport = new ViewportService({
        columnProvider: this.columnProvider,
        dataProvider: this.dataProvider,
        dimensionProvider: this.dimensionProvider,
        viewportProvider: this.viewportProvider,
        scrollingService: this.scrollingService,
        orderService: this.orderService,
        selectionStoreConnector: this.selectionStoreConnector,
        disableVirtualX: this.disableVirtualX,
        disableVirtualY: this.disableVirtualY,
        resize: (te) => this.aftercolumnresize.emit(te)
      }, O);
      const _ = [];
      if (this.rowHeaders && this.viewport.columns.length) {
        const te = this.viewport.columns[0];
        _.push(h("revogr-row-headers", { additionalData: this.additionalData, height: O, rowClass: this.rowClass, resize: this.resize, dataPorts: te.dataPorts, headerProp: te.headerProp, jobsBeforeRender: this.jobsBeforeRender, rowHeaderColumn: typeof this.rowHeaders == "object" ? this.rowHeaders : void 0, onScrollview: ({ detail: ne }) => this.scrollingService.proxyScroll(ne, "headerRow"), onRef: ({ detail: ne }) => this.scrollingService.registerElement(ne, "headerRow") }));
      }
      const H = isMobileDevice(), V = [];
      for (let te of this.viewport.columns) {
        const ne = Object.assign(Object.assign({}, te.headerProp), { type: te.type, additionalData: this.additionalData, viewportCol: te.viewportCol, selectionStore: te.columnSelectionStore, canResize: this.resize, readonly: this.readonly, columnFilter: !!this.filter }), ee = [
          h("revogr-header", Object.assign({}, ne, { slot: HEADER_SLOT }))
        ];
        te.dataPorts.forEach((ae) => {
          const oe = `${ae.type}_${te.type}`, se = h("revogr-overlay-selection", Object.assign({}, ae, { isMobileDevice: H, onSelectall: () => {
            var ce;
            return (ce = this.selectionStoreConnector) === null || ce === void 0 ? void 0 : ce.selectAll();
          }, editors: this.editors, readonly: this.readonly, range: this.range, useClipboard: this.useClipboard, applyChangesOnClose: this.applyOnClose, additionalData: this.additionalData, slot: ae.slot, onBeforenextvpfocus: (ce) => {
            var le;
            return (le = this.selectionStoreConnector) === null || le === void 0 ? void 0 : le.beforeNextFocusCell(ce.detail);
          }, onCanceledit: () => {
            var ce;
            return (ce = this.selectionStoreConnector) === null || ce === void 0 ? void 0 : ce.setEdit(!1);
          }, onSetedit: ({ detail: ce }) => {
            var le;
            this.beforeeditstart.emit(ce).defaultPrevented || (le = this.selectionStoreConnector) === null || le === void 0 || le.setEdit(ce.val);
          } }), h("revogr-data", Object.assign({}, ae, { colType: te.type, key: oe, readonly: this.readonly, range: this.range, rowClass: this.rowClass, rowSelectionStore: ae.rowSelectionStore, additionalData: this.additionalData, jobsBeforeRender: this.jobsBeforeRender, slot: DATA_SLOT }), h("slot", { name: `data-${te.type}-${ae.type}` })), h("revogr-temp-range", { selectionStore: ae.selectionStore, dimensionRow: ae.dimensionRow, dimensionCol: ae.dimensionCol }), h("revogr-focus", { colData: ae.colData, dataStore: ae.dataStore, focusTemplate: this.focusTemplate, rowType: ae.type, colType: te.type, selectionStore: ae.selectionStore, dimensionRow: ae.dimensionRow, dimensionCol: ae.dimensionCol }, h("slot", { name: `focus-${te.type}-${ae.type}` })));
          ee.push(se);
        }), V.push(h("revogr-viewport-scroll", Object.assign({}, te.prop, { ref: (ae) => this.scrollingService.registerElement(ae, `${te.prop.key}`), onScrollviewport: (ae) => this.scrollingService.proxyScroll(ae.detail, `${te.prop.key}`), onScrollviewportsilent: (ae) => this.scrollingService.scrollSilentService(ae.detail, `${te.prop.key}`) }), ee));
      }
      _.push(V);
      const q = "rgRow", G = "rgCol", J = this.viewportProvider.stores, Z = this.dimensionProvider.stores;
      return h(Host, null, this.hideAttribution ? null : h("revogr-attribution", { class: "attribution" }), h("slot", { name: "header" }), h("div", { class: "main-viewport", onClick: (te) => {
        var ne;
        te.currentTarget === te.target && ((ne = this.viewport) === null || ne === void 0 || ne.clearEdit());
      } }, h("div", { class: "viewports" }, h("slot", { name: "viewport" }), _, h("revogr-scroll-virtual", { class: "vertical", dimension: q, viewportStore: J[q].store, dimensionStore: Z[q].store, ref: (te) => this.scrollingService.registerElement(te, "rowScroll"), onScrollvirtual: (te) => this.scrollingService.proxyScroll(te.detail) }), h(OrderRenderer, { ref: (te) => this.orderService = te }))), h("revogr-scroll-virtual", { class: "horizontal", dimension: G, viewportStore: J[G].store, dimensionStore: Z[G].store, ref: (te) => this.scrollingService.registerElement(te, "colScroll"), onScrollvirtual: (te) => this.scrollingService.proxyScroll(te.detail) }), h("revogr-extra", { ref: (te) => this.extraService = te, nodes: this.extraElements }), h("slot", { name: "footer" }));
    }
    disconnectedCallback() {
      this.removePlugins();
    }
    get element() {
      return getElement(this);
    }
    static get watchers() {
      return {
        columnTypes: ["columnTypesChanged"],
        columns: ["columnChanged"],
        disableVirtualX: ["disableVirtualXChanged"],
        rowSize: ["rowSizeChanged"],
        theme: ["themeChanged"],
        source: ["dataSourceChanged"],
        pinnedBottomSource: ["dataSourceChanged"],
        pinnedTopSource: ["dataSourceChanged"],
        disableVirtualY: ["disableVirtualYChanged"],
        rowDefinitions: ["rowDefChanged"],
        trimmedRows: ["trimmedRowsChanged"],
        grouping: ["groupingChanged"],
        stretch: ["applyStretch"],
        filter: ["applyFilter"],
        sorting: ["applySorting"],
        rowHeaders: ["rowHeadersChange"],
        registerVNode: ["registerOutsideVNodes"],
        additionalData: ["additionalDataChanged"],
        plugins: ["pluginsChanged"]
      };
    }
  };
  RevoGridComponent.style = RevoGridStyle0;
  const revoGrid_entry = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    revo_grid: RevoGridComponent
  }, Symbol.toStringTag, { value: "Module" }));
  /*!
   * Built by Revolist OU ❤️
   */
  (function O() {
    Element.prototype.matches || (Element.prototype.matches = Element.prototype.msMatchesSelector || Element.prototype.webkitMatchesSelector), Element.prototype.closest || (Element.prototype.closest = function(_) {
      let H = this;
      do {
        if (Element.prototype.matches.call(H, _))
          return H;
        H = H.parentElement || H.parentNode;
      } while (H !== null && H.nodeType === 1);
      return null;
    });
  })();
  const filterStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revogr-filter-panel{position:absolute;display:block;top:0;left:0;z-index:100;max-height:calc(100% - 80px);min-width:250px;overflow:auto;opacity:1;transform:none;background-color:#fff;transform-origin:62px 0px;box-shadow:0 5px 18px -2px rgba(0, 0, 0, 0.2);padding:10px;border-radius:4px;min-width:220px;text-align:left}revogr-filter-panel .filter-holder>div{display:flex;flex-direction:column}revogr-filter-panel label{font-size:13px;display:block;padding:8px 0}revogr-filter-panel select{width:100%}revogr-filter-panel input[type=text]{border:0;min-height:34px;margin:5px 0;background:#f3f3f3;border-radius:5px;padding:0 10px;box-sizing:border-box;width:100%}revogr-filter-panel button{margin-top:10px;margin-right:5px}revogr-filter-panel .filter-actions{text-align:right;margin-right:-5px}.rgHeaderCell:hover .rv-filter{transition:opacity 267ms cubic-bezier(0.4, 0, 0.2, 1) 0ms, transform 178ms cubic-bezier(0.4, 0, 0.2, 1) 0ms}.rgHeaderCell:hover .rv-filter,.rgHeaderCell .rv-filter.active{opacity:1}.rgHeaderCell .rv-filter{height:24px;width:24px;background:none;border:0;opacity:0;visibility:visible;cursor:pointer;border-radius:4px}.rgHeaderCell .rv-filter.active{color:#10224a}.rgHeaderCell .rv-filter .filter-img{color:gray;width:11px}.select-css{display:block;font-family:sans-serif;font-weight:600;color:#444;line-height:1.3;padding:0.6em 1.4em 0.5em 0.8em;width:100%;max-width:100%;box-sizing:border-box;margin:0;border:1px solid #f1f1f1;box-shadow:transparent;border-radius:0.5em;appearance:none;background-color:#fff;background-image:url("data:image/svg+xml;charset=US-ASCII,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%22292.4%22%20height%3D%22292.4%22%3E%3Cpath%20fill%3D%22%23007CB2%22%20d%3D%22M287%2069.4a17.6%2017.6%200%200%200-13-5.4H18.4c-5%200-9.3%201.8-12.9%205.4A17.6%2017.6%200%200%200%200%2082.2c0%205%201.8%209.3%205.4%2012.9l128%20127.9c3.6%203.6%207.8%205.4%2012.8%205.4s9.2-1.8%2012.8-5.4L287%2095c3.5-3.5%205.4-7.8%205.4-12.8%200-5-1.9-9.2-5.5-12.8z%22%2F%3E%3C%2Fsvg%3E"), linear-gradient(to bottom, #ffffff 0%, #ffffff 100%);background-repeat:no-repeat, repeat;background-position:right 0.7em top 50%, 0 0;background-size:0.65em auto, 100%;}.select-css::-ms-expand{display:none}.select-css:hover{border-color:rgb(197, 197, 197)}.select-css:focus{border-color:#f1f1f1;box-shadow:0 0 1px 3px rgba(59, 153, 252, 0.7);box-shadow:0 0 0 3px -moz-mac-focusring;color:#222;outline:none}.select-css option{font-weight:normal}.select-css:disabled,.select-css[aria-disabled=true]{color:gray;background-image:url("data:image/svg+xml;charset=US-ASCII,%3Csvg%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%20width%3D%22292.4%22%20height%3D%22292.4%22%3E%3Cpath%20fill%3D%22graytext%22%20d%3D%22M287%2069.4a17.6%2017.6%200%200%200-13-5.4H18.4c-5%200-9.3%201.8-12.9%205.4A17.6%2017.6%200%200%200%200%2082.2c0%205%201.8%209.3%205.4%2012.9l128%20127.9c3.6%203.6%207.8%205.4%2012.8%205.4s9.2-1.8%2012.8-5.4L287%2095c3.5-3.5%205.4-7.8%205.4-12.8%200-5-1.9-9.2-5.5-12.8z%22%2F%3E%3C%2Fsvg%3E"), linear-gradient(to bottom, #ffffff 0%, #ffffff 100%)}.select-css:disabled:hover,.select-css[aria-disabled=true]{border-color:#f1f1f1}.multi-filter-list{margin-top:5px;margin-bottom:5px}.multi-filter-list div{white-space:nowrap}.multi-filter-list .multi-filter-list-action{display:flex;justify-content:space-between;align-items:center}.multi-filter-list .and-or-button{margin:0 0 0 10px;min-width:58px;cursor:pointer}.multi-filter-list .trash-button{margin:0 0 -2px 6px;cursor:pointer;width:22px;height:100%;font-size:16px}.multi-filter-list .trash-button .trash-img{width:1em}.add-filter-divider{display:block;margin:0 -10px 10px -10px;border-bottom:1px solid #d9d9d9;height:10px}.select-input{display:flex;justify-content:space-between;align-items:center}revo-grid[theme^=dark] revogr-filter-panel{background-color:#28272b}revo-grid[theme^=dark] revogr-filter-panel .add-filter-divider{border-bottom-color:#28272b}revo-grid[theme^=dark] revogr-filter-panel input[type=text]{background-color:#d8d8d8;color:black}`, RevogrFilterPanelStyle0 = filterStyleCss, defaultType = "none", FILTER_LIST_CLASS = "multi-filter-list", FILTER_LIST_CLASS_ACTION = "multi-filter-list-action", FILTER_ID = "add-filter", FilterPanel = class {
    constructor(O) {
      registerInstance(this, O), this.filterChange = createEvent(this, "filterChange", 7), this.resetChange = createEvent(this, "resetChange", 7), this.filterCaptionsInternal = {
        title: "Filter by",
        ok: "Ok",
        save: "Save",
        // drops the filter
        reset: "Reset",
        cancel: "Cancel",
        add: "Add condition",
        placeholder: "Enter value...",
        and: "and",
        or: "or"
      }, this.debouncedApplyFilter = debounce$1(() => {
        this.filterChange.emit(this.filterItems);
      }, 400), this.isFilterIdSet = !1, this.filterId = 0, this.currentFilterId = -1, this.currentFilterType = defaultType, this.changes = void 0, this.filterItems = {}, this.filterNames = {}, this.filterEntities = {}, this.filterCaptions = void 0, this.disableDynamicFiltering = !1;
    }
    onMouseDown(O) {
      if (!this.changes || O.defaultPrevented)
        return;
      const _ = O.composedPath(), H = document.getElementById(FILTER_ID);
      if (H instanceof HTMLSelectElement) {
        if (_.includes(H))
          return;
        H.value = defaultType;
      }
      this.currentFilterType = defaultType, this.changes && (this.changes.type = defaultType), this.currentFilterId = -1;
      const V = !_.includes(this.element);
      O.target instanceof HTMLElement && V && !isFilterBtn(O.target) && (this.changes = void 0);
    }
    async show(O) {
      this.changes = O, this.filterItems = (O == null ? void 0 : O.filterItems) || {}, this.changes && (this.changes.type = this.changes.type || defaultType);
    }
    async getChanges() {
      return this.changes;
    }
    componentWillRender() {
      if (!this.isFilterIdSet) {
        this.isFilterIdSet = !0;
        const O = Object.keys(this.filterItems);
        for (const _ of O)
          this.filterId += this.filterItems[_].length;
      }
    }
    getFilterItemsList() {
      var O, _;
      const H = (O = this.changes) === null || O === void 0 ? void 0 : O.prop;
      if (typeof H > "u")
        return "";
      const V = (_ = this.filterItems[H]) !== null && _ !== void 0 ? _ : [], q = Object.assign(this.filterCaptionsInternal, this.filterCaptions);
      return h("div", { key: this.filterId }, V.map((G, J) => {
        let Z;
        if (!G.hidden)
          return J !== this.filterItems[H].length - 1 && (Z = h("div", { onClick: () => this.toggleFilterAndOr(G.id) }, h(AndOrButton, { text: G.relation === "and" ? q.and : q.or }))), h("div", { key: G.id, class: FILTER_LIST_CLASS }, h("div", { class: { "select-input": !0 } }, h("select", { class: "select-css select-filter", onChange: (te) => this.onFilterTypeChange(te, H, J) }, this.renderSelectOptions(this.filterItems[H][J].type, !0)), h("div", { class: FILTER_LIST_CLASS_ACTION }, Z), h("div", { onClick: () => this.onRemoveFilter(G.id) }, h(TrashButton, null))), h("div", null, this.renderExtra(H, J)));
      }), V.filter((G) => !G.hidden).length > 0 ? h("div", { class: "add-filter-divider" }) : "");
    }
    autoCorrect(O) {
      var _, H;
      if (!O)
        return;
      const V = O.getBoundingClientRect(), q = window.innerWidth - V.width;
      V.left > q && O.offsetLeft && (O.style.left = `${q - ((H = (_ = O.parentElement) === null || _ === void 0 ? void 0 : _.getBoundingClientRect().left) !== null && H !== void 0 ? H : 0)}px`);
    }
    onFilterTypeChange(O, _, H) {
      O.target instanceof HTMLSelectElement && (this.filterItems[_][H].type = O.target.value, this.filterId++, setTimeout(() => {
        const V = document.getElementById("filter-input-" + this.filterItems[_][H].id);
        V instanceof HTMLInputElement && V.focus();
      }, 0), this.disableDynamicFiltering || this.debouncedApplyFilter());
    }
    onAddNewFilter(O) {
      const _ = O.target;
      this.currentFilterType = _.value, this.addNewFilterToProp();
      const H = document.getElementById("add-filter");
      H && (H.value = defaultType, this.currentFilterType = defaultType), this.disableDynamicFiltering || this.debouncedApplyFilter();
    }
    addNewFilterToProp() {
      var O;
      const _ = (O = this.changes) === null || O === void 0 ? void 0 : O.prop;
      (_ || _ === 0) && (this.filterItems[_] || (this.filterItems[_] = []), this.currentFilterType !== "none" && (this.filterId++, this.currentFilterId = this.filterId, this.filterItems[_].push({
        id: this.currentFilterId,
        type: this.currentFilterType,
        value: "",
        relation: "and"
      }), setTimeout(() => {
        const H = document.getElementById("filter-input-" + this.currentFilterId);
        H && H.focus();
      }, 0)));
    }
    onUserInput(O, _, H) {
      this.filterItems[_][O].value = H.target.value, this.disableDynamicFiltering || this.debouncedApplyFilter();
    }
    onKeyDown(O) {
      if (O.key.toLowerCase() === "enter") {
        const _ = document.getElementById("add-filter");
        _ && (_.value = defaultType, this.currentFilterType = defaultType, this.addNewFilterToProp(), _.focus());
        return;
      }
      O.stopPropagation();
    }
    onSave() {
      this.filterChange.emit(this.filterItems);
    }
    onCancel() {
      this.changes = void 0;
    }
    onReset() {
      var O;
      this.assertChanges(), this.resetChange.emit((O = this.changes) === null || O === void 0 ? void 0 : O.prop), this.filterId++;
    }
    onRemoveFilter(O) {
      var _;
      this.assertChanges(), this.filterId++;
      const H = (_ = this.changes) === null || _ === void 0 ? void 0 : _.prop, V = this.filterItems[H ?? ""];
      if (!V)
        return;
      const q = V.findIndex((G) => G.id === O);
      q !== -1 && (V.splice(q, 1), V.length === 0 && delete this.filterItems[H ?? ""], this.disableDynamicFiltering || this.debouncedApplyFilter());
    }
    toggleFilterAndOr(O) {
      var _;
      this.assertChanges(), this.filterId++;
      const H = (_ = this.changes) === null || _ === void 0 ? void 0 : _.prop, V = this.filterItems[H ?? ""];
      if (!V)
        return;
      const q = V.findIndex((G) => G.id === O);
      q !== -1 && (V[q].relation = V[q].relation === "and" ? "or" : "and", this.disableDynamicFiltering || this.debouncedApplyFilter());
    }
    assertChanges() {
      if (!this.changes)
        throw new Error("Changes required per edit");
    }
    renderSelectOptions(O, _ = !1) {
      if (!this.changes)
        return;
      const H = [], V = this.changes.prop, q = /* @__PURE__ */ new Set();
      if (Object.entries(this.filterItems).forEach(([G, J]) => {
        J.forEach((Z) => {
          Z.hidden && q.add(Z.type);
        });
      }), !_) {
        const G = Object.assign(this.filterCaptionsInternal, this.filterCaptions);
        H.push(h("option", { selected: this.currentFilterType === defaultType, value: defaultType }, V && this.filterItems[V] && this.filterItems[V].length > 0 ? G.add : this.filterNames[defaultType]));
      }
      for (let G in this.changes.filterTypes) {
        const J = this.changes.filterTypes[G].filter((Z) => !q.has(Z));
        J.length && (H.push(...J.map((Z) => h("option", { value: Z, selected: O === Z }, this.filterNames[Z]))), H.push(h("option", { disabled: !0 })));
      }
      return H;
    }
    renderExtra(O, _) {
      const H = this.filterItems[O];
      if (!H || this.filterEntities[H[_].type].extra !== "input")
        return "";
      const V = Object.assign(this.filterCaptionsInternal, this.filterCaptions);
      return h("input", { id: `filter-input-${H[_].id}`, placeholder: V.placeholder, type: "text", value: H[_].value, onInput: this.onUserInput.bind(this, _, O), onKeyDown: (q) => this.onKeyDown(q) });
    }
    render() {
      var O, _, H;
      if (!this.changes)
        return h(Host, { style: { display: "none" } });
      const V = {
        display: "block",
        left: `${this.changes.x}px`,
        top: `${this.changes.y}px`
      }, q = Object.assign(this.filterCaptionsInternal, this.filterCaptions);
      return h(Host, { style: V, ref: (G) => {
        var J;
        !((J = this.changes) === null || J === void 0) && J.autoCorrect && this.autoCorrect(G);
      } }, h("slot", { slot: "header" }), ((_ = (O = this.changes).extraContent) === null || _ === void 0 ? void 0 : _.call(O, this.changes)) || "", ((H = this.changes) === null || H === void 0 ? void 0 : H.hideDefaultFilters) !== !0 && [
        h("label", null, q.title),
        h("div", { class: "filter-holder" }, this.getFilterItemsList()),
        h("div", { class: "add-filter" }, h("select", { id: FILTER_ID, class: "select-css", onChange: (G) => this.onAddNewFilter(G) }, this.renderSelectOptions(this.currentFilterType)))
      ], h("slot", null), h("div", { class: "filter-actions" }, this.disableDynamicFiltering && [
        h("button", { id: "revo-button-save", "aria-label": "save", class: "revo-button green", onClick: () => this.onSave() }, q.save),
        h("button", { id: "revo-button-ok", "aria-label": "ok", class: "revo-button green", onClick: () => this.onCancel() }, q.cancel)
      ], !this.disableDynamicFiltering && [
        h("button", { id: "revo-button-ok", "aria-label": "ok", class: "revo-button green", onClick: () => this.onCancel() }, q.ok),
        h("button", { id: "revo-button-reset", "aria-label": "reset", class: "revo-button outline", onClick: () => this.onReset() }, q.reset)
      ]), h("slot", { slot: "footer" }));
    }
    get element() {
      return getElement(this);
    }
  };
  FilterPanel.style = RevogrFilterPanelStyle0;
  const revogrFilterPanel_entry = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    revogr_filter_panel: FilterPanel
  }, Symbol.toStringTag, { value: "Module" }));
  /*!
   * Built by Revolist OU ❤️
   */
  function isEditInput(O) {
    return !!(O != null && O.closest(`.${EDIT_INPUT_WR}`));
  }
  function isEditorCtrConstructible(O) {
    return typeof O == "function" && typeof O.prototype == "object";
  }
  /*!
   * Built by Revolist OU ❤️
   */
  const initialParams = {
    contentSize: 0,
    clientSize: 0,
    virtualSize: 0,
    maxSize: 0
  }, NO_COORDINATE = -1;
  function getContentSize(O, _, H = 0) {
    return O + (H ? _ - H : 0);
  }
  class LocalScrollService {
    constructor(_) {
      this.cfg = _, this.preventArtificialScroll = {
        rgRow: null,
        rgCol: null
      }, this.previousScroll = {
        rgRow: NO_COORDINATE,
        rgCol: NO_COORDINATE
      }, this.params = {
        rgRow: Object.assign({}, initialParams),
        rgCol: Object.assign({}, initialParams)
      };
    }
    setParams(_, H) {
      const V = getContentSize(_.contentSize, _.clientSize, _.virtualSize);
      this.params[H] = Object.assign(Object.assign({}, _), { maxSize: V - _.clientSize, virtualContentSize: V });
    }
    // apply scroll values after scroll done
    async setScroll(_) {
      this.cancelScroll(_.dimension);
      const H = new Promise((V, q) => {
        if (this.cfg.skipAnimationFrame)
          return V();
        const G = window.requestAnimationFrame(() => {
          V();
        });
        this.preventArtificialScroll[_.dimension] = q.bind(null, G);
      });
      try {
        await H;
        const V = this.getParams(_.dimension);
        _.coordinate = Math.ceil(_.coordinate), this.previousScroll[_.dimension] = this.wrapCoordinate(_.coordinate, V), this.preventArtificialScroll[_.dimension] = null, this.cfg.applyScroll(Object.assign(Object.assign({}, _), { coordinate: V.virtualSize ? this.convert(_.coordinate, V, !1) : _.coordinate }));
      } catch (V) {
        window.cancelAnimationFrame(V);
      }
    }
    /**
     * On scroll event started
     */
    scroll(_, H, V = !1, q, G = !1) {
      if (this.cancelScroll(H), !V && this.previousScroll[H] === _) {
        this.previousScroll[H] = NO_COORDINATE;
        return;
      }
      const J = this.getParams(H);
      this.cfg.runScroll({
        dimension: H,
        coordinate: J.virtualSize ? this.convert(_, J) : _,
        delta: q,
        outside: G
      });
    }
    getParams(_) {
      return this.params[_];
    }
    // check if scroll outside of region to avoid looping
    wrapCoordinate(_, H) {
      return _ < 0 ? NO_COORDINATE : typeof H.maxSize == "number" && _ > H.maxSize ? H.maxSize : _;
    }
    // prevent already started scroll, performance optimization
    cancelScroll(_) {
      var H, V;
      (V = (H = this.preventArtificialScroll)[_]) === null || V === void 0 || V.call(H), this.preventArtificialScroll[_] = null;
    }
    /* convert virtual to real and back, scale range */
    convert(_, H, V = !0) {
      var q;
      const G = H.clientSize, J = [0, ((q = H.virtualContentSize) !== null && q !== void 0 ? q : G) - G], Z = [0, H.contentSize - H.virtualSize];
      return V ? scaleValue(_, J, Z) : scaleValue(_, Z, J);
    }
  }
  class LocalScrollTimer {
    constructor(_ = 10) {
      this.scrollThrottling = _, this.mouseWheelScrollTimestamp = {
        rgCol: 0,
        rgRow: 0
      }, this.lastKnownScrollCoordinate = {
        rgCol: 0,
        rgRow: 0
      };
    }
    setCoordinate(_) {
      this.lastKnownScrollCoordinate[_.dimension] = _.coordinate;
    }
    /**
     * Remember last mw event time
     */
    latestScrollUpdate(_) {
      this.mouseWheelScrollTimestamp[_] = (/* @__PURE__ */ new Date()).getTime();
    }
    /**
     * Check if scroll is ready to accept new value
     */
    isReady(_, H) {
      return (/* @__PURE__ */ new Date()).getTime() - this.mouseWheelScrollTimestamp[_] > this.scrollThrottling && H !== this.lastKnownScrollCoordinate[_];
    }
  }
  var FUNC_ERROR_TEXT = "Expected a function";
  function throttle(O, _, H) {
    var V = !0, q = !0;
    if (typeof O != "function")
      throw new TypeError(FUNC_ERROR_TEXT);
    return isObject$1(H) && (V = "leading" in H ? !!H.leading : V, q = "trailing" in H ? !!H.trailing : q), debounce$1(O, _, {
      leading: V,
      maxWait: _,
      trailing: q
    });
  }
  /*!
   * Built by Revolist OU ❤️
   */
  const Attribution = class {
    constructor(O) {
      registerInstance(this, O);
    }
    render() {
      return h(Host, { key: "3d66475a019010c24b6c610ccc047e01c35178f9" }, h("a", { key: "777afddffef0309a697b9c14ee73c0001ac22b71", href: "https://rv-grid.com/guide/attribution", target: "_blank", rel: "noopener noreferrer", title: "Made with ❤️ by Revolist OU Team", class: "value" }, "RevoGrid"));
    }
  }, RevoGridExtra = class {
    constructor(O) {
      registerInstance(this, O), this.nodes = [], this.update = 1;
    }
    /**
     * Refreshes the extra component. Useful if you want to manually
     * force the component to re-render.
     */
    async refresh() {
      this.update *= -1;
    }
    render() {
      var O;
      return (O = this.nodes) === null || O === void 0 ? void 0 : O.map((_) => {
        if (typeof _ == "function") {
          const H = {}, V = () => [_({ refresh: () => {
            var q;
            return (q = H.refresh) === null || q === void 0 ? void 0 : q.call(H);
          } })];
          return h("revogr-extra", { nodes: V(), ref: (q) => {
            q && (H.refresh = () => {
              q.nodes = V();
            });
          } });
        }
        return _;
      });
    }
  };
  function collectModelsOfRange(O, _) {
    const H = {};
    for (let V in O) {
      const q = parseInt(V, 10);
      H[q] = getSourceItem(_, q);
    }
    return H;
  }
  function getFocusCellBasedOnEvent(O, _) {
    if (O.defaultPrevented)
      return null;
    const H = getPropertyFromEvent(O, "clientX"), V = getPropertyFromEvent(O, "clientY");
    if (H === null || V === null)
      return null;
    const q = getCurrentCell({ x: H, y: V }, _);
    return isAfterLast(q, _.lastCell) ? null : q;
  }
  function getCurrentCell({ x: O, y: _ }, { el: H, rows: V, cols: q }) {
    const { top: G, left: J, height: Z, width: te } = H.getBoundingClientRect();
    let ne = _ - G, ee = O - J;
    ne >= Z && (ne = Z - 1), ee >= te && (ee = te - 1);
    const ae = getItemByPosition(V, ne), oe = getItemByPosition(q, ee);
    return oe.itemIndex < 0 && (oe.itemIndex = 0), ae.itemIndex < 0 && (ae.itemIndex = 0), { x: oe.itemIndex, y: ae.itemIndex };
  }
  function getCoordinate(O, _, H, V = !1) {
    const q = (G, J = 0) => {
      const Z = { x: O.x, y: O.y }, te = V ? { x: O.x1, y: O.y1 } : Z, ne = te[G] > _[G] ? te : Z;
      return ne[G] += J, { start: Z, end: te };
    };
    return H.x ? q("x", H.x) : H.y ? q("y", H.y) : null;
  }
  function isAfterLast({ x: O, y: _ }, H) {
    return O >= H.x || _ >= H.y;
  }
  function isBeforeFirst({ x: O, y: _ }) {
    return O < 0 || _ < 0;
  }
  function styleByCellProps(O) {
    return {
      left: `${O.left}px`,
      top: `${O.top}px`,
      width: `${O.width}px`,
      height: `${O.height}px`
    };
  }
  function getCell({ x: O, y: _, x1: H, y1: V }, q, G) {
    const J = getItemByIndex(q, _).start, Z = getItemByIndex(G, O).start, te = getItemByIndex(q, V).end, ne = getItemByIndex(G, H).end;
    return {
      left: Z,
      right: ne,
      top: J,
      bottom: te,
      width: ne - Z,
      height: te - J
    };
  }
  const revogrFocusStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revogr-focus.focused-cell{box-shadow:-1px 0 0 #0d63e8 inset, 1px 0 0 #0d63e8 inset, 0 -1px 0 #0d63e8 inset, 0 1px 0 #0d63e8 inset;position:absolute;pointer-events:none;z-index:9;display:block !important}`, RevogrFocusStyle0 = revogrFocusStyleCss, RevogrFocus$1 = class {
    constructor(O) {
      registerInstance(this, O), this.beforeFocusRender = createEvent(this, "beforefocusrender", 7), this.beforeScrollIntoView = createEvent(this, "beforescrollintoview", 7), this.afterFocus = createEvent(this, "afterfocus", 7), this.activeFocus = null, this.colType = void 0, this.rowType = void 0, this.selectionStore = void 0, this.dimensionRow = void 0, this.dimensionCol = void 0, this.dataStore = void 0, this.colData = void 0, this.focusTemplate = null;
    }
    componentDidRender() {
      var O, _;
      const H = this.selectionStore.get("focus");
      if (!(((O = this.activeFocus) === null || O === void 0 ? void 0 : O.x) === (H == null ? void 0 : H.x) && ((_ = this.activeFocus) === null || _ === void 0 ? void 0 : _.y) === (H == null ? void 0 : H.y)) && (this.activeFocus = H, H && this.el)) {
        this.beforeScrollIntoView.emit({ el: this.el }).defaultPrevented || this.el.scrollIntoView({
          block: "nearest",
          inline: "nearest"
        });
        const q = getSourceItem(this.dataStore, H.y), G = getSourceItem(this.colData, H.x);
        this.afterFocus.emit({
          model: q,
          column: G,
          rowType: this.rowType,
          colType: this.colType,
          rowIndex: H.y,
          colIndex: H.x
        });
      }
    }
    render() {
      var O;
      if (this.selectionStore.get("edit"))
        return;
      const H = this.selectionStore.get("focus");
      if (!H)
        return;
      const V = this.beforeFocusRender.emit({
        range: Object.assign(Object.assign({}, H), { x1: H.x, y1: H.y }),
        rowType: this.rowType,
        colType: this.colType,
        rowDimension: Object.assign({}, this.dimensionRow.state),
        colDimension: Object.assign({}, this.dimensionCol.state)
      });
      if (V.defaultPrevented)
        return h("slot", null);
      const { detail: q } = V, G = getCell(q.range, V.detail.rowDimension, V.detail.colDimension), J = styleByCellProps(G), Z = (O = this.focusTemplate) === null || O === void 0 ? void 0 : O.call(this, h, q), te = {
        class: { [FOCUS_CLASS]: !0 },
        style: J
      };
      return h(Host, Object.assign({}, te), h("slot", null), Z);
    }
    get el() {
      return getElement(this);
    }
  };
  RevogrFocus$1.style = RevogrFocusStyle0;
  const DIRECTION_CODES = [
    codesLetter.TAB,
    codesLetter.ARROW_UP,
    codesLetter.ARROW_DOWN,
    codesLetter.ARROW_LEFT,
    codesLetter.ARROW_RIGHT
  ];
  class KeyboardService {
    constructor(_) {
      this.sv = _;
    }
    async keyDown(_, H, V, { range: q, focus: G }) {
      if (V) {
        switch (_.code) {
          case codesLetter.ESCAPE:
            this.sv.cancel();
            break;
          case codesLetter.TAB:
            this.keyChangeSelection(_, H);
            break;
        }
        return;
      }
      if (q && isClear(_.code)) {
        this.sv.clearCell();
        return;
      }
      if (G) {
        if (isTab(_.code)) {
          this.keyChangeSelection(_, H);
          return;
        }
        if (isEnterKeyValue(_.key)) {
          this.sv.change();
          return;
        }
        if (!isCopy(_) && !isCut(_)) {
          if (isPaste(_)) {
            this.sv.internalPaste();
            return;
          }
          if (isAll(_)) {
            H && this.selectAll(_);
            return;
          }
          if (_.key.length === 1) {
            this.sv.change(_.key);
            return;
          }
          await this.keyChangeSelection(_, H);
        }
      }
    }
    selectAll(_) {
      const H = this.sv.selectionStore.get("range"), V = this.sv.selectionStore.get("focus");
      !H || !V || (_.preventDefault(), this.sv.selectAll());
    }
    async keyChangeSelection(_, H) {
      const V = this.changeDirectionKey(_, H);
      if (!V)
        return !1;
      await timeout(RESIZE_INTERVAL + 30);
      const q = this.sv.selectionStore.get("range"), G = this.sv.selectionStore.get("focus");
      return this.keyPositionChange(V.changes, q, G, V.isMulti);
    }
    keyPositionChange(_, H, V, q = !1) {
      if (!H || !V)
        return !1;
      const G = getCoordinate(H, V, _, q);
      if (!G)
        return !1;
      const J = this.sv.getData();
      if (q) {
        if (isAfterLast(G.end, J.lastCell) || isBeforeFirst(G.start))
          return !1;
        const Z = getRange(G.start, G.end);
        return this.sv.range(Z);
      }
      return this.sv.focus(G.start, _, isAfterLast(G.start, J.lastCell) ? 1 : isBeforeFirst(G.start) ? -1 : 0);
    }
    /** Monitor key direction changes */
    changeDirectionKey(_, H) {
      const V = H && _.shiftKey;
      if (DIRECTION_CODES.includes(_.code) && _.preventDefault(), _.shiftKey)
        switch (_.code) {
          case codesLetter.TAB:
            return { changes: { x: -1 }, isMulti: !1 };
        }
      switch (_.code) {
        case codesLetter.ARROW_UP:
          return { changes: { y: -1 }, isMulti: V };
        case codesLetter.ARROW_DOWN:
          return { changes: { y: 1 }, isMulti: V };
        case codesLetter.ARROW_LEFT:
          return { changes: { x: -1 }, isMulti: V };
        case codesLetter.TAB:
        case codesLetter.ARROW_RIGHT:
          return { changes: { x: 1 }, isMulti: V };
      }
    }
  }
  class AutoFillService {
    constructor(_) {
      this.sv = _, this.autoFillType = null, this.autoFillInitial = null, this.autoFillStart = null, this.autoFillLast = null;
    }
    /**
     * Render autofill box
     * @param range
     * @param selectionFocus
     */
    renderAutofill(_, H) {
      let V;
      return _ ? V = getCell(_, this.sv.dimensionRow.state, this.sv.dimensionCol.state) : V = getCell(Object.assign(Object.assign({}, H), { x1: H.x, y1: H.y }), this.sv.dimensionRow.state, this.sv.dimensionCol.state), h("div", { class: {
        [CELL_HANDLER_CLASS]: !0,
        [MOBILE_CLASS]: !0
      }, style: {
        left: `${V.right}px`,
        top: `${V.bottom}px`
      }, onMouseDown: (q) => this.autoFillHandler(q), onTouchStart: (q) => this.autoFillHandler(q) });
    }
    autoFillHandler(_, H = "AutoFill") {
      let V = null;
      _.target instanceof Element && (V = _.target), V && (this.selectionStart(V, this.sv.getData(), H), _.preventDefault());
    }
    get isAutoFill() {
      return !!this.autoFillType;
    }
    /**
     * Process mouse move events
     */
    selectionMouseMove(_) {
      this.onMouseMoveAutofill || (this.onMouseMoveAutofill = debounce$1((H, V) => this.doAutofillMouseMove(H, V), 5)), this.isAutoFill && this.onMouseMoveAutofill(_, this.sv.getData());
    }
    getFocus(_, H) {
      return !_ && H && (_ = { x: H.x, y: H.y }), _ || null;
    }
    /**
     * Autofill logic:
     * on mouse move apply based on previous direction (if present)
     */
    doAutofillMouseMove(_, H) {
      if (!this.autoFillInitial)
        return;
      const V = getPropertyFromEvent(_, "clientX", MOBILE_CLASS), q = getPropertyFromEvent(_, "clientY", MOBILE_CLASS);
      if (V === null || q === null)
        return;
      const G = getCurrentCell({ x: V, y: q }, H);
      if (this.autoFillLast || this.autoFillLast || (this.autoFillLast = this.autoFillStart), isAfterLast(G, H.lastCell))
        return;
      if (this.autoFillLast = G, G.x === this.autoFillInitial.x && G.y === this.autoFillInitial.y)
        this.sv.setTempRange(null);
      else {
        const Z = getRange(this.autoFillInitial, this.autoFillLast);
        this.sv.setTempRange({
          area: Z,
          type: this.autoFillType
        });
      }
    }
    /**
     * Range selection started
     * Mode @param type:
     * Can be triggered from MouseDown selection on element
     * Or can be triggered on corner square drag
     */
    selectionStart(_, H, V = "Selection") {
      const { top: q, left: G } = _.getBoundingClientRect();
      this.autoFillInitial = this.getFocus(H.focus, H.range), this.autoFillType = V, this.autoFillStart = getCurrentCell({ x: G, y: q }, H);
    }
    /**
     * Clear current range selection on mouse up and mouse leave events
     */
    clearAutoFillSelection(_, H) {
      if (this.autoFillInitial)
        if (this.autoFillInitial = this.getFocus(_, H), this.autoFillType === "AutoFill") {
          const V = getRange(this.autoFillInitial, this.autoFillLast);
          if (V) {
            const { defaultPrevented: q, detail: { range: G } } = this.sv.clearRangeDataApply({
              range: V
            });
            !q && H ? this.applyRangeWithData(G, H) : this.sv.setTempRange(null);
          }
        } else
          this.applyRangeOnly(this.autoFillInitial, this.autoFillLast);
      this.resetAutoFillState();
    }
    /**
     * Reset autofill state
     */
    resetAutoFillState() {
      this.autoFillType = null, this.autoFillInitial = null, this.autoFillLast = null, this.autoFillStart = null;
    }
    /**
     * Trigger range apply events and handle responses
     */
    onRangeApply(_, H, V) {
      this.sv.rangeDataApply({
        data: _,
        models: collectModelsOfRange(_, this.sv.dataStore),
        type: this.sv.dataStore.get("type"),
        oldRange: V,
        newRange: H
      }), this.sv.setRange(H);
    }
    /** Apply range and copy data during range application */
    applyRangeWithData(_, H) {
      const V = {
        type: this.sv.dataStore.get("type"),
        colType: this.sv.columnService.type,
        newData: {},
        mapping: {},
        newRange: _,
        oldRange: H
      }, { mapping: q, changed: G } = this.sv.columnService.getRangeData(V, this.sv.columnService.columns);
      V.newData = G, V.mapping = q;
      let J = this.sv.selectionChanged(V);
      if (J.defaultPrevented) {
        this.sv.setTempRange(null);
        return;
      }
      if (J = this.sv.rangeCopy(V), J.defaultPrevented) {
        this.sv.setRange(_);
        return;
      }
      this.onRangeApply(V.newData, _, H);
    }
    /**
     * Update range selection only,
     * no data change (mouse selection)
     */
    applyRangeOnly(_, H) {
      if (!_ || !H)
        return;
      const V = getRange(_, H);
      this.sv.setRange(V);
    }
  }
  const revogrOverlayStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revogr-overlay-selection{display:block;position:relative;width:100%}revogr-overlay-selection .autofill-handle{position:absolute;width:14px;height:14px;margin-left:-13px;margin-top:-13px;z-index:10;cursor:crosshair}revogr-overlay-selection .autofill-handle::before{content:"";position:absolute;right:0;bottom:0;width:10px;height:10px;background:#0d63e8;border:1px solid white;box-sizing:border-box}revogr-overlay-selection.mobile .autofill-handle{position:absolute;width:30px;height:30px;margin-left:-29px;margin-top:-29px;z-index:10;cursor:crosshair}revogr-overlay-selection.mobile .autofill-handle::before{content:"";position:absolute;right:0;bottom:0;width:12px;height:12px;background:#0d63e8;border:1px solid white;box-sizing:border-box}revogr-overlay-selection .selection-border-range{position:absolute;pointer-events:none;z-index:9;box-shadow:-1px 0 0 #0d63e8 inset, 1px 0 0 #0d63e8 inset, 0 -1px 0 #0d63e8 inset, 0 1px 0 #0d63e8 inset}revogr-overlay-selection .selection-border-range .range-handlers{height:100%;background-color:transparent;width:75%;max-width:50px;min-width:20px;left:50%;transform:translateX(-50%);position:absolute}revogr-overlay-selection .selection-border-range .range-handlers>span{pointer-events:auto;height:20px;width:20px;position:absolute;left:50%;transform:translateX(-50%)}revogr-overlay-selection .selection-border-range .range-handlers>span:before,revogr-overlay-selection .selection-border-range .range-handlers>span:after{position:absolute;border-radius:5px;width:15px;height:5px;left:50%;transform:translateX(-50%);background-color:rgba(0, 0, 0, 0.2)}revogr-overlay-selection .selection-border-range .range-handlers>span:first-child{top:-7px}revogr-overlay-selection .selection-border-range .range-handlers>span:first-child:before{content:"";top:0}revogr-overlay-selection .selection-border-range .range-handlers>span:last-child{bottom:-7px}revogr-overlay-selection .selection-border-range .range-handlers>span:last-child:after{content:"";bottom:0}revogr-overlay-selection revogr-edit{z-index:10}`, RevogrOverlaySelectionStyle0 = revogrOverlayStyleCss, OverlaySelection = class {
    constructor(O) {
      registerInstance(this, O), this.beforeCopyRegion = createEvent(this, "beforecopyregion", 7), this.beforeRegionPaste = createEvent(this, "beforepasteregion", 7), this.cellEditApply = createEvent(this, "celleditapply", 7), this.beforeFocusCell = createEvent(this, "beforecellfocusinit", 7), this.beforeNextViewportFocus = createEvent(this, "beforenextvpfocus", 7), this.setEdit = createEvent(this, "setedit", 7), this.beforeApplyRange = createEvent(this, "beforeapplyrange", 7), this.beforeSetRange = createEvent(this, "beforesetrange", 7), this.setRange = createEvent(this, "setrange", 7), this.beforeEditRender = createEvent(this, "beforeeditrender", 7), this.selectAll = createEvent(this, "selectall", 7), this.cancelEdit = createEvent(this, "canceledit", 7), this.setTempRange = createEvent(this, "settemprange", 7), this.beforeSetTempRange = createEvent(this, "beforesettemprange", 7), this.applyFocus = createEvent(this, "applyfocus", 7), this.focusCell = createEvent(this, "focuscell", 7), this.beforeRangeDataApply = createEvent(this, "beforerangedataapply", 7), this.selectionChange = createEvent(this, "selectionchangeinit", 7), this.beforeRangeCopyApply = createEvent(this, "beforerangecopyapply", 7), this.rangeEditApply = createEvent(this, "rangeeditapply", 7), this.rangeClipboardCopy = createEvent(this, "clipboardrangecopy", 7), this.rangeClipboardPaste = createEvent(this, "clipboardrangepaste", 7), this.beforeKeyDown = createEvent(this, "beforekeydown", 7), this.beforeKeyUp = createEvent(this, "beforekeyup", 7), this.beforeCellSave = createEvent(this, "beforecellsave", 7), this.keyboardService = null, this.autoFillService = null, this.unsubscribeSelectionStore = [], this.readonly = void 0, this.range = void 0, this.canDrag = void 0, this.useClipboard = void 0, this.selectionStore = void 0, this.dimensionRow = void 0, this.dimensionCol = void 0, this.dataStore = void 0, this.colData = void 0, this.lastCell = void 0, this.editors = void 0, this.applyChangesOnClose = !1, this.additionalData = void 0, this.isMobileDevice = void 0;
    }
    // #endregion
    // #region Listeners
    onMouseMove(O) {
      var _;
      this.selectionStore.get("focus") && ((_ = this.autoFillService) === null || _ === void 0 || _.selectionMouseMove(O));
    }
    /**
     * Action finished inside the document.
     * Pointer left document, clear any active operation.
     */
    onMouseUp() {
      var O;
      (O = this.autoFillService) === null || O === void 0 || O.clearAutoFillSelection(this.selectionStore.get("focus"), this.selectionStore.get("range"));
    }
    /**
     * Row drag started.
     * This event is fired when drag action started on cell.
     */
    onCellDrag(O) {
      var _;
      (_ = this.orderEditor) === null || _ === void 0 || _.dragStart(O.detail);
    }
    /**
     * Get keyboard down from element.
     * This event is fired when keyboard key is released.
     */
    onKeyUp(O) {
      this.beforeKeyUp.emit(Object.assign({ original: O }, this.getData()));
    }
    /**
     * Get keyboard down from element.
     * This event is fired when keyboard key is pressed.
     */
    onKeyDown(O) {
      var _;
      const H = this.beforeKeyDown.emit(Object.assign({ original: O }, this.getData()));
      O.defaultPrevented || H.defaultPrevented || (_ = this.keyboardService) === null || _ === void 0 || _.keyDown(O, this.range, !!this.selectionStore.get("edit"), {
        focus: this.selectionStore.get("focus"),
        range: this.selectionStore.get("range")
      });
    }
    // #endregion
    /**
     * Selection & Keyboard
     */
    selectionServiceSet(O) {
      this.unsubscribeSelectionStore.forEach((_) => _()), this.unsubscribeSelectionStore.length = 0, this.unsubscribeSelectionStore.push(O.onChange("nextFocus", (_) => _ && this.doFocus(_, _))), this.keyboardService = new KeyboardService({
        selectionStore: O,
        range: (_) => !!_ && this.triggerRangeEvent(_),
        focus: (_, H, V) => V ? (this.beforeNextViewportFocus.emit(_), !1) : this.doFocus(_, _, H),
        change: (_) => {
          this.readonly || this.doEdit(_);
        },
        cancel: async () => {
          var _;
          await ((_ = this.revogrEdit) === null || _ === void 0 ? void 0 : _.cancelChanges()), this.closeEdit();
        },
        clearCell: () => !this.readonly && this.clearCell(),
        internalPaste: () => !this.readonly && this.beforeRegionPaste.emit(),
        getData: () => this.getData(),
        selectAll: () => this.selectAll.emit()
      }), this.createAutoFillService();
    }
    /** Autofill */
    createAutoFillService() {
      this.autoFillService = new AutoFillService({
        dimensionRow: this.dimensionRow,
        dimensionCol: this.dimensionCol,
        columnService: this.columnService,
        dataStore: this.dataStore,
        clearRangeDataApply: (O) => this.beforeRangeDataApply.emit(Object.assign(Object.assign(Object.assign({}, O), this.types), { rowDimension: Object.assign({}, this.dimensionRow.state), colDimension: Object.assign({}, this.dimensionCol.state) })),
        setTempRange: (O) => {
          const _ = this.beforeSetTempRange.emit(Object.assign(Object.assign({ tempRange: O }, this.getData()), this.types));
          return _.defaultPrevented ? null : this.setTempRange.emit(_.detail.tempRange);
        },
        selectionChanged: (O) => this.selectionChange.emit(O),
        rangeCopy: (O) => this.beforeRangeCopyApply.emit(O),
        rangeDataApply: (O) => this.rangeEditApply.emit(O),
        setRange: (O) => !!O && this.triggerRangeEvent(O),
        getData: () => this.getData()
      });
    }
    /** Columns */
    columnServiceSet() {
      var O;
      (O = this.columnService) === null || O === void 0 || O.destroy(), this.columnService = new ColumnService(this.dataStore, this.colData), this.createAutoFillService();
    }
    connectedCallback() {
      this.columnServiceSet(), this.selectionServiceSet(this.selectionStore);
    }
    disconnectedCallback() {
      var O;
      this.unsubscribeSelectionStore.forEach((_) => _()), this.unsubscribeSelectionStore.length = 0, (O = this.columnService) === null || O === void 0 || O.destroy();
    }
    async componentWillRender() {
      var O, _;
      this.selectionStore.get("edit") || await ((_ = (O = this.revogrEdit) === null || O === void 0 ? void 0 : O.beforeDisconnect) === null || _ === void 0 ? void 0 : _.call(O));
    }
    renderRange(O) {
      const _ = getCell(O, this.dimensionRow.state, this.dimensionCol.state), H = styleByCellProps(_);
      return [
        h("div", { class: SELECTION_BORDER_CLASS, style: H }, this.isMobileDevice && h("div", { class: "range-handlers" }, h("span", { class: MOBILE_CLASS }), h("span", { class: MOBILE_CLASS })))
      ];
    }
    renderEditor() {
      const O = this.selectionStore.get("edit");
      if (this.readonly || !O)
        return null;
      const _ = O.val || getCellData(this.columnService.rowDataModel(O.y, O.x).value), H = Object.assign(Object.assign({}, O), this.columnService.getSaveData(O.y, O.x, _)), V = this.beforeEditRender.emit(Object.assign(Object.assign({ range: Object.assign(Object.assign({}, O), { x1: O.x, y1: O.y }) }, this.types), { rowDimension: Object.assign({}, this.dimensionRow.state), colDimension: Object.assign({}, this.dimensionCol.state) }));
      if (V.defaultPrevented)
        return null;
      const q = getCell(V.detail.range, V.detail.rowDimension, V.detail.colDimension), G = styleByCellProps(q);
      return h("revogr-edit", { style: G, ref: (J) => this.revogrEdit = J, additionalData: this.additionalData, editCell: H, saveOnClose: this.applyChangesOnClose, column: this.columnService.rowDataModel(O.y, O.x), editor: this.columnService.getCellEditor(O.y, O.x, this.editors), onCloseedit: (J) => this.closeEdit(J), onCelledit: (J) => {
        const Z = this.beforeCellSave.emit(J.detail);
        Z.defaultPrevented || this.cellEdit(Z.detail), Z.detail.preventFocus || this.focusNext();
      } });
    }
    render() {
      var O;
      const _ = [], H = this.renderEditor();
      if (H)
        _.push(H);
      else {
        const V = this.selectionStore.get("range"), q = this.selectionStore.get("focus");
        (V || q) && this.useClipboard && _.push(h("revogr-clipboard", { readonly: this.readonly, onCopyregion: (G) => this.onCopy(G.detail), onClearregion: () => !this.readonly && this.clearCell(), ref: (G) => this.clipboard = G, onPasteregion: (G) => this.onPaste(G.detail) })), V && _.push(...this.renderRange(V)), q && !this.readonly && this.range && _.push((O = this.autoFillService) === null || O === void 0 ? void 0 : O.renderAutofill(V, q)), this.canDrag && _.push(h("revogr-order-editor", { ref: (G) => this.orderEditor = G, dataStore: this.dataStore, dimensionRow: this.dimensionRow, dimensionCol: this.dimensionCol, parent: this.element, rowType: this.types.rowType, onRowdragstartinit: (G) => this.rowDragStart(G) }));
      }
      return h(Host, { key: "16eb46317cda519daf53f0b347700e8c1cdbd144", class: { mobile: this.isMobileDevice }, onDblClick: (V) => this.onElementDblClick(V), onMouseDown: (V) => this.onElementMouseDown(V), onTouchStart: (V) => this.onElementMouseDown(V, !0) }, _, h("slot", { key: "8e71ab9946f8b8251aaee867c6551e1fada92c6d", name: "data" }));
    }
    /**
     * Executes the focus operation on the specified range of cells.
     */
    doFocus(O, _, H) {
      const { defaultPrevented: V } = this.beforeFocusCell.emit(this.columnService.getSaveData(O.y, O.x));
      if (V)
        return !1;
      const q = Object.assign(Object.assign({ range: Object.assign(Object.assign({}, O), { x1: _.x, y1: _.y }), next: H }, this.types), { rowDimension: Object.assign({}, this.dimensionRow.state), colDimension: Object.assign({}, this.dimensionCol.state) }), G = this.applyFocus.emit(q);
      if (G.defaultPrevented)
        return !1;
      const { range: J } = G.detail;
      return !this.focusCell.emit(Object.assign({ focus: {
        x: J.x,
        y: J.y
      }, end: {
        x: J.x1,
        y: J.y1
      } }, G.detail)).defaultPrevented;
    }
    triggerRangeEvent(O) {
      const _ = this.types.rowType, H = this.beforeApplyRange.emit(Object.assign(Object.assign({ range: Object.assign({}, O) }, this.types), { rowDimension: Object.assign({}, this.dimensionRow.state), colDimension: Object.assign({}, this.dimensionCol.state) }));
      if (H.defaultPrevented)
        return !1;
      const V = this.columnService.getRangeTransformedToProps(H.detail.range, this.dataStore);
      let q = this.beforeSetRange.emit(V);
      return q.defaultPrevented || (q = this.setRange.emit(Object.assign(Object.assign({}, H.detail.range), { type: _ })), q.defaultPrevented) ? !1 : !q.defaultPrevented;
    }
    /**
     * Open Editor on DblClick
     */
    onElementDblClick(O) {
      if (O.defaultPrevented)
        return;
      const _ = this.getData();
      getFocusCellBasedOnEvent(O, _) && this.doEdit();
    }
    /**
     * Handle mouse down event on Host element
     */
    onElementMouseDown(O, _ = !1) {
      var H;
      const V = O.target;
      if (isEditInput(V) || O.defaultPrevented)
        return;
      const q = this.getData(), G = getFocusCellBasedOnEvent(O, q);
      G && (this.focus(G, this.range && O.shiftKey), this.range && (V && ((H = this.autoFillService) === null || H === void 0 || H.selectionStart(V, this.getData())), _ ? verifyTouchTarget(O.touches[0], MOBILE_CLASS) && O.preventDefault() : O.preventDefault()));
    }
    /**
     * Start cell editing
     */
    doEdit(O = "") {
      var _;
      if (this.canEdit()) {
        const H = this.selectionStore.get("focus");
        if (!H)
          return;
        const V = this.columnService.getSaveData(H.y, H.x);
        (_ = this.setEdit) === null || _ === void 0 || _.emit(Object.assign(Object.assign({}, V), { val: O }));
      }
    }
    /**
     * Close editor event triggered
     * @param details - if it requires focus next
     */
    async closeEdit(O) {
      this.cancelEdit.emit(), O != null && O.detail && await this.focusNext();
    }
    /**
     * Edit finished.
     * Close Editor and save.
     */
    cellEdit(O) {
      const _ = this.columnService.getSaveData(O.rgRow, O.rgCol, O.val);
      this.cellEditApply.emit(_);
    }
    getRegion() {
      const O = this.selectionStore.get("focus");
      let _ = this.selectionStore.get("range");
      return _ || (_ = getRange(O, O)), _;
    }
    onCopy(O) {
      var _;
      const H = this.getRegion();
      if (this.beforeCopyRegion.emit(H).defaultPrevented)
        return !1;
      let q;
      if (H) {
        const { data: G, mapping: J } = this.columnService.copyRangeArray(H, this.dataStore), Z = this.rangeClipboardCopy.emit(Object.assign({
          range: H,
          data: G,
          mapping: J
        }, this.types));
        Z.defaultPrevented || (q = Z.detail.data);
      }
      return (_ = this.clipboard) === null || _ === void 0 || _.doCopy(O, q), !0;
    }
    onPaste(O) {
      var _;
      const H = this.selectionStore.get("focus"), V = this.selectionStore.get("edit") !== null;
      if (!H || V)
        return;
      let { changed: q, range: G } = this.columnService.getTransformedDataToApply(H, O);
      const { defaultPrevented: J } = this.rangeClipboardPaste.emit(Object.assign({ data: q, models: collectModelsOfRange(q, this.dataStore), range: G }, this.types));
      J || (_ = this.autoFillService) === null || _ === void 0 || _.onRangeApply(q, G, G);
    }
    async focusNext() {
      var O;
      await ((O = this.keyboardService) === null || O === void 0 ? void 0 : O.keyChangeSelection(new KeyboardEvent("keydown", {
        code: codesLetter.ARROW_DOWN
      }), this.range)) || this.closeEdit();
    }
    clearCell() {
      var O;
      const _ = this.selectionStore.get("range");
      if (_ && !isRangeSingleCell(_)) {
        const H = this.columnService.getRangeStaticData(_, "");
        (O = this.autoFillService) === null || O === void 0 || O.onRangeApply(H, _, _);
      } else if (this.canEdit()) {
        const H = this.selectionStore.get("focus");
        if (!H)
          return;
        const V = this.columnService.getSaveData(H.y, H.x);
        this.cellEdit({
          rgRow: H.y,
          rgCol: H.x,
          val: "",
          type: V.type,
          prop: V.prop
        });
      }
    }
    rowDragStart({ detail: O }) {
      O.text = getCellData(this.columnService.rowDataModel(O.cell.y, O.cell.x).value);
    }
    /**
     * Verify if edit allowed.
     */
    canEdit() {
      var O;
      if (this.readonly)
        return !1;
      const _ = this.selectionStore.get("focus");
      return _ && !(!((O = this.columnService) === null || O === void 0) && O.isReadOnly(_.y, _.x));
    }
    get edited() {
      return this.selectionStore.get("edit");
    }
    /**
     * Sets the focus on a cell and optionally edits a range.
     */
    focus(O, _ = !1) {
      if (!O)
        return !1;
      const H = O, V = this.selectionStore.get("focus");
      if (_ && V) {
        const q = getRange(V, H);
        if (q)
          return this.triggerRangeEvent(q);
      }
      return this.doFocus(O, H);
    }
    get types() {
      return {
        rowType: this.dataStore.get("type"),
        colType: this.columnService.type
      };
    }
    /**
     * Collect data
     */
    getData() {
      return {
        el: this.element,
        rows: this.dimensionRow.state,
        cols: this.dimensionCol.state,
        lastCell: this.lastCell,
        focus: this.selectionStore.get("focus"),
        range: this.selectionStore.get("range"),
        edit: this.selectionStore.get("edit")
      };
    }
    get element() {
      return getElement(this);
    }
    static get watchers() {
      return {
        selectionStore: ["selectionServiceSet"],
        dimensionRow: ["createAutoFillService"],
        dimensionCol: ["createAutoFillService"],
        dataStore: ["columnServiceSet"],
        colData: ["columnServiceSet"]
      };
    }
  };
  OverlaySelection.style = RevogrOverlaySelectionStyle0;
  const RowHeaderRender = (O) => (_, { rowIndex: H }) => O + H, RevogrRowHeaders = class {
    constructor(O) {
      registerInstance(this, O), this.scrollViewport = createEvent(this, "scrollview", 3), this.elementToScroll = createEvent(this, "ref", 3), this.height = void 0, this.dataPorts = void 0, this.headerProp = void 0, this.rowClass = void 0, this.resize = void 0, this.rowHeaderColumn = void 0, this.additionalData = void 0, this.jobsBeforeRender = [];
    }
    render() {
      const O = [], _ = new ViewportStore("colPinStart");
      let H = 1;
      for (let J of this.dataPorts) {
        const Z = J.dataStore.get("items").length, te = new DataStore(J.type, Object.assign({}, J.dataStore.state)), ne = new DataStore("colPinStart"), ee = Object.assign({ cellTemplate: RowHeaderRender(H) }, this.rowHeaderColumn);
        ne.updateData([ee]), O.push(h("revogr-data", Object.assign({}, J, { colType: "rowHeaders", jobsBeforeRender: this.jobsBeforeRender, rowClass: this.rowClass, dataStore: te.store, colData: ne.store, viewportCol: _.store, readonly: !0, range: !1 }))), H += Z;
      }
      const V = calculateRowHeaderSize(H, this.rowHeaderColumn);
      _.setViewport({
        realCount: 1,
        virtualSize: 0,
        items: [
          {
            size: V,
            start: 0,
            end: V,
            itemIndex: 0
          }
        ]
      });
      const q = {
        contentHeight: this.height,
        contentWidth: 0,
        style: { minWidth: `${V}px` },
        colType: "rowHeaders",
        ref: (J) => this.elementToScroll.emit(J),
        onScrollviewport: (J) => this.scrollViewport.emit(J.detail)
      }, G = Object.assign(Object.assign({}, this.headerProp), {
        colData: typeof this.rowHeaderColumn == "object" ? [this.rowHeaderColumn] : [],
        viewportCol: _.store,
        canResize: !1,
        type: ROW_HEADER_TYPE,
        // parent,
        slot: HEADER_SLOT
      });
      return h(Host, { class: { [ROW_HEADER_TYPE]: !0 }, key: ROW_HEADER_TYPE }, h("revogr-viewport-scroll", Object.assign({ key: "addb87ab3406b5691cecf810119582bda3a4b4a2" }, q, { "row-header": !0 }), h("revogr-header", Object.assign({ key: "8621e76b132e7d1507794b515b40f525577413e2" }, G)), O));
    }
  };
  class AutohideScrollPlugin {
    constructor(_) {
      this.element = _, this.autohideScrollTimeout = 0;
    }
    /**
     * When scroll size updates set it up for autohide
     */
    setScrollSize(_) {
      _ ? this.element.removeAttribute("autohide") : this.element.setAttribute("autohide", "true");
    }
    /**
     * On each scroll check if it's time to show
     */
    checkScroll({ scrollSize: _, contentSize: H, virtualSize: V }) {
      const q = H > V;
      !_ && q && (this.element.setAttribute("visible", "true"), this.autohideScrollTimeout = this.show(this.element, this.autohideScrollTimeout));
    }
    show(_, H) {
      return clearTimeout(H), Number(setTimeout(() => {
        _ == null || _.removeAttribute("visible");
      }, 1e3));
    }
    clear() {
      clearTimeout(this.autohideScrollTimeout);
    }
  }
  const revogrScrollStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revogr-scroll-virtual[autohide]{position:absolute;z-index:100 !important}revogr-scroll-virtual[autohide].vertical{top:0;right:0}revogr-scroll-virtual[autohide].vertical[visible]{min-width:20px !important}revogr-scroll-virtual[autohide].horizontal{bottom:0;left:0}revogr-scroll-virtual[autohide].horizontal[visible]{min-height:20px !important}revogr-scroll-virtual.vertical{overflow-y:auto;overflow-x:hidden;height:100%}revogr-scroll-virtual.vertical>div{width:1px}revogr-scroll-virtual.horizontal{overflow-x:auto;overflow-y:hidden;width:100%}revogr-scroll-virtual.horizontal>div{height:1px}`, RevogrScrollVirtualStyle0 = revogrScrollStyleCss, RevogrScrollVirtual = class {
    constructor(O) {
      registerInstance(this, O), this.scrollVirtual = createEvent(this, "scrollvirtual", 7), this.scrollSize = 0, this.dimension = "rgRow", this.viewportStore = void 0, this.dimensionStore = void 0;
    }
    async setScroll(O) {
      var _;
      this.dimension === O.dimension && (this.localScrollTimer.latestScrollUpdate(O.dimension), (_ = this.localScrollService) === null || _ === void 0 || _.setScroll(O), O.coordinate && this.autohideScrollPlugin.checkScroll({
        scrollSize: this.scrollSize,
        contentSize: this.dimensionStore.get("realSize"),
        virtualSize: this.viewportStore.get("virtualSize")
      }));
    }
    /**
     * Update if `delta` exists in case we don't know current position or if it's external change
     */
    async changeScroll(O) {
      if (O.delta) {
        switch (O.dimension) {
          case "rgCol":
            O.coordinate = this.element.scrollLeft + O.delta;
            break;
          case "rgRow":
            O.coordinate = this.element.scrollTop + O.delta;
            break;
        }
        this.setScroll(O);
      }
      return O;
    }
    set size(O) {
      if (this.autohideScrollPlugin.setScrollSize(O), this.dimension === "rgRow") {
        this.element.style.minWidth = `${O}px`;
        return;
      }
      this.element.style.minHeight = `${O}px`;
    }
    get size() {
      return this.dimension === "rgRow" ? this.element.clientHeight : this.element.clientWidth;
    }
    connectedCallback() {
      this.autohideScrollPlugin = new AutohideScrollPlugin(this.element), this.localScrollTimer = new LocalScrollTimer("ontouchstart" in document.documentElement ? 0 : 10), this.localScrollService = new LocalScrollService({
        runScroll: (O) => this.scrollVirtual.emit(O),
        applyScroll: (O) => {
          this.localScrollTimer.setCoordinate(O);
          const _ = O.dimension === "rgRow" ? "scrollTop" : "scrollLeft";
          this.element[_] = O.coordinate;
        }
      });
    }
    disconnectedCallback() {
      this.autohideScrollPlugin.clear();
    }
    componentWillLoad() {
      this.scrollSize = getScrollbarSize(document);
    }
    componentDidRender() {
      const O = this.dimension === "rgRow" ? "scrollHeight" : "scrollWidth";
      this.element[O] > this.size ? this.size = this.scrollSize : this.size = 0, this.localScrollService.setParams({
        contentSize: this.dimensionStore.get("realSize"),
        clientSize: this.size,
        virtualSize: this.viewportStore.get("clientSize")
      }, this.dimension);
    }
    onScroll(O) {
      var _;
      if (!(O.target instanceof Element))
        return;
      const H = O.target;
      let V = "scrollLeft";
      this.dimension === "rgRow" && (V = "scrollTop"), this.localScrollTimer.isReady(this.dimension, H[V] || 0) && ((_ = this.localScrollService) === null || _ === void 0 || _.scroll(H[V] || 0, this.dimension));
    }
    render() {
      const O = this.dimension === "rgRow" ? "height" : "width", _ = getContentSize(
        this.dimensionStore.get("realSize"),
        this.size,
        this.viewportStore.get("clientSize")
        // content viewport size
      );
      return h(Host, { key: "2d11902403db45b805bde88b6fc40acff4b6b910", onScroll: (H) => this.onScroll(H) }, h("div", { key: "74978e90cbf66e3fb6dcc25bd5882346f1515c39", style: {
        [O]: `${_}px`
      } }));
    }
    get element() {
      return getElement(this);
    }
  };
  RevogrScrollVirtual.style = RevogrScrollVirtualStyle0;
  const revogrTempRangeStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}.temp-bg-range{display:block !important;position:absolute;pointer-events:none;z-index:9;border:1px solid rgb(255, 94, 0);box-sizing:border-box}.temp-bg-range.Selection{border:1px dashed gray}.temp-bg-range>div{width:1px;height:1px;position:absolute}.temp-bg-range>div.top{top:-1px}.temp-bg-range>div.bottom{bottom:-1px}.temp-bg-range>div.left{left:-1px}.temp-bg-range>div.right{right:-1px}`, RevogrTempRangeStyle0 = revogrTempRangeStyleCss, RevogrFocus = class {
    constructor(O) {
      registerInstance(this, O), this.onChange = throttle((_) => this.doChange(_), 300), this.selectionStore = void 0, this.dimensionRow = void 0, this.dimensionCol = void 0;
    }
    doChange(O) {
      O == null || O.scrollIntoView({
        block: "nearest",
        inline: "nearest"
      });
    }
    componentDidRender() {
      this.el && this.onChange(this.el);
    }
    render() {
      const O = this.selectionStore.get("tempRange"), _ = this.selectionStore.get("tempRangeType");
      if (!O)
        return;
      let H = "bottom", V = "right";
      const q = this.getRange();
      if (!q)
        return;
      O.y < q.y && (H = "top"), O.x < q.x && (V = "left");
      const G = `${V} ${H}`, J = getCell(O, this.dimensionRow.state, this.dimensionCol.state), Z = styleByCellProps(J), te = {
        class: {
          [TMP_SELECTION_BG_CLASS]: !0,
          [_ || ""]: !0
        },
        style: Z,
        hidden: !1
      };
      return h(Host, Object.assign({}, te), h("div", { class: G, ref: (ne) => this.el = ne }));
    }
    getRange() {
      const O = this.selectionStore.get("range");
      if (O)
        return O;
      const _ = this.selectionStore.get("focus");
      return _ ? Object.assign(Object.assign({}, _), { x1: _.x, y1: _.y }) : null;
    }
  };
  RevogrFocus.style = RevogrTempRangeStyle0;
  const revogrAttribution_7_entry = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    revogr_attribution: Attribution,
    revogr_extra: RevoGridExtra,
    revogr_focus: RevogrFocus$1,
    revogr_overlay_selection: OverlaySelection,
    revogr_row_headers: RevogrRowHeaders,
    revogr_scroll_virtual: RevogrScrollVirtual,
    revogr_temp_range: RevogrFocus
  }, Symbol.toStringTag, { value: "Module" }));
  /*!
   * Built by Revolist OU ❤️
   */
  const Clipboard = class {
    constructor(O) {
      registerInstance(this, O), this.beforePaste = createEvent(this, "beforepaste", 7), this.beforePasteApply = createEvent(this, "beforepasteapply", 7), this.pasteRegion = createEvent(this, "pasteregion", 7), this.afterPasteApply = createEvent(this, "afterpasteapply", 7), this.beforeCut = createEvent(this, "beforecut", 7), this.clearRegion = createEvent(this, "clearregion", 7), this.beforeCopy = createEvent(this, "beforecopy", 7), this.beforeCopyApply = createEvent(this, "beforecopyapply", 7), this.copyRegion = createEvent(this, "copyregion", 7), this.readonly = void 0;
    }
    onPaste(O) {
      if (this.readonly)
        return;
      const _ = this.getData(O), H = ((_ == null ? void 0 : _.types.indexOf("text/html")) || -1) > -1, V = (H ? _ == null ? void 0 : _.getData("text/html") : _ == null ? void 0 : _.getData("text")) || "", q = (_ == null ? void 0 : _.getData("text")) || "", G = this.beforePaste.emit({
        raw: V,
        dataText: q,
        isHTML: H,
        event: O
      });
      if (G.defaultPrevented)
        return;
      let J;
      G.detail.isHTML ? J = this.htmlParse(G.detail.raw) || this.textParse(q || "") : J = this.textParse(G.detail.raw);
      const Z = this.beforePasteApply.emit({
        raw: V,
        parsed: J,
        event: O
      });
      Z.defaultPrevented || (this.pasteRegion.emit(Z.detail.parsed), this.afterPasteApply.emit({
        raw: V,
        parsed: J,
        event: O
      }).defaultPrevented) || O.preventDefault();
    }
    /**
     * Listen to copy event and emit copy region event
     */
    copyStarted(O) {
      const _ = this.beforeCopy.emit({
        event: O
      });
      if (_.defaultPrevented)
        return;
      const H = this.getData(_.detail.event);
      this.copyRegion.emit(H || void 0), O.preventDefault();
    }
    /**
     * Listen to copy event and emit copy region event
     */
    cutStarted(O) {
      const _ = this.beforeCut.emit({
        event: O
      });
      if (_.defaultPrevented)
        return;
      const H = this.getData(_.detail.event);
      this.copyStarted(O), !this.readonly && (this.clearRegion.emit(H || void 0), O.preventDefault());
    }
    async doCopy(O, _) {
      if (this.beforeCopyApply.emit({
        event: O,
        data: _
      }).defaultPrevented)
        return;
      const V = _ ? this.parserCopy(_) : "";
      O.setData("text/plain", V);
    }
    parserCopy(O) {
      return O.map((_) => _.join("	")).join(`
`);
    }
    textParse(O) {
      const _ = [], H = O.split(/\r\n|\n|\r/);
      for (let V in H)
        _.push(H[V].split("	"));
      return _;
    }
    htmlParse(O) {
      const _ = [], V = document.createRange().createContextualFragment(O).querySelector("table");
      if (!V)
        return null;
      for (const q of Array.from(V.rows))
        _.push(Array.from(q.cells).map((G) => G.innerText));
      return _;
    }
    getData(O) {
      return O.clipboardData || (window == null ? void 0 : window.clipboardData);
    }
  }, revogrEditStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revogr-edit{display:block;position:absolute;background-color:#fff}revogr-edit input{height:100%;width:100%;box-sizing:border-box}revogr-edit revo-dropdown{height:100%}revogr-edit revo-dropdown.shrink fieldset legend>span{display:none}`, RevogrEditStyle0 = revogrEditStyleCss, RevoEdit = class {
    constructor(O) {
      registerInstance(this, O), this.cellEdit = createEvent(this, "celledit", 7), this.closeEdit = createEvent(this, "closeedit", 7), this.currentEditor = null, this.preventSaveOnClose = !1, this.editCell = void 0, this.column = void 0, this.editor = void 0, this.saveOnClose = !1, this.additionalData = void 0;
    }
    /**
     * Cancel pending changes flag. Editor will be closed without autosave.
     */
    async cancelChanges() {
      this.preventSaveOnClose = !0;
    }
    /**
     * Before editor got disconnected.
     * Can be triggered multiple times before actual disconnect.
     */
    async beforeDisconnect() {
      var O, _;
      (_ = (O = this.currentEditor) === null || O === void 0 ? void 0 : O.beforeDisconnect) === null || _ === void 0 || _.call(O);
    }
    onAutoSave() {
      var O, _, H;
      this.preventSaveOnClose = !0;
      const V = (_ = (O = this.currentEditor) === null || O === void 0 ? void 0 : O.getValue) === null || _ === void 0 ? void 0 : _.call(O);
      !((H = this.currentEditor) === null || H === void 0) && H.beforeAutoSave && this.currentEditor.beforeAutoSave(V) === !1 || this.onSave(V, !0);
    }
    /**
     * Callback triggered when cell editor saved.
     * Closes editor when called.
     * @param preventFocus - if true, editor will not be closed & next cell will not be focused.
     */
    onSave(O, _) {
      this.preventSaveOnClose = !0, this.editCell && this.cellEdit.emit({
        rgCol: this.editCell.x,
        rgRow: this.editCell.y,
        type: this.editCell.type,
        prop: this.editCell.prop,
        val: O,
        preventFocus: _
      });
    }
    componentWillRender() {
      if (!(this.currentEditor || !this.column)) {
        if (this.preventSaveOnClose = !1, this.editor) {
          isEditorCtrConstructible(this.editor) ? this.currentEditor = new this.editor(
            this.column,
            // save callback
            (O, _) => {
              this.onSave(O, _);
            },
            // cancel callback
            (O) => {
              this.preventSaveOnClose = !0, this.closeEdit.emit(O);
            }
          ) : this.currentEditor = this.editor(
            this.column,
            // save callback
            (O, _) => {
              this.onSave(O, _);
            },
            // cancel callback
            (O) => {
              this.preventSaveOnClose = !0, this.closeEdit.emit(O);
            }
          );
          return;
        }
        this.currentEditor = new TextEditor(this.column, (O, _) => this.onSave(O, _));
      }
    }
    componentDidRender() {
      var O, _;
      this.currentEditor && (this.currentEditor.element = this.element.firstElementChild, (_ = (O = this.currentEditor).componentDidRender) === null || _ === void 0 || _.call(O));
    }
    disconnectedCallback() {
      var O, _;
      this.saveOnClose && (this.preventSaveOnClose || this.onAutoSave()), this.preventSaveOnClose = !1, this.currentEditor && ((_ = (O = this.currentEditor).disconnectedCallback) === null || _ === void 0 || _.call(O), this.currentEditor.element = null, this.currentEditor = null);
    }
    render() {
      return this.currentEditor ? (this.currentEditor.editCell = this.editCell, h(Host, { class: EDIT_INPUT_WR }, this.currentEditor.render(h, this.additionalData))) : "";
    }
    get element() {
      return getElement(this);
    }
  };
  RevoEdit.style = RevogrEditStyle0;
  class RowOrderService {
    constructor(_) {
      this.config = _, this.currentCell = null, this.previousRow = null;
    }
    /** Drag finished, calculate and apply changes */
    endOrder(_, H) {
      if (this.currentCell === null)
        return;
      const V = this.getCell(_, H);
      V.y !== this.currentCell.y && (V.y < 0 ? V.y = 0 : V.y < this.currentCell.y && V.y++, this.config.positionChanged(this.currentCell.y, V.y)), this.clear();
    }
    /** Drag started, reserve initial cell for farther use */
    startOrder(_, H) {
      return this.currentCell = this.getCell(_, H), this.currentCell;
    }
    move(_, H) {
      const V = this.getRow(_, H);
      return this.previousRow === V.itemIndex || V.itemIndex < -1 ? null : (this.previousRow = V.itemIndex, V);
    }
    /** Drag stopped, probably cursor outside of document area */
    clear() {
      this.currentCell = null, this.previousRow = null;
    }
    /** Calculate cell based on x, y position */
    getRow(_, { el: H, rows: V }) {
      const { top: q } = H.getBoundingClientRect(), G = _ - q, J = getItemByPosition(V, G);
      return {
        itemIndex: J.itemIndex,
        start: J.start + q,
        end: J.end + q
      };
    }
    /** Calculate cell based on x, y position */
    getCell({ x: _, y: H }, { el: V, rows: q, cols: G }) {
      const { top: J, left: Z } = V.getBoundingClientRect(), te = H - J, ne = _ - Z, ee = getItemByPosition(q, te);
      return { x: getItemByPosition(G, ne).itemIndex, y: ee.itemIndex };
    }
  }
  const OrderEditor = class {
    constructor(O) {
      registerInstance(this, O), this.rowDragStart = createEvent(this, "rowdragstartinit", 7), this.rowDragEnd = createEvent(this, "rowdragendinit", 7), this.rowDrag = createEvent(this, "rowdragmoveinit", 7), this.rowMouseMove = createEvent(this, "rowdragmousemove", 7), this.rowDropped = createEvent(this, "rowdropinit", 7), this.rowOrderChange = createEvent(this, "roworderchange", 7), this.events = [], this.rowMoveFunc = debounce$1((_) => {
        const H = this.rowOrderService.move(_, this.getData());
        H !== null && this.rowDrag.emit(Object.assign(Object.assign({}, H), { rowType: this.rowType }));
      }, 5), this.parent = void 0, this.dimensionRow = void 0, this.dimensionCol = void 0, this.dataStore = void 0, this.rowType = void 0;
    }
    // #endregion
    // #region Methods
    async dragStart(O) {
      O.originalEvent.preventDefault(), this.events.length && this.clearOrder();
      const _ = this.getData(), H = this.rowOrderService.startOrder(O.originalEvent, _), V = this.rowOrderService.getRow(O.originalEvent.y, _);
      if (this.rowDragStart.emit({
        cell: H,
        text: DRAGG_TEXT,
        pos: V,
        event: O.originalEvent,
        rowType: this.rowType,
        model: getSourceItem(this.dataStore, V.itemIndex)
      }).defaultPrevented)
        return;
      const G = (te) => this.move(te), J = (te) => this.endOrder(te), Z = () => this.clearOrder();
      this.events.push({
        name: "mousemove",
        listener: G
      }, {
        name: "mouseup",
        listener: J
      }, {
        name: "mouseleave",
        listener: Z
      }), document.addEventListener("mousemove", G), document.addEventListener("mouseup", J), document.addEventListener("mouseleave", Z);
    }
    async endOrder(O) {
      this.rowOrderService.endOrder(O, this.getData()), this.clearOrder();
    }
    async clearOrder() {
      this.rowOrderService.clear(), this.events.forEach((O) => document.removeEventListener(O.name, O.listener)), this.events.length = 0, this.rowDragEnd.emit({ rowType: this.rowType });
    }
    // #endregion
    move({ x: O, y: _ }) {
      this.rowMouseMove.emit({ x: O, y: _, rowType: this.rowType }), this.rowMoveFunc(_);
    }
    connectedCallback() {
      this.rowOrderService = new RowOrderService({
        positionChanged: (O, _) => {
          const H = this.rowDropped.emit({
            from: O,
            to: _,
            rowType: this.rowType
          });
          H.defaultPrevented || this.rowOrderChange.emit(H.detail);
        }
      });
    }
    getData() {
      return {
        el: this.parent,
        rows: this.dimensionRow.state,
        cols: this.dimensionCol.state
      };
    }
  }, revogrClipboard_3_entry = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    revogr_clipboard: Clipboard,
    revogr_edit: RevoEdit,
    revogr_order_editor: OrderEditor
  }, Symbol.toStringTag, { value: "Module" }));
  /*!
   * Built by Revolist OU ❤️
   */
  const GroupHeaderRenderer = (O) => {
    const _ = {
      canResize: O.canResize,
      minWidth: O.group.ids.length * MIN_COL_SIZE,
      maxWidth: 0,
      active: O.active || ["r"],
      class: {
        [HEADER_CLASS]: !0
      },
      style: {
        transform: `translateX(${O.start}px)`,
        width: `${O.end - O.start}px`
      },
      onResize: O.onResize
    };
    return h(HeaderCellRenderer, { data: Object.assign(Object.assign({}, O.group), { prop: "", providers: O.providers, index: O.start }), props: _, additionalData: O.additionalData });
  }, ColumnGroupsRenderer = ({ additionalData: O, providers: _, depth: H, groups: V, visibleProps: q, dimensionCol: G, canResize: J, active: Z, onResize: te }) => {
    const ne = [];
    for (let ee = 0; ee < H; ee++) {
      if (V[ee])
        for (let ae of V[ee]) {
          const oe = findIndex(ae.ids, (se) => typeof q[se] == "number");
          if (oe > -1) {
            const ce = q[ae.ids[oe]] - oe, le = ce + ae.ids.length - 1, ue = getItemByIndex(G, ce).start, de = getItemByIndex(G, le).end;
            ne.push(h(GroupHeaderRenderer, { providers: _, start: ue, end: de, group: ae, active: Z, canResize: J, onResize: (fe) => {
              var he;
              return te((he = fe.changedX) !== null && he !== void 0 ? he : 0, ce, le);
            }, additionalData: O }));
          }
        }
      ne.push(h("div", { class: `${HEADER_ROW_CLASS} group` }));
    }
    return ne;
  }, PADDING_DEPTH = 10, RowRenderer = ({ rowClass: O, index: _, size: H, start: V, depth: q }, G) => {
    const J = Object.assign({ [DATA_ROW]: _ });
    return h("div", Object.assign({}, J, { class: `rgRow ${O || ""}`, style: {
      height: `${H}px`,
      transform: `translateY(${V}px)`,
      paddingLeft: q ? `${PADDING_DEPTH * q}px` : void 0
    } }), G);
  };
  function expandEvent(O, _, H) {
    var V;
    const q = new CustomEvent("groupexpandclick", {
      detail: {
        model: _,
        virtualIndex: H
      },
      cancelable: !0,
      bubbles: !0
    });
    (V = O.target) === null || V === void 0 || V.dispatchEvent(q);
  }
  const GroupingRowRenderer = (O) => {
    const { model: _, itemIndex: H, hasExpand: V, groupingCustomRenderer: q } = O, G = _[PSEUDO_GROUP_ITEM], J = _[GROUP_EXPANDED], Z = parseInt(_[GROUP_DEPTH], 10) || 0;
    return V ? q ? h(
      RowRenderer,
      Object.assign({}, O, { rowClass: "groupingRow", depth: Z }),
      h("div", { onClick: (te) => expandEvent(te, _, H) }, q(h, { name: G, model: _, itemIndex: H, expanded: J, depth: Z }))
    ) : h(
      RowRenderer,
      Object.assign({}, O, { rowClass: "groupingRow", depth: Z }),
      h(
        "button",
        { class: { [GROUP_EXPAND_BTN]: !0 }, onClick: (te) => expandEvent(te, _, H) },
        h(
          "svg",
          { "aria-hidden": "true", style: { transform: `rotate(${J ? 0 : -90}deg)` }, focusable: "false", viewBox: "0 0 448 512" },
          h("path", { fill: "currentColor", d: "M207.029 381.476L12.686 187.132c-9.373-9.373-9.373-24.569 0-33.941l22.667-22.667c9.357-9.357 24.522-9.375 33.901-.04L224 284.505l154.745-154.021c9.379-9.335 24.544-9.317 33.901.04l22.667 22.667c9.373 9.373 9.373 24.569 0 33.941L240.971 381.476c-9.373 9.372-24.569 9.372-33.942 0z" })
        )
      ),
      G
    ) : h(RowRenderer, Object.assign({}, O, { rowClass: "groupingRow", depth: Z }));
  };
  class RowHighlightPlugin {
    constructor() {
      this.currentRange = null;
    }
    selectionChange(_, H) {
      if (this.currentRange && H.forEach((V, q) => {
        var G;
        _ && q >= _.y && q <= _.y1 || V && V.$elm$ instanceof HTMLElement && V.$elm$.classList.contains(ROW_FOCUSED_CLASS) && (V.$elm$.classList.remove(ROW_FOCUSED_CLASS), !((G = V.$attrs$) === null || G === void 0) && G.class.includes(ROW_FOCUSED_CLASS) && (V.$attrs$.class = V.$attrs$.class.replace(ROW_FOCUSED_CLASS, "")));
      }), _)
        for (let V = _.y; V <= _.y1; V++) {
          const q = H.get(V);
          if (q && q.$elm$ instanceof HTMLElement && !q.$elm$.classList.contains(ROW_FOCUSED_CLASS)) {
            const G = q.$attrs$ = q.$attrs$ || {};
            G.class = (G.class || "") + " " + ROW_FOCUSED_CLASS, q.$elm$.classList.add(ROW_FOCUSED_CLASS);
          }
        }
      this.currentRange = _;
    }
    isRowFocused(_) {
      return this.currentRange && _ >= this.currentRange.y && _ <= this.currentRange.y1;
    }
  }
  function convertVNodeToHTML(O, _) {
    return new Promise((H) => {
      const V = document.createElement("vnode-html");
      O.appendChild(V), V.redraw = _, V.addEventListener("html", (q) => {
        V.remove(), H(q.detail);
      });
    });
  }
  function renderCell(O) {
    var _;
    const H = [], V = (_ = O.schemaModel.column) === null || _ === void 0 ? void 0 : _.cellTemplate;
    if (V)
      H.push(V(h, O.schemaModel, O.additionalData));
    else {
      if (!O.schemaModel.column)
        return "";
      O.schemaModel.column.rowDrag && isRowDragService(O.schemaModel.column.rowDrag, O.schemaModel) && H.push(h(
        "span",
        { class: DRAGGABLE_CLASS, onMouseDown: (q) => {
          var G;
          return (G = O.dragStartCell) === null || G === void 0 ? void 0 : G.emit({
            originalEvent: q,
            model: O.schemaModel
          });
        } },
        h("span", { class: DRAG_ICON_CLASS })
      )), H.push(`${getCellDataParsed(O.schemaModel.model, O.schemaModel.column)}`);
    }
    return H;
  }
  const CellRenderer = ({ renderProps: O, cellProps: _ }) => {
    const H = renderCell.bind(null, O);
    return h("div", Object.assign({}, _, { redraw: H }), H());
  }, revogrDataStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revogr-data{display:block;width:100%;position:relative}revogr-data .rgRow{position:absolute;width:100%;left:0}revogr-data .rgRow.groupingRow{font-weight:600;text-align:left}revogr-data .rgRow.groupingRow .group-expand{width:25px;height:100%;max-height:25px;margin-right:2px;background-color:transparent;border-color:transparent;vertical-align:middle;padding-left:5px}revogr-data .rgRow.groupingRow .group-expand svg{width:7px}revogr-data .revo-draggable{border:none;height:32px;display:inline-flex;outline:0;padding:0;font-size:0.8125rem;box-sizing:border-box;align-items:center;white-space:nowrap;vertical-align:middle;justify-content:center;text-decoration:none;width:24px;height:100%;cursor:pointer;display:inline-flex}revogr-data .revo-draggable:hover>.revo-drag-icon{opacity:1;zoom:1.2;font-weight:600}revogr-data .revo-draggable>.revo-drag-icon{pointer-events:none;transition:opacity 300ms cubic-bezier(0.4, 0, 0.2, 1) 0ms, zoom 300ms cubic-bezier(0.4, 0, 0.2, 1) 0ms}revogr-data .rgCell{top:0;left:0;position:absolute;box-sizing:border-box;height:100%;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;outline:none}revogr-data .rgCell.align-center{text-align:center}revogr-data .rgCell.align-left{text-align:left}revogr-data .rgCell.align-right{text-align:right}`, RevogrDataStyle0 = revogrDataStyleCss, RevogrData = class {
    constructor(O) {
      registerInstance(this, O), this.beforerowrender = createEvent(this, "beforerowrender", 7), this.afterrender = createEvent(this, "afterrender", 7), this.beforeCellRender = createEvent(this, "beforecellrender", 7), this.beforeDataRender = createEvent(this, "beforedatarender", 7), this.dragStartCell = createEvent(this, "dragstartcell", 7), this.renderedRows = /* @__PURE__ */ new Map(), this.readonly = void 0, this.range = void 0, this.rowClass = void 0, this.additionalData = void 0, this.rowSelectionStore = void 0, this.viewportRow = void 0, this.viewportCol = void 0, this.dimensionRow = void 0, this.colData = void 0, this.dataStore = void 0, this.type = void 0, this.colType = void 0, this.jobsBeforeRender = [], this.providers = void 0;
    }
    /**
     * Pointed cell update.
     */
    async updateCell(O) {
      var _, H, V;
      const q = (H = (_ = this.renderedRows.get(O.row)) === null || _ === void 0 ? void 0 : _.$children$) === null || H === void 0 ? void 0 : H[O.col];
      if (!((V = q == null ? void 0 : q.$attrs$) === null || V === void 0) && V.redraw) {
        const G = await convertVNodeToHTML(this.element, q.$attrs$.redraw);
        q.$elm$.innerHTML = G.html, q.$key$ = Math.random();
      }
    }
    onDataStoreChange() {
      this.onStoreChange();
    }
    onColDataChange() {
      this.onStoreChange();
    }
    onStoreChange() {
      var O, _;
      (O = this.columnService) === null || O === void 0 || O.destroy(), this.columnService = new ColumnService(this.dataStore, this.colData), this.providers = {
        type: this.type,
        readonly: this.readonly,
        data: this.dataStore,
        viewport: this.viewportCol,
        dimension: this.dimensionRow,
        selection: this.rowSelectionStore
      }, (_ = this.rangeUnsubscribe) === null || _ === void 0 || _.call(this), this.rangeUnsubscribe = this.rowSelectionStore.onChange("range", (H) => this.rowHighlightPlugin.selectionChange(H, this.renderedRows));
    }
    connectedCallback() {
      this.rowHighlightPlugin = new RowHighlightPlugin(), this.onStoreChange();
    }
    disconnectedCallback() {
      var O, _;
      (O = this.columnService) === null || O === void 0 || O.destroy(), (_ = this.rangeUnsubscribe) === null || _ === void 0 || _.call(this);
    }
    async componentWillRender() {
      return this.beforeDataRender.emit({
        rowType: this.type,
        colType: this.colType
      }), Promise.all(this.jobsBeforeRender.map((O) => typeof O == "function" ? O() : O));
    }
    componentDidRender() {
      this.afterrender.emit({ type: this.type });
    }
    render() {
      if (this.renderedRows = /* @__PURE__ */ new Map(), !this.columnService.columns.length)
        return;
      const _ = this.viewportRow.get("items");
      if (!_.length)
        return;
      const H = this.viewportCol.get("items");
      if (!H.length)
        return;
      const V = [], q = this.dataStore.get("groupingDepth"), G = this.dataStore.get("groupingCustomRenderer"), J = this.columnService.hasGrouping ? q : 0;
      for (let Z of _) {
        const te = getSourceItem(this.dataStore, Z.itemIndex);
        if (isGrouping(te)) {
          V.push(h(GroupingRowRenderer, Object.assign({}, Z, { index: Z.itemIndex, model: te, groupingCustomRenderer: G, hasExpand: this.columnService.hasGrouping })));
          continue;
        }
        const ne = [];
        for (let oe of H) {
          const se = Object.assign(Object.assign({}, this.columnService.rowDataModel(Z.itemIndex, oe.itemIndex)), { providers: this.providers }), ce = this.triggerBeforeCellRender(se, Z, oe);
          if (ce.defaultPrevented)
            continue;
          const { detail: { column: le, row: ue, model: de } } = ce, fe = {
            [DATA_COL]: le.itemIndex,
            [DATA_ROW]: ue.itemIndex,
            style: {
              width: `${le.size}px`,
              transform: `translateX(${le.start}px)`,
              height: ue.size ? `${ue.size}px` : void 0
            }
          };
          J && !le.itemIndex && fe.style && (fe.style.paddingLeft = `${PADDING_DEPTH * J}px`);
          const he = this.columnService.mergeProperties(ue.itemIndex, le.itemIndex, fe, de), me = h(CellRenderer, { renderProps: {
            schemaModel: de,
            additionalData: this.additionalData,
            dragStartCell: this.dragStartCell
          }, cellProps: he });
          ne.push(me);
        }
        let ee = this.rowClass ? this.columnService.getRowClass(Z.itemIndex, this.rowClass) : "";
        this.rowHighlightPlugin.isRowFocused(Z.itemIndex) && (ee += ` ${ROW_FOCUSED_CLASS}`);
        const ae = h(RowRenderer, { index: Z.itemIndex, rowClass: ee, size: Z.size, start: Z.start }, ne);
        this.beforerowrender.emit({
          node: ae,
          item: Z,
          model: te,
          colType: this.columnService.type,
          rowType: this.type
        }), V.push(ae), this.renderedRows.set(Z.itemIndex, ae);
      }
      return h(Host, null, h("slot", null), V);
    }
    triggerBeforeCellRender(O, _, H) {
      const V = {
        column: H,
        row: _,
        model: O,
        rowType: O.type,
        colType: O.colType
      };
      return this.beforeCellRender.emit(V);
    }
    get element() {
      return getElement(this);
    }
    static get watchers() {
      return {
        dataStore: ["onDataStoreChange"],
        colData: ["onColDataChange"]
      };
    }
  };
  RevogrData.style = RevogrDataStyle0;
  var defineProperty = function() {
    try {
      var O = getNative(Object, "defineProperty");
      return O({}, "", {}), O;
    } catch {
    }
  }();
  const defineProperty$1 = defineProperty;
  function baseAssignValue(O, _, H) {
    _ == "__proto__" && defineProperty$1 ? defineProperty$1(O, _, {
      configurable: !0,
      enumerable: !0,
      value: H,
      writable: !0
    }) : O[_] = H;
  }
  function arrayAggregator(O, _, H, V) {
    for (var q = -1, G = O == null ? 0 : O.length; ++q < G; ) {
      var J = O[q];
      _(V, J, H(J), O);
    }
    return V;
  }
  function baseAggregator(O, _, H, V) {
    return baseEach$1(O, function(q, G, J) {
      _(V, q, H(q), J);
    }), V;
  }
  function createAggregator(O, _) {
    return function(H, V) {
      var q = isArray$1(H) ? arrayAggregator : baseAggregator, G = {};
      return q(H, O, baseIteratee(V), G);
    };
  }
  var keyBy = createAggregator(function(O, _, H) {
    baseAssignValue(O, H, _);
  });
  const keyBy$1 = keyBy, HeaderRenderer = (O) => {
    var _, H, V, q, G;
    const J = {
      [HEADER_CLASS]: !0,
      [HEADER_SORTABLE_CLASS]: !!(!((_ = O.data) === null || _ === void 0) && _.sortable)
    };
    !((H = O.data) === null || H === void 0) && H.order && (J[O.data.order] = !0);
    const Z = {
      [DATA_COL]: O.column.itemIndex,
      canResize: O.canResize,
      minWidth: ((V = O.data) === null || V === void 0 ? void 0 : V.minSize) || MIN_COL_SIZE,
      maxWidth: (q = O.data) === null || q === void 0 ? void 0 : q.maxSize,
      active: O.active || ["r"],
      class: J,
      style: {
        width: `${O.column.size}px`,
        transform: `translateX(${O.column.start}px)`
      },
      onResize: O.onResize,
      onDblClick(te) {
        var ne;
        (ne = O.onDblClick) === null || ne === void 0 || ne.call(O, {
          column: O.data,
          index: O.column.itemIndex,
          originalEvent: te,
          providers: O.data.providers
        });
      },
      onClick(te) {
        te.defaultPrevented || !O.onClick || O.onClick({
          column: O.data,
          index: O.column.itemIndex,
          originalEvent: te,
          providers: O.data.providers
        });
      }
    };
    return O.range && O.column.itemIndex >= O.range.x && O.column.itemIndex <= O.range.x1 && typeof Z.class == "object" && (Z.class[FOCUS_CLASS] = !0), h(
      HeaderCellRenderer,
      { data: O.data, props: Z, additionalData: O.additionalData },
      h(SortingSign, { column: O.data }),
      O.canFilter && ((G = O.data) === null || G === void 0 ? void 0 : G.filter) !== !1 ? h(FilterButton, { column: O.data }) : ""
    );
  }, revogrHeaderStyleCss = `@charset "UTF-8";.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}revogr-header{position:relative;z-index:5;display:block}revogr-header .rgHeaderCell{display:flex}revogr-header .rgHeaderCell.align-center{text-align:center}revogr-header .rgHeaderCell.align-left{text-align:left}revogr-header .rgHeaderCell.align-right{text-align:right}revogr-header .rgHeaderCell.sortable{cursor:pointer}revogr-header .rgHeaderCell i.asc:after,revogr-header .rgHeaderCell i.desc:after{font-size:13px}revogr-header .rgHeaderCell i.asc:after{content:"↑"}revogr-header .rgHeaderCell i.desc:after{content:"↓"}revogr-header .rgHeaderCell,revogr-header .grouped-cell{position:absolute;box-sizing:border-box;height:100%;z-index:1}revogr-header .header-rgRow{display:block;position:relative}revogr-header .header-rgRow.group{z-index:0}revogr-header .group-rgRow{position:relative}revogr-header .rgHeaderCell.active{z-index:10}revogr-header .rgHeaderCell.active .resizable{background-color:deepskyblue}revogr-header .rgHeaderCell .header-content{overflow:hidden;text-overflow:ellipsis;white-space:nowrap;flex-grow:1}revogr-header .rgHeaderCell .resizable{display:block;position:absolute;z-index:90;touch-action:none;user-select:none}revogr-header .rgHeaderCell .resizable:hover{background-color:deepskyblue}revogr-header .rgHeaderCell>.resizable-r{cursor:ew-resize;width:6px;right:0;top:0;height:100%}revogr-header .rgHeaderCell>.resizable-rb{cursor:se-resize;width:6px;height:6px;right:0;bottom:0}revogr-header .rgHeaderCell>.resizable-b{cursor:s-resize;height:6px;bottom:0;width:100%;left:0}revogr-header .rgHeaderCell>.resizable-lb{cursor:sw-resize;width:6px;height:6px;left:0;bottom:0}revogr-header .rgHeaderCell>.resizable-l{cursor:w-resize;width:6px;left:0;height:100%;top:0}revogr-header .rgHeaderCell>.resizable-lt{cursor:nw-resize;width:6px;height:6px;left:0;top:0}revogr-header .rgHeaderCell>.resizable-t{cursor:n-resize;height:6px;top:0;width:100%;left:0}revogr-header .rgHeaderCell>.resizable-rt{cursor:ne-resize;width:6px;height:6px;right:0;top:0}revogr-header .rv-filter{visibility:hidden}`, RevogrHeaderStyle0 = revogrHeaderStyleCss, RevogrHeaderComponent = class {
    constructor(O) {
      registerInstance(this, O), this.initialHeaderClick = createEvent(this, "beforeheaderclick", 7), this.headerresize = createEvent(this, "headerresize", 7), this.beforeResize = createEvent(this, "beforeheaderresize", 7), this.headerdblClick = createEvent(this, "headerdblclick", 7), this.beforeHeaderRender = createEvent(this, "beforeheaderrender", 7), this.afterHeaderRender = createEvent(this, "afterheaderrender", 7), this.viewportCol = void 0, this.dimensionCol = void 0, this.selectionStore = void 0, this.groups = void 0, this.groupingDepth = 0, this.readonly = void 0, this.canResize = void 0, this.resizeHandler = void 0, this.colData = void 0, this.columnFilter = void 0, this.type = void 0, this.additionalData = {};
    }
    onResize({ width: O }, _) {
      const H = this.colData[_];
      this.beforeResize.emit([
        Object.assign(Object.assign({}, H), { size: O || void 0 })
      ]).defaultPrevented || this.headerresize.emit({ [_]: O || 0 });
    }
    onResizeGroup(O, _, H) {
      const V = {}, q = keyBy$1(this.viewportCol.get("items"), "itemIndex"), G = O / (H - _ + 1);
      for (let J = _; J <= H; J++) {
        const Z = q[J];
        Z && (V[J] = Z.size + G);
      }
      this.headerresize.emit(V);
    }
    componentDidRender() {
      this.afterHeaderRender.emit(this.providers);
    }
    render() {
      var O;
      const _ = this.viewportCol.get("items"), H = (O = this.selectionStore) === null || O === void 0 ? void 0 : O.get("range"), V = [], q = {};
      for (let G of _) {
        const J = this.colData[G.itemIndex], Z = {
          range: H,
          column: G,
          data: Object.assign(Object.assign({}, J), { index: G.itemIndex, providers: this.providers }),
          canFilter: !!this.columnFilter,
          canResize: this.canResize,
          active: this.resizeHandler,
          onResize: (ne) => this.onResize(ne, G.itemIndex),
          onDblClick: (ne) => this.headerdblClick.emit(ne),
          onClick: (ne) => this.initialHeaderClick.emit(ne),
          additionalData: this.additionalData
        }, te = this.beforeHeaderRender.emit(Z);
        te.defaultPrevented || (V.push(h(HeaderRenderer, Object.assign({}, te.detail))), q[J == null ? void 0 : J.prop] = G.itemIndex);
      }
      return [
        h("div", { key: "f01cf39610e7e66f7bf4fa09f52455875a940ced", class: "group-rgRow" }, h(ColumnGroupsRenderer, { key: "fa60d0aa9bb63d86f7e07ed7c1372207147a13c4", canResize: this.canResize, active: this.resizeHandler, visibleProps: q, providers: this.providers, groups: this.groups, dimensionCol: this.dimensionCol.state, depth: this.groupingDepth, onResize: (G, J, Z) => this.onResizeGroup(G, J, Z), additionalData: this.additionalData })),
        h("div", { key: "65ab0e1d0ea5a548c7dc4abfda116beb29316b3d", class: `${HEADER_ROW_CLASS} ${HEADER_ACTUAL_ROW_CLASS}` }, V)
      ];
    }
    get providers() {
      return {
        type: this.type,
        readonly: this.readonly,
        data: this.colData,
        viewport: this.viewportCol,
        dimension: this.dimensionCol,
        selection: this.selectionStore
      };
    }
    get element() {
      return getElement(this);
    }
  };
  RevogrHeaderComponent.style = RevogrHeaderStyle0;
  async function resizeObserver() {
    if (!("ResizeObserver" in window)) {
      const O = await Promise.resolve().then(() => resizeObserver83c9e167);
      window.ResizeObserver = O.ResizeObserver;
    }
  }
  class GridResizeService {
    constructor(_, H) {
      this.events = H, this.resizeObserver = null, this.resize = throttle((V, q) => {
        var G;
        return (G = this.events) === null || G === void 0 ? void 0 : G.resize(V, q);
      }, 10), this.init(_);
    }
    async init(_) {
      var H;
      await resizeObserver(), this.resizeObserver = new ResizeObserver(this.resize), (H = this.resizeObserver) === null || H === void 0 || H.observe(_);
    }
    destroy() {
      var _;
      (_ = this.resizeObserver) === null || _ === void 0 || _.disconnect(), this.resizeObserver = null;
    }
  }
  const revogrViewportScrollStyleCss = `.revo-drag-icon{width:11px;opacity:0.8}.revo-drag-icon::before{content:"::"}.revo-alt-icon{-webkit-mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");mask-image:url("data:image/svg+xml,%3C%3Fxml version='1.0' encoding='UTF-8'%3F%3E%3Csvg viewBox='0 0 384 383' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'%3E%3Cg%3E%3Cpath d='M192.4375,383 C197.424479,383 201.663411,381.254557 205.154297,377.763672 L205.154297,377.763672 L264.25,318.667969 C270.234375,312.683594 271.605794,306.075846 268.364258,298.844727 C265.122721,291.613607 259.51237,287.998047 251.533203,287.998047 L251.533203,287.998047 L213.382812,287.998047 L213.382812,212.445312 L288.935547,212.445312 L288.935547,250.595703 C288.935547,258.57487 292.551107,264.185221 299.782227,267.426758 C307.013346,270.668294 313.621094,269.296875 319.605469,263.3125 L319.605469,263.3125 L378.701172,204.216797 C382.192057,200.725911 383.9375,196.486979 383.9375,191.5 C383.9375,186.513021 382.192057,182.274089 378.701172,178.783203 L378.701172,178.783203 L319.605469,119.6875 C313.621094,114.201823 307.013346,112.955078 299.782227,115.947266 C292.551107,118.939453 288.935547,124.42513 288.935547,132.404297 L288.935547,132.404297 L288.935547,170.554688 L213.382812,170.554688 L213.382812,95.0019531 L251.533203,95.0019531 C259.51237,95.0019531 264.998047,91.3863932 267.990234,84.1552734 C270.982422,76.9241536 269.735677,70.3164062 264.25,64.3320312 L264.25,64.3320312 L205.154297,5.23632812 C201.663411,1.74544271 197.424479,0 192.4375,0 C187.450521,0 183.211589,1.74544271 179.720703,5.23632812 L179.720703,5.23632812 L120.625,64.3320312 C114.640625,70.3164062 113.269206,76.9241536 116.510742,84.1552734 C119.752279,91.3863932 125.36263,95.0019531 133.341797,95.0019531 L133.341797,95.0019531 L171.492188,95.0019531 L171.492188,170.554688 L95.9394531,170.554688 L95.9394531,132.404297 C95.9394531,124.42513 92.3238932,118.814779 85.0927734,115.573242 C77.8616536,112.331706 71.2539062,113.703125 65.2695312,119.6875 L65.2695312,119.6875 L6.17382812,178.783203 C2.68294271,182.274089 0.9375,186.513021 0.9375,191.5 C0.9375,196.486979 2.68294271,200.725911 6.17382812,204.216797 L6.17382812,204.216797 L65.2695312,263.3125 C71.2539062,268.798177 77.8616536,270.044922 85.0927734,267.052734 C92.3238932,264.060547 95.9394531,258.57487 95.9394531,250.595703 L95.9394531,250.595703 L95.9394531,212.445312 L171.492188,212.445312 L171.492188,287.998047 L133.341797,287.998047 C125.36263,287.998047 119.876953,291.613607 116.884766,298.844727 C113.892578,306.075846 115.139323,312.683594 120.625,318.667969 L120.625,318.667969 L179.720703,377.763672 C183.211589,381.254557 187.450521,383 192.4375,383 Z'%3E%3C/path%3E%3C/g%3E%3C/svg%3E");width:11px;height:11px;background-size:cover;background-repeat:no-repeat}.arrow-down{position:absolute;right:5px;top:0}.arrow-down svg{width:8px;margin-top:5px;margin-left:5px;opacity:0.4}.cell-value-wrapper{margin-right:10px;overflow:hidden;text-overflow:ellipsis}.revo-button{position:relative;overflow:hidden;color:#fff;background-color:#4545ff;height:32px;line-height:32px;padding:0 15px;outline:0;border:0;border-radius:7px;box-sizing:border-box;cursor:pointer}.revo-button.green{background-color:#009037}.revo-button.red{background-color:#E0662E}.revo-button:disabled,.revo-button[disabled]{cursor:not-allowed !important;filter:opacity(0.35) !important}.revo-button.outline{border:1px solid #dbdbdb;line-height:30px;background:none;color:#000;box-shadow:none}revo-grid[theme^=dark] .revo-button.outline{border:1px solid #404040;color:#d8d8d8}.rowHeaders{z-index:2;font-size:10px;display:flex;height:100%}.rowHeaders revogr-data .rgCell{text-align:center}.rowHeaders .rgCell{padding:0 1em !important;min-width:100%}revogr-viewport-scroll{-ms-overflow-style:none;scrollbar-width:none;overflow-x:auto;overflow-y:hidden;position:relative;z-index:1;height:100%}revogr-viewport-scroll::-webkit-scrollbar{display:none;-webkit-appearance:none}revogr-viewport-scroll.colPinStart,revogr-viewport-scroll.colPinEnd{z-index:2}revogr-viewport-scroll.colPinEnd:has(.active){overflow:visible}revogr-viewport-scroll.rgCol{flex-grow:1}revogr-viewport-scroll .content-wrapper{overflow:hidden}revogr-viewport-scroll .inner-content-table{display:flex;flex-direction:column;max-height:100%;width:100%;min-width:100%;position:relative;z-index:0}revogr-viewport-scroll .vertical-inner{overflow-y:auto;position:relative;width:100%;flex-grow:1;outline:none;-ms-overflow-style:none;scrollbar-width:none;}revogr-viewport-scroll .vertical-inner::-webkit-scrollbar{display:none;-webkit-appearance:none}revogr-viewport-scroll .vertical-inner revogr-data,revogr-viewport-scroll .vertical-inner revogr-overlay-selection{height:100%}`, RevogrViewportScrollStyle0 = revogrViewportScrollStyleCss, RevogrViewportScroll = class {
    constructor(O) {
      registerInstance(this, O), this.scrollViewport = createEvent(this, "scrollviewport", 7), this.resizeViewport = createEvent(this, "resizeviewport", 7), this.scrollchange = createEvent(this, "scrollchange", 7), this.silentScroll = createEvent(this, "scrollviewportsilent", 7), this.rowHeader = void 0, this.contentWidth = 0, this.contentHeight = 0, this.colType = void 0;
    }
    async setScroll(O) {
      var _;
      this.localScrollTimer.latestScrollUpdate(O.dimension), (_ = this.localScrollService) === null || _ === void 0 || _.setScroll(O);
    }
    /**
     * update on delta in case we don't know existing position or external change
     * @param e
     */
    async changeScroll(O, _ = !1) {
      var H, V;
      if (_) {
        if (O.coordinate && this.verticalScroll)
          switch (O.dimension) {
            // for mobile devices to skip negative scroll loop. only on vertical scroll
            case "rgRow":
              this.verticalScroll.style.transform = `translateY(${-1 * O.coordinate}px)`;
              break;
          }
        return;
      }
      if (O.delta) {
        switch (O.dimension) {
          case "rgCol":
            O.coordinate = this.horizontalScroll.scrollLeft + O.delta;
            break;
          case "rgRow":
            O.coordinate = ((V = (H = this.verticalScroll) === null || H === void 0 ? void 0 : H.scrollTop) !== null && V !== void 0 ? V : 0) + O.delta;
            break;
        }
        this.setScroll(O);
      }
      return O;
    }
    /**
     * Dispatch this event to trigger vertical mouse wheel from plugins
     */
    mousewheelVertical({ detail: O }) {
      this.verticalMouseWheel(O);
    }
    /**
     * Dispatch this event to trigger horizontal mouse wheel from plugins
     */
    mousewheelHorizontal({ detail: O }) {
      this.horizontalMouseWheel(O);
    }
    /**
     * Allows to use outside listener
     */
    scrollApply({ detail: { type: O, coordinate: _ } }) {
      this.applyOnScroll(O, _, !0);
    }
    connectedCallback() {
      this.verticalMouseWheel = this.onVerticalMouseWheel.bind(this, "rgRow", "deltaY"), this.horizontalMouseWheel = this.onHorizontalMouseWheel.bind(this, "rgCol", "deltaX"), this.localScrollTimer = new LocalScrollTimer("ontouchstart" in document.documentElement ? 0 : 10), this.localScrollService = new LocalScrollService({
        // to improve safari smoothnes on scroll
        // skipAnimationFrame: isSafariDesktop(),
        runScroll: (O) => this.scrollViewport.emit(O),
        applyScroll: (O) => {
          switch (this.localScrollTimer.setCoordinate(O), O.dimension) {
            case "rgCol":
              this.horizontalScroll.scrollLeft = O.coordinate;
              break;
            case "rgRow":
              this.verticalScroll && (this.verticalScroll.scrollTop = O.coordinate, this.verticalScroll.style.transform && (this.verticalScroll.style.transform = ""));
              break;
          }
        }
      });
    }
    componentDidLoad() {
      this.resizeService = new GridResizeService(this.horizontalScroll, {
        resize: (O) => {
          var _, H, V, q, G, J, Z, te, ne;
          let ee = ((_ = O[0]) === null || _ === void 0 ? void 0 : _.contentRect.height) || 0;
          ee && (ee -= ((V = (H = this.header) === null || H === void 0 ? void 0 : H.clientHeight) !== null && V !== void 0 ? V : 0) + ((G = (q = this.footer) === null || q === void 0 ? void 0 : q.clientHeight) !== null && G !== void 0 ? G : 0));
          const ae = {
            rgRow: {
              size: ee,
              contentSize: this.contentHeight,
              scroll: (J = this.verticalScroll) === null || J === void 0 ? void 0 : J.scrollTop,
              noScroll: !1
            },
            rgCol: {
              size: ((Z = O[0]) === null || Z === void 0 ? void 0 : Z.contentRect.width) || 0,
              contentSize: this.contentWidth,
              scroll: this.horizontalScroll.scrollLeft,
              noScroll: this.colType !== "rgCol"
            }
          };
          for (const [oe, se] of Object.entries(ae)) {
            const ce = oe;
            this.resizeViewport.emit({ dimension: ce, size: se.size, rowHeader: this.rowHeader }), !se.noScroll && ((te = this.localScrollService) === null || te === void 0 || te.scroll((ne = se.scroll) !== null && ne !== void 0 ? ne : 0, ce, !0), this.setScrollVisibility(ce, se.size, se.contentSize));
          }
        }
      });
    }
    /**
     * Check if scroll present or not per type
     * Trigger this method on inner content size change or on outer element size change
     * If inner content bigger then outer size then scroll is present and mousewheel binding required
     * @param type - dimension type 'rgRow/y' or 'rgCol/x'
     * @param size - outer content size
     * @param innerContentSize - inner content size
     */
    setScrollVisibility(O, _, H) {
      const V = _ < H;
      let q;
      switch (O) {
        case "rgCol":
          q = this.horizontalScroll;
          break;
        case "rgRow":
          q = this.verticalScroll;
          break;
      }
      V ? q == null || q.classList.add(`scroll-${O}`) : q == null || q.classList.remove(`scroll-${O}`), this.scrollchange.emit({ type: O, hasScroll: V });
    }
    disconnectedCallback() {
      var O;
      (O = this.resizeService) === null || O === void 0 || O.destroy();
    }
    async componentDidRender() {
      var O, _, H, V;
      this.localScrollService.setParams({
        contentSize: this.contentHeight,
        clientSize: (_ = (O = this.verticalScroll) === null || O === void 0 ? void 0 : O.clientHeight) !== null && _ !== void 0 ? _ : 0,
        virtualSize: 0
      }, "rgRow"), this.localScrollService.setParams({
        contentSize: this.contentWidth,
        clientSize: this.horizontalScroll.clientWidth,
        virtualSize: 0
      }, "rgCol"), this.setScrollVisibility("rgRow", (V = (H = this.verticalScroll) === null || H === void 0 ? void 0 : H.clientHeight) !== null && V !== void 0 ? V : 0, this.contentHeight), this.setScrollVisibility("rgCol", this.horizontalScroll.clientWidth, this.contentWidth);
    }
    render() {
      return h(Host, { key: "9727a9c83d740d8184e12acc06b72c3f28c3b5ac", onWheel: this.horizontalMouseWheel, onScroll: (O) => this.applyScroll("rgCol", O) }, h("div", { key: "44551981dda7401ac9290d003b1ac7e7f55ee006", class: "inner-content-table", style: { width: `${this.contentWidth}px` } }, h("div", { key: "8b4962c352e81c343aed7136b7f7a9d71b10ed70", class: "header-wrapper", ref: (O) => this.header = O }, h("slot", { key: "789868b91507bc50c18dc99c6bec8f1e4699cbab", name: HEADER_SLOT })), h("div", { key: "0efb49db3fbcbe0bdfcfa24afad495c681de6218", class: "vertical-inner", ref: (O) => this.verticalScroll = O, onWheel: this.verticalMouseWheel, onScroll: (O) => this.applyScroll("rgRow", O) }, h("div", { key: "eef434a04062b364805702d73c0b28c9e44a27c5", class: "content-wrapper", style: { height: `${this.contentHeight}px` } }, h("slot", { key: "8490b9f01abebeeaa46da811e9ecdc62090ecb00", name: CONTENT_SLOT }))), h("div", { key: "c3c133d6e73816e9e3c33d5091fc00f959977874", class: "footer-wrapper", ref: (O) => this.footer = O }, h("slot", { key: "e4aae0a9eb10c750306a6bd4c222bcab6d1050e3", name: FOOTER_SLOT }))));
    }
    /**
     * Extra layer for scroll event monitoring, where MouseWheel event is not passing
     * We need to trigger scroll event in case there is no mousewheel event
     */
    async applyScroll(O, _) {
      if (!(_.target instanceof HTMLElement))
        return;
      let H = 0;
      switch (O) {
        case "rgCol":
          H = _.target.scrollLeft;
          break;
        case "rgRow":
          H = _.target.scrollTop;
          break;
      }
      if (H < 0) {
        this.silentScroll.emit({ dimension: O, coordinate: H });
        return;
      }
      this.applyOnScroll(O, H);
    }
    /**
     * Applies change on scroll event only if mousewheel event happened some time ago
     */
    applyOnScroll(O, _, H = !1) {
      var V;
      this.localScrollTimer.isReady(O, _) && ((V = this.localScrollService) === null || V === void 0 || V.scroll(_, O, void 0, void 0, H));
    }
    /**
     * On vertical mousewheel event
     * @param type
     * @param delta
     * @param e
     */
    onVerticalMouseWheel(O, _, H) {
      var V, q, G, J, Z, te, ne, ee;
      const ae = (q = (V = this.verticalScroll) === null || V === void 0 ? void 0 : V.scrollTop) !== null && q !== void 0 ? q : 0, oe = (J = (G = this.verticalScroll) === null || G === void 0 ? void 0 : G.clientHeight) !== null && J !== void 0 ? J : 0, se = (te = (Z = this.verticalScroll) === null || Z === void 0 ? void 0 : Z.scrollHeight) !== null && te !== void 0 ? te : 0, ce = ae + oe >= se && H.deltaY > 0, le = ae === 0 && H.deltaY < 0;
      !ce && !le && ((ne = H.preventDefault) === null || ne === void 0 || ne.call(H));
      const ue = ae + H[_];
      (ee = this.localScrollService) === null || ee === void 0 || ee.scroll(ue, O, void 0, H[_]), this.localScrollTimer.latestScrollUpdate(O);
    }
    /**
     * On horizontal mousewheel event
     * @param type
     * @param delta
     * @param e
     */
    onHorizontalMouseWheel(O, _, H) {
      var V, q;
      if (!H.deltaX)
        return;
      const { scrollLeft: G, scrollWidth: J, clientWidth: Z } = this.horizontalScroll, te = G + Z >= J && H.deltaX > 0, ne = G === 0 && H.deltaX < 0;
      !te && !ne && ((V = H.preventDefault) === null || V === void 0 || V.call(H));
      const ee = G + H[_];
      (q = this.localScrollService) === null || q === void 0 || q.scroll(ee, O, void 0, H[_]), this.localScrollTimer.latestScrollUpdate(O);
    }
    get horizontalScroll() {
      return getElement(this);
    }
  };
  RevogrViewportScroll.style = RevogrViewportScrollStyle0;
  const VNodeToHtml = class {
    constructor(O) {
      registerInstance(this, O), this.html = createEvent(this, "html", 7), this.vnodes = [], this.redraw = null;
    }
    componentDidRender() {
      this.html.emit({
        html: this.el.innerHTML,
        vnodes: this.vnodes
      });
    }
    render() {
      var O, _;
      return this.vnodes = (_ = (O = this.redraw) === null || O === void 0 ? void 0 : O.call(this)) !== null && _ !== void 0 ? _ : null, h(Host, { key: "73a019dbc046a9d4504bc84a80868e8023d1f502", style: { visibility: "hidden", position: "absolute" } }, this.vnodes);
    }
    get el() {
      return getElement(this);
    }
  }, revogrData_4_entry = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    revogr_data: RevogrData,
    revogr_header: RevogrHeaderComponent,
    revogr_viewport_scroll: RevogrViewportScroll,
    vnode_html: VNodeToHtml
  }, Symbol.toStringTag, { value: "Module" }));
  /*!
   * Built by Revolist OU ❤️
   */
  var resizeObservers = [], hasActiveObservations = function() {
    return resizeObservers.some(function(O) {
      return O.activeTargets.length > 0;
    });
  }, hasSkippedObservations = function() {
    return resizeObservers.some(function(O) {
      return O.skippedTargets.length > 0;
    });
  }, msg = "ResizeObserver loop completed with undelivered notifications.", deliverResizeLoopError = function() {
    var O;
    typeof ErrorEvent == "function" ? O = new ErrorEvent("error", {
      message: msg
    }) : (O = document.createEvent("Event"), O.initEvent("error", !1, !1), O.message = msg), window.dispatchEvent(O);
  }, ResizeObserverBoxOptions;
  (function(O) {
    O.BORDER_BOX = "border-box", O.CONTENT_BOX = "content-box", O.DEVICE_PIXEL_CONTENT_BOX = "device-pixel-content-box";
  })(ResizeObserverBoxOptions || (ResizeObserverBoxOptions = {}));
  var freeze = function(O) {
    return Object.freeze(O);
  }, ResizeObserverSize = /* @__PURE__ */ function() {
    function O(_, H) {
      this.inlineSize = _, this.blockSize = H, freeze(this);
    }
    return O;
  }(), DOMRectReadOnly = function() {
    function O(_, H, V, q) {
      return this.x = _, this.y = H, this.width = V, this.height = q, this.top = this.y, this.left = this.x, this.bottom = this.top + this.height, this.right = this.left + this.width, freeze(this);
    }
    return O.prototype.toJSON = function() {
      var _ = this, H = _.x, V = _.y, q = _.top, G = _.right, J = _.bottom, Z = _.left, te = _.width, ne = _.height;
      return { x: H, y: V, top: q, right: G, bottom: J, left: Z, width: te, height: ne };
    }, O.fromRect = function(_) {
      return new O(_.x, _.y, _.width, _.height);
    }, O;
  }(), isSVG = function(O) {
    return O instanceof SVGElement && "getBBox" in O;
  }, isHidden = function(O) {
    if (isSVG(O)) {
      var _ = O.getBBox(), H = _.width, V = _.height;
      return !H && !V;
    }
    var q = O, G = q.offsetWidth, J = q.offsetHeight;
    return !(G || J || O.getClientRects().length);
  }, isElement = function(O) {
    var _;
    if (O instanceof Element)
      return !0;
    var H = (_ = O == null ? void 0 : O.ownerDocument) === null || _ === void 0 ? void 0 : _.defaultView;
    return !!(H && O instanceof H.Element);
  }, isReplacedElement = function(O) {
    switch (O.tagName) {
      case "INPUT":
        if (O.type !== "image")
          break;
      case "VIDEO":
      case "AUDIO":
      case "EMBED":
      case "OBJECT":
      case "CANVAS":
      case "IFRAME":
      case "IMG":
        return !0;
    }
    return !1;
  }, global$1 = typeof window < "u" ? window : {}, cache = /* @__PURE__ */ new WeakMap(), scrollRegexp = /auto|scroll/, verticalRegexp = /^tb|vertical/, IE = /msie|trident/i.test(global$1.navigator && global$1.navigator.userAgent), parseDimension = function(O) {
    return parseFloat(O || "0");
  }, size = function(O, _, H) {
    return O === void 0 && (O = 0), _ === void 0 && (_ = 0), H === void 0 && (H = !1), new ResizeObserverSize((H ? _ : O) || 0, (H ? O : _) || 0);
  }, zeroBoxes = freeze({
    devicePixelContentBoxSize: size(),
    borderBoxSize: size(),
    contentBoxSize: size(),
    contentRect: new DOMRectReadOnly(0, 0, 0, 0)
  }), calculateBoxSizes = function(O, _) {
    if (_ === void 0 && (_ = !1), cache.has(O) && !_)
      return cache.get(O);
    if (isHidden(O))
      return cache.set(O, zeroBoxes), zeroBoxes;
    var H = getComputedStyle(O), V = isSVG(O) && O.ownerSVGElement && O.getBBox(), q = !IE && H.boxSizing === "border-box", G = verticalRegexp.test(H.writingMode || ""), J = !V && scrollRegexp.test(H.overflowY || ""), Z = !V && scrollRegexp.test(H.overflowX || ""), te = V ? 0 : parseDimension(H.paddingTop), ne = V ? 0 : parseDimension(H.paddingRight), ee = V ? 0 : parseDimension(H.paddingBottom), ae = V ? 0 : parseDimension(H.paddingLeft), oe = V ? 0 : parseDimension(H.borderTopWidth), se = V ? 0 : parseDimension(H.borderRightWidth), ce = V ? 0 : parseDimension(H.borderBottomWidth), le = V ? 0 : parseDimension(H.borderLeftWidth), ue = ae + ne, de = te + ee, fe = le + se, he = oe + ce, me = Z ? O.offsetHeight - he - O.clientHeight : 0, pe = J ? O.offsetWidth - fe - O.clientWidth : 0, ye = q ? ue + fe : 0, ge = q ? de + he : 0, ve = V ? V.width : parseDimension(H.width) - ye - pe, be = V ? V.height : parseDimension(H.height) - ge - me, Ce = ve + ue + pe + fe, Se = be + de + me + he, we = freeze({
      devicePixelContentBoxSize: size(Math.round(ve * devicePixelRatio), Math.round(be * devicePixelRatio), G),
      borderBoxSize: size(Ce, Se, G),
      contentBoxSize: size(ve, be, G),
      contentRect: new DOMRectReadOnly(ae, te, ve, be)
    });
    return cache.set(O, we), we;
  }, calculateBoxSize = function(O, _, H) {
    var V = calculateBoxSizes(O, H), q = V.borderBoxSize, G = V.contentBoxSize, J = V.devicePixelContentBoxSize;
    switch (_) {
      case ResizeObserverBoxOptions.DEVICE_PIXEL_CONTENT_BOX:
        return J;
      case ResizeObserverBoxOptions.BORDER_BOX:
        return q;
      default:
        return G;
    }
  }, ResizeObserverEntry = /* @__PURE__ */ function() {
    function O(_) {
      var H = calculateBoxSizes(_);
      this.target = _, this.contentRect = H.contentRect, this.borderBoxSize = freeze([H.borderBoxSize]), this.contentBoxSize = freeze([H.contentBoxSize]), this.devicePixelContentBoxSize = freeze([H.devicePixelContentBoxSize]);
    }
    return O;
  }(), calculateDepthForNode = function(O) {
    if (isHidden(O))
      return 1 / 0;
    for (var _ = 0, H = O.parentNode; H; )
      _ += 1, H = H.parentNode;
    return _;
  }, broadcastActiveObservations = function() {
    var O = 1 / 0, _ = [];
    resizeObservers.forEach(function(J) {
      if (J.activeTargets.length !== 0) {
        var Z = [];
        J.activeTargets.forEach(function(ne) {
          var ee = new ResizeObserverEntry(ne.target), ae = calculateDepthForNode(ne.target);
          Z.push(ee), ne.lastReportedSize = calculateBoxSize(ne.target, ne.observedBox), ae < O && (O = ae);
        }), _.push(function() {
          J.callback.call(J.observer, Z, J.observer);
        }), J.activeTargets.splice(0, J.activeTargets.length);
      }
    });
    for (var H = 0, V = _; H < V.length; H++) {
      var q = V[H];
      q();
    }
    return O;
  }, gatherActiveObservationsAtDepth = function(O) {
    resizeObservers.forEach(function(H) {
      H.activeTargets.splice(0, H.activeTargets.length), H.skippedTargets.splice(0, H.skippedTargets.length), H.observationTargets.forEach(function(q) {
        q.isActive() && (calculateDepthForNode(q.target) > O ? H.activeTargets.push(q) : H.skippedTargets.push(q));
      });
    });
  }, process$1 = function() {
    var O = 0;
    for (gatherActiveObservationsAtDepth(O); hasActiveObservations(); )
      O = broadcastActiveObservations(), gatherActiveObservationsAtDepth(O);
    return hasSkippedObservations() && deliverResizeLoopError(), O > 0;
  }, trigger, callbacks = [], notify = function() {
    return callbacks.splice(0).forEach(function(O) {
      return O();
    });
  }, queueMicroTask = function(O) {
    if (!trigger) {
      var _ = 0, H = document.createTextNode(""), V = { characterData: !0 };
      new MutationObserver(function() {
        return notify();
      }).observe(H, V), trigger = function() {
        H.textContent = "".concat(_ ? _-- : _++);
      };
    }
    callbacks.push(O), trigger();
  }, queueResizeObserver = function(O) {
    queueMicroTask(function() {
      requestAnimationFrame(O);
    });
  }, watching = 0, isWatching = function() {
    return !!watching;
  }, CATCH_PERIOD = 250, observerConfig = { attributes: !0, characterData: !0, childList: !0, subtree: !0 }, events = [
    "resize",
    "load",
    "transitionend",
    "animationend",
    "animationstart",
    "animationiteration",
    "keyup",
    "keydown",
    "mouseup",
    "mousedown",
    "mouseover",
    "mouseout",
    "blur",
    "focus"
  ], time = function(O) {
    return O === void 0 && (O = 0), Date.now() + O;
  }, scheduled = !1, Scheduler = function() {
    function O() {
      var _ = this;
      this.stopped = !0, this.listener = function() {
        return _.schedule();
      };
    }
    return O.prototype.run = function(_) {
      var H = this;
      if (_ === void 0 && (_ = CATCH_PERIOD), !scheduled) {
        scheduled = !0;
        var V = time(_);
        queueResizeObserver(function() {
          var q = !1;
          try {
            q = process$1();
          } finally {
            if (scheduled = !1, _ = V - time(), !isWatching())
              return;
            q ? H.run(1e3) : _ > 0 ? H.run(_) : H.start();
          }
        });
      }
    }, O.prototype.schedule = function() {
      this.stop(), this.run();
    }, O.prototype.observe = function() {
      var _ = this, H = function() {
        return _.observer && _.observer.observe(document.body, observerConfig);
      };
      document.body ? H() : global$1.addEventListener("DOMContentLoaded", H);
    }, O.prototype.start = function() {
      var _ = this;
      this.stopped && (this.stopped = !1, this.observer = new MutationObserver(this.listener), this.observe(), events.forEach(function(H) {
        return global$1.addEventListener(H, _.listener, !0);
      }));
    }, O.prototype.stop = function() {
      var _ = this;
      this.stopped || (this.observer && this.observer.disconnect(), events.forEach(function(H) {
        return global$1.removeEventListener(H, _.listener, !0);
      }), this.stopped = !0);
    }, O;
  }(), scheduler = new Scheduler(), updateCount = function(O) {
    !watching && O > 0 && scheduler.start(), watching += O, !watching && scheduler.stop();
  }, skipNotifyOnElement = function(O) {
    return !isSVG(O) && !isReplacedElement(O) && getComputedStyle(O).display === "inline";
  }, ResizeObservation = function() {
    function O(_, H) {
      this.target = _, this.observedBox = H || ResizeObserverBoxOptions.CONTENT_BOX, this.lastReportedSize = {
        inlineSize: 0,
        blockSize: 0
      };
    }
    return O.prototype.isActive = function() {
      var _ = calculateBoxSize(this.target, this.observedBox, !0);
      return skipNotifyOnElement(this.target) && (this.lastReportedSize = _), this.lastReportedSize.inlineSize !== _.inlineSize || this.lastReportedSize.blockSize !== _.blockSize;
    }, O;
  }(), ResizeObserverDetail = /* @__PURE__ */ function() {
    function O(_, H) {
      this.activeTargets = [], this.skippedTargets = [], this.observationTargets = [], this.observer = _, this.callback = H;
    }
    return O;
  }(), observerMap = /* @__PURE__ */ new WeakMap(), getObservationIndex = function(O, _) {
    for (var H = 0; H < O.length; H += 1)
      if (O[H].target === _)
        return H;
    return -1;
  }, ResizeObserverController = function() {
    function O() {
    }
    return O.connect = function(_, H) {
      var V = new ResizeObserverDetail(_, H);
      observerMap.set(_, V);
    }, O.observe = function(_, H, V) {
      var q = observerMap.get(_), G = q.observationTargets.length === 0;
      getObservationIndex(q.observationTargets, H) < 0 && (G && resizeObservers.push(q), q.observationTargets.push(new ResizeObservation(H, V && V.box)), updateCount(1), scheduler.schedule());
    }, O.unobserve = function(_, H) {
      var V = observerMap.get(_), q = getObservationIndex(V.observationTargets, H), G = V.observationTargets.length === 1;
      q >= 0 && (G && resizeObservers.splice(resizeObservers.indexOf(V), 1), V.observationTargets.splice(q, 1), updateCount(-1));
    }, O.disconnect = function(_) {
      var H = this, V = observerMap.get(_);
      V.observationTargets.slice().forEach(function(q) {
        return H.unobserve(_, q.target);
      }), V.activeTargets.splice(0, V.activeTargets.length);
    }, O;
  }(), ResizeObserver$1 = function() {
    function O(_) {
      if (arguments.length === 0)
        throw new TypeError("Failed to construct 'ResizeObserver': 1 argument required, but only 0 present.");
      if (typeof _ != "function")
        throw new TypeError("Failed to construct 'ResizeObserver': The callback provided as parameter 1 is not a function.");
      ResizeObserverController.connect(this, _);
    }
    return O.prototype.observe = function(_, H) {
      if (arguments.length === 0)
        throw new TypeError("Failed to execute 'observe' on 'ResizeObserver': 1 argument required, but only 0 present.");
      if (!isElement(_))
        throw new TypeError("Failed to execute 'observe' on 'ResizeObserver': parameter 1 is not of type 'Element");
      ResizeObserverController.observe(this, _, H);
    }, O.prototype.unobserve = function(_) {
      if (arguments.length === 0)
        throw new TypeError("Failed to execute 'unobserve' on 'ResizeObserver': 1 argument required, but only 0 present.");
      if (!isElement(_))
        throw new TypeError("Failed to execute 'unobserve' on 'ResizeObserver': parameter 1 is not of type 'Element");
      ResizeObserverController.unobserve(this, _);
    }, O.prototype.disconnect = function() {
      ResizeObserverController.disconnect(this);
    }, O.toString = function() {
      return "function ResizeObserver () { [polyfill code] }";
    }, O;
  }();
  const resizeObserver83c9e167 = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
    __proto__: null,
    ResizeObserver: ResizeObserver$1,
    ResizeObserverEntry,
    ResizeObserverSize
  }, Symbol.toStringTag, { value: "Module" }));
});
export default _7();
